
#define N_TRAIN_FIELDS  2
#define N_TAG_FIELDS    1
#define BEAM_SIZE       4
#define TAGGER_NAME     "suc"
#define PyInit_TAGGER_NAME PyInit_suc

#include <stdint.h>

typedef uint32_t   partial_hash_t;
typedef uint32_t   feat_hash_t;
typedef uint32_t   label;

#ifndef __HASH_C__
#define __HASH_C__

#include <stdint.h>
#include <string.h>
#include <stddef.h>

// Code adapted from: http://bjoern.hoehrmann.de/utf-8/decoder/dfa/

#define UTF8_ACCEPT 0
#define UTF8_REJECT 12

static const uint8_t utf8d[] = {
  // The first part of the table maps bytes to character classes that
  // to reduce the size of the transition table and create bitmasks.
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
   7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,  7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
   8,8,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  10,3,3,3,3,3,3,3,3,3,3,3,3,4,3,3, 11,6,6,6,5,8,8,8,8,8,8,8,8,8,8,8,

  // The second part is a transition table that maps a combination
  // of a state of the automaton and a character class to a state.
   0,12,24,36,60,96,84,12,12,12,48,72, 12,12,12,12,12,12,12,12,12,12,12,12,
  12, 0,12,12,12,12,12, 0,12, 0,12,12, 12,24,12,12,12,12,12,24,12,24,12,12,
  12,12,12,12,12,12,12,24,12,12,12,12, 12,24,12,12,12,12,12,12,12,24,12,12,
  12,12,12,12,12,12,12,36,12,36,12,12, 12,36,12,12,12,12,12,36,12,36,12,12,
  12,36,12,12,12,12,12,12,12,12,12,12, 
};

static inline void utf8_step(uint32_t *state, uint32_t *codep, uint32_t byte) {
    const uint32_t type = utf8d[byte];
    
    *codep = (*state != UTF8_ACCEPT) ?
        (byte & 0x3fu) | (*codep << 6) :
        (0xff >> type) & (byte);
    *state = utf8d[256 + *state + type];
}

static inline int utf8_decode(
        uint32_t *dest,
        size_t *dest_len,
        const uint8_t *src,
        size_t src_len)
{
    size_t i;
    uint32_t state = 0;
    uint32_t codep;
    size_t j = 0;
    for (i=0; i<src_len; i++) {
        utf8_step(&state, &codep, src[i]);
        if (!state) {
            dest[j++] = codep;
            if (j >= *dest_len) return -1;
        }
    }
    *dest_len = j;
    if (state != UTF8_ACCEPT) return -1;
    return 0;
}

static inline uint32_t rotl32(uint32_t x, int r) {
    return (x << r) | (x >> (32 - r));
}

static inline uint64_t rotl64(uint64_t x, int r) {
    return (x << r) | (x >> (64 - r));
}

static inline uint32_t hash32_mix(uint32_t x, uint32_t y) {
    x = rotl32(x * 0xcc9e2d51, 15) * 0x1b873593;
    y = rotl32(y ^ x, 13) * 5 + 0xe6546b64;
    return y;
}

static inline uint64_t hash64_mix(uint64_t x, uint64_t y) {
    x = rotl64(x * 14029467366897019727ULL, 31) * 11400714785074694791ULL;
    y = rotl64(y ^ x, 31) * 5 + 0xbdef9f91b243c6e6ULL;
    return y;
}

static inline uint32_t hash32_mix_tail(uint32_t x, uint32_t y) {
    x = rotl32(x * 0xcc9e2d51, 15) * 0x1b873593;
    return y ^ x;
}

static inline uint64_t hash64_mix_tail(uint64_t x, uint64_t y) {
    x = rotl64(x * 14029467366897019727ULL, 31) * 11400714785074694791ULL;
    return y ^ x;
}

static inline uint32_t hash32_fmix(uint32_t x) {
    x = 0x85ebca6b * (x ^ (x >> 16));
    x = 0xc2b2ae35 * (x ^ (x >> 13));
    return x ^ (x >> 16);
}

static inline uint64_t hash64_fmix(uint64_t x) {
    x = (x ^ (x >> 33)) * 14029467366897019727ULL;
    x = (x ^ (x >> 29)) * 1609587929392839161ULL;
    return x ^ (x >> 32);
}

static inline uint32_t hash32_partial_unicode(
        const uint32_t *p,
        size_t len)
{
    if (len == 0) return 0x3a5c441;

    uint32_t h1 = p[0];
    size_t i;

    for (i=1; i<len; i++)
        h1 = hash32_mix(p[i], h1);

    return h1 ^ (uint32_t)len;
}

static inline uint64_t hash64_partial_unicode(
        const uint32_t *p,
        size_t len)
{
    if (len == 0) return 0x7fb838a8a0a95046ULL;

    uint64_t h1 = (len == 1)? (uint64_t)p[0] :
                              ((uint64_t)p[0] | ((uint64_t)p[1] << 32));
    size_t i;

    for (i=1; i<len/2; i++)
        h1 = hash64_mix((uint64_t)p[i*2] | ((uint64_t)p[i*2+1] << 32), h1);

    if (len % 2)
        h1 = hash64_mix((uint64_t)p[len-1], h1);

    return h1 ^ (uint64_t)len;
}

static inline uint32_t hash32_partial_unicode_suffix(
        const uint32_t *p,
        size_t len,
        size_t suffix_len,
        size_t min_stem)
{
    if (min_stem + suffix_len > len) return 0x34b020cc;
    return hash32_partial_unicode(p+(len-suffix_len), suffix_len);
}

static inline uint64_t hash64_partial_unicode_suffix(
        const uint32_t *p,
        size_t len,
        size_t suffix_len,
        size_t min_stem)
{
    if (min_stem + suffix_len > len) return 0xb9d9d9fb4440f7bbULL;
    return hash64_partial_unicode(p+(len-suffix_len), suffix_len);
}

static inline uint32_t hash32_partial_unicode_prefix(
        const uint32_t *p,
        size_t len,
        size_t prefix_len,
        size_t min_stem)
{
    if (min_stem + prefix_len > len) return 0x719986aa;
    return hash32_partial_unicode(p, prefix_len);
}

static inline uint64_t hash64_partial_unicode_prefix(
        const uint32_t *p,
        size_t len,
        size_t prefix_len,
        size_t min_stem)
{
    if (min_stem + prefix_len > len) return 0xc1a7bd3b4e853fc9ULL;
    return hash64_partial_unicode(p, prefix_len);
}

static inline uint32_t read32(const void *p) {
    uint32_t x;
    memcpy(&x, p, sizeof(x));
    return x;
}

static inline uint64_t read64(const void *p) {
    uint64_t x;
    memcpy(&x, p, sizeof(x));
    return x;
}

static inline uint32_t read32_part(const void *p, size_t len) {
    const uint8_t *b = p;
    if (len == 3) return b[0] | (b[1] << 8) | (b[2] << 16);
    else if (len == 2) return b[0] | (b[1] << 8);
    return b[0];
}

static inline uint64_t read64_part(const void *p, size_t len) {
    const uint8_t *b = p;
    uint64_t x = b[0];
    switch (len) {
        case 6: x |= (uint64_t)b[6] << 48;
        case 5: x |= (uint64_t)b[5] << 40;
        case 4: x |= (uint64_t)b[4] << 32;
        case 3: x |= (uint64_t)b[3] << 24;
        case 2: x |= (uint64_t)b[2] << 16;
        case 1: x |= (uint64_t)b[1] << 8;
    }
    return x;
}

static uint32_t hash32_data(uint32_t seed, const void *ptr, size_t len) {
    uint32_t h1 = seed;
    size_t i;

    for (i=0; i+4<=len; i+=4)
        h1 = hash32_mix(read32(ptr+i), h1);

    if (len-i)
        h1 = hash32_mix_tail(read32_part(ptr+i, len-i), h1);

    return hash32_fmix(h1 ^ (uint32_t)len);
}

static uint64_t hash64_data(uint64_t seed, const void *ptr, size_t len) {
    uint64_t h1 = seed;
    size_t i;

    for (i=0; i+8<=len; i+=4)
        h1 = hash64_mix(read64(ptr+i), h1);

    if (len-i)
        h1 = hash64_mix_tail(read64_part(ptr+i, len-i), h1);

    return hash64_fmix(h1 ^ (uint64_t)len);
}

// TODO: less naive implementation, if necessary
static uint32_t hash32_utf8_prefix(
        const void *p,
        size_t len,
        size_t prefix_len,
        size_t min_stem)
{
    uint32_t buf[len];
    size_t buf_len = len;
    utf8_decode(buf, &buf_len, p, len);
    return hash32_fmix(
            hash32_partial_unicode_prefix(buf, buf_len, prefix_len, min_stem));
}

// TODO: less naive implementation, if necessary
static uint32_t hash32_utf8_suffix(
        const void *p,
        size_t len,
        size_t suffix_len,
        size_t min_stem)
{
    uint32_t buf[len];
    size_t buf_len = len;
    utf8_decode(buf, &buf_len, p, len);
    return hash32_fmix(
            hash32_partial_unicode_suffix(buf, buf_len, suffix_len, min_stem));
}

#endif

#ifndef __HEADERS_H__
#define __HEADERS_H__

// Using branching hints gives a significant (~10%) speed improvement on my
// system since the innermost loop uses plenty of them.
#define likely(x)       __builtin_expect((x),1)
#define unlikely(x)     __builtin_expect((x),0)

#define MAX_STR         0x1000

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include <float.h>

// Uncomment this to do perform separate training over a number of weight
// vector lengths, choosing the best model (on the dev set):
//#define MIN_WEIGHTS_LEN     0x40000
//#define MAX_WEIGHTS_LEN     0x4000000

// Or just fix the length:
#define MIN_WEIGHTS_LEN     0x4000000
#define MAX_WEIGHTS_LEN     0x4000000

// Uncomment this to compress the weight vector after training:
#define POST_TRAINING_COMPRESSION

// A non-zero value means dropout is used, but empirically this doesn't seem
// to help
#define DROPOUT_RATE        0
#define DROPOUT_CONSTANT    ((uint32_t)((double)DROPOUT_RATE*4294967296.0))

typedef float real;
#define REAL_MAX        FLT_MAX

typedef struct {
    uint32_t hash;
    const void *value;
} hash32_kv;

typedef struct {
    uint64_t hash;
    const void *value;
} hash64_kv;

typedef struct {
    uint32_t hash;
    label value;
} hash32_kv_label;

typedef struct {
    uint64_t hash;
    label value;
} hash64_kv_label;

static inline real get_score(
        const feat_hash_t *hashes,
        size_t len,
        const real *weights,
        size_t weights_len,
        int use_dropout,
        feat_hash_t dropout_seed)
{
    size_t i;
    const feat_hash_t mask = (feat_hash_t)weights_len - 1;
    if (use_dropout) {
        real sum = (real)0.0;
        for (i=0; i<len; i++) {
            const feat_hash_t h = hashes[i];
            if (hash32_mix(dropout_seed, h) >= DROPOUT_CONSTANT)
                sum += weights[h & mask];
        }
        // NOTE: since the scale of the feature vector does not matter, we do
        //       not scale the sum here.
        return sum;
    } else {
        real sum = weights[hashes[0] & mask];
        for (i=1; i<len; i++) sum += weights[hashes[i] & mask];
        return sum;
    }
}


#endif

static const uint32_t normalize_tab[] = {
    0,
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
    10,
    11,
    12,
    13,
    14,
    15,
    16,
    17,
    18,
    19,
    20,
    21,
    22,
    23,
    24,
    25,
    26,
    27,
    28,
    29,
    30,
    31,
    32,
    33,
    34,
    35,
    36,
    37,
    38,
    39,
    40,
    41,
    42,
    43,
    44,
    45,
    46,
    47,
    48,
    49,
    50,
    51,
    52,
    53,
    54,
    55,
    56,
    57,
    58,
    59,
    60,
    61,
    62,
    63,
    64,
    97,
    98,
    99,
    100,
    101,
    102,
    103,
    104,
    105,
    106,
    107,
    108,
    109,
    110,
    111,
    112,
    113,
    114,
    115,
    116,
    117,
    118,
    119,
    120,
    121,
    122,
    91,
    92,
    93,
    94,
    95,
    96,
    97,
    98,
    99,
    100,
    101,
    102,
    103,
    104,
    105,
    106,
    107,
    108,
    109,
    110,
    111,
    112,
    113,
    114,
    115,
    116,
    117,
    118,
    119,
    120,
    121,
    122,
    123,
    124,
    125,
    126,
    127,
    128,
    129,
    130,
    131,
    132,
    133,
    134,
    135,
    136,
    137,
    138,
    139,
    140,
    141,
    142,
    143,
    144,
    145,
    146,
    147,
    148,
    149,
    150,
    151,
    152,
    153,
    154,
    155,
    156,
    157,
    158,
    159,
    160,
    161,
    162,
    163,
    164,
    165,
    166,
    167,
    168,
    169,
    170,
    171,
    172,
    173,
    174,
    175,
    176,
    177,
    178,
    179,
    180,
    181,
    182,
    183,
    184,
    185,
    186,
    187,
    188,
    189,
    190,
    191,
    224,
    225,
    226,
    227,
    228,
    229,
    230,
    231,
    232,
    233,
    234,
    235,
    236,
    237,
    238,
    239,
    240,
    241,
    242,
    243,
    244,
    245,
    246,
    215,
    248,
    249,
    250,
    251,
    252,
    253,
    254,
    223,
    224,
    225,
    226,
    227,
    228,
    229,
    230,
    231,
    232,
    233,
    234,
    235,
    236,
    237,
    238,
    239,
    240,
    241,
    242,
    243,
    244,
    245,
    246,
    247,
    248,
    249,
    250,
    251,
    252,
    253,
    254,
    255,
    257,
    257,
    259,
    259,
    261,
    261,
    263,
    263,
    265,
    265,
    267,
    267,
    269,
    269,
    271,
    271,
    273,
    273,
    275,
    275,
    277,
    277,
    279,
    279,
    281,
    281,
    283,
    283,
    285,
    285,
    287,
    287,
    289,
    289,
    291,
    291,
    293,
    293,
    295,
    295,
    297,
    297,
    299,
    299,
    301,
    301,
    303,
    303,
    105,
    305,
    307,
    307,
    309,
    309,
    311,
    311,
    312,
    314,
    314,
    316,
    316,
    318,
    318,
    320,
    320,
    322,
    322,
    324,
    324,
    326,
    326,
    328,
    328,
    329,
    331,
    331,
    333,
    333,
    335,
    335,
    337,
    337,
    339,
    339,
    341,
    341,
    343,
    343,
    345,
    345,
    347,
    347,
    349,
    349,
    351,
    351,
    353,
    353,
    355,
    355,
    357,
    357,
    359,
    359,
    361,
    361,
    363,
    363,
    365,
    365,
    367,
    367,
    369,
    369,
    371,
    371,
    373,
    373,
    375,
    375,
    255,
    378,
    378,
    380,
    380,
    382,
    382,
    383,
    384,
    595,
    387,
    387,
    389,
    389,
    596,
    392,
    392,
    598,
    599,
    396,
    396,
    397,
    477,
    601,
    603,
    402,
    402,
    608,
    611,
    405,
    617,
    616,
    409,
    409,
    410,
    411,
    623,
    626,
    414,
    629,
    417,
    417,
    419,
    419,
    421,
    421,
    640,
    424,
    424,
    643,
    426,
    427,
    429,
    429,
    648,
    432,
    432,
    650,
    651,
    436,
    436,
    438,
    438,
    658,
    441,
    441,
    442,
    443,
    445,
    445,
    446,
    447,
    448,
    449,
    450,
    451,
    454,
    454,
    454,
    457,
    457,
    457,
    460,
    460,
    460,
    462,
    462,
    464,
    464,
    466,
    466,
    468,
    468,
    470,
    470,
    472,
    472,
    474,
    474,
    476,
    476,
    477,
    479,
    479,
    481,
    481,
    483,
    483,
    485,
    485,
    487,
    487,
    489,
    489,
    491,
    491,
    493,
    493,
    495,
    495,
    496,
    499,
    499,
    499,
    501,
    501,
    405,
    447,
    505,
    505,
    507,
    507,
    509,
    509,
    511,
    511,
    513,
    513,
    515,
    515,
    517,
    517,
    519,
    519,
    521,
    521,
    523,
    523,
    525,
    525,
    527,
    527,
    529,
    529,
    531,
    531,
    533,
    533,
    535,
    535,
    537,
    537,
    539,
    539,
    541,
    541,
    543,
    543,
    414,
    545,
    547,
    547,
    549,
    549,
    551,
    551,
    553,
    553,
    555,
    555,
    557,
    557,
    559,
    559,
    561,
    561,
    563,
    563,
    564,
    565,
    566,
    567,
    568,
    569,
    11365,
    572,
    572,
    410,
    11366,
    575,
    576,
    578,
    578,
    384,
    649,
    652,
    583,
    583,
    585,
    585,
    587,
    587,
    589,
    589,
    591,
    591,
    592,
    593,
    594,
    595,
    596,
    597,
    598,
    599,
    600,
    601,
    602,
    603,
    604,
    605,
    606,
    607,
    608,
    609,
    610,
    611,
    612,
    613,
    614,
    615,
    616,
    617,
    618,
    619,
    620,
    621,
    622,
    623,
    624,
    625,
    626,
    627,
    628,
    629,
    630,
    631,
    632,
    633,
    634,
    635,
    636,
    637,
    638,
    639,
    640,
    641,
    642,
    643,
    644,
    645,
    646,
    647,
    648,
    649,
    650,
    651,
    652,
    653,
    654,
    655,
    656,
    657,
    658,
    659,
    660,
    661,
    662,
    663,
    664,
    665,
    666,
    667,
    668,
    669,
    670,
    671,
    672,
    673,
    674,
    675,
    676,
    677,
    678,
    679,
    680,
    681,
    682,
    683,
    684,
    685,
    686,
    687,
    688,
    689,
    690,
    691,
    692,
    693,
    694,
    695,
    696,
    697,
    698,
    699,
    700,
    701,
    702,
    703,
    704,
    705,
    706,
    707,
    708,
    709,
    710,
    711,
    712,
    713,
    714,
    715,
    716,
    717,
    718,
    719,
    720,
    721,
    722,
    723,
    724,
    725,
    726,
    727,
    728,
    729,
    730,
    731,
    732,
    733,
    734,
    735,
    736,
    737,
    738,
    739,
    740,
    741,
    742,
    743,
    744,
    745,
    746,
    747,
    748,
    749,
    750,
    751,
    752,
    753,
    754,
    755,
    756,
    757,
    758,
    759,
    760,
    761,
    762,
    763,
    764,
    765,
    766,
    767,
    768,
    769,
    770,
    771,
    772,
    773,
    774,
    775,
    776,
    777,
    778,
    779,
    780,
    781,
    782,
    783,
    784,
    785,
    786,
    787,
    788,
    789,
    790,
    791,
    792,
    793,
    794,
    795,
    796,
    797,
    798,
    799,
    800,
    801,
    802,
    803,
    804,
    805,
    806,
    807,
    808,
    809,
    810,
    811,
    812,
    813,
    814,
    815,
    816,
    817,
    818,
    819,
    820,
    821,
    822,
    823,
    824,
    825,
    826,
    827,
    828,
    829,
    830,
    831,
    768,
    769,
    834,
    787,
    776,
    837,
    838,
    839,
    840,
    841,
    842,
    843,
    844,
    845,
    846,
    847,
    848,
    849,
    850,
    851,
    852,
    853,
    854,
    855,
    856,
    857,
    858,
    859,
    860,
    861,
    862,
    863,
    864,
    865,
    866,
    867,
    868,
    869,
    870,
    871,
    872,
    873,
    874,
    875,
    876,
    877,
    878,
    879,
    881,
    881,
    883,
    883,
    697,
    885,
    887,
    887,
    888,
    889,
    890,
    891,
    892,
    893,
    59,
    895,
    896,
    897,
    898,
    899,
    900,
    901,
    940,
    183,
    941,
    942,
    943,
    907,
    972,
    909,
    973,
    974,
    912,
    945,
    946,
    947,
    948,
    949,
    950,
    951,
    952,
    953,
    954,
    955,
    956,
    957,
    958,
    959,
    960,
    961,
    930,
    963,
    964,
    965,
    966,
    967,
    968,
    969,
    970,
    971,
    940,
    941,
    942,
    943,
    944,
    945,
    946,
    947,
    948,
    949,
    950,
    951,
    952,
    953,
    954,
    955,
    956,
    957,
    958,
    959,
    960,
    961,
    962,
    963,
    964,
    965,
    966,
    967,
    968,
    969,
    970,
    971,
    972,
    973,
    974,
    983,
    976,
    977,
    978,
    979,
    980,
    981,
    982,
    983,
    985,
    985,
    987,
    987,
    989,
    989,
    991,
    991,
    993,
    993,
    995,
    995,
    997,
    997,
    999,
    999,
    1001,
    1001,
    1003,
    1003,
    1005,
    1005,
    1007,
    1007,
    1008,
    1009,
    1010,
    1011,
    952,
    1013,
    1014,
    1016,
    1016,
    1010,
    1019,
    1019,
    1020,
    891,
    892,
    893,
    1104,
    1105,
    1106,
    1107,
    1108,
    1109,
    1110,
    1111,
    1112,
    1113,
    1114,
    1115,
    1116,
    1117,
    1118,
    1119,
    1072,
    1073,
    1074,
    1075,
    1076,
    1077,
    1078,
    1079,
    1080,
    1081,
    1082,
    1083,
    1084,
    1085,
    1086,
    1087,
    1088,
    1089,
    1090,
    1091,
    1092,
    1093,
    1094,
    1095,
    1096,
    1097,
    1098,
    1099,
    1100,
    1101,
    1102,
    1103,
    1072,
    1073,
    1074,
    1075,
    1076,
    1077,
    1078,
    1079,
    1080,
    1081,
    1082,
    1083,
    1084,
    1085,
    1086,
    1087,
    1088,
    1089,
    1090,
    1091,
    1092,
    1093,
    1094,
    1095,
    1096,
    1097,
    1098,
    1099,
    1100,
    1101,
    1102,
    1103,
    1104,
    1105,
    1106,
    1107,
    1108,
    1109,
    1110,
    1111,
    1112,
    1113,
    1114,
    1115,
    1116,
    1117,
    1118,
    1119,
    1121,
    1121,
    1123,
    1123,
    1125,
    1125,
    1127,
    1127,
    1129,
    1129,
    1131,
    1131,
    1133,
    1133,
    1135,
    1135,
    1137,
    1137,
    1139,
    1139,
    1141,
    1141,
    1143,
    1143,
    1145,
    1145,
    1147,
    1147,
    1149,
    1149,
    1151,
    1151,
    1153,
    1153,
    1154,
    1155,
    1156,
    1157,
    1158,
    1159,
    1160,
    1161,
    1163,
    1163,
    1165,
    1165,
    1167,
    1167,
    1169,
    1169,
    1171,
    1171,
    1173,
    1173,
    1175,
    1175,
    1177,
    1177,
    1179,
    1179,
    1181,
    1181,
    1183,
    1183,
    1185,
    1185,
    1187,
    1187,
    1189,
    1189,
    1191,
    1191,
    1193,
    1193,
    1195,
    1195,
    1197,
    1197,
    1199,
    1199,
    1201,
    1201,
    1203,
    1203,
    1205,
    1205,
    1207,
    1207,
    1209,
    1209,
    1211,
    1211,
    1213,
    1213,
    1215,
    1215,
    1231,
    1218,
    1218,
    1220,
    1220,
    1222,
    1222,
    1224,
    1224,
    1226,
    1226,
    1228,
    1228,
    1230,
    1230,
    1231,
    1233,
    1233,
    1235,
    1235,
    1237,
    1237,
    1239,
    1239,
    1241,
    1241,
    1243,
    1243,
    1245,
    1245,
    1247,
    1247,
    1249,
    1249,
    1251,
    1251,
    1253,
    1253,
    1255,
    1255,
    1257,
    1257,
    1259,
    1259,
    1261,
    1261,
    1263,
    1263,
    1265,
    1265,
    1267,
    1267,
    1269,
    1269,
    1271,
    1271,
    1273,
    1273,
    1275,
    1275,
    1277,
    1277,
    1279,
    1279,
    1281,
    1281,
    1283,
    1283,
    1285,
    1285,
    1287,
    1287,
    1289,
    1289,
    1291,
    1291,
    1293,
    1293,
    1295,
    1295,
    1297,
    1297,
    1299,
    1299,
    1301,
    1301,
    1303,
    1303,
    1305,
    1305,
    1307,
    1307,
    1309,
    1309,
    1311,
    1311,
    1313,
    1313,
    1315,
    1315,
    1317,
    1317,
    1319,
    1319,
    1320,
    1321,
    1322,
    1323,
    1324,
    1325,
    1326,
    1327
};

static const uint8_t delex_tab[] = {
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    29,
    21,
    21,
    21,
    23,
    21,
    21,
    21,
    22,
    18,
    21,
    25,
    21,
    17,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    25,
    25,
    25,
    21,
    21,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    22,
    21,
    18,
    24,
    16,
    24,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    22,
    25,
    18,
    25,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    29,
    21,
    23,
    23,
    23,
    23,
    26,
    21,
    24,
    26,
    7,
    20,
    25,
    1,
    26,
    24,
    26,
    25,
    15,
    15,
    24,
    5,
    21,
    21,
    24,
    15,
    7,
    19,
    15,
    15,
    15,
    21,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    25,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    25,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    9,
    9,
    5,
    5,
    9,
    9,
    9,
    9,
    5,
    9,
    9,
    5,
    9,
    9,
    9,
    5,
    5,
    5,
    9,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    5,
    7,
    9,
    5,
    5,
    5,
    7,
    7,
    7,
    7,
    9,
    8,
    5,
    9,
    8,
    5,
    9,
    8,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    8,
    5,
    9,
    5,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    5,
    9,
    9,
    5,
    5,
    9,
    5,
    9,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    7,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    24,
    24,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    6,
    24,
    6,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    9,
    5,
    9,
    5,
    6,
    24,
    9,
    5,
    2,
    2,
    6,
    5,
    5,
    5,
    21,
    2,
    2,
    2,
    2,
    2,
    24,
    24,
    9,
    21,
    9,
    9,
    9,
    2,
    9,
    2,
    9,
    9,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    5,
    9,
    9,
    9,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    25,
    9,
    5,
    9,
    9,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    26,
    12,
    12,
    12,
    12,
    12,
    11,
    11,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    2,
    6,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    21,
    17,
    2,
    2,
    2,
    2,
    23,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    17,
    12,
    21,
    12,
    12,
    21,
    12,
    12,
    21,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    1,
    1,
    1,
    1,
    1,
    2,
    25,
    25,
    25,
    21,
    21,
    23,
    21,
    21,
    26,
    26,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    1,
    2,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    21,
    21,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    1,
    26,
    12,
    12,
    12,
    12,
    12,
    12,
    6,
    6,
    12,
    12,
    26,
    12,
    12,
    12,
    12,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    26,
    26,
    7,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    1,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    6,
    6,
    26,
    21,
    21,
    21,
    6,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    6,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    6,
    12,
    12,
    12,
    6,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    2,
    2,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    12,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    12,
    10,
    10,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    12,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    2,
    2,
    10,
    10,
    2,
    2,
    10,
    10,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    23,
    23,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    23,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    2,
    2,
    12,
    2,
    10,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    2,
    12,
    12,
    2,
    2,
    12,
    12,
    12,
    2,
    2,
    2,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    12,
    12,
    7,
    7,
    7,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    10,
    2,
    10,
    10,
    12,
    2,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    23,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    12,
    10,
    12,
    12,
    12,
    12,
    2,
    2,
    10,
    10,
    2,
    2,
    10,
    10,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    10,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    26,
    7,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    10,
    10,
    12,
    10,
    10,
    2,
    2,
    2,
    10,
    10,
    10,
    2,
    10,
    10,
    10,
    12,
    2,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    23,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    2,
    12,
    12,
    12,
    2,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    2,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    2,
    2,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    7,
    10,
    12,
    10,
    10,
    10,
    10,
    10,
    2,
    12,
    10,
    10,
    2,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    2,
    10,
    10,
    10,
    2,
    10,
    10,
    10,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    10,
    10,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    12,
    2,
    2,
    2,
    2,
    10,
    10,
    10,
    12,
    12,
    12,
    2,
    12,
    2,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    23,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    2,
    7,
    2,
    2,
    7,
    7,
    2,
    7,
    2,
    2,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    2,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    6,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    26,
    26,
    26,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    26,
    21,
    26,
    26,
    26,
    12,
    12,
    26,
    26,
    26,
    26,
    26,
    26,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    12,
    26,
    12,
    26,
    12,
    22,
    18,
    22,
    18,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    21,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    12,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    21,
    21,
    21,
    21,
    21,
    26,
    26,
    26,
    26,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    10,
    10,
    12,
    12,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    21,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    12,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    7,
    10,
    10,
    10,
    7,
    7,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    10,
    12,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    7,
    10,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    10,
    10,
    10,
    12,
    26,
    26,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    9,
    2,
    2,
    2,
    2,
    2,
    9,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    12,
    12,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    17,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    29,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    22,
    18,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    21,
    14,
    14,
    14,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    2,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    21,
    6,
    21,
    21,
    21,
    23,
    7,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    17,
    21,
    21,
    21,
    21,
    12,
    12,
    12,
    1,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    12,
    12,
    10,
    10,
    10,
    2,
    2,
    2,
    2,
    10,
    10,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    26,
    2,
    2,
    2,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    15,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    10,
    10,
    12,
    2,
    2,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    12,
    10,
    12,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    12,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    6,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    10,
    10,
    10,
    10,
    10,
    12,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    12,
    10,
    12,
    10,
    10,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    12,
    12,
    10,
    10,
    10,
    12,
    10,
    12,
    12,
    12,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    7,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    6,
    6,
    6,
    6,
    6,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    21,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    9,
    2,
    9,
    2,
    9,
    2,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    8,
    5,
    5,
    5,
    5,
    5,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    8,
    24,
    5,
    24,
    24,
    24,
    5,
    5,
    5,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    8,
    24,
    24,
    24,
    5,
    5,
    5,
    5,
    2,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    2,
    24,
    24,
    24,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    9,
    9,
    9,
    9,
    24,
    24,
    24,
    2,
    2,
    5,
    5,
    5,
    2,
    5,
    5,
    9,
    9,
    9,
    9,
    8,
    24,
    24,
    2,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    29,
    1,
    1,
    1,
    1,
    1,
    17,
    17,
    17,
    17,
    17,
    17,
    21,
    21,
    20,
    19,
    22,
    20,
    20,
    19,
    22,
    20,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    27,
    28,
    1,
    1,
    1,
    1,
    1,
    29,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    20,
    19,
    21,
    21,
    21,
    21,
    16,
    16,
    21,
    21,
    21,
    25,
    22,
    18,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    25,
    21,
    16,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    29,
    1,
    1,
    1,
    1,
    1,
    2,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    15,
    6,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    25,
    25,
    25,
    22,
    18,
    6,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    25,
    25,
    25,
    22,
    18,
    2,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    2,
    2,
    2,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    23,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    11,
    11,
    11,
    11,
    12,
    11,
    11,
    11,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    9,
    26,
    26,
    26,
    26,
    9,
    26,
    26,
    5,
    9,
    9,
    9,
    5,
    5,
    9,
    9,
    9,
    5,
    26,
    9,
    26,
    26,
    25,
    9,
    9,
    9,
    9,
    9,
    26,
    26,
    26,
    26,
    26,
    26,
    9,
    26,
    9,
    26,
    9,
    26,
    9,
    9,
    9,
    9,
    26,
    5,
    9,
    9,
    9,
    9,
    5,
    7,
    7,
    7,
    7,
    5,
    26,
    26,
    5,
    5,
    9,
    9,
    25,
    25,
    25,
    25,
    25,
    9,
    5,
    5,
    5,
    5,
    26,
    25,
    26,
    26,
    5,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    9,
    5,
    14,
    14,
    14,
    14,
    15,
    2,
    2,
    2,
    2,
    2,
    2,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    25,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    26,
    26,
    25,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    22,
    18,
    22,
    18,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    22,
    18,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    22,
    18,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    25,
    26,
    26,
    25,
    25,
    25,
    25,
    25,
    25,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    9,
    5,
    9,
    9,
    9,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    9,
    9,
    5,
    9,
    5,
    5,
    9,
    5,
    5,
    5,
    5,
    5,
    5,
    6,
    6,
    9,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    26,
    26,
    26,
    26,
    26,
    26,
    9,
    5,
    9,
    5,
    12,
    12,
    12,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    21,
    21,
    15,
    21,
    21,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    5,
    2,
    2,
    2,
    2,
    2,
    5,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    6,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    20,
    19,
    20,
    19,
    21,
    21,
    21,
    20,
    19,
    21,
    20,
    19,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    17,
    21,
    21,
    17,
    21,
    20,
    19,
    21,
    21,
    20,
    19,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    21,
    21,
    21,
    21,
    21,
    6,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    17,
    17,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    29,
    21,
    21,
    21,
    26,
    6,
    7,
    14,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    26,
    26,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    17,
    22,
    18,
    18,
    26,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    12,
    12,
    12,
    12,
    10,
    10,
    17,
    6,
    6,
    6,
    6,
    6,
    26,
    26,
    14,
    14,
    14,
    6,
    7,
    21,
    26,
    26,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    12,
    12,
    24,
    24,
    6,
    6,
    7,
    17,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    6,
    6,
    6,
    7,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    26,
    26,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    6,
    6,
    6,
    6,
    6,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    7,
    12,
    11,
    11,
    11,
    21,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    6,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    14,
    12,
    12,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    24,
    24,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    6,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    9,
    5,
    9,
    5,
    9,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    6,
    24,
    24,
    9,
    5,
    9,
    5,
    2,
    9,
    5,
    9,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    5,
    9,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    6,
    6,
    5,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    12,
    10,
    26,
    26,
    26,
    26,
    2,
    2,
    2,
    2,
    15,
    15,
    15,
    15,
    15,
    15,
    26,
    26,
    23,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    10,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    10,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    21,
    21,
    21,
    7,
    2,
    2,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    12,
    12,
    12,
    10,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    10,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    10,
    10,
    10,
    10,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    2,
    6,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    21,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    12,
    12,
    12,
    12,
    10,
    10,
    12,
    12,
    10,
    10,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    10,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    21,
    21,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    26,
    26,
    26,
    7,
    10,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    12,
    7,
    12,
    12,
    12,
    7,
    7,
    12,
    12,
    7,
    7,
    7,
    7,
    7,
    12,
    12,
    7,
    12,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    6,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    12,
    12,
    10,
    10,
    21,
    21,
    7,
    6,
    6,
    10,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    10,
    10,
    12,
    10,
    10,
    12,
    10,
    10,
    21,
    10,
    12,
    2,
    2,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5,
    5,
    5,
    5,
    5,
    2,
    2,
    2,
    2,
    2,
    7,
    12,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    25,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    24,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    22,
    18,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    23,
    26,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    21,
    21,
    21,
    21,
    21,
    21,
    21,
    22,
    18,
    21,
    2,
    2,
    2,
    2,
    2,
    2,
    12,
    12,
    12,
    12,
    12,
    12,
    12,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    21,
    17,
    17,
    16,
    16,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    22,
    18,
    21,
    21,
    22,
    18,
    21,
    21,
    21,
    21,
    16,
    16,
    16,
    21,
    21,
    21,
    2,
    21,
    21,
    21,
    21,
    17,
    22,
    18,
    22,
    18,
    22,
    18,
    21,
    21,
    21,
    25,
    17,
    25,
    25,
    25,
    2,
    21,
    23,
    21,
    21,
    2,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    1,
    2,
    21,
    21,
    21,
    23,
    21,
    21,
    21,
    22,
    18,
    21,
    25,
    21,
    17,
    21,
    21,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    13,
    21,
    21,
    25,
    25,
    25,
    21,
    21,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    9,
    22,
    21,
    18,
    24,
    16,
    24,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    22,
    25,
    18,
    25,
    22,
    18,
    21,
    22,
    18,
    21,
    21,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    6,
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    7,
    7,
    7,
    2,
    2,
    7,
    7,
    7,
    2,
    2,
    2,
    23,
    23,
    25,
    24,
    26,
    23,
    23,
    2,
    26,
    25,
    25,
    25,
    25,
    26,
    26,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    1,
    1,
    1,
    26,
    26,
    2,
    2
};


static inline void unicode_translate_normalize(
        uint32_t *dest,
        size_t *dest_len,
        const uint32_t *src,
        size_t src_len)
{
    size_t i, len = (src_len <= *dest_len)? src_len : *dest_len;
    for (i=0; i<len; i++) {
        if (likely(src[i] < 0x530)) dest[i] = normalize_tab[src[i]];
        else dest[i] = src[i];
    }
    *dest_len = len;
}

static inline void unicode_translate_delexicalize(
        uint32_t *dest,
        size_t *dest_len,
        const uint32_t *src,
        size_t src_len)
{
    size_t i, len = (src_len <= *dest_len)? src_len : *dest_len;
    for (i=0; i<len; i++) {
        if (likely(src[i] < 0x10000)) dest[i] = delex_tab[src[i]];
        else dest[i] = 30;
    }
    *dest_len = len;
}

static inline void unicode_translate_abstract(
        uint32_t *dest,
        size_t *dest_len,
        const uint32_t *src,
        size_t src_len)
{
    size_t i, n = 0;
    uint8_t last = 0xff;

    for (i=0; i<src_len; i++) {
        const uint8_t c = (likely(src[i] < 0x10000))? delex_tab[src[i]] : 30;
        if (c != last) {
            dest[n++] = c;
            last = c;
            if (n >= *dest_len) break;
        }
    }
    *dest_len = n;
}

#define COL_TAG 1
#define suc_N_TAGS 165
#define suc_N_BITS 8
#define N_TAGS suc_N_TAGS
#define tag_str suc_tag_str

static const char *suc_tag_str[suc_N_TAGS] = {
    "AB",
    "AB|AN",
    "AB|KOM",
    "AB|POS",
    "AB|SMS",
    "AB|SUV",
    "DT|AN",
    "DT|MAS|SIN|DEF",
    "DT|MAS|SIN|IND",
    "DT|NEU|SIN|DEF",
    "DT|NEU|SIN|IND",
    "DT|NEU|SIN|IND/DEF",
    "DT|UTR/NEU|PLU|DEF",
    "DT|UTR/NEU|PLU|IND",
    "DT|UTR/NEU|PLU|IND/DEF",
    "DT|UTR/NEU|SIN/PLU|IND",
    "DT|UTR/NEU|SIN|DEF",
    "DT|UTR/NEU|SIN|IND",
    "DT|UTR|SIN|DEF",
    "DT|UTR|SIN|IND",
    "DT|UTR|SIN|IND/DEF",
    "HA",
    "HD|NEU|SIN|IND",
    "HD|UTR/NEU|PLU|IND",
    "HD|UTR|SIN|IND",
    "HP|-|-|-",
    "HP|NEU|SIN|IND",
    "HP|NEU|SIN|IND|SMS",
    "HP|UTR/NEU|PLU|IND",
    "HP|UTR|SIN|IND",
    "HS|DEF",
    "IE",
    "IN",
    "JJ|AN",
    "JJ|KOM|UTR/NEU|SIN/PLU|IND/DEF|GEN",
    "JJ|KOM|UTR/NEU|SIN/PLU|IND/DEF|NOM",
    "JJ|KOM|UTR/NEU|SIN/PLU|IND/DEF|SMS",
    "JJ|POS|MAS|SIN|DEF|GEN",
    "JJ|POS|MAS|SIN|DEF|NOM",
    "JJ|POS|NEU|SIN|IND/DEF|NOM",
    "JJ|POS|NEU|SIN|IND|GEN",
    "JJ|POS|NEU|SIN|IND|NOM",
    "JJ|POS|UTR/NEU|-|-|SMS",
    "JJ|POS|UTR/NEU|PLU|DEF|GEN",
    "JJ|POS|UTR/NEU|PLU|DEF|NOM",
    "JJ|POS|UTR/NEU|PLU|IND/DEF|GEN",
    "JJ|POS|UTR/NEU|PLU|IND/DEF|NOM",
    "JJ|POS|UTR/NEU|PLU|IND|GEN",
    "JJ|POS|UTR/NEU|PLU|IND|NOM",
    "JJ|POS|UTR/NEU|SIN/PLU|IND/DEF|NOM",
    "JJ|POS|UTR/NEU|SIN/PLU|IND|NOM",
    "JJ|POS|UTR/NEU|SIN|DEF|GEN",
    "JJ|POS|UTR/NEU|SIN|DEF|NOM",
    "JJ|POS|UTR|-|-|SMS",
    "JJ|POS|UTR|SIN|IND/DEF|NOM",
    "JJ|POS|UTR|SIN|IND|GEN",
    "JJ|POS|UTR|SIN|IND|NOM",
    "JJ|SUV|MAS|SIN/PLU|DEF|GEN",
    "JJ|SUV|MAS|SIN/PLU|DEF|NOM",
    "JJ|SUV|MAS|SIN|DEF|GEN",
    "JJ|SUV|MAS|SIN|DEF|NOM",
    "JJ|SUV|UTR/NEU|PLU|DEF|NOM",
    "JJ|SUV|UTR/NEU|PLU|IND|NOM",
    "JJ|SUV|UTR/NEU|SIN/PLU|DEF|GEN",
    "JJ|SUV|UTR/NEU|SIN/PLU|DEF|NOM",
    "JJ|SUV|UTR/NEU|SIN/PLU|IND|GEN",
    "JJ|SUV|UTR/NEU|SIN/PLU|IND|NOM",
    "KN",
    "KN|AN",
    "MAD",
    "MID",
    "NN|-|-|-|-",
    "NN|-|-|-|SMS",
    "NN|AN",
    "NN|NEU|-|-|-",
    "NN|NEU|-|-|SMS",
    "NN|NEU|PLU|DEF|GEN",
    "NN|NEU|PLU|DEF|NOM",
    "NN|NEU|PLU|IND|GEN",
    "NN|NEU|PLU|IND|NOM",
    "NN|NEU|SIN|DEF|GEN",
    "NN|NEU|SIN|DEF|NOM",
    "NN|NEU|SIN|IND|GEN",
    "NN|NEU|SIN|IND|NOM",
    "NN|UTR|-|-|-",
    "NN|UTR|-|-|SMS",
    "NN|UTR|PLU|DEF|GEN",
    "NN|UTR|PLU|DEF|NOM",
    "NN|UTR|PLU|IND|GEN",
    "NN|UTR|PLU|IND|NOM",
    "NN|UTR|SIN|DEF|GEN",
    "NN|UTR|SIN|DEF|NOM",
    "NN|UTR|SIN|IND|GEN",
    "NN|UTR|SIN|IND|NOM",
    "PAD",
    "PC|AN",
    "PC|PRF|MAS|SIN|DEF|GEN",
    "PC|PRF|MAS|SIN|DEF|NOM",
    "PC|PRF|NEU|SIN|IND|GEN",
    "PC|PRF|NEU|SIN|IND|NOM",
    "PC|PRF|UTR/NEU|PLU|DEF|GEN",
    "PC|PRF|UTR/NEU|PLU|DEF|NOM",
    "PC|PRF|UTR/NEU|PLU|IND/DEF|GEN",
    "PC|PRF|UTR/NEU|PLU|IND/DEF|NOM",
    "PC|PRF|UTR/NEU|PLU|IND|GEN",
    "PC|PRF|UTR/NEU|PLU|IND|NOM",
    "PC|PRF|UTR/NEU|SIN|DEF|GEN",
    "PC|PRF|UTR/NEU|SIN|DEF|NOM",
    "PC|PRF|UTR|SIN|IND|GEN",
    "PC|PRF|UTR|SIN|IND|NOM",
    "PC|PRS|UTR/NEU|SIN/PLU|IND/DEF|GEN",
    "PC|PRS|UTR/NEU|SIN/PLU|IND/DEF|NOM",
    "PL",
    "PM|GEN",
    "PM|NOM",
    "PM|SMS",
    "PN|MAS|SIN|DEF|SUB/OBJ",
    "PN|NEU|SIN|DEF|SUB/OBJ",
    "PN|NEU|SIN|IND|SUB/OBJ",
    "PN|UTR/NEU|PLU|DEF|OBJ",
    "PN|UTR/NEU|PLU|DEF|SUB",
    "PN|UTR/NEU|PLU|DEF|SUB/OBJ",
    "PN|UTR/NEU|PLU|IND|SUB/OBJ",
    "PN|UTR/NEU|SIN/PLU|DEF|OBJ",
    "PN|UTR|PLU|DEF|OBJ",
    "PN|UTR|PLU|DEF|SUB",
    "PN|UTR|SIN|DEF|OBJ",
    "PN|UTR|SIN|DEF|SUB",
    "PN|UTR|SIN|DEF|SUB/OBJ",
    "PN|UTR|SIN|IND|SUB",
    "PN|UTR|SIN|IND|SUB/OBJ",
    "PP",
    "PP|AN",
    "PS|AN",
    "PS|NEU|SIN|DEF",
    "PS|UTR/NEU|PLU|DEF",
    "PS|UTR/NEU|SIN/PLU|DEF",
    "PS|UTR|SIN|DEF",
    "RG|GEN",
    "RG|NEU|SIN|IND|NOM",
    "RG|NOM",
    "RG|SMS",
    "RG|UTR|SIN|IND|NOM",
    "RO|GEN",
    "RO|MAS|SIN|IND/DEF|GEN",
    "RO|MAS|SIN|IND/DEF|NOM",
    "RO|NOM",
    "RO|SMS",
    "SN",
    "UO",
    "VB|AN",
    "VB|IMP|AKT",
    "VB|IMP|SFO",
    "VB|INF|AKT",
    "VB|INF|SFO",
    "VB|KON|PRS|AKT",
    "VB|KON|PRT|AKT",
    "VB|KON|PRT|SFO",
    "VB|PRS|AKT",
    "VB|PRS|SFO",
    "VB|PRT|AKT",
    "VB|PRT|SFO",
    "VB|SMS",
    "VB|SUP|AKT",
    "VB|SUP|SFO"
};


static const int suc_from_str(const char *s) {
    int i;
    for (i=0; i<suc_N_TAGS; i++) if (!strcmp(s, suc_tag_str[i])) return i;
    return -1;
}

#define tagset_from_str suc_from_str
#define SUC_lexicon_size 0x200000

static const label SUC_lexicon_tags_0[] = { 1, 77 };
static const label SUC_lexicon_tags_1[] = { 1, 86 };
static const label SUC_lexicon_tags_2[] = { 1, 88 };
static const label SUC_lexicon_tags_3[] = { 2, 164, 98 };
static const label SUC_lexicon_tags_4[] = { 1, 113 };
static const label SUC_lexicon_tags_5[] = { 1, 140 };
static const label SUC_lexicon_tags_6[] = { 1, 65 };
static const label SUC_lexicon_tags_7[] = { 1, 87 };
static const label SUC_lexicon_tags_8[] = { 1, 38 };
static const label SUC_lexicon_tags_9[] = { 1, 66 };
static const label SUC_lexicon_tags_10[] = { 1, 78 };
static const label SUC_lexicon_tags_11[] = { 1, 90 };
static const label SUC_lexicon_tags_12[] = { 1, 89 };
static const label SUC_lexicon_tags_13[] = { 1, 109 };
static const label SUC_lexicon_tags_14[] = { 1, 81 };
static const label SUC_lexicon_tags_15[] = { 1, 92 };
static const label SUC_lexicon_tags_16[] = { 2, 83, 79 };
static const label SUC_lexicon_tags_17[] = { 1, 91 };
static const label SUC_lexicon_tags_18[] = { 1, 73 };
static const label SUC_lexicon_tags_19[] = { 1, 56 };
static const label SUC_lexicon_tags_20[] = { 1, 49 };
static const label SUC_lexicon_tags_21[] = { 1, 83 };
static const label SUC_lexicon_tags_22[] = { 1, 154 };
static const label SUC_lexicon_tags_23[] = { 1, 76 };
static const label SUC_lexicon_tags_24[] = { 2, 97, 160 };
static const label SUC_lexicon_tags_25[] = { 1, 93 };
static const label SUC_lexicon_tags_26[] = { 1, 37 };
static const label SUC_lexicon_tags_27[] = { 1, 75 };
static const label SUC_lexicon_tags_28[] = { 1, 79 };
static const label SUC_lexicon_tags_29[] = { 1, 114 };
static const label SUC_lexicon_tags_30[] = { 1, 35 };
static const label SUC_lexicon_tags_31[] = { 2, 58, 64 };
static const label SUC_lexicon_tags_32[] = { 2, 83, 111 };
static const label SUC_lexicon_tags_33[] = { 1, 55 };
static const label SUC_lexicon_tags_34[] = { 4, 106, 96, 161, 102 };
static const label SUC_lexicon_tags_35[] = { 3, 37, 51, 45 };
static const label SUC_lexicon_tags_36[] = { 2, 162, 85 };
static const label SUC_lexicon_tags_37[] = { 1, 85 };
static const label SUC_lexicon_tags_38[] = { 1, 40 };
static const label SUC_lexicon_tags_39[] = { 2, 82, 78 };
static const label SUC_lexicon_tags_40[] = { 1, 160 };
static const label SUC_lexicon_tags_41[] = { 2, 55, 40 };
static const label SUC_lexicon_tags_42[] = { 1, 164 };
static const label SUC_lexicon_tags_43[] = { 2, 57, 63 };
static const label SUC_lexicon_tags_44[] = { 1, 163 };
static const label SUC_lexicon_tags_45[] = { 1, 41 };
static const label SUC_lexicon_tags_46[] = { 2, 82, 110 };
static const label SUC_lexicon_tags_47[] = { 1, 80 };
static const label SUC_lexicon_tags_48[] = { 1, 108 };
static const label SUC_lexicon_tags_49[] = { 2, 51, 45 };
static const label SUC_lexicon_tags_50[] = { 2, 56, 55 };
static const label SUC_lexicon_tags_51[] = { 3, 93, 46, 52 };
static const label SUC_lexicon_tags_52[] = { 2, 87, 77 };
static const label SUC_lexicon_tags_53[] = { 1, 149 };
static const label SUC_lexicon_tags_54[] = { 1, 0 };
static const label SUC_lexicon_tags_55[] = { 2, 153, 151 };
static const label SUC_lexicon_tags_56[] = { 4, 97, 103, 107, 160 };
static const label SUC_lexicon_tags_57[] = { 1, 162 };
static const label SUC_lexicon_tags_58[] = { 3, 163, 99, 41 };
static const label SUC_lexicon_tags_59[] = { 2, 114, 113 };
static const label SUC_lexicon_tags_60[] = { 2, 46, 52 };
static const label SUC_lexicon_tags_61[] = { 3, 38, 46, 52 };
static const label SUC_lexicon_tags_62[] = { 2, 92, 93 };
static const label SUC_lexicon_tags_63[] = { 1, 34 };
static const label SUC_lexicon_tags_64[] = { 2, 103, 107 };
static const label SUC_lexicon_tags_65[] = { 1, 158 };
static const label SUC_lexicon_tags_66[] = { 1, 82 };
static const label SUC_lexicon_tags_67[] = { 3, 163, 99, 109 };
static const label SUC_lexicon_tags_68[] = { 2, 154, 159 };
static const label SUC_lexicon_tags_69[] = { 2, 106, 102 };
static const label SUC_lexicon_tags_70[] = { 1, 153 };
static const label SUC_lexicon_tags_71[] = { 2, 83, 93 };
static const label SUC_lexicon_tags_72[] = { 2, 55, 108 };
static const label SUC_lexicon_tags_73[] = { 4, 38, 107, 46, 52 };
static const label SUC_lexicon_tags_74[] = { 2, 85, 75 };
static const label SUC_lexicon_tags_75[] = { 1, 4 };
static const label SUC_lexicon_tags_76[] = { 1, 64 };
static const label SUC_lexicon_tags_77[] = { 2, 92, 88 };
static const label SUC_lexicon_tags_78[] = { 2, 5, 66 };
static const label SUC_lexicon_tags_79[] = { 1, 161 };
static const label SUC_lexicon_tags_80[] = { 2, 96, 161 };
static const label SUC_lexicon_tags_81[] = { 1, 111 };
static const label SUC_lexicon_tags_82[] = { 2, 89, 93 };
static const label SUC_lexicon_tags_83[] = { 4, 97, 103, 160, 107 };
static const label SUC_lexicon_tags_84[] = { 1, 159 };
static const label SUC_lexicon_tags_85[] = { 2, 3, 41 };
static const label SUC_lexicon_tags_86[] = { 2, 87, 114 };
static const label SUC_lexicon_tags_87[] = { 2, 163, 99 };
static const label SUC_lexicon_tags_88[] = { 3, 92, 51, 45 };
static const label SUC_lexicon_tags_89[] = { 2, 4, 75 };
static const label SUC_lexicon_tags_90[] = { 3, 56, 114, 93 };
static const label SUC_lexicon_tags_91[] = { 2, 90, 80 };
static const label SUC_lexicon_tags_92[] = { 1, 151 };
static const label SUC_lexicon_tags_93[] = { 2, 37, 96 };
static const label SUC_lexicon_tags_94[] = { 4, 153, 151, 46, 52 };
static const label SUC_lexicon_tags_95[] = { 2, 114, 93 };
static const label SUC_lexicon_tags_96[] = { 1, 110 };
static const label SUC_lexicon_tags_97[] = { 2, 83, 82 };
static const label SUC_lexicon_tags_98[] = { 4, 103, 46, 107, 52 };
static const label SUC_lexicon_tags_99[] = { 2, 86, 113 };
static const label SUC_lexicon_tags_100[] = { 7, 96, 102, 161, 45, 37, 51, 106 };
static const label SUC_lexicon_tags_101[] = { 2, 111, 49 };
static const label SUC_lexicon_tags_102[] = { 4, 38, 160, 46, 52 };
static const label SUC_lexicon_tags_103[] = { 1, 96 };
static const label SUC_lexicon_tags_104[] = { 1, 99 };
static const label SUC_lexicon_tags_105[] = { 3, 162, 85, 75 };
static const label SUC_lexicon_tags_106[] = { 2, 162, 75 };
static const label SUC_lexicon_tags_107[] = { 2, 89, 158 };
static const label SUC_lexicon_tags_108[] = { 2, 56, 41 };
static const label SUC_lexicon_tags_109[] = { 2, 99, 41 };
static const label SUC_lexicon_tags_110[] = { 3, 83, 151, 93 };
static const label SUC_lexicon_tags_111[] = { 3, 151, 154, 159 };
static const label SUC_lexicon_tags_112[] = { 2, 91, 93 };
static const label SUC_lexicon_tags_113[] = { 2, 56, 109 };
static const label SUC_lexicon_tags_114[] = { 1, 52 };
static const label SUC_lexicon_tags_115[] = { 2, 79, 77 };
static const label SUC_lexicon_tags_116[] = { 3, 92, 154, 159 };
static const label SUC_lexicon_tags_117[] = { 2, 56, 93 };
static const label SUC_lexicon_tags_118[] = { 1, 97 };
static const label SUC_lexicon_tags_119[] = { 3, 0, 35, 2 };
static const label SUC_lexicon_tags_120[] = { 3, 82, 159, 78 };
static const label SUC_lexicon_tags_121[] = { 2, 90, 86 };
static const label SUC_lexicon_tags_122[] = { 2, 83, 146 };
static const label SUC_lexicon_tags_123[] = { 2, 114, 89 };
static const label SUC_lexicon_tags_124[] = { 1, 98 };
static const label SUC_lexicon_tags_125[] = { 3, 56, 3, 41 };
static const label SUC_lexicon_tags_126[] = { 2, 88, 78 };
static const label SUC_lexicon_tags_127[] = { 4, 103, 97, 107, 160 };
static const label SUC_lexicon_tags_128[] = { 3, 82, 159, 161 };
static const label SUC_lexicon_tags_129[] = { 2, 92, 82 };
static const label SUC_lexicon_tags_130[] = { 5, 104, 106, 96, 100, 161 };
static const label SUC_lexicon_tags_131[] = { 2, 83, 114 };
static const label SUC_lexicon_tags_132[] = { 2, 99, 109 };
static const label SUC_lexicon_tags_133[] = { 2, 76, 86 };
static const label SUC_lexicon_tags_134[] = { 3, 56, 46, 52 };
static const label SUC_lexicon_tags_135[] = { 4, 92, 154, 159, 113 };
static const label SUC_lexicon_tags_136[] = { 4, 106, 161, 96, 102 };
static const label SUC_lexicon_tags_137[] = { 2, 91, 109 };
static const label SUC_lexicon_tags_138[] = { 1, 46 };
static const label SUC_lexicon_tags_139[] = { 4, 118, 0, 3, 41 };
static const label SUC_lexicon_tags_140[] = { 3, 153, 151, 93 };
static const label SUC_lexicon_tags_141[] = { 1, 103 };
static const label SUC_lexicon_tags_142[] = { 4, 164, 98, 159, 108 };
static const label SUC_lexicon_tags_143[] = { 4, 92, 154, 159, 93 };
static const label SUC_lexicon_tags_144[] = { 3, 92, 82, 78 };
static const label SUC_lexicon_tags_145[] = { 3, 153, 151, 158 };
static const label SUC_lexicon_tags_146[] = { 4, 51, 106, 45, 102 };
static const label SUC_lexicon_tags_147[] = { 3, 0, 112, 49 };
static const label SUC_lexicon_tags_148[] = { 2, 92, 90 };
static const label SUC_lexicon_tags_149[] = { 2, 151, 159 };
static const label SUC_lexicon_tags_150[] = { 1, 141 };
static const label SUC_lexicon_tags_151[] = { 2, 91, 81 };
static const label SUC_lexicon_tags_152[] = { 1, 1 };
static const label SUC_lexicon_tags_153[] = { 2, 55, 113 };
static const label SUC_lexicon_tags_154[] = { 3, 37, 96, 161 };
static const label SUC_lexicon_tags_155[] = { 2, 49, 93 };
static const label SUC_lexicon_tags_156[] = { 3, 154, 106, 102 };
static const label SUC_lexicon_tags_157[] = { 3, 149, 114, 113 };
static const label SUC_lexicon_tags_158[] = { 3, 83, 111, 49 };
static const label SUC_lexicon_tags_159[] = { 3, 0, 163, 99 };
static const label SUC_lexicon_tags_160[] = { 4, 83, 79, 82, 78 };
static const label SUC_lexicon_tags_161[] = { 3, 51, 55, 45 };
static const label SUC_lexicon_tags_162[] = { 3, 37, 161, 96 };
static const label SUC_lexicon_tags_163[] = { 2, 80, 98 };
static const label SUC_lexicon_tags_164[] = { 3, 103, 46, 107 };
static const label SUC_lexicon_tags_165[] = { 2, 149, 114 };
static const label SUC_lexicon_tags_166[] = { 3, 83, 79, 56 };
static const label SUC_lexicon_tags_167[] = { 3, 92, 88, 89 };
static const label SUC_lexicon_tags_168[] = { 3, 46, 107, 52 };
static const label SUC_lexicon_tags_169[] = { 1, 121 };
static const label SUC_lexicon_tags_170[] = { 3, 164, 98, 108 };
static const label SUC_lexicon_tags_171[] = { 2, 91, 87 };
static const label SUC_lexicon_tags_172[] = { 2, 92, 113 };
static const label SUC_lexicon_tags_173[] = { 2, 0, 21 };
static const label SUC_lexicon_tags_174[] = { 4, 103, 97, 160, 107 };
static const label SUC_lexicon_tags_175[] = { 2, 41, 81 };
static const label SUC_lexicon_tags_176[] = { 4, 92, 55, 82, 40 };
static const label SUC_lexicon_tags_177[] = { 2, 114, 158 };
static const label SUC_lexicon_tags_178[] = { 4, 51, 154, 159, 45 };
static const label SUC_lexicon_tags_179[] = { 2, 91, 114 };
static const label SUC_lexicon_tags_180[] = { 2, 49, 52 };
static const label SUC_lexicon_tags_181[] = { 2, 38, 93 };
static const label SUC_lexicon_tags_182[] = { 3, 97, 38, 160 };
static const label SUC_lexicon_tags_183[] = { 2, 82, 113 };
static const label SUC_lexicon_tags_184[] = { 4, 92, 88, 82, 78 };
static const label SUC_lexicon_tags_185[] = { 2, 92, 108 };
static const label SUC_lexicon_tags_186[] = { 4, 92, 154, 159, 136 };
static const label SUC_lexicon_tags_187[] = { 3, 92, 55, 40 };
static const label SUC_lexicon_tags_188[] = { 2, 73, 114 };
static const label SUC_lexicon_tags_189[] = { 2, 155, 158 };
static const label SUC_lexicon_tags_190[] = { 4, 92, 91, 90, 93 };
static const label SUC_lexicon_tags_191[] = { 2, 91, 77 };
static const label SUC_lexicon_tags_192[] = { 2, 91, 79 };
static const label SUC_lexicon_tags_193[] = { 3, 103, 46, 52 };
static const label SUC_lexicon_tags_194[] = { 1, 32 };
static const label SUC_lexicon_tags_195[] = { 3, 149, 49, 114 };
static const label SUC_lexicon_tags_196[] = { 3, 92, 82, 110 };
static const label SUC_lexicon_tags_197[] = { 1, 3 };
static const label SUC_lexicon_tags_198[] = { 7, 103, 38, 160, 52, 97, 46, 107 };
static const label SUC_lexicon_tags_199[] = { 2, 37, 82 };
static const label SUC_lexicon_tags_200[] = { 4, 163, 99, 109, 151 };
static const label SUC_lexicon_tags_201[] = { 3, 164, 98, 40 };
static const label SUC_lexicon_tags_202[] = { 2, 163, 93 };
static const label SUC_lexicon_tags_203[] = { 5, 0, 93, 79, 83, 131 };
static const label SUC_lexicon_tags_204[] = { 3, 83, 79, 151 };
static const label SUC_lexicon_tags_205[] = { 3, 56, 149, 114 };
static const label SUC_lexicon_tags_206[] = { 3, 55, 82, 78 };
static const label SUC_lexicon_tags_207[] = { 2, 92, 55 };
static const label SUC_lexicon_tags_208[] = { 3, 48, 44, 52 };
static const label SUC_lexicon_tags_209[] = { 2, 38, 46 };
static const label SUC_lexicon_tags_210[] = { 3, 90, 80, 113 };
static const label SUC_lexicon_tags_211[] = { 1, 58 };
static const label SUC_lexicon_tags_212[] = { 3, 83, 79, 114 };
static const label SUC_lexicon_tags_213[] = { 1, 57 };
static const label SUC_lexicon_tags_214[] = { 2, 76, 108 };
static const label SUC_lexicon_tags_215[] = { 4, 103, 38, 46, 52 };
static const label SUC_lexicon_tags_216[] = { 2, 82, 159 };
static const label SUC_lexicon_tags_217[] = { 1, 71 };
static const label SUC_lexicon_tags_218[] = { 2, 88, 113 };
static const label SUC_lexicon_tags_219[] = { 4, 92, 164, 98, 93 };
static const label SUC_lexicon_tags_220[] = { 2, 98, 40 };
static const label SUC_lexicon_tags_221[] = { 3, 56, 109, 77 };
static const label SUC_lexicon_tags_222[] = { 3, 83, 114, 93 };
static const label SUC_lexicon_tags_223[] = { 3, 73, 92, 93 };
static const label SUC_lexicon_tags_224[] = { 2, 151, 158 };
static const label SUC_lexicon_tags_225[] = { 5, 83, 79, 92, 82, 93 };
static const label SUC_lexicon_tags_226[] = { 2, 109, 93 };
static const label SUC_lexicon_tags_227[] = { 2, 114, 71 };
static const label SUC_lexicon_tags_228[] = { 1, 117 };
static const label SUC_lexicon_tags_229[] = { 2, 83, 81 };
static const label SUC_lexicon_tags_230[] = { 2, 161, 96 };
static const label SUC_lexicon_tags_231[] = { 2, 0, 112 };
static const label SUC_lexicon_tags_232[] = { 3, 83, 56, 151 };
static const label SUC_lexicon_tags_233[] = { 3, 93, 56, 41 };
static const label SUC_lexicon_tags_234[] = { 2, 79, 89 };
static const label SUC_lexicon_tags_235[] = { 2, 159, 161 };
static const label SUC_lexicon_tags_236[] = { 2, 114, 81 };
static const label SUC_lexicon_tags_237[] = { 2, 0, 131 };
static const label SUC_lexicon_tags_238[] = { 2, 82, 80 };
static const label SUC_lexicon_tags_239[] = { 7, 49, 163, 114, 99, 0, 93, 109 };
static const label SUC_lexicon_tags_240[] = { 2, 49, 114 };
static const label SUC_lexicon_tags_241[] = { 5, 83, 79, 114, 82, 78 };
static const label SUC_lexicon_tags_242[] = { 2, 73, 158 };
static const label SUC_lexicon_tags_243[] = { 2, 92, 159 };
static const label SUC_lexicon_tags_244[] = { 2, 160, 93 };
static const label SUC_lexicon_tags_245[] = { 3, 114, 151, 93 };
static const label SUC_lexicon_tags_246[] = { 1, 69 };
static const label SUC_lexicon_tags_247[] = { 3, 149, 114, 160 };
static const label SUC_lexicon_tags_248[] = { 2, 114, 38 };
static const label SUC_lexicon_tags_249[] = { 3, 83, 89, 93 };
static const label SUC_lexicon_tags_250[] = { 2, 149, 93 };
static const label SUC_lexicon_tags_251[] = { 3, 91, 163, 81 };
static const label SUC_lexicon_tags_252[] = { 3, 154, 159, 151 };
static const label SUC_lexicon_tags_253[] = { 2, 153, 93 };
static const label SUC_lexicon_tags_254[] = { 3, 51, 47, 43 };
static const label SUC_lexicon_tags_255[] = { 4, 83, 79, 89, 93 };
static const label SUC_lexicon_tags_256[] = { 4, 83, 163, 99, 93 };
static const label SUC_lexicon_tags_257[] = { 1, 115 };
static const label SUC_lexicon_tags_258[] = { 5, 0, 149, 92, 114, 93 };
static const label SUC_lexicon_tags_259[] = { 5, 40, 164, 55, 98, 108 };
static const label SUC_lexicon_tags_260[] = { 2, 83, 71 };
static const label SUC_lexicon_tags_261[] = { 2, 80, 113 };
static const label SUC_lexicon_tags_262[] = { 1, 107 };
static const label SUC_lexicon_tags_263[] = { 3, 0, 56, 55 };
static const label SUC_lexicon_tags_264[] = { 2, 99, 81 };
static const label SUC_lexicon_tags_265[] = { 2, 83, 151 };
static const label SUC_lexicon_tags_266[] = { 4, 163, 99, 3, 109 };
static const label SUC_lexicon_tags_267[] = { 3, 114, 46, 52 };
static const label SUC_lexicon_tags_268[] = { 4, 92, 55, 82, 78 };
static const label SUC_lexicon_tags_269[] = { 2, 37, 92 };
static const label SUC_lexicon_tags_270[] = { 4, 163, 114, 99, 93 };
static const label SUC_lexicon_tags_271[] = { 2, 98, 80 };
static const label SUC_lexicon_tags_272[] = { 3, 0, 91, 93 };
static const label SUC_lexicon_tags_273[] = { 3, 51, 154, 45 };
static const label SUC_lexicon_tags_274[] = { 3, 83, 79, 81 };
static const label SUC_lexicon_tags_275[] = { 2, 97, 38 };
static const label SUC_lexicon_tags_276[] = { 4, 92, 164, 98, 108 };
static const label SUC_lexicon_tags_277[] = { 3, 82, 80, 78 };
static const label SUC_lexicon_tags_278[] = { 3, 56, 114, 41 };
static const label SUC_lexicon_tags_279[] = { 2, 56, 114 };
static const label SUC_lexicon_tags_280[] = { 3, 114, 89, 93 };
static const label SUC_lexicon_tags_281[] = { 3, 92, 0, 67 };
static const label SUC_lexicon_tags_282[] = { 2, 160, 158 };
static const label SUC_lexicon_tags_283[] = { 3, 73, 138, 113 };
static const label SUC_lexicon_tags_284[] = { 2, 85, 93 };
static const label SUC_lexicon_tags_285[] = { 2, 85, 4 };
static const label SUC_lexicon_tags_286[] = { 4, 0, 163, 99, 109 };
static const label SUC_lexicon_tags_287[] = { 2, 73, 85 };
static const label SUC_lexicon_tags_288[] = { 4, 55, 82, 159, 78 };
static const label SUC_lexicon_tags_289[] = { 2, 92, 114 };
static const label SUC_lexicon_tags_290[] = { 2, 73, 149 };
static const label SUC_lexicon_tags_291[] = { 5, 0, 112, 41, 114, 131 };
static const label SUC_lexicon_tags_292[] = { 4, 151, 154, 71, 159 };
static const label SUC_lexicon_tags_293[] = { 3, 163, 109, 99 };
static const label SUC_lexicon_tags_294[] = { 2, 113, 78 };
static const label SUC_lexicon_tags_295[] = { 1, 146 };
static const label SUC_lexicon_tags_296[] = { 3, 159, 96, 161 };
static const label SUC_lexicon_tags_297[] = { 3, 83, 79, 149 };
static const label SUC_lexicon_tags_298[] = { 2, 109, 77 };
static const label SUC_lexicon_tags_299[] = { 3, 83, 79, 93 };
static const label SUC_lexicon_tags_300[] = { 7, 114, 153, 151, 160, 52, 93, 46 };
static const label SUC_lexicon_tags_301[] = { 3, 46, 38, 52 };
static const label SUC_lexicon_tags_302[] = { 3, 80, 98, 40 };
static const label SUC_lexicon_tags_303[] = { 5, 51, 151, 154, 45, 159 };
static const label SUC_lexicon_tags_304[] = { 3, 0, 89, 93 };
static const label SUC_lexicon_tags_305[] = { 2, 73, 93 };
static const label SUC_lexicon_tags_306[] = { 3, 154, 151, 159 };
static const label SUC_lexicon_tags_307[] = { 2, 0, 93 };
static const label SUC_lexicon_tags_308[] = { 2, 0, 158 };
static const label SUC_lexicon_tags_309[] = { 6, 79, 83, 112, 0, 131, 149 };
static const label SUC_lexicon_tags_310[] = { 2, 90, 108 };
static const label SUC_lexicon_tags_311[] = { 2, 83, 158 };
static const label SUC_lexicon_tags_312[] = { 2, 49, 38 };
static const label SUC_lexicon_tags_313[] = { 7, 79, 89, 83, 78, 82, 80, 81 };
static const label SUC_lexicon_tags_314[] = { 5, 149, 91, 114, 160, 93 };
static const label SUC_lexicon_tags_315[] = { 2, 73, 75 };
static const label SUC_lexicon_tags_316[] = { 3, 0, 82, 78 };
static const label SUC_lexicon_tags_317[] = { 3, 76, 90, 80 };
static const label SUC_lexicon_tags_318[] = { 4, 91, 87, 77, 81 };
static const label SUC_lexicon_tags_319[] = { 6, 159, 92, 45, 51, 154, 113 };
static const label SUC_lexicon_tags_320[] = { 3, 41, 99, 81 };
static const label SUC_lexicon_tags_321[] = { 4, 0, 151, 158, 131 };
static const label SUC_lexicon_tags_322[] = { 2, 56, 32 };
static const label SUC_lexicon_tags_323[] = { 3, 114, 89, 158 };
static const label SUC_lexicon_tags_324[] = { 5, 108, 55, 164, 98, 40 };
static const label SUC_lexicon_tags_325[] = { 4, 125, 73, 127, 114 };
static const label SUC_lexicon_tags_326[] = { 9, 49, 130, 83, 19, 0, 153, 151, 160, 21 };
static const label SUC_lexicon_tags_327[] = { 4, 83, 114, 82, 93 };
static const label SUC_lexicon_tags_328[] = { 4, 92, 51, 154, 45 };
static const label SUC_lexicon_tags_329[] = { 3, 83, 56, 79 };
static const label SUC_lexicon_tags_330[] = { 2, 114, 109 };
static const label SUC_lexicon_tags_331[] = { 4, 92, 93, 82, 78 };
static const label SUC_lexicon_tags_332[] = { 2, 68, 1 };
static const label SUC_lexicon_tags_333[] = { 4, 55, 82, 113, 78 };
static const label SUC_lexicon_tags_334[] = { 2, 55, 159 };
static const label SUC_lexicon_tags_335[] = { 2, 82, 161 };
static const label SUC_lexicon_tags_336[] = { 2, 76, 90 };
static const label SUC_lexicon_tags_337[] = { 1, 122 };
static const label SUC_lexicon_tags_338[] = { 5, 153, 151, 46, 71, 52 };
static const label SUC_lexicon_tags_339[] = { 6, 49, 112, 114, 0, 131, 149 };
static const label SUC_lexicon_tags_340[] = { 4, 0, 114, 46, 52 };
static const label SUC_lexicon_tags_341[] = { 4, 83, 92, 82, 93 };
static const label SUC_lexicon_tags_342[] = { 3, 88, 82, 78 };
static const label SUC_lexicon_tags_343[] = { 2, 92, 154 };
static const label SUC_lexicon_tags_344[] = { 4, 83, 82, 71, 113 };
static const label SUC_lexicon_tags_345[] = { 4, 83, 111, 49, 89 };
static const label SUC_lexicon_tags_346[] = { 1, 119 };
static const label SUC_lexicon_tags_347[] = { 7, 79, 83, 78, 92, 82, 80, 81 };
static const label SUC_lexicon_tags_348[] = { 3, 79, 89, 158 };
static const label SUC_lexicon_tags_349[] = { 2, 4, 85 };
static const label SUC_lexicon_tags_350[] = { 3, 76, 55, 108 };
static const label SUC_lexicon_tags_351[] = { 2, 90, 113 };
static const label SUC_lexicon_tags_352[] = { 3, 83, 111, 89 };
static const label SUC_lexicon_tags_353[] = { 1, 45 };
static const label SUC_lexicon_tags_354[] = { 2, 64, 66 };
static const label SUC_lexicon_tags_355[] = { 3, 73, 85, 75 };
static const label SUC_lexicon_tags_356[] = { 4, 83, 111, 49, 3 };
static const label SUC_lexicon_tags_357[] = { 6, 164, 98, 92, 82, 90, 80 };
static const label SUC_lexicon_tags_358[] = { 5, 32, 153, 151, 46, 52 };
static const label SUC_lexicon_tags_359[] = { 3, 153, 103, 107 };
static const label SUC_lexicon_tags_360[] = { 6, 79, 83, 78, 82, 73, 149 };
static const label SUC_lexicon_tags_361[] = { 2, 149, 151 };
static const label SUC_lexicon_tags_362[] = { 2, 53, 162 };
static const label SUC_lexicon_tags_363[] = { 3, 83, 79, 91 };
static const label SUC_lexicon_tags_364[] = { 2, 0, 41 };
static const label SUC_lexicon_tags_365[] = { 6, 114, 153, 151, 52, 93, 46 };
static const label SUC_lexicon_tags_366[] = { 4, 163, 99, 109, 93 };
static const label SUC_lexicon_tags_367[] = { 1, 126 };
static const label SUC_lexicon_tags_368[] = { 2, 135, 93 };
static const label SUC_lexicon_tags_369[] = { 4, 83, 149, 114, 126 };
static const label SUC_lexicon_tags_370[] = { 8, 0, 153, 151, 67, 32, 148, 149, 93 };
static const label SUC_lexicon_tags_371[] = { 4, 0, 82, 114, 78 };
static const label SUC_lexicon_tags_372[] = { 2, 92, 164 };
static const label SUC_lexicon_tags_373[] = { 4, 0, 163, 109, 99 };
static const label SUC_lexicon_tags_374[] = { 3, 90, 86, 78 };
static const label SUC_lexicon_tags_375[] = { 3, 52, 46, 93 };
static const label SUC_lexicon_tags_376[] = { 2, 88, 89 };
static const label SUC_lexicon_tags_377[] = { 3, 159, 161, 96 };
static const label SUC_lexicon_tags_378[] = { 5, 92, 51, 154, 159, 45 };
static const label SUC_lexicon_tags_379[] = { 3, 91, 77, 81 };
static const label SUC_lexicon_tags_380[] = { 5, 104, 106, 161, 96, 100 };
static const label SUC_lexicon_tags_381[] = { 2, 92, 40 };
static const label SUC_lexicon_tags_382[] = { 2, 149, 160 };
static const label SUC_lexicon_tags_383[] = { 3, 73, 91, 77 };
static const label SUC_lexicon_tags_384[] = { 3, 90, 55, 108 };
static const label SUC_lexicon_tags_385[] = { 5, 41, 56, 163, 99, 93 };
static const label SUC_lexicon_tags_386[] = { 2, 56, 77 };
static const label SUC_lexicon_tags_387[] = { 3, 104, 106, 100 };
static const label SUC_lexicon_tags_388[] = { 3, 76, 92, 90 };
static const label SUC_lexicon_tags_389[] = { 2, 83, 32 };
static const label SUC_lexicon_tags_390[] = { 3, 153, 114, 151 };
static const label SUC_lexicon_tags_391[] = { 4, 83, 56, 79, 93 };
static const label SUC_lexicon_tags_392[] = { 3, 92, 90, 93 };
static const label SUC_lexicon_tags_393[] = { 3, 91, 56, 109 };
static const label SUC_lexicon_tags_394[] = { 5, 83, 149, 0, 158, 70 };
static const label SUC_lexicon_tags_395[] = { 2, 82, 86 };
static const label SUC_lexicon_tags_396[] = { 4, 0, 163, 99, 41 };
static const label SUC_lexicon_tags_397[] = { 2, 103, 160 };
static const label SUC_lexicon_tags_398[] = { 2, 162, 4 };
static const label SUC_lexicon_tags_399[] = { 5, 92, 90, 82, 88, 78 };
static const label SUC_lexicon_tags_400[] = { 3, 83, 151, 158 };
static const label SUC_lexicon_tags_401[] = { 1, 21 };
static const label SUC_lexicon_tags_402[] = { 2, 83, 49 };
static const label SUC_lexicon_tags_403[] = { 5, 153, 151, 46, 160, 52 };
static const label SUC_lexicon_tags_404[] = { 4, 56, 149, 114, 41 };
static const label SUC_lexicon_tags_405[] = { 3, 92, 55, 113 };
static const label SUC_lexicon_tags_406[] = { 3, 151, 159, 82 };
static const label SUC_lexicon_tags_407[] = { 5, 83, 79, 163, 114, 99 };
static const label SUC_lexicon_tags_408[] = { 4, 93, 114, 46, 52 };
static const label SUC_lexicon_tags_409[] = { 6, 89, 78, 92, 82, 88, 90 };
static const label SUC_lexicon_tags_410[] = { 3, 153, 151, 160 };
static const label SUC_lexicon_tags_411[] = { 2, 41, 93 };
static const label SUC_lexicon_tags_412[] = { 4, 0, 149, 114, 93 };
static const label SUC_lexicon_tags_413[] = { 9, 49, 159, 55, 161, 78, 92, 82, 151, 73 };
static const label SUC_lexicon_tags_414[] = { 2, 35, 2 };
static const label SUC_lexicon_tags_415[] = { 2, 90, 55 };
static const label SUC_lexicon_tags_416[] = { 2, 37, 159 };
static const label SUC_lexicon_tags_417[] = { 1, 17 };
static const label SUC_lexicon_tags_418[] = { 2, 151, 160 };
static const label SUC_lexicon_tags_419[] = { 2, 76, 78 };
static const label SUC_lexicon_tags_420[] = { 2, 0, 136 };
static const label SUC_lexicon_tags_421[] = { 2, 114, 77 };
static const label SUC_lexicon_tags_422[] = { 2, 0, 3 };
static const label SUC_lexicon_tags_423[] = { 4, 0, 111, 49, 83 };
static const label SUC_lexicon_tags_424[] = { 5, 105, 97, 101, 107, 160 };
static const label SUC_lexicon_tags_425[] = { 2, 63, 65 };
static const label SUC_lexicon_tags_426[] = { 3, 73, 85, 33 };
static const label SUC_lexicon_tags_427[] = { 2, 37, 113 };
static const label SUC_lexicon_tags_428[] = { 5, 73, 79, 83, 89, 93 };
static const label SUC_lexicon_tags_429[] = { 4, 83, 79, 160, 93 };
static const label SUC_lexicon_tags_430[] = { 3, 83, 153, 151 };
static const label SUC_lexicon_tags_431[] = { 2, 149, 56 };
static const label SUC_lexicon_tags_432[] = { 3, 81, 151, 93 };
static const label SUC_lexicon_tags_433[] = { 2, 73, 92 };
static const label SUC_lexicon_tags_434[] = { 1, 5 };
static const label SUC_lexicon_tags_435[] = { 4, 92, 114, 113, 93 };
static const label SUC_lexicon_tags_436[] = { 4, 153, 114, 151, 93 };
static const label SUC_lexicon_tags_437[] = { 5, 103, 38, 107, 46, 52 };
static const label SUC_lexicon_tags_438[] = { 2, 0, 55 };
static const label SUC_lexicon_tags_439[] = { 2, 109, 151 };
static const label SUC_lexicon_tags_440[] = { 1, 36 };
static const label SUC_lexicon_tags_441[] = { 5, 83, 79, 149, 114, 93 };
static const label SUC_lexicon_tags_442[] = { 4, 0, 83, 73, 151 };
static const label SUC_lexicon_tags_443[] = { 4, 73, 79, 83, 114 };
static const label SUC_lexicon_tags_444[] = { 2, 149, 119 };
static const label SUC_lexicon_tags_445[] = { 5, 153, 114, 46, 151, 52 };
static const label SUC_lexicon_tags_446[] = { 3, 114, 150, 113 };
static const label SUC_lexicon_tags_447[] = { 2, 73, 132 };
static const label SUC_lexicon_tags_448[] = { 3, 135, 153, 151 };
static const label SUC_lexicon_tags_449[] = { 4, 92, 154, 159, 151 };
static const label SUC_lexicon_tags_450[] = { 3, 114, 113, 93 };
static const label SUC_lexicon_tags_451[] = { 4, 37, 51, 45, 161 };
static const label SUC_lexicon_tags_452[] = { 2, 0, 159 };
static const label SUC_lexicon_tags_453[] = { 4, 164, 159, 98, 108 };
static const label SUC_lexicon_tags_454[] = { 3, 163, 99, 93 };
static const label SUC_lexicon_tags_455[] = { 3, 83, 79, 49 };
static const label SUC_lexicon_tags_456[] = { 4, 83, 79, 114, 93 };
static const label SUC_lexicon_tags_457[] = { 5, 0, 151, 3, 158, 21 };
static const label SUC_lexicon_tags_458[] = { 2, 81, 41 };
static const label SUC_lexicon_tags_459[] = { 4, 0, 112, 149, 114 };
static const label SUC_lexicon_tags_460[] = { 2, 163, 49 };
static const label SUC_lexicon_tags_461[] = { 3, 0, 83, 93 };
static const label SUC_lexicon_tags_462[] = { 3, 49, 46, 52 };
static const label SUC_lexicon_tags_463[] = { 2, 0, 83 };
static const label SUC_lexicon_tags_464[] = { 6, 31, 67, 32, 93, 73, 131 };
static const label SUC_lexicon_tags_465[] = { 4, 73, 154, 159, 151 };
static const label SUC_lexicon_tags_466[] = { 3, 0, 153, 151 };
static const label SUC_lexicon_tags_467[] = { 3, 49, 114, 93 };
static const label SUC_lexicon_tags_468[] = { 3, 92, 164, 98 };
static const label SUC_lexicon_tags_469[] = { 5, 92, 164, 82, 98, 78 };
static const label SUC_lexicon_tags_470[] = { 3, 99, 3, 41 };
static const label SUC_lexicon_tags_471[] = { 6, 89, 103, 87, 52, 46, 107 };
static const label SUC_lexicon_tags_472[] = { 5, 103, 46, 107, 38, 52 };
static const label SUC_lexicon_tags_473[] = { 2, 79, 91 };
static const label SUC_lexicon_tags_474[] = { 4, 52, 56, 46, 41 };
static const label SUC_lexicon_tags_475[] = { 3, 0, 49, 93 };
static const label SUC_lexicon_tags_476[] = { 1, 94 };
static const label SUC_lexicon_tags_477[] = { 1, 148 };
static const label SUC_lexicon_tags_478[] = { 2, 76, 55 };
static const label SUC_lexicon_tags_479[] = { 2, 38, 160 };
static const label SUC_lexicon_tags_480[] = { 1, 155 };
static const label SUC_lexicon_tags_481[] = { 3, 153, 46, 52 };
static const label SUC_lexicon_tags_482[] = { 5, 73, 114, 82, 113, 78 };
static const label SUC_lexicon_tags_483[] = { 4, 83, 153, 151, 93 };
static const label SUC_lexicon_tags_484[] = { 3, 92, 90, 113 };
static const label SUC_lexicon_tags_485[] = { 2, 151, 93 };
static const label SUC_lexicon_tags_486[] = { 2, 85, 115 };
static const label SUC_lexicon_tags_487[] = { 3, 92, 151, 93 };
static const label SUC_lexicon_tags_488[] = { 4, 162, 4, 75, 85 };
static const label SUC_lexicon_tags_489[] = { 4, 108, 164, 98, 40 };
static const label SUC_lexicon_tags_490[] = { 2, 163, 151 };
static const label SUC_lexicon_tags_491[] = { 3, 92, 55, 93 };
static const label SUC_lexicon_tags_492[] = { 2, 85, 72 };
static const label SUC_lexicon_tags_493[] = { 4, 73, 82, 113, 78 };
static const label SUC_lexicon_tags_494[] = { 3, 82, 80, 159 };
static const label SUC_lexicon_tags_495[] = { 3, 93, 160, 41 };
static const label SUC_lexicon_tags_496[] = { 2, 159, 108 };
static const label SUC_lexicon_tags_497[] = { 4, 0, 56, 149, 41 };
static const label SUC_lexicon_tags_498[] = { 5, 56, 163, 99, 109, 41 };
static const label SUC_lexicon_tags_499[] = { 2, 22, 26 };
static const label SUC_lexicon_tags_500[] = { 3, 151, 159, 154 };
static const label SUC_lexicon_tags_501[] = { 4, 5, 0, 3, 66 };
static const label SUC_lexicon_tags_502[] = { 4, 92, 164, 98, 40 };
static const label SUC_lexicon_tags_503[] = { 3, 83, 91, 93 };
static const label SUC_lexicon_tags_504[] = { 5, 0, 56, 93, 3, 41 };
static const label SUC_lexicon_tags_505[] = { 3, 92, 114, 93 };
static const label SUC_lexicon_tags_506[] = { 2, 80, 40 };
static const label SUC_lexicon_tags_507[] = { 3, 105, 101, 107 };
static const label SUC_lexicon_tags_508[] = { 3, 89, 158, 93 };
static const label SUC_lexicon_tags_509[] = { 3, 37, 144, 114 };
static const label SUC_lexicon_tags_510[] = { 5, 103, 38, 160, 46, 52 };
static const label SUC_lexicon_tags_511[] = { 4, 83, 91, 77, 93 };
static const label SUC_lexicon_tags_512[] = { 5, 153, 103, 46, 107, 52 };
static const label SUC_lexicon_tags_513[] = { 4, 92, 114, 82, 159 };
static const label SUC_lexicon_tags_514[] = { 3, 51, 113, 45 };
static const label SUC_lexicon_tags_515[] = { 3, 90, 80, 78 };
static const label SUC_lexicon_tags_516[] = { 2, 35, 93 };
static const label SUC_lexicon_tags_517[] = { 4, 92, 82, 113, 93 };
static const label SUC_lexicon_tags_518[] = { 2, 140, 146 };
static const label SUC_lexicon_tags_519[] = { 2, 73, 33 };
static const label SUC_lexicon_tags_520[] = { 2, 37, 161 };
static const label SUC_lexicon_tags_521[] = { 3, 164, 98, 113 };
static const label SUC_lexicon_tags_522[] = { 2, 164, 40 };
static const label SUC_lexicon_tags_523[] = { 2, 153, 155 };
static const label SUC_lexicon_tags_524[] = { 5, 163, 99, 109, 151, 93 };
static const label SUC_lexicon_tags_525[] = { 1, 63 };
static const label SUC_lexicon_tags_526[] = { 3, 33, 114, 93 };
static const label SUC_lexicon_tags_527[] = { 3, 92, 113, 93 };
static const label SUC_lexicon_tags_528[] = { 3, 83, 149, 114 };
static const label SUC_lexicon_tags_529[] = { 5, 56, 163, 109, 99, 41 };
static const label SUC_lexicon_tags_530[] = { 3, 114, 82, 113 };
static const label SUC_lexicon_tags_531[] = { 2, 56, 158 };
static const label SUC_lexicon_tags_532[] = { 5, 73, 92, 88, 82, 78 };
static const label SUC_lexicon_tags_533[] = { 4, 83, 114, 38, 93 };
static const label SUC_lexicon_tags_534[] = { 5, 93, 153, 151, 46, 52 };
static const label SUC_lexicon_tags_535[] = { 3, 92, 0, 93 };
static const label SUC_lexicon_tags_536[] = { 4, 153, 103, 151, 107 };
static const label SUC_lexicon_tags_537[] = { 2, 56, 149 };
static const label SUC_lexicon_tags_538[] = { 2, 92, 0 };
static const label SUC_lexicon_tags_539[] = { 3, 92, 114, 82 };
static const label SUC_lexicon_tags_540[] = { 5, 92, 90, 49, 113, 32 };
static const label SUC_lexicon_tags_541[] = { 3, 35, 89, 93 };
static const label SUC_lexicon_tags_542[] = { 4, 76, 90, 80, 86 };
static const label SUC_lexicon_tags_543[] = { 2, 85, 141 };
static const label SUC_lexicon_tags_544[] = { 2, 149, 67 };
static const label SUC_lexicon_tags_545[] = { 5, 52, 153, 151, 46, 93 };
static const label SUC_lexicon_tags_546[] = { 3, 73, 76, 90 };
static const label SUC_lexicon_tags_547[] = { 3, 98, 80, 40 };
static const label SUC_lexicon_tags_548[] = { 2, 153, 160 };
static const label SUC_lexicon_tags_549[] = { 1, 70 };
static const label SUC_lexicon_tags_550[] = { 6, 89, 92, 91, 93, 88, 90 };
static const label SUC_lexicon_tags_551[] = { 3, 56, 109, 41 };
static const label SUC_lexicon_tags_552[] = { 3, 35, 49, 2 };
static const label SUC_lexicon_tags_553[] = { 3, 162, 4, 85 };
static const label SUC_lexicon_tags_554[] = { 3, 154, 159, 113 };
static const label SUC_lexicon_tags_555[] = { 2, 149, 49 };
static const label SUC_lexicon_tags_556[] = { 3, 83, 82, 151 };
static const label SUC_lexicon_tags_557[] = { 5, 83, 149, 79, 82, 78 };
static const label SUC_lexicon_tags_558[] = { 4, 92, 90, 82, 80 };
static const label SUC_lexicon_tags_559[] = { 2, 92, 149 };
static const label SUC_lexicon_tags_560[] = { 5, 83, 56, 93, 3, 41 };
static const label SUC_lexicon_tags_561[] = { 2, 68, 114 };
static const label SUC_lexicon_tags_562[] = { 3, 151, 159, 113 };
static const label SUC_lexicon_tags_563[] = { 3, 5, 0, 66 };
static const label SUC_lexicon_tags_564[] = { 2, 73, 88 };
static const label SUC_lexicon_tags_565[] = { 3, 83, 79, 158 };
static const label SUC_lexicon_tags_566[] = { 2, 73, 71 };
static const label SUC_lexicon_tags_567[] = { 2, 132, 1 };
static const label SUC_lexicon_tags_568[] = { 4, 151, 154, 113, 159 };
static const label SUC_lexicon_tags_569[] = { 2, 92, 80 };
static const label SUC_lexicon_tags_570[] = { 2, 91, 149 };
static const label SUC_lexicon_tags_571[] = { 1, 84 };
static const label SUC_lexicon_tags_572[] = { 1, 131 };
static const label SUC_lexicon_tags_573[] = { 3, 114, 158, 93 };
static const label SUC_lexicon_tags_574[] = { 5, 153, 151, 46, 158, 52 };
static const label SUC_lexicon_tags_575[] = { 5, 0, 56, 163, 99, 41 };
static const label SUC_lexicon_tags_576[] = { 5, 153, 103, 151, 107, 93 };
static const label SUC_lexicon_tags_577[] = { 2, 53, 85 };
static const label SUC_lexicon_tags_578[] = { 4, 99, 109, 151, 93 };
static const label SUC_lexicon_tags_579[] = { 3, 163, 114, 99 };
static const label SUC_lexicon_tags_580[] = { 3, 92, 154, 82 };
static const label SUC_lexicon_tags_581[] = { 2, 164, 108 };
static const label SUC_lexicon_tags_582[] = { 2, 73, 1 };
static const label SUC_lexicon_tags_583[] = { 4, 37, 51, 161, 45 };
static const label SUC_lexicon_tags_584[] = { 2, 55, 82 };
static const label SUC_lexicon_tags_585[] = { 6, 79, 83, 78, 82, 45, 51 };
static const label SUC_lexicon_tags_586[] = { 4, 93, 153, 46, 52 };
static const label SUC_lexicon_tags_587[] = { 2, 82, 151 };
static const label SUC_lexicon_tags_588[] = { 4, 164, 82, 98, 78 };
static const label SUC_lexicon_tags_589[] = { 4, 73, 114, 151, 93 };
static const label SUC_lexicon_tags_590[] = { 2, 159, 113 };
static const label SUC_lexicon_tags_591[] = { 3, 55, 82, 40 };
static const label SUC_lexicon_tags_592[] = { 2, 0, 38 };
static const label SUC_lexicon_tags_593[] = { 3, 76, 90, 86 };
static const label SUC_lexicon_tags_594[] = { 2, 153, 158 };
static const label SUC_lexicon_tags_595[] = { 4, 46, 48, 44, 52 };
static const label SUC_lexicon_tags_596[] = { 4, 0, 83, 10, 118 };
static const label SUC_lexicon_tags_597[] = { 3, 0, 111, 83 };
static const label SUC_lexicon_tags_598[] = { 4, 73, 92, 160, 161 };
static const label SUC_lexicon_tags_599[] = { 1, 72 };
static const label SUC_lexicon_tags_600[] = { 2, 0, 49 };
static const label SUC_lexicon_tags_601[] = { 3, 91, 87, 77 };
static const label SUC_lexicon_tags_602[] = { 3, 155, 160, 158 };
static const label SUC_lexicon_tags_603[] = { 6, 56, 79, 83, 55, 78, 82 };
static const label SUC_lexicon_tags_604[] = { 3, 0, 56, 131 };
static const label SUC_lexicon_tags_605[] = { 1, 51 };
static const label SUC_lexicon_tags_606[] = { 5, 37, 51, 55, 45, 40 };
static const label SUC_lexicon_tags_607[] = { 3, 82, 161, 78 };
static const label SUC_lexicon_tags_608[] = { 3, 114, 99, 109 };
static const label SUC_lexicon_tags_609[] = { 3, 76, 92, 93 };
static const label SUC_lexicon_tags_610[] = { 5, 83, 79, 114, 89, 93 };
static const label SUC_lexicon_tags_611[] = { 3, 153, 151, 77 };
static const label SUC_lexicon_tags_612[] = { 2, 91, 56 };
static const label SUC_lexicon_tags_613[] = { 3, 149, 114, 93 };
static const label SUC_lexicon_tags_614[] = { 3, 92, 49, 93 };
static const label SUC_lexicon_tags_615[] = { 1, 10 };
static const label SUC_lexicon_tags_616[] = { 2, 148, 131 };
static const label SUC_lexicon_tags_617[] = { 2, 73, 56 };
static const label SUC_lexicon_tags_618[] = { 2, 160, 161 };
static const label SUC_lexicon_tags_619[] = { 2, 140, 114 };
static const label SUC_lexicon_tags_620[] = { 3, 154, 159, 161 };
static const label SUC_lexicon_tags_621[] = { 1, 143 };
static const label SUC_lexicon_tags_622[] = { 6, 3, 163, 99, 41, 0, 151 };
static const label SUC_lexicon_tags_623[] = { 1, 137 };
static const label SUC_lexicon_tags_624[] = { 1, 33 };
static const label SUC_lexicon_tags_625[] = { 4, 154, 159, 113, 114 };
static const label SUC_lexicon_tags_626[] = { 4, 154, 106, 159, 102 };
static const label SUC_lexicon_tags_627[] = { 3, 92, 82, 113 };
static const label SUC_lexicon_tags_628[] = { 4, 92, 82, 113, 78 };
static const label SUC_lexicon_tags_629[] = { 3, 82, 159, 151 };
static const label SUC_lexicon_tags_630[] = { 4, 154, 159, 106, 102 };
static const label SUC_lexicon_tags_631[] = { 3, 92, 55, 82 };
static const label SUC_lexicon_tags_632[] = { 5, 164, 82, 98, 113, 78 };
static const label SUC_lexicon_tags_633[] = { 2, 163, 109 };
static const label SUC_lexicon_tags_634[] = { 2, 48, 122 };
static const label SUC_lexicon_tags_635[] = { 4, 153, 151, 160, 158 };
static const label SUC_lexicon_tags_636[] = { 6, 78, 92, 82, 88, 90, 80 };
static const label SUC_lexicon_tags_637[] = { 1, 132 };
static const label SUC_lexicon_tags_638[] = { 2, 0, 56 };
static const label SUC_lexicon_tags_639[] = { 2, 140, 93 };
static const label SUC_lexicon_tags_640[] = { 2, 0, 32 };
static const label SUC_lexicon_tags_641[] = { 4, 92, 82, 154, 159 };
static const label SUC_lexicon_tags_642[] = { 3, 55, 159, 108 };
static const label SUC_lexicon_tags_643[] = { 3, 0, 49, 160 };
static const label SUC_lexicon_tags_644[] = { 1, 136 };
static const label SUC_lexicon_tags_645[] = { 3, 103, 107, 52 };
static const label SUC_lexicon_tags_646[] = { 3, 73, 83, 114 };
static const label SUC_lexicon_tags_647[] = { 1, 118 };
static const label SUC_lexicon_tags_648[] = { 5, 153, 149, 151, 158, 93 };
static const label SUC_lexicon_tags_649[] = { 2, 37, 149 };
static const label SUC_lexicon_tags_650[] = { 5, 0, 111, 49, 83, 3 };
static const label SUC_lexicon_tags_651[] = { 4, 82, 154, 113, 159 };
static const label SUC_lexicon_tags_652[] = { 2, 83, 149 };
static const label SUC_lexicon_tags_653[] = { 2, 103, 52 };
static const label SUC_lexicon_tags_654[] = { 5, 92, 164, 98, 159, 108 };
static const label SUC_lexicon_tags_655[] = { 3, 48, 46, 52 };
static const label SUC_lexicon_tags_656[] = { 2, 83, 56 };
static const label SUC_lexicon_tags_657[] = { 3, 0, 111, 49 };
static const label SUC_lexicon_tags_658[] = { 2, 56, 81 };
static const label SUC_lexicon_tags_659[] = { 6, 159, 92, 151, 45, 51, 154 };
static const label SUC_lexicon_tags_660[] = { 2, 49, 89 };
static const label SUC_lexicon_tags_661[] = { 1, 53 };
static const label SUC_lexicon_tags_662[] = { 2, 114, 52 };
static const label SUC_lexicon_tags_663[] = { 1, 150 };
static const label SUC_lexicon_tags_664[] = { 2, 90, 78 };
static const label SUC_lexicon_tags_665[] = { 5, 83, 153, 149, 151, 73 };
static const label SUC_lexicon_tags_666[] = { 2, 77, 93 };
static const label SUC_lexicon_tags_667[] = { 3, 114, 160, 93 };
static const label SUC_lexicon_tags_668[] = { 3, 153, 149, 114 };
static const label SUC_lexicon_tags_669[] = { 4, 83, 79, 151, 158 };
static const label SUC_lexicon_tags_670[] = { 4, 83, 79, 163, 99 };
static const label SUC_lexicon_tags_671[] = { 6, 79, 83, 78, 82, 73, 113 };
static const label SUC_lexicon_tags_672[] = { 2, 56, 91 };
static const label SUC_lexicon_tags_673[] = { 4, 163, 99, 3, 41 };
static const label SUC_lexicon_tags_674[] = { 4, 153, 114, 151, 158 };
static const label SUC_lexicon_tags_675[] = { 6, 55, 47, 43, 92, 37, 51 };
static const label SUC_lexicon_tags_676[] = { 3, 0, 112, 114 };
static const label SUC_lexicon_tags_677[] = { 6, 104, 102, 100, 45, 51, 106 };
static const label SUC_lexicon_tags_678[] = { 4, 87, 88, 86, 89 };
static const label SUC_lexicon_tags_679[] = { 6, 89, 83, 153, 93, 73, 149 };
static const label SUC_lexicon_tags_680[] = { 5, 83, 49, 114, 82, 113 };
static const label SUC_lexicon_tags_681[] = { 1, 23 };
static const label SUC_lexicon_tags_682[] = { 2, 32, 93 };
static const label SUC_lexicon_tags_683[] = { 2, 88, 114 };
static const label SUC_lexicon_tags_684[] = { 1, 112 };
static const label SUC_lexicon_tags_685[] = { 6, 83, 114, 92, 82, 93, 113 };
static const label SUC_lexicon_tags_686[] = { 1, 25 };
static const label SUC_lexicon_tags_687[] = { 2, 79, 93 };
static const label SUC_lexicon_tags_688[] = { 5, 92, 82, 154, 159, 151 };
static const label SUC_lexicon_tags_689[] = { 3, 92, 88, 34 };
static const label SUC_lexicon_tags_690[] = { 2, 113, 136 };
static const label SUC_lexicon_tags_691[] = { 3, 103, 97, 38 };
static const label SUC_lexicon_tags_692[] = { 2, 58, 60 };
static const label SUC_lexicon_tags_693[] = { 4, 109, 89, 77, 93 };
static const label SUC_lexicon_tags_694[] = { 3, 92, 159, 161 };
static const label SUC_lexicon_tags_695[] = { 3, 83, 111, 93 };
static const label SUC_lexicon_tags_696[] = { 3, 76, 154, 159 };
static const label SUC_lexicon_tags_697[] = { 4, 83, 79, 151, 93 };
static const label SUC_lexicon_tags_698[] = { 3, 114, 128, 18 };
static const label SUC_lexicon_tags_699[] = { 7, 79, 56, 83, 151, 160, 93, 73 };
static const label SUC_lexicon_tags_700[] = { 3, 92, 82, 159 };
static const label SUC_lexicon_tags_701[] = { 5, 83, 56, 163, 109, 41 };
static const label SUC_lexicon_tags_702[] = { 3, 0, 148, 66 };
static const label SUC_lexicon_tags_703[] = { 5, 83, 79, 55, 82, 78 };
static const label SUC_lexicon_tags_704[] = { 4, 153, 49, 151, 93 };
static const label SUC_lexicon_tags_705[] = { 3, 73, 82, 78 };
static const label SUC_lexicon_tags_706[] = { 3, 0, 56, 93 };
static const label SUC_lexicon_tags_707[] = { 4, 114, 159, 113, 154 };
static const label SUC_lexicon_tags_708[] = { 2, 87, 89 };
static const label SUC_lexicon_tags_709[] = { 2, 75, 141 };
static const label SUC_lexicon_tags_710[] = { 3, 83, 56, 93 };
static const label SUC_lexicon_tags_711[] = { 3, 114, 38, 93 };
static const label SUC_lexicon_tags_712[] = { 3, 83, 73, 114 };
static const label SUC_lexicon_tags_713[] = { 4, 92, 154, 106, 102 };
static const label SUC_lexicon_tags_714[] = { 2, 46, 38 };
static const label SUC_lexicon_tags_715[] = { 2, 38, 60 };
static const label SUC_lexicon_tags_716[] = { 3, 114, 38, 113 };
static const label SUC_lexicon_tags_717[] = { 5, 114, 106, 96, 161, 102 };
static const label SUC_lexicon_tags_718[] = { 4, 32, 15, 46, 52 };
static const label SUC_lexicon_tags_719[] = { 3, 83, 79, 160 };
static const label SUC_lexicon_tags_720[] = { 1, 48 };
static const label SUC_lexicon_tags_721[] = { 5, 92, 154, 159, 113, 151 };
static const label SUC_lexicon_tags_722[] = { 6, 164, 98, 108, 55, 92, 40 };
static const label SUC_lexicon_tags_723[] = { 3, 92, 88, 113 };
static const label SUC_lexicon_tags_724[] = { 3, 93, 114, 81 };
static const label SUC_lexicon_tags_725[] = { 2, 159, 40 };
static const label SUC_lexicon_tags_726[] = { 5, 37, 82, 159, 161, 96 };
static const label SUC_lexicon_tags_727[] = { 4, 163, 109, 99, 151 };
static const label SUC_lexicon_tags_728[] = { 3, 114, 154, 159 };
static const label SUC_lexicon_tags_729[] = { 3, 83, 111, 0 };
static const label SUC_lexicon_tags_730[] = { 4, 92, 114, 154, 159 };
static const label SUC_lexicon_tags_731[] = { 3, 92, 114, 113 };
static const label SUC_lexicon_tags_732[] = { 2, 114, 32 };
static const label SUC_lexicon_tags_733[] = { 2, 83, 0 };
static const label SUC_lexicon_tags_734[] = { 4, 0, 11, 83, 118 };
static const label SUC_lexicon_tags_735[] = { 5, 83, 79, 160, 89, 93 };
static const label SUC_lexicon_tags_736[] = { 2, 83, 140 };
static const label SUC_lexicon_tags_737[] = { 2, 163, 41 };
static const label SUC_lexicon_tags_738[] = { 2, 73, 162 };
static const label SUC_lexicon_tags_739[] = { 2, 82, 154 };
static const label SUC_lexicon_tags_740[] = { 2, 154, 113 };
static const label SUC_lexicon_tags_741[] = { 3, 91, 77, 93 };
static const label SUC_lexicon_tags_742[] = { 4, 163, 99, 109, 41 };
static const label SUC_lexicon_tags_743[] = { 3, 92, 149, 93 };
static const label SUC_lexicon_tags_744[] = { 4, 0, 111, 83, 49 };
static const label SUC_lexicon_tags_745[] = { 2, 109, 99 };
static const label SUC_lexicon_tags_746[] = { 5, 83, 111, 49, 89, 93 };
static const label SUC_lexicon_tags_747[] = { 2, 83, 38 };
static const label SUC_lexicon_tags_748[] = { 3, 90, 164, 80 };
static const label SUC_lexicon_tags_749[] = { 5, 51, 154, 159, 45, 151 };
static const label SUC_lexicon_tags_750[] = { 2, 81, 93 };
static const label SUC_lexicon_tags_751[] = { 3, 154, 159, 93 };
static const label SUC_lexicon_tags_752[] = { 3, 55, 151, 159 };
static const label SUC_lexicon_tags_753[] = { 3, 58, 60, 93 };
static const label SUC_lexicon_tags_754[] = { 4, 0, 56, 41, 93 };
static const label SUC_lexicon_tags_755[] = { 3, 76, 90, 78 };
static const label SUC_lexicon_tags_756[] = { 1, 26 };
static const label SUC_lexicon_tags_757[] = { 2, 83, 160 };
static const label SUC_lexicon_tags_758[] = { 6, 26, 153, 151, 160, 21, 32 };
static const label SUC_lexicon_tags_759[] = { 2, 42, 85 };
static const label SUC_lexicon_tags_760[] = { 4, 93, 149, 114, 41 };
static const label SUC_lexicon_tags_761[] = { 2, 73, 113 };
static const label SUC_lexicon_tags_762[] = { 3, 97, 160, 158 };
static const label SUC_lexicon_tags_763[] = { 3, 61, 64, 66 };
static const label SUC_lexicon_tags_764[] = { 2, 71, 93 };
static const label SUC_lexicon_tags_765[] = { 3, 55, 113, 40 };
static const label SUC_lexicon_tags_766[] = { 4, 0, 79, 83, 131 };
static const label SUC_lexicon_tags_767[] = { 2, 92, 49 };
static const label SUC_lexicon_tags_768[] = { 3, 82, 113, 78 };
static const label SUC_lexicon_tags_769[] = { 1, 135 };
static const label SUC_lexicon_tags_770[] = { 2, 8, 160 };
static const label SUC_lexicon_tags_771[] = { 3, 83, 38, 160 };
static const label SUC_lexicon_tags_772[] = { 4, 32, 83, 151, 93 };
static const label SUC_lexicon_tags_773[] = { 5, 51, 104, 106, 45, 100 };
static const label SUC_lexicon_tags_774[] = { 5, 163, 109, 151, 99, 93 };
static const label SUC_lexicon_tags_775[] = { 5, 92, 90, 82, 80, 113 };
static const label SUC_lexicon_tags_776[] = { 6, 164, 159, 98, 108, 78, 82 };
static const label SUC_lexicon_tags_777[] = { 4, 78, 92, 82, 93 };
static const label SUC_lexicon_tags_778[] = { 4, 92, 51, 154, 159 };
static const label SUC_lexicon_tags_779[] = { 4, 81, 103, 107, 52 };
static const label SUC_lexicon_tags_780[] = { 2, 92, 89 };
static const label SUC_lexicon_tags_781[] = { 3, 41, 56, 93 };
static const label SUC_lexicon_tags_782[] = { 2, 92, 37 };
static const label SUC_lexicon_tags_783[] = { 3, 37, 92, 93 };
static const label SUC_lexicon_tags_784[] = { 4, 0, 56, 3, 41 };
static const label SUC_lexicon_tags_785[] = { 4, 73, 149, 114, 160 };
static const label SUC_lexicon_tags_786[] = { 2, 123, 93 };
static const label SUC_lexicon_tags_787[] = { 2, 149, 88 };
static const label SUC_lexicon_tags_788[] = { 3, 0, 49, 52 };
static const label SUC_lexicon_tags_789[] = { 2, 82, 40 };
static const label SUC_lexicon_tags_790[] = { 4, 114, 82, 113, 137 };
static const label SUC_lexicon_tags_791[] = { 4, 73, 85, 75, 162 };
static const label SUC_lexicon_tags_792[] = { 3, 91, 114, 81 };
static const label SUC_lexicon_tags_793[] = { 4, 93, 163, 99, 41 };
static const label SUC_lexicon_tags_794[] = { 2, 0, 67 };
static const label SUC_lexicon_tags_795[] = { 6, 79, 83, 78, 82, 80, 81 };
static const label SUC_lexicon_tags_796[] = { 5, 129, 149, 114, 89, 93 };
static const label SUC_lexicon_tags_797[] = { 2, 56, 151 };
static const label SUC_lexicon_tags_798[] = { 4, 83, 91, 114, 93 };
static const label SUC_lexicon_tags_799[] = { 6, 105, 103, 101, 52, 46, 107 };
static const label SUC_lexicon_tags_800[] = { 2, 26, 21 };
static const label SUC_lexicon_tags_801[] = { 3, 0, 114, 93 };
static const label SUC_lexicon_tags_802[] = { 3, 37, 151, 159 };
static const label SUC_lexicon_tags_803[] = { 4, 92, 88, 113, 89 };
static const label SUC_lexicon_tags_804[] = { 3, 153, 151, 71 };
static const label SUC_lexicon_tags_805[] = { 4, 111, 49, 89, 93 };
static const label SUC_lexicon_tags_806[] = { 3, 0, 99, 41 };
static const label SUC_lexicon_tags_807[] = { 7, 9, 114, 120, 119, 12, 149, 117 };
static const label SUC_lexicon_tags_808[] = { 4, 76, 92, 90, 82 };
static const label SUC_lexicon_tags_809[] = { 5, 76, 92, 90, 55, 93 };
static const label SUC_lexicon_tags_810[] = { 4, 0, 112, 56, 131 };
static const label SUC_lexicon_tags_811[] = { 6, 49, 0, 153, 151, 52, 46 };
static const label SUC_lexicon_tags_812[] = { 2, 149, 38 };
static const label SUC_lexicon_tags_813[] = { 3, 49, 82, 78 };
static const label SUC_lexicon_tags_814[] = { 3, 73, 56, 55 };
static const label SUC_lexicon_tags_815[] = { 4, 0, 112, 3, 41 };
static const label SUC_lexicon_tags_816[] = { 2, 158, 93 };
static const label SUC_lexicon_tags_817[] = { 3, 114, 33, 1 };
static const label SUC_lexicon_tags_818[] = { 3, 91, 56, 77 };
static const label SUC_lexicon_tags_819[] = { 4, 164, 98, 159, 40 };
static const label SUC_lexicon_tags_820[] = { 3, 0, 46, 52 };
static const label SUC_lexicon_tags_821[] = { 2, 38, 158 };
static const label SUC_lexicon_tags_822[] = { 4, 163, 114, 99, 41 };
static const label SUC_lexicon_tags_823[] = { 3, 56, 163, 41 };
static const label SUC_lexicon_tags_824[] = { 4, 83, 56, 79, 160 };
static const label SUC_lexicon_tags_825[] = { 4, 92, 90, 113, 93 };
static const label SUC_lexicon_tags_826[] = { 6, 79, 83, 78, 92, 82, 93 };
static const label SUC_lexicon_tags_827[] = { 5, 52, 103, 46, 107, 81 };
static const label SUC_lexicon_tags_828[] = { 5, 153, 121, 46, 151, 93 };
static const label SUC_lexicon_tags_829[] = { 3, 76, 149, 55 };
static const label SUC_lexicon_tags_830[] = { 3, 83, 49, 89 };
static const label SUC_lexicon_tags_831[] = { 3, 73, 154, 159 };
static const label SUC_lexicon_tags_832[] = { 3, 83, 79, 82 };
static const label SUC_lexicon_tags_833[] = { 4, 92, 90, 114, 93 };
static const label SUC_lexicon_tags_834[] = { 2, 114, 1 };
static const label SUC_lexicon_tags_835[] = { 6, 114, 153, 151, 93, 52, 46 };
static const label SUC_lexicon_tags_836[] = { 3, 56, 109, 151 };
static const label SUC_lexicon_tags_837[] = { 3, 82, 154, 159 };
static const label SUC_lexicon_tags_838[] = { 5, 83, 56, 114, 3, 41 };
static const label SUC_lexicon_tags_839[] = { 5, 0, 112, 79, 83, 131 };
static const label SUC_lexicon_tags_840[] = { 5, 83, 79, 82, 151, 78 };
static const label SUC_lexicon_tags_841[] = { 4, 153, 151, 154, 159 };
static const label SUC_lexicon_tags_842[] = { 4, 83, 127, 114, 32 };
static const label SUC_lexicon_tags_843[] = { 4, 83, 149, 114, 93 };
static const label SUC_lexicon_tags_844[] = { 3, 149, 7, 116 };
static const label SUC_lexicon_tags_845[] = { 3, 51, 45, 113 };
static const label SUC_lexicon_tags_846[] = { 3, 73, 114, 93 };
static const label SUC_lexicon_tags_847[] = { 12, 79, 89, 83, 78, 92, 91, 82, 93, 88, 90, 80, 81 };
static const label SUC_lexicon_tags_848[] = { 3, 83, 114, 81 };
static const label SUC_lexicon_tags_849[] = { 2, 0, 91 };
static const label SUC_lexicon_tags_850[] = { 3, 55, 114, 40 };
static const label SUC_lexicon_tags_851[] = { 4, 103, 46, 38, 52 };
static const label SUC_lexicon_tags_852[] = { 3, 92, 161, 93 };
static const label SUC_lexicon_tags_853[] = { 4, 103, 38, 160, 97 };
static const label SUC_lexicon_tags_854[] = { 5, 83, 153, 114, 151, 158 };
static const label SUC_lexicon_tags_855[] = { 2, 149, 89 };
static const label SUC_lexicon_tags_856[] = { 1, 29 };
static const label SUC_lexicon_tags_857[] = { 4, 0, 56, 114, 93 };
static const label SUC_lexicon_tags_858[] = { 2, 92, 78 };
static const label SUC_lexicon_tags_859[] = { 2, 93, 131 };
static const label SUC_lexicon_tags_860[] = { 3, 0, 79, 83 };
static const label SUC_lexicon_tags_861[] = { 5, 125, 140, 0, 114, 83 };
static const label SUC_lexicon_tags_862[] = { 3, 93, 91, 81 };
static const label SUC_lexicon_tags_863[] = { 6, 104, 96, 102, 161, 100, 106 };
static const label SUC_lexicon_tags_864[] = { 6, 159, 55, 45, 40, 51, 154 };
static const label SUC_lexicon_tags_865[] = { 5, 105, 46, 101, 107, 52 };
static const label SUC_lexicon_tags_866[] = { 4, 73, 83, 114, 82 };
static const label SUC_lexicon_tags_867[] = { 3, 92, 71, 93 };
static const label SUC_lexicon_tags_868[] = { 2, 149, 131 };
static const label SUC_lexicon_tags_869[] = { 3, 91, 79, 114 };
static const label SUC_lexicon_tags_870[] = { 3, 140, 153, 93 };
static const label SUC_lexicon_tags_871[] = { 1, 54 };
static const label SUC_lexicon_tags_872[] = { 3, 92, 90, 82 };
static const label SUC_lexicon_tags_873[] = { 4, 92, 151, 154, 159 };
static const label SUC_lexicon_tags_874[] = { 3, 0, 148, 131 };
static const label SUC_lexicon_tags_875[] = { 5, 83, 56, 79, 151, 158 };
static const label SUC_lexicon_tags_876[] = { 6, 83, 114, 67, 32, 73, 149 };
static const label SUC_lexicon_tags_877[] = { 5, 92, 149, 90, 91, 93 };
static const label SUC_lexicon_tags_878[] = { 2, 93, 81 };
static const label SUC_lexicon_tags_879[] = { 5, 83, 93, 79, 46, 52 };
static const label SUC_lexicon_tags_880[] = { 3, 0, 149, 112 };
static const label SUC_lexicon_tags_881[] = { 5, 0, 152, 154, 159, 151 };
static const label SUC_lexicon_tags_882[] = { 3, 151, 134, 41 };
static const label SUC_lexicon_tags_883[] = { 3, 162, 85, 72 };
static const label SUC_lexicon_tags_884[] = { 3, 149, 46, 52 };
static const label SUC_lexicon_tags_885[] = { 8, 79, 26, 83, 25, 0, 93, 32, 21 };
static const label SUC_lexicon_tags_886[] = { 4, 92, 82, 159, 93 };
static const label SUC_lexicon_tags_887[] = { 4, 61, 64, 49, 66 };
static const label SUC_lexicon_tags_888[] = { 4, 0, 151, 159, 131 };
static const label SUC_lexicon_tags_889[] = { 6, 164, 98, 78, 92, 82, 88 };
static const label SUC_lexicon_tags_890[] = { 3, 92, 55, 159 };
static const label SUC_lexicon_tags_891[] = { 5, 83, 79, 93, 89, 32 };
static const label SUC_lexicon_tags_892[] = { 1, 138 };
static const label SUC_lexicon_tags_893[] = { 4, 153, 114, 151, 113 };
static const label SUC_lexicon_tags_894[] = { 3, 92, 159, 93 };
static const label SUC_lexicon_tags_895[] = { 3, 114, 3, 41 };
static const label SUC_lexicon_tags_896[] = { 3, 131, 0, 93 };
static const label SUC_lexicon_tags_897[] = { 2, 70, 69 };
static const label SUC_lexicon_tags_898[] = { 2, 119, 120 };
static const label SUC_lexicon_tags_899[] = { 3, 91, 114, 93 };
static const label SUC_lexicon_tags_900[] = { 2, 92, 161 };
static const label SUC_lexicon_tags_901[] = { 4, 37, 82, 159, 158 };
static const label SUC_lexicon_tags_902[] = { 2, 55, 161 };
static const label SUC_lexicon_tags_903[] = { 5, 92, 93, 82, 113, 78 };
static const label SUC_lexicon_tags_904[] = { 4, 0, 49, 3, 71 };
static const label SUC_lexicon_tags_905[] = { 3, 149, 56, 93 };
static const label SUC_lexicon_tags_906[] = { 2, 49, 32 };
static const label SUC_lexicon_tags_907[] = { 4, 153, 112, 49, 93 };
static const label SUC_lexicon_tags_908[] = { 6, 56, 163, 99, 41, 0, 109 };
static const label SUC_lexicon_tags_909[] = { 5, 92, 90, 114, 113, 93 };
static const label SUC_lexicon_tags_910[] = { 5, 83, 73, 114, 77, 1 };
static const label SUC_lexicon_tags_911[] = { 3, 0, 112, 131 };
static const label SUC_lexicon_tags_912[] = { 3, 92, 55, 88 };
static const label SUC_lexicon_tags_913[] = { 3, 73, 162, 85 };
static const label SUC_lexicon_tags_914[] = { 2, 73, 115 };
static const label SUC_lexicon_tags_915[] = { 6, 79, 83, 112, 0, 93, 131 };
static const label SUC_lexicon_tags_916[] = { 4, 82, 159, 151, 78 };
static const label SUC_lexicon_tags_917[] = { 3, 0, 82, 110 };
static const label SUC_lexicon_tags_918[] = { 4, 154, 159, 71, 151 };
static const label SUC_lexicon_tags_919[] = { 2, 98, 108 };
static const label SUC_lexicon_tags_920[] = { 3, 0, 3, 41 };
static const label SUC_lexicon_tags_921[] = { 4, 83, 151, 89, 93 };
static const label SUC_lexicon_tags_922[] = { 2, 78, 108 };
static const label SUC_lexicon_tags_923[] = { 3, 83, 79, 3 };
static const label SUC_lexicon_tags_924[] = { 5, 83, 79, 93, 89, 81 };
static const label SUC_lexicon_tags_925[] = { 3, 156, 114, 160 };
static const label SUC_lexicon_tags_926[] = { 4, 51, 55, 45, 40 };
static const label SUC_lexicon_tags_927[] = { 2, 83, 87 };
static const label SUC_lexicon_tags_928[] = { 8, 56, 3, 112, 41, 0, 67, 148, 149 };
static const label SUC_lexicon_tags_929[] = { 2, 49, 82 };
static const label SUC_lexicon_tags_930[] = { 3, 114, 13, 122 };
static const label SUC_lexicon_tags_931[] = { 4, 37, 159, 96, 161 };
static const label SUC_lexicon_tags_932[] = { 2, 91, 137 };
static const label SUC_lexicon_tags_933[] = { 2, 0, 148 };
static const label SUC_lexicon_tags_934[] = { 5, 78, 92, 114, 82, 93 };
static const label SUC_lexicon_tags_935[] = { 3, 0, 49, 67 };
static const label SUC_lexicon_tags_936[] = { 2, 130, 19 };
static const label SUC_lexicon_tags_937[] = { 2, 73, 91 };
static const label SUC_lexicon_tags_938[] = { 6, 124, 114, 137, 73, 149, 126 };
static const label SUC_lexicon_tags_939[] = { 3, 83, 0, 160 };
static const label SUC_lexicon_tags_940[] = { 2, 64, 52 };
static const label SUC_lexicon_tags_941[] = { 11, 79, 159, 89, 83, 78, 92, 151, 82, 93, 88, 80 };
static const label SUC_lexicon_tags_942[] = { 7, 56, 49, 112, 41, 0, 93, 50 };
static const label SUC_lexicon_tags_943[] = { 2, 79, 109 };
static const label SUC_lexicon_tags_944[] = { 1, 19 };
static const label SUC_lexicon_tags_945[] = { 2, 94, 70 };
static const label SUC_lexicon_tags_946[] = { 3, 92, 88, 114 };
static const label SUC_lexicon_tags_947[] = { 4, 92, 88, 82, 110 };
static const label SUC_lexicon_tags_948[] = { 4, 92, 56, 55, 93 };
static const label SUC_lexicon_tags_949[] = { 5, 83, 92, 114, 82, 93 };
static const label SUC_lexicon_tags_950[] = { 2, 79, 158 };
static const label SUC_lexicon_tags_951[] = { 6, 105, 103, 101, 160, 97, 107 };
static const label SUC_lexicon_tags_952[] = { 4, 153, 49, 114, 151 };
static const label SUC_lexicon_tags_953[] = { 4, 83, 79, 114, 81 };
static const label SUC_lexicon_tags_954[] = { 4, 83, 82, 80, 81 };
static const label SUC_lexicon_tags_955[] = { 4, 93, 56, 3, 41 };
static const label SUC_lexicon_tags_956[] = { 3, 0, 67, 131 };
static const label SUC_lexicon_tags_957[] = { 2, 90, 114 };
static const label SUC_lexicon_tags_958[] = { 6, 3, 112, 163, 99, 41, 109 };
static const label SUC_lexicon_tags_959[] = { 3, 153, 49, 151 };
static const label SUC_lexicon_tags_960[] = { 3, 92, 113, 136 };
static const label SUC_lexicon_tags_961[] = { 3, 153, 155, 151 };
static const label SUC_lexicon_tags_962[] = { 6, 103, 153, 151, 52, 46, 107 };
static const label SUC_lexicon_tags_963[] = { 3, 83, 151, 81 };
static const label SUC_lexicon_tags_964[] = { 4, 83, 149, 79, 114 };
static const label SUC_lexicon_tags_965[] = { 4, 92, 164, 82, 98 };
static const label SUC_lexicon_tags_966[] = { 6, 159, 83, 92, 82, 93, 154 };
static const label SUC_lexicon_tags_967[] = { 3, 0, 49, 151 };
static const label SUC_lexicon_tags_968[] = { 3, 37, 92, 161 };
static const label SUC_lexicon_tags_969[] = { 7, 56, 3, 163, 99, 41, 0, 109 };
static const label SUC_lexicon_tags_970[] = { 7, 79, 83, 114, 78, 82, 80, 81 };
static const label SUC_lexicon_tags_971[] = { 10, 83, 112, 114, 122, 0, 131, 73, 140, 149, 32 };
static const label SUC_lexicon_tags_972[] = { 5, 83, 79, 149, 82, 78 };
static const label SUC_lexicon_tags_973[] = { 3, 118, 3, 41 };
static const label SUC_lexicon_tags_974[] = { 3, 149, 114, 117 };
static const label SUC_lexicon_tags_975[] = { 5, 83, 73, 79, 82, 78 };
static const label SUC_lexicon_tags_976[] = { 4, 41, 99, 3, 81 };
static const label SUC_lexicon_tags_977[] = { 3, 135, 114, 93 };
static const label SUC_lexicon_tags_978[] = { 3, 83, 79, 89 };
static const label SUC_lexicon_tags_979[] = { 3, 106, 96, 102 };
static const label SUC_lexicon_tags_980[] = { 7, 78, 92, 82, 93, 88, 90, 80 };
static const label SUC_lexicon_tags_981[] = { 4, 93, 91, 46, 52 };
static const label SUC_lexicon_tags_982[] = { 6, 122, 153, 151, 52, 46, 146 };
static const label SUC_lexicon_tags_983[] = { 4, 92, 82, 151, 93 };
static const label SUC_lexicon_tags_984[] = { 3, 92, 114, 89 };
static const label SUC_lexicon_tags_985[] = { 6, 79, 89, 83, 91, 93, 81 };
static const label SUC_lexicon_tags_986[] = { 6, 49, 3, 112, 0, 153, 151 };
static const label SUC_lexicon_tags_987[] = { 6, 79, 89, 83, 114, 93, 149 };
static const label SUC_lexicon_tags_988[] = { 4, 163, 109, 99, 160 };
static const label SUC_lexicon_tags_989[] = { 3, 83, 153, 93 };
static const label SUC_lexicon_tags_990[] = { 3, 90, 49, 113 };
static const label SUC_lexicon_tags_991[] = { 8, 112, 0, 151, 148, 67, 131, 93, 158 };
static const label SUC_lexicon_tags_992[] = { 1, 2 };
static const label SUC_lexicon_tags_993[] = { 5, 51, 154, 151, 45, 159 };
static const label SUC_lexicon_tags_994[] = { 3, 73, 114, 113 };
static const label SUC_lexicon_tags_995[] = { 2, 114, 160 };
static const label SUC_lexicon_tags_996[] = { 5, 83, 92, 151, 82, 93 };
static const label SUC_lexicon_tags_997[] = { 4, 0, 35, 149, 2 };
static const label SUC_lexicon_tags_998[] = { 3, 52, 46, 81 };
static const label SUC_lexicon_tags_999[] = { 3, 163, 109, 93 };
static const label SUC_lexicon_tags_1000[] = { 3, 87, 109, 77 };
static const label SUC_lexicon_tags_1001[] = { 2, 73, 4 };
static const label SUC_lexicon_tags_1002[] = { 5, 83, 79, 163, 99, 41 };
static const label SUC_lexicon_tags_1003[] = { 2, 92, 151 };
static const label SUC_lexicon_tags_1004[] = { 2, 69, 70 };
static const label SUC_lexicon_tags_1005[] = { 3, 73, 114, 161 };
static const label SUC_lexicon_tags_1006[] = { 4, 0, 32, 3, 41 };
static const label SUC_lexicon_tags_1007[] = { 4, 83, 73, 0, 49 };
static const label SUC_lexicon_tags_1008[] = { 2, 35, 158 };
static const label SUC_lexicon_tags_1009[] = { 3, 79, 87, 91 };
static const label SUC_lexicon_tags_1010[] = { 2, 153, 114 };
static const label SUC_lexicon_tags_1011[] = { 2, 114, 82 };
static const label SUC_lexicon_tags_1012[] = { 4, 83, 79, 151, 32 };
static const label SUC_lexicon_tags_1013[] = { 3, 49, 3, 41 };
static const label SUC_lexicon_tags_1014[] = { 4, 149, 114, 89, 93 };
static const label SUC_lexicon_tags_1015[] = { 2, 92, 65 };
static const label SUC_lexicon_tags_1016[] = { 5, 51, 154, 106, 45, 102 };
static const label SUC_lexicon_tags_1017[] = { 2, 92, 138 };
static const label SUC_lexicon_tags_1018[] = { 5, 51, 55, 154, 45, 40 };
static const label SUC_lexicon_tags_1019[] = { 3, 114, 113, 136 };
static const label SUC_lexicon_tags_1020[] = { 3, 0, 153, 158 };
static const label SUC_lexicon_tags_1021[] = { 5, 73, 83, 79, 114, 89 };
static const label SUC_lexicon_tags_1022[] = { 2, 49, 15 };
static const label SUC_lexicon_tags_1023[] = { 2, 89, 32 };
static const label SUC_lexicon_tags_1024[] = { 3, 151, 158, 93 };
static const label SUC_lexicon_tags_1025[] = { 3, 79, 82, 78 };
static const label SUC_lexicon_tags_1026[] = { 4, 92, 83, 82, 93 };
static const label SUC_lexicon_tags_1027[] = { 7, 159, 102, 92, 45, 51, 154, 106 };
static const label SUC_lexicon_tags_1028[] = { 3, 114, 38, 145 };
static const label SUC_lexicon_tags_1029[] = { 3, 83, 77, 81 };
static const label SUC_lexicon_tags_1030[] = { 3, 49, 113, 161 };
static const label SUC_lexicon_tags_1031[] = { 3, 92, 113, 161 };
static const label SUC_lexicon_tags_1032[] = { 2, 56, 160 };
static const label SUC_lexicon_tags_1033[] = { 5, 105, 97, 160, 101, 107 };
static const label SUC_lexicon_tags_1034[] = { 4, 92, 154, 159, 161 };
static const label SUC_lexicon_tags_1035[] = { 3, 37, 92, 88 };
static const label SUC_lexicon_tags_1036[] = { 4, 35, 2, 89, 93 };
static const label SUC_lexicon_tags_1037[] = { 3, 37, 82, 161 };
static const label SUC_lexicon_tags_1038[] = { 3, 153, 114, 93 };
static const label SUC_lexicon_tags_1039[] = { 4, 92, 55, 80, 93 };
static const label SUC_lexicon_tags_1040[] = { 4, 0, 56, 99, 41 };
static const label SUC_lexicon_tags_1041[] = { 1, 130 };
static const label SUC_lexicon_tags_1042[] = { 3, 79, 89, 93 };
static const label SUC_lexicon_tags_1043[] = { 1, 67 };
static const label SUC_lexicon_tags_1044[] = { 4, 83, 73, 140, 114 };
static const label SUC_lexicon_tags_1045[] = { 5, 83, 111, 114, 89, 93 };
static const label SUC_lexicon_tags_1046[] = { 4, 83, 56, 55, 82 };
static const label SUC_lexicon_tags_1047[] = { 4, 83, 149, 79, 93 };
static const label SUC_lexicon_tags_1048[] = { 1, 134 };
static const label SUC_lexicon_tags_1049[] = { 3, 83, 114, 160 };
static const label SUC_lexicon_tags_1050[] = { 6, 79, 83, 114, 82, 73, 149 };
static const label SUC_lexicon_tags_1051[] = { 5, 0, 56, 83, 79, 41 };
static const label SUC_lexicon_tags_1052[] = { 4, 90, 164, 80, 98 };
static const label SUC_lexicon_tags_1053[] = { 3, 0, 73, 67 };
static const label SUC_lexicon_tags_1054[] = { 5, 0, 153, 49, 151, 93 };
static const label SUC_lexicon_tags_1055[] = { 3, 76, 92, 55 };
static const label SUC_lexicon_tags_1056[] = { 2, 127, 93 };
static const label SUC_lexicon_tags_1057[] = { 3, 73, 56, 114 };
static const label SUC_lexicon_tags_1058[] = { 1, 42 };
static const label SUC_lexicon_tags_1059[] = { 4, 92, 88, 90, 89 };
static const label SUC_lexicon_tags_1060[] = { 3, 92, 37, 82 };
static const label SUC_lexicon_tags_1061[] = { 4, 83, 91, 93, 81 };
static const label SUC_lexicon_tags_1062[] = { 5, 92, 164, 55, 98, 40 };
static const label SUC_lexicon_tags_1063[] = { 4, 78, 55, 82, 40 };
static const label SUC_lexicon_tags_1064[] = { 1, 13 };
static const label SUC_lexicon_tags_1065[] = { 2, 149, 158 };
static const label SUC_lexicon_tags_1066[] = { 4, 73, 149, 137, 93 };
static const label SUC_lexicon_tags_1067[] = { 3, 49, 151, 93 };
static const label SUC_lexicon_tags_1068[] = { 3, 0, 49, 3 };
static const label SUC_lexicon_tags_1069[] = { 4, 56, 163, 99, 41 };
static const label SUC_lexicon_tags_1070[] = { 4, 55, 82, 164, 40 };
static const label SUC_lexicon_tags_1071[] = { 6, 56, 163, 99, 41, 93, 109 };
static const label SUC_lexicon_tags_1072[] = { 2, 143, 138 };
static const label SUC_lexicon_tags_1073[] = { 2, 162, 72 };
static const label SUC_lexicon_tags_1074[] = { 3, 73, 1, 32 };
static const label SUC_lexicon_tags_1075[] = { 3, 56, 89, 93 };
static const label SUC_lexicon_tags_1076[] = { 3, 56, 99, 41 };
static const label SUC_lexicon_tags_1077[] = { 4, 149, 153, 151, 93 };
static const label SUC_lexicon_tags_1078[] = { 2, 149, 159 };
static const label SUC_lexicon_tags_1079[] = { 2, 42, 4 };
static const label SUC_lexicon_tags_1080[] = { 3, 90, 114, 80 };
static const label SUC_lexicon_tags_1081[] = { 4, 41, 163, 99, 93 };
static const label SUC_lexicon_tags_1082[] = { 5, 83, 0, 79, 82, 78 };
static const label SUC_lexicon_tags_1083[] = { 5, 97, 46, 103, 107, 160 };
static const label SUC_lexicon_tags_1084[] = { 2, 113, 40 };
static const label SUC_lexicon_tags_1085[] = { 2, 114, 159 };
static const label SUC_lexicon_tags_1086[] = { 6, 56, 3, 163, 99, 41, 109 };
static const label SUC_lexicon_tags_1087[] = { 4, 92, 51, 88, 45 };
static const label SUC_lexicon_tags_1088[] = { 5, 83, 56, 79, 3, 41 };
static const label SUC_lexicon_tags_1089[] = { 3, 92, 151, 159 };
static const label SUC_lexicon_tags_1090[] = { 5, 92, 0, 84, 3, 113 };
static const label SUC_lexicon_tags_1091[] = { 3, 56, 153, 151 };
static const label SUC_lexicon_tags_1092[] = { 2, 49, 3 };
static const label SUC_lexicon_tags_1093[] = { 6, 55, 47, 43, 40, 37, 51 };
static const label SUC_lexicon_tags_1094[] = { 3, 83, 99, 93 };
static const label SUC_lexicon_tags_1095[] = { 4, 149, 114, 113, 93 };
static const label SUC_lexicon_tags_1096[] = { 4, 0, 83, 149, 32 };
static const label SUC_lexicon_tags_1097[] = { 5, 92, 91, 114, 113, 93 };
static const label SUC_lexicon_tags_1098[] = { 3, 92, 96, 161 };
static const label SUC_lexicon_tags_1099[] = { 5, 0, 114, 46, 148, 52 };
static const label SUC_lexicon_tags_1100[] = { 2, 56, 131 };
static const label SUC_lexicon_tags_1101[] = { 3, 152, 151, 159 };
static const label SUC_lexicon_tags_1102[] = { 2, 156, 160 };
static const label SUC_lexicon_tags_1103[] = { 6, 159, 102, 45, 51, 154, 106 };
static const label SUC_lexicon_tags_1104[] = { 4, 35, 149, 114, 2 };
static const label SUC_lexicon_tags_1105[] = { 3, 0, 67, 148 };
static const label SUC_lexicon_tags_1106[] = { 4, 153, 114, 89, 93 };
static const label SUC_lexicon_tags_1107[] = { 2, 73, 160 };
static const label SUC_lexicon_tags_1108[] = { 3, 83, 158, 93 };
static const label SUC_lexicon_tags_1109[] = { 4, 153, 103, 107, 93 };
static const label SUC_lexicon_tags_1110[] = { 4, 79, 91, 89, 81 };
static const label SUC_lexicon_tags_1111[] = { 2, 0, 92 };
static const label SUC_lexicon_tags_1112[] = { 9, 83, 114, 92, 91, 82, 93, 90, 80, 81 };
static const label SUC_lexicon_tags_1113[] = { 3, 79, 88, 89 };
static const label SUC_lexicon_tags_1114[] = { 3, 51, 45, 102 };
static const label SUC_lexicon_tags_1115[] = { 3, 92, 83, 82 };
static const label SUC_lexicon_tags_1116[] = { 4, 92, 90, 82, 113 };
static const label SUC_lexicon_tags_1117[] = { 5, 50, 56, 149, 114, 93 };
static const label SUC_lexicon_tags_1118[] = { 3, 82, 30, 161 };
static const label SUC_lexicon_tags_1119[] = { 4, 92, 88, 89, 93 };
static const label SUC_lexicon_tags_1120[] = { 3, 25, 67, 21 };
static const label SUC_lexicon_tags_1121[] = { 4, 83, 73, 114, 158 };
static const label SUC_lexicon_tags_1122[] = { 5, 92, 154, 106, 159, 102 };
static const label SUC_lexicon_tags_1123[] = { 4, 154, 159, 113, 151 };
static const label SUC_lexicon_tags_1124[] = { 4, 149, 153, 114, 93 };
static const label SUC_lexicon_tags_1125[] = { 5, 56, 114, 160, 89, 93 };
static const label SUC_lexicon_tags_1126[] = { 3, 96, 161, 157 };
static const label SUC_lexicon_tags_1127[] = { 3, 83, 79, 32 };
static const label SUC_lexicon_tags_1128[] = { 2, 80, 159 };
static const label SUC_lexicon_tags_1129[] = { 4, 153, 151, 103, 107 };
static const label SUC_lexicon_tags_1130[] = { 3, 42, 162, 75 };
static const label SUC_lexicon_tags_1131[] = { 2, 49, 86 };
static const label SUC_lexicon_tags_1132[] = { 4, 92, 164, 82, 78 };
static const label SUC_lexicon_tags_1133[] = { 3, 55, 114, 113 };
static const label SUC_lexicon_tags_1134[] = { 2, 56, 130 };
static const label SUC_lexicon_tags_1135[] = { 5, 0, 93, 114, 148, 131 };
static const label SUC_lexicon_tags_1136[] = { 3, 85, 75, 72 };
static const label SUC_lexicon_tags_1137[] = { 2, 114, 41 };
static const label SUC_lexicon_tags_1138[] = { 3, 149, 31, 148 };
static const label SUC_lexicon_tags_1139[] = { 3, 92, 93, 78 };
static const label SUC_lexicon_tags_1140[] = { 5, 0, 56, 83, 3, 41 };
static const label SUC_lexicon_tags_1141[] = { 3, 55, 49, 40 };
static const label SUC_lexicon_tags_1142[] = { 5, 83, 91, 90, 92, 93 };
static const label SUC_lexicon_tags_1143[] = { 2, 37, 114 };
static const label SUC_lexicon_tags_1144[] = { 3, 92, 159, 113 };
static const label SUC_lexicon_tags_1145[] = { 4, 153, 49, 151, 122 };
static const label SUC_lexicon_tags_1146[] = { 3, 83, 3, 41 };
static const label SUC_lexicon_tags_1147[] = { 4, 92, 82, 159, 78 };
static const label SUC_lexicon_tags_1148[] = { 2, 137, 93 };
static const label SUC_lexicon_tags_1149[] = { 7, 79, 89, 83, 92, 82, 93, 88 };
static const label SUC_lexicon_tags_1150[] = { 2, 0, 78 };
static const label SUC_lexicon_tags_1151[] = { 3, 92, 82, 93 };
static const label SUC_lexicon_tags_1152[] = { 2, 83, 153 };
static const label SUC_lexicon_tags_1153[] = { 3, 0, 83, 3 };
static const label SUC_lexicon_tags_1154[] = { 4, 52, 114, 46, 93 };
static const label SUC_lexicon_tags_1155[] = { 2, 73, 32 };
static const label SUC_lexicon_tags_1156[] = { 7, 56, 49, 3, 41, 38, 52, 46 };
static const label SUC_lexicon_tags_1157[] = { 3, 118, 0, 41 };
static const label SUC_lexicon_tags_1158[] = { 6, 49, 89, 83, 0, 111, 93 };
static const label SUC_lexicon_tags_1159[] = { 6, 112, 153, 151, 93, 52, 46 };
static const label SUC_lexicon_tags_1160[] = { 3, 88, 82, 110 };
static const label SUC_lexicon_tags_1161[] = { 8, 56, 49, 48, 44, 41, 38, 93, 52 };
static const label SUC_lexicon_tags_1162[] = { 3, 83, 56, 114 };
static const label SUC_lexicon_tags_1163[] = { 2, 0, 90 };
static const label SUC_lexicon_tags_1164[] = { 6, 164, 98, 108, 55, 161, 40 };
static const label SUC_lexicon_tags_1165[] = { 3, 73, 33, 93 };
static const label SUC_lexicon_tags_1166[] = { 3, 83, 49, 81 };
static const label SUC_lexicon_tags_1167[] = { 4, 153, 151, 160, 93 };
static const label SUC_lexicon_tags_1168[] = { 3, 91, 49, 93 };
static const label SUC_lexicon_tags_1169[] = { 4, 92, 55, 154, 159 };
static const label SUC_lexicon_tags_1170[] = { 6, 79, 83, 114, 78, 82, 73 };
static const label SUC_lexicon_tags_1171[] = { 4, 73, 149, 114, 93 };
static const label SUC_lexicon_tags_1172[] = { 5, 83, 79, 89, 77, 93 };
static const label SUC_lexicon_tags_1173[] = { 2, 149, 85 };
static const label SUC_lexicon_tags_1174[] = { 4, 92, 51, 113, 45 };
static const label SUC_lexicon_tags_1175[] = { 7, 79, 83, 78, 82, 32, 80, 81 };
static const label SUC_lexicon_tags_1176[] = { 4, 149, 82, 114, 78 };
static const label SUC_lexicon_tags_1177[] = { 4, 79, 153, 151, 77 };
static const label SUC_lexicon_tags_1178[] = { 2, 149, 3 };
static const label SUC_lexicon_tags_1179[] = { 2, 121, 12 };
static const label SUC_lexicon_tags_1180[] = { 3, 0, 56, 41 };
static const label SUC_lexicon_tags_1181[] = { 3, 149, 87, 91 };
static const label SUC_lexicon_tags_1182[] = { 6, 159, 55, 114, 37, 149, 154 };
static const label SUC_lexicon_tags_1183[] = { 4, 92, 82, 161, 78 };
static const label SUC_lexicon_tags_1184[] = { 3, 0, 67, 93 };
static const label SUC_lexicon_tags_1185[] = { 3, 73, 72, 115 };
static const label SUC_lexicon_tags_1186[] = { 1, 147 };
static const label SUC_lexicon_tags_1187[] = { 4, 83, 73, 114, 93 };
static const label SUC_lexicon_tags_1188[] = { 4, 83, 111, 89, 93 };
static const label SUC_lexicon_tags_1189[] = { 6, 89, 114, 92, 93, 88, 113 };
static const label SUC_lexicon_tags_1190[] = { 3, 85, 4, 115 };
static const label SUC_lexicon_tags_1191[] = { 3, 73, 0, 93 };
static const label SUC_lexicon_tags_1192[] = { 3, 149, 128, 18 };
static const label SUC_lexicon_tags_1193[] = { 3, 46, 122, 52 };
static const label SUC_lexicon_tags_1194[] = { 2, 95, 33 };
static const label SUC_lexicon_tags_1195[] = { 4, 37, 92, 82, 159 };
static const label SUC_lexicon_tags_1196[] = { 4, 88, 90, 80, 78 };
static const label SUC_lexicon_tags_1197[] = { 2, 0, 70 };
static const label SUC_lexicon_tags_1198[] = { 2, 140, 89 };
static const label SUC_lexicon_tags_1199[] = { 4, 0, 92, 49, 136 };
static const label SUC_lexicon_tags_1200[] = { 2, 149, 109 };
static const label SUC_lexicon_tags_1201[] = { 1, 18 };
static const label SUC_lexicon_tags_1202[] = { 1, 123 };
static const label SUC_lexicon_tags_1203[] = { 2, 24, 29 };
static const label SUC_lexicon_tags_1204[] = { 3, 112, 3, 41 };
static const label SUC_lexicon_tags_1205[] = { 4, 91, 56, 109, 77 };
static const label SUC_lexicon_tags_1206[] = { 4, 164, 98, 161, 108 };
static const label SUC_lexicon_tags_1207[] = { 4, 92, 159, 71, 161 };
static const label SUC_lexicon_tags_1208[] = { 3, 0, 82, 159 };
static const label SUC_lexicon_tags_1209[] = { 2, 88, 161 };
static const label SUC_lexicon_tags_1210[] = { 3, 83, 38, 93 };
static const label SUC_lexicon_tags_1211[] = { 3, 82, 159, 154 };
static const label SUC_lexicon_tags_1212[] = { 2, 149, 122 };
static const label SUC_lexicon_tags_1213[] = { 4, 0, 99, 3, 41 };
static const label SUC_lexicon_tags_1214[] = { 3, 103, 97, 160 };
static const label SUC_lexicon_tags_1215[] = { 4, 83, 153, 114, 151 };
static const label SUC_lexicon_tags_1216[] = { 4, 56, 99, 109, 41 };
static const label SUC_lexicon_tags_1217[] = { 3, 164, 55, 40 };
static const label SUC_lexicon_tags_1218[] = { 4, 83, 79, 163, 93 };
static const label SUC_lexicon_tags_1219[] = { 3, 83, 81, 32 };
static const label SUC_lexicon_tags_1220[] = { 3, 151, 89, 93 };
static const label SUC_lexicon_tags_1221[] = { 3, 83, 149, 93 };
static const label SUC_lexicon_tags_1222[] = { 3, 55, 154, 159 };
static const label SUC_lexicon_tags_1223[] = { 2, 92, 73 };
static const label SUC_lexicon_tags_1224[] = { 5, 73, 153, 155, 151, 158 };
static const label SUC_lexicon_tags_1225[] = { 5, 25, 149, 0, 67, 21 };
static const label SUC_lexicon_tags_1226[] = { 7, 89, 83, 114, 120, 12, 93, 149 };
static const label SUC_lexicon_tags_1227[] = { 2, 90, 49 };
static const label SUC_lexicon_tags_1228[] = { 6, 83, 114, 92, 82, 73, 113 };
static const label SUC_lexicon_tags_1229[] = { 3, 163, 99, 3 };
static const label SUC_lexicon_tags_1230[] = { 3, 73, 114, 1 };
static const label SUC_lexicon_tags_1231[] = { 2, 49, 88 };
static const label SUC_lexicon_tags_1232[] = { 4, 92, 154, 151, 159 };
static const label SUC_lexicon_tags_1233[] = { 3, 83, 93, 32 };
static const label SUC_lexicon_tags_1234[] = { 3, 151, 160, 93 };
static const label SUC_lexicon_tags_1235[] = { 4, 92, 114, 82, 78 };
static const label SUC_lexicon_tags_1236[] = { 2, 163, 81 };
static const label SUC_lexicon_tags_1237[] = { 4, 92, 149, 114, 93 };
static const label SUC_lexicon_tags_1238[] = { 4, 92, 149, 114, 113 };
static const label SUC_lexicon_tags_1239[] = { 6, 79, 83, 114, 93, 73, 140 };
static const label SUC_lexicon_tags_1240[] = { 2, 33, 114 };
static const label SUC_lexicon_tags_1241[] = { 3, 46, 71, 52 };
static const label SUC_lexicon_tags_1242[] = { 3, 91, 99, 81 };
static const label SUC_lexicon_tags_1243[] = { 4, 83, 38, 160, 52 };
static const label SUC_lexicon_tags_1244[] = { 6, 103, 38, 160, 52, 46, 107 };
static const label SUC_lexicon_tags_1245[] = { 2, 0, 114 };
static const label SUC_lexicon_tags_1246[] = { 4, 164, 98, 161, 40 };
static const label SUC_lexicon_tags_1247[] = { 2, 92, 110 };
static const label SUC_lexicon_tags_1248[] = { 4, 92, 83, 82, 151 };
static const label SUC_lexicon_tags_1249[] = { 4, 83, 56, 79, 41 };
static const label SUC_lexicon_tags_1250[] = { 3, 56, 163, 109 };
static const label SUC_lexicon_tags_1251[] = { 3, 73, 89, 158 };
static const label SUC_lexicon_tags_1252[] = { 3, 90, 82, 78 };
static const label SUC_lexicon_tags_1253[] = { 7, 103, 153, 151, 52, 93, 46, 107 };
static const label SUC_lexicon_tags_1254[] = { 2, 151, 81 };
static const label SUC_lexicon_tags_1255[] = { 2, 121, 16 };
static const label SUC_lexicon_tags_1256[] = { 3, 73, 149, 113 };
static const label SUC_lexicon_tags_1257[] = { 4, 52, 83, 46, 93 };
static const label SUC_lexicon_tags_1258[] = { 3, 162, 85, 93 };
static const label SUC_lexicon_tags_1259[] = { 3, 83, 91, 79 };
static const label SUC_lexicon_tags_1260[] = { 4, 114, 82, 159, 78 };
static const label SUC_lexicon_tags_1261[] = { 1, 30 };
static const label SUC_lexicon_tags_1262[] = { 5, 0, 112, 149, 93, 131 };
static const label SUC_lexicon_tags_1263[] = { 9, 79, 83, 114, 78, 92, 82, 93, 73, 149 };
static const label SUC_lexicon_tags_1264[] = { 8, 79, 89, 83, 78, 92, 82, 93, 88 };
static const label SUC_lexicon_tags_1265[] = { 4, 0, 112, 97, 160 };
static const label SUC_lexicon_tags_1266[] = { 2, 94, 69 };
static const label SUC_lexicon_tags_1267[] = { 2, 71, 158 };
static const label SUC_lexicon_tags_1268[] = { 4, 73, 149, 33, 114 };
static const label SUC_lexicon_tags_1269[] = { 5, 93, 153, 49, 151, 52 };
static const label SUC_lexicon_tags_1270[] = { 3, 114, 113, 32 };
static const label SUC_lexicon_tags_1271[] = { 2, 33, 1 };
static const label SUC_lexicon_tags_1272[] = { 2, 33, 158 };
static const label SUC_lexicon_tags_1273[] = { 2, 93, 41 };
static const label SUC_lexicon_tags_1274[] = { 6, 112, 114, 0, 148, 131, 149 };
static const label SUC_lexicon_tags_1275[] = { 5, 49, 103, 46, 107, 52 };
static const label SUC_lexicon_tags_1276[] = { 6, 79, 83, 114, 78, 82, 113 };
static const label SUC_lexicon_tags_1277[] = { 7, 49, 83, 78, 92, 82, 93, 113 };
static const label SUC_lexicon_tags_1278[] = { 5, 108, 164, 55, 98, 40 };
static const label SUC_lexicon_tags_1279[] = { 4, 149, 114, 9, 117 };
static const label SUC_lexicon_tags_1280[] = { 3, 92, 106, 102 };
static const label SUC_lexicon_tags_1281[] = { 3, 83, 151, 160 };
static const label SUC_lexicon_tags_1282[] = { 3, 153, 151, 131 };
static const label SUC_lexicon_tags_1283[] = { 4, 83, 56, 79, 114 };
static const label SUC_lexicon_tags_1284[] = { 6, 79, 89, 83, 163, 99, 93 };
static const label SUC_lexicon_tags_1285[] = { 2, 35, 49 };
static const label SUC_lexicon_tags_1286[] = { 4, 105, 103, 101, 107 };
static const label SUC_lexicon_tags_1287[] = { 3, 114, 113, 161 };
static const label SUC_lexicon_tags_1288[] = { 3, 92, 88, 154 };
static const label SUC_lexicon_tags_1289[] = { 3, 83, 79, 118 };
static const label SUC_lexicon_tags_1290[] = { 4, 73, 114, 82, 78 };
static const label SUC_lexicon_tags_1291[] = { 4, 73, 149, 114, 32 };
static const label SUC_lexicon_tags_1292[] = { 8, 79, 83, 78, 92, 82, 93, 73, 113 };
static const label SUC_lexicon_tags_1293[] = { 1, 62 };
static const label SUC_lexicon_tags_1294[] = { 2, 9, 117 };
static const label SUC_lexicon_tags_1295[] = { 4, 83, 56, 3, 41 };
static const label SUC_lexicon_tags_1296[] = { 3, 56, 109, 114 };
static const label SUC_lexicon_tags_1297[] = { 3, 79, 149, 93 };
static const label SUC_lexicon_tags_1298[] = { 2, 55, 78 };
static const label SUC_lexicon_tags_1299[] = { 3, 93, 3, 41 };
static const label SUC_lexicon_tags_1300[] = { 3, 73, 162, 75 };
static const label SUC_lexicon_tags_1301[] = { 3, 153, 38, 93 };
static const label SUC_lexicon_tags_1302[] = { 2, 111, 93 };
static const label SUC_lexicon_tags_1303[] = { 4, 83, 149, 127, 114 };
static const label SUC_lexicon_tags_1304[] = { 2, 73, 83 };
static const label SUC_lexicon_tags_1305[] = { 4, 55, 82, 161, 78 };
static const label SUC_lexicon_tags_1306[] = { 4, 0, 83, 3, 41 };
static const label SUC_lexicon_tags_1307[] = { 5, 0, 111, 49, 83, 131 };
static const label SUC_lexicon_tags_1308[] = { 3, 56, 55, 151 };
static const label SUC_lexicon_tags_1309[] = { 3, 0, 112, 163 };
static const label SUC_lexicon_tags_1310[] = { 3, 73, 149, 114 };
static const label SUC_lexicon_tags_1311[] = { 4, 83, 79, 0, 41 };
static const label SUC_lexicon_tags_1312[] = { 4, 162, 85, 75, 27 };
static const label SUC_lexicon_tags_1313[] = { 2, 57, 59 };
static const label SUC_lexicon_tags_1314[] = { 2, 76, 82 };
static const label SUC_lexicon_tags_1315[] = { 5, 92, 93, 49, 82, 78 };
static const label SUC_lexicon_tags_1316[] = { 2, 149, 32 };
static const label SUC_lexicon_tags_1317[] = { 3, 55, 82, 159 };
static const label SUC_lexicon_tags_1318[] = { 4, 163, 109, 99, 41 };
static const label SUC_lexicon_tags_1319[] = { 2, 46, 122 };
static const label SUC_lexicon_tags_1320[] = { 2, 149, 161 };
static const label SUC_lexicon_tags_1321[] = { 2, 1, 32 };
static const label SUC_lexicon_tags_1322[] = { 3, 82, 80, 113 };
static const label SUC_lexicon_tags_1323[] = { 3, 41, 3, 131 };
static const label SUC_lexicon_tags_1324[] = { 4, 0, 49, 160, 131 };
static const label SUC_lexicon_tags_1325[] = { 2, 53, 75 };
static const label SUC_lexicon_tags_1326[] = { 3, 56, 49, 93 };
static const label SUC_lexicon_tags_1327[] = { 3, 92, 0, 32 };
static const label SUC_lexicon_tags_1328[] = { 5, 163, 99, 3, 160, 41 };
static const label SUC_lexicon_tags_1329[] = { 2, 83, 35 };
static const label SUC_lexicon_tags_1330[] = { 2, 79, 114 };
static const label SUC_lexicon_tags_1331[] = { 3, 92, 88, 159 };
static const label SUC_lexicon_tags_1332[] = { 3, 0, 134, 93 };
static const label SUC_lexicon_tags_1333[] = { 5, 83, 0, 112, 148, 131 };
static const label SUC_lexicon_tags_1334[] = { 2, 49, 113 };
static const label SUC_lexicon_tags_1335[] = { 4, 83, 73, 79, 114 };
static const label SUC_lexicon_tags_1336[] = { 3, 37, 114, 82 };
static const label SUC_lexicon_tags_1337[] = { 2, 83, 41 };
static const label SUC_lexicon_tags_1338[] = { 4, 83, 111, 49, 93 };
static const label SUC_lexicon_tags_1339[] = { 4, 149, 153, 114, 151 };
static const label SUC_lexicon_tags_1340[] = { 3, 0, 83, 111 };
static const label SUC_lexicon_tags_1341[] = { 7, 68, 83, 114, 73, 140, 149, 146 };
static const label SUC_lexicon_tags_1342[] = { 3, 37, 92, 113 };
static const label SUC_lexicon_tags_1343[] = { 5, 0, 56, 77, 148, 131 };
static const label SUC_lexicon_tags_1344[] = { 3, 76, 114, 113 };
static const label SUC_lexicon_tags_1345[] = { 2, 0, 149 };
static const label SUC_lexicon_tags_1346[] = { 3, 4, 75, 85 };
static const label SUC_lexicon_tags_1347[] = { 3, 0, 93, 32 };
static const label SUC_lexicon_tags_1348[] = { 5, 83, 79, 49, 114, 32 };
static const label SUC_lexicon_tags_1349[] = { 3, 114, 82, 78 };
static const label SUC_lexicon_tags_1350[] = { 4, 0, 83, 49, 93 };
static const label SUC_lexicon_tags_1351[] = { 2, 164, 80 };
static const label SUC_lexicon_tags_1352[] = { 4, 153, 114, 113, 93 };
static const label SUC_lexicon_tags_1353[] = { 3, 5, 41, 66 };
static const label SUC_lexicon_tags_1354[] = { 2, 39, 41 };
static const label SUC_lexicon_tags_1355[] = { 4, 83, 79, 149, 114 };
static const label SUC_lexicon_tags_1356[] = { 3, 91, 49, 114 };
static const label SUC_lexicon_tags_1357[] = { 6, 83, 84, 153, 151, 52, 46 };
static const label SUC_lexicon_tags_1358[] = { 3, 83, 114, 82 };
static const label SUC_lexicon_tags_1359[] = { 3, 90, 113, 78 };
static const label SUC_lexicon_tags_1360[] = { 4, 0, 112, 32, 131 };
static const label SUC_lexicon_tags_1361[] = { 3, 153, 151, 84 };
static const label SUC_lexicon_tags_1362[] = { 3, 114, 160, 158 };
static const label SUC_lexicon_tags_1363[] = { 2, 91, 49 };
static const label SUC_lexicon_tags_1364[] = { 2, 158, 81 };
static const label SUC_lexicon_tags_1365[] = { 4, 83, 73, 79, 93 };
static const label SUC_lexicon_tags_1366[] = { 5, 93, 56, 83, 3, 41 };
static const label SUC_lexicon_tags_1367[] = { 6, 56, 48, 44, 41, 38, 52 };
static const label SUC_lexicon_tags_1368[] = { 2, 114, 151 };
static const label SUC_lexicon_tags_1369[] = { 1, 145 };
static const label SUC_lexicon_tags_1370[] = { 3, 78, 82, 40 };
static const label SUC_lexicon_tags_1371[] = { 3, 112, 67, 131 };
static const label SUC_lexicon_tags_1372[] = { 3, 51, 106, 45 };
static const label SUC_lexicon_tags_1373[] = { 3, 97, 160, 93 };
static const label SUC_lexicon_tags_1374[] = { 3, 92, 80, 93 };
static const label SUC_lexicon_tags_1375[] = { 2, 73, 89 };
static const label SUC_lexicon_tags_1376[] = { 4, 153, 151, 71, 74 };
static const label SUC_lexicon_tags_1377[] = { 4, 92, 82, 159, 151 };
static const label SUC_lexicon_tags_1378[] = { 2, 92, 51 };
static const label SUC_lexicon_tags_1379[] = { 2, 64, 46 };
static const label SUC_lexicon_tags_1380[] = { 4, 41, 56, 3, 93 };
static const label SUC_lexicon_tags_1381[] = { 3, 156, 160, 93 };
static const label SUC_lexicon_tags_1382[] = { 2, 88, 55 };
static const label SUC_lexicon_tags_1383[] = { 4, 91, 163, 99, 81 };
static const label SUC_lexicon_tags_1384[] = { 7, 56, 114, 153, 151, 38, 149, 158 };
static const label SUC_lexicon_tags_1385[] = { 4, 56, 109, 99, 41 };
static const label SUC_lexicon_tags_1386[] = { 6, 159, 83, 78, 92, 82, 93 };
static const label SUC_lexicon_tags_1387[] = { 5, 83, 81, 91, 114, 93 };
static const label SUC_lexicon_tags_1388[] = { 3, 92, 93, 40 };
static const label SUC_lexicon_tags_1389[] = { 4, 114, 106, 113, 102 };
static const label SUC_lexicon_tags_1390[] = { 5, 153, 151, 46, 114, 52 };
static const label SUC_lexicon_tags_1391[] = { 3, 10, 139, 118 };
static const label SUC_lexicon_tags_1392[] = { 2, 51, 102 };
static const label SUC_lexicon_tags_1393[] = { 3, 153, 158, 93 };
static const label SUC_lexicon_tags_1394[] = { 3, 83, 153, 158 };
static const label SUC_lexicon_tags_1395[] = { 4, 73, 114, 31, 148 };
static const label SUC_lexicon_tags_1396[] = { 4, 0, 149, 114, 131 };
static const label SUC_lexicon_tags_1397[] = { 3, 66, 64, 52 };
static const label SUC_lexicon_tags_1398[] = { 3, 91, 79, 77 };
static const label SUC_lexicon_tags_1399[] = { 3, 56, 114, 81 };
static const label SUC_lexicon_tags_1400[] = { 3, 162, 75, 72 };
static const label SUC_lexicon_tags_1401[] = { 4, 83, 73, 149, 114 };
static const label SUC_lexicon_tags_1402[] = { 2, 76, 113 };
static const label SUC_lexicon_tags_1403[] = { 4, 83, 154, 159, 82 };
static const label SUC_lexicon_tags_1404[] = { 4, 83, 79, 93, 32 };
static const label SUC_lexicon_tags_1405[] = { 5, 0, 112, 149, 49, 131 };
static const label SUC_lexicon_tags_1406[] = { 3, 149, 154, 159 };
static const label SUC_lexicon_tags_1407[] = { 4, 92, 51, 82, 45 };
static const label SUC_lexicon_tags_1408[] = { 2, 91, 151 };
static const label SUC_lexicon_tags_1409[] = { 4, 41, 114, 99, 81 };
static const label SUC_lexicon_tags_1410[] = { 3, 0, 83, 82 };
static const label SUC_lexicon_tags_1411[] = { 9, 56, 49, 3, 114, 41, 153, 131, 52, 46 };
static const label SUC_lexicon_tags_1412[] = { 2, 128, 18 };
static const label SUC_lexicon_tags_1413[] = { 4, 18, 0, 67, 130 };
static const label SUC_lexicon_tags_1414[] = { 4, 83, 163, 151, 93 };
static const label SUC_lexicon_tags_1415[] = { 3, 37, 82, 159 };
static const label SUC_lexicon_tags_1416[] = { 5, 92, 88, 82, 161, 78 };
static const label SUC_lexicon_tags_1417[] = { 3, 92, 89, 93 };
static const label SUC_lexicon_tags_1418[] = { 1, 116 };
static const label SUC_lexicon_tags_1419[] = { 3, 149, 56, 114 };
static const label SUC_lexicon_tags_1420[] = { 2, 148, 67 };
static const label SUC_lexicon_tags_1421[] = { 2, 140, 141 };
static const label SUC_lexicon_tags_1422[] = { 8, 79, 83, 114, 78, 92, 82, 93, 90 };
static const label SUC_lexicon_tags_1423[] = { 1, 95 };
static const label SUC_lexicon_tags_1424[] = { 4, 93, 114, 14, 122 };
static const label SUC_lexicon_tags_1425[] = { 4, 51, 55, 151, 45 };
static const label SUC_lexicon_tags_1426[] = { 5, 0, 153, 149, 151, 32 };
static const label SUC_lexicon_tags_1427[] = { 4, 92, 114, 161, 93 };
static const label SUC_lexicon_tags_1428[] = { 4, 56, 114, 46, 52 };
static const label SUC_lexicon_tags_1429[] = { 3, 83, 56, 46 };
static const label SUC_lexicon_tags_1430[] = { 3, 0, 112, 93 };
static const label SUC_lexicon_tags_1431[] = { 3, 149, 38, 46 };
static const label SUC_lexicon_tags_1432[] = { 5, 0, 112, 49, 148, 131 };
static const label SUC_lexicon_tags_1433[] = { 2, 83, 91 };
static const label SUC_lexicon_tags_1434[] = { 12, 56, 49, 55, 48, 44, 41, 0, 43, 47, 40, 52, 51 };
static const label SUC_lexicon_tags_1435[] = { 4, 83, 79, 114, 151 };
static const label SUC_lexicon_tags_1436[] = { 2, 12, 120 };
static const label SUC_lexicon_tags_1437[] = { 2, 56, 111 };
static const label SUC_lexicon_tags_1438[] = { 5, 51, 154, 159, 113, 45 };
static const label SUC_lexicon_tags_1439[] = { 5, 0, 112, 83, 160, 131 };
static const label SUC_lexicon_tags_1440[] = { 2, 90, 161 };
static const label SUC_lexicon_tags_1441[] = { 3, 151, 46, 52 };
static const label SUC_lexicon_tags_1442[] = { 3, 73, 83, 79 };
static const label SUC_lexicon_tags_1443[] = { 4, 114, 82, 113, 78 };
static const label SUC_lexicon_tags_1444[] = { 3, 155, 67, 158 };
static const label SUC_lexicon_tags_1445[] = { 5, 83, 114, 82, 113, 93 };
static const label SUC_lexicon_tags_1446[] = { 7, 130, 142, 114, 19, 0, 93, 149 };
static const label SUC_lexicon_tags_1447[] = { 6, 76, 78, 92, 82, 148, 93 };
static const label SUC_lexicon_tags_1448[] = { 2, 89, 160 };
static const label SUC_lexicon_tags_1449[] = { 4, 83, 82, 159, 78 };
static const label SUC_lexicon_tags_1450[] = { 3, 114, 161, 113 };
static const label SUC_lexicon_tags_1451[] = { 3, 149, 114, 131 };
static const label SUC_lexicon_tags_1452[] = { 3, 0, 149, 93 };
static const label SUC_lexicon_tags_1453[] = { 2, 102, 93 };
static const label SUC_lexicon_tags_1454[] = { 3, 162, 151, 82 };
static const label SUC_lexicon_tags_1455[] = { 3, 0, 83, 114 };
static const label SUC_lexicon_tags_1456[] = { 3, 56, 77, 93 };
static const label SUC_lexicon_tags_1457[] = { 5, 73, 79, 83, 82, 78 };
static const label SUC_lexicon_tags_1458[] = { 4, 154, 159, 45, 136 };
static const label SUC_lexicon_tags_1459[] = { 2, 91, 160 };
static const label SUC_lexicon_tags_1460[] = { 3, 83, 82, 93 };
static const label SUC_lexicon_tags_1461[] = { 2, 5, 3 };
static const label SUC_lexicon_tags_1462[] = { 4, 163, 109, 99, 93 };
static const label SUC_lexicon_tags_1463[] = { 3, 91, 114, 77 };
static const label SUC_lexicon_tags_1464[] = { 3, 162, 4, 75 };
static const label SUC_lexicon_tags_1465[] = { 4, 83, 97, 38, 160 };
static const label SUC_lexicon_tags_1466[] = { 4, 83, 79, 89, 158 };
static const label SUC_lexicon_tags_1467[] = { 6, 79, 49, 3, 83, 112, 0 };
static const label SUC_lexicon_tags_1468[] = { 1, 127 };
static const label SUC_lexicon_tags_1469[] = { 3, 93, 64, 52 };
static const label SUC_lexicon_tags_1470[] = { 3, 55, 89, 40 };
static const label SUC_lexicon_tags_1471[] = { 4, 76, 92, 88, 108 };
static const label SUC_lexicon_tags_1472[] = { 3, 83, 114, 38 };
static const label SUC_lexicon_tags_1473[] = { 4, 153, 56, 151, 93 };
static const label SUC_lexicon_tags_1474[] = { 2, 149, 113 };
static const label SUC_lexicon_tags_1475[] = { 3, 114, 103, 107 };
static const label SUC_lexicon_tags_1476[] = { 2, 73, 161 };
static const label SUC_lexicon_tags_1477[] = { 5, 92, 91, 88, 90, 93 };
static const label SUC_lexicon_tags_1478[] = { 4, 73, 92, 82, 93 };
static const label SUC_lexicon_tags_1479[] = { 2, 149, 91 };
static const label SUC_lexicon_tags_1480[] = { 2, 10, 118 };
static const label SUC_lexicon_tags_1481[] = { 2, 46, 107 };
static const label SUC_lexicon_tags_1482[] = { 2, 114, 131 };
static const label SUC_lexicon_tags_1483[] = { 5, 83, 79, 56, 114, 93 };
static const label SUC_lexicon_tags_1484[] = { 2, 49, 46 };
static const label SUC_lexicon_tags_1485[] = { 4, 83, 79, 77, 93 };
static const label SUC_lexicon_tags_1486[] = { 3, 73, 85, 4 };
static const label SUC_lexicon_tags_1487[] = { 3, 83, 160, 158 };
static const label SUC_lexicon_tags_1488[] = { 2, 154, 82 };
static const label SUC_lexicon_tags_1489[] = { 3, 83, 10, 118 };
static const label SUC_lexicon_tags_1490[] = { 4, 0, 153, 151, 3 };
static const label SUC_lexicon_tags_1491[] = { 3, 90, 114, 113 };
static const label SUC_lexicon_tags_1492[] = { 6, 114, 92, 93, 73, 149, 113 };
static const label SUC_lexicon_tags_1493[] = { 3, 38, 160, 93 };
static const label SUC_lexicon_tags_1494[] = { 2, 79, 56 };
static const label SUC_lexicon_tags_1495[] = { 5, 0, 56, 114, 3, 41 };
static const label SUC_lexicon_tags_1496[] = { 3, 41, 56, 32 };
static const label SUC_lexicon_tags_1497[] = { 4, 0, 112, 49, 131 };
static const label SUC_lexicon_tags_1498[] = { 2, 52, 93 };
static const label SUC_lexicon_tags_1499[] = { 3, 83, 151, 32 };
static const label SUC_lexicon_tags_1500[] = { 6, 79, 83, 78, 0, 82, 88 };
static const label SUC_lexicon_tags_1501[] = { 3, 38, 89, 93 };
static const label SUC_lexicon_tags_1502[] = { 5, 92, 88, 82, 159, 89 };
static const label SUC_lexicon_tags_1503[] = { 2, 56, 89 };
static const label SUC_lexicon_tags_1504[] = { 3, 160, 113, 161 };
static const label SUC_lexicon_tags_1505[] = { 5, 92, 88, 82, 89, 78 };
static const label SUC_lexicon_tags_1506[] = { 2, 73, 87 };
static const label SUC_lexicon_tags_1507[] = { 4, 83, 114, 151, 93 };
static const label SUC_lexicon_tags_1508[] = { 3, 91, 87, 81 };
static const label SUC_lexicon_tags_1509[] = { 5, 0, 163, 99, 3, 41 };
static const label SUC_lexicon_tags_1510[] = { 2, 135, 114 };
static const label SUC_lexicon_tags_1511[] = { 6, 164, 159, 98, 108, 55, 40 };
static const label SUC_lexicon_tags_1512[] = { 2, 112, 151 };
static const label SUC_lexicon_tags_1513[] = { 4, 92, 88, 89, 113 };
static const label SUC_lexicon_tags_1514[] = { 2, 130, 21 };
static const label SUC_lexicon_tags_1515[] = { 2, 89, 113 };
static const label SUC_lexicon_tags_1516[] = { 5, 83, 73, 114, 33, 6 };
static const label SUC_lexicon_tags_1517[] = { 3, 73, 85, 162 };
static const label SUC_lexicon_tags_1518[] = { 6, 56, 163, 99, 41, 160, 109 };
static const label SUC_lexicon_tags_1519[] = { 4, 56, 163, 3, 41 };
static const label SUC_lexicon_tags_1520[] = { 2, 41, 131 };
static const label SUC_lexicon_tags_1521[] = { 4, 0, 56, 77, 93 };
static const label SUC_lexicon_tags_1522[] = { 3, 56, 151, 93 };
static const label SUC_lexicon_tags_1523[] = { 2, 92, 57 };
static const label SUC_lexicon_tags_1524[] = { 7, 163, 118, 0, 10, 160, 93, 21 };
static const label SUC_lexicon_tags_1525[] = { 3, 93, 10, 118 };
static const label SUC_lexicon_tags_1526[] = { 3, 49, 89, 93 };
static const label SUC_lexicon_tags_1527[] = { 3, 149, 153, 67 };
static const label SUC_lexicon_tags_1528[] = { 2, 111, 131 };
static const label SUC_lexicon_tags_1529[] = { 5, 0, 112, 114, 148, 131 };
static const label SUC_lexicon_tags_1530[] = { 4, 83, 149, 114, 32 };
static const label SUC_lexicon_tags_1531[] = { 3, 114, 151, 158 };
static const label SUC_lexicon_tags_1532[] = { 3, 111, 49, 89 };
static const label SUC_lexicon_tags_1533[] = { 3, 0, 49, 82 };
static const label SUC_lexicon_tags_1534[] = { 3, 73, 75, 72 };
static const label SUC_lexicon_tags_1535[] = { 5, 92, 91, 49, 90, 93 };
static const label SUC_lexicon_tags_1536[] = { 4, 149, 114, 33, 93 };
static const label SUC_lexicon_tags_1537[] = { 4, 153, 149, 151, 158 };
static const label SUC_lexicon_tags_1538[] = { 2, 73, 90 };
static const label SUC_lexicon_tags_1539[] = { 3, 149, 114, 38 };
static const label SUC_lexicon_tags_1540[] = { 6, 79, 159, 83, 78, 82, 154 };
static const label SUC_lexicon_tags_1541[] = { 2, 49, 48 };
static const label SUC_lexicon_tags_1542[] = { 2, 73, 86 };
static const label SUC_lexicon_tags_1543[] = { 4, 92, 98, 159, 108 };
static const label SUC_lexicon_tags_1544[] = { 2, 103, 97 };
static const label SUC_lexicon_tags_1545[] = { 3, 73, 149, 82 };
static const label SUC_lexicon_tags_1546[] = { 3, 82, 113, 161 };
static const label SUC_lexicon_tags_1547[] = { 5, 149, 114, 151, 158, 93 };
static const label SUC_lexicon_tags_1548[] = { 4, 92, 149, 88, 89 };
static const label SUC_lexicon_tags_1549[] = { 3, 0, 151, 93 };
static const label SUC_lexicon_tags_1550[] = { 4, 83, 56, 114, 93 };
static const label SUC_lexicon_tags_1551[] = { 4, 164, 55, 98, 40 };
static const label SUC_lexicon_tags_1552[] = { 3, 78, 82, 108 };
static const label SUC_lexicon_tags_1553[] = { 4, 0, 41, 99, 81 };
static const label SUC_lexicon_tags_1554[] = { 6, 56, 163, 99, 41, 151, 109 };
static const label SUC_lexicon_tags_1555[] = { 4, 163, 10, 99, 118 };
static const label SUC_lexicon_tags_1556[] = { 2, 75, 115 };
static const label SUC_lexicon_tags_1557[] = { 3, 83, 149, 79 };
static const label SUC_lexicon_tags_1558[] = { 2, 58, 93 };
static const label SUC_lexicon_tags_1559[] = { 2, 56, 54 };
static const label SUC_lexicon_tags_1560[] = { 3, 83, 82, 131 };
static const label SUC_lexicon_tags_1561[] = { 7, 114, 78, 92, 82, 73, 149, 88 };
static const label SUC_lexicon_tags_1562[] = { 3, 56, 151, 158 };
static const label SUC_lexicon_tags_1563[] = { 4, 83, 79, 73, 151 };
static const label SUC_lexicon_tags_1564[] = { 4, 56, 114, 89, 158 };
static const label SUC_lexicon_tags_1565[] = { 3, 32, 114, 93 };
static const label SUC_lexicon_tags_1566[] = { 3, 81, 99, 41 };
static const label SUC_lexicon_tags_1567[] = { 4, 92, 93, 161, 40 };
static const label SUC_lexicon_tags_1568[] = { 4, 153, 49, 151, 3 };
static const label SUC_lexicon_tags_1569[] = { 2, 91, 127 };
static const label SUC_lexicon_tags_1570[] = { 2, 159, 71 };
static const label SUC_lexicon_tags_1571[] = { 1, 7 };
static const label SUC_lexicon_tags_1572[] = { 4, 0, 112, 83, 79 };
static const label SUC_lexicon_tags_1573[] = { 6, 71, 83, 114, 153, 151, 93 };
static const label SUC_lexicon_tags_1574[] = { 1, 128 };
static const label SUC_lexicon_tags_1575[] = { 2, 90, 82 };
static const label SUC_lexicon_tags_1576[] = { 5, 73, 153, 151, 158, 32 };
static const label SUC_lexicon_tags_1577[] = { 3, 103, 160, 107 };
static const label SUC_lexicon_tags_1578[] = { 2, 5, 0 };
static const label SUC_lexicon_tags_1579[] = { 4, 76, 92, 113, 93 };
static const label SUC_lexicon_tags_1580[] = { 3, 103, 38, 97 };
static const label SUC_lexicon_tags_1581[] = { 3, 58, 64, 93 };
static const label SUC_lexicon_tags_1582[] = { 3, 164, 55, 108 };
static const label SUC_lexicon_tags_1583[] = { 3, 73, 83, 93 };
static const label SUC_lexicon_tags_1584[] = { 4, 83, 79, 114, 160 };
static const label SUC_lexicon_tags_1585[] = { 5, 91, 87, 79, 77, 81 };
static const label SUC_lexicon_tags_1586[] = { 5, 121, 114, 12, 119, 93 };
static const label SUC_lexicon_tags_1587[] = { 3, 73, 33, 1 };
static const label SUC_lexicon_tags_1588[] = { 3, 0, 148, 67 };
static const label SUC_lexicon_tags_1589[] = { 3, 83, 56, 41 };
static const label SUC_lexicon_tags_1590[] = { 1, 61 };
static const label SUC_lexicon_tags_1591[] = { 5, 73, 92, 114, 113, 93 };
static const label SUC_lexicon_tags_1592[] = { 4, 93, 56, 149, 41 };
static const label SUC_lexicon_tags_1593[] = { 3, 38, 46, 93 };
static const label SUC_lexicon_tags_1594[] = { 3, 153, 56, 151 };
static const label SUC_lexicon_tags_1595[] = { 1, 156 };
static const label SUC_lexicon_tags_1596[] = { 6, 79, 83, 78, 92, 82, 113 };
static const label SUC_lexicon_tags_1597[] = { 3, 37, 82, 113 };
static const label SUC_lexicon_tags_1598[] = { 5, 32, 91, 79, 83, 93 };
static const label SUC_lexicon_tags_1599[] = { 3, 83, 79, 41 };
static const label SUC_lexicon_tags_1600[] = { 2, 162, 141 };
static const label SUC_lexicon_tags_1601[] = { 4, 41, 99, 114, 81 };
static const label SUC_lexicon_tags_1602[] = { 3, 82, 136, 78 };
static const label SUC_lexicon_tags_1603[] = { 3, 140, 153, 151 };
static const label SUC_lexicon_tags_1604[] = { 5, 153, 114, 151, 158, 93 };
static const label SUC_lexicon_tags_1605[] = { 2, 133, 73 };
static const label SUC_lexicon_tags_1606[] = { 5, 83, 79, 151, 89, 158 };
static const label SUC_lexicon_tags_1607[] = { 1, 9 };
static const label SUC_lexicon_tags_1608[] = { 3, 9, 117, 81 };
static const label SUC_lexicon_tags_1609[] = { 2, 83, 77 };
static const label SUC_lexicon_tags_1610[] = { 2, 99, 3 };
static const label SUC_lexicon_tags_1611[] = { 3, 92, 57, 63 };
static const label SUC_lexicon_tags_1612[] = { 3, 0, 163, 41 };
static const label SUC_lexicon_tags_1613[] = { 3, 92, 88, 82 };
static const label SUC_lexicon_tags_1614[] = { 4, 93, 46, 89, 52 };
static const label SUC_lexicon_tags_1615[] = { 4, 92, 151, 159, 93 };
static const label SUC_lexicon_tags_1616[] = { 3, 97, 103, 107 };
static const label SUC_lexicon_tags_1617[] = { 2, 103, 46 };
static const label SUC_lexicon_tags_1618[] = { 3, 0, 49, 114 };
static const label SUC_lexicon_tags_1619[] = { 2, 13, 122 };
static const label SUC_lexicon_tags_1620[] = { 3, 91, 79, 81 };
static const label SUC_lexicon_tags_1621[] = { 7, 55, 47, 43, 92, 40, 37, 51 };
static const label SUC_lexicon_tags_1622[] = { 6, 56, 48, 44, 38, 93, 52 };
static const label SUC_lexicon_tags_1623[] = { 2, 28, 23 };
static const label SUC_lexicon_tags_1624[] = { 2, 153, 112 };
static const label SUC_lexicon_tags_1625[] = { 4, 0, 149, 114, 21 };
static const label SUC_lexicon_tags_1626[] = { 5, 83, 91, 79, 89, 93 };
static const label SUC_lexicon_tags_1627[] = { 3, 83, 73, 0 };
static const label SUC_lexicon_tags_1628[] = { 4, 37, 51, 45, 102 };
static const label SUC_lexicon_tags_1629[] = { 3, 41, 3, 93 };
static const label SUC_lexicon_tags_1630[] = { 3, 163, 99, 131 };
static const label SUC_lexicon_tags_1631[] = { 3, 0, 92, 93 };
static const label SUC_lexicon_tags_1632[] = { 11, 56, 55, 48, 47, 41, 0, 43, 44, 40, 52, 51 };
static const label SUC_lexicon_tags_1633[] = { 2, 37, 45 };
static const label SUC_lexicon_tags_1634[] = { 3, 83, 79, 127 };
static const label SUC_lexicon_tags_1635[] = { 2, 76, 92 };
static const label SUC_lexicon_tags_1636[] = { 2, 140, 49 };
static const label SUC_lexicon_tags_1637[] = { 4, 83, 79, 56, 114 };
static const label SUC_lexicon_tags_1638[] = { 4, 0, 56, 151, 93 };
static const label SUC_lexicon_tags_1639[] = { 3, 0, 112, 149 };
static const label SUC_lexicon_tags_1640[] = { 2, 66, 93 };
static const label SUC_lexicon_tags_1641[] = { 4, 92, 83, 55, 82 };
static const label SUC_lexicon_tags_1642[] = { 3, 76, 90, 113 };
static const label SUC_lexicon_tags_1643[] = { 4, 124, 73, 114, 113 };
static const label SUC_lexicon_tags_1644[] = { 3, 153, 149, 151 };
static const label SUC_lexicon_tags_1645[] = { 2, 151, 109 };
static const label SUC_lexicon_tags_1646[] = { 5, 149, 56, 114, 3, 41 };
static const label SUC_lexicon_tags_1647[] = { 6, 83, 114, 92, 82, 93, 149 };
static const label SUC_lexicon_tags_1648[] = { 6, 79, 83, 163, 99, 151, 109 };
static const label SUC_lexicon_tags_1649[] = { 5, 83, 149, 73, 114, 79 };
static const label SUC_lexicon_tags_1650[] = { 3, 0, 154, 159 };
static const label SUC_lexicon_tags_1651[] = { 1, 28 };
static const label SUC_lexicon_tags_1652[] = { 6, 79, 83, 78, 82, 151, 113 };
static const label SUC_lexicon_tags_1653[] = { 2, 0, 73 };
static const label SUC_lexicon_tags_1654[] = { 6, 114, 153, 151, 73, 149, 158 };
static const label SUC_lexicon_tags_1655[] = { 4, 73, 92, 114, 93 };
static const label SUC_lexicon_tags_1656[] = { 2, 118, 41 };
static const label SUC_lexicon_tags_1657[] = { 3, 91, 149, 77 };
static const label SUC_lexicon_tags_1658[] = { 3, 163, 99, 49 };
static const label SUC_lexicon_tags_1659[] = { 4, 92, 55, 88, 161 };
static const label SUC_lexicon_tags_1660[] = { 4, 92, 90, 88, 113 };
static const label SUC_lexicon_tags_1661[] = { 2, 73, 117 };
static const label SUC_lexicon_tags_1662[] = { 3, 0, 99, 81 };
static const label SUC_lexicon_tags_1663[] = { 3, 73, 153, 151 };
static const label SUC_lexicon_tags_1664[] = { 2, 35, 3 };
static const label SUC_lexicon_tags_1665[] = { 9, 56, 48, 122, 41, 153, 151, 52, 46, 158 };
static const label SUC_lexicon_tags_1666[] = { 2, 46, 93 };
static const label SUC_lexicon_tags_1667[] = { 2, 127, 126 };
static const label SUC_lexicon_tags_1668[] = { 4, 162, 85, 75, 72 };
static const label SUC_lexicon_tags_1669[] = { 3, 83, 79, 109 };
static const label SUC_lexicon_tags_1670[] = { 2, 49, 151 };
static const label SUC_lexicon_tags_1671[] = { 4, 163, 99, 158, 93 };
static const label SUC_lexicon_tags_1672[] = { 3, 56, 49, 114 };
static const label SUC_lexicon_tags_1673[] = { 6, 79, 49, 3, 83, 112, 151 };
static const label SUC_lexicon_tags_1674[] = { 3, 124, 89, 137 };
static const label SUC_lexicon_tags_1675[] = { 3, 92, 90, 80 };
static const label SUC_lexicon_tags_1676[] = { 4, 56, 89, 158, 93 };
static const label SUC_lexicon_tags_1677[] = { 4, 56, 91, 3, 41 };
static const label SUC_lexicon_tags_1678[] = { 5, 93, 56, 149, 3, 41 };
static const label SUC_lexicon_tags_1679[] = { 2, 0, 151 };
static const label SUC_lexicon_tags_1680[] = { 7, 79, 49, 3, 89, 83, 41, 93 };
static const label SUC_lexicon_tags_1681[] = { 5, 56, 49, 114, 3, 41 };
static const label SUC_lexicon_tags_1682[] = { 5, 78, 92, 88, 82, 40 };
static const label SUC_lexicon_tags_1683[] = { 2, 85, 33 };
static const label SUC_lexicon_tags_1684[] = { 5, 83, 79, 2, 67, 77 };
static const label SUC_lexicon_tags_1685[] = { 2, 83, 155 };
static const label SUC_lexicon_tags_1686[] = { 4, 83, 66, 64, 81 };
static const label SUC_lexicon_tags_1687[] = { 4, 162, 85, 75, 114 };
static const label SUC_lexicon_tags_1688[] = { 5, 83, 79, 163, 99, 93 };
static const label SUC_lexicon_tags_1689[] = { 5, 78, 92, 82, 159, 93 };
static const label SUC_lexicon_tags_1690[] = { 3, 52, 46, 54 };
static const label SUC_lexicon_tags_1691[] = { 5, 149, 20, 56, 114, 14 };
static const label SUC_lexicon_tags_1692[] = { 3, 73, 56, 93 };
static const label SUC_lexicon_tags_1693[] = { 5, 73, 83, 92, 82, 93 };
static const label SUC_lexicon_tags_1694[] = { 4, 83, 55, 82, 151 };
static const label SUC_lexicon_tags_1695[] = { 3, 151, 158, 131 };
static const label SUC_lexicon_tags_1696[] = { 4, 91, 56, 109, 149 };
static const label SUC_lexicon_tags_1697[] = { 2, 73, 151 };
static const label SUC_lexicon_tags_1698[] = { 4, 149, 114, 151, 160 };
static const label SUC_lexicon_tags_1699[] = { 3, 162, 85, 4 };
static const label SUC_lexicon_tags_1700[] = { 5, 92, 88, 82, 80, 78 };
static const label SUC_lexicon_tags_1701[] = { 5, 92, 88, 114, 89, 93 };
static const label SUC_lexicon_tags_1702[] = { 3, 0, 49, 32 };
static const label SUC_lexicon_tags_1703[] = { 5, 163, 99, 3, 109, 41 };
static const label SUC_lexicon_tags_1704[] = { 6, 79, 83, 161, 78, 82, 160 };
static const label SUC_lexicon_tags_1705[] = { 6, 112, 114, 0, 131, 73, 149 };
static const label SUC_lexicon_tags_1706[] = { 5, 73, 83, 149, 114, 79 };
static const label SUC_lexicon_tags_1707[] = { 3, 83, 111, 131 };
static const label SUC_lexicon_tags_1708[] = { 6, 79, 83, 78, 82, 93, 81 };
static const label SUC_lexicon_tags_1709[] = { 2, 113, 108 };
static const label SUC_lexicon_tags_1710[] = { 4, 91, 56, 109, 114 };
static const label SUC_lexicon_tags_1711[] = { 2, 93, 32 };
static const label SUC_lexicon_tags_1712[] = { 9, 56, 79, 83, 55, 114, 78, 92, 82, 93 };
static const label SUC_lexicon_tags_1713[] = { 5, 78, 164, 82, 98, 40 };
static const label SUC_lexicon_tags_1714[] = { 5, 83, 140, 149, 114, 82 };
static const label SUC_lexicon_tags_1715[] = { 2, 92, 76 };
static const label SUC_lexicon_tags_1716[] = { 3, 0, 112, 4 };
static const label SUC_lexicon_tags_1717[] = { 3, 73, 92, 131 };
static const label SUC_lexicon_tags_1718[] = { 5, 76, 90, 80, 86, 78 };
static const label SUC_lexicon_tags_1719[] = { 6, 83, 163, 99, 91, 93, 81 };
static const label SUC_lexicon_tags_1720[] = { 2, 82, 34 };
static const label SUC_lexicon_tags_1721[] = { 3, 0, 160, 93 };
static const label SUC_lexicon_tags_1722[] = { 3, 103, 107, 93 };
static const label SUC_lexicon_tags_1723[] = { 3, 76, 90, 55 };
static const label SUC_lexicon_tags_open[] = { 109, 0, 1, 2, 3, 4, 5, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 113, 114, 115, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164 };
static const hash32_kv SUC_lexicon[SUC_lexicon_size] = {
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27200003, SUC_lexicon_tags_1 },
    { 0x75e00004, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c00007, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc1c00009, SUC_lexicon_tags_37 },
    { 0x4f600009, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2be0000c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa00011, SUC_lexicon_tags_25 },
    { 0xb7400012, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e00016, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a0001a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd780001d, SUC_lexicon_tags_46 },
    { 0x4660001e, SUC_lexicon_tags_11 },
    { 0xb60001f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a00023, SUC_lexicon_tags_85 },
    { 0xc400024, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e00027, SUC_lexicon_tags_4 },
    { 0x5ac00028, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9a0002a, SUC_lexicon_tags_7 },
    { 0xc680002b, SUC_lexicon_tags_10 },
    { 0xd760002b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c0002f, SUC_lexicon_tags_46 },
    { 0xb1000030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91600036, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc00039, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c40003b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e00042, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60000045, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20004a, SUC_lexicon_tags_17 },
    { 0x5bc0004a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9800050, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a600053, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2200055, SUC_lexicon_tags_8 },
    { 0x4d600056, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2440005b, SUC_lexicon_tags_68 },
    { 0x1540005b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x340005f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89000061, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x70200064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66e00066, SUC_lexicon_tags_37 },
    { 0x20e00066, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x39a00069, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd4a0006b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa80006d, SUC_lexicon_tags_33 },
    { 0x12c0006e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff200076, SUC_lexicon_tags_43 },
    { 0xadc00077, SUC_lexicon_tags_15 },
    { 0x9000076, SUC_lexicon_tags_11 },
    { 0x3d400077, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe20007b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x81e0007d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf820007f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e000083, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8e00085, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec0008b, SUC_lexicon_tags_27 },
    { 0x14c0008c, SUC_lexicon_tags_15 },
    { 0xaea0008d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa00094, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7ee00096, SUC_lexicon_tags_15 },
    { 0x7c600096, SUC_lexicon_tags_37 },
    { 0x42000096, SUC_lexicon_tags_29 },
    { 0x9e400098, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba0009d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f80009f, SUC_lexicon_tags_39 },
    { 0x832000a0, SUC_lexicon_tags_32 },
    { 0x536000a1, SUC_lexicon_tags_9 },
    { 0x1de0009f, SUC_lexicon_tags_17 },
    { 0x57a000a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4a000a5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786000aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e000ae, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6c2000b0, SUC_lexicon_tags_25 },
    { 0xd32000b1, SUC_lexicon_tags_15 },
    { 0x4a4000b2, SUC_lexicon_tags_7 },
    { 0x98c000b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902000b7, SUC_lexicon_tags_193 },
    { 0x284000b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a000c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08000cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08000db, SUC_lexicon_tags_25 },
    { 0x7a4000dc, SUC_lexicon_tags_38 },
    { 0xc28000dd, SUC_lexicon_tags_26 },
    { 0xae000dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4000e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98000e9, SUC_lexicon_tags_35 },
    { 0xd3c000ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac000ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4000f0, SUC_lexicon_tags_23 },
    { 0x37e000f1, SUC_lexicon_tags_29 },
    { 0x87c000f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888000f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfca000f9, SUC_lexicon_tags_11 },
    { 0xe4000f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0000fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf600101, SUC_lexicon_tags_63 },
    { 0xf7400102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5800108, SUC_lexicon_tags_14 },
    { 0xde000108, SUC_lexicon_tags_87 },
    { 0x9680010a, SUC_lexicon_tags_25 },
    { 0xfb20010a, SUC_lexicon_tags_47 },
    { 0xdfa0010c, SUC_lexicon_tags_29 },
    { 0x2920010c, SUC_lexicon_tags_2 },
    { 0x59c0010d, SUC_lexicon_tags_16 },
    { 0x4960010b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c200115, SUC_lexicon_tags_25 },
    { 0x2dc00116, SUC_lexicon_tags_7 },
    { 0x80a00117, SUC_lexicon_tags_25 },
    { 0x9ae00118, SUC_lexicon_tags_37 },
    { 0x14600118, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe880011b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c0011e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a00121, SUC_lexicon_tags_63 },
    { 0x43600122, SUC_lexicon_tags_12 },
    { 0xcc200122, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f800126, SUC_lexicon_tags_47 },
    { 0xe8a00127, SUC_lexicon_tags_7 },
    { 0x56a00128, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77800130, SUC_lexicon_tags_17 },
    { 0x65600130, SUC_lexicon_tags_8 },
    { 0x74c00132, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c00135, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e400138, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f60013c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc0c0013e, SUC_lexicon_tags_17 },
    { 0x1b60013e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21000141, SUC_lexicon_tags_66 },
    { 0xa9000141, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba00147, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c0014a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc40014c, SUC_lexicon_tags_25 },
    { 0x1c20014c, SUC_lexicon_tags_39 },
    { 0x2780014e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c200155, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf800157, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6840015b, SUC_lexicon_tags_13 },
    { 0x8e80015c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4100015e, SUC_lexicon_tags_38 },
    { 0x6d60015f, SUC_lexicon_tags_37 },
    { 0x26e0015e, SUC_lexicon_tags_63 },
    { 0xb500015e, SUC_lexicon_tags_87 },
    { 0x2b80015e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3260016b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a00172, SUC_lexicon_tags_11 },
    { 0xf3400172, SUC_lexicon_tags_2 },
    { 0x8d800173, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9e00176, SUC_lexicon_tags_12 },
    { 0xba400177, SUC_lexicon_tags_4 },
    { 0x42e00177, SUC_lexicon_tags_26 },
    { 0x22a00179, SUC_lexicon_tags_31 },
    { 0xd7800179, SUC_lexicon_tags_37 },
    { 0x6fa00178, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x10e0017d, SUC_lexicon_tags_64 },
    { 0xb4e0017e, SUC_lexicon_tags_43 },
    { 0xcd20017f, SUC_lexicon_tags_2 },
    { 0x8ca0017e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7600182, SUC_lexicon_tags_26 },
    { 0x24c00183, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0x42200185, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a00187, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x60600189, SUC_lexicon_tags_7 },
    { 0x2ba0018a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3080018c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5720018f, SUC_lexicon_tags_17 },
    { 0x44a00190, SUC_lexicon_tags_0 },
    { 0xf8800191, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5000195, SUC_lexicon_tags_14 },
    { 0xade00196, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a001a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c001ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf48001ae, SUC_lexicon_tags_0 },
    { 0x972001af, SUC_lexicon_tags_23 },
    { 0xa28001af, SUC_lexicon_tags_1 },
    { 0x8ac001ae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e001b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8001bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd10001bf, SUC_lexicon_tags_15 },
    { 0x39e001c0, SUC_lexicon_tags_17 },
    { 0xa3a001c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72001c4, SUC_lexicon_tags_60 },
    { 0xf68001c5, SUC_lexicon_tags_9 },
    { 0x6fe001c6, SUC_lexicon_tags_14 },
    { 0xfc6001c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4001ca, SUC_lexicon_tags_29 },
    { 0x528001cb, SUC_lexicon_tags_55 },
    { 0xb2c001cc, SUC_lexicon_tags_28 },
    { 0x376001cc, SUC_lexicon_tags_63 },
    { 0xec001cc, SUC_lexicon_tags_27 },
    { 0x786001cd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6001d4, SUC_lexicon_tags_2 },
    { 0x72001d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe001da, SUC_lexicon_tags_45 },
    { 0xda0001da, SUC_lexicon_tags_31 },
    { 0x746001dc, SUC_lexicon_tags_45 },
    { 0x9e001dd, SUC_lexicon_tags_13 },
    { 0xf1e001de, SUC_lexicon_tags_63 },
    { 0x21c001da, SUC_lexicon_tags_33 },
    { 0x884001e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda8001e2, SUC_lexicon_tags_15 },
    { 0x8ae001e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d4001e5, SUC_lexicon_tags_17 },
    { 0xe78001e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2001f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x534001f3, SUC_lexicon_tags_17 },
    { 0xe72001f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca001f7, SUC_lexicon_tags_19 },
    { 0xbde001f8, SUC_lexicon_tags_1 },
    { 0xcac001f9, SUC_lexicon_tags_17 },
    { 0xf14001fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x752001fc, SUC_lexicon_tags_37 },
    { 0x5c001fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4f2001ff, SUC_lexicon_tags_11 },
    { 0x3d800200, SUC_lexicon_tags_82 },
    { 0xe000200, SUC_lexicon_tags_15 },
    { 0xf1600202, SUC_lexicon_tags_2 },
    { 0x97600201, SUC_lexicon_tags_0 },
    { 0xff400204, SUC_lexicon_tags_15 },
    { 0xc4000204, SUC_lexicon_tags_68 },
    { 0x102001ff, SUC_lexicon_tags_25 },
    { 0x12400207, SUC_lexicon_tags_27 },
    { 0xa0200208, SUC_lexicon_tags_7 },
    { 0x1a800208, SUC_lexicon_tags_46 },
    { 0xf800208, SUC_lexicon_tags_7 },
    { 0xb2000207, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe7a0020d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c00210, SUC_lexicon_tags_17 },
    { 0xabe00210, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53000213, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8560021b, SUC_lexicon_tags_11 },
    { 0x32a0021c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a00228, SUC_lexicon_tags_105 },
    { 0x8e400229, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc0022e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x96e00230, SUC_lexicon_tags_12 },
    { 0xbcc00231, SUC_lexicon_tags_37 },
    { 0x61800232, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5600235, SUC_lexicon_tags_18 },
    { 0x79c00235, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a0023a, SUC_lexicon_tags_7 },
    { 0xbd40023b, SUC_lexicon_tags_37 },
    { 0xc200023b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x460023e, SUC_lexicon_tags_37 },
    { 0x8d60023e, SUC_lexicon_tags_12 },
    { 0x1e800240, SUC_lexicon_tags_11 },
    { 0x8d400241, SUC_lexicon_tags_12 },
    { 0x1800240, SUC_lexicon_tags_11 },
    { 0x92600243, SUC_lexicon_tags_1 },
    { 0x35400241, SUC_lexicon_tags_2 },
    { 0xb1c00245, SUC_lexicon_tags_1 },
    { 0xfc400246, SUC_lexicon_tags_12 },
    { 0x82c00245, SUC_lexicon_tags_49 },
    { 0x7e800248, SUC_lexicon_tags_1 },
    { 0x5e000249, SUC_lexicon_tags_15 },
    { 0xb1e0024a, SUC_lexicon_tags_31 },
    { 0x4760024b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e400250, SUC_lexicon_tags_95 },
    { 0x93a00251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65800259, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7da0025b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e00262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48600265, SUC_lexicon_tags_27 },
    { 0x8c200265, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf000268, SUC_lexicon_tags_1 },
    { 0xe5400269, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba80026b, SUC_lexicon_tags_27 },
    { 0x3c80026c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c0026f, SUC_lexicon_tags_60 },
    { 0x88400270, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d200272, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf6400274, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb8e00276, SUC_lexicon_tags_12 },
    { 0xd600277, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc0027a, SUC_lexicon_tags_11 },
    { 0xf2c0027b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc00282, SUC_lexicon_tags_1 },
    { 0xf6a00283, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32600287, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a0028b, SUC_lexicon_tags_14 },
    { 0x9380028c, SUC_lexicon_tags_49 },
    { 0x1200028c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6840028f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x23800291, SUC_lexicon_tags_11 },
    { 0xa4000292, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc320029c, SUC_lexicon_tags_17 },
    { 0xe420029c, SUC_lexicon_tags_25 },
    { 0xbc00029e, SUC_lexicon_tags_15 },
    { 0x7dc0029f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa002a4, SUC_lexicon_tags_19 },
    { 0xbc002a4, SUC_lexicon_tags_7 },
    { 0xf78002a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2002aa, SUC_lexicon_tags_17 },
    { 0x61e002ab, SUC_lexicon_tags_42 },
    { 0xe14002ac, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x438002af, SUC_lexicon_tags_11 },
    { 0x96a002af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x22e002b2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce002bd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x882002bf, SUC_lexicon_tags_31 },
    { 0xe2a002c0, SUC_lexicon_tags_42 },
    { 0xbb4002c0, SUC_lexicon_tags_60 },
    { 0x538002c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x676002c4, SUC_lexicon_tags_83 },
    { 0x62c002c4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf04002c7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c002cc, SUC_lexicon_tags_25 },
    { 0x3c002cc, SUC_lexicon_tags_11 },
    { 0x40c002ce, SUC_lexicon_tags_30 },
    { 0x43c002cf, SUC_lexicon_tags_12 },
    { 0x5c6002cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10002d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a002d7, SUC_lexicon_tags_47 },
    { 0xbf8002d8, SUC_lexicon_tags_11 },
    { 0x23c002d9, SUC_lexicon_tags_15 },
    { 0x4b2002da, SUC_lexicon_tags_37 },
    { 0x9c0002d9, SUC_lexicon_tags_28 },
    { 0x3ae002dc, SUC_lexicon_tags_170 },
    { 0x18002da, SUC_lexicon_tags_1 },
    { 0x5dc002d9, SUC_lexicon_tags_11 },
    { 0x400002df, SUC_lexicon_tags_251 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a002e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4002e5, SUC_lexicon_tags_34 },
    { 0x77c002e6, SUC_lexicon_tags_17 },
    { 0xfb4002e7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe4c002e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58002ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca002f2, SUC_lexicon_tags_28 },
    { 0x542002f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2002fd, SUC_lexicon_tags_12 },
    { 0xfae002fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33200303, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xab400305, SUC_lexicon_tags_12 },
    { 0xef600306, SUC_lexicon_tags_14 },
    { 0x88e00307, SUC_lexicon_tags_18 },
    { 0x42200308, SUC_lexicon_tags_11 },
    { 0xf1a00309, SUC_lexicon_tags_9 },
    { 0x63e00306, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xade0030c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa880030e, SUC_lexicon_tags_38 },
    { 0x5400030f, SUC_lexicon_tags_5 },
    { 0x25c00310, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9800313, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc160031a, SUC_lexicon_tags_1 },
    { 0x31c0031a, SUC_lexicon_tags_19 },
    { 0x5800031a, SUC_lexicon_tags_57 },
    { 0x780031d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf800322, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a800324, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20200328, SUC_lexicon_tags_2 },
    { 0x87a00329, SUC_lexicon_tags_11 },
    { 0xd5400328, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e00331, SUC_lexicon_tags_9 },
    { 0xe9a00331, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25c00334, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x56800337, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7d800339, SUC_lexicon_tags_14 },
    { 0x2d60033a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5200342, SUC_lexicon_tags_1 },
    { 0x5ee00343, SUC_lexicon_tags_4 },
    { 0x88000344, SUC_lexicon_tags_4 },
    { 0xa0200344, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81600349, SUC_lexicon_tags_10 },
    { 0x3ae0034a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e00353, SUC_lexicon_tags_25 },
    { 0xfc400354, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29600356, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa620035a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8c60035c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a00362, SUC_lexicon_tags_6 },
    { 0xa0a00363, SUC_lexicon_tags_49 },
    { 0xbc200364, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c00367, SUC_lexicon_tags_1 },
    { 0xce400367, SUC_lexicon_tags_27 },
    { 0x7cc00369, SUC_lexicon_tags_37 },
    { 0x6ee0036a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91e0036c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f00036e, SUC_lexicon_tags_79 },
    { 0xac40036e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b800371, SUC_lexicon_tags_37 },
    { 0xc9600372, SUC_lexicon_tags_0 },
    { 0x9a000372, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e00376, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a00381, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c00386, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc0038b, SUC_lexicon_tags_26 },
    { 0x9d40038b, SUC_lexicon_tags_12 },
    { 0x8920038d, SUC_lexicon_tags_7 },
    { 0x40a0038e, SUC_lexicon_tags_12 },
    { 0x5480038e, SUC_lexicon_tags_74 },
    { 0x49600390, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8400393, SUC_lexicon_tags_2 },
    { 0x7de00394, SUC_lexicon_tags_37 },
    { 0xa3800393, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e00398, SUC_lexicon_tags_7 },
    { 0x41600398, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9160039b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea40039d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac003a9, SUC_lexicon_tags_8 },
    { 0xd70003aa, SUC_lexicon_tags_17 },
    { 0xeb4003ab, SUC_lexicon_tags_37 },
    { 0x58003ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ca003ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x712003b0, SUC_lexicon_tags_9 },
    { 0x836003b1, SUC_lexicon_tags_32 },
    { 0x322003b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6003b7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4f2003b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x294003bb, SUC_lexicon_tags_12 },
    { 0x95a003bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e0003be, SUC_lexicon_tags_37 },
    { 0x2e003bf, SUC_lexicon_tags_20 },
    { 0xfc8003bf, SUC_lexicon_tags_39 },
    { 0xd30003c1, SUC_lexicon_tags_3 },
    { 0x102003c2, SUC_lexicon_tags_37 },
    { 0x33e003c3, SUC_lexicon_tags_9 },
    { 0x436003c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x478003c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6003ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4003ce, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c003d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e003d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4003dc, SUC_lexicon_tags_1 },
    { 0xbca003dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc003e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a003e4, SUC_lexicon_tags_63 },
    { 0x33c003e5, SUC_lexicon_tags_25 },
    { 0x726003e6, SUC_lexicon_tags_45 },
    { 0xcf8003e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4003ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150003ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e003f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39800406, SUC_lexicon_tags_13 },
    { 0x50600407, SUC_lexicon_tags_25 },
    { 0x28200407, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3940040b, SUC_lexicon_tags_27 },
    { 0x3800040c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a0040f, SUC_lexicon_tags_40 },
    { 0xd140040f, SUC_lexicon_tags_7 },
    { 0x1c600411, SUC_lexicon_tags_27 },
    { 0x52800411, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4400414, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c200416, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f000419, SUC_lexicon_tags_37 },
    { 0xfec00419, SUC_lexicon_tags_14 },
    { 0x1c0041a, SUC_lexicon_tags_30 },
    { 0xd600041c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60041f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86800424, SUC_lexicon_tags_10 },
    { 0xe800425, SUC_lexicon_tags_30 },
    { 0xd3200425, SUC_lexicon_tags_11 },
    { 0x5bc00427, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a0042f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa00437, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c40043c, SUC_lexicon_tags_15 },
    { 0x5560043d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x68800440, SUC_lexicon_tags_17 },
    { 0xc6000441, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf5e00443, SUC_lexicon_tags_17 },
    { 0x87a00444, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120044b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb100044d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86400453, SUC_lexicon_tags_2 },
    { 0x66a00453, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xba000457, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe240045b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd60045f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6600463, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b800465, SUC_lexicon_tags_60 },
    { 0xcbc00466, SUC_lexicon_tags_9 },
    { 0x4f200466, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b80046c, SUC_lexicon_tags_113 },
    { 0x4780046d, SUC_lexicon_tags_47 },
    { 0x7280046e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c00472, SUC_lexicon_tags_38 },
    { 0x29200472, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e400477, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1800479, SUC_lexicon_tags_11 },
    { 0x48a00479, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6760047c, SUC_lexicon_tags_11 },
    { 0x5780047d, SUC_lexicon_tags_7 },
    { 0xf0c0047e, SUC_lexicon_tags_39 },
    { 0x68e0047e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46000481, SUC_lexicon_tags_8 },
    { 0x4c00481, SUC_lexicon_tags_10 },
    { 0xf2200483, SUC_lexicon_tags_27 },
    { 0x5fa00484, SUC_lexicon_tags_23 },
    { 0x6600484, SUC_lexicon_tags_29 },
    { 0x1e400484, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e0048b, SUC_lexicon_tags_2 },
    { 0x4180048b, SUC_lexicon_tags_15 },
    { 0xe320048d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7500048f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8800491, SUC_lexicon_tags_33 },
    { 0xcf800491, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58000496, SUC_lexicon_tags_23 },
    { 0x36c00497, SUC_lexicon_tags_29 },
    { 0x12600497, SUC_lexicon_tags_33 },
    { 0x2a400498, SUC_lexicon_tags_37 },
    { 0xbb40049a, SUC_lexicon_tags_29 },
    { 0x2ce0049b, SUC_lexicon_tags_1 },
    { 0xe920049c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da004a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40a004ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660004b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a004b8, SUC_lexicon_tags_60 },
    { 0x5c8004b9, SUC_lexicon_tags_29 },
    { 0xfec004b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c004c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x680004c2, SUC_lexicon_tags_25 },
    { 0xff6004c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666004ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd12004cc, SUC_lexicon_tags_11 },
    { 0x14a004cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x284004d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x90004d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c6004d4, SUC_lexicon_tags_66 },
    { 0x976004d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c004db, SUC_lexicon_tags_8 },
    { 0xe9a004db, SUC_lexicon_tags_15 },
    { 0x516004dc, SUC_lexicon_tags_63 },
    { 0xc32004dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe004e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a004e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2004e8, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766004f0, SUC_lexicon_tags_33 },
    { 0x5ac004f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x670004f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4004f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x58004f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2004fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd400500, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d400504, SUC_lexicon_tags_8 },
    { 0x4da00504, SUC_lexicon_tags_12 },
    { 0x1aa00506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde00509, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5060050b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10a0050d, SUC_lexicon_tags_7 },
    { 0x1800050e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2b000510, SUC_lexicon_tags_12 },
    { 0x95000510, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18600519, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c40051e, SUC_lexicon_tags_17 },
    { 0xcc00051f, SUC_lexicon_tags_29 },
    { 0xc0e0051f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce600522, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee600529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ca0052b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf360052d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a00532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb600536, SUC_lexicon_tags_2 },
    { 0xb7800537, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb3600539, SUC_lexicon_tags_56 },
    { 0x95400539, SUC_lexicon_tags_37 },
    { 0x82e0053b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4800053e, SUC_lexicon_tags_63 },
    { 0x1c0053f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae00542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43800545, SUC_lexicon_tags_14 },
    { 0xe1400546, SUC_lexicon_tags_11 },
    { 0xc7800546, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc0054a, SUC_lexicon_tags_63 },
    { 0x33c0054b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44800554, SUC_lexicon_tags_4 },
    { 0x2b800554, SUC_lexicon_tags_57 },
    { 0xd7a00555, SUC_lexicon_tags_32 },
    { 0x18e00557, SUC_lexicon_tags_11 },
    { 0x54000558, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc0055d, SUC_lexicon_tags_8 },
    { 0x1c80055e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b200563, SUC_lexicon_tags_68 },
    { 0xca200564, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec0056e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10400571, SUC_lexicon_tags_0 },
    { 0xbd600572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a00576, SUC_lexicon_tags_17 },
    { 0xc4800577, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c80057f, SUC_lexicon_tags_15 },
    { 0x78c00580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b400582, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fe00584, SUC_lexicon_tags_25 },
    { 0x42a00585, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c0058b, SUC_lexicon_tags_25 },
    { 0xcac0058c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b80058e, SUC_lexicon_tags_28 },
    { 0x2380058f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ce00591, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79600593, SUC_lexicon_tags_2 },
    { 0x71a00593, SUC_lexicon_tags_17 },
    { 0xba600595, SUC_lexicon_tags_19 },
    { 0x8e800593, SUC_lexicon_tags_10 },
    { 0xc8a00595, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2540059b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158005a0, SUC_lexicon_tags_8 },
    { 0xe00005a1, SUC_lexicon_tags_63 },
    { 0x6a4005a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34005a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48005aa, SUC_lexicon_tags_33 },
    { 0x1be005aa, SUC_lexicon_tags_17 },
    { 0xe7e005ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x722005af, SUC_lexicon_tags_1 },
    { 0xd14005b0, SUC_lexicon_tags_37 },
    { 0x526005b1, SUC_lexicon_tags_485 },
    { 0x98e005b1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc005b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f2005b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560005c1, SUC_lexicon_tags_37 },
    { 0x838005c2, SUC_lexicon_tags_25 },
    { 0xa08005c2, SUC_lexicon_tags_11 },
    { 0x3f6005c3, SUC_lexicon_tags_37 },
    { 0x672005c5, SUC_lexicon_tags_4 },
    { 0x6fc005c6, SUC_lexicon_tags_12 },
    { 0xefc005c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa005ca, SUC_lexicon_tags_38 },
    { 0x790005cb, SUC_lexicon_tags_11 },
    { 0x428005cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7e005ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x408005d0, SUC_lexicon_tags_25 },
    { 0x6c005d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdc005d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4005d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x406005da, SUC_lexicon_tags_30 },
    { 0xc8005db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540005e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2005e6, SUC_lexicon_tags_60 },
    { 0x8e8005e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a005ec, SUC_lexicon_tags_32 },
    { 0xd64005ec, SUC_lexicon_tags_2 },
    { 0x5b4005ee, SUC_lexicon_tags_27 },
    { 0xb9e005ef, SUC_lexicon_tags_21 },
    { 0x96c005f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a005f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68005fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58600600, SUC_lexicon_tags_19 },
    { 0xb6800601, SUC_lexicon_tags_61 },
    { 0x64600601, SUC_lexicon_tags_14 },
    { 0x94600603, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7200606, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0060e, SUC_lexicon_tags_27 },
    { 0xcea0060e, SUC_lexicon_tags_82 },
    { 0x9ee00610, SUC_lexicon_tags_47 },
    { 0x19c00611, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec00614, SUC_lexicon_tags_17 },
    { 0xbf600615, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea00619, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5760061b, SUC_lexicon_tags_37 },
    { 0xf260061c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1f00061e, SUC_lexicon_tags_47 },
    { 0xda40061f, SUC_lexicon_tags_12 },
    { 0x1a40061e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a00625, SUC_lexicon_tags_2 },
    { 0xca200626, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6400629, SUC_lexicon_tags_12 },
    { 0xb100062a, SUC_lexicon_tags_26 },
    { 0xc500062b, SUC_lexicon_tags_35 },
    { 0x6080062c, SUC_lexicon_tags_15 },
    { 0xe520062c, SUC_lexicon_tags_87 },
    { 0xace0062e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a800630, SUC_lexicon_tags_38 },
    { 0x1a600631, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6800636, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c0063a, SUC_lexicon_tags_37 },
    { 0xe360063b, SUC_lexicon_tags_1 },
    { 0xc820063a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac400641, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a00645, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0064d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4200650, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x42400652, SUC_lexicon_tags_23 },
    { 0x48200653, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c00658, SUC_lexicon_tags_27 },
    { 0xe7200659, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e0065d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee00665, SUC_lexicon_tags_7 },
    { 0x80800665, SUC_lexicon_tags_33 },
    { 0x1fa00666, SUC_lexicon_tags_47 },
    { 0x7c000668, SUC_lexicon_tags_15 },
    { 0x2c400669, SUC_lexicon_tags_30 },
    { 0xdf600668, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe280066c, SUC_lexicon_tags_31 },
    { 0xab00066c, SUC_lexicon_tags_11 },
    { 0xf2e0066e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd200674, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7fa00676, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac0067a, SUC_lexicon_tags_11 },
    { 0xae40067a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62800682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc00684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4000687, SUC_lexicon_tags_14 },
    { 0x24800688, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a00698, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb80069d, SUC_lexicon_tags_8 },
    { 0x8c20069e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcc4006a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e006a6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8006b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4006b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39a006b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0006be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69a006c0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x22006c2, SUC_lexicon_tags_63 },
    { 0xd84006c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0006c6, SUC_lexicon_tags_2 },
    { 0x914006c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4006ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e006d1, SUC_lexicon_tags_30 },
    { 0x116006d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf60006d4, SUC_lexicon_tags_2 },
    { 0x32006d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8006dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef2006de, SUC_lexicon_tags_25 },
    { 0x11e006df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508006e3, SUC_lexicon_tags_114 },
    { 0x5f8006e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa22006e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50006ec, SUC_lexicon_tags_1 },
    { 0x458006ed, SUC_lexicon_tags_2 },
    { 0x6a4006ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x664006f0, SUC_lexicon_tags_14 },
    { 0x8a006f1, SUC_lexicon_tags_79 },
    { 0xae2006f0, SUC_lexicon_tags_37 },
    { 0xc3a006f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x74c006f5, SUC_lexicon_tags_8 },
    { 0xd96006f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a006fe, SUC_lexicon_tags_40 },
    { 0xb5e006ff, SUC_lexicon_tags_37 },
    { 0x598006ff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c00705, SUC_lexicon_tags_17 },
    { 0x3a200705, SUC_lexicon_tags_28 },
    { 0x1c200707, SUC_lexicon_tags_85 },
    { 0xf8000707, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc0070b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba00714, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c00718, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5160071b, SUC_lexicon_tags_14 },
    { 0x86a0071c, SUC_lexicon_tags_17 },
    { 0xd780071d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e00721, SUC_lexicon_tags_38 },
    { 0xe2200721, SUC_lexicon_tags_49 },
    { 0x5fe00723, SUC_lexicon_tags_31 },
    { 0x26c00724, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1d200726, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a00739, SUC_lexicon_tags_2 },
    { 0x8740073a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc7e0073c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5200742, SUC_lexicon_tags_46 },
    { 0x4bc00743, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc0074a, SUC_lexicon_tags_7 },
    { 0xfd60074b, SUC_lexicon_tags_2 },
    { 0x7c0074a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd640074f, SUC_lexicon_tags_17 },
    { 0x27800750, SUC_lexicon_tags_25 },
    { 0xe0200751, SUC_lexicon_tags_27 },
    { 0x2de00751, SUC_lexicon_tags_24 },
    { 0x57400753, SUC_lexicon_tags_25 },
    { 0x45200754, SUC_lexicon_tags_60 },
    { 0x18200754, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x13600757, SUC_lexicon_tags_33 },
    { 0x71200757, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d20075b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e80075e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe5000760, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e00765, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a0076a, SUC_lexicon_tags_60 },
    { 0x1d00076a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f000775, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1600778, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd400077a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7200781, SUC_lexicon_tags_37 },
    { 0x7ec00782, SUC_lexicon_tags_1 },
    { 0x8e600783, SUC_lexicon_tags_37 },
    { 0xc1c00784, SUC_lexicon_tags_81 },
    { 0xbac00784, SUC_lexicon_tags_15 },
    { 0x50600786, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x94400788, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5980078c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a00796, SUC_lexicon_tags_11 },
    { 0x83e00797, SUC_lexicon_tags_6 },
    { 0xb2800798, SUC_lexicon_tags_68 },
    { 0x9a000797, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a0079f, SUC_lexicon_tags_61 },
    { 0x14c007a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62007a4, SUC_lexicon_tags_7 },
    { 0x768007a5, SUC_lexicon_tags_10 },
    { 0xb4007a6, SUC_lexicon_tags_18 },
    { 0x2e007a7, SUC_lexicon_tags_26 },
    { 0x50e007a6, SUC_lexicon_tags_15 },
    { 0x72e007a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x192007ab, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce007ae, SUC_lexicon_tags_8 },
    { 0xac6007af, SUC_lexicon_tags_11 },
    { 0x654007b0, SUC_lexicon_tags_1 },
    { 0x5fc007b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x726007b3, SUC_lexicon_tags_12 },
    { 0xf96007b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdca007b6, SUC_lexicon_tags_31 },
    { 0x4c007b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x818007ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b6007bc, SUC_lexicon_tags_38 },
    { 0x1e2007bd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6007c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650007c6, SUC_lexicon_tags_15 },
    { 0x500007c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e007cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80007d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92007d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d6007d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x128007db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd26007dd, SUC_lexicon_tags_1 },
    { 0xc0c007de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84007e5, SUC_lexicon_tags_12 },
    { 0x2ce007e6, SUC_lexicon_tags_9 },
    { 0xb5e007e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a007eb, SUC_lexicon_tags_53 },
    { 0xe8c007ec, SUC_lexicon_tags_32 },
    { 0xbda007ed, SUC_lexicon_tags_55 },
    { 0x2e8007ec, SUC_lexicon_tags_45 },
    { 0xc0007ee, SUC_lexicon_tags_77 },
    { 0x816007f0, SUC_lexicon_tags_11 },
    { 0x2b2007ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa20007f3, SUC_lexicon_tags_15 },
    { 0x4ca007f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc36007f6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c007f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5d2007fb, SUC_lexicon_tags_7 },
    { 0x5ce007fc, SUC_lexicon_tags_14 },
    { 0xe1e007fd, SUC_lexicon_tags_10 },
    { 0x2c2007fe, SUC_lexicon_tags_9 },
    { 0xe5c007fe, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45000803, SUC_lexicon_tags_17 },
    { 0x91200803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c00807, SUC_lexicon_tags_15 },
    { 0xf2400808, SUC_lexicon_tags_33 },
    { 0x8f000807, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf220080b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce40080d, SUC_lexicon_tags_27 },
    { 0x4cc0080d, SUC_lexicon_tags_19 },
    { 0x22c0080e, SUC_lexicon_tags_29 },
    { 0x7c800810, SUC_lexicon_tags_37 },
    { 0xe3600811, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6200813, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x51400815, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5500081a, SUC_lexicon_tags_38 },
    { 0x8ba0081b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc420081f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9640082b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff60082d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x43c0082f, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x80e00831, SUC_lexicon_tags_12 },
    { 0xf3200832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7800836, SUC_lexicon_tags_0 },
    { 0x23400837, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa0083a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4900083c, SUC_lexicon_tags_57 },
    { 0x28e0083d, SUC_lexicon_tags_37 },
    { 0x6860083e, SUC_lexicon_tags_66 },
    { 0xeb40083f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3de00841, SUC_lexicon_tags_45 },
    { 0xdd400842, SUC_lexicon_tags_2 },
    { 0xd1a00841, SUC_lexicon_tags_11 },
    { 0xd1600843, SUC_lexicon_tags_25 },
    { 0x93c00842, SUC_lexicon_tags_15 },
    { 0x3ea00846, SUC_lexicon_tags_10 },
    { 0x3aa00844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ee00849, SUC_lexicon_tags_8 },
    { 0x8f400849, SUC_lexicon_tags_25 },
    { 0xfe40084b, SUC_lexicon_tags_7 },
    { 0x240084c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a800854, SUC_lexicon_tags_30 },
    { 0x2a000854, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e0085a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e40085c, SUC_lexicon_tags_45 },
    { 0xd980085d, SUC_lexicon_tags_15 },
    { 0x7020085c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x31a00860, SUC_lexicon_tags_16 },
    { 0xc2c00861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92200864, SUC_lexicon_tags_63 },
    { 0x1ca00864, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa00869, SUC_lexicon_tags_17 },
    { 0x51000869, SUC_lexicon_tags_7 },
    { 0x73c0086a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa540086f, SUC_lexicon_tags_1 },
    { 0x63400870, SUC_lexicon_tags_1 },
    { 0x5c600871, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5400874, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x38c00876, SUC_lexicon_tags_47 },
    { 0xd2400876, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x20087a, SUC_lexicon_tags_11 },
    { 0x2040087b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76600880, SUC_lexicon_tags_24 },
    { 0xe1e00880, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b600886, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe840088a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7620088c, SUC_lexicon_tags_4 },
    { 0xeea0088d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2600890, SUC_lexicon_tags_33 },
    { 0xb1800891, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e00894, SUC_lexicon_tags_12 },
    { 0xeb400895, SUC_lexicon_tags_37 },
    { 0xc200896, SUC_lexicon_tags_37 },
    { 0x26600896, SUC_lexicon_tags_45 },
    { 0x8a800895, SUC_lexicon_tags_28 },
    { 0x20e00899, SUC_lexicon_tags_35 },
    { 0xf900089a, SUC_lexicon_tags_19 },
    { 0x9880089a, SUC_lexicon_tags_7 },
    { 0x8d200899, SUC_lexicon_tags_43 },
    { 0xb500089b, SUC_lexicon_tags_47 },
    { 0x52400895, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c6008a0, SUC_lexicon_tags_37 },
    { 0x78c008a1, SUC_lexicon_tags_8 },
    { 0x102008a1, SUC_lexicon_tags_33 },
    { 0x996008a3, SUC_lexicon_tags_2 },
    { 0x4d0008a4, SUC_lexicon_tags_68 },
    { 0x7ee008a3, SUC_lexicon_tags_33 },
    { 0xd7a008a1, SUC_lexicon_tags_37 },
    { 0x9ca008a3, SUC_lexicon_tags_56 },
    { 0x23c008a7, SUC_lexicon_tags_9 },
    { 0xc6008a6, SUC_lexicon_tags_2 },
    { 0x794008a4, SUC_lexicon_tags_37 },
    { 0x2b6008a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644008b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a008b4, SUC_lexicon_tags_37 },
    { 0x426008b5, SUC_lexicon_tags_17 },
    { 0xefa008b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c008c1, SUC_lexicon_tags_14 },
    { 0x22c008c2, SUC_lexicon_tags_17 },
    { 0xae008c3, SUC_lexicon_tags_25 },
    { 0x94008c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588008cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc54008ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98008d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a008d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2008dc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688008e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910008eb, SUC_lexicon_tags_21 },
    { 0xd58008ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614008f0, SUC_lexicon_tags_25 },
    { 0x67a008f1, SUC_lexicon_tags_41 },
    { 0x1ec008f2, SUC_lexicon_tags_21 },
    { 0xf24008f3, SUC_lexicon_tags_15 },
    { 0x78e008f4, SUC_lexicon_tags_60 },
    { 0xe2008f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bc008f7, SUC_lexicon_tags_19 },
    { 0x5b4008f8, SUC_lexicon_tags_8 },
    { 0xd16008f7, SUC_lexicon_tags_37 },
    { 0xac8008fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a008fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d2008ff, SUC_lexicon_tags_21 },
    { 0x9e0008ff, SUC_lexicon_tags_5 },
    { 0x24e008ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19200906, SUC_lexicon_tags_53 },
    { 0xf0e00906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf400090b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c0090e, SUC_lexicon_tags_43 },
    { 0x85e0090e, SUC_lexicon_tags_37 },
    { 0xc3400910, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80e00912, SUC_lexicon_tags_82 },
    { 0x59a00912, SUC_lexicon_tags_49 },
    { 0xf9a00913, SUC_lexicon_tags_47 },
    { 0x37a00915, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9dc00917, SUC_lexicon_tags_29 },
    { 0xf4a00918, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbc0091a, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xd520091d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc940091f, SUC_lexicon_tags_17 },
    { 0x2c400920, SUC_lexicon_tags_37 },
    { 0x3aa00921, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7400924, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9400926, SUC_lexicon_tags_22 },
    { 0x80000927, SUC_lexicon_tags_43 },
    { 0xd7800926, SUC_lexicon_tags_12 },
    { 0xe6400927, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb960092e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf400930, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a00933, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48a00935, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81a00937, SUC_lexicon_tags_15 },
    { 0x83800938, SUC_lexicon_tags_15 },
    { 0x54a00937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7600944, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf60094e, SUC_lexicon_tags_57 },
    { 0xc7e0094e, SUC_lexicon_tags_6 },
    { 0xfdc0094f, SUC_lexicon_tags_7 },
    { 0x7aa00951, SUC_lexicon_tags_7 },
    { 0x6c00094e, SUC_lexicon_tags_38 },
    { 0xcc00953, SUC_lexicon_tags_43 },
    { 0x3f600951, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10800956, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b00095a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2220095c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5000960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28000962, SUC_lexicon_tags_47 },
    { 0x54600963, SUC_lexicon_tags_12 },
    { 0xb1c00964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3400967, SUC_lexicon_tags_68 },
    { 0x73a00968, SUC_lexicon_tags_20 },
    { 0xee000969, SUC_lexicon_tags_15 },
    { 0x20096a, SUC_lexicon_tags_33 },
    { 0x9d60096b, SUC_lexicon_tags_15 },
    { 0x1260096c, SUC_lexicon_tags_17 },
    { 0x53c0096c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a00970, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x79400973, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb800977, SUC_lexicon_tags_17 },
    { 0x8e000978, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1220097a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc020097c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c0097f, SUC_lexicon_tags_30 },
    { 0xb6e00980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0200985, SUC_lexicon_tags_7 },
    { 0x20400986, SUC_lexicon_tags_2 },
    { 0x72600986, SUC_lexicon_tags_12 },
    { 0xcb000985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa600098a, SUC_lexicon_tags_16 },
    { 0xc4a0098b, SUC_lexicon_tags_60 },
    { 0xd000098b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6600993, SUC_lexicon_tags_39 },
    { 0xcb000994, SUC_lexicon_tags_47 },
    { 0x88a00993, SUC_lexicon_tags_15 },
    { 0xafc00994, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6200998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf60099b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e009a0, SUC_lexicon_tags_38 },
    { 0x1da009a0, SUC_lexicon_tags_48 },
    { 0x808009a2, SUC_lexicon_tags_25 },
    { 0xeb8009a2, SUC_lexicon_tags_16 },
    { 0x2fa009a0, SUC_lexicon_tags_16 },
    { 0xaa8009a5, SUC_lexicon_tags_25 },
    { 0xbb6009a5, SUC_lexicon_tags_39 },
    { 0x9dc009a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x286009a9, SUC_lexicon_tags_60 },
    { 0xf2e009aa, SUC_lexicon_tags_15 },
    { 0xd4c009aa, SUC_lexicon_tags_2 },
    { 0x50009a9, SUC_lexicon_tags_0 },
    { 0x76e009ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x548009af, SUC_lexicon_tags_0 },
    { 0xb68009b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae009b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1ae009b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d4009b9, SUC_lexicon_tags_77 },
    { 0xf3c009b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ea009bc, SUC_lexicon_tags_70 },
    { 0x590009bd, SUC_lexicon_tags_17 },
    { 0x594009bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc66009c0, SUC_lexicon_tags_1 },
    { 0x9e6009c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e009c4, SUC_lexicon_tags_37 },
    { 0x76c009c5, SUC_lexicon_tags_2 },
    { 0xe24009c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66e009c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60009cb, SUC_lexicon_tags_30 },
    { 0x852009cc, SUC_lexicon_tags_11 },
    { 0xa10009cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2dc009cf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e009d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6009d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc009dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c009e4, SUC_lexicon_tags_7 },
    { 0x100009e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea009e9, SUC_lexicon_tags_74 },
    { 0xa62009ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x498009ed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf84009ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e009f4, SUC_lexicon_tags_60 },
    { 0xce4009f5, SUC_lexicon_tags_25 },
    { 0xffe009f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x562009f8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6009fb, SUC_lexicon_tags_11 },
    { 0x1a2009fc, SUC_lexicon_tags_37 },
    { 0xe5e009fb, SUC_lexicon_tags_83 },
    { 0xba4009fe, SUC_lexicon_tags_26 },
    { 0x892009ff, SUC_lexicon_tags_27 },
    { 0x20e00a00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5800a02, SUC_lexicon_tags_2 },
    { 0xf000a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce00a09, SUC_lexicon_tags_25 },
    { 0x91200a09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78c00a0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c00a0f, SUC_lexicon_tags_34 },
    { 0xc9a00a0f, SUC_lexicon_tags_16 },
    { 0x6de00a11, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3600a15, SUC_lexicon_tags_29 },
    { 0xd0400a16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98600a1b, SUC_lexicon_tags_25 },
    { 0xbc400a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41200a21, SUC_lexicon_tags_46 },
    { 0xace00a22, SUC_lexicon_tags_82 },
    { 0xda00a23, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x63000a25, SUC_lexicon_tags_50 },
    { 0xaac00a26, SUC_lexicon_tags_13 },
    { 0xb7600a26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f400a33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c00a37, SUC_lexicon_tags_27 },
    { 0x20600a37, SUC_lexicon_tags_77 },
    { 0xbd600a39, SUC_lexicon_tags_25 },
    { 0x6f400a38, SUC_lexicon_tags_14 },
    { 0x55c00a3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62800a3d, SUC_lexicon_tags_26 },
    { 0x3f200a3d, SUC_lexicon_tags_39 },
    { 0x8c000a3f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c00a45, SUC_lexicon_tags_26 },
    { 0x30400a46, SUC_lexicon_tags_57 },
    { 0x5cc00a47, SUC_lexicon_tags_15 },
    { 0xad800a48, SUC_lexicon_tags_0 },
    { 0xf4200a45, SUC_lexicon_tags_3 },
    { 0x50c00a49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef600a4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c00a51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83400a55, SUC_lexicon_tags_12 },
    { 0xf3c00a56, SUC_lexicon_tags_22 },
    { 0xb7a00a57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8c00a59, SUC_lexicon_tags_46 },
    { 0x55e00a5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a00a60, SUC_lexicon_tags_48 },
    { 0xf8a00a61, SUC_lexicon_tags_25 },
    { 0x53c00a61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ac00a64, SUC_lexicon_tags_23 },
    { 0xd1600a65, SUC_lexicon_tags_17 },
    { 0x2b800a66, SUC_lexicon_tags_46 },
    { 0x10c00a66, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5000a6c, SUC_lexicon_tags_1 },
    { 0xad800a6d, SUC_lexicon_tags_15 },
    { 0x58400a6c, SUC_lexicon_tags_12 },
    { 0x6a600a6f, SUC_lexicon_tags_13 },
    { 0xfd400a70, SUC_lexicon_tags_1 },
    { 0x8da00a71, SUC_lexicon_tags_0 },
    { 0xff000a71, SUC_lexicon_tags_43 },
    { 0x89800a73, SUC_lexicon_tags_1 },
    { 0x51c00a73, SUC_lexicon_tags_23 },
    { 0x20800a6d, SUC_lexicon_tags_63 },
    { 0xdfa00a6c, SUC_lexicon_tags_7 },
    { 0xcbc00a72, SUC_lexicon_tags_7 },
    { 0xef600a78, SUC_lexicon_tags_10 },
    { 0xe8600a79, SUC_lexicon_tags_2 },
    { 0x4f400a7a, SUC_lexicon_tags_2 },
    { 0xeaa00a7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43200a7d, SUC_lexicon_tags_19 },
    { 0x29c00a7e, SUC_lexicon_tags_27 },
    { 0x98600a7f, SUC_lexicon_tags_12 },
    { 0x60200a7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a00a85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88200a89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc600a8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c800a91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32600a94, SUC_lexicon_tags_0 },
    { 0xd7a00a94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad400a98, SUC_lexicon_tags_28 },
    { 0x40800a98, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8600a9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c00aa1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad400aa6, SUC_lexicon_tags_2 },
    { 0xfa400aa7, SUC_lexicon_tags_1 },
    { 0x2200aa6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a00aab, SUC_lexicon_tags_25 },
    { 0xa5e00aac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe3800aae, SUC_lexicon_tags_2 },
    { 0x14600aae, SUC_lexicon_tags_84 },
    { 0x80000aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76400ab3, SUC_lexicon_tags_17 },
    { 0x79e00ab3, SUC_lexicon_tags_8 },
    { 0x81000ab5, SUC_lexicon_tags_48 },
    { 0xcea00ab6, SUC_lexicon_tags_37 },
    { 0x2200ab7, SUC_lexicon_tags_9 },
    { 0x71c00ab5, SUC_lexicon_tags_95 },
    { 0x84600ab7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7800abe, SUC_lexicon_tags_1 },
    { 0x12600abf, SUC_lexicon_tags_43 },
    { 0xa3400ac0, SUC_lexicon_tags_17 },
    { 0x3b400ac0, SUC_lexicon_tags_272 },
    { 0xb6400ac1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe00ac8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc400acc, SUC_lexicon_tags_15 },
    { 0xed200acd, SUC_lexicon_tags_16 },
    { 0x88e00ace, SUC_lexicon_tags_33 },
    { 0x8b800acf, SUC_lexicon_tags_15 },
    { 0x4b800ace, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a600ad8, SUC_lexicon_tags_9 },
    { 0x42000ad9, SUC_lexicon_tags_2 },
    { 0x66e00ada, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3ae00adc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ae00ade, SUC_lexicon_tags_37 },
    { 0xd8000adf, SUC_lexicon_tags_216 },
    { 0x34a00ae0, SUC_lexicon_tags_40 },
    { 0xa1800ae1, SUC_lexicon_tags_49 },
    { 0x41400ae1, SUC_lexicon_tags_2 },
    { 0x4f000ae3, SUC_lexicon_tags_49 },
    { 0xc4e00ae3, SUC_lexicon_tags_1 },
    { 0x72c00ae5, SUC_lexicon_tags_8 },
    { 0x5a800ae1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a200ae9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd800aec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28400aee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6000af3, SUC_lexicon_tags_17 },
    { 0x12000af4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5e00af6, SUC_lexicon_tags_11 },
    { 0xb1c00af7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa00afc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xade00afe, SUC_lexicon_tags_27 },
    { 0x5d200aff, SUC_lexicon_tags_6 },
    { 0xf6200b00, SUC_lexicon_tags_296 },
    { 0x6de00b01, SUC_lexicon_tags_12 },
    { 0xe7000b02, SUC_lexicon_tags_17 },
    { 0x98e00b00, SUC_lexicon_tags_0 },
    { 0x5ec00b04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2200b0a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35200b0f, SUC_lexicon_tags_12 },
    { 0x75a00b0f, SUC_lexicon_tags_8 },
    { 0x5e200b11, SUC_lexicon_tags_7 },
    { 0xcd000b12, SUC_lexicon_tags_11 },
    { 0x4d200b11, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x43000b15, SUC_lexicon_tags_38 },
    { 0x6ec00b16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c800b19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c400b1c, SUC_lexicon_tags_12 },
    { 0x99000b1c, SUC_lexicon_tags_17 },
    { 0x36000b1c, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x27200b20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bc00b22, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b600b25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97200b2a, SUC_lexicon_tags_25 },
    { 0x4a800b2a, SUC_lexicon_tags_11 },
    { 0x7a200b2a, SUC_lexicon_tags_15 },
    { 0xb6600b2c, SUC_lexicon_tags_60 },
    { 0xeb400b2e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e800b31, SUC_lexicon_tags_25 },
    { 0x56200b32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a00b35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1600b37, SUC_lexicon_tags_35 },
    { 0x5ea00b37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42400b3d, SUC_lexicon_tags_28 },
    { 0x82800b3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70400b41, SUC_lexicon_tags_1 },
    { 0x74c00b41, SUC_lexicon_tags_25 },
    { 0xdde00b43, SUC_lexicon_tags_60 },
    { 0xf9800b43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37600b47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5800b4b, SUC_lexicon_tags_27 },
    { 0x9000b4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88400b4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a00b58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0400b5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e00b63, SUC_lexicon_tags_25 },
    { 0xc7a00b64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8ba00b66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65a00b68, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa5e00b6a, SUC_lexicon_tags_17 },
    { 0x71600b6a, SUC_lexicon_tags_37 },
    { 0x45e00b6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a800b6e, SUC_lexicon_tags_27 },
    { 0xe0c00b6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93000b73, SUC_lexicon_tags_2 },
    { 0x76200b73, SUC_lexicon_tags_11 },
    { 0x22000b73, SUC_lexicon_tags_25 },
    { 0xdd200b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc00b7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x38200b7d, SUC_lexicon_tags_25 },
    { 0x6ce00b7e, SUC_lexicon_tags_87 },
    { 0x21200b7e, SUC_lexicon_tags_14 },
    { 0xf9a00b80, SUC_lexicon_tags_17 },
    { 0x8b400b7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1200b84, SUC_lexicon_tags_17 },
    { 0xa7e00b85, SUC_lexicon_tags_60 },
    { 0xafc00b85, SUC_lexicon_tags_25 },
    { 0x6a000b87, SUC_lexicon_tags_15 },
    { 0x2a200b88, SUC_lexicon_tags_6 },
    { 0x3c800b88, SUC_lexicon_tags_25 },
    { 0xf7a00b8a, SUC_lexicon_tags_8 },
    { 0x4e00b8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37800b94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86400b99, SUC_lexicon_tags_47 },
    { 0xbdc00b9a, SUC_lexicon_tags_11 },
    { 0xcf200b9b, SUC_lexicon_tags_0 },
    { 0xd600b9c, SUC_lexicon_tags_25 },
    { 0xa8000b9c, SUC_lexicon_tags_25 },
    { 0x10400b9e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce00ba5, SUC_lexicon_tags_15 },
    { 0x23800ba6, SUC_lexicon_tags_1 },
    { 0xe1800ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa000bac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e00bb0, SUC_lexicon_tags_66 },
    { 0x38000bb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc00bb4, SUC_lexicon_tags_9 },
    { 0x65a00bb5, SUC_lexicon_tags_31 },
    { 0x92e00bb4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd1c00bb8, SUC_lexicon_tags_45 },
    { 0x6600bb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69400bbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c00bc2, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x58400bc5, SUC_lexicon_tags_30 },
    { 0x3ae00bc6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c800bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5600bce, SUC_lexicon_tags_82 },
    { 0x49800bcf, SUC_lexicon_tags_3 },
    { 0xea00bce, SUC_lexicon_tags_39 },
    { 0x2c400bd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee00be2, SUC_lexicon_tags_12 },
    { 0x3a600be2, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85200bea, SUC_lexicon_tags_8 },
    { 0xb1000beb, SUC_lexicon_tags_7 },
    { 0xc4400bea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x19000bef, SUC_lexicon_tags_27 },
    { 0xa2800bf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71000bf3, SUC_lexicon_tags_38 },
    { 0xc7a00bf3, SUC_lexicon_tags_7 },
    { 0xc5600bf5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94400bfa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a400bff, SUC_lexicon_tags_17 },
    { 0x1ba00c00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96600c05, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b000c0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19c00c10, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b400c14, SUC_lexicon_tags_47 },
    { 0xdfe00c15, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a00c18, SUC_lexicon_tags_45 },
    { 0x6ca00c18, SUC_lexicon_tags_17 },
    { 0xd6c00c18, SUC_lexicon_tags_31 },
    { 0xc0600c1b, SUC_lexicon_tags_70 },
    { 0x70200c1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c000c1e, SUC_lexicon_tags_17 },
    { 0x7f800c1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3800c26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c00c2a, SUC_lexicon_tags_14 },
    { 0xd7200c2a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2400c2e, SUC_lexicon_tags_37 },
    { 0x7d000c2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a00c32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53000c34, SUC_lexicon_tags_1 },
    { 0xaa600c35, SUC_lexicon_tags_25 },
    { 0xe6a00c36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae00c39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c00c3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d200c41, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17000c48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e00c4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd000c50, SUC_lexicon_tags_25 },
    { 0x8e400c51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66000c53, SUC_lexicon_tags_2 },
    { 0x4ea00c54, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e00c57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a800c59, SUC_lexicon_tags_274 },
    { 0x10e00c59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba00c62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e00c66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800c6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf000c72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a00c76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe00c7c, SUC_lexicon_tags_25 },
    { 0x71600c7d, SUC_lexicon_tags_31 },
    { 0xc600c7c, SUC_lexicon_tags_11 },
    { 0xf0400c7d, SUC_lexicon_tags_16 },
    { 0x70600c80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b000c82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a00c87, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xba00c89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e00c8e, SUC_lexicon_tags_19 },
    { 0x6e400c8f, SUC_lexicon_tags_26 },
    { 0x1fe00c90, SUC_lexicon_tags_10 },
    { 0xcc800c91, SUC_lexicon_tags_25 },
    { 0x35e00c92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c00c95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcea00c98, SUC_lexicon_tags_85 },
    { 0xfb000c98, SUC_lexicon_tags_30 },
    { 0xf6800c9a, SUC_lexicon_tags_61 },
    { 0x36400c9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5400c9f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4200ca2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2a00ca4, SUC_lexicon_tags_113 },
    { 0x50200ca4, SUC_lexicon_tags_82 },
    { 0xede00ca4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d400caa, SUC_lexicon_tags_12 },
    { 0x23a00cab, SUC_lexicon_tags_11 },
    { 0x4ae00cac, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x8b000cae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da00cb4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc5000cb6, SUC_lexicon_tags_94 },
    { 0x9ea00cb6, SUC_lexicon_tags_15 },
    { 0xdb600cb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51400cba, SUC_lexicon_tags_43 },
    { 0xe6c00cbb, SUC_lexicon_tags_38 },
    { 0x80a00cbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x73000cbf, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e600cc3, SUC_lexicon_tags_26 },
    { 0x74a00cc4, SUC_lexicon_tags_17 },
    { 0x18a00cc3, SUC_lexicon_tags_1 },
    { 0xfe000cc3, SUC_lexicon_tags_33 },
    { 0xd7200cc7, SUC_lexicon_tags_14 },
    { 0xeca00cc3, SUC_lexicon_tags_23 },
    { 0xd0000cc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a800ccd, SUC_lexicon_tags_37 },
    { 0xea400ccd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d600cd3, SUC_lexicon_tags_11 },
    { 0x6b200cd4, SUC_lexicon_tags_25 },
    { 0x10e00cd4, SUC_lexicon_tags_47 },
    { 0xa7400cd6, SUC_lexicon_tags_27 },
    { 0xbe200cd4, SUC_lexicon_tags_16 },
    { 0xf1e00cd8, SUC_lexicon_tags_11 },
    { 0x6c400cd9, SUC_lexicon_tags_15 },
    { 0x7e600cd6, SUC_lexicon_tags_87 },
    { 0x27c00cd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51a00cdd, SUC_lexicon_tags_13 },
    { 0x79800cde, SUC_lexicon_tags_3 },
    { 0x77c00cdd, SUC_lexicon_tags_14 },
    { 0x5a600ce0, SUC_lexicon_tags_38 },
    { 0xaaa00ce1, SUC_lexicon_tags_2 },
    { 0x1c200ce2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c00ce5, SUC_lexicon_tags_17 },
    { 0x21600ce6, SUC_lexicon_tags_43 },
    { 0x11000ce6, SUC_lexicon_tags_17 },
    { 0xa1a00ce7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8600ceb, SUC_lexicon_tags_7 },
    { 0x17000ceb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4400cef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x66a00cf1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x46200cf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c00cf6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd3600cf8, SUC_lexicon_tags_5 },
    { 0xd3800cf9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5400d00, SUC_lexicon_tags_46 },
    { 0x82a00d01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6600d03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77800d10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88600d12, SUC_lexicon_tags_1 },
    { 0x99800d12, SUC_lexicon_tags_48 },
    { 0x47600d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea00d16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a00d19, SUC_lexicon_tags_2 },
    { 0x53c00d19, SUC_lexicon_tags_11 },
    { 0x3f400d1b, SUC_lexicon_tags_33 },
    { 0xef400d1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40c00d1e, SUC_lexicon_tags_45 },
    { 0x31800d1f, SUC_lexicon_tags_68 },
    { 0x76600d20, SUC_lexicon_tags_26 },
    { 0x5f000d21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c800d26, SUC_lexicon_tags_24 },
    { 0xb600d27, SUC_lexicon_tags_32 },
    { 0xb9a00d26, SUC_lexicon_tags_65 },
    { 0x65200d26, SUC_lexicon_tags_28 },
    { 0x51c00d29, SUC_lexicon_tags_0 },
    { 0x4ae00d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fc00d2d, SUC_lexicon_tags_32 },
    { 0x1ae00d2e, SUC_lexicon_tags_21 },
    { 0xf4800d2f, SUC_lexicon_tags_33 },
    { 0xfca00d2f, SUC_lexicon_tags_15 },
    { 0x40600d31, SUC_lexicon_tags_45 },
    { 0x67800d32, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda800d38, SUC_lexicon_tags_1 },
    { 0xcee00d39, SUC_lexicon_tags_56 },
    { 0x64400d3a, SUC_lexicon_tags_37 },
    { 0x88000d3b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a00d3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a00d43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe7a00d45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41000d49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffc00d4b, SUC_lexicon_tags_60 },
    { 0x26c00d4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4800d50, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x41000d52, SUC_lexicon_tags_4 },
    { 0x44000d53, SUC_lexicon_tags_1 },
    { 0xe0200d54, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc6c00d56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73400d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60600d5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ba00d61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d400d64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44800d66, SUC_lexicon_tags_9 },
    { 0x86c00d67, SUC_lexicon_tags_15 },
    { 0x2c400d66, SUC_lexicon_tags_17 },
    { 0xc3200d68, SUC_lexicon_tags_25 },
    { 0x82000d6a, SUC_lexicon_tags_15 },
    { 0xd0a00d6a, SUC_lexicon_tags_31 },
    { 0x99e00d6c, SUC_lexicon_tags_47 },
    { 0xdb600d6a, SUC_lexicon_tags_39 },
    { 0x5fa00d6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c600d7d, SUC_lexicon_tags_30 },
    { 0x61000d7e, SUC_lexicon_tags_12 },
    { 0x80200d7e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4400d84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d800d87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c00d8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a00d90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c00d94, SUC_lexicon_tags_4 },
    { 0xce400d95, SUC_lexicon_tags_38 },
    { 0x25c00d94, SUC_lexicon_tags_14 },
    { 0x54200d97, SUC_lexicon_tags_47 },
    { 0xc8600d98, SUC_lexicon_tags_45 },
    { 0xdaa00d99, SUC_lexicon_tags_22 },
    { 0x53600d95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69600d9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e00da6, SUC_lexicon_tags_26 },
    { 0x4400da7, SUC_lexicon_tags_7 },
    { 0x39600da6, SUC_lexicon_tags_37 },
    { 0xbae00da9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37e00dac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f000daf, SUC_lexicon_tags_15 },
    { 0xfa000db0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93600db2, SUC_lexicon_tags_7 },
    { 0x15c00db3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe00db7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9a00db9, SUC_lexicon_tags_17 },
    { 0x14400db9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca00dbe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e00dc1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27400dc5, SUC_lexicon_tags_47 },
    { 0xd5600dc6, SUC_lexicon_tags_17 },
    { 0xfe00dc5, SUC_lexicon_tags_17 },
    { 0x5a000dc8, SUC_lexicon_tags_68 },
    { 0xc0200dc5, SUC_lexicon_tags_12 },
    { 0x54800dca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xba200dcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2200dcf, SUC_lexicon_tags_60 },
    { 0xdfa00dd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5400dd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca00dda, SUC_lexicon_tags_7 },
    { 0x28200dda, SUC_lexicon_tags_7 },
    { 0xb2a00ddc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1e600dde, SUC_lexicon_tags_9 },
    { 0xd000ddf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f200de5, SUC_lexicon_tags_12 },
    { 0x96200de6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbf400de8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2600dea, SUC_lexicon_tags_8 },
    { 0x43000dea, SUC_lexicon_tags_63 },
    { 0x65a00deb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb200dee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11600df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7000df6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa00df9, SUC_lexicon_tags_25 },
    { 0xea400df9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f600dfe, SUC_lexicon_tags_7 },
    { 0x57c00dff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5000e02, SUC_lexicon_tags_37 },
    { 0xb8400e02, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x55200e06, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0x36e00e08, SUC_lexicon_tags_27 },
    { 0xcca00e09, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3200e0d, SUC_lexicon_tags_11 },
    { 0x34e00e0e, SUC_lexicon_tags_33 },
    { 0xffe00e0f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe2200e11, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75200e17, SUC_lexicon_tags_6 },
    { 0x2d000e17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5600e20, SUC_lexicon_tags_13 },
    { 0xd7800e20, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa00e25, SUC_lexicon_tags_7 },
    { 0x21600e26, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5000e2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61c00e30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c00e33, SUC_lexicon_tags_38 },
    { 0xa8400e33, SUC_lexicon_tags_16 },
    { 0xc00e34, SUC_lexicon_tags_17 },
    { 0xca600e36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88400e3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5800e3c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c00e3f, SUC_lexicon_tags_15 },
    { 0xd0e00e40, SUC_lexicon_tags_33 },
    { 0xbe400e41, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a00e44, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a00e48, SUC_lexicon_tags_5 },
    { 0x8da00e49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37c00e4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae00e4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa4c00e51, SUC_lexicon_tags_27 },
    { 0x75a00e51, SUC_lexicon_tags_74 },
    { 0x7b800e53, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3ce00e55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a000e5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfa00e5e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d000e62, SUC_lexicon_tags_66 },
    { 0xac00e62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b000e66, SUC_lexicon_tags_160 },
    { 0x14e00e67, SUC_lexicon_tags_12 },
    { 0x16400e67, SUC_lexicon_tags_27 },
    { 0x1de00e69, SUC_lexicon_tags_15 },
    { 0xd6400e6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c00e6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcac00e70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2800e72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42800e77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14400e7d, SUC_lexicon_tags_0 },
    { 0xc9800e7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec00e81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c00e83, SUC_lexicon_tags_12 },
    { 0xc7400e83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x29400e86, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0x7a200e88, SUC_lexicon_tags_60 },
    { 0x1d200e88, SUC_lexicon_tags_63 },
    { 0x47000e89, SUC_lexicon_tags_37 },
    { 0x5d200e8b, SUC_lexicon_tags_25 },
    { 0xb8e00e8c, SUC_lexicon_tags_47 },
    { 0x16400e8b, SUC_lexicon_tags_1 },
    { 0xf5c00e8e, SUC_lexicon_tags_1 },
    { 0x70e00e8e, SUC_lexicon_tags_17 },
    { 0x6800e8e, SUC_lexicon_tags_17 },
    { 0x2f800e8c, SUC_lexicon_tags_37 },
    { 0x4cc00e90, SUC_lexicon_tags_15 },
    { 0x73600e93, SUC_lexicon_tags_0 },
    { 0xc3600e94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x93000e97, SUC_lexicon_tags_65 },
    { 0xd8800e97, SUC_lexicon_tags_68 },
    { 0xac600e99, SUC_lexicon_tags_65 },
    { 0xf4c00e9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92c00e9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae00ea2, SUC_lexicon_tags_7 },
    { 0xfc000ea3, SUC_lexicon_tags_19 },
    { 0xb4c00ea3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0800ea9, SUC_lexicon_tags_60 },
    { 0x89600eaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b000eaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da00eb3, SUC_lexicon_tags_25 },
    { 0x84c00eb3, SUC_lexicon_tags_1 },
    { 0x71200eb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0a00eb7, SUC_lexicon_tags_8 },
    { 0x2aa00eb7, SUC_lexicon_tags_65 },
    { 0xa7400eb7, SUC_lexicon_tags_29 },
    { 0xc7400eba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7800ebc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c00ec0, SUC_lexicon_tags_25 },
    { 0xbba00ec1, SUC_lexicon_tags_47 },
    { 0xc2a00ec1, SUC_lexicon_tags_33 },
    { 0xc0a00ec3, SUC_lexicon_tags_23 },
    { 0x6ec00ec0, SUC_lexicon_tags_31 },
    { 0x1200ec5, SUC_lexicon_tags_15 },
    { 0x200ec0, SUC_lexicon_tags_38 },
    { 0xfa800ec7, SUC_lexicon_tags_23 },
    { 0xbcc00ec7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a800ecc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x83800ece, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae000ed0, SUC_lexicon_tags_17 },
    { 0xa4400ed1, SUC_lexicon_tags_11 },
    { 0x35200ed2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x41000ed5, SUC_lexicon_tags_29 },
    { 0xd3e00ed6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4000eda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58800edd, SUC_lexicon_tags_15 },
    { 0x7a00ede, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70600ee0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa7a00ee2, SUC_lexicon_tags_11 },
    { 0xddc00ee3, SUC_lexicon_tags_2 },
    { 0xfe800ee4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf800ee7, SUC_lexicon_tags_91 },
    { 0xcae00ee7, SUC_lexicon_tags_65 },
    { 0xcc000ee9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e800eef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2000ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89400ef6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71200efb, SUC_lexicon_tags_28 },
    { 0xc8200efc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a00f00, SUC_lexicon_tags_16 },
    { 0xb0400f01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x24800f03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1200f05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e600f0b, SUC_lexicon_tags_2 },
    { 0x49600f0c, SUC_lexicon_tags_4 },
    { 0x7aa00f0d, SUC_lexicon_tags_87 },
    { 0x7e600f0d, SUC_lexicon_tags_12 },
    { 0xb600f0f, SUC_lexicon_tags_17 },
    { 0x3ec00f10, SUC_lexicon_tags_1 },
    { 0xbb800f11, SUC_lexicon_tags_7 },
    { 0xcaa00f12, SUC_lexicon_tags_29 },
    { 0x23800f0e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12800f1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ec00f1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7fc00f1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a600f24, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac00f26, SUC_lexicon_tags_15 },
    { 0xa8600f26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3400f2c, SUC_lexicon_tags_25 },
    { 0xfb600f2d, SUC_lexicon_tags_15 },
    { 0x5e800f2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a00f3b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x1ee00f3d, SUC_lexicon_tags_25 },
    { 0x7c400f3d, SUC_lexicon_tags_299 },
    { 0x1aa00f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51400f44, SUC_lexicon_tags_15 },
    { 0xb3200f45, SUC_lexicon_tags_23 },
    { 0xec000f46, SUC_lexicon_tags_25 },
    { 0x7d800f44, SUC_lexicon_tags_33 },
    { 0x5aa00f48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x43200f4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6200f4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8000f51, SUC_lexicon_tags_15 },
    { 0x12600f52, SUC_lexicon_tags_30 },
    { 0xca000f51, SUC_lexicon_tags_11 },
    { 0xbb200f53, SUC_lexicon_tags_23 },
    { 0x7b200f53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51000f5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52800f63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1e00f65, SUC_lexicon_tags_0 },
    { 0xa9800f66, SUC_lexicon_tags_14 },
    { 0x95c00f67, SUC_lexicon_tags_37 },
    { 0x22000f68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69800f6e, SUC_lexicon_tags_1 },
    { 0x5c200f6f, SUC_lexicon_tags_17 },
    { 0xa9600f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83800f73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e00f7b, SUC_lexicon_tags_46 },
    { 0xad200f7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8000f81, SUC_lexicon_tags_6 },
    { 0x99000f82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda600f86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f000f89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c00f8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a00f94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca00f9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19a00f9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97000f9e, SUC_lexicon_tags_15 },
    { 0xd5200f9e, SUC_lexicon_tags_17 },
    { 0xb3600f9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c600fa2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56200fa9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x36200fac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3800fae, SUC_lexicon_tags_60 },
    { 0x7c400faf, SUC_lexicon_tags_16 },
    { 0xdde00fb0, SUC_lexicon_tags_43 },
    { 0xd7200fae, SUC_lexicon_tags_31 },
    { 0x81e00faf, SUC_lexicon_tags_38 },
    { 0xbc00fb0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x18400fb5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4a00fb7, SUC_lexicon_tags_37 },
    { 0xc8c00fb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94400fba, SUC_lexicon_tags_16 },
    { 0x98200fba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c00fc0, SUC_lexicon_tags_12 },
    { 0xa5400fc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8800fc3, SUC_lexicon_tags_14 },
    { 0xe1e00fc4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0600fc9, SUC_lexicon_tags_7 },
    { 0x61800fca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe00fd0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59400fd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76a00fd9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e00fdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe000fe1, SUC_lexicon_tags_56 },
    { 0x51600fe1, SUC_lexicon_tags_25 },
    { 0x44600fe3, SUC_lexicon_tags_10 },
    { 0x1c400fe3, SUC_lexicon_tags_43 },
    { 0x98400fe1, SUC_lexicon_tags_25 },
    { 0xba600fe6, SUC_lexicon_tags_33 },
    { 0x6e600fe6, SUC_lexicon_tags_37 },
    { 0xad800fe8, SUC_lexicon_tags_28 },
    { 0xdba00fe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e000feb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c600ff0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc00ff4, SUC_lexicon_tags_7 },
    { 0x7de00ff4, SUC_lexicon_tags_4 },
    { 0xde00ff5, SUC_lexicon_tags_27 },
    { 0x9f600ff7, SUC_lexicon_tags_1 },
    { 0x9dc00ff4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79400ffc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87001007, SUC_lexicon_tags_7 },
    { 0x4c201008, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4360100a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcc80100c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc801010, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6601017, SUC_lexicon_tags_26 },
    { 0x5201018, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc0101e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x32801021, SUC_lexicon_tags_15 },
    { 0xdee01021, SUC_lexicon_tags_29 },
    { 0x27401023, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbfa01025, SUC_lexicon_tags_37 },
    { 0xf801026, SUC_lexicon_tags_43 },
    { 0xa1001027, SUC_lexicon_tags_0 },
    { 0x97e01028, SUC_lexicon_tags_158 },
    { 0x36201027, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d20102d, SUC_lexicon_tags_39 },
    { 0xed40102e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce201030, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59801032, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e01037, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c401039, SUC_lexicon_tags_19 },
    { 0x43c0103a, SUC_lexicon_tags_31 },
    { 0x6de0103b, SUC_lexicon_tags_17 },
    { 0x2ce0103c, SUC_lexicon_tags_54 },
    { 0xdd40103d, SUC_lexicon_tags_15 },
    { 0x2ee0103d, SUC_lexicon_tags_17 },
    { 0xc840103e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c401042, SUC_lexicon_tags_11 },
    { 0x3ba01043, SUC_lexicon_tags_37 },
    { 0x27201044, SUC_lexicon_tags_35 },
    { 0x7aa01044, SUC_lexicon_tags_7 },
    { 0x51201045, SUC_lexicon_tags_38 },
    { 0xc3c01047, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60e01049, SUC_lexicon_tags_39 },
    { 0xdaa0104a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbdc0104c, SUC_lexicon_tags_60 },
    { 0x5f80104c, SUC_lexicon_tags_27 },
    { 0x78c0104c, SUC_lexicon_tags_18 },
    { 0x32c0104f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7201054, SUC_lexicon_tags_49 },
    { 0x8fe01055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43401058, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x8240105a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf480105c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3001060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52e01062, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3560106c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc560106f, SUC_lexicon_tags_2 },
    { 0xd0801070, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe01073, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce01076, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0001079, SUC_lexicon_tags_63 },
    { 0x3aa0107a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28c0107c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde01086, SUC_lexicon_tags_1 },
    { 0x9b401086, SUC_lexicon_tags_6 },
    { 0x3a601087, SUC_lexicon_tags_23 },
    { 0xca201089, SUC_lexicon_tags_60 },
    { 0xf2c01089, SUC_lexicon_tags_7 },
    { 0x3a00108b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7160108e, SUC_lexicon_tags_34 },
    { 0x33c0108f, SUC_lexicon_tags_0 },
    { 0xade0108f, SUC_lexicon_tags_11 },
    { 0x4580108e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0109c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70010a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdde010a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede010a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4ae010ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cc010ad, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6010b0, SUC_lexicon_tags_11 },
    { 0xa84010b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0010bb, SUC_lexicon_tags_12 },
    { 0x502010bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108010c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x486010c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa10010c6, SUC_lexicon_tags_2 },
    { 0x624010c7, SUC_lexicon_tags_0 },
    { 0xae2010c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712010cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e010cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc010d3, SUC_lexicon_tags_7 },
    { 0x4a010d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4010d7, SUC_lexicon_tags_7 },
    { 0x8fc010d7, SUC_lexicon_tags_8 },
    { 0x76e010d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48010e5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1be010e7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x84a010e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79c010eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14010f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6010f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x898010f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x294010fb, SUC_lexicon_tags_65 },
    { 0xbf4010fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a401100, SUC_lexicon_tags_11 },
    { 0xf9201100, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67e01103, SUC_lexicon_tags_46 },
    { 0x9f201104, SUC_lexicon_tags_19 },
    { 0x77e01104, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc6c01107, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8240110b, SUC_lexicon_tags_47 },
    { 0xec00110c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e01112, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0401114, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3401116, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e0111d, SUC_lexicon_tags_12 },
    { 0xabc0111e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea001124, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2ba01126, SUC_lexicon_tags_16 },
    { 0xa6601126, SUC_lexicon_tags_15 },
    { 0x89a01128, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfac0112a, SUC_lexicon_tags_54 },
    { 0xe1c0112a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a01132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbee01134, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81801137, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2de01139, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd400113b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4d20113d, SUC_lexicon_tags_0 },
    { 0x8e40113d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb801146, SUC_lexicon_tags_2 },
    { 0x56e01146, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c0114a, SUC_lexicon_tags_1 },
    { 0x87c0114b, SUC_lexicon_tags_1 },
    { 0xe1e0114c, SUC_lexicon_tags_15 },
    { 0x3d60114c, SUC_lexicon_tags_37 },
    { 0x3740114b, SUC_lexicon_tags_25 },
    { 0x3b00114b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x67001151, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2b401153, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a01156, SUC_lexicon_tags_8 },
    { 0x47e01157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef401159, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e0115d, SUC_lexicon_tags_28 },
    { 0x19a0115e, SUC_lexicon_tags_0 },
    { 0x2720115e, SUC_lexicon_tags_60 },
    { 0xe500115d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c01164, SUC_lexicon_tags_45 },
    { 0x2e801165, SUC_lexicon_tags_32 },
    { 0xeb001166, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcb201168, SUC_lexicon_tags_25 },
    { 0x98201169, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8540116b, SUC_lexicon_tags_11 },
    { 0x21c0116b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7c0116e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e01178, SUC_lexicon_tags_32 },
    { 0xd3a01179, SUC_lexicon_tags_28 },
    { 0xfaa0117a, SUC_lexicon_tags_8 },
    { 0xb8401179, SUC_lexicon_tags_25 },
    { 0x5a00117b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec01180, SUC_lexicon_tags_12 },
    { 0xd9401181, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x85c01183, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x80e01185, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89001189, SUC_lexicon_tags_26 },
    { 0x12001189, SUC_lexicon_tags_16 },
    { 0xafa01189, SUC_lexicon_tags_15 },
    { 0x9440118b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c01190, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89601194, SUC_lexicon_tags_11 },
    { 0x93c01194, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be0119a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348011a0, SUC_lexicon_tags_17 },
    { 0x852011a1, SUC_lexicon_tags_234 },
    { 0x98011a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a011a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbfc011a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x284011ac, SUC_lexicon_tags_47 },
    { 0xfd6011ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936011b1, SUC_lexicon_tags_38 },
    { 0xe36011b1, SUC_lexicon_tags_32 },
    { 0xf0e011b3, SUC_lexicon_tags_82 },
    { 0x528011b4, SUC_lexicon_tags_6 },
    { 0x1e6011b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660011b9, SUC_lexicon_tags_43 },
    { 0x560011ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2011be, SUC_lexicon_tags_14 },
    { 0xb1c011be, SUC_lexicon_tags_14 },
    { 0x40011bf, SUC_lexicon_tags_6 },
    { 0x9be011c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x188011c3, SUC_lexicon_tags_40 },
    { 0x9d0011c4, SUC_lexicon_tags_38 },
    { 0xb58011c5, SUC_lexicon_tags_0 },
    { 0x45e011c6, SUC_lexicon_tags_14 },
    { 0x2011c7, SUC_lexicon_tags_70 },
    { 0x262011c4, SUC_lexicon_tags_35 },
    { 0xeb6011c9, SUC_lexicon_tags_38 },
    { 0x768011ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2011cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x54c011cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3c011d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0011d4, SUC_lexicon_tags_7 },
    { 0x6c8011d5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810011d9, SUC_lexicon_tags_77 },
    { 0x504011da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e011de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2011e2, SUC_lexicon_tags_11 },
    { 0xe62011e3, SUC_lexicon_tags_16 },
    { 0x790011e4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8a011e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8011eb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x582011ed, SUC_lexicon_tags_129 },
    { 0xb24011ee, SUC_lexicon_tags_19 },
    { 0x9a011ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x558011f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934011f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa8011f8, SUC_lexicon_tags_37 },
    { 0x1c011f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74011ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66401204, SUC_lexicon_tags_61 },
    { 0xa7a01204, SUC_lexicon_tags_15 },
    { 0xd3401206, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e0120a, SUC_lexicon_tags_17 },
    { 0x5580120b, SUC_lexicon_tags_15 },
    { 0x3600120c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1100120e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e801211, SUC_lexicon_tags_14 },
    { 0x95401211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a01217, SUC_lexicon_tags_17 },
    { 0x42201217, SUC_lexicon_tags_37 },
    { 0x46201217, SUC_lexicon_tags_11 },
    { 0xc80121a, SUC_lexicon_tags_17 },
    { 0xd3c0121b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf840121e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e201221, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f601223, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba01226, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62801228, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c0122a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe320122c, SUC_lexicon_tags_9 },
    { 0x85a0122c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4940122f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf9401231, SUC_lexicon_tags_77 },
    { 0x8aa01232, SUC_lexicon_tags_37 },
    { 0xa4201233, SUC_lexicon_tags_38 },
    { 0x44e01232, SUC_lexicon_tags_1 },
    { 0x63c01235, SUC_lexicon_tags_11 },
    { 0xd0c01236, SUC_lexicon_tags_1 },
    { 0x8ba01236, SUC_lexicon_tags_31 },
    { 0x73001232, SUC_lexicon_tags_15 },
    { 0x2fe01237, SUC_lexicon_tags_27 },
    { 0x7c01237, SUC_lexicon_tags_11 },
    { 0xfb80123b, SUC_lexicon_tags_62 },
    { 0xf240123c, SUC_lexicon_tags_31 },
    { 0xffe0123d, SUC_lexicon_tags_5 },
    { 0xa0a01235, SUC_lexicon_tags_15 },
    { 0x2a80123f, SUC_lexicon_tags_31 },
    { 0x15a01240, SUC_lexicon_tags_17 },
    { 0x3ec01236, SUC_lexicon_tags_42 },
    { 0x34201241, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x26c01244, SUC_lexicon_tags_6 },
    { 0xbfc01245, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd801248, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e0124c, SUC_lexicon_tags_11 },
    { 0x6c80124d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x13e0124f, SUC_lexicon_tags_4 },
    { 0xee401250, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c601256, SUC_lexicon_tags_9 },
    { 0xa9401257, SUC_lexicon_tags_26 },
    { 0x1d001258, SUC_lexicon_tags_17 },
    { 0x4c201259, SUC_lexicon_tags_17 },
    { 0xf3801258, SUC_lexicon_tags_37 },
    { 0x96c0125b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x5700125e, SUC_lexicon_tags_49 },
    { 0x3040125f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7801266, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6640126a, SUC_lexicon_tags_11 },
    { 0x3340126b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfc60126d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9380126f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22601275, SUC_lexicon_tags_38 },
    { 0xce001275, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x94201279, SUC_lexicon_tags_11 },
    { 0xb400127a, SUC_lexicon_tags_1 },
    { 0xfca01279, SUC_lexicon_tags_17 },
    { 0x7ce0127c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8240127e, SUC_lexicon_tags_68 },
    { 0xf600127f, SUC_lexicon_tags_15 },
    { 0xb4c0127f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a01283, SUC_lexicon_tags_47 },
    { 0x12c01284, SUC_lexicon_tags_37 },
    { 0xb9401284, SUC_lexicon_tags_7 },
    { 0x79801284, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2380128a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb660128f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c401297, SUC_lexicon_tags_23 },
    { 0x18201297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb220129c, SUC_lexicon_tags_1 },
    { 0xe840129d, SUC_lexicon_tags_11 },
    { 0x8620129d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbdc012a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b0012a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408012a6, SUC_lexicon_tags_2 },
    { 0xec6012a7, SUC_lexicon_tags_68 },
    { 0xac012a7, SUC_lexicon_tags_87 },
    { 0xd34012a8, SUC_lexicon_tags_38 },
    { 0x86a012a9, SUC_lexicon_tags_15 },
    { 0x32e012ab, SUC_lexicon_tags_47 },
    { 0xe32012ac, SUC_lexicon_tags_28 },
    { 0xc58012ad, SUC_lexicon_tags_8 },
    { 0x26e012a7, SUC_lexicon_tags_19 },
    { 0x6fc012ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x88a012b1, SUC_lexicon_tags_37 },
    { 0xd06012b1, SUC_lexicon_tags_10 },
    { 0x48a012b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x34012b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xecc012b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc36012b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66012bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa0012bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1b4012c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06012c4, SUC_lexicon_tags_7 },
    { 0xc82012c5, SUC_lexicon_tags_19 },
    { 0xfd2012c6, SUC_lexicon_tags_37 },
    { 0xb40012c4, SUC_lexicon_tags_49 },
    { 0x2a0012c6, SUC_lexicon_tags_23 },
    { 0xa04012c9, SUC_lexicon_tags_65 },
    { 0x1ec012ca, SUC_lexicon_tags_19 },
    { 0xe2c012cb, SUC_lexicon_tags_14 },
    { 0x840012c9, SUC_lexicon_tags_8 },
    { 0xa6012c7, SUC_lexicon_tags_1 },
    { 0xada012ce, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd5e012d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298012d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368012d9, SUC_lexicon_tags_9 },
    { 0x7ca012da, SUC_lexicon_tags_19 },
    { 0xe24012db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x31e012dd, SUC_lexicon_tags_8 },
    { 0x760012dd, SUC_lexicon_tags_25 },
    { 0x52012df, SUC_lexicon_tags_32 },
    { 0x2a0012e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4012e7, SUC_lexicon_tags_34 },
    { 0xc04012e7, SUC_lexicon_tags_37 },
    { 0xd28012e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2012ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a8012ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188012f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde012f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x750012f8, SUC_lexicon_tags_17 },
    { 0x49c012f9, SUC_lexicon_tags_28 },
    { 0x446012f8, SUC_lexicon_tags_60 },
    { 0xd60012fa, SUC_lexicon_tags_28 },
    { 0xe92012fc, SUC_lexicon_tags_3 },
    { 0xc0c012fc, SUC_lexicon_tags_12 },
    { 0xfa2012fe, SUC_lexicon_tags_98 },
    { 0x31a012f8, SUC_lexicon_tags_25 },
    { 0x52e01300, SUC_lexicon_tags_14 },
    { 0x57401301, SUC_lexicon_tags_7 },
    { 0xc8c01301, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xefa01304, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe3e01306, SUC_lexicon_tags_26 },
    { 0xcc01307, SUC_lexicon_tags_13 },
    { 0xd6601306, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3560130a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7060130d, SUC_lexicon_tags_48 },
    { 0x9fe0130e, SUC_lexicon_tags_17 },
    { 0x5880130f, SUC_lexicon_tags_60 },
    { 0x2c20130f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1401313, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c01318, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f001321, SUC_lexicon_tags_37 },
    { 0xa4801322, SUC_lexicon_tags_45 },
    { 0x43001323, SUC_lexicon_tags_12 },
    { 0x61001323, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87e01326, SUC_lexicon_tags_27 },
    { 0x11801327, SUC_lexicon_tags_38 },
    { 0x40a01327, SUC_lexicon_tags_63 },
    { 0x99c01328, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49c0132b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6340132d, SUC_lexicon_tags_11 },
    { 0xbc0132e, SUC_lexicon_tags_13 },
    { 0x4820132f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b801332, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e601334, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61601338, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa160133a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2760133e, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x28801340, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c01343, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3601345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73001347, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a00134a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa8e0134c, SUC_lexicon_tags_55 },
    { 0xd240134c, SUC_lexicon_tags_1 },
    { 0x4ce0134d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4a01350, SUC_lexicon_tags_37 },
    { 0xfe01351, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c01357, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe660135b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdca0135d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9020135f, SUC_lexicon_tags_19 },
    { 0x2d401360, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xa6401362, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1801364, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8801367, SUC_lexicon_tags_100 },
    { 0xaa801368, SUC_lexicon_tags_31 },
    { 0xe1c01368, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b20136b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e80136d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2e00136f, SUC_lexicon_tags_37 },
    { 0x3ce01370, SUC_lexicon_tags_26 },
    { 0x96c01371, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8601374, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1001379, SUC_lexicon_tags_38 },
    { 0xffc0137a, SUC_lexicon_tags_12 },
    { 0xeb40137b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25801382, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec01386, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd6801388, SUC_lexicon_tags_60 },
    { 0x59601388, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92c0138b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9900138e, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a01391, SUC_lexicon_tags_23 },
    { 0xf6801392, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f001398, SUC_lexicon_tags_476 },
    { 0, NULL },
    { 0x2600139a, SUC_lexicon_tags_15 },
    { 0xed80139b, SUC_lexicon_tags_45 },
    { 0x1e60139a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9c0139e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06013a1, SUC_lexicon_tags_10 },
    { 0xa58013a1, SUC_lexicon_tags_11 },
    { 0x41e013a3, SUC_lexicon_tags_28 },
    { 0x8c0013a4, SUC_lexicon_tags_65 },
    { 0x172013a5, SUC_lexicon_tags_40 },
    { 0x444013a2, SUC_lexicon_tags_47 },
    { 0xe36013a7, SUC_lexicon_tags_12 },
    { 0xc70013a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x404013ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea013af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc013b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00013b7, SUC_lexicon_tags_7 },
    { 0x93a013b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c013be, SUC_lexicon_tags_27 },
    { 0xce8013bf, SUC_lexicon_tags_15 },
    { 0x318013c0, SUC_lexicon_tags_11 },
    { 0x62013c1, SUC_lexicon_tags_37 },
    { 0xbb0013c2, SUC_lexicon_tags_37 },
    { 0x754013c3, SUC_lexicon_tags_15 },
    { 0xcd4013c4, SUC_lexicon_tags_87 },
    { 0x482013c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x868013c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da013cd, SUC_lexicon_tags_12 },
    { 0x59e013ce, SUC_lexicon_tags_15 },
    { 0xdba013ce, SUC_lexicon_tags_17 },
    { 0xa70013d0, SUC_lexicon_tags_0 },
    { 0x6c6013d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4de013d3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x216013d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8013d9, SUC_lexicon_tags_43 },
    { 0xefc013d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8013dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882013e1, SUC_lexicon_tags_2 },
    { 0x69a013e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc2a013e4, SUC_lexicon_tags_2 },
    { 0xaf2013e5, SUC_lexicon_tags_15 },
    { 0x152013e4, SUC_lexicon_tags_6 },
    { 0xa5e013e7, SUC_lexicon_tags_82 },
    { 0x732013e8, SUC_lexicon_tags_82 },
    { 0x394013e9, SUC_lexicon_tags_5 },
    { 0x5b4013ea, SUC_lexicon_tags_13 },
    { 0xd34013eb, SUC_lexicon_tags_7 },
    { 0x780013e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x572013ee, SUC_lexicon_tags_12 },
    { 0x560013ee, SUC_lexicon_tags_33 },
    { 0xa64013ef, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8013fd, SUC_lexicon_tags_7 },
    { 0x9da013fe, SUC_lexicon_tags_29 },
    { 0x5ca013fd, SUC_lexicon_tags_7 },
    { 0x6c801400, SUC_lexicon_tags_15 },
    { 0x66a01401, SUC_lexicon_tags_49 },
    { 0xb8401402, SUC_lexicon_tags_45 },
    { 0xf0e01403, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16801406, SUC_lexicon_tags_16 },
    { 0xb5401407, SUC_lexicon_tags_33 },
    { 0xe7601408, SUC_lexicon_tags_37 },
    { 0x2d401409, SUC_lexicon_tags_43 },
    { 0x86a0140a, SUC_lexicon_tags_14 },
    { 0x2f20140b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce40140d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce01410, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19201417, SUC_lexicon_tags_49 },
    { 0xa0001418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7001421, SUC_lexicon_tags_7 },
    { 0x81a01422, SUC_lexicon_tags_23 },
    { 0x59801423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3280142f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed401434, SUC_lexicon_tags_17 },
    { 0x32601435, SUC_lexicon_tags_15 },
    { 0x2401435, SUC_lexicon_tags_48 },
    { 0x5aa01437, SUC_lexicon_tags_13 },
    { 0x3ca01438, SUC_lexicon_tags_12 },
    { 0x84201437, SUC_lexicon_tags_77 },
    { 0x6f801437, SUC_lexicon_tags_11 },
    { 0x1120143b, SUC_lexicon_tags_28 },
    { 0x1620143c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x80e0143e, SUC_lexicon_tags_19 },
    { 0x5c0143f, SUC_lexicon_tags_15 },
    { 0x4960143e, SUC_lexicon_tags_14 },
    { 0x69c01441, SUC_lexicon_tags_11 },
    { 0xa7001442, SUC_lexicon_tags_14 },
    { 0x4a801443, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x92801445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8401448, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c0144e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x17801451, SUC_lexicon_tags_25 },
    { 0x6a401452, SUC_lexicon_tags_25 },
    { 0x9e01453, SUC_lexicon_tags_2 },
    { 0xc8e01454, SUC_lexicon_tags_6 },
    { 0x33e01451, SUC_lexicon_tags_29 },
    { 0x9b601456, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b201458, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc0145e, SUC_lexicon_tags_14 },
    { 0xd3c0145e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8201462, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88401467, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e0146a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb00146c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7840146f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c01473, SUC_lexicon_tags_0 },
    { 0x1f401474, SUC_lexicon_tags_2 },
    { 0xc0c01475, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a01479, SUC_lexicon_tags_11 },
    { 0x6ea0147a, SUC_lexicon_tags_46 },
    { 0x6740147a, SUC_lexicon_tags_7 },
    { 0x31e0147a, SUC_lexicon_tags_33 },
    { 0xc160147b, SUC_lexicon_tags_7 },
    { 0xb0e0147b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e01486, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe01489, SUC_lexicon_tags_11 },
    { 0x2ac0148a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9160148c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f001490, SUC_lexicon_tags_27 },
    { 0xf001490, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x60601494, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c01498, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5c0149a, SUC_lexicon_tags_17 },
    { 0xd7c0149a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8014a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca014a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba014a6, SUC_lexicon_tags_63 },
    { 0x9dc014a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83c014a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa26014ab, SUC_lexicon_tags_28 },
    { 0x706014ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70014b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c014bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594014bf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26014c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc014c6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc24014c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf9a014ca, SUC_lexicon_tags_11 },
    { 0x450014ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92014d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4be014d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x316014d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x318014da, SUC_lexicon_tags_37 },
    { 0x78c014da, SUC_lexicon_tags_37 },
    { 0x99c014dc, SUC_lexicon_tags_11 },
    { 0x4ce014dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x47c014df, SUC_lexicon_tags_65 },
    { 0xb0a014e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838014e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284014e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8014ed, SUC_lexicon_tags_38 },
    { 0xe3c014ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3da014f0, SUC_lexicon_tags_27 },
    { 0x110014f0, SUC_lexicon_tags_11 },
    { 0xffa014f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38014f7, SUC_lexicon_tags_15 },
    { 0x890014f8, SUC_lexicon_tags_17 },
    { 0xd10014f8, SUC_lexicon_tags_29 },
    { 0x726014fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c01507, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b40150c, SUC_lexicon_tags_25 },
    { 0x89e0150c, SUC_lexicon_tags_38 },
    { 0x80150c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a01511, SUC_lexicon_tags_69 },
    { 0x11001512, SUC_lexicon_tags_1 },
    { 0x63601513, SUC_lexicon_tags_1 },
    { 0xff001511, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52e01516, SUC_lexicon_tags_11 },
    { 0x6f201516, SUC_lexicon_tags_47 },
    { 0xa2c01518, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1700151e, SUC_lexicon_tags_11 },
    { 0x71a0151f, SUC_lexicon_tags_65 },
    { 0x7620151f, SUC_lexicon_tags_49 },
    { 0x38a01521, SUC_lexicon_tags_6 },
    { 0xe6e01521, SUC_lexicon_tags_11 },
    { 0x9c601523, SUC_lexicon_tags_1 },
    { 0xed801524, SUC_lexicon_tags_38 },
    { 0x84c01521, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400152d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a401531, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c01534, SUC_lexicon_tags_8 },
    { 0x5201535, SUC_lexicon_tags_7 },
    { 0x21201536, SUC_lexicon_tags_56 },
    { 0xacc01535, SUC_lexicon_tags_0 },
    { 0x93401538, SUC_lexicon_tags_15 },
    { 0x21001539, SUC_lexicon_tags_36 },
    { 0x21c0153a, SUC_lexicon_tags_77 },
    { 0x9e201535, SUC_lexicon_tags_11 },
    { 0x5a00153c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a01540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2dc01542, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1601544, SUC_lexicon_tags_27 },
    { 0x5c601545, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3401549, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15a0154b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60154f, SUC_lexicon_tags_25 },
    { 0x40001550, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x55601553, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c01556, SUC_lexicon_tags_7 },
    { 0x4a001557, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fe01559, SUC_lexicon_tags_17 },
    { 0xd1601559, SUC_lexicon_tags_0 },
    { 0x7fa0155a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9a0155d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c201560, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x39a01562, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x15801564, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9201566, SUC_lexicon_tags_0 },
    { 0x25601567, SUC_lexicon_tags_7 },
    { 0x3a401567, SUC_lexicon_tags_7 },
    { 0x2201569, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2220156c, SUC_lexicon_tags_37 },
    { 0xdc0156d, SUC_lexicon_tags_25 },
    { 0xe660156c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a01572, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d401574, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7e01576, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x42201579, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd40157e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec01582, SUC_lexicon_tags_15 },
    { 0x63601583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac201588, SUC_lexicon_tags_17 },
    { 0x40601588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f001594, SUC_lexicon_tags_18 },
    { 0x76c01595, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5040159a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xabc0159c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be015a1, SUC_lexicon_tags_2 },
    { 0x5b2015a2, SUC_lexicon_tags_31 },
    { 0x1e6015a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1da015a5, SUC_lexicon_tags_22 },
    { 0x602015a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c015a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16015ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29c015b0, SUC_lexicon_tags_29 },
    { 0xca015b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0015b4, SUC_lexicon_tags_69 },
    { 0xcde015b4, SUC_lexicon_tags_37 },
    { 0x79a015b6, SUC_lexicon_tags_30 },
    { 0x960015b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x444015ba, SUC_lexicon_tags_16 },
    { 0xba2015bb, SUC_lexicon_tags_11 },
    { 0xcd2015bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa015c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24015c6, SUC_lexicon_tags_25 },
    { 0xcc8015c7, SUC_lexicon_tags_172 },
    { 0x39c015c8, SUC_lexicon_tags_15 },
    { 0xf44015c7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8015cc, SUC_lexicon_tags_26 },
    { 0x6ee015cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188015d3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22015d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592015df, SUC_lexicon_tags_8 },
    { 0xcd6015df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a015e5, SUC_lexicon_tags_25 },
    { 0xbfc015e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe40015e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38015f5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a015f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e015fd, SUC_lexicon_tags_5 },
    { 0xa68015fe, SUC_lexicon_tags_15 },
    { 0x6b2015ff, SUC_lexicon_tags_7 },
    { 0x4da015fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e601602, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb1201604, SUC_lexicon_tags_7 },
    { 0x63c01605, SUC_lexicon_tags_37 },
    { 0xcfc01606, SUC_lexicon_tags_2 },
    { 0x3d201604, SUC_lexicon_tags_13 },
    { 0x30e01608, SUC_lexicon_tags_14 },
    { 0x89201609, SUC_lexicon_tags_37 },
    { 0xa0401606, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c0160d, SUC_lexicon_tags_207 },
    { 0x140160e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f801611, SUC_lexicon_tags_7 },
    { 0x13801611, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c01615, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e0161a, SUC_lexicon_tags_2 },
    { 0x8860161b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e0161f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b001622, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76401629, SUC_lexicon_tags_27 },
    { 0xa480162a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5320162c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31401630, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17a01632, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a01638, SUC_lexicon_tags_1 },
    { 0x32001639, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2a0163b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e40163d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae01640, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb801644, SUC_lexicon_tags_15 },
    { 0xfb201645, SUC_lexicon_tags_33 },
    { 0x5b001646, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb560164b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a01653, SUC_lexicon_tags_21 },
    { 0x14001654, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x67201656, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c0165b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa20165f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7601661, SUC_lexicon_tags_2 },
    { 0xa3001662, SUC_lexicon_tags_9 },
    { 0x3d601663, SUC_lexicon_tags_48 },
    { 0xf7a01664, SUC_lexicon_tags_10 },
    { 0x44001664, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c01669, SUC_lexicon_tags_12 },
    { 0xe2c0166a, SUC_lexicon_tags_30 },
    { 0xf840166a, SUC_lexicon_tags_7 },
    { 0xb320166c, SUC_lexicon_tags_56 },
    { 0x8620166b, SUC_lexicon_tags_28 },
    { 0x6340166b, SUC_lexicon_tags_873 },
    { 0, NULL },
    { 0, NULL },
    { 0x59601671, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb001678, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c0167c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a60167e, SUC_lexicon_tags_23 },
    { 0x2000167e, SUC_lexicon_tags_16 },
    { 0xe4001680, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca01684, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6da01686, SUC_lexicon_tags_37 },
    { 0xb2001687, SUC_lexicon_tags_1 },
    { 0xc1601687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9160168d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e01691, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a01696, SUC_lexicon_tags_38 },
    { 0xb7001696, SUC_lexicon_tags_51 },
    { 0x72801697, SUC_lexicon_tags_7 },
    { 0x91c01699, SUC_lexicon_tags_8 },
    { 0xba401699, SUC_lexicon_tags_46 },
    { 0xc060169b, SUC_lexicon_tags_47 },
    { 0x97801699, SUC_lexicon_tags_17 },
    { 0xa900169a, SUC_lexicon_tags_46 },
    { 0xac00169e, SUC_lexicon_tags_49 },
    { 0x8d00169f, SUC_lexicon_tags_7 },
    { 0x3f80169f, SUC_lexicon_tags_2 },
    { 0xfde016a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c016a4, SUC_lexicon_tags_30 },
    { 0xa7c016a5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0016a9, SUC_lexicon_tags_1 },
    { 0x190016aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580016af, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a016b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc016b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34016bb, SUC_lexicon_tags_63 },
    { 0xf42016bc, SUC_lexicon_tags_11 },
    { 0x26016bd, SUC_lexicon_tags_15 },
    { 0xb40016be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x476016c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6c016c3, SUC_lexicon_tags_124 },
    { 0xb24016c3, SUC_lexicon_tags_33 },
    { 0x3ec016c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe016cc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e016d3, SUC_lexicon_tags_37 },
    { 0xa8c016d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a016d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8016da, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c016e3, SUC_lexicon_tags_33 },
    { 0xffe016e4, SUC_lexicon_tags_1 },
    { 0xd0016e3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfc2016e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8016ec, SUC_lexicon_tags_47 },
    { 0xe0e016ed, SUC_lexicon_tags_37 },
    { 0xc84016ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe4016f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298016f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32016fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10016ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd601702, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61601704, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c0170d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8920170f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e001711, SUC_lexicon_tags_15 },
    { 0x6fc01712, SUC_lexicon_tags_37 },
    { 0x57a01712, SUC_lexicon_tags_31 },
    { 0x5bc01714, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f201718, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d60171e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33401723, SUC_lexicon_tags_11 },
    { 0xbba01724, SUC_lexicon_tags_49 },
    { 0x66801725, SUC_lexicon_tags_129 },
    { 0x59401725, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb720172c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12201730, SUC_lexicon_tags_8 },
    { 0x89601731, SUC_lexicon_tags_7 },
    { 0xd2001732, SUC_lexicon_tags_30 },
    { 0xc1401733, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ee01735, SUC_lexicon_tags_7 },
    { 0xf7801736, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a601738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb780173c, SUC_lexicon_tags_7 },
    { 0x6c20173c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcc0173f, SUC_lexicon_tags_38 },
    { 0x1cc0173f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6e01742, SUC_lexicon_tags_11 },
    { 0xcce01743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe01749, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4420174c, SUC_lexicon_tags_11 },
    { 0x8c80174c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xae01750, SUC_lexicon_tags_11 },
    { 0xc0401751, SUC_lexicon_tags_26 },
    { 0x25801752, SUC_lexicon_tags_51 },
    { 0x61a01753, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea201756, SUC_lexicon_tags_38 },
    { 0x57a01757, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e0175b, SUC_lexicon_tags_87 },
    { 0x3d80175b, SUC_lexicon_tags_15 },
    { 0x90e0175d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x54a0175f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34201762, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cc01764, SUC_lexicon_tags_35 },
    { 0xaa401765, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c01768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce0176b, SUC_lexicon_tags_74 },
    { 0xa5e0176c, SUC_lexicon_tags_17 },
    { 0x4880176d, SUC_lexicon_tags_44 },
    { 0xcbe0176e, SUC_lexicon_tags_19 },
    { 0x20e0176f, SUC_lexicon_tags_12 },
    { 0x9f00176c, SUC_lexicon_tags_23 },
    { 0x58a0176d, SUC_lexicon_tags_125 },
    { 0x7cc01772, SUC_lexicon_tags_25 },
    { 0xc9a01773, SUC_lexicon_tags_11 },
    { 0x5ae01774, SUC_lexicon_tags_17 },
    { 0x16801774, SUC_lexicon_tags_1 },
    { 0xb201776, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5401778, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4600177c, SUC_lexicon_tags_1599 },
    { 0xb7c0177d, SUC_lexicon_tags_27 },
    { 0x1d60177e, SUC_lexicon_tags_0 },
    { 0x71e0177f, SUC_lexicon_tags_29 },
    { 0x140177c, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0x34a01782, SUC_lexicon_tags_11 },
    { 0x37801783, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38601786, SUC_lexicon_tags_15 },
    { 0xbc201786, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc380178e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa01793, SUC_lexicon_tags_4 },
    { 0x83e01794, SUC_lexicon_tags_1 },
    { 0x30c01793, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xefe01797, SUC_lexicon_tags_14 },
    { 0x4b401797, SUC_lexicon_tags_1 },
    { 0xc3401799, SUC_lexicon_tags_25 },
    { 0x2380179a, SUC_lexicon_tags_45 },
    { 0xfb80179b, SUC_lexicon_tags_43 },
    { 0xb1a0179a, SUC_lexicon_tags_0 },
    { 0x4b80179a, SUC_lexicon_tags_66 },
    { 0x3760179e, SUC_lexicon_tags_19 },
    { 0x7880179f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a0017a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x646017a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc017a7, SUC_lexicon_tags_1 },
    { 0x69e017a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4017ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6017b6, SUC_lexicon_tags_2 },
    { 0xbbe017b7, SUC_lexicon_tags_4 },
    { 0xee8017b7, SUC_lexicon_tags_39 },
    { 0xe017b9, SUC_lexicon_tags_17 },
    { 0xc94017ba, SUC_lexicon_tags_6 },
    { 0xcf6017b9, SUC_lexicon_tags_37 },
    { 0x1b6017bc, SUC_lexicon_tags_12 },
    { 0x8e8017bd, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c017c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x704017c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98017c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218017ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c017d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208017d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc12017d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa48017db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e017df, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6017e3, SUC_lexicon_tags_40 },
    { 0xaf0017e3, SUC_lexicon_tags_15 },
    { 0xc58017e5, SUC_lexicon_tags_68 },
    { 0x690017e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186017ea, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x11c017ec, SUC_lexicon_tags_1 },
    { 0xae8017ed, SUC_lexicon_tags_47 },
    { 0xbc4017ee, SUC_lexicon_tags_331 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2017f1, SUC_lexicon_tags_7 },
    { 0xe017f2, SUC_lexicon_tags_25 },
    { 0xb2a017f1, SUC_lexicon_tags_6 },
    { 0xf5a017f4, SUC_lexicon_tags_15 },
    { 0x43e017f5, SUC_lexicon_tags_0 },
    { 0x774017f2, SUC_lexicon_tags_229 },
    { 0x1c017f4, SUC_lexicon_tags_15 },
    { 0xa88017f2, SUC_lexicon_tags_0 },
    { 0xade017f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc90017fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfac017fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8c017ff, SUC_lexicon_tags_19 },
    { 0x1cc01800, SUC_lexicon_tags_2 },
    { 0xeb201800, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xde01803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe01806, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e0180b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f80180d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e601816, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9a401818, SUC_lexicon_tags_2 },
    { 0xb7401819, SUC_lexicon_tags_74 },
    { 0xb420181a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c0181d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2d20181f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17a01821, SUC_lexicon_tags_12 },
    { 0x4ca01821, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x45201825, SUC_lexicon_tags_33 },
    { 0x6b601826, SUC_lexicon_tags_37 },
    { 0x1fc01827, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b401829, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e0182e, SUC_lexicon_tags_65 },
    { 0xe560182f, SUC_lexicon_tags_29 },
    { 0x20e01830, SUC_lexicon_tags_2 },
    { 0xab001830, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e01838, SUC_lexicon_tags_19 },
    { 0xb5601839, SUC_lexicon_tags_7 },
    { 0x8b001839, SUC_lexicon_tags_37 },
    { 0x9100183b, SUC_lexicon_tags_12 },
    { 0x1d60183c, SUC_lexicon_tags_9 },
    { 0x1400183c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a801843, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e01846, SUC_lexicon_tags_54 },
    { 0x5d201847, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f20184c, SUC_lexicon_tags_83 },
    { 0x3420184d, SUC_lexicon_tags_17 },
    { 0x3320184c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf4e01850, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x7a001852, SUC_lexicon_tags_11 },
    { 0xee001853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec01857, SUC_lexicon_tags_11 },
    { 0x32001858, SUC_lexicon_tags_43 },
    { 0x3bc01859, SUC_lexicon_tags_1 },
    { 0x7a0185a, SUC_lexicon_tags_1 },
    { 0xbe801859, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa20185e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a01862, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf801865, SUC_lexicon_tags_49 },
    { 0x62001865, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39201868, SUC_lexicon_tags_87 },
    { 0x64401869, SUC_lexicon_tags_19 },
    { 0x7be0186a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb6a0186c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00186f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc01873, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60001875, SUC_lexicon_tags_11 },
    { 0xc5601876, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa520187d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2001881, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e201883, SUC_lexicon_tags_38 },
    { 0xe2201883, SUC_lexicon_tags_2 },
    { 0xbe801885, SUC_lexicon_tags_26 },
    { 0x18e01885, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c01888, SUC_lexicon_tags_2 },
    { 0xace01888, SUC_lexicon_tags_15 },
    { 0x1e40188a, SUC_lexicon_tags_23 },
    { 0xfae0188a, SUC_lexicon_tags_9 },
    { 0xf020188c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a0188f, SUC_lexicon_tags_11 },
    { 0x90c01890, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbce01892, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe8a01894, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x35801896, SUC_lexicon_tags_12 },
    { 0x9b201897, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0189b, SUC_lexicon_tags_30 },
    { 0x27c0189b, SUC_lexicon_tags_1 },
    { 0xcf40189d, SUC_lexicon_tags_17 },
    { 0x7760189e, SUC_lexicon_tags_12 },
    { 0x3c0189b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa84018a1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2018a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6018a7, SUC_lexicon_tags_25 },
    { 0xbf0018a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a018ad, SUC_lexicon_tags_28 },
    { 0x3e4018ae, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4018b7, SUC_lexicon_tags_69 },
    { 0x5bc018b8, SUC_lexicon_tags_68 },
    { 0x24c018b8, SUC_lexicon_tags_2 },
    { 0x3c8018ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0018c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fe018c3, SUC_lexicon_tags_15 },
    { 0x56018c3, SUC_lexicon_tags_4 },
    { 0xcc4018c5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x244018c8, SUC_lexicon_tags_32 },
    { 0xc82018c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52018ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x868018d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa018d9, SUC_lexicon_tags_37 },
    { 0x546018da, SUC_lexicon_tags_37 },
    { 0x76018db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64018e0, SUC_lexicon_tags_47 },
    { 0xf36018e1, SUC_lexicon_tags_25 },
    { 0x238018e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a018e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a018e8, SUC_lexicon_tags_22 },
    { 0x1e018e9, SUC_lexicon_tags_15 },
    { 0x256018ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a018ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ce018ef, SUC_lexicon_tags_11 },
    { 0x6fa018f0, SUC_lexicon_tags_11 },
    { 0xbc8018f1, SUC_lexicon_tags_33 },
    { 0x35c018f2, SUC_lexicon_tags_25 },
    { 0x80c018f3, SUC_lexicon_tags_17 },
    { 0xcf4018f1, SUC_lexicon_tags_65 },
    { 0xfb0018f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e018f8, SUC_lexicon_tags_17 },
    { 0x6d4018f9, SUC_lexicon_tags_39 },
    { 0x95e018fa, SUC_lexicon_tags_37 },
    { 0xf62018fa, SUC_lexicon_tags_2 },
    { 0x634018fc, SUC_lexicon_tags_39 },
    { 0xa2018fd, SUC_lexicon_tags_6 },
    { 0x22e018f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce601900, SUC_lexicon_tags_4 },
    { 0xc0a01901, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f401906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2e01908, SUC_lexicon_tags_197 },
    { 0xd8c01908, SUC_lexicon_tags_2 },
    { 0xdf60190a, SUC_lexicon_tags_15 },
    { 0xbaa0190b, SUC_lexicon_tags_25 },
    { 0x18a0190c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e60190f, SUC_lexicon_tags_11 },
    { 0x8180190f, SUC_lexicon_tags_1 },
    { 0x87201911, SUC_lexicon_tags_2 },
    { 0x55e01911, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30001916, SUC_lexicon_tags_15 },
    { 0xaa01917, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc0191a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c0191f, SUC_lexicon_tags_8 },
    { 0x5480191f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d401923, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91201926, SUC_lexicon_tags_0 },
    { 0x80401927, SUC_lexicon_tags_33 },
    { 0x2201928, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81a0192a, SUC_lexicon_tags_11 },
    { 0x1960192b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8480192e, SUC_lexicon_tags_11 },
    { 0x86c0192e, SUC_lexicon_tags_49 },
    { 0x3ec01930, SUC_lexicon_tags_37 },
    { 0xfee01931, SUC_lexicon_tags_19 },
    { 0xccc01932, SUC_lexicon_tags_63 },
    { 0x5a401933, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb5801935, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e40193e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf401941, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb201947, SUC_lexicon_tags_0 },
    { 0xb4201948, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc480194a, SUC_lexicon_tags_16 },
    { 0x4c80194b, SUC_lexicon_tags_38 },
    { 0xd3c0194c, SUC_lexicon_tags_18 },
    { 0xcea0194b, SUC_lexicon_tags_68 },
    { 0x4fa0194e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x47001951, SUC_lexicon_tags_31 },
    { 0x49e01952, SUC_lexicon_tags_37 },
    { 0x51a01952, SUC_lexicon_tags_7 },
    { 0xbf201954, SUC_lexicon_tags_32 },
    { 0xc601955, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae601960, SUC_lexicon_tags_13 },
    { 0xf3201960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1201964, SUC_lexicon_tags_25 },
    { 0x53401964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4801968, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e0196c, SUC_lexicon_tags_47 },
    { 0x8ca0196d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93601970, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a01977, SUC_lexicon_tags_23 },
    { 0xd5c01978, SUC_lexicon_tags_63 },
    { 0x29c01979, SUC_lexicon_tags_65 },
    { 0x23e0197a, SUC_lexicon_tags_43 },
    { 0xf900197b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e01981, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a01988, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1e00198a, SUC_lexicon_tags_151 },
    { 0x59c0198a, SUC_lexicon_tags_14 },
    { 0xf9c0198a, SUC_lexicon_tags_79 },
    { 0x9740198b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9540198f, SUC_lexicon_tags_25 },
    { 0x1f60198f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5001994, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a01997, SUC_lexicon_tags_37 },
    { 0xe9401998, SUC_lexicon_tags_46 },
    { 0x60a01999, SUC_lexicon_tags_87 },
    { 0x3f801999, SUC_lexicon_tags_1 },
    { 0x3f20199a, SUC_lexicon_tags_47 },
    { 0xa7801999, SUC_lexicon_tags_16 },
    { 0x5020199d, SUC_lexicon_tags_4 },
    { 0xf8801997, SUC_lexicon_tags_55 },
    { 0x2680199f, SUC_lexicon_tags_31 },
    { 0xa4019a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe48019a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be019a9, SUC_lexicon_tags_1 },
    { 0x150019aa, SUC_lexicon_tags_14 },
    { 0xbac019ab, SUC_lexicon_tags_14 },
    { 0x6d8019a9, SUC_lexicon_tags_37 },
    { 0x8a2019ad, SUC_lexicon_tags_0 },
    { 0xe56019ae, SUC_lexicon_tags_19 },
    { 0x712019ae, SUC_lexicon_tags_37 },
    { 0xe2c019b0, SUC_lexicon_tags_82 },
    { 0xf28019b0, SUC_lexicon_tags_28 },
    { 0x502019b2, SUC_lexicon_tags_9 },
    { 0xb5e019b0, SUC_lexicon_tags_45 },
    { 0xd4c019b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66019b8, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e019bb, SUC_lexicon_tags_33 },
    { 0x608019bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93e019be, SUC_lexicon_tags_37 },
    { 0x272019bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde6019c1, SUC_lexicon_tags_31 },
    { 0x346019c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da019c7, SUC_lexicon_tags_65 },
    { 0x338019c7, SUC_lexicon_tags_7 },
    { 0xc76019c9, SUC_lexicon_tags_39 },
    { 0x75a019c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfa2019cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x326019ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf96019d0, SUC_lexicon_tags_9 },
    { 0xb82019d1, SUC_lexicon_tags_2 },
    { 0x2c4019d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a019d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc019db, SUC_lexicon_tags_15 },
    { 0x2a019dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x630019de, SUC_lexicon_tags_2 },
    { 0x394019df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcae019e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x390019e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d2019e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810019e9, SUC_lexicon_tags_113 },
    { 0x76c019ea, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88019ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e019f1, SUC_lexicon_tags_87 },
    { 0xa0c019f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96019f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274019fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e01a01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda01a04, SUC_lexicon_tags_11 },
    { 0xd3a01a05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c01a0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1401a0d, SUC_lexicon_tags_25 },
    { 0xbfa01a0e, SUC_lexicon_tags_26 },
    { 0x25601a0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66e01a11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b801a14, SUC_lexicon_tags_15 },
    { 0x76601a14, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8801a19, SUC_lexicon_tags_46 },
    { 0x75801a19, SUC_lexicon_tags_49 },
    { 0xc8a01a1b, SUC_lexicon_tags_12 },
    { 0x8f401a19, SUC_lexicon_tags_37 },
    { 0xd1401a19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x81401a20, SUC_lexicon_tags_12 },
    { 0xb401a21, SUC_lexicon_tags_25 },
    { 0xfde01a22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82001a26, SUC_lexicon_tags_11 },
    { 0x38e01a27, SUC_lexicon_tags_14 },
    { 0x7d001a28, SUC_lexicon_tags_0 },
    { 0xf1e01a29, SUC_lexicon_tags_38 },
    { 0xf7e01a2a, SUC_lexicon_tags_37 },
    { 0x9d201a2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe01a30, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e01a33, SUC_lexicon_tags_15 },
    { 0x14201a33, SUC_lexicon_tags_8 },
    { 0x23a01a35, SUC_lexicon_tags_43 },
    { 0xcba01a33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84801a38, SUC_lexicon_tags_33 },
    { 0xf5a01a39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x19401a3c, SUC_lexicon_tags_2 },
    { 0x58c01a3d, SUC_lexicon_tags_37 },
    { 0x48601a3e, SUC_lexicon_tags_2 },
    { 0x82401a3d, SUC_lexicon_tags_17 },
    { 0x42e01a40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12801a47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x97001a4a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d601a50, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb801a53, SUC_lexicon_tags_38 },
    { 0x7da01a53, SUC_lexicon_tags_8 },
    { 0xdba01a54, SUC_lexicon_tags_50 },
    { 0x2e601a56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x29a01a58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf001a5b, SUC_lexicon_tags_7 },
    { 0x43c01a5b, SUC_lexicon_tags_17 },
    { 0xdce01a5d, SUC_lexicon_tags_8 },
    { 0x99801a5e, SUC_lexicon_tags_47 },
    { 0xa6c01a5b, SUC_lexicon_tags_17 },
    { 0x55201a60, SUC_lexicon_tags_19 },
    { 0xd8e01a61, SUC_lexicon_tags_14 },
    { 0x28a01a62, SUC_lexicon_tags_7 },
    { 0xba801a62, SUC_lexicon_tags_4 },
    { 0x8fa01a61, SUC_lexicon_tags_30 },
    { 0xcdc01a65, SUC_lexicon_tags_45 },
    { 0x6ae01a65, SUC_lexicon_tags_2 },
    { 0x70a01a66, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x58401a69, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e801a6c, SUC_lexicon_tags_11 },
    { 0x6e001a6d, SUC_lexicon_tags_39 },
    { 0xc6601a6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20801a72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59801a77, SUC_lexicon_tags_7 },
    { 0xca401a78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83801a7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49001a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc01a81, SUC_lexicon_tags_15 },
    { 0xf7c01a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ea01a84, SUC_lexicon_tags_70 },
    { 0xba201a85, SUC_lexicon_tags_30 },
    { 0xda01a86, SUC_lexicon_tags_7 },
    { 0x35a01a87, SUC_lexicon_tags_0 },
    { 0xf7201a87, SUC_lexicon_tags_11 },
    { 0xa6001a89, SUC_lexicon_tags_60 },
    { 0x7b401a8a, SUC_lexicon_tags_30 },
    { 0x8d601a8b, SUC_lexicon_tags_39 },
    { 0x3401a84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece01a91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d001a96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x63201a99, SUC_lexicon_tags_25 },
    { 0x35801a99, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f401a9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5801aa0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24601aa6, SUC_lexicon_tags_11 },
    { 0x22a01aa6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9d601aa9, SUC_lexicon_tags_37 },
    { 0xcf601aa9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x13c01aac, SUC_lexicon_tags_25 },
    { 0x69001aad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d401ab0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1e01ab2, SUC_lexicon_tags_12 },
    { 0x18a01ab3, SUC_lexicon_tags_7 },
    { 0x42601ab4, SUC_lexicon_tags_27 },
    { 0xe6201ab4, SUC_lexicon_tags_7 },
    { 0x27e01ab6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x40601ab8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62401abe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d001ac2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e01ac6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7401ac9, SUC_lexicon_tags_25 },
    { 0xcca01aca, SUC_lexicon_tags_37 },
    { 0x24401acb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27601acd, SUC_lexicon_tags_2 },
    { 0x89401ace, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d001ad0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e601ad9, SUC_lexicon_tags_55 },
    { 0x45201ad9, SUC_lexicon_tags_15 },
    { 0x86801ad9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9c01add, SUC_lexicon_tags_2 },
    { 0xc0601ade, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d801ae3, SUC_lexicon_tags_1 },
    { 0x7c401ae3, SUC_lexicon_tags_49 },
    { 0xa9c01ae5, SUC_lexicon_tags_45 },
    { 0x3a01ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51601ae8, SUC_lexicon_tags_17 },
    { 0xfc601ae9, SUC_lexicon_tags_7 },
    { 0x4e801aea, SUC_lexicon_tags_39 },
    { 0x52201ae9, SUC_lexicon_tags_43 },
    { 0xec801aec, SUC_lexicon_tags_112 },
    { 0xe3201ae9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e001af2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4401afd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd8e01aff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd001b01, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee01b0d, SUC_lexicon_tags_37 },
    { 0x83e01b0e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb0001b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc01b13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d801b15, SUC_lexicon_tags_48 },
    { 0x20801b15, SUC_lexicon_tags_11 },
    { 0x27801b17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1401b1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x47801b1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e401b22, SUC_lexicon_tags_45 },
    { 0x73201b23, SUC_lexicon_tags_3 },
    { 0xd8c01b24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e01b29, SUC_lexicon_tags_66 },
    { 0xc7601b2a, SUC_lexicon_tags_15 },
    { 0x33401b29, SUC_lexicon_tags_37 },
    { 0x31a01b2b, SUC_lexicon_tags_10 },
    { 0xc201b2d, SUC_lexicon_tags_17 },
    { 0x95c01b2e, SUC_lexicon_tags_60 },
    { 0xde01b2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f201b32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1e01b34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce01b3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a401b45, SUC_lexicon_tags_45 },
    { 0x5aa01b46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c01b4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2801b4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9801b4f, SUC_lexicon_tags_68 },
    { 0x73201b50, SUC_lexicon_tags_21 },
    { 0x26c01b51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22001b53, SUC_lexicon_tags_17 },
    { 0x8b601b54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a601b57, SUC_lexicon_tags_33 },
    { 0x4201b58, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2601b5b, SUC_lexicon_tags_17 },
    { 0x5f801b5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ea01b5e, SUC_lexicon_tags_37 },
    { 0x90201b5e, SUC_lexicon_tags_16 },
    { 0xdc601b60, SUC_lexicon_tags_26 },
    { 0x2601b61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21801b64, SUC_lexicon_tags_7 },
    { 0x28001b65, SUC_lexicon_tags_2 },
    { 0x10201b64, SUC_lexicon_tags_26 },
    { 0x99401b67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e601b6a, SUC_lexicon_tags_2 },
    { 0x3c001b6a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa2c01b6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c01b71, SUC_lexicon_tags_30 },
    { 0xe0401b72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a01b75, SUC_lexicon_tags_2 },
    { 0x39601b75, SUC_lexicon_tags_22 },
    { 0xa801b77, SUC_lexicon_tags_15 },
    { 0x16601b75, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e01b7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0201b80, SUC_lexicon_tags_60 },
    { 0x50c01b81, SUC_lexicon_tags_29 },
    { 0x7fc01b81, SUC_lexicon_tags_46 },
    { 0xe7c01b83, SUC_lexicon_tags_17 },
    { 0x45601b82, SUC_lexicon_tags_7 },
    { 0x5bc01b84, SUC_lexicon_tags_8 },
    { 0xf2001b86, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a01b8a, SUC_lexicon_tags_49 },
    { 0xfb001b8a, SUC_lexicon_tags_7 },
    { 0xc0e01b8a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd2801b8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8e01b90, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd4e01b92, SUC_lexicon_tags_33 },
    { 0xe4201b93, SUC_lexicon_tags_82 },
    { 0xfae01b93, SUC_lexicon_tags_64 },
    { 0xf6001b94, SUC_lexicon_tags_11 },
    { 0x47c01b96, SUC_lexicon_tags_26 },
    { 0x56201b97, SUC_lexicon_tags_2 },
    { 0x5cc01b98, SUC_lexicon_tags_11 },
    { 0xf6c01b99, SUC_lexicon_tags_37 },
    { 0x97601b99, SUC_lexicon_tags_17 },
    { 0xe01b9b, SUC_lexicon_tags_2 },
    { 0x37401b99, SUC_lexicon_tags_77 },
    { 0xbe401b9b, SUC_lexicon_tags_1 },
    { 0xa801b9d, SUC_lexicon_tags_138 },
    { 0x4001b9f, SUC_lexicon_tags_1 },
    { 0x81c01ba0, SUC_lexicon_tags_7 },
    { 0x6b601b9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e01ba8, SUC_lexicon_tags_11 },
    { 0xa001ba8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4da01bab, SUC_lexicon_tags_33 },
    { 0xaac01bac, SUC_lexicon_tags_25 },
    { 0x90a01bad, SUC_lexicon_tags_14 },
    { 0xbf601bae, SUC_lexicon_tags_45 },
    { 0x55001baf, SUC_lexicon_tags_7 },
    { 0x6c201baf, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0601bb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec01bbd, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x93c01bbf, SUC_lexicon_tags_1 },
    { 0x9e201bc0, SUC_lexicon_tags_14 },
    { 0xdce01bbf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe9001bc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef001bc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c01bca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44801bcd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe01bd1, SUC_lexicon_tags_12 },
    { 0x32401bd2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe0201bd4, SUC_lexicon_tags_7 },
    { 0x72401bd5, SUC_lexicon_tags_17 },
    { 0xb4c01bd5, SUC_lexicon_tags_25 },
    { 0xfbc01bd7, SUC_lexicon_tags_37 },
    { 0xa9201bd4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xffa01bda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92a01bdc, SUC_lexicon_tags_57 },
    { 0x4e601bdd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89801be5, SUC_lexicon_tags_262 },
    { 0x19801be5, SUC_lexicon_tags_11 },
    { 0x43a01be7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e01bec, SUC_lexicon_tags_16 },
    { 0xa001bed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2601bfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74001c02, SUC_lexicon_tags_7 },
    { 0xfbe01c02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32401c08, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6401c0c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe01c12, SUC_lexicon_tags_39 },
    { 0xfc601c13, SUC_lexicon_tags_27 },
    { 0xf0601c14, SUC_lexicon_tags_124 },
    { 0xd7001c14, SUC_lexicon_tags_6 },
    { 0x5a001c15, SUC_lexicon_tags_11 },
    { 0x49801c17, SUC_lexicon_tags_60 },
    { 0xcc01c18, SUC_lexicon_tags_6 },
    { 0xe5401c18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a01c1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69801c1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1201c21, SUC_lexicon_tags_11 },
    { 0x20201c22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3401c25, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd201c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec401c2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ce01c2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c01c31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ce01c33, SUC_lexicon_tags_115 },
    { 0xf5801c33, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x39001c37, SUC_lexicon_tags_16 },
    { 0x41a01c37, SUC_lexicon_tags_23 },
    { 0xece01c37, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0x4a601c3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13601c41, SUC_lexicon_tags_34 },
    { 0xfc01c42, SUC_lexicon_tags_25 },
    { 0x15401c43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e01c47, SUC_lexicon_tags_2 },
    { 0x4b601c48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b801c4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3201c50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26401c52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8be01c54, SUC_lexicon_tags_168 },
    { 0xd3a01c55, SUC_lexicon_tags_6 },
    { 0xd2a01c55, SUC_lexicon_tags_14 },
    { 0xd4601c55, SUC_lexicon_tags_1 },
    { 0x27401c58, SUC_lexicon_tags_7 },
    { 0x23e01c59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15c01c5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69401c5d, SUC_lexicon_tags_17 },
    { 0xf0201c5e, SUC_lexicon_tags_3 },
    { 0x53c01c5f, SUC_lexicon_tags_9 },
    { 0x34c01c60, SUC_lexicon_tags_28 },
    { 0xdf801c5f, SUC_lexicon_tags_38 },
    { 0x69a01c62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae201c67, SUC_lexicon_tags_17 },
    { 0x20e01c68, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c01c6e, SUC_lexicon_tags_38 },
    { 0x2aa01c6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6801c77, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8801c80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e01c83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7a01c85, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x27201c88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c001c8c, SUC_lexicon_tags_15 },
    { 0x1b201c8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35e01c8f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35801c95, SUC_lexicon_tags_7 },
    { 0x2ce01c96, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87001c9a, SUC_lexicon_tags_37 },
    { 0x98c01c9b, SUC_lexicon_tags_30 },
    { 0x18a01c9c, SUC_lexicon_tags_45 },
    { 0x78801c9c, SUC_lexicon_tags_47 },
    { 0x80401c9e, SUC_lexicon_tags_31 },
    { 0x3e001c9f, SUC_lexicon_tags_12 },
    { 0x3a201c9c, SUC_lexicon_tags_45 },
    { 0x95601ca1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x99001ca3, SUC_lexicon_tags_174 },
    { 0xa0401ca4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed601cb1, SUC_lexicon_tags_31 },
    { 0x87801cb2, SUC_lexicon_tags_113 },
    { 0x90401cb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c801cb5, SUC_lexicon_tags_68 },
    { 0x63a01cb5, SUC_lexicon_tags_25 },
    { 0x6fa01cb7, SUC_lexicon_tags_82 },
    { 0x75a01cb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1dc01cba, SUC_lexicon_tags_23 },
    { 0xe6401cbb, SUC_lexicon_tags_12 },
    { 0x8a001cbb, SUC_lexicon_tags_27 },
    { 0x13401cbc, SUC_lexicon_tags_14 },
    { 0x24a01cbe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x93401cc0, SUC_lexicon_tags_43 },
    { 0xb4c01cc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1201cc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9001cca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a01ccd, SUC_lexicon_tags_57 },
    { 0xa0001cce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a01cd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c01cdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c01ce1, SUC_lexicon_tags_17 },
    { 0x1a401ce2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x74001ce4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd5801ce6, SUC_lexicon_tags_2 },
    { 0xff601ce7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61001ce9, SUC_lexicon_tags_10 },
    { 0x1cc01cea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74601ced, SUC_lexicon_tags_11 },
    { 0xdbe01cee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9001cf0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e01cf3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19001cf8, SUC_lexicon_tags_23 },
    { 0x99e01cf8, SUC_lexicon_tags_49 },
    { 0xd4c01cfa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa001d02, SUC_lexicon_tags_47 },
    { 0xb5c01d03, SUC_lexicon_tags_15 },
    { 0x6e601d04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f001d09, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5201d0e, SUC_lexicon_tags_11 },
    { 0x89001d0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35c01d11, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5601d15, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82401d1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e201d22, SUC_lexicon_tags_2 },
    { 0xd0a01d23, SUC_lexicon_tags_45 },
    { 0x79601d22, SUC_lexicon_tags_2 },
    { 0xaea01d25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83e01d27, SUC_lexicon_tags_43 },
    { 0x77001d28, SUC_lexicon_tags_12 },
    { 0x7201d28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e01d2f, SUC_lexicon_tags_11 },
    { 0x6b801d30, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3001d35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaca01d37, SUC_lexicon_tags_27 },
    { 0x59a01d37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7001d3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb801d3f, SUC_lexicon_tags_11 },
    { 0x5ca01d40, SUC_lexicon_tags_16 },
    { 0x79801d41, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6001d46, SUC_lexicon_tags_15 },
    { 0x47201d47, SUC_lexicon_tags_15 },
    { 0x2d001d48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b201d4a, SUC_lexicon_tags_136 },
    { 0x4201d4b, SUC_lexicon_tags_38 },
    { 0xa9e01d4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c01d53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x81001d56, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50001d5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa01d5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ba01d5f, SUC_lexicon_tags_38 },
    { 0x50801d60, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c01d64, SUC_lexicon_tags_7 },
    { 0x39601d65, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23401d6d, SUC_lexicon_tags_63 },
    { 0x74601d6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c01d71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcac01d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ce01d75, SUC_lexicon_tags_7 },
    { 0x62801d76, SUC_lexicon_tags_25 },
    { 0x93801d77, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x10401d79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdce01d7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x11201d7e, SUC_lexicon_tags_63 },
    { 0xd9a01d7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c01d82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cc01d84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa01d88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69201d8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a01d8d, SUC_lexicon_tags_2 },
    { 0x87001d8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa5001d90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38001d9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a01d9f, SUC_lexicon_tags_17 },
    { 0x7ae01da0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64601da2, SUC_lexicon_tags_6 },
    { 0xc4201da3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xba01da5, SUC_lexicon_tags_107 },
    { 0x51c01da6, SUC_lexicon_tags_25 },
    { 0x8be01da7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ca01da9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c01dae, SUC_lexicon_tags_27 },
    { 0xd1a01daf, SUC_lexicon_tags_37 },
    { 0x13201db0, SUC_lexicon_tags_17 },
    { 0x60201db0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94201db3, SUC_lexicon_tags_81 },
    { 0xd3c01db4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a01db7, SUC_lexicon_tags_26 },
    { 0x17401db8, SUC_lexicon_tags_11 },
    { 0x33001db9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0801dbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3201dc0, SUC_lexicon_tags_25 },
    { 0xef201dc1, SUC_lexicon_tags_60 },
    { 0xe0601dc2, SUC_lexicon_tags_25 },
    { 0x93401dc2, SUC_lexicon_tags_66 },
    { 0x28601dc2, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0xace01dc7, SUC_lexicon_tags_11 },
    { 0x24a01dc8, SUC_lexicon_tags_33 },
    { 0x7ac01dc8, SUC_lexicon_tags_26 },
    { 0x27201dc9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27201dce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ee01dd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdca01dd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70801dd4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe0e01dd6, SUC_lexicon_tags_23 },
    { 0x2e201dd6, SUC_lexicon_tags_29 },
    { 0x43a01dd8, SUC_lexicon_tags_1 },
    { 0xbea01dd9, SUC_lexicon_tags_27 },
    { 0x84a01dda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae601dde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec01de2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3001de5, SUC_lexicon_tags_66 },
    { 0x3e801de6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2601de8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20801deb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee601dee, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf8601df0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc01df5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1be01df7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36c01df9, SUC_lexicon_tags_37 },
    { 0x78801dfa, SUC_lexicon_tags_37 },
    { 0x56001df9, SUC_lexicon_tags_4 },
    { 0x52a01dfc, SUC_lexicon_tags_25 },
    { 0x26801df9, SUC_lexicon_tags_11 },
    { 0xd4e01dfe, SUC_lexicon_tags_201 },
    { 0xbf001dff, SUC_lexicon_tags_15 },
    { 0xd2801dff, SUC_lexicon_tags_62 },
    { 0x9801e01, SUC_lexicon_tags_7 },
    { 0x54801dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c01e07, SUC_lexicon_tags_44 },
    { 0x67801e07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf401e0b, SUC_lexicon_tags_12 },
    { 0x91a01e0c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x27801e0f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6801e12, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c01e17, SUC_lexicon_tags_15 },
    { 0xaac01e18, SUC_lexicon_tags_47 },
    { 0xfce01e18, SUC_lexicon_tags_37 },
    { 0x43401e19, SUC_lexicon_tags_17 },
    { 0xa1801e1b, SUC_lexicon_tags_14 },
    { 0x4a801e18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7201e1f, SUC_lexicon_tags_48 },
    { 0x85601e20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c601e22, SUC_lexicon_tags_2 },
    { 0xb1c01e23, SUC_lexicon_tags_11 },
    { 0x90401e22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53801e2c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae01e2f, SUC_lexicon_tags_11 },
    { 0x96e01e30, SUC_lexicon_tags_85 },
    { 0x76c01e31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ea01e33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x84201e35, SUC_lexicon_tags_68 },
    { 0x4ec01e35, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b201e3b, SUC_lexicon_tags_60 },
    { 0xc8001e3c, SUC_lexicon_tags_14 },
    { 0x1b401e3d, SUC_lexicon_tags_15 },
    { 0x66001e3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13601e42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd401e44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0601e48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55601e4b, SUC_lexicon_tags_12 },
    { 0x7e401e4c, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e01e51, SUC_lexicon_tags_12 },
    { 0xe7001e51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7801e54, SUC_lexicon_tags_2 },
    { 0x47201e54, SUC_lexicon_tags_6 },
    { 0x25801e56, SUC_lexicon_tags_43 },
    { 0xe801e56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4001e5d, SUC_lexicon_tags_1 },
    { 0xda201e5e, SUC_lexicon_tags_27 },
    { 0xd4001e5d, SUC_lexicon_tags_25 },
    { 0x67e01e60, SUC_lexicon_tags_6 },
    { 0x3ec01e5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e01e70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92201e75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa01e79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53801e7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b401e80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd801e83, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e01e8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28601e93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc601e95, SUC_lexicon_tags_26 },
    { 0x2fe01e96, SUC_lexicon_tags_17 },
    { 0xb6001e96, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2c01e99, SUC_lexicon_tags_15 },
    { 0x66a01e99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c01e9d, SUC_lexicon_tags_31 },
    { 0x77e01e9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6201ea3, SUC_lexicon_tags_17 },
    { 0x7e01ea4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x35e01ea6, SUC_lexicon_tags_6 },
    { 0xa8801ea7, SUC_lexicon_tags_11 },
    { 0xf5801ea8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x63201eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34801eac, SUC_lexicon_tags_7 },
    { 0xa2c01ead, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4201eb5, SUC_lexicon_tags_23 },
    { 0x9c01eb5, SUC_lexicon_tags_25 },
    { 0xc4801eb6, SUC_lexicon_tags_107 },
    { 0x46001eb8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x93801ebb, SUC_lexicon_tags_68 },
    { 0x7f001ebc, SUC_lexicon_tags_25 },
    { 0xf4c01ebd, SUC_lexicon_tags_27 },
    { 0xf7001ebd, SUC_lexicon_tags_38 },
    { 0xcfe01ebe, SUC_lexicon_tags_27 },
    { 0xa1a01ebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45001ec6, SUC_lexicon_tags_12 },
    { 0x2cc01ec6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4001ed1, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c01ed4, SUC_lexicon_tags_12 },
    { 0xe4201ed5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44601ed7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x32201eda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a01edd, SUC_lexicon_tags_1 },
    { 0xb9e01edd, SUC_lexicon_tags_15 },
    { 0x79001ede, SUC_lexicon_tags_15 },
    { 0xf4001ee0, SUC_lexicon_tags_65 },
    { 0x27201edf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e01ee4, SUC_lexicon_tags_49 },
    { 0x87801ee5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa01eed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede01ef1, SUC_lexicon_tags_25 },
    { 0x5ca01ef2, SUC_lexicon_tags_38 },
    { 0xfae01ef1, SUC_lexicon_tags_10 },
    { 0x1601ef4, SUC_lexicon_tags_2 },
    { 0xe1401ef5, SUC_lexicon_tags_15 },
    { 0x91a01ef6, SUC_lexicon_tags_299 },
    { 0x77e01ef7, SUC_lexicon_tags_7 },
    { 0x1fc01ef4, SUC_lexicon_tags_30 },
    { 0x31401ef5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9601efc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x19801eff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2001f03, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5d201f05, SUC_lexicon_tags_17 },
    { 0x81201f05, SUC_lexicon_tags_2 },
    { 0x8ac01f07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa01f0b, SUC_lexicon_tags_14 },
    { 0x90401f0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb001f0e, SUC_lexicon_tags_38 },
    { 0x99a01f0f, SUC_lexicon_tags_11 },
    { 0xde401f0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4601f12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e601f18, SUC_lexicon_tags_63 },
    { 0xbd601f19, SUC_lexicon_tags_12 },
    { 0x6fc01f19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e01f1d, SUC_lexicon_tags_85 },
    { 0x31001f1e, SUC_lexicon_tags_60 },
    { 0xe0401f1e, SUC_lexicon_tags_1 },
    { 0x50e01f20, SUC_lexicon_tags_1 },
    { 0xc7a01f21, SUC_lexicon_tags_17 },
    { 0x52601f22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e601f27, SUC_lexicon_tags_11 },
    { 0xaa801f28, SUC_lexicon_tags_15 },
    { 0x76401f29, SUC_lexicon_tags_11 },
    { 0xa3c01f29, SUC_lexicon_tags_7 },
    { 0x2c001f2b, SUC_lexicon_tags_23 },
    { 0x13001f2a, SUC_lexicon_tags_47 },
    { 0x18201f2d, SUC_lexicon_tags_66 },
    { 0x2fe01f27, SUC_lexicon_tags_60 },
    { 0x28001f28, SUC_lexicon_tags_1 },
    { 0x47201f30, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd2801f32, SUC_lexicon_tags_26 },
    { 0xc3401f33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e01f37, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc01f3d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f001f47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27601f52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38001f5a, SUC_lexicon_tags_15 },
    { 0x45201f5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5801f5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c01f63, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee201f67, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba01f6a, SUC_lexicon_tags_1 },
    { 0xb8401f6a, SUC_lexicon_tags_1355 },
    { 0xc8801f6c, SUC_lexicon_tags_6 },
    { 0x1b001f6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0c01f6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f201f72, SUC_lexicon_tags_25 },
    { 0xcd401f73, SUC_lexicon_tags_2 },
    { 0x5a01f74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb201f7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c01f7f, SUC_lexicon_tags_23 },
    { 0xfe401f80, SUC_lexicon_tags_12 },
    { 0xa4001f7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2a01f83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3001f85, SUC_lexicon_tags_11 },
    { 0x34001f86, SUC_lexicon_tags_11 },
    { 0x85c01f86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a01f8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51e01f8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d201f98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e201fa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5801fa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92801fa6, SUC_lexicon_tags_37 },
    { 0xec801fa7, SUC_lexicon_tags_38 },
    { 0x6e201fa8, SUC_lexicon_tags_37 },
    { 0x23201fa9, SUC_lexicon_tags_17 },
    { 0xc7001fa6, SUC_lexicon_tags_23 },
    { 0x14001faa, SUC_lexicon_tags_7 },
    { 0x9ca01fac, SUC_lexicon_tags_39 },
    { 0x8b401fa6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c801fb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc01fb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a01fba, SUC_lexicon_tags_3 },
    { 0x40801fbb, SUC_lexicon_tags_174 },
    { 0xf6601fbc, SUC_lexicon_tags_17 },
    { 0xf8401fbd, SUC_lexicon_tags_11 },
    { 0x67801fbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2e01fc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2401fc2, SUC_lexicon_tags_28 },
    { 0x73201fc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff801fc6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa01fcc, SUC_lexicon_tags_15 },
    { 0x24201fcd, SUC_lexicon_tags_37 },
    { 0x35801fce, SUC_lexicon_tags_17 },
    { 0xeec01fcf, SUC_lexicon_tags_2 },
    { 0x67c01fd0, SUC_lexicon_tags_64 },
    { 0x2ba01fd1, SUC_lexicon_tags_4 },
    { 0x85401fcf, SUC_lexicon_tags_15 },
    { 0xbbc01fd3, SUC_lexicon_tags_48 },
    { 0x21a01fcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1de01fd6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e601fdb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a01fde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32201fe0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6001fe3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9201fe6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ee01fe8, SUC_lexicon_tags_17 },
    { 0x61801fe9, SUC_lexicon_tags_2 },
    { 0x3fa01fe9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4e801fec, SUC_lexicon_tags_66 },
    { 0x98201fed, SUC_lexicon_tags_1 },
    { 0xd3c01fee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e001ff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24201ffe, SUC_lexicon_tags_25 },
    { 0x92201fff, SUC_lexicon_tags_37 },
    { 0xbc802000, SUC_lexicon_tags_1 },
    { 0xc002001, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc602006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a002008, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2440200a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e02010, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x84602013, SUC_lexicon_tags_55 },
    { 0xc2e02013, SUC_lexicon_tags_15 },
    { 0x66a02014, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab40201d, SUC_lexicon_tags_37 },
    { 0xd860201e, SUC_lexicon_tags_37 },
    { 0xba60201f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf1402021, SUC_lexicon_tags_17 },
    { 0x8202022, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b002024, SUC_lexicon_tags_11 },
    { 0x3a02025, SUC_lexicon_tags_2 },
    { 0x40a02025, SUC_lexicon_tags_33 },
    { 0x8e802026, SUC_lexicon_tags_37 },
    { 0x76202028, SUC_lexicon_tags_63 },
    { 0x73602028, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2000202c, SUC_lexicon_tags_77 },
    { 0x9f60202d, SUC_lexicon_tags_29 },
    { 0xa780202c, SUC_lexicon_tags_38 },
    { 0x6980202f, SUC_lexicon_tags_45 },
    { 0xc9402030, SUC_lexicon_tags_8 },
    { 0x4c202031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1402038, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x700203a, SUC_lexicon_tags_1 },
    { 0x4e0203b, SUC_lexicon_tags_12 },
    { 0x4bc0203c, SUC_lexicon_tags_15 },
    { 0xb620203d, SUC_lexicon_tags_8 },
    { 0x9d20203e, SUC_lexicon_tags_39 },
    { 0xd8c0203f, SUC_lexicon_tags_37 },
    { 0xdd00203f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26802045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a02051, SUC_lexicon_tags_3 },
    { 0xf1a02052, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c02056, SUC_lexicon_tags_27 },
    { 0xd4a02056, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd780205c, SUC_lexicon_tags_16 },
    { 0xb940205d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9602061, SUC_lexicon_tags_296 },
    { 0xf9002062, SUC_lexicon_tags_1 },
    { 0xc2802063, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a802065, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae402069, SUC_lexicon_tags_29 },
    { 0x39202069, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e02070, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb3202072, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x13202074, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1802078, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe560207f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97002081, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26202089, SUC_lexicon_tags_15 },
    { 0x53202089, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f00208c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbce0208e, SUC_lexicon_tags_12 },
    { 0x3340208f, SUC_lexicon_tags_38 },
    { 0x5920208f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e02096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd60209b, SUC_lexicon_tags_55 },
    { 0xc040209c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce020a4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a020a8, SUC_lexicon_tags_60 },
    { 0xac0020a9, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xc80020ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x42020ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b2020b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca020b6, SUC_lexicon_tags_57 },
    { 0x636020b7, SUC_lexicon_tags_25 },
    { 0x100020b7, SUC_lexicon_tags_15 },
    { 0x408020b9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x560020bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4a020be, SUC_lexicon_tags_37 },
    { 0x1f2020be, SUC_lexicon_tags_1 },
    { 0xf14020c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa48020c2, SUC_lexicon_tags_17 },
    { 0x548020c2, SUC_lexicon_tags_37 },
    { 0x1b2020c4, SUC_lexicon_tags_23 },
    { 0xac0020c5, SUC_lexicon_tags_49 },
    { 0x94c020c3, SUC_lexicon_tags_25 },
    { 0x732020c7, SUC_lexicon_tags_11 },
    { 0xce8020c5, SUC_lexicon_tags_31 },
    { 0x72020c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c020cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c020d7, SUC_lexicon_tags_47 },
    { 0x962020d7, SUC_lexicon_tags_1 },
    { 0xb58020d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb6020db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4a020dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60020e1, SUC_lexicon_tags_1 },
    { 0xea0020e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c020e5, SUC_lexicon_tags_7 },
    { 0xeea020e6, SUC_lexicon_tags_65 },
    { 0x770020e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x11a020e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a020ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26020f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c020fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0602103, SUC_lexicon_tags_17 },
    { 0x24402104, SUC_lexicon_tags_65 },
    { 0x49602105, SUC_lexicon_tags_11 },
    { 0xaa02106, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f602109, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4740210b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda402112, SUC_lexicon_tags_45 },
    { 0xb6202112, SUC_lexicon_tags_49 },
    { 0x7f802114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f602118, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a0211b, SUC_lexicon_tags_30 },
    { 0xabe0211b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9002121, SUC_lexicon_tags_4 },
    { 0xf7602122, SUC_lexicon_tags_27 },
    { 0x21402122, SUC_lexicon_tags_12 },
    { 0x71a02124, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc02129, SUC_lexicon_tags_25 },
    { 0xa20212a, SUC_lexicon_tags_49 },
    { 0xea0212b, SUC_lexicon_tags_17 },
    { 0x89e0212c, SUC_lexicon_tags_47 },
    { 0xdce0212d, SUC_lexicon_tags_66 },
    { 0xa40212e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2c02130, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb402133, SUC_lexicon_tags_28 },
    { 0xeb202134, SUC_lexicon_tags_60 },
    { 0x10a02133, SUC_lexicon_tags_14 },
    { 0x58402136, SUC_lexicon_tags_25 },
    { 0x58002137, SUC_lexicon_tags_11 },
    { 0x23802137, SUC_lexicon_tags_63 },
    { 0x1f802138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3640213b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c0213e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf602141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5a02143, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcd202145, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00214a, SUC_lexicon_tags_0 },
    { 0x3760214b, SUC_lexicon_tags_11 },
    { 0x96a0214c, SUC_lexicon_tags_47 },
    { 0x4580214d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa260214f, SUC_lexicon_tags_31 },
    { 0xa940214f, SUC_lexicon_tags_2 },
    { 0x2ea02151, SUC_lexicon_tags_12 },
    { 0x3be0214f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11202154, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec0215a, SUC_lexicon_tags_49 },
    { 0x9560215b, SUC_lexicon_tags_37 },
    { 0x51a0215c, SUC_lexicon_tags_15 },
    { 0x5920215c, SUC_lexicon_tags_82 },
    { 0xa540215b, SUC_lexicon_tags_11 },
    { 0xe440215f, SUC_lexicon_tags_2 },
    { 0xb5402160, SUC_lexicon_tags_11 },
    { 0x1800215b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeae02163, SUC_lexicon_tags_37 },
    { 0x46a02164, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f202167, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3d002169, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdc60216b, SUC_lexicon_tags_12 },
    { 0x82a0216c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca0216f, SUC_lexicon_tags_1 },
    { 0xd4202170, SUC_lexicon_tags_37 },
    { 0xb2002170, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x25402174, SUC_lexicon_tags_37 },
    { 0xca202174, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78802179, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4002181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75402184, SUC_lexicon_tags_35 },
    { 0x8e202185, SUC_lexicon_tags_37 },
    { 0x50002186, SUC_lexicon_tags_15 },
    { 0x7ba02185, SUC_lexicon_tags_27 },
    { 0xb002186, SUC_lexicon_tags_243 },
    { 0x9bc02189, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xea60218b, SUC_lexicon_tags_12 },
    { 0x8660218b, SUC_lexicon_tags_25 },
    { 0xcb80218d, SUC_lexicon_tags_0 },
    { 0x3fa0218d, SUC_lexicon_tags_47 },
    { 0x7220218d, SUC_lexicon_tags_506 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c02196, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb80219a, SUC_lexicon_tags_185 },
    { 0xb7e0219b, SUC_lexicon_tags_182 },
    { 0x1ce0219c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e021a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73e021a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e021a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x950021a9, SUC_lexicon_tags_32 },
    { 0x9d0021aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f4021ac, SUC_lexicon_tags_29 },
    { 0xec0021ac, SUC_lexicon_tags_17 },
    { 0x168021ae, SUC_lexicon_tags_9 },
    { 0x42c021ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae021b4, SUC_lexicon_tags_32 },
    { 0xe90021b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec021b9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x5b4021bb, SUC_lexicon_tags_47 },
    { 0x2d6021bc, SUC_lexicon_tags_37 },
    { 0x526021bd, SUC_lexicon_tags_47 },
    { 0xa9e021be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a021c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb4021c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x372021c7, SUC_lexicon_tags_28 },
    { 0xe90021c8, SUC_lexicon_tags_11 },
    { 0xc58021c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244021cf, SUC_lexicon_tags_101 },
    { 0xc42021cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32021d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be021d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e021dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1b8021de, SUC_lexicon_tags_17 },
    { 0x294021df, SUC_lexicon_tags_29 },
    { 0xc6a021e0, SUC_lexicon_tags_37 },
    { 0x7b6021df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1c021e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a021e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x374021e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c021eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4021ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc26021ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638021f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700021fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e021fd, SUC_lexicon_tags_47 },
    { 0x720021fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4002201, SUC_lexicon_tags_9 },
    { 0xeec02202, SUC_lexicon_tags_26 },
    { 0xc1602203, SUC_lexicon_tags_37 },
    { 0xd2402203, SUC_lexicon_tags_26 },
    { 0x16402204, SUC_lexicon_tags_47 },
    { 0xe3802206, SUC_lexicon_tags_45 },
    { 0x9202204, SUC_lexicon_tags_11 },
    { 0x68802202, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e0220b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cc0220d, SUC_lexicon_tags_17 },
    { 0xab20220d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x50c02210, SUC_lexicon_tags_7 },
    { 0x9b402211, SUC_lexicon_tags_7 },
    { 0x9d402211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f602214, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8380221b, SUC_lexicon_tags_17 },
    { 0xad20221c, SUC_lexicon_tags_12 },
    { 0x91a0221d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa802221, SUC_lexicon_tags_30 },
    { 0x12602221, SUC_lexicon_tags_2 },
    { 0x35002223, SUC_lexicon_tags_56 },
    { 0x50602224, SUC_lexicon_tags_1 },
    { 0x42e02224, SUC_lexicon_tags_11 },
    { 0x86002222, SUC_lexicon_tags_56 },
    { 0x4b802227, SUC_lexicon_tags_47 },
    { 0x6ec02226, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0222b, SUC_lexicon_tags_17 },
    { 0xa080222c, SUC_lexicon_tags_30 },
    { 0x3d20222b, SUC_lexicon_tags_27 },
    { 0x9920222e, SUC_lexicon_tags_19 },
    { 0x5960222f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde602233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca02238, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa100223b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e0223d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe02243, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c02249, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e802251, SUC_lexicon_tags_48 },
    { 0xe5602251, SUC_lexicon_tags_25 },
    { 0x54202253, SUC_lexicon_tags_25 },
    { 0x51202253, SUC_lexicon_tags_31 },
    { 0x52c02255, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc1c02257, SUC_lexicon_tags_17 },
    { 0x38802258, SUC_lexicon_tags_33 },
    { 0xb602259, SUC_lexicon_tags_60 },
    { 0xd080225a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7200225e, SUC_lexicon_tags_60 },
    { 0xa8c0225e, SUC_lexicon_tags_7 },
    { 0x6120225e, SUC_lexicon_tags_11 },
    { 0xca02261, SUC_lexicon_tags_7 },
    { 0x2260, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49002266, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x59202269, SUC_lexicon_tags_39 },
    { 0x9020226a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ce0226c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1360226e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a0227a, SUC_lexicon_tags_17 },
    { 0x380227a, SUC_lexicon_tags_23 },
    { 0xfcc0227b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd560227f, SUC_lexicon_tags_2 },
    { 0xede02280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7802286, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xe2202288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a0228b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b00228f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3c402291, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa0c02293, SUC_lexicon_tags_25 },
    { 0x48202294, SUC_lexicon_tags_12 },
    { 0xd1202293, SUC_lexicon_tags_37 },
    { 0x35a02296, SUC_lexicon_tags_79 },
    { 0xfb602297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7420229a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf960229d, SUC_lexicon_tags_0 },
    { 0x4e00229d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x238022a1, SUC_lexicon_tags_43 },
    { 0xd5c022a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x41a022a4, SUC_lexicon_tags_33 },
    { 0x82a022a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x464022a8, SUC_lexicon_tags_65 },
    { 0x886022a8, SUC_lexicon_tags_81 },
    { 0xc98022a8, SUC_lexicon_tags_14 },
    { 0xad8022ab, SUC_lexicon_tags_11 },
    { 0xf30022ab, SUC_lexicon_tags_11 },
    { 0xd84022ad, SUC_lexicon_tags_11 },
    { 0x102022ac, SUC_lexicon_tags_14 },
    { 0x1d6022af, SUC_lexicon_tags_15 },
    { 0x858022ab, SUC_lexicon_tags_29 },
    { 0x526022af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea022b6, SUC_lexicon_tags_1 },
    { 0xa0e022b7, SUC_lexicon_tags_37 },
    { 0xbb0022b8, SUC_lexicon_tags_17 },
    { 0x710022b9, SUC_lexicon_tags_9 },
    { 0x8ae022ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8022bf, SUC_lexicon_tags_14 },
    { 0x36c022bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d8022c2, SUC_lexicon_tags_6 },
    { 0xff8022c3, SUC_lexicon_tags_7 },
    { 0xe26022c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae022c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d2022c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66022ce, SUC_lexicon_tags_63 },
    { 0x500022cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea8022d1, SUC_lexicon_tags_25 },
    { 0x7ca022d2, SUC_lexicon_tags_7 },
    { 0x7d8022d3, SUC_lexicon_tags_27 },
    { 0x884022d4, SUC_lexicon_tags_43 },
    { 0x250022d4, SUC_lexicon_tags_37 },
    { 0xcac022d5, SUC_lexicon_tags_37 },
    { 0x8a2022d7, SUC_lexicon_tags_35 },
    { 0x78e022d4, SUC_lexicon_tags_46 },
    { 0x91e022d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x550022db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x268022de, SUC_lexicon_tags_30 },
    { 0xe94022de, SUC_lexicon_tags_8 },
    { 0xb80022df, SUC_lexicon_tags_37 },
    { 0x486022e0, SUC_lexicon_tags_12 },
    { 0x6f8022e2, SUC_lexicon_tags_40 },
    { 0x574022e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34022e8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x12c022ea, SUC_lexicon_tags_47 },
    { 0xc56022eb, SUC_lexicon_tags_17 },
    { 0xadc022ec, SUC_lexicon_tags_25 },
    { 0xebe022ec, SUC_lexicon_tags_29 },
    { 0xdea022ec, SUC_lexicon_tags_47 },
    { 0x6d8022ef, SUC_lexicon_tags_15 },
    { 0x590022ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204022f4, SUC_lexicon_tags_10 },
    { 0x11a022f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46022f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37a022fb, SUC_lexicon_tags_14 },
    { 0xbee022fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcde022fe, SUC_lexicon_tags_27 },
    { 0x65c022ff, SUC_lexicon_tags_12 },
    { 0xa90022ff, SUC_lexicon_tags_1 },
    { 0x86802301, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3820230d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c02310, SUC_lexicon_tags_2 },
    { 0xd5002310, SUC_lexicon_tags_0 },
    { 0x49202312, SUC_lexicon_tags_33 },
    { 0x1c02313, SUC_lexicon_tags_17 },
    { 0xc802314, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc0231d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e02320, SUC_lexicon_tags_16 },
    { 0xde02321, SUC_lexicon_tags_25 },
    { 0x4a802322, SUC_lexicon_tags_12 },
    { 0x3b602323, SUC_lexicon_tags_19 },
    { 0xcfc02324, SUC_lexicon_tags_17 },
    { 0x58802324, SUC_lexicon_tags_36 },
    { 0xfb602326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5260232a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba0232d, SUC_lexicon_tags_7 },
    { 0xa3e0232e, SUC_lexicon_tags_21 },
    { 0x15a0232f, SUC_lexicon_tags_25 },
    { 0x20e0232f, SUC_lexicon_tags_77 },
    { 0xada02330, SUC_lexicon_tags_7 },
    { 0x3d002332, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f402336, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e0233c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe02340, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9602349, SUC_lexicon_tags_47 },
    { 0xf3e0234a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca02350, SUC_lexicon_tags_8 },
    { 0xd7002351, SUC_lexicon_tags_45 },
    { 0xf5802350, SUC_lexicon_tags_16 },
    { 0x8a402353, SUC_lexicon_tags_66 },
    { 0x35e02350, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3e02356, SUC_lexicon_tags_12 },
    { 0xf9e02357, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcee02359, SUC_lexicon_tags_1 },
    { 0x80802359, SUC_lexicon_tags_7 },
    { 0x6480235a, SUC_lexicon_tags_19 },
    { 0x17e0235c, SUC_lexicon_tags_37 },
    { 0xb2a0235c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e402368, SUC_lexicon_tags_23 },
    { 0xd7002369, SUC_lexicon_tags_26 },
    { 0xe4a02369, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe60236c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31c0236e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41802371, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b802377, SUC_lexicon_tags_25 },
    { 0xa6c02377, SUC_lexicon_tags_2 },
    { 0x67002377, SUC_lexicon_tags_37 },
    { 0xfb602377, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a0237d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x63002380, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x43a02382, SUC_lexicon_tags_45 },
    { 0xe3002383, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1802385, SUC_lexicon_tags_19 },
    { 0xfaa02386, SUC_lexicon_tags_27 },
    { 0x97402385, SUC_lexicon_tags_27 },
    { 0x2e802388, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa640238c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb402390, SUC_lexicon_tags_37 },
    { 0x54202391, SUC_lexicon_tags_19 },
    { 0xecc02391, SUC_lexicon_tags_12 },
    { 0x6c02393, SUC_lexicon_tags_26 },
    { 0xe5e02393, SUC_lexicon_tags_7 },
    { 0xa4e02394, SUC_lexicon_tags_46 },
    { 0x57002395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b602398, SUC_lexicon_tags_11 },
    { 0xa2a02399, SUC_lexicon_tags_1 },
    { 0x52c02398, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba0239d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x412023a0, SUC_lexicon_tags_2 },
    { 0x356023a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x496023a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce2023a5, SUC_lexicon_tags_12 },
    { 0xf74023a5, SUC_lexicon_tags_1 },
    { 0x448023a5, SUC_lexicon_tags_17 },
    { 0x7b0023a7, SUC_lexicon_tags_10 },
    { 0xe34023a7, SUC_lexicon_tags_17 },
    { 0xad4023a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ce023ac, SUC_lexicon_tags_37 },
    { 0xd4a023ac, SUC_lexicon_tags_45 },
    { 0x840023ae, SUC_lexicon_tags_2 },
    { 0x244023af, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xfa4023b1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe40023b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712023b9, SUC_lexicon_tags_25 },
    { 0xfb8023ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48a023bc, SUC_lexicon_tags_33 },
    { 0x198023bd, SUC_lexicon_tags_2 },
    { 0xcdc023bc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32023c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee023c6, SUC_lexicon_tags_17 },
    { 0x524023c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0023cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e8023cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8e023d1, SUC_lexicon_tags_48 },
    { 0xd7a023d2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x60023d5, SUC_lexicon_tags_66 },
    { 0x1c6023d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a023de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa023e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e2023e4, SUC_lexicon_tags_45 },
    { 0xc96023e5, SUC_lexicon_tags_30 },
    { 0x2fc023e6, SUC_lexicon_tags_1 },
    { 0x87a023e6, SUC_lexicon_tags_10 },
    { 0x456023e5, SUC_lexicon_tags_79 },
    { 0x6a8023e6, SUC_lexicon_tags_33 },
    { 0xc14023ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa4023ec, SUC_lexicon_tags_16 },
    { 0xcec023ec, SUC_lexicon_tags_40 },
    { 0x284023ed, SUC_lexicon_tags_35 },
    { 0x47c023ef, SUC_lexicon_tags_31 },
    { 0xdb4023ef, SUC_lexicon_tags_15 },
    { 0x4be023f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a023fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27802401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4002403, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7700240b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c202411, SUC_lexicon_tags_15 },
    { 0x67602411, SUC_lexicon_tags_15 },
    { 0x2ce02413, SUC_lexicon_tags_0 },
    { 0x5a802414, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e0241b, SUC_lexicon_tags_34 },
    { 0xe760241c, SUC_lexicon_tags_64 },
    { 0x9360241d, SUC_lexicon_tags_4 },
    { 0x2500241b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x30602420, SUC_lexicon_tags_43 },
    { 0x88202421, SUC_lexicon_tags_96 },
    { 0xe3802421, SUC_lexicon_tags_61 },
    { 0x6d602421, SUC_lexicon_tags_2 },
    { 0x62c02424, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x99002426, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c02429, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ec0242b, SUC_lexicon_tags_7 },
    { 0x7ea0242c, SUC_lexicon_tags_16 },
    { 0x1f40242d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32002433, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e02436, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xbba02438, SUC_lexicon_tags_11 },
    { 0xe8e02439, SUC_lexicon_tags_27 },
    { 0x2c002439, SUC_lexicon_tags_47 },
    { 0x6da0243b, SUC_lexicon_tags_11 },
    { 0x4e00243a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c02441, SUC_lexicon_tags_11 },
    { 0xe2402442, SUC_lexicon_tags_17 },
    { 0x57002443, SUC_lexicon_tags_30 },
    { 0x85602444, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140244e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x64e02450, SUC_lexicon_tags_47 },
    { 0x93002451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b802453, SUC_lexicon_tags_36 },
    { 0x79402454, SUC_lexicon_tags_1 },
    { 0x91a02454, SUC_lexicon_tags_45 },
    { 0x4a402454, SUC_lexicon_tags_55 },
    { 0x69c02454, SUC_lexicon_tags_11 },
    { 0xf7a02458, SUC_lexicon_tags_7 },
    { 0x3fa02459, SUC_lexicon_tags_69 },
    { 0x3a002459, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b00245c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7240245e, SUC_lexicon_tags_37 },
    { 0x9fa0245f, SUC_lexicon_tags_49 },
    { 0x2b002460, SUC_lexicon_tags_84 },
    { 0x9802460, SUC_lexicon_tags_12 },
    { 0x6ee02462, SUC_lexicon_tags_8 },
    { 0xf440245f, SUC_lexicon_tags_1 },
    { 0xe2802463, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc02468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc980246a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa300246d, SUC_lexicon_tags_1 },
    { 0xeda0246d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86402470, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac02476, SUC_lexicon_tags_17 },
    { 0x9ae02477, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea0247a, SUC_lexicon_tags_2 },
    { 0xfe80247b, SUC_lexicon_tags_7 },
    { 0x90a0247c, SUC_lexicon_tags_25 },
    { 0xa1a0247d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb60247f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8802485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa02488, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e0248f, SUC_lexicon_tags_4 },
    { 0x6f802490, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81202494, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9602496, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc02499, SUC_lexicon_tags_15 },
    { 0x4002499, SUC_lexicon_tags_17 },
    { 0xd2a0249b, SUC_lexicon_tags_8 },
    { 0x24c0249c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc240249f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26024a2, SUC_lexicon_tags_12 },
    { 0x29e024a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a024a7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x81e024a9, SUC_lexicon_tags_118 },
    { 0x62a024aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa7e024ac, SUC_lexicon_tags_1 },
    { 0xefc024ac, SUC_lexicon_tags_28 },
    { 0x3d6024ae, SUC_lexicon_tags_0 },
    { 0x4ee024ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6024b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2024bb, SUC_lexicon_tags_25 },
    { 0x954024bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0024c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a024c3, SUC_lexicon_tags_234 },
    { 0xb52024c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x752024c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa0a024c8, SUC_lexicon_tags_33 },
    { 0x146024c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40024cc, SUC_lexicon_tags_87 },
    { 0x17c024cd, SUC_lexicon_tags_11 },
    { 0xa4e024cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6024d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x404024d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf0024d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be024df, SUC_lexicon_tags_15 },
    { 0xf92024e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e024e3, SUC_lexicon_tags_15 },
    { 0x440024e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e024e7, SUC_lexicon_tags_11 },
    { 0xf1e024e8, SUC_lexicon_tags_43 },
    { 0xf60024e8, SUC_lexicon_tags_21 },
    { 0x478024e9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6024ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4024f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82024f7, SUC_lexicon_tags_30 },
    { 0x538024f8, SUC_lexicon_tags_11 },
    { 0x994024f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3202500, SUC_lexicon_tags_951 },
    { 0x60e02500, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3802507, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a0250d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b402511, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20402514, SUC_lexicon_tags_61 },
    { 0xc2402515, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf202517, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f80251f, SUC_lexicon_tags_172 },
    { 0x84e02520, SUC_lexicon_tags_11 },
    { 0x6c402520, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c602527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50802529, SUC_lexicon_tags_11 },
    { 0xcba0252a, SUC_lexicon_tags_42 },
    { 0x2780252b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee602530, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b002532, SUC_lexicon_tags_25 },
    { 0xf1002533, SUC_lexicon_tags_12 },
    { 0xfa802534, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c802536, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60602538, SUC_lexicon_tags_8 },
    { 0xb8e02538, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c0253c, SUC_lexicon_tags_21 },
    { 0x1b40253c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4c0253f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56e02541, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0802543, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd7802545, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xec02548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d402553, SUC_lexicon_tags_42 },
    { 0x48602554, SUC_lexicon_tags_45 },
    { 0x4e602555, SUC_lexicon_tags_68 },
    { 0x5fc02556, SUC_lexicon_tags_3 },
    { 0x73602555, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc0255d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc60255f, SUC_lexicon_tags_43 },
    { 0x27e02560, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8802563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce0256b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda0256f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x55602571, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4402574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75602579, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1940257c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d60257f, SUC_lexicon_tags_11 },
    { 0x62e02580, SUC_lexicon_tags_48 },
    { 0x32602581, SUC_lexicon_tags_1 },
    { 0x38802582, SUC_lexicon_tags_30 },
    { 0xcd602583, SUC_lexicon_tags_121 },
    { 0x3d002582, SUC_lexicon_tags_39 },
    { 0xa3402585, SUC_lexicon_tags_15 },
    { 0x2d802586, SUC_lexicon_tags_48 },
    { 0x2ba02586, SUC_lexicon_tags_63 },
    { 0x32c02584, SUC_lexicon_tags_37 },
    { 0x4ea02588, SUC_lexicon_tags_25 },
    { 0xba002587, SUC_lexicon_tags_27 },
    { 0xcf602585, SUC_lexicon_tags_1 },
    { 0xf120258c, SUC_lexicon_tags_63 },
    { 0xcfc0258d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7002592, SUC_lexicon_tags_2 },
    { 0x16c02593, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x51402596, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e0259e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e4025a0, SUC_lexicon_tags_38 },
    { 0x8f6025a1, SUC_lexicon_tags_1 },
    { 0x58e025a2, SUC_lexicon_tags_3 },
    { 0xa8025a2, SUC_lexicon_tags_17 },
    { 0xeea025a2, SUC_lexicon_tags_30 },
    { 0x5aa025a5, SUC_lexicon_tags_174 },
    { 0xf2a025a5, SUC_lexicon_tags_26 },
    { 0x4f4025a5, SUC_lexicon_tags_49 },
    { 0x90c025a8, SUC_lexicon_tags_27 },
    { 0x362025a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x878025ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8025b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334025b6, SUC_lexicon_tags_12 },
    { 0x8fc025b7, SUC_lexicon_tags_37 },
    { 0x458025b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8025be, SUC_lexicon_tags_6 },
    { 0x9b6025be, SUC_lexicon_tags_11 },
    { 0x662025c0, SUC_lexicon_tags_1 },
    { 0xd24025c1, SUC_lexicon_tags_105 },
    { 0xf8e025bf, SUC_lexicon_tags_11 },
    { 0x54c025be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74025c8, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a025d1, SUC_lexicon_tags_16 },
    { 0xf70025d2, SUC_lexicon_tags_14 },
    { 0x3c025d3, SUC_lexicon_tags_14 },
    { 0x614025d3, SUC_lexicon_tags_2 },
    { 0xd2c025d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e025de, SUC_lexicon_tags_17 },
    { 0xbf2025de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40025e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x956025e6, SUC_lexicon_tags_7 },
    { 0xe8e025e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc025ea, SUC_lexicon_tags_45 },
    { 0x800025eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74025f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6025fb, SUC_lexicon_tags_37 },
    { 0x8e0025fc, SUC_lexicon_tags_0 },
    { 0x442025fc, SUC_lexicon_tags_49 },
    { 0x6ba025fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81202601, SUC_lexicon_tags_17 },
    { 0x5c802602, SUC_lexicon_tags_33 },
    { 0x13002603, SUC_lexicon_tags_12 },
    { 0x36202603, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a02607, SUC_lexicon_tags_114 },
    { 0xb6a02607, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980260f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a02619, SUC_lexicon_tags_25 },
    { 0xd7802619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a20261e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45402622, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e02624, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c02627, SUC_lexicon_tags_63 },
    { 0x6a802627, SUC_lexicon_tags_11 },
    { 0xbbe02629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a0262e, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3602632, SUC_lexicon_tags_2 },
    { 0x42c02632, SUC_lexicon_tags_1 },
    { 0x62402634, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x79202637, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc02640, SUC_lexicon_tags_12 },
    { 0x44202641, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc360264a, SUC_lexicon_tags_17 },
    { 0x3d40264a, SUC_lexicon_tags_0 },
    { 0x1e0264b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe0264f, SUC_lexicon_tags_11 },
    { 0x4500264f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96202656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2802659, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc0265d, SUC_lexicon_tags_15 },
    { 0xd7a0265d, SUC_lexicon_tags_2 },
    { 0x5d80265f, SUC_lexicon_tags_25 },
    { 0xc8002660, SUC_lexicon_tags_49 },
    { 0xd3a02661, SUC_lexicon_tags_37 },
    { 0xc02662, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8002667, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca0266a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e02670, SUC_lexicon_tags_28 },
    { 0xb6e02671, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa9602673, SUC_lexicon_tags_11 },
    { 0x9ea02674, SUC_lexicon_tags_15 },
    { 0x77402675, SUC_lexicon_tags_4 },
    { 0xd9802674, SUC_lexicon_tags_12 },
    { 0x85602677, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a0267a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25c0267c, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa0267f, SUC_lexicon_tags_26 },
    { 0x48c0267f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xff602682, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1402689, SUC_lexicon_tags_14 },
    { 0x13a0268a, SUC_lexicon_tags_17 },
    { 0x7700268b, SUC_lexicon_tags_19 },
    { 0xe3202689, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa80268e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x64c02690, SUC_lexicon_tags_80 },
    { 0x90602690, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x35c02693, SUC_lexicon_tags_14 },
    { 0x2002694, SUC_lexicon_tags_11 },
    { 0xfea02694, SUC_lexicon_tags_37 },
    { 0xe202696, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf560269b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca026a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6026a5, SUC_lexicon_tags_34 },
    { 0xa5a026a6, SUC_lexicon_tags_7 },
    { 0x156026a6, SUC_lexicon_tags_66 },
    { 0xc88026a8, SUC_lexicon_tags_11 },
    { 0x8da026a6, SUC_lexicon_tags_17 },
    { 0x57c026aa, SUC_lexicon_tags_11 },
    { 0x428026a5, SUC_lexicon_tags_48 },
    { 0x390026a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc026af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78e026b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8026b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x382026b7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522026be, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4026c2, SUC_lexicon_tags_12 },
    { 0x3d4026c3, SUC_lexicon_tags_1 },
    { 0x8d2026c2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84026cc, SUC_lexicon_tags_84 },
    { 0xdd2026cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8026d1, SUC_lexicon_tags_25 },
    { 0xf4e026d2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6026d6, SUC_lexicon_tags_37 },
    { 0xc06026d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x626026d9, SUC_lexicon_tags_37 },
    { 0x3ae026da, SUC_lexicon_tags_7 },
    { 0x362026da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e026de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626026e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46026eb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf12026ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x590026f0, SUC_lexicon_tags_12 },
    { 0x5e026f1, SUC_lexicon_tags_46 },
    { 0xaba026f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f2026f4, SUC_lexicon_tags_29 },
    { 0x734026f4, SUC_lexicon_tags_2 },
    { 0x70026f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11a026f8, SUC_lexicon_tags_29 },
    { 0x45e026f9, SUC_lexicon_tags_2 },
    { 0x89c026fa, SUC_lexicon_tags_2 },
    { 0xa72026fb, SUC_lexicon_tags_82 },
    { 0xe84026f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee02703, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95e02705, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x12a02707, SUC_lexicon_tags_9 },
    { 0xa0402708, SUC_lexicon_tags_7 },
    { 0xe3202709, SUC_lexicon_tags_15 },
    { 0x82202709, SUC_lexicon_tags_19 },
    { 0x16a0270a, SUC_lexicon_tags_11 },
    { 0x4ae02708, SUC_lexicon_tags_15 },
    { 0x140270b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11802716, SUC_lexicon_tags_30 },
    { 0x2ca02716, SUC_lexicon_tags_37 },
    { 0x96002718, SUC_lexicon_tags_18 },
    { 0x1a202716, SUC_lexicon_tags_8 },
    { 0xf880271a, SUC_lexicon_tags_26 },
    { 0x9c40271b, SUC_lexicon_tags_37 },
    { 0x15a02716, SUC_lexicon_tags_11 },
    { 0x83c02717, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a02720, SUC_lexicon_tags_22 },
    { 0x4b202721, SUC_lexicon_tags_25 },
    { 0x54402722, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe02726, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4680272a, SUC_lexicon_tags_47 },
    { 0xf220272b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ca0272d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x16402730, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee02736, SUC_lexicon_tags_12 },
    { 0x55c02736, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae0273e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec02743, SUC_lexicon_tags_14 },
    { 0x57602744, SUC_lexicon_tags_12 },
    { 0xc3c02745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26802748, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xadc0274a, SUC_lexicon_tags_11 },
    { 0xaa20274a, SUC_lexicon_tags_11 },
    { 0x4ca0274c, SUC_lexicon_tags_48 },
    { 0x8b40274d, SUC_lexicon_tags_37 },
    { 0x52a0274e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a02753, SUC_lexicon_tags_60 },
    { 0x1e802753, SUC_lexicon_tags_37 },
    { 0xb5602755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe0275f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f202762, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc160276a, SUC_lexicon_tags_28 },
    { 0x2260276a, SUC_lexicon_tags_31 },
    { 0xb8c0276a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb602772, SUC_lexicon_tags_37 },
    { 0x1002773, SUC_lexicon_tags_25 },
    { 0x4e02774, SUC_lexicon_tags_12 },
    { 0x48e02775, SUC_lexicon_tags_15 },
    { 0x4d602776, SUC_lexicon_tags_30 },
    { 0x9d802776, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ea02779, SUC_lexicon_tags_27 },
    { 0x74e0277a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf240277c, SUC_lexicon_tags_23 },
    { 0x89e0277c, SUC_lexicon_tags_48 },
    { 0xef40277d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a02781, SUC_lexicon_tags_38 },
    { 0x16002781, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99e02784, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac00278b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15802793, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99e02795, SUC_lexicon_tags_27 },
    { 0x4ac02796, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb260279f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6027a6, SUC_lexicon_tags_7 },
    { 0xfd4027a7, SUC_lexicon_tags_43 },
    { 0x6ae027a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84027b0, SUC_lexicon_tags_11 },
    { 0x4a8027b0, SUC_lexicon_tags_37 },
    { 0x422027b1, SUC_lexicon_tags_0 },
    { 0xae2027b3, SUC_lexicon_tags_47 },
    { 0x89c027b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598027b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb6027ba, SUC_lexicon_tags_17 },
    { 0xf72027bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a027be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622027c5, SUC_lexicon_tags_7 },
    { 0xc52027c5, SUC_lexicon_tags_63 },
    { 0x862027c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2027cf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6027d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0027d9, SUC_lexicon_tags_11 },
    { 0xda2027d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6027e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8027e3, SUC_lexicon_tags_1 },
    { 0x94c027e4, SUC_lexicon_tags_15 },
    { 0x9ea027e5, SUC_lexicon_tags_7 },
    { 0x6c0027e6, SUC_lexicon_tags_65 },
    { 0x5ea027e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766027eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x114027ed, SUC_lexicon_tags_2 },
    { 0x156027ee, SUC_lexicon_tags_40 },
    { 0xbda027ee, SUC_lexicon_tags_17 },
    { 0xb5e027ef, SUC_lexicon_tags_43 },
    { 0xd9c027ef, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x406027f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafe027f5, SUC_lexicon_tags_31 },
    { 0xd20027f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56027fb, SUC_lexicon_tags_4 },
    { 0xa10027fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17002803, SUC_lexicon_tags_33 },
    { 0x9b002804, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c02808, SUC_lexicon_tags_17 },
    { 0x6e602809, SUC_lexicon_tags_12 },
    { 0x19602808, SUC_lexicon_tags_23 },
    { 0x63a02809, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a0280e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde802813, SUC_lexicon_tags_1 },
    { 0x80202814, SUC_lexicon_tags_29 },
    { 0x73002815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c0281a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d40281c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16602821, SUC_lexicon_tags_0 },
    { 0x55602822, SUC_lexicon_tags_38 },
    { 0xd6602823, SUC_lexicon_tags_37 },
    { 0x81802823, SUC_lexicon_tags_25 },
    { 0xfc802822, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e002829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbde0282b, SUC_lexicon_tags_29 },
    { 0xd320282c, SUC_lexicon_tags_38 },
    { 0xaac0282c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca02832, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd802834, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4c02836, SUC_lexicon_tags_25 },
    { 0x59002836, SUC_lexicon_tags_1 },
    { 0xbce02838, SUC_lexicon_tags_43 },
    { 0x64602838, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea0283f, SUC_lexicon_tags_37 },
    { 0xb1602840, SUC_lexicon_tags_25 },
    { 0x4a602841, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52a02843, SUC_lexicon_tags_45 },
    { 0x67802844, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98c02846, SUC_lexicon_tags_17 },
    { 0x7ec02847, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00284a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7120284c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe0284e, SUC_lexicon_tags_19 },
    { 0x5be0284f, SUC_lexicon_tags_6 },
    { 0x68e02850, SUC_lexicon_tags_14 },
    { 0x76602851, SUC_lexicon_tags_84 },
    { 0x8d202852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd202855, SUC_lexicon_tags_12 },
    { 0x3a602856, SUC_lexicon_tags_31 },
    { 0x94202857, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4780285a, SUC_lexicon_tags_14 },
    { 0xd1c0285a, SUC_lexicon_tags_17 },
    { 0xa740285c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f402860, SUC_lexicon_tags_43 },
    { 0x76a02861, SUC_lexicon_tags_25 },
    { 0x17602860, SUC_lexicon_tags_65 },
    { 0x81c02860, SUC_lexicon_tags_25 },
    { 0x5d202864, SUC_lexicon_tags_11 },
    { 0xe9e02860, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54602867, SUC_lexicon_tags_68 },
    { 0xb0002868, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb20286f, SUC_lexicon_tags_30 },
    { 0x71c02870, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x4d202872, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1202878, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb040287d, SUC_lexicon_tags_17 },
    { 0x7e40287e, SUC_lexicon_tags_11 },
    { 0xb4e0287f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7300288e, SUC_lexicon_tags_11 },
    { 0x54c0288f, SUC_lexicon_tags_70 },
    { 0x4a002890, SUC_lexicon_tags_17 },
    { 0x48402890, SUC_lexicon_tags_19 },
    { 0xe6202890, SUC_lexicon_tags_30 },
    { 0x96002891, SUC_lexicon_tags_65 },
    { 0x5f202891, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57c02896, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xff402898, SUC_lexicon_tags_3 },
    { 0x7ca02899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1760289b, SUC_lexicon_tags_12 },
    { 0x4c60289c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe900289e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x396028a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08028a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x20e028ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca028af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x286028b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53e028b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42028b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310028c0, SUC_lexicon_tags_30 },
    { 0xc36028c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8028c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x378028c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e6028cb, SUC_lexicon_tags_12 },
    { 0x8ae028cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e028cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6028d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb38028d4, SUC_lexicon_tags_14 },
    { 0xade028d4, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278028da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c028de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98028e5, SUC_lexicon_tags_23 },
    { 0xa04028e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a4028e8, SUC_lexicon_tags_7 },
    { 0x200028e9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x154028eb, SUC_lexicon_tags_46 },
    { 0x662028ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc028f0, SUC_lexicon_tags_84 },
    { 0xabc028f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33a028f3, SUC_lexicon_tags_25 },
    { 0x280028f3, SUC_lexicon_tags_6 },
    { 0xa8c028f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe4e028f7, SUC_lexicon_tags_11 },
    { 0x3a028f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99a028fa, SUC_lexicon_tags_40 },
    { 0x61c028fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a802905, SUC_lexicon_tags_91 },
    { 0x68e02905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0c02908, SUC_lexicon_tags_1 },
    { 0x88c02909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed802910, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3d202912, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e0291b, SUC_lexicon_tags_27 },
    { 0x14e0291b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c02922, SUC_lexicon_tags_17 },
    { 0x36a02923, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc602925, SUC_lexicon_tags_26 },
    { 0x1aa02925, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe340292a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e02931, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e02935, SUC_lexicon_tags_17 },
    { 0x77a02936, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xd2e02938, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe5e0293a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd540293f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91602941, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c02943, SUC_lexicon_tags_37 },
    { 0x13402944, SUC_lexicon_tags_37 },
    { 0x2a402945, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa02948, SUC_lexicon_tags_47 },
    { 0x51802948, SUC_lexicon_tags_43 },
    { 0xcec02949, SUC_lexicon_tags_38 },
    { 0x38e0294b, SUC_lexicon_tags_37 },
    { 0x9c00294c, SUC_lexicon_tags_11 },
    { 0x4180294d, SUC_lexicon_tags_26 },
    { 0xdcc0294d, SUC_lexicon_tags_82 },
    { 0x23e0294c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc02952, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca02955, SUC_lexicon_tags_27 },
    { 0x1b002956, SUC_lexicon_tags_12 },
    { 0x34002956, SUC_lexicon_tags_21 },
    { 0xa8202958, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97a0295a, SUC_lexicon_tags_26 },
    { 0x8a0295a, SUC_lexicon_tags_36 },
    { 0x86a0295a, SUC_lexicon_tags_17 },
    { 0x98c0295d, SUC_lexicon_tags_85 },
    { 0xb2c0295e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e02967, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7602969, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d00296b, SUC_lexicon_tags_47 },
    { 0xece0296b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91602970, SUC_lexicon_tags_12 },
    { 0xd4802971, SUC_lexicon_tags_15 },
    { 0xc2002971, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7b602974, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65802978, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00297c, SUC_lexicon_tags_37 },
    { 0x5000297d, SUC_lexicon_tags_17 },
    { 0xef20297e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b602980, SUC_lexicon_tags_14 },
    { 0x45402981, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54402987, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e0298b, SUC_lexicon_tags_1 },
    { 0x7180298b, SUC_lexicon_tags_11 },
    { 0x46a0298b, SUC_lexicon_tags_25 },
    { 0x84a0298e, SUC_lexicon_tags_25 },
    { 0x1ee0298e, SUC_lexicon_tags_33 },
    { 0x87802990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c02993, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x31002996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82402998, SUC_lexicon_tags_11 },
    { 0x55e02998, SUC_lexicon_tags_12 },
    { 0x48002999, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba0299d, SUC_lexicon_tags_25 },
    { 0xec00299e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe8029a0, SUC_lexicon_tags_6 },
    { 0x444029a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e029a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c029a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f0029ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde4029ad, SUC_lexicon_tags_26 },
    { 0x1f2029ae, SUC_lexicon_tags_33 },
    { 0x9e6029af, SUC_lexicon_tags_10 },
    { 0x24e029b0, SUC_lexicon_tags_32 },
    { 0xf06029b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac0029b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x886029b5, SUC_lexicon_tags_25 },
    { 0x984029b5, SUC_lexicon_tags_2 },
    { 0x574029b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36029bd, SUC_lexicon_tags_19 },
    { 0x702029be, SUC_lexicon_tags_15 },
    { 0xa28029bd, SUC_lexicon_tags_65 },
    { 0x2e2029c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14c029c2, SUC_lexicon_tags_17 },
    { 0xb36029c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x892029c6, SUC_lexicon_tags_23 },
    { 0x918029c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae029cb, SUC_lexicon_tags_7 },
    { 0xda8029cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4029d1, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0029d4, SUC_lexicon_tags_30 },
    { 0x7c029d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff8029d7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6029da, SUC_lexicon_tags_12 },
    { 0x562029da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c029df, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2029e6, SUC_lexicon_tags_47 },
    { 0xc42029e7, SUC_lexicon_tags_25 },
    { 0x9e4029e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa9a029ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62029ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27c029ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc029f2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e029f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430029ff, SUC_lexicon_tags_11 },
    { 0x5c202a00, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa002a02, SUC_lexicon_tags_15 },
    { 0xcc02a03, SUC_lexicon_tags_25 },
    { 0xa9402a04, SUC_lexicon_tags_19 },
    { 0x5e002a04, SUC_lexicon_tags_55 },
    { 0x23402a06, SUC_lexicon_tags_17 },
    { 0x87a02a02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1202a0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c02a0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa402a10, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a02a14, SUC_lexicon_tags_25 },
    { 0x76802a15, SUC_lexicon_tags_26 },
    { 0x1ce02a16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d402a1a, SUC_lexicon_tags_31 },
    { 0xb1602a1b, SUC_lexicon_tags_34 },
    { 0x8c402a1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70c02a1e, SUC_lexicon_tags_1 },
    { 0x94c02a1f, SUC_lexicon_tags_27 },
    { 0x1a402a1f, SUC_lexicon_tags_160 },
    { 0x10802a21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f602a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc02a27, SUC_lexicon_tags_65 },
    { 0xe8602a28, SUC_lexicon_tags_520 },
    { 0x74a02a29, SUC_lexicon_tags_79 },
    { 0x7d202a28, SUC_lexicon_tags_1 },
    { 0x2fe02a2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e02a35, SUC_lexicon_tags_47 },
    { 0x89802a36, SUC_lexicon_tags_23 },
    { 0x5b802a37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd602a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c02a3d, SUC_lexicon_tags_106 },
    { 0xcdc02a3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98202a44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc02a49, SUC_lexicon_tags_7 },
    { 0x72002a4a, SUC_lexicon_tags_1 },
    { 0x6da02a4b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x54a02a4d, SUC_lexicon_tags_65 },
    { 0x88002a4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa02a53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5402a55, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x73602a57, SUC_lexicon_tags_39 },
    { 0xd6202a58, SUC_lexicon_tags_17 },
    { 0x64a02a58, SUC_lexicon_tags_25 },
    { 0x3c402a59, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e602a5c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x25e02a5e, SUC_lexicon_tags_17 },
    { 0xf8602a5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae402a61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa802a63, SUC_lexicon_tags_12 },
    { 0x43802a64, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a402a6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63e02a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc02a73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x88602a76, SUC_lexicon_tags_47 },
    { 0x63c02a77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c02a7a, SUC_lexicon_tags_8 },
    { 0xbe02a7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8e02a7d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d202a81, SUC_lexicon_tags_45 },
    { 0x5f602a82, SUC_lexicon_tags_6 },
    { 0xd5802a83, SUC_lexicon_tags_12 },
    { 0x7b602a82, SUC_lexicon_tags_12 },
    { 0x2b402a85, SUC_lexicon_tags_11 },
    { 0x7ba02a82, SUC_lexicon_tags_15 },
    { 0xbba02a82, SUC_lexicon_tags_15 },
    { 0x90202a88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2202a8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10e02a90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42802a93, SUC_lexicon_tags_23 },
    { 0x89c02a94, SUC_lexicon_tags_45 },
    { 0x88402a95, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e002a99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c02aa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbe02aa7, SUC_lexicon_tags_15 },
    { 0xb202aa8, SUC_lexicon_tags_12 },
    { 0x8802aa7, SUC_lexicon_tags_12 },
    { 0x2aa02aaa, SUC_lexicon_tags_26 },
    { 0x9a402aab, SUC_lexicon_tags_37 },
    { 0xcac02aac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdc02aae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8202ab3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe5a02ab5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x70e02ab7, SUC_lexicon_tags_1 },
    { 0xb3602ab8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22e02aba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc202ac0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9802ac2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c02ac5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6002aca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac02acf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a202ad1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a02ad9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb9002adb, SUC_lexicon_tags_12 },
    { 0xd9002adc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3402ae0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8e02ae2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8002ae8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee002aec, SUC_lexicon_tags_92 },
    { 0xa6e02aed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a02af0, SUC_lexicon_tags_45 },
    { 0x79a02af1, SUC_lexicon_tags_27 },
    { 0x4b802af1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e02af8, SUC_lexicon_tags_60 },
    { 0x88402af9, SUC_lexicon_tags_14 },
    { 0x3c602af9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8402afc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2802aff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe02b02, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1402b05, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a02b08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23802b19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4ac02b1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6002b1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80402b1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1ac02b21, SUC_lexicon_tags_11 },
    { 0x39c02b22, SUC_lexicon_tags_30 },
    { 0x54202b23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29202b27, SUC_lexicon_tags_12 },
    { 0xe3c02b28, SUC_lexicon_tags_55 },
    { 0xc8402b28, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x17802b2c, SUC_lexicon_tags_12 },
    { 0xe4802b2d, SUC_lexicon_tags_255 },
    { 0xe1e02b2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c202b32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ce02b34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa3002b36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x64202b38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa02b3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6be02b3e, SUC_lexicon_tags_19 },
    { 0xccc02b3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd202b41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a02b48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea02b4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x26402b4d, SUC_lexicon_tags_25 },
    { 0xc6602b4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33002b50, SUC_lexicon_tags_6 },
    { 0x78a02b51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e02b55, SUC_lexicon_tags_17 },
    { 0x34602b56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd402b58, SUC_lexicon_tags_11 },
    { 0xd7402b59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63202b64, SUC_lexicon_tags_39 },
    { 0x40002b65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d002b67, SUC_lexicon_tags_17 },
    { 0x98202b68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95402b6b, SUC_lexicon_tags_12 },
    { 0xeea02b6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a02b70, SUC_lexicon_tags_11 },
    { 0xfa02b71, SUC_lexicon_tags_11 },
    { 0xf0c02b72, SUC_lexicon_tags_23 },
    { 0xbc002b73, SUC_lexicon_tags_25 },
    { 0xc5602b72, SUC_lexicon_tags_271 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2802b7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53402b7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c02b83, SUC_lexicon_tags_2 },
    { 0x6e602b84, SUC_lexicon_tags_11 },
    { 0x6d402b83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c02b88, SUC_lexicon_tags_25 },
    { 0x1a802b88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90602b8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a202b90, SUC_lexicon_tags_10 },
    { 0x94602b90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bc02b93, SUC_lexicon_tags_14 },
    { 0xa3802b94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5c02b96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40002b9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e02ba1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50802ba4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea02ba7, SUC_lexicon_tags_50 },
    { 0x99c02ba8, SUC_lexicon_tags_15 },
    { 0x2a402ba9, SUC_lexicon_tags_282 },
    { 0xe1202ba9, SUC_lexicon_tags_2 },
    { 0x3f402ba9, SUC_lexicon_tags_7 },
    { 0xe1a02bac, SUC_lexicon_tags_0 },
    { 0xb8802bad, SUC_lexicon_tags_38 },
    { 0x49002baa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece02bb6, SUC_lexicon_tags_15 },
    { 0x1f402bb7, SUC_lexicon_tags_12 },
    { 0xe0402bb6, SUC_lexicon_tags_3 },
    { 0xa5e02bb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54202bbb, SUC_lexicon_tags_21 },
    { 0x7fa02bbc, SUC_lexicon_tags_12 },
    { 0x4c002bbd, SUC_lexicon_tags_23 },
    { 0xe9a02bbb, SUC_lexicon_tags_51 },
    { 0xce402bbf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e002bc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02bcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79402bd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f002bd2, SUC_lexicon_tags_39 },
    { 0x9a402bd3, SUC_lexicon_tags_60 },
    { 0x1fc02bd3, SUC_lexicon_tags_1 },
    { 0x6b202bd4, SUC_lexicon_tags_2 },
    { 0x4fe02bd3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac02bde, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a02be1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec02be6, SUC_lexicon_tags_17 },
    { 0x81202be6, SUC_lexicon_tags_17 },
    { 0x13802be8, SUC_lexicon_tags_2 },
    { 0xfa202be9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55802beb, SUC_lexicon_tags_55 },
    { 0x6d002bec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a02bef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93802bf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f402bf5, SUC_lexicon_tags_27 },
    { 0xc0c02bf5, SUC_lexicon_tags_38 },
    { 0x10602bf5, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xb0402bf9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4d002bfb, SUC_lexicon_tags_3 },
    { 0xaa602bfc, SUC_lexicon_tags_15 },
    { 0x73602bfd, SUC_lexicon_tags_55 },
    { 0xe4002bfd, SUC_lexicon_tags_34 },
    { 0x2dc02bff, SUC_lexicon_tags_37 },
    { 0x37602c00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73c02c02, SUC_lexicon_tags_27 },
    { 0xaaa02c03, SUC_lexicon_tags_11 },
    { 0x8ce02c04, SUC_lexicon_tags_107 },
    { 0xf1c02c05, SUC_lexicon_tags_1 },
    { 0xc7802c06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56802c0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc02c11, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x38202c13, SUC_lexicon_tags_12 },
    { 0x6d802c14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e02c18, SUC_lexicon_tags_1 },
    { 0x82802c18, SUC_lexicon_tags_4 },
    { 0xd0c02c1a, SUC_lexicon_tags_1 },
    { 0x62402c1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c02c24, SUC_lexicon_tags_25 },
    { 0xd5602c25, SUC_lexicon_tags_12 },
    { 0x5ce02c26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7a02c28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbfe02c2a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c402c31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa02c39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf1202c3b, SUC_lexicon_tags_30 },
    { 0x36202c3c, SUC_lexicon_tags_25 },
    { 0xe4802c3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x10002c3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67202c42, SUC_lexicon_tags_7 },
    { 0xe0002c43, SUC_lexicon_tags_29 },
    { 0x47602c43, SUC_lexicon_tags_9 },
    { 0x9a002c43, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa9e02c47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a02c4a, SUC_lexicon_tags_16 },
    { 0x52802c4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea02c51, SUC_lexicon_tags_11 },
    { 0xd7a02c52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7aa02c54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a202c58, SUC_lexicon_tags_2 },
    { 0x17a02c58, SUC_lexicon_tags_2 },
    { 0xaa602c5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f202c5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeec02c5e, SUC_lexicon_tags_21 },
    { 0xe4802c5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27202c62, SUC_lexicon_tags_6 },
    { 0x6be02c62, SUC_lexicon_tags_11 },
    { 0x25a02c64, SUC_lexicon_tags_15 },
    { 0x35602c64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3802c69, SUC_lexicon_tags_25 },
    { 0x52402c6a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8402c6d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7602c73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6e02c75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e402c77, SUC_lexicon_tags_17 },
    { 0x6dc02c78, SUC_lexicon_tags_11 },
    { 0xdfc02c78, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc02c80, SUC_lexicon_tags_17 },
    { 0x32602c80, SUC_lexicon_tags_68 },
    { 0xc1202c82, SUC_lexicon_tags_60 },
    { 0xa4e02c82, SUC_lexicon_tags_2 },
    { 0xf5602c84, SUC_lexicon_tags_11 },
    { 0xf5602c83, SUC_lexicon_tags_38 },
    { 0xad602c80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42402c88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ac02c8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfc002c8c, SUC_lexicon_tags_17 },
    { 0x3b402c8c, SUC_lexicon_tags_23 },
    { 0xf9802c8e, SUC_lexicon_tags_26 },
    { 0x3a602c8e, SUC_lexicon_tags_37 },
    { 0x84e02c8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad402c94, SUC_lexicon_tags_12 },
    { 0xa3002c95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25e02c97, SUC_lexicon_tags_2 },
    { 0x86202c98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68202c9a, SUC_lexicon_tags_0 },
    { 0x8fe02c9b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x60002c9e, SUC_lexicon_tags_66 },
    { 0xd2a02c9e, SUC_lexicon_tags_9 },
    { 0x78002c9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e202ca2, SUC_lexicon_tags_37 },
    { 0x17e02ca3, SUC_lexicon_tags_0 },
    { 0xe3802ca4, SUC_lexicon_tags_16 },
    { 0x5aa02ca5, SUC_lexicon_tags_27 },
    { 0xb2002ca6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb002ca9, SUC_lexicon_tags_55 },
    { 0xb9c02caa, SUC_lexicon_tags_7 },
    { 0xd202caa, SUC_lexicon_tags_7 },
    { 0x4c602ca9, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd602cb0, SUC_lexicon_tags_25 },
    { 0x60c02cb0, SUC_lexicon_tags_17 },
    { 0x11a02cb2, SUC_lexicon_tags_46 },
    { 0xd0202cb3, SUC_lexicon_tags_45 },
    { 0x71602cb4, SUC_lexicon_tags_7 },
    { 0x10e02cb5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1202cbc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9402cbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5602cc0, SUC_lexicon_tags_31 },
    { 0x7e02cc1, SUC_lexicon_tags_37 },
    { 0xc002cc1, SUC_lexicon_tags_37 },
    { 0x77a02cc3, SUC_lexicon_tags_17 },
    { 0xfd402cc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac202cc6, SUC_lexicon_tags_17 },
    { 0x88402cc6, SUC_lexicon_tags_25 },
    { 0x60e02cc8, SUC_lexicon_tags_56 },
    { 0xa6002cc8, SUC_lexicon_tags_17 },
    { 0x2b602cc8, SUC_lexicon_tags_14 },
    { 0x10802ccb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec602cd0, SUC_lexicon_tags_17 },
    { 0x7d402cd1, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x3ac02cd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c402cd8, SUC_lexicon_tags_25 },
    { 0xb6802cd8, SUC_lexicon_tags_45 },
    { 0x28402cda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6602cde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa202ce0, SUC_lexicon_tags_12 },
    { 0xf0e02ce1, SUC_lexicon_tags_12 },
    { 0xe7602ce0, SUC_lexicon_tags_15 },
    { 0x70c02ce1, SUC_lexicon_tags_25 },
    { 0xb4e02ce0, SUC_lexicon_tags_38 },
    { 0xbe402ce5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5802ce7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a02ced, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54402cef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a802cf2, SUC_lexicon_tags_31 },
    { 0xfce02cf3, SUC_lexicon_tags_37 },
    { 0x80402cf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93002cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83202cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56602cfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a002cff, SUC_lexicon_tags_15 },
    { 0x88c02cff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x41402d03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79202d0a, SUC_lexicon_tags_45 },
    { 0x31e02d0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58602d0d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbde02d0f, SUC_lexicon_tags_3 },
    { 0x12c02d0f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca02d15, SUC_lexicon_tags_8 },
    { 0x53202d16, SUC_lexicon_tags_1 },
    { 0x62002d17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c02d1d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6202d21, SUC_lexicon_tags_26 },
    { 0x11402d21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a002d25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3602d27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da02d2b, SUC_lexicon_tags_0 },
    { 0x47202d2c, SUC_lexicon_tags_25 },
    { 0x94202d2d, SUC_lexicon_tags_33 },
    { 0x35c02d2b, SUC_lexicon_tags_29 },
    { 0xdfe02d2b, SUC_lexicon_tags_27 },
    { 0x87c02d2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x29002d33, SUC_lexicon_tags_17 },
    { 0x5c802d34, SUC_lexicon_tags_2 },
    { 0x46202d33, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c02d39, SUC_lexicon_tags_11 },
    { 0x6ce02d3a, SUC_lexicon_tags_12 },
    { 0x73a02d3b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdf002d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25002d41, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c02d44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a02d49, SUC_lexicon_tags_1 },
    { 0x6a002d4a, SUC_lexicon_tags_31 },
    { 0x14602d4a, SUC_lexicon_tags_26 },
    { 0xe2402d4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa02d50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ee02d52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77802d58, SUC_lexicon_tags_74 },
    { 0xb1202d59, SUC_lexicon_tags_17 },
    { 0x6a402d59, SUC_lexicon_tags_7 },
    { 0xe1202d59, SUC_lexicon_tags_17 },
    { 0xf8a02d58, SUC_lexicon_tags_37 },
    { 0x46c02d5d, SUC_lexicon_tags_63 },
    { 0x58202d5e, SUC_lexicon_tags_25 },
    { 0x7c202d5f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc02d62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x17202d65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e02d6b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d602d6e, SUC_lexicon_tags_1 },
    { 0x77002d6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe202d73, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19802d7b, SUC_lexicon_tags_7 },
    { 0x5bc02d7c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x33202d7f, SUC_lexicon_tags_16 },
    { 0x4c02d80, SUC_lexicon_tags_33 },
    { 0xda402d7f, SUC_lexicon_tags_45 },
    { 0xf6e02d80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4402d84, SUC_lexicon_tags_11 },
    { 0x68a02d84, SUC_lexicon_tags_9 },
    { 0x29a02d86, SUC_lexicon_tags_32 },
    { 0xcd802d86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e02d8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2202d8d, SUC_lexicon_tags_25 },
    { 0x9d002d8d, SUC_lexicon_tags_11 },
    { 0x4d402d8f, SUC_lexicon_tags_0 },
    { 0xafa02d90, SUC_lexicon_tags_25 },
    { 0x89202d91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a02d97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69c02d99, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9402d9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97002da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e02da7, SUC_lexicon_tags_12 },
    { 0x48202da8, SUC_lexicon_tags_8 },
    { 0xdc002da7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c02dac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57a02dae, SUC_lexicon_tags_63 },
    { 0xcf602daf, SUC_lexicon_tags_9 },
    { 0x1b402dae, SUC_lexicon_tags_17 },
    { 0xb5a02db1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7602dba, SUC_lexicon_tags_43 },
    { 0x2c02dbb, SUC_lexicon_tags_11 },
    { 0xf9802dbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3da02dbe, SUC_lexicon_tags_45 },
    { 0xb4a02dbf, SUC_lexicon_tags_25 },
    { 0xc0802dbf, SUC_lexicon_tags_15 },
    { 0x29402dc1, SUC_lexicon_tags_45 },
    { 0xaa802dbf, SUC_lexicon_tags_37 },
    { 0xc5802dc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0e02dc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79802dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57002dcf, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xfda02dd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x86002dd3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x92802dd6, SUC_lexicon_tags_25 },
    { 0x6b002dd7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc6202dd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14802de3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f002def, SUC_lexicon_tags_84 },
    { 0x90c02df0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb802df3, SUC_lexicon_tags_11 },
    { 0xd9602df4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c002dfa, SUC_lexicon_tags_37 },
    { 0x37202dfb, SUC_lexicon_tags_7 },
    { 0xf5202dfc, SUC_lexicon_tags_6 },
    { 0x14802dfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66802e01, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c02e07, SUC_lexicon_tags_9 },
    { 0xce802e08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e02e0b, SUC_lexicon_tags_45 },
    { 0x78402e0b, SUC_lexicon_tags_27 },
    { 0xd2002e0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2602e0f, SUC_lexicon_tags_17 },
    { 0x86a02e10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a02e13, SUC_lexicon_tags_32 },
    { 0xa3e02e14, SUC_lexicon_tags_37 },
    { 0x2202e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69202e18, SUC_lexicon_tags_47 },
    { 0xe7202e19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x34602e1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f002e20, SUC_lexicon_tags_19 },
    { 0x9e802e21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a202e23, SUC_lexicon_tags_30 },
    { 0xe8c02e24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a02e29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0a02e2b, SUC_lexicon_tags_91 },
    { 0x51e02e2b, SUC_lexicon_tags_1 },
    { 0xbe802e2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce02e30, SUC_lexicon_tags_1 },
    { 0x6d402e31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c02e36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x61802e38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68802e3c, SUC_lexicon_tags_1 },
    { 0x6ba02e3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4202e3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c02e43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b802e45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba02e4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80602e51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2602e53, SUC_lexicon_tags_11 },
    { 0x13002e54, SUC_lexicon_tags_31 },
    { 0xc9a02e55, SUC_lexicon_tags_7 },
    { 0x36a02e56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2602e59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c802e5c, SUC_lexicon_tags_2 },
    { 0x5ea02e5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa402e62, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x50a02e64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93002e66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde602e69, SUC_lexicon_tags_17 },
    { 0x41002e6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b602e6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e602e6e, SUC_lexicon_tags_26 },
    { 0xbe402e6f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd6c02e71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d202e77, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xea602e79, SUC_lexicon_tags_7 },
    { 0xe5402e79, SUC_lexicon_tags_37 },
    { 0xbe002e79, SUC_lexicon_tags_1 },
    { 0x3c202e7b, SUC_lexicon_tags_64 },
    { 0x2d202e7d, SUC_lexicon_tags_25 },
    { 0xcb802e7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43202e84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d802e88, SUC_lexicon_tags_17 },
    { 0xe7c02e89, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee02e8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x52202e8e, SUC_lexicon_tags_79 },
    { 0xdca02e8f, SUC_lexicon_tags_16 },
    { 0x93202e8f, SUC_lexicon_tags_47 },
    { 0x59602e91, SUC_lexicon_tags_14 },
    { 0x34002e92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xab402e94, SUC_lexicon_tags_4 },
    { 0x69602e94, SUC_lexicon_tags_33 },
    { 0x62802e96, SUC_lexicon_tags_12 },
    { 0x48202e96, SUC_lexicon_tags_39 },
    { 0x95002e97, SUC_lexicon_tags_25 },
    { 0x10c02e99, SUC_lexicon_tags_47 },
    { 0x9202e9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e02ea0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb4c02ea2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x13602ea4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xde802ea6, SUC_lexicon_tags_25 },
    { 0xc5402ea7, SUC_lexicon_tags_11 },
    { 0x81a02ea8, SUC_lexicon_tags_1 },
    { 0x3ba02ea6, SUC_lexicon_tags_15 },
    { 0xcb602ea6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f002eae, SUC_lexicon_tags_0 },
    { 0x48002eaf, SUC_lexicon_tags_2 },
    { 0x67e02eb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe8802eb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1202eb6, SUC_lexicon_tags_45 },
    { 0x9f202eb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1002eba, SUC_lexicon_tags_71 },
    { 0xffe02ebb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4802ec0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5802ec2, SUC_lexicon_tags_25 },
    { 0xb7202ec2, SUC_lexicon_tags_19 },
    { 0xa0602ec4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95602ecb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4f002ecd, SUC_lexicon_tags_11 },
    { 0xfc002ece, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4c602ed0, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x86e02ed2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d002ed4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29e02ed6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8002edb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec402edd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8602ee3, SUC_lexicon_tags_28 },
    { 0x14002ee3, SUC_lexicon_tags_19 },
    { 0x4a802ee3, SUC_lexicon_tags_46 },
    { 0x28c02ee3, SUC_lexicon_tags_32 },
    { 0x62402ee6, SUC_lexicon_tags_151 },
    { 0xb7602ee8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba02eec, SUC_lexicon_tags_7 },
    { 0x5e202eec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x52e02eef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e002ef2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73002efa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xff002efd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee402f02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabe02f04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e02f08, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab002f0c, SUC_lexicon_tags_17 },
    { 0x33a02f0d, SUC_lexicon_tags_2 },
    { 0xfd202f0e, SUC_lexicon_tags_15 },
    { 0xeb602f0f, SUC_lexicon_tags_19 },
    { 0x3e02f0e, SUC_lexicon_tags_25 },
    { 0xec802f0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49e02f13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e02f17, SUC_lexicon_tags_25 },
    { 0x33202f18, SUC_lexicon_tags_7 },
    { 0xf6202f19, SUC_lexicon_tags_0 },
    { 0x22e02f19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fe02f1c, SUC_lexicon_tags_6 },
    { 0xf9e02f1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbe02f1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85402f28, SUC_lexicon_tags_87 },
    { 0xdf602f28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f002f2c, SUC_lexicon_tags_45 },
    { 0x4d602f2d, SUC_lexicon_tags_16 },
    { 0xaf402f2d, SUC_lexicon_tags_15 },
    { 0x3de02f2d, SUC_lexicon_tags_37 },
    { 0xdc802f30, SUC_lexicon_tags_54 },
    { 0xec002f2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72802f33, SUC_lexicon_tags_61 },
    { 0xcb402f34, SUC_lexicon_tags_82 },
    { 0x53802f34, SUC_lexicon_tags_14 },
    { 0x5a002f33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b802f39, SUC_lexicon_tags_11 },
    { 0xf6802f39, SUC_lexicon_tags_1 },
    { 0xe3e02f3b, SUC_lexicon_tags_19 },
    { 0xb5002f3a, SUC_lexicon_tags_44 },
    { 0x6e002f3d, SUC_lexicon_tags_19 },
    { 0xa2a02f3d, SUC_lexicon_tags_23 },
    { 0x85402f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0402f41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0002f43, SUC_lexicon_tags_11 },
    { 0xea202f44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f602f48, SUC_lexicon_tags_37 },
    { 0x80e02f48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16602f51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0002f56, SUC_lexicon_tags_33 },
    { 0x79002f57, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77602f5c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x72002f5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0e02f60, SUC_lexicon_tags_37 },
    { 0x1f202f61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8802f64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c02f67, SUC_lexicon_tags_37 },
    { 0xec602f68, SUC_lexicon_tags_47 },
    { 0xa9c02f69, SUC_lexicon_tags_9 },
    { 0x66c02f6a, SUC_lexicon_tags_2 },
    { 0x5e402f6b, SUC_lexicon_tags_37 },
    { 0x2f002f6c, SUC_lexicon_tags_39 },
    { 0x32002f6d, SUC_lexicon_tags_27 },
    { 0x1fa02f6e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84202f73, SUC_lexicon_tags_7 },
    { 0x7b002f73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9402f76, SUC_lexicon_tags_26 },
    { 0x8c202f77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd6602f79, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf3e02f7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb402f83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a02f86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d402f8a, SUC_lexicon_tags_38 },
    { 0xa602f8b, SUC_lexicon_tags_5 },
    { 0xaca02f8c, SUC_lexicon_tags_46 },
    { 0x2e202f8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef002f90, SUC_lexicon_tags_29 },
    { 0x28e02f91, SUC_lexicon_tags_60 },
    { 0x99202f92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac02f97, SUC_lexicon_tags_23 },
    { 0xe5002f98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca02f9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x43202f9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8602fa1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a02fa5, SUC_lexicon_tags_25 },
    { 0x60a02fa6, SUC_lexicon_tags_30 },
    { 0xec202fa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7602fac, SUC_lexicon_tags_1 },
    { 0xf1602fad, SUC_lexicon_tags_11 },
    { 0xb2202fad, SUC_lexicon_tags_8 },
    { 0xf4e02fad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b402fb1, SUC_lexicon_tags_38 },
    { 0xc9002fb1, SUC_lexicon_tags_14 },
    { 0x3aa02fb2, SUC_lexicon_tags_32 },
    { 0x94402fb4, SUC_lexicon_tags_13 },
    { 0x7e02fb4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14402fba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13002fbe, SUC_lexicon_tags_22 },
    { 0x6c02fbe, SUC_lexicon_tags_17 },
    { 0x84c02fc0, SUC_lexicon_tags_55 },
    { 0x43002fbf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8202fc4, SUC_lexicon_tags_25 },
    { 0x92602fc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8402fc7, SUC_lexicon_tags_2 },
    { 0x4ba02fc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1002fca, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4402fd0, SUC_lexicon_tags_33 },
    { 0x13002fd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e802fde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a402fe3, SUC_lexicon_tags_1 },
    { 0x72202fe3, SUC_lexicon_tags_37 },
    { 0x7a402fe5, SUC_lexicon_tags_27 },
    { 0x48e02fe4, SUC_lexicon_tags_47 },
    { 0x53202fe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5802fe9, SUC_lexicon_tags_37 },
    { 0x98202fea, SUC_lexicon_tags_30 },
    { 0x71402fe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55602fed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91202ff0, SUC_lexicon_tags_1 },
    { 0x40402ff1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xee202ff4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcb002ff6, SUC_lexicon_tags_12 },
    { 0x42c02ff6, SUC_lexicon_tags_92 },
    { 0x8d002ff8, SUC_lexicon_tags_26 },
    { 0x51602ff8, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c02ffc, SUC_lexicon_tags_1 },
    { 0x72002ffd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x47202fff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc03005, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x87403007, SUC_lexicon_tags_7 },
    { 0x22c03008, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ae0300a, SUC_lexicon_tags_25 },
    { 0x87c0300b, SUC_lexicon_tags_17 },
    { 0xc2c0300a, SUC_lexicon_tags_12 },
    { 0xd460300a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba03010, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb6203012, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a03016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac0301b, SUC_lexicon_tags_555 },
    { 0, NULL },
    { 0xc860301d, SUC_lexicon_tags_28 },
    { 0xbfa0301d, SUC_lexicon_tags_12 },
    { 0xc280301d, SUC_lexicon_tags_15 },
    { 0x3fa03020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1003024, SUC_lexicon_tags_50 },
    { 0x3e603024, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2de03027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde03031, SUC_lexicon_tags_27 },
    { 0x8d603032, SUC_lexicon_tags_15 },
    { 0xb8203032, SUC_lexicon_tags_17 },
    { 0x19e03034, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7603036, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c0303a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c40303f, SUC_lexicon_tags_39 },
    { 0xed20303f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77803045, SUC_lexicon_tags_1 },
    { 0xe203046, SUC_lexicon_tags_77 },
    { 0xf8603047, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa140304a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5520304d, SUC_lexicon_tags_3 },
    { 0xf880304e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec03051, SUC_lexicon_tags_15 },
    { 0xd5e03052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61603057, SUC_lexicon_tags_12 },
    { 0x2c203058, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c0305b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1aa0305d, SUC_lexicon_tags_37 },
    { 0x83a0305d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7203063, SUC_lexicon_tags_17 },
    { 0xebe03064, SUC_lexicon_tags_14 },
    { 0x83603065, SUC_lexicon_tags_6 },
    { 0x17403066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e03069, SUC_lexicon_tags_14 },
    { 0xbd00306a, SUC_lexicon_tags_45 },
    { 0x5460306b, SUC_lexicon_tags_17 },
    { 0x6860306c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d603071, SUC_lexicon_tags_27 },
    { 0x4f803072, SUC_lexicon_tags_18 },
    { 0xd1803073, SUC_lexicon_tags_18 },
    { 0x90a03071, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x15a03076, SUC_lexicon_tags_9 },
    { 0xf1403077, SUC_lexicon_tags_8 },
    { 0xb3c03076, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdea0307a, SUC_lexicon_tags_13 },
    { 0x2480307b, SUC_lexicon_tags_25 },
    { 0xe6e0307a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97203081, SUC_lexicon_tags_0 },
    { 0xa0003081, SUC_lexicon_tags_25 },
    { 0x88a03083, SUC_lexicon_tags_0 },
    { 0xae803081, SUC_lexicon_tags_12 },
    { 0xb2a03083, SUC_lexicon_tags_12 },
    { 0x62603085, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a60308a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e40308d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e03092, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e03095, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260309c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e030a3, SUC_lexicon_tags_14 },
    { 0xd5c030a4, SUC_lexicon_tags_37 },
    { 0x60030a5, SUC_lexicon_tags_84 },
    { 0x682030a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344030af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x474030b2, SUC_lexicon_tags_25 },
    { 0xf86030b3, SUC_lexicon_tags_45 },
    { 0x54030b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54030b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56030c0, SUC_lexicon_tags_38 },
    { 0x144030c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16e030c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36030cc, SUC_lexicon_tags_12 },
    { 0xe9e030cd, SUC_lexicon_tags_14 },
    { 0xbce030ce, SUC_lexicon_tags_59 },
    { 0x222030cf, SUC_lexicon_tags_3 },
    { 0x538030cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf28030d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76e030d4, SUC_lexicon_tags_37 },
    { 0x44e030d5, SUC_lexicon_tags_15 },
    { 0x3ac030d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624030dd, SUC_lexicon_tags_32 },
    { 0xfe8030de, SUC_lexicon_tags_38 },
    { 0x97e030df, SUC_lexicon_tags_27 },
    { 0x90c030e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a8030e2, SUC_lexicon_tags_17 },
    { 0x786030e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6030e6, SUC_lexicon_tags_37 },
    { 0x5b8030e6, SUC_lexicon_tags_23 },
    { 0xc02030e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86e030ea, SUC_lexicon_tags_15 },
    { 0xaea030eb, SUC_lexicon_tags_12 },
    { 0x37c030ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac030ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47c030f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc38030f3, SUC_lexicon_tags_54 },
    { 0xad8030f4, SUC_lexicon_tags_19 },
    { 0xf1a030f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x896030f7, SUC_lexicon_tags_12 },
    { 0xcb2030f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84030fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc30030fd, SUC_lexicon_tags_23 },
    { 0x30030fe, SUC_lexicon_tags_19 },
    { 0x3a8030fd, SUC_lexicon_tags_45 },
    { 0x3ca030fe, SUC_lexicon_tags_28 },
    { 0xe1203101, SUC_lexicon_tags_25 },
    { 0xb0c030ff, SUC_lexicon_tags_37 },
    { 0xcdc030fd, SUC_lexicon_tags_17 },
    { 0x33603102, SUC_lexicon_tags_8 },
    { 0x450030fd, SUC_lexicon_tags_1 },
    { 0x12a03106, SUC_lexicon_tags_15 },
    { 0x716030fd, SUC_lexicon_tags_2 },
    { 0x8203108, SUC_lexicon_tags_37 },
    { 0xb3603108, SUC_lexicon_tags_37 },
    { 0x62603108, SUC_lexicon_tags_9 },
    { 0xf320310a, SUC_lexicon_tags_2 },
    { 0x2020310c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8203110, SUC_lexicon_tags_11 },
    { 0x61a03111, SUC_lexicon_tags_351 },
    { 0xb8603112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9900311b, SUC_lexicon_tags_8 },
    { 0x9380311c, SUC_lexicon_tags_17 },
    { 0xc5e0311b, SUC_lexicon_tags_39 },
    { 0xe2a0311e, SUC_lexicon_tags_35 },
    { 0x2400311f, SUC_lexicon_tags_49 },
    { 0x8ce03120, SUC_lexicon_tags_49 },
    { 0x1e803120, SUC_lexicon_tags_9 },
    { 0x19e03121, SUC_lexicon_tags_30 },
    { 0x68603123, SUC_lexicon_tags_1 },
    { 0xa4c03122, SUC_lexicon_tags_1 },
    { 0x2240311f, SUC_lexicon_tags_43 },
    { 0x16603126, SUC_lexicon_tags_37 },
    { 0x22403127, SUC_lexicon_tags_2 },
    { 0xbc603128, SUC_lexicon_tags_17 },
    { 0x5c003127, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c03133, SUC_lexicon_tags_37 },
    { 0xf9a03134, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a03137, SUC_lexicon_tags_2 },
    { 0x48403138, SUC_lexicon_tags_25 },
    { 0x82603139, SUC_lexicon_tags_156 },
    { 0x71c0313a, SUC_lexicon_tags_27 },
    { 0xf420313b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf420313e, SUC_lexicon_tags_1 },
    { 0xbde0313f, SUC_lexicon_tags_15 },
    { 0xfd003140, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1603143, SUC_lexicon_tags_7 },
    { 0x69e03144, SUC_lexicon_tags_25 },
    { 0x1d803145, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbec03147, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b20314a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e0314f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c03156, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x29603158, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2440315c, SUC_lexicon_tags_27 },
    { 0xbbc0315d, SUC_lexicon_tags_14 },
    { 0x1040315e, SUC_lexicon_tags_25 },
    { 0x7a20315d, SUC_lexicon_tags_11 },
    { 0x39a0315f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdca03162, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e03166, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b603174, SUC_lexicon_tags_38 },
    { 0x5f003175, SUC_lexicon_tags_7 },
    { 0xf0603176, SUC_lexicon_tags_60 },
    { 0xc4a03177, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa000317a, SUC_lexicon_tags_9 },
    { 0x6f80317a, SUC_lexicon_tags_11 },
    { 0x3c80317c, SUC_lexicon_tags_13 },
    { 0xc4a0317d, SUC_lexicon_tags_84 },
    { 0x88a0317e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e03184, SUC_lexicon_tags_37 },
    { 0x2aa03184, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12803188, SUC_lexicon_tags_23 },
    { 0x4fe03189, SUC_lexicon_tags_60 },
    { 0x43e03188, SUC_lexicon_tags_47 },
    { 0x35603189, SUC_lexicon_tags_25 },
    { 0xfca0318c, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c03190, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7ee03192, SUC_lexicon_tags_8 },
    { 0x65603193, SUC_lexicon_tags_31 },
    { 0x63203194, SUC_lexicon_tags_61 },
    { 0x64003194, SUC_lexicon_tags_2 },
    { 0x64a03196, SUC_lexicon_tags_37 },
    { 0x26803197, SUC_lexicon_tags_26 },
    { 0xd9403194, SUC_lexicon_tags_25 },
    { 0x6bc03199, SUC_lexicon_tags_28 },
    { 0x7a40319a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38031a4, SUC_lexicon_tags_12 },
    { 0xaba031a5, SUC_lexicon_tags_30 },
    { 0x1a4031a5, SUC_lexicon_tags_15 },
    { 0x2de031a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x97c031a9, SUC_lexicon_tags_11 },
    { 0xfaa031aa, SUC_lexicon_tags_34 },
    { 0x4bc031ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98031b0, SUC_lexicon_tags_15 },
    { 0x162031b0, SUC_lexicon_tags_39 },
    { 0xc36031b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x852031b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2031b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x532031bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a031bf, SUC_lexicon_tags_1 },
    { 0x106031bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb14031c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x58e031c4, SUC_lexicon_tags_11 },
    { 0xf8c031c5, SUC_lexicon_tags_9 },
    { 0x3e0031c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c031cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e031ce, SUC_lexicon_tags_2 },
    { 0x3ba031ce, SUC_lexicon_tags_56 },
    { 0x964031cf, SUC_lexicon_tags_23 },
    { 0xdb8031d1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x428031d3, SUC_lexicon_tags_7 },
    { 0x56031d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82031d9, SUC_lexicon_tags_37 },
    { 0xbda031d9, SUC_lexicon_tags_11 },
    { 0x4d4031db, SUC_lexicon_tags_14 },
    { 0x2f4031dc, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c031e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0031e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc031e7, SUC_lexicon_tags_37 },
    { 0x1d6031e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4031eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee031f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17e031f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c031f9, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c031fc, SUC_lexicon_tags_40 },
    { 0xfec031fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc03201, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fa03203, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x63203205, SUC_lexicon_tags_17 },
    { 0x1b403206, SUC_lexicon_tags_27 },
    { 0x6203205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb320320b, SUC_lexicon_tags_2 },
    { 0x8e80320c, SUC_lexicon_tags_14 },
    { 0x7c20320c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc003212, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1a03214, SUC_lexicon_tags_25 },
    { 0x3f003215, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ae03217, SUC_lexicon_tags_7 },
    { 0x19603217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbce0321a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x5da0321c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5800321f, SUC_lexicon_tags_30 },
    { 0x4de0321f, SUC_lexicon_tags_351 },
    { 0xee603221, SUC_lexicon_tags_45 },
    { 0x88403222, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93403227, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x56e03229, SUC_lexicon_tags_17 },
    { 0x26803229, SUC_lexicon_tags_17 },
    { 0x93e03229, SUC_lexicon_tags_17 },
    { 0x8f40322c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13003230, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd203236, SUC_lexicon_tags_8 },
    { 0xf6403236, SUC_lexicon_tags_9 },
    { 0xaba03236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6080323e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe603244, SUC_lexicon_tags_46 },
    { 0x70e03244, SUC_lexicon_tags_3 },
    { 0x8ec03246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe403249, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x46c0324b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9aa0324d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c0325e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26803260, SUC_lexicon_tags_37 },
    { 0x38003261, SUC_lexicon_tags_46 },
    { 0xfd403262, SUC_lexicon_tags_27 },
    { 0x9603263, SUC_lexicon_tags_38 },
    { 0xc2803264, SUC_lexicon_tags_21 },
    { 0xd1803263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e003268, SUC_lexicon_tags_7 },
    { 0x41e03269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a0326c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x540326e, SUC_lexicon_tags_15 },
    { 0xa040326f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d403271, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e03275, SUC_lexicon_tags_2 },
    { 0x50a03276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4760327b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a0327f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb403286, SUC_lexicon_tags_48 },
    { 0x71403287, SUC_lexicon_tags_1 },
    { 0xa2803288, SUC_lexicon_tags_1 },
    { 0xb6a03288, SUC_lexicon_tags_23 },
    { 0x3800328a, SUC_lexicon_tags_49 },
    { 0x4e60328b, SUC_lexicon_tags_25 },
    { 0x14a0328b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa03295, SUC_lexicon_tags_25 },
    { 0x6e03295, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa203299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa780329c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424032a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6032a7, SUC_lexicon_tags_33 },
    { 0xf90032a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14032ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504032af, SUC_lexicon_tags_17 },
    { 0xd7a032b0, SUC_lexicon_tags_5 },
    { 0xed8032b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e032b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c032b8, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0, NULL },
    { 0x958032bb, SUC_lexicon_tags_11 },
    { 0x814032bc, SUC_lexicon_tags_63 },
    { 0x514032bd, SUC_lexicon_tags_25 },
    { 0x34a032bd, SUC_lexicon_tags_9 },
    { 0x764032bd, SUC_lexicon_tags_11 },
    { 0x114032be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570032c4, SUC_lexicon_tags_2 },
    { 0x7c032c4, SUC_lexicon_tags_32 },
    { 0xbce032c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4032cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0032d1, SUC_lexicon_tags_60 },
    { 0xff6032d2, SUC_lexicon_tags_3 },
    { 0x2de032d3, SUC_lexicon_tags_11 },
    { 0xffe032d4, SUC_lexicon_tags_85 },
    { 0xe2032d5, SUC_lexicon_tags_11 },
    { 0xde6032d6, SUC_lexicon_tags_25 },
    { 0x51e032d7, SUC_lexicon_tags_45 },
    { 0x24a032d8, SUC_lexicon_tags_27 },
    { 0x1c8032d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704032e0, SUC_lexicon_tags_37 },
    { 0xa0c032e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x646032e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0032e9, SUC_lexicon_tags_33 },
    { 0x9ae032ea, SUC_lexicon_tags_30 },
    { 0xab4032eb, SUC_lexicon_tags_0 },
    { 0xddc032ea, SUC_lexicon_tags_15 },
    { 0x434032ed, SUC_lexicon_tags_1 },
    { 0xc14032ee, SUC_lexicon_tags_18 },
    { 0xc0c032ea, SUC_lexicon_tags_25 },
    { 0xa00032f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0c032f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x658032f4, SUC_lexicon_tags_14 },
    { 0xb3a032f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78032f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82032fe, SUC_lexicon_tags_11 },
    { 0x8a2032ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e803306, SUC_lexicon_tags_61 },
    { 0x3c03307, SUC_lexicon_tags_37 },
    { 0xd0403308, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec0330f, SUC_lexicon_tags_16 },
    { 0xe0c03310, SUC_lexicon_tags_25 },
    { 0x96e03311, SUC_lexicon_tags_32 },
    { 0x68003312, SUC_lexicon_tags_37 },
    { 0x7ea03312, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a03317, SUC_lexicon_tags_55 },
    { 0x8a03318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c0331b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5680331d, SUC_lexicon_tags_26 },
    { 0xcc60331e, SUC_lexicon_tags_15 },
    { 0x1bc0331d, SUC_lexicon_tags_27 },
    { 0xbbc03320, SUC_lexicon_tags_100 },
    { 0xe20331f, SUC_lexicon_tags_31 },
    { 0xd603322, SUC_lexicon_tags_26 },
    { 0x3323, SUC_lexicon_tags_37 },
    { 0x2f603324, SUC_lexicon_tags_65 },
    { 0x69803325, SUC_lexicon_tags_23 },
    { 0xa603320, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e603328, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x820332b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9a20332d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1403333, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0603336, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1803339, SUC_lexicon_tags_10 },
    { 0x2380333a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16203341, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6803344, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad803346, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe4203348, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd803350, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4403352, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x54803354, SUC_lexicon_tags_29 },
    { 0x9ea03355, SUC_lexicon_tags_45 },
    { 0x31803356, SUC_lexicon_tags_9 },
    { 0xe4803357, SUC_lexicon_tags_38 },
    { 0x7d403357, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e80335b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae03362, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe803365, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c0336d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7660337a, SUC_lexicon_tags_2 },
    { 0xb6e0337a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2403380, SUC_lexicon_tags_3 },
    { 0xc2e03380, SUC_lexicon_tags_25 },
    { 0x3f403381, SUC_lexicon_tags_1 },
    { 0x7b003383, SUC_lexicon_tags_14 },
    { 0xda803383, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00338d, SUC_lexicon_tags_29 },
    { 0x3bc0338e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d203391, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7f003393, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3c003395, SUC_lexicon_tags_57 },
    { 0x5003396, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a03399, SUC_lexicon_tags_11 },
    { 0xe200339a, SUC_lexicon_tags_2 },
    { 0xfcc0339b, SUC_lexicon_tags_0 },
    { 0xd780339a, SUC_lexicon_tags_47 },
    { 0x6d20339d, SUC_lexicon_tags_26 },
    { 0xe8a0339a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x156033a1, SUC_lexicon_tags_11 },
    { 0x4a4033a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816033a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x796033a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972033b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x670033b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x752033b5, SUC_lexicon_tags_29 },
    { 0xd48033b5, SUC_lexicon_tags_8 },
    { 0x468033b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0033bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8033c0, SUC_lexicon_tags_23 },
    { 0xd40033c0, SUC_lexicon_tags_38 },
    { 0x12a033c1, SUC_lexicon_tags_2 },
    { 0x3b6033c3, SUC_lexicon_tags_30 },
    { 0xb14033c3, SUC_lexicon_tags_17 },
    { 0x4a033c5, SUC_lexicon_tags_17 },
    { 0x414033c5, SUC_lexicon_tags_11 },
    { 0x428033c4, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a033cf, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8033d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a033d4, SUC_lexicon_tags_1 },
    { 0x30c033d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478033d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332033e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0e033e6, SUC_lexicon_tags_11 },
    { 0xb78033e6, SUC_lexicon_tags_47 },
    { 0x5bc033e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670033ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf52033ee, SUC_lexicon_tags_12 },
    { 0x2b0033ef, SUC_lexicon_tags_7 },
    { 0x21c033f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe24033f2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51403400, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca03404, SUC_lexicon_tags_84 },
    { 0xcb403404, SUC_lexicon_tags_39 },
    { 0x80c03406, SUC_lexicon_tags_21 },
    { 0x44403407, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd580340b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa320340e, SUC_lexicon_tags_17 },
    { 0xcca0340e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c03414, SUC_lexicon_tags_25 },
    { 0x5b603415, SUC_lexicon_tags_17 },
    { 0xb6e03416, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x82603418, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac0341c, SUC_lexicon_tags_45 },
    { 0xa520341d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb60341f, SUC_lexicon_tags_15 },
    { 0xdd803420, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26203422, SUC_lexicon_tags_61 },
    { 0xb3203422, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc03436, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf700343a, SUC_lexicon_tags_33 },
    { 0x5500343b, SUC_lexicon_tags_31 },
    { 0x6720343c, SUC_lexicon_tags_2 },
    { 0x3e20343d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a03440, SUC_lexicon_tags_17 },
    { 0x40e03441, SUC_lexicon_tags_1 },
    { 0xd9203442, SUC_lexicon_tags_1 },
    { 0x5d803443, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b003445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e0344c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x880344e, SUC_lexicon_tags_26 },
    { 0x77c0344f, SUC_lexicon_tags_19 },
    { 0x920344e, SUC_lexicon_tags_12 },
    { 0xab203451, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a003455, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40603459, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa540345b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f203460, SUC_lexicon_tags_6 },
    { 0x1ba03461, SUC_lexicon_tags_30 },
    { 0x59c03462, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f003467, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1003472, SUC_lexicon_tags_26 },
    { 0x31203473, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42603478, SUC_lexicon_tags_60 },
    { 0x80603478, SUC_lexicon_tags_11 },
    { 0x43e0347a, SUC_lexicon_tags_12 },
    { 0xdaa0347a, SUC_lexicon_tags_2 },
    { 0xea40347a, SUC_lexicon_tags_31 },
    { 0x9ae0347d, SUC_lexicon_tags_9 },
    { 0x43a0347a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x32203481, SUC_lexicon_tags_27 },
    { 0x6da03482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c603484, SUC_lexicon_tags_26 },
    { 0x9f603484, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e803487, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8480348b, SUC_lexicon_tags_15 },
    { 0x8f20348c, SUC_lexicon_tags_5 },
    { 0x1260348c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60603491, SUC_lexicon_tags_1 },
    { 0x45a03492, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4003498, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7e40349a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xbfc0349c, SUC_lexicon_tags_8 },
    { 0x2ec0349c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba00349f, SUC_lexicon_tags_15 },
    { 0xbbc034a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c034a3, SUC_lexicon_tags_37 },
    { 0x616034a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0034a9, SUC_lexicon_tags_22 },
    { 0x6b0034aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x172034ac, SUC_lexicon_tags_8 },
    { 0x26e034ad, SUC_lexicon_tags_17 },
    { 0xa08034ac, SUC_lexicon_tags_23 },
    { 0x5e0034af, SUC_lexicon_tags_57 },
    { 0x8034b0, SUC_lexicon_tags_37 },
    { 0xfc8034ac, SUC_lexicon_tags_11 },
    { 0xf4e034b1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6f2034b4, SUC_lexicon_tags_34 },
    { 0x85a034b5, SUC_lexicon_tags_1 },
    { 0x6a0034b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbde034b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec034ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x610034bc, SUC_lexicon_tags_17 },
    { 0x760034bd, SUC_lexicon_tags_4 },
    { 0xbec034bd, SUC_lexicon_tags_45 },
    { 0x522034bf, SUC_lexicon_tags_22 },
    { 0x31c034bf, SUC_lexicon_tags_23 },
    { 0xe00034c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x306034c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12e034c5, SUC_lexicon_tags_25 },
    { 0xd5e034c5, SUC_lexicon_tags_79 },
    { 0x34a034c7, SUC_lexicon_tags_17 },
    { 0x4e4034c8, SUC_lexicon_tags_37 },
    { 0x770034c9, SUC_lexicon_tags_11 },
    { 0x3c2034c9, SUC_lexicon_tags_0 },
    { 0x574034c7, SUC_lexicon_tags_11 },
    { 0x7f8034ca, SUC_lexicon_tags_14 },
    { 0x63a034cd, SUC_lexicon_tags_12 },
    { 0xc4034ce, SUC_lexicon_tags_48 },
    { 0x170034cd, SUC_lexicon_tags_1 },
    { 0x756034d0, SUC_lexicon_tags_43 },
    { 0x98c034d1, SUC_lexicon_tags_15 },
    { 0x5d8034c7, SUC_lexicon_tags_84 },
    { 0x33c034d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c034d8, SUC_lexicon_tags_8 },
    { 0x8c0034d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2034de, SUC_lexicon_tags_0 },
    { 0xda4034df, SUC_lexicon_tags_31 },
    { 0x5d4034df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410034e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400034ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x682034ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x722034f2, SUC_lexicon_tags_56 },
    { 0x24034f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b0034f5, SUC_lexicon_tags_48 },
    { 0xe9e034f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66034f9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde803501, SUC_lexicon_tags_47 },
    { 0xf4003502, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x75403505, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9140350e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e03512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8203517, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5860351e, SUC_lexicon_tags_25 },
    { 0x6a60351f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b003523, SUC_lexicon_tags_17 },
    { 0xc5c03523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be0352b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55403530, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70e03532, SUC_lexicon_tags_26 },
    { 0x12003533, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa2e03535, SUC_lexicon_tags_1 },
    { 0x21203536, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7ba03538, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fc0353a, SUC_lexicon_tags_15 },
    { 0x8a20353b, SUC_lexicon_tags_49 },
    { 0xa260353b, SUC_lexicon_tags_37 },
    { 0xe2e0353d, SUC_lexicon_tags_28 },
    { 0xc280353b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8603549, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dc0354b, SUC_lexicon_tags_1 },
    { 0x2ac0354b, SUC_lexicon_tags_2 },
    { 0xe0a0354d, SUC_lexicon_tags_60 },
    { 0x7b20354c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce203554, SUC_lexicon_tags_19 },
    { 0xf0603555, SUC_lexicon_tags_10 },
    { 0x80c03556, SUC_lexicon_tags_63 },
    { 0x44603555, SUC_lexicon_tags_26 },
    { 0xb9603558, SUC_lexicon_tags_47 },
    { 0xc3a03555, SUC_lexicon_tags_15 },
    { 0xb0a03555, SUC_lexicon_tags_16 },
    { 0x2603558, SUC_lexicon_tags_11 },
    { 0x1300355c, SUC_lexicon_tags_1 },
    { 0xf440355d, SUC_lexicon_tags_84 },
    { 0xaee0355d, SUC_lexicon_tags_37 },
    { 0xa7e0355f, SUC_lexicon_tags_0 },
    { 0x82603560, SUC_lexicon_tags_15 },
    { 0x59a03560, SUC_lexicon_tags_14 },
    { 0x54203562, SUC_lexicon_tags_0 },
    { 0xa603561, SUC_lexicon_tags_45 },
    { 0xf5003564, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fe03566, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce0356c, SUC_lexicon_tags_61 },
    { 0xd260356d, SUC_lexicon_tags_37 },
    { 0xdc40356e, SUC_lexicon_tags_0 },
    { 0x3700356f, SUC_lexicon_tags_23 },
    { 0x53e0356f, SUC_lexicon_tags_21 },
    { 0x7480356e, SUC_lexicon_tags_15 },
    { 0xde603572, SUC_lexicon_tags_63 },
    { 0x83e0356f, SUC_lexicon_tags_15 },
    { 0xdd803570, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8a403576, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a03579, SUC_lexicon_tags_352 },
    { 0x7c00357a, SUC_lexicon_tags_63 },
    { 0x9180357a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6020357f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de03582, SUC_lexicon_tags_17 },
    { 0x8ea03583, SUC_lexicon_tags_6 },
    { 0xe7403584, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x86e03586, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe8403588, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a0358b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55203590, SUC_lexicon_tags_12 },
    { 0xacc03591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53603595, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e03599, SUC_lexicon_tags_32 },
    { 0x82c0359a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da0359e, SUC_lexicon_tags_14 },
    { 0x7c40359f, SUC_lexicon_tags_47 },
    { 0x12c035a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6035a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0035a9, SUC_lexicon_tags_0 },
    { 0x2f6035a9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x14035ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa48035af, SUC_lexicon_tags_66 },
    { 0x5de035af, SUC_lexicon_tags_14 },
    { 0xa12035b0, SUC_lexicon_tags_17 },
    { 0xbdc035b2, SUC_lexicon_tags_61 },
    { 0x8d4035b2, SUC_lexicon_tags_17 },
    { 0x872035b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e035b8, SUC_lexicon_tags_31 },
    { 0xec2035b9, SUC_lexicon_tags_2 },
    { 0x866035b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x170035bc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca035bf, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xe54035c1, SUC_lexicon_tags_17 },
    { 0x6c0035c2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de035c6, SUC_lexicon_tags_38 },
    { 0xfc6035c6, SUC_lexicon_tags_33 },
    { 0x9c035c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a035cc, SUC_lexicon_tags_45 },
    { 0xf60035cc, SUC_lexicon_tags_10 },
    { 0x230035cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64035d2, SUC_lexicon_tags_15 },
    { 0x9fc035d3, SUC_lexicon_tags_12 },
    { 0x4ac035d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a035d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4c035da, SUC_lexicon_tags_0 },
    { 0x1b0035db, SUC_lexicon_tags_2 },
    { 0xb4a035dc, SUC_lexicon_tags_11 },
    { 0xd1c035dd, SUC_lexicon_tags_38 },
    { 0xf26035dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c035e4, SUC_lexicon_tags_23 },
    { 0xd0a035e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84035eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c035ef, SUC_lexicon_tags_30 },
    { 0xe7a035ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68035f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e035f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a035fb, SUC_lexicon_tags_2 },
    { 0x640035fc, SUC_lexicon_tags_7 },
    { 0xa3c035fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x83203600, SUC_lexicon_tags_49 },
    { 0xbc03601, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7003603, SUC_lexicon_tags_43 },
    { 0xc3003603, SUC_lexicon_tags_38 },
    { 0x4bc03604, SUC_lexicon_tags_7 },
    { 0x9fc03603, SUC_lexicon_tags_63 },
    { 0x42a03607, SUC_lexicon_tags_27 },
    { 0x23403607, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc880360e, SUC_lexicon_tags_12 },
    { 0xa360360e, SUC_lexicon_tags_19 },
    { 0xe5e0360f, SUC_lexicon_tags_7 },
    { 0x5660360f, SUC_lexicon_tags_17 },
    { 0x6d20360f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e03616, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x78003619, SUC_lexicon_tags_12 },
    { 0x82a03619, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c0361d, SUC_lexicon_tags_10 },
    { 0x7020361d, SUC_lexicon_tags_54 },
    { 0xde40361e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8b603621, SUC_lexicon_tags_19 },
    { 0x4b403621, SUC_lexicon_tags_15 },
    { 0x2a803621, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a403627, SUC_lexicon_tags_25 },
    { 0x46a03628, SUC_lexicon_tags_7 },
    { 0xec03627, SUC_lexicon_tags_17 },
    { 0x7d403628, SUC_lexicon_tags_47 },
    { 0xe4603628, SUC_lexicon_tags_43 },
    { 0x8260362c, SUC_lexicon_tags_15 },
    { 0xc2c0362c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb100362f, SUC_lexicon_tags_2 },
    { 0x6720362f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xffa03632, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c03635, SUC_lexicon_tags_16 },
    { 0xa203635, SUC_lexicon_tags_9 },
    { 0x1603637, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x77003639, SUC_lexicon_tags_15 },
    { 0x7fa03639, SUC_lexicon_tags_9 },
    { 0x9d403639, SUC_lexicon_tags_17 },
    { 0x7780363c, SUC_lexicon_tags_37 },
    { 0x9fe0363c, SUC_lexicon_tags_47 },
    { 0xcb00363b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d803640, SUC_lexicon_tags_1 },
    { 0x61003641, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5c403643, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ac03645, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd5003647, SUC_lexicon_tags_63 },
    { 0xf9803648, SUC_lexicon_tags_2 },
    { 0xf3e03648, SUC_lexicon_tags_55 },
    { 0x31c0364a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b203653, SUC_lexicon_tags_38 },
    { 0xb6c03654, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a03659, SUC_lexicon_tags_29 },
    { 0x9c603659, SUC_lexicon_tags_15 },
    { 0x7860365b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6a0365d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf403661, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c03668, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1460366d, SUC_lexicon_tags_29 },
    { 0xf460366e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc03672, SUC_lexicon_tags_37 },
    { 0x31203672, SUC_lexicon_tags_55 },
    { 0x5003674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae03677, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xe720367a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e0367e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab603680, SUC_lexicon_tags_7 },
    { 0x6403680, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e203688, SUC_lexicon_tags_37 },
    { 0xeb003689, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6500368d, SUC_lexicon_tags_15 },
    { 0xe640368e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x80203691, SUC_lexicon_tags_9 },
    { 0xabe03692, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50c03694, SUC_lexicon_tags_7 },
    { 0x42603695, SUC_lexicon_tags_17 },
    { 0x8e403695, SUC_lexicon_tags_2 },
    { 0x9b603697, SUC_lexicon_tags_11 },
    { 0xb2803696, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a0369b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716036a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60036a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c2036a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x868036ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62e036ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa036b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6036b3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x10e036b5, SUC_lexicon_tags_30 },
    { 0xc8c036b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba036b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x556036bb, SUC_lexicon_tags_17 },
    { 0x4c4036bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc0036be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e036c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4036c8, SUC_lexicon_tags_12 },
    { 0x26e036c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cc036cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630036d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c036d6, SUC_lexicon_tags_45 },
    { 0x3ae036d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2036dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2036df, SUC_lexicon_tags_14 },
    { 0x286036df, SUC_lexicon_tags_56 },
    { 0x5f6036e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaec036e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6036e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff2036e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x272036eb, SUC_lexicon_tags_1 },
    { 0xa38036ec, SUC_lexicon_tags_0 },
    { 0x68a036ed, SUC_lexicon_tags_77 },
    { 0x86036eb, SUC_lexicon_tags_8 },
    { 0xece036ef, SUC_lexicon_tags_9 },
    { 0x622036f0, SUC_lexicon_tags_0 },
    { 0x43c036f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e036f5, SUC_lexicon_tags_37 },
    { 0xe10036f6, SUC_lexicon_tags_12 },
    { 0x458036f5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x342036f9, SUC_lexicon_tags_48 },
    { 0xdd0036f9, SUC_lexicon_tags_14 },
    { 0xfc0036fb, SUC_lexicon_tags_12 },
    { 0x274036fc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0370c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e03712, SUC_lexicon_tags_23 },
    { 0x87403713, SUC_lexicon_tags_31 },
    { 0xe5603713, SUC_lexicon_tags_11 },
    { 0x9ce03715, SUC_lexicon_tags_27 },
    { 0xaf203715, SUC_lexicon_tags_37 },
    { 0x5e603716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3060371a, SUC_lexicon_tags_61 },
    { 0x7220371b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1160371e, SUC_lexicon_tags_12 },
    { 0x7880371e, SUC_lexicon_tags_11 },
    { 0x9800371f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e03724, SUC_lexicon_tags_30 },
    { 0x80203725, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce03727, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d203733, SUC_lexicon_tags_9 },
    { 0xd2203734, SUC_lexicon_tags_47 },
    { 0x5e003735, SUC_lexicon_tags_7 },
    { 0x7f003736, SUC_lexicon_tags_17 },
    { 0xde03737, SUC_lexicon_tags_17 },
    { 0x69803735, SUC_lexicon_tags_63 },
    { 0xce603739, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e80373c, SUC_lexicon_tags_7 },
    { 0x4340373d, SUC_lexicon_tags_45 },
    { 0xa0373d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7403741, SUC_lexicon_tags_14 },
    { 0x98a03742, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be03746, SUC_lexicon_tags_2 },
    { 0x7c03747, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc0374d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8240374f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3603754, SUC_lexicon_tags_43 },
    { 0x2aa03755, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd00375d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd840375f, SUC_lexicon_tags_33 },
    { 0xea203760, SUC_lexicon_tags_1 },
    { 0x93e03761, SUC_lexicon_tags_2 },
    { 0x5c603761, SUC_lexicon_tags_37 },
    { 0x1c003763, SUC_lexicon_tags_381 },
    { 0, NULL },
    { 0xe3003765, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a0376c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9dc0376e, SUC_lexicon_tags_7 },
    { 0x2440376f, SUC_lexicon_tags_0 },
    { 0x27a03770, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb403775, SUC_lexicon_tags_2 },
    { 0x82603776, SUC_lexicon_tags_19 },
    { 0x8d003777, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x79a03779, SUC_lexicon_tags_25 },
    { 0x90c03779, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e0377d, SUC_lexicon_tags_11 },
    { 0x1080377e, SUC_lexicon_tags_25 },
    { 0x2420377e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6403782, SUC_lexicon_tags_2 },
    { 0xad803782, SUC_lexicon_tags_6 },
    { 0x26a03783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa03788, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa0378d, SUC_lexicon_tags_87 },
    { 0x5840378e, SUC_lexicon_tags_11 },
    { 0x1580378f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e403798, SUC_lexicon_tags_1 },
    { 0x3ba03798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf20379c, SUC_lexicon_tags_37 },
    { 0x1220379d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x146037a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594037a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xab4037a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfc8037a8, SUC_lexicon_tags_9 },
    { 0x842037a9, SUC_lexicon_tags_1 },
    { 0x11e037aa, SUC_lexicon_tags_2 },
    { 0x7e0037ab, SUC_lexicon_tags_82 },
    { 0xc76037ac, SUC_lexicon_tags_118 },
    { 0x35a037ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50037b0, SUC_lexicon_tags_60 },
    { 0x438037b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da037b4, SUC_lexicon_tags_68 },
    { 0x894037b5, SUC_lexicon_tags_11 },
    { 0xbd4037b6, SUC_lexicon_tags_37 },
    { 0x8c8037b7, SUC_lexicon_tags_595 },
    { 0x154037b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d0037ba, SUC_lexicon_tags_32 },
    { 0x1f6037bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284037c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff2037c3, SUC_lexicon_tags_10 },
    { 0x420037c4, SUC_lexicon_tags_539 },
    { 0x45e037c3, SUC_lexicon_tags_31 },
    { 0xdc4037c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x58a037c8, SUC_lexicon_tags_25 },
    { 0x120037c8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4ca037cb, SUC_lexicon_tags_12 },
    { 0x258037cc, SUC_lexicon_tags_137 },
    { 0xb86037cb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824037d4, SUC_lexicon_tags_10 },
    { 0x1aa037d4, SUC_lexicon_tags_17 },
    { 0xf68037d5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c037de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2037e1, SUC_lexicon_tags_12 },
    { 0x550037e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd02037e4, SUC_lexicon_tags_21 },
    { 0xe20037e5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x722037e7, SUC_lexicon_tags_9 },
    { 0xc6e037e8, SUC_lexicon_tags_8 },
    { 0xaf0037e9, SUC_lexicon_tags_12 },
    { 0xec8037e9, SUC_lexicon_tags_32 },
    { 0xf76037e7, SUC_lexicon_tags_37 },
    { 0x1ae037ec, SUC_lexicon_tags_74 },
    { 0x4ec037ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0037f1, SUC_lexicon_tags_12 },
    { 0x3ce037f2, SUC_lexicon_tags_45 },
    { 0xa3e037f3, SUC_lexicon_tags_1 },
    { 0xe3a037f4, SUC_lexicon_tags_10 },
    { 0x144037f1, SUC_lexicon_tags_12 },
    { 0x99a037f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98037fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62037ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16003802, SUC_lexicon_tags_27 },
    { 0x7d603802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f403808, SUC_lexicon_tags_1 },
    { 0x6be03809, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6840380c, SUC_lexicon_tags_33 },
    { 0xc940380c, SUC_lexicon_tags_46 },
    { 0x5720380d, SUC_lexicon_tags_17 },
    { 0x2440380c, SUC_lexicon_tags_47 },
    { 0x94a03810, SUC_lexicon_tags_11 },
    { 0xbc60380c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16003818, SUC_lexicon_tags_38 },
    { 0x15003819, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b60381b, SUC_lexicon_tags_38 },
    { 0x1f20381c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e203822, SUC_lexicon_tags_17 },
    { 0xd5403823, SUC_lexicon_tags_47 },
    { 0xa4c03822, SUC_lexicon_tags_60 },
    { 0xd8603823, SUC_lexicon_tags_33 },
    { 0x60403826, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c403828, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00382d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c403831, SUC_lexicon_tags_87 },
    { 0x55803831, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36003836, SUC_lexicon_tags_1 },
    { 0xe6a03836, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba0383d, SUC_lexicon_tags_11 },
    { 0x79a0383e, SUC_lexicon_tags_17 },
    { 0xe660383f, SUC_lexicon_tags_26 },
    { 0x2de03840, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60403848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10c0384a, SUC_lexicon_tags_25 },
    { 0xfbe0384b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd603852, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a03855, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4a03857, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41803859, SUC_lexicon_tags_8 },
    { 0x17403859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c0385d, SUC_lexicon_tags_15 },
    { 0x6640385e, SUC_lexicon_tags_37 },
    { 0xe2a0385e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e03862, SUC_lexicon_tags_43 },
    { 0x3f403863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c03866, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1b403868, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc0386b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25403876, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16003878, SUC_lexicon_tags_68 },
    { 0xe6203879, SUC_lexicon_tags_29 },
    { 0x3b203878, SUC_lexicon_tags_17 },
    { 0x4380387b, SUC_lexicon_tags_92 },
    { 0x6b60387b, SUC_lexicon_tags_19 },
    { 0x5b80387d, SUC_lexicon_tags_65 },
    { 0x9760387b, SUC_lexicon_tags_27 },
    { 0xb4e0387f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64403882, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x49c03884, SUC_lexicon_tags_26 },
    { 0x85a03885, SUC_lexicon_tags_12 },
    { 0xa803885, SUC_lexicon_tags_16 },
    { 0x62203884, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x47403889, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4120388b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c80388e, SUC_lexicon_tags_0 },
    { 0x87a0388e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x89603892, SUC_lexicon_tags_17 },
    { 0x62e03893, SUC_lexicon_tags_23 },
    { 0x8803894, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf9603896, SUC_lexicon_tags_55 },
    { 0x10403897, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe740389d, SUC_lexicon_tags_25 },
    { 0x6b60389e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8038a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x590038a4, SUC_lexicon_tags_39 },
    { 0x710038a5, SUC_lexicon_tags_66 },
    { 0xac2038a6, SUC_lexicon_tags_1 },
    { 0x380038a7, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4038aa, SUC_lexicon_tags_16 },
    { 0x164038ab, SUC_lexicon_tags_15 },
    { 0x3d0038ab, SUC_lexicon_tags_12 },
    { 0xb36038ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00038b1, SUC_lexicon_tags_15 },
    { 0x41a038b1, SUC_lexicon_tags_235 },
    { 0x1a0038b2, SUC_lexicon_tags_43 },
    { 0x1ae038b4, SUC_lexicon_tags_33 },
    { 0x230038b3, SUC_lexicon_tags_7 },
    { 0xca8038b5, SUC_lexicon_tags_15 },
    { 0x298038b2, SUC_lexicon_tags_13 },
    { 0xdf2038b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32038bc, SUC_lexicon_tags_7 },
    { 0xe26038bc, SUC_lexicon_tags_25 },
    { 0x2f8038be, SUC_lexicon_tags_11 },
    { 0xb40038bf, SUC_lexicon_tags_17 },
    { 0xd44038bf, SUC_lexicon_tags_8 },
    { 0xa9c038c0, SUC_lexicon_tags_2 },
    { 0x11c038c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x90038c5, SUC_lexicon_tags_47 },
    { 0x114038c6, SUC_lexicon_tags_1 },
    { 0x2f6038c7, SUC_lexicon_tags_45 },
    { 0x8e4038c7, SUC_lexicon_tags_6 },
    { 0x58e038c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2038cd, SUC_lexicon_tags_1 },
    { 0xee6038ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd00038d0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x430038d2, SUC_lexicon_tags_2 },
    { 0x498038d2, SUC_lexicon_tags_13 },
    { 0xfae038d4, SUC_lexicon_tags_2 },
    { 0x498038d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x122038d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97c038d9, SUC_lexicon_tags_14 },
    { 0x1ee038d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed0038dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2a038de, SUC_lexicon_tags_15 },
    { 0x692038df, SUC_lexicon_tags_45 },
    { 0xb68038e0, SUC_lexicon_tags_17 },
    { 0x26e038e1, SUC_lexicon_tags_15 },
    { 0x37e038e1, SUC_lexicon_tags_55 },
    { 0xaaa038e3, SUC_lexicon_tags_25 },
    { 0x714038e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7e038e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a038ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ba038ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba038f2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462038f6, SUC_lexicon_tags_1 },
    { 0x340038f6, SUC_lexicon_tags_81 },
    { 0x68038f7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x14038fb, SUC_lexicon_tags_68 },
    { 0xd10038fc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a038ff, SUC_lexicon_tags_15 },
    { 0x59003900, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74a03902, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdb803904, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4803906, SUC_lexicon_tags_37 },
    { 0x1e203907, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7280390e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11203914, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd7203916, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x12003918, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc00391c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2e0391e, SUC_lexicon_tags_37 },
    { 0xff20391e, SUC_lexicon_tags_1 },
    { 0x5a603920, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3922, SUC_lexicon_tags_33 },
    { 0xfe603922, SUC_lexicon_tags_0 },
    { 0xa5603924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e03929, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf400392d, SUC_lexicon_tags_36 },
    { 0xadc0392e, SUC_lexicon_tags_33 },
    { 0x6ce0392e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe6a03931, SUC_lexicon_tags_14 },
    { 0x56403931, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9003935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb60393a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba80393e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e03941, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97603944, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfd803946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be0394f, SUC_lexicon_tags_45 },
    { 0x29603950, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41803957, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea20395e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72803960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c203965, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8203967, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c0396a, SUC_lexicon_tags_17 },
    { 0x9100396b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c203974, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c03977, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef40397a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e40397d, SUC_lexicon_tags_25 },
    { 0x3300397d, SUC_lexicon_tags_7 },
    { 0x73e0397d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c03983, SUC_lexicon_tags_9 },
    { 0x87a03984, SUC_lexicon_tags_12 },
    { 0xc403985, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x83803988, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4960398a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6003992, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb803997, SUC_lexicon_tags_305 },
    { 0xce003998, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc60399c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e0399f, SUC_lexicon_tags_37 },
    { 0x2f4039a0, SUC_lexicon_tags_23 },
    { 0x904039a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a039a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc039aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12039b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x636039b4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4039b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99a039ba, SUC_lexicon_tags_11 },
    { 0x8ae039bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28039c1, SUC_lexicon_tags_7 },
    { 0x694039c2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x654039c5, SUC_lexicon_tags_45 },
    { 0x462039c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958039cb, SUC_lexicon_tags_21 },
    { 0xece039cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a039d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18039d5, SUC_lexicon_tags_28 },
    { 0x244039d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3da039d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba039de, SUC_lexicon_tags_19 },
    { 0xbbc039df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2039e2, SUC_lexicon_tags_22 },
    { 0x682039e2, SUC_lexicon_tags_29 },
    { 0xb22039e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4039e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcea039eb, SUC_lexicon_tags_37 },
    { 0x456039ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6e039ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x248039f0, SUC_lexicon_tags_47 },
    { 0xb12039f1, SUC_lexicon_tags_1 },
    { 0xe16039f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce039f5, SUC_lexicon_tags_27 },
    { 0xdca039f6, SUC_lexicon_tags_12 },
    { 0x5ba039f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2b8039f9, SUC_lexicon_tags_7 },
    { 0x738039fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x774039fd, SUC_lexicon_tags_25 },
    { 0xb60039fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6603a00, SUC_lexicon_tags_17 },
    { 0xc4203a01, SUC_lexicon_tags_16 },
    { 0x6a403a02, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24803a06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed003a0a, SUC_lexicon_tags_25 },
    { 0xd9403a0b, SUC_lexicon_tags_9 },
    { 0xd3403a0b, SUC_lexicon_tags_37 },
    { 0x6403a0d, SUC_lexicon_tags_23 },
    { 0x4da03a0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1803a13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37403a16, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e803a19, SUC_lexicon_tags_33 },
    { 0x99a03a19, SUC_lexicon_tags_3 },
    { 0xfa403a1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56803a1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8f003a21, SUC_lexicon_tags_11 },
    { 0x44403a22, SUC_lexicon_tags_15 },
    { 0x7a803a23, SUC_lexicon_tags_31 },
    { 0x47403a23, SUC_lexicon_tags_63 },
    { 0xa7803a25, SUC_lexicon_tags_2 },
    { 0x8ea03a26, SUC_lexicon_tags_1 },
    { 0x80403a21, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28403a36, SUC_lexicon_tags_25 },
    { 0x71a03a37, SUC_lexicon_tags_2 },
    { 0x52203a37, SUC_lexicon_tags_47 },
    { 0xb5203a39, SUC_lexicon_tags_63 },
    { 0xbfe03a37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc03a3e, SUC_lexicon_tags_45 },
    { 0x4cc03a3f, SUC_lexicon_tags_45 },
    { 0x25e03a40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9403a42, SUC_lexicon_tags_105 },
    { 0x71603a43, SUC_lexicon_tags_40 },
    { 0xc7203a44, SUC_lexicon_tags_15 },
    { 0x9c003a45, SUC_lexicon_tags_27 },
    { 0x75803a46, SUC_lexicon_tags_25 },
    { 0x93803a44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffc03a49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9603a4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c03a50, SUC_lexicon_tags_2 },
    { 0x15203a51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e03a54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58203a56, SUC_lexicon_tags_12 },
    { 0xb4603a56, SUC_lexicon_tags_0 },
    { 0x11003a58, SUC_lexicon_tags_2 },
    { 0xeb803a59, SUC_lexicon_tags_60 },
    { 0x5d203a5a, SUC_lexicon_tags_6 },
    { 0x29c03a5b, SUC_lexicon_tags_38 },
    { 0xc4e03a58, SUC_lexicon_tags_60 },
    { 0xf8e03a56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd603a65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7403a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c603a7b, SUC_lexicon_tags_35 },
    { 0x54203a7c, SUC_lexicon_tags_37 },
    { 0x1ae03a7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb603a80, SUC_lexicon_tags_56 },
    { 0xa1803a80, SUC_lexicon_tags_47 },
    { 0x3f403a81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x93803a84, SUC_lexicon_tags_46 },
    { 0x96c03a85, SUC_lexicon_tags_2 },
    { 0xf6203a85, SUC_lexicon_tags_11 },
    { 0x35603a86, SUC_lexicon_tags_37 },
    { 0x37c03a88, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89003a8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52403a90, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x41803a93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca03a96, SUC_lexicon_tags_47 },
    { 0x11803a97, SUC_lexicon_tags_15 },
    { 0xaa603a97, SUC_lexicon_tags_17 },
    { 0x87e03a99, SUC_lexicon_tags_12 },
    { 0x95603a99, SUC_lexicon_tags_15 },
    { 0x16a03a9b, SUC_lexicon_tags_12 },
    { 0xdb603a9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x68c03a9e, SUC_lexicon_tags_31 },
    { 0x9d803a9e, SUC_lexicon_tags_37 },
    { 0xcec03a9f, SUC_lexicon_tags_65 },
    { 0x42e03aa1, SUC_lexicon_tags_17 },
    { 0xaf403aa2, SUC_lexicon_tags_47 },
    { 0x24003aa1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa6a03aa5, SUC_lexicon_tags_19 },
    { 0x45e03aa6, SUC_lexicon_tags_27 },
    { 0xc6c03aa7, SUC_lexicon_tags_25 },
    { 0x37803aa7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xffc03aaa, SUC_lexicon_tags_1 },
    { 0xb203aaa, SUC_lexicon_tags_41 },
    { 0xcf803aac, SUC_lexicon_tags_17 },
    { 0x22803aab, SUC_lexicon_tags_66 },
    { 0x1403aaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1403ab1, SUC_lexicon_tags_55 },
    { 0xcda03ab2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b803ab6, SUC_lexicon_tags_7 },
    { 0xae03ab7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe4403ab9, SUC_lexicon_tags_17 },
    { 0x9c603aba, SUC_lexicon_tags_69 },
    { 0x40603abb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75a03abd, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xda203ac0, SUC_lexicon_tags_11 },
    { 0xc1c03ac0, SUC_lexicon_tags_33 },
    { 0x6d203ac2, SUC_lexicon_tags_29 },
    { 0xba003ac1, SUC_lexicon_tags_39 },
    { 0x98e03ac3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1803ac6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f003ac9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f603acc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ac03ace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8403ad0, SUC_lexicon_tags_11 },
    { 0xe9c03ad1, SUC_lexicon_tags_85 },
    { 0x4403ad2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a203ad6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16803ada, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da03add, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4403ae0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71003ae3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e03ae6, SUC_lexicon_tags_13 },
    { 0x14803ae7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca03aea, SUC_lexicon_tags_19 },
    { 0xd3803aea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27203af5, SUC_lexicon_tags_11 },
    { 0x76403af5, SUC_lexicon_tags_8 },
    { 0x72a03af7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0403afa, SUC_lexicon_tags_29 },
    { 0xb4603afb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a03aff, SUC_lexicon_tags_23 },
    { 0x63203aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2603b05, SUC_lexicon_tags_12 },
    { 0xd6603b06, SUC_lexicon_tags_14 },
    { 0x73603b05, SUC_lexicon_tags_28 },
    { 0x73c03b05, SUC_lexicon_tags_2 },
    { 0xfb403b09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9803b0b, SUC_lexicon_tags_8 },
    { 0xe6e03b0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc03b0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78803b11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20603b18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c03b21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42c03b23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a03b28, SUC_lexicon_tags_11 },
    { 0xd1603b29, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe003b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7803b2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7003b33, SUC_lexicon_tags_38 },
    { 0x2d203b33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8e03b36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3803b3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41003b40, SUC_lexicon_tags_1 },
    { 0xb3a03b40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59003b43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62c03b45, SUC_lexicon_tags_11 },
    { 0x18203b46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52603b4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e403b4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa6e03b51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14403b53, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba03b56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a203b5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96203b5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf7003b60, SUC_lexicon_tags_25 },
    { 0x55203b61, SUC_lexicon_tags_18 },
    { 0x3b403b62, SUC_lexicon_tags_7 },
    { 0xe9603b63, SUC_lexicon_tags_37 },
    { 0xdcc03b63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7e403b66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc603b68, SUC_lexicon_tags_19 },
    { 0x3f003b68, SUC_lexicon_tags_25 },
    { 0x4b403b6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f003b6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee03b75, SUC_lexicon_tags_15 },
    { 0x82203b76, SUC_lexicon_tags_9 },
    { 0x14a03b76, SUC_lexicon_tags_37 },
    { 0xbd803b76, SUC_lexicon_tags_47 },
    { 0xf8e03b75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c803b7b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x70003b7e, SUC_lexicon_tags_31 },
    { 0x2003b7f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf403b82, SUC_lexicon_tags_4 },
    { 0xca403b82, SUC_lexicon_tags_11 },
    { 0x37803b84, SUC_lexicon_tags_23 },
    { 0x9d203b84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee603b8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa03b92, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe03b97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4803b9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4dc03b9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70003ba6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83203baa, SUC_lexicon_tags_0 },
    { 0x3e603baa, SUC_lexicon_tags_25 },
    { 0x9dc03bab, SUC_lexicon_tags_12 },
    { 0xe9803baa, SUC_lexicon_tags_60 },
    { 0xb8a03bae, SUC_lexicon_tags_8 },
    { 0xf3003baf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c03bb3, SUC_lexicon_tags_66 },
    { 0xc203bb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7803bb9, SUC_lexicon_tags_103 },
    { 0x62803bba, SUC_lexicon_tags_33 },
    { 0x34403bbb, SUC_lexicon_tags_17 },
    { 0xa9603bba, SUC_lexicon_tags_12 },
    { 0x1b003bba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88403bc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98e03bc4, SUC_lexicon_tags_1 },
    { 0x94e03bc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38a03bc7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4fe03bc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79803bcd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0003bd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa03bd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0203bda, SUC_lexicon_tags_14 },
    { 0x27403bdb, SUC_lexicon_tags_1 },
    { 0x2ee03bdc, SUC_lexicon_tags_8 },
    { 0xea03bdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c803bdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf0c03be1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a03be5, SUC_lexicon_tags_12 },
    { 0xbc403be6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e03bea, SUC_lexicon_tags_31 },
    { 0x52403bea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2403bef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb803bf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c03bf6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6a803bf8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8403bfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dc03bfe, SUC_lexicon_tags_32 },
    { 0x68003bfe, SUC_lexicon_tags_21 },
    { 0x5b603c00, SUC_lexicon_tags_43 },
    { 0x47c03c01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1d603c03, SUC_lexicon_tags_32 },
    { 0xc8a03c04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb203c06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd403c0c, SUC_lexicon_tags_47 },
    { 0xc1e03c0c, SUC_lexicon_tags_33 },
    { 0x76203c0e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x33203c11, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd003c15, SUC_lexicon_tags_14 },
    { 0x74203c15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfce03c18, SUC_lexicon_tags_17 },
    { 0xdbc03c18, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72203c1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84a03c1f, SUC_lexicon_tags_27 },
    { 0x94003c1f, SUC_lexicon_tags_30 },
    { 0x62a03c1f, SUC_lexicon_tags_7 },
    { 0xd9a03c22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c03c28, SUC_lexicon_tags_17 },
    { 0x15a03c29, SUC_lexicon_tags_7 },
    { 0xe5403c2a, SUC_lexicon_tags_25 },
    { 0x8d603c2a, SUC_lexicon_tags_23 },
    { 0x39803c29, SUC_lexicon_tags_47 },
    { 0xef803c2d, SUC_lexicon_tags_1 },
    { 0xdae03c2a, SUC_lexicon_tags_39 },
    { 0x84003c2f, SUC_lexicon_tags_12 },
    { 0x87603c28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e03c34, SUC_lexicon_tags_15 },
    { 0x36203c35, SUC_lexicon_tags_12 },
    { 0x98203c35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3203c38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaae03c3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5a03c3c, SUC_lexicon_tags_46 },
    { 0xb6c03c3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e03c44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e403c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66603c4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x49803c4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x14e03c51, SUC_lexicon_tags_15 },
    { 0xeb603c51, SUC_lexicon_tags_15 },
    { 0xe1003c51, SUC_lexicon_tags_17 },
    { 0x40c03c51, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2203c59, SUC_lexicon_tags_37 },
    { 0xc403c5a, SUC_lexicon_tags_33 },
    { 0x6b203c5b, SUC_lexicon_tags_17 },
    { 0x2e203c59, SUC_lexicon_tags_23 },
    { 0x9fc03c5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18603c61, SUC_lexicon_tags_4 },
    { 0x92e03c62, SUC_lexicon_tags_17 },
    { 0x1d803c63, SUC_lexicon_tags_15 },
    { 0xeee03c64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe203c6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x55603c6e, SUC_lexicon_tags_2 },
    { 0x2ea03c6f, SUC_lexicon_tags_45 },
    { 0x2b203c6e, SUC_lexicon_tags_19 },
    { 0x68403c6f, SUC_lexicon_tags_49 },
    { 0xca803c71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e03c77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a03c7f, SUC_lexicon_tags_1 },
    { 0xec03c80, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f203c84, SUC_lexicon_tags_56 },
    { 0xb0c03c84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65803c8a, SUC_lexicon_tags_32 },
    { 0x9d603c8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e03c8e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c03c91, SUC_lexicon_tags_4 },
    { 0x6d003c91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56403c96, SUC_lexicon_tags_60 },
    { 0x26e03c97, SUC_lexicon_tags_37 },
    { 0x73603c98, SUC_lexicon_tags_7 },
    { 0x7b603c96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e03c9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f603ca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f203ca5, SUC_lexicon_tags_37 },
    { 0x70403ca6, SUC_lexicon_tags_37 },
    { 0xd603ca7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x78803ca9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef203cab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce803cb0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22203cb7, SUC_lexicon_tags_68 },
    { 0xffc03cb8, SUC_lexicon_tags_37 },
    { 0x9b403cb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a03cbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e03cbf, SUC_lexicon_tags_63 },
    { 0x26803cc0, SUC_lexicon_tags_11 },
    { 0x1f403cc0, SUC_lexicon_tags_11 },
    { 0xef803cc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e03cc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50803cc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d203cd5, SUC_lexicon_tags_11 },
    { 0xcfa03cd6, SUC_lexicon_tags_7 },
    { 0x27603cd7, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x82203cd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc03cdf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3b403ce1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49003ce3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3d403ce5, SUC_lexicon_tags_49 },
    { 0x7ce03ce6, SUC_lexicon_tags_10 },
    { 0xd9803ce6, SUC_lexicon_tags_37 },
    { 0xc603ce8, SUC_lexicon_tags_32 },
    { 0x3b003ce9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a03ced, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25203cf1, SUC_lexicon_tags_31 },
    { 0xcae03cf2, SUC_lexicon_tags_27 },
    { 0x7e403cf3, SUC_lexicon_tags_7 },
    { 0xae003cf4, SUC_lexicon_tags_16 },
    { 0x2f203cf1, SUC_lexicon_tags_30 },
    { 0x50203cf3, SUC_lexicon_tags_17 },
    { 0x10603cf7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x35c03cf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e03cfc, SUC_lexicon_tags_37 },
    { 0xd5e03cfd, SUC_lexicon_tags_37 },
    { 0x90203cfd, SUC_lexicon_tags_15 },
    { 0x29403cff, SUC_lexicon_tags_12 },
    { 0x3003d00, SUC_lexicon_tags_15 },
    { 0x4c203d00, SUC_lexicon_tags_12 },
    { 0x87003d01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25003d09, SUC_lexicon_tags_66 },
    { 0x51003d0a, SUC_lexicon_tags_55 },
    { 0x35e03d0b, SUC_lexicon_tags_29 },
    { 0xa6603d0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cc03d0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a03d11, SUC_lexicon_tags_15 },
    { 0x77803d12, SUC_lexicon_tags_15 },
    { 0x85203d13, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf603d16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57403d18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e03d1b, SUC_lexicon_tags_25 },
    { 0x5e203d1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f203d1e, SUC_lexicon_tags_33 },
    { 0xa7e03d1e, SUC_lexicon_tags_27 },
    { 0xe8c03d20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fa03d22, SUC_lexicon_tags_1 },
    { 0xb0203d23, SUC_lexicon_tags_15 },
    { 0x48603d22, SUC_lexicon_tags_0 },
    { 0xfa03d24, SUC_lexicon_tags_35 },
    { 0xb4603d22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc603d2b, SUC_lexicon_tags_17 },
    { 0x3b803d2b, SUC_lexicon_tags_32 },
    { 0xf9c03d2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77203d2f, SUC_lexicon_tags_27 },
    { 0x11403d30, SUC_lexicon_tags_29 },
    { 0x3d2f, SUC_lexicon_tags_65 },
    { 0x7fe03d2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4803d34, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0203d3a, SUC_lexicon_tags_37 },
    { 0xaf403d3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x48803d3e, SUC_lexicon_tags_8 },
    { 0x1dc03d3e, SUC_lexicon_tags_39 },
    { 0xc3203d3f, SUC_lexicon_tags_33 },
    { 0xf8e03d41, SUC_lexicon_tags_12 },
    { 0x1203d40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14603d46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea03d4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12003d5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbac03d5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c603d64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc03d68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x64203d6b, SUC_lexicon_tags_27 },
    { 0x2f403d6c, SUC_lexicon_tags_43 },
    { 0x1d603d6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0003d73, SUC_lexicon_tags_6 },
    { 0x90003d74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd603d76, SUC_lexicon_tags_14 },
    { 0x17003d77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c03d7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95803d7d, SUC_lexicon_tags_60 },
    { 0x10603d7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca03d82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec03d85, SUC_lexicon_tags_9 },
    { 0x16203d86, SUC_lexicon_tags_15 },
    { 0x36e03d87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xec003d8a, SUC_lexicon_tags_14 },
    { 0xef803d8b, SUC_lexicon_tags_37 },
    { 0xb7203d8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa203d91, SUC_lexicon_tags_17 },
    { 0xf6603d92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56403d96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a03d9a, SUC_lexicon_tags_46 },
    { 0xd8803d9b, SUC_lexicon_tags_19 },
    { 0xdea03d9c, SUC_lexicon_tags_17 },
    { 0xda603d9b, SUC_lexicon_tags_27 },
    { 0x6a603d9e, SUC_lexicon_tags_30 },
    { 0x6803d9f, SUC_lexicon_tags_1 },
    { 0x5a203d9b, SUC_lexicon_tags_30 },
    { 0xe1c03da1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73e03da3, SUC_lexicon_tags_25 },
    { 0xd4403da4, SUC_lexicon_tags_25 },
    { 0xbd003da5, SUC_lexicon_tags_45 },
    { 0xc4403da6, SUC_lexicon_tags_31 },
    { 0xd9c03da3, SUC_lexicon_tags_27 },
    { 0x5da03da6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc03daf, SUC_lexicon_tags_2 },
    { 0x73c03daf, SUC_lexicon_tags_1 },
    { 0x9d803db1, SUC_lexicon_tags_49 },
    { 0x12603db2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69403db4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeac03db6, SUC_lexicon_tags_38 },
    { 0xcb803db7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c03dbb, SUC_lexicon_tags_11 },
    { 0x6a03dbc, SUC_lexicon_tags_0 },
    { 0x3f003dbb, SUC_lexicon_tags_25 },
    { 0x85203dbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68403dc3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1e003dc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20203dce, SUC_lexicon_tags_12 },
    { 0xce803dce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58203dd6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c03ddb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a03ddf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda403de3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70803de5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9403de9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97603df3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x48403df5, SUC_lexicon_tags_15 },
    { 0x6da03df5, SUC_lexicon_tags_20 },
    { 0x3c803df7, SUC_lexicon_tags_11 },
    { 0x66a03df7, SUC_lexicon_tags_7 },
    { 0x53203df5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84203dfb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea03e02, SUC_lexicon_tags_1 },
    { 0x60603e03, SUC_lexicon_tags_39 },
    { 0xadc03e04, SUC_lexicon_tags_2 },
    { 0xb1403e04, SUC_lexicon_tags_43 },
    { 0x4b603e02, SUC_lexicon_tags_17 },
    { 0xe403e07, SUC_lexicon_tags_25 },
    { 0x33603e07, SUC_lexicon_tags_10 },
    { 0xf1203e02, SUC_lexicon_tags_55 },
    { 0x1a803e0a, SUC_lexicon_tags_11 },
    { 0xc9203e02, SUC_lexicon_tags_108 },
    { 0x3d603e0c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x38a03e0e, SUC_lexicon_tags_12 },
    { 0x92803e0e, SUC_lexicon_tags_43 },
    { 0xd3e03e10, SUC_lexicon_tags_37 },
    { 0xe9403e11, SUC_lexicon_tags_12 },
    { 0x3a03e12, SUC_lexicon_tags_25 },
    { 0x94a03e13, SUC_lexicon_tags_15 },
    { 0x6aa03e0f, SUC_lexicon_tags_38 },
    { 0x88003e15, SUC_lexicon_tags_1 },
    { 0xea203e11, SUC_lexicon_tags_17 },
    { 0xa803e17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab603e1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3803e1e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2603e27, SUC_lexicon_tags_14 },
    { 0x74e03e28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6203e2a, SUC_lexicon_tags_47 },
    { 0xe003e2a, SUC_lexicon_tags_9 },
    { 0x4fa03e2c, SUC_lexicon_tags_37 },
    { 0xf0603e2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18e03e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c03e31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc03e34, SUC_lexicon_tags_55 },
    { 0x73e03e34, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c03e38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c03e3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97403e41, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9f403e43, SUC_lexicon_tags_17 },
    { 0x31a03e43, SUC_lexicon_tags_3 },
    { 0x11803e43, SUC_lexicon_tags_37 },
    { 0xcaa03e46, SUC_lexicon_tags_17 },
    { 0xd3e03e46, SUC_lexicon_tags_25 },
    { 0xc8a03e48, SUC_lexicon_tags_2 },
    { 0x76a03e48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15003e4c, SUC_lexicon_tags_46 },
    { 0x1d603e4d, SUC_lexicon_tags_37 },
    { 0xea403e4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e03e51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e203e54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7c03e56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab203e58, SUC_lexicon_tags_33 },
    { 0x70603e58, SUC_lexicon_tags_47 },
    { 0x803e58, SUC_lexicon_tags_1 },
    { 0xcea03e59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9003e5d, SUC_lexicon_tags_12 },
    { 0xd0c03e5e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52603e63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c03e6c, SUC_lexicon_tags_65 },
    { 0x47003e6d, SUC_lexicon_tags_17 },
    { 0xc8c03e6e, SUC_lexicon_tags_2 },
    { 0x53803e6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79403e71, SUC_lexicon_tags_8 },
    { 0xa8803e72, SUC_lexicon_tags_17 },
    { 0xd4603e73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e03e7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7803e82, SUC_lexicon_tags_16 },
    { 0xcfa03e82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e03e8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14003e93, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfd403e95, SUC_lexicon_tags_11 },
    { 0x37203e95, SUC_lexicon_tags_2 },
    { 0x39603e95, SUC_lexicon_tags_250 },
    { 0x5aa03e98, SUC_lexicon_tags_77 },
    { 0xb5403e99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce603e9d, SUC_lexicon_tags_63 },
    { 0x53e03e9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba03ea4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89a03ea6, SUC_lexicon_tags_11 },
    { 0xccc03ea6, SUC_lexicon_tags_16 },
    { 0x3d803ea8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60003eab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8603eb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a03eb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd803eb5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb7403eb7, SUC_lexicon_tags_0 },
    { 0x53203eb7, SUC_lexicon_tags_11 },
    { 0x5d603eb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b403ecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38c03ecf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15203ed1, SUC_lexicon_tags_47 },
    { 0xd1203ed1, SUC_lexicon_tags_30 },
    { 0x79603ed3, SUC_lexicon_tags_1 },
    { 0xb8c03ed4, SUC_lexicon_tags_7 },
    { 0xb6003ed5, SUC_lexicon_tags_47 },
    { 0x58403ed5, SUC_lexicon_tags_14 },
    { 0x8ee03ed5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20603edc, SUC_lexicon_tags_17 },
    { 0xaaa03edc, SUC_lexicon_tags_46 },
    { 0x47403edc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3003ee0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e03ee4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49003ee8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c603eec, SUC_lexicon_tags_27 },
    { 0x8fe03eed, SUC_lexicon_tags_60 },
    { 0x3e203eee, SUC_lexicon_tags_37 },
    { 0x86403eee, SUC_lexicon_tags_29 },
    { 0x89603ef0, SUC_lexicon_tags_65 },
    { 0xbd603eee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc03ef4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d603ef7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x74803efa, SUC_lexicon_tags_37 },
    { 0x3b403efb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0003eff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e003f01, SUC_lexicon_tags_37 },
    { 0xfc603f01, SUC_lexicon_tags_25 },
    { 0x3f603f01, SUC_lexicon_tags_14 },
    { 0x27203f03, SUC_lexicon_tags_60 },
    { 0xf1a03f05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0803f09, SUC_lexicon_tags_47 },
    { 0x6c203f0a, SUC_lexicon_tags_10 },
    { 0x8a603f0a, SUC_lexicon_tags_2 },
    { 0xae203f09, SUC_lexicon_tags_140 },
    { 0xe403f0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7fa03f0f, SUC_lexicon_tags_14 },
    { 0x51403f0f, SUC_lexicon_tags_6 },
    { 0xff003f11, SUC_lexicon_tags_8 },
    { 0x64c03f10, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43003f19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58603f1d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7603f20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f803f29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f603f2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78203f31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34603f38, SUC_lexicon_tags_14 },
    { 0xbc603f39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5fe03f3b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x14603f3e, SUC_lexicon_tags_1 },
    { 0x74603f3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed403f42, SUC_lexicon_tags_11 },
    { 0x65e03f43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d203f45, SUC_lexicon_tags_7 },
    { 0xe2803f46, SUC_lexicon_tags_12 },
    { 0xb1c03f45, SUC_lexicon_tags_12 },
    { 0x5b003f46, SUC_lexicon_tags_26 },
    { 0x71a03f49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef403f4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4a03f4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef403f51, SUC_lexicon_tags_60 },
    { 0x63203f52, SUC_lexicon_tags_11 },
    { 0x18e03f53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3003f56, SUC_lexicon_tags_47 },
    { 0x5da03f56, SUC_lexicon_tags_33 },
    { 0x2a203f58, SUC_lexicon_tags_37 },
    { 0x44e03f59, SUC_lexicon_tags_17 },
    { 0x97803f58, SUC_lexicon_tags_11 },
    { 0xb3603f57, SUC_lexicon_tags_25 },
    { 0xef603f5c, SUC_lexicon_tags_19 },
    { 0x47803f5c, SUC_lexicon_tags_29 },
    { 0xb5003f5c, SUC_lexicon_tags_37 },
    { 0x4f203f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee03f67, SUC_lexicon_tags_29 },
    { 0xa7203f67, SUC_lexicon_tags_37 },
    { 0x60403f67, SUC_lexicon_tags_37 },
    { 0xdb003f6a, SUC_lexicon_tags_14 },
    { 0x88203f6b, SUC_lexicon_tags_7 },
    { 0xe7003f69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78003f6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ae03f71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51603f7a, SUC_lexicon_tags_45 },
    { 0xdee03f7b, SUC_lexicon_tags_15 },
    { 0x4ec03f7c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xad403f7e, SUC_lexicon_tags_14 },
    { 0x1a03f7f, SUC_lexicon_tags_11 },
    { 0xf2603f80, SUC_lexicon_tags_1 },
    { 0xe6a03f81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c03f89, SUC_lexicon_tags_70 },
    { 0x10403f8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27803f8e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xba03f90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23403f95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17203f97, SUC_lexicon_tags_56 },
    { 0xd4a03f97, SUC_lexicon_tags_2 },
    { 0x39c03f98, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c03f9d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4603fa0, SUC_lexicon_tags_17 },
    { 0x9b803fa1, SUC_lexicon_tags_3 },
    { 0x43a03fa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa603fa4, SUC_lexicon_tags_37 },
    { 0x16803fa5, SUC_lexicon_tags_11 },
    { 0x61403fa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc2e03fa8, SUC_lexicon_tags_37 },
    { 0xd8a03fa9, SUC_lexicon_tags_37 },
    { 0xfa603faa, SUC_lexicon_tags_32 },
    { 0xcc803faa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e03fae, SUC_lexicon_tags_13 },
    { 0xbd003faf, SUC_lexicon_tags_25 },
    { 0x1cc03faf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0603fb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87603fbb, SUC_lexicon_tags_36 },
    { 0x3d603fbc, SUC_lexicon_tags_11 },
    { 0x75003fbd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe03fc0, SUC_lexicon_tags_7 },
    { 0x27803fc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1203fc5, SUC_lexicon_tags_19 },
    { 0x34003fc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c03fcb, SUC_lexicon_tags_13 },
    { 0x7ee03fcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d803fce, SUC_lexicon_tags_23 },
    { 0xd9a03fcf, SUC_lexicon_tags_15 },
    { 0xdda03fd0, SUC_lexicon_tags_30 },
    { 0xea603fd1, SUC_lexicon_tags_212 },
    { 0xa203fd2, SUC_lexicon_tags_48 },
    { 0x29603fd3, SUC_lexicon_tags_49 },
    { 0xc3603fd4, SUC_lexicon_tags_23 },
    { 0x6f403fd2, SUC_lexicon_tags_46 },
    { 0x2c03fd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x14603fd8, SUC_lexicon_tags_7 },
    { 0xd0003fd9, SUC_lexicon_tags_16 },
    { 0x8b603fda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65403fdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a03fe1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9b203fe3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a03fe6, SUC_lexicon_tags_25 },
    { 0xbf803fe7, SUC_lexicon_tags_11 },
    { 0x73003fe6, SUC_lexicon_tags_2 },
    { 0xd3003fe9, SUC_lexicon_tags_6 },
    { 0xed603fea, SUC_lexicon_tags_37 },
    { 0x2f603feb, SUC_lexicon_tags_25 },
    { 0x99c03feb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xe2603fee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a03ff2, SUC_lexicon_tags_38 },
    { 0xf1e03ff3, SUC_lexicon_tags_37 },
    { 0xefc03ff4, SUC_lexicon_tags_39 },
    { 0x72003ff5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f403ff7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa403ff9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a04001, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9e04003, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d80400b, SUC_lexicon_tags_28 },
    { 0x8e40400b, SUC_lexicon_tags_6 },
    { 0x360400b, SUC_lexicon_tags_2 },
    { 0xce80400e, SUC_lexicon_tags_15 },
    { 0xe180400c, SUC_lexicon_tags_19 },
    { 0x93804010, SUC_lexicon_tags_2 },
    { 0x87e04011, SUC_lexicon_tags_1 },
    { 0xf2604011, SUC_lexicon_tags_63 },
    { 0xa404010, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e0401e, SUC_lexicon_tags_43 },
    { 0xba00401f, SUC_lexicon_tags_32 },
    { 0x25a04020, SUC_lexicon_tags_20 },
    { 0xb5a04021, SUC_lexicon_tags_7 },
    { 0x82604022, SUC_lexicon_tags_49 },
    { 0x1ec0401f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e0402b, SUC_lexicon_tags_17 },
    { 0xa5e0402c, SUC_lexicon_tags_27 },
    { 0x3d40402d, SUC_lexicon_tags_26 },
    { 0x8320402e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfa04030, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5004034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc0403a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb020403f, SUC_lexicon_tags_46 },
    { 0x1e804040, SUC_lexicon_tags_11 },
    { 0x53004041, SUC_lexicon_tags_9 },
    { 0xa7404040, SUC_lexicon_tags_60 },
    { 0x58204041, SUC_lexicon_tags_19 },
    { 0x24804040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93404046, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca0404a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe760404c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0404f, SUC_lexicon_tags_27 },
    { 0x8300404f, SUC_lexicon_tags_19 },
    { 0x72804051, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa0e04053, SUC_lexicon_tags_151 },
    { 0xad004054, SUC_lexicon_tags_55 },
    { 0x4f804053, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb8204057, SUC_lexicon_tags_63 },
    { 0xac604058, SUC_lexicon_tags_11 },
    { 0xf0604059, SUC_lexicon_tags_7 },
    { 0xf1804059, SUC_lexicon_tags_10 },
    { 0xf660405b, SUC_lexicon_tags_20 },
    { 0xf560405c, SUC_lexicon_tags_1 },
    { 0xcc20405d, SUC_lexicon_tags_7 },
    { 0x4a00405e, SUC_lexicon_tags_8 },
    { 0x8c20405e, SUC_lexicon_tags_37 },
    { 0x15e0405e, SUC_lexicon_tags_2 },
    { 0x32a04061, SUC_lexicon_tags_26 },
    { 0x2d00405d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3420406a, SUC_lexicon_tags_30 },
    { 0x4c20406a, SUC_lexicon_tags_17 },
    { 0x8680406a, SUC_lexicon_tags_68 },
    { 0xca20406c, SUC_lexicon_tags_71 },
    { 0x1220406e, SUC_lexicon_tags_47 },
    { 0x7c60406d, SUC_lexicon_tags_12 },
    { 0x20e04070, SUC_lexicon_tags_2 },
    { 0x10c0406a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa04079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54a0407b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82804082, SUC_lexicon_tags_37 },
    { 0x1de04082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39204086, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3e04088, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc620408a, SUC_lexicon_tags_11 },
    { 0xb980408b, SUC_lexicon_tags_17 },
    { 0x580408c, SUC_lexicon_tags_37 },
    { 0xcb60408d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17004094, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e0409c, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x6300409f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74040a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e040a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890040a9, SUC_lexicon_tags_31 },
    { 0x8a2040aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc040ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86040b1, SUC_lexicon_tags_25 },
    { 0xeec040b2, SUC_lexicon_tags_364 },
    { 0xd34040b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66040b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76e040b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48040bb, SUC_lexicon_tags_18 },
    { 0x3fc040bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0040bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e040c6, SUC_lexicon_tags_2 },
    { 0xc3e040c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0040cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0040d7, SUC_lexicon_tags_55 },
    { 0xee8040d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa040db, SUC_lexicon_tags_39 },
    { 0x4b0040db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa040e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc92040e5, SUC_lexicon_tags_11 },
    { 0x708040e6, SUC_lexicon_tags_55 },
    { 0x78a040e6, SUC_lexicon_tags_63 },
    { 0x680040e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce040fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x760040ff, SUC_lexicon_tags_59 },
    { 0x16e04100, SUC_lexicon_tags_33 },
    { 0x43e04101, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5600410a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6204113, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c04116, SUC_lexicon_tags_12 },
    { 0xb3604117, SUC_lexicon_tags_37 },
    { 0xe0204118, SUC_lexicon_tags_7 },
    { 0xba004118, SUC_lexicon_tags_15 },
    { 0xb1604117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fe0411c, SUC_lexicon_tags_12 },
    { 0x34c0411d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x600411f, SUC_lexicon_tags_11 },
    { 0x99a04120, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3804123, SUC_lexicon_tags_77 },
    { 0x4e04124, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e04127, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc0412f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe5404131, SUC_lexicon_tags_53 },
    { 0x2204131, SUC_lexicon_tags_11 },
    { 0x84804133, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38804139, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3200413c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a04140, SUC_lexicon_tags_8 },
    { 0xa8004140, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x83004144, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c0414c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87604154, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x204156, SUC_lexicon_tags_9 },
    { 0x13404156, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb9c04159, SUC_lexicon_tags_2 },
    { 0x7060415a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9620415d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x84e0415f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f804165, SUC_lexicon_tags_25 },
    { 0xcf404165, SUC_lexicon_tags_55 },
    { 0x8a804167, SUC_lexicon_tags_25 },
    { 0xdf604168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c0416c, SUC_lexicon_tags_31 },
    { 0x6ae0416d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce04174, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f604176, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x18604178, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c0417f, SUC_lexicon_tags_26 },
    { 0xaa404180, SUC_lexicon_tags_11 },
    { 0xc5804181, SUC_lexicon_tags_23 },
    { 0xc3e04182, SUC_lexicon_tags_45 },
    { 0xa0804180, SUC_lexicon_tags_2 },
    { 0xa3204184, SUC_lexicon_tags_11 },
    { 0x73c0417f, SUC_lexicon_tags_138 },
    { 0xa160417f, SUC_lexicon_tags_17 },
    { 0xda04186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd80418f, SUC_lexicon_tags_27 },
    { 0xe0e04190, SUC_lexicon_tags_21 },
    { 0x85e04191, SUC_lexicon_tags_148 },
    { 0x86404192, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x92004195, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd204198, SUC_lexicon_tags_68 },
    { 0xb3604198, SUC_lexicon_tags_57 },
    { 0xec40419a, SUC_lexicon_tags_17 },
    { 0xb8a0419a, SUC_lexicon_tags_17 },
    { 0xa940419a, SUC_lexicon_tags_0 },
    { 0x8ee0419d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f60419f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95c041a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x22c041a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108041a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2041ae, SUC_lexicon_tags_12 },
    { 0x630041ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x270041b2, SUC_lexicon_tags_1 },
    { 0xe54041b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80041bb, SUC_lexicon_tags_37 },
    { 0x22a041bc, SUC_lexicon_tags_64 },
    { 0xa22041bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e041c4, SUC_lexicon_tags_1 },
    { 0x812041c5, SUC_lexicon_tags_37 },
    { 0x27e041c6, SUC_lexicon_tags_12 },
    { 0x654041c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x372041ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4041cf, SUC_lexicon_tags_6 },
    { 0xe14041cf, SUC_lexicon_tags_16 },
    { 0xf56041d0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a041d4, SUC_lexicon_tags_1 },
    { 0xd96041d4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0041d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e041dd, SUC_lexicon_tags_1 },
    { 0xc96041dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbda041e0, SUC_lexicon_tags_47 },
    { 0x806041e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x750041e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c041e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c2041e8, SUC_lexicon_tags_33 },
    { 0x28e041e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e041ec, SUC_lexicon_tags_11 },
    { 0xea4041ed, SUC_lexicon_tags_129 },
    { 0x904041ee, SUC_lexicon_tags_68 },
    { 0xe98041ed, SUC_lexicon_tags_37 },
    { 0x2f8041ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2041f3, SUC_lexicon_tags_13 },
    { 0x6d2041f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x770041f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c041f9, SUC_lexicon_tags_37 },
    { 0xeda041fa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x40041fc, SUC_lexicon_tags_201 },
    { 0xbdc041fd, SUC_lexicon_tags_11 },
    { 0x80a041fd, SUC_lexicon_tags_11 },
    { 0x64a041ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e04202, SUC_lexicon_tags_23 },
    { 0xd6e04202, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c04208, SUC_lexicon_tags_2 },
    { 0x2b204208, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a0420c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c0420f, SUC_lexicon_tags_68 },
    { 0xf7a0420f, SUC_lexicon_tags_25 },
    { 0x7f00420f, SUC_lexicon_tags_7 },
    { 0x6804212, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadc04214, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61204217, SUC_lexicon_tags_45 },
    { 0x7aa04218, SUC_lexicon_tags_40 },
    { 0x70e04219, SUC_lexicon_tags_11 },
    { 0xfec0421a, SUC_lexicon_tags_37 },
    { 0x26e0421b, SUC_lexicon_tags_26 },
    { 0xef40421c, SUC_lexicon_tags_55 },
    { 0xd4004218, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa04220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5804225, SUC_lexicon_tags_29 },
    { 0x45a04226, SUC_lexicon_tags_12 },
    { 0x51004227, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d804229, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3180422b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38804231, SUC_lexicon_tags_16 },
    { 0xb6604232, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c04235, SUC_lexicon_tags_27 },
    { 0x4f604236, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x86e04238, SUC_lexicon_tags_43 },
    { 0xa7c04238, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0423b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfd20423d, SUC_lexicon_tags_17 },
    { 0xf2e0423e, SUC_lexicon_tags_0 },
    { 0x640423f, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5804244, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf40424e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce04250, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1004252, SUC_lexicon_tags_25 },
    { 0x24e04253, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x5c204255, SUC_lexicon_tags_17 },
    { 0x9b604256, SUC_lexicon_tags_1 },
    { 0x31404255, SUC_lexicon_tags_57 },
    { 0x40204258, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1360425a, SUC_lexicon_tags_25 },
    { 0xaa00425b, SUC_lexicon_tags_37 },
    { 0xc700425c, SUC_lexicon_tags_33 },
    { 0x5d80425b, SUC_lexicon_tags_61 },
    { 0xbec0425e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99204264, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a04267, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee0426d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7380426f, SUC_lexicon_tags_36 },
    { 0x5b604270, SUC_lexicon_tags_1 },
    { 0x60004271, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf8804273, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda604276, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa04279, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf320427b, SUC_lexicon_tags_16 },
    { 0xee40427c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c804280, SUC_lexicon_tags_11 },
    { 0x77804280, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea04290, SUC_lexicon_tags_57 },
    { 0x63404291, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b404296, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e04299, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0429c, SUC_lexicon_tags_7 },
    { 0xa6c0429d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x214042a0, SUC_lexicon_tags_14 },
    { 0xb54042a1, SUC_lexicon_tags_2 },
    { 0xcfe042a2, SUC_lexicon_tags_37 },
    { 0x5b6042a2, SUC_lexicon_tags_9 },
    { 0xc1a042a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56042a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaaa042a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4042aa, SUC_lexicon_tags_19 },
    { 0x8ca042ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2042ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74042b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66042b4, SUC_lexicon_tags_15 },
    { 0xc34042b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa68042b7, SUC_lexicon_tags_33 },
    { 0x9ee042b7, SUC_lexicon_tags_17 },
    { 0x1ba042b7, SUC_lexicon_tags_4 },
    { 0x242042ba, SUC_lexicon_tags_15 },
    { 0x64c042ba, SUC_lexicon_tags_2 },
    { 0x988042bc, SUC_lexicon_tags_25 },
    { 0x664042bb, SUC_lexicon_tags_82 },
    { 0x6f6042be, SUC_lexicon_tags_1 },
    { 0xbe8042bf, SUC_lexicon_tags_6 },
    { 0xac042bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6042c2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160042c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb6042c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0042cb, SUC_lexicon_tags_21 },
    { 0xf36042cc, SUC_lexicon_tags_82 },
    { 0xd1c042cc, SUC_lexicon_tags_25 },
    { 0xb8e042ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x314042d0, SUC_lexicon_tags_31 },
    { 0x53a042d1, SUC_lexicon_tags_9 },
    { 0xacc042d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04042d5, SUC_lexicon_tags_33 },
    { 0x94e042d6, SUC_lexicon_tags_63 },
    { 0x4ca042d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e0042d9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0042dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa66042df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0042e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8042ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550042f0, SUC_lexicon_tags_4 },
    { 0xea2042f0, SUC_lexicon_tags_155 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0042f5, SUC_lexicon_tags_11 },
    { 0xfa8042f6, SUC_lexicon_tags_37 },
    { 0x756042f7, SUC_lexicon_tags_17 },
    { 0x1a6042f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76204301, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87404303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98204308, SUC_lexicon_tags_27 },
    { 0xd1c04309, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a04313, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f204316, SUC_lexicon_tags_66 },
    { 0x72804317, SUC_lexicon_tags_12 },
    { 0xcd204317, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9e00431a, SUC_lexicon_tags_33 },
    { 0x5ea0431a, SUC_lexicon_tags_17 },
    { 0xd180431b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e04322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93004324, SUC_lexicon_tags_56 },
    { 0x28004325, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91604327, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa404329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b40432b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d804331, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc404337, SUC_lexicon_tags_179 },
    { 0x15e04338, SUC_lexicon_tags_25 },
    { 0xce404339, SUC_lexicon_tags_55 },
    { 0xce204339, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51804340, SUC_lexicon_tags_0 },
    { 0x86404341, SUC_lexicon_tags_31 },
    { 0xf2404342, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe04345, SUC_lexicon_tags_6 },
    { 0x87204345, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a80434b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde004350, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a04353, SUC_lexicon_tags_60 },
    { 0x1e004354, SUC_lexicon_tags_11 },
    { 0xa3404354, SUC_lexicon_tags_30 },
    { 0x59e04354, SUC_lexicon_tags_37 },
    { 0xd2804357, SUC_lexicon_tags_40 },
    { 0x26c04357, SUC_lexicon_tags_61 },
    { 0x4a04354, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb140435e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x29804360, SUC_lexicon_tags_27 },
    { 0xae404361, SUC_lexicon_tags_27 },
    { 0x28c04361, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x32804365, SUC_lexicon_tags_39 },
    { 0xe1804366, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0436d, SUC_lexicon_tags_8 },
    { 0x3980436e, SUC_lexicon_tags_43 },
    { 0x6480436e, SUC_lexicon_tags_38 },
    { 0xc5604370, SUC_lexicon_tags_23 },
    { 0xd5c04370, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7804376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c0437a, SUC_lexicon_tags_25 },
    { 0x1d40437b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a04383, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79c04385, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2804389, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x5ee0438b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c04391, SUC_lexicon_tags_25 },
    { 0xcca04392, SUC_lexicon_tags_15 },
    { 0x78404393, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f604398, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2700439b, SUC_lexicon_tags_2 },
    { 0xa040439c, SUC_lexicon_tags_69 },
    { 0x8ea0439d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc380439f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64c043a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ec043a3, SUC_lexicon_tags_34 },
    { 0x9f2043a4, SUC_lexicon_tags_14 },
    { 0x86e043a5, SUC_lexicon_tags_1 },
    { 0x27e043a6, SUC_lexicon_tags_27 },
    { 0x74a043a7, SUC_lexicon_tags_37 },
    { 0x6fc043a3, SUC_lexicon_tags_7 },
    { 0x6fa043a9, SUC_lexicon_tags_38 },
    { 0x4d6043aa, SUC_lexicon_tags_29 },
    { 0xdf6043a8, SUC_lexicon_tags_15 },
    { 0xd70043ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xccc043ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12043b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe043bd, SUC_lexicon_tags_12 },
    { 0x2ac043be, SUC_lexicon_tags_15 },
    { 0x214043bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee043c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x918043c5, SUC_lexicon_tags_12 },
    { 0x692043c6, SUC_lexicon_tags_12 },
    { 0x178043c7, SUC_lexicon_tags_108 },
    { 0x798043c7, SUC_lexicon_tags_11 },
    { 0x62c043c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x350043cb, SUC_lexicon_tags_23 },
    { 0x928043cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa043d0, SUC_lexicon_tags_60 },
    { 0x9a043d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6043d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0043dc, SUC_lexicon_tags_7 },
    { 0xe4043dd, SUC_lexicon_tags_11 },
    { 0x2e6043de, SUC_lexicon_tags_8 },
    { 0x508043de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbe043e1, SUC_lexicon_tags_6 },
    { 0x630043e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4c0043e4, SUC_lexicon_tags_37 },
    { 0x3a4043e5, SUC_lexicon_tags_17 },
    { 0xad0043e6, SUC_lexicon_tags_14 },
    { 0xac2043e7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20043eb, SUC_lexicon_tags_4 },
    { 0x8e2043eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc043ef, SUC_lexicon_tags_35 },
    { 0x7fe043ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d2043f2, SUC_lexicon_tags_37 },
    { 0x1b0043f3, SUC_lexicon_tags_3 },
    { 0x572043f4, SUC_lexicon_tags_15 },
    { 0xc8a043f4, SUC_lexicon_tags_17 },
    { 0x826043f4, SUC_lexicon_tags_33 },
    { 0x5d2043f4, SUC_lexicon_tags_39 },
    { 0x904043f7, SUC_lexicon_tags_25 },
    { 0x8f4043f6, SUC_lexicon_tags_25 },
    { 0xf1e043fa, SUC_lexicon_tags_25 },
    { 0xddc043f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc04400, SUC_lexicon_tags_8 },
    { 0xc6a04400, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb604404, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e04408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd804410, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5604414, SUC_lexicon_tags_43 },
    { 0x40004415, SUC_lexicon_tags_15 },
    { 0xb4804414, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96c04418, SUC_lexicon_tags_178 },
    { 0x1ea04418, SUC_lexicon_tags_1 },
    { 0x48c0441a, SUC_lexicon_tags_28 },
    { 0xc1a04418, SUC_lexicon_tags_7 },
    { 0xef20441a, SUC_lexicon_tags_37 },
    { 0xf540441d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7200441f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0404424, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400442d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc404433, SUC_lexicon_tags_9 },
    { 0x4b604434, SUC_lexicon_tags_25 },
    { 0x3e604434, SUC_lexicon_tags_82 },
    { 0xa3a04436, SUC_lexicon_tags_7 },
    { 0x24004437, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xd780443a, SUC_lexicon_tags_0 },
    { 0x6040443a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee60443d, SUC_lexicon_tags_26 },
    { 0x41c0443e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4ca04440, SUC_lexicon_tags_22 },
    { 0xf604440, SUC_lexicon_tags_63 },
    { 0x34e04442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4404444, SUC_lexicon_tags_27 },
    { 0x67e04445, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf20444a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18004451, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2804453, SUC_lexicon_tags_37 },
    { 0xc2a04454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74e04456, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a00445a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a0445d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5804460, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde004462, SUC_lexicon_tags_29 },
    { 0x7d004463, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40004465, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94604469, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9580446b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee604473, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6004475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac04479, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x22c0447b, SUC_lexicon_tags_11 },
    { 0x6860447c, SUC_lexicon_tags_29 },
    { 0x9fe0447d, SUC_lexicon_tags_17 },
    { 0x7b80447c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ae04480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a204483, SUC_lexicon_tags_15 },
    { 0xbdc04484, SUC_lexicon_tags_23 },
    { 0xbb804485, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e0448a, SUC_lexicon_tags_7 },
    { 0x6fc0448b, SUC_lexicon_tags_15 },
    { 0xa080448c, SUC_lexicon_tags_15 },
    { 0xf360448c, SUC_lexicon_tags_31 },
    { 0x98e0448e, SUC_lexicon_tags_36 },
    { 0x1b20448f, SUC_lexicon_tags_15 },
    { 0x58604490, SUC_lexicon_tags_30 },
    { 0x9660448c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec04498, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e80449d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a044a0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e044a3, SUC_lexicon_tags_15 },
    { 0x2ce044a4, SUC_lexicon_tags_45 },
    { 0x8cc044a5, SUC_lexicon_tags_38 },
    { 0xae8044a6, SUC_lexicon_tags_63 },
    { 0x74044a7, SUC_lexicon_tags_12 },
    { 0x8b0044a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a044ae, SUC_lexicon_tags_15 },
    { 0x31a044af, SUC_lexicon_tags_38 },
    { 0x7c4044af, SUC_lexicon_tags_18 },
    { 0x2a0044b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7a2044b3, SUC_lexicon_tags_103 },
    { 0x34a044b4, SUC_lexicon_tags_44 },
    { 0xa8c044b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x29a044b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c044bd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe044c4, SUC_lexicon_tags_15 },
    { 0x472044c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e044cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86044d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e044d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e2044d8, SUC_lexicon_tags_68 },
    { 0xff0044d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x350044db, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfc0044dd, SUC_lexicon_tags_27 },
    { 0x84a044dd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be044e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55e044e4, SUC_lexicon_tags_60 },
    { 0x280044e5, SUC_lexicon_tags_14 },
    { 0x6f6044e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x158044e8, SUC_lexicon_tags_65 },
    { 0xf58044e9, SUC_lexicon_tags_46 },
    { 0x106044ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3a044ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe38044ee, SUC_lexicon_tags_7 },
    { 0xb16044ef, SUC_lexicon_tags_37 },
    { 0xa86044ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x638044f2, SUC_lexicon_tags_12 },
    { 0x440044f2, SUC_lexicon_tags_46 },
    { 0x2b4044f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e044f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384044fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x398044ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0204501, SUC_lexicon_tags_37 },
    { 0x94004502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89604507, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x43604509, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba0450c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c604517, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb920451b, SUC_lexicon_tags_11 },
    { 0x5c80451b, SUC_lexicon_tags_10 },
    { 0x6a40451d, SUC_lexicon_tags_68 },
    { 0x3d20451e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0c04520, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4204522, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92604524, SUC_lexicon_tags_6 },
    { 0x68e04524, SUC_lexicon_tags_0 },
    { 0xd9c04526, SUC_lexicon_tags_11 },
    { 0xd4c04527, SUC_lexicon_tags_0 },
    { 0x1004527, SUC_lexicon_tags_56 },
    { 0x85004527, SUC_lexicon_tags_47 },
    { 0x9ae04528, SUC_lexicon_tags_4 },
    { 0x30e0452b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1204532, SUC_lexicon_tags_27 },
    { 0x70204532, SUC_lexicon_tags_46 },
    { 0xff204534, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff804537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5a04539, SUC_lexicon_tags_69 },
    { 0x5580453a, SUC_lexicon_tags_1 },
    { 0x67c0453b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae0453e, SUC_lexicon_tags_17 },
    { 0xf1c0453f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3404541, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb5204543, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe80454b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf420454d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xfc20454f, SUC_lexicon_tags_17 },
    { 0xed204550, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21a04552, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f004555, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c0455b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c04563, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x57a04565, SUC_lexicon_tags_45 },
    { 0xb404566, SUC_lexicon_tags_26 },
    { 0xe6004566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa0456d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2e0456f, SUC_lexicon_tags_16 },
    { 0x5e004570, SUC_lexicon_tags_56 },
    { 0xa320456f, SUC_lexicon_tags_1 },
    { 0x38004571, SUC_lexicon_tags_21 },
    { 0x13204573, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe0804575, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e04578, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac60457a, SUC_lexicon_tags_12 },
    { 0x20457a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e404580, SUC_lexicon_tags_1 },
    { 0x46004580, SUC_lexicon_tags_7 },
    { 0xa8204581, SUC_lexicon_tags_10 },
    { 0xd7004583, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc04586, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f604588, SUC_lexicon_tags_37 },
    { 0xd3c04589, SUC_lexicon_tags_39 },
    { 0xa680458a, SUC_lexicon_tags_43 },
    { 0x8580458a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xec0458d, SUC_lexicon_tags_17 },
    { 0xce0458e, SUC_lexicon_tags_2 },
    { 0x8500458d, SUC_lexicon_tags_41 },
    { 0x9a04590, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa04593, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83204598, SUC_lexicon_tags_0 },
    { 0x7b804598, SUC_lexicon_tags_15 },
    { 0x9a00459a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e0459f, SUC_lexicon_tags_11 },
    { 0xa84045a0, SUC_lexicon_tags_27 },
    { 0xbae045a1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68045a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78045ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26045b5, SUC_lexicon_tags_25 },
    { 0xb6045b6, SUC_lexicon_tags_47 },
    { 0xcec045b7, SUC_lexicon_tags_11 },
    { 0x6fe045b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76c045ba, SUC_lexicon_tags_25 },
    { 0xc16045ba, SUC_lexicon_tags_37 },
    { 0x6b4045bc, SUC_lexicon_tags_48 },
    { 0x8ba045bd, SUC_lexicon_tags_37 },
    { 0x278045be, SUC_lexicon_tags_62 },
    { 0xee6045bf, SUC_lexicon_tags_31 },
    { 0x1fc045c0, SUC_lexicon_tags_1 },
    { 0x586045c1, SUC_lexicon_tags_15 },
    { 0x916045c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a045c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x322045c7, SUC_lexicon_tags_25 },
    { 0x8b8045c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x934045ca, SUC_lexicon_tags_1 },
    { 0xa1e045ca, SUC_lexicon_tags_15 },
    { 0x95e045cc, SUC_lexicon_tags_25 },
    { 0x2c045cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e045d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2045d5, SUC_lexicon_tags_37 },
    { 0x5be045d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1e045d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0045dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec045e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8045e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8045e7, SUC_lexicon_tags_17 },
    { 0x7a8045e8, SUC_lexicon_tags_17 },
    { 0x348045e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf6045eb, SUC_lexicon_tags_104 },
    { 0xc7c045ec, SUC_lexicon_tags_23 },
    { 0xd80045eb, SUC_lexicon_tags_1 },
    { 0x8ac045ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654045f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f8045f4, SUC_lexicon_tags_29 },
    { 0x44a045f5, SUC_lexicon_tags_25 },
    { 0x32045f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58045fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d604604, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1804609, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda80460e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3404612, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfee04614, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfc04616, SUC_lexicon_tags_23 },
    { 0x66404617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fc04619, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3480461c, SUC_lexicon_tags_2 },
    { 0xa000461d, SUC_lexicon_tags_45 },
    { 0x8ea0461d, SUC_lexicon_tags_30 },
    { 0xcea0461f, SUC_lexicon_tags_14 },
    { 0xbc20461f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec04624, SUC_lexicon_tags_37 },
    { 0xf7804625, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10004627, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a0462b, SUC_lexicon_tags_15 },
    { 0xbf00462b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4560462e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f804630, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x67604633, SUC_lexicon_tags_28 },
    { 0xbe604634, SUC_lexicon_tags_7 },
    { 0x45204635, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2fe04637, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2320463c, SUC_lexicon_tags_38 },
    { 0x1de0463d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14604641, SUC_lexicon_tags_6 },
    { 0xc3804641, SUC_lexicon_tags_27 },
    { 0x9604643, SUC_lexicon_tags_12 },
    { 0x9a804641, SUC_lexicon_tags_12 },
    { 0x94204645, SUC_lexicon_tags_55 },
    { 0xfb604645, SUC_lexicon_tags_7 },
    { 0xa0404644, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe1604649, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6720464e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x76004650, SUC_lexicon_tags_7 },
    { 0xa0004651, SUC_lexicon_tags_27 },
    { 0x84a04652, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf0204654, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fa04656, SUC_lexicon_tags_17 },
    { 0x1604656, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x12804659, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee0465c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8404662, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f204665, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40404667, SUC_lexicon_tags_2 },
    { 0xa2004667, SUC_lexicon_tags_77 },
    { 0xc6204669, SUC_lexicon_tags_31 },
    { 0x5f004669, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6580466d, SUC_lexicon_tags_63 },
    { 0x22e0466e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4004673, SUC_lexicon_tags_49 },
    { 0x50604673, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea04679, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3940467b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a04681, SUC_lexicon_tags_17 },
    { 0x40204682, SUC_lexicon_tags_45 },
    { 0xaa604681, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x45204686, SUC_lexicon_tags_7 },
    { 0x58804687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96404689, SUC_lexicon_tags_26 },
    { 0xa4a04689, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33604694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea04698, SUC_lexicon_tags_17 },
    { 0xa0c04699, SUC_lexicon_tags_11 },
    { 0x220469a, SUC_lexicon_tags_44 },
    { 0x5e60469b, SUC_lexicon_tags_19 },
    { 0x2e0469c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9200469e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00046a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844046a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86046ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x754046ae, SUC_lexicon_tags_18 },
    { 0x4a8046af, SUC_lexicon_tags_13 },
    { 0xc96046b0, SUC_lexicon_tags_5 },
    { 0xd06046b1, SUC_lexicon_tags_27 },
    { 0x154046af, SUC_lexicon_tags_13 },
    { 0xd24046af, SUC_lexicon_tags_37 },
    { 0xc0e046b0, SUC_lexicon_tags_34 },
    { 0x40046b2, SUC_lexicon_tags_2 },
    { 0xfa8046b6, SUC_lexicon_tags_235 },
    { 0x17e046b7, SUC_lexicon_tags_29 },
    { 0x68046b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68046ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2046bd, SUC_lexicon_tags_17 },
    { 0xa84046be, SUC_lexicon_tags_12 },
    { 0x2a4046bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x42046c2, SUC_lexicon_tags_45 },
    { 0xc82046c3, SUC_lexicon_tags_14 },
    { 0x7c046c3, SUC_lexicon_tags_11 },
    { 0xf60046c5, SUC_lexicon_tags_11 },
    { 0xe5a046c6, SUC_lexicon_tags_34 },
    { 0x488046c6, SUC_lexicon_tags_1 },
    { 0x3b2046c8, SUC_lexicon_tags_25 },
    { 0xdc6046c9, SUC_lexicon_tags_17 },
    { 0x9a6046c8, SUC_lexicon_tags_15 },
    { 0xd94046c2, SUC_lexicon_tags_0 },
    { 0xc06046cc, SUC_lexicon_tags_12 },
    { 0x76c046cd, SUC_lexicon_tags_16 },
    { 0x846046cd, SUC_lexicon_tags_1044 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba046d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc62046d3, SUC_lexicon_tags_10 },
    { 0x5e4046d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbee046d6, SUC_lexicon_tags_37 },
    { 0x9ce046d7, SUC_lexicon_tags_1 },
    { 0x336046d8, SUC_lexicon_tags_15 },
    { 0xd24046d9, SUC_lexicon_tags_37 },
    { 0x83a046da, SUC_lexicon_tags_77 },
    { 0x5d2046da, SUC_lexicon_tags_15 },
    { 0xf64046d8, SUC_lexicon_tags_1 },
    { 0x286046da, SUC_lexicon_tags_7 },
    { 0xa12046de, SUC_lexicon_tags_15 },
    { 0x9e0046db, SUC_lexicon_tags_32 },
    { 0xb4a046d6, SUC_lexicon_tags_11 },
    { 0x462046e1, SUC_lexicon_tags_10 },
    { 0xd1e046df, SUC_lexicon_tags_26 },
    { 0x7b2046e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ee046e5, SUC_lexicon_tags_136 },
    { 0xa98046e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4046e9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa046ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe0a046ef, SUC_lexicon_tags_25 },
    { 0xc5a046f0, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x4e6046f2, SUC_lexicon_tags_23 },
    { 0xc72046f3, SUC_lexicon_tags_4 },
    { 0x740046f3, SUC_lexicon_tags_0 },
    { 0xa8a046f4, SUC_lexicon_tags_68 },
    { 0xe94046f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e046f9, SUC_lexicon_tags_66 },
    { 0x748046f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa24046fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55004702, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a04706, SUC_lexicon_tags_38 },
    { 0x37204707, SUC_lexicon_tags_7 },
    { 0x19404708, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3260470e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ce04710, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a04713, SUC_lexicon_tags_2 },
    { 0xc3204714, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd604719, SUC_lexicon_tags_39 },
    { 0x2204719, SUC_lexicon_tags_79 },
    { 0x4c80471a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7800471d, SUC_lexicon_tags_15 },
    { 0x5de0471e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8804723, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c04728, SUC_lexicon_tags_12 },
    { 0x7e404728, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa04733, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6804736, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae0473b, SUC_lexicon_tags_23 },
    { 0xa40473b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae0473f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78804744, SUC_lexicon_tags_37 },
    { 0xec404745, SUC_lexicon_tags_68 },
    { 0xe7c04745, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27404748, SUC_lexicon_tags_14 },
    { 0xec204748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c0474f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x7c204751, SUC_lexicon_tags_40 },
    { 0x1ee04752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8a04754, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5804756, SUC_lexicon_tags_46 },
    { 0x3e204757, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x90c04759, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa4c0475b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3620475d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xff20475f, SUC_lexicon_tags_11 },
    { 0x44e0475f, SUC_lexicon_tags_17 },
    { 0x48404761, SUC_lexicon_tags_27 },
    { 0x73c04761, SUC_lexicon_tags_1 },
    { 0x46c04763, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40004768, SUC_lexicon_tags_106 },
    { 0xfbe04768, SUC_lexicon_tags_7 },
    { 0x2680476a, SUC_lexicon_tags_37 },
    { 0x7500476a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac20476f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e804772, SUC_lexicon_tags_60 },
    { 0xfbe04772, SUC_lexicon_tags_7 },
    { 0xc6404774, SUC_lexicon_tags_43 },
    { 0x3e204775, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60477b, SUC_lexicon_tags_43 },
    { 0xc860477c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3480477e, SUC_lexicon_tags_87 },
    { 0x10e0477f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29804784, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c04789, SUC_lexicon_tags_11 },
    { 0x5f80478a, SUC_lexicon_tags_32 },
    { 0x4ec0478b, SUC_lexicon_tags_38 },
    { 0x2980478c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e404795, SUC_lexicon_tags_7 },
    { 0xff604795, SUC_lexicon_tags_25 },
    { 0xa7c04796, SUC_lexicon_tags_21 },
    { 0xa7604798, SUC_lexicon_tags_10 },
    { 0xc9604798, SUC_lexicon_tags_66 },
    { 0x4be04799, SUC_lexicon_tags_63 },
    { 0xdac0479b, SUC_lexicon_tags_51 },
    { 0x93c0479c, SUC_lexicon_tags_12 },
    { 0x29a0479d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1100479f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a047a3, SUC_lexicon_tags_951 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa047a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a047ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x284047af, SUC_lexicon_tags_6 },
    { 0xd88047af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x372047b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc047b9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc047be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0047c3, SUC_lexicon_tags_21 },
    { 0x1f0047c4, SUC_lexicon_tags_23 },
    { 0x454047c5, SUC_lexicon_tags_68 },
    { 0x69c047c6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404047cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c047d4, SUC_lexicon_tags_30 },
    { 0x738047d4, SUC_lexicon_tags_23 },
    { 0x2fc047d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa047d9, SUC_lexicon_tags_2 },
    { 0x6b0047da, SUC_lexicon_tags_1 },
    { 0x178047db, SUC_lexicon_tags_15 },
    { 0x59c047dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c047e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2047e8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc047ed, SUC_lexicon_tags_1 },
    { 0x264047ee, SUC_lexicon_tags_11 },
    { 0xafe047ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x106047f1, SUC_lexicon_tags_31 },
    { 0x2ac047f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c4047f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02047f8, SUC_lexicon_tags_12 },
    { 0x66c047f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62047fd, SUC_lexicon_tags_1 },
    { 0x42047fd, SUC_lexicon_tags_17 },
    { 0x802047ff, SUC_lexicon_tags_12 },
    { 0xe2047fe, SUC_lexicon_tags_17 },
    { 0xfda04801, SUC_lexicon_tags_29 },
    { 0xbe804802, SUC_lexicon_tags_1 },
    { 0x402047ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6c04805, SUC_lexicon_tags_29 },
    { 0x3e04806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc0480a, SUC_lexicon_tags_36 },
    { 0xdbc0480b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29a0480d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d40480f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5804814, SUC_lexicon_tags_60 },
    { 0x76e04815, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a804818, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc1c0481a, SUC_lexicon_tags_4 },
    { 0xcd80481b, SUC_lexicon_tags_33 },
    { 0xb360481c, SUC_lexicon_tags_37 },
    { 0xdea0481c, SUC_lexicon_tags_17 },
    { 0xd160481e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x88604821, SUC_lexicon_tags_2 },
    { 0x24204822, SUC_lexicon_tags_11 },
    { 0x1604821, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940482b, SUC_lexicon_tags_23 },
    { 0x2d60482c, SUC_lexicon_tags_12 },
    { 0x3840482d, SUC_lexicon_tags_27 },
    { 0xbb00482c, SUC_lexicon_tags_25 },
    { 0xf9e0482e, SUC_lexicon_tags_25 },
    { 0x99804830, SUC_lexicon_tags_34 },
    { 0x2f80482f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xac004834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec804836, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a0483a, SUC_lexicon_tags_17 },
    { 0x1280483b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc0483f, SUC_lexicon_tags_2 },
    { 0xb8e04840, SUC_lexicon_tags_1 },
    { 0x97c0483f, SUC_lexicon_tags_2 },
    { 0xb9c04842, SUC_lexicon_tags_23 },
    { 0x36404841, SUC_lexicon_tags_7 },
    { 0xfe804842, SUC_lexicon_tags_28 },
    { 0x1be04844, SUC_lexicon_tags_7 },
    { 0x43a04846, SUC_lexicon_tags_12 },
    { 0x66a04846, SUC_lexicon_tags_61 },
    { 0x1a604848, SUC_lexicon_tags_15 },
    { 0x12a04849, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae04850, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4204853, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c404855, SUC_lexicon_tags_79 },
    { 0x4de04856, SUC_lexicon_tags_57 },
    { 0xb3004857, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3940485a, SUC_lexicon_tags_15 },
    { 0x30c0485a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5700485d, SUC_lexicon_tags_9 },
    { 0xe340485d, SUC_lexicon_tags_63 },
    { 0x1140485f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a04865, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x20c04867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18404870, SUC_lexicon_tags_27 },
    { 0x3e604871, SUC_lexicon_tags_15 },
    { 0x3ea04872, SUC_lexicon_tags_47 },
    { 0x7a804873, SUC_lexicon_tags_14 },
    { 0xbac04872, SUC_lexicon_tags_1 },
    { 0x1ba04874, SUC_lexicon_tags_12 },
    { 0xf9204871, SUC_lexicon_tags_17 },
    { 0xf4e04877, SUC_lexicon_tags_25 },
    { 0x71204871, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a80487b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ac0487d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7b40487f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad804881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae04888, SUC_lexicon_tags_11 },
    { 0x64204888, SUC_lexicon_tags_87 },
    { 0x29e0488a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda00488c, SUC_lexicon_tags_33 },
    { 0xbf20488d, SUC_lexicon_tags_60 },
    { 0xe600488e, SUC_lexicon_tags_15 },
    { 0x6640488c, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x17e04891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4604893, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26404895, SUC_lexicon_tags_15 },
    { 0xd5204895, SUC_lexicon_tags_37 },
    { 0x91e04897, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16e04899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b00489f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee048a2, SUC_lexicon_tags_25 },
    { 0x4ec048a3, SUC_lexicon_tags_31 },
    { 0x13a048a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc048a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x792048aa, SUC_lexicon_tags_25 },
    { 0xd5c048ab, SUC_lexicon_tags_44 },
    { 0xefe048ac, SUC_lexicon_tags_25 },
    { 0x30048ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaac048af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4048b2, SUC_lexicon_tags_8 },
    { 0x2f4048b3, SUC_lexicon_tags_15 },
    { 0x5da048b4, SUC_lexicon_tags_23 },
    { 0x97e048b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2048b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbe048bb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e048be, SUC_lexicon_tags_55 },
    { 0xbdc048bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16048c4, SUC_lexicon_tags_33 },
    { 0x9b0048c5, SUC_lexicon_tags_15 },
    { 0xbd4048c6, SUC_lexicon_tags_60 },
    { 0x146048c6, SUC_lexicon_tags_37 },
    { 0x180048c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ca048ca, SUC_lexicon_tags_27 },
    { 0xeac048ca, SUC_lexicon_tags_27 },
    { 0xb6e048cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc048cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa048d3, SUC_lexicon_tags_7 },
    { 0xcf8048d3, SUC_lexicon_tags_25 },
    { 0xd7e048d3, SUC_lexicon_tags_11 },
    { 0x91c048d6, SUC_lexicon_tags_17 },
    { 0x5fc048d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe0c048d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb2a048db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420048e1, SUC_lexicon_tags_26 },
    { 0x1b4048e1, SUC_lexicon_tags_10 },
    { 0x39e048e3, SUC_lexicon_tags_46 },
    { 0x546048e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26e048e6, SUC_lexicon_tags_7 },
    { 0xd40048e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a048eb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8048ee, SUC_lexicon_tags_2 },
    { 0xe18048ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb2048f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c048f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc048f7, SUC_lexicon_tags_26 },
    { 0xd6a048f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x946048fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x226048fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e04900, SUC_lexicon_tags_1 },
    { 0xc404900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56604905, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x37204907, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7f404909, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf840490b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c0490e, SUC_lexicon_tags_1 },
    { 0xfd40490f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22004911, SUC_lexicon_tags_6 },
    { 0xf9204912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e0491d, SUC_lexicon_tags_77 },
    { 0x8b20491d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4da04920, SUC_lexicon_tags_17 },
    { 0xf7c04920, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbdc04923, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c04926, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2940492b, SUC_lexicon_tags_49 },
    { 0x9a80492c, SUC_lexicon_tags_2 },
    { 0x6640492d, SUC_lexicon_tags_23 },
    { 0xea20492e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e04931, SUC_lexicon_tags_1 },
    { 0x9e004932, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeec04934, SUC_lexicon_tags_63 },
    { 0x8f404935, SUC_lexicon_tags_11 },
    { 0x1a604936, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92404938, SUC_lexicon_tags_37 },
    { 0x65804939, SUC_lexicon_tags_37 },
    { 0x9740493a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82204941, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xda804943, SUC_lexicon_tags_27 },
    { 0x12604944, SUC_lexicon_tags_34 },
    { 0x8404944, SUC_lexicon_tags_12 },
    { 0xd2004946, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ba04948, SUC_lexicon_tags_31 },
    { 0xa2a04949, SUC_lexicon_tags_45 },
    { 0xd3c0494a, SUC_lexicon_tags_12 },
    { 0x3ea0494b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe04952, SUC_lexicon_tags_15 },
    { 0x55a04953, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f604957, SUC_lexicon_tags_25 },
    { 0x42204958, SUC_lexicon_tags_37 },
    { 0xd4004958, SUC_lexicon_tags_21 },
    { 0xe060495a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce60495c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac604961, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86e04963, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e04966, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa460496e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4004971, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e04974, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe5604976, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c04979, SUC_lexicon_tags_12 },
    { 0xe8e0497a, SUC_lexicon_tags_14 },
    { 0xc2c0497b, SUC_lexicon_tags_33 },
    { 0x94e0497b, SUC_lexicon_tags_11 },
    { 0xca00497c, SUC_lexicon_tags_0 },
    { 0xa260497e, SUC_lexicon_tags_60 },
    { 0xf580497f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a04983, SUC_lexicon_tags_16 },
    { 0xcce04984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c0498a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e0498e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a04991, SUC_lexicon_tags_25 },
    { 0xece04991, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc04996, SUC_lexicon_tags_9 },
    { 0x8a004997, SUC_lexicon_tags_82 },
    { 0x78a04996, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa800499f, SUC_lexicon_tags_2 },
    { 0x974049a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x25c049a2, SUC_lexicon_tags_60 },
    { 0xc7e049a3, SUC_lexicon_tags_15 },
    { 0xbb4049a4, SUC_lexicon_tags_37 },
    { 0x236049a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2049a9, SUC_lexicon_tags_25 },
    { 0x69c049aa, SUC_lexicon_tags_47 },
    { 0xd4c049a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0049b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2049b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0049bb, SUC_lexicon_tags_45 },
    { 0x8ea049bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4049bf, SUC_lexicon_tags_25 },
    { 0xd28049c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730049c8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae049cb, SUC_lexicon_tags_11 },
    { 0x4f2049cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea6049ce, SUC_lexicon_tags_28 },
    { 0x170049cf, SUC_lexicon_tags_79 },
    { 0x264049d0, SUC_lexicon_tags_11 },
    { 0x86e049d1, SUC_lexicon_tags_20 },
    { 0x7e8049d2, SUC_lexicon_tags_38 },
    { 0xeac049d3, SUC_lexicon_tags_0 },
    { 0xfa8049d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c049d7, SUC_lexicon_tags_7 },
    { 0xe66049d8, SUC_lexicon_tags_13 },
    { 0x21c049d8, SUC_lexicon_tags_17 },
    { 0x386049da, SUC_lexicon_tags_37 },
    { 0x1ee049db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c049e2, SUC_lexicon_tags_17 },
    { 0xc04049e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6049e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70a049ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48049ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956049f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac049fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4049ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2404a01, SUC_lexicon_tags_60 },
    { 0x5bc04a02, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c04a05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9804a08, SUC_lexicon_tags_20 },
    { 0xc0a04a08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda404a0c, SUC_lexicon_tags_19 },
    { 0x7a404a0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b804a13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd604a15, SUC_lexicon_tags_12 },
    { 0xcf804a15, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x77004a19, SUC_lexicon_tags_12 },
    { 0x47004a1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23004a28, SUC_lexicon_tags_37 },
    { 0xdf004a29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec04a2f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6804a31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8b404a33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeca04a35, SUC_lexicon_tags_11 },
    { 0x79804a36, SUC_lexicon_tags_10 },
    { 0x4aa04a37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c04a3c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e04a3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4804a41, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4c404a43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc04a47, SUC_lexicon_tags_8 },
    { 0x3f204a48, SUC_lexicon_tags_37 },
    { 0xdb004a47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66204a4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcc04a4f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x14404a52, SUC_lexicon_tags_11 },
    { 0x42e04a53, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x2ac04a55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7604a5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9604a60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba04a69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c204a6f, SUC_lexicon_tags_65 },
    { 0xf9404a6f, SUC_lexicon_tags_29 },
    { 0xe5204a71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f604a73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xef004a75, SUC_lexicon_tags_7 },
    { 0x56204a76, SUC_lexicon_tags_56 },
    { 0xd8804a76, SUC_lexicon_tags_15 },
    { 0x1804a78, SUC_lexicon_tags_37 },
    { 0x90604a79, SUC_lexicon_tags_17 },
    { 0xa5404a79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e04a80, SUC_lexicon_tags_342 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7204a83, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d804a88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6fc04a8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c04a8e, SUC_lexicon_tags_33 },
    { 0x72c04a8f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59004a93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc804a98, SUC_lexicon_tags_39 },
    { 0x3dc04a99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea04a9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a04aa2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3bc04aa4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x37c04aa6, SUC_lexicon_tags_17 },
    { 0x8c804aa6, SUC_lexicon_tags_4 },
    { 0xaac04aa7, SUC_lexicon_tags_7 },
    { 0xfa604aa9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19804ab1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ae04ab3, SUC_lexicon_tags_49 },
    { 0x87404ab4, SUC_lexicon_tags_31 },
    { 0xda04ab4, SUC_lexicon_tags_43 },
    { 0xd5804ab6, SUC_lexicon_tags_4 },
    { 0x4e804ab6, SUC_lexicon_tags_1 },
    { 0x40e04ab8, SUC_lexicon_tags_65 },
    { 0xd1604ab5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a404abd, SUC_lexicon_tags_16 },
    { 0xf1804abe, SUC_lexicon_tags_17 },
    { 0x5ba04abf, SUC_lexicon_tags_61 },
    { 0xa6004ac0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59804ac4, SUC_lexicon_tags_31 },
    { 0x8ba04ac5, SUC_lexicon_tags_9 },
    { 0xf6204ac6, SUC_lexicon_tags_7 },
    { 0x54404ac6, SUC_lexicon_tags_16 },
    { 0x39004ac8, SUC_lexicon_tags_15 },
    { 0x4c004ac9, SUC_lexicon_tags_15 },
    { 0xbca04ac6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe04acd, SUC_lexicon_tags_82 },
    { 0x9e404ace, SUC_lexicon_tags_17 },
    { 0x39604acf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb204ad3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c04ad6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbc04ad8, SUC_lexicon_tags_7 },
    { 0x77204ad9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a204ade, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc2c04ae0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa404ae4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a604ae6, SUC_lexicon_tags_33 },
    { 0xcf204ae6, SUC_lexicon_tags_31 },
    { 0xb2004ae6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18604aea, SUC_lexicon_tags_30 },
    { 0x6ec04aeb, SUC_lexicon_tags_29 },
    { 0x5ba04aeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3404aee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c04af3, SUC_lexicon_tags_65 },
    { 0x49204af3, SUC_lexicon_tags_37 },
    { 0x2ae04af3, SUC_lexicon_tags_6 },
    { 0x2a604af6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e804af8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee04b01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc204b07, SUC_lexicon_tags_25 },
    { 0xd1804b08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b604b0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd404b12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56604b19, SUC_lexicon_tags_0 },
    { 0x7f404b19, SUC_lexicon_tags_14 },
    { 0x6f604b1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a404b22, SUC_lexicon_tags_38 },
    { 0x99a04b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba04b28, SUC_lexicon_tags_1 },
    { 0xcba04b28, SUC_lexicon_tags_64 },
    { 0xbaa04b2a, SUC_lexicon_tags_37 },
    { 0x80604b28, SUC_lexicon_tags_37 },
    { 0x6d004b2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc04b31, SUC_lexicon_tags_12 },
    { 0x63404b32, SUC_lexicon_tags_25 },
    { 0x6a204b33, SUC_lexicon_tags_7 },
    { 0x14804b33, SUC_lexicon_tags_29 },
    { 0xe2404b31, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6b204b37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea404b3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85604b42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e04b47, SUC_lexicon_tags_7 },
    { 0x404b48, SUC_lexicon_tags_66 },
    { 0xb4004b49, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe5a04b4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90404b4d, SUC_lexicon_tags_23 },
    { 0x37a04b4e, SUC_lexicon_tags_33 },
    { 0xb7004b4f, SUC_lexicon_tags_12 },
    { 0x4404b50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7404b52, SUC_lexicon_tags_12 },
    { 0x52604b53, SUC_lexicon_tags_37 },
    { 0x32804b53, SUC_lexicon_tags_79 },
    { 0x5ac04b55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a04b5a, SUC_lexicon_tags_7 },
    { 0xdaa04b5a, SUC_lexicon_tags_15 },
    { 0x47804b5b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8b604b5e, SUC_lexicon_tags_11 },
    { 0xd7004b5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7c04b61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c04b64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61a04b66, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce04b69, SUC_lexicon_tags_158 },
    { 0x48604b6a, SUC_lexicon_tags_29 },
    { 0x76004b6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e404b6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64204b71, SUC_lexicon_tags_2 },
    { 0xc1004b72, SUC_lexicon_tags_1 },
    { 0xec404b73, SUC_lexicon_tags_7 },
    { 0xeee04b72, SUC_lexicon_tags_9 },
    { 0x55404b73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93804b7b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba04b82, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca204b90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24604b95, SUC_lexicon_tags_2 },
    { 0xa4804b95, SUC_lexicon_tags_2 },
    { 0xb4404b97, SUC_lexicon_tags_16 },
    { 0x93604b98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e604b9a, SUC_lexicon_tags_45 },
    { 0x4fc04b9a, SUC_lexicon_tags_30 },
    { 0x1e604b9c, SUC_lexicon_tags_17 },
    { 0x71c04b9d, SUC_lexicon_tags_190 },
    { 0x91c04b9c, SUC_lexicon_tags_1 },
    { 0xc7e04b9e, SUC_lexicon_tags_14 },
    { 0x15e04ba0, SUC_lexicon_tags_19 },
    { 0xbc04ba1, SUC_lexicon_tags_15 },
    { 0x59c04ba1, SUC_lexicon_tags_7 },
    { 0x3ee04ba3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18804ba5, SUC_lexicon_tags_48 },
    { 0x69e04ba5, SUC_lexicon_tags_60 },
    { 0x23004ba6, SUC_lexicon_tags_11 },
    { 0x31404ba8, SUC_lexicon_tags_22 },
    { 0x2b604ba9, SUC_lexicon_tags_63 },
    { 0xed404ba8, SUC_lexicon_tags_15 },
    { 0xc1004bab, SUC_lexicon_tags_14 },
    { 0x3aa04ba6, SUC_lexicon_tags_63 },
    { 0xd0004bad, SUC_lexicon_tags_7 },
    { 0xc2204ba7, SUC_lexicon_tags_11 },
    { 0x8e604ba7, SUC_lexicon_tags_1 },
    { 0xf9004ba8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac04bb5, SUC_lexicon_tags_136 },
    { 0x5804bb5, SUC_lexicon_tags_27 },
    { 0x2b204bb7, SUC_lexicon_tags_17 },
    { 0x51c04bb5, SUC_lexicon_tags_45 },
    { 0x3ee04bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a04bbe, SUC_lexicon_tags_28 },
    { 0x62204bbf, SUC_lexicon_tags_11 },
    { 0x28404bbf, SUC_lexicon_tags_37 },
    { 0x8fe04bbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2404bc5, SUC_lexicon_tags_8 },
    { 0xe3c04bc6, SUC_lexicon_tags_1 },
    { 0xed804bc7, SUC_lexicon_tags_17 },
    { 0x78a04bc8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9604bcc, SUC_lexicon_tags_4 },
    { 0x14804bcc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x12804bcf, SUC_lexicon_tags_19 },
    { 0xd2604bcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x80804bd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbec04bd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3604bd7, SUC_lexicon_tags_38 },
    { 0x86004bd8, SUC_lexicon_tags_2 },
    { 0x91c04bd8, SUC_lexicon_tags_30 },
    { 0x1da04bda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a04bde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9004be6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fe04be8, SUC_lexicon_tags_2 },
    { 0x38404be8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c204bee, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5004bfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c04c02, SUC_lexicon_tags_2 },
    { 0xd1804c03, SUC_lexicon_tags_27 },
    { 0x50e04c02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a804c09, SUC_lexicon_tags_62 },
    { 0xaf004c0a, SUC_lexicon_tags_27 },
    { 0x10c04c0b, SUC_lexicon_tags_45 },
    { 0x7a804c0b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x83e04c0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4c04c10, SUC_lexicon_tags_37 },
    { 0x84604c11, SUC_lexicon_tags_7 },
    { 0xc1c04c10, SUC_lexicon_tags_43 },
    { 0x3e804c12, SUC_lexicon_tags_63 },
    { 0x84804c12, SUC_lexicon_tags_17 },
    { 0xba04c12, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x2b604c17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba04c1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa04c24, SUC_lexicon_tags_15 },
    { 0x64404c24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc004c2d, SUC_lexicon_tags_15 },
    { 0x2fe04c2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a04c31, SUC_lexicon_tags_30 },
    { 0x67804c32, SUC_lexicon_tags_57 },
    { 0xad404c33, SUC_lexicon_tags_21 },
    { 0xc7404c34, SUC_lexicon_tags_6 },
    { 0x87c04c35, SUC_lexicon_tags_28 },
    { 0x4b404c33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b204c3d, SUC_lexicon_tags_2 },
    { 0x5c04c3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18a04c40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77404c42, SUC_lexicon_tags_5 },
    { 0x67a04c43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75404c47, SUC_lexicon_tags_12 },
    { 0x36004c48, SUC_lexicon_tags_15 },
    { 0x61004c47, SUC_lexicon_tags_17 },
    { 0xac204c4a, SUC_lexicon_tags_252 },
    { 0x5c204c4b, SUC_lexicon_tags_46 },
    { 0x61604c4c, SUC_lexicon_tags_63 },
    { 0xa5604c47, SUC_lexicon_tags_13 },
    { 0xeba04c48, SUC_lexicon_tags_25 },
    { 0xd8204c4f, SUC_lexicon_tags_46 },
    { 0xf6004c48, SUC_lexicon_tags_37 },
    { 0x45804c51, SUC_lexicon_tags_45 },
    { 0x8ea04c47, SUC_lexicon_tags_35 },
    { 0x41404c48, SUC_lexicon_tags_10 },
    { 0xb4e04c54, SUC_lexicon_tags_27 },
    { 0x8ae04c4c, SUC_lexicon_tags_28 },
    { 0x82604c52, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66804c5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea404c65, SUC_lexicon_tags_7 },
    { 0x3f204c66, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2404c69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15604c72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d604c75, SUC_lexicon_tags_60 },
    { 0x5a604c75, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a404c7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45804c81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b204c87, SUC_lexicon_tags_27 },
    { 0x6f004c87, SUC_lexicon_tags_31 },
    { 0x7be04c89, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c04c8d, SUC_lexicon_tags_7 },
    { 0xbe404c8d, SUC_lexicon_tags_14 },
    { 0x6a804c8f, SUC_lexicon_tags_61 },
    { 0x5fe04c90, SUC_lexicon_tags_37 },
    { 0xf2404c8e, SUC_lexicon_tags_11 },
    { 0x4a004c92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33604c96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef004c99, SUC_lexicon_tags_55 },
    { 0xf3604c99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e04c9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x91204ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e04ca6, SUC_lexicon_tags_25 },
    { 0x70c04ca6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1204cb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba04cba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e04cbd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e604cc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c04cc7, SUC_lexicon_tags_25 },
    { 0xce604cc8, SUC_lexicon_tags_11 },
    { 0xfd004cc9, SUC_lexicon_tags_23 },
    { 0x94204cca, SUC_lexicon_tags_25 },
    { 0x19c04cc7, SUC_lexicon_tags_41 },
    { 0x71e04ccc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81e04cce, SUC_lexicon_tags_25 },
    { 0xf0604cce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44404cd2, SUC_lexicon_tags_27 },
    { 0x86804cd3, SUC_lexicon_tags_37 },
    { 0xb0a04cd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b204cd6, SUC_lexicon_tags_469 },
    { 0xf1c04cd7, SUC_lexicon_tags_254 },
    { 0xcd204cd8, SUC_lexicon_tags_10 },
    { 0x78004cd9, SUC_lexicon_tags_37 },
    { 0xd2404cda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a04cde, SUC_lexicon_tags_25 },
    { 0x35e04cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81404ce1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd7a04ce3, SUC_lexicon_tags_26 },
    { 0x40a04ce4, SUC_lexicon_tags_2 },
    { 0x56204ce5, SUC_lexicon_tags_23 },
    { 0x2e204ce5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd3604ce8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5d404cea, SUC_lexicon_tags_14 },
    { 0xeda04ceb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea04cf7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x35804cf9, SUC_lexicon_tags_46 },
    { 0xcfe04cf9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0204cfe, SUC_lexicon_tags_45 },
    { 0x9fa04cff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99804d01, SUC_lexicon_tags_29 },
    { 0xec004d01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64c04d04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b804d14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x99404d17, SUC_lexicon_tags_27 },
    { 0x5ae04d17, SUC_lexicon_tags_43 },
    { 0x13c04d17, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a04d20, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8604d24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a04d30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16404d3a, SUC_lexicon_tags_48 },
    { 0xf5604d3b, SUC_lexicon_tags_29 },
    { 0x9f804d3b, SUC_lexicon_tags_17 },
    { 0x75e04d3d, SUC_lexicon_tags_8 },
    { 0xca204d3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe04d44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3e04d46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x27404d48, SUC_lexicon_tags_15 },
    { 0xa1c04d49, SUC_lexicon_tags_23 },
    { 0x4ec04d49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc04d4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x15e04d50, SUC_lexicon_tags_62 },
    { 0xb1a04d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66804d58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe204d5a, SUC_lexicon_tags_11 },
    { 0xfbe04d5b, SUC_lexicon_tags_14 },
    { 0x3b004d5c, SUC_lexicon_tags_20 },
    { 0xef804d5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedc04d5f, SUC_lexicon_tags_11 },
    { 0x9ea04d60, SUC_lexicon_tags_11 },
    { 0x3dc04d61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc04d69, SUC_lexicon_tags_14 },
    { 0x14a04d6a, SUC_lexicon_tags_17 },
    { 0x5604d6a, SUC_lexicon_tags_46 },
    { 0x4b204d6a, SUC_lexicon_tags_12 },
    { 0x76204d6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c804d71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b004d75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97804d79, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96204d83, SUC_lexicon_tags_213 },
    { 0xafc04d84, SUC_lexicon_tags_33 },
    { 0xd9a04d83, SUC_lexicon_tags_37 },
    { 0xd9804d86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0604d8c, SUC_lexicon_tags_1 },
    { 0xf8204d8c, SUC_lexicon_tags_16 },
    { 0x3b404d8e, SUC_lexicon_tags_7 },
    { 0xcb804d8e, SUC_lexicon_tags_33 },
    { 0xb1804d8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x92604d92, SUC_lexicon_tags_17 },
    { 0xbe04d93, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f604d9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b804da4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c04da8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65004dae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb004db0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x19404db3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88604db7, SUC_lexicon_tags_25 },
    { 0x4bc04db8, SUC_lexicon_tags_2 },
    { 0x71804db7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf804dbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90004dc2, SUC_lexicon_tags_26 },
    { 0x8ca04dc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4804dc5, SUC_lexicon_tags_7 },
    { 0xffe04dc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a04dca, SUC_lexicon_tags_15 },
    { 0xa7204dcb, SUC_lexicon_tags_22 },
    { 0x5c804dca, SUC_lexicon_tags_25 },
    { 0xb0604dcd, SUC_lexicon_tags_26 },
    { 0x5c04dcd, SUC_lexicon_tags_19 },
    { 0x17604dca, SUC_lexicon_tags_16 },
    { 0xbf404dd0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b004dd3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c04dd9, SUC_lexicon_tags_7 },
    { 0x91804dd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23204ddd, SUC_lexicon_tags_11 },
    { 0x29e04dde, SUC_lexicon_tags_33 },
    { 0xaa604ddf, SUC_lexicon_tags_25 },
    { 0x4d004de0, SUC_lexicon_tags_7 },
    { 0xb6804ddd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6a004de3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e04de5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c604dea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3004df1, SUC_lexicon_tags_60 },
    { 0x71804df2, SUC_lexicon_tags_37 },
    { 0x78e04df1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c04df8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a04dfc, SUC_lexicon_tags_15 },
    { 0xe2e04dfd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xdc604dff, SUC_lexicon_tags_25 },
    { 0x14e04e00, SUC_lexicon_tags_87 },
    { 0xd3404e00, SUC_lexicon_tags_31 },
    { 0x4f804e02, SUC_lexicon_tags_39 },
    { 0x41204e03, SUC_lexicon_tags_47 },
    { 0xf7c04e03, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3004e0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe6604e0c, SUC_lexicon_tags_69 },
    { 0x11c04e0c, SUC_lexicon_tags_8 },
    { 0xe6404e0e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c04e11, SUC_lexicon_tags_45 },
    { 0xc2004e12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a04e16, SUC_lexicon_tags_34 },
    { 0xd9604e16, SUC_lexicon_tags_7 },
    { 0x1a404e17, SUC_lexicon_tags_11 },
    { 0xa2804e18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f604e1d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdaa04e1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a04e23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d204e25, SUC_lexicon_tags_13 },
    { 0xa1e04e26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa04e29, SUC_lexicon_tags_12 },
    { 0x29a04e2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a04e2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed204e31, SUC_lexicon_tags_79 },
    { 0x88c04e31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10204e34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb604e37, SUC_lexicon_tags_1 },
    { 0xa6404e38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e604e3b, SUC_lexicon_tags_11 },
    { 0xf5404e3c, SUC_lexicon_tags_15 },
    { 0xaaa04e3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa04e41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45804e45, SUC_lexicon_tags_37 },
    { 0xd3e04e46, SUC_lexicon_tags_37 },
    { 0xae604e47, SUC_lexicon_tags_32 },
    { 0x1404e47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10e04e4a, SUC_lexicon_tags_26 },
    { 0x75804e4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5404e4f, SUC_lexicon_tags_27 },
    { 0x1a004e50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63804e55, SUC_lexicon_tags_47 },
    { 0x81e04e56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41004e5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c04e60, SUC_lexicon_tags_14 },
    { 0x9f404e61, SUC_lexicon_tags_17 },
    { 0x29804e62, SUC_lexicon_tags_17 },
    { 0xcfc04e60, SUC_lexicon_tags_15 },
    { 0x25804e61, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19404e68, SUC_lexicon_tags_26 },
    { 0xf8e04e69, SUC_lexicon_tags_15 },
    { 0x4bc04e69, SUC_lexicon_tags_25 },
    { 0xe7204e6b, SUC_lexicon_tags_16 },
    { 0xe4404e6c, SUC_lexicon_tags_16 },
    { 0x91004e6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c04e71, SUC_lexicon_tags_23 },
    { 0xd5004e72, SUC_lexicon_tags_1 },
    { 0x8ae04e72, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4004e77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa004e79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e204e7d, SUC_lexicon_tags_25 },
    { 0xd8004e7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82a04e80, SUC_lexicon_tags_8 },
    { 0xe8604e80, SUC_lexicon_tags_235 },
    { 0xde04e80, SUC_lexicon_tags_11 },
    { 0x1be04e83, SUC_lexicon_tags_9 },
    { 0x65404e84, SUC_lexicon_tags_15 },
    { 0x4204e83, SUC_lexicon_tags_2 },
    { 0xec804e86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdb804e88, SUC_lexicon_tags_79 },
    { 0x30804e89, SUC_lexicon_tags_15 },
    { 0xa04e88, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x96a04e8c, SUC_lexicon_tags_27 },
    { 0xeb004e8c, SUC_lexicon_tags_45 },
    { 0xb4c04e8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc204e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76804e96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26004ea0, SUC_lexicon_tags_7 },
    { 0x72004ea1, SUC_lexicon_tags_6 },
    { 0x4e804ea2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e04ea5, SUC_lexicon_tags_30 },
    { 0x5de04ea5, SUC_lexicon_tags_12 },
    { 0xdf404ea7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e04eaa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea04eaf, SUC_lexicon_tags_33 },
    { 0x6ae04eaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57404eb2, SUC_lexicon_tags_32 },
    { 0xbb804eb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e04eb6, SUC_lexicon_tags_14 },
    { 0x78e04eb6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x3d404eb9, SUC_lexicon_tags_13 },
    { 0x36404eba, SUC_lexicon_tags_26 },
    { 0x57004eba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3404ebe, SUC_lexicon_tags_15 },
    { 0x30204ebf, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec04ec5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc04ec9, SUC_lexicon_tags_17 },
    { 0x6e04eca, SUC_lexicon_tags_26 },
    { 0x1ea04ecb, SUC_lexicon_tags_12 },
    { 0x4de04ecc, SUC_lexicon_tags_2 },
    { 0x4fc04ecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec04ed5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37804ed9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10004edb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5204edd, SUC_lexicon_tags_39 },
    { 0x3604edd, SUC_lexicon_tags_33 },
    { 0x94204edf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x33004ee1, SUC_lexicon_tags_7 },
    { 0xaca04ee1, SUC_lexicon_tags_8 },
    { 0x7604ee3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11204eea, SUC_lexicon_tags_39 },
    { 0x1b604eeb, SUC_lexicon_tags_11 },
    { 0xa2e04eec, SUC_lexicon_tags_1 },
    { 0xa6c04eeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c04ef4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90e04ef6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e04f00, SUC_lexicon_tags_45 },
    { 0xfee04f01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0004f08, SUC_lexicon_tags_88 },
    { 0x6b804f09, SUC_lexicon_tags_8 },
    { 0xef404f0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81e04f0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d404f12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac04f16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a04f19, SUC_lexicon_tags_2 },
    { 0xba04f19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe404f1d, SUC_lexicon_tags_37 },
    { 0x9004f1d, SUC_lexicon_tags_23 },
    { 0xc1804f1d, SUC_lexicon_tags_7 },
    { 0xe9604f20, SUC_lexicon_tags_31 },
    { 0xcf204f21, SUC_lexicon_tags_30 },
    { 0xb3e04f22, SUC_lexicon_tags_17 },
    { 0xe2a04f1d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae04f2c, SUC_lexicon_tags_14 },
    { 0xf7204f2d, SUC_lexicon_tags_37 },
    { 0x18204f2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c204f37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c04f3e, SUC_lexicon_tags_12 },
    { 0xd0c04f3f, SUC_lexicon_tags_47 },
    { 0x2ca04f40, SUC_lexicon_tags_2 },
    { 0x7804f41, SUC_lexicon_tags_12 },
    { 0xd4804f42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce204f47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa804f49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da04f51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43404f56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23604f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4604f5e, SUC_lexicon_tags_14 },
    { 0x38a04f5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c04f65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b404f68, SUC_lexicon_tags_37 },
    { 0x2b404f68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5804f6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19004f71, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3604f76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4604f7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7a04f7d, SUC_lexicon_tags_17 },
    { 0xd5204f7e, SUC_lexicon_tags_9 },
    { 0x99204f7d, SUC_lexicon_tags_11 },
    { 0xfd804f7d, SUC_lexicon_tags_84 },
    { 0x55204f80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf804f84, SUC_lexicon_tags_15 },
    { 0xa3e04f84, SUC_lexicon_tags_47 },
    { 0xc8604f85, SUC_lexicon_tags_39 },
    { 0x8c604f84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc604f8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c04f8d, SUC_lexicon_tags_47 },
    { 0xece04f8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70804f90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f404f96, SUC_lexicon_tags_23 },
    { 0xa8c04f96, SUC_lexicon_tags_0 },
    { 0x54404f98, SUC_lexicon_tags_15 },
    { 0x2d804f98, SUC_lexicon_tags_11 },
    { 0x32e04f9a, SUC_lexicon_tags_9 },
    { 0x71004f9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e804f9f, SUC_lexicon_tags_11 },
    { 0xd4204fa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd804fa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb404fa6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3be04fa8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x22604fab, SUC_lexicon_tags_37 },
    { 0xaca04fab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x98004fae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c404fb5, SUC_lexicon_tags_26 },
    { 0x1bc04fb5, SUC_lexicon_tags_23 },
    { 0xa5204fb7, SUC_lexicon_tags_46 },
    { 0xee204fb5, SUC_lexicon_tags_37 },
    { 0x4f804fb5, SUC_lexicon_tags_37 },
    { 0xf0804fba, SUC_lexicon_tags_62 },
    { 0xaac04fbb, SUC_lexicon_tags_31 },
    { 0x32204fba, SUC_lexicon_tags_18 },
    { 0xca404fbc, SUC_lexicon_tags_46 },
    { 0x51c04fbe, SUC_lexicon_tags_11 },
    { 0xeac04fbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45e04fc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c04fc5, SUC_lexicon_tags_17 },
    { 0x5aa04fc6, SUC_lexicon_tags_63 },
    { 0x85204fc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2404fcb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x14204fcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a04fd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2004fd3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6b404fd5, SUC_lexicon_tags_49 },
    { 0x9ce04fd6, SUC_lexicon_tags_17 },
    { 0x4204fd7, SUC_lexicon_tags_66 },
    { 0x11004fd8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b004fdb, SUC_lexicon_tags_15 },
    { 0x9e804fdc, SUC_lexicon_tags_6 },
    { 0x74204fdc, SUC_lexicon_tags_47 },
    { 0x9f604fde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdaa04fe0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31804fe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c04fe6, SUC_lexicon_tags_10 },
    { 0x37804fe6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57804fed, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4604ff1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e04ff7, SUC_lexicon_tags_45 },
    { 0x97e04ff7, SUC_lexicon_tags_47 },
    { 0x89204ff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e004ffb, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70205003, SUC_lexicon_tags_47 },
    { 0x87805004, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf1005006, SUC_lexicon_tags_11 },
    { 0x12a05007, SUC_lexicon_tags_37 },
    { 0xa7a05008, SUC_lexicon_tags_44 },
    { 0xb4405009, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2140500d, SUC_lexicon_tags_21 },
    { 0x4fe0500e, SUC_lexicon_tags_1 },
    { 0xc900500f, SUC_lexicon_tags_21 },
    { 0xee60500d, SUC_lexicon_tags_98 },
    { 0xe4a05011, SUC_lexicon_tags_17 },
    { 0x6ae05012, SUC_lexicon_tags_66 },
    { 0x49a05013, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6405017, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a0501e, SUC_lexicon_tags_7 },
    { 0xbc0501f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e805022, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe6e05024, SUC_lexicon_tags_19 },
    { 0x55205025, SUC_lexicon_tags_23 },
    { 0xea605026, SUC_lexicon_tags_45 },
    { 0x34205027, SUC_lexicon_tags_6 },
    { 0xffc05025, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9820502c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a605032, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46e05034, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb3005036, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8ea05038, SUC_lexicon_tags_23 },
    { 0x62e05039, SUC_lexicon_tags_47 },
    { 0x5e005039, SUC_lexicon_tags_2 },
    { 0xba0503b, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0x4540503d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae05044, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e0504d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0005051, SUC_lexicon_tags_2 },
    { 0x5a405052, SUC_lexicon_tags_84 },
    { 0x90605053, SUC_lexicon_tags_6 },
    { 0xf4405051, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a0505a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb20505c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf840505e, SUC_lexicon_tags_6 },
    { 0xc400505f, SUC_lexicon_tags_37 },
    { 0xd9405060, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40506b, SUC_lexicon_tags_96 },
    { 0x8c80506c, SUC_lexicon_tags_15 },
    { 0x8520506d, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05070, SUC_lexicon_tags_18 },
    { 0x40205071, SUC_lexicon_tags_25 },
    { 0x83a05070, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc700507b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb60507e, SUC_lexicon_tags_33 },
    { 0xd960507f, SUC_lexicon_tags_15 },
    { 0x7fa05080, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6205084, SUC_lexicon_tags_19 },
    { 0xa5005085, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x71805087, SUC_lexicon_tags_33 },
    { 0xa4c05087, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad605090, SUC_lexicon_tags_28 },
    { 0xd5e05091, SUC_lexicon_tags_9 },
    { 0x9b605090, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0x22205094, SUC_lexicon_tags_69 },
    { 0xebe05095, SUC_lexicon_tags_26 },
    { 0x99805096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21405098, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8700509e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde050a1, SUC_lexicon_tags_27 },
    { 0xc86050a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46050a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c050ab, SUC_lexicon_tags_7 },
    { 0x7f6050ac, SUC_lexicon_tags_6 },
    { 0x30e050ad, SUC_lexicon_tags_25 },
    { 0x7e8050ac, SUC_lexicon_tags_33 },
    { 0x8cc050af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22c050b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e050b4, SUC_lexicon_tags_113 },
    { 0xd3e050b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59a050b7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae050bd, SUC_lexicon_tags_37 },
    { 0x8f8050be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea050c3, SUC_lexicon_tags_65 },
    { 0x4c6050c4, SUC_lexicon_tags_14 },
    { 0x58a050c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2050ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90050d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446050dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac050e3, SUC_lexicon_tags_1 },
    { 0xa1e050e4, SUC_lexicon_tags_7 },
    { 0x3d8050e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196050e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8050ec, SUC_lexicon_tags_11 },
    { 0x99c050ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3e050ef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa22050f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344050f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1a050f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd76050fb, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494050ff, SUC_lexicon_tags_11 },
    { 0x3e2050ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb405103, SUC_lexicon_tags_1 },
    { 0x18605104, SUC_lexicon_tags_19 },
    { 0x9d005104, SUC_lexicon_tags_25 },
    { 0xb4005106, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81205108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae05115, SUC_lexicon_tags_47 },
    { 0xb8205116, SUC_lexicon_tags_19 },
    { 0xf5405116, SUC_lexicon_tags_2 },
    { 0xc05118, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee40511f, SUC_lexicon_tags_2 },
    { 0xa980511f, SUC_lexicon_tags_37 },
    { 0xac805121, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e05125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde605127, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0a05129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c0512c, SUC_lexicon_tags_49 },
    { 0x25c0512d, SUC_lexicon_tags_37 },
    { 0x20c0512d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9f805130, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3605133, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc05137, SUC_lexicon_tags_1 },
    { 0x90a05137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e0513d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63805143, SUC_lexicon_tags_11 },
    { 0xb5405144, SUC_lexicon_tags_37 },
    { 0x9a205145, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18405147, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf920514c, SUC_lexicon_tags_79 },
    { 0x79c0514c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1405160, SUC_lexicon_tags_26 },
    { 0xab605161, SUC_lexicon_tags_37 },
    { 0xdb405162, SUC_lexicon_tags_15 },
    { 0xe3605163, SUC_lexicon_tags_31 },
    { 0x3b805164, SUC_lexicon_tags_12 },
    { 0x6fc05163, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95805168, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec0516d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc05175, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13405177, SUC_lexicon_tags_17 },
    { 0x6ee05177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c0517c, SUC_lexicon_tags_56 },
    { 0xcd00517d, SUC_lexicon_tags_26 },
    { 0xd4c0517e, SUC_lexicon_tags_15 },
    { 0x7700517f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1205182, SUC_lexicon_tags_2 },
    { 0x8a805183, SUC_lexicon_tags_16 },
    { 0xd9e05184, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2d005186, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c0518b, SUC_lexicon_tags_10 },
    { 0x6b80518b, SUC_lexicon_tags_25 },
    { 0x76a0518c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83405193, SUC_lexicon_tags_49 },
    { 0xa1005194, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf500519b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9200519f, SUC_lexicon_tags_87 },
    { 0x1220519f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da051a3, SUC_lexicon_tags_47 },
    { 0xdb0051a4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90051a9, SUC_lexicon_tags_15 },
    { 0xc5c051a9, SUC_lexicon_tags_43 },
    { 0x6bc051ab, SUC_lexicon_tags_43 },
    { 0xa10051ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20051af, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a051b7, SUC_lexicon_tags_15 },
    { 0x804051b8, SUC_lexicon_tags_21 },
    { 0x18c051b9, SUC_lexicon_tags_23 },
    { 0xbe0051ba, SUC_lexicon_tags_61 },
    { 0x972051bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704051bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6051c3, SUC_lexicon_tags_9 },
    { 0xb0c051c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcba051c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a4051c8, SUC_lexicon_tags_38 },
    { 0x314051c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150051cd, SUC_lexicon_tags_17 },
    { 0x28a051ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x82e051d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760051d4, SUC_lexicon_tags_12 },
    { 0xda051d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x432051d7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe1c051d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278051dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x720051df, SUC_lexicon_tags_37 },
    { 0xb3e051e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c051e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552051e7, SUC_lexicon_tags_37 },
    { 0x8ae051e7, SUC_lexicon_tags_31 },
    { 0x66051e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a051ec, SUC_lexicon_tags_37 },
    { 0x72c051ed, SUC_lexicon_tags_17 },
    { 0x530051ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x848051f0, SUC_lexicon_tags_11 },
    { 0xe1c051f0, SUC_lexicon_tags_25 },
    { 0x840051f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76051f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c051fa, SUC_lexicon_tags_29 },
    { 0x6fc051fa, SUC_lexicon_tags_11 },
    { 0xfd4051fc, SUC_lexicon_tags_19 },
    { 0x612051fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf805203, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea405206, SUC_lexicon_tags_47 },
    { 0xce805206, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c0520b, SUC_lexicon_tags_1 },
    { 0x14c0520b, SUC_lexicon_tags_29 },
    { 0x5ec0520c, SUC_lexicon_tags_15 },
    { 0xeba0520b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc1605210, SUC_lexicon_tags_17 },
    { 0xf5a05210, SUC_lexicon_tags_7 },
    { 0x7fe05212, SUC_lexicon_tags_38 },
    { 0x90205213, SUC_lexicon_tags_25 },
    { 0x68405214, SUC_lexicon_tags_11 },
    { 0xa005213, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce205219, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e40521d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39605228, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd420522b, SUC_lexicon_tags_60 },
    { 0xd580522b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa4c0522e, SUC_lexicon_tags_37 },
    { 0x8660522e, SUC_lexicon_tags_12 },
    { 0x7bc0522f, SUC_lexicon_tags_68 },
    { 0xde05231, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e05234, SUC_lexicon_tags_12 },
    { 0xf8605235, SUC_lexicon_tags_63 },
    { 0xb7205236, SUC_lexicon_tags_8 },
    { 0x7d805236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe0523e, SUC_lexicon_tags_25 },
    { 0x7240523e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55005243, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14005247, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe0524b, SUC_lexicon_tags_11 },
    { 0x4180524c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c05250, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f605253, SUC_lexicon_tags_25 },
    { 0x3f005253, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x8ee05256, SUC_lexicon_tags_16 },
    { 0x2805257, SUC_lexicon_tags_12 },
    { 0x16805258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba05261, SUC_lexicon_tags_10 },
    { 0x64e05262, SUC_lexicon_tags_2 },
    { 0x27a05262, SUC_lexicon_tags_63 },
    { 0x2d405264, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7b205266, SUC_lexicon_tags_29 },
    { 0x5b405267, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a0526a, SUC_lexicon_tags_43 },
    { 0x320526b, SUC_lexicon_tags_179 },
    { 0xb880526c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe20526e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb005270, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf2405272, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa05277, SUC_lexicon_tags_23 },
    { 0xdec05278, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f60527a, SUC_lexicon_tags_3 },
    { 0xf420527a, SUC_lexicon_tags_60 },
    { 0x7260527c, SUC_lexicon_tags_19 },
    { 0xe280527d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c80527f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc820528a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6620528c, SUC_lexicon_tags_19 },
    { 0x4540528d, SUC_lexicon_tags_15 },
    { 0x220528c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1be05290, SUC_lexicon_tags_7 },
    { 0x7fe05290, SUC_lexicon_tags_34 },
    { 0x1205292, SUC_lexicon_tags_11 },
    { 0xa9005292, SUC_lexicon_tags_3 },
    { 0x92605293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a05298, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x18e0529a, SUC_lexicon_tags_77 },
    { 0x89a0529b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba052a1, SUC_lexicon_tags_87 },
    { 0x5be052a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcdc052a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82a052a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34c052a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270052ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xff4052af, SUC_lexicon_tags_55 },
    { 0xff2052b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8052b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52be, SUC_lexicon_tags_39 },
    { 0xe7a052bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a052c2, SUC_lexicon_tags_25 },
    { 0xab8052c3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e052c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4052ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec052d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c052d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc1e052db, SUC_lexicon_tags_2 },
    { 0x13e052dc, SUC_lexicon_tags_37 },
    { 0x2c8052dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0052e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0052e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70052e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80052ea, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e052ed, SUC_lexicon_tags_4 },
    { 0x618052ee, SUC_lexicon_tags_27 },
    { 0x8f0052ed, SUC_lexicon_tags_179 },
    { 0xada052f0, SUC_lexicon_tags_37 },
    { 0xa62052f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84052f5, SUC_lexicon_tags_45 },
    { 0x700052f6, SUC_lexicon_tags_17 },
    { 0x2e2052f7, SUC_lexicon_tags_895 },
    { 0x676052f6, SUC_lexicon_tags_97 },
    { 0xba8052f9, SUC_lexicon_tags_56 },
    { 0xa3e052f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e4052fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53605305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcae05307, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29c05309, SUC_lexicon_tags_148 },
    { 0x1de0530a, SUC_lexicon_tags_2 },
    { 0x7820530b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe840530f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x42605312, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c05316, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd220531a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4180531c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5020531e, SUC_lexicon_tags_15 },
    { 0x5d60531f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35205325, SUC_lexicon_tags_25 },
    { 0xfe405326, SUC_lexicon_tags_77 },
    { 0x6da05327, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc0532b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c05331, SUC_lexicon_tags_11 },
    { 0x28c05332, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd2805334, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21005336, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c0533a, SUC_lexicon_tags_37 },
    { 0x1a00533a, SUC_lexicon_tags_17 },
    { 0x8a60533c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7605343, SUC_lexicon_tags_43 },
    { 0xcd005343, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfc805346, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xada05348, SUC_lexicon_tags_25 },
    { 0x83405349, SUC_lexicon_tags_1 },
    { 0xa5405349, SUC_lexicon_tags_12 },
    { 0xc5e0534b, SUC_lexicon_tags_0 },
    { 0xf7e0534c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1960534e, SUC_lexicon_tags_7 },
    { 0x49e0534f, SUC_lexicon_tags_60 },
    { 0xc2205350, SUC_lexicon_tags_8 },
    { 0x56c05350, SUC_lexicon_tags_47 },
    { 0xf4405352, SUC_lexicon_tags_37 },
    { 0x20005350, SUC_lexicon_tags_7 },
    { 0xb6005354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a05357, SUC_lexicon_tags_17 },
    { 0xc1205357, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2520535a, SUC_lexicon_tags_21 },
    { 0xb8a0535b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e05360, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9605363, SUC_lexicon_tags_77 },
    { 0x5d605364, SUC_lexicon_tags_11 },
    { 0x55005365, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfee05367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d60536a, SUC_lexicon_tags_47 },
    { 0xa620536b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd060536d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe140536f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e05371, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa05374, SUC_lexicon_tags_37 },
    { 0x3005375, SUC_lexicon_tags_2 },
    { 0x94a05376, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a05379, SUC_lexicon_tags_30 },
    { 0xcb00537a, SUC_lexicon_tags_26 },
    { 0x1fa0537a, SUC_lexicon_tags_29 },
    { 0x2260537b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26605387, SUC_lexicon_tags_25 },
    { 0xb7405388, SUC_lexicon_tags_12 },
    { 0x40605389, SUC_lexicon_tags_28 },
    { 0x8800538a, SUC_lexicon_tags_15 },
    { 0x8180538b, SUC_lexicon_tags_27 },
    { 0x7ac0538c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c0538f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a05395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c05399, SUC_lexicon_tags_25 },
    { 0x69a0539a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x56e0539c, SUC_lexicon_tags_11 },
    { 0x4840539d, SUC_lexicon_tags_25 },
    { 0x4cc0539e, SUC_lexicon_tags_6 },
    { 0x4040539d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570053a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950053aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6053af, SUC_lexicon_tags_50 },
    { 0x50e053b0, SUC_lexicon_tags_9 },
    { 0xfc2053b1, SUC_lexicon_tags_7 },
    { 0x98e053b2, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62053b5, SUC_lexicon_tags_74 },
    { 0xcd8053b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66053bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86053be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92053c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x554053c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0053ce, SUC_lexicon_tags_16 },
    { 0x144053ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58053d3, SUC_lexicon_tags_25 },
    { 0x1ee053d4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2053e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4053e5, SUC_lexicon_tags_23 },
    { 0x194053e6, SUC_lexicon_tags_23 },
    { 0xfa6053e6, SUC_lexicon_tags_104 },
    { 0x690053e5, SUC_lexicon_tags_38 },
    { 0x6c8053e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388053ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x246053f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8053f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892053f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb7c053fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x336053fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4605406, SUC_lexicon_tags_17 },
    { 0x10a05407, SUC_lexicon_tags_12 },
    { 0x9d405408, SUC_lexicon_tags_27 },
    { 0xd7605409, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d405414, SUC_lexicon_tags_17 },
    { 0x5ec05415, SUC_lexicon_tags_25 },
    { 0x24805416, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2700541a, SUC_lexicon_tags_65 },
    { 0xa8c0541b, SUC_lexicon_tags_19 },
    { 0xd060541a, SUC_lexicon_tags_25 },
    { 0x8a80541d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10a0541f, SUC_lexicon_tags_396 },
    { 0x8b40541f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc05427, SUC_lexicon_tags_2 },
    { 0x11e05428, SUC_lexicon_tags_15 },
    { 0x81a05429, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c05433, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe405435, SUC_lexicon_tags_38 },
    { 0xe5c05435, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c20543c, SUC_lexicon_tags_37 },
    { 0x6180543c, SUC_lexicon_tags_2 },
    { 0xa00543c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69205441, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb205443, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60605446, SUC_lexicon_tags_63 },
    { 0x3ce05447, SUC_lexicon_tags_46 },
    { 0x68805446, SUC_lexicon_tags_48 },
    { 0xbcc05446, SUC_lexicon_tags_1 },
    { 0xfca0544a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc0544d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa05450, SUC_lexicon_tags_0 },
    { 0x6ca05451, SUC_lexicon_tags_2 },
    { 0x8c605452, SUC_lexicon_tags_65 },
    { 0x3605453, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4205456, SUC_lexicon_tags_37 },
    { 0x39a05456, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4980545c, SUC_lexicon_tags_55 },
    { 0x840545c, SUC_lexicon_tags_37 },
    { 0xf560545e, SUC_lexicon_tags_17 },
    { 0x2a00545f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5f405461, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb805465, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeba05467, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9a05469, SUC_lexicon_tags_65 },
    { 0x66205469, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2d40546c, SUC_lexicon_tags_14 },
    { 0x5ba0546c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd605476, SUC_lexicon_tags_37 },
    { 0x37c05477, SUC_lexicon_tags_26 },
    { 0xa0605477, SUC_lexicon_tags_8 },
    { 0xb2405479, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d00547c, SUC_lexicon_tags_56 },
    { 0xddc0547c, SUC_lexicon_tags_26 },
    { 0xc440547c, SUC_lexicon_tags_1 },
    { 0xef40547e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e05484, SUC_lexicon_tags_12 },
    { 0x23205485, SUC_lexicon_tags_9 },
    { 0x90805484, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc60548d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc05490, SUC_lexicon_tags_2 },
    { 0x29605491, SUC_lexicon_tags_38 },
    { 0xfe005492, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94005499, SUC_lexicon_tags_151 },
    { 0xa3e0549a, SUC_lexicon_tags_36 },
    { 0xc4c0549b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0054a4, SUC_lexicon_tags_37 },
    { 0x5e0054a5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x396054a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x68c054a9, SUC_lexicon_tags_0 },
    { 0xdaa054a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb10054ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964054b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xefa054b2, SUC_lexicon_tags_17 },
    { 0x508054b3, SUC_lexicon_tags_12 },
    { 0x2bc054b2, SUC_lexicon_tags_85 },
    { 0xe76054b5, SUC_lexicon_tags_11 },
    { 0x8cc054b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4dc054b8, SUC_lexicon_tags_9 },
    { 0xf74054b9, SUC_lexicon_tags_17 },
    { 0x430054b9, SUC_lexicon_tags_28 },
    { 0x23c054bb, SUC_lexicon_tags_4 },
    { 0xa60054bc, SUC_lexicon_tags_17 },
    { 0xf54054bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4054c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8054ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46054ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2a054d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ce054d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932054d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4054d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29c054db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa054e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x354054e2, SUC_lexicon_tags_63 },
    { 0x34c054e3, SUC_lexicon_tags_49 },
    { 0x710054e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c054e8, SUC_lexicon_tags_15 },
    { 0x4e0054e8, SUC_lexicon_tags_7 },
    { 0xbee054ea, SUC_lexicon_tags_17 },
    { 0x336054eb, SUC_lexicon_tags_17 },
    { 0x446054ec, SUC_lexicon_tags_45 },
    { 0x31e054ec, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x23e054ef, SUC_lexicon_tags_38 },
    { 0x628054ef, SUC_lexicon_tags_7 },
    { 0xff4054f0, SUC_lexicon_tags_15 },
    { 0x77a054f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x504054f4, SUC_lexicon_tags_49 },
    { 0x29e054f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2054f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40054fc, SUC_lexicon_tags_32 },
    { 0x894054fc, SUC_lexicon_tags_37 },
    { 0xf46054fc, SUC_lexicon_tags_29 },
    { 0x1e054ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x73605502, SUC_lexicon_tags_63 },
    { 0x18805503, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa0550b, SUC_lexicon_tags_11 },
    { 0x3500550c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10805511, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c05514, SUC_lexicon_tags_11 },
    { 0xbb805514, SUC_lexicon_tags_37 },
    { 0x55405516, SUC_lexicon_tags_33 },
    { 0xc3805514, SUC_lexicon_tags_53 },
    { 0xaa05518, SUC_lexicon_tags_2 },
    { 0xed205519, SUC_lexicon_tags_0 },
    { 0xebe0551a, SUC_lexicon_tags_25 },
    { 0x47c0551b, SUC_lexicon_tags_8 },
    { 0xaac0551c, SUC_lexicon_tags_25 },
    { 0x2720551c, SUC_lexicon_tags_12 },
    { 0x9620551e, SUC_lexicon_tags_37 },
    { 0x1020551e, SUC_lexicon_tags_2 },
    { 0x90805520, SUC_lexicon_tags_26 },
    { 0x2e80551d, SUC_lexicon_tags_37 },
    { 0x1b805522, SUC_lexicon_tags_0 },
    { 0xc4605522, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c0552a, SUC_lexicon_tags_31 },
    { 0x16e0552a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf100552e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a05532, SUC_lexicon_tags_25 },
    { 0xed005532, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc605539, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e0553c, SUC_lexicon_tags_14 },
    { 0x81a0553d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc05540, SUC_lexicon_tags_26 },
    { 0x56605541, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3e405543, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca05548, SUC_lexicon_tags_2 },
    { 0x23005549, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4280554c, SUC_lexicon_tags_6 },
    { 0x7c0554d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc8e0554f, SUC_lexicon_tags_114 },
    { 0xd640554f, SUC_lexicon_tags_11 },
    { 0x90605551, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x84a05553, SUC_lexicon_tags_43 },
    { 0x59a05554, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e405557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9960555a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e805561, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33e05563, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb605566, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71005568, SUC_lexicon_tags_15 },
    { 0x5c605568, SUC_lexicon_tags_25 },
    { 0x4600556a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7180556e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e605577, SUC_lexicon_tags_14 },
    { 0xc8805577, SUC_lexicon_tags_39 },
    { 0xd0e05579, SUC_lexicon_tags_19 },
    { 0x45a0557a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5780558a, SUC_lexicon_tags_47 },
    { 0x3dc0558b, SUC_lexicon_tags_6 },
    { 0xf1c0558b, SUC_lexicon_tags_11 },
    { 0xd9c0558a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed205590, SUC_lexicon_tags_23 },
    { 0x82c05591, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1a05593, SUC_lexicon_tags_7 },
    { 0xf3c05594, SUC_lexicon_tags_34 },
    { 0x20005595, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b605599, SUC_lexicon_tags_19 },
    { 0x5d00559a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920055a0, SUC_lexicon_tags_37 },
    { 0x96a055a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4055a8, SUC_lexicon_tags_65 },
    { 0x15a055a9, SUC_lexicon_tags_25 },
    { 0xe4a055a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6055b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a4055b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83e055b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x744055b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb12055bc, SUC_lexicon_tags_24 },
    { 0x8a0055bc, SUC_lexicon_tags_42 },
    { 0xfbc055be, SUC_lexicon_tags_8 },
    { 0x518055bf, SUC_lexicon_tags_2 },
    { 0x780055bf, SUC_lexicon_tags_25 },
    { 0x61a055c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a055c7, SUC_lexicon_tags_148 },
    { 0x4fa055c7, SUC_lexicon_tags_47 },
    { 0xb30055c9, SUC_lexicon_tags_17 },
    { 0x240055ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6055cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a055d6, SUC_lexicon_tags_15 },
    { 0x12e055d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x230055d9, SUC_lexicon_tags_14 },
    { 0x6f4055da, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0x8fe055dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12055df, SUC_lexicon_tags_15 },
    { 0xbc6055e0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x16a055e2, SUC_lexicon_tags_26 },
    { 0xd4a055e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56055e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2055ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x514055ed, SUC_lexicon_tags_28 },
    { 0xc68055ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36055f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa08055f3, SUC_lexicon_tags_56 },
    { 0xbce055f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0055f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x914055f9, SUC_lexicon_tags_34 },
    { 0x6d6055f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4055fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x74205601, SUC_lexicon_tags_39 },
    { 0x91e05602, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9805605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb605607, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb000560a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb800560e, SUC_lexicon_tags_15 },
    { 0x2680560e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x30605612, SUC_lexicon_tags_116 },
    { 0x27a05613, SUC_lexicon_tags_19 },
    { 0x62405614, SUC_lexicon_tags_9 },
    { 0x23c05612, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xda605617, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc40561b, SUC_lexicon_tags_37 },
    { 0x6880561c, SUC_lexicon_tags_12 },
    { 0x9de0561c, SUC_lexicon_tags_47 },
    { 0x8c60561d, SUC_lexicon_tags_1 },
    { 0x99c0561f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25605624, SUC_lexicon_tags_8 },
    { 0xf205625, SUC_lexicon_tags_32 },
    { 0xb7405626, SUC_lexicon_tags_25 },
    { 0x86805627, SUC_lexicon_tags_30 },
    { 0x24405628, SUC_lexicon_tags_79 },
    { 0x2e205624, SUC_lexicon_tags_55 },
    { 0x1080562a, SUC_lexicon_tags_15 },
    { 0x3de0562b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c05630, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10005638, SUC_lexicon_tags_15 },
    { 0xafa05638, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a0563c, SUC_lexicon_tags_6 },
    { 0x9e0563d, SUC_lexicon_tags_11 },
    { 0x60a0563e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdbe05640, SUC_lexicon_tags_46 },
    { 0xd8005641, SUC_lexicon_tags_29 },
    { 0x22a05641, SUC_lexicon_tags_13 },
    { 0xd2c05643, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe005645, SUC_lexicon_tags_8 },
    { 0x1ba05646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6005648, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20564f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74e05651, SUC_lexicon_tags_1 },
    { 0x93c05652, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x74205655, SUC_lexicon_tags_65 },
    { 0x2d805655, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc805658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9460565a, SUC_lexicon_tags_37 },
    { 0xc960565a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea0565e, SUC_lexicon_tags_12 },
    { 0x5100565e, SUC_lexicon_tags_25 },
    { 0xee05660, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2005663, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19605665, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e05669, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc460566c, SUC_lexicon_tags_17 },
    { 0x8b20566d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b605670, SUC_lexicon_tags_37 },
    { 0x1aa05671, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a405673, SUC_lexicon_tags_13 },
    { 0x9f405674, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb405677, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53205679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f60567d, SUC_lexicon_tags_11 },
    { 0x9a40567d, SUC_lexicon_tags_29 },
    { 0x20c0567f, SUC_lexicon_tags_43 },
    { 0x9c05680, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c05683, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba405689, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0568c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d805694, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe3005696, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6605699, SUC_lexicon_tags_29 },
    { 0x2f605699, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca0569d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96056a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398056a7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x61e056a9, SUC_lexicon_tags_25 },
    { 0xca8056a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x308056ad, SUC_lexicon_tags_37 },
    { 0x6b2056ae, SUC_lexicon_tags_1 },
    { 0xd6a056ae, SUC_lexicon_tags_0 },
    { 0xb9c056b0, SUC_lexicon_tags_6 },
    { 0x68c056b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5c8056b3, SUC_lexicon_tags_25 },
    { 0x310056b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a056b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa1e056b9, SUC_lexicon_tags_77 },
    { 0x570056b9, SUC_lexicon_tags_151 },
    { 0x80a056bb, SUC_lexicon_tags_46 },
    { 0x376056ba, SUC_lexicon_tags_1 },
    { 0x9ea056bd, SUC_lexicon_tags_7 },
    { 0xd32056bd, SUC_lexicon_tags_21 },
    { 0x30056ba, SUC_lexicon_tags_17 },
    { 0x5ce056c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e056c4, SUC_lexicon_tags_14 },
    { 0xe4e056c4, SUC_lexicon_tags_63 },
    { 0xaa8056c6, SUC_lexicon_tags_21 },
    { 0x7e6056c7, SUC_lexicon_tags_95 },
    { 0xbc2056c5, SUC_lexicon_tags_61 },
    { 0x54c056c8, SUC_lexicon_tags_68 },
    { 0x1c8056c7, SUC_lexicon_tags_30 },
    { 0x20e056c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a056cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x284056d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x726056d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab2056d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c056db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58056de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366056e2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xbfc056e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe6056e6, SUC_lexicon_tags_1 },
    { 0x858056e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a056ea, SUC_lexicon_tags_23 },
    { 0x97c056ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86056ee, SUC_lexicon_tags_37 },
    { 0x8b8056ef, SUC_lexicon_tags_18 },
    { 0xd6c056f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a056f5, SUC_lexicon_tags_7 },
    { 0x46c056f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa2056f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448056fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44056ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde605701, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x84005703, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7780570c, SUC_lexicon_tags_664 },
    { 0x700570d, SUC_lexicon_tags_1 },
    { 0x10a0570c, SUC_lexicon_tags_68 },
    { 0x6160570f, SUC_lexicon_tags_12 },
    { 0x4720570f, SUC_lexicon_tags_37 },
    { 0x4080570c, SUC_lexicon_tags_15 },
    { 0x25c05711, SUC_lexicon_tags_255 },
    { 0xf8005710, SUC_lexicon_tags_0 },
    { 0x7fc05714, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7120571b, SUC_lexicon_tags_0 },
    { 0xd4c0571b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61805721, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3540572c, SUC_lexicon_tags_15 },
    { 0x38a0572d, SUC_lexicon_tags_6 },
    { 0x7d60572e, SUC_lexicon_tags_37 },
    { 0xd640572c, SUC_lexicon_tags_0 },
    { 0x1b205730, SUC_lexicon_tags_45 },
    { 0x1aa05731, SUC_lexicon_tags_25 },
    { 0x29405731, SUC_lexicon_tags_31 },
    { 0x8ba0572f, SUC_lexicon_tags_49 },
    { 0x65c05732, SUC_lexicon_tags_17 },
    { 0xac605735, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x50605737, SUC_lexicon_tags_11 },
    { 0x7a205737, SUC_lexicon_tags_95 },
    { 0x9005737, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c0573c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1f00573e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4405741, SUC_lexicon_tags_27 },
    { 0xad805742, SUC_lexicon_tags_1 },
    { 0x95605741, SUC_lexicon_tags_17 },
    { 0x60e05744, SUC_lexicon_tags_8 },
    { 0x87e05741, SUC_lexicon_tags_17 },
    { 0x6b205746, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2005749, SUC_lexicon_tags_2 },
    { 0xe640574a, SUC_lexicon_tags_7 },
    { 0xcf20574b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x7600574e, SUC_lexicon_tags_17 },
    { 0xf6e0574f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed605752, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e05755, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc200575b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0575e, SUC_lexicon_tags_17 },
    { 0x2fa0575e, SUC_lexicon_tags_25 },
    { 0x2aa0575f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54205765, SUC_lexicon_tags_37 },
    { 0x57205766, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0576a, SUC_lexicon_tags_63 },
    { 0x500576a, SUC_lexicon_tags_68 },
    { 0x6840576a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b405770, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa05777, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2005779, SUC_lexicon_tags_12 },
    { 0x9d005779, SUC_lexicon_tags_25 },
    { 0x46c0577b, SUC_lexicon_tags_37 },
    { 0x100577c, SUC_lexicon_tags_26 },
    { 0xdba0577d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3205781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66405783, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4a05785, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b405788, SUC_lexicon_tags_15 },
    { 0xd7a05788, SUC_lexicon_tags_66 },
    { 0xe3a0578a, SUC_lexicon_tags_43 },
    { 0xba40578b, SUC_lexicon_tags_0 },
    { 0x3a405788, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef405795, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320579a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c057a6, SUC_lexicon_tags_49 },
    { 0x4fe057a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78057ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec057ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224057b2, SUC_lexicon_tags_29 },
    { 0x576057b3, SUC_lexicon_tags_2 },
    { 0x27c057b4, SUC_lexicon_tags_11 },
    { 0xd84057b5, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x524057b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc20057ba, SUC_lexicon_tags_27 },
    { 0x98c057bb, SUC_lexicon_tags_31 },
    { 0xa6a057bc, SUC_lexicon_tags_63 },
    { 0xda2057bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa56057bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70057c7, SUC_lexicon_tags_38 },
    { 0xe46057c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x294057cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfe057d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20057d7, SUC_lexicon_tags_83 },
    { 0x57d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a057db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162057df, SUC_lexicon_tags_65 },
    { 0xafc057e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410057e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe66057e6, SUC_lexicon_tags_9 },
    { 0xe2a057e6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x458057e9, SUC_lexicon_tags_42 },
    { 0xbc2057ea, SUC_lexicon_tags_84 },
    { 0x8ce057eb, SUC_lexicon_tags_37 },
    { 0xa50057ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee057ef, SUC_lexicon_tags_2 },
    { 0xb6a057f0, SUC_lexicon_tags_15 },
    { 0xbb4057f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6057f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6057f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd24057fb, SUC_lexicon_tags_25 },
    { 0xdfa057fb, SUC_lexicon_tags_9 },
    { 0x51e057fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b605802, SUC_lexicon_tags_33 },
    { 0xa4005803, SUC_lexicon_tags_22 },
    { 0x10805804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3805806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b605809, SUC_lexicon_tags_7 },
    { 0xa1a0580a, SUC_lexicon_tags_32 },
    { 0xc6e0580b, SUC_lexicon_tags_32 },
    { 0x5f60580a, SUC_lexicon_tags_32 },
    { 0x5d00580d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf605810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54e05812, SUC_lexicon_tags_38 },
    { 0x6c005812, SUC_lexicon_tags_0 },
    { 0x40a05814, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63605817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c0581d, SUC_lexicon_tags_37 },
    { 0x9fa0581e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25405820, SUC_lexicon_tags_35 },
    { 0xede05820, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9805825, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf80582f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5405833, SUC_lexicon_tags_17 },
    { 0xe5605833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a05838, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a05843, SUC_lexicon_tags_7 },
    { 0xfce05843, SUC_lexicon_tags_37 },
    { 0x7a205845, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a05848, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c0584b, SUC_lexicon_tags_38 },
    { 0x4d40584b, SUC_lexicon_tags_10 },
    { 0xc260584b, SUC_lexicon_tags_37 },
    { 0xb3a0584b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e05851, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a405854, SUC_lexicon_tags_12 },
    { 0x59005855, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40605861, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e05866, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x79805869, SUC_lexicon_tags_25 },
    { 0x8f20586a, SUC_lexicon_tags_11 },
    { 0x1820586b, SUC_lexicon_tags_12 },
    { 0x3140586a, SUC_lexicon_tags_23 },
    { 0x3960586c, SUC_lexicon_tags_12 },
    { 0x76c0586e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6205872, SUC_lexicon_tags_39 },
    { 0xb3005872, SUC_lexicon_tags_68 },
    { 0x11405874, SUC_lexicon_tags_47 },
    { 0xd0c05872, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77205879, SUC_lexicon_tags_69 },
    { 0xde20587a, SUC_lexicon_tags_30 },
    { 0x12e0587a, SUC_lexicon_tags_10 },
    { 0x5fe0587c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c0587f, SUC_lexicon_tags_0 },
    { 0xbe00587f, SUC_lexicon_tags_104 },
    { 0x6500587f, SUC_lexicon_tags_25 },
    { 0x85c05882, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e205885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc160588d, SUC_lexicon_tags_55 },
    { 0xb4a0588e, SUC_lexicon_tags_43 },
    { 0x1e80588e, SUC_lexicon_tags_47 },
    { 0xa7c05890, SUC_lexicon_tags_11 },
    { 0xb960588e, SUC_lexicon_tags_63 },
    { 0x9d205892, SUC_lexicon_tags_1 },
    { 0x3dc05893, SUC_lexicon_tags_14 },
    { 0xeec05894, SUC_lexicon_tags_31 },
    { 0xbea05894, SUC_lexicon_tags_46 },
    { 0xcb60588f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e05899, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1820589c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd040589e, SUC_lexicon_tags_26 },
    { 0x3920589f, SUC_lexicon_tags_2 },
    { 0x8c80589e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0058a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186058a8, SUC_lexicon_tags_25 },
    { 0xdec058a8, SUC_lexicon_tags_1 },
    { 0xd34058a9, SUC_lexicon_tags_26 },
    { 0xb6a058a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6ca058ad, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x878058af, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90058b5, SUC_lexicon_tags_15 },
    { 0x6a4058b6, SUC_lexicon_tags_1 },
    { 0xd7c058b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c058be, SUC_lexicon_tags_17 },
    { 0xcb4058bf, SUC_lexicon_tags_7 },
    { 0x6ae058c0, SUC_lexicon_tags_17 },
    { 0xacc058bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98058c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c058c6, SUC_lexicon_tags_43 },
    { 0xca4058c6, SUC_lexicon_tags_1 },
    { 0x6d4058c8, SUC_lexicon_tags_25 },
    { 0x906058c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc058cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc058d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2058d6, SUC_lexicon_tags_11 },
    { 0x762058d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780058de, SUC_lexicon_tags_37 },
    { 0x3e058df, SUC_lexicon_tags_37 },
    { 0x7d0058e0, SUC_lexicon_tags_14 },
    { 0xe1e058e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb96058e3, SUC_lexicon_tags_28 },
    { 0xd58058e4, SUC_lexicon_tags_11 },
    { 0xac8058e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce058ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x850058ec, SUC_lexicon_tags_7 },
    { 0x4d6058ec, SUC_lexicon_tags_43 },
    { 0x5e8058ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e058f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ea058f5, SUC_lexicon_tags_37 },
    { 0xc1a058f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8be058f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb04058fa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xae8058fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a005900, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x83805903, SUC_lexicon_tags_60 },
    { 0x4e05903, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc05908, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0590d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e05911, SUC_lexicon_tags_11 },
    { 0xd4205911, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec05915, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae0591a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xef40591c, SUC_lexicon_tags_8 },
    { 0x3160591d, SUC_lexicon_tags_7 },
    { 0xe860591e, SUC_lexicon_tags_0 },
    { 0x9b80591f, SUC_lexicon_tags_37 },
    { 0xfc40591e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf205922, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x70205925, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe05928, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a0592c, SUC_lexicon_tags_25 },
    { 0x71c0592d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3020592f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x50805932, SUC_lexicon_tags_11 },
    { 0xc7a05933, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e05937, SUC_lexicon_tags_13 },
    { 0xbb805937, SUC_lexicon_tags_17 },
    { 0x6fa05939, SUC_lexicon_tags_7 },
    { 0x8f805938, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b60593d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4480593f, SUC_lexicon_tags_43 },
    { 0x2cc0593f, SUC_lexicon_tags_29 },
    { 0x7205941, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc405943, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe3405945, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdba05947, SUC_lexicon_tags_3 },
    { 0xca405947, SUC_lexicon_tags_11 },
    { 0x58205949, SUC_lexicon_tags_44 },
    { 0x85e05949, SUC_lexicon_tags_14 },
    { 0xd0a0594b, SUC_lexicon_tags_2 },
    { 0x92a0594c, SUC_lexicon_tags_37 },
    { 0xcba0594d, SUC_lexicon_tags_11 },
    { 0xe2405947, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd5c05950, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88005954, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8880595a, SUC_lexicon_tags_59 },
    { 0x28a0595b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61405962, SUC_lexicon_tags_1 },
    { 0x7e05962, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8205968, SUC_lexicon_tags_2 },
    { 0x3f405969, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41405971, SUC_lexicon_tags_11 },
    { 0x4005972, SUC_lexicon_tags_8 },
    { 0x9ca05973, SUC_lexicon_tags_33 },
    { 0xc4805973, SUC_lexicon_tags_17 },
    { 0x5cc05971, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc05978, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4580597a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c0597d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66805980, SUC_lexicon_tags_25 },
    { 0xd805980, SUC_lexicon_tags_19 },
    { 0x7c405982, SUC_lexicon_tags_25 },
    { 0xe2c05982, SUC_lexicon_tags_55 },
    { 0xf9205982, SUC_lexicon_tags_27 },
    { 0x2a605982, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e05990, SUC_lexicon_tags_47 },
    { 0x98e05990, SUC_lexicon_tags_49 },
    { 0xe2405992, SUC_lexicon_tags_14 },
    { 0x5d405993, SUC_lexicon_tags_46 },
    { 0x75a05994, SUC_lexicon_tags_11 },
    { 0x7ec05993, SUC_lexicon_tags_15 },
    { 0xbbe05996, SUC_lexicon_tags_37 },
    { 0x1ca05996, SUC_lexicon_tags_37 },
    { 0x77405998, SUC_lexicon_tags_9 },
    { 0x1c605997, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b60599c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6640599f, SUC_lexicon_tags_350 },
    { 0x2780599f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfda059a2, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e059a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38c059a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x882059aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484059b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x754059b3, SUC_lexicon_tags_23 },
    { 0xbae059b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8059b7, SUC_lexicon_tags_45 },
    { 0xc2c059b8, SUC_lexicon_tags_25 },
    { 0xcce059b9, SUC_lexicon_tags_21 },
    { 0x644059b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c059bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68059c0, SUC_lexicon_tags_43 },
    { 0xd2e059c0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x786059c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e059c6, SUC_lexicon_tags_17 },
    { 0xaf2059c7, SUC_lexicon_tags_47 },
    { 0x72c059c7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c059cb, SUC_lexicon_tags_23 },
    { 0x426059cc, SUC_lexicon_tags_0 },
    { 0x356059cc, SUC_lexicon_tags_19 },
    { 0xa1e059ce, SUC_lexicon_tags_12 },
    { 0x436059cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c059d3, SUC_lexicon_tags_23 },
    { 0x390059d4, SUC_lexicon_tags_4 },
    { 0x594059d4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xef6059d7, SUC_lexicon_tags_63 },
    { 0xda059d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e059dd, SUC_lexicon_tags_14 },
    { 0xa3a059dd, SUC_lexicon_tags_11 },
    { 0xd84059de, SUC_lexicon_tags_15 },
    { 0xab2059e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe059e5, SUC_lexicon_tags_32 },
    { 0xe96059e6, SUC_lexicon_tags_25 },
    { 0xacc059e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8059ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7e059ee, SUC_lexicon_tags_12 },
    { 0x98059ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5be059f1, SUC_lexicon_tags_23 },
    { 0xf40059f1, SUC_lexicon_tags_38 },
    { 0x47a059f2, SUC_lexicon_tags_15 },
    { 0xd9a059f4, SUC_lexicon_tags_7 },
    { 0x2a8059f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48059f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42059fe, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62605a03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a405a08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75405a0c, SUC_lexicon_tags_1 },
    { 0xcfc05a0d, SUC_lexicon_tags_37 },
    { 0xe7a05a0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeae05a10, SUC_lexicon_tags_4 },
    { 0xc4605a11, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05a14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6005a1a, SUC_lexicon_tags_25 },
    { 0x63805a1b, SUC_lexicon_tags_33 },
    { 0x1005a1c, SUC_lexicon_tags_2 },
    { 0xf6805a1c, SUC_lexicon_tags_36 },
    { 0x16205a1e, SUC_lexicon_tags_37 },
    { 0xa0a05a1a, SUC_lexicon_tags_148 },
    { 0xfbe05a20, SUC_lexicon_tags_63 },
    { 0xe1205a20, SUC_lexicon_tags_17 },
    { 0x2e805a1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a05a30, SUC_lexicon_tags_7 },
    { 0xc6e05a31, SUC_lexicon_tags_15 },
    { 0x22405a31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4605a34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde05a37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42205a3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d205a3d, SUC_lexicon_tags_6 },
    { 0xda05a3e, SUC_lexicon_tags_29 },
    { 0x2e605a3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb805a44, SUC_lexicon_tags_11 },
    { 0x35a05a45, SUC_lexicon_tags_12 },
    { 0x64605a45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2605a48, SUC_lexicon_tags_14 },
    { 0xe605a49, SUC_lexicon_tags_25 },
    { 0x7a205a49, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49605a4e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7005a53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0805a5c, SUC_lexicon_tags_23 },
    { 0x17405a5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27005a61, SUC_lexicon_tags_33 },
    { 0xa9405a62, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3205a65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1605a6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26405a6d, SUC_lexicon_tags_27 },
    { 0x6f205a6e, SUC_lexicon_tags_12 },
    { 0xd5805a6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c05a73, SUC_lexicon_tags_13 },
    { 0xd205a74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac05a77, SUC_lexicon_tags_37 },
    { 0xc9005a78, SUC_lexicon_tags_7 },
    { 0xb2c05a79, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2205a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0405a7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e05a80, SUC_lexicon_tags_15 },
    { 0xd2205a81, SUC_lexicon_tags_0 },
    { 0xcb005a82, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0805a8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e405a8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33005a90, SUC_lexicon_tags_1 },
    { 0xb5a05a90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25a05a93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e05a9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa05aa1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xefa05aa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c05aa7, SUC_lexicon_tags_37 },
    { 0x72205aa8, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x8c005aaa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d005aaf, SUC_lexicon_tags_23 },
    { 0xd6605ab0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1405ab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42605ab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce205abb, SUC_lexicon_tags_62 },
    { 0xa5e05abb, SUC_lexicon_tags_8 },
    { 0x9ba05abc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da05ac0, SUC_lexicon_tags_79 },
    { 0x1d005ac1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e05ac7, SUC_lexicon_tags_25 },
    { 0x6a405ac7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb005aca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d005ad3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4be05ad5, SUC_lexicon_tags_2 },
    { 0x16605ad6, SUC_lexicon_tags_23 },
    { 0x1005ad7, SUC_lexicon_tags_62 },
    { 0xddc05ad5, SUC_lexicon_tags_15 },
    { 0xd4205ad6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe05add, SUC_lexicon_tags_25 },
    { 0xac05ade, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51805ae3, SUC_lexicon_tags_25 },
    { 0xfe405ae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e05aef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe5e05af1, SUC_lexicon_tags_87 },
    { 0x6da05af2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe05af7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe205afb, SUC_lexicon_tags_25 },
    { 0xbd205afc, SUC_lexicon_tags_25 },
    { 0x6c405afd, SUC_lexicon_tags_12 },
    { 0xa8005afe, SUC_lexicon_tags_10 },
    { 0x8d805aff, SUC_lexicon_tags_21 },
    { 0x23e05afc, SUC_lexicon_tags_17 },
    { 0xe1605b01, SUC_lexicon_tags_7 },
    { 0x94c05aff, SUC_lexicon_tags_11 },
    { 0xf4405b03, SUC_lexicon_tags_28 },
    { 0x7e405b02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xed405b07, SUC_lexicon_tags_25 },
    { 0x83205b08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c05b12, SUC_lexicon_tags_29 },
    { 0xa7c05b13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c05b17, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe605b1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8605b22, SUC_lexicon_tags_31 },
    { 0x43205b23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8205b27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e05b2b, SUC_lexicon_tags_8 },
    { 0x80805b2c, SUC_lexicon_tags_30 },
    { 0xcf205b2c, SUC_lexicon_tags_5 },
    { 0x1dc05b2e, SUC_lexicon_tags_12 },
    { 0x8d605b2d, SUC_lexicon_tags_11 },
    { 0xee005b2e, SUC_lexicon_tags_33 },
    { 0x56605b2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3805b39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91405b3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72405b3d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x30605b3f, SUC_lexicon_tags_68 },
    { 0x3e005b40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1405b42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc205b44, SUC_lexicon_tags_12 },
    { 0xd3205b44, SUC_lexicon_tags_37 },
    { 0x93405b45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3605b4b, SUC_lexicon_tags_0 },
    { 0x32005b4c, SUC_lexicon_tags_37 },
    { 0xfb005b4d, SUC_lexicon_tags_12 },
    { 0x5b205b4e, SUC_lexicon_tags_66 },
    { 0x5b605b4f, SUC_lexicon_tags_2 },
    { 0x77005b4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27c05b52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8805b54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0005b56, SUC_lexicon_tags_45 },
    { 0xe5a05b57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27005b59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c05b5c, SUC_lexicon_tags_11 },
    { 0xd0205b5c, SUC_lexicon_tags_11 },
    { 0x9a005b5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36c05b60, SUC_lexicon_tags_16 },
    { 0x43e05b61, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde05b65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dc05b67, SUC_lexicon_tags_12 },
    { 0x81205b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5205b6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc205b6e, SUC_lexicon_tags_15 },
    { 0x7d405b6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd805b73, SUC_lexicon_tags_46 },
    { 0xb2005b73, SUC_lexicon_tags_25 },
    { 0xfe05b75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d405b7a, SUC_lexicon_tags_37 },
    { 0x70405b7b, SUC_lexicon_tags_37 },
    { 0x47c05b7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c05b81, SUC_lexicon_tags_37 },
    { 0x9c405b81, SUC_lexicon_tags_44 },
    { 0xb5605b83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8a05b85, SUC_lexicon_tags_35 },
    { 0xddc05b86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67605b89, SUC_lexicon_tags_25 },
    { 0x28e05b89, SUC_lexicon_tags_14 },
    { 0x2c405b8b, SUC_lexicon_tags_43 },
    { 0x85605b8c, SUC_lexicon_tags_14 },
    { 0x5b605b8d, SUC_lexicon_tags_19 },
    { 0xe4e05b8b, SUC_lexicon_tags_19 },
    { 0x90005b8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6805b93, SUC_lexicon_tags_27 },
    { 0x58005b94, SUC_lexicon_tags_25 },
    { 0x2fe05b95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9605b98, SUC_lexicon_tags_63 },
    { 0x8e205b99, SUC_lexicon_tags_19 },
    { 0xc2405b9a, SUC_lexicon_tags_17 },
    { 0x83605b98, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e05b9f, SUC_lexicon_tags_7 },
    { 0xee405ba0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad205bac, SUC_lexicon_tags_17 },
    { 0x7b005bac, SUC_lexicon_tags_2 },
    { 0x7ba05bae, SUC_lexicon_tags_47 },
    { 0x98e05baf, SUC_lexicon_tags_11 },
    { 0xa405bae, SUC_lexicon_tags_45 },
    { 0x53a05bb1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4a805bb3, SUC_lexicon_tags_15 },
    { 0xa5c05bb4, SUC_lexicon_tags_15 },
    { 0x85605bb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85205bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a05bc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab005bc4, SUC_lexicon_tags_46 },
    { 0xc7c05bc5, SUC_lexicon_tags_60 },
    { 0x32005bc6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x77605bc8, SUC_lexicon_tags_15 },
    { 0x7e005bc9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7805bd1, SUC_lexicon_tags_3 },
    { 0x1a405bd2, SUC_lexicon_tags_23 },
    { 0xac205bd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86805bd9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcde05bdb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f605bde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0405be1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66205be7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca05beb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x34005bed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x48205bef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb405bf5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f405bfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde05bff, SUC_lexicon_tags_26 },
    { 0x31e05c00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16405c08, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5b205c0a, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x15005c0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a05c10, SUC_lexicon_tags_25 },
    { 0x5b805c10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb805c15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82605c19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa605c1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b805c1e, SUC_lexicon_tags_68 },
    { 0xa5c05c1f, SUC_lexicon_tags_13 },
    { 0x54805c1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7205c24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b205c26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec05c2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52205c2d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd405c30, SUC_lexicon_tags_25 },
    { 0x74605c30, SUC_lexicon_tags_60 },
    { 0xb9c05c32, SUC_lexicon_tags_37 },
    { 0xbc405c32, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7b605c35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb3205c37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63405c3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x98205c3d, SUC_lexicon_tags_6 },
    { 0xc805c3e, SUC_lexicon_tags_37 },
    { 0x6a605c3f, SUC_lexicon_tags_31 },
    { 0xf6605c40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca05c43, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x48405c45, SUC_lexicon_tags_38 },
    { 0xf7205c45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e05c4d, SUC_lexicon_tags_23 },
    { 0xa0805c4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x67e05c50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2005c57, SUC_lexicon_tags_33 },
    { 0x2c405c57, SUC_lexicon_tags_17 },
    { 0x4fc05c59, SUC_lexicon_tags_12 },
    { 0x16205c5a, SUC_lexicon_tags_12 },
    { 0xf2e05c5b, SUC_lexicon_tags_19 },
    { 0x9ac05c5a, SUC_lexicon_tags_15 },
    { 0x85805c5d, SUC_lexicon_tags_29 },
    { 0x78e05c59, SUC_lexicon_tags_1 },
    { 0x95605c5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec405c63, SUC_lexicon_tags_35 },
    { 0x41005c63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb805c68, SUC_lexicon_tags_25 },
    { 0x4fa05c69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e05c78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba05c82, SUC_lexicon_tags_1 },
    { 0xb6805c82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0405c86, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98605c8b, SUC_lexicon_tags_27 },
    { 0xed605c8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e05c93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1da05c95, SUC_lexicon_tags_7 },
    { 0xdca05c96, SUC_lexicon_tags_66 },
    { 0xcb205c97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9005c99, SUC_lexicon_tags_1 },
    { 0xb2e05c9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32205c9c, SUC_lexicon_tags_17 },
    { 0x57005c9c, SUC_lexicon_tags_2 },
    { 0x7c405c9c, SUC_lexicon_tags_15 },
    { 0xd7605c9e, SUC_lexicon_tags_10 },
    { 0xbe405ca0, SUC_lexicon_tags_30 },
    { 0x7aa05c9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f005ca3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1005ca8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a05caf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c05cb2, SUC_lexicon_tags_47 },
    { 0x44405cb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83605cb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f005cba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94205cbd, SUC_lexicon_tags_147 },
    { 0x7c605cbe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e805cc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaa205cc4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a405cc7, SUC_lexicon_tags_27 },
    { 0xbc405cc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f805ccc, SUC_lexicon_tags_37 },
    { 0x8d205ccd, SUC_lexicon_tags_11 },
    { 0xe1205cce, SUC_lexicon_tags_142 },
    { 0x66005ccf, SUC_lexicon_tags_38 },
    { 0x5ca05cd0, SUC_lexicon_tags_44 },
    { 0x16e05ccf, SUC_lexicon_tags_46 },
    { 0x93805ccf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c605cd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc05cd9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea05ce1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e805ce3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x86805ce6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80605ceb, SUC_lexicon_tags_63 },
    { 0xf2a05cec, SUC_lexicon_tags_14 },
    { 0x69c05ceb, SUC_lexicon_tags_43 },
    { 0x25005ced, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26605cf2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4e05cf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e05cf8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x94c05cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94205cfd, SUC_lexicon_tags_43 },
    { 0x29005cfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d805d00, SUC_lexicon_tags_23 },
    { 0x8ec05d00, SUC_lexicon_tags_30 },
    { 0x8d605d00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e005d05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x94205d08, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xce05d0a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a05d0d, SUC_lexicon_tags_60 },
    { 0xf0605d0e, SUC_lexicon_tags_61 },
    { 0x42e05d0f, SUC_lexicon_tags_25 },
    { 0x69005d10, SUC_lexicon_tags_8 },
    { 0x52005d11, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a805d19, SUC_lexicon_tags_8 },
    { 0x78405d1a, SUC_lexicon_tags_17 },
    { 0x6e05d19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd7c05d1d, SUC_lexicon_tags_17 },
    { 0xc4205d1e, SUC_lexicon_tags_170 },
    { 0x9605d1f, SUC_lexicon_tags_14 },
    { 0x25605d1f, SUC_lexicon_tags_28 },
    { 0x1d805d20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39a05d23, SUC_lexicon_tags_12 },
    { 0xa0c05d24, SUC_lexicon_tags_25 },
    { 0x9dc05d25, SUC_lexicon_tags_8 },
    { 0x9f805d24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4205d2a, SUC_lexicon_tags_12 },
    { 0x99c05d2b, SUC_lexicon_tags_3 },
    { 0x9605d2b, SUC_lexicon_tags_14 },
    { 0x2b805d2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1d405d2f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87005d34, SUC_lexicon_tags_49 },
    { 0x5005d34, SUC_lexicon_tags_31 },
    { 0x6da05d36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c205d39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf2805d3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6605d40, SUC_lexicon_tags_10 },
    { 0xfb005d41, SUC_lexicon_tags_1 },
    { 0x5a805d42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a05d49, SUC_lexicon_tags_15 },
    { 0xf7805d4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12405d4c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x50605d4f, SUC_lexicon_tags_14 },
    { 0x87c05d50, SUC_lexicon_tags_37 },
    { 0xfe005d51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x58605d54, SUC_lexicon_tags_1 },
    { 0x23a05d54, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee405d5a, SUC_lexicon_tags_27 },
    { 0x6de05d5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a05d5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1605d60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35c05d62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a405d65, SUC_lexicon_tags_37 },
    { 0xfb405d66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a05d71, SUC_lexicon_tags_60 },
    { 0x94205d72, SUC_lexicon_tags_37 },
    { 0x3e05d71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea05d76, SUC_lexicon_tags_63 },
    { 0xd2a05d77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e05d7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a05d81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6405d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc805d8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a805d8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c05d92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb205d94, SUC_lexicon_tags_27 },
    { 0x7bc05d95, SUC_lexicon_tags_17 },
    { 0x74e05d96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8405d9b, SUC_lexicon_tags_25 },
    { 0x73a05d9c, SUC_lexicon_tags_55 },
    { 0xb0005d9d, SUC_lexicon_tags_85 },
    { 0x34e05d9e, SUC_lexicon_tags_15 },
    { 0xc7805d9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a05da1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x87005da4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84c05da6, SUC_lexicon_tags_83 },
    { 0x35e05da7, SUC_lexicon_tags_12 },
    { 0x24205da6, SUC_lexicon_tags_14 },
    { 0xc3805da9, SUC_lexicon_tags_19 },
    { 0xfe05daa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c05dad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x30005daf, SUC_lexicon_tags_60 },
    { 0x2ba05db0, SUC_lexicon_tags_63 },
    { 0xc005db1, SUC_lexicon_tags_55 },
    { 0x63805daf, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0x91405db4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa05dbc, SUC_lexicon_tags_12 },
    { 0xb0a05dbc, SUC_lexicon_tags_37 },
    { 0x18605dbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e05dc3, SUC_lexicon_tags_14 },
    { 0x82205dc4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e05dc9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f805dcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfe05dce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x41605dd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bc05dd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd205dd5, SUC_lexicon_tags_60 },
    { 0xd2a05dd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3405dd8, SUC_lexicon_tags_39 },
    { 0xa0805dd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51405ddb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8d005ddd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e805de4, SUC_lexicon_tags_17 },
    { 0x30805de5, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x75405de8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88605deb, SUC_lexicon_tags_15 },
    { 0x5ca05deb, SUC_lexicon_tags_37 },
    { 0xb6605ded, SUC_lexicon_tags_15 },
    { 0x80405ded, SUC_lexicon_tags_17 },
    { 0xbaa05deb, SUC_lexicon_tags_63 },
    { 0x3b005df0, SUC_lexicon_tags_15 },
    { 0xfb205ded, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce05df4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9805df9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x605dfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c05dff, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a205e03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63805e08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae05e0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb005e11, SUC_lexicon_tags_15 },
    { 0xcf205e11, SUC_lexicon_tags_27 },
    { 0xd7c05e13, SUC_lexicon_tags_551 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11005e18, SUC_lexicon_tags_45 },
    { 0x9ae05e18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95005e1b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c805e1f, SUC_lexicon_tags_46 },
    { 0xe0c05e20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17405e22, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x34405e24, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xad005e27, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82005e2c, SUC_lexicon_tags_7 },
    { 0xe805e2d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e05e30, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56005e33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa6605e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c05e39, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40005e3e, SUC_lexicon_tags_23 },
    { 0x2605e3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f005e43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c05e46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a05e53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc005e55, SUC_lexicon_tags_8 },
    { 0x38805e55, SUC_lexicon_tags_23 },
    { 0x31005e56, SUC_lexicon_tags_29 },
    { 0x56205e58, SUC_lexicon_tags_23 },
    { 0x45005e59, SUC_lexicon_tags_17 },
    { 0xc5c05e58, SUC_lexicon_tags_23 },
    { 0xcc605e5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41005e5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c05e62, SUC_lexicon_tags_0 },
    { 0x63805e63, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a05e66, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xef205e68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32805e6d, SUC_lexicon_tags_56 },
    { 0xac005e6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a05e74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdea05e76, SUC_lexicon_tags_11 },
    { 0x13805e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32605e7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7205e80, SUC_lexicon_tags_14 },
    { 0xb7205e80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84405e86, SUC_lexicon_tags_45 },
    { 0xa8c05e87, SUC_lexicon_tags_19 },
    { 0x38205e88, SUC_lexicon_tags_7 },
    { 0xfb005e89, SUC_lexicon_tags_26 },
    { 0xafa05e89, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f605e91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4205e96, SUC_lexicon_tags_11 },
    { 0x92c05e97, SUC_lexicon_tags_7 },
    { 0x2e805e97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce05e9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8805ea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2605ea3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x85205ea5, SUC_lexicon_tags_10 },
    { 0x94805ea6, SUC_lexicon_tags_37 },
    { 0xb4405ea7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12605eaa, SUC_lexicon_tags_19 },
    { 0x43c05eab, SUC_lexicon_tags_9 },
    { 0x64c05eaa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4c05eae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e05eb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x20a05eb9, SUC_lexicon_tags_19 },
    { 0x81205eba, SUC_lexicon_tags_15 },
    { 0xeb605eba, SUC_lexicon_tags_10 },
    { 0x76805ebb, SUC_lexicon_tags_70 },
    { 0xbc205ebb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e05ec2, SUC_lexicon_tags_68 },
    { 0x1c205ec2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da05ec9, SUC_lexicon_tags_13 },
    { 0xbc05eca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb2205ecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c05ed0, SUC_lexicon_tags_15 },
    { 0x5e405ed1, SUC_lexicon_tags_15 },
    { 0xd8005ed1, SUC_lexicon_tags_43 },
    { 0x76605ed0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1405ed7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f405edf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x83205ee2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16005ee6, SUC_lexicon_tags_55 },
    { 0x49e05ee6, SUC_lexicon_tags_7 },
    { 0xeba05ee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1405eeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8405eee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5405ef4, SUC_lexicon_tags_46 },
    { 0x40205ef5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26c05ef7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb605eff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf5805f01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15a05f03, SUC_lexicon_tags_1 },
    { 0x6f405f04, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x36605f07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4405f0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25605f10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c05f14, SUC_lexicon_tags_55 },
    { 0xddc05f14, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd3c05f17, SUC_lexicon_tags_11 },
    { 0xcfc05f17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a05f1b, SUC_lexicon_tags_82 },
    { 0x47605f1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f405f22, SUC_lexicon_tags_23 },
    { 0xdb005f22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e05f26, SUC_lexicon_tags_0 },
    { 0x96c05f27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa005f33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8805f36, SUC_lexicon_tags_37 },
    { 0xa2e05f36, SUC_lexicon_tags_17 },
    { 0x88805f38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ba05f3a, SUC_lexicon_tags_377 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a05f3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96605f46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70405f48, SUC_lexicon_tags_26 },
    { 0xa805f48, SUC_lexicon_tags_6 },
    { 0x93405f48, SUC_lexicon_tags_23 },
    { 0xfe405f49, SUC_lexicon_tags_14 },
    { 0xc2005f4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d605f4e, SUC_lexicon_tags_19 },
    { 0x92c05f4f, SUC_lexicon_tags_11 },
    { 0x42405f50, SUC_lexicon_tags_33 },
    { 0x44805f51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d205f54, SUC_lexicon_tags_63 },
    { 0x69405f54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed405f5b, SUC_lexicon_tags_37 },
    { 0x55205f5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1805f5e, SUC_lexicon_tags_25 },
    { 0x18205f5f, SUC_lexicon_tags_43 },
    { 0x24a05f60, SUC_lexicon_tags_7 },
    { 0x78805f61, SUC_lexicon_tags_15 },
    { 0x9e205f5f, SUC_lexicon_tags_13 },
    { 0x7a005f63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9005f67, SUC_lexicon_tags_2 },
    { 0xcee05f67, SUC_lexicon_tags_2 },
    { 0xe3a05f68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91e05f6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30205f6f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd1605f71, SUC_lexicon_tags_46 },
    { 0x4b205f72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd2205f74, SUC_lexicon_tags_42 },
    { 0x97a05f74, SUC_lexicon_tags_7 },
    { 0xf0c05f76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc05f79, SUC_lexicon_tags_499 },
    { 0x81e05f7a, SUC_lexicon_tags_14 },
    { 0xec405f79, SUC_lexicon_tags_2 },
    { 0x55805f7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f605f7f, SUC_lexicon_tags_37 },
    { 0x89c05f7f, SUC_lexicon_tags_15 },
    { 0x4aa05f81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc605f85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfd405f87, SUC_lexicon_tags_17 },
    { 0xeb205f88, SUC_lexicon_tags_37 },
    { 0x82405f87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66805f8f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x42005f92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x30005f94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24005f97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93605f9b, SUC_lexicon_tags_13 },
    { 0x10a05f9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e05f9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe005fa1, SUC_lexicon_tags_25 },
    { 0x2ae05fa2, SUC_lexicon_tags_61 },
    { 0x18005fa3, SUC_lexicon_tags_37 },
    { 0x205fa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64605fa6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34005fab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8005fb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23805fb5, SUC_lexicon_tags_2 },
    { 0x38c05fb6, SUC_lexicon_tags_16 },
    { 0x9205fb5, SUC_lexicon_tags_11 },
    { 0xe1405fb8, SUC_lexicon_tags_25 },
    { 0x65c05fb8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b605fc2, SUC_lexicon_tags_17 },
    { 0x7ca05fc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64005fc5, SUC_lexicon_tags_194 },
    { 0xa6605fc6, SUC_lexicon_tags_63 },
    { 0x3a405fc6, SUC_lexicon_tags_38 },
    { 0x7d605fc8, SUC_lexicon_tags_12 },
    { 0x13205fc9, SUC_lexicon_tags_6 },
    { 0xae805fc9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa8805fcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e05fcf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99205fd3, SUC_lexicon_tags_15 },
    { 0x33005fd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xca805fd7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x71c05fd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69605fdb, SUC_lexicon_tags_14 },
    { 0x78805fdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfcc05fde, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c05fe3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb605fe9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a405feb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10205fed, SUC_lexicon_tags_35 },
    { 0x15a05fee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf405ff3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a05ff9, SUC_lexicon_tags_39 },
    { 0x9c205ff9, SUC_lexicon_tags_9 },
    { 0x3c805ffb, SUC_lexicon_tags_11 },
    { 0x89e05ffb, SUC_lexicon_tags_25 },
    { 0x66a05ff9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e06004, SUC_lexicon_tags_17 },
    { 0xf2806005, SUC_lexicon_tags_1 },
    { 0x1d606006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6080600a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8220600f, SUC_lexicon_tags_12 },
    { 0x5f006010, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0806015, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56206018, SUC_lexicon_tags_7 },
    { 0xad406019, SUC_lexicon_tags_27 },
    { 0xd040601a, SUC_lexicon_tags_1 },
    { 0xc8c06018, SUC_lexicon_tags_14 },
    { 0x4e80601c, SUC_lexicon_tags_7 },
    { 0xc2406018, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc1c0601f, SUC_lexicon_tags_7 },
    { 0x22c0601f, SUC_lexicon_tags_17 },
    { 0x69606021, SUC_lexicon_tags_12 },
    { 0x85c06022, SUC_lexicon_tags_7 },
    { 0x72e06021, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e06026, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3406030, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71606039, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4cc0603b, SUC_lexicon_tags_0 },
    { 0x3b60603c, SUC_lexicon_tags_45 },
    { 0x3fc0603c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40a0603f, SUC_lexicon_tags_556 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee06043, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29606049, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e0604c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa720604f, SUC_lexicon_tags_11 },
    { 0xf00604f, SUC_lexicon_tags_27 },
    { 0x5e806051, SUC_lexicon_tags_11 },
    { 0x38c06051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f406056, SUC_lexicon_tags_37 },
    { 0xfc606057, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d60605b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea0605e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ea06060, SUC_lexicon_tags_2 },
    { 0x4aa06061, SUC_lexicon_tags_37 },
    { 0xa6206062, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9006066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a06069, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6780606b, SUC_lexicon_tags_9 },
    { 0x3260606c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x78a0606e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4006071, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe006076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7006078, SUC_lexicon_tags_12 },
    { 0xd8806079, SUC_lexicon_tags_14 },
    { 0x75406079, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe820607e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f206083, SUC_lexicon_tags_9 },
    { 0x13a06084, SUC_lexicon_tags_8 },
    { 0xda206085, SUC_lexicon_tags_7 },
    { 0x6ee06086, SUC_lexicon_tags_11 },
    { 0xa0006087, SUC_lexicon_tags_70 },
    { 0xe1206088, SUC_lexicon_tags_1 },
    { 0xd5606087, SUC_lexicon_tags_39 },
    { 0x2406089, SUC_lexicon_tags_27 },
    { 0xab40608b, SUC_lexicon_tags_7 },
    { 0xe9e0608b, SUC_lexicon_tags_15 },
    { 0xa320608b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a06090, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48606095, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8206097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c0609a, SUC_lexicon_tags_37 },
    { 0x840609b, SUC_lexicon_tags_11 },
    { 0x6b80609b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x2460609f, SUC_lexicon_tags_17 },
    { 0x21a060a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312060a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x868060a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2a060aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424060b1, SUC_lexicon_tags_25 },
    { 0xb42060b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80060b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x434060b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0060bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc6060c1, SUC_lexicon_tags_9 },
    { 0xe96060c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x494060c4, SUC_lexicon_tags_25 },
    { 0xefc060c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2c060c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8060cd, SUC_lexicon_tags_12 },
    { 0xac060ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144060d4, SUC_lexicon_tags_37 },
    { 0xeb6060d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb0060d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf8060d9, SUC_lexicon_tags_47 },
    { 0x476060d9, SUC_lexicon_tags_37 },
    { 0xed8060db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x310060de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8060e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4060e4, SUC_lexicon_tags_37 },
    { 0x366060e5, SUC_lexicon_tags_14 },
    { 0xedc060e6, SUC_lexicon_tags_12 },
    { 0x50a060e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb5c060e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a060f0, SUC_lexicon_tags_15 },
    { 0xfda060f0, SUC_lexicon_tags_1 },
    { 0xe76060f2, SUC_lexicon_tags_38 },
    { 0xa50060f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c060f7, SUC_lexicon_tags_2 },
    { 0x376060f7, SUC_lexicon_tags_2 },
    { 0xe0e060f9, SUC_lexicon_tags_15 },
    { 0xe02060fa, SUC_lexicon_tags_17 },
    { 0x3e4060fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x130060fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x674060ff, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1c606101, SUC_lexicon_tags_23 },
    { 0x6ee06102, SUC_lexicon_tags_17 },
    { 0xe06102, SUC_lexicon_tags_47 },
    { 0xb5006104, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac0610c, SUC_lexicon_tags_30 },
    { 0x2900610d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc06110, SUC_lexicon_tags_34 },
    { 0xd9606111, SUC_lexicon_tags_11 },
    { 0xa6a06111, SUC_lexicon_tags_7 },
    { 0x6ae06113, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36406115, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xee406117, SUC_lexicon_tags_11 },
    { 0x50206117, SUC_lexicon_tags_37 },
    { 0x8ae06119, SUC_lexicon_tags_47 },
    { 0x8c006119, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c20611e, SUC_lexicon_tags_33 },
    { 0x9e80611f, SUC_lexicon_tags_14 },
    { 0x1d006120, SUC_lexicon_tags_4 },
    { 0x9b406121, SUC_lexicon_tags_25 },
    { 0x7d006122, SUC_lexicon_tags_17 },
    { 0xd7a06123, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2a206125, SUC_lexicon_tags_30 },
    { 0x32006125, SUC_lexicon_tags_38 },
    { 0x8206127, SUC_lexicon_tags_84 },
    { 0x1206128, SUC_lexicon_tags_11 },
    { 0x8f606129, SUC_lexicon_tags_33 },
    { 0x9240612a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c60612d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64406135, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd40613a, SUC_lexicon_tags_31 },
    { 0x1a20613b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c606144, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e06147, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe80614a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa380614c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3606152, SUC_lexicon_tags_25 },
    { 0x87806152, SUC_lexicon_tags_99 },
    { 0x91c06154, SUC_lexicon_tags_10 },
    { 0xf6a06155, SUC_lexicon_tags_95 },
    { 0x70206156, SUC_lexicon_tags_49 },
    { 0x41006156, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe120615b, SUC_lexicon_tags_11 },
    { 0x1a60615b, SUC_lexicon_tags_38 },
    { 0x5360615d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e06160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b406162, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d206166, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a0616a, SUC_lexicon_tags_12 },
    { 0xc20616b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee0616f, SUC_lexicon_tags_2 },
    { 0x85206170, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e606175, SUC_lexicon_tags_26 },
    { 0x3ee06175, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf2c06178, SUC_lexicon_tags_30 },
    { 0x44006179, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xf980617b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb340617e, SUC_lexicon_tags_35 },
    { 0x8180617e, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93006183, SUC_lexicon_tags_2 },
    { 0x51806184, SUC_lexicon_tags_1 },
    { 0x24a06183, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3fc06187, SUC_lexicon_tags_17 },
    { 0x9f406187, SUC_lexicon_tags_25 },
    { 0xb9406189, SUC_lexicon_tags_11 },
    { 0x1900618a, SUC_lexicon_tags_140 },
    { 0x9960618b, SUC_lexicon_tags_26 },
    { 0xa120618c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89406190, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6b006192, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed606196, SUC_lexicon_tags_66 },
    { 0x35206196, SUC_lexicon_tags_29 },
    { 0x86206197, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8220619d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8720619f, SUC_lexicon_tags_49 },
    { 0x9480619f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce061a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd58061a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x654061a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56061b6, SUC_lexicon_tags_12 },
    { 0x9e6061b7, SUC_lexicon_tags_193 },
    { 0xce4061b8, SUC_lexicon_tags_11 },
    { 0x610061b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c0061bb, SUC_lexicon_tags_1310 },
    { 0xfd6061bc, SUC_lexicon_tags_25 },
    { 0x790061bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2061c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08061c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244061cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf84061ce, SUC_lexicon_tags_17 },
    { 0xe68061cf, SUC_lexicon_tags_11 },
    { 0x82a061d0, SUC_lexicon_tags_11 },
    { 0xe84061d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3c4061d3, SUC_lexicon_tags_2 },
    { 0xe46061d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe66061d6, SUC_lexicon_tags_1 },
    { 0x61e061d6, SUC_lexicon_tags_2 },
    { 0xf46061d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7a061da, SUC_lexicon_tags_29 },
    { 0x6fa061da, SUC_lexicon_tags_17 },
    { 0x608061dc, SUC_lexicon_tags_0 },
    { 0x7a4061dc, SUC_lexicon_tags_23 },
    { 0x980061de, SUC_lexicon_tags_45 },
    { 0x8d0061dc, SUC_lexicon_tags_37 },
    { 0x7d6061e0, SUC_lexicon_tags_15 },
    { 0x206061e1, SUC_lexicon_tags_21 },
    { 0x14061e1, SUC_lexicon_tags_27 },
    { 0x210061e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94a061e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde8061e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x764061e9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8b6061eb, SUC_lexicon_tags_17 },
    { 0xa28061ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e061f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27c061f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x836061f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c061fa, SUC_lexicon_tags_7 },
    { 0xcca061fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6006202, SUC_lexicon_tags_22 },
    { 0x80606202, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e06206, SUC_lexicon_tags_29 },
    { 0x24806207, SUC_lexicon_tags_17 },
    { 0xd7a06208, SUC_lexicon_tags_17 },
    { 0xdf406207, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9020620e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc06211, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82206213, SUC_lexicon_tags_7 },
    { 0xfda06213, SUC_lexicon_tags_63 },
    { 0x40006215, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5020621a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a06220, SUC_lexicon_tags_416 },
    { 0, NULL },
    { 0xd8806222, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1006225, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf040622d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1206230, SUC_lexicon_tags_15 },
    { 0xabe06231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e606233, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b406235, SUC_lexicon_tags_63 },
    { 0x78e06236, SUC_lexicon_tags_70 },
    { 0xe6c06235, SUC_lexicon_tags_233 },
    { 0xc1006236, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a40623c, SUC_lexicon_tags_2 },
    { 0xdf00623d, SUC_lexicon_tags_11 },
    { 0xf820623e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9e806240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e06243, SUC_lexicon_tags_7 },
    { 0x9f606243, SUC_lexicon_tags_19 },
    { 0x5da06245, SUC_lexicon_tags_11 },
    { 0xac206244, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0xb2206248, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf080624e, SUC_lexicon_tags_37 },
    { 0xf280624f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e406254, SUC_lexicon_tags_25 },
    { 0x30206255, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21006257, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d806259, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9a40625b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e80625e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e06263, SUC_lexicon_tags_630 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa006268, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8b20626a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1800626c, SUC_lexicon_tags_27 },
    { 0x7960626c, SUC_lexicon_tags_7 },
    { 0xdf80626e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e006275, SUC_lexicon_tags_60 },
    { 0xb3606276, SUC_lexicon_tags_17 },
    { 0x33e06277, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1000627a, SUC_lexicon_tags_37 },
    { 0xd520627a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c06284, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4ea06286, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xae006288, SUC_lexicon_tags_25 },
    { 0x17c06289, SUC_lexicon_tags_31 },
    { 0xbba0628a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e06290, SUC_lexicon_tags_33 },
    { 0xc7806290, SUC_lexicon_tags_37 },
    { 0x39006290, SUC_lexicon_tags_2 },
    { 0xaea06293, SUC_lexicon_tags_7 },
    { 0x3a006294, SUC_lexicon_tags_26 },
    { 0x9b206295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c0629a, SUC_lexicon_tags_11 },
    { 0x66c0629b, SUC_lexicon_tags_38 },
    { 0xbca0629c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb740629f, SUC_lexicon_tags_38 },
    { 0x3ac062a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88062a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c062a7, SUC_lexicon_tags_33 },
    { 0x986062a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8062ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3c062ae, SUC_lexicon_tags_60 },
    { 0x820062ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x488062b1, SUC_lexicon_tags_25 },
    { 0xcf0062b2, SUC_lexicon_tags_19 },
    { 0x5d0062b3, SUC_lexicon_tags_14 },
    { 0x32062b4, SUC_lexicon_tags_11 },
    { 0x490062b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x964062b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e062bf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xac4062c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6062c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e0062c8, SUC_lexicon_tags_49 },
    { 0x53a062c9, SUC_lexicon_tags_0 },
    { 0x31c062ca, SUC_lexicon_tags_17 },
    { 0x62cb, SUC_lexicon_tags_25 },
    { 0x49c062cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a062d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834062d5, SUC_lexicon_tags_2 },
    { 0xa4c062d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8062dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a062e0, SUC_lexicon_tags_34 },
    { 0xd40062e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x752062e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558062e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60062eb, SUC_lexicon_tags_47 },
    { 0xe2c062ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2062f2, SUC_lexicon_tags_33 },
    { 0xaa4062f3, SUC_lexicon_tags_15 },
    { 0xd4062f3, SUC_lexicon_tags_38 },
    { 0xdcc062f2, SUC_lexicon_tags_1 },
    { 0xdba062f3, SUC_lexicon_tags_35 },
    { 0x300062f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24062fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c006300, SUC_lexicon_tags_31 },
    { 0xa2606301, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe06309, SUC_lexicon_tags_25 },
    { 0x8d606309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a00630d, SUC_lexicon_tags_27 },
    { 0x7200630e, SUC_lexicon_tags_1 },
    { 0xfda0630e, SUC_lexicon_tags_19 },
    { 0xe5c06310, SUC_lexicon_tags_49 },
    { 0x89a06310, SUC_lexicon_tags_32 },
    { 0x27c06312, SUC_lexicon_tags_17 },
    { 0x7f806313, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2006317, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe0631b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540631f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb406321, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84406326, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b80632a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22006335, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x50006337, SUC_lexicon_tags_49 },
    { 0x14806337, SUC_lexicon_tags_10 },
    { 0xeb006338, SUC_lexicon_tags_55 },
    { 0xbbc0633a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc0633d, SUC_lexicon_tags_38 },
    { 0x1f80633d, SUC_lexicon_tags_37 },
    { 0xa9c0633f, SUC_lexicon_tags_82 },
    { 0x28006340, SUC_lexicon_tags_60 },
    { 0xf3e06341, SUC_lexicon_tags_33 },
    { 0x95406342, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde206344, SUC_lexicon_tags_1 },
    { 0x6b406344, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x39806347, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a0634b, SUC_lexicon_tags_17 },
    { 0xc500634c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7380634e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f606353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8006355, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00635c, SUC_lexicon_tags_37 },
    { 0x6000635c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54e0635f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0e06361, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a06366, SUC_lexicon_tags_25 },
    { 0xe5606366, SUC_lexicon_tags_25 },
    { 0xee806367, SUC_lexicon_tags_47 },
    { 0xac806366, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c0636c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x3a60636e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc06371, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15206378, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e0637c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc06386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47606388, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47606390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70206396, SUC_lexicon_tags_1 },
    { 0xafe06396, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf860639e, SUC_lexicon_tags_25 },
    { 0x44a0639f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4063a5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa82063a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x596063a9, SUC_lexicon_tags_7 },
    { 0x8f6063aa, SUC_lexicon_tags_17 },
    { 0x796063ab, SUC_lexicon_tags_9 },
    { 0xb30063ac, SUC_lexicon_tags_11 },
    { 0x4fc063a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96063b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc063b6, SUC_lexicon_tags_31 },
    { 0xd9e063b7, SUC_lexicon_tags_15 },
    { 0xebc063b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d4063ba, SUC_lexicon_tags_63 },
    { 0xd4a063bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0063c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2b6063c2, SUC_lexicon_tags_7 },
    { 0xdfe063c2, SUC_lexicon_tags_14 },
    { 0xecc063c4, SUC_lexicon_tags_8 },
    { 0x438063c5, SUC_lexicon_tags_47 },
    { 0xc44063c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75a063c8, SUC_lexicon_tags_45 },
    { 0x430063c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70063d4, SUC_lexicon_tags_15 },
    { 0x468063d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da063da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524063de, SUC_lexicon_tags_46 },
    { 0x814063df, SUC_lexicon_tags_39 },
    { 0x80e063df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2063e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c063ea, SUC_lexicon_tags_11 },
    { 0x7d8063ea, SUC_lexicon_tags_23 },
    { 0x806063ec, SUC_lexicon_tags_11 },
    { 0xb9a063ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe063f5, SUC_lexicon_tags_194 },
    { 0xcc0063f5, SUC_lexicon_tags_6 },
    { 0x7f0063f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa063fb, SUC_lexicon_tags_7 },
    { 0x118063fc, SUC_lexicon_tags_40 },
    { 0x27a063fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5606400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e06405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75406408, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd60640f, SUC_lexicon_tags_17 },
    { 0xdfa06410, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x25806412, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb006416, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46c06418, SUC_lexicon_tags_54 },
    { 0x47606418, SUC_lexicon_tags_2 },
    { 0xe140641a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a40641c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6b20641e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x62406420, SUC_lexicon_tags_11 },
    { 0xd7206420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42006426, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de06429, SUC_lexicon_tags_21 },
    { 0x1a80642a, SUC_lexicon_tags_15 },
    { 0xbe60642b, SUC_lexicon_tags_2 },
    { 0x9d20642c, SUC_lexicon_tags_6 },
    { 0x3440642d, SUC_lexicon_tags_29 },
    { 0x7b40642c, SUC_lexicon_tags_1 },
    { 0xa280642c, SUC_lexicon_tags_11 },
    { 0xabc06430, SUC_lexicon_tags_179 },
    { 0x89a06430, SUC_lexicon_tags_17 },
    { 0xe7206432, SUC_lexicon_tags_9 },
    { 0x90406432, SUC_lexicon_tags_47 },
    { 0x5900642e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c406437, SUC_lexicon_tags_17 },
    { 0x97006438, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba0643b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa06443, SUC_lexicon_tags_61 },
    { 0xa0806443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb606447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7300644e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x4ac06450, SUC_lexicon_tags_48 },
    { 0xaf806451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b406455, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd3206457, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7740645f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb206462, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc06468, SUC_lexicon_tags_15 },
    { 0x49a06468, SUC_lexicon_tags_7 },
    { 0xe5206469, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8820646f, SUC_lexicon_tags_47 },
    { 0x33606470, SUC_lexicon_tags_74 },
    { 0x24c06471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2e06473, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65c06475, SUC_lexicon_tags_43 },
    { 0x8c006476, SUC_lexicon_tags_21 },
    { 0x1f806477, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3860647a, SUC_lexicon_tags_7 },
    { 0x92e0647b, SUC_lexicon_tags_11 },
    { 0x31e0647a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde606480, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaec06482, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x39606484, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6120648e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c06491, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e06498, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb400649c, SUC_lexicon_tags_37 },
    { 0x1ac0649d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc064a0, SUC_lexicon_tags_25 },
    { 0x8ba064a1, SUC_lexicon_tags_63 },
    { 0x114064a2, SUC_lexicon_tags_56 },
    { 0x33e064a2, SUC_lexicon_tags_60 },
    { 0x3b6064a4, SUC_lexicon_tags_7 },
    { 0x82064a0, SUC_lexicon_tags_15 },
    { 0x83a064a6, SUC_lexicon_tags_16 },
    { 0x648064a7, SUC_lexicon_tags_7 },
    { 0x73a064a1, SUC_lexicon_tags_19 },
    { 0x3b8064a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530064ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c064b0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c064bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406064c4, SUC_lexicon_tags_49 },
    { 0xf16064c4, SUC_lexicon_tags_27 },
    { 0x9f6064c4, SUC_lexicon_tags_45 },
    { 0xab8064c7, SUC_lexicon_tags_10 },
    { 0x1c064c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc064ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c064d2, SUC_lexicon_tags_66 },
    { 0x42a064d3, SUC_lexicon_tags_11 },
    { 0x98e064d4, SUC_lexicon_tags_15 },
    { 0xda064d5, SUC_lexicon_tags_14 },
    { 0x86e064d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac064d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f2064db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6064de, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866064e2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2064e9, SUC_lexicon_tags_27 },
    { 0xade064ea, SUC_lexicon_tags_45 },
    { 0xda0064eb, SUC_lexicon_tags_13 },
    { 0x2bc064ec, SUC_lexicon_tags_37 },
    { 0x2d8064eb, SUC_lexicon_tags_17 },
    { 0x434064ee, SUC_lexicon_tags_15 },
    { 0xea064ee, SUC_lexicon_tags_23 },
    { 0x902064f0, SUC_lexicon_tags_17 },
    { 0xad6064f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14064f4, SUC_lexicon_tags_28 },
    { 0x264064f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a064f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c064fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d8064ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1406508, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd520650d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e06511, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c06514, SUC_lexicon_tags_45 },
    { 0x16006515, SUC_lexicon_tags_351 },
    { 0x2ea06515, SUC_lexicon_tags_7 },
    { 0xd9e06517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e206519, SUC_lexicon_tags_15 },
    { 0xb8806519, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5280651e, SUC_lexicon_tags_2 },
    { 0x4760651f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa06523, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de06526, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x48006528, SUC_lexicon_tags_2 },
    { 0xbde06529, SUC_lexicon_tags_49 },
    { 0xdbe0652a, SUC_lexicon_tags_12 },
    { 0x8680652b, SUC_lexicon_tags_6 },
    { 0x2b00652c, SUC_lexicon_tags_47 },
    { 0x9700652d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c06530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34206534, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb806536, SUC_lexicon_tags_23 },
    { 0x66406537, SUC_lexicon_tags_17 },
    { 0x56c06538, SUC_lexicon_tags_21 },
    { 0xfa406539, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3260653b, SUC_lexicon_tags_45 },
    { 0xab20653c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea06541, SUC_lexicon_tags_29 },
    { 0x81806542, SUC_lexicon_tags_26 },
    { 0x8a606541, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1c06545, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c06551, SUC_lexicon_tags_30 },
    { 0xa1006551, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae06555, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0a06557, SUC_lexicon_tags_25 },
    { 0x80e06558, SUC_lexicon_tags_2 },
    { 0x2d606559, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8800655b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcba0655d, SUC_lexicon_tags_0 },
    { 0x5e0655e, SUC_lexicon_tags_1 },
    { 0x7180655f, SUC_lexicon_tags_80 },
    { 0xd7006560, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c06564, SUC_lexicon_tags_29 },
    { 0x38606565, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe4a06567, SUC_lexicon_tags_3 },
    { 0x84c06568, SUC_lexicon_tags_1 },
    { 0x7ee06568, SUC_lexicon_tags_39 },
    { 0xad806568, SUC_lexicon_tags_77 },
    { 0xf1a06568, SUC_lexicon_tags_262 },
    { 0xba606567, SUC_lexicon_tags_25 },
    { 0x6620656d, SUC_lexicon_tags_15 },
    { 0x1b60656d, SUC_lexicon_tags_0 },
    { 0x7be06568, SUC_lexicon_tags_15 },
    { 0x11406570, SUC_lexicon_tags_1 },
    { 0x8ac06570, SUC_lexicon_tags_7 },
    { 0xc0206572, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e06575, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240657b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6806581, SUC_lexicon_tags_7 },
    { 0xb0206582, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x51e06584, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e06587, SUC_lexicon_tags_37 },
    { 0x78606587, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5fe0658a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe0658c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0658f, SUC_lexicon_tags_8 },
    { 0x2206590, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10806592, SUC_lexicon_tags_8 },
    { 0xec006593, SUC_lexicon_tags_23 },
    { 0xb9206593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20806596, SUC_lexicon_tags_9 },
    { 0xadc06597, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xefc06599, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf520659c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1280659f, SUC_lexicon_tags_25 },
    { 0x7e065a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712065a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e065ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x328065af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624065b4, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a065ba, SUC_lexicon_tags_25 },
    { 0x786065ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a065be, SUC_lexicon_tags_27 },
    { 0xa24065bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d2065c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66a065c3, SUC_lexicon_tags_37 },
    { 0xea4065c4, SUC_lexicon_tags_10 },
    { 0x750065c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x704065c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72065cb, SUC_lexicon_tags_23 },
    { 0xda6065cb, SUC_lexicon_tags_11 },
    { 0x4c6065cb, SUC_lexicon_tags_2 },
    { 0xb22065ce, SUC_lexicon_tags_37 },
    { 0xc80065cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2065d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30065d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884065db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76065df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644065e3, SUC_lexicon_tags_31 },
    { 0x730065e4, SUC_lexicon_tags_37 },
    { 0x47a065e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332065f1, SUC_lexicon_tags_6 },
    { 0x9c2065f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564065f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0a065f8, SUC_lexicon_tags_37 },
    { 0xbda065f9, SUC_lexicon_tags_12 },
    { 0xc26065f9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6065fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c06601, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70206607, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9180660c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x9520660f, SUC_lexicon_tags_27 },
    { 0xb0406610, SUC_lexicon_tags_24 },
    { 0xf040660f, SUC_lexicon_tags_33 },
    { 0x42606612, SUC_lexicon_tags_37 },
    { 0x6b606611, SUC_lexicon_tags_15 },
    { 0x6ec06612, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf8206616, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2806618, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8100661e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0806624, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f006629, SUC_lexicon_tags_7 },
    { 0xac60662a, SUC_lexicon_tags_3 },
    { 0x10406629, SUC_lexicon_tags_17 },
    { 0xe620662c, SUC_lexicon_tags_31 },
    { 0xaf40662c, SUC_lexicon_tags_43 },
    { 0x8240662e, SUC_lexicon_tags_11 },
    { 0xbc60662e, SUC_lexicon_tags_1217 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea06632, SUC_lexicon_tags_7 },
    { 0x4e406632, SUC_lexicon_tags_141 },
    { 0x2cc06634, SUC_lexicon_tags_65 },
    { 0x88a06635, SUC_lexicon_tags_31 },
    { 0x5cc06633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc06639, SUC_lexicon_tags_37 },
    { 0xa1c0663a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea0663e, SUC_lexicon_tags_8 },
    { 0x960663e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca06643, SUC_lexicon_tags_11 },
    { 0x21a06643, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa0664b, SUC_lexicon_tags_11 },
    { 0x3240664c, SUC_lexicon_tags_0 },
    { 0xf3c0664c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c00664f, SUC_lexicon_tags_48 },
    { 0x5fa06650, SUC_lexicon_tags_56 },
    { 0x49a06651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33206654, SUC_lexicon_tags_25 },
    { 0x8ac06655, SUC_lexicon_tags_11 },
    { 0xb0406656, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x71806659, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e0665c, SUC_lexicon_tags_17 },
    { 0xef80665d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6006661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51406664, SUC_lexicon_tags_31 },
    { 0xd5806665, SUC_lexicon_tags_87 },
    { 0x58e06664, SUC_lexicon_tags_25 },
    { 0x22c06667, SUC_lexicon_tags_2 },
    { 0x10806668, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a40666b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x500666d, SUC_lexicon_tags_26 },
    { 0xcd80666d, SUC_lexicon_tags_17 },
    { 0x6cc0666f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a06676, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x33206678, SUC_lexicon_tags_28 },
    { 0x81606679, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9560667d, SUC_lexicon_tags_38 },
    { 0xb8c0667e, SUC_lexicon_tags_29 },
    { 0x8220667e, SUC_lexicon_tags_23 },
    { 0x90006680, SUC_lexicon_tags_10 },
    { 0x3fa0667d, SUC_lexicon_tags_27 },
    { 0x4fc06682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc206687, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee0668a, SUC_lexicon_tags_25 },
    { 0xd980668b, SUC_lexicon_tags_27 },
    { 0xc0668c, SUC_lexicon_tags_7 },
    { 0xeec0668d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de0669d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326066a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecc066a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a066a7, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x568066aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a066b0, SUC_lexicon_tags_25 },
    { 0x368066b1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x868066b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x530066b6, SUC_lexicon_tags_7 },
    { 0xfa8066b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c066bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ce066bd, SUC_lexicon_tags_2 },
    { 0x348066be, SUC_lexicon_tags_25 },
    { 0x5da066be, SUC_lexicon_tags_15 },
    { 0x3f8066bf, SUC_lexicon_tags_1 },
    { 0xc8066bd, SUC_lexicon_tags_12 },
    { 0x8dc066c2, SUC_lexicon_tags_4 },
    { 0xfda066c2, SUC_lexicon_tags_11 },
    { 0x128066c4, SUC_lexicon_tags_11 },
    { 0x45c066be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x536066c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc066cb, SUC_lexicon_tags_55 },
    { 0x556066cc, SUC_lexicon_tags_37 },
    { 0xdd6066cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d0, SUC_lexicon_tags_29 },
    { 0xd10066d0, SUC_lexicon_tags_38 },
    { 0x68066d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2066d6, SUC_lexicon_tags_37 },
    { 0xce066d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe48066d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6066dc, SUC_lexicon_tags_43 },
    { 0x56a066dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2066e1, SUC_lexicon_tags_8 },
    { 0xf06066e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6066e8, SUC_lexicon_tags_37 },
    { 0xda0066e9, SUC_lexicon_tags_17 },
    { 0x8f2066ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386066ef, SUC_lexicon_tags_37 },
    { 0x6ac066ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6f2066f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf72066f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43c066f6, SUC_lexicon_tags_83 },
    { 0xd66066f7, SUC_lexicon_tags_0 },
    { 0x5c066f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54e066fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a066ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8c06701, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2360670b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2620670d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c06712, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab806716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39206719, SUC_lexicon_tags_15 },
    { 0xd820671a, SUC_lexicon_tags_15 },
    { 0x7240671b, SUC_lexicon_tags_17 },
    { 0xa3806719, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30406722, SUC_lexicon_tags_12 },
    { 0x50e06723, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf406725, SUC_lexicon_tags_12 },
    { 0x64e06725, SUC_lexicon_tags_0 },
    { 0x39406726, SUC_lexicon_tags_49 },
    { 0x45e06725, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6320672a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a06732, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8806735, SUC_lexicon_tags_15 },
    { 0xe9e06736, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fc06738, SUC_lexicon_tags_12 },
    { 0xb0c06739, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a0673e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c06744, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8006747, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc0674b, SUC_lexicon_tags_57 },
    { 0xe9a0674c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3b20674e, SUC_lexicon_tags_11 },
    { 0x35e0674f, SUC_lexicon_tags_15 },
    { 0x5e006750, SUC_lexicon_tags_87 },
    { 0xb400674e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x75206753, SUC_lexicon_tags_12 },
    { 0x31c06754, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec06757, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa8206759, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10a0675b, SUC_lexicon_tags_25 },
    { 0x8940675c, SUC_lexicon_tags_11 },
    { 0xc1e0675d, SUC_lexicon_tags_4 },
    { 0xb8a0675b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f206760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3406762, SUC_lexicon_tags_167 },
    { 0x49a06763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68006767, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e00676a, SUC_lexicon_tags_34 },
    { 0x7b80676b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a0676f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa0e06771, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2206774, SUC_lexicon_tags_14 },
    { 0xe8c06775, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a0677a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f00677d, SUC_lexicon_tags_302 },
    { 0x67c0677e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x38806780, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfce06782, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99c06784, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8780678e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63806793, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb206798, SUC_lexicon_tags_37 },
    { 0x57606798, SUC_lexicon_tags_25 },
    { 0xec0679a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d80679c, SUC_lexicon_tags_19 },
    { 0x7be0679d, SUC_lexicon_tags_27 },
    { 0x4ca0679e, SUC_lexicon_tags_17 },
    { 0xf740679f, SUC_lexicon_tags_39 },
    { 0x3760679e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe4c067a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1f8067a4, SUC_lexicon_tags_2 },
    { 0x908067a5, SUC_lexicon_tags_37 },
    { 0x818067a6, SUC_lexicon_tags_2 },
    { 0x10067a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x516067a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9c067ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc18067ad, SUC_lexicon_tags_26 },
    { 0xdfa067ad, SUC_lexicon_tags_5 },
    { 0xa5c067af, SUC_lexicon_tags_14 },
    { 0x8f2067b0, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6067b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8067b7, SUC_lexicon_tags_113 },
    { 0xd62067b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8067bf, SUC_lexicon_tags_14 },
    { 0x682067c0, SUC_lexicon_tags_47 },
    { 0xe92067c1, SUC_lexicon_tags_25 },
    { 0x812067c2, SUC_lexicon_tags_7 },
    { 0x8ac067c2, SUC_lexicon_tags_43 },
    { 0xac8067c4, SUC_lexicon_tags_1 },
    { 0x460067c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x298067c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x308067cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864067d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130067d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc067db, SUC_lexicon_tags_37 },
    { 0x882067dc, SUC_lexicon_tags_6 },
    { 0x5dc067dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180067e2, SUC_lexicon_tags_7 },
    { 0xcfa067e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2067e7, SUC_lexicon_tags_25 },
    { 0x3c0067e8, SUC_lexicon_tags_43 },
    { 0x70a067e9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8dc067eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e6067ed, SUC_lexicon_tags_11 },
    { 0x4ba067ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97c067f0, SUC_lexicon_tags_66 },
    { 0xf0a067f0, SUC_lexicon_tags_21 },
    { 0xe7c067f2, SUC_lexicon_tags_11 },
    { 0x154067f3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8067f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1606802, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xab806804, SUC_lexicon_tags_1 },
    { 0xdd806805, SUC_lexicon_tags_31 },
    { 0x76406806, SUC_lexicon_tags_1475 },
    { 0xa0406804, SUC_lexicon_tags_7 },
    { 0x3d806807, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e0680c, SUC_lexicon_tags_13 },
    { 0xc240680d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79206818, SUC_lexicon_tags_55 },
    { 0x57806819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1200681b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa040681f, SUC_lexicon_tags_38 },
    { 0xb6406820, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f206829, SUC_lexicon_tags_33 },
    { 0x6d606829, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46406837, SUC_lexicon_tags_31 },
    { 0xa8806837, SUC_lexicon_tags_2 },
    { 0xeaa06838, SUC_lexicon_tags_0 },
    { 0x5fa0683a, SUC_lexicon_tags_25 },
    { 0x6f80683a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6320683d, SUC_lexicon_tags_74 },
    { 0x940683e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66206844, SUC_lexicon_tags_27 },
    { 0xbb206845, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a06848, SUC_lexicon_tags_37 },
    { 0xdfc06849, SUC_lexicon_tags_21 },
    { 0x4fa0684a, SUC_lexicon_tags_9 },
    { 0xf680684b, SUC_lexicon_tags_33 },
    { 0xf0c0684c, SUC_lexicon_tags_14 },
    { 0x840684c, SUC_lexicon_tags_25 },
    { 0xe5a0684d, SUC_lexicon_tags_19 },
    { 0xce20684f, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de06852, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc806856, SUC_lexicon_tags_25 },
    { 0x70c06857, SUC_lexicon_tags_95 },
    { 0x52206857, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xbac0685a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e0685d, SUC_lexicon_tags_86 },
    { 0xe580685e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d206866, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0686a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3c0686c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e20686f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbba06871, SUC_lexicon_tags_15 },
    { 0x53c06871, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae006878, SUC_lexicon_tags_8 },
    { 0x32a06878, SUC_lexicon_tags_25 },
    { 0x1bc0687a, SUC_lexicon_tags_1 },
    { 0x600687b, SUC_lexicon_tags_17 },
    { 0xdc40687a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x180687e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3406883, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x59406885, SUC_lexicon_tags_27 },
    { 0xde406886, SUC_lexicon_tags_7 },
    { 0xa806886, SUC_lexicon_tags_7 },
    { 0x63e06888, SUC_lexicon_tags_11 },
    { 0xcf006888, SUC_lexicon_tags_34 },
    { 0x1c00688a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb9c0688c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c06890, SUC_lexicon_tags_2 },
    { 0x7dc06890, SUC_lexicon_tags_28 },
    { 0x80c06891, SUC_lexicon_tags_23 },
    { 0x43806890, SUC_lexicon_tags_30 },
    { 0xeac06892, SUC_lexicon_tags_27 },
    { 0xb3e06892, SUC_lexicon_tags_11 },
    { 0x5e06893, SUC_lexicon_tags_1 },
    { 0x48406897, SUC_lexicon_tags_9 },
    { 0x40806897, SUC_lexicon_tags_2 },
    { 0x22e06894, SUC_lexicon_tags_37 },
    { 0x8920689a, SUC_lexicon_tags_25 },
    { 0x5540689a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96c0689d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10068a1, SUC_lexicon_tags_37 },
    { 0x7a068a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e068a6, SUC_lexicon_tags_100 },
    { 0x4fa068a6, SUC_lexicon_tags_0 },
    { 0x77c068a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a068ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c068b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c068b6, SUC_lexicon_tags_26 },
    { 0x86e068b7, SUC_lexicon_tags_27 },
    { 0x78068b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e4068ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08068c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a068c3, SUC_lexicon_tags_1 },
    { 0xb24068c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa28068c7, SUC_lexicon_tags_12 },
    { 0x896068c8, SUC_lexicon_tags_26 },
    { 0xa56068c9, SUC_lexicon_tags_19 },
    { 0x4b2068ca, SUC_lexicon_tags_21 },
    { 0x698068c7, SUC_lexicon_tags_17 },
    { 0xd34068cc, SUC_lexicon_tags_7 },
    { 0xeb8068cd, SUC_lexicon_tags_10 },
    { 0xae2068cc, SUC_lexicon_tags_34 },
    { 0x3b2068ce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42068d3, SUC_lexicon_tags_12 },
    { 0xec0068d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa70068d6, SUC_lexicon_tags_60 },
    { 0x522068d7, SUC_lexicon_tags_11 },
    { 0x18c068d7, SUC_lexicon_tags_12 },
    { 0xac8068d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc068de, SUC_lexicon_tags_38 },
    { 0xa70068df, SUC_lexicon_tags_12 },
    { 0x490068df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x548068e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb34068e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f6068e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cc068e9, SUC_lexicon_tags_37 },
    { 0x79a068ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc96068ec, SUC_lexicon_tags_37 },
    { 0x84e068ec, SUC_lexicon_tags_47 },
    { 0x146068ed, SUC_lexicon_tags_17 },
    { 0x3a068ef, SUC_lexicon_tags_15 },
    { 0xb22068ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ec068f2, SUC_lexicon_tags_45 },
    { 0xfec068f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a6068f5, SUC_lexicon_tags_38 },
    { 0x4fa068f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee068f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e206902, SUC_lexicon_tags_29 },
    { 0xb7006903, SUC_lexicon_tags_12 },
    { 0xa0006904, SUC_lexicon_tags_15 },
    { 0x6ee06905, SUC_lexicon_tags_68 },
    { 0xf7206905, SUC_lexicon_tags_2 },
    { 0x34c06904, SUC_lexicon_tags_7 },
    { 0xbbe06908, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5800690c, SUC_lexicon_tags_9 },
    { 0x6f00690d, SUC_lexicon_tags_25 },
    { 0xef40690e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x60006910, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79406914, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a06919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7280691b, SUC_lexicon_tags_61 },
    { 0x1c40691c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7806923, SUC_lexicon_tags_43 },
    { 0x6ec06924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc06927, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c20692b, SUC_lexicon_tags_47 },
    { 0x6dc0692c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3780692f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2aa06931, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0606933, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf5806935, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcda06937, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67206939, SUC_lexicon_tags_6 },
    { 0x99c0693a, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c0693e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e606943, SUC_lexicon_tags_11 },
    { 0x26a06943, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae06948, SUC_lexicon_tags_12 },
    { 0xe206949, SUC_lexicon_tags_45 },
    { 0xf440694a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c0694e, SUC_lexicon_tags_11 },
    { 0x7760694f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be06954, SUC_lexicon_tags_77 },
    { 0x49e06954, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e06961, SUC_lexicon_tags_2 },
    { 0xf0006962, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xef006964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a06969, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e0696d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b40696f, SUC_lexicon_tags_33 },
    { 0xd1606970, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e06978, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5c0697a, SUC_lexicon_tags_7 },
    { 0x8e20697b, SUC_lexicon_tags_14 },
    { 0x2080697c, SUC_lexicon_tags_6 },
    { 0xff00697b, SUC_lexicon_tags_15 },
    { 0xe580697e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22606982, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8fc06984, SUC_lexicon_tags_38 },
    { 0x36806985, SUC_lexicon_tags_17 },
    { 0x33006986, SUC_lexicon_tags_27 },
    { 0x12e06987, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7180698c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a06991, SUC_lexicon_tags_17 },
    { 0x59606992, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f606994, SUC_lexicon_tags_13 },
    { 0x8d606995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ca06997, SUC_lexicon_tags_37 },
    { 0x9a806997, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca0699c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba0699f, SUC_lexicon_tags_12 },
    { 0x9ae069a0, SUC_lexicon_tags_87 },
    { 0x650069a1, SUC_lexicon_tags_25 },
    { 0xd900699f, SUC_lexicon_tags_56 },
    { 0xcd0069a1, SUC_lexicon_tags_14 },
    { 0xd6c069a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf8a069a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652069ac, SUC_lexicon_tags_1 },
    { 0xe84069ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xea2069af, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500069b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a069bd, SUC_lexicon_tags_26 },
    { 0x91a069bd, SUC_lexicon_tags_17 },
    { 0x674069bf, SUC_lexicon_tags_47 },
    { 0xa56069bf, SUC_lexicon_tags_64 },
    { 0x5e0069c1, SUC_lexicon_tags_63 },
    { 0x842069bd, SUC_lexicon_tags_2 },
    { 0x3ce069c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2ca069c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x398069c8, SUC_lexicon_tags_43 },
    { 0x486069c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a6069cb, SUC_lexicon_tags_15 },
    { 0xc3a069cc, SUC_lexicon_tags_15 },
    { 0x792069cc, SUC_lexicon_tags_47 },
    { 0x5be069ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7e069d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac069d5, SUC_lexicon_tags_11 },
    { 0xb16069d6, SUC_lexicon_tags_37 },
    { 0xf74069d7, SUC_lexicon_tags_7 },
    { 0xf10069d8, SUC_lexicon_tags_37 },
    { 0x6d0069d9, SUC_lexicon_tags_21 },
    { 0xf82069da, SUC_lexicon_tags_37 },
    { 0x4d8069db, SUC_lexicon_tags_9 },
    { 0x1ca069dc, SUC_lexicon_tags_25 },
    { 0xb90069db, SUC_lexicon_tags_37 },
    { 0xbbe069dd, SUC_lexicon_tags_4 },
    { 0x290069d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc26069e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa54069e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e069e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4069f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a8069f2, SUC_lexicon_tags_1 },
    { 0x2a0069f2, SUC_lexicon_tags_23 },
    { 0x7de069f4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a069f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408069fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e06a01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7c06a03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f406a0a, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2406a0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90406a11, SUC_lexicon_tags_2 },
    { 0xb2a06a12, SUC_lexicon_tags_2 },
    { 0xeba06a12, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36206a15, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8406a17, SUC_lexicon_tags_11 },
    { 0xa6e06a17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8406a1a, SUC_lexicon_tags_7 },
    { 0xe8806a1a, SUC_lexicon_tags_11 },
    { 0x78406a1c, SUC_lexicon_tags_17 },
    { 0x9e06a1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72006a1f, SUC_lexicon_tags_1039 },
    { 0xb2e06a20, SUC_lexicon_tags_1201 },
    { 0xd6e06a20, SUC_lexicon_tags_17 },
    { 0xe806a22, SUC_lexicon_tags_17 },
    { 0xfb206a23, SUC_lexicon_tags_17 },
    { 0x4b006a24, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2006a26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde006a2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae06a31, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6006a38, SUC_lexicon_tags_133 },
    { 0x89a06a39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1606a3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4a06a40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be06a43, SUC_lexicon_tags_49 },
    { 0x4b206a44, SUC_lexicon_tags_15 },
    { 0xab606a44, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1406a48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa06a54, SUC_lexicon_tags_11 },
    { 0xcb406a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba06a59, SUC_lexicon_tags_25 },
    { 0xb8606a59, SUC_lexicon_tags_15 },
    { 0xa606a5a, SUC_lexicon_tags_9 },
    { 0x3de06a5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8406a60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06a63, SUC_lexicon_tags_31 },
    { 0xd1606a63, SUC_lexicon_tags_163 },
    { 0xa806a65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35006a68, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa06a6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18406a6e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6ee06a70, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f206a74, SUC_lexicon_tags_31 },
    { 0x4406a75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a206a78, SUC_lexicon_tags_17 },
    { 0x5da06a78, SUC_lexicon_tags_12 },
    { 0x3a06a78, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe4406a7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a06a7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57006a84, SUC_lexicon_tags_17 },
    { 0x90c06a85, SUC_lexicon_tags_13 },
    { 0x2f806a84, SUC_lexicon_tags_45 },
    { 0x80406a85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54406a8b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a206a8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a06a92, SUC_lexicon_tags_12 },
    { 0xc6006a93, SUC_lexicon_tags_19 },
    { 0x8e806a94, SUC_lexicon_tags_60 },
    { 0x83606a95, SUC_lexicon_tags_25 },
    { 0x89c06a95, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c06a9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b406aa0, SUC_lexicon_tags_11 },
    { 0x2f406aa0, SUC_lexicon_tags_47 },
    { 0xa8006aa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad406aa5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae06aa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c06aae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91406ab3, SUC_lexicon_tags_31 },
    { 0x48406ab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83806ab6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1806abb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e06ac0, SUC_lexicon_tags_61 },
    { 0xf1a06ac1, SUC_lexicon_tags_16 },
    { 0x40806ac2, SUC_lexicon_tags_31 },
    { 0xb0606ac3, SUC_lexicon_tags_11 },
    { 0x47206ac4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e06ac7, SUC_lexicon_tags_11 },
    { 0xc5006ac7, SUC_lexicon_tags_15 },
    { 0x26006ac9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdce06acb, SUC_lexicon_tags_55 },
    { 0xe9a06acc, SUC_lexicon_tags_14 },
    { 0x12a06acd, SUC_lexicon_tags_29 },
    { 0x11e06acd, SUC_lexicon_tags_47 },
    { 0xb9e06acf, SUC_lexicon_tags_37 },
    { 0x50a06acf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50206ad3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e06adc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa06ae1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48e06ae3, SUC_lexicon_tags_12 },
    { 0xf2406ae4, SUC_lexicon_tags_11 },
    { 0x8ac06ae4, SUC_lexicon_tags_12 },
    { 0xeda06ae5, SUC_lexicon_tags_15 },
    { 0x39806ae7, SUC_lexicon_tags_11 },
    { 0x29606ae4, SUC_lexicon_tags_33 },
    { 0x12006ae3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3206aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca06aef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50806af1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb406af4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca06afa, SUC_lexicon_tags_9 },
    { 0xa6e06afb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ee06afd, SUC_lexicon_tags_10 },
    { 0xb4806afe, SUC_lexicon_tags_87 },
    { 0x53206afe, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a06b05, SUC_lexicon_tags_11 },
    { 0x59006b06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x88806b09, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaa806b0b, SUC_lexicon_tags_7 },
    { 0x6d006b0c, SUC_lexicon_tags_15 },
    { 0x47606b0c, SUC_lexicon_tags_30 },
    { 0x8ce06b0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b206b12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26006b16, SUC_lexicon_tags_47 },
    { 0xcd606b16, SUC_lexicon_tags_11 },
    { 0x28206b18, SUC_lexicon_tags_19 },
    { 0xb8606b19, SUC_lexicon_tags_65 },
    { 0x39206b18, SUC_lexicon_tags_47 },
    { 0xdb606b1b, SUC_lexicon_tags_65 },
    { 0x78006b17, SUC_lexicon_tags_38 },
    { 0x67406b1d, SUC_lexicon_tags_26 },
    { 0xd9a06b1e, SUC_lexicon_tags_79 },
    { 0xb6206b17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc06b25, SUC_lexicon_tags_2 },
    { 0x43a06b26, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f606b2a, SUC_lexicon_tags_63 },
    { 0x34406b2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e406b2e, SUC_lexicon_tags_2 },
    { 0xcd206b2f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a06b33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc406b39, SUC_lexicon_tags_60 },
    { 0xdc006b3a, SUC_lexicon_tags_3 },
    { 0xe8e06b3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb606b40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4806b45, SUC_lexicon_tags_65 },
    { 0x91e06b45, SUC_lexicon_tags_30 },
    { 0x9d206b47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde06b4b, SUC_lexicon_tags_37 },
    { 0xc8e06b4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28606b51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27406b53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49806b58, SUC_lexicon_tags_7 },
    { 0xd7606b58, SUC_lexicon_tags_1 },
    { 0x3d806b5a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb1c06b5c, SUC_lexicon_tags_30 },
    { 0xc9406b5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2b006b5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0206b63, SUC_lexicon_tags_88 },
    { 0xacc06b64, SUC_lexicon_tags_9 },
    { 0x35406b64, SUC_lexicon_tags_60 },
    { 0x46006b66, SUC_lexicon_tags_12 },
    { 0x3c206b67, SUC_lexicon_tags_0 },
    { 0x3ca06b68, SUC_lexicon_tags_48 },
    { 0x51806b68, SUC_lexicon_tags_2 },
    { 0x2fa06b68, SUC_lexicon_tags_154 },
    { 0x7ae06b6b, SUC_lexicon_tags_19 },
    { 0x51c06b6c, SUC_lexicon_tags_63 },
    { 0x50006b6d, SUC_lexicon_tags_37 },
    { 0x10806b6d, SUC_lexicon_tags_15 },
    { 0x3e206b6f, SUC_lexicon_tags_35 },
    { 0x92e06b70, SUC_lexicon_tags_37 },
    { 0x2c406b70, SUC_lexicon_tags_2 },
    { 0x7fe06b71, SUC_lexicon_tags_11 },
    { 0xcbc06b72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa06b78, SUC_lexicon_tags_9 },
    { 0x2d406b78, SUC_lexicon_tags_25 },
    { 0x79806b7a, SUC_lexicon_tags_38 },
    { 0x1ca06b7b, SUC_lexicon_tags_17 },
    { 0x1206b7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x38006b7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4e06b80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x87206b82, SUC_lexicon_tags_17 },
    { 0x25a06b82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99e06b85, SUC_lexicon_tags_23 },
    { 0xe7a06b86, SUC_lexicon_tags_0 },
    { 0x84006b87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b406b89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e06b8d, SUC_lexicon_tags_11 },
    { 0x92806b8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18e06b90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9606b92, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0xfb406b94, SUC_lexicon_tags_37 },
    { 0xf2206b95, SUC_lexicon_tags_11 },
    { 0x6b206b95, SUC_lexicon_tags_48 },
    { 0x5406b96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35406b9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9406b9d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd006ba0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97606ba4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75606bab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1806baf, SUC_lexicon_tags_9 },
    { 0x5b806bb0, SUC_lexicon_tags_29 },
    { 0xbbc06bb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x42806bb3, SUC_lexicon_tags_66 },
    { 0x1a406bb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac06bb8, SUC_lexicon_tags_45 },
    { 0xdf806bb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae06bbd, SUC_lexicon_tags_43 },
    { 0xc9806bbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x15806bc1, SUC_lexicon_tags_45 },
    { 0xb1c06bc2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x82406bc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc806bc6, SUC_lexicon_tags_25 },
    { 0xc3c06bc7, SUC_lexicon_tags_47 },
    { 0x3de06bc7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd606bcb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe9c06bcd, SUC_lexicon_tags_1 },
    { 0xdcc06bce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5206bd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xde06bd6, SUC_lexicon_tags_63 },
    { 0x4ba06bd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51606bda, SUC_lexicon_tags_25 },
    { 0xdea06bda, SUC_lexicon_tags_17 },
    { 0xd8606bdc, SUC_lexicon_tags_15 },
    { 0x68006bdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c206be0, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec06be4, SUC_lexicon_tags_14 },
    { 0xcb606be4, SUC_lexicon_tags_38 },
    { 0xb4606be6, SUC_lexicon_tags_34 },
    { 0x13406be6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c06beb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76806bf3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8d006bf5, SUC_lexicon_tags_7 },
    { 0x89206bf6, SUC_lexicon_tags_37 },
    { 0x33a06bf7, SUC_lexicon_tags_31 },
    { 0x51e06bf7, SUC_lexicon_tags_11 },
    { 0xa3806bf9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x40806bfb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49406c00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xebc06c02, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c06c06, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4a806c08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c06c0d, SUC_lexicon_tags_37 },
    { 0xb8c06c0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f606c11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50406c15, SUC_lexicon_tags_13 },
    { 0xfb406c16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8806c19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f606c1e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70606c23, SUC_lexicon_tags_82 },
    { 0x54006c24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8006c27, SUC_lexicon_tags_19 },
    { 0x16606c28, SUC_lexicon_tags_160 },
    { 0xf3c06c28, SUC_lexicon_tags_25 },
    { 0xde006c2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c06c2d, SUC_lexicon_tags_2 },
    { 0xb0206c2e, SUC_lexicon_tags_27 },
    { 0x27e06c2f, SUC_lexicon_tags_16 },
    { 0x87406c30, SUC_lexicon_tags_30 },
    { 0x13606c2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82806c34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40606c36, SUC_lexicon_tags_17 },
    { 0xad406c36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c606c3b, SUC_lexicon_tags_11 },
    { 0x9c006c3c, SUC_lexicon_tags_15 },
    { 0x1c006c3b, SUC_lexicon_tags_45 },
    { 0x73806c3b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x48c06c40, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9c06c42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe06c47, SUC_lexicon_tags_37 },
    { 0x70606c48, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0x92a06c4a, SUC_lexicon_tags_7 },
    { 0xff406c4b, SUC_lexicon_tags_11 },
    { 0xc206c4b, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0006c51, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x47c06c53, SUC_lexicon_tags_23 },
    { 0x6f206c54, SUC_lexicon_tags_12 },
    { 0x58e06c54, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5a006c57, SUC_lexicon_tags_15 },
    { 0x97006c58, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda06c5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a06c60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51806c66, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xce606c68, SUC_lexicon_tags_6 },
    { 0x52e06c68, SUC_lexicon_tags_17 },
    { 0xbd806c6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c06c6d, SUC_lexicon_tags_17 },
    { 0x31406c6e, SUC_lexicon_tags_12 },
    { 0x20806c6d, SUC_lexicon_tags_74 },
    { 0x86a06c70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0006c74, SUC_lexicon_tags_14 },
    { 0x6a206c74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d206c77, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd5006c79, SUC_lexicon_tags_33 },
    { 0xd6e06c7a, SUC_lexicon_tags_30 },
    { 0xae406c7a, SUC_lexicon_tags_11 },
    { 0xe06c7c, SUC_lexicon_tags_45 },
    { 0x59c06c7d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59606c86, SUC_lexicon_tags_12 },
    { 0xe006c87, SUC_lexicon_tags_30 },
    { 0x17a06c88, SUC_lexicon_tags_11 },
    { 0xe7a06c89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2e06c8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa06c8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12206c92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa06c96, SUC_lexicon_tags_37 },
    { 0x13e06c97, SUC_lexicon_tags_47 },
    { 0xfd606c98, SUC_lexicon_tags_0 },
    { 0x12606c99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c206ca2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c06ca6, SUC_lexicon_tags_12 },
    { 0x86206ca7, SUC_lexicon_tags_2 },
    { 0xed806ca6, SUC_lexicon_tags_7 },
    { 0xc2806ca8, SUC_lexicon_tags_4 },
    { 0xdd406caa, SUC_lexicon_tags_60 },
    { 0x25006caa, SUC_lexicon_tags_17 },
    { 0x69a06cac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba806cb2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e06cb5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x20206cb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xccc06cb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78606cbd, SUC_lexicon_tags_12 },
    { 0xc1006cbd, SUC_lexicon_tags_19 },
    { 0x3006cbf, SUC_lexicon_tags_26 },
    { 0xc5e06cc0, SUC_lexicon_tags_27 },
    { 0x85c06cc1, SUC_lexicon_tags_37 },
    { 0x8b206cbf, SUC_lexicon_tags_12 },
    { 0x6c806cbf, SUC_lexicon_tags_11 },
    { 0x43406cc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5406cc7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbcc06cc9, SUC_lexicon_tags_37 },
    { 0xef206cca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80606ccc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e06ccf, SUC_lexicon_tags_26 },
    { 0xb406cd0, SUC_lexicon_tags_12 },
    { 0xf5606cd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3806cd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e06cd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c06cda, SUC_lexicon_tags_11 },
    { 0x79c06cdb, SUC_lexicon_tags_10 },
    { 0xc3c06cdb, SUC_lexicon_tags_7 },
    { 0x9a606cdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae06ce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c406ce9, SUC_lexicon_tags_1 },
    { 0x4206cea, SUC_lexicon_tags_12 },
    { 0x83206cea, SUC_lexicon_tags_29 },
    { 0xef606cec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6606cee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3606cf1, SUC_lexicon_tags_31 },
    { 0x58606cf2, SUC_lexicon_tags_6 },
    { 0xc6206cf3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a06cf6, SUC_lexicon_tags_65 },
    { 0x24806cf6, SUC_lexicon_tags_9 },
    { 0xd2806cf8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x81006cfa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba06cfe, SUC_lexicon_tags_47 },
    { 0x4006cff, SUC_lexicon_tags_79 },
    { 0x34006d00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a06d04, SUC_lexicon_tags_6 },
    { 0x65806d04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7206d07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39806d0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8006d10, SUC_lexicon_tags_19 },
    { 0xae206d11, SUC_lexicon_tags_15 },
    { 0x7cc06d11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f606d17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0e06d19, SUC_lexicon_tags_10 },
    { 0x52e06d1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a06d1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb806d25, SUC_lexicon_tags_7 },
    { 0xd606d26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47406d33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91406d3e, SUC_lexicon_tags_27 },
    { 0x8e006d3f, SUC_lexicon_tags_15 },
    { 0xf3406d40, SUC_lexicon_tags_2 },
    { 0x56a06d41, SUC_lexicon_tags_25 },
    { 0xf4406d40, SUC_lexicon_tags_17 },
    { 0x20e06d3e, SUC_lexicon_tags_9 },
    { 0xaae06d44, SUC_lexicon_tags_14 },
    { 0xc3a06d44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a06d48, SUC_lexicon_tags_25 },
    { 0x38806d48, SUC_lexicon_tags_9 },
    { 0x39e06d4a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e06d4f, SUC_lexicon_tags_12 },
    { 0x9ce06d4f, SUC_lexicon_tags_11 },
    { 0xae206d51, SUC_lexicon_tags_37 },
    { 0xda206d52, SUC_lexicon_tags_62 },
    { 0x40406d53, SUC_lexicon_tags_13 },
    { 0x98606d51, SUC_lexicon_tags_9 },
    { 0x95406d52, SUC_lexicon_tags_11 },
    { 0xfd606d52, SUC_lexicon_tags_91 },
    { 0x9e606d57, SUC_lexicon_tags_3 },
    { 0x4ee06d58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ce06d5a, SUC_lexicon_tags_31 },
    { 0x91606d5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c06d5e, SUC_lexicon_tags_46 },
    { 0xbb406d5e, SUC_lexicon_tags_12 },
    { 0x7e006d5f, SUC_lexicon_tags_37 },
    { 0x1be06d5f, SUC_lexicon_tags_2 },
    { 0xe6606d62, SUC_lexicon_tags_11 },
    { 0xe6806d63, SUC_lexicon_tags_66 },
    { 0xb8006d63, SUC_lexicon_tags_65 },
    { 0xf4e06d64, SUC_lexicon_tags_47 },
    { 0xe5406d63, SUC_lexicon_tags_7 },
    { 0xcae06d66, SUC_lexicon_tags_18 },
    { 0x4cc06d68, SUC_lexicon_tags_29 },
    { 0xd1006d64, SUC_lexicon_tags_25 },
    { 0x90806d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a06d6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63206d72, SUC_lexicon_tags_11 },
    { 0x9f606d73, SUC_lexicon_tags_361 },
    { 0x5e006d73, SUC_lexicon_tags_19 },
    { 0x4ba06d72, SUC_lexicon_tags_1 },
    { 0x69406d76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe806d78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e06d7e, SUC_lexicon_tags_7 },
    { 0xa3c06d7e, SUC_lexicon_tags_62 },
    { 0x2dc06d80, SUC_lexicon_tags_25 },
    { 0xa0e06d81, SUC_lexicon_tags_82 },
    { 0xc6206d80, SUC_lexicon_tags_77 },
    { 0xbe406d83, SUC_lexicon_tags_305 },
    { 0x76c06d84, SUC_lexicon_tags_1 },
    { 0x4b006d85, SUC_lexicon_tags_63 },
    { 0x97606d82, SUC_lexicon_tags_6 },
    { 0x7da06d87, SUC_lexicon_tags_14 },
    { 0x85a06d83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xad806d8a, SUC_lexicon_tags_63 },
    { 0x63806d8b, SUC_lexicon_tags_27 },
    { 0xf3006d8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8606d8f, SUC_lexicon_tags_29 },
    { 0xf7606d90, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78206d94, SUC_lexicon_tags_15 },
    { 0x26a06d95, SUC_lexicon_tags_15 },
    { 0x8bc06d96, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85806d98, SUC_lexicon_tags_7 },
    { 0xe1a06d99, SUC_lexicon_tags_15 },
    { 0xb3c06d9a, SUC_lexicon_tags_45 },
    { 0xe6806d9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc206d9e, SUC_lexicon_tags_47 },
    { 0xef006d9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73606da2, SUC_lexicon_tags_13 },
    { 0xf5406da3, SUC_lexicon_tags_58 },
    { 0xf9c06da3, SUC_lexicon_tags_25 },
    { 0xc6406da5, SUC_lexicon_tags_11 },
    { 0xcb006da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6206da9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a06dad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f806daf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f806db7, SUC_lexicon_tags_17 },
    { 0x5bc06db8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a06dbb, SUC_lexicon_tags_45 },
    { 0x77c06dbc, SUC_lexicon_tags_9 },
    { 0xaa06dbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86606dc0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c06dc6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a06dd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e06dd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac06ddc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3406dde, SUC_lexicon_tags_7 },
    { 0x44006dde, SUC_lexicon_tags_75 },
    { 0xa0c06dde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c06de3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6006de6, SUC_lexicon_tags_8 },
    { 0xef806de7, SUC_lexicon_tags_39 },
    { 0xb6006de7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd406df0, SUC_lexicon_tags_68 },
    { 0x9f006df1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70206df4, SUC_lexicon_tags_85 },
    { 0x34e06df5, SUC_lexicon_tags_68 },
    { 0xe7806df6, SUC_lexicon_tags_11 },
    { 0xeae06df7, SUC_lexicon_tags_25 },
    { 0x32e06df5, SUC_lexicon_tags_0 },
    { 0x92a06df7, SUC_lexicon_tags_68 },
    { 0xc6e06dfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c06dff, SUC_lexicon_tags_1 },
    { 0x7e606dff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a606e02, SUC_lexicon_tags_12 },
    { 0x86c06e02, SUC_lexicon_tags_33 },
    { 0x78a06e04, SUC_lexicon_tags_38 },
    { 0x80606e05, SUC_lexicon_tags_7 },
    { 0x60006e04, SUC_lexicon_tags_23 },
    { 0x8c206e07, SUC_lexicon_tags_25 },
    { 0xbce06e08, SUC_lexicon_tags_2 },
    { 0xba606e08, SUC_lexicon_tags_16 },
    { 0xf2806e06, SUC_lexicon_tags_12 },
    { 0x5f606e0b, SUC_lexicon_tags_23 },
    { 0x20406e0c, SUC_lexicon_tags_114 },
    { 0x3c806e0b, SUC_lexicon_tags_55 },
    { 0x37e06e02, SUC_lexicon_tags_63 },
    { 0xfa006e05, SUC_lexicon_tags_1 },
    { 0x85406e10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8a206e12, SUC_lexicon_tags_0 },
    { 0x10806e13, SUC_lexicon_tags_1 },
    { 0x42206e14, SUC_lexicon_tags_23 },
    { 0xb3406e14, SUC_lexicon_tags_65 },
    { 0xdce06e13, SUC_lexicon_tags_25 },
    { 0x4fe06e17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74806e1b, SUC_lexicon_tags_7 },
    { 0x26c06e1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12606e20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26206e25, SUC_lexicon_tags_79 },
    { 0x4d006e26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d606e32, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc1006e34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3606e3a, SUC_lexicon_tags_15 },
    { 0xdaa06e3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98a06e3d, SUC_lexicon_tags_45 },
    { 0x1e006e3e, SUC_lexicon_tags_0 },
    { 0xf2806e3d, SUC_lexicon_tags_0 },
    { 0x58606e40, SUC_lexicon_tags_27 },
    { 0x59206e3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f006e43, SUC_lexicon_tags_27 },
    { 0x7ae06e44, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0x26006e46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5206e49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6a06e4b, SUC_lexicon_tags_14 },
    { 0xa4406e4b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdec06e4f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79006e55, SUC_lexicon_tags_44 },
    { 0xe8606e56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c206e5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5606e5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48606e62, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x95206e65, SUC_lexicon_tags_15 },
    { 0xe5006e66, SUC_lexicon_tags_37 },
    { 0x89606e66, SUC_lexicon_tags_27 },
    { 0x98606e66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e06e71, SUC_lexicon_tags_19 },
    { 0xb2006e72, SUC_lexicon_tags_26 },
    { 0x1b606e73, SUC_lexicon_tags_1 },
    { 0xc3606e73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba206e79, SUC_lexicon_tags_49 },
    { 0xdde06e7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x31406e7c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78006e82, SUC_lexicon_tags_35 },
    { 0x12c06e82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23206e85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12406e8e, SUC_lexicon_tags_0 },
    { 0x73606e8e, SUC_lexicon_tags_1 },
    { 0x61606e90, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8606e96, SUC_lexicon_tags_49 },
    { 0x64606e97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb806e99, SUC_lexicon_tags_7 },
    { 0x51006e9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac06ea4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x16406ea7, SUC_lexicon_tags_23 },
    { 0xb1606ea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83806ead, SUC_lexicon_tags_43 },
    { 0xde606eae, SUC_lexicon_tags_63 },
    { 0xe3e06ead, SUC_lexicon_tags_11 },
    { 0xc1206eb0, SUC_lexicon_tags_2 },
    { 0xa0c06eb1, SUC_lexicon_tags_7 },
    { 0x2d006eb2, SUC_lexicon_tags_17 },
    { 0x53a06eb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1606eb5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1806eb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e06ebb, SUC_lexicon_tags_49 },
    { 0x26e06ebc, SUC_lexicon_tags_4 },
    { 0x8c206ebc, SUC_lexicon_tags_11 },
    { 0xf7006ebd, SUC_lexicon_tags_43 },
    { 0x83006ebf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7406ec8, SUC_lexicon_tags_7 },
    { 0xafc06ec8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xed006ecb, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0x7ea06ecd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6006ed0, SUC_lexicon_tags_7 },
    { 0x96606ed1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69206ed5, SUC_lexicon_tags_37 },
    { 0x19206ed6, SUC_lexicon_tags_58 },
    { 0x8d406ed7, SUC_lexicon_tags_2 },
    { 0x31606ed7, SUC_lexicon_tags_12 },
    { 0x74e06ed9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c06ee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c06ee9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x58806eec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa206eef, SUC_lexicon_tags_34 },
    { 0x85206ef0, SUC_lexicon_tags_7 },
    { 0xd6806ef1, SUC_lexicon_tags_32 },
    { 0x77206ef0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c06ef7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5a206ef9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8d406efb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee206efd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9006f00, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc06f04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42806f06, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa06f0b, SUC_lexicon_tags_46 },
    { 0x6b206f0c, SUC_lexicon_tags_82 },
    { 0xa6006f0d, SUC_lexicon_tags_87 },
    { 0xc4606f0e, SUC_lexicon_tags_9 },
    { 0xe7806f0f, SUC_lexicon_tags_37 },
    { 0x58e06f0b, SUC_lexicon_tags_2 },
    { 0x1f206f11, SUC_lexicon_tags_11 },
    { 0x89a06f12, SUC_lexicon_tags_17 },
    { 0xecc06f13, SUC_lexicon_tags_30 },
    { 0x72c06f14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdc06f16, SUC_lexicon_tags_11 },
    { 0x42006f17, SUC_lexicon_tags_37 },
    { 0xc7a06f17, SUC_lexicon_tags_16 },
    { 0x71a06f18, SUC_lexicon_tags_25 },
    { 0x26206f1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x11006f1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa06f20, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2206f23, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b406f26, SUC_lexicon_tags_13 },
    { 0x14e06f27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3f806f29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb806f2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea06f33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7206f37, SUC_lexicon_tags_8 },
    { 0x11206f38, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67606f42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a06f4b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9606f4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe006f53, SUC_lexicon_tags_32 },
    { 0x85006f53, SUC_lexicon_tags_10 },
    { 0x70406f55, SUC_lexicon_tags_43 },
    { 0x94606f56, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac06f60, SUC_lexicon_tags_12 },
    { 0xca206f60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59206f64, SUC_lexicon_tags_17 },
    { 0x31e06f65, SUC_lexicon_tags_37 },
    { 0xe1c06f66, SUC_lexicon_tags_27 },
    { 0x6cc06f66, SUC_lexicon_tags_12 },
    { 0xbc006f68, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x12806f6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9c06f6c, SUC_lexicon_tags_37 },
    { 0x35c06f6d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x86e06f6f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9206f72, SUC_lexicon_tags_15 },
    { 0x46206f73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8406f78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec06f7c, SUC_lexicon_tags_68 },
    { 0x70a06f7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f406f84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e06f8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5e06f8d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c06f90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x20e06f92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x15806f95, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e06f9a, SUC_lexicon_tags_11 },
    { 0x81006f9b, SUC_lexicon_tags_11 },
    { 0x12606f9c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f806f9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c006fa5, SUC_lexicon_tags_11 },
    { 0x88006fa6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb806fa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc06fac, SUC_lexicon_tags_23 },
    { 0xc3206fac, SUC_lexicon_tags_9 },
    { 0xac006fac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c606fb1, SUC_lexicon_tags_17 },
    { 0x39e06fb1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b806fb5, SUC_lexicon_tags_39 },
    { 0x44a06fb5, SUC_lexicon_tags_23 },
    { 0x1b606fb5, SUC_lexicon_tags_49 },
    { 0xa8a06fb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ce06fba, SUC_lexicon_tags_37 },
    { 0x30a06fbb, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a06fc0, SUC_lexicon_tags_38 },
    { 0xf9006fc1, SUC_lexicon_tags_68 },
    { 0xb4606fc2, SUC_lexicon_tags_87 },
    { 0x9c06fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe06fc6, SUC_lexicon_tags_17 },
    { 0x4fa06fc7, SUC_lexicon_tags_11 },
    { 0x1f806fc6, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xcc206fca, SUC_lexicon_tags_38 },
    { 0x4b206fca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x37606fce, SUC_lexicon_tags_2 },
    { 0x20406fcf, SUC_lexicon_tags_11 },
    { 0xff206fd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62006fd3, SUC_lexicon_tags_1 },
    { 0x1006fd4, SUC_lexicon_tags_29 },
    { 0xec06fd4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a06fd8, SUC_lexicon_tags_55 },
    { 0x2b606fd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d206fde, SUC_lexicon_tags_14 },
    { 0xd3606fdf, SUC_lexicon_tags_25 },
    { 0x1b806fde, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x42c06fe2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b206fe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5406fe9, SUC_lexicon_tags_1 },
    { 0x15a06fea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a606fef, SUC_lexicon_tags_7 },
    { 0x5806ff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5806ff4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b406fff, SUC_lexicon_tags_14 },
    { 0xa4006fff, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc07007, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd20700a, SUC_lexicon_tags_82 },
    { 0xa020700b, SUC_lexicon_tags_37 },
    { 0x64c0700c, SUC_lexicon_tags_17 },
    { 0xb320700b, SUC_lexicon_tags_55 },
    { 0x4280700e, SUC_lexicon_tags_14 },
    { 0x9500700c, SUC_lexicon_tags_33 },
    { 0x56807010, SUC_lexicon_tags_7 },
    { 0x780700f, SUC_lexicon_tags_61 },
    { 0xad00700b, SUC_lexicon_tags_37 },
    { 0xbc407013, SUC_lexicon_tags_15 },
    { 0x7f60700a, SUC_lexicon_tags_23 },
    { 0x45e07015, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91007017, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8020701a, SUC_lexicon_tags_2 },
    { 0xe980701b, SUC_lexicon_tags_2 },
    { 0x1300701c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa140701f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea07022, SUC_lexicon_tags_27 },
    { 0x1807023, SUC_lexicon_tags_496 },
    { 0x8f007023, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2ac07026, SUC_lexicon_tags_1 },
    { 0x7d407027, SUC_lexicon_tags_47 },
    { 0x4007026, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a0702c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c0702f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b007031, SUC_lexicon_tags_15 },
    { 0x4b807032, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2e007034, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62807037, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc700703a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a0703d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xba07040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46807043, SUC_lexicon_tags_37 },
    { 0x3207043, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa407047, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x41a07049, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace07055, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc07058, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc07062, SUC_lexicon_tags_2 },
    { 0x86407062, SUC_lexicon_tags_1 },
    { 0x6f407064, SUC_lexicon_tags_27 },
    { 0x51c07064, SUC_lexicon_tags_6 },
    { 0xc2807066, SUC_lexicon_tags_68 },
    { 0xbae07067, SUC_lexicon_tags_25 },
    { 0x67e07068, SUC_lexicon_tags_37 },
    { 0x18007069, SUC_lexicon_tags_2 },
    { 0x1bc0706a, SUC_lexicon_tags_27 },
    { 0xb4a07069, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba07071, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c07077, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1a07079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e20707b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74c0707d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x65a0707f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63407084, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x85e07086, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce0708b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xff40708e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c07092, SUC_lexicon_tags_1 },
    { 0xec807093, SUC_lexicon_tags_37 },
    { 0x9a607092, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6070a1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c070a4, SUC_lexicon_tags_9 },
    { 0xc10070a5, SUC_lexicon_tags_7 },
    { 0xfb8070a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8070af, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea070b4, SUC_lexicon_tags_9 },
    { 0x9ae070b5, SUC_lexicon_tags_9 },
    { 0xaa2070b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c070bc, SUC_lexicon_tags_23 },
    { 0x2e0070bd, SUC_lexicon_tags_12 },
    { 0x93e070bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7c070c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0070c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x878070c8, SUC_lexicon_tags_14 },
    { 0xeb0070c9, SUC_lexicon_tags_463 },
    { 0x2d8070c8, SUC_lexicon_tags_11 },
    { 0xd3c070c9, SUC_lexicon_tags_1 },
    { 0x4f4070c8, SUC_lexicon_tags_46 },
    { 0xd52070c8, SUC_lexicon_tags_16 },
    { 0x6ee070cc, SUC_lexicon_tags_25 },
    { 0xe14070cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2070d3, SUC_lexicon_tags_33 },
    { 0x5d6070d3, SUC_lexicon_tags_2 },
    { 0x5d2070d3, SUC_lexicon_tags_25 },
    { 0x334070d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x308070d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52070dc, SUC_lexicon_tags_19 },
    { 0xdae070dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6070e0, SUC_lexicon_tags_54 },
    { 0x8a8070e0, SUC_lexicon_tags_8 },
    { 0x964070e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0070e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc070e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fe070eb, SUC_lexicon_tags_15 },
    { 0x1e2070ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0070ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4070f2, SUC_lexicon_tags_224 },
    { 0x748070f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xee070f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e070fb, SUC_lexicon_tags_6 },
    { 0x3fe070fc, SUC_lexicon_tags_13 },
    { 0x492070fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x53807100, SUC_lexicon_tags_25 },
    { 0x63007100, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc07107, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca80710c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6207110, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc007115, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8a07117, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7220711a, SUC_lexicon_tags_25 },
    { 0xaea0711b, SUC_lexicon_tags_612 },
    { 0xa940711c, SUC_lexicon_tags_7 },
    { 0x6100711d, SUC_lexicon_tags_25 },
    { 0x29e0711d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec07123, SUC_lexicon_tags_5 },
    { 0xe07124, SUC_lexicon_tags_2 },
    { 0x1aa07124, SUC_lexicon_tags_17 },
    { 0x98e07126, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4407129, SUC_lexicon_tags_11 },
    { 0x9900712a, SUC_lexicon_tags_15 },
    { 0x1a00712b, SUC_lexicon_tags_1 },
    { 0x5800712c, SUC_lexicon_tags_4 },
    { 0x61a0712a, SUC_lexicon_tags_17 },
    { 0x61c0712e, SUC_lexicon_tags_37 },
    { 0xb0c0712f, SUC_lexicon_tags_10 },
    { 0x84e0712e, SUC_lexicon_tags_49 },
    { 0xe2e0712e, SUC_lexicon_tags_15 },
    { 0xc2c07132, SUC_lexicon_tags_2 },
    { 0x15a07133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51007137, SUC_lexicon_tags_37 },
    { 0x8c607137, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x4600713b, SUC_lexicon_tags_25 },
    { 0x4080713c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf80713e, SUC_lexicon_tags_61 },
    { 0xaa80713f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xae607142, SUC_lexicon_tags_12 },
    { 0xdbe07143, SUC_lexicon_tags_37 },
    { 0xf5607143, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a07148, SUC_lexicon_tags_33 },
    { 0x8f607149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa360714b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc080714f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15c07151, SUC_lexicon_tags_43 },
    { 0xb3807152, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcb207154, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x8f007156, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5dc07158, SUC_lexicon_tags_17 },
    { 0xfba07158, SUC_lexicon_tags_11 },
    { 0x92007158, SUC_lexicon_tags_21 },
    { 0x76207159, SUC_lexicon_tags_11 },
    { 0xcee0715a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e07161, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf807163, SUC_lexicon_tags_11 },
    { 0x1ee07163, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a07167, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45607169, SUC_lexicon_tags_63 },
    { 0xac007169, SUC_lexicon_tags_37 },
    { 0xe260716b, SUC_lexicon_tags_87 },
    { 0xdb20716c, SUC_lexicon_tags_1 },
    { 0xc640716b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a80716f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a07173, SUC_lexicon_tags_37 },
    { 0xe6607174, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90407179, SUC_lexicon_tags_65 },
    { 0x37c07179, SUC_lexicon_tags_47 },
    { 0x3f207179, SUC_lexicon_tags_6 },
    { 0x8f20717b, SUC_lexicon_tags_7 },
    { 0xba40717b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c207184, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5fe07186, SUC_lexicon_tags_12 },
    { 0xe6c07187, SUC_lexicon_tags_14 },
    { 0x70007188, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3980718a, SUC_lexicon_tags_31 },
    { 0xf500718b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5240718d, SUC_lexicon_tags_29 },
    { 0x9140718e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe2207190, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x53c07192, SUC_lexicon_tags_3 },
    { 0x7a207193, SUC_lexicon_tags_43 },
    { 0x50607194, SUC_lexicon_tags_1 },
    { 0xf6407195, SUC_lexicon_tags_37 },
    { 0xaa807194, SUC_lexicon_tags_30 },
    { 0x75207195, SUC_lexicon_tags_15 },
    { 0x70807197, SUC_lexicon_tags_19 },
    { 0xa9a07194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5620719b, SUC_lexicon_tags_1 },
    { 0x80c0719c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa0719f, SUC_lexicon_tags_7 },
    { 0xd440719f, SUC_lexicon_tags_33 },
    { 0xcb20719f, SUC_lexicon_tags_23 },
    { 0x2e8071a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x368071a4, SUC_lexicon_tags_55 },
    { 0xe3a071a4, SUC_lexicon_tags_32 },
    { 0xd4e071a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864071b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34071b7, SUC_lexicon_tags_25 },
    { 0x6c4071b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb38071ba, SUC_lexicon_tags_37 },
    { 0xf8c071bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a8071bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a071c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a071ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834071d4, SUC_lexicon_tags_2 },
    { 0x7ac071d5, SUC_lexicon_tags_37 },
    { 0xdb2071d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf0071d8, SUC_lexicon_tags_11 },
    { 0xa86071d9, SUC_lexicon_tags_11 },
    { 0xa86071da, SUC_lexicon_tags_19 },
    { 0x47a071da, SUC_lexicon_tags_46 },
    { 0x788071dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a071df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592071e7, SUC_lexicon_tags_1 },
    { 0x2de071e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924071ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fa071f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5c071f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf6071f5, SUC_lexicon_tags_28 },
    { 0xfc071f5, SUC_lexicon_tags_32 },
    { 0x93e071f7, SUC_lexicon_tags_17 },
    { 0x92c071f8, SUC_lexicon_tags_61 },
    { 0xddc071f8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbe0071fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c07200, SUC_lexicon_tags_7 },
    { 0xc4407201, SUC_lexicon_tags_23 },
    { 0x85a07202, SUC_lexicon_tags_37 },
    { 0xa3007202, SUC_lexicon_tags_63 },
    { 0x6e207204, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a07209, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e07213, SUC_lexicon_tags_37 },
    { 0xeac07213, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c007217, SUC_lexicon_tags_2 },
    { 0x91407218, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c0721b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3940721d, SUC_lexicon_tags_45 },
    { 0x92a0721d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x64807221, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88607224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e07228, SUC_lexicon_tags_17 },
    { 0x99407229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca0722e, SUC_lexicon_tags_45 },
    { 0xa560722e, SUC_lexicon_tags_45 },
    { 0x14607230, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a0723c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4980723e, SUC_lexicon_tags_83 },
    { 0x340723f, SUC_lexicon_tags_15 },
    { 0x14e0723f, SUC_lexicon_tags_15 },
    { 0xf5607241, SUC_lexicon_tags_15 },
    { 0xf6c07240, SUC_lexicon_tags_45 },
    { 0x72807243, SUC_lexicon_tags_17 },
    { 0xcf207240, SUC_lexicon_tags_0 },
    { 0x15c07242, SUC_lexicon_tags_42 },
    { 0xf4a07244, SUC_lexicon_tags_60 },
    { 0x7da07247, SUC_lexicon_tags_37 },
    { 0xb2a0723f, SUC_lexicon_tags_15 },
    { 0xc6a07242, SUC_lexicon_tags_38 },
    { 0x3520724a, SUC_lexicon_tags_11 },
    { 0xc5e0724b, SUC_lexicon_tags_15 },
    { 0x14a07245, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e0724f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6a07251, SUC_lexicon_tags_17 },
    { 0x6c207252, SUC_lexicon_tags_45 },
    { 0x4d207252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15007257, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec80725b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba607260, SUC_lexicon_tags_4 },
    { 0x5c07261, SUC_lexicon_tags_27 },
    { 0xc0807260, SUC_lexicon_tags_2 },
    { 0x19e07263, SUC_lexicon_tags_56 },
    { 0x45407263, SUC_lexicon_tags_66 },
    { 0x8607264, SUC_lexicon_tags_2 },
    { 0xa3807266, SUC_lexicon_tags_30 },
    { 0x53207267, SUC_lexicon_tags_28 },
    { 0xfce07267, SUC_lexicon_tags_11 },
    { 0x37c07264, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b40726c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d80726e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1007272, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7807274, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c0727c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6280727f, SUC_lexicon_tags_12 },
    { 0xdb60727f, SUC_lexicon_tags_46 },
    { 0xeca07281, SUC_lexicon_tags_7 },
    { 0x4540727f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ec07284, SUC_lexicon_tags_30 },
    { 0x36007284, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa07288, SUC_lexicon_tags_4 },
    { 0x3807288, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2a0728b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2207290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5de07292, SUC_lexicon_tags_201 },
    { 0x7d207293, SUC_lexicon_tags_43 },
    { 0x9807292, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21207296, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf380729a, SUC_lexicon_tags_38 },
    { 0x7880729b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc072a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66072a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2fa072ab, SUC_lexicon_tags_10 },
    { 0x43e072ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8d0072ae, SUC_lexicon_tags_158 },
    { 0xc7c072af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500072b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee072bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x832072bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe82072bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe14072c1, SUC_lexicon_tags_19 },
    { 0xd6a072c2, SUC_lexicon_tags_38 },
    { 0xc0a072c3, SUC_lexicon_tags_37 },
    { 0xb7c072c2, SUC_lexicon_tags_37 },
    { 0xea8072c2, SUC_lexicon_tags_37 },
    { 0xb70072c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58072ca, SUC_lexicon_tags_25 },
    { 0xf06072cb, SUC_lexicon_tags_37 },
    { 0xa16072ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x488072cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x326072d2, SUC_lexicon_tags_11 },
    { 0x2f6072d3, SUC_lexicon_tags_62 },
    { 0x2b4072d4, SUC_lexicon_tags_37 },
    { 0x224072d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x634072d7, SUC_lexicon_tags_37 },
    { 0x77c072d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e072db, SUC_lexicon_tags_0 },
    { 0x9cc072dc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428072e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644072e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978072e9, SUC_lexicon_tags_29 },
    { 0xd2e072e9, SUC_lexicon_tags_60 },
    { 0xa20072e9, SUC_lexicon_tags_7 },
    { 0x3c072ec, SUC_lexicon_tags_19 },
    { 0xf1a072eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa072f0, SUC_lexicon_tags_23 },
    { 0xaee072f1, SUC_lexicon_tags_25 },
    { 0x7ee072f0, SUC_lexicon_tags_7 },
    { 0x18c072f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x344072f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a072fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x92c072fd, SUC_lexicon_tags_37 },
    { 0x36072fe, SUC_lexicon_tags_25 },
    { 0x68072ff, SUC_lexicon_tags_15 },
    { 0x3ce07300, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x62807303, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3207306, SUC_lexicon_tags_7 },
    { 0x47e07306, SUC_lexicon_tags_16 },
    { 0xfda07308, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3560730b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10607311, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x60807314, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a07317, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac60731a, SUC_lexicon_tags_14 },
    { 0x10e0731b, SUC_lexicon_tags_31 },
    { 0x5520731c, SUC_lexicon_tags_1 },
    { 0x8580731d, SUC_lexicon_tags_27 },
    { 0x9080731a, SUC_lexicon_tags_11 },
    { 0xe460731a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a07323, SUC_lexicon_tags_47 },
    { 0x40c07323, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e07327, SUC_lexicon_tags_4 },
    { 0xdc07328, SUC_lexicon_tags_13 },
    { 0x2ec07329, SUC_lexicon_tags_17 },
    { 0x2560732a, SUC_lexicon_tags_1 },
    { 0xdd20732a, SUC_lexicon_tags_65 },
    { 0x8fe07329, SUC_lexicon_tags_39 },
    { 0xbde0732d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1007331, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a207337, SUC_lexicon_tags_33 },
    { 0x30a07338, SUC_lexicon_tags_45 },
    { 0xc0807337, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e0733c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20733f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c07345, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee40734b, SUC_lexicon_tags_60 },
    { 0xa20734b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd3e0734e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e807352, SUC_lexicon_tags_60 },
    { 0xc4a07352, SUC_lexicon_tags_46 },
    { 0x8fe07354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f607356, SUC_lexicon_tags_92 },
    { 0x7d207357, SUC_lexicon_tags_17 },
    { 0x8207358, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa20735a, SUC_lexicon_tags_9 },
    { 0xe180735b, SUC_lexicon_tags_3 },
    { 0x7160735a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb960735f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e07365, SUC_lexicon_tags_7 },
    { 0x77807366, SUC_lexicon_tags_49 },
    { 0xa407365, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1080736a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7400736c, SUC_lexicon_tags_1 },
    { 0xe0a0736d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5007371, SUC_lexicon_tags_15 },
    { 0xdfe07371, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e807377, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd520737a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c00737c, SUC_lexicon_tags_19 },
    { 0xa5e0737c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a007382, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a07385, SUC_lexicon_tags_17 },
    { 0xede07386, SUC_lexicon_tags_7 },
    { 0xd9607386, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b40738f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xca407392, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce607396, SUC_lexicon_tags_63 },
    { 0x65a07397, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7060739d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70073aa, SUC_lexicon_tags_61 },
    { 0xa28073ab, SUC_lexicon_tags_25 },
    { 0x866073ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a073af, SUC_lexicon_tags_17 },
    { 0x6b2073b0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2aa073b2, SUC_lexicon_tags_17 },
    { 0xcee073b2, SUC_lexicon_tags_31 },
    { 0x9d2073b4, SUC_lexicon_tags_37 },
    { 0x8a073b4, SUC_lexicon_tags_15 },
    { 0xcb0073b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x202073b8, SUC_lexicon_tags_35 },
    { 0x666073b8, SUC_lexicon_tags_7 },
    { 0x5f4073b8, SUC_lexicon_tags_49 },
    { 0x1a073ba, SUC_lexicon_tags_16 },
    { 0xe6a073bc, SUC_lexicon_tags_17 },
    { 0x110073bd, SUC_lexicon_tags_61 },
    { 0x186073bc, SUC_lexicon_tags_11 },
    { 0xa5c073bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44073c2, SUC_lexicon_tags_18 },
    { 0xea6073c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0073c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0073ca, SUC_lexicon_tags_14 },
    { 0xda6073cb, SUC_lexicon_tags_51 },
    { 0x68a073cc, SUC_lexicon_tags_25 },
    { 0x3e0073cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a073dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c073e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x934073e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48073f5, SUC_lexicon_tags_7 },
    { 0xdf0073f6, SUC_lexicon_tags_12 },
    { 0x90e073f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa073fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x294073fd, SUC_lexicon_tags_1 },
    { 0x1da073fe, SUC_lexicon_tags_3 },
    { 0x8ac073ff, SUC_lexicon_tags_37 },
    { 0x2e4073fe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc807403, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc07407, SUC_lexicon_tags_88 },
    { 0xc7607408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b60740b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xada0740e, SUC_lexicon_tags_14 },
    { 0x8a20740f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14607413, SUC_lexicon_tags_7 },
    { 0x92a07413, SUC_lexicon_tags_7 },
    { 0xe8a07414, SUC_lexicon_tags_7 },
    { 0x29207416, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3880741a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a00741d, SUC_lexicon_tags_14 },
    { 0xd460741d, SUC_lexicon_tags_1 },
    { 0x9600741f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c07426, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a60742b, SUC_lexicon_tags_19 },
    { 0xb3a0742b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac07430, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c07441, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4207444, SUC_lexicon_tags_23 },
    { 0x39a07445, SUC_lexicon_tags_9 },
    { 0x64a07445, SUC_lexicon_tags_2 },
    { 0x38207447, SUC_lexicon_tags_15 },
    { 0xcda07447, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe400744c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9000744e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee807451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa407453, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c207457, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d607459, SUC_lexicon_tags_12 },
    { 0x2a07459, SUC_lexicon_tags_7 },
    { 0x9fc0745a, SUC_lexicon_tags_37 },
    { 0xb2e0745c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a07466, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeba07468, SUC_lexicon_tags_2 },
    { 0xe0407469, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbe40746b, SUC_lexicon_tags_15 },
    { 0x9b60746b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef007472, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c07476, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e407478, SUC_lexicon_tags_9 },
    { 0x3da07479, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf00747b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a07481, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85407485, SUC_lexicon_tags_10 },
    { 0x8007486, SUC_lexicon_tags_34 },
    { 0x65407487, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc1e07489, SUC_lexicon_tags_47 },
    { 0xdee07489, SUC_lexicon_tags_6 },
    { 0xe4607489, SUC_lexicon_tags_15 },
    { 0xccc0748c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc07492, SUC_lexicon_tags_32 },
    { 0x24007493, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x30007496, SUC_lexicon_tags_25 },
    { 0x41e07497, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220749b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c0749f, SUC_lexicon_tags_37 },
    { 0x63e074a0, SUC_lexicon_tags_19 },
    { 0x4e60749f, SUC_lexicon_tags_17 },
    { 0xec4074a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a074a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2074a8, SUC_lexicon_tags_178 },
    { 0x176074a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112074ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf76074b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x928074b2, SUC_lexicon_tags_12 },
    { 0x266074b2, SUC_lexicon_tags_27 },
    { 0x7c2074b4, SUC_lexicon_tags_46 },
    { 0x614074b4, SUC_lexicon_tags_7 },
    { 0xa96074b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2074c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb6c074c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a074c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da074cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a074d2, SUC_lexicon_tags_1 },
    { 0xcee074d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c074d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x142074da, SUC_lexicon_tags_1 },
    { 0x8c2074da, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x286074dd, SUC_lexicon_tags_60 },
    { 0xe68074de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38074e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27e074e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c074ea, SUC_lexicon_tags_48 },
    { 0xa30074ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10c074ed, SUC_lexicon_tags_11 },
    { 0xe64074ed, SUC_lexicon_tags_5 },
    { 0x3cc074ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc4074f1, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894074f6, SUC_lexicon_tags_0 },
    { 0x6c0074f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf74074f9, SUC_lexicon_tags_4 },
    { 0xa9a074fa, SUC_lexicon_tags_0 },
    { 0x73e074fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce407501, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e07505, SUC_lexicon_tags_37 },
    { 0xaf407505, SUC_lexicon_tags_38 },
    { 0xc6607507, SUC_lexicon_tags_82 },
    { 0xf8007508, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c20750b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd100750d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69a0750f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa5807511, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72807517, SUC_lexicon_tags_25 },
    { 0xc4407518, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c07520, SUC_lexicon_tags_47 },
    { 0x82a07521, SUC_lexicon_tags_37 },
    { 0xc0e07521, SUC_lexicon_tags_11 },
    { 0xe3007523, SUC_lexicon_tags_49 },
    { 0x46a07523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a0752c, SUC_lexicon_tags_14 },
    { 0xf3c0752c, SUC_lexicon_tags_39 },
    { 0xbfe0752e, SUC_lexicon_tags_10 },
    { 0x1640752f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c07537, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xe5a07539, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa20753b, SUC_lexicon_tags_27 },
    { 0x2a80753c, SUC_lexicon_tags_30 },
    { 0x1d60753b, SUC_lexicon_tags_12 },
    { 0x4520753e, SUC_lexicon_tags_25 },
    { 0xafa0753c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8d607541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6c07543, SUC_lexicon_tags_11 },
    { 0xd7807544, SUC_lexicon_tags_25 },
    { 0xac607545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0754b, SUC_lexicon_tags_7 },
    { 0xec0754c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85a0754e, SUC_lexicon_tags_7 },
    { 0xb280754f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55a07551, SUC_lexicon_tags_60 },
    { 0xe2207551, SUC_lexicon_tags_1394 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8960755a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe0755f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c07563, SUC_lexicon_tags_28 },
    { 0x3e807564, SUC_lexicon_tags_11 },
    { 0xcf607563, SUC_lexicon_tags_27 },
    { 0x6ca07564, SUC_lexicon_tags_25 },
    { 0x88e07567, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x31a07569, SUC_lexicon_tags_37 },
    { 0xe5a0756a, SUC_lexicon_tags_7 },
    { 0x4c007569, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda807573, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4e07575, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc340757f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0207582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63807585, SUC_lexicon_tags_25 },
    { 0x9c407586, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x25407589, SUC_lexicon_tags_37 },
    { 0xa820758a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce0758d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7ea0758f, SUC_lexicon_tags_49 },
    { 0xcc407590, SUC_lexicon_tags_27 },
    { 0xaba07591, SUC_lexicon_tags_15 },
    { 0xba407590, SUC_lexicon_tags_48 },
    { 0x1007593, SUC_lexicon_tags_37 },
    { 0x46007594, SUC_lexicon_tags_37 },
    { 0x23607595, SUC_lexicon_tags_15 },
    { 0xfa07596, SUC_lexicon_tags_17 },
    { 0x58207596, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd4a07599, SUC_lexicon_tags_23 },
    { 0xe6407599, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b40759e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6075a1, SUC_lexicon_tags_25 },
    { 0xb7e075a2, SUC_lexicon_tags_45 },
    { 0x738075a3, SUC_lexicon_tags_26 },
    { 0xdf0075a2, SUC_lexicon_tags_17 },
    { 0xeac075a3, SUC_lexicon_tags_11 },
    { 0x4a8075a2, SUC_lexicon_tags_37 },
    { 0xed4075a1, SUC_lexicon_tags_38 },
    { 0xb06075a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x120075aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4075ad, SUC_lexicon_tags_11 },
    { 0xf02075ae, SUC_lexicon_tags_39 },
    { 0x444075af, SUC_lexicon_tags_45 },
    { 0xf5c075ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf0075b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6075b5, SUC_lexicon_tags_79 },
    { 0x89c075b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378075bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6075c5, SUC_lexicon_tags_65 },
    { 0xa6075c5, SUC_lexicon_tags_37 },
    { 0x866075c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462075cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0075d0, SUC_lexicon_tags_69 },
    { 0x416075d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4075d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8075d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c075e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2075e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa075e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6075ea, SUC_lexicon_tags_9 },
    { 0x514075eb, SUC_lexicon_tags_14 },
    { 0x4d0075ec, SUC_lexicon_tags_23 },
    { 0x8b6075ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x778075f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae075f7, SUC_lexicon_tags_60 },
    { 0x73e075f7, SUC_lexicon_tags_41 },
    { 0xf40075f9, SUC_lexicon_tags_13 },
    { 0x91e075f9, SUC_lexicon_tags_25 },
    { 0xfd0075fa, SUC_lexicon_tags_27 },
    { 0xf84075fc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdc2075fe, SUC_lexicon_tags_30 },
    { 0x8e2075fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57e07601, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82207605, SUC_lexicon_tags_7 },
    { 0x59807606, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3140760c, SUC_lexicon_tags_38 },
    { 0x4640760d, SUC_lexicon_tags_15 },
    { 0x9840760e, SUC_lexicon_tags_7 },
    { 0xce0760e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee807612, SUC_lexicon_tags_23 },
    { 0xa8007613, SUC_lexicon_tags_7 },
    { 0x6e407614, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x18007617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33a07619, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8ea0761b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b207620, SUC_lexicon_tags_0 },
    { 0x13c07621, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5b007623, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6007625, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a0762a, SUC_lexicon_tags_8 },
    { 0xb140762a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3460762d, SUC_lexicon_tags_12 },
    { 0x2e80762e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90207630, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x23807632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b607635, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec607637, SUC_lexicon_tags_31 },
    { 0xc8607637, SUC_lexicon_tags_17 },
    { 0x7c07639, SUC_lexicon_tags_33 },
    { 0x4f807637, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc0763d, SUC_lexicon_tags_12 },
    { 0x9e60763e, SUC_lexicon_tags_12 },
    { 0x5a40763e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0007644, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4c07646, SUC_lexicon_tags_56 },
    { 0x8607647, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x71807649, SUC_lexicon_tags_17 },
    { 0xec607649, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb2a0764c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c07650, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35207652, SUC_lexicon_tags_32 },
    { 0xc2e07653, SUC_lexicon_tags_30 },
    { 0x83607654, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd807658, SUC_lexicon_tags_15 },
    { 0xa9007659, SUC_lexicon_tags_61 },
    { 0xc007658, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30807661, SUC_lexicon_tags_14 },
    { 0x6a407662, SUC_lexicon_tags_4 },
    { 0x90e07662, SUC_lexicon_tags_29 },
    { 0x9b007664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40607667, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7880766c, SUC_lexicon_tags_47 },
    { 0x9760766c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b207673, SUC_lexicon_tags_15 },
    { 0x61a07673, SUC_lexicon_tags_15 },
    { 0xab607674, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b407685, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x98007688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e80768a, SUC_lexicon_tags_33 },
    { 0xb320768b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf640768e, SUC_lexicon_tags_12 },
    { 0xe840768f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6d407691, SUC_lexicon_tags_33 },
    { 0xeae07692, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9807695, SUC_lexicon_tags_47 },
    { 0x43c07695, SUC_lexicon_tags_0 },
    { 0x17207697, SUC_lexicon_tags_15 },
    { 0x63407698, SUC_lexicon_tags_37 },
    { 0x5dc07697, SUC_lexicon_tags_19 },
    { 0x9d207695, SUC_lexicon_tags_37 },
    { 0xc3c07698, SUC_lexicon_tags_2 },
    { 0xe120769c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15a0769e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a076a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a076aa, SUC_lexicon_tags_19 },
    { 0xd94076aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510076b2, SUC_lexicon_tags_17 },
    { 0x896076b2, SUC_lexicon_tags_35 },
    { 0xb60076b4, SUC_lexicon_tags_11 },
    { 0x5ec076b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x744076b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c076be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2076c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x928076c8, SUC_lexicon_tags_45 },
    { 0x958076c9, SUC_lexicon_tags_1 },
    { 0xcce076ca, SUC_lexicon_tags_1 },
    { 0xe8a076cb, SUC_lexicon_tags_11 },
    { 0x552076cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x986076ce, SUC_lexicon_tags_4 },
    { 0xe6076ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce076d2, SUC_lexicon_tags_11 },
    { 0x290076d3, SUC_lexicon_tags_25 },
    { 0xfaa076d3, SUC_lexicon_tags_28 },
    { 0xcba076d5, SUC_lexicon_tags_47 },
    { 0xe40076d6, SUC_lexicon_tags_16 },
    { 0x5c0076d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x446076d9, SUC_lexicon_tags_4 },
    { 0x9c8076da, SUC_lexicon_tags_33 },
    { 0x51e076da, SUC_lexicon_tags_29 },
    { 0xecc076db, SUC_lexicon_tags_37 },
    { 0x15c076dd, SUC_lexicon_tags_19 },
    { 0xe2a076de, SUC_lexicon_tags_12 },
    { 0x66076db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e076e4, SUC_lexicon_tags_17 },
    { 0x3d6076e4, SUC_lexicon_tags_38 },
    { 0x30e076e6, SUC_lexicon_tags_151 },
    { 0x196076e5, SUC_lexicon_tags_25 },
    { 0x2a0076e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa80076ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac0076ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a076f8, SUC_lexicon_tags_38 },
    { 0x91a076f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a076fd, SUC_lexicon_tags_66 },
    { 0x382076fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a07704, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a07708, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf60770c, SUC_lexicon_tags_8 },
    { 0x5fa0770c, SUC_lexicon_tags_39 },
    { 0x2480770c, SUC_lexicon_tags_15 },
    { 0x2660770c, SUC_lexicon_tags_2 },
    { 0x79807710, SUC_lexicon_tags_14 },
    { 0xad407711, SUC_lexicon_tags_1 },
    { 0x19407711, SUC_lexicon_tags_10 },
    { 0x19207711, SUC_lexicon_tags_38 },
    { 0xe7007711, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43a07716, SUC_lexicon_tags_12 },
    { 0x30c07717, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4480771d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5300771f, SUC_lexicon_tags_48 },
    { 0x2807720, SUC_lexicon_tags_38 },
    { 0xf5807720, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3207725, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb9c07727, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe300772d, SUC_lexicon_tags_1 },
    { 0x6980772d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xede07731, SUC_lexicon_tags_37 },
    { 0x6ba07731, SUC_lexicon_tags_9 },
    { 0xd8807733, SUC_lexicon_tags_19 },
    { 0x1a07733, SUC_lexicon_tags_15 },
    { 0x7f007733, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa07739, SUC_lexicon_tags_15 },
    { 0xbe20773a, SUC_lexicon_tags_8 },
    { 0x4da0773b, SUC_lexicon_tags_37 },
    { 0xf3407739, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f207740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4207742, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f807744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a07752, SUC_lexicon_tags_49 },
    { 0x3d607753, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x75e07755, SUC_lexicon_tags_28 },
    { 0x46407755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a0775b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48a0775d, SUC_lexicon_tags_45 },
    { 0xd0a0775d, SUC_lexicon_tags_39 },
    { 0x3ec0775f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfe07761, SUC_lexicon_tags_992 },
    { 0x67607761, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1c607764, SUC_lexicon_tags_11 },
    { 0x81807764, SUC_lexicon_tags_63 },
    { 0xc9207764, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c00776b, SUC_lexicon_tags_63 },
    { 0xcde0776b, SUC_lexicon_tags_25 },
    { 0x34a0776d, SUC_lexicon_tags_9 },
    { 0x35c0776d, SUC_lexicon_tags_17 },
    { 0xde20776f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c807779, SUC_lexicon_tags_15 },
    { 0x6a807779, SUC_lexicon_tags_12 },
    { 0x777a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd60777e, SUC_lexicon_tags_12 },
    { 0x2e80777f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6a07781, SUC_lexicon_tags_15 },
    { 0x27a07782, SUC_lexicon_tags_8 },
    { 0x9d607782, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x85607785, SUC_lexicon_tags_23 },
    { 0x23807786, SUC_lexicon_tags_28 },
    { 0x70807787, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x41607789, SUC_lexicon_tags_2 },
    { 0x9920778a, SUC_lexicon_tags_37 },
    { 0x7100778b, SUC_lexicon_tags_15 },
    { 0xcfa07789, SUC_lexicon_tags_1 },
    { 0x7ac0778d, SUC_lexicon_tags_60 },
    { 0xc180778b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0207794, SUC_lexicon_tags_79 },
    { 0x93607795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe007797, SUC_lexicon_tags_32 },
    { 0x9ae07798, SUC_lexicon_tags_46 },
    { 0x15a07799, SUC_lexicon_tags_25 },
    { 0xa607799, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ea0779c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe077a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a077a9, SUC_lexicon_tags_19 },
    { 0x572077aa, SUC_lexicon_tags_23 },
    { 0x882077ab, SUC_lexicon_tags_17 },
    { 0xcba077ac, SUC_lexicon_tags_37 },
    { 0x6f2077ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840077b4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14077b7, SUC_lexicon_tags_19 },
    { 0x3a4077b8, SUC_lexicon_tags_39 },
    { 0x376077b8, SUC_lexicon_tags_1 },
    { 0x25a077b9, SUC_lexicon_tags_31 },
    { 0x89a077b8, SUC_lexicon_tags_19 },
    { 0xd20077bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4077c0, SUC_lexicon_tags_26 },
    { 0x558077c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc077c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x558077c8, SUC_lexicon_tags_37 },
    { 0x35e077c9, SUC_lexicon_tags_43 },
    { 0xa0077ca, SUC_lexicon_tags_65 },
    { 0x8aa077ca, SUC_lexicon_tags_43 },
    { 0xd0077cc, SUC_lexicon_tags_55 },
    { 0xaaa077cc, SUC_lexicon_tags_29 },
    { 0x4f2077cc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x520077d1, SUC_lexicon_tags_7 },
    { 0xc16077d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x208077d4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e077d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e077da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e077df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80c077e1, SUC_lexicon_tags_6 },
    { 0xb60077e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2aa077e4, SUC_lexicon_tags_17 },
    { 0xd96077e5, SUC_lexicon_tags_7 },
    { 0x58c077e5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x338077e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be077f1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334077f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb2077f8, SUC_lexicon_tags_15 },
    { 0x32a077f9, SUC_lexicon_tags_12 },
    { 0x4da077f9, SUC_lexicon_tags_38 },
    { 0x94077fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xade077fe, SUC_lexicon_tags_1 },
    { 0xe78077fe, SUC_lexicon_tags_126 },
    { 0xede07800, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x83007802, SUC_lexicon_tags_26 },
    { 0x39a07802, SUC_lexicon_tags_17 },
    { 0x49407804, SUC_lexicon_tags_19 },
    { 0xfba07805, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da0780b, SUC_lexicon_tags_56 },
    { 0x1700780c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe07813, SUC_lexicon_tags_104 },
    { 0x75007814, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e407819, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a00781e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88407820, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb007823, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c07828, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa120782a, SUC_lexicon_tags_17 },
    { 0xea60782b, SUC_lexicon_tags_2 },
    { 0x5200782a, SUC_lexicon_tags_35 },
    { 0xaa60782d, SUC_lexicon_tags_1 },
    { 0xc300782c, SUC_lexicon_tags_49 },
    { 0x63a0782f, SUC_lexicon_tags_17 },
    { 0xb0c07830, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ea07832, SUC_lexicon_tags_37 },
    { 0xd8207833, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c07836, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x49807838, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac0783e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72207840, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c207842, SUC_lexicon_tags_12 },
    { 0xa0807843, SUC_lexicon_tags_11 },
    { 0x72a07843, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5f807846, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd6007848, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf720784c, SUC_lexicon_tags_9 },
    { 0xc0c0784c, SUC_lexicon_tags_33 },
    { 0xd840784c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca07851, SUC_lexicon_tags_11 },
    { 0xed607851, SUC_lexicon_tags_0 },
    { 0xcce07853, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa07857, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b607859, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a0785c, SUC_lexicon_tags_30 },
    { 0x8bc0785c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad807863, SUC_lexicon_tags_32 },
    { 0xd5c07864, SUC_lexicon_tags_2 },
    { 0x8c807865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8407867, SUC_lexicon_tags_47 },
    { 0x66407868, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf800786a, SUC_lexicon_tags_8 },
    { 0xdfe0786b, SUC_lexicon_tags_17 },
    { 0xa760786b, SUC_lexicon_tags_8 },
    { 0xce0786b, SUC_lexicon_tags_8 },
    { 0x49c0786b, SUC_lexicon_tags_49 },
    { 0xe0c0786a, SUC_lexicon_tags_63 },
    { 0xe4e07870, SUC_lexicon_tags_84 },
    { 0x6d20786c, SUC_lexicon_tags_11 },
    { 0x8aa07872, SUC_lexicon_tags_2 },
    { 0x57a07872, SUC_lexicon_tags_15 },
    { 0xf080786c, SUC_lexicon_tags_37 },
    { 0x43807875, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d60787a, SUC_lexicon_tags_7 },
    { 0x2620787a, SUC_lexicon_tags_1 },
    { 0xf660787c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x36c0787e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75607883, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdd407885, SUC_lexicon_tags_9 },
    { 0x27a07886, SUC_lexicon_tags_47 },
    { 0x6dc07886, SUC_lexicon_tags_25 },
    { 0xac007888, SUC_lexicon_tags_68 },
    { 0x3aa07889, SUC_lexicon_tags_37 },
    { 0x42207885, SUC_lexicon_tags_12 },
    { 0xfc0788b, SUC_lexicon_tags_25 },
    { 0x4320788b, SUC_lexicon_tags_12 },
    { 0x3480788c, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2407890, SUC_lexicon_tags_0 },
    { 0xb4607891, SUC_lexicon_tags_7 },
    { 0x9407891, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90007895, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5207899, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3e0789b, SUC_lexicon_tags_11 },
    { 0x9220789c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xada0789e, SUC_lexicon_tags_25 },
    { 0xe6c0789f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x530078a1, SUC_lexicon_tags_39 },
    { 0x67a078a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6078a7, SUC_lexicon_tags_45 },
    { 0xbb6078a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x976078aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefa078ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50078b0, SUC_lexicon_tags_68 },
    { 0x394078b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612078b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2078b7, SUC_lexicon_tags_8 },
    { 0x54c078b8, SUC_lexicon_tags_17 },
    { 0xb56078b7, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436078c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xca2078c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c078ca, SUC_lexicon_tags_6 },
    { 0x874078ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4078d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x66a078d2, SUC_lexicon_tags_6 },
    { 0x3dc078d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda078d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x826078d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70078e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x20078e3, SUC_lexicon_tags_15 },
    { 0xca078e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba078e9, SUC_lexicon_tags_66 },
    { 0x9f0078e9, SUC_lexicon_tags_8 },
    { 0xe84078e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a078ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62078f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c078fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b8078fd, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4007900, SUC_lexicon_tags_49 },
    { 0x59a07901, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5007904, SUC_lexicon_tags_20 },
    { 0xe0c07905, SUC_lexicon_tags_63 },
    { 0xf6807906, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21807908, SUC_lexicon_tags_74 },
    { 0xaa207909, SUC_lexicon_tags_19 },
    { 0xa000790a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc40790e, SUC_lexicon_tags_37 },
    { 0x8f80790f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22807911, SUC_lexicon_tags_37 },
    { 0xdd807912, SUC_lexicon_tags_13 },
    { 0xb5407911, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x45807916, SUC_lexicon_tags_15 },
    { 0x68607916, SUC_lexicon_tags_15 },
    { 0xf6a07918, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe380791a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc80791f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11407923, SUC_lexicon_tags_32 },
    { 0x33007924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c07927, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80407929, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2500792b, SUC_lexicon_tags_46 },
    { 0xa080792b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d20792e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29007930, SUC_lexicon_tags_15 },
    { 0x35e07930, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0xffa07933, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba07936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a40793a, SUC_lexicon_tags_33 },
    { 0x1d60793b, SUC_lexicon_tags_63 },
    { 0x1120793c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb160793f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd007941, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a07944, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc0794a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf60794e, SUC_lexicon_tags_19 },
    { 0x3f80794e, SUC_lexicon_tags_57 },
    { 0x7b607950, SUC_lexicon_tags_63 },
    { 0x9f007950, SUC_lexicon_tags_6 },
    { 0xe1207950, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23007954, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b407958, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd40795a, SUC_lexicon_tags_2 },
    { 0x97a0795a, SUC_lexicon_tags_3 },
    { 0x66a0795c, SUC_lexicon_tags_12 },
    { 0xae20795d, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0xc500795f, SUC_lexicon_tags_45 },
    { 0xd100795f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7da07962, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87e07964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e07969, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60796f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x87207972, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x61807975, SUC_lexicon_tags_1 },
    { 0xfd207976, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x15c07978, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e0797d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6207985, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a007987, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b007989, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa0798e, SUC_lexicon_tags_7 },
    { 0xbe40798f, SUC_lexicon_tags_112 },
    { 0x88207990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea07995, SUC_lexicon_tags_25 },
    { 0xbec07996, SUC_lexicon_tags_37 },
    { 0x77c07997, SUC_lexicon_tags_0 },
    { 0xce207996, SUC_lexicon_tags_37 },
    { 0x9e807999, SUC_lexicon_tags_26 },
    { 0xc2c07999, SUC_lexicon_tags_29 },
    { 0x5c07999, SUC_lexicon_tags_28 },
    { 0xc8a0799a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0799f, SUC_lexicon_tags_32 },
    { 0x36c079a0, SUC_lexicon_tags_47 },
    { 0x4280799f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe00079a3, SUC_lexicon_tags_49 },
    { 0xa4079a3, SUC_lexicon_tags_12 },
    { 0xa28079a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68079ae, SUC_lexicon_tags_38 },
    { 0xac8079af, SUC_lexicon_tags_23 },
    { 0xfaa079af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96079b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x852079b8, SUC_lexicon_tags_46 },
    { 0x2c079b9, SUC_lexicon_tags_1 },
    { 0x266079ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68079be, SUC_lexicon_tags_37 },
    { 0x9ac079bf, SUC_lexicon_tags_17 },
    { 0xc9a079bf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34079c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa079ce, SUC_lexicon_tags_6 },
    { 0x402079ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448079d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6079d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8079dc, SUC_lexicon_tags_23 },
    { 0x5d6079dc, SUC_lexicon_tags_10 },
    { 0xb6079dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4079e1, SUC_lexicon_tags_36 },
    { 0xc34079e2, SUC_lexicon_tags_27 },
    { 0xa04079e2, SUC_lexicon_tags_13 },
    { 0x46a079e4, SUC_lexicon_tags_37 },
    { 0x7b4079e1, SUC_lexicon_tags_60 },
    { 0x64e079e5, SUC_lexicon_tags_10 },
    { 0xf72079e7, SUC_lexicon_tags_26 },
    { 0x3ac079e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe079ed, SUC_lexicon_tags_0 },
    { 0x270079ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90079f5, SUC_lexicon_tags_19 },
    { 0x542079f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42a079f8, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4079fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e07a00, SUC_lexicon_tags_53 },
    { 0x1f807a01, SUC_lexicon_tags_60 },
    { 0xcc607a02, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x52607a05, SUC_lexicon_tags_29 },
    { 0x86807a06, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e07a0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa207a12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x69207a14, SUC_lexicon_tags_8 },
    { 0x4d407a15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5de07a17, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcc407a19, SUC_lexicon_tags_118 },
    { 0xdda07a1a, SUC_lexicon_tags_11 },
    { 0x68e07a1b, SUC_lexicon_tags_15 },
    { 0xb2a07a19, SUC_lexicon_tags_7 },
    { 0x97a07a1b, SUC_lexicon_tags_17 },
    { 0xd2207a1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d007a23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f607a25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde407a28, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x37807a2a, SUC_lexicon_tags_12 },
    { 0xc8a07a2b, SUC_lexicon_tags_19 },
    { 0xb007a2a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc607a2f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xea607a31, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c607a38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda07a3b, SUC_lexicon_tags_7 },
    { 0xc8007a3c, SUC_lexicon_tags_28 },
    { 0xe9607a3d, SUC_lexicon_tags_15 },
    { 0x22207a3e, SUC_lexicon_tags_7 },
    { 0xfa07a3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e07a44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc07a4a, SUC_lexicon_tags_39 },
    { 0xd2c07a4b, SUC_lexicon_tags_7 },
    { 0x1ba07a4c, SUC_lexicon_tags_108 },
    { 0xa4e07a4c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe0a07a4f, SUC_lexicon_tags_37 },
    { 0xfc407a50, SUC_lexicon_tags_23 },
    { 0x67007a50, SUC_lexicon_tags_28 },
    { 0x8d607a51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc0c07a54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5007a57, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a07a5b, SUC_lexicon_tags_45 },
    { 0x17407a5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86007a62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc007a64, SUC_lexicon_tags_17 },
    { 0x4a07a64, SUC_lexicon_tags_33 },
    { 0x92207a64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x94607a68, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xef407a6a, SUC_lexicon_tags_25 },
    { 0xe7607a6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70407a71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc607a73, SUC_lexicon_tags_15 },
    { 0x83e07a74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40407a79, SUC_lexicon_tags_1 },
    { 0x23e07a7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4ea07a7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8aa07a7e, SUC_lexicon_tags_23 },
    { 0x81807a7f, SUC_lexicon_tags_17 },
    { 0xc6a07a7f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0007a83, SUC_lexicon_tags_17 },
    { 0x81607a84, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x54c07a86, SUC_lexicon_tags_68 },
    { 0x48c07a86, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd3607a89, SUC_lexicon_tags_47 },
    { 0x1d807a8a, SUC_lexicon_tags_63 },
    { 0x5e207a8b, SUC_lexicon_tags_43 },
    { 0x27207a8a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2ca07a8e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x91e07a90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba07a93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78607a98, SUC_lexicon_tags_25 },
    { 0xb2407a99, SUC_lexicon_tags_31 },
    { 0xf2407a99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96007a9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46e07a9e, SUC_lexicon_tags_11 },
    { 0x8b807a9f, SUC_lexicon_tags_37 },
    { 0xc5a07aa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60607aa3, SUC_lexicon_tags_38 },
    { 0x26c07aa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d407aa6, SUC_lexicon_tags_25 },
    { 0x3ce07aa6, SUC_lexicon_tags_37 },
    { 0xbea07aa8, SUC_lexicon_tags_15 },
    { 0x42807aa8, SUC_lexicon_tags_14 },
    { 0xb2607aaa, SUC_lexicon_tags_25 },
    { 0xc7a07aab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaec07aad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c07ab1, SUC_lexicon_tags_30 },
    { 0x58607ab2, SUC_lexicon_tags_1 },
    { 0x21207ab3, SUC_lexicon_tags_37 },
    { 0xa3007ab4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a07ab7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e07aba, SUC_lexicon_tags_37 },
    { 0xb8607abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c07abf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xda407ac2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1407ac5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf207ac7, SUC_lexicon_tags_14 },
    { 0xe8e07ac8, SUC_lexicon_tags_11 },
    { 0xee607ac8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc6607acb, SUC_lexicon_tags_47 },
    { 0x25607acc, SUC_lexicon_tags_66 },
    { 0xa9207acd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x67007ad0, SUC_lexicon_tags_25 },
    { 0x75407ad0, SUC_lexicon_tags_15 },
    { 0x15407ad2, SUC_lexicon_tags_15 },
    { 0xd8207ad0, SUC_lexicon_tags_36 },
    { 0xc6607ad2, SUC_lexicon_tags_11 },
    { 0x47207ad5, SUC_lexicon_tags_17 },
    { 0xe4e07ad6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e807add, SUC_lexicon_tags_66 },
    { 0xf807add, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a07ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56407ae3, SUC_lexicon_tags_17 },
    { 0x1c407ae4, SUC_lexicon_tags_98 },
    { 0xa407ae5, SUC_lexicon_tags_37 },
    { 0xd8607ae6, SUC_lexicon_tags_31 },
    { 0x7e007ae4, SUC_lexicon_tags_0 },
    { 0x82c07ae7, SUC_lexicon_tags_49 },
    { 0x48807ae7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc07af0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5c407af2, SUC_lexicon_tags_26 },
    { 0x8cc07af3, SUC_lexicon_tags_17 },
    { 0x66207af4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x79807af7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x50207af9, SUC_lexicon_tags_2 },
    { 0xf9807af9, SUC_lexicon_tags_47 },
    { 0x95607afa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd0a07afd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x88a07aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4807b06, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf5607b08, SUC_lexicon_tags_1 },
    { 0x6e407b09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e07b0d, SUC_lexicon_tags_12 },
    { 0xf0e07b0e, SUC_lexicon_tags_15 },
    { 0xaf807b0f, SUC_lexicon_tags_25 },
    { 0x21c07b10, SUC_lexicon_tags_47 },
    { 0x8ea07b0e, SUC_lexicon_tags_1 },
    { 0x77207b12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e07b1c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4607b1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2dc07b21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9607b23, SUC_lexicon_tags_108 },
    { 0x52407b24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72607b29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaac07b2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x10e07b2d, SUC_lexicon_tags_25 },
    { 0x30007b2d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc07b31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c07b37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a407b39, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59007b3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f407b3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf407b46, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8207b4a, SUC_lexicon_tags_12 },
    { 0xc407b4b, SUC_lexicon_tags_17 },
    { 0x6807b4c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb9407b4e, SUC_lexicon_tags_38 },
    { 0x5b607b4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xba407b51, SUC_lexicon_tags_17 },
    { 0xf8807b52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xffe07b54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a07b57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64c07b59, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a07b5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x98807b5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a607b63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc6c07b65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea07b6d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4207b70, SUC_lexicon_tags_4 },
    { 0x96007b70, SUC_lexicon_tags_65 },
    { 0xb1a07b72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a07b76, SUC_lexicon_tags_63 },
    { 0x31807b77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3cc07b79, SUC_lexicon_tags_65 },
    { 0xbe607b7a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x84207b7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49807b7f, SUC_lexicon_tags_10 },
    { 0x93807b80, SUC_lexicon_tags_12 },
    { 0xd9407b7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c807b85, SUC_lexicon_tags_11 },
    { 0x2ae07b86, SUC_lexicon_tags_39 },
    { 0x99007b85, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xdcc07b89, SUC_lexicon_tags_23 },
    { 0x16e07b8a, SUC_lexicon_tags_16 },
    { 0xe8007b89, SUC_lexicon_tags_2 },
    { 0xf7007b8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e007b8f, SUC_lexicon_tags_37 },
    { 0xbd807b8f, SUC_lexicon_tags_1 },
    { 0x2c007b8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa1807b93, SUC_lexicon_tags_7 },
    { 0xa7807b94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a07b9d, SUC_lexicon_tags_114 },
    { 0x3ee07b9e, SUC_lexicon_tags_19 },
    { 0xec607b9e, SUC_lexicon_tags_25 },
    { 0x65607b9f, SUC_lexicon_tags_25 },
    { 0x83e07ba1, SUC_lexicon_tags_47 },
    { 0xb2e07ba2, SUC_lexicon_tags_25 },
    { 0x98807ba3, SUC_lexicon_tags_17 },
    { 0x15207ba1, SUC_lexicon_tags_12 },
    { 0xd5807ba0, SUC_lexicon_tags_47 },
    { 0x18a07ba3, SUC_lexicon_tags_1 },
    { 0x61607b9d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a207bab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82607bb4, SUC_lexicon_tags_12 },
    { 0x19a07bb5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e07bbe, SUC_lexicon_tags_87 },
    { 0x42c07bbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd607bc8, SUC_lexicon_tags_82 },
    { 0xfa207bc8, SUC_lexicon_tags_25 },
    { 0xc0e07bca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbd807bcc, SUC_lexicon_tags_44 },
    { 0xed407bcc, SUC_lexicon_tags_11 },
    { 0x9ec07bcd, SUC_lexicon_tags_17 },
    { 0x53e07bcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90207bd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca207bd3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec07bd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa07bdb, SUC_lexicon_tags_15 },
    { 0xc2c07bdc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xab007bde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15007be1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7807be7, SUC_lexicon_tags_9 },
    { 0xd3a07be8, SUC_lexicon_tags_25 },
    { 0xf5807be9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde07bed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81007bef, SUC_lexicon_tags_11 },
    { 0xcb607bef, SUC_lexicon_tags_0 },
    { 0x90e07bef, SUC_lexicon_tags_17 },
    { 0x2de07bef, SUC_lexicon_tags_11 },
    { 0xcf007bf1, SUC_lexicon_tags_49 },
    { 0x5ae07bf4, SUC_lexicon_tags_11 },
    { 0x35207bf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd607bf7, SUC_lexicon_tags_87 },
    { 0xc2807bf8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d207bfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72c07bfe, SUC_lexicon_tags_2 },
    { 0x45807bfe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x81807c01, SUC_lexicon_tags_59 },
    { 0x65607c01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xace07c04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa07c07, SUC_lexicon_tags_25 },
    { 0x48a07c08, SUC_lexicon_tags_77 },
    { 0x31807c08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e07c10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68407c14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8807c1a, SUC_lexicon_tags_25 },
    { 0xe1207c1b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x21207c1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a07c23, SUC_lexicon_tags_15 },
    { 0x16007c24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x59e07c26, SUC_lexicon_tags_12 },
    { 0x9dc07c26, SUC_lexicon_tags_6 },
    { 0x9c207c28, SUC_lexicon_tags_55 },
    { 0xa8e07c29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x74207c2c, SUC_lexicon_tags_0 },
    { 0xd0607c2d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d207c30, SUC_lexicon_tags_11 },
    { 0x66a07c30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a07c34, SUC_lexicon_tags_1 },
    { 0x8b007c35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99807c37, SUC_lexicon_tags_15 },
    { 0x2f007c37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e07c3d, SUC_lexicon_tags_56 },
    { 0x3a207c3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaac07c40, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e07c43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5607c45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65807c4b, SUC_lexicon_tags_1 },
    { 0x9b207c4b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85007c50, SUC_lexicon_tags_7 },
    { 0xb4007c51, SUC_lexicon_tags_31 },
    { 0x8da07c52, SUC_lexicon_tags_29 },
    { 0x58e07c51, SUC_lexicon_tags_66 },
    { 0x58007c51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4ee07c56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc207c58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38407c5f, SUC_lexicon_tags_45 },
    { 0xa1a07c60, SUC_lexicon_tags_12 },
    { 0x3407c61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73a07c63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16407c67, SUC_lexicon_tags_12 },
    { 0x16007c67, SUC_lexicon_tags_25 },
    { 0x5207c69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc3607c6b, SUC_lexicon_tags_23 },
    { 0x8e007c6b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x75007c6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59607c73, SUC_lexicon_tags_4 },
    { 0xd007c74, SUC_lexicon_tags_33 },
    { 0x1f207c75, SUC_lexicon_tags_11 },
    { 0x97807c76, SUC_lexicon_tags_15 },
    { 0xc5407c76, SUC_lexicon_tags_120 },
    { 0xd9807c75, SUC_lexicon_tags_37 },
    { 0x15e07c79, SUC_lexicon_tags_17 },
    { 0xf0007c7a, SUC_lexicon_tags_12 },
    { 0xda607c7b, SUC_lexicon_tags_31 },
    { 0xdaa07c78, SUC_lexicon_tags_3 },
    { 0x1fc07c78, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44807c81, SUC_lexicon_tags_37 },
    { 0x1e607c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54207c85, SUC_lexicon_tags_6 },
    { 0x3c607c86, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x71c07c88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2607c8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea07c90, SUC_lexicon_tags_7 },
    { 0x67607c91, SUC_lexicon_tags_12 },
    { 0x2ee07c92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e407c94, SUC_lexicon_tags_7 },
    { 0x8607c94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfb807c97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a07c9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10607ca0, SUC_lexicon_tags_81 },
    { 0x66407ca0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90a07ca3, SUC_lexicon_tags_25 },
    { 0x72c07ca4, SUC_lexicon_tags_2 },
    { 0x77a07ca5, SUC_lexicon_tags_33 },
    { 0x74c07ca5, SUC_lexicon_tags_2 },
    { 0xc3c07ca6, SUC_lexicon_tags_45 },
    { 0x2e07ca8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b407caa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x83007cad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f607cb1, SUC_lexicon_tags_47 },
    { 0xde007cb2, SUC_lexicon_tags_11 },
    { 0xbea07cb3, SUC_lexicon_tags_12 },
    { 0xfae07cb4, SUC_lexicon_tags_133 },
    { 0x2a807cb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e007cb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd007cbe, SUC_lexicon_tags_50 },
    { 0x1e07cbf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x55007cc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6607cc3, SUC_lexicon_tags_64 },
    { 0xaa807cc4, SUC_lexicon_tags_4 },
    { 0x27207cc5, SUC_lexicon_tags_7 },
    { 0xa4807cc5, SUC_lexicon_tags_32 },
    { 0xd7007cc3, SUC_lexicon_tags_15 },
    { 0x4f807cc8, SUC_lexicon_tags_12 },
    { 0x14e07cc9, SUC_lexicon_tags_82 },
    { 0x46407cca, SUC_lexicon_tags_11 },
    { 0x36407cca, SUC_lexicon_tags_37 },
    { 0x91207ccc, SUC_lexicon_tags_56 },
    { 0x45e07cc9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa407ccf, SUC_lexicon_tags_2 },
    { 0x63007ccf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a07cd4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80807cda, SUC_lexicon_tags_1 },
    { 0xc6a07cdb, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x3ac07cdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa007cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77007ce1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e807ce6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac07cea, SUC_lexicon_tags_37 },
    { 0x6bc07ceb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a07cf1, SUC_lexicon_tags_15 },
    { 0x9fe07cf2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12207cf6, SUC_lexicon_tags_45 },
    { 0x81807cf6, SUC_lexicon_tags_34 },
    { 0xe007cf7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39407cfb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf007cfd, SUC_lexicon_tags_9 },
    { 0xd8407cfe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b007d01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2807d03, SUC_lexicon_tags_65 },
    { 0xde807d04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3a207d06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xca207d08, SUC_lexicon_tags_1 },
    { 0xa5207d08, SUC_lexicon_tags_11 },
    { 0x3007d08, SUC_lexicon_tags_39 },
    { 0x2b407d09, SUC_lexicon_tags_2 },
    { 0x3ca07d0c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea07d10, SUC_lexicon_tags_31 },
    { 0x6f807d11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c07d18, SUC_lexicon_tags_2 },
    { 0x7a007d19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0807d1c, SUC_lexicon_tags_2 },
    { 0x48a07d1d, SUC_lexicon_tags_38 },
    { 0xa2207d1e, SUC_lexicon_tags_21 },
    { 0xd7607d1f, SUC_lexicon_tags_64 },
    { 0x5e207d1e, SUC_lexicon_tags_23 },
    { 0x67c07d1f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed807d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46607d2e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x71807d31, SUC_lexicon_tags_26 },
    { 0x94807d32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1607d34, SUC_lexicon_tags_37 },
    { 0x83c07d34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c607d38, SUC_lexicon_tags_45 },
    { 0xe4e07d39, SUC_lexicon_tags_68 },
    { 0x66c07d39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe07d3c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65807d42, SUC_lexicon_tags_10 },
    { 0xb807d42, SUC_lexicon_tags_11 },
    { 0x15407d44, SUC_lexicon_tags_2 },
    { 0x7ca07d42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17607d48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x77a07d4a, SUC_lexicon_tags_68 },
    { 0x5ee07d4b, SUC_lexicon_tags_12 },
    { 0x98407d4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7007d4f, SUC_lexicon_tags_7 },
    { 0xb5407d4f, SUC_lexicon_tags_25 },
    { 0x8ec07d51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa07d54, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45407d59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb807d5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e07d61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xda207d63, SUC_lexicon_tags_11 },
    { 0x4bc07d64, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc1e07d66, SUC_lexicon_tags_47 },
    { 0x56a07d67, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c607d6b, SUC_lexicon_tags_0 },
    { 0xc1807d6b, SUC_lexicon_tags_7 },
    { 0xdcc07d6d, SUC_lexicon_tags_70 },
    { 0x83407d6e, SUC_lexicon_tags_31 },
    { 0x87007d6f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c07d77, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1807d79, SUC_lexicon_tags_15 },
    { 0xc0007d7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0607d7d, SUC_lexicon_tags_17 },
    { 0x8607d7e, SUC_lexicon_tags_19 },
    { 0x4c607d7f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb2407d81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc07d85, SUC_lexicon_tags_1 },
    { 0x60e07d86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4807d89, SUC_lexicon_tags_11 },
    { 0xe2407d8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5807d8e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc807d91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9da07d93, SUC_lexicon_tags_60 },
    { 0xdd607d94, SUC_lexicon_tags_63 },
    { 0xf3407d94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe07d99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2e07d9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c207d9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0007da1, SUC_lexicon_tags_37 },
    { 0xd9e07da2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5007da6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c07da9, SUC_lexicon_tags_14 },
    { 0x6d607daa, SUC_lexicon_tags_17 },
    { 0xa1407dab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x14207dad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65407db6, SUC_lexicon_tags_47 },
    { 0xcc007db7, SUC_lexicon_tags_39 },
    { 0x5f007db8, SUC_lexicon_tags_17 },
    { 0x61c07db9, SUC_lexicon_tags_1 },
    { 0x74c07db8, SUC_lexicon_tags_1 },
    { 0xeb207dbb, SUC_lexicon_tags_45 },
    { 0x29407dbc, SUC_lexicon_tags_15 },
    { 0x91407dbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x93207dbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8807dc1, SUC_lexicon_tags_9 },
    { 0x61807dc2, SUC_lexicon_tags_25 },
    { 0xac07dc3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xede07dc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20a07dc7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a07dcb, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0x34807dcd, SUC_lexicon_tags_0 },
    { 0x39807dce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9207dd1, SUC_lexicon_tags_48 },
    { 0xa9c07dd2, SUC_lexicon_tags_15 },
    { 0xea407dd3, SUC_lexicon_tags_47 },
    { 0xa3007dd4, SUC_lexicon_tags_2 },
    { 0x8de07dd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba07ddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3607de1, SUC_lexicon_tags_118 },
    { 0x76607de2, SUC_lexicon_tags_26 },
    { 0x5ac07de3, SUC_lexicon_tags_15 },
    { 0xc9c07de4, SUC_lexicon_tags_15 },
    { 0xad207de5, SUC_lexicon_tags_23 },
    { 0xc8407de5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3607de9, SUC_lexicon_tags_7 },
    { 0xfd207dea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae07def, SUC_lexicon_tags_1 },
    { 0x9a807def, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67c07df2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e07df8, SUC_lexicon_tags_55 },
    { 0x1d807df8, SUC_lexicon_tags_17 },
    { 0x67607dfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ea07dfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98807e01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc607e03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e07e06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5007e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a807e0e, SUC_lexicon_tags_9 },
    { 0xafe07e0f, SUC_lexicon_tags_17 },
    { 0xdee07e10, SUC_lexicon_tags_28 },
    { 0x22607e10, SUC_lexicon_tags_47 },
    { 0x29607e11, SUC_lexicon_tags_63 },
    { 0xe9607e13, SUC_lexicon_tags_14 },
    { 0x25607e0e, SUC_lexicon_tags_11 },
    { 0x56407e15, SUC_lexicon_tags_11 },
    { 0xa9a07e12, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45407e1c, SUC_lexicon_tags_37 },
    { 0x96407e1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7a07e1f, SUC_lexicon_tags_91 },
    { 0xdee07e1f, SUC_lexicon_tags_29 },
    { 0xe1a07e21, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10407e26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57007e2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf407e2e, SUC_lexicon_tags_60 },
    { 0x28007e2f, SUC_lexicon_tags_47 },
    { 0x8c207e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c07e33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c07e37, SUC_lexicon_tags_60 },
    { 0x29007e38, SUC_lexicon_tags_7 },
    { 0x55c07e39, SUC_lexicon_tags_23 },
    { 0x16807e3a, SUC_lexicon_tags_30 },
    { 0xe2c07e39, SUC_lexicon_tags_2 },
    { 0x61c07e3a, SUC_lexicon_tags_2 },
    { 0xea807e3d, SUC_lexicon_tags_28 },
    { 0xd4e07e3e, SUC_lexicon_tags_7 },
    { 0x8d207e38, SUC_lexicon_tags_48 },
    { 0x76e07e3d, SUC_lexicon_tags_9 },
    { 0x2ce07e3c, SUC_lexicon_tags_15 },
    { 0x52207e42, SUC_lexicon_tags_5 },
    { 0x59607e43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6407e46, SUC_lexicon_tags_2 },
    { 0x97e07e47, SUC_lexicon_tags_11 },
    { 0xd3e07e47, SUC_lexicon_tags_49 },
    { 0x9c607e49, SUC_lexicon_tags_140 },
    { 0x85207e4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc07e4d, SUC_lexicon_tags_15 },
    { 0xf9807e4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43207e53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca407e58, SUC_lexicon_tags_17 },
    { 0xb3a07e58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee807e5b, SUC_lexicon_tags_182 },
    { 0xd0407e5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9bc07e5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e207e61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0007e64, SUC_lexicon_tags_48 },
    { 0x67007e64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3007e68, SUC_lexicon_tags_12 },
    { 0x6207e69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c07e6c, SUC_lexicon_tags_31 },
    { 0xc3007e6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48007e6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae07e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77607e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fc07e7f, SUC_lexicon_tags_2 },
    { 0x88207e80, SUC_lexicon_tags_7 },
    { 0x99407e7f, SUC_lexicon_tags_9 },
    { 0xdf607e80, SUC_lexicon_tags_1 },
    { 0x14607e83, SUC_lexicon_tags_14 },
    { 0x41207e81, SUC_lexicon_tags_39 },
    { 0xdba07e84, SUC_lexicon_tags_37 },
    { 0x1cc07e7f, SUC_lexicon_tags_25 },
    { 0xe6c07e87, SUC_lexicon_tags_25 },
    { 0x58c07e83, SUC_lexicon_tags_37 },
    { 0x75c07e89, SUC_lexicon_tags_10 },
    { 0x7a607e8a, SUC_lexicon_tags_77 },
    { 0x18c07e8b, SUC_lexicon_tags_65 },
    { 0x1dc07e7f, SUC_lexicon_tags_9 },
    { 0x44007e87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9007e91, SUC_lexicon_tags_87 },
    { 0x38607e91, SUC_lexicon_tags_11 },
    { 0x69c07e93, SUC_lexicon_tags_7 },
    { 0xaf807e91, SUC_lexicon_tags_12 },
    { 0xf7207e95, SUC_lexicon_tags_29 },
    { 0x44407e95, SUC_lexicon_tags_17 },
    { 0x87607e94, SUC_lexicon_tags_47 },
    { 0xe5607e93, SUC_lexicon_tags_10 },
    { 0xf9a07e99, SUC_lexicon_tags_2 },
    { 0x7da07e9a, SUC_lexicon_tags_29 },
    { 0x7b007e9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da07e9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe0a07ea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c07ea3, SUC_lexicon_tags_43 },
    { 0xc9c07ea3, SUC_lexicon_tags_1 },
    { 0x47607ea5, SUC_lexicon_tags_31 },
    { 0xfa007ea6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4c007ea8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x28807eaa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30407eae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x71807eb0, SUC_lexicon_tags_17 },
    { 0xddc07eb0, SUC_lexicon_tags_29 },
    { 0x1fa07eb2, SUC_lexicon_tags_7 },
    { 0xe8a07eb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b007eb6, SUC_lexicon_tags_11 },
    { 0x39007eb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4207ebf, SUC_lexicon_tags_6 },
    { 0x98207ec0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca07ec4, SUC_lexicon_tags_2 },
    { 0x42207ec5, SUC_lexicon_tags_36 },
    { 0x4b007ec6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6207eca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b007ece, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74807ed0, SUC_lexicon_tags_28 },
    { 0xfca07ed0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a07ed3, SUC_lexicon_tags_53 },
    { 0x7fc07ed4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0607ed8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31007edd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16207ee5, SUC_lexicon_tags_9 },
    { 0xe807ee5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e07eea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8407eec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2ec07eee, SUC_lexicon_tags_2 },
    { 0x95e07eef, SUC_lexicon_tags_7 },
    { 0x4c07ef0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c007ef5, SUC_lexicon_tags_43 },
    { 0x67007ef6, SUC_lexicon_tags_7 },
    { 0x31e07ef7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbd007ef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde407f02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f407f07, SUC_lexicon_tags_47 },
    { 0x91407f07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88607f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0e07f0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e207f13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa207f18, SUC_lexicon_tags_12 },
    { 0xc8c07f18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b807f1f, SUC_lexicon_tags_37 },
    { 0x7c407f20, SUC_lexicon_tags_30 },
    { 0xa3007f21, SUC_lexicon_tags_11 },
    { 0xabc07f22, SUC_lexicon_tags_29 },
    { 0x66007f23, SUC_lexicon_tags_21 },
    { 0xb807f20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13207f27, SUC_lexicon_tags_31 },
    { 0xcb807f28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c07f2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4207f31, SUC_lexicon_tags_37 },
    { 0xe1807f31, SUC_lexicon_tags_11 },
    { 0x7ca07f32, SUC_lexicon_tags_31 },
    { 0xc8007f34, SUC_lexicon_tags_43 },
    { 0x38207f35, SUC_lexicon_tags_47 },
    { 0xf3007f35, SUC_lexicon_tags_13 },
    { 0xb807f34, SUC_lexicon_tags_16 },
    { 0x55e07f38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa07f3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46007f43, SUC_lexicon_tags_66 },
    { 0x65e07f43, SUC_lexicon_tags_17 },
    { 0xc5607f45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc207f48, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3207f4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9807f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc407f51, SUC_lexicon_tags_0 },
    { 0x20407f51, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x39007f54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79807f56, SUC_lexicon_tags_40 },
    { 0x40c07f57, SUC_lexicon_tags_31 },
    { 0x84607f56, SUC_lexicon_tags_46 },
    { 0xf8807f59, SUC_lexicon_tags_33 },
    { 0x8da07f57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26807f5c, SUC_lexicon_tags_2 },
    { 0x87607f5c, SUC_lexicon_tags_37 },
    { 0x13607f5d, SUC_lexicon_tags_551 },
    { 0x44a07f5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda007f65, SUC_lexicon_tags_2 },
    { 0x2e07f65, SUC_lexicon_tags_37 },
    { 0x80e07f67, SUC_lexicon_tags_12 },
    { 0xf2607f68, SUC_lexicon_tags_9 },
    { 0x98e07f68, SUC_lexicon_tags_27 },
    { 0x3a207f69, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9207f74, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc1e07f76, SUC_lexicon_tags_23 },
    { 0x89c07f76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xacc07f79, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b807f7e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c07f81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58207f85, SUC_lexicon_tags_37 },
    { 0x2f807f85, SUC_lexicon_tags_33 },
    { 0xbaa07f86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27407f8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf407f8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d207f96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1c07f98, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf407f9c, SUC_lexicon_tags_65 },
    { 0x60e07f9d, SUC_lexicon_tags_15 },
    { 0xfb807f9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad807fa2, SUC_lexicon_tags_11 },
    { 0xf1c07fa3, SUC_lexicon_tags_26 },
    { 0x98807fa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70407fa7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a07faa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7b207fac, SUC_lexicon_tags_29 },
    { 0x48407fad, SUC_lexicon_tags_0 },
    { 0x56a07fae, SUC_lexicon_tags_7 },
    { 0x1fe07fad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c07fb4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de07fb7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x16607fb9, SUC_lexicon_tags_31 },
    { 0x89a07fb9, SUC_lexicon_tags_4 },
    { 0xbf207fbb, SUC_lexicon_tags_15 },
    { 0x1d607fbb, SUC_lexicon_tags_15 },
    { 0xc5e07fbd, SUC_lexicon_tags_15 },
    { 0xe0007fbb, SUC_lexicon_tags_30 },
    { 0xf5a07fba, SUC_lexicon_tags_1 },
    { 0x81007fbe, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3407fc4, SUC_lexicon_tags_29 },
    { 0xd3607fc5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xed807fc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8607fc9, SUC_lexicon_tags_106 },
    { 0x46c07fca, SUC_lexicon_tags_14 },
    { 0xad007fca, SUC_lexicon_tags_47 },
    { 0x9a807fcc, SUC_lexicon_tags_49 },
    { 0xdb207fcd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e07fd1, SUC_lexicon_tags_9 },
    { 0x38e07fd2, SUC_lexicon_tags_25 },
    { 0x2d207fd3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da07fd8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c07fe0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41607fe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c07fe8, SUC_lexicon_tags_25 },
    { 0x89807fe9, SUC_lexicon_tags_11 },
    { 0x74c07fea, SUC_lexicon_tags_9 },
    { 0xff607feb, SUC_lexicon_tags_11 },
    { 0xe007feb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb5007fee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67407ff6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f007ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6607ffe, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x4208000, SUC_lexicon_tags_23 },
    { 0xcfc08000, SUC_lexicon_tags_38 },
    { 0x1a008002, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a08006, SUC_lexicon_tags_30 },
    { 0x66e08006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880800f, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10408015, SUC_lexicon_tags_43 },
    { 0x87008016, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1260801c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1e0801e, SUC_lexicon_tags_68 },
    { 0xda20801f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb608021, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97608024, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea08027, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29608032, SUC_lexicon_tags_37 },
    { 0x6ec08033, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a008035, SUC_lexicon_tags_11 },
    { 0x54008035, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13408045, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8608047, SUC_lexicon_tags_23 },
    { 0x6e608047, SUC_lexicon_tags_37 },
    { 0x12008047, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c0804d, SUC_lexicon_tags_23 },
    { 0xa740804d, SUC_lexicon_tags_107 },
    { 0x84e0804f, SUC_lexicon_tags_25 },
    { 0x8100804d, SUC_lexicon_tags_15 },
    { 0xe7008051, SUC_lexicon_tags_154 },
    { 0x9ae08052, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d808055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67208065, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4520806c, SUC_lexicon_tags_9 },
    { 0xd500806d, SUC_lexicon_tags_18 },
    { 0x420806d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1408071, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc08076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba08078, SUC_lexicon_tags_405 },
    { 0xc4608078, SUC_lexicon_tags_2 },
    { 0xf5208078, SUC_lexicon_tags_38 },
    { 0x4e00807b, SUC_lexicon_tags_7 },
    { 0xdf40807c, SUC_lexicon_tags_70 },
    { 0x3b20807d, SUC_lexicon_tags_1 },
    { 0x2400807e, SUC_lexicon_tags_46 },
    { 0x2700807d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x11808082, SUC_lexicon_tags_63 },
    { 0xf3208082, SUC_lexicon_tags_57 },
    { 0xf0e08082, SUC_lexicon_tags_0 },
    { 0x31008083, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x91808087, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea60808a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e40808e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb608091, SUC_lexicon_tags_37 },
    { 0xc7e08092, SUC_lexicon_tags_11 },
    { 0xe2e08093, SUC_lexicon_tags_37 },
    { 0x2a08094, SUC_lexicon_tags_35 },
    { 0xb0808095, SUC_lexicon_tags_49 },
    { 0x33208096, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c0809b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5240809f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc080a4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe76080a7, SUC_lexicon_tags_91 },
    { 0xe92080a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2da080aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8080ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x168080b0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbae080b2, SUC_lexicon_tags_35 },
    { 0x98080b2, SUC_lexicon_tags_12 },
    { 0xb52080b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6080b7, SUC_lexicon_tags_2 },
    { 0xbd8080b7, SUC_lexicon_tags_40 },
    { 0xf7a080b9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78080bd, SUC_lexicon_tags_17 },
    { 0x546080bd, SUC_lexicon_tags_0 },
    { 0x44a080bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2080c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6080cc, SUC_lexicon_tags_28 },
    { 0x5b6080cd, SUC_lexicon_tags_29 },
    { 0x352080ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2080d2, SUC_lexicon_tags_0 },
    { 0x566080d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x42080d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8080db, SUC_lexicon_tags_25 },
    { 0xa9e080dc, SUC_lexicon_tags_42 },
    { 0x95c080dd, SUC_lexicon_tags_84 },
    { 0xb88080de, SUC_lexicon_tags_1 },
    { 0xfc4080df, SUC_lexicon_tags_30 },
    { 0xc9c080df, SUC_lexicon_tags_16 },
    { 0xad8080e1, SUC_lexicon_tags_37 },
    { 0x3a6080e2, SUC_lexicon_tags_7 },
    { 0x94a080dd, SUC_lexicon_tags_28 },
    { 0xb68080dd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x226080e7, SUC_lexicon_tags_17 },
    { 0xb12080e7, SUC_lexicon_tags_27 },
    { 0x582080e9, SUC_lexicon_tags_1 },
    { 0x860080ea, SUC_lexicon_tags_48 },
    { 0x982080e8, SUC_lexicon_tags_68 },
    { 0xef2080ec, SUC_lexicon_tags_38 },
    { 0xa8e080e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x568080f0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8080f6, SUC_lexicon_tags_47 },
    { 0x164080f6, SUC_lexicon_tags_2 },
    { 0xc7e080f7, SUC_lexicon_tags_17 },
    { 0x7f4080f9, SUC_lexicon_tags_7 },
    { 0x8a0080f6, SUC_lexicon_tags_39 },
    { 0xd66080fa, SUC_lexicon_tags_63 },
    { 0x992080fc, SUC_lexicon_tags_65 },
    { 0xe10080fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc08102, SUC_lexicon_tags_37 },
    { 0x78408103, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53608109, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c08110, SUC_lexicon_tags_27 },
    { 0x79408110, SUC_lexicon_tags_15 },
    { 0x7c08111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee208119, SUC_lexicon_tags_11 },
    { 0x16208119, SUC_lexicon_tags_37 },
    { 0x4560811a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b80811f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0008121, SUC_lexicon_tags_7 },
    { 0xb3a08121, SUC_lexicon_tags_25 },
    { 0x14808123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce08127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb860812f, SUC_lexicon_tags_15 },
    { 0x79008130, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce08135, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3008137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15208139, SUC_lexicon_tags_37 },
    { 0x64408139, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22408142, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a08146, SUC_lexicon_tags_3 },
    { 0xf0008147, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe40814e, SUC_lexicon_tags_37 },
    { 0x6a20814f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c08153, SUC_lexicon_tags_38 },
    { 0xcc808153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a08156, SUC_lexicon_tags_11 },
    { 0x77408156, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3e008159, SUC_lexicon_tags_29 },
    { 0x6ba08159, SUC_lexicon_tags_19 },
    { 0xf9a0815b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca0815f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20408165, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de0816f, SUC_lexicon_tags_15 },
    { 0xb8808170, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a08179, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc0817e, SUC_lexicon_tags_33 },
    { 0xe4e0817f, SUC_lexicon_tags_27 },
    { 0x5840817e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6008182, SUC_lexicon_tags_6 },
    { 0x79008182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6808187, SUC_lexicon_tags_17 },
    { 0x9ca08188, SUC_lexicon_tags_31 },
    { 0xe7408189, SUC_lexicon_tags_47 },
    { 0x68c08188, SUC_lexicon_tags_43 },
    { 0xfca0818b, SUC_lexicon_tags_17 },
    { 0x1480818c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4740818e, SUC_lexicon_tags_28 },
    { 0xc600818f, SUC_lexicon_tags_60 },
    { 0x9e208190, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ac08192, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe08195, SUC_lexicon_tags_17 },
    { 0x6e808196, SUC_lexicon_tags_2 },
    { 0xbf208196, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26c08199, SUC_lexicon_tags_15 },
    { 0x8120819a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb40819d, SUC_lexicon_tags_23 },
    { 0x1460819d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x290081a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912081a5, SUC_lexicon_tags_186 },
    { 0xdda081a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8081aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x732081ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6dc081af, SUC_lexicon_tags_12 },
    { 0xbc6081af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc081b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9a081b5, SUC_lexicon_tags_1 },
    { 0xec8081b5, SUC_lexicon_tags_11 },
    { 0x6c0081b7, SUC_lexicon_tags_37 },
    { 0xb32081b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x48081ba, SUC_lexicon_tags_37 },
    { 0xd36081bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf4081bd, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0081c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd6081c7, SUC_lexicon_tags_15 },
    { 0x832081c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x946081ca, SUC_lexicon_tags_7 },
    { 0x972081ca, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be081ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x404081d0, SUC_lexicon_tags_28 },
    { 0x176081d1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd16081d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x400081d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0081db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c2081dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb10081df, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe82081e1, SUC_lexicon_tags_17 },
    { 0x32081e2, SUC_lexicon_tags_45 },
    { 0x320081e2, SUC_lexicon_tags_25 },
    { 0x740081e4, SUC_lexicon_tags_0 },
    { 0xee6081e5, SUC_lexicon_tags_1 },
    { 0xb08081e6, SUC_lexicon_tags_113 },
    { 0xac0081e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x82081e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda081ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684081f0, SUC_lexicon_tags_35 },
    { 0xc4a081f1, SUC_lexicon_tags_12 },
    { 0xca081f2, SUC_lexicon_tags_31 },
    { 0xc22081f3, SUC_lexicon_tags_17 },
    { 0xe06081f4, SUC_lexicon_tags_25 },
    { 0xb96081f3, SUC_lexicon_tags_37 },
    { 0x86e081f3, SUC_lexicon_tags_27 },
    { 0xfc6081f3, SUC_lexicon_tags_2 },
    { 0xf10081f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14081fb, SUC_lexicon_tags_63 },
    { 0xc3e081fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71808203, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea0820d, SUC_lexicon_tags_6 },
    { 0x6c0820d, SUC_lexicon_tags_38 },
    { 0xcc20820e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c08212, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf808215, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e08218, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe3e0821a, SUC_lexicon_tags_17 },
    { 0xf4c0821b, SUC_lexicon_tags_12 },
    { 0x4200821c, SUC_lexicon_tags_31 },
    { 0xba20821d, SUC_lexicon_tags_6 },
    { 0x620821a, SUC_lexicon_tags_20 },
    { 0x2a00821f, SUC_lexicon_tags_25 },
    { 0xf140821e, SUC_lexicon_tags_9 },
    { 0x5b208221, SUC_lexicon_tags_35 },
    { 0xa4a08221, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf0c08224, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91008226, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40822a, SUC_lexicon_tags_37 },
    { 0xcac0822a, SUC_lexicon_tags_15 },
    { 0xe7a0822c, SUC_lexicon_tags_47 },
    { 0x84a0822c, SUC_lexicon_tags_12 },
    { 0xf400822e, SUC_lexicon_tags_7 },
    { 0xa00822d, SUC_lexicon_tags_17 },
    { 0x8320822d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d608233, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x55608236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cc08238, SUC_lexicon_tags_17 },
    { 0x23c08239, SUC_lexicon_tags_19 },
    { 0x7fa0823a, SUC_lexicon_tags_57 },
    { 0x20823b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb00823f, SUC_lexicon_tags_19 },
    { 0xe4808240, SUC_lexicon_tags_138 },
    { 0xf5a08241, SUC_lexicon_tags_26 },
    { 0x2e08241, SUC_lexicon_tags_30 },
    { 0x4a208242, SUC_lexicon_tags_74 },
    { 0x9008241, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46808246, SUC_lexicon_tags_2 },
    { 0xff008246, SUC_lexicon_tags_14 },
    { 0x7ae08248, SUC_lexicon_tags_17 },
    { 0xd2e08249, SUC_lexicon_tags_25 },
    { 0x3a808249, SUC_lexicon_tags_37 },
    { 0x9100824a, SUC_lexicon_tags_46 },
    { 0x16c0824c, SUC_lexicon_tags_27 },
    { 0xf7e08246, SUC_lexicon_tags_15 },
    { 0x88c0824a, SUC_lexicon_tags_7 },
    { 0xf5e0824d, SUC_lexicon_tags_8 },
    { 0x91a0824e, SUC_lexicon_tags_15 },
    { 0x3120824d, SUC_lexicon_tags_0 },
    { 0xc608252, SUC_lexicon_tags_11 },
    { 0xa7208253, SUC_lexicon_tags_26 },
    { 0x52408252, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf808256, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0825a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8008261, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1aa08263, SUC_lexicon_tags_15 },
    { 0xcdc08264, SUC_lexicon_tags_55 },
    { 0xe0208264, SUC_lexicon_tags_1 },
    { 0x10208266, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda408268, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe600826c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8880826e, SUC_lexicon_tags_37 },
    { 0xa100826f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c0827b, SUC_lexicon_tags_38 },
    { 0xc240827c, SUC_lexicon_tags_36 },
    { 0x7b40827b, SUC_lexicon_tags_0 },
    { 0x8ea0827b, SUC_lexicon_tags_11 },
    { 0x7980827c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97e08281, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e08284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a08287, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46c08289, SUC_lexicon_tags_2 },
    { 0x73808289, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa860828d, SUC_lexicon_tags_28 },
    { 0xe840828e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84208292, SUC_lexicon_tags_30 },
    { 0x6f808293, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99608299, SUC_lexicon_tags_5 },
    { 0x4880829a, SUC_lexicon_tags_43 },
    { 0x8740829b, SUC_lexicon_tags_27 },
    { 0xc9e0829c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12082a6, SUC_lexicon_tags_37 },
    { 0x768082a7, SUC_lexicon_tags_15 },
    { 0x1a8082a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9da082aa, SUC_lexicon_tags_11 },
    { 0xb02082ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x84082ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd42082af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x170082b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118082b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47c082b8, SUC_lexicon_tags_7 },
    { 0xf9c082b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd94082bb, SUC_lexicon_tags_7 },
    { 0x7a0082bc, SUC_lexicon_tags_29 },
    { 0x552082bc, SUC_lexicon_tags_37 },
    { 0xdca082bd, SUC_lexicon_tags_30 },
    { 0x7e4082bf, SUC_lexicon_tags_26 },
    { 0x7c8082c0, SUC_lexicon_tags_43 },
    { 0x1fc082be, SUC_lexicon_tags_2 },
    { 0x3a0082c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6ca082c4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc082c9, SUC_lexicon_tags_17 },
    { 0xcd6082c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c082ce, SUC_lexicon_tags_1 },
    { 0x1a4082cf, SUC_lexicon_tags_1 },
    { 0xf84082ce, SUC_lexicon_tags_47 },
    { 0xddc082ce, SUC_lexicon_tags_17 },
    { 0xea8082ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa082da, SUC_lexicon_tags_38 },
    { 0xdea082db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c082e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4082e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4082e6, SUC_lexicon_tags_12 },
    { 0x80e082e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2082ec, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x844082ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36082f1, SUC_lexicon_tags_0 },
    { 0x50c082f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x426082f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2082f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4082fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19808301, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac08305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe960830b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7900830d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c08315, SUC_lexicon_tags_37 },
    { 0x71a08315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46808318, SUC_lexicon_tags_15 },
    { 0xc2408319, SUC_lexicon_tags_27 },
    { 0xe5a0831a, SUC_lexicon_tags_2 },
    { 0xce40831b, SUC_lexicon_tags_82 },
    { 0xe3c0831b, SUC_lexicon_tags_37 },
    { 0x4120831b, SUC_lexicon_tags_23 },
    { 0x2ea08319, SUC_lexicon_tags_12 },
    { 0x4e20831d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79808323, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42208325, SUC_lexicon_tags_16 },
    { 0xe6e08326, SUC_lexicon_tags_27 },
    { 0x92a08327, SUC_lexicon_tags_20 },
    { 0xbcc08328, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbb40832a, SUC_lexicon_tags_165 },
    { 0xf7c0832a, SUC_lexicon_tags_60 },
    { 0x8680832c, SUC_lexicon_tags_4 },
    { 0x7f00832d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7008330, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c808333, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffe08335, SUC_lexicon_tags_25 },
    { 0xf4608335, SUC_lexicon_tags_23 },
    { 0xd7408337, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c0833a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad40833f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33008346, SUC_lexicon_tags_6 },
    { 0x63808347, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xd4808349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe000834c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4800834e, SUC_lexicon_tags_11 },
    { 0xb320834f, SUC_lexicon_tags_15 },
    { 0x39208350, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c808354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e0835c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fe0835e, SUC_lexicon_tags_37 },
    { 0xe280835e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x11808361, SUC_lexicon_tags_11 },
    { 0xdca08361, SUC_lexicon_tags_19 },
    { 0xa0408362, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x87208365, SUC_lexicon_tags_12 },
    { 0xcfe08366, SUC_lexicon_tags_14 },
    { 0x99208366, SUC_lexicon_tags_2 },
    { 0xc8608368, SUC_lexicon_tags_7 },
    { 0x97608369, SUC_lexicon_tags_7 },
    { 0x90e08368, SUC_lexicon_tags_38 },
    { 0x7640836b, SUC_lexicon_tags_28 },
    { 0x17008369, SUC_lexicon_tags_19 },
    { 0xca808369, SUC_lexicon_tags_7 },
    { 0xbba0836e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31208374, SUC_lexicon_tags_32 },
    { 0xb8808375, SUC_lexicon_tags_28 },
    { 0x61e08376, SUC_lexicon_tags_16 },
    { 0x69e08377, SUC_lexicon_tags_39 },
    { 0xdd408374, SUC_lexicon_tags_11 },
    { 0xf5008379, SUC_lexicon_tags_17 },
    { 0xcc20837a, SUC_lexicon_tags_37 },
    { 0x33e08375, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee08380, SUC_lexicon_tags_19 },
    { 0x1ce08381, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d808386, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a0838a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c20838d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc0839b, SUC_lexicon_tags_27 },
    { 0xaaa0839c, SUC_lexicon_tags_12 },
    { 0xbc00839d, SUC_lexicon_tags_2 },
    { 0xf080839e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10083a1, SUC_lexicon_tags_25 },
    { 0x5a6083a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e083a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x866083aa, SUC_lexicon_tags_2 },
    { 0xbe4083ab, SUC_lexicon_tags_12 },
    { 0xa36083ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc083af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x71a083b1, SUC_lexicon_tags_8 },
    { 0xa22083b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x694083b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x808083b6, SUC_lexicon_tags_14 },
    { 0xe5c083b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ee083b9, SUC_lexicon_tags_151 },
    { 0xaf6083ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796083be, SUC_lexicon_tags_10 },
    { 0x296083bf, SUC_lexicon_tags_31 },
    { 0x30a083bf, SUC_lexicon_tags_33 },
    { 0x9c4083c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2083c4, SUC_lexicon_tags_2 },
    { 0xa6e083c5, SUC_lexicon_tags_6 },
    { 0xf8083c6, SUC_lexicon_tags_1 },
    { 0x3a083c5, SUC_lexicon_tags_55 },
    { 0x574083c5, SUC_lexicon_tags_16 },
    { 0x4f4083c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a083ce, SUC_lexicon_tags_31 },
    { 0xa86083cf, SUC_lexicon_tags_26 },
    { 0xae4083d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x84083d2, SUC_lexicon_tags_27 },
    { 0xeae083d3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762083d7, SUC_lexicon_tags_38 },
    { 0xe06083d8, SUC_lexicon_tags_8 },
    { 0x4ec083d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90e083db, SUC_lexicon_tags_23 },
    { 0x4c8083dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x636083de, SUC_lexicon_tags_37 },
    { 0x986083de, SUC_lexicon_tags_14 },
    { 0x316083df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a0083e2, SUC_lexicon_tags_82 },
    { 0xbe2083e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6083e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7e083eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de083ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c083f3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x144083f6, SUC_lexicon_tags_15 },
    { 0xf1a083f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc083ff, SUC_lexicon_tags_33 },
    { 0x4c0083ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6608404, SUC_lexicon_tags_37 },
    { 0x20808404, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa008407, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2b808409, SUC_lexicon_tags_7 },
    { 0xf020840a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb140840e, SUC_lexicon_tags_2 },
    { 0xd140840e, SUC_lexicon_tags_15 },
    { 0x8d008410, SUC_lexicon_tags_25 },
    { 0x25008411, SUC_lexicon_tags_18 },
    { 0xea08412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50a08414, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x33408417, SUC_lexicon_tags_47 },
    { 0xd8208417, SUC_lexicon_tags_17 },
    { 0x4cc08419, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc0841d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a08421, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xca408424, SUC_lexicon_tags_37 },
    { 0xb8e08425, SUC_lexicon_tags_43 },
    { 0x33608426, SUC_lexicon_tags_7 },
    { 0xb008427, SUC_lexicon_tags_9 },
    { 0xa2008428, SUC_lexicon_tags_3 },
    { 0x2fc08428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7480842e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf9e08430, SUC_lexicon_tags_11 },
    { 0x9e808431, SUC_lexicon_tags_41 },
    { 0x3e608430, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5408436, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6a208438, SUC_lexicon_tags_37 },
    { 0xe9a08439, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc140843b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1560843f, SUC_lexicon_tags_23 },
    { 0x4d808440, SUC_lexicon_tags_1 },
    { 0x9c008441, SUC_lexicon_tags_7 },
    { 0x1e008442, SUC_lexicon_tags_45 },
    { 0x7780843f, SUC_lexicon_tags_1 },
    { 0x7fc08443, SUC_lexicon_tags_28 },
    { 0x2dc08442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4808448, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a0844d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f208454, SUC_lexicon_tags_12 },
    { 0x8ee08454, SUC_lexicon_tags_28 },
    { 0x41808455, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97e08458, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xc440845b, SUC_lexicon_tags_26 },
    { 0xbec0845c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2ea0845e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaf208460, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba08466, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda0846b, SUC_lexicon_tags_17 },
    { 0xe6a0846c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3e0846e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4408470, SUC_lexicon_tags_12 },
    { 0xff408470, SUC_lexicon_tags_17 },
    { 0x96808471, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce08476, SUC_lexicon_tags_17 },
    { 0x18408477, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6f408479, SUC_lexicon_tags_31 },
    { 0xda008479, SUC_lexicon_tags_15 },
    { 0x1760847b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da08482, SUC_lexicon_tags_55 },
    { 0x25008483, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9608485, SUC_lexicon_tags_1 },
    { 0xa7008485, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee20848a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d40848d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x89e0848f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c08494, SUC_lexicon_tags_6 },
    { 0x31c08495, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1080849a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8500849c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e00849f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b2084a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20084ab, SUC_lexicon_tags_11 },
    { 0xa00084ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b6084ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd9e084b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00084b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0084b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc084b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0084bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8e084be, SUC_lexicon_tags_38 },
    { 0x968084bf, SUC_lexicon_tags_2 },
    { 0xa4084be, SUC_lexicon_tags_2 },
    { 0xbf6084c0, SUC_lexicon_tags_11 },
    { 0x2ec084c2, SUC_lexicon_tags_1 },
    { 0xf38084c1, SUC_lexicon_tags_12 },
    { 0x36e084c4, SUC_lexicon_tags_25 },
    { 0x41e084c4, SUC_lexicon_tags_25 },
    { 0x946084c5, SUC_lexicon_tags_11 },
    { 0xf2084c4, SUC_lexicon_tags_14 },
    { 0x1fa084c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x136084ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xef8084cc, SUC_lexicon_tags_15 },
    { 0x944084cd, SUC_lexicon_tags_32 },
    { 0x63c084cc, SUC_lexicon_tags_9 },
    { 0x7b4084cf, SUC_lexicon_tags_1 },
    { 0xf04084cf, SUC_lexicon_tags_12 },
    { 0xeba084d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e084dc, SUC_lexicon_tags_7 },
    { 0xbac084dc, SUC_lexicon_tags_37 },
    { 0xcee084de, SUC_lexicon_tags_17 },
    { 0xf4084de, SUC_lexicon_tags_47 },
    { 0x130084df, SUC_lexicon_tags_17 },
    { 0x65e084e1, SUC_lexicon_tags_23 },
    { 0xece084de, SUC_lexicon_tags_37 },
    { 0x35e084e1, SUC_lexicon_tags_2 },
    { 0x8084e4, SUC_lexicon_tags_30 },
    { 0x9a6084e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c2084e7, SUC_lexicon_tags_11 },
    { 0xe68084e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88084ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae084ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20084f5, SUC_lexicon_tags_48 },
    { 0xb26084f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358084fa, SUC_lexicon_tags_12 },
    { 0xa26084fb, SUC_lexicon_tags_33 },
    { 0xe6e084fc, SUC_lexicon_tags_11 },
    { 0xfd4084fd, SUC_lexicon_tags_11 },
    { 0xea0084fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e08502, SUC_lexicon_tags_11 },
    { 0xf3608503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c208506, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca08509, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb900850b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x8460850e, SUC_lexicon_tags_46 },
    { 0x2a80850e, SUC_lexicon_tags_23 },
    { 0x13808510, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1808519, SUC_lexicon_tags_29 },
    { 0x3ec0851a, SUC_lexicon_tags_1 },
    { 0x3320851a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc00851d, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87608521, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea08526, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb060852d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca08533, SUC_lexicon_tags_32 },
    { 0xa2408534, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf6408536, SUC_lexicon_tags_17 },
    { 0x8a208537, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb808539, SUC_lexicon_tags_6 },
    { 0xdee08539, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7340853c, SUC_lexicon_tags_7 },
    { 0x1680853d, SUC_lexicon_tags_7 },
    { 0x6420853d, SUC_lexicon_tags_12 },
    { 0xcf00853f, SUC_lexicon_tags_3 },
    { 0xf4408540, SUC_lexicon_tags_48 },
    { 0xee408541, SUC_lexicon_tags_3 },
    { 0x8e608541, SUC_lexicon_tags_26 },
    { 0x7a608543, SUC_lexicon_tags_53 },
    { 0x1dc08542, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c08549, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40854f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1ca08551, SUC_lexicon_tags_0 },
    { 0x40a08552, SUC_lexicon_tags_33 },
    { 0x75208553, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86208559, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740855d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa260856a, SUC_lexicon_tags_15 },
    { 0x59a0856b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a0856e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae08571, SUC_lexicon_tags_57 },
    { 0x25208571, SUC_lexicon_tags_25 },
    { 0xdd008573, SUC_lexicon_tags_65 },
    { 0xc2a08573, SUC_lexicon_tags_25 },
    { 0x8e08572, SUC_lexicon_tags_26 },
    { 0xbae08573, SUC_lexicon_tags_7 },
    { 0xd1e08577, SUC_lexicon_tags_37 },
    { 0xcb808578, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5820857e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x88208581, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb408584, SUC_lexicon_tags_63 },
    { 0xdb608585, SUC_lexicon_tags_43 },
    { 0x7fe08584, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e0858b, SUC_lexicon_tags_40 },
    { 0x3500858c, SUC_lexicon_tags_26 },
    { 0xc1e0858c, SUC_lexicon_tags_1 },
    { 0x46e0858d, SUC_lexicon_tags_17 },
    { 0xdd80858d, SUC_lexicon_tags_1 },
    { 0xeae08590, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa0608592, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c08595, SUC_lexicon_tags_62 },
    { 0x50008596, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c08599, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf60859e, SUC_lexicon_tags_14 },
    { 0x6640859f, SUC_lexicon_tags_2 },
    { 0x95a085a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7c085a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2085a5, SUC_lexicon_tags_9 },
    { 0x78085a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ba085a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20085ab, SUC_lexicon_tags_12 },
    { 0xdda085ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfde085ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x994085b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a085b6, SUC_lexicon_tags_105 },
    { 0xadc085b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf2085b9, SUC_lexicon_tags_1 },
    { 0xe32085ba, SUC_lexicon_tags_27 },
    { 0xb16085b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x128085bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98085c1, SUC_lexicon_tags_49 },
    { 0x552085c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb2085c4, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a085c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4085cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21e085cf, SUC_lexicon_tags_28 },
    { 0x946085d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd96085d2, SUC_lexicon_tags_12 },
    { 0x1c085d3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452085dd, SUC_lexicon_tags_34 },
    { 0x4ce085dd, SUC_lexicon_tags_1 },
    { 0x12a085df, SUC_lexicon_tags_33 },
    { 0x45a085e0, SUC_lexicon_tags_15 },
    { 0xc32085e1, SUC_lexicon_tags_11 },
    { 0x95c085e0, SUC_lexicon_tags_8 },
    { 0xaca085e3, SUC_lexicon_tags_17 },
    { 0x4ea085e3, SUC_lexicon_tags_35 },
    { 0x328085e5, SUC_lexicon_tags_37 },
    { 0x1e0085e5, SUC_lexicon_tags_63 },
    { 0x7e6085e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0085eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3da085ed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x17c085ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a085f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e085f9, SUC_lexicon_tags_60 },
    { 0x92a085f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab208600, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1608609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1040860c, SUC_lexicon_tags_34 },
    { 0x1cc0860d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6420860f, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a08613, SUC_lexicon_tags_66 },
    { 0x77608614, SUC_lexicon_tags_15 },
    { 0x4b008615, SUC_lexicon_tags_7 },
    { 0x7b208616, SUC_lexicon_tags_23 },
    { 0xdac08615, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea0861c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcb20861e, SUC_lexicon_tags_15 },
    { 0xc560861f, SUC_lexicon_tags_23 },
    { 0x95e08620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86608623, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf700862c, SUC_lexicon_tags_11 },
    { 0x74a0862d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e08632, SUC_lexicon_tags_0 },
    { 0x76608632, SUC_lexicon_tags_10 },
    { 0xa6608633, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3900863a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e0863e, SUC_lexicon_tags_66 },
    { 0x8400863f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e08644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80a08646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27608648, SUC_lexicon_tags_45 },
    { 0x21208649, SUC_lexicon_tags_42 },
    { 0x8a0864a, SUC_lexicon_tags_65 },
    { 0xe6a0864a, SUC_lexicon_tags_70 },
    { 0x8c40864a, SUC_lexicon_tags_23 },
    { 0x6a40864d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d008652, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xae008655, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e08658, SUC_lexicon_tags_28 },
    { 0x8b008659, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bc0865b, SUC_lexicon_tags_29 },
    { 0xe000865b, SUC_lexicon_tags_35 },
    { 0x4ca0865d, SUC_lexicon_tags_13 },
    { 0xc1c0865e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40208660, SUC_lexicon_tags_27 },
    { 0xe8208661, SUC_lexicon_tags_92 },
    { 0x15a08662, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea08668, SUC_lexicon_tags_1 },
    { 0xa3208668, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8dc0866b, SUC_lexicon_tags_47 },
    { 0x6380866b, SUC_lexicon_tags_37 },
    { 0x43c0866d, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c808676, SUC_lexicon_tags_60 },
    { 0x59408677, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3208679, SUC_lexicon_tags_37 },
    { 0x74c0867a, SUC_lexicon_tags_12 },
    { 0x7dc0867a, SUC_lexicon_tags_87 },
    { 0x4fc0867b, SUC_lexicon_tags_87 },
    { 0xda20867b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f008682, SUC_lexicon_tags_0 },
    { 0x85008682, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c0868c, SUC_lexicon_tags_25 },
    { 0x92a0868d, SUC_lexicon_tags_7 },
    { 0x2e00868e, SUC_lexicon_tags_15 },
    { 0xca60868d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae08696, SUC_lexicon_tags_48 },
    { 0x60008697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12408699, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3800869b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662086a8, SUC_lexicon_tags_23 },
    { 0x424086a9, SUC_lexicon_tags_33 },
    { 0xda8086aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60086ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a086b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc086b3, SUC_lexicon_tags_15 },
    { 0xa12086b4, SUC_lexicon_tags_1 },
    { 0x3ea086b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514086be, SUC_lexicon_tags_17 },
    { 0x1fc086bf, SUC_lexicon_tags_49 },
    { 0x75a086c0, SUC_lexicon_tags_7 },
    { 0x91c086c1, SUC_lexicon_tags_2 },
    { 0x2b4086c2, SUC_lexicon_tags_33 },
    { 0x3ec086bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598086c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42086cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8086d0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd72086d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x396086d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ea086d7, SUC_lexicon_tags_11 },
    { 0x52e086d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e086dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406086e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae086e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f8086e6, SUC_lexicon_tags_1 },
    { 0xaac086e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240086ec, SUC_lexicon_tags_43 },
    { 0x230086ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6086f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc86086f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce086fd, SUC_lexicon_tags_0 },
    { 0xb2c086fe, SUC_lexicon_tags_19 },
    { 0x724086ff, SUC_lexicon_tags_25 },
    { 0xa3a086fd, SUC_lexicon_tags_58 },
    { 0x2d608701, SUC_lexicon_tags_17 },
    { 0x62808701, SUC_lexicon_tags_37 },
    { 0x21408701, SUC_lexicon_tags_30 },
    { 0x8e608704, SUC_lexicon_tags_63 },
    { 0x41a08705, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7280870b, SUC_lexicon_tags_15 },
    { 0x5b20870c, SUC_lexicon_tags_32 },
    { 0x7100870d, SUC_lexicon_tags_25 },
    { 0xb9e0870b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3808711, SUC_lexicon_tags_11 },
    { 0x76208711, SUC_lexicon_tags_12 },
    { 0xb3808713, SUC_lexicon_tags_37 },
    { 0x3ec08711, SUC_lexicon_tags_17 },
    { 0xea208715, SUC_lexicon_tags_7 },
    { 0x6dc08716, SUC_lexicon_tags_17 },
    { 0x18a08713, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed60871d, SUC_lexicon_tags_6 },
    { 0x5ca0871e, SUC_lexicon_tags_12 },
    { 0xb680871f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b008722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3808725, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa920872c, SUC_lexicon_tags_37 },
    { 0xd4a0872d, SUC_lexicon_tags_37 },
    { 0xad40872c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac408732, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27208737, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e0873d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6408749, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a0874c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7840874f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36608751, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be08754, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa808756, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf808758, SUC_lexicon_tags_43 },
    { 0x2fa08759, SUC_lexicon_tags_48 },
    { 0x7808758, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd580875c, SUC_lexicon_tags_26 },
    { 0x5540875d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e08762, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a808767, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25a08769, SUC_lexicon_tags_11 },
    { 0xd060876a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e0876d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29808770, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa608773, SUC_lexicon_tags_6 },
    { 0x5d208773, SUC_lexicon_tags_25 },
    { 0x13608775, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x63a08777, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd000877b, SUC_lexicon_tags_26 },
    { 0xa8a0877b, SUC_lexicon_tags_45 },
    { 0xd700877c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d40877f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e008785, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xece08787, SUC_lexicon_tags_8 },
    { 0x37408787, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6660878b, SUC_lexicon_tags_8 },
    { 0xcc00878c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e60878e, SUC_lexicon_tags_25 },
    { 0xed40878f, SUC_lexicon_tags_60 },
    { 0x7540878e, SUC_lexicon_tags_1 },
    { 0x3ca0878f, SUC_lexicon_tags_27 },
    { 0x29608792, SUC_lexicon_tags_14 },
    { 0x41608792, SUC_lexicon_tags_17 },
    { 0x15208793, SUC_lexicon_tags_0 },
    { 0xfea08793, SUC_lexicon_tags_17 },
    { 0x77408790, SUC_lexicon_tags_25 },
    { 0x88e08797, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0879a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba0879f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29e087a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c087a5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x968087a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x674087aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8087af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf30087b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf20087b3, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x408087b5, SUC_lexicon_tags_37 },
    { 0x79a087b5, SUC_lexicon_tags_7 },
    { 0x4d2087b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x100087ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22087be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd8087c0, SUC_lexicon_tags_49 },
    { 0x472087c1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2087c4, SUC_lexicon_tags_7 },
    { 0x752087c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x778087c7, SUC_lexicon_tags_19 },
    { 0xa4c087c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de087cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a087d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x714087d4, SUC_lexicon_tags_15 },
    { 0x20c087d5, SUC_lexicon_tags_68 },
    { 0xc58087d6, SUC_lexicon_tags_1 },
    { 0x20a087d6, SUC_lexicon_tags_29 },
    { 0xf40087d6, SUC_lexicon_tags_11 },
    { 0x778087d6, SUC_lexicon_tags_22 },
    { 0x16c087da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe90087dc, SUC_lexicon_tags_35 },
    { 0x34e087dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a087e2, SUC_lexicon_tags_11 },
    { 0x466087e3, SUC_lexicon_tags_43 },
    { 0x3da087e4, SUC_lexicon_tags_37 },
    { 0x2dc087e4, SUC_lexicon_tags_39 },
    { 0xab2087e6, SUC_lexicon_tags_49 },
    { 0x70a087e6, SUC_lexicon_tags_25 },
    { 0x1bc087e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e087ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6087f1, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2087f4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4087f7, SUC_lexicon_tags_11 },
    { 0xc4c087f7, SUC_lexicon_tags_23 },
    { 0x482087f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8087fe, SUC_lexicon_tags_8 },
    { 0xbb2087ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x27408801, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8208807, SUC_lexicon_tags_27 },
    { 0x30a08808, SUC_lexicon_tags_14 },
    { 0xb0808809, SUC_lexicon_tags_0 },
    { 0x9ae0880a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1720880c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0880f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec808815, SUC_lexicon_tags_25 },
    { 0x17408816, SUC_lexicon_tags_11 },
    { 0x82008817, SUC_lexicon_tags_33 },
    { 0x39008818, SUC_lexicon_tags_45 },
    { 0xb4408819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa640881c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1008820, SUC_lexicon_tags_2 },
    { 0x4d208821, SUC_lexicon_tags_27 },
    { 0x67208822, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x18608824, SUC_lexicon_tags_1 },
    { 0x22808825, SUC_lexicon_tags_7 },
    { 0xb5e08824, SUC_lexicon_tags_8 },
    { 0x9808827, SUC_lexicon_tags_14 },
    { 0xb5c08826, SUC_lexicon_tags_37 },
    { 0x61c08829, SUC_lexicon_tags_1 },
    { 0x1840882a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e0882d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48a0882f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x33808831, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e408835, SUC_lexicon_tags_38 },
    { 0xc3408835, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf40883c, SUC_lexicon_tags_12 },
    { 0x2d60883c, SUC_lexicon_tags_1 },
    { 0x18a0883e, SUC_lexicon_tags_11 },
    { 0x3e20883f, SUC_lexicon_tags_1 },
    { 0xf9008840, SUC_lexicon_tags_38 },
    { 0x27808840, SUC_lexicon_tags_38 },
    { 0x2ca08840, SUC_lexicon_tags_25 },
    { 0xdbe08843, SUC_lexicon_tags_7 },
    { 0x7500883f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3c08846, SUC_lexicon_tags_63 },
    { 0x3a408847, SUC_lexicon_tags_25 },
    { 0x9008848, SUC_lexicon_tags_70 },
    { 0xbf008846, SUC_lexicon_tags_63 },
    { 0x2640884a, SUC_lexicon_tags_23 },
    { 0xcc40884b, SUC_lexicon_tags_56 },
    { 0x8be0884c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84408857, SUC_lexicon_tags_17 },
    { 0x41e08858, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7300885f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4808866, SUC_lexicon_tags_37 },
    { 0x84208867, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd40886a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a08870, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e08874, SUC_lexicon_tags_63 },
    { 0x3ce08875, SUC_lexicon_tags_38 },
    { 0xf5208876, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x86408878, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xef00887b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9320887f, SUC_lexicon_tags_60 },
    { 0xfa08880, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8ee08882, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c08886, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0888d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb808890, SUC_lexicon_tags_42 },
    { 0x79808890, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x37608893, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca0889b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889f, SUC_lexicon_tags_262 },
    { 0x7440889f, SUC_lexicon_tags_10 },
    { 0x240088a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a088a5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee088a8, SUC_lexicon_tags_6 },
    { 0x156088a9, SUC_lexicon_tags_2 },
    { 0x2a2088aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02088ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x456088b0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be088b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a088b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16088c1, SUC_lexicon_tags_25 },
    { 0xbac088c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40088cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x638088cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc088d0, SUC_lexicon_tags_15 },
    { 0xa32088d0, SUC_lexicon_tags_17 },
    { 0x24c088d2, SUC_lexicon_tags_12 },
    { 0xd52088d3, SUC_lexicon_tags_82 },
    { 0xe08088d2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x122088d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48088dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b2088de, SUC_lexicon_tags_28 },
    { 0xc0088df, SUC_lexicon_tags_2 },
    { 0xe30088de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc6088e2, SUC_lexicon_tags_12 },
    { 0xd4088e2, SUC_lexicon_tags_25 },
    { 0xc74088e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4c088e6, SUC_lexicon_tags_12 },
    { 0x5a0088e7, SUC_lexicon_tags_37 },
    { 0x6fa088e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c088ee, SUC_lexicon_tags_10 },
    { 0x206088ef, SUC_lexicon_tags_14 },
    { 0xc4e088ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78088f5, SUC_lexicon_tags_12 },
    { 0x3ba088f6, SUC_lexicon_tags_2 },
    { 0x88088f7, SUC_lexicon_tags_2 },
    { 0xd06088f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x984088fb, SUC_lexicon_tags_17 },
    { 0x3d4088fc, SUC_lexicon_tags_84 },
    { 0x320088fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c208906, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a0890a, SUC_lexicon_tags_37 },
    { 0x3ca0890b, SUC_lexicon_tags_3 },
    { 0x4e40890c, SUC_lexicon_tags_68 },
    { 0xda80890b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x32608910, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe0808912, SUC_lexicon_tags_37 },
    { 0x41a08913, SUC_lexicon_tags_21 },
    { 0xb0e08913, SUC_lexicon_tags_19 },
    { 0xbc808913, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb620891b, SUC_lexicon_tags_26 },
    { 0x4d40891c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3008920, SUC_lexicon_tags_37 },
    { 0x36e08921, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f208923, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x72208926, SUC_lexicon_tags_45 },
    { 0xe008927, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1280892f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57e08931, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd608936, SUC_lexicon_tags_15 },
    { 0x8e208937, SUC_lexicon_tags_12 },
    { 0xdba08936, SUC_lexicon_tags_12 },
    { 0xd3608937, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420893e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22808944, SUC_lexicon_tags_30 },
    { 0xb0608945, SUC_lexicon_tags_55 },
    { 0x5a08946, SUC_lexicon_tags_6 },
    { 0x48608947, SUC_lexicon_tags_11 },
    { 0x2e008945, SUC_lexicon_tags_14 },
    { 0x1ba08947, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a60894b, SUC_lexicon_tags_43 },
    { 0x9c60894c, SUC_lexicon_tags_5 },
    { 0x2000894d, SUC_lexicon_tags_19 },
    { 0x3c40894d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7da08950, SUC_lexicon_tags_17 },
    { 0xd8a08951, SUC_lexicon_tags_31 },
    { 0xc6008950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab208956, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcc608958, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da0895b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41608961, SUC_lexicon_tags_54 },
    { 0xe2c08962, SUC_lexicon_tags_63 },
    { 0xfda08961, SUC_lexicon_tags_47 },
    { 0xa7408964, SUC_lexicon_tags_63 },
    { 0xffe08961, SUC_lexicon_tags_1 },
    { 0xcf208966, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x46c08968, SUC_lexicon_tags_37 },
    { 0x3e208968, SUC_lexicon_tags_39 },
    { 0x75c0896a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf40896e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35408971, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf008975, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e08978, SUC_lexicon_tags_25 },
    { 0x11808979, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0897d, SUC_lexicon_tags_45 },
    { 0x2580897d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1808981, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8008985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a0898a, SUC_lexicon_tags_39 },
    { 0xa140898a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe240898e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1808991, SUC_lexicon_tags_30 },
    { 0xd0008991, SUC_lexicon_tags_46 },
    { 0x29c08993, SUC_lexicon_tags_77 },
    { 0x3f408994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5a08996, SUC_lexicon_tags_65 },
    { 0x84a08997, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5180899b, SUC_lexicon_tags_11 },
    { 0x3e80899c, SUC_lexicon_tags_10 },
    { 0xcee0899d, SUC_lexicon_tags_11 },
    { 0xeb20899e, SUC_lexicon_tags_37 },
    { 0xd60899f, SUC_lexicon_tags_19 },
    { 0x5700899e, SUC_lexicon_tags_1084 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c089aa, SUC_lexicon_tags_3 },
    { 0x1aa089ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a089ae, SUC_lexicon_tags_133 },
    { 0xb3a089af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0089b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa0089b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x496089b9, SUC_lexicon_tags_32 },
    { 0x63e089ba, SUC_lexicon_tags_220 },
    { 0x27c089bb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd82089bd, SUC_lexicon_tags_32 },
    { 0xa3a089be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616089c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6089d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6089d5, SUC_lexicon_tags_29 },
    { 0xcb0089d6, SUC_lexicon_tags_74 },
    { 0xd12089d6, SUC_lexicon_tags_35 },
    { 0x4b4089d8, SUC_lexicon_tags_37 },
    { 0xb52089d8, SUC_lexicon_tags_37 },
    { 0x26a089d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe089dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae2089df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d2089e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8089ea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a089ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee089f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c089f9, SUC_lexicon_tags_37 },
    { 0x31a089f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2089fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc408a01, SUC_lexicon_tags_28 },
    { 0xfc608a02, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6c608a04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81a08a06, SUC_lexicon_tags_11 },
    { 0x94a08a07, SUC_lexicon_tags_27 },
    { 0xc408a08, SUC_lexicon_tags_37 },
    { 0x40608a09, SUC_lexicon_tags_25 },
    { 0xd6808a0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15408a0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bc08a0e, SUC_lexicon_tags_30 },
    { 0x4ac08a0f, SUC_lexicon_tags_11 },
    { 0x62608a10, SUC_lexicon_tags_27 },
    { 0xf7208a10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93e08a13, SUC_lexicon_tags_10 },
    { 0xc1008a13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d808a1b, SUC_lexicon_tags_29 },
    { 0x6b408a1c, SUC_lexicon_tags_66 },
    { 0xe7e08a1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6c08a1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0608a21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a08a25, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2808a28, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca08a2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c08a2e, SUC_lexicon_tags_11 },
    { 0xa0608a2e, SUC_lexicon_tags_17 },
    { 0xfce08a30, SUC_lexicon_tags_47 },
    { 0x84808a30, SUC_lexicon_tags_17 },
    { 0x69c08a32, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a08a36, SUC_lexicon_tags_39 },
    { 0x3ac08a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92008a3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c08a3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf008a42, SUC_lexicon_tags_15 },
    { 0x7da08a42, SUC_lexicon_tags_46 },
    { 0x31808a44, SUC_lexicon_tags_37 },
    { 0xfe008a45, SUC_lexicon_tags_17 },
    { 0xa3808a45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf808a49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec08a50, SUC_lexicon_tags_14 },
    { 0x14208a50, SUC_lexicon_tags_85 },
    { 0xf6408a52, SUC_lexicon_tags_43 },
    { 0xa2e08a53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27a08a55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b608a57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9808a5b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb208a65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d208a6c, SUC_lexicon_tags_27 },
    { 0xad408a6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a08a70, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x50608a72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c08a7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac08a80, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x71208a82, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6208a85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e08a88, SUC_lexicon_tags_46 },
    { 0xd6408a89, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x18c08a8b, SUC_lexicon_tags_23 },
    { 0xa4c08a8c, SUC_lexicon_tags_2 },
    { 0xa1808a8d, SUC_lexicon_tags_43 },
    { 0x9408a8e, SUC_lexicon_tags_23 },
    { 0x96c08a8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69208a92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46608a97, SUC_lexicon_tags_63 },
    { 0xee08a98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51808a9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f008aa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e08aa5, SUC_lexicon_tags_49 },
    { 0x82a08aa6, SUC_lexicon_tags_235 },
    { 0x30008aa7, SUC_lexicon_tags_9 },
    { 0x5d008aa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfec08aaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ea08aac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6008ab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19008ab3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xad208ab5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x96c08ab7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba808ac1, SUC_lexicon_tags_25 },
    { 0x61a08ac2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32208ac6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec408ac8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x74e08aca, SUC_lexicon_tags_47 },
    { 0x22408acb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73e08acd, SUC_lexicon_tags_11 },
    { 0x14808acd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80a08ad0, SUC_lexicon_tags_1 },
    { 0xe1808ad0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd2008ad3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68608ad6, SUC_lexicon_tags_30 },
    { 0xd6e08ad6, SUC_lexicon_tags_27 },
    { 0xe5808ad8, SUC_lexicon_tags_1 },
    { 0x76008ad7, SUC_lexicon_tags_10 },
    { 0x29608ad6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc08ade, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5e08ae0, SUC_lexicon_tags_31 },
    { 0xcfc08ae1, SUC_lexicon_tags_84 },
    { 0xe3c08ae1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda608ae4, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10208ae9, SUC_lexicon_tags_48 },
    { 0xa7c08aea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a08aed, SUC_lexicon_tags_37 },
    { 0x45408aed, SUC_lexicon_tags_17 },
    { 0x26208aef, SUC_lexicon_tags_17 },
    { 0xf8008aef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e08af6, SUC_lexicon_tags_37 },
    { 0xfce08af7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76208afa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31208b07, SUC_lexicon_tags_7 },
    { 0x8a608b07, SUC_lexicon_tags_31 },
    { 0x64008b09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a08b0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85a08b10, SUC_lexicon_tags_3 },
    { 0x8208b10, SUC_lexicon_tags_65 },
    { 0xe1a08b12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51208b19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a08b1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7408b25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e808b28, SUC_lexicon_tags_11 },
    { 0xab208b29, SUC_lexicon_tags_7 },
    { 0x9da08b29, SUC_lexicon_tags_1 },
    { 0x6ea08b29, SUC_lexicon_tags_27 },
    { 0xc9a08b2c, SUC_lexicon_tags_26 },
    { 0x9dc08b2a, SUC_lexicon_tags_37 },
    { 0x85e08b2e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e08b34, SUC_lexicon_tags_7 },
    { 0x24208b35, SUC_lexicon_tags_15 },
    { 0x9b808b36, SUC_lexicon_tags_7 },
    { 0x98608b36, SUC_lexicon_tags_8 },
    { 0x46c08b38, SUC_lexicon_tags_7 },
    { 0xa1208b39, SUC_lexicon_tags_11 },
    { 0xef408b39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6e08b3c, SUC_lexicon_tags_37 },
    { 0x9d008b3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x79408b40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e408b42, SUC_lexicon_tags_37 },
    { 0x8ce08b42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19008b45, SUC_lexicon_tags_11 },
    { 0xa6208b46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e608b49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0808b50, SUC_lexicon_tags_66 },
    { 0x42208b51, SUC_lexicon_tags_39 },
    { 0x8de08b51, SUC_lexicon_tags_17 },
    { 0xd1808b53, SUC_lexicon_tags_37 },
    { 0xc0a08b53, SUC_lexicon_tags_33 },
    { 0x16408b55, SUC_lexicon_tags_25 },
    { 0xa3208b55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a08b5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e608b61, SUC_lexicon_tags_12 },
    { 0xe0208b61, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e08b65, SUC_lexicon_tags_33 },
    { 0x1aa08b66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb6e08b68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x99008b6a, SUC_lexicon_tags_11 },
    { 0x49808b6b, SUC_lexicon_tags_11 },
    { 0xe3608b6c, SUC_lexicon_tags_26 },
    { 0xa9a08b6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60808b70, SUC_lexicon_tags_49 },
    { 0x3e008b71, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x28808b73, SUC_lexicon_tags_11 },
    { 0x83208b74, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5608b78, SUC_lexicon_tags_1 },
    { 0xcc208b79, SUC_lexicon_tags_12 },
    { 0x8c008b78, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xfaa08b7c, SUC_lexicon_tags_15 },
    { 0xe9808b7d, SUC_lexicon_tags_17 },
    { 0xa5c08b7e, SUC_lexicon_tags_28 },
    { 0x66408b7f, SUC_lexicon_tags_23 },
    { 0xe4408b80, SUC_lexicon_tags_1 },
    { 0xe2808b7f, SUC_lexicon_tags_37 },
    { 0xab408b80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a08b86, SUC_lexicon_tags_17 },
    { 0xf1608b87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a208b89, SUC_lexicon_tags_37 },
    { 0x82a08b89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64e08b8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86408b8f, SUC_lexicon_tags_7 },
    { 0x9208b90, SUC_lexicon_tags_11 },
    { 0xbf608b8f, SUC_lexicon_tags_11 },
    { 0x58408b90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47408b94, SUC_lexicon_tags_7 },
    { 0x70e08b95, SUC_lexicon_tags_16 },
    { 0x6ea08b95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa08b98, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc08b9b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7208b9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e08ba2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c08ba6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a08baf, SUC_lexicon_tags_3 },
    { 0x6de08bb0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d408bb2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a08bb8, SUC_lexicon_tags_3 },
    { 0x91008bb9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x13208bbc, SUC_lexicon_tags_25 },
    { 0xe6a08bbc, SUC_lexicon_tags_17 },
    { 0xb9e08bbe, SUC_lexicon_tags_60 },
    { 0xbb608bbe, SUC_lexicon_tags_25 },
    { 0x4f808bbf, SUC_lexicon_tags_46 },
    { 0x41008bbe, SUC_lexicon_tags_35 },
    { 0x53608bbe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc9608bc4, SUC_lexicon_tags_21 },
    { 0xd3e08bc5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d608bca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8008bcc, SUC_lexicon_tags_87 },
    { 0x98408bcd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc208bd0, SUC_lexicon_tags_2 },
    { 0xd1008bd1, SUC_lexicon_tags_11 },
    { 0xb6208bd2, SUC_lexicon_tags_11 },
    { 0xadc08bd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a08bd6, SUC_lexicon_tags_11 },
    { 0x98e08bd7, SUC_lexicon_tags_7 },
    { 0x20c08bd8, SUC_lexicon_tags_37 },
    { 0x35c08bd9, SUC_lexicon_tags_2 },
    { 0xab808bda, SUC_lexicon_tags_66 },
    { 0x7d608bd7, SUC_lexicon_tags_43 },
    { 0x2cc08bdc, SUC_lexicon_tags_48 },
    { 0xad808bdd, SUC_lexicon_tags_1 },
    { 0xcc208bde, SUC_lexicon_tags_1 },
    { 0x42208bdd, SUC_lexicon_tags_37 },
    { 0xc6208bde, SUC_lexicon_tags_51 },
    { 0xcda08be1, SUC_lexicon_tags_62 },
    { 0x81c08be2, SUC_lexicon_tags_19 },
    { 0xc408bd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b808be5, SUC_lexicon_tags_30 },
    { 0x29e08be6, SUC_lexicon_tags_25 },
    { 0x27408be6, SUC_lexicon_tags_55 },
    { 0x30e08be8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd808bef, SUC_lexicon_tags_17 },
    { 0xfd008bef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e808bf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e08c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47208c03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae808c06, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13008c12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c08c14, SUC_lexicon_tags_7 },
    { 0xa6a08c15, SUC_lexicon_tags_27 },
    { 0xa6e08c16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e08c1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xca808c1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80208c2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c08c2d, SUC_lexicon_tags_39 },
    { 0x7be08c2e, SUC_lexicon_tags_2 },
    { 0xefa08c2e, SUC_lexicon_tags_7 },
    { 0xbe408c2e, SUC_lexicon_tags_33 },
    { 0x73608c31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ee08c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14208c37, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82208c3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45408c3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7fa08c40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86608c42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac808c44, SUC_lexicon_tags_33 },
    { 0xe7408c45, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9ba08c47, SUC_lexicon_tags_48 },
    { 0x76608c47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50208c4f, SUC_lexicon_tags_17 },
    { 0xd2a08c4f, SUC_lexicon_tags_47 },
    { 0xc8808c51, SUC_lexicon_tags_33 },
    { 0xb208c4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56608c55, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7008c5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73208c5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc08c68, SUC_lexicon_tags_28 },
    { 0x63608c69, SUC_lexicon_tags_8 },
    { 0x88e08c68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca08c6e, SUC_lexicon_tags_37 },
    { 0x80c08c6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb408c73, SUC_lexicon_tags_1 },
    { 0xdaa08c74, SUC_lexicon_tags_47 },
    { 0xf7e08c75, SUC_lexicon_tags_37 },
    { 0xe208c76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93208c7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeba08c7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c08c83, SUC_lexicon_tags_8 },
    { 0xc8608c84, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x89008c86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3808c88, SUC_lexicon_tags_27 },
    { 0x8008c89, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43008c8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c08c97, SUC_lexicon_tags_17 },
    { 0x43808c98, SUC_lexicon_tags_27 },
    { 0xa3408c99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3408c9b, SUC_lexicon_tags_23 },
    { 0xebe08c9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a08c9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d408cab, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba08caf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c08cb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7008cb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86c08cb7, SUC_lexicon_tags_37 },
    { 0x4b808cb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95808cba, SUC_lexicon_tags_47 },
    { 0x92a08cbb, SUC_lexicon_tags_32 },
    { 0xc5808cbc, SUC_lexicon_tags_25 },
    { 0x34208cbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e08cc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe208cc4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7c08cc6, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0x3ae08cc8, SUC_lexicon_tags_2 },
    { 0x98808cc9, SUC_lexicon_tags_7 },
    { 0x8c08cca, SUC_lexicon_tags_116 },
    { 0x55608cc8, SUC_lexicon_tags_17 },
    { 0xd008ccc, SUC_lexicon_tags_0 },
    { 0xa4008ccd, SUC_lexicon_tags_4 },
    { 0xe3608cce, SUC_lexicon_tags_49 },
    { 0x9a808cc8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe08cd2, SUC_lexicon_tags_77 },
    { 0xd4808cd3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68808cd7, SUC_lexicon_tags_19 },
    { 0xdc208cd8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe08cdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1408cdd, SUC_lexicon_tags_23 },
    { 0xca808cde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5008ce0, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9208ce4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49c08ce6, SUC_lexicon_tags_15 },
    { 0x50608ce7, SUC_lexicon_tags_12 },
    { 0x59a08ce7, SUC_lexicon_tags_47 },
    { 0x52808ce9, SUC_lexicon_tags_12 },
    { 0xfc808ce9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe5208cec, SUC_lexicon_tags_25 },
    { 0x95408ced, SUC_lexicon_tags_2 },
    { 0x7cc08cee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b208cf0, SUC_lexicon_tags_37 },
    { 0x1e208cf1, SUC_lexicon_tags_31 },
    { 0xc1408cf2, SUC_lexicon_tags_48 },
    { 0xf9608cf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ae08cf5, SUC_lexicon_tags_65 },
    { 0xeca08cf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cc08cf8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa2e08cfa, SUC_lexicon_tags_6 },
    { 0x3d808cfb, SUC_lexicon_tags_37 },
    { 0xfa008cfc, SUC_lexicon_tags_17 },
    { 0x2f008cfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a408d03, SUC_lexicon_tags_37 },
    { 0x73608d04, SUC_lexicon_tags_65 },
    { 0x66408d04, SUC_lexicon_tags_2 },
    { 0x7d008d06, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c08d0c, SUC_lexicon_tags_47 },
    { 0x98808d0d, SUC_lexicon_tags_19 },
    { 0x25208d0d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e08d12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a608d18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c08d1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6208d20, SUC_lexicon_tags_15 },
    { 0x88608d21, SUC_lexicon_tags_11 },
    { 0x92808d21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb608d25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43c08d27, SUC_lexicon_tags_49 },
    { 0xfee08d27, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a08d34, SUC_lexicon_tags_25 },
    { 0xb2e08d34, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e208d39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2808d3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe08d44, SUC_lexicon_tags_15 },
    { 0x4e608d44, SUC_lexicon_tags_37 },
    { 0x76608d46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x12408d49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5de08d4b, SUC_lexicon_tags_7 },
    { 0x11a08d4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32c08d4e, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0xcb008d50, SUC_lexicon_tags_25 },
    { 0x97208d50, SUC_lexicon_tags_69 },
    { 0x38608d52, SUC_lexicon_tags_23 },
    { 0xefe08d50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd608d56, SUC_lexicon_tags_17 },
    { 0x56c08d57, SUC_lexicon_tags_33 },
    { 0x8b208d57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e08d5e, SUC_lexicon_tags_28 },
    { 0x11a08d5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2408d65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a08d69, SUC_lexicon_tags_12 },
    { 0x84c08d6a, SUC_lexicon_tags_60 },
    { 0x80c08d6b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x54c08d6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6008d70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27408d73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2408d76, SUC_lexicon_tags_23 },
    { 0x9e408d77, SUC_lexicon_tags_15 },
    { 0x38a08d76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde808d81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x70008d84, SUC_lexicon_tags_17 },
    { 0x9f008d85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28208d89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78a08d8b, SUC_lexicon_tags_13 },
    { 0x16a08d8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2408d8e, SUC_lexicon_tags_45 },
    { 0xfe008d8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe408d92, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe08d96, SUC_lexicon_tags_23 },
    { 0xc1208d97, SUC_lexicon_tags_27 },
    { 0x75608d98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3208d9f, SUC_lexicon_tags_2 },
    { 0x53a08da0, SUC_lexicon_tags_60 },
    { 0x74008da1, SUC_lexicon_tags_63 },
    { 0x64408da2, SUC_lexicon_tags_15 },
    { 0x3dc08d9f, SUC_lexicon_tags_7 },
    { 0x68e08da4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c08daa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa808dac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c408dae, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a408db2, SUC_lexicon_tags_17 },
    { 0xb4608db2, SUC_lexicon_tags_191 },
    { 0x5de08db3, SUC_lexicon_tags_21 },
    { 0x17208db5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe608db7, SUC_lexicon_tags_11 },
    { 0xfa408db8, SUC_lexicon_tags_19 },
    { 0x41a08db9, SUC_lexicon_tags_1 },
    { 0xeb408dba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c408dbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe08dc1, SUC_lexicon_tags_37 },
    { 0xe3e08dc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67008dc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc08dcc, SUC_lexicon_tags_63 },
    { 0xade08dcd, SUC_lexicon_tags_11 },
    { 0xd3208dce, SUC_lexicon_tags_17 },
    { 0x78608dcf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace08dd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c08dd7, SUC_lexicon_tags_38 },
    { 0xbf808dd8, SUC_lexicon_tags_7 },
    { 0x6d608dd9, SUC_lexicon_tags_17 },
    { 0xddc08dd7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x97a08ddc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74808dde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e08dec, SUC_lexicon_tags_26 },
    { 0xc5408ded, SUC_lexicon_tags_14 },
    { 0x1c208ded, SUC_lexicon_tags_23 },
    { 0x42808def, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9008df5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12808dfb, SUC_lexicon_tags_0 },
    { 0xefc08dfc, SUC_lexicon_tags_45 },
    { 0x74808dfd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e08e01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1a408e03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8408e06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4608e0a, SUC_lexicon_tags_2 },
    { 0x64608e0a, SUC_lexicon_tags_17 },
    { 0x9fe08e0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea08e0f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9008e15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca08e19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e08e1d, SUC_lexicon_tags_14 },
    { 0xb0808e1e, SUC_lexicon_tags_201 },
    { 0x74208e1d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc08e24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a08e27, SUC_lexicon_tags_25 },
    { 0xca408e28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x31e08e2a, SUC_lexicon_tags_60 },
    { 0x34c08e2b, SUC_lexicon_tags_34 },
    { 0xc9a08e2c, SUC_lexicon_tags_11 },
    { 0xa8408e2d, SUC_lexicon_tags_37 },
    { 0x72e08e2a, SUC_lexicon_tags_60 },
    { 0xe4408e2a, SUC_lexicon_tags_8 },
    { 0x12e08e30, SUC_lexicon_tags_7 },
    { 0xa6008e30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de08e3a, SUC_lexicon_tags_12 },
    { 0x97e08e3b, SUC_lexicon_tags_25 },
    { 0xbb808e3c, SUC_lexicon_tags_2 },
    { 0xae208e3c, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82408e41, SUC_lexicon_tags_37 },
    { 0x1d608e41, SUC_lexicon_tags_16 },
    { 0x3a808e43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0208e47, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2ae08e49, SUC_lexicon_tags_33 },
    { 0xc8e08e4a, SUC_lexicon_tags_15 },
    { 0x24a08e4b, SUC_lexicon_tags_37 },
    { 0x90c08e4b, SUC_lexicon_tags_6 },
    { 0x39208e4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27208e52, SUC_lexicon_tags_65 },
    { 0x68808e53, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7808e56, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3408e59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53408e5b, SUC_lexicon_tags_29 },
    { 0xb7808e5c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfe808e5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa08e62, SUC_lexicon_tags_21 },
    { 0x6d608e63, SUC_lexicon_tags_7 },
    { 0x6ee08e64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e67, SUC_lexicon_tags_27 },
    { 0xb1008e68, SUC_lexicon_tags_37 },
    { 0xc7808e69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c208e6e, SUC_lexicon_tags_9 },
    { 0x42208e6f, SUC_lexicon_tags_2 },
    { 0x3608e70, SUC_lexicon_tags_25 },
    { 0x83a08e71, SUC_lexicon_tags_25 },
    { 0x94a08e70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea08e75, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a08e78, SUC_lexicon_tags_2 },
    { 0x8bc08e78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac208e7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d808e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5408e83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a08e8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1608e8f, SUC_lexicon_tags_3 },
    { 0x1b208e90, SUC_lexicon_tags_31 },
    { 0xf7208e91, SUC_lexicon_tags_37 },
    { 0xbc08e92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8008e94, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88208e9b, SUC_lexicon_tags_44 },
    { 0x81408e9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6608e9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf608ea1, SUC_lexicon_tags_12 },
    { 0xb8a08ea2, SUC_lexicon_tags_30 },
    { 0x50e08ea3, SUC_lexicon_tags_87 },
    { 0xbd808ea4, SUC_lexicon_tags_37 },
    { 0x25a08ea5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4808eab, SUC_lexicon_tags_12 },
    { 0xf6a08eac, SUC_lexicon_tags_7 },
    { 0xc0c08eac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc408eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22208eb6, SUC_lexicon_tags_7 },
    { 0xde208eb7, SUC_lexicon_tags_27 },
    { 0xf208eb7, SUC_lexicon_tags_31 },
    { 0x1e208eb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e08ebd, SUC_lexicon_tags_28 },
    { 0xf9c08ebe, SUC_lexicon_tags_25 },
    { 0x89208ebf, SUC_lexicon_tags_25 },
    { 0x65608ec0, SUC_lexicon_tags_29 },
    { 0x8ce08ec1, SUC_lexicon_tags_18 },
    { 0xb6808ec0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41408ec6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e408ecb, SUC_lexicon_tags_25 },
    { 0xaf408ecc, SUC_lexicon_tags_7 },
    { 0xa7008ecd, SUC_lexicon_tags_19 },
    { 0x5b808ecd, SUC_lexicon_tags_23 },
    { 0x10008ecf, SUC_lexicon_tags_27 },
    { 0x60608ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4208ed2, SUC_lexicon_tags_17 },
    { 0xaf808ed2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde808ed5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8808ed8, SUC_lexicon_tags_7 },
    { 0xfd208ed8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1408ede, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbf808ee0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e408ee4, SUC_lexicon_tags_25 },
    { 0x49008ee4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fa08ee7, SUC_lexicon_tags_80 },
    { 0x7ce08ee8, SUC_lexicon_tags_15 },
    { 0x89608ee9, SUC_lexicon_tags_37 },
    { 0xc5408eea, SUC_lexicon_tags_12 },
    { 0xef808eea, SUC_lexicon_tags_37 },
    { 0xc2408eeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3da08eee, SUC_lexicon_tags_33 },
    { 0x6aa08eee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1608ef5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76008efa, SUC_lexicon_tags_30 },
    { 0xc3e08efb, SUC_lexicon_tags_19 },
    { 0x83608efc, SUC_lexicon_tags_1 },
    { 0x4da08efb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2808f00, SUC_lexicon_tags_37 },
    { 0x3de08f01, SUC_lexicon_tags_15 },
    { 0x66e08f02, SUC_lexicon_tags_84 },
    { 0x24008f03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2408f08, SUC_lexicon_tags_693 },
    { 0xc5c08f09, SUC_lexicon_tags_26 },
    { 0x68a08f0a, SUC_lexicon_tags_65 },
    { 0xef208f0a, SUC_lexicon_tags_6 },
    { 0x22808f0b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc608f17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0808f1d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18408f23, SUC_lexicon_tags_15 },
    { 0xcf008f24, SUC_lexicon_tags_14 },
    { 0x26e08f23, SUC_lexicon_tags_10 },
    { 0x6bc08f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee08f2b, SUC_lexicon_tags_32 },
    { 0xa7208f2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e08f34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85408f36, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x78a08f38, SUC_lexicon_tags_12 },
    { 0xb5008f38, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a08f3e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20208f43, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83408f47, SUC_lexicon_tags_15 },
    { 0xb4208f48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3ee08f4a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae08f4d, SUC_lexicon_tags_1 },
    { 0xae208f4e, SUC_lexicon_tags_43 },
    { 0x4e08f4f, SUC_lexicon_tags_12 },
    { 0x3b808f4f, SUC_lexicon_tags_7 },
    { 0xfda08f51, SUC_lexicon_tags_12 },
    { 0xd3808f52, SUC_lexicon_tags_63 },
    { 0x63408f53, SUC_lexicon_tags_49 },
    { 0xb008f54, SUC_lexicon_tags_57 },
    { 0x24608f55, SUC_lexicon_tags_43 },
    { 0x89a08f56, SUC_lexicon_tags_2 },
    { 0x25808f56, SUC_lexicon_tags_1 },
    { 0x61208f4f, SUC_lexicon_tags_17 },
    { 0x78008f4e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1c808f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe08f5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30208f65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a08f69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28808f6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e08f72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c08f79, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x86408f7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8608f7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6608f80, SUC_lexicon_tags_38 },
    { 0x8f208f80, SUC_lexicon_tags_37 },
    { 0xdf608f82, SUC_lexicon_tags_17 },
    { 0xe1608f83, SUC_lexicon_tags_60 },
    { 0x68208f83, SUC_lexicon_tags_60 },
    { 0x5d808f80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c08f89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a08f8f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb6a08f91, SUC_lexicon_tags_15 },
    { 0xd4a08f92, SUC_lexicon_tags_38 },
    { 0x9cc08f92, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x7d808f95, SUC_lexicon_tags_11 },
    { 0x44008f96, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64008f9b, SUC_lexicon_tags_2 },
    { 0x6608f9c, SUC_lexicon_tags_15 },
    { 0x96208f9d, SUC_lexicon_tags_251 },
    { 0x7bc08f9d, SUC_lexicon_tags_47 },
    { 0x54a08f9c, SUC_lexicon_tags_87 },
    { 0xfca08f9e, SUC_lexicon_tags_17 },
    { 0x50208fa1, SUC_lexicon_tags_29 },
    { 0x3c808fa2, SUC_lexicon_tags_60 },
    { 0x75008fa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec08fa5, SUC_lexicon_tags_1 },
    { 0xe3408fa5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac08fa9, SUC_lexicon_tags_27 },
    { 0xee008faa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e808fb0, SUC_lexicon_tags_10 },
    { 0xb0608fb1, SUC_lexicon_tags_37 },
    { 0xd1608fb2, SUC_lexicon_tags_43 },
    { 0x2b608fb1, SUC_lexicon_tags_114 },
    { 0xfce08fb1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa08fb7, SUC_lexicon_tags_33 },
    { 0x4008fb8, SUC_lexicon_tags_37 },
    { 0xb6a08fb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a08fbd, SUC_lexicon_tags_11 },
    { 0x43608fbe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4008fc0, SUC_lexicon_tags_27 },
    { 0xe0408fc1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c008fc5, SUC_lexicon_tags_25 },
    { 0xa7008fc6, SUC_lexicon_tags_37 },
    { 0xa4408fc7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c08fd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69608fd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41c08fd9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34408fdb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6f208fdd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb608fe0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6808fe3, SUC_lexicon_tags_92 },
    { 0x67e08fe4, SUC_lexicon_tags_17 },
    { 0xda008fe5, SUC_lexicon_tags_8 },
    { 0x30a08fe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae08fec, SUC_lexicon_tags_12 },
    { 0x51c08fec, SUC_lexicon_tags_23 },
    { 0xbf008fee, SUC_lexicon_tags_15 },
    { 0x98008fef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a08ff4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55008ffa, SUC_lexicon_tags_11 },
    { 0x87c08ffb, SUC_lexicon_tags_24 },
    { 0xbd808ffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e08fff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5209001, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa09004, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa220900a, SUC_lexicon_tags_37 },
    { 0x5980900a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c09010, SUC_lexicon_tags_46 },
    { 0xfec09010, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b609013, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc09018, SUC_lexicon_tags_1 },
    { 0x31a09019, SUC_lexicon_tags_7 },
    { 0xa2c09019, SUC_lexicon_tags_19 },
    { 0x4920901b, SUC_lexicon_tags_27 },
    { 0x1140901b, SUC_lexicon_tags_60 },
    { 0xdec0901d, SUC_lexicon_tags_9 },
    { 0x94a0901e, SUC_lexicon_tags_49 },
    { 0x23c0901e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6e209021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28409026, SUC_lexicon_tags_75 },
    { 0xb4009027, SUC_lexicon_tags_33 },
    { 0x92a09028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc0902c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c0902f, SUC_lexicon_tags_18 },
    { 0x2e009030, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb009037, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x43609039, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5380903b, SUC_lexicon_tags_17 },
    { 0x23a0903c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc809041, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x67e09043, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d809046, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4aa09048, SUC_lexicon_tags_33 },
    { 0x31609049, SUC_lexicon_tags_62 },
    { 0x7d609048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b40904c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeac0904e, SUC_lexicon_tags_56 },
    { 0xd4e0904e, SUC_lexicon_tags_17 },
    { 0xbf209050, SUC_lexicon_tags_11 },
    { 0x93409051, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d209053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f009055, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e0905a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e0905e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2609061, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89209065, SUC_lexicon_tags_1565 },
    { 0xaa09066, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3809069, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c0906e, SUC_lexicon_tags_21 },
    { 0xa1a0906e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a09071, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6209078, SUC_lexicon_tags_0 },
    { 0x21209078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2320907b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4400907f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa09085, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81809087, SUC_lexicon_tags_37 },
    { 0xc6009088, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c09091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcba09093, SUC_lexicon_tags_37 },
    { 0xcf209093, SUC_lexicon_tags_1 },
    { 0x1e009094, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26809097, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d40909a, SUC_lexicon_tags_29 },
    { 0x3c80909a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b40909d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0090a2, SUC_lexicon_tags_45 },
    { 0xfb0090a2, SUC_lexicon_tags_17 },
    { 0x71c090a4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca090a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x664090a9, SUC_lexicon_tags_47 },
    { 0x992090aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348090ae, SUC_lexicon_tags_12 },
    { 0x2f2090af, SUC_lexicon_tags_5 },
    { 0xc090af, SUC_lexicon_tags_12 },
    { 0x6d2090b1, SUC_lexicon_tags_30 },
    { 0x4ce090b1, SUC_lexicon_tags_23 },
    { 0x7c4090b2, SUC_lexicon_tags_47 },
    { 0x900090b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2090bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e090c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c090c4, SUC_lexicon_tags_14 },
    { 0x130090c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x298090c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762090cc, SUC_lexicon_tags_38 },
    { 0xafa090cd, SUC_lexicon_tags_33 },
    { 0x35c090cc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x486090d1, SUC_lexicon_tags_25 },
    { 0x278090d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ee090d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34090dc, SUC_lexicon_tags_38 },
    { 0x48090dc, SUC_lexicon_tags_47 },
    { 0xdfe090dd, SUC_lexicon_tags_26 },
    { 0x6ac090df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974090e3, SUC_lexicon_tags_9 },
    { 0xc6e090e3, SUC_lexicon_tags_47 },
    { 0xa1e090e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea090e8, SUC_lexicon_tags_7 },
    { 0x826090e9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78090ec, SUC_lexicon_tags_32 },
    { 0x698090ed, SUC_lexicon_tags_11 },
    { 0xc02090ee, SUC_lexicon_tags_37 },
    { 0x2b2090ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870090f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c090f7, SUC_lexicon_tags_37 },
    { 0x2ba090f7, SUC_lexicon_tags_32 },
    { 0xfd6090f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2090fd, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x628090ff, SUC_lexicon_tags_30 },
    { 0xc2c090ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8409104, SUC_lexicon_tags_25 },
    { 0x57609105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa409108, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8780910a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf480910c, SUC_lexicon_tags_1 },
    { 0x1980910c, SUC_lexicon_tags_37 },
    { 0x9340910e, SUC_lexicon_tags_37 },
    { 0xbf80910f, SUC_lexicon_tags_23 },
    { 0x7de09110, SUC_lexicon_tags_26 },
    { 0x49a09111, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16209113, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79609117, SUC_lexicon_tags_8 },
    { 0x21e09118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9be0911a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a09120, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x87209122, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee09125, SUC_lexicon_tags_2 },
    { 0x48209126, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4600912b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xff60912d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x44209130, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46609132, SUC_lexicon_tags_7 },
    { 0x23209132, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce09139, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcf60913b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c0913e, SUC_lexicon_tags_12 },
    { 0xf8c0913e, SUC_lexicon_tags_47 },
    { 0xd2e09140, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c09145, SUC_lexicon_tags_58 },
    { 0x35009146, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5460914a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc780914f, SUC_lexicon_tags_28 },
    { 0xe9409150, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c09153, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0809155, SUC_lexicon_tags_16 },
    { 0x8d409156, SUC_lexicon_tags_25 },
    { 0xd8609156, SUC_lexicon_tags_12 },
    { 0x2ac09158, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd809164, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca09167, SUC_lexicon_tags_31 },
    { 0x7bc09168, SUC_lexicon_tags_44 },
    { 0x12809169, SUC_lexicon_tags_12 },
    { 0xd540916a, SUC_lexicon_tags_39 },
    { 0x4ce0916b, SUC_lexicon_tags_0 },
    { 0x78e0916c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e809174, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x71209176, SUC_lexicon_tags_47 },
    { 0xafa09176, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd809179, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d609186, SUC_lexicon_tags_7 },
    { 0xf7809187, SUC_lexicon_tags_10 },
    { 0x5ca09186, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe400918c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a809190, SUC_lexicon_tags_11 },
    { 0x21409190, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca209193, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99009196, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x77c09198, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fa0919a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7520919c, SUC_lexicon_tags_12 },
    { 0x3ea0919d, SUC_lexicon_tags_37 },
    { 0xf420919e, SUC_lexicon_tags_14 },
    { 0x9360919f, SUC_lexicon_tags_25 },
    { 0x658091a0, SUC_lexicon_tags_70 },
    { 0xbe00919d, SUC_lexicon_tags_38 },
    { 0xf2c091a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc56091a4, SUC_lexicon_tags_2 },
    { 0xa84091a5, SUC_lexicon_tags_23 },
    { 0x7ee091a6, SUC_lexicon_tags_12 },
    { 0xd4091a7, SUC_lexicon_tags_30 },
    { 0x326091a8, SUC_lexicon_tags_47 },
    { 0x600091a8, SUC_lexicon_tags_37 },
    { 0x65e091a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e091ad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce091b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x534091b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e0091b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4a091b8, SUC_lexicon_tags_1 },
    { 0xe86091b9, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0x16091bc, SUC_lexicon_tags_2 },
    { 0x822091bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2091c1, SUC_lexicon_tags_9 },
    { 0x48091c1, SUC_lexicon_tags_37 },
    { 0xbc8091c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4091c9, SUC_lexicon_tags_57 },
    { 0x9d0091ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e091ce, SUC_lexicon_tags_66 },
    { 0x45a091cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x136091d2, SUC_lexicon_tags_68 },
    { 0x82a091d3, SUC_lexicon_tags_63 },
    { 0x908091d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94091da, SUC_lexicon_tags_63 },
    { 0xa28091db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc78091dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa091e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104091e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa0091e8, SUC_lexicon_tags_19 },
    { 0xf20091e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28091ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14091f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e091f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd28091fa, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x9f0091fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9020920a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa780920c, SUC_lexicon_tags_63 },
    { 0x4b20920c, SUC_lexicon_tags_104 },
    { 0xca60920e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87009212, SUC_lexicon_tags_30 },
    { 0x9c809213, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x75809216, SUC_lexicon_tags_14 },
    { 0x4b609217, SUC_lexicon_tags_31 },
    { 0xe7409217, SUC_lexicon_tags_23 },
    { 0x6f409219, SUC_lexicon_tags_60 },
    { 0x26c09217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4460921e, SUC_lexicon_tags_7 },
    { 0xd060921f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c209221, SUC_lexicon_tags_29 },
    { 0x49609222, SUC_lexicon_tags_33 },
    { 0xe9809223, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a09226, SUC_lexicon_tags_29 },
    { 0x6c09227, SUC_lexicon_tags_1 },
    { 0x27e09226, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3140922d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d009230, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b609233, SUC_lexicon_tags_1 },
    { 0x11609234, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3809236, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3340923a, SUC_lexicon_tags_17 },
    { 0x2140923b, SUC_lexicon_tags_33 },
    { 0x320923b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa609243, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0924a, SUC_lexicon_tags_14 },
    { 0x14a0924a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b40924d, SUC_lexicon_tags_7 },
    { 0xb440924e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31809254, SUC_lexicon_tags_37 },
    { 0x94409254, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa6409257, SUC_lexicon_tags_11 },
    { 0xae209258, SUC_lexicon_tags_50 },
    { 0x3ce09259, SUC_lexicon_tags_15 },
    { 0x2dc09257, SUC_lexicon_tags_12 },
    { 0xe960925b, SUC_lexicon_tags_11 },
    { 0x8980925c, SUC_lexicon_tags_11 },
    { 0xbc00925d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xf360925f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60809264, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x63209266, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c09269, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c0926d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d20926f, SUC_lexicon_tags_15 },
    { 0x83c0926f, SUC_lexicon_tags_31 },
    { 0x10c0926f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeda09273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a09278, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9480927c, SUC_lexicon_tags_1 },
    { 0x8f60927d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f809283, SUC_lexicon_tags_57 },
    { 0x7dc09284, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be09287, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b409294, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0929b, SUC_lexicon_tags_17 },
    { 0xd20929c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e092a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc00092a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c092ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4a092b0, SUC_lexicon_tags_63 },
    { 0xe14092b1, SUC_lexicon_tags_37 },
    { 0x976092b1, SUC_lexicon_tags_47 },
    { 0x9fc092b1, SUC_lexicon_tags_18 },
    { 0x880092b4, SUC_lexicon_tags_2 },
    { 0xdd0092b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8092b8, SUC_lexicon_tags_17 },
    { 0xd4c092b9, SUC_lexicon_tags_21 },
    { 0xc4a092ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce092be, SUC_lexicon_tags_12 },
    { 0x638092bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92092c2, SUC_lexicon_tags_12 },
    { 0x840092c3, SUC_lexicon_tags_11 },
    { 0xd02092c4, SUC_lexicon_tags_11 },
    { 0x658092c4, SUC_lexicon_tags_63 },
    { 0x218092c6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xadc092c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c092d4, SUC_lexicon_tags_19 },
    { 0x4b8092d4, SUC_lexicon_tags_17 },
    { 0x75a092d6, SUC_lexicon_tags_47 },
    { 0x886092d5, SUC_lexicon_tags_30 },
    { 0xa22092d8, SUC_lexicon_tags_17 },
    { 0x640092d9, SUC_lexicon_tags_30 },
    { 0xd6092d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8bc092dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e092df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x264092e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x716092e5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0092ea, SUC_lexicon_tags_17 },
    { 0xfac092eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64092ee, SUC_lexicon_tags_17 },
    { 0xbe0092ef, SUC_lexicon_tags_19 },
    { 0x99e092ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x440092f2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6c4092f4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42092fd, SUC_lexicon_tags_2 },
    { 0xfe6092fd, SUC_lexicon_tags_68 },
    { 0xeb2092ff, SUC_lexicon_tags_49 },
    { 0xef6092ff, SUC_lexicon_tags_57 },
    { 0x1e609300, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa280930a, SUC_lexicon_tags_39 },
    { 0x73e0930a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5220930f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82409316, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x28009318, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c0931c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe160931e, SUC_lexicon_tags_1 },
    { 0x9ee0931e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4409321, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7e209323, SUC_lexicon_tags_15 },
    { 0x9a009324, SUC_lexicon_tags_29 },
    { 0x7209324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68e09327, SUC_lexicon_tags_6 },
    { 0xb6c09328, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa180932b, SUC_lexicon_tags_2 },
    { 0x3a80932b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa09331, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9e409333, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1809336, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4409338, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2b40933a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3400933c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1120933e, SUC_lexicon_tags_37 },
    { 0x7a60933e, SUC_lexicon_tags_11 },
    { 0xe1009340, SUC_lexicon_tags_27 },
    { 0xf2a09341, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf009343, SUC_lexicon_tags_35 },
    { 0xac609344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5809349, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2940934e, SUC_lexicon_tags_45 },
    { 0xb8e0934f, SUC_lexicon_tags_49 },
    { 0x1380934e, SUC_lexicon_tags_17 },
    { 0xbca09351, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a09357, SUC_lexicon_tags_45 },
    { 0x26a09357, SUC_lexicon_tags_35 },
    { 0x5ba09359, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc09362, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0409366, SUC_lexicon_tags_57 },
    { 0x21c09366, SUC_lexicon_tags_17 },
    { 0xee609368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c409370, SUC_lexicon_tags_27 },
    { 0x8c009371, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc09376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x61609379, SUC_lexicon_tags_27 },
    { 0x8d20937a, SUC_lexicon_tags_17 },
    { 0x3540937b, SUC_lexicon_tags_37 },
    { 0x37209379, SUC_lexicon_tags_7 },
    { 0xbf40937d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c09380, SUC_lexicon_tags_17 },
    { 0xd6009381, SUC_lexicon_tags_10 },
    { 0xe409382, SUC_lexicon_tags_11 },
    { 0xf4609383, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd8e09385, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e0938d, SUC_lexicon_tags_21 },
    { 0x7b60938d, SUC_lexicon_tags_25 },
    { 0xaaa0938e, SUC_lexicon_tags_27 },
    { 0xb940938d, SUC_lexicon_tags_4 },
    { 0x33609390, SUC_lexicon_tags_69 },
    { 0x3b409392, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf009394, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2209398, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a0939e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x828093a1, SUC_lexicon_tags_7 },
    { 0xd7a093a1, SUC_lexicon_tags_11 },
    { 0xfc2093a3, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca093a6, SUC_lexicon_tags_47 },
    { 0xfca093a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c093ac, SUC_lexicon_tags_25 },
    { 0x1e2093ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752093b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x584093b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c093b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64093b9, SUC_lexicon_tags_25 },
    { 0x4a6093ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61c093bc, SUC_lexicon_tags_31 },
    { 0x8b0093bd, SUC_lexicon_tags_25 },
    { 0x40a093bd, SUC_lexicon_tags_60 },
    { 0x104093bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6093c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8093c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x884093cc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xffe093ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8093d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80093d9, SUC_lexicon_tags_33 },
    { 0x3a4093da, SUC_lexicon_tags_23 },
    { 0x1ea093db, SUC_lexicon_tags_30 },
    { 0x944093db, SUC_lexicon_tags_27 },
    { 0xcbc093dd, SUC_lexicon_tags_1 },
    { 0xf40093d9, SUC_lexicon_tags_4 },
    { 0x1fe093df, SUC_lexicon_tags_7 },
    { 0x338093e0, SUC_lexicon_tags_17 },
    { 0xd76093df, SUC_lexicon_tags_38 },
    { 0xf70093e2, SUC_lexicon_tags_7 },
    { 0x48c093da, SUC_lexicon_tags_12 },
    { 0x5da093e4, SUC_lexicon_tags_38 },
    { 0x9dc093e5, SUC_lexicon_tags_11 },
    { 0x100093e3, SUC_lexicon_tags_7 },
    { 0x7d4093e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a093eb, SUC_lexicon_tags_10 },
    { 0xf46093eb, SUC_lexicon_tags_46 },
    { 0x334093ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424093f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de093f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e093f9, SUC_lexicon_tags_63 },
    { 0x52e093fa, SUC_lexicon_tags_15 },
    { 0x5ae093fa, SUC_lexicon_tags_68 },
    { 0x4cc093f9, SUC_lexicon_tags_11 },
    { 0x6b4093f9, SUC_lexicon_tags_15 },
    { 0x3c2093fe, SUC_lexicon_tags_15 },
    { 0x7cc093ff, SUC_lexicon_tags_21 },
    { 0xd32093ff, SUC_lexicon_tags_7 },
    { 0xd3209401, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe4409403, SUC_lexicon_tags_60 },
    { 0xa209404, SUC_lexicon_tags_1 },
    { 0xeee09405, SUC_lexicon_tags_21 },
    { 0x61809406, SUC_lexicon_tags_45 },
    { 0xd4809403, SUC_lexicon_tags_1 },
    { 0x7de09408, SUC_lexicon_tags_12 },
    { 0x20409408, SUC_lexicon_tags_19 },
    { 0x8e40940a, SUC_lexicon_tags_29 },
    { 0xb820940b, SUC_lexicon_tags_7 },
    { 0x88c09409, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda0940f, SUC_lexicon_tags_7 },
    { 0x8b409410, SUC_lexicon_tags_47 },
    { 0xbf209411, SUC_lexicon_tags_30 },
    { 0x28e0940f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c09415, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc9609417, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc820941c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xc820941e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc2809420, SUC_lexicon_tags_15 },
    { 0x9de09420, SUC_lexicon_tags_49 },
    { 0x21009421, SUC_lexicon_tags_19 },
    { 0xfd009422, SUC_lexicon_tags_31 },
    { 0x77c09423, SUC_lexicon_tags_14 },
    { 0xc8809425, SUC_lexicon_tags_0 },
    { 0x4be09425, SUC_lexicon_tags_11 },
    { 0x7409427, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1400942c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6280942f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd4a09431, SUC_lexicon_tags_1 },
    { 0xc6409431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e09435, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x41e09437, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c0943c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a09440, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x56009443, SUC_lexicon_tags_23 },
    { 0x7cc09444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a409447, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd80944b, SUC_lexicon_tags_11 },
    { 0x5960944b, SUC_lexicon_tags_1 },
    { 0x36e0944c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d209452, SUC_lexicon_tags_14 },
    { 0x5ac09453, SUC_lexicon_tags_11 },
    { 0x5e809452, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdae09456, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c09459, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf809460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1409464, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc09469, SUC_lexicon_tags_224 },
    { 0x53409469, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2a0946c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2380946e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62809470, SUC_lexicon_tags_14 },
    { 0xce609470, SUC_lexicon_tags_12 },
    { 0x44609472, SUC_lexicon_tags_282 },
    { 0xde09473, SUC_lexicon_tags_7 },
    { 0x32e09474, SUC_lexicon_tags_2 },
    { 0xee809471, SUC_lexicon_tags_25 },
    { 0xece09476, SUC_lexicon_tags_27 },
    { 0x31a09477, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc0947d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1940947f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93009481, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd4209483, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1e009485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20009487, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c40948c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d409491, SUC_lexicon_tags_16 },
    { 0x23209492, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbfe09494, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca09498, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa20949d, SUC_lexicon_tags_25 },
    { 0xa840949d, SUC_lexicon_tags_60 },
    { 0xe3e0949e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfc094a1, SUC_lexicon_tags_45 },
    { 0x54e094a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e094a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa094ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e094b1, SUC_lexicon_tags_1 },
    { 0xf9a094b2, SUC_lexicon_tags_37 },
    { 0x48c094b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104094b8, SUC_lexicon_tags_5 },
    { 0xe28094b9, SUC_lexicon_tags_9 },
    { 0xc38094ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2d0094bc, SUC_lexicon_tags_27 },
    { 0x10e094bc, SUC_lexicon_tags_27 },
    { 0x63a094bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe094c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0094c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x430094c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4094ca, SUC_lexicon_tags_28 },
    { 0x15a094cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282094cf, SUC_lexicon_tags_15 },
    { 0x9b4094cf, SUC_lexicon_tags_25 },
    { 0x3cc094d1, SUC_lexicon_tags_16 },
    { 0x2de094d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x912094d5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca094dc, SUC_lexicon_tags_7 },
    { 0x298094dd, SUC_lexicon_tags_60 },
    { 0x8e094dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570094e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4094e9, SUC_lexicon_tags_7 },
    { 0x9a8094ea, SUC_lexicon_tags_57 },
    { 0x4ac094e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0094f1, SUC_lexicon_tags_68 },
    { 0xa60094f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934094f8, SUC_lexicon_tags_37 },
    { 0x5f8094f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8c094fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x804094fe, SUC_lexicon_tags_17 },
    { 0x26094ff, SUC_lexicon_tags_47 },
    { 0xc2c09500, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5a09502, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc6a09504, SUC_lexicon_tags_37 },
    { 0xbc809505, SUC_lexicon_tags_87 },
    { 0x63c09506, SUC_lexicon_tags_82 },
    { 0xb9009506, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe0950d, SUC_lexicon_tags_557 },
    { 0, NULL },
    { 0xde80950f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a409512, SUC_lexicon_tags_25 },
    { 0x9c09512, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc09516, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7f009518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa60951a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66209521, SUC_lexicon_tags_46 },
    { 0x9a409522, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc09525, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e09528, SUC_lexicon_tags_25 },
    { 0x87609529, SUC_lexicon_tags_11 },
    { 0xfa209529, SUC_lexicon_tags_11 },
    { 0x1cc0952b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80009532, SUC_lexicon_tags_0 },
    { 0x52c09532, SUC_lexicon_tags_43 },
    { 0xee209533, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44609537, SUC_lexicon_tags_2 },
    { 0x52609538, SUC_lexicon_tags_9 },
    { 0xb7a09537, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f809542, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a09545, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84209547, SUC_lexicon_tags_32 },
    { 0x40409548, SUC_lexicon_tags_7 },
    { 0x58809547, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91009552, SUC_lexicon_tags_17 },
    { 0xdac09553, SUC_lexicon_tags_25 },
    { 0x2d609553, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c809556, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea09559, SUC_lexicon_tags_27 },
    { 0xe600955a, SUC_lexicon_tags_79 },
    { 0x5be0955b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c0955e, SUC_lexicon_tags_12 },
    { 0x32e0955f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e09562, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xe3809564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7e09566, SUC_lexicon_tags_12 },
    { 0xed009566, SUC_lexicon_tags_25 },
    { 0x71409566, SUC_lexicon_tags_27 },
    { 0x6d609567, SUC_lexicon_tags_10 },
    { 0xef60956a, SUC_lexicon_tags_35 },
    { 0xc20956b, SUC_lexicon_tags_17 },
    { 0x97c09569, SUC_lexicon_tags_2 },
    { 0x4de0956d, SUC_lexicon_tags_18 },
    { 0xbc60956d, SUC_lexicon_tags_60 },
    { 0x43c0956f, SUC_lexicon_tags_37 },
    { 0x4c609570, SUC_lexicon_tags_65 },
    { 0x75e09571, SUC_lexicon_tags_37 },
    { 0x82409572, SUC_lexicon_tags_8 },
    { 0xa4609573, SUC_lexicon_tags_7 },
    { 0x86209574, SUC_lexicon_tags_11 },
    { 0xb9e09575, SUC_lexicon_tags_2 },
    { 0x7b009575, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c409579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ec0957b, SUC_lexicon_tags_1 },
    { 0xa7e0957c, SUC_lexicon_tags_11 },
    { 0xef20957c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3409586, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a09590, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4209592, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdfc09594, SUC_lexicon_tags_43 },
    { 0xed009595, SUC_lexicon_tags_12 },
    { 0xaae09596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e0959a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce0959d, SUC_lexicon_tags_14 },
    { 0x2460959e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be095a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb6095a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0095a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a6095aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52095b2, SUC_lexicon_tags_15 },
    { 0xea2095b2, SUC_lexicon_tags_7 },
    { 0x686095b2, SUC_lexicon_tags_7 },
    { 0xda6095b5, SUC_lexicon_tags_2 },
    { 0x45e095b6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd34095b8, SUC_lexicon_tags_6 },
    { 0xb4095b9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6095be, SUC_lexicon_tags_17 },
    { 0x606095bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0095c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610095ca, SUC_lexicon_tags_15 },
    { 0x870095cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a095cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a095d4, SUC_lexicon_tags_11 },
    { 0xf2a095d5, SUC_lexicon_tags_37 },
    { 0xf7c095d6, SUC_lexicon_tags_43 },
    { 0x944095d7, SUC_lexicon_tags_16 },
    { 0x698095d8, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0x3f8095da, SUC_lexicon_tags_55 },
    { 0xbee095db, SUC_lexicon_tags_14 },
    { 0x214095dc, SUC_lexicon_tags_25 },
    { 0x6bc095dd, SUC_lexicon_tags_2 },
    { 0xa0095de, SUC_lexicon_tags_25 },
    { 0xc74095de, SUC_lexicon_tags_17 },
    { 0x6c2095e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3c095e2, SUC_lexicon_tags_1 },
    { 0x518095e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80095e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e6095e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae095ec, SUC_lexicon_tags_273 },
    { 0, NULL },
    { 0, NULL },
    { 0x638095ef, SUC_lexicon_tags_63 },
    { 0x74a095f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94095f4, SUC_lexicon_tags_61 },
    { 0x464095f5, SUC_lexicon_tags_27 },
    { 0xe6a095f6, SUC_lexicon_tags_27 },
    { 0x894095f7, SUC_lexicon_tags_11 },
    { 0xe58095f6, SUC_lexicon_tags_17 },
    { 0xa1a095f8, SUC_lexicon_tags_17 },
    { 0x7cc095f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x194095fc, SUC_lexicon_tags_11 },
    { 0xe3a095fd, SUC_lexicon_tags_0 },
    { 0xe44095fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x64409600, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca09603, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80960b, SUC_lexicon_tags_2 },
    { 0x4000960c, SUC_lexicon_tags_79 },
    { 0x1ec0960d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0409610, SUC_lexicon_tags_11 },
    { 0x23609610, SUC_lexicon_tags_17 },
    { 0xd7009612, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c0961e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1209624, SUC_lexicon_tags_11 },
    { 0x1ce09625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa009628, SUC_lexicon_tags_25 },
    { 0xb7209628, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc540962b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcf40962d, SUC_lexicon_tags_1 },
    { 0xed60962e, SUC_lexicon_tags_37 },
    { 0xf920962f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc09635, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a80963b, SUC_lexicon_tags_11 },
    { 0xb7c0963b, SUC_lexicon_tags_38 },
    { 0x500963d, SUC_lexicon_tags_12 },
    { 0xd8a0963b, SUC_lexicon_tags_7 },
    { 0x83c0963b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4209642, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84209647, SUC_lexicon_tags_12 },
    { 0x38409648, SUC_lexicon_tags_8 },
    { 0x94c09647, SUC_lexicon_tags_12 },
    { 0x6f809647, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf40964d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7140964f, SUC_lexicon_tags_117 },
    { 0x6e60964f, SUC_lexicon_tags_47 },
    { 0x53209651, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x22609653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e0965a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e0965d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x4320965f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a809661, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x31409663, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5360966e, SUC_lexicon_tags_0 },
    { 0xf980966f, SUC_lexicon_tags_6 },
    { 0xd0e0966f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a09675, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71209679, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfe80967b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c80967e, SUC_lexicon_tags_37 },
    { 0xe300967f, SUC_lexicon_tags_63 },
    { 0xe6809680, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc6e09682, SUC_lexicon_tags_38 },
    { 0x8409683, SUC_lexicon_tags_2 },
    { 0xc0609684, SUC_lexicon_tags_0 },
    { 0xb1e09682, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2609687, SUC_lexicon_tags_64 },
    { 0x58e09688, SUC_lexicon_tags_68 },
    { 0xcd609688, SUC_lexicon_tags_11 },
    { 0xf8c09689, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9609690, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e609694, SUC_lexicon_tags_17 },
    { 0x69809695, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7620969b, SUC_lexicon_tags_2 },
    { 0xb000969c, SUC_lexicon_tags_17 },
    { 0xa40969d, SUC_lexicon_tags_718 },
    { 0x3300969e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc88096a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780096a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a096a9, SUC_lexicon_tags_37 },
    { 0x354096aa, SUC_lexicon_tags_28 },
    { 0x2ca096ab, SUC_lexicon_tags_23 },
    { 0x8f4096ab, SUC_lexicon_tags_7 },
    { 0xef6096aa, SUC_lexicon_tags_1 },
    { 0xa4a096ad, SUC_lexicon_tags_1 },
    { 0x54c096ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x51e096b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x350096b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x262096b6, SUC_lexicon_tags_42 },
    { 0xd98096b6, SUC_lexicon_tags_1 },
    { 0x646096b8, SUC_lexicon_tags_23 },
    { 0x2f6096b6, SUC_lexicon_tags_47 },
    { 0x232096ba, SUC_lexicon_tags_1 },
    { 0xab6096b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566096c7, SUC_lexicon_tags_18 },
    { 0x9a8096c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64096cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2a096d1, SUC_lexicon_tags_37 },
    { 0xae096d2, SUC_lexicon_tags_2 },
    { 0xa9e096d3, SUC_lexicon_tags_39 },
    { 0x9a8096d4, SUC_lexicon_tags_12 },
    { 0x202096d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b6096d7, SUC_lexicon_tags_71 },
    { 0x956096d8, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e096dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40096e0, SUC_lexicon_tags_12 },
    { 0xd00096e1, SUC_lexicon_tags_87 },
    { 0x53e096e2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x704096e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6096e7, SUC_lexicon_tags_8 },
    { 0xe7e096e8, SUC_lexicon_tags_7 },
    { 0x950096e8, SUC_lexicon_tags_21 },
    { 0xee096ea, SUC_lexicon_tags_11 },
    { 0xd2e096eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe00096ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e096f0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xec096f2, SUC_lexicon_tags_29 },
    { 0x8b8096f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22096f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa32096f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6096fd, SUC_lexicon_tags_27 },
    { 0x770096fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30409700, SUC_lexicon_tags_25 },
    { 0xb4c09701, SUC_lexicon_tags_963 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e609704, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2de09706, SUC_lexicon_tags_11 },
    { 0xf3609706, SUC_lexicon_tags_12 },
    { 0xd7209708, SUC_lexicon_tags_17 },
    { 0x62e09709, SUC_lexicon_tags_10 },
    { 0xae009707, SUC_lexicon_tags_6 },
    { 0x2620970b, SUC_lexicon_tags_31 },
    { 0xff20970b, SUC_lexicon_tags_14 },
    { 0xf060970d, SUC_lexicon_tags_25 },
    { 0x23a0970b, SUC_lexicon_tags_11 },
    { 0x560970d, SUC_lexicon_tags_14 },
    { 0x6d409710, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8809716, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86809718, SUC_lexicon_tags_9 },
    { 0xe4a09718, SUC_lexicon_tags_25 },
    { 0xd1e09719, SUC_lexicon_tags_46 },
    { 0xae0971b, SUC_lexicon_tags_19 },
    { 0xa780971a, SUC_lexicon_tags_8 },
    { 0x6280971b, SUC_lexicon_tags_31 },
    { 0xe7a0971e, SUC_lexicon_tags_19 },
    { 0xd400971a, SUC_lexicon_tags_12 },
    { 0x5360971d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba09723, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5009725, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x31809728, SUC_lexicon_tags_15 },
    { 0x1f209729, SUC_lexicon_tags_17 },
    { 0x43609728, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbb60972c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c0972f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8409732, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9409735, SUC_lexicon_tags_6 },
    { 0x4d409735, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde0973c, SUC_lexicon_tags_45 },
    { 0x6c60973d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x55e0973f, SUC_lexicon_tags_29 },
    { 0x50e0973f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6a609742, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98809744, SUC_lexicon_tags_15 },
    { 0x74c09744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78209747, SUC_lexicon_tags_3 },
    { 0xbee09748, SUC_lexicon_tags_6 },
    { 0x8f009749, SUC_lexicon_tags_45 },
    { 0x8340974a, SUC_lexicon_tags_2 },
    { 0x9bc0974b, SUC_lexicon_tags_12 },
    { 0xea0974b, SUC_lexicon_tags_55 },
    { 0x7040974d, SUC_lexicon_tags_19 },
    { 0x21a0974e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c009757, SUC_lexicon_tags_63 },
    { 0xf8809757, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf800975b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c0975e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80409761, SUC_lexicon_tags_45 },
    { 0x56009761, SUC_lexicon_tags_1 },
    { 0xf0809763, SUC_lexicon_tags_2 },
    { 0x36009762, SUC_lexicon_tags_17 },
    { 0xc3a09765, SUC_lexicon_tags_0 },
    { 0x56c09765, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x15209768, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b80976a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2809771, SUC_lexicon_tags_59 },
    { 0xcfc09771, SUC_lexicon_tags_13 },
    { 0xe1809773, SUC_lexicon_tags_23 },
    { 0x4c409772, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce409776, SUC_lexicon_tags_7 },
    { 0x97609777, SUC_lexicon_tags_37 },
    { 0x1c809778, SUC_lexicon_tags_49 },
    { 0xa5009777, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3200977e, SUC_lexicon_tags_26 },
    { 0x4c0977f, SUC_lexicon_tags_14 },
    { 0x86c09780, SUC_lexicon_tags_17 },
    { 0xe9c0977f, SUC_lexicon_tags_713 },
    { 0x7a409780, SUC_lexicon_tags_40 },
    { 0x6a009783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77809788, SUC_lexicon_tags_7 },
    { 0x60c09789, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee0978e, SUC_lexicon_tags_60 },
    { 0x3000978e, SUC_lexicon_tags_12 },
    { 0x7a009790, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc1609792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a209795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b609797, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e097a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x684097a4, SUC_lexicon_tags_77 },
    { 0xd84097a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342097a9, SUC_lexicon_tags_11 },
    { 0x8fc097a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x128097ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e097ba, SUC_lexicon_tags_16 },
    { 0x16a097ba, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4097bf, SUC_lexicon_tags_16 },
    { 0x60a097bf, SUC_lexicon_tags_37 },
    { 0x4f2097c0, SUC_lexicon_tags_68 },
    { 0x934097c2, SUC_lexicon_tags_9 },
    { 0x156097c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30097c6, SUC_lexicon_tags_1 },
    { 0x7c0097c7, SUC_lexicon_tags_47 },
    { 0x7b2097c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14097cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0097d5, SUC_lexicon_tags_1 },
    { 0x2ee097d6, SUC_lexicon_tags_49 },
    { 0xd4a097d7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x908097d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a097e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12c097e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28097f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc06097f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa76097f6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x602097f8, SUC_lexicon_tags_11 },
    { 0xd4c097f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c09800, SUC_lexicon_tags_32 },
    { 0xb2a09801, SUC_lexicon_tags_11 },
    { 0x6c609802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17a09804, SUC_lexicon_tags_85 },
    { 0x51809805, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da09808, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1660980c, SUC_lexicon_tags_7 },
    { 0x2860980c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7609811, SUC_lexicon_tags_56 },
    { 0xcc809812, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe09815, SUC_lexicon_tags_1 },
    { 0x5009816, SUC_lexicon_tags_47 },
    { 0x70009815, SUC_lexicon_tags_25 },
    { 0xf4009818, SUC_lexicon_tags_65 },
    { 0x1ce09819, SUC_lexicon_tags_7 },
    { 0x2e20981a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5cc0981c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1080981e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf809820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9409822, SUC_lexicon_tags_15 },
    { 0x2d009823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63009825, SUC_lexicon_tags_31 },
    { 0x4ee09826, SUC_lexicon_tags_82 },
    { 0x3c009827, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a0982b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb60982f, SUC_lexicon_tags_60 },
    { 0xaa80982f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a209834, SUC_lexicon_tags_19 },
    { 0xb6609835, SUC_lexicon_tags_25 },
    { 0x14809836, SUC_lexicon_tags_11 },
    { 0x4b809837, SUC_lexicon_tags_11 },
    { 0x72a09836, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca60983c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0983f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4809842, SUC_lexicon_tags_10 },
    { 0x84609843, SUC_lexicon_tags_9 },
    { 0x4da09843, SUC_lexicon_tags_49 },
    { 0xf6609845, SUC_lexicon_tags_47 },
    { 0xde609846, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d009848, SUC_lexicon_tags_222 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f80984e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50409858, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5660985e, SUC_lexicon_tags_34 },
    { 0xf40985f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a809865, SUC_lexicon_tags_7 },
    { 0x9b409865, SUC_lexicon_tags_33 },
    { 0x1ae09867, SUC_lexicon_tags_1 },
    { 0xb2609867, SUC_lexicon_tags_29 },
    { 0x46c09866, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1be0986b, SUC_lexicon_tags_8 },
    { 0x5ae0986b, SUC_lexicon_tags_25 },
    { 0x18e0986d, SUC_lexicon_tags_15 },
    { 0x4b80986d, SUC_lexicon_tags_42 },
    { 0x5260986f, SUC_lexicon_tags_13 },
    { 0xb860986e, SUC_lexicon_tags_38 },
    { 0x8da09870, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee09876, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x63209878, SUC_lexicon_tags_43 },
    { 0x94209878, SUC_lexicon_tags_14 },
    { 0x23e09878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3dc0987c, SUC_lexicon_tags_9 },
    { 0x4e60987d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x67609880, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x20c09882, SUC_lexicon_tags_7 },
    { 0xdcc09882, SUC_lexicon_tags_38 },
    { 0x58a09884, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62009889, SUC_lexicon_tags_12 },
    { 0x1480988a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38809891, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9609896, SUC_lexicon_tags_11 },
    { 0x1a009897, SUC_lexicon_tags_25 },
    { 0xfb009898, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b60989b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a60989e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x158098a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408098a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5a098a6, SUC_lexicon_tags_29 },
    { 0x354098a7, SUC_lexicon_tags_68 },
    { 0xf24098a8, SUC_lexicon_tags_15 },
    { 0x282098a7, SUC_lexicon_tags_55 },
    { 0x53a098aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90098af, SUC_lexicon_tags_19 },
    { 0xad2098b0, SUC_lexicon_tags_12 },
    { 0x238098b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0098b5, SUC_lexicon_tags_55 },
    { 0x360098b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca098b9, SUC_lexicon_tags_11 },
    { 0x46c098ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc098bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2098c1, SUC_lexicon_tags_25 },
    { 0xeae098c1, SUC_lexicon_tags_512 },
    { 0x7b2098c1, SUC_lexicon_tags_12 },
    { 0x21e098c1, SUC_lexicon_tags_7 },
    { 0x78098c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x994098c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74098d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0098d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x85a098d5, SUC_lexicon_tags_0 },
    { 0x228098d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x208098d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66098dd, SUC_lexicon_tags_12 },
    { 0x812098de, SUC_lexicon_tags_7 },
    { 0x1e8098df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a098e4, SUC_lexicon_tags_34 },
    { 0xfd4098e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a2098e7, SUC_lexicon_tags_25 },
    { 0x646098e8, SUC_lexicon_tags_63 },
    { 0xcea098e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6098f1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec098f5, SUC_lexicon_tags_48 },
    { 0x692098f6, SUC_lexicon_tags_37 },
    { 0x8e098f5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xee098fa, SUC_lexicon_tags_2 },
    { 0x53a098fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd32098fd, SUC_lexicon_tags_23 },
    { 0xd20098fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfc09900, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc0990c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7820990e, SUC_lexicon_tags_7 },
    { 0xfe20990f, SUC_lexicon_tags_8 },
    { 0x86209910, SUC_lexicon_tags_19 },
    { 0xa5c0990f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a209913, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac09916, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26809918, SUC_lexicon_tags_25 },
    { 0x10e09918, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e0991c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3f60991e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0809935, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea09938, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaca0993a, SUC_lexicon_tags_4 },
    { 0xa780993b, SUC_lexicon_tags_45 },
    { 0x4fa0993c, SUC_lexicon_tags_4 },
    { 0xf6e0993d, SUC_lexicon_tags_14 },
    { 0xc400993e, SUC_lexicon_tags_1 },
    { 0xb140993f, SUC_lexicon_tags_23 },
    { 0x2c209940, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba809944, SUC_lexicon_tags_33 },
    { 0x6de09945, SUC_lexicon_tags_12 },
    { 0xb2409946, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800994d, SUC_lexicon_tags_4 },
    { 0x5260994d, SUC_lexicon_tags_6 },
    { 0x97e0994f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b209951, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x26e09953, SUC_lexicon_tags_45 },
    { 0x68609954, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc09958, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1160995f, SUC_lexicon_tags_11 },
    { 0x97c09960, SUC_lexicon_tags_11 },
    { 0x66009961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f409963, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8409966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe320996a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb7e0996c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8209971, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefa09973, SUC_lexicon_tags_59 },
    { 0x67809974, SUC_lexicon_tags_34 },
    { 0xdea09974, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad409978, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2c80997a, SUC_lexicon_tags_17 },
    { 0x3f20997a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c0997e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d609982, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2809987, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xce09989, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea60998d, SUC_lexicon_tags_46 },
    { 0xba20998e, SUC_lexicon_tags_25 },
    { 0x8c80998f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24809991, SUC_lexicon_tags_33 },
    { 0x21c09992, SUC_lexicon_tags_38 },
    { 0x46409993, SUC_lexicon_tags_29 },
    { 0x2ce09994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71009996, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c09999, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1f60999b, SUC_lexicon_tags_33 },
    { 0x4080999c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616099a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e099a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66099a7, SUC_lexicon_tags_29 },
    { 0xb9a099a8, SUC_lexicon_tags_47 },
    { 0xe54099a9, SUC_lexicon_tags_61 },
    { 0x20a099a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34099ad, SUC_lexicon_tags_84 },
    { 0x6ac099ae, SUC_lexicon_tags_32 },
    { 0x6099af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820099b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28099bc, SUC_lexicon_tags_27 },
    { 0x634099bd, SUC_lexicon_tags_19 },
    { 0x326099bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x35c099c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270099cb, SUC_lexicon_tags_12 },
    { 0x22099cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02099d4, SUC_lexicon_tags_33 },
    { 0xaa099d5, SUC_lexicon_tags_39 },
    { 0xd5a099d4, SUC_lexicon_tags_19 },
    { 0x8c8099d7, SUC_lexicon_tags_25 },
    { 0xd20099d8, SUC_lexicon_tags_8 },
    { 0xeb2099d5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb1c099db, SUC_lexicon_tags_37 },
    { 0x7d8099db, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a099e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6099e7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec099f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf74099f2, SUC_lexicon_tags_61 },
    { 0xa86099f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbb6099f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94099f7, SUC_lexicon_tags_47 },
    { 0xbb6099f8, SUC_lexicon_tags_8 },
    { 0xa94099f7, SUC_lexicon_tags_17 },
    { 0xf62099fa, SUC_lexicon_tags_7 },
    { 0x25c099fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8099fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae809a00, SUC_lexicon_tags_66 },
    { 0x1d609a00, SUC_lexicon_tags_15 },
    { 0x75c09a02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3009a05, SUC_lexicon_tags_25 },
    { 0x8f609a05, SUC_lexicon_tags_25 },
    { 0x78209a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c09a0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4009a11, SUC_lexicon_tags_37 },
    { 0x32809a12, SUC_lexicon_tags_19 },
    { 0xde09a13, SUC_lexicon_tags_37 },
    { 0xda09a14, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac09a19, SUC_lexicon_tags_15 },
    { 0xf4e09a1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8de09a1c, SUC_lexicon_tags_28 },
    { 0x20e09a1c, SUC_lexicon_tags_34 },
    { 0x9a409a1e, SUC_lexicon_tags_47 },
    { 0xb7609a1d, SUC_lexicon_tags_25 },
    { 0x28809a1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad409a22, SUC_lexicon_tags_0 },
    { 0x9c809a22, SUC_lexicon_tags_81 },
    { 0xd5809a24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x18609a27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e09a2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c09a36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e09a3d, SUC_lexicon_tags_7 },
    { 0x5e809a3d, SUC_lexicon_tags_17 },
    { 0xb5609a3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4609a46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be09a4c, SUC_lexicon_tags_64 },
    { 0xeba09a4c, SUC_lexicon_tags_15 },
    { 0xdc809a4e, SUC_lexicon_tags_1 },
    { 0x6e609a4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac609a54, SUC_lexicon_tags_12 },
    { 0xa09a54, SUC_lexicon_tags_11 },
    { 0xc6209a55, SUC_lexicon_tags_11 },
    { 0x4d009a57, SUC_lexicon_tags_65 },
    { 0xaa609a58, SUC_lexicon_tags_2 },
    { 0xae09a59, SUC_lexicon_tags_38 },
    { 0x77409a56, SUC_lexicon_tags_43 },
    { 0x8b609a5b, SUC_lexicon_tags_53 },
    { 0x87c09a5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48409a5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4809a66, SUC_lexicon_tags_17 },
    { 0x43e09a67, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d209a6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3209a6c, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a09a6f, SUC_lexicon_tags_23 },
    { 0x26209a70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4809a79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3809a7b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58609a81, SUC_lexicon_tags_17 },
    { 0x3b809a81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4409a85, SUC_lexicon_tags_11 },
    { 0xa5a09a86, SUC_lexicon_tags_66 },
    { 0x32c09a85, SUC_lexicon_tags_295 },
    { 0x55e09a87, SUC_lexicon_tags_11 },
    { 0x38209a89, SUC_lexicon_tags_31 },
    { 0x97a09a89, SUC_lexicon_tags_14 },
    { 0x41a09a86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12209a98, SUC_lexicon_tags_11 },
    { 0x45e09a98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5609a9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd009a9f, SUC_lexicon_tags_7 },
    { 0xd8609a9f, SUC_lexicon_tags_23 },
    { 0xaa609a9f, SUC_lexicon_tags_68 },
    { 0x8c009aa2, SUC_lexicon_tags_38 },
    { 0xe0609a9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5009aa6, SUC_lexicon_tags_27 },
    { 0xc0a09aa6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a09aaa, SUC_lexicon_tags_33 },
    { 0x68e09aaa, SUC_lexicon_tags_2 },
    { 0x81009aab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64209ab4, SUC_lexicon_tags_12 },
    { 0x63e09ab5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc09ab9, SUC_lexicon_tags_66 },
    { 0x5d809aba, SUC_lexicon_tags_7 },
    { 0x46a09abb, SUC_lexicon_tags_24 },
    { 0x47409abc, SUC_lexicon_tags_25 },
    { 0x12009abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40009acc, SUC_lexicon_tags_2 },
    { 0x93809acd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72009ad5, SUC_lexicon_tags_35 },
    { 0x5fc09ad6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc0e09ad8, SUC_lexicon_tags_23 },
    { 0x1809ad8, SUC_lexicon_tags_23 },
    { 0x62409ada, SUC_lexicon_tags_11 },
    { 0xdf409ada, SUC_lexicon_tags_13 },
    { 0x9c609adc, SUC_lexicon_tags_7 },
    { 0xc6c09add, SUC_lexicon_tags_30 },
    { 0x3c809ade, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5609ae0, SUC_lexicon_tags_65 },
    { 0xa9e09ae0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43209ae5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaca09ae7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x26c09ae9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82609aee, SUC_lexicon_tags_25 },
    { 0x32209aef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc09af2, SUC_lexicon_tags_47 },
    { 0x3be09af3, SUC_lexicon_tags_2 },
    { 0x6aa09af3, SUC_lexicon_tags_35 },
    { 0x9f209af5, SUC_lexicon_tags_3 },
    { 0x55009af6, SUC_lexicon_tags_1 },
    { 0xeb809af7, SUC_lexicon_tags_23 },
    { 0x56009af7, SUC_lexicon_tags_17 },
    { 0x53409af8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e09afc, SUC_lexicon_tags_27 },
    { 0x2609afd, SUC_lexicon_tags_15 },
    { 0xd3e09afe, SUC_lexicon_tags_15 },
    { 0x8f409aff, SUC_lexicon_tags_13 },
    { 0xafc09afc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a09b03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9b05, SUC_lexicon_tags_25 },
    { 0xcd409b06, SUC_lexicon_tags_57 },
    { 0xb1009b07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96409b0b, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xd0609b0d, SUC_lexicon_tags_816 },
    { 0, NULL },
    { 0xe7609b0f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc7e09b11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a409b15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a09b1c, SUC_lexicon_tags_17 },
    { 0x18209b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c409b22, SUC_lexicon_tags_25 },
    { 0x7a09b23, SUC_lexicon_tags_11 },
    { 0xf8409b23, SUC_lexicon_tags_11 },
    { 0xc809b24, SUC_lexicon_tags_56 },
    { 0x1da09b26, SUC_lexicon_tags_31 },
    { 0xde609b23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e409b2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a09b2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91409b3d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x36c09b3f, SUC_lexicon_tags_9 },
    { 0x93e09b40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a09b44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee09b49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x63809b4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e09b4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa409b52, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a09b56, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x51609b58, SUC_lexicon_tags_44 },
    { 0xec09b59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a09b5e, SUC_lexicon_tags_27 },
    { 0x8da09b5f, SUC_lexicon_tags_11 },
    { 0x73a09b60, SUC_lexicon_tags_46 },
    { 0x43609b61, SUC_lexicon_tags_25 },
    { 0xef409b61, SUC_lexicon_tags_63 },
    { 0x77209b61, SUC_lexicon_tags_1 },
    { 0xd5009b62, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xad409b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84009b69, SUC_lexicon_tags_12 },
    { 0x70609b6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x38009b6d, SUC_lexicon_tags_7 },
    { 0xa4c09b6e, SUC_lexicon_tags_62 },
    { 0x75809b6d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x58a09b71, SUC_lexicon_tags_87 },
    { 0xab809b72, SUC_lexicon_tags_26 },
    { 0xee209b73, SUC_lexicon_tags_18 },
    { 0x2fa09b74, SUC_lexicon_tags_37 },
    { 0x1a209b72, SUC_lexicon_tags_49 },
    { 0xfe209b76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1209b7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de09b83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ec09b85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66609b87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0809b89, SUC_lexicon_tags_7 },
    { 0xc8609b8a, SUC_lexicon_tags_29 },
    { 0xb409b8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86809b8e, SUC_lexicon_tags_7 },
    { 0x9409b8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfc09b91, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3be09b93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a009b96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5009b98, SUC_lexicon_tags_32 },
    { 0xb7409b99, SUC_lexicon_tags_59 },
    { 0xa1a09b9a, SUC_lexicon_tags_17 },
    { 0x74c09b9b, SUC_lexicon_tags_0 },
    { 0x10c09b9c, SUC_lexicon_tags_7 },
    { 0xf2e09b9b, SUC_lexicon_tags_38 },
    { 0x7f009b9a, SUC_lexicon_tags_37 },
    { 0xa3809b9c, SUC_lexicon_tags_37 },
    { 0x2a09ba0, SUC_lexicon_tags_49 },
    { 0x20609b9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ee09ba3, SUC_lexicon_tags_17 },
    { 0x2ce09ba3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b009ba6, SUC_lexicon_tags_6 },
    { 0xf0209ba7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25009bac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48009bb2, SUC_lexicon_tags_34 },
    { 0x10a09bb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69009bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a09bb9, SUC_lexicon_tags_37 },
    { 0x15609bb9, SUC_lexicon_tags_112 },
    { 0x8e09bba, SUC_lexicon_tags_37 },
    { 0x10a09bbc, SUC_lexicon_tags_7 },
    { 0xa3809bb9, SUC_lexicon_tags_23 },
    { 0x57809bbe, SUC_lexicon_tags_12 },
    { 0x5b009bbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a09bc3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x45c09bc5, SUC_lexicon_tags_43 },
    { 0x58809bc5, SUC_lexicon_tags_25 },
    { 0x21009bc7, SUC_lexicon_tags_17 },
    { 0xdfa09bc8, SUC_lexicon_tags_17 },
    { 0x51809bc6, SUC_lexicon_tags_23 },
    { 0xef209bca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba09bd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25009bde, SUC_lexicon_tags_49 },
    { 0x6609bdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x38c09be1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e09be4, SUC_lexicon_tags_45 },
    { 0xc5209be4, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x1009be7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd409bec, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c09bfa, SUC_lexicon_tags_4 },
    { 0x43809bfb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89209c00, SUC_lexicon_tags_84 },
    { 0xa5e09c00, SUC_lexicon_tags_36 },
    { 0xe3409c02, SUC_lexicon_tags_30 },
    { 0x6c809c03, SUC_lexicon_tags_6 },
    { 0x82609c04, SUC_lexicon_tags_11 },
    { 0x79609c05, SUC_lexicon_tags_68 },
    { 0xe09c05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe0c09c08, SUC_lexicon_tags_15 },
    { 0x47a09c08, SUC_lexicon_tags_60 },
    { 0x96809c09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99609c0c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a09c12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8009c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ee09c18, SUC_lexicon_tags_45 },
    { 0x96409c19, SUC_lexicon_tags_25 },
    { 0xbbc09c1a, SUC_lexicon_tags_12 },
    { 0xc4609c1a, SUC_lexicon_tags_23 },
    { 0x88a09c1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb409c20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27c09c22, SUC_lexicon_tags_2 },
    { 0x49609c23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83809c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc809c2d, SUC_lexicon_tags_37 },
    { 0x1fc09c2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x24009c31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65609c34, SUC_lexicon_tags_25 },
    { 0xaea09c35, SUC_lexicon_tags_17 },
    { 0xffe09c35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c409c3a, SUC_lexicon_tags_27 },
    { 0xc4e09c3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7009c42, SUC_lexicon_tags_8 },
    { 0x2f809c42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27e09c45, SUC_lexicon_tags_66 },
    { 0x85c09c46, SUC_lexicon_tags_79 },
    { 0x3ca09c45, SUC_lexicon_tags_2 },
    { 0xaa209c48, SUC_lexicon_tags_37 },
    { 0xea009c49, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec09c4e, SUC_lexicon_tags_55 },
    { 0x40c09c4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae609c57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6609c60, SUC_lexicon_tags_23 },
    { 0x29209c61, SUC_lexicon_tags_2 },
    { 0xe8409c61, SUC_lexicon_tags_25 },
    { 0xac809c62, SUC_lexicon_tags_68 },
    { 0x55809c60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6409c66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1fc09c68, SUC_lexicon_tags_17 },
    { 0xbbe09c69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e09c73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e609c77, SUC_lexicon_tags_12 },
    { 0xe5609c77, SUC_lexicon_tags_17 },
    { 0x1c809c79, SUC_lexicon_tags_31 },
    { 0xb2a09c77, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa2a09c7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2809c7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e209c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1609c88, SUC_lexicon_tags_61 },
    { 0x7e609c89, SUC_lexicon_tags_32 },
    { 0xe3609c8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e09c8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea09c94, SUC_lexicon_tags_428 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2609c97, SUC_lexicon_tags_68 },
    { 0x34809c98, SUC_lexicon_tags_0 },
    { 0x82c09c98, SUC_lexicon_tags_6 },
    { 0x68609c99, SUC_lexicon_tags_25 },
    { 0x32209c9b, SUC_lexicon_tags_15 },
    { 0x12409c9c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x43809c9e, SUC_lexicon_tags_7 },
    { 0x36a09c9f, SUC_lexicon_tags_66 },
    { 0xdb809ca0, SUC_lexicon_tags_11 },
    { 0xf9609ca1, SUC_lexicon_tags_12 },
    { 0xc2609c9f, SUC_lexicon_tags_7 },
    { 0xa6209ca3, SUC_lexicon_tags_37 },
    { 0xeb409ca3, SUC_lexicon_tags_17 },
    { 0xb5e09ca0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da09ca8, SUC_lexicon_tags_1 },
    { 0x76c09ca9, SUC_lexicon_tags_17 },
    { 0x61009ca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6609cac, SUC_lexicon_tags_741 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce09caf, SUC_lexicon_tags_33 },
    { 0x5e09cb0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc09cb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e409cb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b409cbc, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xdde09cbe, SUC_lexicon_tags_38 },
    { 0x68209cbf, SUC_lexicon_tags_11 },
    { 0x86809cbf, SUC_lexicon_tags_4 },
    { 0x3ba09cc0, SUC_lexicon_tags_15 },
    { 0x93409cc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfce09cc4, SUC_lexicon_tags_47 },
    { 0x3c609cc5, SUC_lexicon_tags_15 },
    { 0x23209cc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc409cc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77009ccb, SUC_lexicon_tags_15 },
    { 0x89a09ccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfba09cce, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a809cd1, SUC_lexicon_tags_29 },
    { 0x7a09cd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c09cd5, SUC_lexicon_tags_26 },
    { 0x5f409cd5, SUC_lexicon_tags_17 },
    { 0x64a09cd7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce09cdb, SUC_lexicon_tags_28 },
    { 0x53809cdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5809cde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1c09ce0, SUC_lexicon_tags_38 },
    { 0x3d009ce1, SUC_lexicon_tags_1 },
    { 0x3be09ce2, SUC_lexicon_tags_12 },
    { 0xa3009ce1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe209cea, SUC_lexicon_tags_25 },
    { 0xf1209ceb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8009cee, SUC_lexicon_tags_14 },
    { 0x3c609cef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a409cf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1609cf3, SUC_lexicon_tags_13 },
    { 0x6e809cf3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x40009cf6, SUC_lexicon_tags_65 },
    { 0xd4409cf6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0609cfd, SUC_lexicon_tags_35 },
    { 0x4e209cfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0409d01, SUC_lexicon_tags_49 },
    { 0xac209d01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1609d06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d409d08, SUC_lexicon_tags_4 },
    { 0x8d409d09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c09d0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f409d13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46009d15, SUC_lexicon_tags_37 },
    { 0x65c09d16, SUC_lexicon_tags_37 },
    { 0xdee09d17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c609d19, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe2609d1b, SUC_lexicon_tags_25 },
    { 0xf4409d1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6409d1e, SUC_lexicon_tags_1 },
    { 0xd7409d1e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed009d2a, SUC_lexicon_tags_15 },
    { 0x2a609d2a, SUC_lexicon_tags_47 },
    { 0xd609d2b, SUC_lexicon_tags_66 },
    { 0x71009d2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5409d34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e09d38, SUC_lexicon_tags_11 },
    { 0xacc09d39, SUC_lexicon_tags_6 },
    { 0x49809d39, SUC_lexicon_tags_7 },
    { 0x48009d3b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c409d3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46209d46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea609d48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf609d4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18009d4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18609d52, SUC_lexicon_tags_61 },
    { 0x209d53, SUC_lexicon_tags_16 },
    { 0xee09d53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5409d59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a09d5c, SUC_lexicon_tags_27 },
    { 0xad609d5c, SUC_lexicon_tags_1 },
    { 0xcb809d5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a09d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd409d65, SUC_lexicon_tags_133 },
    { 0xcfc09d65, SUC_lexicon_tags_45 },
    { 0xa7609d67, SUC_lexicon_tags_30 },
    { 0xb2809d68, SUC_lexicon_tags_12 },
    { 0xe2809d66, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xbd209d6b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x65609d6d, SUC_lexicon_tags_27 },
    { 0x89c09d6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x17409d70, SUC_lexicon_tags_11 },
    { 0xff209d70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d409d79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xac09d7c, SUC_lexicon_tags_7 },
    { 0xac409d7c, SUC_lexicon_tags_38 },
    { 0x58409d7e, SUC_lexicon_tags_30 },
    { 0xdfc09d7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52009d81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d209d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e609d85, SUC_lexicon_tags_768 },
    { 0x90c09d85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa7a09d88, SUC_lexicon_tags_12 },
    { 0x41e09d89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdda09d8b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60609d8f, SUC_lexicon_tags_6 },
    { 0x2d809d90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf9e09d92, SUC_lexicon_tags_14 },
    { 0x55809d92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab409d95, SUC_lexicon_tags_60 },
    { 0xdcc09d96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15009d99, SUC_lexicon_tags_7 },
    { 0x5bc09d9a, SUC_lexicon_tags_7 },
    { 0x47e09d9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3609d9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7409da1, SUC_lexicon_tags_7 },
    { 0x8c609da2, SUC_lexicon_tags_9 },
    { 0x50c09da3, SUC_lexicon_tags_410 },
    { 0x90209da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f409daa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7809dad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59e09daf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21609db1, SUC_lexicon_tags_84 },
    { 0x1da09db2, SUC_lexicon_tags_11 },
    { 0x77209db2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e09db8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb009dbe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe5409dc0, SUC_lexicon_tags_12 },
    { 0xd0809dc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78209dc5, SUC_lexicon_tags_17 },
    { 0xb0209dc6, SUC_lexicon_tags_79 },
    { 0x58809dc5, SUC_lexicon_tags_22 },
    { 0x18209dc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7209dca, SUC_lexicon_tags_38 },
    { 0xcc209dcb, SUC_lexicon_tags_17 },
    { 0xd7209dcc, SUC_lexicon_tags_6 },
    { 0x1bc09dcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c09dd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x91809dd4, SUC_lexicon_tags_37 },
    { 0x28a09dd4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6009ddd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbbe09ddf, SUC_lexicon_tags_39 },
    { 0xb5409de0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee809df2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9409df5, SUC_lexicon_tags_40 },
    { 0x18809df5, SUC_lexicon_tags_47 },
    { 0xb9e09df5, SUC_lexicon_tags_40 },
    { 0xf9e09df8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3209dfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2809dfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4e09dfe, SUC_lexicon_tags_43 },
    { 0x2aa09dff, SUC_lexicon_tags_25 },
    { 0x2fc09e00, SUC_lexicon_tags_15 },
    { 0xf4e09e00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37209e03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1409e07, SUC_lexicon_tags_19 },
    { 0x7de09e07, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc09e0c, SUC_lexicon_tags_25 },
    { 0x5f009e0d, SUC_lexicon_tags_25 },
    { 0x95409e0e, SUC_lexicon_tags_82 },
    { 0x86209e0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6009e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2609e15, SUC_lexicon_tags_2 },
    { 0x51609e16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e209e18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85209e1a, SUC_lexicon_tags_124 },
    { 0x17e09e1b, SUC_lexicon_tags_70 },
    { 0x48009e1b, SUC_lexicon_tags_12 },
    { 0xac209e1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c09e22, SUC_lexicon_tags_66 },
    { 0x81809e23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0c09e25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9409e2d, SUC_lexicon_tags_27 },
    { 0xdd409e2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e09e31, SUC_lexicon_tags_17 },
    { 0x8f009e32, SUC_lexicon_tags_30 },
    { 0x25e09e33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0009e36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15609e3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f609e3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c09e42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3609e45, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x73409e47, SUC_lexicon_tags_25 },
    { 0x8cc09e48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x49609e4a, SUC_lexicon_tags_29 },
    { 0xd6209e4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59209e4f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xede09e51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce09e56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee09e59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe09e5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x69809e60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc09e65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c09e68, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e09e6e, SUC_lexicon_tags_60 },
    { 0xa2409e6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77209e71, SUC_lexicon_tags_31 },
    { 0x26c09e72, SUC_lexicon_tags_0 },
    { 0xae609e71, SUC_lexicon_tags_14 },
    { 0xbbe09e72, SUC_lexicon_tags_1 },
    { 0xdf609e75, SUC_lexicon_tags_40 },
    { 0x14609e75, SUC_lexicon_tags_15 },
    { 0xcb009e76, SUC_lexicon_tags_66 },
    { 0x63409e78, SUC_lexicon_tags_27 },
    { 0xef009e71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ee09e7b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac209e80, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6809e82, SUC_lexicon_tags_0 },
    { 0xefc09e83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e09e86, SUC_lexicon_tags_45 },
    { 0x53209e87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea09e8f, SUC_lexicon_tags_2 },
    { 0x8d409e90, SUC_lexicon_tags_14 },
    { 0xf0009e91, SUC_lexicon_tags_1 },
    { 0x7b809e91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9609e94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b209e97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e09e9c, SUC_lexicon_tags_17 },
    { 0x60c09e9d, SUC_lexicon_tags_23 },
    { 0xd4409e9e, SUC_lexicon_tags_14 },
    { 0x95609e9e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65009ea3, SUC_lexicon_tags_39 },
    { 0x26609ea4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39209ea7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c209eac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11809eaf, SUC_lexicon_tags_28 },
    { 0x49e09eaf, SUC_lexicon_tags_3 },
    { 0xb0c09eb0, SUC_lexicon_tags_8 },
    { 0x31809eb2, SUC_lexicon_tags_45 },
    { 0xafa09eb3, SUC_lexicon_tags_7 },
    { 0x18c09eb4, SUC_lexicon_tags_136 },
    { 0x69a09eb3, SUC_lexicon_tags_12 },
    { 0x47e09eb6, SUC_lexicon_tags_21 },
    { 0xd809eb6, SUC_lexicon_tags_85 },
    { 0x70009eb8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfa209eba, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7209ebd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b809ec0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e09ec8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe809ecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3009ed1, SUC_lexicon_tags_27 },
    { 0x34c09ed2, SUC_lexicon_tags_14 },
    { 0x55409ed2, SUC_lexicon_tags_12 },
    { 0x29409ed4, SUC_lexicon_tags_12 },
    { 0x6fa09ed5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba209ed8, SUC_lexicon_tags_11 },
    { 0x7b809ed8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5f809edb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde09ee1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88809eec, SUC_lexicon_tags_17 },
    { 0x4d209eed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc5809eef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc09ef4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7cc09ef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49009efa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5809efd, SUC_lexicon_tags_6 },
    { 0x5de09efe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8c009f00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e009f04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0009f08, SUC_lexicon_tags_1 },
    { 0x1b009f09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31e09f0b, SUC_lexicon_tags_2 },
    { 0x34609f0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8c09f0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2209f16, SUC_lexicon_tags_68 },
    { 0xba809f17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e09f1a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x5c609f1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96009f1f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfa409f21, SUC_lexicon_tags_27 },
    { 0xabc09f22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7809f2c, SUC_lexicon_tags_24 },
    { 0x12009f2d, SUC_lexicon_tags_37 },
    { 0xcfc09f2e, SUC_lexicon_tags_17 },
    { 0xc6209f2c, SUC_lexicon_tags_41 },
    { 0x4c09f2d, SUC_lexicon_tags_14 },
    { 0xae809f30, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62609f35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9209f3c, SUC_lexicon_tags_34 },
    { 0xc9809f3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38609f3f, SUC_lexicon_tags_25 },
    { 0x25209f3f, SUC_lexicon_tags_6 },
    { 0x9ea09f40, SUC_lexicon_tags_11 },
    { 0xcd009f3f, SUC_lexicon_tags_62 },
    { 0x73609f40, SUC_lexicon_tags_49 },
    { 0x6f809f3f, SUC_lexicon_tags_15 },
    { 0x4a009f45, SUC_lexicon_tags_28 },
    { 0x33409f44, SUC_lexicon_tags_25 },
    { 0x5ce09f47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48009f4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe09f53, SUC_lexicon_tags_17 },
    { 0x7a809f54, SUC_lexicon_tags_13 },
    { 0xa5e09f55, SUC_lexicon_tags_4 },
    { 0x6c209f54, SUC_lexicon_tags_37 },
    { 0x9c609f53, SUC_lexicon_tags_14 },
    { 0xd9609f58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c09f60, SUC_lexicon_tags_39 },
    { 0x16209f61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1e09f63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a09f66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2c09f68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c09f6c, SUC_lexicon_tags_37 },
    { 0x93c09f6c, SUC_lexicon_tags_1 },
    { 0xc09f6c, SUC_lexicon_tags_52 },
    { 0xae209f6f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe09f74, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x20409f76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea409f80, SUC_lexicon_tags_1 },
    { 0x7dc09f80, SUC_lexicon_tags_31 },
    { 0x61209f82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57209f86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a809f89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25809f8c, SUC_lexicon_tags_63 },
    { 0xf7e09f8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfa009f8f, SUC_lexicon_tags_27 },
    { 0xea09f8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fc09f92, SUC_lexicon_tags_28 },
    { 0xfb209f93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2409f99, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x80009f9c, SUC_lexicon_tags_6 },
    { 0xd5e09f9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ee09f9f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd2e09fa1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82409fa7, SUC_lexicon_tags_12 },
    { 0x9b809fa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2409fac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d809fae, SUC_lexicon_tags_12 },
    { 0x87609fae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f809fb2, SUC_lexicon_tags_47 },
    { 0x44c09fb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e009fb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad409fba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde09fbe, SUC_lexicon_tags_43 },
    { 0x78409fbf, SUC_lexicon_tags_1 },
    { 0x39009fc0, SUC_lexicon_tags_11 },
    { 0xcd209fc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78209fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8a09fc5, SUC_lexicon_tags_2 },
    { 0xfbc09fc6, SUC_lexicon_tags_28 },
    { 0x19409fc6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45209fd0, SUC_lexicon_tags_108 },
    { 0x3e809fd1, SUC_lexicon_tags_1 },
    { 0xd2e09fd1, SUC_lexicon_tags_17 },
    { 0x19409fd0, SUC_lexicon_tags_7 },
    { 0xb209fd4, SUC_lexicon_tags_17 },
    { 0x6b209fd5, SUC_lexicon_tags_19 },
    { 0x9da09fd6, SUC_lexicon_tags_69 },
    { 0x4e09fd0, SUC_lexicon_tags_7 },
    { 0x1d609fd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa09fdb, SUC_lexicon_tags_17 },
    { 0xf8009fdc, SUC_lexicon_tags_12 },
    { 0xe7c09fdc, SUC_lexicon_tags_2 },
    { 0xce009fdc, SUC_lexicon_tags_47 },
    { 0xc2a09fdb, SUC_lexicon_tags_2 },
    { 0x6be09fdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54809fe6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61409fea, SUC_lexicon_tags_63 },
    { 0x7a809fea, SUC_lexicon_tags_816 },
    { 0x29209fec, SUC_lexicon_tags_7 },
    { 0x49209fed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce09fef, SUC_lexicon_tags_48 },
    { 0xf8a09fef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a09ff6, SUC_lexicon_tags_40 },
    { 0x1f809ff6, SUC_lexicon_tags_4 },
    { 0x6e009ff8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48209ffc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa880a001, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a0a005, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xde80a008, SUC_lexicon_tags_15 },
    { 0x460a008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee0a00c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a20a012, SUC_lexicon_tags_11 },
    { 0xd4a0a013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x120a015, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c0a019, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe5a0a01b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e0a01f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ec0a021, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c0a025, SUC_lexicon_tags_12 },
    { 0xd8e0a026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9860a028, SUC_lexicon_tags_7 },
    { 0xc3c0a029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c0a02e, SUC_lexicon_tags_1 },
    { 0xd760a02f, SUC_lexicon_tags_17 },
    { 0xbc60a02f, SUC_lexicon_tags_15 },
    { 0x5d20a02f, SUC_lexicon_tags_37 },
    { 0x6d60a02f, SUC_lexicon_tags_17 },
    { 0xb960a02f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6260a035, SUC_lexicon_tags_27 },
    { 0xe160a035, SUC_lexicon_tags_11 },
    { 0x1ac0a035, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1720a03a, SUC_lexicon_tags_37 },
    { 0xc800a03b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840a040, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0a046, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2700a048, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c0a04d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a0a050, SUC_lexicon_tags_5 },
    { 0xa2a0a051, SUC_lexicon_tags_11 },
    { 0xb6e0a051, SUC_lexicon_tags_4 },
    { 0x620a053, SUC_lexicon_tags_8 },
    { 0x8e80a054, SUC_lexicon_tags_17 },
    { 0x37e0a050, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d20a057, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d40a064, SUC_lexicon_tags_19 },
    { 0x1b00a065, SUC_lexicon_tags_63 },
    { 0xa20a065, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1460a068, SUC_lexicon_tags_14 },
    { 0xb4e0a069, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea0a06d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce20a077, SUC_lexicon_tags_12 },
    { 0xd840a078, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b00a07b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xef80a07d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0a080, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bc0a082, SUC_lexicon_tags_11 },
    { 0xfba0a083, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c0a087, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd340a089, SUC_lexicon_tags_37 },
    { 0x240a08a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca80a092, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40a0a094, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4a0a096, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba0a098, SUC_lexicon_tags_0 },
    { 0xe920a099, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9a0a09b, SUC_lexicon_tags_77 },
    { 0xa860a09b, SUC_lexicon_tags_37 },
    { 0x2660a09d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e0a0aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa0a0b1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe480a0b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc0a0b8, SUC_lexicon_tags_15 },
    { 0xad00a0b9, SUC_lexicon_tags_12 },
    { 0xb140a0ba, SUC_lexicon_tags_37 },
    { 0x7fc0a0b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcb20a0bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x100a0c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e0a0c7, SUC_lexicon_tags_46 },
    { 0x9700a0c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd120a0ca, SUC_lexicon_tags_0 },
    { 0x67a0a0ca, SUC_lexicon_tags_12 },
    { 0x3b80a0cc, SUC_lexicon_tags_2 },
    { 0xa460a0cd, SUC_lexicon_tags_25 },
    { 0x2000a0ce, SUC_lexicon_tags_37 },
    { 0x4880a0cf, SUC_lexicon_tags_32 },
    { 0x30e0a0cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd240a0d5, SUC_lexicon_tags_1 },
    { 0xde0a0d5, SUC_lexicon_tags_17 },
    { 0xe00a0d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb400a0d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa520a0dc, SUC_lexicon_tags_25 },
    { 0xc2a0a0dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a0a0e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b40a0e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1080a0e6, SUC_lexicon_tags_0 },
    { 0x8160a0e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c0a0ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4e0a0ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2160a0f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a0a0fd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf540a100, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c0a103, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5540a10a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xaf00a10c, SUC_lexicon_tags_38 },
    { 0x2a40a10d, SUC_lexicon_tags_322 },
    { 0xb1e0a10c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c0a110, SUC_lexicon_tags_26 },
    { 0xb960a111, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf00a116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a0a119, SUC_lexicon_tags_63 },
    { 0x2a20a119, SUC_lexicon_tags_25 },
    { 0xbb60a11b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a40a11d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa20a121, SUC_lexicon_tags_47 },
    { 0x7ea0a122, SUC_lexicon_tags_11 },
    { 0xb360a123, SUC_lexicon_tags_148 },
    { 0x15c0a122, SUC_lexicon_tags_9 },
    { 0xc1e0a124, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77c0a127, SUC_lexicon_tags_25 },
    { 0x12e0a128, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc00a12c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe600a137, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a80a13a, SUC_lexicon_tags_1 },
    { 0x94a0a13b, SUC_lexicon_tags_29 },
    { 0x1560a13c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe920a140, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d40a144, SUC_lexicon_tags_23 },
    { 0xce0a145, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7240a14b, SUC_lexicon_tags_151 },
    { 0x6d00a14b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a0a14f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c0a152, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e0a155, SUC_lexicon_tags_15 },
    { 0x7ae0a156, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6900a158, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3120a15c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5200a165, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd700a16b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb160a16e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a0a173, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe680a176, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x700a179, SUC_lexicon_tags_60 },
    { 0xd7a0a17a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb8a0a17c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a180, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd020a183, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe480a186, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb180a18e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9100a191, SUC_lexicon_tags_60 },
    { 0x9f60a192, SUC_lexicon_tags_3 },
    { 0xa1c0a193, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9780a198, SUC_lexicon_tags_4 },
    { 0x7b80a198, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf500a19b, SUC_lexicon_tags_47 },
    { 0x4940a19c, SUC_lexicon_tags_12 },
    { 0xbd80a19d, SUC_lexicon_tags_19 },
    { 0x7200a19c, SUC_lexicon_tags_15 },
    { 0xb980a19f, SUC_lexicon_tags_45 },
    { 0xeae0a19f, SUC_lexicon_tags_11 },
    { 0x2e20a19e, SUC_lexicon_tags_23 },
    { 0x3120a1a2, SUC_lexicon_tags_4 },
    { 0x1ac0a1a1, SUC_lexicon_tags_49 },
    { 0xfca0a1a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e0a1a9, SUC_lexicon_tags_9 },
    { 0xf080a1aa, SUC_lexicon_tags_15 },
    { 0xc120a1aa, SUC_lexicon_tags_87 },
    { 0xff20a1ac, SUC_lexicon_tags_1 },
    { 0xcee0a1ab, SUC_lexicon_tags_25 },
    { 0x3e20a1ad, SUC_lexicon_tags_26 },
    { 0x6040a1aa, SUC_lexicon_tags_1 },
    { 0xa940a1b0, SUC_lexicon_tags_8 },
    { 0x300a1aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c0a1b4, SUC_lexicon_tags_23 },
    { 0x86e0a1b5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6200a1b8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x45e0a1ba, SUC_lexicon_tags_15 },
    { 0x1800a1ba, SUC_lexicon_tags_2 },
    { 0xcd20a1bc, SUC_lexicon_tags_15 },
    { 0x9aa0a1bc, SUC_lexicon_tags_30 },
    { 0x7700a1bd, SUC_lexicon_tags_17 },
    { 0xed40a1bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3280a1c1, SUC_lexicon_tags_2 },
    { 0x82c0a1c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc520a1c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc800a1cd, SUC_lexicon_tags_20 },
    { 0x3a40a1ce, SUC_lexicon_tags_25 },
    { 0x6100a1cf, SUC_lexicon_tags_23 },
    { 0x560a1d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2080a1d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd940a1d6, SUC_lexicon_tags_63 },
    { 0x5180a1d6, SUC_lexicon_tags_66 },
    { 0x4300a1d8, SUC_lexicon_tags_37 },
    { 0xf040a1d9, SUC_lexicon_tags_54 },
    { 0x6200a1d7, SUC_lexicon_tags_33 },
    { 0x660a1d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd780a1de, SUC_lexicon_tags_35 },
    { 0xbf60a1df, SUC_lexicon_tags_15 },
    { 0xf460a1df, SUC_lexicon_tags_9 },
    { 0xb260a1e0, SUC_lexicon_tags_69 },
    { 0xd560a1e2, SUC_lexicon_tags_48 },
    { 0x4720a1e3, SUC_lexicon_tags_32 },
    { 0x47e0a1e3, SUC_lexicon_tags_7 },
    { 0xe100a1e2, SUC_lexicon_tags_26 },
    { 0x7800a1e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ca0a1e8, SUC_lexicon_tags_2 },
    { 0x960a1e8, SUC_lexicon_tags_17 },
    { 0x4740a1e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a0a1ed, SUC_lexicon_tags_15 },
    { 0xb5c0a1ee, SUC_lexicon_tags_0 },
    { 0x9d40a1ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ee0a1f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb80a1f3, SUC_lexicon_tags_45 },
    { 0xc980a1f4, SUC_lexicon_tags_39 },
    { 0x60a0a1f4, SUC_lexicon_tags_11 },
    { 0x460a1f6, SUC_lexicon_tags_11 },
    { 0x1ac0a1f7, SUC_lexicon_tags_11 },
    { 0x3d60a1f8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x46a0a1fa, SUC_lexicon_tags_33 },
    { 0xf520a1fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6140a204, SUC_lexicon_tags_15 },
    { 0xd4e0a205, SUC_lexicon_tags_104 },
    { 0x6b80a206, SUC_lexicon_tags_47 },
    { 0x6e0a205, SUC_lexicon_tags_42 },
    { 0xdf60a208, SUC_lexicon_tags_14 },
    { 0x2480a209, SUC_lexicon_tags_37 },
    { 0xbd20a207, SUC_lexicon_tags_79 },
    { 0x1460a20a, SUC_lexicon_tags_29 },
    { 0x6040a204, SUC_lexicon_tags_15 },
    { 0xc60a204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b00a212, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb6e0a214, SUC_lexicon_tags_11 },
    { 0x4b40a214, SUC_lexicon_tags_60 },
    { 0x85a0a216, SUC_lexicon_tags_14 },
    { 0xa820a215, SUC_lexicon_tags_30 },
    { 0xb0a0a218, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb820a21a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x22e0a21c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe060a220, SUC_lexicon_tags_65 },
    { 0xa60a221, SUC_lexicon_tags_11 },
    { 0xc440a221, SUC_lexicon_tags_81 },
    { 0x69c0a223, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2040a228, SUC_lexicon_tags_16 },
    { 0xabc0a229, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4a0a22b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2220a22e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c0a232, SUC_lexicon_tags_27 },
    { 0x1920a233, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2020a238, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc0a23f, SUC_lexicon_tags_2 },
    { 0xa140a23f, SUC_lexicon_tags_1 },
    { 0x4760a241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe40a24d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7220a250, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a60a254, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e20a256, SUC_lexicon_tags_25 },
    { 0x1d40a257, SUC_lexicon_tags_29 },
    { 0x8f40a258, SUC_lexicon_tags_15 },
    { 0xe3a0a258, SUC_lexicon_tags_37 },
    { 0x7680a257, SUC_lexicon_tags_23 },
    { 0x5140a25b, SUC_lexicon_tags_37 },
    { 0x2320a257, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e0a264, SUC_lexicon_tags_37 },
    { 0x6aa0a265, SUC_lexicon_tags_2 },
    { 0x7840a266, SUC_lexicon_tags_38 },
    { 0x6280a265, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c40a26c, SUC_lexicon_tags_37 },
    { 0x3260a26d, SUC_lexicon_tags_55 },
    { 0x28a0a26e, SUC_lexicon_tags_37 },
    { 0xed80a26f, SUC_lexicon_tags_43 },
    { 0x8f40a270, SUC_lexicon_tags_7 },
    { 0x4dc0a271, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9480a273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce20a277, SUC_lexicon_tags_15 },
    { 0x5180a278, SUC_lexicon_tags_0 },
    { 0x98a0a279, SUC_lexicon_tags_33 },
    { 0x4aa0a277, SUC_lexicon_tags_70 },
    { 0x2080a27b, SUC_lexicon_tags_12 },
    { 0x8200a27c, SUC_lexicon_tags_63 },
    { 0x4b00a27a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x440a280, SUC_lexicon_tags_268 },
    { 0, NULL },
    { 0x9be0a282, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa020a284, SUC_lexicon_tags_15 },
    { 0x7000a284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c0a289, SUC_lexicon_tags_8 },
    { 0x3120a28a, SUC_lexicon_tags_43 },
    { 0x88e0a28a, SUC_lexicon_tags_25 },
    { 0x160a28c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8de0a28e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc0a291, SUC_lexicon_tags_0 },
    { 0xc00a292, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc0a297, SUC_lexicon_tags_29 },
    { 0xf100a297, SUC_lexicon_tags_15 },
    { 0x2ee0a299, SUC_lexicon_tags_68 },
    { 0x6080a299, SUC_lexicon_tags_2 },
    { 0xa920a29a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c60a29f, SUC_lexicon_tags_7 },
    { 0x6140a2a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f00a2a8, SUC_lexicon_tags_45 },
    { 0x7660a2a8, SUC_lexicon_tags_127 },
    { 0x6800a2aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39c0a2ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8820a2b3, SUC_lexicon_tags_7 },
    { 0x6620a2b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8400a2b6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xbf20a2b8, SUC_lexicon_tags_1 },
    { 0xf1a0a2b8, SUC_lexicon_tags_33 },
    { 0x98e0a2b8, SUC_lexicon_tags_43 },
    { 0x73c0a2b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc80a2be, SUC_lexicon_tags_1 },
    { 0xc360a2bf, SUC_lexicon_tags_60 },
    { 0x8020a2be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe960a2c4, SUC_lexicon_tags_37 },
    { 0x4400a2c5, SUC_lexicon_tags_11 },
    { 0x2aa0a2c6, SUC_lexicon_tags_2 },
    { 0x7b00a2c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda40a2cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c0a2ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e0a2d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3840a2d6, SUC_lexicon_tags_7 },
    { 0xff00a2d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7920a2d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32a0a2db, SUC_lexicon_tags_56 },
    { 0xfce0a2dc, SUC_lexicon_tags_10 },
    { 0x5fa0a2dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5460a2e2, SUC_lexicon_tags_37 },
    { 0xf480a2e3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1900a2e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e0a2ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8be0a2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a0a2f2, SUC_lexicon_tags_944 },
    { 0x1e40a2f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6740a2f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca0a2fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2300a301, SUC_lexicon_tags_2 },
    { 0x1700a302, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xca40a304, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5180a307, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x5380a30a, SUC_lexicon_tags_55 },
    { 0x8960a30b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5360a311, SUC_lexicon_tags_151 },
    { 0xa7a0a311, SUC_lexicon_tags_60 },
    { 0xccc0a311, SUC_lexicon_tags_11 },
    { 0xe180a312, SUC_lexicon_tags_43 },
    { 0xe2c0a315, SUC_lexicon_tags_7 },
    { 0xf0e0a316, SUC_lexicon_tags_2 },
    { 0x3c0a317, SUC_lexicon_tags_37 },
    { 0x1aa0a315, SUC_lexicon_tags_40 },
    { 0x9a40a319, SUC_lexicon_tags_55 },
    { 0x7020a31a, SUC_lexicon_tags_15 },
    { 0x3dc0a318, SUC_lexicon_tags_33 },
    { 0xb680a31c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3100a321, SUC_lexicon_tags_19 },
    { 0xa140a322, SUC_lexicon_tags_21 },
    { 0x9e40a323, SUC_lexicon_tags_37 },
    { 0x9680a323, SUC_lexicon_tags_1 },
    { 0xefa0a325, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf20a32a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca0a335, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b80a338, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa0a33b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1020a341, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e0a345, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xf9e0a347, SUC_lexicon_tags_17 },
    { 0xc400a348, SUC_lexicon_tags_38 },
    { 0x11e0a349, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6100a350, SUC_lexicon_tags_43 },
    { 0x19a0a350, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc680a353, SUC_lexicon_tags_15 },
    { 0xc680a353, SUC_lexicon_tags_37 },
    { 0xc960a355, SUC_lexicon_tags_23 },
    { 0x3b20a354, SUC_lexicon_tags_46 },
    { 0x3f20a354, SUC_lexicon_tags_37 },
    { 0x2b00a358, SUC_lexicon_tags_160 },
    { 0xcdc0a359, SUC_lexicon_tags_37 },
    { 0x9a40a35a, SUC_lexicon_tags_7 },
    { 0x9740a35b, SUC_lexicon_tags_7 },
    { 0xd6a0a35c, SUC_lexicon_tags_12 },
    { 0xb720a35b, SUC_lexicon_tags_27 },
    { 0x6780a35e, SUC_lexicon_tags_26 },
    { 0x9020a359, SUC_lexicon_tags_37 },
    { 0x8620a360, SUC_lexicon_tags_43 },
    { 0xb260a361, SUC_lexicon_tags_4 },
    { 0xa840a361, SUC_lexicon_tags_7 },
    { 0x3a20a363, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe960a365, SUC_lexicon_tags_2 },
    { 0x4000a365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fc0a368, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a0a372, SUC_lexicon_tags_8 },
    { 0x9760a373, SUC_lexicon_tags_1 },
    { 0x8200a372, SUC_lexicon_tags_58 },
    { 0x7ec0a374, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c0a379, SUC_lexicon_tags_33 },
    { 0xb1a0a37a, SUC_lexicon_tags_66 },
    { 0x18a0a37b, SUC_lexicon_tags_56 },
    { 0xc5c0a37a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1360a385, SUC_lexicon_tags_47 },
    { 0xc780a385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd060a38a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa360a38e, SUC_lexicon_tags_112 },
    { 0xc1e0a38e, SUC_lexicon_tags_25 },
    { 0x6fe0a38e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6400a393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49a0a395, SUC_lexicon_tags_11 },
    { 0xd5a0a395, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4940a39a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed00a39f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a0a3a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a0a3a6, SUC_lexicon_tags_23 },
    { 0x5460a3a7, SUC_lexicon_tags_63 },
    { 0xca60a3a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf40a3b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa80a3b6, SUC_lexicon_tags_65 },
    { 0x1580a3b7, SUC_lexicon_tags_11 },
    { 0x640a3b8, SUC_lexicon_tags_565 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c20a3c0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc7e0a3c2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be0a3c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b60a3c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6660a3cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7300a3ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca20a3d4, SUC_lexicon_tags_17 },
    { 0x92e0a3d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6460a3da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x37a0a3dc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4340a3df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5880a3e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da0a3e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c60a3ed, SUC_lexicon_tags_30 },
    { 0x8280a3ee, SUC_lexicon_tags_25 },
    { 0xbbe0a3ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea0a3f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c0a3f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a0a400, SUC_lexicon_tags_37 },
    { 0x8460a401, SUC_lexicon_tags_0 },
    { 0x3de0a400, SUC_lexicon_tags_25 },
    { 0x4740a403, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c40a405, SUC_lexicon_tags_33 },
    { 0x1560a406, SUC_lexicon_tags_38 },
    { 0x90e0a405, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3bc0a409, SUC_lexicon_tags_25 },
    { 0x1b60a40a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c0a417, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb40a419, SUC_lexicon_tags_12 },
    { 0x81c0a41a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3240a41c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15e0a41e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e0a423, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x760a425, SUC_lexicon_tags_2 },
    { 0x1c20a425, SUC_lexicon_tags_60 },
    { 0xca20a426, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4940a42a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc360a42f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a0a432, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4440a436, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ac0a438, SUC_lexicon_tags_8 },
    { 0x6360a439, SUC_lexicon_tags_28 },
    { 0x3ce0a439, SUC_lexicon_tags_1 },
    { 0xb3e0a43b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc20a43f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x90c0a441, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x26c0a443, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd680a446, SUC_lexicon_tags_27 },
    { 0xb7e0a447, SUC_lexicon_tags_15 },
    { 0x8b60a448, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c00a44a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2500a44d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe0a450, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de0a455, SUC_lexicon_tags_47 },
    { 0x4960a455, SUC_lexicon_tags_25 },
    { 0xf960a457, SUC_lexicon_tags_11 },
    { 0xcc00a456, SUC_lexicon_tags_23 },
    { 0x1460a459, SUC_lexicon_tags_33 },
    { 0xa160a45a, SUC_lexicon_tags_37 },
    { 0x3720a45a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a0a45f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b40a465, SUC_lexicon_tags_1 },
    { 0xeca0a466, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c60a468, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3b40a46a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x24a0a46c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa0a46f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e00a471, SUC_lexicon_tags_47 },
    { 0xbe0a472, SUC_lexicon_tags_1 },
    { 0xf7a0a473, SUC_lexicon_tags_11 },
    { 0x2d60a474, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e0a477, SUC_lexicon_tags_63 },
    { 0x7420a477, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6080a47b, SUC_lexicon_tags_47 },
    { 0x86c0a47c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9260a480, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e0a484, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca60a487, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4620a48a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4260a48d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e0a490, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6340a492, SUC_lexicon_tags_28 },
    { 0xd2a0a492, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf40a4a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c20a4a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e0a4aa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9620a4ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac20a4b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2520a4b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c0a4b7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c0a4ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0a4bd, SUC_lexicon_tags_723 },
    { 0xca40a4bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e0a4c2, SUC_lexicon_tags_37 },
    { 0x4520a4c2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2360a4c5, SUC_lexicon_tags_11 },
    { 0x4fc0a4c6, SUC_lexicon_tags_49 },
    { 0xd200a4c7, SUC_lexicon_tags_9 },
    { 0xa0c0a4c8, SUC_lexicon_tags_12 },
    { 0x3fc0a4c7, SUC_lexicon_tags_37 },
    { 0x6ee0a4ca, SUC_lexicon_tags_25 },
    { 0xa60a4c6, SUC_lexicon_tags_28 },
    { 0x3000a4c6, SUC_lexicon_tags_7 },
    { 0xdfa0a4cd, SUC_lexicon_tags_25 },
    { 0x3880a4ce, SUC_lexicon_tags_1 },
    { 0x3a00a4cf, SUC_lexicon_tags_28 },
    { 0x4180a4c8, SUC_lexicon_tags_1 },
    { 0xdaa0a4d1, SUC_lexicon_tags_11 },
    { 0xe900a4d2, SUC_lexicon_tags_17 },
    { 0x320a4c7, SUC_lexicon_tags_25 },
    { 0xa720a4cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1920a4d6, SUC_lexicon_tags_39 },
    { 0x240a4d7, SUC_lexicon_tags_10 },
    { 0x7280a4d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9980a4df, SUC_lexicon_tags_25 },
    { 0x5680a4e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb3a0a4e2, SUC_lexicon_tags_25 },
    { 0xe1a0a4e3, SUC_lexicon_tags_68 },
    { 0x77a0a4e2, SUC_lexicon_tags_43 },
    { 0x52a0a4e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae0a4ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2020a4f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6840a4f9, SUC_lexicon_tags_23 },
    { 0xd880a4fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c0a4fe, SUC_lexicon_tags_136 },
    { 0xbba0a4fe, SUC_lexicon_tags_69 },
    { 0x6d00a500, SUC_lexicon_tags_15 },
    { 0x62c0a4ff, SUC_lexicon_tags_15 },
    { 0x6a60a502, SUC_lexicon_tags_60 },
    { 0x58a0a502, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xda20a506, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x640a508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc480a50b, SUC_lexicon_tags_7 },
    { 0x5e00a50b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c0a510, SUC_lexicon_tags_82 },
    { 0x1180a511, SUC_lexicon_tags_15 },
    { 0xc4a0a512, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb880a514, SUC_lexicon_tags_2 },
    { 0x1900a514, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfc80a517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e0a51a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f80a51d, SUC_lexicon_tags_33 },
    { 0x31a0a51e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c0a521, SUC_lexicon_tags_17 },
    { 0x320a521, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2760a525, SUC_lexicon_tags_15 },
    { 0x4de0a526, SUC_lexicon_tags_11 },
    { 0x53a0a526, SUC_lexicon_tags_11 },
    { 0xcc80a528, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc0a52c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef20a52e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb800a532, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e0a534, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4c40a536, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3920a53a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd800a53d, SUC_lexicon_tags_14 },
    { 0xb840a53e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa660a545, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49a0a547, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c0a54a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xebc0a54c, SUC_lexicon_tags_14 },
    { 0xd380a54d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8000a550, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b40a552, SUC_lexicon_tags_79 },
    { 0x8f00a553, SUC_lexicon_tags_7 },
    { 0x2060a553, SUC_lexicon_tags_25 },
    { 0x2a00a554, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f00a55c, SUC_lexicon_tags_11 },
    { 0xf020a55c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d60a561, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa840a566, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd500a56b, SUC_lexicon_tags_25 },
    { 0xdca0a56c, SUC_lexicon_tags_33 },
    { 0x240a56b, SUC_lexicon_tags_32 },
    { 0xc080a56e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x660a570, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf020a573, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa220a579, SUC_lexicon_tags_47 },
    { 0x3c00a57a, SUC_lexicon_tags_26 },
    { 0x2880a57b, SUC_lexicon_tags_27 },
    { 0xd520a579, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f00a57e, SUC_lexicon_tags_37 },
    { 0x2d40a57e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa0a582, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6180a585, SUC_lexicon_tags_46 },
    { 0x2020a585, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde0a589, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b00a58f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb460a591, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5940a594, SUC_lexicon_tags_15 },
    { 0xb3a0a594, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0a599, SUC_lexicon_tags_2 },
    { 0xda80a599, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e0a59d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8360a5a4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee0a5a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf720a5ac, SUC_lexicon_tags_12 },
    { 0x6020a5ad, SUC_lexicon_tags_37 },
    { 0x4740a5ad, SUC_lexicon_tags_47 },
    { 0xfe20a5ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba0a5b2, SUC_lexicon_tags_0 },
    { 0xade0a5b3, SUC_lexicon_tags_30 },
    { 0x5f60a5b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6880a5b7, SUC_lexicon_tags_11 },
    { 0x4240a5b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b20a5ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa00a5bd, SUC_lexicon_tags_29 },
    { 0x2fa0a5be, SUC_lexicon_tags_2 },
    { 0xb2e0a5bf, SUC_lexicon_tags_3 },
    { 0x8980a5be, SUC_lexicon_tags_46 },
    { 0x6000a5bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e0a5cc, SUC_lexicon_tags_7 },
    { 0x8860a5cd, SUC_lexicon_tags_15 },
    { 0x620a5ce, SUC_lexicon_tags_47 },
    { 0xf920a5cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe300a5d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1220a5d7, SUC_lexicon_tags_37 },
    { 0x1f80a5d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5120a5dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f20a5e0, SUC_lexicon_tags_19 },
    { 0x1680a5e1, SUC_lexicon_tags_29 },
    { 0x44e0a5e2, SUC_lexicon_tags_63 },
    { 0x5220a5e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0a5e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb760a5e8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf7c0a5ea, SUC_lexicon_tags_15 },
    { 0xa660a5ea, SUC_lexicon_tags_17 },
    { 0x1400a5ec, SUC_lexicon_tags_63 },
    { 0xe800a5ed, SUC_lexicon_tags_42 },
    { 0x1a20a5ea, SUC_lexicon_tags_37 },
    { 0xb0c0a5ef, SUC_lexicon_tags_33 },
    { 0xbd40a5ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9840a5f3, SUC_lexicon_tags_7 },
    { 0x2260a5f4, SUC_lexicon_tags_1 },
    { 0x6fc0a5f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a0a5f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e0a5fc, SUC_lexicon_tags_12 },
    { 0xf80a5fd, SUC_lexicon_tags_11 },
    { 0xdc60a5fc, SUC_lexicon_tags_12 },
    { 0x7be0a5ff, SUC_lexicon_tags_6 },
    { 0x5700a600, SUC_lexicon_tags_1 },
    { 0x4620a600, SUC_lexicon_tags_32 },
    { 0x1560a602, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83a0a604, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc0a60b, SUC_lexicon_tags_1 },
    { 0x2e00a60c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77a0a60e, SUC_lexicon_tags_1 },
    { 0xb800a60f, SUC_lexicon_tags_12 },
    { 0xee20a60e, SUC_lexicon_tags_19 },
    { 0xda20a611, SUC_lexicon_tags_17 },
    { 0x5940a611, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfda0a614, SUC_lexicon_tags_37 },
    { 0xdc60a614, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1360a617, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea00a61b, SUC_lexicon_tags_37 },
    { 0xfa60a61c, SUC_lexicon_tags_2 },
    { 0x3400a61c, SUC_lexicon_tags_87 },
    { 0x3a40a61e, SUC_lexicon_tags_15 },
    { 0xb660a61c, SUC_lexicon_tags_253 },
    { 0x32e0a61b, SUC_lexicon_tags_37 },
    { 0xea40a61c, SUC_lexicon_tags_35 },
    { 0x3e0a61f, SUC_lexicon_tags_13 },
    { 0xc020a61f, SUC_lexicon_tags_16 },
    { 0xfac0a624, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe760a628, SUC_lexicon_tags_44 },
    { 0x95e0a629, SUC_lexicon_tags_56 },
    { 0x3f80a62a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6840a62e, SUC_lexicon_tags_17 },
    { 0x9ec0a62e, SUC_lexicon_tags_29 },
    { 0x6960a630, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf980a633, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa0a63c, SUC_lexicon_tags_17 },
    { 0x5e00a63d, SUC_lexicon_tags_2 },
    { 0x2d60a63e, SUC_lexicon_tags_46 },
    { 0x3e60a63f, SUC_lexicon_tags_27 },
    { 0x2320a640, SUC_lexicon_tags_430 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa980a644, SUC_lexicon_tags_26 },
    { 0xc720a645, SUC_lexicon_tags_140 },
    { 0x5c0a644, SUC_lexicon_tags_25 },
    { 0xf240a647, SUC_lexicon_tags_12 },
    { 0x66e0a648, SUC_lexicon_tags_37 },
    { 0xec00a645, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35e0a64b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc20a64e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5300a651, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f00a653, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e80a658, SUC_lexicon_tags_17 },
    { 0xd840a659, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c0a65b, SUC_lexicon_tags_47 },
    { 0x2420a65c, SUC_lexicon_tags_39 },
    { 0xbba0a65d, SUC_lexicon_tags_11 },
    { 0xa9a0a65d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa880a660, SUC_lexicon_tags_1 },
    { 0x5580a661, SUC_lexicon_tags_37 },
    { 0xf820a662, SUC_lexicon_tags_15 },
    { 0xc8e0a661, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7520a667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2220a66d, SUC_lexicon_tags_7 },
    { 0x7dc0a66e, SUC_lexicon_tags_27 },
    { 0xab60a66f, SUC_lexicon_tags_1 },
    { 0x3c60a66e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb60a67c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9d60a67e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1ec0a680, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc760a683, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xaae0a685, SUC_lexicon_tags_34 },
    { 0x760a686, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e0a690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef00a694, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x640a696, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4840a69d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7240a6a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc380a6a5, SUC_lexicon_tags_37 },
    { 0x4400a6a5, SUC_lexicon_tags_9 },
    { 0x7280a6a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5360a6af, SUC_lexicon_tags_14 },
    { 0x7e0a6b0, SUC_lexicon_tags_6 },
    { 0x26e0a6b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2c0a6b3, SUC_lexicon_tags_13 },
    { 0x1ee0a6b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x11a0a6b6, SUC_lexicon_tags_45 },
    { 0xe020a6b7, SUC_lexicon_tags_33 },
    { 0x7120a6b8, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x7560a6ba, SUC_lexicon_tags_33 },
    { 0x81c0a6bb, SUC_lexicon_tags_1 },
    { 0x8580a6bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de0a6c1, SUC_lexicon_tags_17 },
    { 0x9c80a6c2, SUC_lexicon_tags_30 },
    { 0xca40a6c1, SUC_lexicon_tags_25 },
    { 0xce40a6c2, SUC_lexicon_tags_57 },
    { 0x3660a6c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e0a6c9, SUC_lexicon_tags_60 },
    { 0x2f40a6c9, SUC_lexicon_tags_17 },
    { 0xc180a6cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf00a6ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b40a6d1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb0c0a6d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca60a6d8, SUC_lexicon_tags_33 },
    { 0x7600a6d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9380a6db, SUC_lexicon_tags_12 },
    { 0x8a20a6dc, SUC_lexicon_tags_9 },
    { 0x1fa0a6db, SUC_lexicon_tags_29 },
    { 0xab80a6de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3640a6e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb940a6e8, SUC_lexicon_tags_11 },
    { 0x5000a6e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1220a6eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa140a6f2, SUC_lexicon_tags_82 },
    { 0xa360a6f3, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf340a6f7, SUC_lexicon_tags_11 },
    { 0x2620a6f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f20a6fd, SUC_lexicon_tags_28 },
    { 0x3800a6fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc40a706, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd760a70b, SUC_lexicon_tags_9 },
    { 0x7f00a70c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7040a711, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c80a714, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e0a719, SUC_lexicon_tags_15 },
    { 0xf460a71a, SUC_lexicon_tags_82 },
    { 0xec00a71b, SUC_lexicon_tags_23 },
    { 0x5cc0a71c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a0a721, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1540a723, SUC_lexicon_tags_6 },
    { 0x5ce0a724, SUC_lexicon_tags_49 },
    { 0x4540a724, SUC_lexicon_tags_87 },
    { 0x7f40a725, SUC_lexicon_tags_60 },
    { 0x40e0a727, SUC_lexicon_tags_8 },
    { 0x48a0a727, SUC_lexicon_tags_29 },
    { 0xa340a727, SUC_lexicon_tags_19 },
    { 0x1300a72a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a0a72f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf20a733, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x35e0a735, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c0a73a, SUC_lexicon_tags_63 },
    { 0x9a60a73b, SUC_lexicon_tags_47 },
    { 0xe520a73c, SUC_lexicon_tags_23 },
    { 0x5c80a73d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00a740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0a744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66e0a746, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c0a74b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e20a751, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa280a753, SUC_lexicon_tags_23 },
    { 0x4860a754, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xa620a757, SUC_lexicon_tags_48 },
    { 0x4000a758, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7760a75b, SUC_lexicon_tags_2 },
    { 0x5960a75b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2800a75f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe600a765, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d20a767, SUC_lexicon_tags_2 },
    { 0x3e00a767, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac0a76b, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xcdc0a76d, SUC_lexicon_tags_6 },
    { 0xcb60a76e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc20a773, SUC_lexicon_tags_7 },
    { 0x6a80a774, SUC_lexicon_tags_12 },
    { 0x400a773, SUC_lexicon_tags_17 },
    { 0x8560a774, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6560a778, SUC_lexicon_tags_63 },
    { 0x10c0a779, SUC_lexicon_tags_27 },
    { 0xf120a77a, SUC_lexicon_tags_6 },
    { 0xba40a778, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c0a77f, SUC_lexicon_tags_30 },
    { 0xa560a780, SUC_lexicon_tags_25 },
    { 0xd420a781, SUC_lexicon_tags_0 },
    { 0x8c00a781, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4600a789, SUC_lexicon_tags_7 },
    { 0xa500a78a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b80a78d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe480a794, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7020a79b, SUC_lexicon_tags_14 },
    { 0xa320a79b, SUC_lexicon_tags_11 },
    { 0x3880a79d, SUC_lexicon_tags_37 },
    { 0x4e00a79e, SUC_lexicon_tags_1 },
    { 0xa2e0a79c, SUC_lexicon_tags_15 },
    { 0xb40a79d, SUC_lexicon_tags_2 },
    { 0x7580a79d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb20a7a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd520a7a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb20a7a7, SUC_lexicon_tags_65 },
    { 0x2240a7a8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da0a7ab, SUC_lexicon_tags_7 },
    { 0xbbe0a7ac, SUC_lexicon_tags_47 },
    { 0x8440a7ad, SUC_lexicon_tags_15 },
    { 0xfe60a7ac, SUC_lexicon_tags_39 },
    { 0x80e0a7af, SUC_lexicon_tags_7 },
    { 0x4bc0a7b0, SUC_lexicon_tags_84 },
    { 0xbc00a7af, SUC_lexicon_tags_1 },
    { 0x7ce0a7b2, SUC_lexicon_tags_8 },
    { 0xd020a7ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0a7b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc880a7b9, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xf320a7bb, SUC_lexicon_tags_4 },
    { 0x1b00a7bc, SUC_lexicon_tags_8 },
    { 0xe240a7bd, SUC_lexicon_tags_19 },
    { 0x6a80a7bc, SUC_lexicon_tags_25 },
    { 0x8840a7bf, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8520a7c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x780a7c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a0a7ca, SUC_lexicon_tags_12 },
    { 0xc340a7cb, SUC_lexicon_tags_28 },
    { 0xae60a7cc, SUC_lexicon_tags_15 },
    { 0x6e40a7ca, SUC_lexicon_tags_2 },
    { 0xc0a0a7ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3780a7d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c40a7d3, SUC_lexicon_tags_13 },
    { 0x5c40a7d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6960a7d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6f20a7d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7560a7dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ce0a7df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8840a7ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x560a7ec, SUC_lexicon_tags_7 },
    { 0xe480a7ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3100a7f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9380a7f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc680a7f7, SUC_lexicon_tags_11 },
    { 0xaf80a7f8, SUC_lexicon_tags_84 },
    { 0xa540a7f8, SUC_lexicon_tags_13 },
    { 0x9300a7fa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfc20a7fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7320a7fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb580a801, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3960a804, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf580a80b, SUC_lexicon_tags_697 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00a812, SUC_lexicon_tags_63 },
    { 0x9c20a813, SUC_lexicon_tags_4 },
    { 0x4d80a813, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1360a817, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa0a81f, SUC_lexicon_tags_0 },
    { 0x1640a820, SUC_lexicon_tags_12 },
    { 0xcb20a820, SUC_lexicon_tags_25 },
    { 0xe000a820, SUC_lexicon_tags_66 },
    { 0x9640a820, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x70c0a825, SUC_lexicon_tags_38 },
    { 0x960a825, SUC_lexicon_tags_49 },
    { 0x6180a825, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c80a831, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6720a835, SUC_lexicon_tags_37 },
    { 0x51a0a836, SUC_lexicon_tags_2 },
    { 0xd4e0a837, SUC_lexicon_tags_55 },
    { 0x4a80a838, SUC_lexicon_tags_10 },
    { 0x3c0a837, SUC_lexicon_tags_41 },
    { 0x6300a83a, SUC_lexicon_tags_46 },
    { 0x71a0a83a, SUC_lexicon_tags_2 },
    { 0xd2e0a835, SUC_lexicon_tags_38 },
    { 0xd000a83b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ee0a83f, SUC_lexicon_tags_6 },
    { 0x71e0a840, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300a845, SUC_lexicon_tags_30 },
    { 0xc5a0a845, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4720a84a, SUC_lexicon_tags_7 },
    { 0x2ae0a84a, SUC_lexicon_tags_37 },
    { 0x83a0a84c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1160a84e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bc0a850, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9860a852, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x860a854, SUC_lexicon_tags_63 },
    { 0xe0c0a854, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7800a85a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e40a85c, SUC_lexicon_tags_55 },
    { 0x7b80a85c, SUC_lexicon_tags_27 },
    { 0x8120a85d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c0a863, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc0a866, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc0a869, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf140a86c, SUC_lexicon_tags_17 },
    { 0x2960a86d, SUC_lexicon_tags_177 },
    { 0x64c0a86c, SUC_lexicon_tags_33 },
    { 0x7c00a86f, SUC_lexicon_tags_38 },
    { 0xa480a86c, SUC_lexicon_tags_17 },
    { 0xfc40a870, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf420a873, SUC_lexicon_tags_79 },
    { 0xbba0a873, SUC_lexicon_tags_12 },
    { 0x5ae0a875, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a0a881, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be0a887, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9460a88a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6280a891, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xada0a893, SUC_lexicon_tags_27 },
    { 0xa260a894, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0a898, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e20a89e, SUC_lexicon_tags_17 },
    { 0x4ea0a89f, SUC_lexicon_tags_17 },
    { 0x2920a89f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad40a8a2, SUC_lexicon_tags_6 },
    { 0x7e80a8a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c0a8a5, SUC_lexicon_tags_19 },
    { 0x23a0a8a5, SUC_lexicon_tags_63 },
    { 0xd820a8a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada0a8ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8940a8ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c00a8af, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xd920a8b1, SUC_lexicon_tags_15 },
    { 0x3940a8b2, SUC_lexicon_tags_28 },
    { 0x9080a8b3, SUC_lexicon_tags_79 },
    { 0x9660a8b3, SUC_lexicon_tags_15 },
    { 0x900a8b5, SUC_lexicon_tags_87 },
    { 0x760a8b6, SUC_lexicon_tags_14 },
    { 0xfc20a8b7, SUC_lexicon_tags_17 },
    { 0xaa00a8b8, SUC_lexicon_tags_34 },
    { 0xe860a8b9, SUC_lexicon_tags_7 },
    { 0x8be0a8b6, SUC_lexicon_tags_38 },
    { 0x8640a8bb, SUC_lexicon_tags_37 },
    { 0x6560a8bb, SUC_lexicon_tags_47 },
    { 0x33c0a8bd, SUC_lexicon_tags_0 },
    { 0x4980a8bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd60a8c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb400a8c6, SUC_lexicon_tags_12 },
    { 0x4b00a8c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc0a8cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c60a8d1, SUC_lexicon_tags_48 },
    { 0x7c20a8d2, SUC_lexicon_tags_25 },
    { 0xe4c0a8d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7500a8d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac0a8dc, SUC_lexicon_tags_19 },
    { 0xa600a8dd, SUC_lexicon_tags_4 },
    { 0x57e0a8de, SUC_lexicon_tags_38 },
    { 0x3680a8df, SUC_lexicon_tags_17 },
    { 0xe4e0a8e0, SUC_lexicon_tags_12 },
    { 0x9b80a8e1, SUC_lexicon_tags_15 },
    { 0x7080a8dd, SUC_lexicon_tags_2 },
    { 0x9fa0a8e2, SUC_lexicon_tags_33 },
    { 0xabc0a8e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc620a8e6, SUC_lexicon_tags_62 },
    { 0xce00a8e7, SUC_lexicon_tags_31 },
    { 0x5640a8e7, SUC_lexicon_tags_7 },
    { 0xfbc0a8e6, SUC_lexicon_tags_17 },
    { 0x3aa0a8ea, SUC_lexicon_tags_49 },
    { 0x3b00a8e8, SUC_lexicon_tags_37 },
    { 0xac60a8ec, SUC_lexicon_tags_37 },
    { 0xbe0a8ec, SUC_lexicon_tags_44 },
    { 0xd400a8ea, SUC_lexicon_tags_41 },
    { 0x58e0a8ea, SUC_lexicon_tags_33 },
    { 0xc040a8e8, SUC_lexicon_tags_60 },
    { 0xc700a8ea, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcbe0a8f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8460a8f8, SUC_lexicon_tags_56 },
    { 0xc880a8f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a0a8fe, SUC_lexicon_tags_2 },
    { 0x9240a8ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef40a901, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd800a908, SUC_lexicon_tags_1 },
    { 0x6520a909, SUC_lexicon_tags_17 },
    { 0x87e0a90a, SUC_lexicon_tags_11 },
    { 0x23a0a90b, SUC_lexicon_tags_15 },
    { 0x7d60a90c, SUC_lexicon_tags_37 },
    { 0x73a0a90c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfa0a90f, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa400a913, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5320a915, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa360a91f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8760a924, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf860a929, SUC_lexicon_tags_15 },
    { 0x10e0a92a, SUC_lexicon_tags_133 },
    { 0xe6a0a929, SUC_lexicon_tags_14 },
    { 0xb420a92c, SUC_lexicon_tags_8 },
    { 0x8b00a92b, SUC_lexicon_tags_31 },
    { 0xab40a92e, SUC_lexicon_tags_153 },
    { 0x5560a92e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e0a932, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5de0a934, SUC_lexicon_tags_49 },
    { 0x7620a935, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe40a938, SUC_lexicon_tags_29 },
    { 0x6a80a938, SUC_lexicon_tags_6 },
    { 0xb080a93a, SUC_lexicon_tags_0 },
    { 0xca80a93a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa00a93f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc200a942, SUC_lexicon_tags_2 },
    { 0x6b80a943, SUC_lexicon_tags_23 },
    { 0xb9a0a944, SUC_lexicon_tags_15 },
    { 0xb820a945, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9740a947, SUC_lexicon_tags_15 },
    { 0x5d00a948, SUC_lexicon_tags_15 },
    { 0xd00a948, SUC_lexicon_tags_17 },
    { 0xebe0a94a, SUC_lexicon_tags_45 },
    { 0xa140a94a, SUC_lexicon_tags_25 },
    { 0x53c0a94c, SUC_lexicon_tags_11 },
    { 0xf3a0a94c, SUC_lexicon_tags_37 },
    { 0x8c0a948, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa820a950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84c0a952, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a0a958, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1840a95f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74a0a961, SUC_lexicon_tags_17 },
    { 0x2ba0a962, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c0a967, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4600a96e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2d60a970, SUC_lexicon_tags_60 },
    { 0x9a00a971, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5760a973, SUC_lexicon_tags_17 },
    { 0x82e0a973, SUC_lexicon_tags_49 },
    { 0xb2c0a975, SUC_lexicon_tags_9 },
    { 0x7020a976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0a97b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8960a97f, SUC_lexicon_tags_1 },
    { 0xfd40a980, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe940a984, SUC_lexicon_tags_61 },
    { 0xfca0a984, SUC_lexicon_tags_26 },
    { 0x28a0a986, SUC_lexicon_tags_17 },
    { 0x2fe0a985, SUC_lexicon_tags_8 },
    { 0xf8e0a988, SUC_lexicon_tags_47 },
    { 0x1340a989, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a60a98b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60a998, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920a9a5, SUC_lexicon_tags_16 },
    { 0x57a0a9a6, SUC_lexicon_tags_63 },
    { 0x2c40a9a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa220a9ac, SUC_lexicon_tags_33 },
    { 0x4d60a9ac, SUC_lexicon_tags_33 },
    { 0x880a9ae, SUC_lexicon_tags_17 },
    { 0x1cc0a9af, SUC_lexicon_tags_14 },
    { 0x1c80a9af, SUC_lexicon_tags_25 },
    { 0xd9c0a9af, SUC_lexicon_tags_12 },
    { 0x9ea0a9b1, SUC_lexicon_tags_6 },
    { 0xc440a9b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa140a9b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec0a9bb, SUC_lexicon_tags_15 },
    { 0x3e80a9bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae0a9c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3280a9c6, SUC_lexicon_tags_49 },
    { 0xa920a9c7, SUC_lexicon_tags_43 },
    { 0xc6e0a9c6, SUC_lexicon_tags_1446 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a60a9cb, SUC_lexicon_tags_48 },
    { 0xeae0a9cb, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf480a9d1, SUC_lexicon_tags_27 },
    { 0xb80a9d2, SUC_lexicon_tags_28 },
    { 0xe040a9d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f20a9d7, SUC_lexicon_tags_108 },
    { 0xb580a9d7, SUC_lexicon_tags_108 },
    { 0xe8e0a9d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc920a9dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d60a9e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec0a9e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9300a9e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d40a9e9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd580a9ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2340a9f0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8d60a9f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e40a9f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac0a9f7, SUC_lexicon_tags_7 },
    { 0x5060a9f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc600a9fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa0a9fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf280aa01, SUC_lexicon_tags_55 },
    { 0x2ce0aa02, SUC_lexicon_tags_60 },
    { 0x5e20aa02, SUC_lexicon_tags_17 },
    { 0x18c0aa04, SUC_lexicon_tags_2 },
    { 0xa8a0aa03, SUC_lexicon_tags_2 },
    { 0x3e40aa04, SUC_lexicon_tags_15 },
    { 0x2580aa07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd900aa09, SUC_lexicon_tags_61 },
    { 0xb680aa0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5640aa10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa0aa13, SUC_lexicon_tags_10 },
    { 0x9580aa14, SUC_lexicon_tags_1 },
    { 0x6ba0aa15, SUC_lexicon_tags_29 },
    { 0x5740aa14, SUC_lexicon_tags_19 },
    { 0x26e0aa17, SUC_lexicon_tags_12 },
    { 0x5f80aa14, SUC_lexicon_tags_49 },
    { 0xf740aa19, SUC_lexicon_tags_66 },
    { 0x240aa1a, SUC_lexicon_tags_107 },
    { 0x9040aa19, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc540aa1e, SUC_lexicon_tags_23 },
    { 0xe760aa1f, SUC_lexicon_tags_37 },
    { 0xd020aa20, SUC_lexicon_tags_95 },
    { 0x9b20aa21, SUC_lexicon_tags_11 },
    { 0x3b60aa22, SUC_lexicon_tags_43 },
    { 0x90a0aa21, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xdd00aa25, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x30c0aa27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc0aa34, SUC_lexicon_tags_7 },
    { 0x8f60aa35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae0aa3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6520aa3e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x8300aa40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd620aa43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1060aa4c, SUC_lexicon_tags_47 },
    { 0x7bc0aa4c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c60aa5c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d00aa60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd00aa62, SUC_lexicon_tags_33 },
    { 0xa1a0aa63, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x9580aa66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb40aa6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4420aa6e, SUC_lexicon_tags_37 },
    { 0x14a0aa6f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x8780aa71, SUC_lexicon_tags_55 },
    { 0x2a20aa72, SUC_lexicon_tags_31 },
    { 0x1600aa72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf820aa77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8600aa7c, SUC_lexicon_tags_12 },
    { 0xd920aa7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a0aa80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95c0aa82, SUC_lexicon_tags_43 },
    { 0x3aa0aa83, SUC_lexicon_tags_6 },
    { 0x14a0aa83, SUC_lexicon_tags_31 },
    { 0x3a00aa83, SUC_lexicon_tags_49 },
    { 0x6960aa84, SUC_lexicon_tags_47 },
    { 0xfd00aa87, SUC_lexicon_tags_17 },
    { 0xf2c0aa88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b40aa8b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d00aa90, SUC_lexicon_tags_37 },
    { 0x8140aa90, SUC_lexicon_tags_7 },
    { 0x6f20aa91, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xbf80aa94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc6c0aa96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5940aa99, SUC_lexicon_tags_11 },
    { 0xb40aa9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c0aaa1, SUC_lexicon_tags_23 },
    { 0xf720aaa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c0aaa8, SUC_lexicon_tags_49 },
    { 0xb140aaa9, SUC_lexicon_tags_45 },
    { 0x1fe0aaaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8560aaad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada0aab3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdc20aab5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa60aabc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e0aabe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee0aac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3340aac4, SUC_lexicon_tags_16 },
    { 0x4740aac5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd580aac7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfe40aac9, SUC_lexicon_tags_14 },
    { 0x1300aaca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b60aace, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa0aad6, SUC_lexicon_tags_1 },
    { 0xa620aad6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7500aada, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfaa0aadc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a0aae1, SUC_lexicon_tags_2 },
    { 0x6b00aae2, SUC_lexicon_tags_15 },
    { 0xd2e0aae3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc460aae5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c0aaed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee40aaf0, SUC_lexicon_tags_1 },
    { 0xaa20aaf1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf1e0aaf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a0aaf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb580aaf8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x100aafb, SUC_lexicon_tags_34 },
    { 0x3f00aafc, SUC_lexicon_tags_114 },
    { 0x2720aafd, SUC_lexicon_tags_45 },
    { 0x29c0aafc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc000ab00, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e0ab04, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa6a0ab06, SUC_lexicon_tags_17 },
    { 0xe8a0ab07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7780ab0a, SUC_lexicon_tags_37 },
    { 0xf6e0ab0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a0ab0e, SUC_lexicon_tags_7 },
    { 0xa240ab0f, SUC_lexicon_tags_49 },
    { 0x6f80ab0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3340ab12, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5aa0ab14, SUC_lexicon_tags_11 },
    { 0x9040ab14, SUC_lexicon_tags_2 },
    { 0xab40ab16, SUC_lexicon_tags_69 },
    { 0xe640ab14, SUC_lexicon_tags_31 },
    { 0xb6e0ab18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3460ab1e, SUC_lexicon_tags_23 },
    { 0x1be0ab1f, SUC_lexicon_tags_50 },
    { 0x11c0ab1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c0ab23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf680ab2a, SUC_lexicon_tags_25 },
    { 0x70e0ab2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa680ab2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e0ab33, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a0ab37, SUC_lexicon_tags_30 },
    { 0xf9e0ab37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf60ab3b, SUC_lexicon_tags_21 },
    { 0x7200ab3c, SUC_lexicon_tags_37 },
    { 0xbb00ab3d, SUC_lexicon_tags_0 },
    { 0x9580ab3e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe440ab42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60ab47, SUC_lexicon_tags_2 },
    { 0x7a20ab47, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6160ab4a, SUC_lexicon_tags_25 },
    { 0xab40ab4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a80ab51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb00ab57, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3280ab59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6420ab5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc20ab60, SUC_lexicon_tags_17 },
    { 0xf580ab61, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xbc40ab63, SUC_lexicon_tags_45 },
    { 0xe7e0ab64, SUC_lexicon_tags_38 },
    { 0xc320ab65, SUC_lexicon_tags_68 },
    { 0xe80ab63, SUC_lexicon_tags_17 },
    { 0xfd60ab67, SUC_lexicon_tags_87 },
    { 0x8ac0ab67, SUC_lexicon_tags_29 },
    { 0x62c0ab69, SUC_lexicon_tags_27 },
    { 0x8220ab64, SUC_lexicon_tags_25 },
    { 0x6c00ab65, SUC_lexicon_tags_37 },
    { 0x2100ab6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc0ab71, SUC_lexicon_tags_23 },
    { 0x9b80ab72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe200ab74, SUC_lexicon_tags_17 },
    { 0xb7e0ab75, SUC_lexicon_tags_10 },
    { 0x52c0ab76, SUC_lexicon_tags_25 },
    { 0x9c60ab75, SUC_lexicon_tags_1 },
    { 0xde40ab78, SUC_lexicon_tags_47 },
    { 0x41e0ab79, SUC_lexicon_tags_37 },
    { 0xe260ab7a, SUC_lexicon_tags_60 },
    { 0x1340ab78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf960ab7f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa280ab81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabc0ab83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf320ab85, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb00ab8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1400ab90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7940ab94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3140ab97, SUC_lexicon_tags_0 },
    { 0x5220ab98, SUC_lexicon_tags_48 },
    { 0xbe40ab98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee60ab9b, SUC_lexicon_tags_47 },
    { 0x6000ab9c, SUC_lexicon_tags_113 },
    { 0xf400ab9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a80aba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6220aba4, SUC_lexicon_tags_3 },
    { 0x6a20aba5, SUC_lexicon_tags_68 },
    { 0x9040aba6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc100aba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6360abaa, SUC_lexicon_tags_32 },
    { 0x3b80abab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c0abb1, SUC_lexicon_tags_33 },
    { 0xdb60abb2, SUC_lexicon_tags_25 },
    { 0xb620abb3, SUC_lexicon_tags_1 },
    { 0x1a00abb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a40abc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9580abc4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6080abc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7740abcb, SUC_lexicon_tags_16 },
    { 0xf820abcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x54c0abce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f80abd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc60abd4, SUC_lexicon_tags_37 },
    { 0xff00abd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe440abd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfa0abdb, SUC_lexicon_tags_10 },
    { 0xb700abdb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb100abdf, SUC_lexicon_tags_21 },
    { 0x9040abdf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2720abe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa280abe5, SUC_lexicon_tags_15 },
    { 0x5420abe6, SUC_lexicon_tags_4 },
    { 0x7640abe6, SUC_lexicon_tags_52 },
    { 0x1020abe8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a0abeb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfce0abed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a0abf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60e0abf4, SUC_lexicon_tags_15 },
    { 0x2b00abf4, SUC_lexicon_tags_68 },
    { 0x9c0abf4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7180abfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c0ac03, SUC_lexicon_tags_85 },
    { 0x1a80ac04, SUC_lexicon_tags_31 },
    { 0x99c0ac05, SUC_lexicon_tags_55 },
    { 0x7400ac03, SUC_lexicon_tags_17 },
    { 0x61a0ac07, SUC_lexicon_tags_17 },
    { 0x90a0ac08, SUC_lexicon_tags_10 },
    { 0xe740ac09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc920ac12, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5180ac15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb80ac19, SUC_lexicon_tags_1 },
    { 0x8180ac19, SUC_lexicon_tags_8 },
    { 0xb40ac19, SUC_lexicon_tags_17 },
    { 0x640ac1c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3660ac1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc00ac24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8580ac26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c0ac28, SUC_lexicon_tags_49 },
    { 0x1a0ac29, SUC_lexicon_tags_17 },
    { 0x4da0ac2a, SUC_lexicon_tags_32 },
    { 0x65a0ac2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa7a0ac2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e40ac30, SUC_lexicon_tags_49 },
    { 0x2980ac31, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xe160ac34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8be0ac36, SUC_lexicon_tags_23 },
    { 0x3140ac36, SUC_lexicon_tags_26 },
    { 0xf6a0ac36, SUC_lexicon_tags_60 },
    { 0xc0c0ac39, SUC_lexicon_tags_63 },
    { 0x96c0ac39, SUC_lexicon_tags_27 },
    { 0xa040ac3b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb7e0ac3d, SUC_lexicon_tags_15 },
    { 0xee20ac3e, SUC_lexicon_tags_14 },
    { 0xe340ac3f, SUC_lexicon_tags_12 },
    { 0xb0a0ac40, SUC_lexicon_tags_15 },
    { 0x3ec0ac40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40ac48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf380ac4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c0ac51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc0ac55, SUC_lexicon_tags_11 },
    { 0xd260ac55, SUC_lexicon_tags_63 },
    { 0x1c80ac57, SUC_lexicon_tags_38 },
    { 0x9da0ac58, SUC_lexicon_tags_31 },
    { 0x3f80ac58, SUC_lexicon_tags_7 },
    { 0xe920ac57, SUC_lexicon_tags_8 },
    { 0x6480ac5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f20ac61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a0ac68, SUC_lexicon_tags_60 },
    { 0x8c40ac69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc0ac71, SUC_lexicon_tags_11 },
    { 0xf1a0ac72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe280ac77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xde60ac7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff20ac7c, SUC_lexicon_tags_11 },
    { 0xb00ac7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb420ac7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb60ac81, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4740ac88, SUC_lexicon_tags_7 },
    { 0x45e0ac88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x74c0ac8b, SUC_lexicon_tags_30 },
    { 0x7b80ac8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4b40ac8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd580ac90, SUC_lexicon_tags_37 },
    { 0x5ee0ac91, SUC_lexicon_tags_25 },
    { 0x9a40ac92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa0ac95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e0ac98, SUC_lexicon_tags_133 },
    { 0x87a0ac99, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0aca1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a0aca7, SUC_lexicon_tags_37 },
    { 0x47e0aca8, SUC_lexicon_tags_33 },
    { 0x6240aca9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb740acad, SUC_lexicon_tags_8 },
    { 0xa400acae, SUC_lexicon_tags_28 },
    { 0xf40acae, SUC_lexicon_tags_53 },
    { 0x95c0acb0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc0acb5, SUC_lexicon_tags_1 },
    { 0x5640acb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec0acba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf720acbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd420acc0, SUC_lexicon_tags_14 },
    { 0xc2a0acc0, SUC_lexicon_tags_26 },
    { 0x3ce0acc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f40acc5, SUC_lexicon_tags_14 },
    { 0x5ea0acc6, SUC_lexicon_tags_49 },
    { 0x1da0acc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xff80acc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9760accb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d40acd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a20acd3, SUC_lexicon_tags_11 },
    { 0xe320acd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3880acd7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb7a0acd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf640acdb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1800acdd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e0ace0, SUC_lexicon_tags_38 },
    { 0xb080ace1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf80ace5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf120ace9, SUC_lexicon_tags_49 },
    { 0xd5c0ace9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9660aced, SUC_lexicon_tags_11 },
    { 0x4e80acee, SUC_lexicon_tags_1 },
    { 0xb2c0acef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb120acf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a0acf6, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c0acfb, SUC_lexicon_tags_37 },
    { 0xd2e0acfb, SUC_lexicon_tags_49 },
    { 0x7200acfc, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b00ad06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5900ad0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4560ad0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8340ad0f, SUC_lexicon_tags_17 },
    { 0xfdc0ad0f, SUC_lexicon_tags_23 },
    { 0xbc00ad11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3a0ad13, SUC_lexicon_tags_37 },
    { 0x1b00ad14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9920ad16, SUC_lexicon_tags_47 },
    { 0xe620ad17, SUC_lexicon_tags_26 },
    { 0x6be0ad18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8e0ad1a, SUC_lexicon_tags_9 },
    { 0x6fa0ad1b, SUC_lexicon_tags_19 },
    { 0xbc80ad1c, SUC_lexicon_tags_43 },
    { 0x48e0ad1c, SUC_lexicon_tags_11 },
    { 0xb5a0ad1e, SUC_lexicon_tags_12 },
    { 0x8e80ad1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a0ad22, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x300ad24, SUC_lexicon_tags_30 },
    { 0xe680ad24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99c0ad27, SUC_lexicon_tags_3 },
    { 0x2000ad28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e0ad2e, SUC_lexicon_tags_17 },
    { 0x3ca0ad2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e0ad32, SUC_lexicon_tags_37 },
    { 0x6a80ad33, SUC_lexicon_tags_33 },
    { 0xd320ad34, SUC_lexicon_tags_37 },
    { 0xca0ad35, SUC_lexicon_tags_1 },
    { 0x2340ad33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4460ad39, SUC_lexicon_tags_17 },
    { 0xee00ad39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6e0ad3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8200ad43, SUC_lexicon_tags_60 },
    { 0x2380ad44, SUC_lexicon_tags_11 },
    { 0x8260ad44, SUC_lexicon_tags_11 },
    { 0xf440ad46, SUC_lexicon_tags_60 },
    { 0x1a80ad43, SUC_lexicon_tags_17 },
    { 0x6820ad48, SUC_lexicon_tags_15 },
    { 0x4b80ad48, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea0ad51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e0ad55, SUC_lexicon_tags_3 },
    { 0x8620ad56, SUC_lexicon_tags_29 },
    { 0x3640ad55, SUC_lexicon_tags_15 },
    { 0xf20ad56, SUC_lexicon_tags_16 },
    { 0xed20ad59, SUC_lexicon_tags_2 },
    { 0x3de0ad5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x900ad5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7760ad63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe040ad6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4600ad6c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe380ad70, SUC_lexicon_tags_26 },
    { 0x8e80ad71, SUC_lexicon_tags_38 },
    { 0xb880ad72, SUC_lexicon_tags_14 },
    { 0xf40ad73, SUC_lexicon_tags_30 },
    { 0x1300ad74, SUC_lexicon_tags_38 },
    { 0x6000ad75, SUC_lexicon_tags_37 },
    { 0xd620ad72, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3460ad7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45c0ad80, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9220ad83, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c0ad86, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9080ad88, SUC_lexicon_tags_37 },
    { 0x1e80ad89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7960ad8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7120ad91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9a80ad93, SUC_lexicon_tags_17 },
    { 0xb640ad94, SUC_lexicon_tags_34 },
    { 0xe9c0ad95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c40ad98, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1120ad9e, SUC_lexicon_tags_15 },
    { 0x4140ad9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe620ada3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1820adae, SUC_lexicon_tags_25 },
    { 0x6d20adaf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63a0adb1, SUC_lexicon_tags_47 },
    { 0xcd00adb2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0adb9, SUC_lexicon_tags_19 },
    { 0xd7a0adba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7260adbc, SUC_lexicon_tags_17 },
    { 0x5e0adbd, SUC_lexicon_tags_0 },
    { 0x5440adbe, SUC_lexicon_tags_19 },
    { 0xe180adbd, SUC_lexicon_tags_7 },
    { 0xbf00adbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a0adc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e80adc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2080adc9, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x8d00adcb, SUC_lexicon_tags_17 },
    { 0xd7c0adcc, SUC_lexicon_tags_0 },
    { 0xdc60adcc, SUC_lexicon_tags_19 },
    { 0xea00adce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c20add1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f80add5, SUC_lexicon_tags_47 },
    { 0xe200add5, SUC_lexicon_tags_1 },
    { 0x3080add5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20adda, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8200addd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x500addf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27c0ade1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e0ade4, SUC_lexicon_tags_66 },
    { 0xc5a0ade5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd720ade8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x980adea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0adef, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x3500adf2, SUC_lexicon_tags_25 },
    { 0xffa0adf3, SUC_lexicon_tags_7 },
    { 0xb9e0adf4, SUC_lexicon_tags_12 },
    { 0xea40adf5, SUC_lexicon_tags_12 },
    { 0xa1e0adf3, SUC_lexicon_tags_15 },
    { 0x3940adf5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8220ae09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac0ae0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa00ae10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53e0ae12, SUC_lexicon_tags_38 },
    { 0x8120ae13, SUC_lexicon_tags_1 },
    { 0xa640ae14, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xb740ae17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c0ae1d, SUC_lexicon_tags_1 },
    { 0x9780ae1d, SUC_lexicon_tags_17 },
    { 0xc100ae1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f40ae22, SUC_lexicon_tags_1385 },
    { 0, NULL },
    { 0x8340ae24, SUC_lexicon_tags_35 },
    { 0xd1a0ae25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6260ae2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0ae36, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xec00ae38, SUC_lexicon_tags_62 },
    { 0xb6c0ae38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5140ae3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3780ae46, SUC_lexicon_tags_57 },
    { 0x1420ae47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a60ae51, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2380ae58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9360ae5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1460ae5f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x66e0ae61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c0ae68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc580ae6b, SUC_lexicon_tags_27 },
    { 0x38c0ae6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1780ae70, SUC_lexicon_tags_11 },
    { 0x8a80ae71, SUC_lexicon_tags_7 },
    { 0xe080ae70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf880ae75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe940ae77, SUC_lexicon_tags_25 },
    { 0x9da0ae78, SUC_lexicon_tags_17 },
    { 0x59a0ae77, SUC_lexicon_tags_7 },
    { 0x4b80ae78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0ae7d, SUC_lexicon_tags_68 },
    { 0x9ec0ae7d, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xd400ae80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe760ae82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d40ae85, SUC_lexicon_tags_7 },
    { 0xfb80ae86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec20ae89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da0ae8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc0ae93, SUC_lexicon_tags_27 },
    { 0xdda0ae93, SUC_lexicon_tags_7 },
    { 0x3840ae95, SUC_lexicon_tags_6 },
    { 0x5720ae96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2420ae9a, SUC_lexicon_tags_17 },
    { 0x17e0ae9b, SUC_lexicon_tags_1 },
    { 0x42e0ae9c, SUC_lexicon_tags_57 },
    { 0xa300ae9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c0aea0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0aea3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb000aead, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb20aeb0, SUC_lexicon_tags_38 },
    { 0x14c0aeb1, SUC_lexicon_tags_57 },
    { 0x7440aeb1, SUC_lexicon_tags_30 },
    { 0x2c0aeb3, SUC_lexicon_tags_9 },
    { 0xe8a0aeb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2480aebc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf920aec1, SUC_lexicon_tags_16 },
    { 0xbea0aec2, SUC_lexicon_tags_47 },
    { 0x2280aec3, SUC_lexicon_tags_63 },
    { 0xe000aec2, SUC_lexicon_tags_60 },
    { 0x1c80aec4, SUC_lexicon_tags_60 },
    { 0xe620aec2, SUC_lexicon_tags_23 },
    { 0xe240aec7, SUC_lexicon_tags_17 },
    { 0xaaa0aec8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5120aecb, SUC_lexicon_tags_63 },
    { 0x6a20aecb, SUC_lexicon_tags_10 },
    { 0xe7a0aecc, SUC_lexicon_tags_49 },
    { 0x8900aecb, SUC_lexicon_tags_37 },
    { 0xf980aecf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc9c0aed1, SUC_lexicon_tags_32 },
    { 0x8fa0aed1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd620aed5, SUC_lexicon_tags_12 },
    { 0x9d00aed5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba0aed9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf880aedc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59c0aede, SUC_lexicon_tags_47 },
    { 0x3600aede, SUC_lexicon_tags_23 },
    { 0x3d80aedf, SUC_lexicon_tags_27 },
    { 0x24c0aee0, SUC_lexicon_tags_43 },
    { 0x6fe0aee2, SUC_lexicon_tags_8 },
    { 0x87a0aee3, SUC_lexicon_tags_17 },
    { 0xdcc0aee4, SUC_lexicon_tags_204 },
    { 0x9920aee5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c80aee8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60aeef, SUC_lexicon_tags_37 },
    { 0xa9c0aef0, SUC_lexicon_tags_12 },
    { 0xdd00aeef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0aef4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc880aef9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7260aefb, SUC_lexicon_tags_11 },
    { 0x5620aefb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11e0aefe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7500af00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1440af05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x53e0af07, SUC_lexicon_tags_14 },
    { 0xd920af08, SUC_lexicon_tags_66 },
    { 0xbb40af09, SUC_lexicon_tags_37 },
    { 0xa220af0a, SUC_lexicon_tags_45 },
    { 0xbfc0af0a, SUC_lexicon_tags_11 },
    { 0xde20af0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc0af10, SUC_lexicon_tags_33 },
    { 0xa200af10, SUC_lexicon_tags_1 },
    { 0xce40af11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a0af16, SUC_lexicon_tags_17 },
    { 0x32e0af17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6900af1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c0af20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6240af26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb480af29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c80af30, SUC_lexicon_tags_25 },
    { 0xc320af31, SUC_lexicon_tags_2 },
    { 0xd4a0af31, SUC_lexicon_tags_19 },
    { 0x9360af33, SUC_lexicon_tags_1 },
    { 0xa3e0af34, SUC_lexicon_tags_512 },
    { 0xf060af32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb920af3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe0af3e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd3a0af40, SUC_lexicon_tags_3 },
    { 0x7ac0af41, SUC_lexicon_tags_27 },
    { 0xa620af40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e0af45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa760af47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9680af4a, SUC_lexicon_tags_33 },
    { 0x4880af4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae0af4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c0af51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc640af55, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e20af58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f80af5b, SUC_lexicon_tags_29 },
    { 0xc2e0af5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f20af5e, SUC_lexicon_tags_1 },
    { 0x6d00af5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c60af62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6a0af64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6340af67, SUC_lexicon_tags_2 },
    { 0x8520af68, SUC_lexicon_tags_26 },
    { 0x6820af69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e0af6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f00af70, SUC_lexicon_tags_46 },
    { 0xa380af71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a0af78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4540af7a, SUC_lexicon_tags_25 },
    { 0xcb40af7a, SUC_lexicon_tags_30 },
    { 0xb300af7c, SUC_lexicon_tags_37 },
    { 0x3c80af7c, SUC_lexicon_tags_30 },
    { 0xd5e0af7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc60af86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be0af8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8420af8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a0af92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd420af95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b20af97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x560af99, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa980afad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebc0afaf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9d40afb1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8480afb3, SUC_lexicon_tags_35 },
    { 0x5940afb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf460afb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xac60afb9, SUC_lexicon_tags_17 },
    { 0xfb40afb9, SUC_lexicon_tags_66 },
    { 0x1060afb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa120afbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e20afc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d40afcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf80afd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd280afd5, SUC_lexicon_tags_8 },
    { 0xbbc0afd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60c0afd8, SUC_lexicon_tags_11 },
    { 0x6520afd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f80afdc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x74c0afde, SUC_lexicon_tags_14 },
    { 0xf0c0afdf, SUC_lexicon_tags_29 },
    { 0x91c0afe0, SUC_lexicon_tags_2 },
    { 0xb520afe0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd600afe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3820afe5, SUC_lexicon_tags_15 },
    { 0x6a80afe5, SUC_lexicon_tags_9 },
    { 0xc040afe7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a20afe9, SUC_lexicon_tags_38 },
    { 0xb780afea, SUC_lexicon_tags_95 },
    { 0xa9e0afeb, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xda80afed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7020afef, SUC_lexicon_tags_25 },
    { 0x60c0afef, SUC_lexicon_tags_9 },
    { 0xb080aff0, SUC_lexicon_tags_34 },
    { 0xf560aff0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe980aff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64c0affa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea0affd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbac0afff, SUC_lexicon_tags_28 },
    { 0x58a0afff, SUC_lexicon_tags_37 },
    { 0xffc0afff, SUC_lexicon_tags_11 },
    { 0x2d00b002, SUC_lexicon_tags_11 },
    { 0x4520b003, SUC_lexicon_tags_17 },
    { 0x3000b004, SUC_lexicon_tags_11 },
    { 0x1180b005, SUC_lexicon_tags_2 },
    { 0x9660b006, SUC_lexicon_tags_1 },
    { 0xc1e0b007, SUC_lexicon_tags_26 },
    { 0xe360b008, SUC_lexicon_tags_49 },
    { 0xf980b005, SUC_lexicon_tags_26 },
    { 0x5d20b00a, SUC_lexicon_tags_43 },
    { 0x1460b00b, SUC_lexicon_tags_45 },
    { 0x4ea0b00b, SUC_lexicon_tags_1132 },
    { 0xbb20b00d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc20b00f, SUC_lexicon_tags_17 },
    { 0x5a0b010, SUC_lexicon_tags_0 },
    { 0x6460b011, SUC_lexicon_tags_7 },
    { 0xf2e0b012, SUC_lexicon_tags_17 },
    { 0x3a80b010, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc80b017, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee0b01a, SUC_lexicon_tags_15 },
    { 0x62a0b01a, SUC_lexicon_tags_45 },
    { 0xd2e0b01c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x8460b01e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a0b022, SUC_lexicon_tags_37 },
    { 0x9620b022, SUC_lexicon_tags_15 },
    { 0xcb20b022, SUC_lexicon_tags_7 },
    { 0xf960b025, SUC_lexicon_tags_21 },
    { 0x5f60b025, SUC_lexicon_tags_1 },
    { 0x7e00b027, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8240b029, SUC_lexicon_tags_1 },
    { 0x1060b02a, SUC_lexicon_tags_1 },
    { 0xdc20b02a, SUC_lexicon_tags_28 },
    { 0x9140b02a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3700b02f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e0b031, SUC_lexicon_tags_31 },
    { 0x90c0b032, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a0b037, SUC_lexicon_tags_23 },
    { 0x7300b037, SUC_lexicon_tags_30 },
    { 0x3960b038, SUC_lexicon_tags_37 },
    { 0x3760b03a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd980b03f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da0b043, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb60b048, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa3e0b04a, SUC_lexicon_tags_26 },
    { 0x8020b04b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe2c0b04d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8320b050, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e0b053, SUC_lexicon_tags_1 },
    { 0x800b053, SUC_lexicon_tags_19 },
    { 0xeca0b055, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a60b05d, SUC_lexicon_tags_37 },
    { 0x4600b05e, SUC_lexicon_tags_37 },
    { 0xe360b05f, SUC_lexicon_tags_47 },
    { 0xb7c0b05f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc0b069, SUC_lexicon_tags_26 },
    { 0x8bc0b069, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xbe40b06c, SUC_lexicon_tags_17 },
    { 0xd720b06d, SUC_lexicon_tags_9 },
    { 0x3380b06c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5480b070, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc0b074, SUC_lexicon_tags_25 },
    { 0x5e40b074, SUC_lexicon_tags_29 },
    { 0xbee0b074, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7720b07d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9920b082, SUC_lexicon_tags_23 },
    { 0xbb80b083, SUC_lexicon_tags_37 },
    { 0x4e80b084, SUC_lexicon_tags_2 },
    { 0x3360b083, SUC_lexicon_tags_47 },
    { 0xa820b086, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b20b088, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7660b08a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2160b08c, SUC_lexicon_tags_37 },
    { 0x57a0b08d, SUC_lexicon_tags_14 },
    { 0xcf80b08e, SUC_lexicon_tags_31 },
    { 0x9560b08f, SUC_lexicon_tags_69 },
    { 0x7a0b090, SUC_lexicon_tags_37 },
    { 0x9ba0b091, SUC_lexicon_tags_11 },
    { 0x9020b091, SUC_lexicon_tags_14 },
    { 0xfe0b091, SUC_lexicon_tags_11 },
    { 0xade0b08d, SUC_lexicon_tags_45 },
    { 0xcde0b092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a0b09a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf460b09f, SUC_lexicon_tags_17 },
    { 0xd920b0a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9d00b0a2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd080b0a7, SUC_lexicon_tags_37 },
    { 0x5ae0b0a7, SUC_lexicon_tags_23 },
    { 0xce0b0a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb60b0ac, SUC_lexicon_tags_68 },
    { 0xbf60b0ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x50c0b0af, SUC_lexicon_tags_21 },
    { 0xa100b0af, SUC_lexicon_tags_12 },
    { 0x4660b0b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a0b0b4, SUC_lexicon_tags_60 },
    { 0x5c20b0b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce0b0ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e00b0bc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa240b0c4, SUC_lexicon_tags_15 },
    { 0x5ac0b0c5, SUC_lexicon_tags_49 },
    { 0x9b60b0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca40b0c8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b40b0cc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a0b0cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb260b0d1, SUC_lexicon_tags_54 },
    { 0x1b60b0d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5480b0d4, SUC_lexicon_tags_17 },
    { 0x8780b0d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc0b0d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc360b0db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26a0b0dd, SUC_lexicon_tags_11 },
    { 0x6340b0de, SUC_lexicon_tags_43 },
    { 0x30a0b0dd, SUC_lexicon_tags_47 },
    { 0x1000b0de, SUC_lexicon_tags_28 },
    { 0xad60b0e0, SUC_lexicon_tags_62 },
    { 0x48e0b0e0, SUC_lexicon_tags_255 },
    { 0xcda0b0e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb440b0e5, SUC_lexicon_tags_11 },
    { 0xb140b0e6, SUC_lexicon_tags_1 },
    { 0xc400b0e7, SUC_lexicon_tags_25 },
    { 0x7760b0e7, SUC_lexicon_tags_1 },
    { 0x55a0b0e8, SUC_lexicon_tags_1 },
    { 0xe5c0b0ea, SUC_lexicon_tags_10 },
    { 0x8fc0b0e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7600b0f1, SUC_lexicon_tags_3 },
    { 0x2c20b0f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe0b0f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd460b0f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6000b0fd, SUC_lexicon_tags_244 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9780b101, SUC_lexicon_tags_17 },
    { 0x4920b101, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfb20b104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2740b10c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9200b10e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740b113, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a0b116, SUC_lexicon_tags_60 },
    { 0x9960b116, SUC_lexicon_tags_79 },
    { 0x8380b118, SUC_lexicon_tags_39 },
    { 0x5c80b117, SUC_lexicon_tags_15 },
    { 0x5180b118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec0b11f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9e0b121, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0b124, SUC_lexicon_tags_25 },
    { 0xa020b125, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc520b12c, SUC_lexicon_tags_39 },
    { 0x8a0b12c, SUC_lexicon_tags_1 },
    { 0xef20b12c, SUC_lexicon_tags_38 },
    { 0xe740b12e, SUC_lexicon_tags_29 },
    { 0x7a0b12e, SUC_lexicon_tags_1288 },
    { 0, NULL },
    { 0x3d40b132, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc260b135, SUC_lexicon_tags_11 },
    { 0x3ea0b135, SUC_lexicon_tags_15 },
    { 0x87e0b137, SUC_lexicon_tags_60 },
    { 0x1e0b135, SUC_lexicon_tags_17 },
    { 0xc340b139, SUC_lexicon_tags_2 },
    { 0x8ae0b139, SUC_lexicon_tags_17 },
    { 0x5180b13b, SUC_lexicon_tags_1 },
    { 0x80a0b139, SUC_lexicon_tags_46 },
    { 0xc720b13a, SUC_lexicon_tags_11 },
    { 0xe2e0b136, SUC_lexicon_tags_38 },
    { 0x5340b13f, SUC_lexicon_tags_41 },
    { 0xae0b13f, SUC_lexicon_tags_37 },
    { 0xab60b141, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c80b148, SUC_lexicon_tags_0 },
    { 0x9d20b149, SUC_lexicon_tags_43 },
    { 0xc820b149, SUC_lexicon_tags_9 },
    { 0xe660b14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a20b14e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c0b151, SUC_lexicon_tags_25 },
    { 0x26a0b151, SUC_lexicon_tags_68 },
    { 0x9340b153, SUC_lexicon_tags_60 },
    { 0x90a0b154, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e60b156, SUC_lexicon_tags_17 },
    { 0xfe00b157, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9300b15a, SUC_lexicon_tags_39 },
    { 0x6fc0b15a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7540b162, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de0b166, SUC_lexicon_tags_25 },
    { 0xeca0b167, SUC_lexicon_tags_48 },
    { 0xeca0b167, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c0b16e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6220b172, SUC_lexicon_tags_25 },
    { 0xe100b173, SUC_lexicon_tags_33 },
    { 0x9780b174, SUC_lexicon_tags_29 },
    { 0xe6c0b173, SUC_lexicon_tags_106 },
    { 0xd740b176, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e0b179, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be0b17e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6200b18a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd540b18e, SUC_lexicon_tags_9 },
    { 0x3b00b18f, SUC_lexicon_tags_2 },
    { 0x7e0b18f, SUC_lexicon_tags_25 },
    { 0x73a0b18f, SUC_lexicon_tags_43 },
    { 0xdf60b192, SUC_lexicon_tags_37 },
    { 0x3200b193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbca0b195, SUC_lexicon_tags_37 },
    { 0xba60b196, SUC_lexicon_tags_7 },
    { 0x4960b197, SUC_lexicon_tags_17 },
    { 0x5200b195, SUC_lexicon_tags_15 },
    { 0x3580b199, SUC_lexicon_tags_32 },
    { 0x2dc0b19a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e0b19f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc0b1a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3e40b1a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5380b1aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a0b1af, SUC_lexicon_tags_25 },
    { 0xb460b1af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e0b1b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcbe0b1b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c0b1bf, SUC_lexicon_tags_12 },
    { 0x6e00b1c0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xd900b1c2, SUC_lexicon_tags_15 },
    { 0xfc20b1c3, SUC_lexicon_tags_26 },
    { 0xb800b1c4, SUC_lexicon_tags_15 },
    { 0x71c0b1c5, SUC_lexicon_tags_25 },
    { 0x8d20b1c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7e40b1c8, SUC_lexicon_tags_61 },
    { 0x9640b1c9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460b1cd, SUC_lexicon_tags_17 },
    { 0x7760b1ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8620b1d1, SUC_lexicon_tags_17 },
    { 0x5980b1d2, SUC_lexicon_tags_26 },
    { 0x80e0b1d2, SUC_lexicon_tags_45 },
    { 0x4140b1d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1320b1d9, SUC_lexicon_tags_37 },
    { 0xc020b1d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3560b1e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da0b1e8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea0b1ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5860b1ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ae0b1f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1520b1f4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xca0b1f6, SUC_lexicon_tags_12 },
    { 0xa440b1f7, SUC_lexicon_tags_33 },
    { 0x2c60b1f7, SUC_lexicon_tags_8 },
    { 0xbc00b1f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b00b1fb, SUC_lexicon_tags_103 },
    { 0x5a0b1fb, SUC_lexicon_tags_25 },
    { 0xeae0b1fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e0b200, SUC_lexicon_tags_26 },
    { 0x8a00b200, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d60b209, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2320b210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe760b212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc840b214, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x65a0b216, SUC_lexicon_tags_11 },
    { 0x1120b217, SUC_lexicon_tags_7 },
    { 0xbe80b217, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80b21f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xd960b221, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0b224, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8620b227, SUC_lexicon_tags_37 },
    { 0x8a0b227, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7200b230, SUC_lexicon_tags_0 },
    { 0x58e0b231, SUC_lexicon_tags_0 },
    { 0x5140b232, SUC_lexicon_tags_63 },
    { 0x61c0b233, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xdcc0b235, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6c20b237, SUC_lexicon_tags_14 },
    { 0xb460b238, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e0b23c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7a20b23e, SUC_lexicon_tags_4 },
    { 0xdd60b23e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a0b242, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d60b245, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98c0b247, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a40b24d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd40b24f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3a0b251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2240b253, SUC_lexicon_tags_38 },
    { 0xf760b253, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c0b25b, SUC_lexicon_tags_39 },
    { 0x9c0b25c, SUC_lexicon_tags_201 },
    { 0xe340b25b, SUC_lexicon_tags_11 },
    { 0xfb40b25e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e0b262, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x580b264, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f20b267, SUC_lexicon_tags_68 },
    { 0xe1a0b268, SUC_lexicon_tags_12 },
    { 0xfb40b268, SUC_lexicon_tags_21 },
    { 0x3340b269, SUC_lexicon_tags_15 },
    { 0x23a0b26b, SUC_lexicon_tags_11 },
    { 0x2520b26c, SUC_lexicon_tags_17 },
    { 0xec40b26a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e0b27c, SUC_lexicon_tags_25 },
    { 0xa220b27c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e0b280, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b40b282, SUC_lexicon_tags_27 },
    { 0xac80b283, SUC_lexicon_tags_15 },
    { 0xabc0b284, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce0b28d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc0b290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa920b293, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf480b299, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9920b29c, SUC_lexicon_tags_47 },
    { 0x19e0b29c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a0b2a0, SUC_lexicon_tags_27 },
    { 0x2200b2a1, SUC_lexicon_tags_12 },
    { 0xaba0b2a1, SUC_lexicon_tags_37 },
    { 0x7fc0b2a1, SUC_lexicon_tags_17 },
    { 0x8120b2a2, SUC_lexicon_tags_27 },
    { 0xc960b2a4, SUC_lexicon_tags_46 },
    { 0xf0e0b2a6, SUC_lexicon_tags_25 },
    { 0xcdc0b2a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7340b2aa, SUC_lexicon_tags_33 },
    { 0x4b60b2ab, SUC_lexicon_tags_30 },
    { 0xdc20b2ac, SUC_lexicon_tags_32 },
    { 0x5ce0b2ab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8160b2b3, SUC_lexicon_tags_17 },
    { 0x920b2b4, SUC_lexicon_tags_62 },
    { 0xf9e0b2b4, SUC_lexicon_tags_82 },
    { 0xbf40b2b6, SUC_lexicon_tags_38 },
    { 0xd460b2b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e40b2be, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd220b2c7, SUC_lexicon_tags_17 },
    { 0xe860b2c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c80b2ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbec0b2cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa540b2cf, SUC_lexicon_tags_43 },
    { 0xad00b2d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x48e0b2d2, SUC_lexicon_tags_37 },
    { 0xb240b2d3, SUC_lexicon_tags_28 },
    { 0x52e0b2d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9900b2dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcde0b2de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe360b2e0, SUC_lexicon_tags_11 },
    { 0xcd60b2e0, SUC_lexicon_tags_11 },
    { 0x9ce0b2e2, SUC_lexicon_tags_541 },
    { 0x2180b2e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd80b2e5, SUC_lexicon_tags_21 },
    { 0xa7a0b2e5, SUC_lexicon_tags_2 },
    { 0x5cc0b2e6, SUC_lexicon_tags_14 },
    { 0x7660b2e8, SUC_lexicon_tags_25 },
    { 0x5a00b2e9, SUC_lexicon_tags_12 },
    { 0x77c0b2ea, SUC_lexicon_tags_45 },
    { 0x5d60b2e5, SUC_lexicon_tags_15 },
    { 0x7ea0b2ec, SUC_lexicon_tags_24 },
    { 0x5da0b2e6, SUC_lexicon_tags_107 },
    { 0xe800b2e7, SUC_lexicon_tags_17 },
    { 0x44a0b2ef, SUC_lexicon_tags_60 },
    { 0x3660b2f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e60b2f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3380b2fc, SUC_lexicon_tags_4 },
    { 0xa1e0b2fd, SUC_lexicon_tags_1 },
    { 0xa640b2fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96a0b300, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x540b303, SUC_lexicon_tags_26 },
    { 0x24c0b303, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x7480b306, SUC_lexicon_tags_23 },
    { 0x4f60b307, SUC_lexicon_tags_37 },
    { 0x5960b308, SUC_lexicon_tags_105 },
    { 0x6460b306, SUC_lexicon_tags_1 },
    { 0xed40b30a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9a80b30c, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad40b312, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c0b317, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf060b31b, SUC_lexicon_tags_29 },
    { 0xe620b31b, SUC_lexicon_tags_61 },
    { 0x2c60b31d, SUC_lexicon_tags_23 },
    { 0x1ec0b31e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be0b321, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8460b324, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe360b327, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e60b329, SUC_lexicon_tags_45 },
    { 0xa620b32a, SUC_lexicon_tags_37 },
    { 0x3b20b32a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfec0b32d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac0b332, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xca40b334, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b337, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc20b339, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e20b33b, SUC_lexicon_tags_15 },
    { 0xfe20b33c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfdc0b33e, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c0b342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce60b345, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e40b347, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e80b34b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xf0c0b34d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc0b356, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc00b35b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x140b35d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0b363, SUC_lexicon_tags_37 },
    { 0x6ec0b363, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb920b368, SUC_lexicon_tags_8 },
    { 0x1380b369, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b20b375, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d40b37c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe920b37e, SUC_lexicon_tags_2 },
    { 0xef40b37f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a0b382, SUC_lexicon_tags_2 },
    { 0x4140b383, SUC_lexicon_tags_19 },
    { 0x9060b383, SUC_lexicon_tags_7 },
    { 0x4ec0b382, SUC_lexicon_tags_37 },
    { 0x6ec0b383, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8e20b388, SUC_lexicon_tags_0 },
    { 0xdfc0b389, SUC_lexicon_tags_15 },
    { 0xa940b38a, SUC_lexicon_tags_17 },
    { 0x4f20b389, SUC_lexicon_tags_6 },
    { 0xac80b38c, SUC_lexicon_tags_2 },
    { 0xbd20b38d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c40b38f, SUC_lexicon_tags_1146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a0b393, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80b395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa480b398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8120b39d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe820b3a0, SUC_lexicon_tags_12 },
    { 0xfe60b3a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a0b3a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc0b3a9, SUC_lexicon_tags_37 },
    { 0x1ba0b3a9, SUC_lexicon_tags_7 },
    { 0xe8e0b3ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2280b3ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb980b3b0, SUC_lexicon_tags_19 },
    { 0xdbe0b3b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4680b3bb, SUC_lexicon_tags_7 },
    { 0xd160b3bc, SUC_lexicon_tags_7 },
    { 0x1dc0b3bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa380b3bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c0b3c5, SUC_lexicon_tags_12 },
    { 0x4dc0b3c5, SUC_lexicon_tags_11 },
    { 0x4420b3c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9200b3c9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1680b3ce, SUC_lexicon_tags_12 },
    { 0xdb60b3cf, SUC_lexicon_tags_37 },
    { 0x9820b3d0, SUC_lexicon_tags_12 },
    { 0x1a80b3d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c0b3d4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce0b3d8, SUC_lexicon_tags_29 },
    { 0x9a40b3d9, SUC_lexicon_tags_37 },
    { 0x7d00b3d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c0b3df, SUC_lexicon_tags_7 },
    { 0x12a0b3df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf80b3e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee0b3e8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a0b3f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa400b3f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd000b3f9, SUC_lexicon_tags_17 },
    { 0x2660b3f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe300b3fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3520b3ff, SUC_lexicon_tags_14 },
    { 0xf320b400, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb340b402, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8660b406, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc500b409, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e0b40d, SUC_lexicon_tags_19 },
    { 0x53e0b40e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe0b412, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe60b414, SUC_lexicon_tags_60 },
    { 0x3140b415, SUC_lexicon_tags_7 },
    { 0x1e40b416, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8780b418, SUC_lexicon_tags_30 },
    { 0x5220b419, SUC_lexicon_tags_2 },
    { 0xc420b41a, SUC_lexicon_tags_12 },
    { 0x64c0b41b, SUC_lexicon_tags_30 },
    { 0xc560b41c, SUC_lexicon_tags_4 },
    { 0x5220b41d, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xe280b41f, SUC_lexicon_tags_68 },
    { 0xf340b41f, SUC_lexicon_tags_37 },
    { 0xc6c0b421, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9e0b423, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3860b42a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6e0b42c, SUC_lexicon_tags_68 },
    { 0x8620b42d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdea0b42f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd3e0b431, SUC_lexicon_tags_1 },
    { 0x3720b431, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf80b43b, SUC_lexicon_tags_0 },
    { 0xa8c0b43b, SUC_lexicon_tags_27 },
    { 0x66a0b43c, SUC_lexicon_tags_37 },
    { 0x7d60b43d, SUC_lexicon_tags_13 },
    { 0xcf60b43b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd700b447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb260b449, SUC_lexicon_tags_13 },
    { 0xdd60b44a, SUC_lexicon_tags_11 },
    { 0x3220b44b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x98a0b44d, SUC_lexicon_tags_60 },
    { 0x640b44e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x260b450, SUC_lexicon_tags_37 },
    { 0xdf00b450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee0b454, SUC_lexicon_tags_37 },
    { 0x8d60b454, SUC_lexicon_tags_43 },
    { 0x8820b456, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5860b458, SUC_lexicon_tags_7 },
    { 0xc5c0b459, SUC_lexicon_tags_17 },
    { 0xb240b45a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e0b45e, SUC_lexicon_tags_2 },
    { 0x8720b45e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3de0b461, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7380b471, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe0b475, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x32c0b477, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a0b47b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ee0b47d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd780b481, SUC_lexicon_tags_63 },
    { 0xcfa0b482, SUC_lexicon_tags_17 },
    { 0x1da0b481, SUC_lexicon_tags_14 },
    { 0x5240b484, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe920b489, SUC_lexicon_tags_77 },
    { 0x9100b489, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9380b493, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a00b49a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b40b4a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc6c0b4a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa0b4a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9120b4aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8560b4ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd900b4b3, SUC_lexicon_tags_6 },
    { 0xabe0b4b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61e0b4b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd780b4ba, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e0b4be, SUC_lexicon_tags_10 },
    { 0xf720b4bf, SUC_lexicon_tags_36 },
    { 0x21c0b4c0, SUC_lexicon_tags_37 },
    { 0xa860b4c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e0b4c4, SUC_lexicon_tags_37 },
    { 0xea60b4c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c0b4ca, SUC_lexicon_tags_37 },
    { 0x7960b4cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa20b4cd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340b4d1, SUC_lexicon_tags_4 },
    { 0xfbc0b4d2, SUC_lexicon_tags_2 },
    { 0x72c0b4d3, SUC_lexicon_tags_28 },
    { 0xbe00b4d4, SUC_lexicon_tags_8 },
    { 0x55e0b4d4, SUC_lexicon_tags_4 },
    { 0x3d20b4d4, SUC_lexicon_tags_6 },
    { 0x5920b4d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a00b4d9, SUC_lexicon_tags_45 },
    { 0x4a0b4da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be0b4e2, SUC_lexicon_tags_25 },
    { 0xcc80b4e3, SUC_lexicon_tags_47 },
    { 0xc600b4e4, SUC_lexicon_tags_2 },
    { 0x620b4e4, SUC_lexicon_tags_8 },
    { 0x1c00b4e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe640b4ea, SUC_lexicon_tags_11 },
    { 0x2660b4eb, SUC_lexicon_tags_15 },
    { 0xdc80b4ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e0b4ef, SUC_lexicon_tags_25 },
    { 0x9060b4ef, SUC_lexicon_tags_11 },
    { 0x3920b4ef, SUC_lexicon_tags_12 },
    { 0x83e0b4f2, SUC_lexicon_tags_21 },
    { 0xe540b4f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9460b4f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c0b4fa, SUC_lexicon_tags_28 },
    { 0xb6a0b4fb, SUC_lexicon_tags_37 },
    { 0xa8c0b4fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50a0b4fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb20b502, SUC_lexicon_tags_7 },
    { 0x69e0b503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a0b50c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e80b511, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e40b516, SUC_lexicon_tags_10 },
    { 0x9120b517, SUC_lexicon_tags_38 },
    { 0x6e80b516, SUC_lexicon_tags_2 },
    { 0x93c0b519, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea0b51c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x52c0b51e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2940b520, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2500b522, SUC_lexicon_tags_11 },
    { 0x95c0b523, SUC_lexicon_tags_11 },
    { 0x6b60b524, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9860b52d, SUC_lexicon_tags_0 },
    { 0x1160b52d, SUC_lexicon_tags_17 },
    { 0xfa00b52e, SUC_lexicon_tags_6 },
    { 0x2d20b52f, SUC_lexicon_tags_46 },
    { 0xb580b531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7000b538, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a0b53d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa0b540, SUC_lexicon_tags_9 },
    { 0x6c60b541, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe0b548, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3040b54b, SUC_lexicon_tags_23 },
    { 0x4a80b54c, SUC_lexicon_tags_23 },
    { 0x8940b54c, SUC_lexicon_tags_26 },
    { 0xe8a0b54e, SUC_lexicon_tags_17 },
    { 0x66e0b54f, SUC_lexicon_tags_25 },
    { 0x6580b54f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c40b552, SUC_lexicon_tags_2 },
    { 0x8f80b552, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0b55c, SUC_lexicon_tags_11 },
    { 0x1940b55d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6780b560, SUC_lexicon_tags_11 },
    { 0x8060b560, SUC_lexicon_tags_25 },
    { 0x7bc0b560, SUC_lexicon_tags_2 },
    { 0xfd40b563, SUC_lexicon_tags_37 },
    { 0x20b560, SUC_lexicon_tags_7 },
    { 0xc780b564, SUC_lexicon_tags_43 },
    { 0x8d20b566, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1240b56b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xec40b56e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe3e0b570, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf800b578, SUC_lexicon_tags_48 },
    { 0x7780b579, SUC_lexicon_tags_11 },
    { 0xbe60b57a, SUC_lexicon_tags_37 },
    { 0x96a0b57a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc0b580, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf4e0b582, SUC_lexicon_tags_45 },
    { 0x9240b583, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc720b585, SUC_lexicon_tags_7 },
    { 0xed20b585, SUC_lexicon_tags_6 },
    { 0xffc0b587, SUC_lexicon_tags_14 },
    { 0xd4c0b588, SUC_lexicon_tags_15 },
    { 0x63c0b587, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d00b58c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdd40b58e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd6a0b590, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc80b594, SUC_lexicon_tags_33 },
    { 0x6ac0b595, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e0b59a, SUC_lexicon_tags_53 },
    { 0x3c20b59a, SUC_lexicon_tags_25 },
    { 0x5580b59b, SUC_lexicon_tags_35 },
    { 0x9940b59d, SUC_lexicon_tags_17 },
    { 0x92c0b59e, SUC_lexicon_tags_198 },
    { 0x4ea0b59d, SUC_lexicon_tags_52 },
    { 0x260b59e, SUC_lexicon_tags_32 },
    { 0xf1a0b5a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6480b5a3, SUC_lexicon_tags_17 },
    { 0x3ea0b5a3, SUC_lexicon_tags_21 },
    { 0x3aa0b5a5, SUC_lexicon_tags_10 },
    { 0x7080b5a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8540b5a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e60b5b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb800b5b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec0b5bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72e0b5be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x35e0b5c0, SUC_lexicon_tags_39 },
    { 0xace0b5c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x400b5c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e0b5c6, SUC_lexicon_tags_4 },
    { 0x7b00b5c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6880b5ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1980b5cc, SUC_lexicon_tags_25 },
    { 0x8cc0b5cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0b5d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3880b5d4, SUC_lexicon_tags_11 },
    { 0xd440b5d5, SUC_lexicon_tags_13 },
    { 0xc180b5d5, SUC_lexicon_tags_15 },
    { 0x5720b5d7, SUC_lexicon_tags_2 },
    { 0x2c40b5d8, SUC_lexicon_tags_2 },
    { 0xc840b5d6, SUC_lexicon_tags_65 },
    { 0xb7e0b5da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9000b5df, SUC_lexicon_tags_104 },
    { 0xe520b5e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1180b5e2, SUC_lexicon_tags_37 },
    { 0x24c0b5e3, SUC_lexicon_tags_2 },
    { 0x5f00b5e3, SUC_lexicon_tags_29 },
    { 0x4ee0b5e4, SUC_lexicon_tags_12 },
    { 0x79e0b5e2, SUC_lexicon_tags_0 },
    { 0x5180b5e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40b5eb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf980b5ed, SUC_lexicon_tags_23 },
    { 0x7c00b5ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe660b5f1, SUC_lexicon_tags_6 },
    { 0x48e0b5f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd8a0b5f4, SUC_lexicon_tags_25 },
    { 0x62c0b5f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9520b5f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1ba0b5f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c80b600, SUC_lexicon_tags_27 },
    { 0x8ca0b601, SUC_lexicon_tags_0 },
    { 0x89a0b602, SUC_lexicon_tags_37 },
    { 0xb3e0b603, SUC_lexicon_tags_37 },
    { 0xb40b604, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa60b607, SUC_lexicon_tags_37 },
    { 0xdee0b608, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3720b60a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc6a0b60c, SUC_lexicon_tags_32 },
    { 0x53c0b60d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc680b612, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9760b61a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e60b61c, SUC_lexicon_tags_15 },
    { 0x20b61c, SUC_lexicon_tags_26 },
    { 0x600b61e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x920b620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4140b624, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa140b62c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfee0b62e, SUC_lexicon_tags_16 },
    { 0xc0e0b62f, SUC_lexicon_tags_65 },
    { 0xbf00b62f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf460b632, SUC_lexicon_tags_55 },
    { 0x6500b632, SUC_lexicon_tags_28 },
    { 0x3fc0b632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce0b638, SUC_lexicon_tags_65 },
    { 0x9620b638, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4d40b63b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2480b63f, SUC_lexicon_tags_23 },
    { 0xee00b640, SUC_lexicon_tags_70 },
    { 0xc9e0b641, SUC_lexicon_tags_37 },
    { 0xe600b642, SUC_lexicon_tags_39 },
    { 0xaba0b643, SUC_lexicon_tags_2 },
    { 0x820b644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1420b646, SUC_lexicon_tags_17 },
    { 0xb560b647, SUC_lexicon_tags_11 },
    { 0x6860b648, SUC_lexicon_tags_8 },
    { 0xea60b647, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5e0b64b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc0b64e, SUC_lexicon_tags_12 },
    { 0x3f80b64f, SUC_lexicon_tags_12 },
    { 0x7660b64f, SUC_lexicon_tags_27 },
    { 0xc700b651, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3900b65e, SUC_lexicon_tags_0 },
    { 0xd800b65f, SUC_lexicon_tags_43 },
    { 0x4960b660, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7620b667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8020b669, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa320b66e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3600b672, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd260b675, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb20b677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5240b679, SUC_lexicon_tags_151 },
    { 0x4a60b67a, SUC_lexicon_tags_45 },
    { 0x5bc0b67b, SUC_lexicon_tags_49 },
    { 0x75e0b67b, SUC_lexicon_tags_29 },
    { 0x5d60b67d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc0b682, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa60b685, SUC_lexicon_tags_17 },
    { 0x4520b686, SUC_lexicon_tags_21 },
    { 0x1000b687, SUC_lexicon_tags_23 },
    { 0x1600b687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2620b68c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe200b692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ee0b694, SUC_lexicon_tags_15 },
    { 0xd020b695, SUC_lexicon_tags_38 },
    { 0xe300b696, SUC_lexicon_tags_1 },
    { 0x8a40b697, SUC_lexicon_tags_25 },
    { 0x30a0b696, SUC_lexicon_tags_19 },
    { 0xc260b699, SUC_lexicon_tags_17 },
    { 0xd620b696, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be0b69d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe280b6a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8540b6a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa980b6ad, SUC_lexicon_tags_29 },
    { 0x6c0b6ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e0b6b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3180b6b7, SUC_lexicon_tags_14 },
    { 0x5620b6b8, SUC_lexicon_tags_17 },
    { 0x4c00b6b8, SUC_lexicon_tags_26 },
    { 0x53e0b6b9, SUC_lexicon_tags_84 },
    { 0x29a0b6ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf560b6c0, SUC_lexicon_tags_8 },
    { 0x7fc0b6c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c0b6c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1980b6c7, SUC_lexicon_tags_38 },
    { 0xa040b6c8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa700b6cc, SUC_lexicon_tags_45 },
    { 0xf00b6cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcb80b6cf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6840b6d1, SUC_lexicon_tags_4 },
    { 0x8e60b6d2, SUC_lexicon_tags_37 },
    { 0x5220b6d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6320b6d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a0b6dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xae0b6df, SUC_lexicon_tags_0 },
    { 0x5540b6e0, SUC_lexicon_tags_25 },
    { 0xdae0b6e1, SUC_lexicon_tags_11 },
    { 0x1000b6df, SUC_lexicon_tags_15 },
    { 0x86e0b6e0, SUC_lexicon_tags_1 },
    { 0x5c60b6e4, SUC_lexicon_tags_28 },
    { 0x2e20b6e4, SUC_lexicon_tags_9 },
    { 0xd160b6e6, SUC_lexicon_tags_49 },
    { 0x2260b6e7, SUC_lexicon_tags_49 },
    { 0xb440b6e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6140b6ea, SUC_lexicon_tags_25 },
    { 0xa620b6eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a60b6ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0b6f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2000b6f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da0b6fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfbc0b6fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6260b702, SUC_lexicon_tags_25 },
    { 0xf7c0b703, SUC_lexicon_tags_37 },
    { 0xcf80b702, SUC_lexicon_tags_47 },
    { 0x6e00b702, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba80b709, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd260b70e, SUC_lexicon_tags_14 },
    { 0x22e0b70f, SUC_lexicon_tags_45 },
    { 0x60c0b710, SUC_lexicon_tags_14 },
    { 0xfe0b70e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac0b716, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde40b718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a0b71a, SUC_lexicon_tags_2 },
    { 0xe8e0b71b, SUC_lexicon_tags_82 },
    { 0x1400b71a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc580b71e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d00b723, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a0b72a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x160b72c, SUC_lexicon_tags_37 },
    { 0x2e40b72d, SUC_lexicon_tags_82 },
    { 0x2860b72c, SUC_lexicon_tags_19 },
    { 0x36e0b72e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa780b735, SUC_lexicon_tags_39 },
    { 0x9b20b735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7660b73d, SUC_lexicon_tags_14 },
    { 0x3e80b73e, SUC_lexicon_tags_47 },
    { 0x4100b73f, SUC_lexicon_tags_12 },
    { 0x1ba0b740, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd3e0b742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5720b74b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x4160b74d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e0b753, SUC_lexicon_tags_39 },
    { 0xa040b754, SUC_lexicon_tags_12 },
    { 0xb600b754, SUC_lexicon_tags_12 },
    { 0xe6a0b753, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1540b75a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180b763, SUC_lexicon_tags_30 },
    { 0x26a0b764, SUC_lexicon_tags_25 },
    { 0xb400b763, SUC_lexicon_tags_87 },
    { 0xadc0b763, SUC_lexicon_tags_17 },
    { 0x4940b767, SUC_lexicon_tags_19 },
    { 0x3040b764, SUC_lexicon_tags_12 },
    { 0x5c20b769, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x21e0b76b, SUC_lexicon_tags_47 },
    { 0x1a60b76c, SUC_lexicon_tags_49 },
    { 0x960b76c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb400b775, SUC_lexicon_tags_25 },
    { 0x67a0b776, SUC_lexicon_tags_12 },
    { 0x6460b777, SUC_lexicon_tags_68 },
    { 0xf2c0b776, SUC_lexicon_tags_29 },
    { 0x720b779, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4640b77c, SUC_lexicon_tags_17 },
    { 0x9e40b77d, SUC_lexicon_tags_7 },
    { 0x39e0b77d, SUC_lexicon_tags_17 },
    { 0xf600b77f, SUC_lexicon_tags_33 },
    { 0x9c60b77e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b00b785, SUC_lexicon_tags_1 },
    { 0x3740b786, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4340b78b, SUC_lexicon_tags_17 },
    { 0x7c60b78c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc20b790, SUC_lexicon_tags_25 },
    { 0xb9e0b791, SUC_lexicon_tags_15 },
    { 0xe2a0b792, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb5c0b794, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8940b797, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6800b79a, SUC_lexicon_tags_27 },
    { 0xc340b79a, SUC_lexicon_tags_15 },
    { 0x3ae0b79c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee80b7a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7660b7a3, SUC_lexicon_tags_39 },
    { 0x9c0b7a3, SUC_lexicon_tags_68 },
    { 0x6820b7a3, SUC_lexicon_tags_15 },
    { 0xc140b7a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1620b7a9, SUC_lexicon_tags_95 },
    { 0xa2e0b7aa, SUC_lexicon_tags_7 },
    { 0x50c0b7ab, SUC_lexicon_tags_14 },
    { 0x3a40b7ac, SUC_lexicon_tags_116 },
    { 0x36e0b7a9, SUC_lexicon_tags_11 },
    { 0xeec0b7ae, SUC_lexicon_tags_79 },
    { 0x1620b7ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6500b7b3, SUC_lexicon_tags_28 },
    { 0x5a40b7b4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a0b7bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f00b7bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e0b7c1, SUC_lexicon_tags_25 },
    { 0x7e20b7c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b60b7c6, SUC_lexicon_tags_38 },
    { 0x8c20b7c7, SUC_lexicon_tags_42 },
    { 0x5760b7c7, SUC_lexicon_tags_11 },
    { 0x7220b7c9, SUC_lexicon_tags_37 },
    { 0xa300b7ca, SUC_lexicon_tags_37 },
    { 0xe8e0b7c7, SUC_lexicon_tags_0 },
    { 0x740b7c8, SUC_lexicon_tags_87 },
    { 0xf000b7cd, SUC_lexicon_tags_7 },
    { 0x4940b7cd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7160b7d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd40b7d5, SUC_lexicon_tags_15 },
    { 0x90c0b7d5, SUC_lexicon_tags_25 },
    { 0xbd20b7d7, SUC_lexicon_tags_30 },
    { 0xe180b7d8, SUC_lexicon_tags_19 },
    { 0xd300b7d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3960b7db, SUC_lexicon_tags_11 },
    { 0xa280b7dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfe0b7de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe400b7e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe840b7e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc0b7ea, SUC_lexicon_tags_10 },
    { 0xca40b7ea, SUC_lexicon_tags_49 },
    { 0x7000b7ec, SUC_lexicon_tags_15 },
    { 0x9340b7ec, SUC_lexicon_tags_37 },
    { 0x9060b7ec, SUC_lexicon_tags_7 },
    { 0x8ca0b7ef, SUC_lexicon_tags_47 },
    { 0x5ba0b7ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c40b7f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd780b7ff, SUC_lexicon_tags_11 },
    { 0x66a0b7ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5020b803, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1440b807, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7a80b809, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa380b80d, SUC_lexicon_tags_28 },
    { 0xd9c0b80d, SUC_lexicon_tags_23 },
    { 0x7340b80d, SUC_lexicon_tags_37 },
    { 0x1340b80e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc0b816, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c0b819, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc040b81d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cc0b81f, SUC_lexicon_tags_15 },
    { 0xb9e0b820, SUC_lexicon_tags_84 },
    { 0xa720b820, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa200b823, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b20b826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb80b82a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca0b831, SUC_lexicon_tags_14 },
    { 0xa080b832, SUC_lexicon_tags_17 },
    { 0x6460b832, SUC_lexicon_tags_17 },
    { 0x3820b834, SUC_lexicon_tags_17 },
    { 0x9a20b835, SUC_lexicon_tags_45 },
    { 0x15e0b836, SUC_lexicon_tags_2 },
    { 0xe140b835, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7960b83d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe820b84a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf4a0b84c, SUC_lexicon_tags_12 },
    { 0x9280b84d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c0b856, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1da0b858, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d20b85f, SUC_lexicon_tags_0 },
    { 0xc20b85f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb866, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaac0b868, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd940b86a, SUC_lexicon_tags_4 },
    { 0x6740b86a, SUC_lexicon_tags_61 },
    { 0xca80b86c, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4540b870, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbea0b872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe900b876, SUC_lexicon_tags_25 },
    { 0x89e0b876, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1e40b879, SUC_lexicon_tags_62 },
    { 0x6d40b879, SUC_lexicon_tags_0 },
    { 0x1800b87b, SUC_lexicon_tags_17 },
    { 0x6080b87c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e0b87f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60b883, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a0b889, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec0b88b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad20b88d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d20b88f, SUC_lexicon_tags_11 },
    { 0xa760b890, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa880b892, SUC_lexicon_tags_26 },
    { 0xf680b893, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d00b89a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc740b89d, SUC_lexicon_tags_27 },
    { 0x9420b89e, SUC_lexicon_tags_37 },
    { 0xb040b89f, SUC_lexicon_tags_1 },
    { 0x49e0b89f, SUC_lexicon_tags_11 },
    { 0x1a40b8a0, SUC_lexicon_tags_25 },
    { 0x15a0b89e, SUC_lexicon_tags_19 },
    { 0xe5e0b8a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a0b8ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2260b8b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1980b8b2, SUC_lexicon_tags_65 },
    { 0x1140b8b2, SUC_lexicon_tags_12 },
    { 0x6360b8b2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1560b8b6, SUC_lexicon_tags_23 },
    { 0x6900b8b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2080b8b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb40b8be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbb40b8c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79a0b8c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb20b8c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd640b8c6, SUC_lexicon_tags_17 },
    { 0x1c60b8c7, SUC_lexicon_tags_11 },
    { 0x68c0b8c8, SUC_lexicon_tags_17 },
    { 0x18c0b8c6, SUC_lexicon_tags_14 },
    { 0xdb40b8c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfe0b8cc, SUC_lexicon_tags_84 },
    { 0xdf40b8cd, SUC_lexicon_tags_42 },
    { 0x1b00b8cd, SUC_lexicon_tags_27 },
    { 0xb420b8cf, SUC_lexicon_tags_10 },
    { 0x96e0b8d0, SUC_lexicon_tags_57 },
    { 0xf400b8d1, SUC_lexicon_tags_1 },
    { 0xe5e0b8cc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7800b8d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4120b8d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9340b8db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1340b8dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecc0b8df, SUC_lexicon_tags_37 },
    { 0xba20b8e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb40b8e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7be0b8e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e00b8e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4300b8eb, SUC_lexicon_tags_12 },
    { 0x80a0b8ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a0b8f1, SUC_lexicon_tags_15 },
    { 0x4400b8f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36c0b8f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe060b8f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f40b8fc, SUC_lexicon_tags_65 },
    { 0x86a0b8fd, SUC_lexicon_tags_7 },
    { 0xe920b8fd, SUC_lexicon_tags_12 },
    { 0xf200b8fe, SUC_lexicon_tags_31 },
    { 0x38a0b8fd, SUC_lexicon_tags_63 },
    { 0x4200b8fd, SUC_lexicon_tags_60 },
    { 0x9e20b902, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2140b905, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c0b90b, SUC_lexicon_tags_65 },
    { 0xc220b90c, SUC_lexicon_tags_11 },
    { 0xf540b90b, SUC_lexicon_tags_12 },
    { 0x5c00b90c, SUC_lexicon_tags_16 },
    { 0xc2e0b90f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e0b914, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e0b919, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2840b920, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60b923, SUC_lexicon_tags_33 },
    { 0x67e0b924, SUC_lexicon_tags_0 },
    { 0xd080b925, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1720b92b, SUC_lexicon_tags_17 },
    { 0x9340b92c, SUC_lexicon_tags_2 },
    { 0x12e0b92c, SUC_lexicon_tags_37 },
    { 0x1740b92e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcda0b930, SUC_lexicon_tags_60 },
    { 0xc2c0b931, SUC_lexicon_tags_37 },
    { 0xed60b930, SUC_lexicon_tags_63 },
    { 0xbfe0b930, SUC_lexicon_tags_38 },
    { 0x63c0b930, SUC_lexicon_tags_11 },
    { 0x8d00b935, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf760b939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f80b93b, SUC_lexicon_tags_6 },
    { 0xbe40b93c, SUC_lexicon_tags_14 },
    { 0x7bc0b93c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe200b93f, SUC_lexicon_tags_27 },
    { 0x78c0b940, SUC_lexicon_tags_63 },
    { 0x6a0b941, SUC_lexicon_tags_12 },
    { 0xb1a0b942, SUC_lexicon_tags_45 },
    { 0xa840b943, SUC_lexicon_tags_23 },
    { 0xaf20b941, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5120b946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58c0b948, SUC_lexicon_tags_34 },
    { 0x6620b948, SUC_lexicon_tags_8 },
    { 0x74c0b94a, SUC_lexicon_tags_16 },
    { 0x97a0b949, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa0b94e, SUC_lexicon_tags_179 },
    { 0xfc60b94f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb60b951, SUC_lexicon_tags_32 },
    { 0x9800b952, SUC_lexicon_tags_9 },
    { 0x87a0b951, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c80b95c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd100b95e, SUC_lexicon_tags_37 },
    { 0xc3c0b95f, SUC_lexicon_tags_2 },
    { 0x1f80b95e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3000b963, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xed60b966, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3840b975, SUC_lexicon_tags_14 },
    { 0xb4c0b975, SUC_lexicon_tags_1 },
    { 0x2c0b977, SUC_lexicon_tags_65 },
    { 0x6d20b978, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc320b982, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaa40b984, SUC_lexicon_tags_0 },
    { 0x9020b985, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a0b989, SUC_lexicon_tags_17 },
    { 0xbfc0b98a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e0b98e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4e0b990, SUC_lexicon_tags_33 },
    { 0xdfc0b990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd640b993, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9dc0b995, SUC_lexicon_tags_117 },
    { 0xe720b995, SUC_lexicon_tags_12 },
    { 0xb960b996, SUC_lexicon_tags_27 },
    { 0xe4e0b998, SUC_lexicon_tags_15 },
    { 0xa0a0b997, SUC_lexicon_tags_37 },
    { 0xc1a0b997, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e0b99d, SUC_lexicon_tags_7 },
    { 0xee40b99e, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e0b9a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe280b9a7, SUC_lexicon_tags_1 },
    { 0xb060b9a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2600b9af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2820b9b5, SUC_lexicon_tags_17 },
    { 0xa4e0b9b6, SUC_lexicon_tags_1 },
    { 0x9fa0b9b7, SUC_lexicon_tags_14 },
    { 0x5340b9b8, SUC_lexicon_tags_38 },
    { 0xf500b9b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a0b9bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3780b9bf, SUC_lexicon_tags_25 },
    { 0x35c0b9c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0b9c6, SUC_lexicon_tags_27 },
    { 0x6d00b9c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x26a0b9c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa0b9d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e80b9d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e0b9d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f80b9db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b60b9de, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x58a0b9e0, SUC_lexicon_tags_17 },
    { 0x6be0b9e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8700b9e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa020b9e6, SUC_lexicon_tags_15 },
    { 0xf9e0b9e7, SUC_lexicon_tags_32 },
    { 0xf480b9e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6140b9eb, SUC_lexicon_tags_2 },
    { 0x54a0b9eb, SUC_lexicon_tags_12 },
    { 0x6c60b9eb, SUC_lexicon_tags_87 },
    { 0x5fa0b9ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1240b9f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb020b9f5, SUC_lexicon_tags_61 },
    { 0x1a00b9f6, SUC_lexicon_tags_11 },
    { 0x1f20b9f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3820ba01, SUC_lexicon_tags_65 },
    { 0x9180ba02, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6be0ba04, SUC_lexicon_tags_17 },
    { 0xb7a0ba05, SUC_lexicon_tags_43 },
    { 0x4700ba06, SUC_lexicon_tags_63 },
    { 0x9880ba07, SUC_lexicon_tags_37 },
    { 0xd1e0ba07, SUC_lexicon_tags_4 },
    { 0xaea0ba08, SUC_lexicon_tags_12 },
    { 0xbfc0ba05, SUC_lexicon_tags_38 },
    { 0xf6e0ba0a, SUC_lexicon_tags_8 },
    { 0x67c0ba0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc660ba0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a0ba19, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc20ba1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a0ba1f, SUC_lexicon_tags_31 },
    { 0xd6e0ba20, SUC_lexicon_tags_1 },
    { 0x8360ba21, SUC_lexicon_tags_52 },
    { 0x2060ba22, SUC_lexicon_tags_25 },
    { 0xbb80ba1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff60ba25, SUC_lexicon_tags_28 },
    { 0xdb80ba26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f00ba28, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c80ba30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe820ba32, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4680ba35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6720ba3b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc0ba42, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x1d60ba44, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb920ba4a, SUC_lexicon_tags_11 },
    { 0xb420ba4a, SUC_lexicon_tags_9 },
    { 0x1e80ba4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8a0ba4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb420ba50, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f40ba55, SUC_lexicon_tags_63 },
    { 0x79a0ba56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3900ba59, SUC_lexicon_tags_23 },
    { 0xa540ba5a, SUC_lexicon_tags_28 },
    { 0xef80ba5b, SUC_lexicon_tags_18 },
    { 0x4780ba5a, SUC_lexicon_tags_29 },
    { 0x4480ba5d, SUC_lexicon_tags_11 },
    { 0x70c0ba5d, SUC_lexicon_tags_61 },
    { 0xe840ba5f, SUC_lexicon_tags_27 },
    { 0x91e0ba5f, SUC_lexicon_tags_36 },
    { 0xd2e0ba61, SUC_lexicon_tags_7 },
    { 0x5460ba5f, SUC_lexicon_tags_2 },
    { 0x5040ba62, SUC_lexicon_tags_2 },
    { 0x7840ba60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8940ba66, SUC_lexicon_tags_14 },
    { 0x6ac0ba67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e0ba6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea0ba6e, SUC_lexicon_tags_1 },
    { 0x7ea0ba6f, SUC_lexicon_tags_29 },
    { 0x5ea0ba70, SUC_lexicon_tags_47 },
    { 0x2460ba6e, SUC_lexicon_tags_17 },
    { 0xfe20ba72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3640ba7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6a0ba7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c0ba82, SUC_lexicon_tags_12 },
    { 0xac60ba83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46a0ba85, SUC_lexicon_tags_15 },
    { 0x7c00ba86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7260ba88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xde80ba8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb40ba8c, SUC_lexicon_tags_19 },
    { 0xc2c0ba8c, SUC_lexicon_tags_30 },
    { 0x73a0ba8e, SUC_lexicon_tags_44 },
    { 0xb100ba8f, SUC_lexicon_tags_79 },
    { 0x9c0ba8e, SUC_lexicon_tags_29 },
    { 0xe20ba91, SUC_lexicon_tags_49 },
    { 0x5900ba8f, SUC_lexicon_tags_11 },
    { 0x8780ba8f, SUC_lexicon_tags_77 },
    { 0xac00ba92, SUC_lexicon_tags_25 },
    { 0xc680ba8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2780ba9a, SUC_lexicon_tags_324 },
    { 0x11e0ba9a, SUC_lexicon_tags_11 },
    { 0x59a0ba9a, SUC_lexicon_tags_14 },
    { 0x7800ba9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7320ba9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5480baa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4100baaa, SUC_lexicon_tags_34 },
    { 0x71e0baab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220bab2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6760bab7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb280bab9, SUC_lexicon_tags_37 },
    { 0xbdc0bab9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7280babf, SUC_lexicon_tags_23 },
    { 0x1600bac0, SUC_lexicon_tags_37 },
    { 0x4860bac0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4000bac4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec0bac9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c0bacc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ce0bace, SUC_lexicon_tags_27 },
    { 0x73a0bace, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c0bad2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1040bad5, SUC_lexicon_tags_66 },
    { 0xd60bad6, SUC_lexicon_tags_25 },
    { 0x35a0bad7, SUC_lexicon_tags_1 },
    { 0x54c0bad8, SUC_lexicon_tags_42 },
    { 0x94a0bad9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2240badb, SUC_lexicon_tags_43 },
    { 0x22e0badc, SUC_lexicon_tags_37 },
    { 0x6d00badd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b40bae1, SUC_lexicon_tags_11 },
    { 0x4460bae1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc0bae6, SUC_lexicon_tags_47 },
    { 0x8320bae6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7300baeb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf720baed, SUC_lexicon_tags_10 },
    { 0xd420baee, SUC_lexicon_tags_31 },
    { 0x5aa0baef, SUC_lexicon_tags_2 },
    { 0xd980baee, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x9440baf2, SUC_lexicon_tags_1 },
    { 0x2920baf2, SUC_lexicon_tags_68 },
    { 0xf480baf4, SUC_lexicon_tags_27 },
    { 0x5580baf5, SUC_lexicon_tags_23 },
    { 0xa560baf2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71c0baf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c0bafc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d40bafe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa60bb00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e40bb04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e0bb07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e0bb0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8420bb14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7900bb17, SUC_lexicon_tags_10 },
    { 0x3aa0bb18, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2780bb20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e0bb26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f00bb28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe0bb2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e0bb34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5020bb3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240bb3f, SUC_lexicon_tags_12 },
    { 0xc40bb40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3460bb44, SUC_lexicon_tags_63 },
    { 0x1f20bb45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf80bb47, SUC_lexicon_tags_63 },
    { 0x42a0bb47, SUC_lexicon_tags_6 },
    { 0x6000bb47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa420bb4c, SUC_lexicon_tags_17 },
    { 0x13a0bb4c, SUC_lexicon_tags_23 },
    { 0x2420bb4c, SUC_lexicon_tags_15 },
    { 0xf6e0bb4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba0bb52, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6c80bb54, SUC_lexicon_tags_29 },
    { 0xbd60bb55, SUC_lexicon_tags_26 },
    { 0x8800bb55, SUC_lexicon_tags_61 },
    { 0xf0e0bb57, SUC_lexicon_tags_12 },
    { 0xdd40bb57, SUC_lexicon_tags_15 },
    { 0xb620bb59, SUC_lexicon_tags_11 },
    { 0xa580bb5a, SUC_lexicon_tags_44 },
    { 0x49e0bb5b, SUC_lexicon_tags_49 },
    { 0x87a0bb5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa020bb5e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc260bb61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6520bb63, SUC_lexicon_tags_61 },
    { 0x92e0bb64, SUC_lexicon_tags_43 },
    { 0xc2a0bb65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd320bb68, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x820bb6b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20bb6e, SUC_lexicon_tags_74 },
    { 0x9460bb6e, SUC_lexicon_tags_8 },
    { 0xfda0bb70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a00bb73, SUC_lexicon_tags_25 },
    { 0x8220bb74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a0bb85, SUC_lexicon_tags_11 },
    { 0x6900bb86, SUC_lexicon_tags_63 },
    { 0x2a00bb85, SUC_lexicon_tags_11 },
    { 0x4b20bb86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec0bb8d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e0bb93, SUC_lexicon_tags_12 },
    { 0x6a00bb93, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2280bb9a, SUC_lexicon_tags_1 },
    { 0xc7e0bb9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44c0bb9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b40bba0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7060bbb0, SUC_lexicon_tags_25 },
    { 0xf5c0bbb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da0bbb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab20bbbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b80bbbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6300bbc0, SUC_lexicon_tags_32 },
    { 0x8880bbc0, SUC_lexicon_tags_37 },
    { 0xf180bbc2, SUC_lexicon_tags_10 },
    { 0x5940bbc3, SUC_lexicon_tags_15 },
    { 0x7f80bbc2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa520bbc8, SUC_lexicon_tags_12 },
    { 0x8180bbc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57c0bbcb, SUC_lexicon_tags_37 },
    { 0xdf00bbcc, SUC_lexicon_tags_31 },
    { 0xca60bbcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe0bbd0, SUC_lexicon_tags_1 },
    { 0x4d20bbd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e0bbd6, SUC_lexicon_tags_17 },
    { 0x3d40bbd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0bbda, SUC_lexicon_tags_25 },
    { 0x8020bbdb, SUC_lexicon_tags_2 },
    { 0xd180bbdc, SUC_lexicon_tags_31 },
    { 0x1da0bbdd, SUC_lexicon_tags_11 },
    { 0xf980bbde, SUC_lexicon_tags_7 },
    { 0x380bbdf, SUC_lexicon_tags_17 },
    { 0x7420bbdb, SUC_lexicon_tags_62 },
    { 0xf120bbe1, SUC_lexicon_tags_33 },
    { 0xb360bbe0, SUC_lexicon_tags_34 },
    { 0x4360bbe2, SUC_lexicon_tags_14 },
    { 0x60e0bbe4, SUC_lexicon_tags_47 },
    { 0xe920bbda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfc0bbe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa820bbea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x740bbec, SUC_lexicon_tags_33 },
    { 0x7b20bbed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba0bbf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe40bbf3, SUC_lexicon_tags_37 },
    { 0xd220bbf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee0bbf7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580bbfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade0bbff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c0bc03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba0bc07, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80bc10, SUC_lexicon_tags_4 },
    { 0x39c0bc10, SUC_lexicon_tags_14 },
    { 0x3a60bc11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c0bc1d, SUC_lexicon_tags_17 },
    { 0xb880bc1e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1900bc22, SUC_lexicon_tags_19 },
    { 0x5f00bc23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3000bc26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1940bc29, SUC_lexicon_tags_25 },
    { 0x4fc0bc2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x12c0bc2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf920bc2f, SUC_lexicon_tags_7 },
    { 0x53c0bc30, SUC_lexicon_tags_21 },
    { 0x7fe0bc30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28c0bc33, SUC_lexicon_tags_26 },
    { 0x4380bc34, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce0bc37, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe40bc3b, SUC_lexicon_tags_7 },
    { 0xf7a0bc3c, SUC_lexicon_tags_1 },
    { 0x64c0bc3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c0bc40, SUC_lexicon_tags_43 },
    { 0x4ac0bc41, SUC_lexicon_tags_0 },
    { 0x6b00bc42, SUC_lexicon_tags_31 },
    { 0xc3c0bc41, SUC_lexicon_tags_31 },
    { 0xa140bc44, SUC_lexicon_tags_14 },
    { 0x7520bc43, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbcc0bc47, SUC_lexicon_tags_6 },
    { 0x5da0bc48, SUC_lexicon_tags_17 },
    { 0x7d20bc47, SUC_lexicon_tags_23 },
    { 0x88a0bc48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0bc4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf80bc51, SUC_lexicon_tags_25 },
    { 0x13e0bc51, SUC_lexicon_tags_9 },
    { 0xddc0bc52, SUC_lexicon_tags_25 },
    { 0xc640bc53, SUC_lexicon_tags_29 },
    { 0xd8c0bc55, SUC_lexicon_tags_8 },
    { 0x1560bc55, SUC_lexicon_tags_37 },
    { 0xd240bc57, SUC_lexicon_tags_11 },
    { 0x580bc58, SUC_lexicon_tags_6 },
    { 0x3880bc59, SUC_lexicon_tags_15 },
    { 0xae0bc58, SUC_lexicon_tags_115 },
    { 0x2ce0bc5b, SUC_lexicon_tags_68 },
    { 0x3b20bc5a, SUC_lexicon_tags_25 },
    { 0xaae0bc5b, SUC_lexicon_tags_17 },
    { 0x1020bc5e, SUC_lexicon_tags_62 },
    { 0x9080bc5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe40bc6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ec0bc6c, SUC_lexicon_tags_11 },
    { 0xf9a0bc6c, SUC_lexicon_tags_11 },
    { 0xd5c0bc6e, SUC_lexicon_tags_11 },
    { 0xc640bc6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee0bc73, SUC_lexicon_tags_17 },
    { 0xf2c0bc73, SUC_lexicon_tags_77 },
    { 0xd520bc75, SUC_lexicon_tags_33 },
    { 0x6f20bc75, SUC_lexicon_tags_15 },
    { 0xa40bc75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3080bc79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3960bc7e, SUC_lexicon_tags_2 },
    { 0x76a0bc7f, SUC_lexicon_tags_17 },
    { 0x7460bc80, SUC_lexicon_tags_47 },
    { 0x1c0bc81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c0bc86, SUC_lexicon_tags_10 },
    { 0xa6e0bc87, SUC_lexicon_tags_11 },
    { 0xf380bc88, SUC_lexicon_tags_92 },
    { 0x8920bc88, SUC_lexicon_tags_19 },
    { 0x35c0bc87, SUC_lexicon_tags_2 },
    { 0x6300bc8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e00bc8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb320bc8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e0bc94, SUC_lexicon_tags_28 },
    { 0xbc0bc95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a20bc9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9440bc9f, SUC_lexicon_tags_6 },
    { 0x47c0bca0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba0bca4, SUC_lexicon_tags_9 },
    { 0xc0a0bca4, SUC_lexicon_tags_12 },
    { 0x7140bca4, SUC_lexicon_tags_43 },
    { 0x3900bca5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb300bcaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e0bcaf, SUC_lexicon_tags_1 },
    { 0x7bc0bcaf, SUC_lexicon_tags_21 },
    { 0xdba0bcaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb440bcb4, SUC_lexicon_tags_25 },
    { 0xd400bcb4, SUC_lexicon_tags_25 },
    { 0x62a0bcb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a0bcbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd220bcbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a00bcc3, SUC_lexicon_tags_11 },
    { 0x7ea0bcc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc0bcc7, SUC_lexicon_tags_11 },
    { 0xca0bcc8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9200bccb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6f00bccd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf00bcd0, SUC_lexicon_tags_17 },
    { 0x6240bcd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57c0bcd3, SUC_lexicon_tags_15 },
    { 0xc6e0bcd3, SUC_lexicon_tags_15 },
    { 0xab20bcd4, SUC_lexicon_tags_39 },
    { 0x86e0bcd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e0bcdd, SUC_lexicon_tags_47 },
    { 0x8bc0bcde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a0bce1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83a0bce3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7960bce7, SUC_lexicon_tags_45 },
    { 0x9360bce8, SUC_lexicon_tags_25 },
    { 0x8b40bce9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3e40bceb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7040bced, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x9e0bcef, SUC_lexicon_tags_23 },
    { 0xbae0bcf0, SUC_lexicon_tags_30 },
    { 0x9340bcef, SUC_lexicon_tags_25 },
    { 0x7d40bcf2, SUC_lexicon_tags_17 },
    { 0x1e40bcf3, SUC_lexicon_tags_116 },
    { 0x4360bcf3, SUC_lexicon_tags_296 },
    { 0xe540bcf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8780bcfb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbd80bcfd, SUC_lexicon_tags_8 },
    { 0x95e0bcfd, SUC_lexicon_tags_23 },
    { 0xc6c0bcff, SUC_lexicon_tags_38 },
    { 0x5060bcff, SUC_lexicon_tags_63 },
    { 0xa5a0bcfe, SUC_lexicon_tags_49 },
    { 0xc9a0bd00, SUC_lexicon_tags_11 },
    { 0xd2e0bd03, SUC_lexicon_tags_47 },
    { 0xf060bcfd, SUC_lexicon_tags_28 },
    { 0xba20bd02, SUC_lexicon_tags_21 },
    { 0xa900bd04, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c40bd0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc460bd0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc0bd12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac0bd15, SUC_lexicon_tags_23 },
    { 0x9ba0bd16, SUC_lexicon_tags_32 },
    { 0x2720bd17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58a0bd19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee0bd1d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x62a0bd1f, SUC_lexicon_tags_26 },
    { 0x6dc0bd1f, SUC_lexicon_tags_33 },
    { 0x56c0bd21, SUC_lexicon_tags_37 },
    { 0xfc0bd22, SUC_lexicon_tags_7 },
    { 0x1860bd23, SUC_lexicon_tags_11 },
    { 0xeb60bd20, SUC_lexicon_tags_12 },
    { 0x3cc0bd25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e0bd2c, SUC_lexicon_tags_60 },
    { 0x700bd2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2c0bd2f, SUC_lexicon_tags_25 },
    { 0xddc0bd30, SUC_lexicon_tags_25 },
    { 0x1e20bd31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a0bd38, SUC_lexicon_tags_21 },
    { 0x2fe0bd39, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x31e0bd3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd00bd41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x16e0bd43, SUC_lexicon_tags_63 },
    { 0xafa0bd43, SUC_lexicon_tags_27 },
    { 0x8600bd43, SUC_lexicon_tags_49 },
    { 0xa720bd46, SUC_lexicon_tags_15 },
    { 0xbd60bd46, SUC_lexicon_tags_15 },
    { 0x3600bd44, SUC_lexicon_tags_49 },
    { 0xc020bd49, SUC_lexicon_tags_7 },
    { 0x6fe0bd48, SUC_lexicon_tags_15 },
    { 0x8440bd47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda0bd53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9620bd55, SUC_lexicon_tags_1 },
    { 0x7520bd56, SUC_lexicon_tags_1 },
    { 0xe460bd57, SUC_lexicon_tags_37 },
    { 0x7fc0bd58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a0bd5c, SUC_lexicon_tags_38 },
    { 0xf860bd5c, SUC_lexicon_tags_82 },
    { 0x1ac0bd5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc0bd68, SUC_lexicon_tags_1 },
    { 0x75c0bd68, SUC_lexicon_tags_15 },
    { 0xfb00bd68, SUC_lexicon_tags_29 },
    { 0xb580bd68, SUC_lexicon_tags_33 },
    { 0x2b60bd6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c60bd6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12c0bd71, SUC_lexicon_tags_61 },
    { 0x2ca0bd71, SUC_lexicon_tags_6 },
    { 0xdd80bd71, SUC_lexicon_tags_33 },
    { 0xdf80bd74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a0bd77, SUC_lexicon_tags_7 },
    { 0x1880bd78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf400bd7a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4260bd7c, SUC_lexicon_tags_25 },
    { 0xba0bd7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a60bd82, SUC_lexicon_tags_68 },
    { 0x4880bd82, SUC_lexicon_tags_27 },
    { 0x7540bd84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c0bd87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9880bd89, SUC_lexicon_tags_2 },
    { 0x7a20bd89, SUC_lexicon_tags_47 },
    { 0xbd40bd8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2a0bd8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe80bd8f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf880bd92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc0bd98, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e0bd9f, SUC_lexicon_tags_37 },
    { 0xce20bd9f, SUC_lexicon_tags_1565 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e0bda8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56e0bdaa, SUC_lexicon_tags_26 },
    { 0xba20bdab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb540bdae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa0bdb1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6160bdb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd7e0bdb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee40bdbc, SUC_lexicon_tags_11 },
    { 0x5e00bdbc, SUC_lexicon_tags_2 },
    { 0xaec0bdbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e0bdc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44c0bdc3, SUC_lexicon_tags_0 },
    { 0x2800bdc3, SUC_lexicon_tags_3 },
    { 0x4c80bdc5, SUC_lexicon_tags_60 },
    { 0xb7c0bdc4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x5740bdc9, SUC_lexicon_tags_48 },
    { 0x65e0bdca, SUC_lexicon_tags_14 },
    { 0xbdc0bdca, SUC_lexicon_tags_12 },
    { 0xa4c0bdcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc0bdcf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0bdd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca0bdd6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc0bddc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb080bde0, SUC_lexicon_tags_14 },
    { 0xa2a0bde1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e0bde4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x17c0bde6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa780bde9, SUC_lexicon_tags_66 },
    { 0xb6a0bde9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd140bdf1, SUC_lexicon_tags_7 },
    { 0xece0bdf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8140bdf8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xaae0bdfa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6080bdfc, SUC_lexicon_tags_27 },
    { 0xfb60bdfd, SUC_lexicon_tags_1 },
    { 0xbf20bdfe, SUC_lexicon_tags_63 },
    { 0xcb60bdff, SUC_lexicon_tags_130 },
    { 0x10c0be00, SUC_lexicon_tags_17 },
    { 0x76a0be01, SUC_lexicon_tags_37 },
    { 0xe340bdfe, SUC_lexicon_tags_19 },
    { 0x53c0be03, SUC_lexicon_tags_6 },
    { 0x8580be04, SUC_lexicon_tags_30 },
    { 0x2a0be03, SUC_lexicon_tags_27 },
    { 0x7800be06, SUC_lexicon_tags_17 },
    { 0xa040be05, SUC_lexicon_tags_12 },
    { 0xd180be00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc020be0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e0be0f, SUC_lexicon_tags_17 },
    { 0xe5a0be0f, SUC_lexicon_tags_11 },
    { 0x66e0be11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e0be17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f20be19, SUC_lexicon_tags_14 },
    { 0x7240be19, SUC_lexicon_tags_31 },
    { 0x7a20be1b, SUC_lexicon_tags_17 },
    { 0xada0be1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9880be1e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfb20be20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b00be22, SUC_lexicon_tags_37 },
    { 0xeb60be22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd20be27, SUC_lexicon_tags_45 },
    { 0x9a80be28, SUC_lexicon_tags_21 },
    { 0x6900be29, SUC_lexicon_tags_22 },
    { 0x6b80be2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc0be2f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x780be31, SUC_lexicon_tags_15 },
    { 0x46c0be32, SUC_lexicon_tags_27 },
    { 0x88a0be32, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc400be35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b20be39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5e80be3b, SUC_lexicon_tags_33 },
    { 0x9e80be3c, SUC_lexicon_tags_19 },
    { 0x6140be3d, SUC_lexicon_tags_7 },
    { 0x66a0be3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9ca0be40, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0be46, SUC_lexicon_tags_34 },
    { 0xa840be47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96c0be49, SUC_lexicon_tags_17 },
    { 0x9060be4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e0be4d, SUC_lexicon_tags_29 },
    { 0xc0be4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ee0be50, SUC_lexicon_tags_2 },
    { 0x9260be51, SUC_lexicon_tags_43 },
    { 0xd380be51, SUC_lexicon_tags_43 },
    { 0x2d60be53, SUC_lexicon_tags_28 },
    { 0x4360be54, SUC_lexicon_tags_17 },
    { 0x89c0be55, SUC_lexicon_tags_60 },
    { 0x6ce0be56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc880be59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda00be5c, SUC_lexicon_tags_38 },
    { 0x3fa0be5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d20be61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8940be63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4620be69, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da0be6e, SUC_lexicon_tags_17 },
    { 0x5200be6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c00be73, SUC_lexicon_tags_38 },
    { 0x1280be74, SUC_lexicon_tags_24 },
    { 0xa8c0be74, SUC_lexicon_tags_45 },
    { 0xc400be73, SUC_lexicon_tags_57 },
    { 0xab40be77, SUC_lexicon_tags_15 },
    { 0x8ea0be77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e0be7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0be83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca40be85, SUC_lexicon_tags_18 },
    { 0xb200be86, SUC_lexicon_tags_10 },
    { 0x69c0be87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7b60be89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580be8d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f80be91, SUC_lexicon_tags_1 },
    { 0xebc0be91, SUC_lexicon_tags_1 },
    { 0x8a0be92, SUC_lexicon_tags_26 },
    { 0xbd60be93, SUC_lexicon_tags_7 },
    { 0x8300be95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb00be9d, SUC_lexicon_tags_39 },
    { 0xfa60be9e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaa60bea0, SUC_lexicon_tags_7 },
    { 0x4120bea1, SUC_lexicon_tags_43 },
    { 0x9fa0bea0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd40bea5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc700bea8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e0beab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1280bead, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf120beb0, SUC_lexicon_tags_2 },
    { 0x3e00beb1, SUC_lexicon_tags_25 },
    { 0x21e0beb1, SUC_lexicon_tags_17 },
    { 0xb100beb2, SUC_lexicon_tags_10 },
    { 0xcb60beb4, SUC_lexicon_tags_12 },
    { 0xc300beb5, SUC_lexicon_tags_43 },
    { 0x2aa0beb3, SUC_lexicon_tags_45 },
    { 0xa340beb7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7520bebf, SUC_lexicon_tags_17 },
    { 0x9560bebf, SUC_lexicon_tags_55 },
    { 0xc840bebf, SUC_lexicon_tags_28 },
    { 0xfac0bec0, SUC_lexicon_tags_16 },
    { 0x7340bec0, SUC_lexicon_tags_29 },
    { 0x6e40bec4, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x84c0bec6, SUC_lexicon_tags_1 },
    { 0xc060bec6, SUC_lexicon_tags_38 },
    { 0x7480bec8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce0becb, SUC_lexicon_tags_25 },
    { 0x80a0becb, SUC_lexicon_tags_12 },
    { 0x19a0becd, SUC_lexicon_tags_7 },
    { 0xbba0becd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa940bed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9640bedb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe480bedd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e0bee1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca0beea, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0xc560beec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc440beee, SUC_lexicon_tags_17 },
    { 0x4920beee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa180bef2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4c0bef4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6960befd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd680beff, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a40bf09, SUC_lexicon_tags_82 },
    { 0xab00bf0a, SUC_lexicon_tags_19 },
    { 0x2120bf0b, SUC_lexicon_tags_10 },
    { 0xa420bf0c, SUC_lexicon_tags_17 },
    { 0x72a0bf0c, SUC_lexicon_tags_1 },
    { 0x4f60bf0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9840bf12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40bf1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e40bf20, SUC_lexicon_tags_12 },
    { 0x3380bf21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3080bf29, SUC_lexicon_tags_26 },
    { 0x8900bf2a, SUC_lexicon_tags_0 },
    { 0x23c0bf2b, SUC_lexicon_tags_60 },
    { 0xb080bf2a, SUC_lexicon_tags_118 },
    { 0x4080bf2c, SUC_lexicon_tags_29 },
    { 0x6200bf2e, SUC_lexicon_tags_7 },
    { 0x6400bf2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3660bf33, SUC_lexicon_tags_25 },
    { 0x7ba0bf34, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7560bf36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5360bf38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e0bf3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f60bf43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc60bf47, SUC_lexicon_tags_3 },
    { 0x4d80bf48, SUC_lexicon_tags_33 },
    { 0xdb20bf47, SUC_lexicon_tags_15 },
    { 0x2500bf4a, SUC_lexicon_tags_65 },
    { 0x8680bf4a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5a80bf4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1420bf51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3460bf56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc780bf5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48a0bf5d, SUC_lexicon_tags_29 },
    { 0xd580bf5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1560bf61, SUC_lexicon_tags_7 },
    { 0xd7e0bf62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf000bf65, SUC_lexicon_tags_25 },
    { 0xe060bf66, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e20bf6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c0bf71, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9880bf73, SUC_lexicon_tags_33 },
    { 0xade0bf74, SUC_lexicon_tags_17 },
    { 0x4120bf75, SUC_lexicon_tags_14 },
    { 0x3820bf73, SUC_lexicon_tags_11 },
    { 0x9d00bf77, SUC_lexicon_tags_17 },
    { 0x4840bf78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff00bf7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d60bf7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6c0bf7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a0bf81, SUC_lexicon_tags_60 },
    { 0x5cc0bf81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9460bf84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a0bf89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf520bf98, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae0bf9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde0bf9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1140bfa1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e0bfa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e80bfa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9660bfa9, SUC_lexicon_tags_87 },
    { 0xf660bfaa, SUC_lexicon_tags_12 },
    { 0x8e80bfab, SUC_lexicon_tags_2 },
    { 0xec20bfab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec0bfb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5700bfb8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe8a0bfba, SUC_lexicon_tags_2 },
    { 0xfc20bfbb, SUC_lexicon_tags_24 },
    { 0xee40bfbb, SUC_lexicon_tags_11 },
    { 0x2120bfbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6120bfc0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d40bfc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e0bfc7, SUC_lexicon_tags_37 },
    { 0x9b00bfc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e60bfcb, SUC_lexicon_tags_17 },
    { 0x7dc0bfcc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b20bfd0, SUC_lexicon_tags_11 },
    { 0xf1c0bfd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a00bfd4, SUC_lexicon_tags_27 },
    { 0xc9c0bfd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc300bfd9, SUC_lexicon_tags_12 },
    { 0x6300bfda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e0bfe2, SUC_lexicon_tags_14 },
    { 0xd040bfe3, SUC_lexicon_tags_17 },
    { 0x3360bfe4, SUC_lexicon_tags_0 },
    { 0x8520bfe3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e80bfe8, SUC_lexicon_tags_11 },
    { 0x6360bfe9, SUC_lexicon_tags_1 },
    { 0x9060bfea, SUC_lexicon_tags_37 },
    { 0x3620bfeb, SUC_lexicon_tags_7 },
    { 0x7980bfeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1260bfee, SUC_lexicon_tags_11 },
    { 0x4d60bfef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4600bff2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa460c001, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6800c005, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b60c008, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a0c00c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50c0c00e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4580c012, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce60c017, SUC_lexicon_tags_17 },
    { 0x1ae0c018, SUC_lexicon_tags_17 },
    { 0x3c60c018, SUC_lexicon_tags_29 },
    { 0x5fa0c018, SUC_lexicon_tags_2 },
    { 0x4fa0c01b, SUC_lexicon_tags_63 },
    { 0x620c01c, SUC_lexicon_tags_53 },
    { 0x5440c01d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb60c027, SUC_lexicon_tags_14 },
    { 0x82e0c028, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfb40c02a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6520c02c, SUC_lexicon_tags_25 },
    { 0x5700c02d, SUC_lexicon_tags_38 },
    { 0xd220c02e, SUC_lexicon_tags_19 },
    { 0x4ec0c02f, SUC_lexicon_tags_12 },
    { 0xc8c0c02f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0c034, SUC_lexicon_tags_11 },
    { 0x52a0c035, SUC_lexicon_tags_27 },
    { 0xb060c036, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c0c03a, SUC_lexicon_tags_38 },
    { 0x8ac0c03b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c0c03f, SUC_lexicon_tags_37 },
    { 0x5ca0c040, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de0c043, SUC_lexicon_tags_65 },
    { 0x81a0c043, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100c04a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e0c04d, SUC_lexicon_tags_37 },
    { 0x9ae0c04e, SUC_lexicon_tags_2 },
    { 0x7c20c04d, SUC_lexicon_tags_48 },
    { 0x51e0c04e, SUC_lexicon_tags_16 },
    { 0xc500c051, SUC_lexicon_tags_9 },
    { 0x5e00c052, SUC_lexicon_tags_8 },
    { 0xf460c052, SUC_lexicon_tags_49 },
    { 0xad00c054, SUC_lexicon_tags_23 },
    { 0xbc20c04e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6280c057, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3000c05a, SUC_lexicon_tags_9 },
    { 0x2f00c05b, SUC_lexicon_tags_10 },
    { 0xe520c05c, SUC_lexicon_tags_15 },
    { 0xfbc0c05d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2200c05f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8360c061, SUC_lexicon_tags_11 },
    { 0xc140c061, SUC_lexicon_tags_121 },
    { 0x80e0c062, SUC_lexicon_tags_37 },
    { 0xe9c0c061, SUC_lexicon_tags_1 },
    { 0xab20c065, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60c06a, SUC_lexicon_tags_12 },
    { 0x3f40c06a, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c0c06e, SUC_lexicon_tags_37 },
    { 0xb7a0c06f, SUC_lexicon_tags_79 },
    { 0xce0c070, SUC_lexicon_tags_17 },
    { 0xa0a0c071, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2980c075, SUC_lexicon_tags_53 },
    { 0xa0c0c076, SUC_lexicon_tags_31 },
    { 0xd740c077, SUC_lexicon_tags_11 },
    { 0xea40c078, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec40c07b, SUC_lexicon_tags_47 },
    { 0xd320c07c, SUC_lexicon_tags_23 },
    { 0x77e0c07c, SUC_lexicon_tags_17 },
    { 0x8100c07e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c0c085, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9200c08d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc940c090, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x28c0c092, SUC_lexicon_tags_2 },
    { 0xd980c092, SUC_lexicon_tags_11 },
    { 0xaa00c094, SUC_lexicon_tags_40 },
    { 0x2fc0c094, SUC_lexicon_tags_12 },
    { 0xebe0c094, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf820c098, SUC_lexicon_tags_25 },
    { 0x9760c099, SUC_lexicon_tags_39 },
    { 0x85e0c098, SUC_lexicon_tags_0 },
    { 0xa8e0c09b, SUC_lexicon_tags_5 },
    { 0x4f80c09c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2880c09f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7480c0a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a0c0a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc0c0ac, SUC_lexicon_tags_70 },
    { 0x1e0c0ac, SUC_lexicon_tags_25 },
    { 0xa640c0ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2260c0b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x14a0c0b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3660c0b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa20c0ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1be0c0bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe0c0c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd320c0c2, SUC_lexicon_tags_17 },
    { 0xf0a0c0c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5800c0c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c0c0c9, SUC_lexicon_tags_11 },
    { 0x7720c0ca, SUC_lexicon_tags_121 },
    { 0x8540c0ca, SUC_lexicon_tags_84 },
    { 0x7340c0cc, SUC_lexicon_tags_47 },
    { 0x9e20c0cd, SUC_lexicon_tags_25 },
    { 0x7e40c0ce, SUC_lexicon_tags_12 },
    { 0x160c0cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a0c0d4, SUC_lexicon_tags_1 },
    { 0xd40c0d4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c0c0dc, SUC_lexicon_tags_7 },
    { 0x1e80c0dd, SUC_lexicon_tags_2 },
    { 0x6b20c0de, SUC_lexicon_tags_1 },
    { 0xaae0c0df, SUC_lexicon_tags_42 },
    { 0x90a0c0dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b20c0e2, SUC_lexicon_tags_1 },
    { 0x31c0c0e2, SUC_lexicon_tags_33 },
    { 0x10a0c0e4, SUC_lexicon_tags_47 },
    { 0x8de0c0e4, SUC_lexicon_tags_61 },
    { 0xe4e0c0e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70e0c0e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc0c0eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a0c0ef, SUC_lexicon_tags_30 },
    { 0xf3c0c0f0, SUC_lexicon_tags_0 },
    { 0xba80c0f1, SUC_lexicon_tags_29 },
    { 0x3de0c0f2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5880c0f5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7080c0f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34c0c0f9, SUC_lexicon_tags_28 },
    { 0x83c0c0fa, SUC_lexicon_tags_82 },
    { 0xc700c0fb, SUC_lexicon_tags_15 },
    { 0x99a0c0fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73a0c0fe, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x4400c101, SUC_lexicon_tags_33 },
    { 0x8ee0c102, SUC_lexicon_tags_48 },
    { 0x66e0c102, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x16e0c105, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7560c108, SUC_lexicon_tags_69 },
    { 0xfb00c108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b00c10b, SUC_lexicon_tags_11 },
    { 0x8820c10c, SUC_lexicon_tags_14 },
    { 0x5400c10c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa120c110, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2520c112, SUC_lexicon_tags_17 },
    { 0x4940c113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c40c118, SUC_lexicon_tags_43 },
    { 0xce60c118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55c0c11b, SUC_lexicon_tags_60 },
    { 0xae00c11b, SUC_lexicon_tags_4 },
    { 0xb600c11d, SUC_lexicon_tags_23 },
    { 0x7120c11d, SUC_lexicon_tags_8 },
    { 0xdc60c11f, SUC_lexicon_tags_7 },
    { 0xc560c11d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce0c124, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc0c128, SUC_lexicon_tags_1 },
    { 0xc2e0c129, SUC_lexicon_tags_48 },
    { 0x8cc0c128, SUC_lexicon_tags_4 },
    { 0x3040c128, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe0c134, SUC_lexicon_tags_25 },
    { 0x5240c134, SUC_lexicon_tags_1 },
    { 0x7de0c135, SUC_lexicon_tags_29 },
    { 0xa160c135, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2480c139, SUC_lexicon_tags_11 },
    { 0xcb80c13a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4660c13d, SUC_lexicon_tags_9 },
    { 0xb280c13e, SUC_lexicon_tags_29 },
    { 0xeee0c13e, SUC_lexicon_tags_23 },
    { 0x3680c140, SUC_lexicon_tags_11 },
    { 0xdd00c13d, SUC_lexicon_tags_19 },
    { 0xf3e0c140, SUC_lexicon_tags_0 },
    { 0x4240c143, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9fa0c145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc0c14d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe740c14f, SUC_lexicon_tags_12 },
    { 0x5a20c150, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3500c152, SUC_lexicon_tags_49 },
    { 0x7b00c153, SUC_lexicon_tags_11 },
    { 0x62a0c154, SUC_lexicon_tags_49 },
    { 0x6960c153, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f60c15d, SUC_lexicon_tags_38 },
    { 0xfc40c15e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a0c167, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a20c172, SUC_lexicon_tags_25 },
    { 0x2f40c173, SUC_lexicon_tags_33 },
    { 0x7000c174, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fa0c176, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5a80c178, SUC_lexicon_tags_17 },
    { 0x6fa0c178, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa840c17c, SUC_lexicon_tags_56 },
    { 0x8d40c17d, SUC_lexicon_tags_150 },
    { 0x4ea0c17e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a00c181, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2da0c183, SUC_lexicon_tags_12 },
    { 0x9b40c184, SUC_lexicon_tags_2 },
    { 0xfb40c184, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x4920c187, SUC_lexicon_tags_21 },
    { 0xfc80c188, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3080c18a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7e0c18c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8920c190, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2800c192, SUC_lexicon_tags_27 },
    { 0x1f60c193, SUC_lexicon_tags_56 },
    { 0x7d00c194, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8160c198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bc0c19a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8540c19c, SUC_lexicon_tags_61 },
    { 0x4fe0c19c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa0c1a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3600c1a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae0c1ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c0c1b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa820c1b4, SUC_lexicon_tags_1 },
    { 0xcb20c1b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2420c1b7, SUC_lexicon_tags_113 },
    { 0x80e0c1b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7be0c1ba, SUC_lexicon_tags_25 },
    { 0xfd00c1ba, SUC_lexicon_tags_26 },
    { 0xea80c1bc, SUC_lexicon_tags_9 },
    { 0x8340c1bc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x260c1bf, SUC_lexicon_tags_47 },
    { 0x58e0c1c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a00c1c2, SUC_lexicon_tags_15 },
    { 0x1420c1c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2080c1c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b80c1c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3e0c1cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bc0c1cd, SUC_lexicon_tags_29 },
    { 0x92e0c1ce, SUC_lexicon_tags_55 },
    { 0x2500c1cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a80c1d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6100c1d7, SUC_lexicon_tags_27 },
    { 0xa9a0c1d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x280c1db, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe4e0c1dd, SUC_lexicon_tags_33 },
    { 0xa780c1dd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c60c1e5, SUC_lexicon_tags_114 },
    { 0x4500c1e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd300c1e9, SUC_lexicon_tags_82 },
    { 0x61a0c1ea, SUC_lexicon_tags_47 },
    { 0xd760c1ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a0c1ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1460c1f4, SUC_lexicon_tags_7 },
    { 0x4c60c1f5, SUC_lexicon_tags_25 },
    { 0x6240c1f5, SUC_lexicon_tags_63 },
    { 0x48a0c1f7, SUC_lexicon_tags_32 },
    { 0x5780c1f7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb520c1fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8980c1fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd620c201, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2be0c203, SUC_lexicon_tags_17 },
    { 0x5b60c204, SUC_lexicon_tags_25 },
    { 0x8480c205, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0c20b, SUC_lexicon_tags_12 },
    { 0x8140c20c, SUC_lexicon_tags_60 },
    { 0x7ea0c20b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8980c212, SUC_lexicon_tags_17 },
    { 0xcb40c213, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc00c215, SUC_lexicon_tags_38 },
    { 0x9c20c216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42a0c218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda0c21b, SUC_lexicon_tags_7 },
    { 0x8de0c21c, SUC_lexicon_tags_15 },
    { 0x8e0c21d, SUC_lexicon_tags_12 },
    { 0x9ba0c21c, SUC_lexicon_tags_17 },
    { 0x77c0c21f, SUC_lexicon_tags_47 },
    { 0xd540c220, SUC_lexicon_tags_31 },
    { 0x5e80c221, SUC_lexicon_tags_47 },
    { 0x7280c220, SUC_lexicon_tags_47 },
    { 0x3e0c223, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f80c225, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e0c228, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb740c22b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6de0c22d, SUC_lexicon_tags_47 },
    { 0xf120c22e, SUC_lexicon_tags_0 },
    { 0xdec0c22e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4380c234, SUC_lexicon_tags_19 },
    { 0x9220c235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea20c23a, SUC_lexicon_tags_7 },
    { 0x1fe0c23a, SUC_lexicon_tags_60 },
    { 0xc4c0c23c, SUC_lexicon_tags_15 },
    { 0x4960c23d, SUC_lexicon_tags_17 },
    { 0x28a0c23e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa0c241, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x340c244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe680c246, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf320c24b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c00c24f, SUC_lexicon_tags_31 },
    { 0xa620c250, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6560c253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a40c257, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xee00c259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8c0c25b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ce0c25d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe0c260, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4620c262, SUC_lexicon_tags_19 },
    { 0x7a0c263, SUC_lexicon_tags_55 },
    { 0x3820c264, SUC_lexicon_tags_19 },
    { 0xefa0c265, SUC_lexicon_tags_15 },
    { 0x9620c266, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c0c26b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9180c271, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7120c274, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30e0c276, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f60c278, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca20c27c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3fa0c27e, SUC_lexicon_tags_27 },
    { 0x140c27f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be0c283, SUC_lexicon_tags_37 },
    { 0x3fe0c284, SUC_lexicon_tags_47 },
    { 0xc00c284, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec0c288, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e0c28b, SUC_lexicon_tags_11 },
    { 0x99e0c28c, SUC_lexicon_tags_25 },
    { 0x97c0c28c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62c0c28f, SUC_lexicon_tags_13 },
    { 0xd3a0c28f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e60c293, SUC_lexicon_tags_45 },
    { 0xb660c294, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb80c296, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c40c298, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x87a0c29a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b20c29c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d80c2a0, SUC_lexicon_tags_36 },
    { 0xc1e0c2a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b60c2a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce0c2a7, SUC_lexicon_tags_25 },
    { 0x7c60c2a8, SUC_lexicon_tags_2 },
    { 0x17c0c2a9, SUC_lexicon_tags_49 },
    { 0xc660c2aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd40c2ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4340c2b1, SUC_lexicon_tags_17 },
    { 0x4040c2b2, SUC_lexicon_tags_38 },
    { 0xf620c2b1, SUC_lexicon_tags_38 },
    { 0xfe40c2b4, SUC_lexicon_tags_14 },
    { 0xe9a0c2b5, SUC_lexicon_tags_19 },
    { 0x3680c2b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e00c2c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f60c2ca, SUC_lexicon_tags_0 },
    { 0xf460c2cb, SUC_lexicon_tags_17 },
    { 0xf5e0c2cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3560c2d2, SUC_lexicon_tags_27 },
    { 0x6320c2d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x64a0c2d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbc0c2d7, SUC_lexicon_tags_29 },
    { 0x3f60c2d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf60c2de, SUC_lexicon_tags_49 },
    { 0x8400c2de, SUC_lexicon_tags_35 },
    { 0xff80c2df, SUC_lexicon_tags_17 },
    { 0xce00c2e1, SUC_lexicon_tags_48 },
    { 0xe700c2e2, SUC_lexicon_tags_60 },
    { 0x2ac0c2e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c60c2e6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6920c2e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9200c2ed, SUC_lexicon_tags_11 },
    { 0xe500c2ee, SUC_lexicon_tags_7 },
    { 0xadc0c2ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c0c2f4, SUC_lexicon_tags_15 },
    { 0x7220c2f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c0c303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf580c305, SUC_lexicon_tags_6 },
    { 0xfae0c306, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5320c30c, SUC_lexicon_tags_27 },
    { 0xea0c30c, SUC_lexicon_tags_15 },
    { 0x3e60c30c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220c312, SUC_lexicon_tags_12 },
    { 0xb340c312, SUC_lexicon_tags_26 },
    { 0x2a0c314, SUC_lexicon_tags_25 },
    { 0x4120c315, SUC_lexicon_tags_1 },
    { 0x6120c312, SUC_lexicon_tags_18 },
    { 0xc800c313, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd20c319, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c20c31c, SUC_lexicon_tags_60 },
    { 0x31e0c31d, SUC_lexicon_tags_25 },
    { 0x5f40c31e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa480c322, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd60c326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5680c328, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeca0c32a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8760c32c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9020c32e, SUC_lexicon_tags_25 },
    { 0x78c0c32f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33e0c331, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe300c335, SUC_lexicon_tags_2 },
    { 0xc3a0c335, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e0c33d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xbbe0c33f, SUC_lexicon_tags_11 },
    { 0x1320c340, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20c344, SUC_lexicon_tags_77 },
    { 0x2380c345, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba0c34c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8640c351, SUC_lexicon_tags_11 },
    { 0x1bc0c351, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7580c354, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e0c357, SUC_lexicon_tags_43 },
    { 0x8f40c358, SUC_lexicon_tags_11 },
    { 0x8ee0c358, SUC_lexicon_tags_47 },
    { 0xe400c35a, SUC_lexicon_tags_17 },
    { 0xf520c35b, SUC_lexicon_tags_37 },
    { 0x6ec0c35a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a0c360, SUC_lexicon_tags_1 },
    { 0x95c0c360, SUC_lexicon_tags_37 },
    { 0x3500c362, SUC_lexicon_tags_38 },
    { 0x5ea0c363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a60c36b, SUC_lexicon_tags_15 },
    { 0xae00c36b, SUC_lexicon_tags_6 },
    { 0x68c0c36d, SUC_lexicon_tags_7 },
    { 0xc36e, SUC_lexicon_tags_26 },
    { 0x1fe0c36f, SUC_lexicon_tags_48 },
    { 0xbe40c36f, SUC_lexicon_tags_17 },
    { 0xb640c371, SUC_lexicon_tags_25 },
    { 0x54e0c371, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb920c377, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb080c379, SUC_lexicon_tags_14 },
    { 0x1140c37a, SUC_lexicon_tags_11 },
    { 0x98a0c37b, SUC_lexicon_tags_37 },
    { 0xc8c0c37a, SUC_lexicon_tags_25 },
    { 0xd4a0c379, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3780c382, SUC_lexicon_tags_37 },
    { 0x7780c382, SUC_lexicon_tags_31 },
    { 0xec60c384, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x41e0c386, SUC_lexicon_tags_17 },
    { 0xcd80c386, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7dc0c389, SUC_lexicon_tags_1 },
    { 0x58a0c38a, SUC_lexicon_tags_47 },
    { 0xec20c38b, SUC_lexicon_tags_17 },
    { 0xd1c0c38c, SUC_lexicon_tags_2 },
    { 0xfae0c38b, SUC_lexicon_tags_7 },
    { 0xe80c38e, SUC_lexicon_tags_56 },
    { 0x37c0c38b, SUC_lexicon_tags_29 },
    { 0x53c0c390, SUC_lexicon_tags_17 },
    { 0x55e0c389, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1660c397, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7540c399, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0c39c, SUC_lexicon_tags_36 },
    { 0xd880c39d, SUC_lexicon_tags_7 },
    { 0xe040c39c, SUC_lexicon_tags_15 },
    { 0xd960c39f, SUC_lexicon_tags_10 },
    { 0xb380c3a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfca0c3a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd540c3a5, SUC_lexicon_tags_37 },
    { 0xc60c3a5, SUC_lexicon_tags_136 },
    { 0x7f40c3a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2700c3ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e00c3ae, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe0c3b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e0c3bd, SUC_lexicon_tags_25 },
    { 0x7a40c3be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc0c3c1, SUC_lexicon_tags_7 },
    { 0xb440c3c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1020c3c5, SUC_lexicon_tags_113 },
    { 0x96c0c3c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc0c3cc, SUC_lexicon_tags_33 },
    { 0x30c0c3cc, SUC_lexicon_tags_10 },
    { 0xa960c3ce, SUC_lexicon_tags_21 },
    { 0xc400c3ce, SUC_lexicon_tags_61 },
    { 0x1560c3cf, SUC_lexicon_tags_54 },
    { 0x2320c3ce, SUC_lexicon_tags_2 },
    { 0xa680c3d2, SUC_lexicon_tags_2 },
    { 0xd40c3d1, SUC_lexicon_tags_63 },
    { 0x1760c3d2, SUC_lexicon_tags_120 },
    { 0xcb00c3d5, SUC_lexicon_tags_14 },
    { 0xdc60c3d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc200c3d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4420c3db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8140c3dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa0c3e0, SUC_lexicon_tags_30 },
    { 0x5380c3e0, SUC_lexicon_tags_54 },
    { 0xf6c0c3e2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0c3e7, SUC_lexicon_tags_79 },
    { 0xd080c3e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b00c3ee, SUC_lexicon_tags_19 },
    { 0xaa20c3ef, SUC_lexicon_tags_25 },
    { 0x6680c3f0, SUC_lexicon_tags_1 },
    { 0x27e0c3f0, SUC_lexicon_tags_14 },
    { 0xd500c3ee, SUC_lexicon_tags_25 },
    { 0xf9e0c3f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb180c3f7, SUC_lexicon_tags_35 },
    { 0x5520c3f8, SUC_lexicon_tags_21 },
    { 0x5a80c3f8, SUC_lexicon_tags_48 },
    { 0x5440c3fa, SUC_lexicon_tags_47 },
    { 0x4c80c3fb, SUC_lexicon_tags_87 },
    { 0xe60c3fb, SUC_lexicon_tags_28 },
    { 0x6de0c3f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d00c401, SUC_lexicon_tags_2 },
    { 0x6300c401, SUC_lexicon_tags_15 },
    { 0x55c0c403, SUC_lexicon_tags_25 },
    { 0x8b80c403, SUC_lexicon_tags_37 },
    { 0x49a0c402, SUC_lexicon_tags_66 },
    { 0x6e60c406, SUC_lexicon_tags_19 },
    { 0x3a80c401, SUC_lexicon_tags_32 },
    { 0x5160c408, SUC_lexicon_tags_0 },
    { 0x9a0c407, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd940c40b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc0c40f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0c413, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e00c417, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf040c419, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe760c41d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a0c420, SUC_lexicon_tags_37 },
    { 0xc040c421, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3780c425, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa820c429, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a0c42c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c0c434, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b00c437, SUC_lexicon_tags_1 },
    { 0x92e0c438, SUC_lexicon_tags_47 },
    { 0xd540c439, SUC_lexicon_tags_14 },
    { 0x22c0c43a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8e20c43c, SUC_lexicon_tags_15 },
    { 0xb9a0c43d, SUC_lexicon_tags_1 },
    { 0x8b00c43e, SUC_lexicon_tags_1 },
    { 0x16e0c43f, SUC_lexicon_tags_19 },
    { 0x1000c440, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf640c448, SUC_lexicon_tags_70 },
    { 0xf980c449, SUC_lexicon_tags_0 },
    { 0x6b00c44a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec60c451, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd7e0c453, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3940c458, SUC_lexicon_tags_12 },
    { 0x6c20c459, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f80c45b, SUC_lexicon_tags_25 },
    { 0xf9c0c45c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b00c45f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3980c462, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8c0c464, SUC_lexicon_tags_0 },
    { 0xc40c465, SUC_lexicon_tags_55 },
    { 0x1520c466, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd200c468, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc0e0c46a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc0c46d, SUC_lexicon_tags_46 },
    { 0xae00c46d, SUC_lexicon_tags_170 },
    { 0xb400c46f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc220c471, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f20c475, SUC_lexicon_tags_3 },
    { 0x460c476, SUC_lexicon_tags_34 },
    { 0x6900c476, SUC_lexicon_tags_49 },
    { 0xcb80c478, SUC_lexicon_tags_17 },
    { 0x7300c479, SUC_lexicon_tags_63 },
    { 0x9c60c479, SUC_lexicon_tags_7 },
    { 0x77e0c47b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf900c47d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1fe0c47f, SUC_lexicon_tags_31 },
    { 0x4b20c47f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6040c482, SUC_lexicon_tags_11 },
    { 0xfa00c483, SUC_lexicon_tags_27 },
    { 0xe480c483, SUC_lexicon_tags_45 },
    { 0x4ee0c485, SUC_lexicon_tags_38 },
    { 0xfb40c485, SUC_lexicon_tags_15 },
    { 0x3240c486, SUC_lexicon_tags_1 },
    { 0xfe20c488, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31c0c48a, SUC_lexicon_tags_2 },
    { 0xc0c0c48b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4840c48d, SUC_lexicon_tags_11 },
    { 0x8e40c48d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e0c498, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a0c49d, SUC_lexicon_tags_53 },
    { 0x6fa0c49e, SUC_lexicon_tags_0 },
    { 0x6540c49f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39e0c4a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d00c4a6, SUC_lexicon_tags_14 },
    { 0x1ce0c4a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3000c4a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ae0c4ab, SUC_lexicon_tags_14 },
    { 0xe160c4ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1bc0c4ae, SUC_lexicon_tags_37 },
    { 0xcd80c4af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f40c4b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25c0c4b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc260c4b5, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a00c4b8, SUC_lexicon_tags_38 },
    { 0x1cc0c4b8, SUC_lexicon_tags_31 },
    { 0x2340c4b8, SUC_lexicon_tags_19 },
    { 0xaea0c4b9, SUC_lexicon_tags_17 },
    { 0x8b60c4b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0c4c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c0c4c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d60c4ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c0c4d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x980c4d4, SUC_lexicon_tags_17 },
    { 0x14c0c4d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7260c4d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb20c4dd, SUC_lexicon_tags_19 },
    { 0x4c60c4de, SUC_lexicon_tags_3 },
    { 0xbd40c4de, SUC_lexicon_tags_33 },
    { 0xefc0c4e0, SUC_lexicon_tags_8 },
    { 0x3100c4e1, SUC_lexicon_tags_2 },
    { 0x7900c4e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc60c4e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc0c4eb, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a0c4f0, SUC_lexicon_tags_27 },
    { 0xf480c4f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60c4f4, SUC_lexicon_tags_11 },
    { 0x5000c4f4, SUC_lexicon_tags_18 },
    { 0xf2c0c4f4, SUC_lexicon_tags_28 },
    { 0x8560c4f6, SUC_lexicon_tags_37 },
    { 0x7e0c4f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd820c4fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b00c4fe, SUC_lexicon_tags_0 },
    { 0xf140c4ff, SUC_lexicon_tags_28 },
    { 0x9560c4ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b00c50a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f80c514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c0c518, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620c51c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c0c524, SUC_lexicon_tags_33 },
    { 0x9220c525, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8540c527, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1100c529, SUC_lexicon_tags_2 },
    { 0xf2c0c52a, SUC_lexicon_tags_63 },
    { 0x3180c52b, SUC_lexicon_tags_1 },
    { 0x2180c529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e20c531, SUC_lexicon_tags_17 },
    { 0x8d80c532, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c0c535, SUC_lexicon_tags_0 },
    { 0x38a0c535, SUC_lexicon_tags_37 },
    { 0xf1a0c536, SUC_lexicon_tags_25 },
    { 0x6880c538, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e0c53d, SUC_lexicon_tags_7 },
    { 0x5300c53e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c0c541, SUC_lexicon_tags_11 },
    { 0x40e0c542, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc960c544, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe9e0c546, SUC_lexicon_tags_15 },
    { 0xc300c547, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1280c549, SUC_lexicon_tags_37 },
    { 0xfb20c54a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d00c54c, SUC_lexicon_tags_1 },
    { 0x41c0c54d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ae0c54f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1180c552, SUC_lexicon_tags_17 },
    { 0x9180c553, SUC_lexicon_tags_63 },
    { 0x5500c554, SUC_lexicon_tags_11 },
    { 0xf7a0c554, SUC_lexicon_tags_2 },
    { 0xa500c554, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd580c561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4100c566, SUC_lexicon_tags_15 },
    { 0x5ce0c567, SUC_lexicon_tags_27 },
    { 0x8ee0c566, SUC_lexicon_tags_37 },
    { 0xff00c569, SUC_lexicon_tags_12 },
    { 0xe9c0c569, SUC_lexicon_tags_12 },
    { 0xe920c567, SUC_lexicon_tags_138 },
    { 0xf480c56c, SUC_lexicon_tags_17 },
    { 0xbe20c56d, SUC_lexicon_tags_11 },
    { 0x5220c56e, SUC_lexicon_tags_26 },
    { 0xc840c56f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc0c571, SUC_lexicon_tags_19 },
    { 0x4c80c571, SUC_lexicon_tags_37 },
    { 0xbf40c573, SUC_lexicon_tags_38 },
    { 0xe460c571, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0c578, SUC_lexicon_tags_10 },
    { 0xb2c0c579, SUC_lexicon_tags_8 },
    { 0x3060c579, SUC_lexicon_tags_7 },
    { 0x1e40c579, SUC_lexicon_tags_7 },
    { 0xeba0c57c, SUC_lexicon_tags_11 },
    { 0x29c0c57d, SUC_lexicon_tags_16 },
    { 0x83a0c57c, SUC_lexicon_tags_47 },
    { 0x90e0c57c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5080c581, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b60c585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa80c588, SUC_lexicon_tags_7 },
    { 0xafa0c588, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c0c58d, SUC_lexicon_tags_47 },
    { 0x3700c58e, SUC_lexicon_tags_27 },
    { 0x4dc0c58f, SUC_lexicon_tags_11 },
    { 0x4ea0c590, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1620c593, SUC_lexicon_tags_37 },
    { 0xcd80c593, SUC_lexicon_tags_32 },
    { 0xa560c595, SUC_lexicon_tags_65 },
    { 0x1540c596, SUC_lexicon_tags_1 },
    { 0x97e0c597, SUC_lexicon_tags_20 },
    { 0x640c595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f40c59b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24a0c59d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c0c5a2, SUC_lexicon_tags_23 },
    { 0x1900c5a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf880c5aa, SUC_lexicon_tags_1 },
    { 0x53c0c5ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b20c5b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb600c5bf, SUC_lexicon_tags_34 },
    { 0x8700c5bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4180c5c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7140c5c9, SUC_lexicon_tags_57 },
    { 0xb60c5ca, SUC_lexicon_tags_17 },
    { 0x2d40c5cb, SUC_lexicon_tags_1 },
    { 0x8380c5cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x18e0c5ce, SUC_lexicon_tags_63 },
    { 0x85a0c5ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd80c5d1, SUC_lexicon_tags_8 },
    { 0xe0e0c5d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1280c5d6, SUC_lexicon_tags_1 },
    { 0xd5e0c5d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd60c5d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e0c5df, SUC_lexicon_tags_25 },
    { 0xcfc0c5df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c60c5e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20e0c5e6, SUC_lexicon_tags_12 },
    { 0xb880c5e6, SUC_lexicon_tags_49 },
    { 0xd140c5e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc60c5ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a0c5f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8c0c5f5, SUC_lexicon_tags_7 },
    { 0x77a0c5f5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4560c5f9, SUC_lexicon_tags_17 },
    { 0xfee0c5f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe480c5fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaba0c5ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x780c602, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1900c604, SUC_lexicon_tags_43 },
    { 0x5640c604, SUC_lexicon_tags_15 },
    { 0x5d20c605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b80c60c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe0c60f, SUC_lexicon_tags_7 },
    { 0x75e0c60f, SUC_lexicon_tags_6 },
    { 0x3760c60f, SUC_lexicon_tags_63 },
    { 0xbc80c611, SUC_lexicon_tags_26 },
    { 0xccc0c610, SUC_lexicon_tags_45 },
    { 0xa240c614, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e0c621, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e0c629, SUC_lexicon_tags_9 },
    { 0x29c0c629, SUC_lexicon_tags_23 },
    { 0x940c62b, SUC_lexicon_tags_11 },
    { 0xcd00c62c, SUC_lexicon_tags_25 },
    { 0x5280c629, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2880c633, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6820c639, SUC_lexicon_tags_33 },
    { 0x5ae0c63a, SUC_lexicon_tags_34 },
    { 0xdbe0c63b, SUC_lexicon_tags_4 },
    { 0xf860c63b, SUC_lexicon_tags_46 },
    { 0x9a00c63d, SUC_lexicon_tags_30 },
    { 0x7a20c63e, SUC_lexicon_tags_47 },
    { 0xfec0c639, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a20c644, SUC_lexicon_tags_27 },
    { 0xe0e0c645, SUC_lexicon_tags_19 },
    { 0xe940c645, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa0c64a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0c64d, SUC_lexicon_tags_37 },
    { 0xb420c64e, SUC_lexicon_tags_191 },
    { 0xbb80c64f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde0c654, SUC_lexicon_tags_11 },
    { 0x8620c654, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a0c659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb20c65c, SUC_lexicon_tags_12 },
    { 0x23c0c65c, SUC_lexicon_tags_226 },
    { 0xf400c65e, SUC_lexicon_tags_19 },
    { 0x1200c65f, SUC_lexicon_tags_2 },
    { 0x4380c65d, SUC_lexicon_tags_47 },
    { 0x28c0c661, SUC_lexicon_tags_25 },
    { 0x76c0c662, SUC_lexicon_tags_25 },
    { 0x4780c663, SUC_lexicon_tags_43 },
    { 0x5a60c661, SUC_lexicon_tags_15 },
    { 0x6100c665, SUC_lexicon_tags_334 },
    { 0xff20c666, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b00c669, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabc0c66b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa0c670, SUC_lexicon_tags_17 },
    { 0xf8a0c670, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa460c675, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc3a0c677, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c0c67a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c60c67c, SUC_lexicon_tags_2 },
    { 0xa760c67d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3120c685, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef40c688, SUC_lexicon_tags_146 },
    { 0xbdc0c689, SUC_lexicon_tags_15 },
    { 0x1720c68a, SUC_lexicon_tags_61 },
    { 0xfe00c68a, SUC_lexicon_tags_25 },
    { 0xbda0c68b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6e40c68e, SUC_lexicon_tags_15 },
    { 0xf360c68f, SUC_lexicon_tags_33 },
    { 0xd620c690, SUC_lexicon_tags_23 },
    { 0x8ce0c68e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x86a0c693, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x180c695, SUC_lexicon_tags_8 },
    { 0x53e0c696, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc0c698, SUC_lexicon_tags_23 },
    { 0xc660c699, SUC_lexicon_tags_138 },
    { 0x4060c69a, SUC_lexicon_tags_11 },
    { 0xe080c69a, SUC_lexicon_tags_65 },
    { 0x480c69c, SUC_lexicon_tags_45 },
    { 0x27c0c69d, SUC_lexicon_tags_17 },
    { 0xcfa0c69a, SUC_lexicon_tags_26 },
    { 0xce40c69f, SUC_lexicon_tags_7 },
    { 0x5f20c6a0, SUC_lexicon_tags_37 },
    { 0xc580c6a0, SUC_lexicon_tags_12 },
    { 0x76e0c6a0, SUC_lexicon_tags_33 },
    { 0x55e0c6a1, SUC_lexicon_tags_12 },
    { 0x8c00c6a4, SUC_lexicon_tags_43 },
    { 0xe940c6a0, SUC_lexicon_tags_60 },
    { 0xe540c6a6, SUC_lexicon_tags_7 },
    { 0x8b20c6a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1220c6ad, SUC_lexicon_tags_148 },
    { 0x2620c6ae, SUC_lexicon_tags_84 },
    { 0x5d20c6ae, SUC_lexicon_tags_34 },
    { 0xb6c0c6b0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5060c6b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8940c6b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c0c6b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8360c6be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e40c6c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3120c6c6, SUC_lexicon_tags_87 },
    { 0x99e0c6c7, SUC_lexicon_tags_45 },
    { 0xa820c6c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2360c6cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9820c6d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ec0c6d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c0c6d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8560c6d9, SUC_lexicon_tags_63 },
    { 0x44c0c6da, SUC_lexicon_tags_1 },
    { 0x8bc0c6db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6040c6e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1840c6e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae40c6eb, SUC_lexicon_tags_45 },
    { 0x5720c6ec, SUC_lexicon_tags_60 },
    { 0xd200c6ec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd900c6ef, SUC_lexicon_tags_17 },
    { 0x47e0c6f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa760c6f2, SUC_lexicon_tags_11 },
    { 0x77c0c6f3, SUC_lexicon_tags_25 },
    { 0x4020c6f3, SUC_lexicon_tags_2 },
    { 0x3660c6f5, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe640c6f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e0c6fc, SUC_lexicon_tags_15 },
    { 0xc180c6fd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xf780c6ff, SUC_lexicon_tags_31 },
    { 0x9340c6ff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc200c704, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e00c706, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6260c708, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf560c70b, SUC_lexicon_tags_45 },
    { 0x33e0c70c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea0c710, SUC_lexicon_tags_32 },
    { 0xe9a0c711, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf9a0c713, SUC_lexicon_tags_87 },
    { 0xfc00c714, SUC_lexicon_tags_14 },
    { 0x6960c714, SUC_lexicon_tags_17 },
    { 0x1a0c715, SUC_lexicon_tags_45 },
    { 0xe1c0c715, SUC_lexicon_tags_33 },
    { 0x8340c718, SUC_lexicon_tags_15 },
    { 0x940c717, SUC_lexicon_tags_17 },
    { 0x90c0c716, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d60c71d, SUC_lexicon_tags_43 },
    { 0xd280c71e, SUC_lexicon_tags_15 },
    { 0xe380c71f, SUC_lexicon_tags_47 },
    { 0xd240c71f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c20c723, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa20c72a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5020c72e, SUC_lexicon_tags_15 },
    { 0x3da0c72e, SUC_lexicon_tags_1 },
    { 0xe2c0c730, SUC_lexicon_tags_44 },
    { 0x33e0c730, SUC_lexicon_tags_47 },
    { 0x4c20c730, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c40c736, SUC_lexicon_tags_33 },
    { 0xec80c736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0c73a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3980c73d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf560c741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3440c74d, SUC_lexicon_tags_7 },
    { 0x8e20c74d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9980c750, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d60c755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20c758, SUC_lexicon_tags_7 },
    { 0x5c80c759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c760, SUC_lexicon_tags_1 },
    { 0x9c0c761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa920c763, SUC_lexicon_tags_2 },
    { 0x4760c764, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a0c769, SUC_lexicon_tags_15 },
    { 0x1440c76a, SUC_lexicon_tags_14 },
    { 0x9700c76b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x9520c76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf80c771, SUC_lexicon_tags_7 },
    { 0x9700c772, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd800c774, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca0c778, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb5a0c77a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0c783, SUC_lexicon_tags_12 },
    { 0xaf20c784, SUC_lexicon_tags_1 },
    { 0xdd60c785, SUC_lexicon_tags_47 },
    { 0xb440c785, SUC_lexicon_tags_7 },
    { 0xc980c787, SUC_lexicon_tags_33 },
    { 0xfb00c787, SUC_lexicon_tags_16 },
    { 0xda60c789, SUC_lexicon_tags_19 },
    { 0xbc0c785, SUC_lexicon_tags_23 },
    { 0x1aa0c78b, SUC_lexicon_tags_55 },
    { 0xb4a0c78c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2fa0c78e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4360c790, SUC_lexicon_tags_87 },
    { 0xdc0c791, SUC_lexicon_tags_8 },
    { 0x6560c792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa220c794, SUC_lexicon_tags_14 },
    { 0xa4c0c795, SUC_lexicon_tags_25 },
    { 0x1420c796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9560c79d, SUC_lexicon_tags_14 },
    { 0x1860c79e, SUC_lexicon_tags_15 },
    { 0x3520c79f, SUC_lexicon_tags_0 },
    { 0x86a0c7a0, SUC_lexicon_tags_7 },
    { 0xe400c79e, SUC_lexicon_tags_68 },
    { 0xa9a0c7a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2e20c7a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e0c7a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3580c7a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa40c7ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ae0c7b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f60c7b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe980c7b5, SUC_lexicon_tags_47 },
    { 0xdc60c7b5, SUC_lexicon_tags_15 },
    { 0x4120c7b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c60c7b9, SUC_lexicon_tags_45 },
    { 0xe680c7b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6180c7c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7680c7ca, SUC_lexicon_tags_19 },
    { 0x83e0c7cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe40c7cd, SUC_lexicon_tags_97 },
    { 0xf60c7ce, SUC_lexicon_tags_2 },
    { 0xe140c7ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb300c7d2, SUC_lexicon_tags_12 },
    { 0xb560c7d2, SUC_lexicon_tags_28 },
    { 0x3f80c7d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1220c7d6, SUC_lexicon_tags_37 },
    { 0x1da0c7d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xab00c7da, SUC_lexicon_tags_2 },
    { 0xe780c7db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2040c7dd, SUC_lexicon_tags_96 },
    { 0x8a00c7de, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3960c7e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8220c7e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d80c7e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad40c7ec, SUC_lexicon_tags_11 },
    { 0x3960c7ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00c7f2, SUC_lexicon_tags_79 },
    { 0x64e0c7f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc880c7f7, SUC_lexicon_tags_45 },
    { 0xd0c0c7f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2060c7fb, SUC_lexicon_tags_85 },
    { 0xb700c7fb, SUC_lexicon_tags_87 },
    { 0x8ae0c7fd, SUC_lexicon_tags_66 },
    { 0x7020c7fc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac0c806, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e0c809, SUC_lexicon_tags_44 },
    { 0x7300c809, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe080c80f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa080c817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f60c81a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb640c81d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa900c824, SUC_lexicon_tags_31 },
    { 0xbc60c825, SUC_lexicon_tags_47 },
    { 0xd6c0c826, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5e0c828, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xbd40c82a, SUC_lexicon_tags_2 },
    { 0x3900c82a, SUC_lexicon_tags_15 },
    { 0x8100c82c, SUC_lexicon_tags_37 },
    { 0x8920c82c, SUC_lexicon_tags_38 },
    { 0xae40c82c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97a0c830, SUC_lexicon_tags_37 },
    { 0xe580c830, SUC_lexicon_tags_7 },
    { 0xa4a0c832, SUC_lexicon_tags_10 },
    { 0xa520c833, SUC_lexicon_tags_7 },
    { 0xc560c833, SUC_lexicon_tags_11 },
    { 0x49e0c834, SUC_lexicon_tags_44 },
    { 0xbcc0c832, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xbe20c838, SUC_lexicon_tags_0 },
    { 0x6c60c838, SUC_lexicon_tags_47 },
    { 0xc040c839, SUC_lexicon_tags_95 },
    { 0x9ca0c83b, SUC_lexicon_tags_14 },
    { 0x33e0c83c, SUC_lexicon_tags_23 },
    { 0x7b60c83d, SUC_lexicon_tags_42 },
    { 0x67a0c839, SUC_lexicon_tags_37 },
    { 0x2b60c83f, SUC_lexicon_tags_6 },
    { 0x7c20c840, SUC_lexicon_tags_37 },
    { 0x2b00c838, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca0c848, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e0c84b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d00c850, SUC_lexicon_tags_40 },
    { 0xa7a0c851, SUC_lexicon_tags_37 },
    { 0xc040c852, SUC_lexicon_tags_12 },
    { 0x29c0c851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6dc0c855, SUC_lexicon_tags_33 },
    { 0xee0c855, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be0c859, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b00c85c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b40c85f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde80c868, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe140c86b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2500c86d, SUC_lexicon_tags_14 },
    { 0xdd20c86e, SUC_lexicon_tags_2 },
    { 0xa5a0c86d, SUC_lexicon_tags_34 },
    { 0xe180c870, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b20c879, SUC_lexicon_tags_15 },
    { 0x30c0c87a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a0c87f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc60c881, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe5a0c883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf620c888, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c897, SUC_lexicon_tags_15 },
    { 0x4380c898, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5820c89d, SUC_lexicon_tags_33 },
    { 0xcaa0c89e, SUC_lexicon_tags_0 },
    { 0x6480c89d, SUC_lexicon_tags_7 },
    { 0xf2e0c89e, SUC_lexicon_tags_31 },
    { 0xcdc0c8a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3440c8a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20a0c8a7, SUC_lexicon_tags_12 },
    { 0x3980c8a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fe0c8aa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca0c8af, SUC_lexicon_tags_11 },
    { 0xb260c8af, SUC_lexicon_tags_87 },
    { 0x9ac0c8b1, SUC_lexicon_tags_123 },
    { 0x83e0c8b1, SUC_lexicon_tags_11 },
    { 0xf20c8b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e0c8be, SUC_lexicon_tags_37 },
    { 0x2d40c8bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2500c8c4, SUC_lexicon_tags_15 },
    { 0xf7e0c8c4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8560c8c7, SUC_lexicon_tags_48 },
    { 0xdf20c8c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea0c8cc, SUC_lexicon_tags_11 },
    { 0x9280c8cc, SUC_lexicon_tags_2 },
    { 0xba80c8ce, SUC_lexicon_tags_37 },
    { 0x1940c8cf, SUC_lexicon_tags_34 },
    { 0xe320c8d0, SUC_lexicon_tags_7 },
    { 0x8b00c8d1, SUC_lexicon_tags_27 },
    { 0xd300c8ce, SUC_lexicon_tags_87 },
    { 0x5160c8d3, SUC_lexicon_tags_26 },
    { 0xd560c8cf, SUC_lexicon_tags_29 },
    { 0x2a00c8d5, SUC_lexicon_tags_39 },
    { 0x5320c8d6, SUC_lexicon_tags_34 },
    { 0x4760c8d4, SUC_lexicon_tags_25 },
    { 0x1ee0c8d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f20c8da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2560c8dd, SUC_lexicon_tags_31 },
    { 0xd9c0c8de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3080c8e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec0c8e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e0c8ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a80c8ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b60c8f1, SUC_lexicon_tags_33 },
    { 0xfa40c8f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x31a0c8f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb360c8f6, SUC_lexicon_tags_37 },
    { 0x5820c8f7, SUC_lexicon_tags_16 },
    { 0xe160c8f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x28c0c8fa, SUC_lexicon_tags_65 },
    { 0xf0c0c8fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f40c8ff, SUC_lexicon_tags_18 },
    { 0x7960c900, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb920c906, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf760c909, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9800c90f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc440c912, SUC_lexicon_tags_21 },
    { 0x8980c912, SUC_lexicon_tags_2 },
    { 0x82a0c914, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec0c919, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x120c91b, SUC_lexicon_tags_16 },
    { 0x200c91c, SUC_lexicon_tags_26 },
    { 0xc940c91d, SUC_lexicon_tags_12 },
    { 0x5120c91e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2640c921, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c0c925, SUC_lexicon_tags_11 },
    { 0xa380c925, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a0c929, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa020c92d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7580c934, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae00c938, SUC_lexicon_tags_17 },
    { 0x4c00c938, SUC_lexicon_tags_23 },
    { 0x2160c938, SUC_lexicon_tags_47 },
    { 0xf620c93b, SUC_lexicon_tags_16 },
    { 0x7ee0c93a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d60c93e, SUC_lexicon_tags_23 },
    { 0xa60c93e, SUC_lexicon_tags_2 },
    { 0xe420c940, SUC_lexicon_tags_17 },
    { 0xb580c941, SUC_lexicon_tags_3 },
    { 0x5820c942, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x31e0c944, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c0c947, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe360c94a, SUC_lexicon_tags_12 },
    { 0x30a0c94a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b80c950, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb00c952, SUC_lexicon_tags_37 },
    { 0x7b40c953, SUC_lexicon_tags_3 },
    { 0xf8c0c954, SUC_lexicon_tags_11 },
    { 0xe3c0c955, SUC_lexicon_tags_14 },
    { 0x2a60c956, SUC_lexicon_tags_77 },
    { 0x4f40c957, SUC_lexicon_tags_23 },
    { 0x24c0c956, SUC_lexicon_tags_14 },
    { 0x1900c955, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce0c95d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd420c95f, SUC_lexicon_tags_46 },
    { 0x1fc0c95f, SUC_lexicon_tags_12 },
    { 0xaee0c95f, SUC_lexicon_tags_7 },
    { 0x4ec0c962, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e0c96c, SUC_lexicon_tags_29 },
    { 0x8780c96d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb80c96f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c0c972, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd420c974, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x580c977, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fc0c979, SUC_lexicon_tags_17 },
    { 0x2fe0c97a, SUC_lexicon_tags_7 },
    { 0x22e0c97b, SUC_lexicon_tags_72 },
    { 0x3580c97c, SUC_lexicon_tags_31 },
    { 0x8400c97c, SUC_lexicon_tags_37 },
    { 0x3c0c97e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc0c981, SUC_lexicon_tags_28 },
    { 0xed00c981, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe0c985, SUC_lexicon_tags_55 },
    { 0xd000c986, SUC_lexicon_tags_14 },
    { 0xf800c987, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8160c989, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb000c992, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9600c997, SUC_lexicon_tags_37 },
    { 0x9c0c998, SUC_lexicon_tags_0 },
    { 0x1000c997, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8d60c99b, SUC_lexicon_tags_11 },
    { 0x1160c99c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66a0c99e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5960c9a4, SUC_lexicon_tags_81 },
    { 0xc8e0c9a5, SUC_lexicon_tags_27 },
    { 0x120c9a6, SUC_lexicon_tags_17 },
    { 0xee40c9a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd40c9a9, SUC_lexicon_tags_17 },
    { 0x24e0c9aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe340c9af, SUC_lexicon_tags_10 },
    { 0x8de0c9b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0c9b3, SUC_lexicon_tags_90 },
    { 0xd360c9b4, SUC_lexicon_tags_15 },
    { 0xa8c0c9b3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x520c9b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c60c9bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7800c9c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9240c9c3, SUC_lexicon_tags_12 },
    { 0x7260c9c3, SUC_lexicon_tags_43 },
    { 0x3fe0c9c5, SUC_lexicon_tags_49 },
    { 0x900c9c6, SUC_lexicon_tags_17 },
    { 0xede0c9c7, SUC_lexicon_tags_12 },
    { 0x6e80c9c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e0c9d0, SUC_lexicon_tags_15 },
    { 0xd7a0c9d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5920c9d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf620c9d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea80c9dd, SUC_lexicon_tags_1 },
    { 0xe4a0c9dd, SUC_lexicon_tags_25 },
    { 0x4640c9dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d00c9e1, SUC_lexicon_tags_7 },
    { 0x38e0c9e2, SUC_lexicon_tags_1 },
    { 0x3540c9e3, SUC_lexicon_tags_63 },
    { 0x4160c9e3, SUC_lexicon_tags_9 },
    { 0x6b00c9e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d00c9e7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c60c9eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0c9ee, SUC_lexicon_tags_17 },
    { 0xf4c0c9ef, SUC_lexicon_tags_43 },
    { 0xdc60c9f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce0c9f5, SUC_lexicon_tags_8 },
    { 0xbda0c9f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21c0c9f8, SUC_lexicon_tags_23 },
    { 0xae0c9f8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xee20c9fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd00c9fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c0ca06, SUC_lexicon_tags_15 },
    { 0x9920ca07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6760ca0a, SUC_lexicon_tags_15 },
    { 0x80e0ca0a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x4520ca0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6580ca10, SUC_lexicon_tags_17 },
    { 0x9ac0ca11, SUC_lexicon_tags_0 },
    { 0xe720ca12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d80ca22, SUC_lexicon_tags_11 },
    { 0xf1c0ca22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfa00ca25, SUC_lexicon_tags_19 },
    { 0xb140ca25, SUC_lexicon_tags_60 },
    { 0x6680ca27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x67e0ca29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f60ca2b, SUC_lexicon_tags_15 },
    { 0x7bc0ca2c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x2560ca2f, SUC_lexicon_tags_27 },
    { 0x2920ca30, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c0ca34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f00ca36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9920ca38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb20ca3a, SUC_lexicon_tags_11 },
    { 0x2c60ca3b, SUC_lexicon_tags_11 },
    { 0xe680ca3b, SUC_lexicon_tags_1 },
    { 0x3bc0ca3d, SUC_lexicon_tags_17 },
    { 0x40a0ca3e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xd280ca41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e0ca44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5160ca51, SUC_lexicon_tags_57 },
    { 0xf7e0ca52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1ec0ca54, SUC_lexicon_tags_11 },
    { 0x66e0ca55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab80ca58, SUC_lexicon_tags_16 },
    { 0xef80ca59, SUC_lexicon_tags_12 },
    { 0x660ca5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c0ca5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e0ca62, SUC_lexicon_tags_42 },
    { 0xbda0ca63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf100ca68, SUC_lexicon_tags_1 },
    { 0x7020ca68, SUC_lexicon_tags_87 },
    { 0xa040ca6a, SUC_lexicon_tags_25 },
    { 0x8000ca6b, SUC_lexicon_tags_29 },
    { 0x24c0ca68, SUC_lexicon_tags_2 },
    { 0x2860ca6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0ca70, SUC_lexicon_tags_15 },
    { 0x99c0ca70, SUC_lexicon_tags_45 },
    { 0x1720ca72, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3f80ca74, SUC_lexicon_tags_31 },
    { 0xd580ca75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1120ca79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d40ca7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce0ca7f, SUC_lexicon_tags_27 },
    { 0xb8c0ca80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4c0ca82, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1460ca84, SUC_lexicon_tags_45 },
    { 0x6500ca84, SUC_lexicon_tags_25 },
    { 0x6c60ca86, SUC_lexicon_tags_39 },
    { 0xcbc0ca87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd380ca89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5280ca8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf580ca8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa80ca90, SUC_lexicon_tags_21 },
    { 0x5440ca91, SUC_lexicon_tags_3 },
    { 0xdb80ca90, SUC_lexicon_tags_38 },
    { 0xbdc0ca93, SUC_lexicon_tags_1 },
    { 0xa7e0ca93, SUC_lexicon_tags_48 },
    { 0xdb80ca90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e0ca9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a0caa1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e0caa4, SUC_lexicon_tags_6 },
    { 0x5640caa4, SUC_lexicon_tags_27 },
    { 0xd7e0caa4, SUC_lexicon_tags_47 },
    { 0xf6e0caa7, SUC_lexicon_tags_2 },
    { 0xf180caa8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe0e0caaa, SUC_lexicon_tags_30 },
    { 0xb3c0caab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6840caad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac0cab1, SUC_lexicon_tags_27 },
    { 0x90e0cab2, SUC_lexicon_tags_38 },
    { 0x9840cab2, SUC_lexicon_tags_25 },
    { 0xbc0cab4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a0cab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee60cac0, SUC_lexicon_tags_45 },
    { 0xcb20cac1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1480cac5, SUC_lexicon_tags_12 },
    { 0xba00cac6, SUC_lexicon_tags_11 },
    { 0x7040cac7, SUC_lexicon_tags_11 },
    { 0x7520cac7, SUC_lexicon_tags_17 },
    { 0x72e0cac5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e0cace, SUC_lexicon_tags_19 },
    { 0x7980cacf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7a0cad1, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xd420cad3, SUC_lexicon_tags_7 },
    { 0x4860cad4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a0cad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7740cada, SUC_lexicon_tags_11 },
    { 0xb780cada, SUC_lexicon_tags_14 },
    { 0x81a0cada, SUC_lexicon_tags_230 },
    { 0xaae0cada, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d40cadf, SUC_lexicon_tags_60 },
    { 0x9a60cae0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6800cae6, SUC_lexicon_tags_75 },
    { 0x7b20cae7, SUC_lexicon_tags_25 },
    { 0xfcc0cae8, SUC_lexicon_tags_25 },
    { 0x8fe0cae9, SUC_lexicon_tags_1 },
    { 0x4320caea, SUC_lexicon_tags_13 },
    { 0x96c0cae8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf00caed, SUC_lexicon_tags_37 },
    { 0x8a40caed, SUC_lexicon_tags_17 },
    { 0x10e0caee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c0caf2, SUC_lexicon_tags_17 },
    { 0xe420caf3, SUC_lexicon_tags_1 },
    { 0x3400caf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5700cafb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea0caff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3c0cb01, SUC_lexicon_tags_1 },
    { 0x8120cb02, SUC_lexicon_tags_12 },
    { 0xf100cb03, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc0cb06, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf480cb09, SUC_lexicon_tags_47 },
    { 0xc360cb0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc60cb0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe500cb14, SUC_lexicon_tags_60 },
    { 0xbdc0cb15, SUC_lexicon_tags_25 },
    { 0x3f40cb15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c0cb19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2680cb1b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee00cb1f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd500cb24, SUC_lexicon_tags_6 },
    { 0xe940cb25, SUC_lexicon_tags_15 },
    { 0x7540cb26, SUC_lexicon_tags_33 },
    { 0x3a60cb26, SUC_lexicon_tags_19 },
    { 0x7180cb27, SUC_lexicon_tags_14 },
    { 0x8ae0cb29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee40cb2b, SUC_lexicon_tags_26 },
    { 0xaa40cb2c, SUC_lexicon_tags_38 },
    { 0x85a0cb2d, SUC_lexicon_tags_12 },
    { 0x5bc0cb2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e0cb31, SUC_lexicon_tags_7 },
    { 0x8c20cb32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b00cb38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe60cb3a, SUC_lexicon_tags_56 },
    { 0x2420cb3a, SUC_lexicon_tags_3 },
    { 0xf300cb3b, SUC_lexicon_tags_6 },
    { 0xd500cb3a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde0cb42, SUC_lexicon_tags_12 },
    { 0x5c0cb43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5080cb4d, SUC_lexicon_tags_77 },
    { 0x30c0cb4e, SUC_lexicon_tags_2 },
    { 0x18c0cb4e, SUC_lexicon_tags_19 },
    { 0x7c00cb4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a0cb54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c80cb58, SUC_lexicon_tags_38 },
    { 0x6fa0cb59, SUC_lexicon_tags_2 },
    { 0xa60cb5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00cb60, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd80cb63, SUC_lexicon_tags_3 },
    { 0x93e0cb63, SUC_lexicon_tags_23 },
    { 0x3d80cb64, SUC_lexicon_tags_48 },
    { 0x60c0cb66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6700cb69, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf80cb6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea0cb70, SUC_lexicon_tags_25 },
    { 0x6ce0cb71, SUC_lexicon_tags_49 },
    { 0xf1c0cb72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a0cb76, SUC_lexicon_tags_17 },
    { 0x94e0cb77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9560cb7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc700cb80, SUC_lexicon_tags_11 },
    { 0xc920cb81, SUC_lexicon_tags_17 },
    { 0xf180cb81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5600cb8a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc0cb90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0cb93, SUC_lexicon_tags_2 },
    { 0x1740cb93, SUC_lexicon_tags_37 },
    { 0x26e0cb95, SUC_lexicon_tags_12 },
    { 0x6680cb96, SUC_lexicon_tags_37 },
    { 0xf0e0cb96, SUC_lexicon_tags_25 },
    { 0xe8e0cb98, SUC_lexicon_tags_14 },
    { 0xc3c0cb99, SUC_lexicon_tags_49 },
    { 0x8200cb9a, SUC_lexicon_tags_318 },
    { 0x2c00cb9b, SUC_lexicon_tags_56 },
    { 0x1cc0cb97, SUC_lexicon_tags_62 },
    { 0x3c40cb9d, SUC_lexicon_tags_19 },
    { 0x3960cb9e, SUC_lexicon_tags_2 },
    { 0xe820cb9e, SUC_lexicon_tags_17 },
    { 0xa0cb9e, SUC_lexicon_tags_10 },
    { 0x3620cb9e, SUC_lexicon_tags_3 },
    { 0x82a0cb9f, SUC_lexicon_tags_57 },
    { 0x4f00cba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc000cba5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3900cbaa, SUC_lexicon_tags_2 },
    { 0x52e0cbab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a0cbaf, SUC_lexicon_tags_43 },
    { 0xe640cbb0, SUC_lexicon_tags_109 },
    { 0x5360cbb1, SUC_lexicon_tags_7 },
    { 0xe440cbb2, SUC_lexicon_tags_23 },
    { 0x8880cbb2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a0cbbc, SUC_lexicon_tags_37 },
    { 0xb420cbbc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe580cbc0, SUC_lexicon_tags_28 },
    { 0x23e0cbc1, SUC_lexicon_tags_17 },
    { 0x3fe0cbc1, SUC_lexicon_tags_68 },
    { 0x55a0cbc3, SUC_lexicon_tags_38 },
    { 0xdc60cbc0, SUC_lexicon_tags_11 },
    { 0xf520cbc0, SUC_lexicon_tags_11 },
    { 0x10c0cbc2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e0cbc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7e0cbcb, SUC_lexicon_tags_7 },
    { 0x4d40cbcc, SUC_lexicon_tags_150 },
    { 0xd00cbcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7140cbcf, SUC_lexicon_tags_15 },
    { 0x3900cbd0, SUC_lexicon_tags_1 },
    { 0x1a00cbd0, SUC_lexicon_tags_6 },
    { 0xc800cbd2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x11e0cbd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c0cbd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b40cbda, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3680cbde, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8440cbe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e0cbe4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3260cbee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc0cbf1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xde80cbf3, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b60cbf8, SUC_lexicon_tags_49 },
    { 0x38c0cbf9, SUC_lexicon_tags_15 },
    { 0xeb20cbfa, SUC_lexicon_tags_11 },
    { 0xa120cbfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac80cbfe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1920cc07, SUC_lexicon_tags_2 },
    { 0x69c0cc08, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa040cc0d, SUC_lexicon_tags_19 },
    { 0xfc00cc0e, SUC_lexicon_tags_30 },
    { 0x8480cc0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9de0cc11, SUC_lexicon_tags_17 },
    { 0x38e0cc12, SUC_lexicon_tags_0 },
    { 0x9bc0cc13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a0cc18, SUC_lexicon_tags_34 },
    { 0x5960cc19, SUC_lexicon_tags_11 },
    { 0xa140cc19, SUC_lexicon_tags_29 },
    { 0x3700cc1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2940cc20, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe160cc2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb320cc2e, SUC_lexicon_tags_62 },
    { 0xc700cc2e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5ac0cc31, SUC_lexicon_tags_49 },
    { 0xf840cc31, SUC_lexicon_tags_670 },
    { 0, NULL },
    { 0, NULL },
    { 0x180cc35, SUC_lexicon_tags_25 },
    { 0x6200cc36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba80cc39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7320cc3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa160cc41, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d40cc45, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9f00cc47, SUC_lexicon_tags_7 },
    { 0x42e0cc47, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb20cc4c, SUC_lexicon_tags_591 },
    { 0xa80cc4d, SUC_lexicon_tags_80 },
    { 0x7660cc4e, SUC_lexicon_tags_56 },
    { 0xc880cc4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7860cc52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e0cc57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2260cc59, SUC_lexicon_tags_13 },
    { 0x3ce0cc5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c00cc5e, SUC_lexicon_tags_34 },
    { 0xe40cc5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f00cc62, SUC_lexicon_tags_49 },
    { 0x4bc0cc63, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa440cc65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c0cc68, SUC_lexicon_tags_37 },
    { 0xef60cc69, SUC_lexicon_tags_61 },
    { 0x3160cc6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb540cc6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e0cc70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8840cc74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2260cc76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e20cc7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa940cc81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d00cc8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf040cc8c, SUC_lexicon_tags_7 },
    { 0x5fe0cc8d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8060cc8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a80cc94, SUC_lexicon_tags_17 },
    { 0x6360cc95, SUC_lexicon_tags_11 },
    { 0xd4a0cc96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6300cc9d, SUC_lexicon_tags_17 },
    { 0xa6c0cc9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e0cca7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe580ccaa, SUC_lexicon_tags_25 },
    { 0x9bc0ccaa, SUC_lexicon_tags_11 },
    { 0x4b80ccac, SUC_lexicon_tags_12 },
    { 0x6c80ccad, SUC_lexicon_tags_27 },
    { 0xf080ccac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c0ccb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c0ccb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a0ccba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa940ccc0, SUC_lexicon_tags_37 },
    { 0xe240ccc1, SUC_lexicon_tags_11 },
    { 0x2da0ccc2, SUC_lexicon_tags_16 },
    { 0x1e80ccc2, SUC_lexicon_tags_92 },
    { 0x42e0ccc3, SUC_lexicon_tags_2 },
    { 0xa220ccc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7760ccca, SUC_lexicon_tags_25 },
    { 0xae00cccb, SUC_lexicon_tags_45 },
    { 0xed00cccc, SUC_lexicon_tags_7 },
    { 0x91e0cccc, SUC_lexicon_tags_29 },
    { 0xe880ccce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96c0ccd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c0ccd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a0ccd6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd120ccde, SUC_lexicon_tags_77 },
    { 0xc2e0ccdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d00cce9, SUC_lexicon_tags_49 },
    { 0x12e0ccea, SUC_lexicon_tags_27 },
    { 0xc560cceb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7840cced, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7380ccef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc640ccf5, SUC_lexicon_tags_71 },
    { 0x5e00ccf5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5360ccfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeae0cd00, SUC_lexicon_tags_37 },
    { 0x35a0cd01, SUC_lexicon_tags_17 },
    { 0x80cd01, SUC_lexicon_tags_38 },
    { 0x5b20cd02, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a20cd06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e0cd09, SUC_lexicon_tags_11 },
    { 0x7fc0cd0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe00cd0f, SUC_lexicon_tags_6 },
    { 0x86a0cd10, SUC_lexicon_tags_37 },
    { 0x9840cd11, SUC_lexicon_tags_33 },
    { 0x8220cd11, SUC_lexicon_tags_31 },
    { 0x3f00cd10, SUC_lexicon_tags_37 },
    { 0xa640cd11, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5420cd17, SUC_lexicon_tags_0 },
    { 0xf3e0cd17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a0cd1d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be0cd22, SUC_lexicon_tags_10 },
    { 0x10c0cd23, SUC_lexicon_tags_47 },
    { 0x37e0cd23, SUC_lexicon_tags_12 },
    { 0xb660cd25, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb200cd2d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea0cd33, SUC_lexicon_tags_37 },
    { 0xdda0cd33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee00cd38, SUC_lexicon_tags_23 },
    { 0xa700cd39, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x3400cd3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ea0cd3d, SUC_lexicon_tags_38 },
    { 0xb920cd3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b60cd40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa600cd43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9260cd47, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x56a0cd49, SUC_lexicon_tags_37 },
    { 0x4f80cd4a, SUC_lexicon_tags_193 },
    { 0xa80cd4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea0cd4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c0cd55, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5560cd57, SUC_lexicon_tags_11 },
    { 0x2a40cd58, SUC_lexicon_tags_63 },
    { 0x8f60cd59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa00cd5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c80cd60, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x5200cd63, SUC_lexicon_tags_32 },
    { 0x59c0cd63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9540cd6b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0cd6e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c0cd71, SUC_lexicon_tags_26 },
    { 0x5080cd72, SUC_lexicon_tags_8 },
    { 0xe4e0cd73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf300cd75, SUC_lexicon_tags_27 },
    { 0x70c0cd76, SUC_lexicon_tags_2 },
    { 0xe8a0cd76, SUC_lexicon_tags_37 },
    { 0x5f80cd76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4a0cd7a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4500cd7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e0cd80, SUC_lexicon_tags_1 },
    { 0x4880cd81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6600cd83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e0cd89, SUC_lexicon_tags_12 },
    { 0xf4a0cd8a, SUC_lexicon_tags_7 },
    { 0xcf40cd8b, SUC_lexicon_tags_19 },
    { 0xbfc0cd8a, SUC_lexicon_tags_25 },
    { 0x5d80cd8d, SUC_lexicon_tags_60 },
    { 0x3b60cd8a, SUC_lexicon_tags_7 },
    { 0x8900cd8b, SUC_lexicon_tags_28 },
    { 0x2fe0cd90, SUC_lexicon_tags_0 },
    { 0x6e60cd90, SUC_lexicon_tags_17 },
    { 0xfe80cd92, SUC_lexicon_tags_1 },
    { 0xe800cd93, SUC_lexicon_tags_31 },
    { 0x7fa0cd93, SUC_lexicon_tags_1 },
    { 0x5280cd95, SUC_lexicon_tags_57 },
    { 0xab80cd91, SUC_lexicon_tags_11 },
    { 0x700cd97, SUC_lexicon_tags_17 },
    { 0xb380cd98, SUC_lexicon_tags_25 },
    { 0x9140cd99, SUC_lexicon_tags_8 },
    { 0xa240cd9a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x86e0cd9c, SUC_lexicon_tags_7 },
    { 0xfec0cd9c, SUC_lexicon_tags_27 },
    { 0x7720cd9e, SUC_lexicon_tags_1 },
    { 0x85e0cd9f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6040cda3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc0cda8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf20cdaa, SUC_lexicon_tags_2 },
    { 0x92a0cdaa, SUC_lexicon_tags_48 },
    { 0x8560cdac, SUC_lexicon_tags_47 },
    { 0x3640cdac, SUC_lexicon_tags_17 },
    { 0x4ac0cdac, SUC_lexicon_tags_113 },
    { 0xfca0cdaa, SUC_lexicon_tags_37 },
    { 0x67e0cdaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b00cdb8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8720cdba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a0cdc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8700cdc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49e0cdc9, SUC_lexicon_tags_30 },
    { 0x1520cdca, SUC_lexicon_tags_37 },
    { 0x53c0cdcb, SUC_lexicon_tags_15 },
    { 0xfca0cdcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a0cdd2, SUC_lexicon_tags_26 },
    { 0x9160cdd3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec60cddc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe260cdee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde0cdf5, SUC_lexicon_tags_49 },
    { 0x9ac0cdf6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c0cdf9, SUC_lexicon_tags_17 },
    { 0x7340cdfa, SUC_lexicon_tags_19 },
    { 0xa640cdfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc20cdfd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xad60ce00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d80ce05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf00ce0b, SUC_lexicon_tags_46 },
    { 0x1200ce0c, SUC_lexicon_tags_160 },
    { 0x3620ce0c, SUC_lexicon_tags_37 },
    { 0x1f40ce0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae0ce15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8700ce19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9520ce1f, SUC_lexicon_tags_17 },
    { 0xe300ce20, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1520ce25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c0ce2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8600ce31, SUC_lexicon_tags_1 },
    { 0x80c0ce31, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e0ce3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe960ce3c, SUC_lexicon_tags_37 },
    { 0x6880ce3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1000ce3f, SUC_lexicon_tags_12 },
    { 0x9000ce40, SUC_lexicon_tags_37 },
    { 0x1b40ce41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc0ce46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb640ce4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb060ce4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc600ce4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a60ce54, SUC_lexicon_tags_1 },
    { 0xd3a0ce55, SUC_lexicon_tags_29 },
    { 0x34e0ce56, SUC_lexicon_tags_947 },
    { 0xe480ce57, SUC_lexicon_tags_9 },
    { 0x1540ce58, SUC_lexicon_tags_0 },
    { 0x22e0ce59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3140ce5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8560ce60, SUC_lexicon_tags_15 },
    { 0xfec0ce61, SUC_lexicon_tags_16 },
    { 0x7760ce62, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e0ce6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd00ce6c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf6a0ce6e, SUC_lexicon_tags_43 },
    { 0x6160ce6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbee0ce71, SUC_lexicon_tags_43 },
    { 0x1fc0ce71, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1040ce75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd220ce78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a0ce7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a0ce7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb20ce81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb20ce83, SUC_lexicon_tags_24 },
    { 0x1c20ce84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c0ce8b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb60ce8f, SUC_lexicon_tags_54 },
    { 0xac80ce90, SUC_lexicon_tags_15 },
    { 0xcd80ce90, SUC_lexicon_tags_15 },
    { 0x3280ce92, SUC_lexicon_tags_45 },
    { 0x98e0ce92, SUC_lexicon_tags_11 },
    { 0x12c0ce93, SUC_lexicon_tags_17 },
    { 0x9be0ce95, SUC_lexicon_tags_0 },
    { 0xb2e0ce94, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c0ce9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x920ce9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea20cea0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa0cea3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbbc0cea5, SUC_lexicon_tags_47 },
    { 0x4600cea6, SUC_lexicon_tags_28 },
    { 0xd600cea7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc80ceac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9660ceae, SUC_lexicon_tags_11 },
    { 0xfb80ceaf, SUC_lexicon_tags_4 },
    { 0xf720ceaf, SUC_lexicon_tags_9 },
    { 0x2820ceb1, SUC_lexicon_tags_43 },
    { 0x14a0ceb1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ce0ceb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd460ceb6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8f80ceb8, SUC_lexicon_tags_2 },
    { 0x48a0ceb9, SUC_lexicon_tags_35 },
    { 0xe600ceba, SUC_lexicon_tags_5 },
    { 0x6e0ceba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c0cec4, SUC_lexicon_tags_25 },
    { 0xa40cec5, SUC_lexicon_tags_27 },
    { 0x6500cec5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c0cec9, SUC_lexicon_tags_15 },
    { 0x8480cec9, SUC_lexicon_tags_31 },
    { 0xe160ceca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd340ced1, SUC_lexicon_tags_8 },
    { 0xdb20ced2, SUC_lexicon_tags_12 },
    { 0x5800ced2, SUC_lexicon_tags_27 },
    { 0x91c0ced4, SUC_lexicon_tags_8 },
    { 0x2ca0ced5, SUC_lexicon_tags_37 },
    { 0x4a80ced5, SUC_lexicon_tags_2 },
    { 0xf380ced7, SUC_lexicon_tags_26 },
    { 0xabc0ced5, SUC_lexicon_tags_12 },
    { 0x1c20ced2, SUC_lexicon_tags_113 },
    { 0xac60ced7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9400cedd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa0ceea, SUC_lexicon_tags_17 },
    { 0x3b80ceea, SUC_lexicon_tags_21 },
    { 0x1aa0ceec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc440ceee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7460cef2, SUC_lexicon_tags_47 },
    { 0x4200cef3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe0cef8, SUC_lexicon_tags_14 },
    { 0xd8e0cef9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ec0cefb, SUC_lexicon_tags_60 },
    { 0x9f60cefc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ec0cefe, SUC_lexicon_tags_2 },
    { 0xcd80ceff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4400cf02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1680cf05, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1ba0cf07, SUC_lexicon_tags_12 },
    { 0x1ee0cf08, SUC_lexicon_tags_17 },
    { 0xd1c0cf08, SUC_lexicon_tags_12 },
    { 0xc7a0cf0a, SUC_lexicon_tags_11 },
    { 0x14a0cf0b, SUC_lexicon_tags_43 },
    { 0x4be0cf07, SUC_lexicon_tags_14 },
    { 0x8b80cf0d, SUC_lexicon_tags_84 },
    { 0xcb80cf07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba0cf12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1080cf17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa820cf1a, SUC_lexicon_tags_37 },
    { 0x8b00cf1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64e0cf1d, SUC_lexicon_tags_14 },
    { 0x9000cf1e, SUC_lexicon_tags_37 },
    { 0x5bc0cf1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e80cf23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5080cf25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e0cf29, SUC_lexicon_tags_17 },
    { 0xb960cf29, SUC_lexicon_tags_21 },
    { 0x56c0cf29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e0cf33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5840cf35, SUC_lexicon_tags_46 },
    { 0x5d80cf36, SUC_lexicon_tags_15 },
    { 0xd3e0cf37, SUC_lexicon_tags_4 },
    { 0x5f20cf38, SUC_lexicon_tags_1 },
    { 0x86a0cf39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a0cf3c, SUC_lexicon_tags_22 },
    { 0xece0cf3d, SUC_lexicon_tags_1 },
    { 0xb340cf3e, SUC_lexicon_tags_2 },
    { 0xb040cf3d, SUC_lexicon_tags_42 },
    { 0x3140cf40, SUC_lexicon_tags_37 },
    { 0xbb00cf3f, SUC_lexicon_tags_65 },
    { 0x6d60cf42, SUC_lexicon_tags_65 },
    { 0x4c00cf43, SUC_lexicon_tags_15 },
    { 0x61a0cf44, SUC_lexicon_tags_38 },
    { 0xc820cf45, SUC_lexicon_tags_6 },
    { 0x87e0cf43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c0cf4a, SUC_lexicon_tags_26 },
    { 0x43e0cf4b, SUC_lexicon_tags_34 },
    { 0xb3c0cf4a, SUC_lexicon_tags_49 },
    { 0x5340cf4d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e0cf50, SUC_lexicon_tags_2 },
    { 0x3360cf50, SUC_lexicon_tags_7 },
    { 0xbcc0cf52, SUC_lexicon_tags_47 },
    { 0x6c00cf52, SUC_lexicon_tags_11 },
    { 0x20c0cf54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20cf57, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x580cf5a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x80e0cf5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6540cf5e, SUC_lexicon_tags_25 },
    { 0x4840cf5e, SUC_lexicon_tags_32 },
    { 0x4520cf5f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x44e0cf62, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0cf67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee0cf6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb800cf74, SUC_lexicon_tags_45 },
    { 0x7300cf75, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3820cf78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca0cf7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c20cf83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c0cf87, SUC_lexicon_tags_17 },
    { 0x2c40cf88, SUC_lexicon_tags_1 },
    { 0xdd00cf88, SUC_lexicon_tags_31 },
    { 0xaa0cf8a, SUC_lexicon_tags_14 },
    { 0xe560cf8a, SUC_lexicon_tags_28 },
    { 0xc680cf8a, SUC_lexicon_tags_17 },
    { 0x38e0cf87, SUC_lexicon_tags_16 },
    { 0xec0cf8e, SUC_lexicon_tags_12 },
    { 0x43e0cf87, SUC_lexicon_tags_45 },
    { 0xfe0cf90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf560cf94, SUC_lexicon_tags_35 },
    { 0x9980cf95, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8480cf97, SUC_lexicon_tags_11 },
    { 0xc2a0cf98, SUC_lexicon_tags_11 },
    { 0x2c40cf97, SUC_lexicon_tags_47 },
    { 0x8120cf98, SUC_lexicon_tags_11 },
    { 0x3dc0cf99, SUC_lexicon_tags_77 },
    { 0x84a0cf9c, SUC_lexicon_tags_9 },
    { 0xd000cf9d, SUC_lexicon_tags_56 },
    { 0x3440cf9d, SUC_lexicon_tags_11 },
    { 0xcd60cf9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc0cfa4, SUC_lexicon_tags_45 },
    { 0x5660cfa5, SUC_lexicon_tags_2 },
    { 0xf740cfa6, SUC_lexicon_tags_10 },
    { 0xfb00cfa5, SUC_lexicon_tags_29 },
    { 0x3e60cfa8, SUC_lexicon_tags_31 },
    { 0x9900cfa9, SUC_lexicon_tags_37 },
    { 0x9740cfa8, SUC_lexicon_tags_27 },
    { 0x9a60cfa5, SUC_lexicon_tags_323 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c0cfaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5880cfb1, SUC_lexicon_tags_1 },
    { 0x7b60cfb2, SUC_lexicon_tags_11 },
    { 0x1dc0cfb2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900cfb7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc360cfba, SUC_lexicon_tags_1 },
    { 0xdd60cfba, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x7220cfbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1460cfc2, SUC_lexicon_tags_2 },
    { 0x28a0cfc2, SUC_lexicon_tags_15 },
    { 0x4f00cfc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a0cfc9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1340cfcc, SUC_lexicon_tags_6 },
    { 0x4380cfcc, SUC_lexicon_tags_7 },
    { 0x7360cfce, SUC_lexicon_tags_63 },
    { 0x5ba0cfcc, SUC_lexicon_tags_40 },
    { 0xd100cfd0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7800cfd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa900cfd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9740cfd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c00cfdb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c0cfe0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e00cfe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b20cfe8, SUC_lexicon_tags_17 },
    { 0x1580cfe9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c0cfed, SUC_lexicon_tags_15 },
    { 0x1dc0cfee, SUC_lexicon_tags_69 },
    { 0xd560cfed, SUC_lexicon_tags_12 },
    { 0x820cff0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd8a0cff2, SUC_lexicon_tags_25 },
    { 0xfd00cff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca0cff8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc180cffa, SUC_lexicon_tags_64 },
    { 0xfa0cffa, SUC_lexicon_tags_7 },
    { 0xb0c0cffc, SUC_lexicon_tags_12 },
    { 0x2640cffa, SUC_lexicon_tags_33 },
    { 0xd460cffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbc0d000, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa340d003, SUC_lexicon_tags_25 },
    { 0xbf60d003, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c00d00b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6460d00e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x35a0d010, SUC_lexicon_tags_49 },
    { 0x9a60d011, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e0d014, SUC_lexicon_tags_1 },
    { 0xe8c0d014, SUC_lexicon_tags_37 },
    { 0x1420d016, SUC_lexicon_tags_7 },
    { 0x4b00d017, SUC_lexicon_tags_19 },
    { 0xd920d018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc060d01d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x10a0d01f, SUC_lexicon_tags_3 },
    { 0x8040d020, SUC_lexicon_tags_25 },
    { 0xd660d021, SUC_lexicon_tags_49 },
    { 0xb740d021, SUC_lexicon_tags_45 },
    { 0xa860d023, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a0d026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c0d028, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec0d02b, SUC_lexicon_tags_7 },
    { 0x3f60d02b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840d030, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x31a0d032, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c0d036, SUC_lexicon_tags_19 },
    { 0x7560d037, SUC_lexicon_tags_25 },
    { 0x2880d038, SUC_lexicon_tags_11 },
    { 0x8800d037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa080d03c, SUC_lexicon_tags_25 },
    { 0xc360d03c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xac40d040, SUC_lexicon_tags_10 },
    { 0x61c0d041, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe2c0d043, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e0d046, SUC_lexicon_tags_37 },
    { 0x8c60d046, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c20d053, SUC_lexicon_tags_17 },
    { 0x6d80d053, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xba00d057, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa700d059, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc680d05d, SUC_lexicon_tags_2 },
    { 0xa620d05d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe0d063, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba0d067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc620d069, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e40d06e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2120d070, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde00d077, SUC_lexicon_tags_15 },
    { 0x12c0d078, SUC_lexicon_tags_23 },
    { 0x6f80d078, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb280d07b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfca0d07d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e0d080, SUC_lexicon_tags_34 },
    { 0xea40d080, SUC_lexicon_tags_37 },
    { 0xacc0d081, SUC_lexicon_tags_87 },
    { 0x1980d083, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf60d087, SUC_lexicon_tags_25 },
    { 0xc100d087, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f00d08a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e0d091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4040d096, SUC_lexicon_tags_1 },
    { 0xf040d097, SUC_lexicon_tags_2 },
    { 0x4ee0d098, SUC_lexicon_tags_17 },
    { 0x7760d099, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c40d09c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5060d0a4, SUC_lexicon_tags_87 },
    { 0x5c80d0a5, SUC_lexicon_tags_27 },
    { 0x4800d0a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d20d0ac, SUC_lexicon_tags_56 },
    { 0x9760d0ac, SUC_lexicon_tags_11 },
    { 0x6960d0ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9200d0b1, SUC_lexicon_tags_37 },
    { 0xffe0d0b2, SUC_lexicon_tags_17 },
    { 0xc5e0d0b1, SUC_lexicon_tags_47 },
    { 0x40e0d0b1, SUC_lexicon_tags_2 },
    { 0x59a0d0b3, SUC_lexicon_tags_2 },
    { 0x88e0d0b4, SUC_lexicon_tags_39 },
    { 0x9980d0b7, SUC_lexicon_tags_2 },
    { 0xd7e0d0b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb200d0ba, SUC_lexicon_tags_17 },
    { 0x2d20d0bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6460d0c3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3320d0c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c80d0cc, SUC_lexicon_tags_19 },
    { 0xbce0d0cd, SUC_lexicon_tags_48 },
    { 0x9e60d0cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e20d0d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a0d0d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe480d0dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c0d0df, SUC_lexicon_tags_37 },
    { 0x900d0e0, SUC_lexicon_tags_37 },
    { 0xad60d0df, SUC_lexicon_tags_25 },
    { 0xa900d0e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f00d0e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0d0e8, SUC_lexicon_tags_13 },
    { 0x60a0d0e9, SUC_lexicon_tags_8 },
    { 0xd3c0d0e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240d0ee, SUC_lexicon_tags_63 },
    { 0x7c80d0ee, SUC_lexicon_tags_13 },
    { 0xff60d0f0, SUC_lexicon_tags_37 },
    { 0xa2e0d0f1, SUC_lexicon_tags_40 },
    { 0xe800d0f2, SUC_lexicon_tags_66 },
    { 0xac60d0f1, SUC_lexicon_tags_47 },
    { 0x8aa0d0f1, SUC_lexicon_tags_52 },
    { 0x8f40d0f5, SUC_lexicon_tags_1 },
    { 0x86e0d0f6, SUC_lexicon_tags_1 },
    { 0x8a40d0f7, SUC_lexicon_tags_12 },
    { 0x1740d0f1, SUC_lexicon_tags_2 },
    { 0x95c0d0f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb0a0d0fb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd60d103, SUC_lexicon_tags_6 },
    { 0x5ea0d104, SUC_lexicon_tags_49 },
    { 0xdac0d105, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3760d108, SUC_lexicon_tags_38 },
    { 0xa380d108, SUC_lexicon_tags_28 },
    { 0x5b60d10a, SUC_lexicon_tags_19 },
    { 0xdd00d10b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec0d110, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x600d112, SUC_lexicon_tags_49 },
    { 0x2320d113, SUC_lexicon_tags_60 },
    { 0x6ce0d114, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d60d122, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xada0d124, SUC_lexicon_tags_19 },
    { 0xd660d125, SUC_lexicon_tags_60 },
    { 0x3440d125, SUC_lexicon_tags_6 },
    { 0xb180d124, SUC_lexicon_tags_19 },
    { 0x8740d128, SUC_lexicon_tags_23 },
    { 0x8940d129, SUC_lexicon_tags_47 },
    { 0x6ea0d12a, SUC_lexicon_tags_7 },
    { 0x8900d12b, SUC_lexicon_tags_2 },
    { 0xeee0d12b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd580d12e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xff20d131, SUC_lexicon_tags_77 },
    { 0xe0a0d132, SUC_lexicon_tags_12 },
    { 0x1260d133, SUC_lexicon_tags_749 },
    { 0x3ca0d134, SUC_lexicon_tags_37 },
    { 0xb9c0d135, SUC_lexicon_tags_37 },
    { 0xbf20d136, SUC_lexicon_tags_14 },
    { 0xf3e0d135, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa840d139, SUC_lexicon_tags_19 },
    { 0x6fe0d13a, SUC_lexicon_tags_68 },
    { 0xffe0d13a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0d13f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xffa0d141, SUC_lexicon_tags_10 },
    { 0xff20d142, SUC_lexicon_tags_37 },
    { 0x9240d142, SUC_lexicon_tags_31 },
    { 0x9f60d142, SUC_lexicon_tags_37 },
    { 0x9640d145, SUC_lexicon_tags_37 },
    { 0x52e0d145, SUC_lexicon_tags_7 },
    { 0x9400d144, SUC_lexicon_tags_28 },
    { 0x9d60d148, SUC_lexicon_tags_9 },
    { 0x6740d148, SUC_lexicon_tags_32 },
    { 0xa3e0d14a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8820d150, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c0d154, SUC_lexicon_tags_11 },
    { 0x140d154, SUC_lexicon_tags_19 },
    { 0xa000d154, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2ae0d158, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a0d15d, SUC_lexicon_tags_37 },
    { 0xe780d15d, SUC_lexicon_tags_63 },
    { 0x3160d15f, SUC_lexicon_tags_18 },
    { 0xdbc0d15d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2020d163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa0d166, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6dc0d168, SUC_lexicon_tags_17 },
    { 0xdae0d168, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd40d16d, SUC_lexicon_tags_1 },
    { 0x9a60d16d, SUC_lexicon_tags_11 },
    { 0xd2a0d16d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58e0d171, SUC_lexicon_tags_17 },
    { 0xc620d171, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac0d175, SUC_lexicon_tags_7 },
    { 0x460d176, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba0d179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a00d17b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf40d17e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d80d180, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c0d184, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e0d189, SUC_lexicon_tags_1 },
    { 0x1b60d18a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf040d18d, SUC_lexicon_tags_37 },
    { 0x9a40d18e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf760d190, SUC_lexicon_tags_0 },
    { 0xaa00d191, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa6c0d193, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9340d195, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb720d197, SUC_lexicon_tags_9 },
    { 0x71e0d198, SUC_lexicon_tags_10 },
    { 0x10e0d199, SUC_lexicon_tags_47 },
    { 0x82a0d199, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d80d19e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e80d1aa, SUC_lexicon_tags_37 },
    { 0xf1a0d1ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd460d1ba, SUC_lexicon_tags_2 },
    { 0x9280d1bb, SUC_lexicon_tags_17 },
    { 0xa960d1ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda40d1be, SUC_lexicon_tags_38 },
    { 0x10c0d1bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae0d1c5, SUC_lexicon_tags_12 },
    { 0x18e0d1c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeea0d1c8, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x52c0d1ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa60d1cc, SUC_lexicon_tags_0 },
    { 0x3780d1cd, SUC_lexicon_tags_0 },
    { 0xeb20d1cc, SUC_lexicon_tags_1 },
    { 0xe7a0d1cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22c0d1d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a0d1d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce60d1d9, SUC_lexicon_tags_7 },
    { 0x84a0d1da, SUC_lexicon_tags_38 },
    { 0xf500d1db, SUC_lexicon_tags_60 },
    { 0x7e00d1da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xff60d1df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1f20d1e1, SUC_lexicon_tags_38 },
    { 0x8400d1e1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e0d1e6, SUC_lexicon_tags_17 },
    { 0x7ca0d1e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe560d1e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7660d1eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec60d1ef, SUC_lexicon_tags_2 },
    { 0x8440d1f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3440d1f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e00d1f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de0d1fa, SUC_lexicon_tags_17 },
    { 0xb1a0d1fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da0d1ff, SUC_lexicon_tags_10 },
    { 0xdfe0d200, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce00d202, SUC_lexicon_tags_14 },
    { 0xe120d203, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd860d205, SUC_lexicon_tags_5 },
    { 0xd5a0d206, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b80d20d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a20d210, SUC_lexicon_tags_37 },
    { 0xc320d211, SUC_lexicon_tags_14 },
    { 0x7120d212, SUC_lexicon_tags_8 },
    { 0xd320d212, SUC_lexicon_tags_51 },
    { 0x1180d210, SUC_lexicon_tags_6 },
    { 0xde40d215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72a0d217, SUC_lexicon_tags_2 },
    { 0x6c0d218, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27c0d21a, SUC_lexicon_tags_1 },
    { 0xe8c0d21b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660d220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe240d224, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x140d226, SUC_lexicon_tags_23 },
    { 0x10a0d226, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d20d22c, SUC_lexicon_tags_0 },
    { 0x6a00d22c, SUC_lexicon_tags_12 },
    { 0x5b40d22c, SUC_lexicon_tags_2 },
    { 0xb180d22f, SUC_lexicon_tags_6 },
    { 0xe620d230, SUC_lexicon_tags_17 },
    { 0xad00d22e, SUC_lexicon_tags_37 },
    { 0x1500d232, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0d237, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fe0d239, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a0d23f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea0d242, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe0d246, SUC_lexicon_tags_7 },
    { 0xb80d247, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e0d24a, SUC_lexicon_tags_58 },
    { 0xf5a0d24b, SUC_lexicon_tags_55 },
    { 0x72a0d24a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0d24f, SUC_lexicon_tags_2 },
    { 0x2ac0d24f, SUC_lexicon_tags_37 },
    { 0xb3e0d251, SUC_lexicon_tags_12 },
    { 0x99e0d252, SUC_lexicon_tags_38 },
    { 0xdda0d252, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c0d25d, SUC_lexicon_tags_37 },
    { 0x6420d25e, SUC_lexicon_tags_25 },
    { 0x280d25f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4260d265, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf00d269, SUC_lexicon_tags_1 },
    { 0x400d269, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb480d26f, SUC_lexicon_tags_12 },
    { 0xe380d270, SUC_lexicon_tags_21 },
    { 0x59e0d271, SUC_lexicon_tags_33 },
    { 0xfbc0d272, SUC_lexicon_tags_17 },
    { 0x7c20d270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e0d278, SUC_lexicon_tags_9 },
    { 0x1e20d279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e0d27e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c0d281, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x3240d284, SUC_lexicon_tags_48 },
    { 0xb7c0d285, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3ea0d287, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e20d28d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d20d290, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200d295, SUC_lexicon_tags_15 },
    { 0x7b80d296, SUC_lexicon_tags_8 },
    { 0x2100d295, SUC_lexicon_tags_41 },
    { 0xc8a0d296, SUC_lexicon_tags_5 },
    { 0x3500d299, SUC_lexicon_tags_15 },
    { 0x1280d299, SUC_lexicon_tags_77 },
    { 0x4de0d295, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93c0d29d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae0d2a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd520d2a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c00d2a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd000d2ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa880d2b0, SUC_lexicon_tags_17 },
    { 0x22e0d2b1, SUC_lexicon_tags_25 },
    { 0xe000d2b1, SUC_lexicon_tags_17 },
    { 0xbbc0d2b2, SUC_lexicon_tags_19 },
    { 0xf7c0d2b4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea80d2b9, SUC_lexicon_tags_15 },
    { 0xb1a0d2ba, SUC_lexicon_tags_27 },
    { 0xf700d2bb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcee0d2bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc00d2c0, SUC_lexicon_tags_25 },
    { 0x8f20d2c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x33a0d2c3, SUC_lexicon_tags_1 },
    { 0x7920d2c4, SUC_lexicon_tags_43 },
    { 0x56a0d2c5, SUC_lexicon_tags_28 },
    { 0xd5e0d2c6, SUC_lexicon_tags_23 },
    { 0xf060d2c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a0d2ca, SUC_lexicon_tags_37 },
    { 0x95a0d2cb, SUC_lexicon_tags_160 },
    { 0xe5a0d2cc, SUC_lexicon_tags_10 },
    { 0x9020d2cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6900d2d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc60d2d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0d2db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba00d2e1, SUC_lexicon_tags_0 },
    { 0x39e0d2e2, SUC_lexicon_tags_15 },
    { 0x2240d2e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc940d2e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c0d2ec, SUC_lexicon_tags_19 },
    { 0x5120d2ed, SUC_lexicon_tags_39 },
    { 0x2da0d2ed, SUC_lexicon_tags_82 },
    { 0xdd40d2ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0d2f2, SUC_lexicon_tags_1 },
    { 0xd5c0d2f3, SUC_lexicon_tags_11 },
    { 0x6360d2f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d20d2f7, SUC_lexicon_tags_268 },
    { 0xeea0d2f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a0d2fe, SUC_lexicon_tags_17 },
    { 0x9600d2fe, SUC_lexicon_tags_17 },
    { 0x220d300, SUC_lexicon_tags_5 },
    { 0x4400d301, SUC_lexicon_tags_12 },
    { 0x8ee0d301, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xba60d304, SUC_lexicon_tags_45 },
    { 0x7680d305, SUC_lexicon_tags_11 },
    { 0x8560d305, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c0d30a, SUC_lexicon_tags_14 },
    { 0x1c40d30b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe00d30e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6460d312, SUC_lexicon_tags_6 },
    { 0xc340d313, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b60d317, SUC_lexicon_tags_1 },
    { 0x3ea0d317, SUC_lexicon_tags_11 },
    { 0xcda0d319, SUC_lexicon_tags_17 },
    { 0xec60d317, SUC_lexicon_tags_61 },
    { 0x88c0d31b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c0d31e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc800d322, SUC_lexicon_tags_26 },
    { 0x2140d323, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b20d329, SUC_lexicon_tags_1 },
    { 0xeb00d32a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xefa0d32c, SUC_lexicon_tags_60 },
    { 0xdc60d32d, SUC_lexicon_tags_11 },
    { 0xe7a0d32e, SUC_lexicon_tags_11 },
    { 0x3b20d32f, SUC_lexicon_tags_38 },
    { 0x1e40d330, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e0d337, SUC_lexicon_tags_42 },
    { 0xbd20d337, SUC_lexicon_tags_10 },
    { 0xb2c0d339, SUC_lexicon_tags_7 },
    { 0x5560d33a, SUC_lexicon_tags_72 },
    { 0x9b00d33b, SUC_lexicon_tags_2 },
    { 0xb120d339, SUC_lexicon_tags_17 },
    { 0xf8c0d33d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb960d33f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba20d343, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5240d349, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee0d34c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e00d34e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3c80d350, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1640d352, SUC_lexicon_tags_57 },
    { 0xd620d353, SUC_lexicon_tags_12 },
    { 0x5a00d354, SUC_lexicon_tags_2 },
    { 0x6740d354, SUC_lexicon_tags_25 },
    { 0x900d356, SUC_lexicon_tags_37 },
    { 0xa580d357, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d00d35b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42e0d35d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec80d364, SUC_lexicon_tags_6 },
    { 0x3840d364, SUC_lexicon_tags_12 },
    { 0x8d20d364, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xed60d369, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c20d36b, SUC_lexicon_tags_1 },
    { 0x1a40d36c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9360d36e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x380d371, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf520d377, SUC_lexicon_tags_34 },
    { 0x74e0d378, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8060d37b, SUC_lexicon_tags_7 },
    { 0x25e0d37c, SUC_lexicon_tags_2 },
    { 0x3a40d37d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2220d381, SUC_lexicon_tags_22 },
    { 0x2960d381, SUC_lexicon_tags_23 },
    { 0xeba0d382, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e0d386, SUC_lexicon_tags_26 },
    { 0x29c0d387, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1340d38c, SUC_lexicon_tags_17 },
    { 0xd8e0d38c, SUC_lexicon_tags_25 },
    { 0x7aa0d38e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa960d390, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7060d393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffa0d395, SUC_lexicon_tags_25 },
    { 0x9f40d396, SUC_lexicon_tags_49 },
    { 0x7480d397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75c0d399, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a0d39c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b20d3a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9560d3a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb3e0d3a6, SUC_lexicon_tags_14 },
    { 0x9760d3a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c20d3aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50c0d3ac, SUC_lexicon_tags_35 },
    { 0x3440d3ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa100d3af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8e0d3b1, SUC_lexicon_tags_38 },
    { 0x6b00d3b2, SUC_lexicon_tags_62 },
    { 0x99e0d3b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8680d3b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd480d3b9, SUC_lexicon_tags_6 },
    { 0x7e0d3ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1520d3bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e60d3bf, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xc9c0d3c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb00d3c5, SUC_lexicon_tags_37 },
    { 0x8080d3c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea0d3c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a80d3cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d80d3d1, SUC_lexicon_tags_38 },
    { 0xc6a0d3d2, SUC_lexicon_tags_27 },
    { 0xd8a0d3d3, SUC_lexicon_tags_17 },
    { 0x7fe0d3d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d80d3d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c60d3d9, SUC_lexicon_tags_0 },
    { 0x2380d3d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2920d3dc, SUC_lexicon_tags_2 },
    { 0xfa0d3dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe700d3df, SUC_lexicon_tags_17 },
    { 0xc200d3e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c20d3e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc0d3e4, SUC_lexicon_tags_37 },
    { 0xac80d3e5, SUC_lexicon_tags_23 },
    { 0x3c60d3e6, SUC_lexicon_tags_14 },
    { 0x2fe0d3e4, SUC_lexicon_tags_49 },
    { 0x3b40d3e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc0d3ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca80d3f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb360d3f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7780d3f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1100d3f8, SUC_lexicon_tags_83 },
    { 0x11a0d3f9, SUC_lexicon_tags_2 },
    { 0x5d20d3fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee0d3fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9500d402, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9800d40b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x220d40e, SUC_lexicon_tags_27 },
    { 0x4560d40e, SUC_lexicon_tags_2 },
    { 0x1e80d410, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a0d415, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc0d41c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9d00d41e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3300d424, SUC_lexicon_tags_25 },
    { 0xa260d425, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4940d428, SUC_lexicon_tags_7 },
    { 0x2b80d429, SUC_lexicon_tags_37 },
    { 0x8a0d428, SUC_lexicon_tags_2 },
    { 0x8c0d428, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xb240d42e, SUC_lexicon_tags_2 },
    { 0xc660d42f, SUC_lexicon_tags_48 },
    { 0xc860d430, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe100d433, SUC_lexicon_tags_14 },
    { 0xc620d433, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8420d438, SUC_lexicon_tags_25 },
    { 0x8b00d439, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c0d43c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c40d43f, SUC_lexicon_tags_77 },
    { 0x4220d440, SUC_lexicon_tags_2 },
    { 0xffa0d441, SUC_lexicon_tags_1 },
    { 0xada0d441, SUC_lexicon_tags_12 },
    { 0xfb20d441, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e0d449, SUC_lexicon_tags_127 },
    { 0x4540d44a, SUC_lexicon_tags_23 },
    { 0x4f80d44b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40a0d44d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7020d452, SUC_lexicon_tags_49 },
    { 0x1300d453, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc4c0d455, SUC_lexicon_tags_11 },
    { 0x3ce0d456, SUC_lexicon_tags_3 },
    { 0xbce0d457, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7e00d459, SUC_lexicon_tags_63 },
    { 0x6d80d45a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a40d45c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3800d45f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e80d465, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de0d46e, SUC_lexicon_tags_15 },
    { 0x2800d46f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8280d474, SUC_lexicon_tags_25 },
    { 0x5ee0d474, SUC_lexicon_tags_38 },
    { 0xfe80d475, SUC_lexicon_tags_49 },
    { 0x2e40d475, SUC_lexicon_tags_31 },
    { 0x9b60d478, SUC_lexicon_tags_39 },
    { 0x7ae0d479, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e0d47d, SUC_lexicon_tags_41 },
    { 0x3a0d47e, SUC_lexicon_tags_0 },
    { 0xd5e0d47f, SUC_lexicon_tags_6 },
    { 0x1c00d47e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e0d484, SUC_lexicon_tags_2 },
    { 0xc960d485, SUC_lexicon_tags_25 },
    { 0xfac0d486, SUC_lexicon_tags_82 },
    { 0xea20d487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba0d48d, SUC_lexicon_tags_11 },
    { 0xfec0d48d, SUC_lexicon_tags_86 },
    { 0x4280d48f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da0d496, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7960d49a, SUC_lexicon_tags_45 },
    { 0xeda0d49a, SUC_lexicon_tags_204 },
    { 0x4580d49c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc460d49e, SUC_lexicon_tags_25 },
    { 0x9360d49f, SUC_lexicon_tags_33 },
    { 0xd8e0d4a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e0d4a4, SUC_lexicon_tags_2 },
    { 0x2c60d4a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf720d4a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda0d4af, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xe1e0d4b1, SUC_lexicon_tags_1 },
    { 0x3e60d4b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4420d4b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a0d4be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa060d4c1, SUC_lexicon_tags_11 },
    { 0x1ce0d4c2, SUC_lexicon_tags_152 },
    { 0x8620d4c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc740d4cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc0c0d4d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1700d4d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4620d4d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac00d4d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e0d4dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d80d4de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a0d4eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x140d4ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2660d4f3, SUC_lexicon_tags_28 },
    { 0xfe0d4f4, SUC_lexicon_tags_14 },
    { 0x7340d4f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad80d4fa, SUC_lexicon_tags_15 },
    { 0xcd60d4fa, SUC_lexicon_tags_20 },
    { 0x3aa0d4fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9960d501, SUC_lexicon_tags_39 },
    { 0x7e40d501, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb00d508, SUC_lexicon_tags_7 },
    { 0x13a0d509, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73e0d50b, SUC_lexicon_tags_17 },
    { 0x8ea0d50b, SUC_lexicon_tags_87 },
    { 0x6140d50b, SUC_lexicon_tags_23 },
    { 0xae0d50e, SUC_lexicon_tags_7 },
    { 0xba40d50f, SUC_lexicon_tags_25 },
    { 0xd160d50f, SUC_lexicon_tags_2 },
    { 0xaa00d50e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5220d513, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46e0d515, SUC_lexicon_tags_2 },
    { 0xb040d515, SUC_lexicon_tags_6 },
    { 0xf140d517, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c0d51a, SUC_lexicon_tags_7 },
    { 0x8420d51b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xcae0d51d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb3e0d51f, SUC_lexicon_tags_15 },
    { 0x1fe0d520, SUC_lexicon_tags_49 },
    { 0xc9c0d521, SUC_lexicon_tags_25 },
    { 0xb780d520, SUC_lexicon_tags_47 },
    { 0x53c0d520, SUC_lexicon_tags_29 },
    { 0x8320d522, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f60d526, SUC_lexicon_tags_60 },
    { 0x18e0d526, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52c0d529, SUC_lexicon_tags_25 },
    { 0x1f00d529, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb680d52c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c60d532, SUC_lexicon_tags_7 },
    { 0x2300d532, SUC_lexicon_tags_28 },
    { 0xeba0d534, SUC_lexicon_tags_17 },
    { 0xad80d535, SUC_lexicon_tags_41 },
    { 0x44c0d536, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a0d539, SUC_lexicon_tags_46 },
    { 0x84a0d53a, SUC_lexicon_tags_7 },
    { 0x5d00d53a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99a0d53d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xad20d540, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa820d542, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a0d547, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e00d549, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0d54f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1300d551, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa720d554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf340d557, SUC_lexicon_tags_1 },
    { 0x9040d557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb860d55c, SUC_lexicon_tags_0 },
    { 0x8ac0d55c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x87e0d55f, SUC_lexicon_tags_15 },
    { 0xa640d560, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae0d563, SUC_lexicon_tags_12 },
    { 0x5f20d564, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e0d568, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b00d572, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e60d577, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2c80d579, SUC_lexicon_tags_0 },
    { 0x3bc0d579, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa4c0d57c, SUC_lexicon_tags_12 },
    { 0xd520d57d, SUC_lexicon_tags_39 },
    { 0x2360d57d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda0d581, SUC_lexicon_tags_17 },
    { 0xd1a0d582, SUC_lexicon_tags_66 },
    { 0x82a0d582, SUC_lexicon_tags_32 },
    { 0xfec0d584, SUC_lexicon_tags_14 },
    { 0xf960d584, SUC_lexicon_tags_4 },
    { 0x4d20d582, SUC_lexicon_tags_11 },
    { 0xca40d586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3480d589, SUC_lexicon_tags_11 },
    { 0xbcc0d589, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0d58d, SUC_lexicon_tags_87 },
    { 0xc320d58e, SUC_lexicon_tags_60 },
    { 0xb220d58d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8420d591, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e0d59a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x78c0d59c, SUC_lexicon_tags_120 },
    { 0x3a0d59d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf60d59f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0d5a3, SUC_lexicon_tags_8 },
    { 0x7e0d5a3, SUC_lexicon_tags_37 },
    { 0x1d20d5a5, SUC_lexicon_tags_31 },
    { 0x4920d5a4, SUC_lexicon_tags_8 },
    { 0x1340d5a6, SUC_lexicon_tags_2 },
    { 0x9460d5a8, SUC_lexicon_tags_11 },
    { 0xa360d5a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd640d5ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f80d5b1, SUC_lexicon_tags_10 },
    { 0x4180d5b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf300d5b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9080d5bb, SUC_lexicon_tags_47 },
    { 0x7360d5bb, SUC_lexicon_tags_30 },
    { 0xaf40d5bd, SUC_lexicon_tags_15 },
    { 0x48c0d5be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf80d5c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb880d5c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6c0d5ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9de0d5cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da0d5d3, SUC_lexicon_tags_25 },
    { 0x8ee0d5d3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xcec0d5d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a00d5d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda00d5db, SUC_lexicon_tags_63 },
    { 0x1fa0d5db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc0d5e0, SUC_lexicon_tags_37 },
    { 0x3f40d5e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf440d5e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8aa0d5e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd00d5eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4020d5f3, SUC_lexicon_tags_2 },
    { 0x2b80d5f4, SUC_lexicon_tags_60 },
    { 0x2200d5f3, SUC_lexicon_tags_25 },
    { 0x51c0d5f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe020d5fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60d5fd, SUC_lexicon_tags_27 },
    { 0xfb00d5fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2900d601, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a0d605, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe640d607, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc860d60a, SUC_lexicon_tags_70 },
    { 0xdc00d60b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x660d60d, SUC_lexicon_tags_6 },
    { 0xc280d60e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa40d612, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7580d615, SUC_lexicon_tags_12 },
    { 0xc520d615, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce80d618, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d20d61a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e80d61e, SUC_lexicon_tags_10 },
    { 0x1740d61f, SUC_lexicon_tags_11 },
    { 0x8020d620, SUC_lexicon_tags_1 },
    { 0x3da0d61f, SUC_lexicon_tags_6 },
    { 0x6ce0d622, SUC_lexicon_tags_26 },
    { 0x7ea0d622, SUC_lexicon_tags_25 },
    { 0xb8a0d621, SUC_lexicon_tags_28 },
    { 0x4780d625, SUC_lexicon_tags_11 },
    { 0x380d625, SUC_lexicon_tags_25 },
    { 0x6800d621, SUC_lexicon_tags_150 },
    { 0xb6e0d623, SUC_lexicon_tags_6 },
    { 0x4100d626, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd920d636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaa0d638, SUC_lexicon_tags_11 },
    { 0x2ce0d639, SUC_lexicon_tags_14 },
    { 0x8fa0d63a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e0d63f, SUC_lexicon_tags_6 },
    { 0xd620d640, SUC_lexicon_tags_3 },
    { 0x86a0d641, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeaa0d643, SUC_lexicon_tags_23 },
    { 0x5a20d644, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe640d646, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160d64a, SUC_lexicon_tags_3 },
    { 0x1860d64b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9200d650, SUC_lexicon_tags_17 },
    { 0x9600d651, SUC_lexicon_tags_6 },
    { 0xf420d652, SUC_lexicon_tags_15 },
    { 0xfb40d652, SUC_lexicon_tags_33 },
    { 0x1ee0d652, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e0d65b, SUC_lexicon_tags_13 },
    { 0x6480d65c, SUC_lexicon_tags_26 },
    { 0x2d20d65d, SUC_lexicon_tags_43 },
    { 0x5f20d65c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfac0d660, SUC_lexicon_tags_25 },
    { 0x2e60d661, SUC_lexicon_tags_44 },
    { 0x4a80d662, SUC_lexicon_tags_8 },
    { 0xca80d663, SUC_lexicon_tags_7 },
    { 0xcbe0d663, SUC_lexicon_tags_15 },
    { 0xdb40d663, SUC_lexicon_tags_8 },
    { 0x38a0d660, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf000d668, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10a0d66a, SUC_lexicon_tags_723 },
    { 0xa780d66a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a0d673, SUC_lexicon_tags_26 },
    { 0x17c0d674, SUC_lexicon_tags_38 },
    { 0xf8c0d675, SUC_lexicon_tags_7 },
    { 0x69e0d676, SUC_lexicon_tags_63 },
    { 0x97a0d677, SUC_lexicon_tags_30 },
    { 0x9180d677, SUC_lexicon_tags_15 },
    { 0xd678, SUC_lexicon_tags_1 },
    { 0xe480d676, SUC_lexicon_tags_7 },
    { 0x6260d67b, SUC_lexicon_tags_30 },
    { 0xac00d67b, SUC_lexicon_tags_26 },
    { 0x8ba0d67b, SUC_lexicon_tags_19 },
    { 0x8aa0d67d, SUC_lexicon_tags_31 },
    { 0xcc20d67f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x89e0d681, SUC_lexicon_tags_17 },
    { 0xaf20d681, SUC_lexicon_tags_12 },
    { 0x6600d682, SUC_lexicon_tags_56 },
    { 0x6700d684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf80d687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d40d68a, SUC_lexicon_tags_38 },
    { 0xc380d68a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a0d68e, SUC_lexicon_tags_19 },
    { 0x8240d68f, SUC_lexicon_tags_31 },
    { 0x3c0d690, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe540d694, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7d40d696, SUC_lexicon_tags_25 },
    { 0xde40d697, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9900d69b, SUC_lexicon_tags_0 },
    { 0x3f40d69c, SUC_lexicon_tags_4 },
    { 0x8120d69c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e60d6a1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee20d6aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe0d6b0, SUC_lexicon_tags_17 },
    { 0x3ea0d6b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa0d6b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c0d6bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcec0d6bf, SUC_lexicon_tags_47 },
    { 0x61a0d6bf, SUC_lexicon_tags_27 },
    { 0x9a0d6c1, SUC_lexicon_tags_10 },
    { 0x59a0d6bf, SUC_lexicon_tags_25 },
    { 0x5040d6c3, SUC_lexicon_tags_27 },
    { 0xece0d6c3, SUC_lexicon_tags_14 },
    { 0xd660d6c0, SUC_lexicon_tags_43 },
    { 0xb800d6c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320d6cc, SUC_lexicon_tags_31 },
    { 0xa100d6cc, SUC_lexicon_tags_11 },
    { 0x2e20d6ce, SUC_lexicon_tags_12 },
    { 0xe140d6ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c40d6d4, SUC_lexicon_tags_15 },
    { 0x1180d6d4, SUC_lexicon_tags_12 },
    { 0xaee0d6d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb40d6da, SUC_lexicon_tags_47 },
    { 0x2dc0d6da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5480d6dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ea0d6df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c0d6e4, SUC_lexicon_tags_15 },
    { 0x7de0d6e5, SUC_lexicon_tags_17 },
    { 0x2e40d6e4, SUC_lexicon_tags_17 },
    { 0x2fe0d6e4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e0d6ea, SUC_lexicon_tags_23 },
    { 0x33a0d6eb, SUC_lexicon_tags_25 },
    { 0x1120d6ec, SUC_lexicon_tags_25 },
    { 0xf9a0d6ed, SUC_lexicon_tags_269 },
    { 0x93e0d6ee, SUC_lexicon_tags_30 },
    { 0x9ee0d6ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4520d6f2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4380d6f5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c0d6f9, SUC_lexicon_tags_63 },
    { 0x7f00d6fa, SUC_lexicon_tags_3 },
    { 0x28c0d6fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80d6ff, SUC_lexicon_tags_21 },
    { 0x8f20d700, SUC_lexicon_tags_15 },
    { 0x4f20d701, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7840d706, SUC_lexicon_tags_32 },
    { 0x18e0d707, SUC_lexicon_tags_17 },
    { 0x2b80d707, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc0d70b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6660d710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a0d714, SUC_lexicon_tags_27 },
    { 0x5f00d715, SUC_lexicon_tags_1 },
    { 0x3480d716, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21c0d718, SUC_lexicon_tags_53 },
    { 0xdae0d718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe640d720, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf440d726, SUC_lexicon_tags_87 },
    { 0xc580d726, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd400d72c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9520d72f, SUC_lexicon_tags_1 },
    { 0xb980d730, SUC_lexicon_tags_236 },
    { 0xe900d731, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7100d734, SUC_lexicon_tags_1 },
    { 0x1c60d735, SUC_lexicon_tags_12 },
    { 0xbbe0d734, SUC_lexicon_tags_17 },
    { 0xd5e0d737, SUC_lexicon_tags_17 },
    { 0x7080d735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e0d73c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa020d73f, SUC_lexicon_tags_12 },
    { 0xbc80d740, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb260d742, SUC_lexicon_tags_10 },
    { 0x7520d743, SUC_lexicon_tags_17 },
    { 0xbf40d743, SUC_lexicon_tags_17 },
    { 0x8820d743, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a0d74a, SUC_lexicon_tags_0 },
    { 0x8c60d74b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc0d74f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a0d753, SUC_lexicon_tags_33 },
    { 0x79c0d754, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd500d756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa280d758, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe60d75a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3060d760, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5400d762, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1480d765, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c0d76a, SUC_lexicon_tags_17 },
    { 0x4c40d76a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e0d76e, SUC_lexicon_tags_15 },
    { 0x97a0d76f, SUC_lexicon_tags_17 },
    { 0xa940d770, SUC_lexicon_tags_23 },
    { 0xd260d76f, SUC_lexicon_tags_37 },
    { 0x2a40d76f, SUC_lexicon_tags_62 },
    { 0xad80d76f, SUC_lexicon_tags_18 },
    { 0x90e0d774, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab80d778, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8b60d77a, SUC_lexicon_tags_25 },
    { 0x9be0d77b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7a0d77d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd000d781, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c0d784, SUC_lexicon_tags_2 },
    { 0x6f80d785, SUC_lexicon_tags_37 },
    { 0xe6c0d785, SUC_lexicon_tags_11 },
    { 0xad40d784, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f20d789, SUC_lexicon_tags_1 },
    { 0xe9a0d78a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x7e60d78c, SUC_lexicon_tags_17 },
    { 0x90c0d78c, SUC_lexicon_tags_46 },
    { 0xe260d78e, SUC_lexicon_tags_30 },
    { 0x61e0d78e, SUC_lexicon_tags_47 },
    { 0xc0a0d790, SUC_lexicon_tags_17 },
    { 0xee20d78d, SUC_lexicon_tags_15 },
    { 0x2d60d78d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf720d794, SUC_lexicon_tags_5 },
    { 0xf420d794, SUC_lexicon_tags_25 },
    { 0xafe0d796, SUC_lexicon_tags_31 },
    { 0x5460d795, SUC_lexicon_tags_10 },
    { 0x3020d798, SUC_lexicon_tags_1 },
    { 0xe80d799, SUC_lexicon_tags_84 },
    { 0xf760d79a, SUC_lexicon_tags_17 },
    { 0xdd60d79a, SUC_lexicon_tags_23 },
    { 0xdf00d79b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9060d79f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c0d7a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a0d7aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a0d7b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e40d7bd, SUC_lexicon_tags_15 },
    { 0x3680d7bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b20d7c0, SUC_lexicon_tags_25 },
    { 0xb0a0d7c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb860d7c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7040d7c8, SUC_lexicon_tags_46 },
    { 0xb0a0d7c9, SUC_lexicon_tags_1 },
    { 0x6bc0d7c9, SUC_lexicon_tags_29 },
    { 0xaa00d7cb, SUC_lexicon_tags_29 },
    { 0x4780d7c8, SUC_lexicon_tags_124 },
    { 0x54a0d7c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x74e0d7cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe0d7d3, SUC_lexicon_tags_15 },
    { 0x5480d7d4, SUC_lexicon_tags_43 },
    { 0x4180d7d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2760d7d7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8be0d7d9, SUC_lexicon_tags_12 },
    { 0xb660d7d9, SUC_lexicon_tags_23 },
    { 0xee80d7d9, SUC_lexicon_tags_298 },
    { 0x27c0d7dc, SUC_lexicon_tags_26 },
    { 0xa340d7dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c0d7e1, SUC_lexicon_tags_2 },
    { 0x3d00d7e2, SUC_lexicon_tags_47 },
    { 0x360d7e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2740d7e5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1120d7e9, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc60d7ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e00d7ee, SUC_lexicon_tags_60 },
    { 0x96a0d7ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e0d7f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2220d7f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f60d7fc, SUC_lexicon_tags_27 },
    { 0x2fc0d7fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5e0d7ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd340d801, SUC_lexicon_tags_15 },
    { 0x82c0d802, SUC_lexicon_tags_37 },
    { 0x980d803, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1f00d805, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb40d80d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20d814, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda00d817, SUC_lexicon_tags_25 },
    { 0x2320d818, SUC_lexicon_tags_25 },
    { 0x5a20d817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb060d81d, SUC_lexicon_tags_33 },
    { 0x3260d81e, SUC_lexicon_tags_53 },
    { 0x8f40d81e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c80d824, SUC_lexicon_tags_31 },
    { 0x4b60d824, SUC_lexicon_tags_55 },
    { 0xca20d824, SUC_lexicon_tags_9 },
    { 0xf380d827, SUC_lexicon_tags_11 },
    { 0x9360d828, SUC_lexicon_tags_4 },
    { 0xc960d829, SUC_lexicon_tags_19 },
    { 0x9e40d829, SUC_lexicon_tags_4 },
    { 0x240d82b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c0d82f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe80d831, SUC_lexicon_tags_32 },
    { 0xd7c0d832, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e60d835, SUC_lexicon_tags_2 },
    { 0x3ea0d836, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e0d83a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4600d83c, SUC_lexicon_tags_2 },
    { 0x7f40d83c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda40d841, SUC_lexicon_tags_17 },
    { 0xc2e0d842, SUC_lexicon_tags_27 },
    { 0x200d842, SUC_lexicon_tags_16 },
    { 0x79c0d843, SUC_lexicon_tags_11 },
    { 0xb960d842, SUC_lexicon_tags_45 },
    { 0xbf00d846, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e0d84c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60d84f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc8e0d851, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3700d854, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9a0d856, SUC_lexicon_tags_0 },
    { 0x8a40d857, SUC_lexicon_tags_11 },
    { 0x33c0d858, SUC_lexicon_tags_134 },
    { 0xcee0d859, SUC_lexicon_tags_65 },
    { 0x6680d859, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a0d85d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc80d860, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a0d864, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf720d868, SUC_lexicon_tags_38 },
    { 0x8b80d869, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d00d86d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac0d870, SUC_lexicon_tags_17 },
    { 0x1be0d870, SUC_lexicon_tags_2 },
    { 0x4b60d872, SUC_lexicon_tags_6 },
    { 0x5a80d873, SUC_lexicon_tags_2 },
    { 0x56e0d874, SUC_lexicon_tags_33 },
    { 0xcb00d875, SUC_lexicon_tags_37 },
    { 0x5d80d875, SUC_lexicon_tags_15 },
    { 0xc40d877, SUC_lexicon_tags_23 },
    { 0x540d878, SUC_lexicon_tags_30 },
    { 0x6680d877, SUC_lexicon_tags_15 },
    { 0xe220d87a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b80d87c, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d80d881, SUC_lexicon_tags_14 },
    { 0x4e00d882, SUC_lexicon_tags_37 },
    { 0x4860d883, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0d889, SUC_lexicon_tags_1 },
    { 0x4880d889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc500d88c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5be0d88e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb100d894, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7920d898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9440d89b, SUC_lexicon_tags_12 },
    { 0x45c0d89c, SUC_lexicon_tags_16 },
    { 0x8420d89d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7720d89f, SUC_lexicon_tags_37 },
    { 0x6e20d89f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d00d8a3, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x1160d8a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8780d8a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdae0d8a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeaa0d8ab, SUC_lexicon_tags_23 },
    { 0x1580d8ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e20d8af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e00d8b2, SUC_lexicon_tags_17 },
    { 0xa000d8b2, SUC_lexicon_tags_6 },
    { 0x6440d8b4, SUC_lexicon_tags_37 },
    { 0xfec0d8b2, SUC_lexicon_tags_11 },
    { 0x3260d8b5, SUC_lexicon_tags_25 },
    { 0xcb00d8b4, SUC_lexicon_tags_30 },
    { 0x9840d8b5, SUC_lexicon_tags_0 },
    { 0x73c0d8b9, SUC_lexicon_tags_12 },
    { 0x7320d8b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb500d8c3, SUC_lexicon_tags_25 },
    { 0x9e20d8c4, SUC_lexicon_tags_61 },
    { 0x6400d8c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f20d8c7, SUC_lexicon_tags_8 },
    { 0x1560d8c8, SUC_lexicon_tags_37 },
    { 0x4180d8c8, SUC_lexicon_tags_236 },
    { 0x54a0d8c7, SUC_lexicon_tags_12 },
    { 0x76a0d8ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb720d8cf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea0d8d5, SUC_lexicon_tags_7 },
    { 0x5b60d8d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8520d8d8, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa0d8dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb60d8de, SUC_lexicon_tags_7 },
    { 0x1e00d8df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8280d8e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5240d8e8, SUC_lexicon_tags_23 },
    { 0xcce0d8e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ee0d8eb, SUC_lexicon_tags_22 },
    { 0xfb60d8ec, SUC_lexicon_tags_2 },
    { 0xccc0d8ed, SUC_lexicon_tags_29 },
    { 0xa580d8ee, SUC_lexicon_tags_14 },
    { 0xe9a0d8ee, SUC_lexicon_tags_25 },
    { 0xa760d8f0, SUC_lexicon_tags_5 },
    { 0x5700d8f1, SUC_lexicon_tags_17 },
    { 0xe800d8f2, SUC_lexicon_tags_37 },
    { 0x97e0d8f3, SUC_lexicon_tags_46 },
    { 0x48c0d8f0, SUC_lexicon_tags_19 },
    { 0x57a0d8f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe520d8fe, SUC_lexicon_tags_27 },
    { 0x9100d8ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1020d904, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c0d90b, SUC_lexicon_tags_15 },
    { 0xc4c0d90b, SUC_lexicon_tags_1 },
    { 0x4e40d90d, SUC_lexicon_tags_7 },
    { 0x7e40d90e, SUC_lexicon_tags_30 },
    { 0x4820d90e, SUC_lexicon_tags_15 },
    { 0xf7a0d90d, SUC_lexicon_tags_30 },
    { 0x49a0d911, SUC_lexicon_tags_12 },
    { 0xd380d912, SUC_lexicon_tags_108 },
    { 0xad20d912, SUC_lexicon_tags_87 },
    { 0xd660d911, SUC_lexicon_tags_30 },
    { 0x24e0d915, SUC_lexicon_tags_55 },
    { 0xa460d914, SUC_lexicon_tags_7 },
    { 0xe980d917, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd80d919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3060d91c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c0d91f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1a0d921, SUC_lexicon_tags_38 },
    { 0x2140d922, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe600d924, SUC_lexicon_tags_14 },
    { 0xe9c0d925, SUC_lexicon_tags_71 },
    { 0xf1a0d925, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc5a0d928, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6120d92b, SUC_lexicon_tags_25 },
    { 0x46c0d92c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e0d92f, SUC_lexicon_tags_37 },
    { 0xca40d930, SUC_lexicon_tags_1 },
    { 0x4a0d930, SUC_lexicon_tags_27 },
    { 0xb1a0d931, SUC_lexicon_tags_17 },
    { 0x3c20d933, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00d938, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6e0d93a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c0d940, SUC_lexicon_tags_12 },
    { 0x7360d941, SUC_lexicon_tags_0 },
    { 0xada0d941, SUC_lexicon_tags_42 },
    { 0x60d943, SUC_lexicon_tags_7 },
    { 0x960d943, SUC_lexicon_tags_38 },
    { 0x7300d943, SUC_lexicon_tags_15 },
    { 0x680d946, SUC_lexicon_tags_60 },
    { 0xadc0d947, SUC_lexicon_tags_25 },
    { 0xb6a0d948, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6620d94a, SUC_lexicon_tags_47 },
    { 0xee60d94b, SUC_lexicon_tags_19 },
    { 0x16e0d94b, SUC_lexicon_tags_351 },
    { 0xc4e0d94d, SUC_lexicon_tags_25 },
    { 0x8020d94e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe000d950, SUC_lexicon_tags_12 },
    { 0xc280d951, SUC_lexicon_tags_0 },
    { 0x6e80d951, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f20d954, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xdbe0d956, SUC_lexicon_tags_7 },
    { 0x2e80d956, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf80d959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e0d95c, SUC_lexicon_tags_1 },
    { 0xfe00d95c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc120d961, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c0d965, SUC_lexicon_tags_46 },
    { 0xad60d966, SUC_lexicon_tags_14 },
    { 0x6e60d967, SUC_lexicon_tags_4 },
    { 0x6860d966, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a0d970, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7b00d972, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da0d975, SUC_lexicon_tags_57 },
    { 0xa960d975, SUC_lexicon_tags_26 },
    { 0xfe60d977, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba0d97a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a0d97d, SUC_lexicon_tags_77 },
    { 0x67c0d97d, SUC_lexicon_tags_11 },
    { 0xf520d97d, SUC_lexicon_tags_29 },
    { 0xf6a0d980, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60d982, SUC_lexicon_tags_18 },
    { 0x69e0d983, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0a0d985, SUC_lexicon_tags_64 },
    { 0x5620d986, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc1e0d988, SUC_lexicon_tags_27 },
    { 0xdda0d988, SUC_lexicon_tags_30 },
    { 0x1160d989, SUC_lexicon_tags_7 },
    { 0x9880d98a, SUC_lexicon_tags_26 },
    { 0x6120d98c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2020d98e, SUC_lexicon_tags_44 },
    { 0x46e0d98f, SUC_lexicon_tags_29 },
    { 0x7980d990, SUC_lexicon_tags_23 },
    { 0xf440d991, SUC_lexicon_tags_7 },
    { 0x47e0d992, SUC_lexicon_tags_30 },
    { 0x9560d990, SUC_lexicon_tags_14 },
    { 0xb360d992, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd820d996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a80d99c, SUC_lexicon_tags_28 },
    { 0xba80d99d, SUC_lexicon_tags_19 },
    { 0x3880d99e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb9a0d9a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4160d9a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7340d9a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e00d9aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c0d9b0, SUC_lexicon_tags_1 },
    { 0xc660d9b0, SUC_lexicon_tags_27 },
    { 0x9c00d9b0, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xe500d9b4, SUC_lexicon_tags_2 },
    { 0x4b20d9b5, SUC_lexicon_tags_25 },
    { 0x3260d9b5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xab20d9b9, SUC_lexicon_tags_38 },
    { 0xea40d9ba, SUC_lexicon_tags_33 },
    { 0x24e0d9b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc20d9c7, SUC_lexicon_tags_11 },
    { 0x3fa0d9c8, SUC_lexicon_tags_71 },
    { 0x76a0d9c8, SUC_lexicon_tags_13 },
    { 0x4280d9c8, SUC_lexicon_tags_25 },
    { 0x220d9cb, SUC_lexicon_tags_17 },
    { 0x860d9cc, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e0d9d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc620d9d4, SUC_lexicon_tags_27 },
    { 0xc3c0d9d5, SUC_lexicon_tags_0 },
    { 0x20e0d9d4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9520d9e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf00d9e5, SUC_lexicon_tags_65 },
    { 0x94e0d9e5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4440d9e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf60d9ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be0d9f1, SUC_lexicon_tags_19 },
    { 0xbc80d9f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec0d9f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd00d9f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0d9fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9480da02, SUC_lexicon_tags_12 },
    { 0x8e20da03, SUC_lexicon_tags_37 },
    { 0xa1c0da02, SUC_lexicon_tags_0 },
    { 0x6da0da02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0da08, SUC_lexicon_tags_19 },
    { 0xbac0da09, SUC_lexicon_tags_37 },
    { 0x8c60da08, SUC_lexicon_tags_47 },
    { 0x1320da0b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3f00da0d, SUC_lexicon_tags_45 },
    { 0x3a0da0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc0da13, SUC_lexicon_tags_65 },
    { 0x51e0da14, SUC_lexicon_tags_4 },
    { 0xcd60da15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8840da1b, SUC_lexicon_tags_30 },
    { 0xf960da1c, SUC_lexicon_tags_6 },
    { 0xe620da1d, SUC_lexicon_tags_2 },
    { 0xd800da1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a00da20, SUC_lexicon_tags_47 },
    { 0x6c20da21, SUC_lexicon_tags_25 },
    { 0xa3e0da22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f80da25, SUC_lexicon_tags_7 },
    { 0xc20da25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa600da29, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d80da2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6680da31, SUC_lexicon_tags_25 },
    { 0x3d00da32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3520da35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xace0da37, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9260da3d, SUC_lexicon_tags_12 },
    { 0x22e0da3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc0da41, SUC_lexicon_tags_25 },
    { 0xdb20da42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5860da45, SUC_lexicon_tags_25 },
    { 0xada0da45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a0da4f, SUC_lexicon_tags_68 },
    { 0x64a0da50, SUC_lexicon_tags_1 },
    { 0x7980da50, SUC_lexicon_tags_2 },
    { 0x97e0da52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe0da56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29a0da58, SUC_lexicon_tags_1 },
    { 0x92e0da59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb400da5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e0da62, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe0da6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7980da71, SUC_lexicon_tags_1 },
    { 0xfb80da72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf00da74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4440da76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30c0da78, SUC_lexicon_tags_6 },
    { 0xb1a0da79, SUC_lexicon_tags_26 },
    { 0xb420da7a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d60da7d, SUC_lexicon_tags_2 },
    { 0xa400da7e, SUC_lexicon_tags_45 },
    { 0x8180da7f, SUC_lexicon_tags_11 },
    { 0x96e0da7e, SUC_lexicon_tags_2 },
    { 0x4c80da81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f00da83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe0da8a, SUC_lexicon_tags_37 },
    { 0x4ba0da8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c0da90, SUC_lexicon_tags_8 },
    { 0x180da90, SUC_lexicon_tags_26 },
    { 0x82c0da92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a0da98, SUC_lexicon_tags_23 },
    { 0xe7c0da98, SUC_lexicon_tags_23 },
    { 0x2300da99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa060da9d, SUC_lexicon_tags_17 },
    { 0xf6c0da9d, SUC_lexicon_tags_12 },
    { 0x4b40da9f, SUC_lexicon_tags_37 },
    { 0xf260da9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3420daa4, SUC_lexicon_tags_43 },
    { 0x3680daa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4420daa8, SUC_lexicon_tags_49 },
    { 0xab80daa8, SUC_lexicon_tags_129 },
    { 0xdea0daaa, SUC_lexicon_tags_23 },
    { 0x2080daab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500dab3, SUC_lexicon_tags_60 },
    { 0x6cc0dab4, SUC_lexicon_tags_92 },
    { 0xe720dab5, SUC_lexicon_tags_25 },
    { 0xa180dab6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1300dab9, SUC_lexicon_tags_77 },
    { 0x3640daba, SUC_lexicon_tags_6 },
    { 0x8ea0dabb, SUC_lexicon_tags_21 },
    { 0xee0dabc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8540dabe, SUC_lexicon_tags_106 },
    { 0xa9a0dabf, SUC_lexicon_tags_0 },
    { 0x91e0dabf, SUC_lexicon_tags_37 },
    { 0x6440dac1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4500dac8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea0dacc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c60dad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x180dad3, SUC_lexicon_tags_38 },
    { 0x44a0dad3, SUC_lexicon_tags_7 },
    { 0x1ae0dad3, SUC_lexicon_tags_19 },
    { 0xd9a0dad4, SUC_lexicon_tags_47 },
    { 0x21a0dad3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe80dada, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe9c0dadc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf080dae0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6160dae2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c0dae5, SUC_lexicon_tags_33 },
    { 0x5b80dae6, SUC_lexicon_tags_32 },
    { 0xbd60dae5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x540daea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0daed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbf60daef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e0daf9, SUC_lexicon_tags_11 },
    { 0x2cc0dafa, SUC_lexicon_tags_55 },
    { 0x9020dafb, SUC_lexicon_tags_138 },
    { 0x6d60dafb, SUC_lexicon_tags_7 },
    { 0x4fa0dafd, SUC_lexicon_tags_11 },
    { 0x7c20dafc, SUC_lexicon_tags_2 },
    { 0x8060dafe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa720db03, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1380db05, SUC_lexicon_tags_7 },
    { 0x5e80db06, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb320db0b, SUC_lexicon_tags_39 },
    { 0x9660db0c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x33e0db0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f00db10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e40db17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd40db2b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d20db30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf860db34, SUC_lexicon_tags_13 },
    { 0xdbe0db35, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8660db3c, SUC_lexicon_tags_30 },
    { 0x2b60db3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6420db42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2380db49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd9a0db4b, SUC_lexicon_tags_37 },
    { 0xff00db4b, SUC_lexicon_tags_7 },
    { 0x6220db4b, SUC_lexicon_tags_17 },
    { 0x5040db4c, SUC_lexicon_tags_6 },
    { 0xa3e0db4d, SUC_lexicon_tags_6 },
    { 0xdc40db50, SUC_lexicon_tags_12 },
    { 0x7520db50, SUC_lexicon_tags_0 },
    { 0x1940db51, SUC_lexicon_tags_45 },
    { 0x67a0db4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a0db60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8200db63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f20db6a, SUC_lexicon_tags_11 },
    { 0x260db6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3060db6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2100db72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c00db79, SUC_lexicon_tags_32 },
    { 0xfd00db79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1020db7c, SUC_lexicon_tags_17 },
    { 0x52a0db7c, SUC_lexicon_tags_2 },
    { 0x83c0db7e, SUC_lexicon_tags_37 },
    { 0xec0db7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8e0db81, SUC_lexicon_tags_25 },
    { 0xf9c0db82, SUC_lexicon_tags_30 },
    { 0x4ba0db83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc20db85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9120db88, SUC_lexicon_tags_11 },
    { 0x2580db89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f40db8d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c80db90, SUC_lexicon_tags_12 },
    { 0x51a0db91, SUC_lexicon_tags_16 },
    { 0xb500db92, SUC_lexicon_tags_15 },
    { 0x3ac0db92, SUC_lexicon_tags_63 },
    { 0x2c40db91, SUC_lexicon_tags_63 },
    { 0x3e0db95, SUC_lexicon_tags_15 },
    { 0x13e0db96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4460db99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a0dba0, SUC_lexicon_tags_11 },
    { 0x5080dba0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xec0dba3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x580dba5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe7a0dba7, SUC_lexicon_tags_45 },
    { 0x3840dba8, SUC_lexicon_tags_45 },
    { 0xd540dba8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb320dbae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a20dbb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa440dbb9, SUC_lexicon_tags_0 },
    { 0x9f20dbba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe00dbbe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc0dbc1, SUC_lexicon_tags_63 },
    { 0xef60dbc1, SUC_lexicon_tags_37 },
    { 0xcf80dbc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe200dbc5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b60dbc8, SUC_lexicon_tags_15 },
    { 0xe8e0dbc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3a0dbcb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfdc0dbcd, SUC_lexicon_tags_1 },
    { 0xa8e0dbcd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20dbd4, SUC_lexicon_tags_27 },
    { 0xcce0dbd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce0dbd8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3440dbdc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a0dbe1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4100dbe3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba0dbe7, SUC_lexicon_tags_7 },
    { 0x4c60dbe8, SUC_lexicon_tags_1 },
    { 0xbea0dbe9, SUC_lexicon_tags_25 },
    { 0xa2c0dbea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe0dbee, SUC_lexicon_tags_35 },
    { 0x3080dbef, SUC_lexicon_tags_60 },
    { 0x1280dbef, SUC_lexicon_tags_17 },
    { 0xd6c0dbef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca0dbf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c0dbf8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdc20dbfa, SUC_lexicon_tags_37 },
    { 0xed20dbfb, SUC_lexicon_tags_25 },
    { 0xad00dbfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x700dbff, SUC_lexicon_tags_25 },
    { 0xe540dc00, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6340dc03, SUC_lexicon_tags_25 },
    { 0x66e0dc04, SUC_lexicon_tags_38 },
    { 0x9a60dc05, SUC_lexicon_tags_49 },
    { 0x7800dc06, SUC_lexicon_tags_49 },
    { 0xbc60dc05, SUC_lexicon_tags_47 },
    { 0x4860dc03, SUC_lexicon_tags_26 },
    { 0x1ca0dc09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc0dc0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba0dc14, SUC_lexicon_tags_87 },
    { 0x1a40dc15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc320dc1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0dc1d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d20dc20, SUC_lexicon_tags_11 },
    { 0x3f40dc21, SUC_lexicon_tags_29 },
    { 0x5140dc22, SUC_lexicon_tags_27 },
    { 0xea20dc23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xece0dc25, SUC_lexicon_tags_25 },
    { 0x1c60dc25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c80dc29, SUC_lexicon_tags_1 },
    { 0x3720dc2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9140dc2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf020dc33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3fc0dc35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a0dc40, SUC_lexicon_tags_17 },
    { 0x6c00dc40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c0dc46, SUC_lexicon_tags_28 },
    { 0x55e0dc47, SUC_lexicon_tags_2 },
    { 0x6880dc48, SUC_lexicon_tags_27 },
    { 0xe0c0dc46, SUC_lexicon_tags_19 },
    { 0x9480dc4a, SUC_lexicon_tags_29 },
    { 0xa340dc4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdbc0dc4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e0dc53, SUC_lexicon_tags_6 },
    { 0x99e0dc54, SUC_lexicon_tags_11 },
    { 0xffa0dc54, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdde0dc57, SUC_lexicon_tags_25 },
    { 0x5600dc58, SUC_lexicon_tags_25 },
    { 0xcca0dc59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c0dc5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d40dc62, SUC_lexicon_tags_1 },
    { 0x8f80dc63, SUC_lexicon_tags_23 },
    { 0x5ae0dc64, SUC_lexicon_tags_25 },
    { 0x5fa0dc65, SUC_lexicon_tags_47 },
    { 0x10e0dc66, SUC_lexicon_tags_45 },
    { 0x8840dc67, SUC_lexicon_tags_37 },
    { 0x6fe0dc68, SUC_lexicon_tags_5 },
    { 0xb320dc64, SUC_lexicon_tags_26 },
    { 0x2040dc66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5220dc6f, SUC_lexicon_tags_37 },
    { 0xfca0dc70, SUC_lexicon_tags_17 },
    { 0x9560dc71, SUC_lexicon_tags_7 },
    { 0x9860dc72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc79, SUC_lexicon_tags_43 },
    { 0x4080dc7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd40dc7e, SUC_lexicon_tags_7 },
    { 0x29a0dc7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfba0dc81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa8a0dc83, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xba20dc85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0dc89, SUC_lexicon_tags_16 },
    { 0x7620dc8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e80dc90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e0dc98, SUC_lexicon_tags_27 },
    { 0xc300dc99, SUC_lexicon_tags_34 },
    { 0x1980dc9a, SUC_lexicon_tags_14 },
    { 0x7b00dc99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a0dc9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf200dca1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe660dca3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e0dca7, SUC_lexicon_tags_7 },
    { 0xa520dca8, SUC_lexicon_tags_1 },
    { 0x8d80dca7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c0dcae, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4580dcb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c0dcbd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6340dcc0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4600dcc3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a40dcc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67c0dcc7, SUC_lexicon_tags_11 },
    { 0x5280dcc8, SUC_lexicon_tags_7 },
    { 0xb20dcc8, SUC_lexicon_tags_14 },
    { 0x26c0dcca, SUC_lexicon_tags_35 },
    { 0x9c60dccb, SUC_lexicon_tags_11 },
    { 0xf180dccc, SUC_lexicon_tags_1 },
    { 0x13c0dccd, SUC_lexicon_tags_12 },
    { 0x2f60dcc8, SUC_lexicon_tags_61 },
    { 0x5ee0dccc, SUC_lexicon_tags_17 },
    { 0xd780dcd0, SUC_lexicon_tags_12 },
    { 0x90c0dcd1, SUC_lexicon_tags_47 },
    { 0x9f20dcd2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a60dcd7, SUC_lexicon_tags_47 },
    { 0xc6a0dcd8, SUC_lexicon_tags_12 },
    { 0xee60dcd9, SUC_lexicon_tags_1076 },
    { 0x5520dcda, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0x9940dcdc, SUC_lexicon_tags_2 },
    { 0x63e0dcdc, SUC_lexicon_tags_60 },
    { 0xbd00dcdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb840dce0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c40dce4, SUC_lexicon_tags_463 },
    { 0x8640dce4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d40dcec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9720dcf0, SUC_lexicon_tags_25 },
    { 0xf940dcf1, SUC_lexicon_tags_12 },
    { 0xc20dcf1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4720dcf5, SUC_lexicon_tags_23 },
    { 0xfe80dcf5, SUC_lexicon_tags_17 },
    { 0xac00dcf5, SUC_lexicon_tags_49 },
    { 0x5e40dcf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c0dcfe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc320dd00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10e0dd02, SUC_lexicon_tags_31 },
    { 0x8780dd03, SUC_lexicon_tags_0 },
    { 0xdfa0dd04, SUC_lexicon_tags_8 },
    { 0xbc00dd02, SUC_lexicon_tags_56 },
    { 0xc100dd03, SUC_lexicon_tags_33 },
    { 0x73e0dd07, SUC_lexicon_tags_12 },
    { 0x4560dd08, SUC_lexicon_tags_7 },
    { 0x4d80dd09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2bc0dd0b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x2780dd0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a0dd11, SUC_lexicon_tags_25 },
    { 0xbac0dd12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1060dd15, SUC_lexicon_tags_1 },
    { 0x1540dd15, SUC_lexicon_tags_53 },
    { 0xde0dd17, SUC_lexicon_tags_8 },
    { 0xb5a0dd18, SUC_lexicon_tags_0 },
    { 0xd9e0dd17, SUC_lexicon_tags_15 },
    { 0xde40dd1a, SUC_lexicon_tags_2 },
    { 0x38e0dd1b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1840dd1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd180dd22, SUC_lexicon_tags_17 },
    { 0x8b00dd23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x33a0dd25, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf960dd2a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a0dd2e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe840dd31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7c0dd33, SUC_lexicon_tags_30 },
    { 0x4600dd33, SUC_lexicon_tags_70 },
    { 0xe20dd35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8d00dd37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b40dd39, SUC_lexicon_tags_60 },
    { 0xdc40dd3a, SUC_lexicon_tags_11 },
    { 0x92e0dd39, SUC_lexicon_tags_8 },
    { 0x2800dd3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c0dd3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc040dd43, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a0dd46, SUC_lexicon_tags_27 },
    { 0x100dd47, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d40dd4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf60dd4e, SUC_lexicon_tags_0 },
    { 0x9280dd4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f60dd51, SUC_lexicon_tags_64 },
    { 0xde60dd52, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c0dd58, SUC_lexicon_tags_57 },
    { 0x160dd58, SUC_lexicon_tags_4 },
    { 0xd720dd5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa20dd5c, SUC_lexicon_tags_12 },
    { 0x6b00dd5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe080dd61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7c0dd63, SUC_lexicon_tags_37 },
    { 0x1a00dd64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5260dd6a, SUC_lexicon_tags_37 },
    { 0xffe0dd6a, SUC_lexicon_tags_37 },
    { 0x9280dd6c, SUC_lexicon_tags_1 },
    { 0xcd40dd6d, SUC_lexicon_tags_17 },
    { 0xeb60dd6e, SUC_lexicon_tags_25 },
    { 0xcae0dd6e, SUC_lexicon_tags_9 },
    { 0x4080dd6f, SUC_lexicon_tags_14 },
    { 0x1f20dd6b, SUC_lexicon_tags_14 },
    { 0xfb40dd72, SUC_lexicon_tags_17 },
    { 0xba0dd73, SUC_lexicon_tags_17 },
    { 0x880dd72, SUC_lexicon_tags_37 },
    { 0xdc80dd73, SUC_lexicon_tags_12 },
    { 0xb900dd75, SUC_lexicon_tags_15 },
    { 0xc240dd6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe20dd79, SUC_lexicon_tags_11 },
    { 0x5980dd7a, SUC_lexicon_tags_37 },
    { 0x8600dd7b, SUC_lexicon_tags_88 },
    { 0xf280dd7c, SUC_lexicon_tags_30 },
    { 0xbc00dd7c, SUC_lexicon_tags_28 },
    { 0xc960dd7e, SUC_lexicon_tags_14 },
    { 0x1be0dd7f, SUC_lexicon_tags_17 },
    { 0x88e0dd7c, SUC_lexicon_tags_7 },
    { 0x72e0dd81, SUC_lexicon_tags_39 },
    { 0x2740dd80, SUC_lexicon_tags_57 },
    { 0xeaa0dd83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4a0dd85, SUC_lexicon_tags_3 },
    { 0x8ec0dd85, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc500dd8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8e0dd8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c0dd93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3a0dd95, SUC_lexicon_tags_43 },
    { 0x4ce0dd96, SUC_lexicon_tags_39 },
    { 0x2680dd97, SUC_lexicon_tags_87 },
    { 0x2500dd95, SUC_lexicon_tags_17 },
    { 0x6e40dd99, SUC_lexicon_tags_47 },
    { 0x1340dd9a, SUC_lexicon_tags_15 },
    { 0xf360dd9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa300dd9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e0dda3, SUC_lexicon_tags_17 },
    { 0x9800dda3, SUC_lexicon_tags_39 },
    { 0x4fa0dda5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5380dda7, SUC_lexicon_tags_82 },
    { 0xf040dda8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d20ddad, SUC_lexicon_tags_363 },
    { 0xace0ddae, SUC_lexicon_tags_8 },
    { 0x28e0ddaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e0ddb7, SUC_lexicon_tags_7 },
    { 0x2a80ddb8, SUC_lexicon_tags_1 },
    { 0xe920ddb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe80ddbe, SUC_lexicon_tags_0 },
    { 0xa840ddbe, SUC_lexicon_tags_318 },
    { 0xf080ddbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4120ddc6, SUC_lexicon_tags_2 },
    { 0x9ac0ddc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb200ddcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360ddd0, SUC_lexicon_tags_198 },
    { 0xe480ddd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffa0ddd3, SUC_lexicon_tags_35 },
    { 0x9e60ddd4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b00ddd6, SUC_lexicon_tags_37 },
    { 0x7cc0ddd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6560dddc, SUC_lexicon_tags_1 },
    { 0x8ea0dddd, SUC_lexicon_tags_28 },
    { 0xcfc0dddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd40dde0, SUC_lexicon_tags_0 },
    { 0x1b80dde0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4740dde4, SUC_lexicon_tags_30 },
    { 0xbfe0dde5, SUC_lexicon_tags_7 },
    { 0xbb40dde6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec0ddea, SUC_lexicon_tags_2 },
    { 0xbe20ddeb, SUC_lexicon_tags_17 },
    { 0x17c0ddea, SUC_lexicon_tags_27 },
    { 0xcde0ddea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180ddf4, SUC_lexicon_tags_2 },
    { 0xa6c0ddf5, SUC_lexicon_tags_11 },
    { 0x18e0ddf5, SUC_lexicon_tags_7 },
    { 0xb580ddf4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7f80ddf9, SUC_lexicon_tags_11 },
    { 0xf300ddfa, SUC_lexicon_tags_1 },
    { 0x800ddfa, SUC_lexicon_tags_11 },
    { 0x8600ddfc, SUC_lexicon_tags_12 },
    { 0x5820ddfd, SUC_lexicon_tags_7 },
    { 0xb60ddfc, SUC_lexicon_tags_1 },
    { 0x7420ddfc, SUC_lexicon_tags_15 },
    { 0x67c0de00, SUC_lexicon_tags_14 },
    { 0xe820ddf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57c0de03, SUC_lexicon_tags_8 },
    { 0xc3a0de04, SUC_lexicon_tags_1 },
    { 0xfec0de04, SUC_lexicon_tags_46 },
    { 0x48e0de06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xca0de08, SUC_lexicon_tags_6 },
    { 0x20de09, SUC_lexicon_tags_37 },
    { 0xc680de09, SUC_lexicon_tags_12 },
    { 0x7f80de08, SUC_lexicon_tags_39 },
    { 0xb920de0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba60de11, SUC_lexicon_tags_11 },
    { 0xebc0de12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420de19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e0de1c, SUC_lexicon_tags_37 },
    { 0xef20de1d, SUC_lexicon_tags_337 },
    { 0x13a0de1e, SUC_lexicon_tags_2 },
    { 0xae0de1f, SUC_lexicon_tags_21 },
    { 0x6160de1f, SUC_lexicon_tags_12 },
    { 0xff60de21, SUC_lexicon_tags_11 },
    { 0x1000de22, SUC_lexicon_tags_23 },
    { 0x8e20de23, SUC_lexicon_tags_26 },
    { 0xfa00de1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c80de26, SUC_lexicon_tags_1 },
    { 0xa3c0de26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a0de2a, SUC_lexicon_tags_38 },
    { 0x2ee0de2a, SUC_lexicon_tags_9 },
    { 0xec0de2a, SUC_lexicon_tags_15 },
    { 0xdec0de2d, SUC_lexicon_tags_25 },
    { 0xa4c0de2e, SUC_lexicon_tags_15 },
    { 0xd860de2f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xcb60de31, SUC_lexicon_tags_17 },
    { 0xdd80de32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8080de35, SUC_lexicon_tags_23 },
    { 0x3880de35, SUC_lexicon_tags_23 },
    { 0x1640de36, SUC_lexicon_tags_2 },
    { 0x1f40de35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xace0de3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec0de49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7440de4d, SUC_lexicon_tags_217 },
    { 0x4b80de4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1180de53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc9c0de55, SUC_lexicon_tags_171 },
    { 0xbf00de55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf20de5b, SUC_lexicon_tags_11 },
    { 0x5620de5c, SUC_lexicon_tags_19 },
    { 0x3920de5d, SUC_lexicon_tags_47 },
    { 0xa280de5e, SUC_lexicon_tags_17 },
    { 0xa460de5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf20de63, SUC_lexicon_tags_3 },
    { 0x51e0de64, SUC_lexicon_tags_23 },
    { 0xa620de65, SUC_lexicon_tags_66 },
    { 0x5e80de66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xef20de68, SUC_lexicon_tags_25 },
    { 0xf800de69, SUC_lexicon_tags_10 },
    { 0x6b40de68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0de6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac60de74, SUC_lexicon_tags_9 },
    { 0xcda0de75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a0de7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdea0de7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe140de81, SUC_lexicon_tags_49 },
    { 0x6520de82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe960de84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c0de88, SUC_lexicon_tags_15 },
    { 0xad20de89, SUC_lexicon_tags_14 },
    { 0x9e00de8a, SUC_lexicon_tags_37 },
    { 0x3960de8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e40de91, SUC_lexicon_tags_10 },
    { 0xfa0de92, SUC_lexicon_tags_7 },
    { 0x3d60de91, SUC_lexicon_tags_37 },
    { 0x6640de93, SUC_lexicon_tags_15 },
    { 0x51a0de92, SUC_lexicon_tags_23 },
    { 0x5420de96, SUC_lexicon_tags_12 },
    { 0x9c00de96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee0de9a, SUC_lexicon_tags_27 },
    { 0xba60de9b, SUC_lexicon_tags_2 },
    { 0x86e0de9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2e40de9e, SUC_lexicon_tags_15 },
    { 0x43e0de9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc0dea3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x44e0dea5, SUC_lexicon_tags_23 },
    { 0x1180dea5, SUC_lexicon_tags_46 },
    { 0x80a0dea7, SUC_lexicon_tags_37 },
    { 0x2a40dea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb40deab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e0deb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf40deb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6500deb7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4960deb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5280debb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3180debf, SUC_lexicon_tags_45 },
    { 0x7b00dec0, SUC_lexicon_tags_37 },
    { 0x9dc0dec1, SUC_lexicon_tags_1 },
    { 0xbc00dec0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d00dec9, SUC_lexicon_tags_7 },
    { 0xde0dec9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefc0decc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca0ded3, SUC_lexicon_tags_37 },
    { 0x7120ded3, SUC_lexicon_tags_11 },
    { 0x7760ded5, SUC_lexicon_tags_33 },
    { 0x9c00ded4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e0dedc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5ac0dede, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3bc0dee0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa0dee4, SUC_lexicon_tags_3 },
    { 0x8140dee5, SUC_lexicon_tags_12 },
    { 0x9b40dee4, SUC_lexicon_tags_18 },
    { 0xda80dee7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4660deea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x86c0deec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e0deef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4040def2, SUC_lexicon_tags_8 },
    { 0x8660def3, SUC_lexicon_tags_14 },
    { 0x3260def4, SUC_lexicon_tags_68 },
    { 0x9e0def5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d20def7, SUC_lexicon_tags_29 },
    { 0x6be0def8, SUC_lexicon_tags_72 },
    { 0xb100def9, SUC_lexicon_tags_60 },
    { 0x52c0defa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e0df00, SUC_lexicon_tags_2 },
    { 0x60a0df01, SUC_lexicon_tags_15 },
    { 0x2d00df02, SUC_lexicon_tags_9 },
    { 0xe540df00, SUC_lexicon_tags_0 },
    { 0xda0df03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c0df07, SUC_lexicon_tags_11 },
    { 0x5ee0df08, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2340df0c, SUC_lexicon_tags_37 },
    { 0xb660df0d, SUC_lexicon_tags_9 },
    { 0x4b20df0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ee0df10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc280df13, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0xcee0df15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a0df1f, SUC_lexicon_tags_37 },
    { 0x5180df20, SUC_lexicon_tags_23 },
    { 0x1fe0df21, SUC_lexicon_tags_15 },
    { 0xe2a0df22, SUC_lexicon_tags_542 },
    { 0x8a0df23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x140df25, SUC_lexicon_tags_1 },
    { 0x7200df26, SUC_lexicon_tags_7 },
    { 0x18c0df27, SUC_lexicon_tags_15 },
    { 0x3c00df27, SUC_lexicon_tags_0 },
    { 0xed00df29, SUC_lexicon_tags_25 },
    { 0x83c0df29, SUC_lexicon_tags_65 },
    { 0xe3e0df2a, SUC_lexicon_tags_63 },
    { 0x4500df29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce0df2f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d40df33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca0df36, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e00df3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e0df3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b40df45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdc0df47, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3380df4c, SUC_lexicon_tags_25 },
    { 0xbc0df4d, SUC_lexicon_tags_37 },
    { 0xbf40df4e, SUC_lexicon_tags_28 },
    { 0x5380df4e, SUC_lexicon_tags_49 },
    { 0x8d60df4f, SUC_lexicon_tags_2 },
    { 0x3320df4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2500df56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e0df59, SUC_lexicon_tags_25 },
    { 0xbf80df59, SUC_lexicon_tags_7 },
    { 0xdfe0df59, SUC_lexicon_tags_45 },
    { 0x3360df5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec20df62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff60df67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc0df6a, SUC_lexicon_tags_17 },
    { 0x9680df6a, SUC_lexicon_tags_2 },
    { 0x7020df6c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5200df72, SUC_lexicon_tags_1 },
    { 0x7440df73, SUC_lexicon_tags_46 },
    { 0xb580df74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e0df77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640df7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c00df7f, SUC_lexicon_tags_11 },
    { 0x92e0df80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5560df86, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6100df8a, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xb700df8d, SUC_lexicon_tags_25 },
    { 0xb780df8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99c0df90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c0df94, SUC_lexicon_tags_11 },
    { 0x4f60df95, SUC_lexicon_tags_12 },
    { 0x6d40df96, SUC_lexicon_tags_30 },
    { 0xb200df96, SUC_lexicon_tags_19 },
    { 0x440df98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1540df9b, SUC_lexicon_tags_2 },
    { 0xd120df9c, SUC_lexicon_tags_17 },
    { 0xbac0df9c, SUC_lexicon_tags_55 },
    { 0x4a20df9b, SUC_lexicon_tags_17 },
    { 0x99c0df9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b00dfa2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9c60dfa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9760dfa6, SUC_lexicon_tags_33 },
    { 0x7080dfa6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x46e0dfa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22a0dfab, SUC_lexicon_tags_13 },
    { 0xb400dfac, SUC_lexicon_tags_11 },
    { 0x8360dfac, SUC_lexicon_tags_13 },
    { 0x3980dfac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a20dfb1, SUC_lexicon_tags_12 },
    { 0x12e0dfb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c0dfb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb740dfb9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe80dfc2, SUC_lexicon_tags_35 },
    { 0xbd40dfc2, SUC_lexicon_tags_85 },
    { 0xc740dfc4, SUC_lexicon_tags_37 },
    { 0x8280dfc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x580dfc7, SUC_lexicon_tags_66 },
    { 0x720dfc8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfa0dfca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5a0dfcc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d40dfcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea0dfd3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf720dfd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0dfdd, SUC_lexicon_tags_4 },
    { 0x9100dfde, SUC_lexicon_tags_11 },
    { 0x8e60dfdf, SUC_lexicon_tags_45 },
    { 0xace0dfe0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9000dfe2, SUC_lexicon_tags_56 },
    { 0xf000dfe3, SUC_lexicon_tags_15 },
    { 0x5c80dfe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedc0dfe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3360dfec, SUC_lexicon_tags_11 },
    { 0x1e00dfed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa0dff2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c0dff7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9880dffa, SUC_lexicon_tags_1 },
    { 0xc340dffb, SUC_lexicon_tags_25 },
    { 0xbdc0dffc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5500e001, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a0e004, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb180e007, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7400e009, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd60e010, SUC_lexicon_tags_38 },
    { 0x8100e011, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5520e018, SUC_lexicon_tags_33 },
    { 0x9d40e019, SUC_lexicon_tags_9 },
    { 0xf640e01a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1640e01d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xce40e020, SUC_lexicon_tags_38 },
    { 0x7620e021, SUC_lexicon_tags_9 },
    { 0xf8a0e021, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e40e027, SUC_lexicon_tags_47 },
    { 0x3f80e028, SUC_lexicon_tags_2 },
    { 0xdfa0e028, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4100e02b, SUC_lexicon_tags_2 },
    { 0x5de0e02c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a0e030, SUC_lexicon_tags_17 },
    { 0x4880e030, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e0e037, SUC_lexicon_tags_30 },
    { 0x320e037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed00e03c, SUC_lexicon_tags_25 },
    { 0xfa20e03d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf80e03f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d20e043, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfc0e045, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0e048, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef40e04a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d00e04d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3740e04f, SUC_lexicon_tags_68 },
    { 0x4080e050, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4bc0e052, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5d60e054, SUC_lexicon_tags_11 },
    { 0xe380e055, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xae20e057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c00e059, SUC_lexicon_tags_11 },
    { 0xdce0e05a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b00e05d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7000e05f, SUC_lexicon_tags_7 },
    { 0xb9e0e060, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e063, SUC_lexicon_tags_106 },
    { 0x3020e064, SUC_lexicon_tags_1 },
    { 0x8cc0e065, SUC_lexicon_tags_19 },
    { 0xfcc0e063, SUC_lexicon_tags_0 },
    { 0xdbc0e064, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca0e06d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a0e072, SUC_lexicon_tags_11 },
    { 0x55e0e073, SUC_lexicon_tags_7 },
    { 0x500e074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x640e076, SUC_lexicon_tags_15 },
    { 0x5c60e077, SUC_lexicon_tags_12 },
    { 0x7ac0e076, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a0e07f, SUC_lexicon_tags_56 },
    { 0x200e080, SUC_lexicon_tags_49 },
    { 0x6ea0e081, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c20e087, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b00e08a, SUC_lexicon_tags_49 },
    { 0x1960e08b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc0e08f, SUC_lexicon_tags_25 },
    { 0xff40e090, SUC_lexicon_tags_0 },
    { 0x70c0e090, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc580e093, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6be0e095, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be0e098, SUC_lexicon_tags_12 },
    { 0xe40e099, SUC_lexicon_tags_6 },
    { 0xc440e09a, SUC_lexicon_tags_60 },
    { 0xf2c0e098, SUC_lexicon_tags_47 },
    { 0xad40e098, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4c0e09e, SUC_lexicon_tags_2 },
    { 0x420e09f, SUC_lexicon_tags_11 },
    { 0xe6e0e0a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b00e0a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc80e0aa, SUC_lexicon_tags_43 },
    { 0x4ae0e0ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3900e0b0, SUC_lexicon_tags_39 },
    { 0xd3e0e0b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e0e0b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbbc0e0b6, SUC_lexicon_tags_33 },
    { 0xb080e0b7, SUC_lexicon_tags_2 },
    { 0x1000e0b7, SUC_lexicon_tags_77 },
    { 0xf4c0e0b8, SUC_lexicon_tags_979 },
    { 0x19a0e0b7, SUC_lexicon_tags_49 },
    { 0xb0e0e0b8, SUC_lexicon_tags_31 },
    { 0xc8a0e0bc, SUC_lexicon_tags_11 },
    { 0xde40e0bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x740e0bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f60e0c1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8f60e0c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3480e0c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfee0e0ca, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x10a0e0cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc820e0ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb480e0d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe0e0db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bc0e0dd, SUC_lexicon_tags_60 },
    { 0x8da0e0dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5c0e0e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x120e0e2, SUC_lexicon_tags_17 },
    { 0xad40e0e2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc00e0e6, SUC_lexicon_tags_227 },
    { 0, NULL },
    { 0x7680e0e8, SUC_lexicon_tags_16 },
    { 0xa00e0e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1860e0ec, SUC_lexicon_tags_37 },
    { 0xd0e0e0ed, SUC_lexicon_tags_48 },
    { 0x5900e0ee, SUC_lexicon_tags_14 },
    { 0x9100e0ef, SUC_lexicon_tags_17 },
    { 0xc440e0f0, SUC_lexicon_tags_233 },
    { 0x5b40e0f1, SUC_lexicon_tags_25 },
    { 0x5660e0ee, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe020e0f5, SUC_lexicon_tags_61 },
    { 0xdd40e0f6, SUC_lexicon_tags_32 },
    { 0x3020e0f7, SUC_lexicon_tags_29 },
    { 0x8600e0f8, SUC_lexicon_tags_12 },
    { 0x1940e0f7, SUC_lexicon_tags_16 },
    { 0x80a0e0f9, SUC_lexicon_tags_49 },
    { 0x9b60e0fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb60e102, SUC_lexicon_tags_11 },
    { 0x43a0e103, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4940e105, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe040e108, SUC_lexicon_tags_27 },
    { 0xffa0e109, SUC_lexicon_tags_7 },
    { 0xaea0e10a, SUC_lexicon_tags_37 },
    { 0xe8e0e10a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4500e10d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3380e115, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6700e119, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d20e11c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a0e12e, SUC_lexicon_tags_4 },
    { 0x6ee0e12f, SUC_lexicon_tags_129 },
    { 0x51e0e12e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc0e134, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce0e137, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b00e13b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3700e13e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd6a0e140, SUC_lexicon_tags_25 },
    { 0xe620e140, SUC_lexicon_tags_33 },
    { 0xcc20e142, SUC_lexicon_tags_12 },
    { 0x45e0e142, SUC_lexicon_tags_6 },
    { 0xb740e140, SUC_lexicon_tags_45 },
    { 0x1ba0e144, SUC_lexicon_tags_7 },
    { 0x90a0e146, SUC_lexicon_tags_13 },
    { 0x32a0e147, SUC_lexicon_tags_0 },
    { 0xb300e141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe340e14b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c0e14e, SUC_lexicon_tags_17 },
    { 0x6480e14e, SUC_lexicon_tags_31 },
    { 0xb60e150, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c0e153, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a20e156, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b20e15a, SUC_lexicon_tags_43 },
    { 0xf300e15b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27c0e15d, SUC_lexicon_tags_37 },
    { 0xe2a0e15e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51c0e160, SUC_lexicon_tags_19 },
    { 0x22c0e160, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3a0e163, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800e168, SUC_lexicon_tags_8 },
    { 0x3280e169, SUC_lexicon_tags_65 },
    { 0xbec0e169, SUC_lexicon_tags_16 },
    { 0x6780e16b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd0e0e16d, SUC_lexicon_tags_47 },
    { 0x9340e16d, SUC_lexicon_tags_1 },
    { 0xbc20e16d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37e0e171, SUC_lexicon_tags_17 },
    { 0x7440e172, SUC_lexicon_tags_37 },
    { 0x82e0e171, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60a0e175, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfc40e177, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2140e179, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a0e17e, SUC_lexicon_tags_17 },
    { 0x9060e17f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0e182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde80e18d, SUC_lexicon_tags_2 },
    { 0x3c20e18e, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xd120e191, SUC_lexicon_tags_57 },
    { 0x9f60e192, SUC_lexicon_tags_9 },
    { 0x34a0e191, SUC_lexicon_tags_7 },
    { 0x9fa0e194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1480e199, SUC_lexicon_tags_38 },
    { 0x3fc0e19a, SUC_lexicon_tags_43 },
    { 0x29c0e19a, SUC_lexicon_tags_17 },
    { 0x8b80e19c, SUC_lexicon_tags_7 },
    { 0x34e0e19a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b60e1a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca80e1a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c0e1aa, SUC_lexicon_tags_15 },
    { 0x7b00e1aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5d40e1ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5100e1b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce0e1b4, SUC_lexicon_tags_26 },
    { 0xcc40e1b5, SUC_lexicon_tags_38 },
    { 0xac0e1b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xde80e1b8, SUC_lexicon_tags_117 },
    { 0xfb20e1b9, SUC_lexicon_tags_9 },
    { 0xf2e0e1ba, SUC_lexicon_tags_37 },
    { 0x9e40e1b8, SUC_lexicon_tags_14 },
    { 0x5a40e1bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff80e1c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7a0e1c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed20e1c6, SUC_lexicon_tags_17 },
    { 0xfe20e1c6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdde0e1c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0e1cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb00e1ce, SUC_lexicon_tags_2 },
    { 0xda0e1cf, SUC_lexicon_tags_43 },
    { 0xbf60e1d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6780e1d5, SUC_lexicon_tags_37 },
    { 0x6340e1d5, SUC_lexicon_tags_23 },
    { 0xb280e1d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2d00e1d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e0e1dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d20e1e0, SUC_lexicon_tags_68 },
    { 0x5c60e1e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeb20e1e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8040e1e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac0e1f0, SUC_lexicon_tags_23 },
    { 0xd8c0e1f1, SUC_lexicon_tags_6 },
    { 0xb280e1f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7640e1fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae0e1fd, SUC_lexicon_tags_26 },
    { 0x2c60e1fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f40e201, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0e205, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3860e20a, SUC_lexicon_tags_27 },
    { 0xb9e0e20a, SUC_lexicon_tags_60 },
    { 0x8500e20c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e00e210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6400e21a, SUC_lexicon_tags_17 },
    { 0xb0e0e21a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa020e222, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa060e225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c0e22a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf000e22d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a0e230, SUC_lexicon_tags_37 },
    { 0x79a0e231, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff60e238, SUC_lexicon_tags_1 },
    { 0x4180e238, SUC_lexicon_tags_27 },
    { 0x20a0e23a, SUC_lexicon_tags_37 },
    { 0x7900e238, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be0e241, SUC_lexicon_tags_63 },
    { 0x4d20e241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a20e248, SUC_lexicon_tags_1 },
    { 0x4120e249, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a0e24e, SUC_lexicon_tags_9 },
    { 0xe0c0e24e, SUC_lexicon_tags_11 },
    { 0xd240e24f, SUC_lexicon_tags_25 },
    { 0xed60e251, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b60e257, SUC_lexicon_tags_37 },
    { 0x2f40e257, SUC_lexicon_tags_10 },
    { 0x9c20e259, SUC_lexicon_tags_7 },
    { 0x1100e259, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5040e25c, SUC_lexicon_tags_15 },
    { 0x8f60e25d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c20e25f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2120e267, SUC_lexicon_tags_7 },
    { 0xf920e268, SUC_lexicon_tags_38 },
    { 0xba40e269, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd800e26c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f00e272, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde0e276, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ee0e278, SUC_lexicon_tags_32 },
    { 0x7da0e278, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0e27d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0e281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80e285, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a0e28a, SUC_lexicon_tags_34 },
    { 0x2540e28a, SUC_lexicon_tags_17 },
    { 0xe2c0e28a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7980e298, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xede0e29a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a40e2a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa080e2a2, SUC_lexicon_tags_27 },
    { 0x9a60e2a3, SUC_lexicon_tags_39 },
    { 0x1940e2a4, SUC_lexicon_tags_29 },
    { 0xdfc0e2a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ca0e2a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0e2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa860e2b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcf60e2b2, SUC_lexicon_tags_1473 },
    { 0, NULL },
    { 0x2780e2b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e0e2bb, SUC_lexicon_tags_0 },
    { 0xbc80e2bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae0e2bf, SUC_lexicon_tags_39 },
    { 0xd140e2c0, SUC_lexicon_tags_37 },
    { 0xde00e2c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc0e2c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2fa0e2c5, SUC_lexicon_tags_57 },
    { 0xa400e2c5, SUC_lexicon_tags_15 },
    { 0x3bc0e2c7, SUC_lexicon_tags_12 },
    { 0xda00e2c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a0e2cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3a0e2ce, SUC_lexicon_tags_26 },
    { 0x580e2ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8940e2d3, SUC_lexicon_tags_2 },
    { 0xb820e2d3, SUC_lexicon_tags_11 },
    { 0x78e0e2d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440e2dc, SUC_lexicon_tags_28 },
    { 0x1cc0e2dd, SUC_lexicon_tags_12 },
    { 0x3aa0e2de, SUC_lexicon_tags_16 },
    { 0xa5c0e2df, SUC_lexicon_tags_45 },
    { 0x8e40e2de, SUC_lexicon_tags_7 },
    { 0x3140e2e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b80e2e3, SUC_lexicon_tags_31 },
    { 0x5840e2e4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20e2ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc120e2f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf00e2f4, SUC_lexicon_tags_11 },
    { 0x91a0e2f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc80e2fc, SUC_lexicon_tags_49 },
    { 0xd9c0e2fd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xbe60e2ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf6e0e301, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6820e306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc740e30b, SUC_lexicon_tags_1622 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9500e311, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa340e313, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c0e319, SUC_lexicon_tags_8 },
    { 0x1220e319, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67a0e31c, SUC_lexicon_tags_39 },
    { 0x6ba0e31c, SUC_lexicon_tags_23 },
    { 0xd9e0e31c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa0e322, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb60e324, SUC_lexicon_tags_33 },
    { 0xd880e325, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5000e327, SUC_lexicon_tags_15 },
    { 0x80c0e327, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8300e32d, SUC_lexicon_tags_31 },
    { 0x2900e32d, SUC_lexicon_tags_49 },
    { 0xdee0e32e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e0e334, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c40e33b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc0e342, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1220e345, SUC_lexicon_tags_45 },
    { 0x9ba0e345, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe780e34a, SUC_lexicon_tags_47 },
    { 0x360e34b, SUC_lexicon_tags_25 },
    { 0x3d40e34b, SUC_lexicon_tags_32 },
    { 0x7ee0e34d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0e355, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a0e35b, SUC_lexicon_tags_49 },
    { 0x5ae0e35c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7360e360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f80e365, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e0e368, SUC_lexicon_tags_17 },
    { 0x73a0e369, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad00e372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d60e379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xae40e37c, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x7840e37e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6560e384, SUC_lexicon_tags_37 },
    { 0x3b20e385, SUC_lexicon_tags_15 },
    { 0x59a0e386, SUC_lexicon_tags_6 },
    { 0x6fe0e387, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfbc0e389, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b00e395, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xed40e398, SUC_lexicon_tags_17 },
    { 0x32e0e398, SUC_lexicon_tags_11 },
    { 0x3b20e39a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8440e39f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8520e3a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa40e3a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca0e3a8, SUC_lexicon_tags_33 },
    { 0x7ca0e3a8, SUC_lexicon_tags_48 },
    { 0x3f20e3aa, SUC_lexicon_tags_29 },
    { 0x1a00e3ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xda0e3ad, SUC_lexicon_tags_43 },
    { 0x4de0e3ad, SUC_lexicon_tags_1 },
    { 0xeee0e3af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb220e3b8, SUC_lexicon_tags_38 },
    { 0xb720e3b8, SUC_lexicon_tags_38 },
    { 0xf6a0e3ba, SUC_lexicon_tags_9 },
    { 0xfd40e3bb, SUC_lexicon_tags_57 },
    { 0xe240e3b9, SUC_lexicon_tags_7 },
    { 0x9a00e3b9, SUC_lexicon_tags_21 },
    { 0x95e0e3bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c20e3c1, SUC_lexicon_tags_10 },
    { 0x7700e3c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a60e3c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x34a0e3c8, SUC_lexicon_tags_46 },
    { 0x21a0e3c9, SUC_lexicon_tags_0 },
    { 0xe3c0e3ca, SUC_lexicon_tags_12 },
    { 0xcb00e3cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c00e3d1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e0e3d6, SUC_lexicon_tags_43 },
    { 0xd5a0e3d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1740e3dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c0e3df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8200e3e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b40e3e4, SUC_lexicon_tags_15 },
    { 0x4500e3e5, SUC_lexicon_tags_17 },
    { 0x7920e3e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae0e3e9, SUC_lexicon_tags_1 },
    { 0xe1a0e3ea, SUC_lexicon_tags_46 },
    { 0x6a20e3eb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa0e3f0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb0a0e3f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700e3f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a20e3fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5840e3fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c80e404, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5e0e406, SUC_lexicon_tags_12 },
    { 0xd700e407, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc0e40a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed80e40e, SUC_lexicon_tags_37 },
    { 0x5a0e40e, SUC_lexicon_tags_27 },
    { 0xc9e0e410, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c00e415, SUC_lexicon_tags_4 },
    { 0x4060e416, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc40e41a, SUC_lexicon_tags_12 },
    { 0xcc20e41a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3600e420, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2880e424, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac0e42b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a0e42e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5740e431, SUC_lexicon_tags_42 },
    { 0x6ea0e431, SUC_lexicon_tags_37 },
    { 0x3060e433, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4a0e435, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaea0e437, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa600e439, SUC_lexicon_tags_25 },
    { 0x18a0e439, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e0e44d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c0e455, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de0e458, SUC_lexicon_tags_11 },
    { 0x2ba0e459, SUC_lexicon_tags_11 },
    { 0xc3a0e45a, SUC_lexicon_tags_60 },
    { 0x8f60e45b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7c0e45d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa820e45f, SUC_lexicon_tags_15 },
    { 0xb680e460, SUC_lexicon_tags_7 },
    { 0x8200e45f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x90c0e463, SUC_lexicon_tags_17 },
    { 0xf8a0e463, SUC_lexicon_tags_15 },
    { 0x3240e465, SUC_lexicon_tags_32 },
    { 0x1520e466, SUC_lexicon_tags_6 },
    { 0xca80e465, SUC_lexicon_tags_18 },
    { 0xca80e468, SUC_lexicon_tags_14 },
    { 0x7100e469, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c60e46c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc60e46e, SUC_lexicon_tags_43 },
    { 0xb40e46f, SUC_lexicon_tags_1 },
    { 0x2c20e46e, SUC_lexicon_tags_26 },
    { 0x7f80e470, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be0e475, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61e0e477, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8660e47b, SUC_lexicon_tags_6 },
    { 0x3660e47c, SUC_lexicon_tags_49 },
    { 0x9220e47c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c0e486, SUC_lexicon_tags_1 },
    { 0x8e00e486, SUC_lexicon_tags_6 },
    { 0x56c0e487, SUC_lexicon_tags_25 },
    { 0xa560e487, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefe0e48b, SUC_lexicon_tags_18 },
    { 0x92a0e48c, SUC_lexicon_tags_161 },
    { 0x38e0e48d, SUC_lexicon_tags_37 },
    { 0x4e40e48e, SUC_lexicon_tags_4 },
    { 0x3900e48d, SUC_lexicon_tags_37 },
    { 0x5b20e490, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7460e495, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb940e497, SUC_lexicon_tags_17 },
    { 0xc240e498, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6840e49b, SUC_lexicon_tags_19 },
    { 0xc660e49c, SUC_lexicon_tags_1 },
    { 0xffc0e49d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e0e4a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5380e4ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0e4b0, SUC_lexicon_tags_65 },
    { 0x6240e4b1, SUC_lexicon_tags_49 },
    { 0x4f40e4b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x73a0e4b4, SUC_lexicon_tags_17 },
    { 0x8e80e4b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd800e4b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8980e4bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a0e4bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d80e4c1, SUC_lexicon_tags_29 },
    { 0x8120e4c2, SUC_lexicon_tags_6 },
    { 0xe260e4c3, SUC_lexicon_tags_16 },
    { 0x5ac0e4c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf400e4cb, SUC_lexicon_tags_45 },
    { 0xeb80e4cc, SUC_lexicon_tags_45 },
    { 0x3920e4cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b60e4d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c0e4db, SUC_lexicon_tags_38 },
    { 0x56c0e4db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2680e4df, SUC_lexicon_tags_68 },
    { 0x2380e4e0, SUC_lexicon_tags_33 },
    { 0xb3c0e4e1, SUC_lexicon_tags_84 },
    { 0xe00e4e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77e0e4e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf00e4e8, SUC_lexicon_tags_25 },
    { 0xc3a0e4e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc980e4ec, SUC_lexicon_tags_2 },
    { 0xdd00e4ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e0e4f1, SUC_lexicon_tags_74 },
    { 0xc0e4f2, SUC_lexicon_tags_30 },
    { 0xf880e4f2, SUC_lexicon_tags_11 },
    { 0xfa80e4f4, SUC_lexicon_tags_25 },
    { 0x4e0e4f4, SUC_lexicon_tags_25 },
    { 0x7060e4f4, SUC_lexicon_tags_25 },
    { 0x8f80e4f7, SUC_lexicon_tags_25 },
    { 0x9240e4f8, SUC_lexicon_tags_37 },
    { 0xc800e4f9, SUC_lexicon_tags_15 },
    { 0xdc60e4fa, SUC_lexicon_tags_65 },
    { 0xf940e4fb, SUC_lexicon_tags_25 },
    { 0x6860e4f4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa0e4ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8800e502, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xba00e504, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2520e506, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe500e50a, SUC_lexicon_tags_37 },
    { 0x4700e50b, SUC_lexicon_tags_11 },
    { 0x3920e50c, SUC_lexicon_tags_7 },
    { 0x7340e50a, SUC_lexicon_tags_38 },
    { 0xb2a0e50c, SUC_lexicon_tags_20 },
    { 0x8620e50f, SUC_lexicon_tags_7 },
    { 0x17a0e510, SUC_lexicon_tags_0 },
    { 0x8f80e511, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6720e513, SUC_lexicon_tags_2 },
    { 0x9040e514, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1bc0e516, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3940e518, SUC_lexicon_tags_194 },
    { 0x1c80e518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1080e51b, SUC_lexicon_tags_45 },
    { 0x65c0e51c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2280e51e, SUC_lexicon_tags_9 },
    { 0x7980e51e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6e80e521, SUC_lexicon_tags_15 },
    { 0xfba0e522, SUC_lexicon_tags_92 },
    { 0x25e0e521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4dc0e525, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c0e529, SUC_lexicon_tags_12 },
    { 0xf8a0e52a, SUC_lexicon_tags_8 },
    { 0x8040e52b, SUC_lexicon_tags_60 },
    { 0xa300e52b, SUC_lexicon_tags_11 },
    { 0x79a0e529, SUC_lexicon_tags_35 },
    { 0x8640e52e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb1c0e530, SUC_lexicon_tags_45 },
    { 0xaa00e531, SUC_lexicon_tags_32 },
    { 0x3540e530, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae0e536, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8800e53a, SUC_lexicon_tags_6 },
    { 0xe420e53a, SUC_lexicon_tags_25 },
    { 0x8c00e53c, SUC_lexicon_tags_66 },
    { 0xd020e53d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7a0e53f, SUC_lexicon_tags_1 },
    { 0x5ea0e540, SUC_lexicon_tags_6 },
    { 0x24a0e540, SUC_lexicon_tags_0 },
    { 0xfee0e542, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a0e545, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f20e54b, SUC_lexicon_tags_15 },
    { 0x8e20e54c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5780e54f, SUC_lexicon_tags_7 },
    { 0x9ce0e550, SUC_lexicon_tags_27 },
    { 0x3300e550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b00e553, SUC_lexicon_tags_1 },
    { 0xe320e553, SUC_lexicon_tags_25 },
    { 0x6600e555, SUC_lexicon_tags_17 },
    { 0x7c80e554, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa0e55a, SUC_lexicon_tags_8 },
    { 0xe860e55b, SUC_lexicon_tags_53 },
    { 0xaae0e55b, SUC_lexicon_tags_17 },
    { 0x2660e55b, SUC_lexicon_tags_43 },
    { 0x7240e55b, SUC_lexicon_tags_43 },
    { 0x97e0e55f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd60e564, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a0e567, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a0e572, SUC_lexicon_tags_12 },
    { 0x3a0e573, SUC_lexicon_tags_37 },
    { 0x9020e573, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc0e577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a0e57b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e0e57e, SUC_lexicon_tags_72 },
    { 0x70c0e57f, SUC_lexicon_tags_19 },
    { 0x98e0e580, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e0e584, SUC_lexicon_tags_34 },
    { 0x4ac0e585, SUC_lexicon_tags_12 },
    { 0xf080e586, SUC_lexicon_tags_2 },
    { 0x92a0e587, SUC_lexicon_tags_2 },
    { 0xfc40e588, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6420e58c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a0e58f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5000e592, SUC_lexicon_tags_79 },
    { 0x3680e593, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c60e595, SUC_lexicon_tags_60 },
    { 0x2480e596, SUC_lexicon_tags_27 },
    { 0xcb80e596, SUC_lexicon_tags_37 },
    { 0x3620e596, SUC_lexicon_tags_82 },
    { 0xf9a0e599, SUC_lexicon_tags_25 },
    { 0xd4c0e59a, SUC_lexicon_tags_37 },
    { 0x6e0e59b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd80e59e, SUC_lexicon_tags_17 },
    { 0x4500e59f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcca0e5a1, SUC_lexicon_tags_25 },
    { 0xba80e5a1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8120e5aa, SUC_lexicon_tags_25 },
    { 0xa920e5ab, SUC_lexicon_tags_57 },
    { 0xfee0e5ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65a0e5ae, SUC_lexicon_tags_19 },
    { 0x81a0e5af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e80e5b2, SUC_lexicon_tags_17 },
    { 0xb220e5b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4460e5b5, SUC_lexicon_tags_14 },
    { 0x41a0e5b6, SUC_lexicon_tags_17 },
    { 0x2c00e5b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3080e5b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5f20e5bb, SUC_lexicon_tags_7 },
    { 0x440e5bb, SUC_lexicon_tags_64 },
    { 0xcea0e5bc, SUC_lexicon_tags_25 },
    { 0xbce0e5be, SUC_lexicon_tags_48 },
    { 0x1540e5bb, SUC_lexicon_tags_15 },
    { 0xd40e5bf, SUC_lexicon_tags_25 },
    { 0x6840e5be, SUC_lexicon_tags_17 },
    { 0xe9a0e5c2, SUC_lexicon_tags_7 },
    { 0x62e0e5c3, SUC_lexicon_tags_1 },
    { 0xdb80e5bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a00e5c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7600e5cb, SUC_lexicon_tags_7 },
    { 0xa8c0e5cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2280e5d2, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x4060e5d4, SUC_lexicon_tags_77 },
    { 0x60e5d5, SUC_lexicon_tags_17 },
    { 0x8680e5d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e80e5db, SUC_lexicon_tags_11 },
    { 0x9de0e5db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f20e5e1, SUC_lexicon_tags_63 },
    { 0x8180e5e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c0e5e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d00e5e8, SUC_lexicon_tags_23 },
    { 0x5420e5e9, SUC_lexicon_tags_19 },
    { 0xdbc0e5e8, SUC_lexicon_tags_37 },
    { 0xaaa0e5eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c80e5ef, SUC_lexicon_tags_39 },
    { 0x6da0e5f0, SUC_lexicon_tags_3 },
    { 0xc40e5ef, SUC_lexicon_tags_16 },
    { 0xeee0e5ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ae0e5f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae0e5f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d60e5fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c0e5ff, SUC_lexicon_tags_15 },
    { 0x3540e600, SUC_lexicon_tags_47 },
    { 0x2580e601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa500e603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6200e606, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda0e60c, SUC_lexicon_tags_11 },
    { 0x82e0e60d, SUC_lexicon_tags_6 },
    { 0x8760e60e, SUC_lexicon_tags_37 },
    { 0x5d60e60d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa920e612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe420e615, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7000e61a, SUC_lexicon_tags_68 },
    { 0xa0a0e61b, SUC_lexicon_tags_25 },
    { 0xbd40e61b, SUC_lexicon_tags_1 },
    { 0x1a0e61d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1140e626, SUC_lexicon_tags_8 },
    { 0xb300e627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2100e629, SUC_lexicon_tags_28 },
    { 0x5460e62a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7380e62c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcce0e62e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde60e631, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da0e634, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6960e636, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e0e639, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3480e63c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9000e651, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9e0e653, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a0e656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2840e659, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3ba0e65b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a0e65e, SUC_lexicon_tags_38 },
    { 0x8080e65f, SUC_lexicon_tags_23 },
    { 0x9940e660, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd720e665, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1220e669, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd720e66b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1200e66e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x14e0e670, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4960e675, SUC_lexicon_tags_14 },
    { 0xb000e676, SUC_lexicon_tags_17 },
    { 0xc540e677, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1040e67d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7260e68a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a0e68d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6820e690, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a0e694, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a60e697, SUC_lexicon_tags_37 },
    { 0xbf20e698, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6140e69a, SUC_lexicon_tags_50 },
    { 0x68e0e69b, SUC_lexicon_tags_0 },
    { 0x7a60e69b, SUC_lexicon_tags_2 },
    { 0xd100e69d, SUC_lexicon_tags_4 },
    { 0xfae0e69e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab20e6a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf100e6a6, SUC_lexicon_tags_45 },
    { 0x4980e6a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c0e6ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3280e6af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdea0e6b1, SUC_lexicon_tags_12 },
    { 0xd80e6b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb580e6b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd680e6b9, SUC_lexicon_tags_12 },
    { 0xfb20e6ba, SUC_lexicon_tags_17 },
    { 0xffc0e6bb, SUC_lexicon_tags_46 },
    { 0x1ae0e6bc, SUC_lexicon_tags_11 },
    { 0x5f00e6bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe0e6c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a0e6c5, SUC_lexicon_tags_15 },
    { 0xae40e6c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe740e6ca, SUC_lexicon_tags_8 },
    { 0xb300e6cb, SUC_lexicon_tags_60 },
    { 0xc4a0e6cc, SUC_lexicon_tags_11 },
    { 0x7ca0e6cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c0e6d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f80e6d6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9600e6db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf420e6df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1400e6e5, SUC_lexicon_tags_151 },
    { 0x9140e6e5, SUC_lexicon_tags_21 },
    { 0x84a0e6e7, SUC_lexicon_tags_45 },
    { 0x9d40e6e8, SUC_lexicon_tags_6 },
    { 0xaf00e6e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd540e6f3, SUC_lexicon_tags_25 },
    { 0xbbc0e6f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7220e6f7, SUC_lexicon_tags_26 },
    { 0xdfa0e6f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8160e6fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c0e700, SUC_lexicon_tags_38 },
    { 0xfe00e701, SUC_lexicon_tags_2 },
    { 0x51a0e701, SUC_lexicon_tags_37 },
    { 0x55a0e700, SUC_lexicon_tags_43 },
    { 0xbf40e704, SUC_lexicon_tags_84 },
    { 0xc000e701, SUC_lexicon_tags_60 },
    { 0x37a0e701, SUC_lexicon_tags_15 },
    { 0xba80e707, SUC_lexicon_tags_23 },
    { 0xd260e706, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc0e70d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8520e715, SUC_lexicon_tags_10 },
    { 0xc180e716, SUC_lexicon_tags_1 },
    { 0x9e80e716, SUC_lexicon_tags_15 },
    { 0x3e80e718, SUC_lexicon_tags_55 },
    { 0x22c0e715, SUC_lexicon_tags_87 },
    { 0x66e0e715, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b80e71d, SUC_lexicon_tags_114 },
    { 0x7c80e71d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a0e723, SUC_lexicon_tags_43 },
    { 0x8a40e723, SUC_lexicon_tags_30 },
    { 0x3b60e724, SUC_lexicon_tags_60 },
    { 0x540e726, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x30a0e728, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1e0e72a, SUC_lexicon_tags_15 },
    { 0x25a0e72b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0e72e, SUC_lexicon_tags_1580 },
    { 0xae00e72f, SUC_lexicon_tags_27 },
    { 0xffe0e730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ae0e732, SUC_lexicon_tags_12 },
    { 0x6780e732, SUC_lexicon_tags_37 },
    { 0xa260e734, SUC_lexicon_tags_45 },
    { 0x2c20e732, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7800e739, SUC_lexicon_tags_23 },
    { 0xea60e73a, SUC_lexicon_tags_49 },
    { 0xe060e73b, SUC_lexicon_tags_44 },
    { 0x6c00e73a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4060e741, SUC_lexicon_tags_16 },
    { 0x45c0e742, SUC_lexicon_tags_25 },
    { 0x42c0e743, SUC_lexicon_tags_185 },
    { 0x3160e744, SUC_lexicon_tags_12 },
    { 0xc060e745, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1020e74e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d20e750, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6960e757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8900e75e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b20e769, SUC_lexicon_tags_25 },
    { 0x4e20e76a, SUC_lexicon_tags_28 },
    { 0xf060e76b, SUC_lexicon_tags_7 },
    { 0x52c0e76c, SUC_lexicon_tags_35 },
    { 0x57c0e769, SUC_lexicon_tags_25 },
    { 0x10e0e76c, SUC_lexicon_tags_17 },
    { 0xb960e76f, SUC_lexicon_tags_66 },
    { 0x9be0e770, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c60e77a, SUC_lexicon_tags_37 },
    { 0x6fa0e77b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6c0e77d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3700e782, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6340e786, SUC_lexicon_tags_0 },
    { 0x4820e787, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4960e78a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4240e78e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x72a0e790, SUC_lexicon_tags_7 },
    { 0x4e0e791, SUC_lexicon_tags_17 },
    { 0xd20e790, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc60e796, SUC_lexicon_tags_37 },
    { 0x58c0e797, SUC_lexicon_tags_7 },
    { 0xfe0e796, SUC_lexicon_tags_30 },
    { 0x6be0e796, SUC_lexicon_tags_15 },
    { 0xe760e79a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e40e79f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6440e7a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1660e7af, SUC_lexicon_tags_57 },
    { 0x340e7b0, SUC_lexicon_tags_17 },
    { 0x6220e7b0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x50a0e7b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa0e7b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8840e7bb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20e7bf, SUC_lexicon_tags_38 },
    { 0x7cc0e7c0, SUC_lexicon_tags_60 },
    { 0x7d40e7c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c0e7c6, SUC_lexicon_tags_49 },
    { 0x6c60e7c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0e7cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40e7ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6220e7d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fc0e7d2, SUC_lexicon_tags_271 },
    { 0, NULL },
    { 0xc4e0e7d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc0e7dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xba0e7df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ae0e7e1, SUC_lexicon_tags_11 },
    { 0x6360e7e2, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b00e7e5, SUC_lexicon_tags_129 },
    { 0x8880e7e6, SUC_lexicon_tags_15 },
    { 0x7780e7e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a80e7ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf060e7f1, SUC_lexicon_tags_26 },
    { 0x8400e7f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9280e7f5, SUC_lexicon_tags_43 },
    { 0xe20e7f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x86a0e7f8, SUC_lexicon_tags_21 },
    { 0xb0a0e7f8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d80e7fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf060e7ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce0e805, SUC_lexicon_tags_7 },
    { 0x1d00e806, SUC_lexicon_tags_37 },
    { 0xaee0e806, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc00e80c, SUC_lexicon_tags_17 },
    { 0x28a0e80c, SUC_lexicon_tags_32 },
    { 0x9300e80e, SUC_lexicon_tags_26 },
    { 0x35a0e80f, SUC_lexicon_tags_19 },
    { 0xc0e0e810, SUC_lexicon_tags_46 },
    { 0x8d40e811, SUC_lexicon_tags_146 },
    { 0x15a0e812, SUC_lexicon_tags_11 },
    { 0xfbe0e80c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdc0e815, SUC_lexicon_tags_26 },
    { 0x95a0e815, SUC_lexicon_tags_29 },
    { 0x5380e817, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee40e81c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ae0e81e, SUC_lexicon_tags_60 },
    { 0x9e60e81e, SUC_lexicon_tags_25 },
    { 0xa900e81e, SUC_lexicon_tags_6 },
    { 0x6320e81e, SUC_lexicon_tags_15 },
    { 0xd0e0e822, SUC_lexicon_tags_49 },
    { 0x31a0e823, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9800e826, SUC_lexicon_tags_31 },
    { 0x3b80e826, SUC_lexicon_tags_49 },
    { 0x56a0e828, SUC_lexicon_tags_301 },
    { 0xe560e829, SUC_lexicon_tags_9 },
    { 0x8780e829, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d80e82f, SUC_lexicon_tags_12 },
    { 0xdde0e830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac60e835, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c20e838, SUC_lexicon_tags_11 },
    { 0xe9c0e838, SUC_lexicon_tags_62 },
    { 0x7c40e83a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4ba0e83c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcc60e83e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa080e842, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd720e848, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5340e85c, SUC_lexicon_tags_48 },
    { 0x18a0e85c, SUC_lexicon_tags_25 },
    { 0x52c0e85e, SUC_lexicon_tags_14 },
    { 0x9380e85f, SUC_lexicon_tags_14 },
    { 0xb580e85e, SUC_lexicon_tags_33 },
    { 0xb000e860, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c60e868, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc080e86a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61c0e86c, SUC_lexicon_tags_49 },
    { 0x2f80e86d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x9c60e86f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb40e873, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f00e876, SUC_lexicon_tags_11 },
    { 0xace0e877, SUC_lexicon_tags_16 },
    { 0xac40e876, SUC_lexicon_tags_11 },
    { 0x400e878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce80e87c, SUC_lexicon_tags_7 },
    { 0xefc0e87c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc760e881, SUC_lexicon_tags_14 },
    { 0xf6a0e882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef80e884, SUC_lexicon_tags_15 },
    { 0x120e884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a0e888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0e0e88a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d00e88f, SUC_lexicon_tags_15 },
    { 0xbb00e890, SUC_lexicon_tags_15 },
    { 0x44c0e891, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac0e894, SUC_lexicon_tags_7 },
    { 0xada0e895, SUC_lexicon_tags_160 },
    { 0x320e896, SUC_lexicon_tags_12 },
    { 0x800e897, SUC_lexicon_tags_25 },
    { 0xc0a0e897, SUC_lexicon_tags_45 },
    { 0xee20e897, SUC_lexicon_tags_37 },
    { 0x10e0e89a, SUC_lexicon_tags_13 },
    { 0x9f20e89b, SUC_lexicon_tags_37 },
    { 0x9900e89c, SUC_lexicon_tags_27 },
    { 0x2080e89d, SUC_lexicon_tags_45 },
    { 0xa300e89d, SUC_lexicon_tags_32 },
    { 0x1a0e89d, SUC_lexicon_tags_87 },
    { 0x68e0e89e, SUC_lexicon_tags_2 },
    { 0xb020e8a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a0e8a4, SUC_lexicon_tags_70 },
    { 0xd760e8a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c60e8a7, SUC_lexicon_tags_16 },
    { 0xc320e8a8, SUC_lexicon_tags_6 },
    { 0xb200e8a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a00e8ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7860e8b1, SUC_lexicon_tags_87 },
    { 0x5bc0e8b2, SUC_lexicon_tags_2 },
    { 0x4600e8b3, SUC_lexicon_tags_49 },
    { 0xfac0e8b1, SUC_lexicon_tags_7 },
    { 0x6a0e8b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e0e8b8, SUC_lexicon_tags_4 },
    { 0xcce0e8b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc20e8bb, SUC_lexicon_tags_10 },
    { 0xb100e8bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20e8c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0e8c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7180e8cb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb720e8ce, SUC_lexicon_tags_12 },
    { 0x7660e8cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3100e8d1, SUC_lexicon_tags_8 },
    { 0x9620e8d2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee00e8d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xede0e8db, SUC_lexicon_tags_15 },
    { 0x7360e8db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d00e8e2, SUC_lexicon_tags_19 },
    { 0x6760e8e3, SUC_lexicon_tags_12 },
    { 0xd4e0e8e3, SUC_lexicon_tags_27 },
    { 0x2ba0e8e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6560e8e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9560e8ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1680e8f4, SUC_lexicon_tags_25 },
    { 0x5a40e8f4, SUC_lexicon_tags_38 },
    { 0xfd60e8f6, SUC_lexicon_tags_12 },
    { 0x6980e8f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c40e8fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe680e905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc280e907, SUC_lexicon_tags_43 },
    { 0x49a0e908, SUC_lexicon_tags_49 },
    { 0x7a40e909, SUC_lexicon_tags_2 },
    { 0xb3a0e908, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c0e90e, SUC_lexicon_tags_1 },
    { 0x5320e90e, SUC_lexicon_tags_2 },
    { 0xe3c0e910, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3ba0e912, SUC_lexicon_tags_1 },
    { 0x8980e912, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a60e918, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc0e91b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd160e921, SUC_lexicon_tags_44 },
    { 0x8b60e922, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6480e927, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a0e92a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb60e92c, SUC_lexicon_tags_85 },
    { 0x6b40e92c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9800e930, SUC_lexicon_tags_33 },
    { 0xb360e931, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c0e935, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfaa0e937, SUC_lexicon_tags_28 },
    { 0xf300e938, SUC_lexicon_tags_27 },
    { 0x9f00e939, SUC_lexicon_tags_10 },
    { 0x7740e93a, SUC_lexicon_tags_45 },
    { 0x2880e93a, SUC_lexicon_tags_47 },
    { 0x4680e938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf700e93e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c0e941, SUC_lexicon_tags_64 },
    { 0x7bc0e941, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb320e947, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4820e94b, SUC_lexicon_tags_63 },
    { 0x7220e94c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea0e950, SUC_lexicon_tags_16 },
    { 0xcda0e951, SUC_lexicon_tags_11 },
    { 0x5f40e950, SUC_lexicon_tags_17 },
    { 0x9a80e950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5640e959, SUC_lexicon_tags_25 },
    { 0x7c0e95a, SUC_lexicon_tags_45 },
    { 0xd260e95b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc220e95f, SUC_lexicon_tags_33 },
    { 0xf660e960, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d20e965, SUC_lexicon_tags_63 },
    { 0xdb00e965, SUC_lexicon_tags_8 },
    { 0xe300e966, SUC_lexicon_tags_14 },
    { 0x6840e966, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6d60e96a, SUC_lexicon_tags_11 },
    { 0x3a00e96a, SUC_lexicon_tags_17 },
    { 0x3a00e96c, SUC_lexicon_tags_12 },
    { 0x6080e96c, SUC_lexicon_tags_17 },
    { 0xeb40e96c, SUC_lexicon_tags_1 },
    { 0x7540e96c, SUC_lexicon_tags_87 },
    { 0xdce0e96e, SUC_lexicon_tags_11 },
    { 0xca0e971, SUC_lexicon_tags_42 },
    { 0x12e0e96e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae0e975, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb80e978, SUC_lexicon_tags_27 },
    { 0x3ee0e979, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa760e97b, SUC_lexicon_tags_6 },
    { 0x68e0e97c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c0e980, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4960e984, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x97e0e986, SUC_lexicon_tags_10 },
    { 0xf6e0e986, SUC_lexicon_tags_12 },
    { 0x99e0e988, SUC_lexicon_tags_15 },
    { 0x3da0e989, SUC_lexicon_tags_14 },
    { 0x2360e98a, SUC_lexicon_tags_37 },
    { 0xd520e988, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7b40e98d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8400e991, SUC_lexicon_tags_2 },
    { 0xd3e0e991, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc0e995, SUC_lexicon_tags_2 },
    { 0xfa80e996, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c0e999, SUC_lexicon_tags_49 },
    { 0x7040e99a, SUC_lexicon_tags_12 },
    { 0x2ce0e999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e0e9a2, SUC_lexicon_tags_27 },
    { 0x6b20e9a2, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc0e9a7, SUC_lexicon_tags_37 },
    { 0x9fa0e9a8, SUC_lexicon_tags_43 },
    { 0xb880e9a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3420e9ac, SUC_lexicon_tags_17 },
    { 0x480e9ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e9b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7480e9b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3440e9bb, SUC_lexicon_tags_37 },
    { 0x7f60e9bb, SUC_lexicon_tags_25 },
    { 0xcc0e9bc, SUC_lexicon_tags_108 },
    { 0x9c20e9be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f60e9c0, SUC_lexicon_tags_49 },
    { 0xd400e9c1, SUC_lexicon_tags_37 },
    { 0x8140e9c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4fa0e9c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1540e9cb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e20e9d7, SUC_lexicon_tags_1 },
    { 0x9ba0e9d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a0e9db, SUC_lexicon_tags_17 },
    { 0xfb20e9dc, SUC_lexicon_tags_11 },
    { 0xe5c0e9dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de0e9e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7700e9e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c80e9ec, SUC_lexicon_tags_23 },
    { 0x700e9ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9300e9ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x920e9f1, SUC_lexicon_tags_11 },
    { 0xd200e9f2, SUC_lexicon_tags_84 },
    { 0x8700e9f3, SUC_lexicon_tags_11 },
    { 0x54a0e9f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e0e9f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7180ea02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc0ea12, SUC_lexicon_tags_19 },
    { 0x4de0ea13, SUC_lexicon_tags_0 },
    { 0x9d00ea14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2000ea18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e0ea1c, SUC_lexicon_tags_47 },
    { 0x4220ea1d, SUC_lexicon_tags_33 },
    { 0x21a0ea1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5640ea21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c60ea23, SUC_lexicon_tags_47 },
    { 0xa1c0ea24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8520ea27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9a0ea29, SUC_lexicon_tags_1 },
    { 0xb3a0ea2a, SUC_lexicon_tags_28 },
    { 0x5720ea2b, SUC_lexicon_tags_17 },
    { 0x4dc0ea29, SUC_lexicon_tags_17 },
    { 0xdd40ea2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9300ea31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60ea3c, SUC_lexicon_tags_11 },
    { 0x74a0ea3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c0ea43, SUC_lexicon_tags_11 },
    { 0x7c60ea43, SUC_lexicon_tags_33 },
    { 0x96c0ea45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a0ea49, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xee00ea4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfee0ea4e, SUC_lexicon_tags_46 },
    { 0x8920ea4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5040ea51, SUC_lexicon_tags_6 },
    { 0xa440ea51, SUC_lexicon_tags_15 },
    { 0xd540ea53, SUC_lexicon_tags_19 },
    { 0x6ac0ea53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9140ea59, SUC_lexicon_tags_25 },
    { 0x1120ea5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a0ea5c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2c80ea5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da0ea61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a0ea66, SUC_lexicon_tags_17 },
    { 0x64c0ea67, SUC_lexicon_tags_7 },
    { 0xe0a0ea68, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x64e0ea6a, SUC_lexicon_tags_47 },
    { 0xb660ea6b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4640ea6f, SUC_lexicon_tags_534 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d20ea76, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00ea7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c0ea80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4320ea82, SUC_lexicon_tags_37 },
    { 0xa620ea83, SUC_lexicon_tags_877 },
    { 0, NULL },
    { 0, NULL },
    { 0x7100ea86, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c60ea8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7120ea92, SUC_lexicon_tags_19 },
    { 0x6a0ea92, SUC_lexicon_tags_25 },
    { 0x5f00ea94, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5f00ea96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9960ea99, SUC_lexicon_tags_33 },
    { 0x2ac0ea9a, SUC_lexicon_tags_69 },
    { 0x340ea9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa180ea9d, SUC_lexicon_tags_11 },
    { 0x9100ea9d, SUC_lexicon_tags_14 },
    { 0xe200ea9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c0eaa2, SUC_lexicon_tags_7 },
    { 0x88e0eaa3, SUC_lexicon_tags_14 },
    { 0x51a0eaa3, SUC_lexicon_tags_2 },
    { 0xba60eaa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0eab2, SUC_lexicon_tags_46 },
    { 0x10a0eab3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc00eab9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe0eabd, SUC_lexicon_tags_15 },
    { 0x8c60eabe, SUC_lexicon_tags_17 },
    { 0x97a0eabf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f00eac2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e0eac6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x78a0eac8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6580eacf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ca0ead1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e0eada, SUC_lexicon_tags_14 },
    { 0x2200eada, SUC_lexicon_tags_68 },
    { 0x5fe0eadb, SUC_lexicon_tags_17 },
    { 0xc1c0eadc, SUC_lexicon_tags_14 },
    { 0xfc60eade, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3460eae1, SUC_lexicon_tags_2 },
    { 0xd6e0eae2, SUC_lexicon_tags_63 },
    { 0xa60eae1, SUC_lexicon_tags_37 },
    { 0x2b00eae4, SUC_lexicon_tags_17 },
    { 0x4f00eae5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa3a0eae7, SUC_lexicon_tags_38 },
    { 0x7980eae8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e0eaeb, SUC_lexicon_tags_33 },
    { 0xb380eaec, SUC_lexicon_tags_65 },
    { 0xa2e0eaeb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4440eaf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2040eaf4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c0eaf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5880eafc, SUC_lexicon_tags_14 },
    { 0xb6a0eafd, SUC_lexicon_tags_37 },
    { 0x19e0eafd, SUC_lexicon_tags_47 },
    { 0xec40eaff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac40eb03, SUC_lexicon_tags_63 },
    { 0x7a80eb03, SUC_lexicon_tags_207 },
    { 0x4de0eb05, SUC_lexicon_tags_37 },
    { 0xc8a0eb03, SUC_lexicon_tags_37 },
    { 0x5e20eb03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d60eb09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4f00eb0b, SUC_lexicon_tags_185 },
    { 0x1780eb0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d20eb0f, SUC_lexicon_tags_69 },
    { 0xcc0eb10, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc0eb13, SUC_lexicon_tags_23 },
    { 0xcf00eb13, SUC_lexicon_tags_42 },
    { 0x2540eb15, SUC_lexicon_tags_7 },
    { 0x55c0eb16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6e0eb18, SUC_lexicon_tags_47 },
    { 0x4a0eb18, SUC_lexicon_tags_92 },
    { 0x5920eb1a, SUC_lexicon_tags_141 },
    { 0x1740eb1a, SUC_lexicon_tags_8 },
    { 0xe1c0eb1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf80eb1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c0eb23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xed00eb26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd780eb2a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa0eb2f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5980eb34, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0xe240eb36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b40eb39, SUC_lexicon_tags_15 },
    { 0x780eb39, SUC_lexicon_tags_15 },
    { 0x1740eb39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb00eb48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb500eb52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3300eb54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c40eb57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9100eb5a, SUC_lexicon_tags_17 },
    { 0xd6c0eb5b, SUC_lexicon_tags_38 },
    { 0x8260eb5b, SUC_lexicon_tags_37 },
    { 0x9d00eb5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf40eb5f, SUC_lexicon_tags_47 },
    { 0xf1e0eb60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a0eb65, SUC_lexicon_tags_1 },
    { 0x8080eb65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1760eb6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e40eb71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c60eb74, SUC_lexicon_tags_9 },
    { 0x9b00eb75, SUC_lexicon_tags_30 },
    { 0x4f00eb76, SUC_lexicon_tags_13 },
    { 0x2ac0eb77, SUC_lexicon_tags_87 },
    { 0x89e0eb76, SUC_lexicon_tags_25 },
    { 0x58e0eb76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20eb7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a0eb86, SUC_lexicon_tags_65 },
    { 0x8480eb87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c0eb8c, SUC_lexicon_tags_23 },
    { 0xff20eb8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c0eb90, SUC_lexicon_tags_11 },
    { 0xa20eb90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e0eb99, SUC_lexicon_tags_39 },
    { 0x2760eb9a, SUC_lexicon_tags_25 },
    { 0x9040eb9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x72a0eb9d, SUC_lexicon_tags_9 },
    { 0xbe00eb9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4620eba8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c60ebab, SUC_lexicon_tags_106 },
    { 0x100ebab, SUC_lexicon_tags_0 },
    { 0x24e0ebad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a0ebb0, SUC_lexicon_tags_23 },
    { 0xe100ebb1, SUC_lexicon_tags_30 },
    { 0x6740ebb2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbf80ebb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e00ebb6, SUC_lexicon_tags_33 },
    { 0x5ee0ebb7, SUC_lexicon_tags_47 },
    { 0xec60ebb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b40ebbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2360ebc1, SUC_lexicon_tags_11 },
    { 0x8de0ebc2, SUC_lexicon_tags_11 },
    { 0xe4c0ebc3, SUC_lexicon_tags_50 },
    { 0x5920ebc2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a0ebcd, SUC_lexicon_tags_37 },
    { 0xf5c0ebce, SUC_lexicon_tags_91 },
    { 0x93a0ebcf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c0ebd3, SUC_lexicon_tags_1 },
    { 0x73e0ebd3, SUC_lexicon_tags_5 },
    { 0xb660ebd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe8c0ebd7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf760ebdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc300ebde, SUC_lexicon_tags_9 },
    { 0x5c60ebde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a0ebe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7020ebe4, SUC_lexicon_tags_21 },
    { 0x8500ebe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ca0ebe7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c0ebed, SUC_lexicon_tags_37 },
    { 0xd80ebee, SUC_lexicon_tags_5 },
    { 0x1fc0ebef, SUC_lexicon_tags_79 },
    { 0x9ce0ebf0, SUC_lexicon_tags_33 },
    { 0x3680ebf1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8040ebf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9120ebf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd700ebfb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e0ebfe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4e0ec00, SUC_lexicon_tags_15 },
    { 0xa9e0ec01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd600ec04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf480ec07, SUC_lexicon_tags_45 },
    { 0x3520ec07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900ec0d, SUC_lexicon_tags_57 },
    { 0x4a40ec0e, SUC_lexicon_tags_25 },
    { 0xaf20ec0f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2080ec11, SUC_lexicon_tags_17 },
    { 0x7540ec12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2420ec14, SUC_lexicon_tags_7 },
    { 0x3120ec15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xec80ec18, SUC_lexicon_tags_37 },
    { 0x2a80ec19, SUC_lexicon_tags_43 },
    { 0x3c60ec1a, SUC_lexicon_tags_28 },
    { 0xa7a0ec1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8420ec1e, SUC_lexicon_tags_37 },
    { 0x7ec0ec1f, SUC_lexicon_tags_2 },
    { 0xd260ec20, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0ec23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a00ec29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe380ec2b, SUC_lexicon_tags_17 },
    { 0xd220ec2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba0ec31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc940ec36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x38c0ec38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b80ec3a, SUC_lexicon_tags_29 },
    { 0xb660ec3b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e0ec40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a20ec44, SUC_lexicon_tags_14 },
    { 0xf2e0ec45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xda00ec47, SUC_lexicon_tags_30 },
    { 0xd520ec48, SUC_lexicon_tags_12 },
    { 0x3860ec48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c40ec4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a80ec4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf020ec51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe8a0ec53, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe360ec59, SUC_lexicon_tags_46 },
    { 0xffa0ec5a, SUC_lexicon_tags_1 },
    { 0x86e0ec59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf580ec5e, SUC_lexicon_tags_19 },
    { 0xbd60ec5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc100ec61, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x7420ec64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9c0ec66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740ec6c, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa840ec73, SUC_lexicon_tags_17 },
    { 0x6260ec73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf40ec79, SUC_lexicon_tags_15 },
    { 0xa280ec7a, SUC_lexicon_tags_12 },
    { 0xcac0ec7a, SUC_lexicon_tags_17 },
    { 0xedc0ec7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba80ec82, SUC_lexicon_tags_37 },
    { 0xc020ec82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e60ec8a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf020ec91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdec0ec94, SUC_lexicon_tags_32 },
    { 0xf940ec95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c40ec9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec0eca0, SUC_lexicon_tags_7 },
    { 0x4860eca1, SUC_lexicon_tags_151 },
    { 0xaea0eca1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0eca8, SUC_lexicon_tags_17 },
    { 0xb620eca9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf360ecac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2940ecb4, SUC_lexicon_tags_11 },
    { 0x54e0ecb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe580ecbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e40ecc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d80ecc6, SUC_lexicon_tags_16 },
    { 0x1940ecc7, SUC_lexicon_tags_11 },
    { 0x5820ecc7, SUC_lexicon_tags_0 },
    { 0x6560ecc8, SUC_lexicon_tags_10 },
    { 0x8620ecc7, SUC_lexicon_tags_47 },
    { 0xf740eccb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e0ecce, SUC_lexicon_tags_47 },
    { 0xa6e0ecce, SUC_lexicon_tags_79 },
    { 0xc040ecd0, SUC_lexicon_tags_11 },
    { 0x36e0ecd1, SUC_lexicon_tags_47 },
    { 0x2be0ecd1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcaa0ecd4, SUC_lexicon_tags_47 },
    { 0xbb40ecd5, SUC_lexicon_tags_17 },
    { 0x46e0ecd6, SUC_lexicon_tags_1 },
    { 0xeca0ecd5, SUC_lexicon_tags_47 },
    { 0x8a80ecd8, SUC_lexicon_tags_39 },
    { 0x3220ecd9, SUC_lexicon_tags_7 },
    { 0xb00ecd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30a0ecdc, SUC_lexicon_tags_27 },
    { 0xa660ecdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa760ecdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c0ece4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb00ece6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd20eceb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf780ecee, SUC_lexicon_tags_2 },
    { 0xf0a0ecef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4b80ecf1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca0ecf6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a0ecf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd00ecfb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa0ed01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e0ed04, SUC_lexicon_tags_11 },
    { 0xd4e0ed05, SUC_lexicon_tags_15 },
    { 0xf680ed04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5540ed09, SUC_lexicon_tags_23 },
    { 0xb140ed0a, SUC_lexicon_tags_46 },
    { 0xcd00ed0b, SUC_lexicon_tags_17 },
    { 0x4ec0ed0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa0ed10, SUC_lexicon_tags_1 },
    { 0x4460ed11, SUC_lexicon_tags_7 },
    { 0xf580ed10, SUC_lexicon_tags_0 },
    { 0x9e0ed13, SUC_lexicon_tags_56 },
    { 0x1ce0ed14, SUC_lexicon_tags_31 },
    { 0xebe0ed10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa180ed1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb200ed1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee80ed20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe0ed22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc0ed26, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5740ed2b, SUC_lexicon_tags_11 },
    { 0x3920ed2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6540ed2e, SUC_lexicon_tags_7 },
    { 0x5520ed2f, SUC_lexicon_tags_29 },
    { 0x12e0ed30, SUC_lexicon_tags_1 },
    { 0x8040ed31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e0ed35, SUC_lexicon_tags_11 },
    { 0xe60ed36, SUC_lexicon_tags_37 },
    { 0x1ec0ed37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf20ed39, SUC_lexicon_tags_13 },
    { 0x9680ed3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6180ed3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c0ed43, SUC_lexicon_tags_34 },
    { 0x8060ed44, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7880ed48, SUC_lexicon_tags_14 },
    { 0xc2e0ed49, SUC_lexicon_tags_11 },
    { 0x2b60ed4a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8b60ed4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45a0ed4e, SUC_lexicon_tags_37 },
    { 0xcd60ed4f, SUC_lexicon_tags_37 },
    { 0x98c0ed50, SUC_lexicon_tags_12 },
    { 0x300ed4f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f00ed54, SUC_lexicon_tags_25 },
    { 0x29c0ed55, SUC_lexicon_tags_1 },
    { 0x10e0ed54, SUC_lexicon_tags_69 },
    { 0x6be0ed55, SUC_lexicon_tags_27 },
    { 0xc580ed57, SUC_lexicon_tags_17 },
    { 0xf9e0ed54, SUC_lexicon_tags_1 },
    { 0x5b60ed5a, SUC_lexicon_tags_31 },
    { 0xf320ed5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b40ed60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce0ed69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5880ed6d, SUC_lexicon_tags_1 },
    { 0xaf40ed6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1500ed70, SUC_lexicon_tags_7 },
    { 0x70e0ed70, SUC_lexicon_tags_8 },
    { 0x4840ed72, SUC_lexicon_tags_1 },
    { 0xa7a0ed71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2e0ed75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c0ed7a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x6bc0ed7c, SUC_lexicon_tags_17 },
    { 0xfc0ed7d, SUC_lexicon_tags_25 },
    { 0x7180ed7d, SUC_lexicon_tags_9 },
    { 0x9380ed7d, SUC_lexicon_tags_23 },
    { 0x44c0ed80, SUC_lexicon_tags_27 },
    { 0xd920ed81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec60ed85, SUC_lexicon_tags_8 },
    { 0x4320ed86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1320ed89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x860ed8b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb660ed8e, SUC_lexicon_tags_29 },
    { 0x2140ed8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf60ed93, SUC_lexicon_tags_27 },
    { 0xf620ed93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a20ed96, SUC_lexicon_tags_63 },
    { 0x3600ed97, SUC_lexicon_tags_28 },
    { 0x8180ed97, SUC_lexicon_tags_25 },
    { 0xec0ed96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd060ed9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d60eda3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb00eda5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7100eda8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a0edab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1940edad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e0edb1, SUC_lexicon_tags_146 },
    { 0xd220edb2, SUC_lexicon_tags_11 },
    { 0xbf00edb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3720edb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6a0edb7, SUC_lexicon_tags_49 },
    { 0x26e0edb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa720edbd, SUC_lexicon_tags_87 },
    { 0x6620edbe, SUC_lexicon_tags_9 },
    { 0xf440edbe, SUC_lexicon_tags_29 },
    { 0x2080edbf, SUC_lexicon_tags_49 },
    { 0xa20edbe, SUC_lexicon_tags_37 },
    { 0x51a0edbf, SUC_lexicon_tags_47 },
    { 0xc8c0edc3, SUC_lexicon_tags_43 },
    { 0xbd20edbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20c0edc6, SUC_lexicon_tags_1 },
    { 0x51e0edc6, SUC_lexicon_tags_11 },
    { 0xfe40edc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e40edca, SUC_lexicon_tags_31 },
    { 0x32c0edca, SUC_lexicon_tags_4 },
    { 0x29a0edcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8880edce, SUC_lexicon_tags_1 },
    { 0x8da0edce, SUC_lexicon_tags_37 },
    { 0x5e00edce, SUC_lexicon_tags_2 },
    { 0xd460edd1, SUC_lexicon_tags_2 },
    { 0xb580edd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5100edd4, SUC_lexicon_tags_25 },
    { 0xe400edd5, SUC_lexicon_tags_87 },
    { 0x95c0edd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da0edda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f00eddc, SUC_lexicon_tags_17 },
    { 0x5800eddd, SUC_lexicon_tags_0 },
    { 0xdf00eddd, SUC_lexicon_tags_1 },
    { 0x9580eddf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6080ede4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e0ede9, SUC_lexicon_tags_68 },
    { 0x3ec0edea, SUC_lexicon_tags_15 },
    { 0x1460edeb, SUC_lexicon_tags_33 },
    { 0x540edeb, SUC_lexicon_tags_1 },
    { 0x4780edeb, SUC_lexicon_tags_47 },
    { 0x7420edee, SUC_lexicon_tags_23 },
    { 0xa440edea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec60edf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd0a0edfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf80edfd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x43a0edff, SUC_lexicon_tags_49 },
    { 0x800ee00, SUC_lexicon_tags_82 },
    { 0xe6a0ee00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc40ee03, SUC_lexicon_tags_2 },
    { 0xc80ee04, SUC_lexicon_tags_0 },
    { 0xe600ee03, SUC_lexicon_tags_38 },
    { 0x13a0ee06, SUC_lexicon_tags_25 },
    { 0x6c20ee07, SUC_lexicon_tags_46 },
    { 0xd3e0ee07, SUC_lexicon_tags_25 },
    { 0x3f60ee08, SUC_lexicon_tags_17 },
    { 0xc9e0ee03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa20ee0f, SUC_lexicon_tags_8 },
    { 0x1700ee10, SUC_lexicon_tags_57 },
    { 0x1a60ee11, SUC_lexicon_tags_14 },
    { 0x5660ee12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb600ee16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7640ee18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9280ee1b, SUC_lexicon_tags_34 },
    { 0x840ee1c, SUC_lexicon_tags_8 },
    { 0xdee0ee1c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c0ee20, SUC_lexicon_tags_37 },
    { 0x23e0ee21, SUC_lexicon_tags_60 },
    { 0xa1a0ee22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea0ee25, SUC_lexicon_tags_40 },
    { 0x3780ee26, SUC_lexicon_tags_30 },
    { 0x4c20ee27, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb540ee2b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x840ee2e, SUC_lexicon_tags_15 },
    { 0x5c20ee2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea0ee35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b40ee37, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0xd400ee3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfaa0ee3c, SUC_lexicon_tags_19 },
    { 0x81e0ee3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a0ee4b, SUC_lexicon_tags_4 },
    { 0x3ea0ee4c, SUC_lexicon_tags_7 },
    { 0xd4c0ee4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43e0ee4f, SUC_lexicon_tags_37 },
    { 0x7c80ee4f, SUC_lexicon_tags_11 },
    { 0x53a0ee51, SUC_lexicon_tags_17 },
    { 0x260ee51, SUC_lexicon_tags_15 },
    { 0xd6e0ee53, SUC_lexicon_tags_56 },
    { 0xcfc0ee52, SUC_lexicon_tags_31 },
    { 0x8b20ee55, SUC_lexicon_tags_15 },
    { 0xf240ee54, SUC_lexicon_tags_12 },
    { 0xffe0ee51, SUC_lexicon_tags_4 },
    { 0xf1e0ee58, SUC_lexicon_tags_8 },
    { 0xb2a0ee58, SUC_lexicon_tags_37 },
    { 0x9e00ee58, SUC_lexicon_tags_11 },
    { 0xcb00ee51, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c0ee5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1300ee60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9260ee67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe520ee6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e80ee6c, SUC_lexicon_tags_25 },
    { 0xf260ee6c, SUC_lexicon_tags_38 },
    { 0xc180ee6d, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xa9c0ee70, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x3960ee72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce20ee74, SUC_lexicon_tags_27 },
    { 0x69e0ee75, SUC_lexicon_tags_45 },
    { 0x9de0ee75, SUC_lexicon_tags_151 },
    { 0xd400ee77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3dc0ee79, SUC_lexicon_tags_17 },
    { 0xbfc0ee7a, SUC_lexicon_tags_27 },
    { 0xb640ee7b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa0ee7f, SUC_lexicon_tags_23 },
    { 0xe260ee80, SUC_lexicon_tags_1 },
    { 0x8000ee81, SUC_lexicon_tags_60 },
    { 0x42a0ee82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe240ee84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce0ee8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a0ee8d, SUC_lexicon_tags_6 },
    { 0xf60ee8d, SUC_lexicon_tags_25 },
    { 0x70e0ee8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b00ee97, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa420ee99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe0ee9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e00eea1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x53e0eea3, SUC_lexicon_tags_43 },
    { 0x7020eea4, SUC_lexicon_tags_63 },
    { 0x6e80eea5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfc0eea7, SUC_lexicon_tags_2 },
    { 0x9360eea8, SUC_lexicon_tags_15 },
    { 0xb6e0eea7, SUC_lexicon_tags_37 },
    { 0x8b80eeaa, SUC_lexicon_tags_11 },
    { 0x9ac0eea9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160eeb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb20eeb2, SUC_lexicon_tags_35 },
    { 0x3a0eeb3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d60eeb7, SUC_lexicon_tags_23 },
    { 0xf6a0eeb8, SUC_lexicon_tags_43 },
    { 0x72c0eeb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa20eebf, SUC_lexicon_tags_65 },
    { 0x8a0eec0, SUC_lexicon_tags_30 },
    { 0x6cc0eec1, SUC_lexicon_tags_60 },
    { 0xdf00eec0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40eec6, SUC_lexicon_tags_23 },
    { 0x360eec7, SUC_lexicon_tags_30 },
    { 0x9360eec6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5400eece, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba0eed4, SUC_lexicon_tags_16 },
    { 0xc1a0eed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd060eed7, SUC_lexicon_tags_6 },
    { 0x13c0eed7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8400eede, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3100eee1, SUC_lexicon_tags_54 },
    { 0x5740eee2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x37e0eee4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e00eee6, SUC_lexicon_tags_11 },
    { 0x5de0eee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c40eee9, SUC_lexicon_tags_31 },
    { 0x4300eee9, SUC_lexicon_tags_154 },
    { 0xa020eeeb, SUC_lexicon_tags_15 },
    { 0x2c60eeec, SUC_lexicon_tags_14 },
    { 0xa720eeed, SUC_lexicon_tags_67 },
    { 0xd1a0eeee, SUC_lexicon_tags_6 },
    { 0x6a80eeef, SUC_lexicon_tags_25 },
    { 0x96e0eeee, SUC_lexicon_tags_8 },
    { 0x26c0eeed, SUC_lexicon_tags_37 },
    { 0x1ac0eef2, SUC_lexicon_tags_1 },
    { 0x53c0eef3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf980eef5, SUC_lexicon_tags_15 },
    { 0x54c0eef5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a0eef9, SUC_lexicon_tags_105 },
    { 0xe3c0eefa, SUC_lexicon_tags_31 },
    { 0xd3c0eefa, SUC_lexicon_tags_6 },
    { 0x88e0eefa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6320ef04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe060ef06, SUC_lexicon_tags_63 },
    { 0x3c00ef07, SUC_lexicon_tags_39 },
    { 0xb00ef08, SUC_lexicon_tags_47 },
    { 0x7800ef09, SUC_lexicon_tags_5 },
    { 0x24c0ef0a, SUC_lexicon_tags_3 },
    { 0x6020ef07, SUC_lexicon_tags_1 },
    { 0xf640ef0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3020ef10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcac0ef12, SUC_lexicon_tags_84 },
    { 0xb60ef13, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x100ef15, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb880ef19, SUC_lexicon_tags_10 },
    { 0x8480ef1a, SUC_lexicon_tags_37 },
    { 0x8fa0ef1b, SUC_lexicon_tags_23 },
    { 0xd900ef1c, SUC_lexicon_tags_11 },
    { 0x9fe0ef1d, SUC_lexicon_tags_43 },
    { 0xfee0ef1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2360ef21, SUC_lexicon_tags_33 },
    { 0xb880ef21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5fe0ef24, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe280ef26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfbc0ef28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacc0ef2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfba0ef2c, SUC_lexicon_tags_1 },
    { 0x3540ef2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa60ef33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e0ef39, SUC_lexicon_tags_16 },
    { 0x5000ef39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0ef3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6100ef46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae40ef4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6100ef4f, SUC_lexicon_tags_43 },
    { 0x29c0ef50, SUC_lexicon_tags_7 },
    { 0x8500ef51, SUC_lexicon_tags_30 },
    { 0x7240ef50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0ef5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e0ef5e, SUC_lexicon_tags_25 },
    { 0x7b20ef5e, SUC_lexicon_tags_38 },
    { 0xd0c0ef60, SUC_lexicon_tags_1 },
    { 0xa0c0ef60, SUC_lexicon_tags_14 },
    { 0x30a0ef61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec0ef65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3520ef69, SUC_lexicon_tags_12 },
    { 0x7de0ef6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e00ef6f, SUC_lexicon_tags_14 },
    { 0x7140ef70, SUC_lexicon_tags_2 },
    { 0xcc60ef6f, SUC_lexicon_tags_84 },
    { 0xc740ef72, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd380ef78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a00ef7b, SUC_lexicon_tags_23 },
    { 0x3ce0ef7c, SUC_lexicon_tags_47 },
    { 0x57e0ef7d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa740ef81, SUC_lexicon_tags_17 },
    { 0xdac0ef82, SUC_lexicon_tags_2 },
    { 0x1640ef83, SUC_lexicon_tags_25 },
    { 0x9a00ef84, SUC_lexicon_tags_46 },
    { 0xdae0ef81, SUC_lexicon_tags_12 },
    { 0x8d00ef86, SUC_lexicon_tags_29 },
    { 0xada0ef87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa0ef8c, SUC_lexicon_tags_17 },
    { 0x9e40ef8d, SUC_lexicon_tags_3 },
    { 0xe60ef8e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc720ef95, SUC_lexicon_tags_34 },
    { 0xf1e0ef96, SUC_lexicon_tags_17 },
    { 0x4720ef97, SUC_lexicon_tags_17 },
    { 0x2740ef98, SUC_lexicon_tags_21 },
    { 0x7400ef97, SUC_lexicon_tags_17 },
    { 0xbac0ef99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c0ef9e, SUC_lexicon_tags_11 },
    { 0x5840ef9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc0efa8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec0efad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2860efaf, SUC_lexicon_tags_45 },
    { 0x2140efb0, SUC_lexicon_tags_60 },
    { 0x8500efb1, SUC_lexicon_tags_47 },
    { 0x1da0efb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e80efb6, SUC_lexicon_tags_14 },
    { 0x4fa0efb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e40efb9, SUC_lexicon_tags_37 },
    { 0x1a20efba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc0efc3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc020efc6, SUC_lexicon_tags_12 },
    { 0x4260efc7, SUC_lexicon_tags_25 },
    { 0x9b60efc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1980efcb, SUC_lexicon_tags_68 },
    { 0xf1e0efcb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f00efd0, SUC_lexicon_tags_33 },
    { 0xa800efd1, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xaa20efd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf80efd6, SUC_lexicon_tags_7 },
    { 0xfb60efd6, SUC_lexicon_tags_17 },
    { 0x8780efd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80efda, SUC_lexicon_tags_1 },
    { 0x8620efdb, SUC_lexicon_tags_23 },
    { 0x40a0efdc, SUC_lexicon_tags_87 },
    { 0xa6c0efdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc960efe0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7100efe6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca0efec, SUC_lexicon_tags_18 },
    { 0x1d60efec, SUC_lexicon_tags_15 },
    { 0x5060efed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e00eff0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e0eff4, SUC_lexicon_tags_37 },
    { 0x7ae0eff4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e0effe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda40f001, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4360f003, SUC_lexicon_tags_0 },
    { 0x3b80f004, SUC_lexicon_tags_46 },
    { 0x2000f003, SUC_lexicon_tags_15 },
    { 0x5720f006, SUC_lexicon_tags_37 },
    { 0x4220f006, SUC_lexicon_tags_45 },
    { 0x4ae0f008, SUC_lexicon_tags_15 },
    { 0x9560f003, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x4760f00b, SUC_lexicon_tags_25 },
    { 0x90c0f00c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea0f010, SUC_lexicon_tags_25 },
    { 0x6a60f010, SUC_lexicon_tags_2 },
    { 0xccc0f012, SUC_lexicon_tags_37 },
    { 0x7ae0f013, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc280f016, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e0f019, SUC_lexicon_tags_11 },
    { 0x9ca0f01a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5240f01c, SUC_lexicon_tags_15 },
    { 0x5c0f01d, SUC_lexicon_tags_79 },
    { 0x9660f01e, SUC_lexicon_tags_37 },
    { 0xae20f01e, SUC_lexicon_tags_14 },
    { 0x7020f01f, SUC_lexicon_tags_25 },
    { 0xeee0f021, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d00f028, SUC_lexicon_tags_25 },
    { 0x20e0f028, SUC_lexicon_tags_7 },
    { 0x5dc0f02a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f20f036, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c0f039, SUC_lexicon_tags_1 },
    { 0x8a40f039, SUC_lexicon_tags_11 },
    { 0xb5e0f03b, SUC_lexicon_tags_2 },
    { 0x3520f03b, SUC_lexicon_tags_25 },
    { 0x1120f03c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1700f03f, SUC_lexicon_tags_1 },
    { 0x4380f040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb00f047, SUC_lexicon_tags_11 },
    { 0x67e0f047, SUC_lexicon_tags_15 },
    { 0x6aa0f049, SUC_lexicon_tags_63 },
    { 0xf7e0f04a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42e0f04c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a00f04f, SUC_lexicon_tags_1 },
    { 0x2880f04f, SUC_lexicon_tags_7 },
    { 0x5fc0f04f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa0f054, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe960f059, SUC_lexicon_tags_23 },
    { 0x96c0f05a, SUC_lexicon_tags_1 },
    { 0xdfa0f05a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c00f05e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40f061, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4320f066, SUC_lexicon_tags_80 },
    { 0x9fc0f067, SUC_lexicon_tags_27 },
    { 0x4ae0f068, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a60f06b, SUC_lexicon_tags_1 },
    { 0x1260f06b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d00f075, SUC_lexicon_tags_17 },
    { 0xb9e0f076, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7440f078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d40f07a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b60f080, SUC_lexicon_tags_30 },
    { 0x7800f080, SUC_lexicon_tags_11 },
    { 0x3fc0f082, SUC_lexicon_tags_38 },
    { 0xbfe0f083, SUC_lexicon_tags_43 },
    { 0x5e0f084, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfb00f086, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa540f088, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d80f08d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3e0f08f, SUC_lexicon_tags_1 },
    { 0x77e0f090, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c0f094, SUC_lexicon_tags_2 },
    { 0xea60f095, SUC_lexicon_tags_27 },
    { 0xdbe0f096, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec0f098, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a0f09b, SUC_lexicon_tags_2 },
    { 0x4960f09c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1320f0a0, SUC_lexicon_tags_11 },
    { 0x540f0a1, SUC_lexicon_tags_15 },
    { 0xe080f0a2, SUC_lexicon_tags_37 },
    { 0x4b40f0a2, SUC_lexicon_tags_15 },
    { 0xdb00f0a2, SUC_lexicon_tags_37 },
    { 0xbbe0f0a5, SUC_lexicon_tags_17 },
    { 0x1e40f0a5, SUC_lexicon_tags_65 },
    { 0x7d40f0a7, SUC_lexicon_tags_47 },
    { 0x50a0f0a8, SUC_lexicon_tags_79 },
    { 0x18e0f0a5, SUC_lexicon_tags_15 },
    { 0xff80f0aa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8060f0ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3520f0b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8300f0b8, SUC_lexicon_tags_0 },
    { 0xa620f0b9, SUC_lexicon_tags_11 },
    { 0x5600f0ba, SUC_lexicon_tags_17 },
    { 0x40a0f0b8, SUC_lexicon_tags_34 },
    { 0xa500f0bc, SUC_lexicon_tags_37 },
    { 0x6ce0f0bd, SUC_lexicon_tags_15 },
    { 0xe8c0f0b9, SUC_lexicon_tags_19 },
    { 0xd140f0bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb5e0f0c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2040f0c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5ca0f0c5, SUC_lexicon_tags_12 },
    { 0x95e0f0c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe980f0c8, SUC_lexicon_tags_30 },
    { 0x9760f0c9, SUC_lexicon_tags_39 },
    { 0x3080f0ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e0f0d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd1e0f0d5, SUC_lexicon_tags_129 },
    { 0xe660f0d5, SUC_lexicon_tags_14 },
    { 0x9be0f0d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b20f0db, SUC_lexicon_tags_11 },
    { 0x7c60f0db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa0f0e0, SUC_lexicon_tags_15 },
    { 0xd180f0e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6640f0e4, SUC_lexicon_tags_12 },
    { 0x9340f0e4, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xfee0f0e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac00f0e9, SUC_lexicon_tags_2 },
    { 0x59e0f0e9, SUC_lexicon_tags_45 },
    { 0x9220f0e9, SUC_lexicon_tags_37 },
    { 0x2e80f0ec, SUC_lexicon_tags_46 },
    { 0x9720f0ed, SUC_lexicon_tags_43 },
    { 0xb300f0ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8040f0f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e0f0f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c0f0f9, SUC_lexicon_tags_19 },
    { 0x93c0f0f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd380f0fe, SUC_lexicon_tags_63 },
    { 0x7e0f0fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf940f103, SUC_lexicon_tags_14 },
    { 0xef00f104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f40f108, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc0f10d, SUC_lexicon_tags_14 },
    { 0x7d40f10e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9160f112, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef40f116, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9420f118, SUC_lexicon_tags_17 },
    { 0x6340f119, SUC_lexicon_tags_16 },
    { 0x6ee0f119, SUC_lexicon_tags_87 },
    { 0xb5c0f11b, SUC_lexicon_tags_7 },
    { 0xf2c0f11c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3560f123, SUC_lexicon_tags_14 },
    { 0x3720f124, SUC_lexicon_tags_37 },
    { 0x83c0f125, SUC_lexicon_tags_8 },
    { 0xd3e0f126, SUC_lexicon_tags_47 },
    { 0x63e0f125, SUC_lexicon_tags_37 },
    { 0xfda0f124, SUC_lexicon_tags_2 },
    { 0xdfe0f129, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2960f12e, SUC_lexicon_tags_1 },
    { 0xa580f12f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x15a0f131, SUC_lexicon_tags_55 },
    { 0x3900f132, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe780f138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc520f13d, SUC_lexicon_tags_7 },
    { 0x5f60f13e, SUC_lexicon_tags_15 },
    { 0x4580f13e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c80f142, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xbe60f144, SUC_lexicon_tags_66 },
    { 0xf960f145, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8820f14f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a0f154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e0f157, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb140f15d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e0f160, SUC_lexicon_tags_39 },
    { 0xa500f160, SUC_lexicon_tags_39 },
    { 0x8940f161, SUC_lexicon_tags_2 },
    { 0xc0c0f162, SUC_lexicon_tags_15 },
    { 0x9f40f164, SUC_lexicon_tags_2 },
    { 0x6ca0f165, SUC_lexicon_tags_31 },
    { 0x1000f165, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5960f16b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d00f16d, SUC_lexicon_tags_14 },
    { 0x8da0f16d, SUC_lexicon_tags_0 },
    { 0xb2a0f16f, SUC_lexicon_tags_7 },
    { 0x5480f16e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d60f175, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7720f179, SUC_lexicon_tags_87 },
    { 0xf9c0f179, SUC_lexicon_tags_16 },
    { 0xbb20f17b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce0f180, SUC_lexicon_tags_70 },
    { 0x2480f180, SUC_lexicon_tags_30 },
    { 0x3660f181, SUC_lexicon_tags_44 },
    { 0x4860f183, SUC_lexicon_tags_0 },
    { 0x7500f184, SUC_lexicon_tags_2 },
    { 0xa0e0f184, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa700f18e, SUC_lexicon_tags_15 },
    { 0xb7e0f18f, SUC_lexicon_tags_19 },
    { 0x6040f18e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a0f193, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6280f195, SUC_lexicon_tags_2 },
    { 0xa9c0f195, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8360f19c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x980f19f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f60f1a2, SUC_lexicon_tags_9 },
    { 0x8240f1a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd420f1a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b60f1ac, SUC_lexicon_tags_49 },
    { 0x7320f1ac, SUC_lexicon_tags_7 },
    { 0x8c40f1ae, SUC_lexicon_tags_29 },
    { 0x2d60f1af, SUC_lexicon_tags_17 },
    { 0xd60f1b0, SUC_lexicon_tags_25 },
    { 0x70c0f1b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd3a0f1b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa220f1b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d00f1c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f80f1c3, SUC_lexicon_tags_11 },
    { 0x9460f1c4, SUC_lexicon_tags_70 },
    { 0xc1a0f1c4, SUC_lexicon_tags_15 },
    { 0xe80f1c6, SUC_lexicon_tags_336 },
    { 0x3a0f1c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf40f1cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a0f1cf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c0f1d3, SUC_lexicon_tags_2 },
    { 0xea80f1d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b20f1d6, SUC_lexicon_tags_47 },
    { 0x6240f1d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa0f1df, SUC_lexicon_tags_25 },
    { 0x1960f1e0, SUC_lexicon_tags_1 },
    { 0xee80f1df, SUC_lexicon_tags_14 },
    { 0xfdc0f1e1, SUC_lexicon_tags_14 },
    { 0xc380f1e3, SUC_lexicon_tags_69 },
    { 0xadc0f1e1, SUC_lexicon_tags_31 },
    { 0x66a0f1e2, SUC_lexicon_tags_27 },
    { 0x3de0f1e6, SUC_lexicon_tags_16 },
    { 0x81e0f1e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf60f1ea, SUC_lexicon_tags_12 },
    { 0x60f1eb, SUC_lexicon_tags_67 },
    { 0xeb00f1ea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9e00f1ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e60f1f0, SUC_lexicon_tags_37 },
    { 0xdae0f1f0, SUC_lexicon_tags_2 },
    { 0x220f1f1, SUC_lexicon_tags_9 },
    { 0xbd00f1f3, SUC_lexicon_tags_2 },
    { 0x2000f1f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00f1f8, SUC_lexicon_tags_27 },
    { 0xc260f1f9, SUC_lexicon_tags_27 },
    { 0x4fa0f1f8, SUC_lexicon_tags_0 },
    { 0xa440f1f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb140f201, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c40f207, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ee0f209, SUC_lexicon_tags_17 },
    { 0x29e0f20a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4480f20f, SUC_lexicon_tags_11 },
    { 0xbc20f20f, SUC_lexicon_tags_17 },
    { 0xeda0f211, SUC_lexicon_tags_17 },
    { 0xe620f20f, SUC_lexicon_tags_15 },
    { 0xf180f213, SUC_lexicon_tags_7 },
    { 0x29a0f214, SUC_lexicon_tags_25 },
    { 0x4b00f215, SUC_lexicon_tags_107 },
    { 0xf560f216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8220f21b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3340f21d, SUC_lexicon_tags_15 },
    { 0x1bc0f21e, SUC_lexicon_tags_77 },
    { 0x36c0f21e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7160f222, SUC_lexicon_tags_39 },
    { 0x8ca0f222, SUC_lexicon_tags_15 },
    { 0x5bc0f224, SUC_lexicon_tags_12 },
    { 0x54e0f222, SUC_lexicon_tags_38 },
    { 0x1ba0f222, SUC_lexicon_tags_33 },
    { 0x99e0f223, SUC_lexicon_tags_68 },
    { 0x1ec0f228, SUC_lexicon_tags_0 },
    { 0xef80f229, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6200f22b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c0f22e, SUC_lexicon_tags_4 },
    { 0xb900f22f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e0f232, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6900f234, SUC_lexicon_tags_48 },
    { 0xe440f234, SUC_lexicon_tags_17 },
    { 0xfaa0f236, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4ee0f238, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1d40f23a, SUC_lexicon_tags_1 },
    { 0xb420f23a, SUC_lexicon_tags_64 },
    { 0x2cc0f23b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2320f23e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf80f241, SUC_lexicon_tags_10 },
    { 0xdda0f241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9c0f244, SUC_lexicon_tags_107 },
    { 0xa360f244, SUC_lexicon_tags_45 },
    { 0x2820f244, SUC_lexicon_tags_6 },
    { 0x69a0f245, SUC_lexicon_tags_1 },
    { 0x75c0f246, SUC_lexicon_tags_2 },
    { 0xf660f249, SUC_lexicon_tags_7 },
    { 0x9740f24a, SUC_lexicon_tags_12 },
    { 0x7ec0f249, SUC_lexicon_tags_30 },
    { 0xce60f24a, SUC_lexicon_tags_33 },
    { 0xc000f24a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdd20f24f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf00f253, SUC_lexicon_tags_15 },
    { 0x8620f254, SUC_lexicon_tags_19 },
    { 0x37a0f254, SUC_lexicon_tags_11 },
    { 0x83e0f256, SUC_lexicon_tags_47 },
    { 0xa860f257, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4760f25c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe180f25e, SUC_lexicon_tags_15 },
    { 0x960f25f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2a0f261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc0f265, SUC_lexicon_tags_27 },
    { 0x71e0f266, SUC_lexicon_tags_21 },
    { 0x1220f267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d80f26a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a40f26c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88e0f26e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a0f271, SUC_lexicon_tags_13 },
    { 0xc260f272, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee0f274, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1740f278, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7b60f27a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe260f27f, SUC_lexicon_tags_55 },
    { 0x72c0f280, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x3860f282, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7260f284, SUC_lexicon_tags_15 },
    { 0xbfa0f284, SUC_lexicon_tags_31 },
    { 0x4400f285, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd880f28c, SUC_lexicon_tags_1 },
    { 0x83e0f28c, SUC_lexicon_tags_7 },
    { 0xadc0f28e, SUC_lexicon_tags_35 },
    { 0xd2a0f28f, SUC_lexicon_tags_17 },
    { 0x6940f28e, SUC_lexicon_tags_77 },
    { 0x7320f28c, SUC_lexicon_tags_11 },
    { 0xf540f292, SUC_lexicon_tags_29 },
    { 0xe600f292, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa140f295, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f00f29b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb100f29d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a20f2a1, SUC_lexicon_tags_17 },
    { 0x6e20f2a2, SUC_lexicon_tags_19 },
    { 0x4d20f2a2, SUC_lexicon_tags_1 },
    { 0xcd40f2a2, SUC_lexicon_tags_27 },
    { 0x7f80f2a5, SUC_lexicon_tags_38 },
    { 0x6260f2a5, SUC_lexicon_tags_2 },
    { 0x59e0f2a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f60f2aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf5a0f2ac, SUC_lexicon_tags_6 },
    { 0xf7a0f2ac, SUC_lexicon_tags_16 },
    { 0x1e0f2ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e0f2b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a0f2b8, SUC_lexicon_tags_40 },
    { 0xf0a0f2b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9060f2bc, SUC_lexicon_tags_60 },
    { 0xf340f2bc, SUC_lexicon_tags_11 },
    { 0x6c00f2bc, SUC_lexicon_tags_7 },
    { 0xd1e0f2bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8e0f2c1, SUC_lexicon_tags_25 },
    { 0x1bc0f2c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f2c6, SUC_lexicon_tags_60 },
    { 0x4ca0f2c6, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xdae0f2c9, SUC_lexicon_tags_3 },
    { 0xa960f2ca, SUC_lexicon_tags_8 },
    { 0xd3e0f2c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb940f2ce, SUC_lexicon_tags_26 },
    { 0x9f40f2ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec0f2d2, SUC_lexicon_tags_27 },
    { 0x9e0f2d2, SUC_lexicon_tags_47 },
    { 0xece0f2d2, SUC_lexicon_tags_33 },
    { 0x4ea0f2d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ac0f2d7, SUC_lexicon_tags_2 },
    { 0x9740f2d8, SUC_lexicon_tags_33 },
    { 0x4860f2d9, SUC_lexicon_tags_23 },
    { 0x3cc0f2d9, SUC_lexicon_tags_56 },
    { 0xe40f2d8, SUC_lexicon_tags_11 },
    { 0x140f2dc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e0f2e4, SUC_lexicon_tags_29 },
    { 0xe180f2e5, SUC_lexicon_tags_26 },
    { 0xd5a0f2e5, SUC_lexicon_tags_15 },
    { 0x1c80f2e7, SUC_lexicon_tags_17 },
    { 0x12a0f2e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc20f2ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4000f2ed, SUC_lexicon_tags_1 },
    { 0xde40f2ee, SUC_lexicon_tags_1 },
    { 0xabc0f2ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3fe0f2f1, SUC_lexicon_tags_81 },
    { 0x7be0f2f2, SUC_lexicon_tags_8 },
    { 0xf1a0f2f3, SUC_lexicon_tags_47 },
    { 0x7b00f2f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c0f2f7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2780f2fa, SUC_lexicon_tags_2 },
    { 0xc6c0f2fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53e0f2fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa220f304, SUC_lexicon_tags_3 },
    { 0xff80f304, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe260f307, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde80f309, SUC_lexicon_tags_17 },
    { 0x22c0f309, SUC_lexicon_tags_34 },
    { 0x7060f30a, SUC_lexicon_tags_65 },
    { 0x2ba0f30c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8580f30e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca0f31b, SUC_lexicon_tags_9 },
    { 0xbcc0f31c, SUC_lexicon_tags_46 },
    { 0x82a0f31d, SUC_lexicon_tags_0 },
    { 0x5280f31e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5320f322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7280f327, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be0f32a, SUC_lexicon_tags_33 },
    { 0xff00f32a, SUC_lexicon_tags_38 },
    { 0xb6a0f32c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9020f32e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7dc0f330, SUC_lexicon_tags_7 },
    { 0x1560f331, SUC_lexicon_tags_23 },
    { 0xc300f331, SUC_lexicon_tags_37 },
    { 0x87a0f331, SUC_lexicon_tags_33 },
    { 0xc9e0f332, SUC_lexicon_tags_0 },
    { 0x8800f330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3560f338, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d00f33c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c00f340, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b20f342, SUC_lexicon_tags_9 },
    { 0x2020f343, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed40f347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe040f34d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6140f352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf20f354, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4180f35c, SUC_lexicon_tags_47 },
    { 0xf140f35d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb520f360, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2120f365, SUC_lexicon_tags_68 },
    { 0xc080f365, SUC_lexicon_tags_15 },
    { 0x7b40f366, SUC_lexicon_tags_44 },
    { 0xf7a0f368, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x860f36a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e0f36d, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb860f372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f379, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a00f37f, SUC_lexicon_tags_63 },
    { 0xfce0f37f, SUC_lexicon_tags_49 },
    { 0x6f00f381, SUC_lexicon_tags_31 },
    { 0x5d20f382, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cc0f384, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c0f388, SUC_lexicon_tags_2 },
    { 0xa700f389, SUC_lexicon_tags_68 },
    { 0x4540f38a, SUC_lexicon_tags_8 },
    { 0x14c0f38a, SUC_lexicon_tags_49 },
    { 0x1ca0f389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc0f390, SUC_lexicon_tags_11 },
    { 0x3600f391, SUC_lexicon_tags_2 },
    { 0x5000f392, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe740f399, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6640f39d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd620f39f, SUC_lexicon_tags_33 },
    { 0xc4a0f3a0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9320f3a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb120f3ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5020f3ae, SUC_lexicon_tags_12 },
    { 0xce60f3ae, SUC_lexicon_tags_116 },
    { 0xaf60f3b0, SUC_lexicon_tags_33 },
    { 0x16a0f3af, SUC_lexicon_tags_9 },
    { 0xf340f3b1, SUC_lexicon_tags_3 },
    { 0x6c60f3b3, SUC_lexicon_tags_17 },
    { 0xb120f3b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5160f3b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a0f3bd, SUC_lexicon_tags_11 },
    { 0xefc0f3bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800f3c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2980f3c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6980f3cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c0f3d4, SUC_lexicon_tags_25 },
    { 0x9140f3d4, SUC_lexicon_tags_7 },
    { 0xff40f3d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf00f3d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c0f3dc, SUC_lexicon_tags_27 },
    { 0x8140f3dc, SUC_lexicon_tags_7 },
    { 0x5340f3dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x41a0f3e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fa0f3e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f3e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe0f3eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae00f3ef, SUC_lexicon_tags_2 },
    { 0x8ce0f3ef, SUC_lexicon_tags_57 },
    { 0x7e40f3f1, SUC_lexicon_tags_3 },
    { 0x97e0f3f2, SUC_lexicon_tags_70 },
    { 0xb520f3f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e20f3f8, SUC_lexicon_tags_1 },
    { 0x97e0f3f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd540f3fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a20f405, SUC_lexicon_tags_48 },
    { 0xf2e0f406, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3660f408, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd3e0f40a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a0f40f, SUC_lexicon_tags_49 },
    { 0x24a0f40f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2080f413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc520f415, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e0f41e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e0f424, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e0f428, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd400f42a, SUC_lexicon_tags_15 },
    { 0xc600f42a, SUC_lexicon_tags_17 },
    { 0x5420f42c, SUC_lexicon_tags_27 },
    { 0xefc0f42d, SUC_lexicon_tags_7 },
    { 0xba80f42e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d60f432, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc460f434, SUC_lexicon_tags_2 },
    { 0x6e60f435, SUC_lexicon_tags_17 },
    { 0xe300f435, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc100f438, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f40f441, SUC_lexicon_tags_31 },
    { 0xd940f442, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5620f444, SUC_lexicon_tags_47 },
    { 0x5600f445, SUC_lexicon_tags_8 },
    { 0x8880f446, SUC_lexicon_tags_14 },
    { 0xc060f447, SUC_lexicon_tags_11 },
    { 0xe20f448, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e40f44c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6560f450, SUC_lexicon_tags_14 },
    { 0xcc80f450, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6200f453, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f20f456, SUC_lexicon_tags_11 },
    { 0x3780f457, SUC_lexicon_tags_63 },
    { 0x7de0f456, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd620f45b, SUC_lexicon_tags_15 },
    { 0xa340f45b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9080f45e, SUC_lexicon_tags_2 },
    { 0xc780f45f, SUC_lexicon_tags_49 },
    { 0x70e0f460, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba80f463, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf120f46c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48e0f46e, SUC_lexicon_tags_1 },
    { 0xf6c0f46f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x77c0f471, SUC_lexicon_tags_25 },
    { 0xb00f472, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa20f475, SUC_lexicon_tags_15 },
    { 0x300f475, SUC_lexicon_tags_55 },
    { 0x5bc0f477, SUC_lexicon_tags_2 },
    { 0xa8a0f478, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ee0f47a, SUC_lexicon_tags_9 },
    { 0x5e80f47b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea0f480, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a0f487, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xef20f48a, SUC_lexicon_tags_46 },
    { 0x4780f48a, SUC_lexicon_tags_34 },
    { 0x7340f48a, SUC_lexicon_tags_2 },
    { 0xdc60f48c, SUC_lexicon_tags_118 },
    { 0x72a0f48e, SUC_lexicon_tags_4 },
    { 0xad80f48f, SUC_lexicon_tags_37 },
    { 0x23a0f48d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a0f498, SUC_lexicon_tags_32 },
    { 0x8900f499, SUC_lexicon_tags_23 },
    { 0xa1e0f498, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa0f49f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a0f4a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e40f4a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b00f4ae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x78e0f4b0, SUC_lexicon_tags_2 },
    { 0xebc0f4b1, SUC_lexicon_tags_27 },
    { 0x8d60f4b2, SUC_lexicon_tags_14 },
    { 0x30c0f4b1, SUC_lexicon_tags_1 },
    { 0x8320f4b3, SUC_lexicon_tags_19 },
    { 0x16c0f4b5, SUC_lexicon_tags_53 },
    { 0x8180f4b5, SUC_lexicon_tags_16 },
    { 0xefc0f4b2, SUC_lexicon_tags_30 },
    { 0x3ee0f4b8, SUC_lexicon_tags_10 },
    { 0x4900f4b9, SUC_lexicon_tags_397 },
    { 0x5bc0f4ba, SUC_lexicon_tags_0 },
    { 0xaaa0f4b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7660f4c0, SUC_lexicon_tags_8 },
    { 0xb5a0f4c0, SUC_lexicon_tags_47 },
    { 0xfea0f4c0, SUC_lexicon_tags_0 },
    { 0x5620f4c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1d20f4c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9e60f4c7, SUC_lexicon_tags_45 },
    { 0x1520f4c8, SUC_lexicon_tags_1 },
    { 0x5ec0f4c9, SUC_lexicon_tags_39 },
    { 0xfce0f4c9, SUC_lexicon_tags_12 },
    { 0x2ac0f4cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed20f4ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa360f4d5, SUC_lexicon_tags_10 },
    { 0x8680f4d6, SUC_lexicon_tags_29 },
    { 0x7bc0f4d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e0f4df, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e0f4e2, SUC_lexicon_tags_15 },
    { 0x480f4e3, SUC_lexicon_tags_9 },
    { 0x5d40f4e4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7660f4e6, SUC_lexicon_tags_45 },
    { 0xde60f4e6, SUC_lexicon_tags_15 },
    { 0x49c0f4e7, SUC_lexicon_tags_14 },
    { 0xec40f4e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7020f4ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2900f4f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd60f4f3, SUC_lexicon_tags_14 },
    { 0x4660f4f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d00f4fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e0f4fe, SUC_lexicon_tags_15 },
    { 0x6160f4ff, SUC_lexicon_tags_11 },
    { 0xde20f4ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5060f502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9200f505, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e0f50d, SUC_lexicon_tags_43 },
    { 0xae20f50d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8660f511, SUC_lexicon_tags_0 },
    { 0x73e0f512, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e0f51a, SUC_lexicon_tags_2 },
    { 0xd520f51a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3500f51d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8ea0f51f, SUC_lexicon_tags_11 },
    { 0xb020f51f, SUC_lexicon_tags_61 },
    { 0x8d80f51f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c0f524, SUC_lexicon_tags_17 },
    { 0x36a0f525, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e00f52c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1020f52e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1220f533, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3060f536, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf00f53d, SUC_lexicon_tags_25 },
    { 0x40c0f53d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce20f542, SUC_lexicon_tags_7 },
    { 0x7840f543, SUC_lexicon_tags_25 },
    { 0x8500f544, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c0f548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a20f54b, SUC_lexicon_tags_1 },
    { 0x3e40f54b, SUC_lexicon_tags_27 },
    { 0xc6e0f54c, SUC_lexicon_tags_11 },
    { 0x7c0f54c, SUC_lexicon_tags_11 },
    { 0xdea0f54d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca0f555, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d40f558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc0f55b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5060f560, SUC_lexicon_tags_49 },
    { 0x5c60f560, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bc0f563, SUC_lexicon_tags_39 },
    { 0xf680f564, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17c0f566, SUC_lexicon_tags_4 },
    { 0x5740f567, SUC_lexicon_tags_191 },
    { 0xec80f566, SUC_lexicon_tags_21 },
    { 0x1980f569, SUC_lexicon_tags_11 },
    { 0xa00f566, SUC_lexicon_tags_0 },
    { 0xd140f56b, SUC_lexicon_tags_65 },
    { 0x9600f56c, SUC_lexicon_tags_0 },
    { 0x5ce0f569, SUC_lexicon_tags_21 },
    { 0xfd80f56c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc0f575, SUC_lexicon_tags_37 },
    { 0x44a0f575, SUC_lexicon_tags_11 },
    { 0x2a40f576, SUC_lexicon_tags_43 },
    { 0xe400f578, SUC_lexicon_tags_43 },
    { 0x7b40f579, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f60f57d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae0f581, SUC_lexicon_tags_30 },
    { 0x6160f582, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc0f585, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2100f588, SUC_lexicon_tags_10 },
    { 0x8b20f589, SUC_lexicon_tags_207 },
    { 0x6ce0f58a, SUC_lexicon_tags_7 },
    { 0xd9c0f589, SUC_lexicon_tags_15 },
    { 0x41c0f58c, SUC_lexicon_tags_63 },
    { 0x5920f58c, SUC_lexicon_tags_28 },
    { 0x5960f58e, SUC_lexicon_tags_29 },
    { 0xadc0f588, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x19a0f591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9560f595, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7720f597, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4940f59b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e40f59d, SUC_lexicon_tags_33 },
    { 0x4da0f59e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb860f5a0, SUC_lexicon_tags_1 },
    { 0xa3c0f5a1, SUC_lexicon_tags_32 },
    { 0x3b00f5a2, SUC_lexicon_tags_25 },
    { 0x6100f5a3, SUC_lexicon_tags_21 },
    { 0x65e0f5a4, SUC_lexicon_tags_16 },
    { 0xbe80f5a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9260f5a7, SUC_lexicon_tags_15 },
    { 0xb980f5a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc0f5ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x800f5b0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb40f5b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa260f5b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e0f5bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff40f5c0, SUC_lexicon_tags_47 },
    { 0x15c0f5c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9400f5c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec0f5cc, SUC_lexicon_tags_2 },
    { 0xf400f5cc, SUC_lexicon_tags_36 },
    { 0xe740f5cc, SUC_lexicon_tags_30 },
    { 0x7ae0f5cc, SUC_lexicon_tags_15 },
    { 0x7d20f5d0, SUC_lexicon_tags_1 },
    { 0xff00f5cd, SUC_lexicon_tags_43 },
    { 0x5400f5d2, SUC_lexicon_tags_30 },
    { 0x8c80f5d3, SUC_lexicon_tags_27 },
    { 0xb240f5ce, SUC_lexicon_tags_7 },
    { 0xbbe0f5d5, SUC_lexicon_tags_7 },
    { 0x8960f5cd, SUC_lexicon_tags_39 },
    { 0x9a00f5d1, SUC_lexicon_tags_33 },
    { 0x86e0f5d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd220f5e1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4480f5e4, SUC_lexicon_tags_66 },
    { 0xb3c0f5e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97e0f5e7, SUC_lexicon_tags_21 },
    { 0xbdc0f5e8, SUC_lexicon_tags_19 },
    { 0xf9a0f5e9, SUC_lexicon_tags_29 },
    { 0x68a0f5e7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x79c0f5ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e0f5ef, SUC_lexicon_tags_12 },
    { 0xca80f5f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5980f5f9, SUC_lexicon_tags_25 },
    { 0xfa60f5fa, SUC_lexicon_tags_12 },
    { 0xd0e0f5fb, SUC_lexicon_tags_37 },
    { 0x5680f5fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba0f600, SUC_lexicon_tags_29 },
    { 0x4ac0f600, SUC_lexicon_tags_19 },
    { 0x64e0f602, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a0f607, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfc0f609, SUC_lexicon_tags_43 },
    { 0x6ee0f609, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6a0f60c, SUC_lexicon_tags_15 },
    { 0xdf40f60d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca0f610, SUC_lexicon_tags_16 },
    { 0x40f611, SUC_lexicon_tags_25 },
    { 0xd320f612, SUC_lexicon_tags_26 },
    { 0x5ce0f613, SUC_lexicon_tags_17 },
    { 0xf920f613, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x60f616, SUC_lexicon_tags_15 },
    { 0x8140f617, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf440f619, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d60f61c, SUC_lexicon_tags_38 },
    { 0xdfe0f61d, SUC_lexicon_tags_46 },
    { 0x3c40f61e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8920f621, SUC_lexicon_tags_10 },
    { 0xa2c0f622, SUC_lexicon_tags_37 },
    { 0x7000f621, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c0f627, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c0f62e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1360f630, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf660f633, SUC_lexicon_tags_11 },
    { 0x60c0f634, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c40f63d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd040f643, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa420f649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf560f64b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x3840f64d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ae0f64f, SUC_lexicon_tags_17 },
    { 0xf680f64f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd60f655, SUC_lexicon_tags_26 },
    { 0x6e20f655, SUC_lexicon_tags_4 },
    { 0x7fe0f655, SUC_lexicon_tags_17 },
    { 0x35e0f658, SUC_lexicon_tags_63 },
    { 0xbc60f658, SUC_lexicon_tags_25 },
    { 0xcca0f65a, SUC_lexicon_tags_49 },
    { 0x9b80f65b, SUC_lexicon_tags_7 },
    { 0x1380f65a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e0f662, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc040f66a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3280f66e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca0f672, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff20f674, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc540f678, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xea40f67a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee0f67e, SUC_lexicon_tags_63 },
    { 0xd840f67f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a0f683, SUC_lexicon_tags_39 },
    { 0xc9e0f684, SUC_lexicon_tags_14 },
    { 0x3e60f685, SUC_lexicon_tags_7 },
    { 0xe240f686, SUC_lexicon_tags_25 },
    { 0x9400f686, SUC_lexicon_tags_49 },
    { 0x4800f687, SUC_lexicon_tags_45 },
    { 0xd780f689, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb600f690, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3000f693, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc800f696, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa600f699, SUC_lexicon_tags_141 },
    { 0x4040f699, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a0f6a0, SUC_lexicon_tags_7 },
    { 0x4b60f6a0, SUC_lexicon_tags_14 },
    { 0xc1e0f6a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa860f6a4, SUC_lexicon_tags_14 },
    { 0xf3a0f6a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf400f6ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e40f6b1, SUC_lexicon_tags_12 },
    { 0xda20f6b2, SUC_lexicon_tags_1 },
    { 0x7d00f6b3, SUC_lexicon_tags_11 },
    { 0x2dc0f6b1, SUC_lexicon_tags_351 },
    { 0xd520f6b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c0f6b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1e0f6ba, SUC_lexicon_tags_23 },
    { 0x7e40f6bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99c0f6bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe0f6c5, SUC_lexicon_tags_17 },
    { 0x27e0f6c5, SUC_lexicon_tags_25 },
    { 0xd1a0f6c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c80f6cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e0f6d0, SUC_lexicon_tags_10 },
    { 0x92e0f6d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca0f6da, SUC_lexicon_tags_33 },
    { 0xf380f6db, SUC_lexicon_tags_6 },
    { 0xf0a0f6dc, SUC_lexicon_tags_31 },
    { 0xdee0f6dd, SUC_lexicon_tags_12 },
    { 0x2260f6de, SUC_lexicon_tags_27 },
    { 0xede0f6df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1140f6e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40f6eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fc0f6ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe840f6ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5960f6f2, SUC_lexicon_tags_10 },
    { 0x580f6f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x12a0f6f5, SUC_lexicon_tags_37 },
    { 0xfac0f6f6, SUC_lexicon_tags_27 },
    { 0xed00f6f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec20f6fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6480f6fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb840f700, SUC_lexicon_tags_7 },
    { 0x1de0f701, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2040f708, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c40f70b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb320f70d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e40f710, SUC_lexicon_tags_6 },
    { 0x4520f711, SUC_lexicon_tags_31 },
    { 0xa8e0f710, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e0f715, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa080f717, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae0f71b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd380f71e, SUC_lexicon_tags_11 },
    { 0xb200f71f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a0f723, SUC_lexicon_tags_7 },
    { 0xb500f724, SUC_lexicon_tags_48 },
    { 0x220f725, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae00f728, SUC_lexicon_tags_28 },
    { 0x5e60f729, SUC_lexicon_tags_45 },
    { 0x9880f729, SUC_lexicon_tags_33 },
    { 0x2200f72b, SUC_lexicon_tags_2 },
    { 0xbb60f72a, SUC_lexicon_tags_37 },
    { 0xa7e0f729, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60f730, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c40f734, SUC_lexicon_tags_11 },
    { 0x4c80f734, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ac0f737, SUC_lexicon_tags_15 },
    { 0x72c0f738, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a60f73e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6720f741, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb2a0f743, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a0f746, SUC_lexicon_tags_49 },
    { 0x8860f747, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e20f74d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa980f74f, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c40f754, SUC_lexicon_tags_17 },
    { 0xea40f755, SUC_lexicon_tags_8 },
    { 0xd6e0f756, SUC_lexicon_tags_27 },
    { 0x6e40f754, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ce0f759, SUC_lexicon_tags_1 },
    { 0x5aa0f75a, SUC_lexicon_tags_11 },
    { 0x6a80f75b, SUC_lexicon_tags_61 },
    { 0xc0a0f75a, SUC_lexicon_tags_9 },
    { 0xfae0f75d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6100f763, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48c0f765, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8f60f767, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e0f76a, SUC_lexicon_tags_25 },
    { 0xc440f76b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41c0f76d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd60f773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd40f779, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de0f77c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb300f780, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1560f782, SUC_lexicon_tags_7 },
    { 0x6ac0f783, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa580f786, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f00f78f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc240f791, SUC_lexicon_tags_29 },
    { 0x4ce0f792, SUC_lexicon_tags_16 },
    { 0x8bc0f791, SUC_lexicon_tags_27 },
    { 0x3440f791, SUC_lexicon_tags_17 },
    { 0x8a80f795, SUC_lexicon_tags_15 },
    { 0xa860f795, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c0f799, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4460f7a4, SUC_lexicon_tags_15 },
    { 0x52e0f7a5, SUC_lexicon_tags_27 },
    { 0x58e0f7a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1e0f7a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac00f7b3, SUC_lexicon_tags_0 },
    { 0x680f7b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d40f7b9, SUC_lexicon_tags_25 },
    { 0x4c0f7ba, SUC_lexicon_tags_27 },
    { 0x5ae0f7bb, SUC_lexicon_tags_1 },
    { 0xdde0f7bc, SUC_lexicon_tags_27 },
    { 0x7ae0f7b9, SUC_lexicon_tags_25 },
    { 0x3a80f7be, SUC_lexicon_tags_30 },
    { 0xefc0f7be, SUC_lexicon_tags_47 },
    { 0x6140f7c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa180f7c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f7c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6560f7c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfaa0f7cb, SUC_lexicon_tags_11 },
    { 0xad80f7cb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9160f7ce, SUC_lexicon_tags_17 },
    { 0x6220f7cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83a0f7d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8780f7d3, SUC_lexicon_tags_4 },
    { 0x3da0f7d3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb960f7d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3840f7da, SUC_lexicon_tags_37 },
    { 0x62a0f7db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4120f7dd, SUC_lexicon_tags_37 },
    { 0xe440f7de, SUC_lexicon_tags_87 },
    { 0x13e0f7df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b60f7e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb00f7e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7980f7ea, SUC_lexicon_tags_19 },
    { 0x1ea0f7eb, SUC_lexicon_tags_12 },
    { 0x7e00f7eb, SUC_lexicon_tags_14 },
    { 0x7500f7ed, SUC_lexicon_tags_49 },
    { 0x1c00f7ee, SUC_lexicon_tags_11 },
    { 0xc0f7ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xee60f7f2, SUC_lexicon_tags_12 },
    { 0xda20f7f2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360f7f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6c60f7fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa380f7fe, SUC_lexicon_tags_15 },
    { 0x1e20f7fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1460f805, SUC_lexicon_tags_32 },
    { 0x53a0f806, SUC_lexicon_tags_15 },
    { 0xa880f806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb80f80b, SUC_lexicon_tags_11 },
    { 0x3940f80c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b20f811, SUC_lexicon_tags_27 },
    { 0xe7a0f812, SUC_lexicon_tags_6 },
    { 0x3d60f812, SUC_lexicon_tags_16 },
    { 0x9b60f813, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b80f817, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3140f819, SUC_lexicon_tags_0 },
    { 0x50c0f819, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d60f81d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e80f823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23e0f825, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a60f82c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c0f82f, SUC_lexicon_tags_69 },
    { 0x3e80f830, SUC_lexicon_tags_60 },
    { 0x4f80f831, SUC_lexicon_tags_27 },
    { 0xed80f82f, SUC_lexicon_tags_12 },
    { 0xe580f833, SUC_lexicon_tags_23 },
    { 0x2660f82f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1560f83b, SUC_lexicon_tags_24 },
    { 0x3f40f83b, SUC_lexicon_tags_25 },
    { 0x7c40f83c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7260f841, SUC_lexicon_tags_4 },
    { 0x32e0f842, SUC_lexicon_tags_37 },
    { 0xe940f843, SUC_lexicon_tags_32 },
    { 0xb9a0f844, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca0f84f, SUC_lexicon_tags_27 },
    { 0xb2c0f850, SUC_lexicon_tags_66 },
    { 0x6780f84f, SUC_lexicon_tags_30 },
    { 0xdf80f852, SUC_lexicon_tags_47 },
    { 0xec40f853, SUC_lexicon_tags_1 },
    { 0xcc60f852, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfd00f856, SUC_lexicon_tags_12 },
    { 0xcf00f856, SUC_lexicon_tags_7 },
    { 0xdfc0f857, SUC_lexicon_tags_46 },
    { 0xa60f858, SUC_lexicon_tags_6 },
    { 0x7440f85a, SUC_lexicon_tags_1 },
    { 0xd80f85b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c60f85d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf700f864, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12c0f866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xece0f868, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7940f86b, SUC_lexicon_tags_123 },
    { 0x2440f86c, SUC_lexicon_tags_2 },
    { 0x7460f86c, SUC_lexicon_tags_6 },
    { 0xa9c0f86c, SUC_lexicon_tags_21 },
    { 0x99a0f86d, SUC_lexicon_tags_25 },
    { 0x85e0f870, SUC_lexicon_tags_160 },
    { 0xd5e0f871, SUC_lexicon_tags_30 },
    { 0xb440f872, SUC_lexicon_tags_61 },
    { 0x1480f86f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb100f876, SUC_lexicon_tags_25 },
    { 0xeb20f877, SUC_lexicon_tags_11 },
    { 0x8e40f877, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd720f882, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe980f885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe0f888, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2560f88a, SUC_lexicon_tags_38 },
    { 0x1920f88b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f891, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda60f893, SUC_lexicon_tags_19 },
    { 0x3560f894, SUC_lexicon_tags_1 },
    { 0x6520f893, SUC_lexicon_tags_5 },
    { 0x1400f893, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d00f89b, SUC_lexicon_tags_17 },
    { 0x24e0f89c, SUC_lexicon_tags_37 },
    { 0xe7e0f89d, SUC_lexicon_tags_7 },
    { 0x4800f89d, SUC_lexicon_tags_65 },
    { 0x9980f89d, SUC_lexicon_tags_60 },
    { 0xa020f8a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f60f8a3, SUC_lexicon_tags_23 },
    { 0xa080f8a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0f8a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4200f8ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a0f8b0, SUC_lexicon_tags_66 },
    { 0x3bc0f8b0, SUC_lexicon_tags_26 },
    { 0x3f60f8b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe340f8b6, SUC_lexicon_tags_15 },
    { 0x8e20f8b7, SUC_lexicon_tags_9 },
    { 0xcac0f8b7, SUC_lexicon_tags_43 },
    { 0x5a40f8b9, SUC_lexicon_tags_32 },
    { 0x5320f8b9, SUC_lexicon_tags_11 },
    { 0xd4e0f8bb, SUC_lexicon_tags_33 },
    { 0x3100f8bc, SUC_lexicon_tags_38 },
    { 0xc9a0f8bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x120f8c0, SUC_lexicon_tags_1 },
    { 0x8520f8c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e0f8c5, SUC_lexicon_tags_65 },
    { 0x420f8c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e00f8ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe60f8cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8940f8d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca0f8d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3400f8d9, SUC_lexicon_tags_2 },
    { 0x72e0f8da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45e0f8dc, SUC_lexicon_tags_15 },
    { 0x1840f8dd, SUC_lexicon_tags_11 },
    { 0x4e0f8de, SUC_lexicon_tags_37 },
    { 0x6b00f8df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc0f8e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce0f8e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2280f8ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf980f8ec, SUC_lexicon_tags_2 },
    { 0xe420f8ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd60f8ef, SUC_lexicon_tags_1 },
    { 0xa240f8f0, SUC_lexicon_tags_85 },
    { 0xa3c0f8ef, SUC_lexicon_tags_1 },
    { 0xace0f8f1, SUC_lexicon_tags_47 },
    { 0xe4a0f8ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd620f8f6, SUC_lexicon_tags_12 },
    { 0xafc0f8f6, SUC_lexicon_tags_19 },
    { 0x8c40f8f6, SUC_lexicon_tags_2 },
    { 0xf560f8f6, SUC_lexicon_tags_37 },
    { 0xb600f8fa, SUC_lexicon_tags_38 },
    { 0xb560f8fb, SUC_lexicon_tags_7 },
    { 0x6620f8fc, SUC_lexicon_tags_37 },
    { 0x5680f8fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1520f901, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8640f905, SUC_lexicon_tags_8 },
    { 0xe8e0f906, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920f90a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce0f90e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc0f911, SUC_lexicon_tags_23 },
    { 0xf80f912, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c40f917, SUC_lexicon_tags_27 },
    { 0x3800f917, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2400f91b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e0f91e, SUC_lexicon_tags_39 },
    { 0x27a0f91f, SUC_lexicon_tags_45 },
    { 0x1520f920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba0f925, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8100f929, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6380f92c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc00f930, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320f937, SUC_lexicon_tags_17 },
    { 0x3b80f938, SUC_lexicon_tags_0 },
    { 0x53a0f938, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52c0f93b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1140f93d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef60f93f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe680f941, SUC_lexicon_tags_26 },
    { 0xf020f942, SUC_lexicon_tags_23 },
    { 0x4be0f942, SUC_lexicon_tags_11 },
    { 0x2fa0f941, SUC_lexicon_tags_46 },
    { 0xdf20f945, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb9a0f947, SUC_lexicon_tags_38 },
    { 0x6f40f947, SUC_lexicon_tags_6 },
    { 0x4040f949, SUC_lexicon_tags_37 },
    { 0x4580f949, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2ba0f94c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9540f94e, SUC_lexicon_tags_37 },
    { 0xbee0f94f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc80f964, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c0f967, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89a0f969, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89c0f96b, SUC_lexicon_tags_25 },
    { 0xf200f96b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa40f972, SUC_lexicon_tags_33 },
    { 0xdc80f973, SUC_lexicon_tags_47 },
    { 0xdd00f974, SUC_lexicon_tags_2 },
    { 0xc8e0f975, SUC_lexicon_tags_72 },
    { 0x9a60f973, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2520f978, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41a0f97a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c0f97d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1d60f97f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd160f983, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd340f985, SUC_lexicon_tags_61 },
    { 0xe980f986, SUC_lexicon_tags_25 },
    { 0xcc60f986, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a0f98a, SUC_lexicon_tags_7 },
    { 0xd5e0f98a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5080f995, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3820f998, SUC_lexicon_tags_4 },
    { 0x4860f999, SUC_lexicon_tags_12 },
    { 0x8de0f999, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x11a0f99c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3e60f99e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e0f9a1, SUC_lexicon_tags_17 },
    { 0x4100f9a1, SUC_lexicon_tags_42 },
    { 0xd940f9a1, SUC_lexicon_tags_63 },
    { 0xaf40f9a3, SUC_lexicon_tags_25 },
    { 0x9520f9a5, SUC_lexicon_tags_27 },
    { 0xa400f9a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a20f9b7, SUC_lexicon_tags_7 },
    { 0x6b60f9b7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf840f9ba, SUC_lexicon_tags_17 },
    { 0x3020f9bb, SUC_lexicon_tags_1 },
    { 0x4060f9bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda20f9be, SUC_lexicon_tags_25 },
    { 0x7a20f9be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a0f9c2, SUC_lexicon_tags_39 },
    { 0x4ac0f9c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd260f9c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00f9cc, SUC_lexicon_tags_92 },
    { 0x3280f9cd, SUC_lexicon_tags_38 },
    { 0x22e0f9ce, SUC_lexicon_tags_27 },
    { 0x3200f9cf, SUC_lexicon_tags_37 },
    { 0x5920f9cf, SUC_lexicon_tags_15 },
    { 0x28c0f9d1, SUC_lexicon_tags_2 },
    { 0x7ce0f9ce, SUC_lexicon_tags_25 },
    { 0x160f9d3, SUC_lexicon_tags_33 },
    { 0xd5c0f9d4, SUC_lexicon_tags_37 },
    { 0x2c0f9d5, SUC_lexicon_tags_25 },
    { 0x6400f9d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a0f9dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa100f9de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9920f9e0, SUC_lexicon_tags_26 },
    { 0x9d80f9e0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2fe0f9e3, SUC_lexicon_tags_1 },
    { 0x9ec0f9e4, SUC_lexicon_tags_10 },
    { 0xd860f9e4, SUC_lexicon_tags_15 },
    { 0x41a0f9e5, SUC_lexicon_tags_17 },
    { 0x11a0f9e4, SUC_lexicon_tags_37 },
    { 0x66c0f9e8, SUC_lexicon_tags_25 },
    { 0x8460f9e4, SUC_lexicon_tags_42 },
    { 0xa20f9ea, SUC_lexicon_tags_29 },
    { 0x38c0f9eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa20f9f0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc0f9f5, SUC_lexicon_tags_7 },
    { 0xefa0f9f5, SUC_lexicon_tags_23 },
    { 0x7540f9f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce0f9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9080f9ff, SUC_lexicon_tags_2 },
    { 0xb5e0fa00, SUC_lexicon_tags_85 },
    { 0x4e00fa01, SUC_lexicon_tags_2 },
    { 0x6840fa02, SUC_lexicon_tags_25 },
    { 0x2f60fa00, SUC_lexicon_tags_1 },
    { 0x2fa0fa02, SUC_lexicon_tags_26 },
    { 0x4da0fa05, SUC_lexicon_tags_7 },
    { 0xeb40fa05, SUC_lexicon_tags_29 },
    { 0x4980fa05, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be0fa0f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe560fa13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e0fa17, SUC_lexicon_tags_25 },
    { 0xfa40fa18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2900fa1e, SUC_lexicon_tags_37 },
    { 0x3920fa1f, SUC_lexicon_tags_37 },
    { 0x33e0fa1f, SUC_lexicon_tags_8 },
    { 0xab40fa1e, SUC_lexicon_tags_6 },
    { 0x3080fa22, SUC_lexicon_tags_33 },
    { 0xa780fa1e, SUC_lexicon_tags_15 },
    { 0x2d60fa24, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xb940fa26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4580fa29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f00fa2c, SUC_lexicon_tags_63 },
    { 0xaf20fa2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c60fa31, SUC_lexicon_tags_14 },
    { 0xb980fa32, SUC_lexicon_tags_30 },
    { 0x95e0fa31, SUC_lexicon_tags_17 },
    { 0x29a0fa33, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xfb20fa36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e60fa38, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da0fa43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e0fa46, SUC_lexicon_tags_2 },
    { 0xad60fa47, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6620fa4a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9800fa4c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xff80fa4e, SUC_lexicon_tags_17 },
    { 0x34c0fa4e, SUC_lexicon_tags_7 },
    { 0xe220fa50, SUC_lexicon_tags_6 },
    { 0x5ee0fa50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9da0fa53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb120fa55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x11c0fa57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e0fa5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd480fa5d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1780fa62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa0fa68, SUC_lexicon_tags_38 },
    { 0x120fa68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7280fa6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a20fa70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b20fa73, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c0fa7a, SUC_lexicon_tags_4 },
    { 0x7ea0fa7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf260fa82, SUC_lexicon_tags_60 },
    { 0xb40fa83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ce0fa85, SUC_lexicon_tags_37 },
    { 0xb720fa86, SUC_lexicon_tags_25 },
    { 0x7f60fa87, SUC_lexicon_tags_27 },
    { 0x64c0fa88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a60fa8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa960fa92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a20fa96, SUC_lexicon_tags_17 },
    { 0x6e00fa96, SUC_lexicon_tags_2 },
    { 0xb8c0fa97, SUC_lexicon_tags_14 },
    { 0x79c0fa99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9460fa9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb20fa9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e0faa1, SUC_lexicon_tags_26 },
    { 0xca40faa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4060faa6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa720faa8, SUC_lexicon_tags_47 },
    { 0x7320faa9, SUC_lexicon_tags_15 },
    { 0x7040faaa, SUC_lexicon_tags_12 },
    { 0xc460faa9, SUC_lexicon_tags_40 },
    { 0x560faac, SUC_lexicon_tags_37 },
    { 0x3480faad, SUC_lexicon_tags_17 },
    { 0x48e0faad, SUC_lexicon_tags_17 },
    { 0xdcc0faae, SUC_lexicon_tags_13 },
    { 0xec80fab0, SUC_lexicon_tags_60 },
    { 0x4d20faae, SUC_lexicon_tags_2 },
    { 0xc0e0faa8, SUC_lexicon_tags_1 },
    { 0xd60fab3, SUC_lexicon_tags_46 },
    { 0x2e20fab2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f60fab6, SUC_lexicon_tags_0 },
    { 0xfc80fab7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb500fab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe0fac0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a40fac6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b20facb, SUC_lexicon_tags_17 },
    { 0x3ba0facc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a0fad5, SUC_lexicon_tags_37 },
    { 0x8100fad5, SUC_lexicon_tags_12 },
    { 0x3ec0fad7, SUC_lexicon_tags_11 },
    { 0xd860fad8, SUC_lexicon_tags_1 },
    { 0x2ee0fad7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a80fadb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe920fadf, SUC_lexicon_tags_11 },
    { 0x1580fae0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfa0fae2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60fae9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x31c0faeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00faf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7860faf9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6700fafc, SUC_lexicon_tags_63 },
    { 0x6740fafd, SUC_lexicon_tags_42 },
    { 0xb080fafc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a0fb01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x940fb03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6500fb16, SUC_lexicon_tags_11 },
    { 0x99a0fb17, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a0fb1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb800fb1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6240fb23, SUC_lexicon_tags_37 },
    { 0x5b80fb24, SUC_lexicon_tags_14 },
    { 0xafe0fb24, SUC_lexicon_tags_29 },
    { 0x26e0fb26, SUC_lexicon_tags_17 },
    { 0xd40fb25, SUC_lexicon_tags_2 },
    { 0x32a0fb28, SUC_lexicon_tags_11 },
    { 0x9740fb28, SUC_lexicon_tags_11 },
    { 0xc0a0fb28, SUC_lexicon_tags_25 },
    { 0xf960fb24, SUC_lexicon_tags_7 },
    { 0xaec0fb23, SUC_lexicon_tags_11 },
    { 0xdca0fb2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba0fb31, SUC_lexicon_tags_64 },
    { 0xfe60fb31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3600fb34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd600fb38, SUC_lexicon_tags_21 },
    { 0x3520fb38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2140fb3e, SUC_lexicon_tags_25 },
    { 0x38a0fb3f, SUC_lexicon_tags_27 },
    { 0x9860fb3f, SUC_lexicon_tags_68 },
    { 0x8100fb41, SUC_lexicon_tags_8 },
    { 0xbaa0fb40, SUC_lexicon_tags_15 },
    { 0x7f80fb43, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe9e0fb45, SUC_lexicon_tags_17 },
    { 0x3020fb46, SUC_lexicon_tags_16 },
    { 0x6fa0fb47, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xae80fb49, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x87c0fb4b, SUC_lexicon_tags_14 },
    { 0x5740fb4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e0fb4f, SUC_lexicon_tags_11 },
    { 0x7f20fb50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c0fb54, SUC_lexicon_tags_27 },
    { 0xc5e0fb54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9920fb5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea0fb5f, SUC_lexicon_tags_29 },
    { 0xcf20fb60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e0fb6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bc0fb6d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc0fb71, SUC_lexicon_tags_48 },
    { 0x7620fb72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c0fb76, SUC_lexicon_tags_1233 },
    { 0, NULL },
    { 0, NULL },
    { 0x3600fb79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f80fb7d, SUC_lexicon_tags_2 },
    { 0x2260fb7d, SUC_lexicon_tags_8 },
    { 0x24e0fb7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa0fb82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1700fb85, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e0fb88, SUC_lexicon_tags_31 },
    { 0x3ec0fb89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8280fb8c, SUC_lexicon_tags_11 },
    { 0xe360fb8d, SUC_lexicon_tags_60 },
    { 0xed20fb8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f20fb92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd320fb96, SUC_lexicon_tags_15 },
    { 0x9940fb96, SUC_lexicon_tags_141 },
    { 0xe980fb98, SUC_lexicon_tags_11 },
    { 0xbc60fb98, SUC_lexicon_tags_11 },
    { 0xbce0fb9a, SUC_lexicon_tags_11 },
    { 0xdfc0fb9b, SUC_lexicon_tags_37 },
    { 0x5e80fb98, SUC_lexicon_tags_38 },
    { 0xbac0fb9c, SUC_lexicon_tags_7 },
    { 0x66c0fb98, SUC_lexicon_tags_22 },
    { 0x7c80fb98, SUC_lexicon_tags_12 },
    { 0x4080fba0, SUC_lexicon_tags_35 },
    { 0xf340fba0, SUC_lexicon_tags_25 },
    { 0xa440fb97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaac0fba4, SUC_lexicon_tags_6 },
    { 0xc800fba5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b80fba7, SUC_lexicon_tags_2 },
    { 0x2d60fba8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa00fbac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56a0fbae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee80fbb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e0fbb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1200fbb9, SUC_lexicon_tags_11 },
    { 0xd480fbb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe500fbbc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb840fbbe, SUC_lexicon_tags_39 },
    { 0x88e0fbbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c0fbc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdea0fbca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7d60fbcc, SUC_lexicon_tags_12 },
    { 0x8160fbcc, SUC_lexicon_tags_15 },
    { 0x3440fbce, SUC_lexicon_tags_37 },
    { 0xa620fbce, SUC_lexicon_tags_12 },
    { 0x1c00fbd0, SUC_lexicon_tags_29 },
    { 0x9c40fbd1, SUC_lexicon_tags_60 },
    { 0x52c0fbd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6a0fbd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a0fbd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc0fbdb, SUC_lexicon_tags_29 },
    { 0xf940fbdc, SUC_lexicon_tags_11 },
    { 0xbb40fbdc, SUC_lexicon_tags_12 },
    { 0xcfc0fbde, SUC_lexicon_tags_16 },
    { 0x82c0fbdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da0fbe3, SUC_lexicon_tags_7 },
    { 0x57c0fbe4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f00fbe6, SUC_lexicon_tags_1 },
    { 0x2c0fbe7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9aa0fbe9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd580fbec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a0fbf4, SUC_lexicon_tags_37 },
    { 0x5900fbf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf040fbfe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e0fc04, SUC_lexicon_tags_12 },
    { 0xc3c0fc05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5900fc07, SUC_lexicon_tags_47 },
    { 0xb420fc07, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3340fc0d, SUC_lexicon_tags_46 },
    { 0x8480fc0e, SUC_lexicon_tags_28 },
    { 0x6dc0fc0f, SUC_lexicon_tags_11 },
    { 0xfa00fc0f, SUC_lexicon_tags_30 },
    { 0xd300fc11, SUC_lexicon_tags_15 },
    { 0x5380fc11, SUC_lexicon_tags_5 },
    { 0x4340fc12, SUC_lexicon_tags_42 },
    { 0xccc0fc14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e0fc17, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf400fc1d, SUC_lexicon_tags_12 },
    { 0x1260fc1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6860fc25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe540fc29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b60fc2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7a0fc2e, SUC_lexicon_tags_57 },
    { 0x7080fc2f, SUC_lexicon_tags_14 },
    { 0x5840fc30, SUC_lexicon_tags_207 },
    { 0xdc20fc30, SUC_lexicon_tags_4 },
    { 0x4220fc32, SUC_lexicon_tags_55 },
    { 0x4880fc33, SUC_lexicon_tags_69 },
    { 0x4ec0fc32, SUC_lexicon_tags_19 },
    { 0x5a0fc35, SUC_lexicon_tags_1 },
    { 0xeee0fc36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e20fc39, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80fc3d, SUC_lexicon_tags_29 },
    { 0x25a0fc3e, SUC_lexicon_tags_10 },
    { 0x99e0fc3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9c0fc41, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8f40fc43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe260fc47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f60fc4a, SUC_lexicon_tags_39 },
    { 0xf420fc4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea60fc4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e40fc54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e0fc5b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9a00fc5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c0fc60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf20fc63, SUC_lexicon_tags_26 },
    { 0x9f80fc64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e0fc68, SUC_lexicon_tags_159 },
    { 0x8860fc68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c0fc6c, SUC_lexicon_tags_2 },
    { 0x6ba0fc6d, SUC_lexicon_tags_14 },
    { 0x5640fc6e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4ac0fc70, SUC_lexicon_tags_1 },
    { 0xa4c0fc71, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd20fc75, SUC_lexicon_tags_12 },
    { 0x5480fc76, SUC_lexicon_tags_2 },
    { 0x7600fc76, SUC_lexicon_tags_17 },
    { 0x2440fc78, SUC_lexicon_tags_15 },
    { 0xfa00fc78, SUC_lexicon_tags_2 },
    { 0x9760fc78, SUC_lexicon_tags_15 },
    { 0x8600fc7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5220fc84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb100fc89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf700fc8b, SUC_lexicon_tags_40 },
    { 0x2b40fc8c, SUC_lexicon_tags_2 },
    { 0xb60fc8d, SUC_lexicon_tags_45 },
    { 0x2b60fc8d, SUC_lexicon_tags_8 },
    { 0xd240fc8f, SUC_lexicon_tags_49 },
    { 0xe840fc8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd800fc92, SUC_lexicon_tags_31 },
    { 0x49c0fc93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5400fc98, SUC_lexicon_tags_17 },
    { 0xe320fc99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fc0fc9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d40fc9d, SUC_lexicon_tags_0 },
    { 0xf5a0fc9e, SUC_lexicon_tags_14 },
    { 0x480fc9f, SUC_lexicon_tags_37 },
    { 0x8c40fca0, SUC_lexicon_tags_47 },
    { 0x2180fca1, SUC_lexicon_tags_49 },
    { 0x41e0fc9f, SUC_lexicon_tags_31 },
    { 0xdb60fc9e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfe40fca5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e0fca9, SUC_lexicon_tags_0 },
    { 0xd240fcaa, SUC_lexicon_tags_7 },
    { 0x2a20fcab, SUC_lexicon_tags_14 },
    { 0x5e00fcab, SUC_lexicon_tags_1 },
    { 0x1440fcad, SUC_lexicon_tags_6 },
    { 0xa660fcae, SUC_lexicon_tags_1 },
    { 0xc9c0fcae, SUC_lexicon_tags_15 },
    { 0x2160fcab, SUC_lexicon_tags_12 },
    { 0xafe0fcb1, SUC_lexicon_tags_38 },
    { 0xe940fcaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0fcba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f80fcbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b80fcbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee0fcc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da0fcc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d60fccb, SUC_lexicon_tags_0 },
    { 0xd700fccc, SUC_lexicon_tags_7 },
    { 0xfda0fccd, SUC_lexicon_tags_16 },
    { 0x97e0fcce, SUC_lexicon_tags_14 },
    { 0x9560fccd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bc0fcd1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5880fcd3, SUC_lexicon_tags_1 },
    { 0x9ac0fcd4, SUC_lexicon_tags_39 },
    { 0x2360fcd3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd20fcd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8760fcdb, SUC_lexicon_tags_57 },
    { 0x6dc0fcdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8080fcde, SUC_lexicon_tags_12 },
    { 0xcb20fcdf, SUC_lexicon_tags_17 },
    { 0xc6a0fcde, SUC_lexicon_tags_11 },
    { 0x7bc0fce0, SUC_lexicon_tags_38 },
    { 0x81e0fce2, SUC_lexicon_tags_152 },
    { 0xcf60fce3, SUC_lexicon_tags_47 },
    { 0x280fce4, SUC_lexicon_tags_27 },
    { 0x55c0fce4, SUC_lexicon_tags_98 },
    { 0x6c0fce3, SUC_lexicon_tags_17 },
    { 0x9fe0fce7, SUC_lexicon_tags_57 },
    { 0x5b40fce8, SUC_lexicon_tags_12 },
    { 0x5ae0fce9, SUC_lexicon_tags_63 },
    { 0xd3a0fce8, SUC_lexicon_tags_17 },
    { 0xa1a0fce8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da0fcef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe60fcf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fc0fcf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x760fcf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a20fcfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a0fd01, SUC_lexicon_tags_27 },
    { 0x2ba0fd01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5520fd05, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2140fd07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac0fd0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6060fd12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a0fd15, SUC_lexicon_tags_29 },
    { 0xd520fd16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b00fd18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e0fd21, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba0fd24, SUC_lexicon_tags_25 },
    { 0x6260fd25, SUC_lexicon_tags_45 },
    { 0xfe80fd26, SUC_lexicon_tags_47 },
    { 0xfa20fd27, SUC_lexicon_tags_7 },
    { 0x820fd28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xed40fd2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcc0fd2c, SUC_lexicon_tags_146 },
    { 0xd120fd2d, SUC_lexicon_tags_75 },
    { 0x2c60fd2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1a60fd30, SUC_lexicon_tags_39 },
    { 0x9460fd31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a20fd33, SUC_lexicon_tags_12 },
    { 0x7180fd34, SUC_lexicon_tags_37 },
    { 0xb640fd33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45a0fd37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4760fd3a, SUC_lexicon_tags_25 },
    { 0xec00fd3b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5140fd3d, SUC_lexicon_tags_32 },
    { 0xa4e0fd3e, SUC_lexicon_tags_17 },
    { 0xe5e0fd3d, SUC_lexicon_tags_7 },
    { 0x9fa0fd40, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a0fd45, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3000fd4b, SUC_lexicon_tags_8 },
    { 0x6c60fd4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3120fd50, SUC_lexicon_tags_11 },
    { 0xfb60fd51, SUC_lexicon_tags_30 },
    { 0x8a40fd52, SUC_lexicon_tags_11 },
    { 0xf60fd52, SUC_lexicon_tags_45 },
    { 0x1f00fd53, SUC_lexicon_tags_1 },
    { 0xd00fd55, SUC_lexicon_tags_43 },
    { 0xbd80fd51, SUC_lexicon_tags_1 },
    { 0xa700fd57, SUC_lexicon_tags_65 },
    { 0x7460fd58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1560fd5b, SUC_lexicon_tags_40 },
    { 0x41a0fd5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x46c0fd5e, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x9b60fd60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf020fd62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8aa0fd64, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade0fd68, SUC_lexicon_tags_5 },
    { 0xd760fd69, SUC_lexicon_tags_9 },
    { 0xf720fd69, SUC_lexicon_tags_2 },
    { 0x5800fd6b, SUC_lexicon_tags_2 },
    { 0x6260fd6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1040fd70, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e00fd73, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6b20fd75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da0fd7b, SUC_lexicon_tags_2 },
    { 0x14c0fd7c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9f80fd7e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa760fd80, SUC_lexicon_tags_63 },
    { 0x1080fd81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf220fd83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c0fd8c, SUC_lexicon_tags_1 },
    { 0x4da0fd8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc0fd94, SUC_lexicon_tags_25 },
    { 0x2860fd94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xab80fd97, SUC_lexicon_tags_25 },
    { 0xe7e0fd97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56e0fd9a, SUC_lexicon_tags_11 },
    { 0xa660fd9b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5820fda3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe780fdaa, SUC_lexicon_tags_17 },
    { 0x14c0fdab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7c0fdad, SUC_lexicon_tags_12 },
    { 0x9080fdad, SUC_lexicon_tags_11 },
    { 0xe0e0fdaf, SUC_lexicon_tags_7 },
    { 0xae40fdb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae0fdb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7480fdb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a0fdba, SUC_lexicon_tags_1 },
    { 0xcc00fdba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc920fdbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf40fdbf, SUC_lexicon_tags_21 },
    { 0xe180fdbf, SUC_lexicon_tags_29 },
    { 0x9a0fdbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c0fdc6, SUC_lexicon_tags_0 },
    { 0x1800fdc7, SUC_lexicon_tags_26 },
    { 0x8740fdc8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xef20fdca, SUC_lexicon_tags_25 },
    { 0xbe20fdcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c0fdd0, SUC_lexicon_tags_8 },
    { 0x9de0fdd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc0fdd4, SUC_lexicon_tags_37 },
    { 0xaf40fdd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa920fdd7, SUC_lexicon_tags_68 },
    { 0x2c20fdd8, SUC_lexicon_tags_7 },
    { 0x37e0fdd9, SUC_lexicon_tags_15 },
    { 0xa8e0fdd7, SUC_lexicon_tags_29 },
    { 0xb000fdd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4240fddd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec20fde2, SUC_lexicon_tags_25 },
    { 0xcb80fde3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd620fde7, SUC_lexicon_tags_9 },
    { 0x6140fde7, SUC_lexicon_tags_31 },
    { 0x5d20fde8, SUC_lexicon_tags_17 },
    { 0xbe00fde7, SUC_lexicon_tags_7 },
    { 0xbdc0fde9, SUC_lexicon_tags_45 },
    { 0x43a0fde7, SUC_lexicon_tags_25 },
    { 0x5ea0fded, SUC_lexicon_tags_1 },
    { 0xc260fded, SUC_lexicon_tags_15 },
    { 0xa540fdeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c0fdf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8940fdf5, SUC_lexicon_tags_43 },
    { 0xc2e0fdf6, SUC_lexicon_tags_43 },
    { 0x87c0fdf7, SUC_lexicon_tags_27 },
    { 0x8c80fdf5, SUC_lexicon_tags_23 },
    { 0xfd80fdf7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85a0fdfb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf700fdff, SUC_lexicon_tags_11 },
    { 0x36a0fe00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf380fe03, SUC_lexicon_tags_21 },
    { 0x5900fe04, SUC_lexicon_tags_38 },
    { 0x25a0fe05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c0fe08, SUC_lexicon_tags_7 },
    { 0xaa40fe08, SUC_lexicon_tags_25 },
    { 0xfa00fe0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3120fe10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe0fe13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc540fe15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea0fe19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5480fe1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf00fe20, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc0fe25, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c0fe29, SUC_lexicon_tags_1 },
    { 0x1740fe2a, SUC_lexicon_tags_57 },
    { 0x4f20fe2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe0fe37, SUC_lexicon_tags_10 },
    { 0xa5c0fe38, SUC_lexicon_tags_45 },
    { 0x8e60fe37, SUC_lexicon_tags_15 },
    { 0x8cc0fe3a, SUC_lexicon_tags_11 },
    { 0xf300fe37, SUC_lexicon_tags_11 },
    { 0xc660fe38, SUC_lexicon_tags_2 },
    { 0x1a40fe3d, SUC_lexicon_tags_31 },
    { 0x7fe0fe38, SUC_lexicon_tags_26 },
    { 0xea60fe3f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9500fe41, SUC_lexicon_tags_37 },
    { 0x2b40fe42, SUC_lexicon_tags_25 },
    { 0xe820fe41, SUC_lexicon_tags_48 },
    { 0x32a0fe43, SUC_lexicon_tags_46 },
    { 0x8e00fe45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc40fe48, SUC_lexicon_tags_0 },
    { 0xf840fe49, SUC_lexicon_tags_26 },
    { 0x4240fe4a, SUC_lexicon_tags_11 },
    { 0x8c00fe4b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba0fe4e, SUC_lexicon_tags_43 },
    { 0xe660fe4f, SUC_lexicon_tags_25 },
    { 0x85e0fe4f, SUC_lexicon_tags_11 },
    { 0x69a0fe51, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2540fe53, SUC_lexicon_tags_49 },
    { 0xfa60fe53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6900fe59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9200fe5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4980fe60, SUC_lexicon_tags_1034 },
    { 0x140fe61, SUC_lexicon_tags_16 },
    { 0xc520fe61, SUC_lexicon_tags_11 },
    { 0xe100fe62, SUC_lexicon_tags_2 },
    { 0x4b80fe64, SUC_lexicon_tags_1 },
    { 0xdf40fe62, SUC_lexicon_tags_2 },
    { 0x47a0fe66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b00fe69, SUC_lexicon_tags_10 },
    { 0x7200fe69, SUC_lexicon_tags_10 },
    { 0x6ec0fe6b, SUC_lexicon_tags_7 },
    { 0x8500fe6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6020fe6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6280fe76, SUC_lexicon_tags_37 },
    { 0x1f00fe77, SUC_lexicon_tags_26 },
    { 0x44e0fe78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2720fe7a, SUC_lexicon_tags_12 },
    { 0xc1a0fe7b, SUC_lexicon_tags_23 },
    { 0xf960fe7c, SUC_lexicon_tags_141 },
    { 0x8f20fe7d, SUC_lexicon_tags_16 },
    { 0xf280fe7b, SUC_lexicon_tags_7 },
    { 0xe480fe7f, SUC_lexicon_tags_6 },
    { 0x220fe80, SUC_lexicon_tags_23 },
    { 0x7780fe80, SUC_lexicon_tags_4 },
    { 0x7380fe82, SUC_lexicon_tags_27 },
    { 0xa8c0fe82, SUC_lexicon_tags_11 },
    { 0xb900fe84, SUC_lexicon_tags_51 },
    { 0x2dc0fe85, SUC_lexicon_tags_38 },
    { 0x2120fe7f, SUC_lexicon_tags_12 },
    { 0x4c20fe7c, SUC_lexicon_tags_26 },
    { 0xe0fe88, SUC_lexicon_tags_17 },
    { 0xbb20fe84, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3280fe8d, SUC_lexicon_tags_1 },
    { 0x11a0fe8e, SUC_lexicon_tags_7 },
    { 0xd0e0fe8e, SUC_lexicon_tags_15 },
    { 0x3880fe90, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc0fe95, SUC_lexicon_tags_31 },
    { 0x6ca0fe96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35c0fe98, SUC_lexicon_tags_37 },
    { 0xf340fe98, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e0fe9d, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80fea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff00fea6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac0feab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1480feb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e0feb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8940feb7, SUC_lexicon_tags_33 },
    { 0xe320feb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc560feba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee0febc, SUC_lexicon_tags_29 },
    { 0x2580febd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5000febf, SUC_lexicon_tags_4 },
    { 0xe840febf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d80fecd, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x81e0fecf, SUC_lexicon_tags_82 },
    { 0xf060fed0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda0fed5, SUC_lexicon_tags_27 },
    { 0x1700fed6, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe80fedc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb960fee0, SUC_lexicon_tags_25 },
    { 0xa00fee1, SUC_lexicon_tags_275 },
    { 0x55e0fee2, SUC_lexicon_tags_23 },
    { 0x93a0fee1, SUC_lexicon_tags_38 },
    { 0x2660fee4, SUC_lexicon_tags_37 },
    { 0x9140fee5, SUC_lexicon_tags_16 },
    { 0xa820fee6, SUC_lexicon_tags_30 },
    { 0xe340fee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbc0fee9, SUC_lexicon_tags_9 },
    { 0x2760fee9, SUC_lexicon_tags_11 },
    { 0x1480feeb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc60feed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe560feef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x53c0fef1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6300fef4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2860fef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x640fef9, SUC_lexicon_tags_68 },
    { 0xa940fef9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe400fefd, SUC_lexicon_tags_359 },
    { 0x3560fefd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8760ff07, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe240ff0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf80ff0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc180ff11, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x50c0ff13, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xea0ff15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe300ff1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaba0ff1c, SUC_lexicon_tags_43 },
    { 0xc6c0ff1d, SUC_lexicon_tags_47 },
    { 0x3ae0ff1e, SUC_lexicon_tags_79 },
    { 0xd680ff1c, SUC_lexicon_tags_17 },
    { 0xe8c0ff20, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de0ff23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa580ff2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a00ff2c, SUC_lexicon_tags_66 },
    { 0x91e0ff2d, SUC_lexicon_tags_47 },
    { 0xab40ff2d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x76a0ff30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5a0ff32, SUC_lexicon_tags_11 },
    { 0x8c00ff33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b40ff36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8140ff38, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0ff3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c0ff40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfaa0ff42, SUC_lexicon_tags_25 },
    { 0x83e0ff43, SUC_lexicon_tags_1 },
    { 0xca60ff44, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf860ff46, SUC_lexicon_tags_28 },
    { 0x9fe0ff47, SUC_lexicon_tags_7 },
    { 0x16a0ff48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ff4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a0ff4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe380ff51, SUC_lexicon_tags_31 },
    { 0xdaa0ff51, SUC_lexicon_tags_14 },
    { 0x2f60ff53, SUC_lexicon_tags_25 },
    { 0xd5e0ff54, SUC_lexicon_tags_66 },
    { 0x6620ff53, SUC_lexicon_tags_37 },
    { 0x37a0ff51, SUC_lexicon_tags_15 },
    { 0x8820ff54, SUC_lexicon_tags_46 },
    { 0xc620ff56, SUC_lexicon_tags_7 },
    { 0x72e0ff56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a20ff5d, SUC_lexicon_tags_1 },
    { 0x41a0ff5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1640ff60, SUC_lexicon_tags_1 },
    { 0x3200ff60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa060ff63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f20ff65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a00ff6e, SUC_lexicon_tags_37 },
    { 0xe80ff6f, SUC_lexicon_tags_17 },
    { 0x2580ff6e, SUC_lexicon_tags_25 },
    { 0xe3c0ff6e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf00ff76, SUC_lexicon_tags_15 },
    { 0xf560ff76, SUC_lexicon_tags_37 },
    { 0x3260ff78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a40ff81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca0ff85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa20ff87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3280ff91, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b20ff9b, SUC_lexicon_tags_38 },
    { 0xc2c0ff9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4180ffa3, SUC_lexicon_tags_27 },
    { 0xc020ffa4, SUC_lexicon_tags_11 },
    { 0xc60ffa5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1780ffa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea0ffad, SUC_lexicon_tags_29 },
    { 0xb9c0ffae, SUC_lexicon_tags_127 },
    { 0x2de0ffaf, SUC_lexicon_tags_37 },
    { 0x7040ffb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ffb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1e0ffb6, SUC_lexicon_tags_18 },
    { 0x39c0ffb7, SUC_lexicon_tags_11 },
    { 0xf100ffb8, SUC_lexicon_tags_37 },
    { 0x8fa0ffb7, SUC_lexicon_tags_12 },
    { 0x7bc0ffb8, SUC_lexicon_tags_97 },
    { 0x37c0ffbb, SUC_lexicon_tags_15 },
    { 0xe280ffbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9100ffc1, SUC_lexicon_tags_0 },
    { 0x9220ffc1, SUC_lexicon_tags_7 },
    { 0xca00ffc1, SUC_lexicon_tags_30 },
    { 0xf3e0ffc4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c0ffc8, SUC_lexicon_tags_28 },
    { 0xb200ffc9, SUC_lexicon_tags_15 },
    { 0xab00ffca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb000ffce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9540ffd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4680ffd4, SUC_lexicon_tags_11 },
    { 0x34a0ffd5, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x4fe0ffd7, SUC_lexicon_tags_26 },
    { 0x4f00ffd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a0ffe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd780ffe7, SUC_lexicon_tags_37 },
    { 0x71a0ffe8, SUC_lexicon_tags_17 },
    { 0xe3e0ffe9, SUC_lexicon_tags_2 },
    { 0x2940ffe7, SUC_lexicon_tags_27 },
    { 0x6cc0ffeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea0ffed, SUC_lexicon_tags_10 },
    { 0x5c20ffee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e20fff1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6f20fff3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5440fff5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc0fff9, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x7560fffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2740fffe, SUC_lexicon_tags_63 },
    { 0x3220ffff, SUC_lexicon_tags_7 },
    { 0xf460ffff, SUC_lexicon_tags_25 },
    { 0x9c610001, SUC_lexicon_tags_30 },
    { 0xf610000, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e10005, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a1000b, SUC_lexicon_tags_49 },
    { 0xc501000c, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x7c21000e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa10011, SUC_lexicon_tags_37 },
    { 0x90610012, SUC_lexicon_tags_63 },
    { 0x46210012, SUC_lexicon_tags_27 },
    { 0x5fa10014, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a10023, SUC_lexicon_tags_1 },
    { 0xb1c10023, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c010027, SUC_lexicon_tags_7 },
    { 0x49410028, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe6a1002a, SUC_lexicon_tags_14 },
    { 0xcd01002b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7201002e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e410030, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d810032, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9610037, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c1003c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae1003f, SUC_lexicon_tags_46 },
    { 0xa7010040, SUC_lexicon_tags_25 },
    { 0x45010041, SUC_lexicon_tags_68 },
    { 0x14a10042, SUC_lexicon_tags_1 },
    { 0x1be10043, SUC_lexicon_tags_4 },
    { 0x2da10040, SUC_lexicon_tags_15 },
    { 0xaca10042, SUC_lexicon_tags_45 },
    { 0xf1210046, SUC_lexicon_tags_19 },
    { 0x67010047, SUC_lexicon_tags_12 },
    { 0x9da10048, SUC_lexicon_tags_6 },
    { 0xa1210049, SUC_lexicon_tags_38 },
    { 0x47e1004a, SUC_lexicon_tags_23 },
    { 0x9b21004b, SUC_lexicon_tags_15 },
    { 0x51a1004b, SUC_lexicon_tags_4 },
    { 0x88e1004c, SUC_lexicon_tags_37 },
    { 0x7ac10044, SUC_lexicon_tags_2 },
    { 0xd4c1004b, SUC_lexicon_tags_63 },
    { 0x5b410041, SUC_lexicon_tags_14 },
    { 0xb0810051, SUC_lexicon_tags_17 },
    { 0xc6410052, SUC_lexicon_tags_7 },
    { 0xd6e10051, SUC_lexicon_tags_12 },
    { 0x8b610054, SUC_lexicon_tags_45 },
    { 0x89a10051, SUC_lexicon_tags_25 },
    { 0xec410056, SUC_lexicon_tags_43 },
    { 0x84410057, SUC_lexicon_tags_0 },
    { 0xa5010058, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4941005a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de1005d, SUC_lexicon_tags_9 },
    { 0x7c1005e, SUC_lexicon_tags_15 },
    { 0x5321005f, SUC_lexicon_tags_40 },
    { 0xa6e1005e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3210067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e1007a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb61007c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe10084, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x72a10086, SUC_lexicon_tags_83 },
    { 0xb2810087, SUC_lexicon_tags_15 },
    { 0x5e610088, SUC_lexicon_tags_26 },
    { 0x2d810088, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xc701008b, SUC_lexicon_tags_17 },
    { 0x9161008b, SUC_lexicon_tags_65 },
    { 0x6921008d, SUC_lexicon_tags_43 },
    { 0x1161008e, SUC_lexicon_tags_47 },
    { 0x54e1008f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a610094, SUC_lexicon_tags_2 },
    { 0x1ce10095, SUC_lexicon_tags_84 },
    { 0xa5810095, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1810098, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9761009a, SUC_lexicon_tags_15 },
    { 0x9e41009b, SUC_lexicon_tags_6 },
    { 0x81c1009a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3401009f, SUC_lexicon_tags_37 },
    { 0x9e6100a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206100a4, SUC_lexicon_tags_15 },
    { 0x50e100a5, SUC_lexicon_tags_37 },
    { 0xf3c100a5, SUC_lexicon_tags_49 },
    { 0xe08100a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x876100aa, SUC_lexicon_tags_28 },
    { 0x226100aa, SUC_lexicon_tags_37 },
    { 0xee6100ac, SUC_lexicon_tags_11 },
    { 0x724100ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a100b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440100b7, SUC_lexicon_tags_38 },
    { 0x59e100b8, SUC_lexicon_tags_28 },
    { 0x4f0100b9, SUC_lexicon_tags_1 },
    { 0x562100ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c100bd, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xca8100bf, SUC_lexicon_tags_7 },
    { 0x468100bf, SUC_lexicon_tags_17 },
    { 0xd00100c1, SUC_lexicon_tags_47 },
    { 0x312100c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22100c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c100ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a6100cc, SUC_lexicon_tags_7 },
    { 0x926100cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16a100cf, SUC_lexicon_tags_55 },
    { 0xc8100cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b4100d2, SUC_lexicon_tags_4 },
    { 0x220100d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c100da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8100de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea8100e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506100e5, SUC_lexicon_tags_45 },
    { 0xbaa100e6, SUC_lexicon_tags_7 },
    { 0x802100e7, SUC_lexicon_tags_53 },
    { 0x99e100e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf18100ea, SUC_lexicon_tags_21 },
    { 0xcac100eb, SUC_lexicon_tags_63 },
    { 0xc38100ec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180100f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566100f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93410100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39810102, SUC_lexicon_tags_12 },
    { 0x2f010103, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2410105, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa10108, SUC_lexicon_tags_43 },
    { 0xa2410109, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c1010c, SUC_lexicon_tags_33 },
    { 0x98e1010c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeb61010f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a210114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c210117, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a1011b, SUC_lexicon_tags_11 },
    { 0x42e1011c, SUC_lexicon_tags_6 },
    { 0xb781011d, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x3f61011f, SUC_lexicon_tags_37 },
    { 0x7a1011f, SUC_lexicon_tags_37 },
    { 0x1c210121, SUC_lexicon_tags_1 },
    { 0x861011f, SUC_lexicon_tags_4 },
    { 0x6d010122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada1012a, SUC_lexicon_tags_11 },
    { 0x5e41012b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3f81012d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d610134, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c10137, SUC_lexicon_tags_1 },
    { 0x7ba10138, SUC_lexicon_tags_45 },
    { 0xa6410137, SUC_lexicon_tags_47 },
    { 0xdb210137, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a1013d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d410140, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6410145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa221014b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41010151, SUC_lexicon_tags_46 },
    { 0x84810152, SUC_lexicon_tags_10 },
    { 0xa5610152, SUC_lexicon_tags_7 },
    { 0x1aa10152, SUC_lexicon_tags_17 },
    { 0x70210154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1210158, SUC_lexicon_tags_7 },
    { 0xee410159, SUC_lexicon_tags_70 },
    { 0x4ee1015a, SUC_lexicon_tags_63 },
    { 0x8841015b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4f01015d, SUC_lexicon_tags_19 },
    { 0x2c21015d, SUC_lexicon_tags_68 },
    { 0x5f01015f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8621016d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94810172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9410175, SUC_lexicon_tags_25 },
    { 0x69810175, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9361017b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdf81017d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba10180, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c810186, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9181018c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6861018f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41010191, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93610194, SUC_lexicon_tags_13 },
    { 0x41a10195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca10197, SUC_lexicon_tags_63 },
    { 0x88010197, SUC_lexicon_tags_2 },
    { 0xf2610199, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea1019c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54101a0, SUC_lexicon_tags_15 },
    { 0x796101a1, SUC_lexicon_tags_16 },
    { 0xdb6101a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x612101a4, SUC_lexicon_tags_37 },
    { 0x556101a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec101a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x306101ac, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c101af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da101b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376101ba, SUC_lexicon_tags_26 },
    { 0x77e101ba, SUC_lexicon_tags_82 },
    { 0x488101bc, SUC_lexicon_tags_9 },
    { 0x6fe101ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79a101bf, SUC_lexicon_tags_37 },
    { 0x70e101bf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xea4101c2, SUC_lexicon_tags_1 },
    { 0xc0e101c3, SUC_lexicon_tags_25 },
    { 0x952101c4, SUC_lexicon_tags_15 },
    { 0x74a101c5, SUC_lexicon_tags_47 },
    { 0x3ce101c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24101c9, SUC_lexicon_tags_1 },
    { 0x402101c9, SUC_lexicon_tags_1 },
    { 0x636101cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x154101cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252101d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x126101d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57e101d9, SUC_lexicon_tags_25 },
    { 0xb78101da, SUC_lexicon_tags_68 },
    { 0x858101da, SUC_lexicon_tags_1463 },
    { 0xb8c101dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x580101de, SUC_lexicon_tags_25 },
    { 0x24e101de, SUC_lexicon_tags_17 },
    { 0x8ce101e0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x248101e2, SUC_lexicon_tags_62 },
    { 0x712101e3, SUC_lexicon_tags_31 },
    { 0x1da101e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2101e7, SUC_lexicon_tags_15 },
    { 0xaa101e7, SUC_lexicon_tags_7 },
    { 0x90101e9, SUC_lexicon_tags_8 },
    { 0x8b2101ea, SUC_lexicon_tags_1 },
    { 0xd2c101eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78101f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc101f9, SUC_lexicon_tags_37 },
    { 0x4a8101fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc101fe, SUC_lexicon_tags_2 },
    { 0xc10101ff, SUC_lexicon_tags_60 },
    { 0x9fc10200, SUC_lexicon_tags_11 },
    { 0xaa2101ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5610206, SUC_lexicon_tags_17 },
    { 0x3fe10207, SUC_lexicon_tags_31 },
    { 0x38010206, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef01020c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83a1020e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c10211, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x11410214, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac1021c, SUC_lexicon_tags_47 },
    { 0x6f21021c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66010222, SUC_lexicon_tags_1 },
    { 0x64010223, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e10226, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa1022a, SUC_lexicon_tags_1 },
    { 0x3761022a, SUC_lexicon_tags_38 },
    { 0xf281022c, SUC_lexicon_tags_49 },
    { 0xbce1022c, SUC_lexicon_tags_32 },
    { 0x1001022e, SUC_lexicon_tags_29 },
    { 0x8601022b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a10232, SUC_lexicon_tags_15 },
    { 0x99210233, SUC_lexicon_tags_72 },
    { 0xa1210233, SUC_lexicon_tags_28 },
    { 0x5b410233, SUC_lexicon_tags_12 },
    { 0xb8a10234, SUC_lexicon_tags_61 },
    { 0xfb810237, SUC_lexicon_tags_2 },
    { 0x13a10235, SUC_lexicon_tags_66 },
    { 0xf5c10236, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x22c1023b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d61023f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8410244, SUC_lexicon_tags_33 },
    { 0xf2610245, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e10248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfa1024a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a21024c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc581024e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c10252, SUC_lexicon_tags_15 },
    { 0x6e210252, SUC_lexicon_tags_46 },
    { 0x50c10254, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3210257, SUC_lexicon_tags_9 },
    { 0x4fe10257, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6481025a, SUC_lexicon_tags_25 },
    { 0xa81025b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e1025e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30810260, SUC_lexicon_tags_30 },
    { 0xa3c10261, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeda10263, SUC_lexicon_tags_61 },
    { 0x7fa10264, SUC_lexicon_tags_45 },
    { 0x5ce10264, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x40810267, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a1026d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1521026f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa9810271, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38c10273, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71610276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf481027a, SUC_lexicon_tags_37 },
    { 0x3ec1027b, SUC_lexicon_tags_37 },
    { 0x7a21027c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9981027e, SUC_lexicon_tags_15 },
    { 0x4be1027f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9610281, SUC_lexicon_tags_27 },
    { 0x60010282, SUC_lexicon_tags_30 },
    { 0xd6e10283, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e10287, SUC_lexicon_tags_17 },
    { 0x1e410288, SUC_lexicon_tags_11 },
    { 0x83210287, SUC_lexicon_tags_1 },
    { 0x84c10287, SUC_lexicon_tags_23 },
    { 0x1821028b, SUC_lexicon_tags_32 },
    { 0x38a1028b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc10293, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a10296, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23e10298, SUC_lexicon_tags_1 },
    { 0xbb410299, SUC_lexicon_tags_0 },
    { 0x84c10299, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6201029f, SUC_lexicon_tags_6 },
    { 0xb44102a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e102a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa32102a5, SUC_lexicon_tags_53 },
    { 0x46e102a5, SUC_lexicon_tags_25 },
    { 0x464102a7, SUC_lexicon_tags_12 },
    { 0x66e102a8, SUC_lexicon_tags_8 },
    { 0x864102a9, SUC_lexicon_tags_31 },
    { 0x6d8102aa, SUC_lexicon_tags_12 },
    { 0xbd4102aa, SUC_lexicon_tags_49 },
    { 0x6bc102ab, SUC_lexicon_tags_23 },
    { 0xf68102ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2102b0, SUC_lexicon_tags_42 },
    { 0xe12102b1, SUC_lexicon_tags_37 },
    { 0xb96102b2, SUC_lexicon_tags_38 },
    { 0xa32102b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x808102b5, SUC_lexicon_tags_138 },
    { 0x1d2102b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856102ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456102be, SUC_lexicon_tags_37 },
    { 0xfee102bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa8a102c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304102c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b6102c7, SUC_lexicon_tags_17 },
    { 0xefe102c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106102cc, SUC_lexicon_tags_35 },
    { 0x40102cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e102d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88102da, SUC_lexicon_tags_15 },
    { 0x604102db, SUC_lexicon_tags_47 },
    { 0x182102da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x352102de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x588102e0, SUC_lexicon_tags_47 },
    { 0x6d8102e1, SUC_lexicon_tags_57 },
    { 0x996102e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8102e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0102e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf72102eb, SUC_lexicon_tags_104 },
    { 0xfe0102ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ae102ee, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4102f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x506102f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c102fe, SUC_lexicon_tags_62 },
    { 0x7b0102ff, SUC_lexicon_tags_17 },
    { 0xfbe10300, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83c10302, SUC_lexicon_tags_27 },
    { 0x14e10303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce210307, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e1030f, SUC_lexicon_tags_1 },
    { 0x42610310, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4010313, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be10318, SUC_lexicon_tags_30 },
    { 0x9a410319, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba1031c, SUC_lexicon_tags_15 },
    { 0xcde1031d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c010320, SUC_lexicon_tags_12 },
    { 0xd9010321, SUC_lexicon_tags_15 },
    { 0x89c10322, SUC_lexicon_tags_38 },
    { 0xc010323, SUC_lexicon_tags_45 },
    { 0xe7e10322, SUC_lexicon_tags_1 },
    { 0xd6c10325, SUC_lexicon_tags_0 },
    { 0xed410325, SUC_lexicon_tags_15 },
    { 0xba210325, SUC_lexicon_tags_82 },
    { 0xade10328, SUC_lexicon_tags_17 },
    { 0x8ac10328, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb01032d, SUC_lexicon_tags_1 },
    { 0x86e1032d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4410330, SUC_lexicon_tags_84 },
    { 0xa0610330, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb610335, SUC_lexicon_tags_25 },
    { 0xf4c10336, SUC_lexicon_tags_27 },
    { 0x21610336, SUC_lexicon_tags_1 },
    { 0x61210338, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa1a1033a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa061033d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7441033f, SUC_lexicon_tags_15 },
    { 0x3be1033f, SUC_lexicon_tags_19 },
    { 0xe3810341, SUC_lexicon_tags_419 },
    { 0x2981033f, SUC_lexicon_tags_49 },
    { 0xdb410340, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16610346, SUC_lexicon_tags_2 },
    { 0x1ea10347, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49810349, SUC_lexicon_tags_7 },
    { 0xffa1034a, SUC_lexicon_tags_1 },
    { 0xb001034a, SUC_lexicon_tags_33 },
    { 0xc0e1034a, SUC_lexicon_tags_7 },
    { 0x4161034d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad610352, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb210355, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6610358, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1881035a, SUC_lexicon_tags_11 },
    { 0x62a1035a, SUC_lexicon_tags_0 },
    { 0x5001035c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfb21035e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da10361, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e10364, SUC_lexicon_tags_27 },
    { 0x7de10365, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a1036d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x87e1036f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2810376, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca1037a, SUC_lexicon_tags_15 },
    { 0xf161037a, SUC_lexicon_tags_37 },
    { 0x7ae1037a, SUC_lexicon_tags_11 },
    { 0xf4c1037c, SUC_lexicon_tags_171 },
    { 0x73e1037d, SUC_lexicon_tags_11 },
    { 0x2c01037d, SUC_lexicon_tags_12 },
    { 0xe8e10380, SUC_lexicon_tags_7 },
    { 0xf7010381, SUC_lexicon_tags_1 },
    { 0x85810380, SUC_lexicon_tags_31 },
    { 0x39610382, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f210385, SUC_lexicon_tags_8 },
    { 0xc8010386, SUC_lexicon_tags_17 },
    { 0xd3a10385, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80410389, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e1038d, SUC_lexicon_tags_63 },
    { 0x99e1038e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c810391, SUC_lexicon_tags_37 },
    { 0x44a10392, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x91e10394, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82210396, SUC_lexicon_tags_17 },
    { 0x8e410397, SUC_lexicon_tags_19 },
    { 0x51a10397, SUC_lexicon_tags_16 },
    { 0xa0e10399, SUC_lexicon_tags_61 },
    { 0x68610399, SUC_lexicon_tags_10 },
    { 0xf4010399, SUC_lexicon_tags_29 },
    { 0xa641039c, SUC_lexicon_tags_1 },
    { 0x34810396, SUC_lexicon_tags_25 },
    { 0xc201039e, SUC_lexicon_tags_0 },
    { 0x1ba1039a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x324103a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xad6103a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2103a7, SUC_lexicon_tags_1 },
    { 0x294103a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68103aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee103ad, SUC_lexicon_tags_2 },
    { 0xbb0103ae, SUC_lexicon_tags_47 },
    { 0x57c103af, SUC_lexicon_tags_23 },
    { 0xd9e103b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86103b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x494103b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858103b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c103c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232103cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa7c103cd, SUC_lexicon_tags_49 },
    { 0x58e103ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3e103d0, SUC_lexicon_tags_4 },
    { 0xac8103d1, SUC_lexicon_tags_46 },
    { 0x486103d2, SUC_lexicon_tags_30 },
    { 0x48a103d3, SUC_lexicon_tags_6 },
    { 0x4e2103d1, SUC_lexicon_tags_11 },
    { 0xb62103d5, SUC_lexicon_tags_23 },
    { 0xd0a103d6, SUC_lexicon_tags_26 },
    { 0x80c103d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c103de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56103e3, SUC_lexicon_tags_151 },
    { 0x8ba103e4, SUC_lexicon_tags_11 },
    { 0x78a103e5, SUC_lexicon_tags_38 },
    { 0x8f0103e6, SUC_lexicon_tags_25 },
    { 0x89a103e6, SUC_lexicon_tags_63 },
    { 0xb88103e7, SUC_lexicon_tags_38 },
    { 0x7a0103e9, SUC_lexicon_tags_14 },
    { 0xbd0103ea, SUC_lexicon_tags_49 },
    { 0xc84103eb, SUC_lexicon_tags_63 },
    { 0xdfa103e6, SUC_lexicon_tags_12 },
    { 0x1c2103ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c103f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x178103f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x588103fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4103ff, SUC_lexicon_tags_31 },
    { 0x19e10400, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea010402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a010406, SUC_lexicon_tags_14 },
    { 0x82010407, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1040b, SUC_lexicon_tags_2 },
    { 0x64a1040c, SUC_lexicon_tags_77 },
    { 0x7681040c, SUC_lexicon_tags_25 },
    { 0x9401040e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc10414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0610418, SUC_lexicon_tags_118 },
    { 0x9d610418, SUC_lexicon_tags_25 },
    { 0xec61041a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a1041d, SUC_lexicon_tags_79 },
    { 0x1101041e, SUC_lexicon_tags_2 },
    { 0xc1041f, SUC_lexicon_tags_28 },
    { 0xa7c10420, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba10423, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5a210425, SUC_lexicon_tags_6 },
    { 0xe0a10425, SUC_lexicon_tags_28 },
    { 0xa3a10426, SUC_lexicon_tags_30 },
    { 0xfc010428, SUC_lexicon_tags_2 },
    { 0xdfa10429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1042d, SUC_lexicon_tags_25 },
    { 0x341042e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe10431, SUC_lexicon_tags_45 },
    { 0x6c810431, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8210434, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29a10436, SUC_lexicon_tags_47 },
    { 0x88e10436, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c1043f, SUC_lexicon_tags_27 },
    { 0x3d61043f, SUC_lexicon_tags_25 },
    { 0x4a010441, SUC_lexicon_tags_17 },
    { 0xa5610442, SUC_lexicon_tags_37 },
    { 0xe1c10442, SUC_lexicon_tags_7 },
    { 0xe5e10444, SUC_lexicon_tags_16 },
    { 0x25a10445, SUC_lexicon_tags_35 },
    { 0x121043f, SUC_lexicon_tags_27 },
    { 0xe5c10447, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c1044d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60a1044f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x93210452, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25010456, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e1045a, SUC_lexicon_tags_19 },
    { 0x7ce1045b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x441045e, SUC_lexicon_tags_70 },
    { 0xb141045f, SUC_lexicon_tags_25 },
    { 0x7f410460, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xf3010462, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3610464, SUC_lexicon_tags_7 },
    { 0x24810465, SUC_lexicon_tags_2 },
    { 0xf7a10466, SUC_lexicon_tags_49 },
    { 0xdac10467, SUC_lexicon_tags_7 },
    { 0xfd810468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43e1046a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5461046c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a10470, SUC_lexicon_tags_11 },
    { 0xc7210471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a10474, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae810476, SUC_lexicon_tags_11 },
    { 0x23e10477, SUC_lexicon_tags_0 },
    { 0xf9610476, SUC_lexicon_tags_12 },
    { 0xcc010479, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3441047b, SUC_lexicon_tags_7 },
    { 0x1141047b, SUC_lexicon_tags_17 },
    { 0x4a1047c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xba210480, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13010484, SUC_lexicon_tags_60 },
    { 0x4010485, SUC_lexicon_tags_84 },
    { 0xc7010486, SUC_lexicon_tags_26 },
    { 0x89010486, SUC_lexicon_tags_17 },
    { 0xd9210488, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa801048a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xda01048c, SUC_lexicon_tags_27 },
    { 0x3ac1048c, SUC_lexicon_tags_16 },
    { 0xa0e1048d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32410492, SUC_lexicon_tags_16 },
    { 0xbfe10493, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e610499, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2461049c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2104a0, SUC_lexicon_tags_14 },
    { 0x784104a1, SUC_lexicon_tags_27 },
    { 0xf0104a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0104a6, SUC_lexicon_tags_2 },
    { 0xb76104a7, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca104b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54104b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x748104b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4104bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c104bf, SUC_lexicon_tags_9 },
    { 0x32e104c0, SUC_lexicon_tags_34 },
    { 0x1c0104c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4104c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72104c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0104cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50104d5, SUC_lexicon_tags_12 },
    { 0x2b0104d6, SUC_lexicon_tags_25 },
    { 0x9e0104d7, SUC_lexicon_tags_17 },
    { 0xd1a104d5, SUC_lexicon_tags_17 },
    { 0xb18104d9, SUC_lexicon_tags_17 },
    { 0xb2a104d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda104de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2104e2, SUC_lexicon_tags_38 },
    { 0xf54104e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c104e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc104eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182104f1, SUC_lexicon_tags_48 },
    { 0xc7c104f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x398104f4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee104f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754104fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c10504, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3e10506, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd810509, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c1050c, SUC_lexicon_tags_1 },
    { 0x6141050d, SUC_lexicon_tags_49 },
    { 0xb321050d, SUC_lexicon_tags_32 },
    { 0x3061050e, SUC_lexicon_tags_11 },
    { 0x1cc10510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35810512, SUC_lexicon_tags_3 },
    { 0x6c010513, SUC_lexicon_tags_1 },
    { 0x79410514, SUC_lexicon_tags_1 },
    { 0x56610512, SUC_lexicon_tags_11 },
    { 0x32210512, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a1051a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d61051d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76610520, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x72610523, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b41052e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4210531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe10534, SUC_lexicon_tags_15 },
    { 0x66e10535, SUC_lexicon_tags_11 },
    { 0x21610535, SUC_lexicon_tags_43 },
    { 0x15a10537, SUC_lexicon_tags_11 },
    { 0xfa10538, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9c1053a, SUC_lexicon_tags_33 },
    { 0xec01053b, SUC_lexicon_tags_29 },
    { 0xb9e1053c, SUC_lexicon_tags_47 },
    { 0x6941053c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11410546, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf610548, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1054d, SUC_lexicon_tags_19 },
    { 0x15c1054d, SUC_lexicon_tags_47 },
    { 0xbd21054e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda10552, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57e10554, SUC_lexicon_tags_39 },
    { 0xeb610554, SUC_lexicon_tags_1 },
    { 0x47810554, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x97a10558, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e1055b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23410561, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e10564, SUC_lexicon_tags_25 },
    { 0x96810564, SUC_lexicon_tags_25 },
    { 0x38e10564, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x10569, SUC_lexicon_tags_43 },
    { 0x4e010569, SUC_lexicon_tags_66 },
    { 0x1121056b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ce1056d, SUC_lexicon_tags_27 },
    { 0x7c01056d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa1610570, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc0a10572, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf410575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae410581, SUC_lexicon_tags_15 },
    { 0x90e10581, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9a10584, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d210589, SUC_lexicon_tags_0 },
    { 0xf8c1058a, SUC_lexicon_tags_30 },
    { 0x73c1058b, SUC_lexicon_tags_19 },
    { 0x3e81058b, SUC_lexicon_tags_23 },
    { 0xfc01058b, SUC_lexicon_tags_24 },
    { 0x46a10589, SUC_lexicon_tags_17 },
    { 0xf561058c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71810593, SUC_lexicon_tags_46 },
    { 0x9c810593, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c10597, SUC_lexicon_tags_47 },
    { 0x9b410598, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6241059c, SUC_lexicon_tags_4 },
    { 0xaba1059c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfca1059f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde105a8, SUC_lexicon_tags_62 },
    { 0xb54105a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c105ad, SUC_lexicon_tags_7 },
    { 0x902105ae, SUC_lexicon_tags_19 },
    { 0x93e105af, SUC_lexicon_tags_30 },
    { 0xa2a105b0, SUC_lexicon_tags_1 },
    { 0x4d6105ae, SUC_lexicon_tags_1 },
    { 0xda2105ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812105b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c105b9, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x976105bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a105be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x98a105c0, SUC_lexicon_tags_12 },
    { 0xdcc105c1, SUC_lexicon_tags_32 },
    { 0x4a0105c2, SUC_lexicon_tags_60 },
    { 0x7d2105c3, SUC_lexicon_tags_16 },
    { 0x828105c1, SUC_lexicon_tags_34 },
    { 0xe42105c0, SUC_lexicon_tags_12 },
    { 0xa7a105c1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e105c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8105cd, SUC_lexicon_tags_2 },
    { 0xc3e105cd, SUC_lexicon_tags_32 },
    { 0x68c105cf, SUC_lexicon_tags_3 },
    { 0xe7c105cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x550105d2, SUC_lexicon_tags_14 },
    { 0xe50105d3, SUC_lexicon_tags_27 },
    { 0xb74105d4, SUC_lexicon_tags_23 },
    { 0xe04105d4, SUC_lexicon_tags_11 },
    { 0x94e105d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71e105d8, SUC_lexicon_tags_0 },
    { 0xcc105d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a105df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfda105e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2105e3, SUC_lexicon_tags_6 },
    { 0x1be105e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x832105e6, SUC_lexicon_tags_5 },
    { 0x46e105e6, SUC_lexicon_tags_38 },
    { 0x77c105e6, SUC_lexicon_tags_22 },
    { 0x746105e8, SUC_lexicon_tags_25 },
    { 0x804105e9, SUC_lexicon_tags_82 },
    { 0x3e105e7, SUC_lexicon_tags_44 },
    { 0x5e2105ec, SUC_lexicon_tags_1 },
    { 0x838105ed, SUC_lexicon_tags_11 },
    { 0x7e105ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x554105f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ec105f3, SUC_lexicon_tags_47 },
    { 0xc52105f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512105f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4105fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d210600, SUC_lexicon_tags_25 },
    { 0x2e410600, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee10605, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e1060b, SUC_lexicon_tags_12 },
    { 0xeae1060c, SUC_lexicon_tags_25 },
    { 0x32c1060c, SUC_lexicon_tags_25 },
    { 0xc201060c, SUC_lexicon_tags_13 },
    { 0xd0c1060f, SUC_lexicon_tags_11 },
    { 0x7961060e, SUC_lexicon_tags_65 },
    { 0xfb610611, SUC_lexicon_tags_84 },
    { 0xf141060d, SUC_lexicon_tags_14 },
    { 0xe3210613, SUC_lexicon_tags_12 },
    { 0x62810610, SUC_lexicon_tags_25 },
    { 0xc8210611, SUC_lexicon_tags_84 },
    { 0x8461060c, SUC_lexicon_tags_30 },
    { 0x8da10617, SUC_lexicon_tags_17 },
    { 0x1621060b, SUC_lexicon_tags_14 },
    { 0xc8a10619, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3601061b, SUC_lexicon_tags_2 },
    { 0x4fa1061b, SUC_lexicon_tags_29 },
    { 0xeb01061d, SUC_lexicon_tags_19 },
    { 0x7aa1061e, SUC_lexicon_tags_0 },
    { 0x8121061d, SUC_lexicon_tags_2 },
    { 0xbc41061b, SUC_lexicon_tags_11 },
    { 0x2e010621, SUC_lexicon_tags_2 },
    { 0xa8c10622, SUC_lexicon_tags_47 },
    { 0xa1610623, SUC_lexicon_tags_0 },
    { 0xade10624, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea10628, SUC_lexicon_tags_11 },
    { 0xcf610629, SUC_lexicon_tags_27 },
    { 0x3ee1062a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4e1062c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8fa1062e, SUC_lexicon_tags_19 },
    { 0x4901062f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88410632, SUC_lexicon_tags_17 },
    { 0x83010633, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27610638, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec1063e, SUC_lexicon_tags_8 },
    { 0x6781063e, SUC_lexicon_tags_60 },
    { 0x98610640, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4610644, SUC_lexicon_tags_79 },
    { 0xd0010644, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a81064c, SUC_lexicon_tags_25 },
    { 0xad41064d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c810651, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb210654, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66610658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde01065a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3410660, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ec10662, SUC_lexicon_tags_43 },
    { 0xdea10663, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x71810665, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ea10667, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7681066c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac210675, SUC_lexicon_tags_11 },
    { 0xb6210675, SUC_lexicon_tags_2 },
    { 0xe3610676, SUC_lexicon_tags_49 },
    { 0x44c10676, SUC_lexicon_tags_19 },
    { 0x41810679, SUC_lexicon_tags_63 },
    { 0x37e1067a, SUC_lexicon_tags_10 },
    { 0x35410677, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc181067e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54010684, SUC_lexicon_tags_27 },
    { 0xc2c10684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10010687, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1241068a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d21068f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38610693, SUC_lexicon_tags_97 },
    { 0xe2210694, SUC_lexicon_tags_19 },
    { 0x92e10695, SUC_lexicon_tags_56 },
    { 0x4b810695, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17a10698, SUC_lexicon_tags_57 },
    { 0xb7610699, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b41069c, SUC_lexicon_tags_43 },
    { 0x8f81069d, SUC_lexicon_tags_27 },
    { 0x99e1069d, SUC_lexicon_tags_45 },
    { 0xf841069c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaba106a1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf04106a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf0106a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908106b1, SUC_lexicon_tags_43 },
    { 0x9b6106b1, SUC_lexicon_tags_11 },
    { 0xa3c106b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00106b6, SUC_lexicon_tags_26 },
    { 0xa36106b6, SUC_lexicon_tags_14 },
    { 0x8a2106b8, SUC_lexicon_tags_15 },
    { 0x49c106b8, SUC_lexicon_tags_47 },
    { 0xc3e106b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa106bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc106c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa6106c8, SUC_lexicon_tags_8 },
    { 0x3aa106c8, SUC_lexicon_tags_19 },
    { 0x5ee106c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da106d1, SUC_lexicon_tags_11 },
    { 0xac6106d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10106d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c106db, SUC_lexicon_tags_60 },
    { 0xfe6106dc, SUC_lexicon_tags_224 },
    { 0xe4c106db, SUC_lexicon_tags_32 },
    { 0xc14106dc, SUC_lexicon_tags_47 },
    { 0xe0a106df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x586106e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154106e5, SUC_lexicon_tags_16 },
    { 0xc2106e5, SUC_lexicon_tags_15 },
    { 0x76106e7, SUC_lexicon_tags_11 },
    { 0xf9e106e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58106eb, SUC_lexicon_tags_16 },
    { 0xc5c106eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfd0106ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0106f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70106f6, SUC_lexicon_tags_19 },
    { 0x1d6106f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40106fb, SUC_lexicon_tags_38 },
    { 0x8c106fc, SUC_lexicon_tags_31 },
    { 0x78e106fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e010701, SUC_lexicon_tags_6 },
    { 0x80410702, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed410708, SUC_lexicon_tags_56 },
    { 0x34e10709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ac1070b, SUC_lexicon_tags_53 },
    { 0xcb81070c, SUC_lexicon_tags_2 },
    { 0x4701070d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2210712, SUC_lexicon_tags_13 },
    { 0x8d810713, SUC_lexicon_tags_23 },
    { 0x4aa10714, SUC_lexicon_tags_29 },
    { 0xd1610715, SUC_lexicon_tags_1 },
    { 0x58c10712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd61071c, SUC_lexicon_tags_7 },
    { 0xdee1071d, SUC_lexicon_tags_11 },
    { 0xf421071e, SUC_lexicon_tags_12 },
    { 0x1fa1071f, SUC_lexicon_tags_17 },
    { 0xf1a10720, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea10723, SUC_lexicon_tags_2 },
    { 0x64810723, SUC_lexicon_tags_8 },
    { 0xee610724, SUC_lexicon_tags_37 },
    { 0x50210723, SUC_lexicon_tags_7 },
    { 0x39a10727, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6681072b, SUC_lexicon_tags_17 },
    { 0x3fa1072c, SUC_lexicon_tags_16 },
    { 0xa061072c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff210733, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a810739, SUC_lexicon_tags_2 },
    { 0xe961073a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7341073d, SUC_lexicon_tags_14 },
    { 0xbb61073e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fe10740, SUC_lexicon_tags_47 },
    { 0x1c410741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60010744, SUC_lexicon_tags_1 },
    { 0xf5210744, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x97810747, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5921074b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xeb41074d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x841074f, SUC_lexicon_tags_49 },
    { 0x6a61074f, SUC_lexicon_tags_25 },
    { 0x1901074f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c410754, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc10758, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e1075f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51010763, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa010766, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c1076a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4421076d, SUC_lexicon_tags_60 },
    { 0xdb61076e, SUC_lexicon_tags_0 },
    { 0xf81076f, SUC_lexicon_tags_14 },
    { 0x1fe10770, SUC_lexicon_tags_1 },
    { 0x37a10770, SUC_lexicon_tags_11 },
    { 0x72a10771, SUC_lexicon_tags_26 },
    { 0x29010770, SUC_lexicon_tags_62 },
    { 0x99410774, SUC_lexicon_tags_84 },
    { 0xf7410773, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ac10777, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e1077c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66a1077e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13010780, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15210784, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x51410786, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa810789, SUC_lexicon_tags_7 },
    { 0x3a01078a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a1078f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33210797, SUC_lexicon_tags_12 },
    { 0xec210798, SUC_lexicon_tags_7 },
    { 0x5f410799, SUC_lexicon_tags_9 },
    { 0xdca1079a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca61079c, SUC_lexicon_tags_2 },
    { 0xb9a1079c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x136107a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e107a3, SUC_lexicon_tags_32 },
    { 0xe5c107a3, SUC_lexicon_tags_21 },
    { 0xad4107a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36107a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6107ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370107b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10107b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e107bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896107c3, SUC_lexicon_tags_45 },
    { 0x31a107c3, SUC_lexicon_tags_1 },
    { 0x4ce107c5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a107c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e107cc, SUC_lexicon_tags_13 },
    { 0xd5e107cd, SUC_lexicon_tags_11 },
    { 0x8a8107cc, SUC_lexicon_tags_17 },
    { 0xe62107cf, SUC_lexicon_tags_6 },
    { 0xff6107ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3c107d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84107d6, SUC_lexicon_tags_17 },
    { 0xd58107d7, SUC_lexicon_tags_65 },
    { 0x15a107d7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4107dc, SUC_lexicon_tags_12 },
    { 0x38c107dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be107e0, SUC_lexicon_tags_47 },
    { 0xc04107e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4107ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a107ef, SUC_lexicon_tags_7 },
    { 0xab2107f0, SUC_lexicon_tags_17 },
    { 0x76107f1, SUC_lexicon_tags_31 },
    { 0xe3c107f2, SUC_lexicon_tags_2 },
    { 0x28c107f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04107f6, SUC_lexicon_tags_20 },
    { 0x57c107f7, SUC_lexicon_tags_6 },
    { 0x1d0107f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6107fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c10800, SUC_lexicon_tags_27 },
    { 0x71e10801, SUC_lexicon_tags_30 },
    { 0x7610800, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74810805, SUC_lexicon_tags_45 },
    { 0x5c810806, SUC_lexicon_tags_30 },
    { 0xc4210805, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0x49c10809, SUC_lexicon_tags_17 },
    { 0x95e10809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c1080d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xae1080f, SUC_lexicon_tags_25 },
    { 0x38410810, SUC_lexicon_tags_7 },
    { 0xc5410810, SUC_lexicon_tags_15 },
    { 0xbb810810, SUC_lexicon_tags_23 },
    { 0xe901080f, SUC_lexicon_tags_12 },
    { 0xb121080f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf610817, SUC_lexicon_tags_6 },
    { 0xf7e10817, SUC_lexicon_tags_21 },
    { 0xda210819, SUC_lexicon_tags_21 },
    { 0x6741081a, SUC_lexicon_tags_49 },
    { 0x1ec1081b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a1081f, SUC_lexicon_tags_37 },
    { 0xb381081f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1610823, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x85a10825, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2801082c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99610833, SUC_lexicon_tags_37 },
    { 0xd4e10834, SUC_lexicon_tags_7 },
    { 0xaf810835, SUC_lexicon_tags_2 },
    { 0xc5010836, SUC_lexicon_tags_23 },
    { 0xec210837, SUC_lexicon_tags_17 },
    { 0x62e10838, SUC_lexicon_tags_17 },
    { 0x65010838, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b61083b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9021083f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeec10841, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x38a10843, SUC_lexicon_tags_7 },
    { 0x59210844, SUC_lexicon_tags_2 },
    { 0x31810844, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8ec10847, SUC_lexicon_tags_26 },
    { 0x18610848, SUC_lexicon_tags_14 },
    { 0xbd010847, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c1084c, SUC_lexicon_tags_12 },
    { 0x5921084c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeec1084f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x11610852, SUC_lexicon_tags_29 },
    { 0x8c10852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa10856, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da10859, SUC_lexicon_tags_30 },
    { 0x2010859, SUC_lexicon_tags_23 },
    { 0x2681085b, SUC_lexicon_tags_37 },
    { 0x73c1085c, SUC_lexicon_tags_28 },
    { 0xd41085d, SUC_lexicon_tags_15 },
    { 0x2a01085e, SUC_lexicon_tags_17 },
    { 0x8e1085d, SUC_lexicon_tags_49 },
    { 0x2681085f, SUC_lexicon_tags_55 },
    { 0xd0c10861, SUC_lexicon_tags_15 },
    { 0x4a01085d, SUC_lexicon_tags_47 },
    { 0x2210863, SUC_lexicon_tags_68 },
    { 0x56610864, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8661086b, SUC_lexicon_tags_7 },
    { 0x5321086c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b61086e, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc10873, SUC_lexicon_tags_14 },
    { 0x72610873, SUC_lexicon_tags_2 },
    { 0xdb210875, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0x12e10877, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9381087a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe781087c, SUC_lexicon_tags_23 },
    { 0xe041087c, SUC_lexicon_tags_26 },
    { 0xada1087c, SUC_lexicon_tags_84 },
    { 0xe481087e, SUC_lexicon_tags_68 },
    { 0xa0810880, SUC_lexicon_tags_0 },
    { 0x8d410881, SUC_lexicon_tags_25 },
    { 0x72610880, SUC_lexicon_tags_15 },
    { 0xf0c1087e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a10887, SUC_lexicon_tags_15 },
    { 0x1210888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb941088d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72810898, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc1089b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa761089e, SUC_lexicon_tags_83 },
    { 0x8801089f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc108a3, SUC_lexicon_tags_1 },
    { 0xd62108a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2108a8, SUC_lexicon_tags_11 },
    { 0x3d6108a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a108b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe108b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a108bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98108c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8108c3, SUC_lexicon_tags_37 },
    { 0xc0a108c3, SUC_lexicon_tags_11 },
    { 0xe3e108c4, SUC_lexicon_tags_12 },
    { 0x472108c3, SUC_lexicon_tags_38 },
    { 0x6da108c7, SUC_lexicon_tags_19 },
    { 0xc38108c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26e108ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd58108cc, SUC_lexicon_tags_25 },
    { 0xd9e108cd, SUC_lexicon_tags_14 },
    { 0x610108ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34108d1, SUC_lexicon_tags_2 },
    { 0x1a6108d2, SUC_lexicon_tags_55 },
    { 0x954108d3, SUC_lexicon_tags_43 },
    { 0x4b4108d4, SUC_lexicon_tags_14 },
    { 0xf9c108d5, SUC_lexicon_tags_38 },
    { 0x6e4108d6, SUC_lexicon_tags_118 },
    { 0x72a108d7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0108db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x690108dd, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca108e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x902108e3, SUC_lexicon_tags_12 },
    { 0xfbe108e4, SUC_lexicon_tags_38 },
    { 0x95e108e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40108eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce4108ed, SUC_lexicon_tags_4 },
    { 0x1ea108ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb16108f0, SUC_lexicon_tags_2 },
    { 0x668108f1, SUC_lexicon_tags_21 },
    { 0x90c108f2, SUC_lexicon_tags_114 },
    { 0x21c108f3, SUC_lexicon_tags_47 },
    { 0xf0e108f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42108fb, SUC_lexicon_tags_91 },
    { 0xbd8108fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc108ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf5010901, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47410909, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b61090b, SUC_lexicon_tags_19 },
    { 0x9621090c, SUC_lexicon_tags_37 },
    { 0x5161090c, SUC_lexicon_tags_56 },
    { 0xb141090e, SUC_lexicon_tags_25 },
    { 0xf821090e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa10911, SUC_lexicon_tags_36 },
    { 0xdda10911, SUC_lexicon_tags_32 },
    { 0x62610913, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f010918, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a10922, SUC_lexicon_tags_15 },
    { 0x6ae10923, SUC_lexicon_tags_29 },
    { 0x66610924, SUC_lexicon_tags_15 },
    { 0x29610925, SUC_lexicon_tags_17 },
    { 0x8d810926, SUC_lexicon_tags_14 },
    { 0xde010926, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e01092a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf01092c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3481092e, SUC_lexicon_tags_6 },
    { 0x44a1092f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a10932, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d410935, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c81093b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3a1093d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9410940, SUC_lexicon_tags_82 },
    { 0x81e10941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e10946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ea10948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b610956, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61610958, SUC_lexicon_tags_7 },
    { 0xe9610959, SUC_lexicon_tags_32 },
    { 0x97e1095a, SUC_lexicon_tags_84 },
    { 0xe521095b, SUC_lexicon_tags_351 },
    { 0x1c1095b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4210964, SUC_lexicon_tags_9 },
    { 0x77810964, SUC_lexicon_tags_63 },
    { 0x49610965, SUC_lexicon_tags_40 },
    { 0x61e10965, SUC_lexicon_tags_56 },
    { 0xe2e10964, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee1096d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2610973, SUC_lexicon_tags_6 },
    { 0x8c810974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed810978, SUC_lexicon_tags_60 },
    { 0x54c10979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56010982, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe7010984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabe10986, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc5a10988, SUC_lexicon_tags_65 },
    { 0xd9e10988, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee41098b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa10990, SUC_lexicon_tags_11 },
    { 0xe5e10991, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68010994, SUC_lexicon_tags_25 },
    { 0x5c610995, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c010998, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae1099c, SUC_lexicon_tags_17 },
    { 0xe9e1099d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e109a0, SUC_lexicon_tags_37 },
    { 0x730109a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x418109a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762109aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x338109ac, SUC_lexicon_tags_60 },
    { 0x890109ad, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xcfe109af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x36a109b1, SUC_lexicon_tags_11 },
    { 0x696109b2, SUC_lexicon_tags_60 },
    { 0x772109b3, SUC_lexicon_tags_49 },
    { 0x2d2109b4, SUC_lexicon_tags_37 },
    { 0xc40109b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x344109b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0109bb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa109c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa2c109c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b2109c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e109cd, SUC_lexicon_tags_15 },
    { 0x8fe109ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c2109d0, SUC_lexicon_tags_27 },
    { 0x7e0109d0, SUC_lexicon_tags_1 },
    { 0x804109d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa1a109d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa16109d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80109da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4d0109dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88109df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34109e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x134109e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x620109e7, SUC_lexicon_tags_33 },
    { 0xd16109e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x736109ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e109ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4a8109ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2109f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8109f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a810a08, SUC_lexicon_tags_12 },
    { 0x3b210a09, SUC_lexicon_tags_15 },
    { 0x99410a0a, SUC_lexicon_tags_21 },
    { 0x62e10a0b, SUC_lexicon_tags_0 },
    { 0xec610a0b, SUC_lexicon_tags_8 },
    { 0xf4610a08, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec210a15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae10a1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b410a1f, SUC_lexicon_tags_11 },
    { 0x4aa10a20, SUC_lexicon_tags_47 },
    { 0x29410a21, SUC_lexicon_tags_31 },
    { 0x7a410a1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8410a24, SUC_lexicon_tags_0 },
    { 0x7f210a25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a210a28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3810a2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c10a31, SUC_lexicon_tags_37 },
    { 0xc2210a31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74210a37, SUC_lexicon_tags_14 },
    { 0x46e10a38, SUC_lexicon_tags_8 },
    { 0xb7c10a37, SUC_lexicon_tags_31 },
    { 0x94e10a3a, SUC_lexicon_tags_8 },
    { 0x48a10a38, SUC_lexicon_tags_25 },
    { 0xe5e10a3c, SUC_lexicon_tags_26 },
    { 0x4610a3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e610a3f, SUC_lexicon_tags_17 },
    { 0x26810a3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a810a42, SUC_lexicon_tags_15 },
    { 0x71010a43, SUC_lexicon_tags_37 },
    { 0x10210a43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x19c10a46, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86410a50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x12810a53, SUC_lexicon_tags_2 },
    { 0x1a210a53, SUC_lexicon_tags_15 },
    { 0xa9010a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49010a5c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x26410a5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc610a61, SUC_lexicon_tags_37 },
    { 0x8ba10a61, SUC_lexicon_tags_38 },
    { 0x3ec10a63, SUC_lexicon_tags_17 },
    { 0x99210a64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c10a68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3010a6b, SUC_lexicon_tags_38 },
    { 0x83e10a6b, SUC_lexicon_tags_11 },
    { 0xf4610a6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae10a74, SUC_lexicon_tags_84 },
    { 0x1ec10a74, SUC_lexicon_tags_27 },
    { 0xeaa10a76, SUC_lexicon_tags_32 },
    { 0xbe210a76, SUC_lexicon_tags_11 },
    { 0xad010a75, SUC_lexicon_tags_37 },
    { 0xb610a79, SUC_lexicon_tags_15 },
    { 0x8210a74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d410a7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57610a85, SUC_lexicon_tags_49 },
    { 0xf7a10a85, SUC_lexicon_tags_11 },
    { 0xe5810a85, SUC_lexicon_tags_1445 },
    { 0x98010a85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb610a8b, SUC_lexicon_tags_37 },
    { 0xe0c10a8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4c10a8e, SUC_lexicon_tags_16 },
    { 0x70a10a8f, SUC_lexicon_tags_6 },
    { 0xf1e10a90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11410a94, SUC_lexicon_tags_1 },
    { 0x7f010a95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36810a9c, SUC_lexicon_tags_19 },
    { 0x64a10a9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9410aa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13610aa5, SUC_lexicon_tags_19 },
    { 0xef010aa6, SUC_lexicon_tags_1 },
    { 0xde610aa7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6c10aa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41210ab0, SUC_lexicon_tags_21 },
    { 0xe1410ab1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb1010ab3, SUC_lexicon_tags_17 },
    { 0xe5610ab3, SUC_lexicon_tags_38 },
    { 0x3aa10ab5, SUC_lexicon_tags_11 },
    { 0xb8810ab6, SUC_lexicon_tags_17 },
    { 0x41610ab7, SUC_lexicon_tags_44 },
    { 0x4c410ab8, SUC_lexicon_tags_25 },
    { 0xdb410ab7, SUC_lexicon_tags_19 },
    { 0x4e810aba, SUC_lexicon_tags_25 },
    { 0x2ee10ab3, SUC_lexicon_tags_23 },
    { 0xbbe10ab8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x13010abf, SUC_lexicon_tags_37 },
    { 0xd8410abf, SUC_lexicon_tags_13 },
    { 0x57e10ac1, SUC_lexicon_tags_27 },
    { 0x40410ac1, SUC_lexicon_tags_87 },
    { 0x1ac10ac3, SUC_lexicon_tags_8 },
    { 0x1c010ac1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb810ac6, SUC_lexicon_tags_28 },
    { 0x88210ac7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb410acc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5610ad1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43210ad3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8410ad6, SUC_lexicon_tags_48 },
    { 0xfec10ad6, SUC_lexicon_tags_2 },
    { 0xca410ad7, SUC_lexicon_tags_114 },
    { 0xb5c10ad9, SUC_lexicon_tags_14 },
    { 0xcb410ada, SUC_lexicon_tags_14 },
    { 0x95210ad7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92410ade, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c410ae1, SUC_lexicon_tags_27 },
    { 0xcfe10ae2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b210ae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15810ae9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4210aef, SUC_lexicon_tags_7 },
    { 0xaba10af0, SUC_lexicon_tags_19 },
    { 0x33e10af1, SUC_lexicon_tags_160 },
    { 0xfd410aef, SUC_lexicon_tags_15 },
    { 0x4ea10af2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81a10af5, SUC_lexicon_tags_27 },
    { 0x4c010af5, SUC_lexicon_tags_30 },
    { 0xcba10af6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd8810af9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b410afd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96e10aff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a810b02, SUC_lexicon_tags_0 },
    { 0x47c10b03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c10b06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1210b0b, SUC_lexicon_tags_1063 },
    { 0xf010b0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa410b0e, SUC_lexicon_tags_1 },
    { 0xc3810b0e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc10b12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57410b14, SUC_lexicon_tags_1 },
    { 0xfa810b14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63a10b17, SUC_lexicon_tags_15 },
    { 0x21810b18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e10b1c, SUC_lexicon_tags_9 },
    { 0xc9410b1c, SUC_lexicon_tags_11 },
    { 0x5c810b1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80610b22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c210b29, SUC_lexicon_tags_11 },
    { 0x28e10b2a, SUC_lexicon_tags_60 },
    { 0x1ce10b2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a10b30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44210b37, SUC_lexicon_tags_45 },
    { 0x33810b38, SUC_lexicon_tags_1 },
    { 0x64210b38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76a10b3b, SUC_lexicon_tags_37 },
    { 0xfd610b3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a10b42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c10b45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68e10b47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x69210b49, SUC_lexicon_tags_43 },
    { 0xa0810b4a, SUC_lexicon_tags_43 },
    { 0x66610b4a, SUC_lexicon_tags_9 },
    { 0x26610b4c, SUC_lexicon_tags_0 },
    { 0x96c10b4d, SUC_lexicon_tags_2 },
    { 0x23210b4e, SUC_lexicon_tags_23 },
    { 0xaee10b4f, SUC_lexicon_tags_17 },
    { 0xc9410b4d, SUC_lexicon_tags_66 },
    { 0xf9e10b51, SUC_lexicon_tags_1 },
    { 0x73a10b52, SUC_lexicon_tags_45 },
    { 0xcc210b52, SUC_lexicon_tags_63 },
    { 0xb3810b4e, SUC_lexicon_tags_47 },
    { 0xd3010b52, SUC_lexicon_tags_11 },
    { 0x9410b56, SUC_lexicon_tags_63 },
    { 0x60010b56, SUC_lexicon_tags_25 },
    { 0x48810b57, SUC_lexicon_tags_8 },
    { 0x4e010b55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc10b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c10b60, SUC_lexicon_tags_1 },
    { 0x76c10b60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1810b63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x49c10b65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7ba10b67, SUC_lexicon_tags_17 },
    { 0x5e810b67, SUC_lexicon_tags_19 },
    { 0xee810b69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe10b6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e10b78, SUC_lexicon_tags_51 },
    { 0x28c10b79, SUC_lexicon_tags_39 },
    { 0xbae10b78, SUC_lexicon_tags_45 },
    { 0xc9c10b78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e610b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x210b80, SUC_lexicon_tags_37 },
    { 0x50210b81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefc10b83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd210b86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0210b89, SUC_lexicon_tags_15 },
    { 0x21a10b89, SUC_lexicon_tags_15 },
    { 0xb2610b8b, SUC_lexicon_tags_1 },
    { 0xe9210b8b, SUC_lexicon_tags_68 },
    { 0x3a610b8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62010b95, SUC_lexicon_tags_29 },
    { 0x9cc10b96, SUC_lexicon_tags_37 },
    { 0x86610b96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99210b9f, SUC_lexicon_tags_49 },
    { 0x71a10b9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96010ba4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33610bac, SUC_lexicon_tags_6 },
    { 0xe6810bad, SUC_lexicon_tags_28 },
    { 0x30c10bae, SUC_lexicon_tags_26 },
    { 0xe3610bac, SUC_lexicon_tags_40 },
    { 0xcbc10bb0, SUC_lexicon_tags_28 },
    { 0x6c410bb0, SUC_lexicon_tags_0 },
    { 0x5aa10bb1, SUC_lexicon_tags_25 },
    { 0xbde10bae, SUC_lexicon_tags_21 },
    { 0x14010bb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e10bb6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d810bba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x63a10bbc, SUC_lexicon_tags_60 },
    { 0xaf810bbd, SUC_lexicon_tags_19 },
    { 0xf210bbe, SUC_lexicon_tags_16 },
    { 0xd1810bbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c210bc3, SUC_lexicon_tags_17 },
    { 0x14610bc4, SUC_lexicon_tags_60 },
    { 0xec010bc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3010bc7, SUC_lexicon_tags_38 },
    { 0x54e10bc8, SUC_lexicon_tags_11 },
    { 0x22010bc8, SUC_lexicon_tags_63 },
    { 0x42010bc9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c10bce, SUC_lexicon_tags_1 },
    { 0x89c10bcf, SUC_lexicon_tags_63 },
    { 0xf1e10bd0, SUC_lexicon_tags_1 },
    { 0x35a10bcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c10bd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc10bd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c10be0, SUC_lexicon_tags_17 },
    { 0xc10be1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c10be4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x41810be7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48010bec, SUC_lexicon_tags_28 },
    { 0x6ca10bed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9410bf1, SUC_lexicon_tags_7 },
    { 0x98210bf2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73810bf6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x99610bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d810bfb, SUC_lexicon_tags_11 },
    { 0x91010bfc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47610bfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12a10c00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca10c0c, SUC_lexicon_tags_39 },
    { 0x5c610c0d, SUC_lexicon_tags_25 },
    { 0x2f810c0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e10c15, SUC_lexicon_tags_32 },
    { 0x8fe10c16, SUC_lexicon_tags_1 },
    { 0xcd610c17, SUC_lexicon_tags_37 },
    { 0x60a10c16, SUC_lexicon_tags_43 },
    { 0xb8e10c19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc10c1c, SUC_lexicon_tags_8 },
    { 0x2f210c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1410c24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c10c27, SUC_lexicon_tags_65 },
    { 0xd6810c27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63610c2b, SUC_lexicon_tags_17 },
    { 0x11210c2c, SUC_lexicon_tags_17 },
    { 0x7ca10c2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb8e10c2f, SUC_lexicon_tags_42 },
    { 0xc4610c2f, SUC_lexicon_tags_2 },
    { 0x29410c2f, SUC_lexicon_tags_49 },
    { 0x98810c30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa610c36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12410c40, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x50210c43, SUC_lexicon_tags_42 },
    { 0x10010c44, SUC_lexicon_tags_47 },
    { 0x34610c45, SUC_lexicon_tags_9 },
    { 0x24c10c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1010c48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa210c4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26a10c4c, SUC_lexicon_tags_17 },
    { 0x1ca10c4d, SUC_lexicon_tags_37 },
    { 0xd3810c4d, SUC_lexicon_tags_25 },
    { 0xfce10c4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6810c51, SUC_lexicon_tags_9 },
    { 0xd0210c51, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a10c55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe10c58, SUC_lexicon_tags_68 },
    { 0x27e10c59, SUC_lexicon_tags_12 },
    { 0x96410c5a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9ea10c5c, SUC_lexicon_tags_30 },
    { 0xdea10c5d, SUC_lexicon_tags_1 },
    { 0x92a10c5c, SUC_lexicon_tags_11 },
    { 0xce010c5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e010c62, SUC_lexicon_tags_1 },
    { 0x15610c63, SUC_lexicon_tags_6 },
    { 0x3a010c63, SUC_lexicon_tags_17 },
    { 0x57c10c62, SUC_lexicon_tags_45 },
    { 0x2c010c66, SUC_lexicon_tags_1 },
    { 0x58410c63, SUC_lexicon_tags_47 },
    { 0x50010c67, SUC_lexicon_tags_17 },
    { 0xbfc10c69, SUC_lexicon_tags_12 },
    { 0x37e10c6a, SUC_lexicon_tags_12 },
    { 0x51a10c6b, SUC_lexicon_tags_12 },
    { 0xd8610c65, SUC_lexicon_tags_11 },
    { 0x91a10c62, SUC_lexicon_tags_2 },
    { 0x77810c68, SUC_lexicon_tags_16 },
    { 0xacc10c6f, SUC_lexicon_tags_25 },
    { 0xf810c70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc810c75, SUC_lexicon_tags_21 },
    { 0x69610c76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5610c83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6e10c85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b810c87, SUC_lexicon_tags_17 },
    { 0x22810c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b010c8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89010c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12e10c91, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc810c94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d810c98, SUC_lexicon_tags_31 },
    { 0x47810c99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76810c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5810c9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac10ca2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc1810ca4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e10ca8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81610cab, SUC_lexicon_tags_46 },
    { 0xd2a10cab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8410cb1, SUC_lexicon_tags_32 },
    { 0x47410cb2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x36410cb5, SUC_lexicon_tags_1 },
    { 0x22810cb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe10cb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d610cba, SUC_lexicon_tags_31 },
    { 0x34a10cbb, SUC_lexicon_tags_7 },
    { 0xe9410cbc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e610cbf, SUC_lexicon_tags_47 },
    { 0x1e10cc0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e10cc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f010ccc, SUC_lexicon_tags_44 },
    { 0xd4a10ccc, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62210cd2, SUC_lexicon_tags_48 },
    { 0xe6210cd2, SUC_lexicon_tags_15 },
    { 0x3a410cd4, SUC_lexicon_tags_43 },
    { 0x17210cd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c10cd9, SUC_lexicon_tags_15 },
    { 0xfee10cda, SUC_lexicon_tags_12 },
    { 0x410cd9, SUC_lexicon_tags_1 },
    { 0x8a810cdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f010ce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec10ce8, SUC_lexicon_tags_47 },
    { 0x7ae10ce9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8210ced, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6210cef, SUC_lexicon_tags_27 },
    { 0xdfc10cf0, SUC_lexicon_tags_2 },
    { 0x90810cf0, SUC_lexicon_tags_38 },
    { 0x7b610cf0, SUC_lexicon_tags_60 },
    { 0x22410cf2, SUC_lexicon_tags_27 },
    { 0x96410cf4, SUC_lexicon_tags_33 },
    { 0xc010cf5, SUC_lexicon_tags_1 },
    { 0x7fe10cf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb810cf8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e10cfb, SUC_lexicon_tags_27 },
    { 0x65610cfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a810cfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6810d00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce010d05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb610d08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75810d13, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e10d17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96e10d19, SUC_lexicon_tags_1 },
    { 0x18210d1a, SUC_lexicon_tags_25 },
    { 0x6e410d1b, SUC_lexicon_tags_8 },
    { 0x10010d1c, SUC_lexicon_tags_37 },
    { 0xb5010d1d, SUC_lexicon_tags_17 },
    { 0x74010d1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b010d23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81610d2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1810d2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23810d31, SUC_lexicon_tags_65 },
    { 0xb5210d32, SUC_lexicon_tags_63 },
    { 0x87e10d31, SUC_lexicon_tags_17 },
    { 0xb0010d34, SUC_lexicon_tags_31 },
    { 0x90410d32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71010d39, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c10d40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x94010d43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4010d4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1ee10d4d, SUC_lexicon_tags_46 },
    { 0x9610d4e, SUC_lexicon_tags_25 },
    { 0xb2410d4e, SUC_lexicon_tags_1 },
    { 0x20610d50, SUC_lexicon_tags_65 },
    { 0x91610d4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a010d54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f810d57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c10d5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6010d5c, SUC_lexicon_tags_55 },
    { 0x8d210d5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3410d61, SUC_lexicon_tags_31 },
    { 0x7f210d62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86210d64, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae10d67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xba610d69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3010d6d, SUC_lexicon_tags_63 },
    { 0xda810d6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c10d71, SUC_lexicon_tags_1 },
    { 0x1bc10d72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45010d75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2410d78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa10d7e, SUC_lexicon_tags_1 },
    { 0xc7c10d7f, SUC_lexicon_tags_10 },
    { 0x52a10d80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5210d83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38210d87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12010d8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1410d91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31410d9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c10da2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22e10da4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5410da7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb610dab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x31610dae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x30610db1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f810db8, SUC_lexicon_tags_12 },
    { 0x20c10db9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5410dbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43610dbf, SUC_lexicon_tags_14 },
    { 0xe1c10dc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13e10dc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e10dcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a10dd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb610dd5, SUC_lexicon_tags_39 },
    { 0xd9010dd6, SUC_lexicon_tags_8 },
    { 0x3ec10dd7, SUC_lexicon_tags_4 },
    { 0xc3010dd6, SUC_lexicon_tags_26 },
    { 0x24610dd8, SUC_lexicon_tags_27 },
    { 0x80010dd5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c10ddd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a10de0, SUC_lexicon_tags_43 },
    { 0x23610de0, SUC_lexicon_tags_25 },
    { 0x9c10de2, SUC_lexicon_tags_28 },
    { 0x76010de3, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba10de6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x33010de9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce010ded, SUC_lexicon_tags_17 },
    { 0xe4210dee, SUC_lexicon_tags_23 },
    { 0x5ea10def, SUC_lexicon_tags_17 },
    { 0xaae10df0, SUC_lexicon_tags_33 },
    { 0x4c410df1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c10df8, SUC_lexicon_tags_2 },
    { 0x9c210df9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbba10dfb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8a410dfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8610e00, SUC_lexicon_tags_11 },
    { 0x4a810e00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e10e04, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0x58e10e06, SUC_lexicon_tags_0 },
    { 0x33a10e06, SUC_lexicon_tags_17 },
    { 0xe8810e08, SUC_lexicon_tags_37 },
    { 0xff610e09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc610e0b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc6210e0d, SUC_lexicon_tags_47 },
    { 0x8ce10e0e, SUC_lexicon_tags_37 },
    { 0xb8010e0f, SUC_lexicon_tags_1 },
    { 0xc0610e10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1410e13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde410e1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e10e1f, SUC_lexicon_tags_7 },
    { 0x4ba10e20, SUC_lexicon_tags_12 },
    { 0x79010e21, SUC_lexicon_tags_7 },
    { 0xd2210e22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c10e26, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x47810e29, SUC_lexicon_tags_38 },
    { 0x5b610e2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87410e2e, SUC_lexicon_tags_2 },
    { 0xa5210e2f, SUC_lexicon_tags_16 },
    { 0xcc210e30, SUC_lexicon_tags_27 },
    { 0x4a610e2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5610e33, SUC_lexicon_tags_2 },
    { 0x17c10e33, SUC_lexicon_tags_46 },
    { 0xc1610e34, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbf410e37, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2c410e39, SUC_lexicon_tags_19 },
    { 0x35e10e3a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e10e3e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x59c10e40, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33810e44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25210e46, SUC_lexicon_tags_85 },
    { 0xa2810e47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32610e49, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xd1610e4b, SUC_lexicon_tags_2 },
    { 0x2b810e4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba10e50, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e10e5a, SUC_lexicon_tags_25 },
    { 0x50810e5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5c10e5d, SUC_lexicon_tags_3 },
    { 0x90010e5e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4410e64, SUC_lexicon_tags_9 },
    { 0xdc10e64, SUC_lexicon_tags_17 },
    { 0x6c210e66, SUC_lexicon_tags_1 },
    { 0x11210e67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2a010e69, SUC_lexicon_tags_27 },
    { 0x84c10e6a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x1e810e6c, SUC_lexicon_tags_170 },
    { 0xff810e6d, SUC_lexicon_tags_30 },
    { 0xf2c10e6c, SUC_lexicon_tags_11 },
    { 0xae810e6d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a10e73, SUC_lexicon_tags_19 },
    { 0xb0610e73, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1810e78, SUC_lexicon_tags_16 },
    { 0xb3010e78, SUC_lexicon_tags_26 },
    { 0x78810e78, SUC_lexicon_tags_49 },
    { 0x69610e7a, SUC_lexicon_tags_28 },
    { 0x40010e78, SUC_lexicon_tags_12 },
    { 0x16610e7d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93010e84, SUC_lexicon_tags_8 },
    { 0x2b410e85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a10e88, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4a610e8a, SUC_lexicon_tags_49 },
    { 0x22010e8b, SUC_lexicon_tags_2 },
    { 0xa6810e8c, SUC_lexicon_tags_45 },
    { 0xf7810e8d, SUC_lexicon_tags_6 },
    { 0xdc810e8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x69210e90, SUC_lexicon_tags_60 },
    { 0xd9e10e91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc10e96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d610e99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b010ea0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4a810ea2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c10ea5, SUC_lexicon_tags_37 },
    { 0xc9210ea5, SUC_lexicon_tags_40 },
    { 0xb7410ea7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9210eac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e10eb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc610eb7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a10ebb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45210ebe, SUC_lexicon_tags_37 },
    { 0xba810ebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74410ec4, SUC_lexicon_tags_17 },
    { 0x8c410ec5, SUC_lexicon_tags_4 },
    { 0xa4410ec6, SUC_lexicon_tags_6 },
    { 0x26010ec6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x72810ec9, SUC_lexicon_tags_11 },
    { 0x17a10eca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67010ecf, SUC_lexicon_tags_15 },
    { 0x78010ed0, SUC_lexicon_tags_14 },
    { 0xf6810ed1, SUC_lexicon_tags_1 },
    { 0xd4a10ed0, SUC_lexicon_tags_2 },
    { 0xa8410ed2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e610ed6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b810eda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b410edd, SUC_lexicon_tags_33 },
    { 0x9ec10ede, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee210eed, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6cc10eef, SUC_lexicon_tags_66 },
    { 0x6a210eef, SUC_lexicon_tags_17 },
    { 0xbb610ef1, SUC_lexicon_tags_1 },
    { 0x46e10ef2, SUC_lexicon_tags_853 },
    { 0xabe10ef0, SUC_lexicon_tags_23 },
    { 0x1c210ef0, SUC_lexicon_tags_23 },
    { 0x53810ef0, SUC_lexicon_tags_12 },
    { 0x6be10ef5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50410ef8, SUC_lexicon_tags_10 },
    { 0xde10ef8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4410efd, SUC_lexicon_tags_17 },
    { 0x86810efe, SUC_lexicon_tags_15 },
    { 0xce10eff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c610f02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca10f06, SUC_lexicon_tags_7 },
    { 0xf7e10f07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xab610f0a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae410f0e, SUC_lexicon_tags_15 },
    { 0xf6010f0f, SUC_lexicon_tags_38 },
    { 0x56010f10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x82a10f12, SUC_lexicon_tags_17 },
    { 0x19210f13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87210f15, SUC_lexicon_tags_1 },
    { 0x62c10f15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1810f1c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc10f20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a10f25, SUC_lexicon_tags_28 },
    { 0xeea10f26, SUC_lexicon_tags_21 },
    { 0x32e10f26, SUC_lexicon_tags_35 },
    { 0x22e10f28, SUC_lexicon_tags_5 },
    { 0x63210f29, SUC_lexicon_tags_11 },
    { 0x7c10f25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c10f33, SUC_lexicon_tags_28 },
    { 0xa9c10f34, SUC_lexicon_tags_27 },
    { 0xf0810f35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41410f37, SUC_lexicon_tags_15 },
    { 0x12010f38, SUC_lexicon_tags_2 },
    { 0x49010f39, SUC_lexicon_tags_12 },
    { 0x61010f3a, SUC_lexicon_tags_17 },
    { 0xbf410f3b, SUC_lexicon_tags_43 },
    { 0x16610f39, SUC_lexicon_tags_47 },
    { 0x22210f3d, SUC_lexicon_tags_12 },
    { 0x9a410f3e, SUC_lexicon_tags_12 },
    { 0x73a10f3f, SUC_lexicon_tags_49 },
    { 0x93e10f3f, SUC_lexicon_tags_19 },
    { 0xb0810f3e, SUC_lexicon_tags_37 },
    { 0xc1010f40, SUC_lexicon_tags_7 },
    { 0x44410f43, SUC_lexicon_tags_25 },
    { 0x20e10f3e, SUC_lexicon_tags_33 },
    { 0x51a10f45, SUC_lexicon_tags_12 },
    { 0xc7010f3f, SUC_lexicon_tags_59 },
    { 0x20c10f42, SUC_lexicon_tags_49 },
    { 0x58010f48, SUC_lexicon_tags_6 },
    { 0xdf610f49, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c210f4c, SUC_lexicon_tags_12 },
    { 0x95c10f4d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x14210f4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21410f51, SUC_lexicon_tags_4 },
    { 0x39a10f52, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea010f58, SUC_lexicon_tags_18 },
    { 0x88a10f59, SUC_lexicon_tags_1 },
    { 0xd8010f5a, SUC_lexicon_tags_6 },
    { 0x40610f5b, SUC_lexicon_tags_1 },
    { 0x3c10f5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c10f5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74c10f60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ea10f62, SUC_lexicon_tags_12 },
    { 0xb1210f63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a10f67, SUC_lexicon_tags_11 },
    { 0x35810f67, SUC_lexicon_tags_15 },
    { 0xd8a10f69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12210f6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a10f70, SUC_lexicon_tags_31 },
    { 0xc3c10f71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f210f74, SUC_lexicon_tags_17 },
    { 0x31810f75, SUC_lexicon_tags_11 },
    { 0x79a10f76, SUC_lexicon_tags_37 },
    { 0x69a10f76, SUC_lexicon_tags_1 },
    { 0x76010f76, SUC_lexicon_tags_26 },
    { 0x75210f78, SUC_lexicon_tags_12 },
    { 0x8ac10f7a, SUC_lexicon_tags_11 },
    { 0x53810f7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2410f80, SUC_lexicon_tags_27 },
    { 0x70e10f81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a10f84, SUC_lexicon_tags_63 },
    { 0xe8a10f85, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf810f89, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xae210f8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3e10f8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76010f91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a10f95, SUC_lexicon_tags_17 },
    { 0xe9610f96, SUC_lexicon_tags_16 },
    { 0xefe10f97, SUC_lexicon_tags_6 },
    { 0x5e10f98, SUC_lexicon_tags_7 },
    { 0x6d810f98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e410fa5, SUC_lexicon_tags_1 },
    { 0xf4210fa6, SUC_lexicon_tags_17 },
    { 0xc8a10fa7, SUC_lexicon_tags_34 },
    { 0xa1e10fa7, SUC_lexicon_tags_26 },
    { 0x80c10fa9, SUC_lexicon_tags_25 },
    { 0x88c10faa, SUC_lexicon_tags_38 },
    { 0x24610fab, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd010fb0, SUC_lexicon_tags_16 },
    { 0x1b010fb0, SUC_lexicon_tags_6 },
    { 0x21210fb0, SUC_lexicon_tags_15 },
    { 0x36810fb3, SUC_lexicon_tags_12 },
    { 0x9a210fb0, SUC_lexicon_tags_25 },
    { 0x76410fb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca410fb7, SUC_lexicon_tags_38 },
    { 0x8de10fb8, SUC_lexicon_tags_47 },
    { 0x1c210fb7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e10fbc, SUC_lexicon_tags_11 },
    { 0x8b210fbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8410fc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b010fc2, SUC_lexicon_tags_27 },
    { 0xf5810fc3, SUC_lexicon_tags_82 },
    { 0xe2a10fc3, SUC_lexicon_tags_9 },
    { 0x39810fc5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac10fc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x44610fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac810fce, SUC_lexicon_tags_23 },
    { 0x10c10fce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc10fd2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c810fd5, SUC_lexicon_tags_83 },
    { 0x30a10fd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49410fdd, SUC_lexicon_tags_37 },
    { 0xed610fdd, SUC_lexicon_tags_1 },
    { 0xc9610fdf, SUC_lexicon_tags_45 },
    { 0x4e410fdd, SUC_lexicon_tags_11 },
    { 0x9d210fe0, SUC_lexicon_tags_3 },
    { 0xfa410fe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a10fe5, SUC_lexicon_tags_12 },
    { 0x70010fe6, SUC_lexicon_tags_15 },
    { 0x3c210fe7, SUC_lexicon_tags_25 },
    { 0x35610fe8, SUC_lexicon_tags_1 },
    { 0x58c10fe9, SUC_lexicon_tags_37 },
    { 0x1ac10fea, SUC_lexicon_tags_25 },
    { 0x3ce10fe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae10ff8, SUC_lexicon_tags_25 },
    { 0xfc810ff9, SUC_lexicon_tags_7 },
    { 0x9c610ff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8dc10ffc, SUC_lexicon_tags_15 },
    { 0xf8a10ffd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58411001, SUC_lexicon_tags_8 },
    { 0x99c11002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ea11004, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83e11006, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x82011008, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc61100c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73011010, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9011013, SUC_lexicon_tags_17 },
    { 0x23211013, SUC_lexicon_tags_63 },
    { 0xd6a11015, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2201101e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x64a11020, SUC_lexicon_tags_17 },
    { 0xca211021, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9411025, SUC_lexicon_tags_23 },
    { 0xb5811026, SUC_lexicon_tags_33 },
    { 0xa8c11027, SUC_lexicon_tags_11 },
    { 0x48811027, SUC_lexicon_tags_17 },
    { 0xb3811029, SUC_lexicon_tags_11 },
    { 0x9f01102a, SUC_lexicon_tags_11 },
    { 0x64c11027, SUC_lexicon_tags_17 },
    { 0x9241102c, SUC_lexicon_tags_68 },
    { 0x7cc1102d, SUC_lexicon_tags_194 },
    { 0x3b81102e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba11031, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56e11033, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11e11035, SUC_lexicon_tags_15 },
    { 0x1f411036, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd661103a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81103f, SUC_lexicon_tags_0 },
    { 0xa2a1103f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea11044, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89211049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac11050, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44011052, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa5211054, SUC_lexicon_tags_1 },
    { 0xa5811054, SUC_lexicon_tags_37 },
    { 0x64611056, SUC_lexicon_tags_0 },
    { 0xbb011055, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2811059, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6241105b, SUC_lexicon_tags_15 },
    { 0x1081105c, SUC_lexicon_tags_45 },
    { 0x65c1105d, SUC_lexicon_tags_6 },
    { 0xd981105e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55811062, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e11066, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf41106a, SUC_lexicon_tags_25 },
    { 0xcda1106b, SUC_lexicon_tags_470 },
    { 0xad01106a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2341106e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc11076, SUC_lexicon_tags_12 },
    { 0x6fc11077, SUC_lexicon_tags_6 },
    { 0xc4611078, SUC_lexicon_tags_7 },
    { 0x41611079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a11081, SUC_lexicon_tags_63 },
    { 0x8d011082, SUC_lexicon_tags_0 },
    { 0x92411083, SUC_lexicon_tags_48 },
    { 0xd3c11084, SUC_lexicon_tags_9 },
    { 0x8de11084, SUC_lexicon_tags_12 },
    { 0x58a11086, SUC_lexicon_tags_36 },
    { 0xca811081, SUC_lexicon_tags_46 },
    { 0x8d211088, SUC_lexicon_tags_37 },
    { 0x3a411089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc1108f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7211095, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb3611097, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8301109e, SUC_lexicon_tags_51 },
    { 0xe221109e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e110a4, SUC_lexicon_tags_0 },
    { 0x178110a5, SUC_lexicon_tags_49 },
    { 0x548110a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4a8110a8, SUC_lexicon_tags_25 },
    { 0xef0110a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746110ad, SUC_lexicon_tags_17 },
    { 0xea4110ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa110b1, SUC_lexicon_tags_14 },
    { 0xc5e110b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcae110b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6110b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640110bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b2110bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02110c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a4110c6, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xd3c110c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35e110ca, SUC_lexicon_tags_9 },
    { 0x77c110cb, SUC_lexicon_tags_33 },
    { 0x15a110cb, SUC_lexicon_tags_37 },
    { 0x8c6110cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a110d6, SUC_lexicon_tags_47 },
    { 0xd82110d7, SUC_lexicon_tags_38 },
    { 0x2ee110d7, SUC_lexicon_tags_8 },
    { 0x36c110d9, SUC_lexicon_tags_2 },
    { 0x974110da, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946110df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2110e2, SUC_lexicon_tags_15 },
    { 0x2ba110e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608110e8, SUC_lexicon_tags_7 },
    { 0xac6110e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04110f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560110f6, SUC_lexicon_tags_12 },
    { 0x432110f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc110fa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8110ff, SUC_lexicon_tags_11 },
    { 0x9d211100, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35211102, SUC_lexicon_tags_14 },
    { 0xab811103, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e11106, SUC_lexicon_tags_8 },
    { 0x9e011106, SUC_lexicon_tags_37 },
    { 0xa6811107, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd961110a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee11110, SUC_lexicon_tags_12 },
    { 0xfca11110, SUC_lexicon_tags_56 },
    { 0x9e11112, SUC_lexicon_tags_84 },
    { 0xa7611113, SUC_lexicon_tags_19 },
    { 0x20011114, SUC_lexicon_tags_17 },
    { 0x89211115, SUC_lexicon_tags_37 },
    { 0xf8a11116, SUC_lexicon_tags_17 },
    { 0x58611117, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x54011119, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x83a1111b, SUC_lexicon_tags_87 },
    { 0xef41111c, SUC_lexicon_tags_15 },
    { 0xf901111b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa661111f, SUC_lexicon_tags_19 },
    { 0x60a1111f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e1112a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8681112d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3211133, SUC_lexicon_tags_30 },
    { 0x16a11134, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b811137, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2161113a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff211140, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b211146, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa11149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a1114e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10c11150, SUC_lexicon_tags_25 },
    { 0x39011150, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2211155, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x78011157, SUC_lexicon_tags_44 },
    { 0x5f011157, SUC_lexicon_tags_17 },
    { 0x51c11158, SUC_lexicon_tags_11 },
    { 0xe601115a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a61115f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed011164, SUC_lexicon_tags_60 },
    { 0x9b611164, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe011168, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6241116c, SUC_lexicon_tags_11 },
    { 0xc281116d, SUC_lexicon_tags_12 },
    { 0xbd41116e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x78211170, SUC_lexicon_tags_8 },
    { 0xc1611171, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24011174, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b411178, SUC_lexicon_tags_7 },
    { 0x6011179, SUC_lexicon_tags_10 },
    { 0xa7e1117a, SUC_lexicon_tags_2 },
    { 0xfa81117b, SUC_lexicon_tags_33 },
    { 0x47a1117c, SUC_lexicon_tags_154 },
    { 0x6b81117c, SUC_lexicon_tags_35 },
    { 0xbc1117e, SUC_lexicon_tags_65 },
    { 0x1a21117f, SUC_lexicon_tags_46 },
    { 0x2901117f, SUC_lexicon_tags_25 },
    { 0xe1611181, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf1011183, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcd011185, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c11188, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe1118d, SUC_lexicon_tags_25 },
    { 0xb721118e, SUC_lexicon_tags_37 },
    { 0xba21118f, SUC_lexicon_tags_38 },
    { 0xf6c11190, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x72c11192, SUC_lexicon_tags_1 },
    { 0x71c11193, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd21119e, SUC_lexicon_tags_16 },
    { 0xf3a1119e, SUC_lexicon_tags_12 },
    { 0x756111a0, SUC_lexicon_tags_33 },
    { 0x1ea1119e, SUC_lexicon_tags_0 },
    { 0x25a111a2, SUC_lexicon_tags_47 },
    { 0x95a111a2, SUC_lexicon_tags_43 },
    { 0x6b41119f, SUC_lexicon_tags_43 },
    { 0x890111a5, SUC_lexicon_tags_23 },
    { 0x5b41119e, SUC_lexicon_tags_38 },
    { 0x830111a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6111ab, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34111ae, SUC_lexicon_tags_29 },
    { 0xc0c111af, SUC_lexicon_tags_11 },
    { 0xb32111b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6111b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49a111b5, SUC_lexicon_tags_43 },
    { 0xb0111b6, SUC_lexicon_tags_27 },
    { 0x95e111b5, SUC_lexicon_tags_6 },
    { 0x66c111b7, SUC_lexicon_tags_8 },
    { 0x24111b9, SUC_lexicon_tags_37 },
    { 0x4ec111b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc90111bc, SUC_lexicon_tags_11 },
    { 0xfc2111bd, SUC_lexicon_tags_63 },
    { 0xe4c111bd, SUC_lexicon_tags_1 },
    { 0xbfc111bc, SUC_lexicon_tags_0 },
    { 0x176111bd, SUC_lexicon_tags_15 },
    { 0xa72111bd, SUC_lexicon_tags_25 },
    { 0xc0111bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x20111c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352111d1, SUC_lexicon_tags_19 },
    { 0x1a8111d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf84111d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52111e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa3a111e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26c111e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed8111e7, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80111ea, SUC_lexicon_tags_1 },
    { 0xa2e111ea, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xce0111ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabc111ef, SUC_lexicon_tags_55 },
    { 0x444111ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e0111f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2111f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x102111f7, SUC_lexicon_tags_33 },
    { 0x51e111f8, SUC_lexicon_tags_45 },
    { 0xa50111f9, SUC_lexicon_tags_12 },
    { 0x218111fa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb88111fc, SUC_lexicon_tags_15 },
    { 0xcae111fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32811200, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f011203, SUC_lexicon_tags_2 },
    { 0x6c11204, SUC_lexicon_tags_56 },
    { 0x61411204, SUC_lexicon_tags_1 },
    { 0xafc11204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdea11208, SUC_lexicon_tags_49 },
    { 0x6f011208, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x4321120c, SUC_lexicon_tags_12 },
    { 0x97a1120d, SUC_lexicon_tags_11 },
    { 0x701120d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x46811211, SUC_lexicon_tags_39 },
    { 0x7ca11212, SUC_lexicon_tags_30 },
    { 0x4a11212, SUC_lexicon_tags_8 },
    { 0x5dc11214, SUC_lexicon_tags_8 },
    { 0xd2011215, SUC_lexicon_tags_33 },
    { 0xec611216, SUC_lexicon_tags_25 },
    { 0xc8c11213, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xab41121a, SUC_lexicon_tags_19 },
    { 0x2521121b, SUC_lexicon_tags_33 },
    { 0xfe81121c, SUC_lexicon_tags_23 },
    { 0xf281121d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24011220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0611225, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe1122c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc341122e, SUC_lexicon_tags_32 },
    { 0x8461122f, SUC_lexicon_tags_2 },
    { 0xe8a1122f, SUC_lexicon_tags_33 },
    { 0xcbc1122f, SUC_lexicon_tags_35 },
    { 0xffc11230, SUC_lexicon_tags_235 },
    { 0x92411233, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c1123c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea11243, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d811247, SUC_lexicon_tags_14 },
    { 0x94e11247, SUC_lexicon_tags_47 },
    { 0xce411247, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x761124b, SUC_lexicon_tags_482 },
    { 0xc61124b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce21124e, SUC_lexicon_tags_17 },
    { 0xed21124f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6411254, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd611258, SUC_lexicon_tags_43 },
    { 0x7e11259, SUC_lexicon_tags_7 },
    { 0x2f41125a, SUC_lexicon_tags_25 },
    { 0x3a211259, SUC_lexicon_tags_32 },
    { 0x8c21125c, SUC_lexicon_tags_11 },
    { 0x8c41125c, SUC_lexicon_tags_14 },
    { 0xe501125c, SUC_lexicon_tags_23 },
    { 0x3721125f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e11262, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfc11264, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c1126e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a211274, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba11277, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e1127b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51c1127d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78611282, SUC_lexicon_tags_11 },
    { 0x90811283, SUC_lexicon_tags_17 },
    { 0xb1211283, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e21128b, SUC_lexicon_tags_12 },
    { 0x3721128b, SUC_lexicon_tags_66 },
    { 0xa801128b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3401128f, SUC_lexicon_tags_45 },
    { 0x5d411290, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x51811293, SUC_lexicon_tags_0 },
    { 0x2fe11293, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e1129e, SUC_lexicon_tags_15 },
    { 0x64e1129f, SUC_lexicon_tags_25 },
    { 0x816112a0, SUC_lexicon_tags_12 },
    { 0x7ea1129f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80112a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4112a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94112ac, SUC_lexicon_tags_3 },
    { 0x6ea112ad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9a2112af, SUC_lexicon_tags_0 },
    { 0xa14112af, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdf8112b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c112b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2112bc, SUC_lexicon_tags_43 },
    { 0xf28112bd, SUC_lexicon_tags_26 },
    { 0xc32112be, SUC_lexicon_tags_11 },
    { 0xc46112bc, SUC_lexicon_tags_2 },
    { 0x9f2112c0, SUC_lexicon_tags_31 },
    { 0xcfc112bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6a112c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb0112c5, SUC_lexicon_tags_66 },
    { 0xbf4112c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e112c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa112cd, SUC_lexicon_tags_12 },
    { 0x95e112ce, SUC_lexicon_tags_11 },
    { 0x4112ce, SUC_lexicon_tags_0 },
    { 0xbb4112d0, SUC_lexicon_tags_40 },
    { 0xf64112d1, SUC_lexicon_tags_19 },
    { 0x304112cf, SUC_lexicon_tags_49 },
    { 0x176112d3, SUC_lexicon_tags_17 },
    { 0x772112d3, SUC_lexicon_tags_4 },
    { 0x26112d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a112d9, SUC_lexicon_tags_2 },
    { 0xf24112d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x722112dc, SUC_lexicon_tags_27 },
    { 0xa34112dd, SUC_lexicon_tags_17 },
    { 0x798112de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8ba112e0, SUC_lexicon_tags_47 },
    { 0x10a112e1, SUC_lexicon_tags_0 },
    { 0x51a112e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54a112e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x20112e7, SUC_lexicon_tags_13 },
    { 0x7de112e8, SUC_lexicon_tags_56 },
    { 0x4ea112e8, SUC_lexicon_tags_8 },
    { 0xb10112ea, SUC_lexicon_tags_45 },
    { 0xcc112ea, SUC_lexicon_tags_21 },
    { 0x402112eb, SUC_lexicon_tags_9 },
    { 0x35e112ed, SUC_lexicon_tags_4 },
    { 0x6d0112ee, SUC_lexicon_tags_11 },
    { 0x940112ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892112f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa112f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8112fc, SUC_lexicon_tags_40 },
    { 0x8d6112fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe9e112ff, SUC_lexicon_tags_47 },
    { 0x8f611300, SUC_lexicon_tags_25 },
    { 0x7b011300, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa5211303, SUC_lexicon_tags_12 },
    { 0xa6411304, SUC_lexicon_tags_16 },
    { 0xfe611304, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e011308, SUC_lexicon_tags_17 },
    { 0x18a11308, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd61130b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf401130f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79611312, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c011317, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9481131f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a11324, SUC_lexicon_tags_14 },
    { 0xafc11325, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2fa11327, SUC_lexicon_tags_18 },
    { 0xad411328, SUC_lexicon_tags_2 },
    { 0x5c411329, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc1132c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0411330, SUC_lexicon_tags_85 },
    { 0x46611330, SUC_lexicon_tags_26 },
    { 0xb0211332, SUC_lexicon_tags_107 },
    { 0x1211333, SUC_lexicon_tags_11 },
    { 0xd7811334, SUC_lexicon_tags_25 },
    { 0xb5e11334, SUC_lexicon_tags_2 },
    { 0xb9011333, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6221133a, SUC_lexicon_tags_15 },
    { 0x8861133b, SUC_lexicon_tags_27 },
    { 0x3c01133c, SUC_lexicon_tags_3 },
    { 0xefc1133d, SUC_lexicon_tags_14 },
    { 0x9fa1133e, SUC_lexicon_tags_38 },
    { 0x34c1133e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf1011341, SUC_lexicon_tags_95 },
    { 0xec211341, SUC_lexicon_tags_42 },
    { 0x5aa11341, SUC_lexicon_tags_25 },
    { 0xedc11344, SUC_lexicon_tags_0 },
    { 0xee811344, SUC_lexicon_tags_1 },
    { 0x64011346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc1134b, SUC_lexicon_tags_13 },
    { 0x89e1134c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3811352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5811355, SUC_lexicon_tags_4 },
    { 0x8a11355, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x55811358, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa11362, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c11365, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc7811367, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x62611369, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13a1136b, SUC_lexicon_tags_108 },
    { 0xf441136b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51611370, SUC_lexicon_tags_15 },
    { 0x67e11370, SUC_lexicon_tags_37 },
    { 0x91611371, SUC_lexicon_tags_60 },
    { 0x77611370, SUC_lexicon_tags_66 },
    { 0x8c411374, SUC_lexicon_tags_14 },
    { 0x97011375, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c1137b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc11383, SUC_lexicon_tags_11 },
    { 0x1e611383, SUC_lexicon_tags_28 },
    { 0xd6411385, SUC_lexicon_tags_7 },
    { 0x35211385, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea611388, SUC_lexicon_tags_6 },
    { 0x60411389, SUC_lexicon_tags_33 },
    { 0xaea11389, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf81138c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81611393, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411398, SUC_lexicon_tags_33 },
    { 0x2a611398, SUC_lexicon_tags_87 },
    { 0xbc011399, SUC_lexicon_tags_1 },
    { 0x47411398, SUC_lexicon_tags_63 },
    { 0x261139c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7121139f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12113a4, SUC_lexicon_tags_7 },
    { 0x850113a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a113a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c113ad, SUC_lexicon_tags_32 },
    { 0xbe2113ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x542113b0, SUC_lexicon_tags_1 },
    { 0xee6113b1, SUC_lexicon_tags_3 },
    { 0xd74113b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32113b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98113b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904113bc, SUC_lexicon_tags_15 },
    { 0xcd6113bc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e113c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214113c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x748113c6, SUC_lexicon_tags_37 },
    { 0x7e113c6, SUC_lexicon_tags_37 },
    { 0x48113c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b8113ca, SUC_lexicon_tags_25 },
    { 0x6be113ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee113ce, SUC_lexicon_tags_30 },
    { 0x21a113cf, SUC_lexicon_tags_1 },
    { 0xf56113ce, SUC_lexicon_tags_37 },
    { 0x61a113cf, SUC_lexicon_tags_8 },
    { 0x262113d2, SUC_lexicon_tags_29 },
    { 0x588113d3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588113da, SUC_lexicon_tags_2 },
    { 0x660113da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c113e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c113e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xace113e5, SUC_lexicon_tags_11 },
    { 0x46a113e6, SUC_lexicon_tags_45 },
    { 0x940113e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba113ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516113f1, SUC_lexicon_tags_17 },
    { 0xdf6113f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4113f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc11403, SUC_lexicon_tags_60 },
    { 0x84011404, SUC_lexicon_tags_25 },
    { 0x19e11404, SUC_lexicon_tags_82 },
    { 0x6dc11406, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e41140b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf6c1140d, SUC_lexicon_tags_25 },
    { 0xaae1140e, SUC_lexicon_tags_49 },
    { 0xf7a1140f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de11412, SUC_lexicon_tags_68 },
    { 0x41611412, SUC_lexicon_tags_38 },
    { 0x16c11413, SUC_lexicon_tags_2 },
    { 0xb6211412, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6f411417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa781141a, SUC_lexicon_tags_9 },
    { 0x8b01141b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b01141e, SUC_lexicon_tags_12 },
    { 0x1ae1141e, SUC_lexicon_tags_37 },
    { 0xd1811420, SUC_lexicon_tags_47 },
    { 0xad411421, SUC_lexicon_tags_60 },
    { 0x2f411421, SUC_lexicon_tags_15 },
    { 0xb9c11423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab611426, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdec11428, SUC_lexicon_tags_47 },
    { 0x9ca11429, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ce1142b, SUC_lexicon_tags_65 },
    { 0x5d81142c, SUC_lexicon_tags_19 },
    { 0x81c1142d, SUC_lexicon_tags_37 },
    { 0x9d21142e, SUC_lexicon_tags_48 },
    { 0xf1c1142c, SUC_lexicon_tags_1 },
    { 0x4411430, SUC_lexicon_tags_2 },
    { 0x6be11431, SUC_lexicon_tags_63 },
    { 0x44611432, SUC_lexicon_tags_17 },
    { 0x5da11432, SUC_lexicon_tags_37 },
    { 0xcd011434, SUC_lexicon_tags_4 },
    { 0xa3e11434, SUC_lexicon_tags_39 },
    { 0x12e11435, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x85a11438, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9221143a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43e1143c, SUC_lexicon_tags_12 },
    { 0x22c1143c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc11440, SUC_lexicon_tags_33 },
    { 0xf8411441, SUC_lexicon_tags_1006 },
    { 0xc6011442, SUC_lexicon_tags_77 },
    { 0x45c11443, SUC_lexicon_tags_2 },
    { 0xa8811444, SUC_lexicon_tags_7 },
    { 0x89a11443, SUC_lexicon_tags_2 },
    { 0x6211443, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74611449, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4bc1144b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95011452, SUC_lexicon_tags_47 },
    { 0x68611453, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x22211455, SUC_lexicon_tags_0 },
    { 0xaf611456, SUC_lexicon_tags_7 },
    { 0xdb011457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1145a, SUC_lexicon_tags_11 },
    { 0x2c41145b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1581145d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f611460, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf2c11462, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf211466, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1146a, SUC_lexicon_tags_27 },
    { 0x55c1146b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c01146d, SUC_lexicon_tags_17 },
    { 0x43a1146e, SUC_lexicon_tags_60 },
    { 0xd441146e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56611471, SUC_lexicon_tags_47 },
    { 0xd6611472, SUC_lexicon_tags_65 },
    { 0x23011473, SUC_lexicon_tags_10 },
    { 0xf8411474, SUC_lexicon_tags_27 },
    { 0x34011475, SUC_lexicon_tags_28 },
    { 0x8811472, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e411479, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3261147e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46011482, SUC_lexicon_tags_2 },
    { 0x65411482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b811488, SUC_lexicon_tags_65 },
    { 0x21a11488, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa981148e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x45011490, SUC_lexicon_tags_14 },
    { 0x47a11490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73211493, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d81149e, SUC_lexicon_tags_6 },
    { 0x49a1149e, SUC_lexicon_tags_38 },
    { 0xc1149e, SUC_lexicon_tags_10 },
    { 0x9fc1149e, SUC_lexicon_tags_14 },
    { 0x59c1149e, SUC_lexicon_tags_37 },
    { 0xb12114a1, SUC_lexicon_tags_7 },
    { 0xc72114a3, SUC_lexicon_tags_32 },
    { 0x348114a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33e114a7, SUC_lexicon_tags_65 },
    { 0x382114a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x828114aa, SUC_lexicon_tags_21 },
    { 0x17a114aa, SUC_lexicon_tags_61 },
    { 0xe74114ac, SUC_lexicon_tags_17 },
    { 0x812114ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x744114af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c2114b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a114bc, SUC_lexicon_tags_396 },
    { 0x916114bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x856114bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e114c5, SUC_lexicon_tags_10 },
    { 0x90c114c6, SUC_lexicon_tags_17 },
    { 0x41e114c7, SUC_lexicon_tags_17 },
    { 0xb5e114c8, SUC_lexicon_tags_46 },
    { 0x160114c6, SUC_lexicon_tags_7 },
    { 0xd86114c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x978114cc, SUC_lexicon_tags_40 },
    { 0x3f2114cd, SUC_lexicon_tags_31 },
    { 0x238114cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966114d2, SUC_lexicon_tags_101 },
    { 0x3e0114d2, SUC_lexicon_tags_7 },
    { 0x774114d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x462114d6, SUC_lexicon_tags_17 },
    { 0x522114d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22114df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc98114e1, SUC_lexicon_tags_19 },
    { 0x6e6114e2, SUC_lexicon_tags_12 },
    { 0x402114e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0114e6, SUC_lexicon_tags_37 },
    { 0x2bc114e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0114ec, SUC_lexicon_tags_11 },
    { 0x46a114ed, SUC_lexicon_tags_12 },
    { 0x472114ec, SUC_lexicon_tags_37 },
    { 0xf9c114ef, SUC_lexicon_tags_12 },
    { 0x428114f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50114f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x276114f5, SUC_lexicon_tags_25 },
    { 0x12c114f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4114f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3c114fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2114fe, SUC_lexicon_tags_19 },
    { 0x80c114fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4561150a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b01150e, SUC_lexicon_tags_7 },
    { 0xbe61150f, SUC_lexicon_tags_11 },
    { 0xdf41150e, SUC_lexicon_tags_1 },
    { 0xd6811511, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c1151a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf521151c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1411524, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e011529, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94a1152b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3011533, SUC_lexicon_tags_17 },
    { 0x77211534, SUC_lexicon_tags_6 },
    { 0x5a811534, SUC_lexicon_tags_30 },
    { 0x1ea11533, SUC_lexicon_tags_16 },
    { 0xc2e11536, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff211539, SUC_lexicon_tags_37 },
    { 0xf401153a, SUC_lexicon_tags_15 },
    { 0xf7c1153b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9811540, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x38c11542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa481154a, SUC_lexicon_tags_17 },
    { 0x2f81154b, SUC_lexicon_tags_14 },
    { 0xbe61154c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c1154f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f611551, SUC_lexicon_tags_11 },
    { 0xdc211552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e011556, SUC_lexicon_tags_17 },
    { 0x45011557, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcdc11559, SUC_lexicon_tags_63 },
    { 0xe281155a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50011560, SUC_lexicon_tags_2 },
    { 0x71411561, SUC_lexicon_tags_116 },
    { 0x14611561, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x42411564, SUC_lexicon_tags_17 },
    { 0x41011565, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x36611568, SUC_lexicon_tags_17 },
    { 0xa2a11569, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c01156e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc811572, SUC_lexicon_tags_1152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc11577, SUC_lexicon_tags_37 },
    { 0x38811578, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1101157c, SUC_lexicon_tags_14 },
    { 0x33c1157d, SUC_lexicon_tags_201 },
    { 0x6aa1157d, SUC_lexicon_tags_15 },
    { 0xa6c1157e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x47011582, SUC_lexicon_tags_37 },
    { 0x5ce11583, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e11588, SUC_lexicon_tags_7 },
    { 0x84811589, SUC_lexicon_tags_28 },
    { 0x8da1158a, SUC_lexicon_tags_16 },
    { 0xe9a1158a, SUC_lexicon_tags_25 },
    { 0x86a1158a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a011593, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x75811595, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a1159e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82115a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a115a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e115a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8115ae, SUC_lexicon_tags_10 },
    { 0xdc2115ae, SUC_lexicon_tags_70 },
    { 0x534115b0, SUC_lexicon_tags_17 },
    { 0xc6e115ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426115b5, SUC_lexicon_tags_12 },
    { 0xe0a115b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x414115b8, SUC_lexicon_tags_9 },
    { 0x37a115b9, SUC_lexicon_tags_17 },
    { 0x91e115ba, SUC_lexicon_tags_27 },
    { 0xe94115ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28115c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x74e115c6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6115ca, SUC_lexicon_tags_65 },
    { 0x6b0115cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30115cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb36115cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd48115d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe115d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00115da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4115e0, SUC_lexicon_tags_11 },
    { 0xd3e115e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x124115e3, SUC_lexicon_tags_4 },
    { 0x320115e4, SUC_lexicon_tags_2 },
    { 0xc02115e4, SUC_lexicon_tags_1 },
    { 0x896115e5, SUC_lexicon_tags_16 },
    { 0x486115e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558115ec, SUC_lexicon_tags_2 },
    { 0x9c4115ed, SUC_lexicon_tags_23 },
    { 0x492115ee, SUC_lexicon_tags_118 },
    { 0x388115ed, SUC_lexicon_tags_37 },
    { 0xc6c115f0, SUC_lexicon_tags_17 },
    { 0x9e0115f1, SUC_lexicon_tags_23 },
    { 0x8ba115f2, SUC_lexicon_tags_19 },
    { 0x20a115f3, SUC_lexicon_tags_12 },
    { 0xac8115f4, SUC_lexicon_tags_14 },
    { 0x940115f1, SUC_lexicon_tags_23 },
    { 0x1a115f3, SUC_lexicon_tags_4 },
    { 0xe30115f7, SUC_lexicon_tags_27 },
    { 0x31e115f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3aa115fa, SUC_lexicon_tags_11 },
    { 0x7d0115fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78115ff, SUC_lexicon_tags_55 },
    { 0xec011600, SUC_lexicon_tags_63 },
    { 0xc2811601, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2c11603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xede11605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe821160b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3611610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a11613, SUC_lexicon_tags_37 },
    { 0x99811614, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61011618, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7c01161a, SUC_lexicon_tags_9 },
    { 0x1201161a, SUC_lexicon_tags_1 },
    { 0x72c1161c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7181161e, SUC_lexicon_tags_0 },
    { 0xd2c1161e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffc11621, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x3a411623, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7611628, SUC_lexicon_tags_70 },
    { 0x51811629, SUC_lexicon_tags_37 },
    { 0xa0811628, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e11631, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4211636, SUC_lexicon_tags_26 },
    { 0x92411636, SUC_lexicon_tags_4 },
    { 0x73611638, SUC_lexicon_tags_12 },
    { 0xa2c11639, SUC_lexicon_tags_4 },
    { 0x2c41163a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8461163c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2721163e, SUC_lexicon_tags_7 },
    { 0x9421163f, SUC_lexicon_tags_7 },
    { 0xe3211640, SUC_lexicon_tags_11 },
    { 0x68611641, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34211648, SUC_lexicon_tags_15 },
    { 0x4b011649, SUC_lexicon_tags_11 },
    { 0xe5a11649, SUC_lexicon_tags_79 },
    { 0x1a41164b, SUC_lexicon_tags_25 },
    { 0xee41164b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4261164e, SUC_lexicon_tags_47 },
    { 0x1821164f, SUC_lexicon_tags_15 },
    { 0xfe611650, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8211653, SUC_lexicon_tags_31 },
    { 0x48011653, SUC_lexicon_tags_25 },
    { 0xcf011654, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd581165a, SUC_lexicon_tags_65 },
    { 0xf8c1165b, SUC_lexicon_tags_27 },
    { 0xe881165a, SUC_lexicon_tags_7 },
    { 0xffa1165d, SUC_lexicon_tags_7 },
    { 0xb281165a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44011663, SUC_lexicon_tags_19 },
    { 0x58211664, SUC_lexicon_tags_27 },
    { 0xe7e11664, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e11671, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e11674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2fe11676, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d21167a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b21167d, SUC_lexicon_tags_23 },
    { 0xf4c1167e, SUC_lexicon_tags_7 },
    { 0xf921167d, SUC_lexicon_tags_38 },
    { 0x4f011680, SUC_lexicon_tags_15 },
    { 0x93c11681, SUC_lexicon_tags_47 },
    { 0x49c1167d, SUC_lexicon_tags_34 },
    { 0x58611683, SUC_lexicon_tags_2 },
    { 0x44411681, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2521168d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7e1168f, SUC_lexicon_tags_49 },
    { 0x25c1168f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34c11692, SUC_lexicon_tags_2 },
    { 0x55211693, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d411696, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a1169b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8141169e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2116a3, SUC_lexicon_tags_2 },
    { 0x5d8116a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc10116a6, SUC_lexicon_tags_54 },
    { 0xf0c116a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8116ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0116b3, SUC_lexicon_tags_37 },
    { 0xf34116b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a116b9, SUC_lexicon_tags_7 },
    { 0xebc116ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e116be, SUC_lexicon_tags_15 },
    { 0x7d8116bf, SUC_lexicon_tags_37 },
    { 0x9fc116bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28116c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e116c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57a116c8, SUC_lexicon_tags_127 },
    { 0x99e116c9, SUC_lexicon_tags_12 },
    { 0xe76116c9, SUC_lexicon_tags_9 },
    { 0xc6a116cb, SUC_lexicon_tags_46 },
    { 0x1a8116cb, SUC_lexicon_tags_37 },
    { 0xc3c116c9, SUC_lexicon_tags_26 },
    { 0xd90116ce, SUC_lexicon_tags_37 },
    { 0xf36116ce, SUC_lexicon_tags_63 },
    { 0x144116ce, SUC_lexicon_tags_17 },
    { 0x174116d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c116d9, SUC_lexicon_tags_22 },
    { 0xfac116d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x210116dc, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2116e0, SUC_lexicon_tags_37 },
    { 0x36116e1, SUC_lexicon_tags_15 },
    { 0xb8c116e0, SUC_lexicon_tags_48 },
    { 0xc9a116e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590116e7, SUC_lexicon_tags_11 },
    { 0xe6c116e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184116ec, SUC_lexicon_tags_1 },
    { 0x586116ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x84a116ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a116f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d0116f5, SUC_lexicon_tags_47 },
    { 0x850116f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54116fa, SUC_lexicon_tags_0 },
    { 0xf88116fb, SUC_lexicon_tags_27 },
    { 0x6c8116fb, SUC_lexicon_tags_65 },
    { 0x9e116fd, SUC_lexicon_tags_84 },
    { 0xe88116fa, SUC_lexicon_tags_25 },
    { 0x7a8116ff, SUC_lexicon_tags_45 },
    { 0xc6116fa, SUC_lexicon_tags_15 },
    { 0x44411701, SUC_lexicon_tags_38 },
    { 0xa1e116fa, SUC_lexicon_tags_46 },
    { 0x24e11701, SUC_lexicon_tags_11 },
    { 0xe6116ff, SUC_lexicon_tags_25 },
    { 0x88a116ff, SUC_lexicon_tags_40 },
    { 0x74011706, SUC_lexicon_tags_45 },
    { 0x2ba11706, SUC_lexicon_tags_25 },
    { 0xcb411708, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4061170a, SUC_lexicon_tags_12 },
    { 0xb3c1170b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2781170d, SUC_lexicon_tags_25 },
    { 0x5ca1170e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee11713, SUC_lexicon_tags_12 },
    { 0xd8211714, SUC_lexicon_tags_31 },
    { 0xaba11715, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x49811717, SUC_lexicon_tags_11 },
    { 0x6cc11718, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c21171b, SUC_lexicon_tags_17 },
    { 0x6421171c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e11726, SUC_lexicon_tags_23 },
    { 0xcc11726, SUC_lexicon_tags_11 },
    { 0xe0411728, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf561172b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf941172d, SUC_lexicon_tags_11 },
    { 0xef61172d, SUC_lexicon_tags_47 },
    { 0xdac1172d, SUC_lexicon_tags_12 },
    { 0xb0a11730, SUC_lexicon_tags_31 },
    { 0x2081172d, SUC_lexicon_tags_1652 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3611734, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda411736, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x13c11738, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x62a1173a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a81173d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5081173f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e11744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d211747, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd01174e, SUC_lexicon_tags_2 },
    { 0x47e1174f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9811752, SUC_lexicon_tags_63 },
    { 0xb8611752, SUC_lexicon_tags_15 },
    { 0x8b611753, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6861175e, SUC_lexicon_tags_3 },
    { 0x4d61175e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e11763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a11766, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29211768, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff21176c, SUC_lexicon_tags_17 },
    { 0xdc61176c, SUC_lexicon_tags_37 },
    { 0xd561176c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x47e11770, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa11774, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6211779, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4a1177b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9441177e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e211780, SUC_lexicon_tags_11 },
    { 0x45411780, SUC_lexicon_tags_6 },
    { 0x51411782, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3011786, SUC_lexicon_tags_11 },
    { 0x5ee11787, SUC_lexicon_tags_6 },
    { 0xb2811787, SUC_lexicon_tags_11 },
    { 0x14411787, SUC_lexicon_tags_5 },
    { 0x6b611789, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11211791, SUC_lexicon_tags_6 },
    { 0xe9811792, SUC_lexicon_tags_29 },
    { 0xce11793, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57811797, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd811799, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a1179c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc481179e, SUC_lexicon_tags_12 },
    { 0x70c1179f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x76e117a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x918117a3, SUC_lexicon_tags_25 },
    { 0x24c117a3, SUC_lexicon_tags_2 },
    { 0xc0117a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6117a8, SUC_lexicon_tags_43 },
    { 0x37c117a9, SUC_lexicon_tags_28 },
    { 0x782117aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x824117ac, SUC_lexicon_tags_53 },
    { 0xa72117ad, SUC_lexicon_tags_61 },
    { 0xc50117ae, SUC_lexicon_tags_65 },
    { 0xa8c117ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae117b2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb54117b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e117b7, SUC_lexicon_tags_23 },
    { 0x9e2117b8, SUC_lexicon_tags_65 },
    { 0x3b0117b9, SUC_lexicon_tags_12 },
    { 0xfd0117ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2117bd, SUC_lexicon_tags_1 },
    { 0x3d8117be, SUC_lexicon_tags_19 },
    { 0x63c117bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d0117c1, SUC_lexicon_tags_25 },
    { 0x17c117c2, SUC_lexicon_tags_65 },
    { 0xa117c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940117ca, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2c4117cc, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc117d0, SUC_lexicon_tags_4 },
    { 0xe0a117d0, SUC_lexicon_tags_0 },
    { 0x31e117d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x982117d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a2117d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4d8117d9, SUC_lexicon_tags_29 },
    { 0xd7e117da, SUC_lexicon_tags_17 },
    { 0xf6117d9, SUC_lexicon_tags_38 },
    { 0x166117da, SUC_lexicon_tags_7 },
    { 0x778117dd, SUC_lexicon_tags_1 },
    { 0xbae117de, SUC_lexicon_tags_25 },
    { 0x1d6117df, SUC_lexicon_tags_14 },
    { 0x674117df, SUC_lexicon_tags_37 },
    { 0x4ca117e1, SUC_lexicon_tags_17 },
    { 0xfba117df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e8117e4, SUC_lexicon_tags_47 },
    { 0x416117e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa117ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be117ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88e117ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c117f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88117fe, SUC_lexicon_tags_37 },
    { 0xbb2117fe, SUC_lexicon_tags_17 },
    { 0x84a11800, SUC_lexicon_tags_15 },
    { 0x75811801, SUC_lexicon_tags_30 },
    { 0x57211800, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42211804, SUC_lexicon_tags_8 },
    { 0xe5011805, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b41180a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b41180d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63811816, SUC_lexicon_tags_6 },
    { 0xe3011817, SUC_lexicon_tags_15 },
    { 0xb0011818, SUC_lexicon_tags_11 },
    { 0xf2411819, SUC_lexicon_tags_12 },
    { 0x1ba11819, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5fa1181c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1781181e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a411820, SUC_lexicon_tags_6 },
    { 0x76611821, SUC_lexicon_tags_11 },
    { 0x9fe11822, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf7611824, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb6411826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95611828, SUC_lexicon_tags_11 },
    { 0x2cc11829, SUC_lexicon_tags_39 },
    { 0x10411829, SUC_lexicon_tags_11 },
    { 0x1941182b, SUC_lexicon_tags_25 },
    { 0x1b41182b, SUC_lexicon_tags_30 },
    { 0xa4a1182d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x811830, SUC_lexicon_tags_11 },
    { 0x69c11831, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb4e11833, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a611837, SUC_lexicon_tags_33 },
    { 0x82011838, SUC_lexicon_tags_317 },
    { 0xcce11839, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a11846, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe411848, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca1184b, SUC_lexicon_tags_33 },
    { 0x1ba1184c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a811850, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe411855, SUC_lexicon_tags_25 },
    { 0x52411856, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5321185a, SUC_lexicon_tags_0 },
    { 0x6061185b, SUC_lexicon_tags_1 },
    { 0x37c1185a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed211861, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a411865, SUC_lexicon_tags_37 },
    { 0x5fc11866, SUC_lexicon_tags_14 },
    { 0x5ee11867, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58011869, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10a1186b, SUC_lexicon_tags_2 },
    { 0x3a1186b, SUC_lexicon_tags_26 },
    { 0x3701186d, SUC_lexicon_tags_37 },
    { 0x17a1186e, SUC_lexicon_tags_33 },
    { 0x3a21186d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44411873, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67611878, SUC_lexicon_tags_27 },
    { 0x60e11879, SUC_lexicon_tags_12 },
    { 0x5381187a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa321187d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc211880, SUC_lexicon_tags_16 },
    { 0x1b011881, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc211885, SUC_lexicon_tags_25 },
    { 0xbc011886, SUC_lexicon_tags_6 },
    { 0xe6e11886, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x25211889, SUC_lexicon_tags_6 },
    { 0x4ea1188a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a11893, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40611896, SUC_lexicon_tags_33 },
    { 0x18a11896, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a1189a, SUC_lexicon_tags_7 },
    { 0x5c01189b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae118a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8118a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c118aa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb46118ac, SUC_lexicon_tags_62 },
    { 0xe44118ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492118b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x472118b5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674118ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c118c4, SUC_lexicon_tags_0 },
    { 0xac6118c5, SUC_lexicon_tags_32 },
    { 0x4b8118c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb36118c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9de118ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x942118cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4118d8, SUC_lexicon_tags_2 },
    { 0xe90118d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c118dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e118df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e118e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340118ea, SUC_lexicon_tags_37 },
    { 0x7b2118eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc118ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e118f2, SUC_lexicon_tags_61 },
    { 0x840118f2, SUC_lexicon_tags_167 },
    { 0xa72118f4, SUC_lexicon_tags_25 },
    { 0x89c118f4, SUC_lexicon_tags_46 },
    { 0x97a118f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32118f9, SUC_lexicon_tags_11 },
    { 0xef4118fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0611904, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84211907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8701190a, SUC_lexicon_tags_60 },
    { 0x1190b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7901190d, SUC_lexicon_tags_11 },
    { 0x7501190e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca11914, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee611916, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e1191d, SUC_lexicon_tags_4 },
    { 0x3881191d, SUC_lexicon_tags_172 },
    { 0x1321191e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf011921, SUC_lexicon_tags_37 },
    { 0x34411922, SUC_lexicon_tags_49 },
    { 0x77611923, SUC_lexicon_tags_1 },
    { 0xc4811924, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca11927, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a1192b, SUC_lexicon_tags_37 },
    { 0x6841192b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf661192f, SUC_lexicon_tags_108 },
    { 0x56c11930, SUC_lexicon_tags_25 },
    { 0x9b011930, SUC_lexicon_tags_26 },
    { 0xf1411932, SUC_lexicon_tags_19 },
    { 0x8ae11930, SUC_lexicon_tags_7 },
    { 0x7dc11933, SUC_lexicon_tags_39 },
    { 0xf2411935, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1041193b, SUC_lexicon_tags_21 },
    { 0xa861193c, SUC_lexicon_tags_12 },
    { 0xd521193c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18c1193f, SUC_lexicon_tags_29 },
    { 0xa7e1193f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd1a11942, SUC_lexicon_tags_1 },
    { 0x2b811943, SUC_lexicon_tags_31 },
    { 0x71e11943, SUC_lexicon_tags_194 },
    { 0xf1c11944, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x22411948, SUC_lexicon_tags_14 },
    { 0xc5a11948, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e1194c, SUC_lexicon_tags_28 },
    { 0xa4c1194d, SUC_lexicon_tags_74 },
    { 0x1241194e, SUC_lexicon_tags_11 },
    { 0xa801194f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb0811951, SUC_lexicon_tags_14 },
    { 0x28a11952, SUC_lexicon_tags_11 },
    { 0xd9411953, SUC_lexicon_tags_11 },
    { 0xe6611954, SUC_lexicon_tags_33 },
    { 0xbbc11953, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce1195a, SUC_lexicon_tags_49 },
    { 0xa561195b, SUC_lexicon_tags_12 },
    { 0x54a1195c, SUC_lexicon_tags_25 },
    { 0x4a1195b, SUC_lexicon_tags_15 },
    { 0xf2e1195c, SUC_lexicon_tags_11 },
    { 0xd5e1195a, SUC_lexicon_tags_30 },
    { 0x5c1195b, SUC_lexicon_tags_17 },
    { 0xc421195a, SUC_lexicon_tags_17 },
    { 0xe8011962, SUC_lexicon_tags_15 },
    { 0x8bc11963, SUC_lexicon_tags_15 },
    { 0xd8211964, SUC_lexicon_tags_19 },
    { 0xe4a11964, SUC_lexicon_tags_37 },
    { 0x1f011966, SUC_lexicon_tags_12 },
    { 0x2a11961, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24a11969, SUC_lexicon_tags_17 },
    { 0x16011969, SUC_lexicon_tags_29 },
    { 0x2a61196a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca1196e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61a11970, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a11973, SUC_lexicon_tags_14 },
    { 0x9ea11974, SUC_lexicon_tags_12 },
    { 0xade11973, SUC_lexicon_tags_45 },
    { 0x79c11973, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe1197b, SUC_lexicon_tags_2 },
    { 0xff61197b, SUC_lexicon_tags_2 },
    { 0xad21197b, SUC_lexicon_tags_2 },
    { 0x6541197d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71811980, SUC_lexicon_tags_37 },
    { 0xc9611980, SUC_lexicon_tags_37 },
    { 0x4011982, SUC_lexicon_tags_28 },
    { 0xab411983, SUC_lexicon_tags_2 },
    { 0xc4811983, SUC_lexicon_tags_1 },
    { 0xf2c11985, SUC_lexicon_tags_91 },
    { 0x81011986, SUC_lexicon_tags_10 },
    { 0xe0411987, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee1198a, SUC_lexicon_tags_0 },
    { 0x93c1198b, SUC_lexicon_tags_13 },
    { 0x41198c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc421198e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe1a11990, SUC_lexicon_tags_21 },
    { 0xe811990, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27211996, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba11999, SUC_lexicon_tags_15 },
    { 0xde61199a, SUC_lexicon_tags_43 },
    { 0xb1c1199b, SUC_lexicon_tags_37 },
    { 0x3761199c, SUC_lexicon_tags_37 },
    { 0x901199d, SUC_lexicon_tags_2 },
    { 0x7c01199c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80119a1, SUC_lexicon_tags_25 },
    { 0x5fc119a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56119a6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x596119a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c119ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990119b0, SUC_lexicon_tags_39 },
    { 0x8ae119b0, SUC_lexicon_tags_25 },
    { 0x81c119b2, SUC_lexicon_tags_10 },
    { 0x90a119b2, SUC_lexicon_tags_2 },
    { 0xeb6119b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec8119b6, SUC_lexicon_tags_6 },
    { 0x3f2119b6, SUC_lexicon_tags_1 },
    { 0x498119b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xde119bb, SUC_lexicon_tags_27 },
    { 0xd80119bc, SUC_lexicon_tags_12 },
    { 0x70a119bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4119c0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x684119c2, SUC_lexicon_tags_23 },
    { 0x6a119c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42a119c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x880119c8, SUC_lexicon_tags_23 },
    { 0x4da119c8, SUC_lexicon_tags_2 },
    { 0xc92119c9, SUC_lexicon_tags_104 },
    { 0x4bc119c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ba119cd, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x512119cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30119d3, SUC_lexicon_tags_63 },
    { 0x248119d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e119d7, SUC_lexicon_tags_1 },
    { 0x730119d8, SUC_lexicon_tags_15 },
    { 0xdd4119d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e119dd, SUC_lexicon_tags_17 },
    { 0xb7a119dd, SUC_lexicon_tags_15 },
    { 0x240119df, SUC_lexicon_tags_23 },
    { 0x24c119e0, SUC_lexicon_tags_17 },
    { 0xe20119e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0119e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x378119e9, SUC_lexicon_tags_21 },
    { 0xdee119ea, SUC_lexicon_tags_60 },
    { 0x9e8119ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8de119ed, SUC_lexicon_tags_37 },
    { 0x666119ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40a119f0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdd6119f2, SUC_lexicon_tags_5 },
    { 0xae2119f3, SUC_lexicon_tags_47 },
    { 0xdf2119f4, SUC_lexicon_tags_151 },
    { 0x26119f4, SUC_lexicon_tags_15 },
    { 0x5b2119f2, SUC_lexicon_tags_8 },
    { 0x536119f7, SUC_lexicon_tags_17 },
    { 0x846119f2, SUC_lexicon_tags_2 },
    { 0x40c119f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9b6119fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94119ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8011a04, SUC_lexicon_tags_38 },
    { 0x31411a05, SUC_lexicon_tags_15 },
    { 0x97e11a05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12011a08, SUC_lexicon_tags_522 },
    { 0x41011a09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86411a0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39011a0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14211a12, SUC_lexicon_tags_27 },
    { 0x7b011a13, SUC_lexicon_tags_11 },
    { 0x48011a14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a11a18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82611a1a, SUC_lexicon_tags_29 },
    { 0x8dc11a1b, SUC_lexicon_tags_25 },
    { 0x76811a1b, SUC_lexicon_tags_1 },
    { 0x37411a1c, SUC_lexicon_tags_65 },
    { 0x611a1e, SUC_lexicon_tags_25 },
    { 0x10a11a1e, SUC_lexicon_tags_11 },
    { 0x8bc11a20, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c11a24, SUC_lexicon_tags_11 },
    { 0xd8811a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba811a2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xace11a30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f411a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63211a34, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8011a3b, SUC_lexicon_tags_46 },
    { 0x7b211a3b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f811a43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb811a46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c411a4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fe11a4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa11a51, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae11a56, SUC_lexicon_tags_39 },
    { 0xc8811a56, SUC_lexicon_tags_11 },
    { 0x9411a56, SUC_lexicon_tags_15 },
    { 0x23811a59, SUC_lexicon_tags_7 },
    { 0x4fa11a56, SUC_lexicon_tags_35 },
    { 0xf4a11a5b, SUC_lexicon_tags_60 },
    { 0x49a11a5b, SUC_lexicon_tags_7 },
    { 0xe4011a5d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3a611a5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e611a66, SUC_lexicon_tags_6 },
    { 0x9d211a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf811a69, SUC_lexicon_tags_12 },
    { 0x7d411a6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee11a6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c11a71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41211a75, SUC_lexicon_tags_8 },
    { 0x19411a75, SUC_lexicon_tags_11 },
    { 0x9ce11a76, SUC_lexicon_tags_12 },
    { 0x33811a78, SUC_lexicon_tags_33 },
    { 0x11211a79, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65011a7d, SUC_lexicon_tags_87 },
    { 0x87011a7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43611a85, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf811a8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2811a92, SUC_lexicon_tags_1 },
    { 0xc6011a93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33611a96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e411a98, SUC_lexicon_tags_37 },
    { 0x7c811a99, SUC_lexicon_tags_2 },
    { 0x9e211a99, SUC_lexicon_tags_15 },
    { 0xc9e11a9b, SUC_lexicon_tags_38 },
    { 0x98011a9a, SUC_lexicon_tags_14 },
    { 0xf7411a9d, SUC_lexicon_tags_21 },
    { 0x7c211a9e, SUC_lexicon_tags_14 },
    { 0x19411a9e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa11aa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda811aa8, SUC_lexicon_tags_17 },
    { 0x25c11aa9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4611aac, SUC_lexicon_tags_37 },
    { 0xa7411aad, SUC_lexicon_tags_25 },
    { 0xbc611aae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0811ab4, SUC_lexicon_tags_17 },
    { 0x71611ab4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc2811ab7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7611abe, SUC_lexicon_tags_38 },
    { 0x82a11abf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0a11ac1, SUC_lexicon_tags_82 },
    { 0xfa11ac2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a11ac7, SUC_lexicon_tags_17 },
    { 0x70c11ac8, SUC_lexicon_tags_40 },
    { 0x58411ac7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a411acf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8011ad9, SUC_lexicon_tags_2 },
    { 0x7aa11ada, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75a11adc, SUC_lexicon_tags_11 },
    { 0x99211add, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a11ae0, SUC_lexicon_tags_52 },
    { 0xdc11ae1, SUC_lexicon_tags_48 },
    { 0x3f411ae0, SUC_lexicon_tags_6 },
    { 0xdc611ae2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb611ae6, SUC_lexicon_tags_47 },
    { 0xa0611ae7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x32611aea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84811aef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbda11af1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc11af4, SUC_lexicon_tags_31 },
    { 0xaf811af5, SUC_lexicon_tags_8 },
    { 0xe2411af6, SUC_lexicon_tags_92 },
    { 0xff411af6, SUC_lexicon_tags_11 },
    { 0x39c11af5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x38211afa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92611aff, SUC_lexicon_tags_14 },
    { 0x40211b00, SUC_lexicon_tags_7 },
    { 0x43a11b01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96011b04, SUC_lexicon_tags_7 },
    { 0x8411b05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe11b09, SUC_lexicon_tags_11 },
    { 0xa7011b0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c11b15, SUC_lexicon_tags_107 },
    { 0x82e11b16, SUC_lexicon_tags_1 },
    { 0xf611b17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59811b1b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xef211b1d, SUC_lexicon_tags_27 },
    { 0xa6211b1e, SUC_lexicon_tags_15 },
    { 0x14011b1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc11b23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4fe11b25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14411b2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce11b2e, SUC_lexicon_tags_11 },
    { 0xa5811b2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a11b35, SUC_lexicon_tags_60 },
    { 0x92011b36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c011b38, SUC_lexicon_tags_56 },
    { 0x9ba11b39, SUC_lexicon_tags_14 },
    { 0x9c011b38, SUC_lexicon_tags_9 },
    { 0x18e11b3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa11b3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9011b40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae11b45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce811b47, SUC_lexicon_tags_49 },
    { 0x8811b48, SUC_lexicon_tags_9 },
    { 0x5cc11b48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64611b4b, SUC_lexicon_tags_17 },
    { 0x41611b4c, SUC_lexicon_tags_87 },
    { 0xf3211b4c, SUC_lexicon_tags_39 },
    { 0xe1611b4e, SUC_lexicon_tags_2 },
    { 0x3c011b4f, SUC_lexicon_tags_27 },
    { 0x59811b4f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c11b55, SUC_lexicon_tags_47 },
    { 0x75211b55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c11b59, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8811b63, SUC_lexicon_tags_61 },
    { 0x5b811b64, SUC_lexicon_tags_0 },
    { 0xed611b63, SUC_lexicon_tags_1 },
    { 0xf4211b66, SUC_lexicon_tags_16 },
    { 0x33211b66, SUC_lexicon_tags_30 },
    { 0x18211b65, SUC_lexicon_tags_2 },
    { 0xad211b65, SUC_lexicon_tags_9 },
    { 0xeba11b6a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd1e11b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb811b6f, SUC_lexicon_tags_65 },
    { 0xa0411b70, SUC_lexicon_tags_48 },
    { 0xcde11b71, SUC_lexicon_tags_12 },
    { 0x9611b71, SUC_lexicon_tags_11 },
    { 0xe5411b73, SUC_lexicon_tags_37 },
    { 0x39c11b73, SUC_lexicon_tags_7 },
    { 0xdf611b70, SUC_lexicon_tags_27 },
    { 0xd7611b76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ac11b78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c11b7d, SUC_lexicon_tags_12 },
    { 0x68811b7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13a11b80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e011b84, SUC_lexicon_tags_37 },
    { 0x86011b85, SUC_lexicon_tags_12 },
    { 0xe2a11b86, SUC_lexicon_tags_25 },
    { 0xf1c11b86, SUC_lexicon_tags_77 },
    { 0x94811b88, SUC_lexicon_tags_25 },
    { 0xa4e11b85, SUC_lexicon_tags_26 },
    { 0x9d411b84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1411b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a811b92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4611ba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xede11ba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29611ba5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6811ba9, SUC_lexicon_tags_33 },
    { 0x6a11baa, SUC_lexicon_tags_37 },
    { 0x6a811baa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21811bb1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d011bb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6411bbc, SUC_lexicon_tags_15 },
    { 0x7f611bbc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x20c11bbf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4e11bc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d811bc3, SUC_lexicon_tags_32 },
    { 0x65611bc4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x36611bc6, SUC_lexicon_tags_17 },
    { 0xeb011bc6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c11bca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d611bcd, SUC_lexicon_tags_1 },
    { 0x91a11bcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a11bd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe11bdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1c11bdd, SUC_lexicon_tags_63 },
    { 0x25c11bdd, SUC_lexicon_tags_33 },
    { 0x66e11bde, SUC_lexicon_tags_6 },
    { 0xbb411be0, SUC_lexicon_tags_104 },
    { 0x51411be1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99211be5, SUC_lexicon_tags_60 },
    { 0xe3411be6, SUC_lexicon_tags_66 },
    { 0x1ce11be6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e011bea, SUC_lexicon_tags_37 },
    { 0x1be11beb, SUC_lexicon_tags_12 },
    { 0xe2411beb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x34411bee, SUC_lexicon_tags_25 },
    { 0x5811bef, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x56611bf2, SUC_lexicon_tags_1 },
    { 0xef411bf2, SUC_lexicon_tags_1 },
    { 0xc1a11bf2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7411bf9, SUC_lexicon_tags_14 },
    { 0xc3011bfa, SUC_lexicon_tags_2 },
    { 0xa4411bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83211bfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1dc11bff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec11c02, SUC_lexicon_tags_15 },
    { 0xe1c11c03, SUC_lexicon_tags_17 },
    { 0x8ac11c04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce811c07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63211c0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a11c13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4411c19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d011c1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3811c1e, SUC_lexicon_tags_2 },
    { 0x86011c1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c611c24, SUC_lexicon_tags_23 },
    { 0xef811c25, SUC_lexicon_tags_49 },
    { 0x28011c24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec11c2b, SUC_lexicon_tags_14 },
    { 0x2211c2c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4dc11c2e, SUC_lexicon_tags_83 },
    { 0xfbe11c2f, SUC_lexicon_tags_1 },
    { 0x74211c30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8811c37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19211c39, SUC_lexicon_tags_27 },
    { 0x63c11c3a, SUC_lexicon_tags_30 },
    { 0x35a11c3b, SUC_lexicon_tags_1 },
    { 0x9ae11c3c, SUC_lexicon_tags_46 },
    { 0xa8c11c3d, SUC_lexicon_tags_47 },
    { 0x5611c3e, SUC_lexicon_tags_37 },
    { 0x77e11c3b, SUC_lexicon_tags_60 },
    { 0xae011c3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be11c46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd011c4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec11c4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab411c51, SUC_lexicon_tags_30 },
    { 0x88c11c52, SUC_lexicon_tags_65 },
    { 0x35411c52, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab011c57, SUC_lexicon_tags_45 },
    { 0x54411c58, SUC_lexicon_tags_12 },
    { 0xe6211c58, SUC_lexicon_tags_37 },
    { 0x59c11c5a, SUC_lexicon_tags_49 },
    { 0xe6211c5b, SUC_lexicon_tags_37 },
    { 0x25c11c5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x52011c5e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x90211c60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf211c66, SUC_lexicon_tags_8 },
    { 0x11411c66, SUC_lexicon_tags_25 },
    { 0x30e11c68, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x24811c6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc11c73, SUC_lexicon_tags_8 },
    { 0x53c11c73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88811c76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40611c7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d811c7f, SUC_lexicon_tags_25 },
    { 0x7a211c80, SUC_lexicon_tags_47 },
    { 0xada11c81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c11c87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a11c8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5811c8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x26611c90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec11c94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17411c98, SUC_lexicon_tags_17 },
    { 0xefa11c98, SUC_lexicon_tags_8 },
    { 0x4ec11c99, SUC_lexicon_tags_43 },
    { 0x99c11c9b, SUC_lexicon_tags_47 },
    { 0x11c11c99, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78e11c9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cc11ca0, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9211ca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3a11ca7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e011caa, SUC_lexicon_tags_17 },
    { 0x1be11cab, SUC_lexicon_tags_19 },
    { 0xae011caa, SUC_lexicon_tags_0 },
    { 0x7ce11cad, SUC_lexicon_tags_7 },
    { 0x7aa11cae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16411cb3, SUC_lexicon_tags_25 },
    { 0xea611cb4, SUC_lexicon_tags_33 },
    { 0xda411cb5, SUC_lexicon_tags_37 },
    { 0x5ca11cb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbea11cb8, SUC_lexicon_tags_61 },
    { 0x5ec11cb9, SUC_lexicon_tags_29 },
    { 0xd7e11cb8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a11cbd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce11cc3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x13811cc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a011cc7, SUC_lexicon_tags_37 },
    { 0x3ee11cc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26611cca, SUC_lexicon_tags_25 },
    { 0xfc611cca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c11cce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc3811cd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c11cd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a11cda, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x61611cdc, SUC_lexicon_tags_2 },
    { 0x3a811cdd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa8c11cdf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6da11ce1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xff611ce4, SUC_lexicon_tags_23 },
    { 0x7b011ce4, SUC_lexicon_tags_37 },
    { 0xb8811ce4, SUC_lexicon_tags_37 },
    { 0x74e11ce7, SUC_lexicon_tags_15 },
    { 0xe8e11ce8, SUC_lexicon_tags_17 },
    { 0x6211ce9, SUC_lexicon_tags_77 },
    { 0x5c011cea, SUC_lexicon_tags_0 },
    { 0xa8611cea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7011cee, SUC_lexicon_tags_11 },
    { 0x13e11cef, SUC_lexicon_tags_10 },
    { 0x73a11cee, SUC_lexicon_tags_17 },
    { 0x8ee11cef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc411cf8, SUC_lexicon_tags_1 },
    { 0x4ec11cf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84011cfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c211cff, SUC_lexicon_tags_26 },
    { 0x2f211d00, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26411d07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x85a11d09, SUC_lexicon_tags_26 },
    { 0x9d011d0a, SUC_lexicon_tags_0 },
    { 0x9ca11d0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d411d0e, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0x35a11d10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb611d14, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x37611d16, SUC_lexicon_tags_17 },
    { 0x17e11d17, SUC_lexicon_tags_6 },
    { 0x16611d18, SUC_lexicon_tags_6 },
    { 0x1fc11d19, SUC_lexicon_tags_11 },
    { 0x65411d1a, SUC_lexicon_tags_25 },
    { 0xbc611d1b, SUC_lexicon_tags_15 },
    { 0xa2011d1b, SUC_lexicon_tags_25 },
    { 0x7211d17, SUC_lexicon_tags_37 },
    { 0xdb011d1e, SUC_lexicon_tags_48 },
    { 0xfc611d1e, SUC_lexicon_tags_33 },
    { 0xd8e11d19, SUC_lexicon_tags_17 },
    { 0x94211d21, SUC_lexicon_tags_34 },
    { 0x23e11d22, SUC_lexicon_tags_32 },
    { 0x16411d1f, SUC_lexicon_tags_572 },
    { 0xce211d1d, SUC_lexicon_tags_7 },
    { 0xe9a11d25, SUC_lexicon_tags_45 },
    { 0xaa011d23, SUC_lexicon_tags_15 },
    { 0x5c811d27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83611d2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be11d34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c11d39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50211d40, SUC_lexicon_tags_70 },
    { 0x9e11d41, SUC_lexicon_tags_5 },
    { 0xff611d41, SUC_lexicon_tags_30 },
    { 0x8ac11d43, SUC_lexicon_tags_10 },
    { 0x98811d41, SUC_lexicon_tags_31 },
    { 0xef011d40, SUC_lexicon_tags_1 },
    { 0xf5811d46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73411d4a, SUC_lexicon_tags_46 },
    { 0x78a11d4b, SUC_lexicon_tags_10 },
    { 0xd7411d4a, SUC_lexicon_tags_66 },
    { 0x6e411d4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46a11d4f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4611d52, SUC_lexicon_tags_82 },
    { 0xa8211d53, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8411d5a, SUC_lexicon_tags_43 },
    { 0x4e611d5a, SUC_lexicon_tags_11 },
    { 0x1b811d5c, SUC_lexicon_tags_14 },
    { 0xe2a11d5d, SUC_lexicon_tags_7 },
    { 0x2ee11d5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf411d64, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc411d67, SUC_lexicon_tags_56 },
    { 0xe0411d68, SUC_lexicon_tags_5 },
    { 0x2b011d67, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x15211d6b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd211d74, SUC_lexicon_tags_15 },
    { 0x43211d75, SUC_lexicon_tags_15 },
    { 0x7b011d76, SUC_lexicon_tags_32 },
    { 0x3fa11d77, SUC_lexicon_tags_17 },
    { 0x3bc11d77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49611d7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbee11d7c, SUC_lexicon_tags_0 },
    { 0xc6811d7d, SUC_lexicon_tags_11 },
    { 0x81611d7d, SUC_lexicon_tags_17 },
    { 0xaa611d7f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63211d83, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x69e11d85, SUC_lexicon_tags_9 },
    { 0xefe11d86, SUC_lexicon_tags_33 },
    { 0x78411d85, SUC_lexicon_tags_23 },
    { 0x98211d85, SUC_lexicon_tags_1 },
    { 0x76a11d87, SUC_lexicon_tags_37 },
    { 0x5e011d86, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x16611d8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5211d8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a211d92, SUC_lexicon_tags_9 },
    { 0x77811d93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2611d96, SUC_lexicon_tags_1 },
    { 0x3aa11d97, SUC_lexicon_tags_23 },
    { 0xf6c11d98, SUC_lexicon_tags_65 },
    { 0x56c11d99, SUC_lexicon_tags_15 },
    { 0xbf611d99, SUC_lexicon_tags_27 },
    { 0x40211d99, SUC_lexicon_tags_17 },
    { 0x4e811d9c, SUC_lexicon_tags_8 },
    { 0x8da11d9c, SUC_lexicon_tags_32 },
    { 0x7aa11d9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde11da1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6211da5, SUC_lexicon_tags_37 },
    { 0x8c411da6, SUC_lexicon_tags_17 },
    { 0xfbc11da7, SUC_lexicon_tags_23 },
    { 0x3c211da7, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2411dac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c11db2, SUC_lexicon_tags_27 },
    { 0xb2411db3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a11db6, SUC_lexicon_tags_42 },
    { 0x1b011db7, SUC_lexicon_tags_2 },
    { 0x1cc11db8, SUC_lexicon_tags_15 },
    { 0x1411db9, SUC_lexicon_tags_14 },
    { 0x8d011db8, SUC_lexicon_tags_11 },
    { 0x74211db9, SUC_lexicon_tags_14 },
    { 0x5f611dbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a11dc0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2411dc3, SUC_lexicon_tags_8 },
    { 0x51e11dc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b011dca, SUC_lexicon_tags_37 },
    { 0x6dc11dca, SUC_lexicon_tags_49 },
    { 0xae611dca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb011dd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6e11dd3, SUC_lexicon_tags_2 },
    { 0xd5011dd4, SUC_lexicon_tags_15 },
    { 0x5d811dd5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e011dd8, SUC_lexicon_tags_1 },
    { 0x3bc11dd9, SUC_lexicon_tags_65 },
    { 0x9e011dda, SUC_lexicon_tags_82 },
    { 0x96211ddb, SUC_lexicon_tags_63 },
    { 0x72611ddb, SUC_lexicon_tags_7 },
    { 0x82011dda, SUC_lexicon_tags_17 },
    { 0x8aa11dde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae611de2, SUC_lexicon_tags_9 },
    { 0x6a011de2, SUC_lexicon_tags_11 },
    { 0xe0811de4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a011de6, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xb8a11de8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89611dec, SUC_lexicon_tags_17 },
    { 0xec011ded, SUC_lexicon_tags_24 },
    { 0x59c11dee, SUC_lexicon_tags_39 },
    { 0xc1e11def, SUC_lexicon_tags_38 },
    { 0x9a611df0, SUC_lexicon_tags_32 },
    { 0xad011df1, SUC_lexicon_tags_7 },
    { 0xeaa11df1, SUC_lexicon_tags_82 },
    { 0x9611df2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x85611df5, SUC_lexicon_tags_46 },
    { 0x72a11df5, SUC_lexicon_tags_68 },
    { 0x51c11df7, SUC_lexicon_tags_1 },
    { 0xc4011df5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x88411dfb, SUC_lexicon_tags_26 },
    { 0x1fe11dfb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x49611dfe, SUC_lexicon_tags_1 },
    { 0x4a611dfe, SUC_lexicon_tags_1 },
    { 0x7d011e00, SUC_lexicon_tags_27 },
    { 0x87411e00, SUC_lexicon_tags_24 },
    { 0xcb211dfe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x36011e05, SUC_lexicon_tags_37 },
    { 0x99e11e06, SUC_lexicon_tags_2 },
    { 0x70c11e06, SUC_lexicon_tags_19 },
    { 0x20a11e07, SUC_lexicon_tags_6 },
    { 0xd4211e09, SUC_lexicon_tags_11 },
    { 0xd6811e0a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3cc11e0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe211e10, SUC_lexicon_tags_7 },
    { 0xfee11e10, SUC_lexicon_tags_15 },
    { 0x56211e11, SUC_lexicon_tags_15 },
    { 0x96611e13, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2611e17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24211e1a, SUC_lexicon_tags_200 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de11e1d, SUC_lexicon_tags_12 },
    { 0x9ce11e1e, SUC_lexicon_tags_26 },
    { 0x92211e1d, SUC_lexicon_tags_56 },
    { 0x8be11e20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b411e22, SUC_lexicon_tags_45 },
    { 0x4f611e23, SUC_lexicon_tags_25 },
    { 0xb5811e24, SUC_lexicon_tags_17 },
    { 0x18c11e23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82211e29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a11e2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x84211e30, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69811e36, SUC_lexicon_tags_28 },
    { 0xf4211e37, SUC_lexicon_tags_11 },
    { 0xeba11e37, SUC_lexicon_tags_37 },
    { 0xd2011e37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46811e3b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc11e3f, SUC_lexicon_tags_49 },
    { 0xa2411e40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e11e45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99211e47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c811e4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42211e55, SUC_lexicon_tags_45 },
    { 0x62c11e56, SUC_lexicon_tags_14 },
    { 0xb8811e56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c11e5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9811e63, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc3011e65, SUC_lexicon_tags_39 },
    { 0xab011e66, SUC_lexicon_tags_25 },
    { 0x65411e67, SUC_lexicon_tags_14 },
    { 0x37211e68, SUC_lexicon_tags_14 },
    { 0x63e11e68, SUC_lexicon_tags_1 },
    { 0x23411e65, SUC_lexicon_tags_15 },
    { 0x59e11e6b, SUC_lexicon_tags_14 },
    { 0x37811e6c, SUC_lexicon_tags_1 },
    { 0x8a411e6c, SUC_lexicon_tags_178 },
    { 0x41611e6b, SUC_lexicon_tags_14 },
    { 0xa8811e6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c11e77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5411e7d, SUC_lexicon_tags_14 },
    { 0xd5811e7e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x88011e80, SUC_lexicon_tags_55 },
    { 0xdc011e80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf211e84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32611e8a, SUC_lexicon_tags_57 },
    { 0xe8c11e8b, SUC_lexicon_tags_26 },
    { 0xab611e8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f811e8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5411e91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b411e94, SUC_lexicon_tags_38 },
    { 0xc3e11e95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67611e98, SUC_lexicon_tags_7 },
    { 0x87411e99, SUC_lexicon_tags_11 },
    { 0x9e811e9a, SUC_lexicon_tags_689 },
    { 0xf7211e9b, SUC_lexicon_tags_7 },
    { 0x35011e9c, SUC_lexicon_tags_53 },
    { 0x24211e9d, SUC_lexicon_tags_16 },
    { 0xa8a11e9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47811ea2, SUC_lexicon_tags_11 },
    { 0x83811ea3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c11ea6, SUC_lexicon_tags_7 },
    { 0xe3c11ea7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a11eaa, SUC_lexicon_tags_12 },
    { 0x4fe11eaa, SUC_lexicon_tags_11 },
    { 0x6b811eac, SUC_lexicon_tags_25 },
    { 0xde811ead, SUC_lexicon_tags_23 },
    { 0xe9011eab, SUC_lexicon_tags_44 },
    { 0xef611eab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef411eb4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba11eb7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x28011eb9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a11ebe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88211ec0, SUC_lexicon_tags_7 },
    { 0xa0411ec1, SUC_lexicon_tags_43 },
    { 0xc3c11ec2, SUC_lexicon_tags_15 },
    { 0x27211ec1, SUC_lexicon_tags_23 },
    { 0xc211ec1, SUC_lexicon_tags_15 },
    { 0xad411ec4, SUC_lexicon_tags_60 },
    { 0x10a11ec2, SUC_lexicon_tags_14 },
    { 0xd1e11ec7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c11ecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98411ecf, SUC_lexicon_tags_43 },
    { 0x74c11ed0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x65211ed2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd011ed7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47611edb, SUC_lexicon_tags_2 },
    { 0x2a411edb, SUC_lexicon_tags_45 },
    { 0x62a11edc, SUC_lexicon_tags_17 },
    { 0xc611ede, SUC_lexicon_tags_107 },
    { 0x2e411ede, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8211ee3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27411eeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c611eee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19611ef4, SUC_lexicon_tags_25 },
    { 0xfe211ef5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb211ef7, SUC_lexicon_tags_1 },
    { 0xf0011ef8, SUC_lexicon_tags_68 },
    { 0xd1211ef9, SUC_lexicon_tags_43 },
    { 0x50411ef9, SUC_lexicon_tags_27 },
    { 0xf9e11efb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5611f00, SUC_lexicon_tags_1 },
    { 0x43211f01, SUC_lexicon_tags_2 },
    { 0x1b411f00, SUC_lexicon_tags_10 },
    { 0x51611f03, SUC_lexicon_tags_66 },
    { 0xdb811f00, SUC_lexicon_tags_15 },
    { 0xe6a11f05, SUC_lexicon_tags_63 },
    { 0x29011f03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1011f08, SUC_lexicon_tags_37 },
    { 0xa0211f09, SUC_lexicon_tags_2 },
    { 0x2e11f0a, SUC_lexicon_tags_47 },
    { 0x62611f0b, SUC_lexicon_tags_26 },
    { 0xe2211f0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce11f0f, SUC_lexicon_tags_63 },
    { 0xe9e11f10, SUC_lexicon_tags_17 },
    { 0xcf611f11, SUC_lexicon_tags_184 },
    { 0xcea11f12, SUC_lexicon_tags_21 },
    { 0x51611f13, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xe4011f15, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9811f18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a11f1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a611f21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae11f25, SUC_lexicon_tags_7 },
    { 0xae11f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6211f2b, SUC_lexicon_tags_19 },
    { 0x30411f2c, SUC_lexicon_tags_15 },
    { 0xe3211f2d, SUC_lexicon_tags_63 },
    { 0xfd611f2e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd011f31, SUC_lexicon_tags_29 },
    { 0xd211f32, SUC_lexicon_tags_17 },
    { 0x68411f32, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25611f3a, SUC_lexicon_tags_31 },
    { 0x57c11f3a, SUC_lexicon_tags_39 },
    { 0x46a11f3c, SUC_lexicon_tags_8 },
    { 0x7c11f3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac11f43, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7411f46, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93811f4b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a811f4f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x77a11f51, SUC_lexicon_tags_87 },
    { 0x23c11f52, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xab811f54, SUC_lexicon_tags_47 },
    { 0x93611f54, SUC_lexicon_tags_2 },
    { 0x5c211f56, SUC_lexicon_tags_37 },
    { 0x3b211f56, SUC_lexicon_tags_48 },
    { 0xd1c11f56, SUC_lexicon_tags_15 },
    { 0x93411f59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e11f5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c411f5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac11f63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c611f66, SUC_lexicon_tags_10 },
    { 0x42811f66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14411f6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a11f6d, SUC_lexicon_tags_1 },
    { 0x2411f6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4fe11f70, SUC_lexicon_tags_234 },
    { 0xe1c11f71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4611f76, SUC_lexicon_tags_11 },
    { 0x55011f76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95211f7c, SUC_lexicon_tags_47 },
    { 0x63a11f7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc411f7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58411f83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e11f88, SUC_lexicon_tags_55 },
    { 0xe4611f88, SUC_lexicon_tags_37 },
    { 0xca611f8a, SUC_lexicon_tags_2 },
    { 0x8d811f8b, SUC_lexicon_tags_2 },
    { 0x40811f8b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x64011f8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2611f90, SUC_lexicon_tags_68 },
    { 0xdd411f91, SUC_lexicon_tags_37 },
    { 0xd5a11f92, SUC_lexicon_tags_1 },
    { 0x6d411f91, SUC_lexicon_tags_1 },
    { 0x15c11f92, SUC_lexicon_tags_47 },
    { 0x16211f90, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3b411f97, SUC_lexicon_tags_38 },
    { 0x73011f98, SUC_lexicon_tags_23 },
    { 0xa1011f98, SUC_lexicon_tags_37 },
    { 0x5ea11f9a, SUC_lexicon_tags_7 },
    { 0xf0811f9b, SUC_lexicon_tags_33 },
    { 0x6b011f99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c811fa3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7011fa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fc11fa8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfcc11faa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5011fb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43e11fb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e11fb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6fe11fb9, SUC_lexicon_tags_2 },
    { 0x9fa11fb9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98011fc1, SUC_lexicon_tags_12 },
    { 0x4a011fc2, SUC_lexicon_tags_37 },
    { 0x95811fc3, SUC_lexicon_tags_17 },
    { 0x8a211fc4, SUC_lexicon_tags_29 },
    { 0x4dc11fc5, SUC_lexicon_tags_16 },
    { 0xb8011fc2, SUC_lexicon_tags_1 },
    { 0xda811fc7, SUC_lexicon_tags_29 },
    { 0xeca11fc7, SUC_lexicon_tags_2 },
    { 0x11c11fc8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5411fcb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c11fce, SUC_lexicon_tags_30 },
    { 0x57411fcf, SUC_lexicon_tags_23 },
    { 0xb2011fd0, SUC_lexicon_tags_35 },
    { 0x39411fd1, SUC_lexicon_tags_37 },
    { 0x4211fd0, SUC_lexicon_tags_79 },
    { 0x97411fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c011fd7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c11fda, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87011fe0, SUC_lexicon_tags_35 },
    { 0xb9411fe1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x62c11fe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e11fe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85011feb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52211fef, SUC_lexicon_tags_47 },
    { 0xa011ff0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5211ff3, SUC_lexicon_tags_68 },
    { 0xcbc11ff4, SUC_lexicon_tags_11 },
    { 0x2611ff5, SUC_lexicon_tags_17 },
    { 0x4c211ff6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba11ffb, SUC_lexicon_tags_7 },
    { 0x85011ffc, SUC_lexicon_tags_15 },
    { 0xb9811ffb, SUC_lexicon_tags_0 },
    { 0x8d811ffc, SUC_lexicon_tags_4 },
    { 0x27a11fff, SUC_lexicon_tags_2 },
    { 0x30011ffe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc12002, SUC_lexicon_tags_46 },
    { 0xdf012003, SUC_lexicon_tags_25 },
    { 0x2d012004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82812009, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98e1200b, SUC_lexicon_tags_25 },
    { 0x3c61200c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc1200f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86412013, SUC_lexicon_tags_45 },
    { 0x19a12013, SUC_lexicon_tags_2 },
    { 0xa3e12015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fa12017, SUC_lexicon_tags_1 },
    { 0x8d212018, SUC_lexicon_tags_17 },
    { 0xc5412019, SUC_lexicon_tags_1 },
    { 0x23c12018, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e12022, SUC_lexicon_tags_43 },
    { 0x13e12023, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x30212025, SUC_lexicon_tags_12 },
    { 0x5c412026, SUC_lexicon_tags_28 },
    { 0xc7812027, SUC_lexicon_tags_1 },
    { 0x3d412028, SUC_lexicon_tags_45 },
    { 0xc8c12028, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7f41202b, SUC_lexicon_tags_15 },
    { 0x9b41202c, SUC_lexicon_tags_42 },
    { 0x6161202d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69812034, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8412039, SUC_lexicon_tags_2 },
    { 0x3681203a, SUC_lexicon_tags_5 },
    { 0x7ba1203b, SUC_lexicon_tags_1 },
    { 0xf021203b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e12043, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b41204f, SUC_lexicon_tags_25 },
    { 0xaf412050, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61c12052, SUC_lexicon_tags_10 },
    { 0x4d812053, SUC_lexicon_tags_9 },
    { 0x83212054, SUC_lexicon_tags_27 },
    { 0xc8412054, SUC_lexicon_tags_12 },
    { 0x14812053, SUC_lexicon_tags_15 },
    { 0x76212053, SUC_lexicon_tags_27 },
    { 0x70e12058, SUC_lexicon_tags_38 },
    { 0x91a12059, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb761205d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x26a1205f, SUC_lexicon_tags_1 },
    { 0xcae1205f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a12063, SUC_lexicon_tags_25 },
    { 0xe6c12064, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba12067, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1401206a, SUC_lexicon_tags_47 },
    { 0x14c1206a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c12070, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a12074, SUC_lexicon_tags_33 },
    { 0x88e12075, SUC_lexicon_tags_37 },
    { 0x95212074, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5612079, SUC_lexicon_tags_9 },
    { 0x70212079, SUC_lexicon_tags_4 },
    { 0x85612079, SUC_lexicon_tags_2 },
    { 0x4b61207c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5181207f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe12087, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa321208c, SUC_lexicon_tags_7 },
    { 0xd081208c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75812092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc12095, SUC_lexicon_tags_47 },
    { 0x44a12095, SUC_lexicon_tags_12 },
    { 0xb6412095, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8401209e, SUC_lexicon_tags_35 },
    { 0x78a1209f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6120a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a120a8, SUC_lexicon_tags_23 },
    { 0xed8120a9, SUC_lexicon_tags_11 },
    { 0xf58120aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb8120ac, SUC_lexicon_tags_15 },
    { 0x69a120ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31c120af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84120b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6120b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b4120b9, SUC_lexicon_tags_37 },
    { 0x936120ba, SUC_lexicon_tags_21 },
    { 0x504120ba, SUC_lexicon_tags_14 },
    { 0x7c120bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6120bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec120c5, SUC_lexicon_tags_7 },
    { 0x5be120c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4120c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xece120cb, SUC_lexicon_tags_49 },
    { 0xc96120cb, SUC_lexicon_tags_2 },
    { 0x42e120cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c120d2, SUC_lexicon_tags_47 },
    { 0x326120d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x860120d6, SUC_lexicon_tags_16 },
    { 0x5e2120d7, SUC_lexicon_tags_34 },
    { 0xdb0120d8, SUC_lexicon_tags_2 },
    { 0x464120d9, SUC_lexicon_tags_1 },
    { 0x3e6120da, SUC_lexicon_tags_37 },
    { 0x86a120d9, SUC_lexicon_tags_60 },
    { 0xde6120dc, SUC_lexicon_tags_28 },
    { 0x358120dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c120e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e120e2, SUC_lexicon_tags_17 },
    { 0x92120e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c120e6, SUC_lexicon_tags_37 },
    { 0x3a0120e6, SUC_lexicon_tags_63 },
    { 0xff8120e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8120eb, SUC_lexicon_tags_47 },
    { 0xb90120ec, SUC_lexicon_tags_23 },
    { 0xd70120ed, SUC_lexicon_tags_31 },
    { 0xfda120ee, SUC_lexicon_tags_14 },
    { 0x20120eb, SUC_lexicon_tags_23 },
    { 0xdfc120eb, SUC_lexicon_tags_32 },
    { 0x242120f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc120f6, SUC_lexicon_tags_37 },
    { 0x994120f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba12100, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82812103, SUC_lexicon_tags_1 },
    { 0xbc812104, SUC_lexicon_tags_27 },
    { 0x55e12104, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec12108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7741210b, SUC_lexicon_tags_12 },
    { 0xfb41210b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3412114, SUC_lexicon_tags_6 },
    { 0xef412114, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3412119, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb21211f, SUC_lexicon_tags_7 },
    { 0xba612120, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7212122, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1012127, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xde01212a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d41212c, SUC_lexicon_tags_15 },
    { 0x2681212d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2012135, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf812138, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1421213a, SUC_lexicon_tags_45 },
    { 0x8801213b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda21213e, SUC_lexicon_tags_2 },
    { 0xfa21213f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe12142, SUC_lexicon_tags_32 },
    { 0xfa812142, SUC_lexicon_tags_60 },
    { 0x8fe12144, SUC_lexicon_tags_37 },
    { 0x25412145, SUC_lexicon_tags_17 },
    { 0x8a12145, SUC_lexicon_tags_25 },
    { 0x4612147, SUC_lexicon_tags_49 },
    { 0xda812148, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc1214c, SUC_lexicon_tags_87 },
    { 0x5b81214c, SUC_lexicon_tags_17 },
    { 0xb141214e, SUC_lexicon_tags_124 },
    { 0x2481214f, SUC_lexicon_tags_17 },
    { 0x3161214f, SUC_lexicon_tags_17 },
    { 0x2aa12150, SUC_lexicon_tags_15 },
    { 0x5da12152, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c612154, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f812158, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7a1215a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e1215f, SUC_lexicon_tags_63 },
    { 0xf8812160, SUC_lexicon_tags_27 },
    { 0x5be12161, SUC_lexicon_tags_18 },
    { 0x4e212162, SUC_lexicon_tags_15 },
    { 0x17c12163, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f61216c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e81216f, SUC_lexicon_tags_12 },
    { 0xa2612170, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75412172, SUC_lexicon_tags_16 },
    { 0x6d612173, SUC_lexicon_tags_0 },
    { 0xf2812174, SUC_lexicon_tags_29 },
    { 0x87812173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56e12177, SUC_lexicon_tags_61 },
    { 0x55812178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa981217b, SUC_lexicon_tags_65 },
    { 0xd501217c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda012182, SUC_lexicon_tags_15 },
    { 0x31812182, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3412185, SUC_lexicon_tags_47 },
    { 0xc0412185, SUC_lexicon_tags_39 },
    { 0xb4e12187, SUC_lexicon_tags_17 },
    { 0xcc012188, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e41218a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2541218d, SUC_lexicon_tags_28 },
    { 0xc8c1218e, SUC_lexicon_tags_37 },
    { 0x1321218f, SUC_lexicon_tags_7 },
    { 0xd121218e, SUC_lexicon_tags_15 },
    { 0xf7a1218e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf012196, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda812198, SUC_lexicon_tags_6 },
    { 0x4a012199, SUC_lexicon_tags_47 },
    { 0xae21219a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa1219d, SUC_lexicon_tags_63 },
    { 0x8281219d, SUC_lexicon_tags_12 },
    { 0x9601219e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa22121a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec121a7, SUC_lexicon_tags_46 },
    { 0x5e6121a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x726121ab, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc92121ad, SUC_lexicon_tags_19 },
    { 0xcf0121ad, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xc66121b0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c121b4, SUC_lexicon_tags_11 },
    { 0xbec121b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a121b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a8121bb, SUC_lexicon_tags_14 },
    { 0xe68121bb, SUC_lexicon_tags_45 },
    { 0x528121bd, SUC_lexicon_tags_45 },
    { 0x180121be, SUC_lexicon_tags_12 },
    { 0x3ba121bf, SUC_lexicon_tags_2 },
    { 0xcda121bf, SUC_lexicon_tags_17 },
    { 0xa0121c1, SUC_lexicon_tags_15 },
    { 0xb90121c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26121c9, SUC_lexicon_tags_43 },
    { 0x46a121ca, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x656121cd, SUC_lexicon_tags_27 },
    { 0xa84121ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb62121d0, SUC_lexicon_tags_1 },
    { 0x3e8121d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132121d7, SUC_lexicon_tags_47 },
    { 0xdbe121d8, SUC_lexicon_tags_35 },
    { 0x1a0121d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4121dd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x180121e0, SUC_lexicon_tags_0 },
    { 0x5a0121e0, SUC_lexicon_tags_33 },
    { 0x7d4121e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfa0121e4, SUC_lexicon_tags_15 },
    { 0x1dc121e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62121ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d6121ee, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86121f8, SUC_lexicon_tags_6 },
    { 0x448121f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d012205, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79212209, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb181220c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7812212, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e12215, SUC_lexicon_tags_56 },
    { 0x9412216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15812218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3612221, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a12224, SUC_lexicon_tags_17 },
    { 0x85212225, SUC_lexicon_tags_47 },
    { 0x89612226, SUC_lexicon_tags_1 },
    { 0x5aa12224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3012229, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc581222b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f61222d, SUC_lexicon_tags_55 },
    { 0xba81222e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x14212230, SUC_lexicon_tags_117 },
    { 0xcf812231, SUC_lexicon_tags_21 },
    { 0x5012232, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15812235, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65a12237, SUC_lexicon_tags_21 },
    { 0x23e12237, SUC_lexicon_tags_14 },
    { 0x29c12239, SUC_lexicon_tags_63 },
    { 0x14c1223a, SUC_lexicon_tags_11 },
    { 0x6a41223a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e1223f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e12245, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84a12247, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3941224b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd3c1224d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae12252, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10812256, SUC_lexicon_tags_31 },
    { 0xfda12257, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd41225e, SUC_lexicon_tags_37 },
    { 0xad81225f, SUC_lexicon_tags_37 },
    { 0x7121225f, SUC_lexicon_tags_8 },
    { 0xe2a12261, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x69212263, SUC_lexicon_tags_8 },
    { 0x91012264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69212267, SUC_lexicon_tags_25 },
    { 0xfb212268, SUC_lexicon_tags_2 },
    { 0xc2212269, SUC_lexicon_tags_17 },
    { 0xec012269, SUC_lexicon_tags_4 },
    { 0xb9c1226b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3081226f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a12273, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a12278, SUC_lexicon_tags_49 },
    { 0x83012279, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73612283, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c12287, SUC_lexicon_tags_1 },
    { 0x48612288, SUC_lexicon_tags_61 },
    { 0xf0a12289, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1012290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de12295, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4812297, SUC_lexicon_tags_12 },
    { 0x27a12297, SUC_lexicon_tags_29 },
    { 0x11412299, SUC_lexicon_tags_12 },
    { 0x9541229a, SUC_lexicon_tags_43 },
    { 0xb0812299, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4de1229d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaf81229f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x140122a2, SUC_lexicon_tags_7 },
    { 0x6c2122a3, SUC_lexicon_tags_31 },
    { 0x44a122a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x262122a7, SUC_lexicon_tags_6 },
    { 0xd24122a7, SUC_lexicon_tags_1 },
    { 0x1e4122a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6122ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426122b1, SUC_lexicon_tags_0 },
    { 0xd2122b2, SUC_lexicon_tags_19 },
    { 0x600122b3, SUC_lexicon_tags_47 },
    { 0xc9c122b4, SUC_lexicon_tags_16 },
    { 0x24122b5, SUC_lexicon_tags_25 },
    { 0x1a0122b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe122bc, SUC_lexicon_tags_67 },
    { 0x12e122bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0a122bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0122c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e0122c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x87a122c7, SUC_lexicon_tags_37 },
    { 0xaba122c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc4c122ca, SUC_lexicon_tags_14 },
    { 0xf36122cb, SUC_lexicon_tags_17 },
    { 0xc9e122cc, SUC_lexicon_tags_49 },
    { 0xd62122cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a122d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36122d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4122da, SUC_lexicon_tags_14 },
    { 0xd42122da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe122de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x488122e0, SUC_lexicon_tags_25 },
    { 0xf80122e0, SUC_lexicon_tags_15 },
    { 0xfd2122e2, SUC_lexicon_tags_37 },
    { 0x1f4122e2, SUC_lexicon_tags_37 },
    { 0x13a122e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412122ed, SUC_lexicon_tags_17 },
    { 0x6e4122ed, SUC_lexicon_tags_46 },
    { 0x598122ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x35c122f1, SUC_lexicon_tags_7 },
    { 0x170122f2, SUC_lexicon_tags_19 },
    { 0x852122f3, SUC_lexicon_tags_2 },
    { 0x26e122f4, SUC_lexicon_tags_15 },
    { 0xfae122f5, SUC_lexicon_tags_15 },
    { 0xb16122f6, SUC_lexicon_tags_9 },
    { 0x23e122f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa122fe, SUC_lexicon_tags_37 },
    { 0xb40122ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c12302, SUC_lexicon_tags_26 },
    { 0x2412303, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e12306, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda1230c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e41230f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2212312, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda12316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7381231a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48812324, SUC_lexicon_tags_37 },
    { 0x28812325, SUC_lexicon_tags_11 },
    { 0x64c12326, SUC_lexicon_tags_5 },
    { 0x5c812327, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6561232a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc021232c, SUC_lexicon_tags_17 },
    { 0xde61232c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee61232f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5812333, SUC_lexicon_tags_32 },
    { 0x52612334, SUC_lexicon_tags_0 },
    { 0x51412335, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a12338, SUC_lexicon_tags_12 },
    { 0xae612339, SUC_lexicon_tags_0 },
    { 0x36e12339, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7381233d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1012341, SUC_lexicon_tags_1 },
    { 0xc9612341, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6612348, SUC_lexicon_tags_37 },
    { 0x4d812349, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de1234c, SUC_lexicon_tags_11 },
    { 0x5681234c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c12350, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71412354, SUC_lexicon_tags_19 },
    { 0xcc412354, SUC_lexicon_tags_47 },
    { 0x2c812356, SUC_lexicon_tags_33 },
    { 0xf9a12357, SUC_lexicon_tags_7 },
    { 0xc4412357, SUC_lexicon_tags_28 },
    { 0xec412356, SUC_lexicon_tags_25 },
    { 0xfa1235a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe481235e, SUC_lexicon_tags_25 },
    { 0xd6a1235f, SUC_lexicon_tags_7 },
    { 0xc81235f, SUC_lexicon_tags_44 },
    { 0x68012361, SUC_lexicon_tags_22 },
    { 0x5e012362, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c12367, SUC_lexicon_tags_47 },
    { 0xd9612367, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec1236c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x28c1236e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86012374, SUC_lexicon_tags_27 },
    { 0x59412374, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8212377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9681237b, SUC_lexicon_tags_15 },
    { 0xc1237c, SUC_lexicon_tags_27 },
    { 0x1aa1237d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe41237f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc6e12381, SUC_lexicon_tags_12 },
    { 0x88212382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60612387, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5fe12389, SUC_lexicon_tags_19 },
    { 0x2f41238a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf241238e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0612390, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a12396, SUC_lexicon_tags_28 },
    { 0x3a412396, SUC_lexicon_tags_15 },
    { 0x1a212397, SUC_lexicon_tags_49 },
    { 0xba012399, SUC_lexicon_tags_25 },
    { 0x12a12399, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5141239f, SUC_lexicon_tags_37 },
    { 0xdc6123a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3c123a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310123ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a123af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2123bb, SUC_lexicon_tags_123 },
    { 0x57a123bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x494123be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930123c2, SUC_lexicon_tags_1 },
    { 0x802123c3, SUC_lexicon_tags_25 },
    { 0xee6123c2, SUC_lexicon_tags_11 },
    { 0xf1a123c5, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e123c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa123cb, SUC_lexicon_tags_63 },
    { 0xfec123cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de123d1, SUC_lexicon_tags_7 },
    { 0xb5a123d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2123d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838123dc, SUC_lexicon_tags_11 },
    { 0xba8123dd, SUC_lexicon_tags_84 },
    { 0x718123de, SUC_lexicon_tags_17 },
    { 0xdb8123de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e123e9, SUC_lexicon_tags_21 },
    { 0xff0123e9, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de123ed, SUC_lexicon_tags_7 },
    { 0xc9c123ee, SUC_lexicon_tags_37 },
    { 0x3fe123ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c212405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1041240c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea12413, SUC_lexicon_tags_10 },
    { 0x60612413, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed412418, SUC_lexicon_tags_1 },
    { 0x41812418, SUC_lexicon_tags_7 },
    { 0xdb212418, SUC_lexicon_tags_37 },
    { 0x9bc1241b, SUC_lexicon_tags_25 },
    { 0x6cc1241c, SUC_lexicon_tags_295 },
    { 0xc21241d, SUC_lexicon_tags_15 },
    { 0x81241d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82812420, SUC_lexicon_tags_15 },
    { 0x1e412421, SUC_lexicon_tags_11 },
    { 0xff012421, SUC_lexicon_tags_25 },
    { 0x6fa12422, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e212431, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40012433, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6c612435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe12438, SUC_lexicon_tags_27 },
    { 0x69a12439, SUC_lexicon_tags_14 },
    { 0xe981243a, SUC_lexicon_tags_12 },
    { 0x9f61243b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86412442, SUC_lexicon_tags_47 },
    { 0xe0412443, SUC_lexicon_tags_25 },
    { 0x4cc12443, SUC_lexicon_tags_17 },
    { 0x27612444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8461244a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa961244d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xca61244f, SUC_lexicon_tags_49 },
    { 0x3ac12450, SUC_lexicon_tags_25 },
    { 0x6f012451, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5212459, SUC_lexicon_tags_25 },
    { 0xe7a1245a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x161245c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8621245f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe212462, SUC_lexicon_tags_15 },
    { 0x5a412463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa12467, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69812469, SUC_lexicon_tags_1 },
    { 0xcf612469, SUC_lexicon_tags_87 },
    { 0x9561246a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e1246e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a12472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d612475, SUC_lexicon_tags_6 },
    { 0xe012475, SUC_lexicon_tags_37 },
    { 0x4812475, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c12480, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5012483, SUC_lexicon_tags_11 },
    { 0x1fa12484, SUC_lexicon_tags_21 },
    { 0xb7412485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdca12487, SUC_lexicon_tags_9 },
    { 0x18c12488, SUC_lexicon_tags_45 },
    { 0x52212488, SUC_lexicon_tags_1 },
    { 0xaae1248a, SUC_lexicon_tags_12 },
    { 0x4241248b, SUC_lexicon_tags_11 },
    { 0xddc1248c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5e01248e, SUC_lexicon_tags_63 },
    { 0x5901248f, SUC_lexicon_tags_14 },
    { 0xb6e12490, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f012492, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5612495, SUC_lexicon_tags_15 },
    { 0xc12495, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa212499, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa261249b, SUC_lexicon_tags_61 },
    { 0xb721249c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe401249f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf08124a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390124a6, SUC_lexicon_tags_15 },
    { 0xe3c124a6, SUC_lexicon_tags_16 },
    { 0x26e124a8, SUC_lexicon_tags_49 },
    { 0xb96124a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa16124ab, SUC_lexicon_tags_8 },
    { 0x550124ac, SUC_lexicon_tags_46 },
    { 0xa36124ac, SUC_lexicon_tags_19 },
    { 0x40c124ad, SUC_lexicon_tags_31 },
    { 0x26e124af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780124b3, SUC_lexicon_tags_15 },
    { 0xdea124b4, SUC_lexicon_tags_38 },
    { 0x22e124b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb0124b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8124bc, SUC_lexicon_tags_1 },
    { 0x6c2124bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776124c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x666124c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf80124c8, SUC_lexicon_tags_12 },
    { 0x588124c9, SUC_lexicon_tags_37 },
    { 0x420124c8, SUC_lexicon_tags_17 },
    { 0x3ec124ca, SUC_lexicon_tags_35 },
    { 0x1dc124cc, SUC_lexicon_tags_1 },
    { 0xc98124cc, SUC_lexicon_tags_0 },
    { 0x722124ce, SUC_lexicon_tags_63 },
    { 0xd4e124cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf12124d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x462124d3, SUC_lexicon_tags_8 },
    { 0xa8a124d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808124da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4124e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158124eb, SUC_lexicon_tags_15 },
    { 0x18124ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2124ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe124f6, SUC_lexicon_tags_37 },
    { 0x578124f7, SUC_lexicon_tags_19 },
    { 0x470124f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe3a124fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4124fd, SUC_lexicon_tags_17 },
    { 0x718124fe, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d412502, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae12508, SUC_lexicon_tags_643 },
    { 0x47212508, SUC_lexicon_tags_47 },
    { 0x73a12509, SUC_lexicon_tags_0 },
    { 0xd041250b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7801250d, SUC_lexicon_tags_9 },
    { 0x15c1250d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac12513, SUC_lexicon_tags_37 },
    { 0xebc12513, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc12519, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce12521, SUC_lexicon_tags_19 },
    { 0x3c012522, SUC_lexicon_tags_7 },
    { 0x64612523, SUC_lexicon_tags_82 },
    { 0xa1212524, SUC_lexicon_tags_1 },
    { 0x4a812525, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3212527, SUC_lexicon_tags_23 },
    { 0x85612528, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e61252b, SUC_lexicon_tags_60 },
    { 0x7121252b, SUC_lexicon_tags_39 },
    { 0xfd81252b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc12531, SUC_lexicon_tags_17 },
    { 0xd6612532, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26812537, SUC_lexicon_tags_43 },
    { 0x3612538, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d01253b, SUC_lexicon_tags_38 },
    { 0x4721253b, SUC_lexicon_tags_87 },
    { 0x7701253d, SUC_lexicon_tags_55 },
    { 0x8961253e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec212542, SUC_lexicon_tags_66 },
    { 0xfac12542, SUC_lexicon_tags_63 },
    { 0x89c12542, SUC_lexicon_tags_63 },
    { 0x1412545, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x12012547, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x10e12549, SUC_lexicon_tags_21 },
    { 0x43e12549, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65612550, SUC_lexicon_tags_33 },
    { 0x10012550, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85412557, SUC_lexicon_tags_8 },
    { 0x86612558, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11c1255a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e12560, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d812563, SUC_lexicon_tags_11 },
    { 0xf4412563, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc812569, SUC_lexicon_tags_287 },
    { 0xdb012569, SUC_lexicon_tags_23 },
    { 0x7fc1256b, SUC_lexicon_tags_37 },
    { 0x95a1256c, SUC_lexicon_tags_45 },
    { 0x7f61256a, SUC_lexicon_tags_15 },
    { 0xef81256e, SUC_lexicon_tags_23 },
    { 0x1961256e, SUC_lexicon_tags_0 },
    { 0x5de12570, SUC_lexicon_tags_25 },
    { 0x8781256a, SUC_lexicon_tags_11 },
    { 0x94212572, SUC_lexicon_tags_15 },
    { 0xfca1256e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd412575, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x99212577, SUC_lexicon_tags_1 },
    { 0xc3812578, SUC_lexicon_tags_48 },
    { 0xc8a12579, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8381257f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfe612581, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f612584, SUC_lexicon_tags_33 },
    { 0xfca12585, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b012588, SUC_lexicon_tags_60 },
    { 0xbce12589, SUC_lexicon_tags_8 },
    { 0xd801258a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6321258d, SUC_lexicon_tags_45 },
    { 0xf5c1258e, SUC_lexicon_tags_254 },
    { 0x2241258e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a12592, SUC_lexicon_tags_19 },
    { 0xa1412593, SUC_lexicon_tags_37 },
    { 0x65c12594, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6541259b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7041259d, SUC_lexicon_tags_7 },
    { 0x6c61259e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce6125a0, SUC_lexicon_tags_25 },
    { 0x8c4125a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca125a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2125a7, SUC_lexicon_tags_25 },
    { 0xbe8125a7, SUC_lexicon_tags_40 },
    { 0x436125a8, SUC_lexicon_tags_39 },
    { 0x742125aa, SUC_lexicon_tags_10 },
    { 0x66a125ab, SUC_lexicon_tags_25 },
    { 0xf2a125aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6125af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530125b3, SUC_lexicon_tags_11 },
    { 0xaf4125b3, SUC_lexicon_tags_42 },
    { 0xea8125b4, SUC_lexicon_tags_29 },
    { 0xde125b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe52125b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a125bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a125bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66125c2, SUC_lexicon_tags_37 },
    { 0x19e125c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2e125c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x870125c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeea125ca, SUC_lexicon_tags_28 },
    { 0x67a125ca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4125cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c125d4, SUC_lexicon_tags_17 },
    { 0x67c125d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfde125d7, SUC_lexicon_tags_56 },
    { 0x80e125d8, SUC_lexicon_tags_7 },
    { 0xa8e125d9, SUC_lexicon_tags_11 },
    { 0xfd8125da, SUC_lexicon_tags_19 },
    { 0xc8e125db, SUC_lexicon_tags_30 },
    { 0xa06125d9, SUC_lexicon_tags_37 },
    { 0x5c0125dd, SUC_lexicon_tags_0 },
    { 0xce2125dc, SUC_lexicon_tags_7 },
    { 0xf60125df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6c125e1, SUC_lexicon_tags_17 },
    { 0xcf8125e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c125e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66125e9, SUC_lexicon_tags_2 },
    { 0x4c6125ea, SUC_lexicon_tags_15 },
    { 0x260125eb, SUC_lexicon_tags_11 },
    { 0x48125eb, SUC_lexicon_tags_11 },
    { 0x638125eb, SUC_lexicon_tags_16 },
    { 0xa98125ee, SUC_lexicon_tags_35 },
    { 0x78c125ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4c125f1, SUC_lexicon_tags_14 },
    { 0xf1a125f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaca125f4, SUC_lexicon_tags_7 },
    { 0x990125f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20125fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4125ff, SUC_lexicon_tags_11 },
    { 0xe4125ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c12603, SUC_lexicon_tags_23 },
    { 0x8d412603, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2ee12606, SUC_lexicon_tags_15 },
    { 0x6b612607, SUC_lexicon_tags_14 },
    { 0x63012608, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeca1260a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca1260d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9b01260f, SUC_lexicon_tags_11 },
    { 0xf9012610, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a012616, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e1261d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9212625, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc612628, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc1262d, SUC_lexicon_tags_46 },
    { 0x8fa1262e, SUC_lexicon_tags_12 },
    { 0x6001262f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12634, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe12641, SUC_lexicon_tags_7 },
    { 0x29212641, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe612644, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x26612647, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa161264c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a1264f, SUC_lexicon_tags_54 },
    { 0x2ba12650, SUC_lexicon_tags_15 },
    { 0x5fe12651, SUC_lexicon_tags_18 },
    { 0x72a12651, SUC_lexicon_tags_42 },
    { 0xe2812652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d612657, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c1265d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdaa1265f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0212663, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac12665, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c12669, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe1266c, SUC_lexicon_tags_35 },
    { 0x6441266d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72812673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87012675, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x38612677, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbe212679, SUC_lexicon_tags_46 },
    { 0xa341267a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa1267f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8212687, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73812689, SUC_lexicon_tags_19 },
    { 0x27212689, SUC_lexicon_tags_170 },
    { 0x4961268a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6c61268d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38e1268f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3812692, SUC_lexicon_tags_37 },
    { 0x8d412692, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x28212695, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c12699, SUC_lexicon_tags_14 },
    { 0xd321269a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5581269c, SUC_lexicon_tags_9 },
    { 0xb9c1269c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46126a2, SUC_lexicon_tags_37 },
    { 0x8ca126a2, SUC_lexicon_tags_19 },
    { 0x40c126a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa56126a6, SUC_lexicon_tags_2 },
    { 0xe7e126a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb46126a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43c126ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7ec126ad, SUC_lexicon_tags_26 },
    { 0xc88126ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6126b3, SUC_lexicon_tags_14 },
    { 0xf6a126b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622126b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x760126ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8126bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14126c9, SUC_lexicon_tags_2 },
    { 0x31c126ca, SUC_lexicon_tags_7 },
    { 0x40c126ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2126d0, SUC_lexicon_tags_6 },
    { 0xf88126d0, SUC_lexicon_tags_77 },
    { 0x5d0126d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x126126d4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4126d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4ac126db, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5d6126dd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4126e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xca2126e3, SUC_lexicon_tags_31 },
    { 0xf56126e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6126e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6c126ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140126f6, SUC_lexicon_tags_28 },
    { 0xbfc126f7, SUC_lexicon_tags_17 },
    { 0xf32126f8, SUC_lexicon_tags_27 },
    { 0x110126f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c126ff, SUC_lexicon_tags_55 },
    { 0xf4812700, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8012702, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32012708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fa1270a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8941270c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf481270e, SUC_lexicon_tags_60 },
    { 0x48e1270f, SUC_lexicon_tags_12 },
    { 0x9261270f, SUC_lexicon_tags_7 },
    { 0x6f61270e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8812713, SUC_lexicon_tags_9 },
    { 0x4812714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa212716, SUC_lexicon_tags_25 },
    { 0x98212717, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5641271c, SUC_lexicon_tags_61 },
    { 0xc781271d, SUC_lexicon_tags_91 },
    { 0x9781271e, SUC_lexicon_tags_62 },
    { 0xcec1271f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc12728, SUC_lexicon_tags_25 },
    { 0x1d012729, SUC_lexicon_tags_91 },
    { 0xc441272a, SUC_lexicon_tags_17 },
    { 0xd261272a, SUC_lexicon_tags_11 },
    { 0x73c1272c, SUC_lexicon_tags_15 },
    { 0x6e1272c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2121272f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaee12731, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21273b, SUC_lexicon_tags_12 },
    { 0x8001273c, SUC_lexicon_tags_43 },
    { 0x55a1273b, SUC_lexicon_tags_21 },
    { 0x2001273b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0e12740, SUC_lexicon_tags_46 },
    { 0xaee12741, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e12747, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a1274e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c612750, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41812755, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb641275a, SUC_lexicon_tags_84 },
    { 0x48a1275a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc281275d, SUC_lexicon_tags_15 },
    { 0xa421275e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3be12760, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe812763, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75212765, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6012767, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6a12769, SUC_lexicon_tags_2 },
    { 0x3e812769, SUC_lexicon_tags_12 },
    { 0xb9e1276b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc381276f, SUC_lexicon_tags_38 },
    { 0xf3a1276f, SUC_lexicon_tags_8 },
    { 0x30612771, SUC_lexicon_tags_56 },
    { 0xbe212770, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1212776, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xab012778, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c1277d, SUC_lexicon_tags_15 },
    { 0xde41277d, SUC_lexicon_tags_55 },
    { 0x4501277f, SUC_lexicon_tags_37 },
    { 0x52012780, SUC_lexicon_tags_10 },
    { 0xf3612781, SUC_lexicon_tags_19 },
    { 0xdb412782, SUC_lexicon_tags_11 },
    { 0xe3612781, SUC_lexicon_tags_45 },
    { 0x32412781, SUC_lexicon_tags_11 },
    { 0xba412785, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a41278a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc241278d, SUC_lexicon_tags_9 },
    { 0xe721278d, SUC_lexicon_tags_25 },
    { 0xee61278f, SUC_lexicon_tags_37 },
    { 0xfb81278f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c012797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f01279b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9881279e, SUC_lexicon_tags_39 },
    { 0x7781279f, SUC_lexicon_tags_46 },
    { 0x76e127a0, SUC_lexicon_tags_34 },
    { 0x858127a1, SUC_lexicon_tags_12 },
    { 0xb4a127a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c127a5, SUC_lexicon_tags_60 },
    { 0xccc127a5, SUC_lexicon_tags_6 },
    { 0x180127a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa127ab, SUC_lexicon_tags_0 },
    { 0x78127ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x944127af, SUC_lexicon_tags_9 },
    { 0x42127af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a127b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1b0127b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdb0127b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6127bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576127c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c127c7, SUC_lexicon_tags_12 },
    { 0xa2e127c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c127cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc96127cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68127d2, SUC_lexicon_tags_21 },
    { 0xeb6127d3, SUC_lexicon_tags_2 },
    { 0x9f2127d4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2127d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a8127da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e127e0, SUC_lexicon_tags_37 },
    { 0x632127e0, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434127e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c127e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936127f1, SUC_lexicon_tags_15 },
    { 0x5c8127f2, SUC_lexicon_tags_25 },
    { 0x402127f1, SUC_lexicon_tags_6 },
    { 0x93a127f4, SUC_lexicon_tags_47 },
    { 0xd28127f5, SUC_lexicon_tags_12 },
    { 0xa48127f5, SUC_lexicon_tags_17 },
    { 0x396127f7, SUC_lexicon_tags_33 },
    { 0x7a0127f7, SUC_lexicon_tags_23 },
    { 0x7aa127f8, SUC_lexicon_tags_1 },
    { 0xa3e127fa, SUC_lexicon_tags_2 },
    { 0x85e127fb, SUC_lexicon_tags_579 },
    { 0xfa8127fc, SUC_lexicon_tags_0 },
    { 0xf62127f8, SUC_lexicon_tags_136 },
    { 0x394127f6, SUC_lexicon_tags_32 },
    { 0x730127ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18e12801, SUC_lexicon_tags_28 },
    { 0x17a12802, SUC_lexicon_tags_37 },
    { 0x7ec12801, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd812805, SUC_lexicon_tags_7 },
    { 0x83e12806, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac1280a, SUC_lexicon_tags_25 },
    { 0x6c01280b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e1280e, SUC_lexicon_tags_7 },
    { 0x6e81280e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec12812, SUC_lexicon_tags_7 },
    { 0xffe12813, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2612815, SUC_lexicon_tags_37 },
    { 0xd4012816, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x25c12818, SUC_lexicon_tags_9 },
    { 0xd1212819, SUC_lexicon_tags_1 },
    { 0x40612819, SUC_lexicon_tags_2 },
    { 0x4d81281b, SUC_lexicon_tags_57 },
    { 0x8541281b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f012822, SUC_lexicon_tags_11 },
    { 0x5012822, SUC_lexicon_tags_34 },
    { 0xd6c12823, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec12828, SUC_lexicon_tags_4 },
    { 0xce812829, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe921282b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc1282f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71012831, SUC_lexicon_tags_15 },
    { 0x22e12832, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21012837, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe2212839, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a01283b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d41283e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec12840, SUC_lexicon_tags_19 },
    { 0xc9612840, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2412849, SUC_lexicon_tags_27 },
    { 0x3b01284a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3412850, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26212853, SUC_lexicon_tags_9 },
    { 0xd2412854, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4612858, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a1285a, SUC_lexicon_tags_293 },
    { 0x581285b, SUC_lexicon_tags_17 },
    { 0xfca1285a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b01285f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77412864, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e212866, SUC_lexicon_tags_49 },
    { 0x2ba12867, SUC_lexicon_tags_57 },
    { 0xda012868, SUC_lexicon_tags_11 },
    { 0xcfe12869, SUC_lexicon_tags_7 },
    { 0x3941286a, SUC_lexicon_tags_17 },
    { 0xfb01286b, SUC_lexicon_tags_43 },
    { 0xec01286c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5812873, SUC_lexicon_tags_33 },
    { 0x1b012874, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a12879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c12881, SUC_lexicon_tags_27 },
    { 0x86812881, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3612887, SUC_lexicon_tags_14 },
    { 0x7f012888, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1121288a, SUC_lexicon_tags_17 },
    { 0x3461288a, SUC_lexicon_tags_33 },
    { 0xf161288c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97012892, SUC_lexicon_tags_11 },
    { 0xba812893, SUC_lexicon_tags_27 },
    { 0x66212894, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c1289a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e1289f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfc128a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3e128a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55e128a5, SUC_lexicon_tags_138 },
    { 0x14c128a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66128a9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x458128ac, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xeb0128ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04128b1, SUC_lexicon_tags_0 },
    { 0x96a128b1, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x17e128b4, SUC_lexicon_tags_17 },
    { 0x26e128b4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x508128b8, SUC_lexicon_tags_15 },
    { 0xf44128b9, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c128bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c128c1, SUC_lexicon_tags_45 },
    { 0x3f8128c1, SUC_lexicon_tags_30 },
    { 0x88a128c2, SUC_lexicon_tags_30 },
    { 0xcd0128c1, SUC_lexicon_tags_0 },
    { 0x9b4128c5, SUC_lexicon_tags_14 },
    { 0xa98128c6, SUC_lexicon_tags_13 },
    { 0x1b2128c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0128cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc128d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2128d5, SUC_lexicon_tags_12 },
    { 0x7ec128d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xece128d8, SUC_lexicon_tags_31 },
    { 0x94c128d9, SUC_lexicon_tags_31 },
    { 0x2ea128da, SUC_lexicon_tags_11 },
    { 0xacc128d8, SUC_lexicon_tags_29 },
    { 0x24c128dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ea128de, SUC_lexicon_tags_23 },
    { 0xde6128df, SUC_lexicon_tags_7 },
    { 0xfe2128de, SUC_lexicon_tags_25 },
    { 0xb78128e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x816128e3, SUC_lexicon_tags_60 },
    { 0x210128e4, SUC_lexicon_tags_10 },
    { 0xde0128e5, SUC_lexicon_tags_45 },
    { 0xfa8128e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc128eb, SUC_lexicon_tags_7 },
    { 0x884128ec, SUC_lexicon_tags_11 },
    { 0x7a2128ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa128f2, SUC_lexicon_tags_17 },
    { 0x7d2128f2, SUC_lexicon_tags_33 },
    { 0x258128f4, SUC_lexicon_tags_16 },
    { 0xa14128f3, SUC_lexicon_tags_14 },
    { 0x37c128f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8128f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8128fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f012900, SUC_lexicon_tags_11 },
    { 0xa1812901, SUC_lexicon_tags_43 },
    { 0x70a12902, SUC_lexicon_tags_10 },
    { 0x79012903, SUC_lexicon_tags_11 },
    { 0xc0212904, SUC_lexicon_tags_30 },
    { 0x7ae12904, SUC_lexicon_tags_8 },
    { 0xc6612906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c12909, SUC_lexicon_tags_65 },
    { 0x86c1290a, SUC_lexicon_tags_37 },
    { 0x67012909, SUC_lexicon_tags_17 },
    { 0x7541290c, SUC_lexicon_tags_25 },
    { 0x521290a, SUC_lexicon_tags_7 },
    { 0xdda1290e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a12914, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e1291c, SUC_lexicon_tags_18 },
    { 0x1ee1291d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e12924, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2a12926, SUC_lexicon_tags_6 },
    { 0xb612927, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4f212929, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdae1292b, SUC_lexicon_tags_7 },
    { 0xe9e1292b, SUC_lexicon_tags_17 },
    { 0x9b21292d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9821292f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99412931, SUC_lexicon_tags_37 },
    { 0xe8e12932, SUC_lexicon_tags_26 },
    { 0x78e12933, SUC_lexicon_tags_28 },
    { 0xc4212933, SUC_lexicon_tags_19 },
    { 0x27c12935, SUC_lexicon_tags_37 },
    { 0x32212935, SUC_lexicon_tags_37 },
    { 0x83412937, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x93412939, SUC_lexicon_tags_37 },
    { 0x44412939, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc12940, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15012942, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97812946, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73412948, SUC_lexicon_tags_17 },
    { 0xc012948, SUC_lexicon_tags_31 },
    { 0xa861294a, SUC_lexicon_tags_37 },
    { 0x1941294a, SUC_lexicon_tags_12 },
    { 0xd6e1294b, SUC_lexicon_tags_9 },
    { 0xe5e1294d, SUC_lexicon_tags_1 },
    { 0xb741294b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x36212950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa612953, SUC_lexicon_tags_15 },
    { 0x73a12954, SUC_lexicon_tags_7 },
    { 0x14012955, SUC_lexicon_tags_6 },
    { 0x52412954, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61295c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb612960, SUC_lexicon_tags_55 },
    { 0xe5212961, SUC_lexicon_tags_15 },
    { 0xf0e12962, SUC_lexicon_tags_37 },
    { 0xd7212960, SUC_lexicon_tags_3 },
    { 0x7d612961, SUC_lexicon_tags_48 },
    { 0x7d212965, SUC_lexicon_tags_0 },
    { 0xeee12962, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x51612969, SUC_lexicon_tags_7 },
    { 0x44612969, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xef21296c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x38c1296e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a12972, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4bc12974, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88a12976, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab212978, SUC_lexicon_tags_44 },
    { 0x36212979, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf821297e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece12982, SUC_lexicon_tags_26 },
    { 0x27a12983, SUC_lexicon_tags_17 },
    { 0x8e812982, SUC_lexicon_tags_14 },
    { 0xc5812985, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2801298a, SUC_lexicon_tags_11 },
    { 0x7ee1298b, SUC_lexicon_tags_31 },
    { 0x5d01298c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22812990, SUC_lexicon_tags_46 },
    { 0x24412990, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a212994, SUC_lexicon_tags_37 },
    { 0x4a212994, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e1299b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaec1299d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8129ac, SUC_lexicon_tags_138 },
    { 0x400129ac, SUC_lexicon_tags_47 },
    { 0x72e129ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x362129b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x42a129b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc129b7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac129ba, SUC_lexicon_tags_15 },
    { 0x6a129bb, SUC_lexicon_tags_37 },
    { 0x4ee129bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb98129be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac129c5, SUC_lexicon_tags_2 },
    { 0xeae129c5, SUC_lexicon_tags_1 },
    { 0xd4a129c7, SUC_lexicon_tags_39 },
    { 0x76e129c8, SUC_lexicon_tags_17 },
    { 0xee129c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde129ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2129d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34129d8, SUC_lexicon_tags_15 },
    { 0x48c129d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a129dc, SUC_lexicon_tags_971 },
    { 0x8bc129dd, SUC_lexicon_tags_4 },
    { 0x636129dd, SUC_lexicon_tags_29 },
    { 0xc14129dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c129e4, SUC_lexicon_tags_77 },
    { 0x5f2129e5, SUC_lexicon_tags_6 },
    { 0xe6a129e5, SUC_lexicon_tags_33 },
    { 0x7a2129e5, SUC_lexicon_tags_30 },
    { 0x710129e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112129ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x100129ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x834129f1, SUC_lexicon_tags_0 },
    { 0x960129f2, SUC_lexicon_tags_10 },
    { 0x174129f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e129f6, SUC_lexicon_tags_6 },
    { 0x86e129f7, SUC_lexicon_tags_25 },
    { 0xb4a129f8, SUC_lexicon_tags_9 },
    { 0x9ca129f8, SUC_lexicon_tags_9 },
    { 0x65a129f9, SUC_lexicon_tags_13 },
    { 0x514129fb, SUC_lexicon_tags_35 },
    { 0xefa129fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x114129fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa12a01, SUC_lexicon_tags_7 },
    { 0x87612a01, SUC_lexicon_tags_84 },
    { 0x68412a03, SUC_lexicon_tags_0 },
    { 0x28612a01, SUC_lexicon_tags_25 },
    { 0x1fe12a05, SUC_lexicon_tags_46 },
    { 0xc5a12a06, SUC_lexicon_tags_0 },
    { 0x1e212a06, SUC_lexicon_tags_11 },
    { 0xd1412a01, SUC_lexicon_tags_45 },
    { 0x5412a09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3612a0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaee12a0f, SUC_lexicon_tags_2 },
    { 0x4de12a0f, SUC_lexicon_tags_56 },
    { 0xca012a11, SUC_lexicon_tags_15 },
    { 0x70012a12, SUC_lexicon_tags_11 },
    { 0x1d812a12, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb5c12a15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8812a19, SUC_lexicon_tags_28 },
    { 0xb3e12a1a, SUC_lexicon_tags_28 },
    { 0x4a12a1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2412a20, SUC_lexicon_tags_30 },
    { 0x17212a21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xad212a23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0a12a25, SUC_lexicon_tags_6 },
    { 0xb812a26, SUC_lexicon_tags_15 },
    { 0xd9a12a27, SUC_lexicon_tags_29 },
    { 0xf4a12a28, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xc012a2a, SUC_lexicon_tags_10 },
    { 0x97a12a2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a212a2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2812a30, SUC_lexicon_tags_19 },
    { 0x43812a31, SUC_lexicon_tags_15 },
    { 0xaa612a31, SUC_lexicon_tags_8 },
    { 0xbf412a30, SUC_lexicon_tags_10 },
    { 0x2ba12a34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ce12a36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1612a3b, SUC_lexicon_tags_11 },
    { 0x51212a3c, SUC_lexicon_tags_16 },
    { 0x6412a3d, SUC_lexicon_tags_2 },
    { 0xfe412a3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb412a41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda012a46, SUC_lexicon_tags_11 },
    { 0xcba12a47, SUC_lexicon_tags_48 },
    { 0x3ba12a47, SUC_lexicon_tags_4 },
    { 0xb9012a46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e12a4d, SUC_lexicon_tags_10 },
    { 0xe9e12a4d, SUC_lexicon_tags_25 },
    { 0x2be12a4e, SUC_lexicon_tags_33 },
    { 0xe4412a4e, SUC_lexicon_tags_45 },
    { 0x99c12a4f, SUC_lexicon_tags_1 },
    { 0xeda12a52, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaac12a54, SUC_lexicon_tags_17 },
    { 0x34012a54, SUC_lexicon_tags_7 },
    { 0x9012a56, SUC_lexicon_tags_32 },
    { 0x4cc12a57, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xba012a5a, SUC_lexicon_tags_27 },
    { 0xd612a5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50212a62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1f612a64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92812a6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4412a70, SUC_lexicon_tags_12 },
    { 0x62a12a70, SUC_lexicon_tags_1 },
    { 0xa5812a72, SUC_lexicon_tags_14 },
    { 0x73e12a72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba12a77, SUC_lexicon_tags_63 },
    { 0x66212a78, SUC_lexicon_tags_57 },
    { 0x8a12a78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c12a7c, SUC_lexicon_tags_12 },
    { 0x62412a7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2c12a7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbde12a81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7812a84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b212a86, SUC_lexicon_tags_57 },
    { 0x96812a86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd9a12a89, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe212a8c, SUC_lexicon_tags_49 },
    { 0x81212a8c, SUC_lexicon_tags_17 },
    { 0x44212a8d, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d812a95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c212a9c, SUC_lexicon_tags_17 },
    { 0xea812a9d, SUC_lexicon_tags_49 },
    { 0xc4812a9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a12aa1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc412aa7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e12aab, SUC_lexicon_tags_12 },
    { 0x6bc12aac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x96e12aae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c12ab5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a12aba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812abf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce212acd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x51612acf, SUC_lexicon_tags_27 },
    { 0x97012ad0, SUC_lexicon_tags_18 },
    { 0xcb412ad1, SUC_lexicon_tags_79 },
    { 0x42612ad0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda12ad5, SUC_lexicon_tags_46 },
    { 0x96a12ad6, SUC_lexicon_tags_37 },
    { 0x4dc12ad6, SUC_lexicon_tags_6 },
    { 0xc3612ad8, SUC_lexicon_tags_25 },
    { 0x47012ad6, SUC_lexicon_tags_10 },
    { 0xa3612ad7, SUC_lexicon_tags_6 },
    { 0xa1812ad7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x52812ade, SUC_lexicon_tags_25 },
    { 0xd2012adf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c12ae4, SUC_lexicon_tags_34 },
    { 0xe012ae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6212ae9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a12aec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9b212aee, SUC_lexicon_tags_32 },
    { 0x3812aef, SUC_lexicon_tags_23 },
    { 0xc3412aef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0c12af2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8cc12af4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a12af7, SUC_lexicon_tags_1 },
    { 0x68c12af7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b012afa, SUC_lexicon_tags_7 },
    { 0x28e12afa, SUC_lexicon_tags_1593 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9412b00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x21c12b02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd5c12b04, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x4fc12b06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xad012b08, SUC_lexicon_tags_9 },
    { 0x53c12b09, SUC_lexicon_tags_25 },
    { 0xc8012b0a, SUC_lexicon_tags_32 },
    { 0x6aa12b0a, SUC_lexicon_tags_17 },
    { 0xb812b0c, SUC_lexicon_tags_33 },
    { 0x87a12b0b, SUC_lexicon_tags_24 },
    { 0x5a612b0a, SUC_lexicon_tags_0 },
    { 0x86412b0f, SUC_lexicon_tags_37 },
    { 0x20612b10, SUC_lexicon_tags_7 },
    { 0xcb612b11, SUC_lexicon_tags_4 },
    { 0x9ba12b12, SUC_lexicon_tags_29 },
    { 0x32c12b12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6812b16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x80412b18, SUC_lexicon_tags_65 },
    { 0xde012b19, SUC_lexicon_tags_27 },
    { 0x66c12b1a, SUC_lexicon_tags_1 },
    { 0xd5c12b1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec12b1e, SUC_lexicon_tags_57 },
    { 0xa1412b1f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa4e12b21, SUC_lexicon_tags_37 },
    { 0x36812b22, SUC_lexicon_tags_19 },
    { 0x35e12b23, SUC_lexicon_tags_17 },
    { 0xa412b21, SUC_lexicon_tags_15 },
    { 0xdc412b25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58012b34, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c12b37, SUC_lexicon_tags_63 },
    { 0x6012b38, SUC_lexicon_tags_11 },
    { 0xdfa12b39, SUC_lexicon_tags_87 },
    { 0x31212b3a, SUC_lexicon_tags_63 },
    { 0x50c12b3b, SUC_lexicon_tags_14 },
    { 0xeb412b3c, SUC_lexicon_tags_6 },
    { 0x40812b3d, SUC_lexicon_tags_71 },
    { 0x7fe12b3c, SUC_lexicon_tags_39 },
    { 0x63412b3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34412b41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe7e12b43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c12b47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92012b4c, SUC_lexicon_tags_20 },
    { 0xbe12b4c, SUC_lexicon_tags_7 },
    { 0xbe212b4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54412b51, SUC_lexicon_tags_0 },
    { 0xae612b52, SUC_lexicon_tags_14 },
    { 0x55e12b53, SUC_lexicon_tags_25 },
    { 0x52812b54, SUC_lexicon_tags_1 },
    { 0x19a12b52, SUC_lexicon_tags_8 },
    { 0xec812b56, SUC_lexicon_tags_85 },
    { 0xf9212b56, SUC_lexicon_tags_49 },
    { 0x4fa12b56, SUC_lexicon_tags_37 },
    { 0x2f212b58, SUC_lexicon_tags_87 },
    { 0x8f612b5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3ce12b5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4212b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce12b62, SUC_lexicon_tags_7 },
    { 0x2b412b63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c812b67, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c12b6b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc12b6e, SUC_lexicon_tags_113 },
    { 0xefc12b6f, SUC_lexicon_tags_14 },
    { 0xd5212b70, SUC_lexicon_tags_1 },
    { 0x77812b6f, SUC_lexicon_tags_25 },
    { 0xd1012b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44612b78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3812b7c, SUC_lexicon_tags_46 },
    { 0xe8012b7c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc0e12b7f, SUC_lexicon_tags_267 },
    { 0xcbc12b7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe12b82, SUC_lexicon_tags_72 },
    { 0x26612b82, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe012b85, SUC_lexicon_tags_83 },
    { 0x9bc12b86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9812b89, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72612b8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29012b91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e12b94, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a12b98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97012ba1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7612ba4, SUC_lexicon_tags_46 },
    { 0x16a12ba5, SUC_lexicon_tags_26 },
    { 0x6e612ba6, SUC_lexicon_tags_2 },
    { 0x77612ba7, SUC_lexicon_tags_30 },
    { 0xb0612ba4, SUC_lexicon_tags_24 },
    { 0x5ca12ba9, SUC_lexicon_tags_1 },
    { 0x70412ba7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3b812bac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5b012bae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4612bb1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2212bb7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f612bbc, SUC_lexicon_tags_47 },
    { 0xfc212bbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59e12bbf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c412bc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0612bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf612bc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e012bcc, SUC_lexicon_tags_12 },
    { 0xadc12bcd, SUC_lexicon_tags_1093 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7012bd2, SUC_lexicon_tags_8 },
    { 0x4012bd3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xba612bd6, SUC_lexicon_tags_27 },
    { 0xc9012bd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae12bda, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab012bde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5412be2, SUC_lexicon_tags_31 },
    { 0x45812be3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6212bf2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe012bf5, SUC_lexicon_tags_12 },
    { 0x96812bf6, SUC_lexicon_tags_38 },
    { 0x4c812bf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f212bfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8412bfd, SUC_lexicon_tags_11 },
    { 0x19c12bfe, SUC_lexicon_tags_11 },
    { 0xaf012bff, SUC_lexicon_tags_11 },
    { 0xbd212bff, SUC_lexicon_tags_11 },
    { 0x27212bfd, SUC_lexicon_tags_17 },
    { 0x67412c02, SUC_lexicon_tags_30 },
    { 0x9e812c02, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2812c07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xada12c0a, SUC_lexicon_tags_60 },
    { 0xcbc12c0b, SUC_lexicon_tags_57 },
    { 0x91612c0b, SUC_lexicon_tags_11 },
    { 0x7ca12c0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ca12c0f, SUC_lexicon_tags_30 },
    { 0xfc812c10, SUC_lexicon_tags_22 },
    { 0x2012c11, SUC_lexicon_tags_25 },
    { 0xee12c11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1be12c14, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5f212c16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x57812c19, SUC_lexicon_tags_49 },
    { 0x8012c1a, SUC_lexicon_tags_33 },
    { 0xd0212c19, SUC_lexicon_tags_37 },
    { 0xc12c1c, SUC_lexicon_tags_17 },
    { 0x44e12c1b, SUC_lexicon_tags_54 },
    { 0xce012c1d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65812c24, SUC_lexicon_tags_17 },
    { 0x95012c24, SUC_lexicon_tags_63 },
    { 0xb6e12c26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c012c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90212c2e, SUC_lexicon_tags_37 },
    { 0x69812c2f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x29812c31, SUC_lexicon_tags_26 },
    { 0x39612c31, SUC_lexicon_tags_27 },
    { 0x56812c33, SUC_lexicon_tags_19 },
    { 0x43e12c34, SUC_lexicon_tags_10 },
    { 0xe7412c35, SUC_lexicon_tags_2 },
    { 0xc2612c32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf5812c38, SUC_lexicon_tags_25 },
    { 0x41a12c39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeec12c3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90812c40, SUC_lexicon_tags_65 },
    { 0x44412c40, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e212c44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18c12c46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x38c12c48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b012c4a, SUC_lexicon_tags_30 },
    { 0x9aa12c4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e12c51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22212c55, SUC_lexicon_tags_1 },
    { 0xeea12c56, SUC_lexicon_tags_7 },
    { 0x6c812c57, SUC_lexicon_tags_151 },
    { 0x29a12c57, SUC_lexicon_tags_2 },
    { 0xaf812c58, SUC_lexicon_tags_31 },
    { 0x52412c5a, SUC_lexicon_tags_60 },
    { 0x80812c5a, SUC_lexicon_tags_27 },
    { 0x10412c5c, SUC_lexicon_tags_15 },
    { 0x78012c5d, SUC_lexicon_tags_30 },
    { 0x66612c5e, SUC_lexicon_tags_29 },
    { 0x2212c5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e012c63, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e12c68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe12c6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac412c6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2a12c70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6612c72, SUC_lexicon_tags_1 },
    { 0x94e12c73, SUC_lexicon_tags_68 },
    { 0xe6a12c74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6c12c76, SUC_lexicon_tags_27 },
    { 0xe2812c77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85212c7a, SUC_lexicon_tags_49 },
    { 0x6d212c7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde12c82, SUC_lexicon_tags_26 },
    { 0x1f812c82, SUC_lexicon_tags_4 },
    { 0xa4e12c84, SUC_lexicon_tags_19 },
    { 0xd0812c85, SUC_lexicon_tags_8 },
    { 0xaae12c86, SUC_lexicon_tags_48 },
    { 0xed612c86, SUC_lexicon_tags_0 },
    { 0xe3c12c88, SUC_lexicon_tags_4 },
    { 0xb7c12c89, SUC_lexicon_tags_37 },
    { 0x3fe12c89, SUC_lexicon_tags_17 },
    { 0x1ca12c8b, SUC_lexicon_tags_37 },
    { 0xc5612c86, SUC_lexicon_tags_15 },
    { 0x52012c8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x72c12c8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86a12c91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a12c96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc12c9b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3012c9e, SUC_lexicon_tags_56 },
    { 0x9812c9f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77612ca3, SUC_lexicon_tags_60 },
    { 0xdfe12ca3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a012ca7, SUC_lexicon_tags_19 },
    { 0x1c012ca8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc612caa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97812cb3, SUC_lexicon_tags_45 },
    { 0xe5812cb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76c12cb6, SUC_lexicon_tags_68 },
    { 0x27812cb7, SUC_lexicon_tags_57 },
    { 0x38212cb8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca12cbb, SUC_lexicon_tags_15 },
    { 0xac212cbc, SUC_lexicon_tags_25 },
    { 0x8d612cbd, SUC_lexicon_tags_151 },
    { 0x45212cbc, SUC_lexicon_tags_11 },
    { 0x51812cbf, SUC_lexicon_tags_9 },
    { 0xf8412cc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c12cc7, SUC_lexicon_tags_2 },
    { 0x10812cc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8612ccb, SUC_lexicon_tags_29 },
    { 0xe7a12ccc, SUC_lexicon_tags_43 },
    { 0x3b612ccd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a12cd6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x31012cd8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b812cdd, SUC_lexicon_tags_2 },
    { 0xfbc12cde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8612ce3, SUC_lexicon_tags_16 },
    { 0x45412ce4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e212ce8, SUC_lexicon_tags_9 },
    { 0xe8812ce9, SUC_lexicon_tags_17 },
    { 0x24412ce8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2d012cec, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x2cc12cee, SUC_lexicon_tags_2 },
    { 0xabe12cef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ce12cf1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9d212cf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf012cf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce12cfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d612d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47a12d07, SUC_lexicon_tags_33 },
    { 0x1b412d08, SUC_lexicon_tags_7 },
    { 0xc0e12d09, SUC_lexicon_tags_82 },
    { 0x6f412d09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c12d0e, SUC_lexicon_tags_17 },
    { 0xac12d0e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c12d15, SUC_lexicon_tags_56 },
    { 0xfce12d15, SUC_lexicon_tags_30 },
    { 0x6a012d17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ba12d19, SUC_lexicon_tags_42 },
    { 0xb3612d19, SUC_lexicon_tags_25 },
    { 0xcea12d19, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f612d1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9812d20, SUC_lexicon_tags_65 },
    { 0xcfa12d21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e812d29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa12d30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x77e12d32, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42012d38, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e12d3b, SUC_lexicon_tags_38 },
    { 0xa7212d3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de12d40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x49412d42, SUC_lexicon_tags_7 },
    { 0x16412d43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8412d45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55812d47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc12d4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5612d4f, SUC_lexicon_tags_11 },
    { 0xe1c12d50, SUC_lexicon_tags_39 },
    { 0xf4a12d51, SUC_lexicon_tags_11 },
    { 0x62212d52, SUC_lexicon_tags_55 },
    { 0x8d812d51, SUC_lexicon_tags_19 },
    { 0xb9612d51, SUC_lexicon_tags_65 },
    { 0x78a12d55, SUC_lexicon_tags_33 },
    { 0xf3012d56, SUC_lexicon_tags_28 },
    { 0xc012d53, SUC_lexicon_tags_17 },
    { 0xec212d55, SUC_lexicon_tags_15 },
    { 0x29212d59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e12d61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de12d64, SUC_lexicon_tags_47 },
    { 0x12812d64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x67412d68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b012d6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90c12d6c, SUC_lexicon_tags_60 },
    { 0xc1a12d6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49412d70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbaa12d72, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d612d76, SUC_lexicon_tags_37 },
    { 0x8aa12d76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a612d79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34612d7b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81212d81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4812d84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1012d87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xca12d8a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e12d8d, SUC_lexicon_tags_17 },
    { 0x8d412d8e, SUC_lexicon_tags_82 },
    { 0xd3012d8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a12d92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfca12d94, SUC_lexicon_tags_17 },
    { 0x3812d94, SUC_lexicon_tags_15 },
    { 0x2c12d96, SUC_lexicon_tags_37 },
    { 0xa4a12d94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65212d99, SUC_lexicon_tags_60 },
    { 0x29212d99, SUC_lexicon_tags_80 },
    { 0x15a12d9b, SUC_lexicon_tags_7 },
    { 0xf6c12d9c, SUC_lexicon_tags_2 },
    { 0x80012d9d, SUC_lexicon_tags_1 },
    { 0xea612d9c, SUC_lexicon_tags_0 },
    { 0x7012d9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb412da4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a12dab, SUC_lexicon_tags_1184 },
    { 0x51212dac, SUC_lexicon_tags_37 },
    { 0xf4012dad, SUC_lexicon_tags_47 },
    { 0x98a12dac, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a212db3, SUC_lexicon_tags_33 },
    { 0x8aa12db3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a12db7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33612db9, SUC_lexicon_tags_15 },
    { 0x8e612dba, SUC_lexicon_tags_45 },
    { 0x12dbb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xaec12dbd, SUC_lexicon_tags_6 },
    { 0x8ca12dbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19812dc0, SUC_lexicon_tags_49 },
    { 0xc6a12dc0, SUC_lexicon_tags_10 },
    { 0xb3a12dc2, SUC_lexicon_tags_47 },
    { 0x6a812dc3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36412dc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xccc12dc9, SUC_lexicon_tags_7 },
    { 0xddc12dc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea812dcc, SUC_lexicon_tags_0 },
    { 0x36e12dcd, SUC_lexicon_tags_37 },
    { 0x50a12dcd, SUC_lexicon_tags_15 },
    { 0x30412dcd, SUC_lexicon_tags_15 },
    { 0x84612dcc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x16412dd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc812dd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53412ddc, SUC_lexicon_tags_1 },
    { 0x96612ddd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf212de4, SUC_lexicon_tags_14 },
    { 0xd2012de5, SUC_lexicon_tags_37 },
    { 0xf5a12de4, SUC_lexicon_tags_31 },
    { 0xddc12de7, SUC_lexicon_tags_15 },
    { 0xac412de8, SUC_lexicon_tags_47 },
    { 0x10c12de8, SUC_lexicon_tags_30 },
    { 0x2d212dea, SUC_lexicon_tags_14 },
    { 0x2e012deb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66812dee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdd012df0, SUC_lexicon_tags_45 },
    { 0xc2012df1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x15612df4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9012e03, SUC_lexicon_tags_12 },
    { 0xba412e04, SUC_lexicon_tags_2 },
    { 0xe6612e04, SUC_lexicon_tags_63 },
    { 0x60612e06, SUC_lexicon_tags_11 },
    { 0x6ba12e06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae12e0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae812e10, SUC_lexicon_tags_24 },
    { 0x9bc12e10, SUC_lexicon_tags_38 },
    { 0xb0012e12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea12e14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1a12e16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e12e19, SUC_lexicon_tags_14 },
    { 0x14812e19, SUC_lexicon_tags_10 },
    { 0xfaa12e1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a12e27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b012e29, SUC_lexicon_tags_37 },
    { 0xd6812e2a, SUC_lexicon_tags_25 },
    { 0xf9412e2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf0612e2d, SUC_lexicon_tags_32 },
    { 0xc6212e2e, SUC_lexicon_tags_275 },
    { 0xc3612e2f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6212e33, SUC_lexicon_tags_33 },
    { 0xf8412e34, SUC_lexicon_tags_35 },
    { 0xa5c12e35, SUC_lexicon_tags_12 },
    { 0xeda12e33, SUC_lexicon_tags_27 },
    { 0xd0612e37, SUC_lexicon_tags_5 },
    { 0x5f812e37, SUC_lexicon_tags_23 },
    { 0x21a12e39, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45012e3d, SUC_lexicon_tags_12 },
    { 0x2d212e3e, SUC_lexicon_tags_7 },
    { 0x7be12e3f, SUC_lexicon_tags_43 },
    { 0xd5212e3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x87412e43, SUC_lexicon_tags_15 },
    { 0x49012e44, SUC_lexicon_tags_37 },
    { 0x93812e45, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa12e48, SUC_lexicon_tags_57 },
    { 0xd4a12e49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c612e4c, SUC_lexicon_tags_17 },
    { 0x4d612e4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a12e51, SUC_lexicon_tags_3 },
    { 0xede12e52, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c12e56, SUC_lexicon_tags_7 },
    { 0x16812e57, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xde812e59, SUC_lexicon_tags_37 },
    { 0x51212e5a, SUC_lexicon_tags_17 },
    { 0xe4812e5b, SUC_lexicon_tags_47 },
    { 0x50c12e5c, SUC_lexicon_tags_17 },
    { 0x1412e5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c12e63, SUC_lexicon_tags_2 },
    { 0x69e12e64, SUC_lexicon_tags_1 },
    { 0x4a412e65, SUC_lexicon_tags_1 },
    { 0x65212e63, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x29612e69, SUC_lexicon_tags_31 },
    { 0x38c12e69, SUC_lexicon_tags_1 },
    { 0xa7e12e6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a12e72, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde12e75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce12e77, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x99412e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7212e7d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb5212e7f, SUC_lexicon_tags_47 },
    { 0xbfc12e7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da12e83, SUC_lexicon_tags_11 },
    { 0x4a412e84, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4f012e86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45012e8c, SUC_lexicon_tags_25 },
    { 0xd1012e8d, SUC_lexicon_tags_28 },
    { 0xe2212e8e, SUC_lexicon_tags_17 },
    { 0x8212e8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd0212e91, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x33612e93, SUC_lexicon_tags_34 },
    { 0xd2e12e94, SUC_lexicon_tags_2 },
    { 0xaf812e95, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c12e98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ae12e9a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x11612e9c, SUC_lexicon_tags_1 },
    { 0x39e12e9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a012e9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce12ea3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe812ea8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe9012eaa, SUC_lexicon_tags_31 },
    { 0xa0412eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79212ead, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e12eb3, SUC_lexicon_tags_12 },
    { 0xade12eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17412eb8, SUC_lexicon_tags_2 },
    { 0x89612eb9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c12ec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c12ec7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x98e12ec9, SUC_lexicon_tags_48 },
    { 0xf6212eca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc212ed1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6012ed4, SUC_lexicon_tags_37 },
    { 0x39e12ed5, SUC_lexicon_tags_7 },
    { 0xdda12ed6, SUC_lexicon_tags_31 },
    { 0x19412ed7, SUC_lexicon_tags_17 },
    { 0x4e812ed5, SUC_lexicon_tags_2 },
    { 0x66412ed9, SUC_lexicon_tags_49 },
    { 0xec412ed8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee212edc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xad812edf, SUC_lexicon_tags_12 },
    { 0x10e12edf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde212ee2, SUC_lexicon_tags_7 },
    { 0x79212ee3, SUC_lexicon_tags_25 },
    { 0xd0812ee2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98212ee7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc612eeb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x94212eed, SUC_lexicon_tags_15 },
    { 0x93212eee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xddc12ef0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb6212ef2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2a12ef4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c12ef8, SUC_lexicon_tags_11 },
    { 0x2ce12ef9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb612efc, SUC_lexicon_tags_21 },
    { 0x9d412efd, SUC_lexicon_tags_40 },
    { 0xa6212efe, SUC_lexicon_tags_2 },
    { 0xcc412eff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x66212f01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbce12f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be12f06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9412f08, SUC_lexicon_tags_11 },
    { 0xba012f08, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c12f12, SUC_lexicon_tags_23 },
    { 0x60012f13, SUC_lexicon_tags_15 },
    { 0x23a12f14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fc12f16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x36c12f18, SUC_lexicon_tags_17 },
    { 0x78612f18, SUC_lexicon_tags_48 },
    { 0x79612f1a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae12f20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27412f2b, SUC_lexicon_tags_15 },
    { 0x92412f2c, SUC_lexicon_tags_11 },
    { 0x18412f2c, SUC_lexicon_tags_1 },
    { 0x8a12f2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19a12f30, SUC_lexicon_tags_4 },
    { 0x8c012f31, SUC_lexicon_tags_25 },
    { 0xff812f31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x68012f35, SUC_lexicon_tags_37 },
    { 0xfb612f36, SUC_lexicon_tags_60 },
    { 0xb8212f37, SUC_lexicon_tags_2 },
    { 0xd0a12f37, SUC_lexicon_tags_12 },
    { 0xedc12f36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde412f3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb212f3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a212f3f, SUC_lexicon_tags_30 },
    { 0xcd612f40, SUC_lexicon_tags_161 },
    { 0xa7212f41, SUC_lexicon_tags_15 },
    { 0xd3a12f42, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a12f45, SUC_lexicon_tags_11 },
    { 0xb3612f45, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1ec12f48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be12f4b, SUC_lexicon_tags_341 },
    { 0x11012f4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11012f50, SUC_lexicon_tags_14 },
    { 0x4f412f51, SUC_lexicon_tags_8 },
    { 0x85a12f52, SUC_lexicon_tags_1 },
    { 0x93012f53, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec412f5d, SUC_lexicon_tags_49 },
    { 0xf8012f5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f812f62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d012f64, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b212f6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x24c12f6f, SUC_lexicon_tags_25 },
    { 0x9d212f6f, SUC_lexicon_tags_11 },
    { 0x26612f71, SUC_lexicon_tags_85 },
    { 0x18412f72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x42612f74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90612f78, SUC_lexicon_tags_3 },
    { 0xdf412f78, SUC_lexicon_tags_11 },
    { 0x3d212f7a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x83612f7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea12f87, SUC_lexicon_tags_23 },
    { 0x1bc12f88, SUC_lexicon_tags_11 },
    { 0x6c012f88, SUC_lexicon_tags_25 },
    { 0xa8812f8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0612f90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc412f97, SUC_lexicon_tags_207 },
    { 0xf2c12f97, SUC_lexicon_tags_37 },
    { 0xc8212f99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d012f9e, SUC_lexicon_tags_25 },
    { 0x7e212f9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c12fa9, SUC_lexicon_tags_68 },
    { 0xf8a12faa, SUC_lexicon_tags_17 },
    { 0xb8212fab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e12fb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c812fb3, SUC_lexicon_tags_1 },
    { 0xad012fb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4012fb6, SUC_lexicon_tags_39 },
    { 0x7fe12fb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22812fbd, SUC_lexicon_tags_56 },
    { 0x87812fbd, SUC_lexicon_tags_15 },
    { 0xb3a12fbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f012fc2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32812fc6, SUC_lexicon_tags_1 },
    { 0xd6412fc7, SUC_lexicon_tags_28 },
    { 0x4f012fc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d812fca, SUC_lexicon_tags_11 },
    { 0x41412fcb, SUC_lexicon_tags_56 },
    { 0xcee12fcc, SUC_lexicon_tags_17 },
    { 0x2812fcd, SUC_lexicon_tags_15 },
    { 0x90812fcc, SUC_lexicon_tags_2 },
    { 0x75c12fcc, SUC_lexicon_tags_56 },
    { 0x13412fcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67612fd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x73812fd6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e12fd9, SUC_lexicon_tags_25 },
    { 0xb8812fda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62812fde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e12fe4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7212fe8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x69812feb, SUC_lexicon_tags_17 },
    { 0xffe12fec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc812fee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a812ff2, SUC_lexicon_tags_8 },
    { 0xae812ff3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19c12ff5, SUC_lexicon_tags_63 },
    { 0x5c812ff6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec612ff8, SUC_lexicon_tags_60 },
    { 0xee212ff8, SUC_lexicon_tags_48 },
    { 0xffa12ffa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47213000, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c13003, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a13007, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe81300d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea213010, SUC_lexicon_tags_10 },
    { 0xc3613010, SUC_lexicon_tags_25 },
    { 0x4f413010, SUC_lexicon_tags_13 },
    { 0xe1413012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f613016, SUC_lexicon_tags_2 },
    { 0x5aa13017, SUC_lexicon_tags_13 },
    { 0xf5a13018, SUC_lexicon_tags_28 },
    { 0x8ce13019, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7281301b, SUC_lexicon_tags_19 },
    { 0xad81301b, SUC_lexicon_tags_5 },
    { 0x2f01301d, SUC_lexicon_tags_49 },
    { 0x8d81301d, SUC_lexicon_tags_19 },
    { 0xa5a1301e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5013024, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f013028, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3081302b, SUC_lexicon_tags_7 },
    { 0xb3a1302c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1aa1302e, SUC_lexicon_tags_138 },
    { 0xad01302e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56613039, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb201303b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e41303e, SUC_lexicon_tags_13 },
    { 0xa061303f, SUC_lexicon_tags_61 },
    { 0xb9413040, SUC_lexicon_tags_12 },
    { 0x4be13041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c13048, SUC_lexicon_tags_62 },
    { 0xf9e13048, SUC_lexicon_tags_38 },
    { 0x8001304a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x541304c, SUC_lexicon_tags_45 },
    { 0xc81304c, SUC_lexicon_tags_1 },
    { 0x23e1304e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdea13050, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a13054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46813059, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4201305b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2ba1305d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a13066, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9813069, SUC_lexicon_tags_1 },
    { 0xf6c13069, SUC_lexicon_tags_1 },
    { 0x4b613069, SUC_lexicon_tags_28 },
    { 0xe421306c, SUC_lexicon_tags_8 },
    { 0x8441306c, SUC_lexicon_tags_63 },
    { 0x94e1306e, SUC_lexicon_tags_12 },
    { 0xa7c13069, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8d613071, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d213073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e13078, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x84c1307a, SUC_lexicon_tags_12 },
    { 0x24c1307b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66e1307d, SUC_lexicon_tags_10 },
    { 0x9761307e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd413082, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e13086, SUC_lexicon_tags_37 },
    { 0x87c13087, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1541308a, SUC_lexicon_tags_45 },
    { 0xd741308b, SUC_lexicon_tags_15 },
    { 0x1981308c, SUC_lexicon_tags_17 },
    { 0xbce1308a, SUC_lexicon_tags_38 },
    { 0x2a41308d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf613092, SUC_lexicon_tags_66 },
    { 0xf413093, SUC_lexicon_tags_1 },
    { 0xcbe13093, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa013098, SUC_lexicon_tags_33 },
    { 0x8b413099, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96130a0, SUC_lexicon_tags_25 },
    { 0xd72130a1, SUC_lexicon_tags_10 },
    { 0xd42130a1, SUC_lexicon_tags_0 },
    { 0x79e130a2, SUC_lexicon_tags_2 },
    { 0x572130a4, SUC_lexicon_tags_15 },
    { 0x1cc130a5, SUC_lexicon_tags_15 },
    { 0x512130a5, SUC_lexicon_tags_5 },
    { 0x972130a7, SUC_lexicon_tags_11 },
    { 0x264130a6, SUC_lexicon_tags_33 },
    { 0xb84130a1, SUC_lexicon_tags_39 },
    { 0x4d2130a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe26130ac, SUC_lexicon_tags_17 },
    { 0x742130ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c130b2, SUC_lexicon_tags_7 },
    { 0xda0130b2, SUC_lexicon_tags_14 },
    { 0x1ea130b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a130b7, SUC_lexicon_tags_6 },
    { 0x78c130b8, SUC_lexicon_tags_40 },
    { 0x84130b9, SUC_lexicon_tags_31 },
    { 0xba2130b8, SUC_lexicon_tags_17 },
    { 0x4e4130b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e130c0, SUC_lexicon_tags_2 },
    { 0xeb4130c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6dc130c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92130c7, SUC_lexicon_tags_60 },
    { 0x4fc130c7, SUC_lexicon_tags_49 },
    { 0x708130c8, SUC_lexicon_tags_11 },
    { 0xb70130ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce130cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46130d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece130d8, SUC_lexicon_tags_9 },
    { 0x578130d9, SUC_lexicon_tags_26 },
    { 0x758130d8, SUC_lexicon_tags_7 },
    { 0x176130da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e130e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce4130e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4130e6, SUC_lexicon_tags_15 },
    { 0x264130e7, SUC_lexicon_tags_19 },
    { 0x6bc130e6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc130ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x490130f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0130f4, SUC_lexicon_tags_17 },
    { 0xe32130f5, SUC_lexicon_tags_12 },
    { 0x5ea130f6, SUC_lexicon_tags_9 },
    { 0xb98130f7, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xdbe130f9, SUC_lexicon_tags_12 },
    { 0x9d4130fa, SUC_lexicon_tags_1 },
    { 0x2dc130fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118130ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15e13101, SUC_lexicon_tags_292 },
    { 0x66c13101, SUC_lexicon_tags_49 },
    { 0x9c813102, SUC_lexicon_tags_1 },
    { 0x73813102, SUC_lexicon_tags_0 },
    { 0x1a213105, SUC_lexicon_tags_49 },
    { 0xf7c13105, SUC_lexicon_tags_919 },
    { 0x77813107, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4161310b, SUC_lexicon_tags_27 },
    { 0x9981310b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3461310e, SUC_lexicon_tags_65 },
    { 0xe681310e, SUC_lexicon_tags_25 },
    { 0x52213110, SUC_lexicon_tags_47 },
    { 0x85a13111, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x41813113, SUC_lexicon_tags_45 },
    { 0x3b013114, SUC_lexicon_tags_11 },
    { 0xd1213115, SUC_lexicon_tags_10 },
    { 0x8813114, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc881311b, SUC_lexicon_tags_15 },
    { 0x9501311c, SUC_lexicon_tags_63 },
    { 0xcd01311b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc13121, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf813123, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcee13125, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6121312b, SUC_lexicon_tags_11 },
    { 0x221312c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e213130, SUC_lexicon_tags_47 },
    { 0xcb413131, SUC_lexicon_tags_33 },
    { 0xbc413132, SUC_lexicon_tags_47 },
    { 0xd4613133, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a013135, SUC_lexicon_tags_4 },
    { 0x86c13135, SUC_lexicon_tags_40 },
    { 0x67013137, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7613139, SUC_lexicon_tags_29 },
    { 0x7281313a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xbe81313c, SUC_lexicon_tags_4 },
    { 0x5721313c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15413140, SUC_lexicon_tags_63 },
    { 0x7dc13141, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0213146, SUC_lexicon_tags_25 },
    { 0x6ba13147, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c1314b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41013150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46013152, SUC_lexicon_tags_1 },
    { 0xdb413153, SUC_lexicon_tags_47 },
    { 0xd6013153, SUC_lexicon_tags_1 },
    { 0xc5c13153, SUC_lexicon_tags_23 },
    { 0x9c613156, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6213159, SUC_lexicon_tags_29 },
    { 0x47a1315a, SUC_lexicon_tags_14 },
    { 0xc81315b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7413163, SUC_lexicon_tags_45 },
    { 0xd1013163, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc13167, SUC_lexicon_tags_17 },
    { 0x8a213167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e1316b, SUC_lexicon_tags_92 },
    { 0x9041316c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78613170, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c13173, SUC_lexicon_tags_14 },
    { 0x41613174, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8681317e, SUC_lexicon_tags_2 },
    { 0x5801317e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xebe13181, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe13184, SUC_lexicon_tags_27 },
    { 0x22e13185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69013188, SUC_lexicon_tags_25 },
    { 0x44e13188, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf21318b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e1318e, SUC_lexicon_tags_55 },
    { 0xdce1318e, SUC_lexicon_tags_8 },
    { 0xc2a13190, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x96413193, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae013198, SUC_lexicon_tags_19 },
    { 0xa7613198, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c61319f, SUC_lexicon_tags_49 },
    { 0x570131a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3be131a2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa42131a4, SUC_lexicon_tags_1 },
    { 0x424131a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f0131a7, SUC_lexicon_tags_1 },
    { 0x482131a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba131ae, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6131b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f2131b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372131ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcc131bc, SUC_lexicon_tags_0 },
    { 0xc8131bd, SUC_lexicon_tags_37 },
    { 0x6131be, SUC_lexicon_tags_198 },
    { 0xdbe131bd, SUC_lexicon_tags_1 },
    { 0x68131bd, SUC_lexicon_tags_3 },
    { 0xe76131c1, SUC_lexicon_tags_8 },
    { 0xfc2131c2, SUC_lexicon_tags_31 },
    { 0xe8c131bf, SUC_lexicon_tags_11 },
    { 0x7a8131c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc131c7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e131cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6131cf, SUC_lexicon_tags_37 },
    { 0x4de131d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x820131d2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x228131d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6ba131d6, SUC_lexicon_tags_6 },
    { 0x770131d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24131de, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e131e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc8131e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902131e8, SUC_lexicon_tags_12 },
    { 0xb2c131e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8131ef, SUC_lexicon_tags_37 },
    { 0x546131ef, SUC_lexicon_tags_12 },
    { 0x26e131f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6131fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd66131fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c13203, SUC_lexicon_tags_39 },
    { 0x91413204, SUC_lexicon_tags_37 },
    { 0x87013204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d813208, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1181320a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa281320c, SUC_lexicon_tags_49 },
    { 0x90a1320d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7ea1320f, SUC_lexicon_tags_17 },
    { 0x23a13210, SUC_lexicon_tags_39 },
    { 0x6fe13211, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb813213, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e13217, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f81321d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc13223, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c13227, SUC_lexicon_tags_15 },
    { 0xafc13228, SUC_lexicon_tags_12 },
    { 0x2e413229, SUC_lexicon_tags_1 },
    { 0xec613229, SUC_lexicon_tags_23 },
    { 0x6c613229, SUC_lexicon_tags_15 },
    { 0xea01322c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee13233, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x42613236, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1323b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada13247, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c1324d, SUC_lexicon_tags_25 },
    { 0xb41324e, SUC_lexicon_tags_29 },
    { 0xa81324f, SUC_lexicon_tags_1 },
    { 0x7c1324e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0213259, SUC_lexicon_tags_25 },
    { 0xd0a13259, SUC_lexicon_tags_14 },
    { 0xe1c1325b, SUC_lexicon_tags_1 },
    { 0x8541325c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbba1325e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec13261, SUC_lexicon_tags_23 },
    { 0x9413262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60a13264, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe9213266, SUC_lexicon_tags_11 },
    { 0x8413267, SUC_lexicon_tags_0 },
    { 0xe1013268, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd361326d, SUC_lexicon_tags_25 },
    { 0xf0c1326d, SUC_lexicon_tags_15 },
    { 0xcd01326f, SUC_lexicon_tags_37 },
    { 0x38e1326f, SUC_lexicon_tags_68 },
    { 0xd0813271, SUC_lexicon_tags_10 },
    { 0x6de13272, SUC_lexicon_tags_11 },
    { 0x9a413271, SUC_lexicon_tags_30 },
    { 0x8d813274, SUC_lexicon_tags_2 },
    { 0x8d413274, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e613278, SUC_lexicon_tags_11 },
    { 0xe5813279, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3501327c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e13283, SUC_lexicon_tags_87 },
    { 0x32413284, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d213287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ba13289, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa841328b, SUC_lexicon_tags_4 },
    { 0x1681328c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc1328f, SUC_lexicon_tags_4 },
    { 0x1c213290, SUC_lexicon_tags_15 },
    { 0x4e013291, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a13295, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7413298, SUC_lexicon_tags_11 },
    { 0x4c613298, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x121329b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd921329e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68132a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c132ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x298132ae, SUC_lexicon_tags_11 },
    { 0x1a4132af, SUC_lexicon_tags_11 },
    { 0x32e132b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea132b8, SUC_lexicon_tags_56 },
    { 0xbe132b9, SUC_lexicon_tags_13 },
    { 0x7e2132ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x638132bd, SUC_lexicon_tags_10 },
    { 0x37a132be, SUC_lexicon_tags_21 },
    { 0x13c132bf, SUC_lexicon_tags_26 },
    { 0x5cc132c0, SUC_lexicon_tags_29 },
    { 0x642132c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa20132c3, SUC_lexicon_tags_1 },
    { 0xdd6132c4, SUC_lexicon_tags_14 },
    { 0x88132c5, SUC_lexicon_tags_66 },
    { 0xd36132c6, SUC_lexicon_tags_23 },
    { 0xc86132c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1d0132c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868132ce, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x93a132d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12132d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a132d7, SUC_lexicon_tags_12 },
    { 0x5f4132d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4132db, SUC_lexicon_tags_44 },
    { 0xd36132db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776132e2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x306132e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e132ea, SUC_lexicon_tags_38 },
    { 0x7dc132eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d6132ed, SUC_lexicon_tags_45 },
    { 0x5d2132ee, SUC_lexicon_tags_12 },
    { 0x980132ef, SUC_lexicon_tags_61 },
    { 0x22132f0, SUC_lexicon_tags_13 },
    { 0xd72132ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13a132f3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed813300, SUC_lexicon_tags_65 },
    { 0x10c13300, SUC_lexicon_tags_45 },
    { 0x32613302, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c13307, SUC_lexicon_tags_37 },
    { 0xb6813307, SUC_lexicon_tags_27 },
    { 0x4a413308, SUC_lexicon_tags_12 },
    { 0x59e1330a, SUC_lexicon_tags_25 },
    { 0x39c13308, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7a1330d, SUC_lexicon_tags_60 },
    { 0xaae1330e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25213310, SUC_lexicon_tags_1 },
    { 0x9cc13311, SUC_lexicon_tags_7 },
    { 0x52413311, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9221331f, SUC_lexicon_tags_14 },
    { 0xec21331f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc13325, SUC_lexicon_tags_68 },
    { 0xd1013325, SUC_lexicon_tags_63 },
    { 0x59213327, SUC_lexicon_tags_21 },
    { 0xdcc13327, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5781332a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd381332c, SUC_lexicon_tags_53 },
    { 0x3d61332d, SUC_lexicon_tags_43 },
    { 0x1861332e, SUC_lexicon_tags_27 },
    { 0xbd21332f, SUC_lexicon_tags_11 },
    { 0xce61332f, SUC_lexicon_tags_28 },
    { 0xd1e13330, SUC_lexicon_tags_17 },
    { 0x4ca13332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84e13334, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9541333a, SUC_lexicon_tags_231 },
    { 0xd541333a, SUC_lexicon_tags_1 },
    { 0xe601333c, SUC_lexicon_tags_11 },
    { 0xad21333d, SUC_lexicon_tags_37 },
    { 0x86c1333d, SUC_lexicon_tags_23 },
    { 0xf441333b, SUC_lexicon_tags_14 },
    { 0x6f21333a, SUC_lexicon_tags_37 },
    { 0xc241333a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87e13343, SUC_lexicon_tags_1 },
    { 0x9213344, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23413347, SUC_lexicon_tags_37 },
    { 0x1ca13347, SUC_lexicon_tags_37 },
    { 0x9f613349, SUC_lexicon_tags_37 },
    { 0xf401334a, SUC_lexicon_tags_37 },
    { 0x3061334b, SUC_lexicon_tags_10 },
    { 0x7741334b, SUC_lexicon_tags_1 },
    { 0x57c1334a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87213350, SUC_lexicon_tags_9 },
    { 0xf413351, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0013354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7613356, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7d013358, SUC_lexicon_tags_23 },
    { 0x75c13358, SUC_lexicon_tags_1 },
    { 0x53e1335a, SUC_lexicon_tags_2 },
    { 0x7221335b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1ec1335d, SUC_lexicon_tags_31 },
    { 0xc3c1335e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e213366, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b613368, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc41336e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1613370, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae13373, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe9013375, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7613377, SUC_lexicon_tags_7 },
    { 0x24013377, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee01337d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9421337f, SUC_lexicon_tags_31 },
    { 0x18e13380, SUC_lexicon_tags_7 },
    { 0xdb413381, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec613388, SUC_lexicon_tags_65 },
    { 0x8fc13388, SUC_lexicon_tags_17 },
    { 0x2e81338a, SUC_lexicon_tags_37 },
    { 0xd381338b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd061338d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45c1338f, SUC_lexicon_tags_37 },
    { 0x89613390, SUC_lexicon_tags_1 },
    { 0x74c13391, SUC_lexicon_tags_60 },
    { 0xdbc13391, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e13399, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf301339b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61339f, SUC_lexicon_tags_37 },
    { 0x200133a0, SUC_lexicon_tags_21 },
    { 0xc82133a1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1c0133a3, SUC_lexicon_tags_79 },
    { 0xdb4133a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724133ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc48133ae, SUC_lexicon_tags_17 },
    { 0xdf8133af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a133b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a133b7, SUC_lexicon_tags_32 },
    { 0x6a8133b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe66133ba, SUC_lexicon_tags_37 },
    { 0x24c133ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2133bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x576133bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0133c2, SUC_lexicon_tags_14 },
    { 0xd62133c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x320133c5, SUC_lexicon_tags_0 },
    { 0x200133c6, SUC_lexicon_tags_10 },
    { 0x5ee133c5, SUC_lexicon_tags_66 },
    { 0xce2133c8, SUC_lexicon_tags_17 },
    { 0x208133c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x238133cb, SUC_lexicon_tags_37 },
    { 0x7ee133cc, SUC_lexicon_tags_21 },
    { 0xd9c133cd, SUC_lexicon_tags_10 },
    { 0x5f6133cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e133d1, SUC_lexicon_tags_87 },
    { 0xc6133d1, SUC_lexicon_tags_25 },
    { 0x30133d3, SUC_lexicon_tags_17 },
    { 0x4c133d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16133d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ec133db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5a2133dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x428133df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe54133e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798133e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8133e9, SUC_lexicon_tags_62 },
    { 0x2e2133ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x906133ec, SUC_lexicon_tags_577 },
    { 0, NULL },
    { 0xfc0133ee, SUC_lexicon_tags_17 },
    { 0x6da133ee, SUC_lexicon_tags_29 },
    { 0x252133f0, SUC_lexicon_tags_37 },
    { 0x6c4133f1, SUC_lexicon_tags_72 },
    { 0x138133f1, SUC_lexicon_tags_7 },
    { 0x1e4133f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae133f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x256133f9, SUC_lexicon_tags_25 },
    { 0xb24133fa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1f6133fc, SUC_lexicon_tags_7 },
    { 0x90e133fd, SUC_lexicon_tags_1 },
    { 0x5fe133fe, SUC_lexicon_tags_82 },
    { 0x600133ff, SUC_lexicon_tags_7 },
    { 0x91c13400, SUC_lexicon_tags_7 },
    { 0xd9a133ff, SUC_lexicon_tags_11 },
    { 0x6b213402, SUC_lexicon_tags_1 },
    { 0xf8613402, SUC_lexicon_tags_33 },
    { 0xdae13402, SUC_lexicon_tags_77 },
    { 0xe2213405, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13a13407, SUC_lexicon_tags_2 },
    { 0xc0413408, SUC_lexicon_tags_15 },
    { 0x54013409, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb461340b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcf01340d, SUC_lexicon_tags_607 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a13411, SUC_lexicon_tags_2 },
    { 0xf0a13412, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de13415, SUC_lexicon_tags_7 },
    { 0x71e13416, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b41341e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe13425, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87813427, SUC_lexicon_tags_8 },
    { 0xf6c13427, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc1342d, SUC_lexicon_tags_25 },
    { 0x47e1342e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed013430, SUC_lexicon_tags_1515 },
    { 0xcfa13431, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4a13433, SUC_lexicon_tags_37 },
    { 0x77813433, SUC_lexicon_tags_12 },
    { 0x40613435, SUC_lexicon_tags_0 },
    { 0xaee13433, SUC_lexicon_tags_141 },
    { 0x35813437, SUC_lexicon_tags_1 },
    { 0x89813437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe061343b, SUC_lexicon_tags_414 },
    { 0xb621343b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1b41343e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2613440, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7013443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0213448, SUC_lexicon_tags_39 },
    { 0x42813449, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24013452, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x22213455, SUC_lexicon_tags_25 },
    { 0x38613456, SUC_lexicon_tags_2 },
    { 0xb6013457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f41345a, SUC_lexicon_tags_11 },
    { 0xbe01345a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c1345e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc13461, SUC_lexicon_tags_33 },
    { 0x7b613461, SUC_lexicon_tags_17 },
    { 0x4a813463, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a13466, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x28a13468, SUC_lexicon_tags_33 },
    { 0xc9c13468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b21346b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2e1346d, SUC_lexicon_tags_37 },
    { 0x1cc1346e, SUC_lexicon_tags_12 },
    { 0xa901346f, SUC_lexicon_tags_57 },
    { 0x6fc13470, SUC_lexicon_tags_1 },
    { 0xd981346e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57013476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0413478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e1347c, SUC_lexicon_tags_37 },
    { 0x4261347d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9613482, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c13486, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef41348a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a1348d, SUC_lexicon_tags_17 },
    { 0x6101348d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac13496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b613498, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438134a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6a134a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226134ae, SUC_lexicon_tags_4 },
    { 0x200134af, SUC_lexicon_tags_2 },
    { 0x9c134b0, SUC_lexicon_tags_11 },
    { 0xa24134ae, SUC_lexicon_tags_2 },
    { 0xe00134b1, SUC_lexicon_tags_26 },
    { 0x3de134b3, SUC_lexicon_tags_37 },
    { 0x3c134b4, SUC_lexicon_tags_15 },
    { 0x7d6134b5, SUC_lexicon_tags_30 },
    { 0xa82134b4, SUC_lexicon_tags_25 },
    { 0xe52134b7, SUC_lexicon_tags_11 },
    { 0x704134b5, SUC_lexicon_tags_30 },
    { 0xa8a134b9, SUC_lexicon_tags_40 },
    { 0x460134b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe8a134bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738134c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa90134c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6134c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e134cb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe134d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x468134d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb78134d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x340134d9, SUC_lexicon_tags_11 },
    { 0xe48134da, SUC_lexicon_tags_46 },
    { 0xa56134da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56134df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2134e2, SUC_lexicon_tags_6 },
    { 0x548134e3, SUC_lexicon_tags_37 },
    { 0xa18134e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc134ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460134f2, SUC_lexicon_tags_16 },
    { 0x798134f3, SUC_lexicon_tags_15 },
    { 0x146134f4, SUC_lexicon_tags_45 },
    { 0xeac134f5, SUC_lexicon_tags_2 },
    { 0xe12134f6, SUC_lexicon_tags_47 },
    { 0x79c134f5, SUC_lexicon_tags_11 },
    { 0x39a134f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2134fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22213503, SUC_lexicon_tags_17 },
    { 0x2ba13504, SUC_lexicon_tags_17 },
    { 0x4ae13505, SUC_lexicon_tags_28 },
    { 0xc8a13506, SUC_lexicon_tags_37 },
    { 0x4b413504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca1350b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c213510, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a613517, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7461351a, SUC_lexicon_tags_17 },
    { 0xfa61351b, SUC_lexicon_tags_37 },
    { 0xaa61351c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5613522, SUC_lexicon_tags_11 },
    { 0xde413523, SUC_lexicon_tags_15 },
    { 0xc8a13524, SUC_lexicon_tags_12 },
    { 0x63e13523, SUC_lexicon_tags_56 },
    { 0xd4c13526, SUC_lexicon_tags_19 },
    { 0xd7213527, SUC_lexicon_tags_49 },
    { 0xa7613528, SUC_lexicon_tags_10 },
    { 0xfc613529, SUC_lexicon_tags_26 },
    { 0x6b61352a, SUC_lexicon_tags_45 },
    { 0x6fc1352b, SUC_lexicon_tags_15 },
    { 0xfd413527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5613531, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f213533, SUC_lexicon_tags_29 },
    { 0x71813534, SUC_lexicon_tags_10 },
    { 0xf2e13534, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ae13537, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x31613539, SUC_lexicon_tags_68 },
    { 0xf7c1353a, SUC_lexicon_tags_29 },
    { 0x3781353a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4261353e, SUC_lexicon_tags_38 },
    { 0x3101353f, SUC_lexicon_tags_4 },
    { 0xcea13540, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c813546, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe4013548, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341354d, SUC_lexicon_tags_15 },
    { 0x4501354e, SUC_lexicon_tags_2 },
    { 0xae1354d, SUC_lexicon_tags_8 },
    { 0xb0213550, SUC_lexicon_tags_28 },
    { 0x7c813551, SUC_lexicon_tags_63 },
    { 0xa3213552, SUC_lexicon_tags_26 },
    { 0x34c13551, SUC_lexicon_tags_24 },
    { 0xaf01354e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49213559, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a1355c, SUC_lexicon_tags_27 },
    { 0xc961355c, SUC_lexicon_tags_77 },
    { 0x8c21355e, SUC_lexicon_tags_7 },
    { 0x66a1355f, SUC_lexicon_tags_0 },
    { 0xa5813560, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf561356e, SUC_lexicon_tags_37 },
    { 0x62e1356f, SUC_lexicon_tags_11 },
    { 0x34a13570, SUC_lexicon_tags_104 },
    { 0x5721356f, SUC_lexicon_tags_24 },
    { 0xb5a13571, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1213574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e13578, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d21357b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa281357e, SUC_lexicon_tags_2 },
    { 0x2fa1357f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea13582, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f813585, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3c413587, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82a13589, SUC_lexicon_tags_0 },
    { 0x7421358a, SUC_lexicon_tags_4 },
    { 0x5a41358b, SUC_lexicon_tags_12 },
    { 0x2b61358b, SUC_lexicon_tags_12 },
    { 0x1401358d, SUC_lexicon_tags_27 },
    { 0xb813589, SUC_lexicon_tags_17 },
    { 0x9e01358f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2613597, SUC_lexicon_tags_7 },
    { 0x5ac13598, SUC_lexicon_tags_47 },
    { 0x9613598, SUC_lexicon_tags_37 },
    { 0xc9e1359a, SUC_lexicon_tags_2 },
    { 0x7a41359a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80135a3, SUC_lexicon_tags_12 },
    { 0xbe135a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb6c135a6, SUC_lexicon_tags_7 },
    { 0xab6135a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46135aa, SUC_lexicon_tags_8 },
    { 0x90c135ab, SUC_lexicon_tags_37 },
    { 0x824135ab, SUC_lexicon_tags_23 },
    { 0x478135ad, SUC_lexicon_tags_1 },
    { 0x14a135ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142135b2, SUC_lexicon_tags_10 },
    { 0x528135b2, SUC_lexicon_tags_37 },
    { 0x408135b3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34135b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9f2135bb, SUC_lexicon_tags_28 },
    { 0x7b6135bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x53c135be, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x868135c1, SUC_lexicon_tags_7 },
    { 0x160135c2, SUC_lexicon_tags_0 },
    { 0xa42135c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e135c8, SUC_lexicon_tags_39 },
    { 0xed6135c9, SUC_lexicon_tags_14 },
    { 0x8a0135ca, SUC_lexicon_tags_7 },
    { 0xa90135ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x844135ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x928135d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe6135d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e135d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc135da, SUC_lexicon_tags_17 },
    { 0x608135da, SUC_lexicon_tags_29 },
    { 0xd7e135da, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386135e3, SUC_lexicon_tags_17 },
    { 0x53a135e4, SUC_lexicon_tags_37 },
    { 0x754135e5, SUC_lexicon_tags_4 },
    { 0x6ba135e6, SUC_lexicon_tags_12 },
    { 0x232135e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a135ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x930135ed, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a135f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2135f4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb30135f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446135fc, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2013604, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2213608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c1360e, SUC_lexicon_tags_32 },
    { 0xdce1360e, SUC_lexicon_tags_17 },
    { 0x99e13610, SUC_lexicon_tags_48 },
    { 0x82e13611, SUC_lexicon_tags_25 },
    { 0x26813610, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7213614, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4881361b, SUC_lexicon_tags_17 },
    { 0xc0a1361c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a13621, SUC_lexicon_tags_31 },
    { 0xbba13622, SUC_lexicon_tags_56 },
    { 0x6b413622, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf041362b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7e21362d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a13631, SUC_lexicon_tags_79 },
    { 0x3d413632, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x49e13634, SUC_lexicon_tags_8 },
    { 0x3d413634, SUC_lexicon_tags_47 },
    { 0x70613635, SUC_lexicon_tags_63 },
    { 0xe2a13636, SUC_lexicon_tags_64 },
    { 0xa1013638, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb661363f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c13643, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x73613646, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec813648, SUC_lexicon_tags_2 },
    { 0x14013649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd01364b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf461364d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5213650, SUC_lexicon_tags_17 },
    { 0x7b013650, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50413653, SUC_lexicon_tags_113 },
    { 0x15013653, SUC_lexicon_tags_15 },
    { 0x1ce13655, SUC_lexicon_tags_31 },
    { 0x67213656, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5101365a, SUC_lexicon_tags_17 },
    { 0x621365a, SUC_lexicon_tags_45 },
    { 0xb5c1365c, SUC_lexicon_tags_23 },
    { 0xe8e1365d, SUC_lexicon_tags_12 },
    { 0x1ac1365d, SUC_lexicon_tags_37 },
    { 0x5b41365f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdba13661, SUC_lexicon_tags_81 },
    { 0xa9a13662, SUC_lexicon_tags_15 },
    { 0x7f413663, SUC_lexicon_tags_29 },
    { 0xae013664, SUC_lexicon_tags_1 },
    { 0xefa13664, SUC_lexicon_tags_23 },
    { 0xc13663, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd213669, SUC_lexicon_tags_35 },
    { 0xeaa13669, SUC_lexicon_tags_23 },
    { 0xdee1366b, SUC_lexicon_tags_11 },
    { 0x7c21366a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54813671, SUC_lexicon_tags_9 },
    { 0xdc813672, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b413675, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9013678, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e1367c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe213682, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e13686, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4621368e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27413695, SUC_lexicon_tags_11 },
    { 0x8dc13696, SUC_lexicon_tags_188 },
    { 0x95013695, SUC_lexicon_tags_14 },
    { 0x81413696, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6261369f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x144136a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6136a5, SUC_lexicon_tags_15 },
    { 0x93a136a6, SUC_lexicon_tags_38 },
    { 0x18136a7, SUC_lexicon_tags_0 },
    { 0xd88136a8, SUC_lexicon_tags_7 },
    { 0xec136a8, SUC_lexicon_tags_1 },
    { 0x82c136a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe136ad, SUC_lexicon_tags_11 },
    { 0xe48136ae, SUC_lexicon_tags_0 },
    { 0x518136ae, SUC_lexicon_tags_37 },
    { 0x968136b0, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0xbe8136b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c136be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e136c1, SUC_lexicon_tags_37 },
    { 0x32136c2, SUC_lexicon_tags_7 },
    { 0x6fc136c1, SUC_lexicon_tags_29 },
    { 0x4bc136c4, SUC_lexicon_tags_0 },
    { 0xa8a136c5, SUC_lexicon_tags_2 },
    { 0x36a136c5, SUC_lexicon_tags_31 },
    { 0x4c6136c5, SUC_lexicon_tags_43 },
    { 0x4d8136c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54136d0, SUC_lexicon_tags_29 },
    { 0x634136d0, SUC_lexicon_tags_11 },
    { 0x5fc136d2, SUC_lexicon_tags_61 },
    { 0x4d4136d3, SUC_lexicon_tags_11 },
    { 0x984136d4, SUC_lexicon_tags_68 },
    { 0x33c136d4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa40136d7, SUC_lexicon_tags_11 },
    { 0xf36136d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xedc136da, SUC_lexicon_tags_11 },
    { 0xf4a136da, SUC_lexicon_tags_60 },
    { 0x89a136db, SUC_lexicon_tags_0 },
    { 0x8de136dd, SUC_lexicon_tags_21 },
    { 0x438136de, SUC_lexicon_tags_87 },
    { 0xb90136df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e136e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c8136e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa136ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe8136ec, SUC_lexicon_tags_1 },
    { 0x4fe136ed, SUC_lexicon_tags_72 },
    { 0x3e8136ec, SUC_lexicon_tags_47 },
    { 0x180136ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x384136f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a6136f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe136f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2c136f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e136fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x242136fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa13704, SUC_lexicon_tags_18 },
    { 0xb5413705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7413708, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf761370a, SUC_lexicon_tags_37 },
    { 0x3d01370b, SUC_lexicon_tags_6 },
    { 0x27c1370b, SUC_lexicon_tags_19 },
    { 0x2fe1370c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97013711, SUC_lexicon_tags_17 },
    { 0x42c13711, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac13716, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7013718, SUC_lexicon_tags_25 },
    { 0x2f013719, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf4c1371b, SUC_lexicon_tags_25 },
    { 0x8ae1371c, SUC_lexicon_tags_21 },
    { 0xe621371c, SUC_lexicon_tags_37 },
    { 0x53c1371d, SUC_lexicon_tags_23 },
    { 0x2c21371b, SUC_lexicon_tags_7 },
    { 0x8ca13720, SUC_lexicon_tags_33 },
    { 0xe3e13721, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80c13723, SUC_lexicon_tags_14 },
    { 0xd4213724, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea1372b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1841372f, SUC_lexicon_tags_27 },
    { 0x48e13730, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be13739, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef81373b, SUC_lexicon_tags_14 },
    { 0x9a81373c, SUC_lexicon_tags_63 },
    { 0xb3a1373d, SUC_lexicon_tags_37 },
    { 0xfbc1373d, SUC_lexicon_tags_7 },
    { 0xa861373f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9813742, SUC_lexicon_tags_35 },
    { 0xb6813742, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xae013745, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5413747, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x4613749, SUC_lexicon_tags_46 },
    { 0x68813749, SUC_lexicon_tags_1 },
    { 0xad81374b, SUC_lexicon_tags_48 },
    { 0x5da1374c, SUC_lexicon_tags_65 },
    { 0x7741374c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62613751, SUC_lexicon_tags_25 },
    { 0xc5813752, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75213758, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e1375b, SUC_lexicon_tags_17 },
    { 0xf8c1375b, SUC_lexicon_tags_27 },
    { 0x3cc1375b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b413760, SUC_lexicon_tags_1 },
    { 0x7b013761, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa4013763, SUC_lexicon_tags_19 },
    { 0x72613763, SUC_lexicon_tags_1 },
    { 0xf8013765, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e213767, SUC_lexicon_tags_27 },
    { 0x5213768, SUC_lexicon_tags_38 },
    { 0x7ec13769, SUC_lexicon_tags_47 },
    { 0x8701376a, SUC_lexicon_tags_17 },
    { 0xf3e1376b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac13772, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb8a13774, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d413776, SUC_lexicon_tags_60 },
    { 0x5d013777, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfde13779, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee1377d, SUC_lexicon_tags_23 },
    { 0x18c1377d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf613781, SUC_lexicon_tags_46 },
    { 0x9d213782, SUC_lexicon_tags_37 },
    { 0x31013782, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c13787, SUC_lexicon_tags_243 },
    { 0x43413787, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca1378b, SUC_lexicon_tags_15 },
    { 0xbf01378c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2fa1378e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca13795, SUC_lexicon_tags_55 },
    { 0x51613796, SUC_lexicon_tags_19 },
    { 0x25413795, SUC_lexicon_tags_60 },
    { 0x8e13798, SUC_lexicon_tags_0 },
    { 0x3fe13799, SUC_lexicon_tags_2 },
    { 0x6181379a, SUC_lexicon_tags_15 },
    { 0x1901379a, SUC_lexicon_tags_16 },
    { 0xd341379a, SUC_lexicon_tags_12 },
    { 0xd961379d, SUC_lexicon_tags_159 },
    { 0x82c1379e, SUC_lexicon_tags_25 },
    { 0x22813797, SUC_lexicon_tags_26 },
    { 0xa021379f, SUC_lexicon_tags_17 },
    { 0xc7c13795, SUC_lexicon_tags_47 },
    { 0x3c0137a2, SUC_lexicon_tags_37 },
    { 0xd3e137a2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a137a7, SUC_lexicon_tags_14 },
    { 0x63c137a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7c137aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a137ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a137b0, SUC_lexicon_tags_17 },
    { 0xf44137b0, SUC_lexicon_tags_1 },
    { 0xd28137b2, SUC_lexicon_tags_10 },
    { 0x6fa137b3, SUC_lexicon_tags_7 },
    { 0x44e137b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60137b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x678137bb, SUC_lexicon_tags_178 },
    { 0x978137bc, SUC_lexicon_tags_7 },
    { 0xf4c137bc, SUC_lexicon_tags_12 },
    { 0x4da137bc, SUC_lexicon_tags_7 },
    { 0xa5a137bc, SUC_lexicon_tags_23 },
    { 0x264137c0, SUC_lexicon_tags_47 },
    { 0x428137c0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60137c4, SUC_lexicon_tags_31 },
    { 0xf9c137c5, SUC_lexicon_tags_25 },
    { 0xad4137c6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb92137c8, SUC_lexicon_tags_1 },
    { 0x346137c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea137cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2137d0, SUC_lexicon_tags_4 },
    { 0xf92137d0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x43c137d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x620137d6, SUC_lexicon_tags_15 },
    { 0xd68137d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae137da, SUC_lexicon_tags_25 },
    { 0xfda137db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70137e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8137e5, SUC_lexicon_tags_2 },
    { 0x352137e6, SUC_lexicon_tags_23 },
    { 0x4de137e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xade137ea, SUC_lexicon_tags_47 },
    { 0x33c137eb, SUC_lexicon_tags_32 },
    { 0x5c2137eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468137f0, SUC_lexicon_tags_31 },
    { 0x968137f1, SUC_lexicon_tags_59 },
    { 0x476137f2, SUC_lexicon_tags_14 },
    { 0xc32137f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee137f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x668137f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6137fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd6e137ff, SUC_lexicon_tags_15 },
    { 0xfb813800, SUC_lexicon_tags_8 },
    { 0x81e137ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a013803, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x49213805, SUC_lexicon_tags_15 },
    { 0xfda13806, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9c13808, SUC_lexicon_tags_10 },
    { 0xfa413808, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1521380c, SUC_lexicon_tags_41 },
    { 0xb1a1380d, SUC_lexicon_tags_15 },
    { 0xb241380d, SUC_lexicon_tags_37 },
    { 0xa701380d, SUC_lexicon_tags_2 },
    { 0x8e213810, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d813814, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe13817, SUC_lexicon_tags_32 },
    { 0x8e813818, SUC_lexicon_tags_15 },
    { 0x87013817, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a81381d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6761381f, SUC_lexicon_tags_136 },
    { 0xeac13820, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f213825, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c13832, SUC_lexicon_tags_37 },
    { 0x7fa13833, SUC_lexicon_tags_1 },
    { 0x61613833, SUC_lexicon_tags_0 },
    { 0x6a813834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6413839, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e1383c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73013841, SUC_lexicon_tags_38 },
    { 0x9be13842, SUC_lexicon_tags_8 },
    { 0x5be13841, SUC_lexicon_tags_10 },
    { 0xfc13844, SUC_lexicon_tags_16 },
    { 0xdf213845, SUC_lexicon_tags_15 },
    { 0x49813844, SUC_lexicon_tags_14 },
    { 0xc8013841, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a1384b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb81384d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10213851, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc13856, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6141385a, SUC_lexicon_tags_33 },
    { 0xc041385b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc13860, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x38413862, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0c13864, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd213866, SUC_lexicon_tags_87 },
    { 0xabc13867, SUC_lexicon_tags_2 },
    { 0xcde13868, SUC_lexicon_tags_72 },
    { 0xc4a13869, SUC_lexicon_tags_37 },
    { 0xce813869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a1386d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7213870, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa3813872, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c13877, SUC_lexicon_tags_47 },
    { 0x8d213877, SUC_lexicon_tags_17 },
    { 0xff413879, SUC_lexicon_tags_151 },
    { 0x2d81387a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d01387d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb681387f, SUC_lexicon_tags_47 },
    { 0x9f01387f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea013884, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc613886, SUC_lexicon_tags_37 },
    { 0xfc013887, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7501388a, SUC_lexicon_tags_19 },
    { 0xea81388a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xde81388d, SUC_lexicon_tags_7 },
    { 0xee81388e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e13891, SUC_lexicon_tags_88 },
    { 0x94813892, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc413894, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c813896, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf061389b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8181389f, SUC_lexicon_tags_47 },
    { 0x668138a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20138a9, SUC_lexicon_tags_21 },
    { 0x7da138aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02138ad, SUC_lexicon_tags_30 },
    { 0x5d4138ae, SUC_lexicon_tags_7 },
    { 0xd88138af, SUC_lexicon_tags_47 },
    { 0xa7a138af, SUC_lexicon_tags_7 },
    { 0xfe0138ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac138b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x760138b9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56138bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6138c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3de138c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeea138c8, SUC_lexicon_tags_12 },
    { 0xbbc138c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580138ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce138d4, SUC_lexicon_tags_7 },
    { 0xac0138d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0138db, SUC_lexicon_tags_7 },
    { 0x6a6138dc, SUC_lexicon_tags_17 },
    { 0x734138db, SUC_lexicon_tags_15 },
    { 0xe2e138de, SUC_lexicon_tags_9 },
    { 0x268138dd, SUC_lexicon_tags_63 },
    { 0xdc4138e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c138e3, SUC_lexicon_tags_911 },
    { 0x2b4138e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4f6138e6, SUC_lexicon_tags_1 },
    { 0xa8a138e7, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xb7c138e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0138ec, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c138f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a138f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfa4138f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca138fa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8138fe, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0x42813900, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1413903, SUC_lexicon_tags_12 },
    { 0x3813904, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1613906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c13909, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2041390e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0013913, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec413918, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf81391a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb21391d, SUC_lexicon_tags_34 },
    { 0xbb81391d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c613921, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3e613923, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99413928, SUC_lexicon_tags_25 },
    { 0x9ac13929, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff81392b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9301392d, SUC_lexicon_tags_49 },
    { 0x56e1392d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa613930, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x66213933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c13937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b813939, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee81393e, SUC_lexicon_tags_17 },
    { 0xbdc1393f, SUC_lexicon_tags_6 },
    { 0xef213940, SUC_lexicon_tags_2 },
    { 0x9fe13941, SUC_lexicon_tags_45 },
    { 0xbc01393f, SUC_lexicon_tags_19 },
    { 0x34813941, SUC_lexicon_tags_37 },
    { 0x87c1393f, SUC_lexicon_tags_11 },
    { 0x56013941, SUC_lexicon_tags_33 },
    { 0x72013945, SUC_lexicon_tags_16 },
    { 0xa9a13947, SUC_lexicon_tags_31 },
    { 0x46e13947, SUC_lexicon_tags_11 },
    { 0xc8613940, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa881394b, SUC_lexicon_tags_8 },
    { 0xfa21394c, SUC_lexicon_tags_1 },
    { 0xddc1394c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70013959, SUC_lexicon_tags_25 },
    { 0x2ba1395a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd0a1395c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7413961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a13964, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f413969, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1396c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe13970, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xca813972, SUC_lexicon_tags_23 },
    { 0x98413972, SUC_lexicon_tags_63 },
    { 0xdf413974, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf013979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3661397b, SUC_lexicon_tags_11 },
    { 0x6ec1397b, SUC_lexicon_tags_28 },
    { 0xc041397b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c13983, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x30a13985, SUC_lexicon_tags_47 },
    { 0xeb013986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50613989, SUC_lexicon_tags_12 },
    { 0xef81398a, SUC_lexicon_tags_56 },
    { 0x9201398a, SUC_lexicon_tags_12 },
    { 0x59a1398a, SUC_lexicon_tags_10 },
    { 0xe761398d, SUC_lexicon_tags_14 },
    { 0xf161398d, SUC_lexicon_tags_97 },
    { 0xbd01398f, SUC_lexicon_tags_11 },
    { 0x8e413990, SUC_lexicon_tags_27 },
    { 0x57e13990, SUC_lexicon_tags_29 },
    { 0xb1213992, SUC_lexicon_tags_0 },
    { 0x87813993, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be13997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b21399c, SUC_lexicon_tags_33 },
    { 0x5021399d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa139a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a4139a2, SUC_lexicon_tags_37 },
    { 0x600139a2, SUC_lexicon_tags_7 },
    { 0x44139a3, SUC_lexicon_tags_2 },
    { 0x938139a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2139aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0139b1, SUC_lexicon_tags_37 },
    { 0xc1a139b2, SUC_lexicon_tags_60 },
    { 0xfa4139b1, SUC_lexicon_tags_28 },
    { 0x5da139b3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308139b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a139bc, SUC_lexicon_tags_2 },
    { 0xd82139bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae139c0, SUC_lexicon_tags_1 },
    { 0x492139c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdaa139c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a139c6, SUC_lexicon_tags_37 },
    { 0xa74139c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806139d4, SUC_lexicon_tags_7 },
    { 0x330139d4, SUC_lexicon_tags_0 },
    { 0x190139d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc139da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0139dd, SUC_lexicon_tags_29 },
    { 0xea4139de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb52139e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c139e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad4139e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758139ec, SUC_lexicon_tags_23 },
    { 0xe48139ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e139f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x426139f2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6139f6, SUC_lexicon_tags_0 },
    { 0xcd0139f7, SUC_lexicon_tags_30 },
    { 0xa9c139f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbe139fa, SUC_lexicon_tags_37 },
    { 0x7da139fb, SUC_lexicon_tags_45 },
    { 0x7ca139fc, SUC_lexicon_tags_16 },
    { 0xed8139fc, SUC_lexicon_tags_2 },
    { 0xefc139fa, SUC_lexicon_tags_9 },
    { 0x5cc139ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae13a04, SUC_lexicon_tags_1 },
    { 0x4fa13a05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeea13a07, SUC_lexicon_tags_25 },
    { 0x3b813a07, SUC_lexicon_tags_2 },
    { 0xca013a08, SUC_lexicon_tags_68 },
    { 0xc1813a0a, SUC_lexicon_tags_37 },
    { 0x5fa13a0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c13a11, SUC_lexicon_tags_47 },
    { 0x7c013a12, SUC_lexicon_tags_6 },
    { 0x15213a13, SUC_lexicon_tags_59 },
    { 0x54413a13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa013a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b613a19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4213a1c, SUC_lexicon_tags_70 },
    { 0x1be13a1c, SUC_lexicon_tags_1 },
    { 0x21413a1e, SUC_lexicon_tags_7 },
    { 0x8e613a1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7813a21, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x33613a24, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc3813a26, SUC_lexicon_tags_6 },
    { 0xe4c13a27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe613a29, SUC_lexicon_tags_18 },
    { 0x41213a2a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x79a13a2c, SUC_lexicon_tags_1 },
    { 0xe3413a2c, SUC_lexicon_tags_26 },
    { 0xde213a2e, SUC_lexicon_tags_55 },
    { 0x80413a2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f213a34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe613a36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3c13a38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe213a3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4813a3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89213a41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2c13a43, SUC_lexicon_tags_25 },
    { 0xdee13a44, SUC_lexicon_tags_14 },
    { 0xb3213a43, SUC_lexicon_tags_12 },
    { 0xf0413a46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b813a4b, SUC_lexicon_tags_45 },
    { 0x6a013a4c, SUC_lexicon_tags_1 },
    { 0x4be13a4d, SUC_lexicon_tags_6 },
    { 0xe9813a4e, SUC_lexicon_tags_23 },
    { 0x9ec13a4c, SUC_lexicon_tags_17 },
    { 0x5ec13a50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x52013a53, SUC_lexicon_tags_222 },
    { 0xbaa13a54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xada13a56, SUC_lexicon_tags_25 },
    { 0x50413a57, SUC_lexicon_tags_11 },
    { 0xbb813a58, SUC_lexicon_tags_12 },
    { 0x34613a59, SUC_lexicon_tags_57 },
    { 0x94a13a5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e13a5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5c13a60, SUC_lexicon_tags_7 },
    { 0xd8e13a61, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e13a69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc13a6e, SUC_lexicon_tags_14 },
    { 0xdea13a6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce13a71, SUC_lexicon_tags_68 },
    { 0x8fe13a72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0213a74, SUC_lexicon_tags_15 },
    { 0x65213a74, SUC_lexicon_tags_15 },
    { 0x1ca13a75, SUC_lexicon_tags_11 },
    { 0xf5613a77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de13a7b, SUC_lexicon_tags_12 },
    { 0x30613a7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66213a7f, SUC_lexicon_tags_87 },
    { 0xc0c13a80, SUC_lexicon_tags_46 },
    { 0x3da13a81, SUC_lexicon_tags_25 },
    { 0x95813a81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c13a85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29013a89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a413a90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49013a92, SUC_lexicon_tags_7 },
    { 0xb8a13a93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c13a96, SUC_lexicon_tags_37 },
    { 0xaa213a96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19c13a99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d413a9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac213a9d, SUC_lexicon_tags_7 },
    { 0x6ca13a9e, SUC_lexicon_tags_46 },
    { 0x9813a9e, SUC_lexicon_tags_25 },
    { 0x34013aa0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96a13aa2, SUC_lexicon_tags_6 },
    { 0x47c13aa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f813aa6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e013aab, SUC_lexicon_tags_1 },
    { 0xa8013aac, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x17413aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31213ab0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c13ab3, SUC_lexicon_tags_9 },
    { 0x2b613ab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a13aba, SUC_lexicon_tags_7 },
    { 0x5b613aba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2813ac3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7dc13ac5, SUC_lexicon_tags_4 },
    { 0x8ac13ac6, SUC_lexicon_tags_47 },
    { 0xb6013ac7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52413ac9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc13acf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd013ad5, SUC_lexicon_tags_14 },
    { 0x75013ad6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a13ada, SUC_lexicon_tags_17 },
    { 0xda213adb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c813ade, SUC_lexicon_tags_2 },
    { 0xe3a13adf, SUC_lexicon_tags_1 },
    { 0x30e13adf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda613ae2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb013ae5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e213ae8, SUC_lexicon_tags_66 },
    { 0x4cc13ae9, SUC_lexicon_tags_80 },
    { 0xf4413ae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa613aec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x17013aef, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x33613af2, SUC_lexicon_tags_17 },
    { 0xd8a13af2, SUC_lexicon_tags_49 },
    { 0x7a613af3, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0x97413af6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d613af8, SUC_lexicon_tags_15 },
    { 0x55013af8, SUC_lexicon_tags_1 },
    { 0xc3e13af9, SUC_lexicon_tags_2 },
    { 0xb5413afb, SUC_lexicon_tags_10 },
    { 0xa0a13af9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0013aff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37a13b01, SUC_lexicon_tags_14 },
    { 0xfcc13b02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a813b04, SUC_lexicon_tags_15 },
    { 0x7ac13b04, SUC_lexicon_tags_61 },
    { 0xb9013b06, SUC_lexicon_tags_25 },
    { 0xd9c13b07, SUC_lexicon_tags_12 },
    { 0x2cc13b05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e13b0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x68213b0f, SUC_lexicon_tags_37 },
    { 0xea813b10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee13b14, SUC_lexicon_tags_12 },
    { 0xf4c13b15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43813b24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56a13b26, SUC_lexicon_tags_37 },
    { 0xa3413b27, SUC_lexicon_tags_25 },
    { 0x48013b28, SUC_lexicon_tags_45 },
    { 0xd1c13b26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac13b2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1813b32, SUC_lexicon_tags_37 },
    { 0x69013b33, SUC_lexicon_tags_39 },
    { 0x65e13b34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ba13b36, SUC_lexicon_tags_0 },
    { 0x71a13b37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e13b3a, SUC_lexicon_tags_19 },
    { 0x6a813b3b, SUC_lexicon_tags_25 },
    { 0x4ee13b3b, SUC_lexicon_tags_6 },
    { 0xb5213b3d, SUC_lexicon_tags_10 },
    { 0x58213b3e, SUC_lexicon_tags_12 },
    { 0xa013b3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc13b43, SUC_lexicon_tags_11 },
    { 0x89213b44, SUC_lexicon_tags_8 },
    { 0xca413b45, SUC_lexicon_tags_17 },
    { 0xf413b46, SUC_lexicon_tags_48 },
    { 0xfb213b46, SUC_lexicon_tags_60 },
    { 0xf9413b48, SUC_lexicon_tags_14 },
    { 0xdb813b44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21013b50, SUC_lexicon_tags_2 },
    { 0x16e13b51, SUC_lexicon_tags_38 },
    { 0xb5a13b50, SUC_lexicon_tags_25 },
    { 0xc0a13b53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c813b55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99213b5a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9ea13b5c, SUC_lexicon_tags_31 },
    { 0x8ec13b5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26613b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58413b63, SUC_lexicon_tags_26 },
    { 0xd0413b63, SUC_lexicon_tags_47 },
    { 0x9d613b65, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413b6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44c13b6c, SUC_lexicon_tags_11 },
    { 0x76c13b6d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x88613b6f, SUC_lexicon_tags_29 },
    { 0x52013b70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1613b76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81813b7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f013b80, SUC_lexicon_tags_37 },
    { 0x99013b80, SUC_lexicon_tags_8 },
    { 0x67413b82, SUC_lexicon_tags_14 },
    { 0x59c13b83, SUC_lexicon_tags_14 },
    { 0xa413b80, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfee13b86, SUC_lexicon_tags_37 },
    { 0xa5613b87, SUC_lexicon_tags_15 },
    { 0x98c13b88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e013b8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4e13b8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a813b8f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3813b91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99613b93, SUC_lexicon_tags_43 },
    { 0x75013b93, SUC_lexicon_tags_17 },
    { 0xdf413b94, SUC_lexicon_tags_26 },
    { 0x77c13b96, SUC_lexicon_tags_17 },
    { 0xa7813b96, SUC_lexicon_tags_49 },
    { 0xed613b96, SUC_lexicon_tags_38 },
    { 0xe0a13b98, SUC_lexicon_tags_43 },
    { 0x41613b9a, SUC_lexicon_tags_33 },
    { 0x23213b9b, SUC_lexicon_tags_83 },
    { 0xa013b97, SUC_lexicon_tags_9 },
    { 0x8ce13b97, SUC_lexicon_tags_77 },
    { 0x98e13b9e, SUC_lexicon_tags_116 },
    { 0x34613b9e, SUC_lexicon_tags_17 },
    { 0x63613b9e, SUC_lexicon_tags_1 },
    { 0x66813ba1, SUC_lexicon_tags_15 },
    { 0xb7813ba2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba613ba9, SUC_lexicon_tags_61 },
    { 0xb3613baa, SUC_lexicon_tags_12 },
    { 0x6aa13bab, SUC_lexicon_tags_23 },
    { 0xf7e13bac, SUC_lexicon_tags_1 },
    { 0xfb213bad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf813baf, SUC_lexicon_tags_23 },
    { 0x2e13baf, SUC_lexicon_tags_11 },
    { 0x56813bb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c13bb4, SUC_lexicon_tags_47 },
    { 0x4a613bb5, SUC_lexicon_tags_15 },
    { 0xe3013bb6, SUC_lexicon_tags_151 },
    { 0xa7813bb7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdf813bb9, SUC_lexicon_tags_28 },
    { 0xfa813bba, SUC_lexicon_tags_32 },
    { 0x2b013bba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54813bc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e13bc6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec13bc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4813bcc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x83413bcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a13bd6, SUC_lexicon_tags_17 },
    { 0x91213bd7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfd613bd9, SUC_lexicon_tags_17 },
    { 0xb6c13bd9, SUC_lexicon_tags_25 },
    { 0xc0a13bd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf2213bdd, SUC_lexicon_tags_7 },
    { 0x50c13bdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5213be4, SUC_lexicon_tags_66 },
    { 0xd2c13be5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a13bec, SUC_lexicon_tags_16 },
    { 0xf4613bed, SUC_lexicon_tags_1 },
    { 0x63613bee, SUC_lexicon_tags_12 },
    { 0xfa213bee, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x15c13bf1, SUC_lexicon_tags_2 },
    { 0x17613bf2, SUC_lexicon_tags_37 },
    { 0xe4613bf3, SUC_lexicon_tags_17 },
    { 0xf3013bf3, SUC_lexicon_tags_113 },
    { 0x2c213bf2, SUC_lexicon_tags_12 },
    { 0xad613bf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec13bfa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x613bfc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe3c13bfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x70013c01, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec213c0c, SUC_lexicon_tags_11 },
    { 0x6e13c0d, SUC_lexicon_tags_29 },
    { 0x9bc13c0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e13c13, SUC_lexicon_tags_25 },
    { 0xa3613c14, SUC_lexicon_tags_33 },
    { 0x1f213c15, SUC_lexicon_tags_8 },
    { 0xbde13c15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c13c1a, SUC_lexicon_tags_25 },
    { 0xf5e13c1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22613c22, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x80013c24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe613c29, SUC_lexicon_tags_15 },
    { 0x3be13c2a, SUC_lexicon_tags_25 },
    { 0x9ba13c2b, SUC_lexicon_tags_28 },
    { 0xb4813c2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa13c2f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x18c13c31, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c13c36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe5013c38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86613c3c, SUC_lexicon_tags_15 },
    { 0xab013c3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69813c41, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb8e13c43, SUC_lexicon_tags_11 },
    { 0xc13c44, SUC_lexicon_tags_27 },
    { 0xefa13c45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0613c49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcee13c4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30813c4d, SUC_lexicon_tags_11 },
    { 0x78813c4d, SUC_lexicon_tags_55 },
    { 0x21413c4f, SUC_lexicon_tags_11 },
    { 0x70c13c50, SUC_lexicon_tags_28 },
    { 0xa3413c50, SUC_lexicon_tags_17 },
    { 0x76613c4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0213c56, SUC_lexicon_tags_14 },
    { 0x4e413c57, SUC_lexicon_tags_45 },
    { 0xda013c58, SUC_lexicon_tags_17 },
    { 0xf7413c56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc13c62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0613c65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x77213c67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e13c6a, SUC_lexicon_tags_15 },
    { 0xe0213c6b, SUC_lexicon_tags_8 },
    { 0xe3013c6c, SUC_lexicon_tags_17 },
    { 0xec013c6d, SUC_lexicon_tags_29 },
    { 0xf1813c6d, SUC_lexicon_tags_59 },
    { 0x33a13c6e, SUC_lexicon_tags_27 },
    { 0x7ee13c70, SUC_lexicon_tags_6 },
    { 0x5413c71, SUC_lexicon_tags_6 },
    { 0xa3413c72, SUC_lexicon_tags_37 },
    { 0x8de13c72, SUC_lexicon_tags_31 },
    { 0x55613c71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2213c7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a13c7f, SUC_lexicon_tags_12 },
    { 0x49c13c80, SUC_lexicon_tags_30 },
    { 0x4ac13c7f, SUC_lexicon_tags_43 },
    { 0xa1013c80, SUC_lexicon_tags_37 },
    { 0xd7213c83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3cc13c85, SUC_lexicon_tags_17 },
    { 0x5e413c85, SUC_lexicon_tags_9 },
    { 0x93c13c87, SUC_lexicon_tags_23 },
    { 0xbf413c87, SUC_lexicon_tags_2 },
    { 0x7a813c89, SUC_lexicon_tags_80 },
    { 0x89013c8a, SUC_lexicon_tags_11 },
    { 0x5613c8a, SUC_lexicon_tags_17 },
    { 0xcc813c8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3013c90, SUC_lexicon_tags_17 },
    { 0x18e13c91, SUC_lexicon_tags_30 },
    { 0xe9a13c90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb813c95, SUC_lexicon_tags_7 },
    { 0xe613c95, SUC_lexicon_tags_12 },
    { 0xa3013c95, SUC_lexicon_tags_27 },
    { 0xaa013c95, SUC_lexicon_tags_25 },
    { 0x18813c99, SUC_lexicon_tags_25 },
    { 0x9ec13c95, SUC_lexicon_tags_673 },
    { 0xe2a13c97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be13c9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0813ca2, SUC_lexicon_tags_7 },
    { 0x25a13ca3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x95613ca6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x71613ca9, SUC_lexicon_tags_15 },
    { 0x66c13caa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54213caf, SUC_lexicon_tags_12 },
    { 0x21a13cb0, SUC_lexicon_tags_15 },
    { 0x8413cb1, SUC_lexicon_tags_11 },
    { 0xa0e13cb2, SUC_lexicon_tags_83 },
    { 0xf8013cb3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8813cb6, SUC_lexicon_tags_29 },
    { 0xa3213cb6, SUC_lexicon_tags_27 },
    { 0xea013cb8, SUC_lexicon_tags_34 },
    { 0x2bc13cb9, SUC_lexicon_tags_14 },
    { 0x9413cb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5a13cbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c213cbe, SUC_lexicon_tags_15 },
    { 0x94213cbf, SUC_lexicon_tags_45 },
    { 0x1da13cbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc13cc5, SUC_lexicon_tags_68 },
    { 0xda613cc6, SUC_lexicon_tags_60 },
    { 0x12e13cc7, SUC_lexicon_tags_65 },
    { 0xf4813cc8, SUC_lexicon_tags_25 },
    { 0xc7413cc8, SUC_lexicon_tags_0 },
    { 0xeea13cc9, SUC_lexicon_tags_37 },
    { 0x47813cca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4c13ccd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c13cd3, SUC_lexicon_tags_43 },
    { 0x9f413cd4, SUC_lexicon_tags_43 },
    { 0xe5e13cd5, SUC_lexicon_tags_17 },
    { 0xf1c13cd4, SUC_lexicon_tags_12 },
    { 0xfc13cd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99213cdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c813cdd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x84e13cdf, SUC_lexicon_tags_8 },
    { 0xf8e13ce0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79613ce6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ea13ce8, SUC_lexicon_tags_974 },
    { 0, NULL },
    { 0, NULL },
    { 0x68413ceb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8213cf2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe7613cf4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x74213cf6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c13cfa, SUC_lexicon_tags_158 },
    { 0x4c13cfa, SUC_lexicon_tags_4 },
    { 0xe6a13cfc, SUC_lexicon_tags_37 },
    { 0x3e013cfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeec13cff, SUC_lexicon_tags_25 },
    { 0xeb613cff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde613d05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80413d07, SUC_lexicon_tags_14 },
    { 0xd8e13d08, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf013d0c, SUC_lexicon_tags_8 },
    { 0x5613d0d, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0x57c13d0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22c13d11, SUC_lexicon_tags_12 },
    { 0xf0c13d11, SUC_lexicon_tags_15 },
    { 0x74813d12, SUC_lexicon_tags_2 },
    { 0xeca13d14, SUC_lexicon_tags_87 },
    { 0x45213d15, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae413d1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa413d1d, SUC_lexicon_tags_26 },
    { 0xb9e13d1d, SUC_lexicon_tags_11 },
    { 0x8613d1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xeca13d21, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c13d28, SUC_lexicon_tags_27 },
    { 0xdd613d29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x84213d2c, SUC_lexicon_tags_2 },
    { 0x71a13d2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e13d31, SUC_lexicon_tags_34 },
    { 0xbe413d31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42813d39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaac13d3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0013d42, SUC_lexicon_tags_6 },
    { 0xed213d43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35813d47, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x26213d4a, SUC_lexicon_tags_30 },
    { 0x51a13d4a, SUC_lexicon_tags_11 },
    { 0x70a13d4c, SUC_lexicon_tags_95 },
    { 0x74613d4d, SUC_lexicon_tags_8 },
    { 0x19813d4a, SUC_lexicon_tags_25 },
    { 0x8f013d4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b613d53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd6e13d55, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3c13d57, SUC_lexicon_tags_11 },
    { 0xa8c13d58, SUC_lexicon_tags_14 },
    { 0x9a013d59, SUC_lexicon_tags_23 },
    { 0x7ba13d58, SUC_lexicon_tags_17 },
    { 0x3a613d5b, SUC_lexicon_tags_12 },
    { 0x59a13d59, SUC_lexicon_tags_45 },
    { 0xe8c13d5a, SUC_lexicon_tags_15 },
    { 0xf3013d5c, SUC_lexicon_tags_17 },
    { 0x57613d5d, SUC_lexicon_tags_47 },
    { 0xa5613d60, SUC_lexicon_tags_47 },
    { 0x26413d61, SUC_lexicon_tags_17 },
    { 0xb9213d58, SUC_lexicon_tags_11 },
    { 0x65e13d57, SUC_lexicon_tags_41 },
    { 0xa0613d5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5613d68, SUC_lexicon_tags_14 },
    { 0x45413d68, SUC_lexicon_tags_33 },
    { 0x6aa13d6a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67213d70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1813d72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68213d76, SUC_lexicon_tags_70 },
    { 0x47c13d76, SUC_lexicon_tags_7 },
    { 0x89c13d78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c13d80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95213d85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7c13d87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c213d8b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc9413d8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c13d92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4da13d94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2213d98, SUC_lexicon_tags_1 },
    { 0xa1c13d99, SUC_lexicon_tags_65 },
    { 0x36013d9a, SUC_lexicon_tags_66 },
    { 0x81613d9a, SUC_lexicon_tags_12 },
    { 0xe5813d9c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb613da5, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68813da9, SUC_lexicon_tags_23 },
    { 0xeca13daa, SUC_lexicon_tags_2 },
    { 0xe3613dab, SUC_lexicon_tags_25 },
    { 0xc3c13dac, SUC_lexicon_tags_25 },
    { 0x9e813dab, SUC_lexicon_tags_27 },
    { 0x49c13dad, SUC_lexicon_tags_29 },
    { 0x40213daf, SUC_lexicon_tags_31 },
    { 0x74613dad, SUC_lexicon_tags_12 },
    { 0x19613daf, SUC_lexicon_tags_47 },
    { 0xe213da9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x90413db4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e013dbb, SUC_lexicon_tags_32 },
    { 0xdc013dbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4213dc5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb213dca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec13dcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5213dd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9213dd7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56213dd9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa5413ddb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc213de1, SUC_lexicon_tags_7 },
    { 0xbba13de2, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x51a13de4, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x79813de7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a13deb, SUC_lexicon_tags_29 },
    { 0x13413dec, SUC_lexicon_tags_60 },
    { 0xf7e13ded, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d613df4, SUC_lexicon_tags_23 },
    { 0xad613df5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a13df9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2e613dfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c13e00, SUC_lexicon_tags_35 },
    { 0x72a13e01, SUC_lexicon_tags_2 },
    { 0x4b413e00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82813e04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9c13e06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7413e09, SUC_lexicon_tags_26 },
    { 0x5c013e0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ba13e0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc613e14, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x4e813e16, SUC_lexicon_tags_39 },
    { 0x34a13e16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53213e19, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa0e13e1b, SUC_lexicon_tags_2 },
    { 0x3e213e1c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x54c13e1e, SUC_lexicon_tags_60 },
    { 0x30013e1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55013e23, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7213e2e, SUC_lexicon_tags_30 },
    { 0x8c13e2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d613e31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a13e34, SUC_lexicon_tags_11 },
    { 0x11a13e35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60a13e37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87413e3d, SUC_lexicon_tags_56 },
    { 0x32213e3d, SUC_lexicon_tags_4 },
    { 0x47613e3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9da13e41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39e13e43, SUC_lexicon_tags_17 },
    { 0x76813e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea13e46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c813e4d, SUC_lexicon_tags_242 },
    { 0, NULL },
    { 0xa1213e4f, SUC_lexicon_tags_19 },
    { 0xf3a13e50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6213e58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6213e5a, SUC_lexicon_tags_60 },
    { 0x3be13e5b, SUC_lexicon_tags_31 },
    { 0xe413e5a, SUC_lexicon_tags_1 },
    { 0x7aa13e5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13e5f, SUC_lexicon_tags_37 },
    { 0xcd813e60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae13e63, SUC_lexicon_tags_25 },
    { 0xcd213e64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51613e66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39013e70, SUC_lexicon_tags_38 },
    { 0x52413e70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x80813e73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1aa13e75, SUC_lexicon_tags_33 },
    { 0x2a413e76, SUC_lexicon_tags_37 },
    { 0x51c13e77, SUC_lexicon_tags_7 },
    { 0x47a13e78, SUC_lexicon_tags_27 },
    { 0x41413e76, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb013e7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b613e7e, SUC_lexicon_tags_37 },
    { 0x4b213e7e, SUC_lexicon_tags_25 },
    { 0xec13e7e, SUC_lexicon_tags_37 },
    { 0xa2813e7e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e13e84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3013e88, SUC_lexicon_tags_1 },
    { 0x4ee13e88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a013e8b, SUC_lexicon_tags_25 },
    { 0x1413e8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x67613e8e, SUC_lexicon_tags_112 },
    { 0x74a13e8f, SUC_lexicon_tags_38 },
    { 0x86413e8f, SUC_lexicon_tags_177 },
    { 0x8a613e91, SUC_lexicon_tags_49 },
    { 0x3aa13e92, SUC_lexicon_tags_63 },
    { 0x97c13e93, SUC_lexicon_tags_37 },
    { 0xbd813e8f, SUC_lexicon_tags_27 },
    { 0x22613e93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c13e9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea13ea1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69013ea9, SUC_lexicon_tags_87 },
    { 0x26c13eaa, SUC_lexicon_tags_8 },
    { 0xd8a13eaa, SUC_lexicon_tags_25 },
    { 0x93a13eaa, SUC_lexicon_tags_43 },
    { 0xc1413ead, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xebe13eaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c813eb4, SUC_lexicon_tags_46 },
    { 0xb613eb5, SUC_lexicon_tags_0 },
    { 0x91813eb6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36813eb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f013ebb, SUC_lexicon_tags_2 },
    { 0x23e13ebc, SUC_lexicon_tags_25 },
    { 0x37a13ebd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b213ec2, SUC_lexicon_tags_23 },
    { 0xd0c13ec3, SUC_lexicon_tags_61 },
    { 0xb2613ec3, SUC_lexicon_tags_15 },
    { 0xffa13ec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a13ec8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7d813eca, SUC_lexicon_tags_8 },
    { 0xf2413eca, SUC_lexicon_tags_37 },
    { 0xd7813ecc, SUC_lexicon_tags_8 },
    { 0xc4e13ecd, SUC_lexicon_tags_47 },
    { 0xab613eca, SUC_lexicon_tags_19 },
    { 0x4ca13ecd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53413ed6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69813ede, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5813ee2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c13eec, SUC_lexicon_tags_33 },
    { 0xff613eed, SUC_lexicon_tags_2 },
    { 0x1de13eee, SUC_lexicon_tags_4 },
    { 0xd4c13eee, SUC_lexicon_tags_4 },
    { 0x69813ef0, SUC_lexicon_tags_17 },
    { 0x7c413ef1, SUC_lexicon_tags_9 },
    { 0x95c13ef2, SUC_lexicon_tags_7 },
    { 0xcbc13ef3, SUC_lexicon_tags_7 },
    { 0xa8c13ef3, SUC_lexicon_tags_21 },
    { 0x8aa13ef5, SUC_lexicon_tags_25 },
    { 0x22813eee, SUC_lexicon_tags_18 },
    { 0xd4613ef7, SUC_lexicon_tags_1 },
    { 0x8f213ef1, SUC_lexicon_tags_36 },
    { 0xb4413ef2, SUC_lexicon_tags_6 },
    { 0x91213eee, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x26613efc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeee13efe, SUC_lexicon_tags_38 },
    { 0x91613efe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7013f03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x11413f06, SUC_lexicon_tags_17 },
    { 0x90e13f06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a213f09, SUC_lexicon_tags_38 },
    { 0x68613f0a, SUC_lexicon_tags_14 },
    { 0x7e613f0a, SUC_lexicon_tags_2 },
    { 0x98613f0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb5613f0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe213f12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf813f14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27a13f16, SUC_lexicon_tags_2 },
    { 0xf8e13f17, SUC_lexicon_tags_33 },
    { 0x72613f18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e13f1b, SUC_lexicon_tags_37 },
    { 0xaf013f1b, SUC_lexicon_tags_27 },
    { 0xb9a13f1d, SUC_lexicon_tags_87 },
    { 0x32e13f1e, SUC_lexicon_tags_7 },
    { 0x19813f1f, SUC_lexicon_tags_17 },
    { 0xf3413f1f, SUC_lexicon_tags_40 },
    { 0xe413f21, SUC_lexicon_tags_23 },
    { 0xd6613f1d, SUC_lexicon_tags_7 },
    { 0xea613f23, SUC_lexicon_tags_6 },
    { 0x11213f1e, SUC_lexicon_tags_12 },
    { 0x20e13f1d, SUC_lexicon_tags_25 },
    { 0x8cc13f1f, SUC_lexicon_tags_1 },
    { 0xd5a13f25, SUC_lexicon_tags_55 },
    { 0xde013f23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa13f2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb413f3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90013f3f, SUC_lexicon_tags_0 },
    { 0xf4613f40, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x20a13f42, SUC_lexicon_tags_60 },
    { 0xb2613f42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ac13f45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86813f47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69213f4c, SUC_lexicon_tags_49 },
    { 0xefc13f4c, SUC_lexicon_tags_26 },
    { 0xff813f4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef813f53, SUC_lexicon_tags_23 },
    { 0xb6213f53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3013f59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a13f5f, SUC_lexicon_tags_47 },
    { 0x71213f5f, SUC_lexicon_tags_25 },
    { 0x8fe13f5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a13f67, SUC_lexicon_tags_1 },
    { 0x79c13f68, SUC_lexicon_tags_37 },
    { 0x40c13f69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa13f6c, SUC_lexicon_tags_2 },
    { 0x66613f6d, SUC_lexicon_tags_11 },
    { 0xc8013f6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a13f73, SUC_lexicon_tags_1 },
    { 0x9d213f73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c413f79, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa413f80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a13f85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4013f87, SUC_lexicon_tags_27 },
    { 0xb3a13f87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30413f8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69213f92, SUC_lexicon_tags_71 },
    { 0xb7013f93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x29413f96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a213f98, SUC_lexicon_tags_19 },
    { 0xae13f99, SUC_lexicon_tags_23 },
    { 0x75613f9a, SUC_lexicon_tags_1 },
    { 0xa413f9a, SUC_lexicon_tags_11 },
    { 0xf4613f98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49813fa0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e13fa8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea13fae, SUC_lexicon_tags_10 },
    { 0x3ca13faf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54013fb1, SUC_lexicon_tags_14 },
    { 0x6dc13fb2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa4013fb4, SUC_lexicon_tags_25 },
    { 0xd4613fb5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x90e13fb7, SUC_lexicon_tags_37 },
    { 0x18813fb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8013fbc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6213fbf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e813fc7, SUC_lexicon_tags_17 },
    { 0x9ca13fc8, SUC_lexicon_tags_42 },
    { 0x13c13fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da13fcc, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97013fd0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c13fd4, SUC_lexicon_tags_296 },
    { 0x813fd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa213fdb, SUC_lexicon_tags_154 },
    { 0x68013fdc, SUC_lexicon_tags_21 },
    { 0xc5213fdb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd413fdf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa013fe5, SUC_lexicon_tags_77 },
    { 0x56413fe6, SUC_lexicon_tags_1122 },
    { 0x44413fe7, SUC_lexicon_tags_38 },
    { 0x31813fe8, SUC_lexicon_tags_11 },
    { 0xfb413fe9, SUC_lexicon_tags_0 },
    { 0xd7e13fea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1613fec, SUC_lexicon_tags_0 },
    { 0x34613fed, SUC_lexicon_tags_25 },
    { 0x4813fee, SUC_lexicon_tags_69 },
    { 0xf7a13fee, SUC_lexicon_tags_12 },
    { 0x3a413ff0, SUC_lexicon_tags_7 },
    { 0x66613ff1, SUC_lexicon_tags_12 },
    { 0xf8213ff1, SUC_lexicon_tags_41 },
    { 0x1a13ff3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6c413ff5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be13ff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a13ffe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x81614001, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x98c14003, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda814007, SUC_lexicon_tags_47 },
    { 0x26814008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4de1400a, SUC_lexicon_tags_15 },
    { 0x2541400b, SUC_lexicon_tags_47 },
    { 0x6fe1400c, SUC_lexicon_tags_17 },
    { 0x5581400d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85214010, SUC_lexicon_tags_23 },
    { 0xe0c14011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8014016, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10414022, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf014027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba1402a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc1402d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33e1402f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71414033, SUC_lexicon_tags_15 },
    { 0xc0c14033, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c1403b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9661403f, SUC_lexicon_tags_13 },
    { 0x7e814040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce1404b, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0xce81404d, SUC_lexicon_tags_47 },
    { 0xdea1404e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf014052, SUC_lexicon_tags_53 },
    { 0x26214053, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdbc14055, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac1405c, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x58c1405e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1e14060, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25814068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa321406b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e81406e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8214071, SUC_lexicon_tags_17 },
    { 0xd8614072, SUC_lexicon_tags_19 },
    { 0x6dc14073, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd441407c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7814080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc614083, SUC_lexicon_tags_63 },
    { 0x58214083, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56014088, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc1408c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55414092, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c14095, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e14098, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6601409d, SUC_lexicon_tags_21 },
    { 0x4781409d, SUC_lexicon_tags_60 },
    { 0x37e1409d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954140a3, SUC_lexicon_tags_19 },
    { 0x36e140a4, SUC_lexicon_tags_47 },
    { 0x1ee140a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2140a9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da140ad, SUC_lexicon_tags_15 },
    { 0xe0140ae, SUC_lexicon_tags_2 },
    { 0xce4140af, SUC_lexicon_tags_11 },
    { 0xb26140b0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8140b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44140b8, SUC_lexicon_tags_66 },
    { 0x478140b9, SUC_lexicon_tags_2 },
    { 0x246140ba, SUC_lexicon_tags_12 },
    { 0xf8e140ba, SUC_lexicon_tags_82 },
    { 0x35a140ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a140c3, SUC_lexicon_tags_1 },
    { 0xa3a140c4, SUC_lexicon_tags_47 },
    { 0x302140c4, SUC_lexicon_tags_12 },
    { 0xfd0140c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a140ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x946140cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878140d0, SUC_lexicon_tags_11 },
    { 0xbb0140d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x622140d4, SUC_lexicon_tags_60 },
    { 0x700140d5, SUC_lexicon_tags_87 },
    { 0x72a140d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda8140d8, SUC_lexicon_tags_114 },
    { 0x78e140d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x814140dc, SUC_lexicon_tags_17 },
    { 0x4c0140dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7e140df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5da140e1, SUC_lexicon_tags_65 },
    { 0x9a6140e2, SUC_lexicon_tags_10 },
    { 0x76a140e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2140e6, SUC_lexicon_tags_17 },
    { 0xad6140e7, SUC_lexicon_tags_1 },
    { 0xfde140e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0140ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50140f2, SUC_lexicon_tags_37 },
    { 0x5ac140f3, SUC_lexicon_tags_15 },
    { 0x664140f3, SUC_lexicon_tags_25 },
    { 0x4e4140f5, SUC_lexicon_tags_0 },
    { 0x6c6140f6, SUC_lexicon_tags_48 },
    { 0x244140f7, SUC_lexicon_tags_15 },
    { 0x21e140f8, SUC_lexicon_tags_27 },
    { 0x926140f6, SUC_lexicon_tags_11 },
    { 0x562140f8, SUC_lexicon_tags_21 },
    { 0x9e4140f4, SUC_lexicon_tags_23 },
    { 0x2ca140fc, SUC_lexicon_tags_2 },
    { 0x4fc140fc, SUC_lexicon_tags_29 },
    { 0x930140fe, SUC_lexicon_tags_25 },
    { 0x32e140ff, SUC_lexicon_tags_23 },
    { 0x2f614100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0814102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa14106, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbc014108, SUC_lexicon_tags_87 },
    { 0xa8814108, SUC_lexicon_tags_47 },
    { 0xf0c1410a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0414110, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2e14112, SUC_lexicon_tags_60 },
    { 0x92e14113, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b214117, SUC_lexicon_tags_2 },
    { 0x7614118, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce1411d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b614120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8414124, SUC_lexicon_tags_6 },
    { 0x60a14125, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x19414128, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd21412b, SUC_lexicon_tags_49 },
    { 0x681412c, SUC_lexicon_tags_2 },
    { 0xef21412d, SUC_lexicon_tags_45 },
    { 0x1ae1412b, SUC_lexicon_tags_7 },
    { 0x30c1412c, SUC_lexicon_tags_14 },
    { 0x7361412c, SUC_lexicon_tags_17 },
    { 0x71c1412d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a14137, SUC_lexicon_tags_23 },
    { 0x85214138, SUC_lexicon_tags_31 },
    { 0x61414139, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e14141, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a414146, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6614149, SUC_lexicon_tags_12 },
    { 0xbbe1414a, SUC_lexicon_tags_25 },
    { 0xd4414149, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca14152, SUC_lexicon_tags_43 },
    { 0x92614152, SUC_lexicon_tags_34 },
    { 0x90014154, SUC_lexicon_tags_11 },
    { 0xb1e14153, SUC_lexicon_tags_4 },
    { 0x93814156, SUC_lexicon_tags_49 },
    { 0x55c14157, SUC_lexicon_tags_27 },
    { 0x37014158, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f01415f, SUC_lexicon_tags_17 },
    { 0x3c814160, SUC_lexicon_tags_0 },
    { 0xd5214160, SUC_lexicon_tags_66 },
    { 0x2f414162, SUC_lexicon_tags_11 },
    { 0x4ec14162, SUC_lexicon_tags_7 },
    { 0x9a614162, SUC_lexicon_tags_1 },
    { 0x2f61415f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55014168, SUC_lexicon_tags_12 },
    { 0x1fa14168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7881416c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4814171, SUC_lexicon_tags_70 },
    { 0x8ba14172, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce414174, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb214177, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4961417b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb761417d, SUC_lexicon_tags_25 },
    { 0x74c1417d, SUC_lexicon_tags_12 },
    { 0x2441417f, SUC_lexicon_tags_63 },
    { 0x97a1417d, SUC_lexicon_tags_6 },
    { 0xa01417d, SUC_lexicon_tags_17 },
    { 0x3021417d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e14185, SUC_lexicon_tags_9 },
    { 0xe9414186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4014188, SUC_lexicon_tags_2 },
    { 0x98614189, SUC_lexicon_tags_103 },
    { 0xf941418a, SUC_lexicon_tags_12 },
    { 0x1541418b, SUC_lexicon_tags_37 },
    { 0xdc61418a, SUC_lexicon_tags_23 },
    { 0xe101418d, SUC_lexicon_tags_11 },
    { 0x1fe1418e, SUC_lexicon_tags_33 },
    { 0xe141418c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32614192, SUC_lexicon_tags_14 },
    { 0x6ae14193, SUC_lexicon_tags_7 },
    { 0x62414192, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a1419a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3661419c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc141a4, SUC_lexicon_tags_85 },
    { 0x3b0141a5, SUC_lexicon_tags_25 },
    { 0xdc2141a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2141ab, SUC_lexicon_tags_25 },
    { 0xaf0141ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x782141af, SUC_lexicon_tags_12 },
    { 0xbd0141b0, SUC_lexicon_tags_4 },
    { 0x27c141b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x264141b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4141b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22141bb, SUC_lexicon_tags_33 },
    { 0xc92141bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x832141be, SUC_lexicon_tags_63 },
    { 0xd3c141bf, SUC_lexicon_tags_11 },
    { 0x9f8141c0, SUC_lexicon_tags_28 },
    { 0x364141c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24141c6, SUC_lexicon_tags_95 },
    { 0xfbc141c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2da141c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc141ce, SUC_lexicon_tags_2 },
    { 0x7d6141ce, SUC_lexicon_tags_37 },
    { 0x2cc141ce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22141d4, SUC_lexicon_tags_30 },
    { 0xaa2141d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x208141d7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe74141d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e141dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2141e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda141e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x16a141e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40141ec, SUC_lexicon_tags_60 },
    { 0x138141ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a141f4, SUC_lexicon_tags_11 },
    { 0x6d0141f5, SUC_lexicon_tags_14 },
    { 0x83a141f6, SUC_lexicon_tags_15 },
    { 0x43e141f7, SUC_lexicon_tags_15 },
    { 0xf2c141f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x97c141fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb4e141fc, SUC_lexicon_tags_17 },
    { 0x4ba141fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7414200, SUC_lexicon_tags_160 },
    { 0xe3a14201, SUC_lexicon_tags_33 },
    { 0x9da14202, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc14208, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe1e1420a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4201420d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11414213, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa14216, SUC_lexicon_tags_37 },
    { 0xa3c14217, SUC_lexicon_tags_15 },
    { 0x1d614218, SUC_lexicon_tags_11 },
    { 0xe0614219, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9841421c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a1421f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x50614221, SUC_lexicon_tags_37 },
    { 0x6014221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40614226, SUC_lexicon_tags_2 },
    { 0x91214227, SUC_lexicon_tags_19 },
    { 0xb2214228, SUC_lexicon_tags_11 },
    { 0x73014228, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa861422c, SUC_lexicon_tags_2 },
    { 0xcc61422c, SUC_lexicon_tags_27 },
    { 0x9521422e, SUC_lexicon_tags_15 },
    { 0x8be1422f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a14235, SUC_lexicon_tags_25 },
    { 0x12414236, SUC_lexicon_tags_45 },
    { 0xf5814237, SUC_lexicon_tags_9 },
    { 0xa6e14237, SUC_lexicon_tags_17 },
    { 0xdee14238, SUC_lexicon_tags_15 },
    { 0x16e14238, SUC_lexicon_tags_4 },
    { 0x2ae1423b, SUC_lexicon_tags_82 },
    { 0xc101423c, SUC_lexicon_tags_47 },
    { 0xf4c1423d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4414241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2aa14243, SUC_lexicon_tags_11 },
    { 0x4a214244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48c14246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c1424a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7261424d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8c1424f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a14252, SUC_lexicon_tags_38 },
    { 0xd3814253, SUC_lexicon_tags_15 },
    { 0x36614254, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc14257, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1581425b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4221425d, SUC_lexicon_tags_1 },
    { 0xd201425e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe421426f, SUC_lexicon_tags_15 },
    { 0x95e14270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca14277, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2da14279, SUC_lexicon_tags_14 },
    { 0xcae14279, SUC_lexicon_tags_45 },
    { 0x8c014279, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54014281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15414285, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d214288, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc01428d, SUC_lexicon_tags_11 },
    { 0x2c61428e, SUC_lexicon_tags_0 },
    { 0x2941428e, SUC_lexicon_tags_7 },
    { 0xe6c14290, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea14293, SUC_lexicon_tags_12 },
    { 0x51014293, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d214298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c1429b, SUC_lexicon_tags_68 },
    { 0x1381429c, SUC_lexicon_tags_37 },
    { 0xb441429b, SUC_lexicon_tags_6 },
    { 0xa7c1429e, SUC_lexicon_tags_25 },
    { 0xafe1429e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40142a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e142ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0142b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e142ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96142c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa6e142c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff4142c5, SUC_lexicon_tags_12 },
    { 0x8d0142c6, SUC_lexicon_tags_69 },
    { 0x15e142c5, SUC_lexicon_tags_0 },
    { 0x588142c5, SUC_lexicon_tags_107 },
    { 0x350142c5, SUC_lexicon_tags_33 },
    { 0xcde142ca, SUC_lexicon_tags_1 },
    { 0x6ac142c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a142ce, SUC_lexicon_tags_10 },
    { 0xaee142cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c142d6, SUC_lexicon_tags_38 },
    { 0x922142d7, SUC_lexicon_tags_15 },
    { 0x692142d8, SUC_lexicon_tags_0 },
    { 0x7da142d8, SUC_lexicon_tags_7 },
    { 0x696142da, SUC_lexicon_tags_45 },
    { 0x53c142d9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x980142de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a142e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6142e7, SUC_lexicon_tags_16 },
    { 0xf02142e7, SUC_lexicon_tags_3 },
    { 0x5aa142e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd16142eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980142ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c142f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a142f7, SUC_lexicon_tags_28 },
    { 0xaa0142f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x568142fa, SUC_lexicon_tags_64 },
    { 0x2c2142fb, SUC_lexicon_tags_87 },
    { 0xe26142fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c0142fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6414300, SUC_lexicon_tags_39 },
    { 0x8b414301, SUC_lexicon_tags_17 },
    { 0x33414302, SUC_lexicon_tags_38 },
    { 0x1bc14302, SUC_lexicon_tags_15 },
    { 0x14e14304, SUC_lexicon_tags_65 },
    { 0x2c14305, SUC_lexicon_tags_25 },
    { 0x7a414306, SUC_lexicon_tags_60 },
    { 0x5f614307, SUC_lexicon_tags_127 },
    { 0x5fa14307, SUC_lexicon_tags_37 },
    { 0xb4c14308, SUC_lexicon_tags_12 },
    { 0x16414308, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9e1430c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc1430f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb9614311, SUC_lexicon_tags_19 },
    { 0x11814311, SUC_lexicon_tags_25 },
    { 0xcd414313, SUC_lexicon_tags_12 },
    { 0xfd414311, SUC_lexicon_tags_12 },
    { 0x3e214315, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x43814318, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e1431d, SUC_lexicon_tags_2 },
    { 0x57a1431e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x51e14320, SUC_lexicon_tags_11 },
    { 0x8cc14321, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a14325, SUC_lexicon_tags_7 },
    { 0x18214326, SUC_lexicon_tags_7 },
    { 0xdec14327, SUC_lexicon_tags_39 },
    { 0xfac14328, SUC_lexicon_tags_33 },
    { 0x42a14329, SUC_lexicon_tags_47 },
    { 0x8e14325, SUC_lexicon_tags_38 },
    { 0xe261432b, SUC_lexicon_tags_39 },
    { 0xbea14327, SUC_lexicon_tags_0 },
    { 0x83014328, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf601432f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce14335, SUC_lexicon_tags_7 },
    { 0xa6214336, SUC_lexicon_tags_11 },
    { 0x50414337, SUC_lexicon_tags_44 },
    { 0x88a14338, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83414340, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa14343, SUC_lexicon_tags_66 },
    { 0xc9414344, SUC_lexicon_tags_1 },
    { 0xfb414345, SUC_lexicon_tags_27 },
    { 0x33214346, SUC_lexicon_tags_28 },
    { 0x214344, SUC_lexicon_tags_15 },
    { 0x17814347, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8541434c, SUC_lexicon_tags_3 },
    { 0xb21434c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8901434f, SUC_lexicon_tags_1 },
    { 0x16a14350, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8614352, SUC_lexicon_tags_26 },
    { 0x1ac14352, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9614358, SUC_lexicon_tags_48 },
    { 0xc2c14359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec1435c, SUC_lexicon_tags_11 },
    { 0x3a21435d, SUC_lexicon_tags_11 },
    { 0x3701435e, SUC_lexicon_tags_2 },
    { 0x67a1435f, SUC_lexicon_tags_65 },
    { 0xf2a14360, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfbc14362, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x89814365, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe541436a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de14372, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c14376, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5614380, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27c14382, SUC_lexicon_tags_11 },
    { 0xb8814382, SUC_lexicon_tags_11 },
    { 0x75e14382, SUC_lexicon_tags_11 },
    { 0x13414384, SUC_lexicon_tags_69 },
    { 0xefa14386, SUC_lexicon_tags_22 },
    { 0xd7214387, SUC_lexicon_tags_8 },
    { 0x36014385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbac1438a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc14391, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c14394, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b214396, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce143a0, SUC_lexicon_tags_2 },
    { 0x646143a1, SUC_lexicon_tags_87 },
    { 0xa44143a2, SUC_lexicon_tags_11 },
    { 0xa8143a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac143a9, SUC_lexicon_tags_47 },
    { 0xb3c143a9, SUC_lexicon_tags_39 },
    { 0xe143ab, SUC_lexicon_tags_4 },
    { 0xb00143ab, SUC_lexicon_tags_2 },
    { 0x744143ad, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac143b0, SUC_lexicon_tags_25 },
    { 0x37e143b0, SUC_lexicon_tags_27 },
    { 0x6a4143b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c143b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e143ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2143c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46143c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830143c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x70a143cb, SUC_lexicon_tags_11 },
    { 0xc50143cc, SUC_lexicon_tags_17 },
    { 0xdcc143cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6143d2, SUC_lexicon_tags_4 },
    { 0x70143d3, SUC_lexicon_tags_85 },
    { 0x7c6143d3, SUC_lexicon_tags_12 },
    { 0xf4c143d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4143db, SUC_lexicon_tags_82 },
    { 0x518143dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c143e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70143eb, SUC_lexicon_tags_47 },
    { 0x5ca143ec, SUC_lexicon_tags_7 },
    { 0xe7a143eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x634143ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54143f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9c143f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836143fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c14400, SUC_lexicon_tags_43 },
    { 0xff414400, SUC_lexicon_tags_35 },
    { 0xef214401, SUC_lexicon_tags_10 },
    { 0xb9c14403, SUC_lexicon_tags_47 },
    { 0xe0614403, SUC_lexicon_tags_37 },
    { 0x5d614405, SUC_lexicon_tags_31 },
    { 0x9a614404, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba1440a, SUC_lexicon_tags_0 },
    { 0x64e1440b, SUC_lexicon_tags_11 },
    { 0x5e1440c, SUC_lexicon_tags_15 },
    { 0xb081440a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3414413, SUC_lexicon_tags_33 },
    { 0xf0814413, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf214416, SUC_lexicon_tags_33 },
    { 0x72414417, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf681441b, SUC_lexicon_tags_25 },
    { 0xb41441b, SUC_lexicon_tags_2 },
    { 0x2401441d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e14422, SUC_lexicon_tags_37 },
    { 0x54814423, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e14428, SUC_lexicon_tags_11 },
    { 0xb5814429, SUC_lexicon_tags_12 },
    { 0x2721442a, SUC_lexicon_tags_15 },
    { 0xf861442b, SUC_lexicon_tags_16 },
    { 0xfbc1442c, SUC_lexicon_tags_23 },
    { 0xa761442d, SUC_lexicon_tags_17 },
    { 0x3f41442d, SUC_lexicon_tags_91 },
    { 0xd041442f, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x49614431, SUC_lexicon_tags_15 },
    { 0xe6214431, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e14437, SUC_lexicon_tags_7 },
    { 0x75c14437, SUC_lexicon_tags_11 },
    { 0x54214439, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a01443f, SUC_lexicon_tags_37 },
    { 0x8721443f, SUC_lexicon_tags_5 },
    { 0x88414441, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d214445, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b01444a, SUC_lexicon_tags_37 },
    { 0xc981444b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9801444f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c214451, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe0c14453, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcde14455, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb414457, SUC_lexicon_tags_6 },
    { 0x1b214458, SUC_lexicon_tags_37 },
    { 0xb7614459, SUC_lexicon_tags_29 },
    { 0x44014459, SUC_lexicon_tags_11 },
    { 0x2961445b, SUC_lexicon_tags_4 },
    { 0xf6814457, SUC_lexicon_tags_11 },
    { 0xfe1445c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe61445f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45814465, SUC_lexicon_tags_46 },
    { 0xf2414465, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf861446d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de14471, SUC_lexicon_tags_37 },
    { 0xd1214472, SUC_lexicon_tags_30 },
    { 0x6e214473, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16014475, SUC_lexicon_tags_10 },
    { 0x89014475, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c14479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd901447c, SUC_lexicon_tags_24 },
    { 0x2021447c, SUC_lexicon_tags_16 },
    { 0xff01447e, SUC_lexicon_tags_48 },
    { 0xc0a1447c, SUC_lexicon_tags_17 },
    { 0x17814480, SUC_lexicon_tags_48 },
    { 0xc101447d, SUC_lexicon_tags_47 },
    { 0xf641447f, SUC_lexicon_tags_37 },
    { 0x8a21447d, SUC_lexicon_tags_31 },
    { 0x9c814484, SUC_lexicon_tags_37 },
    { 0xa8214485, SUC_lexicon_tags_3 },
    { 0xa3214486, SUC_lexicon_tags_17 },
    { 0xa9814486, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x1341448a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4ac1448c, SUC_lexicon_tags_37 },
    { 0xaf81448d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3414491, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x22c14493, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9814497, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x65414499, SUC_lexicon_tags_4 },
    { 0xa6a14499, SUC_lexicon_tags_6 },
    { 0x2621449b, SUC_lexicon_tags_17 },
    { 0xb41449c, SUC_lexicon_tags_9 },
    { 0x35a1449d, SUC_lexicon_tags_8 },
    { 0xb141449e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376144a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a144aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82144b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3fc144b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x890144b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9dc144bb, SUC_lexicon_tags_15 },
    { 0x31c144bb, SUC_lexicon_tags_25 },
    { 0x190144bd, SUC_lexicon_tags_43 },
    { 0xe1a144be, SUC_lexicon_tags_66 },
    { 0x7fe144bf, SUC_lexicon_tags_66 },
    { 0xe3c144c0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96144c3, SUC_lexicon_tags_37 },
    { 0x32144c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6aa144c6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x64144c9, SUC_lexicon_tags_60 },
    { 0xbfa144ca, SUC_lexicon_tags_14 },
    { 0xe82144c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92144cf, SUC_lexicon_tags_37 },
    { 0xac2144d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c144d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1c144db, SUC_lexicon_tags_45 },
    { 0xffc144dc, SUC_lexicon_tags_5 },
    { 0xb74144dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162144e4, SUC_lexicon_tags_1 },
    { 0xbde144e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a144e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60e144ea, SUC_lexicon_tags_53 },
    { 0x52144eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804144f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68144fd, SUC_lexicon_tags_37 },
    { 0xf84144fe, SUC_lexicon_tags_65 },
    { 0xdf0144fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c14503, SUC_lexicon_tags_2 },
    { 0x82e14504, SUC_lexicon_tags_1 },
    { 0x80014504, SUC_lexicon_tags_12 },
    { 0xa8614504, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x48414508, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa4e1450a, SUC_lexicon_tags_16 },
    { 0xa001450b, SUC_lexicon_tags_19 },
    { 0x4421450b, SUC_lexicon_tags_25 },
    { 0xe901450b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90e1450f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc14514, SUC_lexicon_tags_7 },
    { 0x3ce14514, SUC_lexicon_tags_31 },
    { 0x71614515, SUC_lexicon_tags_48 },
    { 0x54e14517, SUC_lexicon_tags_43 },
    { 0x2c414518, SUC_lexicon_tags_26 },
    { 0xf9414517, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe081451b, SUC_lexicon_tags_8 },
    { 0xfd81451c, SUC_lexicon_tags_85 },
    { 0x7361451b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe881451f, SUC_lexicon_tags_12 },
    { 0xfd41451f, SUC_lexicon_tags_329 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a14523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ca14525, SUC_lexicon_tags_2 },
    { 0xa3414526, SUC_lexicon_tags_12 },
    { 0x47e14527, SUC_lexicon_tags_15 },
    { 0xbf814525, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd21452d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x21814530, SUC_lexicon_tags_0 },
    { 0xf3214531, SUC_lexicon_tags_31 },
    { 0x51a14532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17614534, SUC_lexicon_tags_23 },
    { 0x3b614535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7614539, SUC_lexicon_tags_17 },
    { 0x57e1453a, SUC_lexicon_tags_17 },
    { 0x7cc14539, SUC_lexicon_tags_15 },
    { 0xef41453c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce14540, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35814542, SUC_lexicon_tags_43 },
    { 0xf1014543, SUC_lexicon_tags_47 },
    { 0x94a14543, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdea14546, SUC_lexicon_tags_2 },
    { 0xf4214547, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac41454b, SUC_lexicon_tags_14 },
    { 0x7de1454b, SUC_lexicon_tags_47 },
    { 0xdd81454d, SUC_lexicon_tags_4 },
    { 0xefe1454d, SUC_lexicon_tags_1 },
    { 0xf7e1454f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e14557, SUC_lexicon_tags_38 },
    { 0x4814558, SUC_lexicon_tags_68 },
    { 0xd8214559, SUC_lexicon_tags_15 },
    { 0x2121455a, SUC_lexicon_tags_7 },
    { 0x9aa1455b, SUC_lexicon_tags_37 },
    { 0xa9a14559, SUC_lexicon_tags_12 },
    { 0xbe21455d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f41455f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a14563, SUC_lexicon_tags_14 },
    { 0x21e14564, SUC_lexicon_tags_0 },
    { 0xbee14565, SUC_lexicon_tags_15 },
    { 0x4ca14566, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54e14568, SUC_lexicon_tags_37 },
    { 0x4e614569, SUC_lexicon_tags_11 },
    { 0xa981456a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f81456c, SUC_lexicon_tags_63 },
    { 0xc61456c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x12014570, SUC_lexicon_tags_84 },
    { 0xf6a14571, SUC_lexicon_tags_31 },
    { 0x9c414572, SUC_lexicon_tags_25 },
    { 0x26614573, SUC_lexicon_tags_37 },
    { 0x39814574, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c14581, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4414583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1614586, SUC_lexicon_tags_47 },
    { 0x19414587, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3721458e, SUC_lexicon_tags_17 },
    { 0x7101458e, SUC_lexicon_tags_7 },
    { 0x77e1458f, SUC_lexicon_tags_38 },
    { 0x44014591, SUC_lexicon_tags_12 },
    { 0x70c14590, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a14596, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a1459b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324145a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52145a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce145b4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x50145b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a8145b9, SUC_lexicon_tags_15 },
    { 0xeb8145b9, SUC_lexicon_tags_0 },
    { 0x306145bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830145c0, SUC_lexicon_tags_0 },
    { 0x46145c0, SUC_lexicon_tags_19 },
    { 0x2a6145c2, SUC_lexicon_tags_25 },
    { 0xf14145c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c145c7, SUC_lexicon_tags_37 },
    { 0x8d0145c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88c145ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596145d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f6145d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b2145d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x768145d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2145dd, SUC_lexicon_tags_63 },
    { 0xcfc145dd, SUC_lexicon_tags_33 },
    { 0xfe6145df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee145e3, SUC_lexicon_tags_25 },
    { 0xa02145e3, SUC_lexicon_tags_1 },
    { 0x13a145e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0145e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70145ec, SUC_lexicon_tags_68 },
    { 0x56a145ed, SUC_lexicon_tags_25 },
    { 0x584145ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a145f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba145f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f6145fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e14605, SUC_lexicon_tags_17 },
    { 0x42414605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d81460c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78014611, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de14614, SUC_lexicon_tags_11 },
    { 0xc0814614, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada1461a, SUC_lexicon_tags_2 },
    { 0xf721461b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1c81461d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b41461f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x92014622, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2214625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd641462a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9814632, SUC_lexicon_tags_35 },
    { 0xd214633, SUC_lexicon_tags_12 },
    { 0x4614634, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1101463f, SUC_lexicon_tags_0 },
    { 0xd5614640, SUC_lexicon_tags_46 },
    { 0xefa14640, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20614648, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde61464a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9be1464c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc861464e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3414650, SUC_lexicon_tags_37 },
    { 0x4fe14651, SUC_lexicon_tags_31 },
    { 0x45814650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8614654, SUC_lexicon_tags_26 },
    { 0x61614655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3aa14657, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf341465a, SUC_lexicon_tags_7 },
    { 0xe201465a, SUC_lexicon_tags_7 },
    { 0xd241465c, SUC_lexicon_tags_38 },
    { 0x6fc1465b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa681465f, SUC_lexicon_tags_48 },
    { 0x66814660, SUC_lexicon_tags_26 },
    { 0x51214661, SUC_lexicon_tags_37 },
    { 0x81a14661, SUC_lexicon_tags_23 },
    { 0xe6214661, SUC_lexicon_tags_15 },
    { 0xcb014662, SUC_lexicon_tags_15 },
    { 0x82614665, SUC_lexicon_tags_32 },
    { 0xd4014665, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xce614669, SUC_lexicon_tags_37 },
    { 0x31a14669, SUC_lexicon_tags_14 },
    { 0x9bc14669, SUC_lexicon_tags_29 },
    { 0x601466c, SUC_lexicon_tags_46 },
    { 0x6be1466c, SUC_lexicon_tags_7 },
    { 0xe2e1466d, SUC_lexicon_tags_31 },
    { 0xecc1466f, SUC_lexicon_tags_47 },
    { 0x62414670, SUC_lexicon_tags_2 },
    { 0x4b81466f, SUC_lexicon_tags_49 },
    { 0x5f614672, SUC_lexicon_tags_11 },
    { 0xb5414673, SUC_lexicon_tags_1 },
    { 0x5361466f, SUC_lexicon_tags_25 },
    { 0xd1214672, SUC_lexicon_tags_30 },
    { 0x12c14676, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8e14678, SUC_lexicon_tags_12 },
    { 0xeac14679, SUC_lexicon_tags_9 },
    { 0x9c81467a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57014681, SUC_lexicon_tags_11 },
    { 0x9d814681, SUC_lexicon_tags_77 },
    { 0xe9a14681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8614685, SUC_lexicon_tags_12 },
    { 0xafe14686, SUC_lexicon_tags_33 },
    { 0x85a14687, SUC_lexicon_tags_11 },
    { 0x9b814688, SUC_lexicon_tags_138 },
    { 0xb8814689, SUC_lexicon_tags_33 },
    { 0x9e14688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd581468c, SUC_lexicon_tags_11 },
    { 0x1401468d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1881468f, SUC_lexicon_tags_27 },
    { 0xda014690, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x75614692, SUC_lexicon_tags_53 },
    { 0xc1014693, SUC_lexicon_tags_55 },
    { 0x75214694, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a146a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6146a5, SUC_lexicon_tags_53 },
    { 0x98c146a6, SUC_lexicon_tags_12 },
    { 0x8ba146a7, SUC_lexicon_tags_37 },
    { 0xf44146a8, SUC_lexicon_tags_17 },
    { 0xb52146a9, SUC_lexicon_tags_15 },
    { 0xb84146aa, SUC_lexicon_tags_9 },
    { 0x9d8146aa, SUC_lexicon_tags_15 },
    { 0x138146a9, SUC_lexicon_tags_25 },
    { 0x55a146a6, SUC_lexicon_tags_48 },
    { 0x41a146a7, SUC_lexicon_tags_2 },
    { 0xbba146ab, SUC_lexicon_tags_19 },
    { 0x434146b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878146b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e146b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe6c146b9, SUC_lexicon_tags_27 },
    { 0xc22146ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x566146bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0146c6, SUC_lexicon_tags_0 },
    { 0x636146c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x776146c9, SUC_lexicon_tags_1 },
    { 0x64e146c9, SUC_lexicon_tags_1 },
    { 0xa2146ca, SUC_lexicon_tags_9 },
    { 0x54e146cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638146d1, SUC_lexicon_tags_26 },
    { 0xece146d2, SUC_lexicon_tags_11 },
    { 0x74146d3, SUC_lexicon_tags_7 },
    { 0x9cc146d4, SUC_lexicon_tags_10 },
    { 0x646146d2, SUC_lexicon_tags_11 },
    { 0xfb4146d6, SUC_lexicon_tags_23 },
    { 0x4e8146d7, SUC_lexicon_tags_63 },
    { 0xaca146d7, SUC_lexicon_tags_11 },
    { 0xa70146d9, SUC_lexicon_tags_37 },
    { 0x70e146d9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x26c146dc, SUC_lexicon_tags_17 },
    { 0xe86146dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774146e1, SUC_lexicon_tags_37 },
    { 0x236146e2, SUC_lexicon_tags_172 },
    { 0x37e146e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c6146e5, SUC_lexicon_tags_9 },
    { 0x568146e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80146ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6146f0, SUC_lexicon_tags_479 },
    { 0x3ca146f1, SUC_lexicon_tags_29 },
    { 0x42a146f2, SUC_lexicon_tags_61 },
    { 0x3ba146f2, SUC_lexicon_tags_12 },
    { 0x202146f4, SUC_lexicon_tags_2 },
    { 0x7fa146f3, SUC_lexicon_tags_68 },
    { 0xf8e146f6, SUC_lexicon_tags_49 },
    { 0x53e146f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x526146fa, SUC_lexicon_tags_14 },
    { 0x4c146fb, SUC_lexicon_tags_68 },
    { 0x45e146fc, SUC_lexicon_tags_60 },
    { 0xb6c146fc, SUC_lexicon_tags_6 },
    { 0x7d0146fe, SUC_lexicon_tags_8 },
    { 0xb9e146fa, SUC_lexicon_tags_29 },
    { 0x78214700, SUC_lexicon_tags_2 },
    { 0xcaa14700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99814703, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38614705, SUC_lexicon_tags_43 },
    { 0x53414705, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd014709, SUC_lexicon_tags_25 },
    { 0x9da14709, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe421470c, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c1470f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4614711, SUC_lexicon_tags_28 },
    { 0xc3c14711, SUC_lexicon_tags_37 },
    { 0xa8e14713, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xba414716, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d814718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c1471b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3814720, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a14725, SUC_lexicon_tags_12 },
    { 0x70a14725, SUC_lexicon_tags_32 },
    { 0x7d614725, SUC_lexicon_tags_37 },
    { 0x38a14725, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b41472b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a1472f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac14733, SUC_lexicon_tags_19 },
    { 0xfd614734, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb214736, SUC_lexicon_tags_47 },
    { 0x17c14737, SUC_lexicon_tags_14 },
    { 0x46014738, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa861473c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe41473e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8814741, SUC_lexicon_tags_37 },
    { 0x28e14742, SUC_lexicon_tags_33 },
    { 0xdc814743, SUC_lexicon_tags_37 },
    { 0x9f414743, SUC_lexicon_tags_16 },
    { 0x33014745, SUC_lexicon_tags_30 },
    { 0xfc814746, SUC_lexicon_tags_91 },
    { 0xb1814747, SUC_lexicon_tags_23 },
    { 0x3a014748, SUC_lexicon_tags_4 },
    { 0xd3c14747, SUC_lexicon_tags_25 },
    { 0x15814745, SUC_lexicon_tags_38 },
    { 0xcf414745, SUC_lexicon_tags_11 },
    { 0xdda1474c, SUC_lexicon_tags_17 },
    { 0x5b41474c, SUC_lexicon_tags_151 },
    { 0x6961474e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56414753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63414755, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x58a14757, SUC_lexicon_tags_26 },
    { 0x96614758, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e41475c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8dc1475e, SUC_lexicon_tags_5 },
    { 0x3841475e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa5014761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2014767, SUC_lexicon_tags_2 },
    { 0xd2414768, SUC_lexicon_tags_49 },
    { 0xf3414769, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4661476d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1a21476f, SUC_lexicon_tags_25 },
    { 0x53e14770, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec14773, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e14775, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x25814777, SUC_lexicon_tags_37 },
    { 0x8c14778, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29c1477a, SUC_lexicon_tags_7 },
    { 0x4821477a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1521477f, SUC_lexicon_tags_30 },
    { 0xb8a1477f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54414786, SUC_lexicon_tags_37 },
    { 0x86214787, SUC_lexicon_tags_15 },
    { 0x38a14788, SUC_lexicon_tags_49 },
    { 0x1e214789, SUC_lexicon_tags_16 },
    { 0x1d214788, SUC_lexicon_tags_4 },
    { 0x96e1478b, SUC_lexicon_tags_17 },
    { 0xf2c1478c, SUC_lexicon_tags_11 },
    { 0xa9a1478a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb841478f, SUC_lexicon_tags_45 },
    { 0x44e14790, SUC_lexicon_tags_61 },
    { 0x63414791, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8814797, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x37214799, SUC_lexicon_tags_31 },
    { 0x41a1479a, SUC_lexicon_tags_48 },
    { 0xedc1479a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1901479e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x376147a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae147a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42147ac, SUC_lexicon_tags_0 },
    { 0x186147ad, SUC_lexicon_tags_11 },
    { 0xfd0147ad, SUC_lexicon_tags_12 },
    { 0x66c147ae, SUC_lexicon_tags_12 },
    { 0x626147b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac147b5, SUC_lexicon_tags_48 },
    { 0x962147b6, SUC_lexicon_tags_0 },
    { 0x2147b7, SUC_lexicon_tags_11 },
    { 0x562147b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc147bc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a147bf, SUC_lexicon_tags_10 },
    { 0x942147c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94c147c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9de147c4, SUC_lexicon_tags_1 },
    { 0xe92147c5, SUC_lexicon_tags_37 },
    { 0x3a8147c6, SUC_lexicon_tags_43 },
    { 0x52c147c4, SUC_lexicon_tags_46 },
    { 0x864147c8, SUC_lexicon_tags_13 },
    { 0xa30147c8, SUC_lexicon_tags_7 },
    { 0x746147c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd96147cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e0147ce, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c147d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c147d7, SUC_lexicon_tags_25 },
    { 0x3aa147d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x652147da, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80147de, SUC_lexicon_tags_32 },
    { 0xf88147df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa147e1, SUC_lexicon_tags_37 },
    { 0xce2147e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a147e6, SUC_lexicon_tags_9 },
    { 0x1f8147e7, SUC_lexicon_tags_11 },
    { 0x972147e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba147ed, SUC_lexicon_tags_49 },
    { 0xda4147ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a147f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70147f7, SUC_lexicon_tags_29 },
    { 0x648147f7, SUC_lexicon_tags_1 },
    { 0x3ac147f9, SUC_lexicon_tags_0 },
    { 0xd48147fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e8147fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e14806, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa281480c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc14814, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8d814816, SUC_lexicon_tags_14 },
    { 0x58a14816, SUC_lexicon_tags_27 },
    { 0x28e14818, SUC_lexicon_tags_16 },
    { 0x3e614818, SUC_lexicon_tags_12 },
    { 0xe081481a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa14820, SUC_lexicon_tags_17 },
    { 0xfe14820, SUC_lexicon_tags_17 },
    { 0x17a14821, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac14825, SUC_lexicon_tags_37 },
    { 0xa9014826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8614828, SUC_lexicon_tags_2 },
    { 0xfc614828, SUC_lexicon_tags_17 },
    { 0xfd41482a, SUC_lexicon_tags_17 },
    { 0x3f814829, SUC_lexicon_tags_49 },
    { 0xcc81482c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1361482f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac214832, SUC_lexicon_tags_30 },
    { 0x4ce14833, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28614839, SUC_lexicon_tags_65 },
    { 0x6901483a, SUC_lexicon_tags_33 },
    { 0xaaa1483b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a41483f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd361484b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcdc1484d, SUC_lexicon_tags_61 },
    { 0x5bc1484d, SUC_lexicon_tags_0 },
    { 0xa3c1484d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3614851, SUC_lexicon_tags_8 },
    { 0xf7214851, SUC_lexicon_tags_47 },
    { 0x2a14853, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x60214856, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a81485c, SUC_lexicon_tags_6 },
    { 0x2d81485d, SUC_lexicon_tags_15 },
    { 0xefe1485e, SUC_lexicon_tags_17 },
    { 0x3161485f, SUC_lexicon_tags_17 },
    { 0xf9c1485f, SUC_lexicon_tags_16 },
    { 0x8a14861, SUC_lexicon_tags_15 },
    { 0x2c014862, SUC_lexicon_tags_37 },
    { 0x40814863, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a14866, SUC_lexicon_tags_47 },
    { 0xd2014866, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa1486b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae414874, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a14877, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7414879, SUC_lexicon_tags_19 },
    { 0xe7214879, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5161487c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86214880, SUC_lexicon_tags_2 },
    { 0xcd214881, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1814884, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea14887, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc81488b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f614890, SUC_lexicon_tags_27 },
    { 0xc8414890, SUC_lexicon_tags_4 },
    { 0x7ea14892, SUC_lexicon_tags_16 },
    { 0x52c14893, SUC_lexicon_tags_23 },
    { 0x9e014893, SUC_lexicon_tags_27 },
    { 0xf1614894, SUC_lexicon_tags_37 },
    { 0xd7c14896, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c61489b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c61489f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728148a4, SUC_lexicon_tags_6 },
    { 0x89a148a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4148a8, SUC_lexicon_tags_761 },
    { 0xf9c148a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12148af, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72148b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70148b7, SUC_lexicon_tags_27 },
    { 0x1da148b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02148bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xafe148be, SUC_lexicon_tags_784 },
    { 0, NULL },
    { 0x3f4148c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc148c3, SUC_lexicon_tags_35 },
    { 0xb3e148c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb04148c6, SUC_lexicon_tags_15 },
    { 0x764148c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e148ca, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7ac148cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7e148ce, SUC_lexicon_tags_11 },
    { 0xc32148cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7c148d1, SUC_lexicon_tags_47 },
    { 0x6e2148d1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe4148d4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x118148d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca148dd, SUC_lexicon_tags_11 },
    { 0xb04148dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2148e3, SUC_lexicon_tags_11 },
    { 0x78c148e3, SUC_lexicon_tags_26 },
    { 0xac4148e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be148ec, SUC_lexicon_tags_2 },
    { 0xf92148ed, SUC_lexicon_tags_12 },
    { 0xc44148ed, SUC_lexicon_tags_43 },
    { 0x34a148ef, SUC_lexicon_tags_17 },
    { 0xec0148ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86148f5, SUC_lexicon_tags_170 },
    { 0x346148f6, SUC_lexicon_tags_25 },
    { 0xec2148f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x896148f9, SUC_lexicon_tags_0 },
    { 0x9b4148f9, SUC_lexicon_tags_15 },
    { 0xda148fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4148ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac614905, SUC_lexicon_tags_47 },
    { 0x6614906, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a1490c, SUC_lexicon_tags_26 },
    { 0xfd21490d, SUC_lexicon_tags_17 },
    { 0x4121490e, SUC_lexicon_tags_37 },
    { 0xe9c1490f, SUC_lexicon_tags_4 },
    { 0xf081490f, SUC_lexicon_tags_1 },
    { 0x31e1490c, SUC_lexicon_tags_21 },
    { 0xfca14912, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1414917, SUC_lexicon_tags_11 },
    { 0x7c614918, SUC_lexicon_tags_47 },
    { 0x75014919, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7061491d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99814921, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3814925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a14929, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff81492b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9014930, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a14935, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e14939, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a14940, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c14944, SUC_lexicon_tags_64 },
    { 0x21414945, SUC_lexicon_tags_15 },
    { 0x99c14946, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35a14948, SUC_lexicon_tags_2 },
    { 0xb9814948, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5041494d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67614959, SUC_lexicon_tags_33 },
    { 0x9ae14959, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa14960, SUC_lexicon_tags_27 },
    { 0xa814961, SUC_lexicon_tags_11 },
    { 0xb8a14962, SUC_lexicon_tags_17 },
    { 0xfca14962, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10614965, SUC_lexicon_tags_39 },
    { 0x78c14966, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e14969, SUC_lexicon_tags_62 },
    { 0xfa1496a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87614971, SUC_lexicon_tags_25 },
    { 0xefe14972, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c14976, SUC_lexicon_tags_46 },
    { 0x87814977, SUC_lexicon_tags_33 },
    { 0xe4814978, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f41497a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c1497e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xd4c14980, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ee14982, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89214986, SUC_lexicon_tags_57 },
    { 0xc214987, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x65c14989, SUC_lexicon_tags_31 },
    { 0xaba14989, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b814995, SUC_lexicon_tags_11 },
    { 0x4a014996, SUC_lexicon_tags_37 },
    { 0xd9214995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91614999, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2661499f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xefc149a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0149a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0149aa, SUC_lexicon_tags_10 },
    { 0x850149ab, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xf88149ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6149b0, SUC_lexicon_tags_2 },
    { 0x476149b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70149b4, SUC_lexicon_tags_17 },
    { 0xea2149b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe04149b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba149b9, SUC_lexicon_tags_15 },
    { 0x944149b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c149c1, SUC_lexicon_tags_23 },
    { 0x800149c1, SUC_lexicon_tags_7 },
    { 0x4ae149c1, SUC_lexicon_tags_32 },
    { 0x304149c4, SUC_lexicon_tags_12 },
    { 0x17c149c5, SUC_lexicon_tags_17 },
    { 0x6a2149c6, SUC_lexicon_tags_47 },
    { 0xd56149c7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xfce149c9, SUC_lexicon_tags_11 },
    { 0xc42149ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de149ce, SUC_lexicon_tags_55 },
    { 0x814149cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0149d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee149d5, SUC_lexicon_tags_33 },
    { 0x9a149d6, SUC_lexicon_tags_32 },
    { 0xda0149d5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x584149d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x838149dc, SUC_lexicon_tags_6 },
    { 0x41e149dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb52149df, SUC_lexicon_tags_17 },
    { 0x806149e0, SUC_lexicon_tags_38 },
    { 0xf40149e1, SUC_lexicon_tags_68 },
    { 0xd4e149e2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x824149e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad6149e6, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc149e9, SUC_lexicon_tags_11 },
    { 0x792149e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1ea149ec, SUC_lexicon_tags_25 },
    { 0x710149ec, SUC_lexicon_tags_26 },
    { 0x4ca149ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d8149f0, SUC_lexicon_tags_4 },
    { 0x292149f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be149f4, SUC_lexicon_tags_12 },
    { 0xa2c149f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde149f8, SUC_lexicon_tags_34 },
    { 0xf5c149f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd18149fb, SUC_lexicon_tags_33 },
    { 0x758149fb, SUC_lexicon_tags_124 },
    { 0xeb4149fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b4149ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56814a03, SUC_lexicon_tags_6 },
    { 0x71e14a04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a614a09, SUC_lexicon_tags_57 },
    { 0x26814a0a, SUC_lexicon_tags_6 },
    { 0xea414a0b, SUC_lexicon_tags_25 },
    { 0xf1c14a0c, SUC_lexicon_tags_19 },
    { 0xa7414a0d, SUC_lexicon_tags_6 },
    { 0x41614a09, SUC_lexicon_tags_6 },
    { 0x98214a0f, SUC_lexicon_tags_1 },
    { 0x4dc14a10, SUC_lexicon_tags_28 },
    { 0x31e14a0f, SUC_lexicon_tags_12 },
    { 0xf3814a12, SUC_lexicon_tags_25 },
    { 0xf4614a0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f014a17, SUC_lexicon_tags_32 },
    { 0x8f214a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a14a1d, SUC_lexicon_tags_7 },
    { 0x7b014a1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3614a20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf014a22, SUC_lexicon_tags_21 },
    { 0xa1a14a23, SUC_lexicon_tags_6 },
    { 0xa6c14a24, SUC_lexicon_tags_15 },
    { 0xaf614a24, SUC_lexicon_tags_6 },
    { 0x26414a26, SUC_lexicon_tags_59 },
    { 0x25214a27, SUC_lexicon_tags_1 },
    { 0xea014a26, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x70614a2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21814a2f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8b414a31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e414a33, SUC_lexicon_tags_17 },
    { 0x54a14a34, SUC_lexicon_tags_7 },
    { 0x62414a34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf014a37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x54c14a39, SUC_lexicon_tags_11 },
    { 0xc1214a39, SUC_lexicon_tags_15 },
    { 0x63414a3b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3814a3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2814a40, SUC_lexicon_tags_45 },
    { 0x72614a40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a14a44, SUC_lexicon_tags_49 },
    { 0x97a14a45, SUC_lexicon_tags_25 },
    { 0xab414a46, SUC_lexicon_tags_1 },
    { 0xd5814a45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e14a4a, SUC_lexicon_tags_7 },
    { 0x5b214a4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce014a4e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec14a52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b014a54, SUC_lexicon_tags_77 },
    { 0xf7214a55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de14a59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76c14a5b, SUC_lexicon_tags_75 },
    { 0x86c14a5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc14a61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fe14a63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcca14a65, SUC_lexicon_tags_15 },
    { 0x8c614a66, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa614a6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5da14a6c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe14a6f, SUC_lexicon_tags_100 },
    { 0x6f014a70, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8b614a72, SUC_lexicon_tags_10 },
    { 0x18e14a72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59614a76, SUC_lexicon_tags_6 },
    { 0x80e14a77, SUC_lexicon_tags_0 },
    { 0x41014a76, SUC_lexicon_tags_25 },
    { 0x3b214a77, SUC_lexicon_tags_26 },
    { 0xbdc14a77, SUC_lexicon_tags_1 },
    { 0xdda14a78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be14a82, SUC_lexicon_tags_11 },
    { 0xde614a82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79014a87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c14a93, SUC_lexicon_tags_23 },
    { 0x3d614a94, SUC_lexicon_tags_47 },
    { 0x36814a95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb614a97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be14a9a, SUC_lexicon_tags_60 },
    { 0x35014a9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5814a9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c214aa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44614aa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb414aa8, SUC_lexicon_tags_38 },
    { 0x11414aa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5814aab, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5014ab0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25014ab3, SUC_lexicon_tags_25 },
    { 0x42014ab3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad014abc, SUC_lexicon_tags_39 },
    { 0x72414abc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4414ac0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20414ac2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d814ac9, SUC_lexicon_tags_17 },
    { 0x2be14ac9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c614acf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc14ad4, SUC_lexicon_tags_33 },
    { 0x21e14ad4, SUC_lexicon_tags_9 },
    { 0xc2614ad6, SUC_lexicon_tags_38 },
    { 0xfe014ad7, SUC_lexicon_tags_11 },
    { 0x5a14ad8, SUC_lexicon_tags_32 },
    { 0x55414ad9, SUC_lexicon_tags_31 },
    { 0x10c14ad9, SUC_lexicon_tags_25 },
    { 0x28014adb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc614ade, SUC_lexicon_tags_25 },
    { 0xa4414adf, SUC_lexicon_tags_1 },
    { 0x1b014adf, SUC_lexicon_tags_8 },
    { 0x4e614ae1, SUC_lexicon_tags_48 },
    { 0x95e14ae2, SUC_lexicon_tags_38 },
    { 0x78614adf, SUC_lexicon_tags_25 },
    { 0x30c14ae4, SUC_lexicon_tags_5 },
    { 0xb4c14ae5, SUC_lexicon_tags_47 },
    { 0x69414ae4, SUC_lexicon_tags_14 },
    { 0x15a14ae5, SUC_lexicon_tags_2 },
    { 0xd8214ae6, SUC_lexicon_tags_37 },
    { 0x4b414ae9, SUC_lexicon_tags_1 },
    { 0x86814ae7, SUC_lexicon_tags_38 },
    { 0xe4014aeb, SUC_lexicon_tags_14 },
    { 0x6b214aec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x60414aef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe14af4, SUC_lexicon_tags_23 },
    { 0xc8814af5, SUC_lexicon_tags_37 },
    { 0xf7a14af6, SUC_lexicon_tags_16 },
    { 0x37c14af6, SUC_lexicon_tags_10 },
    { 0x5ec14af8, SUC_lexicon_tags_23 },
    { 0x5d014af6, SUC_lexicon_tags_15 },
    { 0x28214af6, SUC_lexicon_tags_77 },
    { 0xfb614afb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc14b01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec14b04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9614b08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce14b0c, SUC_lexicon_tags_11 },
    { 0x80c14b0d, SUC_lexicon_tags_15 },
    { 0xe6e14b0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43014b10, SUC_lexicon_tags_0 },
    { 0x61814b11, SUC_lexicon_tags_15 },
    { 0x1c014b10, SUC_lexicon_tags_19 },
    { 0x47a14b13, SUC_lexicon_tags_11 },
    { 0x12414b14, SUC_lexicon_tags_43 },
    { 0x50614b14, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb9414b17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1014b1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96c14b1c, SUC_lexicon_tags_17 },
    { 0x4f614b1d, SUC_lexicon_tags_66 },
    { 0x99614b1d, SUC_lexicon_tags_0 },
    { 0xd0a14b1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4d614b21, SUC_lexicon_tags_32 },
    { 0xa8c14b22, SUC_lexicon_tags_8 },
    { 0x99214b22, SUC_lexicon_tags_38 },
    { 0x8f614b24, SUC_lexicon_tags_7 },
    { 0x9c814b24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56e14b27, SUC_lexicon_tags_49 },
    { 0xdd814b27, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac14b2b, SUC_lexicon_tags_25 },
    { 0x5d414b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c14b35, SUC_lexicon_tags_0 },
    { 0xa7014b36, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5e014b38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ba14b3a, SUC_lexicon_tags_4 },
    { 0xb0014b3a, SUC_lexicon_tags_29 },
    { 0xbb614b3c, SUC_lexicon_tags_17 },
    { 0x6ce14b3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce14b40, SUC_lexicon_tags_55 },
    { 0xb9614b41, SUC_lexicon_tags_29 },
    { 0xa7e14b42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1014b47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ce14b49, SUC_lexicon_tags_25 },
    { 0x1bc14b4a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a14b4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17214b50, SUC_lexicon_tags_34 },
    { 0x70c14b51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2ae14b53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc214b56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb214b59, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9414b5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43214b63, SUC_lexicon_tags_31 },
    { 0x66214b64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3f014b66, SUC_lexicon_tags_6 },
    { 0xbfa14b67, SUC_lexicon_tags_11 },
    { 0xfc14b68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e14b6c, SUC_lexicon_tags_21 },
    { 0x7d814b6c, SUC_lexicon_tags_37 },
    { 0x65014b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67614b70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a214b74, SUC_lexicon_tags_0 },
    { 0x4a14b74, SUC_lexicon_tags_2 },
    { 0xd4c14b74, SUC_lexicon_tags_4 },
    { 0x93814b74, SUC_lexicon_tags_37 },
    { 0xfc614b78, SUC_lexicon_tags_31 },
    { 0x4414b79, SUC_lexicon_tags_9 },
    { 0xa4c14b78, SUC_lexicon_tags_1 },
    { 0x35e14b7a, SUC_lexicon_tags_17 },
    { 0x75014b7c, SUC_lexicon_tags_7 },
    { 0x4aa14b77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a814b83, SUC_lexicon_tags_25 },
    { 0x4cc14b84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e214b87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32014b8b, SUC_lexicon_tags_37 },
    { 0xa0214b8c, SUC_lexicon_tags_11 },
    { 0x76e14b8d, SUC_lexicon_tags_0 },
    { 0x25e14b8e, SUC_lexicon_tags_37 },
    { 0x13a14b8e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c14b9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x69014b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85014ba2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59814ba5, SUC_lexicon_tags_6 },
    { 0x4e814ba5, SUC_lexicon_tags_31 },
    { 0x13814ba7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb7614ba9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c14bac, SUC_lexicon_tags_28 },
    { 0xf7214bad, SUC_lexicon_tags_5 },
    { 0x79e14bad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf414bb3, SUC_lexicon_tags_21 },
    { 0xc8614bb4, SUC_lexicon_tags_7 },
    { 0xe0814bb5, SUC_lexicon_tags_45 },
    { 0x83014bb5, SUC_lexicon_tags_37 },
    { 0xba14bb7, SUC_lexicon_tags_1 },
    { 0xe2014bb6, SUC_lexicon_tags_28 },
    { 0xaa414bb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73414bbc, SUC_lexicon_tags_15 },
    { 0x7ca14bbd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x23a14bbf, SUC_lexicon_tags_23 },
    { 0x32c14bc0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a14bc3, SUC_lexicon_tags_31 },
    { 0xd5814bc4, SUC_lexicon_tags_17 },
    { 0x96e14bc4, SUC_lexicon_tags_25 },
    { 0xdb014bc4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c214bca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc14bdb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59814bec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5614bf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7da14bf2, SUC_lexicon_tags_28 },
    { 0x17014bf3, SUC_lexicon_tags_1 },
    { 0xcd214bf4, SUC_lexicon_tags_65 },
    { 0x98814bf5, SUC_lexicon_tags_5 },
    { 0xaa814bf6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73814bfb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x45014bfe, SUC_lexicon_tags_26 },
    { 0xbc14bfe, SUC_lexicon_tags_11 },
    { 0x71e14bff, SUC_lexicon_tags_9 },
    { 0xd3a14c01, SUC_lexicon_tags_6 },
    { 0xd6014c02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de14c07, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a014c0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da14c0d, SUC_lexicon_tags_45 },
    { 0x814c0e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1c214c10, SUC_lexicon_tags_25 },
    { 0xbf214c10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be14c14, SUC_lexicon_tags_4 },
    { 0x3ca14c15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0a14c17, SUC_lexicon_tags_23 },
    { 0x33414c17, SUC_lexicon_tags_26 },
    { 0xbbe14c19, SUC_lexicon_tags_11 },
    { 0xcc414c1a, SUC_lexicon_tags_34 },
    { 0x1fc14c1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc614c1f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x67614c22, SUC_lexicon_tags_25 },
    { 0xc0214c23, SUC_lexicon_tags_7 },
    { 0x22c14c23, SUC_lexicon_tags_29 },
    { 0x2a814c25, SUC_lexicon_tags_25 },
    { 0x7dc14c26, SUC_lexicon_tags_39 },
    { 0x64014c22, SUC_lexicon_tags_17 },
    { 0xe1e14c27, SUC_lexicon_tags_7 },
    { 0x6a014c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80014c2c, SUC_lexicon_tags_37 },
    { 0xdae14c2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92e14c2f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x20a14c31, SUC_lexicon_tags_17 },
    { 0x3c814c31, SUC_lexicon_tags_29 },
    { 0xd5a14c31, SUC_lexicon_tags_15 },
    { 0x5c414c33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc2e14c36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5214c39, SUC_lexicon_tags_25 },
    { 0x21814c3a, SUC_lexicon_tags_12 },
    { 0x51014c3a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x61a14c3d, SUC_lexicon_tags_39 },
    { 0xd4814c3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be14c45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xacc14c47, SUC_lexicon_tags_7 },
    { 0xc5a14c48, SUC_lexicon_tags_47 },
    { 0xede14c49, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8014c4c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d414c4f, SUC_lexicon_tags_18 },
    { 0x12014c50, SUC_lexicon_tags_12 },
    { 0x54414c50, SUC_lexicon_tags_15 },
    { 0xf4214c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c414c54, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x78614c56, SUC_lexicon_tags_44 },
    { 0x97214c56, SUC_lexicon_tags_49 },
    { 0x2e614c58, SUC_lexicon_tags_15 },
    { 0xac014c59, SUC_lexicon_tags_37 },
    { 0x7c014c58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x70614c5d, SUC_lexicon_tags_77 },
    { 0x71014c5e, SUC_lexicon_tags_51 },
    { 0x49214c5f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a14c63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd614c67, SUC_lexicon_tags_1 },
    { 0xcb814c67, SUC_lexicon_tags_37 },
    { 0x40414c67, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x82214c6b, SUC_lexicon_tags_45 },
    { 0x62414c6b, SUC_lexicon_tags_49 },
    { 0xb414c6d, SUC_lexicon_tags_15 },
    { 0x52c14c6e, SUC_lexicon_tags_1 },
    { 0x1fe14c6e, SUC_lexicon_tags_47 },
    { 0xea014c70, SUC_lexicon_tags_35 },
    { 0xcfc14c70, SUC_lexicon_tags_1 },
    { 0x4014c6f, SUC_lexicon_tags_15 },
    { 0x92214c6c, SUC_lexicon_tags_12 },
    { 0xd5a14c6d, SUC_lexicon_tags_29 },
    { 0xc9414c75, SUC_lexicon_tags_2 },
    { 0xd4c14c70, SUC_lexicon_tags_43 },
    { 0x48c14c6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58614c7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63614c7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c14c84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3214c87, SUC_lexicon_tags_11 },
    { 0x35614c88, SUC_lexicon_tags_64 },
    { 0xe8a14c89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e14c8d, SUC_lexicon_tags_20 },
    { 0xfe014c8d, SUC_lexicon_tags_25 },
    { 0x1414c8e, SUC_lexicon_tags_3 },
    { 0x23814c8e, SUC_lexicon_tags_2 },
    { 0xed214c91, SUC_lexicon_tags_6 },
    { 0xd614c8f, SUC_lexicon_tags_49 },
    { 0xd9414c93, SUC_lexicon_tags_17 },
    { 0x43c14c94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17814c99, SUC_lexicon_tags_12 },
    { 0xc6414c9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c014c9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce614ca5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3014ca9, SUC_lexicon_tags_11 },
    { 0xc9014ca9, SUC_lexicon_tags_10 },
    { 0x5d014cab, SUC_lexicon_tags_11 },
    { 0xd1414ca9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a14caf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4b414cb1, SUC_lexicon_tags_103 },
    { 0x66814cb2, SUC_lexicon_tags_2 },
    { 0x77814cb3, SUC_lexicon_tags_23 },
    { 0x81a14cb2, SUC_lexicon_tags_17 },
    { 0x63814cb3, SUC_lexicon_tags_17 },
    { 0xe1414cb3, SUC_lexicon_tags_37 },
    { 0x79414cb1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf214cba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10c14cbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20414cbe, SUC_lexicon_tags_39 },
    { 0x91814cbe, SUC_lexicon_tags_104 },
    { 0x64614cc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32814cc2, SUC_lexicon_tags_17 },
    { 0xabe14cc3, SUC_lexicon_tags_11 },
    { 0xfee14cc4, SUC_lexicon_tags_25 },
    { 0x67e14cc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c14cc9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x24214ccb, SUC_lexicon_tags_68 },
    { 0xc5e14ccc, SUC_lexicon_tags_37 },
    { 0x4214ccd, SUC_lexicon_tags_11 },
    { 0x68e14cce, SUC_lexicon_tags_25 },
    { 0xb1014ccf, SUC_lexicon_tags_11 },
    { 0x30c14ccf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42414cd2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x65614cd4, SUC_lexicon_tags_63 },
    { 0x3814cd5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xad814cd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e014cdd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae14ce0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8614ce5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28c14ce7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be14ceb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef014ced, SUC_lexicon_tags_15 },
    { 0xb8814cee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9614cf2, SUC_lexicon_tags_25 },
    { 0xacc14cf2, SUC_lexicon_tags_23 },
    { 0x36014cf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36414cf8, SUC_lexicon_tags_11 },
    { 0x80c14cf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a14cfc, SUC_lexicon_tags_37 },
    { 0x81414cfd, SUC_lexicon_tags_29 },
    { 0x6c014cfe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e14d01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc14d06, SUC_lexicon_tags_16 },
    { 0x30e14d07, SUC_lexicon_tags_19 },
    { 0xbc414d07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd8e14d0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ce14d0c, SUC_lexicon_tags_11 },
    { 0x82014d0c, SUC_lexicon_tags_7 },
    { 0x30a14d0c, SUC_lexicon_tags_44 },
    { 0x2ae14d0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c14d15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe0214d17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47214d1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a014d1e, SUC_lexicon_tags_17 },
    { 0x55614d1e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3214d23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdc14d25, SUC_lexicon_tags_11 },
    { 0xeba14d26, SUC_lexicon_tags_25 },
    { 0x72614d27, SUC_lexicon_tags_17 },
    { 0x20c14d27, SUC_lexicon_tags_49 },
    { 0x20014d28, SUC_lexicon_tags_11 },
    { 0x2a814d28, SUC_lexicon_tags_17 },
    { 0xfec14d27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x26a14d2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4414d32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91614d34, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x49414d37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd814d39, SUC_lexicon_tags_7 },
    { 0x4614d3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d614d3d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a14d43, SUC_lexicon_tags_37 },
    { 0x12014d44, SUC_lexicon_tags_26 },
    { 0x65a14d45, SUC_lexicon_tags_66 },
    { 0x5ee14d45, SUC_lexicon_tags_30 },
    { 0xa0614d45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb814d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34214d4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x25014d50, SUC_lexicon_tags_14 },
    { 0xdd014d51, SUC_lexicon_tags_48 },
    { 0x4d214d52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a14d56, SUC_lexicon_tags_38 },
    { 0x4bc14d57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8614d59, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be14d5c, SUC_lexicon_tags_37 },
    { 0xa3214d5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a14d5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23414d61, SUC_lexicon_tags_46 },
    { 0xe214d62, SUC_lexicon_tags_30 },
    { 0xa9a14d62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x94014d66, SUC_lexicon_tags_7 },
    { 0x3c414d67, SUC_lexicon_tags_39 },
    { 0x52414d68, SUC_lexicon_tags_33 },
    { 0xee14d69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d014d6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x95c14d6e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c414d72, SUC_lexicon_tags_63 },
    { 0xe8e14d73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8d214d75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c14d7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1c14d7f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a414d87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4da14d89, SUC_lexicon_tags_8 },
    { 0x31014d8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47614d91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c14d94, SUC_lexicon_tags_16 },
    { 0x5f814d95, SUC_lexicon_tags_12 },
    { 0x3f814d95, SUC_lexicon_tags_63 },
    { 0x38a14d94, SUC_lexicon_tags_27 },
    { 0xf3614d98, SUC_lexicon_tags_12 },
    { 0xe2e14d99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6014d9d, SUC_lexicon_tags_12 },
    { 0x75c14d9e, SUC_lexicon_tags_17 },
    { 0x26414d9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4014da4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5014daa, SUC_lexicon_tags_4 },
    { 0x89c14dab, SUC_lexicon_tags_1 },
    { 0x25c14dab, SUC_lexicon_tags_37 },
    { 0xbd414dad, SUC_lexicon_tags_33 },
    { 0x7b414dae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c14db3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7814db7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd814db9, SUC_lexicon_tags_29 },
    { 0x5c214db9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac14dc3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1d014dc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81414dc8, SUC_lexicon_tags_68 },
    { 0xb5014dc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30814dcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29e14dcd, SUC_lexicon_tags_23 },
    { 0x98414dcd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e414dd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a614dd3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e14dd8, SUC_lexicon_tags_9 },
    { 0x5c014dd9, SUC_lexicon_tags_17 },
    { 0x54014dd8, SUC_lexicon_tags_30 },
    { 0xee414ddb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5d214ddd, SUC_lexicon_tags_4 },
    { 0x96414dde, SUC_lexicon_tags_17 },
    { 0x62014dde, SUC_lexicon_tags_25 },
    { 0xa7c14dde, SUC_lexicon_tags_68 },
    { 0x20614dde, SUC_lexicon_tags_37 },
    { 0x81a14de2, SUC_lexicon_tags_11 },
    { 0x6ca14de1, SUC_lexicon_tags_15 },
    { 0xb3e14de4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27014de9, SUC_lexicon_tags_1 },
    { 0x1b614dea, SUC_lexicon_tags_8 },
    { 0x2a214de9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6014ded, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62214df0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da14df4, SUC_lexicon_tags_26 },
    { 0xd6214df5, SUC_lexicon_tags_31 },
    { 0x1c814df5, SUC_lexicon_tags_6 },
    { 0xfe14df7, SUC_lexicon_tags_25 },
    { 0x67014df7, SUC_lexicon_tags_7 },
    { 0xe6614df9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xcde14dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbae14dfd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc14e03, SUC_lexicon_tags_38 },
    { 0x40814e03, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1414e10, SUC_lexicon_tags_60 },
    { 0x60214e10, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd014e17, SUC_lexicon_tags_17 },
    { 0xa0a14e17, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c14e1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6814e1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89814e21, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x84a14e23, SUC_lexicon_tags_8 },
    { 0x7e414e23, SUC_lexicon_tags_1 },
    { 0xba014e25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e14e29, SUC_lexicon_tags_25 },
    { 0x39c14e2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c14e2d, SUC_lexicon_tags_48 },
    { 0xd8414e2e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb0014e30, SUC_lexicon_tags_1 },
    { 0x13214e31, SUC_lexicon_tags_82 },
    { 0xa2214e32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xca14e35, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2014e37, SUC_lexicon_tags_37 },
    { 0xa6014e38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a14e3d, SUC_lexicon_tags_11 },
    { 0xab014e3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2414e45, SUC_lexicon_tags_33 },
    { 0xf6814e46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55814e48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a14e4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ba14e4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37214e56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a214e5b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x97014e5d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x86414e60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x37814e62, SUC_lexicon_tags_6 },
    { 0xda14e63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10814e67, SUC_lexicon_tags_1 },
    { 0x6dc14e68, SUC_lexicon_tags_16 },
    { 0xe8a14e68, SUC_lexicon_tags_15 },
    { 0x67e14e6a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8414e6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf0c14e6f, SUC_lexicon_tags_1 },
    { 0x7014e70, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8214e74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac14e77, SUC_lexicon_tags_14 },
    { 0x53814e78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3ec14e7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c814e7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a214e82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d614e84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc14e87, SUC_lexicon_tags_2 },
    { 0xffa14e88, SUC_lexicon_tags_25 },
    { 0x39214e88, SUC_lexicon_tags_35 },
    { 0x31614e87, SUC_lexicon_tags_2 },
    { 0xb814e8b, SUC_lexicon_tags_11 },
    { 0x59814e8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9414e8e, SUC_lexicon_tags_94 },
    { 0x79e14e8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd7414e91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1c614e93, SUC_lexicon_tags_33 },
    { 0x7b414e94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x45414e96, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c14e9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e614e9f, SUC_lexicon_tags_14 },
    { 0xe0214ea0, SUC_lexicon_tags_48 },
    { 0x82614ea1, SUC_lexicon_tags_25 },
    { 0x18414ea1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd2414ea4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x24614ea7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e414eac, SUC_lexicon_tags_25 },
    { 0x40214ead, SUC_lexicon_tags_32 },
    { 0xec614ead, SUC_lexicon_tags_25 },
    { 0xb4e14ead, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x74614eb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e414eb5, SUC_lexicon_tags_15 },
    { 0x8b814eb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e14eba, SUC_lexicon_tags_15 },
    { 0xd3a14eba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c14ec0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff814ec2, SUC_lexicon_tags_11 },
    { 0xe6e14ec2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c14eca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c214ecc, SUC_lexicon_tags_9 },
    { 0x37814ecd, SUC_lexicon_tags_2 },
    { 0x71814ece, SUC_lexicon_tags_1 },
    { 0x1bc14ecc, SUC_lexicon_tags_15 },
    { 0x20214ed0, SUC_lexicon_tags_12 },
    { 0x99214ecc, SUC_lexicon_tags_7 },
    { 0xc9614ece, SUC_lexicon_tags_84 },
    { 0xc3614ed1, SUC_lexicon_tags_25 },
    { 0x16a14ed4, SUC_lexicon_tags_29 },
    { 0x61e14ed5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbe614ed7, SUC_lexicon_tags_54 },
    { 0xc9814ed8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce814ede, SUC_lexicon_tags_37 },
    { 0x41014edf, SUC_lexicon_tags_19 },
    { 0x9dc14ee0, SUC_lexicon_tags_14 },
    { 0x7dc14edf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4814ee8, SUC_lexicon_tags_11 },
    { 0x28414ee9, SUC_lexicon_tags_17 },
    { 0x30014ee9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16214eee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73214ef0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x64814ef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37c14ef5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8614ef7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49214ef9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c14efe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e14f02, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x78614f04, SUC_lexicon_tags_9 },
    { 0xe7614f04, SUC_lexicon_tags_11 },
    { 0x2c414f06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3e14f08, SUC_lexicon_tags_12 },
    { 0xb3214f09, SUC_lexicon_tags_28 },
    { 0x6c14f09, SUC_lexicon_tags_19 },
    { 0xc4014f08, SUC_lexicon_tags_11 },
    { 0xa9814f0c, SUC_lexicon_tags_28 },
    { 0xefe14f0c, SUC_lexicon_tags_16 },
    { 0xcd214f0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac14f11, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe2814f13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea14f1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b614f20, SUC_lexicon_tags_23 },
    { 0xdb214f21, SUC_lexicon_tags_2 },
    { 0x1214f22, SUC_lexicon_tags_12 },
    { 0xa7414f23, SUC_lexicon_tags_26 },
    { 0xd7c14f22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a14f27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15214f29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbce14f2b, SUC_lexicon_tags_37 },
    { 0xad014f2c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa814f32, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd014f35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a14f38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc614f3b, SUC_lexicon_tags_43 },
    { 0xc3214f3c, SUC_lexicon_tags_14 },
    { 0xcae14f3b, SUC_lexicon_tags_11 },
    { 0xbe614f3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x73414f41, SUC_lexicon_tags_12 },
    { 0x17a14f42, SUC_lexicon_tags_14 },
    { 0xf7214f42, SUC_lexicon_tags_0 },
    { 0xba414f44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae14f48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ec14f4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c14f52, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa3014f54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb814f5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee14f63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18214f69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e14f6d, SUC_lexicon_tags_14 },
    { 0x69a14f6e, SUC_lexicon_tags_37 },
    { 0x97814f6f, SUC_lexicon_tags_25 },
    { 0xe2414f6f, SUC_lexicon_tags_32 },
    { 0x9a414f6e, SUC_lexicon_tags_79 },
    { 0x9e214f6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a14f78, SUC_lexicon_tags_12 },
    { 0x86614f79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5814f80, SUC_lexicon_tags_46 },
    { 0x1e814f80, SUC_lexicon_tags_37 },
    { 0x3aa14f80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fc14f84, SUC_lexicon_tags_11 },
    { 0xe4e14f85, SUC_lexicon_tags_37 },
    { 0xe2a14f85, SUC_lexicon_tags_7 },
    { 0x27814f86, SUC_lexicon_tags_63 },
    { 0x7d014f88, SUC_lexicon_tags_16 },
    { 0x49814f89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x43814f8b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x44614f8e, SUC_lexicon_tags_16 },
    { 0x50814f8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x73614f91, SUC_lexicon_tags_15 },
    { 0x2e414f91, SUC_lexicon_tags_17 },
    { 0xc7214f93, SUC_lexicon_tags_16 },
    { 0x7ea14f94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfba14f96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5014f9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0a14f9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf814fa3, SUC_lexicon_tags_68 },
    { 0xca414fa3, SUC_lexicon_tags_14 },
    { 0x23014fa5, SUC_lexicon_tags_1 },
    { 0xd3814fa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85414fa8, SUC_lexicon_tags_11 },
    { 0xcac14fa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b814fb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c14fb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a14fb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35014fbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd6414fbe, SUC_lexicon_tags_15 },
    { 0x52814fbf, SUC_lexicon_tags_14 },
    { 0xbca14fbf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda14fc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c614fca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c14fd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1414fd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb814fd8, SUC_lexicon_tags_12 },
    { 0xbd814fd9, SUC_lexicon_tags_17 },
    { 0xf4814fda, SUC_lexicon_tags_17 },
    { 0xd3e14fda, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53814fe2, SUC_lexicon_tags_17 },
    { 0xf1414fe3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c14feb, SUC_lexicon_tags_10 },
    { 0x78614fec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e14fef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7814ff4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82414ffa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18214ffe, SUC_lexicon_tags_37 },
    { 0xe4414ffe, SUC_lexicon_tags_7 },
    { 0x80015000, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a15006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaee15008, SUC_lexicon_tags_61 },
    { 0x5ee15009, SUC_lexicon_tags_7 },
    { 0x29e1500a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdc21500c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5015011, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c15018, SUC_lexicon_tags_2 },
    { 0x32815019, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde15024, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0615029, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf321502c, SUC_lexicon_tags_1494 },
    { 0xce01502d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad615031, SUC_lexicon_tags_47 },
    { 0xdc815032, SUC_lexicon_tags_38 },
    { 0xfe015033, SUC_lexicon_tags_26 },
    { 0xcb015033, SUC_lexicon_tags_82 },
    { 0x68415034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e615038, SUC_lexicon_tags_49 },
    { 0x99a15038, SUC_lexicon_tags_33 },
    { 0x86c1503a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a1503d, SUC_lexicon_tags_25 },
    { 0x4dc1503e, SUC_lexicon_tags_2 },
    { 0xf261503f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a15042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e15046, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a1504b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd521504d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x181504f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11015053, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c1505b, SUC_lexicon_tags_19 },
    { 0xc61505c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84815063, SUC_lexicon_tags_11 },
    { 0xdd215064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be15074, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x65a15076, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65015078, SUC_lexicon_tags_48 },
    { 0x19815079, SUC_lexicon_tags_56 },
    { 0x5a01507a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcee1507c, SUC_lexicon_tags_15 },
    { 0x7441507d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5161507f, SUC_lexicon_tags_0 },
    { 0xe615080, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a15086, SUC_lexicon_tags_103 },
    { 0x28415087, SUC_lexicon_tags_75 },
    { 0x30e15088, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd601508a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb621508f, SUC_lexicon_tags_14 },
    { 0x5381508f, SUC_lexicon_tags_34 },
    { 0x6d215090, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x49815094, SUC_lexicon_tags_17 },
    { 0xbf015095, SUC_lexicon_tags_33 },
    { 0xb2015095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4401509d, SUC_lexicon_tags_17 },
    { 0x6e1509d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x526150a0, SUC_lexicon_tags_65 },
    { 0xc9e150a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca150a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x784150a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8150aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8150af, SUC_lexicon_tags_17 },
    { 0x28a150af, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62150b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce150b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x970150b9, SUC_lexicon_tags_28 },
    { 0xa6a150b9, SUC_lexicon_tags_11 },
    { 0xda150b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc150be, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680150c3, SUC_lexicon_tags_37 },
    { 0xdc4150c4, SUC_lexicon_tags_97 },
    { 0xd98150c5, SUC_lexicon_tags_38 },
    { 0x2fe150c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b0150c8, SUC_lexicon_tags_118 },
    { 0x966150c9, SUC_lexicon_tags_32 },
    { 0x1aa150c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c150d2, SUC_lexicon_tags_7 },
    { 0x8f4150d3, SUC_lexicon_tags_30 },
    { 0x2e6150d4, SUC_lexicon_tags_17 },
    { 0xc4e150d2, SUC_lexicon_tags_35 },
    { 0xbc2150d6, SUC_lexicon_tags_17 },
    { 0x7fe150d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38150da, SUC_lexicon_tags_1 },
    { 0x498150da, SUC_lexicon_tags_8 },
    { 0xe00150dc, SUC_lexicon_tags_1 },
    { 0x792150dd, SUC_lexicon_tags_62 },
    { 0x116150dd, SUC_lexicon_tags_37 },
    { 0xf3e150dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x816150e1, SUC_lexicon_tags_49 },
    { 0x980150e1, SUC_lexicon_tags_61 },
    { 0xca2150e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x872150e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58c150e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0150f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a150f4, SUC_lexicon_tags_2 },
    { 0x8e0150f5, SUC_lexicon_tags_30 },
    { 0x774150f6, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x188150f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40a150fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x586150fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x78615101, SUC_lexicon_tags_47 },
    { 0x81e15101, SUC_lexicon_tags_32 },
    { 0x21415101, SUC_lexicon_tags_14 },
    { 0x2c215104, SUC_lexicon_tags_21 },
    { 0xa9015103, SUC_lexicon_tags_11 },
    { 0xb6015101, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe241510a, SUC_lexicon_tags_1 },
    { 0xf181510a, SUC_lexicon_tags_7 },
    { 0xc41510c, SUC_lexicon_tags_43 },
    { 0x1c01510d, SUC_lexicon_tags_26 },
    { 0xd9e1510e, SUC_lexicon_tags_17 },
    { 0x481510f, SUC_lexicon_tags_6 },
    { 0xe0215110, SUC_lexicon_tags_17 },
    { 0xe8c1510a, SUC_lexicon_tags_25 },
    { 0xd8c15112, SUC_lexicon_tags_14 },
    { 0xc415112, SUC_lexicon_tags_15 },
    { 0x63a15111, SUC_lexicon_tags_43 },
    { 0x6941510c, SUC_lexicon_tags_25 },
    { 0x7b815116, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c1511a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa541511e, SUC_lexicon_tags_25 },
    { 0x7f21511e, SUC_lexicon_tags_12 },
    { 0x9a15120, SUC_lexicon_tags_1 },
    { 0xf601511f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e15129, SUC_lexicon_tags_14 },
    { 0xd0a1512a, SUC_lexicon_tags_14 },
    { 0x42a1512a, SUC_lexicon_tags_12 },
    { 0x701512c, SUC_lexicon_tags_9 },
    { 0x9f61512d, SUC_lexicon_tags_27 },
    { 0xeb81512e, SUC_lexicon_tags_32 },
    { 0xfee1512f, SUC_lexicon_tags_49 },
    { 0x8a615130, SUC_lexicon_tags_7 },
    { 0x4641512f, SUC_lexicon_tags_17 },
    { 0x6d615132, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da15139, SUC_lexicon_tags_13 },
    { 0x42e1513a, SUC_lexicon_tags_1 },
    { 0xc861513b, SUC_lexicon_tags_10 },
    { 0x2dc1513b, SUC_lexicon_tags_37 },
    { 0xf7a1513d, SUC_lexicon_tags_17 },
    { 0xb701513c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7015140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4415143, SUC_lexicon_tags_151 },
    { 0x415143, SUC_lexicon_tags_15 },
    { 0x6c815145, SUC_lexicon_tags_3 },
    { 0x48e15145, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e15150, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da15158, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x7501515a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6aa1515c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc1515f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce15162, SUC_lexicon_tags_33 },
    { 0x30215163, SUC_lexicon_tags_14 },
    { 0x62815163, SUC_lexicon_tags_17 },
    { 0xaa615165, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c15168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd201516c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da1516f, SUC_lexicon_tags_37 },
    { 0x21c15170, SUC_lexicon_tags_4 },
    { 0x6881516f, SUC_lexicon_tags_7 },
    { 0x94a1516f, SUC_lexicon_tags_63 },
    { 0xd4015170, SUC_lexicon_tags_31 },
    { 0x84415170, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e15178, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb581517a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7581517c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5341517f, SUC_lexicon_tags_1 },
    { 0xbb61517f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b615183, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a15188, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f81518b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a41518f, SUC_lexicon_tags_10 },
    { 0x1ba15190, SUC_lexicon_tags_34 },
    { 0x91215191, SUC_lexicon_tags_15 },
    { 0x50615191, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1519d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74151a0, SUC_lexicon_tags_37 },
    { 0xe6151a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8151a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8151a8, SUC_lexicon_tags_45 },
    { 0xd48151a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x332151ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2151b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x340151b2, SUC_lexicon_tags_37 },
    { 0xe4c151b3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a151b6, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a151b9, SUC_lexicon_tags_32 },
    { 0x160151b9, SUC_lexicon_tags_11 },
    { 0x94c151b9, SUC_lexicon_tags_11 },
    { 0x7a0151bc, SUC_lexicon_tags_15 },
    { 0xbce151bc, SUC_lexicon_tags_12 },
    { 0xc32151bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c151c4, SUC_lexicon_tags_37 },
    { 0xbfa151c4, SUC_lexicon_tags_3 },
    { 0xcd4151c6, SUC_lexicon_tags_37 },
    { 0x192151c7, SUC_lexicon_tags_12 },
    { 0x8ae151c8, SUC_lexicon_tags_0 },
    { 0xcd0151c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x47c151cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a151d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa62151d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18151d7, SUC_lexicon_tags_11 },
    { 0xad4151d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcc151da, SUC_lexicon_tags_14 },
    { 0xc66151db, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652151e7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x33a151e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6151ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x822151ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4151f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46c151f3, SUC_lexicon_tags_25 },
    { 0x464151f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74151f8, SUC_lexicon_tags_30 },
    { 0xeb6151f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a6151fb, SUC_lexicon_tags_60 },
    { 0x5ba151fc, SUC_lexicon_tags_12 },
    { 0x42e151fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6c8151ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76415201, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1615205, SUC_lexicon_tags_47 },
    { 0xb2815206, SUC_lexicon_tags_17 },
    { 0x74e15207, SUC_lexicon_tags_17 },
    { 0xd4e15205, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1520b, SUC_lexicon_tags_77 },
    { 0xdea1520c, SUC_lexicon_tags_37 },
    { 0x2281520d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76215214, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3215217, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb601521a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c1521e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb121522d, SUC_lexicon_tags_27 },
    { 0x48a1522e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x615231, SUC_lexicon_tags_65 },
    { 0x80e15232, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb015239, SUC_lexicon_tags_7 },
    { 0x43a15239, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2341523c, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d21523f, SUC_lexicon_tags_11 },
    { 0x1c15240, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e15243, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2815246, SUC_lexicon_tags_11 },
    { 0x25e15246, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x10815249, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1524d, SUC_lexicon_tags_61 },
    { 0xcd01524e, SUC_lexicon_tags_15 },
    { 0x4c41524f, SUC_lexicon_tags_12 },
    { 0xe481524f, SUC_lexicon_tags_32 },
    { 0xd721524f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a15255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b015259, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3801525f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c15262, SUC_lexicon_tags_79 },
    { 0xc6c15262, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x99a15265, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e15268, SUC_lexicon_tags_47 },
    { 0xa215269, SUC_lexicon_tags_77 },
    { 0xcea15268, SUC_lexicon_tags_14 },
    { 0x7d615268, SUC_lexicon_tags_9 },
    { 0x3da1526a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9701526f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2e15271, SUC_lexicon_tags_607 },
    { 0x51c15271, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd0415274, SUC_lexicon_tags_9 },
    { 0xcea15274, SUC_lexicon_tags_7 },
    { 0x72615276, SUC_lexicon_tags_112 },
    { 0x86215277, SUC_lexicon_tags_31 },
    { 0x5fa15277, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd401527b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c1527e, SUC_lexicon_tags_69 },
    { 0x821527f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72415281, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc215284, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e15287, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2521528a, SUC_lexicon_tags_1 },
    { 0x2c01528b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1ee1528d, SUC_lexicon_tags_2 },
    { 0x3661528e, SUC_lexicon_tags_44 },
    { 0x11a1528e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c15294, SUC_lexicon_tags_63 },
    { 0xf3815294, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3815299, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed21529e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xda4152a0, SUC_lexicon_tags_27 },
    { 0x896152a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e152a4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94152a8, SUC_lexicon_tags_17 },
    { 0x74c152a9, SUC_lexicon_tags_31 },
    { 0x11a152aa, SUC_lexicon_tags_9 },
    { 0x7b4152ab, SUC_lexicon_tags_37 },
    { 0xbb2152ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70152af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4152b5, SUC_lexicon_tags_26 },
    { 0x408152b6, SUC_lexicon_tags_37 },
    { 0xbde152b7, SUC_lexicon_tags_0 },
    { 0x378152b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc152be, SUC_lexicon_tags_7 },
    { 0x1e0152bf, SUC_lexicon_tags_27 },
    { 0x262152c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x114152c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa64152c4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c152c7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x510152ca, SUC_lexicon_tags_9 },
    { 0x87a152ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86c152cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25e152cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf28152d1, SUC_lexicon_tags_38 },
    { 0x94c152d2, SUC_lexicon_tags_82 },
    { 0x4de152d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24152d6, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c152db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4152df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce152e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb36152e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0152e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8152ec, SUC_lexicon_tags_37 },
    { 0x912152ed, SUC_lexicon_tags_87 },
    { 0xfe4152ee, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46152f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e152f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae152f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8152fb, SUC_lexicon_tags_34 },
    { 0x54152fc, SUC_lexicon_tags_15 },
    { 0x6b2152fd, SUC_lexicon_tags_37 },
    { 0x64152fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda615302, SUC_lexicon_tags_40 },
    { 0x5e015303, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe961530a, SUC_lexicon_tags_25 },
    { 0x24e1530b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x841530d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5801530f, SUC_lexicon_tags_47 },
    { 0xda81530f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73415312, SUC_lexicon_tags_60 },
    { 0x88c15313, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d215319, SUC_lexicon_tags_8 },
    { 0xfb41531a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfbe1531c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2215320, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf215323, SUC_lexicon_tags_49 },
    { 0xd5215324, SUC_lexicon_tags_23 },
    { 0x7c815323, SUC_lexicon_tags_23 },
    { 0x56815326, SUC_lexicon_tags_29 },
    { 0x90615326, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c1532d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2981532f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68415331, SUC_lexicon_tags_47 },
    { 0xa0c15332, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72015336, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1021533e, SUC_lexicon_tags_33 },
    { 0x5041533f, SUC_lexicon_tags_17 },
    { 0x8301533e, SUC_lexicon_tags_7 },
    { 0x75a15340, SUC_lexicon_tags_12 },
    { 0xcee15342, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd5215344, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff815346, SUC_lexicon_tags_17 },
    { 0xe3215347, SUC_lexicon_tags_25 },
    { 0x4d215347, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c21534a, SUC_lexicon_tags_7 },
    { 0x8aa1534b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c15350, SUC_lexicon_tags_49 },
    { 0xa7415351, SUC_lexicon_tags_7 },
    { 0xa7415352, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae15356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35215358, SUC_lexicon_tags_29 },
    { 0x12c15359, SUC_lexicon_tags_15 },
    { 0x36815358, SUC_lexicon_tags_56 },
    { 0xb501535a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5401535f, SUC_lexicon_tags_29 },
    { 0x8fa15360, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x49e15362, SUC_lexicon_tags_25 },
    { 0xcc215363, SUC_lexicon_tags_37 },
    { 0x65a15364, SUC_lexicon_tags_14 },
    { 0xc9015365, SUC_lexicon_tags_37 },
    { 0x51215366, SUC_lexicon_tags_37 },
    { 0x79615367, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30415370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6215375, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcf015377, SUC_lexicon_tags_8 },
    { 0xbec15378, SUC_lexicon_tags_27 },
    { 0x31c15379, SUC_lexicon_tags_68 },
    { 0x49415377, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51e1537c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1361537e, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e15381, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2815385, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe961538f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95615393, SUC_lexicon_tags_14 },
    { 0xef615393, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7015396, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe15399, SUC_lexicon_tags_23 },
    { 0x8c01539a, SUC_lexicon_tags_283 },
    { 0x89c15399, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8381539d, SUC_lexicon_tags_29 },
    { 0x2801539e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x496153a1, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676153a6, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x57c153a8, SUC_lexicon_tags_65 },
    { 0x7e4153a9, SUC_lexicon_tags_11 },
    { 0xc70153aa, SUC_lexicon_tags_25 },
    { 0xbaa153ab, SUC_lexicon_tags_3 },
    { 0xa50153ac, SUC_lexicon_tags_55 },
    { 0xeac153ab, SUC_lexicon_tags_12 },
    { 0x23e153ae, SUC_lexicon_tags_53 },
    { 0x94153ae, SUC_lexicon_tags_26 },
    { 0xafa153af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4153b3, SUC_lexicon_tags_42 },
    { 0xa28153b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a8153b6, SUC_lexicon_tags_17 },
    { 0x2fc153b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c153ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ea153bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04153c1, SUC_lexicon_tags_25 },
    { 0xe0153c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c153c6, SUC_lexicon_tags_17 },
    { 0xb4153c6, SUC_lexicon_tags_8 },
    { 0xdf4153c8, SUC_lexicon_tags_44 },
    { 0x654153c9, SUC_lexicon_tags_2 },
    { 0xf1e153c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2153cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x628153d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464153d7, SUC_lexicon_tags_19 },
    { 0xecc153d8, SUC_lexicon_tags_7 },
    { 0xfd4153d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0153dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0e153de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4153e4, SUC_lexicon_tags_11 },
    { 0xd88153e5, SUC_lexicon_tags_45 },
    { 0x7c2153e6, SUC_lexicon_tags_61 },
    { 0xb60153e7, SUC_lexicon_tags_229 },
    { 0xdae153e8, SUC_lexicon_tags_11 },
    { 0x36153e9, SUC_lexicon_tags_0 },
    { 0x270153e9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394153f0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918153f6, SUC_lexicon_tags_63 },
    { 0x614153f7, SUC_lexicon_tags_1 },
    { 0xebc153f8, SUC_lexicon_tags_37 },
    { 0x5d2153f8, SUC_lexicon_tags_19 },
    { 0xa54153fa, SUC_lexicon_tags_37 },
    { 0xdbc153fa, SUC_lexicon_tags_66 },
    { 0x6a2153fa, SUC_lexicon_tags_15 },
    { 0xa3e153f8, SUC_lexicon_tags_3 },
    { 0x976153fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63615400, SUC_lexicon_tags_40 },
    { 0x6ae15400, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb215408, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf261540e, SUC_lexicon_tags_87 },
    { 0xa9e1540f, SUC_lexicon_tags_45 },
    { 0xb921540e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74415413, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xdce15415, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7041541b, SUC_lexicon_tags_49 },
    { 0x50c1541c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe15423, SUC_lexicon_tags_14 },
    { 0x90615424, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x12c15426, SUC_lexicon_tags_1 },
    { 0xde615427, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe1542c, SUC_lexicon_tags_65 },
    { 0xb4c1542c, SUC_lexicon_tags_12 },
    { 0x4901542d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x23c15430, SUC_lexicon_tags_9 },
    { 0xf5615431, SUC_lexicon_tags_47 },
    { 0xae15432, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33015434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4415437, SUC_lexicon_tags_25 },
    { 0x37615438, SUC_lexicon_tags_63 },
    { 0x10615437, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861543f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c615441, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x89615444, SUC_lexicon_tags_28 },
    { 0xc1015445, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3a615447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdae15449, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8181544e, SUC_lexicon_tags_65 },
    { 0xe3c1544e, SUC_lexicon_tags_25 },
    { 0x5e615450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba815453, SUC_lexicon_tags_33 },
    { 0xc215453, SUC_lexicon_tags_30 },
    { 0x33415453, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e15458, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d61545a, SUC_lexicon_tags_1 },
    { 0x9121545a, SUC_lexicon_tags_214 },
    { 0x9c81545b, SUC_lexicon_tags_4 },
    { 0xd021545a, SUC_lexicon_tags_28 },
    { 0xbde1545e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97615460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c15464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3615467, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2361546b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb601546d, SUC_lexicon_tags_82 },
    { 0x7c21546e, SUC_lexicon_tags_49 },
    { 0xd941546f, SUC_lexicon_tags_11 },
    { 0x701546e, SUC_lexicon_tags_10 },
    { 0x5e41546f, SUC_lexicon_tags_39 },
    { 0xe2615470, SUC_lexicon_tags_13 },
    { 0xb015473, SUC_lexicon_tags_15 },
    { 0x19615471, SUC_lexicon_tags_12 },
    { 0xbac15475, SUC_lexicon_tags_2 },
    { 0x7e41546d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c15479, SUC_lexicon_tags_5 },
    { 0x96e1547a, SUC_lexicon_tags_31 },
    { 0x4521547b, SUC_lexicon_tags_3 },
    { 0x5c01547b, SUC_lexicon_tags_25 },
    { 0x38415479, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d015481, SUC_lexicon_tags_60 },
    { 0xa6015482, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ac15484, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e15487, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a1548e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48415496, SUC_lexicon_tags_0 },
    { 0xfa15496, SUC_lexicon_tags_15 },
    { 0xd1615498, SUC_lexicon_tags_0 },
    { 0x37615497, SUC_lexicon_tags_1490 },
    { 0x93a1549a, SUC_lexicon_tags_55 },
    { 0xd481549a, SUC_lexicon_tags_27 },
    { 0x8d01549c, SUC_lexicon_tags_14 },
    { 0x8aa1549a, SUC_lexicon_tags_17 },
    { 0xcc61549e, SUC_lexicon_tags_46 },
    { 0x71415497, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ea154a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9c0154a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e154a7, SUC_lexicon_tags_63 },
    { 0xd32154a8, SUC_lexicon_tags_14 },
    { 0x680154a7, SUC_lexicon_tags_7 },
    { 0xd5e154a7, SUC_lexicon_tags_29 },
    { 0xcf6154a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150154b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8aa154b4, SUC_lexicon_tags_34 },
    { 0xfd2154b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38a154b7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de154ba, SUC_lexicon_tags_11 },
    { 0x634154ba, SUC_lexicon_tags_12 },
    { 0xffa154ba, SUC_lexicon_tags_47 },
    { 0x676154bd, SUC_lexicon_tags_14 },
    { 0x648154bd, SUC_lexicon_tags_49 },
    { 0x6ac154bb, SUC_lexicon_tags_37 },
    { 0x2f0154bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4ec154c2, SUC_lexicon_tags_66 },
    { 0x93a154c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8154c9, SUC_lexicon_tags_25 },
    { 0xa22154ca, SUC_lexicon_tags_17 },
    { 0xc9e154cb, SUC_lexicon_tags_14 },
    { 0x772154cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x602154cf, SUC_lexicon_tags_7 },
    { 0x2a0154d0, SUC_lexicon_tags_31 },
    { 0x654154d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a154d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee154da, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a154df, SUC_lexicon_tags_7 },
    { 0xa84154e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0154e4, SUC_lexicon_tags_25 },
    { 0xe1a154e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc98154e7, SUC_lexicon_tags_6 },
    { 0x4dc154e7, SUC_lexicon_tags_11 },
    { 0x654154e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a154ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78154f3, SUC_lexicon_tags_1 },
    { 0xe9a154f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220154f9, SUC_lexicon_tags_25 },
    { 0x1a0154f9, SUC_lexicon_tags_49 },
    { 0x29a154f9, SUC_lexicon_tags_138 },
    { 0x4ce154fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b015501, SUC_lexicon_tags_45 },
    { 0xe815502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d415507, SUC_lexicon_tags_15 },
    { 0xcb415508, SUC_lexicon_tags_38 },
    { 0x90615508, SUC_lexicon_tags_63 },
    { 0xb081550a, SUC_lexicon_tags_25 },
    { 0x9ba1550a, SUC_lexicon_tags_47 },
    { 0xb4415508, SUC_lexicon_tags_47 },
    { 0x49a1550d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11215511, SUC_lexicon_tags_15 },
    { 0x99615512, SUC_lexicon_tags_17 },
    { 0x34015511, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5801551d, SUC_lexicon_tags_49 },
    { 0x6041551e, SUC_lexicon_tags_45 },
    { 0xd981551e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e15522, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xad215524, SUC_lexicon_tags_5 },
    { 0x72215524, SUC_lexicon_tags_17 },
    { 0x3b815525, SUC_lexicon_tags_11 },
    { 0xf8c15527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9181552a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c15533, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1815538, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf21553c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a15540, SUC_lexicon_tags_1 },
    { 0xd2815540, SUC_lexicon_tags_1 },
    { 0x7c015541, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e15546, SUC_lexicon_tags_29 },
    { 0x41e15547, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c1554c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb415554, SUC_lexicon_tags_9 },
    { 0x3b415554, SUC_lexicon_tags_37 },
    { 0xe0815554, SUC_lexicon_tags_11 },
    { 0xd3415557, SUC_lexicon_tags_17 },
    { 0x6015558, SUC_lexicon_tags_26 },
    { 0xd9415559, SUC_lexicon_tags_29 },
    { 0x22e15557, SUC_lexicon_tags_1 },
    { 0xcce15559, SUC_lexicon_tags_2 },
    { 0x47215556, SUC_lexicon_tags_27 },
    { 0x2d21555d, SUC_lexicon_tags_28 },
    { 0xeb41555d, SUC_lexicon_tags_37 },
    { 0x49e1555d, SUC_lexicon_tags_30 },
    { 0xa8c1555f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4615565, SUC_lexicon_tags_37 },
    { 0x51e15565, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d81556e, SUC_lexicon_tags_1 },
    { 0x3f61556f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5815571, SUC_lexicon_tags_17 },
    { 0x6f615571, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c015576, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb661557a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e1557c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda21557f, SUC_lexicon_tags_136 },
    { 0x415580, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x15c15582, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e15587, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb61558a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67c1558c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0815592, SUC_lexicon_tags_17 },
    { 0xa9415592, SUC_lexicon_tags_11 },
    { 0xfb215594, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x71815597, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6441559b, SUC_lexicon_tags_47 },
    { 0x60a1559c, SUC_lexicon_tags_12 },
    { 0x50c1559d, SUC_lexicon_tags_12 },
    { 0x11a1559e, SUC_lexicon_tags_80 },
    { 0x6641559d, SUC_lexicon_tags_65 },
    { 0x34c1559b, SUC_lexicon_tags_279 },
    { 0x9d6155a1, SUC_lexicon_tags_14 },
    { 0xfe61559c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf82155a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e155a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbc155aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc155ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb2155b0, SUC_lexicon_tags_7 },
    { 0xff6155b1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee155c0, SUC_lexicon_tags_25 },
    { 0xb40155c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb06155c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76155c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6155ca, SUC_lexicon_tags_25 },
    { 0x55a155cb, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82155d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4155d3, SUC_lexicon_tags_64 },
    { 0xf78155d3, SUC_lexicon_tags_17 },
    { 0x342155d5, SUC_lexicon_tags_46 },
    { 0x6b0155d6, SUC_lexicon_tags_6 },
    { 0x3e6155d4, SUC_lexicon_tags_14 },
    { 0x51a155d8, SUC_lexicon_tags_55 },
    { 0x604155d9, SUC_lexicon_tags_11 },
    { 0xfe4155da, SUC_lexicon_tags_11 },
    { 0x8b6155d5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8155df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806155e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x29a155e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e155e9, SUC_lexicon_tags_33 },
    { 0x1f2155ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ea155ec, SUC_lexicon_tags_35 },
    { 0xa88155ed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c155f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x944155f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6155fa, SUC_lexicon_tags_15 },
    { 0x6f4155fb, SUC_lexicon_tags_235 },
    { 0x996155fc, SUC_lexicon_tags_37 },
    { 0x774155fd, SUC_lexicon_tags_43 },
    { 0x75e155fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca15601, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3615605, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9da15607, SUC_lexicon_tags_2 },
    { 0xbc15607, SUC_lexicon_tags_6 },
    { 0x72815609, SUC_lexicon_tags_27 },
    { 0xaee1560a, SUC_lexicon_tags_48 },
    { 0x7221560b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc1560e, SUC_lexicon_tags_1458 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e015613, SUC_lexicon_tags_31 },
    { 0xe1215614, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c15617, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb815619, SUC_lexicon_tags_7 },
    { 0x68615619, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c1561e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee015622, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x34015625, SUC_lexicon_tags_1 },
    { 0xede15625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8521562a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e15632, SUC_lexicon_tags_22 },
    { 0xfe815632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e15638, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2861563d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd815641, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6615649, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a1564d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a215653, SUC_lexicon_tags_12 },
    { 0x14615654, SUC_lexicon_tags_43 },
    { 0xfd015654, SUC_lexicon_tags_230 },
    { 0x2e15654, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47615660, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec15663, SUC_lexicon_tags_25 },
    { 0x2ce15663, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf6615666, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfca15668, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1201566a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf641566d, SUC_lexicon_tags_7 },
    { 0x1241566e, SUC_lexicon_tags_14 },
    { 0xa801566f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf215671, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x40615673, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1215676, SUC_lexicon_tags_17 },
    { 0x65415677, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1567c, SUC_lexicon_tags_30 },
    { 0xf741567d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2941567f, SUC_lexicon_tags_11 },
    { 0x86e15680, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c15684, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd5a15686, SUC_lexicon_tags_81 },
    { 0x9c215686, SUC_lexicon_tags_43 },
    { 0xab215688, SUC_lexicon_tags_29 },
    { 0x2e015686, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de1568f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb015691, SUC_lexicon_tags_17 },
    { 0x3ce15692, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x68015695, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93015697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a1569a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a156a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x138156a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x968156a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2156a9, SUC_lexicon_tags_1 },
    { 0x91c156aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2156ad, SUC_lexicon_tags_55 },
    { 0x628156ad, SUC_lexicon_tags_26 },
    { 0x5fe156af, SUC_lexicon_tags_82 },
    { 0x832156af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x46e156b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458156be, SUC_lexicon_tags_37 },
    { 0x1b6156be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24156c2, SUC_lexicon_tags_23 },
    { 0x874156c3, SUC_lexicon_tags_47 },
    { 0xa6156c2, SUC_lexicon_tags_37 },
    { 0x996156c4, SUC_lexicon_tags_38 },
    { 0x9a8156c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c156ca, SUC_lexicon_tags_35 },
    { 0xd8a156cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7fe156cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6156d4, SUC_lexicon_tags_17 },
    { 0x9c156d5, SUC_lexicon_tags_9 },
    { 0x1e0156d6, SUC_lexicon_tags_11 },
    { 0xcbc156d4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200156df, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0156e4, SUC_lexicon_tags_2 },
    { 0x4d8156e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e156e9, SUC_lexicon_tags_17 },
    { 0xed2156ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x830156ec, SUC_lexicon_tags_7 },
    { 0x25c156ed, SUC_lexicon_tags_12 },
    { 0x14e156ec, SUC_lexicon_tags_6 },
    { 0xd4156ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x232156f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8156f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68156fd, SUC_lexicon_tags_25 },
    { 0xc84156fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c15701, SUC_lexicon_tags_25 },
    { 0x1c415702, SUC_lexicon_tags_37 },
    { 0xd6a15701, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe4a15705, SUC_lexicon_tags_1 },
    { 0x4cc15706, SUC_lexicon_tags_17 },
    { 0xaf615707, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x101570a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc1570e, SUC_lexicon_tags_49 },
    { 0xaf81570f, SUC_lexicon_tags_23 },
    { 0x2e15710, SUC_lexicon_tags_17 },
    { 0x1d615710, SUC_lexicon_tags_47 },
    { 0xf9a15712, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf0415714, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e015717, SUC_lexicon_tags_17 },
    { 0x10815718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54a1571a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf41571c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9015723, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e15728, SUC_lexicon_tags_42 },
    { 0x16615729, SUC_lexicon_tags_25 },
    { 0x401572a, SUC_lexicon_tags_17 },
    { 0xda21572a, SUC_lexicon_tags_77 },
    { 0xc861572c, SUC_lexicon_tags_1 },
    { 0x33a1572b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c15731, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2415736, SUC_lexicon_tags_45 },
    { 0xf4215737, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc961573d, SUC_lexicon_tags_12 },
    { 0x7dc1573e, SUC_lexicon_tags_14 },
    { 0x8f81573f, SUC_lexicon_tags_11 },
    { 0xeb01573d, SUC_lexicon_tags_38 },
    { 0x30a1573f, SUC_lexicon_tags_37 },
    { 0x4d815742, SUC_lexicon_tags_8 },
    { 0x3c015741, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c015747, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a1574d, SUC_lexicon_tags_1 },
    { 0x1781574e, SUC_lexicon_tags_2 },
    { 0xe601574e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10615751, SUC_lexicon_tags_26 },
    { 0x90c15752, SUC_lexicon_tags_4 },
    { 0x1a15753, SUC_lexicon_tags_47 },
    { 0x47a15753, SUC_lexicon_tags_1 },
    { 0xecc15755, SUC_lexicon_tags_49 },
    { 0x46415756, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5415758, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c1575f, SUC_lexicon_tags_9 },
    { 0x33415760, SUC_lexicon_tags_34 },
    { 0x21215761, SUC_lexicon_tags_1 },
    { 0x5a415762, SUC_lexicon_tags_2 },
    { 0x39415763, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0x3e415765, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xece15767, SUC_lexicon_tags_14 },
    { 0x89015767, SUC_lexicon_tags_87 },
    { 0x9ca15768, SUC_lexicon_tags_17 },
    { 0x4c01576a, SUC_lexicon_tags_63 },
    { 0xb2e1576b, SUC_lexicon_tags_4 },
    { 0xdce1576c, SUC_lexicon_tags_15 },
    { 0xb981576b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x35415770, SUC_lexicon_tags_2 },
    { 0xbfa15770, SUC_lexicon_tags_23 },
    { 0x83815772, SUC_lexicon_tags_31 },
    { 0xe0015773, SUC_lexicon_tags_15 },
    { 0xe5215772, SUC_lexicon_tags_27 },
    { 0xb3415775, SUC_lexicon_tags_39 },
    { 0x14e15776, SUC_lexicon_tags_17 },
    { 0xf6c15775, SUC_lexicon_tags_32 },
    { 0x8bc15778, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x88c1577a, SUC_lexicon_tags_28 },
    { 0x3241577a, SUC_lexicon_tags_11 },
    { 0xc7a1577b, SUC_lexicon_tags_47 },
    { 0xbfc1577d, SUC_lexicon_tags_12 },
    { 0x9f61577d, SUC_lexicon_tags_27 },
    { 0x6601577f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80415787, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a15790, SUC_lexicon_tags_61 },
    { 0x68215791, SUC_lexicon_tags_4 },
    { 0xe5a15792, SUC_lexicon_tags_11 },
    { 0xc3215791, SUC_lexicon_tags_32 },
    { 0x9b615794, SUC_lexicon_tags_4 },
    { 0xbea15791, SUC_lexicon_tags_33 },
    { 0x81815796, SUC_lexicon_tags_126 },
    { 0x1ce15797, SUC_lexicon_tags_37 },
    { 0x65a15798, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd461579a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf601579d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8157a1, SUC_lexicon_tags_46 },
    { 0x642157a1, SUC_lexicon_tags_29 },
    { 0xc44157a2, SUC_lexicon_tags_33 },
    { 0xbe4157a1, SUC_lexicon_tags_33 },
    { 0x9ee157a5, SUC_lexicon_tags_19 },
    { 0x8b2157a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24157ad, SUC_lexicon_tags_6 },
    { 0xd88157ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14157b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c4157b4, SUC_lexicon_tags_15 },
    { 0xc62157b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x168157b7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x25e157b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0c157bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974157bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5c157c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc157c4, SUC_lexicon_tags_12 },
    { 0xd70157c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x50c157c7, SUC_lexicon_tags_17 },
    { 0xe6a157c7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x640157cb, SUC_lexicon_tags_25 },
    { 0xae2157cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a0157ce, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa0a157d0, SUC_lexicon_tags_7 },
    { 0x3cc157d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30e157d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x926157d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08157db, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60157de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc4157e0, SUC_lexicon_tags_11 },
    { 0x87c157e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458157e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca157ed, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12157f3, SUC_lexicon_tags_320 },
    { 0x99c157f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6157f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x35a157fa, SUC_lexicon_tags_103 },
    { 0xb3e157fb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74157fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1615801, SUC_lexicon_tags_18 },
    { 0xd4e15802, SUC_lexicon_tags_17 },
    { 0x40615802, SUC_lexicon_tags_17 },
    { 0x72215804, SUC_lexicon_tags_7 },
    { 0x7ce15803, SUC_lexicon_tags_31 },
    { 0x3a015806, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x88615809, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1da1580b, SUC_lexicon_tags_25 },
    { 0x79c1580b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a15811, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4f015813, SUC_lexicon_tags_17 },
    { 0xb7815814, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4215816, SUC_lexicon_tags_48 },
    { 0xbe615817, SUC_lexicon_tags_23 },
    { 0x97815818, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa001581b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec1581f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd415825, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc1582a, SUC_lexicon_tags_11 },
    { 0x9861582a, SUC_lexicon_tags_47 },
    { 0x6761582c, SUC_lexicon_tags_6 },
    { 0x82c1582d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec1582f, SUC_lexicon_tags_1 },
    { 0x40615830, SUC_lexicon_tags_33 },
    { 0xa2815831, SUC_lexicon_tags_11 },
    { 0xb4415832, SUC_lexicon_tags_37 },
    { 0xdae15833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d415835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb201583a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6a1583c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e81583f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83815844, SUC_lexicon_tags_7 },
    { 0xa1e15845, SUC_lexicon_tags_68 },
    { 0x59e15846, SUC_lexicon_tags_11 },
    { 0x29415847, SUC_lexicon_tags_6 },
    { 0x4c415848, SUC_lexicon_tags_2 },
    { 0xaa015849, SUC_lexicon_tags_45 },
    { 0x5ca1584a, SUC_lexicon_tags_0 },
    { 0x81584b, SUC_lexicon_tags_12 },
    { 0x4c41584c, SUC_lexicon_tags_161 },
    { 0x48c1584d, SUC_lexicon_tags_2 },
    { 0x93e1584c, SUC_lexicon_tags_55 },
    { 0x1221584b, SUC_lexicon_tags_12 },
    { 0xda61584c, SUC_lexicon_tags_33 },
    { 0x31e15848, SUC_lexicon_tags_60 },
    { 0xe401584b, SUC_lexicon_tags_32 },
    { 0x4ba15850, SUC_lexicon_tags_33 },
    { 0xf5215854, SUC_lexicon_tags_55 },
    { 0x66a15855, SUC_lexicon_tags_11 },
    { 0xe5815856, SUC_lexicon_tags_15 },
    { 0xfb615848, SUC_lexicon_tags_7 },
    { 0x3e15846, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6121585a, SUC_lexicon_tags_2 },
    { 0x25a1585b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf001585e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8615861, SUC_lexicon_tags_7 },
    { 0x94e15862, SUC_lexicon_tags_47 },
    { 0x9f415863, SUC_lexicon_tags_3 },
    { 0x57c15864, SUC_lexicon_tags_11 },
    { 0xe1615865, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da1586a, SUC_lexicon_tags_48 },
    { 0xf061586a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6241586f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3815872, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x7a415874, SUC_lexicon_tags_17 },
    { 0xaa615874, SUC_lexicon_tags_37 },
    { 0xc7c15876, SUC_lexicon_tags_23 },
    { 0x37a15877, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x10c15879, SUC_lexicon_tags_136 },
    { 0x88a15879, SUC_lexicon_tags_31 },
    { 0xa01587b, SUC_lexicon_tags_15 },
    { 0xf5a15879, SUC_lexicon_tags_47 },
    { 0xada1587d, SUC_lexicon_tags_46 },
    { 0xfd61587d, SUC_lexicon_tags_7 },
    { 0x96a15879, SUC_lexicon_tags_25 },
    { 0xec81587f, SUC_lexicon_tags_37 },
    { 0x3ba15881, SUC_lexicon_tags_17 },
    { 0x9ae15881, SUC_lexicon_tags_11 },
    { 0x71a15883, SUC_lexicon_tags_47 },
    { 0xa3215884, SUC_lexicon_tags_82 },
    { 0xb0815885, SUC_lexicon_tags_8 },
    { 0x61a15886, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3da15888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a15892, SUC_lexicon_tags_116 },
    { 0x8f015892, SUC_lexicon_tags_10 },
    { 0x33a15894, SUC_lexicon_tags_3 },
    { 0xa0015895, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d015897, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a1589b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400158a4, SUC_lexicon_tags_33 },
    { 0x486158a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc158a9, SUC_lexicon_tags_17 },
    { 0xcd8158aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c158af, SUC_lexicon_tags_29 },
    { 0xe3a158b0, SUC_lexicon_tags_31 },
    { 0x114158b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f6158b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2158bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4158c5, SUC_lexicon_tags_37 },
    { 0xf7a158c5, SUC_lexicon_tags_1 },
    { 0x6a8158c7, SUC_lexicon_tags_104 },
    { 0x2d2158c8, SUC_lexicon_tags_38 },
    { 0xa8c158c9, SUC_lexicon_tags_1227 },
    { 0, NULL },
    { 0x8c4158cb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa158ce, SUC_lexicon_tags_12 },
    { 0x956158ce, SUC_lexicon_tags_25 },
    { 0x8d6158d0, SUC_lexicon_tags_25 },
    { 0x7ec158d1, SUC_lexicon_tags_26 },
    { 0xabc158d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fe158d4, SUC_lexicon_tags_1 },
    { 0x6a2158d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa158d9, SUC_lexicon_tags_63 },
    { 0x3c158da, SUC_lexicon_tags_17 },
    { 0xc7e158db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900158e3, SUC_lexicon_tags_37 },
    { 0x624158e4, SUC_lexicon_tags_19 },
    { 0x282158e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x148158e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaac158e9, SUC_lexicon_tags_47 },
    { 0xfac158ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea158ee, SUC_lexicon_tags_47 },
    { 0x456158ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f0158f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44158fb, SUC_lexicon_tags_38 },
    { 0x616158fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3815907, SUC_lexicon_tags_2 },
    { 0x6b015907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce1590c, SUC_lexicon_tags_28 },
    { 0xf8e1590d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc15912, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c15917, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xee01591a, SUC_lexicon_tags_37 },
    { 0x58c1591b, SUC_lexicon_tags_9 },
    { 0x2e41591c, SUC_lexicon_tags_10 },
    { 0x9561591b, SUC_lexicon_tags_156 },
    { 0x5321591d, SUC_lexicon_tags_1 },
    { 0xc161591f, SUC_lexicon_tags_45 },
    { 0xfba1591e, SUC_lexicon_tags_35 },
    { 0xef815920, SUC_lexicon_tags_27 },
    { 0xf0015922, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d215925, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x90815927, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e81592d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5de1592f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a15933, SUC_lexicon_tags_17 },
    { 0xdd815933, SUC_lexicon_tags_1 },
    { 0x28415933, SUC_lexicon_tags_38 },
    { 0x47215935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e15939, SUC_lexicon_tags_39 },
    { 0x7f81593a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac15940, SUC_lexicon_tags_37 },
    { 0xdb015941, SUC_lexicon_tags_8 },
    { 0x56415942, SUC_lexicon_tags_23 },
    { 0x64615943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4815947, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8261594b, SUC_lexicon_tags_25 },
    { 0x1561594c, SUC_lexicon_tags_38 },
    { 0x8f61594d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x84815950, SUC_lexicon_tags_6 },
    { 0x75a15951, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb815954, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8741595b, SUC_lexicon_tags_70 },
    { 0xc0a1595b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa1595f, SUC_lexicon_tags_17 },
    { 0x7a15960, SUC_lexicon_tags_11 },
    { 0xebc15961, SUC_lexicon_tags_48 },
    { 0x42415962, SUC_lexicon_tags_8 },
    { 0x94e15962, SUC_lexicon_tags_29 },
    { 0x8aa15964, SUC_lexicon_tags_11 },
    { 0xd1415963, SUC_lexicon_tags_15 },
    { 0x42815963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa815969, SUC_lexicon_tags_4 },
    { 0xeae1596a, SUC_lexicon_tags_7 },
    { 0x7d61596b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4181596e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42215971, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77015973, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9641597a, SUC_lexicon_tags_37 },
    { 0x74a1597b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2615980, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee15983, SUC_lexicon_tags_9 },
    { 0x7b215984, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91e15986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a15989, SUC_lexicon_tags_17 },
    { 0xb761598a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa41598c, SUC_lexicon_tags_57 },
    { 0xdc41598d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6821598f, SUC_lexicon_tags_7 },
    { 0xfe015990, SUC_lexicon_tags_47 },
    { 0x66215991, SUC_lexicon_tags_87 },
    { 0x3ae15992, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3215995, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7221599d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80159a0, SUC_lexicon_tags_25 },
    { 0xad8159a1, SUC_lexicon_tags_7 },
    { 0x810159a1, SUC_lexicon_tags_9 },
    { 0x144159a2, SUC_lexicon_tags_25 },
    { 0x48e159a1, SUC_lexicon_tags_37 },
    { 0x382159a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84159a8, SUC_lexicon_tags_25 },
    { 0x330159a8, SUC_lexicon_tags_26 },
    { 0x422159aa, SUC_lexicon_tags_83 },
    { 0xa5e159ab, SUC_lexicon_tags_114 },
    { 0xcc8159ac, SUC_lexicon_tags_1 },
    { 0x126159ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2159b0, SUC_lexicon_tags_49 },
    { 0x54a159b1, SUC_lexicon_tags_12 },
    { 0x31a159b2, SUC_lexicon_tags_70 },
    { 0xe64159b3, SUC_lexicon_tags_43 },
    { 0x116159b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8159ba, SUC_lexicon_tags_18 },
    { 0xb78159ba, SUC_lexicon_tags_37 },
    { 0xa5c159ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43e159be, SUC_lexicon_tags_15 },
    { 0x480159bf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc159c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e159c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10159cc, SUC_lexicon_tags_65 },
    { 0xdea159cd, SUC_lexicon_tags_11 },
    { 0x720159ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4159d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdce159d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec159da, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf24159dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16159e0, SUC_lexicon_tags_2 },
    { 0xdd2159e1, SUC_lexicon_tags_17 },
    { 0xa06159e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca159e5, SUC_lexicon_tags_48 },
    { 0x89c159e6, SUC_lexicon_tags_63 },
    { 0xd86159e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e159eb, SUC_lexicon_tags_1 },
    { 0x5cc159ec, SUC_lexicon_tags_63 },
    { 0x5f6159ed, SUC_lexicon_tags_75 },
    { 0x3d4159ec, SUC_lexicon_tags_7 },
    { 0xa36159ef, SUC_lexicon_tags_33 },
    { 0x4e2159ed, SUC_lexicon_tags_49 },
    { 0x836159f0, SUC_lexicon_tags_11 },
    { 0x810159f2, SUC_lexicon_tags_46 },
    { 0x784159ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc159f7, SUC_lexicon_tags_2 },
    { 0xc16159f8, SUC_lexicon_tags_10 },
    { 0x3bc159f9, SUC_lexicon_tags_1 },
    { 0xfa4159fa, SUC_lexicon_tags_31 },
    { 0x8e0159fb, SUC_lexicon_tags_97 },
    { 0x9f2159fc, SUC_lexicon_tags_23 },
    { 0x360159fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96015a04, SUC_lexicon_tags_310 },
    { 0x39615a05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb015a08, SUC_lexicon_tags_37 },
    { 0xbb615a09, SUC_lexicon_tags_1 },
    { 0x9d415a08, SUC_lexicon_tags_37 },
    { 0xf0e15a09, SUC_lexicon_tags_3 },
    { 0xa1615a0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc15a10, SUC_lexicon_tags_11 },
    { 0x39a15a11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53815a14, SUC_lexicon_tags_31 },
    { 0xfd015a15, SUC_lexicon_tags_11 },
    { 0x8ba15a15, SUC_lexicon_tags_31 },
    { 0x44615a17, SUC_lexicon_tags_37 },
    { 0xd8c15a18, SUC_lexicon_tags_37 },
    { 0xf6815a16, SUC_lexicon_tags_11 },
    { 0x27e15a16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c215a1c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e15a22, SUC_lexicon_tags_4 },
    { 0xb0415a22, SUC_lexicon_tags_9 },
    { 0x25815a23, SUC_lexicon_tags_3 },
    { 0x49c15a25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1015a29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89615a2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8e15a2d, SUC_lexicon_tags_6 },
    { 0x53615a2e, SUC_lexicon_tags_26 },
    { 0xb1215a2e, SUC_lexicon_tags_47 },
    { 0x88015a30, SUC_lexicon_tags_7 },
    { 0x66e15a31, SUC_lexicon_tags_2 },
    { 0xbc815a32, SUC_lexicon_tags_79 },
    { 0x13815a2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e15a38, SUC_lexicon_tags_19 },
    { 0xa4615a38, SUC_lexicon_tags_11 },
    { 0x4d815a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cc15a3c, SUC_lexicon_tags_32 },
    { 0xad015a3c, SUC_lexicon_tags_8 },
    { 0x80215a3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8215a43, SUC_lexicon_tags_25 },
    { 0x4be15a44, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xca815a46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d415a48, SUC_lexicon_tags_2 },
    { 0x6e415a48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10815a4e, SUC_lexicon_tags_10 },
    { 0xf0c15a4e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd015a54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16415a56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x75215a59, SUC_lexicon_tags_30 },
    { 0x2c215a59, SUC_lexicon_tags_11 },
    { 0x66215a5b, SUC_lexicon_tags_37 },
    { 0x47015a59, SUC_lexicon_tags_1 },
    { 0x3fe15a5b, SUC_lexicon_tags_25 },
    { 0x3b415a5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4415a60, SUC_lexicon_tags_25 },
    { 0x80a15a61, SUC_lexicon_tags_37 },
    { 0xc8c15a61, SUC_lexicon_tags_17 },
    { 0x97215a62, SUC_lexicon_tags_2 },
    { 0xf7215a61, SUC_lexicon_tags_15 },
    { 0xb8615a65, SUC_lexicon_tags_63 },
    { 0xdc415a65, SUC_lexicon_tags_31 },
    { 0x1ec15a62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x48015a69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96015a6c, SUC_lexicon_tags_6 },
    { 0xa8615a6c, SUC_lexicon_tags_17 },
    { 0x63215a6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe15a72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e15a76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a415a7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef815a7d, SUC_lexicon_tags_30 },
    { 0x5dc15a7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72015a81, SUC_lexicon_tags_27 },
    { 0x70015a81, SUC_lexicon_tags_25 },
    { 0x7e215a83, SUC_lexicon_tags_67 },
    { 0x9a015a83, SUC_lexicon_tags_23 },
    { 0xc1615a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f415a88, SUC_lexicon_tags_77 },
    { 0xb1815a88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5615a8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a15a90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a15a93, SUC_lexicon_tags_25 },
    { 0x67415a94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1615a96, SUC_lexicon_tags_7 },
    { 0x42c15a97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x21c15a99, SUC_lexicon_tags_57 },
    { 0x75815a9a, SUC_lexicon_tags_11 },
    { 0x78615a9a, SUC_lexicon_tags_1 },
    { 0xf6615a9c, SUC_lexicon_tags_47 },
    { 0xd9015a9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e15aa0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb615aa7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x26e15aa9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae15aaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba15ab3, SUC_lexicon_tags_35 },
    { 0xe4815ab4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2a15ab6, SUC_lexicon_tags_37 },
    { 0x1fc15ab6, SUC_lexicon_tags_25 },
    { 0x76c15ab8, SUC_lexicon_tags_65 },
    { 0x34c15ab9, SUC_lexicon_tags_37 },
    { 0x50415ab7, SUC_lexicon_tags_33 },
    { 0x83615abb, SUC_lexicon_tags_4 },
    { 0x5e015abb, SUC_lexicon_tags_8 },
    { 0x2aa15abd, SUC_lexicon_tags_45 },
    { 0x56815abb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5215ac4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a15acb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e415ad0, SUC_lexicon_tags_25 },
    { 0xaa615ad1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f415ad6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a15ad9, SUC_lexicon_tags_7 },
    { 0xbea15ada, SUC_lexicon_tags_1 },
    { 0x7e015adb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27815adf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa15ae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d815ae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50615ae7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e15aeb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb7c15aed, SUC_lexicon_tags_6 },
    { 0x8f215aee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a15af4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38615af6, SUC_lexicon_tags_15 },
    { 0x68c15af7, SUC_lexicon_tags_47 },
    { 0xce15af7, SUC_lexicon_tags_16 },
    { 0x2ec15af8, SUC_lexicon_tags_33 },
    { 0x3d215afa, SUC_lexicon_tags_37 },
    { 0xe9c15afa, SUC_lexicon_tags_7 },
    { 0x7d815afc, SUC_lexicon_tags_17 },
    { 0x98a15af6, SUC_lexicon_tags_92 },
    { 0x9815afe, SUC_lexicon_tags_12 },
    { 0x68015afd, SUC_lexicon_tags_37 },
    { 0x2ce15b00, SUC_lexicon_tags_17 },
    { 0xae615afd, SUC_lexicon_tags_11 },
    { 0x81615b01, SUC_lexicon_tags_9 },
    { 0xe9c15b03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c415b05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e15b08, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63415b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ee15b12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc15b15, SUC_lexicon_tags_23 },
    { 0xe1215b16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56c15b18, SUC_lexicon_tags_37 },
    { 0x46615b19, SUC_lexicon_tags_17 },
    { 0xfcc15b1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4615b1e, SUC_lexicon_tags_39 },
    { 0xca815b1f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfa015b21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc5215b23, SUC_lexicon_tags_45 },
    { 0xca815b23, SUC_lexicon_tags_7 },
    { 0x1d415b25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed615b27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d615b2b, SUC_lexicon_tags_7 },
    { 0x32a15b2c, SUC_lexicon_tags_9 },
    { 0xf2e15b2c, SUC_lexicon_tags_7 },
    { 0x9d215b2e, SUC_lexicon_tags_37 },
    { 0x5a615b2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33015b35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x81615b37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da15b3b, SUC_lexicon_tags_11 },
    { 0x3a615b3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae615b40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e15b43, SUC_lexicon_tags_48 },
    { 0xf8c15b43, SUC_lexicon_tags_25 },
    { 0x5de15b44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a15b4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e15b50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee15b58, SUC_lexicon_tags_15 },
    { 0xfdc15b59, SUC_lexicon_tags_15 },
    { 0x46e15b59, SUC_lexicon_tags_11 },
    { 0x59215b5b, SUC_lexicon_tags_10 },
    { 0xbc415b5c, SUC_lexicon_tags_33 },
    { 0xeca15b5d, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63615b61, SUC_lexicon_tags_8 },
    { 0x3ae15b62, SUC_lexicon_tags_68 },
    { 0xf815b63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x85415b65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b015b69, SUC_lexicon_tags_37 },
    { 0x7bc15b6a, SUC_lexicon_tags_26 },
    { 0xe8615b6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c15b75, SUC_lexicon_tags_37 },
    { 0x1da15b75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x88415b78, SUC_lexicon_tags_37 },
    { 0x6fa15b79, SUC_lexicon_tags_37 },
    { 0x30215b79, SUC_lexicon_tags_74 },
    { 0xfc815b7b, SUC_lexicon_tags_7 },
    { 0x28215b7b, SUC_lexicon_tags_2 },
    { 0x3fc15b7d, SUC_lexicon_tags_6 },
    { 0x47015b7e, SUC_lexicon_tags_2 },
    { 0x48215b79, SUC_lexicon_tags_1 },
    { 0xbec15b80, SUC_lexicon_tags_7 },
    { 0x16415b79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87615b85, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa15b89, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x72e15b8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f415b8f, SUC_lexicon_tags_14 },
    { 0x76015b90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbb015b92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62c15b94, SUC_lexicon_tags_21 },
    { 0xdc015b95, SUC_lexicon_tags_17 },
    { 0x94615b94, SUC_lexicon_tags_17 },
    { 0x8a15b97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94615b9d, SUC_lexicon_tags_2 },
    { 0xfec15b9e, SUC_lexicon_tags_35 },
    { 0xc2c15b9d, SUC_lexicon_tags_82 },
    { 0x77e15ba0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7815ba4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfac15ba6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6815ba8, SUC_lexicon_tags_13 },
    { 0x41a15ba9, SUC_lexicon_tags_2 },
    { 0x43615baa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84415bac, SUC_lexicon_tags_23 },
    { 0xf2015bac, SUC_lexicon_tags_60 },
    { 0xb8e15bac, SUC_lexicon_tags_0 },
    { 0xfc815baf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74415bb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc15bb6, SUC_lexicon_tags_17 },
    { 0x25815bb7, SUC_lexicon_tags_10 },
    { 0xa615bb8, SUC_lexicon_tags_26 },
    { 0xd0c15bb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4a15bbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35415bc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa15bc8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8215bca, SUC_lexicon_tags_11 },
    { 0x82015bcb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a15bd0, SUC_lexicon_tags_37 },
    { 0xd2c15bd1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f615bd8, SUC_lexicon_tags_37 },
    { 0x9ea15bd9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18615bdc, SUC_lexicon_tags_37 },
    { 0xda215bdc, SUC_lexicon_tags_17 },
    { 0x87e15bdd, SUC_lexicon_tags_15 },
    { 0xc3e15bdc, SUC_lexicon_tags_19 },
    { 0x6e015bde, SUC_lexicon_tags_28 },
    { 0x2ba15be1, SUC_lexicon_tags_12 },
    { 0x3c615be2, SUC_lexicon_tags_2 },
    { 0x12e15be3, SUC_lexicon_tags_27 },
    { 0xf9a15be2, SUC_lexicon_tags_31 },
    { 0xdb615be2, SUC_lexicon_tags_14 },
    { 0x29e15be6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea15beb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e215bed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1215bef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f815bf3, SUC_lexicon_tags_48 },
    { 0x1d015bf4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99415bf9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e15bfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc15c01, SUC_lexicon_tags_116 },
    { 0x28215c01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdc15c04, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbce15c06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca215c09, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36c15c0b, SUC_lexicon_tags_25 },
    { 0x1f215c0c, SUC_lexicon_tags_17 },
    { 0xa3a15c0c, SUC_lexicon_tags_17 },
    { 0x4be15c0e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa15c12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6615c14, SUC_lexicon_tags_31 },
    { 0x6b815c15, SUC_lexicon_tags_25 },
    { 0xf5c15c16, SUC_lexicon_tags_0 },
    { 0xb815c16, SUC_lexicon_tags_7 },
    { 0xe7e15c18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0215c1d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba415c23, SUC_lexicon_tags_3 },
    { 0xa9815c23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb215c26, SUC_lexicon_tags_17 },
    { 0x6ca15c27, SUC_lexicon_tags_45 },
    { 0x5ce15c27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b015c2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6615c31, SUC_lexicon_tags_686 },
    { 0, NULL },
    { 0xf5615c33, SUC_lexicon_tags_2 },
    { 0x3d415c33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77a15c36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66015c38, SUC_lexicon_tags_37 },
    { 0xc1615c39, SUC_lexicon_tags_55 },
    { 0xc4c15c39, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a015c41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea15c45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a15c48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d015c53, SUC_lexicon_tags_35 },
    { 0xe4215c53, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e15c57, SUC_lexicon_tags_2 },
    { 0x81e15c57, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e15c5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98815c60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab415c64, SUC_lexicon_tags_17 },
    { 0xf5015c65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83015c67, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a15c6a, SUC_lexicon_tags_7 },
    { 0x1ac15c6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe15c75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c15c7a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a815c80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e815c85, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59015c8d, SUC_lexicon_tags_16 },
    { 0x65815c8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda15c94, SUC_lexicon_tags_10 },
    { 0x51415c95, SUC_lexicon_tags_60 },
    { 0x21a15c94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xab015c98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a15c9c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x6c15c9e, SUC_lexicon_tags_1 },
    { 0x3dc15c9e, SUC_lexicon_tags_28 },
    { 0x74c15ca0, SUC_lexicon_tags_45 },
    { 0xeac15ca1, SUC_lexicon_tags_37 },
    { 0x34215ca1, SUC_lexicon_tags_14 },
    { 0xb5a15ca2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa15ca7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2215cac, SUC_lexicon_tags_57 },
    { 0xba815cad, SUC_lexicon_tags_1136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9615cb1, SUC_lexicon_tags_38 },
    { 0x7da15cb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d015cb4, SUC_lexicon_tags_11 },
    { 0x60415cb5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a15cb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92215cbc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ac15cbe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c15cc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37815cc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7215cc7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2215ccb, SUC_lexicon_tags_25 },
    { 0xf9e15ccc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a15ccf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4015cd4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xabe15cd6, SUC_lexicon_tags_25 },
    { 0xdb415cd6, SUC_lexicon_tags_17 },
    { 0xcec15cd8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f215cdb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc15ce0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x87a15ce2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb615cef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a15cf3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb215d00, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c15d04, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa815d08, SUC_lexicon_tags_1 },
    { 0x51815d08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf015d0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e015d0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c15d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5415d16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64015d18, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe3615d1a, SUC_lexicon_tags_1 },
    { 0x46c15d1a, SUC_lexicon_tags_43 },
    { 0x80815d1b, SUC_lexicon_tags_45 },
    { 0xbfe15d1d, SUC_lexicon_tags_17 },
    { 0xa6c15d1e, SUC_lexicon_tags_17 },
    { 0x8a815d1f, SUC_lexicon_tags_15 },
    { 0xae615d1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c15d25, SUC_lexicon_tags_12 },
    { 0x49015d26, SUC_lexicon_tags_32 },
    { 0x54615d27, SUC_lexicon_tags_25 },
    { 0x5b015d25, SUC_lexicon_tags_1 },
    { 0xc9615d29, SUC_lexicon_tags_17 },
    { 0xa5e15d2a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc15d2d, SUC_lexicon_tags_35 },
    { 0xc5c15d2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81215d36, SUC_lexicon_tags_7 },
    { 0x41615d36, SUC_lexicon_tags_32 },
    { 0x95a15d38, SUC_lexicon_tags_25 },
    { 0x92615d39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70215d3c, SUC_lexicon_tags_17 },
    { 0x6c415d3d, SUC_lexicon_tags_0 },
    { 0x9f215d3d, SUC_lexicon_tags_14 },
    { 0x7e215d3f, SUC_lexicon_tags_15 },
    { 0x9f415d3c, SUC_lexicon_tags_25 },
    { 0xf3e15d3f, SUC_lexicon_tags_2 },
    { 0xe6a15d42, SUC_lexicon_tags_37 },
    { 0x8d615d42, SUC_lexicon_tags_26 },
    { 0xce215d43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32615d48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a615d4f, SUC_lexicon_tags_17 },
    { 0xc0e15d50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52615d52, SUC_lexicon_tags_17 },
    { 0x4e015d52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e15d56, SUC_lexicon_tags_1 },
    { 0x1d415d56, SUC_lexicon_tags_28 },
    { 0x9fc15d58, SUC_lexicon_tags_26 },
    { 0x5e615d57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61615d5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15415d5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23815d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a15d64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84015d68, SUC_lexicon_tags_106 },
    { 0xd4a15d69, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb4815d6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33c15d6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5815d74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de15d77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7015d7a, SUC_lexicon_tags_45 },
    { 0x2d615d7b, SUC_lexicon_tags_1 },
    { 0xaf615d7c, SUC_lexicon_tags_28 },
    { 0xe4e15d7b, SUC_lexicon_tags_11 },
    { 0xa15d7e, SUC_lexicon_tags_47 },
    { 0x72815d7f, SUC_lexicon_tags_38 },
    { 0x1ea15d7f, SUC_lexicon_tags_11 },
    { 0x3ca15d80, SUC_lexicon_tags_19 },
    { 0x1e615d7e, SUC_lexicon_tags_25 },
    { 0x27615d83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb815d85, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d415d8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x66615d90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83415d92, SUC_lexicon_tags_33 },
    { 0xd0815d93, SUC_lexicon_tags_12 },
    { 0x8f415d94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f815d96, SUC_lexicon_tags_3 },
    { 0x44015d96, SUC_lexicon_tags_16 },
    { 0xbe815d97, SUC_lexicon_tags_15 },
    { 0x2d615d96, SUC_lexicon_tags_0 },
    { 0x2f415d97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf015d9c, SUC_lexicon_tags_2 },
    { 0x2d815d9c, SUC_lexicon_tags_37 },
    { 0xd5415d9e, SUC_lexicon_tags_53 },
    { 0x10c15d9d, SUC_lexicon_tags_1 },
    { 0x8215da0, SUC_lexicon_tags_25 },
    { 0x7ac15da0, SUC_lexicon_tags_47 },
    { 0xb9015da2, SUC_lexicon_tags_27 },
    { 0x49215da3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf015da6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54615daa, SUC_lexicon_tags_37 },
    { 0xb9a15dab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e15dae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xabe15db0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8bc15db2, SUC_lexicon_tags_17 },
    { 0x8ee15db3, SUC_lexicon_tags_37 },
    { 0x9d015db4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc15dba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16215dbf, SUC_lexicon_tags_4 },
    { 0x2d215dc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54215dc2, SUC_lexicon_tags_11 },
    { 0xf9e15dc3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a15dcc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9015dcf, SUC_lexicon_tags_60 },
    { 0x6c215dd0, SUC_lexicon_tags_37 },
    { 0xcf815dd0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9f015dd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98215dd5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe615ddb, SUC_lexicon_tags_65 },
    { 0x3d215ddc, SUC_lexicon_tags_38 },
    { 0x7d415ddd, SUC_lexicon_tags_29 },
    { 0xfde15dde, SUC_lexicon_tags_17 },
    { 0x2d615dde, SUC_lexicon_tags_2 },
    { 0x50015ddf, SUC_lexicon_tags_15 },
    { 0xe4c15ddd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77815de3, SUC_lexicon_tags_7 },
    { 0xf6615de4, SUC_lexicon_tags_37 },
    { 0xd9815de5, SUC_lexicon_tags_343 },
    { 0xcac15de5, SUC_lexicon_tags_11 },
    { 0xc8815de7, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38815dec, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f015def, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c15df3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1615df6, SUC_lexicon_tags_11 },
    { 0xaf615df7, SUC_lexicon_tags_37 },
    { 0xc4415df6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb415dfa, SUC_lexicon_tags_65 },
    { 0x25015dfb, SUC_lexicon_tags_17 },
    { 0x6ae15dfc, SUC_lexicon_tags_14 },
    { 0xdd215dfc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x72c15dff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b815e04, SUC_lexicon_tags_11 },
    { 0x3e015e05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47815e09, SUC_lexicon_tags_26 },
    { 0xe4415e0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78215e10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3215e17, SUC_lexicon_tags_12 },
    { 0x73615e18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a815e1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee015e22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6415e24, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d615e28, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d415e2b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f415e2e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a15e33, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda15e37, SUC_lexicon_tags_8 },
    { 0xfdc15e37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41615e3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e15e42, SUC_lexicon_tags_60 },
    { 0xf2815e43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e015e4a, SUC_lexicon_tags_37 },
    { 0x8c415e4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbda15e4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90415e54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb415e56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2615e59, SUC_lexicon_tags_6 },
    { 0x46415e5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb815e5d, SUC_lexicon_tags_25 },
    { 0x85815e5d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e15e62, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x4f815e64, SUC_lexicon_tags_1 },
    { 0xa2015e65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55215e68, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e15e6c, SUC_lexicon_tags_0 },
    { 0x9f215e6d, SUC_lexicon_tags_32 },
    { 0x5b615e6e, SUC_lexicon_tags_14 },
    { 0x9a415e6e, SUC_lexicon_tags_12 },
    { 0xbee15e70, SUC_lexicon_tags_23 },
    { 0x3fa15e71, SUC_lexicon_tags_25 },
    { 0x1ea15e6d, SUC_lexicon_tags_17 },
    { 0x26215e73, SUC_lexicon_tags_38 },
    { 0x8f615e74, SUC_lexicon_tags_8 },
    { 0x6c215e75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f015e78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x36615e7b, SUC_lexicon_tags_17 },
    { 0x38415e7c, SUC_lexicon_tags_82 },
    { 0xfac15e7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee15e80, SUC_lexicon_tags_33 },
    { 0x10615e81, SUC_lexicon_tags_4 },
    { 0xb1215e82, SUC_lexicon_tags_15 },
    { 0x4c15e83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x44815e86, SUC_lexicon_tags_12 },
    { 0x18615e87, SUC_lexicon_tags_30 },
    { 0x27e15e86, SUC_lexicon_tags_17 },
    { 0xb9c15e89, SUC_lexicon_tags_17 },
    { 0x81215e89, SUC_lexicon_tags_17 },
    { 0xb0815e8a, SUC_lexicon_tags_49 },
    { 0x8c215e89, SUC_lexicon_tags_37 },
    { 0xba015e89, SUC_lexicon_tags_14 },
    { 0x42215e8e, SUC_lexicon_tags_8 },
    { 0x23415e8c, SUC_lexicon_tags_68 },
    { 0xfa15e90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84615e92, SUC_lexicon_tags_25 },
    { 0xca15e92, SUC_lexicon_tags_33 },
    { 0x39c15e94, SUC_lexicon_tags_77 },
    { 0xb4c15e95, SUC_lexicon_tags_2 },
    { 0xae015e95, SUC_lexicon_tags_37 },
    { 0x46815e96, SUC_lexicon_tags_37 },
    { 0x3a15e92, SUC_lexicon_tags_12 },
    { 0x47e15e99, SUC_lexicon_tags_66 },
    { 0x6f015e9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3015e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e15e9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8615ea6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb815ea8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaca15eaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6015eaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59815eb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e15eb4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x82615eb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96415eba, SUC_lexicon_tags_11 },
    { 0xc8e15ebb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x81c15ebd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615ec1, SUC_lexicon_tags_7 },
    { 0x2815ec2, SUC_lexicon_tags_28 },
    { 0x41615ec1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a815ec6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf7c15ec8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1615eca, SUC_lexicon_tags_1 },
    { 0x36e15eca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac015ed4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46215ed9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80415edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfea15edd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28215edf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bc15ee1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8215ee3, SUC_lexicon_tags_11 },
    { 0x50e15ee3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79815ee6, SUC_lexicon_tags_37 },
    { 0x8f215ee7, SUC_lexicon_tags_25 },
    { 0xd5a15ee8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a015eee, SUC_lexicon_tags_27 },
    { 0x55a15eef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xab415ef2, SUC_lexicon_tags_17 },
    { 0xe1815ef3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae15efe, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc15f02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63215f06, SUC_lexicon_tags_2 },
    { 0x39615f07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf815f0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93615f13, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc15f18, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x99415f1a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf615f1c, SUC_lexicon_tags_7 },
    { 0x3ce15f1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11615f1f, SUC_lexicon_tags_27 },
    { 0xdd015f1f, SUC_lexicon_tags_37 },
    { 0x64015f1f, SUC_lexicon_tags_19 },
    { 0xf4c15f21, SUC_lexicon_tags_17 },
    { 0x2f815f22, SUC_lexicon_tags_37 },
    { 0x3e815f24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6aa15f26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca615f34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6415f3a, SUC_lexicon_tags_17 },
    { 0xf0e15f3a, SUC_lexicon_tags_39 },
    { 0x57415f3c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e215f40, SUC_lexicon_tags_34 },
    { 0x25815f41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe215f46, SUC_lexicon_tags_1 },
    { 0x26e15f47, SUC_lexicon_tags_0 },
    { 0x6ae15f48, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f015f4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3c15f51, SUC_lexicon_tags_61 },
    { 0x2ce15f52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a15f54, SUC_lexicon_tags_67 },
    { 0x2de15f54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47e15f57, SUC_lexicon_tags_25 },
    { 0x83a15f57, SUC_lexicon_tags_39 },
    { 0x7c215f59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c15f66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a15f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c615f6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa15f78, SUC_lexicon_tags_2 },
    { 0xf7c15f79, SUC_lexicon_tags_23 },
    { 0x56815f7a, SUC_lexicon_tags_1 },
    { 0x23815f7a, SUC_lexicon_tags_0 },
    { 0x20615f7c, SUC_lexicon_tags_1 },
    { 0x63415f7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86415f7f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x52015f81, SUC_lexicon_tags_37 },
    { 0x32c15f82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f615f87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc15f8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97015f8e, SUC_lexicon_tags_6 },
    { 0x49015f8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92815f98, SUC_lexicon_tags_23 },
    { 0xdb415f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19415f9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3c15f9d, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc15fa0, SUC_lexicon_tags_12 },
    { 0x73e15fa0, SUC_lexicon_tags_87 },
    { 0xf9015fa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa15fa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58015fb4, SUC_lexicon_tags_15 },
    { 0xc9215fb4, SUC_lexicon_tags_17 },
    { 0xf5e15fb6, SUC_lexicon_tags_0 },
    { 0x31a15fb7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3015fbb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50415fbf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x15215fc2, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1615fc5, SUC_lexicon_tags_25 },
    { 0xfaa15fc6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd5215fc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb0e15fca, SUC_lexicon_tags_33 },
    { 0xa2215fcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da15fd3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x53c15fd5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c15fda, SUC_lexicon_tags_11 },
    { 0x8f815fda, SUC_lexicon_tags_12 },
    { 0x2215fdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc15fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8015fe5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6415fe8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x40015feb, SUC_lexicon_tags_4 },
    { 0x5be15fec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a15ff1, SUC_lexicon_tags_38 },
    { 0x61015ff2, SUC_lexicon_tags_44 },
    { 0x4aa15ff3, SUC_lexicon_tags_1 },
    { 0xc2815ff2, SUC_lexicon_tags_7 },
    { 0x48015ff5, SUC_lexicon_tags_45 },
    { 0xe2a15ff6, SUC_lexicon_tags_37 },
    { 0xaaa15ff6, SUC_lexicon_tags_6 },
    { 0x34815ff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a215ffa, SUC_lexicon_tags_37 },
    { 0x68c15ffa, SUC_lexicon_tags_37 },
    { 0x8be15ffc, SUC_lexicon_tags_12 },
    { 0xe615ffa, SUC_lexicon_tags_25 },
    { 0x98015ffe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c16001, SUC_lexicon_tags_47 },
    { 0x2b816002, SUC_lexicon_tags_0 },
    { 0xede16001, SUC_lexicon_tags_11 },
    { 0xab216004, SUC_lexicon_tags_25 },
    { 0x7fc16005, SUC_lexicon_tags_32 },
    { 0x84816005, SUC_lexicon_tags_19 },
    { 0x18816001, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5861600a, SUC_lexicon_tags_63 },
    { 0xa5c1600b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6001600f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b016014, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c16016, SUC_lexicon_tags_14 },
    { 0xcbc16017, SUC_lexicon_tags_47 },
    { 0x94416018, SUC_lexicon_tags_14 },
    { 0xaf816019, SUC_lexicon_tags_28 },
    { 0x64216018, SUC_lexicon_tags_11 },
    { 0x59c16019, SUC_lexicon_tags_12 },
    { 0x4e81601b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0e1601e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6816022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e16025, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60a16027, SUC_lexicon_tags_17 },
    { 0x1ec16027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb61602a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4816034, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5416038, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8741603a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaa81603c, SUC_lexicon_tags_2 },
    { 0x2361603d, SUC_lexicon_tags_36 },
    { 0x81603e, SUC_lexicon_tags_25 },
    { 0x6141603e, SUC_lexicon_tags_19 },
    { 0x60816040, SUC_lexicon_tags_26 },
    { 0xc9016041, SUC_lexicon_tags_33 },
    { 0x6ae16041, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a616047, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8401604b, SUC_lexicon_tags_25 },
    { 0x5361604b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42e1604e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73616050, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce16054, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a16059, SUC_lexicon_tags_25 },
    { 0x67a1605a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5441605d, SUC_lexicon_tags_6 },
    { 0x21a1605e, SUC_lexicon_tags_2 },
    { 0x4481605f, SUC_lexicon_tags_79 },
    { 0x33a16060, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa3e16062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74616068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2821606c, SUC_lexicon_tags_57 },
    { 0xa281606c, SUC_lexicon_tags_43 },
    { 0x6201606e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3c416070, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc416074, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31416079, SUC_lexicon_tags_15 },
    { 0xbfc16079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1701607c, SUC_lexicon_tags_1 },
    { 0x7ea1607d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6416080, SUC_lexicon_tags_63 },
    { 0xf016081, SUC_lexicon_tags_61 },
    { 0x59c16082, SUC_lexicon_tags_1 },
    { 0x80a16081, SUC_lexicon_tags_34 },
    { 0x27a16082, SUC_lexicon_tags_19 },
    { 0xb816085, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01608a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6e1608c, SUC_lexicon_tags_11 },
    { 0x6041608d, SUC_lexicon_tags_2 },
    { 0xcf81608d, SUC_lexicon_tags_7 },
    { 0x1801608d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x25616091, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e016096, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c21609b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac01609e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc160a5, SUC_lexicon_tags_47 },
    { 0x110160a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf42160a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8160ad, SUC_lexicon_tags_49 },
    { 0x5e4160ae, SUC_lexicon_tags_10 },
    { 0xf14160ae, SUC_lexicon_tags_6 },
    { 0xa92160ae, SUC_lexicon_tags_25 },
    { 0xdb0160b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc160b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5bc160b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe160ba, SUC_lexicon_tags_10 },
    { 0x106160bb, SUC_lexicon_tags_37 },
    { 0xcc0160bb, SUC_lexicon_tags_47 },
    { 0xb9e160bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78160c2, SUC_lexicon_tags_1 },
    { 0x292160c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x894160c5, SUC_lexicon_tags_11 },
    { 0xcae160c6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86160ca, SUC_lexicon_tags_47 },
    { 0xd1a160cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa80160cd, SUC_lexicon_tags_84 },
    { 0xcb8160ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x410160d0, SUC_lexicon_tags_11 },
    { 0x416160d1, SUC_lexicon_tags_30 },
    { 0x2cc160d2, SUC_lexicon_tags_1 },
    { 0x3cc160d3, SUC_lexicon_tags_2 },
    { 0x958160d2, SUC_lexicon_tags_65 },
    { 0xdfe160d3, SUC_lexicon_tags_12 },
    { 0x80a160d6, SUC_lexicon_tags_68 },
    { 0x4fc160d4, SUC_lexicon_tags_4 },
    { 0x382160d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e160db, SUC_lexicon_tags_12 },
    { 0xeaa160dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2160e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2160ec, SUC_lexicon_tags_47 },
    { 0x41a160ec, SUC_lexicon_tags_43 },
    { 0x610160ec, SUC_lexicon_tags_45 },
    { 0x7ee160ef, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc160f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64160f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18160fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9816100, SUC_lexicon_tags_49 },
    { 0xc1616101, SUC_lexicon_tags_87 },
    { 0x45c16101, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e16107, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01610a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc1610d, SUC_lexicon_tags_15 },
    { 0x1c01610e, SUC_lexicon_tags_30 },
    { 0xadc1610f, SUC_lexicon_tags_2 },
    { 0x1461610f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba16113, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7216115, SUC_lexicon_tags_0 },
    { 0xe1216116, SUC_lexicon_tags_25 },
    { 0xeb616117, SUC_lexicon_tags_33 },
    { 0xcf016117, SUC_lexicon_tags_14 },
    { 0x65c16119, SUC_lexicon_tags_129 },
    { 0xace1611a, SUC_lexicon_tags_27 },
    { 0x43a16119, SUC_lexicon_tags_26 },
    { 0x73616116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ce1611e, SUC_lexicon_tags_38 },
    { 0x4681611f, SUC_lexicon_tags_14 },
    { 0x33216120, SUC_lexicon_tags_23 },
    { 0x11816121, SUC_lexicon_tags_47 },
    { 0xcc616122, SUC_lexicon_tags_30 },
    { 0x5a016123, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90416127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c416131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22616135, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb216138, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c1613f, SUC_lexicon_tags_1 },
    { 0xcf41613f, SUC_lexicon_tags_2 },
    { 0x4061613f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc016149, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be1614f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4816152, SUC_lexicon_tags_28 },
    { 0x7f416153, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x96016156, SUC_lexicon_tags_27 },
    { 0x62416157, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa621615b, SUC_lexicon_tags_4 },
    { 0x8641615b, SUC_lexicon_tags_14 },
    { 0x7601615d, SUC_lexicon_tags_61 },
    { 0x13e1615c, SUC_lexicon_tags_43 },
    { 0x8c81615f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0c16161, SUC_lexicon_tags_37 },
    { 0x8ea16162, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca816166, SUC_lexicon_tags_0 },
    { 0x6fc16166, SUC_lexicon_tags_0 },
    { 0x96816168, SUC_lexicon_tags_299 },
    { 0x9e016169, SUC_lexicon_tags_1 },
    { 0x68616169, SUC_lexicon_tags_60 },
    { 0xbe016168, SUC_lexicon_tags_41 },
    { 0x9561616b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71e1616e, SUC_lexicon_tags_25 },
    { 0x8be1616f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b216175, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd941617a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf201617c, SUC_lexicon_tags_31 },
    { 0x2f41617c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7241617f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2416181, SUC_lexicon_tags_68 },
    { 0xf5016181, SUC_lexicon_tags_37 },
    { 0x95216183, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28816188, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e1618d, SUC_lexicon_tags_22 },
    { 0x3341618e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e16193, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8c416195, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c1619f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x13a161a1, SUC_lexicon_tags_27 },
    { 0xfc0161a2, SUC_lexicon_tags_37 },
    { 0x22e161a2, SUC_lexicon_tags_25 },
    { 0xdd0161a4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb06161a6, SUC_lexicon_tags_55 },
    { 0xe96161a6, SUC_lexicon_tags_15 },
    { 0xe10161a8, SUC_lexicon_tags_26 },
    { 0x1f6161a6, SUC_lexicon_tags_17 },
    { 0xa70161aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6161af, SUC_lexicon_tags_127 },
    { 0xc76161b0, SUC_lexicon_tags_61 },
    { 0x188161b1, SUC_lexicon_tags_44 },
    { 0x48e161b2, SUC_lexicon_tags_11 },
    { 0x138161b2, SUC_lexicon_tags_17 },
    { 0xdda161b1, SUC_lexicon_tags_11 },
    { 0xc6a161b1, SUC_lexicon_tags_1 },
    { 0xffc161b6, SUC_lexicon_tags_37 },
    { 0x87a161b7, SUC_lexicon_tags_7 },
    { 0x1c2161b7, SUC_lexicon_tags_17 },
    { 0xe4c161b9, SUC_lexicon_tags_25 },
    { 0xb9c161b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56161bd, SUC_lexicon_tags_37 },
    { 0xacc161be, SUC_lexicon_tags_4 },
    { 0x5fc161be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e8161c1, SUC_lexicon_tags_23 },
    { 0x8a2161c2, SUC_lexicon_tags_7 },
    { 0x5fc161c3, SUC_lexicon_tags_25 },
    { 0x564161c4, SUC_lexicon_tags_82 },
    { 0xbb4161c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4161c8, SUC_lexicon_tags_38 },
    { 0x642161c8, SUC_lexicon_tags_27 },
    { 0x894161c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84161d1, SUC_lexicon_tags_2 },
    { 0x4b8161d1, SUC_lexicon_tags_12 },
    { 0x7b6161d1, SUC_lexicon_tags_14 },
    { 0x7ee161d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18161db, SUC_lexicon_tags_38 },
    { 0x7a4161dc, SUC_lexicon_tags_68 },
    { 0x47e161dd, SUC_lexicon_tags_7 },
    { 0x196161de, SUC_lexicon_tags_37 },
    { 0xa3a161de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32161e3, SUC_lexicon_tags_11 },
    { 0x324161e4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x41e161e6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xdee161e8, SUC_lexicon_tags_37 },
    { 0xff4161e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2161f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x418161f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24161f8, SUC_lexicon_tags_47 },
    { 0x802161f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x804161fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e16202, SUC_lexicon_tags_47 },
    { 0x40816203, SUC_lexicon_tags_33 },
    { 0x40e16204, SUC_lexicon_tags_38 },
    { 0x92616205, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7361620c, SUC_lexicon_tags_47 },
    { 0xb641620c, SUC_lexicon_tags_15 },
    { 0x8721620e, SUC_lexicon_tags_8 },
    { 0xf181620f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11016213, SUC_lexicon_tags_21 },
    { 0x2a416214, SUC_lexicon_tags_43 },
    { 0xfc416215, SUC_lexicon_tags_37 },
    { 0x36a16213, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e16219, SUC_lexicon_tags_25 },
    { 0x2d216219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1381621d, SUC_lexicon_tags_37 },
    { 0x7ba1621e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d016220, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6901622a, SUC_lexicon_tags_49 },
    { 0x19c1622a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfbc1622d, SUC_lexicon_tags_2 },
    { 0x6921622e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c16231, SUC_lexicon_tags_84 },
    { 0x98a16231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e616239, SUC_lexicon_tags_87 },
    { 0xcfa1623a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2701623d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a16241, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e16246, SUC_lexicon_tags_14 },
    { 0xc9816247, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e41624f, SUC_lexicon_tags_53 },
    { 0xa401624f, SUC_lexicon_tags_26 },
    { 0x69e16251, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7216253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2fa16255, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f216257, SUC_lexicon_tags_12 },
    { 0xcd016258, SUC_lexicon_tags_66 },
    { 0x19c16259, SUC_lexicon_tags_30 },
    { 0x5ee1625a, SUC_lexicon_tags_8 },
    { 0x5e81625a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1841625f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8416261, SUC_lexicon_tags_21 },
    { 0xc6816262, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56616267, SUC_lexicon_tags_37 },
    { 0x5c216268, SUC_lexicon_tags_66 },
    { 0xf9e16269, SUC_lexicon_tags_8 },
    { 0xf321626a, SUC_lexicon_tags_13 },
    { 0x26816267, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59616271, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6e16273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc16277, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdba16279, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac21627d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1816280, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9416286, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa141628a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01628e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb9416290, SUC_lexicon_tags_40 },
    { 0x32816291, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb016297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0162a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16162a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194162a9, SUC_lexicon_tags_68 },
    { 0xa84162a9, SUC_lexicon_tags_7 },
    { 0xfb0162ab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x528162ae, SUC_lexicon_tags_60 },
    { 0xda162ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc2162b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0162b8, SUC_lexicon_tags_15 },
    { 0x20e162b9, SUC_lexicon_tags_10 },
    { 0x2e4162b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4162be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26162c1, SUC_lexicon_tags_27 },
    { 0x57c162c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84162c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d0162c9, SUC_lexicon_tags_2 },
    { 0x206162c9, SUC_lexicon_tags_47 },
    { 0x2f0162cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a162d0, SUC_lexicon_tags_32 },
    { 0x466162d1, SUC_lexicon_tags_38 },
    { 0x676162d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612162da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x396162dd, SUC_lexicon_tags_39 },
    { 0x5de162de, SUC_lexicon_tags_6 },
    { 0x2c8162df, SUC_lexicon_tags_30 },
    { 0x2c6162dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c6162e2, SUC_lexicon_tags_15 },
    { 0x842162e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98162e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa162eb, SUC_lexicon_tags_151 },
    { 0xd22162ec, SUC_lexicon_tags_49 },
    { 0x2ca162ec, SUC_lexicon_tags_17 },
    { 0xa7a162ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28162f2, SUC_lexicon_tags_60 },
    { 0x9a2162f2, SUC_lexicon_tags_17 },
    { 0xb98162f3, SUC_lexicon_tags_57 },
    { 0x62e162f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x42c162f7, SUC_lexicon_tags_1 },
    { 0xe08162f7, SUC_lexicon_tags_2 },
    { 0x2ec162f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6162fd, SUC_lexicon_tags_37 },
    { 0xbac162fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc16303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34616308, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5941630a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41630d, SUC_lexicon_tags_45 },
    { 0x5e61630d, SUC_lexicon_tags_11 },
    { 0xf561630d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63816316, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x20e16318, SUC_lexicon_tags_38 },
    { 0x41a16319, SUC_lexicon_tags_17 },
    { 0x2021631a, SUC_lexicon_tags_23 },
    { 0x5341631b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78c1631d, SUC_lexicon_tags_23 },
    { 0x2c1631e, SUC_lexicon_tags_38 },
    { 0x32a1631e, SUC_lexicon_tags_25 },
    { 0xe6016320, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8816325, SUC_lexicon_tags_1 },
    { 0x73816325, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3c16328, SUC_lexicon_tags_1 },
    { 0x3bc16329, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb816331, SUC_lexicon_tags_38 },
    { 0xa1816331, SUC_lexicon_tags_88 },
    { 0xc1416331, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0x94216335, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa841633b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23816340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe616342, SUC_lexicon_tags_26 },
    { 0x48416342, SUC_lexicon_tags_37 },
    { 0xed016343, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x35816346, SUC_lexicon_tags_17 },
    { 0x45216347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc901634a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461634f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2616352, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc16354, SUC_lexicon_tags_1 },
    { 0x40416355, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35416357, SUC_lexicon_tags_19 },
    { 0x51c16358, SUC_lexicon_tags_45 },
    { 0x13216358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a1635f, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xf4a16361, SUC_lexicon_tags_11 },
    { 0x81e16362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf016364, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec16366, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75a16368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8261636d, SUC_lexicon_tags_17 },
    { 0xb201636e, SUC_lexicon_tags_7 },
    { 0x92e1636e, SUC_lexicon_tags_35 },
    { 0xedc16370, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa1637a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e1637e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd3e16380, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d016387, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f416389, SUC_lexicon_tags_28 },
    { 0xf201638a, SUC_lexicon_tags_30 },
    { 0xa121638a, SUC_lexicon_tags_26 },
    { 0x7aa1638b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4cc1638e, SUC_lexicon_tags_39 },
    { 0xa6a1638e, SUC_lexicon_tags_11 },
    { 0xae016390, SUC_lexicon_tags_4 },
    { 0x5721638f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa816397, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d416399, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a1639c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696163a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c163a7, SUC_lexicon_tags_29 },
    { 0x526163a8, SUC_lexicon_tags_230 },
    { 0x7fa163a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd52163ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x866163ad, SUC_lexicon_tags_32 },
    { 0xaf8163ad, SUC_lexicon_tags_16 },
    { 0x4b2163af, SUC_lexicon_tags_38 },
    { 0x7ba163b0, SUC_lexicon_tags_1 },
    { 0x40c163b1, SUC_lexicon_tags_11 },
    { 0xbcc163b2, SUC_lexicon_tags_80 },
    { 0x86c163b3, SUC_lexicon_tags_17 },
    { 0x654163b4, SUC_lexicon_tags_49 },
    { 0x676163b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8163ba, SUC_lexicon_tags_8 },
    { 0xcb0163ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x342163be, SUC_lexicon_tags_8 },
    { 0x788163bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52163c1, SUC_lexicon_tags_10 },
    { 0xf28163c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef8163c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90163c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254163cd, SUC_lexicon_tags_37 },
    { 0x4e163ce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8163d1, SUC_lexicon_tags_7 },
    { 0x672163d2, SUC_lexicon_tags_1 },
    { 0x204163d3, SUC_lexicon_tags_14 },
    { 0xbc163d3, SUC_lexicon_tags_17 },
    { 0xb96163d5, SUC_lexicon_tags_2 },
    { 0x1f4163d4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x676163d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf02163db, SUC_lexicon_tags_37 },
    { 0x6ba163db, SUC_lexicon_tags_0 },
    { 0xa8e163dc, SUC_lexicon_tags_6 },
    { 0x124163de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x60a163e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b0163e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2163ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf2c163ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2163ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae163f2, SUC_lexicon_tags_23 },
    { 0xde8163f2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930163fc, SUC_lexicon_tags_29 },
    { 0xaa4163fd, SUC_lexicon_tags_1 },
    { 0x7e4163fe, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xd2416400, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a216402, SUC_lexicon_tags_4 },
    { 0xe1e16403, SUC_lexicon_tags_41 },
    { 0x1d816404, SUC_lexicon_tags_1 },
    { 0x29216404, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x11816407, SUC_lexicon_tags_28 },
    { 0xc5c16408, SUC_lexicon_tags_38 },
    { 0x65616409, SUC_lexicon_tags_48 },
    { 0xbb21640a, SUC_lexicon_tags_37 },
    { 0xfca1640b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6861640f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x67816411, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96016416, SUC_lexicon_tags_32 },
    { 0x7d016417, SUC_lexicon_tags_23 },
    { 0x25a16416, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc201641a, SUC_lexicon_tags_7 },
    { 0x9641641a, SUC_lexicon_tags_1 },
    { 0x7641641c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd8a1641e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0816420, SUC_lexicon_tags_10 },
    { 0x92c16421, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca16426, SUC_lexicon_tags_60 },
    { 0x44216427, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc81642c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ac1642e, SUC_lexicon_tags_3 },
    { 0xace1642e, SUC_lexicon_tags_8 },
    { 0x7dc16430, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x60216432, SUC_lexicon_tags_23 },
    { 0x5a016432, SUC_lexicon_tags_33 },
    { 0xb2616434, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc16438, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361643c, SUC_lexicon_tags_50 },
    { 0x6a21643d, SUC_lexicon_tags_113 },
    { 0x9541643e, SUC_lexicon_tags_37 },
    { 0xdd41643f, SUC_lexicon_tags_61 },
    { 0x5161643f, SUC_lexicon_tags_17 },
    { 0xefa16441, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed616444, SUC_lexicon_tags_25 },
    { 0xbea16444, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a1644c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3816451, SUC_lexicon_tags_29 },
    { 0x58416451, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa21645c, SUC_lexicon_tags_7 },
    { 0x2c21645d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de16463, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71016465, SUC_lexicon_tags_17 },
    { 0x7e616466, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4c16468, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2021646a, SUC_lexicon_tags_161 },
    { 0x2821646b, SUC_lexicon_tags_27 },
    { 0x13c1646c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9201646e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x45416471, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44016477, SUC_lexicon_tags_7 },
    { 0xb5816478, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb41647b, SUC_lexicon_tags_63 },
    { 0xa1e1647c, SUC_lexicon_tags_7 },
    { 0x86a1647d, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8616484, SUC_lexicon_tags_2 },
    { 0xc5e16485, SUC_lexicon_tags_182 },
    { 0x8fe16486, SUC_lexicon_tags_85 },
    { 0x5ca16484, SUC_lexicon_tags_25 },
    { 0x2016488, SUC_lexicon_tags_87 },
    { 0xf2816489, SUC_lexicon_tags_1 },
    { 0x6e416489, SUC_lexicon_tags_11 },
    { 0x4a1648b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb81648d, SUC_lexicon_tags_0 },
    { 0xe661648e, SUC_lexicon_tags_25 },
    { 0x8661648e, SUC_lexicon_tags_84 },
    { 0x79616490, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e816493, SUC_lexicon_tags_11 },
    { 0x9cc16494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e1649a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9701649d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9b21649f, SUC_lexicon_tags_29 },
    { 0x4c8164a0, SUC_lexicon_tags_23 },
    { 0x1da164a1, SUC_lexicon_tags_66 },
    { 0x224164a2, SUC_lexicon_tags_12 },
    { 0xcbc164a1, SUC_lexicon_tags_28 },
    { 0xc8e164a4, SUC_lexicon_tags_55 },
    { 0x398164a5, SUC_lexicon_tags_15 },
    { 0xfdc164a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x820164a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6164ae, SUC_lexicon_tags_29 },
    { 0xeca164ae, SUC_lexicon_tags_12 },
    { 0xd04164b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a164b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa76164b6, SUC_lexicon_tags_23 },
    { 0x7c4164b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e164ba, SUC_lexicon_tags_77 },
    { 0xf92164bb, SUC_lexicon_tags_67 },
    { 0x5f8164bc, SUC_lexicon_tags_45 },
    { 0xc9a164bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c164c3, SUC_lexicon_tags_19 },
    { 0x764164c4, SUC_lexicon_tags_63 },
    { 0x688164c3, SUC_lexicon_tags_5 },
    { 0x5cc164c4, SUC_lexicon_tags_9 },
    { 0x950164c5, SUC_lexicon_tags_19 },
    { 0xd36164c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c164cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x708164d0, SUC_lexicon_tags_9 },
    { 0x75c164d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac164da, SUC_lexicon_tags_1 },
    { 0x182164db, SUC_lexicon_tags_15 },
    { 0x30164dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xddc164de, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506164e2, SUC_lexicon_tags_68 },
    { 0xd48164e3, SUC_lexicon_tags_12 },
    { 0x646164e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac164ee, SUC_lexicon_tags_37 },
    { 0x7ac164ef, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x9c0164f1, SUC_lexicon_tags_7 },
    { 0xf74164f1, SUC_lexicon_tags_12 },
    { 0xb48164f1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x52164f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908164f9, SUC_lexicon_tags_17 },
    { 0xc4c164f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6164fe, SUC_lexicon_tags_45 },
    { 0x742164ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf616501, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c16505, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xa3816507, SUC_lexicon_tags_25 },
    { 0x5d416507, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac1650b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7641650e, SUC_lexicon_tags_27 },
    { 0x5d01650f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c16517, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde616519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1661651d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb3c1651f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc16523, SUC_lexicon_tags_8 },
    { 0xa816523, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44216526, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e81652f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a16534, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57816536, SUC_lexicon_tags_10 },
    { 0x36616536, SUC_lexicon_tags_29 },
    { 0x84a16538, SUC_lexicon_tags_4 },
    { 0xb1c16539, SUC_lexicon_tags_11 },
    { 0x2981653a, SUC_lexicon_tags_17 },
    { 0x83a1653b, SUC_lexicon_tags_49 },
    { 0xade1653a, SUC_lexicon_tags_19 },
    { 0x27c1653d, SUC_lexicon_tags_1 },
    { 0xdb81653d, SUC_lexicon_tags_32 },
    { 0x3661653e, SUC_lexicon_tags_17 },
    { 0x2e1653d, SUC_lexicon_tags_14 },
    { 0x3416539, SUC_lexicon_tags_14 },
    { 0xa2416540, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa16545, SUC_lexicon_tags_45 },
    { 0xa6016546, SUC_lexicon_tags_25 },
    { 0x10216546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a1654c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd016552, SUC_lexicon_tags_26 },
    { 0xd9a16552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20416558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b41655c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5801655f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae16564, SUC_lexicon_tags_63 },
    { 0xd5016564, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba1656a, SUC_lexicon_tags_32 },
    { 0x8941656b, SUC_lexicon_tags_160 },
    { 0x2a21656c, SUC_lexicon_tags_27 },
    { 0x34e1656d, SUC_lexicon_tags_63 },
    { 0xe4e1656b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da16574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c1657b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c1657f, SUC_lexicon_tags_25 },
    { 0x82016580, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x44016583, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfda16585, SUC_lexicon_tags_15 },
    { 0x4ea16586, SUC_lexicon_tags_2 },
    { 0x2ba16586, SUC_lexicon_tags_25 },
    { 0x68e16585, SUC_lexicon_tags_17 },
    { 0x8ca16589, SUC_lexicon_tags_37 },
    { 0xf5016585, SUC_lexicon_tags_43 },
    { 0x8261658b, SUC_lexicon_tags_19 },
    { 0xc381658c, SUC_lexicon_tags_33 },
    { 0xed41658d, SUC_lexicon_tags_4 },
    { 0x6ec1658e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54816590, SUC_lexicon_tags_49 },
    { 0xbe16591, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf816599, SUC_lexicon_tags_39 },
    { 0x2821659a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9f81659c, SUC_lexicon_tags_37 },
    { 0xa361659d, SUC_lexicon_tags_0 },
    { 0x68e1659c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea165a4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc165ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5fc165ad, SUC_lexicon_tags_25 },
    { 0x83c165ae, SUC_lexicon_tags_49 },
    { 0xbdc165af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8e165b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64165b4, SUC_lexicon_tags_12 },
    { 0x99a165b5, SUC_lexicon_tags_1 },
    { 0x1e4165b5, SUC_lexicon_tags_17 },
    { 0x328165b6, SUC_lexicon_tags_8 },
    { 0xd90165b8, SUC_lexicon_tags_17 },
    { 0x3e0165b8, SUC_lexicon_tags_12 },
    { 0x390165ba, SUC_lexicon_tags_108 },
    { 0xa84165b7, SUC_lexicon_tags_16 },
    { 0xa0e165b5, SUC_lexicon_tags_13 },
    { 0x692165bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c165c4, SUC_lexicon_tags_23 },
    { 0xad6165c4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee165c9, SUC_lexicon_tags_37 },
    { 0xe86165ca, SUC_lexicon_tags_17 },
    { 0x5c0165c9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e165ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34165d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf22165d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8165d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22165dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc165e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc165e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae165e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de165e9, SUC_lexicon_tags_7 },
    { 0xd0a165ea, SUC_lexicon_tags_15 },
    { 0xbc4165e9, SUC_lexicon_tags_42 },
    { 0xaee165e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6165f1, SUC_lexicon_tags_1 },
    { 0xe54165f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x810165f4, SUC_lexicon_tags_48 },
    { 0xdac165f5, SUC_lexicon_tags_11 },
    { 0x4f2165f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x802165f8, SUC_lexicon_tags_11 },
    { 0x97a165f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96165fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a416601, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x42216603, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d81660b, SUC_lexicon_tags_43 },
    { 0xe841660c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35016614, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc0a16616, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59e16618, SUC_lexicon_tags_21 },
    { 0x98216619, SUC_lexicon_tags_15 },
    { 0x8d416619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30016620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16e16622, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc16625, SUC_lexicon_tags_30 },
    { 0x48616625, SUC_lexicon_tags_19 },
    { 0x2ba16627, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7016629, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b61662b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2441662d, SUC_lexicon_tags_47 },
    { 0xc0a1662e, SUC_lexicon_tags_7 },
    { 0xdc01662e, SUC_lexicon_tags_3 },
    { 0xb5216630, SUC_lexicon_tags_19 },
    { 0x62616631, SUC_lexicon_tags_11 },
    { 0x2016632, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a16639, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a16642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55416646, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x86616649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa821664d, SUC_lexicon_tags_641 },
    { 0, NULL },
    { 0xefe1664f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41416651, SUC_lexicon_tags_28 },
    { 0x6e816651, SUC_lexicon_tags_7 },
    { 0x98c16653, SUC_lexicon_tags_37 },
    { 0xfd416653, SUC_lexicon_tags_29 },
    { 0xe2416651, SUC_lexicon_tags_104 },
    { 0x6ea16656, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8581665a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb561665c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3241665e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe4416660, SUC_lexicon_tags_7 },
    { 0x8c216660, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe16667, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3a16669, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e61666b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe201666e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e16671, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f216675, SUC_lexicon_tags_19 },
    { 0x40816676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66216679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e16681, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x89816684, SUC_lexicon_tags_17 },
    { 0x86616684, SUC_lexicon_tags_25 },
    { 0xba816684, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e16689, SUC_lexicon_tags_49 },
    { 0x9d41668a, SUC_lexicon_tags_42 },
    { 0x9661668b, SUC_lexicon_tags_33 },
    { 0x8681668a, SUC_lexicon_tags_8 },
    { 0x1a01668d, SUC_lexicon_tags_54 },
    { 0x87016689, SUC_lexicon_tags_11 },
    { 0xf781668f, SUC_lexicon_tags_11 },
    { 0xfc616690, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcac16692, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17016696, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77c16698, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a1669b, SUC_lexicon_tags_11 },
    { 0xd2a1669c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de166a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa166a5, SUC_lexicon_tags_33 },
    { 0x5de166a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc166a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb4166ab, SUC_lexicon_tags_15 },
    { 0x1c8166ac, SUC_lexicon_tags_27 },
    { 0xd7a166ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a0166af, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5be166b1, SUC_lexicon_tags_8 },
    { 0x4fc166b2, SUC_lexicon_tags_12 },
    { 0x2e6166b3, SUC_lexicon_tags_60 },
    { 0x6cc166b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e166ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd76166bc, SUC_lexicon_tags_37 },
    { 0x330166bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80166c3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf6c166c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fc166c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c166cc, SUC_lexicon_tags_38 },
    { 0xec4166cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0166d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3c166d4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x85c166d6, SUC_lexicon_tags_53 },
    { 0xe2a166d7, SUC_lexicon_tags_37 },
    { 0xebc166d8, SUC_lexicon_tags_2 },
    { 0xef6166d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4166dd, SUC_lexicon_tags_26 },
    { 0xaf4166dd, SUC_lexicon_tags_14 },
    { 0xa96166df, SUC_lexicon_tags_17 },
    { 0xd14166de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ba166e2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232166e6, SUC_lexicon_tags_17 },
    { 0xb0a166e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e166ea, SUC_lexicon_tags_15 },
    { 0x5b8166eb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x844166ee, SUC_lexicon_tags_28 },
    { 0x3d8166ef, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6166f2, SUC_lexicon_tags_26 },
    { 0x6e4166f3, SUC_lexicon_tags_19 },
    { 0xcd4166f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8166f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92166fd, SUC_lexicon_tags_14 },
    { 0xada166fd, SUC_lexicon_tags_37 },
    { 0x8aa166ff, SUC_lexicon_tags_26 },
    { 0x1f616700, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe16707, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f41670b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e1670f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8816719, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a1671d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83616721, SUC_lexicon_tags_47 },
    { 0x2b616722, SUC_lexicon_tags_13 },
    { 0xa2616722, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4616727, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb341672b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c616730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86816733, SUC_lexicon_tags_11 },
    { 0xf2a16733, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfb416736, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf9216738, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e16746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18816748, SUC_lexicon_tags_11 },
    { 0x16e16749, SUC_lexicon_tags_25 },
    { 0xbc616748, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1c41674c, SUC_lexicon_tags_45 },
    { 0x1f41674d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c16750, SUC_lexicon_tags_1 },
    { 0xfe416750, SUC_lexicon_tags_17 },
    { 0x16216752, SUC_lexicon_tags_38 },
    { 0xcb816753, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d01675c, SUC_lexicon_tags_46 },
    { 0xfca1675d, SUC_lexicon_tags_15 },
    { 0x6981675e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39616762, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8816765, SUC_lexicon_tags_0 },
    { 0xdc016766, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37216768, SUC_lexicon_tags_11 },
    { 0x55416769, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd901676d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4601676f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3016773, SUC_lexicon_tags_7 },
    { 0x9d816773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38216779, SUC_lexicon_tags_29 },
    { 0xc921677a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d01677c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ae1677e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7616780, SUC_lexicon_tags_37 },
    { 0x16a16781, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c016784, SUC_lexicon_tags_33 },
    { 0xb3016785, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x16a16787, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3121678b, SUC_lexicon_tags_17 },
    { 0x121678c, SUC_lexicon_tags_29 },
    { 0x46a1678b, SUC_lexicon_tags_17 },
    { 0xbb01678d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa16792, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfda16794, SUC_lexicon_tags_37 },
    { 0xf8e16794, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4216797, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e1679b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc21679d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x601679f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf26167a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4167ae, SUC_lexicon_tags_38 },
    { 0xcde167af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48167b2, SUC_lexicon_tags_14 },
    { 0x6fa167b3, SUC_lexicon_tags_11 },
    { 0x4e167b3, SUC_lexicon_tags_37 },
    { 0xf72167b5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88167bc, SUC_lexicon_tags_7 },
    { 0xf5e167bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2167c0, SUC_lexicon_tags_38 },
    { 0x94167c0, SUC_lexicon_tags_25 },
    { 0x452167c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x88167c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2c167c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e167ca, SUC_lexicon_tags_17 },
    { 0x4dc167ca, SUC_lexicon_tags_6 },
    { 0x4be167cc, SUC_lexicon_tags_31 },
    { 0xbf6167cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c167cf, SUC_lexicon_tags_2 },
    { 0xbe0167d0, SUC_lexicon_tags_23 },
    { 0xd92167cf, SUC_lexicon_tags_40 },
    { 0xb9e167d2, SUC_lexicon_tags_12 },
    { 0x4e8167d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a167d6, SUC_lexicon_tags_11 },
    { 0xfa167d7, SUC_lexicon_tags_15 },
    { 0x4e4167d7, SUC_lexicon_tags_17 },
    { 0x2ac167d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762167de, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e167e3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc167e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e167e9, SUC_lexicon_tags_17 },
    { 0xfac167ea, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20167ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x484167f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdfc167f3, SUC_lexicon_tags_28 },
    { 0x3b2167f4, SUC_lexicon_tags_11 },
    { 0x862167f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a167f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c16801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83016804, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d616807, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4001680d, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de16812, SUC_lexicon_tags_25 },
    { 0x72616812, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51216819, SUC_lexicon_tags_56 },
    { 0xf5a1681a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76e1681c, SUC_lexicon_tags_105 },
    { 0x96e1681d, SUC_lexicon_tags_14 },
    { 0xeb21681e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x37016821, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x33616823, SUC_lexicon_tags_49 },
    { 0x34c16824, SUC_lexicon_tags_37 },
    { 0x24216823, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94416837, SUC_lexicon_tags_37 },
    { 0x64e16838, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a1683b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e16842, SUC_lexicon_tags_1 },
    { 0x99216843, SUC_lexicon_tags_1 },
    { 0x49616842, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf21684d, SUC_lexicon_tags_37 },
    { 0xe81684e, SUC_lexicon_tags_25 },
    { 0x7781684f, SUC_lexicon_tags_27 },
    { 0x4481684d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74416852, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf216855, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64616859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec01685b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fe1685d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a16860, SUC_lexicon_tags_31 },
    { 0x28416860, SUC_lexicon_tags_33 },
    { 0x4b816860, SUC_lexicon_tags_26 },
    { 0xf416860, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e616865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13616868, SUC_lexicon_tags_2 },
    { 0xf8216869, SUC_lexicon_tags_65 },
    { 0x6ce16869, SUC_lexicon_tags_37 },
    { 0x4da1686b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd0e1686d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac16871, SUC_lexicon_tags_66 },
    { 0x18816872, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x76416874, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85216876, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1541687b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xda01687e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x39216880, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1b216882, SUC_lexicon_tags_63 },
    { 0xcaa16883, SUC_lexicon_tags_2 },
    { 0xe9816882, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4216888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e16894, SUC_lexicon_tags_11 },
    { 0x16c16895, SUC_lexicon_tags_1 },
    { 0x10016894, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb41689a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6168a2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2c2168a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7be168a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc68168a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2168ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8168b4, SUC_lexicon_tags_5 },
    { 0x8d0168b4, SUC_lexicon_tags_22 },
    { 0x48e168b4, SUC_lexicon_tags_17 },
    { 0xbb6168b7, SUC_lexicon_tags_14 },
    { 0x86a168b8, SUC_lexicon_tags_12 },
    { 0x81c168b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfca168bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a168be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x638168c1, SUC_lexicon_tags_37 },
    { 0x4a0168c2, SUC_lexicon_tags_33 },
    { 0x5b6168c1, SUC_lexicon_tags_5 },
    { 0x6c2168c1, SUC_lexicon_tags_26 },
    { 0x226168c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb0c168c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc168ca, SUC_lexicon_tags_33 },
    { 0x220168cb, SUC_lexicon_tags_151 },
    { 0x7a8168cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x878168ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0168d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a168d4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1e6168d6, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x628168d8, SUC_lexicon_tags_63 },
    { 0x94a168d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6168e2, SUC_lexicon_tags_60 },
    { 0x876168e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb64168e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99e168e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708168ec, SUC_lexicon_tags_1 },
    { 0xa72168ed, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xf84168ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960168f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba168fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x156168ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7216905, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9616907, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b01690b, SUC_lexicon_tags_7 },
    { 0xc781690c, SUC_lexicon_tags_17 },
    { 0xabe1690d, SUC_lexicon_tags_21 },
    { 0xec21690e, SUC_lexicon_tags_11 },
    { 0x9aa1690f, SUC_lexicon_tags_26 },
    { 0x1881690e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e16913, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c816916, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16616924, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed216927, SUC_lexicon_tags_5 },
    { 0x6e416928, SUC_lexicon_tags_37 },
    { 0x28616929, SUC_lexicon_tags_11 },
    { 0xe661692a, SUC_lexicon_tags_25 },
    { 0x77e1692b, SUC_lexicon_tags_1 },
    { 0xb6a1692c, SUC_lexicon_tags_6 },
    { 0x54e16927, SUC_lexicon_tags_15 },
    { 0x5961692e, SUC_lexicon_tags_1 },
    { 0x8c81692c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87416931, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1b816933, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69616935, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x21416938, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8ba1693a, SUC_lexicon_tags_17 },
    { 0x12e1693b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1be1693d, SUC_lexicon_tags_79 },
    { 0x7be1693e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d616941, SUC_lexicon_tags_12 },
    { 0x78416942, SUC_lexicon_tags_191 },
    { 0x13e16943, SUC_lexicon_tags_38 },
    { 0x32216941, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5416949, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe081694c, SUC_lexicon_tags_35 },
    { 0x8c81694d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc016950, SUC_lexicon_tags_2 },
    { 0x93e16951, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4616953, SUC_lexicon_tags_37 },
    { 0xd7e16954, SUC_lexicon_tags_37 },
    { 0x40416954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8816959, SUC_lexicon_tags_27 },
    { 0x24216959, SUC_lexicon_tags_11 },
    { 0xb9016959, SUC_lexicon_tags_37 },
    { 0xf2c16959, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd1c1695e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc416961, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d416965, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd5416967, SUC_lexicon_tags_1 },
    { 0x76416967, SUC_lexicon_tags_7 },
    { 0x80016967, SUC_lexicon_tags_11 },
    { 0xdc21696a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf101696c, SUC_lexicon_tags_7 },
    { 0x4ec1696c, SUC_lexicon_tags_16 },
    { 0xeae1696e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b016972, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce16977, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc1697d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd421697f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c16987, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c1698d, SUC_lexicon_tags_384 },
    { 0, NULL },
    { 0, NULL },
    { 0x83616990, SUC_lexicon_tags_30 },
    { 0x88816991, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec01699a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8081699f, SUC_lexicon_tags_37 },
    { 0xc601699f, SUC_lexicon_tags_45 },
    { 0x542169a1, SUC_lexicon_tags_2 },
    { 0xffc169a1, SUC_lexicon_tags_65 },
    { 0x612169a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9ca169a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56169a8, SUC_lexicon_tags_1 },
    { 0xda6169a9, SUC_lexicon_tags_9 },
    { 0x8b6169aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0169b0, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa169b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa169bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x262169bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xee8169bf, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xc70169c1, SUC_lexicon_tags_2 },
    { 0x88a169c2, SUC_lexicon_tags_25 },
    { 0xdd2169c1, SUC_lexicon_tags_17 },
    { 0x2f0169c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e169c8, SUC_lexicon_tags_12 },
    { 0x3f4169c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46c169cb, SUC_lexicon_tags_26 },
    { 0xd90169cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c169d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c169d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118169db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c169de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c169e1, SUC_lexicon_tags_6 },
    { 0x4b2169e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6fa169e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e169e8, SUC_lexicon_tags_9 },
    { 0x552169e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb4169eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x354169ed, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52169f0, SUC_lexicon_tags_31 },
    { 0x2f4169f1, SUC_lexicon_tags_65 },
    { 0xfa8169f2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe84169f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x716169f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2169f9, SUC_lexicon_tags_1 },
    { 0x58c169fa, SUC_lexicon_tags_17 },
    { 0x1a6169fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e16a01, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xea416a04, SUC_lexicon_tags_17 },
    { 0x16c16a05, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f616a0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x82616a0d, SUC_lexicon_tags_6 },
    { 0xf0816a0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e416a11, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaf816a13, SUC_lexicon_tags_11 },
    { 0x7ea16a14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7016a18, SUC_lexicon_tags_4 },
    { 0x54c16a19, SUC_lexicon_tags_7 },
    { 0xb616a1a, SUC_lexicon_tags_1 },
    { 0xbde16a18, SUC_lexicon_tags_11 },
    { 0xc816a1c, SUC_lexicon_tags_8 },
    { 0x6e16a18, SUC_lexicon_tags_7 },
    { 0x79e16a1e, SUC_lexicon_tags_6 },
    { 0xcbc16a1c, SUC_lexicon_tags_11 },
    { 0xad816a1f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf816a24, SUC_lexicon_tags_47 },
    { 0x76a16a25, SUC_lexicon_tags_15 },
    { 0x2da16a26, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x95016a29, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c16a32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98416a38, SUC_lexicon_tags_15 },
    { 0x3e616a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d016a3c, SUC_lexicon_tags_8 },
    { 0x9f816a3d, SUC_lexicon_tags_364 },
    { 0x33816a3d, SUC_lexicon_tags_68 },
    { 0xc8016a3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9be16a41, SUC_lexicon_tags_25 },
    { 0xd4616a42, SUC_lexicon_tags_12 },
    { 0xaf016a41, SUC_lexicon_tags_85 },
    { 0x83a16a42, SUC_lexicon_tags_7 },
    { 0x47c16a45, SUC_lexicon_tags_19 },
    { 0x1dc16a45, SUC_lexicon_tags_5 },
    { 0xbce16a44, SUC_lexicon_tags_11 },
    { 0x5be16a46, SUC_lexicon_tags_72 },
    { 0x64c16a49, SUC_lexicon_tags_15 },
    { 0x20a16a4a, SUC_lexicon_tags_11 },
    { 0x5c016a4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c216a4d, SUC_lexicon_tags_1 },
    { 0x36a16a4e, SUC_lexicon_tags_33 },
    { 0x31e16a4e, SUC_lexicon_tags_11 },
    { 0xff216a50, SUC_lexicon_tags_29 },
    { 0x29616a51, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1016a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8c16a57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xece16a59, SUC_lexicon_tags_27 },
    { 0x68816a5a, SUC_lexicon_tags_2 },
    { 0x87616a5b, SUC_lexicon_tags_7 },
    { 0x5c816a5b, SUC_lexicon_tags_90 },
    { 0x4b816a5a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60216a61, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x73216a63, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7616a66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a216a6a, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3816a6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e16a70, SUC_lexicon_tags_29 },
    { 0x68216a71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5016a78, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f016a7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57416a80, SUC_lexicon_tags_32 },
    { 0x82e16a81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc16a85, SUC_lexicon_tags_15 },
    { 0xee016a85, SUC_lexicon_tags_57 },
    { 0x99016a85, SUC_lexicon_tags_17 },
    { 0xe0816a86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17216a92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e016a94, SUC_lexicon_tags_47 },
    { 0x15816a95, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a16a99, SUC_lexicon_tags_7 },
    { 0x53816a9a, SUC_lexicon_tags_60 },
    { 0xe8416a9b, SUC_lexicon_tags_27 },
    { 0x82a16a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30216aa2, SUC_lexicon_tags_39 },
    { 0x97016aa2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1816aa6, SUC_lexicon_tags_25 },
    { 0xb2816aa6, SUC_lexicon_tags_38 },
    { 0xc1616aa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa016aaa, SUC_lexicon_tags_14 },
    { 0x62416aab, SUC_lexicon_tags_65 },
    { 0xc0216aac, SUC_lexicon_tags_14 },
    { 0xab616aac, SUC_lexicon_tags_37 },
    { 0xa9e16aac, SUC_lexicon_tags_1 },
    { 0xc7416aaf, SUC_lexicon_tags_12 },
    { 0xa3e16aac, SUC_lexicon_tags_37 },
    { 0xf2216ab1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40016ab5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d816aba, SUC_lexicon_tags_35 },
    { 0xa1816abb, SUC_lexicon_tags_4 },
    { 0xd1416aba, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e16ac1, SUC_lexicon_tags_12 },
    { 0x95016ac2, SUC_lexicon_tags_44 },
    { 0x4c616ac3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29e16ac5, SUC_lexicon_tags_15 },
    { 0xd9216ac6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb5216ac8, SUC_lexicon_tags_6 },
    { 0x2ae16ac9, SUC_lexicon_tags_27 },
    { 0xbb016aca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f016acc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66e16ace, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7416ad3, SUC_lexicon_tags_23 },
    { 0x16e16ad3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e16ad7, SUC_lexicon_tags_2 },
    { 0x89416ad7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e16adb, SUC_lexicon_tags_9 },
    { 0xfb616adc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82816ae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98216ae5, SUC_lexicon_tags_7 },
    { 0x63e16ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39616ae9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52816aef, SUC_lexicon_tags_1 },
    { 0x9c416af0, SUC_lexicon_tags_1 },
    { 0xb9416af1, SUC_lexicon_tags_82 },
    { 0x4f016af1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ea16af4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c816af6, SUC_lexicon_tags_12 },
    { 0x1c16af7, SUC_lexicon_tags_40 },
    { 0xb7616af7, SUC_lexicon_tags_37 },
    { 0x79a16af7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c16afc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf816b04, SUC_lexicon_tags_25 },
    { 0xdfe16b05, SUC_lexicon_tags_2 },
    { 0xb6616b06, SUC_lexicon_tags_27 },
    { 0x47816b04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92016b0e, SUC_lexicon_tags_10 },
    { 0x75a16b0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d416b11, SUC_lexicon_tags_1 },
    { 0xf2416b12, SUC_lexicon_tags_49 },
    { 0x40a16b11, SUC_lexicon_tags_29 },
    { 0xcec16b14, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc2816b16, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5616b19, SUC_lexicon_tags_7 },
    { 0x67616b1a, SUC_lexicon_tags_39 },
    { 0x23c16b1b, SUC_lexicon_tags_48 },
    { 0x76816b1b, SUC_lexicon_tags_10 },
    { 0xee16b1c, SUC_lexicon_tags_15 },
    { 0x3fe16b1e, SUC_lexicon_tags_47 },
    { 0xca016b19, SUC_lexicon_tags_33 },
    { 0x6f216b1a, SUC_lexicon_tags_15 },
    { 0xb8e16b1a, SUC_lexicon_tags_12 },
    { 0x51816b22, SUC_lexicon_tags_14 },
    { 0x95016b23, SUC_lexicon_tags_17 },
    { 0x1dc16b23, SUC_lexicon_tags_60 },
    { 0x4ac16b24, SUC_lexicon_tags_17 },
    { 0xf1016b26, SUC_lexicon_tags_17 },
    { 0xd1a16b27, SUC_lexicon_tags_38 },
    { 0xa8e16b24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92416b2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c16b31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe6c16b33, SUC_lexicon_tags_12 },
    { 0x16a16b34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1816b38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c016b3a, SUC_lexicon_tags_17 },
    { 0x4e416b3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5216b3f, SUC_lexicon_tags_4 },
    { 0xefe16b40, SUC_lexicon_tags_25 },
    { 0x86a16b40, SUC_lexicon_tags_17 },
    { 0x7ea16b42, SUC_lexicon_tags_60 },
    { 0x26e16b41, SUC_lexicon_tags_44 },
    { 0xd9c16b44, SUC_lexicon_tags_35 },
    { 0x34c16b43, SUC_lexicon_tags_11 },
    { 0x71a16b41, SUC_lexicon_tags_19 },
    { 0x8d416b3f, SUC_lexicon_tags_68 },
    { 0x12c16b44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a416b4d, SUC_lexicon_tags_1 },
    { 0xc7e16b4e, SUC_lexicon_tags_45 },
    { 0xb8616b4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a16b53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0416b57, SUC_lexicon_tags_66 },
    { 0x3d416b58, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44016b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d416b61, SUC_lexicon_tags_22 },
    { 0x1a616b62, SUC_lexicon_tags_7 },
    { 0xb5c16b63, SUC_lexicon_tags_17 },
    { 0x216b62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f616b67, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b616b6a, SUC_lexicon_tags_61 },
    { 0x57816b6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe216b6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68216b72, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa6616b74, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a816b77, SUC_lexicon_tags_47 },
    { 0x50a16b77, SUC_lexicon_tags_38 },
    { 0x3f416b79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82c16b7b, SUC_lexicon_tags_37 },
    { 0x21a16b7b, SUC_lexicon_tags_2 },
    { 0x1b416b7c, SUC_lexicon_tags_17 },
    { 0xb6616b7c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50216b83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d616b87, SUC_lexicon_tags_1 },
    { 0x1ec16b87, SUC_lexicon_tags_26 },
    { 0xf5416b89, SUC_lexicon_tags_61 },
    { 0x21216b8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc16b94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64416b9b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x50016b9e, SUC_lexicon_tags_29 },
    { 0xc216b9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5016ba2, SUC_lexicon_tags_27 },
    { 0x64616ba2, SUC_lexicon_tags_15 },
    { 0xda016ba4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc016bab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4216baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa616bb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a16bb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d416bbb, SUC_lexicon_tags_17 },
    { 0xea16bbc, SUC_lexicon_tags_33 },
    { 0x3f616bbd, SUC_lexicon_tags_12 },
    { 0xa416bbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e16bc0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62016bc4, SUC_lexicon_tags_45 },
    { 0xc4816bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4a16bc7, SUC_lexicon_tags_17 },
    { 0x54a16bc8, SUC_lexicon_tags_2 },
    { 0xae216bc7, SUC_lexicon_tags_236 },
    { 0x8da16bca, SUC_lexicon_tags_13 },
    { 0xfbe16bca, SUC_lexicon_tags_12 },
    { 0x8216bcc, SUC_lexicon_tags_234 },
    { 0x3b016bcc, SUC_lexicon_tags_14 },
    { 0x64a16bce, SUC_lexicon_tags_23 },
    { 0x6ae16bce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7616bd2, SUC_lexicon_tags_37 },
    { 0x6c616bd2, SUC_lexicon_tags_7 },
    { 0xba016bd2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26816bd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7616bda, SUC_lexicon_tags_12 },
    { 0xa0a16bdb, SUC_lexicon_tags_33 },
    { 0x55216bda, SUC_lexicon_tags_17 },
    { 0xa4416bdd, SUC_lexicon_tags_7 },
    { 0x84216bde, SUC_lexicon_tags_17 },
    { 0xfd816bdb, SUC_lexicon_tags_140 },
    { 0xc6416bdb, SUC_lexicon_tags_45 },
    { 0xb2216bdb, SUC_lexicon_tags_26 },
    { 0x43216be2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade16be7, SUC_lexicon_tags_11 },
    { 0xd2216be7, SUC_lexicon_tags_4 },
    { 0xdfc16be9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63216beb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc16bf4, SUC_lexicon_tags_7 },
    { 0x5a616bf5, SUC_lexicon_tags_23 },
    { 0xfa816bf4, SUC_lexicon_tags_12 },
    { 0x58016bf7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xce16bfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e16c02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0a16c04, SUC_lexicon_tags_23 },
    { 0xfb416c05, SUC_lexicon_tags_17 },
    { 0x12a16c06, SUC_lexicon_tags_66 },
    { 0x3b216c06, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x9b016c09, SUC_lexicon_tags_7 },
    { 0xa3416c09, SUC_lexicon_tags_5 },
    { 0x2c816c09, SUC_lexicon_tags_1 },
    { 0x82416c0c, SUC_lexicon_tags_23 },
    { 0xa8216c0d, SUC_lexicon_tags_35 },
    { 0x41c16c0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa16c11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec216c16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72816c1e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x29a16c20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3c016c22, SUC_lexicon_tags_79 },
    { 0x4ea16c23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d616c27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed216c2a, SUC_lexicon_tags_2 },
    { 0xdc816c2b, SUC_lexicon_tags_2 },
    { 0xfca16c2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4616c35, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60616c3e, SUC_lexicon_tags_30 },
    { 0xf0216c3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6816c42, SUC_lexicon_tags_47 },
    { 0x15616c42, SUC_lexicon_tags_25 },
    { 0xd7416c42, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc0016c46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5216c48, SUC_lexicon_tags_16 },
    { 0xb4216c49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8416c4c, SUC_lexicon_tags_47 },
    { 0x7f416c4d, SUC_lexicon_tags_45 },
    { 0xce016c4d, SUC_lexicon_tags_8 },
    { 0x8d416c4d, SUC_lexicon_tags_108 },
    { 0x3c816c50, SUC_lexicon_tags_17 },
    { 0x7c816c51, SUC_lexicon_tags_8 },
    { 0x16c4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4816c54, SUC_lexicon_tags_17 },
    { 0x34616c55, SUC_lexicon_tags_7 },
    { 0xfd016c55, SUC_lexicon_tags_47 },
    { 0x2bc16c56, SUC_lexicon_tags_46 },
    { 0x1a616c55, SUC_lexicon_tags_9 },
    { 0x2fc16c59, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46816c65, SUC_lexicon_tags_38 },
    { 0xc8416c65, SUC_lexicon_tags_55 },
    { 0x5ee16c67, SUC_lexicon_tags_1 },
    { 0x32816c65, SUC_lexicon_tags_19 },
    { 0xd4616c66, SUC_lexicon_tags_37 },
    { 0x67616c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e016c6c, SUC_lexicon_tags_31 },
    { 0x5de16c6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x33816c6f, SUC_lexicon_tags_11 },
    { 0xe5816c70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2a16c72, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x60616c75, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf0216c77, SUC_lexicon_tags_28 },
    { 0x50016c78, SUC_lexicon_tags_87 },
    { 0xbb816c79, SUC_lexicon_tags_27 },
    { 0x10e16c79, SUC_lexicon_tags_25 },
    { 0x86a16c7a, SUC_lexicon_tags_17 },
    { 0x85616c7a, SUC_lexicon_tags_23 },
    { 0x1fe16c7d, SUC_lexicon_tags_11 },
    { 0xa6a16c7e, SUC_lexicon_tags_29 },
    { 0x4b216c7f, SUC_lexicon_tags_39 },
    { 0x4c616c7e, SUC_lexicon_tags_25 },
    { 0x98016c80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7216c86, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5016c8a, SUC_lexicon_tags_23 },
    { 0x81e16c8b, SUC_lexicon_tags_12 },
    { 0xd3416c8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5e16c8e, SUC_lexicon_tags_29 },
    { 0x54a16c8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14016c99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d616ca4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c16ca8, SUC_lexicon_tags_9 },
    { 0x4be16ca8, SUC_lexicon_tags_19 },
    { 0x93616ca8, SUC_lexicon_tags_1 },
    { 0x70016cab, SUC_lexicon_tags_87 },
    { 0x7a016cab, SUC_lexicon_tags_45 },
    { 0x3c816cad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbd616caf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81816cb3, SUC_lexicon_tags_11 },
    { 0xf5816cb4, SUC_lexicon_tags_7 },
    { 0xe9016cb4, SUC_lexicon_tags_43 },
    { 0xe6216cb5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e16cba, SUC_lexicon_tags_2 },
    { 0xf1816cbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c16cbe, SUC_lexicon_tags_38 },
    { 0x1c816cbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37216cc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e16cc5, SUC_lexicon_tags_293 },
    { 0x91616cc6, SUC_lexicon_tags_316 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f616cc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2616ccc, SUC_lexicon_tags_7 },
    { 0x76816ccd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da16cd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8de16cd2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17016cd8, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be16cdb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c16ce1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8016ce4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee16ce9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89c16ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0016ced, SUC_lexicon_tags_2 },
    { 0x45016cee, SUC_lexicon_tags_57 },
    { 0xe6216cef, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c16cf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d416cf8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa9a16cfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84e16cfc, SUC_lexicon_tags_32 },
    { 0x22816cfd, SUC_lexicon_tags_7 },
    { 0x99816cfe, SUC_lexicon_tags_19 },
    { 0x53616cfd, SUC_lexicon_tags_40 },
    { 0x1da16d00, SUC_lexicon_tags_25 },
    { 0xedc16d01, SUC_lexicon_tags_70 },
    { 0xa3a16d02, SUC_lexicon_tags_84 },
    { 0xc9a16d03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84a16d05, SUC_lexicon_tags_25 },
    { 0x93e16d06, SUC_lexicon_tags_37 },
    { 0xf3e16d05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xedc16d09, SUC_lexicon_tags_37 },
    { 0x15c16d0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x37616d0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5016d11, SUC_lexicon_tags_7 },
    { 0x80616d12, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x89216d14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d416d1a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x37216d1c, SUC_lexicon_tags_11 },
    { 0xfa816d1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69216d22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64416d24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a216d27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44816d2b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f816d2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66016d31, SUC_lexicon_tags_11 },
    { 0x55416d32, SUC_lexicon_tags_12 },
    { 0x8ea16d33, SUC_lexicon_tags_11 },
    { 0x79016d32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x11216d37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54416d3c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba16d40, SUC_lexicon_tags_19 },
    { 0xfa216d40, SUC_lexicon_tags_10 },
    { 0xedc16d42, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89816d55, SUC_lexicon_tags_27 },
    { 0xb3216d55, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa16d5a, SUC_lexicon_tags_12 },
    { 0xff616d5a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc216d5e, SUC_lexicon_tags_25 },
    { 0xe6416d5f, SUC_lexicon_tags_25 },
    { 0x6b416d60, SUC_lexicon_tags_43 },
    { 0x1e016d61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b416d64, SUC_lexicon_tags_65 },
    { 0xf1e16d64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28a16d67, SUC_lexicon_tags_45 },
    { 0x78a16d67, SUC_lexicon_tags_27 },
    { 0x44216d68, SUC_lexicon_tags_1 },
    { 0xa8c16d6a, SUC_lexicon_tags_15 },
    { 0xc5616d6a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x31016d6d, SUC_lexicon_tags_38 },
    { 0x8a016d6d, SUC_lexicon_tags_37 },
    { 0x1aa16d6f, SUC_lexicon_tags_15 },
    { 0xd616d6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d016d73, SUC_lexicon_tags_23 },
    { 0xf1a16d74, SUC_lexicon_tags_17 },
    { 0x8f216d74, SUC_lexicon_tags_47 },
    { 0xb416d76, SUC_lexicon_tags_56 },
    { 0xf4c16d77, SUC_lexicon_tags_25 },
    { 0xeaa16d78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf216d7e, SUC_lexicon_tags_11 },
    { 0xa4c16d7f, SUC_lexicon_tags_12 },
    { 0x9e16d7e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xfda16d82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x16816d85, SUC_lexicon_tags_63 },
    { 0x72c16d85, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a16d89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c16d8d, SUC_lexicon_tags_2 },
    { 0xc3016d8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47816d95, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c16d9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80616d9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e16da5, SUC_lexicon_tags_1460 },
    { 0, NULL },
    { 0, NULL },
    { 0x40216da8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdd016daa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4016dae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x31a16db0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7616db2, SUC_lexicon_tags_31 },
    { 0xf016db3, SUC_lexicon_tags_1 },
    { 0x8616db4, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1216db9, SUC_lexicon_tags_7 },
    { 0xdae16dba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa616dbd, SUC_lexicon_tags_65 },
    { 0xd416dbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c16dc2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1816dc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3816dc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1816dcb, SUC_lexicon_tags_11 },
    { 0x5de16dcc, SUC_lexicon_tags_224 },
    { 0x66e16dcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a16dd1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd816dd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc16ddb, SUC_lexicon_tags_8 },
    { 0x2fe16ddb, SUC_lexicon_tags_79 },
    { 0x48a16ddd, SUC_lexicon_tags_2 },
    { 0x73416ddd, SUC_lexicon_tags_39 },
    { 0xf0216ddc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x81816de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41216de4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa0616de6, SUC_lexicon_tags_1 },
    { 0x65a16de7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9616dea, SUC_lexicon_tags_0 },
    { 0x5c416dea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10e16ded, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b616df0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89416df2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a216df8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f416dfb, SUC_lexicon_tags_17 },
    { 0x7d616dfb, SUC_lexicon_tags_19 },
    { 0xa7216dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe816e00, SUC_lexicon_tags_87 },
    { 0xac616e00, SUC_lexicon_tags_49 },
    { 0xf616e02, SUC_lexicon_tags_15 },
    { 0x18a16e03, SUC_lexicon_tags_37 },
    { 0xd2816e01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5d416e06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa416e08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4816e0c, SUC_lexicon_tags_15 },
    { 0xae016e0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x41816e10, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3d416e12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc16e19, SUC_lexicon_tags_297 },
    { 0xdf616e19, SUC_lexicon_tags_29 },
    { 0x82e16e1b, SUC_lexicon_tags_7 },
    { 0x7e16e19, SUC_lexicon_tags_43 },
    { 0x4e16e1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff616e1f, SUC_lexicon_tags_39 },
    { 0xad816e20, SUC_lexicon_tags_33 },
    { 0x29816e21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8216e26, SUC_lexicon_tags_29 },
    { 0xdfc16e26, SUC_lexicon_tags_23 },
    { 0x2816e28, SUC_lexicon_tags_14 },
    { 0xa3c16e28, SUC_lexicon_tags_60 },
    { 0xe5a16e29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc16e2e, SUC_lexicon_tags_11 },
    { 0xc7e16e2f, SUC_lexicon_tags_37 },
    { 0x39a16e30, SUC_lexicon_tags_17 },
    { 0xa4616e31, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe3216e33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf8216e35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde16e39, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5816e3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e616e40, SUC_lexicon_tags_8 },
    { 0xa6616e41, SUC_lexicon_tags_12 },
    { 0x14416e42, SUC_lexicon_tags_3 },
    { 0x51416e43, SUC_lexicon_tags_45 },
    { 0xcba16e43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4016e48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6416e4c, SUC_lexicon_tags_12 },
    { 0x80216e4c, SUC_lexicon_tags_8 },
    { 0xd9c16e4e, SUC_lexicon_tags_15 },
    { 0x2cc16e4f, SUC_lexicon_tags_7 },
    { 0x2da16e50, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6ae16e52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbc216e54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b616e57, SUC_lexicon_tags_15 },
    { 0xe816e58, SUC_lexicon_tags_32 },
    { 0x1216e58, SUC_lexicon_tags_61 },
    { 0x55216e5a, SUC_lexicon_tags_39 },
    { 0xf8616e5a, SUC_lexicon_tags_15 },
    { 0x4b016e5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65016e61, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b016e6b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d616e6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7416e72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa016e76, SUC_lexicon_tags_98 },
    { 0x5ea16e77, SUC_lexicon_tags_7 },
    { 0x19416e78, SUC_lexicon_tags_82 },
    { 0x50a16e79, SUC_lexicon_tags_47 },
    { 0xb816e7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c16e7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f816e84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb816e87, SUC_lexicon_tags_65 },
    { 0x82a16e87, SUC_lexicon_tags_3 },
    { 0xd8e16e87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c16e90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8416e94, SUC_lexicon_tags_32 },
    { 0xf5e16e95, SUC_lexicon_tags_21 },
    { 0x69216e94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7616e98, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99c16e9a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2d616e9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c16e9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7016ea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8816ea5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a16ea8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c216eab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d216eb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f816eb7, SUC_lexicon_tags_37 },
    { 0xd4616eb8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c16ebb, SUC_lexicon_tags_17 },
    { 0xf1a16ebb, SUC_lexicon_tags_82 },
    { 0x4b616ebd, SUC_lexicon_tags_8 },
    { 0xd4c16ebe, SUC_lexicon_tags_84 },
    { 0xc2a16ebf, SUC_lexicon_tags_37 },
    { 0x7b016ebf, SUC_lexicon_tags_19 },
    { 0x8be16ec1, SUC_lexicon_tags_19 },
    { 0x8e816ec0, SUC_lexicon_tags_43 },
    { 0xd7e16ebe, SUC_lexicon_tags_17 },
    { 0xd6a16ec4, SUC_lexicon_tags_17 },
    { 0xd7c16ec4, SUC_lexicon_tags_55 },
    { 0xc2816ec2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49e16ec8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e16ecb, SUC_lexicon_tags_6 },
    { 0x1c416ecc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76816ecf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c16ed2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e16eda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e16edd, SUC_lexicon_tags_18 },
    { 0xad416ede, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27c16ee0, SUC_lexicon_tags_39 },
    { 0x5b216ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39c16ee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0616ee5, SUC_lexicon_tags_11 },
    { 0xe6616ee6, SUC_lexicon_tags_29 },
    { 0x11a16ee7, SUC_lexicon_tags_1 },
    { 0x4a016ee7, SUC_lexicon_tags_25 },
    { 0x64816ee8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c616ef0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf2a16ef2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58416ef5, SUC_lexicon_tags_14 },
    { 0x3816ef5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7416f02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9616f06, SUC_lexicon_tags_15 },
    { 0xfdc16f07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84616f0e, SUC_lexicon_tags_10 },
    { 0x9a16f0e, SUC_lexicon_tags_26 },
    { 0x19a16f10, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa5c16f12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f816f16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e16f1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57a16f20, SUC_lexicon_tags_27 },
    { 0xac816f21, SUC_lexicon_tags_17 },
    { 0xfe616f22, SUC_lexicon_tags_42 },
    { 0xcfc16f23, SUC_lexicon_tags_14 },
    { 0x9a416f24, SUC_lexicon_tags_7 },
    { 0xe3c16f21, SUC_lexicon_tags_31 },
    { 0x2fe16f26, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xec016f28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x85016f2a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20016f30, SUC_lexicon_tags_2 },
    { 0x4a616f31, SUC_lexicon_tags_1 },
    { 0xd6816f32, SUC_lexicon_tags_37 },
    { 0xafc16f32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36216f35, SUC_lexicon_tags_182 },
    { 0xf7c16f36, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a16f39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58a16f3b, SUC_lexicon_tags_25 },
    { 0x7ba16f3b, SUC_lexicon_tags_2 },
    { 0xaf216f3c, SUC_lexicon_tags_0 },
    { 0x3c616f3e, SUC_lexicon_tags_19 },
    { 0x6e616f3d, SUC_lexicon_tags_0 },
    { 0x6fa16f3c, SUC_lexicon_tags_12 },
    { 0x91016f41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa4a16f43, SUC_lexicon_tags_11 },
    { 0x3e216f44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf416f49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2da16f4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e16f4e, SUC_lexicon_tags_49 },
    { 0xbfe16f4f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb7416f51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12416f59, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x27e16f5b, SUC_lexicon_tags_15 },
    { 0x19c16f5c, SUC_lexicon_tags_0 },
    { 0x9a816f5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9416f61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c416f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46616f67, SUC_lexicon_tags_43 },
    { 0x2de16f68, SUC_lexicon_tags_44 },
    { 0x41216f69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86616f6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce616f71, SUC_lexicon_tags_17 },
    { 0x97e16f71, SUC_lexicon_tags_6 },
    { 0xce416f73, SUC_lexicon_tags_11 },
    { 0x19e16f74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0616f77, SUC_lexicon_tags_12 },
    { 0x87216f78, SUC_lexicon_tags_11 },
    { 0x12016f78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc16f7d, SUC_lexicon_tags_7 },
    { 0xde216f7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd616f82, SUC_lexicon_tags_43 },
    { 0x35416f82, SUC_lexicon_tags_2 },
    { 0xd9216f83, SUC_lexicon_tags_8 },
    { 0xf216f85, SUC_lexicon_tags_29 },
    { 0x11216f86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed616f88, SUC_lexicon_tags_6 },
    { 0xe4616f88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92216f8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74016f97, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75216f9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18216f9d, SUC_lexicon_tags_11 },
    { 0xb0216f9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2616fa3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a16fa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca16fad, SUC_lexicon_tags_0 },
    { 0x3b416fae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c216fb6, SUC_lexicon_tags_19 },
    { 0x69616fb7, SUC_lexicon_tags_39 },
    { 0x67a16fb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b216fbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2216fbe, SUC_lexicon_tags_11 },
    { 0x52016fbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d416fca, SUC_lexicon_tags_47 },
    { 0x83e16fcb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4816fcf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb4216fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26416fd8, SUC_lexicon_tags_68 },
    { 0x3c016fd8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2aa16fdb, SUC_lexicon_tags_23 },
    { 0xdf216fdc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30216fe1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3616fe4, SUC_lexicon_tags_7 },
    { 0x83616fe5, SUC_lexicon_tags_17 },
    { 0xbdc16fe4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c216fe8, SUC_lexicon_tags_68 },
    { 0x79416fe8, SUC_lexicon_tags_11 },
    { 0x16416fe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c16ff0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83e16ff2, SUC_lexicon_tags_2 },
    { 0x6b216ff3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38816ff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e16ffb, SUC_lexicon_tags_14 },
    { 0xb5016ffb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb3016ffe, SUC_lexicon_tags_19 },
    { 0x96c16ffe, SUC_lexicon_tags_11 },
    { 0x13c17000, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6217005, SUC_lexicon_tags_11 },
    { 0xeba17005, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca81700b, SUC_lexicon_tags_9 },
    { 0x59e1700c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c1700f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e217012, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe217014, SUC_lexicon_tags_16 },
    { 0xb2817014, SUC_lexicon_tags_37 },
    { 0x4817016, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe6217018, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f21701b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d017020, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x12817023, SUC_lexicon_tags_15 },
    { 0x5c17024, SUC_lexicon_tags_201 },
    { 0xcce17024, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99417028, SUC_lexicon_tags_11 },
    { 0x88817029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c21702c, SUC_lexicon_tags_542 },
    { 0x4d81702c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e17031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccc17033, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5b617035, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a17039, SUC_lexicon_tags_15 },
    { 0x9a61703a, SUC_lexicon_tags_30 },
    { 0xfa41703a, SUC_lexicon_tags_87 },
    { 0x7f21703b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7a01703e, SUC_lexicon_tags_25 },
    { 0x88a1703f, SUC_lexicon_tags_62 },
    { 0x45217040, SUC_lexicon_tags_32 },
    { 0xa9417040, SUC_lexicon_tags_15 },
    { 0x7f617042, SUC_lexicon_tags_17 },
    { 0x2b617043, SUC_lexicon_tags_25 },
    { 0x80217043, SUC_lexicon_tags_12 },
    { 0x20617045, SUC_lexicon_tags_3 },
    { 0x7941703e, SUC_lexicon_tags_12 },
    { 0x9da1703f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42417051, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61817054, SUC_lexicon_tags_37 },
    { 0x7ee17055, SUC_lexicon_tags_28 },
    { 0xe8e17054, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5617059, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a81705c, SUC_lexicon_tags_8 },
    { 0xf041705d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xea017060, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x38817063, SUC_lexicon_tags_7 },
    { 0xf5617064, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e1707c, SUC_lexicon_tags_43 },
    { 0x8461707d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c017084, SUC_lexicon_tags_61 },
    { 0x54417084, SUC_lexicon_tags_17 },
    { 0x10617084, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd921708c, SUC_lexicon_tags_27 },
    { 0xb961708c, SUC_lexicon_tags_25 },
    { 0x26c1708e, SUC_lexicon_tags_25 },
    { 0xbd81708f, SUC_lexicon_tags_9 },
    { 0xf561708c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c017095, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a1709a, SUC_lexicon_tags_39 },
    { 0x49a1709b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefe1709d, SUC_lexicon_tags_93 },
    { 0x9a81709e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x928170a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x976170a3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c170a9, SUC_lexicon_tags_1 },
    { 0xdac170aa, SUC_lexicon_tags_14 },
    { 0xc20170ab, SUC_lexicon_tags_23 },
    { 0x292170ac, SUC_lexicon_tags_7 },
    { 0xe30170ac, SUC_lexicon_tags_9 },
    { 0xf04170ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92e170b0, SUC_lexicon_tags_4 },
    { 0xb2e170b1, SUC_lexicon_tags_17 },
    { 0xf44170b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1da170b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2c170b6, SUC_lexicon_tags_12 },
    { 0x1c6170b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772170bb, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xce8170bd, SUC_lexicon_tags_32 },
    { 0xa50170bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8fa170c0, SUC_lexicon_tags_72 },
    { 0xad0170c1, SUC_lexicon_tags_11 },
    { 0x7fc170c1, SUC_lexicon_tags_35 },
    { 0xb90170c3, SUC_lexicon_tags_61 },
    { 0x14c170c2, SUC_lexicon_tags_77 },
    { 0xd74170c1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd94170c7, SUC_lexicon_tags_38 },
    { 0x14e170c8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774170cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x618170cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e170d4, SUC_lexicon_tags_38 },
    { 0x8b4170d5, SUC_lexicon_tags_13 },
    { 0xf2c170d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834170da, SUC_lexicon_tags_47 },
    { 0x4a4170da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98170e0, SUC_lexicon_tags_28 },
    { 0x140170e1, SUC_lexicon_tags_11 },
    { 0x752170e1, SUC_lexicon_tags_37 },
    { 0xefa170e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca170e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x114170e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x728170eb, SUC_lexicon_tags_37 },
    { 0x99e170ec, SUC_lexicon_tags_25 },
    { 0x714170ec, SUC_lexicon_tags_12 },
    { 0xb64170eb, SUC_lexicon_tags_28 },
    { 0xfdc170ef, SUC_lexicon_tags_28 },
    { 0x90e170f0, SUC_lexicon_tags_17 },
    { 0xf2a170f1, SUC_lexicon_tags_14 },
    { 0xb96170f2, SUC_lexicon_tags_14 },
    { 0x120170f3, SUC_lexicon_tags_23 },
    { 0x2a8170f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2170fb, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x714170fd, SUC_lexicon_tags_17 },
    { 0xda170fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x68417100, SUC_lexicon_tags_1 },
    { 0x9fc17100, SUC_lexicon_tags_60 },
    { 0xc6a17101, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee17106, SUC_lexicon_tags_12 },
    { 0xcd817107, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x15a17109, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe741710b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78e1710d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2a1710f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd217111, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0017114, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x72e17116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad81711a, SUC_lexicon_tags_12 },
    { 0x9001711b, SUC_lexicon_tags_12 },
    { 0xbe01711b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7261711e, SUC_lexicon_tags_1 },
    { 0x9f01711f, SUC_lexicon_tags_33 },
    { 0xbf017120, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8f617122, SUC_lexicon_tags_17 },
    { 0x63e17123, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece17127, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1712a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d01712f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7a17131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70017133, SUC_lexicon_tags_23 },
    { 0x84617134, SUC_lexicon_tags_1 },
    { 0xc8617135, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x65817138, SUC_lexicon_tags_25 },
    { 0x9c217138, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa3c1713b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc1713e, SUC_lexicon_tags_7 },
    { 0x7301713e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e17143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be1714d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e17151, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e17153, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x67017155, SUC_lexicon_tags_25 },
    { 0x7e017156, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3821715d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xac017160, SUC_lexicon_tags_9 },
    { 0x54e17161, SUC_lexicon_tags_55 },
    { 0x5ae17162, SUC_lexicon_tags_48 },
    { 0xf4417162, SUC_lexicon_tags_103 },
    { 0x83817162, SUC_lexicon_tags_15 },
    { 0x24a17162, SUC_lexicon_tags_19 },
    { 0x43817160, SUC_lexicon_tags_63 },
    { 0x70e17167, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1541716a, SUC_lexicon_tags_607 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c81716e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e17171, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x97217173, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30017178, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e01717b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7017180, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1217184, SUC_lexicon_tags_11 },
    { 0x61817185, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56617189, SUC_lexicon_tags_7 },
    { 0x3641718a, SUC_lexicon_tags_12 },
    { 0x701718a, SUC_lexicon_tags_60 },
    { 0xbe21718c, SUC_lexicon_tags_28 },
    { 0x8a01718d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf217190, SUC_lexicon_tags_23 },
    { 0x53c17191, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc17193, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2817195, SUC_lexicon_tags_14 },
    { 0xb9e17195, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa9e17198, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d81719f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x214171a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff2171a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec171a6, SUC_lexicon_tags_15 },
    { 0x92171a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb82171a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aa171ab, SUC_lexicon_tags_17 },
    { 0x7d8171ac, SUC_lexicon_tags_11 },
    { 0xe66171ad, SUC_lexicon_tags_7 },
    { 0xf3e171ae, SUC_lexicon_tags_7 },
    { 0x300171ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68171b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6171c0, SUC_lexicon_tags_1 },
    { 0xfd8171c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68171c6, SUC_lexicon_tags_0 },
    { 0x996171c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc171cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa04171ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x792171d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a171d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x14171da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb78171dc, SUC_lexicon_tags_15 },
    { 0x4de171dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8171e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x410171e3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518171e9, SUC_lexicon_tags_68 },
    { 0x762171e9, SUC_lexicon_tags_1 },
    { 0xe1a171eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa76171ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46171fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea17200, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe41720a, SUC_lexicon_tags_12 },
    { 0x9c81720a, SUC_lexicon_tags_2 },
    { 0xe81720c, SUC_lexicon_tags_27 },
    { 0x4a41720c, SUC_lexicon_tags_11 },
    { 0xcde1720a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b017212, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaf217214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e17217, SUC_lexicon_tags_62 },
    { 0x2a817218, SUC_lexicon_tags_27 },
    { 0xb5417218, SUC_lexicon_tags_27 },
    { 0x12417219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1181721f, SUC_lexicon_tags_12 },
    { 0x4a17220, SUC_lexicon_tags_35 },
    { 0xe161721f, SUC_lexicon_tags_15 },
    { 0x5a01721f, SUC_lexicon_tags_17 },
    { 0xd0a17220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27817227, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe6617229, SUC_lexicon_tags_27 },
    { 0x4fe17229, SUC_lexicon_tags_1 },
    { 0x7d81722a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f81722e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d017230, SUC_lexicon_tags_7 },
    { 0xba217231, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79417237, SUC_lexicon_tags_26 },
    { 0x78017238, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a41723c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6041723e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c617241, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7de17243, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ec17245, SUC_lexicon_tags_37 },
    { 0x86217246, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7841724b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5401724e, SUC_lexicon_tags_37 },
    { 0x1861724f, SUC_lexicon_tags_148 },
    { 0xe041724f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d217255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3841725a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e17261, SUC_lexicon_tags_2 },
    { 0x7c017261, SUC_lexicon_tags_12 },
    { 0x6e17263, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xcea17265, SUC_lexicon_tags_9 },
    { 0x46817265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c1726b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe881726e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcd017270, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0017272, SUC_lexicon_tags_2 },
    { 0x9217272, SUC_lexicon_tags_25 },
    { 0x38a17272, SUC_lexicon_tags_6 },
    { 0x38417273, SUC_lexicon_tags_47 },
    { 0xdbc17276, SUC_lexicon_tags_2 },
    { 0x1bc17277, SUC_lexicon_tags_11 },
    { 0x35c17274, SUC_lexicon_tags_37 },
    { 0x87217276, SUC_lexicon_tags_56 },
    { 0xfae17272, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f81727d, SUC_lexicon_tags_17 },
    { 0x9521727d, SUC_lexicon_tags_14 },
    { 0xe201727f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a617282, SUC_lexicon_tags_12 },
    { 0x5e617283, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f017288, SUC_lexicon_tags_31 },
    { 0x36017288, SUC_lexicon_tags_1 },
    { 0xb621728a, SUC_lexicon_tags_8 },
    { 0x49c1728b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8fe1728d, SUC_lexicon_tags_30 },
    { 0x7841728d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf617290, SUC_lexicon_tags_37 },
    { 0xce417291, SUC_lexicon_tags_2 },
    { 0x5617292, SUC_lexicon_tags_37 },
    { 0x35417291, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c1729b, SUC_lexicon_tags_47 },
    { 0x1b61729c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x67e1729e, SUC_lexicon_tags_96 },
    { 0xafc1729e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836172a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8172a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6172ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x254172ae, SUC_lexicon_tags_49 },
    { 0xffe172af, SUC_lexicon_tags_7 },
    { 0x946172b0, SUC_lexicon_tags_25 },
    { 0x57c172b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72172b5, SUC_lexicon_tags_27 },
    { 0xb16172b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6e2172b8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb1e172ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x466172bd, SUC_lexicon_tags_687 },
    { 0x500172be, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x978172c1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf7c172c3, SUC_lexicon_tags_47 },
    { 0x680172c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf5a172c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586172cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230172cf, SUC_lexicon_tags_27 },
    { 0xee6172cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd0172d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa28172d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236172d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe172df, SUC_lexicon_tags_12 },
    { 0xd2e172df, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x31c172e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba172e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa62172e7, SUC_lexicon_tags_55 },
    { 0x43c172e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf5c172ea, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x536172ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x394172ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78c172f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846172fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc172fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e17302, SUC_lexicon_tags_25 },
    { 0xb4a17302, SUC_lexicon_tags_0 },
    { 0xbf217304, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4217307, SUC_lexicon_tags_12 },
    { 0x37a17308, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb521730a, SUC_lexicon_tags_14 },
    { 0xfac1730a, SUC_lexicon_tags_7 },
    { 0x3c01730a, SUC_lexicon_tags_11 },
    { 0xa681730d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbd81730f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca17313, SUC_lexicon_tags_60 },
    { 0x39617314, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a17317, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe1731c, SUC_lexicon_tags_19 },
    { 0xf641731d, SUC_lexicon_tags_148 },
    { 0xcb61731e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6f217320, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b217323, SUC_lexicon_tags_37 },
    { 0xddc17324, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57017328, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe921732b, SUC_lexicon_tags_1 },
    { 0x4fe1732c, SUC_lexicon_tags_38 },
    { 0xc621732c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85817331, SUC_lexicon_tags_11 },
    { 0xdb617331, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb417337, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xafe17339, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xce01733b, SUC_lexicon_tags_47 },
    { 0xac21733c, SUC_lexicon_tags_65 },
    { 0x4461733d, SUC_lexicon_tags_32 },
    { 0xe861733d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49817342, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca17348, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6081734d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3217357, SUC_lexicon_tags_124 },
    { 0x6be17357, SUC_lexicon_tags_65 },
    { 0xb5617358, SUC_lexicon_tags_53 },
    { 0x39017358, SUC_lexicon_tags_11 },
    { 0x12217359, SUC_lexicon_tags_22 },
    { 0xb61735c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c81735e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22e17360, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae17365, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75817367, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5417369, SUC_lexicon_tags_624 },
    { 0x32a1736a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b01736d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91617370, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc761737a, SUC_lexicon_tags_19 },
    { 0x4b01737a, SUC_lexicon_tags_230 },
    { 0x3841737c, SUC_lexicon_tags_29 },
    { 0x8b21737c, SUC_lexicon_tags_12 },
    { 0x6a01737e, SUC_lexicon_tags_13 },
    { 0xc041737e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5a17381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a17384, SUC_lexicon_tags_49 },
    { 0xd3417385, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa0617387, SUC_lexicon_tags_29 },
    { 0x36217387, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa361738a, SUC_lexicon_tags_22 },
    { 0xbec1738b, SUC_lexicon_tags_11 },
    { 0xb761738a, SUC_lexicon_tags_47 },
    { 0x461738d, SUC_lexicon_tags_59 },
    { 0x3281738e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7fe17390, SUC_lexicon_tags_49 },
    { 0xd6417390, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1217393, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4417396, SUC_lexicon_tags_12 },
    { 0x4c617397, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80e17399, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca1739f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0173a6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876173ab, SUC_lexicon_tags_23 },
    { 0x7ce173ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990173b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4173b9, SUC_lexicon_tags_12 },
    { 0x296173ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73c173bc, SUC_lexicon_tags_14 },
    { 0xd90173bd, SUC_lexicon_tags_14 },
    { 0xc02173be, SUC_lexicon_tags_5 },
    { 0x7e8173bf, SUC_lexicon_tags_9 },
    { 0xd8c173c0, SUC_lexicon_tags_11 },
    { 0x180173bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8173c5, SUC_lexicon_tags_9 },
    { 0x694173c6, SUC_lexicon_tags_37 },
    { 0x55a173c7, SUC_lexicon_tags_17 },
    { 0x1c0173c7, SUC_lexicon_tags_11 },
    { 0x39c173c8, SUC_lexicon_tags_43 },
    { 0xb52173c9, SUC_lexicon_tags_2 },
    { 0x3c8173c7, SUC_lexicon_tags_25 },
    { 0xe84173cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec173d2, SUC_lexicon_tags_15 },
    { 0x8c2173d3, SUC_lexicon_tags_12 },
    { 0x7ce173d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966173dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4173e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268173e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c4173e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e173eb, SUC_lexicon_tags_7 },
    { 0x2ba173ec, SUC_lexicon_tags_17 },
    { 0x46173ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa00173ef, SUC_lexicon_tags_87 },
    { 0x9a4173f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4c173f2, SUC_lexicon_tags_15 },
    { 0x9d2173f2, SUC_lexicon_tags_60 },
    { 0x920173f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2173fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa2173fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5e173fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa17401, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf617404, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc9217406, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x617408, SUC_lexicon_tags_17 },
    { 0x70c17409, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1001740f, SUC_lexicon_tags_7 },
    { 0xa1217410, SUC_lexicon_tags_25 },
    { 0xc0817410, SUC_lexicon_tags_38 },
    { 0x8c017412, SUC_lexicon_tags_12 },
    { 0x72617413, SUC_lexicon_tags_7 },
    { 0xdcc17414, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a1741f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31017425, SUC_lexicon_tags_14 },
    { 0x46417426, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a17433, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ce17435, SUC_lexicon_tags_37 },
    { 0xe9a17436, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa321743a, SUC_lexicon_tags_644 },
    { 0x81743b, SUC_lexicon_tags_4 },
    { 0xa5e1743c, SUC_lexicon_tags_17 },
    { 0xf6a1743d, SUC_lexicon_tags_13 },
    { 0x6061743c, SUC_lexicon_tags_25 },
    { 0xd621743f, SUC_lexicon_tags_35 },
    { 0x13e1743f, SUC_lexicon_tags_47 },
    { 0x6901743e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a17447, SUC_lexicon_tags_37 },
    { 0x3ba17448, SUC_lexicon_tags_4 },
    { 0x95a17449, SUC_lexicon_tags_30 },
    { 0x22217447, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8041744c, SUC_lexicon_tags_12 },
    { 0xca1744d, SUC_lexicon_tags_28 },
    { 0xa201744e, SUC_lexicon_tags_11 },
    { 0x9f61744f, SUC_lexicon_tags_57 },
    { 0x2dc1744d, SUC_lexicon_tags_15 },
    { 0x76217450, SUC_lexicon_tags_11 },
    { 0x14617452, SUC_lexicon_tags_29 },
    { 0xfae17451, SUC_lexicon_tags_25 },
    { 0xb3a17452, SUC_lexicon_tags_43 },
    { 0xebe17455, SUC_lexicon_tags_11 },
    { 0xae17456, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5617459, SUC_lexicon_tags_37 },
    { 0x7c81745a, SUC_lexicon_tags_7 },
    { 0x47417459, SUC_lexicon_tags_30 },
    { 0xaa01745c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d01745f, SUC_lexicon_tags_1 },
    { 0x8bc17460, SUC_lexicon_tags_15 },
    { 0xe9a17461, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbe617463, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x37e17465, SUC_lexicon_tags_32 },
    { 0x53c17465, SUC_lexicon_tags_7 },
    { 0xf3c17467, SUC_lexicon_tags_13 },
    { 0x15a17468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc201746b, SUC_lexicon_tags_15 },
    { 0x9681746c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7001746e, SUC_lexicon_tags_29 },
    { 0xea61746f, SUC_lexicon_tags_37 },
    { 0xf9a1746f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc17473, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a17476, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62217478, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301747c, SUC_lexicon_tags_7 },
    { 0x9be1747d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44c1747f, SUC_lexicon_tags_7 },
    { 0xc0217480, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2417483, SUC_lexicon_tags_82 },
    { 0xbfa17484, SUC_lexicon_tags_0 },
    { 0xf9e17484, SUC_lexicon_tags_15 },
    { 0x3c617485, SUC_lexicon_tags_38 },
    { 0xb5817484, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5f017489, SUC_lexicon_tags_12 },
    { 0x61817489, SUC_lexicon_tags_0 },
    { 0x1f81748b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xca1748d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e17494, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xafe17496, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c1749b, SUC_lexicon_tags_15 },
    { 0x5421749c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbc01749e, SUC_lexicon_tags_5 },
    { 0x4a81749f, SUC_lexicon_tags_0 },
    { 0x4fe1749e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c174a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a174a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4174ab, SUC_lexicon_tags_33 },
    { 0x3b8174ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a8174ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2174b1, SUC_lexicon_tags_7 },
    { 0x464174b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96174b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x53e174bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x228174bd, SUC_lexicon_tags_26 },
    { 0xe06174be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e174c1, SUC_lexicon_tags_68 },
    { 0x4bc174c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530174c6, SUC_lexicon_tags_34 },
    { 0xf22174c7, SUC_lexicon_tags_27 },
    { 0x1d6174c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c174cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5de174cf, SUC_lexicon_tags_2 },
    { 0x914174d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x720174d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70174d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa174e9, SUC_lexicon_tags_32 },
    { 0x5ae174e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa174f1, SUC_lexicon_tags_56 },
    { 0x22a174f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2174f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce174fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8174fe, SUC_lexicon_tags_37 },
    { 0x3ca174fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a17505, SUC_lexicon_tags_17 },
    { 0xff817506, SUC_lexicon_tags_113 },
    { 0x6817507, SUC_lexicon_tags_12 },
    { 0x72817508, SUC_lexicon_tags_14 },
    { 0xb6217509, SUC_lexicon_tags_25 },
    { 0xda61750a, SUC_lexicon_tags_151 },
    { 0xfe217508, SUC_lexicon_tags_114 },
    { 0xf2e17507, SUC_lexicon_tags_30 },
    { 0xb6c1750c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac17510, SUC_lexicon_tags_11 },
    { 0x84e17511, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x16617513, SUC_lexicon_tags_21 },
    { 0xdbc17514, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed217518, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b21751e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5617525, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3617527, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc8c17529, SUC_lexicon_tags_17 },
    { 0x79c1752a, SUC_lexicon_tags_9 },
    { 0xd021752b, SUC_lexicon_tags_28 },
    { 0xf441752c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c61752f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de17532, SUC_lexicon_tags_1 },
    { 0xdaa17533, SUC_lexicon_tags_0 },
    { 0x4de17534, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f217536, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1b417538, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a1753b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85817540, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8217542, SUC_lexicon_tags_32 },
    { 0xbae17542, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3561754f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ae17551, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x36417553, SUC_lexicon_tags_16 },
    { 0xcd817554, SUC_lexicon_tags_17 },
    { 0xc2817555, SUC_lexicon_tags_15 },
    { 0x2dc17556, SUC_lexicon_tags_29 },
    { 0x45617557, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd61755b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa521755f, SUC_lexicon_tags_12 },
    { 0x86417560, SUC_lexicon_tags_47 },
    { 0xaa017561, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x69e17563, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb417566, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77617569, SUC_lexicon_tags_33 },
    { 0x6541756a, SUC_lexicon_tags_194 },
    { 0xad61756b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98817571, SUC_lexicon_tags_69 },
    { 0xcba17572, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x95817574, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a17579, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b81757b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4101757f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9017581, SUC_lexicon_tags_17 },
    { 0xb7017582, SUC_lexicon_tags_45 },
    { 0xeaa17583, SUC_lexicon_tags_14 },
    { 0xe8e17584, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62017587, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b617589, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8561758b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2481758d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7da1758f, SUC_lexicon_tags_39 },
    { 0x2da17590, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac17593, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1217596, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702175a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c175a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c175ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa32175ad, SUC_lexicon_tags_25 },
    { 0x86c175ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe8175b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x368175b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd76175b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2175b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510175bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a175bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32175c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0175c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10c175c8, SUC_lexicon_tags_25 },
    { 0x708175c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x358175cc, SUC_lexicon_tags_33 },
    { 0x760175cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c175d1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc48175d3, SUC_lexicon_tags_27 },
    { 0x702175d4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x132175d6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746175dd, SUC_lexicon_tags_37 },
    { 0xdca175de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28175e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4175ea, SUC_lexicon_tags_45 },
    { 0x7a2175eb, SUC_lexicon_tags_7 },
    { 0x88175ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c175f0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa46175f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0175f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7ae175fb, SUC_lexicon_tags_3 },
    { 0xb86175fc, SUC_lexicon_tags_47 },
    { 0xdae175fc, SUC_lexicon_tags_1 },
    { 0xa18175fd, SUC_lexicon_tags_1 },
    { 0x80175fe, SUC_lexicon_tags_17 },
    { 0x19a17600, SUC_lexicon_tags_95 },
    { 0xb18175fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fa17603, SUC_lexicon_tags_28 },
    { 0x6ae17603, SUC_lexicon_tags_21 },
    { 0x19a17605, SUC_lexicon_tags_8 },
    { 0x9e17606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c01760a, SUC_lexicon_tags_47 },
    { 0x4e61760b, SUC_lexicon_tags_65 },
    { 0xc941760c, SUC_lexicon_tags_57 },
    { 0xde41760c, SUC_lexicon_tags_48 },
    { 0xf41760c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37817611, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e17616, SUC_lexicon_tags_60 },
    { 0x8c217616, SUC_lexicon_tags_25 },
    { 0x4017616, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c41761a, SUC_lexicon_tags_17 },
    { 0x161761b, SUC_lexicon_tags_0 },
    { 0x5ac1761a, SUC_lexicon_tags_27 },
    { 0xe381761d, SUC_lexicon_tags_12 },
    { 0x5681761a, SUC_lexicon_tags_15 },
    { 0x8e41761f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x5ae17621, SUC_lexicon_tags_39 },
    { 0x7bc17622, SUC_lexicon_tags_43 },
    { 0xf3017623, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27617626, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x51017628, SUC_lexicon_tags_37 },
    { 0x4017629, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e1762f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac17639, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe1763c, SUC_lexicon_tags_39 },
    { 0x3e1763d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x52a1763f, SUC_lexicon_tags_25 },
    { 0x37e17640, SUC_lexicon_tags_37 },
    { 0x15017641, SUC_lexicon_tags_19 },
    { 0x57217641, SUC_lexicon_tags_15 },
    { 0xa7017643, SUC_lexicon_tags_45 },
    { 0x321763f, SUC_lexicon_tags_15 },
    { 0xdd417644, SUC_lexicon_tags_27 },
    { 0x2c617642, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8b617648, SUC_lexicon_tags_1 },
    { 0x16217648, SUC_lexicon_tags_13 },
    { 0x69017649, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5b81764c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c1764f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8ca17651, SUC_lexicon_tags_25 },
    { 0x68817652, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f817657, SUC_lexicon_tags_15 },
    { 0xcc017658, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec617663, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5617666, SUC_lexicon_tags_2 },
    { 0xec217667, SUC_lexicon_tags_28 },
    { 0x2fa17667, SUC_lexicon_tags_25 },
    { 0x52e17669, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e1766d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10617672, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46817674, SUC_lexicon_tags_49 },
    { 0xaba17674, SUC_lexicon_tags_60 },
    { 0xbc617676, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a17680, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e17684, SUC_lexicon_tags_2 },
    { 0xc3e17685, SUC_lexicon_tags_60 },
    { 0xbf617684, SUC_lexicon_tags_33 },
    { 0xdc017685, SUC_lexicon_tags_63 },
    { 0x86a17688, SUC_lexicon_tags_11 },
    { 0x69417689, SUC_lexicon_tags_25 },
    { 0x6341768a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88617690, SUC_lexicon_tags_21 },
    { 0x32617691, SUC_lexicon_tags_11 },
    { 0xd4617692, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a17697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb01769b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c1769f, SUC_lexicon_tags_1 },
    { 0x41a176a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7c2176a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a176a7, SUC_lexicon_tags_33 },
    { 0x73e176a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4bc176aa, SUC_lexicon_tags_11 },
    { 0x906176ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa72176ad, SUC_lexicon_tags_63 },
    { 0xa176ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96176b1, SUC_lexicon_tags_25 },
    { 0xda4176b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88176b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4176bb, SUC_lexicon_tags_27 },
    { 0x3de176bc, SUC_lexicon_tags_29 },
    { 0x340176bc, SUC_lexicon_tags_99 },
    { 0xde2176be, SUC_lexicon_tags_11 },
    { 0x5c2176bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b8176c1, SUC_lexicon_tags_23 },
    { 0x62a176c2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758176ca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6176cd, SUC_lexicon_tags_2 },
    { 0x3a8176ce, SUC_lexicon_tags_47 },
    { 0x2d4176cd, SUC_lexicon_tags_2 },
    { 0xc8a176cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6176d4, SUC_lexicon_tags_25 },
    { 0x650176d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x550176d7, SUC_lexicon_tags_19 },
    { 0x362176d8, SUC_lexicon_tags_84 },
    { 0xa9c176d9, SUC_lexicon_tags_27 },
    { 0xbbc176d9, SUC_lexicon_tags_36 },
    { 0x574176d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c176df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38176e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26176e9, SUC_lexicon_tags_7 },
    { 0x35e176ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc176ec, SUC_lexicon_tags_37 },
    { 0x520176ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2176f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c176f3, SUC_lexicon_tags_9 },
    { 0xd4a176f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf4c176f6, SUC_lexicon_tags_100 },
    { 0xd32176f6, SUC_lexicon_tags_43 },
    { 0x58e176f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf18176fa, SUC_lexicon_tags_27 },
    { 0x3b4176fa, SUC_lexicon_tags_27 },
    { 0x896176fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80e176fe, SUC_lexicon_tags_64 },
    { 0x802176ff, SUC_lexicon_tags_2 },
    { 0x42a17700, SUC_lexicon_tags_7 },
    { 0x9e17700, SUC_lexicon_tags_1 },
    { 0x90e176ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff21770b, SUC_lexicon_tags_8 },
    { 0xb381770b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe1770f, SUC_lexicon_tags_60 },
    { 0xb6417710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c817712, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a17715, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x72017717, SUC_lexicon_tags_14 },
    { 0xe4217717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9fa1771a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde61771c, SUC_lexicon_tags_2 },
    { 0x9f01771c, SUC_lexicon_tags_16 },
    { 0x51c1771e, SUC_lexicon_tags_12 },
    { 0x2b41771e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5217722, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d617724, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae017728, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7c81772a, SUC_lexicon_tags_47 },
    { 0x5761772a, SUC_lexicon_tags_27 },
    { 0x28c1772c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c17733, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc17737, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe101773b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca17740, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc17747, SUC_lexicon_tags_15 },
    { 0xa2617748, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf01774b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3761774f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc217753, SUC_lexicon_tags_47 },
    { 0xac17754, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xea017757, SUC_lexicon_tags_63 },
    { 0x417757, SUC_lexicon_tags_8 },
    { 0x3217759, SUC_lexicon_tags_2 },
    { 0x55017758, SUC_lexicon_tags_11 },
    { 0x5aa1775b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3021775d, SUC_lexicon_tags_17 },
    { 0xad61775e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33417762, SUC_lexicon_tags_6 },
    { 0xe6017763, SUC_lexicon_tags_2 },
    { 0x74e17763, SUC_lexicon_tags_2 },
    { 0x74c17764, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7417768, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7941776b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a1776e, SUC_lexicon_tags_25 },
    { 0x59a1776e, SUC_lexicon_tags_4 },
    { 0x31e17770, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43817774, SUC_lexicon_tags_1 },
    { 0x27217774, SUC_lexicon_tags_33 },
    { 0xc3417774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7941777a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xac41777c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe1777f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97e17781, SUC_lexicon_tags_10 },
    { 0x62817782, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b417784, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da17787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30e17789, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c17791, SUC_lexicon_tags_1 },
    { 0x67017791, SUC_lexicon_tags_25 },
    { 0xd6e17791, SUC_lexicon_tags_2 },
    { 0x67817794, SUC_lexicon_tags_11 },
    { 0x55417795, SUC_lexicon_tags_60 },
    { 0xac217796, SUC_lexicon_tags_15 },
    { 0xef017795, SUC_lexicon_tags_49 },
    { 0xfb617798, SUC_lexicon_tags_8 },
    { 0x82017799, SUC_lexicon_tags_2 },
    { 0x7c017797, SUC_lexicon_tags_37 },
    { 0x14e1779a, SUC_lexicon_tags_15 },
    { 0x4761779c, SUC_lexicon_tags_8 },
    { 0xea617799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9761779f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc177a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x538177a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46177a6, SUC_lexicon_tags_2 },
    { 0xd80177a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6177aa, SUC_lexicon_tags_37 },
    { 0x162177ab, SUC_lexicon_tags_60 },
    { 0xf1e177ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2177b3, SUC_lexicon_tags_37 },
    { 0x4a8177b4, SUC_lexicon_tags_7 },
    { 0xda8177b5, SUC_lexicon_tags_37 },
    { 0x4c2177b3, SUC_lexicon_tags_82 },
    { 0xb12177b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4177ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea177be, SUC_lexicon_tags_6 },
    { 0xe6c177bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4177c2, SUC_lexicon_tags_49 },
    { 0x3b4177c3, SUC_lexicon_tags_37 },
    { 0xf4e177c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8c6177c6, SUC_lexicon_tags_9 },
    { 0x22177c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40177ca, SUC_lexicon_tags_13 },
    { 0x69a177ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c2177cd, SUC_lexicon_tags_39 },
    { 0xe90177cd, SUC_lexicon_tags_37 },
    { 0x8cc177cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x46177d1, SUC_lexicon_tags_33 },
    { 0xc6177d1, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4177d7, SUC_lexicon_tags_82 },
    { 0x538177d7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x858177da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x626177dd, SUC_lexicon_tags_37 },
    { 0xfc8177de, SUC_lexicon_tags_37 },
    { 0x5ce177dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6177e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab0177e7, SUC_lexicon_tags_15 },
    { 0xd6c177e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x866177eb, SUC_lexicon_tags_11 },
    { 0x58a177ec, SUC_lexicon_tags_62 },
    { 0x7b2177ed, SUC_lexicon_tags_25 },
    { 0xd1a177ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4177f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe177f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd04177f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c177ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdbc17801, SUC_lexicon_tags_15 },
    { 0x96e17802, SUC_lexicon_tags_25 },
    { 0xe3017802, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2617806, SUC_lexicon_tags_70 },
    { 0x84417807, SUC_lexicon_tags_11 },
    { 0xe0a17806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c1780c, SUC_lexicon_tags_85 },
    { 0x7ea1780d, SUC_lexicon_tags_29 },
    { 0x9b01780e, SUC_lexicon_tags_11 },
    { 0xea41780e, SUC_lexicon_tags_0 },
    { 0xc661780f, SUC_lexicon_tags_46 },
    { 0xb321780d, SUC_lexicon_tags_1 },
    { 0xda017811, SUC_lexicon_tags_485 },
    { 0xf5c17810, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37417815, SUC_lexicon_tags_47 },
    { 0x90817816, SUC_lexicon_tags_65 },
    { 0x2dc17817, SUC_lexicon_tags_30 },
    { 0x7be17818, SUC_lexicon_tags_27 },
    { 0xb9017815, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8fe1781b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2617820, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9ee17822, SUC_lexicon_tags_34 },
    { 0x9f417822, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6217827, SUC_lexicon_tags_19 },
    { 0x92417828, SUC_lexicon_tags_15 },
    { 0xce17829, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc17830, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73617832, SUC_lexicon_tags_49 },
    { 0x13817832, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc417837, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c41783e, SUC_lexicon_tags_37 },
    { 0x51a1783f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe17847, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc61784a, SUC_lexicon_tags_15 },
    { 0xd2e1784a, SUC_lexicon_tags_15 },
    { 0x1c1784b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27417851, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8617854, SUC_lexicon_tags_17 },
    { 0x1dc17855, SUC_lexicon_tags_17 },
    { 0xa3617854, SUC_lexicon_tags_39 },
    { 0x7aa17857, SUC_lexicon_tags_38 },
    { 0x96817858, SUC_lexicon_tags_25 },
    { 0x14617858, SUC_lexicon_tags_3 },
    { 0x7f617857, SUC_lexicon_tags_17 },
    { 0xd1e1785b, SUC_lexicon_tags_92 },
    { 0x71c17859, SUC_lexicon_tags_25 },
    { 0x4261785d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc361785f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43817861, SUC_lexicon_tags_28 },
    { 0xfee17862, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9c17864, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc817868, SUC_lexicon_tags_4 },
    { 0xfa617869, SUC_lexicon_tags_17 },
    { 0x97e1786a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c1786c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7417874, SUC_lexicon_tags_1 },
    { 0xf4217875, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b617878, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e1787b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be1787e, SUC_lexicon_tags_11 },
    { 0x7e21787f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb817881, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc81788a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c1788e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76217893, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6617896, SUC_lexicon_tags_23 },
    { 0xf9a17896, SUC_lexicon_tags_37 },
    { 0xc7017896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa381789f, SUC_lexicon_tags_17 },
    { 0x89a1789f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4178a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38178ae, SUC_lexicon_tags_15 },
    { 0x49a178af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88c178b1, SUC_lexicon_tags_23 },
    { 0x13c178b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94178b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x196178b7, SUC_lexicon_tags_1 },
    { 0x1e2178b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e4178ba, SUC_lexicon_tags_21 },
    { 0xa8a178bb, SUC_lexicon_tags_25 },
    { 0x6e4178bb, SUC_lexicon_tags_25 },
    { 0xa1e178bd, SUC_lexicon_tags_7 },
    { 0xa1c178be, SUC_lexicon_tags_33 },
    { 0x62178bf, SUC_lexicon_tags_14 },
    { 0x2a0178bd, SUC_lexicon_tags_28 },
    { 0x196178bb, SUC_lexicon_tags_47 },
    { 0x8ca178c2, SUC_lexicon_tags_38 },
    { 0x734178c3, SUC_lexicon_tags_7 },
    { 0xa70178c2, SUC_lexicon_tags_11 },
    { 0x59c178c5, SUC_lexicon_tags_22 },
    { 0xd42178c3, SUC_lexicon_tags_57 },
    { 0xc28178c2, SUC_lexicon_tags_9 },
    { 0x360178bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc178cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae6178d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96178d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4178d7, SUC_lexicon_tags_2 },
    { 0x71e178d8, SUC_lexicon_tags_38 },
    { 0xcea178d8, SUC_lexicon_tags_15 },
    { 0x924178d9, SUC_lexicon_tags_235 },
    { 0xe4178db, SUC_lexicon_tags_12 },
    { 0x3e0178dc, SUC_lexicon_tags_13 },
    { 0x4dc178da, SUC_lexicon_tags_27 },
    { 0xd90178dd, SUC_lexicon_tags_17 },
    { 0xd16178df, SUC_lexicon_tags_33 },
    { 0x4178dd, SUC_lexicon_tags_64 },
    { 0x17a178d9, SUC_lexicon_tags_17 },
    { 0x2fa178d8, SUC_lexicon_tags_60 },
    { 0xbe0178e3, SUC_lexicon_tags_60 },
    { 0x676178d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe18178e6, SUC_lexicon_tags_11 },
    { 0xdd6178e7, SUC_lexicon_tags_38 },
    { 0x600178e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa178eb, SUC_lexicon_tags_11 },
    { 0x92178eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6178ef, SUC_lexicon_tags_11 },
    { 0xa34178ef, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c178f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca178f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa178fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e17905, SUC_lexicon_tags_2 },
    { 0x4217906, SUC_lexicon_tags_33 },
    { 0x48e17907, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe641790d, SUC_lexicon_tags_12 },
    { 0x7dc1790e, SUC_lexicon_tags_31 },
    { 0x7e61790f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x59c17911, SUC_lexicon_tags_9 },
    { 0xf3c17911, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e417917, SUC_lexicon_tags_7 },
    { 0xc0a17918, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd61791b, SUC_lexicon_tags_33 },
    { 0xd2a1791c, SUC_lexicon_tags_38 },
    { 0x1301791c, SUC_lexicon_tags_17 },
    { 0xe5a1791e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c17922, SUC_lexicon_tags_8 },
    { 0x26617923, SUC_lexicon_tags_11 },
    { 0xd4c17924, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c17928, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341792f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a17934, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c17938, SUC_lexicon_tags_48 },
    { 0xb1c17938, SUC_lexicon_tags_17 },
    { 0xe5a1793a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4041793d, SUC_lexicon_tags_29 },
    { 0xd2c1793e, SUC_lexicon_tags_49 },
    { 0x9b81793f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63417942, SUC_lexicon_tags_45 },
    { 0x99817943, SUC_lexicon_tags_38 },
    { 0x1ca17944, SUC_lexicon_tags_37 },
    { 0x2017945, SUC_lexicon_tags_11 },
    { 0x85a17946, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf801794a, SUC_lexicon_tags_23 },
    { 0x3701794b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed61794d, SUC_lexicon_tags_37 },
    { 0xa8a1794d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97617953, SUC_lexicon_tags_85 },
    { 0x2ec17953, SUC_lexicon_tags_17 },
    { 0x31417955, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f817958, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa321795c, SUC_lexicon_tags_15 },
    { 0x24c1795d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3017966, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c617969, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4661796b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb501796f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x88417972, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xd9017974, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac017978, SUC_lexicon_tags_2 },
    { 0x1ea17978, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf781797f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d617981, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3a17983, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9be17985, SUC_lexicon_tags_15 },
    { 0x23817985, SUC_lexicon_tags_18 },
    { 0x32017986, SUC_lexicon_tags_11 },
    { 0x5be17988, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee1798b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa217993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27817995, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x61817998, SUC_lexicon_tags_14 },
    { 0xe0e17999, SUC_lexicon_tags_33 },
    { 0x9a01799a, SUC_lexicon_tags_17 },
    { 0xd081799a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0179a0, SUC_lexicon_tags_9 },
    { 0x484179a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb18179a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de179a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28179ab, SUC_lexicon_tags_43 },
    { 0x318179ac, SUC_lexicon_tags_37 },
    { 0xbd0179ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc179b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a179b9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c179be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16179c3, SUC_lexicon_tags_25 },
    { 0x268179c4, SUC_lexicon_tags_5 },
    { 0xe5c179c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876179ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc4179cc, SUC_lexicon_tags_1 },
    { 0x620179cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x38c179cf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x25e179d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x910179d3, SUC_lexicon_tags_46 },
    { 0x2c179d3, SUC_lexicon_tags_19 },
    { 0xbc6179d4, SUC_lexicon_tags_8 },
    { 0x4d4179d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10179da, SUC_lexicon_tags_37 },
    { 0xf98179db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8179e0, SUC_lexicon_tags_1 },
    { 0x9b0179e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a179e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04179ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa179ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x354179f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6179f5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4a6179f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a179fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda17a02, SUC_lexicon_tags_43 },
    { 0xe1617a03, SUC_lexicon_tags_37 },
    { 0xc0e17a04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a17a07, SUC_lexicon_tags_17 },
    { 0xc4e17a07, SUC_lexicon_tags_14 },
    { 0x79e17a07, SUC_lexicon_tags_83 },
    { 0x6d617a0a, SUC_lexicon_tags_11 },
    { 0x7e17a0a, SUC_lexicon_tags_38 },
    { 0xf5417a0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9817a0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e217a16, SUC_lexicon_tags_12 },
    { 0x8cc17a17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe17a1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5617a22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6617a24, SUC_lexicon_tags_25 },
    { 0xb3017a24, SUC_lexicon_tags_12 },
    { 0xf8817a26, SUC_lexicon_tags_2 },
    { 0xbb817a27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe017a29, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x96617a2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a17a31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc217a33, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c17a37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x37417a3a, SUC_lexicon_tags_25 },
    { 0xf1617a3b, SUC_lexicon_tags_38 },
    { 0x6017a3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd617a40, SUC_lexicon_tags_61 },
    { 0xf3c17a41, SUC_lexicon_tags_0 },
    { 0xf7417a42, SUC_lexicon_tags_14 },
    { 0x3f217a43, SUC_lexicon_tags_2 },
    { 0xe1817a44, SUC_lexicon_tags_17 },
    { 0xb1417a45, SUC_lexicon_tags_26 },
    { 0xca617a46, SUC_lexicon_tags_995 },
    { 0x96817a44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1bc17a49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22e17a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97817a50, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1617a54, SUC_lexicon_tags_48 },
    { 0xf3e17a54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a17a5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86c17a5e, SUC_lexicon_tags_49 },
    { 0x9fc17a5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a17a62, SUC_lexicon_tags_37 },
    { 0x3ce17a62, SUC_lexicon_tags_18 },
    { 0xecc17a63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11817a66, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a17a69, SUC_lexicon_tags_16 },
    { 0x26c17a6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x12417a6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6217a71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9617a75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca17a78, SUC_lexicon_tags_2 },
    { 0xe3017a79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea17a7c, SUC_lexicon_tags_1 },
    { 0xc8417a7d, SUC_lexicon_tags_37 },
    { 0xb0817a7e, SUC_lexicon_tags_11 },
    { 0xe9817a7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe617a84, SUC_lexicon_tags_11 },
    { 0x77c17a85, SUC_lexicon_tags_14 },
    { 0x57c17a85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9617a8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c17a8d, SUC_lexicon_tags_1 },
    { 0x7fe17a8e, SUC_lexicon_tags_15 },
    { 0x77e17a8e, SUC_lexicon_tags_12 },
    { 0xb8c17a90, SUC_lexicon_tags_17 },
    { 0x29817a90, SUC_lexicon_tags_1 },
    { 0xf3c17a92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3217a97, SUC_lexicon_tags_17 },
    { 0x84817a98, SUC_lexicon_tags_30 },
    { 0xe5c17a99, SUC_lexicon_tags_60 },
    { 0xa17a99, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x81c17a9c, SUC_lexicon_tags_48 },
    { 0x32e17a9d, SUC_lexicon_tags_32 },
    { 0xcd417a9d, SUC_lexicon_tags_37 },
    { 0x18c17a9f, SUC_lexicon_tags_37 },
    { 0xcee17a9f, SUC_lexicon_tags_30 },
    { 0x2017a9c, SUC_lexicon_tags_1 },
    { 0xf3617a9c, SUC_lexicon_tags_27 },
    { 0x2ea17aa2, SUC_lexicon_tags_47 },
    { 0xf2817aa4, SUC_lexicon_tags_14 },
    { 0xfc417aa5, SUC_lexicon_tags_2 },
    { 0xcba17aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49217aa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54217aab, SUC_lexicon_tags_7 },
    { 0xc617aab, SUC_lexicon_tags_21 },
    { 0x74c17aac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2c417aaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd817ab1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3217ab9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e017abb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc17ac4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de17acc, SUC_lexicon_tags_15 },
    { 0x2f017acd, SUC_lexicon_tags_8 },
    { 0x61017ace, SUC_lexicon_tags_64 },
    { 0xb7017ace, SUC_lexicon_tags_0 },
    { 0x55017ace, SUC_lexicon_tags_25 },
    { 0x6017ad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69c17ad3, SUC_lexicon_tags_27 },
    { 0x97a17ad3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0417ad6, SUC_lexicon_tags_27 },
    { 0xd7a17ad6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x74017ada, SUC_lexicon_tags_31 },
    { 0x5417ada, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c17adf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e17ae2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca17ae5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de17ae8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a17aeb, SUC_lexicon_tags_146 },
    { 0x38e17aec, SUC_lexicon_tags_14 },
    { 0xa417aed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c17af2, SUC_lexicon_tags_37 },
    { 0x5ca17af2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1817af5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xdf217af7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x55417af9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a17afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca17b04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a17b08, SUC_lexicon_tags_8 },
    { 0x79a17b09, SUC_lexicon_tags_19 },
    { 0xac617b08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba17b0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c17b13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65617b19, SUC_lexicon_tags_2 },
    { 0x10817b1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65817b1d, SUC_lexicon_tags_37 },
    { 0xddc17b1e, SUC_lexicon_tags_32 },
    { 0xec217b1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf417b21, SUC_lexicon_tags_0 },
    { 0xaa17b22, SUC_lexicon_tags_46 },
    { 0xf4017b21, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x24217b26, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f817b34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c17b37, SUC_lexicon_tags_29 },
    { 0x41617b37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x90817b3b, SUC_lexicon_tags_35 },
    { 0x44017b3c, SUC_lexicon_tags_28 },
    { 0x4a17b3d, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b417b42, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f217b49, SUC_lexicon_tags_29 },
    { 0x54417b4a, SUC_lexicon_tags_2 },
    { 0x9f817b4b, SUC_lexicon_tags_3 },
    { 0xf6017b4c, SUC_lexicon_tags_11 },
    { 0xa7417b4d, SUC_lexicon_tags_13 },
    { 0x42c17b4e, SUC_lexicon_tags_43 },
    { 0x8d417b4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e17b56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d617b59, SUC_lexicon_tags_39 },
    { 0xe6817b5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c17b5d, SUC_lexicon_tags_42 },
    { 0xe8c17b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6a17b60, SUC_lexicon_tags_37 },
    { 0x6ce17b61, SUC_lexicon_tags_25 },
    { 0xeb217b61, SUC_lexicon_tags_1 },
    { 0x3617b63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x13817b65, SUC_lexicon_tags_1 },
    { 0xc0617b65, SUC_lexicon_tags_19 },
    { 0x63417b66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c17b6b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c17b6e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba217b74, SUC_lexicon_tags_25 },
    { 0xff617b75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1617b78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa217b7d, SUC_lexicon_tags_12 },
    { 0x75817b7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc217b82, SUC_lexicon_tags_1 },
    { 0xe6417b83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31817b87, SUC_lexicon_tags_63 },
    { 0x5fc17b87, SUC_lexicon_tags_2 },
    { 0xdba17b89, SUC_lexicon_tags_2 },
    { 0xcf617b8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33617b8e, SUC_lexicon_tags_19 },
    { 0xd7417b8e, SUC_lexicon_tags_25 },
    { 0x1e217b8e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf417b96, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc17b9d, SUC_lexicon_tags_2 },
    { 0xa7017b9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce17ba2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8617ba5, SUC_lexicon_tags_23 },
    { 0xd5417ba6, SUC_lexicon_tags_61 },
    { 0x1b017ba7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71417ba9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37217baf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83217bb2, SUC_lexicon_tags_30 },
    { 0x61617bb3, SUC_lexicon_tags_28 },
    { 0xd617bb3, SUC_lexicon_tags_0 },
    { 0xf0817bb2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e17bc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c817bc5, SUC_lexicon_tags_11 },
    { 0x53417bc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6617bca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x37617bcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d617bd6, SUC_lexicon_tags_37 },
    { 0xa0c17bd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c17bd9, SUC_lexicon_tags_33 },
    { 0x17617bda, SUC_lexicon_tags_63 },
    { 0xf7e17bda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1817be2, SUC_lexicon_tags_37 },
    { 0x45817be3, SUC_lexicon_tags_37 },
    { 0x89a17be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c17beb, SUC_lexicon_tags_107 },
    { 0xb1417bec, SUC_lexicon_tags_36 },
    { 0x83c17bec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b017bf0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfd217bf2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x25817bf4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e17bf8, SUC_lexicon_tags_49 },
    { 0xbbe17bf9, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7417bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c017c03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf617c06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff617c11, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x60e17c13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac17c18, SUC_lexicon_tags_28 },
    { 0x88017c19, SUC_lexicon_tags_37 },
    { 0x68a17c19, SUC_lexicon_tags_69 },
    { 0x4c17c1a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4217c22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a17c25, SUC_lexicon_tags_17 },
    { 0x68c17c26, SUC_lexicon_tags_25 },
    { 0x43017c27, SUC_lexicon_tags_47 },
    { 0x3e817c28, SUC_lexicon_tags_56 },
    { 0x13a17c27, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0217c30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc17c33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a17c36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11a17c38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63817c3c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3217c3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbc617c41, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c17c46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc17c4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x61417c4c, SUC_lexicon_tags_11 },
    { 0x29c17c4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee17c51, SUC_lexicon_tags_15 },
    { 0xe6417c51, SUC_lexicon_tags_65 },
    { 0x23c17c53, SUC_lexicon_tags_49 },
    { 0x9d217c51, SUC_lexicon_tags_15 },
    { 0x21a17c51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56617c5e, SUC_lexicon_tags_11 },
    { 0xd0417c5e, SUC_lexicon_tags_12 },
    { 0x2ac17c5e, SUC_lexicon_tags_7 },
    { 0xcb017c5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79817c65, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xcf417c67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3a17c69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7817c6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93217c70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa17c7c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a17c7f, SUC_lexicon_tags_33 },
    { 0x1617c7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1a17c82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c217c85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8417c89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed817c8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97817c8e, SUC_lexicon_tags_27 },
    { 0xb5c17c8f, SUC_lexicon_tags_40 },
    { 0x97417c90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa17c94, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de17c9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e17c9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c17ca0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e17ca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3017ca5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9617caa, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6417cae, SUC_lexicon_tags_9 },
    { 0xbac17caf, SUC_lexicon_tags_25 },
    { 0x97c17cb0, SUC_lexicon_tags_2 },
    { 0x37617cb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d017cb4, SUC_lexicon_tags_27 },
    { 0x90a17cb5, SUC_lexicon_tags_66 },
    { 0xa1417cb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2617cb9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93617cbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xefe17cc1, SUC_lexicon_tags_45 },
    { 0x81217cc2, SUC_lexicon_tags_30 },
    { 0x62a17cc3, SUC_lexicon_tags_9 },
    { 0x8aa17cc2, SUC_lexicon_tags_77 },
    { 0x3d417cc5, SUC_lexicon_tags_53 },
    { 0x8ae17cc6, SUC_lexicon_tags_11 },
    { 0x30a17cc1, SUC_lexicon_tags_68 },
    { 0x8d017cc4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4b617cca, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc17cce, SUC_lexicon_tags_57 },
    { 0x61217cce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a17cd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa17cdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0017ce4, SUC_lexicon_tags_0 },
    { 0x42c17ce5, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x32a17ce7, SUC_lexicon_tags_28 },
    { 0xe7217ce8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c17ceb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8dc17ced, SUC_lexicon_tags_15 },
    { 0xc8a17cee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a17cf1, SUC_lexicon_tags_14 },
    { 0xf5e17cf2, SUC_lexicon_tags_45 },
    { 0xe2817cf3, SUC_lexicon_tags_37 },
    { 0x69c17cf4, SUC_lexicon_tags_1 },
    { 0x69217cf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e17cf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7217cfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89017d00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4c217d02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe2e17d04, SUC_lexicon_tags_2 },
    { 0x40a17d04, SUC_lexicon_tags_2 },
    { 0x3617d06, SUC_lexicon_tags_63 },
    { 0xeac17d05, SUC_lexicon_tags_28 },
    { 0xaa417d08, SUC_lexicon_tags_47 },
    { 0x3bc17d06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaf217d0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec17d0f, SUC_lexicon_tags_49 },
    { 0x34217d10, SUC_lexicon_tags_2 },
    { 0x6dc17d11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1017d13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac17d18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6617d1a, SUC_lexicon_tags_15 },
    { 0x34e17d1b, SUC_lexicon_tags_17 },
    { 0x16017d1b, SUC_lexicon_tags_44 },
    { 0x20c17d1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c617d20, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a17d26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x14817d28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0817d2a, SUC_lexicon_tags_12 },
    { 0xec17d2b, SUC_lexicon_tags_26 },
    { 0x52817d2c, SUC_lexicon_tags_68 },
    { 0xea417d2d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c17d35, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc5c17d37, SUC_lexicon_tags_9 },
    { 0xd617d38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad617d3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c17d44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5017d4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf017d4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec17d51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b017d58, SUC_lexicon_tags_23 },
    { 0xdf817d59, SUC_lexicon_tags_43 },
    { 0x9a617d5a, SUC_lexicon_tags_12 },
    { 0xabc17d5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5217d5d, SUC_lexicon_tags_60 },
    { 0x33e17d5e, SUC_lexicon_tags_7 },
    { 0x9e617d5e, SUC_lexicon_tags_79 },
    { 0x9fc17d60, SUC_lexicon_tags_45 },
    { 0xbb017d5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x95817d64, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x66a17d66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c17d6b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x77017d6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d617d70, SUC_lexicon_tags_25 },
    { 0xfda17d71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb617d75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a17d78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3c17d7a, SUC_lexicon_tags_7 },
    { 0x4cc17d7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cc17d7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89417d83, SUC_lexicon_tags_15 },
    { 0x6ac17d83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe617d89, SUC_lexicon_tags_39 },
    { 0x80c17d89, SUC_lexicon_tags_11 },
    { 0x60217d89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54417d8e, SUC_lexicon_tags_15 },
    { 0x76217d8f, SUC_lexicon_tags_25 },
    { 0x62417d8e, SUC_lexicon_tags_27 },
    { 0xc8817d8e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x79617d94, SUC_lexicon_tags_12 },
    { 0x817d95, SUC_lexicon_tags_12 },
    { 0xc5617d96, SUC_lexicon_tags_66 },
    { 0xcea17d97, SUC_lexicon_tags_42 },
    { 0xdf017d98, SUC_lexicon_tags_26 },
    { 0x1ca17d99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d417d9c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c217da0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa2017da2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdbc17da4, SUC_lexicon_tags_27 },
    { 0x8e417da5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27a17da7, SUC_lexicon_tags_17 },
    { 0x9817da8, SUC_lexicon_tags_15 },
    { 0xb8417da8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7817dab, SUC_lexicon_tags_7 },
    { 0xa0417dac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9217db3, SUC_lexicon_tags_0 },
    { 0xaec17db3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb217db7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a17dc3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a17dcb, SUC_lexicon_tags_12 },
    { 0xe2e17dcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43817dd0, SUC_lexicon_tags_11 },
    { 0x9a17dd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa817dd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c17dd8, SUC_lexicon_tags_6 },
    { 0xe0817dd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8d417ddb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74417ddf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb017de4, SUC_lexicon_tags_17 },
    { 0xbac17de5, SUC_lexicon_tags_9 },
    { 0x99a17de6, SUC_lexicon_tags_7 },
    { 0x8ee17de7, SUC_lexicon_tags_17 },
    { 0xd5817de4, SUC_lexicon_tags_201 },
    { 0x20817de9, SUC_lexicon_tags_45 },
    { 0x94e17de9, SUC_lexicon_tags_17 },
    { 0xd8e17de5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad617dee, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c17df1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27217df5, SUC_lexicon_tags_16 },
    { 0x1a217df5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6417df8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c217dfa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f217e01, SUC_lexicon_tags_32 },
    { 0xe8417e01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217e07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d817e09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b217e0c, SUC_lexicon_tags_12 },
    { 0x8017e0c, SUC_lexicon_tags_11 },
    { 0x8f417e0e, SUC_lexicon_tags_789 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a17e17, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa0217e19, SUC_lexicon_tags_1 },
    { 0x12417e1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d817e1e, SUC_lexicon_tags_26 },
    { 0x85417e1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e17e24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x62017e27, SUC_lexicon_tags_0 },
    { 0x86c17e28, SUC_lexicon_tags_33 },
    { 0x42017e29, SUC_lexicon_tags_7 },
    { 0x4ec17e2a, SUC_lexicon_tags_63 },
    { 0x49417e2b, SUC_lexicon_tags_3 },
    { 0xffc17e2c, SUC_lexicon_tags_12 },
    { 0x7b017e2c, SUC_lexicon_tags_25 },
    { 0x36617e2a, SUC_lexicon_tags_31 },
    { 0x4aa17e2d, SUC_lexicon_tags_15 },
    { 0x15c17e30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c17e33, SUC_lexicon_tags_1 },
    { 0xbe017e33, SUC_lexicon_tags_37 },
    { 0x89217e35, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x78a17e37, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9617e49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc3c17e4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15217e52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e17e57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c17e5c, SUC_lexicon_tags_68 },
    { 0xe8217e5d, SUC_lexicon_tags_39 },
    { 0xdc817e5d, SUC_lexicon_tags_47 },
    { 0x25217e5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d617e63, SUC_lexicon_tags_66 },
    { 0xb417e63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f817e6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a17e6e, SUC_lexicon_tags_38 },
    { 0x1f817e6f, SUC_lexicon_tags_43 },
    { 0x2f617e6f, SUC_lexicon_tags_16 },
    { 0xeba17e70, SUC_lexicon_tags_26 },
    { 0xb5617e6f, SUC_lexicon_tags_11 },
    { 0x8b817e73, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b017e76, SUC_lexicon_tags_38 },
    { 0xbe017e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51e17e79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65217e7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf417e84, SUC_lexicon_tags_9 },
    { 0x28e17e85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c17e8c, SUC_lexicon_tags_70 },
    { 0x5d617e8c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba417e94, SUC_lexicon_tags_127 },
    { 0xb7e17e95, SUC_lexicon_tags_70 },
    { 0x93017e96, SUC_lexicon_tags_15 },
    { 0xc9e17e94, SUC_lexicon_tags_37 },
    { 0x14017e98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6617e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31617e9f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8217ea9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63417eb4, SUC_lexicon_tags_15 },
    { 0x41e17eb5, SUC_lexicon_tags_12 },
    { 0xf4817eb5, SUC_lexicon_tags_11 },
    { 0x4bc17eb7, SUC_lexicon_tags_25 },
    { 0x99217eb8, SUC_lexicon_tags_43 },
    { 0x87a17eb9, SUC_lexicon_tags_60 },
    { 0xe6a17eb7, SUC_lexicon_tags_11 },
    { 0x64e17ebb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e017ebd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xeb217ebf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x40017ec2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1217ec4, SUC_lexicon_tags_7 },
    { 0xd8417ec5, SUC_lexicon_tags_2 },
    { 0xbb617ec4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59017eca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a17ecc, SUC_lexicon_tags_37 },
    { 0x52a17ecd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e17ed0, SUC_lexicon_tags_7 },
    { 0x45a17ed1, SUC_lexicon_tags_15 },
    { 0x79e17ed1, SUC_lexicon_tags_7 },
    { 0xad617ed3, SUC_lexicon_tags_37 },
    { 0xc5e17ed1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a17edf, SUC_lexicon_tags_38 },
    { 0xbe817ee0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb6617ee2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0817ee6, SUC_lexicon_tags_49 },
    { 0x94617ee7, SUC_lexicon_tags_1 },
    { 0xe6617ee8, SUC_lexicon_tags_17 },
    { 0x56017ee7, SUC_lexicon_tags_21 },
    { 0x5e817eea, SUC_lexicon_tags_43 },
    { 0x8f817eeb, SUC_lexicon_tags_49 },
    { 0xb6e17eec, SUC_lexicon_tags_0 },
    { 0xc9e17eed, SUC_lexicon_tags_31 },
    { 0x9ee17eed, SUC_lexicon_tags_25 },
    { 0x53017eef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a17ef8, SUC_lexicon_tags_1 },
    { 0xc6c17ef9, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x34617efb, SUC_lexicon_tags_68 },
    { 0x62e17efc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea017f01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9217f03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b217f07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1e17f09, SUC_lexicon_tags_17 },
    { 0x7ac17f09, SUC_lexicon_tags_1 },
    { 0xb8617f0b, SUC_lexicon_tags_45 },
    { 0xfe617f0c, SUC_lexicon_tags_14 },
    { 0x55c17f0d, SUC_lexicon_tags_11 },
    { 0x19c17f0d, SUC_lexicon_tags_4 },
    { 0xcb217f0f, SUC_lexicon_tags_11 },
    { 0xe4c17f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e17f14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac217f16, SUC_lexicon_tags_47 },
    { 0x26217f17, SUC_lexicon_tags_33 },
    { 0xc6017f16, SUC_lexicon_tags_1 },
    { 0x53e17f19, SUC_lexicon_tags_2 },
    { 0x9d617f1a, SUC_lexicon_tags_0 },
    { 0xca417f1b, SUC_lexicon_tags_21 },
    { 0x3c417f16, SUC_lexicon_tags_1038 },
    { 0xdbc17f16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd5417f1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3817f26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35417f2a, SUC_lexicon_tags_12 },
    { 0xd0a17f2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x59017f2d, SUC_lexicon_tags_29 },
    { 0xc5a17f2e, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65817f35, SUC_lexicon_tags_15 },
    { 0x90417f36, SUC_lexicon_tags_15 },
    { 0x68417f37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a417f39, SUC_lexicon_tags_2 },
    { 0xe3817f39, SUC_lexicon_tags_118 },
    { 0x67e17f39, SUC_lexicon_tags_11 },
    { 0xe617f3c, SUC_lexicon_tags_29 },
    { 0x8b817f3d, SUC_lexicon_tags_25 },
    { 0x41a17f3e, SUC_lexicon_tags_37 },
    { 0xb3017f3e, SUC_lexicon_tags_1 },
    { 0x2ba17f3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50c17f42, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd617f45, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x4f217f47, SUC_lexicon_tags_37 },
    { 0xa7417f48, SUC_lexicon_tags_60 },
    { 0xcae17f49, SUC_lexicon_tags_1 },
    { 0x12617f48, SUC_lexicon_tags_14 },
    { 0x36417f4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c17f4f, SUC_lexicon_tags_13 },
    { 0xd8017f4f, SUC_lexicon_tags_47 },
    { 0xcf817f51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49417f53, SUC_lexicon_tags_537 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54017f57, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xac17f5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28017f5e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x53617f61, SUC_lexicon_tags_35 },
    { 0x79a17f62, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xea17f65, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92217f71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94017f73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5617f79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45617f7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c017f80, SUC_lexicon_tags_65 },
    { 0x3ea17f81, SUC_lexicon_tags_38 },
    { 0x15817f82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda17f85, SUC_lexicon_tags_27 },
    { 0x96a17f85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8217f8e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x34217f91, SUC_lexicon_tags_11 },
    { 0xeda17f92, SUC_lexicon_tags_38 },
    { 0x6017f91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x87017f96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee17f99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec17f9c, SUC_lexicon_tags_8 },
    { 0xc5a17f9d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c17fa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x66417fa6, SUC_lexicon_tags_27 },
    { 0x96417fa6, SUC_lexicon_tags_39 },
    { 0xf0017fa7, SUC_lexicon_tags_146 },
    { 0xcc617fa9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa417fb1, SUC_lexicon_tags_46 },
    { 0x15817fb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e17fb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d817fb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86e17fba, SUC_lexicon_tags_11 },
    { 0x82c17fbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27217fc0, SUC_lexicon_tags_49 },
    { 0xafc17fc1, SUC_lexicon_tags_49 },
    { 0x3f617fc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb017fc4, SUC_lexicon_tags_33 },
    { 0xe5617fc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a817fc7, SUC_lexicon_tags_43 },
    { 0x15217fc8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25417fcb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3017fce, SUC_lexicon_tags_60 },
    { 0xa9c17fce, SUC_lexicon_tags_1 },
    { 0x3b417fce, SUC_lexicon_tags_2 },
    { 0xbe817fd1, SUC_lexicon_tags_47 },
    { 0xfce17fd1, SUC_lexicon_tags_29 },
    { 0x65a17fd3, SUC_lexicon_tags_15 },
    { 0x62c17fd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6e17fd6, SUC_lexicon_tags_2 },
    { 0xdd617fd7, SUC_lexicon_tags_15 },
    { 0x95017fd8, SUC_lexicon_tags_14 },
    { 0x58e17fd8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb3e17fdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c17fe0, SUC_lexicon_tags_37 },
    { 0xf8617fe0, SUC_lexicon_tags_11 },
    { 0xfde17fe2, SUC_lexicon_tags_3 },
    { 0xb2217fe2, SUC_lexicon_tags_21 },
    { 0x57417fe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26217fe6, SUC_lexicon_tags_15 },
    { 0xabc17fe7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f817fec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x18217fef, SUC_lexicon_tags_11 },
    { 0x9b817ff0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0817ff4, SUC_lexicon_tags_15 },
    { 0xf0417ff5, SUC_lexicon_tags_7 },
    { 0x6e417ff5, SUC_lexicon_tags_1 },
    { 0x53a17ff4, SUC_lexicon_tags_17 },
    { 0x7c417ff8, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11017ffd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e18001, SUC_lexicon_tags_37 },
    { 0xdc418002, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce818008, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b01801b, SUC_lexicon_tags_33 },
    { 0x66e1801b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e818021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e18025, SUC_lexicon_tags_14 },
    { 0xca218026, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x30618028, SUC_lexicon_tags_14 },
    { 0xeda18029, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xef01802b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c18031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7218036, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac1803a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a1803d, SUC_lexicon_tags_1 },
    { 0x85e1803e, SUC_lexicon_tags_25 },
    { 0x26a1803f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb018049, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e81804f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x87618052, SUC_lexicon_tags_17 },
    { 0x61c18052, SUC_lexicon_tags_17 },
    { 0x6fc18054, SUC_lexicon_tags_27 },
    { 0x35018055, SUC_lexicon_tags_65 },
    { 0x3c18056, SUC_lexicon_tags_39 },
    { 0x7a218055, SUC_lexicon_tags_35 },
    { 0x3e218055, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b61805d, SUC_lexicon_tags_82 },
    { 0xcd61805e, SUC_lexicon_tags_12 },
    { 0x8f01805e, SUC_lexicon_tags_63 },
    { 0xf5e1805e, SUC_lexicon_tags_12 },
    { 0x48018061, SUC_lexicon_tags_33 },
    { 0x1da1805e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf61806a, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0xf81806c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a18070, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe18072, SUC_lexicon_tags_37 },
    { 0x88418072, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7018077, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf541807a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8081807e, SUC_lexicon_tags_17 },
    { 0x7301807f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8018081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e618084, SUC_lexicon_tags_28 },
    { 0x5c418085, SUC_lexicon_tags_48 },
    { 0x47418084, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a1808f, SUC_lexicon_tags_49 },
    { 0x27618090, SUC_lexicon_tags_11 },
    { 0x7c418091, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c618093, SUC_lexicon_tags_33 },
    { 0x77618094, SUC_lexicon_tags_27 },
    { 0x15e18095, SUC_lexicon_tags_17 },
    { 0xb2e18096, SUC_lexicon_tags_27 },
    { 0x3b618097, SUC_lexicon_tags_6 },
    { 0xfd218098, SUC_lexicon_tags_63 },
    { 0x5618095, SUC_lexicon_tags_7 },
    { 0x47a18099, SUC_lexicon_tags_38 },
    { 0xe6a18097, SUC_lexicon_tags_15 },
    { 0x15a18097, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4180a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe180a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e4180a4, SUC_lexicon_tags_30 },
    { 0xaa4180a5, SUC_lexicon_tags_11 },
    { 0x76e180a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc180a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0180af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4180b2, SUC_lexicon_tags_2 },
    { 0x4de180b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad2180b5, SUC_lexicon_tags_0 },
    { 0xb28180b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20180b9, SUC_lexicon_tags_56 },
    { 0x50e180b9, SUC_lexicon_tags_31 },
    { 0xb68180bb, SUC_lexicon_tags_66 },
    { 0x3d4180bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8180c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x896180c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee180c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6fe180cb, SUC_lexicon_tags_31 },
    { 0x9c6180cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6cc180ce, SUC_lexicon_tags_12 },
    { 0xdd0180ce, SUC_lexicon_tags_15 },
    { 0x506180d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604180d4, SUC_lexicon_tags_29 },
    { 0x2180d5, SUC_lexicon_tags_12 },
    { 0x6ae180d5, SUC_lexicon_tags_46 },
    { 0x3f8180d6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0180dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2180df, SUC_lexicon_tags_32 },
    { 0xa90180e0, SUC_lexicon_tags_45 },
    { 0x74e180e1, SUC_lexicon_tags_113 },
    { 0xe06180e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2180e5, SUC_lexicon_tags_2 },
    { 0x2a4180e6, SUC_lexicon_tags_7 },
    { 0x60e180e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166180ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56a180f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a180f5, SUC_lexicon_tags_7 },
    { 0xaca180f6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0180fb, SUC_lexicon_tags_21 },
    { 0x6a8180fb, SUC_lexicon_tags_23 },
    { 0xf96180fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a18103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7218108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe361810e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36618110, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c18113, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e18117, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c01811c, SUC_lexicon_tags_4 },
    { 0xdfc1811c, SUC_lexicon_tags_11 },
    { 0x55e1811c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb018122, SUC_lexicon_tags_14 },
    { 0xc9e18122, SUC_lexicon_tags_12 },
    { 0xb4418122, SUC_lexicon_tags_17 },
    { 0xec018124, SUC_lexicon_tags_21 },
    { 0x8e18124, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a18129, SUC_lexicon_tags_25 },
    { 0x5061812a, SUC_lexicon_tags_37 },
    { 0x49a1812b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c18132, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1ae18134, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c018137, SUC_lexicon_tags_62 },
    { 0xce818138, SUC_lexicon_tags_49 },
    { 0xf0418139, SUC_lexicon_tags_12 },
    { 0x9e218137, SUC_lexicon_tags_2 },
    { 0x5241813b, SUC_lexicon_tags_47 },
    { 0xec81813c, SUC_lexicon_tags_38 },
    { 0x6e21813d, SUC_lexicon_tags_15 },
    { 0xee01813b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e618141, SUC_lexicon_tags_84 },
    { 0x76c18142, SUC_lexicon_tags_49 },
    { 0x65e18142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc418148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe221814c, SUC_lexicon_tags_25 },
    { 0x2841814d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24c1814f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5618151, SUC_lexicon_tags_12 },
    { 0xf4218151, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde18157, SUC_lexicon_tags_17 },
    { 0x2e818158, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea418162, SUC_lexicon_tags_7 },
    { 0x14a18163, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94218165, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17618168, SUC_lexicon_tags_7 },
    { 0xba418168, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb21816e, SUC_lexicon_tags_17 },
    { 0xa341816f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9418175, SUC_lexicon_tags_0 },
    { 0x2c018176, SUC_lexicon_tags_25 },
    { 0x69418175, SUC_lexicon_tags_11 },
    { 0x37a18178, SUC_lexicon_tags_7 },
    { 0x31218179, SUC_lexicon_tags_37 },
    { 0xb5218175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd218181, SUC_lexicon_tags_16 },
    { 0x75a18181, SUC_lexicon_tags_31 },
    { 0x8e018182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c18187, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01818a, SUC_lexicon_tags_17 },
    { 0xb881818b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd521818e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66018196, SUC_lexicon_tags_29 },
    { 0xe5218196, SUC_lexicon_tags_33 },
    { 0x1e018198, SUC_lexicon_tags_2 },
    { 0xdfa18196, SUC_lexicon_tags_49 },
    { 0xb601819a, SUC_lexicon_tags_7 },
    { 0x3261819b, SUC_lexicon_tags_47 },
    { 0x5001819c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e1819f, SUC_lexicon_tags_30 },
    { 0x2b2181a0, SUC_lexicon_tags_39 },
    { 0x124181a0, SUC_lexicon_tags_37 },
    { 0xa6e181a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be181a5, SUC_lexicon_tags_12 },
    { 0x940181a6, SUC_lexicon_tags_56 },
    { 0xefc181a7, SUC_lexicon_tags_17 },
    { 0x608181a8, SUC_lexicon_tags_1 },
    { 0x1cc181a8, SUC_lexicon_tags_35 },
    { 0x79c181a8, SUC_lexicon_tags_28 },
    { 0xb76181a7, SUC_lexicon_tags_77 },
    { 0xe6181ac, SUC_lexicon_tags_17 },
    { 0xe0e181a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x100181af, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2181b2, SUC_lexicon_tags_7 },
    { 0x2f8181b3, SUC_lexicon_tags_27 },
    { 0x4c181b2, SUC_lexicon_tags_0 },
    { 0x7c0181b5, SUC_lexicon_tags_64 },
    { 0xc6181b6, SUC_lexicon_tags_31 },
    { 0x70c181b5, SUC_lexicon_tags_17 },
    { 0xd94181b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x966181ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162181c0, SUC_lexicon_tags_27 },
    { 0xb04181c0, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80181c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8181c9, SUC_lexicon_tags_12 },
    { 0x64e181ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a6181cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc181cf, SUC_lexicon_tags_106 },
    { 0xd7c181cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x66181d3, SUC_lexicon_tags_0 },
    { 0x434181d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be181d7, SUC_lexicon_tags_25 },
    { 0x83c181d7, SUC_lexicon_tags_10 },
    { 0xb50181d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa181de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e181e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x172181e4, SUC_lexicon_tags_29 },
    { 0x4ba181e4, SUC_lexicon_tags_12 },
    { 0xc76181e6, SUC_lexicon_tags_8 },
    { 0x5be181e7, SUC_lexicon_tags_23 },
    { 0xe36181e8, SUC_lexicon_tags_17 },
    { 0x2aa181e9, SUC_lexicon_tags_17 },
    { 0x182181e9, SUC_lexicon_tags_2 },
    { 0x9c181eb, SUC_lexicon_tags_47 },
    { 0x64a181eb, SUC_lexicon_tags_27 },
    { 0xf2c181ec, SUC_lexicon_tags_31 },
    { 0x8ea181ee, SUC_lexicon_tags_11 },
    { 0x252181ef, SUC_lexicon_tags_0 },
    { 0x1a8181f0, SUC_lexicon_tags_351 },
    { 0xb2c181f1, SUC_lexicon_tags_37 },
    { 0xa52181f0, SUC_lexicon_tags_11 },
    { 0x694181f3, SUC_lexicon_tags_56 },
    { 0xfbc181f1, SUC_lexicon_tags_7 },
    { 0x112181f3, SUC_lexicon_tags_37 },
    { 0x578181f3, SUC_lexicon_tags_17 },
    { 0x7c4181f3, SUC_lexicon_tags_7 },
    { 0xf64181ed, SUC_lexicon_tags_15 },
    { 0xba0181f1, SUC_lexicon_tags_17 },
    { 0xe2c181fa, SUC_lexicon_tags_7 },
    { 0x520181ee, SUC_lexicon_tags_12 },
    { 0xad8181eb, SUC_lexicon_tags_3 },
    { 0x378181e4, SUC_lexicon_tags_33 },
    { 0x6ba181eb, SUC_lexicon_tags_11 },
    { 0x518181ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf3618201, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac18206, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79e18208, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6361820c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5218218, SUC_lexicon_tags_0 },
    { 0x5fa18219, SUC_lexicon_tags_11 },
    { 0xc9018219, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0a1821c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e18223, SUC_lexicon_tags_10 },
    { 0x2dc18224, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xce218227, SUC_lexicon_tags_56 },
    { 0xe0c18227, SUC_lexicon_tags_37 },
    { 0x86418228, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x78e1822b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbe1822d, SUC_lexicon_tags_0 },
    { 0x6dc1822d, SUC_lexicon_tags_14 },
    { 0x3781822f, SUC_lexicon_tags_17 },
    { 0xdc01822f, SUC_lexicon_tags_33 },
    { 0x8801822f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc18236, SUC_lexicon_tags_25 },
    { 0x97618237, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x75418239, SUC_lexicon_tags_13 },
    { 0xdae1823a, SUC_lexicon_tags_17 },
    { 0x33c1823b, SUC_lexicon_tags_2 },
    { 0x7ba18239, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6301823f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2a18241, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f018244, SUC_lexicon_tags_11 },
    { 0x90418244, SUC_lexicon_tags_15 },
    { 0x66c18245, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8dc18248, SUC_lexicon_tags_17 },
    { 0xae18249, SUC_lexicon_tags_77 },
    { 0xd781824a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9781824c, SUC_lexicon_tags_37 },
    { 0x22c1824d, SUC_lexicon_tags_31 },
    { 0xe9c1824e, SUC_lexicon_tags_0 },
    { 0xb701824f, SUC_lexicon_tags_19 },
    { 0x95e18250, SUC_lexicon_tags_43 },
    { 0xfca18251, SUC_lexicon_tags_28 },
    { 0xfc618252, SUC_lexicon_tags_63 },
    { 0xeb618252, SUC_lexicon_tags_17 },
    { 0x59018252, SUC_lexicon_tags_6 },
    { 0x83218255, SUC_lexicon_tags_37 },
    { 0x7e418254, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa18259, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabc1825b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x881825d, SUC_lexicon_tags_60 },
    { 0x1021825e, SUC_lexicon_tags_47 },
    { 0x7601825f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c218261, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe8818263, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa861826b, SUC_lexicon_tags_25 },
    { 0x2c1826b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a18272, SUC_lexicon_tags_27 },
    { 0xb8e18273, SUC_lexicon_tags_23 },
    { 0x9e418272, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a818276, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5818279, SUC_lexicon_tags_17 },
    { 0x8de1827a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1827e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa4018280, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83218282, SUC_lexicon_tags_29 },
    { 0xa8a18283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2c18285, SUC_lexicon_tags_10 },
    { 0xf8618286, SUC_lexicon_tags_25 },
    { 0x70c18287, SUC_lexicon_tags_27 },
    { 0x55c18287, SUC_lexicon_tags_12 },
    { 0x9d018289, SUC_lexicon_tags_57 },
    { 0x2fe1828a, SUC_lexicon_tags_49 },
    { 0xd9018287, SUC_lexicon_tags_1 },
    { 0xb4e1828c, SUC_lexicon_tags_37 },
    { 0x65e1828c, SUC_lexicon_tags_61 },
    { 0xea81828e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xe9a18290, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3618297, SUC_lexicon_tags_31 },
    { 0xd9818298, SUC_lexicon_tags_64 },
    { 0xfaa18297, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xed41829b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4081829e, SUC_lexicon_tags_17 },
    { 0xcec1829f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6182a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bc182ab, SUC_lexicon_tags_17 },
    { 0xfca182ab, SUC_lexicon_tags_1 },
    { 0x408182ab, SUC_lexicon_tags_2 },
    { 0x28182ae, SUC_lexicon_tags_71 },
    { 0xf02182ae, SUC_lexicon_tags_12 },
    { 0xc58182b0, SUC_lexicon_tags_47 },
    { 0x85a182ac, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2182b4, SUC_lexicon_tags_7 },
    { 0xdae182b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x302182b8, SUC_lexicon_tags_106 },
    { 0x2f2182b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a182bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc12182bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x268182c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356182c7, SUC_lexicon_tags_60 },
    { 0x42c182c8, SUC_lexicon_tags_188 },
    { 0xe32182c9, SUC_lexicon_tags_37 },
    { 0x1d6182c9, SUC_lexicon_tags_12 },
    { 0xd0182cb, SUC_lexicon_tags_17 },
    { 0x324182cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa182d0, SUC_lexicon_tags_27 },
    { 0xbe2182d1, SUC_lexicon_tags_23 },
    { 0x66c182d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x164182d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ce182d7, SUC_lexicon_tags_25 },
    { 0xa72182d7, SUC_lexicon_tags_17 },
    { 0xfac182d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8182dc, SUC_lexicon_tags_13 },
    { 0xf9e182dc, SUC_lexicon_tags_37 },
    { 0x5f4182de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84182e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x490182e4, SUC_lexicon_tags_63 },
    { 0xb60182e5, SUC_lexicon_tags_11 },
    { 0xc0e182e5, SUC_lexicon_tags_28 },
    { 0x30a182e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648182ec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x44c182ee, SUC_lexicon_tags_220 },
    { 0x560182ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x826182f2, SUC_lexicon_tags_1513 },
    { 0x228182f3, SUC_lexicon_tags_151 },
    { 0x79a182f3, SUC_lexicon_tags_140 },
    { 0xe38182f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe00182f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348182fc, SUC_lexicon_tags_37 },
    { 0xbfc182fd, SUC_lexicon_tags_65 },
    { 0x352182fd, SUC_lexicon_tags_87 },
    { 0x6f2182ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec218303, SUC_lexicon_tags_3 },
    { 0x83e18304, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b618307, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb01830d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa501830f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xede18312, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59018317, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6818319, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd441831d, SUC_lexicon_tags_28 },
    { 0x29e1831e, SUC_lexicon_tags_0 },
    { 0x8641831f, SUC_lexicon_tags_15 },
    { 0xb3a18320, SUC_lexicon_tags_17 },
    { 0x7d81831e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8a18323, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x39418325, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85818329, SUC_lexicon_tags_47 },
    { 0xebe1832a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1741832e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74218333, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c18336, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25218338, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58218341, SUC_lexicon_tags_12 },
    { 0xeb218342, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28c18344, SUC_lexicon_tags_3 },
    { 0x55a18345, SUC_lexicon_tags_68 },
    { 0xfee18346, SUC_lexicon_tags_17 },
    { 0xd4418345, SUC_lexicon_tags_26 },
    { 0x85e18344, SUC_lexicon_tags_29 },
    { 0xb2618349, SUC_lexicon_tags_8 },
    { 0x1fe18349, SUC_lexicon_tags_15 },
    { 0xd3218344, SUC_lexicon_tags_25 },
    { 0x8b218346, SUC_lexicon_tags_47 },
    { 0x9901834d, SUC_lexicon_tags_17 },
    { 0x3041834d, SUC_lexicon_tags_12 },
    { 0x7d01834b, SUC_lexicon_tags_25 },
    { 0xae18347, SUC_lexicon_tags_63 },
    { 0xe0818351, SUC_lexicon_tags_174 },
    { 0x3fa18352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe18356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c1835d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5c41835f, SUC_lexicon_tags_17 },
    { 0xa301835f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28618362, SUC_lexicon_tags_15 },
    { 0x87a18363, SUC_lexicon_tags_17 },
    { 0x42018363, SUC_lexicon_tags_0 },
    { 0xb2618363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c18368, SUC_lexicon_tags_7 },
    { 0x63218369, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc1836e, SUC_lexicon_tags_4 },
    { 0xc901836f, SUC_lexicon_tags_37 },
    { 0x42018370, SUC_lexicon_tags_29 },
    { 0x2b218371, SUC_lexicon_tags_12 },
    { 0xce818370, SUC_lexicon_tags_45 },
    { 0xaf218373, SUC_lexicon_tags_6 },
    { 0xebe18373, SUC_lexicon_tags_27 },
    { 0x57a18373, SUC_lexicon_tags_14 },
    { 0xb2218376, SUC_lexicon_tags_14 },
    { 0x27e18377, SUC_lexicon_tags_17 },
    { 0xc4e18378, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e01837b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a01837f, SUC_lexicon_tags_47 },
    { 0xa618380, SUC_lexicon_tags_25 },
    { 0xa2218381, SUC_lexicon_tags_60 },
    { 0x54a18382, SUC_lexicon_tags_29 },
    { 0xfc818382, SUC_lexicon_tags_37 },
    { 0x3a018382, SUC_lexicon_tags_37 },
    { 0x1bc18381, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22418387, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4361838e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58418390, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x31a18392, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf941839c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e183a1, SUC_lexicon_tags_61 },
    { 0xc1e183a1, SUC_lexicon_tags_17 },
    { 0x24183a3, SUC_lexicon_tags_62 },
    { 0xd96183a3, SUC_lexicon_tags_1 },
    { 0x4ca183a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c8183a7, SUC_lexicon_tags_9 },
    { 0x94183a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd40183aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ae183ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c183b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e183b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb76183b6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e183b9, SUC_lexicon_tags_38 },
    { 0x544183ba, SUC_lexicon_tags_14 },
    { 0x50a183bb, SUC_lexicon_tags_17 },
    { 0x17e183bc, SUC_lexicon_tags_27 },
    { 0x26183bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a183c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae183c4, SUC_lexicon_tags_26 },
    { 0x4d8183c5, SUC_lexicon_tags_8 },
    { 0x5d8183c6, SUC_lexicon_tags_31 },
    { 0xcde183c7, SUC_lexicon_tags_37 },
    { 0xb02183c8, SUC_lexicon_tags_17 },
    { 0x82e183c9, SUC_lexicon_tags_12 },
    { 0x810183c9, SUC_lexicon_tags_15 },
    { 0x63a183cb, SUC_lexicon_tags_23 },
    { 0x1de183cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4183cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c183d2, SUC_lexicon_tags_11 },
    { 0xc12183d3, SUC_lexicon_tags_47 },
    { 0xb4a183d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x51c183d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92183da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922183e2, SUC_lexicon_tags_0 },
    { 0x9c4183e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1c183e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x880183e7, SUC_lexicon_tags_37 },
    { 0x168183e8, SUC_lexicon_tags_15 },
    { 0xaba183e8, SUC_lexicon_tags_60 },
    { 0xf5e183ea, SUC_lexicon_tags_92 },
    { 0xfb8183eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc183ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2c183f1, SUC_lexicon_tags_7 },
    { 0xad2183f1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82183f5, SUC_lexicon_tags_14 },
    { 0x4fc183f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4d0183f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c183fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff018400, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10c18402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2601840d, SUC_lexicon_tags_65 },
    { 0x14c1840e, SUC_lexicon_tags_7 },
    { 0x8a01840f, SUC_lexicon_tags_5 },
    { 0xa841840e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc18415, SUC_lexicon_tags_14 },
    { 0x78218416, SUC_lexicon_tags_11 },
    { 0x10a18417, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x26618419, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4601841c, SUC_lexicon_tags_12 },
    { 0x7f01841d, SUC_lexicon_tags_45 },
    { 0xda21841e, SUC_lexicon_tags_23 },
    { 0x6621841e, SUC_lexicon_tags_39 },
    { 0x4721841e, SUC_lexicon_tags_7 },
    { 0x5161841c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c18428, SUC_lexicon_tags_25 },
    { 0xeb018429, SUC_lexicon_tags_27 },
    { 0xa9618428, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab81842c, SUC_lexicon_tags_38 },
    { 0x7ee1842d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94218432, SUC_lexicon_tags_37 },
    { 0xa0e18432, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc1c18435, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc241843b, SUC_lexicon_tags_10 },
    { 0x11c1843c, SUC_lexicon_tags_28 },
    { 0xa2e1843d, SUC_lexicon_tags_8 },
    { 0x1e01843e, SUC_lexicon_tags_5 },
    { 0x1aa1843f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7d618441, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f418443, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41818447, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8741844a, SUC_lexicon_tags_43 },
    { 0x4ce1844b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf81844e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92818450, SUC_lexicon_tags_15 },
    { 0x98218450, SUC_lexicon_tags_1 },
    { 0x29418451, SUC_lexicon_tags_12 },
    { 0x6f018452, SUC_lexicon_tags_8 },
    { 0xc7218452, SUC_lexicon_tags_49 },
    { 0x1ee18452, SUC_lexicon_tags_7 },
    { 0x8a018456, SUC_lexicon_tags_1 },
    { 0x78418455, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x15618459, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xeae1845b, SUC_lexicon_tags_7 },
    { 0x7b81845c, SUC_lexicon_tags_23 },
    { 0x8aa1845b, SUC_lexicon_tags_11 },
    { 0xdce1845b, SUC_lexicon_tags_6 },
    { 0xede1845f, SUC_lexicon_tags_25 },
    { 0x5461845f, SUC_lexicon_tags_15 },
    { 0x99818461, SUC_lexicon_tags_7 },
    { 0x19218461, SUC_lexicon_tags_0 },
    { 0x18c18463, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x37c18465, SUC_lexicon_tags_12 },
    { 0x25e18466, SUC_lexicon_tags_11 },
    { 0xc18467, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3ac18469, SUC_lexicon_tags_33 },
    { 0x2418469, SUC_lexicon_tags_11 },
    { 0x9ba1846b, SUC_lexicon_tags_26 },
    { 0xc941846c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc1846f, SUC_lexicon_tags_17 },
    { 0xb3a1846f, SUC_lexicon_tags_12 },
    { 0x4da18471, SUC_lexicon_tags_13 },
    { 0x79218472, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44018475, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9ac18477, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd3018479, SUC_lexicon_tags_47 },
    { 0xb4e18479, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf921847c, SUC_lexicon_tags_27 },
    { 0xb21847d, SUC_lexicon_tags_45 },
    { 0x3641847e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca418489, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xde01848c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6818496, SUC_lexicon_tags_15 },
    { 0xbb218496, SUC_lexicon_tags_21 },
    { 0x6c618496, SUC_lexicon_tags_35 },
    { 0x7ee18499, SUC_lexicon_tags_15 },
    { 0x3618498, SUC_lexicon_tags_1 },
    { 0x3ae1849b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e1849e, SUC_lexicon_tags_68 },
    { 0xf3c1849e, SUC_lexicon_tags_14 },
    { 0xb0184a0, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0184a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea184a7, SUC_lexicon_tags_68 },
    { 0xdb2184a7, SUC_lexicon_tags_45 },
    { 0xb7c184a7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xe66184ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a184b1, SUC_lexicon_tags_37 },
    { 0x486184b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc32184b4, SUC_lexicon_tags_1 },
    { 0x508184b4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x174184b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12184b9, SUC_lexicon_tags_37 },
    { 0x14184ba, SUC_lexicon_tags_1 },
    { 0xdd8184bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf02184bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd5c184bf, SUC_lexicon_tags_26 },
    { 0x47e184c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e184c7, SUC_lexicon_tags_2 },
    { 0x7bc184c8, SUC_lexicon_tags_35 },
    { 0xa0e184c7, SUC_lexicon_tags_7 },
    { 0x2b4184ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc20184cc, SUC_lexicon_tags_37 },
    { 0x246184cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28184d5, SUC_lexicon_tags_17 },
    { 0x718184d6, SUC_lexicon_tags_79 },
    { 0x562184d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x110184da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc2184dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02184e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a184e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a184ea, SUC_lexicon_tags_0 },
    { 0xc2a184eb, SUC_lexicon_tags_11 },
    { 0x3e2184ec, SUC_lexicon_tags_17 },
    { 0xb46184ed, SUC_lexicon_tags_7 },
    { 0x328184ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c184f1, SUC_lexicon_tags_17 },
    { 0x6cc184f1, SUC_lexicon_tags_1 },
    { 0x908184f1, SUC_lexicon_tags_7 },
    { 0xc60184f1, SUC_lexicon_tags_12 },
    { 0x54a184f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c184f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x348184fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac18506, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f818508, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741850e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58c18510, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff218514, SUC_lexicon_tags_1 },
    { 0x12418515, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6261851a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81851f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd018523, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea18527, SUC_lexicon_tags_0 },
    { 0x63418527, SUC_lexicon_tags_12 },
    { 0x40a18527, SUC_lexicon_tags_26 },
    { 0xb5e1852a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63618531, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf4e18533, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x97418535, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x81418537, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5541853e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e18546, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4218549, SUC_lexicon_tags_38 },
    { 0xe7818549, SUC_lexicon_tags_2 },
    { 0x36a1854b, SUC_lexicon_tags_2 },
    { 0xc8e1854b, SUC_lexicon_tags_8 },
    { 0x3ba1854c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba018554, SUC_lexicon_tags_25 },
    { 0xd2418555, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72418558, SUC_lexicon_tags_33 },
    { 0x7fa18558, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc261855e, SUC_lexicon_tags_15 },
    { 0xbea1855e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e18562, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9a18564, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4501856a, SUC_lexicon_tags_63 },
    { 0x5ae1856b, SUC_lexicon_tags_12 },
    { 0x9be1856b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e01856e, SUC_lexicon_tags_33 },
    { 0x7241856f, SUC_lexicon_tags_1 },
    { 0x63418570, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7a18572, SUC_lexicon_tags_0 },
    { 0x88a18573, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e218576, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881857a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7be1857c, SUC_lexicon_tags_15 },
    { 0xf361857c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72218580, SUC_lexicon_tags_17 },
    { 0x1ee18580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fa18583, SUC_lexicon_tags_12 },
    { 0xd4e18583, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e1858c, SUC_lexicon_tags_21 },
    { 0x44e1858d, SUC_lexicon_tags_29 },
    { 0xad21858d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be18591, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86818595, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c18598, SUC_lexicon_tags_14 },
    { 0x6618599, SUC_lexicon_tags_45 },
    { 0xb9a1859a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba185a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfe185a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0c185a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452185a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04185ad, SUC_lexicon_tags_104 },
    { 0x546185ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa7e185b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe185b6, SUC_lexicon_tags_7 },
    { 0x332185b7, SUC_lexicon_tags_57 },
    { 0x4da185b8, SUC_lexicon_tags_30 },
    { 0x27a185b9, SUC_lexicon_tags_14 },
    { 0xdbc185ba, SUC_lexicon_tags_49 },
    { 0x830185bb, SUC_lexicon_tags_30 },
    { 0x6a8185ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6185bf, SUC_lexicon_tags_11 },
    { 0xf0a185bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac185c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe185c8, SUC_lexicon_tags_17 },
    { 0xdd0185c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38185cf, SUC_lexicon_tags_12 },
    { 0x412185cf, SUC_lexicon_tags_65 },
    { 0xc24185d0, SUC_lexicon_tags_31 },
    { 0xcd6185d2, SUC_lexicon_tags_14 },
    { 0xf72185d0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6185d6, SUC_lexicon_tags_4 },
    { 0x53a185d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40185da, SUC_lexicon_tags_11 },
    { 0xb7c185db, SUC_lexicon_tags_2 },
    { 0x5c2185da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60185df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12185e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x14185e6, SUC_lexicon_tags_25 },
    { 0x2d6185e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a185ea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbf2185ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d2185ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88185f3, SUC_lexicon_tags_11 },
    { 0x392185f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca185f7, SUC_lexicon_tags_11 },
    { 0x1ce185f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee185fb, SUC_lexicon_tags_29 },
    { 0xb04185fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba418601, SUC_lexicon_tags_17 },
    { 0x55018602, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0818605, SUC_lexicon_tags_28 },
    { 0x43418606, SUC_lexicon_tags_12 },
    { 0x47e18607, SUC_lexicon_tags_0 },
    { 0xadc18608, SUC_lexicon_tags_8 },
    { 0xdc18609, SUC_lexicon_tags_43 },
    { 0x9ea18608, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1501860d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c18614, SUC_lexicon_tags_17 },
    { 0xba018615, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc2c18617, SUC_lexicon_tags_0 },
    { 0x4e018617, SUC_lexicon_tags_14 },
    { 0x51a18619, SUC_lexicon_tags_6 },
    { 0x3381861a, SUC_lexicon_tags_65 },
    { 0x83a18619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c1861e, SUC_lexicon_tags_19 },
    { 0x36c1861e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5418624, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x84a18626, SUC_lexicon_tags_17 },
    { 0xc9018626, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a1862a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3061862c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43818632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d018639, SUC_lexicon_tags_21 },
    { 0xd4e18639, SUC_lexicon_tags_38 },
    { 0xf2018639, SUC_lexicon_tags_12 },
    { 0x1c41863c, SUC_lexicon_tags_15 },
    { 0x6a1863d, SUC_lexicon_tags_57 },
    { 0x72a1863e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72c18640, SUC_lexicon_tags_25 },
    { 0xe618641, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c18644, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb418647, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda1864d, SUC_lexicon_tags_2 },
    { 0x2a61864e, SUC_lexicon_tags_11 },
    { 0x57c1864e, SUC_lexicon_tags_63 },
    { 0x98e18650, SUC_lexicon_tags_6 },
    { 0xd641864e, SUC_lexicon_tags_23 },
    { 0x25c18652, SUC_lexicon_tags_31 },
    { 0x2e618650, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec18656, SUC_lexicon_tags_9 },
    { 0xff818657, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe1865b, SUC_lexicon_tags_151 },
    { 0xb641865c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a41865f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb9c18661, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e18667, SUC_lexicon_tags_15 },
    { 0x76418668, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a18671, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5ce18673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f818675, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6641867b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe21867d, SUC_lexicon_tags_95 },
    { 0xa081867d, SUC_lexicon_tags_34 },
    { 0x3581867d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8018684, SUC_lexicon_tags_17 },
    { 0x40418685, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a18688, SUC_lexicon_tags_12 },
    { 0x4c218689, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21868d, SUC_lexicon_tags_7 },
    { 0x4881868e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87018690, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1618694, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x34818697, SUC_lexicon_tags_32 },
    { 0x8f618698, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb01869f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2186a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8186a7, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xc9e186a9, SUC_lexicon_tags_45 },
    { 0xfea186a9, SUC_lexicon_tags_37 },
    { 0xd28186ab, SUC_lexicon_tags_27 },
    { 0xfe2186ab, SUC_lexicon_tags_12 },
    { 0x40e186ad, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x112186b0, SUC_lexicon_tags_9 },
    { 0x86186b1, SUC_lexicon_tags_27 },
    { 0xa92186b0, SUC_lexicon_tags_31 },
    { 0xe7e186b1, SUC_lexicon_tags_3 },
    { 0xa4186b4, SUC_lexicon_tags_101 },
    { 0xb2c186b4, SUC_lexicon_tags_11 },
    { 0xc08186b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00186c4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x35a186c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdac186c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da186cb, SUC_lexicon_tags_1 },
    { 0x158186cc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde186cf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x560186d2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48186d7, SUC_lexicon_tags_108 },
    { 0x26c186d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfe8186da, SUC_lexicon_tags_9 },
    { 0x55a186db, SUC_lexicon_tags_10 },
    { 0x116186da, SUC_lexicon_tags_17 },
    { 0xe3a186dd, SUC_lexicon_tags_8 },
    { 0x80e186de, SUC_lexicon_tags_37 },
    { 0x854186dd, SUC_lexicon_tags_11 },
    { 0x2e186e0, SUC_lexicon_tags_7 },
    { 0x4b4186e0, SUC_lexicon_tags_37 },
    { 0xc7c186e2, SUC_lexicon_tags_15 },
    { 0x334186e3, SUC_lexicon_tags_17 },
    { 0x1c8186e4, SUC_lexicon_tags_19 },
    { 0xe5e186e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2a186e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886186ec, SUC_lexicon_tags_2 },
    { 0x84186ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x400186ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce186f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a186f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x416186f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce6186fb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x35a186fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a18703, SUC_lexicon_tags_2 },
    { 0x43418704, SUC_lexicon_tags_3 },
    { 0xeba18705, SUC_lexicon_tags_13 },
    { 0x1cc18703, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde018709, SUC_lexicon_tags_27 },
    { 0x58a1870a, SUC_lexicon_tags_7 },
    { 0x1701870b, SUC_lexicon_tags_26 },
    { 0xde21870b, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a1870f, SUC_lexicon_tags_11 },
    { 0x3ae18710, SUC_lexicon_tags_2 },
    { 0xb2618710, SUC_lexicon_tags_14 },
    { 0x20a18712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83618714, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3618717, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x81818719, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4341871b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24618722, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a018725, SUC_lexicon_tags_45 },
    { 0xce218726, SUC_lexicon_tags_23 },
    { 0x2d218727, SUC_lexicon_tags_77 },
    { 0x8e218728, SUC_lexicon_tags_26 },
    { 0x33418728, SUC_lexicon_tags_61 },
    { 0xb601872a, SUC_lexicon_tags_28 },
    { 0x74a1872b, SUC_lexicon_tags_37 },
    { 0x5b81872a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2281872e, SUC_lexicon_tags_11 },
    { 0xd3a1872f, SUC_lexicon_tags_4 },
    { 0x6821872e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83218732, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c18735, SUC_lexicon_tags_30 },
    { 0x5f018736, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70218738, SUC_lexicon_tags_11 },
    { 0x6b818738, SUC_lexicon_tags_33 },
    { 0x4b41873a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc061873d, SUC_lexicon_tags_59 },
    { 0x2781873e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85c18740, SUC_lexicon_tags_16 },
    { 0xbb018741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8018746, SUC_lexicon_tags_32 },
    { 0x17a18747, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d01874d, SUC_lexicon_tags_19 },
    { 0xfbc1874e, SUC_lexicon_tags_37 },
    { 0x3a61874f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2cc18751, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6a18753, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe18757, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7c418759, SUC_lexicon_tags_23 },
    { 0xba018759, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c1875d, SUC_lexicon_tags_12 },
    { 0x6401875e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e18761, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20218767, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8da18769, SUC_lexicon_tags_12 },
    { 0x3f21876a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9261876c, SUC_lexicon_tags_87 },
    { 0x51c1876d, SUC_lexicon_tags_29 },
    { 0x5f21876e, SUC_lexicon_tags_45 },
    { 0x18e1876f, SUC_lexicon_tags_20 },
    { 0x26c1876f, SUC_lexicon_tags_15 },
    { 0xde218770, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e218774, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xba18776, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e418778, SUC_lexicon_tags_37 },
    { 0x49218778, SUC_lexicon_tags_49 },
    { 0xb5418779, SUC_lexicon_tags_37 },
    { 0x7da1877b, SUC_lexicon_tags_0 },
    { 0x88c1877c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9ee1877e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xca418780, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d018783, SUC_lexicon_tags_29 },
    { 0xc2c18784, SUC_lexicon_tags_23 },
    { 0xc6418784, SUC_lexicon_tags_14 },
    { 0x41e18784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8218789, SUC_lexicon_tags_26 },
    { 0xcdc1878a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae18791, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f018793, SUC_lexicon_tags_37 },
    { 0x8f018793, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e1879d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e01879f, SUC_lexicon_tags_33 },
    { 0x272187a0, SUC_lexicon_tags_29 },
    { 0x5ec187a1, SUC_lexicon_tags_17 },
    { 0xd8a187a2, SUC_lexicon_tags_9 },
    { 0x5d2187a3, SUC_lexicon_tags_11 },
    { 0xe90187a0, SUC_lexicon_tags_12 },
    { 0x74a187a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x968187a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48187ab, SUC_lexicon_tags_38 },
    { 0x444187ac, SUC_lexicon_tags_12 },
    { 0xd98187ad, SUC_lexicon_tags_14 },
    { 0x78c187ab, SUC_lexicon_tags_14 },
    { 0x490187af, SUC_lexicon_tags_11 },
    { 0xca2187ab, SUC_lexicon_tags_23 },
    { 0x990187b1, SUC_lexicon_tags_8 },
    { 0x14187ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x78a187b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x542187b6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0187b9, SUC_lexicon_tags_25 },
    { 0xf96187b9, SUC_lexicon_tags_77 },
    { 0xdae187bb, SUC_lexicon_tags_25 },
    { 0x26e187bc, SUC_lexicon_tags_37 },
    { 0xf40187ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b8187bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6187c2, SUC_lexicon_tags_37 },
    { 0xf66187c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe30187c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2187c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe06187cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae187d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54187d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a187d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x432187db, SUC_lexicon_tags_37 },
    { 0x710187dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0187e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c187e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930187e9, SUC_lexicon_tags_33 },
    { 0xc3a187ea, SUC_lexicon_tags_15 },
    { 0xa28187eb, SUC_lexicon_tags_37 },
    { 0x4e4187e9, SUC_lexicon_tags_14 },
    { 0xc5c187ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x566187ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc30187f1, SUC_lexicon_tags_3 },
    { 0x616187f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c4187f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae187f8, SUC_lexicon_tags_15 },
    { 0xf9e187f9, SUC_lexicon_tags_11 },
    { 0x6c6187fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96187fe, SUC_lexicon_tags_9 },
    { 0x2ce187ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e18804, SUC_lexicon_tags_39 },
    { 0x52418805, SUC_lexicon_tags_11 },
    { 0x8618806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25418809, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1880e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b418812, SUC_lexicon_tags_29 },
    { 0x93218813, SUC_lexicon_tags_127 },
    { 0xf4e18812, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e1881d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc18820, SUC_lexicon_tags_23 },
    { 0xb1618820, SUC_lexicon_tags_17 },
    { 0xf0218822, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a18826, SUC_lexicon_tags_34 },
    { 0xd6618827, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x85418829, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41618830, SUC_lexicon_tags_6 },
    { 0x1cc18830, SUC_lexicon_tags_25 },
    { 0xd0a18830, SUC_lexicon_tags_138 },
    { 0xbe218831, SUC_lexicon_tags_17 },
    { 0xd5818834, SUC_lexicon_tags_25 },
    { 0x93c18832, SUC_lexicon_tags_37 },
    { 0x3ca18836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88018838, SUC_lexicon_tags_63 },
    { 0xee218839, SUC_lexicon_tags_11 },
    { 0x97a1883a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf041883c, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe618841, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2618846, SUC_lexicon_tags_9 },
    { 0x2b818847, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21884b, SUC_lexicon_tags_16 },
    { 0x3521884b, SUC_lexicon_tags_74 },
    { 0x9601884c, SUC_lexicon_tags_2 },
    { 0xf7a1884e, SUC_lexicon_tags_7 },
    { 0xc6e1884f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c18854, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2c18856, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a618858, SUC_lexicon_tags_1 },
    { 0x1f818858, SUC_lexicon_tags_7 },
    { 0x5d418859, SUC_lexicon_tags_28 },
    { 0x79418859, SUC_lexicon_tags_61 },
    { 0x43a1885c, SUC_lexicon_tags_80 },
    { 0xa41885d, SUC_lexicon_tags_133 },
    { 0x4081885d, SUC_lexicon_tags_10 },
    { 0xb7018859, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa218865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be1886d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab818872, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x99618875, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8da18877, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe1887a, SUC_lexicon_tags_45 },
    { 0xca41887b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e18887, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561888c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62618892, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca1889a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd981889c, SUC_lexicon_tags_35 },
    { 0xfa41889d, SUC_lexicon_tags_43 },
    { 0xa121889e, SUC_lexicon_tags_43 },
    { 0x78c1889c, SUC_lexicon_tags_113 },
    { 0x2361889e, SUC_lexicon_tags_0 },
    { 0x95e188a1, SUC_lexicon_tags_23 },
    { 0x5ba188a2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e188a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8dc188a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e0188a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x512188ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x108188ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6188b1, SUC_lexicon_tags_11 },
    { 0x6f8188b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74188b7, SUC_lexicon_tags_60 },
    { 0xd0a188b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38188bd, SUC_lexicon_tags_39 },
    { 0x2da188be, SUC_lexicon_tags_14 },
    { 0xbf4188bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90188c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e188c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4188cd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52188d2, SUC_lexicon_tags_7 },
    { 0xcec188d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe02188d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48188d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20188db, SUC_lexicon_tags_37 },
    { 0x9ae188dc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa64188de, SUC_lexicon_tags_48 },
    { 0xdf6188df, SUC_lexicon_tags_14 },
    { 0x7a4188e0, SUC_lexicon_tags_4 },
    { 0x3d8188df, SUC_lexicon_tags_7 },
    { 0xaa188df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x596188e5, SUC_lexicon_tags_12 },
    { 0x17e188e5, SUC_lexicon_tags_15 },
    { 0xb5e188e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaaa188e9, SUC_lexicon_tags_25 },
    { 0x27e188e9, SUC_lexicon_tags_15 },
    { 0xb76188eb, SUC_lexicon_tags_9 },
    { 0x6ea188ec, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898188f1, SUC_lexicon_tags_17 },
    { 0x434188f2, SUC_lexicon_tags_14 },
    { 0x42c188f1, SUC_lexicon_tags_1 },
    { 0xcc6188f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x710188f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82c188f9, SUC_lexicon_tags_47 },
    { 0xbdc188fa, SUC_lexicon_tags_7 },
    { 0x338188fb, SUC_lexicon_tags_1 },
    { 0x4f4188fc, SUC_lexicon_tags_23 },
    { 0x22a188fc, SUC_lexicon_tags_11 },
    { 0xc72188fb, SUC_lexicon_tags_39 },
    { 0xc80188ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce818901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc18907, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea1890a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1321890f, SUC_lexicon_tags_15 },
    { 0x9c018910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e418913, SUC_lexicon_tags_17 },
    { 0x16a18914, SUC_lexicon_tags_1296 },
    { 0, NULL },
    { 0, NULL },
    { 0xee018917, SUC_lexicon_tags_37 },
    { 0x85a18917, SUC_lexicon_tags_1 },
    { 0x7ec18919, SUC_lexicon_tags_27 },
    { 0x58c18919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea1891e, SUC_lexicon_tags_29 },
    { 0xff61891f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1218923, SUC_lexicon_tags_11 },
    { 0x1e418923, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18218929, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa1892c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4281892f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf9818931, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e818938, SUC_lexicon_tags_273 },
    { 0xaee18939, SUC_lexicon_tags_3 },
    { 0xdb21893a, SUC_lexicon_tags_25 },
    { 0x8e41893a, SUC_lexicon_tags_27 },
    { 0x9821893c, SUC_lexicon_tags_1 },
    { 0x1221893a, SUC_lexicon_tags_19 },
    { 0xa721893b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba418948, SUC_lexicon_tags_17 },
    { 0x17a18949, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e81894b, SUC_lexicon_tags_63 },
    { 0x2f01894c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7941894e, SUC_lexicon_tags_48 },
    { 0x83c1894e, SUC_lexicon_tags_21 },
    { 0x64c18950, SUC_lexicon_tags_47 },
    { 0x31c1894f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8e18953, SUC_lexicon_tags_38 },
    { 0xb2418954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdee18956, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x74a18958, SUC_lexicon_tags_2 },
    { 0xb1a18959, SUC_lexicon_tags_12 },
    { 0xa9818958, SUC_lexicon_tags_17 },
    { 0x5c01895b, SUC_lexicon_tags_1 },
    { 0x7a1895c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3a01895e, SUC_lexicon_tags_48 },
    { 0xce21895f, SUC_lexicon_tags_14 },
    { 0xbd41895f, SUC_lexicon_tags_37 },
    { 0x88c18960, SUC_lexicon_tags_15 },
    { 0x69618961, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48618967, SUC_lexicon_tags_72 },
    { 0x23a18968, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c1896b, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e21896e, SUC_lexicon_tags_47 },
    { 0x22a1896e, SUC_lexicon_tags_79 },
    { 0xfb818970, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3618972, SUC_lexicon_tags_63 },
    { 0xf0a18973, SUC_lexicon_tags_299 },
    { 0x44e18974, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81897d, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xcc1897f, SUC_lexicon_tags_37 },
    { 0xe0a1897f, SUC_lexicon_tags_27 },
    { 0x26c18980, SUC_lexicon_tags_49 },
    { 0x3ac1897f, SUC_lexicon_tags_47 },
    { 0x44c18983, SUC_lexicon_tags_23 },
    { 0x40018980, SUC_lexicon_tags_7 },
    { 0x4221897f, SUC_lexicon_tags_7 },
    { 0x90e18980, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc18989, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f21898c, SUC_lexicon_tags_8 },
    { 0xeda1898d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35618990, SUC_lexicon_tags_15 },
    { 0x80218991, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe9e18993, SUC_lexicon_tags_12 },
    { 0x80c18993, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8a18996, SUC_lexicon_tags_15 },
    { 0x96018997, SUC_lexicon_tags_1 },
    { 0xde418998, SUC_lexicon_tags_25 },
    { 0x85e18997, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1899c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8189a1, SUC_lexicon_tags_17 },
    { 0x97c189a2, SUC_lexicon_tags_60 },
    { 0x76a189a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0189a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46189ae, SUC_lexicon_tags_0 },
    { 0x940189af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae189ba, SUC_lexicon_tags_37 },
    { 0x220189ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe189be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58189c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8189c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x652189c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0189cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86189d6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6189da, SUC_lexicon_tags_60 },
    { 0x4fe189da, SUC_lexicon_tags_17 },
    { 0x22e189dc, SUC_lexicon_tags_17 },
    { 0x818189dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6e189df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf8189e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4189e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112189e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x234189ea, SUC_lexicon_tags_15 },
    { 0xcf4189eb, SUC_lexicon_tags_1 },
    { 0x8c189ec, SUC_lexicon_tags_11 },
    { 0x514189ed, SUC_lexicon_tags_60 },
    { 0x1d2189ed, SUC_lexicon_tags_1 },
    { 0xf66189eb, SUC_lexicon_tags_37 },
    { 0x398189f0, SUC_lexicon_tags_16 },
    { 0x3f0189ec, SUC_lexicon_tags_14 },
    { 0xcc6189ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2a189f4, SUC_lexicon_tags_24 },
    { 0xd30189f5, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x738189f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48a189f9, SUC_lexicon_tags_89 },
    { 0x7b4189f9, SUC_lexicon_tags_45 },
    { 0x3b8189fb, SUC_lexicon_tags_17 },
    { 0xf80189fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x242189ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8f618a01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99818a04, SUC_lexicon_tags_21 },
    { 0xf6018a05, SUC_lexicon_tags_9 },
    { 0x43418a06, SUC_lexicon_tags_1 },
    { 0xf9e18a07, SUC_lexicon_tags_17 },
    { 0xcb218a05, SUC_lexicon_tags_11 },
    { 0xb2018a09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4e18a0b, SUC_lexicon_tags_47 },
    { 0x6d418a0b, SUC_lexicon_tags_25 },
    { 0xefc18a0d, SUC_lexicon_tags_24 },
    { 0xf418a0d, SUC_lexicon_tags_28 },
    { 0x45a18a0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4218a12, SUC_lexicon_tags_17 },
    { 0x58a18a12, SUC_lexicon_tags_38 },
    { 0xbf418a13, SUC_lexicon_tags_25 },
    { 0x79618a15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e18a18, SUC_lexicon_tags_37 },
    { 0x92818a19, SUC_lexicon_tags_28 },
    { 0x4f818a19, SUC_lexicon_tags_25 },
    { 0xb7418a1b, SUC_lexicon_tags_1 },
    { 0x32418a18, SUC_lexicon_tags_133 },
    { 0xe2418a1d, SUC_lexicon_tags_25 },
    { 0x69218a19, SUC_lexicon_tags_15 },
    { 0xab418a1f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9c018a21, SUC_lexicon_tags_17 },
    { 0x8c618a22, SUC_lexicon_tags_13 },
    { 0x4ce18a23, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c18a26, SUC_lexicon_tags_17 },
    { 0x95618a26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28418a35, SUC_lexicon_tags_10 },
    { 0x91418a36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a18a3a, SUC_lexicon_tags_29 },
    { 0x32c18a3b, SUC_lexicon_tags_84 },
    { 0xcee18a3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad418a42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x36e18a44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e418a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26c18a49, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87618a56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a18a5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec018a5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d218a61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81018a64, SUC_lexicon_tags_23 },
    { 0x47e18a64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70018a68, SUC_lexicon_tags_37 },
    { 0x3ea18a69, SUC_lexicon_tags_47 },
    { 0xdf418a69, SUC_lexicon_tags_11 },
    { 0x95e18a6b, SUC_lexicon_tags_47 },
    { 0xd8618a6b, SUC_lexicon_tags_17 },
    { 0xada18a6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c18a6f, SUC_lexicon_tags_9 },
    { 0xb3218a70, SUC_lexicon_tags_60 },
    { 0xf2e18a6f, SUC_lexicon_tags_12 },
    { 0x9a018a72, SUC_lexicon_tags_70 },
    { 0x4d618a72, SUC_lexicon_tags_15 },
    { 0x24618a74, SUC_lexicon_tags_19 },
    { 0xcd418a72, SUC_lexicon_tags_23 },
    { 0xdfa18a71, SUC_lexicon_tags_17 },
    { 0x58018a77, SUC_lexicon_tags_9 },
    { 0xb218a78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61a18a7a, SUC_lexicon_tags_6 },
    { 0x2ae18a7a, SUC_lexicon_tags_2 },
    { 0x9ec18a7c, SUC_lexicon_tags_0 },
    { 0xc6018a7c, SUC_lexicon_tags_23 },
    { 0xbe818a7d, SUC_lexicon_tags_17 },
    { 0x3ae18a7f, SUC_lexicon_tags_25 },
    { 0x55018a80, SUC_lexicon_tags_11 },
    { 0xf6218a81, SUC_lexicon_tags_2 },
    { 0x24418a7f, SUC_lexicon_tags_45 },
    { 0x31618a7c, SUC_lexicon_tags_124 },
    { 0xe8c18a84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47e18a86, SUC_lexicon_tags_17 },
    { 0x33618a87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37818a8d, SUC_lexicon_tags_2 },
    { 0x2ae18a8e, SUC_lexicon_tags_1 },
    { 0xa0618a8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3418a92, SUC_lexicon_tags_46 },
    { 0x45218a93, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcf218a95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3618a9b, SUC_lexicon_tags_37 },
    { 0x39618a9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dc18a9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40818aa4, SUC_lexicon_tags_0 },
    { 0x8c618aa4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x52018aa7, SUC_lexicon_tags_45 },
    { 0xa7818aa8, SUC_lexicon_tags_79 },
    { 0xbfa18aa9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x37e18aab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17c18aad, SUC_lexicon_tags_17 },
    { 0xaf818aad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec18ab3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xce618ab5, SUC_lexicon_tags_10 },
    { 0x8e218ab6, SUC_lexicon_tags_49 },
    { 0x78018ab6, SUC_lexicon_tags_11 },
    { 0xcec18ab8, SUC_lexicon_tags_15 },
    { 0xd8c18ab9, SUC_lexicon_tags_7 },
    { 0xef018ab9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc18abe, SUC_lexicon_tags_26 },
    { 0xb0218abf, SUC_lexicon_tags_12 },
    { 0x15a18abf, SUC_lexicon_tags_12 },
    { 0xdda18ac0, SUC_lexicon_tags_25 },
    { 0x8b418ac0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e18ac8, SUC_lexicon_tags_23 },
    { 0xe2a18ac9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce618ad0, SUC_lexicon_tags_61 },
    { 0xeb218ad1, SUC_lexicon_tags_60 },
    { 0x20c18ad2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76018ad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa18ae2, SUC_lexicon_tags_2 },
    { 0xd3e18ae3, SUC_lexicon_tags_19 },
    { 0xf6c18ae2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc18ae7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41018aef, SUC_lexicon_tags_27 },
    { 0x67c18aef, SUC_lexicon_tags_7 },
    { 0xd3818aef, SUC_lexicon_tags_26 },
    { 0x1a18af0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14218af4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3e618af6, SUC_lexicon_tags_29 },
    { 0x57018af7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc18afa, SUC_lexicon_tags_12 },
    { 0x27e18afb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5418b07, SUC_lexicon_tags_37 },
    { 0xcba18b07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a18b0b, SUC_lexicon_tags_38 },
    { 0x3c618b0c, SUC_lexicon_tags_87 },
    { 0x3c818b0d, SUC_lexicon_tags_12 },
    { 0xc0018b0e, SUC_lexicon_tags_14 },
    { 0xf8818b0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x22818b11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98a18b13, SUC_lexicon_tags_49 },
    { 0xc1418b13, SUC_lexicon_tags_9 },
    { 0xba418b15, SUC_lexicon_tags_43 },
    { 0x4be18b16, SUC_lexicon_tags_30 },
    { 0x4a018b17, SUC_lexicon_tags_23 },
    { 0x48c18b15, SUC_lexicon_tags_47 },
    { 0x34a18b19, SUC_lexicon_tags_187 },
    { 0xeba18b1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e18b1f, SUC_lexicon_tags_84 },
    { 0xdf418b20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c18b27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f418b30, SUC_lexicon_tags_15 },
    { 0x66018b30, SUC_lexicon_tags_21 },
    { 0xc0018b32, SUC_lexicon_tags_4 },
    { 0x48018b31, SUC_lexicon_tags_8 },
    { 0x4d818b32, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6fe18b36, SUC_lexicon_tags_27 },
    { 0x54418b37, SUC_lexicon_tags_63 },
    { 0xc4018b38, SUC_lexicon_tags_7 },
    { 0xdba18b39, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1ec18b3b, SUC_lexicon_tags_15 },
    { 0xf3818b3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd218b3f, SUC_lexicon_tags_23 },
    { 0xae418b40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6818b42, SUC_lexicon_tags_31 },
    { 0x70c18b43, SUC_lexicon_tags_25 },
    { 0x6c18b42, SUC_lexicon_tags_4 },
    { 0x73818b45, SUC_lexicon_tags_15 },
    { 0x68a18b45, SUC_lexicon_tags_1 },
    { 0x97c18b46, SUC_lexicon_tags_45 },
    { 0x80a18b45, SUC_lexicon_tags_10 },
    { 0xdf218b49, SUC_lexicon_tags_23 },
    { 0xeb218b42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x56218b4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45018b51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5c218b53, SUC_lexicon_tags_91 },
    { 0x67218b54, SUC_lexicon_tags_25 },
    { 0xbfa18b55, SUC_lexicon_tags_45 },
    { 0xfee18b53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4818b59, SUC_lexicon_tags_37 },
    { 0x73e18b5a, SUC_lexicon_tags_60 },
    { 0x4c018b5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee18b60, SUC_lexicon_tags_7 },
    { 0xac818b61, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a18b64, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9418b66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7c18b68, SUC_lexicon_tags_4 },
    { 0x5de18b69, SUC_lexicon_tags_60 },
    { 0xe5818b6a, SUC_lexicon_tags_10 },
    { 0xd1218b69, SUC_lexicon_tags_3 },
    { 0xc6c18b6c, SUC_lexicon_tags_9 },
    { 0xee218b6d, SUC_lexicon_tags_9 },
    { 0x67018b6e, SUC_lexicon_tags_17 },
    { 0xc1c18b6c, SUC_lexicon_tags_12 },
    { 0x80818b6c, SUC_lexicon_tags_11 },
    { 0x6d618b6d, SUC_lexicon_tags_25 },
    { 0x78618b72, SUC_lexicon_tags_15 },
    { 0x5e018b73, SUC_lexicon_tags_47 },
    { 0x4f418b74, SUC_lexicon_tags_13 },
    { 0x9c18b75, SUC_lexicon_tags_66 },
    { 0x21a18b76, SUC_lexicon_tags_26 },
    { 0x86618b77, SUC_lexicon_tags_23 },
    { 0x1f018b76, SUC_lexicon_tags_45 },
    { 0x20218b79, SUC_lexicon_tags_27 },
    { 0x6c418b79, SUC_lexicon_tags_12 },
    { 0xa0818b79, SUC_lexicon_tags_37 },
    { 0x43618b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0c18b7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d418b80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59618b82, SUC_lexicon_tags_32 },
    { 0x30418b82, SUC_lexicon_tags_37 },
    { 0x9a818b84, SUC_lexicon_tags_21 },
    { 0x1b818b83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a18b89, SUC_lexicon_tags_2 },
    { 0x8be18b8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b618b8f, SUC_lexicon_tags_15 },
    { 0x46618b90, SUC_lexicon_tags_43 },
    { 0x6a418b91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52618b94, SUC_lexicon_tags_32 },
    { 0x7b018b95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ea18b97, SUC_lexicon_tags_1 },
    { 0x76418b97, SUC_lexicon_tags_15 },
    { 0x30618b98, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a18b9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8a018b9e, SUC_lexicon_tags_15 },
    { 0xa9218b9f, SUC_lexicon_tags_14 },
    { 0x4c018ba0, SUC_lexicon_tags_27 },
    { 0xbca18ba0, SUC_lexicon_tags_46 },
    { 0x8da18ba1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1618ba9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28418bac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c18baf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde18bb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f818bb9, SUC_lexicon_tags_23 },
    { 0x73c18bb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e18bbd, SUC_lexicon_tags_17 },
    { 0xf6418bbe, SUC_lexicon_tags_26 },
    { 0x76c18bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2818bc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a18bcc, SUC_lexicon_tags_820 },
    { 0xdb818bcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0818bd3, SUC_lexicon_tags_29 },
    { 0xe9c18bd3, SUC_lexicon_tags_2 },
    { 0x5e818bd3, SUC_lexicon_tags_31 },
    { 0x70818bd6, SUC_lexicon_tags_4 },
    { 0x82218bd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4218bd9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaf218bdb, SUC_lexicon_tags_36 },
    { 0xfc018bdb, SUC_lexicon_tags_2 },
    { 0x98018bdb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a18be5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92818be9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade18bef, SUC_lexicon_tags_37 },
    { 0xfaa18bef, SUC_lexicon_tags_11 },
    { 0x67c18bf1, SUC_lexicon_tags_11 },
    { 0xfee18bef, SUC_lexicon_tags_14 },
    { 0xf9c18bf3, SUC_lexicon_tags_7 },
    { 0x59a18bf0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae18bf7, SUC_lexicon_tags_87 },
    { 0xcbc18bf7, SUC_lexicon_tags_19 },
    { 0xe7618bf8, SUC_lexicon_tags_1 },
    { 0x7d018bfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16818bfc, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7818bff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2b218c01, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x62a18c03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c18c08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x95e18c0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42818c13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae18c16, SUC_lexicon_tags_39 },
    { 0xd0618c17, SUC_lexicon_tags_68 },
    { 0x1f818c16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c18c1d, SUC_lexicon_tags_10 },
    { 0x80218c1d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3d618c20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8218c26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55418c28, SUC_lexicon_tags_33 },
    { 0x81418c28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7818c30, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3618c33, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0x1018c35, SUC_lexicon_tags_0 },
    { 0x75618c35, SUC_lexicon_tags_10 },
    { 0x58218c37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac18c3c, SUC_lexicon_tags_44 },
    { 0x2ae18c3d, SUC_lexicon_tags_34 },
    { 0x86218c3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64818c40, SUC_lexicon_tags_17 },
    { 0x8ec18c41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4418c44, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2418c46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84e18c48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd7a18c4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73218c4f, SUC_lexicon_tags_2 },
    { 0x7fc18c50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2618c57, SUC_lexicon_tags_2 },
    { 0xa018c58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c18c5e, SUC_lexicon_tags_14 },
    { 0x5018c5e, SUC_lexicon_tags_38 },
    { 0xc8a18c60, SUC_lexicon_tags_66 },
    { 0x16e18c61, SUC_lexicon_tags_2 },
    { 0xc9418c60, SUC_lexicon_tags_33 },
    { 0x9ea18c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8e18c65, SUC_lexicon_tags_0 },
    { 0x38e18c66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba18c6b, SUC_lexicon_tags_25 },
    { 0x8c818c6b, SUC_lexicon_tags_32 },
    { 0x68218c6d, SUC_lexicon_tags_6 },
    { 0x27a18c6d, SUC_lexicon_tags_37 },
    { 0x75218c6b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40418c76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36618c78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb018c7b, SUC_lexicon_tags_66 },
    { 0x5dc18c7c, SUC_lexicon_tags_37 },
    { 0x87618c7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e618c86, SUC_lexicon_tags_2 },
    { 0xb5818c87, SUC_lexicon_tags_54 },
    { 0xf6e18c88, SUC_lexicon_tags_0 },
    { 0xafc18c89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a18c8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda018c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dc18c91, SUC_lexicon_tags_138 },
    { 0x51c18c91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc218c95, SUC_lexicon_tags_55 },
    { 0x7da18c96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a18c99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca18c9e, SUC_lexicon_tags_38 },
    { 0x54818c9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a18ca2, SUC_lexicon_tags_17 },
    { 0x5b018ca3, SUC_lexicon_tags_15 },
    { 0x6d818ca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e18ca7, SUC_lexicon_tags_7 },
    { 0xc5218ca8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5818cac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c18caf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43218cb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b218cb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae818cba, SUC_lexicon_tags_17 },
    { 0x86818cbb, SUC_lexicon_tags_31 },
    { 0xe9a18cbc, SUC_lexicon_tags_43 },
    { 0x25218cbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbb418cbf, SUC_lexicon_tags_7 },
    { 0xb8a18cc0, SUC_lexicon_tags_124 },
    { 0xe5418cbf, SUC_lexicon_tags_449 },
    { 0xd3a18cc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd018cc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e618cc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce618ccd, SUC_lexicon_tags_18 },
    { 0x15018cce, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a18cd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71618cd6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e18cdf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac818ce4, SUC_lexicon_tags_25 },
    { 0xf8818ce5, SUC_lexicon_tags_151 },
    { 0x2ba18ce4, SUC_lexicon_tags_25 },
    { 0x56418ce6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78018cea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec18ced, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc0418cef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91418cf5, SUC_lexicon_tags_38 },
    { 0xeb018cf6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45018cfa, SUC_lexicon_tags_38 },
    { 0x1ac18cfa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x24418cfd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x56818cff, SUC_lexicon_tags_1 },
    { 0x1f018d00, SUC_lexicon_tags_45 },
    { 0x7b818d01, SUC_lexicon_tags_15 },
    { 0x32618d02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a818d04, SUC_lexicon_tags_45 },
    { 0x2da18d05, SUC_lexicon_tags_17 },
    { 0x7c018d06, SUC_lexicon_tags_12 },
    { 0x83018d07, SUC_lexicon_tags_48 },
    { 0xe4e18d08, SUC_lexicon_tags_25 },
    { 0xf2218d04, SUC_lexicon_tags_7 },
    { 0x1be18d0a, SUC_lexicon_tags_16 },
    { 0xcaa18d0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a18d0e, SUC_lexicon_tags_0 },
    { 0xd6018d0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26818d11, SUC_lexicon_tags_12 },
    { 0xbe818d11, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c18d18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63818d1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0418d22, SUC_lexicon_tags_45 },
    { 0x5ec18d23, SUC_lexicon_tags_15 },
    { 0x38018d23, SUC_lexicon_tags_11 },
    { 0x2a218d22, SUC_lexicon_tags_44 },
    { 0x32218d26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7818d2b, SUC_lexicon_tags_37 },
    { 0xa7418d2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c18d31, SUC_lexicon_tags_9 },
    { 0xdae18d32, SUC_lexicon_tags_37 },
    { 0xc0018d33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcee18d35, SUC_lexicon_tags_32 },
    { 0x4ca18d36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c18d39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618d3d, SUC_lexicon_tags_2 },
    { 0xe0618d3e, SUC_lexicon_tags_33 },
    { 0x2018d3f, SUC_lexicon_tags_28 },
    { 0xafa18d3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb6418d42, SUC_lexicon_tags_9 },
    { 0xe0a18d42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89018d46, SUC_lexicon_tags_33 },
    { 0x34618d47, SUC_lexicon_tags_37 },
    { 0xe9c18d48, SUC_lexicon_tags_45 },
    { 0xcd418d49, SUC_lexicon_tags_17 },
    { 0xfda18d49, SUC_lexicon_tags_7 },
    { 0x36218d4b, SUC_lexicon_tags_11 },
    { 0x31018d4c, SUC_lexicon_tags_17 },
    { 0xfb818d49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf218d51, SUC_lexicon_tags_77 },
    { 0x1f418d52, SUC_lexicon_tags_45 },
    { 0x6a818d53, SUC_lexicon_tags_1 },
    { 0x2e218d54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5818d59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a218d5c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12618d61, SUC_lexicon_tags_17 },
    { 0xdf218d61, SUC_lexicon_tags_2 },
    { 0xab418d61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc18d68, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x93218d6b, SUC_lexicon_tags_14 },
    { 0x44818d6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c18d70, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad818d76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e18d7a, SUC_lexicon_tags_19 },
    { 0xb3e18d7b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1818d81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ba18d83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa18d86, SUC_lexicon_tags_47 },
    { 0x1c018d86, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69418d8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2218d91, SUC_lexicon_tags_15 },
    { 0x2aa18d92, SUC_lexicon_tags_1 },
    { 0x29018d91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f418d95, SUC_lexicon_tags_37 },
    { 0x70e18d95, SUC_lexicon_tags_22 },
    { 0x45a18d97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd618d99, SUC_lexicon_tags_37 },
    { 0x47c18d9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5418d9d, SUC_lexicon_tags_7 },
    { 0x8dc18d9d, SUC_lexicon_tags_0 },
    { 0xc1018d9d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62218da7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x81c18da9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90618dad, SUC_lexicon_tags_25 },
    { 0x28618dad, SUC_lexicon_tags_151 },
    { 0xf5c18daf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6218dba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18618dbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee18dbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ee18dc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14a18dc3, SUC_lexicon_tags_57 },
    { 0x82418dc4, SUC_lexicon_tags_38 },
    { 0x82e18dc4, SUC_lexicon_tags_12 },
    { 0x1f218dc6, SUC_lexicon_tags_27 },
    { 0x5da18dc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc618dcc, SUC_lexicon_tags_11 },
    { 0xf0c18dcc, SUC_lexicon_tags_45 },
    { 0x6a18dcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc18dd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4c18dd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfae18dd8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda618de0, SUC_lexicon_tags_92 },
    { 0xbb818de0, SUC_lexicon_tags_25 },
    { 0x60618de2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9a018de4, SUC_lexicon_tags_11 },
    { 0x2c018de5, SUC_lexicon_tags_12 },
    { 0x5818de6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce18de9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64618dee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8218df1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4218df5, SUC_lexicon_tags_10 },
    { 0x69618df5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7a18df8, SUC_lexicon_tags_47 },
    { 0x69218df9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b418dfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x84218dff, SUC_lexicon_tags_14 },
    { 0x5fc18e00, SUC_lexicon_tags_14 },
    { 0xa4a18dff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x83418e04, SUC_lexicon_tags_0 },
    { 0xf4e18e04, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6fa18e07, SUC_lexicon_tags_0 },
    { 0x74218e08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3218e0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe418e0f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbd018e11, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3f818e13, SUC_lexicon_tags_11 },
    { 0xa6c18e13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35818e19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a18e1c, SUC_lexicon_tags_28 },
    { 0xc9618e1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c18e21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7018e29, SUC_lexicon_tags_29 },
    { 0xb7018e2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59818e2f, SUC_lexicon_tags_33 },
    { 0x5218e30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49418e32, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff218e39, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x45218e3b, SUC_lexicon_tags_2 },
    { 0xfaa18e3b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2618e41, SUC_lexicon_tags_37 },
    { 0x3c818e41, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb018e4c, SUC_lexicon_tags_12 },
    { 0xf018e4d, SUC_lexicon_tags_43 },
    { 0xc4418e4e, SUC_lexicon_tags_7 },
    { 0x6dc18e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c618e52, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a18e58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7618e5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0418e5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78618e61, SUC_lexicon_tags_1 },
    { 0xb7618e61, SUC_lexicon_tags_45 },
    { 0xcb418e63, SUC_lexicon_tags_37 },
    { 0x2b618e64, SUC_lexicon_tags_1 },
    { 0x7d618e62, SUC_lexicon_tags_14 },
    { 0x3ca18e64, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa18e69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66018e6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38018e75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7f218e77, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5618e79, SUC_lexicon_tags_61 },
    { 0x95618e7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25618e7d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeee18e7f, SUC_lexicon_tags_25 },
    { 0xad618e80, SUC_lexicon_tags_11 },
    { 0x2a818e80, SUC_lexicon_tags_6 },
    { 0x4a218e82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f618e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a18e8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5818e8e, SUC_lexicon_tags_25 },
    { 0x4ae18e8e, SUC_lexicon_tags_142 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec018e9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8818ea1, SUC_lexicon_tags_11 },
    { 0x2418ea2, SUC_lexicon_tags_7 },
    { 0x1fe18ea2, SUC_lexicon_tags_26 },
    { 0x7ca18ea4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c18ea7, SUC_lexicon_tags_12 },
    { 0xac418ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a18eac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2418eb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c18eb5, SUC_lexicon_tags_25 },
    { 0xef218eb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc018ebb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b218ebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66418ec5, SUC_lexicon_tags_60 },
    { 0x7f418ec5, SUC_lexicon_tags_11 },
    { 0xc1218ec5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a18ed1, SUC_lexicon_tags_66 },
    { 0xc018ed1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc18ed8, SUC_lexicon_tags_47 },
    { 0xce618ed8, SUC_lexicon_tags_1 },
    { 0xed818eda, SUC_lexicon_tags_11 },
    { 0xe9c18ed9, SUC_lexicon_tags_433 },
    { 0xb7a18edc, SUC_lexicon_tags_49 },
    { 0xa1e18edc, SUC_lexicon_tags_43 },
    { 0x2c218ede, SUC_lexicon_tags_0 },
    { 0x1b018ed9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc018ee2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74218ee4, SUC_lexicon_tags_45 },
    { 0x15818ee4, SUC_lexicon_tags_27 },
    { 0x26418ee6, SUC_lexicon_tags_0 },
    { 0x63a18ee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e18eef, SUC_lexicon_tags_11 },
    { 0xe7418ef0, SUC_lexicon_tags_11 },
    { 0x3ae18ef1, SUC_lexicon_tags_12 },
    { 0x29e18ef1, SUC_lexicon_tags_45 },
    { 0xb5e18ef3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2418ef9, SUC_lexicon_tags_15 },
    { 0x1a218efa, SUC_lexicon_tags_171 },
    { 0xd6618efa, SUC_lexicon_tags_11 },
    { 0x88018efc, SUC_lexicon_tags_38 },
    { 0x4c018efd, SUC_lexicon_tags_124 },
    { 0x39618efe, SUC_lexicon_tags_2 },
    { 0xc8418efc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c818f03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85818f06, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfbe18f08, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x5d818f0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9a018f0c, SUC_lexicon_tags_19 },
    { 0x6b618f0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a18f13, SUC_lexicon_tags_11 },
    { 0x2d618f13, SUC_lexicon_tags_28 },
    { 0x35a18f15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x93e18f17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0018f19, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x69a18f1b, SUC_lexicon_tags_61 },
    { 0x6a18f1c, SUC_lexicon_tags_92 },
    { 0x53e18f1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c18f26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf4a18f28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30818f2e, SUC_lexicon_tags_29 },
    { 0x2e818f2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1ea18f31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a618f35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e18f38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68218f3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff018f3d, SUC_lexicon_tags_43 },
    { 0x88c18f3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42218f44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47018f4c, SUC_lexicon_tags_7 },
    { 0x5aa18f4c, SUC_lexicon_tags_37 },
    { 0xaa818f4e, SUC_lexicon_tags_0 },
    { 0xa9618f4c, SUC_lexicon_tags_126 },
    { 0x1ae18f50, SUC_lexicon_tags_63 },
    { 0x60218f50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce418f57, SUC_lexicon_tags_16 },
    { 0xb3818f58, SUC_lexicon_tags_29 },
    { 0x54418f59, SUC_lexicon_tags_28 },
    { 0xa4818f58, SUC_lexicon_tags_29 },
    { 0xc2618f58, SUC_lexicon_tags_17 },
    { 0x56a18f5b, SUC_lexicon_tags_12 },
    { 0xcb418f5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb218f61, SUC_lexicon_tags_1 },
    { 0xee618f62, SUC_lexicon_tags_25 },
    { 0x52018f63, SUC_lexicon_tags_43 },
    { 0x39418f63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa418f69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1418f6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd218f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b218f7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c18f7e, SUC_lexicon_tags_27 },
    { 0xf6018f7f, SUC_lexicon_tags_37 },
    { 0x84618f7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8818f84, SUC_lexicon_tags_6 },
    { 0x76c18f84, SUC_lexicon_tags_63 },
    { 0x7f218f86, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x66018f89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d018f8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61818f8e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a18f92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f818f95, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e18f99, SUC_lexicon_tags_63 },
    { 0x34c18f9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcfe18f9c, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xe0e18f9e, SUC_lexicon_tags_25 },
    { 0xbb418f9f, SUC_lexicon_tags_5 },
    { 0x85c18f9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3618fa2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6ae18fa4, SUC_lexicon_tags_38 },
    { 0xed618fa5, SUC_lexicon_tags_15 },
    { 0xf9018fa6, SUC_lexicon_tags_11 },
    { 0x2da18fa7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6618faf, SUC_lexicon_tags_2 },
    { 0x4fc18fb0, SUC_lexicon_tags_38 },
    { 0xc3618faf, SUC_lexicon_tags_17 },
    { 0xc0018fb1, SUC_lexicon_tags_48 },
    { 0xbfe18fb3, SUC_lexicon_tags_2 },
    { 0x5a618fb3, SUC_lexicon_tags_7 },
    { 0x74c18fb3, SUC_lexicon_tags_37 },
    { 0x22e18fb0, SUC_lexicon_tags_25 },
    { 0x9fc18fb6, SUC_lexicon_tags_6 },
    { 0x2d618fb4, SUC_lexicon_tags_4 },
    { 0x6b618fb9, SUC_lexicon_tags_29 },
    { 0x37e18fb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c18fbd, SUC_lexicon_tags_37 },
    { 0x97018fbd, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3418fc2, SUC_lexicon_tags_31 },
    { 0x92c18fc2, SUC_lexicon_tags_32 },
    { 0xc9218fc4, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0x218fc6, SUC_lexicon_tags_23 },
    { 0x9e218fc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1618fd0, SUC_lexicon_tags_11 },
    { 0x83418fd1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc818fd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0418fd6, SUC_lexicon_tags_38 },
    { 0x15018fd7, SUC_lexicon_tags_5 },
    { 0x4c418fd7, SUC_lexicon_tags_63 },
    { 0x62e18fd9, SUC_lexicon_tags_7 },
    { 0xf4818fda, SUC_lexicon_tags_25 },
    { 0x4a018fdb, SUC_lexicon_tags_7 },
    { 0x9c818fd8, SUC_lexicon_tags_12 },
    { 0xa818fd6, SUC_lexicon_tags_43 },
    { 0xe9c18fde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a18fe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8018fe8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc18feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be18fee, SUC_lexicon_tags_84 },
    { 0xada18fee, SUC_lexicon_tags_12 },
    { 0xfc18ff0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15818ff5, SUC_lexicon_tags_14 },
    { 0x6c018ff6, SUC_lexicon_tags_27 },
    { 0xb0618ff6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a018ffa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb618fff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac19005, SUC_lexicon_tags_30 },
    { 0xc9c19005, SUC_lexicon_tags_68 },
    { 0xd4819007, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14419009, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x58e1900b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c19012, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf1619014, SUC_lexicon_tags_33 },
    { 0x1a819015, SUC_lexicon_tags_14 },
    { 0xf1c19016, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf741901d, SUC_lexicon_tags_0 },
    { 0x9c41901e, SUC_lexicon_tags_23 },
    { 0xe821901d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98c19021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e019023, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1401902d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58419030, SUC_lexicon_tags_19 },
    { 0x2c619031, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c419038, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34e1903a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed01903e, SUC_lexicon_tags_53 },
    { 0xcec1903e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfa19041, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3219043, SUC_lexicon_tags_8 },
    { 0x42419043, SUC_lexicon_tags_2 },
    { 0x77419045, SUC_lexicon_tags_60 },
    { 0x83c19046, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6921904a, SUC_lexicon_tags_37 },
    { 0x5361904b, SUC_lexicon_tags_1 },
    { 0x5741904c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a19050, SUC_lexicon_tags_56 },
    { 0x4e619051, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88619054, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x88419057, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf5c19059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75c1905b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e1905e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb819060, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84619065, SUC_lexicon_tags_44 },
    { 0x43619066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a19068, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd741906a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a19071, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda19075, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d41907e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c19082, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a19086, SUC_lexicon_tags_27 },
    { 0x36a19087, SUC_lexicon_tags_26 },
    { 0xcfe19088, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81908b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf81908d, SUC_lexicon_tags_6 },
    { 0x3be1908e, SUC_lexicon_tags_7 },
    { 0x3f61908f, SUC_lexicon_tags_12 },
    { 0xd0a19090, SUC_lexicon_tags_3 },
    { 0xf5a1908e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51419096, SUC_lexicon_tags_29 },
    { 0x38019097, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0x4e19099, SUC_lexicon_tags_11 },
    { 0x14019099, SUC_lexicon_tags_1 },
    { 0x2461909b, SUC_lexicon_tags_11 },
    { 0x8a1909c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbca1909e, SUC_lexicon_tags_29 },
    { 0x6541909e, SUC_lexicon_tags_30 },
    { 0xd761909e, SUC_lexicon_tags_19 },
    { 0x84190a1, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1a8190a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x296190a5, SUC_lexicon_tags_1 },
    { 0x9fa190a6, SUC_lexicon_tags_16 },
    { 0x1b2190a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a190ad, SUC_lexicon_tags_2 },
    { 0x8d4190ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x748190b1, SUC_lexicon_tags_2 },
    { 0xa8e190b2, SUC_lexicon_tags_37 },
    { 0x8d4190b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca190b7, SUC_lexicon_tags_24 },
    { 0x5d0190b8, SUC_lexicon_tags_48 },
    { 0xeb8190b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x66e190bb, SUC_lexicon_tags_60 },
    { 0x80190bc, SUC_lexicon_tags_11 },
    { 0xfd6190bb, SUC_lexicon_tags_31 },
    { 0xe82190be, SUC_lexicon_tags_15 },
    { 0x7e4190bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c190c5, SUC_lexicon_tags_11 },
    { 0x7a190c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd8190c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690190d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a190d9, SUC_lexicon_tags_38 },
    { 0xe80190d9, SUC_lexicon_tags_11 },
    { 0x194190db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8190de, SUC_lexicon_tags_14 },
    { 0x758190df, SUC_lexicon_tags_48 },
    { 0x35e190e0, SUC_lexicon_tags_27 },
    { 0xa66190e1, SUC_lexicon_tags_39 },
    { 0x6b4190e1, SUC_lexicon_tags_1 },
    { 0x3ec190e3, SUC_lexicon_tags_28 },
    { 0xd6190e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8190e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x778190ea, SUC_lexicon_tags_12 },
    { 0xcd0190eb, SUC_lexicon_tags_13 },
    { 0x966190ea, SUC_lexicon_tags_11 },
    { 0x95c190ed, SUC_lexicon_tags_85 },
    { 0x240190ed, SUC_lexicon_tags_15 },
    { 0x90190ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c190f2, SUC_lexicon_tags_108 },
    { 0xb02190f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a190f8, SUC_lexicon_tags_85 },
    { 0xf32190f8, SUC_lexicon_tags_37 },
    { 0xeea190fa, SUC_lexicon_tags_11 },
    { 0x3d4190fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2190fe, SUC_lexicon_tags_23 },
    { 0xcbc190ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0019108, SUC_lexicon_tags_34 },
    { 0x7c19109, SUC_lexicon_tags_55 },
    { 0x2901910a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce19112, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e619115, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3fe19117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc1911b, SUC_lexicon_tags_2 },
    { 0x1aa1911c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xdf41911e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24819127, SUC_lexicon_tags_14 },
    { 0xb8619127, SUC_lexicon_tags_37 },
    { 0xc5c19129, SUC_lexicon_tags_28 },
    { 0xa0a19129, SUC_lexicon_tags_44 },
    { 0x1a1912b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b01912f, SUC_lexicon_tags_12 },
    { 0x3161912f, SUC_lexicon_tags_34 },
    { 0xa0819131, SUC_lexicon_tags_27 },
    { 0x65219132, SUC_lexicon_tags_2 },
    { 0xeb619133, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a619138, SUC_lexicon_tags_17 },
    { 0xa7219139, SUC_lexicon_tags_26 },
    { 0x70619139, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed41914f, SUC_lexicon_tags_17 },
    { 0x4501914f, SUC_lexicon_tags_60 },
    { 0xd721914f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9419158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c41915e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c19161, SUC_lexicon_tags_19 },
    { 0xf1c19162, SUC_lexicon_tags_26 },
    { 0xeb819163, SUC_lexicon_tags_2 },
    { 0x4ac19162, SUC_lexicon_tags_15 },
    { 0x4e619165, SUC_lexicon_tags_11 },
    { 0x30419165, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd019169, SUC_lexicon_tags_32 },
    { 0x8181916a, SUC_lexicon_tags_32 },
    { 0xcc01916b, SUC_lexicon_tags_55 },
    { 0x4a41916a, SUC_lexicon_tags_82 },
    { 0x7d41916d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e19170, SUC_lexicon_tags_35 },
    { 0x1b219171, SUC_lexicon_tags_37 },
    { 0x1a619172, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7019174, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f219177, SUC_lexicon_tags_60 },
    { 0xa7c19178, SUC_lexicon_tags_11 },
    { 0x96c19177, SUC_lexicon_tags_1 },
    { 0x85a1917a, SUC_lexicon_tags_15 },
    { 0xabe19179, SUC_lexicon_tags_21 },
    { 0x2ac1917c, SUC_lexicon_tags_7 },
    { 0x4d219178, SUC_lexicon_tags_4 },
    { 0xa4e1917c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xbee19180, SUC_lexicon_tags_12 },
    { 0x5dc19180, SUC_lexicon_tags_37 },
    { 0x2da19182, SUC_lexicon_tags_15 },
    { 0x31419183, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1819187, SUC_lexicon_tags_1 },
    { 0x47419188, SUC_lexicon_tags_1 },
    { 0xb0c19189, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1141918c, SUC_lexicon_tags_3 },
    { 0x6821918d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e21918f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a19192, SUC_lexicon_tags_60 },
    { 0xbc219193, SUC_lexicon_tags_65 },
    { 0x58819192, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0xd5419196, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x45819199, SUC_lexicon_tags_48 },
    { 0x2e61919a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc01919d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2701919f, SUC_lexicon_tags_17 },
    { 0x6681919f, SUC_lexicon_tags_12 },
    { 0x5d2191a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf2191a3, SUC_lexicon_tags_27 },
    { 0x7bc191a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd0a191a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36a191a8, SUC_lexicon_tags_19 },
    { 0x16191a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436191b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa191b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30a191b6, SUC_lexicon_tags_17 },
    { 0x284191b7, SUC_lexicon_tags_31 },
    { 0x794191b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74191bc, SUC_lexicon_tags_19 },
    { 0xc5a191bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x62191bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7a191c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc191c5, SUC_lexicon_tags_38 },
    { 0x552191c6, SUC_lexicon_tags_2 },
    { 0xb3e191c7, SUC_lexicon_tags_45 },
    { 0xe0e191c6, SUC_lexicon_tags_30 },
    { 0x882191c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c191cd, SUC_lexicon_tags_33 },
    { 0xeda191cd, SUC_lexicon_tags_48 },
    { 0xe40191cd, SUC_lexicon_tags_49 },
    { 0x16a191d0, SUC_lexicon_tags_11 },
    { 0xa0191d1, SUC_lexicon_tags_47 },
    { 0xf8e191cd, SUC_lexicon_tags_15 },
    { 0x526191d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786191d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e2191d9, SUC_lexicon_tags_14 },
    { 0x398191da, SUC_lexicon_tags_35 },
    { 0x5ba191d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32191e0, SUC_lexicon_tags_26 },
    { 0xc8a191e1, SUC_lexicon_tags_2 },
    { 0x1ae191e2, SUC_lexicon_tags_25 },
    { 0x82c191e2, SUC_lexicon_tags_11 },
    { 0x6e0191e4, SUC_lexicon_tags_15 },
    { 0x8c4191e4, SUC_lexicon_tags_37 },
    { 0x582191e6, SUC_lexicon_tags_27 },
    { 0x3ea191e7, SUC_lexicon_tags_7 },
    { 0x2e0191e7, SUC_lexicon_tags_12 },
    { 0x1d0191e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626191f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58191f6, SUC_lexicon_tags_11 },
    { 0xefe191f7, SUC_lexicon_tags_4 },
    { 0x54191f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c191fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8191fe, SUC_lexicon_tags_11 },
    { 0xf08191ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84419204, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e19207, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c01920a, SUC_lexicon_tags_12 },
    { 0xf5a1920b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbee1920d, SUC_lexicon_tags_38 },
    { 0x45c1920e, SUC_lexicon_tags_43 },
    { 0x3921920f, SUC_lexicon_tags_30 },
    { 0x4a01920f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2619213, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb619215, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a1921c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x61c1921e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e19224, SUC_lexicon_tags_14 },
    { 0xe9819225, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xefa19227, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4481922d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c19231, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb01923b, SUC_lexicon_tags_11 },
    { 0xd8c1923b, SUC_lexicon_tags_79 },
    { 0x9821923d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62c1923f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc19244, SUC_lexicon_tags_0 },
    { 0x8819245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2019247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7961924f, SUC_lexicon_tags_7 },
    { 0x83a1924f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c19256, SUC_lexicon_tags_37 },
    { 0xd8419256, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce61925c, SUC_lexicon_tags_33 },
    { 0x88c1925d, SUC_lexicon_tags_11 },
    { 0x4101925d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc19265, SUC_lexicon_tags_0 },
    { 0x57219266, SUC_lexicon_tags_17 },
    { 0xbe819267, SUC_lexicon_tags_15 },
    { 0xb1a19268, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4da1926a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa1926f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac19273, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbcc19275, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x92219278, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff81927a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca19280, SUC_lexicon_tags_91 },
    { 0xb3c19281, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2219285, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee19293, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2819297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4461929b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba192a9, SUC_lexicon_tags_47 },
    { 0x50c192aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac192ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34192b4, SUC_lexicon_tags_2 },
    { 0x252192b4, SUC_lexicon_tags_12 },
    { 0x4bc192b4, SUC_lexicon_tags_194 },
    { 0xed2192b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45a192b9, SUC_lexicon_tags_15 },
    { 0x3b4192ba, SUC_lexicon_tags_69 },
    { 0x6e6192ba, SUC_lexicon_tags_39 },
    { 0x86192bc, SUC_lexicon_tags_37 },
    { 0x114192bd, SUC_lexicon_tags_26 },
    { 0x422192bd, SUC_lexicon_tags_11 },
    { 0x212192bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6192c4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x218192c6, SUC_lexicon_tags_23 },
    { 0xf9a192c7, SUC_lexicon_tags_47 },
    { 0xc5c192c8, SUC_lexicon_tags_33 },
    { 0xa4192c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700192ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18192d1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb18192d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da192d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xabe192d9, SUC_lexicon_tags_10 },
    { 0xf06192da, SUC_lexicon_tags_26 },
    { 0xe30192d9, SUC_lexicon_tags_12 },
    { 0x3ac192dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x558192df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x30192e2, SUC_lexicon_tags_15 },
    { 0x624192e2, SUC_lexicon_tags_1 },
    { 0xae0192e4, SUC_lexicon_tags_10 },
    { 0x5f6192e3, SUC_lexicon_tags_7 },
    { 0xf4c192e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70192ec, SUC_lexicon_tags_37 },
    { 0x6b2192ed, SUC_lexicon_tags_25 },
    { 0x2b8192ed, SUC_lexicon_tags_2 },
    { 0x970192ef, SUC_lexicon_tags_45 },
    { 0x874192ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x874192f2, SUC_lexicon_tags_25 },
    { 0xd9a192f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x510192f6, SUC_lexicon_tags_38 },
    { 0x358192f6, SUC_lexicon_tags_37 },
    { 0x53a192f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21a192fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9e192fc, SUC_lexicon_tags_62 },
    { 0x992192fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x294192ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6419302, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9819305, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba19310, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29e19312, SUC_lexicon_tags_63 },
    { 0x35a19313, SUC_lexicon_tags_4 },
    { 0xf019313, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98419318, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c619321, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10219324, SUC_lexicon_tags_49 },
    { 0xd6019325, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa1932e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a019331, SUC_lexicon_tags_25 },
    { 0xb5219332, SUC_lexicon_tags_6 },
    { 0xe9c19333, SUC_lexicon_tags_14 },
    { 0x5b419332, SUC_lexicon_tags_1 },
    { 0x2ee19333, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4619339, SUC_lexicon_tags_2 },
    { 0x2a81933a, SUC_lexicon_tags_44 },
    { 0x1cc1933b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3419349, SUC_lexicon_tags_11 },
    { 0x2201934a, SUC_lexicon_tags_1 },
    { 0x5c81934b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdee1934d, SUC_lexicon_tags_38 },
    { 0x5401934e, SUC_lexicon_tags_10 },
    { 0x9241934e, SUC_lexicon_tags_18 },
    { 0x7021934e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5619354, SUC_lexicon_tags_17 },
    { 0x8a019354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a1935c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe421935f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe619361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83419364, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad419367, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x81e19369, SUC_lexicon_tags_43 },
    { 0xee19369, SUC_lexicon_tags_2 },
    { 0x78a1936a, SUC_lexicon_tags_17 },
    { 0xd61936c, SUC_lexicon_tags_37 },
    { 0x3819369, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e19373, SUC_lexicon_tags_65 },
    { 0xcbc19374, SUC_lexicon_tags_23 },
    { 0x89219375, SUC_lexicon_tags_68 },
    { 0x8fa19374, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x99219378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb01937d, SUC_lexicon_tags_37 },
    { 0xea41937e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35819382, SUC_lexicon_tags_11 },
    { 0xbe19383, SUC_lexicon_tags_1 },
    { 0x60419384, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x58c19386, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19419389, SUC_lexicon_tags_30 },
    { 0xf8a19389, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b41938d, SUC_lexicon_tags_37 },
    { 0xaac1938e, SUC_lexicon_tags_7 },
    { 0xa9e1938d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e19396, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf8219398, SUC_lexicon_tags_37 },
    { 0x48419399, SUC_lexicon_tags_29 },
    { 0x66a1939a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6e1939c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8193a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc193a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530193ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb80193ae, SUC_lexicon_tags_45 },
    { 0x51a193af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7c4193b1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xdee193b3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x514193b6, SUC_lexicon_tags_43 },
    { 0x5d6193b7, SUC_lexicon_tags_1 },
    { 0x642193b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26193be, SUC_lexicon_tags_0 },
    { 0x62193bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2193c2, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0xde193c4, SUC_lexicon_tags_174 },
    { 0x574193c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724193cc, SUC_lexicon_tags_2 },
    { 0x676193cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe12193cf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa193d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf14193d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x900193d6, SUC_lexicon_tags_46 },
    { 0x7a193d6, SUC_lexicon_tags_25 },
    { 0x92e193d8, SUC_lexicon_tags_53 },
    { 0xe92193d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa1e193db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x798193dd, SUC_lexicon_tags_2 },
    { 0x268193de, SUC_lexicon_tags_104 },
    { 0x402193de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0a193e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac193e9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0193ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c193f4, SUC_lexicon_tags_28 },
    { 0x1a193f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7d0193f7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdfe193f9, SUC_lexicon_tags_33 },
    { 0x1c193fa, SUC_lexicon_tags_1 },
    { 0xbe8193fb, SUC_lexicon_tags_2 },
    { 0x7ec193fc, SUC_lexicon_tags_106 },
    { 0x24c193fd, SUC_lexicon_tags_10 },
    { 0x84e193fe, SUC_lexicon_tags_23 },
    { 0x916193fb, SUC_lexicon_tags_17 },
    { 0x34419400, SUC_lexicon_tags_47 },
    { 0x20419401, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb4019403, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e19407, SUC_lexicon_tags_37 },
    { 0xf6819408, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed01940a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c41940d, SUC_lexicon_tags_77 },
    { 0x4a61940d, SUC_lexicon_tags_7 },
    { 0xdb61940e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc19413, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe221941a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfaa1941c, SUC_lexicon_tags_15 },
    { 0x4641941d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa19422, SUC_lexicon_tags_45 },
    { 0xc9e19423, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4619426, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x51419428, SUC_lexicon_tags_29 },
    { 0xdcc19429, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf5a1942b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd81942f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba219433, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb419435, SUC_lexicon_tags_82 },
    { 0xaf619436, SUC_lexicon_tags_27 },
    { 0xc3819435, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x65419439, SUC_lexicon_tags_31 },
    { 0x3f61943a, SUC_lexicon_tags_30 },
    { 0xd6c1943b, SUC_lexicon_tags_45 },
    { 0xdc19439, SUC_lexicon_tags_16 },
    { 0x9841943b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6619441, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6819443, SUC_lexicon_tags_2 },
    { 0x6da19444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c19448, SUC_lexicon_tags_27 },
    { 0x3c019449, SUC_lexicon_tags_37 },
    { 0xca019448, SUC_lexicon_tags_2 },
    { 0x61e19448, SUC_lexicon_tags_0 },
    { 0x941944c, SUC_lexicon_tags_11 },
    { 0x92a1944c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4321944f, SUC_lexicon_tags_11 },
    { 0x81c1944f, SUC_lexicon_tags_6 },
    { 0x8ae19451, SUC_lexicon_tags_11 },
    { 0xa861944f, SUC_lexicon_tags_6 },
    { 0x63819452, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed619460, SUC_lexicon_tags_70 },
    { 0x7cc19460, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc4419463, SUC_lexicon_tags_17 },
    { 0xb9c19463, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb501946c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f01946e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25819472, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44019476, SUC_lexicon_tags_1 },
    { 0x4d619477, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc3819479, SUC_lexicon_tags_11 },
    { 0x75e1947a, SUC_lexicon_tags_56 },
    { 0x2201947a, SUC_lexicon_tags_27 },
    { 0x8a81947c, SUC_lexicon_tags_11 },
    { 0x1801947c, SUC_lexicon_tags_68 },
    { 0xbc61947e, SUC_lexicon_tags_37 },
    { 0x5d41947c, SUC_lexicon_tags_30 },
    { 0x38419480, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1948f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81a19491, SUC_lexicon_tags_28 },
    { 0x61219492, SUC_lexicon_tags_27 },
    { 0x49419492, SUC_lexicon_tags_11 },
    { 0x5fe19494, SUC_lexicon_tags_15 },
    { 0x50c19494, SUC_lexicon_tags_1 },
    { 0x60819492, SUC_lexicon_tags_12 },
    { 0xe19497, SUC_lexicon_tags_3 },
    { 0xfc619498, SUC_lexicon_tags_8 },
    { 0x4b619499, SUC_lexicon_tags_0 },
    { 0xa241949a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e1949d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e194a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60194aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258194af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a194b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a194b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x870194b8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c194bb, SUC_lexicon_tags_33 },
    { 0x2d0194bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80194bf, SUC_lexicon_tags_15 },
    { 0xaa194bf, SUC_lexicon_tags_1 },
    { 0xa2e194c1, SUC_lexicon_tags_88 },
    { 0xe88194c2, SUC_lexicon_tags_37 },
    { 0x98e194c2, SUC_lexicon_tags_25 },
    { 0x454194c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4e8194c6, SUC_lexicon_tags_17 },
    { 0x3aa194c6, SUC_lexicon_tags_25 },
    { 0x768194c7, SUC_lexicon_tags_11 },
    { 0x90e194c9, SUC_lexicon_tags_2 },
    { 0x71e194ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a194ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22194d2, SUC_lexicon_tags_11 },
    { 0xafc194d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e194d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34194df, SUC_lexicon_tags_25 },
    { 0x338194e0, SUC_lexicon_tags_2 },
    { 0xaba194e1, SUC_lexicon_tags_15 },
    { 0xc00194e2, SUC_lexicon_tags_0 },
    { 0x9d4194e3, SUC_lexicon_tags_37 },
    { 0x8ae194e4, SUC_lexicon_tags_0 },
    { 0x670194e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1c8194e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90194eb, SUC_lexicon_tags_79 },
    { 0x65a194eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea194ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e194f3, SUC_lexicon_tags_30 },
    { 0x41e194f4, SUC_lexicon_tags_11 },
    { 0xbbc194f5, SUC_lexicon_tags_17 },
    { 0x9a6194f6, SUC_lexicon_tags_17 },
    { 0x68a194f7, SUC_lexicon_tags_63 },
    { 0x550194f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a194fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8194fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a819503, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66019506, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa1019508, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e1950b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x63a1950d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x29a1950f, SUC_lexicon_tags_60 },
    { 0x8c19510, SUC_lexicon_tags_25 },
    { 0x6c019510, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a19518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc441951d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c19523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf019526, SUC_lexicon_tags_45 },
    { 0x64c19527, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e1952a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c01952f, SUC_lexicon_tags_7 },
    { 0x97a19530, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43819534, SUC_lexicon_tags_14 },
    { 0x5c019535, SUC_lexicon_tags_3 },
    { 0xb2a19535, SUC_lexicon_tags_141 },
    { 0xcbc19537, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e219539, SUC_lexicon_tags_29 },
    { 0xdae1953a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3941953d, SUC_lexicon_tags_43 },
    { 0x4041953d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae019543, SUC_lexicon_tags_1 },
    { 0xffc19544, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94619547, SUC_lexicon_tags_19 },
    { 0xf8e19548, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1954b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf601954d, SUC_lexicon_tags_17 },
    { 0xbca1954e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa0819550, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c19553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49219556, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb9019558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec1955c, SUC_lexicon_tags_30 },
    { 0xfc61955c, SUC_lexicon_tags_48 },
    { 0xab81955e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x92619560, SUC_lexicon_tags_68 },
    { 0x59c19561, SUC_lexicon_tags_29 },
    { 0x4a219562, SUC_lexicon_tags_17 },
    { 0x26619563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a19567, SUC_lexicon_tags_37 },
    { 0x78c19567, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc1956c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73619572, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c19577, SUC_lexicon_tags_19 },
    { 0x7da19578, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb81957e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c819587, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc841958f, SUC_lexicon_tags_11 },
    { 0xb7e1958f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8419594, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9a419596, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a19599, SUC_lexicon_tags_23 },
    { 0x7de19599, SUC_lexicon_tags_45 },
    { 0x4821959b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xea21959e, SUC_lexicon_tags_27 },
    { 0x2941959f, SUC_lexicon_tags_8 },
    { 0x7d2195a0, SUC_lexicon_tags_31 },
    { 0x106195a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x264195a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3dc195a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c195a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6195ad, SUC_lexicon_tags_11 },
    { 0xc0a195ae, SUC_lexicon_tags_46 },
    { 0x2bc195ad, SUC_lexicon_tags_25 },
    { 0x864195b0, SUC_lexicon_tags_11 },
    { 0x57c195ad, SUC_lexicon_tags_11 },
    { 0xabc195ae, SUC_lexicon_tags_33 },
    { 0x8a0195b0, SUC_lexicon_tags_30 },
    { 0x28a195b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388195b8, SUC_lexicon_tags_82 },
    { 0xb2c195b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ca195bb, SUC_lexicon_tags_46 },
    { 0x9c4195bc, SUC_lexicon_tags_49 },
    { 0x364195bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ec195bf, SUC_lexicon_tags_23 },
    { 0xdca195c0, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x740195c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b2195c5, SUC_lexicon_tags_14 },
    { 0x60e195c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa195c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a195cd, SUC_lexicon_tags_2 },
    { 0x558195cd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da195d3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c195d7, SUC_lexicon_tags_46 },
    { 0x9ea195d7, SUC_lexicon_tags_19 },
    { 0x9d0195d9, SUC_lexicon_tags_43 },
    { 0x24c195d9, SUC_lexicon_tags_37 },
    { 0x1ee195d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2195de, SUC_lexicon_tags_2 },
    { 0xa7a195df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb70195e1, SUC_lexicon_tags_26 },
    { 0x84a195e2, SUC_lexicon_tags_14 },
    { 0xb40195e1, SUC_lexicon_tags_47 },
    { 0xe54195e2, SUC_lexicon_tags_1698 },
    { 0xcda195e5, SUC_lexicon_tags_15 },
    { 0x7e4195e6, SUC_lexicon_tags_0 },
    { 0xa0c195e7, SUC_lexicon_tags_10 },
    { 0xb68195e8, SUC_lexicon_tags_6 },
    { 0x330195e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x454195ec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4195f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c195f3, SUC_lexicon_tags_48 },
    { 0xe7c195f4, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230195f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c195fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8b619600, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c19604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f619606, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d219608, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a41960b, SUC_lexicon_tags_2 },
    { 0xd641960c, SUC_lexicon_tags_48 },
    { 0xd3a1960b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da19610, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99419612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca219615, SUC_lexicon_tags_17 },
    { 0x6ce19616, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd819621, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c19624, SUC_lexicon_tags_7 },
    { 0x5c619625, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xade19627, SUC_lexicon_tags_48 },
    { 0xcfa19628, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2461962b, SUC_lexicon_tags_1 },
    { 0x91a1962c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4f21962e, SUC_lexicon_tags_37 },
    { 0x5281962f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58019631, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47819637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a1963c, SUC_lexicon_tags_31 },
    { 0x3c41963d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd861963f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84019645, SUC_lexicon_tags_12 },
    { 0x6019645, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e61964a, SUC_lexicon_tags_0 },
    { 0xb6c1964b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf001964e, SUC_lexicon_tags_9 },
    { 0xeb41964f, SUC_lexicon_tags_2 },
    { 0x2621964e, SUC_lexicon_tags_1 },
    { 0x52a19651, SUC_lexicon_tags_2 },
    { 0x8f219651, SUC_lexicon_tags_51 },
    { 0x6a61964f, SUC_lexicon_tags_38 },
    { 0x1601964f, SUC_lexicon_tags_11 },
    { 0x6181964f, SUC_lexicon_tags_82 },
    { 0x84419655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce1965b, SUC_lexicon_tags_47 },
    { 0x581965c, SUC_lexicon_tags_11 },
    { 0x9301965b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e19660, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x32019663, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x77819666, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa8619668, SUC_lexicon_tags_7 },
    { 0x7019668, SUC_lexicon_tags_7 },
    { 0xbc419668, SUC_lexicon_tags_37 },
    { 0xbe619668, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa221966e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x52819671, SUC_lexicon_tags_34 },
    { 0xa2e19672, SUC_lexicon_tags_37 },
    { 0x8a19672, SUC_lexicon_tags_8 },
    { 0x9019674, SUC_lexicon_tags_16 },
    { 0x90219674, SUC_lexicon_tags_25 },
    { 0x4e619676, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee1967b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1c01967d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa19680, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcce19682, SUC_lexicon_tags_27 },
    { 0xce619683, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96619688, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e19694, SUC_lexicon_tags_7 },
    { 0xe4819695, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9521969d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf1e1969f, SUC_lexicon_tags_11 },
    { 0xa28196a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82196a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278196ac, SUC_lexicon_tags_8 },
    { 0xc00196ad, SUC_lexicon_tags_15 },
    { 0xad2196ad, SUC_lexicon_tags_37 },
    { 0x3e196ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c196b3, SUC_lexicon_tags_14 },
    { 0xbbc196b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378196bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894196c0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6196c3, SUC_lexicon_tags_25 },
    { 0x730196c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc0c196c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x20c196c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d0196ca, SUC_lexicon_tags_11 },
    { 0xc94196cb, SUC_lexicon_tags_63 },
    { 0x5ee196cb, SUC_lexicon_tags_23 },
    { 0x9be196cb, SUC_lexicon_tags_19 },
    { 0x2f0196ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990196d4, SUC_lexicon_tags_25 },
    { 0xd60196d5, SUC_lexicon_tags_2 },
    { 0x568196d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fe196d8, SUC_lexicon_tags_23 },
    { 0x988196d9, SUC_lexicon_tags_31 },
    { 0xde6196da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x818196dc, SUC_lexicon_tags_44 },
    { 0xc06196dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a196e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4196e3, SUC_lexicon_tags_30 },
    { 0xe62196e4, SUC_lexicon_tags_25 },
    { 0xaa8196e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a196ea, SUC_lexicon_tags_39 },
    { 0x876196ea, SUC_lexicon_tags_38 },
    { 0xa4e196ec, SUC_lexicon_tags_37 },
    { 0x2c6196ed, SUC_lexicon_tags_16 },
    { 0x19e196ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736196f3, SUC_lexicon_tags_1 },
    { 0xfbe196f4, SUC_lexicon_tags_17 },
    { 0x4fc196f4, SUC_lexicon_tags_0 },
    { 0xa38196f5, SUC_lexicon_tags_9 },
    { 0x96e196f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x336196f9, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a196ff, SUC_lexicon_tags_77 },
    { 0x58c19700, SUC_lexicon_tags_68 },
    { 0x47819700, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c19705, SUC_lexicon_tags_29 },
    { 0xcd419705, SUC_lexicon_tags_53 },
    { 0x9c19707, SUC_lexicon_tags_17 },
    { 0xb5a19708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8781970e, SUC_lexicon_tags_14 },
    { 0xe8a1970f, SUC_lexicon_tags_35 },
    { 0x16a19710, SUC_lexicon_tags_17 },
    { 0x4e19710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4819716, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf5819718, SUC_lexicon_tags_1 },
    { 0x4b019719, SUC_lexicon_tags_60 },
    { 0x58e19719, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9da1971c, SUC_lexicon_tags_34 },
    { 0x2c01971d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee1971f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35219722, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e19728, SUC_lexicon_tags_17 },
    { 0xc7219729, SUC_lexicon_tags_29 },
    { 0x7de1972a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda1972d, SUC_lexicon_tags_17 },
    { 0x1361972e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3819739, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81973d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a19740, SUC_lexicon_tags_12 },
    { 0x6ce19741, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x53a19743, SUC_lexicon_tags_14 },
    { 0x71219744, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf2a19746, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4419749, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c61974e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c619750, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb0819752, SUC_lexicon_tags_1 },
    { 0x79419753, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd219755, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4019758, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd419761, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x56619763, SUC_lexicon_tags_7 },
    { 0x27619764, SUC_lexicon_tags_29 },
    { 0xa5019765, SUC_lexicon_tags_8 },
    { 0xd6419763, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a19769, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a1976c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xab21976e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x50e19770, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7a19772, SUC_lexicon_tags_11 },
    { 0x67c19773, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7b819775, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e1977c, SUC_lexicon_tags_17 },
    { 0xb621977c, SUC_lexicon_tags_25 },
    { 0x7221977e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74619780, SUC_lexicon_tags_12 },
    { 0x51219780, SUC_lexicon_tags_6 },
    { 0x7ce19780, SUC_lexicon_tags_15 },
    { 0xd5419782, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c19787, SUC_lexicon_tags_17 },
    { 0x2b819788, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd361978c, SUC_lexicon_tags_8 },
    { 0x47c1978c, SUC_lexicon_tags_11 },
    { 0x32e1978e, SUC_lexicon_tags_0 },
    { 0xe261978e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf219794, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9619796, SUC_lexicon_tags_31 },
    { 0xbea19797, SUC_lexicon_tags_47 },
    { 0x49019798, SUC_lexicon_tags_7 },
    { 0x56a19799, SUC_lexicon_tags_107 },
    { 0x46619796, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc197a0, SUC_lexicon_tags_7 },
    { 0x778197a0, SUC_lexicon_tags_2 },
    { 0xefc197a2, SUC_lexicon_tags_40 },
    { 0x24a197a1, SUC_lexicon_tags_11 },
    { 0xb7a197a4, SUC_lexicon_tags_25 },
    { 0x2fc197a5, SUC_lexicon_tags_1 },
    { 0x406197a6, SUC_lexicon_tags_29 },
    { 0xb46197a6, SUC_lexicon_tags_15 },
    { 0x2b6197a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24197ad, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x366197af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4197b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e4197b5, SUC_lexicon_tags_2 },
    { 0x176197b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e6197b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7e4197ba, SUC_lexicon_tags_62 },
    { 0xb72197bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d4197bd, SUC_lexicon_tags_37 },
    { 0x252197bd, SUC_lexicon_tags_23 },
    { 0x106197be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2197c5, SUC_lexicon_tags_27 },
    { 0x404197c5, SUC_lexicon_tags_4 },
    { 0x902197c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77e197c9, SUC_lexicon_tags_0 },
    { 0xb04197ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab8197cc, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x992197ce, SUC_lexicon_tags_15 },
    { 0xb6e197ce, SUC_lexicon_tags_1 },
    { 0x894197d0, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428197d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40197d7, SUC_lexicon_tags_4 },
    { 0x5c6197d8, SUC_lexicon_tags_1 },
    { 0x12197d8, SUC_lexicon_tags_55 },
    { 0x15e197d8, SUC_lexicon_tags_7 },
    { 0x85c197d9, SUC_lexicon_tags_114 },
    { 0x84c197d7, SUC_lexicon_tags_6 },
    { 0x574197dd, SUC_lexicon_tags_2 },
    { 0x2cc197de, SUC_lexicon_tags_27 },
    { 0x41a197dd, SUC_lexicon_tags_14 },
    { 0x122197e0, SUC_lexicon_tags_37 },
    { 0x6ca197e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2197e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ea197e6, SUC_lexicon_tags_49 },
    { 0x70a197e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c2197e9, SUC_lexicon_tags_1 },
    { 0xea4197ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dc197ec, SUC_lexicon_tags_44 },
    { 0xbc8197ed, SUC_lexicon_tags_17 },
    { 0xdf6197ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a197f1, SUC_lexicon_tags_87 },
    { 0xbc8197f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x100197f4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9e8197f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x454197f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc06197fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c619801, SUC_lexicon_tags_23 },
    { 0xb5219802, SUC_lexicon_tags_25 },
    { 0xa6c19803, SUC_lexicon_tags_27 },
    { 0x4ae19804, SUC_lexicon_tags_38 },
    { 0xb0e19804, SUC_lexicon_tags_14 },
    { 0xad619806, SUC_lexicon_tags_43 },
    { 0x17819804, SUC_lexicon_tags_39 },
    { 0x1de19808, SUC_lexicon_tags_35 },
    { 0x79619808, SUC_lexicon_tags_48 },
    { 0xc461980a, SUC_lexicon_tags_15 },
    { 0x621980b, SUC_lexicon_tags_8 },
    { 0x2061980c, SUC_lexicon_tags_37 },
    { 0x20c1980d, SUC_lexicon_tags_8 },
    { 0x4401980e, SUC_lexicon_tags_14 },
    { 0x91c1980f, SUC_lexicon_tags_7 },
    { 0x5681980f, SUC_lexicon_tags_59 },
    { 0xff019811, SUC_lexicon_tags_25 },
    { 0x6d61980f, SUC_lexicon_tags_27 },
    { 0x82019812, SUC_lexicon_tags_15 },
    { 0xfcc19810, SUC_lexicon_tags_37 },
    { 0x48e19815, SUC_lexicon_tags_8 },
    { 0x78a19816, SUC_lexicon_tags_2 },
    { 0x3ea19816, SUC_lexicon_tags_7 },
    { 0x8ac19818, SUC_lexicon_tags_8 },
    { 0x2101980a, SUC_lexicon_tags_24 },
    { 0x421981a, SUC_lexicon_tags_2 },
    { 0x3a619806, SUC_lexicon_tags_21 },
    { 0x16a1981c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca1981f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbb819821, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16619823, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65019829, SUC_lexicon_tags_11 },
    { 0x5a01982a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8021982d, SUC_lexicon_tags_65 },
    { 0x7121982d, SUC_lexicon_tags_84 },
    { 0x7e41982f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb219835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48419837, SUC_lexicon_tags_1 },
    { 0xb0a19838, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6801983b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a1983f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd619842, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x55019845, SUC_lexicon_tags_46 },
    { 0xf4819846, SUC_lexicon_tags_46 },
    { 0xa8e19846, SUC_lexicon_tags_23 },
    { 0x98a19848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x281984a, SUC_lexicon_tags_45 },
    { 0xb261984b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4d01984d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5261984f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4a19851, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52819855, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16019858, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2101985b, SUC_lexicon_tags_33 },
    { 0x98e1985c, SUC_lexicon_tags_43 },
    { 0x7161985d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf421985f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25219863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5819865, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7819868, SUC_lexicon_tags_11 },
    { 0x71019869, SUC_lexicon_tags_31 },
    { 0x7bc1986a, SUC_lexicon_tags_15 },
    { 0xa081986b, SUC_lexicon_tags_60 },
    { 0x54a1986b, SUC_lexicon_tags_47 },
    { 0xbf01986d, SUC_lexicon_tags_1 },
    { 0xa481986e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72019870, SUC_lexicon_tags_11 },
    { 0x29a19871, SUC_lexicon_tags_16 },
    { 0x75419872, SUC_lexicon_tags_43 },
    { 0x9d619873, SUC_lexicon_tags_0 },
    { 0xece19874, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xae19877, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5821987f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3419885, SUC_lexicon_tags_23 },
    { 0x62619886, SUC_lexicon_tags_19 },
    { 0xfb219887, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf621988a, SUC_lexicon_tags_45 },
    { 0x3c21988b, SUC_lexicon_tags_15 },
    { 0x4c81988b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70419890, SUC_lexicon_tags_7 },
    { 0x5fc19891, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x31219894, SUC_lexicon_tags_16 },
    { 0x7b819894, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e19898, SUC_lexicon_tags_11 },
    { 0xb7819898, SUC_lexicon_tags_2 },
    { 0x36c1989a, SUC_lexicon_tags_37 },
    { 0xeba1989a, SUC_lexicon_tags_2 },
    { 0x1801989c, SUC_lexicon_tags_7 },
    { 0x5e81989d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e01989f, SUC_lexicon_tags_14 },
    { 0x416198a0, SUC_lexicon_tags_37 },
    { 0x858198a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2198ab, SUC_lexicon_tags_12 },
    { 0xdac198ab, SUC_lexicon_tags_37 },
    { 0x6a4198ad, SUC_lexicon_tags_0 },
    { 0x988198ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c198b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe198b5, SUC_lexicon_tags_38 },
    { 0x480198b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f0198b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a198bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20198c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7e198c5, SUC_lexicon_tags_1 },
    { 0xde198c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc36198c8, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74198cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80198d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbda198d9, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xc34198db, SUC_lexicon_tags_19 },
    { 0x230198db, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae198e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc10198e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d8198e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784198e9, SUC_lexicon_tags_15 },
    { 0x70198e9, SUC_lexicon_tags_68 },
    { 0x7ba198eb, SUC_lexicon_tags_29 },
    { 0x990198ec, SUC_lexicon_tags_32 },
    { 0x40c198ed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a198f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xac198f5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa2198f7, SUC_lexicon_tags_70 },
    { 0x8a2198f8, SUC_lexicon_tags_28 },
    { 0x9a6198f9, SUC_lexicon_tags_3 },
    { 0x9e198fa, SUC_lexicon_tags_47 },
    { 0x952198fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec198fe, SUC_lexicon_tags_16 },
    { 0xa64198ff, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x83219901, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50419903, SUC_lexicon_tags_37 },
    { 0x2a819904, SUC_lexicon_tags_30 },
    { 0x4819904, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18219909, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd141990b, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481990f, SUC_lexicon_tags_7 },
    { 0x72819910, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd9e19912, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e19918, SUC_lexicon_tags_47 },
    { 0x83419919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca1991c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2721991e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x74c19920, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a819926, SUC_lexicon_tags_66 },
    { 0x8e619927, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee1992a, SUC_lexicon_tags_15 },
    { 0x6521992b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa581992e, SUC_lexicon_tags_7 },
    { 0xe801992f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30019933, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa19936, SUC_lexicon_tags_7 },
    { 0xcc19936, SUC_lexicon_tags_79 },
    { 0x5e019936, SUC_lexicon_tags_25 },
    { 0xa6019939, SUC_lexicon_tags_38 },
    { 0xcf01993a, SUC_lexicon_tags_39 },
    { 0x87e1993a, SUC_lexicon_tags_141 },
    { 0xa6e19936, SUC_lexicon_tags_7 },
    { 0xf819937, SUC_lexicon_tags_38 },
    { 0xfd61993e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf219941, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18e19943, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf619945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb481994b, SUC_lexicon_tags_23 },
    { 0x1d21994c, SUC_lexicon_tags_14 },
    { 0x6c61994b, SUC_lexicon_tags_49 },
    { 0x74e1994e, SUC_lexicon_tags_37 },
    { 0x6e01994f, SUC_lexicon_tags_12 },
    { 0x35e19950, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda61995c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca1995f, SUC_lexicon_tags_45 },
    { 0x8841995f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47819964, SUC_lexicon_tags_17 },
    { 0x36619964, SUC_lexicon_tags_7 },
    { 0xc0819966, SUC_lexicon_tags_37 },
    { 0xa9c19966, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9be19969, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20e1996b, SUC_lexicon_tags_11 },
    { 0xbc21996b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fc1996e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b219974, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4b819976, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa0419978, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x4c01997a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5401997c, SUC_lexicon_tags_8 },
    { 0x5121997c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10019982, SUC_lexicon_tags_37 },
    { 0xaca19983, SUC_lexicon_tags_68 },
    { 0x69419984, SUC_lexicon_tags_47 },
    { 0xd9019983, SUC_lexicon_tags_8 },
    { 0xece19986, SUC_lexicon_tags_25 },
    { 0xace19987, SUC_lexicon_tags_17 },
    { 0x49819988, SUC_lexicon_tags_0 },
    { 0xdba19988, SUC_lexicon_tags_37 },
    { 0xc4419986, SUC_lexicon_tags_29 },
    { 0xe741998b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6c61998d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x36619990, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57219993, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701999e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe199a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ec199a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf76199a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88a199a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8199ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c199af, SUC_lexicon_tags_12 },
    { 0x330199b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x710199b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1c2199b5, SUC_lexicon_tags_37 },
    { 0x898199b6, SUC_lexicon_tags_12 },
    { 0xf6e199b7, SUC_lexicon_tags_2 },
    { 0xbe2199b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b2199ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c199be, SUC_lexicon_tags_11 },
    { 0x8ce199bf, SUC_lexicon_tags_193 },
    { 0x7d4199c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0199c4, SUC_lexicon_tags_4 },
    { 0xad8199c5, SUC_lexicon_tags_27 },
    { 0xe28199c5, SUC_lexicon_tags_37 },
    { 0x380199c4, SUC_lexicon_tags_6 },
    { 0xae2199c8, SUC_lexicon_tags_46 },
    { 0x5a8199c9, SUC_lexicon_tags_2 },
    { 0x3b4199ca, SUC_lexicon_tags_32 },
    { 0xa66199cb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x544199ce, SUC_lexicon_tags_1 },
    { 0x5b8199ce, SUC_lexicon_tags_7 },
    { 0x2ec199d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x68199d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x780199d4, SUC_lexicon_tags_15 },
    { 0x1d0199d5, SUC_lexicon_tags_2 },
    { 0x1be199d5, SUC_lexicon_tags_1 },
    { 0x79c199d7, SUC_lexicon_tags_17 },
    { 0xef4199d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4199db, SUC_lexicon_tags_14 },
    { 0x604199db, SUC_lexicon_tags_25 },
    { 0x34c199dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a199e3, SUC_lexicon_tags_12 },
    { 0xf24199e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83c199e6, SUC_lexicon_tags_38 },
    { 0x9f0199e7, SUC_lexicon_tags_151 },
    { 0x3c6199e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a6199ea, SUC_lexicon_tags_0 },
    { 0xd18199eb, SUC_lexicon_tags_25 },
    { 0xf34199ec, SUC_lexicon_tags_15 },
    { 0xfbe199ed, SUC_lexicon_tags_21 },
    { 0xb74199ee, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a199f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x95c199f5, SUC_lexicon_tags_55 },
    { 0xf0c199f6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x498199f8, SUC_lexicon_tags_45 },
    { 0x172199f9, SUC_lexicon_tags_8 },
    { 0x40e199fa, SUC_lexicon_tags_37 },
    { 0xab4199fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea199fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc219a01, SUC_lexicon_tags_49 },
    { 0xcdc19a02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16819a06, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94419a0f, SUC_lexicon_tags_19 },
    { 0x84619a10, SUC_lexicon_tags_26 },
    { 0x1ec19a0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94619a15, SUC_lexicon_tags_33 },
    { 0x45819a16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98419a1a, SUC_lexicon_tags_15 },
    { 0xf4419a1b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x78219a1d, SUC_lexicon_tags_11 },
    { 0x71219a1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ce19a20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc419a26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25419a28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c19a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a019a2f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c019a36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84819a3c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x96e19a3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c019a40, SUC_lexicon_tags_129 },
    { 0x4be19a41, SUC_lexicon_tags_3 },
    { 0x7a619a42, SUC_lexicon_tags_7 },
    { 0xd9419a41, SUC_lexicon_tags_37 },
    { 0x10a19a44, SUC_lexicon_tags_49 },
    { 0xa1219a41, SUC_lexicon_tags_29 },
    { 0xe0019a46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0819a48, SUC_lexicon_tags_25 },
    { 0xfd419a49, SUC_lexicon_tags_11 },
    { 0xf5419a49, SUC_lexicon_tags_37 },
    { 0x71019a4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c19a55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c419a58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24819a5a, SUC_lexicon_tags_19 },
    { 0xea019a5b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0619a62, SUC_lexicon_tags_35 },
    { 0x88019a62, SUC_lexicon_tags_43 },
    { 0x94a19a64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e19a66, SUC_lexicon_tags_92 },
    { 0x3619a66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a19a6d, SUC_lexicon_tags_12 },
    { 0x80e19a6d, SUC_lexicon_tags_17 },
    { 0x82819a6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf419a73, SUC_lexicon_tags_23 },
    { 0x49c19a73, SUC_lexicon_tags_7 },
    { 0xcac19a75, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb819a77, SUC_lexicon_tags_29 },
    { 0x8019a77, SUC_lexicon_tags_23 },
    { 0x45a19a78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5419a7d, SUC_lexicon_tags_31 },
    { 0xd7419a7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a19a80, SUC_lexicon_tags_65 },
    { 0x1e219a81, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c19a86, SUC_lexicon_tags_48 },
    { 0xf6419a87, SUC_lexicon_tags_43 },
    { 0x3d419a86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95819a8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6ea19a8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4019a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2619a92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x19a19a94, SUC_lexicon_tags_1 },
    { 0xfd819a95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c19a98, SUC_lexicon_tags_15 },
    { 0xe9819a99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e19a9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac19aa1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c019aa6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a19aaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c19ab3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7419abc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d019abf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdde19ac1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad619ac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65419ace, SUC_lexicon_tags_32 },
    { 0xcd419acf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4819ad1, SUC_lexicon_tags_11 },
    { 0x97219ad2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c19ad5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c19ad9, SUC_lexicon_tags_2 },
    { 0xd7e19ada, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9419adc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1da19ade, SUC_lexicon_tags_6 },
    { 0x37819adf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4819ae2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca19ae8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc19aeb, SUC_lexicon_tags_17 },
    { 0xb7019aec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4819af0, SUC_lexicon_tags_1 },
    { 0xd3819af1, SUC_lexicon_tags_14 },
    { 0xcf019af1, SUC_lexicon_tags_25 },
    { 0x9619af3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c19af5, SUC_lexicon_tags_12 },
    { 0xd0019af6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca19af9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45419afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c19b05, SUC_lexicon_tags_2 },
    { 0x2c019b05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78219b0a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xad419b0c, SUC_lexicon_tags_2 },
    { 0x3fe19b0d, SUC_lexicon_tags_37 },
    { 0xbaa19b0d, SUC_lexicon_tags_254 },
    { 0x21819b0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e19b1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x22219b1f, SUC_lexicon_tags_1 },
    { 0x4c819b20, SUC_lexicon_tags_61 },
    { 0x2e619b20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51a19b23, SUC_lexicon_tags_17 },
    { 0x30219b23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x83019b27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca619b29, SUC_lexicon_tags_11 },
    { 0xa0219b2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xef419b2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c19b31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xad019b34, SUC_lexicon_tags_63 },
    { 0xba019b35, SUC_lexicon_tags_15 },
    { 0xf9a19b34, SUC_lexicon_tags_68 },
    { 0xd5619b37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36419b39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9619b41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18819b43, SUC_lexicon_tags_47 },
    { 0xea219b44, SUC_lexicon_tags_17 },
    { 0xa5019b45, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x37619b47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd3019b49, SUC_lexicon_tags_10 },
    { 0xc8219b4a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48819b4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c19b51, SUC_lexicon_tags_16 },
    { 0x8e19b51, SUC_lexicon_tags_17 },
    { 0x88019b51, SUC_lexicon_tags_28 },
    { 0x54c19b54, SUC_lexicon_tags_47 },
    { 0xc3019b54, SUC_lexicon_tags_12 },
    { 0xe7819b54, SUC_lexicon_tags_104 },
    { 0xff419b55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6419b5d, SUC_lexicon_tags_66 },
    { 0x3619b5e, SUC_lexicon_tags_17 },
    { 0x92619b5e, SUC_lexicon_tags_14 },
    { 0x36019b60, SUC_lexicon_tags_46 },
    { 0xe3019b61, SUC_lexicon_tags_12 },
    { 0x60219b62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0219b65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca19b6b, SUC_lexicon_tags_10 },
    { 0xbae19b6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9cc19b6e, SUC_lexicon_tags_37 },
    { 0x35619b6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a219b72, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x44219b74, SUC_lexicon_tags_12 },
    { 0xca219b75, SUC_lexicon_tags_14 },
    { 0x39019b75, SUC_lexicon_tags_10 },
    { 0x64619b75, SUC_lexicon_tags_7 },
    { 0x51219b77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8e19b7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe19b7d, SUC_lexicon_tags_12 },
    { 0x19819b7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17c19b80, SUC_lexicon_tags_2 },
    { 0x1fe19b81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70019b86, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f219b8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96419b95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4619b9d, SUC_lexicon_tags_11 },
    { 0xa7219b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0619ba1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd819ba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de19ba6, SUC_lexicon_tags_37 },
    { 0xbc419ba6, SUC_lexicon_tags_37 },
    { 0xa7219ba7, SUC_lexicon_tags_14 },
    { 0xd6619ba8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77219bac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c019bb0, SUC_lexicon_tags_0 },
    { 0x82819bb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6619bb7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x90419bb9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95819bbf, SUC_lexicon_tags_7 },
    { 0xfb619bbf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x42019bc2, SUC_lexicon_tags_2 },
    { 0x80019bc3, SUC_lexicon_tags_6 },
    { 0x2019bc4, SUC_lexicon_tags_28 },
    { 0x2ee19bc5, SUC_lexicon_tags_35 },
    { 0xc9a19bc6, SUC_lexicon_tags_29 },
    { 0x55219bc5, SUC_lexicon_tags_11 },
    { 0x8aa19bc7, SUC_lexicon_tags_25 },
    { 0xb419bc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31419bcc, SUC_lexicon_tags_14 },
    { 0x7ce19bcd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa5019bcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac19bd5, SUC_lexicon_tags_7 },
    { 0x4a019bd6, SUC_lexicon_tags_27 },
    { 0xce819bd6, SUC_lexicon_tags_49 },
    { 0xc7619bd7, SUC_lexicon_tags_40 },
    { 0x55219bd9, SUC_lexicon_tags_11 },
    { 0x11419bda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38219bdc, SUC_lexicon_tags_9 },
    { 0xf6219bdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f419be7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc19bec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe1019bee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec19bf3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x42219bf6, SUC_lexicon_tags_14 },
    { 0xdd819bf7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a19bfc, SUC_lexicon_tags_56 },
    { 0x78419bfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c019bff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a419c01, SUC_lexicon_tags_6 },
    { 0xef619c01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb219c04, SUC_lexicon_tags_30 },
    { 0xe1019c05, SUC_lexicon_tags_25 },
    { 0x9ea19c06, SUC_lexicon_tags_65 },
    { 0x5b419c06, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x75819c09, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe19c0e, SUC_lexicon_tags_2 },
    { 0x31c19c0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d419c11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee19c17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98419c19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8219c1b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7419c20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7819c24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x39819c26, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x95619c29, SUC_lexicon_tags_11 },
    { 0x4419c2a, SUC_lexicon_tags_216 },
    { 0xfd019c2b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a619c2f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x11619c31, SUC_lexicon_tags_38 },
    { 0xd0819c31, SUC_lexicon_tags_149 },
    { 0x87e19c33, SUC_lexicon_tags_25 },
    { 0x74e19c33, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x60219c36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f619c39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fc19c3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd0e19c3d, SUC_lexicon_tags_11 },
    { 0x25219c3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe5419c40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de19c43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c19c46, SUC_lexicon_tags_45 },
    { 0x1aa19c46, SUC_lexicon_tags_45 },
    { 0xe2219c48, SUC_lexicon_tags_5 },
    { 0x4e419c48, SUC_lexicon_tags_0 },
    { 0x93819c4a, SUC_lexicon_tags_63 },
    { 0x2619c4a, SUC_lexicon_tags_23 },
    { 0xa0819c4b, SUC_lexicon_tags_29 },
    { 0x82e19c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad419c50, SUC_lexicon_tags_1 },
    { 0x80819c51, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32219c53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a819c5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88419c62, SUC_lexicon_tags_12 },
    { 0xf2e19c63, SUC_lexicon_tags_1 },
    { 0x19e19c64, SUC_lexicon_tags_7 },
    { 0xf5419c63, SUC_lexicon_tags_11 },
    { 0xc7a19c66, SUC_lexicon_tags_9 },
    { 0x5b219c67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4019c6f, SUC_lexicon_tags_37 },
    { 0x1a819c70, SUC_lexicon_tags_66 },
    { 0x99a19c71, SUC_lexicon_tags_7 },
    { 0x8f819c72, SUC_lexicon_tags_1 },
    { 0x32a19c71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a19c77, SUC_lexicon_tags_15 },
    { 0xa6c19c78, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea19c7d, SUC_lexicon_tags_32 },
    { 0x4e819c7d, SUC_lexicon_tags_11 },
    { 0x40419c7f, SUC_lexicon_tags_45 },
    { 0x52c19c80, SUC_lexicon_tags_7 },
    { 0xf3a19c7e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee19c86, SUC_lexicon_tags_9 },
    { 0xef619c87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9219c8c, SUC_lexicon_tags_42 },
    { 0xfac19c8d, SUC_lexicon_tags_44 },
    { 0x6fa19c8c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e419c95, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0xf6819c97, SUC_lexicon_tags_108 },
    { 0x92819c98, SUC_lexicon_tags_15 },
    { 0xfe219c97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e019c9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16419c9e, SUC_lexicon_tags_27 },
    { 0xf3c19c9f, SUC_lexicon_tags_15 },
    { 0xa219ca0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde219ca4, SUC_lexicon_tags_29 },
    { 0x6819ca5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5219ca9, SUC_lexicon_tags_88 },
    { 0x65219ca9, SUC_lexicon_tags_7 },
    { 0x9c619cab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9219cb1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff019cb6, SUC_lexicon_tags_3 },
    { 0x2819cb7, SUC_lexicon_tags_33 },
    { 0x8fe19cb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc19cbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38019cbd, SUC_lexicon_tags_55 },
    { 0x50a19cbe, SUC_lexicon_tags_47 },
    { 0x99619cbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b019cc3, SUC_lexicon_tags_6 },
    { 0x7619cc4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff819cca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec219ccf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a19cd5, SUC_lexicon_tags_8 },
    { 0x5a419cd5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e19cdf, SUC_lexicon_tags_62 },
    { 0x4ca19ce0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5da19ce2, SUC_lexicon_tags_14 },
    { 0xbe19ce3, SUC_lexicon_tags_17 },
    { 0x1c419ce2, SUC_lexicon_tags_25 },
    { 0x6b619ce3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc19ce8, SUC_lexicon_tags_64 },
    { 0xd1619ce9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80019ceb, SUC_lexicon_tags_38 },
    { 0x6b419cec, SUC_lexicon_tags_1 },
    { 0xfae19ced, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc19cf0, SUC_lexicon_tags_1 },
    { 0x91819cf1, SUC_lexicon_tags_14 },
    { 0xfca19cf2, SUC_lexicon_tags_47 },
    { 0xd4219cf2, SUC_lexicon_tags_39 },
    { 0x58c19cf3, SUC_lexicon_tags_33 },
    { 0x70e19cf5, SUC_lexicon_tags_0 },
    { 0xd9819cf6, SUC_lexicon_tags_37 },
    { 0xfd419cf6, SUC_lexicon_tags_11 },
    { 0x1f419cf8, SUC_lexicon_tags_6 },
    { 0x86e19cf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90019cfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1819d00, SUC_lexicon_tags_25 },
    { 0x63419d01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9e19d03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27619d05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b619d08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa019d0a, SUC_lexicon_tags_1 },
    { 0xaa219d0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xda619d0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x31219d10, SUC_lexicon_tags_63 },
    { 0x16419d10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc419d15, SUC_lexicon_tags_12 },
    { 0x6ec19d16, SUC_lexicon_tags_28 },
    { 0x3219d16, SUC_lexicon_tags_25 },
    { 0x10819d15, SUC_lexicon_tags_15 },
    { 0x5aa19d19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5619d1c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x38e19d1e, SUC_lexicon_tags_17 },
    { 0x8a619d1e, SUC_lexicon_tags_7 },
    { 0x31819d20, SUC_lexicon_tags_2 },
    { 0xe2a19d21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a219d23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f019d28, SUC_lexicon_tags_40 },
    { 0x71619d29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a19d2e, SUC_lexicon_tags_1 },
    { 0xa7819d2e, SUC_lexicon_tags_7 },
    { 0x80419d30, SUC_lexicon_tags_7 },
    { 0x93819d30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c19d34, SUC_lexicon_tags_0 },
    { 0x68019d35, SUC_lexicon_tags_2 },
    { 0x4a19d34, SUC_lexicon_tags_50 },
    { 0x22419d37, SUC_lexicon_tags_25 },
    { 0x6da19d36, SUC_lexicon_tags_28 },
    { 0x54019d39, SUC_lexicon_tags_39 },
    { 0xd9c19d34, SUC_lexicon_tags_216 },
    { 0xa0419d3b, SUC_lexicon_tags_1 },
    { 0x85a19d39, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x72819d3f, SUC_lexicon_tags_29 },
    { 0x62c19d3f, SUC_lexicon_tags_2 },
    { 0x21219d41, SUC_lexicon_tags_37 },
    { 0x2aa19d40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc19d46, SUC_lexicon_tags_1 },
    { 0x4a819d47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e19d4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0219d4e, SUC_lexicon_tags_11 },
    { 0x77c19d4f, SUC_lexicon_tags_12 },
    { 0x19819d4f, SUC_lexicon_tags_27 },
    { 0x7019d4f, SUC_lexicon_tags_29 },
    { 0xc6819d52, SUC_lexicon_tags_48 },
    { 0xe8819d52, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa8419d55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5819d5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8419d5f, SUC_lexicon_tags_33 },
    { 0x4ce19d5f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd6a19d62, SUC_lexicon_tags_49 },
    { 0xb8e19d63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5819d66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94019d6c, SUC_lexicon_tags_37 },
    { 0x7c219d6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32219d7b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7ea19d7d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf9819d7f, SUC_lexicon_tags_0 },
    { 0xba819d7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee619d82, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e419d89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95819d8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e19d90, SUC_lexicon_tags_26 },
    { 0x9ca19d91, SUC_lexicon_tags_63 },
    { 0xd7219d92, SUC_lexicon_tags_37 },
    { 0x2a619d93, SUC_lexicon_tags_31 },
    { 0xc2c19d93, SUC_lexicon_tags_15 },
    { 0x52819d95, SUC_lexicon_tags_14 },
    { 0xe0019d96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a19d9a, SUC_lexicon_tags_17 },
    { 0x72419d9a, SUC_lexicon_tags_37 },
    { 0x9019d9c, SUC_lexicon_tags_15 },
    { 0xecc19d9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a19da1, SUC_lexicon_tags_11 },
    { 0xef819da2, SUC_lexicon_tags_8 },
    { 0x4ca19da1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9019da6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x87619da9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2019daf, SUC_lexicon_tags_12 },
    { 0x5619db0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25219db3, SUC_lexicon_tags_48 },
    { 0xf1019db3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa619db9, SUC_lexicon_tags_16 },
    { 0x29c19dba, SUC_lexicon_tags_11 },
    { 0xd5c19db9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc6a19dbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5819dc2, SUC_lexicon_tags_27 },
    { 0x93a19dc3, SUC_lexicon_tags_11 },
    { 0x52c19dc4, SUC_lexicon_tags_16 },
    { 0xf8419dc5, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x8ea19dc7, SUC_lexicon_tags_5 },
    { 0xa7819dc8, SUC_lexicon_tags_2 },
    { 0x4e219dc9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x95e19dcb, SUC_lexicon_tags_1 },
    { 0x61419dcc, SUC_lexicon_tags_47 },
    { 0xa1a19dcb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e19dd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92c19dd2, SUC_lexicon_tags_37 },
    { 0xf1019dd3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa2819dd5, SUC_lexicon_tags_16 },
    { 0xf0819dd5, SUC_lexicon_tags_11 },
    { 0x96c19dd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa19dda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d619dde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xed419de0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c19de3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a19de7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f019dea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b019dee, SUC_lexicon_tags_37 },
    { 0x1c619dee, SUC_lexicon_tags_15 },
    { 0x70419dee, SUC_lexicon_tags_19 },
    { 0xfd619def, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e19df4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f419df8, SUC_lexicon_tags_34 },
    { 0x7d419df9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd219dfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c219e00, SUC_lexicon_tags_37 },
    { 0xaf619e00, SUC_lexicon_tags_9 },
    { 0x5ea19e02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d819e04, SUC_lexicon_tags_17 },
    { 0xebc19e04, SUC_lexicon_tags_11 },
    { 0xda419e05, SUC_lexicon_tags_14 },
    { 0x9cc19e07, SUC_lexicon_tags_29 },
    { 0xc4e19e05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c019e0b, SUC_lexicon_tags_64 },
    { 0xe419e0b, SUC_lexicon_tags_8 },
    { 0x61819e0d, SUC_lexicon_tags_25 },
    { 0x7de19e0c, SUC_lexicon_tags_1 },
    { 0x5b419e0f, SUC_lexicon_tags_8 },
    { 0xe0819e10, SUC_lexicon_tags_17 },
    { 0xd3219e10, SUC_lexicon_tags_48 },
    { 0x8d219e12, SUC_lexicon_tags_7 },
    { 0x80419e13, SUC_lexicon_tags_0 },
    { 0xea19e13, SUC_lexicon_tags_32 },
    { 0x1e419e10, SUC_lexicon_tags_48 },
    { 0xdb619e0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf6819e18, SUC_lexicon_tags_32 },
    { 0x12c19e18, SUC_lexicon_tags_37 },
    { 0x6c19e1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57819e1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7619e21, SUC_lexicon_tags_17 },
    { 0x33a19e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc19e26, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x57c19e28, SUC_lexicon_tags_56 },
    { 0x10419e29, SUC_lexicon_tags_12 },
    { 0xc6019e2a, SUC_lexicon_tags_9 },
    { 0xc6019e2b, SUC_lexicon_tags_36 },
    { 0xb9a19e2b, SUC_lexicon_tags_11 },
    { 0x52c19e2b, SUC_lexicon_tags_43 },
    { 0xa5c19e28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56819e30, SUC_lexicon_tags_15 },
    { 0x58419e30, SUC_lexicon_tags_37 },
    { 0x78a19e30, SUC_lexicon_tags_68 },
    { 0x5ce19e33, SUC_lexicon_tags_17 },
    { 0x9b419e34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b019e37, SUC_lexicon_tags_253 },
    { 0x17019e38, SUC_lexicon_tags_31 },
    { 0x49a19e38, SUC_lexicon_tags_2 },
    { 0x2fc19e3a, SUC_lexicon_tags_12 },
    { 0x19c19e3b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf8e19e3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e219e40, SUC_lexicon_tags_7 },
    { 0xde219e41, SUC_lexicon_tags_25 },
    { 0x72e19e41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7619e49, SUC_lexicon_tags_146 },
    { 0x50019e4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0619e5a, SUC_lexicon_tags_7 },
    { 0x78619e5b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8619e5f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc19e64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfa19e66, SUC_lexicon_tags_37 },
    { 0x7e219e66, SUC_lexicon_tags_23 },
    { 0xf6c19e68, SUC_lexicon_tags_7 },
    { 0xb619e69, SUC_lexicon_tags_12 },
    { 0x5d019e69, SUC_lexicon_tags_15 },
    { 0xda619e6b, SUC_lexicon_tags_37 },
    { 0x6de19e6c, SUC_lexicon_tags_8 },
    { 0x73419e6d, SUC_lexicon_tags_23 },
    { 0xc0c19e6d, SUC_lexicon_tags_84 },
    { 0xa9a19e6e, SUC_lexicon_tags_18 },
    { 0xc8419e6f, SUC_lexicon_tags_25 },
    { 0xcfe19e6f, SUC_lexicon_tags_15 },
    { 0x44e19e6a, SUC_lexicon_tags_19 },
    { 0xa1e19e73, SUC_lexicon_tags_17 },
    { 0x82a19e74, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec19e79, SUC_lexicon_tags_45 },
    { 0xf9019e79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2aa19e7c, SUC_lexicon_tags_23 },
    { 0xe3e19e7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71819e80, SUC_lexicon_tags_16 },
    { 0x97c19e80, SUC_lexicon_tags_19 },
    { 0xdc819e82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd7619e84, SUC_lexicon_tags_0 },
    { 0xe6819e85, SUC_lexicon_tags_39 },
    { 0x35019e86, SUC_lexicon_tags_10 },
    { 0x74a19e87, SUC_lexicon_tags_0 },
    { 0xbc019e85, SUC_lexicon_tags_7 },
    { 0xc4819e87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5019e8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c619e8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e19e91, SUC_lexicon_tags_2 },
    { 0x23819e92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c19e9d, SUC_lexicon_tags_0 },
    { 0x18419e9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1c19ea0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18819ea5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f419eae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x48e19eb0, SUC_lexicon_tags_19 },
    { 0x46619eb0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x34819eb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa19eb7, SUC_lexicon_tags_3 },
    { 0x8c619eb8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e19ebd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19019ebf, SUC_lexicon_tags_2 },
    { 0xb019ec0, SUC_lexicon_tags_9 },
    { 0x7e819ec0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4219ec3, SUC_lexicon_tags_1 },
    { 0x4c419ec4, SUC_lexicon_tags_311 },
    { 0xc2219ec3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9019ece, SUC_lexicon_tags_63 },
    { 0xadc19ecf, SUC_lexicon_tags_63 },
    { 0x46619ecf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7819ed4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee19eda, SUC_lexicon_tags_23 },
    { 0xbd419edb, SUC_lexicon_tags_15 },
    { 0xd4c19edb, SUC_lexicon_tags_31 },
    { 0x1bc19edd, SUC_lexicon_tags_11 },
    { 0xb7419eda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c419ee2, SUC_lexicon_tags_15 },
    { 0xa1019ee2, SUC_lexicon_tags_17 },
    { 0x86e19ee3, SUC_lexicon_tags_37 },
    { 0x7b619ee5, SUC_lexicon_tags_69 },
    { 0xd5419ee5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa419eec, SUC_lexicon_tags_17 },
    { 0x5d419eed, SUC_lexicon_tags_66 },
    { 0x8c219eec, SUC_lexicon_tags_2 },
    { 0xb8819eef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x80019ef2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a619ef8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe4019efa, SUC_lexicon_tags_6 },
    { 0x61819efb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d219f00, SUC_lexicon_tags_26 },
    { 0xa19f01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9419f05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f219f08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7619f0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3819f12, SUC_lexicon_tags_11 },
    { 0x9e819f13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeac19f15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc19f1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfde19f21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a619f24, SUC_lexicon_tags_12 },
    { 0x1c619f25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x70a19f27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73819f29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e619f2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d019f30, SUC_lexicon_tags_131 },
    { 0xe3419f31, SUC_lexicon_tags_31 },
    { 0x4819f31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11219f37, SUC_lexicon_tags_2 },
    { 0x97619f37, SUC_lexicon_tags_16 },
    { 0x4819f39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55619f3b, SUC_lexicon_tags_37 },
    { 0x35619f3b, SUC_lexicon_tags_81 },
    { 0xde619f3d, SUC_lexicon_tags_0 },
    { 0x21a19f3e, SUC_lexicon_tags_63 },
    { 0x16a19f3f, SUC_lexicon_tags_2 },
    { 0xb1e19f3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11019f42, SUC_lexicon_tags_37 },
    { 0x8ac19f42, SUC_lexicon_tags_37 },
    { 0xa6019f44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81219f46, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9ec19f48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94419f4c, SUC_lexicon_tags_12 },
    { 0x43219f4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2019f57, SUC_lexicon_tags_2 },
    { 0x89c19f58, SUC_lexicon_tags_25 },
    { 0xaee19f59, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de19f5d, SUC_lexicon_tags_2 },
    { 0xe2419f5e, SUC_lexicon_tags_37 },
    { 0x7dc19f5d, SUC_lexicon_tags_7 },
    { 0xfd819f60, SUC_lexicon_tags_32 },
    { 0x7c419f5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d419f65, SUC_lexicon_tags_33 },
    { 0xa019f65, SUC_lexicon_tags_23 },
    { 0xbf419f67, SUC_lexicon_tags_1 },
    { 0xf6019f68, SUC_lexicon_tags_17 },
    { 0xac419f69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x95019f6b, SUC_lexicon_tags_8 },
    { 0x64a19f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29419f70, SUC_lexicon_tags_19 },
    { 0xa3219f71, SUC_lexicon_tags_82 },
    { 0x71e19f70, SUC_lexicon_tags_17 },
    { 0x2aa19f71, SUC_lexicon_tags_0 },
    { 0x5e419f73, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb819f7a, SUC_lexicon_tags_37 },
    { 0x54c19f7b, SUC_lexicon_tags_12 },
    { 0x16619f7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a19f7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85619f81, SUC_lexicon_tags_1 },
    { 0x4cc19f82, SUC_lexicon_tags_15 },
    { 0x96e19f83, SUC_lexicon_tags_7 },
    { 0xaa19f82, SUC_lexicon_tags_37 },
    { 0xaf219f85, SUC_lexicon_tags_37 },
    { 0xa819f85, SUC_lexicon_tags_31 },
    { 0xec219f87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78419f89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x43819f8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e19f90, SUC_lexicon_tags_17 },
    { 0xd7019f91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e219f93, SUC_lexicon_tags_12 },
    { 0xe419f94, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e19f9a, SUC_lexicon_tags_8 },
    { 0x6cc19f9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53419f9d, SUC_lexicon_tags_6 },
    { 0xb6c19f9e, SUC_lexicon_tags_2 },
    { 0xb8c19f9f, SUC_lexicon_tags_12 },
    { 0x1ec19fa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d619fa4, SUC_lexicon_tags_1 },
    { 0xf0819fa5, SUC_lexicon_tags_19 },
    { 0x3b619fa4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b019fae, SUC_lexicon_tags_55 },
    { 0x89419fae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50019fb3, SUC_lexicon_tags_35 },
    { 0x1f419fb3, SUC_lexicon_tags_194 },
    { 0xaba19fb3, SUC_lexicon_tags_37 },
    { 0xd0219fb6, SUC_lexicon_tags_12 },
    { 0xbcc19fb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd219fbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc19fc2, SUC_lexicon_tags_15 },
    { 0xf3019fc3, SUC_lexicon_tags_14 },
    { 0xbe19fc4, SUC_lexicon_tags_1 },
    { 0xffe19fc5, SUC_lexicon_tags_30 },
    { 0xa1419fc6, SUC_lexicon_tags_9 },
    { 0x45219fc7, SUC_lexicon_tags_7 },
    { 0x70219fc5, SUC_lexicon_tags_4 },
    { 0x14a19fc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c19fcd, SUC_lexicon_tags_7 },
    { 0x9cc19fce, SUC_lexicon_tags_66 },
    { 0xfe219fcf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da19fd3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c19fd6, SUC_lexicon_tags_15 },
    { 0xda219fd6, SUC_lexicon_tags_25 },
    { 0xa6a19fd8, SUC_lexicon_tags_26 },
    { 0xdbc19fd8, SUC_lexicon_tags_36 },
    { 0x51219fda, SUC_lexicon_tags_14 },
    { 0xb3619fda, SUC_lexicon_tags_15 },
    { 0xa8419fd8, SUC_lexicon_tags_9 },
    { 0xcb419fdd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc19fe0, SUC_lexicon_tags_2 },
    { 0xb9e19fe1, SUC_lexicon_tags_23 },
    { 0x3a19fe0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee19fe7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4219fee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x56219ff1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f819ff3, SUC_lexicon_tags_45 },
    { 0x7ec19ff3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2d619ff6, SUC_lexicon_tags_49 },
    { 0x41619ff7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa19ffa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7019ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9841a002, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c1a007, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc1a00f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb81a012, SUC_lexicon_tags_40 },
    { 0x7f21a013, SUC_lexicon_tags_12 },
    { 0x4e61a013, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf361a019, SUC_lexicon_tags_11 },
    { 0x9a1a01a, SUC_lexicon_tags_29 },
    { 0x9721a01a, SUC_lexicon_tags_46 },
    { 0x19c1a019, SUC_lexicon_tags_92 },
    { 0x9541a01d, SUC_lexicon_tags_12 },
    { 0xd121a01e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d61a020, SUC_lexicon_tags_2 },
    { 0x1d61a021, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe1a029, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x42a1a02b, SUC_lexicon_tags_38 },
    { 0xfd81a02c, SUC_lexicon_tags_235 },
    { 0x4d81a02d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2c01a02f, SUC_lexicon_tags_7 },
    { 0xdb21a030, SUC_lexicon_tags_15 },
    { 0xac41a031, SUC_lexicon_tags_28 },
    { 0x3e21a030, SUC_lexicon_tags_17 },
    { 0xba41a033, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c1a03b, SUC_lexicon_tags_39 },
    { 0xb2a1a03c, SUC_lexicon_tags_16 },
    { 0x5b61a03d, SUC_lexicon_tags_0 },
    { 0xe7e1a03d, SUC_lexicon_tags_47 },
    { 0x1c01a03f, SUC_lexicon_tags_48 },
    { 0x26a1a040, SUC_lexicon_tags_48 },
    { 0x4641a03f, SUC_lexicon_tags_6 },
    { 0x87c1a03d, SUC_lexicon_tags_48 },
    { 0xc301a03e, SUC_lexicon_tags_7 },
    { 0xba21a042, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca1a048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab41a04d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x601a04f, SUC_lexicon_tags_42 },
    { 0xd6a1a050, SUC_lexicon_tags_95 },
    { 0xe801a050, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe201a05f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a1a062, SUC_lexicon_tags_15 },
    { 0x7341a062, SUC_lexicon_tags_1 },
    { 0x70e1a064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14a1a066, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7741a069, SUC_lexicon_tags_25 },
    { 0xe461a06a, SUC_lexicon_tags_68 },
    { 0x82c1a06b, SUC_lexicon_tags_42 },
    { 0x2221a06a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x841a06e, SUC_lexicon_tags_11 },
    { 0xeaa1a06e, SUC_lexicon_tags_7 },
    { 0x7fe1a070, SUC_lexicon_tags_65 },
    { 0x5841a071, SUC_lexicon_tags_2 },
    { 0xc701a070, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe841a075, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1821a077, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae1a07c, SUC_lexicon_tags_47 },
    { 0x7e1a07c, SUC_lexicon_tags_2 },
    { 0x4721a07e, SUC_lexicon_tags_11 },
    { 0x1ba1a07f, SUC_lexicon_tags_38 },
    { 0xaac1a07f, SUC_lexicon_tags_17 },
    { 0x5161a081, SUC_lexicon_tags_66 },
    { 0x66a1a082, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc241a084, SUC_lexicon_tags_66 },
    { 0xc501a085, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb201a087, SUC_lexicon_tags_17 },
    { 0x5fe1a087, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x26e1a08a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e1a092, SUC_lexicon_tags_26 },
    { 0xe5a1a093, SUC_lexicon_tags_11 },
    { 0x3921a094, SUC_lexicon_tags_25 },
    { 0xc221a095, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa841a09d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc41a0a1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9fe1a0a3, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca1a0a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4081a0ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1781a0b1, SUC_lexicon_tags_37 },
    { 0x4321a0b2, SUC_lexicon_tags_25 },
    { 0x1d01a0b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xee01a0b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafa1a0b7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9ee1a0b9, SUC_lexicon_tags_17 },
    { 0xf4a1a0ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2861a0be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd281a0c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6101a0c3, SUC_lexicon_tags_2 },
    { 0x2581a0c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3881a0c6, SUC_lexicon_tags_38 },
    { 0x23c1a0c6, SUC_lexicon_tags_15 },
    { 0x37a1a0c8, SUC_lexicon_tags_1 },
    { 0x3b01a0c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2261a0d1, SUC_lexicon_tags_9 },
    { 0xfc61a0d2, SUC_lexicon_tags_84 },
    { 0x17e1a0d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e81a0d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b81a0da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8621a0e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1a0e6, SUC_lexicon_tags_45 },
    { 0x4f41a0e7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3861a0e9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc141a0ed, SUC_lexicon_tags_7 },
    { 0x661a0ee, SUC_lexicon_tags_23 },
    { 0xd961a0ee, SUC_lexicon_tags_55 },
    { 0xac01a0f0, SUC_lexicon_tags_7 },
    { 0xe901a0ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e1a0f4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41a0f8, SUC_lexicon_tags_23 },
    { 0x3501a0f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca1a0fe, SUC_lexicon_tags_2 },
    { 0xb2c1a0ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d01a101, SUC_lexicon_tags_12 },
    { 0x2521a102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3e1a104, SUC_lexicon_tags_15 },
    { 0xe521a104, SUC_lexicon_tags_15 },
    { 0x7d21a106, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xee41a108, SUC_lexicon_tags_28 },
    { 0x7b01a109, SUC_lexicon_tags_336 },
    { 0xa101a108, SUC_lexicon_tags_0 },
    { 0xf641a10a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d81a10e, SUC_lexicon_tags_15 },
    { 0x23a1a10f, SUC_lexicon_tags_17 },
    { 0xadc1a10e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfce1a112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27e1a114, SUC_lexicon_tags_2 },
    { 0x5a21a115, SUC_lexicon_tags_33 },
    { 0x5e01a114, SUC_lexicon_tags_8 },
    { 0x8e1a114, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe221a11e, SUC_lexicon_tags_11 },
    { 0xe4c1a11f, SUC_lexicon_tags_229 },
    { 0xafe1a11f, SUC_lexicon_tags_57 },
    { 0x5ce1a121, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2021a123, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c1a127, SUC_lexicon_tags_16 },
    { 0x5fc1a128, SUC_lexicon_tags_2 },
    { 0x5961a128, SUC_lexicon_tags_11 },
    { 0x461a12a, SUC_lexicon_tags_25 },
    { 0xeb41a12b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb661a12e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5bc1a130, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2061a133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x721a135, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa241a138, SUC_lexicon_tags_19 },
    { 0x9161a139, SUC_lexicon_tags_16 },
    { 0xb061a13a, SUC_lexicon_tags_37 },
    { 0x1321a13b, SUC_lexicon_tags_25 },
    { 0x7741a13c, SUC_lexicon_tags_2 },
    { 0x1241a13c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x641a140, SUC_lexicon_tags_10 },
    { 0x30e1a141, SUC_lexicon_tags_2 },
    { 0x581a140, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e1a147, SUC_lexicon_tags_83 },
    { 0x8141a148, SUC_lexicon_tags_31 },
    { 0x4401a149, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a1a14f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a1a153, SUC_lexicon_tags_160 },
    { 0x8b41a154, SUC_lexicon_tags_35 },
    { 0x2281a155, SUC_lexicon_tags_17 },
    { 0xca21a156, SUC_lexicon_tags_1 },
    { 0x3201a157, SUC_lexicon_tags_12 },
    { 0x84e1a158, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c1a160, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9e1a162, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd41a167, SUC_lexicon_tags_7 },
    { 0xcb41a168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa581a16a, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x3a01a16c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa441a171, SUC_lexicon_tags_12 },
    { 0xf281a172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80c1a174, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f61a176, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd561a17d, SUC_lexicon_tags_49 },
    { 0x73e1a17e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf821a180, SUC_lexicon_tags_57 },
    { 0x5401a181, SUC_lexicon_tags_12 },
    { 0xafa1a180, SUC_lexicon_tags_34 },
    { 0xac21a181, SUC_lexicon_tags_1 },
    { 0x67a1a184, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c1a187, SUC_lexicon_tags_87 },
    { 0x61c1a187, SUC_lexicon_tags_68 },
    { 0x3981a189, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c1a194, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf021a196, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca1a19e, SUC_lexicon_tags_7 },
    { 0xb421a19e, SUC_lexicon_tags_19 },
    { 0x8841a1a0, SUC_lexicon_tags_47 },
    { 0xb121a1a1, SUC_lexicon_tags_11 },
    { 0xffc1a1a2, SUC_lexicon_tags_1 },
    { 0x7981a19e, SUC_lexicon_tags_0 },
    { 0x321a1a2, SUC_lexicon_tags_60 },
    { 0x6581a19e, SUC_lexicon_tags_10 },
    { 0xaa41a1a6, SUC_lexicon_tags_1 },
    { 0x3bc1a1a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a81a1aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2361a1af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f21a1b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6901a1b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a1a1b7, SUC_lexicon_tags_3 },
    { 0x3d21a1b8, SUC_lexicon_tags_49 },
    { 0xc2c1a1b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf21a1be, SUC_lexicon_tags_1 },
    { 0xe801a1bf, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4d21a1c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8241a1c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a1a1c8, SUC_lexicon_tags_17 },
    { 0xd81a1c8, SUC_lexicon_tags_49 },
    { 0x4121a1c9, SUC_lexicon_tags_12 },
    { 0x9361a1cb, SUC_lexicon_tags_0 },
    { 0xc221a1c8, SUC_lexicon_tags_91 },
    { 0x6761a1cd, SUC_lexicon_tags_9 },
    { 0x3c81a1ca, SUC_lexicon_tags_0 },
    { 0x7c01a1cf, SUC_lexicon_tags_7 },
    { 0x7841a1cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce1a1d4, SUC_lexicon_tags_11 },
    { 0x61e1a1d5, SUC_lexicon_tags_47 },
    { 0xfe61a1d6, SUC_lexicon_tags_1 },
    { 0x7de1a1d7, SUC_lexicon_tags_11 },
    { 0x7ce1a1d4, SUC_lexicon_tags_49 },
    { 0x841a1d9, SUC_lexicon_tags_28 },
    { 0x1e61a1da, SUC_lexicon_tags_160 },
    { 0x7a61a1db, SUC_lexicon_tags_37 },
    { 0xd821a1db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c1a1df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a1a1e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe921a1e7, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xc4e1a1e9, SUC_lexicon_tags_390 },
    { 0xbcc1a1ea, SUC_lexicon_tags_17 },
    { 0xed81a1eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c1a1ef, SUC_lexicon_tags_48 },
    { 0x8281a1f0, SUC_lexicon_tags_8 },
    { 0xfa01a1f1, SUC_lexicon_tags_45 },
    { 0xea1a1f2, SUC_lexicon_tags_33 },
    { 0x5921a1f3, SUC_lexicon_tags_17 },
    { 0xa41a1f1, SUC_lexicon_tags_14 },
    { 0x5c61a1f2, SUC_lexicon_tags_12 },
    { 0x95a1a1f0, SUC_lexicon_tags_11 },
    { 0xa541a1f7, SUC_lexicon_tags_14 },
    { 0xe521a1f7, SUC_lexicon_tags_23 },
    { 0xdce1a1f7, SUC_lexicon_tags_32 },
    { 0x82c1a1f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e1a1fd, SUC_lexicon_tags_37 },
    { 0x8fa1a1fd, SUC_lexicon_tags_31 },
    { 0xf601a1ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc1a202, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7281a207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ee1a209, SUC_lexicon_tags_1 },
    { 0x6ac1a20a, SUC_lexicon_tags_87 },
    { 0x9321a20b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c41a210, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40a1a212, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4081a215, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2421a21c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de1a21f, SUC_lexicon_tags_0 },
    { 0x67a1a21f, SUC_lexicon_tags_19 },
    { 0x6c61a221, SUC_lexicon_tags_8 },
    { 0x6361a222, SUC_lexicon_tags_68 },
    { 0x2001a221, SUC_lexicon_tags_21 },
    { 0x4aa1a224, SUC_lexicon_tags_2 },
    { 0xf561a21f, SUC_lexicon_tags_15 },
    { 0xa501a226, SUC_lexicon_tags_32 },
    { 0x1b21a227, SUC_lexicon_tags_15 },
    { 0xc921a227, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac1a22d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe1a231, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a01a233, SUC_lexicon_tags_33 },
    { 0x8841a233, SUC_lexicon_tags_65 },
    { 0xc7a1a233, SUC_lexicon_tags_47 },
    { 0xb5e1a236, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d41a23b, SUC_lexicon_tags_29 },
    { 0xa961a23c, SUC_lexicon_tags_15 },
    { 0xa781a23c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd041a23f, SUC_lexicon_tags_43 },
    { 0xbb41a240, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4821a248, SUC_lexicon_tags_17 },
    { 0xcc41a249, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8841a24c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd621a24e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd381a250, SUC_lexicon_tags_1 },
    { 0xe9e1a250, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa1a254, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b61a256, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301a25a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5941a25e, SUC_lexicon_tags_17 },
    { 0xc461a25f, SUC_lexicon_tags_2 },
    { 0x9ea1a25f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a21a263, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53e1a265, SUC_lexicon_tags_71 },
    { 0xe1e1a266, SUC_lexicon_tags_12 },
    { 0x9cc1a266, SUC_lexicon_tags_8 },
    { 0x5c81a267, SUC_lexicon_tags_37 },
    { 0xc781a268, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec1a26e, SUC_lexicon_tags_11 },
    { 0xa1e1a26f, SUC_lexicon_tags_49 },
    { 0xe81a270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7921a272, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8981a27e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a1a285, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb6a1a287, SUC_lexicon_tags_17 },
    { 0xe7c1a287, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd521a292, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x621a294, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e81a297, SUC_lexicon_tags_38 },
    { 0x1d01a298, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xdb61a29a, SUC_lexicon_tags_262 },
    { 0xf41a29b, SUC_lexicon_tags_7 },
    { 0xf741a29c, SUC_lexicon_tags_477 },
    { 0xec21a29b, SUC_lexicon_tags_32 },
    { 0x6221a29d, SUC_lexicon_tags_10 },
    { 0xf781a29f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9621a2a1, SUC_lexicon_tags_2 },
    { 0xc5e1a2a2, SUC_lexicon_tags_63 },
    { 0x1b81a2a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcc41a2a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3701a2ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x841a2af, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d01a2b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841a2ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73c1a2bc, SUC_lexicon_tags_10 },
    { 0xe21a2bd, SUC_lexicon_tags_25 },
    { 0x9301a2be, SUC_lexicon_tags_23 },
    { 0x44a1a2be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf01a2c1, SUC_lexicon_tags_46 },
    { 0x51a1a2c2, SUC_lexicon_tags_16 },
    { 0xd9e1a2c3, SUC_lexicon_tags_13 },
    { 0x69a1a2c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2ca1a2c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd321a2c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a81a2ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e1a2d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe481a2d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f61a2d6, SUC_lexicon_tags_11 },
    { 0xe461a2d6, SUC_lexicon_tags_7 },
    { 0x61a2d8, SUC_lexicon_tags_79 },
    { 0x2201a2d9, SUC_lexicon_tags_8 },
    { 0x3261a2d8, SUC_lexicon_tags_2 },
    { 0x3e21a2d6, SUC_lexicon_tags_17 },
    { 0xd921a2dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c1a2e0, SUC_lexicon_tags_17 },
    { 0x53e1a2e1, SUC_lexicon_tags_60 },
    { 0xbc01a2e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd021a2e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe081a2ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa1a2f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b81a2f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x47e1a2f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2581a2fa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc141a2fe, SUC_lexicon_tags_25 },
    { 0xb9a1a2ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d21a302, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd7c1a304, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe841a307, SUC_lexicon_tags_7 },
    { 0xd021a308, SUC_lexicon_tags_12 },
    { 0x6241a309, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1a30c, SUC_lexicon_tags_25 },
    { 0x7641a30c, SUC_lexicon_tags_12 },
    { 0x9241a30e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8261a311, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ac1a313, SUC_lexicon_tags_30 },
    { 0x6081a313, SUC_lexicon_tags_9 },
    { 0x281a313, SUC_lexicon_tags_8 },
    { 0x9581a316, SUC_lexicon_tags_9 },
    { 0x12e1a317, SUC_lexicon_tags_17 },
    { 0xb801a313, SUC_lexicon_tags_11 },
    { 0x281a313, SUC_lexicon_tags_33 },
    { 0xaa21a31a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1a31d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a1a322, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x19c1a324, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761a32b, SUC_lexicon_tags_60 },
    { 0xe4c1a32c, SUC_lexicon_tags_1138 },
    { 0xf821a32c, SUC_lexicon_tags_55 },
    { 0x53e1a32d, SUC_lexicon_tags_134 },
    { 0x4961a32f, SUC_lexicon_tags_2 },
    { 0x72e1a32f, SUC_lexicon_tags_84 },
    { 0x8901a331, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2981a333, SUC_lexicon_tags_16 },
    { 0x5de1a334, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4841a339, SUC_lexicon_tags_27 },
    { 0x63a1a33a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd701a33c, SUC_lexicon_tags_11 },
    { 0xafa1a33c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa221a33f, SUC_lexicon_tags_25 },
    { 0xee01a33f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdac1a342, SUC_lexicon_tags_11 },
    { 0x7941a343, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e1a346, SUC_lexicon_tags_47 },
    { 0xb81a347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2e1a349, SUC_lexicon_tags_60 },
    { 0x1f61a349, SUC_lexicon_tags_30 },
    { 0x8dc1a34a, SUC_lexicon_tags_31 },
    { 0xf41a34c, SUC_lexicon_tags_15 },
    { 0x5061a34c, SUC_lexicon_tags_51 },
    { 0xba81a34e, SUC_lexicon_tags_1 },
    { 0xa581a34e, SUC_lexicon_tags_68 },
    { 0x5801a349, SUC_lexicon_tags_7 },
    { 0xc0a1a351, SUC_lexicon_tags_55 },
    { 0x2c81a352, SUC_lexicon_tags_84 },
    { 0xfba1a353, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a1a358, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a1a35e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa1a363, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd41a367, SUC_lexicon_tags_26 },
    { 0x78a1a368, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc1a36d, SUC_lexicon_tags_1 },
    { 0x6661a36d, SUC_lexicon_tags_12 },
    { 0xa9a1a36f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a1a372, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e21a379, SUC_lexicon_tags_17 },
    { 0xe41a37a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa781a37c, SUC_lexicon_tags_11 },
    { 0xbba1a37d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9101a37f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4561a384, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb781a386, SUC_lexicon_tags_23 },
    { 0xb501a387, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6961a38a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e1a38e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf641a392, SUC_lexicon_tags_35 },
    { 0xc561a392, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa821a396, SUC_lexicon_tags_17 },
    { 0x9aa1a396, SUC_lexicon_tags_1 },
    { 0x3561a398, SUC_lexicon_tags_19 },
    { 0xeba1a398, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc741a39f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5141a3a2, SUC_lexicon_tags_15 },
    { 0xc941a3a3, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0xe6c1a3a5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf41a3ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1161a3ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4721a3b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe441a3b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7141a3b6, SUC_lexicon_tags_7 },
    { 0x5c01a3b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e21a3ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c01a3bf, SUC_lexicon_tags_49 },
    { 0xd2e1a3c0, SUC_lexicon_tags_35 },
    { 0xc6a1a3c1, SUC_lexicon_tags_26 },
    { 0x44e1a3c2, SUC_lexicon_tags_14 },
    { 0xbb61a3c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc21a3c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c1a3cd, SUC_lexicon_tags_26 },
    { 0xb5c1a3ce, SUC_lexicon_tags_17 },
    { 0xd481a3cf, SUC_lexicon_tags_38 },
    { 0x7d41a3d0, SUC_lexicon_tags_26 },
    { 0xc421a3d1, SUC_lexicon_tags_25 },
    { 0x72c1a3d1, SUC_lexicon_tags_27 },
    { 0xcb81a3d2, SUC_lexicon_tags_46 },
    { 0xd2a1a3d0, SUC_lexicon_tags_25 },
    { 0x65c1a3d2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1f81a3d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a1a3df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a01a3e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c1a3e4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9541a3e7, SUC_lexicon_tags_92 },
    { 0x3b21a3e8, SUC_lexicon_tags_63 },
    { 0x9ba1a3e9, SUC_lexicon_tags_15 },
    { 0xf981a3e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad61a3ef, SUC_lexicon_tags_37 },
    { 0x8061a3f0, SUC_lexicon_tags_9 },
    { 0x1361a3f1, SUC_lexicon_tags_0 },
    { 0x2e81a3f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d61a3f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4421a3f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd841a3f9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac1a3fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb981a401, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6381a403, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x67e1a405, SUC_lexicon_tags_25 },
    { 0x4c41a406, SUC_lexicon_tags_13 },
    { 0x3001a405, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d01a411, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec81a418, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be1a41c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa1a421, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe761a424, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf681a427, SUC_lexicon_tags_25 },
    { 0x68e1a427, SUC_lexicon_tags_25 },
    { 0x5341a429, SUC_lexicon_tags_8 },
    { 0xdbe1a42a, SUC_lexicon_tags_47 },
    { 0xc701a429, SUC_lexicon_tags_107 },
    { 0xc541a42c, SUC_lexicon_tags_37 },
    { 0xc561a42b, SUC_lexicon_tags_38 },
    { 0xe0e1a42c, SUC_lexicon_tags_17 },
    { 0x5001a42f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x421a431, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c1a43a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7341a43c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2a1a43e, SUC_lexicon_tags_17 },
    { 0x8401a43f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbb21a441, SUC_lexicon_tags_12 },
    { 0xb661a441, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca1a446, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac1a44e, SUC_lexicon_tags_18 },
    { 0x4d01a44e, SUC_lexicon_tags_25 },
    { 0xae61a44f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c81a458, SUC_lexicon_tags_17 },
    { 0x1a458, SUC_lexicon_tags_23 },
    { 0x32a1a45a, SUC_lexicon_tags_21 },
    { 0xff41a45a, SUC_lexicon_tags_15 },
    { 0xe061a45c, SUC_lexicon_tags_95 },
    { 0x40e1a45c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59e1a45f, SUC_lexicon_tags_65 },
    { 0x2a21a460, SUC_lexicon_tags_7 },
    { 0xdb21a45f, SUC_lexicon_tags_15 },
    { 0x5541a462, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xac21a464, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x401a466, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6181a468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed21a46a, SUC_lexicon_tags_60 },
    { 0x53e1a46b, SUC_lexicon_tags_21 },
    { 0x5d61a46a, SUC_lexicon_tags_12 },
    { 0xd721a46a, SUC_lexicon_tags_0 },
    { 0x7e61a46e, SUC_lexicon_tags_19 },
    { 0x741a46f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6921a471, SUC_lexicon_tags_37 },
    { 0xc4e1a472, SUC_lexicon_tags_35 },
    { 0x84a1a472, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xce81a476, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e1a47a, SUC_lexicon_tags_1 },
    { 0x4281a47b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6de1a47d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f81a47f, SUC_lexicon_tags_33 },
    { 0xbae1a480, SUC_lexicon_tags_217 },
    { 0xb401a481, SUC_lexicon_tags_15 },
    { 0xc101a480, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2921a487, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c1a48b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca1a48f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c1a491, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d61a495, SUC_lexicon_tags_15 },
    { 0x7e61a496, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181a4a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9701a4a6, SUC_lexicon_tags_27 },
    { 0xe841a4a7, SUC_lexicon_tags_25 },
    { 0xd181a4a6, SUC_lexicon_tags_131 },
    { 0x69e1a4a6, SUC_lexicon_tags_28 },
    { 0x2561a4a7, SUC_lexicon_tags_27 },
    { 0xb681a4ab, SUC_lexicon_tags_17 },
    { 0x7ce1a4ab, SUC_lexicon_tags_11 },
    { 0x5441a4ad, SUC_lexicon_tags_25 },
    { 0x9401a4ae, SUC_lexicon_tags_11 },
    { 0x32a1a4ad, SUC_lexicon_tags_45 },
    { 0x50c1a4b0, SUC_lexicon_tags_17 },
    { 0x8b21a4b0, SUC_lexicon_tags_2 },
    { 0x281a4b1, SUC_lexicon_tags_1 },
    { 0x4501a4ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c1a4b7, SUC_lexicon_tags_45 },
    { 0x1241a4b7, SUC_lexicon_tags_21 },
    { 0x2e81a4b9, SUC_lexicon_tags_39 },
    { 0x6ee1a4ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae1a4bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2441a4c2, SUC_lexicon_tags_11 },
    { 0xa461a4c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf421a4c9, SUC_lexicon_tags_57 },
    { 0xc2a1a4ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3681a4cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a61a4d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe941a4d3, SUC_lexicon_tags_17 },
    { 0xe241a4d3, SUC_lexicon_tags_17 },
    { 0x5f21a4d3, SUC_lexicon_tags_12 },
    { 0x7ba1a4d5, SUC_lexicon_tags_65 },
    { 0xf561a4d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc321a4e4, SUC_lexicon_tags_47 },
    { 0x7521a4e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41a4eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d21a4ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd21a4f3, SUC_lexicon_tags_39 },
    { 0xa241a4f4, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x1181a4f7, SUC_lexicon_tags_26 },
    { 0xca81a4f8, SUC_lexicon_tags_17 },
    { 0x9281a4f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe0a1a4fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e1a501, SUC_lexicon_tags_11 },
    { 0x9101a501, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf21a506, SUC_lexicon_tags_37 },
    { 0xfd61a507, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8101a50a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a1a50e, SUC_lexicon_tags_2 },
    { 0x2c1a50f, SUC_lexicon_tags_37 },
    { 0x9f81a510, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc1a513, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1701a519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada1a51e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2821a520, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab61a526, SUC_lexicon_tags_47 },
    { 0x10e1a527, SUC_lexicon_tags_15 },
    { 0x9161a526, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53c1a52a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf541a52c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x341a52e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a21a530, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd81a532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90e1a534, SUC_lexicon_tags_37 },
    { 0x5661a534, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfba1a537, SUC_lexicon_tags_15 },
    { 0x8cc1a538, SUC_lexicon_tags_2 },
    { 0x2981a537, SUC_lexicon_tags_7 },
    { 0xdc61a53a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7f01a53c, SUC_lexicon_tags_15 },
    { 0x85e1a53d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfa81a53f, SUC_lexicon_tags_31 },
    { 0x8481a540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e41a542, SUC_lexicon_tags_35 },
    { 0x3fe1a543, SUC_lexicon_tags_23 },
    { 0xe421a543, SUC_lexicon_tags_2 },
    { 0x58c1a543, SUC_lexicon_tags_15 },
    { 0x60a1a546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbc1a548, SUC_lexicon_tags_37 },
    { 0x5a41a549, SUC_lexicon_tags_17 },
    { 0xf3a1a54a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6121a54e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1881a550, SUC_lexicon_tags_11 },
    { 0xbac1a550, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c1a558, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e1a55b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd01a55d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe1a561, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf41a564, SUC_lexicon_tags_19 },
    { 0xce21a564, SUC_lexicon_tags_17 },
    { 0x85c1a565, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd1a1a568, SUC_lexicon_tags_37 },
    { 0xa701a569, SUC_lexicon_tags_26 },
    { 0x8e1a56a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa101a56d, SUC_lexicon_tags_2 },
    { 0x2e61a56e, SUC_lexicon_tags_33 },
    { 0x1901a56f, SUC_lexicon_tags_15 },
    { 0x15a1a56e, SUC_lexicon_tags_65 },
    { 0xcee1a56e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc461a573, SUC_lexicon_tags_7 },
    { 0xd481a574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf301a581, SUC_lexicon_tags_12 },
    { 0x97a1a582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6321a589, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a01a58e, SUC_lexicon_tags_27 },
    { 0xf1a1a58f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32c1a591, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fc1a593, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c01a595, SUC_lexicon_tags_9 },
    { 0xd0a1a596, SUC_lexicon_tags_66 },
    { 0x5c61a597, SUC_lexicon_tags_15 },
    { 0xf8e1a597, SUC_lexicon_tags_37 },
    { 0x2241a599, SUC_lexicon_tags_37 },
    { 0x6b21a59a, SUC_lexicon_tags_17 },
    { 0x2b21a59b, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x3301a59d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec1a5a2, SUC_lexicon_tags_17 },
    { 0xd1c1a5a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb981a5a5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c1a5a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7301a5b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1a5b4, SUC_lexicon_tags_15 },
    { 0xe9e1a5b4, SUC_lexicon_tags_11 },
    { 0x9fa1a5b6, SUC_lexicon_tags_2 },
    { 0x6ba1a5b6, SUC_lexicon_tags_12 },
    { 0x1ee1a5b5, SUC_lexicon_tags_37 },
    { 0xb701a5b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c21a5bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f01a5bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xffc1a5c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae1a5c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a1a5c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3161a5c8, SUC_lexicon_tags_40 },
    { 0x75e1a5c9, SUC_lexicon_tags_49 },
    { 0x3721a5c9, SUC_lexicon_tags_32 },
    { 0xc5a1a5cb, SUC_lexicon_tags_87 },
    { 0xcba1a5c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a1a5cf, SUC_lexicon_tags_37 },
    { 0x80c1a5cf, SUC_lexicon_tags_11 },
    { 0x38c1a5cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3281a5d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1a5d8, SUC_lexicon_tags_68 },
    { 0xc641a5d9, SUC_lexicon_tags_11 },
    { 0xeec1a5da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9441a5dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5541a5e1, SUC_lexicon_tags_7 },
    { 0xafe1a5e2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a1a5e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae21a5eb, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a21a5ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb041a5f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe381a5f7, SUC_lexicon_tags_4 },
    { 0x20e1a5f8, SUC_lexicon_tags_259 },
    { 0x6481a5f7, SUC_lexicon_tags_0 },
    { 0x5a21a5fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e1a5fd, SUC_lexicon_tags_17 },
    { 0xe761a5fd, SUC_lexicon_tags_19 },
    { 0x8ac1a5ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5901a602, SUC_lexicon_tags_49 },
    { 0x60c1a602, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x56e1a605, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9621a608, SUC_lexicon_tags_29 },
    { 0xe5e1a609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a1a60e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a1a611, SUC_lexicon_tags_49 },
    { 0x1b61a611, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7361a617, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3221a619, SUC_lexicon_tags_3 },
    { 0x4901a61a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd81a61f, SUC_lexicon_tags_1 },
    { 0xee81a620, SUC_lexicon_tags_31 },
    { 0x5501a621, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce1a625, SUC_lexicon_tags_3 },
    { 0x5661a626, SUC_lexicon_tags_12 },
    { 0x2201a627, SUC_lexicon_tags_1 },
    { 0xdf61a628, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc1a62d, SUC_lexicon_tags_15 },
    { 0xcd61a62e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7001a632, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff81a637, SUC_lexicon_tags_15 },
    { 0x2521a638, SUC_lexicon_tags_30 },
    { 0x1c81a638, SUC_lexicon_tags_17 },
    { 0x78e1a63a, SUC_lexicon_tags_26 },
    { 0x12e1a63b, SUC_lexicon_tags_1 },
    { 0x3941a63b, SUC_lexicon_tags_25 },
    { 0x12e1a63b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1361a647, SUC_lexicon_tags_14 },
    { 0x11e1a648, SUC_lexicon_tags_2 },
    { 0xa021a649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5301a653, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e1a65a, SUC_lexicon_tags_11 },
    { 0x7da1a65b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a1a65e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfe1a660, SUC_lexicon_tags_14 },
    { 0xaa1a661, SUC_lexicon_tags_9 },
    { 0x2001a661, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981a666, SUC_lexicon_tags_55 },
    { 0x7de1a667, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a1a66a, SUC_lexicon_tags_37 },
    { 0x2d21a66b, SUC_lexicon_tags_17 },
    { 0x9e1a66c, SUC_lexicon_tags_37 },
    { 0xcb21a66d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d81a670, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4221a673, SUC_lexicon_tags_15 },
    { 0x8061a673, SUC_lexicon_tags_28 },
    { 0xb8a1a673, SUC_lexicon_tags_37 },
    { 0x5ba1a676, SUC_lexicon_tags_37 },
    { 0xcc1a677, SUC_lexicon_tags_13 },
    { 0x5a61a675, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c1a67c, SUC_lexicon_tags_241 },
    { 0xbde1a67d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce1a682, SUC_lexicon_tags_37 },
    { 0x8de1a683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab61a686, SUC_lexicon_tags_37 },
    { 0x201a686, SUC_lexicon_tags_23 },
    { 0x4821a688, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a68b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe1a68e, SUC_lexicon_tags_19 },
    { 0x5181a68f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa661a6a0, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe1a6a5, SUC_lexicon_tags_37 },
    { 0xaf61a6a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6621a6a9, SUC_lexicon_tags_37 },
    { 0xaba1a6aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9321a6b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be1a6b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8921a6bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c1a6c3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e21a6c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa1a6c9, SUC_lexicon_tags_1 },
    { 0xf6a1a6c9, SUC_lexicon_tags_11 },
    { 0x2b01a6cb, SUC_lexicon_tags_48 },
    { 0x49c1a6cc, SUC_lexicon_tags_17 },
    { 0x7921a6cd, SUC_lexicon_tags_2 },
    { 0x79c1a6ca, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb21a6d4, SUC_lexicon_tags_23 },
    { 0x9ca1a6d5, SUC_lexicon_tags_15 },
    { 0xc941a6d5, SUC_lexicon_tags_1 },
    { 0xc201a6d4, SUC_lexicon_tags_12 },
    { 0x881a6d7, SUC_lexicon_tags_15 },
    { 0x7861a6d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f41a6dd, SUC_lexicon_tags_9 },
    { 0xd9e1a6de, SUC_lexicon_tags_15 },
    { 0x49c1a6df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd001a6e2, SUC_lexicon_tags_47 },
    { 0x6161a6e3, SUC_lexicon_tags_11 },
    { 0x47c1a6e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61a6e9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8a1a6eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f81a6f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34e1a6f7, SUC_lexicon_tags_28 },
    { 0x8dc1a6f8, SUC_lexicon_tags_7 },
    { 0x6641a6f9, SUC_lexicon_tags_11 },
    { 0xebc1a6fa, SUC_lexicon_tags_47 },
    { 0xf41a6fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7aa1a6fd, SUC_lexicon_tags_25 },
    { 0x8841a6fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15e1a700, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce1a704, SUC_lexicon_tags_1 },
    { 0xb1e1a704, SUC_lexicon_tags_11 },
    { 0x3a41a706, SUC_lexicon_tags_47 },
    { 0x5c61a706, SUC_lexicon_tags_2 },
    { 0xffe1a704, SUC_lexicon_tags_12 },
    { 0xaa41a709, SUC_lexicon_tags_37 },
    { 0x7c61a705, SUC_lexicon_tags_15 },
    { 0x6ae1a70a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a710, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c1a717, SUC_lexicon_tags_26 },
    { 0xad01a718, SUC_lexicon_tags_15 },
    { 0xb8a1a717, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6d61a71b, SUC_lexicon_tags_1 },
    { 0xaf41a71b, SUC_lexicon_tags_27 },
    { 0x6341a71b, SUC_lexicon_tags_5 },
    { 0xbf21a71e, SUC_lexicon_tags_1 },
    { 0x2b81a71f, SUC_lexicon_tags_15 },
    { 0x26e1a71f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa21a724, SUC_lexicon_tags_8 },
    { 0x1121a724, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3981a728, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac41a72c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2041a730, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f41a732, SUC_lexicon_tags_11 },
    { 0xfca1a733, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb101a735, SUC_lexicon_tags_160 },
    { 0x3701a736, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f81a73b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde21a73d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7081a743, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3501a745, SUC_lexicon_tags_2 },
    { 0x44e1a746, SUC_lexicon_tags_6 },
    { 0xa661a747, SUC_lexicon_tags_33 },
    { 0x3101a747, SUC_lexicon_tags_85 },
    { 0x9b41a749, SUC_lexicon_tags_68 },
    { 0x7c1a74a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd281a74c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xac61a74f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb581a751, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7081a753, SUC_lexicon_tags_38 },
    { 0x17a1a753, SUC_lexicon_tags_27 },
    { 0x3ae1a755, SUC_lexicon_tags_33 },
    { 0x9441a754, SUC_lexicon_tags_1 },
    { 0x4a21a757, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde1a75b, SUC_lexicon_tags_36 },
    { 0x2ee1a75c, SUC_lexicon_tags_46 },
    { 0x8a21a75c, SUC_lexicon_tags_2 },
    { 0xeb21a75b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6161a762, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1101a76f, SUC_lexicon_tags_4 },
    { 0x6b21a770, SUC_lexicon_tags_26 },
    { 0x1ac1a770, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a1a775, SUC_lexicon_tags_11 },
    { 0x1521a775, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9241a779, SUC_lexicon_tags_33 },
    { 0x1e1a77a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae1a780, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb801a784, SUC_lexicon_tags_8 },
    { 0xa701a785, SUC_lexicon_tags_11 },
    { 0x74a1a786, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1a78a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a1a793, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa1a799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18c1a79b, SUC_lexicon_tags_17 },
    { 0xb2a1a79b, SUC_lexicon_tags_12 },
    { 0xc321a79d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc81a7a0, SUC_lexicon_tags_69 },
    { 0x8de1a7a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa441a7a4, SUC_lexicon_tags_47 },
    { 0x1641a7a5, SUC_lexicon_tags_15 },
    { 0x7381a7a6, SUC_lexicon_tags_5 },
    { 0x3aa1a7a7, SUC_lexicon_tags_37 },
    { 0x3c21a7a8, SUC_lexicon_tags_37 },
    { 0xff61a7a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c21a7ae, SUC_lexicon_tags_12 },
    { 0x601a7af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a1a7b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6061a7b6, SUC_lexicon_tags_25 },
    { 0x1521a7b7, SUC_lexicon_tags_12 },
    { 0x861a7b8, SUC_lexicon_tags_8 },
    { 0x9ae1a7b8, SUC_lexicon_tags_7 },
    { 0x8461a7b9, SUC_lexicon_tags_0 },
    { 0x7f81a7b8, SUC_lexicon_tags_0 },
    { 0x3101a7bb, SUC_lexicon_tags_2 },
    { 0xa61a7bd, SUC_lexicon_tags_1443 },
    { 0x6881a7be, SUC_lexicon_tags_8 },
    { 0xa9c1a7be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c01a7cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6401a7ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7581a7d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9461a7da, SUC_lexicon_tags_7 },
    { 0x7461a7db, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa9c1a7dd, SUC_lexicon_tags_25 },
    { 0xac41a7de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc21a7e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf901a7e4, SUC_lexicon_tags_68 },
    { 0xfd01a7e4, SUC_lexicon_tags_6 },
    { 0x36e1a7e4, SUC_lexicon_tags_15 },
    { 0x70c1a7e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa701a7ec, SUC_lexicon_tags_31 },
    { 0x3c01a7ed, SUC_lexicon_tags_11 },
    { 0xe8a1a7ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de1a7f3, SUC_lexicon_tags_61 },
    { 0x7481a7f4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621a7f8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x2fe1a7fa, SUC_lexicon_tags_29 },
    { 0xd41a7fb, SUC_lexicon_tags_6 },
    { 0x2d41a7fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6c1a7fe, SUC_lexicon_tags_25 },
    { 0x85e1a7fe, SUC_lexicon_tags_63 },
    { 0x8dc1a800, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1a808, SUC_lexicon_tags_0 },
    { 0xd641a809, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc641a812, SUC_lexicon_tags_6 },
    { 0xa141a812, SUC_lexicon_tags_28 },
    { 0x5da1a812, SUC_lexicon_tags_28 },
    { 0x9061a815, SUC_lexicon_tags_15 },
    { 0x8621a816, SUC_lexicon_tags_27 },
    { 0x1441a816, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e1a81c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1841a81e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c1a821, SUC_lexicon_tags_17 },
    { 0x36a1a822, SUC_lexicon_tags_12 },
    { 0x8e01a822, SUC_lexicon_tags_7 },
    { 0x3361a823, SUC_lexicon_tags_27 },
    { 0xf101a825, SUC_lexicon_tags_19 },
    { 0xefc1a823, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe401a829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefc1a82b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb841a82d, SUC_lexicon_tags_37 },
    { 0x1ba1a82e, SUC_lexicon_tags_30 },
    { 0xdc21a82f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xce1a831, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14c1a833, SUC_lexicon_tags_15 },
    { 0xcea1a834, SUC_lexicon_tags_47 },
    { 0xcfc1a835, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e61a838, SUC_lexicon_tags_23 },
    { 0x67e1a838, SUC_lexicon_tags_2 },
    { 0x7ce1a83a, SUC_lexicon_tags_41 },
    { 0x9621a83b, SUC_lexicon_tags_29 },
    { 0x3cc1a83c, SUC_lexicon_tags_0 },
    { 0x9981a838, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad61a842, SUC_lexicon_tags_79 },
    { 0x1da1a843, SUC_lexicon_tags_87 },
    { 0x5b61a843, SUC_lexicon_tags_25 },
    { 0x7081a845, SUC_lexicon_tags_1 },
    { 0x2e01a842, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3841a848, SUC_lexicon_tags_25 },
    { 0x3481a848, SUC_lexicon_tags_77 },
    { 0xd621a84a, SUC_lexicon_tags_37 },
    { 0xfaa1a84b, SUC_lexicon_tags_45 },
    { 0xe321a84b, SUC_lexicon_tags_65 },
    { 0x3d81a84b, SUC_lexicon_tags_1 },
    { 0x45c1a84e, SUC_lexicon_tags_25 },
    { 0xc1c1a84a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe301a851, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3121a856, SUC_lexicon_tags_2 },
    { 0x45a1a857, SUC_lexicon_tags_1 },
    { 0x3a41a857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da1a85b, SUC_lexicon_tags_17 },
    { 0x47e1a85b, SUC_lexicon_tags_9 },
    { 0xe0c1a85d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83e1a85f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2a1a861, SUC_lexicon_tags_23 },
    { 0xf9a1a862, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x5221a864, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b81a868, SUC_lexicon_tags_1 },
    { 0x4441a868, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2981a86c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9081a86e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1321a871, SUC_lexicon_tags_26 },
    { 0xc361a872, SUC_lexicon_tags_25 },
    { 0x5c1a872, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa61a87d, SUC_lexicon_tags_17 },
    { 0x5d61a87d, SUC_lexicon_tags_26 },
    { 0x7921a87f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5921a888, SUC_lexicon_tags_15 },
    { 0xc1a889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf61a88c, SUC_lexicon_tags_1 },
    { 0xd561a88c, SUC_lexicon_tags_48 },
    { 0x9bc1a88c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7101a893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8621a895, SUC_lexicon_tags_7 },
    { 0x30c1a895, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x28c1a898, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbbe1a89a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd461a89e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e1a8a3, SUC_lexicon_tags_16 },
    { 0x8441a8a4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7261a8a6, SUC_lexicon_tags_12 },
    { 0x3321a8a7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d21a8ab, SUC_lexicon_tags_4 },
    { 0xba21a8ac, SUC_lexicon_tags_17 },
    { 0xef01a8ad, SUC_lexicon_tags_47 },
    { 0x95c1a8ac, SUC_lexicon_tags_2 },
    { 0xf8c1a8af, SUC_lexicon_tags_1 },
    { 0x5da1a8ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c1a8b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9701a8b9, SUC_lexicon_tags_11 },
    { 0x69e1a8b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc1a8bd, SUC_lexicon_tags_9 },
    { 0x6ca1a8be, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xc221a8c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c1a8c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32e1a8cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c1a8ce, SUC_lexicon_tags_47 },
    { 0xb081a8cf, SUC_lexicon_tags_57 },
    { 0xada1a8d0, SUC_lexicon_tags_33 },
    { 0x22c1a8cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5421a8d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1d41a8d5, SUC_lexicon_tags_1 },
    { 0x1c01a8d6, SUC_lexicon_tags_1 },
    { 0xe441a8d7, SUC_lexicon_tags_2 },
    { 0x3bc1a8d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c1a8dd, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff61a8e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab41a8e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e1a8f2, SUC_lexicon_tags_14 },
    { 0x9741a8f2, SUC_lexicon_tags_7 },
    { 0xf2e1a8f3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe661a8fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc1a904, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e01a907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8841a90a, SUC_lexicon_tags_10 },
    { 0xfbe1a90a, SUC_lexicon_tags_12 },
    { 0x9361a90a, SUC_lexicon_tags_55 },
    { 0x5121a90d, SUC_lexicon_tags_8 },
    { 0x1881a90d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a41a910, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f61a913, SUC_lexicon_tags_87 },
    { 0xc621a914, SUC_lexicon_tags_43 },
    { 0x97a1a914, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6101a917, SUC_lexicon_tags_14 },
    { 0x5da1a917, SUC_lexicon_tags_26 },
    { 0x2bc1a918, SUC_lexicon_tags_11 },
    { 0x68e1a91a, SUC_lexicon_tags_25 },
    { 0x18c1a91a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x581a91d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c61a91f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c1a927, SUC_lexicon_tags_11 },
    { 0x3a1a928, SUC_lexicon_tags_17 },
    { 0x31e1a929, SUC_lexicon_tags_1 },
    { 0xc041a928, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e1a92d, SUC_lexicon_tags_37 },
    { 0x7b01a92e, SUC_lexicon_tags_11 },
    { 0x7581a92f, SUC_lexicon_tags_37 },
    { 0x7fa1a930, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc81a932, SUC_lexicon_tags_62 },
    { 0x4d41a933, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1c21a935, SUC_lexicon_tags_15 },
    { 0x7341a936, SUC_lexicon_tags_1 },
    { 0x4b41a935, SUC_lexicon_tags_1 },
    { 0x2501a938, SUC_lexicon_tags_25 },
    { 0x4c1a939, SUC_lexicon_tags_6 },
    { 0x1f21a936, SUC_lexicon_tags_29 },
    { 0xd9e1a938, SUC_lexicon_tags_12 },
    { 0x84e1a93c, SUC_lexicon_tags_11 },
    { 0xa8e1a93d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ce1a93f, SUC_lexicon_tags_37 },
    { 0x6a1a940, SUC_lexicon_tags_28 },
    { 0xfcc1a941, SUC_lexicon_tags_17 },
    { 0x4101a940, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1881a94b, SUC_lexicon_tags_14 },
    { 0xcc61a94b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a1a950, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea1a953, SUC_lexicon_tags_0 },
    { 0xd281a954, SUC_lexicon_tags_16 },
    { 0xae01a955, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c61a959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e1a962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4301a965, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4681a967, SUC_lexicon_tags_49 },
    { 0x1b61a968, SUC_lexicon_tags_19 },
    { 0x8e81a967, SUC_lexicon_tags_23 },
    { 0x3ac1a96a, SUC_lexicon_tags_19 },
    { 0x9321a96a, SUC_lexicon_tags_55 },
    { 0x1de1a96b, SUC_lexicon_tags_60 },
    { 0xe81a96b, SUC_lexicon_tags_28 },
    { 0xc981a96e, SUC_lexicon_tags_75 },
    { 0x39c1a96f, SUC_lexicon_tags_43 },
    { 0x6f21a96f, SUC_lexicon_tags_63 },
    { 0x6241a971, SUC_lexicon_tags_2 },
    { 0xf421a96e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d61a974, SUC_lexicon_tags_12 },
    { 0x43e1a974, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a1a97f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb341a982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa1a986, SUC_lexicon_tags_29 },
    { 0x9781a986, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb81a98a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c1a98e, SUC_lexicon_tags_2 },
    { 0x3a01a98f, SUC_lexicon_tags_5 },
    { 0xe421a990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19e1a992, SUC_lexicon_tags_47 },
    { 0xab01a992, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec1a997, SUC_lexicon_tags_46 },
    { 0xacc1a998, SUC_lexicon_tags_12 },
    { 0xe741a999, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf81a99e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a41a9a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de1a9a5, SUC_lexicon_tags_25 },
    { 0x4081a9a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3521a9aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb541a9ad, SUC_lexicon_tags_15 },
    { 0x4f61a9ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee1a9b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb221a9b4, SUC_lexicon_tags_12 },
    { 0x5021a9b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4081a9b7, SUC_lexicon_tags_11 },
    { 0xb721a9b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86a1a9ba, SUC_lexicon_tags_23 },
    { 0xc4e1a9ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb281a9bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e21a9c4, SUC_lexicon_tags_38 },
    { 0x9441a9c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1241a9c9, SUC_lexicon_tags_5 },
    { 0x5141a9ca, SUC_lexicon_tags_40 },
    { 0x7e01a9ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e01a9cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7741a9d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce1a9d6, SUC_lexicon_tags_65 },
    { 0x1381a9d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c61a9da, SUC_lexicon_tags_7 },
    { 0x1081a9db, SUC_lexicon_tags_30 },
    { 0xc0c1a9da, SUC_lexicon_tags_37 },
    { 0x60a1a9db, SUC_lexicon_tags_48 },
    { 0x5761a9de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca1a9e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc1a9e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9581a9e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa861a9ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e1a9ed, SUC_lexicon_tags_33 },
    { 0x98e1a9ee, SUC_lexicon_tags_48 },
    { 0x8001a9ef, SUC_lexicon_tags_19 },
    { 0x9f41a9f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a01a9f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd941a9f7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e41a9fa, SUC_lexicon_tags_7 },
    { 0x2381a9fb, SUC_lexicon_tags_27 },
    { 0xb2e1a9fb, SUC_lexicon_tags_25 },
    { 0x6841a9fc, SUC_lexicon_tags_12 },
    { 0xf721a9fe, SUC_lexicon_tags_25 },
    { 0x3b41a9fe, SUC_lexicon_tags_65 },
    { 0x5941a9ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1241aa04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3da1aa06, SUC_lexicon_tags_16 },
    { 0x45e1aa07, SUC_lexicon_tags_16 },
    { 0x3101aa08, SUC_lexicon_tags_2 },
    { 0xcdc1aa08, SUC_lexicon_tags_10 },
    { 0xdf21aa0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc1aa0e, SUC_lexicon_tags_17 },
    { 0x8881aa0f, SUC_lexicon_tags_17 },
    { 0x2d41aa10, SUC_lexicon_tags_7 },
    { 0x9681aa10, SUC_lexicon_tags_37 },
    { 0xf321aa0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5701aa15, SUC_lexicon_tags_103 },
    { 0xf481aa15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1aa1a, SUC_lexicon_tags_26 },
    { 0x37a1aa1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf221aa1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa01aa1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x701aa21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa001aa25, SUC_lexicon_tags_113 },
    { 0x3a1aa26, SUC_lexicon_tags_17 },
    { 0xda61aa27, SUC_lexicon_tags_37 },
    { 0xbea1aa27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4741aa2b, SUC_lexicon_tags_37 },
    { 0x7f41aa2c, SUC_lexicon_tags_3 },
    { 0x6001aa2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf0a1aa2f, SUC_lexicon_tags_39 },
    { 0x99a1aa30, SUC_lexicon_tags_84 },
    { 0x62a1aa30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1aa37, SUC_lexicon_tags_27 },
    { 0x941aa37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2341aa3c, SUC_lexicon_tags_7 },
    { 0x24e1aa3c, SUC_lexicon_tags_11 },
    { 0x2a21aa3e, SUC_lexicon_tags_25 },
    { 0xce21aa3e, SUC_lexicon_tags_7 },
    { 0xa421aa3c, SUC_lexicon_tags_68 },
    { 0x47c1aa3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xada1aa43, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de1aa46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee1aa4a, SUC_lexicon_tags_37 },
    { 0xe4e1aa4b, SUC_lexicon_tags_17 },
    { 0x6841aa4b, SUC_lexicon_tags_60 },
    { 0x4821aa4b, SUC_lexicon_tags_14 },
    { 0x7821aa4e, SUC_lexicon_tags_17 },
    { 0xd361aa4e, SUC_lexicon_tags_17 },
    { 0x4981aa4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7001aa53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9501aa55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa181aa5a, SUC_lexicon_tags_23 },
    { 0x6ac1aa5a, SUC_lexicon_tags_1 },
    { 0x2221aa5b, SUC_lexicon_tags_8 },
    { 0x201aa5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a61aa5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c1aa62, SUC_lexicon_tags_63 },
    { 0x1681aa63, SUC_lexicon_tags_33 },
    { 0xb841aa63, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8f61aa66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e1aa6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16c1aa6c, SUC_lexicon_tags_49 },
    { 0xc0a1aa6d, SUC_lexicon_tags_48 },
    { 0x1a81aa6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e1aa71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b21aa77, SUC_lexicon_tags_17 },
    { 0xc461aa77, SUC_lexicon_tags_37 },
    { 0x2761aa78, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63a1aa7b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd1c1aa7d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcf21aa7f, SUC_lexicon_tags_3 },
    { 0x10a1aa80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48e1aa82, SUC_lexicon_tags_37 },
    { 0x8981aa83, SUC_lexicon_tags_30 },
    { 0xc4e1aa84, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa581aa86, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7001aa88, SUC_lexicon_tags_60 },
    { 0x4b21aa89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf21aa8e, SUC_lexicon_tags_12 },
    { 0x2ca1aa8f, SUC_lexicon_tags_17 },
    { 0xff21aa8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba01aa92, SUC_lexicon_tags_136 },
    { 0x2381aa93, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb481aa95, SUC_lexicon_tags_8 },
    { 0x17e1aa96, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46c1aa98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4281aa9a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x4081aa9d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4141aaa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe621aaa3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc921aaa6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a1aaa9, SUC_lexicon_tags_1 },
    { 0xe61aaaa, SUC_lexicon_tags_56 },
    { 0x9fc1aaab, SUC_lexicon_tags_37 },
    { 0x8841aaac, SUC_lexicon_tags_27 },
    { 0x4c81aaad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd81aab3, SUC_lexicon_tags_65 },
    { 0x35c1aab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49c1aab6, SUC_lexicon_tags_37 },
    { 0xe7a1aab7, SUC_lexicon_tags_0 },
    { 0x4f21aab8, SUC_lexicon_tags_47 },
    { 0xfbc1aab9, SUC_lexicon_tags_2 },
    { 0x7b41aab9, SUC_lexicon_tags_6 },
    { 0x31a1aabb, SUC_lexicon_tags_15 },
    { 0x4a41aabb, SUC_lexicon_tags_17 },
    { 0xf7e1aabd, SUC_lexicon_tags_11 },
    { 0x8e61aabe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2e1aac0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e1aac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25a1aac9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c1aacd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa201aacf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c61aad2, SUC_lexicon_tags_11 },
    { 0x6701aad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ce1aad5, SUC_lexicon_tags_17 },
    { 0x62e1aad6, SUC_lexicon_tags_25 },
    { 0xa341aad5, SUC_lexicon_tags_63 },
    { 0xfb61aad8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8561aadc, SUC_lexicon_tags_27 },
    { 0x13c1aadd, SUC_lexicon_tags_17 },
    { 0x1ba1aadd, SUC_lexicon_tags_11 },
    { 0x8721aadf, SUC_lexicon_tags_1 },
    { 0x3281aae0, SUC_lexicon_tags_46 },
    { 0x8f41aae0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae1aae4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae1aaea, SUC_lexicon_tags_23 },
    { 0xc41aaeb, SUC_lexicon_tags_25 },
    { 0xb7c1aaeb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4ce1aaee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5741aaf0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8441aaf2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b21aaff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca1ab09, SUC_lexicon_tags_2 },
    { 0x4001ab09, SUC_lexicon_tags_12 },
    { 0x7561ab0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda61ab10, SUC_lexicon_tags_25 },
    { 0xd1c1ab11, SUC_lexicon_tags_68 },
    { 0xb421ab11, SUC_lexicon_tags_37 },
    { 0x7c01ab13, SUC_lexicon_tags_37 },
    { 0x2f21ab14, SUC_lexicon_tags_2 },
    { 0x90e1ab15, SUC_lexicon_tags_159 },
    { 0xb7c1ab14, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x56e1ab18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6161ab1a, SUC_lexicon_tags_1 },
    { 0xab41ab1b, SUC_lexicon_tags_48 },
    { 0x4601ab1c, SUC_lexicon_tags_37 },
    { 0x9c61ab1a, SUC_lexicon_tags_23 },
    { 0x3141ab1e, SUC_lexicon_tags_27 },
    { 0x6981ab1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6421ab21, SUC_lexicon_tags_2 },
    { 0x5281ab22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf61ab26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3061ab2a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd21ab31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d81ab35, SUC_lexicon_tags_1 },
    { 0x4fc1ab36, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc361ab39, SUC_lexicon_tags_25 },
    { 0x2321ab3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22a1ab3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa1ab3f, SUC_lexicon_tags_11 },
    { 0x2841ab40, SUC_lexicon_tags_27 },
    { 0xfb61ab41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfae1ab43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9801ab45, SUC_lexicon_tags_8 },
    { 0xcc61ab45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3561ab4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1e1ab4d, SUC_lexicon_tags_45 },
    { 0xfa61ab4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe941ab53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6501ab56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9201ab59, SUC_lexicon_tags_37 },
    { 0xdb01ab59, SUC_lexicon_tags_25 },
    { 0xcb41ab59, SUC_lexicon_tags_70 },
    { 0x9041ab5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf0e1ab5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1381ab62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1081ab64, SUC_lexicon_tags_38 },
    { 0x5ba1ab65, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa841ab6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6281ab6e, SUC_lexicon_tags_107 },
    { 0x5281ab6f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe0c1ab71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5281ab73, SUC_lexicon_tags_1 },
    { 0xc7a1ab74, SUC_lexicon_tags_84 },
    { 0x7221ab74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b01ab7a, SUC_lexicon_tags_12 },
    { 0x2d81ab7b, SUC_lexicon_tags_15 },
    { 0x38a1ab7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb821ab81, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff41ab85, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f21ab8b, SUC_lexicon_tags_23 },
    { 0xfb81ab8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd021ab8f, SUC_lexicon_tags_66 },
    { 0x4f21ab8f, SUC_lexicon_tags_43 },
    { 0x41e1ab91, SUC_lexicon_tags_17 },
    { 0x3321ab90, SUC_lexicon_tags_87 },
    { 0xdcc1ab93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68e1ab95, SUC_lexicon_tags_25 },
    { 0x1341ab96, SUC_lexicon_tags_27 },
    { 0x60c1ab96, SUC_lexicon_tags_7 },
    { 0xa901ab97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab81ab9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10e1ab9e, SUC_lexicon_tags_21 },
    { 0xec61ab9e, SUC_lexicon_tags_45 },
    { 0xce41aba0, SUC_lexicon_tags_6 },
    { 0xd141aba0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x88c1aba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821aba8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1abad, SUC_lexicon_tags_3 },
    { 0x241abae, SUC_lexicon_tags_4 },
    { 0xa301abae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb381abb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e1abb7, SUC_lexicon_tags_6 },
    { 0xae1abb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe581abbb, SUC_lexicon_tags_19 },
    { 0x30a1abbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b81abc1, SUC_lexicon_tags_12 },
    { 0xe5c1abc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc81abc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb61abce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf881abd1, SUC_lexicon_tags_0 },
    { 0xbf61abd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa41abd5, SUC_lexicon_tags_30 },
    { 0x3641abd6, SUC_lexicon_tags_33 },
    { 0x97a1abd5, SUC_lexicon_tags_15 },
    { 0x5641abd8, SUC_lexicon_tags_10 },
    { 0x9901abd8, SUC_lexicon_tags_17 },
    { 0xdec1abda, SUC_lexicon_tags_27 },
    { 0x1221abd9, SUC_lexicon_tags_583 },
    { 0x9801abdc, SUC_lexicon_tags_68 },
    { 0xf9a1abd7, SUC_lexicon_tags_63 },
    { 0x6781abd7, SUC_lexicon_tags_16 },
    { 0x91c1abdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x561abe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe841abe3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9821abe7, SUC_lexicon_tags_160 },
    { 0xc081abe8, SUC_lexicon_tags_2 },
    { 0x861abe9, SUC_lexicon_tags_25 },
    { 0x4541abe9, SUC_lexicon_tags_33 },
    { 0xe7e1abe7, SUC_lexicon_tags_57 },
    { 0x39a1abec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e81abf5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc1abf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d41abfc, SUC_lexicon_tags_28 },
    { 0x7ca1abfc, SUC_lexicon_tags_17 },
    { 0xb321abfd, SUC_lexicon_tags_12 },
    { 0xa661abfe, SUC_lexicon_tags_29 },
    { 0x7701ac00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x601ac03, SUC_lexicon_tags_15 },
    { 0xdec1ac04, SUC_lexicon_tags_74 },
    { 0x5aa1ac05, SUC_lexicon_tags_0 },
    { 0x3c61ac06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7781ac08, SUC_lexicon_tags_9 },
    { 0x3de1ac08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8641ac0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5841ac0d, SUC_lexicon_tags_77 },
    { 0x3f01ac0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30a1ac10, SUC_lexicon_tags_0 },
    { 0xeac1ac11, SUC_lexicon_tags_0 },
    { 0x10e1ac11, SUC_lexicon_tags_30 },
    { 0xcde1ac13, SUC_lexicon_tags_14 },
    { 0xc4c1ac14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc921ac17, SUC_lexicon_tags_37 },
    { 0x61ac18, SUC_lexicon_tags_1193 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d01ac1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c1ac1f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e1ac22, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe321ac26, SUC_lexicon_tags_42 },
    { 0xca41ac27, SUC_lexicon_tags_28 },
    { 0x4c21ac28, SUC_lexicon_tags_40 },
    { 0x2801ac29, SUC_lexicon_tags_5 },
    { 0x6fe1ac27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8101ac2e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc861ac31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91e1ac33, SUC_lexicon_tags_61 },
    { 0x4b21ac34, SUC_lexicon_tags_25 },
    { 0x4a61ac35, SUC_lexicon_tags_1 },
    { 0x59a1ac36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c1ac3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6841ac3f, SUC_lexicon_tags_27 },
    { 0xea41ac3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5921ac43, SUC_lexicon_tags_11 },
    { 0x8881ac44, SUC_lexicon_tags_15 },
    { 0x5ea1ac45, SUC_lexicon_tags_25 },
    { 0xe9e1ac46, SUC_lexicon_tags_37 },
    { 0x42e1ac47, SUC_lexicon_tags_5 },
    { 0x26e1ac48, SUC_lexicon_tags_12 },
    { 0xbea1ac44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd961ac4d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f81ac52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a41ac55, SUC_lexicon_tags_47 },
    { 0xe981ac56, SUC_lexicon_tags_0 },
    { 0xed01ac57, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a61ac5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2081ac60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81ac65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xacc1ac67, SUC_lexicon_tags_15 },
    { 0x83a1ac67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac1ac6d, SUC_lexicon_tags_21 },
    { 0x4901ac6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c81ac75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921ac7b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c1ac7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c1ac82, SUC_lexicon_tags_12 },
    { 0x4dc1ac83, SUC_lexicon_tags_79 },
    { 0x63c1ac83, SUC_lexicon_tags_37 },
    { 0xd5a1ac84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec1ac89, SUC_lexicon_tags_17 },
    { 0x34e1ac8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x13a1ac8c, SUC_lexicon_tags_83 },
    { 0x9961ac8c, SUC_lexicon_tags_37 },
    { 0xac41ac8d, SUC_lexicon_tags_1 },
    { 0x98c1ac8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb901ac91, SUC_lexicon_tags_47 },
    { 0x3c1ac92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a1ac9b, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xa5e1ac9d, SUC_lexicon_tags_31 },
    { 0x4361ac9e, SUC_lexicon_tags_0 },
    { 0x9261ac9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e1aca3, SUC_lexicon_tags_4 },
    { 0x1241aca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa241acac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc1acaf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa861acb9, SUC_lexicon_tags_28 },
    { 0x7da1acba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f01acbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5bc1acbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a41acc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a1acc9, SUC_lexicon_tags_25 },
    { 0xade1acca, SUC_lexicon_tags_47 },
    { 0x39e1accb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7821acd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c1acd7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e1acdc, SUC_lexicon_tags_17 },
    { 0x8ce1acdc, SUC_lexicon_tags_15 },
    { 0x7a1acdc, SUC_lexicon_tags_15 },
    { 0xdfe1acdf, SUC_lexicon_tags_2 },
    { 0x6061ace0, SUC_lexicon_tags_9 },
    { 0x4c61acdd, SUC_lexicon_tags_25 },
    { 0x521ace2, SUC_lexicon_tags_8 },
    { 0xbfa1ace3, SUC_lexicon_tags_7 },
    { 0x2621ace4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2761acea, SUC_lexicon_tags_37 },
    { 0xf8a1acea, SUC_lexicon_tags_25 },
    { 0xcde1acea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3081acee, SUC_lexicon_tags_15 },
    { 0xc1acee, SUC_lexicon_tags_28 },
    { 0x6581acf0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5461acf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fc1acf6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x21e1acf8, SUC_lexicon_tags_0 },
    { 0x5c61acf8, SUC_lexicon_tags_46 },
    { 0x5001acfa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5381acfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ad00, SUC_lexicon_tags_15 },
    { 0xe901ad01, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e1ad05, SUC_lexicon_tags_25 },
    { 0x29e1ad06, SUC_lexicon_tags_1 },
    { 0xe3c1ad07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe921ad09, SUC_lexicon_tags_37 },
    { 0x4041ad09, SUC_lexicon_tags_30 },
    { 0x35a1ad0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a1ad13, SUC_lexicon_tags_56 },
    { 0x5c41ad14, SUC_lexicon_tags_63 },
    { 0x2f81ad15, SUC_lexicon_tags_28 },
    { 0xa0a1ad15, SUC_lexicon_tags_27 },
    { 0x3661ad17, SUC_lexicon_tags_37 },
    { 0x7c81ad18, SUC_lexicon_tags_1 },
    { 0x54a1ad19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2981ad1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce1ad21, SUC_lexicon_tags_31 },
    { 0x8281ad21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x861ad24, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8941ad2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cc1ad2f, SUC_lexicon_tags_2 },
    { 0x6fa1ad30, SUC_lexicon_tags_26 },
    { 0x1e81ad30, SUC_lexicon_tags_17 },
    { 0xfa61ad30, SUC_lexicon_tags_22 },
    { 0x18c1ad30, SUC_lexicon_tags_9 },
    { 0x23e1ad34, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c21ad37, SUC_lexicon_tags_17 },
    { 0x9a1ad38, SUC_lexicon_tags_7 },
    { 0x7e1ad39, SUC_lexicon_tags_33 },
    { 0x9ea1ad37, SUC_lexicon_tags_44 },
    { 0xe681ad37, SUC_lexicon_tags_66 },
    { 0xf081ad3c, SUC_lexicon_tags_17 },
    { 0x3421ad3d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5481ad3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a01ad41, SUC_lexicon_tags_65 },
    { 0x5a41ad42, SUC_lexicon_tags_49 },
    { 0x2f21ad42, SUC_lexicon_tags_1 },
    { 0xdb61ad41, SUC_lexicon_tags_2 },
    { 0x3501ad45, SUC_lexicon_tags_0 },
    { 0xa0c1ad41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2801ad49, SUC_lexicon_tags_17 },
    { 0xe4e1ad4a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe21ad4e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e81ad51, SUC_lexicon_tags_37 },
    { 0x4f41ad51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e1ad5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde1ad5d, SUC_lexicon_tags_25 },
    { 0x67c1ad5d, SUC_lexicon_tags_1 },
    { 0x9b81ad5f, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a61ad67, SUC_lexicon_tags_37 },
    { 0x2601ad68, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a1ad6c, SUC_lexicon_tags_63 },
    { 0x6601ad6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x261ad70, SUC_lexicon_tags_25 },
    { 0x2001ad71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9321ad76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92e1ad78, SUC_lexicon_tags_32 },
    { 0x6c41ad79, SUC_lexicon_tags_37 },
    { 0xdd01ad7a, SUC_lexicon_tags_7 },
    { 0xaee1ad79, SUC_lexicon_tags_15 },
    { 0x63e1ad7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e1ad7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e1ad84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f01ad87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf601ad8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa321ad92, SUC_lexicon_tags_17 },
    { 0x2ac1ad93, SUC_lexicon_tags_28 },
    { 0x5b81ad93, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe1ad97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4e1ad99, SUC_lexicon_tags_17 },
    { 0xb101ad9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa21ad9c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4821ada1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01ada6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ce1ada8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c1adaf, SUC_lexicon_tags_7 },
    { 0xb161adb0, SUC_lexicon_tags_0 },
    { 0xf3e1adb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc981adb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1dc1adb5, SUC_lexicon_tags_8 },
    { 0xd3e1adb6, SUC_lexicon_tags_15 },
    { 0xb1e1adb7, SUC_lexicon_tags_38 },
    { 0xc501adb8, SUC_lexicon_tags_28 },
    { 0x12e1adb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1101adbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5821adc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c1adc5, SUC_lexicon_tags_26 },
    { 0x11a1adc5, SUC_lexicon_tags_27 },
    { 0x48c1adc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a1adcc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfb01adce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3281add1, SUC_lexicon_tags_25 },
    { 0x1881add1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3261add4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb881addb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d01adde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be1ade4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a1ade7, SUC_lexicon_tags_37 },
    { 0xd61ade8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8901adea, SUC_lexicon_tags_1 },
    { 0x3781adeb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfc1aded, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5a81adef, SUC_lexicon_tags_31 },
    { 0xa7e1adef, SUC_lexicon_tags_25 },
    { 0x4aa1adf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2a1adf3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc521adf6, SUC_lexicon_tags_30 },
    { 0x541adf7, SUC_lexicon_tags_27 },
    { 0x9c61adf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac1adfc, SUC_lexicon_tags_1 },
    { 0x6621adfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9681ae00, SUC_lexicon_tags_27 },
    { 0x17e1ae01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc021ae03, SUC_lexicon_tags_12 },
    { 0xfda1ae03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba1ae08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf61ae0b, SUC_lexicon_tags_15 },
    { 0xbda1ae0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x57a1ae0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5021ae10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7901ae15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8841ae18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc681ae1b, SUC_lexicon_tags_23 },
    { 0xe681ae1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5341ae23, SUC_lexicon_tags_33 },
    { 0x9381ae24, SUC_lexicon_tags_15 },
    { 0xa801ae25, SUC_lexicon_tags_7 },
    { 0x5981ae26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ee1ae28, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd361ae2c, SUC_lexicon_tags_87 },
    { 0x9781ae2c, SUC_lexicon_tags_37 },
    { 0x3901ae2e, SUC_lexicon_tags_11 },
    { 0x88c1ae2c, SUC_lexicon_tags_12 },
    { 0xcfa1ae30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8901ae33, SUC_lexicon_tags_60 },
    { 0x2fa1ae33, SUC_lexicon_tags_11 },
    { 0xc081ae35, SUC_lexicon_tags_14 },
    { 0xa8e1ae36, SUC_lexicon_tags_9 },
    { 0xf3a1ae35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xea01ae3a, SUC_lexicon_tags_77 },
    { 0xb461ae3b, SUC_lexicon_tags_25 },
    { 0x7ac1ae3c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe541ae3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdec1ae40, SUC_lexicon_tags_38 },
    { 0x28a1ae41, SUC_lexicon_tags_8 },
    { 0xa041ae40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a1ae46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9641ae48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7601ae4b, SUC_lexicon_tags_60 },
    { 0xa1a1ae4c, SUC_lexicon_tags_7 },
    { 0x4b81ae4d, SUC_lexicon_tags_136 },
    { 0x9661ae4b, SUC_lexicon_tags_25 },
    { 0xed81ae4b, SUC_lexicon_tags_17 },
    { 0x8761ae50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb641ae52, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa1ae57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2381ae5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6561ae5c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a1ae61, SUC_lexicon_tags_63 },
    { 0x3b61ae62, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9ba1ae64, SUC_lexicon_tags_15 },
    { 0xe181ae65, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd781ae67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721ae6c, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0xd001ae6e, SUC_lexicon_tags_61 },
    { 0x8241ae6f, SUC_lexicon_tags_33 },
    { 0xd281ae70, SUC_lexicon_tags_11 },
    { 0x9be1ae70, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcd81ae73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1961ae77, SUC_lexicon_tags_70 },
    { 0xaa41ae77, SUC_lexicon_tags_63 },
    { 0xcb01ae78, SUC_lexicon_tags_17 },
    { 0x8641ae77, SUC_lexicon_tags_15 },
    { 0xe761ae79, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8061ae83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a1ae89, SUC_lexicon_tags_7 },
    { 0xdc41ae8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a1ae91, SUC_lexicon_tags_0 },
    { 0x4fc1ae91, SUC_lexicon_tags_7 },
    { 0xc021ae92, SUC_lexicon_tags_15 },
    { 0xba1ae92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb201ae9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a1aea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8e1aea2, SUC_lexicon_tags_47 },
    { 0xb881aea3, SUC_lexicon_tags_45 },
    { 0x3b81aea4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b81aea8, SUC_lexicon_tags_38 },
    { 0x381aea8, SUC_lexicon_tags_2 },
    { 0x4601aeaa, SUC_lexicon_tags_11 },
    { 0x3c21aea9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5201aeb1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9101aeb3, SUC_lexicon_tags_27 },
    { 0xe1a1aeb4, SUC_lexicon_tags_11 },
    { 0xd501aeb5, SUC_lexicon_tags_3 },
    { 0xa2c1aeb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c1aeba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3d61aebc, SUC_lexicon_tags_49 },
    { 0x9cc1aebd, SUC_lexicon_tags_17 },
    { 0x6081aebe, SUC_lexicon_tags_66 },
    { 0x581aebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1441aec4, SUC_lexicon_tags_29 },
    { 0x3741aec4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a41aec9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a81aecc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de1aed6, SUC_lexicon_tags_11 },
    { 0x6221aed7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd241aed9, SUC_lexicon_tags_19 },
    { 0x1401aeda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3821aedc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c21aedf, SUC_lexicon_tags_37 },
    { 0x5421aedf, SUC_lexicon_tags_43 },
    { 0x8e01aee1, SUC_lexicon_tags_17 },
    { 0x8461aee2, SUC_lexicon_tags_17 },
    { 0xc281aee2, SUC_lexicon_tags_7 },
    { 0xb7c1aee4, SUC_lexicon_tags_49 },
    { 0xc621aee5, SUC_lexicon_tags_65 },
    { 0x5f81aee4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5e1aee8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5721aeec, SUC_lexicon_tags_37 },
    { 0x5fe1aeed, SUC_lexicon_tags_554 },
    { 0x1ac1aeee, SUC_lexicon_tags_37 },
    { 0x91c1aeef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada1aef3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf41aef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4da1aef8, SUC_lexicon_tags_11 },
    { 0x1ca1aef9, SUC_lexicon_tags_38 },
    { 0xa221aef8, SUC_lexicon_tags_37 },
    { 0x80c1aefb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc1aeff, SUC_lexicon_tags_66 },
    { 0x51a1af00, SUC_lexicon_tags_49 },
    { 0x6bc1af00, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b81af04, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7c61af06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f21af09, SUC_lexicon_tags_17 },
    { 0xef61af0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5281af0c, SUC_lexicon_tags_14 },
    { 0x17e1af0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de1af10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe621af14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e1af1c, SUC_lexicon_tags_37 },
    { 0x181af1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c1af20, SUC_lexicon_tags_1 },
    { 0x7221af21, SUC_lexicon_tags_12 },
    { 0x9c61af22, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9661af24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c1af28, SUC_lexicon_tags_23 },
    { 0x56c1af29, SUC_lexicon_tags_28 },
    { 0xee61af2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e1af2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdba1af2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd221af3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x93e1af3e, SUC_lexicon_tags_7 },
    { 0xe621af3e, SUC_lexicon_tags_8 },
    { 0xc481af40, SUC_lexicon_tags_37 },
    { 0xf701af41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9081af45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15c1af47, SUC_lexicon_tags_17 },
    { 0x8b21af48, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce1af4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1161af50, SUC_lexicon_tags_25 },
    { 0xe1c1af50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2481af54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4601af56, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf941af58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3521af5d, SUC_lexicon_tags_1 },
    { 0xb941af5e, SUC_lexicon_tags_443 },
    { 0xf901af5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4e1af61, SUC_lexicon_tags_25 },
    { 0xef01af62, SUC_lexicon_tags_66 },
    { 0xede1af62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13c1af65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1af69, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6de1af6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce21af6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec1af71, SUC_lexicon_tags_37 },
    { 0x48c1af72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6141af76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f21af78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9de1af7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbe1af7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c1af88, SUC_lexicon_tags_11 },
    { 0xbfa1af89, SUC_lexicon_tags_6 },
    { 0xb961af89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad21af8c, SUC_lexicon_tags_17 },
    { 0xe8a1af8c, SUC_lexicon_tags_29 },
    { 0x4041af8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2681af92, SUC_lexicon_tags_12 },
    { 0xc0e1af93, SUC_lexicon_tags_37 },
    { 0xeb81af94, SUC_lexicon_tags_7 },
    { 0x5c21af94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e1af99, SUC_lexicon_tags_7 },
    { 0xc2e1af9a, SUC_lexicon_tags_15 },
    { 0xe121af9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1af9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x13c1afa0, SUC_lexicon_tags_37 },
    { 0xfaa1afa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b41afa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc041afa6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a1afa9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf61afad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a41afb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac81afb3, SUC_lexicon_tags_47 },
    { 0x59c1afb3, SUC_lexicon_tags_15 },
    { 0x1301afb4, SUC_lexicon_tags_49 },
    { 0x21c1afb4, SUC_lexicon_tags_27 },
    { 0x28c1afb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2c1afb9, SUC_lexicon_tags_2 },
    { 0x6241afb9, SUC_lexicon_tags_37 },
    { 0xb061afbb, SUC_lexicon_tags_37 },
    { 0xf901afbb, SUC_lexicon_tags_55 },
    { 0x1f01afbc, SUC_lexicon_tags_45 },
    { 0xf661afba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb61afc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83e1afc2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xab01afc5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde01afcd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa561afcf, SUC_lexicon_tags_2 },
    { 0x1441afcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6641afd2, SUC_lexicon_tags_11 },
    { 0xd201afd2, SUC_lexicon_tags_66 },
    { 0x8421afd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dc1afd6, SUC_lexicon_tags_12 },
    { 0xc0a1afd6, SUC_lexicon_tags_23 },
    { 0x5221afd8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba41afe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c61afec, SUC_lexicon_tags_12 },
    { 0x3741afed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d41afef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb321aff1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec1aff3, SUC_lexicon_tags_39 },
    { 0x8641aff4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1ec1aff6, SUC_lexicon_tags_23 },
    { 0xf021aff7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55e1aff9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc861affb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25c1affd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de1b001, SUC_lexicon_tags_17 },
    { 0x86a1b001, SUC_lexicon_tags_4 },
    { 0x2021b001, SUC_lexicon_tags_19 },
    { 0x5281b001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be1b00a, SUC_lexicon_tags_2 },
    { 0x98e1b00b, SUC_lexicon_tags_6 },
    { 0xce81b00a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d61b015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1561b017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a1b01b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a1b021, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e01b023, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b01b026, SUC_lexicon_tags_26 },
    { 0x24c1b027, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1b02a, SUC_lexicon_tags_11 },
    { 0x6d01b02b, SUC_lexicon_tags_87 },
    { 0x4c61b02b, SUC_lexicon_tags_11 },
    { 0x401b02d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c1b031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcde1b033, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdfa1b035, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c1b03d, SUC_lexicon_tags_47 },
    { 0x8301b03e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c1b041, SUC_lexicon_tags_12 },
    { 0x2061b041, SUC_lexicon_tags_11 },
    { 0xe321b043, SUC_lexicon_tags_28 },
    { 0xfc21b044, SUC_lexicon_tags_14 },
    { 0x9e81b045, SUC_lexicon_tags_40 },
    { 0x4541b045, SUC_lexicon_tags_56 },
    { 0xb041b047, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c81b04a, SUC_lexicon_tags_17 },
    { 0x9e81b04b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf61b04d, SUC_lexicon_tags_30 },
    { 0xb9a1b04e, SUC_lexicon_tags_45 },
    { 0xbe01b04f, SUC_lexicon_tags_28 },
    { 0x9441b04f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xafc1b052, SUC_lexicon_tags_47 },
    { 0xaa41b053, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x57e1b055, SUC_lexicon_tags_11 },
    { 0x3181b055, SUC_lexicon_tags_25 },
    { 0x22e1b057, SUC_lexicon_tags_1 },
    { 0xd241b058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c1b05e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a41b060, SUC_lexicon_tags_63 },
    { 0x3f61b061, SUC_lexicon_tags_3 },
    { 0x6a01b062, SUC_lexicon_tags_11 },
    { 0xdc41b060, SUC_lexicon_tags_29 },
    { 0xd961b064, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f81b06e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a1b071, SUC_lexicon_tags_15 },
    { 0x74a1b072, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74a1b074, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2bc1b076, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1981b07a, SUC_lexicon_tags_7 },
    { 0x9f21b07b, SUC_lexicon_tags_13 },
    { 0x63a1b07c, SUC_lexicon_tags_11 },
    { 0xd321b07b, SUC_lexicon_tags_63 },
    { 0x1841b07e, SUC_lexicon_tags_17 },
    { 0x8e61b07f, SUC_lexicon_tags_1 },
    { 0xbfa1b07e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8e1b082, SUC_lexicon_tags_15 },
    { 0xaa81b083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca81b086, SUC_lexicon_tags_29 },
    { 0x12a1b087, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6801b08e, SUC_lexicon_tags_12 },
    { 0x2901b08f, SUC_lexicon_tags_37 },
    { 0x3901b090, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe341b092, SUC_lexicon_tags_6 },
    { 0xd841b093, SUC_lexicon_tags_21 },
    { 0x8dc1b092, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c41b097, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa921b09a, SUC_lexicon_tags_37 },
    { 0xf61b09a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9941b09f, SUC_lexicon_tags_66 },
    { 0xf3e1b0a0, SUC_lexicon_tags_49 },
    { 0xd21b0a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd7a1b0a3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6381b0a8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x6521b0ab, SUC_lexicon_tags_39 },
    { 0x7b61b0ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e1b0af, SUC_lexicon_tags_1 },
    { 0xda21b0af, SUC_lexicon_tags_49 },
    { 0x2cc1b0b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4aa1b0b3, SUC_lexicon_tags_17 },
    { 0x2ec1b0b4, SUC_lexicon_tags_63 },
    { 0x2921b0b4, SUC_lexicon_tags_14 },
    { 0x88a1b0b6, SUC_lexicon_tags_21 },
    { 0x5d01b0b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea1b0ba, SUC_lexicon_tags_26 },
    { 0xdcc1b0bb, SUC_lexicon_tags_37 },
    { 0x9d21b0bc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf261b0c6, SUC_lexicon_tags_31 },
    { 0x66e1b0c6, SUC_lexicon_tags_2 },
    { 0xc0c1b0c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeae1b0ca, SUC_lexicon_tags_15 },
    { 0x5341b0cb, SUC_lexicon_tags_12 },
    { 0x22a1b0cb, SUC_lexicon_tags_12 },
    { 0xf461b0cd, SUC_lexicon_tags_19 },
    { 0xb121b0cd, SUC_lexicon_tags_17 },
    { 0x33e1b0cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c1b0d6, SUC_lexicon_tags_47 },
    { 0x6bc1b0d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7121b0d9, SUC_lexicon_tags_29 },
    { 0x7581b0da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5741b0de, SUC_lexicon_tags_37 },
    { 0xfd01b0df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa761b0e6, SUC_lexicon_tags_12 },
    { 0x1561b0e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b61b0ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9241b0f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16c1b0f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce1b0f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef61b0f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc1b0fc, SUC_lexicon_tags_30 },
    { 0x4b81b0fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1821b0ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94c1b101, SUC_lexicon_tags_0 },
    { 0x2ca1b102, SUC_lexicon_tags_63 },
    { 0x70a1b101, SUC_lexicon_tags_15 },
    { 0x6f41b103, SUC_lexicon_tags_11 },
    { 0x4e61b103, SUC_lexicon_tags_2 },
    { 0xf681b106, SUC_lexicon_tags_2 },
    { 0x4bc1b103, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a1b10c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b01b110, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c1b114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e1b118, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf421b11a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3381b11f, SUC_lexicon_tags_70 },
    { 0xec21b11f, SUC_lexicon_tags_27 },
    { 0x5761b121, SUC_lexicon_tags_45 },
    { 0xc641b121, SUC_lexicon_tags_911 },
    { 0x4de1b122, SUC_lexicon_tags_45 },
    { 0x5781b122, SUC_lexicon_tags_1 },
    { 0x15c1b125, SUC_lexicon_tags_1 },
    { 0x6081b126, SUC_lexicon_tags_11 },
    { 0xad81b126, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6821b129, SUC_lexicon_tags_15 },
    { 0xdf21b12a, SUC_lexicon_tags_9 },
    { 0x31c1b12b, SUC_lexicon_tags_411 },
    { 0x54a1b12c, SUC_lexicon_tags_71 },
    { 0x4b01b12c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc241b130, SUC_lexicon_tags_48 },
    { 0x7461b131, SUC_lexicon_tags_7 },
    { 0x5b61b131, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7fa1b134, SUC_lexicon_tags_56 },
    { 0x35a1b135, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b01b137, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da1b13a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a1b13f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc81b142, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf7e1b144, SUC_lexicon_tags_31 },
    { 0xfac1b144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99e1b147, SUC_lexicon_tags_269 },
    { 0x3261b148, SUC_lexicon_tags_15 },
    { 0xb8e1b148, SUC_lexicon_tags_74 },
    { 0xca21b14a, SUC_lexicon_tags_14 },
    { 0x3b81b14b, SUC_lexicon_tags_3 },
    { 0x3f61b14c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99e1b14e, SUC_lexicon_tags_261 },
    { 0x6561b14f, SUC_lexicon_tags_4 },
    { 0xea1b14e, SUC_lexicon_tags_2 },
    { 0xaf61b151, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2261b153, SUC_lexicon_tags_17 },
    { 0x89a1b154, SUC_lexicon_tags_11 },
    { 0x1f61b155, SUC_lexicon_tags_12 },
    { 0x5841b153, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa1b15a, SUC_lexicon_tags_15 },
    { 0xd121b15a, SUC_lexicon_tags_25 },
    { 0x1041b15b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e1b160, SUC_lexicon_tags_0 },
    { 0x301b161, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6221b165, SUC_lexicon_tags_17 },
    { 0xda1b166, SUC_lexicon_tags_15 },
    { 0x9461b167, SUC_lexicon_tags_37 },
    { 0x6c1b168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4401b16a, SUC_lexicon_tags_25 },
    { 0x61a1b16a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x881b16e, SUC_lexicon_tags_28 },
    { 0x76c1b16f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc1b174, SUC_lexicon_tags_37 },
    { 0x38c1b175, SUC_lexicon_tags_66 },
    { 0xd5c1b176, SUC_lexicon_tags_46 },
    { 0xeda1b175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e1b17c, SUC_lexicon_tags_6 },
    { 0x3461b17c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1741b17f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc1b185, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac1b188, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xebe1b18a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee21b18d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b41b18f, SUC_lexicon_tags_3 },
    { 0xb5c1b190, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb161b195, SUC_lexicon_tags_23 },
    { 0xb741b195, SUC_lexicon_tags_6 },
    { 0x33c1b196, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1441b1a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc001b1a2, SUC_lexicon_tags_107 },
    { 0x5cc1b1a3, SUC_lexicon_tags_7 },
    { 0x8a41b1a3, SUC_lexicon_tags_11 },
    { 0x4c1b1a5, SUC_lexicon_tags_61 },
    { 0xfa81b1a6, SUC_lexicon_tags_49 },
    { 0x9721b1a6, SUC_lexicon_tags_14 },
    { 0xbe81b1a8, SUC_lexicon_tags_12 },
    { 0x6001b1a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761b1b1, SUC_lexicon_tags_11 },
    { 0x9a81b1b2, SUC_lexicon_tags_2 },
    { 0x54c1b1b3, SUC_lexicon_tags_49 },
    { 0x16e1b1b4, SUC_lexicon_tags_25 },
    { 0x1201b1b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e1b1bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7161b1be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e61b1c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de1b1ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf341b1d0, SUC_lexicon_tags_17 },
    { 0xbd21b1d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf701b1d3, SUC_lexicon_tags_32 },
    { 0xd901b1d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1401b1d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec1b1da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x541b1dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2461b1e1, SUC_lexicon_tags_7 },
    { 0xf801b1e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e1b1e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa81b1e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeda1b1e9, SUC_lexicon_tags_1 },
    { 0x1c21b1e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf201b1ed, SUC_lexicon_tags_25 },
    { 0xe1a1b1ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1181b1f6, SUC_lexicon_tags_12 },
    { 0x3821b1f6, SUC_lexicon_tags_37 },
    { 0x1f01b1f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3e1b1fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8441b1fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfc1b1fe, SUC_lexicon_tags_17 },
    { 0x1a41b1ff, SUC_lexicon_tags_7 },
    { 0x78a1b200, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc761b206, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e1b20b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc141b20d, SUC_lexicon_tags_47 },
    { 0x6ce1b20e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa1b213, SUC_lexicon_tags_25 },
    { 0xfe01b214, SUC_lexicon_tags_29 },
    { 0x50c1b215, SUC_lexicon_tags_37 },
    { 0xd901b216, SUC_lexicon_tags_23 },
    { 0xb321b217, SUC_lexicon_tags_25 },
    { 0x681b218, SUC_lexicon_tags_1 },
    { 0xcc61b213, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd201b21b, SUC_lexicon_tags_4 },
    { 0x2c21b21c, SUC_lexicon_tags_15 },
    { 0x9b21b21d, SUC_lexicon_tags_7 },
    { 0xc6a1b21d, SUC_lexicon_tags_7 },
    { 0xcf61b21d, SUC_lexicon_tags_108 },
    { 0x3041b21b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9fc1b222, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x98e1b224, SUC_lexicon_tags_15 },
    { 0x99c1b225, SUC_lexicon_tags_25 },
    { 0x9501b226, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3101b22c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1641b230, SUC_lexicon_tags_79 },
    { 0xd141b231, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e1b234, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a1b237, SUC_lexicon_tags_25 },
    { 0x9a61b238, SUC_lexicon_tags_25 },
    { 0xa41b238, SUC_lexicon_tags_33 },
    { 0x6761b239, SUC_lexicon_tags_11 },
    { 0x8661b237, SUC_lexicon_tags_12 },
    { 0xcb41b237, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde81b242, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6821b24a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf81b24c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd961b24e, SUC_lexicon_tags_37 },
    { 0x6aa1b24e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5e1b251, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x47a1b253, SUC_lexicon_tags_47 },
    { 0x3e61b254, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a1b25a, SUC_lexicon_tags_410 },
    { 0x97c1b25b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9c41b25d, SUC_lexicon_tags_17 },
    { 0x2161b25e, SUC_lexicon_tags_55 },
    { 0x1261b25f, SUC_lexicon_tags_14 },
    { 0x5481b260, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb01b262, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1341b264, SUC_lexicon_tags_38 },
    { 0x2841b265, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfaa1b267, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a1b26a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c1b26e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e1b274, SUC_lexicon_tags_37 },
    { 0xaca1b275, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de1b27a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e1b27d, SUC_lexicon_tags_29 },
    { 0x3b01b27d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4221b285, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca1b288, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e1b28b, SUC_lexicon_tags_17 },
    { 0xa21b28c, SUC_lexicon_tags_12 },
    { 0xab41b28d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a1b293, SUC_lexicon_tags_27 },
    { 0xe9e1b293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d41b29e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7001b2a0, SUC_lexicon_tags_82 },
    { 0x121b2a1, SUC_lexicon_tags_1 },
    { 0x98e1b2a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a1b2a8, SUC_lexicon_tags_30 },
    { 0xd7c1b2a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc081b2ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf81b2b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c1b2b5, SUC_lexicon_tags_68 },
    { 0xbac1b2b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe541b2be, SUC_lexicon_tags_25 },
    { 0xca61b2bf, SUC_lexicon_tags_56 },
    { 0x221b2c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e01b2c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe1b2c6, SUC_lexicon_tags_11 },
    { 0x5421b2c6, SUC_lexicon_tags_17 },
    { 0x7a41b2c8, SUC_lexicon_tags_49 },
    { 0x5a21b2c9, SUC_lexicon_tags_21 },
    { 0xa281b2c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3261b2cd, SUC_lexicon_tags_37 },
    { 0x5a21b2ce, SUC_lexicon_tags_14 },
    { 0xc161b2cf, SUC_lexicon_tags_30 },
    { 0x45a1b2cf, SUC_lexicon_tags_2 },
    { 0xe441b2d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7181b2d4, SUC_lexicon_tags_47 },
    { 0x8681b2d5, SUC_lexicon_tags_11 },
    { 0x3261b2d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac1b2db, SUC_lexicon_tags_25 },
    { 0x51a1b2dc, SUC_lexicon_tags_15 },
    { 0xc5c1b2dd, SUC_lexicon_tags_62 },
    { 0xc6e1b2de, SUC_lexicon_tags_11 },
    { 0x2aa1b2df, SUC_lexicon_tags_48 },
    { 0x45a1b2df, SUC_lexicon_tags_12 },
    { 0x7a01b2e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa281b2e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e1b2e8, SUC_lexicon_tags_26 },
    { 0x4e01b2e8, SUC_lexicon_tags_38 },
    { 0xbda1b2ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a1b2f1, SUC_lexicon_tags_17 },
    { 0x1f61b2f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4fc1b2f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a1b301, SUC_lexicon_tags_43 },
    { 0xea21b302, SUC_lexicon_tags_38 },
    { 0x5e61b303, SUC_lexicon_tags_17 },
    { 0xd5c1b303, SUC_lexicon_tags_47 },
    { 0x9221b305, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c1b30a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6641b30d, SUC_lexicon_tags_29 },
    { 0x36a1b30e, SUC_lexicon_tags_7 },
    { 0xabc1b30f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc381b311, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe741b319, SUC_lexicon_tags_33 },
    { 0xbdc1b31a, SUC_lexicon_tags_2 },
    { 0xcd41b319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda41b321, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c1b326, SUC_lexicon_tags_29 },
    { 0x2861b327, SUC_lexicon_tags_12 },
    { 0xc4c1b328, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf221b333, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c1b336, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f41b33b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c1b341, SUC_lexicon_tags_108 },
    { 0x5de1b341, SUC_lexicon_tags_29 },
    { 0x58e1b343, SUC_lexicon_tags_4 },
    { 0x5841b343, SUC_lexicon_tags_47 },
    { 0xe221b342, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b81b347, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3321b34a, SUC_lexicon_tags_25 },
    { 0x3881b34b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e1b351, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c1b357, SUC_lexicon_tags_23 },
    { 0x2201b357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c1b35d, SUC_lexicon_tags_19 },
    { 0xc781b35d, SUC_lexicon_tags_37 },
    { 0x3041b35e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa1b363, SUC_lexicon_tags_10 },
    { 0x7041b364, SUC_lexicon_tags_11 },
    { 0xe161b364, SUC_lexicon_tags_31 },
    { 0xe181b366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f41b369, SUC_lexicon_tags_37 },
    { 0xb8a1b36a, SUC_lexicon_tags_55 },
    { 0x8b21b369, SUC_lexicon_tags_27 },
    { 0x7581b36c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc1b36f, SUC_lexicon_tags_14 },
    { 0xef41b370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7281b377, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfe1b379, SUC_lexicon_tags_12 },
    { 0x2d21b37a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5f61b37c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c1b37f, SUC_lexicon_tags_49 },
    { 0x21b380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde21b386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a1b388, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2781b38a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc81b38c, SUC_lexicon_tags_19 },
    { 0x7de1b38d, SUC_lexicon_tags_17 },
    { 0x7a41b38e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e1b394, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xca61b396, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c81b398, SUC_lexicon_tags_151 },
    { 0xba81b399, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e1b39d, SUC_lexicon_tags_25 },
    { 0xb2e1b39e, SUC_lexicon_tags_11 },
    { 0xcf61b39f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9981b3a2, SUC_lexicon_tags_15 },
    { 0xeca1b3a3, SUC_lexicon_tags_43 },
    { 0x7221b3a4, SUC_lexicon_tags_17 },
    { 0x4dc1b3a5, SUC_lexicon_tags_37 },
    { 0x1981b3a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2841b3aa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c1b3af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda21b3b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c1b3b7, SUC_lexicon_tags_25 },
    { 0xa101b3b8, SUC_lexicon_tags_30 },
    { 0xe21b3b9, SUC_lexicon_tags_25 },
    { 0x9c41b3b8, SUC_lexicon_tags_2 },
    { 0x7a21b3b9, SUC_lexicon_tags_5 },
    { 0xaaa1b3bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1061b3be, SUC_lexicon_tags_92 },
    { 0xbd41b3be, SUC_lexicon_tags_27 },
    { 0x9461b3bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a81b3c4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb21b3c9, SUC_lexicon_tags_18 },
    { 0x6281b3c9, SUC_lexicon_tags_37 },
    { 0x9501b3cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1b3d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc961b3d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e61b3d7, SUC_lexicon_tags_23 },
    { 0xae1b3d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4301b3db, SUC_lexicon_tags_12 },
    { 0xc1b3dc, SUC_lexicon_tags_49 },
    { 0xab01b3dd, SUC_lexicon_tags_39 },
    { 0x7d61b3de, SUC_lexicon_tags_37 },
    { 0x9701b3db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c1b3e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9261b3e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c1b3eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa841b3f0, SUC_lexicon_tags_58 },
    { 0x48e1b3f1, SUC_lexicon_tags_11 },
    { 0x6cc1b3f1, SUC_lexicon_tags_19 },
    { 0x6b61b3f3, SUC_lexicon_tags_0 },
    { 0x9101b3f4, SUC_lexicon_tags_174 },
    { 0x21e1b3f4, SUC_lexicon_tags_32 },
    { 0x3f41b3f6, SUC_lexicon_tags_17 },
    { 0x381b3f4, SUC_lexicon_tags_79 },
    { 0xb621b3f8, SUC_lexicon_tags_8 },
    { 0x4061b3f9, SUC_lexicon_tags_23 },
    { 0x8381b3f0, SUC_lexicon_tags_171 },
    { 0x30e1b3fb, SUC_lexicon_tags_14 },
    { 0xcd21b3f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6201b402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa041b40b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe461b40d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a1b410, SUC_lexicon_tags_25 },
    { 0x8381b411, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1621b414, SUC_lexicon_tags_11 },
    { 0x27c1b414, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8341b418, SUC_lexicon_tags_37 },
    { 0x23c1b419, SUC_lexicon_tags_15 },
    { 0x201b41a, SUC_lexicon_tags_26 },
    { 0x4a21b41a, SUC_lexicon_tags_17 },
    { 0xbb41b41a, SUC_lexicon_tags_194 },
    { 0xa561b41d, SUC_lexicon_tags_25 },
    { 0x5641b41e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf61b421, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e1b425, SUC_lexicon_tags_48 },
    { 0x9e01b426, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5381b42a, SUC_lexicon_tags_11 },
    { 0x6e01b42b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7541b42f, SUC_lexicon_tags_39 },
    { 0x1961b430, SUC_lexicon_tags_2 },
    { 0x8241b431, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5181b433, SUC_lexicon_tags_65 },
    { 0xa6c1b433, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf01b439, SUC_lexicon_tags_7 },
    { 0x1761b43a, SUC_lexicon_tags_2 },
    { 0xc241b439, SUC_lexicon_tags_25 },
    { 0x5ee1b43c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a1b43f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc1b442, SUC_lexicon_tags_65 },
    { 0xb501b443, SUC_lexicon_tags_34 },
    { 0xde01b444, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9761b447, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae1b44d, SUC_lexicon_tags_11 },
    { 0xebc1b44d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8dc1b450, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9921b452, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf941b459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b21b45d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8461b463, SUC_lexicon_tags_33 },
    { 0xe1e1b464, SUC_lexicon_tags_27 },
    { 0xd3a1b465, SUC_lexicon_tags_25 },
    { 0xb141b465, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd021b46a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa9c1b46c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7c1b46e, SUC_lexicon_tags_11 },
    { 0x7e1b46f, SUC_lexicon_tags_70 },
    { 0x7921b470, SUC_lexicon_tags_14 },
    { 0xa41b46e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e1b474, SUC_lexicon_tags_48 },
    { 0x7721b475, SUC_lexicon_tags_33 },
    { 0x1c01b476, SUC_lexicon_tags_12 },
    { 0xd81b475, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc101b479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba1b47f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a1b484, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac61b489, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd361b48d, SUC_lexicon_tags_61 },
    { 0x4981b48d, SUC_lexicon_tags_49 },
    { 0x13c1b48e, SUC_lexicon_tags_8 },
    { 0x5321b48e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b21b493, SUC_lexicon_tags_25 },
    { 0x31c1b494, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c41b496, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e1b499, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da1b49d, SUC_lexicon_tags_37 },
    { 0x9b41b49e, SUC_lexicon_tags_23 },
    { 0x6a1b49d, SUC_lexicon_tags_39 },
    { 0xc161b4a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde1b4a2, SUC_lexicon_tags_7 },
    { 0x5f21b4a3, SUC_lexicon_tags_12 },
    { 0x8a21b4a2, SUC_lexicon_tags_2 },
    { 0x8c81b4a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc1b4a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8081b4b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6881b4b6, SUC_lexicon_tags_43 },
    { 0xcb01b4b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x46e1b4b9, SUC_lexicon_tags_10 },
    { 0x2781b4ba, SUC_lexicon_tags_31 },
    { 0x5e41b4bb, SUC_lexicon_tags_8 },
    { 0x2861b4bb, SUC_lexicon_tags_33 },
    { 0xdc01b4b9, SUC_lexicon_tags_15 },
    { 0x96c1b4bc, SUC_lexicon_tags_43 },
    { 0x30e1b4bf, SUC_lexicon_tags_68 },
    { 0x3841b4ba, SUC_lexicon_tags_29 },
    { 0xc781b4c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f01b4c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e1b4cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22c1b4ce, SUC_lexicon_tags_15 },
    { 0x6c01b4ce, SUC_lexicon_tags_14 },
    { 0x981b4ce, SUC_lexicon_tags_7 },
    { 0xc81b4d1, SUC_lexicon_tags_37 },
    { 0x9521b4d2, SUC_lexicon_tags_46 },
    { 0x6941b4d3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe721b4dc, SUC_lexicon_tags_17 },
    { 0xc9c1b4dd, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xece1b4e0, SUC_lexicon_tags_12 },
    { 0x4741b4e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8481b4e4, SUC_lexicon_tags_29 },
    { 0x15a1b4e5, SUC_lexicon_tags_8 },
    { 0x45a1b4e6, SUC_lexicon_tags_2 },
    { 0x2d01b4e5, SUC_lexicon_tags_9 },
    { 0x6881b4e8, SUC_lexicon_tags_17 },
    { 0x32c1b4e9, SUC_lexicon_tags_23 },
    { 0x1421b4ea, SUC_lexicon_tags_2 },
    { 0xb3a1b4ea, SUC_lexicon_tags_37 },
    { 0x4521b4eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f41b4f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb521b4f8, SUC_lexicon_tags_23 },
    { 0xf21b4f9, SUC_lexicon_tags_38 },
    { 0x1601b4fa, SUC_lexicon_tags_0 },
    { 0xa261b4fb, SUC_lexicon_tags_17 },
    { 0xcfe1b4fc, SUC_lexicon_tags_2 },
    { 0xf001b4fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a81b4ff, SUC_lexicon_tags_15 },
    { 0xdcc1b500, SUC_lexicon_tags_37 },
    { 0x5661b500, SUC_lexicon_tags_20 },
    { 0x68e1b502, SUC_lexicon_tags_11 },
    { 0xe441b500, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b01b505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd321b507, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a1b509, SUC_lexicon_tags_4 },
    { 0x5a21b50a, SUC_lexicon_tags_27 },
    { 0x3661b50a, SUC_lexicon_tags_112 },
    { 0xa141b50c, SUC_lexicon_tags_120 },
    { 0x3401b50d, SUC_lexicon_tags_2 },
    { 0x2ec1b50e, SUC_lexicon_tags_25 },
    { 0x97c1b50f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc501b515, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a1b518, SUC_lexicon_tags_47 },
    { 0x5dc1b518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f21b51b, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0xcdc1b51d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a1b522, SUC_lexicon_tags_105 },
    { 0x3ba1b522, SUC_lexicon_tags_9 },
    { 0xb741b524, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a1b529, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc581b52c, SUC_lexicon_tags_16 },
    { 0x4261b52c, SUC_lexicon_tags_81 },
    { 0x50a1b52c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a1b532, SUC_lexicon_tags_43 },
    { 0x5421b533, SUC_lexicon_tags_2 },
    { 0x64a1b533, SUC_lexicon_tags_31 },
    { 0x6e01b532, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc61b537, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9941b53f, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec1b543, SUC_lexicon_tags_10 },
    { 0x3161b543, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b61b547, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa281b550, SUC_lexicon_tags_7 },
    { 0xbf61b551, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba61b557, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0c1b559, SUC_lexicon_tags_32 },
    { 0xdea1b559, SUC_lexicon_tags_11 },
    { 0x5021b55a, SUC_lexicon_tags_2 },
    { 0x2361b559, SUC_lexicon_tags_87 },
    { 0x1e61b55b, SUC_lexicon_tags_2 },
    { 0xcb41b559, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4521b564, SUC_lexicon_tags_37 },
    { 0xd0e1b565, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3641b56d, SUC_lexicon_tags_7 },
    { 0x13a1b56e, SUC_lexicon_tags_6 },
    { 0xd501b56f, SUC_lexicon_tags_7 },
    { 0xb641b570, SUC_lexicon_tags_12 },
    { 0x7a61b570, SUC_lexicon_tags_11 },
    { 0x72c1b572, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc1b57b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9181b580, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf2e1b582, SUC_lexicon_tags_13 },
    { 0xc4e1b583, SUC_lexicon_tags_12 },
    { 0x2161b584, SUC_lexicon_tags_17 },
    { 0x46a1b585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a1b58c, SUC_lexicon_tags_47 },
    { 0x1ba1b58d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf61b598, SUC_lexicon_tags_17 },
    { 0xf21b599, SUC_lexicon_tags_12 },
    { 0xcc1b599, SUC_lexicon_tags_27 },
    { 0xa1b599, SUC_lexicon_tags_65 },
    { 0xc901b59b, SUC_lexicon_tags_48 },
    { 0x9721b59b, SUC_lexicon_tags_5 },
    { 0x8c41b59d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x781b5a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x23a1b5a3, SUC_lexicon_tags_45 },
    { 0x8541b5a4, SUC_lexicon_tags_49 },
    { 0x4b81b5a3, SUC_lexicon_tags_37 },
    { 0x39e1b5a6, SUC_lexicon_tags_25 },
    { 0x7521b5a7, SUC_lexicon_tags_1 },
    { 0x2a21b5a8, SUC_lexicon_tags_37 },
    { 0x73c1b5a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa1e1b5ab, SUC_lexicon_tags_61 },
    { 0x8d21b5ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29c1b5ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6c41b5b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf5c1b5b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5c01b5b4, SUC_lexicon_tags_48 },
    { 0x56e1b5b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1b5b7, SUC_lexicon_tags_15 },
    { 0x91c1b5b8, SUC_lexicon_tags_1 },
    { 0xa9c1b5b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb01b5bd, SUC_lexicon_tags_2 },
    { 0x6481b5be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a1b5c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcac1b5c4, SUC_lexicon_tags_12 },
    { 0x68e1b5c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6341b5c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4d81b5cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c1b5d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb261b5db, SUC_lexicon_tags_63 },
    { 0x6d21b5dc, SUC_lexicon_tags_37 },
    { 0x3fa1b5dd, SUC_lexicon_tags_1 },
    { 0x9fe1b5de, SUC_lexicon_tags_56 },
    { 0xf801b5db, SUC_lexicon_tags_49 },
    { 0x8ee1b5e0, SUC_lexicon_tags_49 },
    { 0x4a1b5e1, SUC_lexicon_tags_25 },
    { 0x3dc1b5e2, SUC_lexicon_tags_645 },
    { 0xd01b5e1, SUC_lexicon_tags_11 },
    { 0xcd81b5de, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2861b5e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e41b5ec, SUC_lexicon_tags_84 },
    { 0x8b21b5ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba1b5f2, SUC_lexicon_tags_47 },
    { 0x3ce1b5f2, SUC_lexicon_tags_15 },
    { 0xada1b5f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d81b5f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7de1b5fa, SUC_lexicon_tags_48 },
    { 0x4541b5fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa561b604, SUC_lexicon_tags_25 },
    { 0x1e81b604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb21b608, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3c61b60a, SUC_lexicon_tags_37 },
    { 0x3461b60b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x21b60d, SUC_lexicon_tags_17 },
    { 0x29c1b60e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fa1b610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e1b615, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb01b618, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa761b61a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6501b61f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8901b621, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4101b625, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa4c1b627, SUC_lexicon_tags_0 },
    { 0x8721b627, SUC_lexicon_tags_17 },
    { 0xcce1b629, SUC_lexicon_tags_605 },
    { 0xc6a1b629, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae1b632, SUC_lexicon_tags_38 },
    { 0x6461b633, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a1b639, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac1b645, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a61b64a, SUC_lexicon_tags_37 },
    { 0x8c41b64a, SUC_lexicon_tags_37 },
    { 0x1e01b64c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a1b64f, SUC_lexicon_tags_11 },
    { 0x31a1b64f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e01b652, SUC_lexicon_tags_25 },
    { 0xf961b653, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xeca1b655, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x81e1b657, SUC_lexicon_tags_17 },
    { 0xdc61b658, SUC_lexicon_tags_0 },
    { 0x401b659, SUC_lexicon_tags_11 },
    { 0x7a41b658, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae81b662, SUC_lexicon_tags_7 },
    { 0x9b81b662, SUC_lexicon_tags_9 },
    { 0x1441b664, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a61b668, SUC_lexicon_tags_1 },
    { 0xc421b668, SUC_lexicon_tags_11 },
    { 0x1521b669, SUC_lexicon_tags_46 },
    { 0xa6e1b66a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca21b670, SUC_lexicon_tags_15 },
    { 0xe781b671, SUC_lexicon_tags_8 },
    { 0x261b672, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae1b676, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7aa1b678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4921b67c, SUC_lexicon_tags_15 },
    { 0x6d81b67c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece1b682, SUC_lexicon_tags_63 },
    { 0x5ca1b683, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4041b687, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ea1b689, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e61b68c, SUC_lexicon_tags_1 },
    { 0x1ec1b68d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce41b693, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9861b695, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe61b697, SUC_lexicon_tags_47 },
    { 0x8841b698, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c1b69c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e1b69f, SUC_lexicon_tags_30 },
    { 0xcf01b6a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2721b6a4, SUC_lexicon_tags_7 },
    { 0x6a61b6a5, SUC_lexicon_tags_17 },
    { 0x43a1b6a4, SUC_lexicon_tags_7 },
    { 0xa1a1b6a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1701b6aa, SUC_lexicon_tags_17 },
    { 0xaaa1b6aa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd761b6ae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdbc1b6b0, SUC_lexicon_tags_7 },
    { 0x7ea1b6b1, SUC_lexicon_tags_13 },
    { 0x8721b6b1, SUC_lexicon_tags_37 },
    { 0x1a21b6b3, SUC_lexicon_tags_27 },
    { 0x9de1b6b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc81b6b7, SUC_lexicon_tags_37 },
    { 0xd0e1b6b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e1b6bf, SUC_lexicon_tags_29 },
    { 0x5d21b6c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c1b6c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e1b6c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa061b6c8, SUC_lexicon_tags_846 },
    { 0, NULL },
    { 0x85c1b6ca, SUC_lexicon_tags_13 },
    { 0xcf41b6cb, SUC_lexicon_tags_15 },
    { 0xe4c1b6ca, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf61b6cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc1b6d2, SUC_lexicon_tags_17 },
    { 0x2441b6d3, SUC_lexicon_tags_1 },
    { 0x1e01b6d4, SUC_lexicon_tags_25 },
    { 0xc7c1b6d5, SUC_lexicon_tags_1 },
    { 0xcd41b6d2, SUC_lexicon_tags_1 },
    { 0x7541b6d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c1b6da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb501b6dd, SUC_lexicon_tags_29 },
    { 0xe161b6de, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec41b6e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1b6e5, SUC_lexicon_tags_7 },
    { 0x3c41b6e6, SUC_lexicon_tags_8 },
    { 0x64a1b6e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4141b6eb, SUC_lexicon_tags_1 },
    { 0xc561b6ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5801b6f4, SUC_lexicon_tags_49 },
    { 0x5cc1b6f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c1b6f9, SUC_lexicon_tags_11 },
    { 0x8921b6f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc201b6ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be1b710, SUC_lexicon_tags_12 },
    { 0xdb81b710, SUC_lexicon_tags_43 },
    { 0xd381b710, SUC_lexicon_tags_1 },
    { 0x1be1b712, SUC_lexicon_tags_37 },
    { 0x66e1b714, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6521b716, SUC_lexicon_tags_16 },
    { 0x92c1b717, SUC_lexicon_tags_1 },
    { 0xdc61b718, SUC_lexicon_tags_11 },
    { 0x45a1b716, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa481b71b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2841b71d, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461b722, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5a1b724, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7321b727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc61b729, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9781b72b, SUC_lexicon_tags_17 },
    { 0x71c1b72b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc1b736, SUC_lexicon_tags_7 },
    { 0x8b41b736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34c1b739, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c1b73c, SUC_lexicon_tags_5 },
    { 0x8901b73d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e1b740, SUC_lexicon_tags_32 },
    { 0x3ec1b741, SUC_lexicon_tags_33 },
    { 0xeba1b741, SUC_lexicon_tags_12 },
    { 0x5801b741, SUC_lexicon_tags_7 },
    { 0x7ee1b744, SUC_lexicon_tags_9 },
    { 0x5981b744, SUC_lexicon_tags_46 },
    { 0xc161b746, SUC_lexicon_tags_25 },
    { 0x69e1b747, SUC_lexicon_tags_25 },
    { 0x46e1b742, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x75a1b74a, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b81b74d, SUC_lexicon_tags_84 },
    { 0x2be1b74e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe41b752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a1b756, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2801b75a, SUC_lexicon_tags_49 },
    { 0xbe61b75b, SUC_lexicon_tags_2 },
    { 0x3321b75c, SUC_lexicon_tags_0 },
    { 0x2e01b75a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f81b760, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c01b763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1c1b765, SUC_lexicon_tags_37 },
    { 0xdee1b765, SUC_lexicon_tags_7 },
    { 0xab21b765, SUC_lexicon_tags_1 },
    { 0x43c1b768, SUC_lexicon_tags_179 },
    { 0x8421b768, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a1b76e, SUC_lexicon_tags_33 },
    { 0x5321b76f, SUC_lexicon_tags_15 },
    { 0xd961b76e, SUC_lexicon_tags_37 },
    { 0x9481b770, SUC_lexicon_tags_57 },
    { 0x28e1b772, SUC_lexicon_tags_17 },
    { 0xee61b773, SUC_lexicon_tags_15 },
    { 0x2921b774, SUC_lexicon_tags_63 },
    { 0xff41b775, SUC_lexicon_tags_31 },
    { 0x21e1b774, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe521b77a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf161b77c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8921b77f, SUC_lexicon_tags_65 },
    { 0x5901b780, SUC_lexicon_tags_7 },
    { 0x87c1b781, SUC_lexicon_tags_6 },
    { 0x8741b780, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8621b785, SUC_lexicon_tags_2 },
    { 0x25e1b786, SUC_lexicon_tags_23 },
    { 0xb61b787, SUC_lexicon_tags_25 },
    { 0x45e1b787, SUC_lexicon_tags_0 },
    { 0x9461b789, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdfa1b78b, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc1b78e, SUC_lexicon_tags_29 },
    { 0x84e1b78f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaea1b791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac1b795, SUC_lexicon_tags_34 },
    { 0x5981b796, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9561b79b, SUC_lexicon_tags_19 },
    { 0xac01b79c, SUC_lexicon_tags_43 },
    { 0xc6a1b79b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a1b7a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41b7a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9c21b7a8, SUC_lexicon_tags_60 },
    { 0x6dc1b7a8, SUC_lexicon_tags_4 },
    { 0x4f41b7a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9161b7ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d81b7b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e1b7b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xed61b7b7, SUC_lexicon_tags_15 },
    { 0xda1b7b7, SUC_lexicon_tags_2 },
    { 0xe1e1b7b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e1b7bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e61b7bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c1b7c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe41b7c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7861b7c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9181b7d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1081b7d4, SUC_lexicon_tags_63 },
    { 0x7b41b7d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a1b7d8, SUC_lexicon_tags_56 },
    { 0x3961b7d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c1b7dd, SUC_lexicon_tags_17 },
    { 0x9481b7de, SUC_lexicon_tags_15 },
    { 0x7e21b7df, SUC_lexicon_tags_34 },
    { 0x5381b7de, SUC_lexicon_tags_25 },
    { 0x9361b7dd, SUC_lexicon_tags_9 },
    { 0x7a61b7de, SUC_lexicon_tags_43 },
    { 0xedc1b7e3, SUC_lexicon_tags_12 },
    { 0x99c1b7e4, SUC_lexicon_tags_26 },
    { 0xa1a1b7e5, SUC_lexicon_tags_49 },
    { 0x5121b7e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4001b7eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc361b7f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf801b7f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b21b7fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7781b805, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b21b807, SUC_lexicon_tags_9 },
    { 0x9581b808, SUC_lexicon_tags_37 },
    { 0xcdc1b807, SUC_lexicon_tags_35 },
    { 0xe081b807, SUC_lexicon_tags_48 },
    { 0x3881b808, SUC_lexicon_tags_35 },
    { 0x2dc1b80a, SUC_lexicon_tags_32 },
    { 0xc4e1b80d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b81b80f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd901b814, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe1b818, SUC_lexicon_tags_70 },
    { 0x8ce1b818, SUC_lexicon_tags_84 },
    { 0x5e61b818, SUC_lexicon_tags_6 },
    { 0x2061b81a, SUC_lexicon_tags_17 },
    { 0x45c1b81c, SUC_lexicon_tags_37 },
    { 0x4d81b81d, SUC_lexicon_tags_19 },
    { 0x3ea1b81e, SUC_lexicon_tags_1 },
    { 0x421b81f, SUC_lexicon_tags_15 },
    { 0xf201b820, SUC_lexicon_tags_46 },
    { 0xdb01b81f, SUC_lexicon_tags_26 },
    { 0xf561b81d, SUC_lexicon_tags_57 },
    { 0x3ac1b823, SUC_lexicon_tags_31 },
    { 0xa281b824, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x44a1b826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4201b82e, SUC_lexicon_tags_1 },
    { 0xf121b82f, SUC_lexicon_tags_525 },
    { 0x8ac1b82f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31c1b832, SUC_lexicon_tags_46 },
    { 0x8021b833, SUC_lexicon_tags_25 },
    { 0xc21b833, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3581b839, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a1b83c, SUC_lexicon_tags_37 },
    { 0xe61b83d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5021b840, SUC_lexicon_tags_38 },
    { 0x76c1b840, SUC_lexicon_tags_17 },
    { 0x1881b841, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1301b844, SUC_lexicon_tags_25 },
    { 0x4a21b845, SUC_lexicon_tags_7 },
    { 0x1641b845, SUC_lexicon_tags_42 },
    { 0x30c1b844, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc041b849, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9961b84b, SUC_lexicon_tags_31 },
    { 0x1e1b84b, SUC_lexicon_tags_23 },
    { 0xfac1b84c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd581b84f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3341b851, SUC_lexicon_tags_14 },
    { 0x89e1b851, SUC_lexicon_tags_23 },
    { 0x6d41b853, SUC_lexicon_tags_37 },
    { 0x8481b851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc081b85c, SUC_lexicon_tags_7 },
    { 0x46c1b85d, SUC_lexicon_tags_15 },
    { 0xf0e1b85e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2501b861, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e81b863, SUC_lexicon_tags_28 },
    { 0x5c81b864, SUC_lexicon_tags_48 },
    { 0x43e1b864, SUC_lexicon_tags_31 },
    { 0x6ae1b864, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4301b868, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9061b86a, SUC_lexicon_tags_77 },
    { 0x9d41b86b, SUC_lexicon_tags_38 },
    { 0x21e1b86b, SUC_lexicon_tags_55 },
    { 0xb441b86c, SUC_lexicon_tags_31 },
    { 0x96c1b86b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd481b872, SUC_lexicon_tags_38 },
    { 0x45c1b873, SUC_lexicon_tags_62 },
    { 0x4f21b874, SUC_lexicon_tags_5 },
    { 0xfe1b872, SUC_lexicon_tags_17 },
    { 0x5dc1b876, SUC_lexicon_tags_43 },
    { 0xf221b877, SUC_lexicon_tags_43 },
    { 0xede1b878, SUC_lexicon_tags_58 },
    { 0x1be1b873, SUC_lexicon_tags_1 },
    { 0xcfc1b879, SUC_lexicon_tags_33 },
    { 0x82a1b875, SUC_lexicon_tags_33 },
    { 0x2ce1b87c, SUC_lexicon_tags_47 },
    { 0x7b81b87c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c81b881, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c1b889, SUC_lexicon_tags_27 },
    { 0x1f81b88a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c21b88c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e1b88f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2e1b891, SUC_lexicon_tags_12 },
    { 0xa2e1b891, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb641b894, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f41b899, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x84e1b89b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b81b89d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2381b8a5, SUC_lexicon_tags_2 },
    { 0x33c1b8a6, SUC_lexicon_tags_30 },
    { 0x16c1b8a7, SUC_lexicon_tags_37 },
    { 0x55a1b8a7, SUC_lexicon_tags_11 },
    { 0xfdc1b8a8, SUC_lexicon_tags_17 },
    { 0xf701b8a7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb981b8ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5421b8b0, SUC_lexicon_tags_17 },
    { 0x93e1b8b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c1b8b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a1b8ba, SUC_lexicon_tags_85 },
    { 0xdf81b8bb, SUC_lexicon_tags_66 },
    { 0xc8e1b8bb, SUC_lexicon_tags_12 },
    { 0xc5e1b8bc, SUC_lexicon_tags_7 },
    { 0x2481b8bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1161b8c3, SUC_lexicon_tags_39 },
    { 0xf2c1b8c4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f41b8d0, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01b8d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8601b8d9, SUC_lexicon_tags_7 },
    { 0xd161b8d9, SUC_lexicon_tags_22 },
    { 0x5361b8da, SUC_lexicon_tags_14 },
    { 0x4921b8da, SUC_lexicon_tags_6 },
    { 0x3a81b8dd, SUC_lexicon_tags_40 },
    { 0x36a1b8d9, SUC_lexicon_tags_27 },
    { 0xe61b8dc, SUC_lexicon_tags_15 },
    { 0x9d81b8de, SUC_lexicon_tags_31 },
    { 0x5c81b8e1, SUC_lexicon_tags_14 },
    { 0xbce1b8df, SUC_lexicon_tags_15 },
    { 0x2661b8e2, SUC_lexicon_tags_56 },
    { 0xf721b8e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c1b8e6, SUC_lexicon_tags_27 },
    { 0x2601b8e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5421b8e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6861b8f1, SUC_lexicon_tags_39 },
    { 0xe121b8f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe881b8f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2741b8f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc6c1b8f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d81b900, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc1b904, SUC_lexicon_tags_17 },
    { 0x3821b905, SUC_lexicon_tags_1 },
    { 0x7ac1b904, SUC_lexicon_tags_0 },
    { 0xdb41b907, SUC_lexicon_tags_16 },
    { 0xd561b908, SUC_lexicon_tags_26 },
    { 0x8161b907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10c1b90b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e61b90e, SUC_lexicon_tags_12 },
    { 0x6d41b90f, SUC_lexicon_tags_14 },
    { 0xe341b90f, SUC_lexicon_tags_82 },
    { 0xec21b911, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3361b913, SUC_lexicon_tags_32 },
    { 0x6e1b913, SUC_lexicon_tags_6 },
    { 0x8ba1b914, SUC_lexicon_tags_25 },
    { 0x5741b916, SUC_lexicon_tags_28 },
    { 0xdac1b913, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x9321b91a, SUC_lexicon_tags_65 },
    { 0xa541b91b, SUC_lexicon_tags_27 },
    { 0xadc1b91a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa1b921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cc1b923, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd61b930, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb601b932, SUC_lexicon_tags_1 },
    { 0xce61b932, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541b937, SUC_lexicon_tags_2 },
    { 0xbf81b938, SUC_lexicon_tags_23 },
    { 0x7c41b938, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa1b93d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4821b93f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b41b944, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21b94c, SUC_lexicon_tags_11 },
    { 0x6e21b94d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe701b950, SUC_lexicon_tags_28 },
    { 0x1681b951, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb601b953, SUC_lexicon_tags_7 },
    { 0x2861b954, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a1b958, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62e1b95a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1581b95d, SUC_lexicon_tags_17 },
    { 0xddc1b95e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a1b968, SUC_lexicon_tags_27 },
    { 0x42e1b969, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a1b971, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2061b973, SUC_lexicon_tags_28 },
    { 0x1581b974, SUC_lexicon_tags_25 },
    { 0xfb41b975, SUC_lexicon_tags_37 },
    { 0xd0c1b976, SUC_lexicon_tags_17 },
    { 0xabc1b977, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1b97b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x45c1b97d, SUC_lexicon_tags_1 },
    { 0x8a41b97e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d01b982, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba1b986, SUC_lexicon_tags_38 },
    { 0x8441b987, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff41b989, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6221b98c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x92c1b98e, SUC_lexicon_tags_3 },
    { 0x4661b98f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba21b992, SUC_lexicon_tags_15 },
    { 0x7301b993, SUC_lexicon_tags_2 },
    { 0x2321b994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb821b998, SUC_lexicon_tags_60 },
    { 0x79a1b998, SUC_lexicon_tags_46 },
    { 0x74e1b999, SUC_lexicon_tags_25 },
    { 0x1fe1b99b, SUC_lexicon_tags_26 },
    { 0x1041b999, SUC_lexicon_tags_25 },
    { 0xbe41b99b, SUC_lexicon_tags_3 },
    { 0x61c1b99d, SUC_lexicon_tags_26 },
    { 0xb741b99b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a1b9a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6661b9a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e01b9ae, SUC_lexicon_tags_11 },
    { 0x92c1b9af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9901b9b1, SUC_lexicon_tags_19 },
    { 0xa881b9b2, SUC_lexicon_tags_49 },
    { 0x7261b9b3, SUC_lexicon_tags_7 },
    { 0x39e1b9b4, SUC_lexicon_tags_7 },
    { 0xb201b9b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b81b9b9, SUC_lexicon_tags_15 },
    { 0xa601b9ba, SUC_lexicon_tags_4 },
    { 0x5be1b9b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66a1b9bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e1b9c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9701b9c4, SUC_lexicon_tags_11 },
    { 0x4341b9c5, SUC_lexicon_tags_7 },
    { 0x781b9c6, SUC_lexicon_tags_21 },
    { 0xda81b9c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6721b9cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7141b9ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf21b9d0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9841b9d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6001b9d8, SUC_lexicon_tags_2 },
    { 0x8301b9d9, SUC_lexicon_tags_27 },
    { 0xe901b9d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed61b9dd, SUC_lexicon_tags_37 },
    { 0x1a21b9de, SUC_lexicon_tags_33 },
    { 0x6821b9de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21e1b9e1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e1b9e4, SUC_lexicon_tags_19 },
    { 0xd561b9e5, SUC_lexicon_tags_11 },
    { 0x4bc1b9e4, SUC_lexicon_tags_7 },
    { 0xaa01b9e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa981b9e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa1b9ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a41b9f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee1b9f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd461b9f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2141b9fb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a1b9ff, SUC_lexicon_tags_84 },
    { 0x9281ba00, SUC_lexicon_tags_7 },
    { 0x88e1ba01, SUC_lexicon_tags_45 },
    { 0x5cc1ba02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba81ba05, SUC_lexicon_tags_38 },
    { 0x1161ba06, SUC_lexicon_tags_15 },
    { 0xe741ba07, SUC_lexicon_tags_2 },
    { 0x67e1ba08, SUC_lexicon_tags_65 },
    { 0x6441ba06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda1ba0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1721ba10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb81ba19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb21ba1b, SUC_lexicon_tags_12 },
    { 0x9ce1ba1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7221ba1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9461ba22, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x20c1ba24, SUC_lexicon_tags_47 },
    { 0xc1ba24, SUC_lexicon_tags_21 },
    { 0xd4c1ba25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcba1ba28, SUC_lexicon_tags_20 },
    { 0x5fe1ba29, SUC_lexicon_tags_32 },
    { 0x581ba2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99a1ba2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9761ba2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2181ba30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc4a1ba32, SUC_lexicon_tags_3 },
    { 0x6b41ba33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc021ba35, SUC_lexicon_tags_1 },
    { 0x78e1ba36, SUC_lexicon_tags_36 },
    { 0xc3e1ba37, SUC_lexicon_tags_17 },
    { 0xb7c1ba38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c1ba40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa1ba43, SUC_lexicon_tags_2 },
    { 0xb401ba44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdb41ba46, SUC_lexicon_tags_30 },
    { 0x9d81ba47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b01ba4a, SUC_lexicon_tags_37 },
    { 0xe7e1ba4b, SUC_lexicon_tags_37 },
    { 0xe101ba4b, SUC_lexicon_tags_19 },
    { 0xa1e1ba4d, SUC_lexicon_tags_7 },
    { 0x5441ba4e, SUC_lexicon_tags_11 },
    { 0x541ba4d, SUC_lexicon_tags_17 },
    { 0x7801ba4c, SUC_lexicon_tags_31 },
    { 0xe161ba51, SUC_lexicon_tags_49 },
    { 0xdac1ba51, SUC_lexicon_tags_131 },
    { 0x9e81ba53, SUC_lexicon_tags_49 },
    { 0xaac1ba52, SUC_lexicon_tags_21 },
    { 0x1a21ba51, SUC_lexicon_tags_17 },
    { 0x3341ba4b, SUC_lexicon_tags_1 },
    { 0x4ca1ba57, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6681ba59, SUC_lexicon_tags_129 },
    { 0x5f81ba5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8861ba5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee1ba61, SUC_lexicon_tags_12 },
    { 0x5be1ba62, SUC_lexicon_tags_37 },
    { 0xae1ba62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb841ba65, SUC_lexicon_tags_31 },
    { 0x3321ba66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a1ba6b, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd01ba6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c1ba75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd721ba78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc1ba7c, SUC_lexicon_tags_30 },
    { 0x6921ba7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb441ba81, SUC_lexicon_tags_47 },
    { 0x7aa1ba82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf61ba8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf361ba8c, SUC_lexicon_tags_17 },
    { 0x9ba1ba8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf681ba92, SUC_lexicon_tags_11 },
    { 0x1b01ba92, SUC_lexicon_tags_23 },
    { 0xfb61ba94, SUC_lexicon_tags_17 },
    { 0x2361ba93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee1ba99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1c1ba9b, SUC_lexicon_tags_48 },
    { 0x1541ba9c, SUC_lexicon_tags_8 },
    { 0x6721ba9d, SUC_lexicon_tags_2 },
    { 0xfc01ba9d, SUC_lexicon_tags_13 },
    { 0x4701ba9c, SUC_lexicon_tags_17 },
    { 0xfd41baa0, SUC_lexicon_tags_7 },
    { 0xf341baa0, SUC_lexicon_tags_19 },
    { 0xb701baa0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb981baa5, SUC_lexicon_tags_14 },
    { 0x8901baa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc1baaa, SUC_lexicon_tags_23 },
    { 0xe601baab, SUC_lexicon_tags_34 },
    { 0x3b61baaa, SUC_lexicon_tags_17 },
    { 0x10e1baaa, SUC_lexicon_tags_1 },
    { 0xf41baae, SUC_lexicon_tags_15 },
    { 0x1701baaf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8161bab2, SUC_lexicon_tags_37 },
    { 0x1b01bab2, SUC_lexicon_tags_12 },
    { 0xad81bab4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xde61bab6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7601bab9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x2921babb, SUC_lexicon_tags_17 },
    { 0xe461babb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3261bac0, SUC_lexicon_tags_56 },
    { 0x6521bac0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a01bac6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede1bacc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf241bad1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb001bad6, SUC_lexicon_tags_0 },
    { 0xad01bad6, SUC_lexicon_tags_30 },
    { 0xcec1bad8, SUC_lexicon_tags_37 },
    { 0xd141bad9, SUC_lexicon_tags_28 },
    { 0xf1e1bada, SUC_lexicon_tags_60 },
    { 0x20c1bad8, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0xe6e1badd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45c1badf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7501bae1, SUC_lexicon_tags_2 },
    { 0xae01bae2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c1bae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f81bae7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23a1bae9, SUC_lexicon_tags_27 },
    { 0x6561baea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x361baec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6101baf0, SUC_lexicon_tags_31 },
    { 0x21baf0, SUC_lexicon_tags_25 },
    { 0xf4e1baf1, SUC_lexicon_tags_9 },
    { 0x9ee1baf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6561bafb, SUC_lexicon_tags_1 },
    { 0x3401bafb, SUC_lexicon_tags_30 },
    { 0xd1e1bafd, SUC_lexicon_tags_25 },
    { 0x9a81bafe, SUC_lexicon_tags_15 },
    { 0x9b81bafb, SUC_lexicon_tags_12 },
    { 0xc741bafd, SUC_lexicon_tags_49 },
    { 0xfd41bb01, SUC_lexicon_tags_28 },
    { 0x1a1bb02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e1bb05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe001bb0a, SUC_lexicon_tags_15 },
    { 0xb181bb0b, SUC_lexicon_tags_25 },
    { 0x8aa1bb0c, SUC_lexicon_tags_10 },
    { 0xefa1bb0d, SUC_lexicon_tags_57 },
    { 0x60a1bb0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe21bb11, SUC_lexicon_tags_44 },
    { 0xcae1bb12, SUC_lexicon_tags_1 },
    { 0xbea1bb13, SUC_lexicon_tags_14 },
    { 0xfb41bb14, SUC_lexicon_tags_49 },
    { 0xa501bb13, SUC_lexicon_tags_3 },
    { 0xdc1bb11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c41bb1d, SUC_lexicon_tags_43 },
    { 0xa621bb1e, SUC_lexicon_tags_37 },
    { 0xa181bb1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8341bb21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x661bb23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe241bb25, SUC_lexicon_tags_141 },
    { 0xb301bb25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe1bb2c, SUC_lexicon_tags_2 },
    { 0x33a1bb2d, SUC_lexicon_tags_25 },
    { 0xc8a1bb2c, SUC_lexicon_tags_11 },
    { 0xd7c1bb2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7581bb32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a81bb34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xada1bb37, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e1bb3b, SUC_lexicon_tags_37 },
    { 0xcb21bb3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3061bb3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6041bb44, SUC_lexicon_tags_7 },
    { 0xd321bb45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e1bb55, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8fa1bb57, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc61bb5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b41bb66, SUC_lexicon_tags_17 },
    { 0xf2c1bb67, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c1bb6c, SUC_lexicon_tags_28 },
    { 0xdb81bb6d, SUC_lexicon_tags_17 },
    { 0xa2c1bb6e, SUC_lexicon_tags_14 },
    { 0x5601bb6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1bb73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc761bb75, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3181bb78, SUC_lexicon_tags_26 },
    { 0x601bb78, SUC_lexicon_tags_33 },
    { 0x7901bb78, SUC_lexicon_tags_15 },
    { 0x5821bb79, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb961bb7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x43e1bb81, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf901bb87, SUC_lexicon_tags_33 },
    { 0x8d61bb88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9161bb8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4a61bb8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e1bb91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2621bb94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb81bb9a, SUC_lexicon_tags_12 },
    { 0xa781bb9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb981bba0, SUC_lexicon_tags_16 },
    { 0xd261bba1, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa81bba6, SUC_lexicon_tags_15 },
    { 0x8b41bba7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa1bba9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8a1bbab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3a1bbad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe701bbaf, SUC_lexicon_tags_2 },
    { 0xf861bbb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d41bbb4, SUC_lexicon_tags_1 },
    { 0x5001bbb4, SUC_lexicon_tags_15 },
    { 0x2301bbb6, SUC_lexicon_tags_47 },
    { 0x2901bbb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa81bbb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f21bbbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c1bbc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba81bbc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc661bbc7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2421bbcd, SUC_lexicon_tags_46 },
    { 0x8441bbce, SUC_lexicon_tags_43 },
    { 0x1321bbcf, SUC_lexicon_tags_26 },
    { 0x2221bbcf, SUC_lexicon_tags_37 },
    { 0x67c1bbce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1d41bbd3, SUC_lexicon_tags_17 },
    { 0x6361bbd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e21bbd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33a1bbd9, SUC_lexicon_tags_37 },
    { 0x9be1bbda, SUC_lexicon_tags_42 },
    { 0x9f61bbd9, SUC_lexicon_tags_12 },
    { 0x5dc1bbdc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7661bbe2, SUC_lexicon_tags_19 },
    { 0x93a1bbe3, SUC_lexicon_tags_17 },
    { 0x4f21bbe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6361bbee, SUC_lexicon_tags_17 },
    { 0xe641bbef, SUC_lexicon_tags_1 },
    { 0xbc21bbf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6901bbf2, SUC_lexicon_tags_46 },
    { 0xf761bbf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48e1bbf5, SUC_lexicon_tags_6 },
    { 0xdc81bbf6, SUC_lexicon_tags_7 },
    { 0x4de1bbf5, SUC_lexicon_tags_63 },
    { 0x1541bbf8, SUC_lexicon_tags_45 },
    { 0x7ac1bbf9, SUC_lexicon_tags_11 },
    { 0x4c1bbf7, SUC_lexicon_tags_29 },
    { 0x3f41bbfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa241bc01, SUC_lexicon_tags_56 },
    { 0x621bc02, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf661bc04, SUC_lexicon_tags_6 },
    { 0xe61bc05, SUC_lexicon_tags_30 },
    { 0x4701bc05, SUC_lexicon_tags_11 },
    { 0xd701bc07, SUC_lexicon_tags_33 },
    { 0xa2a1bc04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8621bc0c, SUC_lexicon_tags_11 },
    { 0xb7a1bc0d, SUC_lexicon_tags_8 },
    { 0x55c1bc0e, SUC_lexicon_tags_12 },
    { 0x6081bc0d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a1bc16, SUC_lexicon_tags_26 },
    { 0xca21bc17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f01bc1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741bc1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb681bc20, SUC_lexicon_tags_12 },
    { 0x8721bc21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc741bc24, SUC_lexicon_tags_8 },
    { 0xe081bc24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c1bc29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a1bc2b, SUC_lexicon_tags_60 },
    { 0x6dc1bc2c, SUC_lexicon_tags_15 },
    { 0xaee1bc2d, SUC_lexicon_tags_0 },
    { 0x181bc2c, SUC_lexicon_tags_21 },
    { 0x1c81bc2c, SUC_lexicon_tags_0 },
    { 0xa01bc2b, SUC_lexicon_tags_11 },
    { 0x6521bc2d, SUC_lexicon_tags_55 },
    { 0xeb21bc32, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82e1bc34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce41bc36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9481bc3a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc1bc3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfe41bc3f, SUC_lexicon_tags_37 },
    { 0xbd81bc40, SUC_lexicon_tags_15 },
    { 0xb9a1bc41, SUC_lexicon_tags_60 },
    { 0xac1bc42, SUC_lexicon_tags_1 },
    { 0xe01bc43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6881bc46, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa661bc4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e61bc4f, SUC_lexicon_tags_11 },
    { 0x84a1bc50, SUC_lexicon_tags_33 },
    { 0x31a1bc51, SUC_lexicon_tags_17 },
    { 0x7501bc51, SUC_lexicon_tags_25 },
    { 0x3121bc53, SUC_lexicon_tags_15 },
    { 0x2741bc54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a1bc59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf661bc5d, SUC_lexicon_tags_9 },
    { 0x3761bc5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c41bc61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x181bc63, SUC_lexicon_tags_25 },
    { 0x36a1bc64, SUC_lexicon_tags_27 },
    { 0xa721bc63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc1bc68, SUC_lexicon_tags_33 },
    { 0x37e1bc69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc01bc6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8401bc6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3fe1bc70, SUC_lexicon_tags_7 },
    { 0x1041bc71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e1bc76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5881bc78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c1bc7c, SUC_lexicon_tags_0 },
    { 0xcda1bc7d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x4da1bc7f, SUC_lexicon_tags_43 },
    { 0xde1bc7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6541bc84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xad21bc86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1bc8a, SUC_lexicon_tags_0 },
    { 0x39e1bc8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a1bc95, SUC_lexicon_tags_14 },
    { 0x1601bc96, SUC_lexicon_tags_23 },
    { 0xa8a1bc95, SUC_lexicon_tags_31 },
    { 0x9fa1bc98, SUC_lexicon_tags_11 },
    { 0x2881bc99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a1bc9d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c41bca4, SUC_lexicon_tags_30 },
    { 0xb441bca4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab41bca7, SUC_lexicon_tags_14 },
    { 0xbaa1bca8, SUC_lexicon_tags_2 },
    { 0x2ea1bca7, SUC_lexicon_tags_63 },
    { 0xa461bca9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb01bcb6, SUC_lexicon_tags_25 },
    { 0x12a1bcb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee1bcbb, SUC_lexicon_tags_11 },
    { 0x17e1bcbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc321bcbf, SUC_lexicon_tags_47 },
    { 0xd781bcbf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe761bcc4, SUC_lexicon_tags_2 },
    { 0xf9c1bcc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa041bcc7, SUC_lexicon_tags_25 },
    { 0x1cc1bcc8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9581bccb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e41bcd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2921bcd6, SUC_lexicon_tags_60 },
    { 0x77a1bcd7, SUC_lexicon_tags_47 },
    { 0x6fe1bcd6, SUC_lexicon_tags_11 },
    { 0xc1a1bcd9, SUC_lexicon_tags_16 },
    { 0x34e1bcda, SUC_lexicon_tags_45 },
    { 0x2c21bcd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f41bcde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc21bce1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de1bce6, SUC_lexicon_tags_66 },
    { 0x2641bce6, SUC_lexicon_tags_1 },
    { 0x7461bce7, SUC_lexicon_tags_7 },
    { 0x921bce9, SUC_lexicon_tags_17 },
    { 0x3c01bce7, SUC_lexicon_tags_60 },
    { 0x8081bce7, SUC_lexicon_tags_30 },
    { 0xc3c1bce6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c1bcf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee21bcf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d61bcf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x581bcfb, SUC_lexicon_tags_31 },
    { 0xd441bcfc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c1bd03, SUC_lexicon_tags_37 },
    { 0x80e1bd04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbe1bd06, SUC_lexicon_tags_2 },
    { 0x7c21bd06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4001bd09, SUC_lexicon_tags_60 },
    { 0xfa41bd0a, SUC_lexicon_tags_15 },
    { 0xdda1bd0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc81bd10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd601bd13, SUC_lexicon_tags_4 },
    { 0x9c1bd14, SUC_lexicon_tags_37 },
    { 0x9041bd15, SUC_lexicon_tags_12 },
    { 0xcde1bd14, SUC_lexicon_tags_37 },
    { 0x7c01bd15, SUC_lexicon_tags_7 },
    { 0x4801bd18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e1bd1d, SUC_lexicon_tags_7 },
    { 0x4101bd1d, SUC_lexicon_tags_10 },
    { 0x8441bd1f, SUC_lexicon_tags_23 },
    { 0x6b41bd20, SUC_lexicon_tags_16 },
    { 0x1c41bd20, SUC_lexicon_tags_17 },
    { 0x1ac1bd21, SUC_lexicon_tags_15 },
    { 0xd141bd23, SUC_lexicon_tags_17 },
    { 0x2361bd24, SUC_lexicon_tags_204 },
    { 0x18e1bd25, SUC_lexicon_tags_63 },
    { 0x8881bd26, SUC_lexicon_tags_38 },
    { 0x6c41bd27, SUC_lexicon_tags_7 },
    { 0x5361bd27, SUC_lexicon_tags_38 },
    { 0x3201bd27, SUC_lexicon_tags_43 },
    { 0xd421bd23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2241bd2c, SUC_lexicon_tags_37 },
    { 0xba81bd2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x80e1bd2f, SUC_lexicon_tags_0 },
    { 0x7d01bd2f, SUC_lexicon_tags_23 },
    { 0x4a41bd31, SUC_lexicon_tags_27 },
    { 0x1541bd32, SUC_lexicon_tags_17 },
    { 0x22c1bd32, SUC_lexicon_tags_43 },
    { 0xa2c1bd2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf921bd38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa341bd3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b01bd3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6581bd3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4a1bd41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de1bd45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c1bd4a, SUC_lexicon_tags_31 },
    { 0x8e01bd4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e1bd52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e1bd59, SUC_lexicon_tags_15 },
    { 0x1b21bd59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bd5d, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0x1141bd5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7921bd61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e1bd66, SUC_lexicon_tags_10 },
    { 0xb5e1bd67, SUC_lexicon_tags_30 },
    { 0xae21bd67, SUC_lexicon_tags_14 },
    { 0xf0a1bd69, SUC_lexicon_tags_2 },
    { 0x3021bd6a, SUC_lexicon_tags_45 },
    { 0x181bd69, SUC_lexicon_tags_60 },
    { 0xf101bd6a, SUC_lexicon_tags_16 },
    { 0x7961bd6d, SUC_lexicon_tags_54 },
    { 0x1441bd66, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1361bd70, SUC_lexicon_tags_27 },
    { 0x8e41bd71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9361bd74, SUC_lexicon_tags_12 },
    { 0x96e1bd74, SUC_lexicon_tags_24 },
    { 0x55c1bd76, SUC_lexicon_tags_37 },
    { 0xe021bd74, SUC_lexicon_tags_28 },
    { 0x9381bd76, SUC_lexicon_tags_60 },
    { 0x2e01bd75, SUC_lexicon_tags_375 },
    { 0xa961bd74, SUC_lexicon_tags_11 },
    { 0x6fe1bd7b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6381bd83, SUC_lexicon_tags_12 },
    { 0x8861bd84, SUC_lexicon_tags_12 },
    { 0x641bd85, SUC_lexicon_tags_26 },
    { 0x2e61bd85, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x71a1bd88, SUC_lexicon_tags_23 },
    { 0xf3e1bd89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9521bd8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e1bd95, SUC_lexicon_tags_10 },
    { 0x7b21bd96, SUC_lexicon_tags_12 },
    { 0x7b01bd97, SUC_lexicon_tags_37 },
    { 0x29e1bd96, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x681bd9a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc1bda0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bc1bda2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81bdae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdda1bdb0, SUC_lexicon_tags_0 },
    { 0x621bdb1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b01bdb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea61bdbd, SUC_lexicon_tags_23 },
    { 0xfb81bdbd, SUC_lexicon_tags_38 },
    { 0x55e1bdbd, SUC_lexicon_tags_17 },
    { 0xaa01bdc0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7a1bdc2, SUC_lexicon_tags_15 },
    { 0xb741bdc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2421bdc6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x70e1bdc8, SUC_lexicon_tags_11 },
    { 0xd3a1bdc9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c41bdd0, SUC_lexicon_tags_2 },
    { 0x7281bdd1, SUC_lexicon_tags_17 },
    { 0xf341bdd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa81bdd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed01bdd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e1bdde, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781bde9, SUC_lexicon_tags_15 },
    { 0x1e61bdea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b81bded, SUC_lexicon_tags_17 },
    { 0x9da1bdee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a1bdf2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81bdf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3161bdfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa61bdfd, SUC_lexicon_tags_14 },
    { 0x7401bdfe, SUC_lexicon_tags_32 },
    { 0x74a1bdff, SUC_lexicon_tags_17 },
    { 0xe01bdff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3761be07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5581be0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c61be0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0c1be10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a1be14, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee1be17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1181be1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c81be1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e1be20, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c1be2a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbf01be2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4281be2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e1be33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1341be3a, SUC_lexicon_tags_32 },
    { 0x20c1be3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a1be44, SUC_lexicon_tags_33 },
    { 0xc8a1be45, SUC_lexicon_tags_47 },
    { 0xf7c1be46, SUC_lexicon_tags_9 },
    { 0x72c1be47, SUC_lexicon_tags_30 },
    { 0xfc41be47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5841be4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb641be4f, SUC_lexicon_tags_30 },
    { 0xec81be4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd01be54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66a1be56, SUC_lexicon_tags_28 },
    { 0x6fa1be57, SUC_lexicon_tags_28 },
    { 0x901be56, SUC_lexicon_tags_63 },
    { 0xade1be59, SUC_lexicon_tags_87 },
    { 0x4ea1be59, SUC_lexicon_tags_12 },
    { 0x5941be59, SUC_lexicon_tags_33 },
    { 0xeac1be5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a1be61, SUC_lexicon_tags_0 },
    { 0xeea1be62, SUC_lexicon_tags_25 },
    { 0x9241be63, SUC_lexicon_tags_27 },
    { 0xfee1be62, SUC_lexicon_tags_1 },
    { 0xb61be63, SUC_lexicon_tags_12 },
    { 0x27e1be66, SUC_lexicon_tags_21 },
    { 0xe281be67, SUC_lexicon_tags_14 },
    { 0x921be68, SUC_lexicon_tags_12 },
    { 0x8ce1be64, SUC_lexicon_tags_65 },
    { 0x741be6a, SUC_lexicon_tags_37 },
    { 0x70a1be6b, SUC_lexicon_tags_1 },
    { 0xf361be6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e1be74, SUC_lexicon_tags_19 },
    { 0x5321be75, SUC_lexicon_tags_19 },
    { 0xb3a1be75, SUC_lexicon_tags_30 },
    { 0xb801be77, SUC_lexicon_tags_3 },
    { 0xa001be78, SUC_lexicon_tags_0 },
    { 0x5061be75, SUC_lexicon_tags_1 },
    { 0x2161be7a, SUC_lexicon_tags_32 },
    { 0xde01be7b, SUC_lexicon_tags_23 },
    { 0xcd21be77, SUC_lexicon_tags_31 },
    { 0x7f81be7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1541be81, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc041be85, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xc8e1be87, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e1be8c, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xc901be8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e1be92, SUC_lexicon_tags_2 },
    { 0x43e1be93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f61be95, SUC_lexicon_tags_17 },
    { 0xfc41be96, SUC_lexicon_tags_37 },
    { 0x6d81be96, SUC_lexicon_tags_19 },
    { 0xf141be96, SUC_lexicon_tags_11 },
    { 0xf7c1be99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2221be9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de1bea2, SUC_lexicon_tags_2 },
    { 0xa6a1bea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec1bea7, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a1beab, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa421beae, SUC_lexicon_tags_60 },
    { 0x7d61beaf, SUC_lexicon_tags_0 },
    { 0xf9c1beaf, SUC_lexicon_tags_15 },
    { 0x7181beaf, SUC_lexicon_tags_15 },
    { 0xbfe1beae, SUC_lexicon_tags_62 },
    { 0xf8c1beb3, SUC_lexicon_tags_25 },
    { 0x7fe1beb4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe1bebd, SUC_lexicon_tags_25 },
    { 0xdb21bebe, SUC_lexicon_tags_9 },
    { 0x1ee1bebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc1bece, SUC_lexicon_tags_15 },
    { 0x8601becf, SUC_lexicon_tags_7 },
    { 0x7c61bed0, SUC_lexicon_tags_11 },
    { 0x3361becf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8561bed5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c1bed8, SUC_lexicon_tags_62 },
    { 0x37c1bed9, SUC_lexicon_tags_1 },
    { 0xa01beda, SUC_lexicon_tags_37 },
    { 0x2a61bed9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1941bedd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2241bee8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7341beeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01bef0, SUC_lexicon_tags_45 },
    { 0xfc01bef1, SUC_lexicon_tags_61 },
    { 0xe5a1bef2, SUC_lexicon_tags_4 },
    { 0x45e1bef1, SUC_lexicon_tags_25 },
    { 0x8f41bef0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1621bef6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61bf00, SUC_lexicon_tags_28 },
    { 0x5cc1bf01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8081bf04, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb61bf09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd021bf0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaec1bf0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f01bf10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a1bf14, SUC_lexicon_tags_174 },
    { 0xef01bf15, SUC_lexicon_tags_87 },
    { 0xdb21bf15, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e1bf1d, SUC_lexicon_tags_63 },
    { 0xbf21bf1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b21bf20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2141bf23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481bf28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc721bf2b, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec61bf31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e1bf34, SUC_lexicon_tags_5 },
    { 0x4901bf34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93e1bf37, SUC_lexicon_tags_11 },
    { 0x681bf38, SUC_lexicon_tags_2 },
    { 0xa321bf39, SUC_lexicon_tags_14 },
    { 0x45e1bf3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28e1bf3c, SUC_lexicon_tags_63 },
    { 0x25a1bf3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1d21bf3f, SUC_lexicon_tags_69 },
    { 0x66a1bf40, SUC_lexicon_tags_37 },
    { 0xe401bf41, SUC_lexicon_tags_37 },
    { 0x16c1bf42, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc361bf48, SUC_lexicon_tags_158 },
    { 0x7ea1bf49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70e1bf4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5341bf50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36c1bf52, SUC_lexicon_tags_1 },
    { 0x4fa1bf53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3721bf56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e1bf59, SUC_lexicon_tags_63 },
    { 0xa121bf5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf301bf5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb361bf63, SUC_lexicon_tags_7 },
    { 0x6661bf64, SUC_lexicon_tags_7 },
    { 0x7a61bf63, SUC_lexicon_tags_0 },
    { 0x8a41bf66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e1bf6a, SUC_lexicon_tags_11 },
    { 0x621bf6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f81bf6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2861bf71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1021bf73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f81bf79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b21bf7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a1bf7e, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea1bf81, SUC_lexicon_tags_37 },
    { 0x95e1bf81, SUC_lexicon_tags_33 },
    { 0x4e41bf83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1841bf86, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d61bf89, SUC_lexicon_tags_15 },
    { 0x7dc1bf8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae1bf91, SUC_lexicon_tags_17 },
    { 0xaba1bf92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc341bf98, SUC_lexicon_tags_7 },
    { 0xdd21bf99, SUC_lexicon_tags_1 },
    { 0x4361bf9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c1bfa0, SUC_lexicon_tags_12 },
    { 0xdfc1bfa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab41bfa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb561bfa8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5fc1bfaa, SUC_lexicon_tags_25 },
    { 0x63e1bfab, SUC_lexicon_tags_45 },
    { 0x63c1bfab, SUC_lexicon_tags_60 },
    { 0xeee1bfad, SUC_lexicon_tags_43 },
    { 0x7d41bfad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1a1bfb0, SUC_lexicon_tags_2 },
    { 0x10a1bfb1, SUC_lexicon_tags_45 },
    { 0xc3c1bfb1, SUC_lexicon_tags_16 },
    { 0x8b01bfb3, SUC_lexicon_tags_7 },
    { 0xde1bfb0, SUC_lexicon_tags_47 },
    { 0x76c1bfb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c1bfb9, SUC_lexicon_tags_2 },
    { 0xe181bfba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e1bfc0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd641bfc5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc1a1bfc7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a1bfcb, SUC_lexicon_tags_0 },
    { 0x1f01bfcb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfea1bfce, SUC_lexicon_tags_38 },
    { 0xcf61bfcf, SUC_lexicon_tags_66 },
    { 0x9561bfd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3aa1bfd2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2441bfd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84a1bfd6, SUC_lexicon_tags_38 },
    { 0x7201bfd7, SUC_lexicon_tags_7 },
    { 0x8161bfd8, SUC_lexicon_tags_14 },
    { 0x32c1bfd9, SUC_lexicon_tags_11 },
    { 0x5221bfd8, SUC_lexicon_tags_262 },
    { 0xebc1bfdb, SUC_lexicon_tags_40 },
    { 0xb61bfda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41bfe2, SUC_lexicon_tags_47 },
    { 0x5721bfe3, SUC_lexicon_tags_12 },
    { 0x3fe1bfe4, SUC_lexicon_tags_17 },
    { 0xe1e1bfe3, SUC_lexicon_tags_1 },
    { 0x9d21bfe6, SUC_lexicon_tags_2 },
    { 0x1da1bfe7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7041bfe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1bfed, SUC_lexicon_tags_15 },
    { 0x1661bfed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c21bff2, SUC_lexicon_tags_45 },
    { 0xed01bff3, SUC_lexicon_tags_10 },
    { 0x12c1bff4, SUC_lexicon_tags_25 },
    { 0x33a1bff5, SUC_lexicon_tags_12 },
    { 0x7cc1bff5, SUC_lexicon_tags_17 },
    { 0x36c1bff3, SUC_lexicon_tags_11 },
    { 0x4d21bff6, SUC_lexicon_tags_47 },
    { 0x22e1bff9, SUC_lexicon_tags_12 },
    { 0x52a1bffa, SUC_lexicon_tags_2 },
    { 0x3f01bffb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01bfff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e1c002, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa261c00b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5741c00d, SUC_lexicon_tags_14 },
    { 0x28a1c00d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea1c011, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd81c014, SUC_lexicon_tags_25 },
    { 0xbe1c014, SUC_lexicon_tags_43 },
    { 0x3921c015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0c1c018, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c1c01c, SUC_lexicon_tags_49 },
    { 0x21e1c01d, SUC_lexicon_tags_25 },
    { 0xd041c01e, SUC_lexicon_tags_60 },
    { 0x3c61c01f, SUC_lexicon_tags_19 },
    { 0x4881c01e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f61c02b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a1c030, SUC_lexicon_tags_23 },
    { 0x8e61c031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca1c038, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab21c03b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7381c03e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd421c047, SUC_lexicon_tags_1 },
    { 0x6c1c048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a1c04f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3341c051, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5081c05a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e1c05e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa1c062, SUC_lexicon_tags_25 },
    { 0xe241c063, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8101c069, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57a1c06b, SUC_lexicon_tags_15 },
    { 0x7501c06b, SUC_lexicon_tags_33 },
    { 0x3d41c06d, SUC_lexicon_tags_28 },
    { 0xbf01c06d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1c072, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2941c074, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e1c077, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe861c07b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab41c07e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e1c083, SUC_lexicon_tags_34 },
    { 0xe6a1c083, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4761c087, SUC_lexicon_tags_32 },
    { 0xf101c088, SUC_lexicon_tags_62 },
    { 0xac61c087, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa621c08e, SUC_lexicon_tags_12 },
    { 0x4081c08f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xecc1c091, SUC_lexicon_tags_17 },
    { 0x9c01c091, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe261c0a4, SUC_lexicon_tags_25 },
    { 0x1e1c0a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9881c0ad, SUC_lexicon_tags_15 },
    { 0x5f01c0ae, SUC_lexicon_tags_39 },
    { 0x1781c0af, SUC_lexicon_tags_37 },
    { 0x6801c0af, SUC_lexicon_tags_7 },
    { 0x8bc1c0af, SUC_lexicon_tags_43 },
    { 0xb921c0ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701c0b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc821c0c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf01c0c6, SUC_lexicon_tags_17 },
    { 0x43a1c0c7, SUC_lexicon_tags_16 },
    { 0x5ce1c0c6, SUC_lexicon_tags_37 },
    { 0x4b41c0c9, SUC_lexicon_tags_34 },
    { 0xaba1c0ca, SUC_lexicon_tags_2 },
    { 0x8c01c0ca, SUC_lexicon_tags_37 },
    { 0xf661c0cb, SUC_lexicon_tags_107 },
    { 0x7e01c0c8, SUC_lexicon_tags_62 },
    { 0x1d01c0ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc1c0d1, SUC_lexicon_tags_7 },
    { 0x3ec1c0d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa6a1c0d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8901c0d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4441c0db, SUC_lexicon_tags_11 },
    { 0xfc01c0db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301c0e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5421c0e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1981c0e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xca21c0e9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2041c0f0, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x1ac1c0f2, SUC_lexicon_tags_25 },
    { 0xc601c0f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a21c0f7, SUC_lexicon_tags_11 },
    { 0xf2c1c0f8, SUC_lexicon_tags_0 },
    { 0x5061c0f7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd41c0fc, SUC_lexicon_tags_43 },
    { 0x26e1c0fd, SUC_lexicon_tags_25 },
    { 0x7a61c0fd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd021c101, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa281c104, SUC_lexicon_tags_38 },
    { 0xb1a1c104, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf041c108, SUC_lexicon_tags_31 },
    { 0x6ae1c109, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc801c10b, SUC_lexicon_tags_56 },
    { 0xbba1c10c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe441c112, SUC_lexicon_tags_23 },
    { 0xd021c112, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc761c115, SUC_lexicon_tags_30 },
    { 0x33a1c116, SUC_lexicon_tags_38 },
    { 0x2ca1c117, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a1c11a, SUC_lexicon_tags_47 },
    { 0xb361c11b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2381c11e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc1c120, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4061c122, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a1c125, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf861c129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c1c12f, SUC_lexicon_tags_20 },
    { 0x95c1c12f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3241c135, SUC_lexicon_tags_40 },
    { 0xbca1c136, SUC_lexicon_tags_65 },
    { 0x2001c136, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3be1c139, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a21c140, SUC_lexicon_tags_140 },
    { 0x14e1c141, SUC_lexicon_tags_25 },
    { 0xe161c141, SUC_lexicon_tags_53 },
    { 0xd961c143, SUC_lexicon_tags_71 },
    { 0xf081c141, SUC_lexicon_tags_37 },
    { 0x1ee1c145, SUC_lexicon_tags_39 },
    { 0x6d61c145, SUC_lexicon_tags_38 },
    { 0xc8a1c147, SUC_lexicon_tags_68 },
    { 0x7e01c148, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd01c14b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb01c14f, SUC_lexicon_tags_25 },
    { 0xfe1c150, SUC_lexicon_tags_1 },
    { 0xc8e1c151, SUC_lexicon_tags_33 },
    { 0x7e21c152, SUC_lexicon_tags_39 },
    { 0x8761c152, SUC_lexicon_tags_2 },
    { 0xade1c151, SUC_lexicon_tags_15 },
    { 0xcc81c155, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd81c15f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae1c163, SUC_lexicon_tags_41 },
    { 0x3521c164, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8301c166, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x561c169, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b41c16f, SUC_lexicon_tags_4 },
    { 0x3741c170, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a1c173, SUC_lexicon_tags_15 },
    { 0x3841c173, SUC_lexicon_tags_8 },
    { 0x6b01c175, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x6701c177, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xfdc1c179, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x801c17c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb001c17f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9301c184, SUC_lexicon_tags_45 },
    { 0x3761c184, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a1c189, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4021c18b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01c18f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xde21c191, SUC_lexicon_tags_11 },
    { 0x5041c191, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3301c194, SUC_lexicon_tags_14 },
    { 0x71c1c195, SUC_lexicon_tags_12 },
    { 0x9b41c194, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x89c1c198, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7601c1a0, SUC_lexicon_tags_27 },
    { 0x1021c1a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd81c1a3, SUC_lexicon_tags_45 },
    { 0xeb61c1a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c1c1aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba01c1b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da1c1b4, SUC_lexicon_tags_2 },
    { 0x8a1c1b5, SUC_lexicon_tags_37 },
    { 0xb081c1b6, SUC_lexicon_tags_43 },
    { 0x8801c1b7, SUC_lexicon_tags_1 },
    { 0x48e1c1b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac1c1bc, SUC_lexicon_tags_15 },
    { 0xec41c1bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4ae1c1bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xad81c1c1, SUC_lexicon_tags_10 },
    { 0x7ec1c1c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc1c1c6, SUC_lexicon_tags_70 },
    { 0x5e61c1c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe81c1c9, SUC_lexicon_tags_61 },
    { 0x73e1c1ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb181c1ce, SUC_lexicon_tags_15 },
    { 0xcec1c1cf, SUC_lexicon_tags_1 },
    { 0xe381c1d0, SUC_lexicon_tags_0 },
    { 0x4d21c1d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a21c1d7, SUC_lexicon_tags_60 },
    { 0x6321c1d8, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xe401c1db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6241c1dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a21c1df, SUC_lexicon_tags_9 },
    { 0x24e1c1df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c1c1e4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b81c1ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb061c1ee, SUC_lexicon_tags_11 },
    { 0x4f21c1ef, SUC_lexicon_tags_23 },
    { 0x1761c1f0, SUC_lexicon_tags_37 },
    { 0x72e1c1ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e1c1f6, SUC_lexicon_tags_10 },
    { 0x1da1c1f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5521c1fc, SUC_lexicon_tags_47 },
    { 0x97c1c1fd, SUC_lexicon_tags_66 },
    { 0x9101c1fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7a1c200, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee81c204, SUC_lexicon_tags_6 },
    { 0x2581c205, SUC_lexicon_tags_14 },
    { 0x6b81c206, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc881c212, SUC_lexicon_tags_82 },
    { 0x61e1c213, SUC_lexicon_tags_13 },
    { 0x7041c213, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4481c219, SUC_lexicon_tags_127 },
    { 0x3fa1c219, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f61c21e, SUC_lexicon_tags_66 },
    { 0x4501c21f, SUC_lexicon_tags_11 },
    { 0xd981c21e, SUC_lexicon_tags_28 },
    { 0x4c81c220, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xea81c223, SUC_lexicon_tags_15 },
    { 0x6161c224, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1c227, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e1c22b, SUC_lexicon_tags_27 },
    { 0x20e1c22c, SUC_lexicon_tags_551 },
    { 0x7f81c22c, SUC_lexicon_tags_0 },
    { 0xf6e1c22e, SUC_lexicon_tags_56 },
    { 0x6c81c22f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb01c231, SUC_lexicon_tags_15 },
    { 0xa6a1c231, SUC_lexicon_tags_15 },
    { 0x5881c233, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea1c23a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a1c23d, SUC_lexicon_tags_16 },
    { 0xf5c1c23e, SUC_lexicon_tags_17 },
    { 0x2a1c23f, SUC_lexicon_tags_45 },
    { 0x941c240, SUC_lexicon_tags_12 },
    { 0x8ec1c241, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe981c244, SUC_lexicon_tags_38 },
    { 0x2321c244, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf21c247, SUC_lexicon_tags_65 },
    { 0x6d81c248, SUC_lexicon_tags_69 },
    { 0x7fe1c249, SUC_lexicon_tags_63 },
    { 0xf0e1c248, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6421c24d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa1c25b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b41c262, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5a21c264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc301c272, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda61c274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1201c27d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee21c281, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf201c284, SUC_lexicon_tags_0 },
    { 0x48c1c284, SUC_lexicon_tags_2 },
    { 0xdda1c286, SUC_lexicon_tags_17 },
    { 0xc5c1c285, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac1c28b, SUC_lexicon_tags_12 },
    { 0xee81c28b, SUC_lexicon_tags_4 },
    { 0x48a1c28d, SUC_lexicon_tags_7 },
    { 0x6661c28e, SUC_lexicon_tags_45 },
    { 0x54a1c28f, SUC_lexicon_tags_45 },
    { 0x4ec1c290, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e61c294, SUC_lexicon_tags_12 },
    { 0x9901c294, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x38c1c297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9c1c299, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7441c29b, SUC_lexicon_tags_12 },
    { 0x9c21c29c, SUC_lexicon_tags_37 },
    { 0x7061c29c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9261c2a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb861c2a5, SUC_lexicon_tags_21 },
    { 0x40a1c2a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc81c2a8, SUC_lexicon_tags_37 },
    { 0xd7e1c2a8, SUC_lexicon_tags_12 },
    { 0x52e1c2aa, SUC_lexicon_tags_1 },
    { 0xaca1c2aa, SUC_lexicon_tags_68 },
    { 0x1001c2ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9681c2b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa681c2b3, SUC_lexicon_tags_11 },
    { 0x27a1c2b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c21c2b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3561c2bd, SUC_lexicon_tags_12 },
    { 0x6c81c2be, SUC_lexicon_tags_17 },
    { 0x981c2bf, SUC_lexicon_tags_46 },
    { 0x281c2c0, SUC_lexicon_tags_63 },
    { 0x66c1c2c1, SUC_lexicon_tags_41 },
    { 0x3181c2c2, SUC_lexicon_tags_25 },
    { 0xc201c2c3, SUC_lexicon_tags_49 },
    { 0x9d81c2c2, SUC_lexicon_tags_113 },
    { 0xe9e1c2c0, SUC_lexicon_tags_151 },
    { 0xe081c2c0, SUC_lexicon_tags_37 },
    { 0x5b21c2c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe81c2cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7de1c2ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6801c2d0, SUC_lexicon_tags_15 },
    { 0x9281c2d1, SUC_lexicon_tags_37 },
    { 0x1121c2d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7181c2d4, SUC_lexicon_tags_74 },
    { 0x2ca1c2d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e41c2d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe1c2db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6501c2e1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6041c2e3, SUC_lexicon_tags_2 },
    { 0x4201c2e4, SUC_lexicon_tags_17 },
    { 0x2001c2e5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc381c2e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd081c2f1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a81c2f4, SUC_lexicon_tags_17 },
    { 0xf3a1c2f5, SUC_lexicon_tags_129 },
    { 0x8981c2f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec1c2fa, SUC_lexicon_tags_15 },
    { 0x3f61c2fb, SUC_lexicon_tags_47 },
    { 0xb8e1c2fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f61c2fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3221c300, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2421c304, SUC_lexicon_tags_25 },
    { 0x3581c304, SUC_lexicon_tags_25 },
    { 0x681c306, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e81c308, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1521c30a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3481c31e, SUC_lexicon_tags_48 },
    { 0x6fe1c31f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7061c321, SUC_lexicon_tags_17 },
    { 0xe0e1c322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5601c324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2041c326, SUC_lexicon_tags_25 },
    { 0x7ba1c327, SUC_lexicon_tags_37 },
    { 0xace1c327, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2261c32d, SUC_lexicon_tags_79 },
    { 0x4a21c32e, SUC_lexicon_tags_47 },
    { 0x5ae1c32f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b21c332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a1c336, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2f61c338, SUC_lexicon_tags_1 },
    { 0x7921c338, SUC_lexicon_tags_12 },
    { 0x3ea1c33a, SUC_lexicon_tags_65 },
    { 0xc601c33b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a1c33e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdaa1c340, SUC_lexicon_tags_39 },
    { 0xcf81c341, SUC_lexicon_tags_37 },
    { 0x35a1c342, SUC_lexicon_tags_45 },
    { 0x3421c340, SUC_lexicon_tags_37 },
    { 0xa9a1c344, SUC_lexicon_tags_21 },
    { 0x6561c345, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd3c1c347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc921c34f, SUC_lexicon_tags_37 },
    { 0x7c01c34f, SUC_lexicon_tags_29 },
    { 0xa861c350, SUC_lexicon_tags_7 },
    { 0x7381c34f, SUC_lexicon_tags_740 },
    { 0x1e01c353, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x841c355, SUC_lexicon_tags_14 },
    { 0x4c61c356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1701c359, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4641c35b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a81c35e, SUC_lexicon_tags_12 },
    { 0xffa1c35f, SUC_lexicon_tags_1 },
    { 0x7e81c360, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5521c362, SUC_lexicon_tags_17 },
    { 0x53e1c363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe941c368, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3c1c36a, SUC_lexicon_tags_60 },
    { 0x91c1c36b, SUC_lexicon_tags_17 },
    { 0x321c36b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a41c36f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e01c371, SUC_lexicon_tags_37 },
    { 0x4b21c372, SUC_lexicon_tags_1 },
    { 0xa621c373, SUC_lexicon_tags_2 },
    { 0xc601c374, SUC_lexicon_tags_25 },
    { 0x2fe1c371, SUC_lexicon_tags_1 },
    { 0xc321c376, SUC_lexicon_tags_25 },
    { 0x7721c371, SUC_lexicon_tags_23 },
    { 0xf01c378, SUC_lexicon_tags_12 },
    { 0x5e61c379, SUC_lexicon_tags_25 },
    { 0xb361c373, SUC_lexicon_tags_37 },
    { 0xa3a1c376, SUC_lexicon_tags_8 },
    { 0x8ac1c37c, SUC_lexicon_tags_8 },
    { 0x59e1c37d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc1c380, SUC_lexicon_tags_37 },
    { 0x5781c380, SUC_lexicon_tags_37 },
    { 0x5301c382, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e41c389, SUC_lexicon_tags_19 },
    { 0x8a81c38a, SUC_lexicon_tags_17 },
    { 0x1b81c38b, SUC_lexicon_tags_63 },
    { 0x9121c38c, SUC_lexicon_tags_27 },
    { 0x9281c38c, SUC_lexicon_tags_33 },
    { 0x87c1c38c, SUC_lexicon_tags_17 },
    { 0x2861c38f, SUC_lexicon_tags_37 },
    { 0x8cc1c38d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x9341c393, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c1c396, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa921c3a0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa521c3a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda61c3a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9fc1c3aa, SUC_lexicon_tags_61 },
    { 0x9981c3ab, SUC_lexicon_tags_25 },
    { 0xd41c3ab, SUC_lexicon_tags_1427 },
    { 0x4021c3aa, SUC_lexicon_tags_16 },
    { 0x3721c3ab, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xb501c3b0, SUC_lexicon_tags_47 },
    { 0x3061c3b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8021c3b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd181c3ba, SUC_lexicon_tags_82 },
    { 0x9ce1c3bb, SUC_lexicon_tags_77 },
    { 0x53c1c3bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d41c3be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa341c3c0, SUC_lexicon_tags_47 },
    { 0x1ea1c3c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1c3c4, SUC_lexicon_tags_32 },
    { 0x2d21c3c5, SUC_lexicon_tags_31 },
    { 0xde21c3c4, SUC_lexicon_tags_14 },
    { 0x1fa1c3c7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x461c3ca, SUC_lexicon_tags_14 },
    { 0x2aa1c3cb, SUC_lexicon_tags_0 },
    { 0x1921c3cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa641c3ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf041c3d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c1c3d5, SUC_lexicon_tags_37 },
    { 0xc4c1c3d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c1c3db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x68c1c3dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1321c3df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc1c3e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafc1c3e8, SUC_lexicon_tags_39 },
    { 0xa101c3e8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7101c3ef, SUC_lexicon_tags_14 },
    { 0xe081c3f0, SUC_lexicon_tags_6 },
    { 0xc7c1c3f1, SUC_lexicon_tags_38 },
    { 0x1a01c3f2, SUC_lexicon_tags_7 },
    { 0x3981c3f3, SUC_lexicon_tags_37 },
    { 0x7d61c3f4, SUC_lexicon_tags_23 },
    { 0xb4e1c3f3, SUC_lexicon_tags_19 },
    { 0x5c41c3f6, SUC_lexicon_tags_17 },
    { 0xb7a1c3ef, SUC_lexicon_tags_10 },
    { 0x361c3f2, SUC_lexicon_tags_11 },
    { 0xa2e1c3f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe1c400, SUC_lexicon_tags_1167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a1c405, SUC_lexicon_tags_12 },
    { 0xf501c406, SUC_lexicon_tags_63 },
    { 0xc161c405, SUC_lexicon_tags_8 },
    { 0x8261c408, SUC_lexicon_tags_37 },
    { 0xbc61c406, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae1c418, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5da1c41a, SUC_lexicon_tags_63 },
    { 0x9201c41b, SUC_lexicon_tags_37 },
    { 0x281c41c, SUC_lexicon_tags_37 },
    { 0x8761c41b, SUC_lexicon_tags_19 },
    { 0x9aa1c41e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d21c420, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb701c422, SUC_lexicon_tags_19 },
    { 0x4a01c423, SUC_lexicon_tags_14 },
    { 0x9761c424, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a1c429, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd441c42b, SUC_lexicon_tags_37 },
    { 0x3201c42c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5fc1c42e, SUC_lexicon_tags_85 },
    { 0xa221c42f, SUC_lexicon_tags_43 },
    { 0x8ca1c42f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd701c436, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x141c439, SUC_lexicon_tags_17 },
    { 0x4a1c439, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2341c43d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac1c441, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9041c444, SUC_lexicon_tags_65 },
    { 0x77a1c444, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7561c44a, SUC_lexicon_tags_11 },
    { 0xc801c44a, SUC_lexicon_tags_27 },
    { 0xaae1c44c, SUC_lexicon_tags_57 },
    { 0x3921c44d, SUC_lexicon_tags_15 },
    { 0xd901c44c, SUC_lexicon_tags_23 },
    { 0xe3a1c44f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd001c452, SUC_lexicon_tags_6 },
    { 0x85c1c453, SUC_lexicon_tags_24 },
    { 0x2361c452, SUC_lexicon_tags_2 },
    { 0x6c41c455, SUC_lexicon_tags_31 },
    { 0x2a1c454, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd601c458, SUC_lexicon_tags_37 },
    { 0x3581c459, SUC_lexicon_tags_33 },
    { 0x7fe1c45a, SUC_lexicon_tags_17 },
    { 0x5921c45a, SUC_lexicon_tags_49 },
    { 0xf641c45c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce1c45e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6881c464, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea01c46a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe1c475, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x721c477, SUC_lexicon_tags_38 },
    { 0xca21c478, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41a1c47a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5281c481, SUC_lexicon_tags_12 },
    { 0xf781c482, SUC_lexicon_tags_277 },
    { 0x6061c483, SUC_lexicon_tags_31 },
    { 0xf5c1c481, SUC_lexicon_tags_37 },
    { 0x7f41c482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc81c488, SUC_lexicon_tags_7 },
    { 0x8621c489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e1c48e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6c1c490, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe581c492, SUC_lexicon_tags_1 },
    { 0xeee1c493, SUC_lexicon_tags_26 },
    { 0x6741c494, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd561c496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1161c49a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8941c4a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a21c4a2, SUC_lexicon_tags_33 },
    { 0xba41c4a3, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc1c4a6, SUC_lexicon_tags_0 },
    { 0x4c21c4a6, SUC_lexicon_tags_25 },
    { 0xb341c4a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1541c4b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee1c4bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa961c4be, SUC_lexicon_tags_57 },
    { 0x8181c4be, SUC_lexicon_tags_7 },
    { 0x9ae1c4be, SUC_lexicon_tags_35 },
    { 0xeb01c4c1, SUC_lexicon_tags_37 },
    { 0x9c61c4c2, SUC_lexicon_tags_25 },
    { 0x43a1c4c0, SUC_lexicon_tags_11 },
    { 0x34e1c4c4, SUC_lexicon_tags_2 },
    { 0x4b01c4c3, SUC_lexicon_tags_17 },
    { 0xea41c4c4, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0xeaa1c4c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe1c4cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf661c4ce, SUC_lexicon_tags_15 },
    { 0x4bc1c4cf, SUC_lexicon_tags_63 },
    { 0x2361c4d0, SUC_lexicon_tags_11 },
    { 0xbdc1c4d1, SUC_lexicon_tags_25 },
    { 0x6161c4d2, SUC_lexicon_tags_27 },
    { 0xd81c4d1, SUC_lexicon_tags_4 },
    { 0x6a61c4d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3881c4d6, SUC_lexicon_tags_15 },
    { 0x481c4d7, SUC_lexicon_tags_63 },
    { 0x8e21c4d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa1c4db, SUC_lexicon_tags_60 },
    { 0xa0a1c4db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4961c4de, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb701c4e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89c1c4e2, SUC_lexicon_tags_17 },
    { 0xf281c4e3, SUC_lexicon_tags_15 },
    { 0xaf61c4e4, SUC_lexicon_tags_38 },
    { 0x7861c4e4, SUC_lexicon_tags_2 },
    { 0x9961c4e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb961c4ec, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x7041c4ee, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xaee1c4f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f41c4f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e1c4fc, SUC_lexicon_tags_2 },
    { 0xf421c4fc, SUC_lexicon_tags_63 },
    { 0x9b01c4fe, SUC_lexicon_tags_2 },
    { 0x2941c4fe, SUC_lexicon_tags_11 },
    { 0x5681c4ff, SUC_lexicon_tags_1 },
    { 0x51e1c500, SUC_lexicon_tags_25 },
    { 0xd761c502, SUC_lexicon_tags_19 },
    { 0x8ec1c500, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b81c507, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3a1c509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c1c50c, SUC_lexicon_tags_7 },
    { 0x641c50c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5901c511, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf21c513, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2301c519, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d21c51c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe121c520, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6e81c522, SUC_lexicon_tags_2 },
    { 0x8501c523, SUC_lexicon_tags_15 },
    { 0x6d01c524, SUC_lexicon_tags_6 },
    { 0xafc1c525, SUC_lexicon_tags_19 },
    { 0xe541c526, SUC_lexicon_tags_70 },
    { 0xfd61c526, SUC_lexicon_tags_17 },
    { 0xe521c526, SUC_lexicon_tags_4 },
    { 0xfce1c528, SUC_lexicon_tags_2 },
    { 0x9901c525, SUC_lexicon_tags_33 },
    { 0xa2e1c52b, SUC_lexicon_tags_9 },
    { 0xfec1c52c, SUC_lexicon_tags_17 },
    { 0xe9e1c529, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7441c530, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb21c532, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe21c535, SUC_lexicon_tags_11 },
    { 0x6f01c536, SUC_lexicon_tags_14 },
    { 0x381c537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c1c53b, SUC_lexicon_tags_7 },
    { 0x7841c53c, SUC_lexicon_tags_1 },
    { 0xf161c53d, SUC_lexicon_tags_29 },
    { 0x56c1c53d, SUC_lexicon_tags_29 },
    { 0x61a1c53f, SUC_lexicon_tags_37 },
    { 0x3541c53e, SUC_lexicon_tags_17 },
    { 0x3641c53c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1a1c543, SUC_lexicon_tags_47 },
    { 0xdc21c544, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc1c548, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e1c54d, SUC_lexicon_tags_15 },
    { 0xb9a1c54d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa21c550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a1c553, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc1c556, SUC_lexicon_tags_63 },
    { 0x3301c557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c1c55a, SUC_lexicon_tags_37 },
    { 0xbd01c55b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf901c563, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a1c567, SUC_lexicon_tags_27 },
    { 0x6ea1c568, SUC_lexicon_tags_15 },
    { 0x74a1c567, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a1c56d, SUC_lexicon_tags_23 },
    { 0xecc1c56e, SUC_lexicon_tags_75 },
    { 0xd461c56e, SUC_lexicon_tags_15 },
    { 0xd921c570, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd481c576, SUC_lexicon_tags_15 },
    { 0x5ee1c577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c1c57a, SUC_lexicon_tags_27 },
    { 0x5d41c57b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe1c57f, SUC_lexicon_tags_17 },
    { 0xb141c57f, SUC_lexicon_tags_2 },
    { 0xc661c580, SUC_lexicon_tags_23 },
    { 0x18c1c580, SUC_lexicon_tags_33 },
    { 0x1b21c580, SUC_lexicon_tags_7 },
    { 0xc9e1c581, SUC_lexicon_tags_27 },
    { 0x8be1c585, SUC_lexicon_tags_25 },
    { 0xaee1c585, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a1c58c, SUC_lexicon_tags_7 },
    { 0xdf21c58c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc21c58f, SUC_lexicon_tags_23 },
    { 0x7301c58f, SUC_lexicon_tags_29 },
    { 0x1861c591, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6681c593, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x281c596, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc1c59c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9821c59f, SUC_lexicon_tags_40 },
    { 0xed01c5a0, SUC_lexicon_tags_11 },
    { 0x3da1c5a1, SUC_lexicon_tags_25 },
    { 0x8aa1c5a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa141c5a7, SUC_lexicon_tags_38 },
    { 0xa901c5a8, SUC_lexicon_tags_10 },
    { 0xc121c5a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9361c5ac, SUC_lexicon_tags_81 },
    { 0x53e1c5ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae41c5af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa121c5b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8581c5be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfc1c5c0, SUC_lexicon_tags_17 },
    { 0xa41c5c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681c5c5, SUC_lexicon_tags_37 },
    { 0x7aa1c5c6, SUC_lexicon_tags_12 },
    { 0x9941c5c5, SUC_lexicon_tags_62 },
    { 0x45c1c5c6, SUC_lexicon_tags_9 },
    { 0xf2c1c5c7, SUC_lexicon_tags_25 },
    { 0x5de1c5ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c1c5cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1c5d3, SUC_lexicon_tags_12 },
    { 0xf7e1c5d4, SUC_lexicon_tags_56 },
    { 0x2f61c5d3, SUC_lexicon_tags_19 },
    { 0x8e1c5d6, SUC_lexicon_tags_25 },
    { 0xaba1c5d7, SUC_lexicon_tags_857 },
    { 0xeb81c5d7, SUC_lexicon_tags_19 },
    { 0x75e1c5d4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa1c5dc, SUC_lexicon_tags_17 },
    { 0x5b21c5dc, SUC_lexicon_tags_60 },
    { 0xee01c5dc, SUC_lexicon_tags_15 },
    { 0x5ac1c5df, SUC_lexicon_tags_4 },
    { 0x9aa1c5df, SUC_lexicon_tags_31 },
    { 0x3aa1c5e0, SUC_lexicon_tags_18 },
    { 0x5f01c5e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7021c5e5, SUC_lexicon_tags_54 },
    { 0x6241c5e6, SUC_lexicon_tags_6 },
    { 0xd01c5e7, SUC_lexicon_tags_17 },
    { 0xcee1c5e6, SUC_lexicon_tags_29 },
    { 0x3d01c5e9, SUC_lexicon_tags_25 },
    { 0xcb41c5e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f81c5f2, SUC_lexicon_tags_31 },
    { 0x9781c5f3, SUC_lexicon_tags_25 },
    { 0xe9c1c5f4, SUC_lexicon_tags_31 },
    { 0xa2c1c5f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a1c5f8, SUC_lexicon_tags_46 },
    { 0x5c01c5f9, SUC_lexicon_tags_477 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee1c5fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9dc1c5fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ba1c600, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ba1c602, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a1c605, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1e1c607, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4e1c609, SUC_lexicon_tags_47 },
    { 0x40c1c609, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7461c60c, SUC_lexicon_tags_2 },
    { 0xbfa1c60d, SUC_lexicon_tags_7 },
    { 0x3a01c60d, SUC_lexicon_tags_11 },
    { 0x2b81c60d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5121c612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e1c615, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e21c618, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a1c61c, SUC_lexicon_tags_39 },
    { 0x6de1c61c, SUC_lexicon_tags_60 },
    { 0x3cc1c61d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd821c626, SUC_lexicon_tags_11 },
    { 0xb7e1c627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc1c634, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa8c1c636, SUC_lexicon_tags_1 },
    { 0xb681c636, SUC_lexicon_tags_11 },
    { 0x9ce1c638, SUC_lexicon_tags_158 },
    { 0x5661c639, SUC_lexicon_tags_33 },
    { 0xd041c639, SUC_lexicon_tags_34 },
    { 0x42a1c63b, SUC_lexicon_tags_2 },
    { 0xce81c636, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2521c63f, SUC_lexicon_tags_25 },
    { 0x45e1c640, SUC_lexicon_tags_12 },
    { 0x8e81c640, SUC_lexicon_tags_39 },
    { 0x5201c642, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ea1c644, SUC_lexicon_tags_1 },
    { 0x7dc1c645, SUC_lexicon_tags_6 },
    { 0xdc01c644, SUC_lexicon_tags_9 },
    { 0x2d61c647, SUC_lexicon_tags_16 },
    { 0x2ce1c647, SUC_lexicon_tags_35 },
    { 0x40c1c649, SUC_lexicon_tags_25 },
    { 0x8941c644, SUC_lexicon_tags_77 },
    { 0x64e1c64b, SUC_lexicon_tags_77 },
    { 0x3581c64a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x42a1c64e, SUC_lexicon_tags_17 },
    { 0xa1a1c64e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3181c651, SUC_lexicon_tags_27 },
    { 0x1261c652, SUC_lexicon_tags_199 },
    { 0xbb81c653, SUC_lexicon_tags_60 },
    { 0x861c652, SUC_lexicon_tags_26 },
    { 0xcf41c655, SUC_lexicon_tags_12 },
    { 0xa4e1c651, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x401c658, SUC_lexicon_tags_1 },
    { 0xd881c659, SUC_lexicon_tags_35 },
    { 0x8621c65a, SUC_lexicon_tags_25 },
    { 0x4381c658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c65e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf21c660, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x60c1c662, SUC_lexicon_tags_23 },
    { 0x72a1c663, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6181c66d, SUC_lexicon_tags_17 },
    { 0xde81c66e, SUC_lexicon_tags_7 },
    { 0x2821c66f, SUC_lexicon_tags_7 },
    { 0x2381c670, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf01c678, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8501c67c, SUC_lexicon_tags_27 },
    { 0xee81c67d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcc41c67f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5261c681, SUC_lexicon_tags_1 },
    { 0xe8c1c681, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5a1c684, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf761c688, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb561c68f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3c01c691, SUC_lexicon_tags_63 },
    { 0x2ec1c692, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de1c698, SUC_lexicon_tags_22 },
    { 0x9a61c698, SUC_lexicon_tags_5 },
    { 0x3541c698, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e61c69c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e1c6a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93c1c6a6, SUC_lexicon_tags_25 },
    { 0x93e1c6a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41c6aa, SUC_lexicon_tags_17 },
    { 0xea1c6ab, SUC_lexicon_tags_37 },
    { 0x54e1c6ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92e1c6ae, SUC_lexicon_tags_39 },
    { 0xfe41c6af, SUC_lexicon_tags_1 },
    { 0x5341c6b0, SUC_lexicon_tags_25 },
    { 0xdc41c6af, SUC_lexicon_tags_12 },
    { 0x4601c6b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2981c6b4, SUC_lexicon_tags_10 },
    { 0x3f21c6b5, SUC_lexicon_tags_11 },
    { 0xad41c6b5, SUC_lexicon_tags_12 },
    { 0xf2c1c6b7, SUC_lexicon_tags_33 },
    { 0x80e1c6b6, SUC_lexicon_tags_1 },
    { 0x9ea1c6b9, SUC_lexicon_tags_31 },
    { 0xeba1c6ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb701c6bc, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xbf41c6be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca1c6c0, SUC_lexicon_tags_49 },
    { 0xb981c6c0, SUC_lexicon_tags_11 },
    { 0x481c6c2, SUC_lexicon_tags_33 },
    { 0x5221c6c3, SUC_lexicon_tags_49 },
    { 0xcc61c6c0, SUC_lexicon_tags_9 },
    { 0xd7a1c6c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb141c6c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf941c6ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe441c6d5, SUC_lexicon_tags_11 },
    { 0xe4c1c6d5, SUC_lexicon_tags_17 },
    { 0xea81c6d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc41c6d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8281c6db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe1c6de, SUC_lexicon_tags_25 },
    { 0x89a1c6de, SUC_lexicon_tags_1 },
    { 0xf01c6de, SUC_lexicon_tags_44 },
    { 0x9681c6e1, SUC_lexicon_tags_37 },
    { 0xd041c6e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3041c6e4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1c6e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e1c6ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x93c1c6f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6281c6f3, SUC_lexicon_tags_33 },
    { 0x2381c6f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e1c6f9, SUC_lexicon_tags_19 },
    { 0xb581c6fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde01c703, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe1c709, SUC_lexicon_tags_61 },
    { 0x2e21c70a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f01c70d, SUC_lexicon_tags_4 },
    { 0x18a1c70e, SUC_lexicon_tags_1 },
    { 0x11c1c70f, SUC_lexicon_tags_28 },
    { 0x85e1c70d, SUC_lexicon_tags_62 },
    { 0x2201c711, SUC_lexicon_tags_63 },
    { 0x2721c711, SUC_lexicon_tags_53 },
    { 0xd2e1c712, SUC_lexicon_tags_25 },
    { 0x7e41c70f, SUC_lexicon_tags_47 },
    { 0xa781c715, SUC_lexicon_tags_1 },
    { 0x5a1c716, SUC_lexicon_tags_57 },
    { 0x8ac1c70f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2441c719, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf161c71c, SUC_lexicon_tags_85 },
    { 0xd441c71d, SUC_lexicon_tags_25 },
    { 0x6fe1c71e, SUC_lexicon_tags_14 },
    { 0x5681c71f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76a1c721, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc01c724, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6121c729, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f41c72b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb21c72d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c1c731, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8861c733, SUC_lexicon_tags_38 },
    { 0x2a21c734, SUC_lexicon_tags_37 },
    { 0xac41c735, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0xfa1c737, SUC_lexicon_tags_49 },
    { 0x9ec1c738, SUC_lexicon_tags_8 },
    { 0x6001c739, SUC_lexicon_tags_0 },
    { 0xd1c1c73a, SUC_lexicon_tags_63 },
    { 0x93e1c73b, SUC_lexicon_tags_12 },
    { 0xfda1c73c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1921c743, SUC_lexicon_tags_87 },
    { 0x3901c744, SUC_lexicon_tags_47 },
    { 0x3601c745, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6741c74a, SUC_lexicon_tags_47 },
    { 0xfe61c74b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa0c1c74d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa1c750, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a1c753, SUC_lexicon_tags_31 },
    { 0x2701c753, SUC_lexicon_tags_15 },
    { 0xe401c755, SUC_lexicon_tags_129 },
    { 0x2de1c753, SUC_lexicon_tags_12 },
    { 0x6b21c757, SUC_lexicon_tags_35 },
    { 0xf2e1c757, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca61c75c, SUC_lexicon_tags_762 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa681c761, SUC_lexicon_tags_15 },
    { 0x25a1c762, SUC_lexicon_tags_14 },
    { 0x47a1c762, SUC_lexicon_tags_31 },
    { 0x3681c764, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb81c768, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c1c76b, SUC_lexicon_tags_47 },
    { 0xd221c76b, SUC_lexicon_tags_87 },
    { 0x5b81c76d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee61c76f, SUC_lexicon_tags_4 },
    { 0xa721c76f, SUC_lexicon_tags_15 },
    { 0xb481c76f, SUC_lexicon_tags_37 },
    { 0x8be1c772, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3441c779, SUC_lexicon_tags_25 },
    { 0xec1c77a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3821c77c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a1c786, SUC_lexicon_tags_10 },
    { 0xaa81c787, SUC_lexicon_tags_35 },
    { 0x7801c788, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f41c797, SUC_lexicon_tags_53 },
    { 0xc781c798, SUC_lexicon_tags_12 },
    { 0xd981c799, SUC_lexicon_tags_17 },
    { 0xa721c799, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc341c79d, SUC_lexicon_tags_7 },
    { 0xa9a1c79e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a1c7a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x31e1c7a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac1c7a8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381c7ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e1c7af, SUC_lexicon_tags_79 },
    { 0xf681c7b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e1c7b4, SUC_lexicon_tags_19 },
    { 0x7341c7b5, SUC_lexicon_tags_14 },
    { 0x15a1c7b6, SUC_lexicon_tags_61 },
    { 0xce81c7b5, SUC_lexicon_tags_37 },
    { 0xa5e1c7b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c1c7bb, SUC_lexicon_tags_11 },
    { 0xdd21c7bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2801c7c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421c7c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3c21c7c8, SUC_lexicon_tags_25 },
    { 0x16e1c7c8, SUC_lexicon_tags_1 },
    { 0x9a1c7c8, SUC_lexicon_tags_49 },
    { 0x6781c7c9, SUC_lexicon_tags_11 },
    { 0x73c1c7cc, SUC_lexicon_tags_7 },
    { 0x2541c7c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41a1c7cf, SUC_lexicon_tags_14 },
    { 0x2f01c7d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c1c7d6, SUC_lexicon_tags_28 },
    { 0x6881c7d6, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61c7da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a61c7de, SUC_lexicon_tags_6 },
    { 0xefe1c7df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4721c7e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb941c7e5, SUC_lexicon_tags_921 },
    { 0x5a81c7e6, SUC_lexicon_tags_68 },
    { 0x8461c7e7, SUC_lexicon_tags_33 },
    { 0xbd01c7e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4161c7f2, SUC_lexicon_tags_15 },
    { 0xcb21c7f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9981c7f6, SUC_lexicon_tags_17 },
    { 0xca01c7f7, SUC_lexicon_tags_2 },
    { 0xb181c7f8, SUC_lexicon_tags_77 },
    { 0xfa21c7f9, SUC_lexicon_tags_30 },
    { 0xef41c7fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca41c801, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1061c805, SUC_lexicon_tags_38 },
    { 0x8ca1c806, SUC_lexicon_tags_38 },
    { 0xc5a1c806, SUC_lexicon_tags_25 },
    { 0xe81c807, SUC_lexicon_tags_37 },
    { 0x5041c809, SUC_lexicon_tags_179 },
    { 0x5101c809, SUC_lexicon_tags_25 },
    { 0xdf21c80b, SUC_lexicon_tags_33 },
    { 0x9ec1c808, SUC_lexicon_tags_17 },
    { 0x15c1c80d, SUC_lexicon_tags_26 },
    { 0x4bc1c80e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6481c810, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c1c813, SUC_lexicon_tags_11 },
    { 0x92c1c814, SUC_lexicon_tags_7 },
    { 0x8001c814, SUC_lexicon_tags_5 },
    { 0xdea1c814, SUC_lexicon_tags_37 },
    { 0x5141c817, SUC_lexicon_tags_28 },
    { 0x7201c817, SUC_lexicon_tags_17 },
    { 0xa1e1c819, SUC_lexicon_tags_25 },
    { 0xc5a1c81a, SUC_lexicon_tags_31 },
    { 0x761c81a, SUC_lexicon_tags_37 },
    { 0xd761c819, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a81c826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a1c829, SUC_lexicon_tags_2 },
    { 0xc4e1c82a, SUC_lexicon_tags_7 },
    { 0x2941c82b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd261c82f, SUC_lexicon_tags_0 },
    { 0x4c1c82f, SUC_lexicon_tags_7 },
    { 0xccc1c830, SUC_lexicon_tags_11 },
    { 0xd981c832, SUC_lexicon_tags_56 },
    { 0x5ec1c833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd461c835, SUC_lexicon_tags_55 },
    { 0xa981c836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d01c83c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1261c842, SUC_lexicon_tags_45 },
    { 0xc541c842, SUC_lexicon_tags_17 },
    { 0xf561c842, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c1c849, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca1c853, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd381c856, SUC_lexicon_tags_25 },
    { 0xc701c857, SUC_lexicon_tags_0 },
    { 0x53a1c858, SUC_lexicon_tags_60 },
    { 0x6601c858, SUC_lexicon_tags_30 },
    { 0x9ba1c857, SUC_lexicon_tags_7 },
    { 0xd581c85a, SUC_lexicon_tags_37 },
    { 0xd9e1c859, SUC_lexicon_tags_12 },
    { 0x9ce1c858, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9601c860, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c21c863, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9641c866, SUC_lexicon_tags_30 },
    { 0xc261c867, SUC_lexicon_tags_28 },
    { 0x1a21c868, SUC_lexicon_tags_7 },
    { 0x76a1c869, SUC_lexicon_tags_46 },
    { 0x9ea1c868, SUC_lexicon_tags_55 },
    { 0x5401c866, SUC_lexicon_tags_60 },
    { 0x87a1c86c, SUC_lexicon_tags_7 },
    { 0xfbe1c86d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4921c870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc1c873, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21c877, SUC_lexicon_tags_42 },
    { 0xd941c878, SUC_lexicon_tags_26 },
    { 0x27a1c879, SUC_lexicon_tags_11 },
    { 0x6f41c877, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1321c87d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5e61c87f, SUC_lexicon_tags_28 },
    { 0x92e1c880, SUC_lexicon_tags_11 },
    { 0x4ba1c880, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe81c883, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3a1c885, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5681c88a, SUC_lexicon_tags_19 },
    { 0xa101c88a, SUC_lexicon_tags_17 },
    { 0xd61c88b, SUC_lexicon_tags_27 },
    { 0x54a1c88d, SUC_lexicon_tags_55 },
    { 0x4aa1c88e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc841c890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d41c892, SUC_lexicon_tags_29 },
    { 0x5341c893, SUC_lexicon_tags_38 },
    { 0xd221c892, SUC_lexicon_tags_1 },
    { 0xebe1c895, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe1c898, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6a1c89a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9e1c89c, SUC_lexicon_tags_12 },
    { 0xa2e1c89d, SUC_lexicon_tags_95 },
    { 0x53a1c89e, SUC_lexicon_tags_10 },
    { 0x9c41c89f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9661c8a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee41c8a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b61c8af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6f81c8b1, SUC_lexicon_tags_30 },
    { 0x6c1c8b2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa7c1c8b4, SUC_lexicon_tags_25 },
    { 0x9941c8b5, SUC_lexicon_tags_7 },
    { 0xafc1c8b6, SUC_lexicon_tags_11 },
    { 0xc281c8b5, SUC_lexicon_tags_47 },
    { 0x25c1c8b6, SUC_lexicon_tags_25 },
    { 0x1e81c8b5, SUC_lexicon_tags_46 },
    { 0x6ea1c8ba, SUC_lexicon_tags_38 },
    { 0xa401c8bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7701c8bd, SUC_lexicon_tags_25 },
    { 0x9801c8be, SUC_lexicon_tags_13 },
    { 0x6741c8bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6761c8c2, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3621c8c9, SUC_lexicon_tags_66 },
    { 0x5a01c8ca, SUC_lexicon_tags_40 },
    { 0x681c8c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d81c8cf, SUC_lexicon_tags_12 },
    { 0x3d81c8d0, SUC_lexicon_tags_31 },
    { 0x13c1c8cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a61c8d3, SUC_lexicon_tags_16 },
    { 0x5f21c8d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f61c8d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x71e1c8d8, SUC_lexicon_tags_1 },
    { 0x3b01c8d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d01c8e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e1c8e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a21c8ea, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda1c8ee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe461c8f2, SUC_lexicon_tags_60 },
    { 0x5801c8f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x99a1c8f5, SUC_lexicon_tags_45 },
    { 0x5081c8f6, SUC_lexicon_tags_37 },
    { 0xbb81c8f6, SUC_lexicon_tags_2 },
    { 0x4661c8f8, SUC_lexicon_tags_6 },
    { 0xcb61c8f9, SUC_lexicon_tags_43 },
    { 0x76a1c8f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc1c900, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbd01c902, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a1c905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c1c90a, SUC_lexicon_tags_26 },
    { 0xf6c1c90a, SUC_lexicon_tags_26 },
    { 0x9f01c90a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a1c90f, SUC_lexicon_tags_37 },
    { 0x8e01c90f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf641c916, SUC_lexicon_tags_0 },
    { 0x6941c917, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6281c919, SUC_lexicon_tags_12 },
    { 0x4481c91a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca1c923, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x381c926, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d01c929, SUC_lexicon_tags_25 },
    { 0x2c1c929, SUC_lexicon_tags_65 },
    { 0xd4e1c92b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5701c92f, SUC_lexicon_tags_27 },
    { 0xc101c930, SUC_lexicon_tags_7 },
    { 0xbe01c92f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3c1c933, SUC_lexicon_tags_11 },
    { 0xb961c934, SUC_lexicon_tags_9 },
    { 0xcae1c934, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c1c939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9761c93b, SUC_lexicon_tags_11 },
    { 0xdcc1c93c, SUC_lexicon_tags_47 },
    { 0xd4e1c93d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c1c944, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c1c947, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x75e1c949, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb21c94d, SUC_lexicon_tags_65 },
    { 0xd941c94e, SUC_lexicon_tags_11 },
    { 0xe8e1c94e, SUC_lexicon_tags_43 },
    { 0xf4e1c94d, SUC_lexicon_tags_63 },
    { 0x4ee1c951, SUC_lexicon_tags_1 },
    { 0xea61c952, SUC_lexicon_tags_12 },
    { 0xb2e1c952, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5e1c955, SUC_lexicon_tags_0 },
    { 0x3141c955, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3d61c958, SUC_lexicon_tags_2 },
    { 0xd6a1c959, SUC_lexicon_tags_7 },
    { 0xfb41c95a, SUC_lexicon_tags_25 },
    { 0xeb01c95b, SUC_lexicon_tags_61 },
    { 0x92a1c95c, SUC_lexicon_tags_26 },
    { 0x68c1c959, SUC_lexicon_tags_49 },
    { 0xef81c95b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e01c960, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8481c965, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa0e1c967, SUC_lexicon_tags_25 },
    { 0x6ec1c967, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1c96b, SUC_lexicon_tags_11 },
    { 0xd341c96c, SUC_lexicon_tags_47 },
    { 0x8dc1c96d, SUC_lexicon_tags_30 },
    { 0xf281c96e, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc1c972, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7d81c974, SUC_lexicon_tags_14 },
    { 0xb481c975, SUC_lexicon_tags_25 },
    { 0x7221c976, SUC_lexicon_tags_37 },
    { 0xf2c1c977, SUC_lexicon_tags_28 },
    { 0x35e1c977, SUC_lexicon_tags_19 },
    { 0xb7c1c978, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1101c97c, SUC_lexicon_tags_27 },
    { 0x1481c97c, SUC_lexicon_tags_12 },
    { 0x7301c97e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a1c984, SUC_lexicon_tags_37 },
    { 0x1e81c984, SUC_lexicon_tags_9 },
    { 0xbb41c985, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf561c988, SUC_lexicon_tags_7 },
    { 0xf2c1c988, SUC_lexicon_tags_19 },
    { 0xae21c98a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e1c98e, SUC_lexicon_tags_27 },
    { 0xf7e1c98f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d81c992, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa721c999, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e41c99f, SUC_lexicon_tags_66 },
    { 0xbfc1c9a0, SUC_lexicon_tags_1 },
    { 0xea61c9a1, SUC_lexicon_tags_39 },
    { 0x2101c99f, SUC_lexicon_tags_25 },
    { 0x3d61c9a3, SUC_lexicon_tags_11 },
    { 0x4821c9a0, SUC_lexicon_tags_7 },
    { 0x26c1c9a5, SUC_lexicon_tags_57 },
    { 0x6a41c9a6, SUC_lexicon_tags_31 },
    { 0x85a1c9a6, SUC_lexicon_tags_12 },
    { 0x6601c9a7, SUC_lexicon_tags_37 },
    { 0xa7c1c9a9, SUC_lexicon_tags_14 },
    { 0x45c1c9aa, SUC_lexicon_tags_11 },
    { 0x2b81c9ab, SUC_lexicon_tags_11 },
    { 0x6061c9a9, SUC_lexicon_tags_48 },
    { 0x3ea1c9a9, SUC_lexicon_tags_1 },
    { 0xaa01c9ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x861c9b1, SUC_lexicon_tags_29 },
    { 0x6641c9b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e1c9b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc1c9bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed01c9bd, SUC_lexicon_tags_17 },
    { 0xde01c9be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2041c9c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x101c9c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa881c9c7, SUC_lexicon_tags_11 },
    { 0x9fe1c9c8, SUC_lexicon_tags_31 },
    { 0xb121c9c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a1c9cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeba1c9cf, SUC_lexicon_tags_37 },
    { 0x5941c9cf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x16c1c9d2, SUC_lexicon_tags_14 },
    { 0x8641c9d3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9441c9d5, SUC_lexicon_tags_46 },
    { 0xa2a1c9d5, SUC_lexicon_tags_45 },
    { 0x94a1c9d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea1c9da, SUC_lexicon_tags_25 },
    { 0xbbc1c9db, SUC_lexicon_tags_14 },
    { 0xda81c9dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7001c9de, SUC_lexicon_tags_15 },
    { 0x3141c9df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa481c9e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c1c9eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa21c9ed, SUC_lexicon_tags_16 },
    { 0x8941c9ee, SUC_lexicon_tags_29 },
    { 0xeb21c9ef, SUC_lexicon_tags_12 },
    { 0x6341c9f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc181c9f3, SUC_lexicon_tags_7 },
    { 0x7981c9f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2101c9f6, SUC_lexicon_tags_11 },
    { 0xbf41c9f7, SUC_lexicon_tags_0 },
    { 0x6161c9f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9281c9fb, SUC_lexicon_tags_43 },
    { 0x1501c9fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7021c9ff, SUC_lexicon_tags_38 },
    { 0x18a1ca00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c1ca0a, SUC_lexicon_tags_37 },
    { 0xa961ca0b, SUC_lexicon_tags_2 },
    { 0xd8e1ca0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a1ca0f, SUC_lexicon_tags_13 },
    { 0xcdc1ca10, SUC_lexicon_tags_84 },
    { 0x5421ca11, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xef01ca14, SUC_lexicon_tags_19 },
    { 0xb341ca15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1281ca19, SUC_lexicon_tags_49 },
    { 0xa801ca1a, SUC_lexicon_tags_7 },
    { 0x8fe1ca1b, SUC_lexicon_tags_31 },
    { 0x3701ca1c, SUC_lexicon_tags_43 },
    { 0x66c1ca1b, SUC_lexicon_tags_11 },
    { 0xc0e1ca19, SUC_lexicon_tags_91 },
    { 0x4d61ca1b, SUC_lexicon_tags_38 },
    { 0xda61ca1c, SUC_lexicon_tags_47 },
    { 0x4221ca21, SUC_lexicon_tags_31 },
    { 0xf9e1ca22, SUC_lexicon_tags_2 },
    { 0x73c1ca1e, SUC_lexicon_tags_10 },
    { 0xa701ca1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9aa1ca26, SUC_lexicon_tags_7 },
    { 0x7901ca27, SUC_lexicon_tags_45 },
    { 0x77c1ca28, SUC_lexicon_tags_11 },
    { 0x3041ca29, SUC_lexicon_tags_7 },
    { 0x5ae1ca29, SUC_lexicon_tags_23 },
    { 0xf4c1ca2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4201ca2e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8de1ca30, SUC_lexicon_tags_11 },
    { 0xb041ca30, SUC_lexicon_tags_1 },
    { 0xe941ca31, SUC_lexicon_tags_17 },
    { 0xe8a1ca33, SUC_lexicon_tags_37 },
    { 0xbdc1ca34, SUC_lexicon_tags_1 },
    { 0xd401ca35, SUC_lexicon_tags_63 },
    { 0x57e1ca35, SUC_lexicon_tags_43 },
    { 0xbb41ca37, SUC_lexicon_tags_14 },
    { 0x1a1ca36, SUC_lexicon_tags_11 },
    { 0xba81ca39, SUC_lexicon_tags_68 },
    { 0xe881ca39, SUC_lexicon_tags_27 },
    { 0x5201ca3b, SUC_lexicon_tags_11 },
    { 0xb4c1ca3a, SUC_lexicon_tags_13 },
    { 0x9d41ca3d, SUC_lexicon_tags_2 },
    { 0xe8a1ca3a, SUC_lexicon_tags_14 },
    { 0x2961ca3f, SUC_lexicon_tags_6 },
    { 0x7301ca3c, SUC_lexicon_tags_45 },
    { 0xbea1ca39, SUC_lexicon_tags_15 },
    { 0xb0c1ca42, SUC_lexicon_tags_25 },
    { 0xdd81ca43, SUC_lexicon_tags_63 },
    { 0xf521ca43, SUC_lexicon_tags_56 },
    { 0xee01ca36, SUC_lexicon_tags_77 },
    { 0x1241ca46, SUC_lexicon_tags_2 },
    { 0x8141ca47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a21ca4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac1ca52, SUC_lexicon_tags_11 },
    { 0x7101ca53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f81ca56, SUC_lexicon_tags_46 },
    { 0x6e61ca56, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc7c1ca59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e1ca5f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e1ca62, SUC_lexicon_tags_49 },
    { 0x6fa1ca63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a1ca6b, SUC_lexicon_tags_25 },
    { 0xb661ca6c, SUC_lexicon_tags_23 },
    { 0x28a1ca6d, SUC_lexicon_tags_0 },
    { 0x7101ca6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb21ca71, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xd681ca73, SUC_lexicon_tags_45 },
    { 0xe061ca74, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e41ca78, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a1ca7e, SUC_lexicon_tags_6 },
    { 0x2661ca7f, SUC_lexicon_tags_2 },
    { 0x65c1ca7f, SUC_lexicon_tags_2 },
    { 0x8de1ca81, SUC_lexicon_tags_25 },
    { 0x1621ca82, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5561ca85, SUC_lexicon_tags_12 },
    { 0x2121ca85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8a1ca88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b01ca8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1ca91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5041ca93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8c1ca95, SUC_lexicon_tags_12 },
    { 0x94c1ca96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf301ca9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6041caa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1a1caa7, SUC_lexicon_tags_39 },
    { 0x8461caa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1ac1caaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6221caac, SUC_lexicon_tags_25 },
    { 0xb21caad, SUC_lexicon_tags_8 },
    { 0x1caae, SUC_lexicon_tags_8 },
    { 0x6ca1caae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce1cab2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4861cab5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c81caba, SUC_lexicon_tags_30 },
    { 0xb61cabb, SUC_lexicon_tags_17 },
    { 0x2ba1cabb, SUC_lexicon_tags_15 },
    { 0xf441cabd, SUC_lexicon_tags_27 },
    { 0x2981cabe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e1cac1, SUC_lexicon_tags_17 },
    { 0x9621cac2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb561cac5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e1cace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c1cad3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8161cad6, SUC_lexicon_tags_15 },
    { 0xc541cad6, SUC_lexicon_tags_900 },
    { 0xcde1cad7, SUC_lexicon_tags_38 },
    { 0xb5c1cad9, SUC_lexicon_tags_65 },
    { 0x2ac1cada, SUC_lexicon_tags_16 },
    { 0x1961cad8, SUC_lexicon_tags_133 },
    { 0xbae1cadc, SUC_lexicon_tags_49 },
    { 0x1da1cadc, SUC_lexicon_tags_15 },
    { 0x3821cade, SUC_lexicon_tags_15 },
    { 0xdf81cadf, SUC_lexicon_tags_2 },
    { 0x161cadf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd21cae2, SUC_lexicon_tags_15 },
    { 0x5c41cae2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe201caf5, SUC_lexicon_tags_0 },
    { 0x8fa1caf6, SUC_lexicon_tags_27 },
    { 0x74a1caf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a1cafa, SUC_lexicon_tags_6 },
    { 0x4d41cafa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4121caff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c1cb02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e1cb05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8261cb07, SUC_lexicon_tags_37 },
    { 0x81c1cb07, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x38a1cb0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc1cb0e, SUC_lexicon_tags_45 },
    { 0x4ac1cb0e, SUC_lexicon_tags_16 },
    { 0xf81cb0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9801cb18, SUC_lexicon_tags_27 },
    { 0x7881cb19, SUC_lexicon_tags_47 },
    { 0xc201cb1a, SUC_lexicon_tags_8 },
    { 0x9901cb19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3e1cb1d, SUC_lexicon_tags_25 },
    { 0xf8a1cb1e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a1cb28, SUC_lexicon_tags_30 },
    { 0xbce1cb29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfba1cb2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd1c1cb2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe41cb30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a1cb34, SUC_lexicon_tags_250 },
    { 0xd041cb35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f81cb39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1421cb3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2701cb3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b21cb40, SUC_lexicon_tags_2 },
    { 0x3041cb41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a1cb46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc561cb48, SUC_lexicon_tags_17 },
    { 0x6141cb48, SUC_lexicon_tags_2 },
    { 0xf9c1cb49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1081cb51, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8101cb55, SUC_lexicon_tags_34 },
    { 0x8421cb55, SUC_lexicon_tags_65 },
    { 0x53c1cb57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7261cb59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2961cb5d, SUC_lexicon_tags_39 },
    { 0x4961cb5d, SUC_lexicon_tags_1 },
    { 0x9161cb5f, SUC_lexicon_tags_11 },
    { 0xa3a1cb5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb821cb63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc641cb67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3221cb6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6421cb6f, SUC_lexicon_tags_1 },
    { 0xa681cb6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c1cb75, SUC_lexicon_tags_127 },
    { 0xeee1cb75, SUC_lexicon_tags_0 },
    { 0x2ec1cb76, SUC_lexicon_tags_6 },
    { 0x1581cb78, SUC_lexicon_tags_25 },
    { 0x3141cb79, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4f01cb7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a1cb81, SUC_lexicon_tags_518 },
    { 0xf801cb82, SUC_lexicon_tags_17 },
    { 0x6ca1cb82, SUC_lexicon_tags_47 },
    { 0xeb61cb84, SUC_lexicon_tags_37 },
    { 0x8121cb85, SUC_lexicon_tags_33 },
    { 0x5481cb85, SUC_lexicon_tags_62 },
    { 0x7861cb87, SUC_lexicon_tags_47 },
    { 0x50c1cb87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd361cb8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a1cb8e, SUC_lexicon_tags_15 },
    { 0x66c1cb8e, SUC_lexicon_tags_25 },
    { 0x5161cb90, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe921cb96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1cb99, SUC_lexicon_tags_6 },
    { 0x1ae1cb99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c1cb9d, SUC_lexicon_tags_1 },
    { 0x9f81cb9e, SUC_lexicon_tags_1048 },
    { 0, NULL },
    { 0x7921cba0, SUC_lexicon_tags_7 },
    { 0x7f01cba0, SUC_lexicon_tags_19 },
    { 0xefc1cba0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c1cba8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8881cbaa, SUC_lexicon_tags_157 },
    { 0x4e61cbaa, SUC_lexicon_tags_56 },
    { 0x1481cbac, SUC_lexicon_tags_11 },
    { 0x7301cbad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8321cbb3, SUC_lexicon_tags_25 },
    { 0xa921cbb3, SUC_lexicon_tags_17 },
    { 0x2061cbb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c1cbb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf041cbc3, SUC_lexicon_tags_12 },
    { 0xd021cbc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa1cbc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26e1cbca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c61cbcc, SUC_lexicon_tags_39 },
    { 0xfa61cbcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8761cbd3, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x3141cbd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a1cbd8, SUC_lexicon_tags_17 },
    { 0x7ee1cbd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4001cbdb, SUC_lexicon_tags_35 },
    { 0x9e1cbdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4141cbe2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75c1cbe4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7221cbe6, SUC_lexicon_tags_65 },
    { 0xb801cbe7, SUC_lexicon_tags_11 },
    { 0x67c1cbe8, SUC_lexicon_tags_38 },
    { 0xe1c1cbe9, SUC_lexicon_tags_14 },
    { 0xd741cbea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xee01cbec, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xe4e1cbee, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc1cbf2, SUC_lexicon_tags_10 },
    { 0x3ae1cbf3, SUC_lexicon_tags_17 },
    { 0xa8c1cbf4, SUC_lexicon_tags_11 },
    { 0xb301cbf4, SUC_lexicon_tags_15 },
    { 0x5961cbf6, SUC_lexicon_tags_6 },
    { 0xe761cbf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7601cbf9, SUC_lexicon_tags_15 },
    { 0x7c81cbf9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc6e1cbfc, SUC_lexicon_tags_23 },
    { 0x1ea1cbfc, SUC_lexicon_tags_7 },
    { 0x4e01cbfd, SUC_lexicon_tags_47 },
    { 0xec41cbfe, SUC_lexicon_tags_14 },
    { 0x4321cbff, SUC_lexicon_tags_27 },
    { 0x2a1cc01, SUC_lexicon_tags_17 },
    { 0x3081cc02, SUC_lexicon_tags_9 },
    { 0xb621cc01, SUC_lexicon_tags_22 },
    { 0x1d41cc01, SUC_lexicon_tags_613 },
    { 0, NULL },
    { 0x1821cc06, SUC_lexicon_tags_11 },
    { 0xdc1cc07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c1cc0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17c1cc0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe821cc13, SUC_lexicon_tags_21 },
    { 0x9381cc13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa721cc16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9581cc19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6701cc1d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x61e1cc1f, SUC_lexicon_tags_2 },
    { 0x2061cc20, SUC_lexicon_tags_57 },
    { 0xc8c1cc1f, SUC_lexicon_tags_7 },
    { 0x83e1cc1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe181cc29, SUC_lexicon_tags_38 },
    { 0x4d01cc29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd541cc2c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c1cc2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3541cc31, SUC_lexicon_tags_25 },
    { 0xbe61cc32, SUC_lexicon_tags_2 },
    { 0x16a1cc32, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd6c1cc35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7321cc3e, SUC_lexicon_tags_11 },
    { 0x67e1cc3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9a61cc41, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x1841cc44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa1cc49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea61cc4d, SUC_lexicon_tags_1 },
    { 0x641cc4e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x301cc50, SUC_lexicon_tags_7 },
    { 0xd5e1cc51, SUC_lexicon_tags_11 },
    { 0xec61cc50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2c1cc54, SUC_lexicon_tags_30 },
    { 0x6dc1cc55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc1cc58, SUC_lexicon_tags_25 },
    { 0x5441cc58, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e1cc5f, SUC_lexicon_tags_113 },
    { 0xa2e1cc60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c1cc65, SUC_lexicon_tags_113 },
    { 0xba01cc65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5881cc69, SUC_lexicon_tags_17 },
    { 0xcc1cc6a, SUC_lexicon_tags_18 },
    { 0x93c1cc69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf881cc6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d21cc71, SUC_lexicon_tags_11 },
    { 0x1661cc72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2641cc74, SUC_lexicon_tags_7 },
    { 0xd721cc75, SUC_lexicon_tags_2 },
    { 0x3f41cc75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f41cc79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc941cc7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfd01cc7f, SUC_lexicon_tags_25 },
    { 0xdc01cc7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x141cc82, SUC_lexicon_tags_66 },
    { 0x2ca1cc83, SUC_lexicon_tags_47 },
    { 0x5361cc84, SUC_lexicon_tags_21 },
    { 0x5361cc85, SUC_lexicon_tags_16 },
    { 0xf401cc86, SUC_lexicon_tags_32 },
    { 0xd281cc87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf721cc89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b61cc8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb21cc97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e1cc9a, SUC_lexicon_tags_27 },
    { 0x1f61cc9b, SUC_lexicon_tags_28 },
    { 0xe681cc9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2b41cc9e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe881cca2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2661cca4, SUC_lexicon_tags_15 },
    { 0xad21cca4, SUC_lexicon_tags_11 },
    { 0x4181cca4, SUC_lexicon_tags_37 },
    { 0x221cca5, SUC_lexicon_tags_85 },
    { 0x7981cca8, SUC_lexicon_tags_30 },
    { 0x4201cca8, SUC_lexicon_tags_15 },
    { 0x1a81ccaa, SUC_lexicon_tags_1 },
    { 0xedc1cca5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e1ccae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe61ccb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e21ccb4, SUC_lexicon_tags_28 },
    { 0x47a1ccb4, SUC_lexicon_tags_9 },
    { 0xd781ccb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b81ccb8, SUC_lexicon_tags_27 },
    { 0xe361ccb8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1201ccbc, SUC_lexicon_tags_12 },
    { 0xb701ccbd, SUC_lexicon_tags_37 },
    { 0xc501ccbe, SUC_lexicon_tags_25 },
    { 0x5fa1ccbe, SUC_lexicon_tags_38 },
    { 0xc6c1ccc0, SUC_lexicon_tags_60 },
    { 0x9e61ccc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1441ccc4, SUC_lexicon_tags_2 },
    { 0x51e1ccc5, SUC_lexicon_tags_37 },
    { 0x3961ccc6, SUC_lexicon_tags_47 },
    { 0xb441ccc7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2681ccca, SUC_lexicon_tags_25 },
    { 0x6aa1ccca, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6821ccd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01ccd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1ccd8, SUC_lexicon_tags_23 },
    { 0xedc1ccd8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa581ccde, SUC_lexicon_tags_15 },
    { 0x46a1ccde, SUC_lexicon_tags_21 },
    { 0xa6a1cce0, SUC_lexicon_tags_17 },
    { 0x4f81cce1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a1cce6, SUC_lexicon_tags_2 },
    { 0x1d01cce7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4b21cce9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace1ccef, SUC_lexicon_tags_28 },
    { 0xe221ccef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6621ccf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee1ccf9, SUC_lexicon_tags_9 },
    { 0x7621ccfa, SUC_lexicon_tags_25 },
    { 0x9e41ccfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4221ccfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a1cd00, SUC_lexicon_tags_45 },
    { 0xcdc1cd01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8421cd04, SUC_lexicon_tags_11 },
    { 0xae41cd05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1641cd09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba1cd0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d41cd13, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca1cd1a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde1cd1d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa1cd21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a1cd24, SUC_lexicon_tags_27 },
    { 0xfda1cd24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8721cd29, SUC_lexicon_tags_5 },
    { 0x441cd2a, SUC_lexicon_tags_83 },
    { 0x5ae1cd29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd781cd2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241cd32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a1cd36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a1cd3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a1cd3f, SUC_lexicon_tags_31 },
    { 0x9ba1cd3f, SUC_lexicon_tags_66 },
    { 0x3861cd41, SUC_lexicon_tags_7 },
    { 0x3061cd42, SUC_lexicon_tags_37 },
    { 0x9441cd41, SUC_lexicon_tags_25 },
    { 0x1941cd44, SUC_lexicon_tags_45 },
    { 0xac41cd45, SUC_lexicon_tags_62 },
    { 0x8ca1cd46, SUC_lexicon_tags_15 },
    { 0xfd81cd46, SUC_lexicon_tags_45 },
    { 0xc021cd48, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc41cd4b, SUC_lexicon_tags_16 },
    { 0xd9a1cd4b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4881cd50, SUC_lexicon_tags_11 },
    { 0x6721cd51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa721cd54, SUC_lexicon_tags_15 },
    { 0xcb61cd55, SUC_lexicon_tags_28 },
    { 0x14e1cd56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e1cd59, SUC_lexicon_tags_1 },
    { 0x4dc1cd5a, SUC_lexicon_tags_25 },
    { 0x6cc1cd5b, SUC_lexicon_tags_55 },
    { 0x91a1cd5c, SUC_lexicon_tags_25 },
    { 0x7b41cd5d, SUC_lexicon_tags_66 },
    { 0x5281cd59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa261cd61, SUC_lexicon_tags_66 },
    { 0x51e1cd62, SUC_lexicon_tags_17 },
    { 0x7b61cd63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa1cd69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca1cd6c, SUC_lexicon_tags_15 },
    { 0x3ca1cd6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc161cd72, SUC_lexicon_tags_53 },
    { 0x41a1cd73, SUC_lexicon_tags_4 },
    { 0x65e1cd74, SUC_lexicon_tags_47 },
    { 0x36c1cd75, SUC_lexicon_tags_15 },
    { 0xed21cd76, SUC_lexicon_tags_18 },
    { 0x3861cd76, SUC_lexicon_tags_8 },
    { 0x5da1cd76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc481cd7a, SUC_lexicon_tags_1 },
    { 0xd1a1cd7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c1cd7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9341cd82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa1cd86, SUC_lexicon_tags_17 },
    { 0xc6a1cd87, SUC_lexicon_tags_45 },
    { 0x96a1cd86, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2361cd90, SUC_lexicon_tags_16 },
    { 0x9a61cd91, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9e21cd93, SUC_lexicon_tags_8 },
    { 0xf6e1cd93, SUC_lexicon_tags_1 },
    { 0x7241cd93, SUC_lexicon_tags_79 },
    { 0x6bc1cd93, SUC_lexicon_tags_31 },
    { 0x5ba1cd97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e1cd9c, SUC_lexicon_tags_7 },
    { 0xa1cd9d, SUC_lexicon_tags_37 },
    { 0x4c41cd9e, SUC_lexicon_tags_7 },
    { 0x22e1cd9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c01cda2, SUC_lexicon_tags_34 },
    { 0xf6a1cda3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a81cdaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b01cdac, SUC_lexicon_tags_23 },
    { 0xe2c1cdac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc461cdb2, SUC_lexicon_tags_14 },
    { 0xf6e1cdb2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1101cdb5, SUC_lexicon_tags_27 },
    { 0x88c1cdb6, SUC_lexicon_tags_48 },
    { 0xe941cdb5, SUC_lexicon_tags_25 },
    { 0x76a1cdb5, SUC_lexicon_tags_26 },
    { 0xa461cdb5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d01cdbe, SUC_lexicon_tags_25 },
    { 0x36a1cdbe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe921cdc7, SUC_lexicon_tags_16 },
    { 0x51e1cdc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1361cdce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f21cdd8, SUC_lexicon_tags_1 },
    { 0x6f81cdd9, SUC_lexicon_tags_25 },
    { 0xe3a1cdda, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc1cde0, SUC_lexicon_tags_17 },
    { 0x86a1cde1, SUC_lexicon_tags_39 },
    { 0xb061cde2, SUC_lexicon_tags_19 },
    { 0xd8c1cde3, SUC_lexicon_tags_2 },
    { 0xb5e1cde2, SUC_lexicon_tags_39 },
    { 0x8d61cde3, SUC_lexicon_tags_15 },
    { 0xd621cde3, SUC_lexicon_tags_2 },
    { 0x9181cde7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe21cdea, SUC_lexicon_tags_15 },
    { 0xaa1cdeb, SUC_lexicon_tags_11 },
    { 0x9f41cdec, SUC_lexicon_tags_6 },
    { 0xa241cdec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e1cdf0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa1cdf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe81cdf8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb101cdfa, SUC_lexicon_tags_2 },
    { 0x8481cdfb, SUC_lexicon_tags_49 },
    { 0x6ae1cdfc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe1ce02, SUC_lexicon_tags_11 },
    { 0xe6e1ce03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x51c1ce05, SUC_lexicon_tags_9 },
    { 0x221ce05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3641ce08, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1281ce0c, SUC_lexicon_tags_49 },
    { 0x1181ce0c, SUC_lexicon_tags_38 },
    { 0xa481ce0e, SUC_lexicon_tags_17 },
    { 0xf141ce0f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5101ce11, SUC_lexicon_tags_12 },
    { 0xcb81ce12, SUC_lexicon_tags_17 },
    { 0x83e1ce12, SUC_lexicon_tags_34 },
    { 0xf941ce14, SUC_lexicon_tags_7 },
    { 0x1ce15, SUC_lexicon_tags_38 },
    { 0xc241ce16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe281ce18, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8c41ce1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde41ce1d, SUC_lexicon_tags_2 },
    { 0x1ec1ce1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x43e1ce20, SUC_lexicon_tags_37 },
    { 0xe6e1ce21, SUC_lexicon_tags_15 },
    { 0x66c1ce22, SUC_lexicon_tags_74 },
    { 0x6561ce23, SUC_lexicon_tags_38 },
    { 0x9f01ce23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf81ce27, SUC_lexicon_tags_13 },
    { 0xd61ce28, SUC_lexicon_tags_2 },
    { 0x5e21ce28, SUC_lexicon_tags_38 },
    { 0xa721ce2a, SUC_lexicon_tags_17 },
    { 0x541ce2b, SUC_lexicon_tags_63 },
    { 0xbc61ce2b, SUC_lexicon_tags_23 },
    { 0xf621ce2d, SUC_lexicon_tags_15 },
    { 0x55e1ce2e, SUC_lexicon_tags_37 },
    { 0x52c1ce27, SUC_lexicon_tags_68 },
    { 0xf581ce30, SUC_lexicon_tags_25 },
    { 0xfdc1ce31, SUC_lexicon_tags_9 },
    { 0x14a1ce29, SUC_lexicon_tags_39 },
    { 0x5f01ce33, SUC_lexicon_tags_15 },
    { 0xefa1ce2b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xed81ce36, SUC_lexicon_tags_43 },
    { 0xdae1ce37, SUC_lexicon_tags_49 },
    { 0x6be1ce38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3441ce3a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x23a1ce3c, SUC_lexicon_tags_47 },
    { 0x1221ce3d, SUC_lexicon_tags_30 },
    { 0xd61ce3d, SUC_lexicon_tags_47 },
    { 0xab81ce3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf861ce41, SUC_lexicon_tags_17 },
    { 0x9681ce42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26e1ce44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3161ce4d, SUC_lexicon_tags_25 },
    { 0x9b81ce4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe481ce51, SUC_lexicon_tags_600 },
    { 0x9ac1ce51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa101ce56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc241ce58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe1ce5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e1ce60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x14a1ce62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc1ce67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc1ce6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce1ce6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c1ce71, SUC_lexicon_tags_37 },
    { 0x7321ce72, SUC_lexicon_tags_14 },
    { 0x79c1ce72, SUC_lexicon_tags_84 },
    { 0x781ce73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd61ce76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1201ce78, SUC_lexicon_tags_11 },
    { 0x73c1ce78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb861ce7c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6381ce82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e61ce85, SUC_lexicon_tags_47 },
    { 0x3601ce86, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa1ce8a, SUC_lexicon_tags_737 },
    { 0x1e21ce8b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e1ce93, SUC_lexicon_tags_0 },
    { 0xbfc1ce93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8741ce96, SUC_lexicon_tags_7 },
    { 0x2b21ce97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf21ce99, SUC_lexicon_tags_1 },
    { 0x281ce9a, SUC_lexicon_tags_37 },
    { 0xc181ce9a, SUC_lexicon_tags_7 },
    { 0x2d81ce9c, SUC_lexicon_tags_63 },
    { 0x30e1ce9c, SUC_lexicon_tags_53 },
    { 0x4ca1ce9c, SUC_lexicon_tags_37 },
    { 0x45c1ce9f, SUC_lexicon_tags_7 },
    { 0xe161cea0, SUC_lexicon_tags_4 },
    { 0xa2c1ce9a, SUC_lexicon_tags_2 },
    { 0x61c1cea2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c61cea8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1561ceaa, SUC_lexicon_tags_7 },
    { 0x3101ceab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8c1cead, SUC_lexicon_tags_1 },
    { 0xa781ceae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x43a1ceb0, SUC_lexicon_tags_11 },
    { 0x3621ceb1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1b81ceb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5901ceb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5001cebe, SUC_lexicon_tags_27 },
    { 0xe701cebf, SUC_lexicon_tags_37 },
    { 0x67e1cebf, SUC_lexicon_tags_47 },
    { 0xce41cebf, SUC_lexicon_tags_72 },
    { 0xe0a1cec0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce1cec5, SUC_lexicon_tags_60 },
    { 0x74a1cec6, SUC_lexicon_tags_1 },
    { 0x5fe1cec7, SUC_lexicon_tags_25 },
    { 0x67c1cec5, SUC_lexicon_tags_16 },
    { 0x9a81cec9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb081cecd, SUC_lexicon_tags_47 },
    { 0xdb81cece, SUC_lexicon_tags_37 },
    { 0x33e1cecd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b41ced5, SUC_lexicon_tags_17 },
    { 0xfb81ced5, SUC_lexicon_tags_4 },
    { 0x28c1ced6, SUC_lexicon_tags_47 },
    { 0x8021ced8, SUC_lexicon_tags_60 },
    { 0x2761ced5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a1cede, SUC_lexicon_tags_25 },
    { 0x8f01cedf, SUC_lexicon_tags_30 },
    { 0xd741cee0, SUC_lexicon_tags_37 },
    { 0x2a41cee1, SUC_lexicon_tags_46 },
    { 0x8f41cede, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c1cee7, SUC_lexicon_tags_47 },
    { 0x4581cee8, SUC_lexicon_tags_28 },
    { 0x9401cee9, SUC_lexicon_tags_6 },
    { 0x8301ceea, SUC_lexicon_tags_11 },
    { 0x52c1ceeb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xeaa1ceed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a1cef1, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xdce1cef3, SUC_lexicon_tags_151 },
    { 0x94e1cef3, SUC_lexicon_tags_62 },
    { 0x3781cef5, SUC_lexicon_tags_30 },
    { 0x9341cef4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc641cefa, SUC_lexicon_tags_32 },
    { 0x4161cefb, SUC_lexicon_tags_23 },
    { 0xa0a1cefc, SUC_lexicon_tags_11 },
    { 0xf201cefd, SUC_lexicon_tags_27 },
    { 0x4f61cefd, SUC_lexicon_tags_9 },
    { 0x6441ceff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb141cf01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7041cf03, SUC_lexicon_tags_3 },
    { 0x7341cf04, SUC_lexicon_tags_72 },
    { 0xfa61cf04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba1cf0e, SUC_lexicon_tags_27 },
    { 0xe801cf0f, SUC_lexicon_tags_0 },
    { 0xd461cf10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8341cf12, SUC_lexicon_tags_82 },
    { 0xfea1cf13, SUC_lexicon_tags_37 },
    { 0x8221cf14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad41cf16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1921cf18, SUC_lexicon_tags_7 },
    { 0xc3a1cf19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf61cf1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41cf1f, SUC_lexicon_tags_25 },
    { 0x9b81cf1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6961cf22, SUC_lexicon_tags_12 },
    { 0xe041cf23, SUC_lexicon_tags_0 },
    { 0xdda1cf23, SUC_lexicon_tags_4 },
    { 0xf101cf22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x301cf28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2921cf2d, SUC_lexicon_tags_17 },
    { 0x6161cf2e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c1cf31, SUC_lexicon_tags_124 },
    { 0x9ba1cf31, SUC_lexicon_tags_17 },
    { 0xdee1cf33, SUC_lexicon_tags_4 },
    { 0xa1e1cf32, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be1cf37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe841cf3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9c1cf3f, SUC_lexicon_tags_4 },
    { 0x3081cf40, SUC_lexicon_tags_25 },
    { 0x7861cf41, SUC_lexicon_tags_37 },
    { 0x2c41cf40, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7e21cf44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce1cf4a, SUC_lexicon_tags_15 },
    { 0xd701cf4b, SUC_lexicon_tags_28 },
    { 0xbf01cf4b, SUC_lexicon_tags_19 },
    { 0x4d01cf4d, SUC_lexicon_tags_7 },
    { 0xe4c1cf4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2601cf51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c1cf5c, SUC_lexicon_tags_0 },
    { 0xd741cf5d, SUC_lexicon_tags_37 },
    { 0xa01cf5e, SUC_lexicon_tags_54 },
    { 0x86a1cf5f, SUC_lexicon_tags_12 },
    { 0x2641cf5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c1cf66, SUC_lexicon_tags_91 },
    { 0xc0c1cf67, SUC_lexicon_tags_37 },
    { 0x8b21cf68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b41cf6b, SUC_lexicon_tags_33 },
    { 0xe541cf6b, SUC_lexicon_tags_0 },
    { 0xd981cf6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea21cf6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a1cf73, SUC_lexicon_tags_7 },
    { 0xa481cf74, SUC_lexicon_tags_2 },
    { 0x6ba1cf75, SUC_lexicon_tags_23 },
    { 0xc01cf73, SUC_lexicon_tags_7 },
    { 0x3e1cf75, SUC_lexicon_tags_6 },
    { 0x15a1cf78, SUC_lexicon_tags_84 },
    { 0x1901cf78, SUC_lexicon_tags_116 },
    { 0x86a1cf78, SUC_lexicon_tags_17 },
    { 0x4081cf77, SUC_lexicon_tags_35 },
    { 0x98c1cf7c, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd521cf80, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a1cf8a, SUC_lexicon_tags_8 },
    { 0x4a1cf8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c1cf8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xec81cf91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc21cf93, SUC_lexicon_tags_23 },
    { 0x9fe1cf94, SUC_lexicon_tags_77 },
    { 0x3f81cf93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe21cf98, SUC_lexicon_tags_25 },
    { 0x4ac1cf99, SUC_lexicon_tags_10 },
    { 0xda41cf99, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd61cf9c, SUC_lexicon_tags_25 },
    { 0xf301cf9d, SUC_lexicon_tags_0 },
    { 0x8d01cf9e, SUC_lexicon_tags_17 },
    { 0x4801cf9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb661cfa2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e1cfa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1cfa8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9361cfab, SUC_lexicon_tags_1 },
    { 0x4a61cfab, SUC_lexicon_tags_63 },
    { 0x27c1cfad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1701cfb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3b01cfb3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f41cfb6, SUC_lexicon_tags_47 },
    { 0xfc01cfb7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc921cfbf, SUC_lexicon_tags_17 },
    { 0x1c41cfc0, SUC_lexicon_tags_61 },
    { 0x9bc1cfc1, SUC_lexicon_tags_0 },
    { 0x6001cfc2, SUC_lexicon_tags_43 },
    { 0x5321cfc2, SUC_lexicon_tags_8 },
    { 0x1a21cfc0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9221cfc6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6141cfca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe81cfcd, SUC_lexicon_tags_17 },
    { 0x1ac1cfcd, SUC_lexicon_tags_60 },
    { 0xece1cfcf, SUC_lexicon_tags_60 },
    { 0xe5c1cfd0, SUC_lexicon_tags_68 },
    { 0x9f01cfd1, SUC_lexicon_tags_25 },
    { 0xb621cfd0, SUC_lexicon_tags_46 },
    { 0x1981cfd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e1cfd7, SUC_lexicon_tags_37 },
    { 0xb8c1cfd7, SUC_lexicon_tags_17 },
    { 0x38a1cfd7, SUC_lexicon_tags_8 },
    { 0x5721cfda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9981cfdd, SUC_lexicon_tags_12 },
    { 0x36c1cfde, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881cfe2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfc61cfe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3801cfe7, SUC_lexicon_tags_47 },
    { 0x5681cfe8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa5a1cfea, SUC_lexicon_tags_33 },
    { 0x3c41cfea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ea1cfed, SUC_lexicon_tags_9 },
    { 0x3f01cfed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a1cff3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf01cff8, SUC_lexicon_tags_63 },
    { 0x75c1cff8, SUC_lexicon_tags_7 },
    { 0xa841cff8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe541cffc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf241cfff, SUC_lexicon_tags_25 },
    { 0xe981d000, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9201d003, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa1d00e, SUC_lexicon_tags_33 },
    { 0xb0c1d00e, SUC_lexicon_tags_87 },
    { 0x3761d010, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc821d014, SUC_lexicon_tags_10 },
    { 0x9bc1d015, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1141d01b, SUC_lexicon_tags_37 },
    { 0xa681d01b, SUC_lexicon_tags_12 },
    { 0x17c1d01c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5481d01f, SUC_lexicon_tags_33 },
    { 0xa321d01f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7c41d022, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1581d026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1d034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1d037, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd421d03e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcde1d040, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea1d043, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec41d048, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c1d04c, SUC_lexicon_tags_17 },
    { 0xd141d04c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba61d051, SUC_lexicon_tags_25 },
    { 0x4c01d051, SUC_lexicon_tags_26 },
    { 0x1d21d052, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5241d055, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1d059, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb841d061, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe1d064, SUC_lexicon_tags_17 },
    { 0x1001d065, SUC_lexicon_tags_14 },
    { 0x24e1d064, SUC_lexicon_tags_15 },
    { 0x7941d067, SUC_lexicon_tags_37 },
    { 0x7401d066, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x97c1d06a, SUC_lexicon_tags_14 },
    { 0x9e61d06b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9681d06e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8941d070, SUC_lexicon_tags_6 },
    { 0x68e1d070, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc1d076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3481d07a, SUC_lexicon_tags_49 },
    { 0x61e1d07b, SUC_lexicon_tags_54 },
    { 0xf221d07c, SUC_lexicon_tags_28 },
    { 0x6b21d07c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c21d07f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd841d084, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe1d091, SUC_lexicon_tags_17 },
    { 0x4a21d092, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e81d095, SUC_lexicon_tags_37 },
    { 0xd7c1d096, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a61d09d, SUC_lexicon_tags_10 },
    { 0x2021d09d, SUC_lexicon_tags_37 },
    { 0xd81d09f, SUC_lexicon_tags_26 },
    { 0xe421d09f, SUC_lexicon_tags_46 },
    { 0xa941d0a1, SUC_lexicon_tags_47 },
    { 0xb8c1d0a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfe1d0a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfa1d0a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d61d0aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89e1d0ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8a81d0ae, SUC_lexicon_tags_37 },
    { 0x34c1d0af, SUC_lexicon_tags_1018 },
    { 0x4b61d0b0, SUC_lexicon_tags_7 },
    { 0x4661d0b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc1d0b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8d61d0b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b21d0b9, SUC_lexicon_tags_37 },
    { 0xdc1d0b9, SUC_lexicon_tags_17 },
    { 0xeaa1d0ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c41d0bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1d0c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a61d0c8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8b21d0ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf301d0cc, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0xa1c1d0ce, SUC_lexicon_tags_2 },
    { 0x6021d0cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a1d0d5, SUC_lexicon_tags_17 },
    { 0x8a01d0d6, SUC_lexicon_tags_15 },
    { 0x6961d0d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a21d0de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a1d0e7, SUC_lexicon_tags_37 },
    { 0x53c1d0e8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e01d0f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a41d0f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c1d0fe, SUC_lexicon_tags_38 },
    { 0x99e1d0ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb521d102, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcec1d104, SUC_lexicon_tags_25 },
    { 0xf81d105, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd281d10b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa1d10d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xffa1d10f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa601d113, SUC_lexicon_tags_47 },
    { 0x461d114, SUC_lexicon_tags_17 },
    { 0xd961d115, SUC_lexicon_tags_68 },
    { 0x6f01d113, SUC_lexicon_tags_0 },
    { 0x301d117, SUC_lexicon_tags_37 },
    { 0x2181d118, SUC_lexicon_tags_0 },
    { 0x9941d119, SUC_lexicon_tags_29 },
    { 0x6441d117, SUC_lexicon_tags_38 },
    { 0xf841d11b, SUC_lexicon_tags_0 },
    { 0x8981d11b, SUC_lexicon_tags_25 },
    { 0x85e1d11a, SUC_lexicon_tags_12 },
    { 0xa801d11a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac1d123, SUC_lexicon_tags_45 },
    { 0xc21d124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1461d126, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5201d12b, SUC_lexicon_tags_4 },
    { 0xc6e1d12c, SUC_lexicon_tags_14 },
    { 0xdc21d12d, SUC_lexicon_tags_6 },
    { 0xaca1d12c, SUC_lexicon_tags_31 },
    { 0xa1d12b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc1d132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa81d134, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa041d136, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e1d13d, SUC_lexicon_tags_62 },
    { 0x9f41d13e, SUC_lexicon_tags_7 },
    { 0xc1a1d13f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94c1d141, SUC_lexicon_tags_19 },
    { 0x6721d142, SUC_lexicon_tags_55 },
    { 0x4701d142, SUC_lexicon_tags_25 },
    { 0x2801d141, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc161d148, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ce1d14a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb741d14d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a1d151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb721d157, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1041d159, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2661d15c, SUC_lexicon_tags_43 },
    { 0x8da1d15d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d41d161, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac1d164, SUC_lexicon_tags_4 },
    { 0xbf21d164, SUC_lexicon_tags_1 },
    { 0x9361d166, SUC_lexicon_tags_32 },
    { 0x3441d164, SUC_lexicon_tags_25 },
    { 0xf321d168, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1081d16e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xab21d171, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb81d177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3201d17b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbc1d17d, SUC_lexicon_tags_7 },
    { 0x8301d17e, SUC_lexicon_tags_1 },
    { 0xe3a1d17f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c1d185, SUC_lexicon_tags_41 },
    { 0xd661d186, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7141d18a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a1d191, SUC_lexicon_tags_27 },
    { 0xe801d192, SUC_lexicon_tags_37 },
    { 0x7141d191, SUC_lexicon_tags_14 },
    { 0x19c1d194, SUC_lexicon_tags_2 },
    { 0x5ee1d195, SUC_lexicon_tags_33 },
    { 0x1c01d191, SUC_lexicon_tags_33 },
    { 0x55a1d197, SUC_lexicon_tags_27 },
    { 0x4ca1d196, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c1d19b, SUC_lexicon_tags_19 },
    { 0xe4e1d19c, SUC_lexicon_tags_60 },
    { 0x9c81d19d, SUC_lexicon_tags_6 },
    { 0xf41d19e, SUC_lexicon_tags_39 },
    { 0xfd01d19f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a61d1a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc1d1aa, SUC_lexicon_tags_63 },
    { 0x4501d1ab, SUC_lexicon_tags_14 },
    { 0x5ca1d1ac, SUC_lexicon_tags_37 },
    { 0x7b41d1ab, SUC_lexicon_tags_19 },
    { 0x51a1d1ae, SUC_lexicon_tags_37 },
    { 0x9b61d1ac, SUC_lexicon_tags_15 },
    { 0x7d01d1b0, SUC_lexicon_tags_37 },
    { 0x4cc1d1b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd541d1b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdce1d1b7, SUC_lexicon_tags_37 },
    { 0xba1d1b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc161d1bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a1d1c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c1d1c5, SUC_lexicon_tags_37 },
    { 0xf3e1d1c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c1d1cd, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x4301d1d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2281d1d2, SUC_lexicon_tags_37 },
    { 0x5521d1d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2841d1d8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c1d1dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb421d1df, SUC_lexicon_tags_11 },
    { 0xa441d1df, SUC_lexicon_tags_37 },
    { 0xd961d1e1, SUC_lexicon_tags_12 },
    { 0xd301d1e2, SUC_lexicon_tags_26 },
    { 0x9c61d1e3, SUC_lexicon_tags_4 },
    { 0x2b41d1e4, SUC_lexicon_tags_62 },
    { 0x64e1d1e4, SUC_lexicon_tags_13 },
    { 0x1b81d1e6, SUC_lexicon_tags_46 },
    { 0x6821d1e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a1d1ea, SUC_lexicon_tags_25 },
    { 0xfa01d1eb, SUC_lexicon_tags_47 },
    { 0x8101d1eb, SUC_lexicon_tags_7 },
    { 0xbf01d1ed, SUC_lexicon_tags_1066 },
    { 0x36a1d1ee, SUC_lexicon_tags_8 },
    { 0x5d41d1ef, SUC_lexicon_tags_60 },
    { 0xe241d1f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6981d1f9, SUC_lexicon_tags_11 },
    { 0x8181d1fa, SUC_lexicon_tags_15 },
    { 0x17e1d1f9, SUC_lexicon_tags_49 },
    { 0x2861d1fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2a21d1fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f41d205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd601d207, SUC_lexicon_tags_45 },
    { 0x2ec1d207, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb141d20c, SUC_lexicon_tags_38 },
    { 0x8641d20d, SUC_lexicon_tags_9 },
    { 0xaf61d20e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa1d211, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5981d214, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1881d216, SUC_lexicon_tags_37 },
    { 0xf3a1d217, SUC_lexicon_tags_1 },
    { 0x6ce1d217, SUC_lexicon_tags_7 },
    { 0x3ee1d219, SUC_lexicon_tags_17 },
    { 0x88c1d219, SUC_lexicon_tags_30 },
    { 0x381d219, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c1d222, SUC_lexicon_tags_15 },
    { 0x9f21d223, SUC_lexicon_tags_15 },
    { 0xe661d224, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4de1d226, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9741d228, SUC_lexicon_tags_10 },
    { 0x8321d228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0a1d22b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7361d22e, SUC_lexicon_tags_1 },
    { 0x7b61d22f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87a1d231, SUC_lexicon_tags_17 },
    { 0x301d232, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5361d237, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c21d239, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e1d240, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c1d245, SUC_lexicon_tags_55 },
    { 0x57c1d245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0a1d248, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa8e1d24a, SUC_lexicon_tags_19 },
    { 0x4121d24b, SUC_lexicon_tags_12 },
    { 0xc8e1d24a, SUC_lexicon_tags_47 },
    { 0x6261d24c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8561d24f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f41d252, SUC_lexicon_tags_37 },
    { 0x6601d252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d41d257, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb141d25b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41d25e, SUC_lexicon_tags_7 },
    { 0x2841d25e, SUC_lexicon_tags_787 },
    { 0, NULL },
    { 0x5c81d261, SUC_lexicon_tags_140 },
    { 0x7661d262, SUC_lexicon_tags_27 },
    { 0x5b01d263, SUC_lexicon_tags_12 },
    { 0xeb81d262, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e81d267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee1d269, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f61d26f, SUC_lexicon_tags_7 },
    { 0xa8e1d270, SUC_lexicon_tags_46 },
    { 0xd5e1d271, SUC_lexicon_tags_7 },
    { 0x6d81d272, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a1d275, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb461d278, SUC_lexicon_tags_31 },
    { 0xaa61d279, SUC_lexicon_tags_14 },
    { 0x7601d27a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1d285, SUC_lexicon_tags_49 },
    { 0xb081d286, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xce81d288, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa901d28b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a1d291, SUC_lexicon_tags_1 },
    { 0xd881d291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2081d294, SUC_lexicon_tags_44 },
    { 0x85c1d295, SUC_lexicon_tags_14 },
    { 0x79e1d294, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb541d29e, SUC_lexicon_tags_37 },
    { 0x3661d29f, SUC_lexicon_tags_11 },
    { 0xf3c1d2a0, SUC_lexicon_tags_66 },
    { 0x7961d29f, SUC_lexicon_tags_9 },
    { 0xd5e1d29f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9a1d2a4, SUC_lexicon_tags_11 },
    { 0x3e01d2a5, SUC_lexicon_tags_19 },
    { 0xde81d2a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e21d2a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4a1d2ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa941d2ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3761d2af, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8521d2b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6561d2b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a81d2ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a1d2bd, SUC_lexicon_tags_37 },
    { 0x6e41d2be, SUC_lexicon_tags_7 },
    { 0xaf01d2bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf61d2c1, SUC_lexicon_tags_63 },
    { 0x88e1d2c2, SUC_lexicon_tags_82 },
    { 0xd2c1d2c2, SUC_lexicon_tags_27 },
    { 0xf601d2c3, SUC_lexicon_tags_7 },
    { 0x35a1d2c3, SUC_lexicon_tags_12 },
    { 0xeea1d2c4, SUC_lexicon_tags_2 },
    { 0x4561d2c7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1241d2c9, SUC_lexicon_tags_47 },
    { 0x4521d2ca, SUC_lexicon_tags_7 },
    { 0xf7e1d2cb, SUC_lexicon_tags_47 },
    { 0x9181d2cc, SUC_lexicon_tags_45 },
    { 0xdc61d2ca, SUC_lexicon_tags_63 },
    { 0x2181d2cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62c1d2d0, SUC_lexicon_tags_27 },
    { 0x2d61d2d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc181d2d3, SUC_lexicon_tags_63 },
    { 0x2781d2d4, SUC_lexicon_tags_11 },
    { 0x51a1d2d5, SUC_lexicon_tags_17 },
    { 0x9e81d2d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe21d2da, SUC_lexicon_tags_29 },
    { 0x37e1d2db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc21d2dd, SUC_lexicon_tags_10 },
    { 0xf1e1d2de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede1d2e2, SUC_lexicon_tags_262 },
    { 0x4d01d2e3, SUC_lexicon_tags_21 },
    { 0x7541d2e4, SUC_lexicon_tags_60 },
    { 0xafe1d2e5, SUC_lexicon_tags_16 },
    { 0xb021d2e6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a1d2ea, SUC_lexicon_tags_2 },
    { 0xd4c1d2ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac81d2f0, SUC_lexicon_tags_35 },
    { 0x8aa1d2f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee41d2f3, SUC_lexicon_tags_43 },
    { 0xad01d2f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc1d2f9, SUC_lexicon_tags_49 },
    { 0x52e1d2fa, SUC_lexicon_tags_45 },
    { 0xe981d2fb, SUC_lexicon_tags_25 },
    { 0x9901d2fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7641d2fe, SUC_lexicon_tags_2 },
    { 0xf2a1d2fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa1d302, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa741d304, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde41d306, SUC_lexicon_tags_32 },
    { 0xb881d306, SUC_lexicon_tags_35 },
    { 0xc2a1d308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97c1d30a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf181d30c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2301d311, SUC_lexicon_tags_14 },
    { 0x7de1d311, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe1d318, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf5e1d31a, SUC_lexicon_tags_14 },
    { 0x1c1d31b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a1d31f, SUC_lexicon_tags_17 },
    { 0xb81d320, SUC_lexicon_tags_14 },
    { 0x9061d321, SUC_lexicon_tags_27 },
    { 0xaa41d320, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf121d324, SUC_lexicon_tags_8 },
    { 0xb981d325, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x60a1d327, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e41d329, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac1d332, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8381d337, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5b61d339, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2401d33b, SUC_lexicon_tags_11 },
    { 0xe221d33c, SUC_lexicon_tags_17 },
    { 0x7c01d33d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f21d33f, SUC_lexicon_tags_82 },
    { 0xfa61d33f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1d344, SUC_lexicon_tags_8 },
    { 0xfb61d345, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xda41d348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bc1d34a, SUC_lexicon_tags_1 },
    { 0xd01d34b, SUC_lexicon_tags_33 },
    { 0x8621d34c, SUC_lexicon_tags_16 },
    { 0xedc1d34c, SUC_lexicon_tags_37 },
    { 0x2181d34e, SUC_lexicon_tags_19 },
    { 0x8761d34a, SUC_lexicon_tags_29 },
    { 0xe01d350, SUC_lexicon_tags_11 },
    { 0x1221d351, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b61d35b, SUC_lexicon_tags_11 },
    { 0x93a1d35c, SUC_lexicon_tags_66 },
    { 0x37e1d35d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x94e1d35f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc1d36a, SUC_lexicon_tags_1 },
    { 0x5b81d36b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc881d372, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a1d375, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a1d37b, SUC_lexicon_tags_0 },
    { 0xce61d37c, SUC_lexicon_tags_17 },
    { 0x63e1d37b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9221d380, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b61d382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e1d387, SUC_lexicon_tags_17 },
    { 0x5d21d388, SUC_lexicon_tags_7 },
    { 0x4641d389, SUC_lexicon_tags_63 },
    { 0x79e1d38a, SUC_lexicon_tags_25 },
    { 0xf481d38a, SUC_lexicon_tags_33 },
    { 0x5861d38c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a81d38e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5c81d390, SUC_lexicon_tags_31 },
    { 0x6aa1d390, SUC_lexicon_tags_84 },
    { 0xb981d391, SUC_lexicon_tags_27 },
    { 0x94e1d390, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c1d396, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9481d39a, SUC_lexicon_tags_25 },
    { 0xf021d39a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac1d39e, SUC_lexicon_tags_33 },
    { 0x8161d39f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e1d3a2, SUC_lexicon_tags_25 },
    { 0xb001d3a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf41d3a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd501d3ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9f81d3ad, SUC_lexicon_tags_2 },
    { 0xb141d3ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e1d3b1, SUC_lexicon_tags_17 },
    { 0x9241d3b2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2da1d3b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5101d3b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a01d3b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9961d3bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8a1d3bd, SUC_lexicon_tags_49 },
    { 0xbd41d3be, SUC_lexicon_tags_57 },
    { 0x96a1d3bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c1d3c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a1d3c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5721d3cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7c01d3ce, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c01d3d2, SUC_lexicon_tags_11 },
    { 0x20c1d3d3, SUC_lexicon_tags_43 },
    { 0x1941d3d2, SUC_lexicon_tags_11 },
    { 0xb001d3d5, SUC_lexicon_tags_66 },
    { 0xc641d3d4, SUC_lexicon_tags_15 },
    { 0x2361d3d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e1d3dc, SUC_lexicon_tags_23 },
    { 0x9641d3dd, SUC_lexicon_tags_7 },
    { 0x57e1d3de, SUC_lexicon_tags_37 },
    { 0xc7a1d3dc, SUC_lexicon_tags_30 },
    { 0x1101d3e0, SUC_lexicon_tags_23 },
    { 0x4481d3e1, SUC_lexicon_tags_2 },
    { 0xc2a1d3e0, SUC_lexicon_tags_1 },
    { 0x9e41d3dc, SUC_lexicon_tags_82 },
    { 0xf261d3e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ac1d3e6, SUC_lexicon_tags_60 },
    { 0xa961d3e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2a41d3e9, SUC_lexicon_tags_37 },
    { 0xb401d3ea, SUC_lexicon_tags_17 },
    { 0x2481d3eb, SUC_lexicon_tags_45 },
    { 0x40c1d3ea, SUC_lexicon_tags_7 },
    { 0x3da1d3ed, SUC_lexicon_tags_2 },
    { 0x3e41d3ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe1d3f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b01d3f4, SUC_lexicon_tags_63 },
    { 0xdbc1d3f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9041d3f7, SUC_lexicon_tags_37 },
    { 0x1d61d3f8, SUC_lexicon_tags_17 },
    { 0xc0e1d3f8, SUC_lexicon_tags_2 },
    { 0xdea1d3fa, SUC_lexicon_tags_12 },
    { 0xea61d3f9, SUC_lexicon_tags_49 },
    { 0xec81d3fc, SUC_lexicon_tags_6 },
    { 0x11e1d3fd, SUC_lexicon_tags_15 },
    { 0x1f01d3f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1081d403, SUC_lexicon_tags_39 },
    { 0xc4c1d404, SUC_lexicon_tags_17 },
    { 0x6e81d405, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d21d40d, SUC_lexicon_tags_11 },
    { 0xa1e1d40e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de1d412, SUC_lexicon_tags_7 },
    { 0xdf01d413, SUC_lexicon_tags_0 },
    { 0x1101d414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc681d418, SUC_lexicon_tags_17 },
    { 0x7881d419, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3061d41c, SUC_lexicon_tags_12 },
    { 0x23e1d41c, SUC_lexicon_tags_38 },
    { 0x5041d41e, SUC_lexicon_tags_37 },
    { 0x5de1d41e, SUC_lexicon_tags_37 },
    { 0xdd61d420, SUC_lexicon_tags_82 },
    { 0xa7e1d420, SUC_lexicon_tags_56 },
    { 0x1fc1d420, SUC_lexicon_tags_15 },
    { 0xf741d422, SUC_lexicon_tags_9 },
    { 0xc361d424, SUC_lexicon_tags_39 },
    { 0x68c1d423, SUC_lexicon_tags_85 },
    { 0xfa41d426, SUC_lexicon_tags_32 },
    { 0xa201d426, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb61d429, SUC_lexicon_tags_17 },
    { 0xade1d42a, SUC_lexicon_tags_30 },
    { 0xf861d42b, SUC_lexicon_tags_11 },
    { 0x41e1d42b, SUC_lexicon_tags_19 },
    { 0xa621d42d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e1d431, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc81d433, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xac41d435, SUC_lexicon_tags_0 },
    { 0xd001d436, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be1d43e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce1d441, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x3601d444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd781d447, SUC_lexicon_tags_1 },
    { 0x8201d448, SUC_lexicon_tags_47 },
    { 0x7cc1d449, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf081d44b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9021d44d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf81d453, SUC_lexicon_tags_87 },
    { 0x76e1d454, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e1d457, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9761d45d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8001d469, SUC_lexicon_tags_26 },
    { 0xbb81d469, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82a1d46c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7841d46f, SUC_lexicon_tags_27 },
    { 0x4dc1d470, SUC_lexicon_tags_63 },
    { 0x6861d471, SUC_lexicon_tags_34 },
    { 0x3381d470, SUC_lexicon_tags_15 },
    { 0x3061d470, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc261d476, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5d41d478, SUC_lexicon_tags_29 },
    { 0x2761d479, SUC_lexicon_tags_17 },
    { 0x8781d478, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe1d47e, SUC_lexicon_tags_27 },
    { 0x9fa1d47e, SUC_lexicon_tags_17 },
    { 0xae1d480, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1cc1d482, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7121d486, SUC_lexicon_tags_14 },
    { 0xa961d487, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82e1d489, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a1d48e, SUC_lexicon_tags_40 },
    { 0xf9a1d48f, SUC_lexicon_tags_26 },
    { 0xc4a1d48f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe1d492, SUC_lexicon_tags_37 },
    { 0x5a81d492, SUC_lexicon_tags_46 },
    { 0x2101d493, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc821d498, SUC_lexicon_tags_26 },
    { 0x6fe1d498, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a1d4a2, SUC_lexicon_tags_47 },
    { 0x5861d4a3, SUC_lexicon_tags_49 },
    { 0x5381d4a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc41d4a6, SUC_lexicon_tags_37 },
    { 0xa541d4a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5681d4b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc401d4b5, SUC_lexicon_tags_37 },
    { 0xe41d4b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b81d4ba, SUC_lexicon_tags_49 },
    { 0xce1d4ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc21d4bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2341d4c3, SUC_lexicon_tags_63 },
    { 0x9ee1d4c4, SUC_lexicon_tags_40 },
    { 0xd381d4c5, SUC_lexicon_tags_15 },
    { 0x7881d4c5, SUC_lexicon_tags_29 },
    { 0x8fa1d4c6, SUC_lexicon_tags_9 },
    { 0x5461d4c5, SUC_lexicon_tags_37 },
    { 0x41a1d4c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfd41d4cb, SUC_lexicon_tags_26 },
    { 0x5501d4cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9741d4ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd981d4d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e1d4d6, SUC_lexicon_tags_25 },
    { 0xb21d4d7, SUC_lexicon_tags_15 },
    { 0x1381d4d8, SUC_lexicon_tags_27 },
    { 0x9fc1d4d9, SUC_lexicon_tags_47 },
    { 0xd221d4d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f21d4df, SUC_lexicon_tags_13 },
    { 0xc21d4df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e81d4e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9681d4e7, SUC_lexicon_tags_27 },
    { 0x4441d4e7, SUC_lexicon_tags_25 },
    { 0xf021d4e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf41d4eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8661d4ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba1d4f5, SUC_lexicon_tags_47 },
    { 0x35e1d4f6, SUC_lexicon_tags_0 },
    { 0x5201d4f7, SUC_lexicon_tags_17 },
    { 0x2fc1d4f7, SUC_lexicon_tags_3 },
    { 0xe6a1d4f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e1d4fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a1d501, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf461d504, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb901d50f, SUC_lexicon_tags_31 },
    { 0x12a1d510, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada1d514, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0xc181d516, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6221d51b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d61d51d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1d520, SUC_lexicon_tags_25 },
    { 0xa941d521, SUC_lexicon_tags_12 },
    { 0x3fa1d522, SUC_lexicon_tags_2 },
    { 0xdd01d523, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d41d526, SUC_lexicon_tags_29 },
    { 0x51e1d526, SUC_lexicon_tags_1 },
    { 0xb8c1d528, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e1d52b, SUC_lexicon_tags_37 },
    { 0x5821d52c, SUC_lexicon_tags_55 },
    { 0xdd61d52d, SUC_lexicon_tags_45 },
    { 0xf161d52e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4881d530, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd6a1d532, SUC_lexicon_tags_28 },
    { 0x41e1d532, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa81d53c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a1d542, SUC_lexicon_tags_29 },
    { 0x8801d543, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4481d545, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c61d547, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc181d54b, SUC_lexicon_tags_1 },
    { 0x5ac1d54b, SUC_lexicon_tags_15 },
    { 0x5fe1d54d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb961d551, SUC_lexicon_tags_1 },
    { 0x9a01d551, SUC_lexicon_tags_49 },
    { 0x1861d553, SUC_lexicon_tags_2 },
    { 0xe041d554, SUC_lexicon_tags_10 },
    { 0x55c1d551, SUC_lexicon_tags_87 },
    { 0x9501d556, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5681d55c, SUC_lexicon_tags_11 },
    { 0xbe61d55d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc761d55f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27c1d561, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3921d568, SUC_lexicon_tags_39 },
    { 0xe7c1d569, SUC_lexicon_tags_14 },
    { 0x8701d56a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x92c1d56c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd81d56e, SUC_lexicon_tags_49 },
    { 0xb4a1d56f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57a1d571, SUC_lexicon_tags_19 },
    { 0x3e41d572, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x16a1d574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda1d57d, SUC_lexicon_tags_37 },
    { 0xc9e1d57d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c1d583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6641d585, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e61d58c, SUC_lexicon_tags_11 },
    { 0xb4e1d58c, SUC_lexicon_tags_25 },
    { 0x2d61d58e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb81d593, SUC_lexicon_tags_19 },
    { 0x2d01d594, SUC_lexicon_tags_2 },
    { 0xd7c1d593, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1101d59b, SUC_lexicon_tags_23 },
    { 0xe121d59c, SUC_lexicon_tags_11 },
    { 0x97e1d59c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe101d5a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5561d5a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e61d5a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2881d5aa, SUC_lexicon_tags_1 },
    { 0x5e1d5ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf41d5ad, SUC_lexicon_tags_49 },
    { 0xda41d5ae, SUC_lexicon_tags_14 },
    { 0x1101d5ae, SUC_lexicon_tags_37 },
    { 0x8fe1d5b0, SUC_lexicon_tags_49 },
    { 0xef21d5b1, SUC_lexicon_tags_25 },
    { 0x5901d5b2, SUC_lexicon_tags_17 },
    { 0x561d5b2, SUC_lexicon_tags_6 },
    { 0x121d5ae, SUC_lexicon_tags_45 },
    { 0x5a21d5b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed61d5ba, SUC_lexicon_tags_15 },
    { 0xc2e1d5bb, SUC_lexicon_tags_12 },
    { 0xefc1d5bb, SUC_lexicon_tags_63 },
    { 0x821d5bd, SUC_lexicon_tags_39 },
    { 0x95e1d5be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2701d5c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf101d5c6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8e01d5c8, SUC_lexicon_tags_12 },
    { 0xa281d5c9, SUC_lexicon_tags_27 },
    { 0x3161d5c8, SUC_lexicon_tags_7 },
    { 0x86c1d5c9, SUC_lexicon_tags_4 },
    { 0x9821d5cc, SUC_lexicon_tags_15 },
    { 0x6da1d5cd, SUC_lexicon_tags_17 },
    { 0xde81d5cb, SUC_lexicon_tags_45 },
    { 0x2da1d5cc, SUC_lexicon_tags_17 },
    { 0xcd41d5d0, SUC_lexicon_tags_8 },
    { 0x54c1d5d0, SUC_lexicon_tags_124 },
    { 0x56c1d5d1, SUC_lexicon_tags_11 },
    { 0x4781d5d0, SUC_lexicon_tags_14 },
    { 0xb921d5d4, SUC_lexicon_tags_37 },
    { 0x4721d5d3, SUC_lexicon_tags_30 },
    { 0xb8a1d5d6, SUC_lexicon_tags_27 },
    { 0x2461d5cb, SUC_lexicon_tags_14 },
    { 0xed41d5d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4c1d5da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cc1d5dc, SUC_lexicon_tags_11 },
    { 0x3421d5dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc181d5e2, SUC_lexicon_tags_25 },
    { 0x8b01d5e3, SUC_lexicon_tags_14 },
    { 0xc6c1d5e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x721d5e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba41d5ea, SUC_lexicon_tags_14 },
    { 0x3ee1d5ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e1d5f2, SUC_lexicon_tags_17 },
    { 0xd0c1d5f2, SUC_lexicon_tags_6 },
    { 0x5761d5f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c1d5fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4e1d5fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3481d5fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a1d604, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc181d60a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a1d612, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c1d618, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0a1d61a, SUC_lexicon_tags_60 },
    { 0xe3e1d61b, SUC_lexicon_tags_12 },
    { 0x9f01d61b, SUC_lexicon_tags_37 },
    { 0x5581d61c, SUC_lexicon_tags_11 },
    { 0xd281d61e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e01d622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8a1d624, SUC_lexicon_tags_30 },
    { 0xe3e1d624, SUC_lexicon_tags_0 },
    { 0x3c61d626, SUC_lexicon_tags_45 },
    { 0x2741d626, SUC_lexicon_tags_3 },
    { 0x4041d628, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc21d62c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c1d630, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca1d633, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae1d637, SUC_lexicon_tags_295 },
    { 0xfa61d637, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8b41d63a, SUC_lexicon_tags_1 },
    { 0xfe1d63b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b41d63e, SUC_lexicon_tags_10 },
    { 0x70c1d63e, SUC_lexicon_tags_7 },
    { 0x55a1d640, SUC_lexicon_tags_45 },
    { 0x3a1d63e, SUC_lexicon_tags_55 },
    { 0xd581d640, SUC_lexicon_tags_15 },
    { 0xd481d63f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7981d64a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f21d64f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e1d656, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf981d658, SUC_lexicon_tags_27 },
    { 0xdba1d659, SUC_lexicon_tags_15 },
    { 0x99a1d65a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc41d661, SUC_lexicon_tags_45 },
    { 0x4a21d661, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61d665, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe161d667, SUC_lexicon_tags_38 },
    { 0xb021d668, SUC_lexicon_tags_174 },
    { 0x8de1d669, SUC_lexicon_tags_31 },
    { 0xadc1d667, SUC_lexicon_tags_21 },
    { 0x2b01d669, SUC_lexicon_tags_27 },
    { 0x7fe1d66c, SUC_lexicon_tags_37 },
    { 0xdb01d66a, SUC_lexicon_tags_4 },
    { 0xb361d66e, SUC_lexicon_tags_25 },
    { 0xf7e1d66f, SUC_lexicon_tags_84 },
    { 0xf801d66e, SUC_lexicon_tags_37 },
    { 0xa701d671, SUC_lexicon_tags_55 },
    { 0xf601d672, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74a1d674, SUC_lexicon_tags_87 },
    { 0x3f21d675, SUC_lexicon_tags_21 },
    { 0xf21d675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e1d67a, SUC_lexicon_tags_11 },
    { 0x46e1d67a, SUC_lexicon_tags_11 },
    { 0xeca1d67c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7bc1d67e, SUC_lexicon_tags_61 },
    { 0x8ec1d67f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e1d684, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc801d686, SUC_lexicon_tags_7 },
    { 0x40e1d687, SUC_lexicon_tags_6 },
    { 0x5d41d687, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf61d68a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd01d68d, SUC_lexicon_tags_15 },
    { 0xaa1d68e, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x5181d690, SUC_lexicon_tags_12 },
    { 0x30e1d691, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa061d694, SUC_lexicon_tags_4 },
    { 0xa7c1d695, SUC_lexicon_tags_38 },
    { 0xffc1d694, SUC_lexicon_tags_9 },
    { 0x8741d694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2521d69b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef01d6a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a1d6a5, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x7901d6a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea41d6ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37a1d6af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e1d6b4, SUC_lexicon_tags_85 },
    { 0xbaa1d6b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb061d6b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e1d6be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2941d6c0, SUC_lexicon_tags_25 },
    { 0x9181d6c1, SUC_lexicon_tags_17 },
    { 0x1ce1d6c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b21d6c8, SUC_lexicon_tags_7 },
    { 0x4581d6c8, SUC_lexicon_tags_129 },
    { 0xa181d6c9, SUC_lexicon_tags_37 },
    { 0xfd21d6cb, SUC_lexicon_tags_17 },
    { 0x2841d6cc, SUC_lexicon_tags_32 },
    { 0x2da1d6cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe621d6d4, SUC_lexicon_tags_63 },
    { 0xb8e1d6d5, SUC_lexicon_tags_70 },
    { 0xc7c1d6d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d81d6d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d6dc, SUC_lexicon_tags_12 },
    { 0xae81d6dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc1d6e0, SUC_lexicon_tags_0 },
    { 0x961d6e1, SUC_lexicon_tags_12 },
    { 0xf41d6e1, SUC_lexicon_tags_2 },
    { 0x4a01d6e3, SUC_lexicon_tags_14 },
    { 0xbc21d6e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb841d6e8, SUC_lexicon_tags_60 },
    { 0x2ea1d6e9, SUC_lexicon_tags_83 },
    { 0xfaa1d6e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3ac1d6ec, SUC_lexicon_tags_29 },
    { 0x6c81d6ed, SUC_lexicon_tags_21 },
    { 0xc8e1d6ec, SUC_lexicon_tags_25 },
    { 0x4de1d6ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c1d6f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7861d6f4, SUC_lexicon_tags_12 },
    { 0x8e21d6f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7341d6f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe6a1d6fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba1d6fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8301d703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd61d707, SUC_lexicon_tags_33 },
    { 0x76e1d707, SUC_lexicon_tags_25 },
    { 0x201d709, SUC_lexicon_tags_5 },
    { 0x6d81d70a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a1d70f, SUC_lexicon_tags_17 },
    { 0x501d710, SUC_lexicon_tags_26 },
    { 0x2f61d710, SUC_lexicon_tags_1 },
    { 0x3c41d712, SUC_lexicon_tags_63 },
    { 0xc101d70f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a1d716, SUC_lexicon_tags_33 },
    { 0x9161d717, SUC_lexicon_tags_26 },
    { 0xab01d716, SUC_lexicon_tags_6 },
    { 0xb061d716, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e1d71b, SUC_lexicon_tags_2 },
    { 0x3d81d71c, SUC_lexicon_tags_0 },
    { 0x7ea1d71c, SUC_lexicon_tags_17 },
    { 0x48c1d71e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f61d720, SUC_lexicon_tags_32 },
    { 0x3dc1d721, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb421d724, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e1d728, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6801d72a, SUC_lexicon_tags_43 },
    { 0xbce1d72a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb981d732, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac1d735, SUC_lexicon_tags_11 },
    { 0xf121d736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3421d73a, SUC_lexicon_tags_1 },
    { 0xfbc1d73b, SUC_lexicon_tags_46 },
    { 0xfe41d73b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d73f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c1d743, SUC_lexicon_tags_18 },
    { 0x7fa1d744, SUC_lexicon_tags_32 },
    { 0xc4a1d745, SUC_lexicon_tags_1 },
    { 0xe8c1d744, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781d74f, SUC_lexicon_tags_2 },
    { 0x2341d74f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x721d752, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6141d75d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9e1d75f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9521d766, SUC_lexicon_tags_21 },
    { 0x20e1d767, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c1d76b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e1d76f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4a1d771, SUC_lexicon_tags_26 },
    { 0xb5c1d772, SUC_lexicon_tags_11 },
    { 0xd221d772, SUC_lexicon_tags_1 },
    { 0x54e1d773, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4681d77c, SUC_lexicon_tags_15 },
    { 0x5361d77d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e1d782, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd41d785, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf81d788, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4c21d78a, SUC_lexicon_tags_13 },
    { 0x1dc1d78b, SUC_lexicon_tags_8 },
    { 0x8181d78c, SUC_lexicon_tags_7 },
    { 0x8d41d78a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6981d78f, SUC_lexicon_tags_25 },
    { 0xf761d790, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa801d794, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcb41d796, SUC_lexicon_tags_8 },
    { 0x5be1d797, SUC_lexicon_tags_7 },
    { 0xb081d798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5501d79a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac01d7a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a41d7a5, SUC_lexicon_tags_6 },
    { 0x7cc1d7a5, SUC_lexicon_tags_62 },
    { 0x9b01d7a5, SUC_lexicon_tags_1 },
    { 0x84c1d7a8, SUC_lexicon_tags_4 },
    { 0xeec1d7a9, SUC_lexicon_tags_15 },
    { 0x4a01d7aa, SUC_lexicon_tags_27 },
    { 0xa9e1d7a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e1d7af, SUC_lexicon_tags_15 },
    { 0x7f01d7b0, SUC_lexicon_tags_11 },
    { 0x5e01d7b1, SUC_lexicon_tags_12 },
    { 0x8421d7b0, SUC_lexicon_tags_60 },
    { 0x84e1d7b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91c1d7b5, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x95a1d7b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d01d7b9, SUC_lexicon_tags_6 },
    { 0xcde1d7ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61d7bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a41d7c1, SUC_lexicon_tags_37 },
    { 0xd2a1d7c2, SUC_lexicon_tags_7 },
    { 0xaaa1d7c2, SUC_lexicon_tags_17 },
    { 0x4a41d7c4, SUC_lexicon_tags_40 },
    { 0xdf01d7c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb241d7c9, SUC_lexicon_tags_38 },
    { 0x9561d7c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a61d7cf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea1d7d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e1d7db, SUC_lexicon_tags_9 },
    { 0x2e01d7dc, SUC_lexicon_tags_17 },
    { 0x3d41d7dd, SUC_lexicon_tags_49 },
    { 0x7c61d7dd, SUC_lexicon_tags_15 },
    { 0x7fa1d7df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe721d7e3, SUC_lexicon_tags_6 },
    { 0x501d7e4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea1d7ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7401d7ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e01d7f1, SUC_lexicon_tags_12 },
    { 0xccc1d7f2, SUC_lexicon_tags_25 },
    { 0xbba1d7f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8321d7f7, SUC_lexicon_tags_16 },
    { 0xd4e1d7f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe701d7fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701d800, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c1d804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe881d808, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241d80c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98c1d80e, SUC_lexicon_tags_126 },
    { 0xbb01d80f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2481d815, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa421d817, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae1d81c, SUC_lexicon_tags_29 },
    { 0xd061d81c, SUC_lexicon_tags_47 },
    { 0xd1c1d81e, SUC_lexicon_tags_148 },
    { 0x76a1d81e, SUC_lexicon_tags_65 },
    { 0x1701d81d, SUC_lexicon_tags_275 },
    { 0xcc61d821, SUC_lexicon_tags_17 },
    { 0x63c1d822, SUC_lexicon_tags_7 },
    { 0xa1d821, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c1d826, SUC_lexicon_tags_2 },
    { 0xbf01d827, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca1d82a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e1d830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a1d835, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc601d83a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2461d83d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e01d841, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ba1d843, SUC_lexicon_tags_39 },
    { 0x2ac1d844, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c61d849, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c1d84f, SUC_lexicon_tags_12 },
    { 0x5ce1d850, SUC_lexicon_tags_15 },
    { 0x9b21d851, SUC_lexicon_tags_25 },
    { 0xef21d850, SUC_lexicon_tags_7 },
    { 0x9e61d852, SUC_lexicon_tags_37 },
    { 0x1821d854, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad81d85a, SUC_lexicon_tags_35 },
    { 0xf701d85a, SUC_lexicon_tags_15 },
    { 0x5361d85c, SUC_lexicon_tags_7 },
    { 0x5481d85d, SUC_lexicon_tags_3 },
    { 0x1e41d85e, SUC_lexicon_tags_25 },
    { 0x6881d85f, SUC_lexicon_tags_11 },
    { 0x5781d85c, SUC_lexicon_tags_60 },
    { 0x2c1d861, SUC_lexicon_tags_17 },
    { 0xe61d861, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b61d865, SUC_lexicon_tags_9 },
    { 0x2f21d865, SUC_lexicon_tags_12 },
    { 0xa5e1d867, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc1d86d, SUC_lexicon_tags_45 },
    { 0xf561d86e, SUC_lexicon_tags_1 },
    { 0x2061d86f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x201d872, SUC_lexicon_tags_2 },
    { 0xaea1d873, SUC_lexicon_tags_17 },
    { 0x3a81d872, SUC_lexicon_tags_60 },
    { 0x201d874, SUC_lexicon_tags_160 },
    { 0xaa61d876, SUC_lexicon_tags_19 },
    { 0xc3a1d873, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4661d879, SUC_lexicon_tags_112 },
    { 0xb2c1d87a, SUC_lexicon_tags_11 },
    { 0x4fa1d87b, SUC_lexicon_tags_23 },
    { 0xc341d87b, SUC_lexicon_tags_22 },
    { 0x23e1d87d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c1d880, SUC_lexicon_tags_14 },
    { 0xf4a1d880, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa041d884, SUC_lexicon_tags_2 },
    { 0xe661d884, SUC_lexicon_tags_15 },
    { 0x81a1d886, SUC_lexicon_tags_14 },
    { 0xf0e1d884, SUC_lexicon_tags_1 },
    { 0x6481d888, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3621d88a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac1d894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf521d897, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e41d89a, SUC_lexicon_tags_23 },
    { 0x8f41d89b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e61d89f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a81d8a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5341d8a4, SUC_lexicon_tags_10 },
    { 0xe421d8a5, SUC_lexicon_tags_37 },
    { 0x3581d8a6, SUC_lexicon_tags_25 },
    { 0x45c1d8a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb961d8a9, SUC_lexicon_tags_2 },
    { 0x4361d8a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf01d8b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f01d8b5, SUC_lexicon_tags_26 },
    { 0xbda1d8b6, SUC_lexicon_tags_11 },
    { 0x4e41d8b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x48a1d8b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15a1d8bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1221d8c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97a1d8c7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab81d8cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5341d8ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e1d8d4, SUC_lexicon_tags_63 },
    { 0x89e1d8d5, SUC_lexicon_tags_16 },
    { 0x2ac1d8d6, SUC_lexicon_tags_21 },
    { 0xc9c1d8d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe01d8db, SUC_lexicon_tags_17 },
    { 0xca41d8db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c1d8e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c61d8f0, SUC_lexicon_tags_12 },
    { 0xbaa1d8f1, SUC_lexicon_tags_7 },
    { 0xe061d8f1, SUC_lexicon_tags_25 },
    { 0x2e41d8f3, SUC_lexicon_tags_69 },
    { 0xcf21d8f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f61d8f7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5981d900, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c21d902, SUC_lexicon_tags_14 },
    { 0x14a1d902, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba1d907, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca1d90b, SUC_lexicon_tags_17 },
    { 0x55a1d90b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4621d910, SUC_lexicon_tags_31 },
    { 0xfca1d910, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x881d914, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3001d916, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7141d918, SUC_lexicon_tags_12 },
    { 0x4221d919, SUC_lexicon_tags_30 },
    { 0x3d41d91a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941d921, SUC_lexicon_tags_37 },
    { 0xcc81d922, SUC_lexicon_tags_15 },
    { 0xac1d923, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4101d925, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1201d928, SUC_lexicon_tags_29 },
    { 0x8e1d928, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2041d92f, SUC_lexicon_tags_7 },
    { 0x2341d92f, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61d936, SUC_lexicon_tags_63 },
    { 0x34c1d937, SUC_lexicon_tags_14 },
    { 0x5141d937, SUC_lexicon_tags_3 },
    { 0xbd81d939, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1361d93e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f61d941, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf21d944, SUC_lexicon_tags_25 },
    { 0xe61d945, SUC_lexicon_tags_15 },
    { 0xc781d945, SUC_lexicon_tags_7 },
    { 0xf7c1d947, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e1d94b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e01d94d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fc1d94f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c1d955, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec1d957, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1e41d959, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xab21d95c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xed21d95f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe581d962, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6441d965, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe1d96e, SUC_lexicon_tags_29 },
    { 0x4421d96e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c41d972, SUC_lexicon_tags_15 },
    { 0x7721d973, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d21d977, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a1d97f, SUC_lexicon_tags_13 },
    { 0x1a81d980, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea41d982, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4161d984, SUC_lexicon_tags_7 },
    { 0xc7e1d984, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f21d98a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53c1d98c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8441d98e, SUC_lexicon_tags_29 },
    { 0x8aa1d98e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521d993, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c21d996, SUC_lexicon_tags_12 },
    { 0xd01d997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84c1d999, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd01d99c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a1d9a1, SUC_lexicon_tags_19 },
    { 0x50e1d9a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e01d9a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de1d9ad, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3581d9b0, SUC_lexicon_tags_37 },
    { 0xed21d9b0, SUC_lexicon_tags_25 },
    { 0xda81d9b1, SUC_lexicon_tags_40 },
    { 0x381d9b1, SUC_lexicon_tags_43 },
    { 0x7241d9b4, SUC_lexicon_tags_55 },
    { 0xc061d9b4, SUC_lexicon_tags_25 },
    { 0x19a1d9b6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b81d9be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f01d9c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9281d9c6, SUC_lexicon_tags_12 },
    { 0x6f41d9c7, SUC_lexicon_tags_47 },
    { 0x5701d9c7, SUC_lexicon_tags_26 },
    { 0x8e61d9c7, SUC_lexicon_tags_9 },
    { 0x8c01d9ca, SUC_lexicon_tags_16 },
    { 0xc341d9cb, SUC_lexicon_tags_11 },
    { 0x7401d9ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb641d9d0, SUC_lexicon_tags_14 },
    { 0x861d9d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d01d9d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc401d9d5, SUC_lexicon_tags_43 },
    { 0x83c1d9d6, SUC_lexicon_tags_25 },
    { 0x4901d9d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1d9dc, SUC_lexicon_tags_25 },
    { 0x3181d9dd, SUC_lexicon_tags_11 },
    { 0x3981d9dc, SUC_lexicon_tags_15 },
    { 0x9c1d9df, SUC_lexicon_tags_38 },
    { 0x2aa1d9df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc1d9e3, SUC_lexicon_tags_60 },
    { 0xd7c1d9e3, SUC_lexicon_tags_39 },
    { 0x3ee1d9e5, SUC_lexicon_tags_0 },
    { 0x83a1d9e4, SUC_lexicon_tags_12 },
    { 0xecc1d9e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81d9eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfda1d9ed, SUC_lexicon_tags_11 },
    { 0x6461d9ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5401d9f7, SUC_lexicon_tags_69 },
    { 0xb821d9f7, SUC_lexicon_tags_11 },
    { 0x9ce1d9f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8941d9fb, SUC_lexicon_tags_35 },
    { 0x1fe1d9fc, SUC_lexicon_tags_15 },
    { 0x7961d9fd, SUC_lexicon_tags_55 },
    { 0xa881d9fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c1da02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x77a1da04, SUC_lexicon_tags_11 },
    { 0xf421da04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9201da07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2341da09, SUC_lexicon_tags_4 },
    { 0x5401da09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe701da0d, SUC_lexicon_tags_30 },
    { 0x1561da0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8581da14, SUC_lexicon_tags_49 },
    { 0x2ce1da14, SUC_lexicon_tags_45 },
    { 0x63e1da16, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9661da18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a61da1b, SUC_lexicon_tags_28 },
    { 0x4501da1c, SUC_lexicon_tags_2 },
    { 0x5a21da1d, SUC_lexicon_tags_26 },
    { 0x2941da1d, SUC_lexicon_tags_17 },
    { 0x9861da1e, SUC_lexicon_tags_45 },
    { 0x8b61da1f, SUC_lexicon_tags_33 },
    { 0x3341da1c, SUC_lexicon_tags_37 },
    { 0xb881da21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4161da25, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd4a1da27, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2221da29, SUC_lexicon_tags_17 },
    { 0x9ac1da2a, SUC_lexicon_tags_60 },
    { 0x3ee1da2b, SUC_lexicon_tags_60 },
    { 0x2ea1da2b, SUC_lexicon_tags_2 },
    { 0x23a1da2d, SUC_lexicon_tags_5 },
    { 0x2fc1da2d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3181da30, SUC_lexicon_tags_1 },
    { 0x6a81da30, SUC_lexicon_tags_14 },
    { 0xe281da32, SUC_lexicon_tags_25 },
    { 0xc321da30, SUC_lexicon_tags_7 },
    { 0x85a1da33, SUC_lexicon_tags_17 },
    { 0x1da1da35, SUC_lexicon_tags_11 },
    { 0x3a21da36, SUC_lexicon_tags_15 },
    { 0xe2c1da37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2841da39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8361da3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1241da42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbba1da44, SUC_lexicon_tags_57 },
    { 0x5da1da44, SUC_lexicon_tags_37 },
    { 0x4ae1da46, SUC_lexicon_tags_7 },
    { 0xfce1da46, SUC_lexicon_tags_47 },
    { 0x59a1da46, SUC_lexicon_tags_45 },
    { 0xce81da47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf861da4f, SUC_lexicon_tags_34 },
    { 0x7981da50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3401da52, SUC_lexicon_tags_0 },
    { 0x8181da52, SUC_lexicon_tags_14 },
    { 0x6c41da54, SUC_lexicon_tags_1 },
    { 0xef61da54, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x16a1da57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed81da59, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xf5c1da5b, SUC_lexicon_tags_17 },
    { 0x9c1da5c, SUC_lexicon_tags_11 },
    { 0x6c81da5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a81da60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a1da63, SUC_lexicon_tags_14 },
    { 0x9641da63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c1da67, SUC_lexicon_tags_4 },
    { 0xf5c1da68, SUC_lexicon_tags_48 },
    { 0x46e1da69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7021da6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561da70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7501da72, SUC_lexicon_tags_19 },
    { 0xcaa1da73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1a1da75, SUC_lexicon_tags_25 },
    { 0x8ae1da75, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe1da7c, SUC_lexicon_tags_11 },
    { 0x9fc1da7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd01da81, SUC_lexicon_tags_37 },
    { 0x2c1da82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d21da84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2421da8c, SUC_lexicon_tags_4 },
    { 0xb361da8d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe401da91, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1121da93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5061da9a, SUC_lexicon_tags_17 },
    { 0x7e81da9b, SUC_lexicon_tags_113 },
    { 0x7e21da9c, SUC_lexicon_tags_25 },
    { 0xe601da9b, SUC_lexicon_tags_33 },
    { 0x4501da9d, SUC_lexicon_tags_106 },
    { 0xc61da9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c01daa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a1daa4, SUC_lexicon_tags_61 },
    { 0xc321daa5, SUC_lexicon_tags_11 },
    { 0x7341daa6, SUC_lexicon_tags_68 },
    { 0xfc81daa7, SUC_lexicon_tags_2 },
    { 0x6421daa6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6181daaa, SUC_lexicon_tags_60 },
    { 0x9381daab, SUC_lexicon_tags_43 },
    { 0xd5e1daac, SUC_lexicon_tags_37 },
    { 0x5741daad, SUC_lexicon_tags_27 },
    { 0x7361daae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa6c1dab0, SUC_lexicon_tags_17 },
    { 0xefe1dab0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9aa1dab3, SUC_lexicon_tags_55 },
    { 0x9561dab4, SUC_lexicon_tags_2 },
    { 0x7cc1dab5, SUC_lexicon_tags_43 },
    { 0xaee1dab5, SUC_lexicon_tags_53 },
    { 0x49a1dab7, SUC_lexicon_tags_23 },
    { 0x1201dab8, SUC_lexicon_tags_38 },
    { 0x1f41dab8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa661dabc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7701dac0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd861dac2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dac7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1621daca, SUC_lexicon_tags_17 },
    { 0xf781dacb, SUC_lexicon_tags_15 },
    { 0x9641dacc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe1dad3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6141dad6, SUC_lexicon_tags_43 },
    { 0x9841dad7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xca81dad9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7641dadc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9281dade, SUC_lexicon_tags_16 },
    { 0xfc61dadf, SUC_lexicon_tags_44 },
    { 0xf641dadf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfbc1dae2, SUC_lexicon_tags_10 },
    { 0xaaa1dae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32c1dae5, SUC_lexicon_tags_15 },
    { 0x85e1dae6, SUC_lexicon_tags_19 },
    { 0xf4a1dae5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa901daed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6981daf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa741dafe, SUC_lexicon_tags_11 },
    { 0x2f01daff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd01db01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ba1db03, SUC_lexicon_tags_9 },
    { 0xe081db03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb781db0d, SUC_lexicon_tags_1 },
    { 0x2ca1db0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca1db13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa081db15, SUC_lexicon_tags_37 },
    { 0xa261db16, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4061db1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2241db21, SUC_lexicon_tags_28 },
    { 0x4ea1db22, SUC_lexicon_tags_6 },
    { 0x5de1db23, SUC_lexicon_tags_11 },
    { 0x2da1db24, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1db27, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9fc1db29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce21db2b, SUC_lexicon_tags_79 },
    { 0xe601db2b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xef01db2e, SUC_lexicon_tags_37 },
    { 0x9d61db2e, SUC_lexicon_tags_25 },
    { 0xe01db2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c01db32, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2541db34, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2701db3b, SUC_lexicon_tags_6 },
    { 0x44e1db3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e1db3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd381db42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81db45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a81db51, SUC_lexicon_tags_8 },
    { 0xa0c1db52, SUC_lexicon_tags_17 },
    { 0x5a1db53, SUC_lexicon_tags_23 },
    { 0x6c1db54, SUC_lexicon_tags_39 },
    { 0xd41db55, SUC_lexicon_tags_33 },
    { 0x2781db55, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7621db58, SUC_lexicon_tags_7 },
    { 0xbee1db59, SUC_lexicon_tags_25 },
    { 0xc7c1db58, SUC_lexicon_tags_37 },
    { 0xbcc1db5a, SUC_lexicon_tags_7 },
    { 0xa301db5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9841db60, SUC_lexicon_tags_1 },
    { 0xfd81db60, SUC_lexicon_tags_14 },
    { 0xd581db62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c61db64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e1db67, SUC_lexicon_tags_14 },
    { 0xbe81db68, SUC_lexicon_tags_17 },
    { 0x64a1db69, SUC_lexicon_tags_25 },
    { 0x3041db68, SUC_lexicon_tags_37 },
    { 0xcb41db6a, SUC_lexicon_tags_23 },
    { 0x4b61db6c, SUC_lexicon_tags_7 },
    { 0xa8a1db6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2c1db6f, SUC_lexicon_tags_33 },
    { 0x1041db6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c1db73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e1db78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a1db7d, SUC_lexicon_tags_23 },
    { 0xe321db7e, SUC_lexicon_tags_48 },
    { 0x6681db7e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a1db82, SUC_lexicon_tags_23 },
    { 0xb561db82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc621db88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf441db8a, SUC_lexicon_tags_6 },
    { 0x62a1db8a, SUC_lexicon_tags_29 },
    { 0x8461db8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d61db91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b81db94, SUC_lexicon_tags_15 },
    { 0x46a1db95, SUC_lexicon_tags_12 },
    { 0xc5a1db96, SUC_lexicon_tags_33 },
    { 0x1781db97, SUC_lexicon_tags_2 },
    { 0x42c1db98, SUC_lexicon_tags_12 },
    { 0x73a1db99, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3a1db9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1db9e, SUC_lexicon_tags_11 },
    { 0xcb41db9f, SUC_lexicon_tags_7 },
    { 0x1021db9f, SUC_lexicon_tags_15 },
    { 0x8321db9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6981dba4, SUC_lexicon_tags_2 },
    { 0xa7a1dba4, SUC_lexicon_tags_25 },
    { 0xba81dba6, SUC_lexicon_tags_1 },
    { 0x4461dba4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21dbac, SUC_lexicon_tags_7 },
    { 0x41dbad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7761dbb2, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b21dbb6, SUC_lexicon_tags_37 },
    { 0xaa61dbb7, SUC_lexicon_tags_63 },
    { 0x77a1dbb8, SUC_lexicon_tags_30 },
    { 0x22e1dbb6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1c41dbbb, SUC_lexicon_tags_39 },
    { 0x2a81dbbc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1241dbc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b41dbc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7e1dbc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a1dbcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e1dbce, SUC_lexicon_tags_35 },
    { 0xc821dbcf, SUC_lexicon_tags_11 },
    { 0xa3e1dbd0, SUC_lexicon_tags_14 },
    { 0xf921dbd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc1dbd6, SUC_lexicon_tags_31 },
    { 0xb8c1dbd7, SUC_lexicon_tags_0 },
    { 0x4ae1dbd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7481dbdc, SUC_lexicon_tags_11 },
    { 0xc01dbdd, SUC_lexicon_tags_12 },
    { 0x6d81dbdd, SUC_lexicon_tags_82 },
    { 0x35c1dbdc, SUC_lexicon_tags_84 },
    { 0x82c1dbdf, SUC_lexicon_tags_63 },
    { 0x5241dbdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2461dbe6, SUC_lexicon_tags_17 },
    { 0x2901dbe7, SUC_lexicon_tags_47 },
    { 0x1ae1dbe6, SUC_lexicon_tags_1 },
    { 0xdca1dbe8, SUC_lexicon_tags_4 },
    { 0xeba1dbe7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4f41dbec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6301dbef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xee81dbf2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c1dbfb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da1dbfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8281dc01, SUC_lexicon_tags_1 },
    { 0x9d81dc01, SUC_lexicon_tags_60 },
    { 0x3b61dc02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7121dc07, SUC_lexicon_tags_55 },
    { 0x79c1dc08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3141dc0d, SUC_lexicon_tags_2 },
    { 0xc461dc0e, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xc781dc11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38c1dc13, SUC_lexicon_tags_30 },
    { 0x61e1dc13, SUC_lexicon_tags_10 },
    { 0xf3e1dc15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcca1dc17, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e1dc1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda41dc1c, SUC_lexicon_tags_14 },
    { 0x4fe1dc1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd161dc1f, SUC_lexicon_tags_1 },
    { 0xaa41dc1f, SUC_lexicon_tags_7 },
    { 0x48a1dc21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3821dc24, SUC_lexicon_tags_48 },
    { 0x78a1dc25, SUC_lexicon_tags_46 },
    { 0x4801dc26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9741dc2e, SUC_lexicon_tags_15 },
    { 0xc261dc2f, SUC_lexicon_tags_27 },
    { 0x8a21dc2f, SUC_lexicon_tags_17 },
    { 0xbb81dc31, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4001dc38, SUC_lexicon_tags_43 },
    { 0x81e1dc39, SUC_lexicon_tags_9 },
    { 0xc1e1dc3a, SUC_lexicon_tags_1 },
    { 0xb0e1dc39, SUC_lexicon_tags_12 },
    { 0x8b41dc38, SUC_lexicon_tags_77 },
    { 0x6061dc3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc81dc3f, SUC_lexicon_tags_2 },
    { 0xba41dc40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e1dc48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a1dc4e, SUC_lexicon_tags_11 },
    { 0x3941dc4e, SUC_lexicon_tags_2 },
    { 0x5d41dc4e, SUC_lexicon_tags_7 },
    { 0x5e41dc51, SUC_lexicon_tags_17 },
    { 0x91a1dc52, SUC_lexicon_tags_87 },
    { 0x2141dc52, SUC_lexicon_tags_25 },
    { 0xd9c1dc53, SUC_lexicon_tags_47 },
    { 0xb161dc54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c81dc57, SUC_lexicon_tags_23 },
    { 0x8321dc57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec1dc5b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5861dc5f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2921dc63, SUC_lexicon_tags_45 },
    { 0xc5c1dc64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a41dc6b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb321dc6d, SUC_lexicon_tags_2 },
    { 0x4921dc6d, SUC_lexicon_tags_253 },
    { 0x63c1dc6f, SUC_lexicon_tags_12 },
    { 0x88c1dc70, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1121dc77, SUC_lexicon_tags_25 },
    { 0x1521dc78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5fa1dc7a, SUC_lexicon_tags_17 },
    { 0x9fc1dc7b, SUC_lexicon_tags_2 },
    { 0x57c1dc7c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbe21dc7e, SUC_lexicon_tags_60 },
    { 0x16a1dc7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9501dc83, SUC_lexicon_tags_0 },
    { 0x7541dc84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a1dc86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2161dc88, SUC_lexicon_tags_49 },
    { 0x5221dc89, SUC_lexicon_tags_25 },
    { 0x36a1dc88, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9481dc8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a81dc94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa961dc97, SUC_lexicon_tags_25 },
    { 0xa2c1dc97, SUC_lexicon_tags_8 },
    { 0x4e1dc99, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x1041dc9b, SUC_lexicon_tags_17 },
    { 0xf221dc9c, SUC_lexicon_tags_45 },
    { 0xa9a1dc9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a1dca0, SUC_lexicon_tags_28 },
    { 0xd6c1dca1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8141dca9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb21dcac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7381dcb0, SUC_lexicon_tags_12 },
    { 0x44e1dcb1, SUC_lexicon_tags_63 },
    { 0xf9c1dcb1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x961dcb5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad81dcbb, SUC_lexicon_tags_11 },
    { 0x9fa1dcbc, SUC_lexicon_tags_7 },
    { 0x7801dcbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e1dcc2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8901dcc6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcc1dcc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7881dcca, SUC_lexicon_tags_12 },
    { 0x3fa1dcca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e41dccf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3581dcd1, SUC_lexicon_tags_11 },
    { 0x2c1dcd1, SUC_lexicon_tags_10 },
    { 0x52a1dcd3, SUC_lexicon_tags_7 },
    { 0x76c1dcd4, SUC_lexicon_tags_35 },
    { 0xbe1dcd5, SUC_lexicon_tags_56 },
    { 0x7941dcd5, SUC_lexicon_tags_17 },
    { 0x6501dcd7, SUC_lexicon_tags_17 },
    { 0x2501dcd8, SUC_lexicon_tags_44 },
    { 0xb61dcd7, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a1dcdd, SUC_lexicon_tags_26 },
    { 0x5121dcde, SUC_lexicon_tags_100 },
    { 0x5481dcdd, SUC_lexicon_tags_12 },
    { 0x33c1dcdf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf521dce4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf61dce7, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xd821dce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf281dcec, SUC_lexicon_tags_12 },
    { 0x2481dced, SUC_lexicon_tags_32 },
    { 0xc321dced, SUC_lexicon_tags_11 },
    { 0xec21dcec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c1dcf2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4861dcf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb181dcf8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b01dcfb, SUC_lexicon_tags_47 },
    { 0xf361dcfb, SUC_lexicon_tags_1 },
    { 0x7c21dcfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f41dd02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9681dd06, SUC_lexicon_tags_2 },
    { 0xd701dd06, SUC_lexicon_tags_17 },
    { 0x9601dd08, SUC_lexicon_tags_21 },
    { 0x57c1dd09, SUC_lexicon_tags_48 },
    { 0x2381dd0a, SUC_lexicon_tags_15 },
    { 0x94a1dd08, SUC_lexicon_tags_26 },
    { 0xaa21dd0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc1dd0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5941dd13, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8e81dd15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb661dd17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4281dd19, SUC_lexicon_tags_22 },
    { 0x2ee1dd19, SUC_lexicon_tags_37 },
    { 0xa0a1dd1b, SUC_lexicon_tags_1 },
    { 0x9be1dd1c, SUC_lexicon_tags_61 },
    { 0x24c1dd1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8501dd21, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be1dd24, SUC_lexicon_tags_29 },
    { 0xc401dd25, SUC_lexicon_tags_16 },
    { 0xce61dd26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf041dd2c, SUC_lexicon_tags_17 },
    { 0x1821dd2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7121dd30, SUC_lexicon_tags_38 },
    { 0xc1a1dd31, SUC_lexicon_tags_26 },
    { 0x9b61dd32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42a1dd34, SUC_lexicon_tags_70 },
    { 0x83c1dd35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc581dd39, SUC_lexicon_tags_43 },
    { 0xed01dd3a, SUC_lexicon_tags_6 },
    { 0x7381dd3a, SUC_lexicon_tags_17 },
    { 0xdb21dd3c, SUC_lexicon_tags_2 },
    { 0x221dd3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaec1dd3f, SUC_lexicon_tags_37 },
    { 0x5a21dd3f, SUC_lexicon_tags_8 },
    { 0x18a1dd40, SUC_lexicon_tags_91 },
    { 0xeb01dd42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1dd45, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a1dd4c, SUC_lexicon_tags_37 },
    { 0x1621dd4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc1dd51, SUC_lexicon_tags_14 },
    { 0x6001dd52, SUC_lexicon_tags_63 },
    { 0xb541dd52, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfb81dd55, SUC_lexicon_tags_46 },
    { 0xf9e1dd56, SUC_lexicon_tags_12 },
    { 0x50a1dd56, SUC_lexicon_tags_17 },
    { 0x9581dd56, SUC_lexicon_tags_12 },
    { 0x3a21dd59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb61dd5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8561dd5f, SUC_lexicon_tags_12 },
    { 0xa741dd5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e21dd66, SUC_lexicon_tags_1 },
    { 0xa5e1dd67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7141dd69, SUC_lexicon_tags_0 },
    { 0x7861dd6a, SUC_lexicon_tags_27 },
    { 0x1301dd6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a1dd6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece1dd74, SUC_lexicon_tags_8 },
    { 0xe421dd75, SUC_lexicon_tags_1 },
    { 0x9261dd75, SUC_lexicon_tags_23 },
    { 0x72a1dd77, SUC_lexicon_tags_37 },
    { 0xb961dd77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3581dd7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xef41dd7f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4401dd86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca81dd8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe681dd8e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de1dd95, SUC_lexicon_tags_6 },
    { 0xcd01dd96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x32c1dd98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed01dd9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee01dda0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2a81dda2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3801dda4, SUC_lexicon_tags_25 },
    { 0xf161dda4, SUC_lexicon_tags_30 },
    { 0x2261dda4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd721ddab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e1ddaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c21ddb1, SUC_lexicon_tags_66 },
    { 0xdde1ddb2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb581ddb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ac1ddb6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xca41ddb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa201ddbb, SUC_lexicon_tags_8 },
    { 0x4381ddbb, SUC_lexicon_tags_11 },
    { 0x8641ddbd, SUC_lexicon_tags_1 },
    { 0xd361ddbe, SUC_lexicon_tags_38 },
    { 0x9e21ddbf, SUC_lexicon_tags_258 },
    { 0xf141ddbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a21ddc3, SUC_lexicon_tags_2 },
    { 0xb241ddc4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2981ddd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75c1ddd6, SUC_lexicon_tags_17 },
    { 0x4dc1ddd7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1ddda, SUC_lexicon_tags_37 },
    { 0x6201dddb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1621dddd, SUC_lexicon_tags_6 },
    { 0xf381dddd, SUC_lexicon_tags_9 },
    { 0x38e1dddf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c1dde3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac1dde6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4241dde9, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee1ddec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf081ddf2, SUC_lexicon_tags_17 },
    { 0xd881ddf3, SUC_lexicon_tags_11 },
    { 0x1061ddf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94a1ddf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fa1ddf8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe081ddfc, SUC_lexicon_tags_11 },
    { 0x3881ddfd, SUC_lexicon_tags_43 },
    { 0xc661ddfe, SUC_lexicon_tags_25 },
    { 0x45e1ddff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4e61de01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1441de06, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac1de0a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x301de0d, SUC_lexicon_tags_11 },
    { 0xbbe1de0e, SUC_lexicon_tags_15 },
    { 0x4d81de0f, SUC_lexicon_tags_12 },
    { 0xbc1de0f, SUC_lexicon_tags_7 },
    { 0x7ce1de11, SUC_lexicon_tags_40 },
    { 0x3a81de10, SUC_lexicon_tags_12 },
    { 0xb141de13, SUC_lexicon_tags_17 },
    { 0xe8e1de13, SUC_lexicon_tags_7 },
    { 0xe161de14, SUC_lexicon_tags_37 },
    { 0xa0c1de16, SUC_lexicon_tags_38 },
    { 0x3601de13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x841de19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6261de1b, SUC_lexicon_tags_33 },
    { 0x9081de1c, SUC_lexicon_tags_37 },
    { 0x64e1de1c, SUC_lexicon_tags_63 },
    { 0x6721de1e, SUC_lexicon_tags_0 },
    { 0xfee1de1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce81de22, SUC_lexicon_tags_8 },
    { 0xa961de23, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c1de2d, SUC_lexicon_tags_25 },
    { 0xa141de2e, SUC_lexicon_tags_37 },
    { 0x40a1de2f, SUC_lexicon_tags_0 },
    { 0x9d41de2d, SUC_lexicon_tags_37 },
    { 0x9141de31, SUC_lexicon_tags_1 },
    { 0xc9a1de32, SUC_lexicon_tags_23 },
    { 0x3c21de2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x781de35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec41de39, SUC_lexicon_tags_11 },
    { 0xec21de3a, SUC_lexicon_tags_27 },
    { 0xa341de3a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a1de3f, SUC_lexicon_tags_55 },
    { 0x5ec1de3f, SUC_lexicon_tags_17 },
    { 0x2161de41, SUC_lexicon_tags_7 },
    { 0x1b21de42, SUC_lexicon_tags_45 },
    { 0x3bc1de42, SUC_lexicon_tags_45 },
    { 0xfc81de42, SUC_lexicon_tags_37 },
    { 0x4fe1de45, SUC_lexicon_tags_152 },
    { 0xf101de45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd181de4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5401de4e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4361de50, SUC_lexicon_tags_29 },
    { 0x4f21de50, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x521de53, SUC_lexicon_tags_47 },
    { 0x6dc1de54, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xfe01de56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce21de5a, SUC_lexicon_tags_45 },
    { 0x7cc1de5b, SUC_lexicon_tags_77 },
    { 0x15e1de5c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda1de61, SUC_lexicon_tags_38 },
    { 0xea1de62, SUC_lexicon_tags_7 },
    { 0x5ec1de63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe581de65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281de6b, SUC_lexicon_tags_38 },
    { 0x6de1de6b, SUC_lexicon_tags_17 },
    { 0xc5a1de6d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x19e1de6f, SUC_lexicon_tags_3 },
    { 0xd1a1de70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9521de74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8d61de76, SUC_lexicon_tags_77 },
    { 0x8c21de77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf201de7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd61de7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e1de84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7981de88, SUC_lexicon_tags_18 },
    { 0xd1c1de89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b41de8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7c1de8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9421de93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5481de95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaea1de97, SUC_lexicon_tags_17 },
    { 0x9761de98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf001dea5, SUC_lexicon_tags_28 },
    { 0xa61dea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc401deaa, SUC_lexicon_tags_0 },
    { 0x781deab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e41deae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a1deb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa981deb6, SUC_lexicon_tags_55 },
    { 0x361deb7, SUC_lexicon_tags_17 },
    { 0x8a1deb7, SUC_lexicon_tags_2 },
    { 0x1deb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa881debf, SUC_lexicon_tags_10 },
    { 0x9661dec0, SUC_lexicon_tags_2 },
    { 0x49c1dec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4641dec6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c61dec9, SUC_lexicon_tags_17 },
    { 0x64e1deca, SUC_lexicon_tags_138 },
    { 0xbce1decb, SUC_lexicon_tags_47 },
    { 0x21decb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca1ded1, SUC_lexicon_tags_37 },
    { 0xbbe1ded2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x34a1ded4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7241ded7, SUC_lexicon_tags_38 },
    { 0x97e1ded7, SUC_lexicon_tags_47 },
    { 0xe7e1ded9, SUC_lexicon_tags_55 },
    { 0x5ee1deda, SUC_lexicon_tags_9 },
    { 0x33a1dedb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7241dee3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xede1dee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b21dee7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e1deed, SUC_lexicon_tags_46 },
    { 0xe1a1deed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2481def0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e1def3, SUC_lexicon_tags_33 },
    { 0x5d81def4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c01def7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c1defb, SUC_lexicon_tags_38 },
    { 0xc761defc, SUC_lexicon_tags_11 },
    { 0x2d61defd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1101df00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb401df03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3181df09, SUC_lexicon_tags_118 },
    { 0xa941df0a, SUC_lexicon_tags_27 },
    { 0x10a1df0b, SUC_lexicon_tags_63 },
    { 0x5061df0c, SUC_lexicon_tags_0 },
    { 0xb901df0d, SUC_lexicon_tags_47 },
    { 0x201df0d, SUC_lexicon_tags_30 },
    { 0x3061df0d, SUC_lexicon_tags_15 },
    { 0x6ea1df10, SUC_lexicon_tags_7 },
    { 0x1ee1df0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3921df13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe961df15, SUC_lexicon_tags_19 },
    { 0x27e1df16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e1df19, SUC_lexicon_tags_7 },
    { 0xb121df1a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x6461df1d, SUC_lexicon_tags_9 },
    { 0x9501df1d, SUC_lexicon_tags_2 },
    { 0x2221df1f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c01df23, SUC_lexicon_tags_11 },
    { 0x16c1df24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5641df2e, SUC_lexicon_tags_28 },
    { 0xd0a1df2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a61df34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbce1df36, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5121df39, SUC_lexicon_tags_23 },
    { 0x8e81df3a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc5e1df3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7701df40, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461df45, SUC_lexicon_tags_25 },
    { 0x6161df46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8881df4b, SUC_lexicon_tags_0 },
    { 0x6f81df4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de1df4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x521df52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a1df55, SUC_lexicon_tags_15 },
    { 0x4de1df55, SUC_lexicon_tags_17 },
    { 0x79e1df57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae1df5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221df61, SUC_lexicon_tags_11 },
    { 0xc701df62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea1df66, SUC_lexicon_tags_68 },
    { 0x9221df67, SUC_lexicon_tags_17 },
    { 0x16e1df68, SUC_lexicon_tags_8 },
    { 0xd681df69, SUC_lexicon_tags_15 },
    { 0x4aa1df68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa361df6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a1df72, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9d21df74, SUC_lexicon_tags_51 },
    { 0xaea1df74, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb01df7d, SUC_lexicon_tags_1 },
    { 0x34a1df7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc401df81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60c1df83, SUC_lexicon_tags_38 },
    { 0x2d01df84, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc961df89, SUC_lexicon_tags_25 },
    { 0x7821df89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac81df8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe121df8f, SUC_lexicon_tags_2 },
    { 0xc621df90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd41df9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc21df9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc81dfa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec1dfa3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6181dfa5, SUC_lexicon_tags_61 },
    { 0x46e1dfa5, SUC_lexicon_tags_26 },
    { 0x74e1dfa7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdc81dfa9, SUC_lexicon_tags_60 },
    { 0x6b41dfaa, SUC_lexicon_tags_63 },
    { 0x78c1dfaa, SUC_lexicon_tags_30 },
    { 0x7161dfaa, SUC_lexicon_tags_11 },
    { 0xbee1dfad, SUC_lexicon_tags_1 },
    { 0xee1dfab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3921dfb1, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae1dfb5, SUC_lexicon_tags_20 },
    { 0xf921dfb5, SUC_lexicon_tags_1 },
    { 0x8061dfb7, SUC_lexicon_tags_57 },
    { 0x96e1dfb8, SUC_lexicon_tags_2 },
    { 0x2f61dfb5, SUC_lexicon_tags_38 },
    { 0xb961dfb6, SUC_lexicon_tags_37 },
    { 0xa61dfbb, SUC_lexicon_tags_33 },
    { 0xe141dfb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc1dfc1, SUC_lexicon_tags_37 },
    { 0x9381dfc1, SUC_lexicon_tags_45 },
    { 0x31c1dfc2, SUC_lexicon_tags_9 },
    { 0xc8e1dfc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e1dfca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd3c1dfcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c81dfcf, SUC_lexicon_tags_14 },
    { 0x7941dfd0, SUC_lexicon_tags_5 },
    { 0xe1dfd1, SUC_lexicon_tags_27 },
    { 0xb381dfd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdea1dfd4, SUC_lexicon_tags_226 },
    { 0xb241dfd4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c21dfd9, SUC_lexicon_tags_390 },
    { 0xe6e1dfda, SUC_lexicon_tags_1 },
    { 0x5ac1dfda, SUC_lexicon_tags_1 },
    { 0xae1dfd9, SUC_lexicon_tags_74 },
    { 0xe721dfdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e1dfe1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c1dfe4, SUC_lexicon_tags_20 },
    { 0x46c1dfe5, SUC_lexicon_tags_64 },
    { 0x1b21dfe6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe1dfec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9401dfee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce21dff4, SUC_lexicon_tags_27 },
    { 0x4e1dff5, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x5741dff7, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0xb21dff9, SUC_lexicon_tags_1 },
    { 0x8181dffa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x97e1dffc, SUC_lexicon_tags_12 },
    { 0x3e1dffc, SUC_lexicon_tags_19 },
    { 0xfea1dffe, SUC_lexicon_tags_57 },
    { 0xad81dffc, SUC_lexicon_tags_17 },
    { 0xf5e1e000, SUC_lexicon_tags_15 },
    { 0x9381dffe, SUC_lexicon_tags_7 },
    { 0xfaa1e000, SUC_lexicon_tags_23 },
    { 0x2001e003, SUC_lexicon_tags_37 },
    { 0xee41e004, SUC_lexicon_tags_24 },
    { 0x6781e002, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c1e008, SUC_lexicon_tags_7 },
    { 0x3da1e009, SUC_lexicon_tags_12 },
    { 0x2121e008, SUC_lexicon_tags_10 },
    { 0xb881e00b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb441e011, SUC_lexicon_tags_12 },
    { 0x6c01e011, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1061e016, SUC_lexicon_tags_23 },
    { 0xa0c1e016, SUC_lexicon_tags_9 },
    { 0x7681e018, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c1e01d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a1e020, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a1e023, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2701e025, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a1e02d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc641e030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca1e036, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1561e03b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6fe1e03d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3541e03f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8dc1e041, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e21e047, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c81e04a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc1e04e, SUC_lexicon_tags_25 },
    { 0xa461e04e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e61e052, SUC_lexicon_tags_37 },
    { 0x1b81e053, SUC_lexicon_tags_27 },
    { 0xaa01e054, SUC_lexicon_tags_2 },
    { 0x88a1e055, SUC_lexicon_tags_62 },
    { 0x8641e053, SUC_lexicon_tags_63 },
    { 0xf5c1e057, SUC_lexicon_tags_56 },
    { 0xe481e058, SUC_lexicon_tags_25 },
    { 0xb961e057, SUC_lexicon_tags_37 },
    { 0xecc1e05a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff21e061, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a21e063, SUC_lexicon_tags_1 },
    { 0xebe1e064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c1e068, SUC_lexicon_tags_45 },
    { 0xcd61e069, SUC_lexicon_tags_2 },
    { 0xb301e06a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6361e06e, SUC_lexicon_tags_38 },
    { 0x3461e06f, SUC_lexicon_tags_31 },
    { 0x18e1e06f, SUC_lexicon_tags_47 },
    { 0x28a1e071, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc1e074, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea1e078, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61e1e07a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc1e07f, SUC_lexicon_tags_25 },
    { 0x95e1e07f, SUC_lexicon_tags_23 },
    { 0x4a81e080, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe5c1e083, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb421e086, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5141e08a, SUC_lexicon_tags_911 },
    { 0xea41e08b, SUC_lexicon_tags_37 },
    { 0x5fc1e08c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea01e090, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81e093, SUC_lexicon_tags_26 },
    { 0xdec1e093, SUC_lexicon_tags_2 },
    { 0x52a1e093, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6121e099, SUC_lexicon_tags_49 },
    { 0x2fe1e099, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e1e09d, SUC_lexicon_tags_18 },
    { 0x2001e09d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c1e0a1, SUC_lexicon_tags_1 },
    { 0xf5e1e0a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba1e0a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e61e0a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d41e0ac, SUC_lexicon_tags_13 },
    { 0x5c61e0ad, SUC_lexicon_tags_9 },
    { 0xa681e0ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7301e0b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa201e0b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd821e0b7, SUC_lexicon_tags_61 },
    { 0xcda1e0b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee1e0c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4321e0c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x301e0c7, SUC_lexicon_tags_12 },
    { 0x34e1e0c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c1e0cd, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xa761e0d0, SUC_lexicon_tags_29 },
    { 0xeb01e0d0, SUC_lexicon_tags_19 },
    { 0xe5c1e0d2, SUC_lexicon_tags_12 },
    { 0x5ce1e0d2, SUC_lexicon_tags_25 },
    { 0x5641e0d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c01e0d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c1e0da, SUC_lexicon_tags_27 },
    { 0xd5c1e0db, SUC_lexicon_tags_57 },
    { 0x5bc1e0dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe521e0df, SUC_lexicon_tags_25 },
    { 0x3c01e0e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e61e0e2, SUC_lexicon_tags_6 },
    { 0x841e0e2, SUC_lexicon_tags_7 },
    { 0xc901e0e4, SUC_lexicon_tags_1 },
    { 0x5e01e0e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe261e0eb, SUC_lexicon_tags_17 },
    { 0x1e0eb, SUC_lexicon_tags_12 },
    { 0x7241e0ed, SUC_lexicon_tags_7 },
    { 0x35c1e0ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc101e0f1, SUC_lexicon_tags_25 },
    { 0xd9c1e0f2, SUC_lexicon_tags_60 },
    { 0xcc01e0f2, SUC_lexicon_tags_11 },
    { 0xe9e1e0f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e1e0f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8241e0fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d61e101, SUC_lexicon_tags_19 },
    { 0x6861e102, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3a61e104, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a1e108, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa421e111, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef01e114, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3101e117, SUC_lexicon_tags_37 },
    { 0xcd61e117, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6101e11d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae1e124, SUC_lexicon_tags_37 },
    { 0xc981e124, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f81e127, SUC_lexicon_tags_2 },
    { 0x65c1e128, SUC_lexicon_tags_12 },
    { 0xe8e1e129, SUC_lexicon_tags_37 },
    { 0x2861e129, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb641e12e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba1e133, SUC_lexicon_tags_1 },
    { 0x2c81e133, SUC_lexicon_tags_38 },
    { 0x2681e133, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba01e137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f61e13a, SUC_lexicon_tags_11 },
    { 0x57c1e13a, SUC_lexicon_tags_8 },
    { 0x99c1e13a, SUC_lexicon_tags_47 },
    { 0x9801e13a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4001e142, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4141e145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c1e14e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3481e151, SUC_lexicon_tags_6 },
    { 0x45a1e152, SUC_lexicon_tags_33 },
    { 0x5441e152, SUC_lexicon_tags_11 },
    { 0x36e1e151, SUC_lexicon_tags_11 },
    { 0xeca1e155, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a01e158, SUC_lexicon_tags_25 },
    { 0xf941e159, SUC_lexicon_tags_15 },
    { 0xc0c1e158, SUC_lexicon_tags_1 },
    { 0xea41e15b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc961e15e, SUC_lexicon_tags_49 },
    { 0xfd21e15f, SUC_lexicon_tags_14 },
    { 0xb901e15e, SUC_lexicon_tags_11 },
    { 0xea41e161, SUC_lexicon_tags_37 },
    { 0xa701e161, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a1e165, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x321e167, SUC_lexicon_tags_28 },
    { 0xb081e168, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf561e172, SUC_lexicon_tags_43 },
    { 0xf0a1e172, SUC_lexicon_tags_57 },
    { 0xc0c1e174, SUC_lexicon_tags_80 },
    { 0x1fa1e175, SUC_lexicon_tags_2 },
    { 0x7621e175, SUC_lexicon_tags_65 },
    { 0xc921e175, SUC_lexicon_tags_12 },
    { 0x46e1e177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5c1e17a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a1e180, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf41e185, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc1e18a, SUC_lexicon_tags_30 },
    { 0xc1c1e18b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a1e18e, SUC_lexicon_tags_38 },
    { 0x44a1e18e, SUC_lexicon_tags_7 },
    { 0xb3a1e190, SUC_lexicon_tags_7 },
    { 0x4641e190, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d61e19c, SUC_lexicon_tags_33 },
    { 0x61c1e19d, SUC_lexicon_tags_25 },
    { 0xd8c1e19e, SUC_lexicon_tags_49 },
    { 0x88a1e19e, SUC_lexicon_tags_12 },
    { 0x1601e1a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x52a1e1a2, SUC_lexicon_tags_37 },
    { 0xb7a1e1a2, SUC_lexicon_tags_138 },
    { 0x4641e1a4, SUC_lexicon_tags_27 },
    { 0xd5e1e1a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2901e1a8, SUC_lexicon_tags_55 },
    { 0xd401e1a9, SUC_lexicon_tags_11 },
    { 0x5501e1aa, SUC_lexicon_tags_19 },
    { 0x7961e1a8, SUC_lexicon_tags_21 },
    { 0xb21e1ac, SUC_lexicon_tags_33 },
    { 0xc501e1ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9981e1af, SUC_lexicon_tags_1 },
    { 0xa201e1b0, SUC_lexicon_tags_17 },
    { 0xfc21e1af, SUC_lexicon_tags_0 },
    { 0x23c1e1b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd21e1b4, SUC_lexicon_tags_1 },
    { 0xa221e1b5, SUC_lexicon_tags_15 },
    { 0x97e1e1b6, SUC_lexicon_tags_29 },
    { 0x80c1e1b7, SUC_lexicon_tags_11 },
    { 0x7c1e1b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1c61e1ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a1e1bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3801e1bf, SUC_lexicon_tags_30 },
    { 0x78e1e1bf, SUC_lexicon_tags_47 },
    { 0x1c1e1c1, SUC_lexicon_tags_1 },
    { 0xbf21e1c1, SUC_lexicon_tags_60 },
    { 0x5d81e1c2, SUC_lexicon_tags_37 },
    { 0x1821e1c4, SUC_lexicon_tags_33 },
    { 0x9641e1c2, SUC_lexicon_tags_37 },
    { 0x4e1e1c2, SUC_lexicon_tags_47 },
    { 0x8721e1c7, SUC_lexicon_tags_23 },
    { 0x3e61e1c8, SUC_lexicon_tags_2 },
    { 0x7d01e1c9, SUC_lexicon_tags_82 },
    { 0x3441e1c5, SUC_lexicon_tags_12 },
    { 0xb3e1e1cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b21e1cf, SUC_lexicon_tags_49 },
    { 0xe921e1d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf021e1d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e81e1d6, SUC_lexicon_tags_35 },
    { 0x33c1e1d7, SUC_lexicon_tags_17 },
    { 0x91a1e1d8, SUC_lexicon_tags_4 },
    { 0xafe1e1d9, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xf261e1db, SUC_lexicon_tags_29 },
    { 0x8fe1e1db, SUC_lexicon_tags_63 },
    { 0x4201e1dd, SUC_lexicon_tags_39 },
    { 0xa001e1dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x97c1e1e0, SUC_lexicon_tags_0 },
    { 0xfc81e1e1, SUC_lexicon_tags_7 },
    { 0x2c21e1e2, SUC_lexicon_tags_37 },
    { 0x3bc1e1e3, SUC_lexicon_tags_1 },
    { 0xd01e1e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca1e1e7, SUC_lexicon_tags_61 },
    { 0x7d21e1e8, SUC_lexicon_tags_40 },
    { 0x8701e1e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd61e1ed, SUC_lexicon_tags_17 },
    { 0x16c1e1ee, SUC_lexicon_tags_2 },
    { 0xdf61e1ef, SUC_lexicon_tags_29 },
    { 0x2e61e1f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb721e1fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe1e203, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3421e206, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e01e209, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xef61e20b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e21e20d, SUC_lexicon_tags_13 },
    { 0xa0c1e20d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1261e210, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3181e215, SUC_lexicon_tags_7 },
    { 0x8e1e215, SUC_lexicon_tags_12 },
    { 0x3ba1e217, SUC_lexicon_tags_39 },
    { 0x4861e217, SUC_lexicon_tags_1 },
    { 0xff81e218, SUC_lexicon_tags_15 },
    { 0xc8c1e218, SUC_lexicon_tags_37 },
    { 0x2081e21a, SUC_lexicon_tags_27 },
    { 0x58e1e215, SUC_lexicon_tags_49 },
    { 0x3301e21d, SUC_lexicon_tags_12 },
    { 0xb701e21e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x25a1e220, SUC_lexicon_tags_38 },
    { 0xf361e221, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd501e22b, SUC_lexicon_tags_1 },
    { 0x4f81e22b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdfe1e22e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc601e230, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d61e234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc721e238, SUC_lexicon_tags_1 },
    { 0xa861e239, SUC_lexicon_tags_17 },
    { 0x1bc1e239, SUC_lexicon_tags_57 },
    { 0xb1e1e23a, SUC_lexicon_tags_43 },
    { 0x7e41e23c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb201e240, SUC_lexicon_tags_37 },
    { 0xc5a1e240, SUC_lexicon_tags_25 },
    { 0x71a1e242, SUC_lexicon_tags_104 },
    { 0xf341e242, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1e24b, SUC_lexicon_tags_4 },
    { 0xae01e24b, SUC_lexicon_tags_63 },
    { 0x5781e24d, SUC_lexicon_tags_14 },
    { 0x56a1e24c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd101e251, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701e256, SUC_lexicon_tags_7 },
    { 0x4661e256, SUC_lexicon_tags_6 },
    { 0xa181e258, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1601e25a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be1e266, SUC_lexicon_tags_87 },
    { 0x16a1e267, SUC_lexicon_tags_10 },
    { 0xbf01e268, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1e26c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e1e270, SUC_lexicon_tags_10 },
    { 0x1501e271, SUC_lexicon_tags_8 },
    { 0x60c1e270, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f21e275, SUC_lexicon_tags_16 },
    { 0x8761e276, SUC_lexicon_tags_60 },
    { 0x1921e276, SUC_lexicon_tags_47 },
    { 0x32a1e277, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a1e27b, SUC_lexicon_tags_31 },
    { 0x1541e27c, SUC_lexicon_tags_5 },
    { 0x5f21e27c, SUC_lexicon_tags_25 },
    { 0x3dc1e27e, SUC_lexicon_tags_48 },
    { 0xa7c1e27f, SUC_lexicon_tags_16 },
    { 0x6961e280, SUC_lexicon_tags_37 },
    { 0xbe81e27f, SUC_lexicon_tags_7 },
    { 0xdc1e282, SUC_lexicon_tags_25 },
    { 0x6041e27b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb941e286, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3501e288, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80e1e28a, SUC_lexicon_tags_82 },
    { 0xa101e28b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c1e28e, SUC_lexicon_tags_706 },
    { 0xa9a1e28e, SUC_lexicon_tags_28 },
    { 0xb8e1e28f, SUC_lexicon_tags_67 },
    { 0x38c1e291, SUC_lexicon_tags_28 },
    { 0x86a1e290, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4861e299, SUC_lexicon_tags_12 },
    { 0x6641e29a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf981e2a1, SUC_lexicon_tags_26 },
    { 0x8be1e2a1, SUC_lexicon_tags_63 },
    { 0x38e1e2a3, SUC_lexicon_tags_11 },
    { 0xba41e2a3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdc21e2a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba1e2a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf141e2ac, SUC_lexicon_tags_31 },
    { 0x72a1e2ac, SUC_lexicon_tags_25 },
    { 0x2cc1e2ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4121e2b0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x24a1e2b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1e2b6, SUC_lexicon_tags_87 },
    { 0xe3c1e2b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb81e2b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd841e2bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c1e2c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c1e2c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2621e2c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd01e2d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e1e2d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a1e2d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e1e2dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdac1e2df, SUC_lexicon_tags_68 },
    { 0x45e1e2e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5461e2e2, SUC_lexicon_tags_4 },
    { 0x9d01e2e2, SUC_lexicon_tags_4 },
    { 0xf4e1e2e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf41e2ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb101e2ed, SUC_lexicon_tags_65 },
    { 0x36e1e2ed, SUC_lexicon_tags_15 },
    { 0x7821e2ef, SUC_lexicon_tags_82 },
    { 0x6ec1e2f0, SUC_lexicon_tags_27 },
    { 0x3801e2f0, SUC_lexicon_tags_63 },
    { 0x9001e2f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c01e2f5, SUC_lexicon_tags_14 },
    { 0x4ae1e2f6, SUC_lexicon_tags_12 },
    { 0x5fe1e2f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c1e2fa, SUC_lexicon_tags_15 },
    { 0x9d81e2fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c01e2ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc7a1e301, SUC_lexicon_tags_9 },
    { 0xf341e302, SUC_lexicon_tags_13 },
    { 0xdd81e302, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfb01e305, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec1e30a, SUC_lexicon_tags_37 },
    { 0x9041e30a, SUC_lexicon_tags_38 },
    { 0xa41e30c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a81e310, SUC_lexicon_tags_60 },
    { 0xee41e310, SUC_lexicon_tags_11 },
    { 0xffc1e312, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d81e31e, SUC_lexicon_tags_0 },
    { 0xbce1e31f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da1e324, SUC_lexicon_tags_15 },
    { 0x1da1e325, SUC_lexicon_tags_2 },
    { 0xbac1e326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c1e329, SUC_lexicon_tags_11 },
    { 0x9201e32a, SUC_lexicon_tags_87 },
    { 0xd8a1e32b, SUC_lexicon_tags_7 },
    { 0xf421e32a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69a1e32e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20c1e330, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb7c1e332, SUC_lexicon_tags_43 },
    { 0x88c1e332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc01e33e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3761e342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d01e346, SUC_lexicon_tags_47 },
    { 0x281e347, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1181e349, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca1e34d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa721e351, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a1e354, SUC_lexicon_tags_33 },
    { 0xfa1e355, SUC_lexicon_tags_37 },
    { 0x1501e356, SUC_lexicon_tags_2 },
    { 0xcc1e357, SUC_lexicon_tags_8 },
    { 0x8e41e358, SUC_lexicon_tags_7 },
    { 0x1181e357, SUC_lexicon_tags_32 },
    { 0xc01e35a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61e360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201e364, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb301e366, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1101e36a, SUC_lexicon_tags_7 },
    { 0x4fc1e36b, SUC_lexicon_tags_16 },
    { 0x6461e36b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb7a1e36e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe041e370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6481e372, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x95a1e374, SUC_lexicon_tags_28 },
    { 0x2641e374, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8281e377, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8fe1e379, SUC_lexicon_tags_15 },
    { 0xac21e37a, SUC_lexicon_tags_37 },
    { 0xf801e37a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe581e380, SUC_lexicon_tags_11 },
    { 0xad21e380, SUC_lexicon_tags_17 },
    { 0x9401e382, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1e385, SUC_lexicon_tags_15 },
    { 0xf1e1e386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca41e388, SUC_lexicon_tags_19 },
    { 0x9521e389, SUC_lexicon_tags_1 },
    { 0x6b01e38a, SUC_lexicon_tags_15 },
    { 0x7481e38b, SUC_lexicon_tags_49 },
    { 0xbfa1e38c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfcc1e38e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e1e391, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a1e396, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f21e39a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3501e39e, SUC_lexicon_tags_29 },
    { 0x1361e39f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4521e3a2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdea1e3a4, SUC_lexicon_tags_7 },
    { 0xe141e3a4, SUC_lexicon_tags_2 },
    { 0xc701e3a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9781e3aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x58c1e3ac, SUC_lexicon_tags_1 },
    { 0x94a1e3ad, SUC_lexicon_tags_65 },
    { 0x5c01e3ae, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd421e3b9, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7941e3bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a1e3c9, SUC_lexicon_tags_26 },
    { 0x6841e3ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf61e3cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa541e3d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6321e3d9, SUC_lexicon_tags_11 },
    { 0x9001e3d9, SUC_lexicon_tags_37 },
    { 0xe81e3d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b01e3de, SUC_lexicon_tags_18 },
    { 0xfa81e3df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e1e3e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa421e3e5, SUC_lexicon_tags_37 },
    { 0xb9a1e3e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f61e3e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e1e3eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbf61e3ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4361e3f5, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xc401e3f7, SUC_lexicon_tags_37 },
    { 0x9fa1e3f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a01e3fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf21e402, SUC_lexicon_tags_63 },
    { 0xebe1e403, SUC_lexicon_tags_27 },
    { 0x3c81e402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e1e40f, SUC_lexicon_tags_74 },
    { 0x9981e410, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xdd41e412, SUC_lexicon_tags_17 },
    { 0xca1e412, SUC_lexicon_tags_12 },
    { 0xf2a1e414, SUC_lexicon_tags_27 },
    { 0x12e1e412, SUC_lexicon_tags_17 },
    { 0xe841e413, SUC_lexicon_tags_16 },
    { 0xedc1e417, SUC_lexicon_tags_25 },
    { 0x6f61e417, SUC_lexicon_tags_1 },
    { 0x3701e412, SUC_lexicon_tags_19 },
    { 0xa1e1e415, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fe1e41c, SUC_lexicon_tags_7 },
    { 0x46e1e41d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9341e421, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca61e424, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca41e42b, SUC_lexicon_tags_77 },
    { 0xb5a1e42b, SUC_lexicon_tags_38 },
    { 0x4901e42d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a1e430, SUC_lexicon_tags_37 },
    { 0x5301e431, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a1e436, SUC_lexicon_tags_7 },
    { 0x3741e437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f41e43b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de1e43e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f01e440, SUC_lexicon_tags_65 },
    { 0xb8c1e441, SUC_lexicon_tags_11 },
    { 0xd221e440, SUC_lexicon_tags_37 },
    { 0xc261e443, SUC_lexicon_tags_10 },
    { 0x2541e444, SUC_lexicon_tags_116 },
    { 0x9821e445, SUC_lexicon_tags_129 },
    { 0x9ac1e446, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee1e44a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee1e44d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe1e451, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf961e454, SUC_lexicon_tags_17 },
    { 0xce1e455, SUC_lexicon_tags_38 },
    { 0xb01e456, SUC_lexicon_tags_25 },
    { 0x84a1e457, SUC_lexicon_tags_48 },
    { 0x5301e458, SUC_lexicon_tags_39 },
    { 0xcdc1e458, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601e45f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac1e463, SUC_lexicon_tags_31 },
    { 0x7e01e464, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf01e468, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1141e46a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8b01e46c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a1e46f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a1e473, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9821e47b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6921e47e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xece1e480, SUC_lexicon_tags_15 },
    { 0xad21e481, SUC_lexicon_tags_27 },
    { 0x9121e482, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3821e485, SUC_lexicon_tags_2 },
    { 0xe01e485, SUC_lexicon_tags_20 },
    { 0xed81e485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a1e48b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeba1e48d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de1e490, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6881e494, SUC_lexicon_tags_11 },
    { 0x34c1e494, SUC_lexicon_tags_15 },
    { 0xe461e494, SUC_lexicon_tags_19 },
    { 0xe1e497, SUC_lexicon_tags_66 },
    { 0xdec1e498, SUC_lexicon_tags_77 },
    { 0xfb21e495, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f81e49b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc621e4a0, SUC_lexicon_tags_47 },
    { 0x5ae1e4a0, SUC_lexicon_tags_14 },
    { 0xac41e4a1, SUC_lexicon_tags_15 },
    { 0x63c1e4a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1581e4a5, SUC_lexicon_tags_11 },
    { 0x2521e4a6, SUC_lexicon_tags_62 },
    { 0x1e41e4a6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc281e4b0, SUC_lexicon_tags_25 },
    { 0x9f61e4b1, SUC_lexicon_tags_15 },
    { 0x1fc1e4b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a21e4b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58c1e4b8, SUC_lexicon_tags_84 },
    { 0x3861e4b8, SUC_lexicon_tags_47 },
    { 0x1a1e4ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae1e4bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9681e4c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9781e4c5, SUC_lexicon_tags_11 },
    { 0x4421e4c6, SUC_lexicon_tags_48 },
    { 0xde41e4c7, SUC_lexicon_tags_43 },
    { 0x5aa1e4c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe861e4ca, SUC_lexicon_tags_12 },
    { 0x4081e4cb, SUC_lexicon_tags_39 },
    { 0x161e4cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2301e4d8, SUC_lexicon_tags_31 },
    { 0x3ec1e4d8, SUC_lexicon_tags_19 },
    { 0x5881e4da, SUC_lexicon_tags_47 },
    { 0xa281e4d8, SUC_lexicon_tags_37 },
    { 0x6421e4d8, SUC_lexicon_tags_433 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae1e4df, SUC_lexicon_tags_11 },
    { 0x4a61e4e0, SUC_lexicon_tags_8 },
    { 0x4901e4df, SUC_lexicon_tags_17 },
    { 0x3ca1e4e2, SUC_lexicon_tags_25 },
    { 0x9821e4e2, SUC_lexicon_tags_44 },
    { 0x1a61e4e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e1e4ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a1e4ed, SUC_lexicon_tags_25 },
    { 0xb5c1e4ee, SUC_lexicon_tags_87 },
    { 0xda21e4ee, SUC_lexicon_tags_47 },
    { 0x7581e4ef, SUC_lexicon_tags_26 },
    { 0xe1a1e4f1, SUC_lexicon_tags_43 },
    { 0x6ea1e4f2, SUC_lexicon_tags_12 },
    { 0xe4a1e4f2, SUC_lexicon_tags_15 },
    { 0xf101e4f2, SUC_lexicon_tags_46 },
    { 0xe221e4f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c21e4f9, SUC_lexicon_tags_15 },
    { 0xd761e4f9, SUC_lexicon_tags_15 },
    { 0x1f01e4f9, SUC_lexicon_tags_17 },
    { 0x5d81e4f9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c01e500, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdba1e502, SUC_lexicon_tags_68 },
    { 0xdf81e503, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d01e507, SUC_lexicon_tags_43 },
    { 0x9dc1e507, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc241e50a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c1e50e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad41e510, SUC_lexicon_tags_2 },
    { 0xd161e511, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb201e515, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e1e519, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc1e51d, SUC_lexicon_tags_49 },
    { 0x16a1e51e, SUC_lexicon_tags_14 },
    { 0x4721e51f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5961e522, SUC_lexicon_tags_0 },
    { 0x7f61e523, SUC_lexicon_tags_23 },
    { 0xfec1e524, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6781e526, SUC_lexicon_tags_11 },
    { 0x2e21e527, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc361e52b, SUC_lexicon_tags_0 },
    { 0xea61e52c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc1e530, SUC_lexicon_tags_24 },
    { 0x28c1e531, SUC_lexicon_tags_37 },
    { 0x3a1e531, SUC_lexicon_tags_16 },
    { 0x9161e533, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc61e53c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c1e53e, SUC_lexicon_tags_7 },
    { 0xd5c1e53e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed01e542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf161e545, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa01e54a, SUC_lexicon_tags_40 },
    { 0xa561e54b, SUC_lexicon_tags_21 },
    { 0x4b21e54a, SUC_lexicon_tags_19 },
    { 0xf441e54c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2061e550, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a1e55e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf881e562, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac1e565, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x54e1e567, SUC_lexicon_tags_61 },
    { 0x8f21e568, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf761e56d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc101e571, SUC_lexicon_tags_19 },
    { 0xcce1e572, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf9a1e574, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01e57b, SUC_lexicon_tags_15 },
    { 0xd7e1e57b, SUC_lexicon_tags_15 },
    { 0xb601e57d, SUC_lexicon_tags_11 },
    { 0x7121e57d, SUC_lexicon_tags_47 },
    { 0xa721e57f, SUC_lexicon_tags_0 },
    { 0x40a1e57d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21e587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe101e58d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe381e58f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa1e593, SUC_lexicon_tags_28 },
    { 0x8201e593, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1f21e596, SUC_lexicon_tags_32 },
    { 0xdfc1e596, SUC_lexicon_tags_27 },
    { 0x9ec1e598, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5281e59a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1761e59d, SUC_lexicon_tags_45 },
    { 0xee21e59e, SUC_lexicon_tags_12 },
    { 0x5e81e59e, SUC_lexicon_tags_56 },
    { 0x1301e59f, SUC_lexicon_tags_19 },
    { 0x5a01e59e, SUC_lexicon_tags_49 },
    { 0xa881e5a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c1e5a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7021e5a9, SUC_lexicon_tags_32 },
    { 0x6ac1e5aa, SUC_lexicon_tags_12 },
    { 0xb061e5ab, SUC_lexicon_tags_30 },
    { 0x43a1e5ab, SUC_lexicon_tags_1 },
    { 0xb821e5ad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6341e5b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x201e5b2, SUC_lexicon_tags_19 },
    { 0x50e1e5b3, SUC_lexicon_tags_49 },
    { 0xf1c1e5b3, SUC_lexicon_tags_3 },
    { 0x6ac1e5b5, SUC_lexicon_tags_11 },
    { 0x7241e5b6, SUC_lexicon_tags_17 },
    { 0x8181e5b6, SUC_lexicon_tags_39 },
    { 0xf1a1e5b8, SUC_lexicon_tags_11 },
    { 0x3c1e5b9, SUC_lexicon_tags_11 },
    { 0x29a1e5b8, SUC_lexicon_tags_38 },
    { 0x93e1e5b6, SUC_lexicon_tags_60 },
    { 0x5f41e5ba, SUC_lexicon_tags_14 },
    { 0xd241e5bd, SUC_lexicon_tags_60 },
    { 0x8b61e5bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf81e5c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa781e5c6, SUC_lexicon_tags_9 },
    { 0xaa1e5c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7fc1e5c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d61e5d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2781e5d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc901e5dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xafa1e5de, SUC_lexicon_tags_37 },
    { 0xc5e1e5df, SUC_lexicon_tags_40 },
    { 0xd341e5e0, SUC_lexicon_tags_15 },
    { 0x4c41e5e1, SUC_lexicon_tags_48 },
    { 0x2601e5e2, SUC_lexicon_tags_17 },
    { 0x7b61e5e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e41e5e7, SUC_lexicon_tags_31 },
    { 0xc21e5e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a1e5eb, SUC_lexicon_tags_14 },
    { 0x3921e5eb, SUC_lexicon_tags_45 },
    { 0xa6c1e5eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4521e5f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc961e5f5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c1e5fc, SUC_lexicon_tags_37 },
    { 0xebe1e5fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba1e603, SUC_lexicon_tags_37 },
    { 0xfa01e604, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa41e607, SUC_lexicon_tags_7 },
    { 0x20a1e608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x781e60a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5341e612, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc921e618, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6801e61b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8221e61e, SUC_lexicon_tags_19 },
    { 0xa481e61f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e1e623, SUC_lexicon_tags_45 },
    { 0xd4e1e623, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a1e628, SUC_lexicon_tags_17 },
    { 0xb0a1e628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fc1e62b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb741e62d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe1e632, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa1e634, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea1e639, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e01e641, SUC_lexicon_tags_8 },
    { 0xbcc1e642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3081e647, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8421e649, SUC_lexicon_tags_25 },
    { 0x54c1e64a, SUC_lexicon_tags_13 },
    { 0xa721e649, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa1e64e, SUC_lexicon_tags_15 },
    { 0x681e64f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a1e654, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc1e657, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x99c1e659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1e65c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x1e1e65e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9961e660, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab41e663, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c1e669, SUC_lexicon_tags_11 },
    { 0x49e1e66a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf81e670, SUC_lexicon_tags_11 },
    { 0x5861e671, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2601e675, SUC_lexicon_tags_39 },
    { 0xf681e676, SUC_lexicon_tags_43 },
    { 0x16e1e677, SUC_lexicon_tags_37 },
    { 0xa861e678, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c01e67b, SUC_lexicon_tags_7 },
    { 0xa301e67b, SUC_lexicon_tags_9 },
    { 0xf701e67b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed41e680, SUC_lexicon_tags_37 },
    { 0x3421e681, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e21e689, SUC_lexicon_tags_27 },
    { 0x4e61e689, SUC_lexicon_tags_22 },
    { 0x1de1e68b, SUC_lexicon_tags_47 },
    { 0x19c1e68c, SUC_lexicon_tags_27 },
    { 0xc21e68c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1501e694, SUC_lexicon_tags_25 },
    { 0x27c1e694, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19a1e697, SUC_lexicon_tags_48 },
    { 0xe8a1e698, SUC_lexicon_tags_1 },
    { 0x5f61e699, SUC_lexicon_tags_16 },
    { 0xe8e1e69a, SUC_lexicon_tags_7 },
    { 0x36a1e69b, SUC_lexicon_tags_51 },
    { 0xf3a1e69c, SUC_lexicon_tags_11 },
    { 0x5c21e69d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e6a3, SUC_lexicon_tags_38 },
    { 0xf801e6a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7521e6a6, SUC_lexicon_tags_8 },
    { 0x9901e6a7, SUC_lexicon_tags_15 },
    { 0x9961e6a8, SUC_lexicon_tags_17 },
    { 0x16c1e6a9, SUC_lexicon_tags_2 },
    { 0x8341e6a9, SUC_lexicon_tags_25 },
    { 0x6701e6ab, SUC_lexicon_tags_58 },
    { 0x2481e6ac, SUC_lexicon_tags_29 },
    { 0x6b41e6ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a1e6b2, SUC_lexicon_tags_21 },
    { 0x7a41e6b3, SUC_lexicon_tags_28 },
    { 0x3621e6b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6881e6b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a1e6ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed81e6bf, SUC_lexicon_tags_11 },
    { 0x52a1e6c0, SUC_lexicon_tags_17 },
    { 0x13c1e6c1, SUC_lexicon_tags_1 },
    { 0x5d21e6c0, SUC_lexicon_tags_55 },
    { 0x7da1e6c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5581e6c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb221e6ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8921e6d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e1e6e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc341e6ef, SUC_lexicon_tags_14 },
    { 0x13e1e6f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c61e6f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe341e6f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xab61e6f8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c1e6fb, SUC_lexicon_tags_45 },
    { 0xe901e6fc, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a21e700, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a1e703, SUC_lexicon_tags_15 },
    { 0xb341e703, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8661e707, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e1e70a, SUC_lexicon_tags_80 },
    { 0x34c1e70b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f41e70d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc1e715, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e61e719, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e1e71d, SUC_lexicon_tags_4 },
    { 0xc581e71e, SUC_lexicon_tags_23 },
    { 0x1541e71f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb281e722, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a41e724, SUC_lexicon_tags_82 },
    { 0x36a1e725, SUC_lexicon_tags_16 },
    { 0x2a21e725, SUC_lexicon_tags_11 },
    { 0x1841e725, SUC_lexicon_tags_2 },
    { 0x2561e728, SUC_lexicon_tags_30 },
    { 0xf821e729, SUC_lexicon_tags_26 },
    { 0xba81e728, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a01e72f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b01e731, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba01e735, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be1e73d, SUC_lexicon_tags_9 },
    { 0xf381e73e, SUC_lexicon_tags_37 },
    { 0xa61e73f, SUC_lexicon_tags_1 },
    { 0x4f01e740, SUC_lexicon_tags_21 },
    { 0x8241e740, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b61e745, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8681e751, SUC_lexicon_tags_55 },
    { 0x1761e752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec1e758, SUC_lexicon_tags_1 },
    { 0x2081e759, SUC_lexicon_tags_37 },
    { 0xf9e1e758, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x47a1e75c, SUC_lexicon_tags_1 },
    { 0x6c21e75d, SUC_lexicon_tags_1 },
    { 0xad21e75e, SUC_lexicon_tags_1 },
    { 0xd601e75d, SUC_lexicon_tags_12 },
    { 0x3041e760, SUC_lexicon_tags_27 },
    { 0x95e1e760, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc561e763, SUC_lexicon_tags_7 },
    { 0x4f81e764, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b81e768, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd101e76b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc1e770, SUC_lexicon_tags_23 },
    { 0x1e1e770, SUC_lexicon_tags_17 },
    { 0x24e1e772, SUC_lexicon_tags_26 },
    { 0x7d41e773, SUC_lexicon_tags_8 },
    { 0xe9a1e773, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf481e778, SUC_lexicon_tags_26 },
    { 0x4c81e779, SUC_lexicon_tags_25 },
    { 0x5561e77a, SUC_lexicon_tags_17 },
    { 0x7421e77b, SUC_lexicon_tags_6 },
    { 0x4381e77c, SUC_lexicon_tags_7 },
    { 0xddc1e77b, SUC_lexicon_tags_37 },
    { 0x2881e77e, SUC_lexicon_tags_48 },
    { 0xb2c1e77b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e1e784, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6161e787, SUC_lexicon_tags_8 },
    { 0x4bc1e787, SUC_lexicon_tags_1 },
    { 0x1641e789, SUC_lexicon_tags_37 },
    { 0xa61e789, SUC_lexicon_tags_16 },
    { 0xe41e78b, SUC_lexicon_tags_25 },
    { 0x1d61e78b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e1e78f, SUC_lexicon_tags_11 },
    { 0xe9e1e78f, SUC_lexicon_tags_25 },
    { 0xf681e791, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e1e794, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee1e798, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e1e79c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f81e79e, SUC_lexicon_tags_37 },
    { 0x1a01e79f, SUC_lexicon_tags_10 },
    { 0x4161e79f, SUC_lexicon_tags_12 },
    { 0x5c61e79f, SUC_lexicon_tags_77 },
    { 0x1881e7a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c1e7a5, SUC_lexicon_tags_15 },
    { 0xb101e7a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcc1e7a8, SUC_lexicon_tags_17 },
    { 0xf5c1e7a9, SUC_lexicon_tags_1 },
    { 0xcc61e7aa, SUC_lexicon_tags_9 },
    { 0xef01e7ab, SUC_lexicon_tags_4 },
    { 0x6781e7ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9981e7af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc1e7b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcb81e7b4, SUC_lexicon_tags_43 },
    { 0xeca1e7b5, SUC_lexicon_tags_584 },
    { 0, NULL },
    { 0x5b01e7b7, SUC_lexicon_tags_29 },
    { 0x94c1e7b8, SUC_lexicon_tags_37 },
    { 0x1e1e7b7, SUC_lexicon_tags_12 },
    { 0xedc1e7ba, SUC_lexicon_tags_37 },
    { 0xdac1e7bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b41e7bd, SUC_lexicon_tags_364 },
    { 0x9901e7be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3781e7c1, SUC_lexicon_tags_15 },
    { 0x3fc1e7c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c1e7c6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae1e7ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc421e7cc, SUC_lexicon_tags_25 },
    { 0xc8c1e7cd, SUC_lexicon_tags_1 },
    { 0xbc21e7ce, SUC_lexicon_tags_61 },
    { 0x5321e7cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ba1e7d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c1e7d6, SUC_lexicon_tags_64 },
    { 0x65c1e7d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb301e7de, SUC_lexicon_tags_2 },
    { 0xa101e7df, SUC_lexicon_tags_2 },
    { 0x28e1e7df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e1e7e3, SUC_lexicon_tags_1 },
    { 0xfa1e7e4, SUC_lexicon_tags_66 },
    { 0x1b61e7e5, SUC_lexicon_tags_28 },
    { 0xc881e7e3, SUC_lexicon_tags_27 },
    { 0x6221e7e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2561e7ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e7f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e1e7f5, SUC_lexicon_tags_8 },
    { 0xa301e7f6, SUC_lexicon_tags_0 },
    { 0x5e41e7f5, SUC_lexicon_tags_37 },
    { 0x23c1e7f8, SUC_lexicon_tags_15 },
    { 0x5fa1e7f5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x20a1e7fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a1e7fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5c1e800, SUC_lexicon_tags_74 },
    { 0x8301e801, SUC_lexicon_tags_37 },
    { 0x53c1e802, SUC_lexicon_tags_12 },
    { 0x1041e803, SUC_lexicon_tags_10 },
    { 0x7181e803, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe061e807, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1721e80b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d61e810, SUC_lexicon_tags_65 },
    { 0xfc01e811, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x41a1e813, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5101e815, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd41e81a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9821e81c, SUC_lexicon_tags_10 },
    { 0xf081e81d, SUC_lexicon_tags_30 },
    { 0xa3c1e81c, SUC_lexicon_tags_3 },
    { 0xbf21e81f, SUC_lexicon_tags_15 },
    { 0xd7a1e81c, SUC_lexicon_tags_39 },
    { 0x2861e821, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5141e825, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6261e82f, SUC_lexicon_tags_7 },
    { 0x99e1e830, SUC_lexicon_tags_2 },
    { 0x5521e82f, SUC_lexicon_tags_71 },
    { 0x9221e82f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a1e835, SUC_lexicon_tags_2 },
    { 0xd1e1e836, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xece1e838, SUC_lexicon_tags_113 },
    { 0xfa1e839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe1e83f, SUC_lexicon_tags_63 },
    { 0x5901e840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce21e842, SUC_lexicon_tags_39 },
    { 0xce1e843, SUC_lexicon_tags_43 },
    { 0x2fa1e844, SUC_lexicon_tags_31 },
    { 0x6181e845, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c1e849, SUC_lexicon_tags_34 },
    { 0x6b81e84a, SUC_lexicon_tags_15 },
    { 0x55e1e84b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb01e84e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x7281e850, SUC_lexicon_tags_3 },
    { 0xbd01e851, SUC_lexicon_tags_107 },
    { 0xa901e852, SUC_lexicon_tags_14 },
    { 0xd501e850, SUC_lexicon_tags_11 },
    { 0x8f01e852, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9881e856, SUC_lexicon_tags_11 },
    { 0xfa01e856, SUC_lexicon_tags_37 },
    { 0xd6c1e856, SUC_lexicon_tags_7 },
    { 0x2801e857, SUC_lexicon_tags_22 },
    { 0xbb41e856, SUC_lexicon_tags_12 },
    { 0xec01e85b, SUC_lexicon_tags_31 },
    { 0x8961e856, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1e860, SUC_lexicon_tags_17 },
    { 0x5301e861, SUC_lexicon_tags_3 },
    { 0x1941e862, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x81c1e864, SUC_lexicon_tags_23 },
    { 0x51e1e864, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec1e868, SUC_lexicon_tags_33 },
    { 0x86c1e869, SUC_lexicon_tags_30 },
    { 0xebe1e86a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9701e86f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37a1e871, SUC_lexicon_tags_15 },
    { 0x9121e872, SUC_lexicon_tags_33 },
    { 0xa4c1e873, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c1e876, SUC_lexicon_tags_11 },
    { 0x5921e877, SUC_lexicon_tags_60 },
    { 0xbce1e878, SUC_lexicon_tags_37 },
    { 0xf981e879, SUC_lexicon_tags_15 },
    { 0xe0a1e87a, SUC_lexicon_tags_16 },
    { 0x2661e87a, SUC_lexicon_tags_12 },
    { 0x4941e877, SUC_lexicon_tags_15 },
    { 0xd7a1e87d, SUC_lexicon_tags_25 },
    { 0xdb41e87b, SUC_lexicon_tags_13 },
    { 0xe521e87b, SUC_lexicon_tags_0 },
    { 0x3581e87c, SUC_lexicon_tags_17 },
    { 0x6901e87f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6541e885, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a01e88b, SUC_lexicon_tags_27 },
    { 0xe0e1e88b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a1e890, SUC_lexicon_tags_82 },
    { 0x4a41e891, SUC_lexicon_tags_17 },
    { 0xf1c1e891, SUC_lexicon_tags_28 },
    { 0xd301e893, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff61e897, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc761e899, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e1e89e, SUC_lexicon_tags_23 },
    { 0x9681e89f, SUC_lexicon_tags_2 },
    { 0xa1e1e8a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b41e8a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721e8a9, SUC_lexicon_tags_23 },
    { 0x8361e8aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8421e8ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf961e8ae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c1e8b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e41e8b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8321e8b8, SUC_lexicon_tags_12 },
    { 0x1e21e8b8, SUC_lexicon_tags_0 },
    { 0x5b01e8b8, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda1e8c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6441e8c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb21e8c8, SUC_lexicon_tags_12 },
    { 0x7041e8c9, SUC_lexicon_tags_37 },
    { 0xc201e8ca, SUC_lexicon_tags_17 },
    { 0xb2a1e8cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8781e8ce, SUC_lexicon_tags_39 },
    { 0x43c1e8cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9861e8d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3601e8d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e1e8d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c1e8db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe821e8dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de1e8e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd481e8e5, SUC_lexicon_tags_65 },
    { 0x4f61e8e6, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xaae1e8e8, SUC_lexicon_tags_26 },
    { 0xa6e1e8e9, SUC_lexicon_tags_38 },
    { 0x1a61e8e9, SUC_lexicon_tags_19 },
    { 0x94a1e8e8, SUC_lexicon_tags_17 },
    { 0x2621e8ec, SUC_lexicon_tags_1 },
    { 0xa7a1e8ec, SUC_lexicon_tags_12 },
    { 0x36e1e8ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa501e8f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee01e8f3, SUC_lexicon_tags_1 },
    { 0x1a41e8f4, SUC_lexicon_tags_49 },
    { 0xa201e8f4, SUC_lexicon_tags_15 },
    { 0xbfe1e8f6, SUC_lexicon_tags_39 },
    { 0xec81e8f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1761e8fd, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c1e900, SUC_lexicon_tags_37 },
    { 0xb0c1e900, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc1e908, SUC_lexicon_tags_2 },
    { 0x9dc1e909, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4861e90d, SUC_lexicon_tags_1 },
    { 0xc201e90d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e21e911, SUC_lexicon_tags_47 },
    { 0xad81e912, SUC_lexicon_tags_37 },
    { 0x8181e912, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c41e918, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15a1e91a, SUC_lexicon_tags_7 },
    { 0x6441e91a, SUC_lexicon_tags_38 },
    { 0x8821e91c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa941e91e, SUC_lexicon_tags_5 },
    { 0x3361e91f, SUC_lexicon_tags_65 },
    { 0x72a1e920, SUC_lexicon_tags_28 },
    { 0x2a41e920, SUC_lexicon_tags_2 },
    { 0xb201e91f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c1e927, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e1e92d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d61e930, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3381e935, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4821e937, SUC_lexicon_tags_25 },
    { 0x4f41e938, SUC_lexicon_tags_4 },
    { 0x4c01e939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8361e93b, SUC_lexicon_tags_16 },
    { 0x8361e93c, SUC_lexicon_tags_33 },
    { 0xc1e1e93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2561e940, SUC_lexicon_tags_39 },
    { 0xe3c1e940, SUC_lexicon_tags_35 },
    { 0x7881e940, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e1e947, SUC_lexicon_tags_7 },
    { 0x6ea1e948, SUC_lexicon_tags_43 },
    { 0xeee1e949, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4ca1e94b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281e951, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x65e1e953, SUC_lexicon_tags_11 },
    { 0x5e61e954, SUC_lexicon_tags_28 },
    { 0x2de1e955, SUC_lexicon_tags_9 },
    { 0xcc41e956, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb681e95a, SUC_lexicon_tags_23 },
    { 0xf301e95b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a1e95f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361e964, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7ac1e966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8881e968, SUC_lexicon_tags_1 },
    { 0xd141e968, SUC_lexicon_tags_17 },
    { 0x7841e969, SUC_lexicon_tags_4 },
    { 0x6001e96b, SUC_lexicon_tags_37 },
    { 0x7fe1e96c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e41e96f, SUC_lexicon_tags_25 },
    { 0xbf41e970, SUC_lexicon_tags_1 },
    { 0xd321e971, SUC_lexicon_tags_45 },
    { 0x4121e972, SUC_lexicon_tags_12 },
    { 0xce81e973, SUC_lexicon_tags_38 },
    { 0x9a21e972, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdb41e976, SUC_lexicon_tags_2 },
    { 0x9881e976, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdbe1e979, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a1e97d, SUC_lexicon_tags_1 },
    { 0x6a41e97d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8201e981, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3101e987, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8881e989, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7e1e98b, SUC_lexicon_tags_30 },
    { 0xf061e98b, SUC_lexicon_tags_15 },
    { 0x7421e98d, SUC_lexicon_tags_26 },
    { 0x28e1e98e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a21e995, SUC_lexicon_tags_25 },
    { 0xf541e996, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x341e999, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a1e99d, SUC_lexicon_tags_11 },
    { 0xa4e1e99e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde01e9a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfcc1e9a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1901e9a9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5521e9ad, SUC_lexicon_tags_37 },
    { 0x7b01e9ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe1e9b1, SUC_lexicon_tags_14 },
    { 0xa2a1e9b2, SUC_lexicon_tags_39 },
    { 0xd821e9b3, SUC_lexicon_tags_37 },
    { 0xec61e9b4, SUC_lexicon_tags_15 },
    { 0x6f61e9b1, SUC_lexicon_tags_37 },
    { 0xf241e9b6, SUC_lexicon_tags_15 },
    { 0x48e1e9b6, SUC_lexicon_tags_8 },
    { 0x5381e9b8, SUC_lexicon_tags_66 },
    { 0xdc21e9b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9981e9c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a21e9c2, SUC_lexicon_tags_27 },
    { 0xb7a1e9c3, SUC_lexicon_tags_17 },
    { 0x1ea1e9c4, SUC_lexicon_tags_11 },
    { 0x4621e9c3, SUC_lexicon_tags_20 },
    { 0x9161e9c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa1e9ce, SUC_lexicon_tags_66 },
    { 0xbde1e9ce, SUC_lexicon_tags_8 },
    { 0x7f01e9d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3501e9df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc81e9e2, SUC_lexicon_tags_47 },
    { 0xa7a1e9e3, SUC_lexicon_tags_37 },
    { 0x8b21e9e4, SUC_lexicon_tags_66 },
    { 0x1fe1e9e5, SUC_lexicon_tags_15 },
    { 0xbd21e9e6, SUC_lexicon_tags_37 },
    { 0xf201e9e7, SUC_lexicon_tags_23 },
    { 0x2941e9e8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e1e9ed, SUC_lexicon_tags_43 },
    { 0x7981e9ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c1e9fb, SUC_lexicon_tags_29 },
    { 0xd821e9fb, SUC_lexicon_tags_45 },
    { 0x8921e9fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8481ea02, SUC_lexicon_tags_7 },
    { 0xcc21ea02, SUC_lexicon_tags_8 },
    { 0xe381ea04, SUC_lexicon_tags_29 },
    { 0x7a41ea04, SUC_lexicon_tags_43 },
    { 0x4de1ea06, SUC_lexicon_tags_28 },
    { 0x5781ea07, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd7c1ea09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe321ea0b, SUC_lexicon_tags_1 },
    { 0x1ae1ea0b, SUC_lexicon_tags_15 },
    { 0x5841ea0c, SUC_lexicon_tags_57 },
    { 0x1de1ea0e, SUC_lexicon_tags_37 },
    { 0x78a1ea0f, SUC_lexicon_tags_243 },
    { 0xf381ea10, SUC_lexicon_tags_51 },
    { 0x7d21ea0f, SUC_lexicon_tags_31 },
    { 0x8421ea0c, SUC_lexicon_tags_32 },
    { 0xd4a1ea13, SUC_lexicon_tags_31 },
    { 0xdae1ea14, SUC_lexicon_tags_47 },
    { 0xb581ea0c, SUC_lexicon_tags_37 },
    { 0x37c1ea15, SUC_lexicon_tags_7 },
    { 0x6ce1ea17, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee61ea19, SUC_lexicon_tags_8 },
    { 0xfac1ea1a, SUC_lexicon_tags_7 },
    { 0xe0c1ea1b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa1ea21, SUC_lexicon_tags_12 },
    { 0x2ee1ea22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce1ea28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4921ea2c, SUC_lexicon_tags_47 },
    { 0xc01ea2c, SUC_lexicon_tags_23 },
    { 0x2e81ea2c, SUC_lexicon_tags_13 },
    { 0x9da1ea2f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe81ea31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7a1ea33, SUC_lexicon_tags_7 },
    { 0xcba1ea34, SUC_lexicon_tags_47 },
    { 0x9701ea35, SUC_lexicon_tags_19 },
    { 0x3d81ea36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa701ea38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8441ea3a, SUC_lexicon_tags_12 },
    { 0xf561ea3b, SUC_lexicon_tags_60 },
    { 0x7201ea3a, SUC_lexicon_tags_61 },
    { 0xe4c1ea3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda01ea40, SUC_lexicon_tags_1 },
    { 0xc9a1ea41, SUC_lexicon_tags_8 },
    { 0xff21ea42, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa581ea44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3001ea46, SUC_lexicon_tags_49 },
    { 0x9b61ea46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621ea4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8621ea4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe41ea51, SUC_lexicon_tags_40 },
    { 0x7481ea51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c41ea56, SUC_lexicon_tags_14 },
    { 0x30e1ea57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e1ea5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea1ea5f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd61ea64, SUC_lexicon_tags_37 },
    { 0xc21ea65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e1ea6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf701ea71, SUC_lexicon_tags_31 },
    { 0xe4c1ea71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a1ea75, SUC_lexicon_tags_9 },
    { 0x64a1ea76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68c1ea78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f81ea80, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf101ea84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce41ea86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca1ea88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1501ea8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e1ea91, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c01ea96, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x8061ea98, SUC_lexicon_tags_21 },
    { 0x7ec1ea99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f61ea9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a1ea9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a1eaa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e41eaa3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd721eaa6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c1eaa9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c61eaac, SUC_lexicon_tags_49 },
    { 0xc701eaad, SUC_lexicon_tags_12 },
    { 0x94c1eaad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea1eab3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c61eab5, SUC_lexicon_tags_1 },
    { 0xe701eab5, SUC_lexicon_tags_16 },
    { 0xa341eab7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3181eab9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b41eabb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6721eabf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce1eac4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b01eac6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb21eac9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99c1eacb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee1ead0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a21ead4, SUC_lexicon_tags_11 },
    { 0xb61ead5, SUC_lexicon_tags_17 },
    { 0x6621ead6, SUC_lexicon_tags_15 },
    { 0x2321ead6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd841eadc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd41eae2, SUC_lexicon_tags_16 },
    { 0x8ce1eae3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a01eae7, SUC_lexicon_tags_30 },
    { 0x1341eae8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc41eaea, SUC_lexicon_tags_63 },
    { 0xbbc1eaeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3181eaee, SUC_lexicon_tags_12 },
    { 0xee1eaee, SUC_lexicon_tags_75 },
    { 0x5901eaf0, SUC_lexicon_tags_47 },
    { 0x8ae1eaf0, SUC_lexicon_tags_3 },
    { 0x7461eaef, SUC_lexicon_tags_106 },
    { 0x9061eaf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa141eaf8, SUC_lexicon_tags_48 },
    { 0x21c1eaf9, SUC_lexicon_tags_38 },
    { 0xe0e1eafa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d41eb05, SUC_lexicon_tags_37 },
    { 0x8661eb05, SUC_lexicon_tags_30 },
    { 0xe01eb07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c61eb0a, SUC_lexicon_tags_47 },
    { 0xebe1eb0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81eb10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641eb16, SUC_lexicon_tags_15 },
    { 0xb3e1eb17, SUC_lexicon_tags_0 },
    { 0x8d41eb16, SUC_lexicon_tags_17 },
    { 0xe41eb19, SUC_lexicon_tags_26 },
    { 0xc21eb1a, SUC_lexicon_tags_27 },
    { 0x5621eb1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd81eb1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6a1eb1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4861eb26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb41eb28, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce21eb2d, SUC_lexicon_tags_8 },
    { 0x52a1eb2e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9901eb30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c1eb35, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x11a1eb37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c1eb3a, SUC_lexicon_tags_18 },
    { 0x4881eb3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf21eb3f, SUC_lexicon_tags_28 },
    { 0x4f01eb3f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a1eb43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e1eb46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41eb49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1eb4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e41eb51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef81eb54, SUC_lexicon_tags_1 },
    { 0x33c1eb55, SUC_lexicon_tags_116 },
    { 0xade1eb54, SUC_lexicon_tags_17 },
    { 0x6041eb57, SUC_lexicon_tags_25 },
    { 0x3981eb57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc1eb5b, SUC_lexicon_tags_8 },
    { 0xa541eb5b, SUC_lexicon_tags_5 },
    { 0x4b01eb5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9821eb5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4a01eb61, SUC_lexicon_tags_57 },
    { 0x8281eb62, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa201eb65, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f61eb68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf41eb6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x44e1eb6d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4921eb75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b01eb7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b81eb83, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1fe1eb85, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x2481eb87, SUC_lexicon_tags_26 },
    { 0xe941eb88, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4841eb8a, SUC_lexicon_tags_66 },
    { 0x3f61eb8a, SUC_lexicon_tags_7 },
    { 0x5181eb8c, SUC_lexicon_tags_19 },
    { 0x9a61eb8c, SUC_lexicon_tags_37 },
    { 0xc341eb8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c61eb90, SUC_lexicon_tags_32 },
    { 0x38c1eb91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa201eb94, SUC_lexicon_tags_17 },
    { 0x2461eb94, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x31c1eb97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d61eb9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6761eba4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c1eba8, SUC_lexicon_tags_37 },
    { 0x1601eba9, SUC_lexicon_tags_11 },
    { 0x4b81eba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb741ebb2, SUC_lexicon_tags_30 },
    { 0x74e1ebb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5161ebb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd301ebb7, SUC_lexicon_tags_1 },
    { 0xefc1ebb8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c1ebbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4861ebc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8801ebc5, SUC_lexicon_tags_2 },
    { 0xc1c1ebc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12e1ebc8, SUC_lexicon_tags_19 },
    { 0xa901ebc9, SUC_lexicon_tags_1 },
    { 0x30e1ebc9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1ebcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3781ebd3, SUC_lexicon_tags_1 },
    { 0x1881ebd4, SUC_lexicon_tags_27 },
    { 0x5181ebd5, SUC_lexicon_tags_21 },
    { 0xd221ebd6, SUC_lexicon_tags_11 },
    { 0xa21ebd6, SUC_lexicon_tags_0 },
    { 0x2221ebd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7781ebda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b81ebdc, SUC_lexicon_tags_12 },
    { 0x9ec1ebdd, SUC_lexicon_tags_8 },
    { 0x6121ebde, SUC_lexicon_tags_7 },
    { 0xe61ebde, SUC_lexicon_tags_27 },
    { 0x4081ebde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a1ebe4, SUC_lexicon_tags_14 },
    { 0x8a61ebe5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c1ebe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a1ebea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf621ebec, SUC_lexicon_tags_48 },
    { 0xc7e1ebec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8501ebf2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4541ebf5, SUC_lexicon_tags_88 },
    { 0x7261ebf6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e1ebf9, SUC_lexicon_tags_70 },
    { 0xa6c1ebfa, SUC_lexicon_tags_12 },
    { 0x39c1ebfb, SUC_lexicon_tags_12 },
    { 0x77c1ebfb, SUC_lexicon_tags_14 },
    { 0xa381ebfa, SUC_lexicon_tags_60 },
    { 0x42a1ebfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d81ec00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c1ec0a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2e1ec0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae41ec0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf261ec1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2861ec25, SUC_lexicon_tags_92 },
    { 0x2861ec26, SUC_lexicon_tags_12 },
    { 0x72e1ec27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e1ec2a, SUC_lexicon_tags_40 },
    { 0x7ce1ec2a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7601ec2d, SUC_lexicon_tags_33 },
    { 0x6d21ec2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61ec35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef41ec38, SUC_lexicon_tags_12 },
    { 0xbdc1ec39, SUC_lexicon_tags_66 },
    { 0x26a1ec3a, SUC_lexicon_tags_11 },
    { 0x4301ec39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b21ec3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa1ec40, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e1ec46, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xf7c1ec48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e1ec4f, SUC_lexicon_tags_2 },
    { 0xa1e1ec50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aa1ec52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa1ec55, SUC_lexicon_tags_2 },
    { 0x5181ec56, SUC_lexicon_tags_17 },
    { 0xf2e1ec57, SUC_lexicon_tags_1 },
    { 0xd661ec55, SUC_lexicon_tags_27 },
    { 0xc9e1ec55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3e1ec5b, SUC_lexicon_tags_45 },
    { 0x9de1ec5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd341ec5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a1ec68, SUC_lexicon_tags_11 },
    { 0x30e1ec69, SUC_lexicon_tags_16 },
    { 0x6e41ec68, SUC_lexicon_tags_71 },
    { 0x70e1ec69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1721ec6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2481ec73, SUC_lexicon_tags_87 },
    { 0xe801ec74, SUC_lexicon_tags_8 },
    { 0xa701ec74, SUC_lexicon_tags_15 },
    { 0xb3e1ec76, SUC_lexicon_tags_47 },
    { 0x1741ec77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe481ec7b, SUC_lexicon_tags_60 },
    { 0xe741ec7c, SUC_lexicon_tags_17 },
    { 0x2301ec7b, SUC_lexicon_tags_37 },
    { 0xe121ec7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dc1ec80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc41ec82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa401ec86, SUC_lexicon_tags_43 },
    { 0xd261ec87, SUC_lexicon_tags_0 },
    { 0x4a81ec86, SUC_lexicon_tags_37 },
    { 0xe181ec89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9441ec8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbaa1ec8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc781ec90, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1081ec92, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1101ec95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdc1ec97, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdd21ec99, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2941ec9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc201ec9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c1eca0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1be1eca2, SUC_lexicon_tags_10 },
    { 0xc461eca3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf461eca5, SUC_lexicon_tags_35 },
    { 0x27c1eca6, SUC_lexicon_tags_37 },
    { 0xb621eca7, SUC_lexicon_tags_47 },
    { 0x5de1eca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc161ecac, SUC_lexicon_tags_14 },
    { 0xa461ecac, SUC_lexicon_tags_8 },
    { 0xbe01ecae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa421ecb1, SUC_lexicon_tags_33 },
    { 0x1201ecb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e81ecb5, SUC_lexicon_tags_62 },
    { 0xd7a1ecb5, SUC_lexicon_tags_9 },
    { 0xfba1ecb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1ecbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b61ecbf, SUC_lexicon_tags_1 },
    { 0x9301ecc0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a1ecc7, SUC_lexicon_tags_12 },
    { 0x4d41ecc8, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0x1681ecca, SUC_lexicon_tags_46 },
    { 0x521ecca, SUC_lexicon_tags_80 },
    { 0xa421eccc, SUC_lexicon_tags_12 },
    { 0xc981eccd, SUC_lexicon_tags_12 },
    { 0xfca1ecce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b61ecd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e1ecd5, SUC_lexicon_tags_26 },
    { 0xbe41ecd6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf41ecdb, SUC_lexicon_tags_6 },
    { 0x85e1ecdb, SUC_lexicon_tags_11 },
    { 0xf841ecdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe61ece4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca61ece8, SUC_lexicon_tags_11 },
    { 0x6f81ece9, SUC_lexicon_tags_2 },
    { 0x43a1ece8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e21eced, SUC_lexicon_tags_75 },
    { 0xf961eced, SUC_lexicon_tags_37 },
    { 0x88c1ecee, SUC_lexicon_tags_97 },
    { 0xcd21ecf0, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0x461ecf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc421ecf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c1ecfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba81ed01, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba41ed09, SUC_lexicon_tags_65 },
    { 0x9821ed0a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb181ed0c, SUC_lexicon_tags_31 },
    { 0x1a1ed0d, SUC_lexicon_tags_27 },
    { 0xe001ed0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd101ed11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e1ed16, SUC_lexicon_tags_3 },
    { 0x421ed17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c1ed1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a1ed1e, SUC_lexicon_tags_1 },
    { 0xb581ed1f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb981ed21, SUC_lexicon_tags_17 },
    { 0xf781ed22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6401ed25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e1ed28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01ed2d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2141ed2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7de1ed31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc1ed39, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8721ed3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac1ed40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61ed46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd721ed4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80a1ed4d, SUC_lexicon_tags_12 },
    { 0x2781ed4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea1ed54, SUC_lexicon_tags_10 },
    { 0xe901ed55, SUC_lexicon_tags_136 },
    { 0xd521ed56, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae41ed5c, SUC_lexicon_tags_11 },
    { 0xfc01ed5d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8541ed68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3421ed6a, SUC_lexicon_tags_27 },
    { 0x6001ed6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5301ed70, SUC_lexicon_tags_37 },
    { 0x2281ed71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1f21ed73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5621ed75, SUC_lexicon_tags_33 },
    { 0x1fe1ed76, SUC_lexicon_tags_37 },
    { 0x3521ed77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9e1ed79, SUC_lexicon_tags_11 },
    { 0xf81ed79, SUC_lexicon_tags_28 },
    { 0x421ed79, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xcc01ed7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e1ed87, SUC_lexicon_tags_14 },
    { 0x5fa1ed88, SUC_lexicon_tags_56 },
    { 0x72a1ed89, SUC_lexicon_tags_38 },
    { 0x57e1ed88, SUC_lexicon_tags_95 },
    { 0xfe1ed8b, SUC_lexicon_tags_17 },
    { 0xe221ed8c, SUC_lexicon_tags_14 },
    { 0x3ce1ed8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481ed93, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0xc9e1ed95, SUC_lexicon_tags_46 },
    { 0xa661ed96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1381ed9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7041ed9d, SUC_lexicon_tags_37 },
    { 0x8a41ed9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7561eda0, SUC_lexicon_tags_48 },
    { 0xc5a1eda1, SUC_lexicon_tags_7 },
    { 0x7b01eda0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c61edab, SUC_lexicon_tags_7 },
    { 0x3c01edac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc481edae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be1edb3, SUC_lexicon_tags_23 },
    { 0xe7c1edb4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce1edb7, SUC_lexicon_tags_17 },
    { 0x5661edb8, SUC_lexicon_tags_15 },
    { 0x87e1edb8, SUC_lexicon_tags_17 },
    { 0xe621edb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9e1edbc, SUC_lexicon_tags_11 },
    { 0xe301edbd, SUC_lexicon_tags_2 },
    { 0xfd81edbe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee1edc1, SUC_lexicon_tags_15 },
    { 0x4021edc2, SUC_lexicon_tags_11 },
    { 0xdc01edc2, SUC_lexicon_tags_11 },
    { 0x6521edc4, SUC_lexicon_tags_45 },
    { 0xf0a1edc4, SUC_lexicon_tags_2 },
    { 0xbbe1edc6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec1edc9, SUC_lexicon_tags_15 },
    { 0x8bc1edc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da1edce, SUC_lexicon_tags_37 },
    { 0xa2c1edcf, SUC_lexicon_tags_17 },
    { 0x3de1edd0, SUC_lexicon_tags_68 },
    { 0xa8c1edd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c1edd5, SUC_lexicon_tags_12 },
    { 0xbd21edd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5d21edd8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb01eddc, SUC_lexicon_tags_310 },
    { 0xbcc1eddd, SUC_lexicon_tags_11 },
    { 0x4e1eddd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a1ede3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca1ede7, SUC_lexicon_tags_25 },
    { 0x72e1ede8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe21edee, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa7c1edf0, SUC_lexicon_tags_127 },
    { 0xe561edf1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6601edf3, SUC_lexicon_tags_37 },
    { 0x94e1edf4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4581edf6, SUC_lexicon_tags_32 },
    { 0x6921edf7, SUC_lexicon_tags_27 },
    { 0xaea1edf8, SUC_lexicon_tags_17 },
    { 0x8621edf8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf661edfd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ae1edff, SUC_lexicon_tags_1 },
    { 0x9fe1ee00, SUC_lexicon_tags_35 },
    { 0xe6a1ee00, SUC_lexicon_tags_32 },
    { 0x31a1ee01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c1ee06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4441ee09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7661ee0b, SUC_lexicon_tags_47 },
    { 0xf4a1ee0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff81ee16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3881ee1b, SUC_lexicon_tags_21 },
    { 0xc9e1ee1c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd781ee21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xca61ee24, SUC_lexicon_tags_37 },
    { 0xb161ee24, SUC_lexicon_tags_15 },
    { 0x2b01ee25, SUC_lexicon_tags_5 },
    { 0xa841ee27, SUC_lexicon_tags_28 },
    { 0x8641ee28, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7a1ee2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7621ee2c, SUC_lexicon_tags_47 },
    { 0xe361ee2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8981ee37, SUC_lexicon_tags_53 },
    { 0xa6c1ee38, SUC_lexicon_tags_2 },
    { 0xdf01ee37, SUC_lexicon_tags_5 },
    { 0xfc81ee3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc1ee3d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e61ee40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x681ee42, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7861ee44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e1ee47, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9741ee49, SUC_lexicon_tags_15 },
    { 0x9d21ee49, SUC_lexicon_tags_47 },
    { 0xfe81ee49, SUC_lexicon_tags_17 },
    { 0x3a41ee4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba1ee4f, SUC_lexicon_tags_7 },
    { 0xcb01ee50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f61ee55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4781ee5a, SUC_lexicon_tags_33 },
    { 0x5281ee5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x95a1ee5d, SUC_lexicon_tags_7 },
    { 0x47a1ee5e, SUC_lexicon_tags_27 },
    { 0xbf81ee5f, SUC_lexicon_tags_65 },
    { 0xb721ee60, SUC_lexicon_tags_43 },
    { 0x5c01ee61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe901ee6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf021ee6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3701ee78, SUC_lexicon_tags_17 },
    { 0xdee1ee79, SUC_lexicon_tags_7 },
    { 0xfc01ee7a, SUC_lexicon_tags_61 },
    { 0xd781ee7a, SUC_lexicon_tags_17 },
    { 0xa5c1ee78, SUC_lexicon_tags_25 },
    { 0xd601ee7d, SUC_lexicon_tags_4 },
    { 0x7e01ee7e, SUC_lexicon_tags_15 },
    { 0x9a1ee7f, SUC_lexicon_tags_87 },
    { 0x1e81ee80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a01ee82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23e1ee84, SUC_lexicon_tags_29 },
    { 0x9081ee84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3101ee91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4261ee94, SUC_lexicon_tags_11 },
    { 0xbcc1ee95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c1ee99, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xca1ee9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5221ee9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe1eea0, SUC_lexicon_tags_7 },
    { 0xad21eea1, SUC_lexicon_tags_63 },
    { 0x4541eea2, SUC_lexicon_tags_23 },
    { 0xb761eea3, SUC_lexicon_tags_17 },
    { 0x7ae1eea1, SUC_lexicon_tags_62 },
    { 0x33e1eea5, SUC_lexicon_tags_14 },
    { 0x1aa1eea1, SUC_lexicon_tags_53 },
    { 0xb881eea2, SUC_lexicon_tags_133 },
    { 0x71e1eea8, SUC_lexicon_tags_55 },
    { 0xd801eea8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe6e1eeab, SUC_lexicon_tags_23 },
    { 0xf9a1eeab, SUC_lexicon_tags_25 },
    { 0x4141eeac, SUC_lexicon_tags_15 },
    { 0x60c1eeae, SUC_lexicon_tags_82 },
    { 0xad01eeae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec1eeb2, SUC_lexicon_tags_14 },
    { 0x4e21eeb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea21eeb9, SUC_lexicon_tags_31 },
    { 0xb7e1eeb9, SUC_lexicon_tags_23 },
    { 0xbbc1eeb9, SUC_lexicon_tags_12 },
    { 0x2c81eebc, SUC_lexicon_tags_44 },
    { 0x5561eebd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c1eec1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c1eec8, SUC_lexicon_tags_33 },
    { 0xb3e1eec9, SUC_lexicon_tags_61 },
    { 0x6ce1eec9, SUC_lexicon_tags_63 },
    { 0xde1eec9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3721eed0, SUC_lexicon_tags_107 },
    { 0x22e1eed0, SUC_lexicon_tags_15 },
    { 0xd6e1eed2, SUC_lexicon_tags_16 },
    { 0x6c81eed0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe981eed5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae1eedb, SUC_lexicon_tags_49 },
    { 0xb121eedc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de1eee3, SUC_lexicon_tags_15 },
    { 0x2821eee4, SUC_lexicon_tags_30 },
    { 0xfc61eee3, SUC_lexicon_tags_15 },
    { 0x78a1eee6, SUC_lexicon_tags_31 },
    { 0x5b81eee7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3a1eee9, SUC_lexicon_tags_94 },
    { 0x8ac1eeea, SUC_lexicon_tags_33 },
    { 0xc5a1eeea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c1eef0, SUC_lexicon_tags_12 },
    { 0x58c1eef1, SUC_lexicon_tags_15 },
    { 0xd1c1eef0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1981eef4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x86c1eef6, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee41eefa, SUC_lexicon_tags_54 },
    { 0x5d21eefb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc1eeff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc41ef01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c1ef05, SUC_lexicon_tags_14 },
    { 0xb901ef06, SUC_lexicon_tags_4 },
    { 0x621ef06, SUC_lexicon_tags_0 },
    { 0xb641ef08, SUC_lexicon_tags_82 },
    { 0x9201ef09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb61ef0b, SUC_lexicon_tags_17 },
    { 0xa901ef0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce1ef13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x141ef16, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0, NULL },
    { 0x7841ef19, SUC_lexicon_tags_26 },
    { 0x5701ef1a, SUC_lexicon_tags_12 },
    { 0x9c01ef1b, SUC_lexicon_tags_0 },
    { 0xd161ef1c, SUC_lexicon_tags_108 },
    { 0x43e1ef1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc881ef21, SUC_lexicon_tags_37 },
    { 0xc1ef21, SUC_lexicon_tags_45 },
    { 0xa41ef23, SUC_lexicon_tags_2 },
    { 0x8b61ef23, SUC_lexicon_tags_60 },
    { 0xe8a1ef23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ef2c, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4181ef32, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a1ef36, SUC_lexicon_tags_17 },
    { 0x54a1ef36, SUC_lexicon_tags_43 },
    { 0xbb01ef38, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7941ef3a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d21ef3e, SUC_lexicon_tags_2 },
    { 0x7d01ef3f, SUC_lexicon_tags_17 },
    { 0xb0a1ef40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9441ef42, SUC_lexicon_tags_7 },
    { 0x3081ef43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe1a1ef45, SUC_lexicon_tags_33 },
    { 0x21c1ef46, SUC_lexicon_tags_30 },
    { 0x6b61ef45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdc1ef49, SUC_lexicon_tags_68 },
    { 0xf021ef4a, SUC_lexicon_tags_8 },
    { 0xfa81ef4b, SUC_lexicon_tags_11 },
    { 0x4181ef4b, SUC_lexicon_tags_37 },
    { 0xba61ef4b, SUC_lexicon_tags_16 },
    { 0xb7e1ef4e, SUC_lexicon_tags_43 },
    { 0x15e1ef4d, SUC_lexicon_tags_6 },
    { 0x35e1ef50, SUC_lexicon_tags_62 },
    { 0x83c1ef4d, SUC_lexicon_tags_25 },
    { 0xc5c1ef52, SUC_lexicon_tags_17 },
    { 0x7f41ef4c, SUC_lexicon_tags_25 },
    { 0x39e1ef51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6601ef59, SUC_lexicon_tags_11 },
    { 0x8b81ef59, SUC_lexicon_tags_28 },
    { 0x4ea1ef59, SUC_lexicon_tags_30 },
    { 0xac61ef5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe321ef61, SUC_lexicon_tags_13 },
    { 0xa481ef62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf3a1ef64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a21ef66, SUC_lexicon_tags_1 },
    { 0xc721ef66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1b61ef69, SUC_lexicon_tags_27 },
    { 0xb861ef69, SUC_lexicon_tags_49 },
    { 0x5c1ef6b, SUC_lexicon_tags_37 },
    { 0x4941ef6b, SUC_lexicon_tags_37 },
    { 0x4481ef6a, SUC_lexicon_tags_25 },
    { 0x2841ef6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf301ef72, SUC_lexicon_tags_6 },
    { 0x3481ef73, SUC_lexicon_tags_43 },
    { 0xac21ef73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6561ef77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c01ef7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa1ef80, SUC_lexicon_tags_1 },
    { 0xb261ef80, SUC_lexicon_tags_47 },
    { 0x6201ef80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5581ef85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea21ef8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd701ef8c, SUC_lexicon_tags_2 },
    { 0x47a1ef8d, SUC_lexicon_tags_26 },
    { 0x5d21ef8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd341ef91, SUC_lexicon_tags_17 },
    { 0x8fc1ef92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae1ef96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c1ef9b, SUC_lexicon_tags_65 },
    { 0x8481ef9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb901ef9e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6c41efa0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6261efa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa781efa8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc1efad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2581efb0, SUC_lexicon_tags_1 },
    { 0xf221efb1, SUC_lexicon_tags_25 },
    { 0xb4c1efb2, SUC_lexicon_tags_87 },
    { 0x8941efb2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c1efb8, SUC_lexicon_tags_47 },
    { 0xfc1efb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b01efbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a1efbd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681efc1, SUC_lexicon_tags_33 },
    { 0x59a1efc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa541efc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7261efc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b61efca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb61efcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e81efd2, SUC_lexicon_tags_8 },
    { 0xf0a1efd3, SUC_lexicon_tags_60 },
    { 0xa201efd4, SUC_lexicon_tags_197 },
    { 0x23c1efd2, SUC_lexicon_tags_15 },
    { 0xd561efd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6581efda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81efde, SUC_lexicon_tags_32 },
    { 0x3681efde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71c1efe1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a21efe3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1efe6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d41efec, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5221eff1, SUC_lexicon_tags_68 },
    { 0xfe81eff2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc361eff9, SUC_lexicon_tags_7 },
    { 0x8ee1effa, SUC_lexicon_tags_27 },
    { 0xfbe1effa, SUC_lexicon_tags_320 },
    { 0xce1effc, SUC_lexicon_tags_0 },
    { 0xf141effc, SUC_lexicon_tags_11 },
    { 0x4601effe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89c1f000, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1f006, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a1f00c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e1f00f, SUC_lexicon_tags_39 },
    { 0x6701f00f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d01f013, SUC_lexicon_tags_49 },
    { 0x8ee1f014, SUC_lexicon_tags_11 },
    { 0xbe61f013, SUC_lexicon_tags_6 },
    { 0xb7e1f016, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69e1f018, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2e1f01a, SUC_lexicon_tags_1 },
    { 0xb421f01b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a1f01f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2601f021, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e01f026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5801f028, SUC_lexicon_tags_2 },
    { 0x6ca1f028, SUC_lexicon_tags_19 },
    { 0xe7c1f02a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4321f031, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1f035, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xb7e1f037, SUC_lexicon_tags_37 },
    { 0xa0e1f038, SUC_lexicon_tags_30 },
    { 0xf721f039, SUC_lexicon_tags_920 },
    { 0xb41f03a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1001f03d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe1f03f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3161f047, SUC_lexicon_tags_85 },
    { 0x74a1f048, SUC_lexicon_tags_29 },
    { 0x7141f048, SUC_lexicon_tags_15 },
    { 0xc241f048, SUC_lexicon_tags_47 },
    { 0xc61f04b, SUC_lexicon_tags_38 },
    { 0x7081f04c, SUC_lexicon_tags_2 },
    { 0x23c1f04c, SUC_lexicon_tags_7 },
    { 0xec61f04a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab41f050, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x141f053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da1f05f, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf001f06c, SUC_lexicon_tags_23 },
    { 0x5a01f06c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e1f072, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6061f07a, SUC_lexicon_tags_30 },
    { 0x8f81f07b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7861f082, SUC_lexicon_tags_39 },
    { 0xc21f083, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e1f087, SUC_lexicon_tags_11 },
    { 0x7641f088, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde21f08f, SUC_lexicon_tags_29 },
    { 0xdb41f090, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c1f094, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe641f096, SUC_lexicon_tags_45 },
    { 0x1681f097, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb81f09a, SUC_lexicon_tags_329 },
    { 0x97e1f09b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a1f09e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf81f0a0, SUC_lexicon_tags_6 },
    { 0x4e41f0a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4541f0a3, SUC_lexicon_tags_87 },
    { 0x7881f0a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de1f0a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa221f0af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7dc1f0b1, SUC_lexicon_tags_38 },
    { 0xb81f0b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e1f0be, SUC_lexicon_tags_23 },
    { 0xb41f0bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca1f0c3, SUC_lexicon_tags_17 },
    { 0x3e21f0c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b01f0c7, SUC_lexicon_tags_37 },
    { 0xd041f0c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301f0cc, SUC_lexicon_tags_4 },
    { 0x2de1f0cd, SUC_lexicon_tags_15 },
    { 0x27c1f0ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf781f0d0, SUC_lexicon_tags_7 },
    { 0xc701f0d0, SUC_lexicon_tags_1 },
    { 0x34c1f0d2, SUC_lexicon_tags_11 },
    { 0x4e41f0d1, SUC_lexicon_tags_25 },
    { 0x34a1f0d0, SUC_lexicon_tags_15 },
    { 0x14e1f0d5, SUC_lexicon_tags_14 },
    { 0xaee1f0d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d61f0d8, SUC_lexicon_tags_17 },
    { 0x1c01f0d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71e1f0db, SUC_lexicon_tags_33 },
    { 0xe081f0dc, SUC_lexicon_tags_12 },
    { 0x3501f0dd, SUC_lexicon_tags_30 },
    { 0xfe61f0de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe181f0e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d81f0e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f01f0ec, SUC_lexicon_tags_27 },
    { 0xa501f0ec, SUC_lexicon_tags_82 },
    { 0x7a01f0ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201f0f4, SUC_lexicon_tags_37 },
    { 0x5a01f0f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1201f0fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x561f0ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1761f102, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f105, SUC_lexicon_tags_44 },
    { 0x5be1f106, SUC_lexicon_tags_33 },
    { 0xe201f107, SUC_lexicon_tags_37 },
    { 0x4fa1f108, SUC_lexicon_tags_103 },
    { 0x2381f108, SUC_lexicon_tags_47 },
    { 0x6c61f105, SUC_lexicon_tags_0 },
    { 0x1da1f10b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1f81f10d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6101f110, SUC_lexicon_tags_12 },
    { 0xa581f110, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea61f118, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae1f11c, SUC_lexicon_tags_37 },
    { 0x9781f11c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd081f11f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6861f122, SUC_lexicon_tags_17 },
    { 0xb9c1f123, SUC_lexicon_tags_11 },
    { 0xaf21f122, SUC_lexicon_tags_30 },
    { 0x1961f125, SUC_lexicon_tags_27 },
    { 0x6a81f124, SUC_lexicon_tags_15 },
    { 0xa481f127, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa421f12a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f81f12d, SUC_lexicon_tags_27 },
    { 0x77a1f12e, SUC_lexicon_tags_17 },
    { 0xae1f12f, SUC_lexicon_tags_0 },
    { 0xb0c1f12f, SUC_lexicon_tags_62 },
    { 0x3ac1f131, SUC_lexicon_tags_2 },
    { 0x5c01f132, SUC_lexicon_tags_749 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c1f136, SUC_lexicon_tags_15 },
    { 0xc701f136, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58c1f139, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c1f13d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac1f145, SUC_lexicon_tags_82 },
    { 0x4f21f146, SUC_lexicon_tags_9 },
    { 0xc701f145, SUC_lexicon_tags_11 },
    { 0x5921f148, SUC_lexicon_tags_12 },
    { 0x9d01f149, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x84e1f14b, SUC_lexicon_tags_25 },
    { 0x9d01f14c, SUC_lexicon_tags_37 },
    { 0xba41f14d, SUC_lexicon_tags_2 },
    { 0xb6e1f14d, SUC_lexicon_tags_37 },
    { 0x48c1f14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a1f153, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c1f155, SUC_lexicon_tags_15 },
    { 0x9661f156, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e1f15d, SUC_lexicon_tags_7 },
    { 0xc4c1f15d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8221f163, SUC_lexicon_tags_27 },
    { 0x601f164, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e1f16d, SUC_lexicon_tags_12 },
    { 0x4421f16e, SUC_lexicon_tags_55 },
    { 0x2cc1f16f, SUC_lexicon_tags_26 },
    { 0xcda1f170, SUC_lexicon_tags_15 },
    { 0x7541f171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe1f177, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8181f17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a21f17c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e41f17e, SUC_lexicon_tags_34 },
    { 0xcbe1f17f, SUC_lexicon_tags_7 },
    { 0xd481f180, SUC_lexicon_tags_47 },
    { 0x7f01f181, SUC_lexicon_tags_14 },
    { 0x7a81f182, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c1f188, SUC_lexicon_tags_8 },
    { 0x7e41f189, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4421f191, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b01f194, SUC_lexicon_tags_7 },
    { 0xae61f195, SUC_lexicon_tags_25 },
    { 0x3e01f196, SUC_lexicon_tags_37 },
    { 0x1a1f197, SUC_lexicon_tags_1 },
    { 0x8641f198, SUC_lexicon_tags_31 },
    { 0xafa1f195, SUC_lexicon_tags_7 },
    { 0x6ee1f19a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa1f19f, SUC_lexicon_tags_418 },
    { 0x6ec1f19f, SUC_lexicon_tags_22 },
    { 0x2ee1f1a1, SUC_lexicon_tags_11 },
    { 0x1e21f1a2, SUC_lexicon_tags_25 },
    { 0xd5a1f1a3, SUC_lexicon_tags_23 },
    { 0x5d81f1a4, SUC_lexicon_tags_15 },
    { 0x5a21f1a5, SUC_lexicon_tags_14 },
    { 0x5401f1a6, SUC_lexicon_tags_3 },
    { 0xf801f1a7, SUC_lexicon_tags_44 },
    { 0x6d41f1a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6241f1ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8901f1b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e21f1bd, SUC_lexicon_tags_49 },
    { 0xe181f1bd, SUC_lexicon_tags_85 },
    { 0x7081f1bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd421f1c4, SUC_lexicon_tags_3 },
    { 0xf6c1f1c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa141f1cf, SUC_lexicon_tags_7 },
    { 0xb421f1d0, SUC_lexicon_tags_28 },
    { 0xdb01f1d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c81f1d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe81f1d7, SUC_lexicon_tags_7 },
    { 0x3ba1f1d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f21f1db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d21f1dd, SUC_lexicon_tags_19 },
    { 0x1d61f1de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1381f1ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23a1f1ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a1f1f3, SUC_lexicon_tags_31 },
    { 0xa941f1f4, SUC_lexicon_tags_26 },
    { 0x7521f1f4, SUC_lexicon_tags_30 },
    { 0x77e1f1f6, SUC_lexicon_tags_17 },
    { 0xaea1f1f7, SUC_lexicon_tags_1 },
    { 0x74c1f1f8, SUC_lexicon_tags_47 },
    { 0x1a41f1f9, SUC_lexicon_tags_0 },
    { 0x7121f1f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d21f1fd, SUC_lexicon_tags_15 },
    { 0x4661f1fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe601f203, SUC_lexicon_tags_37 },
    { 0xb561f204, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99e1f206, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e1f209, SUC_lexicon_tags_11 },
    { 0xb521f20a, SUC_lexicon_tags_19 },
    { 0x3261f209, SUC_lexicon_tags_23 },
    { 0x36a1f20c, SUC_lexicon_tags_27 },
    { 0x5a21f20c, SUC_lexicon_tags_63 },
    { 0xe0a1f20d, SUC_lexicon_tags_53 },
    { 0xb5c1f20c, SUC_lexicon_tags_66 },
    { 0x1841f210, SUC_lexicon_tags_30 },
    { 0x2da1f210, SUC_lexicon_tags_21 },
    { 0xa361f212, SUC_lexicon_tags_43 },
    { 0xf141f211, SUC_lexicon_tags_25 },
    { 0x1501f212, SUC_lexicon_tags_29 },
    { 0x2d41f211, SUC_lexicon_tags_12 },
    { 0x7d81f214, SUC_lexicon_tags_15 },
    { 0xe441f214, SUC_lexicon_tags_35 },
    { 0xb8e1f218, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbd21f21a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbee1f21c, SUC_lexicon_tags_60 },
    { 0xcba1f21d, SUC_lexicon_tags_68 },
    { 0x2941f21e, SUC_lexicon_tags_63 },
    { 0xe1c1f21f, SUC_lexicon_tags_8 },
    { 0xd801f220, SUC_lexicon_tags_1 },
    { 0xf0e1f21d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdb61f223, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c61f225, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1181f22a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2761f22c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1f22f, SUC_lexicon_tags_16 },
    { 0x7241f22f, SUC_lexicon_tags_74 },
    { 0x2801f231, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca1f234, SUC_lexicon_tags_0 },
    { 0x84c1f235, SUC_lexicon_tags_37 },
    { 0x8601f236, SUC_lexicon_tags_25 },
    { 0x9cc1f236, SUC_lexicon_tags_19 },
    { 0x8f61f238, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xed01f23a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe741f242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e1f248, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a61f24c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76c1f24e, SUC_lexicon_tags_8 },
    { 0xa761f24f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ae1f251, SUC_lexicon_tags_2 },
    { 0xcf21f251, SUC_lexicon_tags_15 },
    { 0xbe21f252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4081f256, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e41f25a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b01f25c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea1f260, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc761f262, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfb41f264, SUC_lexicon_tags_47 },
    { 0x3de1f265, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf101f267, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa761f26a, SUC_lexicon_tags_1 },
    { 0x11a1f26b, SUC_lexicon_tags_17 },
    { 0xaac1f26b, SUC_lexicon_tags_43 },
    { 0xbf01f26d, SUC_lexicon_tags_12 },
    { 0xe941f26e, SUC_lexicon_tags_2 },
    { 0xb0c1f26f, SUC_lexicon_tags_2 },
    { 0xfc61f26d, SUC_lexicon_tags_65 },
    { 0xd301f26d, SUC_lexicon_tags_15 },
    { 0xf1c1f272, SUC_lexicon_tags_23 },
    { 0x32c1f26f, SUC_lexicon_tags_43 },
    { 0x11c1f274, SUC_lexicon_tags_60 },
    { 0x86e1f274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa1f278, SUC_lexicon_tags_11 },
    { 0x2b81f279, SUC_lexicon_tags_37 },
    { 0x9461f278, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1301f27c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a1f27f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8221f281, SUC_lexicon_tags_7 },
    { 0x90e1f281, SUC_lexicon_tags_60 },
    { 0x1f41f281, SUC_lexicon_tags_45 },
    { 0x8561f282, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa661f287, SUC_lexicon_tags_65 },
    { 0x3701f288, SUC_lexicon_tags_11 },
    { 0x2de1f287, SUC_lexicon_tags_8 },
    { 0x87a1f289, SUC_lexicon_tags_11 },
    { 0x17a1f28b, SUC_lexicon_tags_27 },
    { 0x9781f28c, SUC_lexicon_tags_37 },
    { 0x3f41f28b, SUC_lexicon_tags_194 },
    { 0xb421f28e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5141f290, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce1f296, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb81f298, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2201f29d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5e21f29f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4261f2a6, SUC_lexicon_tags_43 },
    { 0x7bc1f2a7, SUC_lexicon_tags_21 },
    { 0x8901f2a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x70c1f2aa, SUC_lexicon_tags_3 },
    { 0x6a1f2ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5961f2af, SUC_lexicon_tags_38 },
    { 0xf81f2b0, SUC_lexicon_tags_2 },
    { 0xd0a1f2b0, SUC_lexicon_tags_2 },
    { 0xb201f2af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6161f2b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9381f2bd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba1f2c1, SUC_lexicon_tags_60 },
    { 0xcb41f2c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd241f2c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2401f2c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c21f2cb, SUC_lexicon_tags_7 },
    { 0xd681f2cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd621f2d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3561f2d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8021f2d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e1f2db, SUC_lexicon_tags_25 },
    { 0xa801f2dc, SUC_lexicon_tags_0 },
    { 0xe1e1f2dc, SUC_lexicon_tags_25 },
    { 0x97a1f2de, SUC_lexicon_tags_11 },
    { 0x421f2df, SUC_lexicon_tags_47 },
    { 0xc781f2e0, SUC_lexicon_tags_15 },
    { 0xbac1f2e1, SUC_lexicon_tags_3 },
    { 0xbea1f2e2, SUC_lexicon_tags_0 },
    { 0x7341f2e3, SUC_lexicon_tags_17 },
    { 0x6401f2e4, SUC_lexicon_tags_37 },
    { 0x7401f2e3, SUC_lexicon_tags_14 },
    { 0xe541f2dc, SUC_lexicon_tags_35 },
    { 0xcbe1f2e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2081f2eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x161f2ed, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4301f2f3, SUC_lexicon_tags_9 },
    { 0xaee1f2f3, SUC_lexicon_tags_60 },
    { 0x19a1f2f3, SUC_lexicon_tags_1 },
    { 0xd201f2f6, SUC_lexicon_tags_43 },
    { 0xd5c1f2f6, SUC_lexicon_tags_47 },
    { 0x4c1f2f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8381f2fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf01f302, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf681f309, SUC_lexicon_tags_15 },
    { 0xac01f30a, SUC_lexicon_tags_32 },
    { 0x7921f30b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6901f311, SUC_lexicon_tags_15 },
    { 0xf921f312, SUC_lexicon_tags_27 },
    { 0x7841f312, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2a1f315, SUC_lexicon_tags_38 },
    { 0xe8c1f316, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcf41f318, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7661f31a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81f321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa361f323, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4aa1f325, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x10e1f327, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2ec1f329, SUC_lexicon_tags_7 },
    { 0x9a1f32a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1f32d, SUC_lexicon_tags_17 },
    { 0xe801f32d, SUC_lexicon_tags_37 },
    { 0xd3e1f32f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1541f336, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1f339, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca1f33c, SUC_lexicon_tags_17 },
    { 0xb7e1f33d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4121f33f, SUC_lexicon_tags_63 },
    { 0xd6a1f340, SUC_lexicon_tags_19 },
    { 0xb921f340, SUC_lexicon_tags_23 },
    { 0x32c1f33f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc181f344, SUC_lexicon_tags_14 },
    { 0x6841f345, SUC_lexicon_tags_47 },
    { 0xba81f346, SUC_lexicon_tags_25 },
    { 0x9981f347, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d01f34a, SUC_lexicon_tags_37 },
    { 0x40a1f34b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2141f34d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf5a1f34f, SUC_lexicon_tags_13 },
    { 0xcd61f350, SUC_lexicon_tags_66 },
    { 0x2fa1f350, SUC_lexicon_tags_19 },
    { 0xfc61f352, SUC_lexicon_tags_65 },
    { 0xb801f350, SUC_lexicon_tags_25 },
    { 0xbe81f354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7661f359, SUC_lexicon_tags_56 },
    { 0xe2e1f35a, SUC_lexicon_tags_31 },
    { 0x3321f359, SUC_lexicon_tags_37 },
    { 0xf21f35c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5641f35f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c41f362, SUC_lexicon_tags_11 },
    { 0xc6a1f362, SUC_lexicon_tags_2 },
    { 0x1361f364, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xee1f366, SUC_lexicon_tags_30 },
    { 0x7ea1f366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe1f36e, SUC_lexicon_tags_28 },
    { 0x78c1f36e, SUC_lexicon_tags_11 },
    { 0xe221f370, SUC_lexicon_tags_43 },
    { 0x3321f371, SUC_lexicon_tags_17 },
    { 0x5241f372, SUC_lexicon_tags_33 },
    { 0x8fc1f371, SUC_lexicon_tags_11 },
    { 0x3ca1f370, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3901f376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c81f37a, SUC_lexicon_tags_6 },
    { 0xf681f37b, SUC_lexicon_tags_17 },
    { 0x1fe1f37c, SUC_lexicon_tags_12 },
    { 0xdfc1f37a, SUC_lexicon_tags_19 },
    { 0x3181f37e, SUC_lexicon_tags_49 },
    { 0x2021f37d, SUC_lexicon_tags_6 },
    { 0x8a81f37e, SUC_lexicon_tags_27 },
    { 0x30c1f381, SUC_lexicon_tags_29 },
    { 0xdee1f382, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cc1f384, SUC_lexicon_tags_49 },
    { 0xfd01f385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7341f38b, SUC_lexicon_tags_1 },
    { 0x48c1f38c, SUC_lexicon_tags_17 },
    { 0x24e1f38d, SUC_lexicon_tags_37 },
    { 0x1a41f38e, SUC_lexicon_tags_47 },
    { 0xa3e1f38e, SUC_lexicon_tags_37 },
    { 0xd7c1f38f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea1f393, SUC_lexicon_tags_84 },
    { 0x3f21f394, SUC_lexicon_tags_20 },
    { 0x7ec1f393, SUC_lexicon_tags_11 },
    { 0xdfe1f393, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2621f398, SUC_lexicon_tags_146 },
    { 0xfe01f398, SUC_lexicon_tags_23 },
    { 0xaba1f39a, SUC_lexicon_tags_37 },
    { 0x7101f399, SUC_lexicon_tags_7 },
    { 0x1e41f399, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e1f3ae, SUC_lexicon_tags_37 },
    { 0x7501f3af, SUC_lexicon_tags_5 },
    { 0x4c41f3b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb701f3b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc041f3bd, SUC_lexicon_tags_37 },
    { 0xe661f3be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2821f3c0, SUC_lexicon_tags_15 },
    { 0x8021f3c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa561f3c3, SUC_lexicon_tags_27 },
    { 0xde41f3c3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe101f3c6, SUC_lexicon_tags_104 },
    { 0xb701f3c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3421f3c9, SUC_lexicon_tags_7 },
    { 0xfc41f3ca, SUC_lexicon_tags_14 },
    { 0x6181f3cb, SUC_lexicon_tags_48 },
    { 0xd281f3ca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x21f3ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2361f3d0, SUC_lexicon_tags_44 },
    { 0xa701f3d1, SUC_lexicon_tags_11 },
    { 0x70e1f3d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc481f3d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61e1f3d6, SUC_lexicon_tags_25 },
    { 0xeb41f3d6, SUC_lexicon_tags_37 },
    { 0x9821f3d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1381f3de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5e61f3e0, SUC_lexicon_tags_11 },
    { 0xe01f3e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8101f3e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa601f3e6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda41f3eb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8341f3f5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8421f3fa, SUC_lexicon_tags_11 },
    { 0xd621f3fb, SUC_lexicon_tags_87 },
    { 0xfd41f3fc, SUC_lexicon_tags_7 },
    { 0x9901f3fd, SUC_lexicon_tags_11 },
    { 0xdd21f3fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fe1f400, SUC_lexicon_tags_3 },
    { 0xe841f401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3461f404, SUC_lexicon_tags_21 },
    { 0x5501f405, SUC_lexicon_tags_37 },
    { 0x81e1f406, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c01f409, SUC_lexicon_tags_12 },
    { 0x4d21f40a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4381f40d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8641f411, SUC_lexicon_tags_63 },
    { 0x3201f412, SUC_lexicon_tags_37 },
    { 0x2a81f413, SUC_lexicon_tags_55 },
    { 0x78a1f414, SUC_lexicon_tags_38 },
    { 0x9c81f412, SUC_lexicon_tags_2 },
    { 0x43c1f416, SUC_lexicon_tags_17 },
    { 0x9881f417, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6501f41b, SUC_lexicon_tags_11 },
    { 0xaa41f41b, SUC_lexicon_tags_21 },
    { 0xcfa1f41c, SUC_lexicon_tags_26 },
    { 0xee21f41c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a1f422, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9101f425, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc661f427, SUC_lexicon_tags_0 },
    { 0x8d01f427, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3e81f42a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1481f42c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaa1f42e, SUC_lexicon_tags_17 },
    { 0xe861f42e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d01f431, SUC_lexicon_tags_38 },
    { 0xaf21f432, SUC_lexicon_tags_33 },
    { 0x341f433, SUC_lexicon_tags_45 },
    { 0x25a1f434, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4941f438, SUC_lexicon_tags_33 },
    { 0x60e1f438, SUC_lexicon_tags_72 },
    { 0xa0e1f439, SUC_lexicon_tags_61 },
    { 0x9361f43b, SUC_lexicon_tags_37 },
    { 0x5d41f43b, SUC_lexicon_tags_25 },
    { 0x7781f43d, SUC_lexicon_tags_30 },
    { 0x7061f43e, SUC_lexicon_tags_63 },
    { 0xd8c1f43f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a1f44f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8401f451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a01f454, SUC_lexicon_tags_23 },
    { 0x9ac1f455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3701f458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda1f45b, SUC_lexicon_tags_17 },
    { 0xdee1f45c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf121f45f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241f468, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d61f46c, SUC_lexicon_tags_21 },
    { 0x4be1f46d, SUC_lexicon_tags_15 },
    { 0x5ea1f46d, SUC_lexicon_tags_27 },
    { 0xde61f46c, SUC_lexicon_tags_37 },
    { 0x1b61f470, SUC_lexicon_tags_11 },
    { 0x9741f470, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec1f47f, SUC_lexicon_tags_27 },
    { 0x9961f480, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c21f482, SUC_lexicon_tags_37 },
    { 0xda61f483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa821f48e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b81f490, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x70c1f492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa81f494, SUC_lexicon_tags_1 },
    { 0x4141f494, SUC_lexicon_tags_38 },
    { 0x9ce1f496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdde1f498, SUC_lexicon_tags_12 },
    { 0xce01f498, SUC_lexicon_tags_12 },
    { 0x12c1f499, SUC_lexicon_tags_38 },
    { 0x3121f49b, SUC_lexicon_tags_2 },
    { 0x55e1f49b, SUC_lexicon_tags_37 },
    { 0xf261f49b, SUC_lexicon_tags_10 },
    { 0x21f49b, SUC_lexicon_tags_17 },
    { 0x87a1f49b, SUC_lexicon_tags_65 },
    { 0xab61f4a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3421f4a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xda1f4a4, SUC_lexicon_tags_37 },
    { 0x3c41f4a5, SUC_lexicon_tags_17 },
    { 0x5181f4a5, SUC_lexicon_tags_15 },
    { 0x2f81f4a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf461f4ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x46c1f4ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7961f4b0, SUC_lexicon_tags_6 },
    { 0x6981f4b1, SUC_lexicon_tags_1 },
    { 0x7d61f4b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce1f4b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d81f4c3, SUC_lexicon_tags_19 },
    { 0xbfc1f4c4, SUC_lexicon_tags_37 },
    { 0x29c1f4c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa661f4c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5921f4c9, SUC_lexicon_tags_61 },
    { 0x9bc1f4ca, SUC_lexicon_tags_37 },
    { 0xede1f4cb, SUC_lexicon_tags_37 },
    { 0xc841f4cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2281f4d2, SUC_lexicon_tags_14 },
    { 0xae61f4d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e1f4d7, SUC_lexicon_tags_4 },
    { 0x28c1f4d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d21f4da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf01f4dd, SUC_lexicon_tags_34 },
    { 0x1ce1f4de, SUC_lexicon_tags_25 },
    { 0xaa1f4df, SUC_lexicon_tags_2 },
    { 0x341f4e0, SUC_lexicon_tags_26 },
    { 0x7621f4e1, SUC_lexicon_tags_12 },
    { 0xae61f4de, SUC_lexicon_tags_27 },
    { 0x7201f4df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58c1f4e5, SUC_lexicon_tags_15 },
    { 0x3c41f4e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3361f4e8, SUC_lexicon_tags_22 },
    { 0x47e1f4e8, SUC_lexicon_tags_11 },
    { 0xdc81f4e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce1f4ef, SUC_lexicon_tags_37 },
    { 0xb741f4ef, SUC_lexicon_tags_17 },
    { 0xf101f4ef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a01f4f5, SUC_lexicon_tags_26 },
    { 0x15e1f4f6, SUC_lexicon_tags_17 },
    { 0x1d21f4f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4821f4fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31a1f4fe, SUC_lexicon_tags_17 },
    { 0x9301f4fe, SUC_lexicon_tags_37 },
    { 0x35a1f500, SUC_lexicon_tags_11 },
    { 0x80e1f500, SUC_lexicon_tags_15 },
    { 0x53e1f502, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb361f505, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2781f507, SUC_lexicon_tags_37 },
    { 0x1881f508, SUC_lexicon_tags_9 },
    { 0xeec1f509, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6761f50c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe681f510, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88c1f512, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc461f518, SUC_lexicon_tags_14 },
    { 0xf701f518, SUC_lexicon_tags_14 },
    { 0x6101f519, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc1f51d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d61f523, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xace1f526, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c1f531, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69c1f533, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd061f535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be1f53c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a1f545, SUC_lexicon_tags_15 },
    { 0x39a1f546, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe221f54a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1cc1f54c, SUC_lexicon_tags_12 },
    { 0x6fa1f54d, SUC_lexicon_tags_63 },
    { 0x8261f54d, SUC_lexicon_tags_11 },
    { 0xd8a1f54f, SUC_lexicon_tags_47 },
    { 0x34e1f54f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd761f555, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a1f562, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3121f565, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b61f56b, SUC_lexicon_tags_43 },
    { 0x3b81f56c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xec01f56f, SUC_lexicon_tags_47 },
    { 0xdd81f570, SUC_lexicon_tags_17 },
    { 0xe921f571, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4e1f573, SUC_lexicon_tags_32 },
    { 0xd321f574, SUC_lexicon_tags_2 },
    { 0x3621f574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf641f579, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0a1f57b, SUC_lexicon_tags_25 },
    { 0x81f57c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a1f57f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd101f582, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa901f584, SUC_lexicon_tags_43 },
    { 0x5ce1f585, SUC_lexicon_tags_15 },
    { 0xcc01f586, SUC_lexicon_tags_28 },
    { 0x4861f584, SUC_lexicon_tags_14 },
    { 0x88e1f588, SUC_lexicon_tags_17 },
    { 0xe681f589, SUC_lexicon_tags_1 },
    { 0xdbe1f585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2281f58c, SUC_lexicon_tags_33 },
    { 0x5b01f58c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e1f590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e41f592, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c1f598, SUC_lexicon_tags_42 },
    { 0x6f81f599, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c1f59e, SUC_lexicon_tags_47 },
    { 0x7fe1f59f, SUC_lexicon_tags_11 },
    { 0xa581f59f, SUC_lexicon_tags_17 },
    { 0x5541f5a1, SUC_lexicon_tags_11 },
    { 0xa4a1f59f, SUC_lexicon_tags_60 },
    { 0xe6a1f5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fc1f5a5, SUC_lexicon_tags_14 },
    { 0xdce1f5a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1f5aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7901f5b0, SUC_lexicon_tags_85 },
    { 0x7a61f5b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e1f5b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84a1f5b7, SUC_lexicon_tags_4 },
    { 0xdfa1f5b8, SUC_lexicon_tags_25 },
    { 0xcc1f5b9, SUC_lexicon_tags_28 },
    { 0xa021f5b7, SUC_lexicon_tags_43 },
    { 0x9b41f5b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e41f5be, SUC_lexicon_tags_37 },
    { 0x91a1f5bf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe121f5c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3661f5c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a41f5cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc01f5d2, SUC_lexicon_tags_43 },
    { 0xfb21f5d3, SUC_lexicon_tags_26 },
    { 0x9c41f5d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc581f5d6, SUC_lexicon_tags_38 },
    { 0x2c61f5d6, SUC_lexicon_tags_25 },
    { 0x21e1f5d8, SUC_lexicon_tags_17 },
    { 0x9121f5d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa541f5dd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e1f5e3, SUC_lexicon_tags_17 },
    { 0x4b01f5e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c81f5ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a81f5f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a01f5f7, SUC_lexicon_tags_14 },
    { 0x5961f5f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7661f5fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f81f603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa661f609, SUC_lexicon_tags_38 },
    { 0xd0a1f60a, SUC_lexicon_tags_7 },
    { 0xc841f60b, SUC_lexicon_tags_25 },
    { 0x40a1f60a, SUC_lexicon_tags_25 },
    { 0xcda1f60d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf001f60f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f61f614, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41f617, SUC_lexicon_tags_30 },
    { 0xfb41f618, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7901f61c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc981f61e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301f625, SUC_lexicon_tags_15 },
    { 0x8aa1f625, SUC_lexicon_tags_2 },
    { 0x91e1f627, SUC_lexicon_tags_79 },
    { 0xcc1f626, SUC_lexicon_tags_7 },
    { 0x6f01f627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab01f62d, SUC_lexicon_tags_1 },
    { 0x9be1f62e, SUC_lexicon_tags_207 },
    { 0x5ce1f62f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa1f632, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f61f63a, SUC_lexicon_tags_7 },
    { 0xc9e1f63a, SUC_lexicon_tags_65 },
    { 0xc1e1f63b, SUC_lexicon_tags_14 },
    { 0xdb81f63d, SUC_lexicon_tags_25 },
    { 0x42e1f63d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a01f642, SUC_lexicon_tags_7 },
    { 0x6061f642, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83a1f645, SUC_lexicon_tags_10 },
    { 0x8721f646, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd81f648, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6161f64b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d01f64f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba61f651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac61f654, SUC_lexicon_tags_12 },
    { 0x1781f655, SUC_lexicon_tags_17 },
    { 0x75c1f655, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3261f660, SUC_lexicon_tags_17 },
    { 0xda41f660, SUC_lexicon_tags_2 },
    { 0x1b41f662, SUC_lexicon_tags_12 },
    { 0xf741f662, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6941f668, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc461f66b, SUC_lexicon_tags_15 },
    { 0xfd61f66c, SUC_lexicon_tags_29 },
    { 0xd301f66d, SUC_lexicon_tags_15 },
    { 0xfaa1f66e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4881f676, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9c1f678, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7501f67c, SUC_lexicon_tags_43 },
    { 0xd461f67d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe061f67f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c1f682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a01f684, SUC_lexicon_tags_27 },
    { 0xe421f685, SUC_lexicon_tags_16 },
    { 0x6a1f686, SUC_lexicon_tags_11 },
    { 0x1441f687, SUC_lexicon_tags_33 },
    { 0x2c81f688, SUC_lexicon_tags_12 },
    { 0x761f689, SUC_lexicon_tags_46 },
    { 0x5bc1f684, SUC_lexicon_tags_27 },
    { 0x6561f685, SUC_lexicon_tags_37 },
    { 0xea41f68c, SUC_lexicon_tags_37 },
    { 0x2261f68d, SUC_lexicon_tags_27 },
    { 0xa241f686, SUC_lexicon_tags_65 },
    { 0x6de1f68b, SUC_lexicon_tags_9 },
    { 0x5a81f690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8901f699, SUC_lexicon_tags_2 },
    { 0x7a21f69a, SUC_lexicon_tags_37 },
    { 0xbd21f69b, SUC_lexicon_tags_15 },
    { 0x6981f69b, SUC_lexicon_tags_2 },
    { 0x6321f69c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8761f6a0, SUC_lexicon_tags_25 },
    { 0x3f21f6a0, SUC_lexicon_tags_61 },
    { 0x6961f6a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c1f6a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a1f6ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8261f6ad, SUC_lexicon_tags_37 },
    { 0xd2a1f6ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2641f6b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa61f6b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01f6b7, SUC_lexicon_tags_0 },
    { 0x4361f6b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd01f6bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01f6c4, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb641f6c8, SUC_lexicon_tags_4 },
    { 0xee61f6c9, SUC_lexicon_tags_19 },
    { 0x1761f6ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd81f6cc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe041f6d3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6e41f6d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5761f6d8, SUC_lexicon_tags_37 },
    { 0x1e61f6d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdf01f6db, SUC_lexicon_tags_7 },
    { 0xf781f6dc, SUC_lexicon_tags_11 },
    { 0xe1e1f6db, SUC_lexicon_tags_15 },
    { 0x8661f6de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd621f6e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3e81f6e8, SUC_lexicon_tags_12 },
    { 0xd421f6e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5641f6ec, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a1f6ef, SUC_lexicon_tags_2 },
    { 0xf1e1f6f0, SUC_lexicon_tags_6 },
    { 0x3ca1f6f1, SUC_lexicon_tags_12 },
    { 0x7be1f6f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc881f6f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae41f6f9, SUC_lexicon_tags_31 },
    { 0x3fc1f6f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e41f6ff, SUC_lexicon_tags_15 },
    { 0xb081f700, SUC_lexicon_tags_2 },
    { 0x87e1f700, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e1f707, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe901f709, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa1f70f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb21f712, SUC_lexicon_tags_40 },
    { 0x75e1f713, SUC_lexicon_tags_15 },
    { 0xdc21f714, SUC_lexicon_tags_12 },
    { 0xa0c1f715, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfda1f717, SUC_lexicon_tags_20 },
    { 0xe221f717, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9fe1f71a, SUC_lexicon_tags_23 },
    { 0xe161f71b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc61f71e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6701f720, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd701f722, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361f727, SUC_lexicon_tags_32 },
    { 0x7dc1f728, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c1f72c, SUC_lexicon_tags_63 },
    { 0x99a1f72c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa141f731, SUC_lexicon_tags_37 },
    { 0x3cc1f732, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde41f736, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4021f738, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e1f73b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d41f744, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51a1f746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4781f748, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd941f74a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2821f74c, SUC_lexicon_tags_2 },
    { 0x7061f74d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa81f74f, SUC_lexicon_tags_15 },
    { 0x69e1f74f, SUC_lexicon_tags_32 },
    { 0x9041f751, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9521f754, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a1f757, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4501f759, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x5b41f75b, SUC_lexicon_tags_37 },
    { 0x4681f75c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a1f760, SUC_lexicon_tags_37 },
    { 0xb5a1f761, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd1a1f763, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b21f766, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcb41f768, SUC_lexicon_tags_10 },
    { 0x5c61f769, SUC_lexicon_tags_25 },
    { 0xf921f768, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6341f76c, SUC_lexicon_tags_15 },
    { 0x4ec1f76c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed61f774, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1f77e, SUC_lexicon_tags_11 },
    { 0x67a1f77e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a1f785, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5de1f787, SUC_lexicon_tags_25 },
    { 0x2241f787, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x3041f78b, SUC_lexicon_tags_14 },
    { 0x81c1f78c, SUC_lexicon_tags_60 },
    { 0x85c1f78c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c1f792, SUC_lexicon_tags_11 },
    { 0x62a1f792, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc961f795, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a61f79a, SUC_lexicon_tags_37 },
    { 0x94a1f79a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf721f79d, SUC_lexicon_tags_1369 },
    { 0, NULL },
    { 0x6481f79f, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x6d41f7a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc1f7a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a1f7a8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3681f7ac, SUC_lexicon_tags_19 },
    { 0x1621f7ad, SUC_lexicon_tags_7 },
    { 0x2401f7ac, SUC_lexicon_tags_42 },
    { 0xfea1f7ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9c1f7b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4481f7b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9561f7b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3dc1f7b8, SUC_lexicon_tags_15 },
    { 0x941f7b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc981f7bc, SUC_lexicon_tags_37 },
    { 0xbcc1f7bc, SUC_lexicon_tags_27 },
    { 0x1361f7be, SUC_lexicon_tags_25 },
    { 0x361f7be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6241f7c4, SUC_lexicon_tags_11 },
    { 0x1a41f7c5, SUC_lexicon_tags_8 },
    { 0x9821f7c5, SUC_lexicon_tags_63 },
    { 0x8a1f7c6, SUC_lexicon_tags_7 },
    { 0xd8a1f7c8, SUC_lexicon_tags_23 },
    { 0xba81f7c7, SUC_lexicon_tags_82 },
    { 0x6a21f7ca, SUC_lexicon_tags_25 },
    { 0xb641f7cb, SUC_lexicon_tags_25 },
    { 0x3b21f7cc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e1f7d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c81f7ec, SUC_lexicon_tags_47 },
    { 0xf141f7ed, SUC_lexicon_tags_25 },
    { 0x6e81f7ee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd61f7f2, SUC_lexicon_tags_0 },
    { 0x7ce1f7f2, SUC_lexicon_tags_27 },
    { 0xd6c1f7f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1901f7f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa1f7fc, SUC_lexicon_tags_15 },
    { 0x9261f7fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1361f808, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf681f80a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e1f818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa861f81b, SUC_lexicon_tags_65 },
    { 0xbc01f81c, SUC_lexicon_tags_61 },
    { 0x6be1f81d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4181f820, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc21f823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa1f825, SUC_lexicon_tags_11 },
    { 0x84e1f826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6161f829, SUC_lexicon_tags_79 },
    { 0x1261f82a, SUC_lexicon_tags_2 },
    { 0x4781f82a, SUC_lexicon_tags_0 },
    { 0x2d01f82c, SUC_lexicon_tags_14 },
    { 0x8dc1f82c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xef21f82f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea1f833, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd041f835, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xec21f837, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9261f83b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5421f83e, SUC_lexicon_tags_29 },
    { 0xfb41f83e, SUC_lexicon_tags_11 },
    { 0xf081f83e, SUC_lexicon_tags_37 },
    { 0x7201f841, SUC_lexicon_tags_17 },
    { 0x36e1f842, SUC_lexicon_tags_37 },
    { 0x9ba1f83e, SUC_lexicon_tags_30 },
    { 0x6c21f841, SUC_lexicon_tags_87 },
    { 0x5de1f845, SUC_lexicon_tags_63 },
    { 0xb9c1f846, SUC_lexicon_tags_31 },
    { 0xb781f846, SUC_lexicon_tags_2 },
    { 0x38c1f848, SUC_lexicon_tags_16 },
    { 0xdec1f844, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e61f84d, SUC_lexicon_tags_16 },
    { 0xe261f84e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d21f850, SUC_lexicon_tags_17 },
    { 0x1221f851, SUC_lexicon_tags_17 },
    { 0x6f61f850, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc1f855, SUC_lexicon_tags_17 },
    { 0x3161f855, SUC_lexicon_tags_483 },
    { 0x8c81f857, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01f85b, SUC_lexicon_tags_38 },
    { 0x5321f85c, SUC_lexicon_tags_7 },
    { 0x44a1f85c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5181f865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe501f867, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc1f86f, SUC_lexicon_tags_11 },
    { 0x6921f870, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1441f872, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd561f874, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a1f87a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf261f87d, SUC_lexicon_tags_37 },
    { 0x6641f87e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7501f881, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd101f886, SUC_lexicon_tags_60 },
    { 0x5e41f887, SUC_lexicon_tags_87 },
    { 0xb7a1f888, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3bc1f88a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8541f88f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15e1f891, SUC_lexicon_tags_25 },
    { 0x6601f891, SUC_lexicon_tags_15 },
    { 0x7c1f892, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ca1f895, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a41f897, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8681f899, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a1f89d, SUC_lexicon_tags_25 },
    { 0x7261f89d, SUC_lexicon_tags_63 },
    { 0x5441f89d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e21f8a1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb2a1f8a3, SUC_lexicon_tags_19 },
    { 0xc41f8a3, SUC_lexicon_tags_45 },
    { 0xa9a1f8a4, SUC_lexicon_tags_12 },
    { 0xfac1f8a4, SUC_lexicon_tags_0 },
    { 0x7dc1f8a7, SUC_lexicon_tags_37 },
    { 0x4bc1f8a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfbc1f8aa, SUC_lexicon_tags_1 },
    { 0x5661f8ab, SUC_lexicon_tags_11 },
    { 0xb921f8aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc21f8af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6241f8b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8841f8b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1761f8b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3a1f8b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca01f8ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e1f8c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf901f8c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ca1f8c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9281f8cb, SUC_lexicon_tags_45 },
    { 0x8041f8cc, SUC_lexicon_tags_45 },
    { 0x2661f8cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d21f8d4, SUC_lexicon_tags_4 },
    { 0x3a81f8d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc21f8d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec1f8e0, SUC_lexicon_tags_50 },
    { 0x2b01f8e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c1f8e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4041f8e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdfc1f8e7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2861f8e9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e81f8ec, SUC_lexicon_tags_60 },
    { 0x8be1f8ec, SUC_lexicon_tags_2 },
    { 0xe441f8ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d81f8f0, SUC_lexicon_tags_37 },
    { 0xa301f8f1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc721f8f3, SUC_lexicon_tags_2 },
    { 0xe821f8f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x481f8f7, SUC_lexicon_tags_27 },
    { 0x3e21f8f8, SUC_lexicon_tags_49 },
    { 0xb921f8f7, SUC_lexicon_tags_15 },
    { 0xcd81f8fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd801f8fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2081f8fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e1f901, SUC_lexicon_tags_37 },
    { 0x1ce1f902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac01f904, SUC_lexicon_tags_17 },
    { 0xab61f905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd281f907, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3901f910, SUC_lexicon_tags_1 },
    { 0xea41f911, SUC_lexicon_tags_45 },
    { 0xcee1f912, SUC_lexicon_tags_45 },
    { 0x9881f911, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfd01f915, SUC_lexicon_tags_23 },
    { 0xba61f915, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae1f91b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e1f91f, SUC_lexicon_tags_8 },
    { 0x54e1f920, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f01f922, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc61f924, SUC_lexicon_tags_17 },
    { 0x6761f925, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee1f92c, SUC_lexicon_tags_9 },
    { 0x5261f92d, SUC_lexicon_tags_38 },
    { 0xf61f92c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6621f933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c1f93c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6921f942, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x961f944, SUC_lexicon_tags_0 },
    { 0xfae1f945, SUC_lexicon_tags_2 },
    { 0x3541f945, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea1f949, SUC_lexicon_tags_1 },
    { 0x5b21f94a, SUC_lexicon_tags_11 },
    { 0x71a1f94a, SUC_lexicon_tags_19 },
    { 0xa461f94c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7421f94f, SUC_lexicon_tags_1 },
    { 0x31e1f950, SUC_lexicon_tags_8 },
    { 0x9a61f951, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8be1f953, SUC_lexicon_tags_60 },
    { 0xcf41f954, SUC_lexicon_tags_48 },
    { 0x2961f955, SUC_lexicon_tags_49 },
    { 0x1001f953, SUC_lexicon_tags_8 },
    { 0x1c21f957, SUC_lexicon_tags_49 },
    { 0xfce1f956, SUC_lexicon_tags_43 },
    { 0x2421f956, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9581f95b, SUC_lexicon_tags_1 },
    { 0x1ec1f95b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e1f961, SUC_lexicon_tags_27 },
    { 0x4161f961, SUC_lexicon_tags_45 },
    { 0x6f21f963, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6601f965, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6f61f967, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a1f96a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2601f96d, SUC_lexicon_tags_47 },
    { 0xfaa1f96e, SUC_lexicon_tags_29 },
    { 0x7c81f96e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa101f971, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x721f973, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a1f977, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1f97c, SUC_lexicon_tags_13 },
    { 0x1f41f97d, SUC_lexicon_tags_63 },
    { 0x44a1f97d, SUC_lexicon_tags_7 },
    { 0x5fa1f97e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5081f984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cc1f986, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a1f98b, SUC_lexicon_tags_55 },
    { 0x84e1f98c, SUC_lexicon_tags_33 },
    { 0x5c81f98b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3681f996, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb701f999, SUC_lexicon_tags_17 },
    { 0x3621f99a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd81f99c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea1f99f, SUC_lexicon_tags_43 },
    { 0xf601f99f, SUC_lexicon_tags_33 },
    { 0xe441f99f, SUC_lexicon_tags_12 },
    { 0xdb01f9a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9781f9a8, SUC_lexicon_tags_10 },
    { 0xfac1f9a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e1f9af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe1e1f9b1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4c1f9b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8401f9bb, SUC_lexicon_tags_12 },
    { 0x23c1f9bb, SUC_lexicon_tags_1 },
    { 0x3501f9bb, SUC_lexicon_tags_1 },
    { 0x5961f9be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae81f9c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac01f9c7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c1f9ca, SUC_lexicon_tags_0 },
    { 0x701f9cb, SUC_lexicon_tags_29 },
    { 0x8201f9ca, SUC_lexicon_tags_37 },
    { 0x7f81f9cd, SUC_lexicon_tags_14 },
    { 0xba61f9cd, SUC_lexicon_tags_0 },
    { 0xe501f9cf, SUC_lexicon_tags_2 },
    { 0xc8c1f9d0, SUC_lexicon_tags_2 },
    { 0xe021f9cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc1f9d4, SUC_lexicon_tags_37 },
    { 0x3861f9d4, SUC_lexicon_tags_24 },
    { 0x8021f9d6, SUC_lexicon_tags_3 },
    { 0xb2e1f9d7, SUC_lexicon_tags_14 },
    { 0xf1a1f9d8, SUC_lexicon_tags_17 },
    { 0x3861f9d9, SUC_lexicon_tags_214 },
    { 0xec61f9d8, SUC_lexicon_tags_47 },
    { 0x9d01f9d9, SUC_lexicon_tags_27 },
    { 0xb4a1f9da, SUC_lexicon_tags_2 },
    { 0x33a1f9dd, SUC_lexicon_tags_15 },
    { 0x2b41f9d8, SUC_lexicon_tags_57 },
    { 0xdc81f9df, SUC_lexicon_tags_17 },
    { 0xac41f9d8, SUC_lexicon_tags_43 },
    { 0xf6a1f9e1, SUC_lexicon_tags_49 },
    { 0x8401f9e2, SUC_lexicon_tags_37 },
    { 0xfe01f9e3, SUC_lexicon_tags_26 },
    { 0x4a01f9e2, SUC_lexicon_tags_26 },
    { 0x7021f9e2, SUC_lexicon_tags_15 },
    { 0xdc01f9de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f61f9ea, SUC_lexicon_tags_0 },
    { 0x40a1f9eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4941f9ed, SUC_lexicon_tags_7 },
    { 0x1641f9ee, SUC_lexicon_tags_47 },
    { 0xf341f9ef, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe121f9f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9a1f9f4, SUC_lexicon_tags_11 },
    { 0x8341f9f5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed81f9fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5861fa00, SUC_lexicon_tags_27 },
    { 0xa681fa00, SUC_lexicon_tags_37 },
    { 0x5401fa01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x12e1fa04, SUC_lexicon_tags_43 },
    { 0x3421fa04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e81fa0a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8821fa0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17e1fa0e, SUC_lexicon_tags_39 },
    { 0x8fe1fa0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4181fa11, SUC_lexicon_tags_10 },
    { 0xf01fa12, SUC_lexicon_tags_47 },
    { 0x7a61fa11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6581fa15, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3b61fa17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1781fa19, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x61a1fa1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca1fa1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7001fa21, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1781fa23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c1fa27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c1fa2c, SUC_lexicon_tags_34 },
    { 0xa181fa2d, SUC_lexicon_tags_17 },
    { 0xb4c1fa2e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xd501fa30, SUC_lexicon_tags_22 },
    { 0x8a41fa30, SUC_lexicon_tags_27 },
    { 0x581fa32, SUC_lexicon_tags_27 },
    { 0x75a1fa33, SUC_lexicon_tags_12 },
    { 0xd281fa34, SUC_lexicon_tags_6 },
    { 0xcfa1fa31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe9e1fa37, SUC_lexicon_tags_31 },
    { 0x2a61fa37, SUC_lexicon_tags_12 },
    { 0x7741fa39, SUC_lexicon_tags_1 },
    { 0xb21fa39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece1fa3e, SUC_lexicon_tags_0 },
    { 0x7301fa3f, SUC_lexicon_tags_11 },
    { 0x4261fa3f, SUC_lexicon_tags_10 },
    { 0xc4e1fa40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee81fa46, SUC_lexicon_tags_4 },
    { 0x4921fa47, SUC_lexicon_tags_28 },
    { 0xd4a1fa48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc21fa4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e01fa55, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8641fa60, SUC_lexicon_tags_25 },
    { 0x16e1fa61, SUC_lexicon_tags_11 },
    { 0x5301fa60, SUC_lexicon_tags_11 },
    { 0x56a1fa63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a01fa65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc1fa6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c61fa6f, SUC_lexicon_tags_11 },
    { 0x39e1fa70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5781fa72, SUC_lexicon_tags_56 },
    { 0xa681fa72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf4c1fa75, SUC_lexicon_tags_11 },
    { 0x3e21fa76, SUC_lexicon_tags_27 },
    { 0x1101fa77, SUC_lexicon_tags_26 },
    { 0xeec1fa75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd801fa7c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc941fa80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee81fa82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce1fa85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8461fa90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1121fa95, SUC_lexicon_tags_63 },
    { 0xb561fa95, SUC_lexicon_tags_17 },
    { 0x8181fa96, SUC_lexicon_tags_11 },
    { 0x58e1fa96, SUC_lexicon_tags_33 },
    { 0x9641fa99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d81fa9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde41fa9f, SUC_lexicon_tags_1 },
    { 0xd4e1faa0, SUC_lexicon_tags_47 },
    { 0x84c1faa0, SUC_lexicon_tags_2 },
    { 0x21a1faa0, SUC_lexicon_tags_38 },
    { 0x5941fa9f, SUC_lexicon_tags_7 },
    { 0x9ee1faa2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc221faa8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5341faab, SUC_lexicon_tags_11 },
    { 0x6901faac, SUC_lexicon_tags_107 },
    { 0xaa61faad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf281fab3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb281fabc, SUC_lexicon_tags_12 },
    { 0x4b21fabc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b21fabf, SUC_lexicon_tags_11 },
    { 0x3e61fabf, SUC_lexicon_tags_37 },
    { 0x88c1fabf, SUC_lexicon_tags_30 },
    { 0xcc61fac2, SUC_lexicon_tags_11 },
    { 0xc3a1fac0, SUC_lexicon_tags_26 },
    { 0x5121fac4, SUC_lexicon_tags_36 },
    { 0x3b21fac4, SUC_lexicon_tags_11 },
    { 0x901fac6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba1face, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd01fad2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e1fad9, SUC_lexicon_tags_160 },
    { 0xeae1fada, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42c1fadc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68c1fade, SUC_lexicon_tags_49 },
    { 0xffe1fade, SUC_lexicon_tags_64 },
    { 0x4101fadf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a1fae3, SUC_lexicon_tags_57 },
    { 0x4381fae4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2e21fae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1fae9, SUC_lexicon_tags_7 },
    { 0xda1fae9, SUC_lexicon_tags_13 },
    { 0xe941faeb, SUC_lexicon_tags_87 },
    { 0x9da1faec, SUC_lexicon_tags_23 },
    { 0x9f01faed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87e1faef, SUC_lexicon_tags_8 },
    { 0xa581faf0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa261faf3, SUC_lexicon_tags_32 },
    { 0x4ac1faf3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3601faf8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf501fafd, SUC_lexicon_tags_12 },
    { 0x80e1fafe, SUC_lexicon_tags_17 },
    { 0xed01faff, SUC_lexicon_tags_23 },
    { 0xf381fafe, SUC_lexicon_tags_91 },
    { 0x8421fb01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a1fb04, SUC_lexicon_tags_15 },
    { 0x8fc1fb05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de1fb09, SUC_lexicon_tags_7 },
    { 0x3e1fb0a, SUC_lexicon_tags_77 },
    { 0xfc61fb0a, SUC_lexicon_tags_50 },
    { 0x6d41fb0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee61fb0e, SUC_lexicon_tags_46 },
    { 0x5061fb0f, SUC_lexicon_tags_30 },
    { 0xba01fb10, SUC_lexicon_tags_37 },
    { 0x11c1fb10, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c1fb1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef81fb1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b41fb1f, SUC_lexicon_tags_15 },
    { 0x9f21fb1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7921fb23, SUC_lexicon_tags_2 },
    { 0xa321fb24, SUC_lexicon_tags_5 },
    { 0xd1a1fb23, SUC_lexicon_tags_33 },
    { 0x1081fb26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9841fb29, SUC_lexicon_tags_25 },
    { 0xb821fb2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa21fb2e, SUC_lexicon_tags_2 },
    { 0x2541fb2f, SUC_lexicon_tags_2 },
    { 0xd141fb2e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9d81fb32, SUC_lexicon_tags_15 },
    { 0x3461fb32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf641fb36, SUC_lexicon_tags_25 },
    { 0xe0a1fb36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5261fb39, SUC_lexicon_tags_29 },
    { 0x8ae1fb39, SUC_lexicon_tags_17 },
    { 0xcf81fb3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea1fb3e, SUC_lexicon_tags_17 },
    { 0x5621fb3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8261fb43, SUC_lexicon_tags_16 },
    { 0x9e41fb44, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa01fb4a, SUC_lexicon_tags_2 },
    { 0x2661fb4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda1fb4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8681fb52, SUC_lexicon_tags_37 },
    { 0x1ae1fb53, SUC_lexicon_tags_29 },
    { 0xe901fb54, SUC_lexicon_tags_30 },
    { 0xb441fb55, SUC_lexicon_tags_60 },
    { 0xa0a1fb52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3621fb58, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f21fb60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2a1fb62, SUC_lexicon_tags_29 },
    { 0x9841fb63, SUC_lexicon_tags_37 },
    { 0x3821fb63, SUC_lexicon_tags_26 },
    { 0xf81fb64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1901fb67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9c1fb69, SUC_lexicon_tags_12 },
    { 0xcee1fb6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39c1fb6c, SUC_lexicon_tags_47 },
    { 0x3641fb6c, SUC_lexicon_tags_27 },
    { 0x7761fb6e, SUC_lexicon_tags_190 },
    { 0xb801fb6f, SUC_lexicon_tags_2 },
    { 0x34e1fb6f, SUC_lexicon_tags_19 },
    { 0xf821fb71, SUC_lexicon_tags_12 },
    { 0x2a01fb72, SUC_lexicon_tags_17 },
    { 0x9221fb71, SUC_lexicon_tags_60 },
    { 0x56a1fb71, SUC_lexicon_tags_28 },
    { 0xd2a1fb75, SUC_lexicon_tags_65 },
    { 0xab81fb76, SUC_lexicon_tags_1 },
    { 0x4c01fb77, SUC_lexicon_tags_11 },
    { 0xb9c1fb76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc1fb7c, SUC_lexicon_tags_14 },
    { 0x1e81fb7d, SUC_lexicon_tags_12 },
    { 0x8641fb7e, SUC_lexicon_tags_30 },
    { 0x3dc1fb7f, SUC_lexicon_tags_11 },
    { 0x7b21fb80, SUC_lexicon_tags_43 },
    { 0xfc61fb81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc941fb88, SUC_lexicon_tags_45 },
    { 0xa941fb89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8281fb8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc461fb90, SUC_lexicon_tags_27 },
    { 0xcc01fb90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf581fb93, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf61fb98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c1fb9c, SUC_lexicon_tags_39 },
    { 0xe321fb9c, SUC_lexicon_tags_17 },
    { 0x9fe1fb9e, SUC_lexicon_tags_1 },
    { 0xf221fb9c, SUC_lexicon_tags_61 },
    { 0xbcc1fba0, SUC_lexicon_tags_113 },
    { 0x4901fba0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac1fba6, SUC_lexicon_tags_39 },
    { 0xc181fba6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5361fba9, SUC_lexicon_tags_15 },
    { 0x5581fbaa, SUC_lexicon_tags_7 },
    { 0x5021fbaa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea1fbae, SUC_lexicon_tags_6 },
    { 0x8aa1fbaf, SUC_lexicon_tags_26 },
    { 0x72a1fbae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc1fbb3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5901fbb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9061fbb9, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc721fbc0, SUC_lexicon_tags_14 },
    { 0x5ea1fbc1, SUC_lexicon_tags_8 },
    { 0x2e61fbc2, SUC_lexicon_tags_42 },
    { 0x4401fbc3, SUC_lexicon_tags_24 },
    { 0xbae1fbc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a1fbc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9281fbcb, SUC_lexicon_tags_77 },
    { 0xc801fbcc, SUC_lexicon_tags_92 },
    { 0x5f81fbcd, SUC_lexicon_tags_118 },
    { 0xbd21fbce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a1fbd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba21fbd4, SUC_lexicon_tags_34 },
    { 0x3841fbd5, SUC_lexicon_tags_17 },
    { 0x56a1fbd5, SUC_lexicon_tags_17 },
    { 0x52a1fbd7, SUC_lexicon_tags_11 },
    { 0xed41fbd6, SUC_lexicon_tags_31 },
    { 0x2e81fbd9, SUC_lexicon_tags_11 },
    { 0x1321fbda, SUC_lexicon_tags_11 },
    { 0x9741fbda, SUC_lexicon_tags_49 },
    { 0x33e1fbdc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4981fbdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4561fbe2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb41fbe7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x2401fbea, SUC_lexicon_tags_37 },
    { 0x7361fbea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc1fbf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce1fbf6, SUC_lexicon_tags_17 },
    { 0xf6c1fbf6, SUC_lexicon_tags_25 },
    { 0xda41fbf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c1fc07, SUC_lexicon_tags_8 },
    { 0x5781fc08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe1fc0d, SUC_lexicon_tags_38 },
    { 0x7fa1fc0e, SUC_lexicon_tags_15 },
    { 0x2a01fc0f, SUC_lexicon_tags_26 },
    { 0x441fc0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a1fc13, SUC_lexicon_tags_47 },
    { 0x9b61fc14, SUC_lexicon_tags_11 },
    { 0xaa61fc15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1fc17, SUC_lexicon_tags_113 },
    { 0x9701fc18, SUC_lexicon_tags_37 },
    { 0xf601fc19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbae1fc1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f41fc1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6461fc21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb741fc23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c1fc27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea1fc2b, SUC_lexicon_tags_25 },
    { 0xdec1fc2c, SUC_lexicon_tags_29 },
    { 0x7d41fc2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2181fc32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa421fc37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c1fc3a, SUC_lexicon_tags_0 },
    { 0x1e81fc3b, SUC_lexicon_tags_26 },
    { 0x48e1fc3b, SUC_lexicon_tags_12 },
    { 0x9801fc3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f61fc3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c41fc42, SUC_lexicon_tags_11 },
    { 0xdb81fc42, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2201fc46, SUC_lexicon_tags_12 },
    { 0x4be1fc47, SUC_lexicon_tags_16 },
    { 0x2361fc48, SUC_lexicon_tags_11 },
    { 0xcfc1fc49, SUC_lexicon_tags_0 },
    { 0x9a1fc48, SUC_lexicon_tags_39 },
    { 0xf061fc49, SUC_lexicon_tags_12 },
    { 0xd061fc4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dc1fc4e, SUC_lexicon_tags_28 },
    { 0x5c21fc4e, SUC_lexicon_tags_2 },
    { 0x3f41fc4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea21fc53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7de1fc55, SUC_lexicon_tags_16 },
    { 0xa6c1fc56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd721fc58, SUC_lexicon_tags_19 },
    { 0x4221fc58, SUC_lexicon_tags_65 },
    { 0x541fc5a, SUC_lexicon_tags_1 },
    { 0x7f41fc59, SUC_lexicon_tags_12 },
    { 0x2421fc5c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xdc81fc5e, SUC_lexicon_tags_12 },
    { 0xe761fc5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a1fc64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7821fc6e, SUC_lexicon_tags_82 },
    { 0xd2a1fc6f, SUC_lexicon_tags_37 },
    { 0xc501fc70, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa341fc74, SUC_lexicon_tags_2 },
    { 0xf7e1fc75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e1fc78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd281fc7b, SUC_lexicon_tags_16 },
    { 0x6fc1fc7c, SUC_lexicon_tags_92 },
    { 0xd381fc7b, SUC_lexicon_tags_12 },
    { 0x11e1fc7b, SUC_lexicon_tags_2 },
    { 0x501fc7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3661fc81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a81fc84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e1fc87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd281fc8b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2741fc8f, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e1fc92, SUC_lexicon_tags_7 },
    { 0x6de1fc93, SUC_lexicon_tags_17 },
    { 0x7061fc93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc1fc98, SUC_lexicon_tags_43 },
    { 0x6221fc98, SUC_lexicon_tags_63 },
    { 0xa421fc98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a1fc9d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e1fca1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1621fca3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51e1fca5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9561fca8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe301fcaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a01fcaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1361fcb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x181fcb8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2241fcbf, SUC_lexicon_tags_12 },
    { 0x5a81fcbf, SUC_lexicon_tags_17 },
    { 0x5aa1fcbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8601fccb, SUC_lexicon_tags_8 },
    { 0x4ec1fccb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e1fcd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29a1fcd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be1fcdc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de1fcdf, SUC_lexicon_tags_7 },
    { 0xbdc1fce0, SUC_lexicon_tags_6 },
    { 0x44c1fce1, SUC_lexicon_tags_11 },
    { 0x9181fce2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf161fce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92a1fce8, SUC_lexicon_tags_46 },
    { 0xe621fce9, SUC_lexicon_tags_2 },
    { 0xce21fcea, SUC_lexicon_tags_31 },
    { 0x24e1fcea, SUC_lexicon_tags_11 },
    { 0x38e1fcea, SUC_lexicon_tags_65 },
    { 0xeac1fce8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac1fcf5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca1fcfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31a1fcfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae1fd01, SUC_lexicon_tags_103 },
    { 0xe2c1fd02, SUC_lexicon_tags_11 },
    { 0xc021fd02, SUC_lexicon_tags_66 },
    { 0xf801fd04, SUC_lexicon_tags_60 },
    { 0x3bc1fd03, SUC_lexicon_tags_10 },
    { 0x71a1fd06, SUC_lexicon_tags_17 },
    { 0xd3e1fd06, SUC_lexicon_tags_22 },
    { 0x3cc1fd08, SUC_lexicon_tags_146 },
    { 0x2cc1fd08, SUC_lexicon_tags_11 },
    { 0x5d41fd08, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x28a1fd0c, SUC_lexicon_tags_105 },
    { 0x2841fd0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3921fd10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7e1fd12, SUC_lexicon_tags_16 },
    { 0x46e1fd12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a61fd1d, SUC_lexicon_tags_51 },
    { 0x6381fd1e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e21fd26, SUC_lexicon_tags_17 },
    { 0x69c1fd26, SUC_lexicon_tags_46 },
    { 0x5f61fd26, SUC_lexicon_tags_37 },
    { 0x6201fd28, SUC_lexicon_tags_99 },
    { 0x641fd2a, SUC_lexicon_tags_17 },
    { 0xc861fd2b, SUC_lexicon_tags_60 },
    { 0x4361fd29, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e1fd2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a1fd32, SUC_lexicon_tags_10 },
    { 0xaea1fd33, SUC_lexicon_tags_124 },
    { 0x16c1fd33, SUC_lexicon_tags_11 },
    { 0xb041fd33, SUC_lexicon_tags_25 },
    { 0x7341fd35, SUC_lexicon_tags_9 },
    { 0x9ee1fd36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e1fd3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6821fd41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5301fd43, SUC_lexicon_tags_39 },
    { 0x6921fd44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2461fd46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf101fd48, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3121fd4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a1fd4f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9381fd53, SUC_lexicon_tags_12 },
    { 0x1c21fd53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb121fd59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f41fd5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa1fd5e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa461fd64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c1fd6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8ba1fd6c, SUC_lexicon_tags_33 },
    { 0x8b81fd6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c1fd75, SUC_lexicon_tags_48 },
    { 0x6121fd75, SUC_lexicon_tags_43 },
    { 0x8de1fd77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e1fd7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe821fd7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c01fd83, SUC_lexicon_tags_15 },
    { 0xb821fd83, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb61fd88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c1fd8f, SUC_lexicon_tags_11 },
    { 0x65a1fd90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe961fd93, SUC_lexicon_tags_17 },
    { 0x67a1fd94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81c1fd96, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb441fd9b, SUC_lexicon_tags_8 },
    { 0x81fd9c, SUC_lexicon_tags_26 },
    { 0x5321fd9c, SUC_lexicon_tags_19 },
    { 0x7681fd9e, SUC_lexicon_tags_37 },
    { 0xc681fd9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e1fda3, SUC_lexicon_tags_25 },
    { 0x3ce1fda4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd121fda7, SUC_lexicon_tags_15 },
    { 0xef41fda8, SUC_lexicon_tags_45 },
    { 0x4261fda9, SUC_lexicon_tags_0 },
    { 0xbf01fdaa, SUC_lexicon_tags_1 },
    { 0xa621fda8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf461fdad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fdb4, SUC_lexicon_tags_15 },
    { 0xb321fdb5, SUC_lexicon_tags_19 },
    { 0x69c1fdb6, SUC_lexicon_tags_7 },
    { 0xca61fdb7, SUC_lexicon_tags_9 },
    { 0x1ce1fdb8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3421fdba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa621fdbc, SUC_lexicon_tags_12 },
    { 0xa8a1fdbd, SUC_lexicon_tags_14 },
    { 0x7d81fdbe, SUC_lexicon_tags_33 },
    { 0x9101fdbf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea01fdc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7401fdc8, SUC_lexicon_tags_33 },
    { 0xf8a1fdc9, SUC_lexicon_tags_7 },
    { 0xe521fdc9, SUC_lexicon_tags_731 },
    { 0xa9e1fdc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76a1fdcd, SUC_lexicon_tags_17 },
    { 0x51e1fdce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a21fdd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x35c1fdd4, SUC_lexicon_tags_25 },
    { 0x39a1fdd5, SUC_lexicon_tags_25 },
    { 0x3461fdd6, SUC_lexicon_tags_0 },
    { 0xb601fdd4, SUC_lexicon_tags_3 },
    { 0xc401fdd6, SUC_lexicon_tags_37 },
    { 0xbaa1fdd9, SUC_lexicon_tags_2 },
    { 0xbec1fdd9, SUC_lexicon_tags_17 },
    { 0xa901fdd6, SUC_lexicon_tags_87 },
    { 0xa461fdd5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe81fde0, SUC_lexicon_tags_82 },
    { 0x5b61fde1, SUC_lexicon_tags_10 },
    { 0x57e1fde1, SUC_lexicon_tags_23 },
    { 0xa961fde2, SUC_lexicon_tags_30 },
    { 0xcee1fde4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6801fde6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb281fde9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b61fdf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a21fdf6, SUC_lexicon_tags_35 },
    { 0xeea1fdf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da1fdfb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c1fe03, SUC_lexicon_tags_37 },
    { 0x401fe04, SUC_lexicon_tags_11 },
    { 0x601fe05, SUC_lexicon_tags_7 },
    { 0x5d61fe03, SUC_lexicon_tags_1 },
    { 0x2421fe07, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6641fe0a, SUC_lexicon_tags_45 },
    { 0xd421fe0b, SUC_lexicon_tags_46 },
    { 0xb021fe0a, SUC_lexicon_tags_16 },
    { 0x7b41fe0d, SUC_lexicon_tags_66 },
    { 0xec81fe0d, SUC_lexicon_tags_37 },
    { 0x6021fe0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee1fe15, SUC_lexicon_tags_82 },
    { 0x77e1fe16, SUC_lexicon_tags_65 },
    { 0x4221fe17, SUC_lexicon_tags_8 },
    { 0xbc61fe17, SUC_lexicon_tags_2 },
    { 0x16a1fe19, SUC_lexicon_tags_49 },
    { 0x89e1fe16, SUC_lexicon_tags_84 },
    { 0x2c41fe1a, SUC_lexicon_tags_32 },
    { 0x9c01fe18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a1fe26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7b61fe28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c1fe2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb21fe2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c81fe33, SUC_lexicon_tags_66 },
    { 0x9cc1fe34, SUC_lexicon_tags_1196 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd521fe39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae81fe3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca81fe3d, SUC_lexicon_tags_1 },
    { 0xb541fe3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01fe41, SUC_lexicon_tags_2 },
    { 0x4701fe42, SUC_lexicon_tags_9 },
    { 0x3e41fe42, SUC_lexicon_tags_136 },
    { 0x2461fe44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd061fe47, SUC_lexicon_tags_41 },
    { 0x2a21fe48, SUC_lexicon_tags_47 },
    { 0x7961fe47, SUC_lexicon_tags_11 },
    { 0xdca1fe49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd661fe4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec1fe50, SUC_lexicon_tags_108 },
    { 0x8e81fe51, SUC_lexicon_tags_11 },
    { 0x8e61fe50, SUC_lexicon_tags_27 },
    { 0x7621fe53, SUC_lexicon_tags_8 },
    { 0xd281fe54, SUC_lexicon_tags_37 },
    { 0x47e1fe55, SUC_lexicon_tags_7 },
    { 0xae21fe56, SUC_lexicon_tags_15 },
    { 0x14e1fe56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2201fe5b, SUC_lexicon_tags_39 },
    { 0xdd81fe5c, SUC_lexicon_tags_25 },
    { 0x3761fe5c, SUC_lexicon_tags_4 },
    { 0x2901fe5e, SUC_lexicon_tags_12 },
    { 0x5f61fe5f, SUC_lexicon_tags_63 },
    { 0xee21fe5b, SUC_lexicon_tags_60 },
    { 0x12e1fe61, SUC_lexicon_tags_7 },
    { 0xf141fe62, SUC_lexicon_tags_12 },
    { 0x7be1fe63, SUC_lexicon_tags_37 },
    { 0x5ea1fe63, SUC_lexicon_tags_25 },
    { 0x9661fe62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c41fe6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e61fe6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb501fe77, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41fe7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7a01fe80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3941fe84, SUC_lexicon_tags_1 },
    { 0x9b61fe85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b21fe88, SUC_lexicon_tags_79 },
    { 0xaa41fe89, SUC_lexicon_tags_1 },
    { 0x7b01fe8a, SUC_lexicon_tags_27 },
    { 0xe4e1fe8b, SUC_lexicon_tags_55 },
    { 0xb001fe8a, SUC_lexicon_tags_2 },
    { 0x3dc1fe8c, SUC_lexicon_tags_2 },
    { 0x1421fe8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87c1fe90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea1fe93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e21fe95, SUC_lexicon_tags_25 },
    { 0xc6a1fe95, SUC_lexicon_tags_1 },
    { 0x81c1fe96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc41fe9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e1fea1, SUC_lexicon_tags_55 },
    { 0xe2c1fea2, SUC_lexicon_tags_16 },
    { 0x8bc1fea3, SUC_lexicon_tags_65 },
    { 0x3621fea2, SUC_lexicon_tags_60 },
    { 0x81c1fea5, SUC_lexicon_tags_19 },
    { 0xf41fea6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf841fea8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2881feac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4021feb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9781feb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9221feb8, SUC_lexicon_tags_7 },
    { 0x70e1feb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c1febd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4381febf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c1fec3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3221fec5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf461feca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6e61fecc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x63e1fece, SUC_lexicon_tags_30 },
    { 0x2ea1fecf, SUC_lexicon_tags_15 },
    { 0x9581fece, SUC_lexicon_tags_43 },
    { 0x4421fed1, SUC_lexicon_tags_31 },
    { 0xd7a1fed1, SUC_lexicon_tags_33 },
    { 0xd7e1fed3, SUC_lexicon_tags_39 },
    { 0xff21fed2, SUC_lexicon_tags_10 },
    { 0xecc1fed5, SUC_lexicon_tags_37 },
    { 0x7001fed3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe281fedd, SUC_lexicon_tags_223 },
    { 0, NULL },
    { 0xf361fedf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8261fee2, SUC_lexicon_tags_1 },
    { 0xaaa1fee3, SUC_lexicon_tags_46 },
    { 0xc8e1fee2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb681feef, SUC_lexicon_tags_17 },
    { 0xb2c1fef0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e61fef6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7881fef8, SUC_lexicon_tags_48 },
    { 0xeee1fef8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe81fefc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6161fefe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1d61ff00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe21ff03, SUC_lexicon_tags_2 },
    { 0x5721ff04, SUC_lexicon_tags_11 },
    { 0xd3e1ff05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1a1ff07, SUC_lexicon_tags_7 },
    { 0xe641ff07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5161ff0d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf01ff10, SUC_lexicon_tags_33 },
    { 0x7e61ff11, SUC_lexicon_tags_49 },
    { 0x4a81ff12, SUC_lexicon_tags_13 },
    { 0xeb01ff12, SUC_lexicon_tags_43 },
    { 0x90a1ff14, SUC_lexicon_tags_2 },
    { 0x8741ff15, SUC_lexicon_tags_19 },
    { 0x6e21ff12, SUC_lexicon_tags_17 },
    { 0x7ee1ff16, SUC_lexicon_tags_27 },
    { 0xdce1ff18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5701ff1b, SUC_lexicon_tags_28 },
    { 0xc461ff1b, SUC_lexicon_tags_5 },
    { 0x9c21ff1c, SUC_lexicon_tags_30 },
    { 0x69c1ff1b, SUC_lexicon_tags_87 },
    { 0xe201ff1f, SUC_lexicon_tags_6 },
    { 0xdc61ff20, SUC_lexicon_tags_25 },
    { 0x2661ff21, SUC_lexicon_tags_15 },
    { 0xe5c1ff21, SUC_lexicon_tags_27 },
    { 0xfc1ff21, SUC_lexicon_tags_8 },
    { 0x98c1ff24, SUC_lexicon_tags_0 },
    { 0xad61ff25, SUC_lexicon_tags_17 },
    { 0xbfe1ff25, SUC_lexicon_tags_1 },
    { 0x1261ff1f, SUC_lexicon_tags_45 },
    { 0x7a41ff22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1ff2f, SUC_lexicon_tags_3 },
    { 0x45a1ff2f, SUC_lexicon_tags_25 },
    { 0x8b41ff31, SUC_lexicon_tags_13 },
    { 0x8561ff31, SUC_lexicon_tags_37 },
    { 0x5461ff31, SUC_lexicon_tags_17 },
    { 0xfe21ff34, SUC_lexicon_tags_3 },
    { 0xa61ff33, SUC_lexicon_tags_14 },
    { 0x3581ff36, SUC_lexicon_tags_6 },
    { 0xd781ff34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe1ff3b, SUC_lexicon_tags_17 },
    { 0xfe81ff3c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541ff40, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xd8e1ff42, SUC_lexicon_tags_1 },
    { 0xc481ff43, SUC_lexicon_tags_43 },
    { 0x1dc1ff43, SUC_lexicon_tags_0 },
    { 0x5cc1ff45, SUC_lexicon_tags_26 },
    { 0xc501ff45, SUC_lexicon_tags_25 },
    { 0x6721ff46, SUC_lexicon_tags_27 },
    { 0x64a1ff43, SUC_lexicon_tags_79 },
    { 0xec81ff49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c01ff4c, SUC_lexicon_tags_0 },
    { 0x8c1ff4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac1ff53, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d81ff5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe121ff67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c1ff69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9ca1ff6b, SUC_lexicon_tags_12 },
    { 0xd2c1ff6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3461ff6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1ff71, SUC_lexicon_tags_68 },
    { 0xecc1ff71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ce1ff74, SUC_lexicon_tags_2 },
    { 0xb9c1ff74, SUC_lexicon_tags_25 },
    { 0xf3e1ff76, SUC_lexicon_tags_18 },
    { 0x8aa1ff77, SUC_lexicon_tags_16 },
    { 0x3d01ff75, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x1d81ff7a, SUC_lexicon_tags_12 },
    { 0x3e21ff7a, SUC_lexicon_tags_37 },
    { 0xb3e1ff7a, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xe721ff7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa341ff80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c81ff83, SUC_lexicon_tags_2 },
    { 0x4681ff84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee1ff87, SUC_lexicon_tags_87 },
    { 0x89e1ff88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc181ff95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b61ff98, SUC_lexicon_tags_79 },
    { 0x9b61ff98, SUC_lexicon_tags_26 },
    { 0xfb61ff9a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf061ff9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb401ff9f, SUC_lexicon_tags_11 },
    { 0xc0c1ffa0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc661ffa3, SUC_lexicon_tags_27 },
    { 0xb6c1ffa4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x39e1ffa6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa461ffa8, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x6321ffaa, SUC_lexicon_tags_33 },
    { 0xe481ffab, SUC_lexicon_tags_113 },
    { 0xb9c1ffab, SUC_lexicon_tags_37 },
    { 0xbc61ffab, SUC_lexicon_tags_40 },
    { 0x75a1ffae, SUC_lexicon_tags_0 },
    { 0x8981ffaf, SUC_lexicon_tags_8 },
    { 0x83c1ffb0, SUC_lexicon_tags_37 },
    { 0xe601ffb1, SUC_lexicon_tags_15 },
    { 0xf61ffb2, SUC_lexicon_tags_25 },
    { 0x8381ffb0, SUC_lexicon_tags_79 },
    { 0xb021ffae, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5c01ffb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd241ffb8, SUC_lexicon_tags_23 },
    { 0x1f41ffb9, SUC_lexicon_tags_14 },
    { 0x2641ffba, SUC_lexicon_tags_2 },
    { 0x68c1ffbb, SUC_lexicon_tags_6 },
    { 0x91a1ffbc, SUC_lexicon_tags_0 },
    { 0x4301ffb8, SUC_lexicon_tags_33 },
    { 0x4ee1ffb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f61ffc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e41ffc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3881ffc6, SUC_lexicon_tags_37 },
    { 0x761ffc7, SUC_lexicon_tags_37 },
    { 0xcea1ffc8, SUC_lexicon_tags_15 },
    { 0x76a1ffc9, SUC_lexicon_tags_14 },
    { 0xd201ffc8, SUC_lexicon_tags_7 },
    { 0x5661ffcb, SUC_lexicon_tags_15 },
    { 0xcda1ffca, SUC_lexicon_tags_15 },
    { 0x1c01ffcd, SUC_lexicon_tags_29 },
    { 0x78e1ffcd, SUC_lexicon_tags_8 },
    { 0x3e01ffcf, SUC_lexicon_tags_1 },
    { 0x6921ffcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb161ffd3, SUC_lexicon_tags_23 },
    { 0x1261ffd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4181ffd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7b01ffd8, SUC_lexicon_tags_17 },
    { 0xa881ffd9, SUC_lexicon_tags_60 },
    { 0x3561ffda, SUC_lexicon_tags_46 },
    { 0x43a1ffdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe961ffdd, SUC_lexicon_tags_11 },
    { 0xa721ffde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1961ffe0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1781ffe2, SUC_lexicon_tags_46 },
    { 0x7061ffe3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8161ffe7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f61ffeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23c1ffed, SUC_lexicon_tags_12 },
    { 0x8ca1ffed, SUC_lexicon_tags_7 },
    { 0xa821ffed, SUC_lexicon_tags_30 },
    { 0xeb41fff0, SUC_lexicon_tags_17 },
    { 0x9ac1fff1, SUC_lexicon_tags_35 },
    { 0xf5c1fff0, SUC_lexicon_tags_1 },
    { 0x761fff3, SUC_lexicon_tags_2 },
    { 0x461fff4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c1fffc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7c1fffe, SUC_lexicon_tags_46 },
    { 0x9b21fffe, SUC_lexicon_tags_30 },
    { 0xf941fffe, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c620003, SUC_lexicon_tags_12 },
    { 0x36e20003, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf9220006, SUC_lexicon_tags_9 },
    { 0x1b620007, SUC_lexicon_tags_15 },
    { 0xc3e20008, SUC_lexicon_tags_37 },
    { 0xc8e20009, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca42000d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd5c2000f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef820012, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e20016, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0xd1420018, SUC_lexicon_tags_106 },
    { 0x3d220019, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa982001b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e02001e, SUC_lexicon_tags_29 },
    { 0x1342001f, SUC_lexicon_tags_30 },
    { 0x7a22001f, SUC_lexicon_tags_10 },
    { 0xe262001e, SUC_lexicon_tags_11 },
    { 0xb2c20022, SUC_lexicon_tags_62 },
    { 0xdda2001e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24020028, SUC_lexicon_tags_60 },
    { 0x27420028, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3482002b, SUC_lexicon_tags_15 },
    { 0x61e2002b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c2002f, SUC_lexicon_tags_25 },
    { 0xd2820030, SUC_lexicon_tags_55 },
    { 0x44020030, SUC_lexicon_tags_2 },
    { 0x3a620032, SUC_lexicon_tags_83 },
    { 0x4282002f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2a020035, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee20038, SUC_lexicon_tags_1 },
    { 0x76e20038, SUC_lexicon_tags_15 },
    { 0x55e20038, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c02003d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe982003f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb020045, SUC_lexicon_tags_79 },
    { 0x30220045, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x66a20048, SUC_lexicon_tags_45 },
    { 0xa9a20048, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6062004c, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de20053, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac020057, SUC_lexicon_tags_17 },
    { 0x29a20058, SUC_lexicon_tags_15 },
    { 0x91820059, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6842005d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa982005f, SUC_lexicon_tags_12 },
    { 0xa9a20060, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc20065, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc020069, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6342006f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e20071, SUC_lexicon_tags_47 },
    { 0x84c20072, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33c20074, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae20079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1742007b, SUC_lexicon_tags_12 },
    { 0x74a2007b, SUC_lexicon_tags_15 },
    { 0xb42007d, SUC_lexicon_tags_37 },
    { 0xe802007e, SUC_lexicon_tags_25 },
    { 0x5262007d, SUC_lexicon_tags_33 },
    { 0x8282007d, SUC_lexicon_tags_1 },
    { 0x5f220081, SUC_lexicon_tags_2 },
    { 0x79c20081, SUC_lexicon_tags_3 },
    { 0x2220082, SUC_lexicon_tags_60 },
    { 0x8e2007e, SUC_lexicon_tags_1 },
    { 0x52820082, SUC_lexicon_tags_38 },
    { 0x5f620086, SUC_lexicon_tags_107 },
    { 0x1c820087, SUC_lexicon_tags_25 },
    { 0x66620088, SUC_lexicon_tags_38 },
    { 0xdbc20082, SUC_lexicon_tags_38 },
    { 0x9fe2008a, SUC_lexicon_tags_0 },
    { 0x6d82008b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2008f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5620092, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd420096, SUC_lexicon_tags_39 },
    { 0xfb620096, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec2009a, SUC_lexicon_tags_12 },
    { 0xccc2009b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814200a4, SUC_lexicon_tags_7 },
    { 0xc7a200a5, SUC_lexicon_tags_65 },
    { 0xb1a200a6, SUC_lexicon_tags_14 },
    { 0x5ae200a7, SUC_lexicon_tags_0 },
    { 0xc16200a7, SUC_lexicon_tags_11 },
    { 0x742200a9, SUC_lexicon_tags_14 },
    { 0x7ea200aa, SUC_lexicon_tags_7 },
    { 0xbaa200aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa200b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda200b5, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a200b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x566200ba, SUC_lexicon_tags_101 },
    { 0xec4200ba, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916200c0, SUC_lexicon_tags_33 },
    { 0x2ac200c0, SUC_lexicon_tags_7 },
    { 0x12e200c0, SUC_lexicon_tags_82 },
    { 0x7e2200c3, SUC_lexicon_tags_12 },
    { 0x2f0200c4, SUC_lexicon_tags_45 },
    { 0xf7c200c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x940200c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb56200c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a200cd, SUC_lexicon_tags_37 },
    { 0x43c200ce, SUC_lexicon_tags_12 },
    { 0x234200ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb8200d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0200d9, SUC_lexicon_tags_9 },
    { 0xa9a200d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e200dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca200e4, SUC_lexicon_tags_17 },
    { 0x5be200e5, SUC_lexicon_tags_2 },
    { 0xd40200e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef8200e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66200f5, SUC_lexicon_tags_49 },
    { 0xa54200f5, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866200ff, SUC_lexicon_tags_2 },
    { 0xd6220100, SUC_lexicon_tags_990 },
    { 0, NULL },
    { 0xb5820102, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x74c20104, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8382010b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5e2010d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c20110, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e20114, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21020116, SUC_lexicon_tags_2 },
    { 0xea220117, SUC_lexicon_tags_11 },
    { 0xd6c20118, SUC_lexicon_tags_63 },
    { 0xa6420119, SUC_lexicon_tags_12 },
    { 0xdbc2011a, SUC_lexicon_tags_60 },
    { 0x33e2011a, SUC_lexicon_tags_11 },
    { 0x6020118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c20121, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d420123, SUC_lexicon_tags_14 },
    { 0x4a220123, SUC_lexicon_tags_15 },
    { 0xcb620125, SUC_lexicon_tags_25 },
    { 0x59820125, SUC_lexicon_tags_45 },
    { 0xee220126, SUC_lexicon_tags_8 },
    { 0x3220128, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a22012b, SUC_lexicon_tags_9 },
    { 0x7e82012b, SUC_lexicon_tags_1 },
    { 0x34a2012d, SUC_lexicon_tags_27 },
    { 0xa582012e, SUC_lexicon_tags_32 },
    { 0x1182012f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa5020131, SUC_lexicon_tags_17 },
    { 0xf7e20132, SUC_lexicon_tags_1 },
    { 0xaa220132, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4982013a, SUC_lexicon_tags_37 },
    { 0x3402013a, SUC_lexicon_tags_37 },
    { 0x5c02013c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fe2013e, SUC_lexicon_tags_1 },
    { 0x41c2013f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c20143, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5202014c, SUC_lexicon_tags_17 },
    { 0x1722014c, SUC_lexicon_tags_66 },
    { 0x6c82014e, SUC_lexicon_tags_15 },
    { 0x3c22014f, SUC_lexicon_tags_17 },
    { 0xdce20150, SUC_lexicon_tags_15 },
    { 0x4a82014d, SUC_lexicon_tags_60 },
    { 0x45e20150, SUC_lexicon_tags_65 },
    { 0xc122014d, SUC_lexicon_tags_14 },
    { 0xf1a20154, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac620156, SUC_lexicon_tags_416 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e2015a, SUC_lexicon_tags_68 },
    { 0x9622015a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b82015e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff420169, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8582016f, SUC_lexicon_tags_1 },
    { 0x12e20170, SUC_lexicon_tags_12 },
    { 0x2cc20170, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2e20173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa20176, SUC_lexicon_tags_17 },
    { 0x43020177, SUC_lexicon_tags_19 },
    { 0x8fa20178, SUC_lexicon_tags_38 },
    { 0xd4820178, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26a2017b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7462017e, SUC_lexicon_tags_25 },
    { 0x682017e, SUC_lexicon_tags_2 },
    { 0xe9820180, SUC_lexicon_tags_9 },
    { 0xb422017f, SUC_lexicon_tags_2 },
    { 0x1a020182, SUC_lexicon_tags_59 },
    { 0x38c20181, SUC_lexicon_tags_11 },
    { 0xff420184, SUC_lexicon_tags_12 },
    { 0x1be20185, SUC_lexicon_tags_25 },
    { 0xd7c20180, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb102018e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x52a20190, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8620196, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8820199, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c2019e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346201a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x870201aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x176201ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26201b1, SUC_lexicon_tags_9 },
    { 0x4f4201b2, SUC_lexicon_tags_37 },
    { 0x3da201b3, SUC_lexicon_tags_15 },
    { 0x344201b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc6201b6, SUC_lexicon_tags_25 },
    { 0x4c0201b6, SUC_lexicon_tags_46 },
    { 0x8d2201b8, SUC_lexicon_tags_6 },
    { 0xff2201b9, SUC_lexicon_tags_26 },
    { 0x782201b9, SUC_lexicon_tags_47 },
    { 0xf38201bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530201c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa201c7, SUC_lexicon_tags_0 },
    { 0x14201c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5c201ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33c201cc, SUC_lexicon_tags_11 },
    { 0x260201cd, SUC_lexicon_tags_14 },
    { 0xbee201cc, SUC_lexicon_tags_33 },
    { 0x10201cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a201d4, SUC_lexicon_tags_15 },
    { 0x348201d5, SUC_lexicon_tags_35 },
    { 0xa46201d6, SUC_lexicon_tags_34 },
    { 0x99e201d7, SUC_lexicon_tags_2 },
    { 0xc4e201d8, SUC_lexicon_tags_25 },
    { 0x30a201d8, SUC_lexicon_tags_65 },
    { 0x3d2201da, SUC_lexicon_tags_17 },
    { 0xea201da, SUC_lexicon_tags_12 },
    { 0x8a8201d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a6201de, SUC_lexicon_tags_12 },
    { 0xb3a201df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14201e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2201ea, SUC_lexicon_tags_19 },
    { 0x380201eb, SUC_lexicon_tags_21 },
    { 0x7cc201eb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0201f5, SUC_lexicon_tags_29 },
    { 0x4ec201f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2201fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59220201, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e20204, SUC_lexicon_tags_7 },
    { 0xfc820204, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a2020c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac2020f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5820212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94e20214, SUC_lexicon_tags_14 },
    { 0x45e20215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61620217, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3d220219, SUC_lexicon_tags_46 },
    { 0xad220219, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3022021e, SUC_lexicon_tags_11 },
    { 0xb302021f, SUC_lexicon_tags_47 },
    { 0x9502021e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8d220222, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeea20224, SUC_lexicon_tags_7 },
    { 0x8b220225, SUC_lexicon_tags_33 },
    { 0x55220226, SUC_lexicon_tags_8 },
    { 0xf3620225, SUC_lexicon_tags_37 },
    { 0x17420228, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c2022b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1462022d, SUC_lexicon_tags_38 },
    { 0xd222022e, SUC_lexicon_tags_15 },
    { 0xcf62022d, SUC_lexicon_tags_77 },
    { 0xc9e2022f, SUC_lexicon_tags_17 },
    { 0xd8620231, SUC_lexicon_tags_12 },
    { 0x70420232, SUC_lexicon_tags_15 },
    { 0x13c20233, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2620237, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe0c20239, SUC_lexicon_tags_21 },
    { 0x75c2023a, SUC_lexicon_tags_49 },
    { 0x61c20239, SUC_lexicon_tags_6 },
    { 0x8762023c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68820247, SUC_lexicon_tags_11 },
    { 0x39620247, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8de2024a, SUC_lexicon_tags_37 },
    { 0xd122024b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e20250, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b220252, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c20259, SUC_lexicon_tags_1 },
    { 0xde420259, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2042025c, SUC_lexicon_tags_0 },
    { 0x7882025d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x25220260, SUC_lexicon_tags_25 },
    { 0x14620261, SUC_lexicon_tags_9 },
    { 0x2e620262, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31620264, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0820267, SUC_lexicon_tags_7 },
    { 0x820268, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa20270, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x75220272, SUC_lexicon_tags_15 },
    { 0xcf620273, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1020276, SUC_lexicon_tags_235 },
    { 0xd8420277, SUC_lexicon_tags_11 },
    { 0xa4020278, SUC_lexicon_tags_37 },
    { 0x5f420278, SUC_lexicon_tags_37 },
    { 0xfba2027a, SUC_lexicon_tags_2 },
    { 0x90020279, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93420280, SUC_lexicon_tags_13 },
    { 0x14620281, SUC_lexicon_tags_15 },
    { 0x47a20282, SUC_lexicon_tags_0 },
    { 0xd5c20283, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49020285, SUC_lexicon_tags_39 },
    { 0x86220286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90220289, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2662028e, SUC_lexicon_tags_17 },
    { 0x55e2028e, SUC_lexicon_tags_25 },
    { 0x55a20290, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1020299, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6f02029b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2029f, SUC_lexicon_tags_15 },
    { 0x2202a0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x660202a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe202a8, SUC_lexicon_tags_14 },
    { 0x1ca202a8, SUC_lexicon_tags_11 },
    { 0xf5a202a8, SUC_lexicon_tags_46 },
    { 0x6ec202ab, SUC_lexicon_tags_42 },
    { 0x296202ab, SUC_lexicon_tags_7 },
    { 0xff8202ac, SUC_lexicon_tags_40 },
    { 0x252202ae, SUC_lexicon_tags_12 },
    { 0x9bc202aa, SUC_lexicon_tags_82 },
    { 0x864202b0, SUC_lexicon_tags_8 },
    { 0x1d8202b1, SUC_lexicon_tags_33 },
    { 0xe9a202b2, SUC_lexicon_tags_87 },
    { 0x9e2202b2, SUC_lexicon_tags_2 },
    { 0x922202b4, SUC_lexicon_tags_16 },
    { 0x73e202b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x368202b7, SUC_lexicon_tags_7 },
    { 0xbd8202b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa8202ba, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54202bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc76202bf, SUC_lexicon_tags_25 },
    { 0x4fc202bf, SUC_lexicon_tags_22 },
    { 0x55c202c1, SUC_lexicon_tags_37 },
    { 0xc30202c1, SUC_lexicon_tags_21 },
    { 0xef8202c3, SUC_lexicon_tags_28 },
    { 0x898202c4, SUC_lexicon_tags_0 },
    { 0xe64202c3, SUC_lexicon_tags_13 },
    { 0x60e202c0, SUC_lexicon_tags_17 },
    { 0xde8202c7, SUC_lexicon_tags_12 },
    { 0xc68202c5, SUC_lexicon_tags_38 },
    { 0x352202c6, SUC_lexicon_tags_37 },
    { 0xce8202ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74202d3, SUC_lexicon_tags_68 },
    { 0x892202d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x932202d7, SUC_lexicon_tags_14 },
    { 0xecc202d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x726202db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da202de, SUC_lexicon_tags_53 },
    { 0x404202df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a202e4, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c202ea, SUC_lexicon_tags_12 },
    { 0x2ce202eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xea8202ed, SUC_lexicon_tags_23 },
    { 0x2be202ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1fa202f0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfe8202f2, SUC_lexicon_tags_214 },
    { 0xd14202f2, SUC_lexicon_tags_7 },
    { 0x17e202f2, SUC_lexicon_tags_15 },
    { 0x6d4202f4, SUC_lexicon_tags_33 },
    { 0x52e202f5, SUC_lexicon_tags_37 },
    { 0x2f0202f7, SUC_lexicon_tags_7 },
    { 0x95e202f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6202fc, SUC_lexicon_tags_26 },
    { 0x37c202fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0820301, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x84420303, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xbba20305, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc442030d, SUC_lexicon_tags_6 },
    { 0x8082030d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x27820311, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b420319, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf22031c, SUC_lexicon_tags_15 },
    { 0xf082031d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe20322, SUC_lexicon_tags_27 },
    { 0x14c20323, SUC_lexicon_tags_2 },
    { 0xf5e20323, SUC_lexicon_tags_15 },
    { 0xdd620325, SUC_lexicon_tags_63 },
    { 0xd8620322, SUC_lexicon_tags_6 },
    { 0xae20327, SUC_lexicon_tags_2 },
    { 0xa8220328, SUC_lexicon_tags_38 },
    { 0x6ce20328, SUC_lexicon_tags_32 },
    { 0xf9020326, SUC_lexicon_tags_49 },
    { 0x15020329, SUC_lexicon_tags_11 },
    { 0x16e2032c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f220332, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa820334, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1820337, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2be20339, SUC_lexicon_tags_28 },
    { 0x17c20339, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa820341, SUC_lexicon_tags_49 },
    { 0x4a820342, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9e20344, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4020348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42034b, SUC_lexicon_tags_48 },
    { 0x93e2034c, SUC_lexicon_tags_17 },
    { 0x2062034d, SUC_lexicon_tags_25 },
    { 0x25e2034e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e20354, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e20357, SUC_lexicon_tags_45 },
    { 0xf4e20358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1522035b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9502035f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99020361, SUC_lexicon_tags_25 },
    { 0x9c620361, SUC_lexicon_tags_2 },
    { 0xf5420363, SUC_lexicon_tags_83 },
    { 0x94620361, SUC_lexicon_tags_42 },
    { 0x73c20365, SUC_lexicon_tags_35 },
    { 0xeb620362, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e2036c, SUC_lexicon_tags_0 },
    { 0xc682036d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c20372, SUC_lexicon_tags_15 },
    { 0x72820373, SUC_lexicon_tags_37 },
    { 0xa1020374, SUC_lexicon_tags_37 },
    { 0x62220375, SUC_lexicon_tags_17 },
    { 0xc8620373, SUC_lexicon_tags_2 },
    { 0xc4a20376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a62037b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc842037f, SUC_lexicon_tags_37 },
    { 0xd820380, SUC_lexicon_tags_10 },
    { 0x8d620381, SUC_lexicon_tags_8 },
    { 0xa6820382, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9620388, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5022038c, SUC_lexicon_tags_3 },
    { 0xa822038d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11e2038f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d220391, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc220394, SUC_lexicon_tags_14 },
    { 0x82e20395, SUC_lexicon_tags_19 },
    { 0xb8820395, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28020398, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8482039d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e203a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x984203a6, SUC_lexicon_tags_33 },
    { 0x582203a7, SUC_lexicon_tags_15 },
    { 0x5b2203a8, SUC_lexicon_tags_2 },
    { 0x902203a9, SUC_lexicon_tags_18 },
    { 0xc48203a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd2203ac, SUC_lexicon_tags_14 },
    { 0xccc203ad, SUC_lexicon_tags_15 },
    { 0x6d2203ae, SUC_lexicon_tags_17 },
    { 0xa94203ae, SUC_lexicon_tags_14 },
    { 0xa28203ad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86203b3, SUC_lexicon_tags_34 },
    { 0x74a203b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f4203b6, SUC_lexicon_tags_1 },
    { 0x902203b7, SUC_lexicon_tags_17 },
    { 0x938203b8, SUC_lexicon_tags_27 },
    { 0x816203b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02203c1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xd0a203c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e203c6, SUC_lexicon_tags_14 },
    { 0x5b2203c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704203cc, SUC_lexicon_tags_12 },
    { 0x14a203cd, SUC_lexicon_tags_17 },
    { 0x4e6203ce, SUC_lexicon_tags_11 },
    { 0x31c203ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc203d1, SUC_lexicon_tags_15 },
    { 0xc86203d2, SUC_lexicon_tags_17 },
    { 0x31a203d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40203d9, SUC_lexicon_tags_37 },
    { 0x5ea203da, SUC_lexicon_tags_40 },
    { 0xa44203db, SUC_lexicon_tags_2 },
    { 0xbc8203db, SUC_lexicon_tags_2 },
    { 0xb98203db, SUC_lexicon_tags_63 },
    { 0xaba203d9, SUC_lexicon_tags_27 },
    { 0x724203da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x34e203e1, SUC_lexicon_tags_23 },
    { 0x212203e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x854203e4, SUC_lexicon_tags_11 },
    { 0x57a203e5, SUC_lexicon_tags_17 },
    { 0x91c203e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0203eb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08203f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba203f6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x94a203f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e6203fa, SUC_lexicon_tags_7 },
    { 0xdd2203fb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c20401, SUC_lexicon_tags_15 },
    { 0xd3020402, SUC_lexicon_tags_49 },
    { 0xe7820402, SUC_lexicon_tags_25 },
    { 0xd4a20401, SUC_lexicon_tags_28 },
    { 0x64420405, SUC_lexicon_tags_47 },
    { 0x19620401, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1322040d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x83420410, SUC_lexicon_tags_43 },
    { 0xd9a20411, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24c20413, SUC_lexicon_tags_68 },
    { 0x50620414, SUC_lexicon_tags_4 },
    { 0x97a20415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d82041a, SUC_lexicon_tags_14 },
    { 0xcd22041b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf682041d, SUC_lexicon_tags_14 },
    { 0x5b82041d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7e20420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c20423, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed620425, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcc420427, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x55a20429, SUC_lexicon_tags_7 },
    { 0xd1a20429, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf802042e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c20431, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec820433, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f820438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5902043d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa20441, SUC_lexicon_tags_63 },
    { 0x7a20442, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c2044a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c420454, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78420459, SUC_lexicon_tags_37 },
    { 0x4382045a, SUC_lexicon_tags_65 },
    { 0xfc220459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8802045f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75020461, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc620466, SUC_lexicon_tags_17 },
    { 0xe3020467, SUC_lexicon_tags_9 },
    { 0x3a220468, SUC_lexicon_tags_21 },
    { 0x49020469, SUC_lexicon_tags_11 },
    { 0x86c20469, SUC_lexicon_tags_12 },
    { 0xb9020469, SUC_lexicon_tags_35 },
    { 0x6b02046c, SUC_lexicon_tags_45 },
    { 0xd062046c, SUC_lexicon_tags_9 },
    { 0xfb220468, SUC_lexicon_tags_12 },
    { 0x8f02046f, SUC_lexicon_tags_14 },
    { 0x8082046d, SUC_lexicon_tags_25 },
    { 0x8662046e, SUC_lexicon_tags_1 },
    { 0xb8820472, SUC_lexicon_tags_1 },
    { 0xee20473, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e20476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3020478, SUC_lexicon_tags_7 },
    { 0x2ba20479, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c02047c, SUC_lexicon_tags_37 },
    { 0x7502047d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a420480, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23820482, SUC_lexicon_tags_160 },
    { 0xbe020482, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8420486, SUC_lexicon_tags_23 },
    { 0x50c20487, SUC_lexicon_tags_1 },
    { 0x80620488, SUC_lexicon_tags_48 },
    { 0x52620487, SUC_lexicon_tags_1 },
    { 0xf9c2048a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a2048e, SUC_lexicon_tags_25 },
    { 0xaca2048e, SUC_lexicon_tags_1 },
    { 0x22420490, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42020493, SUC_lexicon_tags_45 },
    { 0xdae20494, SUC_lexicon_tags_49 },
    { 0xc620495, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be2049c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c2049f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852204a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb6204a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x15c204a8, SUC_lexicon_tags_64 },
    { 0x246204a8, SUC_lexicon_tags_15 },
    { 0xb2e204a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae204b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x244204b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834204b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7e2204b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c204c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862204cd, SUC_lexicon_tags_14 },
    { 0x7c2204cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c204d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4204d5, SUC_lexicon_tags_27 },
    { 0x416204d6, SUC_lexicon_tags_15 },
    { 0x8a8204d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44204da, SUC_lexicon_tags_63 },
    { 0xfa8204db, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a204de, SUC_lexicon_tags_17 },
    { 0xc34204df, SUC_lexicon_tags_17 },
    { 0xe2e204e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0204e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x234204e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae204ec, SUC_lexicon_tags_38 },
    { 0x8cc204ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58204f8, SUC_lexicon_tags_141 },
    { 0xe02204f9, SUC_lexicon_tags_39 },
    { 0xafe204f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6204fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39620500, SUC_lexicon_tags_23 },
    { 0xd5620501, SUC_lexicon_tags_107 },
    { 0xeb620500, SUC_lexicon_tags_33 },
    { 0xb9420503, SUC_lexicon_tags_17 },
    { 0xa220500, SUC_lexicon_tags_30 },
    { 0xea220501, SUC_lexicon_tags_11 },
    { 0xee820506, SUC_lexicon_tags_7 },
    { 0x76420507, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3782050a, SUC_lexicon_tags_37 },
    { 0x7322050b, SUC_lexicon_tags_2 },
    { 0xc4e2050c, SUC_lexicon_tags_689 },
    { 0xf7c2050c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa20510, SUC_lexicon_tags_25 },
    { 0xd1420510, SUC_lexicon_tags_2 },
    { 0x6ee20512, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b020515, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xce620517, SUC_lexicon_tags_7 },
    { 0x38220518, SUC_lexicon_tags_1 },
    { 0xf620519, SUC_lexicon_tags_1 },
    { 0x14020519, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe22051c, SUC_lexicon_tags_45 },
    { 0x8702051d, SUC_lexicon_tags_29 },
    { 0x1142051e, SUC_lexicon_tags_25 },
    { 0x5342051c, SUC_lexicon_tags_29 },
    { 0x4020520, SUC_lexicon_tags_23 },
    { 0x27a20520, SUC_lexicon_tags_25 },
    { 0x7fa2051e, SUC_lexicon_tags_37 },
    { 0x39420523, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4820525, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x61020528, SUC_lexicon_tags_37 },
    { 0xfec20528, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8620530, SUC_lexicon_tags_14 },
    { 0xcf820531, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e20534, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x57620537, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a82053a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x762053c, SUC_lexicon_tags_91 },
    { 0x72c2053c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9820547, SUC_lexicon_tags_7 },
    { 0xfcc20548, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc22054a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7502054c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15220550, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a620552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30a20554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c20556, SUC_lexicon_tags_37 },
    { 0xd2420557, SUC_lexicon_tags_37 },
    { 0xca020558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcac2055a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f02055c, SUC_lexicon_tags_26 },
    { 0x3a2055c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd122055f, SUC_lexicon_tags_2 },
    { 0xaae20560, SUC_lexicon_tags_11 },
    { 0x6a420561, SUC_lexicon_tags_11 },
    { 0x68c20562, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xb9c20564, SUC_lexicon_tags_518 },
    { 0x54620564, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6820567, SUC_lexicon_tags_37 },
    { 0xcfe20568, SUC_lexicon_tags_0 },
    { 0x11020569, SUC_lexicon_tags_12 },
    { 0xcbe2056a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e20571, SUC_lexicon_tags_14 },
    { 0xa8420572, SUC_lexicon_tags_1 },
    { 0xe9c20571, SUC_lexicon_tags_39 },
    { 0xee220572, SUC_lexicon_tags_37 },
    { 0x15620572, SUC_lexicon_tags_23 },
    { 0x9f820572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc022057a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d020581, SUC_lexicon_tags_55 },
    { 0x1f420581, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7420588, SUC_lexicon_tags_17 },
    { 0x64c20589, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5122058b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xac42058d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb102058f, SUC_lexicon_tags_6 },
    { 0xd2220590, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2620593, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b82059a, SUC_lexicon_tags_42 },
    { 0xeb02059b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d22059d, SUC_lexicon_tags_16 },
    { 0xf02059e, SUC_lexicon_tags_15 },
    { 0x7762059f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63c205a1, SUC_lexicon_tags_30 },
    { 0x5f0205a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e205a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a205a8, SUC_lexicon_tags_12 },
    { 0x296205a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8205ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22205b0, SUC_lexicon_tags_6 },
    { 0x3a0205b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e8205b3, SUC_lexicon_tags_27 },
    { 0x3c6205b4, SUC_lexicon_tags_37 },
    { 0x9ba205b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cc205b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbdc205b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956205c5, SUC_lexicon_tags_49 },
    { 0xc7c205c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818205ca, SUC_lexicon_tags_12 },
    { 0xf6205cb, SUC_lexicon_tags_62 },
    { 0xd7e205cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638205d1, SUC_lexicon_tags_17 },
    { 0x7ea205d2, SUC_lexicon_tags_43 },
    { 0x39c205d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39c205d5, SUC_lexicon_tags_33 },
    { 0x50a205d6, SUC_lexicon_tags_12 },
    { 0xbae205d7, SUC_lexicon_tags_14 },
    { 0x756205d6, SUC_lexicon_tags_46 },
    { 0xe3c205d9, SUC_lexicon_tags_60 },
    { 0x6a8205d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a205df, SUC_lexicon_tags_55 },
    { 0x9f2205e0, SUC_lexicon_tags_4 },
    { 0x4d8205e1, SUC_lexicon_tags_2 },
    { 0x2f0205e2, SUC_lexicon_tags_26 },
    { 0x67c205e3, SUC_lexicon_tags_9 },
    { 0x14205e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd98205e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb28205e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c205ec, SUC_lexicon_tags_12 },
    { 0x11c205ec, SUC_lexicon_tags_38 },
    { 0x3b4205ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf98205f0, SUC_lexicon_tags_1524 },
    { 0x304205f0, SUC_lexicon_tags_43 },
    { 0x84205f2, SUC_lexicon_tags_10 },
    { 0x4205f3, SUC_lexicon_tags_11 },
    { 0x948205f0, SUC_lexicon_tags_34 },
    { 0xef8205f5, SUC_lexicon_tags_17 },
    { 0x77e205f6, SUC_lexicon_tags_2 },
    { 0xc0e205f5, SUC_lexicon_tags_25 },
    { 0x57c205f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x202205fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14c205fc, SUC_lexicon_tags_38 },
    { 0x9f4205fd, SUC_lexicon_tags_57 },
    { 0xa14205fe, SUC_lexicon_tags_26 },
    { 0x1c0205fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a20603, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf502060c, SUC_lexicon_tags_0 },
    { 0xece2060c, SUC_lexicon_tags_43 },
    { 0x6b62060e, SUC_lexicon_tags_25 },
    { 0x8f82060f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x93820611, SUC_lexicon_tags_950 },
    { 0xd2820612, SUC_lexicon_tags_43 },
    { 0x5c20613, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xae020616, SUC_lexicon_tags_2 },
    { 0x32c20617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22420619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52020620, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe2820622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a220626, SUC_lexicon_tags_2 },
    { 0xcc420627, SUC_lexicon_tags_40 },
    { 0x12020626, SUC_lexicon_tags_39 },
    { 0xf7420626, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7342062b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a2062e, SUC_lexicon_tags_1 },
    { 0xf9c2062e, SUC_lexicon_tags_25 },
    { 0x74c20630, SUC_lexicon_tags_2 },
    { 0x7a20631, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1e20633, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80020635, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19e20637, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4a220639, SUC_lexicon_tags_31 },
    { 0xf882063a, SUC_lexicon_tags_25 },
    { 0x3d02063b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc20645, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c20648, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e2064b, SUC_lexicon_tags_49 },
    { 0x1982064b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f62064e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7820651, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab02065a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaca2065c, SUC_lexicon_tags_74 },
    { 0xf762065c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xad02065f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71220663, SUC_lexicon_tags_47 },
    { 0xe9620664, SUC_lexicon_tags_700 },
    { 0xf7a20665, SUC_lexicon_tags_2 },
    { 0x19620663, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb202066d, SUC_lexicon_tags_11 },
    { 0x3262066d, SUC_lexicon_tags_66 },
    { 0x12c2066f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc20675, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19a20677, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x23e20679, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b82067d, SUC_lexicon_tags_32 },
    { 0x8e82067e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c820688, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f42068b, SUC_lexicon_tags_0 },
    { 0x1c82068c, SUC_lexicon_tags_29 },
    { 0xfde2068c, SUC_lexicon_tags_25 },
    { 0xfd62068b, SUC_lexicon_tags_43 },
    { 0x1aa2068b, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc20693, SUC_lexicon_tags_11 },
    { 0x99220693, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb7020696, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad22069a, SUC_lexicon_tags_16 },
    { 0x4d42069a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x5f82069d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x178206a0, SUC_lexicon_tags_43 },
    { 0x318206a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca206a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e206af, SUC_lexicon_tags_11 },
    { 0xa2a206af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6206b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc92206b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfda206ba, SUC_lexicon_tags_17 },
    { 0x436206ba, SUC_lexicon_tags_11 },
    { 0x58a206bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x49c206be, SUC_lexicon_tags_25 },
    { 0xdb8206bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2206c4, SUC_lexicon_tags_0 },
    { 0x6c206c4, SUC_lexicon_tags_160 },
    { 0x558206c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4206d0, SUC_lexicon_tags_68 },
    { 0x9ca206d1, SUC_lexicon_tags_14 },
    { 0x794206d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a206d6, SUC_lexicon_tags_1 },
    { 0xa74206d7, SUC_lexicon_tags_60 },
    { 0x784206d8, SUC_lexicon_tags_19 },
    { 0x492206d6, SUC_lexicon_tags_14 },
    { 0x6aa206d9, SUC_lexicon_tags_31 },
    { 0xc6206db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60206df, SUC_lexicon_tags_11 },
    { 0x428206e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa206e6, SUC_lexicon_tags_5 },
    { 0xf42206e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e206ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x848206f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76a206f4, SUC_lexicon_tags_48 },
    { 0xcda206f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb80206f7, SUC_lexicon_tags_99 },
    { 0x4fa206f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c206fd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x680206ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7e20701, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb820707, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb982070e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae220710, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3820716, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a2071b, SUC_lexicon_tags_11 },
    { 0xaee2071b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6e2071e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b220721, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a20724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8c20726, SUC_lexicon_tags_30 },
    { 0xccc20726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e2072e, SUC_lexicon_tags_38 },
    { 0x7222072e, SUC_lexicon_tags_11 },
    { 0xf562072e, SUC_lexicon_tags_7 },
    { 0xe2220731, SUC_lexicon_tags_15 },
    { 0xf062072e, SUC_lexicon_tags_19 },
    { 0xf0420733, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5820735, SUC_lexicon_tags_25 },
    { 0x95220736, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x2e20738, SUC_lexicon_tags_33 },
    { 0xf6420738, SUC_lexicon_tags_49 },
    { 0xb0c2073a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc2073d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e20741, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c20744, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee20753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a20759, SUC_lexicon_tags_61 },
    { 0xb7a2075a, SUC_lexicon_tags_26 },
    { 0x5722075b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd322076b, SUC_lexicon_tags_15 },
    { 0x44e2076c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe242076e, SUC_lexicon_tags_12 },
    { 0x3aa2076f, SUC_lexicon_tags_11 },
    { 0x99620770, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e20776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9820779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f42077c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20a2077e, SUC_lexicon_tags_17 },
    { 0x5e02077e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9202078a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6902078f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf420791, SUC_lexicon_tags_28 },
    { 0xfbc20792, SUC_lexicon_tags_0 },
    { 0x46220792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e220797, SUC_lexicon_tags_27 },
    { 0xe1420797, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e2079c, SUC_lexicon_tags_47 },
    { 0x66a2079d, SUC_lexicon_tags_8 },
    { 0x8bc2079c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ca207a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ee207a2, SUC_lexicon_tags_17 },
    { 0xe8a207a3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca207a8, SUC_lexicon_tags_18 },
    { 0x182207a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a207ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x56c207af, SUC_lexicon_tags_25 },
    { 0x5fc207b0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x14c207b2, SUC_lexicon_tags_37 },
    { 0x79a207b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc207b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e207bd, SUC_lexicon_tags_47 },
    { 0xc3c207bd, SUC_lexicon_tags_45 },
    { 0xab8207bf, SUC_lexicon_tags_37 },
    { 0x120207c0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264207c4, SUC_lexicon_tags_27 },
    { 0xbd8207c5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x168207c8, SUC_lexicon_tags_55 },
    { 0x994207c9, SUC_lexicon_tags_92 },
    { 0x1f4207c9, SUC_lexicon_tags_11 },
    { 0xe4a207ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9a207cd, SUC_lexicon_tags_8 },
    { 0xfd4207ce, SUC_lexicon_tags_15 },
    { 0xbfc207cf, SUC_lexicon_tags_17 },
    { 0x96a207cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec207d5, SUC_lexicon_tags_15 },
    { 0xe2207d6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466207e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea207e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214207e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726207f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82207f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74207fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x116207ff, SUC_lexicon_tags_37 },
    { 0x1bc20800, SUC_lexicon_tags_39 },
    { 0xcc620801, SUC_lexicon_tags_37 },
    { 0x46020802, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf820807, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd062080a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c220810, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c20815, SUC_lexicon_tags_49 },
    { 0x83e20816, SUC_lexicon_tags_12 },
    { 0x9d020817, SUC_lexicon_tags_0 },
    { 0xe2220815, SUC_lexicon_tags_12 },
    { 0x19020819, SUC_lexicon_tags_8 },
    { 0x48020819, SUC_lexicon_tags_27 },
    { 0x7062081a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55a2081d, SUC_lexicon_tags_25 },
    { 0xd3a2081e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98220820, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8020826, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14820829, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce62082d, SUC_lexicon_tags_2 },
    { 0x34c2082e, SUC_lexicon_tags_11 },
    { 0xcac2082f, SUC_lexicon_tags_12 },
    { 0xc5620830, SUC_lexicon_tags_9 },
    { 0x1b420830, SUC_lexicon_tags_3 },
    { 0xb3020832, SUC_lexicon_tags_29 },
    { 0x74e20831, SUC_lexicon_tags_1 },
    { 0x14e20834, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b220839, SUC_lexicon_tags_37 },
    { 0x2082083a, SUC_lexicon_tags_12 },
    { 0x21c20839, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f02083e, SUC_lexicon_tags_25 },
    { 0x2082083e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc20844, SUC_lexicon_tags_37 },
    { 0xcdc20845, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95c20847, SUC_lexicon_tags_2 },
    { 0x50620848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c2084b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d42084e, SUC_lexicon_tags_37 },
    { 0x2ce2084f, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52420854, SUC_lexicon_tags_6 },
    { 0x1a020855, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3402085c, SUC_lexicon_tags_15 },
    { 0xdd82085c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6022085f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40020864, SUC_lexicon_tags_33 },
    { 0x24a20865, SUC_lexicon_tags_12 },
    { 0xd6020865, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7042086a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f62086c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9782086e, SUC_lexicon_tags_25 },
    { 0x3f82086f, SUC_lexicon_tags_28 },
    { 0x27020870, SUC_lexicon_tags_61 },
    { 0x9bc2086e, SUC_lexicon_tags_37 },
    { 0x46a20870, SUC_lexicon_tags_57 },
    { 0x76820870, SUC_lexicon_tags_9 },
    { 0xad820870, SUC_lexicon_tags_47 },
    { 0x93e20874, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8dc20877, SUC_lexicon_tags_12 },
    { 0x22e20878, SUC_lexicon_tags_48 },
    { 0xdc620878, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f02087e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4220884, SUC_lexicon_tags_26 },
    { 0xe4420885, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdc20887, SUC_lexicon_tags_12 },
    { 0x26620888, SUC_lexicon_tags_17 },
    { 0x48020889, SUC_lexicon_tags_27 },
    { 0x73a20888, SUC_lexicon_tags_17 },
    { 0xc362088b, SUC_lexicon_tags_15 },
    { 0x6002088c, SUC_lexicon_tags_63 },
    { 0x3422088d, SUC_lexicon_tags_37 },
    { 0xbe82088b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e20892, SUC_lexicon_tags_11 },
    { 0x11420892, SUC_lexicon_tags_11 },
    { 0xeb020894, SUC_lexicon_tags_37 },
    { 0x14620895, SUC_lexicon_tags_9 },
    { 0x5ea20895, SUC_lexicon_tags_39 },
    { 0x1b820896, SUC_lexicon_tags_38 },
    { 0xe1620894, SUC_lexicon_tags_23 },
    { 0xf3620893, SUC_lexicon_tags_57 },
    { 0x6062089a, SUC_lexicon_tags_37 },
    { 0xd842089b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8208a0, SUC_lexicon_tags_23 },
    { 0x4e8208a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770208a8, SUC_lexicon_tags_17 },
    { 0x3de208a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63e208ab, SUC_lexicon_tags_29 },
    { 0x6ae208ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c208af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a208b2, SUC_lexicon_tags_30 },
    { 0xa16208b2, SUC_lexicon_tags_2 },
    { 0x3d4208b4, SUC_lexicon_tags_43 },
    { 0xcca208b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x92e208b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x732208ba, SUC_lexicon_tags_25 },
    { 0xad8208bb, SUC_lexicon_tags_12 },
    { 0x2c0208bc, SUC_lexicon_tags_37 },
    { 0x6d6208bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4208c3, SUC_lexicon_tags_25 },
    { 0x69c208c3, SUC_lexicon_tags_11 },
    { 0x508208c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e208c8, SUC_lexicon_tags_12 },
    { 0xcd4208c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe4208cb, SUC_lexicon_tags_37 },
    { 0x968208cc, SUC_lexicon_tags_49 },
    { 0x4ba208cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e208d5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8208d9, SUC_lexicon_tags_11 },
    { 0x332208da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48208dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e208df, SUC_lexicon_tags_7 },
    { 0x1ac208df, SUC_lexicon_tags_7 },
    { 0x3c0208e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c208e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce208e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8208f0, SUC_lexicon_tags_25 },
    { 0xfc2208f1, SUC_lexicon_tags_8 },
    { 0x4b4208f2, SUC_lexicon_tags_37 },
    { 0x45e208f3, SUC_lexicon_tags_590 },
    { 0xc208f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xad8208f6, SUC_lexicon_tags_182 },
    { 0x5a6208f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0220902, SUC_lexicon_tags_28 },
    { 0x7ca20902, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9062090c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee2090e, SUC_lexicon_tags_49 },
    { 0xaec2090f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa420911, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a220916, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc062091e, SUC_lexicon_tags_7 },
    { 0xad02091f, SUC_lexicon_tags_25 },
    { 0x6042091f, SUC_lexicon_tags_33 },
    { 0x81020921, SUC_lexicon_tags_13 },
    { 0x3dc20922, SUC_lexicon_tags_37 },
    { 0xb820920, SUC_lexicon_tags_19 },
    { 0xa7220924, SUC_lexicon_tags_7 },
    { 0x51a20925, SUC_lexicon_tags_0 },
    { 0x32420925, SUC_lexicon_tags_2 },
    { 0x17420922, SUC_lexicon_tags_80 },
    { 0x25220926, SUC_lexicon_tags_17 },
    { 0xfc220929, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba02092b, SUC_lexicon_tags_83 },
    { 0x2342092c, SUC_lexicon_tags_39 },
    { 0x7342092d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20930, SUC_lexicon_tags_7 },
    { 0xf2a20930, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3ca20933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e2093a, SUC_lexicon_tags_62 },
    { 0x7242093a, SUC_lexicon_tags_11 },
    { 0x5f82093a, SUC_lexicon_tags_15 },
    { 0x5882093d, SUC_lexicon_tags_1 },
    { 0x4482093e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4420941, SUC_lexicon_tags_33 },
    { 0x90a20942, SUC_lexicon_tags_25 },
    { 0xb7820943, SUC_lexicon_tags_43 },
    { 0x69220944, SUC_lexicon_tags_7 },
    { 0x58a20941, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa20948, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd802094d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de20950, SUC_lexicon_tags_37 },
    { 0xb420951, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42020957, SUC_lexicon_tags_11 },
    { 0xfb020957, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca020964, SUC_lexicon_tags_13 },
    { 0xf2e20964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f820969, SUC_lexicon_tags_11 },
    { 0xd0c20969, SUC_lexicon_tags_25 },
    { 0x1620969, SUC_lexicon_tags_6 },
    { 0x7662096b, SUC_lexicon_tags_31 },
    { 0xa0e2096d, SUC_lexicon_tags_31 },
    { 0x3122096d, SUC_lexicon_tags_30 },
    { 0x4862096f, SUC_lexicon_tags_43 },
    { 0x2682096c, SUC_lexicon_tags_25 },
    { 0x92e20971, SUC_lexicon_tags_11 },
    { 0xffa20971, SUC_lexicon_tags_30 },
    { 0xff020973, SUC_lexicon_tags_12 },
    { 0x85420974, SUC_lexicon_tags_27 },
    { 0x6fe20975, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a20979, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce2097c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaca2097e, SUC_lexicon_tags_16 },
    { 0xe0e2097e, SUC_lexicon_tags_17 },
    { 0x46220980, SUC_lexicon_tags_27 },
    { 0xa620980, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f620985, SUC_lexicon_tags_46 },
    { 0x51220986, SUC_lexicon_tags_28 },
    { 0x84e20987, SUC_lexicon_tags_299 },
    { 0x9e620988, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf42098e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb7820990, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10a20992, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c620997, SUC_lexicon_tags_15 },
    { 0x15620998, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9e62099a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2099f, SUC_lexicon_tags_27 },
    { 0xc5a209a0, SUC_lexicon_tags_19 },
    { 0x5e6209a0, SUC_lexicon_tags_27 },
    { 0x5cc209a2, SUC_lexicon_tags_12 },
    { 0x248209a3, SUC_lexicon_tags_25 },
    { 0x362209a4, SUC_lexicon_tags_27 },
    { 0x30e209a5, SUC_lexicon_tags_4 },
    { 0x9c22099f, SUC_lexicon_tags_12 },
    { 0x4bc209a5, SUC_lexicon_tags_43 },
    { 0x346209a8, SUC_lexicon_tags_49 },
    { 0xe42209a9, SUC_lexicon_tags_11 },
    { 0x3f6209a1, SUC_lexicon_tags_28 },
    { 0xe82099f, SUC_lexicon_tags_11 },
    { 0xbd4209ac, SUC_lexicon_tags_21 },
    { 0xe42209a8, SUC_lexicon_tags_38 },
    { 0x5102099f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbec209b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6e209b2, SUC_lexicon_tags_37 },
    { 0xa92209b2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e209b8, SUC_lexicon_tags_35 },
    { 0x6b4209b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2209bc, SUC_lexicon_tags_11 },
    { 0xd7a209bd, SUC_lexicon_tags_7 },
    { 0xd5a209be, SUC_lexicon_tags_37 },
    { 0xb4e209bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa209c6, SUC_lexicon_tags_23 },
    { 0x6a6209c7, SUC_lexicon_tags_23 },
    { 0x9f8209c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914209cc, SUC_lexicon_tags_15 },
    { 0x124209cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe209d7, SUC_lexicon_tags_6 },
    { 0x374209d8, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0xf22209da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04209de, SUC_lexicon_tags_26 },
    { 0x614209df, SUC_lexicon_tags_17 },
    { 0xef0209e0, SUC_lexicon_tags_0 },
    { 0x25c209e1, SUC_lexicon_tags_0 },
    { 0x4c4209e0, SUC_lexicon_tags_7 },
    { 0x4c4209e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec209e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb0c209e9, SUC_lexicon_tags_37 },
    { 0x2be209ea, SUC_lexicon_tags_12 },
    { 0x412209e9, SUC_lexicon_tags_1 },
    { 0xd0c209eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89c209ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f4209f0, SUC_lexicon_tags_48 },
    { 0x948209f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0209f6, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2209fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52620a02, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5620a05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0220a09, SUC_lexicon_tags_7 },
    { 0xdfa20a0a, SUC_lexicon_tags_2 },
    { 0x71620a0a, SUC_lexicon_tags_11 },
    { 0x52620a0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb620a0e, SUC_lexicon_tags_11 },
    { 0x88e20a0f, SUC_lexicon_tags_66 },
    { 0x6f420a10, SUC_lexicon_tags_25 },
    { 0xb0e20a10, SUC_lexicon_tags_1 },
    { 0xd1220a11, SUC_lexicon_tags_4 },
    { 0xf9820a13, SUC_lexicon_tags_37 },
    { 0xe3620a14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b220a1e, SUC_lexicon_tags_19 },
    { 0xac220a1f, SUC_lexicon_tags_37 },
    { 0x97c20a20, SUC_lexicon_tags_17 },
    { 0xbd620a1e, SUC_lexicon_tags_14 },
    { 0xae020a22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca020a26, SUC_lexicon_tags_25 },
    { 0xe1620a27, SUC_lexicon_tags_34 },
    { 0x5a820a26, SUC_lexicon_tags_17 },
    { 0x42e20a29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc20a2d, SUC_lexicon_tags_7 },
    { 0x1b020a2e, SUC_lexicon_tags_37 },
    { 0xbfe20a2f, SUC_lexicon_tags_27 },
    { 0x30820a2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99420a32, SUC_lexicon_tags_87 },
    { 0xf5a20a32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e020a37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e420a3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81220a3f, SUC_lexicon_tags_12 },
    { 0x55620a40, SUC_lexicon_tags_0 },
    { 0xba020a41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4420a45, SUC_lexicon_tags_45 },
    { 0xdf020a46, SUC_lexicon_tags_4 },
    { 0x1c620a46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7420a4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26820a4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e20a52, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4fc20a54, SUC_lexicon_tags_19 },
    { 0x90020a55, SUC_lexicon_tags_9 },
    { 0xe1220a54, SUC_lexicon_tags_47 },
    { 0x7f020a57, SUC_lexicon_tags_11 },
    { 0x52220a58, SUC_lexicon_tags_11 },
    { 0xfdc20a54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f820a5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71820a60, SUC_lexicon_tags_37 },
    { 0x3620a61, SUC_lexicon_tags_0 },
    { 0xac420a62, SUC_lexicon_tags_17 },
    { 0x8c220a61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a020a65, SUC_lexicon_tags_19 },
    { 0x98e20a66, SUC_lexicon_tags_37 },
    { 0x47e20a67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce20a6a, SUC_lexicon_tags_15 },
    { 0x7420a6a, SUC_lexicon_tags_16 },
    { 0x16c20a6a, SUC_lexicon_tags_17 },
    { 0xe220a6d, SUC_lexicon_tags_12 },
    { 0xe6620a6e, SUC_lexicon_tags_25 },
    { 0xbe620a6f, SUC_lexicon_tags_15 },
    { 0x3d220a6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x46220a73, SUC_lexicon_tags_57 },
    { 0xed220a73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab820a77, SUC_lexicon_tags_8 },
    { 0x33420a78, SUC_lexicon_tags_7 },
    { 0x8b620a79, SUC_lexicon_tags_28 },
    { 0x1e420a79, SUC_lexicon_tags_12 },
    { 0x30220a7a, SUC_lexicon_tags_27 },
    { 0xba620a7b, SUC_lexicon_tags_2 },
    { 0xc5220a7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5020a81, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b820a84, SUC_lexicon_tags_85 },
    { 0x70020a85, SUC_lexicon_tags_32 },
    { 0xed420a86, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x79020a88, SUC_lexicon_tags_1 },
    { 0x30220a89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0e20a8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2620a92, SUC_lexicon_tags_54 },
    { 0xfe20a93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bc20a95, SUC_lexicon_tags_229 },
    { 0x8620a95, SUC_lexicon_tags_43 },
    { 0x4a420a97, SUC_lexicon_tags_19 },
    { 0x13a20a97, SUC_lexicon_tags_60 },
    { 0xec820a97, SUC_lexicon_tags_25 },
    { 0x32020a9a, SUC_lexicon_tags_17 },
    { 0xb2620a96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71820aa1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83020aa5, SUC_lexicon_tags_15 },
    { 0x51020aa6, SUC_lexicon_tags_9 },
    { 0x9f620aa7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x23c20aa9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c20aac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e20ab2, SUC_lexicon_tags_12 },
    { 0xabc20ab2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x39e20ab5, SUC_lexicon_tags_19 },
    { 0x39620ab6, SUC_lexicon_tags_25 },
    { 0xb8c20ab7, SUC_lexicon_tags_1 },
    { 0x83e20ab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4820aba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e20abf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a20ac2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8fe20ac4, SUC_lexicon_tags_15 },
    { 0x80c20ac4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45020aca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf1620acc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1420ad1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f820ad4, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c20ad9, SUC_lexicon_tags_30 },
    { 0xdde20ada, SUC_lexicon_tags_33 },
    { 0x5d620adb, SUC_lexicon_tags_47 },
    { 0x11420adc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32a20ade, SUC_lexicon_tags_27 },
    { 0x30820adf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36220ae5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d820ae9, SUC_lexicon_tags_11 },
    { 0xc6420ae9, SUC_lexicon_tags_45 },
    { 0xcfc20aeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9420aee, SUC_lexicon_tags_2 },
    { 0x96a20aee, SUC_lexicon_tags_26 },
    { 0x99220af0, SUC_lexicon_tags_88 },
    { 0xf0020aef, SUC_lexicon_tags_35 },
    { 0x60620af1, SUC_lexicon_tags_1 },
    { 0x18a20af3, SUC_lexicon_tags_44 },
    { 0xb4c20aef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e20af9, SUC_lexicon_tags_38 },
    { 0xe620af9, SUC_lexicon_tags_46 },
    { 0x5f620afb, SUC_lexicon_tags_112 },
    { 0x8f220afc, SUC_lexicon_tags_26 },
    { 0xbc620afd, SUC_lexicon_tags_33 },
    { 0xb6420afd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf420b02, SUC_lexicon_tags_15 },
    { 0x2ee20b03, SUC_lexicon_tags_17 },
    { 0x4e420b04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a220b07, SUC_lexicon_tags_2 },
    { 0x1a420b08, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x83820b0a, SUC_lexicon_tags_37 },
    { 0x69420b0b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49220b0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d820b11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a20b14, SUC_lexicon_tags_2 },
    { 0xe9a20b14, SUC_lexicon_tags_14 },
    { 0x37220b15, SUC_lexicon_tags_11 },
    { 0xc020b17, SUC_lexicon_tags_25 },
    { 0x53220b16, SUC_lexicon_tags_27 },
    { 0xbc20b15, SUC_lexicon_tags_8 },
    { 0x8d420b15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b620b1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbe20b1e, SUC_lexicon_tags_0 },
    { 0x17e20b1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada20b23, SUC_lexicon_tags_75 },
    { 0x34220b24, SUC_lexicon_tags_3 },
    { 0xbf220b25, SUC_lexicon_tags_9 },
    { 0xa8420b24, SUC_lexicon_tags_26 },
    { 0x4620b26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24820b2b, SUC_lexicon_tags_37 },
    { 0x93620b2c, SUC_lexicon_tags_37 },
    { 0x1a020b2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44620b2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c20b37, SUC_lexicon_tags_6 },
    { 0xeb620b38, SUC_lexicon_tags_15 },
    { 0x26a20b39, SUC_lexicon_tags_17 },
    { 0xb6c20b38, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9b620b3c, SUC_lexicon_tags_45 },
    { 0xc6e20b3c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae20b40, SUC_lexicon_tags_25 },
    { 0x88420b40, SUC_lexicon_tags_30 },
    { 0x48c20b42, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c20b45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22820b48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62220b4a, SUC_lexicon_tags_31 },
    { 0xf3620b4b, SUC_lexicon_tags_11 },
    { 0x3a020b4a, SUC_lexicon_tags_12 },
    { 0x76020b4d, SUC_lexicon_tags_43 },
    { 0xb0620b4e, SUC_lexicon_tags_46 },
    { 0x49220b4c, SUC_lexicon_tags_17 },
    { 0xfaa20b4e, SUC_lexicon_tags_45 },
    { 0x2ca20b51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57620b59, SUC_lexicon_tags_11 },
    { 0xcf220b59, SUC_lexicon_tags_9 },
    { 0xf5620b5b, SUC_lexicon_tags_14 },
    { 0x75020b5c, SUC_lexicon_tags_49 },
    { 0x65820b5b, SUC_lexicon_tags_37 },
    { 0xfaa20b5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94620b61, SUC_lexicon_tags_12 },
    { 0x77e20b62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4620b64, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbe420b66, SUC_lexicon_tags_15 },
    { 0xa3a20b66, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1d620b69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x91a20b6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x88c20b6d, SUC_lexicon_tags_63 },
    { 0xf5420b6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be20b71, SUC_lexicon_tags_38 },
    { 0xd0e20b72, SUC_lexicon_tags_60 },
    { 0x6cc20b71, SUC_lexicon_tags_60 },
    { 0xe6220b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad620b78, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8220b7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb220b7e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2220b82, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a20b87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ec20b89, SUC_lexicon_tags_11 },
    { 0xa0220b8a, SUC_lexicon_tags_25 },
    { 0x820b8b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2820b8f, SUC_lexicon_tags_17 },
    { 0xe4820b8f, SUC_lexicon_tags_17 },
    { 0xbfe20b8f, SUC_lexicon_tags_10 },
    { 0xbd020b92, SUC_lexicon_tags_17 },
    { 0x8c820b92, SUC_lexicon_tags_37 },
    { 0x74a20b93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x78e20b96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37620b98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbdc20b9a, SUC_lexicon_tags_19 },
    { 0x26420b9b, SUC_lexicon_tags_60 },
    { 0x43e20b9c, SUC_lexicon_tags_61 },
    { 0x10620b9c, SUC_lexicon_tags_1 },
    { 0x6a020b9d, SUC_lexicon_tags_25 },
    { 0xfde20b9e, SUC_lexicon_tags_26 },
    { 0xbee20ba0, SUC_lexicon_tags_46 },
    { 0x98620b9f, SUC_lexicon_tags_25 },
    { 0xaa620b9b, SUC_lexicon_tags_60 },
    { 0xe7220ba1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb1220ba5, SUC_lexicon_tags_39 },
    { 0xed820ba6, SUC_lexicon_tags_4 },
    { 0xfde20ba7, SUC_lexicon_tags_7 },
    { 0xeae20ba8, SUC_lexicon_tags_15 },
    { 0x2d620ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38e20bab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9c20bad, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe7a20baf, SUC_lexicon_tags_33 },
    { 0x74e20bb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2220bb2, SUC_lexicon_tags_2 },
    { 0xeda20bb3, SUC_lexicon_tags_1 },
    { 0x5f820bb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89820bb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a20bbd, SUC_lexicon_tags_12 },
    { 0x96820bbe, SUC_lexicon_tags_17 },
    { 0xd5a20bbf, SUC_lexicon_tags_31 },
    { 0xa2c20bc0, SUC_lexicon_tags_37 },
    { 0x57a20bc0, SUC_lexicon_tags_15 },
    { 0xf1c20bc2, SUC_lexicon_tags_37 },
    { 0x2ea20bc3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe2c20bc5, SUC_lexicon_tags_19 },
    { 0x20220bc5, SUC_lexicon_tags_37 },
    { 0xa7220bc7, SUC_lexicon_tags_1 },
    { 0xd5c20bc8, SUC_lexicon_tags_15 },
    { 0xcbe20bc9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae20bcc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5820bd0, SUC_lexicon_tags_37 },
    { 0x3d220bd1, SUC_lexicon_tags_15 },
    { 0x3fa20bd1, SUC_lexicon_tags_28 },
    { 0x6bc20bd3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e20bd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5fa20bd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1420bdb, SUC_lexicon_tags_19 },
    { 0xaf020bdc, SUC_lexicon_tags_107 },
    { 0xeec20bdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5a20bdf, SUC_lexicon_tags_5 },
    { 0xea20bdf, SUC_lexicon_tags_4 },
    { 0xfde20bdf, SUC_lexicon_tags_0 },
    { 0x42020be2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5420be7, SUC_lexicon_tags_37 },
    { 0xcea20be8, SUC_lexicon_tags_44 },
    { 0xcdc20be8, SUC_lexicon_tags_75 },
    { 0x59a20bea, SUC_lexicon_tags_33 },
    { 0x66420bea, SUC_lexicon_tags_47 },
    { 0xbea20bec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b820bf0, SUC_lexicon_tags_63 },
    { 0x3f220bf1, SUC_lexicon_tags_37 },
    { 0x97e20bf2, SUC_lexicon_tags_37 },
    { 0x81a20bf2, SUC_lexicon_tags_29 },
    { 0xb8820bf0, SUC_lexicon_tags_12 },
    { 0x19820bf5, SUC_lexicon_tags_11 },
    { 0xfbe20bf1, SUC_lexicon_tags_18 },
    { 0xc8420bf6, SUC_lexicon_tags_70 },
    { 0x75c20bf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27620bfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3420bfd, SUC_lexicon_tags_29 },
    { 0x32420bfe, SUC_lexicon_tags_12 },
    { 0xfda20bfe, SUC_lexicon_tags_65 },
    { 0x68a20bfe, SUC_lexicon_tags_25 },
    { 0x96c20c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67c20c03, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2820c07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e20c0a, SUC_lexicon_tags_33 },
    { 0x28a20c0b, SUC_lexicon_tags_11 },
    { 0x91220c0c, SUC_lexicon_tags_6 },
    { 0xb2020c0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e20c10, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeec20c12, SUC_lexicon_tags_25 },
    { 0xf8620c12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9820c19, SUC_lexicon_tags_3 },
    { 0x4a620c1a, SUC_lexicon_tags_25 },
    { 0x55620c1b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a20c21, SUC_lexicon_tags_2 },
    { 0x88e20c21, SUC_lexicon_tags_37 },
    { 0xfa020c23, SUC_lexicon_tags_26 },
    { 0xfdc20c21, SUC_lexicon_tags_2 },
    { 0x18620c25, SUC_lexicon_tags_25 },
    { 0x39020c26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5820c2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a20c2f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40020c33, SUC_lexicon_tags_46 },
    { 0x66e20c34, SUC_lexicon_tags_7 },
    { 0x75220c35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59820c37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18620c3c, SUC_lexicon_tags_6 },
    { 0x7fe20c3d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec20c40, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3820c43, SUC_lexicon_tags_37 },
    { 0xaac20c44, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a20c47, SUC_lexicon_tags_26 },
    { 0x93c20c48, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xefe20c4a, SUC_lexicon_tags_29 },
    { 0xb2620c4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x17420c4e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x15620c51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53620c53, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93820c58, SUC_lexicon_tags_29 },
    { 0x74c20c59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8d220c5b, SUC_lexicon_tags_38 },
    { 0x38420c5c, SUC_lexicon_tags_17 },
    { 0xf7a20c5d, SUC_lexicon_tags_37 },
    { 0x3d020c5e, SUC_lexicon_tags_43 },
    { 0x3cc20c5f, SUC_lexicon_tags_12 },
    { 0x58220c5e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a20c63, SUC_lexicon_tags_37 },
    { 0xb2420c63, SUC_lexicon_tags_15 },
    { 0xd5820c64, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a20c71, SUC_lexicon_tags_14 },
    { 0x99220c72, SUC_lexicon_tags_6 },
    { 0xc9e20c73, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x58820c75, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c20c7d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x61c20c7f, SUC_lexicon_tags_46 },
    { 0x1020c7f, SUC_lexicon_tags_17 },
    { 0x82c20c7f, SUC_lexicon_tags_17 },
    { 0xba620c82, SUC_lexicon_tags_16 },
    { 0xafc20c83, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x72220c85, SUC_lexicon_tags_27 },
    { 0xd8820c86, SUC_lexicon_tags_117 },
    { 0xd0e20c87, SUC_lexicon_tags_19 },
    { 0x41820c85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b820c8e, SUC_lexicon_tags_43 },
    { 0x98e20c8e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3e420c91, SUC_lexicon_tags_1 },
    { 0x83220c91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c220c94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c220c99, SUC_lexicon_tags_2 },
    { 0xe3e20c9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77e20c9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3a420c9e, SUC_lexicon_tags_11 },
    { 0x9da20c9f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97220ca4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2e420ca6, SUC_lexicon_tags_38 },
    { 0x2c20ca7, SUC_lexicon_tags_31 },
    { 0x69020ca8, SUC_lexicon_tags_28 },
    { 0x60c20ca6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb620cab, SUC_lexicon_tags_9 },
    { 0xdd420cac, SUC_lexicon_tags_33 },
    { 0x2c20cab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa20cb0, SUC_lexicon_tags_7 },
    { 0xd3420cb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e820cb5, SUC_lexicon_tags_52 },
    { 0xaba20cb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d220cb8, SUC_lexicon_tags_15 },
    { 0xa7620cb9, SUC_lexicon_tags_37 },
    { 0xf7c20cba, SUC_lexicon_tags_37 },
    { 0x4b820cbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8420cc1, SUC_lexicon_tags_21 },
    { 0x12620cc2, SUC_lexicon_tags_27 },
    { 0x3ea20cc1, SUC_lexicon_tags_13 },
    { 0xbca20cc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50c20cc6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x57a20cc8, SUC_lexicon_tags_10 },
    { 0x9a020cc9, SUC_lexicon_tags_1 },
    { 0xd8420cca, SUC_lexicon_tags_29 },
    { 0x17220ccb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e20cd2, SUC_lexicon_tags_17 },
    { 0x3ac20cd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbae20cd5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c20cd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba620ce2, SUC_lexicon_tags_25 },
    { 0x16420ce2, SUC_lexicon_tags_17 },
    { 0xb020ce2, SUC_lexicon_tags_37 },
    { 0x88020ce5, SUC_lexicon_tags_11 },
    { 0xaf020ce6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae20ce9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33420ceb, SUC_lexicon_tags_37 },
    { 0xb3020cec, SUC_lexicon_tags_68 },
    { 0xede20ceb, SUC_lexicon_tags_21 },
    { 0x33c20cee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd620cf0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc20cf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64420cfc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e820d00, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x12220d03, SUC_lexicon_tags_12 },
    { 0xdca20d04, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xde420d06, SUC_lexicon_tags_45 },
    { 0x9a20d07, SUC_lexicon_tags_7 },
    { 0x7b820d07, SUC_lexicon_tags_37 },
    { 0x56020d06, SUC_lexicon_tags_0 },
    { 0xaa020d06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90420d0c, SUC_lexicon_tags_25 },
    { 0x32a20d0d, SUC_lexicon_tags_12 },
    { 0x3ce20d0e, SUC_lexicon_tags_19 },
    { 0x6c020d0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6620d11, SUC_lexicon_tags_9 },
    { 0x31420d12, SUC_lexicon_tags_1 },
    { 0x1ba20d13, SUC_lexicon_tags_15 },
    { 0x58c20d14, SUC_lexicon_tags_15 },
    { 0xab620d15, SUC_lexicon_tags_17 },
    { 0xc8e20d14, SUC_lexicon_tags_11 },
    { 0x5d620d14, SUC_lexicon_tags_6 },
    { 0x42220d17, SUC_lexicon_tags_2 },
    { 0xfe620d19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb2c20d1b, SUC_lexicon_tags_46 },
    { 0x33020d1c, SUC_lexicon_tags_68 },
    { 0x9e20d1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64020d24, SUC_lexicon_tags_60 },
    { 0xb2420d25, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xe420d28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee20d2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0020d2d, SUC_lexicon_tags_65 },
    { 0xb6a20d2e, SUC_lexicon_tags_23 },
    { 0x2e620d2e, SUC_lexicon_tags_0 },
    { 0xed820d2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa820d3f, SUC_lexicon_tags_11 },
    { 0x2d020d40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c20d43, SUC_lexicon_tags_29 },
    { 0xf6a20d43, SUC_lexicon_tags_23 },
    { 0xe2020d43, SUC_lexicon_tags_28 },
    { 0x35020d44, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda20d4b, SUC_lexicon_tags_27 },
    { 0xa8e20d4c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x14420d4f, SUC_lexicon_tags_57 },
    { 0x45620d4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa20d53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xace20d55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd820d5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e620d60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe620d64, SUC_lexicon_tags_2 },
    { 0x8c620d65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81420d6f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x36620d72, SUC_lexicon_tags_38 },
    { 0x7a20d72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe220d76, SUC_lexicon_tags_8 },
    { 0x4ec20d77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a20d7c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x50620d7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6420d81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20220d85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x30620d87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe9820d89, SUC_lexicon_tags_11 },
    { 0xebc20d8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f220d8c, SUC_lexicon_tags_29 },
    { 0x98020d8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1620d90, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e20d93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a20d97, SUC_lexicon_tags_68 },
    { 0x4e820d98, SUC_lexicon_tags_21 },
    { 0x11220d98, SUC_lexicon_tags_38 },
    { 0x6a420d99, SUC_lexicon_tags_1 },
    { 0x75020d98, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x83020d9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc20da1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x60c20da3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3420da5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed220da7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c20dab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33820daf, SUC_lexicon_tags_12 },
    { 0xd620daf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcac20db2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x23020db4, SUC_lexicon_tags_21 },
    { 0xe1a20db5, SUC_lexicon_tags_16 },
    { 0x4e420db6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f020dc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba20dc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41420dcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95e20dcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee20dd4, SUC_lexicon_tags_60 },
    { 0x15420dd4, SUC_lexicon_tags_2 },
    { 0x5820dd6, SUC_lexicon_tags_7 },
    { 0xac20dd7, SUC_lexicon_tags_43 },
    { 0x83620dd4, SUC_lexicon_tags_25 },
    { 0x53a20dd9, SUC_lexicon_tags_25 },
    { 0x43820dda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc20ddf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4be20de1, SUC_lexicon_tags_25 },
    { 0xc9e20de2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e20de6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9c20de8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x3ca20dea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80220def, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e20df2, SUC_lexicon_tags_63 },
    { 0xfa820df2, SUC_lexicon_tags_17 },
    { 0xc0a20df4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15220df6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f620df8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fe20dfa, SUC_lexicon_tags_17 },
    { 0xb4e20dfb, SUC_lexicon_tags_25 },
    { 0xdfa20dfc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17220e04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47620e0a, SUC_lexicon_tags_14 },
    { 0x7c20e0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52020e0e, SUC_lexicon_tags_11 },
    { 0xee20e0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8220e13, SUC_lexicon_tags_26 },
    { 0xf0e20e14, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe420e19, SUC_lexicon_tags_19 },
    { 0xcfa20e19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6220e1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa620e1e, SUC_lexicon_tags_45 },
    { 0x16a20e1e, SUC_lexicon_tags_17 },
    { 0x7de20e1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3020e22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec20e28, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x22820e2a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e20e2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde020e37, SUC_lexicon_tags_40 },
    { 0x54a20e38, SUC_lexicon_tags_91 },
    { 0x1ae20e38, SUC_lexicon_tags_46 },
    { 0x41620e39, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e20e3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa20e42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x36020e45, SUC_lexicon_tags_37 },
    { 0xe20e46, SUC_lexicon_tags_2 },
    { 0xb9820e45, SUC_lexicon_tags_17 },
    { 0x10020e46, SUC_lexicon_tags_18 },
    { 0xee620e49, SUC_lexicon_tags_17 },
    { 0xe8020e45, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe9820e4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc20e4f, SUC_lexicon_tags_15 },
    { 0x66c20e50, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7bc20e52, SUC_lexicon_tags_2 },
    { 0x79820e53, SUC_lexicon_tags_17 },
    { 0xaae20e54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d020e5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c020e5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf420e63, SUC_lexicon_tags_12 },
    { 0xbf620e64, SUC_lexicon_tags_7 },
    { 0xe2020e64, SUC_lexicon_tags_2 },
    { 0x5420e66, SUC_lexicon_tags_14 },
    { 0x2420e63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a020e69, SUC_lexicon_tags_19 },
    { 0x9020e69, SUC_lexicon_tags_11 },
    { 0x11a20e69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9620e6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdac20e71, SUC_lexicon_tags_65 },
    { 0xc3220e72, SUC_lexicon_tags_7 },
    { 0x7b220e73, SUC_lexicon_tags_23 },
    { 0xef620e74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeae20e76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed420e7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46220e7d, SUC_lexicon_tags_37 },
    { 0xc9220e7e, SUC_lexicon_tags_11 },
    { 0x3820e7d, SUC_lexicon_tags_66 },
    { 0xa2620e7f, SUC_lexicon_tags_25 },
    { 0x9ec20e7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e20e86, SUC_lexicon_tags_49 },
    { 0xf4620e87, SUC_lexicon_tags_25 },
    { 0xf1620e87, SUC_lexicon_tags_63 },
    { 0x26820e89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1020e92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e420e94, SUC_lexicon_tags_17 },
    { 0x76020e94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe9220e97, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x56220e99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0620e9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x73420ea2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x92020ea4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8420ea7, SUC_lexicon_tags_8 },
    { 0x5da20ea8, SUC_lexicon_tags_38 },
    { 0xdf220ea8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15220eb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19020eb4, SUC_lexicon_tags_27 },
    { 0x36e20eb4, SUC_lexicon_tags_17 },
    { 0xc7220eb6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7a820eb8, SUC_lexicon_tags_25 },
    { 0x5ec20eb9, SUC_lexicon_tags_1 },
    { 0x27820eb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac020ebe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a20ec1, SUC_lexicon_tags_37 },
    { 0x4be20ec2, SUC_lexicon_tags_15 },
    { 0xd4620ec2, SUC_lexicon_tags_38 },
    { 0x5d220ec4, SUC_lexicon_tags_12 },
    { 0x89620ec5, SUC_lexicon_tags_0 },
    { 0xe4a20ec5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf2a20ec8, SUC_lexicon_tags_12 },
    { 0xaec20ec9, SUC_lexicon_tags_11 },
    { 0x24420eca, SUC_lexicon_tags_38 },
    { 0x1820eca, SUC_lexicon_tags_45 },
    { 0xa8020eca, SUC_lexicon_tags_0 },
    { 0x72420ecd, SUC_lexicon_tags_14 },
    { 0xc3c20ec9, SUC_lexicon_tags_48 },
    { 0x4a220ecb, SUC_lexicon_tags_45 },
    { 0x22220ec9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea20ed3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0820ed5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3020ed9, SUC_lexicon_tags_12 },
    { 0x41620eda, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5420edd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x5820edf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3020ee2, SUC_lexicon_tags_38 },
    { 0xd2620ee3, SUC_lexicon_tags_6 },
    { 0x2620ee4, SUC_lexicon_tags_12 },
    { 0x620ee5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57220ee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d820ee9, SUC_lexicon_tags_7 },
    { 0x92420eea, SUC_lexicon_tags_48 },
    { 0x2d220eeb, SUC_lexicon_tags_29 },
    { 0xa0e20eec, SUC_lexicon_tags_17 },
    { 0x1d020eea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c20ef4, SUC_lexicon_tags_55 },
    { 0x2fe20ef4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4820ef9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d220efb, SUC_lexicon_tags_60 },
    { 0x3c820efb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae20f00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d420f05, SUC_lexicon_tags_25 },
    { 0x76420f06, SUC_lexicon_tags_11 },
    { 0xe5220f06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x13220f09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e20f0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda420f0f, SUC_lexicon_tags_15 },
    { 0xa420f10, SUC_lexicon_tags_1 },
    { 0xda820f11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc20f13, SUC_lexicon_tags_13 },
    { 0x87420f14, SUC_lexicon_tags_9 },
    { 0xdfa20f15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69a20f17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5820f19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6220f1d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9a420f1f, SUC_lexicon_tags_37 },
    { 0xc7e20f20, SUC_lexicon_tags_13 },
    { 0x52420f21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6620f26, SUC_lexicon_tags_27 },
    { 0x47e20f27, SUC_lexicon_tags_17 },
    { 0x21020f28, SUC_lexicon_tags_0 },
    { 0x7220f29, SUC_lexicon_tags_12 },
    { 0x2ce20f28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc20f2d, SUC_lexicon_tags_63 },
    { 0x2f620f2d, SUC_lexicon_tags_15 },
    { 0x39820f2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8420f32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a20f35, SUC_lexicon_tags_38 },
    { 0x38820f36, SUC_lexicon_tags_11 },
    { 0x9de20f37, SUC_lexicon_tags_11 },
    { 0xb7820f36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2420f3a, SUC_lexicon_tags_17 },
    { 0xa4220f3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce220f3f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x8ba20f41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee20f45, SUC_lexicon_tags_55 },
    { 0x50220f46, SUC_lexicon_tags_62 },
    { 0x88420f47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e020f4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20420f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb420f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf220f51, SUC_lexicon_tags_7 },
    { 0x6f220f52, SUC_lexicon_tags_37 },
    { 0x80220f53, SUC_lexicon_tags_11 },
    { 0x3e820f54, SUC_lexicon_tags_31 },
    { 0xf3820f55, SUC_lexicon_tags_4 },
    { 0x2820f56, SUC_lexicon_tags_12 },
    { 0x43420f54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf220f59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb420f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4620f60, SUC_lexicon_tags_11 },
    { 0x3f620f61, SUC_lexicon_tags_16 },
    { 0x74620f62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e220f64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6820f67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5420f69, SUC_lexicon_tags_11 },
    { 0xe3c20f6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa20f70, SUC_lexicon_tags_33 },
    { 0xcbc20f71, SUC_lexicon_tags_61 },
    { 0x73220f71, SUC_lexicon_tags_33 },
    { 0x13e20f72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xad820f76, SUC_lexicon_tags_12 },
    { 0xdac20f77, SUC_lexicon_tags_12 },
    { 0xbb620f78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6220f7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36420f85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd620f87, SUC_lexicon_tags_19 },
    { 0xa6820f88, SUC_lexicon_tags_49 },
    { 0x77420f89, SUC_lexicon_tags_27 },
    { 0xbd420f8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb620f8e, SUC_lexicon_tags_17 },
    { 0x4e620f8e, SUC_lexicon_tags_14 },
    { 0x9c420f90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95220f9a, SUC_lexicon_tags_61 },
    { 0x67820f9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a20fa1, SUC_lexicon_tags_11 },
    { 0xc5620fa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9420fa4, SUC_lexicon_tags_0 },
    { 0xd1220fa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29c20fa7, SUC_lexicon_tags_7 },
    { 0x8b220fa8, SUC_lexicon_tags_2 },
    { 0x47220fa9, SUC_lexicon_tags_37 },
    { 0x5e820faa, SUC_lexicon_tags_37 },
    { 0x6020fab, SUC_lexicon_tags_15 },
    { 0xaa220fab, SUC_lexicon_tags_32 },
    { 0x6c620fad, SUC_lexicon_tags_19 },
    { 0x6cc20fae, SUC_lexicon_tags_15 },
    { 0x1ba20faf, SUC_lexicon_tags_34 },
    { 0xc4620fb0, SUC_lexicon_tags_27 },
    { 0x9aa20fb1, SUC_lexicon_tags_1 },
    { 0xaf220fae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5fc20fb4, SUC_lexicon_tags_63 },
    { 0xf1c20fb5, SUC_lexicon_tags_60 },
    { 0xf020fb5, SUC_lexicon_tags_49 },
    { 0x46820fb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa20fba, SUC_lexicon_tags_6 },
    { 0x85e20fbb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe20fc0, SUC_lexicon_tags_0 },
    { 0xdb220fc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xece20fc3, SUC_lexicon_tags_14 },
    { 0xacc20fc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a20fc9, SUC_lexicon_tags_25 },
    { 0x33020fca, SUC_lexicon_tags_38 },
    { 0xab620fcb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5020fce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de20fd6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc20fdc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6ac20fde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16220fe7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x89420fe9, SUC_lexicon_tags_11 },
    { 0x51820fea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbae20fec, SUC_lexicon_tags_46 },
    { 0x79620fec, SUC_lexicon_tags_47 },
    { 0x6ea20fec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97020ff3, SUC_lexicon_tags_48 },
    { 0x23c20ff4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2e20ff6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x16c20ff8, SUC_lexicon_tags_45 },
    { 0x7c620ff9, SUC_lexicon_tags_25 },
    { 0xa620ffa, SUC_lexicon_tags_25 },
    { 0xa4c20ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc20ffe, SUC_lexicon_tags_12 },
    { 0x24020ffe, SUC_lexicon_tags_12 },
    { 0x89a21000, SUC_lexicon_tags_14 },
    { 0xd9621001, SUC_lexicon_tags_47 },
    { 0xdcc21002, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea421008, SUC_lexicon_tags_48 },
    { 0x64421009, SUC_lexicon_tags_17 },
    { 0x4ce2100a, SUC_lexicon_tags_34 },
    { 0x5302100b, SUC_lexicon_tags_63 },
    { 0x9b62100c, SUC_lexicon_tags_49 },
    { 0x3082100d, SUC_lexicon_tags_66 },
    { 0xe7c2100d, SUC_lexicon_tags_19 },
    { 0x7ae2100f, SUC_lexicon_tags_14 },
    { 0x7b82100d, SUC_lexicon_tags_1 },
    { 0x642100f, SUC_lexicon_tags_25 },
    { 0xf182100d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c021015, SUC_lexicon_tags_34 },
    { 0x74e21015, SUC_lexicon_tags_12 },
    { 0x24a21016, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5802101a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee2101d, SUC_lexicon_tags_11 },
    { 0x7082101e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf6421020, SUC_lexicon_tags_7 },
    { 0xe2a21021, SUC_lexicon_tags_15 },
    { 0x26221021, SUC_lexicon_tags_4 },
    { 0x72021021, SUC_lexicon_tags_5 },
    { 0x10021020, SUC_lexicon_tags_32 },
    { 0x7a821023, SUC_lexicon_tags_11 },
    { 0xe6621020, SUC_lexicon_tags_38 },
    { 0x53021027, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3642102b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e42102e, SUC_lexicon_tags_25 },
    { 0xad02102f, SUC_lexicon_tags_25 },
    { 0x66621030, SUC_lexicon_tags_45 },
    { 0x63821030, SUC_lexicon_tags_7 },
    { 0x85421030, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59c21034, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec21037, SUC_lexicon_tags_2 },
    { 0xb1821038, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x5bc2103a, SUC_lexicon_tags_47 },
    { 0xbb42103b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf02103e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x47821041, SUC_lexicon_tags_19 },
    { 0xa2421042, SUC_lexicon_tags_7 },
    { 0x1ac21043, SUC_lexicon_tags_64 },
    { 0x70821042, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c21048, SUC_lexicon_tags_15 },
    { 0x66421049, SUC_lexicon_tags_11 },
    { 0x14821049, SUC_lexicon_tags_14 },
    { 0x3ee21049, SUC_lexicon_tags_25 },
    { 0xb1e2104c, SUC_lexicon_tags_29 },
    { 0x69e2104d, SUC_lexicon_tags_55 },
    { 0xc722104e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8621050, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x61a21052, SUC_lexicon_tags_28 },
    { 0x42e21053, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de21057, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3a021059, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e2105c, SUC_lexicon_tags_12 },
    { 0xc7a2105d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f621060, SUC_lexicon_tags_31 },
    { 0xa4e21061, SUC_lexicon_tags_11 },
    { 0x86221061, SUC_lexicon_tags_7 },
    { 0xa6621061, SUC_lexicon_tags_9 },
    { 0xb1021064, SUC_lexicon_tags_43 },
    { 0x5ec21061, SUC_lexicon_tags_15 },
    { 0x60821066, SUC_lexicon_tags_38 },
    { 0xd2421067, SUC_lexicon_tags_45 },
    { 0x9c021062, SUC_lexicon_tags_1 },
    { 0x56221069, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e2106d, SUC_lexicon_tags_2 },
    { 0xa822106e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc382107b, SUC_lexicon_tags_12 },
    { 0xbba2107b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda2107f, SUC_lexicon_tags_43 },
    { 0xd802107f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44021085, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea2108d, SUC_lexicon_tags_17 },
    { 0xf702108e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x78021090, SUC_lexicon_tags_11 },
    { 0x1a421090, SUC_lexicon_tags_11 },
    { 0x8c221090, SUC_lexicon_tags_38 },
    { 0x29e21092, SUC_lexicon_tags_16 },
    { 0x13a21093, SUC_lexicon_tags_25 },
    { 0x9ce21095, SUC_lexicon_tags_25 },
    { 0x8f221095, SUC_lexicon_tags_15 },
    { 0x42e21094, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1922109a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea210a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6210a9, SUC_lexicon_tags_15 },
    { 0xac6210aa, SUC_lexicon_tags_59 },
    { 0xc1a210ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd06210ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5c210af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x414210b1, SUC_lexicon_tags_23 },
    { 0x210210b1, SUC_lexicon_tags_25 },
    { 0x298210b2, SUC_lexicon_tags_49 },
    { 0x61a210b3, SUC_lexicon_tags_12 },
    { 0x41e210b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86210bb, SUC_lexicon_tags_19 },
    { 0x87e210bc, SUC_lexicon_tags_16 },
    { 0xa64210bd, SUC_lexicon_tags_2 },
    { 0xf7a210bc, SUC_lexicon_tags_2 },
    { 0xb3c210bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6210c3, SUC_lexicon_tags_14 },
    { 0x4e8210c3, SUC_lexicon_tags_2 },
    { 0xe210c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690210c9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x966210cc, SUC_lexicon_tags_56 },
    { 0x8c6210cd, SUC_lexicon_tags_9 },
    { 0x548210ce, SUC_lexicon_tags_43 },
    { 0xd4e210ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6210d2, SUC_lexicon_tags_24 },
    { 0x628210d2, SUC_lexicon_tags_17 },
    { 0x65c210d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8210dc, SUC_lexicon_tags_8 },
    { 0xc6c210dd, SUC_lexicon_tags_144 },
    { 0xf98210dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82210e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac210ee, SUC_lexicon_tags_21 },
    { 0x1b0210ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ac210f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24210f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc10210f8, SUC_lexicon_tags_12 },
    { 0xa76210f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48210fc, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x4c6210fe, SUC_lexicon_tags_1 },
    { 0x80210ff, SUC_lexicon_tags_4 },
    { 0x93821100, SUC_lexicon_tags_17 },
    { 0x2a021100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11021104, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b821107, SUC_lexicon_tags_9 },
    { 0x5ca21108, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7082110a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c2110d, SUC_lexicon_tags_47 },
    { 0xcae2110e, SUC_lexicon_tags_82 },
    { 0x4722110f, SUC_lexicon_tags_12 },
    { 0x8f821110, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd821117, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x80421119, SUC_lexicon_tags_37 },
    { 0x77a2111a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca21123, SUC_lexicon_tags_15 },
    { 0xd4821123, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3fe21126, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6821129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b021130, SUC_lexicon_tags_12 },
    { 0xaa421130, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8e021133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9621135, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2a21137, SUC_lexicon_tags_37 },
    { 0x26421138, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdba2113a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee02113e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f221143, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a21146, SUC_lexicon_tags_1 },
    { 0x31c21147, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad62114a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3542114e, SUC_lexicon_tags_8 },
    { 0xee2114f, SUC_lexicon_tags_7 },
    { 0x94221150, SUC_lexicon_tags_13 },
    { 0xc8c21150, SUC_lexicon_tags_2 },
    { 0xd2e21152, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1221156, SUC_lexicon_tags_11 },
    { 0x6cc21157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2842115c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5822115e, SUC_lexicon_tags_3 },
    { 0x1ec2115f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd021163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e42116c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a22116f, SUC_lexicon_tags_61 },
    { 0x50221170, SUC_lexicon_tags_2 },
    { 0xc5e21171, SUC_lexicon_tags_33 },
    { 0x65421172, SUC_lexicon_tags_45 },
    { 0x93e21173, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7542117e, SUC_lexicon_tags_7 },
    { 0xae02117e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e2118b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac22118f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42e21191, SUC_lexicon_tags_7 },
    { 0xd8221191, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea221194, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c021196, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e21199, SUC_lexicon_tags_4 },
    { 0x56e2119a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x42c2119c, SUC_lexicon_tags_17 },
    { 0xae02119d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6211a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc46211a2, SUC_lexicon_tags_39 },
    { 0xa1e211a2, SUC_lexicon_tags_17 },
    { 0x55c211a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a211a8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa88211aa, SUC_lexicon_tags_25 },
    { 0xa88211aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822211af, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2211b2, SUC_lexicon_tags_0 },
    { 0x946211b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdac211b5, SUC_lexicon_tags_11 },
    { 0xeba211b6, SUC_lexicon_tags_30 },
    { 0x23c211b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02211ba, SUC_lexicon_tags_15 },
    { 0xc12211bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b8211bd, SUC_lexicon_tags_84 },
    { 0x3e4211be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd2211c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6211c8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe9c211ca, SUC_lexicon_tags_60 },
    { 0x498211cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c211cf, SUC_lexicon_tags_174 },
    { 0x966211d0, SUC_lexicon_tags_14 },
    { 0x53c211d1, SUC_lexicon_tags_5 },
    { 0xb98211d1, SUC_lexicon_tags_23 },
    { 0xe76211d1, SUC_lexicon_tags_87 },
    { 0x2a2211d4, SUC_lexicon_tags_27 },
    { 0xbe8211d5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0211db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392211e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x682211e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x400211e9, SUC_lexicon_tags_32 },
    { 0xb04211ea, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26211ed, SUC_lexicon_tags_17 },
    { 0x680211ee, SUC_lexicon_tags_33 },
    { 0x47c211ef, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x730211f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0211f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcca211f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe211fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85821202, SUC_lexicon_tags_11 },
    { 0xe221202, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c421208, SUC_lexicon_tags_37 },
    { 0xda221208, SUC_lexicon_tags_32 },
    { 0xff82120a, SUC_lexicon_tags_6 },
    { 0x5782120b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88421210, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x12621213, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdce21215, SUC_lexicon_tags_2 },
    { 0xa2621215, SUC_lexicon_tags_47 },
    { 0x42a21217, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3622121d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58021221, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e21224, SUC_lexicon_tags_29 },
    { 0x42421224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50c21227, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f42122a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302122e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde221232, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c21235, SUC_lexicon_tags_37 },
    { 0x8d821235, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c2123c, SUC_lexicon_tags_0 },
    { 0x6d62123d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef821241, SUC_lexicon_tags_45 },
    { 0xbfa21242, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x98821244, SUC_lexicon_tags_38 },
    { 0xd5e21245, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e22124b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4782124e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62621251, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe21258, SUC_lexicon_tags_42 },
    { 0x6bc21259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3322125c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbac2125e, SUC_lexicon_tags_46 },
    { 0x7c62125e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c421261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a21264, SUC_lexicon_tags_17 },
    { 0xd7e21265, SUC_lexicon_tags_35 },
    { 0x6da21266, SUC_lexicon_tags_60 },
    { 0xf3221267, SUC_lexicon_tags_14 },
    { 0x2a821264, SUC_lexicon_tags_29 },
    { 0xd0621269, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xba82126b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a21271, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33021274, SUC_lexicon_tags_68 },
    { 0x57c21275, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7021278, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc21282, SUC_lexicon_tags_1 },
    { 0xd7621283, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6322128c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfee2128e, SUC_lexicon_tags_15 },
    { 0x9dc2128f, SUC_lexicon_tags_2 },
    { 0x5102128f, SUC_lexicon_tags_17 },
    { 0xeb42128f, SUC_lexicon_tags_23 },
    { 0x1ae21292, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee621297, SUC_lexicon_tags_23 },
    { 0x8a421298, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c212a3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858212a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce212ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcbc212ae, SUC_lexicon_tags_12 },
    { 0x564212ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6212b4, SUC_lexicon_tags_47 },
    { 0x15e212b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfd2212b7, SUC_lexicon_tags_37 },
    { 0xcac212b8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a212bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6212bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6212c3, SUC_lexicon_tags_38 },
    { 0x380212c3, SUC_lexicon_tags_11 },
    { 0x7c212c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516212ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc212cd, SUC_lexicon_tags_15 },
    { 0x9c0212cd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c212d4, SUC_lexicon_tags_25 },
    { 0xb2c212d5, SUC_lexicon_tags_47 },
    { 0xa3a212d6, SUC_lexicon_tags_63 },
    { 0x568212d7, SUC_lexicon_tags_28 },
    { 0x49c212d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442212dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe58212de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86212e2, SUC_lexicon_tags_37 },
    { 0x506212e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c212e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x712212eb, SUC_lexicon_tags_17 },
    { 0x7de212ec, SUC_lexicon_tags_82 },
    { 0x2f4212ed, SUC_lexicon_tags_226 },
    { 0xe84212ed, SUC_lexicon_tags_25 },
    { 0x8f8212ef, SUC_lexicon_tags_174 },
    { 0x93c212f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x388212f3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2212fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed4212fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1e2212ff, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x4c421301, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x26021304, SUC_lexicon_tags_1 },
    { 0x52621305, SUC_lexicon_tags_30 },
    { 0xce021305, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902130c, SUC_lexicon_tags_53 },
    { 0xcae2130d, SUC_lexicon_tags_7 },
    { 0x3e42130e, SUC_lexicon_tags_30 },
    { 0xc0e2130f, SUC_lexicon_tags_37 },
    { 0x2222130f, SUC_lexicon_tags_60 },
    { 0x41c21311, SUC_lexicon_tags_37 },
    { 0x1fa2130f, SUC_lexicon_tags_12 },
    { 0xf7821313, SUC_lexicon_tags_37 },
    { 0xf2c21314, SUC_lexicon_tags_65 },
    { 0x5d021312, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e421318, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc302131b, SUC_lexicon_tags_12 },
    { 0xd662131b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a82131f, SUC_lexicon_tags_10 },
    { 0x57221320, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfc21322, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a21325, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce21329, SUC_lexicon_tags_62 },
    { 0xf0a2132a, SUC_lexicon_tags_7 },
    { 0xd002132b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc022132d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fe2132f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c021333, SUC_lexicon_tags_25 },
    { 0x7fc21334, SUC_lexicon_tags_126 },
    { 0x85421334, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd8221337, SUC_lexicon_tags_49 },
    { 0x4ec21338, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da2133b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b62133f, SUC_lexicon_tags_17 },
    { 0x1da21340, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e421342, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a21345, SUC_lexicon_tags_45 },
    { 0x9e821346, SUC_lexicon_tags_26 },
    { 0x15e21347, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd382134d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a42134f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8021351, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe3621353, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3f421355, SUC_lexicon_tags_17 },
    { 0x49c21355, SUC_lexicon_tags_28 },
    { 0xa2821356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b22135a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6dc2135c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7221362, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68621366, SUC_lexicon_tags_17 },
    { 0x38621367, SUC_lexicon_tags_37 },
    { 0x48a21367, SUC_lexicon_tags_40 },
    { 0xd2421369, SUC_lexicon_tags_7 },
    { 0x5062136a, SUC_lexicon_tags_92 },
    { 0x5802136b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa142136d, SUC_lexicon_tags_6 },
    { 0x1442136e, SUC_lexicon_tags_17 },
    { 0xe5a2136e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a82137d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba21380, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfda21382, SUC_lexicon_tags_47 },
    { 0xa0821383, SUC_lexicon_tags_64 },
    { 0xa8421384, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0221387, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf282138d, SUC_lexicon_tags_27 },
    { 0x7f22138e, SUC_lexicon_tags_1 },
    { 0x6522138d, SUC_lexicon_tags_11 },
    { 0x262138e, SUC_lexicon_tags_17 },
    { 0x42c21390, SUC_lexicon_tags_28 },
    { 0xfb42138d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f421395, SUC_lexicon_tags_17 },
    { 0x97821395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c621398, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c2139b, SUC_lexicon_tags_11 },
    { 0x6d02139c, SUC_lexicon_tags_2 },
    { 0x8de2139d, SUC_lexicon_tags_7 },
    { 0xe342139e, SUC_lexicon_tags_30 },
    { 0x5e22139f, SUC_lexicon_tags_47 },
    { 0x9142139e, SUC_lexicon_tags_63 },
    { 0x3fa2139c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc40213a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8cc213a5, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0x8e4213a7, SUC_lexicon_tags_60 },
    { 0x4fc213a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a213aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x554213ad, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0213b0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4c2213b2, SUC_lexicon_tags_63 },
    { 0x87e213b3, SUC_lexicon_tags_49 },
    { 0x2ee213b2, SUC_lexicon_tags_60 },
    { 0x920213b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a213ba, SUC_lexicon_tags_63 },
    { 0xe16213bb, SUC_lexicon_tags_14 },
    { 0x3f4213bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x688213be, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xb16213c0, SUC_lexicon_tags_17 },
    { 0xdd2213c1, SUC_lexicon_tags_17 },
    { 0x988213c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x344213c5, SUC_lexicon_tags_37 },
    { 0x9bc213c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2c6213c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2213ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x314213cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x992213ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x792213d1, SUC_lexicon_tags_16 },
    { 0x3a8213d2, SUC_lexicon_tags_11 },
    { 0x34a213d2, SUC_lexicon_tags_151 },
    { 0x716213d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x228213d6, SUC_lexicon_tags_1 },
    { 0x5e8213d7, SUC_lexicon_tags_60 },
    { 0xb0a213d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0213dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de213e4, SUC_lexicon_tags_60 },
    { 0xa90213e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16213e8, SUC_lexicon_tags_8 },
    { 0xf54213e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee213ee, SUC_lexicon_tags_92 },
    { 0xc06213ee, SUC_lexicon_tags_11 },
    { 0xbe4213f0, SUC_lexicon_tags_39 },
    { 0x95e213f1, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba213f4, SUC_lexicon_tags_39 },
    { 0xda6213f4, SUC_lexicon_tags_14 },
    { 0x628213f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c213f9, SUC_lexicon_tags_6 },
    { 0x740213fa, SUC_lexicon_tags_7 },
    { 0x842213fa, SUC_lexicon_tags_29 },
    { 0x770213fb, SUC_lexicon_tags_15 },
    { 0x166213fd, SUC_lexicon_tags_29 },
    { 0xa7a213fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c21401, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e821409, SUC_lexicon_tags_47 },
    { 0x9862140a, SUC_lexicon_tags_28 },
    { 0x8d62140b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x502140e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a421411, SUC_lexicon_tags_27 },
    { 0x54a21411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa421416, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd02141b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c62141f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78221425, SUC_lexicon_tags_19 },
    { 0xdce21425, SUC_lexicon_tags_37 },
    { 0x65c21426, SUC_lexicon_tags_1 },
    { 0x88c21425, SUC_lexicon_tags_17 },
    { 0xd8221429, SUC_lexicon_tags_15 },
    { 0xff421429, SUC_lexicon_tags_17 },
    { 0x7ca2142b, SUC_lexicon_tags_69 },
    { 0xdfc2142c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ae2142e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c21433, SUC_lexicon_tags_8 },
    { 0x66421434, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x18e21436, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c821439, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9482143e, SUC_lexicon_tags_29 },
    { 0x1ae2143e, SUC_lexicon_tags_0 },
    { 0x2902143f, SUC_lexicon_tags_32 },
    { 0x67421441, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca221443, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b42144a, SUC_lexicon_tags_23 },
    { 0xacc2144b, SUC_lexicon_tags_60 },
    { 0x9e2144c, SUC_lexicon_tags_11 },
    { 0xb242144d, SUC_lexicon_tags_17 },
    { 0x13c2144d, SUC_lexicon_tags_15 },
    { 0x8ba2144c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7821454, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83421457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c2145a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe342145e, SUC_lexicon_tags_12 },
    { 0xf6a2145e, SUC_lexicon_tags_37 },
    { 0xa442145e, SUC_lexicon_tags_63 },
    { 0xa22145e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a021463, SUC_lexicon_tags_23 },
    { 0x4dc21464, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9a621466, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b22146a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c82146e, SUC_lexicon_tags_17 },
    { 0xe142146f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef621472, SUC_lexicon_tags_37 },
    { 0x62421472, SUC_lexicon_tags_44 },
    { 0x9b621474, SUC_lexicon_tags_1363 },
    { 0x2e21475, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2a621477, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfa821479, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc2a2147b, SUC_lexicon_tags_0 },
    { 0x8782147c, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xd882147f, SUC_lexicon_tags_9 },
    { 0xe5421480, SUC_lexicon_tags_2 },
    { 0xda221481, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2148a, SUC_lexicon_tags_49 },
    { 0x60e2148a, SUC_lexicon_tags_12 },
    { 0x8042148c, SUC_lexicon_tags_82 },
    { 0x20c2148a, SUC_lexicon_tags_9 },
    { 0xf382148e, SUC_lexicon_tags_15 },
    { 0xc082148e, SUC_lexicon_tags_2 },
    { 0xf6a21490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be21496, SUC_lexicon_tags_15 },
    { 0x89621496, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2582149a, SUC_lexicon_tags_1 },
    { 0x2fc2149b, SUC_lexicon_tags_11 },
    { 0xd0c2149a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba214a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb54214a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6214a5, SUC_lexicon_tags_10 },
    { 0x1b4214a6, SUC_lexicon_tags_7 },
    { 0x940214a7, SUC_lexicon_tags_60 },
    { 0xffc214a8, SUC_lexicon_tags_11 },
    { 0xee4214a7, SUC_lexicon_tags_19 },
    { 0x7c6214aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x950214ac, SUC_lexicon_tags_2 },
    { 0x79a214ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x670214b0, SUC_lexicon_tags_23 },
    { 0x992214b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20214b7, SUC_lexicon_tags_10 },
    { 0x90a214b8, SUC_lexicon_tags_15 },
    { 0x74214b8, SUC_lexicon_tags_10 },
    { 0x672214ba, SUC_lexicon_tags_468 },
    { 0xb9e214ba, SUC_lexicon_tags_2 },
    { 0x518214ba, SUC_lexicon_tags_26 },
    { 0xd62214bd, SUC_lexicon_tags_23 },
    { 0xa34214be, SUC_lexicon_tags_2 },
    { 0x92a214bf, SUC_lexicon_tags_37 },
    { 0x89a214c0, SUC_lexicon_tags_29 },
    { 0xaf6214c1, SUC_lexicon_tags_7 },
    { 0x3f0214c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e214c5, SUC_lexicon_tags_2 },
    { 0x6da214c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8214c9, SUC_lexicon_tags_29 },
    { 0x5fa214ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2214cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8214d0, SUC_lexicon_tags_37 },
    { 0x480214d1, SUC_lexicon_tags_1 },
    { 0xfe4214d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe214d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70214e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb26214e4, SUC_lexicon_tags_11 },
    { 0x766214e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8214e8, SUC_lexicon_tags_1 },
    { 0xe94214e9, SUC_lexicon_tags_38 },
    { 0x258214ea, SUC_lexicon_tags_17 },
    { 0x656214eb, SUC_lexicon_tags_27 },
    { 0xaa0214ec, SUC_lexicon_tags_390 },
    { 0xc5e214e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a214f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac214f8, SUC_lexicon_tags_37 },
    { 0x410214f9, SUC_lexicon_tags_25 },
    { 0x7ae214f8, SUC_lexicon_tags_6 },
    { 0x50c214fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca214fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4b621500, SUC_lexicon_tags_11 },
    { 0xa1c21501, SUC_lexicon_tags_27 },
    { 0x3e821501, SUC_lexicon_tags_2 },
    { 0x5d821503, SUC_lexicon_tags_61 },
    { 0x37e21501, SUC_lexicon_tags_26 },
    { 0x84821502, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe422150a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1662150d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9221517, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e22151b, SUC_lexicon_tags_31 },
    { 0x19e2151b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de21520, SUC_lexicon_tags_47 },
    { 0xc3221520, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c21525, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62821527, SUC_lexicon_tags_62 },
    { 0xc0821528, SUC_lexicon_tags_43 },
    { 0x89e21529, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb62152b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f621530, SUC_lexicon_tags_1 },
    { 0x3b221531, SUC_lexicon_tags_26 },
    { 0xf2e21532, SUC_lexicon_tags_2 },
    { 0x3e021532, SUC_lexicon_tags_1 },
    { 0x7ae21533, SUC_lexicon_tags_17 },
    { 0x3d021531, SUC_lexicon_tags_4 },
    { 0x66c21534, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc522153d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a221541, SUC_lexicon_tags_15 },
    { 0x50421542, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec821545, SUC_lexicon_tags_3 },
    { 0xb1821546, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d021549, SUC_lexicon_tags_27 },
    { 0x69821549, SUC_lexicon_tags_10 },
    { 0x8962154b, SUC_lexicon_tags_37 },
    { 0xda62154c, SUC_lexicon_tags_28 },
    { 0x69a2154b, SUC_lexicon_tags_31 },
    { 0x7c02154a, SUC_lexicon_tags_25 },
    { 0x3ae2154f, SUC_lexicon_tags_15 },
    { 0xef82154d, SUC_lexicon_tags_1 },
    { 0x29021550, SUC_lexicon_tags_14 },
    { 0xc4021552, SUC_lexicon_tags_38 },
    { 0x53e21553, SUC_lexicon_tags_1 },
    { 0x77621554, SUC_lexicon_tags_27 },
    { 0x38a21555, SUC_lexicon_tags_7 },
    { 0xe9821556, SUC_lexicon_tags_47 },
    { 0x87821553, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f82155c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd821562, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4621564, SUC_lexicon_tags_4 },
    { 0x86621564, SUC_lexicon_tags_32 },
    { 0x37621566, SUC_lexicon_tags_8 },
    { 0xdec21564, SUC_lexicon_tags_1 },
    { 0x44621564, SUC_lexicon_tags_16 },
    { 0x1f221569, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde2156d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c421578, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5d82157a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb582157d, SUC_lexicon_tags_45 },
    { 0x9102157e, SUC_lexicon_tags_30 },
    { 0xab42157f, SUC_lexicon_tags_84 },
    { 0x3f221580, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8021585, SUC_lexicon_tags_33 },
    { 0xf3821586, SUC_lexicon_tags_65 },
    { 0xbaa21585, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x70021589, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa2158d, SUC_lexicon_tags_23 },
    { 0x3742158e, SUC_lexicon_tags_31 },
    { 0x9162158e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1dc21591, SUC_lexicon_tags_17 },
    { 0x48421592, SUC_lexicon_tags_37 },
    { 0xd7221593, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda21596, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc9621598, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e2159b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xfde2159d, SUC_lexicon_tags_15 },
    { 0xbd22159d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x926215a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4215a4, SUC_lexicon_tags_12 },
    { 0x794215a5, SUC_lexicon_tags_35 },
    { 0xb10215a6, SUC_lexicon_tags_2 },
    { 0x6ec215a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66215aa, SUC_lexicon_tags_11 },
    { 0x3c215ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894215b3, SUC_lexicon_tags_27 },
    { 0x25e215b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e215b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3aa215b9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x122215bc, SUC_lexicon_tags_22 },
    { 0x3c8215bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe18215bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6bc215c1, SUC_lexicon_tags_83 },
    { 0xe5a215c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x108215c4, SUC_lexicon_tags_10 },
    { 0x8a2215c5, SUC_lexicon_tags_63 },
    { 0xa7a215c6, SUC_lexicon_tags_38 },
    { 0x14c215c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x614215ca, SUC_lexicon_tags_23 },
    { 0xe2e215cb, SUC_lexicon_tags_35 },
    { 0x1f6215cc, SUC_lexicon_tags_14 },
    { 0x8c4215cd, SUC_lexicon_tags_0 },
    { 0xec6215ce, SUC_lexicon_tags_11 },
    { 0x558215ce, SUC_lexicon_tags_11 },
    { 0xef0215d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a215d3, SUC_lexicon_tags_61 },
    { 0x918215d4, SUC_lexicon_tags_11 },
    { 0xd70215d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4c215d7, SUC_lexicon_tags_12 },
    { 0xf14215d8, SUC_lexicon_tags_19 },
    { 0x27c215d9, SUC_lexicon_tags_11 },
    { 0xc76215da, SUC_lexicon_tags_6 },
    { 0xdb6215db, SUC_lexicon_tags_19 },
    { 0x88c215dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6215e0, SUC_lexicon_tags_33 },
    { 0xa68215e1, SUC_lexicon_tags_7 },
    { 0x62215e1, SUC_lexicon_tags_7 },
    { 0x206215e3, SUC_lexicon_tags_26 },
    { 0x86c215e4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c215ec, SUC_lexicon_tags_12 },
    { 0x3c8215ed, SUC_lexicon_tags_8 },
    { 0x5cc215ee, SUC_lexicon_tags_1 },
    { 0x1b2215ec, SUC_lexicon_tags_45 },
    { 0xb2e215f0, SUC_lexicon_tags_17 },
    { 0x5a4215f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ce215f3, SUC_lexicon_tags_27 },
    { 0xee6215f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208215fb, SUC_lexicon_tags_43 },
    { 0x14c215fc, SUC_lexicon_tags_14 },
    { 0x1b0215fb, SUC_lexicon_tags_15 },
    { 0xf8a215fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a421601, SUC_lexicon_tags_2 },
    { 0xdae21602, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d421604, SUC_lexicon_tags_10 },
    { 0xbf221604, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec2160b, SUC_lexicon_tags_21 },
    { 0x67e2160c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce21611, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a21617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7102161a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6282161c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e2161f, SUC_lexicon_tags_45 },
    { 0x96021620, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a21623, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a21628, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5322162a, SUC_lexicon_tags_12 },
    { 0x7962162b, SUC_lexicon_tags_13 },
    { 0xf302162c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x942162f, SUC_lexicon_tags_12 },
    { 0x36621630, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a21634, SUC_lexicon_tags_25 },
    { 0x87821635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e21638, SUC_lexicon_tags_37 },
    { 0xd7021639, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3242163b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec21643, SUC_lexicon_tags_44 },
    { 0xc9821643, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb42164b, SUC_lexicon_tags_19 },
    { 0x1e62164b, SUC_lexicon_tags_26 },
    { 0x9722164d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc802164f, SUC_lexicon_tags_56 },
    { 0xf7e21650, SUC_lexicon_tags_37 },
    { 0xc1e2164f, SUC_lexicon_tags_42 },
    { 0xc6021652, SUC_lexicon_tags_19 },
    { 0xe0c21652, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac21656, SUC_lexicon_tags_12 },
    { 0x79621657, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e2165d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb421660, SUC_lexicon_tags_17 },
    { 0x57221660, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2221667, SUC_lexicon_tags_16 },
    { 0x31621667, SUC_lexicon_tags_25 },
    { 0x6c621668, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3ce2166b, SUC_lexicon_tags_17 },
    { 0xb942166b, SUC_lexicon_tags_1 },
    { 0x9542166d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c21671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a21679, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b42167d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e21684, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2168b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x82a2168d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac821691, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecc21693, SUC_lexicon_tags_0 },
    { 0xd5821694, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28221698, SUC_lexicon_tags_31 },
    { 0xff021698, SUC_lexicon_tags_37 },
    { 0x5b62169a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4902169c, SUC_lexicon_tags_7 },
    { 0x5ae2169d, SUC_lexicon_tags_37 },
    { 0xc1e2169d, SUC_lexicon_tags_37 },
    { 0xa382169f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x568216a2, SUC_lexicon_tags_63 },
    { 0x874216a3, SUC_lexicon_tags_26 },
    { 0x68c216a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c216a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c216ae, SUC_lexicon_tags_57 },
    { 0x4b6216af, SUC_lexicon_tags_15 },
    { 0x350216b0, SUC_lexicon_tags_65 },
    { 0xcc6216b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92216b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244216c3, SUC_lexicon_tags_8 },
    { 0x8e0216c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a216c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac216cb, SUC_lexicon_tags_104 },
    { 0xcde216cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24216d0, SUC_lexicon_tags_37 },
    { 0x55a216d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e216d5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe216e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9ee216e4, SUC_lexicon_tags_2 },
    { 0x308216e4, SUC_lexicon_tags_29 },
    { 0xed2216e6, SUC_lexicon_tags_18 },
    { 0xd10216e6, SUC_lexicon_tags_32 },
    { 0x518216e8, SUC_lexicon_tags_45 },
    { 0x2c2216e9, SUC_lexicon_tags_15 },
    { 0x6ac216ea, SUC_lexicon_tags_47 },
    { 0x654216e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81e216ed, SUC_lexicon_tags_25 },
    { 0x924216ed, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x49c216f0, SUC_lexicon_tags_23 },
    { 0x438216f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0216f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1dc216f6, SUC_lexicon_tags_2 },
    { 0x1d4216f6, SUC_lexicon_tags_17 },
    { 0xd74216f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a216fe, SUC_lexicon_tags_103 },
    { 0xc7e216ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6421704, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7042170a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc462170e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc21711, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc621715, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9621719, SUC_lexicon_tags_3 },
    { 0x35a21719, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb9a2171c, SUC_lexicon_tags_7 },
    { 0xf522171d, SUC_lexicon_tags_77 },
    { 0xce42171e, SUC_lexicon_tags_55 },
    { 0x1b22171f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8021722, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6ae21724, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5621728, SUC_lexicon_tags_63 },
    { 0x30a21729, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8002172c, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x9082172f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e21732, SUC_lexicon_tags_65 },
    { 0x93e21733, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c21738, SUC_lexicon_tags_17 },
    { 0x1c621739, SUC_lexicon_tags_1 },
    { 0x48a21738, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762173e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdba21740, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e21746, SUC_lexicon_tags_34 },
    { 0xf1621747, SUC_lexicon_tags_17 },
    { 0xbc021747, SUC_lexicon_tags_15 },
    { 0xc4221749, SUC_lexicon_tags_48 },
    { 0x17421748, SUC_lexicon_tags_12 },
    { 0xdde2174b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a22174f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa21752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6421756, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf021758, SUC_lexicon_tags_43 },
    { 0x30821759, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c02175b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5742175e, SUC_lexicon_tags_11 },
    { 0x102175f, SUC_lexicon_tags_37 },
    { 0x1c021760, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15821764, SUC_lexicon_tags_29 },
    { 0x60421765, SUC_lexicon_tags_2 },
    { 0x54421765, SUC_lexicon_tags_28 },
    { 0x9a021766, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea2176d, SUC_lexicon_tags_7 },
    { 0xb9a2176e, SUC_lexicon_tags_29 },
    { 0x39c2176f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee21772, SUC_lexicon_tags_6 },
    { 0xafe21773, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87421778, SUC_lexicon_tags_11 },
    { 0xd3221779, SUC_lexicon_tags_15 },
    { 0x1e21779, SUC_lexicon_tags_7 },
    { 0x7ce2177b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c21781, SUC_lexicon_tags_15 },
    { 0xcde21782, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x86a21784, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb502178a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21021798, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9802179e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcd8217a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84a217a2, SUC_lexicon_tags_14 },
    { 0xf00217a3, SUC_lexicon_tags_7 },
    { 0x5f6217a4, SUC_lexicon_tags_39 },
    { 0x746217a3, SUC_lexicon_tags_21 },
    { 0xdfc217a6, SUC_lexicon_tags_49 },
    { 0xaf4217a7, SUC_lexicon_tags_31 },
    { 0x6e2217a6, SUC_lexicon_tags_7 },
    { 0xdd8217a9, SUC_lexicon_tags_27 },
    { 0xa10217aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc48217ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8217af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4217bd, SUC_lexicon_tags_26 },
    { 0x566217be, SUC_lexicon_tags_30 },
    { 0xc2a217bf, SUC_lexicon_tags_35 },
    { 0x53c217be, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a217c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x738217c6, SUC_lexicon_tags_129 },
    { 0xe9e217c6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c217cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88217d1, SUC_lexicon_tags_37 },
    { 0x258217d1, SUC_lexicon_tags_43 },
    { 0x424217d1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4217d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb12217da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd72217dc, SUC_lexicon_tags_8 },
    { 0x130217dc, SUC_lexicon_tags_1 },
    { 0xc9c217dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a217e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x438217e5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x430217e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34217ec, SUC_lexicon_tags_37 },
    { 0x954217ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0217f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660217f5, SUC_lexicon_tags_37 },
    { 0x51a217f6, SUC_lexicon_tags_2 },
    { 0xf24217f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x74e217f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6ee217fb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c21800, SUC_lexicon_tags_22 },
    { 0xb0621801, SUC_lexicon_tags_11 },
    { 0xf8a21801, SUC_lexicon_tags_14 },
    { 0x92a21803, SUC_lexicon_tags_12 },
    { 0xca621803, SUC_lexicon_tags_1 },
    { 0xc1021802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8021807, SUC_lexicon_tags_18 },
    { 0xe9c21808, SUC_lexicon_tags_38 },
    { 0x74821809, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe002180d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9ea2180f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1421812, SUC_lexicon_tags_2 },
    { 0xf9821813, SUC_lexicon_tags_17 },
    { 0x40e21813, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d02181b, SUC_lexicon_tags_1 },
    { 0xa4a2181c, SUC_lexicon_tags_25 },
    { 0xfde2181d, SUC_lexicon_tags_29 },
    { 0xc4a2181e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa821828, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a2182c, SUC_lexicon_tags_136 },
    { 0xeb02182d, SUC_lexicon_tags_108 },
    { 0x4102182d, SUC_lexicon_tags_46 },
    { 0x7082182f, SUC_lexicon_tags_18 },
    { 0xc82182d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9221832, SUC_lexicon_tags_33 },
    { 0xc8621833, SUC_lexicon_tags_37 },
    { 0x80e21834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c21838, SUC_lexicon_tags_17 },
    { 0xcce21839, SUC_lexicon_tags_4 },
    { 0x36c2183a, SUC_lexicon_tags_45 },
    { 0x48a2183b, SUC_lexicon_tags_17 },
    { 0xaf82183a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3722183e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc7421840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a21844, SUC_lexicon_tags_0 },
    { 0x72421845, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x22221847, SUC_lexicon_tags_25 },
    { 0xce21848, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2184c, SUC_lexicon_tags_7 },
    { 0xaa82184c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75021852, SUC_lexicon_tags_63 },
    { 0x2f021853, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a221855, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x17221857, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e2185c, SUC_lexicon_tags_33 },
    { 0x2d02185d, SUC_lexicon_tags_47 },
    { 0x2cc2185c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c21866, SUC_lexicon_tags_49 },
    { 0x49021867, SUC_lexicon_tags_14 },
    { 0xa2421866, SUC_lexicon_tags_11 },
    { 0x23421869, SUC_lexicon_tags_43 },
    { 0x40621869, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1522186d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e21876, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a2187d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf621884, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac021886, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x65421889, SUC_lexicon_tags_17 },
    { 0x5242188a, SUC_lexicon_tags_15 },
    { 0x2702188b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c2188f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70421898, SUC_lexicon_tags_11 },
    { 0xc0c21899, SUC_lexicon_tags_15 },
    { 0xb3421899, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x67e2189c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e218a8, SUC_lexicon_tags_23 },
    { 0x104218a9, SUC_lexicon_tags_37 },
    { 0x9d4218aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xac2218ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0218b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be218b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a218bc, SUC_lexicon_tags_68 },
    { 0x2a218bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85e218bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4218c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xea218c5, SUC_lexicon_tags_17 },
    { 0xfe4218c5, SUC_lexicon_tags_37 },
    { 0xd3c218c7, SUC_lexicon_tags_39 },
    { 0x694218c7, SUC_lexicon_tags_12 },
    { 0xc4a218c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67a218cb, SUC_lexicon_tags_4 },
    { 0x3e0218cc, SUC_lexicon_tags_7 },
    { 0x23e218cd, SUC_lexicon_tags_25 },
    { 0x312218ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a218d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2218d7, SUC_lexicon_tags_38 },
    { 0x6aa218d8, SUC_lexicon_tags_25 },
    { 0x1cc218d8, SUC_lexicon_tags_25 },
    { 0x4be218d9, SUC_lexicon_tags_12 },
    { 0x76218db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x96e218dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda218e3, SUC_lexicon_tags_1 },
    { 0xff2218e3, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8218e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x786218e9, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4218ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2a218f0, SUC_lexicon_tags_63 },
    { 0x76e218f1, SUC_lexicon_tags_30 },
    { 0x3ae218f2, SUC_lexicon_tags_37 },
    { 0x1cc218f1, SUC_lexicon_tags_25 },
    { 0x7f0218f4, SUC_lexicon_tags_43 },
    { 0xe14218f4, SUC_lexicon_tags_37 },
    { 0xe0c218f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x912218f8, SUC_lexicon_tags_7 },
    { 0x7e4218f9, SUC_lexicon_tags_32 },
    { 0x382218fa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4c4218fc, SUC_lexicon_tags_60 },
    { 0x5aa218fc, SUC_lexicon_tags_0 },
    { 0x9d4218fe, SUC_lexicon_tags_21 },
    { 0xebc218ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe21905, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7d821907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4442190a, SUC_lexicon_tags_23 },
    { 0x60e2190b, SUC_lexicon_tags_23 },
    { 0xaf62190c, SUC_lexicon_tags_218 },
    { 0x1422190d, SUC_lexicon_tags_32 },
    { 0xf2e2190b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de21913, SUC_lexicon_tags_46 },
    { 0x24221914, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7562191c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c2191f, SUC_lexicon_tags_39 },
    { 0x5722191f, SUC_lexicon_tags_37 },
    { 0x23821921, SUC_lexicon_tags_38 },
    { 0x1cc21922, SUC_lexicon_tags_37 },
    { 0x7ce21923, SUC_lexicon_tags_12 },
    { 0x77221921, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x61021926, SUC_lexicon_tags_16 },
    { 0xc4021927, SUC_lexicon_tags_2 },
    { 0x6cc21928, SUC_lexicon_tags_49 },
    { 0xfe421929, SUC_lexicon_tags_16 },
    { 0x9aa21929, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c2192e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc421933, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39821937, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e2193d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b221940, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec21946, SUC_lexicon_tags_45 },
    { 0x6221947, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfe621949, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb762194d, SUC_lexicon_tags_12 },
    { 0x82c2194d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1021951, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x39021953, SUC_lexicon_tags_6 },
    { 0x63221954, SUC_lexicon_tags_15 },
    { 0xdba21955, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4142195a, SUC_lexicon_tags_43 },
    { 0xcb82195b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a821961, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac21965, SUC_lexicon_tags_43 },
    { 0x51c21966, SUC_lexicon_tags_37 },
    { 0x4f821967, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb922196a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a2196e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe21972, SUC_lexicon_tags_30 },
    { 0x5d821972, SUC_lexicon_tags_18 },
    { 0xe1621972, SUC_lexicon_tags_60 },
    { 0x3da21975, SUC_lexicon_tags_12 },
    { 0xd7621976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe2197a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x19e2197c, SUC_lexicon_tags_15 },
    { 0x9e62197c, SUC_lexicon_tags_6 },
    { 0x8f02197e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63421982, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5221985, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b421989, SUC_lexicon_tags_29 },
    { 0xba62198a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9421990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68821992, SUC_lexicon_tags_37 },
    { 0xfde21993, SUC_lexicon_tags_43 },
    { 0xd621994, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa6021996, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e21998, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e219a0, SUC_lexicon_tags_25 },
    { 0xd02219a1, SUC_lexicon_tags_63 },
    { 0x81a219a2, SUC_lexicon_tags_30 },
    { 0xd0e219a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34219a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6219bb, SUC_lexicon_tags_14 },
    { 0x44c219bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x438219bf, SUC_lexicon_tags_37 },
    { 0x324219c0, SUC_lexicon_tags_55 },
    { 0xae0219c1, SUC_lexicon_tags_25 },
    { 0xdec219c2, SUC_lexicon_tags_11 },
    { 0xf8219c0, SUC_lexicon_tags_10 },
    { 0xe94219bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c6219c6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e219cc, SUC_lexicon_tags_37 },
    { 0xb0a219cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502219d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4219d5, SUC_lexicon_tags_30 },
    { 0xc24219d6, SUC_lexicon_tags_37 },
    { 0xb56219d7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc219db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e219de, SUC_lexicon_tags_19 },
    { 0x512219de, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36219e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3a219e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9da219e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe219e9, SUC_lexicon_tags_55 },
    { 0x608219ea, SUC_lexicon_tags_45 },
    { 0x3f0219eb, SUC_lexicon_tags_21 },
    { 0x1ea219ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xec8219ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c219fa, SUC_lexicon_tags_14 },
    { 0xedc219fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ac219fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec21a01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31421a07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf421a09, SUC_lexicon_tags_11 },
    { 0x4ee21a0a, SUC_lexicon_tags_25 },
    { 0xc6821a0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c21a11, SUC_lexicon_tags_17 },
    { 0xa4e21a11, SUC_lexicon_tags_68 },
    { 0xa9e21a13, SUC_lexicon_tags_45 },
    { 0xe6221a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce21a1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x821a1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77421a1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ae21a21, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b021a26, SUC_lexicon_tags_32 },
    { 0x29e21a26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30421a29, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x68c21a2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34621a2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0221a33, SUC_lexicon_tags_16 },
    { 0x24021a33, SUC_lexicon_tags_7 },
    { 0x59621a35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f621a3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a21a3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef221a40, SUC_lexicon_tags_62 },
    { 0x2c21a41, SUC_lexicon_tags_1 },
    { 0x8aa21a42, SUC_lexicon_tags_25 },
    { 0xeda21a43, SUC_lexicon_tags_61 },
    { 0x40221a44, SUC_lexicon_tags_1 },
    { 0x2221a41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf621a47, SUC_lexicon_tags_1 },
    { 0xeda21a47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76021a4c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x31621a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a21a55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf221a59, SUC_lexicon_tags_12 },
    { 0x61a21a59, SUC_lexicon_tags_11 },
    { 0x48c21a59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57621a5e, SUC_lexicon_tags_28 },
    { 0x4d421a5f, SUC_lexicon_tags_37 },
    { 0x33421a5e, SUC_lexicon_tags_37 },
    { 0x5f221a60, SUC_lexicon_tags_14 },
    { 0x9421a62, SUC_lexicon_tags_17 },
    { 0x4ac21a60, SUC_lexicon_tags_37 },
    { 0x69621a61, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c21a6a, SUC_lexicon_tags_37 },
    { 0x26e21a6b, SUC_lexicon_tags_12 },
    { 0xcb221a6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f421a6e, SUC_lexicon_tags_45 },
    { 0x61421a6e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7bc21a71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac21a76, SUC_lexicon_tags_49 },
    { 0x7e21a77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72621a7c, SUC_lexicon_tags_63 },
    { 0x92c21a7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83621a7f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7221a82, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0621a8a, SUC_lexicon_tags_77 },
    { 0x56221a8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8021a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2c21a94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa9221a96, SUC_lexicon_tags_17 },
    { 0xeac21a97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c21a9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0021aa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc021aa2, SUC_lexicon_tags_60 },
    { 0x4821aa2, SUC_lexicon_tags_37 },
    { 0x6d221aa4, SUC_lexicon_tags_27 },
    { 0xac21aa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62021aa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee21aae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3221ab6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73421abb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe21abd, SUC_lexicon_tags_27 },
    { 0xcbe21abe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d621ac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbae21ac3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73421acc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5421ad2, SUC_lexicon_tags_40 },
    { 0xcfe21ad3, SUC_lexicon_tags_2 },
    { 0x4d021ad4, SUC_lexicon_tags_11 },
    { 0xf7e21ad5, SUC_lexicon_tags_17 },
    { 0x82821ad4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c821ad9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd421add, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb821ae0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79621ae2, SUC_lexicon_tags_11 },
    { 0x3d421ae2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x68e21ae5, SUC_lexicon_tags_201 },
    { 0x1d821ae6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x65821ae9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79421aed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde221aef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde221af4, SUC_lexicon_tags_106 },
    { 0x93c21af5, SUC_lexicon_tags_70 },
    { 0x47e21af6, SUC_lexicon_tags_85 },
    { 0xdbe21af7, SUC_lexicon_tags_48 },
    { 0x8d821af8, SUC_lexicon_tags_34 },
    { 0x9c21af9, SUC_lexicon_tags_75 },
    { 0x69421af9, SUC_lexicon_tags_49 },
    { 0xe2221af9, SUC_lexicon_tags_37 },
    { 0x86c21afb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69821afe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda21b05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8821b0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3621b0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8821b10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89021b12, SUC_lexicon_tags_48 },
    { 0x62a21b13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab621b17, SUC_lexicon_tags_27 },
    { 0x1f421b17, SUC_lexicon_tags_103 },
    { 0x5f421b19, SUC_lexicon_tags_136 },
    { 0x25621b19, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0821b1e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e821b21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa21b27, SUC_lexicon_tags_37 },
    { 0x21821b28, SUC_lexicon_tags_11 },
    { 0x27621b29, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x83c21b2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35621b30, SUC_lexicon_tags_25 },
    { 0x1b221b31, SUC_lexicon_tags_2 },
    { 0xe2a21b32, SUC_lexicon_tags_14 },
    { 0x19c21b33, SUC_lexicon_tags_25 },
    { 0x36e21b34, SUC_lexicon_tags_21 },
    { 0x7ba21b35, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27421b39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0221b3b, SUC_lexicon_tags_12 },
    { 0xde421b3c, SUC_lexicon_tags_15 },
    { 0xa1621b3d, SUC_lexicon_tags_14 },
    { 0x37421b3d, SUC_lexicon_tags_15 },
    { 0xac821b3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49c21b41, SUC_lexicon_tags_37 },
    { 0x9b421b42, SUC_lexicon_tags_45 },
    { 0x8bc21b43, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58421b48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb621b4e, SUC_lexicon_tags_17 },
    { 0xb8421b4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20e21b51, SUC_lexicon_tags_138 },
    { 0x1ac21b52, SUC_lexicon_tags_25 },
    { 0xb621b53, SUC_lexicon_tags_27 },
    { 0x52421b52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3621b60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde21b63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27021b6b, SUC_lexicon_tags_1 },
    { 0xa0021b6b, SUC_lexicon_tags_7 },
    { 0x7ac21b6b, SUC_lexicon_tags_8 },
    { 0xdb621b6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4021b71, SUC_lexicon_tags_37 },
    { 0x75021b72, SUC_lexicon_tags_15 },
    { 0x59421b71, SUC_lexicon_tags_17 },
    { 0xd3e21b71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a21b77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3621b7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6221b7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75621b80, SUC_lexicon_tags_60 },
    { 0xec221b80, SUC_lexicon_tags_1 },
    { 0xa821b82, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64821b87, SUC_lexicon_tags_26 },
    { 0x55c21b88, SUC_lexicon_tags_6 },
    { 0x59c21b89, SUC_lexicon_tags_6 },
    { 0xf7421b89, SUC_lexicon_tags_0 },
    { 0xc7c21b8a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a21b8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61421b94, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x11a21b96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7821b9a, SUC_lexicon_tags_68 },
    { 0x85421b9b, SUC_lexicon_tags_25 },
    { 0x78e21b9c, SUC_lexicon_tags_12 },
    { 0x63c21b9b, SUC_lexicon_tags_33 },
    { 0x5f221b9e, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xbc021ba0, SUC_lexicon_tags_27 },
    { 0xec621ba1, SUC_lexicon_tags_48 },
    { 0xc0c21ba1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea21ba5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c221ba8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd421baf, SUC_lexicon_tags_25 },
    { 0xc7e21bb0, SUC_lexicon_tags_2 },
    { 0x9fa21bb1, SUC_lexicon_tags_17 },
    { 0x64a21bb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2421bb4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c21bbc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a21bc0, SUC_lexicon_tags_11 },
    { 0xd1421bc0, SUC_lexicon_tags_25 },
    { 0x9e21bc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95621bc5, SUC_lexicon_tags_25 },
    { 0xb4c21bc5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3621bc9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a21bcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c21bd1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf221bd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7421bd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc821bd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4c21bdb, SUC_lexicon_tags_37 },
    { 0xd421bdc, SUC_lexicon_tags_17 },
    { 0x36e21bdd, SUC_lexicon_tags_23 },
    { 0xad221bdc, SUC_lexicon_tags_26 },
    { 0x26e21bdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd821be2, SUC_lexicon_tags_25 },
    { 0x6b221be3, SUC_lexicon_tags_6 },
    { 0x52421be4, SUC_lexicon_tags_12 },
    { 0x30221be3, SUC_lexicon_tags_14 },
    { 0xec21be6, SUC_lexicon_tags_38 },
    { 0x1621be7, SUC_lexicon_tags_66 },
    { 0xbf421be8, SUC_lexicon_tags_19 },
    { 0xe9a21be6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a21bed, SUC_lexicon_tags_26 },
    { 0x8ca21bee, SUC_lexicon_tags_17 },
    { 0x2b621bef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2c21bf1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47821bf5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da21bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa421bfc, SUC_lexicon_tags_29 },
    { 0xd4c21bfd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0021c00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfbc21c02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85221c04, SUC_lexicon_tags_25 },
    { 0x6ea21c05, SUC_lexicon_tags_0 },
    { 0x58c21c04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49221c08, SUC_lexicon_tags_30 },
    { 0x74a21c09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a21c10, SUC_lexicon_tags_12 },
    { 0xd9c21c11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7621c14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23c21c16, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xee621c18, SUC_lexicon_tags_49 },
    { 0xc3621c19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc0e21c1b, SUC_lexicon_tags_81 },
    { 0x1aa21c1c, SUC_lexicon_tags_16 },
    { 0xaae21c1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae221c1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c21c26, SUC_lexicon_tags_30 },
    { 0xf2821c27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c21c2c, SUC_lexicon_tags_79 },
    { 0x33e21c2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9621c30, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f821c33, SUC_lexicon_tags_19 },
    { 0x44c21c34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c421c38, SUC_lexicon_tags_17 },
    { 0xff221c38, SUC_lexicon_tags_37 },
    { 0x2b221c39, SUC_lexicon_tags_0 },
    { 0xe7621c3b, SUC_lexicon_tags_6 },
    { 0xa0821c3c, SUC_lexicon_tags_38 },
    { 0xc3421c3c, SUC_lexicon_tags_37 },
    { 0x69621c3d, SUC_lexicon_tags_31 },
    { 0x1c21c3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x79021c42, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c21c45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec21c4b, SUC_lexicon_tags_2 },
    { 0xff421c4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e21c52, SUC_lexicon_tags_9 },
    { 0x1ba21c53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x78821c56, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c21c59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee821c5d, SUC_lexicon_tags_12 },
    { 0x32a21c5e, SUC_lexicon_tags_28 },
    { 0xdfe21c5f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5e621c61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63421c63, SUC_lexicon_tags_37 },
    { 0xa6621c64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81821c66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12821c6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c21c74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3221c79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd621c7b, SUC_lexicon_tags_12 },
    { 0xff621c7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba21c7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a21c82, SUC_lexicon_tags_8 },
    { 0xaec21c83, SUC_lexicon_tags_19 },
    { 0x8a221c84, SUC_lexicon_tags_17 },
    { 0x95621c85, SUC_lexicon_tags_28 },
    { 0x73221c86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c21c8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c021c8e, SUC_lexicon_tags_31 },
    { 0x8e21c8f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20621c96, SUC_lexicon_tags_15 },
    { 0xd6021c97, SUC_lexicon_tags_31 },
    { 0x7a021c96, SUC_lexicon_tags_1 },
    { 0x29621c99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83621c9b, SUC_lexicon_tags_46 },
    { 0xcac21c9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10221c9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce621ca5, SUC_lexicon_tags_37 },
    { 0x44421ca5, SUC_lexicon_tags_25 },
    { 0x13821ca7, SUC_lexicon_tags_15 },
    { 0xc5421ca5, SUC_lexicon_tags_19 },
    { 0x1f821ca7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69c21cab, SUC_lexicon_tags_9 },
    { 0xf5c21cac, SUC_lexicon_tags_19 },
    { 0xd3021cad, SUC_lexicon_tags_16 },
    { 0x1b021cac, SUC_lexicon_tags_33 },
    { 0xd6e21caf, SUC_lexicon_tags_17 },
    { 0xcf621caf, SUC_lexicon_tags_29 },
    { 0xef221cac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44421cb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b621cb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e21cbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4621cbf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e821cc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3621cc6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a21ccd, SUC_lexicon_tags_9 },
    { 0x8ac21ccd, SUC_lexicon_tags_37 },
    { 0x6d821ccf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c21cd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81421cd8, SUC_lexicon_tags_30 },
    { 0xcce21cd9, SUC_lexicon_tags_33 },
    { 0xa2a21cd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0021cdc, SUC_lexicon_tags_17 },
    { 0x3b421cdd, SUC_lexicon_tags_25 },
    { 0xdac21cde, SUC_lexicon_tags_17 },
    { 0xa4e21cdf, SUC_lexicon_tags_9 },
    { 0x50021cdf, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2621ce4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d021ce6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ba21ce8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b421cea, SUC_lexicon_tags_909 },
    { 0, NULL },
    { 0x34c21cec, SUC_lexicon_tags_27 },
    { 0xa4421ced, SUC_lexicon_tags_20 },
    { 0x5ea21cec, SUC_lexicon_tags_25 },
    { 0xaee21cef, SUC_lexicon_tags_56 },
    { 0xe3021cf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3dc21cf2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b621cf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8221cfd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2221d01, SUC_lexicon_tags_28 },
    { 0x3ca21d01, SUC_lexicon_tags_17 },
    { 0x61021d03, SUC_lexicon_tags_37 },
    { 0x6421d04, SUC_lexicon_tags_17 },
    { 0x96a21d04, SUC_lexicon_tags_63 },
    { 0xee421d06, SUC_lexicon_tags_33 },
    { 0xe0821d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f821d0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1be21d0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25421d11, SUC_lexicon_tags_25 },
    { 0x1a621d12, SUC_lexicon_tags_11 },
    { 0xd3421d11, SUC_lexicon_tags_1 },
    { 0x27621d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c21d17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e021d19, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x11621d1c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3e021d1e, SUC_lexicon_tags_60 },
    { 0x12221d1f, SUC_lexicon_tags_15 },
    { 0x8d821d20, SUC_lexicon_tags_33 },
    { 0x77a21d21, SUC_lexicon_tags_25 },
    { 0xf2a21d20, SUC_lexicon_tags_37 },
    { 0xc1821d23, SUC_lexicon_tags_26 },
    { 0x70621d1f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e21d27, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be21d2c, SUC_lexicon_tags_43 },
    { 0x3a021d2d, SUC_lexicon_tags_29 },
    { 0x72821d2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9621d35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b621d3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d821d45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7421d47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee421d4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c21d4d, SUC_lexicon_tags_68 },
    { 0x14c21d4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef021d52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ca21d54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e21d5b, SUC_lexicon_tags_65 },
    { 0x9c621d5b, SUC_lexicon_tags_23 },
    { 0xc821d5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f621d5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x35821d62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c821d68, SUC_lexicon_tags_55 },
    { 0x8c821d68, SUC_lexicon_tags_19 },
    { 0x77621d6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c21d71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f421d76, SUC_lexicon_tags_47 },
    { 0xe9a21d76, SUC_lexicon_tags_39 },
    { 0xeb821d78, SUC_lexicon_tags_17 },
    { 0xbd221d79, SUC_lexicon_tags_35 },
    { 0x9bc21d7a, SUC_lexicon_tags_38 },
    { 0xe021d78, SUC_lexicon_tags_63 },
    { 0x60221d7c, SUC_lexicon_tags_47 },
    { 0x7ee21d7d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a21d82, SUC_lexicon_tags_2 },
    { 0x39e21d83, SUC_lexicon_tags_9 },
    { 0xa3221d84, SUC_lexicon_tags_14 },
    { 0x3a21d84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0621d92, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e21d97, SUC_lexicon_tags_46 },
    { 0x5421d98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c21d9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa821d9d, SUC_lexicon_tags_7 },
    { 0x8f821d9d, SUC_lexicon_tags_43 },
    { 0x7021d9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4021da1, SUC_lexicon_tags_34 },
    { 0x97821da1, SUC_lexicon_tags_12 },
    { 0x8c21da3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e21da6, SUC_lexicon_tags_14 },
    { 0x95c21da6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2421dad, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x1e021daf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48021db2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82221db9, SUC_lexicon_tags_47 },
    { 0x61821dba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e21dbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde621dc0, SUC_lexicon_tags_9 },
    { 0x2de21dc1, SUC_lexicon_tags_11 },
    { 0x48c21dc1, SUC_lexicon_tags_11 },
    { 0xc21dc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e21dc7, SUC_lexicon_tags_12 },
    { 0x76421dc8, SUC_lexicon_tags_11 },
    { 0x2c821dc9, SUC_lexicon_tags_61 },
    { 0xbd821dc7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd8c21dcc, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xa7e21dce, SUC_lexicon_tags_31 },
    { 0xdda21dcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b421dd9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7821ddd, SUC_lexicon_tags_7 },
    { 0x3f821ddd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6e21de0, SUC_lexicon_tags_66 },
    { 0xe9c21de1, SUC_lexicon_tags_74 },
    { 0x37e21de2, SUC_lexicon_tags_25 },
    { 0xe2021de2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8221de7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da21df6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80621df8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x17221dfa, SUC_lexicon_tags_29 },
    { 0xc2c21dfb, SUC_lexicon_tags_19 },
    { 0xfaa21dfc, SUC_lexicon_tags_11 },
    { 0xbf421dfb, SUC_lexicon_tags_37 },
    { 0xe2c21dfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9821e00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7ca21e02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a21e06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7da21e08, SUC_lexicon_tags_15 },
    { 0x84a21e09, SUC_lexicon_tags_32 },
    { 0x4dc21e0a, SUC_lexicon_tags_11 },
    { 0xa6021e0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca621e10, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d621e14, SUC_lexicon_tags_2 },
    { 0xc4021e15, SUC_lexicon_tags_87 },
    { 0x6dc21e16, SUC_lexicon_tags_0 },
    { 0xa3821e14, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54421e21, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xba021e23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d221e25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30821e2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22621e2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6821e32, SUC_lexicon_tags_17 },
    { 0x38221e33, SUC_lexicon_tags_0 },
    { 0x82621e34, SUC_lexicon_tags_45 },
    { 0x8e21e34, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c21e3a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12221e3f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48621e44, SUC_lexicon_tags_15 },
    { 0xad821e44, SUC_lexicon_tags_10 },
    { 0x62c21e45, SUC_lexicon_tags_37 },
    { 0x64621e47, SUC_lexicon_tags_56 },
    { 0x7f821e48, SUC_lexicon_tags_14 },
    { 0x80a21e49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a21e4f, SUC_lexicon_tags_0 },
    { 0x63221e50, SUC_lexicon_tags_37 },
    { 0x1a21e51, SUC_lexicon_tags_48 },
    { 0x21621e52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x59221e54, SUC_lexicon_tags_12 },
    { 0xd4221e55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33c21e57, SUC_lexicon_tags_19 },
    { 0xd2a21e58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea621e5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34421e5d, SUC_lexicon_tags_60 },
    { 0x3fc21e5e, SUC_lexicon_tags_47 },
    { 0xcb421e5d, SUC_lexicon_tags_62 },
    { 0xf3421e60, SUC_lexicon_tags_34 },
    { 0x93421e5e, SUC_lexicon_tags_16 },
    { 0x9e221e5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a221e65, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc21e6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ca21e6d, SUC_lexicon_tags_0 },
    { 0xdec21e6e, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a21e71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf1821e73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76221e76, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea21e7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e21e80, SUC_lexicon_tags_14 },
    { 0xe9621e80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc21e86, SUC_lexicon_tags_7 },
    { 0x27821e87, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x5fc21e89, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1821e8e, SUC_lexicon_tags_9 },
    { 0xae21e8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15421e93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b421e97, SUC_lexicon_tags_25 },
    { 0x38621e98, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a221e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30021e9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba21ea1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c21ea4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a21ea7, SUC_lexicon_tags_38 },
    { 0x84a21ea8, SUC_lexicon_tags_79 },
    { 0xf8621ea9, SUC_lexicon_tags_11 },
    { 0xaf821eaa, SUC_lexicon_tags_29 },
    { 0x5621eab, SUC_lexicon_tags_26 },
    { 0xa2a21eaa, SUC_lexicon_tags_38 },
    { 0x5a221ead, SUC_lexicon_tags_15 },
    { 0xe7c21eae, SUC_lexicon_tags_19 },
    { 0x75e21eac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72021eb1, SUC_lexicon_tags_2 },
    { 0xb8021eb2, SUC_lexicon_tags_1 },
    { 0x22a21eb2, SUC_lexicon_tags_15 },
    { 0x34221eb4, SUC_lexicon_tags_37 },
    { 0x81c21eb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84e21eb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79421eba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e21ebd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd021ec1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa221ec7, SUC_lexicon_tags_33 },
    { 0x17421ec8, SUC_lexicon_tags_1 },
    { 0xf7621ec9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x79a21ecb, SUC_lexicon_tags_15 },
    { 0x3e421ecc, SUC_lexicon_tags_7 },
    { 0x49a21ecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16021ed2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce21ed5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1421ed7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39621edc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4021ee0, SUC_lexicon_tags_113 },
    { 0x19a21ee1, SUC_lexicon_tags_12 },
    { 0xfe021ee1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e21ee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37421ee7, SUC_lexicon_tags_107 },
    { 0xb3621ee8, SUC_lexicon_tags_9 },
    { 0x9ec21ee9, SUC_lexicon_tags_29 },
    { 0x48421eea, SUC_lexicon_tags_151 },
    { 0xcea21eeb, SUC_lexicon_tags_24 },
    { 0xb0021eeb, SUC_lexicon_tags_63 },
    { 0xd8621eed, SUC_lexicon_tags_25 },
    { 0x1c21eee, SUC_lexicon_tags_47 },
    { 0x9b221ee9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a21ef3, SUC_lexicon_tags_11 },
    { 0x64021ef3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcbc21ef6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa821f05, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e21f09, SUC_lexicon_tags_873 },
    { 0x8f821f0a, SUC_lexicon_tags_45 },
    { 0xde021f0b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x13c21f0d, SUC_lexicon_tags_0 },
    { 0x3dc21f0e, SUC_lexicon_tags_124 },
    { 0xe8821f0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e821f11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d421f17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x28a21f19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf421f1b, SUC_lexicon_tags_12 },
    { 0x54c21f1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea21f22, SUC_lexicon_tags_15 },
    { 0x80821f23, SUC_lexicon_tags_43 },
    { 0xa7421f24, SUC_lexicon_tags_23 },
    { 0xb6e21f22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb221f28, SUC_lexicon_tags_7 },
    { 0xc4021f29, SUC_lexicon_tags_14 },
    { 0xcd021f2a, SUC_lexicon_tags_1 },
    { 0xe4c21f28, SUC_lexicon_tags_33 },
    { 0xdac21f28, SUC_lexicon_tags_8 },
    { 0x7221f2a, SUC_lexicon_tags_27 },
    { 0x30e21f29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce21f31, SUC_lexicon_tags_7 },
    { 0xd3a21f32, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x91421f35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea21f39, SUC_lexicon_tags_116 },
    { 0x74021f3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb021f3c, SUC_lexicon_tags_15 },
    { 0x30421f3d, SUC_lexicon_tags_37 },
    { 0xd5621f3e, SUC_lexicon_tags_85 },
    { 0x10c21f3f, SUC_lexicon_tags_1 },
    { 0xb2221f3c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2821f43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f021f45, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7621f47, SUC_lexicon_tags_11 },
    { 0xa2021f48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77e21f4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6e21f4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34421f50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57221f54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38221f56, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda821f5a, SUC_lexicon_tags_33 },
    { 0xc2a21f5b, SUC_lexicon_tags_55 },
    { 0x421f5a, SUC_lexicon_tags_29 },
    { 0xce821f5c, SUC_lexicon_tags_25 },
    { 0x71c21f5c, SUC_lexicon_tags_61 },
    { 0x9c21f5f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x67221f62, SUC_lexicon_tags_42 },
    { 0x51621f63, SUC_lexicon_tags_92 },
    { 0x76221f64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95421f69, SUC_lexicon_tags_1 },
    { 0x48c21f69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed421f6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52621f6f, SUC_lexicon_tags_26 },
    { 0x83c21f6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6221f72, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x28821f75, SUC_lexicon_tags_100 },
    { 0x3421f75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd021f7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6421f88, SUC_lexicon_tags_6 },
    { 0xefa21f88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99621f8d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7221f91, SUC_lexicon_tags_694 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25621f99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e21f9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83821fa4, SUC_lexicon_tags_31 },
    { 0xec221fa5, SUC_lexicon_tags_44 },
    { 0xae21fa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x30821fa9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcbc21fab, SUC_lexicon_tags_1396 },
    { 0, NULL },
    { 0xde821fad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d421fb0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xce621fb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4f221fb5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9c21fb7, SUC_lexicon_tags_7 },
    { 0x1f821fb8, SUC_lexicon_tags_68 },
    { 0x56e21fb7, SUC_lexicon_tags_63 },
    { 0x92821fba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6dc21fbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e21fbf, SUC_lexicon_tags_17 },
    { 0xe5e21fc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad021fc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xea021fc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2421fc7, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de21fca, SUC_lexicon_tags_45 },
    { 0xf1e21fca, SUC_lexicon_tags_43 },
    { 0x33c21fcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3621fce, SUC_lexicon_tags_37 },
    { 0x18821fcf, SUC_lexicon_tags_154 },
    { 0x76821fcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbec21fd2, SUC_lexicon_tags_0 },
    { 0x56821fd3, SUC_lexicon_tags_37 },
    { 0x3ea21fd4, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x97621fd7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x49e21fd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8021fde, SUC_lexicon_tags_57 },
    { 0x33621fdf, SUC_lexicon_tags_8 },
    { 0x75021fde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a21fe5, SUC_lexicon_tags_11 },
    { 0xe6221fe6, SUC_lexicon_tags_17 },
    { 0x9b821fe7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff021fed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a21ff1, SUC_lexicon_tags_25 },
    { 0xb4421ff2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e621ff5, SUC_lexicon_tags_35 },
    { 0x50c21ff6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3821ffe, SUC_lexicon_tags_65 },
    { 0x30621fff, SUC_lexicon_tags_398 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58222006, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x58822008, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d82200b, SUC_lexicon_tags_12 },
    { 0x5502200b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf842200f, SUC_lexicon_tags_397 },
    { 0xd1c22010, SUC_lexicon_tags_23 },
    { 0x3a2200f, SUC_lexicon_tags_25 },
    { 0x4682200f, SUC_lexicon_tags_8 },
    { 0xbbe22013, SUC_lexicon_tags_4 },
    { 0x7e622014, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9e22016, SUC_lexicon_tags_17 },
    { 0x80a22017, SUC_lexicon_tags_0 },
    { 0x12622018, SUC_lexicon_tags_63 },
    { 0xe8622018, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35422022, SUC_lexicon_tags_32 },
    { 0xa4c22022, SUC_lexicon_tags_35 },
    { 0xb7c22024, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcea22026, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca22031, SUC_lexicon_tags_23 },
    { 0xa2c22032, SUC_lexicon_tags_14 },
    { 0x19a22033, SUC_lexicon_tags_27 },
    { 0x6fc22034, SUC_lexicon_tags_2 },
    { 0x3da22035, SUC_lexicon_tags_38 },
    { 0xc022035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a22039, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6262203c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b622043, SUC_lexicon_tags_25 },
    { 0x48222044, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafa22046, SUC_lexicon_tags_14 },
    { 0xf8c22047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ae22049, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaea2204b, SUC_lexicon_tags_15 },
    { 0xa082204b, SUC_lexicon_tags_9 },
    { 0x8bc2204c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26c2204f, SUC_lexicon_tags_23 },
    { 0xee2204f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c22054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0422056, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc8822058, SUC_lexicon_tags_11 },
    { 0x99022059, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8962205e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac22062, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9222065, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e22069, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f02206f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc622071, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e22074, SUC_lexicon_tags_21 },
    { 0xe0e22075, SUC_lexicon_tags_7 },
    { 0x6b622076, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa22079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e22207b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f02207d, SUC_lexicon_tags_17 },
    { 0xbe22207e, SUC_lexicon_tags_25 },
    { 0x8482207f, SUC_lexicon_tags_1 },
    { 0x26422080, SUC_lexicon_tags_15 },
    { 0xa822081, SUC_lexicon_tags_39 },
    { 0xef822080, SUC_lexicon_tags_25 },
    { 0x7c22083, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac2208b, SUC_lexicon_tags_14 },
    { 0x9742208c, SUC_lexicon_tags_32 },
    { 0xece2208d, SUC_lexicon_tags_17 },
    { 0x1f02208e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c422091, SUC_lexicon_tags_47 },
    { 0x2f622091, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xace22094, SUC_lexicon_tags_25 },
    { 0x64022094, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c2209c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a220a8, SUC_lexicon_tags_11 },
    { 0x63a220a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa6220ab, SUC_lexicon_tags_30 },
    { 0x110220ac, SUC_lexicon_tags_17 },
    { 0x85a220ad, SUC_lexicon_tags_9 },
    { 0x556220ae, SUC_lexicon_tags_17 },
    { 0x8aa220ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x908220b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d8220b3, SUC_lexicon_tags_11 },
    { 0x550220b4, SUC_lexicon_tags_7 },
    { 0x5f4220b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee220b8, SUC_lexicon_tags_37 },
    { 0xf34220b9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188220c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c220ca, SUC_lexicon_tags_11 },
    { 0xe04220cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2220d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84220d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8220de, SUC_lexicon_tags_12 },
    { 0xb22220de, SUC_lexicon_tags_32 },
    { 0x56c220df, SUC_lexicon_tags_25 },
    { 0x87c220e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4220e5, SUC_lexicon_tags_66 },
    { 0xf78220e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0220ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x910220ed, SUC_lexicon_tags_15 },
    { 0x3c6220ee, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x138220f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x480220f2, SUC_lexicon_tags_6 },
    { 0x87c220f3, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0220f6, SUC_lexicon_tags_29 },
    { 0x1a6220f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4a220f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c220fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab222102, SUC_lexicon_tags_19 },
    { 0x3c22103, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3822105, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde22210b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc2210e, SUC_lexicon_tags_12 },
    { 0xa4c2210f, SUC_lexicon_tags_23 },
    { 0x7d62210f, SUC_lexicon_tags_26 },
    { 0x2a22111, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e22116, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17022119, SUC_lexicon_tags_53 },
    { 0x21822119, SUC_lexicon_tags_66 },
    { 0xdba22119, SUC_lexicon_tags_2 },
    { 0x2bc2211c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69022122, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x71e22124, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c422127, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x93e22129, SUC_lexicon_tags_38 },
    { 0xc8422129, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9f62212c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c2212f, SUC_lexicon_tags_11 },
    { 0x2e822130, SUC_lexicon_tags_11 },
    { 0x8622131, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd422135, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f422138, SUC_lexicon_tags_40 },
    { 0x6d822139, SUC_lexicon_tags_63 },
    { 0xdb62213a, SUC_lexicon_tags_2 },
    { 0x482213b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd2a2213d, SUC_lexicon_tags_17 },
    { 0x5f02213e, SUC_lexicon_tags_47 },
    { 0xf5c2213f, SUC_lexicon_tags_7 },
    { 0x30a22140, SUC_lexicon_tags_15 },
    { 0x2be2213f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x89822143, SUC_lexicon_tags_23 },
    { 0x71422144, SUC_lexicon_tags_12 },
    { 0xf6e22145, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5cc22147, SUC_lexicon_tags_8 },
    { 0x6cc22147, SUC_lexicon_tags_43 },
    { 0xf6a22147, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea2214e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a2215a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x422215c, SUC_lexicon_tags_6 },
    { 0x23a2215d, SUC_lexicon_tags_37 },
    { 0x1302215e, SUC_lexicon_tags_15 },
    { 0xe8e2215f, SUC_lexicon_tags_10 },
    { 0xd382215d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x19c22162, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee022166, SUC_lexicon_tags_12 },
    { 0xbcc22167, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39622169, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x26a2216b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4482216d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf562216f, SUC_lexicon_tags_0 },
    { 0xcc22216f, SUC_lexicon_tags_38 },
    { 0x3042216f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99e22173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a22177, SUC_lexicon_tags_17 },
    { 0xd9422178, SUC_lexicon_tags_7 },
    { 0x1c822179, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3142217c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16422180, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7222182, SUC_lexicon_tags_4 },
    { 0xf1822182, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2622186, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68c22188, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc2218f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9fc22191, SUC_lexicon_tags_7 },
    { 0xd9822191, SUC_lexicon_tags_37 },
    { 0x35822192, SUC_lexicon_tags_25 },
    { 0x18822194, SUC_lexicon_tags_15 },
    { 0xd0622194, SUC_lexicon_tags_27 },
    { 0xde022194, SUC_lexicon_tags_1 },
    { 0x1b622197, SUC_lexicon_tags_3 },
    { 0x72e22198, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcae2219a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a82219d, SUC_lexicon_tags_1 },
    { 0xef02219e, SUC_lexicon_tags_27 },
    { 0x8cc2219e, SUC_lexicon_tags_2 },
    { 0x4842219e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390221ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x874221ae, SUC_lexicon_tags_9 },
    { 0x46221ae, SUC_lexicon_tags_25 },
    { 0x682221b0, SUC_lexicon_tags_0 },
    { 0x8a6221b1, SUC_lexicon_tags_33 },
    { 0xe2c221b1, SUC_lexicon_tags_14 },
    { 0xf06221b0, SUC_lexicon_tags_15 },
    { 0xa1e221b0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea221b8, SUC_lexicon_tags_10 },
    { 0xbd6221b9, SUC_lexicon_tags_6 },
    { 0xf12221ba, SUC_lexicon_tags_23 },
    { 0xa0e221ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b4221bd, SUC_lexicon_tags_9 },
    { 0x17a221be, SUC_lexicon_tags_11 },
    { 0xda4221bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a2221c1, SUC_lexicon_tags_11 },
    { 0xbbc221c1, SUC_lexicon_tags_25 },
    { 0x524221c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e221c6, SUC_lexicon_tags_35 },
    { 0x3e2221c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be221cb, SUC_lexicon_tags_37 },
    { 0x2c6221cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc26221ce, SUC_lexicon_tags_28 },
    { 0x614221ce, SUC_lexicon_tags_33 },
    { 0x31e221d0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e221d5, SUC_lexicon_tags_25 },
    { 0x7e4221d6, SUC_lexicon_tags_26 },
    { 0xe54221d6, SUC_lexicon_tags_9 },
    { 0x4f6221d8, SUC_lexicon_tags_37 },
    { 0x7ee221d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x312221db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x46c221dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa221e0, SUC_lexicon_tags_11 },
    { 0x330221e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x296221e3, SUC_lexicon_tags_1 },
    { 0x33a221e3, SUC_lexicon_tags_434 },
    { 0x414221e5, SUC_lexicon_tags_1 },
    { 0xe0e221e3, SUC_lexicon_tags_4 },
    { 0x8ce221e7, SUC_lexicon_tags_11 },
    { 0xb56221e5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x274221ea, SUC_lexicon_tags_49 },
    { 0x428221eb, SUC_lexicon_tags_0 },
    { 0x922221ec, SUC_lexicon_tags_17 },
    { 0x5c2221ed, SUC_lexicon_tags_25 },
    { 0x9c4221ea, SUC_lexicon_tags_17 },
    { 0xd60221ef, SUC_lexicon_tags_38 },
    { 0x122221ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10221f2, SUC_lexicon_tags_7 },
    { 0xc1e221f2, SUC_lexicon_tags_17 },
    { 0x7ac221f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a221f9, SUC_lexicon_tags_7 },
    { 0x582221fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7422206, SUC_lexicon_tags_71 },
    { 0x6a822207, SUC_lexicon_tags_31 },
    { 0xd2a22208, SUC_lexicon_tags_7 },
    { 0x1e822209, SUC_lexicon_tags_21 },
    { 0x1d02220a, SUC_lexicon_tags_32 },
    { 0x8ec22208, SUC_lexicon_tags_11 },
    { 0xbba2220b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13022210, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35222212, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ae22214, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c22216, SUC_lexicon_tags_2 },
    { 0xfee22216, SUC_lexicon_tags_1 },
    { 0xe2622218, SUC_lexicon_tags_37 },
    { 0x89222218, SUC_lexicon_tags_1 },
    { 0xb4022218, SUC_lexicon_tags_16 },
    { 0xa2c2221b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da2221e, SUC_lexicon_tags_33 },
    { 0xd602221e, SUC_lexicon_tags_32 },
    { 0xaba22220, SUC_lexicon_tags_7 },
    { 0xce82221e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e22225, SUC_lexicon_tags_7 },
    { 0xdf422225, SUC_lexicon_tags_14 },
    { 0xcfa22227, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47a22229, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8382222d, SUC_lexicon_tags_12 },
    { 0xf822222d, SUC_lexicon_tags_21 },
    { 0x8da2222f, SUC_lexicon_tags_87 },
    { 0xb642222d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa22236, SUC_lexicon_tags_30 },
    { 0x1c622236, SUC_lexicon_tags_92 },
    { 0xfb622238, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4862223a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6a2223c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15422240, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc22243, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8022246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1c22248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6be2224a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4482224c, SUC_lexicon_tags_8 },
    { 0x6862224c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c22252, SUC_lexicon_tags_1 },
    { 0x72022252, SUC_lexicon_tags_2 },
    { 0xa2c22253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc2225c, SUC_lexicon_tags_17 },
    { 0x37e2225d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xec62225f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe622226b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf982227b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58c2227d, SUC_lexicon_tags_6 },
    { 0x1e22227e, SUC_lexicon_tags_1 },
    { 0x6ac2227d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e22283, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf5822285, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b02228c, SUC_lexicon_tags_23 },
    { 0x9762228c, SUC_lexicon_tags_25 },
    { 0x4e22228e, SUC_lexicon_tags_77 },
    { 0xf9e2228e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96422291, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x55822294, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6022297, SUC_lexicon_tags_57 },
    { 0x69222297, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36a2229a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7222229c, SUC_lexicon_tags_17 },
    { 0xa942229d, SUC_lexicon_tags_17 },
    { 0xecc2229c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc8222a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0222a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x510222a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0222aa, SUC_lexicon_tags_17 },
    { 0xdb4222ab, SUC_lexicon_tags_15 },
    { 0x382222ab, SUC_lexicon_tags_12 },
    { 0x346222aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb28222af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0222b3, SUC_lexicon_tags_45 },
    { 0x6c6222b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68222b8, SUC_lexicon_tags_37 },
    { 0x6f2222b9, SUC_lexicon_tags_47 },
    { 0x890222ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x996222bd, SUC_lexicon_tags_21 },
    { 0xaa222be, SUC_lexicon_tags_2 },
    { 0x3e222bd, SUC_lexicon_tags_23 },
    { 0xca222c0, SUC_lexicon_tags_47 },
    { 0xc32222bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77c222c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc90222c5, SUC_lexicon_tags_43 },
    { 0xe3a222c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4ee222c8, SUC_lexicon_tags_33 },
    { 0xc74222c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6222cc, SUC_lexicon_tags_23 },
    { 0x5c0222cd, SUC_lexicon_tags_7 },
    { 0xba0222cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x850222d0, SUC_lexicon_tags_7 },
    { 0x80c222d0, SUC_lexicon_tags_30 },
    { 0xaa8222d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2222d5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c222d8, SUC_lexicon_tags_14 },
    { 0xaf4222d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe7e222db, SUC_lexicon_tags_37 },
    { 0xe9c222dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46222e9, SUC_lexicon_tags_1 },
    { 0x708222ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2222ee, SUC_lexicon_tags_26 },
    { 0x7f4222ef, SUC_lexicon_tags_17 },
    { 0x33a222f0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4222f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x812222f9, SUC_lexicon_tags_21 },
    { 0x120222fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a22302, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac222304, SUC_lexicon_tags_37 },
    { 0x87822305, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a22308, SUC_lexicon_tags_22 },
    { 0xc6822309, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b62230e, SUC_lexicon_tags_37 },
    { 0xe622230e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf022312, SUC_lexicon_tags_14 },
    { 0x3a22313, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e22316, SUC_lexicon_tags_26 },
    { 0xdf422317, SUC_lexicon_tags_12 },
    { 0x92622318, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2422320, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e22324, SUC_lexicon_tags_70 },
    { 0x9de22324, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x67022327, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d42232a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf02232d, SUC_lexicon_tags_7 },
    { 0x9962232e, SUC_lexicon_tags_11 },
    { 0x34e2232e, SUC_lexicon_tags_12 },
    { 0xe202232f, SUC_lexicon_tags_11 },
    { 0x9142232f, SUC_lexicon_tags_37 },
    { 0x2ac22331, SUC_lexicon_tags_17 },
    { 0xcb622333, SUC_lexicon_tags_11 },
    { 0xed022334, SUC_lexicon_tags_38 },
    { 0xa0e22335, SUC_lexicon_tags_37 },
    { 0xcb222334, SUC_lexicon_tags_43 },
    { 0x40622337, SUC_lexicon_tags_14 },
    { 0x89222338, SUC_lexicon_tags_238 },
    { 0xa5822337, SUC_lexicon_tags_57 },
    { 0xba42233a, SUC_lexicon_tags_45 },
    { 0x87e2233a, SUC_lexicon_tags_12 },
    { 0xc2e22336, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2042233e, SUC_lexicon_tags_37 },
    { 0xb902233e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74422344, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ae22346, SUC_lexicon_tags_23 },
    { 0x20222347, SUC_lexicon_tags_49 },
    { 0xf3022348, SUC_lexicon_tags_10 },
    { 0x1f222349, SUC_lexicon_tags_15 },
    { 0x20822348, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5562234e, SUC_lexicon_tags_7 },
    { 0x56a2234f, SUC_lexicon_tags_63 },
    { 0xc5e22350, SUC_lexicon_tags_4 },
    { 0xf3222351, SUC_lexicon_tags_1 },
    { 0x8a22350, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a022356, SUC_lexicon_tags_33 },
    { 0x9fe22357, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9942235b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e22361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a22366, SUC_lexicon_tags_19 },
    { 0xc4a22367, SUC_lexicon_tags_37 },
    { 0x62022368, SUC_lexicon_tags_15 },
    { 0x2b422367, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e2236e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70022371, SUC_lexicon_tags_15 },
    { 0x6e422372, SUC_lexicon_tags_17 },
    { 0xc8222373, SUC_lexicon_tags_38 },
    { 0x54a22372, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa2237c, SUC_lexicon_tags_2 },
    { 0xd4e2237c, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xdba2237f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f822384, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3de22386, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c22389, SUC_lexicon_tags_45 },
    { 0xd122238a, SUC_lexicon_tags_11 },
    { 0x5722238a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8102238d, SUC_lexicon_tags_7 },
    { 0xfb02238e, SUC_lexicon_tags_79 },
    { 0x9b82238d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee222396, SUC_lexicon_tags_11 },
    { 0x59a22396, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc822399, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd542239f, SUC_lexicon_tags_2 },
    { 0xcd22239f, SUC_lexicon_tags_47 },
    { 0x9d6223a0, SUC_lexicon_tags_23 },
    { 0x908223a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e223a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76223a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820223ad, SUC_lexicon_tags_60 },
    { 0x550223ad, SUC_lexicon_tags_3 },
    { 0x8ee223af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68223b2, SUC_lexicon_tags_37 },
    { 0x5c4223b3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1d4223b5, SUC_lexicon_tags_38 },
    { 0xb18223b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20223ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdce223bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00223c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8223c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78223c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8223d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8223d5, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc223d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce223dd, SUC_lexicon_tags_37 },
    { 0x434223de, SUC_lexicon_tags_1588 },
    { 0x1ec223df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbac223e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e223e9, SUC_lexicon_tags_49 },
    { 0x80a223e9, SUC_lexicon_tags_38 },
    { 0xfa4223eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6223f0, SUC_lexicon_tags_17 },
    { 0xe82223f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6223f4, SUC_lexicon_tags_29 },
    { 0x81a223f5, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x1e8223f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10a223f9, SUC_lexicon_tags_19 },
    { 0x854223f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c223fe, SUC_lexicon_tags_8 },
    { 0xd64223ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9e22401, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5222404, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c22409, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e2240c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c022410, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6622413, SUC_lexicon_tags_68 },
    { 0xbe622414, SUC_lexicon_tags_11 },
    { 0x36a22414, SUC_lexicon_tags_47 },
    { 0x1a822415, SUC_lexicon_tags_37 },
    { 0x65222415, SUC_lexicon_tags_37 },
    { 0x63a22416, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc2241c, SUC_lexicon_tags_31 },
    { 0x5342241d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec22420, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8822423, SUC_lexicon_tags_46 },
    { 0x72c22423, SUC_lexicon_tags_7 },
    { 0x5d622425, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2222427, SUC_lexicon_tags_12 },
    { 0xe8e22428, SUC_lexicon_tags_6 },
    { 0xf2c22428, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd942242b, SUC_lexicon_tags_14 },
    { 0x9942242c, SUC_lexicon_tags_27 },
    { 0x4122242d, SUC_lexicon_tags_17 },
    { 0x4082242b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b422431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80222433, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba422437, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a2243c, SUC_lexicon_tags_8 },
    { 0xb8a2243d, SUC_lexicon_tags_16 },
    { 0x4cc2243c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x60222441, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b022444, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42622448, SUC_lexicon_tags_25 },
    { 0x19622448, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75a2244b, SUC_lexicon_tags_12 },
    { 0x41e2244c, SUC_lexicon_tags_37 },
    { 0x9b82244d, SUC_lexicon_tags_57 },
    { 0xe742244d, SUC_lexicon_tags_30 },
    { 0x96e2244e, SUC_lexicon_tags_37 },
    { 0xdb22244f, SUC_lexicon_tags_38 },
    { 0x7ce22451, SUC_lexicon_tags_15 },
    { 0x50a22452, SUC_lexicon_tags_2 },
    { 0x4602244e, SUC_lexicon_tags_60 },
    { 0x99422454, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc082245e, SUC_lexicon_tags_25 },
    { 0xd682245f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4822465, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8262246f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce22476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c22479, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55822480, SUC_lexicon_tags_15 },
    { 0x44e22481, SUC_lexicon_tags_6 },
    { 0x2f022481, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad222489, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe542248c, SUC_lexicon_tags_60 },
    { 0x4ec2248d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be22493, SUC_lexicon_tags_1 },
    { 0xd9022493, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5822499, SUC_lexicon_tags_39 },
    { 0x22c22499, SUC_lexicon_tags_80 },
    { 0x43e22499, SUC_lexicon_tags_25 },
    { 0x8982249c, SUC_lexicon_tags_43 },
    { 0x5582249b, SUC_lexicon_tags_31 },
    { 0xc7c2249e, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc224a1, SUC_lexicon_tags_84 },
    { 0xbe4224a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca2224a4, SUC_lexicon_tags_3 },
    { 0x53e224a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e224ab, SUC_lexicon_tags_37 },
    { 0x2dc224ac, SUC_lexicon_tags_18 },
    { 0xbbe224ad, SUC_lexicon_tags_45 },
    { 0xc1c224ae, SUC_lexicon_tags_2 },
    { 0x530224af, SUC_lexicon_tags_2 },
    { 0x22a224ad, SUC_lexicon_tags_16 },
    { 0xf44224ae, SUC_lexicon_tags_46 },
    { 0x408224ad, SUC_lexicon_tags_27 },
    { 0x6ea224ab, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6224b7, SUC_lexicon_tags_37 },
    { 0x944224b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370224bc, SUC_lexicon_tags_6 },
    { 0x310224bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xea0224bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04224c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a224c5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x52e224c7, SUC_lexicon_tags_14 },
    { 0xf48224c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96224d1, SUC_lexicon_tags_11 },
    { 0x4fe224d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50224d7, SUC_lexicon_tags_15 },
    { 0x1e224d8, SUC_lexicon_tags_17 },
    { 0xf6e224d8, SUC_lexicon_tags_37 },
    { 0x888224da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe4224dc, SUC_lexicon_tags_38 },
    { 0x3b2224dd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcc224df, SUC_lexicon_tags_1 },
    { 0xc9c224e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f6224e2, SUC_lexicon_tags_17 },
    { 0x8ce224e3, SUC_lexicon_tags_0 },
    { 0x4c6224e4, SUC_lexicon_tags_103 },
    { 0xc3e224e3, SUC_lexicon_tags_7 },
    { 0x2bc224e6, SUC_lexicon_tags_2 },
    { 0xaa4224e5, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92224ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x482224ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ec224f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6224f4, SUC_lexicon_tags_60 },
    { 0xff2224f4, SUC_lexicon_tags_25 },
    { 0x2c8224f6, SUC_lexicon_tags_37 },
    { 0xe3e224f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x36e224f9, SUC_lexicon_tags_17 },
    { 0xae4224fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacc224fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4224fe, SUC_lexicon_tags_26 },
    { 0x618224fe, SUC_lexicon_tags_17 },
    { 0x7ea224ff, SUC_lexicon_tags_45 },
    { 0x1b6224ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae2250b, SUC_lexicon_tags_62 },
    { 0x94e2250b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9962250f, SUC_lexicon_tags_10 },
    { 0x42250f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad622515, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be22518, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e22521, SUC_lexicon_tags_27 },
    { 0x11e22521, SUC_lexicon_tags_37 },
    { 0xfea22523, SUC_lexicon_tags_35 },
    { 0xb1e22524, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca022527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da2252a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53222530, SUC_lexicon_tags_11 },
    { 0x7b022531, SUC_lexicon_tags_31 },
    { 0x20822532, SUC_lexicon_tags_19 },
    { 0x5b422531, SUC_lexicon_tags_95 },
    { 0xb6222533, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f22253b, SUC_lexicon_tags_15 },
    { 0xf422253c, SUC_lexicon_tags_33 },
    { 0xe982253d, SUC_lexicon_tags_12 },
    { 0x5ec2253e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5e22540, SUC_lexicon_tags_25 },
    { 0x92622541, SUC_lexicon_tags_27 },
    { 0xf8a22542, SUC_lexicon_tags_11 },
    { 0xd3a22543, SUC_lexicon_tags_25 },
    { 0xd7a22544, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c22547, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x74e22549, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe2254c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4082254f, SUC_lexicon_tags_43 },
    { 0x8b222550, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85422555, SUC_lexicon_tags_38 },
    { 0x89022556, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f02255d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefa2255f, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0x73622562, SUC_lexicon_tags_87 },
    { 0x4d222562, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9622568, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce2256f, SUC_lexicon_tags_37 },
    { 0x3722256f, SUC_lexicon_tags_2 },
    { 0x69a22570, SUC_lexicon_tags_2 },
    { 0xe7022571, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5022574, SUC_lexicon_tags_26 },
    { 0x8d222574, SUC_lexicon_tags_12 },
    { 0xbd022575, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1cc22578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1362257e, SUC_lexicon_tags_9 },
    { 0xce42257e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa222581, SUC_lexicon_tags_25 },
    { 0x91222582, SUC_lexicon_tags_15 },
    { 0x95a22581, SUC_lexicon_tags_11 },
    { 0xbe022584, SUC_lexicon_tags_19 },
    { 0x50a22585, SUC_lexicon_tags_11 },
    { 0xc7022584, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3622589, SUC_lexicon_tags_26 },
    { 0xe542258a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25a2258c, SUC_lexicon_tags_2 },
    { 0x16c2258d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82022590, SUC_lexicon_tags_37 },
    { 0x30422591, SUC_lexicon_tags_10 },
    { 0x9e022592, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a22595, SUC_lexicon_tags_19 },
    { 0x33c22595, SUC_lexicon_tags_11 },
    { 0x42e22597, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc02259a, SUC_lexicon_tags_48 },
    { 0x9e42259b, SUC_lexicon_tags_14 },
    { 0x9022259b, SUC_lexicon_tags_17 },
    { 0x63a2259d, SUC_lexicon_tags_30 },
    { 0xfd02259e, SUC_lexicon_tags_27 },
    { 0xc482259f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa225a3, SUC_lexicon_tags_37 },
    { 0xa0e225a4, SUC_lexicon_tags_17 },
    { 0xfb6225a4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74225a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe225ac, SUC_lexicon_tags_15 },
    { 0x336225ad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe32225af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66225b2, SUC_lexicon_tags_45 },
    { 0x8ac225b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x412225b6, SUC_lexicon_tags_25 },
    { 0xf56225b6, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e225ba, SUC_lexicon_tags_8 },
    { 0x5e6225bb, SUC_lexicon_tags_37 },
    { 0x92225bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98225c0, SUC_lexicon_tags_12 },
    { 0x3c2225c1, SUC_lexicon_tags_63 },
    { 0x978225c2, SUC_lexicon_tags_1 },
    { 0x6c2225c0, SUC_lexicon_tags_25 },
    { 0xfd6225c4, SUC_lexicon_tags_47 },
    { 0x2cc225c5, SUC_lexicon_tags_7 },
    { 0x148225c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7e225c8, SUC_lexicon_tags_37 },
    { 0xe66225c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ca225cb, SUC_lexicon_tags_17 },
    { 0x356225cc, SUC_lexicon_tags_29 },
    { 0x7c6225cd, SUC_lexicon_tags_6 },
    { 0xf8c225cb, SUC_lexicon_tags_4 },
    { 0xd7a225cf, SUC_lexicon_tags_4 },
    { 0xdce225ce, SUC_lexicon_tags_1 },
    { 0xff2225d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8225d5, SUC_lexicon_tags_15 },
    { 0xf4e225d6, SUC_lexicon_tags_37 },
    { 0x28225d5, SUC_lexicon_tags_11 },
    { 0x13c225d6, SUC_lexicon_tags_39 },
    { 0xf38225d7, SUC_lexicon_tags_24 },
    { 0xea6225da, SUC_lexicon_tags_27 },
    { 0x85a225db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60225df, SUC_lexicon_tags_45 },
    { 0xd5c225df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c225e6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc225eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0225ee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34225f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x39a225fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e225fe, SUC_lexicon_tags_29 },
    { 0xbb0225ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a822601, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74022607, SUC_lexicon_tags_7 },
    { 0xfe422608, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x77a2260a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x72c2260c, SUC_lexicon_tags_38 },
    { 0xf0e2260c, SUC_lexicon_tags_11 },
    { 0x4982260e, SUC_lexicon_tags_48 },
    { 0x4e82260f, SUC_lexicon_tags_41 },
    { 0xa222260e, SUC_lexicon_tags_17 },
    { 0x6fa22611, SUC_lexicon_tags_77 },
    { 0x6e22260e, SUC_lexicon_tags_201 },
    { 0xcca22613, SUC_lexicon_tags_33 },
    { 0x4702260d, SUC_lexicon_tags_1 },
    { 0xd2e22615, SUC_lexicon_tags_14 },
    { 0xd0822612, SUC_lexicon_tags_38 },
    { 0x7882260f, SUC_lexicon_tags_68 },
    { 0x33822614, SUC_lexicon_tags_17 },
    { 0xfde22619, SUC_lexicon_tags_15 },
    { 0xf002261a, SUC_lexicon_tags_17 },
    { 0x7a22616, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41e2261d, SUC_lexicon_tags_37 },
    { 0xf642261e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b622621, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2a22623, SUC_lexicon_tags_45 },
    { 0xd2022624, SUC_lexicon_tags_38 },
    { 0x83e22623, SUC_lexicon_tags_25 },
    { 0xa4a22626, SUC_lexicon_tags_12 },
    { 0x3e22626, SUC_lexicon_tags_23 },
    { 0x7aa22627, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa582262a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe022632, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe22639, SUC_lexicon_tags_193 },
    { 0xd942263a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf462263c, SUC_lexicon_tags_25 },
    { 0xba02263d, SUC_lexicon_tags_61 },
    { 0x6d62263d, SUC_lexicon_tags_82 },
    { 0xe8c2263d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc22644, SUC_lexicon_tags_30 },
    { 0x4fc22644, SUC_lexicon_tags_22 },
    { 0x59422646, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3222649, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2102264f, SUC_lexicon_tags_17 },
    { 0x9b42264f, SUC_lexicon_tags_62 },
    { 0xdea22651, SUC_lexicon_tags_27 },
    { 0x5d022652, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x30e22654, SUC_lexicon_tags_4 },
    { 0x52822655, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662265b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff82265d, SUC_lexicon_tags_25 },
    { 0xd742265d, SUC_lexicon_tags_61 },
    { 0xcca2265e, SUC_lexicon_tags_12 },
    { 0x65c22660, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2222663, SUC_lexicon_tags_19 },
    { 0xa0622664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a022666, SUC_lexicon_tags_15 },
    { 0xd2422666, SUC_lexicon_tags_7 },
    { 0x17022668, SUC_lexicon_tags_63 },
    { 0xa4222669, SUC_lexicon_tags_11 },
    { 0xffa22667, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xedc2266c, SUC_lexicon_tags_17 },
    { 0x6522266d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d822670, SUC_lexicon_tags_0 },
    { 0x62e22671, SUC_lexicon_tags_11 },
    { 0x10822672, SUC_lexicon_tags_43 },
    { 0xf3422672, SUC_lexicon_tags_37 },
    { 0x40c22672, SUC_lexicon_tags_37 },
    { 0x8e22673, SUC_lexicon_tags_26 },
    { 0xc1c22676, SUC_lexicon_tags_29 },
    { 0xfba22672, SUC_lexicon_tags_1 },
    { 0x91622677, SUC_lexicon_tags_17 },
    { 0x6fc22679, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5342267c, SUC_lexicon_tags_2 },
    { 0x4122267d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4e2267f, SUC_lexicon_tags_11 },
    { 0x9822680, SUC_lexicon_tags_25 },
    { 0x4042267f, SUC_lexicon_tags_45 },
    { 0x58022682, SUC_lexicon_tags_27 },
    { 0x7e022683, SUC_lexicon_tags_2 },
    { 0x8f422684, SUC_lexicon_tags_12 },
    { 0x23022683, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb642268c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a22268f, SUC_lexicon_tags_13 },
    { 0x57822690, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23822694, SUC_lexicon_tags_8 },
    { 0x11c22695, SUC_lexicon_tags_25 },
    { 0xc9822696, SUC_lexicon_tags_15 },
    { 0x1f022697, SUC_lexicon_tags_17 },
    { 0xbf222695, SUC_lexicon_tags_11 },
    { 0x47422699, SUC_lexicon_tags_2 },
    { 0xa5c22696, SUC_lexicon_tags_11 },
    { 0xdba22696, SUC_lexicon_tags_65 },
    { 0x6e62269c, SUC_lexicon_tags_2 },
    { 0x84622699, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x53e2269f, SUC_lexicon_tags_11 },
    { 0x72a2269f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x790226a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36c226a4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e226a7, SUC_lexicon_tags_9 },
    { 0xbe8226a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29e226aa, SUC_lexicon_tags_15 },
    { 0xc18226aa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272226b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72226b3, SUC_lexicon_tags_45 },
    { 0xbc2226b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea8226b6, SUC_lexicon_tags_60 },
    { 0x50c226b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976226c1, SUC_lexicon_tags_39 },
    { 0x104226c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa1e226c4, SUC_lexicon_tags_15 },
    { 0x282226c4, SUC_lexicon_tags_27 },
    { 0x5c4226c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe0226c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0226cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c226d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff2226d6, SUC_lexicon_tags_25 },
    { 0xd0c226d7, SUC_lexicon_tags_82 },
    { 0x396226d8, SUC_lexicon_tags_43 },
    { 0xd46226d9, SUC_lexicon_tags_49 },
    { 0xb80226d8, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2226de, SUC_lexicon_tags_37 },
    { 0xa4226df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc18226e1, SUC_lexicon_tags_1 },
    { 0x388226e1, SUC_lexicon_tags_74 },
    { 0x558226e3, SUC_lexicon_tags_15 },
    { 0x812226e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb38226e6, SUC_lexicon_tags_12 },
    { 0x7de226e7, SUC_lexicon_tags_15 },
    { 0xc5e226e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0226eb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x25e226ed, SUC_lexicon_tags_27 },
    { 0xb30226ee, SUC_lexicon_tags_8 },
    { 0x75e226ef, SUC_lexicon_tags_32 },
    { 0x466226f0, SUC_lexicon_tags_25 },
    { 0x9f0226f1, SUC_lexicon_tags_23 },
    { 0xd0e226ef, SUC_lexicon_tags_37 },
    { 0x1b4226f1, SUC_lexicon_tags_12 },
    { 0xf4226f1, SUC_lexicon_tags_8 },
    { 0xa1c226f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8226fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe422701, SUC_lexicon_tags_6 },
    { 0x7c622702, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e222705, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f022707, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea2270e, SUC_lexicon_tags_14 },
    { 0xfbc2270f, SUC_lexicon_tags_45 },
    { 0x94c22710, SUC_lexicon_tags_0 },
    { 0xb9622710, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c422716, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2271a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe02271f, SUC_lexicon_tags_6 },
    { 0xa9e2271f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5022722, SUC_lexicon_tags_47 },
    { 0xa3422722, SUC_lexicon_tags_60 },
    { 0xbc422724, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29c22726, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc822728, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x182272a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x43a2272c, SUC_lexicon_tags_15 },
    { 0x5522272c, SUC_lexicon_tags_29 },
    { 0xf342272e, SUC_lexicon_tags_6 },
    { 0x60c2272f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d422732, SUC_lexicon_tags_0 },
    { 0x92e22732, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50a22735, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae2273b, SUC_lexicon_tags_1 },
    { 0xf202273b, SUC_lexicon_tags_60 },
    { 0xffc2273d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8622273f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5622743, SUC_lexicon_tags_77 },
    { 0xc5a22744, SUC_lexicon_tags_7 },
    { 0x30a22745, SUC_lexicon_tags_48 },
    { 0xb8622743, SUC_lexicon_tags_37 },
    { 0x58c22744, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcec22749, SUC_lexicon_tags_0 },
    { 0xea22274a, SUC_lexicon_tags_37 },
    { 0x5222274b, SUC_lexicon_tags_7 },
    { 0xadc2274c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa22752, SUC_lexicon_tags_60 },
    { 0x55622753, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19222758, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa022760, SUC_lexicon_tags_39 },
    { 0x3222761, SUC_lexicon_tags_25 },
    { 0xd8022762, SUC_lexicon_tags_60 },
    { 0x19222763, SUC_lexicon_tags_2 },
    { 0x7aa22761, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3222767, SUC_lexicon_tags_66 },
    { 0x52e22768, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a22770, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0622772, SUC_lexicon_tags_47 },
    { 0x19822773, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x6c622775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34622778, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8462277a, SUC_lexicon_tags_56 },
    { 0x2762277b, SUC_lexicon_tags_17 },
    { 0xba82277c, SUC_lexicon_tags_29 },
    { 0x6a22277b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc822784, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83022786, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c2278b, SUC_lexicon_tags_0 },
    { 0x8ea2278c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0222791, SUC_lexicon_tags_27 },
    { 0x1822791, SUC_lexicon_tags_11 },
    { 0xb8822791, SUC_lexicon_tags_5 },
    { 0x42222793, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x94422796, SUC_lexicon_tags_7 },
    { 0x88e22796, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x31622799, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3622279b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd62279e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e227a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x948227a3, SUC_lexicon_tags_49 },
    { 0x2e6227a4, SUC_lexicon_tags_201 },
    { 0xaf2227a5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca227ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x63e227ae, SUC_lexicon_tags_48 },
    { 0x632227af, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2227b3, SUC_lexicon_tags_15 },
    { 0xd6a227b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6227b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8227bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x680227bd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x776227bf, SUC_lexicon_tags_37 },
    { 0x612227c0, SUC_lexicon_tags_4 },
    { 0x2fe227c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee227c7, SUC_lexicon_tags_19 },
    { 0x2f0227c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76227cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a0227ce, SUC_lexicon_tags_10 },
    { 0x426227cf, SUC_lexicon_tags_11 },
    { 0x64c227ce, SUC_lexicon_tags_27 },
    { 0x5a8227d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0227d5, SUC_lexicon_tags_15 },
    { 0xbd4227d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666227dd, SUC_lexicon_tags_17 },
    { 0xc36227dd, SUC_lexicon_tags_733 },
    { 0xf5e227de, SUC_lexicon_tags_16 },
    { 0x814227df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80227e3, SUC_lexicon_tags_11 },
    { 0xcf2227e3, SUC_lexicon_tags_39 },
    { 0x7ba227e5, SUC_lexicon_tags_17 },
    { 0xe7a227e6, SUC_lexicon_tags_37 },
    { 0xf44227e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x610227e9, SUC_lexicon_tags_6 },
    { 0xe78227ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e6227ec, SUC_lexicon_tags_12 },
    { 0xfb4227ec, SUC_lexicon_tags_15 },
    { 0x558227ec, SUC_lexicon_tags_17 },
    { 0xebc227ed, SUC_lexicon_tags_17 },
    { 0x250227f0, SUC_lexicon_tags_3 },
    { 0x612227f0, SUC_lexicon_tags_79 },
    { 0xa16227f2, SUC_lexicon_tags_103 },
    { 0x1a6227f2, SUC_lexicon_tags_19 },
    { 0x882227f3, SUC_lexicon_tags_19 },
    { 0x698227f2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48227fb, SUC_lexicon_tags_65 },
    { 0x5fa227fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42422801, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70822803, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9422806, SUC_lexicon_tags_0 },
    { 0xe7822807, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa2280b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8ba2280d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a622813, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf622816, SUC_lexicon_tags_1 },
    { 0x10622816, SUC_lexicon_tags_87 },
    { 0x1fa22818, SUC_lexicon_tags_12 },
    { 0xc0822819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e2281d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5aa2281f, SUC_lexicon_tags_87 },
    { 0x41e22820, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xef222823, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11022825, SUC_lexicon_tags_32 },
    { 0x9be22825, SUC_lexicon_tags_1 },
    { 0xc4e22826, SUC_lexicon_tags_37 },
    { 0x72c22828, SUC_lexicon_tags_2 },
    { 0x18422829, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb02282c, SUC_lexicon_tags_25 },
    { 0x97a2282c, SUC_lexicon_tags_49 },
    { 0xaa02282e, SUC_lexicon_tags_12 },
    { 0x1da2282d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2622831, SUC_lexicon_tags_17 },
    { 0xb9a22831, SUC_lexicon_tags_34 },
    { 0x93c22833, SUC_lexicon_tags_14 },
    { 0x8f022834, SUC_lexicon_tags_27 },
    { 0x97e22835, SUC_lexicon_tags_11 },
    { 0x5ce22835, SUC_lexicon_tags_60 },
    { 0x3e222837, SUC_lexicon_tags_47 },
    { 0x94022835, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2283d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3462283f, SUC_lexicon_tags_32 },
    { 0x43622840, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82822844, SUC_lexicon_tags_11 },
    { 0xed622845, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e622847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x622284a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4982284c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a82284e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30022854, SUC_lexicon_tags_37 },
    { 0x8ce22855, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xa1022857, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc2285a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30a2285c, SUC_lexicon_tags_63 },
    { 0xcbe2285d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e22862, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242286c, SUC_lexicon_tags_28 },
    { 0x8ac2286d, SUC_lexicon_tags_11 },
    { 0x9fa2286e, SUC_lexicon_tags_15 },
    { 0x382286d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e22872, SUC_lexicon_tags_19 },
    { 0xf2c22872, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c22877, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e82287b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf222287f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d422884, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69c22886, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54622888, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6b82288a, SUC_lexicon_tags_25 },
    { 0x3202288b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5142288f, SUC_lexicon_tags_1 },
    { 0xfe822890, SUC_lexicon_tags_215 },
    { 0xa7c22891, SUC_lexicon_tags_3 },
    { 0xf9022890, SUC_lexicon_tags_1 },
    { 0x91222890, SUC_lexicon_tags_37 },
    { 0xa4a22894, SUC_lexicon_tags_14 },
    { 0x29a2288f, SUC_lexicon_tags_26 },
    { 0x2c222896, SUC_lexicon_tags_11 },
    { 0xdba22890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be2289d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1e2289f, SUC_lexicon_tags_27 },
    { 0x266228a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca0228a2, SUC_lexicon_tags_31 },
    { 0xcb4228a3, SUC_lexicon_tags_26 },
    { 0x5b0228a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a228ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf64228ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6de228b0, SUC_lexicon_tags_68 },
    { 0xd9c228b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d0228b3, SUC_lexicon_tags_25 },
    { 0x73a228b3, SUC_lexicon_tags_18 },
    { 0xaca228b5, SUC_lexicon_tags_0 },
    { 0xbb6228b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdec228b9, SUC_lexicon_tags_47 },
    { 0xc74228b9, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x362228bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a228c4, SUC_lexicon_tags_25 },
    { 0xe56228c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfa2228c7, SUC_lexicon_tags_25 },
    { 0x12228c7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa228cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6228ce, SUC_lexicon_tags_29 },
    { 0x13c228cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e228d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e228d7, SUC_lexicon_tags_37 },
    { 0xef0228d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea228db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c228de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec0228e0, SUC_lexicon_tags_7 },
    { 0x7de228e1, SUC_lexicon_tags_28 },
    { 0x7c6228e2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb12228e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x13e228e7, SUC_lexicon_tags_45 },
    { 0x4fa228e8, SUC_lexicon_tags_14 },
    { 0x4f6228e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x990228eb, SUC_lexicon_tags_2 },
    { 0x58e228ec, SUC_lexicon_tags_37 },
    { 0x14e228eb, SUC_lexicon_tags_11 },
    { 0xc20228ee, SUC_lexicon_tags_60 },
    { 0x5b6228ec, SUC_lexicon_tags_9 },
    { 0x988228f0, SUC_lexicon_tags_23 },
    { 0x51e228eb, SUC_lexicon_tags_2 },
    { 0x752228f2, SUC_lexicon_tags_41 },
    { 0x6be228f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6228f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386228fd, SUC_lexicon_tags_27 },
    { 0x76e228fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c22903, SUC_lexicon_tags_7 },
    { 0xfa622904, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x69222907, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8442290c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3582290e, SUC_lexicon_tags_48 },
    { 0x31a2290f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64c22911, SUC_lexicon_tags_43 },
    { 0x4cc22911, SUC_lexicon_tags_8 },
    { 0x67022913, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x18222915, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45a22917, SUC_lexicon_tags_12 },
    { 0x60c22918, SUC_lexicon_tags_11 },
    { 0x37222919, SUC_lexicon_tags_16 },
    { 0xdf42291a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde2291d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ae2291f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75622921, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7e22923, SUC_lexicon_tags_9 },
    { 0x62e22923, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d822927, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba2292a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa2292e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x17222931, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8822935, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9bc22937, SUC_lexicon_tags_15 },
    { 0xf4c22937, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x542293b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf7e2293d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c22942, SUC_lexicon_tags_7 },
    { 0xc7022943, SUC_lexicon_tags_17 },
    { 0xb622944, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8502294b, SUC_lexicon_tags_25 },
    { 0x90a2294c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a22958, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9262295a, SUC_lexicon_tags_27 },
    { 0xb302295b, SUC_lexicon_tags_45 },
    { 0x7ce2295b, SUC_lexicon_tags_1 },
    { 0xa2a2295a, SUC_lexicon_tags_11 },
    { 0x8ba2295e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc222960, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec22966, SUC_lexicon_tags_2 },
    { 0x6622967, SUC_lexicon_tags_11 },
    { 0x95822968, SUC_lexicon_tags_27 },
    { 0x90022969, SUC_lexicon_tags_11 },
    { 0x4762296a, SUC_lexicon_tags_12 },
    { 0x8c02296b, SUC_lexicon_tags_25 },
    { 0xa2c2296c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea22970, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582297b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa642297d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a22980, SUC_lexicon_tags_17 },
    { 0xe2422981, SUC_lexicon_tags_14 },
    { 0xc4e22982, SUC_lexicon_tags_47 },
    { 0x65022983, SUC_lexicon_tags_12 },
    { 0x25c22984, SUC_lexicon_tags_37 },
    { 0x56622985, SUC_lexicon_tags_29 },
    { 0x8ca22982, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf542298a, SUC_lexicon_tags_12 },
    { 0x7f22298b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9c2298d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5722298f, SUC_lexicon_tags_11 },
    { 0x5ca2298f, SUC_lexicon_tags_7 },
    { 0x22022991, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x84c22993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e22996, SUC_lexicon_tags_15 },
    { 0xfa222996, SUC_lexicon_tags_37 },
    { 0x72c22998, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a2299d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c229a0, SUC_lexicon_tags_1688 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88229a5, SUC_lexicon_tags_7 },
    { 0xb38229a6, SUC_lexicon_tags_63 },
    { 0xbd4229a7, SUC_lexicon_tags_29 },
    { 0x95c229a8, SUC_lexicon_tags_49 },
    { 0x88229a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580229af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc0e229b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940229b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc229b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbb8229bb, SUC_lexicon_tags_2 },
    { 0xb4e229bb, SUC_lexicon_tags_47 },
    { 0x294229bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e229c0, SUC_lexicon_tags_37 },
    { 0x860229c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e229c3, SUC_lexicon_tags_43 },
    { 0x86a229c3, SUC_lexicon_tags_14 },
    { 0xa36229c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394229d1, SUC_lexicon_tags_43 },
    { 0xdbc229d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50229d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4229d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38229df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x594229e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0229e5, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8229e8, SUC_lexicon_tags_11 },
    { 0x7ca229e9, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xf90229eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4229fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14222a00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f222a02, SUC_lexicon_tags_22 },
    { 0xe0222a02, SUC_lexicon_tags_28 },
    { 0xcee22a04, SUC_lexicon_tags_12 },
    { 0xc622a04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a22a0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf222a0f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b422a16, SUC_lexicon_tags_8 },
    { 0xd7c22a16, SUC_lexicon_tags_45 },
    { 0xe822a18, SUC_lexicon_tags_0 },
    { 0xd1822a18, SUC_lexicon_tags_17 },
    { 0x7a22a18, SUC_lexicon_tags_11 },
    { 0x3e022a1b, SUC_lexicon_tags_25 },
    { 0xb7422a1a, SUC_lexicon_tags_17 },
    { 0x47022a17, SUC_lexicon_tags_60 },
    { 0x42222a1e, SUC_lexicon_tags_158 },
    { 0x95a22a1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e22a24, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x60822a26, SUC_lexicon_tags_4 },
    { 0x2cc22a27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c22a2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf622a2d, SUC_lexicon_tags_28 },
    { 0x1e622a2d, SUC_lexicon_tags_63 },
    { 0xc1822a2e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97622a31, SUC_lexicon_tags_17 },
    { 0xba022a32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff022a36, SUC_lexicon_tags_49 },
    { 0xb6022a37, SUC_lexicon_tags_7 },
    { 0xb622a37, SUC_lexicon_tags_19 },
    { 0xaca22a39, SUC_lexicon_tags_11 },
    { 0x97222a39, SUC_lexicon_tags_11 },
    { 0xef622a3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1a22a3d, SUC_lexicon_tags_3 },
    { 0x24c22a3d, SUC_lexicon_tags_14 },
    { 0x7d822a3f, SUC_lexicon_tags_79 },
    { 0x44022a40, SUC_lexicon_tags_7 },
    { 0x5a822a3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ae22a43, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89422a48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16222a4a, SUC_lexicon_tags_49 },
    { 0x9ac22a4b, SUC_lexicon_tags_37 },
    { 0x96a22a4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48422a51, SUC_lexicon_tags_68 },
    { 0x38a22a52, SUC_lexicon_tags_10 },
    { 0x6e822a53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f422a57, SUC_lexicon_tags_12 },
    { 0x22422a57, SUC_lexicon_tags_17 },
    { 0xa3c22a59, SUC_lexicon_tags_1 },
    { 0x50c22a58, SUC_lexicon_tags_5 },
    { 0x1fa22a5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81422a5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a22a60, SUC_lexicon_tags_27 },
    { 0xdca22a61, SUC_lexicon_tags_11 },
    { 0xa2022a61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22e22a64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec22a67, SUC_lexicon_tags_33 },
    { 0x17222a67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72622a6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8222a71, SUC_lexicon_tags_45 },
    { 0xc2022a72, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0xe4022a74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda622a79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5022a7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde622a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e22a84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a22a87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a622a8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac22a94, SUC_lexicon_tags_518 },
    { 0x17622a94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76222a98, SUC_lexicon_tags_8 },
    { 0x79e22a99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49222a9b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c22a9f, SUC_lexicon_tags_10 },
    { 0xc422aa0, SUC_lexicon_tags_43 },
    { 0x91222a9f, SUC_lexicon_tags_1301 },
    { 0xd3222aa2, SUC_lexicon_tags_14 },
    { 0x7e22aa3, SUC_lexicon_tags_30 },
    { 0xa5022a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e822aa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5022aad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c22ab5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x89622ab7, SUC_lexicon_tags_7 },
    { 0xca422ab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1822abc, SUC_lexicon_tags_23 },
    { 0xc3e22abc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c22ac1, SUC_lexicon_tags_11 },
    { 0x74c22ac1, SUC_lexicon_tags_2 },
    { 0xff422ac3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7e22ac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a22ac9, SUC_lexicon_tags_29 },
    { 0xd6022aca, SUC_lexicon_tags_45 },
    { 0x9cc22acb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c822ad3, SUC_lexicon_tags_35 },
    { 0x78222ad3, SUC_lexicon_tags_39 },
    { 0x40a22ad5, SUC_lexicon_tags_30 },
    { 0x9b222ad6, SUC_lexicon_tags_11 },
    { 0xdc022ad7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb222ada, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45022ae0, SUC_lexicon_tags_182 },
    { 0xc9622ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20622ae4, SUC_lexicon_tags_11 },
    { 0xc8822ae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ec22ae7, SUC_lexicon_tags_38 },
    { 0xe0e22ae8, SUC_lexicon_tags_16 },
    { 0x64822ae9, SUC_lexicon_tags_31 },
    { 0xac022ae9, SUC_lexicon_tags_11 },
    { 0x2d822aea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa22af2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x32022af5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc22b05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0022b07, SUC_lexicon_tags_28 },
    { 0x9b422b08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c022b12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d822b14, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc1a22b16, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba222b20, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f222b29, SUC_lexicon_tags_15 },
    { 0xa2e22b2a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8d422b2c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c22b2f, SUC_lexicon_tags_33 },
    { 0xa8c22b2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43822b33, SUC_lexicon_tags_11 },
    { 0x27222b34, SUC_lexicon_tags_2 },
    { 0xd3422b35, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc3422b37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d422b3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c22b3e, SUC_lexicon_tags_35 },
    { 0x5d022b3f, SUC_lexicon_tags_68 },
    { 0x95822b3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5022b42, SUC_lexicon_tags_31 },
    { 0x50222b43, SUC_lexicon_tags_13 },
    { 0xe1622b44, SUC_lexicon_tags_6 },
    { 0xf0222b45, SUC_lexicon_tags_2 },
    { 0x54022b46, SUC_lexicon_tags_14 },
    { 0x1e822b46, SUC_lexicon_tags_25 },
    { 0xeec22b45, SUC_lexicon_tags_47 },
    { 0x13a22b47, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a22b52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x17222b55, SUC_lexicon_tags_39 },
    { 0x16a22b56, SUC_lexicon_tags_25 },
    { 0x81222b57, SUC_lexicon_tags_15 },
    { 0x69222b57, SUC_lexicon_tags_8 },
    { 0xa4a22b56, SUC_lexicon_tags_15 },
    { 0x1ce22b5a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac22b5e, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74622b66, SUC_lexicon_tags_7 },
    { 0x26822b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c022b6a, SUC_lexicon_tags_47 },
    { 0x39022b6a, SUC_lexicon_tags_25 },
    { 0x30822b6a, SUC_lexicon_tags_25 },
    { 0xa0e22b6c, SUC_lexicon_tags_15 },
    { 0xddc22b6e, SUC_lexicon_tags_29 },
    { 0x98e22b6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88622b73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49822b7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd622b7c, SUC_lexicon_tags_34 },
    { 0x24e22b7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa22b81, SUC_lexicon_tags_7 },
    { 0x4c22b82, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xea822b84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20222b8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd622b8d, SUC_lexicon_tags_39 },
    { 0x2022b8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3222b91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f622b95, SUC_lexicon_tags_17 },
    { 0x7e822b96, SUC_lexicon_tags_68 },
    { 0xa8422b96, SUC_lexicon_tags_10 },
    { 0x57e22b98, SUC_lexicon_tags_7 },
    { 0x39422b99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a22b9d, SUC_lexicon_tags_33 },
    { 0xe3022b9e, SUC_lexicon_tags_63 },
    { 0x33622b9d, SUC_lexicon_tags_15 },
    { 0xf022b9d, SUC_lexicon_tags_14 },
    { 0x95822ba1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4e22ba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb622ba5, SUC_lexicon_tags_11 },
    { 0xd3422ba6, SUC_lexicon_tags_87 },
    { 0x9f622ba7, SUC_lexicon_tags_27 },
    { 0x55c22ba7, SUC_lexicon_tags_38 },
    { 0x3a422ba9, SUC_lexicon_tags_21 },
    { 0x9f622ba9, SUC_lexicon_tags_37 },
    { 0x77c22bab, SUC_lexicon_tags_0 },
    { 0x3d022bac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1822bb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2b822bb2, SUC_lexicon_tags_17 },
    { 0x71a22bb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e22bb6, SUC_lexicon_tags_11 },
    { 0x1e022bb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e22bbb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7822bbf, SUC_lexicon_tags_10 },
    { 0xe6e22bc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7822bc2, SUC_lexicon_tags_42 },
    { 0x4b422bc3, SUC_lexicon_tags_37 },
    { 0xce622bc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2e22bc6, SUC_lexicon_tags_55 },
    { 0xdca22bc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0222bcd, SUC_lexicon_tags_134 },
    { 0xa022bce, SUC_lexicon_tags_33 },
    { 0xb5222bce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ee22bd1, SUC_lexicon_tags_1 },
    { 0xf6e22bd2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x54022bd5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf2622bd7, SUC_lexicon_tags_7 },
    { 0xdf022bd7, SUC_lexicon_tags_12 },
    { 0x7822bd9, SUC_lexicon_tags_26 },
    { 0x9d822bd7, SUC_lexicon_tags_34 },
    { 0x6c222bdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf022bdd, SUC_lexicon_tags_17 },
    { 0xc2622bdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec22be2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c22be5, SUC_lexicon_tags_2 },
    { 0xe9622be5, SUC_lexicon_tags_7 },
    { 0xa1a22be7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a22bea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfac22bec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2622bf2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa22bf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f022bf7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c622bfa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2022bfd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c22c03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e22c0c, SUC_lexicon_tags_61 },
    { 0x62c22c0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42e22c0f, SUC_lexicon_tags_25 },
    { 0xe3622c0f, SUC_lexicon_tags_12 },
    { 0x11a22c11, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e22c15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8422c17, SUC_lexicon_tags_60 },
    { 0x17022c17, SUC_lexicon_tags_37 },
    { 0x4ba22c17, SUC_lexicon_tags_63 },
    { 0x8ea22c1a, SUC_lexicon_tags_8 },
    { 0x63622c19, SUC_lexicon_tags_17 },
    { 0x1a022c1c, SUC_lexicon_tags_158 },
    { 0xc5a22c1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde622c20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d622c23, SUC_lexicon_tags_12 },
    { 0x69622c23, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xf5822c26, SUC_lexicon_tags_2 },
    { 0xec822c26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41822c2d, SUC_lexicon_tags_27 },
    { 0x19c22c2e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc9022c30, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec22c35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a22c3a, SUC_lexicon_tags_11 },
    { 0x57822c3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2222c3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6622c41, SUC_lexicon_tags_1 },
    { 0xc0c22c42, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8022c46, SUC_lexicon_tags_25 },
    { 0xac022c46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99022c49, SUC_lexicon_tags_65 },
    { 0x8e622c49, SUC_lexicon_tags_15 },
    { 0x8cc22c4b, SUC_lexicon_tags_34 },
    { 0x14c22c4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95e22c4e, SUC_lexicon_tags_37 },
    { 0xa6c22c4f, SUC_lexicon_tags_28 },
    { 0xfec22c4f, SUC_lexicon_tags_4 },
    { 0x6b822c51, SUC_lexicon_tags_15 },
    { 0x83022c50, SUC_lexicon_tags_19 },
    { 0xbfa22c53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da22c5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28c22c60, SUC_lexicon_tags_37 },
    { 0xd3e22c61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6e22c63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x59622c65, SUC_lexicon_tags_37 },
    { 0x88e22c66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16622c6a, SUC_lexicon_tags_43 },
    { 0xfb822c6b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcb822c6d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f222c71, SUC_lexicon_tags_63 },
    { 0x8d822c71, SUC_lexicon_tags_9 },
    { 0xac222c73, SUC_lexicon_tags_32 },
    { 0x23622c74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd422c80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f422c82, SUC_lexicon_tags_33 },
    { 0xe2222c83, SUC_lexicon_tags_87 },
    { 0xa3822c84, SUC_lexicon_tags_11 },
    { 0x61e22c85, SUC_lexicon_tags_11 },
    { 0xbe022c85, SUC_lexicon_tags_12 },
    { 0x8bc22c87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe622c8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x94822c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6a22c91, SUC_lexicon_tags_25 },
    { 0x2bc22c92, SUC_lexicon_tags_21 },
    { 0xcf422c93, SUC_lexicon_tags_33 },
    { 0x7b422c94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5222c99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4422c9d, SUC_lexicon_tags_12 },
    { 0x4822c9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79622ca2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e22ca5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcca22ca7, SUC_lexicon_tags_27 },
    { 0x79622ca8, SUC_lexicon_tags_18 },
    { 0x38c22ca9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b422cad, SUC_lexicon_tags_49 },
    { 0x1a622cae, SUC_lexicon_tags_30 },
    { 0x61822caf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfa822cb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81222cbe, SUC_lexicon_tags_0 },
    { 0xda822cbf, SUC_lexicon_tags_25 },
    { 0xc5822cbf, SUC_lexicon_tags_31 },
    { 0x9c222cc1, SUC_lexicon_tags_7 },
    { 0x99622cbe, SUC_lexicon_tags_140 },
    { 0xa8022cc3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfa822cc5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb022cc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6822ccf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c22cd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcba22cd8, SUC_lexicon_tags_7 },
    { 0xc5a22cd8, SUC_lexicon_tags_49 },
    { 0xd5c22cd9, SUC_lexicon_tags_10 },
    { 0x66822cdb, SUC_lexicon_tags_25 },
    { 0x8da22cdc, SUC_lexicon_tags_13 },
    { 0x1ae22cdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6622ce0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee622ce2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53022ce5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae22cea, SUC_lexicon_tags_2 },
    { 0x4aa22ceb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa22cef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9222cf1, SUC_lexicon_tags_23 },
    { 0x61822cf2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf3022cf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95e22cf6, SUC_lexicon_tags_23 },
    { 0xbd222cf7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f622cfa, SUC_lexicon_tags_21 },
    { 0x10a22cfb, SUC_lexicon_tags_34 },
    { 0xa6622cfb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3622cff, SUC_lexicon_tags_27 },
    { 0x14422d00, SUC_lexicon_tags_23 },
    { 0x8e622d00, SUC_lexicon_tags_16 },
    { 0x66222d02, SUC_lexicon_tags_15 },
    { 0xff422d02, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x89222d06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c022d08, SUC_lexicon_tags_5 },
    { 0x2f622d09, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x89e22d0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b422d0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca422d11, SUC_lexicon_tags_2 },
    { 0xa22d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21022d14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67622d17, SUC_lexicon_tags_45 },
    { 0x6c422d18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e22d1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66022d20, SUC_lexicon_tags_37 },
    { 0xf0c22d21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8422d24, SUC_lexicon_tags_2 },
    { 0x1fe22d25, SUC_lexicon_tags_43 },
    { 0xae622d26, SUC_lexicon_tags_2 },
    { 0xcf822d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50622d2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21022d2e, SUC_lexicon_tags_87 },
    { 0x4cc22d2f, SUC_lexicon_tags_12 },
    { 0x3c022d30, SUC_lexicon_tags_75 },
    { 0x1422d31, SUC_lexicon_tags_11 },
    { 0x83422d2f, SUC_lexicon_tags_32 },
    { 0x70a22d33, SUC_lexicon_tags_29 },
    { 0x2e22d34, SUC_lexicon_tags_14 },
    { 0x6ee22d2f, SUC_lexicon_tags_30 },
    { 0xe4a22d36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24c22d38, SUC_lexicon_tags_33 },
    { 0x5a622d39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5422d3d, SUC_lexicon_tags_65 },
    { 0x6d222d3d, SUC_lexicon_tags_33 },
    { 0xf2a22d3f, SUC_lexicon_tags_1 },
    { 0xe2222d3d, SUC_lexicon_tags_38 },
    { 0x1ec22d3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde022d44, SUC_lexicon_tags_12 },
    { 0x52022d45, SUC_lexicon_tags_8 },
    { 0x63422d46, SUC_lexicon_tags_37 },
    { 0x69a22d44, SUC_lexicon_tags_11 },
    { 0xcce22d48, SUC_lexicon_tags_11 },
    { 0x39e22d49, SUC_lexicon_tags_63 },
    { 0xc5822d49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49822d4f, SUC_lexicon_tags_17 },
    { 0xdb822d50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a022d53, SUC_lexicon_tags_32 },
    { 0xc9222d53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa22d57, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xada22d5a, SUC_lexicon_tags_38 },
    { 0x59a22d5b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x62222d5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95422d60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e22d63, SUC_lexicon_tags_23 },
    { 0x26022d64, SUC_lexicon_tags_17 },
    { 0x68022d65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76022d67, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1ea22d69, SUC_lexicon_tags_1 },
    { 0xf1422d69, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4622d6d, SUC_lexicon_tags_17 },
    { 0xe0822d6e, SUC_lexicon_tags_30 },
    { 0x68c22d6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89622d74, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7422d7b, SUC_lexicon_tags_31 },
    { 0xa0022d7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xec022d7f, SUC_lexicon_tags_27 },
    { 0x20c22d7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75022d84, SUC_lexicon_tags_2 },
    { 0x3f622d85, SUC_lexicon_tags_30 },
    { 0xc9022d86, SUC_lexicon_tags_37 },
    { 0x99622d87, SUC_lexicon_tags_60 },
    { 0x85422d87, SUC_lexicon_tags_15 },
    { 0x5e222d89, SUC_lexicon_tags_11 },
    { 0x22422d88, SUC_lexicon_tags_33 },
    { 0x37822d8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c22d8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x65822d91, SUC_lexicon_tags_79 },
    { 0x43c22d92, SUC_lexicon_tags_11 },
    { 0x1ea22d93, SUC_lexicon_tags_7 },
    { 0xa4822d94, SUC_lexicon_tags_25 },
    { 0x5ec22d91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1822d9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc5022d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcea22da2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68822da4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4222da6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70022da9, SUC_lexicon_tags_9 },
    { 0x86622daa, SUC_lexicon_tags_7 },
    { 0x48622daa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1422dae, SUC_lexicon_tags_11 },
    { 0x52022dae, SUC_lexicon_tags_17 },
    { 0xad422db0, SUC_lexicon_tags_44 },
    { 0xbee22db0, SUC_lexicon_tags_19 },
    { 0xa9e22db2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86622dbd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1422dc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c22dc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9622dcb, SUC_lexicon_tags_29 },
    { 0xfbc22dcb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32022dd3, SUC_lexicon_tags_27 },
    { 0x7622dd4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee22dd8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x34222dda, SUC_lexicon_tags_17 },
    { 0x39622dda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc22dde, SUC_lexicon_tags_38 },
    { 0x2ea22ddf, SUC_lexicon_tags_19 },
    { 0x2f822de0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c22de7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e422de9, SUC_lexicon_tags_23 },
    { 0x80822dea, SUC_lexicon_tags_37 },
    { 0x75c22de9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4422ded, SUC_lexicon_tags_15 },
    { 0xa5c22dee, SUC_lexicon_tags_23 },
    { 0xc6222dee, SUC_lexicon_tags_14 },
    { 0xb8e22df0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d022df2, SUC_lexicon_tags_63 },
    { 0x43e22df3, SUC_lexicon_tags_63 },
    { 0x44022df3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca022dfa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9022dfc, SUC_lexicon_tags_15 },
    { 0x26422dfd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e22e00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c022e05, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c422e0b, SUC_lexicon_tags_0 },
    { 0x5bc22e0c, SUC_lexicon_tags_25 },
    { 0xc8422e0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a22e10, SUC_lexicon_tags_2 },
    { 0x6b022e11, SUC_lexicon_tags_104 },
    { 0xad422e12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43c22e14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x47622e17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf822e1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcd422e1c, SUC_lexicon_tags_38 },
    { 0x55222e1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90622e23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c22e27, SUC_lexicon_tags_32 },
    { 0x8b622e27, SUC_lexicon_tags_17 },
    { 0xb1822e28, SUC_lexicon_tags_7 },
    { 0x7ae22e27, SUC_lexicon_tags_17 },
    { 0x11222e2b, SUC_lexicon_tags_11 },
    { 0x6f422e29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f222e2e, SUC_lexicon_tags_87 },
    { 0xd9e22e2e, SUC_lexicon_tags_64 },
    { 0x84022e2f, SUC_lexicon_tags_53 },
    { 0xfda22e31, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x76622e33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87622e36, SUC_lexicon_tags_37 },
    { 0xf1622e37, SUC_lexicon_tags_27 },
    { 0xa2422e38, SUC_lexicon_tags_14 },
    { 0x34222e39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0822e3b, SUC_lexicon_tags_26 },
    { 0xfa022e3c, SUC_lexicon_tags_43 },
    { 0xbbc22e3c, SUC_lexicon_tags_114 },
    { 0x91a22e3e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1622e48, SUC_lexicon_tags_25 },
    { 0x82e22e49, SUC_lexicon_tags_32 },
    { 0x12222e4a, SUC_lexicon_tags_56 },
    { 0xaac22e4b, SUC_lexicon_tags_1 },
    { 0xd2422e49, SUC_lexicon_tags_0 },
    { 0x1a22e4d, SUC_lexicon_tags_23 },
    { 0x84422e4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c22e54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde22e5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8822e62, SUC_lexicon_tags_160 },
    { 0x4e222e62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e22e66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70222e6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72622e73, SUC_lexicon_tags_11 },
    { 0x63e22e74, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c22e76, SUC_lexicon_tags_0 },
    { 0x63e22e76, SUC_lexicon_tags_37 },
    { 0xe0222e78, SUC_lexicon_tags_19 },
    { 0x50622e78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfea22e7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43222e7f, SUC_lexicon_tags_35 },
    { 0xc22e7f, SUC_lexicon_tags_43 },
    { 0x35222e7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c22e85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e22e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff222e8b, SUC_lexicon_tags_17 },
    { 0x7bc22e8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa22e8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32822e91, SUC_lexicon_tags_7 },
    { 0xf2e22e92, SUC_lexicon_tags_5 },
    { 0x45c22e92, SUC_lexicon_tags_17 },
    { 0x3da22e93, SUC_lexicon_tags_11 },
    { 0x62222e95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee22e99, SUC_lexicon_tags_25 },
    { 0x31422e9a, SUC_lexicon_tags_7 },
    { 0xe0e22e9b, SUC_lexicon_tags_1 },
    { 0x38a22e9c, SUC_lexicon_tags_3 },
    { 0xd7e22e99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de22ea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83222ea2, SUC_lexicon_tags_8 },
    { 0xf1e22ea2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9022ea6, SUC_lexicon_tags_17 },
    { 0xc2422ea7, SUC_lexicon_tags_68 },
    { 0xf5e22ea8, SUC_lexicon_tags_15 },
    { 0x91e22ea8, SUC_lexicon_tags_54 },
    { 0xa22ea8, SUC_lexicon_tags_38 },
    { 0x78e22eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34422eaf, SUC_lexicon_tags_1 },
    { 0x1f222eb0, SUC_lexicon_tags_47 },
    { 0x4e22eb0, SUC_lexicon_tags_6 },
    { 0x76622eb2, SUC_lexicon_tags_26 },
    { 0x14222eb1, SUC_lexicon_tags_107 },
    { 0x31422eb0, SUC_lexicon_tags_17 },
    { 0x90c22eb4, SUC_lexicon_tags_63 },
    { 0x16422eb0, SUC_lexicon_tags_4 },
    { 0xd8822eb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3f222eb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1222ebb, SUC_lexicon_tags_47 },
    { 0x67622ebc, SUC_lexicon_tags_12 },
    { 0x6fa22ebd, SUC_lexicon_tags_37 },
    { 0x94222ebd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79222ec3, SUC_lexicon_tags_31 },
    { 0xd0822ec3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41022ec6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7022eca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c822ecc, SUC_lexicon_tags_7 },
    { 0xba22ecc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93222ecf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1622ed2, SUC_lexicon_tags_33 },
    { 0x48c22ed3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc22ed7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c22edd, SUC_lexicon_tags_29 },
    { 0x43222ede, SUC_lexicon_tags_17 },
    { 0x9a422edf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b622ee5, SUC_lexicon_tags_66 },
    { 0x1e422ee6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb022ee8, SUC_lexicon_tags_37 },
    { 0x2c422ee9, SUC_lexicon_tags_43 },
    { 0x1e822eea, SUC_lexicon_tags_37 },
    { 0x62022eeb, SUC_lexicon_tags_11 },
    { 0x95222ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3822ef0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3222ef2, SUC_lexicon_tags_37 },
    { 0xbce22ef3, SUC_lexicon_tags_26 },
    { 0x30c22ef4, SUC_lexicon_tags_3 },
    { 0xc3222ef5, SUC_lexicon_tags_12 },
    { 0xfce22ef6, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x80422ef8, SUC_lexicon_tags_47 },
    { 0x5d022ef8, SUC_lexicon_tags_12 },
    { 0xfc222efa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xada22efc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee022f00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e22f0a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31622f0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a22f11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde22f14, SUC_lexicon_tags_12 },
    { 0x89222f15, SUC_lexicon_tags_9 },
    { 0x61622f16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94622f19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6222f1c, SUC_lexicon_tags_31 },
    { 0xcca22f1d, SUC_lexicon_tags_201 },
    { 0xcb622f1e, SUC_lexicon_tags_14 },
    { 0xa8022f1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36222f23, SUC_lexicon_tags_37 },
    { 0x5dc22f24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f222f26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf622f28, SUC_lexicon_tags_8 },
    { 0xa0622f29, SUC_lexicon_tags_37 },
    { 0x70222f2a, SUC_lexicon_tags_29 },
    { 0xf9622f29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e22f31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84e22f33, SUC_lexicon_tags_17 },
    { 0x2a22f34, SUC_lexicon_tags_33 },
    { 0xcc022f35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4222f38, SUC_lexicon_tags_63 },
    { 0x71222f38, SUC_lexicon_tags_230 },
    { 0x95022f3a, SUC_lexicon_tags_2 },
    { 0xbd222f3b, SUC_lexicon_tags_15 },
    { 0x12222f39, SUC_lexicon_tags_25 },
    { 0xc9822f3d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x99622f3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7622f48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8422f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46022f4c, SUC_lexicon_tags_37 },
    { 0x60622f4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e22f53, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a22f58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e622f5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35a22f5d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de22f61, SUC_lexicon_tags_37 },
    { 0x53622f62, SUC_lexicon_tags_68 },
    { 0x88822f63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f222f6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44a22f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e22f72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a22f77, SUC_lexicon_tags_27 },
    { 0xd5422f77, SUC_lexicon_tags_11 },
    { 0x8b022f77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e22f7d, SUC_lexicon_tags_2 },
    { 0xf6c22f7e, SUC_lexicon_tags_15 },
    { 0xcbe22f7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39822f81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec22f84, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a222f87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21822f89, SUC_lexicon_tags_15 },
    { 0x2cc22f8a, SUC_lexicon_tags_37 },
    { 0xb9c22f8a, SUC_lexicon_tags_15 },
    { 0xf8022f8c, SUC_lexicon_tags_7 },
    { 0x39622f8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c222f92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66622f94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b822f98, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x57022f9a, SUC_lexicon_tags_31 },
    { 0x6ce22f9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5622f9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d422fa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc622fa4, SUC_lexicon_tags_66 },
    { 0x33822fa5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x93422fa8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xaf222faa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31822fac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8c22fae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xeea22fb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6822fb2, SUC_lexicon_tags_27 },
    { 0x85422fb2, SUC_lexicon_tags_15 },
    { 0x75222fb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12822fb6, SUC_lexicon_tags_23 },
    { 0x2ec22fb6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44a22fb9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x63222fbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d822fbf, SUC_lexicon_tags_1 },
    { 0x63422fbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39a22fc2, SUC_lexicon_tags_17 },
    { 0xd6022fc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe22fcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac22fce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60022fd2, SUC_lexicon_tags_12 },
    { 0x6b422fd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0e22fd5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a22fdb, SUC_lexicon_tags_25 },
    { 0x6dc22fdc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec22fdf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3022fe4, SUC_lexicon_tags_12 },
    { 0xf2c22fe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e022fe9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4422fec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5022fef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a422ff3, SUC_lexicon_tags_6 },
    { 0xb3c22ff3, SUC_lexicon_tags_1 },
    { 0xa822ff4, SUC_lexicon_tags_37 },
    { 0x8ba22ff4, SUC_lexicon_tags_17 },
    { 0x8ca22ff3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc222ff9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xbde22ffb, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x3f822ffd, SUC_lexicon_tags_11 },
    { 0x2da22ffd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5823000, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca23004, SUC_lexicon_tags_15 },
    { 0x30e23004, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa423007, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4b023009, SUC_lexicon_tags_1 },
    { 0x8062300a, SUC_lexicon_tags_15 },
    { 0x9e22300b, SUC_lexicon_tags_7 },
    { 0x1c62300b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac2300f, SUC_lexicon_tags_0 },
    { 0x2b423010, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18023012, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24023015, SUC_lexicon_tags_8 },
    { 0xec823016, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x31223018, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xd582301b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf062301d, SUC_lexicon_tags_14 },
    { 0xf782301e, SUC_lexicon_tags_82 },
    { 0x5e62301e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c23025, SUC_lexicon_tags_63 },
    { 0x25023026, SUC_lexicon_tags_7 },
    { 0xc3823027, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9a423029, SUC_lexicon_tags_37 },
    { 0x8cc2302a, SUC_lexicon_tags_2 },
    { 0xfa23029, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe522302e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3823030, SUC_lexicon_tags_17 },
    { 0xf3823030, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23034, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x19623037, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7c023039, SUC_lexicon_tags_767 },
    { 0, NULL },
    { 0x1cc2303b, SUC_lexicon_tags_25 },
    { 0xfe22303c, SUC_lexicon_tags_14 },
    { 0x8a62303d, SUC_lexicon_tags_32 },
    { 0x5002303c, SUC_lexicon_tags_37 },
    { 0xb982303f, SUC_lexicon_tags_17 },
    { 0x8982303d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7cc23042, SUC_lexicon_tags_4 },
    { 0x59e23042, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e823047, SUC_lexicon_tags_4 },
    { 0x76423047, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a2304c, SUC_lexicon_tags_12 },
    { 0x86c2304c, SUC_lexicon_tags_4 },
    { 0x31a2304e, SUC_lexicon_tags_2 },
    { 0x4e42304e, SUC_lexicon_tags_8 },
    { 0x3a623050, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c623054, SUC_lexicon_tags_30 },
    { 0x16423054, SUC_lexicon_tags_12 },
    { 0xd9023056, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c23058, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ec2305a, SUC_lexicon_tags_1 },
    { 0x33e2305b, SUC_lexicon_tags_88 },
    { 0xc42305c, SUC_lexicon_tags_17 },
    { 0xb8a2305d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c23062, SUC_lexicon_tags_80 },
    { 0x27223063, SUC_lexicon_tags_37 },
    { 0x9d023063, SUC_lexicon_tags_43 },
    { 0x55823063, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3862306b, SUC_lexicon_tags_29 },
    { 0x64e2306c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7e22306e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81223076, SUC_lexicon_tags_15 },
    { 0xab423077, SUC_lexicon_tags_25 },
    { 0x5b623078, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1fa2307a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa02307f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4023083, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x42023086, SUC_lexicon_tags_31 },
    { 0xc6a23086, SUC_lexicon_tags_29 },
    { 0xa8823086, SUC_lexicon_tags_17 },
    { 0xeb223089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a42308c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xea82308f, SUC_lexicon_tags_46 },
    { 0x39223090, SUC_lexicon_tags_7 },
    { 0x62e23091, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c023094, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee23099, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe542309d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0230a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66a230a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2230a7, SUC_lexicon_tags_15 },
    { 0x3fc230a7, SUC_lexicon_tags_19 },
    { 0x586230a9, SUC_lexicon_tags_45 },
    { 0x10230aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12c230ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04230b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79c230b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x886230b5, SUC_lexicon_tags_47 },
    { 0xec8230b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa38230b8, SUC_lexicon_tags_40 },
    { 0x954230b8, SUC_lexicon_tags_11 },
    { 0x92230b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4230c1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a230c5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8230c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf0e230ca, SUC_lexicon_tags_14 },
    { 0xd7e230ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2230ce, SUC_lexicon_tags_15 },
    { 0xa2230cf, SUC_lexicon_tags_47 },
    { 0x9ba230d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e230d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x182230d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x938230d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x832230db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30230df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6230f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28230fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x654230fd, SUC_lexicon_tags_26 },
    { 0xc78230fe, SUC_lexicon_tags_9 },
    { 0x766230fe, SUC_lexicon_tags_23 },
    { 0xbc823100, SUC_lexicon_tags_48 },
    { 0xbc823100, SUC_lexicon_tags_37 },
    { 0x3c423100, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b023107, SUC_lexicon_tags_33 },
    { 0x9b423107, SUC_lexicon_tags_0 },
    { 0x25823109, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9482310d, SUC_lexicon_tags_7 },
    { 0xec2310e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e423113, SUC_lexicon_tags_1 },
    { 0x7023113, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd42311b, SUC_lexicon_tags_70 },
    { 0x5202311c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6842311e, SUC_lexicon_tags_1 },
    { 0x2fe2311f, SUC_lexicon_tags_12 },
    { 0xb6423120, SUC_lexicon_tags_23 },
    { 0xa0223121, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0823123, SUC_lexicon_tags_37 },
    { 0x20a23124, SUC_lexicon_tags_12 },
    { 0x8de23124, SUC_lexicon_tags_0 },
    { 0xa0e23124, SUC_lexicon_tags_45 },
    { 0x19423125, SUC_lexicon_tags_33 },
    { 0x50023127, SUC_lexicon_tags_35 },
    { 0xaea23129, SUC_lexicon_tags_11 },
    { 0xccc2312a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc82312d, SUC_lexicon_tags_19 },
    { 0xee2312e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfb023130, SUC_lexicon_tags_37 },
    { 0xbd623131, SUC_lexicon_tags_37 },
    { 0xbb223132, SUC_lexicon_tags_13 },
    { 0x3de23133, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa4623135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8423137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2702313c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6702313f, SUC_lexicon_tags_65 },
    { 0x8562313f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4fe23142, SUC_lexicon_tags_37 },
    { 0x90c23142, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb22314e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x55623150, SUC_lexicon_tags_2 },
    { 0x8d023150, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc23154, SUC_lexicon_tags_16 },
    { 0xdb623155, SUC_lexicon_tags_11 },
    { 0xe5e23156, SUC_lexicon_tags_8 },
    { 0xbfe23157, SUC_lexicon_tags_25 },
    { 0xf2023158, SUC_lexicon_tags_17 },
    { 0xf4c23159, SUC_lexicon_tags_30 },
    { 0xe2a23154, SUC_lexicon_tags_29 },
    { 0xbf42315b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40623161, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf223169, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8142316b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c2316e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a23172, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a23175, SUC_lexicon_tags_0 },
    { 0x6cc23176, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x30a23178, SUC_lexicon_tags_34 },
    { 0x1e23179, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e2317f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e23182, SUC_lexicon_tags_10 },
    { 0x1e823183, SUC_lexicon_tags_60 },
    { 0x3f423184, SUC_lexicon_tags_17 },
    { 0x4e823184, SUC_lexicon_tags_60 },
    { 0x78423183, SUC_lexicon_tags_1 },
    { 0x18823187, SUC_lexicon_tags_82 },
    { 0x9b423188, SUC_lexicon_tags_25 },
    { 0x3a423187, SUC_lexicon_tags_15 },
    { 0xa9423189, SUC_lexicon_tags_14 },
    { 0x3862318b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee23192, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d623197, SUC_lexicon_tags_11 },
    { 0x40823198, SUC_lexicon_tags_37 },
    { 0xace23197, SUC_lexicon_tags_23 },
    { 0x5ca2319a, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b22319d, SUC_lexicon_tags_11 },
    { 0x11e2319e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0231aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x310231ac, SUC_lexicon_tags_65 },
    { 0x1be231ac, SUC_lexicon_tags_37 },
    { 0x28c231ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2f2231b0, SUC_lexicon_tags_30 },
    { 0xe6c231b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88231b6, SUC_lexicon_tags_15 },
    { 0x8d8231b6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6231bd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988231c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c0231c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684231ce, SUC_lexicon_tags_3 },
    { 0x15c231cf, SUC_lexicon_tags_61 },
    { 0xd8e231d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256231d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x498231d7, SUC_lexicon_tags_0 },
    { 0x216231d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84231df, SUC_lexicon_tags_14 },
    { 0x6b2231e0, SUC_lexicon_tags_37 },
    { 0xce2231e1, SUC_lexicon_tags_127 },
    { 0xf10231df, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x74a231e4, SUC_lexicon_tags_93 },
    { 0xe36231e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16a231e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc231eb, SUC_lexicon_tags_35 },
    { 0xdea231ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a231f4, SUC_lexicon_tags_7 },
    { 0xce2231f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc231f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36231ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44a23201, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70823209, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7522320c, SUC_lexicon_tags_91 },
    { 0x9f22320d, SUC_lexicon_tags_38 },
    { 0x51e2320e, SUC_lexicon_tags_12 },
    { 0x93e2320f, SUC_lexicon_tags_11 },
    { 0xb2023210, SUC_lexicon_tags_25 },
    { 0xc7623211, SUC_lexicon_tags_96 },
    { 0xa3c2320e, SUC_lexicon_tags_138 },
    { 0x3682320e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c23215, SUC_lexicon_tags_7 },
    { 0x72e23215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46023218, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e2321b, SUC_lexicon_tags_1 },
    { 0xc7a2321c, SUC_lexicon_tags_14 },
    { 0xc922321c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8b82321f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20a23221, SUC_lexicon_tags_0 },
    { 0x4dc23222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc23225, SUC_lexicon_tags_25 },
    { 0x91a23226, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc542322b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x75e2322d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1b42322f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6023232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11a23234, SUC_lexicon_tags_1335 },
    { 0x7b023235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95e23237, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe3423239, SUC_lexicon_tags_12 },
    { 0x81a2323a, SUC_lexicon_tags_27 },
    { 0x77223239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa82323f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8023243, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29623245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c2324b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5662324f, SUC_lexicon_tags_27 },
    { 0xc482324f, SUC_lexicon_tags_11 },
    { 0x7823251, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2cc23253, SUC_lexicon_tags_7 },
    { 0xab823254, SUC_lexicon_tags_28 },
    { 0x67e23254, SUC_lexicon_tags_11 },
    { 0x63c23254, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e2325d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c423262, SUC_lexicon_tags_7 },
    { 0x48e23263, SUC_lexicon_tags_47 },
    { 0x4f823263, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82623266, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6f423268, SUC_lexicon_tags_26 },
    { 0x75423269, SUC_lexicon_tags_25 },
    { 0x5ac23269, SUC_lexicon_tags_1 },
    { 0xaea2326b, SUC_lexicon_tags_60 },
    { 0xe6a2326a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3dc2326e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc023270, SUC_lexicon_tags_7 },
    { 0xb2623271, SUC_lexicon_tags_17 },
    { 0xbdc23272, SUC_lexicon_tags_2 },
    { 0x25623272, SUC_lexicon_tags_27 },
    { 0x75e23273, SUC_lexicon_tags_31 },
    { 0xc7823275, SUC_lexicon_tags_43 },
    { 0x4da23274, SUC_lexicon_tags_34 },
    { 0xc6423277, SUC_lexicon_tags_0 },
    { 0x2f623278, SUC_lexicon_tags_17 },
    { 0x8a823278, SUC_lexicon_tags_1 },
    { 0xc902327a, SUC_lexicon_tags_11 },
    { 0x7842327b, SUC_lexicon_tags_250 },
    { 0xd002327a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5fa2327e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xedc23280, SUC_lexicon_tags_1 },
    { 0x7a23281, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31023285, SUC_lexicon_tags_253 },
    { 0xe0c23286, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e23289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3842328b, SUC_lexicon_tags_0 },
    { 0x97e2328b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8162328e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7823291, SUC_lexicon_tags_37 },
    { 0xe5423291, SUC_lexicon_tags_60 },
    { 0x8f023292, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c2329c, SUC_lexicon_tags_12 },
    { 0x3b02329d, SUC_lexicon_tags_26 },
    { 0x9da2329e, SUC_lexicon_tags_32 },
    { 0x3922329c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f4232a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8232a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4a232ab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da232b3, SUC_lexicon_tags_37 },
    { 0x60232b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x812232b7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c232ba, SUC_lexicon_tags_33 },
    { 0xe7c232bb, SUC_lexicon_tags_38 },
    { 0x2bc232bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1ae232be, SUC_lexicon_tags_34 },
    { 0x418232bf, SUC_lexicon_tags_6 },
    { 0xb66232c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2232c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e2232c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312232c9, SUC_lexicon_tags_27 },
    { 0xea6232ca, SUC_lexicon_tags_0 },
    { 0x390232cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2232d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa232d3, SUC_lexicon_tags_23 },
    { 0x68e232d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7c232d6, SUC_lexicon_tags_17 },
    { 0x548232d7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec232db, SUC_lexicon_tags_0 },
    { 0x7f8232dc, SUC_lexicon_tags_27 },
    { 0x83a232dd, SUC_lexicon_tags_7 },
    { 0xdfa232dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98232e4, SUC_lexicon_tags_61 },
    { 0xed8232e5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x528232e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c232eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x724232ed, SUC_lexicon_tags_2 },
    { 0xc40232ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc232f3, SUC_lexicon_tags_23 },
    { 0xa96232f3, SUC_lexicon_tags_47 },
    { 0xd30232f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8a0232f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x962232fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838232fe, SUC_lexicon_tags_25 },
    { 0xb4232ff, SUC_lexicon_tags_0 },
    { 0x67e232ff, SUC_lexicon_tags_11 },
    { 0xdb223301, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40a23303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69823308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42330d, SUC_lexicon_tags_23 },
    { 0x29e2330e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a23317, SUC_lexicon_tags_6 },
    { 0xbe623318, SUC_lexicon_tags_0 },
    { 0x70423317, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3ba2331b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece23323, SUC_lexicon_tags_47 },
    { 0x6f023324, SUC_lexicon_tags_27 },
    { 0xbec23325, SUC_lexicon_tags_23 },
    { 0x6e423326, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x44223329, SUC_lexicon_tags_27 },
    { 0x3ce23329, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd022332c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ba2332e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33623330, SUC_lexicon_tags_47 },
    { 0xfe823331, SUC_lexicon_tags_0 },
    { 0x94823332, SUC_lexicon_tags_2 },
    { 0xb6423332, SUC_lexicon_tags_1 },
    { 0x50a23333, SUC_lexicon_tags_25 },
    { 0x13423334, SUC_lexicon_tags_12 },
    { 0xcb423336, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf062333a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf42333c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf6e2333e, SUC_lexicon_tags_17 },
    { 0x3c02333f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3223342, SUC_lexicon_tags_26 },
    { 0x81223343, SUC_lexicon_tags_11 },
    { 0x90623343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da23347, SUC_lexicon_tags_42 },
    { 0x64423348, SUC_lexicon_tags_16 },
    { 0x82423349, SUC_lexicon_tags_343 },
    { 0x52623347, SUC_lexicon_tags_3 },
    { 0x69423347, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7822334f, SUC_lexicon_tags_38 },
    { 0xd4423350, SUC_lexicon_tags_15 },
    { 0xe8823351, SUC_lexicon_tags_38 },
    { 0x55c23350, SUC_lexicon_tags_21 },
    { 0x4d023353, SUC_lexicon_tags_47 },
    { 0x11223354, SUC_lexicon_tags_25 },
    { 0x22423355, SUC_lexicon_tags_929 },
    { 0x98023356, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7202335c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2462335e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1623361, SUC_lexicon_tags_33 },
    { 0x9b823362, SUC_lexicon_tags_15 },
    { 0x9e023361, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25623365, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x65223367, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89023369, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7623373, SUC_lexicon_tags_2 },
    { 0x1e023374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26623378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x122337b, SUC_lexicon_tags_15 },
    { 0xd362337c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb02337e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17823382, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be2338b, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e02338f, SUC_lexicon_tags_49 },
    { 0x35823390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3223393, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x78023395, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4902339c, SUC_lexicon_tags_25 },
    { 0x2f82339d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc222339f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x610233a1, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c233a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c233a8, SUC_lexicon_tags_65 },
    { 0xfce233a9, SUC_lexicon_tags_14 },
    { 0x600233a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1e233ac, SUC_lexicon_tags_2 },
    { 0x706233ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54233b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0233b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984233bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef2233bd, SUC_lexicon_tags_2 },
    { 0x550233be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e233c3, SUC_lexicon_tags_37 },
    { 0xb2e233c4, SUC_lexicon_tags_17 },
    { 0x57c233c3, SUC_lexicon_tags_15 },
    { 0x848233c6, SUC_lexicon_tags_3 },
    { 0xfee233c4, SUC_lexicon_tags_33 },
    { 0x5f0233c8, SUC_lexicon_tags_17 },
    { 0xcaa233c9, SUC_lexicon_tags_17 },
    { 0x348233c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc233cd, SUC_lexicon_tags_33 },
    { 0xe26233ce, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e233d2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266233d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10233d9, SUC_lexicon_tags_10 },
    { 0xd78233d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e233dd, SUC_lexicon_tags_68 },
    { 0x9e4233de, SUC_lexicon_tags_5 },
    { 0xb98233df, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a233e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a233e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x898233ea, SUC_lexicon_tags_273 },
    { 0, NULL },
    { 0xd50233ec, SUC_lexicon_tags_11 },
    { 0xebe233ec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460233f1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa233fb, SUC_lexicon_tags_31 },
    { 0x300233fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2233ff, SUC_lexicon_tags_17 },
    { 0x5ee233ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7962340c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3982340e, SUC_lexicon_tags_7 },
    { 0x5622340f, SUC_lexicon_tags_10 },
    { 0x5fc2340f, SUC_lexicon_tags_14 },
    { 0xe0823410, SUC_lexicon_tags_65 },
    { 0x59423412, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c23415, SUC_lexicon_tags_17 },
    { 0x70423415, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e23419, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa942341b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd42341d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c23424, SUC_lexicon_tags_6 },
    { 0x51823425, SUC_lexicon_tags_8 },
    { 0x3c23426, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59423428, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2962342a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2482342c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32c2342e, SUC_lexicon_tags_27 },
    { 0x782342e, SUC_lexicon_tags_37 },
    { 0xa0023430, SUC_lexicon_tags_31 },
    { 0xf5a2342f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba823433, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e23439, SUC_lexicon_tags_8 },
    { 0x8ce2343a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e23443, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x14823446, SUC_lexicon_tags_17 },
    { 0xa5223446, SUC_lexicon_tags_1 },
    { 0xf9423447, SUC_lexicon_tags_25 },
    { 0x1623449, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba23452, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x22623454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c23459, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb302345d, SUC_lexicon_tags_23 },
    { 0xc502345e, SUC_lexicon_tags_84 },
    { 0x9ca2345f, SUC_lexicon_tags_320 },
    { 0x21623460, SUC_lexicon_tags_37 },
    { 0x75623461, SUC_lexicon_tags_1 },
    { 0x9f423462, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15423466, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce2346b, SUC_lexicon_tags_25 },
    { 0xad42346c, SUC_lexicon_tags_7 },
    { 0xf682346d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf82346f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26223471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a23473, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28023475, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3242347f, SUC_lexicon_tags_2 },
    { 0x9d823480, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb423482, SUC_lexicon_tags_45 },
    { 0xfc823483, SUC_lexicon_tags_37 },
    { 0x6a823483, SUC_lexicon_tags_11 },
    { 0xd8a23485, SUC_lexicon_tags_1 },
    { 0xda023483, SUC_lexicon_tags_92 },
    { 0xf2223487, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8fa23489, SUC_lexicon_tags_2 },
    { 0x9b82348a, SUC_lexicon_tags_38 },
    { 0x20c2348b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e02348f, SUC_lexicon_tags_15 },
    { 0xf1423490, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25223495, SUC_lexicon_tags_0 },
    { 0x60223495, SUC_lexicon_tags_0 },
    { 0xa8823497, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56a23499, SUC_lexicon_tags_15 },
    { 0xd322349a, SUC_lexicon_tags_37 },
    { 0x1582349b, SUC_lexicon_tags_9 },
    { 0x7982349c, SUC_lexicon_tags_47 },
    { 0x1b42349d, SUC_lexicon_tags_0 },
    { 0xb0a2349e, SUC_lexicon_tags_2 },
    { 0x4582349f, SUC_lexicon_tags_0 },
    { 0xb2e234a0, SUC_lexicon_tags_11 },
    { 0x6ec2349b, SUC_lexicon_tags_43 },
    { 0xfb8234a2, SUC_lexicon_tags_31 },
    { 0xbca234a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598234ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44234af, SUC_lexicon_tags_12 },
    { 0x516234af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860234b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74234b8, SUC_lexicon_tags_25 },
    { 0xa6e234b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x68c234bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e234c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x904234c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39e234c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06234ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226234cf, SUC_lexicon_tags_60 },
    { 0x232234cf, SUC_lexicon_tags_46 },
    { 0x7aa234d1, SUC_lexicon_tags_49 },
    { 0x75c234d2, SUC_lexicon_tags_2 },
    { 0x8e8234cf, SUC_lexicon_tags_31 },
    { 0xee6234d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c234db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c234de, SUC_lexicon_tags_11 },
    { 0x36234de, SUC_lexicon_tags_47 },
    { 0xa5a234e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x508234e3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48234e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96234e9, SUC_lexicon_tags_15 },
    { 0xdd8234ea, SUC_lexicon_tags_19 },
    { 0xd48234eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8a234ed, SUC_lexicon_tags_12 },
    { 0x78e234ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6234f2, SUC_lexicon_tags_49 },
    { 0xef8234f2, SUC_lexicon_tags_9 },
    { 0x86234f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2234f9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20234fc, SUC_lexicon_tags_47 },
    { 0x72e234fd, SUC_lexicon_tags_33 },
    { 0x4f8234fe, SUC_lexicon_tags_43 },
    { 0xd38234ff, SUC_lexicon_tags_11 },
    { 0xc86234fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a23507, SUC_lexicon_tags_6 },
    { 0x26623508, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe382350a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76623515, SUC_lexicon_tags_12 },
    { 0x17023516, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9482351a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97423522, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e23527, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe222352a, SUC_lexicon_tags_31 },
    { 0xb222352b, SUC_lexicon_tags_37 },
    { 0x4cc2352c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90a2352e, SUC_lexicon_tags_12 },
    { 0x5702352e, SUC_lexicon_tags_26 },
    { 0xf3023530, SUC_lexicon_tags_46 },
    { 0x62423531, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee23535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed223539, SUC_lexicon_tags_55 },
    { 0x3b62353a, SUC_lexicon_tags_12 },
    { 0x7802353b, SUC_lexicon_tags_113 },
    { 0x93a2353c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59623543, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a23546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e2354b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a62354e, SUC_lexicon_tags_56 },
    { 0x93e2354e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e423551, SUC_lexicon_tags_26 },
    { 0x34023552, SUC_lexicon_tags_12 },
    { 0x82e23551, SUC_lexicon_tags_65 },
    { 0x65023553, SUC_lexicon_tags_11 },
    { 0x5be23555, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad02355d, SUC_lexicon_tags_87 },
    { 0xbe2355e, SUC_lexicon_tags_17 },
    { 0xff82355f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe7c23561, SUC_lexicon_tags_37 },
    { 0x33a23562, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab023567, SUC_lexicon_tags_37 },
    { 0x9623568, SUC_lexicon_tags_1 },
    { 0x94823567, SUC_lexicon_tags_149 },
    { 0x41e23568, SUC_lexicon_tags_37 },
    { 0x88c23568, SUC_lexicon_tags_0 },
    { 0x67a2356c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa722356e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a23572, SUC_lexicon_tags_17 },
    { 0xdde23572, SUC_lexicon_tags_66 },
    { 0x2c823574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e23580, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85423582, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe623590, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a623596, SUC_lexicon_tags_44 },
    { 0xa8e23597, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f62359a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cc2359c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70235a2, SUC_lexicon_tags_26 },
    { 0x9e4235a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c235a5, SUC_lexicon_tags_27 },
    { 0xbc4235a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98235a8, SUC_lexicon_tags_42 },
    { 0x8c8235a9, SUC_lexicon_tags_6 },
    { 0xabc235a9, SUC_lexicon_tags_12 },
    { 0x428235a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x300235ad, SUC_lexicon_tags_29 },
    { 0x50a235ad, SUC_lexicon_tags_15 },
    { 0xe1c235af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc235b3, SUC_lexicon_tags_25 },
    { 0x8dc235b4, SUC_lexicon_tags_17 },
    { 0xc44235b5, SUC_lexicon_tags_21 },
    { 0xb6a235b6, SUC_lexicon_tags_30 },
    { 0x458235b6, SUC_lexicon_tags_21 },
    { 0xbba235b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512235bd, SUC_lexicon_tags_23 },
    { 0x7d0235bd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16235c4, SUC_lexicon_tags_9 },
    { 0x1ec235c5, SUC_lexicon_tags_55 },
    { 0x5f8235c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb2e235c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a235cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe235d3, SUC_lexicon_tags_17 },
    { 0x7a0235d3, SUC_lexicon_tags_38 },
    { 0x1b8235d5, SUC_lexicon_tags_60 },
    { 0x600235d6, SUC_lexicon_tags_7 },
    { 0x12a235d4, SUC_lexicon_tags_12 },
    { 0x54235d3, SUC_lexicon_tags_25 },
    { 0x7ba235d9, SUC_lexicon_tags_39 },
    { 0xe50235da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x270235dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04235e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x726235e3, SUC_lexicon_tags_38 },
    { 0x89c235e4, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806235e8, SUC_lexicon_tags_57 },
    { 0x6aa235e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe1a235eb, SUC_lexicon_tags_45 },
    { 0x53c235ec, SUC_lexicon_tags_31 },
    { 0x92e235ed, SUC_lexicon_tags_38 },
    { 0x99a235ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8235f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842235f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42235fa, SUC_lexicon_tags_37 },
    { 0x44c235fb, SUC_lexicon_tags_63 },
    { 0x29c235fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42235ff, SUC_lexicon_tags_26 },
    { 0x10e235ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9423605, SUC_lexicon_tags_2 },
    { 0xe2823606, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a23609, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee2360d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac23613, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea23619, SUC_lexicon_tags_43 },
    { 0x34e2361a, SUC_lexicon_tags_25 },
    { 0x17623619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe402361f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62e23621, SUC_lexicon_tags_141 },
    { 0x70223622, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3223625, SUC_lexicon_tags_1 },
    { 0x6a023626, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49623628, SUC_lexicon_tags_5 },
    { 0x5f023629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d023630, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ca23632, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee23637, SUC_lexicon_tags_0 },
    { 0x65423638, SUC_lexicon_tags_106 },
    { 0x2c823638, SUC_lexicon_tags_7 },
    { 0x9c22363a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a2363d, SUC_lexicon_tags_1 },
    { 0x8fc2363e, SUC_lexicon_tags_32 },
    { 0x32e2363d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc23643, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a23647, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf142364c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21223653, SUC_lexicon_tags_37 },
    { 0x57c23654, SUC_lexicon_tags_2 },
    { 0x2d223653, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c023658, SUC_lexicon_tags_37 },
    { 0xc8223659, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4862365d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec223661, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c423664, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x88223666, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1402366d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a23670, SUC_lexicon_tags_5 },
    { 0x87023671, SUC_lexicon_tags_37 },
    { 0x68423671, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x623675, SUC_lexicon_tags_7 },
    { 0x97e23676, SUC_lexicon_tags_12 },
    { 0x49e23675, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec2367b, SUC_lexicon_tags_28 },
    { 0xabe2367b, SUC_lexicon_tags_10 },
    { 0x75e2367d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x53823680, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77223686, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d02368f, SUC_lexicon_tags_60 },
    { 0x4e2368f, SUC_lexicon_tags_10 },
    { 0xd62368f, SUC_lexicon_tags_38 },
    { 0x5c2368f, SUC_lexicon_tags_4 },
    { 0x3f023693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0623696, SUC_lexicon_tags_82 },
    { 0xbea23697, SUC_lexicon_tags_17 },
    { 0x6fa23697, SUC_lexicon_tags_11 },
    { 0x5d623699, SUC_lexicon_tags_1 },
    { 0xdae2369a, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac236a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522236a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27e236a9, SUC_lexicon_tags_38 },
    { 0x666236a9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x696236ac, SUC_lexicon_tags_56 },
    { 0x7d4236ad, SUC_lexicon_tags_37 },
    { 0xfcc236ae, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x56c236b0, SUC_lexicon_tags_65 },
    { 0xb24236b1, SUC_lexicon_tags_47 },
    { 0xeb2236b1, SUC_lexicon_tags_38 },
    { 0x200236b2, SUC_lexicon_tags_25 },
    { 0x62e236b4, SUC_lexicon_tags_37 },
    { 0xf52236b1, SUC_lexicon_tags_37 },
    { 0xfba236b4, SUC_lexicon_tags_66 },
    { 0x154236b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x78236b9, SUC_lexicon_tags_11 },
    { 0xece236ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x70236bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee236c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x226236c8, SUC_lexicon_tags_223 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c236cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90236cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe90236cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x980236d2, SUC_lexicon_tags_49 },
    { 0x434236d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e236d6, SUC_lexicon_tags_31 },
    { 0xe0e236d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96236db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa236de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x952236e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6236e7, SUC_lexicon_tags_15 },
    { 0x56a236e8, SUC_lexicon_tags_32 },
    { 0x180236e8, SUC_lexicon_tags_48 },
    { 0xc16236ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4236f3, SUC_lexicon_tags_1 },
    { 0xc0c236f3, SUC_lexicon_tags_386 },
    { 0x874236f5, SUC_lexicon_tags_11 },
    { 0x176236f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4236f9, SUC_lexicon_tags_37 },
    { 0xf3c236fa, SUC_lexicon_tags_63 },
    { 0x55e236fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8e236fd, SUC_lexicon_tags_8 },
    { 0x7de236fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2423703, SUC_lexicon_tags_25 },
    { 0x70223704, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68a23706, SUC_lexicon_tags_33 },
    { 0x69823707, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd562370c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c23712, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c23716, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2371d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5702371f, SUC_lexicon_tags_38 },
    { 0xc5c23720, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be23726, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2962372b, SUC_lexicon_tags_17 },
    { 0x4cc2372c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70823731, SUC_lexicon_tags_13 },
    { 0xd9a23731, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8ba23734, SUC_lexicon_tags_187 },
    { 0xc6023735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2373a, SUC_lexicon_tags_79 },
    { 0x3bc2373a, SUC_lexicon_tags_16 },
    { 0x8b82373b, SUC_lexicon_tags_15 },
    { 0x62a2373b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3022373f, SUC_lexicon_tags_37 },
    { 0x5423740, SUC_lexicon_tags_15 },
    { 0xcc623741, SUC_lexicon_tags_1 },
    { 0x98823742, SUC_lexicon_tags_8 },
    { 0x19a23743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf623745, SUC_lexicon_tags_37 },
    { 0x8b023745, SUC_lexicon_tags_26 },
    { 0xb0223747, SUC_lexicon_tags_45 },
    { 0xece23748, SUC_lexicon_tags_79 },
    { 0xd7e23749, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d42374c, SUC_lexicon_tags_98 },
    { 0x42e2374d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0823751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e23754, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b223759, SUC_lexicon_tags_46 },
    { 0xc322375a, SUC_lexicon_tags_1 },
    { 0xa302375b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5a2375d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9023761, SUC_lexicon_tags_17 },
    { 0x4ac23762, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27823766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb202376d, SUC_lexicon_tags_7 },
    { 0x7e22376d, SUC_lexicon_tags_25 },
    { 0xe242376f, SUC_lexicon_tags_47 },
    { 0x2562376f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x25023773, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd023776, SUC_lexicon_tags_11 },
    { 0x68a23777, SUC_lexicon_tags_19 },
    { 0xd823778, SUC_lexicon_tags_79 },
    { 0xf4023778, SUC_lexicon_tags_45 },
    { 0x3c22377a, SUC_lexicon_tags_1 },
    { 0x4642377b, SUC_lexicon_tags_25 },
    { 0xa1a2377b, SUC_lexicon_tags_48 },
    { 0xa042377d, SUC_lexicon_tags_47 },
    { 0x1402377e, SUC_lexicon_tags_49 },
    { 0x9e82377a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b423781, SUC_lexicon_tags_7 },
    { 0xc7c23782, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6623786, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ce23788, SUC_lexicon_tags_0 },
    { 0xd223788, SUC_lexicon_tags_7 },
    { 0x9fa2378a, SUC_lexicon_tags_49 },
    { 0x34e2378a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0a2378d, SUC_lexicon_tags_11 },
    { 0x3722378e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a23791, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa282379b, SUC_lexicon_tags_7 },
    { 0x2862379c, SUC_lexicon_tags_8 },
    { 0x8882379b, SUC_lexicon_tags_55 },
    { 0xef02379b, SUC_lexicon_tags_15 },
    { 0x802379b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330237a3, SUC_lexicon_tags_23 },
    { 0xc12237a4, SUC_lexicon_tags_1 },
    { 0x6da237a3, SUC_lexicon_tags_8 },
    { 0x648237a6, SUC_lexicon_tags_1 },
    { 0xe88237a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e237aa, SUC_lexicon_tags_25 },
    { 0x89c237aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be237af, SUC_lexicon_tags_17 },
    { 0x4d4237b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac237b4, SUC_lexicon_tags_151 },
    { 0xb92237b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0237b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfac237ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x820237bc, SUC_lexicon_tags_27 },
    { 0x79e237bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ce237bf, SUC_lexicon_tags_11 },
    { 0xce0237c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f8237c2, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x604237c5, SUC_lexicon_tags_33 },
    { 0x75e237c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e2237c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a237cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe237d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90237d7, SUC_lexicon_tags_14 },
    { 0xec0237d8, SUC_lexicon_tags_25 },
    { 0x684237d9, SUC_lexicon_tags_37 },
    { 0xcd0237d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36237de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2237e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274237e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8237eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a237f6, SUC_lexicon_tags_45 },
    { 0xa4e237f7, SUC_lexicon_tags_12 },
    { 0xb5a237f7, SUC_lexicon_tags_26 },
    { 0x6de237f9, SUC_lexicon_tags_25 },
    { 0x58237fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28237fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x72e23800, SUC_lexicon_tags_1 },
    { 0xc023801, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f02380a, SUC_lexicon_tags_9 },
    { 0x73a2380a, SUC_lexicon_tags_2 },
    { 0xd982380c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc582380f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a823811, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c23815, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e22381a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf023822, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd023826, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x25a23828, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e2382e, SUC_lexicon_tags_60 },
    { 0xcca2382e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb823832, SUC_lexicon_tags_47 },
    { 0xf3a23833, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe3223835, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7623838, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa222383b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae23841, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce23848, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3742384b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf622384e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xce23850, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a23854, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68223857, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd82385a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2623862, SUC_lexicon_tags_27 },
    { 0x9e023862, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7623866, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e2386a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7882386e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xab223871, SUC_lexicon_tags_25 },
    { 0x3a223872, SUC_lexicon_tags_21 },
    { 0x3c023873, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e2387a, SUC_lexicon_tags_60 },
    { 0x80a2387b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2de2387d, SUC_lexicon_tags_11 },
    { 0x80e2387d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5023881, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5823884, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c823887, SUC_lexicon_tags_14 },
    { 0x91223888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca2388b, SUC_lexicon_tags_27 },
    { 0x1262388c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e023890, SUC_lexicon_tags_6 },
    { 0x15423891, SUC_lexicon_tags_31 },
    { 0x1f423892, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x22423894, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a23897, SUC_lexicon_tags_7 },
    { 0xe623898, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0238a2, SUC_lexicon_tags_104 },
    { 0x2ae238a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642238a7, SUC_lexicon_tags_47 },
    { 0x76238a7, SUC_lexicon_tags_82 },
    { 0xbf8238a8, SUC_lexicon_tags_37 },
    { 0x846238a7, SUC_lexicon_tags_48 },
    { 0xeec238a7, SUC_lexicon_tags_9 },
    { 0x3d6238aa, SUC_lexicon_tags_40 },
    { 0x3fc238ad, SUC_lexicon_tags_11 },
    { 0x624238ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820238b4, SUC_lexicon_tags_15 },
    { 0xd64238b4, SUC_lexicon_tags_2 },
    { 0xa0a238b5, SUC_lexicon_tags_2 },
    { 0xefa238b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d4238b9, SUC_lexicon_tags_10 },
    { 0x57a238ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726238be, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2238c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b6238c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x664238c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc12238c8, SUC_lexicon_tags_1 },
    { 0xe8238c8, SUC_lexicon_tags_17 },
    { 0x758238ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762238ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0238d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6b4238d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6238d7, SUC_lexicon_tags_15 },
    { 0xa58238d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x436238da, SUC_lexicon_tags_11 },
    { 0x2be238da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2238e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a238e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66238e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbf0238eb, SUC_lexicon_tags_7 },
    { 0x4e4238eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7c238ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbf0238f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x704238f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8238f8, SUC_lexicon_tags_34 },
    { 0x6d2238f9, SUC_lexicon_tags_47 },
    { 0xede238f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4238fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52023902, SUC_lexicon_tags_11 },
    { 0x2d423903, SUC_lexicon_tags_17 },
    { 0xace23904, SUC_lexicon_tags_26 },
    { 0xc5223905, SUC_lexicon_tags_23 },
    { 0xf5e23905, SUC_lexicon_tags_13 },
    { 0x60423907, SUC_lexicon_tags_7 },
    { 0x91e23904, SUC_lexicon_tags_57 },
    { 0xb1023902, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc22390c, SUC_lexicon_tags_12 },
    { 0x2602390d, SUC_lexicon_tags_23 },
    { 0x5302390e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31a23910, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x20a23912, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe223914, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c23917, SUC_lexicon_tags_61 },
    { 0x2c623917, SUC_lexicon_tags_15 },
    { 0xad423919, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1922391b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b62391d, SUC_lexicon_tags_49 },
    { 0xd6c2391e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87223922, SUC_lexicon_tags_1 },
    { 0x31823923, SUC_lexicon_tags_21 },
    { 0xb9223922, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7a423926, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1823929, SUC_lexicon_tags_66 },
    { 0xc782392a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb823934, SUC_lexicon_tags_151 },
    { 0x2fa23935, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2393c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xab42393e, SUC_lexicon_tags_11 },
    { 0x41c2393e, SUC_lexicon_tags_27 },
    { 0x4782393f, SUC_lexicon_tags_2 },
    { 0x77a23940, SUC_lexicon_tags_60 },
    { 0x31223942, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2223944, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e23947, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc2394a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8202394c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a2394f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb1a23951, SUC_lexicon_tags_65 },
    { 0xf1423951, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c823954, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa9623956, SUC_lexicon_tags_37 },
    { 0x91a23957, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc122395d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a23962, SUC_lexicon_tags_72 },
    { 0xde023962, SUC_lexicon_tags_2 },
    { 0xf8823962, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a2396a, SUC_lexicon_tags_7 },
    { 0xdd62396b, SUC_lexicon_tags_15 },
    { 0x3c82396b, SUC_lexicon_tags_31 },
    { 0xb8a2396a, SUC_lexicon_tags_23 },
    { 0x6f22396a, SUC_lexicon_tags_16 },
    { 0x9d62396d, SUC_lexicon_tags_37 },
    { 0xa2223970, SUC_lexicon_tags_25 },
    { 0xfa823971, SUC_lexicon_tags_87 },
    { 0x70223971, SUC_lexicon_tags_15 },
    { 0xea823972, SUC_lexicon_tags_27 },
    { 0xa3423974, SUC_lexicon_tags_37 },
    { 0x73e23975, SUC_lexicon_tags_11 },
    { 0x67823976, SUC_lexicon_tags_31 },
    { 0xc6a23977, SUC_lexicon_tags_10 },
    { 0xfd223977, SUC_lexicon_tags_2 },
    { 0x9e623979, SUC_lexicon_tags_7 },
    { 0x7c23978, SUC_lexicon_tags_25 },
    { 0x2823974, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c2397f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3023985, SUC_lexicon_tags_7 },
    { 0x5dc23985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21023989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c22398c, SUC_lexicon_tags_25 },
    { 0xf682398d, SUC_lexicon_tags_25 },
    { 0xd602398e, SUC_lexicon_tags_11 },
    { 0x3a2398f, SUC_lexicon_tags_7 },
    { 0xba823990, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83623994, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec2399c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6239a1, SUC_lexicon_tags_11 },
    { 0x8f6239a2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a239a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06239aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x206239ac, SUC_lexicon_tags_0 },
    { 0xb98239ad, SUC_lexicon_tags_12 },
    { 0xa8c239ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2239b1, SUC_lexicon_tags_37 },
    { 0xb5e239b1, SUC_lexicon_tags_14 },
    { 0x35c239b3, SUC_lexicon_tags_25 },
    { 0xd80239b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf98239b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432239bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace239c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5e6239c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a239cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x36a239cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x944239cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a239d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86239d7, SUC_lexicon_tags_37 },
    { 0x8ba239d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a239db, SUC_lexicon_tags_2 },
    { 0xa1a239dc, SUC_lexicon_tags_55 },
    { 0xdae239dc, SUC_lexicon_tags_1 },
    { 0xeee239de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc239e5, SUC_lexicon_tags_30 },
    { 0x6be239e5, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372239eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2e4239ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2239f0, SUC_lexicon_tags_11 },
    { 0x842239f1, SUC_lexicon_tags_11 },
    { 0xf36239f2, SUC_lexicon_tags_11 },
    { 0x7c239f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a239fa, SUC_lexicon_tags_647 },
    { 0x7f2239fb, SUC_lexicon_tags_82 },
    { 0x998239fc, SUC_lexicon_tags_43 },
    { 0xfce239fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9be239ff, SUC_lexicon_tags_25 },
    { 0xdbe23a00, SUC_lexicon_tags_26 },
    { 0x15823a01, SUC_lexicon_tags_62 },
    { 0x81a23a00, SUC_lexicon_tags_15 },
    { 0x9ec239ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce23a09, SUC_lexicon_tags_17 },
    { 0x47023a0a, SUC_lexicon_tags_25 },
    { 0x1e23a0a, SUC_lexicon_tags_12 },
    { 0x9da23a0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d223a0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a823a14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba023a18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b023a1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10e23a20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a23a25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc623a28, SUC_lexicon_tags_19 },
    { 0xba623a29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd623a2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94823a30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e623a35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe6823a37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be23a3d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x63e23a3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e23a45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9623a4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53423a4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18c23a51, SUC_lexicon_tags_15 },
    { 0x20c23a52, SUC_lexicon_tags_39 },
    { 0xb5a23a53, SUC_lexicon_tags_14 },
    { 0xc3a23a52, SUC_lexicon_tags_9 },
    { 0xc9223a55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e423a57, SUC_lexicon_tags_9 },
    { 0x5823a58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a823a5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f623a67, SUC_lexicon_tags_27 },
    { 0x94223a68, SUC_lexicon_tags_11 },
    { 0xd5423a69, SUC_lexicon_tags_11 },
    { 0x4423a6a, SUC_lexicon_tags_17 },
    { 0x13423a6a, SUC_lexicon_tags_11 },
    { 0xe2023a6a, SUC_lexicon_tags_43 },
    { 0xf4e23a6d, SUC_lexicon_tags_11 },
    { 0x49623a6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3623a75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc823a77, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbec23a79, SUC_lexicon_tags_26 },
    { 0x53023a7a, SUC_lexicon_tags_11 },
    { 0xf9623a7b, SUC_lexicon_tags_8 },
    { 0xcfc23a7c, SUC_lexicon_tags_48 },
    { 0x37623a79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc4823a7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40623a83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4223a87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83223a89, SUC_lexicon_tags_15 },
    { 0xa3023a8a, SUC_lexicon_tags_2 },
    { 0xdd823a8b, SUC_lexicon_tags_61 },
    { 0x79823a8b, SUC_lexicon_tags_30 },
    { 0x2fe23a8c, SUC_lexicon_tags_23 },
    { 0x8ce23a8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3223a92, SUC_lexicon_tags_15 },
    { 0x5b023a93, SUC_lexicon_tags_17 },
    { 0x38823a94, SUC_lexicon_tags_11 },
    { 0x70623a93, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba223a9b, SUC_lexicon_tags_31 },
    { 0xc8223a9c, SUC_lexicon_tags_2 },
    { 0xd7c23a9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c23aa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41223ab0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb823ab5, SUC_lexicon_tags_113 },
    { 0x15623ab5, SUC_lexicon_tags_10 },
    { 0x12623ab5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b423ab9, SUC_lexicon_tags_7 },
    { 0x3ee23aba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c23abd, SUC_lexicon_tags_45 },
    { 0xec423abd, SUC_lexicon_tags_9 },
    { 0xa4623abf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb023ac1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c23aca, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e23acd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8823ad0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26a23ad2, SUC_lexicon_tags_151 },
    { 0x97023ad3, SUC_lexicon_tags_0 },
    { 0xd3823ad2, SUC_lexicon_tags_12 },
    { 0x4ae23ad4, SUC_lexicon_tags_17 },
    { 0xbbc23ad6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x91823ad9, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d423add, SUC_lexicon_tags_11 },
    { 0xad223add, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x78423ae0, SUC_lexicon_tags_96 },
    { 0x95023ae1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a423ae5, SUC_lexicon_tags_25 },
    { 0xc1823ae5, SUC_lexicon_tags_82 },
    { 0x4d623ae7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a23aea, SUC_lexicon_tags_7 },
    { 0x4f623aea, SUC_lexicon_tags_13 },
    { 0x5cc23aec, SUC_lexicon_tags_12 },
    { 0xbbc23aed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b623aef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe0423af1, SUC_lexicon_tags_39 },
    { 0x8e423af1, SUC_lexicon_tags_17 },
    { 0xa5023af1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86623af6, SUC_lexicon_tags_15 },
    { 0x94a23af6, SUC_lexicon_tags_87 },
    { 0xeea23af8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde023afa, SUC_lexicon_tags_15 },
    { 0x9aa23afa, SUC_lexicon_tags_17 },
    { 0x78c23afb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c223afe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x89623b01, SUC_lexicon_tags_10 },
    { 0xed623b02, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e23b05, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a423b0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38223b13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e623b19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e23b1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f623b23, SUC_lexicon_tags_25 },
    { 0xeae23b23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88023b28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7023b2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc5023b2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c23b31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6823b37, SUC_lexicon_tags_11 },
    { 0x8d223b38, SUC_lexicon_tags_49 },
    { 0xc6a23b38, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe6223b3b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a223b3e, SUC_lexicon_tags_37 },
    { 0xd0223b3f, SUC_lexicon_tags_9 },
    { 0x18a23b3e, SUC_lexicon_tags_87 },
    { 0x88623b41, SUC_lexicon_tags_39 },
    { 0xd5623b42, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc23b45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x88623b48, SUC_lexicon_tags_12 },
    { 0xb8a23b49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fa23b4b, SUC_lexicon_tags_7 },
    { 0x94823b4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea823b50, SUC_lexicon_tags_31 },
    { 0xe2e23b51, SUC_lexicon_tags_12 },
    { 0x56a23b50, SUC_lexicon_tags_23 },
    { 0xac623b53, SUC_lexicon_tags_55 },
    { 0xaf223b53, SUC_lexicon_tags_15 },
    { 0xb1623b53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe23b5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57623b60, SUC_lexicon_tags_26 },
    { 0x8ec23b60, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5623b67, SUC_lexicon_tags_15 },
    { 0xc2e23b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c23b6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a23b72, SUC_lexicon_tags_12 },
    { 0x72823b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e23b76, SUC_lexicon_tags_27 },
    { 0xf9623b76, SUC_lexicon_tags_7 },
    { 0x26a23b78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2623b82, SUC_lexicon_tags_47 },
    { 0x16e23b83, SUC_lexicon_tags_7 },
    { 0xc3423b83, SUC_lexicon_tags_2 },
    { 0xf4223b85, SUC_lexicon_tags_14 },
    { 0xba423b83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a23b89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac223b8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a823b8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24023b94, SUC_lexicon_tags_5 },
    { 0x95223b95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a23b99, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1623b9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x43a23ba0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30e23ba2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xada23ba4, SUC_lexicon_tags_1 },
    { 0x89023ba5, SUC_lexicon_tags_0 },
    { 0xe7823ba6, SUC_lexicon_tags_13 },
    { 0x6fa23ba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc23baa, SUC_lexicon_tags_16 },
    { 0x1023bab, SUC_lexicon_tags_11 },
    { 0x7b423bab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8e423bae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a623bb1, SUC_lexicon_tags_31 },
    { 0x29223bb2, SUC_lexicon_tags_12 },
    { 0x97c23bb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed423bb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c023bbf, SUC_lexicon_tags_220 },
    { 0x92023bbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c23bc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8c23bc8, SUC_lexicon_tags_17 },
    { 0xb6e23bc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0823bce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x93223bd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae23bd7, SUC_lexicon_tags_46 },
    { 0xb8e23bd8, SUC_lexicon_tags_0 },
    { 0x85e23bd9, SUC_lexicon_tags_10 },
    { 0xa4023bd7, SUC_lexicon_tags_14 },
    { 0x9dc23bd7, SUC_lexicon_tags_105 },
    { 0xf8623bdc, SUC_lexicon_tags_8 },
    { 0x97a23bdd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f023bea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2d423bec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec23bf3, SUC_lexicon_tags_49 },
    { 0xbb423bf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a423bf7, SUC_lexicon_tags_23 },
    { 0x8ba23bf8, SUC_lexicon_tags_14 },
    { 0xfb423bf9, SUC_lexicon_tags_14 },
    { 0x55623bf8, SUC_lexicon_tags_33 },
    { 0x8de23bf9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc23bff, SUC_lexicon_tags_31 },
    { 0x57223c00, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x59823c02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc23c05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c23c0a, SUC_lexicon_tags_17 },
    { 0x91e23c0b, SUC_lexicon_tags_11 },
    { 0x73e23c0c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e23c0f, SUC_lexicon_tags_1 },
    { 0xea23c10, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc8c23c12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e23c17, SUC_lexicon_tags_10 },
    { 0x40623c18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc23c1c, SUC_lexicon_tags_23 },
    { 0x5a423c1d, SUC_lexicon_tags_12 },
    { 0x62c23c1c, SUC_lexicon_tags_38 },
    { 0x3023c1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88423c21, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf223c25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa9a23c27, SUC_lexicon_tags_27 },
    { 0xb6a23c28, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb3a23c2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3423c2e, SUC_lexicon_tags_7 },
    { 0x98823c2f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x2023c32, SUC_lexicon_tags_54 },
    { 0x28a23c33, SUC_lexicon_tags_7 },
    { 0x64823c34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f223c39, SUC_lexicon_tags_63 },
    { 0x29e23c39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4823c3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33423c3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ba23c40, SUC_lexicon_tags_11 },
    { 0xa5c23c40, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12423c47, SUC_lexicon_tags_15 },
    { 0xe1623c47, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68623c51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27e23c53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3823c56, SUC_lexicon_tags_60 },
    { 0x10a23c57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e23c5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb023c5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e23c62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f223c69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33223c6d, SUC_lexicon_tags_2 },
    { 0x1a23c6d, SUC_lexicon_tags_49 },
    { 0xf3223c6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7f623c71, SUC_lexicon_tags_9 },
    { 0xddc23c72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b223c76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c623c7a, SUC_lexicon_tags_15 },
    { 0x4b023c7b, SUC_lexicon_tags_29 },
    { 0x89223c7b, SUC_lexicon_tags_27 },
    { 0xab023c7c, SUC_lexicon_tags_55 },
    { 0xe7623c7e, SUC_lexicon_tags_15 },
    { 0x1f623c7e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa1823c81, SUC_lexicon_tags_31 },
    { 0x3ce23c82, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf5c23c84, SUC_lexicon_tags_0 },
    { 0xff823c85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9623c89, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x35423c8c, SUC_lexicon_tags_17 },
    { 0x7e423c8d, SUC_lexicon_tags_57 },
    { 0x7e23c8e, SUC_lexicon_tags_38 },
    { 0xef423c8f, SUC_lexicon_tags_1 },
    { 0x6b423c8d, SUC_lexicon_tags_2 },
    { 0x75e23c8f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec23c95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc623c9b, SUC_lexicon_tags_194 },
    { 0xeb423c9c, SUC_lexicon_tags_8 },
    { 0x3a023c9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd023ca0, SUC_lexicon_tags_60 },
    { 0xa8623ca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4823ca6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2223caa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42423cac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65e23cae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c23cb5, SUC_lexicon_tags_68 },
    { 0x94e23cb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9423cc1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc23cc7, SUC_lexicon_tags_17 },
    { 0x7a623cc7, SUC_lexicon_tags_11 },
    { 0xe7c23cc7, SUC_lexicon_tags_7 },
    { 0x83c23cca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe9823ccc, SUC_lexicon_tags_16 },
    { 0x37423ccc, SUC_lexicon_tags_6 },
    { 0xa3023cce, SUC_lexicon_tags_17 },
    { 0x83c23cce, SUC_lexicon_tags_124 },
    { 0x6ca23ccf, SUC_lexicon_tags_25 },
    { 0x5d823cce, SUC_lexicon_tags_11 },
    { 0xc9023cd2, SUC_lexicon_tags_16 },
    { 0x18823cd3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b623cd7, SUC_lexicon_tags_32 },
    { 0x78e23cd8, SUC_lexicon_tags_1 },
    { 0x9fc23cd7, SUC_lexicon_tags_37 },
    { 0x24e23cda, SUC_lexicon_tags_30 },
    { 0xab223cdb, SUC_lexicon_tags_7 },
    { 0xd1e23cdc, SUC_lexicon_tags_2 },
    { 0xe1a23cdd, SUC_lexicon_tags_737 },
    { 0x42e23cdb, SUC_lexicon_tags_15 },
    { 0xc3623cdf, SUC_lexicon_tags_37 },
    { 0xf5423ce0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc023ce2, SUC_lexicon_tags_54 },
    { 0xdaa23ce3, SUC_lexicon_tags_11 },
    { 0xace23ce3, SUC_lexicon_tags_66 },
    { 0x81023ce5, SUC_lexicon_tags_15 },
    { 0xcfc23ce6, SUC_lexicon_tags_29 },
    { 0x9a223ce4, SUC_lexicon_tags_47 },
    { 0xe023ce2, SUC_lexicon_tags_2 },
    { 0xd1023ce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20223ced, SUC_lexicon_tags_63 },
    { 0x93e23ced, SUC_lexicon_tags_2 },
    { 0x1223cef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e23cf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc23cf6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e23cfa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfc223cfc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x31c23cfe, SUC_lexicon_tags_29 },
    { 0x62a23cff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf223d01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd823d05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19423d07, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e023d0b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6823d0e, SUC_lexicon_tags_30 },
    { 0x8d023d0f, SUC_lexicon_tags_25 },
    { 0x51a23d10, SUC_lexicon_tags_15 },
    { 0x9a223d0f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e23d15, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65623d1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b423d20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fc23d22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf0e23d24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27423d26, SUC_lexicon_tags_55 },
    { 0xb8423d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb223d2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada23d2f, SUC_lexicon_tags_2 },
    { 0x24423d30, SUC_lexicon_tags_7 },
    { 0xdca23d31, SUC_lexicon_tags_11 },
    { 0x6e023d32, SUC_lexicon_tags_1 },
    { 0xa2823d32, SUC_lexicon_tags_17 },
    { 0xcd023d34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3223d39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e823d3c, SUC_lexicon_tags_15 },
    { 0x7023d3d, SUC_lexicon_tags_3 },
    { 0xb6023d3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9023d41, SUC_lexicon_tags_11 },
    { 0x7ce23d41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4223d46, SUC_lexicon_tags_11 },
    { 0x71c23d47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x83023d49, SUC_lexicon_tags_66 },
    { 0xb8823d4a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e223d4e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe4223d50, SUC_lexicon_tags_25 },
    { 0xde623d51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaf623d53, SUC_lexicon_tags_60 },
    { 0x49023d54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe23d60, SUC_lexicon_tags_46 },
    { 0xb4423d60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15623d63, SUC_lexicon_tags_15 },
    { 0x85423d64, SUC_lexicon_tags_6 },
    { 0x56623d65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44423d69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63423d6b, SUC_lexicon_tags_55 },
    { 0x69223d6b, SUC_lexicon_tags_12 },
    { 0x8a823d6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea23d71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75623d80, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x61223d83, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d023d89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8823d8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdd623d8d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c23d94, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7be23d96, SUC_lexicon_tags_3 },
    { 0x7e823d97, SUC_lexicon_tags_60 },
    { 0x15e23d98, SUC_lexicon_tags_2 },
    { 0x56e23d98, SUC_lexicon_tags_43 },
    { 0xaa623d99, SUC_lexicon_tags_23 },
    { 0x85e23d9b, SUC_lexicon_tags_25 },
    { 0xfb223d99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c423da0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb1223da2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63623da9, SUC_lexicon_tags_26 },
    { 0x38423da9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c623dae, SUC_lexicon_tags_47 },
    { 0xcea23dae, SUC_lexicon_tags_1 },
    { 0xc1223db0, SUC_lexicon_tags_3 },
    { 0xeba23db0, SUC_lexicon_tags_12 },
    { 0x60c23db2, SUC_lexicon_tags_57 },
    { 0xe9623db3, SUC_lexicon_tags_1 },
    { 0x55423db4, SUC_lexicon_tags_1 },
    { 0xb2423db0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3ee23db7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6b223db9, SUC_lexicon_tags_17 },
    { 0x87023dba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba423dbc, SUC_lexicon_tags_64 },
    { 0x2f023dbd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a223dc0, SUC_lexicon_tags_25 },
    { 0x69c23dc1, SUC_lexicon_tags_33 },
    { 0x6de23dc2, SUC_lexicon_tags_17 },
    { 0x6e423dc3, SUC_lexicon_tags_38 },
    { 0x59c23dc4, SUC_lexicon_tags_10 },
    { 0x35223dc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0023dc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d823dcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda23dd3, SUC_lexicon_tags_1 },
    { 0xa0c23dd3, SUC_lexicon_tags_17 },
    { 0xfa623dd5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c23dda, SUC_lexicon_tags_43 },
    { 0xe6223ddb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e23dde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5023de2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a423de5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be23de9, SUC_lexicon_tags_246 },
    { 0xf5e23dea, SUC_lexicon_tags_37 },
    { 0xd6223dea, SUC_lexicon_tags_65 },
    { 0x31a23dec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de23def, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f623df3, SUC_lexicon_tags_178 },
    { 0xce623df3, SUC_lexicon_tags_47 },
    { 0xd9e23df4, SUC_lexicon_tags_82 },
    { 0xaac23df5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x87223df8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e623dfd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc23e08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3423e13, SUC_lexicon_tags_70 },
    { 0xe0e23e14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa23e18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35623e1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f623e23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63423e28, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39023e2d, SUC_lexicon_tags_29 },
    { 0x7ac23e2e, SUC_lexicon_tags_26 },
    { 0x9d023e2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e823e32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4623e34, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a23e39, SUC_lexicon_tags_17 },
    { 0xba023e3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5c23e3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c23e3f, SUC_lexicon_tags_11 },
    { 0xc1c23e40, SUC_lexicon_tags_4 },
    { 0x5a023e40, SUC_lexicon_tags_77 },
    { 0xb7823e42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20823e46, SUC_lexicon_tags_25 },
    { 0xe2423e47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15423e4f, SUC_lexicon_tags_37 },
    { 0x90623e50, SUC_lexicon_tags_17 },
    { 0x66223e4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6223e53, SUC_lexicon_tags_6 },
    { 0x4b223e54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x69423e57, SUC_lexicon_tags_27 },
    { 0x9b823e58, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91823e5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b223e60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e23e63, SUC_lexicon_tags_65 },
    { 0x85223e63, SUC_lexicon_tags_20 },
    { 0xb2023e64, SUC_lexicon_tags_2 },
    { 0x8c223e65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9623e68, SUC_lexicon_tags_12 },
    { 0x25823e68, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f423e6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5023e70, SUC_lexicon_tags_38 },
    { 0x50223e71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda623e73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec023e77, SUC_lexicon_tags_30 },
    { 0x43e23e77, SUC_lexicon_tags_25 },
    { 0x1023e79, SUC_lexicon_tags_37 },
    { 0x4c823e7a, SUC_lexicon_tags_7 },
    { 0x23623e7b, SUC_lexicon_tags_6 },
    { 0x15c23e7c, SUC_lexicon_tags_14 },
    { 0x3c623e7a, SUC_lexicon_tags_17 },
    { 0x3c023e7e, SUC_lexicon_tags_37 },
    { 0xfa423e7e, SUC_lexicon_tags_38 },
    { 0xc2223e7a, SUC_lexicon_tags_46 },
    { 0x4a023e81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8623e83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c23e86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0823e89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7623e8c, SUC_lexicon_tags_7 },
    { 0x4b823e8d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a23e99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea623e9b, SUC_lexicon_tags_25 },
    { 0x76423e9b, SUC_lexicon_tags_1 },
    { 0x67a23e9d, SUC_lexicon_tags_2 },
    { 0xd5423e9e, SUC_lexicon_tags_7 },
    { 0x92823e9e, SUC_lexicon_tags_17 },
    { 0xa8423e9e, SUC_lexicon_tags_47 },
    { 0x6a223ea1, SUC_lexicon_tags_37 },
    { 0x8623ea2, SUC_lexicon_tags_37 },
    { 0xb5223ea3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e23ea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca223ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c23eac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a23eb0, SUC_lexicon_tags_29 },
    { 0xe4e23eb1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac23eb4, SUC_lexicon_tags_8 },
    { 0xc3023eb4, SUC_lexicon_tags_11 },
    { 0xd7223eb4, SUC_lexicon_tags_31 },
    { 0x8e423eb7, SUC_lexicon_tags_43 },
    { 0x24823eb4, SUC_lexicon_tags_2 },
    { 0x76423eb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec23ec4, SUC_lexicon_tags_23 },
    { 0xa823ec5, SUC_lexicon_tags_38 },
    { 0x2d423ec5, SUC_lexicon_tags_37 },
    { 0xf023ec7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2ba23ec9, SUC_lexicon_tags_46 },
    { 0x4e223eca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa7a23ecc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x25223ece, SUC_lexicon_tags_1 },
    { 0xac823ecf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56423ed1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8023ed6, SUC_lexicon_tags_2 },
    { 0xc3423ed7, SUC_lexicon_tags_1 },
    { 0x37223ed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b823edb, SUC_lexicon_tags_15 },
    { 0xc3c23edc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb023ee2, SUC_lexicon_tags_48 },
    { 0x99e23ee3, SUC_lexicon_tags_4 },
    { 0x91823ee4, SUC_lexicon_tags_7 },
    { 0x75423ee4, SUC_lexicon_tags_15 },
    { 0xa6223ee6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16423eed, SUC_lexicon_tags_65 },
    { 0xa8c23eee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c823ef0, SUC_lexicon_tags_434 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63823ef5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8623ef9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b223efc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a423f00, SUC_lexicon_tags_19 },
    { 0xeb023f00, SUC_lexicon_tags_17 },
    { 0x9c423f00, SUC_lexicon_tags_8 },
    { 0x27a23f03, SUC_lexicon_tags_70 },
    { 0xcd223f03, SUC_lexicon_tags_0 },
    { 0x1c223f00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c023f07, SUC_lexicon_tags_35 },
    { 0xe8623f08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81823f0f, SUC_lexicon_tags_14 },
    { 0x15c23f10, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x55e23f12, SUC_lexicon_tags_12 },
    { 0xd4a23f13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28023f15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c23f1a, SUC_lexicon_tags_37 },
    { 0x60c23f1a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc23f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1823f22, SUC_lexicon_tags_37 },
    { 0x1e823f22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8623f25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce423f27, SUC_lexicon_tags_34 },
    { 0x31c23f28, SUC_lexicon_tags_11 },
    { 0xdba23f29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4623f2d, SUC_lexicon_tags_17 },
    { 0x38823f2e, SUC_lexicon_tags_12 },
    { 0x5623f2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33023f34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c23f38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30223f3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6823f42, SUC_lexicon_tags_25 },
    { 0x96c23f42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58823f48, SUC_lexicon_tags_41 },
    { 0xaa823f49, SUC_lexicon_tags_37 },
    { 0xb6c23f4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93823f55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c23f5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea623f5f, SUC_lexicon_tags_11 },
    { 0xce023f5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba23f64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29a23f66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x41823f68, SUC_lexicon_tags_29 },
    { 0xb2023f69, SUC_lexicon_tags_390 },
    { 0x96623f68, SUC_lexicon_tags_25 },
    { 0xb0a23f6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5423f6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c23f71, SUC_lexicon_tags_33 },
    { 0x70623f72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0423f75, SUC_lexicon_tags_17 },
    { 0x6d623f76, SUC_lexicon_tags_38 },
    { 0x6f823f77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91023f83, SUC_lexicon_tags_31 },
    { 0x9a823f84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59423f86, SUC_lexicon_tags_23 },
    { 0xbcc23f86, SUC_lexicon_tags_68 },
    { 0x70623f88, SUC_lexicon_tags_11 },
    { 0x4223f89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d423f8d, SUC_lexicon_tags_25 },
    { 0xd023f8e, SUC_lexicon_tags_4 },
    { 0x56023f8f, SUC_lexicon_tags_70 },
    { 0xe7a23f8f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x1c423f92, SUC_lexicon_tags_11 },
    { 0xa9823f93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16c23f95, SUC_lexicon_tags_23 },
    { 0xeec23f96, SUC_lexicon_tags_47 },
    { 0x42223f97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe623f99, SUC_lexicon_tags_26 },
    { 0x24223f9a, SUC_lexicon_tags_37 },
    { 0xa4823f99, SUC_lexicon_tags_38 },
    { 0x89e23f9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8223fa5, SUC_lexicon_tags_49 },
    { 0x6be23fa5, SUC_lexicon_tags_25 },
    { 0x88823fa5, SUC_lexicon_tags_26 },
    { 0x1a623fa8, SUC_lexicon_tags_17 },
    { 0x6d023fa6, SUC_lexicon_tags_12 },
    { 0xb3223faa, SUC_lexicon_tags_29 },
    { 0x62023faa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81023fb5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a23fbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee23fbf, SUC_lexicon_tags_27 },
    { 0xfb023fc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b823fc4, SUC_lexicon_tags_68 },
    { 0x87a23fc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49423fce, SUC_lexicon_tags_77 },
    { 0x25223fce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdd223fd1, SUC_lexicon_tags_7 },
    { 0x23623fd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce23fd6, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96423fdc, SUC_lexicon_tags_33 },
    { 0xd3e23fdd, SUC_lexicon_tags_98 },
    { 0xa4623fde, SUC_lexicon_tags_15 },
    { 0x7423fdf, SUC_lexicon_tags_29 },
    { 0x12223fdd, SUC_lexicon_tags_47 },
    { 0x25023fe1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f623fe6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89223fee, SUC_lexicon_tags_45 },
    { 0xc4623fef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a23ff2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5423ff6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbcc23ff8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe6623ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8023ffc, SUC_lexicon_tags_2 },
    { 0x2623ffc, SUC_lexicon_tags_28 },
    { 0xf7c23ffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1824000, SUC_lexicon_tags_15 },
    { 0x62224000, SUC_lexicon_tags_28 },
    { 0x9bc24002, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6424009, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe24011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95224013, SUC_lexicon_tags_15 },
    { 0xb7024014, SUC_lexicon_tags_38 },
    { 0xac424015, SUC_lexicon_tags_15 },
    { 0x3ec24013, SUC_lexicon_tags_3 },
    { 0x33424017, SUC_lexicon_tags_87 },
    { 0xfe224018, SUC_lexicon_tags_37 },
    { 0x65a24019, SUC_lexicon_tags_9 },
    { 0xe4624014, SUC_lexicon_tags_29 },
    { 0x61624016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c2401e, SUC_lexicon_tags_25 },
    { 0x26e2401e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed424025, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36424027, SUC_lexicon_tags_15 },
    { 0xf0024028, SUC_lexicon_tags_25 },
    { 0x22424029, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d42402c, SUC_lexicon_tags_14 },
    { 0xf762402d, SUC_lexicon_tags_17 },
    { 0xe502402c, SUC_lexicon_tags_9 },
    { 0xb9e2402f, SUC_lexicon_tags_15 },
    { 0x78424030, SUC_lexicon_tags_12 },
    { 0x6702402d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e24034, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81224036, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf7424038, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3824040, SUC_lexicon_tags_3 },
    { 0x1a624041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e24045, SUC_lexicon_tags_23 },
    { 0xcac24046, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f02404a, SUC_lexicon_tags_37 },
    { 0x2ec2404b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722404f, SUC_lexicon_tags_26 },
    { 0x98e24050, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58024054, SUC_lexicon_tags_7 },
    { 0x4f824055, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc24058, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeea2405a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc862405e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c24066, SUC_lexicon_tags_48 },
    { 0x17e24067, SUC_lexicon_tags_1 },
    { 0x63a24068, SUC_lexicon_tags_72 },
    { 0xe2624069, SUC_lexicon_tags_25 },
    { 0x5b424068, SUC_lexicon_tags_33 },
    { 0x602406b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c2406f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3424076, SUC_lexicon_tags_37 },
    { 0x8b824077, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7182407e, SUC_lexicon_tags_2 },
    { 0x9c22407f, SUC_lexicon_tags_45 },
    { 0xe382407e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec24083, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c224088, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5602408e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e24096, SUC_lexicon_tags_37 },
    { 0x2ca24097, SUC_lexicon_tags_37 },
    { 0x44024098, SUC_lexicon_tags_17 },
    { 0x24224099, SUC_lexicon_tags_1 },
    { 0x18a2409a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce62409d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7f02409f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56a240a1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36240a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a240ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6240b1, SUC_lexicon_tags_17 },
    { 0x3e2240b2, SUC_lexicon_tags_21 },
    { 0xada240b3, SUC_lexicon_tags_7 },
    { 0xb80240b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450240b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00240bc, SUC_lexicon_tags_29 },
    { 0x156240bd, SUC_lexicon_tags_7 },
    { 0x996240bd, SUC_lexicon_tags_60 },
    { 0xc22240bf, SUC_lexicon_tags_56 },
    { 0x46c240c0, SUC_lexicon_tags_61 },
    { 0x766240c1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec240c9, SUC_lexicon_tags_6 },
    { 0xa74240ca, SUC_lexicon_tags_12 },
    { 0x218240ca, SUC_lexicon_tags_11 },
    { 0x7fe240cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4240d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x598240d2, SUC_lexicon_tags_8 },
    { 0xf36240d3, SUC_lexicon_tags_35 },
    { 0xa32240d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc240d7, SUC_lexicon_tags_34 },
    { 0x388240d7, SUC_lexicon_tags_23 },
    { 0xf88240d8, SUC_lexicon_tags_31 },
    { 0x31e240da, SUC_lexicon_tags_43 },
    { 0x5fe240db, SUC_lexicon_tags_19 },
    { 0x1de240da, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd94240de, SUC_lexicon_tags_69 },
    { 0xb68240de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde240e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2f2240e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xebe240e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ba240e9, SUC_lexicon_tags_87 },
    { 0x936240e9, SUC_lexicon_tags_29 },
    { 0x6c8240e9, SUC_lexicon_tags_6 },
    { 0x122240e9, SUC_lexicon_tags_10 },
    { 0x296240eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c240f2, SUC_lexicon_tags_25 },
    { 0x952240f3, SUC_lexicon_tags_29 },
    { 0xa38240f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe6240f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950240fa, SUC_lexicon_tags_2 },
    { 0xfe8240fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc424100, SUC_lexicon_tags_28 },
    { 0x3d624101, SUC_lexicon_tags_9 },
    { 0x7aa24101, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31824105, SUC_lexicon_tags_21 },
    { 0x16a24105, SUC_lexicon_tags_33 },
    { 0x4ae24105, SUC_lexicon_tags_29 },
    { 0x5a624105, SUC_lexicon_tags_28 },
    { 0x5024109, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xd542410b, SUC_lexicon_tags_17 },
    { 0x8c82410b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e24112, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c824115, SUC_lexicon_tags_17 },
    { 0x78024116, SUC_lexicon_tags_7 },
    { 0xf024117, SUC_lexicon_tags_2 },
    { 0x20c24116, SUC_lexicon_tags_1 },
    { 0xb2624119, SUC_lexicon_tags_37 },
    { 0x22a24119, SUC_lexicon_tags_14 },
    { 0xffe24119, SUC_lexicon_tags_7 },
    { 0xeca24118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f82411e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc424120, SUC_lexicon_tags_2 },
    { 0xc3624120, SUC_lexicon_tags_88 },
    { 0x10024120, SUC_lexicon_tags_32 },
    { 0x3d624123, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b424125, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x55824127, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c2412b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b42412e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c24133, SUC_lexicon_tags_25 },
    { 0x62024134, SUC_lexicon_tags_33 },
    { 0x1b824133, SUC_lexicon_tags_37 },
    { 0x8a624136, SUC_lexicon_tags_37 },
    { 0x37a24137, SUC_lexicon_tags_48 },
    { 0x98824135, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2413d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb302413f, SUC_lexicon_tags_17 },
    { 0xf3224140, SUC_lexicon_tags_19 },
    { 0x6b224141, SUC_lexicon_tags_15 },
    { 0xe024142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb624144, SUC_lexicon_tags_68 },
    { 0x1cc24145, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89c24147, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c024149, SUC_lexicon_tags_17 },
    { 0x6e02414a, SUC_lexicon_tags_45 },
    { 0xc7c2414a, SUC_lexicon_tags_7 },
    { 0xd862414c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e22414e, SUC_lexicon_tags_7 },
    { 0xb4a2414f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd024152, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d224157, SUC_lexicon_tags_11 },
    { 0x8f824158, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a2415d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8824165, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f024169, SUC_lexicon_tags_4 },
    { 0xfce2416a, SUC_lexicon_tags_37 },
    { 0xa5c2416b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e2416f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73e24171, SUC_lexicon_tags_66 },
    { 0x6de24172, SUC_lexicon_tags_11 },
    { 0xc2024173, SUC_lexicon_tags_0 },
    { 0x6ba24172, SUC_lexicon_tags_15 },
    { 0xd024174, SUC_lexicon_tags_60 },
    { 0x1ce24176, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc1824178, SUC_lexicon_tags_11 },
    { 0x9d024178, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5602417c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54c2417e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4424180, SUC_lexicon_tags_15 },
    { 0x26624181, SUC_lexicon_tags_6 },
    { 0x3c224181, SUC_lexicon_tags_25 },
    { 0x22224183, SUC_lexicon_tags_37 },
    { 0x5be24184, SUC_lexicon_tags_15 },
    { 0xc6824184, SUC_lexicon_tags_25 },
    { 0xd1624186, SUC_lexicon_tags_18 },
    { 0xfb024187, SUC_lexicon_tags_1 },
    { 0x44424188, SUC_lexicon_tags_17 },
    { 0xbd224189, SUC_lexicon_tags_151 },
    { 0x4282418a, SUC_lexicon_tags_47 },
    { 0x43224189, SUC_lexicon_tags_26 },
    { 0x9762418b, SUC_lexicon_tags_63 },
    { 0x262418d, SUC_lexicon_tags_25 },
    { 0x7082418e, SUC_lexicon_tags_1 },
    { 0x98624187, SUC_lexicon_tags_26 },
    { 0x29624190, SUC_lexicon_tags_17 },
    { 0xdd024191, SUC_lexicon_tags_82 },
    { 0x8ac24182, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9522419d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0241a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x996241a4, SUC_lexicon_tags_46 },
    { 0x238241a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46241a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe00241aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8241ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc54241ae, SUC_lexicon_tags_11 },
    { 0x5d0241af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a241b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63c241b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x270241b8, SUC_lexicon_tags_33 },
    { 0x86241b9, SUC_lexicon_tags_19 },
    { 0x678241b9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xe1e241bc, SUC_lexicon_tags_9 },
    { 0x94a241bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4241c0, SUC_lexicon_tags_40 },
    { 0xffc241c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570241c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71c241c9, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6241cd, SUC_lexicon_tags_15 },
    { 0x850241ce, SUC_lexicon_tags_17 },
    { 0x48241cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10c241d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e241d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba241dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x750241df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x964241e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8241e5, SUC_lexicon_tags_12 },
    { 0x868241e6, SUC_lexicon_tags_63 },
    { 0x9d6241e7, SUC_lexicon_tags_16 },
    { 0x3aa241e8, SUC_lexicon_tags_30 },
    { 0xeec241e9, SUC_lexicon_tags_63 },
    { 0xe4a241e7, SUC_lexicon_tags_2 },
    { 0xefe241e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f6241ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68241f0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84241f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8241f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x398241fb, SUC_lexicon_tags_37 },
    { 0xf5a241fb, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4824200, SUC_lexicon_tags_30 },
    { 0xf8424201, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc824209, SUC_lexicon_tags_47 },
    { 0xa482420a, SUC_lexicon_tags_63 },
    { 0xfdc2420b, SUC_lexicon_tags_17 },
    { 0xcc02420c, SUC_lexicon_tags_48 },
    { 0x1002420d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63624210, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22224215, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x26424217, SUC_lexicon_tags_15 },
    { 0xd4c24218, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9882421a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc642421f, SUC_lexicon_tags_21 },
    { 0x5e62421f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28824227, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x35224229, SUC_lexicon_tags_35 },
    { 0x46624229, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4462423b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a2423e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec24242, SUC_lexicon_tags_55 },
    { 0x77024243, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6aa24245, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e2424a, SUC_lexicon_tags_23 },
    { 0x7b62424b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e24257, SUC_lexicon_tags_11 },
    { 0x81224258, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02425c, SUC_lexicon_tags_0 },
    { 0xbfe2425d, SUC_lexicon_tags_17 },
    { 0x1562425e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc8e24260, SUC_lexicon_tags_65 },
    { 0xa4024261, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x87024263, SUC_lexicon_tags_37 },
    { 0x57c24264, SUC_lexicon_tags_45 },
    { 0xb4224263, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf1024267, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce824269, SUC_lexicon_tags_87 },
    { 0x9ec2426a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ba2426c, SUC_lexicon_tags_68 },
    { 0xd8c2426d, SUC_lexicon_tags_12 },
    { 0x1202426e, SUC_lexicon_tags_37 },
    { 0x5da2426f, SUC_lexicon_tags_77 },
    { 0x98424270, SUC_lexicon_tags_38 },
    { 0x7e024270, SUC_lexicon_tags_27 },
    { 0x15a2426c, SUC_lexicon_tags_25 },
    { 0x54c24271, SUC_lexicon_tags_31 },
    { 0xd0224271, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77024278, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3622427a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98224280, SUC_lexicon_tags_21 },
    { 0x6e024281, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c24284, SUC_lexicon_tags_12 },
    { 0x28c24284, SUC_lexicon_tags_4 },
    { 0x15c24285, SUC_lexicon_tags_11 },
    { 0x34624287, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba24292, SUC_lexicon_tags_14 },
    { 0x51a24292, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe24296, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfe424298, SUC_lexicon_tags_82 },
    { 0x62824299, SUC_lexicon_tags_38 },
    { 0xb22429a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2862429e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e242a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e6242a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60242a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8242ad, SUC_lexicon_tags_6 },
    { 0x354242ae, SUC_lexicon_tags_279 },
    { 0x426242af, SUC_lexicon_tags_14 },
    { 0x416242af, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf20242b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30242b5, SUC_lexicon_tags_32 },
    { 0xda4242b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4c4242b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704242be, SUC_lexicon_tags_7 },
    { 0x946242be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcde242c1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0242c7, SUC_lexicon_tags_17 },
    { 0xa24242c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x808242ca, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0242cd, SUC_lexicon_tags_38 },
    { 0x954242cd, SUC_lexicon_tags_30 },
    { 0x5f8242cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x842242d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6242d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e242dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x52a242de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd88242e0, SUC_lexicon_tags_39 },
    { 0xb46242e0, SUC_lexicon_tags_7 },
    { 0x178242e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ce242e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c242ed, SUC_lexicon_tags_39 },
    { 0x91a242ee, SUC_lexicon_tags_17 },
    { 0x1e8242ef, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88242f2, SUC_lexicon_tags_25 },
    { 0x30242f2, SUC_lexicon_tags_118 },
    { 0x588242f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e242f8, SUC_lexicon_tags_11 },
    { 0xe5a242f8, SUC_lexicon_tags_30 },
    { 0x768242fa, SUC_lexicon_tags_2 },
    { 0xdd8242fb, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c24302, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c24308, SUC_lexicon_tags_23 },
    { 0x2d224308, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4562430c, SUC_lexicon_tags_27 },
    { 0x3442430d, SUC_lexicon_tags_31 },
    { 0xda62430e, SUC_lexicon_tags_13 },
    { 0x1862430f, SUC_lexicon_tags_6 },
    { 0x7024310, SUC_lexicon_tags_37 },
    { 0xf4e2430f, SUC_lexicon_tags_37 },
    { 0x1fa2430f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f424314, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ca24316, SUC_lexicon_tags_17 },
    { 0xfc224317, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e2431a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86c2431c, SUC_lexicon_tags_15 },
    { 0x5e2431c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xae2431f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18424322, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c24325, SUC_lexicon_tags_37 },
    { 0xe6c24326, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0024329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb302432f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c024339, SUC_lexicon_tags_45 },
    { 0x8b82433a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2433d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef024346, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeb424348, SUC_lexicon_tags_2 },
    { 0x2fc24348, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0xf2e2434b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e2434e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d624354, SUC_lexicon_tags_8 },
    { 0xbce24354, SUC_lexicon_tags_2 },
    { 0x73824356, SUC_lexicon_tags_15 },
    { 0xe3424355, SUC_lexicon_tags_295 },
    { 0x54e24355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9682435a, SUC_lexicon_tags_26 },
    { 0xe1e2435b, SUC_lexicon_tags_1 },
    { 0x2a22435a, SUC_lexicon_tags_43 },
    { 0x6ec2435d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e24360, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf2024362, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e24365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c424369, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ae2436b, SUC_lexicon_tags_17 },
    { 0xc4e2436c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c62436f, SUC_lexicon_tags_7 },
    { 0x67624370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95a24372, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de24379, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x142437b, SUC_lexicon_tags_2 },
    { 0x13e2437c, SUC_lexicon_tags_1 },
    { 0x74a2437d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12624388, SUC_lexicon_tags_12 },
    { 0x34424388, SUC_lexicon_tags_17 },
    { 0x28c2438a, SUC_lexicon_tags_37 },
    { 0x3902438b, SUC_lexicon_tags_25 },
    { 0xaa62438c, SUC_lexicon_tags_1 },
    { 0xcd42438b, SUC_lexicon_tags_60 },
    { 0x6c2438b, SUC_lexicon_tags_7 },
    { 0x22a2438d, SUC_lexicon_tags_27 },
    { 0x49e24390, SUC_lexicon_tags_47 },
    { 0x81c24391, SUC_lexicon_tags_15 },
    { 0x80624392, SUC_lexicon_tags_2 },
    { 0xa8a24392, SUC_lexicon_tags_55 },
    { 0xc3424393, SUC_lexicon_tags_33 },
    { 0x6dc24392, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9ec24397, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4282439a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4482439d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe2a2439f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb72243a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc243a6, SUC_lexicon_tags_12 },
    { 0x756243a7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820243b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c243ba, SUC_lexicon_tags_2 },
    { 0xd8243bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce243c2, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0243c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x312243c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0243cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x628243ce, SUC_lexicon_tags_49 },
    { 0xa4c243cf, SUC_lexicon_tags_37 },
    { 0x98243cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x124243d2, SUC_lexicon_tags_751 },
    { 0, NULL },
    { 0x736243d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe7c243d6, SUC_lexicon_tags_17 },
    { 0xa66243d7, SUC_lexicon_tags_7 },
    { 0xa26243d8, SUC_lexicon_tags_37 },
    { 0x9aa243d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce243dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10243e1, SUC_lexicon_tags_68 },
    { 0xae0243e1, SUC_lexicon_tags_28 },
    { 0x59a243e1, SUC_lexicon_tags_63 },
    { 0x31a243e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460243eb, SUC_lexicon_tags_7 },
    { 0xac6243ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24243f0, SUC_lexicon_tags_25 },
    { 0xda0243f1, SUC_lexicon_tags_26 },
    { 0xe8c243f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6243f5, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c243fa, SUC_lexicon_tags_11 },
    { 0xec0243fb, SUC_lexicon_tags_37 },
    { 0x7c4243fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9024401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc24405, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb224408, SUC_lexicon_tags_15 },
    { 0xc0c24409, SUC_lexicon_tags_25 },
    { 0x42440a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x142440c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa9e2440e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xffa24410, SUC_lexicon_tags_2 },
    { 0x82424410, SUC_lexicon_tags_12 },
    { 0xebc24412, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7424415, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc424419, SUC_lexicon_tags_4 },
    { 0xeec24419, SUC_lexicon_tags_69 },
    { 0x66c2441b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa562441f, SUC_lexicon_tags_8 },
    { 0x7b824420, SUC_lexicon_tags_33 },
    { 0x78c24421, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ce24423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb02442b, SUC_lexicon_tags_4 },
    { 0xdb62442c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72024430, SUC_lexicon_tags_4 },
    { 0x44624431, SUC_lexicon_tags_30 },
    { 0xb6a24432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a24436, SUC_lexicon_tags_25 },
    { 0x1b624437, SUC_lexicon_tags_16 },
    { 0xa0c24436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef82443d, SUC_lexicon_tags_1 },
    { 0x9642443e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41624449, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93a2444b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12424450, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x59824452, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24224459, SUC_lexicon_tags_6 },
    { 0x5c42445a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4762445d, SUC_lexicon_tags_2 },
    { 0x2a2445e, SUC_lexicon_tags_87 },
    { 0x5c22445d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85224462, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19424464, SUC_lexicon_tags_12 },
    { 0xb8424465, SUC_lexicon_tags_37 },
    { 0x3dc24465, SUC_lexicon_tags_37 },
    { 0x3e024464, SUC_lexicon_tags_27 },
    { 0xe5024468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe062446a, SUC_lexicon_tags_45 },
    { 0xd842446a, SUC_lexicon_tags_7 },
    { 0xe882446b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf624471, SUC_lexicon_tags_7 },
    { 0xe5024472, SUC_lexicon_tags_104 },
    { 0x4c424473, SUC_lexicon_tags_15 },
    { 0x9fe24474, SUC_lexicon_tags_27 },
    { 0xf1e24473, SUC_lexicon_tags_2 },
    { 0xba424476, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b82447d, SUC_lexicon_tags_27 },
    { 0x8a22447d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd624481, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x80024484, SUC_lexicon_tags_38 },
    { 0x1d024485, SUC_lexicon_tags_37 },
    { 0x39e24486, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b02448b, SUC_lexicon_tags_2 },
    { 0xeaa2448b, SUC_lexicon_tags_11 },
    { 0xa0e2448b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44824492, SUC_lexicon_tags_12 },
    { 0x7e224492, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa4024495, SUC_lexicon_tags_49 },
    { 0x4b224495, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c824498, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4322449c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3e42449e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2244a5, SUC_lexicon_tags_25 },
    { 0xa92244a6, SUC_lexicon_tags_16 },
    { 0x4b4244a7, SUC_lexicon_tags_17 },
    { 0x5b0244a7, SUC_lexicon_tags_0 },
    { 0xe24244a7, SUC_lexicon_tags_17 },
    { 0xf0c244aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24244ad, SUC_lexicon_tags_87 },
    { 0x874244ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c244b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780244ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x840244bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164244c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c244c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc84244c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaa2244c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c244ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x418244cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56244ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e244d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeae244d5, SUC_lexicon_tags_14 },
    { 0xaf6244d6, SUC_lexicon_tags_29 },
    { 0xec0244d6, SUC_lexicon_tags_8 },
    { 0xa18244d7, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x124244db, SUC_lexicon_tags_43 },
    { 0x812244db, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc244df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x376244e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x150244e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca244e9, SUC_lexicon_tags_15 },
    { 0x19a244e9, SUC_lexicon_tags_0 },
    { 0x7d0244eb, SUC_lexicon_tags_28 },
    { 0xf7a244e9, SUC_lexicon_tags_27 },
    { 0x99e244ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e244f6, SUC_lexicon_tags_19 },
    { 0x454244f7, SUC_lexicon_tags_1 },
    { 0x64e244f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d8244fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x224244fc, SUC_lexicon_tags_14 },
    { 0x2bc244fc, SUC_lexicon_tags_7 },
    { 0xa6e244fe, SUC_lexicon_tags_14 },
    { 0xd6a244ff, SUC_lexicon_tags_15 },
    { 0x46e24500, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce224505, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa0a24507, SUC_lexicon_tags_63 },
    { 0x8aa24507, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e2450d, SUC_lexicon_tags_27 },
    { 0x702450e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0824516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa582451f, SUC_lexicon_tags_43 },
    { 0x4224520, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2824524, SUC_lexicon_tags_63 },
    { 0x98a24525, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e42452a, SUC_lexicon_tags_47 },
    { 0x4e62452b, SUC_lexicon_tags_7 },
    { 0x4862452b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5024532, SUC_lexicon_tags_12 },
    { 0x56824533, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21024535, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x45024537, SUC_lexicon_tags_38 },
    { 0x72024538, SUC_lexicon_tags_32 },
    { 0x96624538, SUC_lexicon_tags_49 },
    { 0x93024539, SUC_lexicon_tags_31 },
    { 0x88e2453b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a22453e, SUC_lexicon_tags_1 },
    { 0xbd22453f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x94624542, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xad224544, SUC_lexicon_tags_25 },
    { 0xc0224545, SUC_lexicon_tags_51 },
    { 0x2624546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7182454e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be2455b, SUC_lexicon_tags_70 },
    { 0x66a2455c, SUC_lexicon_tags_25 },
    { 0xfe22455d, SUC_lexicon_tags_23 },
    { 0x4842455e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8224563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a24566, SUC_lexicon_tags_61 },
    { 0x65024567, SUC_lexicon_tags_37 },
    { 0xec024567, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a62456a, SUC_lexicon_tags_14 },
    { 0xe062456a, SUC_lexicon_tags_17 },
    { 0xa9a2456a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb624571, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b824573, SUC_lexicon_tags_15 },
    { 0x97224574, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38e24576, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c2457b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90e2457d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e22457f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85024581, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e24584, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4442458a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca2458e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e224593, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xada24595, SUC_lexicon_tags_29 },
    { 0x49824596, SUC_lexicon_tags_46 },
    { 0x9d624597, SUC_lexicon_tags_37 },
    { 0x7ca24598, SUC_lexicon_tags_42 },
    { 0x7b824599, SUC_lexicon_tags_15 },
    { 0x1862459a, SUC_lexicon_tags_6 },
    { 0xff824596, SUC_lexicon_tags_11 },
    { 0x91a2459c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532245a0, SUC_lexicon_tags_37 },
    { 0xdb8245a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a245a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x118245a8, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa245ac, SUC_lexicon_tags_17 },
    { 0x6aa245ac, SUC_lexicon_tags_87 },
    { 0x940245ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28c245b0, SUC_lexicon_tags_37 },
    { 0x5f2245b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea245b8, SUC_lexicon_tags_7 },
    { 0xb5e245b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7bc245bb, SUC_lexicon_tags_7 },
    { 0xea6245bc, SUC_lexicon_tags_38 },
    { 0xf98245bd, SUC_lexicon_tags_37 },
    { 0x90e245be, SUC_lexicon_tags_27 },
    { 0x300245be, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x5d4245c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x468245c3, SUC_lexicon_tags_15 },
    { 0x330245c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e245c7, SUC_lexicon_tags_25 },
    { 0xf42245c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8245cb, SUC_lexicon_tags_33 },
    { 0x8dc245cc, SUC_lexicon_tags_15 },
    { 0xc06245cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f4245cf, SUC_lexicon_tags_275 },
    { 0x16c245cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a245d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x426245d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x476245d8, SUC_lexicon_tags_60 },
    { 0x8be245d9, SUC_lexicon_tags_37 },
    { 0xfaa245d8, SUC_lexicon_tags_17 },
    { 0x1cc245db, SUC_lexicon_tags_45 },
    { 0x6f4245da, SUC_lexicon_tags_7 },
    { 0x412245dd, SUC_lexicon_tags_15 },
    { 0x7bc245de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656245e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a245e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276245ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8245ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6245f0, SUC_lexicon_tags_1 },
    { 0x1b6245f1, SUC_lexicon_tags_8 },
    { 0xfc6245f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8e245f4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x15a245f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a245f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6245fc, SUC_lexicon_tags_23 },
    { 0x26c245fc, SUC_lexicon_tags_56 },
    { 0xbe8245fe, SUC_lexicon_tags_27 },
    { 0x886245fe, SUC_lexicon_tags_17 },
    { 0xaa824600, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa24604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5224606, SUC_lexicon_tags_27 },
    { 0xba224607, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x59424609, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd962460d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc24610, SUC_lexicon_tags_7 },
    { 0xeac24611, SUC_lexicon_tags_1 },
    { 0x72e24612, SUC_lexicon_tags_30 },
    { 0xc8c24613, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e24619, SUC_lexicon_tags_1 },
    { 0xe742461a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x97c2461c, SUC_lexicon_tags_218 },
    { 0x4322461d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e24626, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20c24628, SUC_lexicon_tags_17 },
    { 0x55424629, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74024630, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc24633, SUC_lexicon_tags_32 },
    { 0x52224634, SUC_lexicon_tags_17 },
    { 0x5c024635, SUC_lexicon_tags_37 },
    { 0x44224635, SUC_lexicon_tags_37 },
    { 0x1c824637, SUC_lexicon_tags_38 },
    { 0xd7624638, SUC_lexicon_tags_12 },
    { 0xde224639, SUC_lexicon_tags_7 },
    { 0xada2463a, SUC_lexicon_tags_49 },
    { 0x5e42463a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e24641, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92624645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a024648, SUC_lexicon_tags_2 },
    { 0x94024648, SUC_lexicon_tags_55 },
    { 0xb522464a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf222464c, SUC_lexicon_tags_46 },
    { 0x4442464d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c24650, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb624653, SUC_lexicon_tags_28 },
    { 0xbb224653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4024657, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb82465b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3322465e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x35624660, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab424665, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23624669, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e62466c, SUC_lexicon_tags_14 },
    { 0x8282466c, SUC_lexicon_tags_0 },
    { 0xa9e2466d, SUC_lexicon_tags_12 },
    { 0x59e2466f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27024671, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb424673, SUC_lexicon_tags_7 },
    { 0xe0e24674, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7a24676, SUC_lexicon_tags_49 },
    { 0xa7824677, SUC_lexicon_tags_15 },
    { 0xb6624676, SUC_lexicon_tags_19 },
    { 0x89824677, SUC_lexicon_tags_165 },
    { 0x2a2467a, SUC_lexicon_tags_11 },
    { 0x65c2467b, SUC_lexicon_tags_39 },
    { 0x77e2467a, SUC_lexicon_tags_24 },
    { 0xa562467d, SUC_lexicon_tags_4 },
    { 0x1862467e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c24681, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51024684, SUC_lexicon_tags_11 },
    { 0x8e24685, SUC_lexicon_tags_57 },
    { 0x2da24684, SUC_lexicon_tags_23 },
    { 0xee624687, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda82468a, SUC_lexicon_tags_16 },
    { 0x5862468b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c2468e, SUC_lexicon_tags_63 },
    { 0x7362468f, SUC_lexicon_tags_43 },
    { 0xeb624690, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a24696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf624698, SUC_lexicon_tags_6 },
    { 0xdb024698, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbb62469b, SUC_lexicon_tags_3 },
    { 0x4b02469c, SUC_lexicon_tags_15 },
    { 0x61c2469d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce02469f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfd8246a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92246a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a246ad, SUC_lexicon_tags_45 },
    { 0x5d2246ae, SUC_lexicon_tags_1 },
    { 0xc00246af, SUC_lexicon_tags_38 },
    { 0x5f2246b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x106246b3, SUC_lexicon_tags_17 },
    { 0x348246b4, SUC_lexicon_tags_14 },
    { 0x1e8246b5, SUC_lexicon_tags_31 },
    { 0x128246b3, SUC_lexicon_tags_20 },
    { 0x70246b6, SUC_lexicon_tags_63 },
    { 0x356246b4, SUC_lexicon_tags_23 },
    { 0x29e246b9, SUC_lexicon_tags_27 },
    { 0x76246ba, SUC_lexicon_tags_0 },
    { 0xb1e246bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x514246bd, SUC_lexicon_tags_23 },
    { 0xcfa246bd, SUC_lexicon_tags_11 },
    { 0xbe8246bf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8246c9, SUC_lexicon_tags_26 },
    { 0x6a6246ca, SUC_lexicon_tags_15 },
    { 0xc6c246ca, SUC_lexicon_tags_1 },
    { 0xa32246cc, SUC_lexicon_tags_7 },
    { 0xf4a246cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510246d5, SUC_lexicon_tags_12 },
    { 0x6d0246d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd80246d8, SUC_lexicon_tags_7 },
    { 0xc2a246d8, SUC_lexicon_tags_10 },
    { 0xf7a246da, SUC_lexicon_tags_12 },
    { 0x248246da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd86246dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a246e1, SUC_lexicon_tags_60 },
    { 0x922246e2, SUC_lexicon_tags_45 },
    { 0xcee246e3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2246ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf0246ec, SUC_lexicon_tags_11 },
    { 0x9e8246ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986246f1, SUC_lexicon_tags_37 },
    { 0x856246f2, SUC_lexicon_tags_27 },
    { 0x72c246f3, SUC_lexicon_tags_17 },
    { 0xf3e246f1, SUC_lexicon_tags_25 },
    { 0x9fa246f5, SUC_lexicon_tags_66 },
    { 0xaa6246f1, SUC_lexicon_tags_69 },
    { 0xde2246f7, SUC_lexicon_tags_0 },
    { 0xd4c246f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3e246fa, SUC_lexicon_tags_11 },
    { 0xe30246fa, SUC_lexicon_tags_11 },
    { 0x7e4246fc, SUC_lexicon_tags_63 },
    { 0x4e4246fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d8246ff, SUC_lexicon_tags_87 },
    { 0x6bc24700, SUC_lexicon_tags_7 },
    { 0x2f624701, SUC_lexicon_tags_16 },
    { 0x99c24700, SUC_lexicon_tags_11 },
    { 0x7a24703, SUC_lexicon_tags_16 },
    { 0xdc824704, SUC_lexicon_tags_1 },
    { 0x77e246ff, SUC_lexicon_tags_11 },
    { 0x73224706, SUC_lexicon_tags_37 },
    { 0x8e24707, SUC_lexicon_tags_179 },
    { 0x65624705, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c2470d, SUC_lexicon_tags_15 },
    { 0xe982470e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf824715, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6624718, SUC_lexicon_tags_1 },
    { 0x61e24719, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba2471c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78024720, SUC_lexicon_tags_12 },
    { 0xf6224720, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac24726, SUC_lexicon_tags_2 },
    { 0x87224727, SUC_lexicon_tags_79 },
    { 0x57c24728, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd722472c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xb302472e, SUC_lexicon_tags_15 },
    { 0xbae2472f, SUC_lexicon_tags_37 },
    { 0x5b22472e, SUC_lexicon_tags_234 },
    { 0x14224731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66224734, SUC_lexicon_tags_17 },
    { 0xc9e24735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39024739, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf62473d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda24740, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74824745, SUC_lexicon_tags_15 },
    { 0x57024746, SUC_lexicon_tags_7 },
    { 0xf8a24746, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x8342474a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122474e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4624751, SUC_lexicon_tags_91 },
    { 0x3e424752, SUC_lexicon_tags_5 },
    { 0x7fe24753, SUC_lexicon_tags_23 },
    { 0xbf424754, SUC_lexicon_tags_1 },
    { 0xa6e24754, SUC_lexicon_tags_1 },
    { 0x20824755, SUC_lexicon_tags_5 },
    { 0x70e24757, SUC_lexicon_tags_28 },
    { 0x58024758, SUC_lexicon_tags_47 },
    { 0x81624759, SUC_lexicon_tags_27 },
    { 0x22a2475a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b82475d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d024762, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22024769, SUC_lexicon_tags_56 },
    { 0xf2a24769, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3562476e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc24772, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9024775, SUC_lexicon_tags_12 },
    { 0x87c24776, SUC_lexicon_tags_11 },
    { 0x98c24776, SUC_lexicon_tags_17 },
    { 0x71824777, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c2477e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e24780, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e24785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13824787, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7262478e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x82624790, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2a24792, SUC_lexicon_tags_43 },
    { 0x36224793, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e2479a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf142479c, SUC_lexicon_tags_14 },
    { 0x1602479d, SUC_lexicon_tags_7 },
    { 0xd0e2479e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e247a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496247a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c247ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08247af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba247b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x876247b6, SUC_lexicon_tags_15 },
    { 0x434247b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce247ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc0a247bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8247c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa247c6, SUC_lexicon_tags_65 },
    { 0x34247c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9a6247c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80247d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x90247d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x208247d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e247da, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766247de, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x97e247e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3c2247e2, SUC_lexicon_tags_352 },
    { 0x7ce247e3, SUC_lexicon_tags_10 },
    { 0x2be247e4, SUC_lexicon_tags_11 },
    { 0x95e247e5, SUC_lexicon_tags_11 },
    { 0x650247e3, SUC_lexicon_tags_11 },
    { 0xcc8247e7, SUC_lexicon_tags_15 },
    { 0x3de247e8, SUC_lexicon_tags_28 },
    { 0x828247e5, SUC_lexicon_tags_25 },
    { 0x30e247e9, SUC_lexicon_tags_14 },
    { 0x610247e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c247f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e247fd, SUC_lexicon_tags_39 },
    { 0x68247fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1024801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55024805, SUC_lexicon_tags_47 },
    { 0xdf024806, SUC_lexicon_tags_7 },
    { 0x3e24807, SUC_lexicon_tags_11 },
    { 0x42c24806, SUC_lexicon_tags_21 },
    { 0xba024806, SUC_lexicon_tags_12 },
    { 0xd024806, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c2480f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0424813, SUC_lexicon_tags_39 },
    { 0x54624814, SUC_lexicon_tags_30 },
    { 0xa1c24815, SUC_lexicon_tags_1 },
    { 0xde224816, SUC_lexicon_tags_8 },
    { 0x92824817, SUC_lexicon_tags_37 },
    { 0x6d024817, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc2481b, SUC_lexicon_tags_17 },
    { 0x96c2481c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e2481f, SUC_lexicon_tags_9 },
    { 0x65024820, SUC_lexicon_tags_43 },
    { 0x78824820, SUC_lexicon_tags_17 },
    { 0x9a024821, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c24824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c624828, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8f82482a, SUC_lexicon_tags_12 },
    { 0xa422482b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9222482e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1de24830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b424834, SUC_lexicon_tags_38 },
    { 0xe2424835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a624839, SUC_lexicon_tags_26 },
    { 0xde424839, SUC_lexicon_tags_7 },
    { 0xb2224839, SUC_lexicon_tags_8 },
    { 0xe0e2483c, SUC_lexicon_tags_29 },
    { 0xc722483d, SUC_lexicon_tags_43 },
    { 0x8be2483e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3424840, SUC_lexicon_tags_2 },
    { 0x36624840, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4a24843, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e24846, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4942484b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb42484d, SUC_lexicon_tags_25 },
    { 0x9ea2484e, SUC_lexicon_tags_28 },
    { 0x52a2484e, SUC_lexicon_tags_33 },
    { 0xddc2484e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c24853, SUC_lexicon_tags_55 },
    { 0x77024853, SUC_lexicon_tags_11 },
    { 0x8a624855, SUC_lexicon_tags_38 },
    { 0xec24856, SUC_lexicon_tags_7 },
    { 0x8da24856, SUC_lexicon_tags_29 },
    { 0x48024857, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae2485d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdba2485f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e24863, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68024868, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde2486c, SUC_lexicon_tags_0 },
    { 0x9182486d, SUC_lexicon_tags_27 },
    { 0xe2e2486d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea24871, SUC_lexicon_tags_16 },
    { 0xf5624872, SUC_lexicon_tags_61 },
    { 0x2de24873, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x37624875, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6442487a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49424880, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e024886, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb682488a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2682488d, SUC_lexicon_tags_2 },
    { 0x8322488e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7824890, SUC_lexicon_tags_60 },
    { 0xb1a24891, SUC_lexicon_tags_29 },
    { 0x35424892, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c2489a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc248a2, SUC_lexicon_tags_11 },
    { 0x470248a3, SUC_lexicon_tags_27 },
    { 0x2e248a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc248aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a248b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x702248b4, SUC_lexicon_tags_7 },
    { 0x26c248b5, SUC_lexicon_tags_5 },
    { 0x69e248b6, SUC_lexicon_tags_15 },
    { 0x762248b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4248ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a248bf, SUC_lexicon_tags_10 },
    { 0x876248c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a248c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2bc248c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd08248c8, SUC_lexicon_tags_84 },
    { 0x6e248c9, SUC_lexicon_tags_37 },
    { 0xd42248ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4248d1, SUC_lexicon_tags_25 },
    { 0x732248d2, SUC_lexicon_tags_0 },
    { 0x738248d3, SUC_lexicon_tags_6 },
    { 0xf08248d3, SUC_lexicon_tags_1 },
    { 0x9f0248d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b0248d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48248de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e248e7, SUC_lexicon_tags_37 },
    { 0xa30248e7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6248eb, SUC_lexicon_tags_37 },
    { 0xf26248ec, SUC_lexicon_tags_37 },
    { 0xcd6248ed, SUC_lexicon_tags_15 },
    { 0x1c8248ee, SUC_lexicon_tags_31 },
    { 0x67e248ee, SUC_lexicon_tags_14 },
    { 0x6a8248f0, SUC_lexicon_tags_49 },
    { 0x220248ee, SUC_lexicon_tags_77 },
    { 0x31c248ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa30248f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854248f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508248fc, SUC_lexicon_tags_11 },
    { 0x362248fc, SUC_lexicon_tags_63 },
    { 0x4a6248fd, SUC_lexicon_tags_1 },
    { 0x8c8248ff, SUC_lexicon_tags_11 },
    { 0x94224900, SUC_lexicon_tags_14 },
    { 0x986248fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae24907, SUC_lexicon_tags_3 },
    { 0x23c24908, SUC_lexicon_tags_37 },
    { 0xc2424908, SUC_lexicon_tags_11 },
    { 0x16c24908, SUC_lexicon_tags_17 },
    { 0xf622490b, SUC_lexicon_tags_23 },
    { 0x8242490b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf02490e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea24911, SUC_lexicon_tags_37 },
    { 0xf0424912, SUC_lexicon_tags_17 },
    { 0x74624913, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6ee24915, SUC_lexicon_tags_82 },
    { 0x56224916, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe24919, SUC_lexicon_tags_48 },
    { 0x16624919, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa6e2491c, SUC_lexicon_tags_129 },
    { 0x10c2491d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x94024920, SUC_lexicon_tags_27 },
    { 0xe0e24921, SUC_lexicon_tags_37 },
    { 0xcae24922, SUC_lexicon_tags_11 },
    { 0xc1024923, SUC_lexicon_tags_2 },
    { 0x67c24924, SUC_lexicon_tags_10 },
    { 0x30a24923, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5e024927, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae2492d, SUC_lexicon_tags_28 },
    { 0x6dc2492d, SUC_lexicon_tags_34 },
    { 0xaee2492d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5624933, SUC_lexicon_tags_65 },
    { 0x2224933, SUC_lexicon_tags_38 },
    { 0x52824935, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6222493a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a22493f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbca24941, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b224944, SUC_lexicon_tags_38 },
    { 0x53424945, SUC_lexicon_tags_45 },
    { 0x8a024946, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43824948, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a2494e, SUC_lexicon_tags_14 },
    { 0x2a2494f, SUC_lexicon_tags_29 },
    { 0x99e24950, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc24954, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc2495b, SUC_lexicon_tags_33 },
    { 0xc982495c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45224961, SUC_lexicon_tags_87 },
    { 0x55a24962, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e24966, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff62496a, SUC_lexicon_tags_25 },
    { 0xafc2496a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf024974, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30824978, SUC_lexicon_tags_15 },
    { 0xa7424978, SUC_lexicon_tags_12 },
    { 0xfa22497a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6522497c, SUC_lexicon_tags_29 },
    { 0x7ca2497d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec24982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff424987, SUC_lexicon_tags_25 },
    { 0xd6c24988, SUC_lexicon_tags_12 },
    { 0x15824989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc122498b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16424990, SUC_lexicon_tags_17 },
    { 0x99a24991, SUC_lexicon_tags_34 },
    { 0x72224992, SUC_lexicon_tags_11 },
    { 0x77a24993, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c424996, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3aa24998, SUC_lexicon_tags_0 },
    { 0xda424999, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9322499b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa582499d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee249a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x462249a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf56249a6, SUC_lexicon_tags_47 },
    { 0x7a6249a7, SUC_lexicon_tags_0 },
    { 0xd3e249a8, SUC_lexicon_tags_37 },
    { 0x738249a9, SUC_lexicon_tags_11 },
    { 0x2ac249a8, SUC_lexicon_tags_25 },
    { 0xada249ab, SUC_lexicon_tags_33 },
    { 0x9f6249ac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8f8249ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x850249b1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xca4249b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78249b6, SUC_lexicon_tags_12 },
    { 0x10e249b7, SUC_lexicon_tags_14 },
    { 0xa42249b8, SUC_lexicon_tags_49 },
    { 0xfc2249b9, SUC_lexicon_tags_40 },
    { 0x912249b8, SUC_lexicon_tags_2 },
    { 0x70c249bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e249c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7ce249c2, SUC_lexicon_tags_63 },
    { 0x8f4249c3, SUC_lexicon_tags_15 },
    { 0x2f2249c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82249cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x894249cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x414249d1, SUC_lexicon_tags_33 },
    { 0xa00249d2, SUC_lexicon_tags_23 },
    { 0x4f6249d2, SUC_lexicon_tags_47 },
    { 0x10249d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x292249d6, SUC_lexicon_tags_2 },
    { 0x992249d6, SUC_lexicon_tags_2 },
    { 0xd52249d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9e8249da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f0249dc, SUC_lexicon_tags_14 },
    { 0x2c4249dd, SUC_lexicon_tags_30 },
    { 0xc50249dd, SUC_lexicon_tags_37 },
    { 0x690249df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x274249e1, SUC_lexicon_tags_12 },
    { 0x4e2249e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x506249e4, SUC_lexicon_tags_61 },
    { 0x730249e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9a6249e7, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64249ec, SUC_lexicon_tags_2 },
    { 0x332249ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12249f9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6de249fb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0249fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb1424a00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e824a06, SUC_lexicon_tags_9 },
    { 0x5c24a06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede24a0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da24a0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ae24a10, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a24a14, SUC_lexicon_tags_33 },
    { 0x66c24a15, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c24a19, SUC_lexicon_tags_10 },
    { 0x32424a1a, SUC_lexicon_tags_37 },
    { 0x30224a1b, SUC_lexicon_tags_25 },
    { 0x6a824a1c, SUC_lexicon_tags_37 },
    { 0x75224a1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a24a21, SUC_lexicon_tags_45 },
    { 0xb1224a22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac24a27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde624a2a, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0x9b624a2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73c24a2e, SUC_lexicon_tags_32 },
    { 0xdd424a2e, SUC_lexicon_tags_12 },
    { 0x47c24a30, SUC_lexicon_tags_25 },
    { 0x4824a31, SUC_lexicon_tags_45 },
    { 0x10a24a32, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc024a3c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfee24a3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0c24a40, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e24a44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda24a47, SUC_lexicon_tags_16 },
    { 0x59a24a48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8624a4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2d224a4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec624a4e, SUC_lexicon_tags_7 },
    { 0x74624a4f, SUC_lexicon_tags_37 },
    { 0xe24a50, SUC_lexicon_tags_14 },
    { 0xe0024a51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e24a56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1aa24a58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1424a5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6e24a5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c24a60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad824a66, SUC_lexicon_tags_60 },
    { 0x77c24a67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2424a69, SUC_lexicon_tags_0 },
    { 0x2b424a6a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2224a6d, SUC_lexicon_tags_37 },
    { 0x65c24a6e, SUC_lexicon_tags_5 },
    { 0x13a24a6f, SUC_lexicon_tags_26 },
    { 0xd8424a70, SUC_lexicon_tags_2 },
    { 0xe0824a71, SUC_lexicon_tags_49 },
    { 0x2a624a70, SUC_lexicon_tags_17 },
    { 0xa4424a73, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d624a78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd2c24a7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5824a7c, SUC_lexicon_tags_23 },
    { 0xcda24a7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec24a80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6824a84, SUC_lexicon_tags_23 },
    { 0x98424a85, SUC_lexicon_tags_31 },
    { 0xa5824a86, SUC_lexicon_tags_26 },
    { 0xf5a24a87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x86024a89, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcda24a8b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa24a94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a24a97, SUC_lexicon_tags_45 },
    { 0x64e24a98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4ae24a9a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x41624a9d, SUC_lexicon_tags_14 },
    { 0xc3a24a9d, SUC_lexicon_tags_2 },
    { 0x6b624a9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56e24aa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8e24aa3, SUC_lexicon_tags_2 },
    { 0xfce24aa4, SUC_lexicon_tags_11 },
    { 0x1824aa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98624aa7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29624aac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x45624aaf, SUC_lexicon_tags_68 },
    { 0x65824ab0, SUC_lexicon_tags_38 },
    { 0xf0a24aaf, SUC_lexicon_tags_25 },
    { 0x26224ab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a824ab6, SUC_lexicon_tags_85 },
    { 0x21a24ab7, SUC_lexicon_tags_37 },
    { 0xb9e24ab8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae24abc, SUC_lexicon_tags_55 },
    { 0x5f224abc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a24ac0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8824ac2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c24ac5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0224ac7, SUC_lexicon_tags_6 },
    { 0xbfa24ac8, SUC_lexicon_tags_57 },
    { 0xf9224ac9, SUC_lexicon_tags_15 },
    { 0xad024aca, SUC_lexicon_tags_38 },
    { 0x2f824acb, SUC_lexicon_tags_7 },
    { 0x9824aca, SUC_lexicon_tags_29 },
    { 0x52624acd, SUC_lexicon_tags_87 },
    { 0xb9e24acd, SUC_lexicon_tags_87 },
    { 0xfac24ace, SUC_lexicon_tags_49 },
    { 0xe8824ac8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf5424ad2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7624ad5, SUC_lexicon_tags_10 },
    { 0xc5e24ad5, SUC_lexicon_tags_11 },
    { 0x76424ad6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x87824ad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa24adc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de24ae0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5e24ae2, SUC_lexicon_tags_17 },
    { 0x7224ae2, SUC_lexicon_tags_17 },
    { 0x3a224ae4, SUC_lexicon_tags_63 },
    { 0x2e24ae5, SUC_lexicon_tags_37 },
    { 0x28c24ae2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf424aeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c424aef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9224af1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c24af5, SUC_lexicon_tags_1 },
    { 0xab024af6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe024aff, SUC_lexicon_tags_48 },
    { 0xbc24b00, SUC_lexicon_tags_48 },
    { 0x93e24b01, SUC_lexicon_tags_39 },
    { 0x2ec24b02, SUC_lexicon_tags_7 },
    { 0x2024b03, SUC_lexicon_tags_2 },
    { 0x8b224b01, SUC_lexicon_tags_39 },
    { 0xc4a24b05, SUC_lexicon_tags_6 },
    { 0x3b624b06, SUC_lexicon_tags_6 },
    { 0xa3424b07, SUC_lexicon_tags_27 },
    { 0x63824b05, SUC_lexicon_tags_1 },
    { 0x78224b09, SUC_lexicon_tags_25 },
    { 0xfee24b0a, SUC_lexicon_tags_8 },
    { 0x50424b01, SUC_lexicon_tags_11 },
    { 0xb6a24b0c, SUC_lexicon_tags_35 },
    { 0x45e24b0a, SUC_lexicon_tags_1 },
    { 0x86c24b0e, SUC_lexicon_tags_1 },
    { 0x8a024b05, SUC_lexicon_tags_79 },
    { 0xd0024b10, SUC_lexicon_tags_12 },
    { 0xe5424b11, SUC_lexicon_tags_25 },
    { 0x30624b10, SUC_lexicon_tags_29 },
    { 0xc8a24b11, SUC_lexicon_tags_35 },
    { 0xca024b14, SUC_lexicon_tags_62 },
    { 0xfa424b05, SUC_lexicon_tags_19 },
    { 0x98a24b03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa24b1d, SUC_lexicon_tags_1 },
    { 0xea424b1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15824b24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84a24b26, SUC_lexicon_tags_23 },
    { 0x3b024b26, SUC_lexicon_tags_65 },
    { 0x82e24b26, SUC_lexicon_tags_2 },
    { 0x91824b26, SUC_lexicon_tags_25 },
    { 0x19c24b2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1624b2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98024b2f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb024b32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca24b36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59624b3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5624b42, SUC_lexicon_tags_37 },
    { 0x10424b43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc24b47, SUC_lexicon_tags_3 },
    { 0x89824b48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e624b4d, SUC_lexicon_tags_60 },
    { 0x71624b4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8624b50, SUC_lexicon_tags_12 },
    { 0x86024b51, SUC_lexicon_tags_87 },
    { 0xf4a24b50, SUC_lexicon_tags_37 },
    { 0x34424b53, SUC_lexicon_tags_69 },
    { 0x43424b54, SUC_lexicon_tags_25 },
    { 0x83224b54, SUC_lexicon_tags_28 },
    { 0x8fe24b56, SUC_lexicon_tags_2 },
    { 0x94024b51, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27024b5b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc24b5e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x37424b60, SUC_lexicon_tags_37 },
    { 0x21624b61, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x80a24b63, SUC_lexicon_tags_8 },
    { 0xd7424b64, SUC_lexicon_tags_15 },
    { 0x2d024b65, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7f824b67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6a24b69, SUC_lexicon_tags_2 },
    { 0x1c24b69, SUC_lexicon_tags_17 },
    { 0x35c24b6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37224b6f, SUC_lexicon_tags_61 },
    { 0x2b024b6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6224b73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da24b78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x75824b7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c824b7d, SUC_lexicon_tags_11 },
    { 0xde24b7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46824b85, SUC_lexicon_tags_37 },
    { 0x41c24b85, SUC_lexicon_tags_68 },
    { 0xfa224b87, SUC_lexicon_tags_8 },
    { 0x41424b88, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43624b8c, SUC_lexicon_tags_10 },
    { 0x4c24b8c, SUC_lexicon_tags_15 },
    { 0x1f824b8e, SUC_lexicon_tags_15 },
    { 0x1c024b8f, SUC_lexicon_tags_37 },
    { 0x18624b8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8424b92, SUC_lexicon_tags_40 },
    { 0x18024b93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe24b97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c24b9b, SUC_lexicon_tags_2 },
    { 0xce824b9c, SUC_lexicon_tags_63 },
    { 0xc0c24b9d, SUC_lexicon_tags_31 },
    { 0x7e624b9e, SUC_lexicon_tags_4 },
    { 0xae424b9f, SUC_lexicon_tags_29 },
    { 0x7e024ba0, SUC_lexicon_tags_1 },
    { 0x73e24b9d, SUC_lexicon_tags_26 },
    { 0x16c24ba2, SUC_lexicon_tags_68 },
    { 0x78424ba3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58424ba5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c24bb3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb824bb6, SUC_lexicon_tags_25 },
    { 0xc0024bb7, SUC_lexicon_tags_11 },
    { 0x5e624bb8, SUC_lexicon_tags_7 },
    { 0x1be24bb9, SUC_lexicon_tags_9 },
    { 0x63824bba, SUC_lexicon_tags_2 },
    { 0x97a24bba, SUC_lexicon_tags_25 },
    { 0x71a24bba, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x81424bbe, SUC_lexicon_tags_26 },
    { 0xe224bbf, SUC_lexicon_tags_15 },
    { 0x43024bbe, SUC_lexicon_tags_23 },
    { 0x52c24bc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82424bc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41424bcb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f824bce, SUC_lexicon_tags_29 },
    { 0x6c024bcf, SUC_lexicon_tags_33 },
    { 0x7ba24bce, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5824bd5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51624bda, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f024bdd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c624be5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3624bec, SUC_lexicon_tags_2 },
    { 0xf4a24bec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51024bf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2e024bf5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4424bfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e24bff, SUC_lexicon_tags_25 },
    { 0x36424c00, SUC_lexicon_tags_2 },
    { 0xf7624c00, SUC_lexicon_tags_39 },
    { 0x98224c02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce24c0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74424c0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x18424c0f, SUC_lexicon_tags_28 },
    { 0x11624c10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b624c13, SUC_lexicon_tags_38 },
    { 0xdd224c14, SUC_lexicon_tags_11 },
    { 0x14824c14, SUC_lexicon_tags_1 },
    { 0xc7a24c15, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0xe2e24c18, SUC_lexicon_tags_45 },
    { 0xba024c19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x10224c1b, SUC_lexicon_tags_9 },
    { 0xc3a24c1b, SUC_lexicon_tags_15 },
    { 0x84e24c1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x64624c20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43624c22, SUC_lexicon_tags_32 },
    { 0xffa24c22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcf824c25, SUC_lexicon_tags_27 },
    { 0xc1024c25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbfe24c28, SUC_lexicon_tags_57 },
    { 0x71624c29, SUC_lexicon_tags_12 },
    { 0x83824c29, SUC_lexicon_tags_60 },
    { 0x82824c2b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93824c2f, SUC_lexicon_tags_2 },
    { 0x4c424c2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba624c36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fa24c38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88824c3c, SUC_lexicon_tags_5 },
    { 0x84e24c3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2224c43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2424c46, SUC_lexicon_tags_20 },
    { 0x26224c47, SUC_lexicon_tags_37 },
    { 0xf9424c48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc224c55, SUC_lexicon_tags_7 },
    { 0x4d824c56, SUC_lexicon_tags_33 },
    { 0x9c24c57, SUC_lexicon_tags_21 },
    { 0xc7e24c58, SUC_lexicon_tags_15 },
    { 0x58c24c56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa24c5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x18824c60, SUC_lexicon_tags_7 },
    { 0xa5a24c61, SUC_lexicon_tags_46 },
    { 0xff224c60, SUC_lexicon_tags_2 },
    { 0x95224c63, SUC_lexicon_tags_9 },
    { 0x15824c62, SUC_lexicon_tags_37 },
    { 0xe1624c65, SUC_lexicon_tags_49 },
    { 0x80424c66, SUC_lexicon_tags_30 },
    { 0xcbc24c67, SUC_lexicon_tags_37 },
    { 0xa0824c68, SUC_lexicon_tags_63 },
    { 0xf6e24c69, SUC_lexicon_tags_46 },
    { 0xf4224c6a, SUC_lexicon_tags_11 },
    { 0x32a24c65, SUC_lexicon_tags_28 },
    { 0x5e224c60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a24c70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80024c72, SUC_lexicon_tags_11 },
    { 0x2f624c73, SUC_lexicon_tags_11 },
    { 0x92c24c72, SUC_lexicon_tags_7 },
    { 0x76824c75, SUC_lexicon_tags_28 },
    { 0x85624c72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x75024c78, SUC_lexicon_tags_7 },
    { 0x9da24c79, SUC_lexicon_tags_49 },
    { 0x48e24c7a, SUC_lexicon_tags_136 },
    { 0x5ea24c7a, SUC_lexicon_tags_1 },
    { 0xd424c7a, SUC_lexicon_tags_37 },
    { 0x31424c7d, SUC_lexicon_tags_6 },
    { 0x14224c7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b424c80, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x60024c82, SUC_lexicon_tags_31 },
    { 0xaa24c83, SUC_lexicon_tags_9 },
    { 0xf4224c82, SUC_lexicon_tags_14 },
    { 0x9a824c85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66824c8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe824c8e, SUC_lexicon_tags_27 },
    { 0x88c24c8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66424c91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e24c96, SUC_lexicon_tags_11 },
    { 0xe2024c97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0624c99, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd824ca8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce424cac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d224caf, SUC_lexicon_tags_13 },
    { 0xb4e24cb0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec24cb3, SUC_lexicon_tags_26 },
    { 0x65a24cb4, SUC_lexicon_tags_45 },
    { 0xe4424cb5, SUC_lexicon_tags_26 },
    { 0x424cb4, SUC_lexicon_tags_25 },
    { 0x5a224cb5, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c24cba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93024cc1, SUC_lexicon_tags_25 },
    { 0x32424cc2, SUC_lexicon_tags_26 },
    { 0xaec24cc3, SUC_lexicon_tags_9 },
    { 0xed824cc4, SUC_lexicon_tags_27 },
    { 0x25c24cc4, SUC_lexicon_tags_15 },
    { 0x20224cc6, SUC_lexicon_tags_5 },
    { 0x64824cc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8224cca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac24ccf, SUC_lexicon_tags_77 },
    { 0xd1c24cd0, SUC_lexicon_tags_12 },
    { 0x50a24cd0, SUC_lexicon_tags_12 },
    { 0x99224cd1, SUC_lexicon_tags_16 },
    { 0xa8224cd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1c824cd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa24cd9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c424cdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf624ce1, SUC_lexicon_tags_0 },
    { 0x81424ce2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6cc24ce4, SUC_lexicon_tags_30 },
    { 0xf0624ce4, SUC_lexicon_tags_15 },
    { 0x90224ce4, SUC_lexicon_tags_7 },
    { 0xde824ce7, SUC_lexicon_tags_1 },
    { 0xbb424ce7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x86424ceb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca824cf3, SUC_lexicon_tags_32 },
    { 0x68024cf4, SUC_lexicon_tags_8 },
    { 0x51624cf5, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x85424cf8, SUC_lexicon_tags_167 },
    { 0x6024cf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57c24cfb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88824cff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43024d02, SUC_lexicon_tags_1 },
    { 0xeda24d02, SUC_lexicon_tags_17 },
    { 0x4de24d02, SUC_lexicon_tags_12 },
    { 0x95824d03, SUC_lexicon_tags_66 },
    { 0x69224d02, SUC_lexicon_tags_33 },
    { 0xa7424d05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a24d0a, SUC_lexicon_tags_12 },
    { 0xa5624d0a, SUC_lexicon_tags_6 },
    { 0xf3224d0c, SUC_lexicon_tags_87 },
    { 0xfea24d0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a224d0f, SUC_lexicon_tags_12 },
    { 0x1b624d0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c24d1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e24d22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf3a24d24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2024d31, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e624d34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc24d39, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c24d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76024d3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7c24d41, SUC_lexicon_tags_1 },
    { 0xc4424d41, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed624d48, SUC_lexicon_tags_29 },
    { 0x8f824d49, SUC_lexicon_tags_15 },
    { 0x50224d4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x76624d4c, SUC_lexicon_tags_2 },
    { 0x4cc24d4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be24d52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6024d54, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcd024d56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de24d59, SUC_lexicon_tags_15 },
    { 0xece24d59, SUC_lexicon_tags_19 },
    { 0x8d424d5b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa24d5f, SUC_lexicon_tags_25 },
    { 0x98624d5f, SUC_lexicon_tags_14 },
    { 0xfda24d61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c624d69, SUC_lexicon_tags_27 },
    { 0x9c24d6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43424d6e, SUC_lexicon_tags_17 },
    { 0xb5a24d6f, SUC_lexicon_tags_17 },
    { 0xe5424d6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58c24d72, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58624d77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50824d7c, SUC_lexicon_tags_33 },
    { 0xc5a24d7d, SUC_lexicon_tags_2 },
    { 0x29c24d7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94424d8b, SUC_lexicon_tags_2 },
    { 0x59824d8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4224d90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a24d94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a24d97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f824d9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9424d9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63624da0, SUC_lexicon_tags_12 },
    { 0xf2224da1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe24dac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c24daf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e24db5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32424db9, SUC_lexicon_tags_19 },
    { 0xf6a24db9, SUC_lexicon_tags_15 },
    { 0xde424dbb, SUC_lexicon_tags_25 },
    { 0x59624dbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ba24dbe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a24dc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1824dc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d024dc9, SUC_lexicon_tags_17 },
    { 0x73c24dca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de24dce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b424dd2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a24dd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34024de2, SUC_lexicon_tags_15 },
    { 0x97824de2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb024de8, SUC_lexicon_tags_2 },
    { 0xcc24de8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafe24deb, SUC_lexicon_tags_10 },
    { 0x83024dec, SUC_lexicon_tags_25 },
    { 0xb7024ded, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdfa24def, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3624df1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa24df4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c24df8, SUC_lexicon_tags_12 },
    { 0x4f824df9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a24dff, SUC_lexicon_tags_65 },
    { 0x95024e00, SUC_lexicon_tags_25 },
    { 0xb5224e01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c24e05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c224e09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce24e0c, SUC_lexicon_tags_4 },
    { 0x91024e0d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x35e24e0f, SUC_lexicon_tags_0 },
    { 0xf3824e10, SUC_lexicon_tags_25 },
    { 0xf5c24e10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab824e14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96024e17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29a24e19, SUC_lexicon_tags_6 },
    { 0xe2024e19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c24e2a, SUC_lexicon_tags_17 },
    { 0x76a24e2a, SUC_lexicon_tags_63 },
    { 0x73424e2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a224e32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3624e34, SUC_lexicon_tags_27 },
    { 0x42a24e35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48024e37, SUC_lexicon_tags_2 },
    { 0x41424e37, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0x5d224e3a, SUC_lexicon_tags_55 },
    { 0xee624e3b, SUC_lexicon_tags_25 },
    { 0x58424e3c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b824e3f, SUC_lexicon_tags_7 },
    { 0x21c24e40, SUC_lexicon_tags_10 },
    { 0xc7424e40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca24e44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba24e47, SUC_lexicon_tags_17 },
    { 0xa6624e48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1824e4a, SUC_lexicon_tags_7 },
    { 0x3bc24e4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c24e52, SUC_lexicon_tags_11 },
    { 0x38024e53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21224e57, SUC_lexicon_tags_10 },
    { 0x69a24e58, SUC_lexicon_tags_26 },
    { 0x67824e57, SUC_lexicon_tags_12 },
    { 0x23624e5a, SUC_lexicon_tags_4 },
    { 0x55024e58, SUC_lexicon_tags_53 },
    { 0x5e024e5c, SUC_lexicon_tags_25 },
    { 0x9a224e5a, SUC_lexicon_tags_23 },
    { 0xf8c24e5e, SUC_lexicon_tags_15 },
    { 0x59424e5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c24e65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf3424e67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9e24e69, SUC_lexicon_tags_2 },
    { 0x16a24e6a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x84824e6d, SUC_lexicon_tags_17 },
    { 0xaa824e6d, SUC_lexicon_tags_37 },
    { 0xa9a24e6d, SUC_lexicon_tags_2 },
    { 0x80224e70, SUC_lexicon_tags_31 },
    { 0x82a24e71, SUC_lexicon_tags_15 },
    { 0xbac24e71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2224e77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75224e79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac24e82, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xf3e24e84, SUC_lexicon_tags_43 },
    { 0xa3c24e85, SUC_lexicon_tags_37 },
    { 0xac824e86, SUC_lexicon_tags_7 },
    { 0x91c24e87, SUC_lexicon_tags_14 },
    { 0x29e24e84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a424e8a, SUC_lexicon_tags_11 },
    { 0x89a24e8b, SUC_lexicon_tags_43 },
    { 0x7d024e8b, SUC_lexicon_tags_87 },
    { 0xeae24e8d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x37a24e8f, SUC_lexicon_tags_2 },
    { 0xbfa24e90, SUC_lexicon_tags_37 },
    { 0x8b024e8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b224e94, SUC_lexicon_tags_37 },
    { 0x7c424e95, SUC_lexicon_tags_12 },
    { 0x7624e94, SUC_lexicon_tags_29 },
    { 0x5ac24e97, SUC_lexicon_tags_33 },
    { 0xf1e24e96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6824e9a, SUC_lexicon_tags_13 },
    { 0x54024e9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c24e9e, SUC_lexicon_tags_17 },
    { 0xa7e24e9f, SUC_lexicon_tags_9 },
    { 0xf4a24ea0, SUC_lexicon_tags_25 },
    { 0x31e24ea1, SUC_lexicon_tags_37 },
    { 0x4c24ea1, SUC_lexicon_tags_37 },
    { 0x33824e9f, SUC_lexicon_tags_8 },
    { 0xe6424ea4, SUC_lexicon_tags_39 },
    { 0x74824ea5, SUC_lexicon_tags_46 },
    { 0xec824ea6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeae24ea8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe624eab, SUC_lexicon_tags_17 },
    { 0x64a24eab, SUC_lexicon_tags_28 },
    { 0xb5c24eab, SUC_lexicon_tags_45 },
    { 0x10824eae, SUC_lexicon_tags_1146 },
    { 0xe3a24eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3224eb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea24ebd, SUC_lexicon_tags_1 },
    { 0xf4e24ebe, SUC_lexicon_tags_21 },
    { 0xd2c24ebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27824ec2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8424ec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e24ec7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0624eca, SUC_lexicon_tags_38 },
    { 0x5ca24eca, SUC_lexicon_tags_198 },
    { 0xdc424eca, SUC_lexicon_tags_17 },
    { 0x38624ecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda24ed2, SUC_lexicon_tags_17 },
    { 0x92424ed2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22624ed6, SUC_lexicon_tags_37 },
    { 0xfc624ed7, SUC_lexicon_tags_17 },
    { 0xb3824ed8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae24edc, SUC_lexicon_tags_25 },
    { 0xc3824edd, SUC_lexicon_tags_37 },
    { 0xe6e24edc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf024ee0, SUC_lexicon_tags_0 },
    { 0x90e24ee1, SUC_lexicon_tags_33 },
    { 0x59e24ee2, SUC_lexicon_tags_25 },
    { 0x1824ee1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a624ee5, SUC_lexicon_tags_60 },
    { 0x1f424ee6, SUC_lexicon_tags_37 },
    { 0x61624ee7, SUC_lexicon_tags_17 },
    { 0x1e24ee5, SUC_lexicon_tags_25 },
    { 0xc9424ee9, SUC_lexicon_tags_37 },
    { 0x12a24ee7, SUC_lexicon_tags_43 },
    { 0xb0e24eeb, SUC_lexicon_tags_17 },
    { 0xc1e24eec, SUC_lexicon_tags_87 },
    { 0xba224eed, SUC_lexicon_tags_17 },
    { 0xab224eed, SUC_lexicon_tags_28 },
    { 0xeae24eef, SUC_lexicon_tags_14 },
    { 0x67624eef, SUC_lexicon_tags_38 },
    { 0x1b424eec, SUC_lexicon_tags_11 },
    { 0x2de24ef2, SUC_lexicon_tags_4 },
    { 0xea224ef1, SUC_lexicon_tags_31 },
    { 0x4f024ef4, SUC_lexicon_tags_37 },
    { 0x81024ef5, SUC_lexicon_tags_82 },
    { 0x99824ef5, SUC_lexicon_tags_7 },
    { 0x3b024ef7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a824efb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c24efe, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e24f01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c24f05, SUC_lexicon_tags_2 },
    { 0x5b424f05, SUC_lexicon_tags_3 },
    { 0xda624f06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fc24f09, SUC_lexicon_tags_77 },
    { 0x20824f09, SUC_lexicon_tags_15 },
    { 0xde424f0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a24f0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a24f15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b224f18, SUC_lexicon_tags_11 },
    { 0xe6224f19, SUC_lexicon_tags_17 },
    { 0xa7424f18, SUC_lexicon_tags_19 },
    { 0x8e024f1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x65024f1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be24f20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab624f22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe24f29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e424f2f, SUC_lexicon_tags_27 },
    { 0xc24f30, SUC_lexicon_tags_63 },
    { 0xef024f31, SUC_lexicon_tags_15 },
    { 0xf2a24f32, SUC_lexicon_tags_14 },
    { 0xd4824f30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd824f3d, SUC_lexicon_tags_26 },
    { 0x17424f3d, SUC_lexicon_tags_76 },
    { 0x2e24f3f, SUC_lexicon_tags_26 },
    { 0x91e24f3f, SUC_lexicon_tags_38 },
    { 0x8c24f40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e24f47, SUC_lexicon_tags_37 },
    { 0x23824f48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x28024f4b, SUC_lexicon_tags_15 },
    { 0xeb424f4b, SUC_lexicon_tags_15 },
    { 0xfec24f4c, SUC_lexicon_tags_37 },
    { 0xfec24f4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb024f50, SUC_lexicon_tags_107 },
    { 0x51224f51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9824f54, SUC_lexicon_tags_2 },
    { 0xdd024f55, SUC_lexicon_tags_37 },
    { 0x1a24f55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5024f58, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52224f5a, SUC_lexicon_tags_77 },
    { 0x8ee24f5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e24f5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9fe24f60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x17824f62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xba824f64, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x16e24f66, SUC_lexicon_tags_47 },
    { 0x1c024f67, SUC_lexicon_tags_37 },
    { 0x3a824f67, SUC_lexicon_tags_6 },
    { 0x63c24f68, SUC_lexicon_tags_15 },
    { 0x7cc24f6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3424f70, SUC_lexicon_tags_12 },
    { 0xde624f71, SUC_lexicon_tags_63 },
    { 0x15424f71, SUC_lexicon_tags_28 },
    { 0xbaa24f73, SUC_lexicon_tags_37 },
    { 0xa2624f72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63624f77, SUC_lexicon_tags_2 },
    { 0xfca24f78, SUC_lexicon_tags_30 },
    { 0x2f224f78, SUC_lexicon_tags_172 },
    { 0x4e224f79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc24f7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c24f82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c624f85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c224f89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1824f8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17424f97, SUC_lexicon_tags_2 },
    { 0xa7a24f98, SUC_lexicon_tags_13 },
    { 0x58024f99, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43224fa1, SUC_lexicon_tags_60 },
    { 0x24424fa1, SUC_lexicon_tags_7 },
    { 0xf8224fa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec24fa7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8a024fa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62a24fab, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed024faf, SUC_lexicon_tags_8 },
    { 0x32a24fb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a24fb7, SUC_lexicon_tags_12 },
    { 0x15624fb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d824fbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1624fc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92224fca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824fd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e24fd6, SUC_lexicon_tags_7 },
    { 0x53e24fd6, SUC_lexicon_tags_26 },
    { 0xce624fd8, SUC_lexicon_tags_15 },
    { 0xdac24fd6, SUC_lexicon_tags_2 },
    { 0xf9e24fda, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5424fdf, SUC_lexicon_tags_1 },
    { 0x8024fe0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7024fe7, SUC_lexicon_tags_12 },
    { 0xc6024fe8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c24fee, SUC_lexicon_tags_28 },
    { 0x95624fef, SUC_lexicon_tags_82 },
    { 0xa424fef, SUC_lexicon_tags_37 },
    { 0x6f624ff0, SUC_lexicon_tags_70 },
    { 0x60624fee, SUC_lexicon_tags_11 },
    { 0xfca24fef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc024ff6, SUC_lexicon_tags_7 },
    { 0xf1a24ff7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84425000, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28625003, SUC_lexicon_tags_1 },
    { 0xbee25003, SUC_lexicon_tags_7 },
    { 0x33225004, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32625009, SUC_lexicon_tags_23 },
    { 0x6aa25009, SUC_lexicon_tags_81 },
    { 0x35e2500b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c2500e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94e25010, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8f225012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39625015, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e625018, SUC_lexicon_tags_26 },
    { 0xa0c25018, SUC_lexicon_tags_129 },
    { 0xc8e25019, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7542501c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff425026, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea25035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd425038, SUC_lexicon_tags_0 },
    { 0x71425039, SUC_lexicon_tags_104 },
    { 0x10c25039, SUC_lexicon_tags_11 },
    { 0x7482503b, SUC_lexicon_tags_40 },
    { 0xf662503c, SUC_lexicon_tags_31 },
    { 0xd5e2503c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a25041, SUC_lexicon_tags_2 },
    { 0x5b825042, SUC_lexicon_tags_27 },
    { 0x55025042, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x30a25045, SUC_lexicon_tags_49 },
    { 0x76e25046, SUC_lexicon_tags_2 },
    { 0xe6425047, SUC_lexicon_tags_81 },
    { 0x4c625048, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc2504e, SUC_lexicon_tags_82 },
    { 0xa082504f, SUC_lexicon_tags_14 },
    { 0x5ac25050, SUC_lexicon_tags_47 },
    { 0x8ea25050, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xca025053, SUC_lexicon_tags_19 },
    { 0x80425054, SUC_lexicon_tags_3 },
    { 0xad625055, SUC_lexicon_tags_2 },
    { 0xd7c25056, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a25059, SUC_lexicon_tags_17 },
    { 0x5042505a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a2505e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a25061, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee25064, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede25068, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922506c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5802506e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a25077, SUC_lexicon_tags_15 },
    { 0xf5425078, SUC_lexicon_tags_11 },
    { 0x11e25079, SUC_lexicon_tags_46 },
    { 0x6542507a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3242507d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99225080, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a25086, SUC_lexicon_tags_2 },
    { 0x6425086, SUC_lexicon_tags_17 },
    { 0x17825088, SUC_lexicon_tags_87 },
    { 0x9f425089, SUC_lexicon_tags_2 },
    { 0x9e02508a, SUC_lexicon_tags_17 },
    { 0x7402508b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8002508e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8a25090, SUC_lexicon_tags_49 },
    { 0x10225091, SUC_lexicon_tags_17 },
    { 0xc5625091, SUC_lexicon_tags_45 },
    { 0x3aa25092, SUC_lexicon_tags_12 },
    { 0x81825094, SUC_lexicon_tags_12 },
    { 0x2b425095, SUC_lexicon_tags_25 },
    { 0xb6e25096, SUC_lexicon_tags_38 },
    { 0x6ac25096, SUC_lexicon_tags_31 },
    { 0xaaa25097, SUC_lexicon_tags_49 },
    { 0x1a025096, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba2509d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6562509f, SUC_lexicon_tags_15 },
    { 0x35a250a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x572250a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea250a6, SUC_lexicon_tags_18 },
    { 0xab8250a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac250ad, SUC_lexicon_tags_32 },
    { 0xdb0250ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c250b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x168250b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce250be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0250c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e250ce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508250d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c250d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2250da, SUC_lexicon_tags_12 },
    { 0xa8a250da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966250df, SUC_lexicon_tags_6 },
    { 0x2aa250e0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc250ea, SUC_lexicon_tags_61 },
    { 0x1d2250eb, SUC_lexicon_tags_26 },
    { 0x3ec250eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0250f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa250f9, SUC_lexicon_tags_25 },
    { 0x9ca250fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd56250fc, SUC_lexicon_tags_27 },
    { 0x6de250fc, SUC_lexicon_tags_16 },
    { 0x8de250fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x15e25100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafe25102, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x26425104, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d22510a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc822510c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8c2510e, SUC_lexicon_tags_27 },
    { 0x5402510e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4225111, SUC_lexicon_tags_70 },
    { 0xbfe25112, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc2a25114, SUC_lexicon_tags_9 },
    { 0x10c25114, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a2511a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd42511c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17825124, SUC_lexicon_tags_7 },
    { 0xca025124, SUC_lexicon_tags_6 },
    { 0x48025126, SUC_lexicon_tags_30 },
    { 0x76225127, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x83025129, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c2512c, SUC_lexicon_tags_37 },
    { 0x48e2512d, SUC_lexicon_tags_7 },
    { 0xb042512d, SUC_lexicon_tags_25 },
    { 0xac02512d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe825133, SUC_lexicon_tags_37 },
    { 0xfce25134, SUC_lexicon_tags_17 },
    { 0x65625135, SUC_lexicon_tags_1 },
    { 0x6a025136, SUC_lexicon_tags_7 },
    { 0xe1225137, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42513a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xeb82513c, SUC_lexicon_tags_47 },
    { 0x8ea2513d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c25141, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x18a25143, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7825145, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x84a25147, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc2514d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc02514f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37025151, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac25156, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c42515c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c225161, SUC_lexicon_tags_8 },
    { 0x9425161, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x2ae25164, SUC_lexicon_tags_45 },
    { 0x31a25165, SUC_lexicon_tags_27 },
    { 0x67c25166, SUC_lexicon_tags_25 },
    { 0x36025167, SUC_lexicon_tags_60 },
    { 0x57225168, SUC_lexicon_tags_11 },
    { 0x12025169, SUC_lexicon_tags_32 },
    { 0x30e2516a, SUC_lexicon_tags_49 },
    { 0xfc2516b, SUC_lexicon_tags_25 },
    { 0x4c2516b, SUC_lexicon_tags_60 },
    { 0xbf42516b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e225170, SUC_lexicon_tags_47 },
    { 0x16425171, SUC_lexicon_tags_11 },
    { 0x98225171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefa25174, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xff225177, SUC_lexicon_tags_27 },
    { 0x2dc25178, SUC_lexicon_tags_17 },
    { 0x21025179, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be2517c, SUC_lexicon_tags_37 },
    { 0xb382517d, SUC_lexicon_tags_12 },
    { 0xf182517c, SUC_lexicon_tags_27 },
    { 0x5ce2517f, SUC_lexicon_tags_12 },
    { 0xc502517c, SUC_lexicon_tags_17 },
    { 0x65025181, SUC_lexicon_tags_0 },
    { 0x92e25182, SUC_lexicon_tags_12 },
    { 0xe7c25181, SUC_lexicon_tags_23 },
    { 0x85825184, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7162518d, SUC_lexicon_tags_74 },
    { 0x702518d, SUC_lexicon_tags_63 },
    { 0xd4a2518f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79025191, SUC_lexicon_tags_235 },
    { 0x9da25191, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60251a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0251a3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08251a7, SUC_lexicon_tags_34 },
    { 0xf78251a8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a251ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40251af, SUC_lexicon_tags_47 },
    { 0x17c251b0, SUC_lexicon_tags_60 },
    { 0x108251b1, SUC_lexicon_tags_25 },
    { 0x7dc251b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43e251b4, SUC_lexicon_tags_69 },
    { 0xfa8251b4, SUC_lexicon_tags_2 },
    { 0x622251b6, SUC_lexicon_tags_15 },
    { 0x862251b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f8251b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2251c2, SUC_lexicon_tags_48 },
    { 0xa12251c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96251c8, SUC_lexicon_tags_38 },
    { 0x266251c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65e251cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf04251cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4251d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc22251d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x790251d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4251d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9ee251da, SUC_lexicon_tags_15 },
    { 0x864251db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c251e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x868251e2, SUC_lexicon_tags_7 },
    { 0xb16251e3, SUC_lexicon_tags_0 },
    { 0x4c0251e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xef8251e6, SUC_lexicon_tags_37 },
    { 0x2c0251e6, SUC_lexicon_tags_14 },
    { 0x2a2251e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a251ef, SUC_lexicon_tags_31 },
    { 0x5be251ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32251f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeec251fb, SUC_lexicon_tags_25 },
    { 0x7c0251fc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x236251fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x39025201, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7825204, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c825207, SUC_lexicon_tags_1 },
    { 0xd5c25207, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e2520b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ae2520d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b825213, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f425218, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2521d, SUC_lexicon_tags_17 },
    { 0xa42521d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf625222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb825225, SUC_lexicon_tags_1 },
    { 0xde625225, SUC_lexicon_tags_12 },
    { 0xbc425227, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaf625229, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7942522c, SUC_lexicon_tags_1 },
    { 0x5ca2522d, SUC_lexicon_tags_60 },
    { 0x45a2522e, SUC_lexicon_tags_12 },
    { 0x6202522f, SUC_lexicon_tags_17 },
    { 0x9fa2522f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6ce25232, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc2523d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee825241, SUC_lexicon_tags_2 },
    { 0x83a25241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7825245, SUC_lexicon_tags_17 },
    { 0xff025246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c2524b, SUC_lexicon_tags_26 },
    { 0x1aa2524c, SUC_lexicon_tags_7 },
    { 0x8fa2524c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c25253, SUC_lexicon_tags_11 },
    { 0x98a25253, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x87225256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70825258, SUC_lexicon_tags_15 },
    { 0x39a25258, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4722525b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c82525d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf225260, SUC_lexicon_tags_11 },
    { 0x44625261, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5425265, SUC_lexicon_tags_29 },
    { 0x4025265, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0025269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e025272, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a025278, SUC_lexicon_tags_1 },
    { 0xad425279, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf22527e, SUC_lexicon_tags_15 },
    { 0x5ea2527f, SUC_lexicon_tags_15 },
    { 0xf0e25280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac25285, SUC_lexicon_tags_26 },
    { 0x81625286, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a2528b, SUC_lexicon_tags_27 },
    { 0x8e42528c, SUC_lexicon_tags_68 },
    { 0x78a2528c, SUC_lexicon_tags_15 },
    { 0x4ce2528e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab625291, SUC_lexicon_tags_7 },
    { 0x7d425292, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa2529c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc252a3, SUC_lexicon_tags_60 },
    { 0xaf0252a4, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x810252a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa78252a8, SUC_lexicon_tags_15 },
    { 0x66a252a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0252ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76252b0, SUC_lexicon_tags_1 },
    { 0xf2e252b1, SUC_lexicon_tags_63 },
    { 0x4da252b2, SUC_lexicon_tags_63 },
    { 0x342252b1, SUC_lexicon_tags_9 },
    { 0x728252b4, SUC_lexicon_tags_61 },
    { 0x7de252b4, SUC_lexicon_tags_27 },
    { 0x1fe252b0, SUC_lexicon_tags_32 },
    { 0xc38252b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf84252b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e252bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10252c0, SUC_lexicon_tags_37 },
    { 0x424252c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26c252c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c252c7, SUC_lexicon_tags_14 },
    { 0x98a252c8, SUC_lexicon_tags_30 },
    { 0xd9e252c9, SUC_lexicon_tags_14 },
    { 0xcb2252ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c4252cc, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x88c252ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4252d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x676252d8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8252e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c252e3, SUC_lexicon_tags_40 },
    { 0x7c6252e4, SUC_lexicon_tags_3 },
    { 0xb4c252e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce0252e7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8252ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22252ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352252fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a025302, SUC_lexicon_tags_17 },
    { 0xe7a25303, SUC_lexicon_tags_17 },
    { 0x99c25304, SUC_lexicon_tags_37 },
    { 0x21e25305, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8825309, SUC_lexicon_tags_7 },
    { 0x3742530a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33a2530c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9625310, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ec25312, SUC_lexicon_tags_36 },
    { 0xc3e25313, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a25318, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a2531d, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c25321, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2425325, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc0c25327, SUC_lexicon_tags_15 },
    { 0x825328, SUC_lexicon_tags_10 },
    { 0xd1025328, SUC_lexicon_tags_17 },
    { 0x3425327, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a82532c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8962532e, SUC_lexicon_tags_68 },
    { 0xaea2532f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f825332, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x75a25334, SUC_lexicon_tags_43 },
    { 0x8e625335, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5842533a, SUC_lexicon_tags_17 },
    { 0x6b62533a, SUC_lexicon_tags_1 },
    { 0x3c2533c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e62533f, SUC_lexicon_tags_33 },
    { 0x40025340, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba25346, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x48a25348, SUC_lexicon_tags_7 },
    { 0x64025349, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x8ee2534b, SUC_lexicon_tags_28 },
    { 0x2462534b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x72c2534e, SUC_lexicon_tags_45 },
    { 0xf322534f, SUC_lexicon_tags_1 },
    { 0x982534f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a25355, SUC_lexicon_tags_11 },
    { 0x5de25355, SUC_lexicon_tags_8 },
    { 0x8f425355, SUC_lexicon_tags_1 },
    { 0x72a25355, SUC_lexicon_tags_25 },
    { 0x6f625358, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4782535b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc2535f, SUC_lexicon_tags_28 },
    { 0x9642535f, SUC_lexicon_tags_1 },
    { 0x66c25360, SUC_lexicon_tags_15 },
    { 0xed825362, SUC_lexicon_tags_11 },
    { 0x96e25363, SUC_lexicon_tags_17 },
    { 0x9b225362, SUC_lexicon_tags_11 },
    { 0xf3825363, SUC_lexicon_tags_8 },
    { 0x62625362, SUC_lexicon_tags_12 },
    { 0x8f425367, SUC_lexicon_tags_58 },
    { 0xe5825368, SUC_lexicon_tags_25 },
    { 0x2ee25364, SUC_lexicon_tags_37 },
    { 0x43425369, SUC_lexicon_tags_17 },
    { 0x53c25361, SUC_lexicon_tags_26 },
    { 0x77c2536c, SUC_lexicon_tags_38 },
    { 0x5142536d, SUC_lexicon_tags_25 },
    { 0x1162536c, SUC_lexicon_tags_6 },
    { 0xfee2536f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca25375, SUC_lexicon_tags_17 },
    { 0x82e25375, SUC_lexicon_tags_43 },
    { 0x29025377, SUC_lexicon_tags_13 },
    { 0xa0c25377, SUC_lexicon_tags_8 },
    { 0x7a825377, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a2537f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee425383, SUC_lexicon_tags_9 },
    { 0xb9425383, SUC_lexicon_tags_1 },
    { 0xebe25385, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a25388, SUC_lexicon_tags_25 },
    { 0xa9425389, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc062538b, SUC_lexicon_tags_45 },
    { 0x8da2538c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa302538f, SUC_lexicon_tags_46 },
    { 0xf1225390, SUC_lexicon_tags_25 },
    { 0x7bc25390, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31625396, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c22539d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ac2539f, SUC_lexicon_tags_15 },
    { 0xf52253a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x646253a2, SUC_lexicon_tags_33 },
    { 0x266253a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2253a6, SUC_lexicon_tags_45 },
    { 0x974253a7, SUC_lexicon_tags_28 },
    { 0xffc253a8, SUC_lexicon_tags_158 },
    { 0x96a253a7, SUC_lexicon_tags_12 },
    { 0x30a253aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924253b7, SUC_lexicon_tags_4 },
    { 0x104253b8, SUC_lexicon_tags_12 },
    { 0x22253b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x630253bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1c2253bd, SUC_lexicon_tags_1 },
    { 0xb92253be, SUC_lexicon_tags_60 },
    { 0xb58253bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e253c2, SUC_lexicon_tags_15 },
    { 0x5c8253c3, SUC_lexicon_tags_37 },
    { 0xb70253c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1d0253c6, SUC_lexicon_tags_8 },
    { 0xcee253c7, SUC_lexicon_tags_26 },
    { 0x430253c8, SUC_lexicon_tags_82 },
    { 0xa36253c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768253cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6253d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8253dc, SUC_lexicon_tags_11 },
    { 0x25a253dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4253e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe1a253e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e253ed, SUC_lexicon_tags_15 },
    { 0x844253ee, SUC_lexicon_tags_64 },
    { 0x830253ef, SUC_lexicon_tags_47 },
    { 0x6fc253ee, SUC_lexicon_tags_1 },
    { 0x6ca253ef, SUC_lexicon_tags_37 },
    { 0xd92253f2, SUC_lexicon_tags_1 },
    { 0x10c253f2, SUC_lexicon_tags_31 },
    { 0xbe253f4, SUC_lexicon_tags_56 },
    { 0xaec253f4, SUC_lexicon_tags_25 },
    { 0x18c253f4, SUC_lexicon_tags_35 },
    { 0xae8253ef, SUC_lexicon_tags_37 },
    { 0xf56253f8, SUC_lexicon_tags_4 },
    { 0xbba253f8, SUC_lexicon_tags_908 },
    { 0xe74253fa, SUC_lexicon_tags_171 },
    { 0xa82253fb, SUC_lexicon_tags_3 },
    { 0x170253f5, SUC_lexicon_tags_30 },
    { 0xb88253fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce225404, SUC_lexicon_tags_49 },
    { 0xa4425405, SUC_lexicon_tags_11 },
    { 0xda625406, SUC_lexicon_tags_2 },
    { 0xd9025405, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4642540a, SUC_lexicon_tags_19 },
    { 0xeaa2540a, SUC_lexicon_tags_9 },
    { 0x802540b, SUC_lexicon_tags_2 },
    { 0xc122540d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x54a2540f, SUC_lexicon_tags_48 },
    { 0x95825410, SUC_lexicon_tags_15 },
    { 0x27425410, SUC_lexicon_tags_12 },
    { 0x1a225412, SUC_lexicon_tags_17 },
    { 0xc225413, SUC_lexicon_tags_151 },
    { 0x6625413, SUC_lexicon_tags_28 },
    { 0x6d625415, SUC_lexicon_tags_25 },
    { 0x67425415, SUC_lexicon_tags_8 },
    { 0x1ca25410, SUC_lexicon_tags_28 },
    { 0xa9225418, SUC_lexicon_tags_2 },
    { 0xb4a25412, SUC_lexicon_tags_8 },
    { 0x3122541a, SUC_lexicon_tags_37 },
    { 0x3a25418, SUC_lexicon_tags_2 },
    { 0xf1025418, SUC_lexicon_tags_25 },
    { 0x8d025418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa82541f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a25429, SUC_lexicon_tags_37 },
    { 0xf502542a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae2542e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b825430, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde025433, SUC_lexicon_tags_25 },
    { 0x9a025434, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x16c25436, SUC_lexicon_tags_30 },
    { 0x62e25437, SUC_lexicon_tags_17 },
    { 0x8d025438, SUC_lexicon_tags_1 },
    { 0xfe25437, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6742543f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c25447, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c2544f, SUC_lexicon_tags_6 },
    { 0xe5a25450, SUC_lexicon_tags_68 },
    { 0x4d825451, SUC_lexicon_tags_7 },
    { 0x84025452, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa9825454, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6025456, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc0425458, SUC_lexicon_tags_49 },
    { 0x36c25459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d82545d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e25460, SUC_lexicon_tags_14 },
    { 0x6e825461, SUC_lexicon_tags_26 },
    { 0x7bc25462, SUC_lexicon_tags_12 },
    { 0x94225461, SUC_lexicon_tags_8 },
    { 0x29a25464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c25469, SUC_lexicon_tags_60 },
    { 0x23a25469, SUC_lexicon_tags_23 },
    { 0x7e22546b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef02546d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9625471, SUC_lexicon_tags_30 },
    { 0x66225472, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d425474, SUC_lexicon_tags_37 },
    { 0x82e25475, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc25478, SUC_lexicon_tags_31 },
    { 0xa7425479, SUC_lexicon_tags_17 },
    { 0x7d42547a, SUC_lexicon_tags_60 },
    { 0xfda25479, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc25480, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d225489, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d82548b, SUC_lexicon_tags_12 },
    { 0x7b22548b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd22548f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a25492, SUC_lexicon_tags_12 },
    { 0xcce25493, SUC_lexicon_tags_21 },
    { 0x57225492, SUC_lexicon_tags_14 },
    { 0x3f625495, SUC_lexicon_tags_63 },
    { 0x65225495, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8062549d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe42549f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a254a3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x152254a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x496254a8, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x42a254aa, SUC_lexicon_tags_47 },
    { 0x784254aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64254af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c254b2, SUC_lexicon_tags_17 },
    { 0xdaa254b2, SUC_lexicon_tags_77 },
    { 0x9ac254b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d4254b6, SUC_lexicon_tags_38 },
    { 0xc42254b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2254ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e254be, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0254c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc88254c4, SUC_lexicon_tags_77 },
    { 0x65c254c5, SUC_lexicon_tags_12 },
    { 0xe62254c4, SUC_lexicon_tags_45 },
    { 0xb02254c7, SUC_lexicon_tags_37 },
    { 0x882254c8, SUC_lexicon_tags_20 },
    { 0x59c254c5, SUC_lexicon_tags_43 },
    { 0x6f4254ca, SUC_lexicon_tags_25 },
    { 0x306254cb, SUC_lexicon_tags_23 },
    { 0x6a0254cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22254cf, SUC_lexicon_tags_11 },
    { 0xe4c254d0, SUC_lexicon_tags_60 },
    { 0xa4254d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04254d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcca254d6, SUC_lexicon_tags_0 },
    { 0x466254d7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180254dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc254df, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x16a254e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2254e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78c254e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf2254e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x118254eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce254ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be254f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8fc254f3, SUC_lexicon_tags_19 },
    { 0xffc254f4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02254fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7fe254fc, SUC_lexicon_tags_25 },
    { 0xcca254fd, SUC_lexicon_tags_6 },
    { 0x38254fd, SUC_lexicon_tags_14 },
    { 0xc68254fd, SUC_lexicon_tags_4 },
    { 0x8da25500, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58a25502, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a25506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca225508, SUC_lexicon_tags_48 },
    { 0xf3a25509, SUC_lexicon_tags_7 },
    { 0x32c2550a, SUC_lexicon_tags_55 },
    { 0xc6c25508, SUC_lexicon_tags_29 },
    { 0xa582550b, SUC_lexicon_tags_43 },
    { 0x4722550d, SUC_lexicon_tags_2 },
    { 0x86025509, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x37e25510, SUC_lexicon_tags_25 },
    { 0x16a25511, SUC_lexicon_tags_14 },
    { 0xede25510, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e25519, SUC_lexicon_tags_113 },
    { 0x3002551a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1682551c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xd362551f, SUC_lexicon_tags_17 },
    { 0x5c025520, SUC_lexicon_tags_37 },
    { 0xd0a25521, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x59625524, SUC_lexicon_tags_0 },
    { 0xf5025525, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e25529, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa082552b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e82552e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c25534, SUC_lexicon_tags_45 },
    { 0xf2a25535, SUC_lexicon_tags_37 },
    { 0xee825536, SUC_lexicon_tags_6 },
    { 0x3425537, SUC_lexicon_tags_33 },
    { 0xcfc25538, SUC_lexicon_tags_2 },
    { 0x17e25535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15c2553b, SUC_lexicon_tags_2 },
    { 0xe942553b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa25544, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a2554e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbfa25550, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1225554, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcb825556, SUC_lexicon_tags_12 },
    { 0x3ec25557, SUC_lexicon_tags_7 },
    { 0x64625558, SUC_lexicon_tags_25 },
    { 0xb6a25558, SUC_lexicon_tags_11 },
    { 0xe662555a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6425563, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a225566, SUC_lexicon_tags_351 },
    { 0x98025566, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb42556d, SUC_lexicon_tags_1 },
    { 0x39e2556d, SUC_lexicon_tags_28 },
    { 0xce02556d, SUC_lexicon_tags_25 },
    { 0x82c25570, SUC_lexicon_tags_15 },
    { 0x25a2556d, SUC_lexicon_tags_31 },
    { 0x9d225572, SUC_lexicon_tags_19 },
    { 0x80425573, SUC_lexicon_tags_26 },
    { 0x95e25574, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6082557a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2557d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe502557f, SUC_lexicon_tags_12 },
    { 0x96e25580, SUC_lexicon_tags_11 },
    { 0xff02557f, SUC_lexicon_tags_11 },
    { 0x7f425582, SUC_lexicon_tags_0 },
    { 0x74c25582, SUC_lexicon_tags_6 },
    { 0x38e25584, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4822558a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1842558d, SUC_lexicon_tags_25 },
    { 0x7082558e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e2559c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786255a0, SUC_lexicon_tags_60 },
    { 0x8255a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa28255a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd4255a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x194255a7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x190255a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a0255ab, SUC_lexicon_tags_17 },
    { 0x2b6255ab, SUC_lexicon_tags_29 },
    { 0x9a255ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd6255af, SUC_lexicon_tags_7 },
    { 0xc4e255b0, SUC_lexicon_tags_60 },
    { 0xb78255af, SUC_lexicon_tags_224 },
    { 0x758255b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e255bd, SUC_lexicon_tags_7 },
    { 0xca255be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940255c2, SUC_lexicon_tags_14 },
    { 0x526255c2, SUC_lexicon_tags_17 },
    { 0x5cc255c3, SUC_lexicon_tags_7 },
    { 0x10255c2, SUC_lexicon_tags_33 },
    { 0xcf4255c6, SUC_lexicon_tags_49 },
    { 0xb5c255c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e255ca, SUC_lexicon_tags_31 },
    { 0x16c255cb, SUC_lexicon_tags_39 },
    { 0x538255cc, SUC_lexicon_tags_19 },
    { 0x2f0255cb, SUC_lexicon_tags_37 },
    { 0x7e2255cd, SUC_lexicon_tags_10 },
    { 0x7f6255cf, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xdda255d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8255d4, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a255d7, SUC_lexicon_tags_41 },
    { 0x822255d7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x200255db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf00255dd, SUC_lexicon_tags_37 },
    { 0x664255de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x780255e0, SUC_lexicon_tags_19 },
    { 0x6e6255e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54255e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0c255e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818255ee, SUC_lexicon_tags_30 },
    { 0x910255ee, SUC_lexicon_tags_9 },
    { 0x6a4255ee, SUC_lexicon_tags_26 },
    { 0x4cc255f1, SUC_lexicon_tags_38 },
    { 0x86255f2, SUC_lexicon_tags_8 },
    { 0x6c4255f3, SUC_lexicon_tags_13 },
    { 0x7dc255f4, SUC_lexicon_tags_7 },
    { 0x1c2255f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c255fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55c255fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22825600, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39025605, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x58825608, SUC_lexicon_tags_2 },
    { 0xf7025608, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc2560c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e25617, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c2561a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfd62561c, SUC_lexicon_tags_37 },
    { 0x782561d, SUC_lexicon_tags_7 },
    { 0x8522561e, SUC_lexicon_tags_66 },
    { 0x3562561e, SUC_lexicon_tags_14 },
    { 0xea82561d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4225624, SUC_lexicon_tags_27 },
    { 0xabc25624, SUC_lexicon_tags_25 },
    { 0x3ee25626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1182562a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec225630, SUC_lexicon_tags_38 },
    { 0x6825631, SUC_lexicon_tags_49 },
    { 0x60c25631, SUC_lexicon_tags_17 },
    { 0x83625631, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22025639, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1962563d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6625640, SUC_lexicon_tags_7 },
    { 0xa6825641, SUC_lexicon_tags_12 },
    { 0xd5c25642, SUC_lexicon_tags_23 },
    { 0xfc25642, SUC_lexicon_tags_25 },
    { 0x68025644, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6625647, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa5a25649, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2002564e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x85c25650, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c25658, SUC_lexicon_tags_2 },
    { 0xd7225659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9962565b, SUC_lexicon_tags_7 },
    { 0x2162565b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e42565e, SUC_lexicon_tags_33 },
    { 0x5d22565f, SUC_lexicon_tags_25 },
    { 0x6c225660, SUC_lexicon_tags_29 },
    { 0xc5c2565e, SUC_lexicon_tags_17 },
    { 0x39825661, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2025665, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b025668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6662566d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0a2566f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd025673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e25678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d62567a, SUC_lexicon_tags_29 },
    { 0x48e2567b, SUC_lexicon_tags_46 },
    { 0x7c62567c, SUC_lexicon_tags_0 },
    { 0xff22567c, SUC_lexicon_tags_12 },
    { 0x1122567d, SUC_lexicon_tags_17 },
    { 0xcc22567f, SUC_lexicon_tags_6 },
    { 0x93425680, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x78625683, SUC_lexicon_tags_63 },
    { 0x58e25684, SUC_lexicon_tags_17 },
    { 0xac625685, SUC_lexicon_tags_2 },
    { 0x77c25685, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be2568c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf002568e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f425690, SUC_lexicon_tags_16 },
    { 0xb8825691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67c25693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52025696, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6825699, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5262569b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130256a5, SUC_lexicon_tags_4 },
    { 0xba0256a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992256aa, SUC_lexicon_tags_48 },
    { 0x762256aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a256ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4256b2, SUC_lexicon_tags_23 },
    { 0x684256b3, SUC_lexicon_tags_37 },
    { 0xc8c256b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8256c0, SUC_lexicon_tags_60 },
    { 0x830256c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2256c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a256ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38256d0, SUC_lexicon_tags_12 },
    { 0xa32256d0, SUC_lexicon_tags_7 },
    { 0xc6e256d2, SUC_lexicon_tags_7 },
    { 0x420256d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa70256d5, SUC_lexicon_tags_12 },
    { 0xe12256d6, SUC_lexicon_tags_25 },
    { 0x570256d6, SUC_lexicon_tags_10 },
    { 0x4fe256d8, SUC_lexicon_tags_37 },
    { 0xece256d9, SUC_lexicon_tags_47 },
    { 0x72c256d6, SUC_lexicon_tags_69 },
    { 0xb2256db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde256dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94256e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6256e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda256e8, SUC_lexicon_tags_15 },
    { 0x9e256e8, SUC_lexicon_tags_32 },
    { 0x4ec256e8, SUC_lexicon_tags_15 },
    { 0x186256eb, SUC_lexicon_tags_12 },
    { 0xec6256ea, SUC_lexicon_tags_29 },
    { 0x81a256eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc256f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd32256f4, SUC_lexicon_tags_11 },
    { 0x72e256f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5c256f7, SUC_lexicon_tags_11 },
    { 0x274256f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe256fb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xff6256fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e25700, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a25706, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a2570e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x55625710, SUC_lexicon_tags_15 },
    { 0xd9225710, SUC_lexicon_tags_61 },
    { 0x90425712, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1425716, SUC_lexicon_tags_1 },
    { 0xf3825716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a2571a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb522571e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c25722, SUC_lexicon_tags_23 },
    { 0xf1e25722, SUC_lexicon_tags_47 },
    { 0x28625724, SUC_lexicon_tags_12 },
    { 0xd5e25724, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40025728, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e2572c, SUC_lexicon_tags_350 },
    { 0x8062572c, SUC_lexicon_tags_12 },
    { 0x33a2572e, SUC_lexicon_tags_27 },
    { 0x3fa2572e, SUC_lexicon_tags_55 },
    { 0xcd225730, SUC_lexicon_tags_11 },
    { 0x9622572e, SUC_lexicon_tags_2 },
    { 0x3825732, SUC_lexicon_tags_29 },
    { 0x15025731, SUC_lexicon_tags_84 },
    { 0x87425731, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1022573a, SUC_lexicon_tags_23 },
    { 0xa7e2573b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e25748, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed62574a, SUC_lexicon_tags_63 },
    { 0xde82574a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xece2574d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe25750, SUC_lexicon_tags_11 },
    { 0xe9c25750, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18225756, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d025759, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e2575c, SUC_lexicon_tags_1 },
    { 0xcf22575c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5862575f, SUC_lexicon_tags_26 },
    { 0x15825760, SUC_lexicon_tags_63 },
    { 0xe4225761, SUC_lexicon_tags_27 },
    { 0x67e25762, SUC_lexicon_tags_563 },
    { 0x3fc25761, SUC_lexicon_tags_17 },
    { 0xa7225763, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b825767, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x92625769, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80825772, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52c25774, SUC_lexicon_tags_17 },
    { 0x5a825774, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c25779, SUC_lexicon_tags_40 },
    { 0x2f225779, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7022577d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb802577f, SUC_lexicon_tags_37 },
    { 0x3b42577f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0025785, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe322578b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a82578d, SUC_lexicon_tags_0 },
    { 0x3be2578e, SUC_lexicon_tags_69 },
    { 0xdfc2578e, SUC_lexicon_tags_17 },
    { 0x78225790, SUC_lexicon_tags_13 },
    { 0xb5c2578f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f225793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa025795, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd8425797, SUC_lexicon_tags_688 },
    { 0xc7225797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa702579b, SUC_lexicon_tags_1 },
    { 0x6362579c, SUC_lexicon_tags_21 },
    { 0x8e02579c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a257a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4257a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc257aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9c0257ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e257b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd0a257b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206257b6, SUC_lexicon_tags_37 },
    { 0x292257b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24257ba, SUC_lexicon_tags_25 },
    { 0xc34257bb, SUC_lexicon_tags_17 },
    { 0x4a257ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5dc257be, SUC_lexicon_tags_109 },
    { 0x20e257be, SUC_lexicon_tags_37 },
    { 0x8da257bf, SUC_lexicon_tags_12 },
    { 0x45c257be, SUC_lexicon_tags_29 },
    { 0xa32257bf, SUC_lexicon_tags_7 },
    { 0xe08257be, SUC_lexicon_tags_7 },
    { 0xff2257c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c257cc, SUC_lexicon_tags_55 },
    { 0xe54257cd, SUC_lexicon_tags_37 },
    { 0x116257cd, SUC_lexicon_tags_47 },
    { 0x8f2257cf, SUC_lexicon_tags_3 },
    { 0x388257d0, SUC_lexicon_tags_11 },
    { 0x95e257d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c257db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918257e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52257e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b4257e6, SUC_lexicon_tags_69 },
    { 0x8d8257e6, SUC_lexicon_tags_15 },
    { 0x3fe257e8, SUC_lexicon_tags_11 },
    { 0xd94257e9, SUC_lexicon_tags_17 },
    { 0x186257ea, SUC_lexicon_tags_15 },
    { 0xc16257e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4257ee, SUC_lexicon_tags_8 },
    { 0x66c257ef, SUC_lexicon_tags_1 },
    { 0xd10257ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4257f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc257fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2257ff, SUC_lexicon_tags_11 },
    { 0x350257ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7625803, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c25809, SUC_lexicon_tags_23 },
    { 0xcce2580a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2282580e, SUC_lexicon_tags_542 },
    { 0x1022580e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c25811, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48e25813, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb502581c, SUC_lexicon_tags_33 },
    { 0xe462581c, SUC_lexicon_tags_30 },
    { 0x2722581e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x71025820, SUC_lexicon_tags_31 },
    { 0xd8c25821, SUC_lexicon_tags_14 },
    { 0xe3a25821, SUC_lexicon_tags_1 },
    { 0x85a25823, SUC_lexicon_tags_14 },
    { 0xed225823, SUC_lexicon_tags_2 },
    { 0x71a25821, SUC_lexicon_tags_14 },
    { 0x48c25825, SUC_lexicon_tags_21 },
    { 0xbc225827, SUC_lexicon_tags_1 },
    { 0xbc025824, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde2582d, SUC_lexicon_tags_49 },
    { 0x4be2582d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fa25830, SUC_lexicon_tags_6 },
    { 0x8d025831, SUC_lexicon_tags_17 },
    { 0x4a025831, SUC_lexicon_tags_11 },
    { 0xf9a25830, SUC_lexicon_tags_33 },
    { 0x9aa25834, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7625837, SUC_lexicon_tags_15 },
    { 0xa4c25837, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a2583a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2583e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2e825840, SUC_lexicon_tags_70 },
    { 0x1fa25841, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc25844, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5825846, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x24025848, SUC_lexicon_tags_27 },
    { 0x44e25849, SUC_lexicon_tags_17 },
    { 0xc4625849, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3002584d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21e2584f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xccc25851, SUC_lexicon_tags_113 },
    { 0x21225852, SUC_lexicon_tags_17 },
    { 0x9dc25853, SUC_lexicon_tags_37 },
    { 0x5ee25854, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10225856, SUC_lexicon_tags_25 },
    { 0xc6a25857, SUC_lexicon_tags_55 },
    { 0x5b625858, SUC_lexicon_tags_65 },
    { 0xb3225857, SUC_lexicon_tags_31 },
    { 0x24225856, SUC_lexicon_tags_12 },
    { 0x5e225857, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6ec2585d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7342585f, SUC_lexicon_tags_65 },
    { 0x1cc25860, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc625863, SUC_lexicon_tags_66 },
    { 0xcc625863, SUC_lexicon_tags_19 },
    { 0xe5225863, SUC_lexicon_tags_7 },
    { 0x95225866, SUC_lexicon_tags_16 },
    { 0x74825867, SUC_lexicon_tags_87 },
    { 0x77025866, SUC_lexicon_tags_48 },
    { 0x35625868, SUC_lexicon_tags_1 },
    { 0x4442586a, SUC_lexicon_tags_11 },
    { 0x14c25864, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa462586d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3825870, SUC_lexicon_tags_15 },
    { 0xcd425870, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53225876, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4cc25878, SUC_lexicon_tags_5 },
    { 0xf0625879, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5262587e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ea25880, SUC_lexicon_tags_2 },
    { 0x6ec25881, SUC_lexicon_tags_17 },
    { 0x7225880, SUC_lexicon_tags_12 },
    { 0x4de25883, SUC_lexicon_tags_11 },
    { 0x9da25884, SUC_lexicon_tags_11 },
    { 0x18225885, SUC_lexicon_tags_16 },
    { 0xd9825885, SUC_lexicon_tags_15 },
    { 0x2f625887, SUC_lexicon_tags_31 },
    { 0x76225881, SUC_lexicon_tags_23 },
    { 0x5da25887, SUC_lexicon_tags_27 },
    { 0x4d625888, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb025898, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a82589e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x566258a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a258a6, SUC_lexicon_tags_9 },
    { 0x6b8258a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4258b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6258b8, SUC_lexicon_tags_0 },
    { 0xe84258b9, SUC_lexicon_tags_7 },
    { 0x86e258ba, SUC_lexicon_tags_25 },
    { 0x73e258ba, SUC_lexicon_tags_36 },
    { 0x970258ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62258bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12258c5, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212258cd, SUC_lexicon_tags_23 },
    { 0x3e6258ce, SUC_lexicon_tags_12 },
    { 0x856258cf, SUC_lexicon_tags_7 },
    { 0x33a258d0, SUC_lexicon_tags_1 },
    { 0x8ea258ce, SUC_lexicon_tags_15 },
    { 0xf04258d2, SUC_lexicon_tags_8 },
    { 0xefe258d3, SUC_lexicon_tags_14 },
    { 0x8c258d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea258d9, SUC_lexicon_tags_37 },
    { 0xab6258d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0258e0, SUC_lexicon_tags_17 },
    { 0xe00258e1, SUC_lexicon_tags_11 },
    { 0x336258e2, SUC_lexicon_tags_15 },
    { 0x83a258e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46258e6, SUC_lexicon_tags_16 },
    { 0xd9c258e7, SUC_lexicon_tags_16 },
    { 0x6b0258e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60258eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d2258ed, SUC_lexicon_tags_14 },
    { 0xe2e258ee, SUC_lexicon_tags_46 },
    { 0xefc258ef, SUC_lexicon_tags_38 },
    { 0x806258ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4258f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x804258f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498258fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25901, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2025905, SUC_lexicon_tags_18 },
    { 0xa25906, SUC_lexicon_tags_6 },
    { 0x7dc25907, SUC_lexicon_tags_30 },
    { 0x6e025906, SUC_lexicon_tags_38 },
    { 0x1b425906, SUC_lexicon_tags_33 },
    { 0xa282590a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebe2590c, SUC_lexicon_tags_37 },
    { 0xd822590c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec25910, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc562591b, SUC_lexicon_tags_45 },
    { 0xf042591c, SUC_lexicon_tags_17 },
    { 0xb7a2591b, SUC_lexicon_tags_82 },
    { 0x81c2591e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6425921, SUC_lexicon_tags_37 },
    { 0xfde25922, SUC_lexicon_tags_32 },
    { 0x44e25923, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9425925, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a2592b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c42592e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f025931, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c25935, SUC_lexicon_tags_39 },
    { 0x10625935, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32825939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a2593c, SUC_lexicon_tags_19 },
    { 0x7d22593c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39825940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc425942, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e025944, SUC_lexicon_tags_25 },
    { 0xe0625944, SUC_lexicon_tags_49 },
    { 0x75225946, SUC_lexicon_tags_15 },
    { 0xb3a25947, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe282594c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c25952, SUC_lexicon_tags_17 },
    { 0xcc025953, SUC_lexicon_tags_37 },
    { 0xdec25954, SUC_lexicon_tags_37 },
    { 0x53025954, SUC_lexicon_tags_26 },
    { 0xfe25954, SUC_lexicon_tags_87 },
    { 0xb1a25953, SUC_lexicon_tags_30 },
    { 0x6e425953, SUC_lexicon_tags_2 },
    { 0xbe025959, SUC_lexicon_tags_13 },
    { 0x83a25959, SUC_lexicon_tags_11 },
    { 0x45e25959, SUC_lexicon_tags_29 },
    { 0xb422595c, SUC_lexicon_tags_11 },
    { 0x3002595d, SUC_lexicon_tags_249 },
    { 0x9dc25959, SUC_lexicon_tags_17 },
    { 0x1962595e, SUC_lexicon_tags_7 },
    { 0xd3425960, SUC_lexicon_tags_2 },
    { 0xee025961, SUC_lexicon_tags_8 },
    { 0xd3625957, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3302596e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e25973, SUC_lexicon_tags_37 },
    { 0x8da25974, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce825976, SUC_lexicon_tags_25 },
    { 0x38e25977, SUC_lexicon_tags_25 },
    { 0x4dc25976, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb42597b, SUC_lexicon_tags_17 },
    { 0x2ca2597b, SUC_lexicon_tags_2 },
    { 0x8082597d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5825985, SUC_lexicon_tags_0 },
    { 0xa4025986, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa402598b, SUC_lexicon_tags_17 },
    { 0x6ac2598c, SUC_lexicon_tags_14 },
    { 0x9e42598d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8c2598f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed425991, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe25998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a2599d, SUC_lexicon_tags_25 },
    { 0xa722599e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6259a3, SUC_lexicon_tags_60 },
    { 0x5da259a3, SUC_lexicon_tags_21 },
    { 0x554259a4, SUC_lexicon_tags_11 },
    { 0x8ee259a6, SUC_lexicon_tags_65 },
    { 0x45e259a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c259aa, SUC_lexicon_tags_16 },
    { 0x82a259ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81a259ad, SUC_lexicon_tags_47 },
    { 0x8e0259ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xede259b0, SUC_lexicon_tags_12 },
    { 0xb18259b1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a259b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee2259b6, SUC_lexicon_tags_15 },
    { 0xb94259b7, SUC_lexicon_tags_15 },
    { 0x2da259b7, SUC_lexicon_tags_15 },
    { 0x8c0259b9, SUC_lexicon_tags_17 },
    { 0x810259b9, SUC_lexicon_tags_8 },
    { 0x874259bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0259c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618259c8, SUC_lexicon_tags_28 },
    { 0x11c259c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78259cc, SUC_lexicon_tags_17 },
    { 0x742259cd, SUC_lexicon_tags_47 },
    { 0xea4259ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1a259d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60259d3, SUC_lexicon_tags_4 },
    { 0x2ee259d3, SUC_lexicon_tags_26 },
    { 0x3a259d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798259d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x38259db, SUC_lexicon_tags_13 },
    { 0x2cc259dc, SUC_lexicon_tags_62 },
    { 0xd3e259dd, SUC_lexicon_tags_60 },
    { 0x998259de, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e259e2, SUC_lexicon_tags_27 },
    { 0x6d2259e2, SUC_lexicon_tags_1 },
    { 0xcf6259e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646259e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660259ee, SUC_lexicon_tags_1 },
    { 0x61e259ef, SUC_lexicon_tags_13 },
    { 0x6e259f0, SUC_lexicon_tags_15 },
    { 0x24e259ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e259f4, SUC_lexicon_tags_1 },
    { 0x4f4259f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6b4259f7, SUC_lexicon_tags_15 },
    { 0xec6259f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4fe259fa, SUC_lexicon_tags_17 },
    { 0x658259fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc08259fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaf0259ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4425a04, SUC_lexicon_tags_15 },
    { 0xcc225a05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e25a09, SUC_lexicon_tags_31 },
    { 0x52425a0a, SUC_lexicon_tags_17 },
    { 0xc9025a0b, SUC_lexicon_tags_15 },
    { 0xd7425a0a, SUC_lexicon_tags_10 },
    { 0x34c25a09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe425a11, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca625a1b, SUC_lexicon_tags_14 },
    { 0x63a25a1c, SUC_lexicon_tags_35 },
    { 0x56625a1b, SUC_lexicon_tags_26 },
    { 0x5c025a1d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xce025a21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56825a23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd225a26, SUC_lexicon_tags_55 },
    { 0xc6825a27, SUC_lexicon_tags_45 },
    { 0xa3e25a28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x26825a2a, SUC_lexicon_tags_88 },
    { 0xe4225a2b, SUC_lexicon_tags_28 },
    { 0xfe425a2a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb9e25a2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e825a31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fe25a33, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a25a36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4825a38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15825a3c, SUC_lexicon_tags_1 },
    { 0xbac25a3d, SUC_lexicon_tags_1 },
    { 0x14625a3e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae25a44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9025a47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc25a4a, SUC_lexicon_tags_7 },
    { 0xed025a4b, SUC_lexicon_tags_28 },
    { 0x86025a4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c25a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97425a51, SUC_lexicon_tags_55 },
    { 0xada25a51, SUC_lexicon_tags_8 },
    { 0x5e425a53, SUC_lexicon_tags_25 },
    { 0x6fa25a54, SUC_lexicon_tags_48 },
    { 0x4be25a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e025a5d, SUC_lexicon_tags_7 },
    { 0xa8e25a5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3825a61, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x1a225a63, SUC_lexicon_tags_14 },
    { 0xa3625a64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27025a66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2825a69, SUC_lexicon_tags_25 },
    { 0x2ba25a6a, SUC_lexicon_tags_39 },
    { 0x43025a6b, SUC_lexicon_tags_7 },
    { 0xd825a6b, SUC_lexicon_tags_44 },
    { 0x42c25a6d, SUC_lexicon_tags_3 },
    { 0x7b825a6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73625a70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x83025a73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24a25a75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3a25a77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27c25a79, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x14425a7b, SUC_lexicon_tags_33 },
    { 0x7c425a7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac25a7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0825a81, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x66425a84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc25a86, SUC_lexicon_tags_15 },
    { 0x70a25a87, SUC_lexicon_tags_38 },
    { 0x2fe25a86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55425a8a, SUC_lexicon_tags_11 },
    { 0x75825a8a, SUC_lexicon_tags_7 },
    { 0x61e25a8c, SUC_lexicon_tags_21 },
    { 0xdda25a8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x42825a8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e825a94, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e25a9d, SUC_lexicon_tags_25 },
    { 0x68625a9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58225aa1, SUC_lexicon_tags_2 },
    { 0x29225aa2, SUC_lexicon_tags_25 },
    { 0x38a25aa3, SUC_lexicon_tags_11 },
    { 0x7a425aa4, SUC_lexicon_tags_63 },
    { 0xfd225aa5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa25aa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78225aaa, SUC_lexicon_tags_32 },
    { 0x44a25aab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda25ab1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b225ab4, SUC_lexicon_tags_68 },
    { 0x4c425ab5, SUC_lexicon_tags_65 },
    { 0x83e25ab6, SUC_lexicon_tags_29 },
    { 0x1825ab6, SUC_lexicon_tags_18 },
    { 0x1ae25ab4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38825aba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x22225abc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d625ac1, SUC_lexicon_tags_12 },
    { 0x26c25ac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63e25ac4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc9a25ac6, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74025aca, SUC_lexicon_tags_23 },
    { 0x5d625acb, SUC_lexicon_tags_12 },
    { 0xb8225acb, SUC_lexicon_tags_8 },
    { 0x97a25acd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b025ad0, SUC_lexicon_tags_27 },
    { 0x7825ad0, SUC_lexicon_tags_0 },
    { 0xc5e25ad1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c25ad5, SUC_lexicon_tags_14 },
    { 0x67625ad5, SUC_lexicon_tags_30 },
    { 0xf2425ad7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a825ad9, SUC_lexicon_tags_66 },
    { 0xd225ada, SUC_lexicon_tags_69 },
    { 0x42625ada, SUC_lexicon_tags_15 },
    { 0xc2e25ada, SUC_lexicon_tags_14 },
    { 0x43625ada, SUC_lexicon_tags_37 },
    { 0x8025ade, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68625ae0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b425ae2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba25ae5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a25aeb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b025aee, SUC_lexicon_tags_37 },
    { 0x9f025aef, SUC_lexicon_tags_37 },
    { 0x95c25aee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3425af4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e025afb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c425b06, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9625b0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15625b10, SUC_lexicon_tags_29 },
    { 0x6f625b11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f825b18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50225b20, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a25b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa25b28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a25b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a025b31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5425b34, SUC_lexicon_tags_32 },
    { 0x35225b34, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64625b3c, SUC_lexicon_tags_2 },
    { 0x24a25b3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28425b3f, SUC_lexicon_tags_25 },
    { 0xe3a25b40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf0025b42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x74a25b44, SUC_lexicon_tags_28 },
    { 0x25025b45, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec25b49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a25b4e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf5425b50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86825b57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6825b59, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a25b5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x40425b60, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de25b66, SUC_lexicon_tags_7 },
    { 0x6e225b67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x27425b6a, SUC_lexicon_tags_170 },
    { 0x91825b6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae25b6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd2225b70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8a25b72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa25b76, SUC_lexicon_tags_12 },
    { 0xbbe25b77, SUC_lexicon_tags_14 },
    { 0xe6e25b78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a25b7e, SUC_lexicon_tags_2 },
    { 0xd7e25b7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa25b83, SUC_lexicon_tags_49 },
    { 0x47c25b83, SUC_lexicon_tags_68 },
    { 0x4e25b85, SUC_lexicon_tags_28 },
    { 0x85a25b83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafe25b88, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xec625b8b, SUC_lexicon_tags_12 },
    { 0x52a25b8c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be25b97, SUC_lexicon_tags_8 },
    { 0x9aa25b97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b825b9c, SUC_lexicon_tags_28 },
    { 0x9b425b9c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43425ba2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3b825ba4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x95425ba7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x63425baa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62825bac, SUC_lexicon_tags_7 },
    { 0x8ea25bad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a25bb7, SUC_lexicon_tags_15 },
    { 0xa1825bb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1225bba, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x39c25bbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece25bc9, SUC_lexicon_tags_1 },
    { 0x56e25bca, SUC_lexicon_tags_16 },
    { 0xe9825bcb, SUC_lexicon_tags_1 },
    { 0x99c25bcc, SUC_lexicon_tags_25 },
    { 0x19225bcb, SUC_lexicon_tags_1 },
    { 0x8e825bca, SUC_lexicon_tags_15 },
    { 0x7aa25bcf, SUC_lexicon_tags_27 },
    { 0x48225bd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79625bd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x87625bd6, SUC_lexicon_tags_57 },
    { 0x21a25bd6, SUC_lexicon_tags_60 },
    { 0x89825bd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0025bdb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe2425bdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb825be0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66025be6, SUC_lexicon_tags_38 },
    { 0xcc825be7, SUC_lexicon_tags_30 },
    { 0xede25be8, SUC_lexicon_tags_7 },
    { 0x84a25be8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1da25beb, SUC_lexicon_tags_27 },
    { 0x89025bec, SUC_lexicon_tags_12 },
    { 0x3ea25bec, SUC_lexicon_tags_19 },
    { 0xaa25bee, SUC_lexicon_tags_19 },
    { 0x35225bef, SUC_lexicon_tags_38 },
    { 0x10625bee, SUC_lexicon_tags_23 },
    { 0x75825bee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25825bfc, SUC_lexicon_tags_60 },
    { 0x2ba25bfd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7225bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11025c03, SUC_lexicon_tags_17 },
    { 0x68625c03, SUC_lexicon_tags_27 },
    { 0xbca25c03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e25c09, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa1625c0b, SUC_lexicon_tags_46 },
    { 0xa5425c0b, SUC_lexicon_tags_39 },
    { 0x72a25c0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9025c10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89625c13, SUC_lexicon_tags_0 },
    { 0x85825c13, SUC_lexicon_tags_17 },
    { 0xa025c15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19e25c17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85e25c19, SUC_lexicon_tags_6 },
    { 0x41225c1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33025c1d, SUC_lexicon_tags_37 },
    { 0x97c25c1e, SUC_lexicon_tags_3 },
    { 0x6de25c1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fa25c21, SUC_lexicon_tags_34 },
    { 0x8025c22, SUC_lexicon_tags_37 },
    { 0xb1425c23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed625c26, SUC_lexicon_tags_12 },
    { 0xac625c27, SUC_lexicon_tags_11 },
    { 0x9d025c27, SUC_lexicon_tags_2 },
    { 0x36425c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7625c2e, SUC_lexicon_tags_25 },
    { 0x61625c2f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5ac25c31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc825c33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4825c39, SUC_lexicon_tags_38 },
    { 0x59425c39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f225c3d, SUC_lexicon_tags_6 },
    { 0x34225c3e, SUC_lexicon_tags_37 },
    { 0x6f425c3f, SUC_lexicon_tags_14 },
    { 0x2c225c40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a25c43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77225c4f, SUC_lexicon_tags_55 },
    { 0xe25c4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f425c52, SUC_lexicon_tags_19 },
    { 0x50825c53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70025c55, SUC_lexicon_tags_14 },
    { 0x12425c56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94825c58, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf225c5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a25c5e, SUC_lexicon_tags_2 },
    { 0x90625c5f, SUC_lexicon_tags_12 },
    { 0x77a25c60, SUC_lexicon_tags_38 },
    { 0xfb025c61, SUC_lexicon_tags_25 },
    { 0xcb025c62, SUC_lexicon_tags_47 },
    { 0xba625c63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x49625c66, SUC_lexicon_tags_103 },
    { 0xb3a25c67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb8625c69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e225c6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5dc25c6e, SUC_lexicon_tags_27 },
    { 0xa9225c6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35625c73, SUC_lexicon_tags_49 },
    { 0xaa025c74, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x54e25c76, SUC_lexicon_tags_2 },
    { 0x20025c77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3825c7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacc25c7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda25c82, SUC_lexicon_tags_15 },
    { 0xed625c83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca25c89, SUC_lexicon_tags_33 },
    { 0xa3225c8a, SUC_lexicon_tags_14 },
    { 0xe7625c8a, SUC_lexicon_tags_17 },
    { 0xac425c8c, SUC_lexicon_tags_82 },
    { 0xaa425c8b, SUC_lexicon_tags_56 },
    { 0x10e25c8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a225c91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8425c94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98625c96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb825c9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0425ca0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ca25ca2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8425ca9, SUC_lexicon_tags_30 },
    { 0xfc225caa, SUC_lexicon_tags_65 },
    { 0x1da25cab, SUC_lexicon_tags_16 },
    { 0xb7425cac, SUC_lexicon_tags_43 },
    { 0xe6225caa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x53425cb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x57025cb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x69a25cb5, SUC_lexicon_tags_37 },
    { 0x29025cb6, SUC_lexicon_tags_43 },
    { 0xcf425cb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0025cbc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a25cc3, SUC_lexicon_tags_37 },
    { 0x44225cc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa25ccc, SUC_lexicon_tags_28 },
    { 0xe6025ccc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x67625cd0, SUC_lexicon_tags_12 },
    { 0x83025cd1, SUC_lexicon_tags_37 },
    { 0x53825cd2, SUC_lexicon_tags_45 },
    { 0x95425cd0, SUC_lexicon_tags_33 },
    { 0xa9025cd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e25cd7, SUC_lexicon_tags_4 },
    { 0xc4825cd8, SUC_lexicon_tags_2 },
    { 0x31225cd9, SUC_lexicon_tags_23 },
    { 0xdc25cda, SUC_lexicon_tags_82 },
    { 0xa3425cdb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce25ce2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb625ce4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a425ce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36a25ce8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74625cea, SUC_lexicon_tags_15 },
    { 0xb7425cea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74425cf0, SUC_lexicon_tags_25 },
    { 0x7c25cf0, SUC_lexicon_tags_12 },
    { 0x50c25cf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7025cf9, SUC_lexicon_tags_17 },
    { 0xfc625cfa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83825cff, SUC_lexicon_tags_29 },
    { 0xed425d00, SUC_lexicon_tags_37 },
    { 0x6ae25d01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68025d06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8ae25d08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a25d0c, SUC_lexicon_tags_65 },
    { 0x8f425d0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda25d10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54025d12, SUC_lexicon_tags_551 },
    { 0x58025d12, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb2625d15, SUC_lexicon_tags_43 },
    { 0xe9a25d15, SUC_lexicon_tags_17 },
    { 0xa1425d17, SUC_lexicon_tags_11 },
    { 0x22625d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b425d1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e25d23, SUC_lexicon_tags_8 },
    { 0x67c25d23, SUC_lexicon_tags_15 },
    { 0x6d425d24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c25d2c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x62825d2e, SUC_lexicon_tags_15 },
    { 0x13a25d2e, SUC_lexicon_tags_7 },
    { 0xa25d30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc25d33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64825d35, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f225d3a, SUC_lexicon_tags_31 },
    { 0x46e25d3a, SUC_lexicon_tags_23 },
    { 0xb2225d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e25d3f, SUC_lexicon_tags_47 },
    { 0xc3625d3f, SUC_lexicon_tags_7 },
    { 0xd2625d40, SUC_lexicon_tags_33 },
    { 0x4a625d42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x12425d45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c25d48, SUC_lexicon_tags_61 },
    { 0xc1025d49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92025d4b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92025d54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44825d56, SUC_lexicon_tags_25 },
    { 0x2e425d57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57625d5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb825d62, SUC_lexicon_tags_27 },
    { 0x16625d63, SUC_lexicon_tags_8 },
    { 0xf2a25d64, SUC_lexicon_tags_11 },
    { 0x80825d65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2225d68, SUC_lexicon_tags_38 },
    { 0x7bc25d69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe25d6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63a25d6e, SUC_lexicon_tags_2 },
    { 0xf7e25d6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x49225d72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c25d77, SUC_lexicon_tags_25 },
    { 0x5a425d78, SUC_lexicon_tags_12 },
    { 0xd6c25d77, SUC_lexicon_tags_27 },
    { 0xafe25d78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd225d7e, SUC_lexicon_tags_7 },
    { 0xf6c25d7f, SUC_lexicon_tags_11 },
    { 0x4c25d80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x17225d82, SUC_lexicon_tags_83 },
    { 0xc2c25d83, SUC_lexicon_tags_9 },
    { 0x39c25d84, SUC_lexicon_tags_27 },
    { 0x38825d85, SUC_lexicon_tags_65 },
    { 0x63625d84, SUC_lexicon_tags_31 },
    { 0xea425d87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a25d8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89825d8d, SUC_lexicon_tags_17 },
    { 0x92425d8d, SUC_lexicon_tags_27 },
    { 0x82025d8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54425d95, SUC_lexicon_tags_65 },
    { 0x73a25d96, SUC_lexicon_tags_8 },
    { 0xa3225d97, SUC_lexicon_tags_0 },
    { 0xd0025d96, SUC_lexicon_tags_6 },
    { 0x43a25d99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba025d9d, SUC_lexicon_tags_11 },
    { 0x8a625d9d, SUC_lexicon_tags_12 },
    { 0x10c25d9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e825da2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79a25da4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe225da7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1225dad, SUC_lexicon_tags_416 },
    { 0x9c425dae, SUC_lexicon_tags_29 },
    { 0x9a025daf, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0xf8c25db1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c25db4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6825dbb, SUC_lexicon_tags_15 },
    { 0xb8c25dbc, SUC_lexicon_tags_13 },
    { 0x1625dbd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0425dc7, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x90025dc9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c25dcc, SUC_lexicon_tags_14 },
    { 0xe6c25dcc, SUC_lexicon_tags_0 },
    { 0x6425dcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6425dd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff425dd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a25ddd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2f625ddf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f825de4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc25deb, SUC_lexicon_tags_7 },
    { 0xc1e25dec, SUC_lexicon_tags_39 },
    { 0xbc025dec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd425df0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc25df3, SUC_lexicon_tags_15 },
    { 0x21425df4, SUC_lexicon_tags_12 },
    { 0x1bc25df3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf0425df7, SUC_lexicon_tags_1 },
    { 0xed825df8, SUC_lexicon_tags_1 },
    { 0x3be25df9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e25dfc, SUC_lexicon_tags_62 },
    { 0x8be25dfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6225dff, SUC_lexicon_tags_151 },
    { 0x6a225e00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe625e0c, SUC_lexicon_tags_31 },
    { 0xab825e0d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5825e13, SUC_lexicon_tags_87 },
    { 0x9c025e14, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb3825e16, SUC_lexicon_tags_37 },
    { 0x61425e17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3e25e19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d625e1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf625e1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5825e2a, SUC_lexicon_tags_25 },
    { 0xaee25e2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2425e2d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c25e38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c825e3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95425e40, SUC_lexicon_tags_1 },
    { 0xa7025e41, SUC_lexicon_tags_11 },
    { 0xb7a25e42, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xace25e44, SUC_lexicon_tags_38 },
    { 0xcdc25e45, SUC_lexicon_tags_11 },
    { 0x24e25e46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde225e49, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c25e51, SUC_lexicon_tags_17 },
    { 0xeee25e52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a425e54, SUC_lexicon_tags_151 },
    { 0xcb025e55, SUC_lexicon_tags_37 },
    { 0x60e25e56, SUC_lexicon_tags_47 },
    { 0x94a25e54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e225e5a, SUC_lexicon_tags_23 },
    { 0x3aa25e5b, SUC_lexicon_tags_17 },
    { 0xdb025e5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a25e62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8225e6c, SUC_lexicon_tags_32 },
    { 0xe1825e6d, SUC_lexicon_tags_47 },
    { 0x1425e6e, SUC_lexicon_tags_2 },
    { 0xb7025e6f, SUC_lexicon_tags_37 },
    { 0x95825e70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34825e79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a425e7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e625e84, SUC_lexicon_tags_17 },
    { 0x6e25e85, SUC_lexicon_tags_10 },
    { 0xab825e86, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3625e88, SUC_lexicon_tags_35 },
    { 0x28e25e88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78225e90, SUC_lexicon_tags_17 },
    { 0xc3425e91, SUC_lexicon_tags_11 },
    { 0x96625e91, SUC_lexicon_tags_88 },
    { 0x3a25e92, SUC_lexicon_tags_34 },
    { 0xdcc25e92, SUC_lexicon_tags_39 },
    { 0x89225e95, SUC_lexicon_tags_40 },
    { 0x71825e96, SUC_lexicon_tags_37 },
    { 0xa5a25e97, SUC_lexicon_tags_1 },
    { 0xc4225e98, SUC_lexicon_tags_15 },
    { 0x50e25e95, SUC_lexicon_tags_18 },
    { 0x72025e95, SUC_lexicon_tags_12 },
    { 0xe7025e98, SUC_lexicon_tags_60 },
    { 0x45a25e9b, SUC_lexicon_tags_23 },
    { 0x8e625e94, SUC_lexicon_tags_12 },
    { 0xe4e25e9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e25ea4, SUC_lexicon_tags_11 },
    { 0x19e25ea5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7625ea9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e25ead, SUC_lexicon_tags_26 },
    { 0xdf425eae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27a25eb0, SUC_lexicon_tags_160 },
    { 0xbd025eb1, SUC_lexicon_tags_63 },
    { 0x86425eb2, SUC_lexicon_tags_7 },
    { 0x1ac25eb3, SUC_lexicon_tags_41 },
    { 0x12825eb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9025eb9, SUC_lexicon_tags_6 },
    { 0xa9c25eb9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x79225ebc, SUC_lexicon_tags_37 },
    { 0xa6c25ebd, SUC_lexicon_tags_31 },
    { 0xe5625ebd, SUC_lexicon_tags_15 },
    { 0x92025ebd, SUC_lexicon_tags_12 },
    { 0x63625ebf, SUC_lexicon_tags_15 },
    { 0x97a25ec1, SUC_lexicon_tags_11 },
    { 0x28c25ec2, SUC_lexicon_tags_66 },
    { 0x4f025ec2, SUC_lexicon_tags_35 },
    { 0xce825ec3, SUC_lexicon_tags_43 },
    { 0xc7a25ec5, SUC_lexicon_tags_104 },
    { 0x6e425ebf, SUC_lexicon_tags_12 },
    { 0x4da25ec4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeee25ec9, SUC_lexicon_tags_7 },
    { 0x60825eca, SUC_lexicon_tags_8 },
    { 0xf1225eca, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8d825ecd, SUC_lexicon_tags_45 },
    { 0xe1a25ecd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x28625ed1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc25ed6, SUC_lexicon_tags_31 },
    { 0x2ac25ed7, SUC_lexicon_tags_13 },
    { 0x41025ed7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b025eda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a25ee0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9425ee4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8bc25ee6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8025ee9, SUC_lexicon_tags_1 },
    { 0xe8625ee9, SUC_lexicon_tags_82 },
    { 0x20e25ee9, SUC_lexicon_tags_12 },
    { 0xcde25eeb, SUC_lexicon_tags_15 },
    { 0x45c25eea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd825ef1, SUC_lexicon_tags_92 },
    { 0x9ac25ef2, SUC_lexicon_tags_2 },
    { 0x89c25ef3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84625ef5, SUC_lexicon_tags_1 },
    { 0xec625ef6, SUC_lexicon_tags_9 },
    { 0x8ac25ef7, SUC_lexicon_tags_12 },
    { 0x68425ef8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xee625efb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff625f01, SUC_lexicon_tags_68 },
    { 0xdc625f02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a25f08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23425f0a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4825f0d, SUC_lexicon_tags_15 },
    { 0x5d425f0d, SUC_lexicon_tags_34 },
    { 0xc1a25f0d, SUC_lexicon_tags_14 },
    { 0x79225f10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x31825f12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a825f16, SUC_lexicon_tags_0 },
    { 0x3ae25f17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde025f1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e25f1d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0225f1f, SUC_lexicon_tags_10 },
    { 0x9fc25f20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1025f27, SUC_lexicon_tags_2 },
    { 0x56a25f27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x81225f2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c025f2f, SUC_lexicon_tags_2 },
    { 0x68425f30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81a25f32, SUC_lexicon_tags_2 },
    { 0xc7c25f33, SUC_lexicon_tags_37 },
    { 0x99025f34, SUC_lexicon_tags_17 },
    { 0x32625f35, SUC_lexicon_tags_11 },
    { 0xe5825f35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b225f39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d225f3c, SUC_lexicon_tags_7 },
    { 0x3d825f3d, SUC_lexicon_tags_11 },
    { 0xf6a25f3c, SUC_lexicon_tags_10 },
    { 0xd1225f3f, SUC_lexicon_tags_13 },
    { 0xd0225f40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56a25f42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6825f46, SUC_lexicon_tags_19 },
    { 0x46425f47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3025f49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a225f4b, SUC_lexicon_tags_3 },
    { 0x69c25f4c, SUC_lexicon_tags_2 },
    { 0xac225f4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43025f4f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xece25f51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1c25f53, SUC_lexicon_tags_6 },
    { 0xc1825f54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee625f57, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda25f5a, SUC_lexicon_tags_103 },
    { 0xc9025f5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc25f5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e25f62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e25f66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21825f69, SUC_lexicon_tags_60 },
    { 0xb6425f6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa4c25f6c, SUC_lexicon_tags_25 },
    { 0xab425f6c, SUC_lexicon_tags_2 },
    { 0x55c25f6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa25f74, SUC_lexicon_tags_49 },
    { 0x19a25f74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x67025f77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e25f7a, SUC_lexicon_tags_1 },
    { 0x1b025f7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5425f7e, SUC_lexicon_tags_15 },
    { 0x7b225f7e, SUC_lexicon_tags_82 },
    { 0x65c25f7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe225f84, SUC_lexicon_tags_2 },
    { 0xb5225f85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90425f8c, SUC_lexicon_tags_7 },
    { 0xba625f8c, SUC_lexicon_tags_1 },
    { 0xfde25f8d, SUC_lexicon_tags_14 },
    { 0x68c25f8d, SUC_lexicon_tags_45 },
    { 0xb2c25f90, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1a025f92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7a625f94, SUC_lexicon_tags_15 },
    { 0x90025f95, SUC_lexicon_tags_27 },
    { 0x56225f96, SUC_lexicon_tags_17 },
    { 0x85c25f95, SUC_lexicon_tags_33 },
    { 0xef625f97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25425f9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e25f9f, SUC_lexicon_tags_23 },
    { 0xef625fa0, SUC_lexicon_tags_15 },
    { 0x5625fa1, SUC_lexicon_tags_7 },
    { 0x14e25fa0, SUC_lexicon_tags_63 },
    { 0x16a25fa3, SUC_lexicon_tags_85 },
    { 0xd3625fa4, SUC_lexicon_tags_17 },
    { 0xe6225fa1, SUC_lexicon_tags_25 },
    { 0xf3c25fa6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a25fac, SUC_lexicon_tags_17 },
    { 0x78825fac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b425fbc, SUC_lexicon_tags_9 },
    { 0x53e25fbd, SUC_lexicon_tags_7 },
    { 0x81825fbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8a25fc0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a25fc5, SUC_lexicon_tags_2 },
    { 0x3d625fc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa25fca, SUC_lexicon_tags_1 },
    { 0x79a25fcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60425fcd, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x17025fcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde25fd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a25fd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d625fd9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc25fdd, SUC_lexicon_tags_5 },
    { 0xa5e25fdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96625fe2, SUC_lexicon_tags_17 },
    { 0x3f025fe3, SUC_lexicon_tags_9 },
    { 0xba25fe4, SUC_lexicon_tags_74 },
    { 0xd2a25fe5, SUC_lexicon_tags_2 },
    { 0x20025fe4, SUC_lexicon_tags_15 },
    { 0x3825fe7, SUC_lexicon_tags_37 },
    { 0x7025fe8, SUC_lexicon_tags_33 },
    { 0xd1825fe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c25fee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e25ff8, SUC_lexicon_tags_11 },
    { 0x42225ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5626003, SUC_lexicon_tags_17 },
    { 0x93826004, SUC_lexicon_tags_26 },
    { 0xd3a26005, SUC_lexicon_tags_1 },
    { 0x8ec26005, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7226009, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e26010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f626013, SUC_lexicon_tags_7 },
    { 0x86626014, SUC_lexicon_tags_21 },
    { 0x68026014, SUC_lexicon_tags_7 },
    { 0x86426016, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e2601a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba26020, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32026027, SUC_lexicon_tags_32 },
    { 0x7dc26028, SUC_lexicon_tags_38 },
    { 0xea826028, SUC_lexicon_tags_336 },
    { 0xfd42602a, SUC_lexicon_tags_33 },
    { 0x442602a, SUC_lexicon_tags_25 },
    { 0x9be2602c, SUC_lexicon_tags_26 },
    { 0x22a2602c, SUC_lexicon_tags_46 },
    { 0x91a2602e, SUC_lexicon_tags_38 },
    { 0x53c2602e, SUC_lexicon_tags_47 },
    { 0xf8426030, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c26033, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7626035, SUC_lexicon_tags_19 },
    { 0x4b626036, SUC_lexicon_tags_11 },
    { 0xb4826037, SUC_lexicon_tags_12 },
    { 0x97e26036, SUC_lexicon_tags_15 },
    { 0xe426039, SUC_lexicon_tags_37 },
    { 0x11226039, SUC_lexicon_tags_7 },
    { 0x2a82603b, SUC_lexicon_tags_25 },
    { 0x20a2603b, SUC_lexicon_tags_10 },
    { 0xe4a2603d, SUC_lexicon_tags_45 },
    { 0x89c2603a, SUC_lexicon_tags_29 },
    { 0xfb82603f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33426043, SUC_lexicon_tags_37 },
    { 0x91226044, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda26047, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6522604c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5022604f, SUC_lexicon_tags_27 },
    { 0x17c26050, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xad226052, SUC_lexicon_tags_61 },
    { 0x55226052, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1626055, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda26058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c22605b, SUC_lexicon_tags_31 },
    { 0x1322605c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e26060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8aa26062, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e426065, SUC_lexicon_tags_28 },
    { 0xc2426066, SUC_lexicon_tags_12 },
    { 0x49026066, SUC_lexicon_tags_63 },
    { 0x82e26066, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a2606b, SUC_lexicon_tags_43 },
    { 0x5d42606b, SUC_lexicon_tags_28 },
    { 0xc6c2606b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22626073, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39626075, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8626078, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a2607b, SUC_lexicon_tags_39 },
    { 0x2402607b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4626082, SUC_lexicon_tags_7 },
    { 0x5a426083, SUC_lexicon_tags_1167 },
    { 0x5fe26084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8826086, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a2608b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3242608d, SUC_lexicon_tags_68 },
    { 0xfc22608d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be26095, SUC_lexicon_tags_23 },
    { 0x67a26096, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3ca26098, SUC_lexicon_tags_39 },
    { 0xb226099, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef22609c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb42609e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca260a1, SUC_lexicon_tags_25 },
    { 0x39a260a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20260a5, SUC_lexicon_tags_37 },
    { 0x74260a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc90260a8, SUC_lexicon_tags_9 },
    { 0x784260a8, SUC_lexicon_tags_25 },
    { 0x426260a9, SUC_lexicon_tags_66 },
    { 0xc08260aa, SUC_lexicon_tags_45 },
    { 0x710260ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x668260ae, SUC_lexicon_tags_1 },
    { 0x44260ae, SUC_lexicon_tags_57 },
    { 0x740260ae, SUC_lexicon_tags_12 },
    { 0x896260b1, SUC_lexicon_tags_37 },
    { 0x3b2260b2, SUC_lexicon_tags_1 },
    { 0x70a260b3, SUC_lexicon_tags_1 },
    { 0xd06260b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2260bc, SUC_lexicon_tags_12 },
    { 0xd4260bd, SUC_lexicon_tags_27 },
    { 0x41c260bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18260c5, SUC_lexicon_tags_27 },
    { 0x144260c6, SUC_lexicon_tags_37 },
    { 0xff8260c7, SUC_lexicon_tags_11 },
    { 0xd32260c8, SUC_lexicon_tags_9 },
    { 0x26c260c7, SUC_lexicon_tags_65 },
    { 0x85a260c9, SUC_lexicon_tags_11 },
    { 0xf9e260cb, SUC_lexicon_tags_12 },
    { 0x3b0260cb, SUC_lexicon_tags_23 },
    { 0x6c2260c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee260d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4260d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaca260d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2d8260d8, SUC_lexicon_tags_25 },
    { 0x6e0260d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe260de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xba4260e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344260e5, SUC_lexicon_tags_2 },
    { 0x1c2260e6, SUC_lexicon_tags_4 },
    { 0xcd2260e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x470260ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76260ef, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0xf88260f2, SUC_lexicon_tags_43 },
    { 0x1a2260f3, SUC_lexicon_tags_21 },
    { 0x86e260f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec26100, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe26103, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64626107, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9142610d, SUC_lexicon_tags_17 },
    { 0x5622610e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x52a26110, SUC_lexicon_tags_37 },
    { 0xb4e26110, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x98026114, SUC_lexicon_tags_17 },
    { 0x8b426115, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xba826118, SUC_lexicon_tags_0 },
    { 0xd2026119, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc62611d, SUC_lexicon_tags_27 },
    { 0x3642611e, SUC_lexicon_tags_12 },
    { 0x7d02611f, SUC_lexicon_tags_25 },
    { 0x67a26120, SUC_lexicon_tags_38 },
    { 0x37e26120, SUC_lexicon_tags_14 },
    { 0x3d826122, SUC_lexicon_tags_30 },
    { 0xd7a26123, SUC_lexicon_tags_1 },
    { 0x7902611e, SUC_lexicon_tags_7 },
    { 0x7aa26123, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39026129, SUC_lexicon_tags_15 },
    { 0x6422612a, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2612f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x94a26131, SUC_lexicon_tags_63 },
    { 0x39826131, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c426136, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b026138, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5802613a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6862613c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x54a2613e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda26141, SUC_lexicon_tags_11 },
    { 0xe4626142, SUC_lexicon_tags_27 },
    { 0x3f626142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc626146, SUC_lexicon_tags_66 },
    { 0x3b026146, SUC_lexicon_tags_18 },
    { 0xdf026146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93e2614a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6026150, SUC_lexicon_tags_37 },
    { 0x4de26151, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe26154, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9226159, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4e2615b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3426163, SUC_lexicon_tags_12 },
    { 0xbd426164, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c426167, SUC_lexicon_tags_19 },
    { 0x72e26167, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdaa2616a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e2616d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d42616f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a26175, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a62617a, SUC_lexicon_tags_56 },
    { 0xb7e2617b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa2617f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f026187, SUC_lexicon_tags_38 },
    { 0x9d826188, SUC_lexicon_tags_65 },
    { 0xd4226189, SUC_lexicon_tags_15 },
    { 0x42e2618a, SUC_lexicon_tags_0 },
    { 0xdbc2618b, SUC_lexicon_tags_25 },
    { 0xecc2618c, SUC_lexicon_tags_17 },
    { 0xc082618c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe26191, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4626193, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da26196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5dc26198, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6261a0, SUC_lexicon_tags_55 },
    { 0x64261a1, SUC_lexicon_tags_0 },
    { 0x82e261a1, SUC_lexicon_tags_2 },
    { 0x9d4261a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da261a6, SUC_lexicon_tags_17 },
    { 0x6fa261a7, SUC_lexicon_tags_7 },
    { 0xa5c261a8, SUC_lexicon_tags_63 },
    { 0xc5a261a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea261b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814261b6, SUC_lexicon_tags_63 },
    { 0x12a261b6, SUC_lexicon_tags_12 },
    { 0xde8261b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a261bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32261c6, SUC_lexicon_tags_29 },
    { 0xaa4261c6, SUC_lexicon_tags_29 },
    { 0xf38261c8, SUC_lexicon_tags_2 },
    { 0xf96261c9, SUC_lexicon_tags_11 },
    { 0x3b6261ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9a261cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76261d0, SUC_lexicon_tags_34 },
    { 0xaf6261d1, SUC_lexicon_tags_62 },
    { 0xe4c261d2, SUC_lexicon_tags_6 },
    { 0xcd6261d0, SUC_lexicon_tags_47 },
    { 0x1a6261d3, SUC_lexicon_tags_41 },
    { 0x7f2261d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa4c261d7, SUC_lexicon_tags_108 },
    { 0x332261d8, SUC_lexicon_tags_25 },
    { 0xda2261d9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc261de, SUC_lexicon_tags_45 },
    { 0x524261df, SUC_lexicon_tags_12 },
    { 0x5dc261e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e261e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a261eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2261ee, SUC_lexicon_tags_27 },
    { 0x150261ee, SUC_lexicon_tags_82 },
    { 0x794261f0, SUC_lexicon_tags_13 },
    { 0x922261f0, SUC_lexicon_tags_8 },
    { 0xbfa261f1, SUC_lexicon_tags_1 },
    { 0x646261f3, SUC_lexicon_tags_87 },
    { 0x3b0261f4, SUC_lexicon_tags_19 },
    { 0x966261f5, SUC_lexicon_tags_37 },
    { 0x95a261f2, SUC_lexicon_tags_63 },
    { 0xf9e261f4, SUC_lexicon_tags_7 },
    { 0x19e261f1, SUC_lexicon_tags_1 },
    { 0x160261f9, SUC_lexicon_tags_0 },
    { 0xd3c261fa, SUC_lexicon_tags_39 },
    { 0x7c0261ef, SUC_lexicon_tags_0 },
    { 0xd28261f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x462261ff, SUC_lexicon_tags_5 },
    { 0x19226200, SUC_lexicon_tags_45 },
    { 0x79426200, SUC_lexicon_tags_1 },
    { 0x11626202, SUC_lexicon_tags_63 },
    { 0xf1c26203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9826209, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22620c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe1a2620e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x46626211, SUC_lexicon_tags_17 },
    { 0x28026212, SUC_lexicon_tags_11 },
    { 0x40a26213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4226217, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c2621b, SUC_lexicon_tags_38 },
    { 0xe062621c, SUC_lexicon_tags_17 },
    { 0x4202621c, SUC_lexicon_tags_7 },
    { 0xdfc2621e, SUC_lexicon_tags_15 },
    { 0x4e62621e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x69026221, SUC_lexicon_tags_60 },
    { 0x64626221, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97e26224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6626227, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c2622b, SUC_lexicon_tags_37 },
    { 0x20e2622c, SUC_lexicon_tags_8 },
    { 0x6302622c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x23226230, SUC_lexicon_tags_27 },
    { 0x73226231, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d426237, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35826239, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3122623c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c26241, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61826243, SUC_lexicon_tags_3 },
    { 0x90226244, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c026246, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8426249, SUC_lexicon_tags_11 },
    { 0x6662624a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19e2624c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c2624f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa8026251, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26426256, SUC_lexicon_tags_11 },
    { 0x62a26256, SUC_lexicon_tags_63 },
    { 0x36826258, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd122625c, SUC_lexicon_tags_39 },
    { 0xaf82625d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29826260, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a26269, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea2626c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c2626f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5e626271, SUC_lexicon_tags_17 },
    { 0x8bc26271, SUC_lexicon_tags_8 },
    { 0xb3e26271, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e26278, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e2627c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a26280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed426287, SUC_lexicon_tags_25 },
    { 0x10a26288, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ec2628a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca2628e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e226290, SUC_lexicon_tags_11 },
    { 0xfb426291, SUC_lexicon_tags_65 },
    { 0x60626292, SUC_lexicon_tags_3 },
    { 0x6e626293, SUC_lexicon_tags_2 },
    { 0x5f226291, SUC_lexicon_tags_16 },
    { 0x91226295, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x61226298, SUC_lexicon_tags_61 },
    { 0xa6626299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82629e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6262a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378262a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5ee262ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba8262ad, SUC_lexicon_tags_23 },
    { 0xf06262ad, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a262b5, SUC_lexicon_tags_0 },
    { 0x9c4262b6, SUC_lexicon_tags_7 },
    { 0xca4262b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0262be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810262c4, SUC_lexicon_tags_17 },
    { 0xcb0262c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28262c8, SUC_lexicon_tags_2 },
    { 0x2f2262c9, SUC_lexicon_tags_32 },
    { 0xe1c262ca, SUC_lexicon_tags_15 },
    { 0x51a262cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378262cf, SUC_lexicon_tags_15 },
    { 0x7a2262d0, SUC_lexicon_tags_37 },
    { 0x140262cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4262d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300262de, SUC_lexicon_tags_9 },
    { 0x466262de, SUC_lexicon_tags_17 },
    { 0x1aa262e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84262e3, SUC_lexicon_tags_8 },
    { 0xcd2262e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe16262e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e262ef, SUC_lexicon_tags_12 },
    { 0xb02262ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9bc262f2, SUC_lexicon_tags_30 },
    { 0xc76262f2, SUC_lexicon_tags_25 },
    { 0x2b6262f4, SUC_lexicon_tags_32 },
    { 0x56e262f4, SUC_lexicon_tags_15 },
    { 0xfb8262f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcf6262f8, SUC_lexicon_tags_12 },
    { 0x3a0262f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x666262fc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xecc262fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54426303, SUC_lexicon_tags_38 },
    { 0xd626304, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x73826307, SUC_lexicon_tags_23 },
    { 0x5f426308, SUC_lexicon_tags_1 },
    { 0xb3226309, SUC_lexicon_tags_826 },
    { 0x6402630a, SUC_lexicon_tags_15 },
    { 0xf5026309, SUC_lexicon_tags_11 },
    { 0x9a62630a, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0xa802630f, SUC_lexicon_tags_1 },
    { 0x9626310, SUC_lexicon_tags_33 },
    { 0x35826311, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82226319, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa26320, SUC_lexicon_tags_116 },
    { 0xbac26321, SUC_lexicon_tags_7 },
    { 0x83426322, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae26327, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x55826329, SUC_lexicon_tags_215 },
    { 0xae42632a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c2632d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a26331, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc0a26333, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8162633c, SUC_lexicon_tags_1 },
    { 0x5e42633c, SUC_lexicon_tags_55 },
    { 0x97e2633e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef426342, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea426347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa26350, SUC_lexicon_tags_17 },
    { 0xad426351, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70826353, SUC_lexicon_tags_3 },
    { 0xf0026353, SUC_lexicon_tags_37 },
    { 0x5c626355, SUC_lexicon_tags_30 },
    { 0x67226356, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef226361, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce826363, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2026367, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9fe26369, SUC_lexicon_tags_8 },
    { 0xccc2636a, SUC_lexicon_tags_6 },
    { 0x8a42636a, SUC_lexicon_tags_39 },
    { 0x65c2636b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd382636e, SUC_lexicon_tags_12 },
    { 0x6de2636f, SUC_lexicon_tags_12 },
    { 0x5bc26370, SUC_lexicon_tags_12 },
    { 0x17826370, SUC_lexicon_tags_1 },
    { 0x8ec26370, SUC_lexicon_tags_17 },
    { 0xd502636f, SUC_lexicon_tags_38 },
    { 0x6d226370, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a62637a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xf8a2637c, SUC_lexicon_tags_15 },
    { 0x1142637d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a26385, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc26389, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4782638b, SUC_lexicon_tags_38 },
    { 0x22e2638c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4922638e, SUC_lexicon_tags_25 },
    { 0x5642638f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70826392, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37a26394, SUC_lexicon_tags_37 },
    { 0x79e26395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdec26397, SUC_lexicon_tags_40 },
    { 0xf2226398, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6462639f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0263a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90263a8, SUC_lexicon_tags_12 },
    { 0xa4c263a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c263ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744263b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94a263b3, SUC_lexicon_tags_1 },
    { 0x440263b4, SUC_lexicon_tags_15 },
    { 0x8d4263b4, SUC_lexicon_tags_37 },
    { 0x80a263b6, SUC_lexicon_tags_138 },
    { 0x6ae263b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa263b9, SUC_lexicon_tags_6 },
    { 0x794263b9, SUC_lexicon_tags_15 },
    { 0xd08263bb, SUC_lexicon_tags_28 },
    { 0x8ce263ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a263c5, SUC_lexicon_tags_7 },
    { 0x2c4263c6, SUC_lexicon_tags_25 },
    { 0x52a263c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6263cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca263cd, SUC_lexicon_tags_15 },
    { 0x662263ce, SUC_lexicon_tags_1 },
    { 0xaa2263cf, SUC_lexicon_tags_41 },
    { 0xa6e263d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xabe263d2, SUC_lexicon_tags_31 },
    { 0x738263d2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x566263d5, SUC_lexicon_tags_17 },
    { 0xf10263d6, SUC_lexicon_tags_37 },
    { 0x364263d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc263da, SUC_lexicon_tags_38 },
    { 0x320263db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x602263dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c263e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0263e3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a263ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x514263f0, SUC_lexicon_tags_2 },
    { 0xc56263f0, SUC_lexicon_tags_26 },
    { 0xafa263f2, SUC_lexicon_tags_7 },
    { 0x410263f3, SUC_lexicon_tags_31 },
    { 0xfac263f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2263f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9b2263f9, SUC_lexicon_tags_9 },
    { 0x430263fa, SUC_lexicon_tags_35 },
    { 0x610263fb, SUC_lexicon_tags_77 },
    { 0x702263f9, SUC_lexicon_tags_17 },
    { 0xda4263fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c26401, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0826403, SUC_lexicon_tags_2 },
    { 0x7c426404, SUC_lexicon_tags_28 },
    { 0x36e26405, SUC_lexicon_tags_61 },
    { 0xc8626406, SUC_lexicon_tags_37 },
    { 0x5026405, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde826410, SUC_lexicon_tags_11 },
    { 0xa1826411, SUC_lexicon_tags_9 },
    { 0xb9c26411, SUC_lexicon_tags_3 },
    { 0xd4026410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1226418, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e62641c, SUC_lexicon_tags_55 },
    { 0xcd42641c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5942641f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x19e26421, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd226424, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b026427, SUC_lexicon_tags_11 },
    { 0xcc226428, SUC_lexicon_tags_37 },
    { 0xee226427, SUC_lexicon_tags_7 },
    { 0xcb82642a, SUC_lexicon_tags_25 },
    { 0xc4c26429, SUC_lexicon_tags_10 },
    { 0x4762642c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc26433, SUC_lexicon_tags_2 },
    { 0x78426434, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47626438, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c2643e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71e26440, SUC_lexicon_tags_35 },
    { 0x8e826441, SUC_lexicon_tags_32 },
    { 0xe9c26442, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b026449, SUC_lexicon_tags_0 },
    { 0x6e2644a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb022644c, SUC_lexicon_tags_43 },
    { 0x5402644d, SUC_lexicon_tags_17 },
    { 0xab02644e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35826451, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be26454, SUC_lexicon_tags_17 },
    { 0xe8626455, SUC_lexicon_tags_39 },
    { 0x81626455, SUC_lexicon_tags_27 },
    { 0xdec26457, SUC_lexicon_tags_11 },
    { 0x83e26458, SUC_lexicon_tags_2 },
    { 0x7a026457, SUC_lexicon_tags_21 },
    { 0xac82645a, SUC_lexicon_tags_15 },
    { 0xdc826458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6226463, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32826465, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33826469, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2646c, SUC_lexicon_tags_25 },
    { 0x9fe2646d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3162646f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd02647a, SUC_lexicon_tags_25 },
    { 0x6d02647b, SUC_lexicon_tags_12 },
    { 0x4b02647b, SUC_lexicon_tags_17 },
    { 0x4982647d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a026481, SUC_lexicon_tags_23 },
    { 0xb9226482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6226488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf22648a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8826492, SUC_lexicon_tags_25 },
    { 0xb6826492, SUC_lexicon_tags_4 },
    { 0x3f626494, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e26498, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x66a2649a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc2649f, SUC_lexicon_tags_1 },
    { 0xd36264a0, SUC_lexicon_tags_37 },
    { 0x9a0264a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4264a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c264a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x148264ab, SUC_lexicon_tags_9 },
    { 0x848264ac, SUC_lexicon_tags_11 },
    { 0xa18264ab, SUC_lexicon_tags_15 },
    { 0x278264ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8264b2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xa84264b4, SUC_lexicon_tags_31 },
    { 0x71c264b5, SUC_lexicon_tags_12 },
    { 0x3ae264b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6264bc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x708264be, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf84264c0, SUC_lexicon_tags_29 },
    { 0x932264c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4264c6, SUC_lexicon_tags_25 },
    { 0xdac264c7, SUC_lexicon_tags_17 },
    { 0x6e6264c8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x368264cb, SUC_lexicon_tags_7 },
    { 0x16c264cc, SUC_lexicon_tags_49 },
    { 0x7ac264cd, SUC_lexicon_tags_25 },
    { 0x990264ce, SUC_lexicon_tags_17 },
    { 0xd1a264cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaa6264d1, SUC_lexicon_tags_14 },
    { 0xdf8264d1, SUC_lexicon_tags_7 },
    { 0x468264d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae264d7, SUC_lexicon_tags_87 },
    { 0x460264d8, SUC_lexicon_tags_60 },
    { 0xe2264d7, SUC_lexicon_tags_63 },
    { 0xd96264da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x68264dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6264e1, SUC_lexicon_tags_15 },
    { 0x114264e2, SUC_lexicon_tags_25 },
    { 0xfa2264e2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02264e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170264eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe00264ed, SUC_lexicon_tags_45 },
    { 0x3fa264ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec264f5, SUC_lexicon_tags_37 },
    { 0x786264f6, SUC_lexicon_tags_4 },
    { 0xf70264f5, SUC_lexicon_tags_37 },
    { 0xf2264f8, SUC_lexicon_tags_1 },
    { 0x96264f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc264fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda264ff, SUC_lexicon_tags_48 },
    { 0x33c264ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3e26502, SUC_lexicon_tags_25 },
    { 0x1bc26503, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33426508, SUC_lexicon_tags_25 },
    { 0x7f626509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8422650c, SUC_lexicon_tags_7 },
    { 0x6fc2650c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1826512, SUC_lexicon_tags_17 },
    { 0xbfc26513, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a226515, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b22651c, SUC_lexicon_tags_43 },
    { 0xdb82651d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24a2651f, SUC_lexicon_tags_68 },
    { 0x8bc2651f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1aa26522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1626525, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9626528, SUC_lexicon_tags_25 },
    { 0xeba26528, SUC_lexicon_tags_12 },
    { 0x27426529, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc562652c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc402652e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a26531, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9826537, SUC_lexicon_tags_118 },
    { 0xad426538, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac2653b, SUC_lexicon_tags_7 },
    { 0xfbe2653c, SUC_lexicon_tags_13 },
    { 0x3c2653d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe26542, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4226546, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd962654a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c26552, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x14026554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35826556, SUC_lexicon_tags_37 },
    { 0xc2226557, SUC_lexicon_tags_11 },
    { 0x8be26556, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe2655b, SUC_lexicon_tags_14 },
    { 0x8f42655c, SUC_lexicon_tags_13 },
    { 0x5f22655d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6e82655f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcc426561, SUC_lexicon_tags_12 },
    { 0x52626562, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2656e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7026578, SUC_lexicon_tags_15 },
    { 0x9e26578, SUC_lexicon_tags_31 },
    { 0xe8426578, SUC_lexicon_tags_37 },
    { 0x7422657b, SUC_lexicon_tags_39 },
    { 0xff826579, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcce2657e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x92c26580, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e26587, SUC_lexicon_tags_63 },
    { 0xef426588, SUC_lexicon_tags_12 },
    { 0x42c26588, SUC_lexicon_tags_33 },
    { 0x1e42658a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4026590, SUC_lexicon_tags_2 },
    { 0x3fc26590, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc026594, SUC_lexicon_tags_17 },
    { 0xe8626594, SUC_lexicon_tags_0 },
    { 0x6b826595, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x87226598, SUC_lexicon_tags_7 },
    { 0x3fe26599, SUC_lexicon_tags_33 },
    { 0xcec2659a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9e2659c, SUC_lexicon_tags_4 },
    { 0x8b42659c, SUC_lexicon_tags_11 },
    { 0x8922659c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4265a2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e265a5, SUC_lexicon_tags_25 },
    { 0x430265a5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac265ac, SUC_lexicon_tags_16 },
    { 0xff4265ac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0265ba, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x350265bc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd2265be, SUC_lexicon_tags_37 },
    { 0x49c265bf, SUC_lexicon_tags_33 },
    { 0xad6265c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x67c265c2, SUC_lexicon_tags_12 },
    { 0x108265c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x454265c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c265c8, SUC_lexicon_tags_7 },
    { 0x834265c9, SUC_lexicon_tags_32 },
    { 0x608265ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xff2265cc, SUC_lexicon_tags_37 },
    { 0xbba265cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6265d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08265d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cc265d8, SUC_lexicon_tags_15 },
    { 0x9a265d9, SUC_lexicon_tags_14 },
    { 0x280265d9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x964265dc, SUC_lexicon_tags_34 },
    { 0x4c2265dd, SUC_lexicon_tags_3 },
    { 0x6f0265dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa56265e0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92265e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb16265e5, SUC_lexicon_tags_17 },
    { 0xfcc265e5, SUC_lexicon_tags_1 },
    { 0x978265e7, SUC_lexicon_tags_47 },
    { 0xb9c265e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486265ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x420265f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e265f5, SUC_lexicon_tags_37 },
    { 0x6ac265f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2265f9, SUC_lexicon_tags_15 },
    { 0x360265f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x406265fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44026601, SUC_lexicon_tags_30 },
    { 0x34026602, SUC_lexicon_tags_63 },
    { 0xe9226603, SUC_lexicon_tags_37 },
    { 0xf4026604, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e226609, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2d62660b, SUC_lexicon_tags_11 },
    { 0x3e82660b, SUC_lexicon_tags_55 },
    { 0x2422660b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be26615, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a22661a, SUC_lexicon_tags_0 },
    { 0x9722661b, SUC_lexicon_tags_10 },
    { 0x5fe2661c, SUC_lexicon_tags_21 },
    { 0xb9e2661d, SUC_lexicon_tags_6 },
    { 0xd0e2661c, SUC_lexicon_tags_63 },
    { 0xf02661f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a26625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0226628, SUC_lexicon_tags_27 },
    { 0xcf826628, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e2662e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a26632, SUC_lexicon_tags_17 },
    { 0x13426633, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae26636, SUC_lexicon_tags_17 },
    { 0x88226637, SUC_lexicon_tags_21 },
    { 0xf4e26638, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe2663b, SUC_lexicon_tags_43 },
    { 0xaa62663c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a2663f, SUC_lexicon_tags_60 },
    { 0x9f22663f, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x28426642, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec26645, SUC_lexicon_tags_37 },
    { 0xfca26645, SUC_lexicon_tags_15 },
    { 0x49e26646, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x77626649, SUC_lexicon_tags_27 },
    { 0x5282664a, SUC_lexicon_tags_43 },
    { 0xafa26649, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1662664d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49026657, SUC_lexicon_tags_31 },
    { 0x1e26658, SUC_lexicon_tags_11 },
    { 0xe5c26658, SUC_lexicon_tags_6 },
    { 0x7c2665a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33c2665c, SUC_lexicon_tags_17 },
    { 0x70a2665d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2182665f, SUC_lexicon_tags_12 },
    { 0x75a26660, SUC_lexicon_tags_31 },
    { 0x89826661, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc26664, SUC_lexicon_tags_8 },
    { 0xfa426664, SUC_lexicon_tags_25 },
    { 0x14a26666, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe626669, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3422666c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x702666f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d026672, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c26678, SUC_lexicon_tags_17 },
    { 0x82426679, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x602667c, SUC_lexicon_tags_43 },
    { 0x4aa2667c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda626683, SUC_lexicon_tags_17 },
    { 0x1fe26684, SUC_lexicon_tags_30 },
    { 0x6f426685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ba26687, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc362668a, SUC_lexicon_tags_17 },
    { 0x10e2668b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa382668d, SUC_lexicon_tags_38 },
    { 0x4e2668e, SUC_lexicon_tags_60 },
    { 0xd7c2668f, SUC_lexicon_tags_11 },
    { 0x9de2668f, SUC_lexicon_tags_30 },
    { 0x15426691, SUC_lexicon_tags_25 },
    { 0xfde26691, SUC_lexicon_tags_17 },
    { 0x2f826691, SUC_lexicon_tags_3 },
    { 0xa7e26694, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda2669f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0266a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2266a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9cc266a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56266aa, SUC_lexicon_tags_27 },
    { 0xd34266aa, SUC_lexicon_tags_2 },
    { 0xace266ac, SUC_lexicon_tags_25 },
    { 0x464266ad, SUC_lexicon_tags_57 },
    { 0x776266ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18266b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8266b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e266ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2fc266bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20266c1, SUC_lexicon_tags_48 },
    { 0x1a266c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c266c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77e266ca, SUC_lexicon_tags_21 },
    { 0x952266cb, SUC_lexicon_tags_66 },
    { 0xa5a266cb, SUC_lexicon_tags_11 },
    { 0x6a0266cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e266da, SUC_lexicon_tags_37 },
    { 0xcc2266db, SUC_lexicon_tags_62 },
    { 0x214266dc, SUC_lexicon_tags_33 },
    { 0xca4266dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58266e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x266266e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a266ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c266f1, SUC_lexicon_tags_25 },
    { 0xe92266f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e6266f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0266fb, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654266ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa26704, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5e26706, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd1626708, SUC_lexicon_tags_6 },
    { 0x47026709, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6242670b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b026710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c26715, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb626721, SUC_lexicon_tags_25 },
    { 0x2a026721, SUC_lexicon_tags_45 },
    { 0x78226721, SUC_lexicon_tags_47 },
    { 0xbc426721, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf626728, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f22672d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa26733, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef026738, SUC_lexicon_tags_7 },
    { 0xaae26739, SUC_lexicon_tags_11 },
    { 0x97426739, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f22673d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x53a2673f, SUC_lexicon_tags_33 },
    { 0xc8826740, SUC_lexicon_tags_37 },
    { 0x3c826741, SUC_lexicon_tags_11 },
    { 0xb6026740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a26748, SUC_lexicon_tags_11 },
    { 0x98026749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1422674b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9522674f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb426754, SUC_lexicon_tags_0 },
    { 0xcc26754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2226759, SUC_lexicon_tags_9 },
    { 0xa7a26759, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3182675d, SUC_lexicon_tags_65 },
    { 0xb5e2675e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1026763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5542676b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c2676e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc26776, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a2677b, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5826780, SUC_lexicon_tags_17 },
    { 0xabe26781, SUC_lexicon_tags_17 },
    { 0xa2826782, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6462678e, SUC_lexicon_tags_47 },
    { 0xbac2678e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce26796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda2679c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95a2679e, SUC_lexicon_tags_28 },
    { 0xaf22679f, SUC_lexicon_tags_28 },
    { 0x1682679f, SUC_lexicon_tags_37 },
    { 0x4cc267a1, SUC_lexicon_tags_38 },
    { 0x1bc267a1, SUC_lexicon_tags_87 },
    { 0x532267a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa267a6, SUC_lexicon_tags_39 },
    { 0x9c4267a6, SUC_lexicon_tags_49 },
    { 0xfc2267a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce267ad, SUC_lexicon_tags_23 },
    { 0x44267ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0267b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee267b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390267bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf4267bf, SUC_lexicon_tags_25 },
    { 0xfe4267bf, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458267c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x174267c6, SUC_lexicon_tags_37 },
    { 0xb1e267c7, SUC_lexicon_tags_62 },
    { 0x81a267c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4267ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c267d1, SUC_lexicon_tags_8 },
    { 0x1ca267d2, SUC_lexicon_tags_1 },
    { 0x328267d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76267d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8267da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe10267dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb2267de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42267e3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x2c4267e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90267e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe267eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x44a267ed, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce267f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c267fc, SUC_lexicon_tags_2 },
    { 0x872267fd, SUC_lexicon_tags_2 },
    { 0x18e267fe, SUC_lexicon_tags_63 },
    { 0x660267ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4026801, SUC_lexicon_tags_11 },
    { 0x42e26802, SUC_lexicon_tags_10 },
    { 0x1a26801, SUC_lexicon_tags_25 },
    { 0x56626804, SUC_lexicon_tags_4 },
    { 0x76a26802, SUC_lexicon_tags_63 },
    { 0x3fe26802, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf002680a, SUC_lexicon_tags_17 },
    { 0xa4a2680b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2702680e, SUC_lexicon_tags_7 },
    { 0x9222680f, SUC_lexicon_tags_7 },
    { 0x19e2680e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a26818, SUC_lexicon_tags_31 },
    { 0x28026819, SUC_lexicon_tags_9 },
    { 0xd9026818, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe482681d, SUC_lexicon_tags_33 },
    { 0x142681e, SUC_lexicon_tags_1 },
    { 0x30a2681e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95426825, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c82682b, SUC_lexicon_tags_12 },
    { 0xfe62682c, SUC_lexicon_tags_11 },
    { 0xbbe2682d, SUC_lexicon_tags_37 },
    { 0x2fc2682d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22826833, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3642683a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e42683e, SUC_lexicon_tags_79 },
    { 0xb002683f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1626843, SUC_lexicon_tags_11 },
    { 0x66826844, SUC_lexicon_tags_3 },
    { 0xb3c26845, SUC_lexicon_tags_6 },
    { 0x8c26845, SUC_lexicon_tags_14 },
    { 0xe9e26846, SUC_lexicon_tags_11 },
    { 0x3ca26848, SUC_lexicon_tags_31 },
    { 0x9ba26849, SUC_lexicon_tags_46 },
    { 0x4402684a, SUC_lexicon_tags_11 },
    { 0xf2626845, SUC_lexicon_tags_37 },
    { 0x96e26847, SUC_lexicon_tags_17 },
    { 0x80a2684d, SUC_lexicon_tags_1 },
    { 0x9fc2684b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ae26850, SUC_lexicon_tags_14 },
    { 0xe7626851, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa26855, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85226857, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a2685c, SUC_lexicon_tags_7 },
    { 0x2382685d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xadc2685f, SUC_lexicon_tags_29 },
    { 0x99826860, SUC_lexicon_tags_12 },
    { 0x99626860, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c26864, SUC_lexicon_tags_25 },
    { 0x9d626865, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe7626867, SUC_lexicon_tags_12 },
    { 0x97c26868, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7262686c, SUC_lexicon_tags_37 },
    { 0x87a2686c, SUC_lexicon_tags_15 },
    { 0x7bc2686e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9c26870, SUC_lexicon_tags_11 },
    { 0xad226870, SUC_lexicon_tags_14 },
    { 0x96e26872, SUC_lexicon_tags_17 },
    { 0xf6026873, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2c26875, SUC_lexicon_tags_0 },
    { 0x86626876, SUC_lexicon_tags_17 },
    { 0xf7a26877, SUC_lexicon_tags_45 },
    { 0xd0826877, SUC_lexicon_tags_66 },
    { 0x4026879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a2687c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4226884, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe26887, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a2688a, SUC_lexicon_tags_43 },
    { 0xafc2688b, SUC_lexicon_tags_33 },
    { 0x27e2688a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f026891, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a26894, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e26897, SUC_lexicon_tags_28 },
    { 0xb0a26898, SUC_lexicon_tags_37 },
    { 0xf6c26899, SUC_lexicon_tags_1 },
    { 0x2b62689a, SUC_lexicon_tags_15 },
    { 0xad82689b, SUC_lexicon_tags_11 },
    { 0xaac26898, SUC_lexicon_tags_37 },
    { 0x8e22689d, SUC_lexicon_tags_15 },
    { 0xc2a2689e, SUC_lexicon_tags_12 },
    { 0xc8c2689e, SUC_lexicon_tags_63 },
    { 0xf6e268a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4268a3, SUC_lexicon_tags_17 },
    { 0x70c268a3, SUC_lexicon_tags_49 },
    { 0x4f4268a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06268a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0268af, SUC_lexicon_tags_12 },
    { 0xfea268b0, SUC_lexicon_tags_2 },
    { 0xff4268b1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234268b6, SUC_lexicon_tags_14 },
    { 0xa00268b6, SUC_lexicon_tags_23 },
    { 0x8da268b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2268bd, SUC_lexicon_tags_55 },
    { 0xefe268bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a268c2, SUC_lexicon_tags_61 },
    { 0xfb6268c3, SUC_lexicon_tags_15 },
    { 0xe6268c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc2e268c6, SUC_lexicon_tags_8 },
    { 0x15a268c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe268ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c268d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c268d9, SUC_lexicon_tags_10 },
    { 0xd08268d9, SUC_lexicon_tags_7 },
    { 0xd12268db, SUC_lexicon_tags_0 },
    { 0x386268dc, SUC_lexicon_tags_2 },
    { 0x4be268da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8268e0, SUC_lexicon_tags_11 },
    { 0x19a268e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb08268e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc8268e5, SUC_lexicon_tags_37 },
    { 0x6d6268e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd36268e8, SUC_lexicon_tags_25 },
    { 0xe9a268e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600268ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e268f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2268f3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbbe268f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9e268f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6268fa, SUC_lexicon_tags_8 },
    { 0x162268fb, SUC_lexicon_tags_85 },
    { 0xa30268fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea268ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84226901, SUC_lexicon_tags_37 },
    { 0x1a026902, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6de26904, SUC_lexicon_tags_47 },
    { 0xa0826904, SUC_lexicon_tags_60 },
    { 0xc8a26906, SUC_lexicon_tags_15 },
    { 0x38426907, SUC_lexicon_tags_28 },
    { 0xccc26907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2062690d, SUC_lexicon_tags_37 },
    { 0x65e2690d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1a26910, SUC_lexicon_tags_38 },
    { 0xa4a26911, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6426913, SUC_lexicon_tags_23 },
    { 0xc226913, SUC_lexicon_tags_62 },
    { 0xf1e26915, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xdd626917, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a2691c, SUC_lexicon_tags_27 },
    { 0x8422691c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4a2691f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71426921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e2692b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63a2692d, SUC_lexicon_tags_43 },
    { 0x3de2692d, SUC_lexicon_tags_8 },
    { 0xdc2692f, SUC_lexicon_tags_91 },
    { 0x7c226930, SUC_lexicon_tags_2 },
    { 0xd6826930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a26934, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a2693c, SUC_lexicon_tags_2 },
    { 0xef22693c, SUC_lexicon_tags_2 },
    { 0xcc42693d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42826942, SUC_lexicon_tags_2 },
    { 0x93c26943, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7026946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d42694a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd082694d, SUC_lexicon_tags_38 },
    { 0x8b22694e, SUC_lexicon_tags_8 },
    { 0xed82694e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c426956, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc626958, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c42695c, SUC_lexicon_tags_0 },
    { 0xa702695d, SUC_lexicon_tags_7 },
    { 0x7f62695e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfa26960, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67626967, SUC_lexicon_tags_17 },
    { 0x53e26968, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d82696a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf302696d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0026970, SUC_lexicon_tags_1 },
    { 0x62026971, SUC_lexicon_tags_1 },
    { 0x98e26971, SUC_lexicon_tags_48 },
    { 0xe7a26973, SUC_lexicon_tags_25 },
    { 0x9fa26974, SUC_lexicon_tags_10 },
    { 0x71a26971, SUC_lexicon_tags_47 },
    { 0x6b826973, SUC_lexicon_tags_28 },
    { 0xb8826972, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa0026979, SUC_lexicon_tags_47 },
    { 0x4f826979, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56c2697c, SUC_lexicon_tags_38 },
    { 0xdda2697d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x88c2697f, SUC_lexicon_tags_31 },
    { 0x4a026980, SUC_lexicon_tags_9 },
    { 0x1d22697f, SUC_lexicon_tags_9 },
    { 0x6fe26981, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c226985, SUC_lexicon_tags_11 },
    { 0x6226985, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf226989, SUC_lexicon_tags_84 },
    { 0x2a626989, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce2698d, SUC_lexicon_tags_33 },
    { 0xf6a2698e, SUC_lexicon_tags_1 },
    { 0xa8c2698e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0026991, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2226993, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d226996, SUC_lexicon_tags_23 },
    { 0x5a626997, SUC_lexicon_tags_11 },
    { 0x4be26998, SUC_lexicon_tags_91 },
    { 0xc1226999, SUC_lexicon_tags_37 },
    { 0x9ac2699a, SUC_lexicon_tags_14 },
    { 0xa4026999, SUC_lexicon_tags_45 },
    { 0x6aa2699c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc269a0, SUC_lexicon_tags_11 },
    { 0xcfa269a1, SUC_lexicon_tags_37 },
    { 0x8c0269a2, SUC_lexicon_tags_17 },
    { 0x5fa269a3, SUC_lexicon_tags_19 },
    { 0x10c269a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e269a7, SUC_lexicon_tags_21 },
    { 0xa88269a8, SUC_lexicon_tags_45 },
    { 0xc92269a9, SUC_lexicon_tags_45 },
    { 0x262269aa, SUC_lexicon_tags_6 },
    { 0xfbe269a9, SUC_lexicon_tags_17 },
    { 0xdd0269a9, SUC_lexicon_tags_8 },
    { 0x192269a7, SUC_lexicon_tags_12 },
    { 0x470269a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364269b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe24269b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5c0269b8, SUC_lexicon_tags_4 },
    { 0x194269b8, SUC_lexicon_tags_2 },
    { 0x842269ba, SUC_lexicon_tags_11 },
    { 0xe26269bb, SUC_lexicon_tags_12 },
    { 0xbc269b9, SUC_lexicon_tags_63 },
    { 0x6bc269bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8269c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988269ca, SUC_lexicon_tags_25 },
    { 0xc10269ca, SUC_lexicon_tags_37 },
    { 0x50e269cc, SUC_lexicon_tags_25 },
    { 0x734269cd, SUC_lexicon_tags_43 },
    { 0xfc6269ce, SUC_lexicon_tags_12 },
    { 0x6ac269ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd18269d1, SUC_lexicon_tags_15 },
    { 0x7d8269d2, SUC_lexicon_tags_7 },
    { 0x9aa269d3, SUC_lexicon_tags_10 },
    { 0x3a8269d1, SUC_lexicon_tags_38 },
    { 0x28a269d5, SUC_lexicon_tags_47 },
    { 0x258269d6, SUC_lexicon_tags_12 },
    { 0x58269d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2269dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6c269df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34269e3, SUC_lexicon_tags_15 },
    { 0xffe269e3, SUC_lexicon_tags_84 },
    { 0x100269e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75a269e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332269f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72269fa, SUC_lexicon_tags_11 },
    { 0xe8e269fb, SUC_lexicon_tags_11 },
    { 0xc7e269fb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc226a00, SUC_lexicon_tags_17 },
    { 0xf8626a01, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b426a04, SUC_lexicon_tags_2 },
    { 0xc4226a05, SUC_lexicon_tags_21 },
    { 0x82826a06, SUC_lexicon_tags_14 },
    { 0x7a026a07, SUC_lexicon_tags_12 },
    { 0xc4426a08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd6026a0a, SUC_lexicon_tags_66 },
    { 0x25e26a0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e26a17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a826a1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36e26a1f, SUC_lexicon_tags_46 },
    { 0xefa26a20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94626a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcda26a26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x16826a29, SUC_lexicon_tags_179 },
    { 0x4a26a29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e226a2d, SUC_lexicon_tags_11 },
    { 0x81826a2e, SUC_lexicon_tags_16 },
    { 0x59a26a2f, SUC_lexicon_tags_1 },
    { 0x2de26a30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e26a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef226a38, SUC_lexicon_tags_37 },
    { 0x13426a39, SUC_lexicon_tags_15 },
    { 0xdb026a39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f426a3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x33a26a3e, SUC_lexicon_tags_1 },
    { 0xf4826a3f, SUC_lexicon_tags_80 },
    { 0x65e26a40, SUC_lexicon_tags_39 },
    { 0x61026a41, SUC_lexicon_tags_31 },
    { 0xc8426a41, SUC_lexicon_tags_12 },
    { 0xcd826a3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe426a49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a26a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46426a55, SUC_lexicon_tags_8 },
    { 0x39826a56, SUC_lexicon_tags_17 },
    { 0x69026a55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b026a59, SUC_lexicon_tags_0 },
    { 0xce626a5a, SUC_lexicon_tags_2 },
    { 0xb9826a5a, SUC_lexicon_tags_19 },
    { 0xb8226a5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a26a63, SUC_lexicon_tags_37 },
    { 0xa1a26a63, SUC_lexicon_tags_27 },
    { 0x44626a65, SUC_lexicon_tags_25 },
    { 0x6ce26a65, SUC_lexicon_tags_19 },
    { 0xa0426a65, SUC_lexicon_tags_33 },
    { 0xa8c26a68, SUC_lexicon_tags_7 },
    { 0xa6026a69, SUC_lexicon_tags_27 },
    { 0x44026a63, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x4f026a6c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c26a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20026a83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f226a86, SUC_lexicon_tags_49 },
    { 0x13e26a87, SUC_lexicon_tags_21 },
    { 0x13426a87, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde826a8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae26a92, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xea826a95, SUC_lexicon_tags_60 },
    { 0x2d426a96, SUC_lexicon_tags_26 },
    { 0x66626a97, SUC_lexicon_tags_37 },
    { 0xa3226a98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60026a9d, SUC_lexicon_tags_47 },
    { 0x8d226a9e, SUC_lexicon_tags_7 },
    { 0xa6e26a9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca26aa4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1b426aa6, SUC_lexicon_tags_12 },
    { 0x3826aa7, SUC_lexicon_tags_19 },
    { 0x9a426aa8, SUC_lexicon_tags_29 },
    { 0xa5026aa8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcfe26aab, SUC_lexicon_tags_63 },
    { 0xa2826aab, SUC_lexicon_tags_25 },
    { 0x94826aad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2226ab0, SUC_lexicon_tags_23 },
    { 0xce826ab1, SUC_lexicon_tags_9 },
    { 0x3ae26ab1, SUC_lexicon_tags_43 },
    { 0xd826ab3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd826ab6, SUC_lexicon_tags_25 },
    { 0x96626ab6, SUC_lexicon_tags_82 },
    { 0x5d426ab8, SUC_lexicon_tags_49 },
    { 0xde826ab9, SUC_lexicon_tags_31 },
    { 0x16826aba, SUC_lexicon_tags_37 },
    { 0x38626abb, SUC_lexicon_tags_15 },
    { 0xfda26abb, SUC_lexicon_tags_26 },
    { 0xc3c26abb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f026abf, SUC_lexicon_tags_37 },
    { 0x2c026abf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe826ac4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10426ac7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x87a26ac9, SUC_lexicon_tags_15 },
    { 0x3226ac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3026ad0, SUC_lexicon_tags_1 },
    { 0xf9a26ad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6226ad5, SUC_lexicon_tags_6 },
    { 0xc9626ad6, SUC_lexicon_tags_49 },
    { 0xa4026ad5, SUC_lexicon_tags_11 },
    { 0xee626ad7, SUC_lexicon_tags_2 },
    { 0xcbe26ad9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e26ade, SUC_lexicon_tags_59 },
    { 0xf2e26adf, SUC_lexicon_tags_32 },
    { 0xdc626ae0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a26ae3, SUC_lexicon_tags_0 },
    { 0x9c226ae4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e26aec, SUC_lexicon_tags_16 },
    { 0x7bc26aec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c626af0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x85c26af2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a26af8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc026afd, SUC_lexicon_tags_44 },
    { 0x98626afd, SUC_lexicon_tags_46 },
    { 0x29a26aff, SUC_lexicon_tags_34 },
    { 0x2ba26b00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaee26b02, SUC_lexicon_tags_15 },
    { 0xa6826b03, SUC_lexicon_tags_19 },
    { 0x15e26b04, SUC_lexicon_tags_33 },
    { 0x9b426b05, SUC_lexicon_tags_103 },
    { 0x94626b06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8226b08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88826b10, SUC_lexicon_tags_15 },
    { 0x99626b11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96626b13, SUC_lexicon_tags_47 },
    { 0xe6a26b14, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62826b18, SUC_lexicon_tags_37 },
    { 0xaba26b18, SUC_lexicon_tags_8 },
    { 0x66c26b19, SUC_lexicon_tags_17 },
    { 0x17826b1b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81226b1f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x73826b22, SUC_lexicon_tags_15 },
    { 0x29226b23, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x82626b26, SUC_lexicon_tags_45 },
    { 0xcb826b26, SUC_lexicon_tags_17 },
    { 0xaf826b28, SUC_lexicon_tags_6 },
    { 0x1f226b29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x76026b2c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x39826b2e, SUC_lexicon_tags_1 },
    { 0xb0226b2f, SUC_lexicon_tags_1 },
    { 0x9ca26b2e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xba226b33, SUC_lexicon_tags_15 },
    { 0xc9226b34, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a226b37, SUC_lexicon_tags_2 },
    { 0xd5626b38, SUC_lexicon_tags_57 },
    { 0xd4a26b39, SUC_lexicon_tags_25 },
    { 0x44626b38, SUC_lexicon_tags_26 },
    { 0x7b026b3b, SUC_lexicon_tags_7 },
    { 0x8c426b37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6a26b3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb626b40, SUC_lexicon_tags_28 },
    { 0xbc226b40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c26b46, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c26b4b, SUC_lexicon_tags_37 },
    { 0x44826b4b, SUC_lexicon_tags_15 },
    { 0x72026b4b, SUC_lexicon_tags_15 },
    { 0xfa226b4d, SUC_lexicon_tags_49 },
    { 0xedc26b4f, SUC_lexicon_tags_12 },
    { 0xf3a26b4f, SUC_lexicon_tags_21 },
    { 0x7226b51, SUC_lexicon_tags_2 },
    { 0xd9626b4c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82226b59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab826b5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3226b5e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8d826b60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c26b66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29e26b68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98226b6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe626b6e, SUC_lexicon_tags_11 },
    { 0x83826b6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6026b76, SUC_lexicon_tags_12 },
    { 0xebe26b77, SUC_lexicon_tags_0 },
    { 0xdbc26b78, SUC_lexicon_tags_17 },
    { 0xc7a26b76, SUC_lexicon_tags_7 },
    { 0x7b226b78, SUC_lexicon_tags_77 },
    { 0x3d026b7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e26b7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19826b83, SUC_lexicon_tags_6 },
    { 0xd9426b84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae26b89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90e26b8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e626b8d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc26b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8826b93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac26b98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90026ba0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f626ba7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15826bb3, SUC_lexicon_tags_1 },
    { 0x7d826bb4, SUC_lexicon_tags_11 },
    { 0xbe226bb5, SUC_lexicon_tags_33 },
    { 0xb3426bb6, SUC_lexicon_tags_1 },
    { 0x6f226bb4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62026bbf, SUC_lexicon_tags_33 },
    { 0xc2426bc0, SUC_lexicon_tags_17 },
    { 0x7a626bc1, SUC_lexicon_tags_1 },
    { 0xf1626bc2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3026bc7, SUC_lexicon_tags_0 },
    { 0x87e26bc8, SUC_lexicon_tags_3 },
    { 0x91626bc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96826bd0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8026bd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7c26bd5, SUC_lexicon_tags_25 },
    { 0x9f826bd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e26bdc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1226bdf, SUC_lexicon_tags_13 },
    { 0x14e26be0, SUC_lexicon_tags_2 },
    { 0x26626be1, SUC_lexicon_tags_27 },
    { 0x92e26be1, SUC_lexicon_tags_37 },
    { 0x52626be2, SUC_lexicon_tags_49 },
    { 0x9ee26be4, SUC_lexicon_tags_62 },
    { 0x2ce26be4, SUC_lexicon_tags_25 },
    { 0x1b026be4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8826be9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98026bed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a26bf0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x61826bf2, SUC_lexicon_tags_19 },
    { 0xa5826bf3, SUC_lexicon_tags_37 },
    { 0x7e426bf4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c26bf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x18626bfc, SUC_lexicon_tags_25 },
    { 0xc3e26bfc, SUC_lexicon_tags_35 },
    { 0x20a26bfc, SUC_lexicon_tags_1 },
    { 0xf2c26bff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4026c05, SUC_lexicon_tags_15 },
    { 0xb3a26c06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc26c09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38226c0f, SUC_lexicon_tags_30 },
    { 0xc3e26c10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc26c16, SUC_lexicon_tags_43 },
    { 0x88426c17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4626c19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a426c1c, SUC_lexicon_tags_19 },
    { 0x88a26c1d, SUC_lexicon_tags_60 },
    { 0x53626c1e, SUC_lexicon_tags_37 },
    { 0x48c26c1e, SUC_lexicon_tags_12 },
    { 0xd5c26c20, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca26c24, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9c826c26, SUC_lexicon_tags_63 },
    { 0xaf626c27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac26c2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f426c2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a26c30, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba26c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaca26c36, SUC_lexicon_tags_68 },
    { 0xb5226c37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49626c39, SUC_lexicon_tags_7 },
    { 0xa8a26c3a, SUC_lexicon_tags_25 },
    { 0x7a426c3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe826c41, SUC_lexicon_tags_17 },
    { 0x6aa26c41, SUC_lexicon_tags_26 },
    { 0x70a26c43, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x49226c46, SUC_lexicon_tags_63 },
    { 0xf7a26c46, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa426c49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e226c4b, SUC_lexicon_tags_2 },
    { 0x84026c4c, SUC_lexicon_tags_37 },
    { 0x31026c4d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28826c51, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xbd226c53, SUC_lexicon_tags_0 },
    { 0x67e26c54, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xece26c56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b426c58, SUC_lexicon_tags_11 },
    { 0x95a26c59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x25e26c5b, SUC_lexicon_tags_8 },
    { 0xc1826c5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c26c5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e26c62, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a26c68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63826c6d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec26c72, SUC_lexicon_tags_23 },
    { 0x49426c73, SUC_lexicon_tags_25 },
    { 0x3b026c73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67226c77, SUC_lexicon_tags_19 },
    { 0x29e26c77, SUC_lexicon_tags_25 },
    { 0x83626c79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4426c86, SUC_lexicon_tags_14 },
    { 0x48626c86, SUC_lexicon_tags_1 },
    { 0xabc26c88, SUC_lexicon_tags_7 },
    { 0x9ca26c87, SUC_lexicon_tags_37 },
    { 0xe9e26c8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc26c8e, SUC_lexicon_tags_23 },
    { 0x69e26c8f, SUC_lexicon_tags_11 },
    { 0x41826c8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2226c92, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa26c99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1426c9c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb9626c9e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc26ca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a26ca8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66426cad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19226caf, SUC_lexicon_tags_7 },
    { 0xd4a26cb0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64826cb6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x46026cb9, SUC_lexicon_tags_7 },
    { 0xbee26cba, SUC_lexicon_tags_25 },
    { 0xf8626cba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6226cbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a426ccd, SUC_lexicon_tags_25 },
    { 0x3ce26cce, SUC_lexicon_tags_26 },
    { 0x7e26cce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x39626cd1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1826cd8, SUC_lexicon_tags_11 },
    { 0x46026cd9, SUC_lexicon_tags_37 },
    { 0x96e26cda, SUC_lexicon_tags_14 },
    { 0x41226cd9, SUC_lexicon_tags_11 },
    { 0x59626cdc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7426cde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c26ce3, SUC_lexicon_tags_11 },
    { 0x5ae26ce3, SUC_lexicon_tags_151 },
    { 0x32826ce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73226cea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49626ced, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c226cf0, SUC_lexicon_tags_2 },
    { 0x9d026cf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96226cf4, SUC_lexicon_tags_15 },
    { 0x52026cf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18426cf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x68826cf9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda226d05, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xacc26d07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51626d09, SUC_lexicon_tags_11 },
    { 0xec626d09, SUC_lexicon_tags_14 },
    { 0xb3c26d0a, SUC_lexicon_tags_25 },
    { 0x22226d0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x87426d0e, SUC_lexicon_tags_17 },
    { 0xff226d0f, SUC_lexicon_tags_6 },
    { 0x69e26d0f, SUC_lexicon_tags_43 },
    { 0xc1826d0f, SUC_lexicon_tags_11 },
    { 0x1d826d11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b626d15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a26d19, SUC_lexicon_tags_28 },
    { 0xe1e26d1a, SUC_lexicon_tags_8 },
    { 0xa7a26d1b, SUC_lexicon_tags_7 },
    { 0xb6e26d1c, SUC_lexicon_tags_21 },
    { 0x86626d1b, SUC_lexicon_tags_14 },
    { 0x3b426d19, SUC_lexicon_tags_63 },
    { 0x86c26d1f, SUC_lexicon_tags_1 },
    { 0xca026d20, SUC_lexicon_tags_1 },
    { 0x77826d21, SUC_lexicon_tags_68 },
    { 0xf3026d20, SUC_lexicon_tags_11 },
    { 0xdec26d19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16426d28, SUC_lexicon_tags_16 },
    { 0x69226d29, SUC_lexicon_tags_60 },
    { 0xc5826d2a, SUC_lexicon_tags_11 },
    { 0xd9426d2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c826d31, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbaa26d33, SUC_lexicon_tags_12 },
    { 0xa8026d34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c26d36, SUC_lexicon_tags_25 },
    { 0x9cc26d36, SUC_lexicon_tags_11 },
    { 0x52226d38, SUC_lexicon_tags_66 },
    { 0x9e826d39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18c26d3b, SUC_lexicon_tags_37 },
    { 0x74026d3c, SUC_lexicon_tags_12 },
    { 0x41026d3c, SUC_lexicon_tags_23 },
    { 0x2f426d3e, SUC_lexicon_tags_11 },
    { 0x93426d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee26d42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee026d44, SUC_lexicon_tags_9 },
    { 0xcb226d44, SUC_lexicon_tags_12 },
    { 0x1e226d46, SUC_lexicon_tags_37 },
    { 0xb5c26d45, SUC_lexicon_tags_21 },
    { 0xc3826d45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84026d4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78826d4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3026d55, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x25626d57, SUC_lexicon_tags_47 },
    { 0xe5a26d58, SUC_lexicon_tags_60 },
    { 0x19e26d58, SUC_lexicon_tags_3 },
    { 0x61e26d5a, SUC_lexicon_tags_26 },
    { 0xa1026d5a, SUC_lexicon_tags_92 },
    { 0xc2626d5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e26d5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94226d62, SUC_lexicon_tags_15 },
    { 0x7f626d63, SUC_lexicon_tags_45 },
    { 0x22426d64, SUC_lexicon_tags_25 },
    { 0xf4a26d65, SUC_lexicon_tags_30 },
    { 0xee026d66, SUC_lexicon_tags_25 },
    { 0xbba26d67, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x71e26d69, SUC_lexicon_tags_25 },
    { 0x24826d6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80626d70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x65226d73, SUC_lexicon_tags_28 },
    { 0x8026d74, SUC_lexicon_tags_25 },
    { 0xba26d74, SUC_lexicon_tags_85 },
    { 0x89026d73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e26d7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0426d7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2426d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca26d82, SUC_lexicon_tags_2 },
    { 0xcce26d83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e26d87, SUC_lexicon_tags_39 },
    { 0x2c426d88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8c26d8a, SUC_lexicon_tags_11 },
    { 0x81a26d8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d826d8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62826d92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9026d96, SUC_lexicon_tags_12 },
    { 0x61a26d97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfc26d99, SUC_lexicon_tags_1 },
    { 0x92a26d9a, SUC_lexicon_tags_1 },
    { 0x86e26d9b, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x16e26d9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30226da3, SUC_lexicon_tags_63 },
    { 0xaf826da3, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c626da7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9c26da9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c26db3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x83e26db5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe226db8, SUC_lexicon_tags_66 },
    { 0xe1c26db9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe826dbb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc26dc4, SUC_lexicon_tags_0 },
    { 0x56426dc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62626dcb, SUC_lexicon_tags_17 },
    { 0xd8e26dcb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd5426dce, SUC_lexicon_tags_47 },
    { 0x69226dcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26826ddb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5fc26ddd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c626de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12626de3, SUC_lexicon_tags_0 },
    { 0xfbe26de4, SUC_lexicon_tags_16 },
    { 0x12e26de5, SUC_lexicon_tags_17 },
    { 0x50a26de6, SUC_lexicon_tags_38 },
    { 0xbd426de3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26426de9, SUC_lexicon_tags_25 },
    { 0xe3c26dea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a26df3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e26df7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63826df9, SUC_lexicon_tags_31 },
    { 0x92a26dfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a26dfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda226dff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52426e03, SUC_lexicon_tags_68 },
    { 0xf0026e04, SUC_lexicon_tags_1 },
    { 0x43c26e05, SUC_lexicon_tags_42 },
    { 0x79626e04, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xeec26e08, SUC_lexicon_tags_2 },
    { 0x5226e09, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e26e10, SUC_lexicon_tags_86 },
    { 0x57826e11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37226e16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82226e18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf626e1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc626e1d, SUC_lexicon_tags_31 },
    { 0xea826e1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc626e21, SUC_lexicon_tags_15 },
    { 0x1a26e21, SUC_lexicon_tags_17 },
    { 0xbd226e22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46226e25, SUC_lexicon_tags_479 },
    { 0, NULL },
    { 0xfc626e27, SUC_lexicon_tags_25 },
    { 0x4f026e28, SUC_lexicon_tags_11 },
    { 0x1ec26e29, SUC_lexicon_tags_10 },
    { 0x1f826e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5826e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda26e31, SUC_lexicon_tags_17 },
    { 0xbc826e32, SUC_lexicon_tags_161 },
    { 0x61626e32, SUC_lexicon_tags_15 },
    { 0x9d026e34, SUC_lexicon_tags_17 },
    { 0x4fa26e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71226e37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a26e3b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x78a26e3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5626e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35e26e43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0626e46, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2b426e48, SUC_lexicon_tags_11 },
    { 0x5ba26e49, SUC_lexicon_tags_37 },
    { 0xc8a26e4a, SUC_lexicon_tags_24 },
    { 0x4a026e4b, SUC_lexicon_tags_68 },
    { 0xf3a26e4c, SUC_lexicon_tags_65 },
    { 0xe4626e4d, SUC_lexicon_tags_11 },
    { 0x2b626e4e, SUC_lexicon_tags_18 },
    { 0x49826e4e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa026e54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2226e58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59e26e5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3426e5d, SUC_lexicon_tags_2 },
    { 0x16426e5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c26e68, SUC_lexicon_tags_2 },
    { 0x50026e69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd826e6f, SUC_lexicon_tags_77 },
    { 0x2b026e70, SUC_lexicon_tags_12 },
    { 0x8026e6f, SUC_lexicon_tags_25 },
    { 0xd0426e72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7c26e74, SUC_lexicon_tags_27 },
    { 0x6f226e75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17626e77, SUC_lexicon_tags_37 },
    { 0xb8826e78, SUC_lexicon_tags_0 },
    { 0xe6626e79, SUC_lexicon_tags_11 },
    { 0x9f026e7a, SUC_lexicon_tags_33 },
    { 0x28826e78, SUC_lexicon_tags_16 },
    { 0x65a26e7c, SUC_lexicon_tags_2 },
    { 0x2f426e78, SUC_lexicon_tags_27 },
    { 0x57426e7e, SUC_lexicon_tags_14 },
    { 0xb8626e7d, SUC_lexicon_tags_11 },
    { 0x5ca26e80, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x24426e83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x27826e86, SUC_lexicon_tags_37 },
    { 0x2c626e87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14c26e89, SUC_lexicon_tags_17 },
    { 0xf2426e8a, SUC_lexicon_tags_60 },
    { 0xdfc26e8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35826e8f, SUC_lexicon_tags_11 },
    { 0x78e26e8f, SUC_lexicon_tags_37 },
    { 0x8ce26e8f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf426e94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5426e97, SUC_lexicon_tags_29 },
    { 0x59826e98, SUC_lexicon_tags_37 },
    { 0x88a26e99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d626ea6, SUC_lexicon_tags_15 },
    { 0x83626ea7, SUC_lexicon_tags_28 },
    { 0xd8026ea8, SUC_lexicon_tags_19 },
    { 0xb4026ea9, SUC_lexicon_tags_29 },
    { 0xab026eaa, SUC_lexicon_tags_5 },
    { 0x65626ea7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e226eb1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa7e26eb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c26ebf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1b426ec1, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x94026ec3, SUC_lexicon_tags_15 },
    { 0x60826ec3, SUC_lexicon_tags_2 },
    { 0xe1e26ec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97826eca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab226ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa426ed4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83426ed9, SUC_lexicon_tags_17 },
    { 0xd9626ed9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c26ee7, SUC_lexicon_tags_5 },
    { 0x24826ee8, SUC_lexicon_tags_49 },
    { 0x2026ee9, SUC_lexicon_tags_15 },
    { 0xcde26eea, SUC_lexicon_tags_34 },
    { 0x71626eeb, SUC_lexicon_tags_85 },
    { 0x90826eea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a26eef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0226ef2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52826ef5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x71426ef8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a26efb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc26f03, SUC_lexicon_tags_0 },
    { 0xd1a26f03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc26f08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30626f0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc026f0f, SUC_lexicon_tags_11 },
    { 0x36226f10, SUC_lexicon_tags_33 },
    { 0xf3e26f11, SUC_lexicon_tags_19 },
    { 0x2f426f12, SUC_lexicon_tags_85 },
    { 0x56826f13, SUC_lexicon_tags_61 },
    { 0x30826f13, SUC_lexicon_tags_2 },
    { 0x73e26f12, SUC_lexicon_tags_48 },
    { 0x8ea26f15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73826f1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6626f20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbfe26f22, SUC_lexicon_tags_38 },
    { 0xca26f23, SUC_lexicon_tags_37 },
    { 0xf8a26f24, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce426f29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e426f2c, SUC_lexicon_tags_43 },
    { 0x74a26f2d, SUC_lexicon_tags_11 },
    { 0xa8c26f2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x826f30, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x39426f32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x10826f34, SUC_lexicon_tags_25 },
    { 0x5de26f34, SUC_lexicon_tags_2 },
    { 0xce226f34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdda26f38, SUC_lexicon_tags_5 },
    { 0xdc826f39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc5426f3b, SUC_lexicon_tags_11 },
    { 0x53426f3b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x66226f3e, SUC_lexicon_tags_23 },
    { 0x22c26f3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52226f41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7026f48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca26f51, SUC_lexicon_tags_1 },
    { 0xabc26f52, SUC_lexicon_tags_15 },
    { 0x2d226f53, SUC_lexicon_tags_9 },
    { 0x2d426f52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac26f57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0626f59, SUC_lexicon_tags_27 },
    { 0x66026f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a26f62, SUC_lexicon_tags_47 },
    { 0x4e826f62, SUC_lexicon_tags_63 },
    { 0x80626f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6826f6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca26f70, SUC_lexicon_tags_6 },
    { 0xd9426f70, SUC_lexicon_tags_33 },
    { 0x3aa26f72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0426f75, SUC_lexicon_tags_7 },
    { 0x48426f76, SUC_lexicon_tags_37 },
    { 0x15e26f75, SUC_lexicon_tags_7 },
    { 0xbda26f78, SUC_lexicon_tags_82 },
    { 0xc3826f79, SUC_lexicon_tags_33 },
    { 0x4b626f7a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f226f7f, SUC_lexicon_tags_17 },
    { 0x72e26f7f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x65c26f82, SUC_lexicon_tags_37 },
    { 0x46026f83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5226f86, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1426f8e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a26f94, SUC_lexicon_tags_27 },
    { 0x2ca26f94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6426f98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2ca26f9a, SUC_lexicon_tags_21 },
    { 0xe0c26f9b, SUC_lexicon_tags_24 },
    { 0x8ac26f9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x34c26f9e, SUC_lexicon_tags_56 },
    { 0x5ce26f9f, SUC_lexicon_tags_4 },
    { 0x7f226f9e, SUC_lexicon_tags_37 },
    { 0xc6426fa1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6826fa6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0626faa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe9226fac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8826fb0, SUC_lexicon_tags_33 },
    { 0x8be26fb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x17426fb4, SUC_lexicon_tags_27 },
    { 0xec226fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc426fbb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d026fbe, SUC_lexicon_tags_2 },
    { 0xa5026fbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e26fc3, SUC_lexicon_tags_7 },
    { 0x54626fc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc26fca, SUC_lexicon_tags_25 },
    { 0x86026fcb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x20626fce, SUC_lexicon_tags_29 },
    { 0x9b026fce, SUC_lexicon_tags_7 },
    { 0x42a26fd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70826fd4, SUC_lexicon_tags_10 },
    { 0x68626fd5, SUC_lexicon_tags_0 },
    { 0xd7826fd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0226fd9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba26fdd, SUC_lexicon_tags_33 },
    { 0xd8226fde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e826fe0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x56e26fe2, SUC_lexicon_tags_14 },
    { 0xc8a26fe2, SUC_lexicon_tags_10 },
    { 0x4b226fe4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd826fe7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79826fec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6626ff0, SUC_lexicon_tags_21 },
    { 0x27826ff0, SUC_lexicon_tags_47 },
    { 0x75026ff0, SUC_lexicon_tags_27 },
    { 0x98226ff1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2826ff5, SUC_lexicon_tags_4 },
    { 0x67226ff5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x226ff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa26ffe, SUC_lexicon_tags_17 },
    { 0xcac26fff, SUC_lexicon_tags_14 },
    { 0x1ba26fff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fa27002, SUC_lexicon_tags_15 },
    { 0x89827002, SUC_lexicon_tags_17 },
    { 0xef427003, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ce27006, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a27013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c27016, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a22701a, SUC_lexicon_tags_11 },
    { 0x1b22701a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd942701d, SUC_lexicon_tags_17 },
    { 0x9362701e, SUC_lexicon_tags_18 },
    { 0x102701e, SUC_lexicon_tags_2 },
    { 0xa9a2701f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa4227022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec27025, SUC_lexicon_tags_17 },
    { 0x27027025, SUC_lexicon_tags_33 },
    { 0xd0a27025, SUC_lexicon_tags_33 },
    { 0xd2427025, SUC_lexicon_tags_37 },
    { 0xb1827029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb82702b, SUC_lexicon_tags_37 },
    { 0x5122702c, SUC_lexicon_tags_47 },
    { 0x1002702b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94427032, SUC_lexicon_tags_15 },
    { 0xea027033, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c27036, SUC_lexicon_tags_30 },
    { 0x6b627036, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x3cc27039, SUC_lexicon_tags_60 },
    { 0xd722703a, SUC_lexicon_tags_307 },
    { 0x1522703b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48a2703d, SUC_lexicon_tags_21 },
    { 0x4182703e, SUC_lexicon_tags_17 },
    { 0x9e22703e, SUC_lexicon_tags_33 },
    { 0x46c2703e, SUC_lexicon_tags_27 },
    { 0x11227041, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c27045, SUC_lexicon_tags_12 },
    { 0x78c27046, SUC_lexicon_tags_19 },
    { 0x79a27047, SUC_lexicon_tags_14 },
    { 0x10227047, SUC_lexicon_tags_27 },
    { 0x3227049, SUC_lexicon_tags_38 },
    { 0xa5227049, SUC_lexicon_tags_4 },
    { 0x3c42704b, SUC_lexicon_tags_11 },
    { 0x99a2704c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb627050, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d427053, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c27059, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1f82705b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb302705d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce27061, SUC_lexicon_tags_1 },
    { 0x3d827062, SUC_lexicon_tags_27 },
    { 0x80827063, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a27068, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde27071, SUC_lexicon_tags_14 },
    { 0xa7c27072, SUC_lexicon_tags_15 },
    { 0x26027072, SUC_lexicon_tags_46 },
    { 0x6a427074, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x36627076, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ec27078, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c2707c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe27086, SUC_lexicon_tags_23 },
    { 0x92e27086, SUC_lexicon_tags_10 },
    { 0xe6a27088, SUC_lexicon_tags_7 },
    { 0x2e027088, SUC_lexicon_tags_28 },
    { 0x9422708a, SUC_lexicon_tags_16 },
    { 0xb9627089, SUC_lexicon_tags_9 },
    { 0xada2708b, SUC_lexicon_tags_47 },
    { 0x46c2708a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x65e2708f, SUC_lexicon_tags_7 },
    { 0x36e27090, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x427093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb627095, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d027099, SUC_lexicon_tags_14 },
    { 0xe882709a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962709e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x422270a0, SUC_lexicon_tags_28 },
    { 0x396270a1, SUC_lexicon_tags_2 },
    { 0x1b2270a0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x822270a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94270a8, SUC_lexicon_tags_2 },
    { 0x98e270a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x754270ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x368270af, SUC_lexicon_tags_47 },
    { 0xeb2270af, SUC_lexicon_tags_0 },
    { 0x21c270af, SUC_lexicon_tags_37 },
    { 0xa5c270b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30270b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2270bb, SUC_lexicon_tags_82 },
    { 0x57e270bc, SUC_lexicon_tags_2 },
    { 0x6270bb, SUC_lexicon_tags_6 },
    { 0xbe6270bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380270c4, SUC_lexicon_tags_11 },
    { 0x692270c5, SUC_lexicon_tags_25 },
    { 0x806270c4, SUC_lexicon_tags_30 },
    { 0xbde270c5, SUC_lexicon_tags_25 },
    { 0x51e270c8, SUC_lexicon_tags_0 },
    { 0x868270c9, SUC_lexicon_tags_25 },
    { 0x900270c9, SUC_lexicon_tags_12 },
    { 0xd54270c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0270ce, SUC_lexicon_tags_19 },
    { 0x46a270ce, SUC_lexicon_tags_45 },
    { 0xb68270d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e270db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4270df, SUC_lexicon_tags_2 },
    { 0xcc8270e0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2c4270e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e270e6, SUC_lexicon_tags_1 },
    { 0x4b4270e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e270ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa270ed, SUC_lexicon_tags_12 },
    { 0x83e270ed, SUC_lexicon_tags_11 },
    { 0x724270ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb64270f1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12270f4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1f2270f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc270fd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa27105, SUC_lexicon_tags_4 },
    { 0xe2627105, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x99827108, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69a2710a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc602710d, SUC_lexicon_tags_38 },
    { 0x56c2710d, SUC_lexicon_tags_16 },
    { 0xc182710f, SUC_lexicon_tags_31 },
    { 0x4ba2710f, SUC_lexicon_tags_38 },
    { 0xc6c2710d, SUC_lexicon_tags_2 },
    { 0xf1a27112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e27115, SUC_lexicon_tags_1 },
    { 0x32227116, SUC_lexicon_tags_37 },
    { 0x54627116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fa27119, SUC_lexicon_tags_38 },
    { 0x2aa2711a, SUC_lexicon_tags_8 },
    { 0x31c2711b, SUC_lexicon_tags_2 },
    { 0x5782711a, SUC_lexicon_tags_30 },
    { 0xe5a2711d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1222712d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde27132, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e27137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c2713f, SUC_lexicon_tags_25 },
    { 0x86a27140, SUC_lexicon_tags_37 },
    { 0x39227141, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14227145, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb9027147, SUC_lexicon_tags_17 },
    { 0xe3c27148, SUC_lexicon_tags_0 },
    { 0x40227148, SUC_lexicon_tags_19 },
    { 0x64e2714a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa642714c, SUC_lexicon_tags_57 },
    { 0x28e2714d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa627157, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482715e, SUC_lexicon_tags_25 },
    { 0xd302715f, SUC_lexicon_tags_27 },
    { 0x7f827160, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8e27162, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x76827165, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe27168, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe082716a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa142716c, SUC_lexicon_tags_0 },
    { 0x6302716d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7682716f, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0xa8a27171, SUC_lexicon_tags_26 },
    { 0xf9627172, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95827179, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a22717e, SUC_lexicon_tags_19 },
    { 0x8c62717f, SUC_lexicon_tags_17 },
    { 0xd1c27180, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a827182, SUC_lexicon_tags_19 },
    { 0x1e027183, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52827185, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x95427187, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c2718c, SUC_lexicon_tags_0 },
    { 0xb462718d, SUC_lexicon_tags_6 },
    { 0x9f02718c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a427192, SUC_lexicon_tags_32 },
    { 0x5d827193, SUC_lexicon_tags_2 },
    { 0x9c27192, SUC_lexicon_tags_11 },
    { 0xb0627195, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c2719a, SUC_lexicon_tags_15 },
    { 0x2942719b, SUC_lexicon_tags_15 },
    { 0x43c2719b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c82719e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x30271a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6271a7, SUC_lexicon_tags_15 },
    { 0xafa271a7, SUC_lexicon_tags_62 },
    { 0xd8271a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb3c271ab, SUC_lexicon_tags_46 },
    { 0x510271ab, SUC_lexicon_tags_38 },
    { 0x248271ab, SUC_lexicon_tags_1 },
    { 0x186271ad, SUC_lexicon_tags_3 },
    { 0xe98271af, SUC_lexicon_tags_37 },
    { 0x2a2271b0, SUC_lexicon_tags_30 },
    { 0x17e271b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8271b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96c271b6, SUC_lexicon_tags_33 },
    { 0x124271b6, SUC_lexicon_tags_2 },
    { 0x7cc271b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468271bd, SUC_lexicon_tags_17 },
    { 0x850271bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b4271c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e271c3, SUC_lexicon_tags_12 },
    { 0x63c271c4, SUC_lexicon_tags_23 },
    { 0xa1e271c4, SUC_lexicon_tags_7 },
    { 0x174271c3, SUC_lexicon_tags_60 },
    { 0x50e271c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cc271c9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1b2271cb, SUC_lexicon_tags_3 },
    { 0x8de271cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a271cf, SUC_lexicon_tags_12 },
    { 0xaba271d0, SUC_lexicon_tags_4 },
    { 0x41c271d1, SUC_lexicon_tags_30 },
    { 0x3ce271d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c271dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37a271df, SUC_lexicon_tags_47 },
    { 0x8c8271e0, SUC_lexicon_tags_1 },
    { 0x266271df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1bc271e3, SUC_lexicon_tags_14 },
    { 0xcda271e3, SUC_lexicon_tags_37 },
    { 0x7e6271e5, SUC_lexicon_tags_2 },
    { 0xf66271e5, SUC_lexicon_tags_43 },
    { 0xf6c271e7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x864271ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79a271ec, SUC_lexicon_tags_6 },
    { 0xdb4271ed, SUC_lexicon_tags_64 },
    { 0xf74271ed, SUC_lexicon_tags_2 },
    { 0x2be271ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x928271f1, SUC_lexicon_tags_8 },
    { 0x5d8271f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf46271f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a271fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98027202, SUC_lexicon_tags_92 },
    { 0xf6c27203, SUC_lexicon_tags_12 },
    { 0x63427203, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c27207, SUC_lexicon_tags_29 },
    { 0x87e27207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x542720b, SUC_lexicon_tags_6 },
    { 0x2162720b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5882720f, SUC_lexicon_tags_6 },
    { 0xa242720f, SUC_lexicon_tags_9 },
    { 0xf3627211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa27214, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a227218, SUC_lexicon_tags_25 },
    { 0x94027218, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffa2721b, SUC_lexicon_tags_30 },
    { 0x62e2721b, SUC_lexicon_tags_14 },
    { 0xbaa2721d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4a2721f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e27226, SUC_lexicon_tags_11 },
    { 0x6ee27227, SUC_lexicon_tags_9 },
    { 0xd2c27226, SUC_lexicon_tags_25 },
    { 0xdde27226, SUC_lexicon_tags_11 },
    { 0x39827226, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa22722d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c82722f, SUC_lexicon_tags_66 },
    { 0x582722f, SUC_lexicon_tags_15 },
    { 0x6aa27231, SUC_lexicon_tags_63 },
    { 0x54427232, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc227239, SUC_lexicon_tags_47 },
    { 0xdd62723a, SUC_lexicon_tags_4 },
    { 0xf002723b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4b22723d, SUC_lexicon_tags_11 },
    { 0xd82723e, SUC_lexicon_tags_38 },
    { 0x86a2723f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x73627242, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e27245, SUC_lexicon_tags_2 },
    { 0x9c627246, SUC_lexicon_tags_7 },
    { 0x14027247, SUC_lexicon_tags_49 },
    { 0x82c27247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe662724a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce2724e, SUC_lexicon_tags_2 },
    { 0xd62724e, SUC_lexicon_tags_63 },
    { 0x40e27250, SUC_lexicon_tags_37 },
    { 0x68e27251, SUC_lexicon_tags_15 },
    { 0xc7827252, SUC_lexicon_tags_85 },
    { 0x1142724e, SUC_lexicon_tags_19 },
    { 0xf5c27250, SUC_lexicon_tags_17 },
    { 0x48027255, SUC_lexicon_tags_0 },
    { 0xfa427253, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8622725c, SUC_lexicon_tags_32 },
    { 0x6502725d, SUC_lexicon_tags_14 },
    { 0xc5e2725c, SUC_lexicon_tags_2 },
    { 0xd02725c, SUC_lexicon_tags_27 },
    { 0x37e2725f, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3227263, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33c27265, SUC_lexicon_tags_39 },
    { 0xffe27266, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc82726a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b82726c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e27272, SUC_lexicon_tags_37 },
    { 0xf7c27272, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0x4ea27275, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca27278, SUC_lexicon_tags_117 },
    { 0xf6c27279, SUC_lexicon_tags_10 },
    { 0xf9227279, SUC_lexicon_tags_11 },
    { 0x15c2727a, SUC_lexicon_tags_33 },
    { 0x2e827279, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a027281, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdde27283, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0627286, SUC_lexicon_tags_38 },
    { 0xca027287, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x89627289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2862728b, SUC_lexicon_tags_63 },
    { 0xfc22728c, SUC_lexicon_tags_11 },
    { 0xd722728b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a27290, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77c27292, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4d027294, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f02729b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c272a1, SUC_lexicon_tags_15 },
    { 0xcca272a2, SUC_lexicon_tags_31 },
    { 0xd7a272a3, SUC_lexicon_tags_11 },
    { 0x2e6272a4, SUC_lexicon_tags_47 },
    { 0xb10272a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x574272a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x158272aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe56272ac, SUC_lexicon_tags_30 },
    { 0x8ba272ad, SUC_lexicon_tags_6 },
    { 0xa22272ae, SUC_lexicon_tags_2 },
    { 0x460272ad, SUC_lexicon_tags_1 },
    { 0xae2272ae, SUC_lexicon_tags_29 },
    { 0x59a272b1, SUC_lexicon_tags_49 },
    { 0x992272b2, SUC_lexicon_tags_12 },
    { 0x388272b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6272b8, SUC_lexicon_tags_11 },
    { 0x7d2272b9, SUC_lexicon_tags_47 },
    { 0x3b6272ba, SUC_lexicon_tags_23 },
    { 0x934272bb, SUC_lexicon_tags_45 },
    { 0x7e272bb, SUC_lexicon_tags_0 },
    { 0xa9c272bd, SUC_lexicon_tags_37 },
    { 0x63c272bd, SUC_lexicon_tags_25 },
    { 0xb42272be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e272c2, SUC_lexicon_tags_12 },
    { 0x1e4272c2, SUC_lexicon_tags_25 },
    { 0x736272c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50272cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc8272ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c272dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaa6272de, SUC_lexicon_tags_179 },
    { 0xf42272de, SUC_lexicon_tags_6 },
    { 0x47c272df, SUC_lexicon_tags_61 },
    { 0x382272de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128272e5, SUC_lexicon_tags_7 },
    { 0x14c272e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f4272e8, SUC_lexicon_tags_0 },
    { 0xef4272e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6272f2, SUC_lexicon_tags_1 },
    { 0x43a272f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58272f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea272fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa272ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc27302, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9627305, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x71427308, SUC_lexicon_tags_11 },
    { 0xa1627308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e2730b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a27312, SUC_lexicon_tags_7 },
    { 0x94027313, SUC_lexicon_tags_7 },
    { 0x37a27314, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ce27316, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x19a27318, SUC_lexicon_tags_15 },
    { 0xefc27319, SUC_lexicon_tags_25 },
    { 0xf502731a, SUC_lexicon_tags_664 },
    { 0x2ec2731b, SUC_lexicon_tags_27 },
    { 0xb782731c, SUC_lexicon_tags_61 },
    { 0xe602731d, SUC_lexicon_tags_118 },
    { 0xbda2731e, SUC_lexicon_tags_66 },
    { 0xfce2731e, SUC_lexicon_tags_15 },
    { 0xdc62731c, SUC_lexicon_tags_47 },
    { 0x2ce2731b, SUC_lexicon_tags_33 },
    { 0x3542731d, SUC_lexicon_tags_47 },
    { 0x23827323, SUC_lexicon_tags_2 },
    { 0x3ee27324, SUC_lexicon_tags_35 },
    { 0x88e27325, SUC_lexicon_tags_27 },
    { 0x8a27326, SUC_lexicon_tags_1 },
    { 0x1a227327, SUC_lexicon_tags_17 },
    { 0x2227326, SUC_lexicon_tags_0 },
    { 0x8b227327, SUC_lexicon_tags_66 },
    { 0x7c2731d, SUC_lexicon_tags_11 },
    { 0x9e82732b, SUC_lexicon_tags_49 },
    { 0x7ce2732c, SUC_lexicon_tags_11 },
    { 0x17227320, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a27330, SUC_lexicon_tags_1 },
    { 0xabe27330, SUC_lexicon_tags_12 },
    { 0x34827330, SUC_lexicon_tags_46 },
    { 0x39c27333, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc2733f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85027343, SUC_lexicon_tags_19 },
    { 0x45227344, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a227348, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe602734d, SUC_lexicon_tags_37 },
    { 0x2f02734e, SUC_lexicon_tags_30 },
    { 0x5c82734e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9e27351, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21627355, SUC_lexicon_tags_17 },
    { 0x6b027355, SUC_lexicon_tags_2 },
    { 0xe0027357, SUC_lexicon_tags_14 },
    { 0x5cc27358, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd422735b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e42735d, SUC_lexicon_tags_7 },
    { 0x31c2735e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xbc427360, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1d827362, SUC_lexicon_tags_57 },
    { 0x55827363, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdea27365, SUC_lexicon_tags_0 },
    { 0x87827366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96c27368, SUC_lexicon_tags_106 },
    { 0xec227369, SUC_lexicon_tags_82 },
    { 0xa722736a, SUC_lexicon_tags_33 },
    { 0xe627369, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c27373, SUC_lexicon_tags_25 },
    { 0x30027373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee2737a, SUC_lexicon_tags_14 },
    { 0xadc2737b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4062737d, SUC_lexicon_tags_37 },
    { 0xa762737d, SUC_lexicon_tags_14 },
    { 0xc2a2737d, SUC_lexicon_tags_55 },
    { 0x1c827380, SUC_lexicon_tags_1 },
    { 0xb4427381, SUC_lexicon_tags_0 },
    { 0x72e27381, SUC_lexicon_tags_15 },
    { 0x9fa27383, SUC_lexicon_tags_63 },
    { 0xe5e27380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1027388, SUC_lexicon_tags_17 },
    { 0xaf027389, SUC_lexicon_tags_30 },
    { 0x8e22738a, SUC_lexicon_tags_63 },
    { 0x63c27388, SUC_lexicon_tags_15 },
    { 0x51227388, SUC_lexicon_tags_11 },
    { 0x6c2738c, SUC_lexicon_tags_38 },
    { 0x26c2738e, SUC_lexicon_tags_17 },
    { 0xf662738c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed627391, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c27393, SUC_lexicon_tags_12 },
    { 0x9a027394, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf562739c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a42739e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64273a1, SUC_lexicon_tags_30 },
    { 0x480273a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd36273a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe34273a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce273a9, SUC_lexicon_tags_0 },
    { 0xd3a273aa, SUC_lexicon_tags_39 },
    { 0xdce273aa, SUC_lexicon_tags_25 },
    { 0x5ce273ac, SUC_lexicon_tags_17 },
    { 0xa42273ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x148273b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a273b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb34273b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1e273b9, SUC_lexicon_tags_0 },
    { 0xa68273b9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c273c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec273c4, SUC_lexicon_tags_27 },
    { 0xfc8273c5, SUC_lexicon_tags_12 },
    { 0xece273c6, SUC_lexicon_tags_12 },
    { 0x922273c7, SUC_lexicon_tags_19 },
    { 0x4e6273c6, SUC_lexicon_tags_17 },
    { 0x2ec273c7, SUC_lexicon_tags_55 },
    { 0xbfe273ca, SUC_lexicon_tags_37 },
    { 0x8dc273cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380273cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4273d8, SUC_lexicon_tags_18 },
    { 0xcd6273d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6a273db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46273e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a2273e7, SUC_lexicon_tags_25 },
    { 0x352273e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64273ec, SUC_lexicon_tags_6 },
    { 0x752273ed, SUC_lexicon_tags_43 },
    { 0xe18273ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71c273f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa24273f2, SUC_lexicon_tags_25 },
    { 0xeb2273f3, SUC_lexicon_tags_8 },
    { 0x584273f4, SUC_lexicon_tags_16 },
    { 0xdc2273f4, SUC_lexicon_tags_11 },
    { 0xeb8273f6, SUC_lexicon_tags_9 },
    { 0xf70273f6, SUC_lexicon_tags_17 },
    { 0x2a6273f3, SUC_lexicon_tags_17 },
    { 0x5d2273f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136273fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x280273ff, SUC_lexicon_tags_11 },
    { 0xa1627400, SUC_lexicon_tags_2 },
    { 0x4ac27401, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b427403, SUC_lexicon_tags_60 },
    { 0x97427404, SUC_lexicon_tags_15 },
    { 0xc8227405, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f227408, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c62740b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba62740d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5427412, SUC_lexicon_tags_12 },
    { 0x97227412, SUC_lexicon_tags_17 },
    { 0x45e27412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4227418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7427421, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x97027423, SUC_lexicon_tags_14 },
    { 0xf6627424, SUC_lexicon_tags_37 },
    { 0xe5627424, SUC_lexicon_tags_17 },
    { 0x83227426, SUC_lexicon_tags_38 },
    { 0x8f427423, SUC_lexicon_tags_37 },
    { 0x5027428, SUC_lexicon_tags_12 },
    { 0xe8627426, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf002742b, SUC_lexicon_tags_16 },
    { 0x662742c, SUC_lexicon_tags_21 },
    { 0x9522742c, SUC_lexicon_tags_11 },
    { 0xada2742e, SUC_lexicon_tags_29 },
    { 0x3c42742d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff027437, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8ce27439, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c42743e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87c27440, SUC_lexicon_tags_1261 },
    { 0xa5627441, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b627443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe027446, SUC_lexicon_tags_12 },
    { 0x3da27447, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdcc27449, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe542744b, SUC_lexicon_tags_14 },
    { 0x3f42744c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63627451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20227455, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa2745c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e027461, SUC_lexicon_tags_29 },
    { 0x26e27462, SUC_lexicon_tags_2 },
    { 0xa8027461, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e627466, SUC_lexicon_tags_29 },
    { 0x9fe27467, SUC_lexicon_tags_517 },
    { 0x81227467, SUC_lexicon_tags_37 },
    { 0x13027469, SUC_lexicon_tags_45 },
    { 0xfdc27468, SUC_lexicon_tags_17 },
    { 0x67a27467, SUC_lexicon_tags_38 },
    { 0x8c42746c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e27470, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c027472, SUC_lexicon_tags_6 },
    { 0xfa227472, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdee27475, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x81e27477, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaea27479, SUC_lexicon_tags_25 },
    { 0x482747a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2122747c, SUC_lexicon_tags_35 },
    { 0x1842747c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e227480, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x55227483, SUC_lexicon_tags_0 },
    { 0x52c27484, SUC_lexicon_tags_14 },
    { 0xe2a27484, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee22748d, SUC_lexicon_tags_49 },
    { 0xd462748e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee27492, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd027495, SUC_lexicon_tags_9 },
    { 0xe1227496, SUC_lexicon_tags_7 },
    { 0x72c27495, SUC_lexicon_tags_15 },
    { 0x59227495, SUC_lexicon_tags_37 },
    { 0x72427499, SUC_lexicon_tags_37 },
    { 0x97a27499, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e22749c, SUC_lexicon_tags_11 },
    { 0x7b82749c, SUC_lexicon_tags_37 },
    { 0x3642749c, SUC_lexicon_tags_43 },
    { 0x3fe2749f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa4e274a1, SUC_lexicon_tags_60 },
    { 0xd92274a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x604274a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6274a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x752274ab, SUC_lexicon_tags_29 },
    { 0x264274ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78274b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc274b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354274ba, SUC_lexicon_tags_11 },
    { 0x8f2274ba, SUC_lexicon_tags_23 },
    { 0xeac274bc, SUC_lexicon_tags_65 },
    { 0xe7c274bb, SUC_lexicon_tags_2 },
    { 0xb40274be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c274c1, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x340274c3, SUC_lexicon_tags_429 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162274c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4274ca, SUC_lexicon_tags_87 },
    { 0x486274cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672274d0, SUC_lexicon_tags_6 },
    { 0x334274d1, SUC_lexicon_tags_12 },
    { 0x480274d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73e274d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4274d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08274dc, SUC_lexicon_tags_15 },
    { 0x452274dd, SUC_lexicon_tags_84 },
    { 0xce6274de, SUC_lexicon_tags_43 },
    { 0xc60274df, SUC_lexicon_tags_15 },
    { 0x618274dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e274e6, SUC_lexicon_tags_8 },
    { 0xf0c274e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c274eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6274ef, SUC_lexicon_tags_2 },
    { 0x16274f0, SUC_lexicon_tags_11 },
    { 0xe1e274f0, SUC_lexicon_tags_63 },
    { 0xea0274f2, SUC_lexicon_tags_33 },
    { 0xae4274f3, SUC_lexicon_tags_62 },
    { 0x374274f3, SUC_lexicon_tags_46 },
    { 0x200274f2, SUC_lexicon_tags_17 },
    { 0xdc8274f6, SUC_lexicon_tags_49 },
    { 0x10274f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bc274f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace274ff, SUC_lexicon_tags_47 },
    { 0x7ba27500, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67027506, SUC_lexicon_tags_1 },
    { 0xdf027506, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e2750b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95a2750d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x97627510, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe27513, SUC_lexicon_tags_12 },
    { 0xe9c27514, SUC_lexicon_tags_17 },
    { 0x9e627514, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x87827518, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b22751a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7ea2751c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee27523, SUC_lexicon_tags_37 },
    { 0x23e27524, SUC_lexicon_tags_8 },
    { 0x51a27525, SUC_lexicon_tags_6 },
    { 0x49c27524, SUC_lexicon_tags_25 },
    { 0x8ec27527, SUC_lexicon_tags_17 },
    { 0xa5827528, SUC_lexicon_tags_2 },
    { 0xb2a27529, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc02752d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b027532, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ae27534, SUC_lexicon_tags_37 },
    { 0x82e27535, SUC_lexicon_tags_9 },
    { 0x53427535, SUC_lexicon_tags_15 },
    { 0x84e27535, SUC_lexicon_tags_15 },
    { 0x4627538, SUC_lexicon_tags_17 },
    { 0x6627538, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8827540, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc027545, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb427547, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba627549, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e627551, SUC_lexicon_tags_30 },
    { 0xaae27551, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8427555, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc827559, SUC_lexicon_tags_37 },
    { 0xfb827559, SUC_lexicon_tags_23 },
    { 0x3882755b, SUC_lexicon_tags_37 },
    { 0x3522755b, SUC_lexicon_tags_57 },
    { 0x5f22755a, SUC_lexicon_tags_1 },
    { 0x4982755c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce27562, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xedc27564, SUC_lexicon_tags_29 },
    { 0xdb427565, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3827568, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1ca2756a, SUC_lexicon_tags_33 },
    { 0x3922756a, SUC_lexicon_tags_38 },
    { 0xac22756c, SUC_lexicon_tags_14 },
    { 0x5e82756b, SUC_lexicon_tags_47 },
    { 0x4322756d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8cc27570, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e27573, SUC_lexicon_tags_7 },
    { 0x37e27573, SUC_lexicon_tags_40 },
    { 0xe7627574, SUC_lexicon_tags_6 },
    { 0xdd227576, SUC_lexicon_tags_31 },
    { 0x55a27576, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4342757a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a2757d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40227583, SUC_lexicon_tags_37 },
    { 0x5f427584, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d027587, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5482758c, SUC_lexicon_tags_179 },
    { 0xed22758c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1762758f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45627594, SUC_lexicon_tags_1 },
    { 0x8ac27595, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0427598, SUC_lexicon_tags_14 },
    { 0x98e27598, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac2759f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a275aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c275b1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc8275b3, SUC_lexicon_tags_84 },
    { 0x82a275b4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x93c275b6, SUC_lexicon_tags_7 },
    { 0xa38275b7, SUC_lexicon_tags_14 },
    { 0x416275b7, SUC_lexicon_tags_37 },
    { 0xafe275b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274275bf, SUC_lexicon_tags_7 },
    { 0x66e275c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd32275c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0275c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f0275c9, SUC_lexicon_tags_11 },
    { 0x5a8275c9, SUC_lexicon_tags_25 },
    { 0x96275cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd3a275cd, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xd06275cf, SUC_lexicon_tags_17 },
    { 0x8bc275d0, SUC_lexicon_tags_67 },
    { 0x1be275d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0275d4, SUC_lexicon_tags_21 },
    { 0x5c8275d5, SUC_lexicon_tags_43 },
    { 0x81a275d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb1e275d8, SUC_lexicon_tags_63 },
    { 0xef6275d9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0275df, SUC_lexicon_tags_11 },
    { 0xc0275e0, SUC_lexicon_tags_17 },
    { 0x498275e1, SUC_lexicon_tags_14 },
    { 0xc60275e1, SUC_lexicon_tags_85 },
    { 0xa8c275df, SUC_lexicon_tags_15 },
    { 0x6b2275e1, SUC_lexicon_tags_38 },
    { 0x152275e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x48e275e7, SUC_lexicon_tags_47 },
    { 0x64e275e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32a275ea, SUC_lexicon_tags_34 },
    { 0x67c275ea, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba275f4, SUC_lexicon_tags_32 },
    { 0x5f2275f5, SUC_lexicon_tags_45 },
    { 0xa1c275f6, SUC_lexicon_tags_45 },
    { 0x342275f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e275fe, SUC_lexicon_tags_1 },
    { 0x512275ff, SUC_lexicon_tags_46 },
    { 0x21027600, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x84027602, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb827604, SUC_lexicon_tags_201 },
    { 0x9a227605, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf227609, SUC_lexicon_tags_23 },
    { 0x3982760a, SUC_lexicon_tags_17 },
    { 0x2c62760a, SUC_lexicon_tags_13 },
    { 0x9cc2760c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6827612, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f227618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce2761b, SUC_lexicon_tags_27 },
    { 0x3e62761c, SUC_lexicon_tags_82 },
    { 0xc662761d, SUC_lexicon_tags_104 },
    { 0x2a42761e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3627623, SUC_lexicon_tags_8 },
    { 0x3c227624, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2422762b, SUC_lexicon_tags_37 },
    { 0xd462762c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c427632, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e27635, SUC_lexicon_tags_23 },
    { 0x5027635, SUC_lexicon_tags_12 },
    { 0x4b427637, SUC_lexicon_tags_17 },
    { 0x79027635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a02763a, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xea82763c, SUC_lexicon_tags_34 },
    { 0x8dc2763d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf162763f, SUC_lexicon_tags_23 },
    { 0x91e2763f, SUC_lexicon_tags_1 },
    { 0x2827641, SUC_lexicon_tags_37 },
    { 0x37827642, SUC_lexicon_tags_14 },
    { 0x25a27640, SUC_lexicon_tags_32 },
    { 0xd302763f, SUC_lexicon_tags_17 },
    { 0x8a27645, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc227647, SUC_lexicon_tags_49 },
    { 0x83027648, SUC_lexicon_tags_27 },
    { 0x15427648, SUC_lexicon_tags_60 },
    { 0xe422764a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3462764d, SUC_lexicon_tags_14 },
    { 0x3ae2764e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x59827650, SUC_lexicon_tags_19 },
    { 0xa5827651, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe8827653, SUC_lexicon_tags_20 },
    { 0x29a27654, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8427659, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a42765c, SUC_lexicon_tags_49 },
    { 0x7f02765d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc827662, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5627667, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7962766b, SUC_lexicon_tags_30 },
    { 0xda2766c, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83827670, SUC_lexicon_tags_63 },
    { 0x7ce27670, SUC_lexicon_tags_80 },
    { 0x63027672, SUC_lexicon_tags_29 },
    { 0x5da27672, SUC_lexicon_tags_49 },
    { 0x60227674, SUC_lexicon_tags_25 },
    { 0xaea27675, SUC_lexicon_tags_25 },
    { 0xa427675, SUC_lexicon_tags_23 },
    { 0xbb427674, SUC_lexicon_tags_2 },
    { 0xe4e27675, SUC_lexicon_tags_2 },
    { 0x78427679, SUC_lexicon_tags_29 },
    { 0xe982767a, SUC_lexicon_tags_45 },
    { 0x63627679, SUC_lexicon_tags_35 },
    { 0xb362767b, SUC_lexicon_tags_47 },
    { 0x2002767a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80227680, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5427682, SUC_lexicon_tags_37 },
    { 0x39c27683, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6827687, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662768e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62c27690, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8027695, SUC_lexicon_tags_43 },
    { 0x7f427696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89a27698, SUC_lexicon_tags_38 },
    { 0xec427698, SUC_lexicon_tags_15 },
    { 0x9e42769a, SUC_lexicon_tags_12 },
    { 0x4ea2769a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8276a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f8276a2, SUC_lexicon_tags_38 },
    { 0xa0276a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcec276a5, SUC_lexicon_tags_60 },
    { 0x554276a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80276ab, SUC_lexicon_tags_9 },
    { 0xbae276ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdbe276ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c276b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf6276b3, SUC_lexicon_tags_19 },
    { 0xb08276b3, SUC_lexicon_tags_17 },
    { 0x502276b5, SUC_lexicon_tags_37 },
    { 0x620276b6, SUC_lexicon_tags_2 },
    { 0x762276b6, SUC_lexicon_tags_17 },
    { 0xdbe276b7, SUC_lexicon_tags_37 },
    { 0x6e276b9, SUC_lexicon_tags_14 },
    { 0xa30276b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4276bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x964276bf, SUC_lexicon_tags_2 },
    { 0x220276bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82276c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x59a276c7, SUC_lexicon_tags_17 },
    { 0x10e276c8, SUC_lexicon_tags_12 },
    { 0x6b6276c7, SUC_lexicon_tags_33 },
    { 0xbbc276c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa10276cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6276cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb36276d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c276d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf64276d8, SUC_lexicon_tags_25 },
    { 0x5c0276d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2276dc, SUC_lexicon_tags_12 },
    { 0xe3a276dc, SUC_lexicon_tags_38 },
    { 0x88c276dd, SUC_lexicon_tags_10 },
    { 0x624276df, SUC_lexicon_tags_11 },
    { 0x9e4276e0, SUC_lexicon_tags_11 },
    { 0x922276e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0276e4, SUC_lexicon_tags_77 },
    { 0x52276e5, SUC_lexicon_tags_37 },
    { 0xc2e276e6, SUC_lexicon_tags_27 },
    { 0xe32276e4, SUC_lexicon_tags_3 },
    { 0x850276e6, SUC_lexicon_tags_25 },
    { 0x88a276e9, SUC_lexicon_tags_17 },
    { 0xc1e276ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c276ed, SUC_lexicon_tags_45 },
    { 0xc2a276ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a276f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72276f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x222276fa, SUC_lexicon_tags_8 },
    { 0x1ec276fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3a6276fd, SUC_lexicon_tags_8 },
    { 0xad4276fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa627701, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3827708, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc2770e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9ee27710, SUC_lexicon_tags_77 },
    { 0xffe27710, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc827716, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb942771a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9262771c, SUC_lexicon_tags_30 },
    { 0xafc2771d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea27720, SUC_lexicon_tags_3 },
    { 0x22827721, SUC_lexicon_tags_55 },
    { 0xbb627722, SUC_lexicon_tags_14 },
    { 0xb4e27723, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40427728, SUC_lexicon_tags_11 },
    { 0x93827729, SUC_lexicon_tags_11 },
    { 0x7662772a, SUC_lexicon_tags_2 },
    { 0xc842772b, SUC_lexicon_tags_27 },
    { 0x6542772c, SUC_lexicon_tags_17 },
    { 0x33e2772d, SUC_lexicon_tags_56 },
    { 0xabe2772e, SUC_lexicon_tags_301 },
    { 0xe262772f, SUC_lexicon_tags_37 },
    { 0xec027728, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c27735, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e27739, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf322773c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5427743, SUC_lexicon_tags_25 },
    { 0xcba27743, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18027746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd062774c, SUC_lexicon_tags_6 },
    { 0xf482774d, SUC_lexicon_tags_25 },
    { 0x91e2774d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x80227750, SUC_lexicon_tags_49 },
    { 0xc7e27751, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a027754, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4227758, SUC_lexicon_tags_49 },
    { 0x1627759, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c27761, SUC_lexicon_tags_38 },
    { 0x6d027762, SUC_lexicon_tags_45 },
    { 0xa0c27762, SUC_lexicon_tags_11 },
    { 0x52827763, SUC_lexicon_tags_15 },
    { 0x53027765, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4627769, SUC_lexicon_tags_38 },
    { 0x95a2776a, SUC_lexicon_tags_47 },
    { 0x6402776b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46a2776d, SUC_lexicon_tags_12 },
    { 0x1b22776e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84a27770, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47c27772, SUC_lexicon_tags_12 },
    { 0xd7c27772, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a27779, SUC_lexicon_tags_0 },
    { 0x2c42777a, SUC_lexicon_tags_7 },
    { 0xd322777a, SUC_lexicon_tags_37 },
    { 0x5102777c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d427780, SUC_lexicon_tags_27 },
    { 0xb8827781, SUC_lexicon_tags_65 },
    { 0x28427781, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x84e27784, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1227789, SUC_lexicon_tags_19 },
    { 0xcee2778a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde227790, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcc27792, SUC_lexicon_tags_25 },
    { 0x41227792, SUC_lexicon_tags_46 },
    { 0xb4a27794, SUC_lexicon_tags_16 },
    { 0x2ea27794, SUC_lexicon_tags_31 },
    { 0x90427795, SUC_lexicon_tags_1 },
    { 0xb8027797, SUC_lexicon_tags_16 },
    { 0xcbc27798, SUC_lexicon_tags_12 },
    { 0x43e27799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd962779b, SUC_lexicon_tags_11 },
    { 0xf222779c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868277a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114277a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a277aa, SUC_lexicon_tags_12 },
    { 0xfc0277ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc277af, SUC_lexicon_tags_38 },
    { 0x78277b0, SUC_lexicon_tags_17 },
    { 0x830277b1, SUC_lexicon_tags_65 },
    { 0xfba277b1, SUC_lexicon_tags_15 },
    { 0xbe8277b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef6277b5, SUC_lexicon_tags_31 },
    { 0x3ee277b6, SUC_lexicon_tags_19 },
    { 0x672277b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2e277b9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c277bd, SUC_lexicon_tags_65 },
    { 0x5f8277be, SUC_lexicon_tags_49 },
    { 0xa84277bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44277c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x366277c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x654277cb, SUC_lexicon_tags_9 },
    { 0x7a0277cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2277d0, SUC_lexicon_tags_11 },
    { 0x3a277d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e277d7, SUC_lexicon_tags_37 },
    { 0x98a277d7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe5e277da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168277de, SUC_lexicon_tags_10 },
    { 0x898277df, SUC_lexicon_tags_34 },
    { 0xfac277e0, SUC_lexicon_tags_1 },
    { 0xe8c277e1, SUC_lexicon_tags_37 },
    { 0x7a277e2, SUC_lexicon_tags_63 },
    { 0x4d6277e3, SUC_lexicon_tags_49 },
    { 0x1a0277e0, SUC_lexicon_tags_32 },
    { 0x7a6277e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8277eb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936277f2, SUC_lexicon_tags_45 },
    { 0x4f0277f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4277f7, SUC_lexicon_tags_47 },
    { 0xec6277f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x622277fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c277ff, SUC_lexicon_tags_2 },
    { 0x7b4277ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c27803, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5627809, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x622780c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2622780f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6627812, SUC_lexicon_tags_7 },
    { 0x9a827813, SUC_lexicon_tags_8 },
    { 0xfa827813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba827817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa302781a, SUC_lexicon_tags_6 },
    { 0x98c2781b, SUC_lexicon_tags_7 },
    { 0x2ee2781c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f62781e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e27823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e27829, SUC_lexicon_tags_11 },
    { 0xf0c27829, SUC_lexicon_tags_25 },
    { 0x5582782b, SUC_lexicon_tags_11 },
    { 0x5542782c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc27832, SUC_lexicon_tags_43 },
    { 0x3f027832, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0xab827835, SUC_lexicon_tags_39 },
    { 0xbe427836, SUC_lexicon_tags_11 },
    { 0xe7627835, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xafc27839, SUC_lexicon_tags_38 },
    { 0x58427839, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4502783e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3c227840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94027847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c27851, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7227854, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a27858, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2da2785a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9722785c, SUC_lexicon_tags_7 },
    { 0x8f22785c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x95827860, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c2786e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf227871, SUC_lexicon_tags_11 },
    { 0x41227871, SUC_lexicon_tags_11 },
    { 0x2c27873, SUC_lexicon_tags_37 },
    { 0x2a27873, SUC_lexicon_tags_29 },
    { 0x2d027871, SUC_lexicon_tags_12 },
    { 0x49227876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75027879, SUC_lexicon_tags_16 },
    { 0xbb82787a, SUC_lexicon_tags_15 },
    { 0xa902787a, SUC_lexicon_tags_1 },
    { 0xb7a2787b, SUC_lexicon_tags_37 },
    { 0x3062787b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d027881, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c27888, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4262788a, SUC_lexicon_tags_37 },
    { 0xe62788b, SUC_lexicon_tags_43 },
    { 0xef02788a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd982788f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c027892, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9a227894, SUC_lexicon_tags_37 },
    { 0x9b227895, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e627899, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1d22789b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc762789d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02278a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554278a5, SUC_lexicon_tags_11 },
    { 0x7ee278a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432278ac, SUC_lexicon_tags_33 },
    { 0xe5a278ad, SUC_lexicon_tags_37 },
    { 0xf74278ae, SUC_lexicon_tags_98 },
    { 0x3d4278af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70278b6, SUC_lexicon_tags_26 },
    { 0xc96278b7, SUC_lexicon_tags_38 },
    { 0x322278b8, SUC_lexicon_tags_900 },
    { 0x20c278b9, SUC_lexicon_tags_1 },
    { 0x97a278ba, SUC_lexicon_tags_31 },
    { 0xdb0278bb, SUC_lexicon_tags_27 },
    { 0x818278bc, SUC_lexicon_tags_274 },
    { 0x1d4278bd, SUC_lexicon_tags_26 },
    { 0xea6278be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x500278c1, SUC_lexicon_tags_103 },
    { 0x69e278c2, SUC_lexicon_tags_7 },
    { 0x480278c2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772278c7, SUC_lexicon_tags_37 },
    { 0x272278c8, SUC_lexicon_tags_17 },
    { 0xc66278c7, SUC_lexicon_tags_1 },
    { 0x310278c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222278ce, SUC_lexicon_tags_77 },
    { 0xe02278cf, SUC_lexicon_tags_0 },
    { 0x450278d0, SUC_lexicon_tags_15 },
    { 0xe8278d0, SUC_lexicon_tags_37 },
    { 0xb36278d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6278dd, SUC_lexicon_tags_28 },
    { 0xf16278de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x336278e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc278e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338278ef, SUC_lexicon_tags_35 },
    { 0x70278ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe14278f2, SUC_lexicon_tags_33 },
    { 0x680278f2, SUC_lexicon_tags_23 },
    { 0x18a278f2, SUC_lexicon_tags_84 },
    { 0xefe278f5, SUC_lexicon_tags_12 },
    { 0xf80278f6, SUC_lexicon_tags_87 },
    { 0x752278f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498278ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdf827901, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x62627903, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe827905, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x44427908, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee2790c, SUC_lexicon_tags_28 },
    { 0x7842790c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe27910, SUC_lexicon_tags_7 },
    { 0xb4027910, SUC_lexicon_tags_37 },
    { 0xfd827912, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed227916, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xa6827918, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e2791c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x29c2791e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6027920, SUC_lexicon_tags_19 },
    { 0x3c627921, SUC_lexicon_tags_12 },
    { 0x7ba27922, SUC_lexicon_tags_28 },
    { 0xc9827923, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x25027925, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c27929, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb562792b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7062792d, SUC_lexicon_tags_2 },
    { 0x2782792e, SUC_lexicon_tags_53 },
    { 0x31e2792f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85427932, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0827935, SUC_lexicon_tags_31 },
    { 0xd6e27935, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a2793a, SUC_lexicon_tags_7 },
    { 0xed42793b, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe2793e, SUC_lexicon_tags_31 },
    { 0x8322793f, SUC_lexicon_tags_29 },
    { 0x7a27940, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41827944, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a27949, SUC_lexicon_tags_33 },
    { 0x322794a, SUC_lexicon_tags_27 },
    { 0x2d42794b, SUC_lexicon_tags_25 },
    { 0xea62794a, SUC_lexicon_tags_0 },
    { 0xc342794d, SUC_lexicon_tags_48 },
    { 0x21e2794e, SUC_lexicon_tags_33 },
    { 0xf9c2794f, SUC_lexicon_tags_47 },
    { 0xdc027950, SUC_lexicon_tags_25 },
    { 0x58a27951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5827953, SUC_lexicon_tags_0 },
    { 0x5de27954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca227957, SUC_lexicon_tags_37 },
    { 0x4c227958, SUC_lexicon_tags_17 },
    { 0x18227959, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf102795c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be27962, SUC_lexicon_tags_56 },
    { 0xe3027963, SUC_lexicon_tags_7 },
    { 0x75e27964, SUC_lexicon_tags_10 },
    { 0xb8a27964, SUC_lexicon_tags_37 },
    { 0x7627962, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462796b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaf22796d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d62796f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c827973, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19027975, SUC_lexicon_tags_8 },
    { 0x47027976, SUC_lexicon_tags_316 },
    { 0, NULL },
    { 0x14a27978, SUC_lexicon_tags_14 },
    { 0x98c27978, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1522797c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9427980, SUC_lexicon_tags_18 },
    { 0xec427980, SUC_lexicon_tags_0 },
    { 0xf4827981, SUC_lexicon_tags_49 },
    { 0x83427983, SUC_lexicon_tags_45 },
    { 0xe8027982, SUC_lexicon_tags_38 },
    { 0xfb227982, SUC_lexicon_tags_37 },
    { 0x31427986, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2462798a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e2798e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a27991, SUC_lexicon_tags_10 },
    { 0x26827991, SUC_lexicon_tags_7 },
    { 0x1fe27993, SUC_lexicon_tags_25 },
    { 0x32627994, SUC_lexicon_tags_2 },
    { 0xf2827994, SUC_lexicon_tags_12 },
    { 0xd0027996, SUC_lexicon_tags_27 },
    { 0xcdc27997, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36627999, SUC_lexicon_tags_7 },
    { 0xc6427999, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582799e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xebc279a0, SUC_lexicon_tags_17 },
    { 0x298279a0, SUC_lexicon_tags_31 },
    { 0xd5e279a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad8279a4, SUC_lexicon_tags_28 },
    { 0x2c6279a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238279aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24279ac, SUC_lexicon_tags_37 },
    { 0x342279ad, SUC_lexicon_tags_311 },
    { 0x2c6279ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ec279b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640279b7, SUC_lexicon_tags_30 },
    { 0x2f8279b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4279bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34279c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4279c3, SUC_lexicon_tags_2 },
    { 0x15a279c4, SUC_lexicon_tags_15 },
    { 0x6ba279c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc279ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca279d0, SUC_lexicon_tags_19 },
    { 0xcac279d1, SUC_lexicon_tags_79 },
    { 0xe52279d1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x586279d4, SUC_lexicon_tags_47 },
    { 0x67c279d4, SUC_lexicon_tags_5 },
    { 0x876279d5, SUC_lexicon_tags_35 },
    { 0x266279d6, SUC_lexicon_tags_2 },
    { 0x650279d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6279db, SUC_lexicon_tags_37 },
    { 0xa24279dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c279e1, SUC_lexicon_tags_25 },
    { 0xa9e279e2, SUC_lexicon_tags_11 },
    { 0xbe8279e3, SUC_lexicon_tags_140 },
    { 0x816279e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2279e7, SUC_lexicon_tags_19 },
    { 0xa6a279e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538279ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2279ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6279f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5aa279f6, SUC_lexicon_tags_2 },
    { 0x4ba279f6, SUC_lexicon_tags_27 },
    { 0x9a4279f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x808279fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212279fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c27a03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a27a07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca027a09, SUC_lexicon_tags_63 },
    { 0xb7a27a0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f027a0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58027a0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a27a12, SUC_lexicon_tags_68 },
    { 0x12627a13, SUC_lexicon_tags_27 },
    { 0x73a27a14, SUC_lexicon_tags_17 },
    { 0xa27a15, SUC_lexicon_tags_17 },
    { 0x4f627a14, SUC_lexicon_tags_69 },
    { 0xb7627a16, SUC_lexicon_tags_32 },
    { 0xb1227a18, SUC_lexicon_tags_17 },
    { 0x82227a19, SUC_lexicon_tags_37 },
    { 0x56027a19, SUC_lexicon_tags_17 },
    { 0xe3e27a18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa27a21, SUC_lexicon_tags_65 },
    { 0x54827a21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11827a2a, SUC_lexicon_tags_9 },
    { 0x79e27a2b, SUC_lexicon_tags_15 },
    { 0x9b827a2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde827a2e, SUC_lexicon_tags_9 },
    { 0x80627a2f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xad827a32, SUC_lexicon_tags_25 },
    { 0x56827a32, SUC_lexicon_tags_17 },
    { 0x7a27a34, SUC_lexicon_tags_11 },
    { 0x41627a35, SUC_lexicon_tags_1 },
    { 0x11a27a35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2e227a38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e27a3e, SUC_lexicon_tags_25 },
    { 0xeac27a3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x58e27a41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79427a43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a027a45, SUC_lexicon_tags_17 },
    { 0xad227a46, SUC_lexicon_tags_17 },
    { 0x89227a47, SUC_lexicon_tags_39 },
    { 0x80227a48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31227a50, SUC_lexicon_tags_27 },
    { 0xcbe27a50, SUC_lexicon_tags_46 },
    { 0x16a27a50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae27a54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c27a5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf827a5c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c27a66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd427a68, SUC_lexicon_tags_30 },
    { 0x50827a69, SUC_lexicon_tags_253 },
    { 0x25227a6a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3227a6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c27a76, SUC_lexicon_tags_7 },
    { 0x5b427a77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33027a79, SUC_lexicon_tags_11 },
    { 0xee227a7a, SUC_lexicon_tags_7 },
    { 0x8f227a7b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda027a82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e27a8c, SUC_lexicon_tags_19 },
    { 0xbce27a8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca027a8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3027a91, SUC_lexicon_tags_43 },
    { 0xcda27a92, SUC_lexicon_tags_7 },
    { 0x10c27a93, SUC_lexicon_tags_31 },
    { 0xc3827a93, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c227a98, SUC_lexicon_tags_9 },
    { 0xb9c27a99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0427aa3, SUC_lexicon_tags_7 },
    { 0xc8827aa3, SUC_lexicon_tags_15 },
    { 0xc2827aa5, SUC_lexicon_tags_236 },
    { 0x21627aa4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a27aa9, SUC_lexicon_tags_48 },
    { 0xebc27aaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54827aad, SUC_lexicon_tags_38 },
    { 0xf5227aad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94827ab3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8627ab7, SUC_lexicon_tags_1 },
    { 0xac827ab7, SUC_lexicon_tags_37 },
    { 0xf0c27ab8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2427abe, SUC_lexicon_tags_17 },
    { 0xb6027abf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6e27ac1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x99027ac4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6ac27ac6, SUC_lexicon_tags_15 },
    { 0x4c027ac7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5827acb, SUC_lexicon_tags_5 },
    { 0x6da27acc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3fc27ace, SUC_lexicon_tags_45 },
    { 0xc4e27acf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c627ad5, SUC_lexicon_tags_246 },
    { 0xb4427ad6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7627ada, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c227ae4, SUC_lexicon_tags_79 },
    { 0xf8827ae4, SUC_lexicon_tags_15 },
    { 0x96a27ae4, SUC_lexicon_tags_7 },
    { 0xbb227ae7, SUC_lexicon_tags_2 },
    { 0x5cc27ae4, SUC_lexicon_tags_30 },
    { 0xe9027ae6, SUC_lexicon_tags_17 },
    { 0xffc27ae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e427aee, SUC_lexicon_tags_31 },
    { 0x57027aef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf9a27af1, SUC_lexicon_tags_27 },
    { 0x1a427af2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a27af8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73827afc, SUC_lexicon_tags_21 },
    { 0x35227afc, SUC_lexicon_tags_2 },
    { 0x8fc27afe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7627b00, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f027b05, SUC_lexicon_tags_275 },
    { 0x2ac27b06, SUC_lexicon_tags_14 },
    { 0x5a827b07, SUC_lexicon_tags_25 },
    { 0x9c227b07, SUC_lexicon_tags_17 },
    { 0x3cc27b09, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x62827b0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x53027b0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a27b21, SUC_lexicon_tags_43 },
    { 0xa6227b22, SUC_lexicon_tags_7 },
    { 0x8b227b23, SUC_lexicon_tags_39 },
    { 0x63827b24, SUC_lexicon_tags_37 },
    { 0x23c27b25, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e27b28, SUC_lexicon_tags_26 },
    { 0xe0027b29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c227b2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94027b31, SUC_lexicon_tags_8 },
    { 0x51427b31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7627b34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5827b38, SUC_lexicon_tags_2 },
    { 0x43a27b38, SUC_lexicon_tags_19 },
    { 0xb5827b3a, SUC_lexicon_tags_44 },
    { 0xae27b3b, SUC_lexicon_tags_48 },
    { 0xb9027b38, SUC_lexicon_tags_65 },
    { 0xf1e27b3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x16227b3f, SUC_lexicon_tags_66 },
    { 0x5ce27b3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23827b43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xee627b45, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc27b48, SUC_lexicon_tags_17 },
    { 0xb0627b49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a27b4c, SUC_lexicon_tags_47 },
    { 0xc3c27b4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8da27b4f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f027b54, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x7d227b56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a27b59, SUC_lexicon_tags_25 },
    { 0xd6827b59, SUC_lexicon_tags_27 },
    { 0x51c27b5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5627b5f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49627b63, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a27b66, SUC_lexicon_tags_72 },
    { 0xb9c27b66, SUC_lexicon_tags_1 },
    { 0x21a27b68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef827b6b, SUC_lexicon_tags_14 },
    { 0x23e27b6c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8227b6e, SUC_lexicon_tags_38 },
    { 0x4e627b6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8027b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8427b79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa227b7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32c27b7d, SUC_lexicon_tags_33 },
    { 0x56a27b7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12027b81, SUC_lexicon_tags_7 },
    { 0xa6827b82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc27b85, SUC_lexicon_tags_23 },
    { 0x7aa27b86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a27b8d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac27b95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3027b97, SUC_lexicon_tags_37 },
    { 0x19627b98, SUC_lexicon_tags_16 },
    { 0x91e27b97, SUC_lexicon_tags_38 },
    { 0xcc27b9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31427b9f, SUC_lexicon_tags_45 },
    { 0x5bc27b9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25827ba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8827ba6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda627ba9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5027bb8, SUC_lexicon_tags_28 },
    { 0xbba27bb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1227bbc, SUC_lexicon_tags_11 },
    { 0x66227bbd, SUC_lexicon_tags_37 },
    { 0x69c27bbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47427bc1, SUC_lexicon_tags_33 },
    { 0x3ce27bc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6227bc6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f227bc9, SUC_lexicon_tags_25 },
    { 0x5e227bca, SUC_lexicon_tags_17 },
    { 0xabc27bcb, SUC_lexicon_tags_80 },
    { 0x56c27bcc, SUC_lexicon_tags_45 },
    { 0xd6e27bcd, SUC_lexicon_tags_47 },
    { 0x9e627bcc, SUC_lexicon_tags_1 },
    { 0x92027bcf, SUC_lexicon_tags_15 },
    { 0x4a27bcf, SUC_lexicon_tags_15 },
    { 0xf6227bcd, SUC_lexicon_tags_15 },
    { 0xdde27bd2, SUC_lexicon_tags_127 },
    { 0x1be27bd1, SUC_lexicon_tags_66 },
    { 0xfde27bd1, SUC_lexicon_tags_46 },
    { 0x37227bcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66027bde, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf8a27be0, SUC_lexicon_tags_7 },
    { 0x27027be1, SUC_lexicon_tags_12 },
    { 0x63e27be2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86427be7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ba27be9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42227bee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90827bf6, SUC_lexicon_tags_79 },
    { 0x56827bf6, SUC_lexicon_tags_7 },
    { 0x5d427bf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x35627bfa, SUC_lexicon_tags_26 },
    { 0xe5627bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0627c03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4027c05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43827c0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee627c0d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b027c11, SUC_lexicon_tags_11 },
    { 0xe7a27c12, SUC_lexicon_tags_3 },
    { 0x83627c13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a227c15, SUC_lexicon_tags_25 },
    { 0xc6e27c15, SUC_lexicon_tags_45 },
    { 0xbb227c17, SUC_lexicon_tags_60 },
    { 0xba27c18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5ac27c1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea827c1f, SUC_lexicon_tags_50 },
    { 0xf5627c20, SUC_lexicon_tags_12 },
    { 0x78427c20, SUC_lexicon_tags_25 },
    { 0x9627c22, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e27c26, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6e227c28, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9027c30, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf8827c32, SUC_lexicon_tags_1 },
    { 0x4c427c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81427c36, SUC_lexicon_tags_25 },
    { 0xf5027c37, SUC_lexicon_tags_56 },
    { 0x7de27c38, SUC_lexicon_tags_25 },
    { 0xa4627c38, SUC_lexicon_tags_25 },
    { 0xcb227c3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc27c3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4227c45, SUC_lexicon_tags_9 },
    { 0xb7227c45, SUC_lexicon_tags_15 },
    { 0xa6427c47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce27c4c, SUC_lexicon_tags_0 },
    { 0x5b227c4c, SUC_lexicon_tags_47 },
    { 0xcb027c4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x38427c50, SUC_lexicon_tags_31 },
    { 0xf427c50, SUC_lexicon_tags_38 },
    { 0x83a27c52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c27c55, SUC_lexicon_tags_7 },
    { 0x4a427c56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc27c59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea027c5f, SUC_lexicon_tags_17 },
    { 0x22627c60, SUC_lexicon_tags_37 },
    { 0x45827c61, SUC_lexicon_tags_9 },
    { 0x90427c62, SUC_lexicon_tags_14 },
    { 0xc8c27c62, SUC_lexicon_tags_17 },
    { 0x93e27c61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3a27c66, SUC_lexicon_tags_33 },
    { 0xaee27c66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a27c6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59027c6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x74c27c6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93e27c71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3427c7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce27c7f, SUC_lexicon_tags_9 },
    { 0x6c227c7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8c27c82, SUC_lexicon_tags_15 },
    { 0x96c27c82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43827c8a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x52a27c8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b827c8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec27c97, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a27ca0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44a27ca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c27ca5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e27ca8, SUC_lexicon_tags_60 },
    { 0xff027ca9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb227cae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4827cb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae27cb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf227cb6, SUC_lexicon_tags_7 },
    { 0x3b427cb7, SUC_lexicon_tags_9 },
    { 0x17227cb8, SUC_lexicon_tags_17 },
    { 0x13e27cb9, SUC_lexicon_tags_17 },
    { 0xb5827cb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5627cbf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e27cc3, SUC_lexicon_tags_49 },
    { 0x19c27cc4, SUC_lexicon_tags_17 },
    { 0x36c27cc5, SUC_lexicon_tags_30 },
    { 0x25227cc6, SUC_lexicon_tags_38 },
    { 0x5b427cc6, SUC_lexicon_tags_45 },
    { 0x11227cc6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d427cce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ba27cd0, SUC_lexicon_tags_17 },
    { 0xa8227cd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c27cd6, SUC_lexicon_tags_23 },
    { 0x1be27cd6, SUC_lexicon_tags_14 },
    { 0xd3227cd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc2627cda, SUC_lexicon_tags_33 },
    { 0xbc427cdb, SUC_lexicon_tags_37 },
    { 0x53a27cdb, SUC_lexicon_tags_114 },
    { 0x57027cdd, SUC_lexicon_tags_27 },
    { 0x6ae27cdd, SUC_lexicon_tags_15 },
    { 0x12e27cdf, SUC_lexicon_tags_11 },
    { 0x1c027cdd, SUC_lexicon_tags_66 },
    { 0x3f427ce1, SUC_lexicon_tags_140 },
    { 0x76827ce1, SUC_lexicon_tags_17 },
    { 0x47c27ce3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a827ce7, SUC_lexicon_tags_23 },
    { 0xf2c27ce8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c27cec, SUC_lexicon_tags_16 },
    { 0xe6427ced, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xaa227cef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7627cf1, SUC_lexicon_tags_25 },
    { 0x10427cf1, SUC_lexicon_tags_74 },
    { 0x31e27cf3, SUC_lexicon_tags_11 },
    { 0x75827cf3, SUC_lexicon_tags_79 },
    { 0x46227cf3, SUC_lexicon_tags_11 },
    { 0xc427cf6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x77227cf8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9d827cfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a227cff, SUC_lexicon_tags_7 },
    { 0x7ae27d00, SUC_lexicon_tags_31 },
    { 0x3f027d00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe027d03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32627d05, SUC_lexicon_tags_17 },
    { 0x8827d05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51c27d08, SUC_lexicon_tags_15 },
    { 0x9c027d09, SUC_lexicon_tags_30 },
    { 0x82027d0a, SUC_lexicon_tags_295 },
    { 0x92827d0a, SUC_lexicon_tags_37 },
    { 0xcc227d09, SUC_lexicon_tags_15 },
    { 0x18e27d0c, SUC_lexicon_tags_19 },
    { 0x9ac27d0d, SUC_lexicon_tags_19 },
    { 0xa6227d0e, SUC_lexicon_tags_39 },
    { 0xc8227d09, SUC_lexicon_tags_49 },
    { 0x62427d0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5827d16, SUC_lexicon_tags_15 },
    { 0x9e827d16, SUC_lexicon_tags_39 },
    { 0x8a027d18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc27d1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25c27d20, SUC_lexicon_tags_31 },
    { 0x48227d21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x10827d23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38827d27, SUC_lexicon_tags_29 },
    { 0x2ba27d28, SUC_lexicon_tags_29 },
    { 0x3f627d29, SUC_lexicon_tags_15 },
    { 0x63a27d29, SUC_lexicon_tags_5 },
    { 0x1a027d29, SUC_lexicon_tags_0 },
    { 0x21a27d28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21227d31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e827d33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x19c27d35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd2c27d37, SUC_lexicon_tags_60 },
    { 0xad027d38, SUC_lexicon_tags_15 },
    { 0x5d827d39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x81827d3b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7fe27d3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e27d41, SUC_lexicon_tags_27 },
    { 0x2f427d41, SUC_lexicon_tags_17 },
    { 0x40427d42, SUC_lexicon_tags_13 },
    { 0x57027d44, SUC_lexicon_tags_61 },
    { 0x35627d45, SUC_lexicon_tags_68 },
    { 0xb6227d46, SUC_lexicon_tags_38 },
    { 0xc7827d44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ea27d49, SUC_lexicon_tags_17 },
    { 0x32827d4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3c27d4c, SUC_lexicon_tags_38 },
    { 0x6aa27d4d, SUC_lexicon_tags_12 },
    { 0xb0027d4d, SUC_lexicon_tags_1 },
    { 0xa8827d4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x67227d52, SUC_lexicon_tags_17 },
    { 0x5a427d53, SUC_lexicon_tags_7 },
    { 0xec027d53, SUC_lexicon_tags_15 },
    { 0x2a827d53, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x42227d58, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a27d5b, SUC_lexicon_tags_37 },
    { 0xc3827d5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca27d61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c427d65, SUC_lexicon_tags_11 },
    { 0x6ac27d66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3d227d68, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac27d6d, SUC_lexicon_tags_57 },
    { 0xe7027d6d, SUC_lexicon_tags_19 },
    { 0xfce27d6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c027d72, SUC_lexicon_tags_68 },
    { 0xa8027d73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39e27d75, SUC_lexicon_tags_32 },
    { 0x17c27d76, SUC_lexicon_tags_17 },
    { 0xbde27d76, SUC_lexicon_tags_29 },
    { 0xa8627d77, SUC_lexicon_tags_17 },
    { 0x1e827d76, SUC_lexicon_tags_37 },
    { 0x35e27d7a, SUC_lexicon_tags_30 },
    { 0xb1427d7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a27d80, SUC_lexicon_tags_35 },
    { 0x73827d81, SUC_lexicon_tags_23 },
    { 0xe2427d82, SUC_lexicon_tags_2 },
    { 0x90227d82, SUC_lexicon_tags_0 },
    { 0xde227d84, SUC_lexicon_tags_38 },
    { 0x6a627d85, SUC_lexicon_tags_12 },
    { 0xc027d86, SUC_lexicon_tags_33 },
    { 0x31427d87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4627d8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9227d93, SUC_lexicon_tags_22 },
    { 0x8be27d94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa27d98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15827d9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a27d9e, SUC_lexicon_tags_27 },
    { 0x28e27d9f, SUC_lexicon_tags_27 },
    { 0x1c027d9e, SUC_lexicon_tags_15 },
    { 0x80a27d9f, SUC_lexicon_tags_30 },
    { 0x5b027da1, SUC_lexicon_tags_28 },
    { 0xb1227da3, SUC_lexicon_tags_18 },
    { 0xeb227da1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76027da8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x48227dab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12227daf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52627db2, SUC_lexicon_tags_7 },
    { 0x17827db3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee27db6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42827dba, SUC_lexicon_tags_37 },
    { 0x9f027dbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4427dbe, SUC_lexicon_tags_11 },
    { 0xed027dbe, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a27dc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac27dcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68027dd0, SUC_lexicon_tags_11 },
    { 0xcec27dd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8e27dd3, SUC_lexicon_tags_7 },
    { 0x9ce27dd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd3e27dd6, SUC_lexicon_tags_55 },
    { 0xb4c27dd7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71427ddb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f027dde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29a27de0, SUC_lexicon_tags_37 },
    { 0x51827de1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48627de7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0427deb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38627df2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec27dfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5e27dfc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfd227dfe, SUC_lexicon_tags_54 },
    { 0xada27dff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e27e04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78827e09, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xee427e0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3627e13, SUC_lexicon_tags_26 },
    { 0x39027e14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c27e17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e427e1b, SUC_lexicon_tags_7 },
    { 0x28027e1c, SUC_lexicon_tags_49 },
    { 0x7a427e1d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7827e22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e27e27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e27e2f, SUC_lexicon_tags_45 },
    { 0x68427e30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94627e3c, SUC_lexicon_tags_43 },
    { 0x67e27e3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d427e3f, SUC_lexicon_tags_5 },
    { 0x72627e40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d227e45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8c27e47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf027e4d, SUC_lexicon_tags_37 },
    { 0x12427e4e, SUC_lexicon_tags_17 },
    { 0x4f227e4f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a27e59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4a27e5b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97427e61, SUC_lexicon_tags_4 },
    { 0xfb427e62, SUC_lexicon_tags_30 },
    { 0x52e27e63, SUC_lexicon_tags_63 },
    { 0x79c27e64, SUC_lexicon_tags_26 },
    { 0x7b427e65, SUC_lexicon_tags_23 },
    { 0x44427e66, SUC_lexicon_tags_7 },
    { 0xf3e27e64, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c427e6a, SUC_lexicon_tags_17 },
    { 0xd8227e6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43627e6f, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e27e73, SUC_lexicon_tags_1 },
    { 0x59e27e74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63c27e76, SUC_lexicon_tags_39 },
    { 0x9827e76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b227e79, SUC_lexicon_tags_23 },
    { 0x9a827e79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c427e7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x227e7e, SUC_lexicon_tags_47 },
    { 0x4227e7f, SUC_lexicon_tags_25 },
    { 0xa1027e80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea27e84, SUC_lexicon_tags_107 },
    { 0x5ec27e84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43627e87, SUC_lexicon_tags_37 },
    { 0x35e27e87, SUC_lexicon_tags_12 },
    { 0xc5427e88, SUC_lexicon_tags_25 },
    { 0x2dc27e8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a27e8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a27e9c, SUC_lexicon_tags_17 },
    { 0xd7627e9d, SUC_lexicon_tags_12 },
    { 0x15427e9e, SUC_lexicon_tags_23 },
    { 0x6c227e9c, SUC_lexicon_tags_38 },
    { 0x6d827ea0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a27ea4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x21227ea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6427eac, SUC_lexicon_tags_30 },
    { 0xcc827ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be27eb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a27eb8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf6027eba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a627ec1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3427ec7, SUC_lexicon_tags_25 },
    { 0x3427ec8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc827ed1, SUC_lexicon_tags_13 },
    { 0x94627ed2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77827eda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5627edd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa0627edf, SUC_lexicon_tags_37 },
    { 0xa7627ee0, SUC_lexicon_tags_2 },
    { 0x8cc27ee1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x71a27ee3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41427ee5, SUC_lexicon_tags_25 },
    { 0xbc27ee6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a27ee9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da27eed, SUC_lexicon_tags_15 },
    { 0x6c27eee, SUC_lexicon_tags_15 },
    { 0x19227eed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a27ef2, SUC_lexicon_tags_9 },
    { 0x9ba27ef3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x16227ef6, SUC_lexicon_tags_39 },
    { 0x8c827ef6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf427efc, SUC_lexicon_tags_37 },
    { 0x36227efd, SUC_lexicon_tags_37 },
    { 0xf3e27efc, SUC_lexicon_tags_11 },
    { 0xa427efd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54227f03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4827f06, SUC_lexicon_tags_15 },
    { 0xdc427f06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb027f09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c27f17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d227f1a, SUC_lexicon_tags_377 },
    { 0x2d827f1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93027f1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc5027f21, SUC_lexicon_tags_23 },
    { 0x7f827f22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4427f2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca227f2c, SUC_lexicon_tags_7 },
    { 0xb4827f2c, SUC_lexicon_tags_7 },
    { 0xb4a27f2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4827f31, SUC_lexicon_tags_17 },
    { 0xcd427f32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e27f38, SUC_lexicon_tags_15 },
    { 0xe2a27f39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11227f3f, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x27a27f41, SUC_lexicon_tags_37 },
    { 0xdd427f42, SUC_lexicon_tags_33 },
    { 0x94227f43, SUC_lexicon_tags_61 },
    { 0xa5827f42, SUC_lexicon_tags_43 },
    { 0x90e27f43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc27f49, SUC_lexicon_tags_11 },
    { 0x4de27f49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e27f4d, SUC_lexicon_tags_15 },
    { 0x6c827f4e, SUC_lexicon_tags_25 },
    { 0xdd027f4e, SUC_lexicon_tags_47 },
    { 0x54c27f50, SUC_lexicon_tags_85 },
    { 0x92427f4d, SUC_lexicon_tags_12 },
    { 0xcae27f52, SUC_lexicon_tags_4 },
    { 0x87227f50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e27f59, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0x2ac27f5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63827f60, SUC_lexicon_tags_27 },
    { 0x5c027f61, SUC_lexicon_tags_25 },
    { 0xaec27f61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb427f64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb627f6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17227f75, SUC_lexicon_tags_9 },
    { 0xd4427f76, SUC_lexicon_tags_32 },
    { 0xff827f75, SUC_lexicon_tags_0 },
    { 0x12827f78, SUC_lexicon_tags_25 },
    { 0x6fa27f78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57627f7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3a627f81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c27f8a, SUC_lexicon_tags_0 },
    { 0xf5c27f8a, SUC_lexicon_tags_15 },
    { 0x50027f8b, SUC_lexicon_tags_12 },
    { 0xec427f8c, SUC_lexicon_tags_2 },
    { 0xaae27f8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a227f91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e27f96, SUC_lexicon_tags_33 },
    { 0xcf427f97, SUC_lexicon_tags_11 },
    { 0xf5e27f97, SUC_lexicon_tags_5 },
    { 0xffe27f97, SUC_lexicon_tags_15 },
    { 0x2aa27f98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce27f9f, SUC_lexicon_tags_29 },
    { 0x98627fa0, SUC_lexicon_tags_129 },
    { 0x65a27fa1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72227fa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be27fa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69427fa9, SUC_lexicon_tags_49 },
    { 0x55427faa, SUC_lexicon_tags_1 },
    { 0xdf227fab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e27fb2, SUC_lexicon_tags_37 },
    { 0xfa427fb3, SUC_lexicon_tags_6 },
    { 0x80e27fb4, SUC_lexicon_tags_1 },
    { 0xde027fb5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xab427fb8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3027fc0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84227fc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd827fc7, SUC_lexicon_tags_17 },
    { 0x25a27fc8, SUC_lexicon_tags_4 },
    { 0x87027fc9, SUC_lexicon_tags_11 },
    { 0x66227fca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a27fcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b227fd1, SUC_lexicon_tags_11 },
    { 0xbfc27fd2, SUC_lexicon_tags_17 },
    { 0x3f227fd3, SUC_lexicon_tags_13 },
    { 0x62027fd4, SUC_lexicon_tags_148 },
    { 0xb6027fd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3c627fd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ac27fd9, SUC_lexicon_tags_55 },
    { 0xdf627fd9, SUC_lexicon_tags_25 },
    { 0x1e27fda, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc27fe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c27feb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0227fed, SUC_lexicon_tags_17 },
    { 0x7da27fee, SUC_lexicon_tags_87 },
    { 0x7fe27fee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc6027ff1, SUC_lexicon_tags_63 },
    { 0x4ac27ff1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71227ff7, SUC_lexicon_tags_47 },
    { 0x93027ff8, SUC_lexicon_tags_37 },
    { 0xaa27ff9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c27ffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48027ffd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2828001, SUC_lexicon_tags_27 },
    { 0xa5a28002, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xefa28004, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf8628006, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54428008, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5ca2800a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c28010, SUC_lexicon_tags_7 },
    { 0x67428011, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e28016, SUC_lexicon_tags_14 },
    { 0x69828016, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda2801c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d628020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9828026, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e2802a, SUC_lexicon_tags_25 },
    { 0x8762802a, SUC_lexicon_tags_33 },
    { 0xe122802c, SUC_lexicon_tags_63 },
    { 0xaac2802c, SUC_lexicon_tags_17 },
    { 0x1b02802c, SUC_lexicon_tags_29 },
    { 0xea42802b, SUC_lexicon_tags_23 },
    { 0x422802f, SUC_lexicon_tags_70 },
    { 0xa7a28031, SUC_lexicon_tags_40 },
    { 0xa2428032, SUC_lexicon_tags_23 },
    { 0x4428033, SUC_lexicon_tags_1 },
    { 0xd1028034, SUC_lexicon_tags_7 },
    { 0x7aa28031, SUC_lexicon_tags_25 },
    { 0xd7028036, SUC_lexicon_tags_15 },
    { 0xb9c28037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e2803a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fa2803c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f228041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a28045, SUC_lexicon_tags_22 },
    { 0x53c28045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a22804b, SUC_lexicon_tags_66 },
    { 0x5422804c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e828050, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a28054, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f228056, SUC_lexicon_tags_68 },
    { 0xf2228057, SUC_lexicon_tags_37 },
    { 0x1d428056, SUC_lexicon_tags_33 },
    { 0xac228056, SUC_lexicon_tags_17 },
    { 0xfd828056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9628061, SUC_lexicon_tags_9 },
    { 0xe3828062, SUC_lexicon_tags_51 },
    { 0xa8628063, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c28067, SUC_lexicon_tags_25 },
    { 0xec228067, SUC_lexicon_tags_1 },
    { 0x78a28068, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe102806e, SUC_lexicon_tags_19 },
    { 0x59c2806f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde28071, SUC_lexicon_tags_11 },
    { 0x34028072, SUC_lexicon_tags_27 },
    { 0x99e28072, SUC_lexicon_tags_11 },
    { 0x55028074, SUC_lexicon_tags_33 },
    { 0xba28075, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae28078, SUC_lexicon_tags_10 },
    { 0xd4e28079, SUC_lexicon_tags_27 },
    { 0x5142807a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e42807d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x10e2807f, SUC_lexicon_tags_4 },
    { 0x8ac28080, SUC_lexicon_tags_19 },
    { 0xa6228081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc28084, SUC_lexicon_tags_7 },
    { 0x24428085, SUC_lexicon_tags_30 },
    { 0xfe828085, SUC_lexicon_tags_11 },
    { 0x94828086, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c82808d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49628091, SUC_lexicon_tags_8 },
    { 0x5a828092, SUC_lexicon_tags_45 },
    { 0x55228091, SUC_lexicon_tags_17 },
    { 0x90228094, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26e28096, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e22809a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa2809e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc2280a0, SUC_lexicon_tags_47 },
    { 0xe1a280a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a280aa, SUC_lexicon_tags_0 },
    { 0xe42280ab, SUC_lexicon_tags_23 },
    { 0x6a8280ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc8e280ae, SUC_lexicon_tags_1 },
    { 0x992280af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca280b2, SUC_lexicon_tags_17 },
    { 0x826280b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf8280b5, SUC_lexicon_tags_6 },
    { 0x790280b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00280b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4280bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8a280be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a280c1, SUC_lexicon_tags_8 },
    { 0x736280c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf36280c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33c280c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c280ce, SUC_lexicon_tags_37 },
    { 0xe00280ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x30280d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a280d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x494280d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa82280db, SUC_lexicon_tags_7 },
    { 0xe0c280db, SUC_lexicon_tags_0 },
    { 0x870280dd, SUC_lexicon_tags_16 },
    { 0xb62280db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce280e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x434280e6, SUC_lexicon_tags_10 },
    { 0xa9c280e7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6280ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c280f1, SUC_lexicon_tags_14 },
    { 0x1dc280f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11a280f4, SUC_lexicon_tags_43 },
    { 0xa76280f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204280fe, SUC_lexicon_tags_554 },
    { 0x7ec280ff, SUC_lexicon_tags_48 },
    { 0x9e28100, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75828106, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x91828109, SUC_lexicon_tags_14 },
    { 0xfc82810a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7182810d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39828121, SUC_lexicon_tags_45 },
    { 0xc0428122, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d828127, SUC_lexicon_tags_2 },
    { 0x9a828127, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba2812b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17428131, SUC_lexicon_tags_37 },
    { 0x9e228131, SUC_lexicon_tags_31 },
    { 0xff628133, SUC_lexicon_tags_11 },
    { 0xd5e28134, SUC_lexicon_tags_9 },
    { 0x66428134, SUC_lexicon_tags_35 },
    { 0x70c28133, SUC_lexicon_tags_28 },
    { 0xbdc28137, SUC_lexicon_tags_15 },
    { 0xde428138, SUC_lexicon_tags_250 },
    { 0xe628139, SUC_lexicon_tags_11 },
    { 0x22a28139, SUC_lexicon_tags_25 },
    { 0xca628138, SUC_lexicon_tags_26 },
    { 0x6ac2813c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb742813e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a428144, SUC_lexicon_tags_17 },
    { 0xcca28145, SUC_lexicon_tags_45 },
    { 0xd4a28144, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5882814f, SUC_lexicon_tags_0 },
    { 0x5a428150, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6828152, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x98028155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1702815a, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xca42815c, SUC_lexicon_tags_2 },
    { 0x9522815c, SUC_lexicon_tags_11 },
    { 0xa942815e, SUC_lexicon_tags_17 },
    { 0x73e2815f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c28163, SUC_lexicon_tags_11 },
    { 0xdc228164, SUC_lexicon_tags_37 },
    { 0x8f428165, SUC_lexicon_tags_45 },
    { 0x44028166, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1028169, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e2816c, SUC_lexicon_tags_32 },
    { 0xc402816d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x32428170, SUC_lexicon_tags_7 },
    { 0xb2428171, SUC_lexicon_tags_11 },
    { 0xf1428172, SUC_lexicon_tags_10 },
    { 0x49c28172, SUC_lexicon_tags_11 },
    { 0xa7228173, SUC_lexicon_tags_60 },
    { 0xfac28175, SUC_lexicon_tags_6 },
    { 0xd4c28175, SUC_lexicon_tags_57 },
    { 0xfda28177, SUC_lexicon_tags_14 },
    { 0xa4c28174, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20e2817a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc7e2817c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e428182, SUC_lexicon_tags_37 },
    { 0xf6c28183, SUC_lexicon_tags_37 },
    { 0xeb428184, SUC_lexicon_tags_43 },
    { 0xc6828183, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d028188, SUC_lexicon_tags_37 },
    { 0xaa228189, SUC_lexicon_tags_7 },
    { 0xc9a28189, SUC_lexicon_tags_37 },
    { 0x9322818a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6942818d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3422818f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f428194, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e2819b, SUC_lexicon_tags_25 },
    { 0x9aa2819b, SUC_lexicon_tags_27 },
    { 0x3e82819d, SUC_lexicon_tags_16 },
    { 0x9fe2819d, SUC_lexicon_tags_14 },
    { 0x1182819f, SUC_lexicon_tags_60 },
    { 0xd42819f, SUC_lexicon_tags_25 },
    { 0x9e22819b, SUC_lexicon_tags_253 },
    { 0x4da281a2, SUC_lexicon_tags_25 },
    { 0x9282819b, SUC_lexicon_tags_63 },
    { 0xade281a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c281a7, SUC_lexicon_tags_37 },
    { 0x7ea281a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x894281aa, SUC_lexicon_tags_37 },
    { 0xf8c281ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528281b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe74281b4, SUC_lexicon_tags_43 },
    { 0x7d0281b4, SUC_lexicon_tags_47 },
    { 0xc20281b6, SUC_lexicon_tags_26 },
    { 0x6be281b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f2281b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec281c4, SUC_lexicon_tags_62 },
    { 0xb96281c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4281c9, SUC_lexicon_tags_37 },
    { 0xeca281ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2281cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278281d3, SUC_lexicon_tags_14 },
    { 0xd7a281d4, SUC_lexicon_tags_31 },
    { 0xb0c281d5, SUC_lexicon_tags_43 },
    { 0xd76281d6, SUC_lexicon_tags_7 },
    { 0x51c281d6, SUC_lexicon_tags_18 },
    { 0x2d2281d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86281db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68c281dd, SUC_lexicon_tags_2 },
    { 0x328281de, SUC_lexicon_tags_12 },
    { 0xdfe281dd, SUC_lexicon_tags_31 },
    { 0x574281e0, SUC_lexicon_tags_63 },
    { 0x25e281e1, SUC_lexicon_tags_19 },
    { 0x39c281e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2281e6, SUC_lexicon_tags_8 },
    { 0x8c4281e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200281ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f0281ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e281f6, SUC_lexicon_tags_4 },
    { 0x782281f6, SUC_lexicon_tags_63 },
    { 0xc92281f8, SUC_lexicon_tags_27 },
    { 0x1a2281f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x968281fb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0281ff, SUC_lexicon_tags_12 },
    { 0x6e8281ff, SUC_lexicon_tags_77 },
    { 0xa4228200, SUC_lexicon_tags_37 },
    { 0xaf2281ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6b228204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc28209, SUC_lexicon_tags_49 },
    { 0x2a02820a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f22820f, SUC_lexicon_tags_12 },
    { 0x65228210, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87c28212, SUC_lexicon_tags_878 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14628219, SUC_lexicon_tags_25 },
    { 0x6c22821a, SUC_lexicon_tags_46 },
    { 0x3ec2821a, SUC_lexicon_tags_19 },
    { 0x9762821c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee28221, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c28224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c28229, SUC_lexicon_tags_13 },
    { 0x2ae2822a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c28231, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90228236, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe502823a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d62823e, SUC_lexicon_tags_15 },
    { 0x7562823f, SUC_lexicon_tags_17 },
    { 0x32c28240, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c28244, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0e28246, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe9428248, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf262824a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda82824d, SUC_lexicon_tags_19 },
    { 0xe622824e, SUC_lexicon_tags_7 },
    { 0xcca2824f, SUC_lexicon_tags_25 },
    { 0x92228250, SUC_lexicon_tags_1 },
    { 0x20828250, SUC_lexicon_tags_37 },
    { 0x6de28252, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70028256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2642825b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3982825e, SUC_lexicon_tags_11 },
    { 0x4be2825f, SUC_lexicon_tags_6 },
    { 0xe982825f, SUC_lexicon_tags_9 },
    { 0xcde28261, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x56228264, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2228269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc722826b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f228271, SUC_lexicon_tags_11 },
    { 0x95a28272, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd228278, SUC_lexicon_tags_7 },
    { 0xd6628279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7c2827b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02827e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27828285, SUC_lexicon_tags_27 },
    { 0x25428286, SUC_lexicon_tags_37 },
    { 0xeae28287, SUC_lexicon_tags_1 },
    { 0x4d428288, SUC_lexicon_tags_57 },
    { 0xe1828287, SUC_lexicon_tags_17 },
    { 0x99628288, SUC_lexicon_tags_17 },
    { 0xf162828b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9428294, SUC_lexicon_tags_31 },
    { 0x75e28294, SUC_lexicon_tags_25 },
    { 0x33228296, SUC_lexicon_tags_1 },
    { 0x2b228296, SUC_lexicon_tags_10 },
    { 0xc9028298, SUC_lexicon_tags_22 },
    { 0x71a28299, SUC_lexicon_tags_63 },
    { 0x92828299, SUC_lexicon_tags_17 },
    { 0x3ac28296, SUC_lexicon_tags_1 },
    { 0xb1428296, SUC_lexicon_tags_25 },
    { 0x78c2829a, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0x51a2829f, SUC_lexicon_tags_6 },
    { 0x9fa282a0, SUC_lexicon_tags_7 },
    { 0xf0282a1, SUC_lexicon_tags_11 },
    { 0x782282a2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5b4282a4, SUC_lexicon_tags_21 },
    { 0x23e282a5, SUC_lexicon_tags_11 },
    { 0xdb8282a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16282aa, SUC_lexicon_tags_25 },
    { 0x534282ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc86282ad, SUC_lexicon_tags_2 },
    { 0x472282ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac282b3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x354282b6, SUC_lexicon_tags_13 },
    { 0x802282b6, SUC_lexicon_tags_66 },
    { 0xb88282b6, SUC_lexicon_tags_158 },
    { 0x484282b8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a282bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a282c2, SUC_lexicon_tags_55 },
    { 0x82282c2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc282cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e282cf, SUC_lexicon_tags_45 },
    { 0x3a282d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2282d3, SUC_lexicon_tags_11 },
    { 0x6d8282d4, SUC_lexicon_tags_70 },
    { 0x158282d5, SUC_lexicon_tags_47 },
    { 0x46c282d6, SUC_lexicon_tags_17 },
    { 0x9ae282d6, SUC_lexicon_tags_31 },
    { 0xe76282d8, SUC_lexicon_tags_55 },
    { 0x934282d9, SUC_lexicon_tags_21 },
    { 0x806282da, SUC_lexicon_tags_15 },
    { 0x954282db, SUC_lexicon_tags_16 },
    { 0x9a6282d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9c282de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18282e2, SUC_lexicon_tags_26 },
    { 0xf3e282e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc64282e5, SUC_lexicon_tags_12 },
    { 0xd14282e6, SUC_lexicon_tags_25 },
    { 0xc4c282e7, SUC_lexicon_tags_16 },
    { 0xf30282e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce282eb, SUC_lexicon_tags_25 },
    { 0xb14282eb, SUC_lexicon_tags_43 },
    { 0xe3e282ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x962282f0, SUC_lexicon_tags_83 },
    { 0xae4282f1, SUC_lexicon_tags_17 },
    { 0xefa282f0, SUC_lexicon_tags_39 },
    { 0xeba282f3, SUC_lexicon_tags_43 },
    { 0xc50282f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70e282f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe92282f8, SUC_lexicon_tags_23 },
    { 0xfe4282f9, SUC_lexicon_tags_11 },
    { 0xe34282fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16628300, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a28305, SUC_lexicon_tags_12 },
    { 0x61c28305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e28309, SUC_lexicon_tags_72 },
    { 0x7962830a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9102830d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa28311, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0828314, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c2831a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e2831e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x45228321, SUC_lexicon_tags_33 },
    { 0x83828321, SUC_lexicon_tags_63 },
    { 0xb1828323, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0228328, SUC_lexicon_tags_55 },
    { 0x58a28328, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x82a2832b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68428331, SUC_lexicon_tags_82 },
    { 0x7da28332, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97828335, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3628337, SUC_lexicon_tags_7 },
    { 0xe7828338, SUC_lexicon_tags_7 },
    { 0xa5e28339, SUC_lexicon_tags_7 },
    { 0x5de2833a, SUC_lexicon_tags_6 },
    { 0xc462833b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3462833d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e428342, SUC_lexicon_tags_17 },
    { 0x1c028342, SUC_lexicon_tags_33 },
    { 0x98628342, SUC_lexicon_tags_38 },
    { 0x75228344, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8602834a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3da2834c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf82834e, SUC_lexicon_tags_2 },
    { 0xadc2834f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23828353, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xea028356, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1782835b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf3a2835d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91028362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9428366, SUC_lexicon_tags_27 },
    { 0x17828367, SUC_lexicon_tags_37 },
    { 0xa5228368, SUC_lexicon_tags_63 },
    { 0x35028366, SUC_lexicon_tags_16 },
    { 0x6e28366, SUC_lexicon_tags_57 },
    { 0x90a28368, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75828374, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e2837f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e828382, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb828384, SUC_lexicon_tags_40 },
    { 0x30e28385, SUC_lexicon_tags_62 },
    { 0xec628386, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbd028388, SUC_lexicon_tags_13 },
    { 0x628389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd22838d, SUC_lexicon_tags_6 },
    { 0x2142838d, SUC_lexicon_tags_11 },
    { 0xed02838d, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac28398, SUC_lexicon_tags_62 },
    { 0xaba28399, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e2839f, SUC_lexicon_tags_138 },
    { 0xa242839f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c283a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2e6283a5, SUC_lexicon_tags_38 },
    { 0x4f8283a6, SUC_lexicon_tags_15 },
    { 0xea6283a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca283aa, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c283b1, SUC_lexicon_tags_27 },
    { 0xa7a283b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc283b5, SUC_lexicon_tags_21 },
    { 0x16a283b6, SUC_lexicon_tags_11 },
    { 0x5ee283b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2283ba, SUC_lexicon_tags_29 },
    { 0x868283ba, SUC_lexicon_tags_49 },
    { 0x108283bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a283c1, SUC_lexicon_tags_2 },
    { 0xf5c283c2, SUC_lexicon_tags_47 },
    { 0x886283c3, SUC_lexicon_tags_11 },
    { 0xf30283c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e283c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56283ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a283cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f6283cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8283d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a283d8, SUC_lexicon_tags_8 },
    { 0xd24283d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a283de, SUC_lexicon_tags_47 },
    { 0xd48283df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8f2283e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x476283e3, SUC_lexicon_tags_22 },
    { 0x8e2283e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x724283e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a283eb, SUC_lexicon_tags_12 },
    { 0x6a8283ec, SUC_lexicon_tags_11 },
    { 0x842283ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e283f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e283f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94283f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae6283fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882283ff, SUC_lexicon_tags_148 },
    { 0x104283ff, SUC_lexicon_tags_49 },
    { 0xd2428401, SUC_lexicon_tags_29 },
    { 0x4a828401, SUC_lexicon_tags_47 },
    { 0x11028403, SUC_lexicon_tags_1179 },
    { 0xc3228404, SUC_lexicon_tags_14 },
    { 0x1f428405, SUC_lexicon_tags_45 },
    { 0x4f428406, SUC_lexicon_tags_65 },
    { 0x7c028407, SUC_lexicon_tags_2 },
    { 0xa4428408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e2840b, SUC_lexicon_tags_60 },
    { 0xd362840b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9722840e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1028411, SUC_lexicon_tags_47 },
    { 0xbb628412, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x51828415, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2628417, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e2841b, SUC_lexicon_tags_7 },
    { 0xafa2841c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a2841f, SUC_lexicon_tags_53 },
    { 0x2828420, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e228423, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e28426, SUC_lexicon_tags_2 },
    { 0x7a828427, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xedc28429, SUC_lexicon_tags_3 },
    { 0x742842a, SUC_lexicon_tags_40 },
    { 0xb402842a, SUC_lexicon_tags_27 },
    { 0x19e2842b, SUC_lexicon_tags_82 },
    { 0x4722842d, SUC_lexicon_tags_12 },
    { 0xd3e2842d, SUC_lexicon_tags_33 },
    { 0x9102842f, SUC_lexicon_tags_25 },
    { 0x2ec2842f, SUC_lexicon_tags_17 },
    { 0xa942842d, SUC_lexicon_tags_35 },
    { 0x6dc28430, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a428437, SUC_lexicon_tags_47 },
    { 0x6ba28438, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95e2843a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d62843c, SUC_lexicon_tags_7 },
    { 0xaa2843d, SUC_lexicon_tags_1 },
    { 0x3522843e, SUC_lexicon_tags_26 },
    { 0x9c02843e, SUC_lexicon_tags_2 },
    { 0xcf628440, SUC_lexicon_tags_25 },
    { 0x18628440, SUC_lexicon_tags_30 },
    { 0xafc28440, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc262844b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xabe2844d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2642844f, SUC_lexicon_tags_15 },
    { 0x1622844f, SUC_lexicon_tags_15 },
    { 0x6922844f, SUC_lexicon_tags_48 },
    { 0x22c28452, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8182845a, SUC_lexicon_tags_23 },
    { 0xac62845a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8282845d, SUC_lexicon_tags_1 },
    { 0xa82845e, SUC_lexicon_tags_62 },
    { 0xbea2845f, SUC_lexicon_tags_37 },
    { 0xe1a28460, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdc228462, SUC_lexicon_tags_23 },
    { 0x59028462, SUC_lexicon_tags_47 },
    { 0xd1628462, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e028469, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a2846c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2228472, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc5028474, SUC_lexicon_tags_37 },
    { 0xbe628474, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9e28477, SUC_lexicon_tags_11 },
    { 0xbb228478, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7802847e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e628482, SUC_lexicon_tags_56 },
    { 0xdbc28483, SUC_lexicon_tags_25 },
    { 0xe7028484, SUC_lexicon_tags_62 },
    { 0xa9828485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca28488, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c2848e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1028493, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39228497, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2849c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de284a3, SUC_lexicon_tags_1 },
    { 0x574284a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44284a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e284ad, SUC_lexicon_tags_8 },
    { 0x45e284ad, SUC_lexicon_tags_40 },
    { 0xc2e284af, SUC_lexicon_tags_0 },
    { 0x16284af, SUC_lexicon_tags_37 },
    { 0xab6284ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30284b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80284bb, SUC_lexicon_tags_11 },
    { 0x656284bb, SUC_lexicon_tags_1 },
    { 0xb70284bb, SUC_lexicon_tags_108 },
    { 0x62284bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x852284c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x604284c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x306284c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x434284c9, SUC_lexicon_tags_17 },
    { 0x798284ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae284ce, SUC_lexicon_tags_900 },
    { 0xb5c284ce, SUC_lexicon_tags_7 },
    { 0x192284cf, SUC_lexicon_tags_7 },
    { 0x4a4284d1, SUC_lexicon_tags_11 },
    { 0xdc0284d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe8284d4, SUC_lexicon_tags_12 },
    { 0x250284d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x358284d8, SUC_lexicon_tags_17 },
    { 0xe98284d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x82284db, SUC_lexicon_tags_25 },
    { 0x204284db, SUC_lexicon_tags_11 },
    { 0x152284dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622284e1, SUC_lexicon_tags_29 },
    { 0xa7a284e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170284e6, SUC_lexicon_tags_37 },
    { 0xa00284e7, SUC_lexicon_tags_37 },
    { 0xcfc284e8, SUC_lexicon_tags_33 },
    { 0xd86284e9, SUC_lexicon_tags_12 },
    { 0x280284ea, SUC_lexicon_tags_11 },
    { 0x676284eb, SUC_lexicon_tags_15 },
    { 0x46284ec, SUC_lexicon_tags_7 },
    { 0x35a284eb, SUC_lexicon_tags_100 },
    { 0x5ee284ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe74284f0, SUC_lexicon_tags_25 },
    { 0xefa284f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xebc284f3, SUC_lexicon_tags_19 },
    { 0x4ec284f3, SUC_lexicon_tags_17 },
    { 0xc60284f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xac284f8, SUC_lexicon_tags_38 },
    { 0x162284f8, SUC_lexicon_tags_47 },
    { 0x686284fa, SUC_lexicon_tags_17 },
    { 0x5ae284f8, SUC_lexicon_tags_17 },
    { 0x892284f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d428503, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8c28505, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2a28507, SUC_lexicon_tags_26 },
    { 0x9e428507, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c2850a, SUC_lexicon_tags_25 },
    { 0xd522850b, SUC_lexicon_tags_55 },
    { 0xe8e2850c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc28510, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xff028512, SUC_lexicon_tags_1 },
    { 0xd8828513, SUC_lexicon_tags_0 },
    { 0xeec28513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7828518, SUC_lexicon_tags_17 },
    { 0x66428519, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdce2851b, SUC_lexicon_tags_25 },
    { 0xdf62851b, SUC_lexicon_tags_565 },
    { 0, NULL },
    { 0x1542851e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34e28520, SUC_lexicon_tags_29 },
    { 0x18628520, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1828526, SUC_lexicon_tags_25 },
    { 0xc4028527, SUC_lexicon_tags_12 },
    { 0xd0628528, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe382852b, SUC_lexicon_tags_37 },
    { 0xbf02852b, SUC_lexicon_tags_2 },
    { 0x2262852c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef428531, SUC_lexicon_tags_63 },
    { 0x34628532, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36228537, SUC_lexicon_tags_11 },
    { 0xe1228538, SUC_lexicon_tags_25 },
    { 0xc2a28538, SUC_lexicon_tags_17 },
    { 0x2a02853a, SUC_lexicon_tags_30 },
    { 0xbc28539, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf422853e, SUC_lexicon_tags_1 },
    { 0x362853f, SUC_lexicon_tags_1 },
    { 0xc8828540, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a28544, SUC_lexicon_tags_12 },
    { 0x2b228544, SUC_lexicon_tags_17 },
    { 0x25828546, SUC_lexicon_tags_12 },
    { 0xd1628546, SUC_lexicon_tags_47 },
    { 0x33a28545, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6722854a, SUC_lexicon_tags_17 },
    { 0x3ec2854a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e2854e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49028557, SUC_lexicon_tags_63 },
    { 0xf6828558, SUC_lexicon_tags_25 },
    { 0x42a28559, SUC_lexicon_tags_60 },
    { 0x5ba2855a, SUC_lexicon_tags_55 },
    { 0x50828558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4502855d, SUC_lexicon_tags_17 },
    { 0x2982855e, SUC_lexicon_tags_8 },
    { 0xdb02855e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21e28561, SUC_lexicon_tags_37 },
    { 0xe6a28561, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4628569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a2856d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f228573, SUC_lexicon_tags_17 },
    { 0x5d828573, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26028583, SUC_lexicon_tags_11 },
    { 0xb5228583, SUC_lexicon_tags_30 },
    { 0xba828585, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c28588, SUC_lexicon_tags_37 },
    { 0xbbc28589, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d42858b, SUC_lexicon_tags_17 },
    { 0xcea2858b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f02858f, SUC_lexicon_tags_45 },
    { 0xf2a28590, SUC_lexicon_tags_4 },
    { 0x3d828591, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f428594, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x25c28596, SUC_lexicon_tags_77 },
    { 0x4de28597, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2302859d, SUC_lexicon_tags_19 },
    { 0xf002859e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a6285a0, SUC_lexicon_tags_37 },
    { 0xa36285a0, SUC_lexicon_tags_21 },
    { 0xf42285a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x272285a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88285a8, SUC_lexicon_tags_7 },
    { 0xc14285a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0285ad, SUC_lexicon_tags_57 },
    { 0xba0285ae, SUC_lexicon_tags_37 },
    { 0xa00285af, SUC_lexicon_tags_3 },
    { 0x750285b0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa285ba, SUC_lexicon_tags_2 },
    { 0x300285bb, SUC_lexicon_tags_37 },
    { 0x240285bc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02285c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c6285c3, SUC_lexicon_tags_14 },
    { 0xdf0285c4, SUC_lexicon_tags_12 },
    { 0x860285c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e8285c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3ba285c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x86e285cb, SUC_lexicon_tags_47 },
    { 0xe0e285cc, SUC_lexicon_tags_15 },
    { 0xb90285cd, SUC_lexicon_tags_37 },
    { 0x8a0285cc, SUC_lexicon_tags_21 },
    { 0xec2285cd, SUC_lexicon_tags_15 },
    { 0x364285ce, SUC_lexicon_tags_15 },
    { 0x80c285ce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2285d4, SUC_lexicon_tags_46 },
    { 0xe98285d4, SUC_lexicon_tags_29 },
    { 0xf28285d6, SUC_lexicon_tags_15 },
    { 0xc6a285d7, SUC_lexicon_tags_37 },
    { 0x828285d4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912285dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10a285de, SUC_lexicon_tags_17 },
    { 0x828285df, SUC_lexicon_tags_1 },
    { 0xab8285de, SUC_lexicon_tags_2 },
    { 0xb8e285e1, SUC_lexicon_tags_29 },
    { 0xb8a285e1, SUC_lexicon_tags_40 },
    { 0xcea285e3, SUC_lexicon_tags_29 },
    { 0xb68285de, SUC_lexicon_tags_30 },
    { 0x8c6285e2, SUC_lexicon_tags_39 },
    { 0x776285e6, SUC_lexicon_tags_47 },
    { 0x590285e7, SUC_lexicon_tags_17 },
    { 0x574285e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84285ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xde6285ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52285f0, SUC_lexicon_tags_15 },
    { 0xb2285f1, SUC_lexicon_tags_1 },
    { 0x2f4285f0, SUC_lexicon_tags_27 },
    { 0x4be285f1, SUC_lexicon_tags_2 },
    { 0xf74285f4, SUC_lexicon_tags_15 },
    { 0x58c285f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x282285f7, SUC_lexicon_tags_9 },
    { 0x72285f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa285fc, SUC_lexicon_tags_12 },
    { 0x982285fd, SUC_lexicon_tags_38 },
    { 0x274285fc, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x98c28600, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e228602, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75e28604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9082860a, SUC_lexicon_tags_28 },
    { 0x3a22860a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54428610, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xff228613, SUC_lexicon_tags_12 },
    { 0xd3828614, SUC_lexicon_tags_23 },
    { 0x31428615, SUC_lexicon_tags_7 },
    { 0x40428616, SUC_lexicon_tags_13 },
    { 0x1ce28617, SUC_lexicon_tags_31 },
    { 0x89228618, SUC_lexicon_tags_46 },
    { 0x95c28618, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3882861b, SUC_lexicon_tags_17 },
    { 0xa282861b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d828621, SUC_lexicon_tags_33 },
    { 0xc2228622, SUC_lexicon_tags_25 },
    { 0xb3628623, SUC_lexicon_tags_12 },
    { 0x47028623, SUC_lexicon_tags_45 },
    { 0x6d628623, SUC_lexicon_tags_20 },
    { 0xf8428626, SUC_lexicon_tags_2 },
    { 0x31828621, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41028629, SUC_lexicon_tags_39 },
    { 0x7842862a, SUC_lexicon_tags_8 },
    { 0x8482862b, SUC_lexicon_tags_27 },
    { 0x7c42862a, SUC_lexicon_tags_37 },
    { 0x3322862b, SUC_lexicon_tags_2 },
    { 0x39a2862e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x29628630, SUC_lexicon_tags_15 },
    { 0xbc28630, SUC_lexicon_tags_1 },
    { 0xd3a28632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe628638, SUC_lexicon_tags_65 },
    { 0xa5028639, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b62863c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0828641, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7a28643, SUC_lexicon_tags_12 },
    { 0x99428643, SUC_lexicon_tags_24 },
    { 0xfb628644, SUC_lexicon_tags_37 },
    { 0xebc28646, SUC_lexicon_tags_8 },
    { 0x20428643, SUC_lexicon_tags_30 },
    { 0x46428648, SUC_lexicon_tags_31 },
    { 0xa3628645, SUC_lexicon_tags_65 },
    { 0x3d62864a, SUC_lexicon_tags_17 },
    { 0x49a2864a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcdc2864d, SUC_lexicon_tags_29 },
    { 0x36c2864e, SUC_lexicon_tags_129 },
    { 0xd842864e, SUC_lexicon_tags_19 },
    { 0xc4c28650, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a28654, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x86628656, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfd428658, SUC_lexicon_tags_17 },
    { 0xf8228659, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe2865d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c28662, SUC_lexicon_tags_15 },
    { 0x2828662, SUC_lexicon_tags_17 },
    { 0x5228662, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c28667, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a22866a, SUC_lexicon_tags_15 },
    { 0x9602866b, SUC_lexicon_tags_11 },
    { 0x6562866c, SUC_lexicon_tags_21 },
    { 0x1742866c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e828670, SUC_lexicon_tags_11 },
    { 0xf9c28671, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6a228673, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb228677, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e2867a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb628687, SUC_lexicon_tags_45 },
    { 0x8dc28688, SUC_lexicon_tags_68 },
    { 0xe4628688, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef62868b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3828692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e2869c, SUC_lexicon_tags_60 },
    { 0x3722869d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee286a3, SUC_lexicon_tags_8 },
    { 0x938286a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a0286a6, SUC_lexicon_tags_19 },
    { 0x18e286a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e286ac, SUC_lexicon_tags_17 },
    { 0xfc2286ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d2286af, SUC_lexicon_tags_43 },
    { 0x65a286b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c286b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4286b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ca286ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x364286bd, SUC_lexicon_tags_2 },
    { 0x4b8286bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa26286c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c286c3, SUC_lexicon_tags_63 },
    { 0x4c8286c4, SUC_lexicon_tags_27 },
    { 0x712286c4, SUC_lexicon_tags_65 },
    { 0x378286c6, SUC_lexicon_tags_2 },
    { 0x74c286c7, SUC_lexicon_tags_14 },
    { 0xffa286c3, SUC_lexicon_tags_2 },
    { 0xe5e286c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba286cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed2286cf, SUC_lexicon_tags_26 },
    { 0x39a286d0, SUC_lexicon_tags_33 },
    { 0x6d0286d1, SUC_lexicon_tags_11 },
    { 0xc3e286d2, SUC_lexicon_tags_4 },
    { 0x232286d0, SUC_lexicon_tags_17 },
    { 0x96e286d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a286de, SUC_lexicon_tags_15 },
    { 0xbce286de, SUC_lexicon_tags_15 },
    { 0x4f6286de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8286e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc286ea, SUC_lexicon_tags_17 },
    { 0xa96286eb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4f4286ed, SUC_lexicon_tags_7 },
    { 0x56a286ed, SUC_lexicon_tags_15 },
    { 0xe24286ee, SUC_lexicon_tags_45 },
    { 0x9fc286ee, SUC_lexicon_tags_17 },
    { 0xff8286ee, SUC_lexicon_tags_40 },
    { 0x2da286f2, SUC_lexicon_tags_45 },
    { 0x748286f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x120286f5, SUC_lexicon_tags_14 },
    { 0xb52286f6, SUC_lexicon_tags_35 },
    { 0xfaa286f5, SUC_lexicon_tags_30 },
    { 0x918286f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564286fd, SUC_lexicon_tags_27 },
    { 0xa3e286fd, SUC_lexicon_tags_19 },
    { 0xcb6286fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4228704, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86828707, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b62870a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e2870d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56628714, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3828717, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3ca28719, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e2871c, SUC_lexicon_tags_14 },
    { 0xd642871c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e28721, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe228725, SUC_lexicon_tags_43 },
    { 0xade28725, SUC_lexicon_tags_37 },
    { 0xc8e28727, SUC_lexicon_tags_34 },
    { 0x56a28726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa662872a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47828730, SUC_lexicon_tags_47 },
    { 0x1b628730, SUC_lexicon_tags_35 },
    { 0xe7c28732, SUC_lexicon_tags_3 },
    { 0x67a28733, SUC_lexicon_tags_60 },
    { 0xb3228734, SUC_lexicon_tags_2 },
    { 0x32028734, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a28738, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce2873b, SUC_lexicon_tags_11 },
    { 0x722873b, SUC_lexicon_tags_57 },
    { 0x4fa2873d, SUC_lexicon_tags_37 },
    { 0x1522873b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9228742, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa2874a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8002874c, SUC_lexicon_tags_15 },
    { 0x1f82874c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e28751, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x62828753, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26628756, SUC_lexicon_tags_2 },
    { 0x38e28756, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa428760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70428762, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70e28764, SUC_lexicon_tags_29 },
    { 0xdc228765, SUC_lexicon_tags_27 },
    { 0x14028766, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d42876a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422876e, SUC_lexicon_tags_15 },
    { 0xcc42876f, SUC_lexicon_tags_38 },
    { 0x9b02876f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78828774, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83a28776, SUC_lexicon_tags_45 },
    { 0x9b428777, SUC_lexicon_tags_4 },
    { 0x7dc28778, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c2877d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3628780, SUC_lexicon_tags_15 },
    { 0xc8028781, SUC_lexicon_tags_27 },
    { 0xd0628782, SUC_lexicon_tags_66 },
    { 0x8d028782, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61628786, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d22878a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x662878d, SUC_lexicon_tags_82 },
    { 0x26a2878e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6762879f, SUC_lexicon_tags_37 },
    { 0xcd82879f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea2287a2, SUC_lexicon_tags_10 },
    { 0x320287a3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4d6287a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x722287a8, SUC_lexicon_tags_29 },
    { 0xe92287a8, SUC_lexicon_tags_138 },
    { 0x266287aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0287b0, SUC_lexicon_tags_49 },
    { 0x788287b0, SUC_lexicon_tags_61 },
    { 0xb60287b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a6287b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6287bc, SUC_lexicon_tags_47 },
    { 0xed4287bc, SUC_lexicon_tags_25 },
    { 0x5a8287be, SUC_lexicon_tags_12 },
    { 0x646287bc, SUC_lexicon_tags_43 },
    { 0x6b2287c0, SUC_lexicon_tags_1 },
    { 0x5a8287c1, SUC_lexicon_tags_11 },
    { 0xada287c1, SUC_lexicon_tags_1 },
    { 0x298287c3, SUC_lexicon_tags_2 },
    { 0x3e2287c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea287c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12287cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae287cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750287d3, SUC_lexicon_tags_79 },
    { 0xfac287d4, SUC_lexicon_tags_24 },
    { 0x2d0287d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2287d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd72287db, SUC_lexicon_tags_7 },
    { 0xb4c287dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8287e1, SUC_lexicon_tags_0 },
    { 0xe32287e2, SUC_lexicon_tags_37 },
    { 0xf16287e1, SUC_lexicon_tags_17 },
    { 0xaae287e4, SUC_lexicon_tags_17 },
    { 0x4e4287e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38287ec, SUC_lexicon_tags_1 },
    { 0x8c4287ec, SUC_lexicon_tags_25 },
    { 0xc8a287ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x524287f1, SUC_lexicon_tags_17 },
    { 0xad2287f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02287f9, SUC_lexicon_tags_1 },
    { 0x546287fa, SUC_lexicon_tags_21 },
    { 0xae6287fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba28802, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7a228804, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b628806, SUC_lexicon_tags_15 },
    { 0xfb428806, SUC_lexicon_tags_0 },
    { 0x1228807, SUC_lexicon_tags_15 },
    { 0x34828809, SUC_lexicon_tags_19 },
    { 0xafe28809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde2880d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e28810, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9828813, SUC_lexicon_tags_129 },
    { 0x2d628813, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88228820, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde428825, SUC_lexicon_tags_37 },
    { 0xdbc28826, SUC_lexicon_tags_17 },
    { 0x35828826, SUC_lexicon_tags_2 },
    { 0xf9a28828, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf82882b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e28830, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e28835, SUC_lexicon_tags_16 },
    { 0x62428836, SUC_lexicon_tags_37 },
    { 0x9a028835, SUC_lexicon_tags_16 },
    { 0xfe828838, SUC_lexicon_tags_29 },
    { 0xef628839, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd228840, SUC_lexicon_tags_21 },
    { 0x12428841, SUC_lexicon_tags_1 },
    { 0x25228841, SUC_lexicon_tags_9 },
    { 0xbd828842, SUC_lexicon_tags_31 },
    { 0x96428844, SUC_lexicon_tags_2 },
    { 0x31228840, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c2884e, SUC_lexicon_tags_15 },
    { 0x67c2884f, SUC_lexicon_tags_7 },
    { 0xedc2884f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64e28852, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59028855, SUC_lexicon_tags_45 },
    { 0x7a228856, SUC_lexicon_tags_19 },
    { 0x13828857, SUC_lexicon_tags_15 },
    { 0x2b828858, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaae2885a, SUC_lexicon_tags_8 },
    { 0xf7a2885b, SUC_lexicon_tags_15 },
    { 0x26e2885b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5a2885e, SUC_lexicon_tags_12 },
    { 0x902885f, SUC_lexicon_tags_2 },
    { 0xb9028860, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a28865, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x84028868, SUC_lexicon_tags_25 },
    { 0xd0e28868, SUC_lexicon_tags_25 },
    { 0xbb02886a, SUC_lexicon_tags_2 },
    { 0xdc82886a, SUC_lexicon_tags_30 },
    { 0xe522886c, SUC_lexicon_tags_25 },
    { 0xb7e2886d, SUC_lexicon_tags_7 },
    { 0xd8e2886c, SUC_lexicon_tags_2 },
    { 0x55a2886f, SUC_lexicon_tags_29 },
    { 0xe6228870, SUC_lexicon_tags_14 },
    { 0x9d828871, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc9228873, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1628876, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xcc228878, SUC_lexicon_tags_1 },
    { 0x6a028878, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8182887c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3182887f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4628885, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e28889, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2222888b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9722888f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a28893, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff62889d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35a2889f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc288a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886288a7, SUC_lexicon_tags_15 },
    { 0x4e6288a8, SUC_lexicon_tags_26 },
    { 0xd6288a9, SUC_lexicon_tags_63 },
    { 0x736288a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6288af, SUC_lexicon_tags_84 },
    { 0x934288b0, SUC_lexicon_tags_17 },
    { 0xe9a288b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x28a288b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600288b7, SUC_lexicon_tags_25 },
    { 0x554288b8, SUC_lexicon_tags_27 },
    { 0xc3e288b7, SUC_lexicon_tags_77 },
    { 0x6fe288ba, SUC_lexicon_tags_29 },
    { 0x45a288bb, SUC_lexicon_tags_2 },
    { 0xcac288b7, SUC_lexicon_tags_2 },
    { 0xc9e288bd, SUC_lexicon_tags_37 },
    { 0xe14288bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x68288c0, SUC_lexicon_tags_34 },
    { 0xcf8288c1, SUC_lexicon_tags_19 },
    { 0xbfc288c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa288c6, SUC_lexicon_tags_35 },
    { 0x260288c6, SUC_lexicon_tags_37 },
    { 0xd16288c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba288cc, SUC_lexicon_tags_25 },
    { 0x23c288cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd5e288cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980288d9, SUC_lexicon_tags_25 },
    { 0x5aa288da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8288dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc288df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae0288e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a288e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x67a288e7, SUC_lexicon_tags_2 },
    { 0x2ee288e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34288eb, SUC_lexicon_tags_7 },
    { 0x372288ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x626288ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5e288f0, SUC_lexicon_tags_43 },
    { 0x438288f1, SUC_lexicon_tags_0 },
    { 0xc88288f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b2288f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb56288f6, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x6fa288f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4228901, SUC_lexicon_tags_38 },
    { 0x11428902, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25428906, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x57628909, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79028910, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92628913, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe562891f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a28926, SUC_lexicon_tags_15 },
    { 0x42828926, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c2892d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x71428930, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b828932, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8428939, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc2893c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd402893e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b428941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa28946, SUC_lexicon_tags_17 },
    { 0xb0828947, SUC_lexicon_tags_7 },
    { 0x57228948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b02894a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4602894c, SUC_lexicon_tags_12 },
    { 0xa622894d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5bc2894f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x40828951, SUC_lexicon_tags_31 },
    { 0x31828951, SUC_lexicon_tags_38 },
    { 0x94428953, SUC_lexicon_tags_1 },
    { 0xcdc28951, SUC_lexicon_tags_7 },
    { 0x9fe28953, SUC_lexicon_tags_17 },
    { 0x8a628955, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce2895b, SUC_lexicon_tags_25 },
    { 0xfd82895c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62895f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa8a28961, SUC_lexicon_tags_8 },
    { 0x84428962, SUC_lexicon_tags_320 },
    { 0xb3228963, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25228965, SUC_lexicon_tags_15 },
    { 0x86028966, SUC_lexicon_tags_11 },
    { 0x41428966, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a2896a, SUC_lexicon_tags_31 },
    { 0x5842896a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31228973, SUC_lexicon_tags_11 },
    { 0xb8828973, SUC_lexicon_tags_30 },
    { 0x82428974, SUC_lexicon_tags_7 },
    { 0x15428976, SUC_lexicon_tags_17 },
    { 0xfca28977, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de2897b, SUC_lexicon_tags_21 },
    { 0x18e2897c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e02897f, SUC_lexicon_tags_29 },
    { 0x5cc28980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5028982, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee428986, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x84028988, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b02898f, SUC_lexicon_tags_45 },
    { 0x3be28990, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e28997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7622899d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7f22899f, SUC_lexicon_tags_11 },
    { 0x18e2899f, SUC_lexicon_tags_56 },
    { 0xa9e2899f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142289a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x584289a9, SUC_lexicon_tags_38 },
    { 0x2e0289aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a289ad, SUC_lexicon_tags_60 },
    { 0x4fa289ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c289b3, SUC_lexicon_tags_17 },
    { 0x1e8289b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b0289b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x250289b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x140289bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c289be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b2289c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x31e289c2, SUC_lexicon_tags_15 },
    { 0xdfa289c3, SUC_lexicon_tags_25 },
    { 0x8c8289c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e289c8, SUC_lexicon_tags_19 },
    { 0x4d6289c9, SUC_lexicon_tags_1 },
    { 0x936289c9, SUC_lexicon_tags_48 },
    { 0x586289ca, SUC_lexicon_tags_49 },
    { 0x7e289c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e289d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8a289d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16289de, SUC_lexicon_tags_27 },
    { 0x5c6289de, SUC_lexicon_tags_21 },
    { 0x9d2289e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x216289e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92289e6, SUC_lexicon_tags_43 },
    { 0xe86289e7, SUC_lexicon_tags_66 },
    { 0x4f6289e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332289f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18289f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4289fb, SUC_lexicon_tags_34 },
    { 0x452289fb, SUC_lexicon_tags_8 },
    { 0xd08289fb, SUC_lexicon_tags_37 },
    { 0x25e289fe, SUC_lexicon_tags_37 },
    { 0xc48289ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c28a03, SUC_lexicon_tags_2 },
    { 0xe2628a04, SUC_lexicon_tags_12 },
    { 0xe3e28a05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2c628a07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a28a0c, SUC_lexicon_tags_12 },
    { 0x17228a0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e28a10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec28a14, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc3228a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff828a19, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11028a1e, SUC_lexicon_tags_12 },
    { 0x3b628a1f, SUC_lexicon_tags_37 },
    { 0xb7e28a20, SUC_lexicon_tags_13 },
    { 0x7ac28a1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe028a24, SUC_lexicon_tags_25 },
    { 0xf7828a25, SUC_lexicon_tags_37 },
    { 0x4d028a26, SUC_lexicon_tags_8 },
    { 0xd6828a27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa2c28a29, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae28a30, SUC_lexicon_tags_23 },
    { 0x87828a31, SUC_lexicon_tags_87 },
    { 0x70a28a31, SUC_lexicon_tags_15 },
    { 0x3b428a30, SUC_lexicon_tags_27 },
    { 0xa0228a34, SUC_lexicon_tags_11 },
    { 0xa2428a35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6228a37, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19028a3b, SUC_lexicon_tags_25 },
    { 0x8f628a3b, SUC_lexicon_tags_17 },
    { 0x2f628a3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9028a40, SUC_lexicon_tags_28 },
    { 0xefc28a41, SUC_lexicon_tags_65 },
    { 0xae228a42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7c28a44, SUC_lexicon_tags_25 },
    { 0xfd828a45, SUC_lexicon_tags_25 },
    { 0xa9828a46, SUC_lexicon_tags_60 },
    { 0xf4e28a47, SUC_lexicon_tags_1 },
    { 0x1ac28a46, SUC_lexicon_tags_12 },
    { 0x43e28a48, SUC_lexicon_tags_17 },
    { 0xb6828a4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e28a50, SUC_lexicon_tags_47 },
    { 0x5a28a51, SUC_lexicon_tags_49 },
    { 0xb5828a50, SUC_lexicon_tags_47 },
    { 0x9428a53, SUC_lexicon_tags_23 },
    { 0xc3a28a53, SUC_lexicon_tags_15 },
    { 0x7de28a53, SUC_lexicon_tags_55 },
    { 0x6fc28a56, SUC_lexicon_tags_23 },
    { 0x66c28a53, SUC_lexicon_tags_27 },
    { 0x87a28a58, SUC_lexicon_tags_23 },
    { 0xb4e28a59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14028a5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67a28a5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec28a62, SUC_lexicon_tags_27 },
    { 0x44228a63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c228a67, SUC_lexicon_tags_37 },
    { 0xd4428a68, SUC_lexicon_tags_11 },
    { 0x31428a69, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9028a6d, SUC_lexicon_tags_77 },
    { 0xbde28a6e, SUC_lexicon_tags_15 },
    { 0xed828a6e, SUC_lexicon_tags_26 },
    { 0x6e428a6f, SUC_lexicon_tags_13 },
    { 0xf428a71, SUC_lexicon_tags_1 },
    { 0xb4a28a72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9828a77, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2ee28a79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e28a7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd2a28a7e, SUC_lexicon_tags_17 },
    { 0x57028a7e, SUC_lexicon_tags_45 },
    { 0x1d228a7f, SUC_lexicon_tags_1 },
    { 0xca028a81, SUC_lexicon_tags_25 },
    { 0x1f828a82, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe3428a84, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a428a8c, SUC_lexicon_tags_11 },
    { 0x2b828a8d, SUC_lexicon_tags_14 },
    { 0x52828a8e, SUC_lexicon_tags_19 },
    { 0xaa028a8f, SUC_lexicon_tags_87 },
    { 0xb7428a90, SUC_lexicon_tags_30 },
    { 0xa4c28a91, SUC_lexicon_tags_47 },
    { 0x3fe28a91, SUC_lexicon_tags_21 },
    { 0x2628a90, SUC_lexicon_tags_30 },
    { 0x20228a8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e628a97, SUC_lexicon_tags_43 },
    { 0x92628a98, SUC_lexicon_tags_15 },
    { 0x4a028a99, SUC_lexicon_tags_17 },
    { 0x78028a9a, SUC_lexicon_tags_1 },
    { 0xa5028a98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a28a9d, SUC_lexicon_tags_17 },
    { 0xabe28a9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1628aa1, SUC_lexicon_tags_16 },
    { 0x7ec28aa1, SUC_lexicon_tags_11 },
    { 0x31a28aa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf828aa7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c628ab1, SUC_lexicon_tags_13 },
    { 0x26028ab1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e28ab7, SUC_lexicon_tags_70 },
    { 0xc6e28ab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae28abb, SUC_lexicon_tags_19 },
    { 0x24228abc, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6028ac4, SUC_lexicon_tags_19 },
    { 0x7bc28ac5, SUC_lexicon_tags_35 },
    { 0xb0828ac6, SUC_lexicon_tags_25 },
    { 0xbcc28ac6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98628acc, SUC_lexicon_tags_29 },
    { 0x56828acd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7228ad0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3028ad5, SUC_lexicon_tags_84 },
    { 0x7fc28ad6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92428ade, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69e28ae0, SUC_lexicon_tags_37 },
    { 0x4de28ae1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb428ae4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8228ae6, SUC_lexicon_tags_43 },
    { 0x36a28ae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61828aea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20e28aec, SUC_lexicon_tags_63 },
    { 0x51428aed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcf228aef, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe28af2, SUC_lexicon_tags_49 },
    { 0xf6628af3, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xac228af5, SUC_lexicon_tags_7 },
    { 0x83428af6, SUC_lexicon_tags_84 },
    { 0x7d028af7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8028afa, SUC_lexicon_tags_38 },
    { 0x4e028afb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf828b05, SUC_lexicon_tags_38 },
    { 0x40c28b06, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe28b0a, SUC_lexicon_tags_46 },
    { 0x2a28b0b, SUC_lexicon_tags_15 },
    { 0x20a28b0c, SUC_lexicon_tags_2 },
    { 0x21e28b0c, SUC_lexicon_tags_11 },
    { 0x5828b0d, SUC_lexicon_tags_27 },
    { 0x8fe28b0f, SUC_lexicon_tags_17 },
    { 0x29e28b10, SUC_lexicon_tags_84 },
    { 0x4a028b11, SUC_lexicon_tags_37 },
    { 0x6f228b0c, SUC_lexicon_tags_15 },
    { 0x32828b13, SUC_lexicon_tags_7 },
    { 0x3d028b14, SUC_lexicon_tags_12 },
    { 0x8fc28b15, SUC_lexicon_tags_27 },
    { 0x80828b16, SUC_lexicon_tags_33 },
    { 0xe0428b17, SUC_lexicon_tags_32 },
    { 0xf3c28b18, SUC_lexicon_tags_12 },
    { 0x82828b19, SUC_lexicon_tags_37 },
    { 0x66428b1a, SUC_lexicon_tags_6 },
    { 0x29028b1b, SUC_lexicon_tags_17 },
    { 0x51a28b1b, SUC_lexicon_tags_118 },
    { 0xcd428b1c, SUC_lexicon_tags_9 },
    { 0x8ac28b16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4628b22, SUC_lexicon_tags_47 },
    { 0x89a28b23, SUC_lexicon_tags_0 },
    { 0x32c28b22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b828b2c, SUC_lexicon_tags_30 },
    { 0x4b228b2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f228b30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c28b33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1428b36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec428b38, SUC_lexicon_tags_7 },
    { 0x45828b38, SUC_lexicon_tags_85 },
    { 0xf7e28b39, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbfc28b3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e28b40, SUC_lexicon_tags_37 },
    { 0x28428b41, SUC_lexicon_tags_9 },
    { 0x18a28b40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae28b48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7828b4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e28b54, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a28b58, SUC_lexicon_tags_84 },
    { 0x65828b59, SUC_lexicon_tags_17 },
    { 0xdde28b59, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f028b5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7228b61, SUC_lexicon_tags_49 },
    { 0xd7a28b62, SUC_lexicon_tags_2 },
    { 0xfb828b63, SUC_lexicon_tags_63 },
    { 0x2a828b64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea28b6a, SUC_lexicon_tags_0 },
    { 0x24a28b6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3828b6d, SUC_lexicon_tags_11 },
    { 0xaba28b6e, SUC_lexicon_tags_43 },
    { 0x94628b6d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30628b74, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba28b79, SUC_lexicon_tags_37 },
    { 0x8ce28b7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3828b7e, SUC_lexicon_tags_25 },
    { 0xac628b7f, SUC_lexicon_tags_35 },
    { 0xfa228b80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e28b83, SUC_lexicon_tags_37 },
    { 0x4be28b84, SUC_lexicon_tags_23 },
    { 0xf9e28b85, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x83a28b87, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97628b8e, SUC_lexicon_tags_60 },
    { 0xf2428b8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a28b92, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c28b9b, SUC_lexicon_tags_17 },
    { 0xbac28b9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb828ba0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3228baa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5e28bac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8428bb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a028bb3, SUC_lexicon_tags_1 },
    { 0x30e28bb4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec28bbb, SUC_lexicon_tags_136 },
    { 0x59028bbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae28bc0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c628bc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88c28bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48828bd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e28bd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1028bd7, SUC_lexicon_tags_15 },
    { 0x1a28bd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0028bda, SUC_lexicon_tags_40 },
    { 0x4b228bda, SUC_lexicon_tags_12 },
    { 0x78628bda, SUC_lexicon_tags_2 },
    { 0x80228bdb, SUC_lexicon_tags_65 },
    { 0x41e28bde, SUC_lexicon_tags_0 },
    { 0x12c28bda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c28be3, SUC_lexicon_tags_48 },
    { 0xc9028be3, SUC_lexicon_tags_37 },
    { 0xa8228be5, SUC_lexicon_tags_30 },
    { 0x3d028be3, SUC_lexicon_tags_25 },
    { 0xea228be6, SUC_lexicon_tags_30 },
    { 0x5de28be8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7028bed, SUC_lexicon_tags_12 },
    { 0x2c828bee, SUC_lexicon_tags_17 },
    { 0x56a28bef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa4228bf1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9228bf4, SUC_lexicon_tags_47 },
    { 0x43628bf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46228bf8, SUC_lexicon_tags_30 },
    { 0xf1828bf8, SUC_lexicon_tags_1 },
    { 0x4e028bf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a028bfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8228c02, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x24c28c04, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b628c07, SUC_lexicon_tags_19 },
    { 0x7d028c08, SUC_lexicon_tags_63 },
    { 0x79228c08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce28c0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51228c11, SUC_lexicon_tags_12 },
    { 0xc4e28c12, SUC_lexicon_tags_38 },
    { 0xa2a28c11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee028c15, SUC_lexicon_tags_10 },
    { 0x7b428c16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1028c1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca28c20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62228c22, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x82428c24, SUC_lexicon_tags_37 },
    { 0x79228c25, SUC_lexicon_tags_25 },
    { 0xc6c28c26, SUC_lexicon_tags_7 },
    { 0x25428c27, SUC_lexicon_tags_25 },
    { 0x40c28c26, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79028c2f, SUC_lexicon_tags_66 },
    { 0xec828c2f, SUC_lexicon_tags_15 },
    { 0x24228c31, SUC_lexicon_tags_12 },
    { 0x7bc28c32, SUC_lexicon_tags_15 },
    { 0xb2828c33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a28c37, SUC_lexicon_tags_17 },
    { 0xbb028c37, SUC_lexicon_tags_60 },
    { 0xd9828c39, SUC_lexicon_tags_32 },
    { 0xc5828c3a, SUC_lexicon_tags_31 },
    { 0x3e428c3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5228c3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37028c44, SUC_lexicon_tags_9 },
    { 0x9da28c45, SUC_lexicon_tags_15 },
    { 0xfbc28c45, SUC_lexicon_tags_11 },
    { 0x1f628c47, SUC_lexicon_tags_37 },
    { 0x76828c47, SUC_lexicon_tags_9 },
    { 0x4ba28c49, SUC_lexicon_tags_2 },
    { 0xefc28c4a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe6028c4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c28c50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70628c54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e28c57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8828c5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa28c60, SUC_lexicon_tags_45 },
    { 0x6bc28c60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd4c28c63, SUC_lexicon_tags_7 },
    { 0x20a28c64, SUC_lexicon_tags_23 },
    { 0x8ac28c65, SUC_lexicon_tags_3 },
    { 0x41e28c66, SUC_lexicon_tags_12 },
    { 0xb3428c67, SUC_lexicon_tags_12 },
    { 0x4a28c68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83028c6c, SUC_lexicon_tags_56 },
    { 0xa3628c6d, SUC_lexicon_tags_60 },
    { 0xd2628c6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5628c70, SUC_lexicon_tags_19 },
    { 0xe0828c70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228c75, SUC_lexicon_tags_1 },
    { 0x81a28c76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8428c7d, SUC_lexicon_tags_25 },
    { 0x69428c7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac28c83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb028c86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a428c89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52228c8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcae28c8d, SUC_lexicon_tags_6 },
    { 0xb0628c8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d228c92, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f028c98, SUC_lexicon_tags_49 },
    { 0x63028c99, SUC_lexicon_tags_17 },
    { 0xca828c9a, SUC_lexicon_tags_63 },
    { 0xf5028c9b, SUC_lexicon_tags_21 },
    { 0x9ce28c9c, SUC_lexicon_tags_7 },
    { 0x35428c9d, SUC_lexicon_tags_17 },
    { 0x1ba28c9d, SUC_lexicon_tags_12 },
    { 0x5ce28c9b, SUC_lexicon_tags_24 },
    { 0xa3e28ca0, SUC_lexicon_tags_11 },
    { 0xc3a28ca1, SUC_lexicon_tags_87 },
    { 0x8ae28ca2, SUC_lexicon_tags_37 },
    { 0x4f028ca1, SUC_lexicon_tags_17 },
    { 0xf5028ca0, SUC_lexicon_tags_1 },
    { 0x8f028ca4, SUC_lexicon_tags_19 },
    { 0xe0628ca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7028ca9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xd3028cab, SUC_lexicon_tags_15 },
    { 0xa7e28cab, SUC_lexicon_tags_14 },
    { 0xc5c28cac, SUC_lexicon_tags_107 },
    { 0xea428cae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x12028cb1, SUC_lexicon_tags_1 },
    { 0x44628cb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x60828cb4, SUC_lexicon_tags_48 },
    { 0x50428cb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c628cb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabe28cbb, SUC_lexicon_tags_15 },
    { 0xfa028cbc, SUC_lexicon_tags_27 },
    { 0xa628cbb, SUC_lexicon_tags_28 },
    { 0x52828cbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1828cc0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb2a28cc2, SUC_lexicon_tags_77 },
    { 0x7dc28cc2, SUC_lexicon_tags_11 },
    { 0x3f228cc3, SUC_lexicon_tags_59 },
    { 0xc0628cc4, SUC_lexicon_tags_34 },
    { 0x70828cc6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x94628cc9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x71428ccb, SUC_lexicon_tags_31 },
    { 0xaa828ccc, SUC_lexicon_tags_15 },
    { 0x9d228ccd, SUC_lexicon_tags_47 },
    { 0xcce28cce, SUC_lexicon_tags_28 },
    { 0x97e28ccd, SUC_lexicon_tags_9 },
    { 0xb4c28cd0, SUC_lexicon_tags_15 },
    { 0x26c28ccd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87828cd4, SUC_lexicon_tags_1 },
    { 0xb9e28cd5, SUC_lexicon_tags_36 },
    { 0x4aa28cd6, SUC_lexicon_tags_27 },
    { 0xdae28cd7, SUC_lexicon_tags_38 },
    { 0x3e628cd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64428cdd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee28ce2, SUC_lexicon_tags_45 },
    { 0x33c28ce3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e28ce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40228ceb, SUC_lexicon_tags_11 },
    { 0x5c628cec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30228cf2, SUC_lexicon_tags_7 },
    { 0x39628cf3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xea828cf5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a28cf8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c28cfd, SUC_lexicon_tags_38 },
    { 0xd4628cfe, SUC_lexicon_tags_15 },
    { 0x2a228cff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80228d03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7428d05, SUC_lexicon_tags_1678 },
    { 0, NULL },
    { 0x50028d07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86028d09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c428d0c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56428d10, SUC_lexicon_tags_15 },
    { 0xa7028d11, SUC_lexicon_tags_29 },
    { 0xbc028d12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c28d16, SUC_lexicon_tags_17 },
    { 0x94628d17, SUC_lexicon_tags_16 },
    { 0x9a428d17, SUC_lexicon_tags_37 },
    { 0xbb428d19, SUC_lexicon_tags_12 },
    { 0xe2a28d19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf028d1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68828d20, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x22828d23, SUC_lexicon_tags_11 },
    { 0x52028d24, SUC_lexicon_tags_37 },
    { 0xd1c28d23, SUC_lexicon_tags_8 },
    { 0x38428d23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed428d30, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a28d35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea28d3c, SUC_lexicon_tags_11 },
    { 0xb8a28d3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc028d42, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e28d45, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x27428d47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa28d4b, SUC_lexicon_tags_45 },
    { 0x7d828d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5228d4e, SUC_lexicon_tags_65 },
    { 0x90628d4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ce28d51, SUC_lexicon_tags_37 },
    { 0x8a228d52, SUC_lexicon_tags_48 },
    { 0x9da28d53, SUC_lexicon_tags_47 },
    { 0x32e28d54, SUC_lexicon_tags_14 },
    { 0x6e828d54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c28d5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2ce28d5c, SUC_lexicon_tags_29 },
    { 0xae28d5c, SUC_lexicon_tags_43 },
    { 0xbc28d5e, SUC_lexicon_tags_2 },
    { 0xc6828d5c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x90428d61, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x828d64, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab228d66, SUC_lexicon_tags_30 },
    { 0x3428d66, SUC_lexicon_tags_33 },
    { 0x5a428d68, SUC_lexicon_tags_1 },
    { 0x91628d66, SUC_lexicon_tags_12 },
    { 0x4028d6a, SUC_lexicon_tags_25 },
    { 0xd6228d6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7e28d6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaa28d6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9428d72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xada28d75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91428d7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55628d7d, SUC_lexicon_tags_22 },
    { 0x35c28d7d, SUC_lexicon_tags_49 },
    { 0x7428d7f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x73628d81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc28d87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da28d8b, SUC_lexicon_tags_25 },
    { 0xbb628d8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7228d8e, SUC_lexicon_tags_15 },
    { 0x71828d8f, SUC_lexicon_tags_2 },
    { 0x82c28d8f, SUC_lexicon_tags_25 },
    { 0x428d91, SUC_lexicon_tags_47 },
    { 0x28228d92, SUC_lexicon_tags_30 },
    { 0xa5628d92, SUC_lexicon_tags_46 },
    { 0x30428d8e, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a28da0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb428da2, SUC_lexicon_tags_30 },
    { 0xb4a28da3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c28da6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea28da9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99028dab, SUC_lexicon_tags_7 },
    { 0x10828dac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae28db0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c28db9, SUC_lexicon_tags_15 },
    { 0x27028dba, SUC_lexicon_tags_27 },
    { 0x40428db9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c28dc4, SUC_lexicon_tags_23 },
    { 0xc7c28dc5, SUC_lexicon_tags_7 },
    { 0x9fa28dc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28428dc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c28dcb, SUC_lexicon_tags_25 },
    { 0xcf228dcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8428dcf, SUC_lexicon_tags_25 },
    { 0x64828dd0, SUC_lexicon_tags_2 },
    { 0x10428dcf, SUC_lexicon_tags_23 },
    { 0x31c28dd1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xca028dd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a28dd9, SUC_lexicon_tags_31 },
    { 0x3ea28dda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ea28ddc, SUC_lexicon_tags_12 },
    { 0x9ca28ddd, SUC_lexicon_tags_19 },
    { 0xa7028dde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5e628de0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a28de3, SUC_lexicon_tags_10 },
    { 0xa3a28de4, SUC_lexicon_tags_63 },
    { 0xe8028de5, SUC_lexicon_tags_12 },
    { 0x42628de6, SUC_lexicon_tags_17 },
    { 0x33828de7, SUC_lexicon_tags_4 },
    { 0xe5228de8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e28deb, SUC_lexicon_tags_7 },
    { 0x77a28deb, SUC_lexicon_tags_22 },
    { 0x20a28dec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23228def, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6628df2, SUC_lexicon_tags_61 },
    { 0xbfe28df2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc028df5, SUC_lexicon_tags_7 },
    { 0x34228df6, SUC_lexicon_tags_12 },
    { 0x2e028df5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28228dfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd628e06, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc28e0a, SUC_lexicon_tags_25 },
    { 0xa8028e0b, SUC_lexicon_tags_25 },
    { 0xa2228e0c, SUC_lexicon_tags_28 },
    { 0x4ec28e0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26228e12, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa028e14, SUC_lexicon_tags_12 },
    { 0x14628e15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c28e17, SUC_lexicon_tags_55 },
    { 0xa9828e17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b428e1b, SUC_lexicon_tags_15 },
    { 0x9ec28e1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c28e25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4428e27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c28e2a, SUC_lexicon_tags_31 },
    { 0xb9c28e2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb828e2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x30428e30, SUC_lexicon_tags_57 },
    { 0x6c628e31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33228e37, SUC_lexicon_tags_0 },
    { 0xcd628e38, SUC_lexicon_tags_19 },
    { 0xede28e38, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c28e3e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c28e43, SUC_lexicon_tags_0 },
    { 0x25628e44, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec28e4a, SUC_lexicon_tags_37 },
    { 0x47a28e4b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f628e4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64a28e51, SUC_lexicon_tags_25 },
    { 0x3e828e52, SUC_lexicon_tags_26 },
    { 0x7c228e52, SUC_lexicon_tags_25 },
    { 0x5628e51, SUC_lexicon_tags_2 },
    { 0x13028e55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e028e5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23228e62, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d428e67, SUC_lexicon_tags_201 },
    { 0xa8028e68, SUC_lexicon_tags_91 },
    { 0xfe628e68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda828e6c, SUC_lexicon_tags_15 },
    { 0x92628e6d, SUC_lexicon_tags_47 },
    { 0x19028e6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc828e71, SUC_lexicon_tags_19 },
    { 0x2e628e72, SUC_lexicon_tags_38 },
    { 0xb1628e72, SUC_lexicon_tags_26 },
    { 0xfec28e72, SUC_lexicon_tags_33 },
    { 0x9be28e75, SUC_lexicon_tags_23 },
    { 0x96828e75, SUC_lexicon_tags_12 },
    { 0x60628e75, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a28e7c, SUC_lexicon_tags_15 },
    { 0x3a228e7c, SUC_lexicon_tags_47 },
    { 0x79a28e7e, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xa4028e80, SUC_lexicon_tags_19 },
    { 0xd428e81, SUC_lexicon_tags_46 },
    { 0xde228e82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde628e86, SUC_lexicon_tags_38 },
    { 0xc9428e87, SUC_lexicon_tags_39 },
    { 0x10228e87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2e28e8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7ce28e8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73228e8e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1028e91, SUC_lexicon_tags_23 },
    { 0xa8428e92, SUC_lexicon_tags_2 },
    { 0x6f828e93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82028e96, SUC_lexicon_tags_11 },
    { 0x31c28e97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0428e9a, SUC_lexicon_tags_31 },
    { 0xcc028e9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa28e9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7a28e9f, SUC_lexicon_tags_19 },
    { 0xf8028ea0, SUC_lexicon_tags_84 },
    { 0x49228ea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0428ea5, SUC_lexicon_tags_60 },
    { 0x9de28ea6, SUC_lexicon_tags_11 },
    { 0xe0828ea7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86628ea9, SUC_lexicon_tags_47 },
    { 0xbd228eaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa28eaf, SUC_lexicon_tags_12 },
    { 0xe6828eaf, SUC_lexicon_tags_17 },
    { 0xe9628eb1, SUC_lexicon_tags_12 },
    { 0x2f228eb2, SUC_lexicon_tags_7 },
    { 0x7de28eaf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e28eb7, SUC_lexicon_tags_0 },
    { 0xe7028eb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce28ebb, SUC_lexicon_tags_12 },
    { 0x5ec28ebb, SUC_lexicon_tags_37 },
    { 0xc9828ebd, SUC_lexicon_tags_12 },
    { 0x5a228ebe, SUC_lexicon_tags_11 },
    { 0xabc28ebe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a28ec6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82828eca, SUC_lexicon_tags_38 },
    { 0x90a28ecb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdb028ecd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6628ed0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a28ed4, SUC_lexicon_tags_15 },
    { 0x9be28ed5, SUC_lexicon_tags_37 },
    { 0xa8a28ed4, SUC_lexicon_tags_11 },
    { 0x7628ed5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe28edd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3a28edf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7228ee4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a228ee6, SUC_lexicon_tags_46 },
    { 0xc3628ee7, SUC_lexicon_tags_11 },
    { 0x88a28ee8, SUC_lexicon_tags_37 },
    { 0x95e28ee7, SUC_lexicon_tags_37 },
    { 0x7628eea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec28eef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8228ef3, SUC_lexicon_tags_43 },
    { 0x29628ef4, SUC_lexicon_tags_27 },
    { 0x8e628ef5, SUC_lexicon_tags_63 },
    { 0x4b428ef6, SUC_lexicon_tags_2 },
    { 0xc4828ef7, SUC_lexicon_tags_49 },
    { 0x51e28ef8, SUC_lexicon_tags_0 },
    { 0x63028ef9, SUC_lexicon_tags_25 },
    { 0xf1828ef9, SUC_lexicon_tags_19 },
    { 0x9d428efa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7628efe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a828f01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4c28f03, SUC_lexicon_tags_12 },
    { 0xe4828f04, SUC_lexicon_tags_63 },
    { 0xdb228f04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40228f08, SUC_lexicon_tags_49 },
    { 0x4c28f09, SUC_lexicon_tags_11 },
    { 0x5f028f09, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee828f10, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e28f13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa28f17, SUC_lexicon_tags_1 },
    { 0x71428f17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e28f1b, SUC_lexicon_tags_37 },
    { 0x14028f1c, SUC_lexicon_tags_21 },
    { 0xb5228f1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda28f22, SUC_lexicon_tags_9 },
    { 0x4bc28f23, SUC_lexicon_tags_15 },
    { 0xc1e28f22, SUC_lexicon_tags_4 },
    { 0x85028f24, SUC_lexicon_tags_15 },
    { 0xa1228f26, SUC_lexicon_tags_11 },
    { 0xabc28f24, SUC_lexicon_tags_17 },
    { 0x20228f28, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a228f2c, SUC_lexicon_tags_12 },
    { 0x9ac28f2d, SUC_lexicon_tags_25 },
    { 0xc1628f2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65828f30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c28f38, SUC_lexicon_tags_1 },
    { 0xff828f38, SUC_lexicon_tags_18 },
    { 0x5a428f3a, SUC_lexicon_tags_28 },
    { 0x69e28f3b, SUC_lexicon_tags_39 },
    { 0xd6428f3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdce28f3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4c28f40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93e28f42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a28f47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a28f4b, SUC_lexicon_tags_0 },
    { 0x7e028f4c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x11228f4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x58e28f51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfd028f53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab228f58, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x57e28f5a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd9828f5c, SUC_lexicon_tags_61 },
    { 0xd7828f5d, SUC_lexicon_tags_12 },
    { 0xf0828f5e, SUC_lexicon_tags_12 },
    { 0x5a828f5c, SUC_lexicon_tags_15 },
    { 0x35228f60, SUC_lexicon_tags_63 },
    { 0x5ce28f5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a28f65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad228f67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb428f70, SUC_lexicon_tags_25 },
    { 0xa3a28f71, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdba28f73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c428f7b, SUC_lexicon_tags_17 },
    { 0x82628f7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7028f7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88028f87, SUC_lexicon_tags_38 },
    { 0xb2c28f87, SUC_lexicon_tags_12 },
    { 0x8f228f87, SUC_lexicon_tags_1 },
    { 0x89228f8a, SUC_lexicon_tags_28 },
    { 0xcb828f8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x228f8e, SUC_lexicon_tags_10 },
    { 0xf828f8f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x48a28f91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39628f93, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xd7c28f95, SUC_lexicon_tags_7 },
    { 0xd8e28f95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18828f99, SUC_lexicon_tags_45 },
    { 0x7da28f9a, SUC_lexicon_tags_1 },
    { 0xf8628f9b, SUC_lexicon_tags_5 },
    { 0xc2428f9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e28fa3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x85a28fa5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828fae, SUC_lexicon_tags_63 },
    { 0xf6e28faf, SUC_lexicon_tags_7 },
    { 0x91228fb0, SUC_lexicon_tags_37 },
    { 0x50a28fb1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c28fb6, SUC_lexicon_tags_13 },
    { 0x83428fb7, SUC_lexicon_tags_55 },
    { 0xd2028fb7, SUC_lexicon_tags_33 },
    { 0xf428fb7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58428fc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e28fc9, SUC_lexicon_tags_28 },
    { 0xeea28fc9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e28fcd, SUC_lexicon_tags_25 },
    { 0x29c28fce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b628fd1, SUC_lexicon_tags_14 },
    { 0x1a28fd2, SUC_lexicon_tags_61 },
    { 0x1b028fd3, SUC_lexicon_tags_30 },
    { 0x93628fd4, SUC_lexicon_tags_17 },
    { 0x1a228fd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53a28fd7, SUC_lexicon_tags_37 },
    { 0xef628fd7, SUC_lexicon_tags_25 },
    { 0x20228fd8, SUC_lexicon_tags_7 },
    { 0x8bc28fda, SUC_lexicon_tags_39 },
    { 0x5ec28fda, SUC_lexicon_tags_49 },
    { 0x44a28fd7, SUC_lexicon_tags_23 },
    { 0xd8e28fdd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6fa28fdf, SUC_lexicon_tags_26 },
    { 0x51a28fdf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x88428fe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c828fec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1e28fee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0228ff4, SUC_lexicon_tags_82 },
    { 0xc028ff5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9628ffb, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xda628ffd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2628fff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c229003, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e29009, SUC_lexicon_tags_127 },
    { 0xd5c2900a, SUC_lexicon_tags_17 },
    { 0x1362900b, SUC_lexicon_tags_37 },
    { 0xbfe2900b, SUC_lexicon_tags_75 },
    { 0xc182900d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae29010, SUC_lexicon_tags_15 },
    { 0x14429010, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47829018, SUC_lexicon_tags_55 },
    { 0x86c29019, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b42901d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28229020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca629022, SUC_lexicon_tags_7 },
    { 0xc7e29022, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff029025, SUC_lexicon_tags_77 },
    { 0xe3829025, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee2902b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c42902e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a82903c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c2903f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x73a29041, SUC_lexicon_tags_17 },
    { 0x3f829042, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x79829044, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf0029046, SUC_lexicon_tags_27 },
    { 0xff229047, SUC_lexicon_tags_1 },
    { 0xb2c29048, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc2904e, SUC_lexicon_tags_43 },
    { 0x7902904f, SUC_lexicon_tags_17 },
    { 0x8b82904e, SUC_lexicon_tags_2 },
    { 0xffc29051, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1f629053, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a29060, SUC_lexicon_tags_30 },
    { 0x31e29061, SUC_lexicon_tags_62 },
    { 0x74629061, SUC_lexicon_tags_158 },
    { 0x7ac29063, SUC_lexicon_tags_7 },
    { 0xd429064, SUC_lexicon_tags_63 },
    { 0x5de29063, SUC_lexicon_tags_27 },
    { 0x2dc29063, SUC_lexicon_tags_107 },
    { 0x4ba29067, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22906e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8429071, SUC_lexicon_tags_14 },
    { 0xe8029071, SUC_lexicon_tags_17 },
    { 0x71029073, SUC_lexicon_tags_14 },
    { 0x4be29074, SUC_lexicon_tags_118 },
    { 0x56e29072, SUC_lexicon_tags_6 },
    { 0x5b029076, SUC_lexicon_tags_10 },
    { 0x45229076, SUC_lexicon_tags_2 },
    { 0x7ce29078, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd682907b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4fe2907d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e29080, SUC_lexicon_tags_43 },
    { 0x2fa29080, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x56229084, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xc9029086, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2302908c, SUC_lexicon_tags_47 },
    { 0xee02908d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1622908f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3a29091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a29096, SUC_lexicon_tags_17 },
    { 0xb6429097, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc842909a, SUC_lexicon_tags_33 },
    { 0xbc22909b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62290a0, SUC_lexicon_tags_61 },
    { 0x972290a1, SUC_lexicon_tags_37 },
    { 0xa3a290a0, SUC_lexicon_tags_7 },
    { 0x57e290a2, SUC_lexicon_tags_4 },
    { 0x8e0290a4, SUC_lexicon_tags_47 },
    { 0xd8a290a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e290a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a290ab, SUC_lexicon_tags_2 },
    { 0x5aa290ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c290b0, SUC_lexicon_tags_37 },
    { 0x4ea290b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50290b8, SUC_lexicon_tags_23 },
    { 0xfb8290b9, SUC_lexicon_tags_49 },
    { 0x10c290ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e290bf, SUC_lexicon_tags_8 },
    { 0x9b0290c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa290c3, SUC_lexicon_tags_15 },
    { 0xdb0290c3, SUC_lexicon_tags_12 },
    { 0xe96290c3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xe54290c7, SUC_lexicon_tags_28 },
    { 0x670290c8, SUC_lexicon_tags_43 },
    { 0xe1c290c8, SUC_lexicon_tags_27 },
    { 0x370290ca, SUC_lexicon_tags_65 },
    { 0xd3c290cb, SUC_lexicon_tags_1 },
    { 0xd2c290cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x460290cf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a290d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a290d9, SUC_lexicon_tags_3 },
    { 0x41a290da, SUC_lexicon_tags_37 },
    { 0x89a290db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e290de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a290e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc34290e6, SUC_lexicon_tags_7 },
    { 0xb64290e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a290ef, SUC_lexicon_tags_2 },
    { 0x7c0290ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc290f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546290f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd56290fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46290fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xab229100, SUC_lexicon_tags_27 },
    { 0xf1229100, SUC_lexicon_tags_7 },
    { 0xf8029102, SUC_lexicon_tags_11 },
    { 0x58a29101, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa429112, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc29115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe222911b, SUC_lexicon_tags_47 },
    { 0x902911c, SUC_lexicon_tags_2 },
    { 0x3662911d, SUC_lexicon_tags_27 },
    { 0xbd42911e, SUC_lexicon_tags_133 },
    { 0x8902911f, SUC_lexicon_tags_21 },
    { 0xa802911c, SUC_lexicon_tags_15 },
    { 0xb0e29120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b229128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb62912c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f029130, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa29133, SUC_lexicon_tags_27 },
    { 0x36829134, SUC_lexicon_tags_34 },
    { 0x95629135, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c29138, SUC_lexicon_tags_25 },
    { 0xd8229138, SUC_lexicon_tags_1500 },
    { 0xb1229139, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3442913d, SUC_lexicon_tags_60 },
    { 0x6902913d, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc429141, SUC_lexicon_tags_7 },
    { 0x5a429142, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282914b, SUC_lexicon_tags_1 },
    { 0x4f42914c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8462914e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a29153, SUC_lexicon_tags_39 },
    { 0x29a29154, SUC_lexicon_tags_17 },
    { 0x9f629154, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a2915b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e2915e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9429162, SUC_lexicon_tags_15 },
    { 0xed029163, SUC_lexicon_tags_11 },
    { 0x40829164, SUC_lexicon_tags_11 },
    { 0xf6a29163, SUC_lexicon_tags_2 },
    { 0xe29163, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122916a, SUC_lexicon_tags_43 },
    { 0x2fe2916b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c2916e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc29173, SUC_lexicon_tags_9 },
    { 0xf4e29173, SUC_lexicon_tags_4 },
    { 0x5c629175, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed22917f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e29183, SUC_lexicon_tags_19 },
    { 0x6d829184, SUC_lexicon_tags_17 },
    { 0x32629184, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79429187, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a229189, SUC_lexicon_tags_63 },
    { 0xf1429189, SUC_lexicon_tags_38 },
    { 0xa822918b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ac2918d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39229190, SUC_lexicon_tags_65 },
    { 0x37629190, SUC_lexicon_tags_37 },
    { 0xd8229192, SUC_lexicon_tags_31 },
    { 0x72829193, SUC_lexicon_tags_63 },
    { 0x4dc29194, SUC_lexicon_tags_15 },
    { 0xe4429193, SUC_lexicon_tags_51 },
    { 0x3d629196, SUC_lexicon_tags_1 },
    { 0xc4e29196, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc7829199, SUC_lexicon_tags_57 },
    { 0x4282919a, SUC_lexicon_tags_1 },
    { 0xe2a2919a, SUC_lexicon_tags_15 },
    { 0xfd829199, SUC_lexicon_tags_38 },
    { 0xb102919d, SUC_lexicon_tags_15 },
    { 0x9b02919e, SUC_lexicon_tags_29 },
    { 0x4282919f, SUC_lexicon_tags_39 },
    { 0x886291a0, SUC_lexicon_tags_45 },
    { 0xb28291a1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x1ea291a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x984291a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4291aa, SUC_lexicon_tags_17 },
    { 0xd46291aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe291ae, SUC_lexicon_tags_23 },
    { 0x9c291af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92291b2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6291b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x772291ba, SUC_lexicon_tags_17 },
    { 0x912291bb, SUC_lexicon_tags_49 },
    { 0xcd4291bc, SUC_lexicon_tags_7 },
    { 0x3fa291bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80291c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7da291c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88291ca, SUC_lexicon_tags_48 },
    { 0xbae291cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8291ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x394291d0, SUC_lexicon_tags_7 },
    { 0xdee291d1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52291d4, SUC_lexicon_tags_47 },
    { 0xde4291d5, SUC_lexicon_tags_15 },
    { 0xff2291d6, SUC_lexicon_tags_37 },
    { 0xc20291d6, SUC_lexicon_tags_37 },
    { 0x934291d7, SUC_lexicon_tags_2 },
    { 0x9a2291d9, SUC_lexicon_tags_19 },
    { 0xdc2291da, SUC_lexicon_tags_1 },
    { 0x22c291d7, SUC_lexicon_tags_25 },
    { 0x242291dc, SUC_lexicon_tags_11 },
    { 0x742291dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de291e1, SUC_lexicon_tags_13 },
    { 0x35e291e2, SUC_lexicon_tags_48 },
    { 0xbf0291e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x89a291e5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x90291e7, SUC_lexicon_tags_15 },
    { 0x47a291e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x950291ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x104291ec, SUC_lexicon_tags_25 },
    { 0x60a291ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee291f3, SUC_lexicon_tags_0 },
    { 0xc9a291f4, SUC_lexicon_tags_11 },
    { 0xd84291f5, SUC_lexicon_tags_29 },
    { 0xfba291f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4aa291f8, SUC_lexicon_tags_25 },
    { 0x89a291f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0291fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb28291fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7629200, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x38829202, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8429204, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30029206, SUC_lexicon_tags_49 },
    { 0x54629207, SUC_lexicon_tags_68 },
    { 0xe429207, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd422920c, SUC_lexicon_tags_0 },
    { 0xa522920d, SUC_lexicon_tags_9 },
    { 0xee2920e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x83829210, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51429214, SUC_lexicon_tags_15 },
    { 0x56829214, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76829218, SUC_lexicon_tags_13 },
    { 0x61629218, SUC_lexicon_tags_11 },
    { 0x1aa2921a, SUC_lexicon_tags_1 },
    { 0x41a2921b, SUC_lexicon_tags_43 },
    { 0xe242921c, SUC_lexicon_tags_47 },
    { 0x3762921d, SUC_lexicon_tags_47 },
    { 0xb3e2921d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae29223, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e29228, SUC_lexicon_tags_25 },
    { 0xffe29228, SUC_lexicon_tags_11 },
    { 0x8b429229, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e29232, SUC_lexicon_tags_11 },
    { 0xada29233, SUC_lexicon_tags_7 },
    { 0x1ea29232, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa2029236, SUC_lexicon_tags_17 },
    { 0x67829236, SUC_lexicon_tags_8 },
    { 0x41829238, SUC_lexicon_tags_63 },
    { 0x95229236, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2d62923b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec29243, SUC_lexicon_tags_45 },
    { 0xfa29244, SUC_lexicon_tags_25 },
    { 0x6c229245, SUC_lexicon_tags_27 },
    { 0x39429246, SUC_lexicon_tags_43 },
    { 0xdc29247, SUC_lexicon_tags_12 },
    { 0x34229243, SUC_lexicon_tags_17 },
    { 0x37a29243, SUC_lexicon_tags_7 },
    { 0x6782924a, SUC_lexicon_tags_66 },
    { 0x15a2924b, SUC_lexicon_tags_8 },
    { 0x5382924b, SUC_lexicon_tags_26 },
    { 0xb62924d, SUC_lexicon_tags_15 },
    { 0xefe2924a, SUC_lexicon_tags_17 },
    { 0x2462924f, SUC_lexicon_tags_25 },
    { 0x3c22924c, SUC_lexicon_tags_7 },
    { 0x1162924f, SUC_lexicon_tags_436 },
    { 0xeb229251, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d629254, SUC_lexicon_tags_23 },
    { 0x73029255, SUC_lexicon_tags_65 },
    { 0x18e29256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a22925a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be2925d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb429264, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf429267, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd229269, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e2926b, SUC_lexicon_tags_17 },
    { 0x6802926c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c2926f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfda29271, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46a29273, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e29276, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a2927a, SUC_lexicon_tags_17 },
    { 0xf362927a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd622927e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16829282, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7029284, SUC_lexicon_tags_80 },
    { 0x20629284, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x28229287, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xee82928a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x982928d, SUC_lexicon_tags_38 },
    { 0x91c2928e, SUC_lexicon_tags_15 },
    { 0x1f82928e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5829291, SUC_lexicon_tags_0 },
    { 0x28829292, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11829295, SUC_lexicon_tags_5 },
    { 0x57a29295, SUC_lexicon_tags_7 },
    { 0x36229297, SUC_lexicon_tags_43 },
    { 0x15e29298, SUC_lexicon_tags_7 },
    { 0xa5c29298, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522929d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e292a0, SUC_lexicon_tags_13 },
    { 0x11e292a1, SUC_lexicon_tags_12 },
    { 0xf40292a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54292a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x652292aa, SUC_lexicon_tags_57 },
    { 0xae4292ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc24292ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc292b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788292b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72292ba, SUC_lexicon_tags_13 },
    { 0xed8292bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58292be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ba292c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbcc292c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a292c6, SUC_lexicon_tags_68 },
    { 0xafe292c7, SUC_lexicon_tags_25 },
    { 0xa6292c8, SUC_lexicon_tags_7 },
    { 0xabc292c9, SUC_lexicon_tags_11 },
    { 0xb78292c9, SUC_lexicon_tags_17 },
    { 0x894292c7, SUC_lexicon_tags_17 },
    { 0x416292cc, SUC_lexicon_tags_1 },
    { 0xfac292cd, SUC_lexicon_tags_55 },
    { 0xf76292ce, SUC_lexicon_tags_11 },
    { 0xe2a292cd, SUC_lexicon_tags_11 },
    { 0xac4292d0, SUC_lexicon_tags_14 },
    { 0xd8292d1, SUC_lexicon_tags_9 },
    { 0x992292d2, SUC_lexicon_tags_37 },
    { 0x34292d1, SUC_lexicon_tags_1 },
    { 0x4ce292d0, SUC_lexicon_tags_3 },
    { 0xed2292cf, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x27a292d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x312292da, SUC_lexicon_tags_39 },
    { 0xf96292db, SUC_lexicon_tags_60 },
    { 0x42292db, SUC_lexicon_tags_11 },
    { 0xe3c292dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528292e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e292e5, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x710292e7, SUC_lexicon_tags_37 },
    { 0x366292e8, SUC_lexicon_tags_1 },
    { 0x25e292e7, SUC_lexicon_tags_38 },
    { 0xa36292ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x364292ec, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c292f0, SUC_lexicon_tags_37 },
    { 0xc06292f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee292f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da292fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0292fd, SUC_lexicon_tags_29 },
    { 0xd3e292fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b029302, SUC_lexicon_tags_12 },
    { 0xb6629303, SUC_lexicon_tags_49 },
    { 0x3ba29302, SUC_lexicon_tags_15 },
    { 0x65029305, SUC_lexicon_tags_38 },
    { 0xc0429302, SUC_lexicon_tags_60 },
    { 0x97c29307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb22931d, SUC_lexicon_tags_37 },
    { 0x9fa2931e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x95429320, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e29324, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd3829326, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x30229328, SUC_lexicon_tags_15 },
    { 0xdaa29329, SUC_lexicon_tags_49 },
    { 0x6c02932a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b02932d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee29330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b629332, SUC_lexicon_tags_39 },
    { 0xd2e29332, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0429337, SUC_lexicon_tags_17 },
    { 0x42829338, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc2933d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c29343, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f829346, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaaa29348, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5082934a, SUC_lexicon_tags_17 },
    { 0xbfa2934b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa829354, SUC_lexicon_tags_17 },
    { 0x17029355, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x3fe29357, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd502935e, SUC_lexicon_tags_17 },
    { 0x9ee2935f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a29363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87029365, SUC_lexicon_tags_45 },
    { 0x61029366, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7fa29368, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe2936b, SUC_lexicon_tags_28 },
    { 0x89a2936c, SUC_lexicon_tags_60 },
    { 0x4b22936c, SUC_lexicon_tags_7 },
    { 0xbb02936d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf8229370, SUC_lexicon_tags_37 },
    { 0x8fe29371, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c29375, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4ba29377, SUC_lexicon_tags_14 },
    { 0x3c29378, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b22937b, SUC_lexicon_tags_15 },
    { 0xf302937b, SUC_lexicon_tags_13 },
    { 0xf462937b, SUC_lexicon_tags_57 },
    { 0x5ae2937d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a29384, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a429388, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f42938b, SUC_lexicon_tags_11 },
    { 0xd302938c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xab22938e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x86029390, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf029397, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc2939e, SUC_lexicon_tags_47 },
    { 0xd502939f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb64293a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x91e293a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ce293a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8293a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x908293aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634293b1, SUC_lexicon_tags_43 },
    { 0xfdc293b2, SUC_lexicon_tags_7 },
    { 0xb7c293b3, SUC_lexicon_tags_28 },
    { 0xf5c293b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba293ba, SUC_lexicon_tags_2 },
    { 0xcd8293bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3bc293bd, SUC_lexicon_tags_1 },
    { 0xed4293bd, SUC_lexicon_tags_17 },
    { 0x7aa293bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x870293c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2293c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x42293c6, SUC_lexicon_tags_11 },
    { 0xafc293c7, SUC_lexicon_tags_25 },
    { 0xf5e293c8, SUC_lexicon_tags_17 },
    { 0x420293c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80e293cb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x414293cd, SUC_lexicon_tags_2 },
    { 0x2bc293cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb04293d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8293da, SUC_lexicon_tags_12 },
    { 0x83a293da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38e293dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62293e2, SUC_lexicon_tags_37 },
    { 0x64a293e3, SUC_lexicon_tags_45 },
    { 0xc74293e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238293ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0293f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xade293f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x222293f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x690293f7, SUC_lexicon_tags_11 },
    { 0xa28293f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc293fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38029402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c29405, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc29408, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8882940f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73629413, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xcdc29415, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc5c29417, SUC_lexicon_tags_12 },
    { 0xefe29418, SUC_lexicon_tags_4 },
    { 0x52429418, SUC_lexicon_tags_12 },
    { 0x2b22941a, SUC_lexicon_tags_77 },
    { 0x7e429419, SUC_lexicon_tags_11 },
    { 0x7e42941c, SUC_lexicon_tags_17 },
    { 0xbd22941d, SUC_lexicon_tags_2 },
    { 0x7f82941b, SUC_lexicon_tags_11 },
    { 0x84c2941f, SUC_lexicon_tags_23 },
    { 0xabc29420, SUC_lexicon_tags_25 },
    { 0xc782941e, SUC_lexicon_tags_777 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a29424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75829426, SUC_lexicon_tags_12 },
    { 0xb1e29426, SUC_lexicon_tags_28 },
    { 0x8ee29428, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7d42942a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfcc2942c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d429434, SUC_lexicon_tags_29 },
    { 0x92429435, SUC_lexicon_tags_32 },
    { 0x29029436, SUC_lexicon_tags_37 },
    { 0x3ba29435, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24429439, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc782943b, SUC_lexicon_tags_9 },
    { 0x8ac2943c, SUC_lexicon_tags_11 },
    { 0xcc62943c, SUC_lexicon_tags_32 },
    { 0xfd22943c, SUC_lexicon_tags_0 },
    { 0xfd22943b, SUC_lexicon_tags_69 },
    { 0x1dc29440, SUC_lexicon_tags_31 },
    { 0xa7829440, SUC_lexicon_tags_15 },
    { 0xf7029440, SUC_lexicon_tags_17 },
    { 0xa8629442, SUC_lexicon_tags_15 },
    { 0xbee29440, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c829449, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d62944b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66429452, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5c29454, SUC_lexicon_tags_63 },
    { 0x4d429455, SUC_lexicon_tags_23 },
    { 0x9b029456, SUC_lexicon_tags_31 },
    { 0x5be29457, SUC_lexicon_tags_30 },
    { 0x1ea29458, SUC_lexicon_tags_25 },
    { 0x5ca29458, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c29464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf22946d, SUC_lexicon_tags_25 },
    { 0xdd22946d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x80a29470, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece29474, SUC_lexicon_tags_82 },
    { 0x99629475, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd5829477, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61029479, SUC_lexicon_tags_37 },
    { 0x7602947a, SUC_lexicon_tags_0 },
    { 0xe822947b, SUC_lexicon_tags_25 },
    { 0x6b629479, SUC_lexicon_tags_7 },
    { 0xf62947d, SUC_lexicon_tags_25 },
    { 0x3d02947d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3629481, SUC_lexicon_tags_2 },
    { 0xe6629482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea29485, SUC_lexicon_tags_49 },
    { 0x5aa29486, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7b429488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8422948a, SUC_lexicon_tags_7 },
    { 0x80c2948b, SUC_lexicon_tags_6 },
    { 0x6062948b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34029490, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ee29492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72829495, SUC_lexicon_tags_27 },
    { 0x1ec29496, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9402949a, SUC_lexicon_tags_26 },
    { 0x93c2949b, SUC_lexicon_tags_7 },
    { 0xd62949c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8294a2, SUC_lexicon_tags_1 },
    { 0x38e294a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd64294a5, SUC_lexicon_tags_38 },
    { 0x5e8294a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x748294a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0294ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x896294b0, SUC_lexicon_tags_28 },
    { 0x200294b1, SUC_lexicon_tags_11 },
    { 0x17e294b2, SUC_lexicon_tags_7 },
    { 0x294b1, SUC_lexicon_tags_17 },
    { 0x8ca294b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba294b7, SUC_lexicon_tags_19 },
    { 0x276294b8, SUC_lexicon_tags_53 },
    { 0x444294b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x934294bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50294c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88294c4, SUC_lexicon_tags_29 },
    { 0xfb8294c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0294c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe294cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x602294ce, SUC_lexicon_tags_47 },
    { 0x15c294cf, SUC_lexicon_tags_16 },
    { 0x19c294d0, SUC_lexicon_tags_62 },
    { 0x29a294d1, SUC_lexicon_tags_0 },
    { 0xc08294d0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2294d5, SUC_lexicon_tags_5 },
    { 0x464294d6, SUC_lexicon_tags_15 },
    { 0x274294d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0294dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90294e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4294e5, SUC_lexicon_tags_30 },
    { 0xfba294e6, SUC_lexicon_tags_32 },
    { 0x432294e7, SUC_lexicon_tags_4 },
    { 0x52294e7, SUC_lexicon_tags_45 },
    { 0xf0a294e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44c294eb, SUC_lexicon_tags_6 },
    { 0x5ec294ec, SUC_lexicon_tags_15 },
    { 0x93a294ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96294f0, SUC_lexicon_tags_193 },
    { 0x2e8294f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8294f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36294fa, SUC_lexicon_tags_57 },
    { 0x580294fa, SUC_lexicon_tags_2 },
    { 0x77c294fc, SUC_lexicon_tags_12 },
    { 0x90294fd, SUC_lexicon_tags_1 },
    { 0x1d4294fe, SUC_lexicon_tags_92 },
    { 0xad8294ff, SUC_lexicon_tags_12 },
    { 0xcee29500, SUC_lexicon_tags_1 },
    { 0xb3c29500, SUC_lexicon_tags_2 },
    { 0x2a029502, SUC_lexicon_tags_47 },
    { 0xe1429503, SUC_lexicon_tags_2 },
    { 0x9429500, SUC_lexicon_tags_87 },
    { 0x36e29501, SUC_lexicon_tags_2 },
    { 0x17c29506, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35029508, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa42950c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84429510, SUC_lexicon_tags_10 },
    { 0x6ac29510, SUC_lexicon_tags_80 },
    { 0x47629512, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea29515, SUC_lexicon_tags_25 },
    { 0xd9e29516, SUC_lexicon_tags_2 },
    { 0xec29516, SUC_lexicon_tags_4 },
    { 0x5ba29516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f42951f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a29521, SUC_lexicon_tags_47 },
    { 0xcfc29521, SUC_lexicon_tags_29 },
    { 0xace29523, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20829527, SUC_lexicon_tags_25 },
    { 0x1ee29527, SUC_lexicon_tags_25 },
    { 0xb7829527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e82952c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa502952f, SUC_lexicon_tags_47 },
    { 0xf9829530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd029534, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf1029536, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77629539, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce2953d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5429543, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fe29545, SUC_lexicon_tags_10 },
    { 0xf8829546, SUC_lexicon_tags_37 },
    { 0x1f829545, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd0629549, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7422954f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x19829552, SUC_lexicon_tags_23 },
    { 0x1bc29552, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x91829555, SUC_lexicon_tags_10 },
    { 0x10229555, SUC_lexicon_tags_47 },
    { 0x53a29557, SUC_lexicon_tags_31 },
    { 0x4c29558, SUC_lexicon_tags_1 },
    { 0x4c429559, SUC_lexicon_tags_17 },
    { 0x5b829559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a62955d, SUC_lexicon_tags_15 },
    { 0x1482955d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8429561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c229564, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16629569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb222956c, SUC_lexicon_tags_37 },
    { 0x362956c, SUC_lexicon_tags_1 },
    { 0xd962956c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34829571, SUC_lexicon_tags_59 },
    { 0x15e29572, SUC_lexicon_tags_306 },
    { 0xb3e29572, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff229579, SUC_lexicon_tags_12 },
    { 0xf382957a, SUC_lexicon_tags_15 },
    { 0x1ee2957a, SUC_lexicon_tags_21 },
    { 0xec829579, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee29586, SUC_lexicon_tags_25 },
    { 0x37e29586, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2958b, SUC_lexicon_tags_25 },
    { 0x3d42958c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeaa2958e, SUC_lexicon_tags_30 },
    { 0x2ec2958f, SUC_lexicon_tags_23 },
    { 0x3e429590, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b629593, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1229597, SUC_lexicon_tags_4 },
    { 0xd9829598, SUC_lexicon_tags_38 },
    { 0x8e229599, SUC_lexicon_tags_15 },
    { 0xfd22959a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722959f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x732295a1, SUC_lexicon_tags_14 },
    { 0xd4e295a1, SUC_lexicon_tags_17 },
    { 0x340295a3, SUC_lexicon_tags_33 },
    { 0x392295a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d8295a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1bc295a8, SUC_lexicon_tags_28 },
    { 0xcca295a9, SUC_lexicon_tags_7 },
    { 0xbaa295aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458295ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44295b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206295bb, SUC_lexicon_tags_49 },
    { 0x5ac295bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26295be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0295c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x804295c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8295cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a295d0, SUC_lexicon_tags_49 },
    { 0x102295d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8295d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x906295d8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1c8295da, SUC_lexicon_tags_4 },
    { 0xf02295da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282295e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x580295e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x858295e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x126295e6, SUC_lexicon_tags_25 },
    { 0x8de295e7, SUC_lexicon_tags_17 },
    { 0x88295e8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae295ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c295f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8295fb, SUC_lexicon_tags_48 },
    { 0x88c295fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0295ff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f029602, SUC_lexicon_tags_7 },
    { 0xfd629602, SUC_lexicon_tags_37 },
    { 0x37429604, SUC_lexicon_tags_37 },
    { 0x58a29605, SUC_lexicon_tags_13 },
    { 0xce29602, SUC_lexicon_tags_1 },
    { 0x57c29606, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6e29609, SUC_lexicon_tags_82 },
    { 0x71a29609, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9aa2960c, SUC_lexicon_tags_27 },
    { 0xb282960c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa429613, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3029618, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec2961d, SUC_lexicon_tags_17 },
    { 0x7ae2961e, SUC_lexicon_tags_15 },
    { 0x2582961e, SUC_lexicon_tags_37 },
    { 0x9629620, SUC_lexicon_tags_17 },
    { 0x3a02961e, SUC_lexicon_tags_7 },
    { 0x2b829621, SUC_lexicon_tags_37 },
    { 0x2e429620, SUC_lexicon_tags_11 },
    { 0x14029622, SUC_lexicon_tags_35 },
    { 0x2582961f, SUC_lexicon_tags_49 },
    { 0x14c29625, SUC_lexicon_tags_12 },
    { 0x22a2961d, SUC_lexicon_tags_74 },
    { 0x9e429621, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfa02962a, SUC_lexicon_tags_2 },
    { 0x7062962a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de29630, SUC_lexicon_tags_151 },
    { 0x84a29631, SUC_lexicon_tags_15 },
    { 0xdea29632, SUC_lexicon_tags_3 },
    { 0x78429633, SUC_lexicon_tags_17 },
    { 0x85e29633, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d029637, SUC_lexicon_tags_46 },
    { 0xef029637, SUC_lexicon_tags_15 },
    { 0x62c29639, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5429649, SUC_lexicon_tags_10 },
    { 0x5de2964a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f82964d, SUC_lexicon_tags_12 },
    { 0x3162964e, SUC_lexicon_tags_708 },
    { 0x4122964f, SUC_lexicon_tags_38 },
    { 0x17e2964f, SUC_lexicon_tags_48 },
    { 0xa9229650, SUC_lexicon_tags_2 },
    { 0x8302964e, SUC_lexicon_tags_17 },
    { 0x80029653, SUC_lexicon_tags_26 },
    { 0x41e29654, SUC_lexicon_tags_25 },
    { 0xe0429653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59229657, SUC_lexicon_tags_121 },
    { 0xada29658, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0xd722965b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2462965d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3842965f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0029661, SUC_lexicon_tags_28 },
    { 0xe6a29662, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8029664, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x14229666, SUC_lexicon_tags_7 },
    { 0xc829666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x622966a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f82966d, SUC_lexicon_tags_59 },
    { 0x7542966d, SUC_lexicon_tags_47 },
    { 0xa0c2966f, SUC_lexicon_tags_14 },
    { 0x2d829670, SUC_lexicon_tags_8 },
    { 0xa462966e, SUC_lexicon_tags_15 },
    { 0x8a29672, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18429676, SUC_lexicon_tags_2 },
    { 0x47829676, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6282967f, SUC_lexicon_tags_37 },
    { 0x7fa29680, SUC_lexicon_tags_17 },
    { 0x76829681, SUC_lexicon_tags_79 },
    { 0x5ea29681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429686, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce42968d, SUC_lexicon_tags_15 },
    { 0x4562968d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4a429690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6829699, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b42969d, SUC_lexicon_tags_37 },
    { 0xfea2969e, SUC_lexicon_tags_19 },
    { 0x4fa2969f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6296a2, SUC_lexicon_tags_45 },
    { 0xefe296a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be296a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2be296a8, SUC_lexicon_tags_37 },
    { 0x57e296a8, SUC_lexicon_tags_37 },
    { 0xfda296a8, SUC_lexicon_tags_61 },
    { 0x6c0296ab, SUC_lexicon_tags_30 },
    { 0x1fc296a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84296ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e296b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc296b8, SUC_lexicon_tags_14 },
    { 0xf6a296b9, SUC_lexicon_tags_63 },
    { 0xffe296b9, SUC_lexicon_tags_15 },
    { 0x28c296bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e296be, SUC_lexicon_tags_32 },
    { 0x4d8296bf, SUC_lexicon_tags_37 },
    { 0x7f8296bf, SUC_lexicon_tags_12 },
    { 0xbf2296c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae296c6, SUC_lexicon_tags_37 },
    { 0x2c4296c7, SUC_lexicon_tags_2 },
    { 0xefe296c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa296cb, SUC_lexicon_tags_63 },
    { 0x8f0296cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d8296ce, SUC_lexicon_tags_113 },
    { 0xb32296cf, SUC_lexicon_tags_15 },
    { 0xc26296d0, SUC_lexicon_tags_49 },
    { 0xdd8296d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e296d4, SUC_lexicon_tags_37 },
    { 0x76296d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2296de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4296ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c296ed, SUC_lexicon_tags_129 },
    { 0x3a296ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e6296f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c296f3, SUC_lexicon_tags_37 },
    { 0xa2e296f3, SUC_lexicon_tags_15 },
    { 0x2de296f5, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0296f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc04296fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e29701, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6429705, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c29708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202970c, SUC_lexicon_tags_12 },
    { 0xccc2970d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7522970f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x82a29711, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf7229713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4962971b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa542971e, SUC_lexicon_tags_11 },
    { 0x1582971e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf029722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52229724, SUC_lexicon_tags_7 },
    { 0x6aa29725, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x87829727, SUC_lexicon_tags_12 },
    { 0x15229728, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb442972c, SUC_lexicon_tags_64 },
    { 0xca42972d, SUC_lexicon_tags_62 },
    { 0x3862972e, SUC_lexicon_tags_25 },
    { 0x7482972f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa3029731, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43629734, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe0029736, SUC_lexicon_tags_11 },
    { 0x21829737, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d629739, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2973f, SUC_lexicon_tags_46 },
    { 0x98629740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38029744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28429746, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10629750, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e29753, SUC_lexicon_tags_25 },
    { 0xa9229753, SUC_lexicon_tags_1430 },
    { 0x2e29755, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x78429757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56029759, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a29761, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2229766, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xee029769, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfe2976b, SUC_lexicon_tags_28 },
    { 0x4de2976b, SUC_lexicon_tags_11 },
    { 0x8de2976d, SUC_lexicon_tags_12 },
    { 0xe122976e, SUC_lexicon_tags_17 },
    { 0x7582976f, SUC_lexicon_tags_45 },
    { 0x2cc2976e, SUC_lexicon_tags_19 },
    { 0x4e029771, SUC_lexicon_tags_15 },
    { 0xe1a29772, SUC_lexicon_tags_19 },
    { 0x1c62976d, SUC_lexicon_tags_11 },
    { 0x3d42976b, SUC_lexicon_tags_43 },
    { 0x9722976d, SUC_lexicon_tags_29 },
    { 0x35e29776, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde29779, SUC_lexicon_tags_12 },
    { 0xc2a29779, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f62977e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe29784, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9829786, SUC_lexicon_tags_7 },
    { 0xce429787, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca2978d, SUC_lexicon_tags_48 },
    { 0xe242978e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3429793, SUC_lexicon_tags_11 },
    { 0xe6c29794, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d429798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a2979c, SUC_lexicon_tags_25 },
    { 0x1a22979d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30297a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x346297a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc297a9, SUC_lexicon_tags_15 },
    { 0xd88297a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724297b0, SUC_lexicon_tags_28 },
    { 0x268297b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec297b5, SUC_lexicon_tags_62 },
    { 0x1c8297b6, SUC_lexicon_tags_13 },
    { 0xa7a297b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c297bc, SUC_lexicon_tags_63 },
    { 0x6a4297bd, SUC_lexicon_tags_51 },
    { 0x8b0297be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60297c7, SUC_lexicon_tags_37 },
    { 0x868297c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea297cc, SUC_lexicon_tags_12 },
    { 0x2e297cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96e297cf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c297d5, SUC_lexicon_tags_19 },
    { 0x81c297d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa297da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666297de, SUC_lexicon_tags_7 },
    { 0x12a297df, SUC_lexicon_tags_27 },
    { 0xf54297e0, SUC_lexicon_tags_1 },
    { 0x1ee297e1, SUC_lexicon_tags_27 },
    { 0x6fa297e2, SUC_lexicon_tags_33 },
    { 0x27e297e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe297e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76297ec, SUC_lexicon_tags_16 },
    { 0x55c297ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82297f0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418297f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf74297f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e297fe, SUC_lexicon_tags_57 },
    { 0x8a8297fe, SUC_lexicon_tags_1 },
    { 0x2ac29800, SUC_lexicon_tags_17 },
    { 0x236297fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf8429803, SUC_lexicon_tags_1 },
    { 0x63029804, SUC_lexicon_tags_87 },
    { 0x1b429805, SUC_lexicon_tags_15 },
    { 0x83a29806, SUC_lexicon_tags_12 },
    { 0x5ea29807, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5682980b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a29810, SUC_lexicon_tags_17 },
    { 0x3da29810, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf4829813, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23629816, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a29821, SUC_lexicon_tags_46 },
    { 0xf8629822, SUC_lexicon_tags_47 },
    { 0x2f029822, SUC_lexicon_tags_37 },
    { 0x52229821, SUC_lexicon_tags_17 },
    { 0xc6029825, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a429828, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe29830, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13229837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c2983b, SUC_lexicon_tags_14 },
    { 0xcb62983b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd429842, SUC_lexicon_tags_37 },
    { 0xc6e29843, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c829845, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x17229848, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa29859, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x742985b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a29861, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c29868, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e2986c, SUC_lexicon_tags_2 },
    { 0xc482986d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a29873, SUC_lexicon_tags_27 },
    { 0x80029873, SUC_lexicon_tags_14 },
    { 0x9c29875, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf22987b, SUC_lexicon_tags_11 },
    { 0x1982987b, SUC_lexicon_tags_37 },
    { 0xb7c2987d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2229885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7229888, SUC_lexicon_tags_17 },
    { 0xd1429888, SUC_lexicon_tags_21 },
    { 0x35829888, SUC_lexicon_tags_26 },
    { 0x47429888, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf502988d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6229899, SUC_lexicon_tags_27 },
    { 0x1682989a, SUC_lexicon_tags_10 },
    { 0x7202989a, SUC_lexicon_tags_7 },
    { 0x35c2989a, SUC_lexicon_tags_25 },
    { 0x2b22989c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa862989f, SUC_lexicon_tags_15 },
    { 0x640298a0, SUC_lexicon_tags_31 },
    { 0x6a62989f, SUC_lexicon_tags_25 },
    { 0x8ca298a2, SUC_lexicon_tags_15 },
    { 0xf86298a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x19c298a5, SUC_lexicon_tags_57 },
    { 0x60a298a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa298ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754298b6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c298b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb6a298bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698298c4, SUC_lexicon_tags_25 },
    { 0x912298c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8298c9, SUC_lexicon_tags_0 },
    { 0x90c298ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18a298cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x41c298ce, SUC_lexicon_tags_11 },
    { 0x90c298ce, SUC_lexicon_tags_37 },
    { 0x112298d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7ea298d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93a298d4, SUC_lexicon_tags_65 },
    { 0x26c298d5, SUC_lexicon_tags_63 },
    { 0x16298d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe2298d8, SUC_lexicon_tags_17 },
    { 0xa3c298d9, SUC_lexicon_tags_29 },
    { 0x52298da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c298e0, SUC_lexicon_tags_37 },
    { 0x12a298e1, SUC_lexicon_tags_38 },
    { 0xdb6298e0, SUC_lexicon_tags_38 },
    { 0xa9e298e3, SUC_lexicon_tags_26 },
    { 0xe298e4, SUC_lexicon_tags_19 },
    { 0x8de298e3, SUC_lexicon_tags_9 },
    { 0x454298e2, SUC_lexicon_tags_7 },
    { 0x392298e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a0298e9, SUC_lexicon_tags_27 },
    { 0x1be298ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x418298ec, SUC_lexicon_tags_40 },
    { 0x22e298ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x82c298ef, SUC_lexicon_tags_32 },
    { 0x6de298f0, SUC_lexicon_tags_11 },
    { 0x208298f1, SUC_lexicon_tags_55 },
    { 0x81c298f1, SUC_lexicon_tags_31 },
    { 0x5b2298f2, SUC_lexicon_tags_9 },
    { 0x98e298f2, SUC_lexicon_tags_60 },
    { 0x520298f2, SUC_lexicon_tags_14 },
    { 0xf84298f6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262298fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaca298ff, SUC_lexicon_tags_33 },
    { 0x1a229900, SUC_lexicon_tags_11 },
    { 0xba629901, SUC_lexicon_tags_63 },
    { 0x24829900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a29905, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x55829908, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6922990b, SUC_lexicon_tags_37 },
    { 0x68a2990c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb222990e, SUC_lexicon_tags_60 },
    { 0xef62990f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0429916, SUC_lexicon_tags_27 },
    { 0xdcc29917, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e2991b, SUC_lexicon_tags_1 },
    { 0x1822991c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x882991f, SUC_lexicon_tags_2 },
    { 0x12029920, SUC_lexicon_tags_45 },
    { 0xe222991f, SUC_lexicon_tags_38 },
    { 0x5ee29922, SUC_lexicon_tags_42 },
    { 0xcfc29923, SUC_lexicon_tags_31 },
    { 0x2542991f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a29935, SUC_lexicon_tags_129 },
    { 0x43629935, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc829938, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd982993b, SUC_lexicon_tags_15 },
    { 0x4202993b, SUC_lexicon_tags_79 },
    { 0xa8e2993d, SUC_lexicon_tags_6 },
    { 0xaca2993e, SUC_lexicon_tags_1 },
    { 0xfc62993f, SUC_lexicon_tags_21 },
    { 0x9d229940, SUC_lexicon_tags_11 },
    { 0xf0e2993f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a29944, SUC_lexicon_tags_63 },
    { 0x36a29945, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa1c29947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x30629949, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f029950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3829953, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba2995a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79a2995c, SUC_lexicon_tags_61 },
    { 0xcf42995d, SUC_lexicon_tags_12 },
    { 0xfdc2995e, SUC_lexicon_tags_11 },
    { 0x1042995e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x12e29961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94629965, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x47029968, SUC_lexicon_tags_11 },
    { 0xbae29969, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc22996e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e29973, SUC_lexicon_tags_25 },
    { 0x3c429974, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x91e29976, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e2997a, SUC_lexicon_tags_0 },
    { 0xbc42997b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde29988, SUC_lexicon_tags_1 },
    { 0xa7629989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80629997, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd222999c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270299a0, SUC_lexicon_tags_8 },
    { 0xa3c299a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba299a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992299ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46299ae, SUC_lexicon_tags_35 },
    { 0xfd0299af, SUC_lexicon_tags_14 },
    { 0xa14299b0, SUC_lexicon_tags_37 },
    { 0x416299b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78299b4, SUC_lexicon_tags_25 },
    { 0x272299b4, SUC_lexicon_tags_45 },
    { 0x302299b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa299ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398299c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x428299c2, SUC_lexicon_tags_66 },
    { 0xf52299c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xde6299c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2299c8, SUC_lexicon_tags_11 },
    { 0xe16299c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562299d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2299db, SUC_lexicon_tags_7 },
    { 0xbe4299db, SUC_lexicon_tags_65 },
    { 0x448299db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8299e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a299ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc299f0, SUC_lexicon_tags_13 },
    { 0x150299f1, SUC_lexicon_tags_25 },
    { 0x3a8299f2, SUC_lexicon_tags_7 },
    { 0xcc6299f2, SUC_lexicon_tags_1 },
    { 0x146299f2, SUC_lexicon_tags_11 },
    { 0x57e299f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x652299f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x970299fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc8299fc, SUC_lexicon_tags_33 },
    { 0x38299fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae29a02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaea29a04, SUC_lexicon_tags_17 },
    { 0x4bc29a05, SUC_lexicon_tags_7 },
    { 0xd3829a04, SUC_lexicon_tags_38 },
    { 0x15029a07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1629a0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x17229a0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce29a13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb029a16, SUC_lexicon_tags_11 },
    { 0x79229a16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f029a19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcce29a1b, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7429a1e, SUC_lexicon_tags_37 },
    { 0x84629a1f, SUC_lexicon_tags_45 },
    { 0x3fe29a20, SUC_lexicon_tags_37 },
    { 0x81a29a20, SUC_lexicon_tags_14 },
    { 0x7b829a22, SUC_lexicon_tags_17 },
    { 0x5ce29a23, SUC_lexicon_tags_37 },
    { 0x51629a24, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x63c29a26, SUC_lexicon_tags_19 },
    { 0x78e29a27, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7c229a29, SUC_lexicon_tags_25 },
    { 0xc8629a2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc829a37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47629a39, SUC_lexicon_tags_63 },
    { 0x45629a3a, SUC_lexicon_tags_14 },
    { 0xe0829a3b, SUC_lexicon_tags_15 },
    { 0xb1829a3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d629a3e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x13429a40, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48629a47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac29a55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e29a59, SUC_lexicon_tags_17 },
    { 0x6829a59, SUC_lexicon_tags_21 },
    { 0x8f029a5b, SUC_lexicon_tags_21 },
    { 0x54829a5c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf829a63, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7b029a65, SUC_lexicon_tags_38 },
    { 0x5c29a65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad229a68, SUC_lexicon_tags_26 },
    { 0x1a629a69, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x66e29a6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a629a71, SUC_lexicon_tags_15 },
    { 0xe1029a72, SUC_lexicon_tags_2 },
    { 0x18429a72, SUC_lexicon_tags_43 },
    { 0x88a29a73, SUC_lexicon_tags_3 },
    { 0xe6029a75, SUC_lexicon_tags_65 },
    { 0xeb229a72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde29a7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x78429a7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44829a81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbca29a83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8829a85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5629a87, SUC_lexicon_tags_38 },
    { 0x26829a88, SUC_lexicon_tags_11 },
    { 0xa1429a89, SUC_lexicon_tags_17 },
    { 0xf2029a8a, SUC_lexicon_tags_15 },
    { 0x42c29a8a, SUC_lexicon_tags_11 },
    { 0x4aa29a88, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5829a95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e829a97, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e029a9c, SUC_lexicon_tags_11 },
    { 0x7e29a9c, SUC_lexicon_tags_11 },
    { 0x3a829a9e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb829aa2, SUC_lexicon_tags_12 },
    { 0xcc229aa3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ba29aa5, SUC_lexicon_tags_38 },
    { 0x31029aa6, SUC_lexicon_tags_14 },
    { 0xc9229aa6, SUC_lexicon_tags_11 },
    { 0x3ae29aa8, SUC_lexicon_tags_15 },
    { 0xc8c29aa9, SUC_lexicon_tags_60 },
    { 0xf9429aaa, SUC_lexicon_tags_29 },
    { 0x58229aa7, SUC_lexicon_tags_17 },
    { 0x75629aaa, SUC_lexicon_tags_19 },
    { 0x6e629aad, SUC_lexicon_tags_12 },
    { 0xc2a29aa8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa8029ab0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x89429ab2, SUC_lexicon_tags_1 },
    { 0x1fa29ab3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c29ab7, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e29aba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1429abd, SUC_lexicon_tags_26 },
    { 0x3c429abd, SUC_lexicon_tags_17 },
    { 0xb9229abf, SUC_lexicon_tags_46 },
    { 0xdda29abf, SUC_lexicon_tags_15 },
    { 0xd3629ac1, SUC_lexicon_tags_26 },
    { 0x48c29ac2, SUC_lexicon_tags_13 },
    { 0xe3a29abd, SUC_lexicon_tags_1 },
    { 0x3429abe, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e29ac8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe629ad1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99829ad8, SUC_lexicon_tags_6 },
    { 0x42a29ad9, SUC_lexicon_tags_77 },
    { 0x85a29ad9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc29ae1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3fc29ae3, SUC_lexicon_tags_11 },
    { 0xfbe29ae3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57429ae8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5229aeb, SUC_lexicon_tags_15 },
    { 0x2629aeb, SUC_lexicon_tags_0 },
    { 0x76829aeb, SUC_lexicon_tags_37 },
    { 0xebe29aec, SUC_lexicon_tags_174 },
    { 0x61c29aef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28629af1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60a29af3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x49a29af5, SUC_lexicon_tags_6 },
    { 0xe3229af6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e29afa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae629afe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3629b00, SUC_lexicon_tags_14 },
    { 0xe6a29b00, SUC_lexicon_tags_1 },
    { 0x4a029b02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d629b06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a29b09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81c29b0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xda629b0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b429b11, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca29b16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d429b1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c29b1d, SUC_lexicon_tags_68 },
    { 0x48629b1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7c29b20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef229b25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4629b27, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x20c29b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9cc29b2b, SUC_lexicon_tags_25 },
    { 0x1a029b2c, SUC_lexicon_tags_1 },
    { 0x19e29b2c, SUC_lexicon_tags_25 },
    { 0x39629b2c, SUC_lexicon_tags_63 },
    { 0xfbc29b2f, SUC_lexicon_tags_33 },
    { 0x76e29b2e, SUC_lexicon_tags_87 },
    { 0xd3a29b2e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b429b3f, SUC_lexicon_tags_25 },
    { 0x1ae29b40, SUC_lexicon_tags_2 },
    { 0x18229b40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5a29b43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1429b4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1629b55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be29b59, SUC_lexicon_tags_25 },
    { 0xa9629b59, SUC_lexicon_tags_65 },
    { 0x7c029b5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75629b5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2029b60, SUC_lexicon_tags_37 },
    { 0xfca29b60, SUC_lexicon_tags_17 },
    { 0xa4e29b62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55029b67, SUC_lexicon_tags_11 },
    { 0xca429b68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a29b6c, SUC_lexicon_tags_198 },
    { 0xd429b6c, SUC_lexicon_tags_49 },
    { 0x5e229b6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7029b72, SUC_lexicon_tags_63 },
    { 0xf1629b73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66229b75, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x32629b77, SUC_lexicon_tags_26 },
    { 0xbd229b78, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x69429b7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6829b7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e629b80, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa29b8f, SUC_lexicon_tags_37 },
    { 0x229b8f, SUC_lexicon_tags_47 },
    { 0x5e429b91, SUC_lexicon_tags_17 },
    { 0x9a229b92, SUC_lexicon_tags_25 },
    { 0x9c429b92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44629b96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5829b98, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5029b9d, SUC_lexicon_tags_77 },
    { 0xcc829b9e, SUC_lexicon_tags_2 },
    { 0xb229b9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a29ba4, SUC_lexicon_tags_31 },
    { 0x45e29ba5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2a429ba7, SUC_lexicon_tags_15 },
    { 0x66629ba7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51429bac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa429bb0, SUC_lexicon_tags_37 },
    { 0xdca29bb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36029bb3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a229bc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78429bc5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17229bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e29bce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x41629bd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6029bd2, SUC_lexicon_tags_7 },
    { 0xf4e29bd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a29bd9, SUC_lexicon_tags_16 },
    { 0xab429bd9, SUC_lexicon_tags_2 },
    { 0xfea29bdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3c29bdd, SUC_lexicon_tags_47 },
    { 0x4f429bde, SUC_lexicon_tags_2 },
    { 0x59c29bdd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2ca29be1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac429be3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43629be8, SUC_lexicon_tags_17 },
    { 0xbcc29be9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a29bec, SUC_lexicon_tags_11 },
    { 0xee229bec, SUC_lexicon_tags_2 },
    { 0xfd629bee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc29bf3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd229bf6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb029bfa, SUC_lexicon_tags_17 },
    { 0x45e29bfb, SUC_lexicon_tags_6 },
    { 0x63629bfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda229bfe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2629c00, SUC_lexicon_tags_15 },
    { 0x65e29c01, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc29c06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80029c08, SUC_lexicon_tags_29 },
    { 0x5c29c09, SUC_lexicon_tags_8 },
    { 0x41829c0a, SUC_lexicon_tags_38 },
    { 0xa7429c0b, SUC_lexicon_tags_1 },
    { 0x65429c0c, SUC_lexicon_tags_1 },
    { 0x87429c0b, SUC_lexicon_tags_37 },
    { 0x60e29c0d, SUC_lexicon_tags_7 },
    { 0x58c29c0f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3029c13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x65c29c15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35029c25, SUC_lexicon_tags_62 },
    { 0x84629c26, SUC_lexicon_tags_17 },
    { 0x2c629c27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8229c2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ba29c2c, SUC_lexicon_tags_46 },
    { 0x3629c2d, SUC_lexicon_tags_60 },
    { 0xf8e29c2d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a829c34, SUC_lexicon_tags_11 },
    { 0x1c429c35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1229c3a, SUC_lexicon_tags_61 },
    { 0xf6c29c3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be29c3e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb229c44, SUC_lexicon_tags_57 },
    { 0xd8c29c45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e829c4a, SUC_lexicon_tags_287 },
    { 0xc1429c4b, SUC_lexicon_tags_49 },
    { 0x2b029c4c, SUC_lexicon_tags_31 },
    { 0xd8a29c4d, SUC_lexicon_tags_38 },
    { 0x91a29c4e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf829c50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce29c54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a29c5c, SUC_lexicon_tags_37 },
    { 0xaea29c5d, SUC_lexicon_tags_45 },
    { 0x28a29c5c, SUC_lexicon_tags_0 },
    { 0x90829c5f, SUC_lexicon_tags_12 },
    { 0x88029c60, SUC_lexicon_tags_11 },
    { 0x26429c5f, SUC_lexicon_tags_104 },
    { 0xc7429c62, SUC_lexicon_tags_19 },
    { 0xa029c62, SUC_lexicon_tags_6 },
    { 0x81c29c64, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0x1c029c66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c29c6d, SUC_lexicon_tags_10 },
    { 0x97e29c6d, SUC_lexicon_tags_14 },
    { 0xe1c29c6f, SUC_lexicon_tags_30 },
    { 0xd9a29c70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1229c72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9029c77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50029c7a, SUC_lexicon_tags_2 },
    { 0x3b429c7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65829c7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5429c85, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac29c8e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x12e29c90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d229c92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc229c96, SUC_lexicon_tags_25 },
    { 0x86e29c96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a29c9a, SUC_lexicon_tags_0 },
    { 0xa4029c9b, SUC_lexicon_tags_12 },
    { 0x60829c9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d629ca3, SUC_lexicon_tags_25 },
    { 0x35229ca4, SUC_lexicon_tags_41 },
    { 0xe4229ca5, SUC_lexicon_tags_7 },
    { 0xda029ca3, SUC_lexicon_tags_46 },
    { 0xc1a29ca3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce29cac, SUC_lexicon_tags_28 },
    { 0x21029cad, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e629cb4, SUC_lexicon_tags_1 },
    { 0x9fc29cb4, SUC_lexicon_tags_10 },
    { 0xc3029cb4, SUC_lexicon_tags_1 },
    { 0xa5029cb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c29cbb, SUC_lexicon_tags_47 },
    { 0x9d629cbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c29cc0, SUC_lexicon_tags_7 },
    { 0xc1229cc0, SUC_lexicon_tags_82 },
    { 0x41c29cc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a29cc5, SUC_lexicon_tags_65 },
    { 0x1da29cc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6029cc8, SUC_lexicon_tags_60 },
    { 0x34a29cc9, SUC_lexicon_tags_9 },
    { 0x7d629cca, SUC_lexicon_tags_207 },
    { 0xba029cc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e29ccf, SUC_lexicon_tags_1608 },
    { 0x94029cd0, SUC_lexicon_tags_7 },
    { 0x28629cd1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55029cd5, SUC_lexicon_tags_14 },
    { 0xc0429cd6, SUC_lexicon_tags_47 },
    { 0xb1c29cd5, SUC_lexicon_tags_4 },
    { 0x2de29cd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc829cda, SUC_lexicon_tags_25 },
    { 0xe1029cdb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b029cde, SUC_lexicon_tags_15 },
    { 0x3d829cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a229ce2, SUC_lexicon_tags_2 },
    { 0xaee29ce3, SUC_lexicon_tags_37 },
    { 0x23a29ce4, SUC_lexicon_tags_60 },
    { 0x3ca29ce5, SUC_lexicon_tags_87 },
    { 0xdb629ce6, SUC_lexicon_tags_14 },
    { 0x46029ce6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c29cee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2cc29cf0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b229cf3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a29cf7, SUC_lexicon_tags_45 },
    { 0x80629cf7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58229cfc, SUC_lexicon_tags_1 },
    { 0x18829cfc, SUC_lexicon_tags_47 },
    { 0xe4c29cfc, SUC_lexicon_tags_37 },
    { 0x63029cfe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f829d01, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57629d05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a29d08, SUC_lexicon_tags_27 },
    { 0xe7029d09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e29d0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a29d15, SUC_lexicon_tags_17 },
    { 0x86629d16, SUC_lexicon_tags_1 },
    { 0x84a29d17, SUC_lexicon_tags_23 },
    { 0x5e429d18, SUC_lexicon_tags_29 },
    { 0x4829d18, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb9829d1b, SUC_lexicon_tags_15 },
    { 0x70e29d1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1629d1e, SUC_lexicon_tags_12 },
    { 0x7e629d1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74229d24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53629d27, SUC_lexicon_tags_11 },
    { 0x5b829d27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfa29d2a, SUC_lexicon_tags_0 },
    { 0xb6029d2b, SUC_lexicon_tags_39 },
    { 0xeba29d2b, SUC_lexicon_tags_25 },
    { 0x4d629d2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26c29d2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3429d31, SUC_lexicon_tags_25 },
    { 0x6e029d32, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba629d34, SUC_lexicon_tags_25 },
    { 0x16229d34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1da29d37, SUC_lexicon_tags_46 },
    { 0x8e029d37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47829d40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ee29d42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc829d44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc2a29d46, SUC_lexicon_tags_0 },
    { 0xe29d46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de29d4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e829d50, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x18829d52, SUC_lexicon_tags_15 },
    { 0x9aa29d53, SUC_lexicon_tags_28 },
    { 0x72e29d52, SUC_lexicon_tags_2 },
    { 0xa0829d55, SUC_lexicon_tags_11 },
    { 0x3e029d56, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x78a29d58, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0x83829d5a, SUC_lexicon_tags_12 },
    { 0x55029d5a, SUC_lexicon_tags_15 },
    { 0x9f429d5c, SUC_lexicon_tags_0 },
    { 0x75e29d5c, SUC_lexicon_tags_2 },
    { 0x3c029d5e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a29d62, SUC_lexicon_tags_38 },
    { 0x28c29d63, SUC_lexicon_tags_23 },
    { 0xd6029d62, SUC_lexicon_tags_15 },
    { 0x9b629d64, SUC_lexicon_tags_17 },
    { 0x1a29d63, SUC_lexicon_tags_37 },
    { 0x45029d67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7c29d69, SUC_lexicon_tags_49 },
    { 0x1fa29d6a, SUC_lexicon_tags_31 },
    { 0xc4429d6a, SUC_lexicon_tags_25 },
    { 0xd6629d6c, SUC_lexicon_tags_7 },
    { 0x53829d6d, SUC_lexicon_tags_138 },
    { 0xa9629d6d, SUC_lexicon_tags_29 },
    { 0x58029d6b, SUC_lexicon_tags_33 },
    { 0x68029d69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae29d75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd829d7a, SUC_lexicon_tags_296 },
    { 0xb4a29d7b, SUC_lexicon_tags_27 },
    { 0xa5029d7c, SUC_lexicon_tags_28 },
    { 0x3da29d7c, SUC_lexicon_tags_14 },
    { 0x9b829d7e, SUC_lexicon_tags_39 },
    { 0xdc429d7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc29d82, SUC_lexicon_tags_15 },
    { 0xb2029d82, SUC_lexicon_tags_2 },
    { 0xe7e29d84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e29d87, SUC_lexicon_tags_37 },
    { 0xf6029d88, SUC_lexicon_tags_1 },
    { 0x47629d88, SUC_lexicon_tags_23 },
    { 0xf8229d8a, SUC_lexicon_tags_31 },
    { 0xf9829d8b, SUC_lexicon_tags_12 },
    { 0x8b229d8c, SUC_lexicon_tags_28 },
    { 0xb7829d88, SUC_lexicon_tags_27 },
    { 0x87229d8d, SUC_lexicon_tags_7 },
    { 0x76829d8f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e29d92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58229d97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96a29d99, SUC_lexicon_tags_232 },
    { 0xc7a29d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24429d9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7b429d9f, SUC_lexicon_tags_44 },
    { 0x23829da0, SUC_lexicon_tags_12 },
    { 0x35429da1, SUC_lexicon_tags_17 },
    { 0x8b429da0, SUC_lexicon_tags_0 },
    { 0x57e29da1, SUC_lexicon_tags_4 },
    { 0x2d029da1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb629dab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x36029dad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20229daf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e29db3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a29dba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x42629dbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x40429dbf, SUC_lexicon_tags_3 },
    { 0xd3229dbf, SUC_lexicon_tags_47 },
    { 0x6e629dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4629dc8, SUC_lexicon_tags_57 },
    { 0x73029dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f429dcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54029dd4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f629dd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9029dda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40a29ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0e29dde, SUC_lexicon_tags_0 },
    { 0x1429ddf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x56829de1, SUC_lexicon_tags_2 },
    { 0xbf829de2, SUC_lexicon_tags_11 },
    { 0xc7c29de1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c29de7, SUC_lexicon_tags_35 },
    { 0xe3829de8, SUC_lexicon_tags_37 },
    { 0x1ba29de9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42829dee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e29df3, SUC_lexicon_tags_31 },
    { 0xf8029df3, SUC_lexicon_tags_28 },
    { 0x6aa29df3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e29df9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5429dfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe29e01, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x22029e04, SUC_lexicon_tags_56 },
    { 0x25629e04, SUC_lexicon_tags_55 },
    { 0x39429e06, SUC_lexicon_tags_12 },
    { 0x18c29e07, SUC_lexicon_tags_13 },
    { 0x4f229e07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46429e0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33a29e0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f229e10, SUC_lexicon_tags_25 },
    { 0x11429e11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31e29e13, SUC_lexicon_tags_15 },
    { 0x7d629e13, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc0229e16, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e429e1a, SUC_lexicon_tags_12 },
    { 0x24e29e1b, SUC_lexicon_tags_26 },
    { 0xd0629e1b, SUC_lexicon_tags_11 },
    { 0x2bc29e1c, SUC_lexicon_tags_17 },
    { 0xf9c29e1e, SUC_lexicon_tags_38 },
    { 0x12829e1a, SUC_lexicon_tags_37 },
    { 0xb6c29e1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf8629e22, SUC_lexicon_tags_25 },
    { 0x22629e23, SUC_lexicon_tags_32 },
    { 0x2fe29e23, SUC_lexicon_tags_11 },
    { 0xe3c29e22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x93229e28, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86029e2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16829e32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e829e35, SUC_lexicon_tags_6 },
    { 0xf9829e36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69229e3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf029e40, SUC_lexicon_tags_19 },
    { 0xe0c29e41, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a29e47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7a29e49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1829e4b, SUC_lexicon_tags_108 },
    { 0x86e29e4c, SUC_lexicon_tags_38 },
    { 0x3029e4d, SUC_lexicon_tags_16 },
    { 0xdf429e4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a29e51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f629e54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2229e57, SUC_lexicon_tags_11 },
    { 0x8b029e58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28029e5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4c29e5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda229e60, SUC_lexicon_tags_14 },
    { 0x59c29e61, SUC_lexicon_tags_11 },
    { 0x66229e60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f229e67, SUC_lexicon_tags_45 },
    { 0xdb829e68, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6629e6c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75029e72, SUC_lexicon_tags_1 },
    { 0xc3429e73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c029e75, SUC_lexicon_tags_19 },
    { 0xff629e76, SUC_lexicon_tags_11 },
    { 0x4e229e76, SUC_lexicon_tags_29 },
    { 0xa1229e76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c29e7b, SUC_lexicon_tags_12 },
    { 0x78029e7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a29e7f, SUC_lexicon_tags_7 },
    { 0x86a29e7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43429e85, SUC_lexicon_tags_25 },
    { 0xc4629e86, SUC_lexicon_tags_10 },
    { 0x73829e87, SUC_lexicon_tags_14 },
    { 0xab629e87, SUC_lexicon_tags_2 },
    { 0x29a29e87, SUC_lexicon_tags_30 },
    { 0x74829e87, SUC_lexicon_tags_47 },
    { 0xa8429e8b, SUC_lexicon_tags_1 },
    { 0x2b229e86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e29e8f, SUC_lexicon_tags_27 },
    { 0xadc29e8f, SUC_lexicon_tags_46 },
    { 0xb0229e91, SUC_lexicon_tags_17 },
    { 0xe0229e91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e29e96, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae29e99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc29e9c, SUC_lexicon_tags_47 },
    { 0x1c029e9c, SUC_lexicon_tags_0 },
    { 0xf9629e9d, SUC_lexicon_tags_1 },
    { 0x85829e9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x77429ea2, SUC_lexicon_tags_65 },
    { 0xd029ea3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd429ea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba229ea9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd229eac, SUC_lexicon_tags_7 },
    { 0x22429ead, SUC_lexicon_tags_11 },
    { 0x52829eac, SUC_lexicon_tags_11 },
    { 0xeea29eaf, SUC_lexicon_tags_17 },
    { 0x24429ead, SUC_lexicon_tags_32 },
    { 0xd2629eae, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x18629eb3, SUC_lexicon_tags_12 },
    { 0x5b629eb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf029eb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0a29eb9, SUC_lexicon_tags_39 },
    { 0xa8429eba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e29ebd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96c29ebf, SUC_lexicon_tags_17 },
    { 0x3c429ebf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba29ec5, SUC_lexicon_tags_14 },
    { 0xd5829ec6, SUC_lexicon_tags_37 },
    { 0x9a29ec6, SUC_lexicon_tags_15 },
    { 0xfc429ec8, SUC_lexicon_tags_32 },
    { 0xa3029ec9, SUC_lexicon_tags_37 },
    { 0x94c29eca, SUC_lexicon_tags_7 },
    { 0x2ec29ecb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c229ed0, SUC_lexicon_tags_12 },
    { 0xe5629ed1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82029ed4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3629ed6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9829edc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80229ede, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba29ee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a029ee8, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8629eec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe629ef5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c29ef8, SUC_lexicon_tags_38 },
    { 0x27a29ef9, SUC_lexicon_tags_11 },
    { 0x31429efa, SUC_lexicon_tags_6 },
    { 0x4ee29efb, SUC_lexicon_tags_30 },
    { 0xbf429ef9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4429f05, SUC_lexicon_tags_63 },
    { 0x2ca29f05, SUC_lexicon_tags_12 },
    { 0xce629f07, SUC_lexicon_tags_37 },
    { 0xd4429f08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b029f0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d429f0e, SUC_lexicon_tags_60 },
    { 0x13a29f0e, SUC_lexicon_tags_17 },
    { 0x3b029f10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6829f16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb629f18, SUC_lexicon_tags_8 },
    { 0xdf429f18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5029f1b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa6629f1d, SUC_lexicon_tags_25 },
    { 0xcde29f1d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x5e829f20, SUC_lexicon_tags_47 },
    { 0xfc629f20, SUC_lexicon_tags_4 },
    { 0xb8829f20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51029f27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xadc29f29, SUC_lexicon_tags_15 },
    { 0x12e29f2a, SUC_lexicon_tags_37 },
    { 0x2d229f2a, SUC_lexicon_tags_43 },
    { 0xbe829f2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47e29f2e, SUC_lexicon_tags_37 },
    { 0x94829f2f, SUC_lexicon_tags_38 },
    { 0xfb629f30, SUC_lexicon_tags_28 },
    { 0xca429f31, SUC_lexicon_tags_25 },
    { 0x4f229f30, SUC_lexicon_tags_12 },
    { 0xd0c29f33, SUC_lexicon_tags_11 },
    { 0xf7229f34, SUC_lexicon_tags_14 },
    { 0x2de29f34, SUC_lexicon_tags_37 },
    { 0x5ee29f34, SUC_lexicon_tags_37 },
    { 0xb8629f37, SUC_lexicon_tags_15 },
    { 0x4c029f2e, SUC_lexicon_tags_17 },
    { 0x36c29f32, SUC_lexicon_tags_91 },
    { 0x63229f35, SUC_lexicon_tags_37 },
    { 0x5c429f3b, SUC_lexicon_tags_7 },
    { 0xfaa29f3c, SUC_lexicon_tags_39 },
    { 0x7ba29f39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e29f41, SUC_lexicon_tags_34 },
    { 0x5829f42, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x22829f44, SUC_lexicon_tags_12 },
    { 0x65229f45, SUC_lexicon_tags_12 },
    { 0xd2c29f44, SUC_lexicon_tags_25 },
    { 0x59c29f47, SUC_lexicon_tags_9 },
    { 0xd7c29f48, SUC_lexicon_tags_12 },
    { 0xf1629f48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x65829f4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e29f51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e829f54, SUC_lexicon_tags_31 },
    { 0xaa029f55, SUC_lexicon_tags_7 },
    { 0xff429f56, SUC_lexicon_tags_7 },
    { 0x51e29f57, SUC_lexicon_tags_16 },
    { 0xcba29f58, SUC_lexicon_tags_62 },
    { 0xb6429f57, SUC_lexicon_tags_36 },
    { 0x9c629f59, SUC_lexicon_tags_17 },
    { 0x44829f5b, SUC_lexicon_tags_15 },
    { 0xe9029f5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6629f5e, SUC_lexicon_tags_9 },
    { 0x80229f5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d829f65, SUC_lexicon_tags_7 },
    { 0xc9429f66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeb629f68, SUC_lexicon_tags_11 },
    { 0xbf029f68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1e29f6b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe1229f6d, SUC_lexicon_tags_37 },
    { 0x17429f6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1c229f70, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42029f77, SUC_lexicon_tags_12 },
    { 0xc9a29f78, SUC_lexicon_tags_11 },
    { 0x9ba29f77, SUC_lexicon_tags_1 },
    { 0x4229f7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51629f7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0829f84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d029f87, SUC_lexicon_tags_68 },
    { 0x9f429f88, SUC_lexicon_tags_2 },
    { 0xda829f89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6c29f8b, SUC_lexicon_tags_32 },
    { 0x6029f8c, SUC_lexicon_tags_23 },
    { 0xc2829f8d, SUC_lexicon_tags_11 },
    { 0xbb429f8e, SUC_lexicon_tags_36 },
    { 0x96029f8b, SUC_lexicon_tags_34 },
    { 0xd4c29f90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4be29f92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x31429f94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10429f97, SUC_lexicon_tags_60 },
    { 0xf7a29f98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c29f9b, SUC_lexicon_tags_47 },
    { 0x88a29f9c, SUC_lexicon_tags_12 },
    { 0x7ce29f9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c29fa1, SUC_lexicon_tags_29 },
    { 0x38629fa1, SUC_lexicon_tags_25 },
    { 0x6de29fa3, SUC_lexicon_tags_0 },
    { 0xaa229fa1, SUC_lexicon_tags_2 },
    { 0x74c29fa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ce29fa7, SUC_lexicon_tags_37 },
    { 0x7de29fa8, SUC_lexicon_tags_6 },
    { 0x72e29fa9, SUC_lexicon_tags_45 },
    { 0x2c629fa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3829fad, SUC_lexicon_tags_45 },
    { 0x7b029fae, SUC_lexicon_tags_15 },
    { 0x5d829fae, SUC_lexicon_tags_39 },
    { 0xd7a29fae, SUC_lexicon_tags_49 },
    { 0x1f229fb1, SUC_lexicon_tags_25 },
    { 0xe5a29fb1, SUC_lexicon_tags_37 },
    { 0xc0e29fb1, SUC_lexicon_tags_25 },
    { 0xcc29fb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc29fb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3829fba, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee29fbd, SUC_lexicon_tags_19 },
    { 0xb9429fbe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2c429fc0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64229fc5, SUC_lexicon_tags_43 },
    { 0xcde29fc5, SUC_lexicon_tags_29 },
    { 0xcfa29fc7, SUC_lexicon_tags_4 },
    { 0x89229fc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87829fcc, SUC_lexicon_tags_7 },
    { 0xe6c29fcd, SUC_lexicon_tags_45 },
    { 0xe9229fce, SUC_lexicon_tags_33 },
    { 0xc7a29fcd, SUC_lexicon_tags_13 },
    { 0x88229fce, SUC_lexicon_tags_17 },
    { 0xba029fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a029fd5, SUC_lexicon_tags_7 },
    { 0x78029fd5, SUC_lexicon_tags_30 },
    { 0x50029fd7, SUC_lexicon_tags_17 },
    { 0xeb629fd5, SUC_lexicon_tags_23 },
    { 0xea629fd9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x11829fdb, SUC_lexicon_tags_38 },
    { 0x8f829fdb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f429fe1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x52829fe4, SUC_lexicon_tags_12 },
    { 0x36829fe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae229fe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28429fea, SUC_lexicon_tags_38 },
    { 0x4b629fea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3429ff0, SUC_lexicon_tags_43 },
    { 0x20c29ff0, SUC_lexicon_tags_7 },
    { 0xa5c29ff0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a29ff5, SUC_lexicon_tags_29 },
    { 0xae229ff6, SUC_lexicon_tags_63 },
    { 0x8229ff6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34629ff9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e29ffe, SUC_lexicon_tags_37 },
    { 0xa7829ffe, SUC_lexicon_tags_84 },
    { 0xfd629fff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1482a002, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9862a005, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5562a007, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc1a2a009, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe982a010, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf62a018, SUC_lexicon_tags_12 },
    { 0x1442a019, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3c22a01b, SUC_lexicon_tags_39 },
    { 0xc522a01c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9842a021, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a02a026, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x202a028, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2922a02e, SUC_lexicon_tags_28 },
    { 0xb062a02f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc2a032, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9be2a034, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8162a038, SUC_lexicon_tags_6 },
    { 0x8762a039, SUC_lexicon_tags_17 },
    { 0x75a2a03a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7962a03f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e62a046, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5662a049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9102a04c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c62a04e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1322a053, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x982a055, SUC_lexicon_tags_7 },
    { 0xe8a2a056, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b62a058, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe2a05f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec2a062, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba2a066, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe502a06a, SUC_lexicon_tags_23 },
    { 0x8b62a06b, SUC_lexicon_tags_1 },
    { 0xda02a06c, SUC_lexicon_tags_7 },
    { 0x7062a06d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x682a06f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5882a074, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c42a077, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe2a07e, SUC_lexicon_tags_2 },
    { 0x9c42a07f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb82a083, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd562a085, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3602a087, SUC_lexicon_tags_2 },
    { 0x2a42a088, SUC_lexicon_tags_30 },
    { 0xb042a089, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a02a08b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8322a08f, SUC_lexicon_tags_38 },
    { 0xac62a08f, SUC_lexicon_tags_17 },
    { 0xb082a08f, SUC_lexicon_tags_47 },
    { 0x9ce2a092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18a2a094, SUC_lexicon_tags_1 },
    { 0x2362a095, SUC_lexicon_tags_43 },
    { 0xb742a096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1862a09e, SUC_lexicon_tags_15 },
    { 0xb342a09e, SUC_lexicon_tags_46 },
    { 0xbbe2a0a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1722a0a3, SUC_lexicon_tags_12 },
    { 0x7d62a0a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb62a0a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6902a0a8, SUC_lexicon_tags_29 },
    { 0xe542a0a9, SUC_lexicon_tags_2 },
    { 0x7002a0a9, SUC_lexicon_tags_21 },
    { 0x3542a0ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99e2a0ad, SUC_lexicon_tags_37 },
    { 0xb02a0ae, SUC_lexicon_tags_39 },
    { 0x63c2a0ad, SUC_lexicon_tags_0 },
    { 0xe42a0af, SUC_lexicon_tags_1 },
    { 0x2322a0af, SUC_lexicon_tags_937 },
    { 0xe222a0b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c2a0b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc22a0b8, SUC_lexicon_tags_70 },
    { 0xabc2a0b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3ce2a0bb, SUC_lexicon_tags_38 },
    { 0x3fc2a0bc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a2a0c6, SUC_lexicon_tags_7 },
    { 0xeec2a0c7, SUC_lexicon_tags_11 },
    { 0x2662a0c8, SUC_lexicon_tags_39 },
    { 0xae62a0c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39a2a0cb, SUC_lexicon_tags_12 },
    { 0x2b42a0cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22a0cf, SUC_lexicon_tags_1 },
    { 0x64e2a0cf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be2a0d3, SUC_lexicon_tags_37 },
    { 0xd9c2a0d4, SUC_lexicon_tags_61 },
    { 0x1e62a0d5, SUC_lexicon_tags_26 },
    { 0xd202a0d6, SUC_lexicon_tags_2 },
    { 0xf122a0d7, SUC_lexicon_tags_1 },
    { 0x8702a0d8, SUC_lexicon_tags_17 },
    { 0x1922a0d9, SUC_lexicon_tags_2 },
    { 0xabe2a0d6, SUC_lexicon_tags_2 },
    { 0xbd82a0db, SUC_lexicon_tags_47 },
    { 0x3c02a0d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e62a0e0, SUC_lexicon_tags_87 },
    { 0xb3e2a0e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2902a0e4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f02a0e9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5be2a0eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d62a0ed, SUC_lexicon_tags_38 },
    { 0x8de2a0ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba82a0f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcce2a0f4, SUC_lexicon_tags_28 },
    { 0x5382a0f5, SUC_lexicon_tags_16 },
    { 0x8e02a0f6, SUC_lexicon_tags_27 },
    { 0xfa62a0f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc2a0fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc062a0ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad02a101, SUC_lexicon_tags_5 },
    { 0xd762a102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9782a106, SUC_lexicon_tags_7 },
    { 0x27a2a106, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a109, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf402a10e, SUC_lexicon_tags_60 },
    { 0xdc42a10f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4e2a111, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe2a118, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc2a11b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c2a120, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba2a125, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a2a128, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b02a12a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de2a12d, SUC_lexicon_tags_11 },
    { 0x4c62a12e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9702a130, SUC_lexicon_tags_5 },
    { 0xdd22a130, SUC_lexicon_tags_39 },
    { 0x72c2a132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc822a134, SUC_lexicon_tags_25 },
    { 0x4282a134, SUC_lexicon_tags_47 },
    { 0x7ac2a136, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a2a139, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7042a13d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d22a149, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd182a14c, SUC_lexicon_tags_6 },
    { 0x4402a14c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb302a150, SUC_lexicon_tags_38 },
    { 0x80c2a150, SUC_lexicon_tags_46 },
    { 0xcce2a151, SUC_lexicon_tags_2 },
    { 0x7a2a153, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5382a157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82a15d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd722a160, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2202a166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac2a16a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5882a171, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c42a173, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb62a17b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe082a17e, SUC_lexicon_tags_49 },
    { 0xb2e2a17f, SUC_lexicon_tags_3 },
    { 0xd02a17f, SUC_lexicon_tags_17 },
    { 0x8b02a17f, SUC_lexicon_tags_11 },
    { 0x6f62a180, SUC_lexicon_tags_33 },
    { 0xcd22a17f, SUC_lexicon_tags_12 },
    { 0xfcc2a181, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42a187, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x41a2a189, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5242a18b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad02a193, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc2a196, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a2a19b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e22a19d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3762a19f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f82a1a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1922a1a8, SUC_lexicon_tags_13 },
    { 0xffa2a1a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5002a1ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d42a1b0, SUC_lexicon_tags_28 },
    { 0x442a1b1, SUC_lexicon_tags_15 },
    { 0xf3c2a1b2, SUC_lexicon_tags_15 },
    { 0x4d62a1b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe162a1ba, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6702a1bf, SUC_lexicon_tags_7 },
    { 0x2c2a1c0, SUC_lexicon_tags_92 },
    { 0xef62a1c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4cc2a1c3, SUC_lexicon_tags_29 },
    { 0xe9c2a1c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64c2a1c6, SUC_lexicon_tags_63 },
    { 0xc6c2a1c7, SUC_lexicon_tags_68 },
    { 0x6802a1c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de2a1cd, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02a1d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5402a1d5, SUC_lexicon_tags_63 },
    { 0xf6a2a1d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f42a1da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3782a1e1, SUC_lexicon_tags_7 },
    { 0xbec2a1e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa2a1e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d82a1e9, SUC_lexicon_tags_33 },
    { 0xa5e2a1e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c2a1ee, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7822a1f3, SUC_lexicon_tags_15 },
    { 0x7382a1f4, SUC_lexicon_tags_30 },
    { 0x122a1f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d22a1f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a2a1fa, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc2a1fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4762a201, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcda2a203, SUC_lexicon_tags_37 },
    { 0x4542a204, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa902a207, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e2a210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c02a214, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be2a217, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeba2a219, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9042a223, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea2a226, SUC_lexicon_tags_17 },
    { 0xc262a227, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa462a229, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe502a22c, SUC_lexicon_tags_35 },
    { 0xf202a22c, SUC_lexicon_tags_49 },
    { 0x4742a22d, SUC_lexicon_tags_15 },
    { 0xc762a22e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e2a233, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3322a236, SUC_lexicon_tags_12 },
    { 0x46c2a237, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9f02a239, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb562a23f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa82a243, SUC_lexicon_tags_45 },
    { 0xe282a243, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b02a249, SUC_lexicon_tags_37 },
    { 0x9162a24a, SUC_lexicon_tags_4 },
    { 0x84a2a24b, SUC_lexicon_tags_66 },
    { 0xebc2a24c, SUC_lexicon_tags_37 },
    { 0x5182a24a, SUC_lexicon_tags_15 },
    { 0x80a2a24e, SUC_lexicon_tags_17 },
    { 0x73e2a24e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c62a251, SUC_lexicon_tags_35 },
    { 0xa2a2a252, SUC_lexicon_tags_15 },
    { 0xfc82a253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4162a258, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42a25b, SUC_lexicon_tags_49 },
    { 0xb962a25c, SUC_lexicon_tags_11 },
    { 0x9a22a25c, SUC_lexicon_tags_37 },
    { 0x9d82a25d, SUC_lexicon_tags_25 },
    { 0x5222a25f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62a263, SUC_lexicon_tags_34 },
    { 0x75c2a264, SUC_lexicon_tags_43 },
    { 0x8be2a264, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4bc2a267, SUC_lexicon_tags_25 },
    { 0xc882a268, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e2a26b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd22a26d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2982a26f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a82a273, SUC_lexicon_tags_15 },
    { 0xd962a274, SUC_lexicon_tags_19 },
    { 0x7d22a275, SUC_lexicon_tags_85 },
    { 0x57a2a276, SUC_lexicon_tags_39 },
    { 0x51a2a277, SUC_lexicon_tags_82 },
    { 0xe8a2a274, SUC_lexicon_tags_49 },
    { 0xb8e2a279, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9362a27d, SUC_lexicon_tags_21 },
    { 0xf502a27e, SUC_lexicon_tags_33 },
    { 0xe2e2a27d, SUC_lexicon_tags_15 },
    { 0x8622a280, SUC_lexicon_tags_11 },
    { 0xef22a27e, SUC_lexicon_tags_15 },
    { 0x2762a282, SUC_lexicon_tags_79 },
    { 0x7302a283, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc002a289, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac2a28d, SUC_lexicon_tags_1 },
    { 0x6762a28d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e42a291, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3202a293, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc422a296, SUC_lexicon_tags_7 },
    { 0x8122a297, SUC_lexicon_tags_6 },
    { 0x4162a298, SUC_lexicon_tags_17 },
    { 0x11e2a298, SUC_lexicon_tags_37 },
    { 0xcf22a299, SUC_lexicon_tags_11 },
    { 0x2222a29a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7462a29d, SUC_lexicon_tags_26 },
    { 0x7222a29e, SUC_lexicon_tags_17 },
    { 0x6da2a29f, SUC_lexicon_tags_27 },
    { 0xd682a2a0, SUC_lexicon_tags_25 },
    { 0x2402a2a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f82a2a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe062a2a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5822a2a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c2a2b1, SUC_lexicon_tags_39 },
    { 0x88c2a2b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf42a2b4, SUC_lexicon_tags_10 },
    { 0xd2c2a2b4, SUC_lexicon_tags_47 },
    { 0xc202a2b6, SUC_lexicon_tags_37 },
    { 0xc882a2b7, SUC_lexicon_tags_17 },
    { 0xc4e2a2b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac22a2ba, SUC_lexicon_tags_47 },
    { 0x5de2a2bb, SUC_lexicon_tags_63 },
    { 0x8562a2bc, SUC_lexicon_tags_37 },
    { 0x21a2a2bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda62a2c2, SUC_lexicon_tags_34 },
    { 0x52e2a2c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a2a2c8, SUC_lexicon_tags_30 },
    { 0xac22a2c8, SUC_lexicon_tags_26 },
    { 0x6802a2ca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa82a2ce, SUC_lexicon_tags_4 },
    { 0xc702a2cf, SUC_lexicon_tags_12 },
    { 0x81e2a2d0, SUC_lexicon_tags_27 },
    { 0x6962a2d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc02a2d3, SUC_lexicon_tags_11 },
    { 0x15a2a2d3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf0a2a2d6, SUC_lexicon_tags_21 },
    { 0x6c2a2d7, SUC_lexicon_tags_25 },
    { 0x222a2d8, SUC_lexicon_tags_1 },
    { 0x3802a2d6, SUC_lexicon_tags_692 },
    { 0xc4c2a2da, SUC_lexicon_tags_11 },
    { 0x6c42a2da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe582a2de, SUC_lexicon_tags_2 },
    { 0x7282a2df, SUC_lexicon_tags_63 },
    { 0x2242a2e0, SUC_lexicon_tags_1 },
    { 0x8ce2a2df, SUC_lexicon_tags_37 },
    { 0x1a2a2e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc042a2e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba62a2e9, SUC_lexicon_tags_27 },
    { 0xfca2a2ea, SUC_lexicon_tags_26 },
    { 0xc482a2eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222a2f6, SUC_lexicon_tags_25 },
    { 0xafa2a2f7, SUC_lexicon_tags_43 },
    { 0x7342a2f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2a2fb, SUC_lexicon_tags_23 },
    { 0x8ba2a2fb, SUC_lexicon_tags_7 },
    { 0xcee2a2fd, SUC_lexicon_tags_11 },
    { 0xe0c2a2fb, SUC_lexicon_tags_60 },
    { 0x35a2a2fb, SUC_lexicon_tags_12 },
    { 0xcec2a300, SUC_lexicon_tags_43 },
    { 0xa982a2ff, SUC_lexicon_tags_6 },
    { 0xf6a2a302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a2a307, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b62a30a, SUC_lexicon_tags_11 },
    { 0x71a2a30b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec2a310, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2902a312, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a22a314, SUC_lexicon_tags_38 },
    { 0xe202a315, SUC_lexicon_tags_7 },
    { 0x3622a316, SUC_lexicon_tags_31 },
    { 0xae42a315, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf502a323, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd82a325, SUC_lexicon_tags_14 },
    { 0xc1e2a326, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e02a328, SUC_lexicon_tags_65 },
    { 0xefa2a329, SUC_lexicon_tags_37 },
    { 0x6c42a32a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2882a32f, SUC_lexicon_tags_19 },
    { 0x4d42a330, SUC_lexicon_tags_15 },
    { 0x1062a330, SUC_lexicon_tags_39 },
    { 0x4a62a332, SUC_lexicon_tags_17 },
    { 0xc822a331, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e62a335, SUC_lexicon_tags_11 },
    { 0xeb22a336, SUC_lexicon_tags_32 },
    { 0xde62a337, SUC_lexicon_tags_27 },
    { 0x85c2a338, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea2a33e, SUC_lexicon_tags_0 },
    { 0xd2c2a33f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcf02a341, SUC_lexicon_tags_37 },
    { 0xbaa2a342, SUC_lexicon_tags_17 },
    { 0x2282a342, SUC_lexicon_tags_4 },
    { 0x6e42a342, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf62a346, SUC_lexicon_tags_11 },
    { 0x4342a347, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4842a349, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb82a34c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x93c2a34e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3802a350, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe162a353, SUC_lexicon_tags_11 },
    { 0xec42a354, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa242a356, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae42a35c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7402a35e, SUC_lexicon_tags_49 },
    { 0x8222a35f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e62a36b, SUC_lexicon_tags_38 },
    { 0x9de2a36c, SUC_lexicon_tags_5 },
    { 0x7382a36d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e2a370, SUC_lexicon_tags_15 },
    { 0x6242a371, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3482a373, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe682a375, SUC_lexicon_tags_47 },
    { 0x3822a376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3602a378, SUC_lexicon_tags_8 },
    { 0x2402a379, SUC_lexicon_tags_37 },
    { 0x5562a379, SUC_lexicon_tags_55 },
    { 0x3e22a37b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2fc2a37d, SUC_lexicon_tags_17 },
    { 0xb402a37d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3522a387, SUC_lexicon_tags_34 },
    { 0x58e2a388, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9802a38b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2a391, SUC_lexicon_tags_2 },
    { 0xe802a391, SUC_lexicon_tags_30 },
    { 0x402a392, SUC_lexicon_tags_46 },
    { 0x53c2a394, SUC_lexicon_tags_46 },
    { 0x56a2a393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c2a39b, SUC_lexicon_tags_14 },
    { 0x25e2a39c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa022a39f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ce2a3a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8082a3a4, SUC_lexicon_tags_19 },
    { 0xb962a3a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7ec2a3a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf02a3b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2a3b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe2a3ba, SUC_lexicon_tags_33 },
    { 0x8902a3ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1962a3be, SUC_lexicon_tags_39 },
    { 0x99a2a3bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xafe2a3c1, SUC_lexicon_tags_141 },
    { 0x8682a3c2, SUC_lexicon_tags_43 },
    { 0xcbe2a3c2, SUC_lexicon_tags_14 },
    { 0x5702a3c2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c2a3c9, SUC_lexicon_tags_60 },
    { 0xe982a3ca, SUC_lexicon_tags_4 },
    { 0x862a3cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6a82a3cd, SUC_lexicon_tags_5 },
    { 0x6cc2a3ce, SUC_lexicon_tags_25 },
    { 0x5222a3ce, SUC_lexicon_tags_11 },
    { 0xc022a3d0, SUC_lexicon_tags_37 },
    { 0xd402a3d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1402a3d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7542a3d7, SUC_lexicon_tags_30 },
    { 0xecc2a3d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd82a3dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5322a3df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa822a3e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d62a3e8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x4702a3eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79c2a3ed, SUC_lexicon_tags_47 },
    { 0x3522a3ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7c22a3f0, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b02a3f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82e2a3f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4482a3f7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8a62a3f9, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a2a406, SUC_lexicon_tags_37 },
    { 0x9042a407, SUC_lexicon_tags_37 },
    { 0x9c82a408, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe922a40b, SUC_lexicon_tags_30 },
    { 0xa842a40c, SUC_lexicon_tags_17 },
    { 0x302a40c, SUC_lexicon_tags_15 },
    { 0x9e02a40e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b82a411, SUC_lexicon_tags_27 },
    { 0x2482a411, SUC_lexicon_tags_47 },
    { 0xe4c2a413, SUC_lexicon_tags_0 },
    { 0x1dc2a413, SUC_lexicon_tags_37 },
    { 0x49e2a413, SUC_lexicon_tags_37 },
    { 0xaf82a416, SUC_lexicon_tags_32 },
    { 0x5662a411, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5162a41d, SUC_lexicon_tags_28 },
    { 0x8d02a41e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa062a420, SUC_lexicon_tags_15 },
    { 0x5562a420, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ce2a423, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2542a428, SUC_lexicon_tags_37 },
    { 0xf9e2a429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfa2a42b, SUC_lexicon_tags_87 },
    { 0xb9a2a42c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab42a42e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9962a431, SUC_lexicon_tags_9 },
    { 0xf4e2a432, SUC_lexicon_tags_7 },
    { 0x1c82a433, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x922a435, SUC_lexicon_tags_1 },
    { 0x942a435, SUC_lexicon_tags_10 },
    { 0x2f62a437, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7942a43b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c22a43d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5a2a43f, SUC_lexicon_tags_11 },
    { 0x6162a440, SUC_lexicon_tags_25 },
    { 0x20c2a43f, SUC_lexicon_tags_25 },
    { 0xd1c2a440, SUC_lexicon_tags_1 },
    { 0xd062a442, SUC_lexicon_tags_43 },
    { 0xf082a444, SUC_lexicon_tags_6 },
    { 0x5e02a444, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1da2a447, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4de2a449, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b82a44b, SUC_lexicon_tags_45 },
    { 0xc022a44b, SUC_lexicon_tags_25 },
    { 0x5d22a44c, SUC_lexicon_tags_37 },
    { 0xa7c2a44e, SUC_lexicon_tags_45 },
    { 0x77a2a44d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6322a45b, SUC_lexicon_tags_63 },
    { 0x7dc2a45b, SUC_lexicon_tags_43 },
    { 0x72e2a45d, SUC_lexicon_tags_12 },
    { 0xf602a45c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3262a461, SUC_lexicon_tags_30 },
    { 0xb982a462, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6122a468, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9102a46b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a2a46f, SUC_lexicon_tags_37 },
    { 0x9f22a470, SUC_lexicon_tags_1 },
    { 0x202a470, SUC_lexicon_tags_25 },
    { 0x78e2a470, SUC_lexicon_tags_46 },
    { 0x1782a473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2a476, SUC_lexicon_tags_8 },
    { 0xd502a477, SUC_lexicon_tags_43 },
    { 0xeb62a478, SUC_lexicon_tags_12 },
    { 0x52a2a476, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfa62a47b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b82a481, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a492, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6742a49a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2a2a49c, SUC_lexicon_tags_12 },
    { 0xfc02a49d, SUC_lexicon_tags_15 },
    { 0x6242a49d, SUC_lexicon_tags_6 },
    { 0xb262a49f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e2a4a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc2a4a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c2a4aa, SUC_lexicon_tags_26 },
    { 0x1162a4aa, SUC_lexicon_tags_33 },
    { 0x2722a4ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd82a4ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc2a4b1, SUC_lexicon_tags_17 },
    { 0x83c2a4b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7402a4b5, SUC_lexicon_tags_4 },
    { 0x5362a4b5, SUC_lexicon_tags_2 },
    { 0x9282a4b7, SUC_lexicon_tags_12 },
    { 0x52e2a4b8, SUC_lexicon_tags_37 },
    { 0xf942a4b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de2a4bc, SUC_lexicon_tags_11 },
    { 0xdd82a4bd, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x71e2a4bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc242a4c2, SUC_lexicon_tags_4 },
    { 0x3b02a4c3, SUC_lexicon_tags_23 },
    { 0x8b02a4c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb02a4c6, SUC_lexicon_tags_12 },
    { 0xaf02a4c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a42a4cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf802a4d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c62a4d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4102a4dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ce2a4df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0c2a4e1, SUC_lexicon_tags_15 },
    { 0x6c2a4e2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5482a4e5, SUC_lexicon_tags_6 },
    { 0x5c02a4e6, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x72c2a4e8, SUC_lexicon_tags_25 },
    { 0xb042a4e9, SUC_lexicon_tags_208 },
    { 0x962a4ea, SUC_lexicon_tags_8 },
    { 0x3e82a4e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac2a4f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd262a4f7, SUC_lexicon_tags_63 },
    { 0x6ec2a4f8, SUC_lexicon_tags_11 },
    { 0x4c2a4f9, SUC_lexicon_tags_29 },
    { 0x2ce2a4fa, SUC_lexicon_tags_31 },
    { 0x9302a4f8, SUC_lexicon_tags_2 },
    { 0xb6c2a4fc, SUC_lexicon_tags_7 },
    { 0xe942a4fd, SUC_lexicon_tags_151 },
    { 0xd562a4fe, SUC_lexicon_tags_18 },
    { 0x1ba2a4ff, SUC_lexicon_tags_12 },
    { 0xd0e2a500, SUC_lexicon_tags_37 },
    { 0x3322a501, SUC_lexicon_tags_31 },
    { 0x5662a4ff, SUC_lexicon_tags_2 },
    { 0x87c2a4fa, SUC_lexicon_tags_17 },
    { 0x18a2a504, SUC_lexicon_tags_60 },
    { 0x462a504, SUC_lexicon_tags_45 },
    { 0x9482a504, SUC_lexicon_tags_31 },
    { 0xb282a507, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xfb22a509, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3042a50c, SUC_lexicon_tags_23 },
    { 0xe822a50d, SUC_lexicon_tags_33 },
    { 0x68c2a50e, SUC_lexicon_tags_37 },
    { 0xf282a50d, SUC_lexicon_tags_12 },
    { 0xa7c2a50f, SUC_lexicon_tags_23 },
    { 0x64c2a511, SUC_lexicon_tags_85 },
    { 0x81e2a512, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe122a516, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ae2a518, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e02a51d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3662a522, SUC_lexicon_tags_47 },
    { 0x4742a523, SUC_lexicon_tags_37 },
    { 0xd7a2a524, SUC_lexicon_tags_0 },
    { 0x8a02a524, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc502a529, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c42a52b, SUC_lexicon_tags_23 },
    { 0x63c2a52c, SUC_lexicon_tags_12 },
    { 0xdbe2a52d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8c2a52f, SUC_lexicon_tags_15 },
    { 0xe882a530, SUC_lexicon_tags_2 },
    { 0xce82a52f, SUC_lexicon_tags_19 },
    { 0xf382a532, SUC_lexicon_tags_60 },
    { 0x9362a532, SUC_lexicon_tags_201 },
    { 0x7182a530, SUC_lexicon_tags_23 },
    { 0x1882a535, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8922a537, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xab22a53a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c2a540, SUC_lexicon_tags_2 },
    { 0xf082a541, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa422a543, SUC_lexicon_tags_15 },
    { 0xd822a544, SUC_lexicon_tags_1 },
    { 0x6242a544, SUC_lexicon_tags_5 },
    { 0xf0a2a544, SUC_lexicon_tags_11 },
    { 0xbfe2a543, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d82a54a, SUC_lexicon_tags_31 },
    { 0x9582a54b, SUC_lexicon_tags_32 },
    { 0x4de2a54c, SUC_lexicon_tags_37 },
    { 0xf742a54b, SUC_lexicon_tags_12 },
    { 0x7282a54e, SUC_lexicon_tags_11 },
    { 0x3aa2a54f, SUC_lexicon_tags_37 },
    { 0xe922a550, SUC_lexicon_tags_25 },
    { 0x1b62a551, SUC_lexicon_tags_11 },
    { 0x3382a54a, SUC_lexicon_tags_28 },
    { 0xddc2a553, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf22a556, SUC_lexicon_tags_60 },
    { 0x9c22a556, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca2a55b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2342a55e, SUC_lexicon_tags_11 },
    { 0x9922a55f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb22a563, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x922a565, SUC_lexicon_tags_1 },
    { 0x6722a566, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69e2a568, SUC_lexicon_tags_7 },
    { 0x9242a568, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e2a56c, SUC_lexicon_tags_17 },
    { 0x1dc2a56c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d62a579, SUC_lexicon_tags_27 },
    { 0x39a2a57a, SUC_lexicon_tags_29 },
    { 0xc442a57a, SUC_lexicon_tags_7 },
    { 0x8682a57a, SUC_lexicon_tags_37 },
    { 0x8982a57d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf182a57f, SUC_lexicon_tags_7 },
    { 0x8182a580, SUC_lexicon_tags_56 },
    { 0x1302a581, SUC_lexicon_tags_1 },
    { 0x9822a582, SUC_lexicon_tags_79 },
    { 0x89a2a583, SUC_lexicon_tags_17 },
    { 0x342a583, SUC_lexicon_tags_106 },
    { 0xd802a583, SUC_lexicon_tags_49 },
    { 0x8782a586, SUC_lexicon_tags_37 },
    { 0x1fa2a585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d82a58b, SUC_lexicon_tags_84 },
    { 0xc562a58c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd42a58e, SUC_lexicon_tags_45 },
    { 0x562a58f, SUC_lexicon_tags_4 },
    { 0x6742a58e, SUC_lexicon_tags_11 },
    { 0xe2a2a591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb942a595, SUC_lexicon_tags_11 },
    { 0xd042a595, SUC_lexicon_tags_31 },
    { 0xd7a2a597, SUC_lexicon_tags_38 },
    { 0xc0e2a598, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a2a59c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4122a59e, SUC_lexicon_tags_1 },
    { 0x42a59f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a2a5a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da2a5a6, SUC_lexicon_tags_8 },
    { 0x8202a5a7, SUC_lexicon_tags_26 },
    { 0xefa2a5a8, SUC_lexicon_tags_17 },
    { 0x5962a5a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce2a5ad, SUC_lexicon_tags_48 },
    { 0x9302a5ae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a2a5b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6622a5b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a82a5b7, SUC_lexicon_tags_37 },
    { 0x3962a5b7, SUC_lexicon_tags_18 },
    { 0x6f02a5b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c2a5bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cc2a5c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb02a5c3, SUC_lexicon_tags_0 },
    { 0x8e2a5c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9782a5c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6602a5c8, SUC_lexicon_tags_25 },
    { 0xf3a2a5c9, SUC_lexicon_tags_34 },
    { 0x9aa2a5ca, SUC_lexicon_tags_38 },
    { 0x5b62a5cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd102a5cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c2a5d0, SUC_lexicon_tags_38 },
    { 0x1602a5d1, SUC_lexicon_tags_30 },
    { 0x3202a5d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbee2a5d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f82a5d7, SUC_lexicon_tags_87 },
    { 0x61c2a5d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f62a5dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb042a5df, SUC_lexicon_tags_1 },
    { 0xc022a5df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4062a5e8, SUC_lexicon_tags_1 },
    { 0xa362a5e9, SUC_lexicon_tags_28 },
    { 0xfc82a5ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c82a5ed, SUC_lexicon_tags_37 },
    { 0x47e2a5ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf222a5f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba2a5f9, SUC_lexicon_tags_33 },
    { 0xe702a5f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf122a5fd, SUC_lexicon_tags_20 },
    { 0xeb22a5fd, SUC_lexicon_tags_14 },
    { 0xce82a5ff, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce82a604, SUC_lexicon_tags_25 },
    { 0xa182a605, SUC_lexicon_tags_68 },
    { 0x9902a606, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a2a60c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc2a612, SUC_lexicon_tags_17 },
    { 0x8a42a613, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c2a618, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8182a61b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d22a621, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf82a626, SUC_lexicon_tags_85 },
    { 0xf442a627, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc862a62a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac2a630, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4382a638, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc62a63b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1202a63d, SUC_lexicon_tags_47 },
    { 0x382a63e, SUC_lexicon_tags_37 },
    { 0x3402a63d, SUC_lexicon_tags_33 },
    { 0x47e2a640, SUC_lexicon_tags_25 },
    { 0x1582a641, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd782a643, SUC_lexicon_tags_28 },
    { 0x5162a643, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb542a646, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f42a648, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xdb82a64a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93a2a64c, SUC_lexicon_tags_12 },
    { 0x5702a64d, SUC_lexicon_tags_37 },
    { 0x3082a64e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc302a655, SUC_lexicon_tags_10 },
    { 0x9f62a655, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9e2a658, SUC_lexicon_tags_6 },
    { 0x10c2a659, SUC_lexicon_tags_15 },
    { 0x62a2a659, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee22a662, SUC_lexicon_tags_14 },
    { 0x4842a663, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcca2a665, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef02a67f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6962a681, SUC_lexicon_tags_19 },
    { 0xa6e2a682, SUC_lexicon_tags_35 },
    { 0x6222a683, SUC_lexicon_tags_37 },
    { 0xfde2a683, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb582a68b, SUC_lexicon_tags_6 },
    { 0x82a2a68c, SUC_lexicon_tags_9 },
    { 0xa722a68d, SUC_lexicon_tags_31 },
    { 0x4ae2a68c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd462a692, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe2a699, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b62a69b, SUC_lexicon_tags_38 },
    { 0x69a2a69b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf222a69e, SUC_lexicon_tags_61 },
    { 0x41e2a69f, SUC_lexicon_tags_11 },
    { 0x5002a69f, SUC_lexicon_tags_47 },
    { 0x722a6a1, SUC_lexicon_tags_37 },
    { 0xc3c2a6a2, SUC_lexicon_tags_49 },
    { 0x2882a6a2, SUC_lexicon_tags_87 },
    { 0x9fa2a6a1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb0a2a6a6, SUC_lexicon_tags_15 },
    { 0x1a42a6a7, SUC_lexicon_tags_19 },
    { 0xd762a6a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb162a6b2, SUC_lexicon_tags_25 },
    { 0x4c42a6b2, SUC_lexicon_tags_17 },
    { 0x5e82a6b2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x26e2a6b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa2a6b9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc522a6bd, SUC_lexicon_tags_23 },
    { 0x6382a6be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac2a6c3, SUC_lexicon_tags_17 },
    { 0xaf42a6c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf042a6c8, SUC_lexicon_tags_12 },
    { 0xe062a6c9, SUC_lexicon_tags_17 },
    { 0xa882a6ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x55c2a6cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6802a6cf, SUC_lexicon_tags_12 },
    { 0x1d82a6cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642a6d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b22a6d8, SUC_lexicon_tags_31 },
    { 0xb582a6d9, SUC_lexicon_tags_37 },
    { 0x8922a6da, SUC_lexicon_tags_97 },
    { 0xd402a6da, SUC_lexicon_tags_59 },
    { 0x4482a6da, SUC_lexicon_tags_7 },
    { 0xe1a2a6db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc2a6e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd742a6eb, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e2a6f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef62a6f2, SUC_lexicon_tags_46 },
    { 0x2602a6f3, SUC_lexicon_tags_46 },
    { 0xf7e2a6f3, SUC_lexicon_tags_25 },
    { 0x3822a6f2, SUC_lexicon_tags_25 },
    { 0x37a2a6f6, SUC_lexicon_tags_4 },
    { 0x4d42a6f7, SUC_lexicon_tags_17 },
    { 0xe6e2a6f2, SUC_lexicon_tags_38 },
    { 0x58a2a6f5, SUC_lexicon_tags_66 },
    { 0x6d42a6fa, SUC_lexicon_tags_37 },
    { 0xa882a6fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8e2a6fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f62a702, SUC_lexicon_tags_47 },
    { 0xcb62a702, SUC_lexicon_tags_25 },
    { 0x70a2a704, SUC_lexicon_tags_1 },
    { 0xa022a705, SUC_lexicon_tags_25 },
    { 0x9122a702, SUC_lexicon_tags_14 },
    { 0xe7a2a707, SUC_lexicon_tags_1 },
    { 0xa5c2a707, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9962a70e, SUC_lexicon_tags_8 },
    { 0xe3e2a70f, SUC_lexicon_tags_37 },
    { 0x7f42a70f, SUC_lexicon_tags_47 },
    { 0xa602a711, SUC_lexicon_tags_14 },
    { 0xa9e2a712, SUC_lexicon_tags_11 },
    { 0x7882a713, SUC_lexicon_tags_19 },
    { 0xf122a70e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8842a718, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1002a71f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2a725, SUC_lexicon_tags_208 },
    { 0xbe02a725, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c42a728, SUC_lexicon_tags_47 },
    { 0x35a2a728, SUC_lexicon_tags_37 },
    { 0x98e2a728, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xe9c2a72c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa262a72f, SUC_lexicon_tags_7 },
    { 0xc8a2a72f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb082a732, SUC_lexicon_tags_15 },
    { 0x79c2a733, SUC_lexicon_tags_45 },
    { 0x82a733, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc262a738, SUC_lexicon_tags_87 },
    { 0x7c62a739, SUC_lexicon_tags_37 },
    { 0xb62a738, SUC_lexicon_tags_1 },
    { 0xcda2a73b, SUC_lexicon_tags_14 },
    { 0x1662a73c, SUC_lexicon_tags_39 },
    { 0xe82a73c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xaea2a73f, SUC_lexicon_tags_43 },
    { 0xba22a73f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc02a742, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc2a745, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc82a749, SUC_lexicon_tags_35 },
    { 0xdec2a749, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6222a74d, SUC_lexicon_tags_1 },
    { 0xcf42a74e, SUC_lexicon_tags_17 },
    { 0x92a2a74f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8442a754, SUC_lexicon_tags_11 },
    { 0x61e2a754, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb822a758, SUC_lexicon_tags_15 },
    { 0x9522a759, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xc1c2a75b, SUC_lexicon_tags_7 },
    { 0xeec2a75c, SUC_lexicon_tags_47 },
    { 0xef62a75d, SUC_lexicon_tags_1 },
    { 0xe282a75e, SUC_lexicon_tags_15 },
    { 0xd922a75e, SUC_lexicon_tags_42 },
    { 0xb302a75c, SUC_lexicon_tags_19 },
    { 0x8942a75b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8122a764, SUC_lexicon_tags_9 },
    { 0x2d82a765, SUC_lexicon_tags_16 },
    { 0x8362a766, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e2a769, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1722a770, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7782a773, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e2a77c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7202a77e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba2a783, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3742a789, SUC_lexicon_tags_87 },
    { 0x5682a789, SUC_lexicon_tags_33 },
    { 0x4662a78b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7762a78d, SUC_lexicon_tags_17 },
    { 0x6502a78d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc2a791, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe322a794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c2a797, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa2a79a, SUC_lexicon_tags_5 },
    { 0x482a79b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c2a7a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e2a7ac, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a02a7b0, SUC_lexicon_tags_33 },
    { 0xfae2a7b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf462a7b8, SUC_lexicon_tags_43 },
    { 0x2902a7b8, SUC_lexicon_tags_11 },
    { 0xe262a7ba, SUC_lexicon_tags_26 },
    { 0xbbe2a7b9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa1a2a7bd, SUC_lexicon_tags_11 },
    { 0x9f42a7bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd682a7c0, SUC_lexicon_tags_17 },
    { 0x2b42a7c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a2a7ca, SUC_lexicon_tags_6 },
    { 0x8cc2a7cb, SUC_lexicon_tags_9 },
    { 0x5fe2a7ca, SUC_lexicon_tags_12 },
    { 0x7202a7cd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb742a7d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22e2a7d3, SUC_lexicon_tags_17 },
    { 0x9022a7d3, SUC_lexicon_tags_26 },
    { 0x9382a7d4, SUC_lexicon_tags_27 },
    { 0x6302a7d4, SUC_lexicon_tags_63 },
    { 0xa382a7d3, SUC_lexicon_tags_12 },
    { 0x7882a7d8, SUC_lexicon_tags_127 },
    { 0xc642a7d9, SUC_lexicon_tags_17 },
    { 0x75a2a7d8, SUC_lexicon_tags_10 },
    { 0x92e2a7db, SUC_lexicon_tags_46 },
    { 0x9d22a7d7, SUC_lexicon_tags_37 },
    { 0xb762a7dd, SUC_lexicon_tags_38 },
    { 0x8022a7de, SUC_lexicon_tags_12 },
    { 0x17c2a7d6, SUC_lexicon_tags_96 },
    { 0x6d22a7d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc82a7e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b02a7e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e2a7eb, SUC_lexicon_tags_2 },
    { 0x9522a7eb, SUC_lexicon_tags_25 },
    { 0x7ce2a7ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa862a7f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8402a7f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8722a7f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e2a7f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb822a7fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e2a801, SUC_lexicon_tags_60 },
    { 0x1702a801, SUC_lexicon_tags_17 },
    { 0x8102a803, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf542a805, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca2a808, SUC_lexicon_tags_26 },
    { 0x5102a809, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb62a80b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c2a815, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7102a818, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa42a820, SUC_lexicon_tags_1 },
    { 0x1962a820, SUC_lexicon_tags_26 },
    { 0x59e2a822, SUC_lexicon_tags_14 },
    { 0x782a823, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c2a826, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d82a829, SUC_lexicon_tags_2 },
    { 0xa0a2a829, SUC_lexicon_tags_65 },
    { 0xf022a82b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa222a82e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce62a832, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa2a835, SUC_lexicon_tags_60 },
    { 0x5082a836, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc582a83b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa2a83e, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4802a842, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7222a844, SUC_lexicon_tags_25 },
    { 0xafa2a845, SUC_lexicon_tags_1 },
    { 0xed22a846, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7542a84b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f02a84e, SUC_lexicon_tags_27 },
    { 0x5342a84f, SUC_lexicon_tags_65 },
    { 0x42e2a850, SUC_lexicon_tags_25 },
    { 0xb842a851, SUC_lexicon_tags_30 },
    { 0xc462a84f, SUC_lexicon_tags_25 },
    { 0xb982a853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fe2a855, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5c02a857, SUC_lexicon_tags_17 },
    { 0x7a42a858, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de2a85b, SUC_lexicon_tags_21 },
    { 0x8022a85c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc2a85f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e2a862, SUC_lexicon_tags_7 },
    { 0x9f02a862, SUC_lexicon_tags_80 },
    { 0x7d22a862, SUC_lexicon_tags_14 },
    { 0x5a82a864, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb782a869, SUC_lexicon_tags_49 },
    { 0x94a2a86a, SUC_lexicon_tags_21 },
    { 0x70a2a86a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xec62a86e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x38e2a870, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc42a878, SUC_lexicon_tags_17 },
    { 0xa562a879, SUC_lexicon_tags_63 },
    { 0xe182a879, SUC_lexicon_tags_37 },
    { 0x14e2a879, SUC_lexicon_tags_29 },
    { 0x1ce2a87c, SUC_lexicon_tags_27 },
    { 0x6362a87d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2602a87f, SUC_lexicon_tags_6 },
    { 0x3e02a87f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde22a886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e2a88b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd442a891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb62a895, SUC_lexicon_tags_17 },
    { 0xe0a2a896, SUC_lexicon_tags_11 },
    { 0x6322a897, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8ea2a899, SUC_lexicon_tags_49 },
    { 0x6be2a89a, SUC_lexicon_tags_2 },
    { 0xfe62a89b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c2a89f, SUC_lexicon_tags_29 },
    { 0xe4a2a89f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e2a8a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1102a8a6, SUC_lexicon_tags_35 },
    { 0xe742a8a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3f42a8a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62a8ac, SUC_lexicon_tags_38 },
    { 0xc1c2a8ad, SUC_lexicon_tags_34 },
    { 0x5ec2a8ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb6a2a8b0, SUC_lexicon_tags_84 },
    { 0x7b02a8b1, SUC_lexicon_tags_11 },
    { 0x6de2a8b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6bc2a8b4, SUC_lexicon_tags_1 },
    { 0x4ea2a8b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e42a8ba, SUC_lexicon_tags_179 },
    { 0xa922a8bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x98e2a8bd, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x3902a8bf, SUC_lexicon_tags_38 },
    { 0xd1a2a8c0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x81e2a8c2, SUC_lexicon_tags_34 },
    { 0xbc62a8c3, SUC_lexicon_tags_11 },
    { 0x1fe2a8c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc2a8c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6102a8cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x91c2a8cd, SUC_lexicon_tags_63 },
    { 0x95c2a8cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcb82a8d0, SUC_lexicon_tags_11 },
    { 0xc3a2a8d1, SUC_lexicon_tags_25 },
    { 0x38a2a8d2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5202a8d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6922a8dc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5022a8df, SUC_lexicon_tags_0 },
    { 0x3982a8e0, SUC_lexicon_tags_7 },
    { 0x4a82a8df, SUC_lexicon_tags_19 },
    { 0x76e2a8e1, SUC_lexicon_tags_65 },
    { 0xeee2a8e2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c2a8e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac82a8e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a2a8ee, SUC_lexicon_tags_23 },
    { 0x3d62a8ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f42a8f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f22a8f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a8fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5fc2a8fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc2a904, SUC_lexicon_tags_6 },
    { 0xfee2a905, SUC_lexicon_tags_5 },
    { 0xf9c2a905, SUC_lexicon_tags_26 },
    { 0x6902a906, SUC_lexicon_tags_25 },
    { 0x8262a907, SUC_lexicon_tags_13 },
    { 0x9482a909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd142a90f, SUC_lexicon_tags_7 },
    { 0x8b02a90f, SUC_lexicon_tags_23 },
    { 0xfce2a910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c2a914, SUC_lexicon_tags_7 },
    { 0xbd42a914, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x422a917, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c2a91c, SUC_lexicon_tags_43 },
    { 0x4302a91d, SUC_lexicon_tags_2 },
    { 0xf702a91e, SUC_lexicon_tags_1 },
    { 0x1b42a91f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7382a924, SUC_lexicon_tags_11 },
    { 0xbca2a925, SUC_lexicon_tags_28 },
    { 0xa642a925, SUC_lexicon_tags_7 },
    { 0xfd62a927, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf562a92a, SUC_lexicon_tags_12 },
    { 0x51c2a92a, SUC_lexicon_tags_6 },
    { 0x63a2a92c, SUC_lexicon_tags_15 },
    { 0x8f02a92c, SUC_lexicon_tags_25 },
    { 0x5a02a92e, SUC_lexicon_tags_31 },
    { 0x802a92e, SUC_lexicon_tags_8 },
    { 0xac2a92e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a2a935, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c2a939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e2a93d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3822a946, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6322a948, SUC_lexicon_tags_7 },
    { 0xcc2a949, SUC_lexicon_tags_17 },
    { 0xe702a94a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c2a94d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe082a94f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa062a958, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a2a95b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea42a962, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3902a965, SUC_lexicon_tags_11 },
    { 0xabc2a965, SUC_lexicon_tags_37 },
    { 0x28a2a967, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a2a96a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc062a96c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2fe2a96e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xee42a970, SUC_lexicon_tags_17 },
    { 0x28e2a971, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe2a975, SUC_lexicon_tags_15 },
    { 0x7422a975, SUC_lexicon_tags_47 },
    { 0x4f42a975, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5702a97a, SUC_lexicon_tags_45 },
    { 0xf5c2a97b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x79e2a97d, SUC_lexicon_tags_34 },
    { 0x8762a97d, SUC_lexicon_tags_92 },
    { 0xc3e2a97f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1542a982, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1742a984, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a2a989, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba02a98d, SUC_lexicon_tags_29 },
    { 0x8c42a98d, SUC_lexicon_tags_37 },
    { 0xc162a98d, SUC_lexicon_tags_1 },
    { 0xde22a990, SUC_lexicon_tags_107 },
    { 0xc902a990, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee2a995, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8322a997, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbb22a999, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2482a99c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab22a9a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe122a9ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85c2a9ad, SUC_lexicon_tags_0 },
    { 0x7c22a9ad, SUC_lexicon_tags_27 },
    { 0x722a9ad, SUC_lexicon_tags_45 },
    { 0xa282a9b0, SUC_lexicon_tags_558 },
    { 0, NULL },
    { 0xf1e2a9b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e2a9b7, SUC_lexicon_tags_17 },
    { 0x10a2a9b7, SUC_lexicon_tags_15 },
    { 0xd0c2a9b9, SUC_lexicon_tags_1 },
    { 0x8c2a9b9, SUC_lexicon_tags_39 },
    { 0x15c2a9b8, SUC_lexicon_tags_1 },
    { 0xef02a9bc, SUC_lexicon_tags_6 },
    { 0x4362a9bd, SUC_lexicon_tags_9 },
    { 0x8c62a9bc, SUC_lexicon_tags_11 },
    { 0x8062a9bf, SUC_lexicon_tags_37 },
    { 0x5462a9c0, SUC_lexicon_tags_1 },
    { 0x1242a9c1, SUC_lexicon_tags_1 },
    { 0xbcc2a9bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba02a9c7, SUC_lexicon_tags_7 },
    { 0xedc2a9c8, SUC_lexicon_tags_28 },
    { 0xc402a9c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x722a9cb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442a9cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe402a9d2, SUC_lexicon_tags_27 },
    { 0x5662a9d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3662a9da, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd82a9dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c2a9e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce2a9e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad22a9e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf962a9ea, SUC_lexicon_tags_45 },
    { 0xfc82a9eb, SUC_lexicon_tags_1 },
    { 0x4862a9ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe302a9f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2902a9f5, SUC_lexicon_tags_19 },
    { 0x8002a9f6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c2a9fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a02aa00, SUC_lexicon_tags_37 },
    { 0x32a2aa01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb862aa03, SUC_lexicon_tags_25 },
    { 0xd782aa04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8782aa07, SUC_lexicon_tags_37 },
    { 0x2962aa08, SUC_lexicon_tags_37 },
    { 0x762aa09, SUC_lexicon_tags_30 },
    { 0xb202aa0a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22c2aa0c, SUC_lexicon_tags_109 },
    { 0x3242aa0d, SUC_lexicon_tags_15 },
    { 0x2d02aa0e, SUC_lexicon_tags_17 },
    { 0x4ac2aa0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da2aa16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec62aa18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b62aa1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa4c2aa1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c2aa27, SUC_lexicon_tags_25 },
    { 0x402aa28, SUC_lexicon_tags_65 },
    { 0x65c2aa29, SUC_lexicon_tags_67 },
    { 0x4982aa2a, SUC_lexicon_tags_37 },
    { 0xc662aa2b, SUC_lexicon_tags_25 },
    { 0xb962aa2c, SUC_lexicon_tags_62 },
    { 0x3a02aa29, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7302aa2f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb942aa31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a22aa34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c2aa3a, SUC_lexicon_tags_12 },
    { 0x24c2aa3b, SUC_lexicon_tags_10 },
    { 0xc3e2aa3c, SUC_lexicon_tags_28 },
    { 0x65a2aa3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffe2aa3f, SUC_lexicon_tags_11 },
    { 0x9142aa40, SUC_lexicon_tags_47 },
    { 0x8282aa41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae62aa45, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe4c2aa47, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822aa52, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e2aa56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c2aa5d, SUC_lexicon_tags_0 },
    { 0x4342aa5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd882aa66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a02aa69, SUC_lexicon_tags_12 },
    { 0xb682aa6a, SUC_lexicon_tags_57 },
    { 0xd522aa6b, SUC_lexicon_tags_33 },
    { 0x42e2aa6c, SUC_lexicon_tags_11 },
    { 0x7202aa6b, SUC_lexicon_tags_17 },
    { 0xf8a2aa6e, SUC_lexicon_tags_12 },
    { 0xa9e2aa6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb62aa72, SUC_lexicon_tags_23 },
    { 0xffc2aa72, SUC_lexicon_tags_45 },
    { 0xc882aa73, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e62aa79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c2aa7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcd02aa80, SUC_lexicon_tags_55 },
    { 0x1f22aa81, SUC_lexicon_tags_66 },
    { 0x882aa81, SUC_lexicon_tags_51 },
    { 0x4f42aa83, SUC_lexicon_tags_7 },
    { 0x9ba2aa84, SUC_lexicon_tags_7 },
    { 0x8082aa84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f22aa89, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8822aa8e, SUC_lexicon_tags_38 },
    { 0x6a42aa8f, SUC_lexicon_tags_27 },
    { 0x81a2aa90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab42aa92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc2aa97, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c02aa9c, SUC_lexicon_tags_33 },
    { 0x36a2aa9d, SUC_lexicon_tags_35 },
    { 0x8ac2aa9e, SUC_lexicon_tags_1 },
    { 0xf662aa9f, SUC_lexicon_tags_0 },
    { 0xe6c2aaa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca02aaa3, SUC_lexicon_tags_27 },
    { 0xba2aaa4, SUC_lexicon_tags_14 },
    { 0xd302aaa5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c2aaa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc782aaaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e2aab3, SUC_lexicon_tags_398 },
    { 0xf982aab3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec2aab8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6062aaba, SUC_lexicon_tags_11 },
    { 0x3e82aabb, SUC_lexicon_tags_21 },
    { 0x1622aabc, SUC_lexicon_tags_55 },
    { 0x9ea2aaba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc662aabf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3082aac5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8442aac9, SUC_lexicon_tags_17 },
    { 0xf5c2aac9, SUC_lexicon_tags_2 },
    { 0xa0e2aacb, SUC_lexicon_tags_31 },
    { 0x6e82aacc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a02aad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c2aad4, SUC_lexicon_tags_25 },
    { 0x6f82aad5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x40c2aad7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2e2aad9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e2aae1, SUC_lexicon_tags_25 },
    { 0x3842aae1, SUC_lexicon_tags_60 },
    { 0xe2aae3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa2aae8, SUC_lexicon_tags_37 },
    { 0x6122aae8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2c2aaeb, SUC_lexicon_tags_17 },
    { 0x2e2aaeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c2aaf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a2aaf3, SUC_lexicon_tags_15 },
    { 0x5322aaf4, SUC_lexicon_tags_7 },
    { 0x61c2aaf5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ac2aaf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c2aafb, SUC_lexicon_tags_66 },
    { 0x6542aafc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2862aaff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf742ab04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc22ab0a, SUC_lexicon_tags_79 },
    { 0x7f22ab0b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc9c2ab0d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd22ab11, SUC_lexicon_tags_11 },
    { 0xf9a2ab12, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9302ab1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea2ab1c, SUC_lexicon_tags_11 },
    { 0xf042ab1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee2ab22, SUC_lexicon_tags_6 },
    { 0x6122ab23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5922ab25, SUC_lexicon_tags_138 },
    { 0x6c62ab26, SUC_lexicon_tags_63 },
    { 0x7f22ab27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c2ab2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca2ab2e, SUC_lexicon_tags_68 },
    { 0xf7c2ab2e, SUC_lexicon_tags_5 },
    { 0x1062ab2e, SUC_lexicon_tags_11 },
    { 0xe842ab31, SUC_lexicon_tags_34 },
    { 0x7a22ab32, SUC_lexicon_tags_63 },
    { 0xd422ab2f, SUC_lexicon_tags_38 },
    { 0x8fe2ab31, SUC_lexicon_tags_55 },
    { 0x7f82ab35, SUC_lexicon_tags_2 },
    { 0xf942ab36, SUC_lexicon_tags_12 },
    { 0xbee2ab36, SUC_lexicon_tags_39 },
    { 0xa1a2ab32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd02ab3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5382ab3f, SUC_lexicon_tags_23 },
    { 0x2aa2ab40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7862ab42, SUC_lexicon_tags_37 },
    { 0x262ab43, SUC_lexicon_tags_17 },
    { 0xf002ab43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c2ab47, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e82ab4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6562ab4d, SUC_lexicon_tags_12 },
    { 0x6ee2ab4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff42ab51, SUC_lexicon_tags_104 },
    { 0xf202ab52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa002ab56, SUC_lexicon_tags_23 },
    { 0xcfa2ab57, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe642ab59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a42ab61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c82ab64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a2ab68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa422ab6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c2ab6e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x77a2ab70, SUC_lexicon_tags_12 },
    { 0x6ec2ab71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8482ab73, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0x4402ab75, SUC_lexicon_tags_13 },
    { 0x72a2ab76, SUC_lexicon_tags_2 },
    { 0x7da2ab75, SUC_lexicon_tags_30 },
    { 0xd322ab78, SUC_lexicon_tags_30 },
    { 0x3062ab79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9502ab83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb82ab89, SUC_lexicon_tags_33 },
    { 0x1622ab8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae2ab8f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a2ab93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf642ab96, SUC_lexicon_tags_29 },
    { 0xc382ab97, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e2ab9d, SUC_lexicon_tags_31 },
    { 0x2e42ab9e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe802aba1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc142aba7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a2abaa, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb822abb2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x80e2abb4, SUC_lexicon_tags_4 },
    { 0x72c2abb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeba2abb7, SUC_lexicon_tags_15 },
    { 0x6022abb8, SUC_lexicon_tags_7 },
    { 0x3442abb8, SUC_lexicon_tags_2 },
    { 0xe8c2abb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1602abbd, SUC_lexicon_tags_13 },
    { 0x44a2abbd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xed82abc1, SUC_lexicon_tags_26 },
    { 0x3c02abc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa82abc4, SUC_lexicon_tags_39 },
    { 0x6602abc4, SUC_lexicon_tags_235 },
    { 0x7b62abc5, SUC_lexicon_tags_0 },
    { 0x1f82abc4, SUC_lexicon_tags_45 },
    { 0x342abc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9322abca, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0x5682abcc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf522abd0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9ec2abd2, SUC_lexicon_tags_17 },
    { 0x98a2abd2, SUC_lexicon_tags_2 },
    { 0xf62abd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4542abd7, SUC_lexicon_tags_6 },
    { 0xbd42abd8, SUC_lexicon_tags_27 },
    { 0xdbc2abd9, SUC_lexicon_tags_37 },
    { 0xe822abda, SUC_lexicon_tags_11 },
    { 0xd8a2abdb, SUC_lexicon_tags_21 },
    { 0xeec2abd9, SUC_lexicon_tags_9 },
    { 0x5da2abdd, SUC_lexicon_tags_14 },
    { 0x6fa2abdb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e02abe5, SUC_lexicon_tags_25 },
    { 0x6bc2abe6, SUC_lexicon_tags_11 },
    { 0x8e2abe5, SUC_lexicon_tags_26 },
    { 0x54c2abe7, SUC_lexicon_tags_38 },
    { 0xa522abe9, SUC_lexicon_tags_46 },
    { 0x3aa2abea, SUC_lexicon_tags_43 },
    { 0x70a2abe8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe302abee, SUC_lexicon_tags_45 },
    { 0x8e62abef, SUC_lexicon_tags_7 },
    { 0x6962abef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8802abf3, SUC_lexicon_tags_7 },
    { 0x84c2abf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd22abfa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8282abfd, SUC_lexicon_tags_37 },
    { 0xa3e2abfe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8e42ac00, SUC_lexicon_tags_11 },
    { 0xa502ac01, SUC_lexicon_tags_45 },
    { 0x3b22ac01, SUC_lexicon_tags_49 },
    { 0x4a42ac03, SUC_lexicon_tags_16 },
    { 0x2c22ac04, SUC_lexicon_tags_37 },
    { 0xc042ac04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e82ac08, SUC_lexicon_tags_112 },
    { 0x2282ac09, SUC_lexicon_tags_19 },
    { 0x73a2ac0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a2ac0c, SUC_lexicon_tags_61 },
    { 0x8642ac0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec42ac0f, SUC_lexicon_tags_4 },
    { 0x56c2ac10, SUC_lexicon_tags_15 },
    { 0x3002ac11, SUC_lexicon_tags_1 },
    { 0x9082ac12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4562ac15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c42ac1d, SUC_lexicon_tags_37 },
    { 0x1142ac1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c2ac21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61e2ac23, SUC_lexicon_tags_47 },
    { 0x2102ac24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb742ac2b, SUC_lexicon_tags_87 },
    { 0x1e82ac2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa382ac2e, SUC_lexicon_tags_1 },
    { 0xe402ac2f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d22ac34, SUC_lexicon_tags_26 },
    { 0xfc22ac35, SUC_lexicon_tags_10 },
    { 0xda42ac34, SUC_lexicon_tags_32 },
    { 0x6202ac35, SUC_lexicon_tags_7 },
    { 0x93a2ac36, SUC_lexicon_tags_60 },
    { 0xebe2ac34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7f02ac3b, SUC_lexicon_tags_0 },
    { 0x9a42ac3c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x85e2ac3e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d22ac41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd762ac44, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a2ac4a, SUC_lexicon_tags_84 },
    { 0x5f22ac4a, SUC_lexicon_tags_1 },
    { 0x2ce2ac4c, SUC_lexicon_tags_43 },
    { 0x7022ac4c, SUC_lexicon_tags_23 },
    { 0x2da2ac4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c2ac51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbea2ac53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd82ac57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402ac5f, SUC_lexicon_tags_12 },
    { 0xf022ac60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4782ac63, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c2ac66, SUC_lexicon_tags_79 },
    { 0xe902ac67, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9682ac6a, SUC_lexicon_tags_1 },
    { 0x8202ac6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd8e2ac6d, SUC_lexicon_tags_23 },
    { 0xc62ac6d, SUC_lexicon_tags_31 },
    { 0x7042ac6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1582ac76, SUC_lexicon_tags_37 },
    { 0xdc22ac76, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaf42ac79, SUC_lexicon_tags_29 },
    { 0xcb22ac7a, SUC_lexicon_tags_3 },
    { 0x5d02ac7a, SUC_lexicon_tags_8 },
    { 0xbfe2ac79, SUC_lexicon_tags_66 },
    { 0xc822ac7d, SUC_lexicon_tags_1 },
    { 0xadc2ac7c, SUC_lexicon_tags_12 },
    { 0xe5a2ac7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6a2ac81, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2ac87, SUC_lexicon_tags_25 },
    { 0xdb22ac88, SUC_lexicon_tags_45 },
    { 0xe02ac87, SUC_lexicon_tags_25 },
    { 0xe562ac8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5222ac8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6622ac92, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8142ac94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22ac97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6782ac99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc622ac9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6622aca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662aca6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5bc2aca8, SUC_lexicon_tags_33 },
    { 0xb1c2aca9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb942acab, SUC_lexicon_tags_14 },
    { 0x22a2acab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8802acaf, SUC_lexicon_tags_33 },
    { 0xfb42acb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd922acb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8be2acb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f02acb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xece2acba, SUC_lexicon_tags_23 },
    { 0xa462acbb, SUC_lexicon_tags_37 },
    { 0xab82acbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa2acbf, SUC_lexicon_tags_19 },
    { 0x33e2acc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3902acc7, SUC_lexicon_tags_7 },
    { 0xcc2acc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8502accb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfc82accd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c2acd3, SUC_lexicon_tags_26 },
    { 0x5bc2acd4, SUC_lexicon_tags_2 },
    { 0xdfc2acd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e2acd9, SUC_lexicon_tags_2 },
    { 0x7c62acd9, SUC_lexicon_tags_17 },
    { 0x1fa2acd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4222acdd, SUC_lexicon_tags_15 },
    { 0xbf22acde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f42ace1, SUC_lexicon_tags_17 },
    { 0x9e42ace2, SUC_lexicon_tags_45 },
    { 0x2962ace3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3002ace7, SUC_lexicon_tags_1 },
    { 0x6462ace8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7262aceb, SUC_lexicon_tags_6 },
    { 0x7ca2acec, SUC_lexicon_tags_38 },
    { 0xe742aced, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5742acef, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x2082acf2, SUC_lexicon_tags_27 },
    { 0x8782acf2, SUC_lexicon_tags_47 },
    { 0xff02acf4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5082acf7, SUC_lexicon_tags_4 },
    { 0xc762acf8, SUC_lexicon_tags_25 },
    { 0xeca2acf8, SUC_lexicon_tags_38 },
    { 0x8882acf9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcfc2acfc, SUC_lexicon_tags_387 },
    { 0x3862acfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1362acff, SUC_lexicon_tags_38 },
    { 0x5842acff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc8a2ad02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7222ad04, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf02ad07, SUC_lexicon_tags_33 },
    { 0xab82ad08, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc2ad0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e2ad17, SUC_lexicon_tags_87 },
    { 0x8482ad18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ce2ad1a, SUC_lexicon_tags_68 },
    { 0x31e2ad1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc342ad20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a02ad2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d82ad34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea2ad39, SUC_lexicon_tags_63 },
    { 0xb402ad39, SUC_lexicon_tags_0 },
    { 0xa862ad39, SUC_lexicon_tags_11 },
    { 0xe8e2ad3c, SUC_lexicon_tags_2 },
    { 0x8742ad3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6302ad40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd382ad42, SUC_lexicon_tags_25 },
    { 0x2e82ad42, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922ad47, SUC_lexicon_tags_56 },
    { 0xbf02ad47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x22e2ad4a, SUC_lexicon_tags_11 },
    { 0x4f82ad4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb22ad4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c2ad52, SUC_lexicon_tags_38 },
    { 0x10c2ad53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48c2ad55, SUC_lexicon_tags_63 },
    { 0x7f62ad56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2382ad59, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2ad5d, SUC_lexicon_tags_7 },
    { 0x6e02ad5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b42ad62, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xba02ad65, SUC_lexicon_tags_31 },
    { 0x2622ad66, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x53c2ad68, SUC_lexicon_tags_33 },
    { 0xfa62ad69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a2ad6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2d82ad70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2262ad72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8502ad78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ad7b, SUC_lexicon_tags_25 },
    { 0xf462ad7b, SUC_lexicon_tags_16 },
    { 0x9dc2ad7d, SUC_lexicon_tags_1 },
    { 0xdd82ad7e, SUC_lexicon_tags_63 },
    { 0xb242ad7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19c2ad81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c2ad84, SUC_lexicon_tags_14 },
    { 0xa7c2ad85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa02ad88, SUC_lexicon_tags_87 },
    { 0xa302ad89, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0xfa62ad8b, SUC_lexicon_tags_6 },
    { 0x65a2ad8b, SUC_lexicon_tags_25 },
    { 0xb2a2ad8d, SUC_lexicon_tags_2 },
    { 0xb02ad8e, SUC_lexicon_tags_14 },
    { 0x9142ad8e, SUC_lexicon_tags_32 },
    { 0xf882ad8c, SUC_lexicon_tags_17 },
    { 0xaa2ad8d, SUC_lexicon_tags_25 },
    { 0xd0e2ad92, SUC_lexicon_tags_12 },
    { 0x5022ad93, SUC_lexicon_tags_84 },
    { 0xc802ad8c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be2ad97, SUC_lexicon_tags_46 },
    { 0xb922ad98, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5142ad9b, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de2ad9e, SUC_lexicon_tags_14 },
    { 0xbbe2ad9f, SUC_lexicon_tags_32 },
    { 0x32a2ad9f, SUC_lexicon_tags_13 },
    { 0x582ad9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e2adaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22adb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e02adb4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x85e2adb6, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x7002adb8, SUC_lexicon_tags_17 },
    { 0xa5c2adb9, SUC_lexicon_tags_77 },
    { 0xe702adba, SUC_lexicon_tags_15 },
    { 0x51c2adba, SUC_lexicon_tags_12 },
    { 0xd582adba, SUC_lexicon_tags_37 },
    { 0x58e2adbd, SUC_lexicon_tags_37 },
    { 0x8982adbc, SUC_lexicon_tags_7 },
    { 0xc022adbf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4382adc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc822adc5, SUC_lexicon_tags_15 },
    { 0xea2adc5, SUC_lexicon_tags_37 },
    { 0x2ca2adc6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c2add1, SUC_lexicon_tags_4 },
    { 0xba82add1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd022add6, SUC_lexicon_tags_63 },
    { 0xa322add7, SUC_lexicon_tags_8 },
    { 0xfb42add8, SUC_lexicon_tags_32 },
    { 0x3b22add9, SUC_lexicon_tags_26 },
    { 0xda42adda, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e2addd, SUC_lexicon_tags_21 },
    { 0xffc2addd, SUC_lexicon_tags_15 },
    { 0x5e42addf, SUC_lexicon_tags_48 },
    { 0xbde2addf, SUC_lexicon_tags_12 },
    { 0x19a2ade0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf442ade4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc082ade9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc5c2adeb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e2adef, SUC_lexicon_tags_6 },
    { 0xbba2adf0, SUC_lexicon_tags_11 },
    { 0x3242adf1, SUC_lexicon_tags_2 },
    { 0xb962adf1, SUC_lexicon_tags_25 },
    { 0x61e2adf2, SUC_lexicon_tags_11 },
    { 0x9962adf4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xc0c2adf6, SUC_lexicon_tags_25 },
    { 0x41a2adf7, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x2462adf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1922adfd, SUC_lexicon_tags_2 },
    { 0x1c42adfe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea2ae02, SUC_lexicon_tags_17 },
    { 0xaf42ae03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582ae08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae0d, SUC_lexicon_tags_23 },
    { 0xa382ae0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e2ae12, SUC_lexicon_tags_25 },
    { 0x2802ae13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f42ae17, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x7422ae19, SUC_lexicon_tags_31 },
    { 0xde22ae19, SUC_lexicon_tags_30 },
    { 0x2c22ae19, SUC_lexicon_tags_55 },
    { 0xa342ae1c, SUC_lexicon_tags_30 },
    { 0x6262ae1c, SUC_lexicon_tags_65 },
    { 0xd902ae1e, SUC_lexicon_tags_33 },
    { 0x36c2ae1e, SUC_lexicon_tags_63 },
    { 0x50a2ae1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3822ae23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2762ae27, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d62ae2b, SUC_lexicon_tags_7 },
    { 0x34a2ae2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc322ae2e, SUC_lexicon_tags_11 },
    { 0xd9c2ae2f, SUC_lexicon_tags_1 },
    { 0xa962ae30, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcba2ae32, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a2ae36, SUC_lexicon_tags_23 },
    { 0x322ae37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2742ae39, SUC_lexicon_tags_37 },
    { 0xff82ae3a, SUC_lexicon_tags_37 },
    { 0xe042ae3b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf4a2ae3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f42ae41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc462ae47, SUC_lexicon_tags_21 },
    { 0x3802ae48, SUC_lexicon_tags_30 },
    { 0x1242ae49, SUC_lexicon_tags_2 },
    { 0xe382ae4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4382ae4e, SUC_lexicon_tags_49 },
    { 0x38c2ae4f, SUC_lexicon_tags_15 },
    { 0x4682ae4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb562ae52, SUC_lexicon_tags_61 },
    { 0x1222ae52, SUC_lexicon_tags_2 },
    { 0x22a2ae54, SUC_lexicon_tags_17 },
    { 0x8282ae55, SUC_lexicon_tags_25 },
    { 0x78c2ae54, SUC_lexicon_tags_2 },
    { 0x6862ae56, SUC_lexicon_tags_32 },
    { 0xaf02ae58, SUC_lexicon_tags_32 },
    { 0xac82ae57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaae2ae5b, SUC_lexicon_tags_60 },
    { 0x8e62ae5c, SUC_lexicon_tags_7 },
    { 0xd362ae5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb22ae63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b62ae65, SUC_lexicon_tags_37 },
    { 0xfc42ae66, SUC_lexicon_tags_15 },
    { 0xa9a2ae67, SUC_lexicon_tags_284 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b62ae6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf042ae6d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d02ae7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45e2ae7d, SUC_lexicon_tags_14 },
    { 0xdec2ae7e, SUC_lexicon_tags_12 },
    { 0x9a62ae7e, SUC_lexicon_tags_85 },
    { 0xffe2ae7e, SUC_lexicon_tags_37 },
    { 0xfa82ae81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5362ae84, SUC_lexicon_tags_6 },
    { 0x8422ae85, SUC_lexicon_tags_17 },
    { 0xbc42ae86, SUC_lexicon_tags_14 },
    { 0x1f22ae87, SUC_lexicon_tags_1 },
    { 0x7042ae88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d22ae8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae2ae90, SUC_lexicon_tags_17 },
    { 0x9182ae91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f22ae99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xee22ae9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d22aea0, SUC_lexicon_tags_37 },
    { 0x782aea0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf82aea5, SUC_lexicon_tags_15 },
    { 0x2ba2aea6, SUC_lexicon_tags_17 },
    { 0xda2aea7, SUC_lexicon_tags_14 },
    { 0x1802aea7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x31a2aeaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab02aeac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82aeaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e2aeb2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c62aeb5, SUC_lexicon_tags_15 },
    { 0x8262aeb5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x522aeb8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c2aebb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7662aebd, SUC_lexicon_tags_15 },
    { 0xa942aebe, SUC_lexicon_tags_45 },
    { 0x27e2aebf, SUC_lexicon_tags_57 },
    { 0x6802aec0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb82aec3, SUC_lexicon_tags_11 },
    { 0x2922aec3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3062aec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35c2aeca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2062aecc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42aed1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd942aedb, SUC_lexicon_tags_17 },
    { 0x962aedc, SUC_lexicon_tags_37 },
    { 0xce82aedc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe842aee0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf622aee2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c2aee7, SUC_lexicon_tags_11 },
    { 0x2a2aee7, SUC_lexicon_tags_8 },
    { 0x8362aee9, SUC_lexicon_tags_11 },
    { 0xede2aee9, SUC_lexicon_tags_16 },
    { 0xf162aeeb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4282aeed, SUC_lexicon_tags_6 },
    { 0x7082aeed, SUC_lexicon_tags_6 },
    { 0xcce2aeef, SUC_lexicon_tags_47 },
    { 0xce2aef0, SUC_lexicon_tags_25 },
    { 0x52c2aef0, SUC_lexicon_tags_118 },
    { 0x4842aef1, SUC_lexicon_tags_38 },
    { 0x142aef3, SUC_lexicon_tags_45 },
    { 0xf482aef4, SUC_lexicon_tags_27 },
    { 0x7b82aef4, SUC_lexicon_tags_7 },
    { 0x6282aeed, SUC_lexicon_tags_129 },
    { 0x83e2aef1, SUC_lexicon_tags_234 },
    { 0x3482aef8, SUC_lexicon_tags_17 },
    { 0xf522aef9, SUC_lexicon_tags_15 },
    { 0x3082aef9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a2af02, SUC_lexicon_tags_1 },
    { 0x9d42af03, SUC_lexicon_tags_31 },
    { 0x4462af04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba2af0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7702af10, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be2af13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a2af19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e2af20, SUC_lexicon_tags_12 },
    { 0xe462af21, SUC_lexicon_tags_38 },
    { 0xc8c2af21, SUC_lexicon_tags_4 },
    { 0xf802af22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e22af27, SUC_lexicon_tags_23 },
    { 0xf802af28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c2af2c, SUC_lexicon_tags_25 },
    { 0x3be2af2d, SUC_lexicon_tags_36 },
    { 0xba82af2e, SUC_lexicon_tags_11 },
    { 0x2202af2d, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xf5e2af31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22af34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5062af36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e2af3b, SUC_lexicon_tags_87 },
    { 0x2942af3c, SUC_lexicon_tags_2 },
    { 0x9382af3c, SUC_lexicon_tags_25 },
    { 0xe62af3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81a2af40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2e2af42, SUC_lexicon_tags_14 },
    { 0x2e22af42, SUC_lexicon_tags_8 },
    { 0x5f62af44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a82af48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc342af4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb02af4e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc782af50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7542af54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a22af57, SUC_lexicon_tags_56 },
    { 0x3922af58, SUC_lexicon_tags_15 },
    { 0x6622af58, SUC_lexicon_tags_45 },
    { 0x9ba2af58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac82af5c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec2af61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x25e2af63, SUC_lexicon_tags_47 },
    { 0x32e2af64, SUC_lexicon_tags_49 },
    { 0x9382af64, SUC_lexicon_tags_25 },
    { 0x8e2af65, SUC_lexicon_tags_7 },
    { 0x1642af67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2af70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3bc2af72, SUC_lexicon_tags_2 },
    { 0x4f42af73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6282af75, SUC_lexicon_tags_7 },
    { 0x8b62af75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d62af78, SUC_lexicon_tags_65 },
    { 0xb0a2af79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf462af7b, SUC_lexicon_tags_2 },
    { 0xd162af7c, SUC_lexicon_tags_31 },
    { 0xa522af7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c02af7f, SUC_lexicon_tags_61 },
    { 0x9142af80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe502af84, SUC_lexicon_tags_14 },
    { 0x3082af85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6842af88, SUC_lexicon_tags_11 },
    { 0x5322af89, SUC_lexicon_tags_23 },
    { 0x302af88, SUC_lexicon_tags_15 },
    { 0x5282af8b, SUC_lexicon_tags_25 },
    { 0xea2af8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c2af8f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5162af91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x642af94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd842af97, SUC_lexicon_tags_24 },
    { 0x2822af97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c42af9a, SUC_lexicon_tags_7 },
    { 0x9882af9b, SUC_lexicon_tags_27 },
    { 0xebe2af9c, SUC_lexicon_tags_11 },
    { 0x51a2af9c, SUC_lexicon_tags_36 },
    { 0x8582af9e, SUC_lexicon_tags_11 },
    { 0x3402af9f, SUC_lexicon_tags_12 },
    { 0x72a2af9f, SUC_lexicon_tags_28 },
    { 0x17a2af9f, SUC_lexicon_tags_27 },
    { 0x4aa2afa2, SUC_lexicon_tags_2 },
    { 0x1e22afa3, SUC_lexicon_tags_68 },
    { 0x20c2afa2, SUC_lexicon_tags_15 },
    { 0x4d62afa5, SUC_lexicon_tags_19 },
    { 0xaf62afa6, SUC_lexicon_tags_11 },
    { 0xff42af9f, SUC_lexicon_tags_7 },
    { 0xb642afa7, SUC_lexicon_tags_15 },
    { 0x6742af9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf62afac, SUC_lexicon_tags_8 },
    { 0xb3c2afac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e2afb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf682afb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8422afb5, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xed02afb8, SUC_lexicon_tags_19 },
    { 0x2a42afb9, SUC_lexicon_tags_47 },
    { 0xd5c2afba, SUC_lexicon_tags_49 },
    { 0x2d22afba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d42afc2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c2afc6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2322afc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe2afcc, SUC_lexicon_tags_34 },
    { 0x5022afcd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c2afd0, SUC_lexicon_tags_60 },
    { 0x4282afd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a2afd6, SUC_lexicon_tags_37 },
    { 0x2f62afd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c2afda, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x5a02afdc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe622afe0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2afe4, SUC_lexicon_tags_12 },
    { 0x4042afe5, SUC_lexicon_tags_1 },
    { 0xd82afe6, SUC_lexicon_tags_8 },
    { 0x6da2afe4, SUC_lexicon_tags_2 },
    { 0xc162afe8, SUC_lexicon_tags_8 },
    { 0xbb22afe9, SUC_lexicon_tags_2 },
    { 0xbb02afe8, SUC_lexicon_tags_45 },
    { 0x9ce2afe7, SUC_lexicon_tags_12 },
    { 0xba22afe8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4e2afee, SUC_lexicon_tags_11 },
    { 0xb9a2afef, SUC_lexicon_tags_30 },
    { 0x11e2aff0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf7a2aff2, SUC_lexicon_tags_23 },
    { 0x9ec2aff3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb962aff5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x63c2aff7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf842affd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd162afff, SUC_lexicon_tags_14 },
    { 0xf982b000, SUC_lexicon_tags_31 },
    { 0x7902b000, SUC_lexicon_tags_12 },
    { 0x21c2afff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccc2b004, SUC_lexicon_tags_11 },
    { 0x27c2b005, SUC_lexicon_tags_8 },
    { 0xb602b005, SUC_lexicon_tags_39 },
    { 0xc3c2b005, SUC_lexicon_tags_12 },
    { 0x5b62b008, SUC_lexicon_tags_25 },
    { 0xaee2b008, SUC_lexicon_tags_11 },
    { 0x4822b004, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8602b00c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c02b010, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3202b013, SUC_lexicon_tags_7 },
    { 0x8582b014, SUC_lexicon_tags_60 },
    { 0xea2b015, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8402b018, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a2b01b, SUC_lexicon_tags_25 },
    { 0x5d62b01c, SUC_lexicon_tags_7 },
    { 0x4ea2b01d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc02b01f, SUC_lexicon_tags_14 },
    { 0x3b02b020, SUC_lexicon_tags_31 },
    { 0xb9a2b020, SUC_lexicon_tags_15 },
    { 0xc7a2b022, SUC_lexicon_tags_3 },
    { 0x95a2b023, SUC_lexicon_tags_7 },
    { 0x49a2b024, SUC_lexicon_tags_35 },
    { 0xb542b025, SUC_lexicon_tags_37 },
    { 0x3162b023, SUC_lexicon_tags_25 },
    { 0x6f02b025, SUC_lexicon_tags_15 },
    { 0x2322b028, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee82b02c, SUC_lexicon_tags_8 },
    { 0xdf42b02c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc22b031, SUC_lexicon_tags_11 },
    { 0x1562b032, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c42b035, SUC_lexicon_tags_47 },
    { 0xb802b036, SUC_lexicon_tags_47 },
    { 0x7822b037, SUC_lexicon_tags_15 },
    { 0x5202b038, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e2b03c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x77a2b03e, SUC_lexicon_tags_1 },
    { 0x7302b03f, SUC_lexicon_tags_38 },
    { 0xf082b040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef42b044, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2b048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28e2b04a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02b053, SUC_lexicon_tags_2 },
    { 0x422b053, SUC_lexicon_tags_28 },
    { 0xc722b055, SUC_lexicon_tags_49 },
    { 0x3b22b053, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be2b059, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd162b05b, SUC_lexicon_tags_3 },
    { 0xc382b05b, SUC_lexicon_tags_1 },
    { 0xe3e2b05c, SUC_lexicon_tags_15 },
    { 0xdfa2b05c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6bc2b060, SUC_lexicon_tags_26 },
    { 0xd0a2b060, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1062b066, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e2b06d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf842b06f, SUC_lexicon_tags_1 },
    { 0x15a2b070, SUC_lexicon_tags_0 },
    { 0xdac2b071, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd962b073, SUC_lexicon_tags_14 },
    { 0xcd02b073, SUC_lexicon_tags_45 },
    { 0xac42b075, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b02b077, SUC_lexicon_tags_179 },
    { 0x1122b077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e2b07b, SUC_lexicon_tags_1 },
    { 0x6fa2b07c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd42b084, SUC_lexicon_tags_14 },
    { 0xaea2b085, SUC_lexicon_tags_15 },
    { 0x39e2b085, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd942b08e, SUC_lexicon_tags_84 },
    { 0x6de2b08f, SUC_lexicon_tags_25 },
    { 0xc462b090, SUC_lexicon_tags_17 },
    { 0x1162b08e, SUC_lexicon_tags_37 },
    { 0xf762b08e, SUC_lexicon_tags_1 },
    { 0x1242b08f, SUC_lexicon_tags_38 },
    { 0x1022b094, SUC_lexicon_tags_7 },
    { 0x2182b094, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8e2b097, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7402b099, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc762b09c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe222b09e, SUC_lexicon_tags_27 },
    { 0xcb02b09f, SUC_lexicon_tags_15 },
    { 0x9182b0a0, SUC_lexicon_tags_23 },
    { 0x89c2b0a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe2b0a5, SUC_lexicon_tags_12 },
    { 0xf6a2b0a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6842b0a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd822b0ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab62b0ae, SUC_lexicon_tags_0 },
    { 0x9a02b0af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda2b0b2, SUC_lexicon_tags_72 },
    { 0xccc2b0b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b42b0b9, SUC_lexicon_tags_26 },
    { 0xd182b0ba, SUC_lexicon_tags_37 },
    { 0x8602b0bb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c2b0bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e2b0c6, SUC_lexicon_tags_37 },
    { 0x46c2b0c7, SUC_lexicon_tags_12 },
    { 0xd882b0c7, SUC_lexicon_tags_14 },
    { 0xc4c2b0c9, SUC_lexicon_tags_151 },
    { 0x6082b0c6, SUC_lexicon_tags_43 },
    { 0x7822b0cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc2b0cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b62b0d2, SUC_lexicon_tags_15 },
    { 0x4e62b0d2, SUC_lexicon_tags_2 },
    { 0x8702b0d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7762b0d7, SUC_lexicon_tags_14 },
    { 0x18e2b0d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3282b0dc, SUC_lexicon_tags_26 },
    { 0xfb82b0dd, SUC_lexicon_tags_33 },
    { 0xea82b0de, SUC_lexicon_tags_37 },
    { 0xd3c2b0dc, SUC_lexicon_tags_25 },
    { 0x9662b0e0, SUC_lexicon_tags_47 },
    { 0xf702b0e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc2b0e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe222b0ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6802b0ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd082b0ee, SUC_lexicon_tags_4 },
    { 0xeb22b0ef, SUC_lexicon_tags_17 },
    { 0x8662b0ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6402b0f6, SUC_lexicon_tags_11 },
    { 0xa122b0f7, SUC_lexicon_tags_43 },
    { 0xf062b0f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62b0fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7282b0fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb82b103, SUC_lexicon_tags_32 },
    { 0x9402b104, SUC_lexicon_tags_7 },
    { 0xc02b104, SUC_lexicon_tags_39 },
    { 0x8642b104, SUC_lexicon_tags_1 },
    { 0x5222b106, SUC_lexicon_tags_37 },
    { 0x4d22b106, SUC_lexicon_tags_65 },
    { 0x1b42b109, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2582b10c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee2b111, SUC_lexicon_tags_158 },
    { 0x7682b112, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d62b118, SUC_lexicon_tags_37 },
    { 0x46a2b118, SUC_lexicon_tags_44 },
    { 0x4d22b11a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c42b11e, SUC_lexicon_tags_19 },
    { 0x70e2b11e, SUC_lexicon_tags_49 },
    { 0x7da2b120, SUC_lexicon_tags_15 },
    { 0xc482b121, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da2b128, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a82b131, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a2b137, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa742b140, SUC_lexicon_tags_7 },
    { 0x5e62b141, SUC_lexicon_tags_55 },
    { 0xa8e2b142, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe2b148, SUC_lexicon_tags_17 },
    { 0xd762b149, SUC_lexicon_tags_60 },
    { 0xbb22b148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9a2b14c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9c02b14e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa982b154, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf282b157, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdd82b159, SUC_lexicon_tags_12 },
    { 0xf7c2b159, SUC_lexicon_tags_29 },
    { 0xf102b15b, SUC_lexicon_tags_29 },
    { 0xdc82b15c, SUC_lexicon_tags_2 },
    { 0x14a2b15b, SUC_lexicon_tags_23 },
    { 0xc82b15e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22b162, SUC_lexicon_tags_84 },
    { 0x5542b162, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3162b167, SUC_lexicon_tags_15 },
    { 0xc222b168, SUC_lexicon_tags_12 },
    { 0xfce2b167, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7262b16b, SUC_lexicon_tags_16 },
    { 0xcee2b16c, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x63e2b16e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca2b173, SUC_lexicon_tags_77 },
    { 0xe7e2b174, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3842b177, SUC_lexicon_tags_47 },
    { 0xc3a2b178, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc2b17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3262b17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c2b17f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb142b181, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b82b183, SUC_lexicon_tags_38 },
    { 0x2962b184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be2b187, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982b18b, SUC_lexicon_tags_17 },
    { 0xa62b18c, SUC_lexicon_tags_37 },
    { 0xcf22b18c, SUC_lexicon_tags_17 },
    { 0x8522b18e, SUC_lexicon_tags_21 },
    { 0x2d62b18d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa2b192, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c2b196, SUC_lexicon_tags_63 },
    { 0xb1a2b197, SUC_lexicon_tags_410 },
    { 0x93c2b198, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a2b19b, SUC_lexicon_tags_37 },
    { 0x3082b19b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf22b19f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8b62b1a1, SUC_lexicon_tags_12 },
    { 0xa3c2b1a2, SUC_lexicon_tags_30 },
    { 0x4d82b1a2, SUC_lexicon_tags_17 },
    { 0x63a2b1a4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7302b1a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d02b1ab, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x29a2b1ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a2b1b0, SUC_lexicon_tags_12 },
    { 0x8642b1b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb482b1b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3262b1bb, SUC_lexicon_tags_27 },
    { 0x3682b1bc, SUC_lexicon_tags_17 },
    { 0xf462b1bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce82b1c8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd922b1ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2562b1cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50a2b1ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9622b1d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa2b1d4, SUC_lexicon_tags_46 },
    { 0xea22b1d4, SUC_lexicon_tags_2 },
    { 0x2022b1d6, SUC_lexicon_tags_27 },
    { 0x2462b1d7, SUC_lexicon_tags_39 },
    { 0xdb42b1d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae2b1dd, SUC_lexicon_tags_0 },
    { 0xb602b1de, SUC_lexicon_tags_7 },
    { 0x9962b1df, SUC_lexicon_tags_37 },
    { 0x6242b1e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2902b1e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe962b1e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a2b1ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a2b1f3, SUC_lexicon_tags_12 },
    { 0xa8c2b1f4, SUC_lexicon_tags_27 },
    { 0x9322b1f5, SUC_lexicon_tags_49 },
    { 0xde82b1f6, SUC_lexicon_tags_15 },
    { 0xa22b1f7, SUC_lexicon_tags_37 },
    { 0x89c2b1f6, SUC_lexicon_tags_17 },
    { 0x3362b1f9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2402b205, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa402b209, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36a2b20b, SUC_lexicon_tags_63 },
    { 0x2a62b20c, SUC_lexicon_tags_66 },
    { 0x3ae2b20d, SUC_lexicon_tags_29 },
    { 0x5ac2b20e, SUC_lexicon_tags_9 },
    { 0xdf42b20e, SUC_lexicon_tags_44 },
    { 0xe702b210, SUC_lexicon_tags_37 },
    { 0x96a2b211, SUC_lexicon_tags_15 },
    { 0xad62b20e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3242b215, SUC_lexicon_tags_7 },
    { 0xf982b216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb062b21a, SUC_lexicon_tags_61 },
    { 0xf602b21a, SUC_lexicon_tags_82 },
    { 0x8422b21a, SUC_lexicon_tags_14 },
    { 0x70e2b21d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c2b229, SUC_lexicon_tags_47 },
    { 0xd122b22a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e62b22c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7622b22e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd62b230, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe402b233, SUC_lexicon_tags_8 },
    { 0x7e22b233, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde2b240, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x682b243, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef42b247, SUC_lexicon_tags_19 },
    { 0xc6c2b247, SUC_lexicon_tags_65 },
    { 0xc3a2b249, SUC_lexicon_tags_63 },
    { 0xed02b247, SUC_lexicon_tags_33 },
    { 0x2ae2b24a, SUC_lexicon_tags_0 },
    { 0xe402b24c, SUC_lexicon_tags_7 },
    { 0xb122b24c, SUC_lexicon_tags_25 },
    { 0xe2c2b24e, SUC_lexicon_tags_17 },
    { 0xc102b24e, SUC_lexicon_tags_8 },
    { 0xf1e2b250, SUC_lexicon_tags_12 },
    { 0xdd62b250, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c2b255, SUC_lexicon_tags_11 },
    { 0xdd42b256, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa2b259, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x482b25b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x122b25e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4802b263, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5122b268, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca2b26b, SUC_lexicon_tags_37 },
    { 0x7002b26b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2742b270, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb6e2b272, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae2b279, SUC_lexicon_tags_11 },
    { 0x19e2b27a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4162b27d, SUC_lexicon_tags_25 },
    { 0x3b62b27e, SUC_lexicon_tags_17 },
    { 0x76e2b27f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac02b283, SUC_lexicon_tags_17 },
    { 0x8ee2b284, SUC_lexicon_tags_17 },
    { 0xc2c2b285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3202b289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6422b28b, SUC_lexicon_tags_37 },
    { 0xbe62b28c, SUC_lexicon_tags_25 },
    { 0x4282b28d, SUC_lexicon_tags_37 },
    { 0x8da2b28e, SUC_lexicon_tags_38 },
    { 0xfc82b28f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1882b294, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf8c2b296, SUC_lexicon_tags_77 },
    { 0x54c2b297, SUC_lexicon_tags_15 },
    { 0xdd82b297, SUC_lexicon_tags_11 },
    { 0xe242b299, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5502b29c, SUC_lexicon_tags_43 },
    { 0x79c2b29c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fa2b29f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e2b2a4, SUC_lexicon_tags_37 },
    { 0x422b2a5, SUC_lexicon_tags_32 },
    { 0x4802b2a6, SUC_lexicon_tags_15 },
    { 0xf282b2a6, SUC_lexicon_tags_15 },
    { 0x8d62b2a6, SUC_lexicon_tags_37 },
    { 0x11a2b2a9, SUC_lexicon_tags_37 },
    { 0x9342b2a5, SUC_lexicon_tags_19 },
    { 0xae22b2a7, SUC_lexicon_tags_14 },
    { 0x4a2b2ac, SUC_lexicon_tags_26 },
    { 0x78e2b2a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1442b2b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e2b2b7, SUC_lexicon_tags_1 },
    { 0x7b22b2b8, SUC_lexicon_tags_30 },
    { 0x4bc2b2b8, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0xe9a2b2bb, SUC_lexicon_tags_40 },
    { 0x2d62b2bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6622b2c2, SUC_lexicon_tags_4 },
    { 0x4082b2c3, SUC_lexicon_tags_65 },
    { 0x2042b2c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf002b2c8, SUC_lexicon_tags_14 },
    { 0xaee2b2c9, SUC_lexicon_tags_37 },
    { 0x30e2b2c9, SUC_lexicon_tags_27 },
    { 0x3562b2cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa842b2cd, SUC_lexicon_tags_9 },
    { 0xf882b2cd, SUC_lexicon_tags_25 },
    { 0x85c2b2cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a2b2d2, SUC_lexicon_tags_37 },
    { 0xd962b2d3, SUC_lexicon_tags_27 },
    { 0x58c2b2d3, SUC_lexicon_tags_28 },
    { 0xb562b2d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4582b2d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba82b2dc, SUC_lexicon_tags_61 },
    { 0xce02b2dd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcf82b2df, SUC_lexicon_tags_37 },
    { 0xfc82b2e0, SUC_lexicon_tags_57 },
    { 0x6b42b2e1, SUC_lexicon_tags_15 },
    { 0x3f02b2e0, SUC_lexicon_tags_2 },
    { 0x4ce2b2e1, SUC_lexicon_tags_6 },
    { 0xa5c2b2e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb402b2e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca2b2ed, SUC_lexicon_tags_33 },
    { 0xecc2b2ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2b2f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe42b2fe, SUC_lexicon_tags_11 },
    { 0xb9a2b2ff, SUC_lexicon_tags_48 },
    { 0x2022b2fe, SUC_lexicon_tags_27 },
    { 0x23c2b301, SUC_lexicon_tags_15 },
    { 0x6882b302, SUC_lexicon_tags_34 },
    { 0x1382b302, SUC_lexicon_tags_63 },
    { 0xb9e2b2fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c22b308, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xce42b30a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b30e, SUC_lexicon_tags_229 },
    { 0x7482b30f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3602b312, SUC_lexicon_tags_12 },
    { 0x8ea2b313, SUC_lexicon_tags_49 },
    { 0xbb82b313, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d62b316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9402b318, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be2b31b, SUC_lexicon_tags_17 },
    { 0x4002b31c, SUC_lexicon_tags_2 },
    { 0xb9a2b31d, SUC_lexicon_tags_15 },
    { 0x802b31e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2642b320, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e22b326, SUC_lexicon_tags_11 },
    { 0x4a42b327, SUC_lexicon_tags_7 },
    { 0xe7c2b327, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2b32c, SUC_lexicon_tags_17 },
    { 0x29e2b32d, SUC_lexicon_tags_25 },
    { 0x5c2b32d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4cc2b330, SUC_lexicon_tags_37 },
    { 0x29e2b330, SUC_lexicon_tags_106 },
    { 0x3362b332, SUC_lexicon_tags_40 },
    { 0x1aa2b331, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a62b337, SUC_lexicon_tags_43 },
    { 0x6562b338, SUC_lexicon_tags_25 },
    { 0x3022b339, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc322b33c, SUC_lexicon_tags_0 },
    { 0xe482b33d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa142b33f, SUC_lexicon_tags_25 },
    { 0x8442b340, SUC_lexicon_tags_2 },
    { 0xa822b340, SUC_lexicon_tags_15 },
    { 0xe6a2b342, SUC_lexicon_tags_7 },
    { 0xe7e2b343, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4302b34e, SUC_lexicon_tags_68 },
    { 0xc982b34e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed42b354, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa202b35e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f02b366, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca2b36f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4042b377, SUC_lexicon_tags_1 },
    { 0x4282b378, SUC_lexicon_tags_49 },
    { 0xbe02b379, SUC_lexicon_tags_4 },
    { 0x85c2b37a, SUC_lexicon_tags_3 },
    { 0xa062b37b, SUC_lexicon_tags_11 },
    { 0x8642b379, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb0c2b37e, SUC_lexicon_tags_11 },
    { 0x4a22b37e, SUC_lexicon_tags_9 },
    { 0x35c2b380, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f82b385, SUC_lexicon_tags_25 },
    { 0x8322b386, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8882b38a, SUC_lexicon_tags_23 },
    { 0xb202b38b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc402b38d, SUC_lexicon_tags_25 },
    { 0x5b42b38e, SUC_lexicon_tags_11 },
    { 0xbf62b38e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c2b392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3402b395, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd62b397, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd42b399, SUC_lexicon_tags_63 },
    { 0x16c2b39a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6862b39c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a62b3a1, SUC_lexicon_tags_6 },
    { 0x8862b3a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4362b3a5, SUC_lexicon_tags_1 },
    { 0x8782b3a6, SUC_lexicon_tags_37 },
    { 0xefc2b3a6, SUC_lexicon_tags_87 },
    { 0x1182b3a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1642b3ab, SUC_lexicon_tags_27 },
    { 0xaae2b3ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc482b3af, SUC_lexicon_tags_38 },
    { 0xec62b3b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc2b3b8, SUC_lexicon_tags_49 },
    { 0x4262b3b8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f02b3bd, SUC_lexicon_tags_7 },
    { 0xebe2b3bd, SUC_lexicon_tags_37 },
    { 0xa2a2b3bf, SUC_lexicon_tags_11 },
    { 0x8da2b3c0, SUC_lexicon_tags_1 },
    { 0x6962b3bf, SUC_lexicon_tags_2 },
    { 0xff42b3c2, SUC_lexicon_tags_26 },
    { 0x89e2b3c3, SUC_lexicon_tags_17 },
    { 0xa3e2b3c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7522b3cc, SUC_lexicon_tags_7 },
    { 0xbde2b3cd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e22b3d0, SUC_lexicon_tags_56 },
    { 0x4362b3d1, SUC_lexicon_tags_1 },
    { 0x7e22b3d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6942b3d4, SUC_lexicon_tags_14 },
    { 0xf462b3d5, SUC_lexicon_tags_1 },
    { 0x8342b3d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfe22b3d8, SUC_lexicon_tags_1 },
    { 0xa842b3d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9482b3db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e82b3dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa882b3e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbc62b3e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c2b3e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x542b3e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2002b3f0, SUC_lexicon_tags_3 },
    { 0xcea2b3f0, SUC_lexicon_tags_17 },
    { 0xdaa2b3f2, SUC_lexicon_tags_47 },
    { 0x9562b3f2, SUC_lexicon_tags_11 },
    { 0x8922b3f4, SUC_lexicon_tags_47 },
    { 0xb042b3f5, SUC_lexicon_tags_58 },
    { 0x9fa2b3f5, SUC_lexicon_tags_15 },
    { 0xcf02b3f1, SUC_lexicon_tags_43 },
    { 0x8822b3f8, SUC_lexicon_tags_12 },
    { 0xe6e2b3f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd722b3ff, SUC_lexicon_tags_21 },
    { 0x1482b3ff, SUC_lexicon_tags_87 },
    { 0x38c2b3ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x15c2b403, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2ba2b405, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c2b408, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa922b40d, SUC_lexicon_tags_15 },
    { 0xfb22b40d, SUC_lexicon_tags_14 },
    { 0xf522b40f, SUC_lexicon_tags_37 },
    { 0xa042b410, SUC_lexicon_tags_55 },
    { 0x7a42b411, SUC_lexicon_tags_19 },
    { 0xb742b412, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4622b416, SUC_lexicon_tags_7 },
    { 0x9622b417, SUC_lexicon_tags_29 },
    { 0x8ce2b417, SUC_lexicon_tags_11 },
    { 0x5a22b419, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xef62b41c, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0x862b41f, SUC_lexicon_tags_21 },
    { 0xb0e2b420, SUC_lexicon_tags_82 },
    { 0x93c2b421, SUC_lexicon_tags_37 },
    { 0xa682b420, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95a2b424, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7de2b426, SUC_lexicon_tags_30 },
    { 0xbf62b427, SUC_lexicon_tags_10 },
    { 0xb602b427, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ba2b42a, SUC_lexicon_tags_21 },
    { 0x72a2b42b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd362b42e, SUC_lexicon_tags_873 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb02b438, SUC_lexicon_tags_0 },
    { 0x69e2b439, SUC_lexicon_tags_63 },
    { 0xe642b43a, SUC_lexicon_tags_68 },
    { 0x902b43b, SUC_lexicon_tags_37 },
    { 0x11c2b438, SUC_lexicon_tags_17 },
    { 0x2d82b43d, SUC_lexicon_tags_37 },
    { 0x2d42b43d, SUC_lexicon_tags_2 },
    { 0x2702b43f, SUC_lexicon_tags_30 },
    { 0xa182b440, SUC_lexicon_tags_1 },
    { 0xf002b43f, SUC_lexicon_tags_2 },
    { 0x6fa2b439, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc82b445, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xec22b448, SUC_lexicon_tags_87 },
    { 0x3fc2b449, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1642b450, SUC_lexicon_tags_32 },
    { 0x7762b451, SUC_lexicon_tags_77 },
    { 0xdc2b450, SUC_lexicon_tags_15 },
    { 0x3262b453, SUC_lexicon_tags_15 },
    { 0x15a2b453, SUC_lexicon_tags_60 },
    { 0x1e62b451, SUC_lexicon_tags_12 },
    { 0x9522b456, SUC_lexicon_tags_12 },
    { 0xfa2b453, SUC_lexicon_tags_55 },
    { 0x1562b458, SUC_lexicon_tags_21 },
    { 0x62e2b452, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a2b45c, SUC_lexicon_tags_7 },
    { 0x48a2b45d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5042b460, SUC_lexicon_tags_11 },
    { 0x5fc2b461, SUC_lexicon_tags_14 },
    { 0xf6e2b462, SUC_lexicon_tags_66 },
    { 0xf022b461, SUC_lexicon_tags_38 },
    { 0xcb02b462, SUC_lexicon_tags_63 },
    { 0x4f62b464, SUC_lexicon_tags_32 },
    { 0xbcc2b460, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaaa2b468, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x1502b46a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6042b46d, SUC_lexicon_tags_3 },
    { 0x84c2b46e, SUC_lexicon_tags_57 },
    { 0x6e82b46f, SUC_lexicon_tags_63 },
    { 0x47a2b470, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8362b474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c42b47a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82b47e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfac2b480, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe242b485, SUC_lexicon_tags_14 },
    { 0x62a2b485, SUC_lexicon_tags_15 },
    { 0x42b485, SUC_lexicon_tags_3 },
    { 0x1f82b485, SUC_lexicon_tags_7 },
    { 0xe8a2b485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa162b48b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1302b48d, SUC_lexicon_tags_23 },
    { 0x16e2b48e, SUC_lexicon_tags_12 },
    { 0xf0c2b48e, SUC_lexicon_tags_4 },
    { 0x8a42b490, SUC_lexicon_tags_28 },
    { 0x9522b491, SUC_lexicon_tags_25 },
    { 0xc3e2b490, SUC_lexicon_tags_15 },
    { 0x1402b491, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf22b495, SUC_lexicon_tags_17 },
    { 0xc3a2b496, SUC_lexicon_tags_0 },
    { 0xc742b497, SUC_lexicon_tags_0 },
    { 0xda42b498, SUC_lexicon_tags_11 },
    { 0x37a2b499, SUC_lexicon_tags_21 },
    { 0x34e2b497, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd82b49f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6562b4a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a2b4a7, SUC_lexicon_tags_63 },
    { 0x6cc2b4a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c2b4b0, SUC_lexicon_tags_53 },
    { 0x7022b4b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc042b4b4, SUC_lexicon_tags_7 },
    { 0x97e2b4b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c2b4b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb982b4bc, SUC_lexicon_tags_16 },
    { 0x3ae2b4bc, SUC_lexicon_tags_17 },
    { 0xc922b4bd, SUC_lexicon_tags_0 },
    { 0xbac2b4bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8622b4c3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2402b4c5, SUC_lexicon_tags_8 },
    { 0xf502b4c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae2b4c9, SUC_lexicon_tags_11 },
    { 0x28c2b4c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf82b4cc, SUC_lexicon_tags_56 },
    { 0x7d42b4cd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c2b4d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5322b4d8, SUC_lexicon_tags_11 },
    { 0x28c2b4d9, SUC_lexicon_tags_2 },
    { 0xd942b4da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9262b4dd, SUC_lexicon_tags_12 },
    { 0xbb22b4de, SUC_lexicon_tags_19 },
    { 0x8dc2b4df, SUC_lexicon_tags_4 },
    { 0xfd62b4de, SUC_lexicon_tags_17 },
    { 0xfe82b4e1, SUC_lexicon_tags_12 },
    { 0xcf82b4e1, SUC_lexicon_tags_12 },
    { 0x1202b4e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb002b4ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa342b4ec, SUC_lexicon_tags_11 },
    { 0x7982b4ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e22b4f8, SUC_lexicon_tags_15 },
    { 0xab82b4f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f02b4fb, SUC_lexicon_tags_10 },
    { 0xb4e2b4fc, SUC_lexicon_tags_40 },
    { 0xdfc2b4fd, SUC_lexicon_tags_37 },
    { 0x8362b4fc, SUC_lexicon_tags_33 },
    { 0xe162b4fc, SUC_lexicon_tags_80 },
    { 0x5202b4fb, SUC_lexicon_tags_17 },
    { 0x3b82b4ff, SUC_lexicon_tags_12 },
    { 0x8a82b4fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe882b505, SUC_lexicon_tags_79 },
    { 0xf0a2b506, SUC_lexicon_tags_45 },
    { 0x5ee2b507, SUC_lexicon_tags_2 },
    { 0x7862b506, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82b50c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a2b510, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1602b513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2b518, SUC_lexicon_tags_1 },
    { 0x2a22b519, SUC_lexicon_tags_0 },
    { 0xbdc2b51a, SUC_lexicon_tags_25 },
    { 0x77a2b51a, SUC_lexicon_tags_47 },
    { 0x5202b51c, SUC_lexicon_tags_7 },
    { 0x9582b51c, SUC_lexicon_tags_1 },
    { 0x2182b51b, SUC_lexicon_tags_15 },
    { 0x4cc2b51d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c2b523, SUC_lexicon_tags_17 },
    { 0x4802b524, SUC_lexicon_tags_47 },
    { 0x7482b524, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x65c2b527, SUC_lexicon_tags_7 },
    { 0x9f42b527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc62b52a, SUC_lexicon_tags_26 },
    { 0xe702b52a, SUC_lexicon_tags_43 },
    { 0x1562b52c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb1c2b52e, SUC_lexicon_tags_41 },
    { 0xfd02b52f, SUC_lexicon_tags_9 },
    { 0xcb22b52f, SUC_lexicon_tags_25 },
    { 0x6ee2b531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6da2b533, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc842b538, SUC_lexicon_tags_63 },
    { 0xba22b539, SUC_lexicon_tags_16 },
    { 0xff02b538, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8922b53d, SUC_lexicon_tags_27 },
    { 0x54e2b53e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf542b544, SUC_lexicon_tags_0 },
    { 0xea22b545, SUC_lexicon_tags_19 },
    { 0xb042b546, SUC_lexicon_tags_48 },
    { 0xa122b544, SUC_lexicon_tags_7 },
    { 0xb3a2b548, SUC_lexicon_tags_25 },
    { 0x8f62b547, SUC_lexicon_tags_336 },
    { 0x6ce2b54a, SUC_lexicon_tags_12 },
    { 0xe802b54b, SUC_lexicon_tags_201 },
    { 0x2662b54b, SUC_lexicon_tags_11 },
    { 0x9322b54a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa522b553, SUC_lexicon_tags_12 },
    { 0x9a22b554, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82b557, SUC_lexicon_tags_0 },
    { 0x5ac2b558, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6722b561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea2b566, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a2b56b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a2b572, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffc2b574, SUC_lexicon_tags_6 },
    { 0x6282b575, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x7102b578, SUC_lexicon_tags_77 },
    { 0xda42b579, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1002b583, SUC_lexicon_tags_6 },
    { 0x24a2b584, SUC_lexicon_tags_9 },
    { 0x4e2b585, SUC_lexicon_tags_37 },
    { 0x7722b584, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb22b588, SUC_lexicon_tags_49 },
    { 0xd942b589, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2202b58d, SUC_lexicon_tags_33 },
    { 0xa922b58d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d02b590, SUC_lexicon_tags_47 },
    { 0x28a2b591, SUC_lexicon_tags_26 },
    { 0x2062b592, SUC_lexicon_tags_6 },
    { 0x96c2b590, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a62b595, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xc522b598, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1482b59e, SUC_lexicon_tags_65 },
    { 0x2cc2b59e, SUC_lexicon_tags_30 },
    { 0x7d02b5a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e2b5a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1142b5ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9022b5b4, SUC_lexicon_tags_12 },
    { 0x8302b5b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1182b5c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa782b5c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeb82b5c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdec2b5c8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa582b5cd, SUC_lexicon_tags_7 },
    { 0xcec2b5ce, SUC_lexicon_tags_25 },
    { 0xcda2b5cf, SUC_lexicon_tags_17 },
    { 0xf7c2b5cd, SUC_lexicon_tags_10 },
    { 0xd242b5d1, SUC_lexicon_tags_17 },
    { 0xff62b5d1, SUC_lexicon_tags_2 },
    { 0x4d82b5cd, SUC_lexicon_tags_37 },
    { 0x48a2b5d4, SUC_lexicon_tags_25 },
    { 0xa422b5d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9b42b5d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe002b5d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd382b5dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d42b5e6, SUC_lexicon_tags_1 },
    { 0x29c2b5e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee2b5eb, SUC_lexicon_tags_38 },
    { 0xa6a2b5eb, SUC_lexicon_tags_12 },
    { 0x4f02b5ed, SUC_lexicon_tags_37 },
    { 0x422b5ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3722b5f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29c2b5f9, SUC_lexicon_tags_26 },
    { 0x2dc2b5fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a2b5fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1da2b600, SUC_lexicon_tags_38 },
    { 0x7322b600, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa2b604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f22b606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x902b608, SUC_lexicon_tags_37 },
    { 0xd742b609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc62b611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b22b616, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9922b618, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7062b61f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4a2b621, SUC_lexicon_tags_60 },
    { 0x7402b622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f02b625, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc822b628, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb202b62a, SUC_lexicon_tags_0 },
    { 0x4102b62b, SUC_lexicon_tags_11 },
    { 0x55e2b62b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb822b62e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b62b631, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a2b634, SUC_lexicon_tags_106 },
    { 0x40e2b634, SUC_lexicon_tags_37 },
    { 0x33c2b636, SUC_lexicon_tags_7 },
    { 0x4a22b637, SUC_lexicon_tags_45 },
    { 0x94c2b636, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e2b63b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc2b644, SUC_lexicon_tags_31 },
    { 0x4e02b645, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d02b648, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc2b64b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa02b64e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4902b652, SUC_lexicon_tags_16 },
    { 0xd582b653, SUC_lexicon_tags_38 },
    { 0xb42b652, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2962b656, SUC_lexicon_tags_9 },
    { 0xbda2b656, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4962b659, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea2b65e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e2b666, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc62b66b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7442b66e, SUC_lexicon_tags_60 },
    { 0xa9e2b66e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbce2b671, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c2b675, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b02b67c, SUC_lexicon_tags_31 },
    { 0x1142b67d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa742b680, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a2b686, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7822b689, SUC_lexicon_tags_10 },
    { 0xde02b68a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3e2b68c, SUC_lexicon_tags_25 },
    { 0xa6c2b68d, SUC_lexicon_tags_33 },
    { 0x2382b68d, SUC_lexicon_tags_619 },
    { 0xbf42b68f, SUC_lexicon_tags_28 },
    { 0x1f62b68d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe802b692, SUC_lexicon_tags_40 },
    { 0xa102b692, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63a2b695, SUC_lexicon_tags_68 },
    { 0x4aa2b696, SUC_lexicon_tags_25 },
    { 0xf5c2b695, SUC_lexicon_tags_9 },
    { 0xd082b698, SUC_lexicon_tags_17 },
    { 0x3f82b695, SUC_lexicon_tags_37 },
    { 0x84c2b695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e22b69e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c2b6a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a2b6a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde42b6ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf042b6ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1842b6bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf902b6c4, SUC_lexicon_tags_37 },
    { 0xf4e2b6c5, SUC_lexicon_tags_19 },
    { 0xde2b6c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e2b6cb, SUC_lexicon_tags_11 },
    { 0x81e2b6cc, SUC_lexicon_tags_38 },
    { 0x4842b6cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e2b6d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc822b6d6, SUC_lexicon_tags_27 },
    { 0xf802b6d7, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xf9c2b6d9, SUC_lexicon_tags_15 },
    { 0xb262b6d9, SUC_lexicon_tags_17 },
    { 0xe102b6db, SUC_lexicon_tags_37 },
    { 0xe7a2b6dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1282b6e3, SUC_lexicon_tags_10 },
    { 0x3282b6e3, SUC_lexicon_tags_25 },
    { 0x5e62b6e5, SUC_lexicon_tags_44 },
    { 0xf5c2b6e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf402b6eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91c2b6ed, SUC_lexicon_tags_151 },
    { 0x73a2b6ee, SUC_lexicon_tags_11 },
    { 0xc682b6ed, SUC_lexicon_tags_11 },
    { 0x8522b6ef, SUC_lexicon_tags_32 },
    { 0x4b62b6f1, SUC_lexicon_tags_57 },
    { 0x4262b6f0, SUC_lexicon_tags_47 },
    { 0x9c42b6f3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a2b6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6302b6f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3822b6fc, SUC_lexicon_tags_17 },
    { 0x7cc2b6fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd422b6ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce2b707, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7202b70a, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0x8282b70d, SUC_lexicon_tags_23 },
    { 0xde62b70d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc2b714, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362b719, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa842b71d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba2b720, SUC_lexicon_tags_7 },
    { 0x4f22b721, SUC_lexicon_tags_45 },
    { 0x2782b721, SUC_lexicon_tags_8 },
    { 0x122b723, SUC_lexicon_tags_28 },
    { 0xfae2b724, SUC_lexicon_tags_44 },
    { 0xb802b724, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9182b727, SUC_lexicon_tags_12 },
    { 0x4d62b727, SUC_lexicon_tags_30 },
    { 0xca2b729, SUC_lexicon_tags_25 },
    { 0xc0c2b72a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd282b72c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb9c2b72e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xefc2b730, SUC_lexicon_tags_9 },
    { 0xa362b731, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5382b735, SUC_lexicon_tags_2 },
    { 0x46a2b736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea2b73a, SUC_lexicon_tags_11 },
    { 0x2362b73b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5722b741, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee2b744, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba22b746, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd502b74c, SUC_lexicon_tags_2 },
    { 0xf682b74c, SUC_lexicon_tags_34 },
    { 0x5fc2b74e, SUC_lexicon_tags_17 },
    { 0x8062b74f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd282b754, SUC_lexicon_tags_37 },
    { 0x62e2b755, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e2b75a, SUC_lexicon_tags_2 },
    { 0xa4a2b75a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c82b75f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb22b764, SUC_lexicon_tags_65 },
    { 0x8102b765, SUC_lexicon_tags_83 },
    { 0x7002b766, SUC_lexicon_tags_11 },
    { 0x2682b766, SUC_lexicon_tags_25 },
    { 0xb62b768, SUC_lexicon_tags_37 },
    { 0xa242b769, SUC_lexicon_tags_37 },
    { 0x3d82b76a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9142b76d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742b772, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6222b774, SUC_lexicon_tags_35 },
    { 0x4702b775, SUC_lexicon_tags_3 },
    { 0x8d02b774, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd102b778, SUC_lexicon_tags_11 },
    { 0xc4e2b778, SUC_lexicon_tags_63 },
    { 0xd942b77a, SUC_lexicon_tags_43 },
    { 0xafc2b77b, SUC_lexicon_tags_1 },
    { 0xa6e2b778, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c2b77e, SUC_lexicon_tags_12 },
    { 0x5562b77f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51c2b781, SUC_lexicon_tags_12 },
    { 0x6ce2b782, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e2b785, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7a02b787, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xce62b78a, SUC_lexicon_tags_9 },
    { 0x3242b78b, SUC_lexicon_tags_25 },
    { 0x8162b78c, SUC_lexicon_tags_21 },
    { 0xfaa2b78b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7762b790, SUC_lexicon_tags_47 },
    { 0x45e2b791, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x382b794, SUC_lexicon_tags_56 },
    { 0x8742b794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1762b797, SUC_lexicon_tags_15 },
    { 0xd422b798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe222b7a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc2b7a5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6842b7aa, SUC_lexicon_tags_7 },
    { 0x3bc2b7ab, SUC_lexicon_tags_17 },
    { 0x94a2b7ac, SUC_lexicon_tags_25 },
    { 0x802b7ab, SUC_lexicon_tags_12 },
    { 0xae82b7ae, SUC_lexicon_tags_17 },
    { 0x4a2b7ab, SUC_lexicon_tags_38 },
    { 0x4802b7b0, SUC_lexicon_tags_8 },
    { 0x522b7b1, SUC_lexicon_tags_7 },
    { 0x3682b7b2, SUC_lexicon_tags_7 },
    { 0xbfe2b7ae, SUC_lexicon_tags_282 },
    { 0xae82b7b0, SUC_lexicon_tags_19 },
    { 0xea2b7b5, SUC_lexicon_tags_26 },
    { 0xe942b7b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a02b7bc, SUC_lexicon_tags_31 },
    { 0x2502b7bd, SUC_lexicon_tags_17 },
    { 0x8182b7be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb22b7c1, SUC_lexicon_tags_47 },
    { 0x3282b7c2, SUC_lexicon_tags_27 },
    { 0x4322b7c1, SUC_lexicon_tags_2 },
    { 0xbec2b7c4, SUC_lexicon_tags_45 },
    { 0x5662b7c4, SUC_lexicon_tags_27 },
    { 0xb542b7c6, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x6402b7c8, SUC_lexicon_tags_30 },
    { 0xae62b7c9, SUC_lexicon_tags_17 },
    { 0x4102b7c8, SUC_lexicon_tags_47 },
    { 0x6342b7cb, SUC_lexicon_tags_10 },
    { 0x9122b7c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7902b7cf, SUC_lexicon_tags_31 },
    { 0x5d82b7cf, SUC_lexicon_tags_60 },
    { 0xc3a2b7cf, SUC_lexicon_tags_37 },
    { 0xd762b7d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc062b7d4, SUC_lexicon_tags_60 },
    { 0xa0a2b7d5, SUC_lexicon_tags_37 },
    { 0x4ae2b7d5, SUC_lexicon_tags_12 },
    { 0x57a2b7d7, SUC_lexicon_tags_25 },
    { 0xde62b7d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe702b7da, SUC_lexicon_tags_11 },
    { 0xa902b7da, SUC_lexicon_tags_2 },
    { 0xe2e2b7da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3702b7e0, SUC_lexicon_tags_43 },
    { 0xc782b7e0, SUC_lexicon_tags_11 },
    { 0xf822b7e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2482b7e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2622b7ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc62b7ef, SUC_lexicon_tags_29 },
    { 0x9a42b7ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa2b7f3, SUC_lexicon_tags_31 },
    { 0x6802b7f4, SUC_lexicon_tags_8 },
    { 0x5582b7f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2b7f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7062b7fb, SUC_lexicon_tags_12 },
    { 0x9182b7fb, SUC_lexicon_tags_11 },
    { 0x91e2b7fd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be2b807, SUC_lexicon_tags_0 },
    { 0x21e2b807, SUC_lexicon_tags_11 },
    { 0x6dc2b807, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8042b80c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeae2b80e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa2b812, SUC_lexicon_tags_25 },
    { 0xec2b813, SUC_lexicon_tags_77 },
    { 0xdce2b814, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5602b817, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53c2b819, SUC_lexicon_tags_33 },
    { 0x9be2b819, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc0e2b81c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa922b820, SUC_lexicon_tags_43 },
    { 0x61e2b820, SUC_lexicon_tags_47 },
    { 0x29a2b822, SUC_lexicon_tags_38 },
    { 0xca2b822, SUC_lexicon_tags_0 },
    { 0x122b823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e2b827, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e2b82b, SUC_lexicon_tags_37 },
    { 0x10c2b82b, SUC_lexicon_tags_60 },
    { 0x7382b82b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6902b832, SUC_lexicon_tags_11 },
    { 0x39c2b833, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ba2b835, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xac02b837, SUC_lexicon_tags_27 },
    { 0xe42b837, SUC_lexicon_tags_4 },
    { 0x26e2b839, SUC_lexicon_tags_25 },
    { 0x7d82b83a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f22b840, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc902b843, SUC_lexicon_tags_12 },
    { 0xaec2b844, SUC_lexicon_tags_7 },
    { 0x9ac2b845, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b02b848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6dc2b84a, SUC_lexicon_tags_23 },
    { 0xd022b84b, SUC_lexicon_tags_69 },
    { 0xa222b84a, SUC_lexicon_tags_23 },
    { 0xa42b84d, SUC_lexicon_tags_15 },
    { 0x4ea2b84a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c2b854, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb062b858, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8402b85b, SUC_lexicon_tags_1 },
    { 0x6c62b85b, SUC_lexicon_tags_17 },
    { 0x43a2b85b, SUC_lexicon_tags_10 },
    { 0x7262b85d, SUC_lexicon_tags_5 },
    { 0xc3c2b85f, SUC_lexicon_tags_17 },
    { 0xaf22b860, SUC_lexicon_tags_82 },
    { 0x3122b860, SUC_lexicon_tags_48 },
    { 0xd5a2b862, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702b866, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc222b869, SUC_lexicon_tags_16 },
    { 0x6a22b869, SUC_lexicon_tags_25 },
    { 0x1862b869, SUC_lexicon_tags_1 },
    { 0x4182b86a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc2b86f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf902b872, SUC_lexicon_tags_37 },
    { 0x72e2b873, SUC_lexicon_tags_25 },
    { 0x4ca2b874, SUC_lexicon_tags_65 },
    { 0xb002b874, SUC_lexicon_tags_23 },
    { 0x54a2b876, SUC_lexicon_tags_14 },
    { 0x6c42b877, SUC_lexicon_tags_63 },
    { 0xeb62b878, SUC_lexicon_tags_15 },
    { 0x82a2b879, SUC_lexicon_tags_45 },
    { 0x4d82b879, SUC_lexicon_tags_3 },
    { 0xe722b87a, SUC_lexicon_tags_37 },
    { 0x4fa2b87a, SUC_lexicon_tags_2 },
    { 0xda2b878, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x52e2b87f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe2b884, SUC_lexicon_tags_61 },
    { 0x1d62b885, SUC_lexicon_tags_17 },
    { 0x8282b885, SUC_lexicon_tags_25 },
    { 0x92a2b886, SUC_lexicon_tags_23 },
    { 0x4302b885, SUC_lexicon_tags_25 },
    { 0xf662b886, SUC_lexicon_tags_19 },
    { 0x6642b886, SUC_lexicon_tags_17 },
    { 0xb442b884, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc142b88e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd02b892, SUC_lexicon_tags_38 },
    { 0x2d42b893, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc2b898, SUC_lexicon_tags_17 },
    { 0x9042b899, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xa522b89c, SUC_lexicon_tags_15 },
    { 0xebe2b89c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb062b8a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c2b8a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde22b8a8, SUC_lexicon_tags_54 },
    { 0x65e2b8a9, SUC_lexicon_tags_31 },
    { 0x9e22b8aa, SUC_lexicon_tags_63 },
    { 0x1662b8ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2b8ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc2b8b2, SUC_lexicon_tags_103 },
    { 0x7302b8b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a2b8b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe562b8bd, SUC_lexicon_tags_37 },
    { 0xee22b8bd, SUC_lexicon_tags_31 },
    { 0x85c2b8bf, SUC_lexicon_tags_28 },
    { 0x4f42b8c0, SUC_lexicon_tags_34 },
    { 0x6a82b8c1, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x4ba2b8c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21e2b8c5, SUC_lexicon_tags_11 },
    { 0x3e42b8c5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xbd82b8c8, SUC_lexicon_tags_107 },
    { 0xd002b8c9, SUC_lexicon_tags_9 },
    { 0x4e82b8ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3782b8ce, SUC_lexicon_tags_23 },
    { 0xe962b8cf, SUC_lexicon_tags_19 },
    { 0x442b8d0, SUC_lexicon_tags_11 },
    { 0xc9c2b8cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a2b8d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91e2b8d7, SUC_lexicon_tags_17 },
    { 0x6b42b8d7, SUC_lexicon_tags_1 },
    { 0x7c42b8d9, SUC_lexicon_tags_9 },
    { 0x7e82b8d7, SUC_lexicon_tags_39 },
    { 0xc9a2b8db, SUC_lexicon_tags_15 },
    { 0x6f02b8db, SUC_lexicon_tags_6 },
    { 0xa7c2b8dd, SUC_lexicon_tags_10 },
    { 0x9a62b8dc, SUC_lexicon_tags_7 },
    { 0xdce2b8df, SUC_lexicon_tags_69 },
    { 0x4fe2b8e0, SUC_lexicon_tags_45 },
    { 0x49c2b8e1, SUC_lexicon_tags_51 },
    { 0x29e2b8db, SUC_lexicon_tags_1 },
    { 0xb462b8e3, SUC_lexicon_tags_28 },
    { 0x1a22b8e4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6222b8e6, SUC_lexicon_tags_77 },
    { 0xe5c2b8e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe2b8ea, SUC_lexicon_tags_85 },
    { 0x6182b8ea, SUC_lexicon_tags_7 },
    { 0x4862b8ec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbe62b8ee, SUC_lexicon_tags_1 },
    { 0xe6c2b8ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c42b8f1, SUC_lexicon_tags_37 },
    { 0x5042b8f1, SUC_lexicon_tags_49 },
    { 0x2662b8f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e2b8fe, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3b02b900, SUC_lexicon_tags_11 },
    { 0x1022b900, SUC_lexicon_tags_39 },
    { 0xf2c2b900, SUC_lexicon_tags_49 },
    { 0x1da2b902, SUC_lexicon_tags_2 },
    { 0x5222b904, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9a02b906, SUC_lexicon_tags_22 },
    { 0x1c02b906, SUC_lexicon_tags_1 },
    { 0xff02b908, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a2b90b, SUC_lexicon_tags_11 },
    { 0x2e22b90b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b02b912, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xffe2b914, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x182b916, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb22b918, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf082b91e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd842b920, SUC_lexicon_tags_47 },
    { 0xeb82b921, SUC_lexicon_tags_32 },
    { 0xa622b922, SUC_lexicon_tags_7 },
    { 0xce62b921, SUC_lexicon_tags_17 },
    { 0xf182b920, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d62b927, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1d02b929, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1822b932, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe122b937, SUC_lexicon_tags_38 },
    { 0x2882b938, SUC_lexicon_tags_1 },
    { 0xae62b939, SUC_lexicon_tags_14 },
    { 0xf2c2b93a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa2b93e, SUC_lexicon_tags_11 },
    { 0xa9e2b93e, SUC_lexicon_tags_25 },
    { 0x8922b940, SUC_lexicon_tags_11 },
    { 0xf542b941, SUC_lexicon_tags_10 },
    { 0xa582b942, SUC_lexicon_tags_16 },
    { 0x2f82b942, SUC_lexicon_tags_63 },
    { 0x7322b940, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb4a2b946, SUC_lexicon_tags_38 },
    { 0x2302b947, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b82b949, SUC_lexicon_tags_11 },
    { 0x3d82b94a, SUC_lexicon_tags_17 },
    { 0x3a42b94b, SUC_lexicon_tags_112 },
    { 0x33a2b94c, SUC_lexicon_tags_11 },
    { 0x822b94a, SUC_lexicon_tags_17 },
    { 0xb422b94d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b62b952, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2b959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c2b95f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4962b961, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7082b964, SUC_lexicon_tags_11 },
    { 0x6142b965, SUC_lexicon_tags_0 },
    { 0x93c2b966, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a2b96b, SUC_lexicon_tags_66 },
    { 0xf5e2b96b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8222b96f, SUC_lexicon_tags_7 },
    { 0x2362b970, SUC_lexicon_tags_11 },
    { 0xe202b970, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2562b975, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2302b978, SUC_lexicon_tags_323 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c42b97d, SUC_lexicon_tags_4 },
    { 0xaa62b97e, SUC_lexicon_tags_31 },
    { 0xe362b97e, SUC_lexicon_tags_37 },
    { 0x76a2b97e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e2b987, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc2b98a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef62b98d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3682b98f, SUC_lexicon_tags_30 },
    { 0xac62b98f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb42b993, SUC_lexicon_tags_11 },
    { 0xec82b993, SUC_lexicon_tags_133 },
    { 0x5ea2b994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c2b999, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e2b99e, SUC_lexicon_tags_19 },
    { 0x8162b99e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d22b9a1, SUC_lexicon_tags_6 },
    { 0x8d42b9a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6402b9a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x142b9a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae2b9aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b02b9ad, SUC_lexicon_tags_6 },
    { 0x9582b9ad, SUC_lexicon_tags_6 },
    { 0x9062b9ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf782b9b2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcf22b9b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc82b9b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa742b9bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf02b9bd, SUC_lexicon_tags_207 },
    { 0x22a2b9be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd782b9c1, SUC_lexicon_tags_7 },
    { 0xb982b9c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d62b9c7, SUC_lexicon_tags_12 },
    { 0xd4e2b9c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea2b9cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb482b9d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf42b9d6, SUC_lexicon_tags_7 },
    { 0xc842b9d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a2b9dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde22b9de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e2b9e2, SUC_lexicon_tags_79 },
    { 0xfc42b9e3, SUC_lexicon_tags_68 },
    { 0x9642b9e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc2b9ed, SUC_lexicon_tags_37 },
    { 0x2482b9ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e02b9f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1942b9f2, SUC_lexicon_tags_15 },
    { 0xd602b9f2, SUC_lexicon_tags_45 },
    { 0x59a2b9f4, SUC_lexicon_tags_89 },
    { 0x3b02b9f5, SUC_lexicon_tags_49 },
    { 0xde62b9f6, SUC_lexicon_tags_7 },
    { 0x4dc2b9f6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7202b9fa, SUC_lexicon_tags_63 },
    { 0x8422b9fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9562ba00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a2ba06, SUC_lexicon_tags_6 },
    { 0x2682ba07, SUC_lexicon_tags_27 },
    { 0xc242ba08, SUC_lexicon_tags_19 },
    { 0x6c62ba07, SUC_lexicon_tags_37 },
    { 0xf5a2ba0a, SUC_lexicon_tags_106 },
    { 0x6a22ba0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a22ba0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7102ba12, SUC_lexicon_tags_45 },
    { 0xf22ba12, SUC_lexicon_tags_37 },
    { 0xb202ba13, SUC_lexicon_tags_19 },
    { 0x4262ba15, SUC_lexicon_tags_1 },
    { 0xa9c2ba16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc2ba1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x822ba1f, SUC_lexicon_tags_7 },
    { 0x4e02ba20, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf22ba22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae02ba24, SUC_lexicon_tags_48 },
    { 0x1882ba25, SUC_lexicon_tags_66 },
    { 0xc442ba26, SUC_lexicon_tags_77 },
    { 0xbbe2ba24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7362ba2b, SUC_lexicon_tags_25 },
    { 0x4282ba2c, SUC_lexicon_tags_7 },
    { 0x5fa2ba2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5742ba31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97c2ba33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c02ba37, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdfa2ba39, SUC_lexicon_tags_15 },
    { 0x822ba3a, SUC_lexicon_tags_14 },
    { 0x5002ba3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd02ba3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd622ba40, SUC_lexicon_tags_2 },
    { 0x1662ba40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21c2ba43, SUC_lexicon_tags_11 },
    { 0xdfe2ba44, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x54e2ba46, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c2ba4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b82ba52, SUC_lexicon_tags_29 },
    { 0x6da2ba53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee2ba58, SUC_lexicon_tags_7 },
    { 0x9282ba59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2202ba5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6282ba65, SUC_lexicon_tags_2 },
    { 0x3362ba65, SUC_lexicon_tags_31 },
    { 0x8ec2ba65, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x202ba69, SUC_lexicon_tags_2 },
    { 0xae62ba6a, SUC_lexicon_tags_25 },
    { 0x8ae2ba69, SUC_lexicon_tags_29 },
    { 0x97a2ba6b, SUC_lexicon_tags_37 },
    { 0x15c2ba6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2ba71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec2ba75, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf722ba77, SUC_lexicon_tags_12 },
    { 0x7ac2ba78, SUC_lexicon_tags_2 },
    { 0x51e2ba79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe662ba7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be2ba84, SUC_lexicon_tags_17 },
    { 0x97e2ba85, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc2ba88, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc2ba8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5ac2ba8d, SUC_lexicon_tags_12 },
    { 0x88a2ba8e, SUC_lexicon_tags_11 },
    { 0xe2ba8f, SUC_lexicon_tags_2 },
    { 0x74c2ba8e, SUC_lexicon_tags_11 },
    { 0x9ca2ba8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf0e2ba93, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd522ba95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdcc2ba97, SUC_lexicon_tags_38 },
    { 0x902ba97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee2ba9e, SUC_lexicon_tags_37 },
    { 0xaa42ba9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa2baa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f62baa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a42baa9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf722baac, SUC_lexicon_tags_7 },
    { 0x1ba2baad, SUC_lexicon_tags_28 },
    { 0x4062baae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2902bab0, SUC_lexicon_tags_33 },
    { 0xa3e2bab1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a2bab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc382bab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc222bab9, SUC_lexicon_tags_37 },
    { 0x1122bab9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf122bac1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e42bac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc02bac7, SUC_lexicon_tags_11 },
    { 0xb242bac8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9e2baca, SUC_lexicon_tags_11 },
    { 0x4e42bacb, SUC_lexicon_tags_55 },
    { 0xcf62bacc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a2bad0, SUC_lexicon_tags_17 },
    { 0x4182bad1, SUC_lexicon_tags_7 },
    { 0xca82bad2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2bad7, SUC_lexicon_tags_65 },
    { 0x2e22bad8, SUC_lexicon_tags_27 },
    { 0xe702bad8, SUC_lexicon_tags_17 },
    { 0x3b02bad9, SUC_lexicon_tags_66 },
    { 0x7ac2badb, SUC_lexicon_tags_29 },
    { 0xdf42badc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c02bade, SUC_lexicon_tags_15 },
    { 0xcc02badf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9222bae2, SUC_lexicon_tags_15 },
    { 0xdee2bae3, SUC_lexicon_tags_39 },
    { 0xe282bae2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2902bae7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc282bae9, SUC_lexicon_tags_17 },
    { 0xdb62bae9, SUC_lexicon_tags_0 },
    { 0xf8e2baeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb62baf5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd22baf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6382bafb, SUC_lexicon_tags_16 },
    { 0x1fa2bafb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5782baff, SUC_lexicon_tags_55 },
    { 0x35e2bb00, SUC_lexicon_tags_23 },
    { 0x5642bb01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb342bb06, SUC_lexicon_tags_17 },
    { 0x862bb07, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c2bb0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b82bb10, SUC_lexicon_tags_7 },
    { 0x16e2bb11, SUC_lexicon_tags_3 },
    { 0xa842bb11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c2bb19, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc2bb20, SUC_lexicon_tags_31 },
    { 0x2b22bb21, SUC_lexicon_tags_43 },
    { 0x6322bb21, SUC_lexicon_tags_47 },
    { 0x3942bb23, SUC_lexicon_tags_37 },
    { 0x3242bb24, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9a82bb26, SUC_lexicon_tags_25 },
    { 0x8fc2bb27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7062bb2a, SUC_lexicon_tags_6 },
    { 0x2762bb2b, SUC_lexicon_tags_11 },
    { 0x9ea2bb2b, SUC_lexicon_tags_17 },
    { 0x5302bb2d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc2bb30, SUC_lexicon_tags_60 },
    { 0x83a2bb31, SUC_lexicon_tags_26 },
    { 0x1042bb31, SUC_lexicon_tags_15 },
    { 0x6462bb30, SUC_lexicon_tags_1 },
    { 0xde82bb33, SUC_lexicon_tags_37 },
    { 0xc4a2bb35, SUC_lexicon_tags_113 },
    { 0xe682bb36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd22bb3a, SUC_lexicon_tags_25 },
    { 0xf142bb3a, SUC_lexicon_tags_9 },
    { 0x1502bb3c, SUC_lexicon_tags_60 },
    { 0xb202bb3b, SUC_lexicon_tags_45 },
    { 0xd722bb3e, SUC_lexicon_tags_14 },
    { 0x9842bb3e, SUC_lexicon_tags_23 },
    { 0xb8e2bb40, SUC_lexicon_tags_7 },
    { 0xf802bb3e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf222bb47, SUC_lexicon_tags_60 },
    { 0x8802bb48, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da2bb4c, SUC_lexicon_tags_45 },
    { 0x5582bb4d, SUC_lexicon_tags_43 },
    { 0xac02bb4c, SUC_lexicon_tags_33 },
    { 0x2962bb4f, SUC_lexicon_tags_23 },
    { 0x962bb50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42bb5a, SUC_lexicon_tags_7 },
    { 0x4102bb5b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1a2bb5d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x882bb5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca2bb64, SUC_lexicon_tags_43 },
    { 0xc6e2bb65, SUC_lexicon_tags_11 },
    { 0xde02bb66, SUC_lexicon_tags_1 },
    { 0xba22bb64, SUC_lexicon_tags_32 },
    { 0xf8c2bb64, SUC_lexicon_tags_1 },
    { 0xbc22bb64, SUC_lexicon_tags_25 },
    { 0xbe2bb67, SUC_lexicon_tags_12 },
    { 0x9de2bb64, SUC_lexicon_tags_11 },
    { 0xdc02bb65, SUC_lexicon_tags_7 },
    { 0x3c42bb6a, SUC_lexicon_tags_7 },
    { 0x7822bb68, SUC_lexicon_tags_15 },
    { 0xafe2bb6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe742bb72, SUC_lexicon_tags_14 },
    { 0xc7c2bb73, SUC_lexicon_tags_30 },
    { 0x3022bb74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa2bb77, SUC_lexicon_tags_85 },
    { 0xb722bb78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xce22bb7b, SUC_lexicon_tags_0 },
    { 0xbc2bb7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf7c2bb7e, SUC_lexicon_tags_12 },
    { 0x11a2bb7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd782bb81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9082bb83, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe322bb8b, SUC_lexicon_tags_74 },
    { 0xc122bb8c, SUC_lexicon_tags_13 },
    { 0x1a02bb8b, SUC_lexicon_tags_9 },
    { 0x2c42bb8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd082bb97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30c2bb99, SUC_lexicon_tags_25 },
    { 0x33c2bb9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc2bb9f, SUC_lexicon_tags_7 },
    { 0xf522bba0, SUC_lexicon_tags_37 },
    { 0xae82bb9f, SUC_lexicon_tags_4 },
    { 0xb2c2bba2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92a2bba4, SUC_lexicon_tags_4 },
    { 0xe042bba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1c2bba7, SUC_lexicon_tags_54 },
    { 0x8ce2bba8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9382bbb7, SUC_lexicon_tags_9 },
    { 0xd2e2bbb8, SUC_lexicon_tags_60 },
    { 0x2882bbb7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f02bbbd, SUC_lexicon_tags_40 },
    { 0xb82bbbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe742bbc2, SUC_lexicon_tags_2 },
    { 0x3f42bbc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9522bbc6, SUC_lexicon_tags_87 },
    { 0x8e22bbc7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac2bbd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d22bbdd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6982bbdf, SUC_lexicon_tags_37 },
    { 0xede2bbe0, SUC_lexicon_tags_15 },
    { 0x1b22bbe0, SUC_lexicon_tags_64 },
    { 0x78c2bbdf, SUC_lexicon_tags_33 },
    { 0xcec2bbe3, SUC_lexicon_tags_37 },
    { 0x42a2bbe4, SUC_lexicon_tags_47 },
    { 0x94a2bbe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2bbea, SUC_lexicon_tags_1 },
    { 0x9502bbeb, SUC_lexicon_tags_1 },
    { 0x9ce2bbec, SUC_lexicon_tags_15 },
    { 0x6982bbed, SUC_lexicon_tags_47 },
    { 0x83c2bbee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd82bbf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c42bbf7, SUC_lexicon_tags_39 },
    { 0x8aa2bbf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x542bbfb, SUC_lexicon_tags_47 },
    { 0xca02bbfb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4302bbff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe02bc02, SUC_lexicon_tags_25 },
    { 0xc582bc03, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef82bc07, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x11c2bc09, SUC_lexicon_tags_32 },
    { 0xa462bc0a, SUC_lexicon_tags_7 },
    { 0x1a82bc0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e2bc0e, SUC_lexicon_tags_2 },
    { 0xf0a2bc0f, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5482bc15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9962bc1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc662bc1f, SUC_lexicon_tags_37 },
    { 0xcf82bc20, SUC_lexicon_tags_29 },
    { 0x4d82bc21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe02bc2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaae2bc2c, SUC_lexicon_tags_2 },
    { 0x4962bc2c, SUC_lexicon_tags_32 },
    { 0xdec2bc2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba2bc33, SUC_lexicon_tags_17 },
    { 0x8462bc34, SUC_lexicon_tags_1 },
    { 0xb682bc34, SUC_lexicon_tags_24 },
    { 0xd142bc33, SUC_lexicon_tags_0 },
    { 0x94c2bc35, SUC_lexicon_tags_37 },
    { 0x4e2bc38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1462bc3b, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a62bc3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea82bc40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8362bc46, SUC_lexicon_tags_0 },
    { 0x9702bc47, SUC_lexicon_tags_11 },
    { 0xa882bc47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4222bc4f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe2bc56, SUC_lexicon_tags_79 },
    { 0x25c2bc57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5c2bc59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8422bc63, SUC_lexicon_tags_15 },
    { 0x95e2bc64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d02bc6b, SUC_lexicon_tags_17 },
    { 0xfb02bc6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8582bc72, SUC_lexicon_tags_4 },
    { 0xc0e2bc72, SUC_lexicon_tags_0 },
    { 0xeaa2bc74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f22bc80, SUC_lexicon_tags_65 },
    { 0x1842bc81, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc622bc83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb82bc86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e2bc8b, SUC_lexicon_tags_12 },
    { 0x1802bc8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc662bc90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8682bc92, SUC_lexicon_tags_40 },
    { 0x9222bc92, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee2bc98, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c82bc9e, SUC_lexicon_tags_29 },
    { 0x462bc9f, SUC_lexicon_tags_1069 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c2bca5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d22bca9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa42bcac, SUC_lexicon_tags_2 },
    { 0xfb22bcac, SUC_lexicon_tags_12 },
    { 0xb342bcae, SUC_lexicon_tags_18 },
    { 0x5502bcae, SUC_lexicon_tags_27 },
    { 0xd62bcaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8bc2bcb2, SUC_lexicon_tags_14 },
    { 0x1ca2bcb3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b42bcb5, SUC_lexicon_tags_48 },
    { 0xedc2bcb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2622bcb8, SUC_lexicon_tags_15 },
    { 0x85e2bcb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb42bcbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc2bcc0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62bcca, SUC_lexicon_tags_37 },
    { 0xbf42bcca, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x55c2bccd, SUC_lexicon_tags_15 },
    { 0x5fe2bcce, SUC_lexicon_tags_1 },
    { 0x6682bccf, SUC_lexicon_tags_25 },
    { 0x4f02bcce, SUC_lexicon_tags_25 },
    { 0x3522bcd0, SUC_lexicon_tags_48 },
    { 0xf622bcd2, SUC_lexicon_tags_11 },
    { 0x1dc2bcce, SUC_lexicon_tags_2 },
    { 0xdc02bcce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff62bcd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce2bcdc, SUC_lexicon_tags_15 },
    { 0xa7c2bcdc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a82bce0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc82bce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a2bce9, SUC_lexicon_tags_2 },
    { 0x5c22bce9, SUC_lexicon_tags_12 },
    { 0xd1a2bceb, SUC_lexicon_tags_1 },
    { 0xae2bcec, SUC_lexicon_tags_28 },
    { 0xe2bced, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x762bcf0, SUC_lexicon_tags_8 },
    { 0x5902bcf1, SUC_lexicon_tags_17 },
    { 0x5802bcf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe122bcf4, SUC_lexicon_tags_28 },
    { 0x47e2bcf5, SUC_lexicon_tags_68 },
    { 0xc42bcf5, SUC_lexicon_tags_13 },
    { 0x3962bcf7, SUC_lexicon_tags_7 },
    { 0x4922bcf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0e2bcfa, SUC_lexicon_tags_50 },
    { 0xc2bcfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f22bcfd, SUC_lexicon_tags_43 },
    { 0xc5e2bcfe, SUC_lexicon_tags_43 },
    { 0x94e2bcfd, SUC_lexicon_tags_6 },
    { 0xff42bd00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7dc2bd02, SUC_lexicon_tags_25 },
    { 0x8a2bd03, SUC_lexicon_tags_25 },
    { 0xf582bd04, SUC_lexicon_tags_65 },
    { 0xfbc2bd05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa82bd07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d42bd0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa2bd11, SUC_lexicon_tags_91 },
    { 0xe482bd12, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2402bd18, SUC_lexicon_tags_27 },
    { 0x8a22bd19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b22bd1f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb842bd21, SUC_lexicon_tags_29 },
    { 0x4622bd22, SUC_lexicon_tags_2 },
    { 0x3442bd23, SUC_lexicon_tags_9 },
    { 0xc2bd24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c2bd27, SUC_lexicon_tags_33 },
    { 0x2d22bd28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e02bd2b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x122bd2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2402bd30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc642bd32, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9162bd3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f22bd3e, SUC_lexicon_tags_7 },
    { 0x5502bd3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ec2bd41, SUC_lexicon_tags_33 },
    { 0xe302bd42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62bd46, SUC_lexicon_tags_1 },
    { 0xb622bd47, SUC_lexicon_tags_85 },
    { 0xd3e2bd47, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbd42bd4a, SUC_lexicon_tags_30 },
    { 0xdb22bd4a, SUC_lexicon_tags_15 },
    { 0xf9c2bd4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4862bd51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea62bd56, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdfc2bd58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee2bd5c, SUC_lexicon_tags_32 },
    { 0xf442bd5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb162bd60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1662bd63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae2bd66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc2bd6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x42bd6f, SUC_lexicon_tags_31 },
    { 0xfce2bd6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf42bd74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc2bd77, SUC_lexicon_tags_33 },
    { 0x6ec2bd78, SUC_lexicon_tags_1 },
    { 0xab42bd79, SUC_lexicon_tags_47 },
    { 0x5162bd7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdd02bd7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e2bd85, SUC_lexicon_tags_14 },
    { 0x3c82bd86, SUC_lexicon_tags_2 },
    { 0x5582bd87, SUC_lexicon_tags_30 },
    { 0x3262bd88, SUC_lexicon_tags_17 },
    { 0xe1e2bd87, SUC_lexicon_tags_68 },
    { 0xc5a2bd8a, SUC_lexicon_tags_25 },
    { 0x1b82bd8b, SUC_lexicon_tags_92 },
    { 0xdca2bd8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e2bd92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa422bd94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb22bd9d, SUC_lexicon_tags_63 },
    { 0x50c2bd9e, SUC_lexicon_tags_63 },
    { 0x7f22bd9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad42bda2, SUC_lexicon_tags_66 },
    { 0xa282bda3, SUC_lexicon_tags_28 },
    { 0x4cc2bda4, SUC_lexicon_tags_44 },
    { 0x93e2bda5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa342bda7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c2bdac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a2bdaf, SUC_lexicon_tags_33 },
    { 0x4662bdb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3382bdb2, SUC_lexicon_tags_37 },
    { 0x6322bdb3, SUC_lexicon_tags_12 },
    { 0x7742bdb3, SUC_lexicon_tags_9 },
    { 0x7ce2bdb3, SUC_lexicon_tags_49 },
    { 0x3ee2bdb5, SUC_lexicon_tags_2 },
    { 0x4d42bdb6, SUC_lexicon_tags_23 },
    { 0x2742bdb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8422bdbd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea2bdc4, SUC_lexicon_tags_60 },
    { 0x4602bdc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5422bdc8, SUC_lexicon_tags_31 },
    { 0x1042bdc9, SUC_lexicon_tags_57 },
    { 0x8e62bdca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83c2bdcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e42bdcf, SUC_lexicon_tags_19 },
    { 0x6822bdd0, SUC_lexicon_tags_11 },
    { 0x6e82bdd0, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x4f42bdd3, SUC_lexicon_tags_45 },
    { 0x2ec2bdd3, SUC_lexicon_tags_35 },
    { 0x1e82bdd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b42bdd7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7562bde1, SUC_lexicon_tags_15 },
    { 0x8a62bde2, SUC_lexicon_tags_11 },
    { 0xef82bde3, SUC_lexicon_tags_27 },
    { 0x3062bde3, SUC_lexicon_tags_65 },
    { 0x3ea2bde5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c2bde8, SUC_lexicon_tags_7 },
    { 0x4f02bde9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c2bdec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8722bdf3, SUC_lexicon_tags_91 },
    { 0xabe2bdf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1682bdfa, SUC_lexicon_tags_17 },
    { 0xa282bdfb, SUC_lexicon_tags_27 },
    { 0xc2bdfa, SUC_lexicon_tags_15 },
    { 0xa9c2bdfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da2be01, SUC_lexicon_tags_18 },
    { 0xfd22be02, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4542be09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2782be13, SUC_lexicon_tags_17 },
    { 0x3fa2be13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2be1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe522be21, SUC_lexicon_tags_79 },
    { 0x1ac2be22, SUC_lexicon_tags_1 },
    { 0x722be22, SUC_lexicon_tags_6 },
    { 0x9002be24, SUC_lexicon_tags_15 },
    { 0x9342be24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa22be28, SUC_lexicon_tags_33 },
    { 0x5422be28, SUC_lexicon_tags_25 },
    { 0x4562be2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b62be2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a2be30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8802be35, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2be39, SUC_lexicon_tags_6 },
    { 0x1b42be3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d42be3e, SUC_lexicon_tags_7 },
    { 0x9ca2be3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9402be42, SUC_lexicon_tags_30 },
    { 0x2c22be43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5582be46, SUC_lexicon_tags_11 },
    { 0xf6c2be47, SUC_lexicon_tags_47 },
    { 0x7262be48, SUC_lexicon_tags_28 },
    { 0x5fe2be49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a42be4b, SUC_lexicon_tags_7 },
    { 0x6222be4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de2be54, SUC_lexicon_tags_47 },
    { 0x65e2be55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d42be57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b22be5b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xaba2be5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5042be62, SUC_lexicon_tags_55 },
    { 0x6682be63, SUC_lexicon_tags_14 },
    { 0x22c2be64, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc2be67, SUC_lexicon_tags_23 },
    { 0x2b22be67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c2be6b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982be6f, SUC_lexicon_tags_17 },
    { 0x2042be70, SUC_lexicon_tags_11 },
    { 0x3e62be70, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9422be74, SUC_lexicon_tags_23 },
    { 0xf02be75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe202be78, SUC_lexicon_tags_12 },
    { 0xde2be78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa542be7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a2be87, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e02be8e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa02be92, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb02be96, SUC_lexicon_tags_37 },
    { 0x6942be97, SUC_lexicon_tags_25 },
    { 0xef82be98, SUC_lexicon_tags_37 },
    { 0xc1e2be99, SUC_lexicon_tags_12 },
    { 0xb9a2be9a, SUC_lexicon_tags_11 },
    { 0x4062be98, SUC_lexicon_tags_47 },
    { 0x93e2be9a, SUC_lexicon_tags_19 },
    { 0xa362be9d, SUC_lexicon_tags_1 },
    { 0x25e2be9e, SUC_lexicon_tags_32 },
    { 0xf242be9f, SUC_lexicon_tags_63 },
    { 0x162bea0, SUC_lexicon_tags_12 },
    { 0xbf62bea1, SUC_lexicon_tags_83 },
    { 0xf042bea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bc2bea4, SUC_lexicon_tags_33 },
    { 0x26c2bea5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8842bea9, SUC_lexicon_tags_39 },
    { 0x4622bea9, SUC_lexicon_tags_123 },
    { 0x2d42bea9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e2beaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22beb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e22beb9, SUC_lexicon_tags_1 },
    { 0x53e2beba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd142bebd, SUC_lexicon_tags_17 },
    { 0x8f42bebd, SUC_lexicon_tags_19 },
    { 0x3c2bebf, SUC_lexicon_tags_0 },
    { 0x5542bec0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe862bec2, SUC_lexicon_tags_2 },
    { 0xf282bec3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa2bec7, SUC_lexicon_tags_38 },
    { 0xaf42bec7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x462beca, SUC_lexicon_tags_47 },
    { 0x9182becb, SUC_lexicon_tags_7 },
    { 0xbb42becc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92a2bece, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c2bed1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b22bed4, SUC_lexicon_tags_25 },
    { 0xd3c2bed4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5302bed7, SUC_lexicon_tags_17 },
    { 0xcb82bed7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82bedc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb42bede, SUC_lexicon_tags_172 },
    { 0x1e22bedf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca2bee4, SUC_lexicon_tags_17 },
    { 0x7702bee5, SUC_lexicon_tags_31 },
    { 0x3702bee6, SUC_lexicon_tags_32 },
    { 0xf042bee6, SUC_lexicon_tags_17 },
    { 0x4de2bee5, SUC_lexicon_tags_280 },
    { 0xa662bee9, SUC_lexicon_tags_23 },
    { 0x742bee9, SUC_lexicon_tags_13 },
    { 0xe82bee8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc62beee, SUC_lexicon_tags_17 },
    { 0x1042beef, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9022bef3, SUC_lexicon_tags_15 },
    { 0x1562bef3, SUC_lexicon_tags_63 },
    { 0x4842bef3, SUC_lexicon_tags_60 },
    { 0xe042bef5, SUC_lexicon_tags_1 },
    { 0x8e82bef3, SUC_lexicon_tags_3 },
    { 0x7142bef8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4882befa, SUC_lexicon_tags_5 },
    { 0x1bc2befb, SUC_lexicon_tags_14 },
    { 0x1a02befc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a02bf02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd122bf06, SUC_lexicon_tags_37 },
    { 0x29e2bf07, SUC_lexicon_tags_33 },
    { 0xc142bf08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cc2bf0a, SUC_lexicon_tags_38 },
    { 0x41a2bf0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe2bf0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a2bf13, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d22bf19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9662bf1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2522bf1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c02bf1f, SUC_lexicon_tags_17 },
    { 0xba2bf20, SUC_lexicon_tags_12 },
    { 0x55a2bf21, SUC_lexicon_tags_25 },
    { 0x6e22bf21, SUC_lexicon_tags_25 },
    { 0xbc42bf23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdba2bf25, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd82bf28, SUC_lexicon_tags_337 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1462bf2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac2bf34, SUC_lexicon_tags_15 },
    { 0x6c42bf34, SUC_lexicon_tags_25 },
    { 0x4142bf36, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a2bf39, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3a22bf3b, SUC_lexicon_tags_28 },
    { 0x6ee2bf3b, SUC_lexicon_tags_1 },
    { 0xc862bf3b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc2bf40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c2bf44, SUC_lexicon_tags_47 },
    { 0x4342bf45, SUC_lexicon_tags_31 },
    { 0x3e2bf45, SUC_lexicon_tags_19 },
    { 0xd662bf47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec22bf4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb7a2bf4d, SUC_lexicon_tags_1120 },
    { 0x1882bf4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7d02bf50, SUC_lexicon_tags_8 },
    { 0x1a42bf51, SUC_lexicon_tags_60 },
    { 0x42bf51, SUC_lexicon_tags_37 },
    { 0x942bf53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdac2bf55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb582bf5c, SUC_lexicon_tags_30 },
    { 0x15a2bf5c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xa182bf60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc82bf62, SUC_lexicon_tags_1 },
    { 0xf782bf63, SUC_lexicon_tags_0 },
    { 0x862bf64, SUC_lexicon_tags_19 },
    { 0x55c2bf65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc302bf68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2bf6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e22bf75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcdc2bf77, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc702bf86, SUC_lexicon_tags_49 },
    { 0xd4e2bf86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd122bf89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e02bf8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a2bf8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdb02bf90, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6542bf95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5102bf97, SUC_lexicon_tags_19 },
    { 0x5362bf98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2182bf9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a2bfa4, SUC_lexicon_tags_29 },
    { 0x6da2bfa5, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1382bfa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a2bfad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c2bfb0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c2bfbb, SUC_lexicon_tags_32 },
    { 0x9aa2bfbc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbc82bfbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a2bfc1, SUC_lexicon_tags_40 },
    { 0xa442bfc2, SUC_lexicon_tags_8 },
    { 0xfa82bfc2, SUC_lexicon_tags_25 },
    { 0xce02bfc4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd022bfca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaae2bfcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c2bfd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2342bfd4, SUC_lexicon_tags_13 },
    { 0x8f02bfd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c2bfdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a62bfde, SUC_lexicon_tags_37 },
    { 0x78c2bfdf, SUC_lexicon_tags_23 },
    { 0xd482bfdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf662bfe5, SUC_lexicon_tags_15 },
    { 0xa2a2bfe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9602bfeb, SUC_lexicon_tags_11 },
    { 0xe022bfec, SUC_lexicon_tags_1 },
    { 0x4102bfed, SUC_lexicon_tags_16 },
    { 0xfb02bfee, SUC_lexicon_tags_17 },
    { 0x6a2bfef, SUC_lexicon_tags_7 },
    { 0x9162bfef, SUC_lexicon_tags_2 },
    { 0x43a2bff0, SUC_lexicon_tags_7 },
    { 0xdea2bff0, SUC_lexicon_tags_19 },
    { 0xcae2bff1, SUC_lexicon_tags_69 },
    { 0x7242bff4, SUC_lexicon_tags_25 },
    { 0x37e2bff5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62bff8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb62bffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26e2bffe, SUC_lexicon_tags_39 },
    { 0xab82bffe, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba2c002, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a02c006, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaf62c008, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e2c00c, SUC_lexicon_tags_33 },
    { 0xd022c00d, SUC_lexicon_tags_2 },
    { 0x62c00e, SUC_lexicon_tags_27 },
    { 0x87e2c00f, SUC_lexicon_tags_2 },
    { 0x8162c00e, SUC_lexicon_tags_15 },
    { 0xb42c011, SUC_lexicon_tags_35 },
    { 0x19a2c011, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf22c014, SUC_lexicon_tags_47 },
    { 0x4a2c015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb222c01c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd442c01f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc722c021, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8dc2c023, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc422c025, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc222c028, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c2c02b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5fc2c02d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3422c036, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6162c038, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a62c03d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc382c043, SUC_lexicon_tags_15 },
    { 0xd3a2c043, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa2c047, SUC_lexicon_tags_27 },
    { 0x1cc2c047, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a2c04b, SUC_lexicon_tags_46 },
    { 0xdd42c04c, SUC_lexicon_tags_66 },
    { 0x6bc2c04c, SUC_lexicon_tags_60 },
    { 0xee02c04e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc6c2c050, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb442c053, SUC_lexicon_tags_2 },
    { 0x2b82c054, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd922c056, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a2c05c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6942c05f, SUC_lexicon_tags_25 },
    { 0x4a22c060, SUC_lexicon_tags_25 },
    { 0x6c82c061, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9562c063, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8322c066, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd722c069, SUC_lexicon_tags_37 },
    { 0x5042c06a, SUC_lexicon_tags_33 },
    { 0xe2c06b, SUC_lexicon_tags_14 },
    { 0x4d22c06c, SUC_lexicon_tags_2 },
    { 0x2802c06d, SUC_lexicon_tags_53 },
    { 0xb1a2c06b, SUC_lexicon_tags_8 },
    { 0x4882c069, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9122c071, SUC_lexicon_tags_38 },
    { 0x46e2c071, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e2c078, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c2c081, SUC_lexicon_tags_37 },
    { 0xdc82c082, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x69a2c084, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca2c087, SUC_lexicon_tags_6 },
    { 0xd0e2c087, SUC_lexicon_tags_45 },
    { 0x6e2c089, SUC_lexicon_tags_9 },
    { 0x1722c089, SUC_lexicon_tags_17 },
    { 0x3342c08b, SUC_lexicon_tags_0 },
    { 0x9da2c08c, SUC_lexicon_tags_32 },
    { 0xe082c08a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1162c091, SUC_lexicon_tags_46 },
    { 0x5482c092, SUC_lexicon_tags_3 },
    { 0xe042c092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c2c097, SUC_lexicon_tags_13 },
    { 0x5802c097, SUC_lexicon_tags_37 },
    { 0xe322c099, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa362c09b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa382c09e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca2c0a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf922c0a7, SUC_lexicon_tags_8 },
    { 0xf922c0a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a42c0ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b02c0b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5502c0bb, SUC_lexicon_tags_47 },
    { 0x3802c0bc, SUC_lexicon_tags_190 },
    { 0x3e82c0bd, SUC_lexicon_tags_17 },
    { 0x21a2c0bd, SUC_lexicon_tags_11 },
    { 0x822c0bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8dc2c0c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc2c0c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a2c0ca, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3342c0cd, SUC_lexicon_tags_49 },
    { 0x38c2c0ce, SUC_lexicon_tags_47 },
    { 0x3702c0cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c62c0d2, SUC_lexicon_tags_49 },
    { 0xbea2c0d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce2c0d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4162c0d8, SUC_lexicon_tags_17 },
    { 0x1682c0d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea2c0df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1522c0e1, SUC_lexicon_tags_27 },
    { 0xfde2c0e2, SUC_lexicon_tags_65 },
    { 0x5a2c0e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7162c0e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d62c0e9, SUC_lexicon_tags_30 },
    { 0x4522c0e9, SUC_lexicon_tags_14 },
    { 0xfbe2c0e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd762c0ee, SUC_lexicon_tags_38 },
    { 0x8362c0ee, SUC_lexicon_tags_15 },
    { 0xc62c0f0, SUC_lexicon_tags_25 },
    { 0x43a2c0f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6622c0f6, SUC_lexicon_tags_21 },
    { 0x2c62c0f7, SUC_lexicon_tags_65 },
    { 0xbc02c0f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2c0fc, SUC_lexicon_tags_33 },
    { 0xad42c0fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f22c109, SUC_lexicon_tags_50 },
    { 0x7982c10a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf82c10e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c2c111, SUC_lexicon_tags_14 },
    { 0xfc62c111, SUC_lexicon_tags_17 },
    { 0x41a2c113, SUC_lexicon_tags_30 },
    { 0xa782c114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa2c118, SUC_lexicon_tags_7 },
    { 0x9ba2c119, SUC_lexicon_tags_0 },
    { 0x1ac2c11a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8802c11c, SUC_lexicon_tags_12 },
    { 0x7b42c11c, SUC_lexicon_tags_46 },
    { 0x8142c11c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c2c125, SUC_lexicon_tags_11 },
    { 0xd7e2c126, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa762c129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd02c12d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcc2c12f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e2c136, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6ee2c138, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c2c13b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc202c13d, SUC_lexicon_tags_17 },
    { 0x5ae2c13d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3522c141, SUC_lexicon_tags_17 },
    { 0x2ca2c142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd02c144, SUC_lexicon_tags_17 },
    { 0x7a22c144, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42c149, SUC_lexicon_tags_2 },
    { 0xaa42c149, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c42c14c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe722c14e, SUC_lexicon_tags_7 },
    { 0xb8e2c14e, SUC_lexicon_tags_30 },
    { 0x3fc2c150, SUC_lexicon_tags_27 },
    { 0x6d22c151, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec2c156, SUC_lexicon_tags_37 },
    { 0xbbe2c156, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf5e2c159, SUC_lexicon_tags_12 },
    { 0xb442c15a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e2c161, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea2c166, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec2c16f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d02c171, SUC_lexicon_tags_14 },
    { 0x7862c172, SUC_lexicon_tags_56 },
    { 0x622c173, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc882c176, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa2c17b, SUC_lexicon_tags_15 },
    { 0xc2e2c17c, SUC_lexicon_tags_7 },
    { 0xa742c17d, SUC_lexicon_tags_19 },
    { 0xcc42c17e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d82c182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a2c186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa782c18a, SUC_lexicon_tags_1 },
    { 0x9dc2c18a, SUC_lexicon_tags_45 },
    { 0xc442c18c, SUC_lexicon_tags_27 },
    { 0xdcc2c18d, SUC_lexicon_tags_39 },
    { 0x8702c18d, SUC_lexicon_tags_34 },
    { 0x1762c18f, SUC_lexicon_tags_7 },
    { 0xb622c190, SUC_lexicon_tags_15 },
    { 0xbc22c18f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac2c194, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9802c19d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e2c1a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd122c1a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb942c1a7, SUC_lexicon_tags_25 },
    { 0xd62c1a8, SUC_lexicon_tags_15 },
    { 0xeb82c1a7, SUC_lexicon_tags_32 },
    { 0xe7c2c1aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6282c1ae, SUC_lexicon_tags_42 },
    { 0x4ce2c1af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb22c1b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb562c1b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab42c1bd, SUC_lexicon_tags_37 },
    { 0x48e2c1bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70c2c1c0, SUC_lexicon_tags_21 },
    { 0x17a2c1c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaae2c1c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbc62c1c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba82c1c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e2c1ca, SUC_lexicon_tags_29 },
    { 0x942c1ca, SUC_lexicon_tags_11 },
    { 0x322c1cc, SUC_lexicon_tags_37 },
    { 0x87c2c1ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42e2c1cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2c1d3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbb82c1d5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbe42c1d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8102c1db, SUC_lexicon_tags_47 },
    { 0xb122c1dc, SUC_lexicon_tags_201 },
    { 0x2462c1dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8242c1df, SUC_lexicon_tags_13 },
    { 0x27a2c1e0, SUC_lexicon_tags_9 },
    { 0x5762c1e0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbd42c1e3, SUC_lexicon_tags_17 },
    { 0x4842c1e3, SUC_lexicon_tags_1 },
    { 0x3542c1e5, SUC_lexicon_tags_2 },
    { 0x9282c1e5, SUC_lexicon_tags_6 },
    { 0xbb22c1e7, SUC_lexicon_tags_11 },
    { 0xcea2c1e7, SUC_lexicon_tags_45 },
    { 0x9442c1e7, SUC_lexicon_tags_2 },
    { 0x1962c1ea, SUC_lexicon_tags_15 },
    { 0x2342c1ea, SUC_lexicon_tags_23 },
    { 0xf882c1eb, SUC_lexicon_tags_37 },
    { 0x1182c1eb, SUC_lexicon_tags_1 },
    { 0x7522c1e9, SUC_lexicon_tags_1 },
    { 0x62c2c1ef, SUC_lexicon_tags_11 },
    { 0x57e2c1f0, SUC_lexicon_tags_15 },
    { 0xdd62c1f1, SUC_lexicon_tags_49 },
    { 0xe42c1e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5922c1f6, SUC_lexicon_tags_23 },
    { 0x1922c1f6, SUC_lexicon_tags_25 },
    { 0x8442c1f7, SUC_lexicon_tags_1221 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4582c1fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f22c201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa62c209, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x162c20c, SUC_lexicon_tags_17 },
    { 0x23a2c20d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2482c215, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5c82c217, SUC_lexicon_tags_6 },
    { 0xbdc2c218, SUC_lexicon_tags_37 },
    { 0xc4c2c217, SUC_lexicon_tags_18 },
    { 0x8022c21a, SUC_lexicon_tags_11 },
    { 0x1982c21b, SUC_lexicon_tags_38 },
    { 0x8ae2c21b, SUC_lexicon_tags_47 },
    { 0xa2e2c21b, SUC_lexicon_tags_25 },
    { 0x59c2c21e, SUC_lexicon_tags_11 },
    { 0x5482c21f, SUC_lexicon_tags_17 },
    { 0x5bc2c21f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e2c224, SUC_lexicon_tags_4 },
    { 0x6c42c225, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3462c236, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb842c23c, SUC_lexicon_tags_31 },
    { 0x5a82c23d, SUC_lexicon_tags_11 },
    { 0x982c23e, SUC_lexicon_tags_6 },
    { 0xd402c23f, SUC_lexicon_tags_20 },
    { 0xea62c23e, SUC_lexicon_tags_31 },
    { 0x5d02c241, SUC_lexicon_tags_87 },
    { 0xabc2c240, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1982c246, SUC_lexicon_tags_39 },
    { 0x5342c247, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6662c249, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb742c24b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x36e2c24d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a2c253, SUC_lexicon_tags_17 },
    { 0xe8a2c254, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de2c258, SUC_lexicon_tags_17 },
    { 0x1202c259, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c2c260, SUC_lexicon_tags_12 },
    { 0xa8a2c261, SUC_lexicon_tags_15 },
    { 0x5542c260, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c2c26c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4042c26e, SUC_lexicon_tags_25 },
    { 0xf802c26f, SUC_lexicon_tags_49 },
    { 0xd2e2c26f, SUC_lexicon_tags_11 },
    { 0xd22c270, SUC_lexicon_tags_2 },
    { 0x402c272, SUC_lexicon_tags_1 },
    { 0x3d42c26e, SUC_lexicon_tags_37 },
    { 0x6922c274, SUC_lexicon_tags_23 },
    { 0x1d82c274, SUC_lexicon_tags_43 },
    { 0xa0e2c270, SUC_lexicon_tags_47 },
    { 0x7ac2c277, SUC_lexicon_tags_33 },
    { 0xdf42c278, SUC_lexicon_tags_43 },
    { 0xe6c2c279, SUC_lexicon_tags_17 },
    { 0x3a02c27a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc02c283, SUC_lexicon_tags_6 },
    { 0xe9e2c284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e2c28a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd22c293, SUC_lexicon_tags_61 },
    { 0x1382c294, SUC_lexicon_tags_25 },
    { 0xeaa2c293, SUC_lexicon_tags_37 },
    { 0x72c2c295, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4982c298, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b62c29d, SUC_lexicon_tags_47 },
    { 0x2d02c29e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec2c2a2, SUC_lexicon_tags_37 },
    { 0x5662c2a2, SUC_lexicon_tags_14 },
    { 0x2002c2a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdee2c2a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2a2c2a8, SUC_lexicon_tags_11 },
    { 0x4e42c2a8, SUC_lexicon_tags_37 },
    { 0xbd42c2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3562c2b2, SUC_lexicon_tags_1 },
    { 0x1242c2b3, SUC_lexicon_tags_12 },
    { 0xc042c2b4, SUC_lexicon_tags_6 },
    { 0x6ea2c2b5, SUC_lexicon_tags_10 },
    { 0x7a82c2b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95e2c2b8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a2c2bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac42c2bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e62c2bf, SUC_lexicon_tags_0 },
    { 0x2fe2c2c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e2c2c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xda62c2c8, SUC_lexicon_tags_30 },
    { 0x3bc2c2c9, SUC_lexicon_tags_47 },
    { 0x8d62c2ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de2c2ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x18c2c2d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a2c2d3, SUC_lexicon_tags_7 },
    { 0xe102c2d3, SUC_lexicon_tags_37 },
    { 0x3382c2d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82c2db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7e2c2dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc2c2e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6342c2ec, SUC_lexicon_tags_15 },
    { 0xcce2c2ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8902c2ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e2c2f3, SUC_lexicon_tags_7 },
    { 0xdc2c2f3, SUC_lexicon_tags_9 },
    { 0xe82c2f4, SUC_lexicon_tags_27 },
    { 0x6422c2f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe4a2c2f8, SUC_lexicon_tags_19 },
    { 0xfe2c2f9, SUC_lexicon_tags_17 },
    { 0xe0a2c2fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4262c2fc, SUC_lexicon_tags_66 },
    { 0xdca2c2fd, SUC_lexicon_tags_60 },
    { 0x40c2c2fe, SUC_lexicon_tags_87 },
    { 0xb202c2fe, SUC_lexicon_tags_10 },
    { 0x4a22c2fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ec2c302, SUC_lexicon_tags_11 },
    { 0x3b82c303, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c2c309, SUC_lexicon_tags_17 },
    { 0x2a2c30a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c2c311, SUC_lexicon_tags_25 },
    { 0x4902c312, SUC_lexicon_tags_12 },
    { 0x2c22c311, SUC_lexicon_tags_49 },
    { 0xf1c2c314, SUC_lexicon_tags_37 },
    { 0x5c02c313, SUC_lexicon_tags_2 },
    { 0x30c2c312, SUC_lexicon_tags_38 },
    { 0x6702c317, SUC_lexicon_tags_37 },
    { 0x16a2c318, SUC_lexicon_tags_45 },
    { 0xa322c319, SUC_lexicon_tags_16 },
    { 0xed22c31a, SUC_lexicon_tags_27 },
    { 0xbe02c319, SUC_lexicon_tags_17 },
    { 0x3022c317, SUC_lexicon_tags_24 },
    { 0x4d62c315, SUC_lexicon_tags_47 },
    { 0xf202c31a, SUC_lexicon_tags_17 },
    { 0xb722c31f, SUC_lexicon_tags_12 },
    { 0x4902c320, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f22c324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa2c32e, SUC_lexicon_tags_18 },
    { 0x6de2c32f, SUC_lexicon_tags_187 },
    { 0x67c2c32f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e22c334, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa142c337, SUC_lexicon_tags_49 },
    { 0x32a2c338, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce62c33a, SUC_lexicon_tags_25 },
    { 0xc102c33b, SUC_lexicon_tags_34 },
    { 0x742c33c, SUC_lexicon_tags_2 },
    { 0xca82c33c, SUC_lexicon_tags_37 },
    { 0xaf42c33e, SUC_lexicon_tags_30 },
    { 0xece2c33f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb6a2c341, SUC_lexicon_tags_23 },
    { 0x5ce2c341, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36a2c344, SUC_lexicon_tags_16 },
    { 0x3702c345, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e02c348, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3662c34a, SUC_lexicon_tags_17 },
    { 0x8682c34a, SUC_lexicon_tags_48 },
    { 0x8a22c34a, SUC_lexicon_tags_23 },
    { 0x8562c34c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x262c34f, SUC_lexicon_tags_45 },
    { 0xe0a2c350, SUC_lexicon_tags_31 },
    { 0x3cc2c34f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc9e2c353, SUC_lexicon_tags_11 },
    { 0x9c2c354, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x17c2c356, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9702c35b, SUC_lexicon_tags_25 },
    { 0x8be2c35b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd82c35e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe62c362, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a2c367, SUC_lexicon_tags_16 },
    { 0x5042c368, SUC_lexicon_tags_1 },
    { 0xd182c368, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a2c36c, SUC_lexicon_tags_94 },
    { 0x6802c36c, SUC_lexicon_tags_1 },
    { 0x1602c36e, SUC_lexicon_tags_37 },
    { 0x6a62c36f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdf02c371, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68e2c373, SUC_lexicon_tags_23 },
    { 0x41e2c374, SUC_lexicon_tags_27 },
    { 0x2122c373, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee2c377, SUC_lexicon_tags_9 },
    { 0x4822c378, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70e2c37a, SUC_lexicon_tags_1 },
    { 0x62c37b, SUC_lexicon_tags_7 },
    { 0xea22c37b, SUC_lexicon_tags_31 },
    { 0x7ce2c37d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b82c37f, SUC_lexicon_tags_43 },
    { 0xbd02c37f, SUC_lexicon_tags_66 },
    { 0x602c380, SUC_lexicon_tags_45 },
    { 0x23c2c380, SUC_lexicon_tags_29 },
    { 0xa6c2c383, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a22c385, SUC_lexicon_tags_1 },
    { 0x4d42c386, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b02c388, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62c38c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x48c2c38e, SUC_lexicon_tags_31 },
    { 0x3a2c38f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a2c399, SUC_lexicon_tags_66 },
    { 0xe322c39a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa462c3a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc2c3a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbfa2c3a9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c2c3ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c2c3b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4782c3b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc2c3b9, SUC_lexicon_tags_17 },
    { 0x47c2c3b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf962c3bc, SUC_lexicon_tags_60 },
    { 0x7262c3bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3482c3c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9242c3c4, SUC_lexicon_tags_4 },
    { 0xdde2c3c5, SUC_lexicon_tags_8 },
    { 0xa102c3c6, SUC_lexicon_tags_13 },
    { 0xf002c3c6, SUC_lexicon_tags_2 },
    { 0xd02c3c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce2c3d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe7e2c3d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7122c3d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2222c3e0, SUC_lexicon_tags_49 },
    { 0x7382c3e1, SUC_lexicon_tags_44 },
    { 0xa102c3e2, SUC_lexicon_tags_63 },
    { 0x5442c3e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1842c3e5, SUC_lexicon_tags_60 },
    { 0xa902c3e6, SUC_lexicon_tags_15 },
    { 0x8462c3e7, SUC_lexicon_tags_48 },
    { 0x1d22c3e8, SUC_lexicon_tags_7 },
    { 0x77c2c3e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ce2c3eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a2c3ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61a2c3f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe622c3f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522c3f8, SUC_lexicon_tags_27 },
    { 0x41a2c3f9, SUC_lexicon_tags_23 },
    { 0xd722c3fa, SUC_lexicon_tags_29 },
    { 0x5a2c3f9, SUC_lexicon_tags_14 },
    { 0xd282c3fa, SUC_lexicon_tags_10 },
    { 0x2702c3fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2c407, SUC_lexicon_tags_37 },
    { 0xb362c408, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe222c40c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa042c410, SUC_lexicon_tags_2 },
    { 0xc742c411, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x782c413, SUC_lexicon_tags_30 },
    { 0x7162c413, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb0e2c416, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7022c421, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2c424, SUC_lexicon_tags_17 },
    { 0xc82c424, SUC_lexicon_tags_25 },
    { 0xd002c426, SUC_lexicon_tags_65 },
    { 0x402c425, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7862c429, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a02c42e, SUC_lexicon_tags_1 },
    { 0xb242c42e, SUC_lexicon_tags_7 },
    { 0xa1e2c430, SUC_lexicon_tags_1 },
    { 0xf442c430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2822c433, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2c439, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb102c43c, SUC_lexicon_tags_12 },
    { 0xd242c43c, SUC_lexicon_tags_33 },
    { 0x16c2c43e, SUC_lexicon_tags_15 },
    { 0x77a2c43e, SUC_lexicon_tags_25 },
    { 0x6b82c440, SUC_lexicon_tags_60 },
    { 0xd202c441, SUC_lexicon_tags_31 },
    { 0xfaa2c43f, SUC_lexicon_tags_151 },
    { 0xf1e2c441, SUC_lexicon_tags_60 },
    { 0xfe62c444, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1142c44e, SUC_lexicon_tags_0 },
    { 0x5d22c44f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16e2c451, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6042c453, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a2c456, SUC_lexicon_tags_9 },
    { 0x58c2c456, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b42c45a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ec2c45c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5482c45f, SUC_lexicon_tags_11 },
    { 0x29c2c45f, SUC_lexicon_tags_37 },
    { 0x8a2c461, SUC_lexicon_tags_2 },
    { 0x2ac2c462, SUC_lexicon_tags_68 },
    { 0x8982c463, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4342c465, SUC_lexicon_tags_33 },
    { 0x262c465, SUC_lexicon_tags_15 },
    { 0x2882c467, SUC_lexicon_tags_15 },
    { 0x64c2c466, SUC_lexicon_tags_7 },
    { 0x21a2c469, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c22c46b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c2c46f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02c473, SUC_lexicon_tags_34 },
    { 0xe202c474, SUC_lexicon_tags_7 },
    { 0x3a42c473, SUC_lexicon_tags_77 },
    { 0xcf22c473, SUC_lexicon_tags_11 },
    { 0xdc2c477, SUC_lexicon_tags_63 },
    { 0xfc42c477, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc2c47d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a2c482, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d62c486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53c2c488, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e2c491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9582c493, SUC_lexicon_tags_4 },
    { 0xdb82c494, SUC_lexicon_tags_63 },
    { 0x9582c493, SUC_lexicon_tags_38 },
    { 0x20a2c496, SUC_lexicon_tags_17 },
    { 0x84c2c497, SUC_lexicon_tags_23 },
    { 0x8822c497, SUC_lexicon_tags_25 },
    { 0xf282c498, SUC_lexicon_tags_39 },
    { 0x6202c49a, SUC_lexicon_tags_14 },
    { 0x2c2c49a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf62c49f, SUC_lexicon_tags_82 },
    { 0xd402c4a0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x1ec2c4a2, SUC_lexicon_tags_37 },
    { 0xba02c4a2, SUC_lexicon_tags_6 },
    { 0x8c22c4a2, SUC_lexicon_tags_10 },
    { 0x21a2c4a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e2c4ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed62c4b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ac2c4b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc2c4b7, SUC_lexicon_tags_17 },
    { 0x38a2c4b8, SUC_lexicon_tags_17 },
    { 0xad42c4b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be2c4c2, SUC_lexicon_tags_17 },
    { 0x72c2c4c3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x88c2c4c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2102c4c9, SUC_lexicon_tags_47 },
    { 0x79c2c4ca, SUC_lexicon_tags_57 },
    { 0x2322c4cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e82c4cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa462c4cf, SUC_lexicon_tags_25 },
    { 0xd002c4d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5942c4d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8582c4db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc2c4de, SUC_lexicon_tags_7 },
    { 0x522c4de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf322c4e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec2c4e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02c4eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xda62c4ed, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6b02c4ef, SUC_lexicon_tags_1 },
    { 0x8b02c4f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeea2c4f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4682c4f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c2c4fb, SUC_lexicon_tags_42 },
    { 0xab82c4fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe962c500, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a22c505, SUC_lexicon_tags_17 },
    { 0xfbe2c505, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9582c50a, SUC_lexicon_tags_7 },
    { 0x1ae2c50b, SUC_lexicon_tags_7 },
    { 0x82e2c50c, SUC_lexicon_tags_37 },
    { 0x7a2c50c, SUC_lexicon_tags_47 },
    { 0x8402c50e, SUC_lexicon_tags_25 },
    { 0x5082c50f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b22c512, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2422c514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e2c517, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a2c51a, SUC_lexicon_tags_2 },
    { 0xbb42c51a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4442c51d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e2c529, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x78c2c52b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b62c532, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x80a2c534, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6562c536, SUC_lexicon_tags_17 },
    { 0x26e2c536, SUC_lexicon_tags_70 },
    { 0xf6e2c537, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902c53d, SUC_lexicon_tags_70 },
    { 0xa262c53e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c82c542, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa222c548, SUC_lexicon_tags_46 },
    { 0xc982c549, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaec2c54b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a42c552, SUC_lexicon_tags_17 },
    { 0x5da2c553, SUC_lexicon_tags_17 },
    { 0x8122c554, SUC_lexicon_tags_15 },
    { 0x442c553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd742c557, SUC_lexicon_tags_15 },
    { 0x1c82c558, SUC_lexicon_tags_57 },
    { 0x8ee2c559, SUC_lexicon_tags_15 },
    { 0xe6c2c55a, SUC_lexicon_tags_8 },
    { 0xc22c559, SUC_lexicon_tags_25 },
    { 0xae62c55c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2602c55e, SUC_lexicon_tags_7 },
    { 0x6b02c55f, SUC_lexicon_tags_7 },
    { 0x2ac2c55e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4042c563, SUC_lexicon_tags_60 },
    { 0x5fa2c564, SUC_lexicon_tags_15 },
    { 0xc6c2c565, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8702c567, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ee2c569, SUC_lexicon_tags_37 },
    { 0xdda2c56a, SUC_lexicon_tags_10 },
    { 0xa1e2c56b, SUC_lexicon_tags_13 },
    { 0xbc2c56b, SUC_lexicon_tags_17 },
    { 0x54c2c56d, SUC_lexicon_tags_17 },
    { 0xfd82c56e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e2c573, SUC_lexicon_tags_7 },
    { 0xc6c2c574, SUC_lexicon_tags_7 },
    { 0xb7a2c574, SUC_lexicon_tags_29 },
    { 0x3522c576, SUC_lexicon_tags_2 },
    { 0xc462c577, SUC_lexicon_tags_43 },
    { 0x6ea2c575, SUC_lexicon_tags_43 },
    { 0x9682c576, SUC_lexicon_tags_25 },
    { 0x1b42c57a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa82c57c, SUC_lexicon_tags_65 },
    { 0x6f42c57c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc2c583, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa2c58a, SUC_lexicon_tags_32 },
    { 0xe6c2c58b, SUC_lexicon_tags_15 },
    { 0xeb62c58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7c2c58e, SUC_lexicon_tags_43 },
    { 0x8c82c58f, SUC_lexicon_tags_25 },
    { 0xbb02c58f, SUC_lexicon_tags_25 },
    { 0x6ca2c58e, SUC_lexicon_tags_60 },
    { 0x6082c592, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf582c598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc882c59a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf562c5a2, SUC_lexicon_tags_21 },
    { 0x462c5a3, SUC_lexicon_tags_2 },
    { 0x3682c5a4, SUC_lexicon_tags_1 },
    { 0xd162c5a5, SUC_lexicon_tags_30 },
    { 0xc1e2c5a3, SUC_lexicon_tags_23 },
    { 0x1c22c5a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e2c5ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d42c5ae, SUC_lexicon_tags_0 },
    { 0xfce2c5ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3362c5b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2382c5b9, SUC_lexicon_tags_12 },
    { 0x6a82c5b9, SUC_lexicon_tags_31 },
    { 0xe202c5b9, SUC_lexicon_tags_19 },
    { 0xdd42c5bc, SUC_lexicon_tags_25 },
    { 0xde22c5bd, SUC_lexicon_tags_26 },
    { 0xd4e2c5be, SUC_lexicon_tags_69 },
    { 0xb7c2c5bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeac2c5c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1dc2c5c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f22c5c7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5202c5cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e82c5d0, SUC_lexicon_tags_29 },
    { 0x1c02c5d0, SUC_lexicon_tags_7 },
    { 0xcd22c5d2, SUC_lexicon_tags_30 },
    { 0x7342c5d3, SUC_lexicon_tags_37 },
    { 0x4642c5d0, SUC_lexicon_tags_27 },
    { 0x7722c5d5, SUC_lexicon_tags_28 },
    { 0x43a2c5d4, SUC_lexicon_tags_12 },
    { 0xd22c5d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb02c5da, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3bc2c5dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e2c5e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2742c5e3, SUC_lexicon_tags_91 },
    { 0xa922c5e3, SUC_lexicon_tags_37 },
    { 0x71c2c5e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea82c5ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffa2c5f1, SUC_lexicon_tags_6 },
    { 0x42c2c5f2, SUC_lexicon_tags_25 },
    { 0x47e2c5f3, SUC_lexicon_tags_25 },
    { 0x4f42c5f3, SUC_lexicon_tags_57 },
    { 0xa342c5f5, SUC_lexicon_tags_11 },
    { 0xf882c5f6, SUC_lexicon_tags_25 },
    { 0xc822c5f7, SUC_lexicon_tags_27 },
    { 0x62e2c5f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf262c5fa, SUC_lexicon_tags_37 },
    { 0xe6c2c5fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdde2c5fd, SUC_lexicon_tags_2 },
    { 0x7522c5fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba2c603, SUC_lexicon_tags_25 },
    { 0xe82c603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf242c608, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a2c60c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c2c60f, SUC_lexicon_tags_2 },
    { 0x7de2c60f, SUC_lexicon_tags_44 },
    { 0x8c02c611, SUC_lexicon_tags_11 },
    { 0xe802c612, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1602c615, SUC_lexicon_tags_30 },
    { 0xafc2c615, SUC_lexicon_tags_81 },
    { 0x2882c616, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbee2c619, SUC_lexicon_tags_37 },
    { 0xa542c619, SUC_lexicon_tags_15 },
    { 0x8e82c61b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda42c61e, SUC_lexicon_tags_25 },
    { 0xee2c61f, SUC_lexicon_tags_5 },
    { 0x3f82c620, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf6e2c622, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5542c624, SUC_lexicon_tags_16 },
    { 0x422c625, SUC_lexicon_tags_23 },
    { 0x38c2c626, SUC_lexicon_tags_12 },
    { 0xcea2c627, SUC_lexicon_tags_14 },
    { 0xfd62c625, SUC_lexicon_tags_25 },
    { 0xce82c627, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb02c631, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x7aa2c633, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e82c635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e2c638, SUC_lexicon_tags_25 },
    { 0xa642c639, SUC_lexicon_tags_11 },
    { 0xf002c63a, SUC_lexicon_tags_2 },
    { 0x2a22c639, SUC_lexicon_tags_38 },
    { 0x222c63a, SUC_lexicon_tags_37 },
    { 0xabe2c63d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e2c644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa02c646, SUC_lexicon_tags_7 },
    { 0x5922c646, SUC_lexicon_tags_25 },
    { 0x3c2c646, SUC_lexicon_tags_63 },
    { 0x3362c648, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f62c64d, SUC_lexicon_tags_30 },
    { 0xa0a2c64e, SUC_lexicon_tags_12 },
    { 0x9c62c64e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e22c652, SUC_lexicon_tags_39 },
    { 0x5562c653, SUC_lexicon_tags_10 },
    { 0x61a2c652, SUC_lexicon_tags_11 },
    { 0x3b82c652, SUC_lexicon_tags_31 },
    { 0x2d02c656, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e2c659, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342c65e, SUC_lexicon_tags_23 },
    { 0x1d62c65e, SUC_lexicon_tags_15 },
    { 0x9762c660, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7962c665, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcdc2c667, SUC_lexicon_tags_25 },
    { 0x5422c668, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac2c66c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a2c66f, SUC_lexicon_tags_87 },
    { 0xa1e2c66f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8f42c672, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1442c676, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c2c67a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4082c67c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c2c67f, SUC_lexicon_tags_17 },
    { 0xebe2c680, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d02c682, SUC_lexicon_tags_48 },
    { 0xe602c682, SUC_lexicon_tags_16 },
    { 0x6422c683, SUC_lexicon_tags_37 },
    { 0x842c684, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3582c68e, SUC_lexicon_tags_14 },
    { 0x3002c68f, SUC_lexicon_tags_19 },
    { 0xadc2c690, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd682c695, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf02c6a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262c6ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2a2c6af, SUC_lexicon_tags_25 },
    { 0xc022c6b0, SUC_lexicon_tags_0 },
    { 0xe482c6af, SUC_lexicon_tags_47 },
    { 0xdbe2c6b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab22c6b5, SUC_lexicon_tags_19 },
    { 0x6882c6b6, SUC_lexicon_tags_11 },
    { 0xfe02c6b5, SUC_lexicon_tags_25 },
    { 0xd582c6b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a2c6bb, SUC_lexicon_tags_15 },
    { 0x3f82c6bb, SUC_lexicon_tags_3 },
    { 0x4a42c6bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee2c6c0, SUC_lexicon_tags_27 },
    { 0x5a42c6c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7842c6c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc42c6c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e2c6cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a62c6d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5742c6d3, SUC_lexicon_tags_11 },
    { 0xaf22c6d4, SUC_lexicon_tags_12 },
    { 0xd0c2c6d4, SUC_lexicon_tags_2 },
    { 0xf282c6d6, SUC_lexicon_tags_28 },
    { 0x4362c6d4, SUC_lexicon_tags_7 },
    { 0x3822c6d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb62c6da, SUC_lexicon_tags_82 },
    { 0x11e2c6db, SUC_lexicon_tags_25 },
    { 0x1b02c6db, SUC_lexicon_tags_38 },
    { 0x1ea2c6db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8182c6e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe642c6e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x12a2c6e7, SUC_lexicon_tags_10 },
    { 0xebe2c6e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ce2c6ea, SUC_lexicon_tags_11 },
    { 0x7e62c6ea, SUC_lexicon_tags_35 },
    { 0x1ae2c6ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda42c6ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f22c6f4, SUC_lexicon_tags_33 },
    { 0x58c2c6f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7762c6f9, SUC_lexicon_tags_26 },
    { 0x8b62c6fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc042c6fd, SUC_lexicon_tags_1 },
    { 0x6842c6fd, SUC_lexicon_tags_48 },
    { 0xafe2c6ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7602c701, SUC_lexicon_tags_49 },
    { 0x8ea2c702, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x87c2c704, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb02c70b, SUC_lexicon_tags_10 },
    { 0x8a22c70c, SUC_lexicon_tags_11 },
    { 0xeb42c70d, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82c713, SUC_lexicon_tags_1 },
    { 0xec42c714, SUC_lexicon_tags_11 },
    { 0x9b42c715, SUC_lexicon_tags_27 },
    { 0xef62c715, SUC_lexicon_tags_82 },
    { 0x2442c716, SUC_lexicon_tags_65 },
    { 0x9242c717, SUC_lexicon_tags_47 },
    { 0x45c2c719, SUC_lexicon_tags_19 },
    { 0xd762c719, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b22c71f, SUC_lexicon_tags_46 },
    { 0xb4e2c71f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da2c723, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x18a2c725, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc702c72a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd122c72d, SUC_lexicon_tags_25 },
    { 0x1b02c72d, SUC_lexicon_tags_4 },
    { 0xae2c72f, SUC_lexicon_tags_1 },
    { 0x59e2c72e, SUC_lexicon_tags_6 },
    { 0xc7c2c731, SUC_lexicon_tags_47 },
    { 0x17a2c72f, SUC_lexicon_tags_12 },
    { 0xb602c733, SUC_lexicon_tags_25 },
    { 0x7f42c733, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3622c736, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e2c73b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd542c73d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c2c742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf222c744, SUC_lexicon_tags_25 },
    { 0xcd82c744, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3dc2c747, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ac2c749, SUC_lexicon_tags_25 },
    { 0xcf02c749, SUC_lexicon_tags_1619 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb062c74f, SUC_lexicon_tags_0 },
    { 0x1ea2c750, SUC_lexicon_tags_23 },
    { 0x3fa2c750, SUC_lexicon_tags_31 },
    { 0xdde2c750, SUC_lexicon_tags_63 },
    { 0x9722c753, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd782c755, SUC_lexicon_tags_12 },
    { 0x34c2c756, SUC_lexicon_tags_0 },
    { 0xeee2c757, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6702c75d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f42c763, SUC_lexicon_tags_25 },
    { 0xd802c764, SUC_lexicon_tags_25 },
    { 0x5782c764, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc62c767, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfc02c769, SUC_lexicon_tags_2 },
    { 0xbf42c769, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe862c76d, SUC_lexicon_tags_17 },
    { 0xa882c76d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa462c772, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3102c77b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f82c77d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9962c77f, SUC_lexicon_tags_30 },
    { 0x8722c780, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c2c783, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a2c78d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x91e2c78f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a02c793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5522c79a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb842c79c, SUC_lexicon_tags_56 },
    { 0x8d22c79c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c2c7a2, SUC_lexicon_tags_10 },
    { 0x1642c7a3, SUC_lexicon_tags_25 },
    { 0x4522c7a4, SUC_lexicon_tags_18 },
    { 0x9b02c7a5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x7002c7a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3602c7aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd942c7af, SUC_lexicon_tags_14 },
    { 0x3402c7b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6302c7b2, SUC_lexicon_tags_33 },
    { 0x3602c7b2, SUC_lexicon_tags_1 },
    { 0x6bc2c7b4, SUC_lexicon_tags_25 },
    { 0xafe2c7b5, SUC_lexicon_tags_17 },
    { 0x6f02c7b6, SUC_lexicon_tags_27 },
    { 0x7102c7b6, SUC_lexicon_tags_30 },
    { 0xa0e2c7b8, SUC_lexicon_tags_172 },
    { 0x23e2c7b9, SUC_lexicon_tags_17 },
    { 0xd482c7b8, SUC_lexicon_tags_85 },
    { 0xdf02c7bb, SUC_lexicon_tags_17 },
    { 0xa922c7bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6722c7bf, SUC_lexicon_tags_9 },
    { 0xee62c7c0, SUC_lexicon_tags_37 },
    { 0xc322c7c1, SUC_lexicon_tags_38 },
    { 0xd342c7c2, SUC_lexicon_tags_12 },
    { 0xbbe2c7c2, SUC_lexicon_tags_2 },
    { 0xf242c7c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1962c7ca, SUC_lexicon_tags_63 },
    { 0x6082c7cb, SUC_lexicon_tags_11 },
    { 0xe7e2c7cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc502c7d4, SUC_lexicon_tags_25 },
    { 0x3842c7d4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b22c7d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x922c7dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6922c7de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da2c7e3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc262c7eb, SUC_lexicon_tags_12 },
    { 0x8462c7ec, SUC_lexicon_tags_65 },
    { 0xab82c7ec, SUC_lexicon_tags_12 },
    { 0x3802c7eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd82c7f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3e2c7f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb162c7f6, SUC_lexicon_tags_11 },
    { 0xac42c7f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83a2c7f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63a2c7fb, SUC_lexicon_tags_17 },
    { 0x56a2c7fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf062c803, SUC_lexicon_tags_0 },
    { 0xe202c803, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb42c809, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c2c80d, SUC_lexicon_tags_27 },
    { 0x63a2c80e, SUC_lexicon_tags_20 },
    { 0x4822c80f, SUC_lexicon_tags_1 },
    { 0x4142c810, SUC_lexicon_tags_55 },
    { 0xdde2c811, SUC_lexicon_tags_7 },
    { 0x6b62c812, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf622c816, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe6a2c818, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7642c81b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e42c827, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3922c829, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb02c830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9362c836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b02c838, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9862c83c, SUC_lexicon_tags_37 },
    { 0xd5e2c83c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c2c841, SUC_lexicon_tags_11 },
    { 0x3de2c841, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c2c850, SUC_lexicon_tags_85 },
    { 0xa7c2c850, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2882c853, SUC_lexicon_tags_0 },
    { 0x13a2c853, SUC_lexicon_tags_9 },
    { 0xbc02c855, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9742c85e, SUC_lexicon_tags_15 },
    { 0xfa82c85f, SUC_lexicon_tags_23 },
    { 0x64a2c85e, SUC_lexicon_tags_13 },
    { 0x2142c85e, SUC_lexicon_tags_37 },
    { 0x3d82c862, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3702c867, SUC_lexicon_tags_17 },
    { 0x9a62c868, SUC_lexicon_tags_66 },
    { 0x1ba2c868, SUC_lexicon_tags_30 },
    { 0xd5a2c869, SUC_lexicon_tags_47 },
    { 0xf3e2c86b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e2c86e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b42c872, SUC_lexicon_tags_33 },
    { 0x6a62c873, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc2c879, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe22c87f, SUC_lexicon_tags_65 },
    { 0x98c2c880, SUC_lexicon_tags_17 },
    { 0xfa2c87f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb002c884, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b82c887, SUC_lexicon_tags_17 },
    { 0x7ec2c888, SUC_lexicon_tags_46 },
    { 0x5f22c887, SUC_lexicon_tags_33 },
    { 0x7162c889, SUC_lexicon_tags_11 },
    { 0x9ec2c88b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa302c891, SUC_lexicon_tags_11 },
    { 0x9902c891, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0a2c894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e2c898, SUC_lexicon_tags_14 },
    { 0x90a2c899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63e2c89b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e2c89f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b02c8a2, SUC_lexicon_tags_79 },
    { 0xcb22c8a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde02c8a8, SUC_lexicon_tags_37 },
    { 0xd782c8a9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x79a2c8ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf62c8b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb82c8b7, SUC_lexicon_tags_60 },
    { 0xeba2c8b8, SUC_lexicon_tags_38 },
    { 0x35a2c8b8, SUC_lexicon_tags_66 },
    { 0xc582c8ba, SUC_lexicon_tags_37 },
    { 0x45c2c8bb, SUC_lexicon_tags_63 },
    { 0xb722c8bb, SUC_lexicon_tags_31 },
    { 0x4242c8bd, SUC_lexicon_tags_68 },
    { 0xcc02c8bd, SUC_lexicon_tags_65 },
    { 0xee22c8bb, SUC_lexicon_tags_6 },
    { 0x55e2c8c0, SUC_lexicon_tags_68 },
    { 0x2242c8c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8d62c8c3, SUC_lexicon_tags_2 },
    { 0x10e2c8c4, SUC_lexicon_tags_23 },
    { 0x482c8c5, SUC_lexicon_tags_23 },
    { 0x1622c8c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd402c8c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae2c8d1, SUC_lexicon_tags_43 },
    { 0xf042c8d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc8c2c8d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2722c8d9, SUC_lexicon_tags_14 },
    { 0x9a62c8da, SUC_lexicon_tags_23 },
    { 0x7ca2c8d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4762c8dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x482c8e0, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a2c8e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x18c2c8e5, SUC_lexicon_tags_28 },
    { 0x1de2c8e6, SUC_lexicon_tags_148 },
    { 0x4882c8e6, SUC_lexicon_tags_11 },
    { 0xd2a2c8e8, SUC_lexicon_tags_17 },
    { 0x17e2c8e7, SUC_lexicon_tags_8 },
    { 0xdfc2c8ea, SUC_lexicon_tags_26 },
    { 0x7ea2c8e7, SUC_lexicon_tags_6 },
    { 0x9142c8e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac2c8ef, SUC_lexicon_tags_37 },
    { 0x3202c8f0, SUC_lexicon_tags_45 },
    { 0x88c2c8f1, SUC_lexicon_tags_23 },
    { 0x13a2c8f2, SUC_lexicon_tags_14 },
    { 0x102c8f1, SUC_lexicon_tags_21 },
    { 0xbf62c8f4, SUC_lexicon_tags_2 },
    { 0x2022c8f5, SUC_lexicon_tags_26 },
    { 0x70c2c8f3, SUC_lexicon_tags_65 },
    { 0xfe62c8f7, SUC_lexicon_tags_11 },
    { 0xc442c8f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c22c901, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4482c904, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xba62c906, SUC_lexicon_tags_17 },
    { 0x7d42c907, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x6222c90a, SUC_lexicon_tags_1 },
    { 0x6062c90b, SUC_lexicon_tags_31 },
    { 0xc3c2c90c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe722c910, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3202c912, SUC_lexicon_tags_37 },
    { 0x6842c913, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd362c917, SUC_lexicon_tags_1 },
    { 0xfde2c917, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8bc2c91a, SUC_lexicon_tags_85 },
    { 0xf1a2c91a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf422c91d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1ce2c91f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8c22c921, SUC_lexicon_tags_14 },
    { 0xc282c922, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e2c927, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a2c92a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8282c92d, SUC_lexicon_tags_10 },
    { 0xde22c92e, SUC_lexicon_tags_37 },
    { 0xd3e2c92e, SUC_lexicon_tags_37 },
    { 0x5ce2c92f, SUC_lexicon_tags_37 },
    { 0x3a02c92f, SUC_lexicon_tags_28 },
    { 0x66c2c92e, SUC_lexicon_tags_9 },
    { 0xa302c92d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd42c93a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a2c942, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9322c944, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42c94a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c94d, SUC_lexicon_tags_17 },
    { 0x94e2c94e, SUC_lexicon_tags_56 },
    { 0x8dc2c94e, SUC_lexicon_tags_1 },
    { 0x402c950, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2902c954, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd102c957, SUC_lexicon_tags_23 },
    { 0x2fe2c958, SUC_lexicon_tags_48 },
    { 0xb042c959, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa162c95b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4062c960, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff22c962, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c2c965, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8242c969, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d02c96c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5362c96e, SUC_lexicon_tags_1 },
    { 0x4ea2c96e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xce22c972, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e2c975, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83e2c977, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb122c97d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a02c980, SUC_lexicon_tags_17 },
    { 0x5342c980, SUC_lexicon_tags_1 },
    { 0x95a2c980, SUC_lexicon_tags_32 },
    { 0x6f02c981, SUC_lexicon_tags_8 },
    { 0xc0a2c981, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe682c987, SUC_lexicon_tags_45 },
    { 0x7d02c988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4982c98c, SUC_lexicon_tags_10 },
    { 0xfdc2c98d, SUC_lexicon_tags_45 },
    { 0xd4a2c98e, SUC_lexicon_tags_28 },
    { 0x9c62c98f, SUC_lexicon_tags_14 },
    { 0x4f02c98f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9382c996, SUC_lexicon_tags_39 },
    { 0xd342c997, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6462c99a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2382c99f, SUC_lexicon_tags_2 },
    { 0x6a2c9a0, SUC_lexicon_tags_25 },
    { 0xe7a2c9a0, SUC_lexicon_tags_2 },
    { 0xa942c9a2, SUC_lexicon_tags_7 },
    { 0xd6e2c9a1, SUC_lexicon_tags_39 },
    { 0xaa42c9a2, SUC_lexicon_tags_31 },
    { 0xe862c9a5, SUC_lexicon_tags_11 },
    { 0xf7c2c9a6, SUC_lexicon_tags_24 },
    { 0x2522c9a4, SUC_lexicon_tags_25 },
    { 0x8022c9a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb462c9aa, SUC_lexicon_tags_29 },
    { 0xd382c9ab, SUC_lexicon_tags_14 },
    { 0xe702c9aa, SUC_lexicon_tags_4 },
    { 0xbd22c9aa, SUC_lexicon_tags_9 },
    { 0x7822c9ae, SUC_lexicon_tags_65 },
    { 0xad62c9ad, SUC_lexicon_tags_45 },
    { 0x4362c9b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce2c9b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xadc2c9b5, SUC_lexicon_tags_8 },
    { 0x2d02c9b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafe2c9b8, SUC_lexicon_tags_26 },
    { 0xb2e2c9b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6822c9bc, SUC_lexicon_tags_17 },
    { 0x1242c9bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77c2c9bf, SUC_lexicon_tags_15 },
    { 0x9a42c9bf, SUC_lexicon_tags_17 },
    { 0x2302c9c1, SUC_lexicon_tags_33 },
    { 0x88a2c9c1, SUC_lexicon_tags_21 },
    { 0xbb82c9c3, SUC_lexicon_tags_11 },
    { 0x97a2c9c0, SUC_lexicon_tags_37 },
    { 0x9ce2c9c5, SUC_lexicon_tags_7 },
    { 0x5f62c9c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd62c9cf, SUC_lexicon_tags_27 },
    { 0xae02c9d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e82c9d8, SUC_lexicon_tags_37 },
    { 0x3a62c9d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa882c9db, SUC_lexicon_tags_14 },
    { 0xf982c9db, SUC_lexicon_tags_17 },
    { 0x5122c9dd, SUC_lexicon_tags_37 },
    { 0x7722c9db, SUC_lexicon_tags_7 },
    { 0x6a2c9dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e2c9e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c02c9e6, SUC_lexicon_tags_18 },
    { 0x3942c9e6, SUC_lexicon_tags_66 },
    { 0x8962c9e8, SUC_lexicon_tags_198 },
    { 0x95c2c9e9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xec82c9ec, SUC_lexicon_tags_47 },
    { 0x26c2c9ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb502c9ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e2c9f2, SUC_lexicon_tags_37 },
    { 0xc3a2c9f3, SUC_lexicon_tags_15 },
    { 0xe402c9f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x95a2c9f6, SUC_lexicon_tags_1 },
    { 0x3662c9f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa962c9fa, SUC_lexicon_tags_19 },
    { 0x1862c9fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xede2c9fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c2ca02, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8662ca04, SUC_lexicon_tags_25 },
    { 0x25c2ca04, SUC_lexicon_tags_19 },
    { 0x1e22ca06, SUC_lexicon_tags_63 },
    { 0x3182ca06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c62ca0a, SUC_lexicon_tags_32 },
    { 0x3b02ca0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde2ca0e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x782ca10, SUC_lexicon_tags_25 },
    { 0x3282ca11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c2ca19, SUC_lexicon_tags_37 },
    { 0xd202ca1a, SUC_lexicon_tags_17 },
    { 0x9f82ca1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a42ca1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x31a2ca1f, SUC_lexicon_tags_12 },
    { 0x4a82ca1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a2ca23, SUC_lexicon_tags_11 },
    { 0x52c2ca23, SUC_lexicon_tags_37 },
    { 0xc9a2ca25, SUC_lexicon_tags_25 },
    { 0xbba2ca24, SUC_lexicon_tags_17 },
    { 0xe422ca27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c62ca2a, SUC_lexicon_tags_7 },
    { 0x66c2ca2a, SUC_lexicon_tags_17 },
    { 0xa3e2ca2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca2ca31, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfca2ca33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ac2ca35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7582ca37, SUC_lexicon_tags_2 },
    { 0xd082ca37, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac02ca3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcc2ca3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e2ca41, SUC_lexicon_tags_27 },
    { 0x59a2ca41, SUC_lexicon_tags_39 },
    { 0x2a2ca43, SUC_lexicon_tags_17 },
    { 0x18a2ca44, SUC_lexicon_tags_1 },
    { 0x2f02ca43, SUC_lexicon_tags_12 },
    { 0xdbc2ca46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c2ca49, SUC_lexicon_tags_9 },
    { 0x13c2ca4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a2ca4f, SUC_lexicon_tags_39 },
    { 0x4382ca50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a2ca53, SUC_lexicon_tags_30 },
    { 0xc622ca54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa142ca57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a2ca5a, SUC_lexicon_tags_2 },
    { 0x4282ca5b, SUC_lexicon_tags_148 },
    { 0xa9a2ca5c, SUC_lexicon_tags_37 },
    { 0x77e2ca5c, SUC_lexicon_tags_21 },
    { 0x7a42ca5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b02ca62, SUC_lexicon_tags_12 },
    { 0x73c2ca63, SUC_lexicon_tags_37 },
    { 0x7c62ca64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb702ca67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c2ca6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac42ca6c, SUC_lexicon_tags_3 },
    { 0x51a2ca6d, SUC_lexicon_tags_2 },
    { 0xd682ca6c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca2ca71, SUC_lexicon_tags_2 },
    { 0xfc42ca72, SUC_lexicon_tags_1 },
    { 0xf0c2ca72, SUC_lexicon_tags_32 },
    { 0x28e2ca74, SUC_lexicon_tags_13 },
    { 0x6602ca74, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6702ca7c, SUC_lexicon_tags_3 },
    { 0x7782ca7c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x59a2ca7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682ca85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bc2ca87, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc02ca8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b62ca8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf62ca90, SUC_lexicon_tags_48 },
    { 0x3502ca91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e2ca99, SUC_lexicon_tags_47 },
    { 0x9ac2ca9a, SUC_lexicon_tags_17 },
    { 0xc882ca9b, SUC_lexicon_tags_7 },
    { 0x2002ca99, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3f62ca9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc42caa1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b02caa5, SUC_lexicon_tags_68 },
    { 0xdde2caa5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e2caaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe342cab1, SUC_lexicon_tags_26 },
    { 0xe042cab1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c62cabb, SUC_lexicon_tags_15 },
    { 0x7a42cabc, SUC_lexicon_tags_23 },
    { 0xe462cabc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1962cabf, SUC_lexicon_tags_43 },
    { 0xc722cac0, SUC_lexicon_tags_17 },
    { 0x9702cac1, SUC_lexicon_tags_12 },
    { 0x4d82cac2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e2cac7, SUC_lexicon_tags_2 },
    { 0x9082cac8, SUC_lexicon_tags_27 },
    { 0xcf02cac9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e2cacc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e2cad0, SUC_lexicon_tags_26 },
    { 0x21c2cad1, SUC_lexicon_tags_11 },
    { 0x60a2cad2, SUC_lexicon_tags_23 },
    { 0x802cad2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8742cad7, SUC_lexicon_tags_49 },
    { 0x80e2cad8, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de2cadb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae2cade, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5962cae3, SUC_lexicon_tags_11 },
    { 0xf682cae4, SUC_lexicon_tags_14 },
    { 0xe722cae5, SUC_lexicon_tags_27 },
    { 0x4c22cae5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9da2cae8, SUC_lexicon_tags_7 },
    { 0x5f22cae9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf162caf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72a2caf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8c2caf4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5282caf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf82cafb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb1e2cafd, SUC_lexicon_tags_43 },
    { 0x1aa2cafd, SUC_lexicon_tags_37 },
    { 0x3e22caff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c2cb02, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfa22cb04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e02cb07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd62cb09, SUC_lexicon_tags_282 },
    { 0x5102cb09, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9c42cb0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1362cb10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8982cb12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6802cb17, SUC_lexicon_tags_56 },
    { 0x31c2cb18, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x102cb1a, SUC_lexicon_tags_39 },
    { 0xed82cb1b, SUC_lexicon_tags_31 },
    { 0x7662cb1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc022cb21, SUC_lexicon_tags_47 },
    { 0x7a42cb22, SUC_lexicon_tags_3 },
    { 0xf7a2cb22, SUC_lexicon_tags_151 },
    { 0x6a22cb24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9122cb29, SUC_lexicon_tags_30 },
    { 0x1862cb2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9882cb2c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xbb22cb2e, SUC_lexicon_tags_63 },
    { 0x5bc2cb2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e42cb31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c2cb35, SUC_lexicon_tags_37 },
    { 0x87c2cb35, SUC_lexicon_tags_45 },
    { 0x9522cb37, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa2cb3b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1962cb3f, SUC_lexicon_tags_2 },
    { 0x14a2cb3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a02cb43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9862cb45, SUC_lexicon_tags_26 },
    { 0x18c2cb45, SUC_lexicon_tags_2 },
    { 0xe762cb47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe502cb4b, SUC_lexicon_tags_15 },
    { 0x5ce2cb4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa302cb4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a2cb53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c2cb56, SUC_lexicon_tags_39 },
    { 0xe0e2cb57, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5622cb5c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4c02cb5e, SUC_lexicon_tags_43 },
    { 0xa762cb5f, SUC_lexicon_tags_26 },
    { 0xcf22cb5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb922cb62, SUC_lexicon_tags_38 },
    { 0xd02cb62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf862cb65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ae2cb67, SUC_lexicon_tags_6 },
    { 0x4062cb67, SUC_lexicon_tags_19 },
    { 0xe662cb67, SUC_lexicon_tags_37 },
    { 0x25c2cb6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe642cb74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6162cb77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ba2cb79, SUC_lexicon_tags_25 },
    { 0x5ca2cb7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a2cb81, SUC_lexicon_tags_38 },
    { 0x4902cb81, SUC_lexicon_tags_17 },
    { 0x6c82cb82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa082cb86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2cb8a, SUC_lexicon_tags_11 },
    { 0x6a62cb8a, SUC_lexicon_tags_23 },
    { 0xf622cb8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec62cb95, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3cc2cb97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e2cb9a, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0x182cb9c, SUC_lexicon_tags_63 },
    { 0x5762cb9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9c2cb9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd142cba2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f82cba6, SUC_lexicon_tags_1 },
    { 0x6982cba7, SUC_lexicon_tags_61 },
    { 0x7ba2cba8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e2cbad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2782cbb4, SUC_lexicon_tags_25 },
    { 0xd222cbb5, SUC_lexicon_tags_38 },
    { 0x3cc2cbb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f02cbba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb762cbbc, SUC_lexicon_tags_12 },
    { 0x8ee2cbbc, SUC_lexicon_tags_318 },
    { 0x5322cbbc, SUC_lexicon_tags_7 },
    { 0x7462cbbe, SUC_lexicon_tags_25 },
    { 0xd342cbc0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xff62cbc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68e2cbc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac42cbcb, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xa22cbcd, SUC_lexicon_tags_7 },
    { 0x6062cbcd, SUC_lexicon_tags_43 },
    { 0x23c2cbcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e02cbd3, SUC_lexicon_tags_1 },
    { 0xec42cbd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c2cbda, SUC_lexicon_tags_12 },
    { 0x7302cbdb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec2cbe2, SUC_lexicon_tags_25 },
    { 0xd622cbe2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d42cbe7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5122cbeb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc742cbf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca2cbf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa122cbf8, SUC_lexicon_tags_1 },
    { 0x9d02cbf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c2cbfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa2cbff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x402cc02, SUC_lexicon_tags_0 },
    { 0x13a2cc03, SUC_lexicon_tags_17 },
    { 0xcf02cc04, SUC_lexicon_tags_3 },
    { 0x8482cc04, SUC_lexicon_tags_30 },
    { 0x3542cc06, SUC_lexicon_tags_29 },
    { 0x95c2cc06, SUC_lexicon_tags_6 },
    { 0x1802cc07, SUC_lexicon_tags_16 },
    { 0x80a2cc08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e2cc0d, SUC_lexicon_tags_65 },
    { 0x45c2cc0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6662cc11, SUC_lexicon_tags_7 },
    { 0xec02cc12, SUC_lexicon_tags_25 },
    { 0x5bc2cc11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3202cc15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fa2cc17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb022cc1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c2cc22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac2cc26, SUC_lexicon_tags_15 },
    { 0x4be2cc26, SUC_lexicon_tags_32 },
    { 0xfc22cc28, SUC_lexicon_tags_45 },
    { 0x5b02cc29, SUC_lexicon_tags_37 },
    { 0xb3e2cc27, SUC_lexicon_tags_6 },
    { 0x2b62cc26, SUC_lexicon_tags_37 },
    { 0x3f42cc2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb882cc31, SUC_lexicon_tags_1201 },
    { 0x1862cc32, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xf3a2cc34, SUC_lexicon_tags_8 },
    { 0xe9c2cc35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1a2cc37, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a2cc3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a02cc40, SUC_lexicon_tags_43 },
    { 0x2e42cc41, SUC_lexicon_tags_33 },
    { 0x5e42cc42, SUC_lexicon_tags_3 },
    { 0x96a2cc43, SUC_lexicon_tags_25 },
    { 0xcf82cc44, SUC_lexicon_tags_38 },
    { 0xee02cc42, SUC_lexicon_tags_15 },
    { 0x3c22cc44, SUC_lexicon_tags_17 },
    { 0xd582cc43, SUC_lexicon_tags_63 },
    { 0x9462cc42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae2cc4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22a2cc4d, SUC_lexicon_tags_7 },
    { 0x8462cc4d, SUC_lexicon_tags_25 },
    { 0x5462cc4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e2cc54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe1a2cc56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd702cc5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc782cc5f, SUC_lexicon_tags_30 },
    { 0xf5c2cc60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3882cc62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e2cc70, SUC_lexicon_tags_28 },
    { 0xa242cc71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc2cc75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a2cc77, SUC_lexicon_tags_26 },
    { 0x2622cc78, SUC_lexicon_tags_32 },
    { 0x8f62cc77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e2cc80, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d02cc85, SUC_lexicon_tags_27 },
    { 0x1822cc86, SUC_lexicon_tags_25 },
    { 0x12a2cc87, SUC_lexicon_tags_27 },
    { 0x2262cc85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d82cc8a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8082cc90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3e42cc92, SUC_lexicon_tags_11 },
    { 0x1ac2cc92, SUC_lexicon_tags_37 },
    { 0xfe42cc93, SUC_lexicon_tags_136 },
    { 0xfe22cc93, SUC_lexicon_tags_11 },
    { 0xcf62cc93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5362cc9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a2cc9f, SUC_lexicon_tags_2 },
    { 0xbce2cc9f, SUC_lexicon_tags_7 },
    { 0xefa2cca1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x742cca4, SUC_lexicon_tags_0 },
    { 0x3e82cca5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de2cca8, SUC_lexicon_tags_25 },
    { 0xd062cca9, SUC_lexicon_tags_0 },
    { 0x6262cca8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5682ccb1, SUC_lexicon_tags_11 },
    { 0x1242ccb2, SUC_lexicon_tags_22 },
    { 0xf4e2ccb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e2ccb6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac2ccba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe542ccbd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbd02ccbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d42ccc1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1462ccc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e2cccc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c42ccce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ba2ccd0, SUC_lexicon_tags_16 },
    { 0xf5a2ccd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e2ccd6, SUC_lexicon_tags_66 },
    { 0xde42ccd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20c2ccd9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa2ccdf, SUC_lexicon_tags_25 },
    { 0xf562cce0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdea2cce2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e2cce7, SUC_lexicon_tags_1 },
    { 0x43a2cce7, SUC_lexicon_tags_34 },
    { 0x6102cce7, SUC_lexicon_tags_14 },
    { 0x28e2ccea, SUC_lexicon_tags_26 },
    { 0x7322cceb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb382ccee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e82ccf3, SUC_lexicon_tags_60 },
    { 0x4342ccf4, SUC_lexicon_tags_23 },
    { 0x4702ccf5, SUC_lexicon_tags_30 },
    { 0xbde2ccf6, SUC_lexicon_tags_12 },
    { 0x36e2ccf3, SUC_lexicon_tags_7 },
    { 0x7982ccf8, SUC_lexicon_tags_29 },
    { 0x5e22ccf5, SUC_lexicon_tags_7 },
    { 0x5a62ccf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a2ccfc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a22ccff, SUC_lexicon_tags_234 },
    { 0x5682cd00, SUC_lexicon_tags_6 },
    { 0x4622ccff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x77c2cd03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6702cd05, SUC_lexicon_tags_87 },
    { 0xf1c2cd05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a62cd09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c02cd0d, SUC_lexicon_tags_45 },
    { 0xf722cd0d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e2cd12, SUC_lexicon_tags_27 },
    { 0xd342cd13, SUC_lexicon_tags_37 },
    { 0x5ea2cd14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ce2cd16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b02cd19, SUC_lexicon_tags_6 },
    { 0x4be2cd1a, SUC_lexicon_tags_45 },
    { 0xabc2cd1a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a2cd20, SUC_lexicon_tags_15 },
    { 0xbae2cd20, SUC_lexicon_tags_7 },
    { 0x5342cd22, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c2cd26, SUC_lexicon_tags_25 },
    { 0x3ce2cd27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4922cd29, SUC_lexicon_tags_11 },
    { 0x9b42cd29, SUC_lexicon_tags_25 },
    { 0x4762cd2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c2cd35, SUC_lexicon_tags_37 },
    { 0x9f82cd36, SUC_lexicon_tags_27 },
    { 0x5742cd37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3622cd39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78c2cd3b, SUC_lexicon_tags_47 },
    { 0x9ac2cd3b, SUC_lexicon_tags_37 },
    { 0xb182cd3d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8682cd3f, SUC_lexicon_tags_48 },
    { 0xf782cd40, SUC_lexicon_tags_69 },
    { 0x46e2cd41, SUC_lexicon_tags_7 },
    { 0x1b02cd42, SUC_lexicon_tags_64 },
    { 0x2fa2cd43, SUC_lexicon_tags_2 },
    { 0x2a42cd44, SUC_lexicon_tags_12 },
    { 0x24c2cd44, SUC_lexicon_tags_45 },
    { 0x7682cd44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b62cd49, SUC_lexicon_tags_0 },
    { 0x54c2cd49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9802cd51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9342cd54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1082cd57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2802cd5a, SUC_lexicon_tags_15 },
    { 0x82e2cd5b, SUC_lexicon_tags_116 },
    { 0x7762cd5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb922cd5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a2cd65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd862cd67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3402cd69, SUC_lexicon_tags_35 },
    { 0xdd22cd69, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0c2cd6c, SUC_lexicon_tags_17 },
    { 0x66e2cd6c, SUC_lexicon_tags_63 },
    { 0xf882cd6d, SUC_lexicon_tags_14 },
    { 0x5782cd6f, SUC_lexicon_tags_7 },
    { 0xf982cd6e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c2cd73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2242cd79, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdb02cd7b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3a2cd7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e2cd82, SUC_lexicon_tags_11 },
    { 0x7a42cd83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17c2cd85, SUC_lexicon_tags_17 },
    { 0x4222cd86, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe2cd89, SUC_lexicon_tags_11 },
    { 0x5662cd8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a42cd8d, SUC_lexicon_tags_30 },
    { 0x9b02cd8e, SUC_lexicon_tags_38 },
    { 0x9c22cd8e, SUC_lexicon_tags_32 },
    { 0x16e2cd90, SUC_lexicon_tags_25 },
    { 0x3a2cd90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb862cd9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76e2cd9f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3842cda5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6882cdad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd782cdb0, SUC_lexicon_tags_11 },
    { 0xb642cdb0, SUC_lexicon_tags_7 },
    { 0xf0a2cdb2, SUC_lexicon_tags_18 },
    { 0xef22cdb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9382cdba, SUC_lexicon_tags_37 },
    { 0x742cdbb, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac2cdbf, SUC_lexicon_tags_25 },
    { 0xcba2cdc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f42cdc3, SUC_lexicon_tags_37 },
    { 0xc5e2cdc4, SUC_lexicon_tags_33 },
    { 0x8c42cdc4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1dc2cdc7, SUC_lexicon_tags_7 },
    { 0x5e2cdc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd222cdcb, SUC_lexicon_tags_21 },
    { 0xcde2cdcb, SUC_lexicon_tags_7 },
    { 0xd7a2cdcc, SUC_lexicon_tags_15 },
    { 0x1ce2cdcd, SUC_lexicon_tags_45 },
    { 0xefa2cdcb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa122cdd3, SUC_lexicon_tags_55 },
    { 0xeee2cdd4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7982cdd9, SUC_lexicon_tags_25 },
    { 0x5e82cdda, SUC_lexicon_tags_47 },
    { 0x2a2cddb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6562cde1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd62cde4, SUC_lexicon_tags_53 },
    { 0x8da2cde5, SUC_lexicon_tags_40 },
    { 0x6602cde5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a2cde9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c2cdf8, SUC_lexicon_tags_37 },
    { 0x4f62cdf9, SUC_lexicon_tags_33 },
    { 0x1c82cdf9, SUC_lexicon_tags_2 },
    { 0x6982cdfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e82cdfe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c22ce03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b62ce07, SUC_lexicon_tags_10 },
    { 0x48e2ce07, SUC_lexicon_tags_56 },
    { 0x3a02ce09, SUC_lexicon_tags_14 },
    { 0x6122ce09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa442ce0f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9982ce11, SUC_lexicon_tags_19 },
    { 0x7fc2ce11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6282ce14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5de2ce16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc222ce19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e82ce1e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe622ce22, SUC_lexicon_tags_68 },
    { 0x55e2ce23, SUC_lexicon_tags_17 },
    { 0x8122ce23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x382ce26, SUC_lexicon_tags_37 },
    { 0x2a62ce26, SUC_lexicon_tags_17 },
    { 0x6642ce28, SUC_lexicon_tags_25 },
    { 0x22c2ce29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7262ce2b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde82ce30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe02ce33, SUC_lexicon_tags_37 },
    { 0x76e2ce33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2882ce38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea82ce3d, SUC_lexicon_tags_25 },
    { 0xf662ce3e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae2ce42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5882ce46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4222ce48, SUC_lexicon_tags_37 },
    { 0x68e2ce49, SUC_lexicon_tags_6 },
    { 0xa4e2ce4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ae2ce4c, SUC_lexicon_tags_23 },
    { 0xea22ce4c, SUC_lexicon_tags_9 },
    { 0xca42ce4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b02ce50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18e2ce52, SUC_lexicon_tags_170 },
    { 0x1582ce52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x822ce55, SUC_lexicon_tags_77 },
    { 0xe2a2ce56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8022ce58, SUC_lexicon_tags_15 },
    { 0x48c2ce59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf302ce5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf42ce60, SUC_lexicon_tags_65 },
    { 0xdb82ce61, SUC_lexicon_tags_15 },
    { 0xd2e2ce61, SUC_lexicon_tags_60 },
    { 0xee82ce61, SUC_lexicon_tags_82 },
    { 0xc282ce63, SUC_lexicon_tags_49 },
    { 0xf482ce62, SUC_lexicon_tags_1 },
    { 0x8322ce65, SUC_lexicon_tags_1 },
    { 0x3a2ce66, SUC_lexicon_tags_1305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c2ce6c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff02ce72, SUC_lexicon_tags_7 },
    { 0x2aa2ce73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5102ce78, SUC_lexicon_tags_11 },
    { 0xda22ce78, SUC_lexicon_tags_37 },
    { 0x5542ce7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4822ce81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2502ce85, SUC_lexicon_tags_38 },
    { 0x49a2ce86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a2ce8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca2ce90, SUC_lexicon_tags_40 },
    { 0x8ac2ce90, SUC_lexicon_tags_39 },
    { 0xd602ce92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e22ce96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf542ce99, SUC_lexicon_tags_0 },
    { 0xdd42ce9a, SUC_lexicon_tags_1 },
    { 0xe9e2ce9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e2ce9d, SUC_lexicon_tags_17 },
    { 0x66c2ce9e, SUC_lexicon_tags_68 },
    { 0xede2ce9f, SUC_lexicon_tags_84 },
    { 0x9ae2cea0, SUC_lexicon_tags_37 },
    { 0xe5a2cea1, SUC_lexicon_tags_88 },
    { 0x8762cea2, SUC_lexicon_tags_0 },
    { 0x52a2cea2, SUC_lexicon_tags_38 },
    { 0xf7a2ce9e, SUC_lexicon_tags_11 },
    { 0xc5c2cea5, SUC_lexicon_tags_25 },
    { 0x40c2cea6, SUC_lexicon_tags_61 },
    { 0x6402cea7, SUC_lexicon_tags_15 },
    { 0x7c22cea8, SUC_lexicon_tags_29 },
    { 0x6782cea9, SUC_lexicon_tags_10 },
    { 0x69e2cea9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa2cead, SUC_lexicon_tags_14 },
    { 0xd902ceae, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5f02ceb0, SUC_lexicon_tags_15 },
    { 0xa902ceb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8582ceb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90a2ceb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6402ceb8, SUC_lexicon_tags_25 },
    { 0xfa62ceb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf662cebb, SUC_lexicon_tags_57 },
    { 0xe62cebc, SUC_lexicon_tags_43 },
    { 0x1642cebc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x59c2cebf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2c82cec1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xe542cec4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b62cec7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6542ceca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf62cecd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb02ced0, SUC_lexicon_tags_30 },
    { 0x37e2ced1, SUC_lexicon_tags_19 },
    { 0x4f42ced2, SUC_lexicon_tags_17 },
    { 0x2ea2ced3, SUC_lexicon_tags_65 },
    { 0xe0a2ced3, SUC_lexicon_tags_7 },
    { 0xbf02ced0, SUC_lexicon_tags_37 },
    { 0x4b42ced6, SUC_lexicon_tags_69 },
    { 0x4262ced7, SUC_lexicon_tags_68 },
    { 0x95a2ced3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be2cee1, SUC_lexicon_tags_12 },
    { 0xee82cee1, SUC_lexicon_tags_38 },
    { 0x1702cee1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb962cee9, SUC_lexicon_tags_61 },
    { 0x3542cee9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3662ceec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c2cef5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbe62cef7, SUC_lexicon_tags_13 },
    { 0x5b02cef8, SUC_lexicon_tags_11 },
    { 0x1f02cef8, SUC_lexicon_tags_26 },
    { 0xd042cefa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7842cf01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742cf0b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xbf22cf0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae2cf12, SUC_lexicon_tags_25 },
    { 0xa702cf13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x59a2cf15, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc2cf24, SUC_lexicon_tags_80 },
    { 0xf282cf25, SUC_lexicon_tags_29 },
    { 0x3e22cf24, SUC_lexicon_tags_12 },
    { 0x6da2cf24, SUC_lexicon_tags_15 },
    { 0x20e2cf24, SUC_lexicon_tags_25 },
    { 0xffe2cf27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a82cf2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8c2cf2e, SUC_lexicon_tags_26 },
    { 0x9c02cf2e, SUC_lexicon_tags_37 },
    { 0x64a2cf2f, SUC_lexicon_tags_38 },
    { 0x7f62cf2e, SUC_lexicon_tags_0 },
    { 0x422cf30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1142cf35, SUC_lexicon_tags_15 },
    { 0xcb62cf36, SUC_lexicon_tags_15 },
    { 0xd362cf37, SUC_lexicon_tags_37 },
    { 0x5c22cf38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7882cf3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa2cf3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e2cf42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f62cf44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e2cf47, SUC_lexicon_tags_19 },
    { 0x7ba2cf48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d82cf4f, SUC_lexicon_tags_98 },
    { 0xfe62cf50, SUC_lexicon_tags_46 },
    { 0x6582cf51, SUC_lexicon_tags_17 },
    { 0x4de2cf4f, SUC_lexicon_tags_14 },
    { 0x34e2cf4f, SUC_lexicon_tags_12 },
    { 0xfd42cf54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd282cf58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa142cf5b, SUC_lexicon_tags_27 },
    { 0x9482cf5c, SUC_lexicon_tags_7 },
    { 0xaca2cf5d, SUC_lexicon_tags_28 },
    { 0x4c02cf5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e2cf63, SUC_lexicon_tags_25 },
    { 0xee62cf64, SUC_lexicon_tags_17 },
    { 0xe022cf65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf82cf67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6542cf6a, SUC_lexicon_tags_51 },
    { 0x6342cf6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1542cf6f, SUC_lexicon_tags_175 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e2cf76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c2cf79, SUC_lexicon_tags_74 },
    { 0x8fa2cf7a, SUC_lexicon_tags_2 },
    { 0x2422cf7b, SUC_lexicon_tags_63 },
    { 0x1ce2cf79, SUC_lexicon_tags_23 },
    { 0x9c62cf7d, SUC_lexicon_tags_49 },
    { 0xdd42cf79, SUC_lexicon_tags_37 },
    { 0x7562cf7d, SUC_lexicon_tags_45 },
    { 0xbe82cf80, SUC_lexicon_tags_7 },
    { 0x48a2cf81, SUC_lexicon_tags_9 },
    { 0x3a82cf80, SUC_lexicon_tags_27 },
    { 0x3062cf83, SUC_lexicon_tags_39 },
    { 0x522cf84, SUC_lexicon_tags_17 },
    { 0x94a2cf85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a2cf88, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x922cf8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c02cf8c, SUC_lexicon_tags_0 },
    { 0xb0c2cf8d, SUC_lexicon_tags_8 },
    { 0xa962cf8d, SUC_lexicon_tags_11 },
    { 0xf762cf8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf902cf96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd562cf98, SUC_lexicon_tags_1 },
    { 0x8622cf99, SUC_lexicon_tags_37 },
    { 0x7202cf99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a22cf9d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8082cfa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e2cfa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7942cfa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6982cfab, SUC_lexicon_tags_37 },
    { 0x6122cfac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e2cfb1, SUC_lexicon_tags_12 },
    { 0xbc02cfb2, SUC_lexicon_tags_7 },
    { 0xb02cfb1, SUC_lexicon_tags_2 },
    { 0xf42cfb4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8162cfb6, SUC_lexicon_tags_1 },
    { 0x5ca2cfb7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd262cfbb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6182cfc3, SUC_lexicon_tags_11 },
    { 0x56e2cfc4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xef42cfc7, SUC_lexicon_tags_18 },
    { 0xd022cfc8, SUC_lexicon_tags_158 },
    { 0x5d82cfc8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a2cfcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce02cfcf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e2cfd4, SUC_lexicon_tags_30 },
    { 0x9fa2cfd4, SUC_lexicon_tags_8 },
    { 0x8d62cfd6, SUC_lexicon_tags_15 },
    { 0x1002cfd7, SUC_lexicon_tags_11 },
    { 0x20c2cfd8, SUC_lexicon_tags_47 },
    { 0x8c82cfd9, SUC_lexicon_tags_17 },
    { 0xeb02cfda, SUC_lexicon_tags_25 },
    { 0x4cc2cfdb, SUC_lexicon_tags_2 },
    { 0x5e02cfd9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6302cfde, SUC_lexicon_tags_30 },
    { 0x3422cfdf, SUC_lexicon_tags_26 },
    { 0x7ae2cfe0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x262cfe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1002cfe4, SUC_lexicon_tags_555 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42cfea, SUC_lexicon_tags_15 },
    { 0x2622cfeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c2cfee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf062cff9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3802cffc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3342cfff, SUC_lexicon_tags_3 },
    { 0x24a2d000, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf162d007, SUC_lexicon_tags_151 },
    { 0x6802d008, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122d00d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8042d019, SUC_lexicon_tags_45 },
    { 0x2322d01a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782d020, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xefc2d022, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2e22d024, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x482d026, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31e2d028, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6562d02c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a2d02e, SUC_lexicon_tags_6 },
    { 0x1842d02f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f22d034, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c2d037, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d22d03a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c22d03d, SUC_lexicon_tags_25 },
    { 0xd8e2d03e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1182d040, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c2d043, SUC_lexicon_tags_31 },
    { 0x2342d044, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2d049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5442d050, SUC_lexicon_tags_100 },
    { 0xb702d051, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d02d054, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa982d056, SUC_lexicon_tags_7 },
    { 0x6cc2d056, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb002d059, SUC_lexicon_tags_26 },
    { 0xdac2d059, SUC_lexicon_tags_63 },
    { 0xb222d059, SUC_lexicon_tags_7 },
    { 0x14e2d05c, SUC_lexicon_tags_34 },
    { 0x8d22d05a, SUC_lexicon_tags_12 },
    { 0x5d62d05e, SUC_lexicon_tags_1 },
    { 0xcd42d05f, SUC_lexicon_tags_55 },
    { 0xba82d060, SUC_lexicon_tags_29 },
    { 0x7e82d061, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba02d066, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d82d06b, SUC_lexicon_tags_7 },
    { 0xdba2d06c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ea2d06e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c2d071, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562d076, SUC_lexicon_tags_23 },
    { 0x7a22d077, SUC_lexicon_tags_25 },
    { 0x2582d077, SUC_lexicon_tags_63 },
    { 0x8802d079, SUC_lexicon_tags_23 },
    { 0xa42d079, SUC_lexicon_tags_25 },
    { 0x29c2d07b, SUC_lexicon_tags_7 },
    { 0x2502d07c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3162d082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc2d08a, SUC_lexicon_tags_37 },
    { 0x8d82d08a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5a2d08d, SUC_lexicon_tags_37 },
    { 0x3c22d08e, SUC_lexicon_tags_37 },
    { 0x5c22d08f, SUC_lexicon_tags_46 },
    { 0x24c2d090, SUC_lexicon_tags_25 },
    { 0x21a2d091, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c2d096, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed62d09c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34a2d09e, SUC_lexicon_tags_31 },
    { 0x9822d09f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe62d0a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5282d0a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c2d0a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x73c2d0aa, SUC_lexicon_tags_63 },
    { 0xfbe2d0aa, SUC_lexicon_tags_37 },
    { 0x5982d0ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e62d0b0, SUC_lexicon_tags_14 },
    { 0xc062d0b1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc2d0b8, SUC_lexicon_tags_37 },
    { 0x7da2d0b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e22d0c1, SUC_lexicon_tags_49 },
    { 0x11c2d0c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dc2d0c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e2d0cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf402d0cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f02d0d1, SUC_lexicon_tags_151 },
    { 0xda82d0d2, SUC_lexicon_tags_7 },
    { 0x4122d0d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9262d0d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb162d0db, SUC_lexicon_tags_37 },
    { 0x4582d0dc, SUC_lexicon_tags_30 },
    { 0x8e22d0dd, SUC_lexicon_tags_11 },
    { 0xa7c2d0dd, SUC_lexicon_tags_43 },
    { 0xf6a2d0df, SUC_lexicon_tags_37 },
    { 0x6922d0e0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb822d0e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x442d0e4, SUC_lexicon_tags_25 },
    { 0xc022d0e5, SUC_lexicon_tags_8 },
    { 0x8502d0e6, SUC_lexicon_tags_14 },
    { 0x9be2d0e7, SUC_lexicon_tags_32 },
    { 0x4882d0e7, SUC_lexicon_tags_23 },
    { 0x53a2d0e7, SUC_lexicon_tags_1 },
    { 0x4c42d0ea, SUC_lexicon_tags_25 },
    { 0xfba2d0e9, SUC_lexicon_tags_15 },
    { 0x8302d0ec, SUC_lexicon_tags_7 },
    { 0x1ce2d0e9, SUC_lexicon_tags_21 },
    { 0xa782d0e8, SUC_lexicon_tags_82 },
    { 0x7102d0ef, SUC_lexicon_tags_7 },
    { 0xb9a2d0ef, SUC_lexicon_tags_11 },
    { 0x8c22d0f1, SUC_lexicon_tags_62 },
    { 0xe682d0f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3382d0f5, SUC_lexicon_tags_37 },
    { 0xbfc2d0f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb7c2d0f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc2d0fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a82d100, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab22d102, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5662d108, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae2d10c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4742d10e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf602d112, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d02d117, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf82d11c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7f62d11e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8082d121, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6702d124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8042d126, SUC_lexicon_tags_35 },
    { 0x33e2d127, SUC_lexicon_tags_160 },
    { 0xfdc2d128, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb742d12e, SUC_lexicon_tags_11 },
    { 0x7e42d12e, SUC_lexicon_tags_40 },
    { 0xd982d130, SUC_lexicon_tags_37 },
    { 0x5962d131, SUC_lexicon_tags_15 },
    { 0x3102d130, SUC_lexicon_tags_48 },
    { 0x44c2d133, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a2d139, SUC_lexicon_tags_29 },
    { 0xfe42d139, SUC_lexicon_tags_12 },
    { 0xb982d13b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb362d143, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5622d14f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e2d154, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xde42d156, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1802d158, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf482d15b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xda42d15d, SUC_lexicon_tags_3 },
    { 0x7462d15e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb262d163, SUC_lexicon_tags_11 },
    { 0xc702d163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a62d167, SUC_lexicon_tags_10 },
    { 0x2ea2d168, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6662d16c, SUC_lexicon_tags_31 },
    { 0xad02d16c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6762d172, SUC_lexicon_tags_7 },
    { 0x30e2d173, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0xaa62d175, SUC_lexicon_tags_17 },
    { 0x5162d175, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a2d182, SUC_lexicon_tags_9 },
    { 0x7dc2d182, SUC_lexicon_tags_11 },
    { 0x2f62d184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5a2d186, SUC_lexicon_tags_39 },
    { 0x47a2d187, SUC_lexicon_tags_84 },
    { 0x9c02d186, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c2d18d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd362d191, SUC_lexicon_tags_27 },
    { 0xbd02d191, SUC_lexicon_tags_7 },
    { 0x5422d193, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe022d195, SUC_lexicon_tags_35 },
    { 0x2802d196, SUC_lexicon_tags_48 },
    { 0x8f62d196, SUC_lexicon_tags_37 },
    { 0x3c02d196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2d19c, SUC_lexicon_tags_1 },
    { 0xb6e2d19d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf9a2d19f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc22d1a1, SUC_lexicon_tags_10 },
    { 0xb742d1a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9de2d1a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9942d1b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc2d1b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2702d1b9, SUC_lexicon_tags_6 },
    { 0xc622d1b9, SUC_lexicon_tags_45 },
    { 0xb1c2d1bb, SUC_lexicon_tags_15 },
    { 0x9982d1bb, SUC_lexicon_tags_23 },
    { 0x3a42d1bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102d1c2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd262d1c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6142d1c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4702d1ca, SUC_lexicon_tags_19 },
    { 0x7722d1cb, SUC_lexicon_tags_17 },
    { 0x2362d1cc, SUC_lexicon_tags_11 },
    { 0x9ee2d1cd, SUC_lexicon_tags_2 },
    { 0x8aa2d1cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa402d1d0, SUC_lexicon_tags_37 },
    { 0x44e2d1d1, SUC_lexicon_tags_17 },
    { 0x2aa2d1d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8842d1d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5e2d1d6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4582d1d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83e2d1da, SUC_lexicon_tags_26 },
    { 0x2742d1db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e2d1e0, SUC_lexicon_tags_15 },
    { 0xbe82d1e1, SUC_lexicon_tags_37 },
    { 0x3c82d1e1, SUC_lexicon_tags_1 },
    { 0x3462d1e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac42d1e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e2d1ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a82d1f0, SUC_lexicon_tags_84 },
    { 0xf22d1f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab22d1f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c2d1f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a2d205, SUC_lexicon_tags_38 },
    { 0x68c2d205, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e2d20c, SUC_lexicon_tags_2 },
    { 0x36c2d20d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b42d212, SUC_lexicon_tags_45 },
    { 0x6142d213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe542d217, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60a2d219, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb7e2d21b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc2d21f, SUC_lexicon_tags_2 },
    { 0xd382d220, SUC_lexicon_tags_8 },
    { 0x2e22d220, SUC_lexicon_tags_49 },
    { 0x54a2d21f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec2d225, SUC_lexicon_tags_60 },
    { 0x5f02d225, SUC_lexicon_tags_12 },
    { 0x5fe2d225, SUC_lexicon_tags_11 },
    { 0xc542d228, SUC_lexicon_tags_43 },
    { 0x2202d228, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc162d22b, SUC_lexicon_tags_12 },
    { 0xf6c2d22c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc522d22e, SUC_lexicon_tags_45 },
    { 0x91c2d22f, SUC_lexicon_tags_37 },
    { 0xc2d230, SUC_lexicon_tags_68 },
    { 0x5942d22e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fa2d233, SUC_lexicon_tags_25 },
    { 0xce2d234, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5622d236, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd022d238, SUC_lexicon_tags_25 },
    { 0xbc2d238, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f42d23b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e2d24c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7482d251, SUC_lexicon_tags_7 },
    { 0xcac2d251, SUC_lexicon_tags_37 },
    { 0x5782d253, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x8e22d255, SUC_lexicon_tags_15 },
    { 0xb2e2d255, SUC_lexicon_tags_60 },
    { 0x7f22d257, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3422d259, SUC_lexicon_tags_13 },
    { 0xd842d259, SUC_lexicon_tags_45 },
    { 0x9f42d25b, SUC_lexicon_tags_17 },
    { 0xb802d25c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf682d265, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c2d26c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf82d26f, SUC_lexicon_tags_63 },
    { 0x9982d270, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb762d279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6382d27f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d42d281, SUC_lexicon_tags_11 },
    { 0xd422d282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32e2d284, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e2d288, SUC_lexicon_tags_63 },
    { 0x9682d289, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7542d28c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f42d28e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb22d294, SUC_lexicon_tags_17 },
    { 0xf142d295, SUC_lexicon_tags_28 },
    { 0x6462d295, SUC_lexicon_tags_37 },
    { 0x6c62d297, SUC_lexicon_tags_84 },
    { 0xed82d298, SUC_lexicon_tags_25 },
    { 0xfe22d299, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb742d29b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee2d29e, SUC_lexicon_tags_39 },
    { 0xdb02d29e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1742d2a2, SUC_lexicon_tags_60 },
    { 0x2142d2a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5c42d2a5, SUC_lexicon_tags_33 },
    { 0xe222d2a5, SUC_lexicon_tags_63 },
    { 0xba22d2a5, SUC_lexicon_tags_1 },
    { 0x2f22d2a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd22d2aa, SUC_lexicon_tags_7 },
    { 0xe022d2aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9022d2ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ac2d2af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd022d2b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e2d2b5, SUC_lexicon_tags_7 },
    { 0xdfc2d2b6, SUC_lexicon_tags_2 },
    { 0x3502d2b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c2d2bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c2d2be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x38c2d2c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8622d2c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea2d2c9, SUC_lexicon_tags_14 },
    { 0xba82d2ca, SUC_lexicon_tags_65 },
    { 0x22d2c9, SUC_lexicon_tags_32 },
    { 0x8ce2d2ca, SUC_lexicon_tags_31 },
    { 0xc22d2c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4042d2d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d62d2d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb22d2d9, SUC_lexicon_tags_0 },
    { 0xf662d2da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc862d2e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162d2e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xda42d2ea, SUC_lexicon_tags_37 },
    { 0xbd82d2ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf02d2f5, SUC_lexicon_tags_17 },
    { 0xe4a2d2f6, SUC_lexicon_tags_25 },
    { 0xc7e2d2f6, SUC_lexicon_tags_60 },
    { 0x1f42d2f7, SUC_lexicon_tags_25 },
    { 0x5c42d2f9, SUC_lexicon_tags_49 },
    { 0x6a82d2f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b42d2fc, SUC_lexicon_tags_2 },
    { 0x1d82d2fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82d304, SUC_lexicon_tags_14 },
    { 0xe622d304, SUC_lexicon_tags_14 },
    { 0xc3c2d306, SUC_lexicon_tags_15 },
    { 0x2882d307, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb602d30c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e2d30f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa022d315, SUC_lexicon_tags_11 },
    { 0x8102d315, SUC_lexicon_tags_30 },
    { 0x3702d317, SUC_lexicon_tags_17 },
    { 0x48e2d318, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2bc2d31a, SUC_lexicon_tags_79 },
    { 0x5002d31b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc22d31d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb62d323, SUC_lexicon_tags_15 },
    { 0xe1c2d323, SUC_lexicon_tags_11 },
    { 0xb422d323, SUC_lexicon_tags_31 },
    { 0x7f82d326, SUC_lexicon_tags_19 },
    { 0x90c2d327, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf722d329, SUC_lexicon_tags_1 },
    { 0x2162d32a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe2d32d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf042d335, SUC_lexicon_tags_2 },
    { 0xc822d336, SUC_lexicon_tags_15 },
    { 0x2a22d335, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc742d33a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8322d33d, SUC_lexicon_tags_27 },
    { 0x7382d33e, SUC_lexicon_tags_15 },
    { 0x4702d33f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe942d342, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe02d345, SUC_lexicon_tags_93 },
    { 0xfd02d345, SUC_lexicon_tags_14 },
    { 0x5fc2d347, SUC_lexicon_tags_1 },
    { 0xb62d347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7a2d34a, SUC_lexicon_tags_59 },
    { 0x1b62d34a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf262d350, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc2d355, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b42d35a, SUC_lexicon_tags_37 },
    { 0xf22d35a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe342d35e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd022d360, SUC_lexicon_tags_45 },
    { 0x33a2d361, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d82d364, SUC_lexicon_tags_16 },
    { 0x3dc2d365, SUC_lexicon_tags_26 },
    { 0xf182d366, SUC_lexicon_tags_23 },
    { 0x33a2d367, SUC_lexicon_tags_1 },
    { 0x49a2d366, SUC_lexicon_tags_14 },
    { 0x9802d368, SUC_lexicon_tags_33 },
    { 0xcd82d366, SUC_lexicon_tags_45 },
    { 0xe362d368, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8742d370, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3862d373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4882d375, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9502d37d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda02d380, SUC_lexicon_tags_25 },
    { 0x9902d381, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd202d385, SUC_lexicon_tags_7 },
    { 0x64a2d386, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd702d389, SUC_lexicon_tags_16 },
    { 0xb482d389, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422d38f, SUC_lexicon_tags_23 },
    { 0x8942d390, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0xcb42d392, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1922d399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5602d3a0, SUC_lexicon_tags_15 },
    { 0x9122d3a1, SUC_lexicon_tags_25 },
    { 0x5922d3a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e2d3a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3582d3a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa982d3ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc02d3ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9662d3af, SUC_lexicon_tags_46 },
    { 0x1262d3b0, SUC_lexicon_tags_1 },
    { 0x7d02d3b1, SUC_lexicon_tags_14 },
    { 0x50a2d3b0, SUC_lexicon_tags_16 },
    { 0xf902d3af, SUC_lexicon_tags_12 },
    { 0x38c2d3b4, SUC_lexicon_tags_58 },
    { 0xd242d3b5, SUC_lexicon_tags_37 },
    { 0xb042d3b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf482d3b9, SUC_lexicon_tags_25 },
    { 0x76e2d3b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd962d3bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdb82d3be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e2d3c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe802d3c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a2d3ca, SUC_lexicon_tags_2 },
    { 0x3c2d3ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca2d3ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5022d3d3, SUC_lexicon_tags_33 },
    { 0x7342d3d4, SUC_lexicon_tags_15 },
    { 0x9d82d3d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6482d3dc, SUC_lexicon_tags_43 },
    { 0x1c42d3dd, SUC_lexicon_tags_9 },
    { 0x5ea2d3dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6f02d3e0, SUC_lexicon_tags_7 },
    { 0x7362d3e1, SUC_lexicon_tags_17 },
    { 0x9f02d3e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0c2d3e4, SUC_lexicon_tags_1 },
    { 0x5fc2d3e4, SUC_lexicon_tags_46 },
    { 0x1c82d3e5, SUC_lexicon_tags_1 },
    { 0x6a02d3e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d02d3ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8a2d3ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7042d3ee, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9442d3f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc822d3f3, SUC_lexicon_tags_1 },
    { 0x8f62d3f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c2d3f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c2d3fd, SUC_lexicon_tags_23 },
    { 0x6c42d3fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa62d401, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5b42d403, SUC_lexicon_tags_3 },
    { 0x7502d403, SUC_lexicon_tags_2 },
    { 0xaa42d405, SUC_lexicon_tags_59 },
    { 0xe2a2d406, SUC_lexicon_tags_4 },
    { 0x4a82d407, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaec2d409, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee2d412, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd22d416, SUC_lexicon_tags_19 },
    { 0x1382d416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb02d41b, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0xfb22d41d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6742d420, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec2d423, SUC_lexicon_tags_25 },
    { 0x6fa2d423, SUC_lexicon_tags_13 },
    { 0x6402d425, SUC_lexicon_tags_2 },
    { 0x9802d423, SUC_lexicon_tags_27 },
    { 0x5482d427, SUC_lexicon_tags_39 },
    { 0x27c2d428, SUC_lexicon_tags_23 },
    { 0xd82d425, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa682d42d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2702d42f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f02d433, SUC_lexicon_tags_77 },
    { 0xda2d434, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc2d43d, SUC_lexicon_tags_19 },
    { 0x7042d43e, SUC_lexicon_tags_16 },
    { 0x6a62d43f, SUC_lexicon_tags_57 },
    { 0x5c02d43d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf882d444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a82d447, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe2e2d449, SUC_lexicon_tags_25 },
    { 0x94c2d449, SUC_lexicon_tags_15 },
    { 0x7b42d449, SUC_lexicon_tags_3 },
    { 0x2562d449, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e02d44f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe502d451, SUC_lexicon_tags_27 },
    { 0xeea2d452, SUC_lexicon_tags_55 },
    { 0x5922d453, SUC_lexicon_tags_17 },
    { 0x1e02d453, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8402d456, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a2d45a, SUC_lexicon_tags_27 },
    { 0x5a42d45b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4462d45f, SUC_lexicon_tags_47 },
    { 0x9ec2d460, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb182d465, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb262d467, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2b42d469, SUC_lexicon_tags_26 },
    { 0x6e02d469, SUC_lexicon_tags_37 },
    { 0xaa2d46b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c2d46f, SUC_lexicon_tags_12 },
    { 0xc7a2d470, SUC_lexicon_tags_25 },
    { 0xede2d471, SUC_lexicon_tags_10 },
    { 0x5202d472, SUC_lexicon_tags_27 },
    { 0x83e2d470, SUC_lexicon_tags_44 },
    { 0xee2d474, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f62d47a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f42d47c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4202d483, SUC_lexicon_tags_57 },
    { 0x6062d483, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe2d488, SUC_lexicon_tags_63 },
    { 0xb622d489, SUC_lexicon_tags_15 },
    { 0xea2d48a, SUC_lexicon_tags_15 },
    { 0x2922d48b, SUC_lexicon_tags_15 },
    { 0x98e2d48a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe762d48e, SUC_lexicon_tags_11 },
    { 0x9a22d48e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b22d492, SUC_lexicon_tags_2 },
    { 0xc6a2d493, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x63e2d495, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2362d49a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3bc2d49c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e2d4a0, SUC_lexicon_tags_1 },
    { 0x9e42d4a1, SUC_lexicon_tags_10 },
    { 0xe9a2d4a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc522d4aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa942d4ad, SUC_lexicon_tags_23 },
    { 0x7822d4ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e42d4b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb462d4b2, SUC_lexicon_tags_17 },
    { 0x8422d4b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c2d4b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe2d4bd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4ae2d4bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6782d4c2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb602d4c4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa842d4c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf22d4d1, SUC_lexicon_tags_15 },
    { 0xe902d4d2, SUC_lexicon_tags_14 },
    { 0xb1a2d4d3, SUC_lexicon_tags_113 },
    { 0xcb62d4d4, SUC_lexicon_tags_30 },
    { 0x65e2d4d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c2d4db, SUC_lexicon_tags_7 },
    { 0xff42d4dc, SUC_lexicon_tags_15 },
    { 0xd582d4dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea2d4e1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa2d4e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2d62d4eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8102d4ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c2d4f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc222d4f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb22d4f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a62d4fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3682d50b, SUC_lexicon_tags_17 },
    { 0xb382d50b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf082d510, SUC_lexicon_tags_7 },
    { 0xca62d511, SUC_lexicon_tags_11 },
    { 0x77c2d512, SUC_lexicon_tags_43 },
    { 0x54c2d513, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5722d515, SUC_lexicon_tags_25 },
    { 0x9ec2d515, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f02d51a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee2d51e, SUC_lexicon_tags_38 },
    { 0x51c2d51f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba2d526, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf82d52a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a62d530, SUC_lexicon_tags_41 },
    { 0x9b82d531, SUC_lexicon_tags_25 },
    { 0x8a42d530, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b62d537, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2422d53b, SUC_lexicon_tags_56 },
    { 0xa842d53c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3da2d53e, SUC_lexicon_tags_25 },
    { 0xf1a2d53f, SUC_lexicon_tags_28 },
    { 0xe582d53e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d543, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c2d54d, SUC_lexicon_tags_9 },
    { 0xc1e2d54e, SUC_lexicon_tags_43 },
    { 0xb482d54f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8242d551, SUC_lexicon_tags_106 },
    { 0xffc2d551, SUC_lexicon_tags_30 },
    { 0xa182d551, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca2d556, SUC_lexicon_tags_0 },
    { 0xddc2d557, SUC_lexicon_tags_11 },
    { 0xe502d557, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bc2d55a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e2d55d, SUC_lexicon_tags_65 },
    { 0x1462d55e, SUC_lexicon_tags_37 },
    { 0x1ee2d55f, SUC_lexicon_tags_37 },
    { 0xff42d560, SUC_lexicon_tags_12 },
    { 0xa22d560, SUC_lexicon_tags_17 },
    { 0x8c2d561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb002d564, SUC_lexicon_tags_48 },
    { 0x2382d564, SUC_lexicon_tags_10 },
    { 0xf6a2d566, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec2d56e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec2d571, SUC_lexicon_tags_37 },
    { 0xd42d572, SUC_lexicon_tags_39 },
    { 0x1562d573, SUC_lexicon_tags_5 },
    { 0xd182d573, SUC_lexicon_tags_82 },
    { 0x8842d572, SUC_lexicon_tags_1 },
    { 0x6de2d571, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4a2d578, SUC_lexicon_tags_11 },
    { 0xd782d578, SUC_lexicon_tags_17 },
    { 0x5b22d57a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1062d57c, SUC_lexicon_tags_12 },
    { 0xea82d57c, SUC_lexicon_tags_25 },
    { 0xeda2d57e, SUC_lexicon_tags_33 },
    { 0xa522d57f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e82d581, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd42d586, SUC_lexicon_tags_77 },
    { 0xef42d587, SUC_lexicon_tags_2 },
    { 0xb722d588, SUC_lexicon_tags_25 },
    { 0x9922d588, SUC_lexicon_tags_11 },
    { 0x4342d586, SUC_lexicon_tags_27 },
    { 0x2042d58b, SUC_lexicon_tags_19 },
    { 0x7242d586, SUC_lexicon_tags_7 },
    { 0xfbc2d58d, SUC_lexicon_tags_39 },
    { 0x54e2d58d, SUC_lexicon_tags_29 },
    { 0xe222d58d, SUC_lexicon_tags_8 },
    { 0x822d590, SUC_lexicon_tags_12 },
    { 0xc082d591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcec2d593, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a2d598, SUC_lexicon_tags_17 },
    { 0xff82d599, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd622d59c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f42d5a3, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c2d5a6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd222d5aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5442d5ad, SUC_lexicon_tags_12 },
    { 0xb482d5ae, SUC_lexicon_tags_7 },
    { 0x5c22d5af, SUC_lexicon_tags_15 },
    { 0x7842d5af, SUC_lexicon_tags_7 },
    { 0x1782d5af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ee2d5b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2622d5b5, SUC_lexicon_tags_60 },
    { 0x4a42d5b5, SUC_lexicon_tags_14 },
    { 0x8c22d5b7, SUC_lexicon_tags_14 },
    { 0xfe62d5b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c62d5bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab62d5c2, SUC_lexicon_tags_31 },
    { 0xa002d5c3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf062d5c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb602d5c8, SUC_lexicon_tags_17 },
    { 0x1862d5c9, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x5982d5cb, SUC_lexicon_tags_0 },
    { 0xffe2d5cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a2d5d2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb062d5d7, SUC_lexicon_tags_25 },
    { 0xe982d5d7, SUC_lexicon_tags_15 },
    { 0x3b42d5d9, SUC_lexicon_tags_39 },
    { 0x762d5d8, SUC_lexicon_tags_37 },
    { 0x1862d5db, SUC_lexicon_tags_60 },
    { 0xffe2d5dc, SUC_lexicon_tags_11 },
    { 0xd842d5db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c2d5e2, SUC_lexicon_tags_2 },
    { 0x6a22d5e3, SUC_lexicon_tags_32 },
    { 0xd2c2d5e4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xea02d5e7, SUC_lexicon_tags_9 },
    { 0x8ce2d5e8, SUC_lexicon_tags_25 },
    { 0x4a42d5e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8882d5ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2d5f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8ae2d5f4, SUC_lexicon_tags_23 },
    { 0x4e82d5f4, SUC_lexicon_tags_45 },
    { 0x6782d5f6, SUC_lexicon_tags_66 },
    { 0x83a2d5f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a2d601, SUC_lexicon_tags_1 },
    { 0x7382d602, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5082d606, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7822d609, SUC_lexicon_tags_25 },
    { 0x5a62d60a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5c2d60c, SUC_lexicon_tags_25 },
    { 0x5fe2d60c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae2d610, SUC_lexicon_tags_7 },
    { 0x7c22d611, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c2d618, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e82d61a, SUC_lexicon_tags_1 },
    { 0xfaa2d61b, SUC_lexicon_tags_37 },
    { 0x9c22d61c, SUC_lexicon_tags_11 },
    { 0x9722d61a, SUC_lexicon_tags_17 },
    { 0x4802d61e, SUC_lexicon_tags_15 },
    { 0x1782d61f, SUC_lexicon_tags_26 },
    { 0x4962d61f, SUC_lexicon_tags_29 },
    { 0x3262d620, SUC_lexicon_tags_37 },
    { 0x50c2d622, SUC_lexicon_tags_37 },
    { 0xb4e2d622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x302d625, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d02d627, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9542d629, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7082d62f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6442d631, SUC_lexicon_tags_25 },
    { 0xfd02d632, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xebc2d634, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x8162d637, SUC_lexicon_tags_8 },
    { 0xafe2d638, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d22d63b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa802d63f, SUC_lexicon_tags_1 },
    { 0x74e2d640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae2d649, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc042d652, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d42d658, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a2d65e, SUC_lexicon_tags_17 },
    { 0x89e2d65f, SUC_lexicon_tags_63 },
    { 0x67e2d65e, SUC_lexicon_tags_66 },
    { 0x9e62d661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe62d664, SUC_lexicon_tags_19 },
    { 0xba62d665, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc2d66a, SUC_lexicon_tags_11 },
    { 0xd9a2d66a, SUC_lexicon_tags_25 },
    { 0x8d42d66c, SUC_lexicon_tags_7 },
    { 0xbc22d66d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2362d670, SUC_lexicon_tags_7 },
    { 0xa7e2d671, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa142d675, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2d678, SUC_lexicon_tags_43 },
    { 0xe322d679, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74c2d67b, SUC_lexicon_tags_33 },
    { 0x2142d67c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54e2d67e, SUC_lexicon_tags_34 },
    { 0x2502d67f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa502d681, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8662d686, SUC_lexicon_tags_12 },
    { 0x4862d687, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb282d689, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e2d68c, SUC_lexicon_tags_37 },
    { 0x5282d68c, SUC_lexicon_tags_29 },
    { 0x4502d68e, SUC_lexicon_tags_15 },
    { 0x3b22d68c, SUC_lexicon_tags_11 },
    { 0x6442d690, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea2d697, SUC_lexicon_tags_7 },
    { 0x2382d698, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8082d69a, SUC_lexicon_tags_1 },
    { 0x6482d69a, SUC_lexicon_tags_11 },
    { 0x522d69c, SUC_lexicon_tags_1 },
    { 0xeb02d69b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3042d6a1, SUC_lexicon_tags_15 },
    { 0x39c2d6a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf42d6a4, SUC_lexicon_tags_37 },
    { 0x8f42d6a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe2d6ad, SUC_lexicon_tags_46 },
    { 0x91c2d6ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e2d6b2, SUC_lexicon_tags_25 },
    { 0x3b42d6b2, SUC_lexicon_tags_11 },
    { 0x7482d6b4, SUC_lexicon_tags_25 },
    { 0x5862d6b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f22d6b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1062d6ba, SUC_lexicon_tags_15 },
    { 0xa482d6bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c2d6c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef62d6c8, SUC_lexicon_tags_12 },
    { 0x8f02d6c9, SUC_lexicon_tags_626 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e62d6cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282d6d9, SUC_lexicon_tags_11 },
    { 0x7082d6da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8022d6dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a2d6e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2c22d6e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb882d6e8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2d6eb, SUC_lexicon_tags_7 },
    { 0x6642d6ec, SUC_lexicon_tags_46 },
    { 0x1bc2d6eb, SUC_lexicon_tags_15 },
    { 0x74c2d6eb, SUC_lexicon_tags_17 },
    { 0x2662d6ef, SUC_lexicon_tags_224 },
    { 0xf722d6ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1702d6f3, SUC_lexicon_tags_4 },
    { 0xbc82d6f4, SUC_lexicon_tags_37 },
    { 0x62e2d6f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c2d6fa, SUC_lexicon_tags_45 },
    { 0x29c2d6fb, SUC_lexicon_tags_85 },
    { 0x8d82d6fc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1102d6fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2d701, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6502d709, SUC_lexicon_tags_17 },
    { 0x9e62d709, SUC_lexicon_tags_12 },
    { 0xd8e2d70a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882d710, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2d717, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf482d71a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e02d725, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc502d727, SUC_lexicon_tags_25 },
    { 0x3ea2d727, SUC_lexicon_tags_15 },
    { 0xfc42d729, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c2d732, SUC_lexicon_tags_32 },
    { 0x2ce2d733, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c42d73e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f62d740, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2d745, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5282d747, SUC_lexicon_tags_1 },
    { 0xf342d747, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c82d74e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c2d757, SUC_lexicon_tags_7 },
    { 0xe102d758, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe942d75b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec2d75f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b02d761, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78a2d763, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b62d768, SUC_lexicon_tags_47 },
    { 0x5202d769, SUC_lexicon_tags_29 },
    { 0x7ae2d76a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde2d76d, SUC_lexicon_tags_49 },
    { 0x47e2d76e, SUC_lexicon_tags_29 },
    { 0xcd22d76e, SUC_lexicon_tags_7 },
    { 0x37e2d76d, SUC_lexicon_tags_11 },
    { 0x3b62d76d, SUC_lexicon_tags_7 },
    { 0xa042d770, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b02d777, SUC_lexicon_tags_12 },
    { 0x3202d778, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb782d77c, SUC_lexicon_tags_47 },
    { 0xa282d77d, SUC_lexicon_tags_14 },
    { 0x922d77e, SUC_lexicon_tags_66 },
    { 0x97a2d77f, SUC_lexicon_tags_12 },
    { 0xd522d77f, SUC_lexicon_tags_85 },
    { 0x8202d781, SUC_lexicon_tags_14 },
    { 0x5f62d782, SUC_lexicon_tags_2 },
    { 0xbc42d783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e2d792, SUC_lexicon_tags_12 },
    { 0x142d792, SUC_lexicon_tags_12 },
    { 0xa4c2d794, SUC_lexicon_tags_56 },
    { 0x53e2d794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa982d797, SUC_lexicon_tags_26 },
    { 0x79c2d798, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa2d79c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1122d79e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d22d7a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a2d7a6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de2d7aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54e2d7ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79a2d7ae, SUC_lexicon_tags_31 },
    { 0x3462d7af, SUC_lexicon_tags_23 },
    { 0xfdc2d7af, SUC_lexicon_tags_42 },
    { 0xd522d7b1, SUC_lexicon_tags_12 },
    { 0xe562d7b2, SUC_lexicon_tags_11 },
    { 0xa8c2d7b2, SUC_lexicon_tags_60 },
    { 0xa3e2d7b4, SUC_lexicon_tags_19 },
    { 0x7722d7ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae2d7bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc402d7bf, SUC_lexicon_tags_253 },
    { 0xc062d7c0, SUC_lexicon_tags_2 },
    { 0x53a2d7c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c2d7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8822d7c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2962d7cc, SUC_lexicon_tags_1 },
    { 0x94e2d7cd, SUC_lexicon_tags_15 },
    { 0x5f22d7cd, SUC_lexicon_tags_37 },
    { 0x1982d7cf, SUC_lexicon_tags_21 },
    { 0xafc2d7cf, SUC_lexicon_tags_35 },
    { 0xf0a2d7ce, SUC_lexicon_tags_43 },
    { 0xe142d7cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc2d7d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b82d7dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb462d7e0, SUC_lexicon_tags_66 },
    { 0xe982d7e0, SUC_lexicon_tags_37 },
    { 0x63c2d7e2, SUC_lexicon_tags_15 },
    { 0xdaa2d7e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb642d7e6, SUC_lexicon_tags_1119 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb42d7e9, SUC_lexicon_tags_2 },
    { 0x3942d7ea, SUC_lexicon_tags_25 },
    { 0xa1a2d7eb, SUC_lexicon_tags_7 },
    { 0x80e2d7e9, SUC_lexicon_tags_38 },
    { 0x52a2d7ed, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x9c42d7ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa2d7f2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xae62d7f4, SUC_lexicon_tags_25 },
    { 0x8c42d7f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e2d7f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1202d7fb, SUC_lexicon_tags_47 },
    { 0xdce2d7fb, SUC_lexicon_tags_3 },
    { 0x1e62d7fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a2d801, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3482d805, SUC_lexicon_tags_61 },
    { 0xf5a2d805, SUC_lexicon_tags_23 },
    { 0x1742d807, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5182d809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf002d80b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8982d80e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e2d813, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc642d816, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe342d819, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1dc2d81b, SUC_lexicon_tags_25 },
    { 0x2ae2d81c, SUC_lexicon_tags_7 },
    { 0xc82d81b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e82d821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc2d824, SUC_lexicon_tags_17 },
    { 0x2482d825, SUC_lexicon_tags_21 },
    { 0x7202d825, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3862d82c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c2d82f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a02d831, SUC_lexicon_tags_1 },
    { 0x4a22d832, SUC_lexicon_tags_17 },
    { 0xcb22d831, SUC_lexicon_tags_2 },
    { 0xf22d834, SUC_lexicon_tags_11 },
    { 0xc082d833, SUC_lexicon_tags_1 },
    { 0x2902d836, SUC_lexicon_tags_1 },
    { 0xa582d837, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9022d83b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e2d842, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c2d84a, SUC_lexicon_tags_66 },
    { 0x6702d84b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc162d84d, SUC_lexicon_tags_11 },
    { 0x91e2d84d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2f22d850, SUC_lexicon_tags_7 },
    { 0x60a2d850, SUC_lexicon_tags_33 },
    { 0x6182d852, SUC_lexicon_tags_56 },
    { 0xb4c2d853, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9382d859, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1102d85c, SUC_lexicon_tags_23 },
    { 0xaae2d85d, SUC_lexicon_tags_124 },
    { 0xb3e2d85e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa2d863, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe662d866, SUC_lexicon_tags_17 },
    { 0xc282d867, SUC_lexicon_tags_37 },
    { 0x6462d868, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfda2d86a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2202d86f, SUC_lexicon_tags_15 },
    { 0x5002d870, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb262d876, SUC_lexicon_tags_15 },
    { 0xa942d877, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b82d87a, SUC_lexicon_tags_15 },
    { 0x2622d87b, SUC_lexicon_tags_37 },
    { 0xd6e2d87c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe462d880, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a2d882, SUC_lexicon_tags_19 },
    { 0x9e62d882, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77a2d885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2e2d887, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c2d88b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3262d890, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8502d894, SUC_lexicon_tags_79 },
    { 0x4702d895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e2d89d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x802d89f, SUC_lexicon_tags_60 },
    { 0x5242d89f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e2d8a2, SUC_lexicon_tags_95 },
    { 0xc102d8a2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xdec2d8a5, SUC_lexicon_tags_1 },
    { 0x7382d8a6, SUC_lexicon_tags_12 },
    { 0xc642d8a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc3e2d8a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a2d8ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb362d8af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5022d8b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c2d8c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf062d8c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba42d8cd, SUC_lexicon_tags_11 },
    { 0xe422d8ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c82d8d3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4902d8d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a62d8db, SUC_lexicon_tags_60 },
    { 0x6382d8dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2e2d8de, SUC_lexicon_tags_23 },
    { 0xf7a2d8df, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8482d8e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c2d8e5, SUC_lexicon_tags_1 },
    { 0x2a02d8e6, SUC_lexicon_tags_49 },
    { 0x802d8e6, SUC_lexicon_tags_14 },
    { 0x3282d8e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed62d8ea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da2d8ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbd02d8ef, SUC_lexicon_tags_31 },
    { 0x6fe2d8ef, SUC_lexicon_tags_95 },
    { 0x3942d8f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a2d8f9, SUC_lexicon_tags_2 },
    { 0x4762d8fa, SUC_lexicon_tags_13 },
    { 0x482d8fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7782d8fe, SUC_lexicon_tags_47 },
    { 0x3442d8ff, SUC_lexicon_tags_14 },
    { 0xdaa2d900, SUC_lexicon_tags_20 },
    { 0x6702d901, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e22d903, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3202d909, SUC_lexicon_tags_63 },
    { 0xfe02d90a, SUC_lexicon_tags_12 },
    { 0xf822d90a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc102d90d, SUC_lexicon_tags_11 },
    { 0xf382d90d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x582d910, SUC_lexicon_tags_12 },
    { 0x9062d911, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4362d918, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e2d91f, SUC_lexicon_tags_37 },
    { 0x65c2d920, SUC_lexicon_tags_33 },
    { 0x7902d921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9402d925, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2322d928, SUC_lexicon_tags_60 },
    { 0x82c2d929, SUC_lexicon_tags_1 },
    { 0x6c02d92a, SUC_lexicon_tags_7 },
    { 0xea42d92b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca2d932, SUC_lexicon_tags_63 },
    { 0x10e2d933, SUC_lexicon_tags_7 },
    { 0x64e2d934, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x95e2d936, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a62d93a, SUC_lexicon_tags_0 },
    { 0x9822d93a, SUC_lexicon_tags_29 },
    { 0x5582d93c, SUC_lexicon_tags_49 },
    { 0xe62d93b, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf222d941, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc2d944, SUC_lexicon_tags_37 },
    { 0xd142d944, SUC_lexicon_tags_27 },
    { 0xb5c2d945, SUC_lexicon_tags_29 },
    { 0x3582d947, SUC_lexicon_tags_19 },
    { 0x8722d947, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7142d94a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c42d94c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4782d94f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a2d952, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2fe2d954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1382d956, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd902d959, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7862d95c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xab42d95e, SUC_lexicon_tags_8 },
    { 0x6162d95e, SUC_lexicon_tags_2 },
    { 0x3e82d960, SUC_lexicon_tags_15 },
    { 0xcfc2d961, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xd422d963, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce82d966, SUC_lexicon_tags_37 },
    { 0x44c2d966, SUC_lexicon_tags_43 },
    { 0x9e02d968, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322d96e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf862d974, SUC_lexicon_tags_37 },
    { 0x3ec2d974, SUC_lexicon_tags_17 },
    { 0xc2a2d974, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf722d978, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d97b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd2e2d97d, SUC_lexicon_tags_25 },
    { 0x1822d97e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc042d980, SUC_lexicon_tags_60 },
    { 0xb2c2d981, SUC_lexicon_tags_11 },
    { 0x2022d982, SUC_lexicon_tags_12 },
    { 0x68a2d982, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf02d986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d02d988, SUC_lexicon_tags_39 },
    { 0xe02d989, SUC_lexicon_tags_38 },
    { 0x9062d988, SUC_lexicon_tags_61 },
    { 0x73e2d989, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2d990, SUC_lexicon_tags_1 },
    { 0xb982d991, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x4742d994, SUC_lexicon_tags_25 },
    { 0x8ca2d994, SUC_lexicon_tags_158 },
    { 0x7522d994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc82d99b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9282d99d, SUC_lexicon_tags_3 },
    { 0x1422d99e, SUC_lexicon_tags_6 },
    { 0xad02d99f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a2d9a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18a2d9a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc362d9a8, SUC_lexicon_tags_47 },
    { 0x8142d9a8, SUC_lexicon_tags_1 },
    { 0xdfc2d9aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe622d9b3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x59a2d9b5, SUC_lexicon_tags_37 },
    { 0xd0c2d9b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32c2d9b8, SUC_lexicon_tags_29 },
    { 0x1782d9b9, SUC_lexicon_tags_26 },
    { 0xdc02d9ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c02d9bc, SUC_lexicon_tags_37 },
    { 0x41c2d9bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe782d9c1, SUC_lexicon_tags_33 },
    { 0xdd82d9c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c2d9c5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcd62d9c7, SUC_lexicon_tags_12 },
    { 0x5302d9c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x402d9ca, SUC_lexicon_tags_11 },
    { 0x5a82d9cb, SUC_lexicon_tags_61 },
    { 0x85a2d9cc, SUC_lexicon_tags_1 },
    { 0xb662d9cd, SUC_lexicon_tags_10 },
    { 0x4042d9cb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e2d9d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a42d9d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a2d9d7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9162d9da, SUC_lexicon_tags_32 },
    { 0xdc42d9db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ce2d9dd, SUC_lexicon_tags_19 },
    { 0x9e22d9de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe422d9e0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe82d9e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe22d9e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a2d9e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x162d9eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e2d9ef, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2682d9f2, SUC_lexicon_tags_14 },
    { 0xa802d9f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea2d9f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c2d9f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa2da00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd22da03, SUC_lexicon_tags_17 },
    { 0x30a2da04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf902da08, SUC_lexicon_tags_63 },
    { 0x39a2da09, SUC_lexicon_tags_11 },
    { 0x52e2da08, SUC_lexicon_tags_29 },
    { 0x2342da0b, SUC_lexicon_tags_43 },
    { 0xd2a2da0c, SUC_lexicon_tags_11 },
    { 0x7d42da0d, SUC_lexicon_tags_14 },
    { 0xb962da0a, SUC_lexicon_tags_37 },
    { 0x6a42da0c, SUC_lexicon_tags_15 },
    { 0xa082da10, SUC_lexicon_tags_11 },
    { 0x4802da10, SUC_lexicon_tags_11 },
    { 0xa782da10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d02da17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd702da1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd922da20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2da25, SUC_lexicon_tags_11 },
    { 0x3ae2da26, SUC_lexicon_tags_14 },
    { 0x6e82da26, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8042da2e, SUC_lexicon_tags_82 },
    { 0x46e2da2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d02da32, SUC_lexicon_tags_25 },
    { 0x56e2da33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc2da36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3082da3a, SUC_lexicon_tags_43 },
    { 0x19a2da3b, SUC_lexicon_tags_39 },
    { 0xcb82da3c, SUC_lexicon_tags_37 },
    { 0xdc82da3c, SUC_lexicon_tags_63 },
    { 0xd402da3a, SUC_lexicon_tags_4 },
    { 0x42e2da3f, SUC_lexicon_tags_36 },
    { 0xf342da3e, SUC_lexicon_tags_101 },
    { 0x9ba2da41, SUC_lexicon_tags_184 },
    { 0xb302da3f, SUC_lexicon_tags_33 },
    { 0xfae2da43, SUC_lexicon_tags_15 },
    { 0xc842da44, SUC_lexicon_tags_12 },
    { 0x9f02da45, SUC_lexicon_tags_38 },
    { 0x25a2da45, SUC_lexicon_tags_1 },
    { 0x8362da46, SUC_lexicon_tags_45 },
    { 0x1b62da48, SUC_lexicon_tags_25 },
    { 0x9ee2da49, SUC_lexicon_tags_27 },
    { 0xb222da44, SUC_lexicon_tags_29 },
    { 0xa4a2da4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5c2da4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4482da50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1262da53, SUC_lexicon_tags_12 },
    { 0x3da2da54, SUC_lexicon_tags_16 },
    { 0x1402da55, SUC_lexicon_tags_12 },
    { 0x96c2da56, SUC_lexicon_tags_10 },
    { 0xffa2da57, SUC_lexicon_tags_43 },
    { 0xa062da58, SUC_lexicon_tags_17 },
    { 0xcce2da56, SUC_lexicon_tags_26 },
    { 0x1782da53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd02da5e, SUC_lexicon_tags_4 },
    { 0xa022da5f, SUC_lexicon_tags_46 },
    { 0xe1c2da5e, SUC_lexicon_tags_12 },
    { 0xcf02da61, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3782da69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3402da6e, SUC_lexicon_tags_2 },
    { 0x142da6e, SUC_lexicon_tags_37 },
    { 0xe402da70, SUC_lexicon_tags_15 },
    { 0x8f62da71, SUC_lexicon_tags_2 },
    { 0x1da2da72, SUC_lexicon_tags_14 },
    { 0x3102da73, SUC_lexicon_tags_15 },
    { 0xd582da74, SUC_lexicon_tags_26 },
    { 0xac02da73, SUC_lexicon_tags_47 },
    { 0xf4a2da74, SUC_lexicon_tags_19 },
    { 0xbb02da75, SUC_lexicon_tags_47 },
    { 0xe5e2da73, SUC_lexicon_tags_88 },
    { 0xa662da79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a2da80, SUC_lexicon_tags_7 },
    { 0x202da81, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c2da84, SUC_lexicon_tags_60 },
    { 0x8162da85, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd62da8a, SUC_lexicon_tags_68 },
    { 0x3ec2da8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9042da9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb382da9e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a2daa2, SUC_lexicon_tags_15 },
    { 0x6a42daa3, SUC_lexicon_tags_45 },
    { 0x7702daa3, SUC_lexicon_tags_25 },
    { 0x12a2daa4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcdc2daa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe62daaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7202dab5, SUC_lexicon_tags_7 },
    { 0x922dab6, SUC_lexicon_tags_27 },
    { 0xd062dab7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3fa2dab9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a2dabc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9342dabf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c2dac3, SUC_lexicon_tags_60 },
    { 0xaae2dac3, SUC_lexicon_tags_8 },
    { 0x9fc2dac3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1142dac9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbc62dacb, SUC_lexicon_tags_37 },
    { 0xc3a2dacb, SUC_lexicon_tags_9 },
    { 0x9962dacd, SUC_lexicon_tags_33 },
    { 0xd642dacd, SUC_lexicon_tags_19 },
    { 0x522dacf, SUC_lexicon_tags_25 },
    { 0x30e2dacb, SUC_lexicon_tags_7 },
    { 0x9b42dad1, SUC_lexicon_tags_15 },
    { 0xe862dad1, SUC_lexicon_tags_49 },
    { 0x7aa2dad2, SUC_lexicon_tags_37 },
    { 0xee2dad4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62dad8, SUC_lexicon_tags_11 },
    { 0xa8a2dad9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe342dae4, SUC_lexicon_tags_15 },
    { 0xb1c2dae5, SUC_lexicon_tags_0 },
    { 0x142dae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe02dae8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2a42daea, SUC_lexicon_tags_1 },
    { 0xaa2daeb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9142daed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5422daf6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xffc2daf8, SUC_lexicon_tags_17 },
    { 0xcbc2daf8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2562dafb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb322dafd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xab42daff, SUC_lexicon_tags_12 },
    { 0x2f02daff, SUC_lexicon_tags_0 },
    { 0x2d62db01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9762db05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c2db0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6962db14, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7c2db16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c2db19, SUC_lexicon_tags_4 },
    { 0x3702db1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf062db1d, SUC_lexicon_tags_15 },
    { 0xa802db1e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e2db22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2db26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x36e2db28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe2db2b, SUC_lexicon_tags_68 },
    { 0xd862db2c, SUC_lexicon_tags_2 },
    { 0x1bc2db2d, SUC_lexicon_tags_47 },
    { 0x8a2db2e, SUC_lexicon_tags_82 },
    { 0x8542db2c, SUC_lexicon_tags_61 },
    { 0xdc42db30, SUC_lexicon_tags_39 },
    { 0x6c02db31, SUC_lexicon_tags_15 },
    { 0xd22db2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c22db34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e22db37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc42db3c, SUC_lexicon_tags_14 },
    { 0x8562db3d, SUC_lexicon_tags_49 },
    { 0xa282db3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d22db44, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc62db4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4922db4e, SUC_lexicon_tags_19 },
    { 0xb842db4e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a2db53, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe222db5b, SUC_lexicon_tags_11 },
    { 0x5542db5b, SUC_lexicon_tags_11 },
    { 0xda22db5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd42db60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd42db62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e2db69, SUC_lexicon_tags_21 },
    { 0x6722db6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf582db6e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa502db73, SUC_lexicon_tags_25 },
    { 0xd5c2db74, SUC_lexicon_tags_2 },
    { 0x4e2db74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1ac2db77, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9662db80, SUC_lexicon_tags_11 },
    { 0xd582db81, SUC_lexicon_tags_17 },
    { 0x8162db82, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe602db86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a62db88, SUC_lexicon_tags_7 },
    { 0xc462db88, SUC_lexicon_tags_17 },
    { 0xe962db88, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a2db8e, SUC_lexicon_tags_167 },
    { 0x3662db8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c2db92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82db95, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb162db99, SUC_lexicon_tags_47 },
    { 0xbe62db9a, SUC_lexicon_tags_1 },
    { 0x662db9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5742db9d, SUC_lexicon_tags_17 },
    { 0xb822db9e, SUC_lexicon_tags_17 },
    { 0x87a2db9f, SUC_lexicon_tags_19 },
    { 0xa2dba0, SUC_lexicon_tags_38 },
    { 0xa7e2dba1, SUC_lexicon_tags_23 },
    { 0xeca2db9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa62dbaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7782dbac, SUC_lexicon_tags_45 },
    { 0x6a02dbad, SUC_lexicon_tags_17 },
    { 0x22a2dbac, SUC_lexicon_tags_1 },
    { 0x9e82dbaf, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a82dbb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1862dbb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a82dbb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe262dbbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c42dbbd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8e22dbbf, SUC_lexicon_tags_13 },
    { 0xc602dbc0, SUC_lexicon_tags_27 },
    { 0x4a62dbc1, SUC_lexicon_tags_25 },
    { 0x3ba2dbc1, SUC_lexicon_tags_3 },
    { 0x22e2dbc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a02dbc7, SUC_lexicon_tags_37 },
    { 0x6a22dbc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63c2dbca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed62dbcc, SUC_lexicon_tags_25 },
    { 0x85a2dbcc, SUC_lexicon_tags_12 },
    { 0x4702dbcd, SUC_lexicon_tags_10 },
    { 0xfe2dbcf, SUC_lexicon_tags_45 },
    { 0xd802dbd0, SUC_lexicon_tags_4 },
    { 0xfe02dbd0, SUC_lexicon_tags_1 },
    { 0x2a2dbd2, SUC_lexicon_tags_55 },
    { 0x5bc2dbd2, SUC_lexicon_tags_25 },
    { 0x2262dbd4, SUC_lexicon_tags_12 },
    { 0x6462dbd5, SUC_lexicon_tags_11 },
    { 0x7c42dbd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb842dbdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec02dbde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac82dbe4, SUC_lexicon_tags_1 },
    { 0x4c22dbe4, SUC_lexicon_tags_23 },
    { 0xcfa2dbe6, SUC_lexicon_tags_25 },
    { 0xdc82dbe5, SUC_lexicon_tags_49 },
    { 0xfca2dbe7, SUC_lexicon_tags_63 },
    { 0xc8e2dbe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc22dbec, SUC_lexicon_tags_43 },
    { 0xa282dbec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f62dbf2, SUC_lexicon_tags_59 },
    { 0x9362dbf3, SUC_lexicon_tags_6 },
    { 0xe562dbf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc922dbf7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc422dbfa, SUC_lexicon_tags_69 },
    { 0x5022dbfb, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c2dbfe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca2dc03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2982dc05, SUC_lexicon_tags_15 },
    { 0x6922dc06, SUC_lexicon_tags_17 },
    { 0xf662dc05, SUC_lexicon_tags_15 },
    { 0xfca2dc08, SUC_lexicon_tags_6 },
    { 0x2922dc07, SUC_lexicon_tags_12 },
    { 0x4f62dc08, SUC_lexicon_tags_48 },
    { 0x6aa2dc07, SUC_lexicon_tags_15 },
    { 0xdb22dc0b, SUC_lexicon_tags_38 },
    { 0xc6a2dc0a, SUC_lexicon_tags_47 },
    { 0x7522dc0e, SUC_lexicon_tags_14 },
    { 0xb582dc0a, SUC_lexicon_tags_2 },
    { 0xe8c2dc0f, SUC_lexicon_tags_19 },
    { 0xa782dc0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02dc14, SUC_lexicon_tags_68 },
    { 0xe922dc14, SUC_lexicon_tags_25 },
    { 0x25a2dc16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a42dc1c, SUC_lexicon_tags_17 },
    { 0xb962dc1d, SUC_lexicon_tags_15 },
    { 0x1da2dc1e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6c22dc20, SUC_lexicon_tags_32 },
    { 0xe222dc20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b82dc24, SUC_lexicon_tags_12 },
    { 0x6a22dc25, SUC_lexicon_tags_17 },
    { 0x5102dc25, SUC_lexicon_tags_17 },
    { 0xe222dc27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2dc2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc822dc2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c2dc3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2dc3e, SUC_lexicon_tags_1 },
    { 0xc8a2dc3e, SUC_lexicon_tags_25 },
    { 0x75c2dc40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee02dc44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf002dc47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4602dc49, SUC_lexicon_tags_25 },
    { 0xad02dc49, SUC_lexicon_tags_2 },
    { 0xcb22dc4b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf0c2dc4d, SUC_lexicon_tags_47 },
    { 0x2262dc4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e2dc51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dc2dc53, SUC_lexicon_tags_17 },
    { 0xf8e2dc53, SUC_lexicon_tags_37 },
    { 0x5b02dc55, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be2dc59, SUC_lexicon_tags_23 },
    { 0xf3c2dc5a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3122dc5d, SUC_lexicon_tags_1 },
    { 0xfc82dc5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9c2dc60, SUC_lexicon_tags_32 },
    { 0xaaa2dc61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa642dc63, SUC_lexicon_tags_12 },
    { 0x402dc63, SUC_lexicon_tags_15 },
    { 0xcea2dc64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e2dc68, SUC_lexicon_tags_6 },
    { 0x1ce2dc68, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2dc6e, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc2dc71, SUC_lexicon_tags_66 },
    { 0x7622dc72, SUC_lexicon_tags_45 },
    { 0x4ca2dc72, SUC_lexicon_tags_46 },
    { 0xdb62dc74, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6222dc76, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e42dc7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b02dc7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3442dc7f, SUC_lexicon_tags_37 },
    { 0x21c2dc80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb82dc82, SUC_lexicon_tags_37 },
    { 0xdfc2dc82, SUC_lexicon_tags_15 },
    { 0x8c62dc84, SUC_lexicon_tags_33 },
    { 0xfb82dc84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf222dc88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff22dc8c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a2dc93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe2dc96, SUC_lexicon_tags_23 },
    { 0x5302dc97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2dc9c, SUC_lexicon_tags_26 },
    { 0xafc2dc9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0a2dc9f, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc2dca2, SUC_lexicon_tags_12 },
    { 0x4582dca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa342dca6, SUC_lexicon_tags_11 },
    { 0x8582dca7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb222dcac, SUC_lexicon_tags_71 },
    { 0xd962dcac, SUC_lexicon_tags_7 },
    { 0x9d42dcad, SUC_lexicon_tags_23 },
    { 0x3282dcaf, SUC_lexicon_tags_47 },
    { 0xcfa2dcac, SUC_lexicon_tags_25 },
    { 0x22a2dcb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1722dcb3, SUC_lexicon_tags_12 },
    { 0xfea2dcb3, SUC_lexicon_tags_11 },
    { 0xa982dcb4, SUC_lexicon_tags_19 },
    { 0x7ca2dcb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x942dcb8, SUC_lexicon_tags_37 },
    { 0x3382dcb9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf682dcbb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3a02dcbd, SUC_lexicon_tags_32 },
    { 0xb562dcbd, SUC_lexicon_tags_15 },
    { 0x7682dcbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22dcc2, SUC_lexicon_tags_18 },
    { 0xa8e2dcc2, SUC_lexicon_tags_30 },
    { 0xb742dcc2, SUC_lexicon_tags_7 },
    { 0xd682dcc4, SUC_lexicon_tags_7 },
    { 0xfcc2dcc6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6542dcc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3a2dcca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3862dccc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba2dcd0, SUC_lexicon_tags_15 },
    { 0xab82dcd1, SUC_lexicon_tags_16 },
    { 0xb3a2dcd0, SUC_lexicon_tags_6 },
    { 0x5c62dcd2, SUC_lexicon_tags_47 },
    { 0x9c02dcd4, SUC_lexicon_tags_11 },
    { 0x42e2dcd5, SUC_lexicon_tags_15 },
    { 0x8402dcd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f42dcd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4422dcdf, SUC_lexicon_tags_2 },
    { 0x3382dce0, SUC_lexicon_tags_11 },
    { 0x5a42dce0, SUC_lexicon_tags_26 },
    { 0xa782dce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa2dcec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1522dcee, SUC_lexicon_tags_61 },
    { 0x2b82dcef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc42dcf2, SUC_lexicon_tags_16 },
    { 0xe382dcf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c2dcf6, SUC_lexicon_tags_27 },
    { 0xefa2dcf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf522dcfd, SUC_lexicon_tags_18 },
    { 0x37e2dcfe, SUC_lexicon_tags_15 },
    { 0xcf42dcfe, SUC_lexicon_tags_8 },
    { 0xfb42dd00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7402dd05, SUC_lexicon_tags_84 },
    { 0x2d22dd06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xce82dd09, SUC_lexicon_tags_17 },
    { 0x81e2dd0a, SUC_lexicon_tags_2 },
    { 0x7142dd0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4422dd0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1302dd10, SUC_lexicon_tags_23 },
    { 0xda22dd10, SUC_lexicon_tags_30 },
    { 0xecc2dd12, SUC_lexicon_tags_37 },
    { 0x522dd13, SUC_lexicon_tags_33 },
    { 0xd602dd13, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc922dd16, SUC_lexicon_tags_21 },
    { 0x2e82dd17, SUC_lexicon_tags_61 },
    { 0x942dd18, SUC_lexicon_tags_35 },
    { 0x2ca2dd19, SUC_lexicon_tags_9 },
    { 0x9102dd19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd402dd1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c2dd1f, SUC_lexicon_tags_10 },
    { 0xe6a2dd20, SUC_lexicon_tags_47 },
    { 0x702dd20, SUC_lexicon_tags_2 },
    { 0x5dc2dd1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9402dd30, SUC_lexicon_tags_43 },
    { 0x51e2dd30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb62dd34, SUC_lexicon_tags_56 },
    { 0x10e2dd35, SUC_lexicon_tags_1 },
    { 0x1ec2dd36, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f02dd3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb602dd42, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3742dd47, SUC_lexicon_tags_37 },
    { 0x68a2dd48, SUC_lexicon_tags_30 },
    { 0x3542dd47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb462dd4e, SUC_lexicon_tags_11 },
    { 0xa7e2dd4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2dd53, SUC_lexicon_tags_55 },
    { 0x27c2dd54, SUC_lexicon_tags_37 },
    { 0xbf22dd55, SUC_lexicon_tags_52 },
    { 0xac02dd56, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9182dd5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7022dd61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x23a2dd63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd782dd66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a62dd68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7442dd6a, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0xd942dd6d, SUC_lexicon_tags_12 },
    { 0x1ee2dd6e, SUC_lexicon_tags_17 },
    { 0xd362dd6f, SUC_lexicon_tags_25 },
    { 0xbe22dd70, SUC_lexicon_tags_84 },
    { 0xb982dd70, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c2dd75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1022dd79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a42dd7d, SUC_lexicon_tags_23 },
    { 0x9862dd7e, SUC_lexicon_tags_17 },
    { 0xa5a2dd7f, SUC_lexicon_tags_15 },
    { 0xae02dd80, SUC_lexicon_tags_12 },
    { 0x3102dd7e, SUC_lexicon_tags_17 },
    { 0x1682dd82, SUC_lexicon_tags_12 },
    { 0x4a22dd83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca02dd86, SUC_lexicon_tags_14 },
    { 0x93e2dd86, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x7062dd8a, SUC_lexicon_tags_19 },
    { 0xbbe2dd8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6162dd8e, SUC_lexicon_tags_37 },
    { 0x4ac2dd8f, SUC_lexicon_tags_33 },
    { 0xa342dd90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c2dd93, SUC_lexicon_tags_35 },
    { 0xeda2dd93, SUC_lexicon_tags_47 },
    { 0x6942dd93, SUC_lexicon_tags_27 },
    { 0x40c2dd96, SUC_lexicon_tags_61 },
    { 0x5ea2dd97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec2dd9b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x182dd9d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2ec2dd9f, SUC_lexicon_tags_38 },
    { 0x8d22dda0, SUC_lexicon_tags_1 },
    { 0xbe82dda0, SUC_lexicon_tags_31 },
    { 0x94e2dda2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb42dda8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf062ddad, SUC_lexicon_tags_3 },
    { 0xbb42ddae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27c2ddb0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x9962ddb2, SUC_lexicon_tags_2 },
    { 0x6682ddb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2ddb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e42ddba, SUC_lexicon_tags_505 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec2ddc2, SUC_lexicon_tags_12 },
    { 0x4de2ddc2, SUC_lexicon_tags_8 },
    { 0x8ba2ddc4, SUC_lexicon_tags_30 },
    { 0xc982ddc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4902ddc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682ddcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7462ddd0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbae2ddd2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8322dddc, SUC_lexicon_tags_16 },
    { 0x35e2dddc, SUC_lexicon_tags_0 },
    { 0xab82ddde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa2dde1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaf02dde3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c2ddea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f22ddef, SUC_lexicon_tags_2 },
    { 0x42c2ddef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee22ddf4, SUC_lexicon_tags_84 },
    { 0x22a2ddf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba2ddf8, SUC_lexicon_tags_19 },
    { 0x5862ddf9, SUC_lexicon_tags_15 },
    { 0xf402ddf9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8262ddfc, SUC_lexicon_tags_14 },
    { 0x9562ddfd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5822ddff, SUC_lexicon_tags_1 },
    { 0x8482de00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2de08, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe902de0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6302de0f, SUC_lexicon_tags_15 },
    { 0x83c2de10, SUC_lexicon_tags_11 },
    { 0x45c2de11, SUC_lexicon_tags_69 },
    { 0x5a22de12, SUC_lexicon_tags_26 },
    { 0x7d22de13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbba2de15, SUC_lexicon_tags_25 },
    { 0x7542de16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd322de1b, SUC_lexicon_tags_11 },
    { 0xd762de1b, SUC_lexicon_tags_60 },
    { 0xa3c2de1d, SUC_lexicon_tags_45 },
    { 0x66e2de1d, SUC_lexicon_tags_15 },
    { 0x1b42de1f, SUC_lexicon_tags_25 },
    { 0x2942de20, SUC_lexicon_tags_34 },
    { 0xc642de21, SUC_lexicon_tags_9 },
    { 0xd562de22, SUC_lexicon_tags_21 },
    { 0x9aa2de1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd022de25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe262de29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c2de2f, SUC_lexicon_tags_77 },
    { 0xaa2de2f, SUC_lexicon_tags_29 },
    { 0xa5e2de31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec2de34, SUC_lexicon_tags_25 },
    { 0xfd02de35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42de39, SUC_lexicon_tags_1 },
    { 0x5a62de39, SUC_lexicon_tags_9 },
    { 0x5502de3b, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd22de42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc662de45, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6cc2de47, SUC_lexicon_tags_6 },
    { 0xb342de48, SUC_lexicon_tags_43 },
    { 0x9182de49, SUC_lexicon_tags_49 },
    { 0x5902de48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c2de50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b82de55, SUC_lexicon_tags_15 },
    { 0xc8e2de56, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xd42de58, SUC_lexicon_tags_12 },
    { 0xd782de59, SUC_lexicon_tags_29 },
    { 0x98a2de5a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xaa82de5c, SUC_lexicon_tags_1 },
    { 0x6922de5c, SUC_lexicon_tags_25 },
    { 0x7dc2de5d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea2de61, SUC_lexicon_tags_31 },
    { 0x8ce2de62, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab02de67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2de6c, SUC_lexicon_tags_38 },
    { 0xcc2de6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ca2de6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5282de71, SUC_lexicon_tags_68 },
    { 0xac62de72, SUC_lexicon_tags_14 },
    { 0x92e2de73, SUC_lexicon_tags_14 },
    { 0x88c2de72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9e2de76, SUC_lexicon_tags_25 },
    { 0x8642de77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a2de7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33a2de7c, SUC_lexicon_tags_12 },
    { 0x35e2de7d, SUC_lexicon_tags_14 },
    { 0x57a2de7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2902de83, SUC_lexicon_tags_37 },
    { 0x7482de84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe2de87, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x1062de8a, SUC_lexicon_tags_23 },
    { 0xcd42de8a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a2de93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa662de95, SUC_lexicon_tags_25 },
    { 0xdf82de96, SUC_lexicon_tags_1 },
    { 0x7102de97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c2de9e, SUC_lexicon_tags_11 },
    { 0xf462de9f, SUC_lexicon_tags_17 },
    { 0x8fe2de9f, SUC_lexicon_tags_15 },
    { 0x5302dea0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9b42dea3, SUC_lexicon_tags_26 },
    { 0xf002dea3, SUC_lexicon_tags_4 },
    { 0x4c22dea5, SUC_lexicon_tags_31 },
    { 0x9762dea6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6602dea9, SUC_lexicon_tags_14 },
    { 0x2d42deaa, SUC_lexicon_tags_1 },
    { 0xf842dea9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x39c2dead, SUC_lexicon_tags_25 },
    { 0xb8a2dead, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3162deb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2182deb5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd502deb9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14e2debb, SUC_lexicon_tags_17 },
    { 0xfac2debb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6722debe, SUC_lexicon_tags_28 },
    { 0xce2debf, SUC_lexicon_tags_5 },
    { 0xdec2debe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7462dec2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c02dec5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf82dec8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e2decb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3062ded2, SUC_lexicon_tags_15 },
    { 0xe0e2ded3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca2dedc, SUC_lexicon_tags_9 },
    { 0xe422dedd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf82dedf, SUC_lexicon_tags_28 },
    { 0x47c2dee0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb322dee7, SUC_lexicon_tags_37 },
    { 0xe7c2dee8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae2deed, SUC_lexicon_tags_514 },
    { 0x95a2deed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f42def1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9762def3, SUC_lexicon_tags_63 },
    { 0xcd82def4, SUC_lexicon_tags_0 },
    { 0xae82def4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20a2def7, SUC_lexicon_tags_8 },
    { 0xe082def8, SUC_lexicon_tags_12 },
    { 0xb442def8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1102deff, SUC_lexicon_tags_57 },
    { 0x5042deff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee02df03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6202df08, SUC_lexicon_tags_15 },
    { 0x9b82df09, SUC_lexicon_tags_2 },
    { 0x1162df0a, SUC_lexicon_tags_15 },
    { 0xc162df0b, SUC_lexicon_tags_12 },
    { 0xc842df09, SUC_lexicon_tags_87 },
    { 0xd3a2df0d, SUC_lexicon_tags_45 },
    { 0x37a2df0e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6c62df10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac2df13, SUC_lexicon_tags_11 },
    { 0xbca2df14, SUC_lexicon_tags_12 },
    { 0x6ae2df15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e62df18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e2df1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b62df20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe502df23, SUC_lexicon_tags_66 },
    { 0xd8c2df23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc562df28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2182df2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa2df2f, SUC_lexicon_tags_10 },
    { 0xd42df2f, SUC_lexicon_tags_15 },
    { 0x4322df2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa302df38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e2df3c, SUC_lexicon_tags_77 },
    { 0xb6e2df3d, SUC_lexicon_tags_8 },
    { 0x4562df3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8fc2df40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11e2df42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae22df46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb782df48, SUC_lexicon_tags_37 },
    { 0xe7c2df48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e2df4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9842df51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c2df54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd382df56, SUC_lexicon_tags_1 },
    { 0x6422df57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d42df59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a02df5b, SUC_lexicon_tags_7 },
    { 0xc362df5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee02df5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea2df66, SUC_lexicon_tags_13 },
    { 0x9b42df67, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea02df69, SUC_lexicon_tags_47 },
    { 0x4942df69, SUC_lexicon_tags_2 },
    { 0xb702df6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc2df6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec22df76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd42df7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb62df7d, SUC_lexicon_tags_13 },
    { 0x7b22df7e, SUC_lexicon_tags_1 },
    { 0x1862df7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc2df8b, SUC_lexicon_tags_16 },
    { 0xcca2df8b, SUC_lexicon_tags_8 },
    { 0x6042df8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac2df91, SUC_lexicon_tags_47 },
    { 0xfe42df92, SUC_lexicon_tags_17 },
    { 0xf1a2df93, SUC_lexicon_tags_19 },
    { 0x5802df92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2df97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1402df9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5422dfa0, SUC_lexicon_tags_15 },
    { 0x94a2dfa0, SUC_lexicon_tags_37 },
    { 0xc962dfa2, SUC_lexicon_tags_31 },
    { 0x5042dfa3, SUC_lexicon_tags_14 },
    { 0xfb42dfa2, SUC_lexicon_tags_22 },
    { 0xe502dfa5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9b62dfa7, SUC_lexicon_tags_0 },
    { 0x8d42dfa7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c2dfab, SUC_lexicon_tags_43 },
    { 0x8942dfac, SUC_lexicon_tags_15 },
    { 0xe922dfac, SUC_lexicon_tags_46 },
    { 0xf042dfab, SUC_lexicon_tags_31 },
    { 0xb1a2dfaf, SUC_lexicon_tags_15 },
    { 0xccc2dfaf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc2dfb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2e22dfb5, SUC_lexicon_tags_32 },
    { 0xfa62dfb6, SUC_lexicon_tags_25 },
    { 0xbe42dfb7, SUC_lexicon_tags_229 },
    { 0x68e2dfb5, SUC_lexicon_tags_0 },
    { 0x51e2dfb7, SUC_lexicon_tags_26 },
    { 0xa142dfb7, SUC_lexicon_tags_47 },
    { 0x5e82dfbb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b42dfc0, SUC_lexicon_tags_25 },
    { 0x2122dfc0, SUC_lexicon_tags_15 },
    { 0x9f62dfc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b22dfc4, SUC_lexicon_tags_12 },
    { 0xc202dfc5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe562dfc7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a62dfcb, SUC_lexicon_tags_34 },
    { 0x1c22dfcb, SUC_lexicon_tags_47 },
    { 0xb462dfcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9642dfd0, SUC_lexicon_tags_48 },
    { 0x8222dfd1, SUC_lexicon_tags_17 },
    { 0xb902dfd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c2dfd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcc2dfd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90a2dfda, SUC_lexicon_tags_14 },
    { 0x1422dfda, SUC_lexicon_tags_60 },
    { 0x5362dfdc, SUC_lexicon_tags_19 },
    { 0xdda2dfdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf402dfe0, SUC_lexicon_tags_15 },
    { 0x51c2dfe1, SUC_lexicon_tags_17 },
    { 0x10e2dfe2, SUC_lexicon_tags_33 },
    { 0x3b82dfe3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf042dfe5, SUC_lexicon_tags_7 },
    { 0x2682dfe6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa982dfea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa582dfec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8582dff0, SUC_lexicon_tags_7 },
    { 0x7042dff1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ce2dff3, SUC_lexicon_tags_6 },
    { 0xebe2dff3, SUC_lexicon_tags_17 },
    { 0x5d82dff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf42dff7, SUC_lexicon_tags_37 },
    { 0xd982dff7, SUC_lexicon_tags_2 },
    { 0xe062dff9, SUC_lexicon_tags_43 },
    { 0x20c2dffa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb782dffd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd782dfff, SUC_lexicon_tags_1 },
    { 0x26c2dfff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2e006, SUC_lexicon_tags_11 },
    { 0xe5a2e007, SUC_lexicon_tags_15 },
    { 0x13a2e008, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e2e00c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8e2e00e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ae2e010, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6762e013, SUC_lexicon_tags_25 },
    { 0x26c2e013, SUC_lexicon_tags_12 },
    { 0xf582e015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa402e017, SUC_lexicon_tags_33 },
    { 0x1aa2e017, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a2e01d, SUC_lexicon_tags_37 },
    { 0x24c2e01e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea2e026, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd42e028, SUC_lexicon_tags_29 },
    { 0x1922e028, SUC_lexicon_tags_21 },
    { 0x9282e029, SUC_lexicon_tags_21 },
    { 0x3262e02b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef02e032, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7422e039, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa22e03b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3582e041, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa142e043, SUC_lexicon_tags_43 },
    { 0x6102e043, SUC_lexicon_tags_63 },
    { 0x84e2e045, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122e04e, SUC_lexicon_tags_7 },
    { 0x9d82e04e, SUC_lexicon_tags_8 },
    { 0x2aa2e050, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a2e053, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e2e056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c2e05b, SUC_lexicon_tags_15 },
    { 0x3242e05c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b62e060, SUC_lexicon_tags_12 },
    { 0xe662e061, SUC_lexicon_tags_46 },
    { 0xad62e062, SUC_lexicon_tags_17 },
    { 0xd562e062, SUC_lexicon_tags_12 },
    { 0x83e2e064, SUC_lexicon_tags_7 },
    { 0xbf02e064, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xab62e067, SUC_lexicon_tags_0 },
    { 0xbc42e068, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x83c2e06a, SUC_lexicon_tags_17 },
    { 0x3b82e06b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a2e071, SUC_lexicon_tags_23 },
    { 0x6182e071, SUC_lexicon_tags_29 },
    { 0x67e2e073, SUC_lexicon_tags_60 },
    { 0xf102e073, SUC_lexicon_tags_17 },
    { 0x4ec2e074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c2e078, SUC_lexicon_tags_4 },
    { 0xeae2e078, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6822e07d, SUC_lexicon_tags_30 },
    { 0xbc42e07d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5802e080, SUC_lexicon_tags_23 },
    { 0x8ec2e081, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6822e089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6642e08c, SUC_lexicon_tags_14 },
    { 0x9e62e08d, SUC_lexicon_tags_3 },
    { 0x8f62e08e, SUC_lexicon_tags_4 },
    { 0xfa82e08f, SUC_lexicon_tags_5 },
    { 0x9302e08e, SUC_lexicon_tags_2 },
    { 0x2042e091, SUC_lexicon_tags_2 },
    { 0xda82e08f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1522e095, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd82e097, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd782e09a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c2e09e, SUC_lexicon_tags_29 },
    { 0x69e2e09e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33c2e0a1, SUC_lexicon_tags_8 },
    { 0x3042e0a2, SUC_lexicon_tags_14 },
    { 0x8b62e0a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4362e0a5, SUC_lexicon_tags_17 },
    { 0xf9c2e0a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7202e0a8, SUC_lexicon_tags_2 },
    { 0xedc2e0a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2242e0ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f82e0af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee02e0b4, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0xb702e0b7, SUC_lexicon_tags_0 },
    { 0xb122e0b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1be2e0ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3da2e0bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ca2e0be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2e0c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee2e0c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f02e0d0, SUC_lexicon_tags_15 },
    { 0x5e42e0d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xce42e0d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2862e0d7, SUC_lexicon_tags_49 },
    { 0xe782e0d8, SUC_lexicon_tags_14 },
    { 0x8b82e0d9, SUC_lexicon_tags_25 },
    { 0x3302e0da, SUC_lexicon_tags_11 },
    { 0x54c2e0da, SUC_lexicon_tags_63 },
    { 0xa2e0dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24c2e0de, SUC_lexicon_tags_15 },
    { 0x32c2e0df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbf82e0e1, SUC_lexicon_tags_25 },
    { 0x6cc2e0e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec2e0e6, SUC_lexicon_tags_37 },
    { 0xaa02e0e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da2e0ec, SUC_lexicon_tags_12 },
    { 0x75a2e0ed, SUC_lexicon_tags_2 },
    { 0x4382e0ee, SUC_lexicon_tags_37 },
    { 0x5e2e0ee, SUC_lexicon_tags_17 },
    { 0x1842e0f0, SUC_lexicon_tags_19 },
    { 0x99a2e0f1, SUC_lexicon_tags_7 },
    { 0xd262e0f2, SUC_lexicon_tags_25 },
    { 0x61c2e0f3, SUC_lexicon_tags_25 },
    { 0x95e2e0f2, SUC_lexicon_tags_2 },
    { 0x67c2e0f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a2e0fc, SUC_lexicon_tags_51 },
    { 0xef82e0fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5742e0ff, SUC_lexicon_tags_27 },
    { 0x2242e100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7542e106, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2182e108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3362e10a, SUC_lexicon_tags_14 },
    { 0xaf62e10b, SUC_lexicon_tags_44 },
    { 0xfe42e10c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf162e10f, SUC_lexicon_tags_31 },
    { 0xc262e110, SUC_lexicon_tags_37 },
    { 0x1b42e10f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e2e116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c82e11c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2e11f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c2e123, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c2e127, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e2e12f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9102e131, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c2e136, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4522e13c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef22e13e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa02e140, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b82e144, SUC_lexicon_tags_57 },
    { 0xd782e144, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9da2e147, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4102e14b, SUC_lexicon_tags_11 },
    { 0xd1a2e14c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa842e14e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3162e153, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x8902e155, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4562e158, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2762e15b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e2e161, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2e2e163, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba2e166, SUC_lexicon_tags_48 },
    { 0xd442e167, SUC_lexicon_tags_11 },
    { 0x8b02e167, SUC_lexicon_tags_59 },
    { 0xf782e167, SUC_lexicon_tags_2 },
    { 0x1262e16a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd502e170, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9282e173, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca2e176, SUC_lexicon_tags_22 },
    { 0x2922e177, SUC_lexicon_tags_27 },
    { 0xb102e176, SUC_lexicon_tags_29 },
    { 0x24a2e179, SUC_lexicon_tags_37 },
    { 0x8762e179, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbc2e17c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c62e183, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd182e185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e82e188, SUC_lexicon_tags_45 },
    { 0x32e2e188, SUC_lexicon_tags_1 },
    { 0x7182e18a, SUC_lexicon_tags_138 },
    { 0xcc62e18b, SUC_lexicon_tags_55 },
    { 0xf582e18b, SUC_lexicon_tags_47 },
    { 0x9282e18a, SUC_lexicon_tags_43 },
    { 0x3a02e18d, SUC_lexicon_tags_49 },
    { 0xb6a2e18f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a82e195, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6722e198, SUC_lexicon_tags_25 },
    { 0x4ca2e198, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf002e1a1, SUC_lexicon_tags_14 },
    { 0xeaa2e1a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfa2e1a4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b22e1a9, SUC_lexicon_tags_14 },
    { 0x742e1aa, SUC_lexicon_tags_66 },
    { 0xcc2e1ab, SUC_lexicon_tags_15 },
    { 0xae02e1a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a2e1af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5242e1b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2e1ba, SUC_lexicon_tags_17 },
    { 0xc5e2e1ba, SUC_lexicon_tags_7 },
    { 0xc142e1bc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e2e1bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e2e1c4, SUC_lexicon_tags_47 },
    { 0xcaa2e1c4, SUC_lexicon_tags_17 },
    { 0x98c2e1c5, SUC_lexicon_tags_37 },
    { 0x9c82e1c7, SUC_lexicon_tags_19 },
    { 0xb9e2e1c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c2e1cf, SUC_lexicon_tags_61 },
    { 0xdb22e1d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9142e1d2, SUC_lexicon_tags_0 },
    { 0x6f02e1d3, SUC_lexicon_tags_17 },
    { 0xdd62e1d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a02e1d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4002e1d9, SUC_lexicon_tags_25 },
    { 0x3022e1da, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc22e1dd, SUC_lexicon_tags_23 },
    { 0xea2e1de, SUC_lexicon_tags_28 },
    { 0x202e1df, SUC_lexicon_tags_49 },
    { 0xa4e2e1df, SUC_lexicon_tags_11 },
    { 0xb082e1e1, SUC_lexicon_tags_1 },
    { 0x5482e1e2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e2e1e7, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x9562e1e9, SUC_lexicon_tags_6 },
    { 0xc3a2e1ea, SUC_lexicon_tags_33 },
    { 0x3a82e1eb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf082e1ed, SUC_lexicon_tags_37 },
    { 0xbb22e1ee, SUC_lexicon_tags_1 },
    { 0xa162e1ee, SUC_lexicon_tags_19 },
    { 0x6da2e1f0, SUC_lexicon_tags_25 },
    { 0x7b42e1f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a2e1f4, SUC_lexicon_tags_1 },
    { 0xb522e1f4, SUC_lexicon_tags_57 },
    { 0x9962e1f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1562e1fc, SUC_lexicon_tags_1 },
    { 0x2e82e1fd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc02e202, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a2e206, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e82e20a, SUC_lexicon_tags_25 },
    { 0x8d42e20a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xede2e20e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982e214, SUC_lexicon_tags_15 },
    { 0xc182e215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d42e218, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x42a2e21a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a82e21e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2702e226, SUC_lexicon_tags_29 },
    { 0x3282e227, SUC_lexicon_tags_66 },
    { 0x12c2e226, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a2e22b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5802e22f, SUC_lexicon_tags_48 },
    { 0x3ec2e22f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf162e232, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f02e235, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9022e237, SUC_lexicon_tags_3 },
    { 0xf4e2e238, SUC_lexicon_tags_25 },
    { 0x43a2e238, SUC_lexicon_tags_14 },
    { 0x7e82e237, SUC_lexicon_tags_37 },
    { 0x4082e23b, SUC_lexicon_tags_1 },
    { 0xd8a2e238, SUC_lexicon_tags_61 },
    { 0x4262e23c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe422e241, SUC_lexicon_tags_1 },
    { 0x7c02e241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42e246, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x96c2e248, SUC_lexicon_tags_125 },
    { 0xa922e249, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xade2e24b, SUC_lexicon_tags_17 },
    { 0x3502e24c, SUC_lexicon_tags_15 },
    { 0x1e42e24d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc2e254, SUC_lexicon_tags_31 },
    { 0x5502e255, SUC_lexicon_tags_43 },
    { 0x96a2e256, SUC_lexicon_tags_33 },
    { 0x52c2e257, SUC_lexicon_tags_7 },
    { 0x9702e257, SUC_lexicon_tags_38 },
    { 0xab82e259, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa022e25b, SUC_lexicon_tags_11 },
    { 0x42a2e25c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc862e25f, SUC_lexicon_tags_9 },
    { 0xe202e260, SUC_lexicon_tags_13 },
    { 0xedc2e261, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf9e2e263, SUC_lexicon_tags_6 },
    { 0x6622e263, SUC_lexicon_tags_37 },
    { 0xeca2e265, SUC_lexicon_tags_113 },
    { 0xb362e266, SUC_lexicon_tags_17 },
    { 0xe982e265, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d62e26c, SUC_lexicon_tags_23 },
    { 0x5ba2e26d, SUC_lexicon_tags_43 },
    { 0x1442e26e, SUC_lexicon_tags_26 },
    { 0xdde2e26c, SUC_lexicon_tags_17 },
    { 0xaae2e270, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4682e275, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd02e277, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc142e27a, SUC_lexicon_tags_48 },
    { 0xfac2e27a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb22e280, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce2e283, SUC_lexicon_tags_17 },
    { 0x142e284, SUC_lexicon_tags_29 },
    { 0x7982e285, SUC_lexicon_tags_17 },
    { 0x4f82e284, SUC_lexicon_tags_37 },
    { 0x7b02e285, SUC_lexicon_tags_37 },
    { 0xdd62e288, SUC_lexicon_tags_1 },
    { 0xc6e2e288, SUC_lexicon_tags_19 },
    { 0x2362e28a, SUC_lexicon_tags_19 },
    { 0x4bc2e28b, SUC_lexicon_tags_2 },
    { 0x5602e28b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3862e28e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe222e294, SUC_lexicon_tags_6 },
    { 0x422e294, SUC_lexicon_tags_56 },
    { 0x7882e294, SUC_lexicon_tags_35 },
    { 0x1fe2e297, SUC_lexicon_tags_14 },
    { 0x4242e298, SUC_lexicon_tags_2 },
    { 0xa22e299, SUC_lexicon_tags_45 },
    { 0xb602e298, SUC_lexicon_tags_63 },
    { 0xf5e2e29b, SUC_lexicon_tags_61 },
    { 0x9ea2e299, SUC_lexicon_tags_2 },
    { 0xbd82e295, SUC_lexicon_tags_60 },
    { 0x7942e296, SUC_lexicon_tags_107 },
    { 0xaa02e29f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x46a2e2a1, SUC_lexicon_tags_99 },
    { 0x8282e2a2, SUC_lexicon_tags_2 },
    { 0x1082e2a2, SUC_lexicon_tags_15 },
    { 0x6d42e2a1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf5e2e2a6, SUC_lexicon_tags_19 },
    { 0xdf02e2a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc2e2ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9402e2ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2442e2af, SUC_lexicon_tags_61 },
    { 0xa102e2b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c02e2b2, SUC_lexicon_tags_108 },
    { 0x8b02e2b2, SUC_lexicon_tags_12 },
    { 0x7302e2b4, SUC_lexicon_tags_224 },
    { 0x5b62e2b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac2e2bf, SUC_lexicon_tags_39 },
    { 0x36e2e2c0, SUC_lexicon_tags_26 },
    { 0x4c22e2bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b02e2c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8322e2c8, SUC_lexicon_tags_25 },
    { 0xebc2e2c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xad02e2cb, SUC_lexicon_tags_2 },
    { 0xaf02e2cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c2e2cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd302e2d1, SUC_lexicon_tags_56 },
    { 0xdc62e2d2, SUC_lexicon_tags_25 },
    { 0xc982e2d3, SUC_lexicon_tags_25 },
    { 0x57c2e2d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c2e2d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ba2e2da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bc2e2dc, SUC_lexicon_tags_9 },
    { 0xa82e2dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa462e2df, SUC_lexicon_tags_2 },
    { 0xae42e2e0, SUC_lexicon_tags_25 },
    { 0x882e2e1, SUC_lexicon_tags_1 },
    { 0x6242e2e1, SUC_lexicon_tags_1 },
    { 0xf562e2df, SUC_lexicon_tags_45 },
    { 0xf562e2e4, SUC_lexicon_tags_22 },
    { 0xf262e2e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c22e2e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc22e2ee, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb62e2f4, SUC_lexicon_tags_26 },
    { 0x6ce2e2f5, SUC_lexicon_tags_46 },
    { 0x2242e2f6, SUC_lexicon_tags_31 },
    { 0x5022e2f7, SUC_lexicon_tags_47 },
    { 0xefe2e2f8, SUC_lexicon_tags_37 },
    { 0x6122e2f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf7a2e2fb, SUC_lexicon_tags_37 },
    { 0x3782e2fc, SUC_lexicon_tags_6 },
    { 0xb402e2fd, SUC_lexicon_tags_7 },
    { 0xd902e2fe, SUC_lexicon_tags_68 },
    { 0xf562e2fd, SUC_lexicon_tags_38 },
    { 0x1a82e300, SUC_lexicon_tags_1 },
    { 0x95e2e301, SUC_lexicon_tags_37 },
    { 0x7022e300, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcae2e304, SUC_lexicon_tags_47 },
    { 0x17a2e305, SUC_lexicon_tags_7 },
    { 0x6522e306, SUC_lexicon_tags_1356 },
    { 0x8e02e307, SUC_lexicon_tags_31 },
    { 0x4822e306, SUC_lexicon_tags_25 },
    { 0x5042e309, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea2e30c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc62e30f, SUC_lexicon_tags_49 },
    { 0x8442e310, SUC_lexicon_tags_159 },
    { 0x80a2e30f, SUC_lexicon_tags_55 },
    { 0xac22e312, SUC_lexicon_tags_9 },
    { 0xf222e313, SUC_lexicon_tags_45 },
    { 0xe9a2e314, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce2e31a, SUC_lexicon_tags_7 },
    { 0x40c2e31a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2e2e31d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae02e31f, SUC_lexicon_tags_30 },
    { 0x7222e320, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3742e322, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c2e325, SUC_lexicon_tags_7 },
    { 0xa282e326, SUC_lexicon_tags_14 },
    { 0x1402e327, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd162e329, SUC_lexicon_tags_37 },
    { 0xeac2e329, SUC_lexicon_tags_1 },
    { 0xa8a2e32b, SUC_lexicon_tags_27 },
    { 0xdd02e32c, SUC_lexicon_tags_60 },
    { 0xe2e329, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82e330, SUC_lexicon_tags_23 },
    { 0x5662e330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb842e333, SUC_lexicon_tags_63 },
    { 0x5b42e334, SUC_lexicon_tags_37 },
    { 0xed82e335, SUC_lexicon_tags_11 },
    { 0x1962e336, SUC_lexicon_tags_250 },
    { 0x77c2e337, SUC_lexicon_tags_656 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec2e33d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe2e340, SUC_lexicon_tags_23 },
    { 0xbfe2e340, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1ec2e343, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc2e346, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8422e34a, SUC_lexicon_tags_21 },
    { 0xdc42e34a, SUC_lexicon_tags_0 },
    { 0x1482e34b, SUC_lexicon_tags_63 },
    { 0x33e2e34d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1162e350, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc602e352, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x86e2e354, SUC_lexicon_tags_14 },
    { 0xdfa2e355, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4802e35b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe82e362, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae22e365, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4922e369, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefc2e36b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5e2e36d, SUC_lexicon_tags_2 },
    { 0xc442e36d, SUC_lexicon_tags_17 },
    { 0x47c2e36f, SUC_lexicon_tags_5 },
    { 0x8602e36e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9862e372, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe2e377, SUC_lexicon_tags_49 },
    { 0xeb02e378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e2e37f, SUC_lexicon_tags_4 },
    { 0xacc2e380, SUC_lexicon_tags_151 },
    { 0xf502e381, SUC_lexicon_tags_28 },
    { 0x82a2e381, SUC_lexicon_tags_29 },
    { 0xb622e381, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x4962e386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd02e388, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c82e38f, SUC_lexicon_tags_0 },
    { 0xa122e390, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc62e392, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e82e398, SUC_lexicon_tags_12 },
    { 0xca22e399, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e2e3a1, SUC_lexicon_tags_2 },
    { 0x9482e3a2, SUC_lexicon_tags_37 },
    { 0xce42e3a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac62e3a6, SUC_lexicon_tags_25 },
    { 0x4922e3a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7e82e3a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c2e3af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd62e3b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88e2e3b3, SUC_lexicon_tags_30 },
    { 0x6862e3b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7822e3b6, SUC_lexicon_tags_7 },
    { 0xbfc2e3b7, SUC_lexicon_tags_1207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf822e3bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcc02e3be, SUC_lexicon_tags_1 },
    { 0x7382e3bf, SUC_lexicon_tags_30 },
    { 0xad02e3bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2e3c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb002e3c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc2e3ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e2e3d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2e3d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1482e3dc, SUC_lexicon_tags_46 },
    { 0xfe82e3dc, SUC_lexicon_tags_32 },
    { 0xb442e3de, SUC_lexicon_tags_87 },
    { 0xd4c2e3dd, SUC_lexicon_tags_134 },
    { 0x64e2e3dc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x61a2e3e2, SUC_lexicon_tags_11 },
    { 0x98c2e3e3, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2e3e6, SUC_lexicon_tags_15 },
    { 0x44c2e3e7, SUC_lexicon_tags_63 },
    { 0x2682e3e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77c2e3ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48c2e3ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9dc2e3ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e2e3f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3362e3fa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9ec2e3fc, SUC_lexicon_tags_17 },
    { 0xd202e3fd, SUC_lexicon_tags_87 },
    { 0xbea2e3fe, SUC_lexicon_tags_7 },
    { 0x4042e3ff, SUC_lexicon_tags_2 },
    { 0xa922e400, SUC_lexicon_tags_11 },
    { 0xfa82e400, SUC_lexicon_tags_16 },
    { 0x8c62e402, SUC_lexicon_tags_12 },
    { 0x9ea2e3fd, SUC_lexicon_tags_0 },
    { 0x2b42e3ff, SUC_lexicon_tags_2 },
    { 0xe82e405, SUC_lexicon_tags_32 },
    { 0x9b22e406, SUC_lexicon_tags_7 },
    { 0xfd82e406, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f82e409, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e62e412, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xac82e414, SUC_lexicon_tags_14 },
    { 0x90a2e415, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbc22e417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac2e41a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c2e41e, SUC_lexicon_tags_16 },
    { 0x3c82e41e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9e2e421, SUC_lexicon_tags_14 },
    { 0xf642e422, SUC_lexicon_tags_17 },
    { 0xcee2e422, SUC_lexicon_tags_0 },
    { 0x7062e424, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd522e427, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc662e429, SUC_lexicon_tags_33 },
    { 0x85e2e42a, SUC_lexicon_tags_14 },
    { 0xf542e42a, SUC_lexicon_tags_60 },
    { 0x5d82e42c, SUC_lexicon_tags_118 },
    { 0xa0e2e429, SUC_lexicon_tags_8 },
    { 0xa1c2e42e, SUC_lexicon_tags_17 },
    { 0xc82e42f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c2e434, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8602e437, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f02e43b, SUC_lexicon_tags_37 },
    { 0xdaa2e43c, SUC_lexicon_tags_37 },
    { 0x2002e43d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc2e440, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2f62e442, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a22e445, SUC_lexicon_tags_2 },
    { 0x5702e445, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa622e44a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b22e44e, SUC_lexicon_tags_30 },
    { 0xaf62e44e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c2e452, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb62e454, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6402e458, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb702e45d, SUC_lexicon_tags_17 },
    { 0xef82e45e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c2e461, SUC_lexicon_tags_3 },
    { 0xa02e462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a2e465, SUC_lexicon_tags_31 },
    { 0x8062e466, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4962e468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c02e46a, SUC_lexicon_tags_45 },
    { 0xe122e46a, SUC_lexicon_tags_12 },
    { 0x582e46c, SUC_lexicon_tags_45 },
    { 0x1f62e46d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a2e470, SUC_lexicon_tags_2 },
    { 0x922e471, SUC_lexicon_tags_2 },
    { 0xd902e470, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e2e475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1182e477, SUC_lexicon_tags_2 },
    { 0x68a2e477, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1922e47c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be2e484, SUC_lexicon_tags_32 },
    { 0x4222e484, SUC_lexicon_tags_9 },
    { 0x1da2e484, SUC_lexicon_tags_100 },
    { 0x1b22e487, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa82e48d, SUC_lexicon_tags_8 },
    { 0x32a2e48d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfe82e490, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x4c02e492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c2e497, SUC_lexicon_tags_10 },
    { 0xaaa2e497, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48c2e49a, SUC_lexicon_tags_49 },
    { 0xbf42e49b, SUC_lexicon_tags_5 },
    { 0x6a82e49b, SUC_lexicon_tags_25 },
    { 0xed62e49a, SUC_lexicon_tags_10 },
    { 0xe042e49d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8842e4a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e2e4a4, SUC_lexicon_tags_11 },
    { 0x9942e4a5, SUC_lexicon_tags_25 },
    { 0xebc2e4a6, SUC_lexicon_tags_12 },
    { 0x9562e4a7, SUC_lexicon_tags_12 },
    { 0x4f82e4a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9162e4aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8542e4ac, SUC_lexicon_tags_38 },
    { 0x2622e4ad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb42e4bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1542e4c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f82e4c8, SUC_lexicon_tags_7 },
    { 0x24e2e4c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a82e4cd, SUC_lexicon_tags_11 },
    { 0x38c2e4ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a2e4d4, SUC_lexicon_tags_1 },
    { 0xb242e4d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd522e4d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0c2e4db, SUC_lexicon_tags_38 },
    { 0xec42e4db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c2e4df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c2e4e2, SUC_lexicon_tags_12 },
    { 0xd742e4e3, SUC_lexicon_tags_1 },
    { 0xc342e4e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f42e4e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a2e4ec, SUC_lexicon_tags_29 },
    { 0xa3a2e4ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa442e4f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe562e4f7, SUC_lexicon_tags_15 },
    { 0x5662e4f8, SUC_lexicon_tags_14 },
    { 0xc82e4f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d22e4fb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc842e501, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6082e503, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a02e508, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9682e50b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf802e50e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cc2e510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3542e512, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a2e515, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xbf82e517, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84c2e519, SUC_lexicon_tags_49 },
    { 0xfa62e51a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x96c2e51c, SUC_lexicon_tags_33 },
    { 0x1282e51d, SUC_lexicon_tags_49 },
    { 0x6f82e51c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2322e521, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba62e525, SUC_lexicon_tags_32 },
    { 0x3fe2e526, SUC_lexicon_tags_32 },
    { 0x29e2e526, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2e529, SUC_lexicon_tags_14 },
    { 0x3542e52a, SUC_lexicon_tags_30 },
    { 0xcbc2e52b, SUC_lexicon_tags_0 },
    { 0x63a2e52b, SUC_lexicon_tags_37 },
    { 0xa362e52a, SUC_lexicon_tags_7 },
    { 0x2b62e52b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac62e534, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xa962e537, SUC_lexicon_tags_37 },
    { 0x8b02e538, SUC_lexicon_tags_63 },
    { 0x9482e537, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4882e53b, SUC_lexicon_tags_38 },
    { 0x96c2e53c, SUC_lexicon_tags_53 },
    { 0x4622e53b, SUC_lexicon_tags_8 },
    { 0x4882e53e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc162e546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62e54b, SUC_lexicon_tags_7 },
    { 0x1aa2e54c, SUC_lexicon_tags_1 },
    { 0x5d02e54d, SUC_lexicon_tags_17 },
    { 0x97e2e54b, SUC_lexicon_tags_7 },
    { 0x2dc2e54d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1c2e551, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xd1e2e553, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x462e555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8c2e557, SUC_lexicon_tags_30 },
    { 0x1bc2e558, SUC_lexicon_tags_26 },
    { 0x9902e559, SUC_lexicon_tags_7 },
    { 0x6362e559, SUC_lexicon_tags_11 },
    { 0xdac2e557, SUC_lexicon_tags_45 },
    { 0x4022e558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac42e55e, SUC_lexicon_tags_45 },
    { 0xc302e55f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa2e565, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa562e567, SUC_lexicon_tags_69 },
    { 0x2102e567, SUC_lexicon_tags_63 },
    { 0xe482e568, SUC_lexicon_tags_66 },
    { 0x9fa2e56a, SUC_lexicon_tags_2 },
    { 0xc02e567, SUC_lexicon_tags_14 },
    { 0x8fe2e56b, SUC_lexicon_tags_2 },
    { 0x5cc2e569, SUC_lexicon_tags_2 },
    { 0xb022e56e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5782e570, SUC_lexicon_tags_60 },
    { 0x9742e571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6982e573, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c02e576, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa22e579, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c2e57d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62e583, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb422e585, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b62e589, SUC_lexicon_tags_8 },
    { 0x7d02e58a, SUC_lexicon_tags_7 },
    { 0x21c2e58b, SUC_lexicon_tags_11 },
    { 0x8c42e58a, SUC_lexicon_tags_49 },
    { 0xaf22e58d, SUC_lexicon_tags_7 },
    { 0x2242e58e, SUC_lexicon_tags_37 },
    { 0x9442e58e, SUC_lexicon_tags_18 },
    { 0xb862e590, SUC_lexicon_tags_7 },
    { 0xc4e2e591, SUC_lexicon_tags_15 },
    { 0xcb02e592, SUC_lexicon_tags_33 },
    { 0xac22e591, SUC_lexicon_tags_7 },
    { 0x6b02e594, SUC_lexicon_tags_61 },
    { 0xa62e595, SUC_lexicon_tags_49 },
    { 0xff22e595, SUC_lexicon_tags_63 },
    { 0x5ce2e592, SUC_lexicon_tags_46 },
    { 0x64e2e58f, SUC_lexicon_tags_23 },
    { 0x8a2e58b, SUC_lexicon_tags_37 },
    { 0x7b42e59a, SUC_lexicon_tags_28 },
    { 0x6562e59a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7402e59e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e2e5a1, SUC_lexicon_tags_2 },
    { 0x3942e5a2, SUC_lexicon_tags_38 },
    { 0x9882e5a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da2e5a8, SUC_lexicon_tags_14 },
    { 0x5da2e5a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe2e5b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b22e5b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6a2e5b6, SUC_lexicon_tags_38 },
    { 0x49a2e5b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c2e5bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6382e5bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6022e5c3, SUC_lexicon_tags_19 },
    { 0x2922e5c3, SUC_lexicon_tags_60 },
    { 0xc722e5c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8282e5c8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2922e5ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e2e5cd, SUC_lexicon_tags_11 },
    { 0xb122e5ce, SUC_lexicon_tags_9 },
    { 0x3ca2e5ce, SUC_lexicon_tags_31 },
    { 0xc642e5d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882e5d4, SUC_lexicon_tags_37 },
    { 0x8602e5d4, SUC_lexicon_tags_26 },
    { 0xe162e5d5, SUC_lexicon_tags_25 },
    { 0x6202e5d6, SUC_lexicon_tags_11 },
    { 0x1182e5d8, SUC_lexicon_tags_1 },
    { 0x2822e5d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xec62e5db, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c82e5de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1362e5e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfc2e5ea, SUC_lexicon_tags_6 },
    { 0xd642e5eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e2e5ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de2e5f2, SUC_lexicon_tags_2 },
    { 0x2222e5f2, SUC_lexicon_tags_1 },
    { 0xd742e5f2, SUC_lexicon_tags_38 },
    { 0x9982e5f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb002e5f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2442e5fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf62e5ff, SUC_lexicon_tags_37 },
    { 0xb842e600, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc782e60a, SUC_lexicon_tags_12 },
    { 0x5322e60b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2522e60f, SUC_lexicon_tags_11 },
    { 0xeca2e610, SUC_lexicon_tags_14 },
    { 0xa282e611, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6d02e613, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2362e616, SUC_lexicon_tags_1 },
    { 0x9862e616, SUC_lexicon_tags_17 },
    { 0x7d22e618, SUC_lexicon_tags_12 },
    { 0x1782e618, SUC_lexicon_tags_7 },
    { 0x3962e61a, SUC_lexicon_tags_68 },
    { 0x6c62e619, SUC_lexicon_tags_12 },
    { 0xcac2e61b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3222e61e, SUC_lexicon_tags_33 },
    { 0x7a2e61f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402e625, SUC_lexicon_tags_13 },
    { 0xa702e626, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c2e62c, SUC_lexicon_tags_17 },
    { 0x9d22e62d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5522e630, SUC_lexicon_tags_15 },
    { 0xd2e2e630, SUC_lexicon_tags_37 },
    { 0x5942e630, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82e635, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4042e637, SUC_lexicon_tags_11 },
    { 0x8aa2e638, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x17e2e63a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf702e63c, SUC_lexicon_tags_82 },
    { 0x54e2e63c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x822e63f, SUC_lexicon_tags_25 },
    { 0x8d62e640, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xae42e642, SUC_lexicon_tags_2 },
    { 0x36e2e643, SUC_lexicon_tags_45 },
    { 0xc9a2e643, SUC_lexicon_tags_12 },
    { 0xcda2e645, SUC_lexicon_tags_29 },
    { 0x8542e645, SUC_lexicon_tags_25 },
    { 0xcc22e647, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8082e649, SUC_lexicon_tags_0 },
    { 0x2d82e64a, SUC_lexicon_tags_12 },
    { 0x12c2e649, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb42e64f, SUC_lexicon_tags_2 },
    { 0x6f22e650, SUC_lexicon_tags_3 },
    { 0xe162e64f, SUC_lexicon_tags_1413 },
    { 0x12e2e64f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x942e654, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf102e65b, SUC_lexicon_tags_23 },
    { 0xbf62e65c, SUC_lexicon_tags_49 },
    { 0x5c42e65c, SUC_lexicon_tags_15 },
    { 0x6522e65e, SUC_lexicon_tags_14 },
    { 0x4562e65f, SUC_lexicon_tags_10 },
    { 0x71c2e660, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe02e662, SUC_lexicon_tags_25 },
    { 0xaa2e662, SUC_lexicon_tags_15 },
    { 0x4042e664, SUC_lexicon_tags_14 },
    { 0x3ac2e665, SUC_lexicon_tags_65 },
    { 0x5642e663, SUC_lexicon_tags_11 },
    { 0xe522e664, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4782e669, SUC_lexicon_tags_17 },
    { 0x3002e66a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac2e66d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd482e672, SUC_lexicon_tags_1 },
    { 0x6642e673, SUC_lexicon_tags_12 },
    { 0x6922e673, SUC_lexicon_tags_14 },
    { 0x6802e674, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xf142e677, SUC_lexicon_tags_37 },
    { 0x3a82e677, SUC_lexicon_tags_17 },
    { 0xc782e677, SUC_lexicon_tags_1 },
    { 0x4ca2e678, SUC_lexicon_tags_26 },
    { 0x41a2e67b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x45c2e67d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e2e682, SUC_lexicon_tags_61 },
    { 0x4962e682, SUC_lexicon_tags_59 },
    { 0x982e684, SUC_lexicon_tags_37 },
    { 0x3a22e685, SUC_lexicon_tags_19 },
    { 0xe202e686, SUC_lexicon_tags_17 },
    { 0xd22e686, SUC_lexicon_tags_37 },
    { 0x3a82e685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c2e68b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd322e68d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ea2e68f, SUC_lexicon_tags_0 },
    { 0x89a2e690, SUC_lexicon_tags_0 },
    { 0x18a2e68f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e2e696, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4002e698, SUC_lexicon_tags_37 },
    { 0xc0c2e698, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa102e69b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7102e6a1, SUC_lexicon_tags_7 },
    { 0x1782e6a2, SUC_lexicon_tags_2 },
    { 0xa482e6a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24e2e6a5, SUC_lexicon_tags_8 },
    { 0x9dc2e6a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc582e6a8, SUC_lexicon_tags_66 },
    { 0xf702e6a9, SUC_lexicon_tags_57 },
    { 0x5742e6a9, SUC_lexicon_tags_28 },
    { 0x4b42e6ab, SUC_lexicon_tags_65 },
    { 0x6ba2e6ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa02e6ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3862e6b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2e6ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2e6c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9722e6c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x22c2e6c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xee02e6cb, SUC_lexicon_tags_34 },
    { 0xe862e6cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b02e6ce, SUC_lexicon_tags_17 },
    { 0xf3a2e6cf, SUC_lexicon_tags_27 },
    { 0xade2e6ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb1c2e6d2, SUC_lexicon_tags_11 },
    { 0xf62e6d3, SUC_lexicon_tags_25 },
    { 0x5142e6d3, SUC_lexicon_tags_37 },
    { 0xf62e6d4, SUC_lexicon_tags_7 },
    { 0xa302e6d6, SUC_lexicon_tags_39 },
    { 0x8e2e6d6, SUC_lexicon_tags_4 },
    { 0x8c02e6d8, SUC_lexicon_tags_116 },
    { 0x42c2e6d9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc2e6df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe522e6e1, SUC_lexicon_tags_6 },
    { 0xfde2e6e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf742e6e5, SUC_lexicon_tags_27 },
    { 0x4042e6e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe662e6ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5942e6f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3502e6f8, SUC_lexicon_tags_27 },
    { 0x4282e6f8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xab22e6fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c2e704, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5482e706, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec2e709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5e2e70b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf222e712, SUC_lexicon_tags_33 },
    { 0xe0e2e712, SUC_lexicon_tags_27 },
    { 0x1e22e714, SUC_lexicon_tags_37 },
    { 0xb582e715, SUC_lexicon_tags_21 },
    { 0x2602e715, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb742e71f, SUC_lexicon_tags_11 },
    { 0x64e2e720, SUC_lexicon_tags_37 },
    { 0x21c2e721, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9242e723, SUC_lexicon_tags_7 },
    { 0xa9c2e724, SUC_lexicon_tags_65 },
    { 0xf482e725, SUC_lexicon_tags_44 },
    { 0x4f62e724, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa862e731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9102e733, SUC_lexicon_tags_25 },
    { 0xaac2e733, SUC_lexicon_tags_1 },
    { 0x5282e735, SUC_lexicon_tags_7 },
    { 0xb402e735, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf7e2e738, SUC_lexicon_tags_6 },
    { 0x7582e738, SUC_lexicon_tags_15 },
    { 0xc7c2e73a, SUC_lexicon_tags_21 },
    { 0x9002e739, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4222e73f, SUC_lexicon_tags_6 },
    { 0x85c2e740, SUC_lexicon_tags_8 },
    { 0xad02e740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd62e747, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1842e74a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd602e74c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe82e754, SUC_lexicon_tags_25 },
    { 0xe282e755, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e2e76a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1042e771, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8242e774, SUC_lexicon_tags_47 },
    { 0x2822e775, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xae02e777, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e77a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f02e77d, SUC_lexicon_tags_42 },
    { 0x1702e77e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e2e783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a2e786, SUC_lexicon_tags_11 },
    { 0x7a42e787, SUC_lexicon_tags_11 },
    { 0xdee2e788, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5a2e78a, SUC_lexicon_tags_32 },
    { 0x6fa2e78b, SUC_lexicon_tags_7 },
    { 0xd8e2e78c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a2e790, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda2e793, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf522e799, SUC_lexicon_tags_25 },
    { 0xa1a2e79a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f42e79e, SUC_lexicon_tags_11 },
    { 0x5642e79f, SUC_lexicon_tags_204 },
    { 0xdfc2e79f, SUC_lexicon_tags_7 },
    { 0x6ce2e7a0, SUC_lexicon_tags_1 },
    { 0x4ba2e7a2, SUC_lexicon_tags_17 },
    { 0x64e2e7a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b02e7a5, SUC_lexicon_tags_38 },
    { 0x9e02e7a6, SUC_lexicon_tags_16 },
    { 0x2c02e7a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc282e7af, SUC_lexicon_tags_27 },
    { 0xe9a2e7b0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac2e7b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2e7b8, SUC_lexicon_tags_15 },
    { 0x21e2e7b9, SUC_lexicon_tags_8 },
    { 0x1542e7b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde2e7bf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3582e7c2, SUC_lexicon_tags_296 },
    { 0xca42e7c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb42e7c5, SUC_lexicon_tags_66 },
    { 0x4222e7c6, SUC_lexicon_tags_17 },
    { 0x6fc2e7c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8622e7c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe42e7cb, SUC_lexicon_tags_47 },
    { 0x5a22e7cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd22e7cf, SUC_lexicon_tags_16 },
    { 0xb862e7d0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f02e7d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xef62e7d8, SUC_lexicon_tags_17 },
    { 0xadc2e7d8, SUC_lexicon_tags_37 },
    { 0x762e7d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4442e7dc, SUC_lexicon_tags_37 },
    { 0xc3c2e7dc, SUC_lexicon_tags_2 },
    { 0x7882e7dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2742e7e0, SUC_lexicon_tags_17 },
    { 0xec22e7e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a62e7e6, SUC_lexicon_tags_47 },
    { 0xb942e7e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16a2e7e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6662e7eb, SUC_lexicon_tags_37 },
    { 0xf802e7ec, SUC_lexicon_tags_113 },
    { 0x3c2e7eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa042e7f1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a2e7f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x722e7f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe2e7fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe42e801, SUC_lexicon_tags_29 },
    { 0x7542e802, SUC_lexicon_tags_25 },
    { 0xabc2e803, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9362e805, SUC_lexicon_tags_35 },
    { 0x9142e805, SUC_lexicon_tags_60 },
    { 0x3b42e806, SUC_lexicon_tags_23 },
    { 0xb462e808, SUC_lexicon_tags_0 },
    { 0x3e22e808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea2e80f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf62e811, SUC_lexicon_tags_22 },
    { 0xa4c2e812, SUC_lexicon_tags_25 },
    { 0xdb82e813, SUC_lexicon_tags_23 },
    { 0x8322e814, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x16a2e816, SUC_lexicon_tags_61 },
    { 0x7f62e817, SUC_lexicon_tags_2 },
    { 0xc5a2e817, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a2e81e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfae2e820, SUC_lexicon_tags_37 },
    { 0x7cc2e820, SUC_lexicon_tags_7 },
    { 0x3bc2e821, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e2e826, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8a22e828, SUC_lexicon_tags_17 },
    { 0xd0c2e829, SUC_lexicon_tags_7 },
    { 0xe642e82a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c2e830, SUC_lexicon_tags_39 },
    { 0x2702e831, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae2e835, SUC_lexicon_tags_25 },
    { 0x8082e836, SUC_lexicon_tags_37 },
    { 0xce2e835, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e2e83c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac2e840, SUC_lexicon_tags_47 },
    { 0xfce2e841, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf102e843, SUC_lexicon_tags_28 },
    { 0x9e02e844, SUC_lexicon_tags_12 },
    { 0xcf42e843, SUC_lexicon_tags_38 },
    { 0xfa22e846, SUC_lexicon_tags_17 },
    { 0x1802e844, SUC_lexicon_tags_150 },
    { 0x54e2e848, SUC_lexicon_tags_1 },
    { 0x6322e847, SUC_lexicon_tags_69 },
    { 0xbf02e84a, SUC_lexicon_tags_56 },
    { 0x1982e84b, SUC_lexicon_tags_30 },
    { 0x9322e846, SUC_lexicon_tags_7 },
    { 0xd1e2e846, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc402e854, SUC_lexicon_tags_11 },
    { 0x4ec2e855, SUC_lexicon_tags_15 },
    { 0xe6e2e856, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xfdc2e858, SUC_lexicon_tags_15 },
    { 0x6a02e858, SUC_lexicon_tags_47 },
    { 0x4482e85a, SUC_lexicon_tags_17 },
    { 0xebc2e85b, SUC_lexicon_tags_12 },
    { 0xcac2e85a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2402e85f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f62e865, SUC_lexicon_tags_0 },
    { 0xeb82e866, SUC_lexicon_tags_39 },
    { 0xe7c2e866, SUC_lexicon_tags_68 },
    { 0x8702e866, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c2e871, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee22e875, SUC_lexicon_tags_27 },
    { 0xca42e876, SUC_lexicon_tags_59 },
    { 0x4e02e877, SUC_lexicon_tags_37 },
    { 0x59e2e878, SUC_lexicon_tags_2 },
    { 0xa702e879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca2e87c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x402e87e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72e2e880, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a2e884, SUC_lexicon_tags_27 },
    { 0xc3a2e885, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8062e88c, SUC_lexicon_tags_27 },
    { 0x6642e88d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a2e892, SUC_lexicon_tags_27 },
    { 0xe762e893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1222e895, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602e899, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7642e89b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x79c2e89d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a2e8a0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c02e8a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea2e8a6, SUC_lexicon_tags_48 },
    { 0x52a2e8a7, SUC_lexicon_tags_6 },
    { 0x1542e8a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc822e8ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4a2e8b0, SUC_lexicon_tags_15 },
    { 0x49c2e8b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4922e8b3, SUC_lexicon_tags_7 },
    { 0x682e8b4, SUC_lexicon_tags_216 },
    { 0x5962e8b4, SUC_lexicon_tags_24 },
    { 0x2002e8b5, SUC_lexicon_tags_18 },
    { 0x1e2e8b7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3762e8bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb082e8c1, SUC_lexicon_tags_37 },
    { 0x7662e8c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7562e8c4, SUC_lexicon_tags_11 },
    { 0xa0c2e8c5, SUC_lexicon_tags_1 },
    { 0x7122e8c6, SUC_lexicon_tags_40 },
    { 0x9182e8c7, SUC_lexicon_tags_32 },
    { 0x6a42e8c7, SUC_lexicon_tags_61 },
    { 0x32e2e8c9, SUC_lexicon_tags_63 },
    { 0xed82e8c5, SUC_lexicon_tags_28 },
    { 0x53e2e8c9, SUC_lexicon_tags_25 },
    { 0x1782e8cc, SUC_lexicon_tags_16 },
    { 0x52e2e8c5, SUC_lexicon_tags_45 },
    { 0x8f42e8c4, SUC_lexicon_tags_9 },
    { 0x4c82e8c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda2e8d3, SUC_lexicon_tags_49 },
    { 0xd102e8d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ac2e8d6, SUC_lexicon_tags_19 },
    { 0x76c2e8d7, SUC_lexicon_tags_29 },
    { 0x2f62e8d8, SUC_lexicon_tags_32 },
    { 0x2f02e8d7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a2e8dd, SUC_lexicon_tags_30 },
    { 0xa822e8dd, SUC_lexicon_tags_25 },
    { 0x9e62e8de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb02e8e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe0e2e8e5, SUC_lexicon_tags_17 },
    { 0xa3a2e8e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc482e8e9, SUC_lexicon_tags_38 },
    { 0x3ba2e8ea, SUC_lexicon_tags_61 },
    { 0x4962e8ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4602e8ed, SUC_lexicon_tags_11 },
    { 0x1162e8ee, SUC_lexicon_tags_64 },
    { 0xbea2e8ef, SUC_lexicon_tags_30 },
    { 0x8222e8ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7482e8f2, SUC_lexicon_tags_65 },
    { 0x6242e8f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf162e8f6, SUC_lexicon_tags_11 },
    { 0x3282e8f6, SUC_lexicon_tags_39 },
    { 0xdee2e8f8, SUC_lexicon_tags_17 },
    { 0xf922e8f8, SUC_lexicon_tags_60 },
    { 0xeee2e8fa, SUC_lexicon_tags_65 },
    { 0xc3c2e8fa, SUC_lexicon_tags_23 },
    { 0x2e8f9, SUC_lexicon_tags_17 },
    { 0xbbe2e8fb, SUC_lexicon_tags_14 },
    { 0x7342e8fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e22e900, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec2e905, SUC_lexicon_tags_47 },
    { 0x5042e905, SUC_lexicon_tags_17 },
    { 0xafa2e906, SUC_lexicon_tags_17 },
    { 0x9a62e908, SUC_lexicon_tags_29 },
    { 0x15a2e905, SUC_lexicon_tags_1 },
    { 0xafc2e90a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a2e910, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd4a2e912, SUC_lexicon_tags_37 },
    { 0xd502e912, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf222e917, SUC_lexicon_tags_37 },
    { 0x7002e917, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6c02e91a, SUC_lexicon_tags_11 },
    { 0x8fe2e91b, SUC_lexicon_tags_25 },
    { 0x6ae2e91c, SUC_lexicon_tags_15 },
    { 0x6a42e91d, SUC_lexicon_tags_30 },
    { 0x68e2e91e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2162e92b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a2e92e, SUC_lexicon_tags_19 },
    { 0xdae2e92f, SUC_lexicon_tags_7 },
    { 0xd922e930, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4862e932, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4322e936, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe602e938, SUC_lexicon_tags_25 },
    { 0xf9c2e939, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x362e93c, SUC_lexicon_tags_14 },
    { 0x9502e93d, SUC_lexicon_tags_117 },
    { 0x7fc2e93e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76e2e940, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd02e948, SUC_lexicon_tags_27 },
    { 0x2582e949, SUC_lexicon_tags_1 },
    { 0xfb22e949, SUC_lexicon_tags_25 },
    { 0x4f02e94b, SUC_lexicon_tags_17 },
    { 0x78c2e94c, SUC_lexicon_tags_10 },
    { 0x4342e94d, SUC_lexicon_tags_10 },
    { 0x9782e94e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e2e951, SUC_lexicon_tags_3 },
    { 0x8a02e952, SUC_lexicon_tags_15 },
    { 0x3d62e953, SUC_lexicon_tags_37 },
    { 0xc162e954, SUC_lexicon_tags_36 },
    { 0x72c2e955, SUC_lexicon_tags_17 },
    { 0xf7a2e956, SUC_lexicon_tags_6 },
    { 0xf662e956, SUC_lexicon_tags_47 },
    { 0x8442e958, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xede2e95b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc242e95e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xec82e961, SUC_lexicon_tags_12 },
    { 0x4122e961, SUC_lexicon_tags_12 },
    { 0xa562e963, SUC_lexicon_tags_38 },
    { 0xba2e963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0e2e966, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5582e968, SUC_lexicon_tags_25 },
    { 0x9622e968, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2862e96c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf62e96e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e82e973, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a2e977, SUC_lexicon_tags_25 },
    { 0x1202e978, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2242e97a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8562e97d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa202e980, SUC_lexicon_tags_15 },
    { 0x6122e981, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe142e987, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a42e989, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1382e98d, SUC_lexicon_tags_25 },
    { 0xfe62e98d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c2e998, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a2e99d, SUC_lexicon_tags_25 },
    { 0xc302e99e, SUC_lexicon_tags_11 },
    { 0xcb62e99d, SUC_lexicon_tags_14 },
    { 0x1002e9a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50e2e9a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762e9a6, SUC_lexicon_tags_49 },
    { 0xbb42e9a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1362e9ab, SUC_lexicon_tags_65 },
    { 0xa802e9ac, SUC_lexicon_tags_37 },
    { 0xa402e9ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e2e9b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdd82e9b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6942e9b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f02e9b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab62e9bb, SUC_lexicon_tags_37 },
    { 0x9322e9bc, SUC_lexicon_tags_25 },
    { 0xfd02e9bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b22e9bf, SUC_lexicon_tags_23 },
    { 0x2002e9c0, SUC_lexicon_tags_106 },
    { 0x1762e9c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf522e9c8, SUC_lexicon_tags_87 },
    { 0xb9c2e9c9, SUC_lexicon_tags_1 },
    { 0xaf82e9ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x122e9cd, SUC_lexicon_tags_7 },
    { 0x82e9ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7522e9d1, SUC_lexicon_tags_91 },
    { 0x3842e9d2, SUC_lexicon_tags_11 },
    { 0x8ac2e9d3, SUC_lexicon_tags_17 },
    { 0x4342e9d4, SUC_lexicon_tags_11 },
    { 0xfaa2e9d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9182e9d8, SUC_lexicon_tags_11 },
    { 0x99e2e9d9, SUC_lexicon_tags_15 },
    { 0xb7a2e9d8, SUC_lexicon_tags_27 },
    { 0xe162e9db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c02e9dd, SUC_lexicon_tags_1 },
    { 0x2602e9dd, SUC_lexicon_tags_7 },
    { 0x8fc2e9df, SUC_lexicon_tags_25 },
    { 0xb7c2e9dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce2e9e4, SUC_lexicon_tags_15 },
    { 0x9ac2e9e5, SUC_lexicon_tags_11 },
    { 0x6862e9e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa802e9ec, SUC_lexicon_tags_11 },
    { 0x9662e9ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xda42e9ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa62e9f1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9482e9f5, SUC_lexicon_tags_12 },
    { 0x6462e9f5, SUC_lexicon_tags_77 },
    { 0xfc82e9f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba2e9fb, SUC_lexicon_tags_28 },
    { 0x1c22e9fc, SUC_lexicon_tags_47 },
    { 0x1e42e9fd, SUC_lexicon_tags_37 },
    { 0x2f62e9fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2742ea04, SUC_lexicon_tags_37 },
    { 0xa102ea05, SUC_lexicon_tags_63 },
    { 0xf382ea04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6122ea08, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a2ea10, SUC_lexicon_tags_37 },
    { 0x6c82ea10, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6922ea16, SUC_lexicon_tags_23 },
    { 0x65a2ea17, SUC_lexicon_tags_37 },
    { 0x22e2ea18, SUC_lexicon_tags_10 },
    { 0xc622ea18, SUC_lexicon_tags_11 },
    { 0x6862ea17, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf482ea1e, SUC_lexicon_tags_37 },
    { 0xfca2ea1f, SUC_lexicon_tags_12 },
    { 0x5b62ea1e, SUC_lexicon_tags_17 },
    { 0xd8c2ea21, SUC_lexicon_tags_39 },
    { 0xa722ea1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda2ea29, SUC_lexicon_tags_43 },
    { 0xb922ea2a, SUC_lexicon_tags_351 },
    { 0x1e02ea29, SUC_lexicon_tags_26 },
    { 0xa7a2ea2a, SUC_lexicon_tags_25 },
    { 0x3be2ea2a, SUC_lexicon_tags_38 },
    { 0x33c2ea2a, SUC_lexicon_tags_31 },
    { 0xede2ea2f, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a2ea32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e2ea37, SUC_lexicon_tags_11 },
    { 0xa702ea38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fe2ea3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x222ea3d, SUC_lexicon_tags_21 },
    { 0xdc82ea3e, SUC_lexicon_tags_23 },
    { 0x82ea3f, SUC_lexicon_tags_95 },
    { 0xe7c2ea3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc022ea44, SUC_lexicon_tags_25 },
    { 0x9962ea44, SUC_lexicon_tags_1 },
    { 0x1322ea46, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc2ea52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e2ea57, SUC_lexicon_tags_12 },
    { 0x7062ea58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf02ea5c, SUC_lexicon_tags_37 },
    { 0xed22ea5c, SUC_lexicon_tags_25 },
    { 0x7122ea5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94c2ea60, SUC_lexicon_tags_63 },
    { 0xefe2ea61, SUC_lexicon_tags_37 },
    { 0xa5c2ea62, SUC_lexicon_tags_84 },
    { 0xe9a2ea61, SUC_lexicon_tags_17 },
    { 0x2122ea61, SUC_lexicon_tags_79 },
    { 0x3fc2ea65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52a2ea67, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e2ea6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x782ea6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf162ea70, SUC_lexicon_tags_25 },
    { 0x7f82ea71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6722ea78, SUC_lexicon_tags_14 },
    { 0xee82ea79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2ac2ea7b, SUC_lexicon_tags_33 },
    { 0x5402ea7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc662ea80, SUC_lexicon_tags_11 },
    { 0x33e2ea80, SUC_lexicon_tags_60 },
    { 0xf302ea82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa2ea86, SUC_lexicon_tags_15 },
    { 0x1e82ea86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9862ea89, SUC_lexicon_tags_13 },
    { 0x71e2ea89, SUC_lexicon_tags_45 },
    { 0xb4c2ea89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c2ea8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdac2ea90, SUC_lexicon_tags_13 },
    { 0x28a2ea91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a2ea95, SUC_lexicon_tags_25 },
    { 0x6822ea96, SUC_lexicon_tags_45 },
    { 0xcac2ea97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a2ea9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b42ea9d, SUC_lexicon_tags_37 },
    { 0x5542ea9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x682eaa1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f22eaa5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb582eaa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51c2eaa9, SUC_lexicon_tags_1 },
    { 0xe702eaaa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad02eaac, SUC_lexicon_tags_48 },
    { 0xcda2eaad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade2eab1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d62eab4, SUC_lexicon_tags_65 },
    { 0xd702eab5, SUC_lexicon_tags_7 },
    { 0xe982eab4, SUC_lexicon_tags_1 },
    { 0x4c62eab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc62eab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd22eabb, SUC_lexicon_tags_39 },
    { 0x83e2eabc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca2eac0, SUC_lexicon_tags_68 },
    { 0x15a2eac0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2682eac3, SUC_lexicon_tags_63 },
    { 0xbd82eac3, SUC_lexicon_tags_27 },
    { 0x9e2eac5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a2eac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e2eacb, SUC_lexicon_tags_3 },
    { 0xe82eacc, SUC_lexicon_tags_15 },
    { 0xfa22eacd, SUC_lexicon_tags_39 },
    { 0xf362eace, SUC_lexicon_tags_17 },
    { 0x23a2eacf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6002ead4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7362eae3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5542eaea, SUC_lexicon_tags_79 },
    { 0x6762eaea, SUC_lexicon_tags_15 },
    { 0x3a22eaec, SUC_lexicon_tags_17 },
    { 0x5642eaeb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a2eaf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea2eaf4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5582eaf8, SUC_lexicon_tags_7 },
    { 0x9902eaf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a2eafd, SUC_lexicon_tags_15 },
    { 0x2ee2eafe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8562eb00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a2eb06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a2eb13, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4302eb18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8282eb1e, SUC_lexicon_tags_11 },
    { 0x24e2eb1e, SUC_lexicon_tags_29 },
    { 0x242eb1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf622eb23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1282eb26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe362eb28, SUC_lexicon_tags_37 },
    { 0xf722eb29, SUC_lexicon_tags_1 },
    { 0xf502eb2a, SUC_lexicon_tags_12 },
    { 0x1502eb2b, SUC_lexicon_tags_65 },
    { 0xfe22eb29, SUC_lexicon_tags_31 },
    { 0xf2a2eb28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe0e2eb2f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a2eb34, SUC_lexicon_tags_37 },
    { 0x2d42eb35, SUC_lexicon_tags_5 },
    { 0x7d42eb36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe2eb3d, SUC_lexicon_tags_2 },
    { 0x9b22eb3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a2eb49, SUC_lexicon_tags_28 },
    { 0xca62eb49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422eb4f, SUC_lexicon_tags_82 },
    { 0x44a2eb50, SUC_lexicon_tags_25 },
    { 0x96e2eb51, SUC_lexicon_tags_11 },
    { 0xa162eb52, SUC_lexicon_tags_19 },
    { 0xdee2eb51, SUC_lexicon_tags_60 },
    { 0x2e2eb53, SUC_lexicon_tags_33 },
    { 0x4e62eb55, SUC_lexicon_tags_1 },
    { 0xd922eb56, SUC_lexicon_tags_63 },
    { 0x6fc2eb57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb62eb5b, SUC_lexicon_tags_57 },
    { 0x2762eb5c, SUC_lexicon_tags_14 },
    { 0xbe22eb5d, SUC_lexicon_tags_39 },
    { 0xf242eb5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xffa2eb60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd82eb66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf322eb69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa2eb6f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9ce2eb71, SUC_lexicon_tags_11 },
    { 0xd3c2eb71, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7402eb74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3502eb76, SUC_lexicon_tags_11 },
    { 0x70a2eb76, SUC_lexicon_tags_37 },
    { 0x5e02eb78, SUC_lexicon_tags_23 },
    { 0xf542eb79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c82eb7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e2eb82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5662eb86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc2eb8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd682eb8f, SUC_lexicon_tags_29 },
    { 0x6d02eb90, SUC_lexicon_tags_26 },
    { 0xe9c2eb91, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf522eb93, SUC_lexicon_tags_77 },
    { 0x982eb93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2142eb97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2662eb9e, SUC_lexicon_tags_11 },
    { 0x75e2eb9f, SUC_lexicon_tags_12 },
    { 0x522eba0, SUC_lexicon_tags_43 },
    { 0x7282eb9e, SUC_lexicon_tags_9 },
    { 0xb362eba2, SUC_lexicon_tags_49 },
    { 0x7a62eb9e, SUC_lexicon_tags_72 },
    { 0x84a2eba4, SUC_lexicon_tags_12 },
    { 0x45c2eba5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4042ebae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1602ebb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb42ebb9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f02ebbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1722ebbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed02ebc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a2ebca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8262ebcc, SUC_lexicon_tags_15 },
    { 0xc162ebcd, SUC_lexicon_tags_5 },
    { 0x3942ebce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb22ebd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a2ebd3, SUC_lexicon_tags_11 },
    { 0xde22ebd4, SUC_lexicon_tags_17 },
    { 0x1982ebd4, SUC_lexicon_tags_27 },
    { 0x2c02ebd6, SUC_lexicon_tags_37 },
    { 0x1fe2ebd7, SUC_lexicon_tags_12 },
    { 0x8b22ebd7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6a2ebda, SUC_lexicon_tags_1 },
    { 0x4c22ebda, SUC_lexicon_tags_47 },
    { 0x7b02ebdb, SUC_lexicon_tags_31 },
    { 0xcb62ebdd, SUC_lexicon_tags_4 },
    { 0x182ebda, SUC_lexicon_tags_2 },
    { 0xf2c2ebdf, SUC_lexicon_tags_37 },
    { 0xf1e2ebdf, SUC_lexicon_tags_57 },
    { 0x52a2ebe1, SUC_lexicon_tags_38 },
    { 0xb062ebe2, SUC_lexicon_tags_39 },
    { 0x7342ebe0, SUC_lexicon_tags_26 },
    { 0x3a2ebe4, SUC_lexicon_tags_11 },
    { 0xe5a2ebe5, SUC_lexicon_tags_0 },
    { 0x6f42ebe6, SUC_lexicon_tags_28 },
    { 0x3f22ebe7, SUC_lexicon_tags_11 },
    { 0x562ebdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de2ebec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd6e2ebee, SUC_lexicon_tags_17 },
    { 0xbd22ebee, SUC_lexicon_tags_50 },
    { 0x4742ebef, SUC_lexicon_tags_11 },
    { 0x41a2ebf1, SUC_lexicon_tags_38 },
    { 0xd42ebf2, SUC_lexicon_tags_37 },
    { 0x2ec2ebf3, SUC_lexicon_tags_15 },
    { 0xea42ebf3, SUC_lexicon_tags_15 },
    { 0xadc2ebf3, SUC_lexicon_tags_151 },
    { 0x7742ebf4, SUC_lexicon_tags_47 },
    { 0xa0a2ebf4, SUC_lexicon_tags_37 },
    { 0x66c2ebf1, SUC_lexicon_tags_86 },
    { 0xd7c2ebf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe062ec03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44a2ec05, SUC_lexicon_tags_27 },
    { 0xfe2ec06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2ec09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc482ec0b, SUC_lexicon_tags_60 },
    { 0xe3e2ec0c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4b82ec0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f82ec19, SUC_lexicon_tags_49 },
    { 0x17a2ec1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5062ec1c, SUC_lexicon_tags_23 },
    { 0x4662ec1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e2ec1f, SUC_lexicon_tags_15 },
    { 0xf522ec20, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2e62ec22, SUC_lexicon_tags_37 },
    { 0x54c2ec23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e2ec28, SUC_lexicon_tags_38 },
    { 0x4642ec28, SUC_lexicon_tags_15 },
    { 0x96a2ec2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5042ec2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1022ec2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a2ec33, SUC_lexicon_tags_19 },
    { 0x7722ec34, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa622ec37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae2ec3d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb22ec42, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c2ec46, SUC_lexicon_tags_12 },
    { 0xbb42ec47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe8c2ec49, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e2ec4d, SUC_lexicon_tags_2 },
    { 0xc62ec4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e2ec54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc22ec58, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4422ec5b, SUC_lexicon_tags_8 },
    { 0xdba2ec5c, SUC_lexicon_tags_7 },
    { 0x7542ec5c, SUC_lexicon_tags_17 },
    { 0x3d02ec5c, SUC_lexicon_tags_12 },
    { 0x6442ec5f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc322ec62, SUC_lexicon_tags_61 },
    { 0xa7a2ec63, SUC_lexicon_tags_27 },
    { 0x5b82ec64, SUC_lexicon_tags_17 },
    { 0x7902ec65, SUC_lexicon_tags_11 },
    { 0x49e2ec66, SUC_lexicon_tags_63 },
    { 0x42e2ec66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa2ec6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf062ec71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc2ec77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1da2ec79, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc2ec86, SUC_lexicon_tags_30 },
    { 0x3482ec87, SUC_lexicon_tags_7 },
    { 0xfc02ec88, SUC_lexicon_tags_30 },
    { 0xff82ec89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1682ec8f, SUC_lexicon_tags_33 },
    { 0xaf22ec90, SUC_lexicon_tags_27 },
    { 0xbca2ec8f, SUC_lexicon_tags_1 },
    { 0x3122ec90, SUC_lexicon_tags_33 },
    { 0xd442ec93, SUC_lexicon_tags_21 },
    { 0x9a62ec94, SUC_lexicon_tags_8 },
    { 0x7a2ec95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6642ec97, SUC_lexicon_tags_151 },
    { 0x6ac2ec97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4462ec9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2eca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102eca6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf222eca8, SUC_lexicon_tags_11 },
    { 0xd282eca9, SUC_lexicon_tags_62 },
    { 0x5942eca8, SUC_lexicon_tags_25 },
    { 0x3182ecab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1be2ecad, SUC_lexicon_tags_37 },
    { 0xe802ecae, SUC_lexicon_tags_37 },
    { 0xeee2ecaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa2ecb8, SUC_lexicon_tags_30 },
    { 0x2462ecb9, SUC_lexicon_tags_77 },
    { 0x71e2ecb9, SUC_lexicon_tags_37 },
    { 0x2e02ecbb, SUC_lexicon_tags_30 },
    { 0x2742ecba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e2ecc0, SUC_lexicon_tags_30 },
    { 0x6062ecc1, SUC_lexicon_tags_21 },
    { 0x6682ecc2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc2ecc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4382eccd, SUC_lexicon_tags_2 },
    { 0x3902ecce, SUC_lexicon_tags_49 },
    { 0x2a82ecce, SUC_lexicon_tags_38 },
    { 0xa7a2ecce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2ecd7, SUC_lexicon_tags_32 },
    { 0xfe82ecd7, SUC_lexicon_tags_17 },
    { 0x8c22ecd9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622ecdf, SUC_lexicon_tags_17 },
    { 0x6f02ecdf, SUC_lexicon_tags_39 },
    { 0x2ce2ecdf, SUC_lexicon_tags_11 },
    { 0x3002ece1, SUC_lexicon_tags_19 },
    { 0xae62ece1, SUC_lexicon_tags_45 },
    { 0xb5a2ece2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a62ece6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe02ecec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3fc2ecee, SUC_lexicon_tags_1 },
    { 0x8ce2ecee, SUC_lexicon_tags_33 },
    { 0x5e2ecf0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c82ecf6, SUC_lexicon_tags_2 },
    { 0xada2ecf7, SUC_lexicon_tags_1 },
    { 0x7622ecf6, SUC_lexicon_tags_15 },
    { 0xf162ecf9, SUC_lexicon_tags_5 },
    { 0xca02ecf9, SUC_lexicon_tags_34 },
    { 0xe442ecf7, SUC_lexicon_tags_2 },
    { 0x16c2ecfb, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0xee62ecff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd862ed03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec22ed05, SUC_lexicon_tags_23 },
    { 0xfd62ed06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea2ed0a, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x782ed0c, SUC_lexicon_tags_37 },
    { 0x4642ed0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6702ed0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa882ed11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6882ed14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77c2ed16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7002ed18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e2ed1c, SUC_lexicon_tags_33 },
    { 0x4a02ed1d, SUC_lexicon_tags_39 },
    { 0xd422ed1d, SUC_lexicon_tags_27 },
    { 0xd202ed1f, SUC_lexicon_tags_37 },
    { 0x2962ed20, SUC_lexicon_tags_17 },
    { 0x84c2ed21, SUC_lexicon_tags_25 },
    { 0xfa2ed21, SUC_lexicon_tags_14 },
    { 0x7722ed20, SUC_lexicon_tags_9 },
    { 0xd982ed20, SUC_lexicon_tags_8 },
    { 0xc542ed1f, SUC_lexicon_tags_37 },
    { 0xfcc2ed23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a2ed29, SUC_lexicon_tags_30 },
    { 0x4c2ed2a, SUC_lexicon_tags_15 },
    { 0x44e2ed2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7722ed2f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e22ed32, SUC_lexicon_tags_36 },
    { 0x2ec2ed33, SUC_lexicon_tags_47 },
    { 0xd0e2ed34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f22ed39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1162ed3e, SUC_lexicon_tags_59 },
    { 0x51e2ed3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b22ed44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb62ed46, SUC_lexicon_tags_17 },
    { 0x1502ed46, SUC_lexicon_tags_12 },
    { 0xd902ed46, SUC_lexicon_tags_47 },
    { 0x2a02ed49, SUC_lexicon_tags_13 },
    { 0xe2a2ed4a, SUC_lexicon_tags_25 },
    { 0x22a2ed47, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e2ed4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7642ed53, SUC_lexicon_tags_15 },
    { 0x9e82ed53, SUC_lexicon_tags_15 },
    { 0x6442ed54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe2ed58, SUC_lexicon_tags_117 },
    { 0x7722ed59, SUC_lexicon_tags_60 },
    { 0xb982ed5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f42ed5c, SUC_lexicon_tags_14 },
    { 0xfc82ed5d, SUC_lexicon_tags_15 },
    { 0xe702ed5c, SUC_lexicon_tags_30 },
    { 0x4782ed5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1722ed62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e2ed67, SUC_lexicon_tags_29 },
    { 0x4c62ed67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9542ed6a, SUC_lexicon_tags_47 },
    { 0x82c2ed6a, SUC_lexicon_tags_37 },
    { 0x60c2ed6c, SUC_lexicon_tags_17 },
    { 0x9782ed6c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x69c2ed6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da2ed72, SUC_lexicon_tags_37 },
    { 0x88a2ed73, SUC_lexicon_tags_23 },
    { 0x2222ed74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b42ed79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6382ed7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2142ed84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e62ed88, SUC_lexicon_tags_60 },
    { 0x4aa2ed89, SUC_lexicon_tags_17 },
    { 0x4602ed89, SUC_lexicon_tags_37 },
    { 0x8742ed89, SUC_lexicon_tags_64 },
    { 0xcf22ed89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8402ed90, SUC_lexicon_tags_25 },
    { 0xc5e2ed91, SUC_lexicon_tags_47 },
    { 0xf902ed91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e2ed98, SUC_lexicon_tags_37 },
    { 0xd482ed99, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x62e2ed9b, SUC_lexicon_tags_15 },
    { 0x2ee2ed9c, SUC_lexicon_tags_15 },
    { 0x3d62ed9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8762eda2, SUC_lexicon_tags_7 },
    { 0xf502eda3, SUC_lexicon_tags_19 },
    { 0xc242eda3, SUC_lexicon_tags_6 },
    { 0xee2eda5, SUC_lexicon_tags_43 },
    { 0xfb02eda2, SUC_lexicon_tags_60 },
    { 0x25a2eda7, SUC_lexicon_tags_8 },
    { 0x32e2eda6, SUC_lexicon_tags_25 },
    { 0xe462eda9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb002edab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb02edad, SUC_lexicon_tags_26 },
    { 0xa02edae, SUC_lexicon_tags_74 },
    { 0xfba2edaf, SUC_lexicon_tags_15 },
    { 0x9f62edaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e2edb3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2edb6, SUC_lexicon_tags_29 },
    { 0x3642edb6, SUC_lexicon_tags_12 },
    { 0xb02edb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6242edba, SUC_lexicon_tags_1 },
    { 0xf482edbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc302edbd, SUC_lexicon_tags_37 },
    { 0xc842edbe, SUC_lexicon_tags_19 },
    { 0x1a62edbf, SUC_lexicon_tags_9 },
    { 0xfa02edc0, SUC_lexicon_tags_31 },
    { 0xaea2edc1, SUC_lexicon_tags_9 },
    { 0x15a2edc2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc402edc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb522edc9, SUC_lexicon_tags_28 },
    { 0x3382edca, SUC_lexicon_tags_4 },
    { 0x7d82edcb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe2edd0, SUC_lexicon_tags_4 },
    { 0x59e2edd0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a02edd4, SUC_lexicon_tags_54 },
    { 0xd962edd4, SUC_lexicon_tags_46 },
    { 0x6382edd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a42edd8, SUC_lexicon_tags_17 },
    { 0xa522edd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8002eddb, SUC_lexicon_tags_37 },
    { 0x60c2eddc, SUC_lexicon_tags_15 },
    { 0x69e2eddc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f82ede1, SUC_lexicon_tags_47 },
    { 0x5b22ede1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6362ede4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc2ede7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c2edea, SUC_lexicon_tags_2 },
    { 0x7482edeb, SUC_lexicon_tags_14 },
    { 0x9162edec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe4e2edee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c2edf3, SUC_lexicon_tags_47 },
    { 0xcea2edf3, SUC_lexicon_tags_45 },
    { 0x1d02edf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1622edfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da2edfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e82ee01, SUC_lexicon_tags_1 },
    { 0x69a2ee01, SUC_lexicon_tags_63 },
    { 0xdc82ee03, SUC_lexicon_tags_2 },
    { 0xd202ee04, SUC_lexicon_tags_25 },
    { 0xa302ee05, SUC_lexicon_tags_17 },
    { 0xcf62ee05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5202ee0a, SUC_lexicon_tags_27 },
    { 0x5822ee0b, SUC_lexicon_tags_33 },
    { 0x8ba2ee0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a2ee11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a2ee15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x442ee17, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xda02ee19, SUC_lexicon_tags_2 },
    { 0x4c82ee19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe162ee1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb422ee25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe2ee2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e2ee30, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5302ee34, SUC_lexicon_tags_37 },
    { 0xde22ee35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae62ee37, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5362ee3b, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x6c02ee3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a2ee40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee22ee43, SUC_lexicon_tags_164 },
    { 0, NULL },
    { 0, NULL },
    { 0x7902ee46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5182ee4a, SUC_lexicon_tags_46 },
    { 0x3562ee4b, SUC_lexicon_tags_25 },
    { 0xb9a2ee4b, SUC_lexicon_tags_14 },
    { 0xb3c2ee4d, SUC_lexicon_tags_0 },
    { 0x1d22ee4e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac2ee53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe2ee5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda2ee60, SUC_lexicon_tags_61 },
    { 0xcc02ee61, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e2ee64, SUC_lexicon_tags_45 },
    { 0x6d22ee65, SUC_lexicon_tags_17 },
    { 0x1f62ee64, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4742ee69, SUC_lexicon_tags_30 },
    { 0x4e42ee6a, SUC_lexicon_tags_31 },
    { 0x7e42ee6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc042ee6d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c2ee71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x18e2ee73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce42ee77, SUC_lexicon_tags_43 },
    { 0x242ee78, SUC_lexicon_tags_17 },
    { 0x3882ee78, SUC_lexicon_tags_582 },
    { 0x5962ee78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf122ee7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c2ee83, SUC_lexicon_tags_2 },
    { 0x5162ee84, SUC_lexicon_tags_146 },
    { 0x2942ee85, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e2ee8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd22ee8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc382ee8f, SUC_lexicon_tags_10 },
    { 0x2802ee90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82ee93, SUC_lexicon_tags_17 },
    { 0x8ba2ee94, SUC_lexicon_tags_47 },
    { 0xdfc2ee94, SUC_lexicon_tags_25 },
    { 0x9902ee96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd7c2ee98, SUC_lexicon_tags_15 },
    { 0xc882ee99, SUC_lexicon_tags_8 },
    { 0x1282ee9a, SUC_lexicon_tags_14 },
    { 0x8ae2ee9b, SUC_lexicon_tags_37 },
    { 0xf742ee9a, SUC_lexicon_tags_38 },
    { 0xdca2ee9d, SUC_lexicon_tags_38 },
    { 0x2682ee9e, SUC_lexicon_tags_14 },
    { 0xb922ee9d, SUC_lexicon_tags_14 },
    { 0x4502ee9e, SUC_lexicon_tags_31 },
    { 0xce02eea1, SUC_lexicon_tags_7 },
    { 0xae22ee9d, SUC_lexicon_tags_15 },
    { 0x7ea2eea3, SUC_lexicon_tags_37 },
    { 0xab22eea4, SUC_lexicon_tags_30 },
    { 0x6c02eea5, SUC_lexicon_tags_16 },
    { 0xeae2eea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23c2eea8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2922eeb5, SUC_lexicon_tags_0 },
    { 0x90a2eeb6, SUC_lexicon_tags_37 },
    { 0x41e2eeb7, SUC_lexicon_tags_12 },
    { 0xada2eeb8, SUC_lexicon_tags_68 },
    { 0x46c2eeb8, SUC_lexicon_tags_15 },
    { 0x562eeb6, SUC_lexicon_tags_12 },
    { 0x41e2eebb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e2eec4, SUC_lexicon_tags_1 },
    { 0xb6a2eec4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa62eec8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d02eecb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf022eece, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8762eed2, SUC_lexicon_tags_91 },
    { 0x92c2eed3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2eed9, SUC_lexicon_tags_47 },
    { 0x4ce2eeda, SUC_lexicon_tags_11 },
    { 0xb4c2eedb, SUC_lexicon_tags_18 },
    { 0xad62eed9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8662eee1, SUC_lexicon_tags_60 },
    { 0x6262eee2, SUC_lexicon_tags_62 },
    { 0x4842eee2, SUC_lexicon_tags_6 },
    { 0x3d82eee3, SUC_lexicon_tags_13 },
    { 0xe342eee5, SUC_lexicon_tags_17 },
    { 0x59e2eee6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3342eee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f42eef2, SUC_lexicon_tags_11 },
    { 0x1ea2eef2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb22eef5, SUC_lexicon_tags_21 },
    { 0x4622eef6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff82eef8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafa2eefa, SUC_lexicon_tags_25 },
    { 0x8362eefb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22eeff, SUC_lexicon_tags_954 },
    { 0x90e2ef00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f82ef03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3dc2ef05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2c02ef07, SUC_lexicon_tags_37 },
    { 0x88a2ef08, SUC_lexicon_tags_19 },
    { 0xd1c2ef09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2682ef0d, SUC_lexicon_tags_12 },
    { 0xee42ef0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88e2ef10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc2ef15, SUC_lexicon_tags_26 },
    { 0xa7e2ef16, SUC_lexicon_tags_25 },
    { 0x8962ef16, SUC_lexicon_tags_15 },
    { 0x5d22ef18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa942ef1b, SUC_lexicon_tags_25 },
    { 0x17e2ef1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb02ef1e, SUC_lexicon_tags_25 },
    { 0x5e62ef1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9de2ef21, SUC_lexicon_tags_1 },
    { 0xfa2ef22, SUC_lexicon_tags_6 },
    { 0x14c2ef23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe402ef29, SUC_lexicon_tags_77 },
    { 0xbcc2ef2a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe02ef2e, SUC_lexicon_tags_16 },
    { 0xcb82ef2e, SUC_lexicon_tags_12 },
    { 0x2182ef30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a2ef33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa222ef3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea2ef3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc142ef47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe02ef49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a2ef4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c2ef57, SUC_lexicon_tags_37 },
    { 0x8862ef58, SUC_lexicon_tags_39 },
    { 0x4da2ef59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2842ef5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x90e2ef5e, SUC_lexicon_tags_37 },
    { 0xfb02ef5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a2ef68, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf122ef6b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd822ef6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc2ef76, SUC_lexicon_tags_11 },
    { 0x2a62ef77, SUC_lexicon_tags_12 },
    { 0xbac2ef77, SUC_lexicon_tags_31 },
    { 0x3ca2ef79, SUC_lexicon_tags_39 },
    { 0x262ef79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x422ef7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5442ef80, SUC_lexicon_tags_29 },
    { 0x922ef81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb342ef85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc282ef87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc702ef89, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e2ef8c, SUC_lexicon_tags_12 },
    { 0xa202ef8c, SUC_lexicon_tags_39 },
    { 0xd262ef8d, SUC_lexicon_tags_12 },
    { 0x25e2ef8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb22ef92, SUC_lexicon_tags_29 },
    { 0xdea2ef93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa822ef96, SUC_lexicon_tags_29 },
    { 0x8ca2ef97, SUC_lexicon_tags_30 },
    { 0x2642ef98, SUC_lexicon_tags_17 },
    { 0x5362ef98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9582ef9d, SUC_lexicon_tags_37 },
    { 0x3382ef9d, SUC_lexicon_tags_25 },
    { 0xef62ef9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9362efaa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7862efac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a2efaf, SUC_lexicon_tags_12 },
    { 0xaae2efb0, SUC_lexicon_tags_17 },
    { 0x6102efb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a62efb3, SUC_lexicon_tags_13 },
    { 0x5902efb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a2efba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb442efbd, SUC_lexicon_tags_9 },
    { 0xee62efbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2142efc2, SUC_lexicon_tags_1 },
    { 0xa822efc3, SUC_lexicon_tags_37 },
    { 0x5062efc4, SUC_lexicon_tags_45 },
    { 0x2982efc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf182efc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e2efcb, SUC_lexicon_tags_8 },
    { 0x1302efcc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5522efce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa9c2efd0, SUC_lexicon_tags_38 },
    { 0x8902efd1, SUC_lexicon_tags_23 },
    { 0x1d62efd1, SUC_lexicon_tags_5 },
    { 0xbe22efd0, SUC_lexicon_tags_228 },
    { 0xcba2efd3, SUC_lexicon_tags_642 },
    { 0xbdc2efd5, SUC_lexicon_tags_7 },
    { 0xfb22efd1, SUC_lexicon_tags_1 },
    { 0xf622efd7, SUC_lexicon_tags_12 },
    { 0xbe22efd8, SUC_lexicon_tags_31 },
    { 0xd662efd8, SUC_lexicon_tags_6 },
    { 0x7b22efda, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8c02efdc, SUC_lexicon_tags_47 },
    { 0x8d22efdd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d42efe0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe622efeb, SUC_lexicon_tags_32 },
    { 0xe822efeb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe6c2efee, SUC_lexicon_tags_0 },
    { 0x9022efef, SUC_lexicon_tags_2 },
    { 0x97c2efef, SUC_lexicon_tags_5 },
    { 0xa982eff1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6322eff3, SUC_lexicon_tags_17 },
    { 0x33a2eff3, SUC_lexicon_tags_1 },
    { 0x7c02eff5, SUC_lexicon_tags_15 },
    { 0xe682eff6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7082eff9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc402effe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a82f002, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb642f005, SUC_lexicon_tags_23 },
    { 0x98c2f006, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd382f00a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe422f00d, SUC_lexicon_tags_19 },
    { 0x4aa2f00e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7882f012, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xfd22f014, SUC_lexicon_tags_25 },
    { 0xe782f015, SUC_lexicon_tags_301 },
    { 0xa762f015, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e2f01b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2f01f, SUC_lexicon_tags_9 },
    { 0x9f42f020, SUC_lexicon_tags_8 },
    { 0xbe2f020, SUC_lexicon_tags_1 },
    { 0x8d62f021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36c2f024, SUC_lexicon_tags_6 },
    { 0x7a62f025, SUC_lexicon_tags_14 },
    { 0xbd82f024, SUC_lexicon_tags_31 },
    { 0x7e2f024, SUC_lexicon_tags_25 },
    { 0xfee2f026, SUC_lexicon_tags_1 },
    { 0xee02f025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c2f02d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4682f02f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3282f036, SUC_lexicon_tags_12 },
    { 0x8a02f037, SUC_lexicon_tags_45 },
    { 0x5d42f037, SUC_lexicon_tags_43 },
    { 0xcdc2f039, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaea2f03b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7e82f03d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e2f043, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1642f046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa322f04f, SUC_lexicon_tags_47 },
    { 0xc762f04f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x25e2f052, SUC_lexicon_tags_12 },
    { 0x3ba2f053, SUC_lexicon_tags_25 },
    { 0xc102f053, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e2f057, SUC_lexicon_tags_0 },
    { 0x4842f057, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x442f05b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa242f05f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9f82f061, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c2f064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3e2f066, SUC_lexicon_tags_1 },
    { 0x8b62f067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7542f06c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb302f06f, SUC_lexicon_tags_11 },
    { 0x1a62f070, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2f073, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xb922f075, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2602f079, SUC_lexicon_tags_17 },
    { 0xf282f079, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc02f07c, SUC_lexicon_tags_37 },
    { 0x7582f07d, SUC_lexicon_tags_47 },
    { 0x1ba2f07e, SUC_lexicon_tags_37 },
    { 0x86a2f07f, SUC_lexicon_tags_28 },
    { 0xaee2f07f, SUC_lexicon_tags_47 },
    { 0x5282f07d, SUC_lexicon_tags_11 },
    { 0xcc22f082, SUC_lexicon_tags_17 },
    { 0xb002f082, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7302f086, SUC_lexicon_tags_27 },
    { 0xf862f086, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc42f08b, SUC_lexicon_tags_11 },
    { 0x75c2f08b, SUC_lexicon_tags_11 },
    { 0xc4a2f08b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee2f090, SUC_lexicon_tags_9 },
    { 0x7c42f090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6262f096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d42f098, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c2f09b, SUC_lexicon_tags_17 },
    { 0x5582f09b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3362f0a0, SUC_lexicon_tags_2 },
    { 0x1c2f0a0, SUC_lexicon_tags_10 },
    { 0x8082f0a1, SUC_lexicon_tags_11 },
    { 0xb242f0a3, SUC_lexicon_tags_48 },
    { 0x3582f0a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e02f0a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a42f0aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7562f0ac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa622f0b3, SUC_lexicon_tags_125 },
    { 0x59e2f0b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee02f0b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e2f0bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa2f0c1, SUC_lexicon_tags_14 },
    { 0x2b02f0c1, SUC_lexicon_tags_30 },
    { 0x3f02f0c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e2f0ce, SUC_lexicon_tags_24 },
    { 0x4e02f0cf, SUC_lexicon_tags_31 },
    { 0x8982f0d0, SUC_lexicon_tags_29 },
    { 0x7c02f0d1, SUC_lexicon_tags_38 },
    { 0xdc02f0d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842f0d6, SUC_lexicon_tags_55 },
    { 0xb322f0d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcb82f0d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbf62f0db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca82f0e4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd382f0e7, SUC_lexicon_tags_19 },
    { 0xb9e2f0e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9162f0eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc2f0f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc662f0f4, SUC_lexicon_tags_11 },
    { 0x40c2f0f5, SUC_lexicon_tags_8 },
    { 0x2182f0f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62f0fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa7a2f0ff, SUC_lexicon_tags_25 },
    { 0xe282f100, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c2f104, SUC_lexicon_tags_25 },
    { 0xcc22f105, SUC_lexicon_tags_11 },
    { 0x4c22f106, SUC_lexicon_tags_14 },
    { 0x8c62f106, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3882f10b, SUC_lexicon_tags_9 },
    { 0x33a2f10c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c2f10e, SUC_lexicon_tags_68 },
    { 0xf282f10f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba62f111, SUC_lexicon_tags_26 },
    { 0x3882f112, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d42f119, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a2f11d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfee2f11f, SUC_lexicon_tags_45 },
    { 0xcf62f11f, SUC_lexicon_tags_29 },
    { 0x8442f121, SUC_lexicon_tags_25 },
    { 0x3fe2f122, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4902f125, SUC_lexicon_tags_37 },
    { 0x3682f126, SUC_lexicon_tags_45 },
    { 0x1602f127, SUC_lexicon_tags_1 },
    { 0xe202f128, SUC_lexicon_tags_28 },
    { 0x3aa2f127, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdf22f12b, SUC_lexicon_tags_47 },
    { 0x3e2f12b, SUC_lexicon_tags_37 },
    { 0xe742f12d, SUC_lexicon_tags_11 },
    { 0xda62f12e, SUC_lexicon_tags_19 },
    { 0xd1a2f12f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2702f133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e82f135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87c2f137, SUC_lexicon_tags_12 },
    { 0x6c02f138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe82f13b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9062f13f, SUC_lexicon_tags_15 },
    { 0x8f42f13f, SUC_lexicon_tags_7 },
    { 0x3fe2f141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a2f148, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6982f14a, SUC_lexicon_tags_17 },
    { 0xd742f14b, SUC_lexicon_tags_43 },
    { 0xef22f14b, SUC_lexicon_tags_47 },
    { 0x442f14c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a2f152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa2f155, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7782f15a, SUC_lexicon_tags_22 },
    { 0xaac2f15a, SUC_lexicon_tags_30 },
    { 0x8482f15c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b22f160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5002f164, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7c2f166, SUC_lexicon_tags_11 },
    { 0x98a2f167, SUC_lexicon_tags_8 },
    { 0xa822f168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9dc2f16a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x33c2f16c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac2f16f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf802f176, SUC_lexicon_tags_8 },
    { 0xee82f177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11a2f179, SUC_lexicon_tags_8 },
    { 0x2142f179, SUC_lexicon_tags_48 },
    { 0xa5c2f17b, SUC_lexicon_tags_14 },
    { 0x8ec2f17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5982f17f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6822f183, SUC_lexicon_tags_37 },
    { 0x42c2f184, SUC_lexicon_tags_40 },
    { 0xb442f185, SUC_lexicon_tags_17 },
    { 0x9262f186, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc742f189, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e02f18c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c2f18e, SUC_lexicon_tags_37 },
    { 0xc1c2f18e, SUC_lexicon_tags_83 },
    { 0x3122f190, SUC_lexicon_tags_17 },
    { 0x6962f191, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c2f198, SUC_lexicon_tags_2 },
    { 0x95c2f199, SUC_lexicon_tags_66 },
    { 0x482f198, SUC_lexicon_tags_2 },
    { 0xba2f19b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb962f19f, SUC_lexicon_tags_33 },
    { 0x9942f19f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe442f1a5, SUC_lexicon_tags_37 },
    { 0x2322f1a6, SUC_lexicon_tags_37 },
    { 0x3662f1a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa2f1ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbfe2f1af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9482f1b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a2f1b4, SUC_lexicon_tags_254 },
    { 0xe882f1b5, SUC_lexicon_tags_17 },
    { 0x5aa2f1b4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc722f1bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1422f1bf, SUC_lexicon_tags_8 },
    { 0xb602f1c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdd82f1c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xab22f1c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e2f1c9, SUC_lexicon_tags_33 },
    { 0xb142f1c9, SUC_lexicon_tags_11 },
    { 0xc2f1cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a82f1cf, SUC_lexicon_tags_2 },
    { 0x4342f1d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c2f1d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4182f1d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe322f1df, SUC_lexicon_tags_1 },
    { 0x48c2f1df, SUC_lexicon_tags_17 },
    { 0x9de2f1e1, SUC_lexicon_tags_31 },
    { 0x6182f1e0, SUC_lexicon_tags_1 },
    { 0xec42f1e3, SUC_lexicon_tags_12 },
    { 0x98a2f1e4, SUC_lexicon_tags_37 },
    { 0x5aa2f1e5, SUC_lexicon_tags_37 },
    { 0x1a42f1e6, SUC_lexicon_tags_1 },
    { 0x5aa2f1e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x85c2f1e9, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa322f1ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa822f1f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8342f1f7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c2f1fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c2f1fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b22f202, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa2f205, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e2f209, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd062f211, SUC_lexicon_tags_63 },
    { 0x7c42f212, SUC_lexicon_tags_1 },
    { 0x1c42f212, SUC_lexicon_tags_9 },
    { 0x2e82f214, SUC_lexicon_tags_37 },
    { 0xb642f211, SUC_lexicon_tags_12 },
    { 0xba82f211, SUC_lexicon_tags_37 },
    { 0x22a2f217, SUC_lexicon_tags_77 },
    { 0x682f217, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf22f21a, SUC_lexicon_tags_87 },
    { 0xdb02f21b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2122f21e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8962f220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa702f224, SUC_lexicon_tags_27 },
    { 0xdb82f225, SUC_lexicon_tags_60 },
    { 0xe342f226, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x35a2f228, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f82f22a, SUC_lexicon_tags_11 },
    { 0x4782f22b, SUC_lexicon_tags_17 },
    { 0x1442f22c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac2f22f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2762f231, SUC_lexicon_tags_31 },
    { 0x542f232, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a2f235, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf722f238, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2f23d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd402f23f, SUC_lexicon_tags_15 },
    { 0x5562f23f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b62f242, SUC_lexicon_tags_15 },
    { 0x2ee2f242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da2f24b, SUC_lexicon_tags_0 },
    { 0x5ac2f24c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b02f252, SUC_lexicon_tags_26 },
    { 0x2d62f252, SUC_lexicon_tags_15 },
    { 0x9e22f253, SUC_lexicon_tags_47 },
    { 0x8882f254, SUC_lexicon_tags_35 },
    { 0x53c2f253, SUC_lexicon_tags_1 },
    { 0x4742f257, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c2f25a, SUC_lexicon_tags_15 },
    { 0xff82f25a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf22f25f, SUC_lexicon_tags_33 },
    { 0xfc02f25f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8802f265, SUC_lexicon_tags_1 },
    { 0x8962f266, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c2f26c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e2f273, SUC_lexicon_tags_14 },
    { 0xe5a2f273, SUC_lexicon_tags_15 },
    { 0xe942f273, SUC_lexicon_tags_42 },
    { 0x2702f276, SUC_lexicon_tags_23 },
    { 0x72c2f276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a62f27c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a2f281, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff62f285, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f62f289, SUC_lexicon_tags_12 },
    { 0xee2f28a, SUC_lexicon_tags_32 },
    { 0x2d02f28b, SUC_lexicon_tags_12 },
    { 0xf322f28b, SUC_lexicon_tags_17 },
    { 0xfde2f289, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6662f292, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb842f294, SUC_lexicon_tags_8 },
    { 0x4082f295, SUC_lexicon_tags_60 },
    { 0x3be2f296, SUC_lexicon_tags_57 },
    { 0x4a62f297, SUC_lexicon_tags_12 },
    { 0x3622f298, SUC_lexicon_tags_30 },
    { 0x1fa2f298, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3182f29c, SUC_lexicon_tags_2 },
    { 0x3c62f29c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7502f2a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb02f2a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea2f2a9, SUC_lexicon_tags_17 },
    { 0x1342f2aa, SUC_lexicon_tags_79 },
    { 0x7962f2ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb42f2ad, SUC_lexicon_tags_17 },
    { 0x5ac2f2ae, SUC_lexicon_tags_11 },
    { 0xf142f2af, SUC_lexicon_tags_47 },
    { 0x8ca2f2b0, SUC_lexicon_tags_11 },
    { 0x3b42f2b0, SUC_lexicon_tags_0 },
    { 0x1c82f2b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1382f2b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c2f2b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e2f2c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6962f2c4, SUC_lexicon_tags_0 },
    { 0xd842f2c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d22f2c8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ac2f2ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf02f2d0, SUC_lexicon_tags_27 },
    { 0x2e42f2d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66a2f2d3, SUC_lexicon_tags_57 },
    { 0x16e2f2d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb22f2d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea82f2de, SUC_lexicon_tags_83 },
    { 0x7e82f2df, SUC_lexicon_tags_60 },
    { 0xe822f2e0, SUC_lexicon_tags_45 },
    { 0x7aa2f2e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a62f2e3, SUC_lexicon_tags_69 },
    { 0xef22f2e4, SUC_lexicon_tags_47 },
    { 0x2802f2e4, SUC_lexicon_tags_37 },
    { 0xcbc2f2e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b22f2eb, SUC_lexicon_tags_26 },
    { 0xabe2f2ec, SUC_lexicon_tags_45 },
    { 0x6282f2ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e2f2f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x76a2f2f5, SUC_lexicon_tags_61 },
    { 0xa4e2f2f5, SUC_lexicon_tags_17 },
    { 0x1d42f2f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3242f2fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5782f2fc, SUC_lexicon_tags_37 },
    { 0xb42f2fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b22f305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a2f308, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf702f30a, SUC_lexicon_tags_17 },
    { 0x482f30b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2482f311, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb962f314, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf962f317, SUC_lexicon_tags_69 },
    { 0x3222f317, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf742f31c, SUC_lexicon_tags_46 },
    { 0x49c2f31d, SUC_lexicon_tags_55 },
    { 0xdea2f31e, SUC_lexicon_tags_37 },
    { 0xe702f31e, SUC_lexicon_tags_37 },
    { 0x29e2f320, SUC_lexicon_tags_188 },
    { 0xf202f321, SUC_lexicon_tags_9 },
    { 0x5642f322, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2de2f324, SUC_lexicon_tags_45 },
    { 0x1782f325, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb22f32c, SUC_lexicon_tags_11 },
    { 0x8222f32d, SUC_lexicon_tags_15 },
    { 0x6262f32d, SUC_lexicon_tags_30 },
    { 0x8fa2f32f, SUC_lexicon_tags_36 },
    { 0x1d02f32e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c02f333, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9722f335, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab02f33b, SUC_lexicon_tags_31 },
    { 0xf722f33b, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf82f33f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece2f346, SUC_lexicon_tags_29 },
    { 0x2622f346, SUC_lexicon_tags_60 },
    { 0xc242f347, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd0a2f34a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae2f34f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1862f351, SUC_lexicon_tags_30 },
    { 0xeee2f352, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf902f356, SUC_lexicon_tags_25 },
    { 0x49a2f356, SUC_lexicon_tags_0 },
    { 0xf9a2f358, SUC_lexicon_tags_23 },
    { 0x9022f359, SUC_lexicon_tags_25 },
    { 0xb0a2f357, SUC_lexicon_tags_45 },
    { 0xf562f35b, SUC_lexicon_tags_17 },
    { 0x7422f35c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc442f362, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe322f364, SUC_lexicon_tags_45 },
    { 0x12a2f365, SUC_lexicon_tags_647 },
    { 0, NULL },
    { 0xb3c2f367, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa502f369, SUC_lexicon_tags_11 },
    { 0xec42f369, SUC_lexicon_tags_12 },
    { 0xe682f36b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x16a2f36d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f62f372, SUC_lexicon_tags_37 },
    { 0x2562f373, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc502f376, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1b82f378, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f82f37a, SUC_lexicon_tags_12 },
    { 0x9f22f37b, SUC_lexicon_tags_10 },
    { 0x762f37c, SUC_lexicon_tags_0 },
    { 0x88e2f37a, SUC_lexicon_tags_23 },
    { 0xe602f37a, SUC_lexicon_tags_47 },
    { 0x2c22f37c, SUC_lexicon_tags_28 },
    { 0x9082f380, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a2f38b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc2f392, SUC_lexicon_tags_31 },
    { 0xf342f393, SUC_lexicon_tags_33 },
    { 0xbb02f394, SUC_lexicon_tags_9 },
    { 0xb62f395, SUC_lexicon_tags_17 },
    { 0xe942f392, SUC_lexicon_tags_7 },
    { 0x9e82f396, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9942f39b, SUC_lexicon_tags_11 },
    { 0xe222f39c, SUC_lexicon_tags_7 },
    { 0x9d62f39c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be2f3a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea2f3a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be2f3a9, SUC_lexicon_tags_16 },
    { 0xaac2f3aa, SUC_lexicon_tags_38 },
    { 0x68e2f3ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc42f3ae, SUC_lexicon_tags_2 },
    { 0x4622f3af, SUC_lexicon_tags_17 },
    { 0xe6a2f3b0, SUC_lexicon_tags_2 },
    { 0x1982f3af, SUC_lexicon_tags_1 },
    { 0xf782f3b0, SUC_lexicon_tags_25 },
    { 0x29c2f3b0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe2f3b6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a2f3bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c2f3c0, SUC_lexicon_tags_1 },
    { 0x1722f3c0, SUC_lexicon_tags_32 },
    { 0x2c2f3c2, SUC_lexicon_tags_29 },
    { 0x8622f3c2, SUC_lexicon_tags_1 },
    { 0xee62f3c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc22f3c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa282f3cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e2f3d0, SUC_lexicon_tags_1 },
    { 0xac02f3d1, SUC_lexicon_tags_60 },
    { 0x32e2f3d1, SUC_lexicon_tags_37 },
    { 0x1c82f3d3, SUC_lexicon_tags_47 },
    { 0x3242f3d0, SUC_lexicon_tags_7 },
    { 0x55a2f3d5, SUC_lexicon_tags_1 },
    { 0xb5a2f3d0, SUC_lexicon_tags_151 },
    { 0xeb82f3d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bc2f3d9, SUC_lexicon_tags_0 },
    { 0x4d82f3d9, SUC_lexicon_tags_28 },
    { 0xc142f3db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca2f3df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe2f3e4, SUC_lexicon_tags_26 },
    { 0xb582f3e4, SUC_lexicon_tags_17 },
    { 0x8c42f3e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa2f3ec, SUC_lexicon_tags_25 },
    { 0x9682f3ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2362f3f0, SUC_lexicon_tags_45 },
    { 0xf6c2f3f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0a2f3f3, SUC_lexicon_tags_25 },
    { 0xce62f3f4, SUC_lexicon_tags_15 },
    { 0x45c2f3f4, SUC_lexicon_tags_37 },
    { 0x4b02f3f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xada2f3f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc922f3fc, SUC_lexicon_tags_55 },
    { 0xd742f3fd, SUC_lexicon_tags_13 },
    { 0xfe62f3fe, SUC_lexicon_tags_7 },
    { 0xec82f3fe, SUC_lexicon_tags_7 },
    { 0x20c2f3ff, SUC_lexicon_tags_15 },
    { 0xfa42f401, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e2f405, SUC_lexicon_tags_19 },
    { 0x922f405, SUC_lexicon_tags_39 },
    { 0x7fc2f407, SUC_lexicon_tags_17 },
    { 0x42e2f408, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcd42f40a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc2f411, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2f414, SUC_lexicon_tags_25 },
    { 0x67e2f415, SUC_lexicon_tags_29 },
    { 0xef22f416, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8442f418, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd042f41b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a42f41e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2f42d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa922f433, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10a2f435, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f02f43b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdae2f43d, SUC_lexicon_tags_39 },
    { 0x8002f43e, SUC_lexicon_tags_61 },
    { 0x4642f43f, SUC_lexicon_tags_21 },
    { 0x3a22f440, SUC_lexicon_tags_17 },
    { 0xa7e2f441, SUC_lexicon_tags_63 },
    { 0xc122f442, SUC_lexicon_tags_12 },
    { 0x71e2f443, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5462f446, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a2f44b, SUC_lexicon_tags_25 },
    { 0x36a2f44c, SUC_lexicon_tags_82 },
    { 0x7382f44d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x4282f450, SUC_lexicon_tags_40 },
    { 0x6fa2f450, SUC_lexicon_tags_1581 },
    { 0, NULL },
    { 0x38c2f453, SUC_lexicon_tags_15 },
    { 0xeee2f454, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7822f45b, SUC_lexicon_tags_2 },
    { 0xa22f45c, SUC_lexicon_tags_11 },
    { 0x64a2f45d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a62f464, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ba2f466, SUC_lexicon_tags_1 },
    { 0xce22f466, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2262f46d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6ec2f46f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd542f474, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe42f477, SUC_lexicon_tags_38 },
    { 0x1c2f478, SUC_lexicon_tags_82 },
    { 0xc8c2f479, SUC_lexicon_tags_5 },
    { 0xc2e2f47a, SUC_lexicon_tags_26 },
    { 0xd802f47a, SUC_lexicon_tags_5 },
    { 0x2cc2f477, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e42f47e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec82f482, SUC_lexicon_tags_6 },
    { 0x9202f482, SUC_lexicon_tags_25 },
    { 0xc082f483, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1a22f486, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb62f488, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6382f48a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9582f491, SUC_lexicon_tags_1 },
    { 0x3102f492, SUC_lexicon_tags_12 },
    { 0x42a2f493, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5182f496, SUC_lexicon_tags_46 },
    { 0x5d62f497, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b62f499, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ba2f49b, SUC_lexicon_tags_30 },
    { 0x8562f49b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac2f49f, SUC_lexicon_tags_15 },
    { 0x9402f4a0, SUC_lexicon_tags_12 },
    { 0x53c2f4a1, SUC_lexicon_tags_47 },
    { 0xcfc2f4a1, SUC_lexicon_tags_15 },
    { 0x5ea2f4a3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe3a2f4a5, SUC_lexicon_tags_8 },
    { 0x75e2f4a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf82f4aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xede2f4ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6362f4b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e2f4b9, SUC_lexicon_tags_7 },
    { 0x1e42f4ba, SUC_lexicon_tags_19 },
    { 0xa262f4ba, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5322f4bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e2f4c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a2f4c8, SUC_lexicon_tags_11 },
    { 0x9742f4c9, SUC_lexicon_tags_37 },
    { 0xea22f4ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab42f4d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd822f4d5, SUC_lexicon_tags_17 },
    { 0x3ec2f4d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a2f4d9, SUC_lexicon_tags_63 },
    { 0x4662f4da, SUC_lexicon_tags_130 },
    { 0xb422f4db, SUC_lexicon_tags_15 },
    { 0x8442f4d9, SUC_lexicon_tags_7 },
    { 0xea02f4dd, SUC_lexicon_tags_83 },
    { 0x4342f4dd, SUC_lexicon_tags_11 },
    { 0x6ac2f4dd, SUC_lexicon_tags_17 },
    { 0x28a2f4e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c2f4e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1682f4e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e42f4eb, SUC_lexicon_tags_46 },
    { 0x282f4ec, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb862f4f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c2f4fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a02f500, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c2f507, SUC_lexicon_tags_7 },
    { 0xfb42f508, SUC_lexicon_tags_17 },
    { 0x32a2f508, SUC_lexicon_tags_29 },
    { 0xea22f50a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd82f50d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c62f50f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa502f516, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82f51c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6782f521, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e2f524, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x12a2f526, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f62f528, SUC_lexicon_tags_23 },
    { 0xca02f529, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaba2f52b, SUC_lexicon_tags_10 },
    { 0xf962f52b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf262f52e, SUC_lexicon_tags_63 },
    { 0x942f52f, SUC_lexicon_tags_23 },
    { 0x8fe2f52f, SUC_lexicon_tags_25 },
    { 0xcb42f531, SUC_lexicon_tags_45 },
    { 0x4802f532, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0a2f534, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa042f537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe142f53a, SUC_lexicon_tags_12 },
    { 0x63c2f53b, SUC_lexicon_tags_15 },
    { 0x2da2f53a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb422f53e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd8c2f540, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc2f544, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e2f54d, SUC_lexicon_tags_1 },
    { 0x9222f54d, SUC_lexicon_tags_15 },
    { 0xf9e2f54e, SUC_lexicon_tags_12 },
    { 0xc802f550, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5462f556, SUC_lexicon_tags_15 },
    { 0xa962f556, SUC_lexicon_tags_0 },
    { 0x7722f557, SUC_lexicon_tags_18 },
    { 0x50a2f559, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe02f55f, SUC_lexicon_tags_127 },
    { 0xe382f55f, SUC_lexicon_tags_12 },
    { 0xd062f560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1b62f563, SUC_lexicon_tags_29 },
    { 0x7da2f564, SUC_lexicon_tags_38 },
    { 0x1c62f565, SUC_lexicon_tags_1 },
    { 0x20c2f564, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c2f569, SUC_lexicon_tags_49 },
    { 0xe82f569, SUC_lexicon_tags_48 },
    { 0xefe2f56b, SUC_lexicon_tags_115 },
    { 0x9ec2f569, SUC_lexicon_tags_45 },
    { 0x8fc2f56b, SUC_lexicon_tags_2 },
    { 0x5782f56c, SUC_lexicon_tags_57 },
    { 0x642f56f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda2f574, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b22f578, SUC_lexicon_tags_18 },
    { 0xf122f579, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed62f57b, SUC_lexicon_tags_114 },
    { 0x1442f57c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa162f580, SUC_lexicon_tags_17 },
    { 0x5be2f581, SUC_lexicon_tags_11 },
    { 0x9422f582, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7cc2f584, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d62f58b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c2f58e, SUC_lexicon_tags_23 },
    { 0x10c2f58e, SUC_lexicon_tags_12 },
    { 0xd0c2f590, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b82f595, SUC_lexicon_tags_77 },
    { 0xf9c2f596, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf42f598, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7522f59f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6822f5a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c2f5a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c2f5ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a22f5ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3622f5b0, SUC_lexicon_tags_11 },
    { 0x8762f5b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e42f5b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2502f5b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c2f5bb, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x6ea2f5bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c2f5c0, SUC_lexicon_tags_15 },
    { 0x142f5c1, SUC_lexicon_tags_55 },
    { 0xf442f5c2, SUC_lexicon_tags_66 },
    { 0xc182f5c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a2f5c7, SUC_lexicon_tags_2 },
    { 0xd422f5c8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf962f5cb, SUC_lexicon_tags_1 },
    { 0x3142f5cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1382f5ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf3e2f5d0, SUC_lexicon_tags_0 },
    { 0xf242f5d1, SUC_lexicon_tags_17 },
    { 0xb682f5d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9162f5d8, SUC_lexicon_tags_23 },
    { 0x402f5d9, SUC_lexicon_tags_15 },
    { 0x4162f5da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18e2f5dc, SUC_lexicon_tags_32 },
    { 0xe4c2f5dd, SUC_lexicon_tags_27 },
    { 0xa142f5dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd282f5e3, SUC_lexicon_tags_14 },
    { 0xd902f5e3, SUC_lexicon_tags_15 },
    { 0x79a2f5e4, SUC_lexicon_tags_61 },
    { 0xfd42f5e6, SUC_lexicon_tags_3 },
    { 0x4962f5e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba2f5ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6422f5ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d62f5f0, SUC_lexicon_tags_49 },
    { 0xa202f5f0, SUC_lexicon_tags_7 },
    { 0x85e2f5f2, SUC_lexicon_tags_30 },
    { 0xca22f5f3, SUC_lexicon_tags_17 },
    { 0xcf62f5f4, SUC_lexicon_tags_15 },
    { 0xba02f5f5, SUC_lexicon_tags_17 },
    { 0xbba2f5f4, SUC_lexicon_tags_74 },
    { 0xc182f5f7, SUC_lexicon_tags_41 },
    { 0x4522f5f1, SUC_lexicon_tags_47 },
    { 0xdfe2f5f5, SUC_lexicon_tags_38 },
    { 0xcb82f5f5, SUC_lexicon_tags_12 },
    { 0x9582f5f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf62f5ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c2f603, SUC_lexicon_tags_79 },
    { 0x6302f604, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf82f607, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x562f60a, SUC_lexicon_tags_1 },
    { 0x2982f60b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ee2f60d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e2f610, SUC_lexicon_tags_28 },
    { 0x9222f610, SUC_lexicon_tags_28 },
    { 0xbb82f612, SUC_lexicon_tags_6 },
    { 0x3722f612, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e62f616, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7142f619, SUC_lexicon_tags_55 },
    { 0x7082f619, SUC_lexicon_tags_37 },
    { 0x482f61b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe402f620, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e2f627, SUC_lexicon_tags_30 },
    { 0xe6e2f628, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee2f62c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa22f632, SUC_lexicon_tags_7 },
    { 0x8822f633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7e2f635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a2f638, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4142f63f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xada2f642, SUC_lexicon_tags_25 },
    { 0xc7c2f643, SUC_lexicon_tags_30 },
    { 0x5de2f644, SUC_lexicon_tags_43 },
    { 0xaaa2f643, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc2f648, SUC_lexicon_tags_61 },
    { 0x22c2f649, SUC_lexicon_tags_6 },
    { 0x46a2f649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe42f64c, SUC_lexicon_tags_25 },
    { 0x48c2f64c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa342f653, SUC_lexicon_tags_11 },
    { 0x8e82f654, SUC_lexicon_tags_1 },
    { 0xa502f654, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb482f659, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3a2f65b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d42f65f, SUC_lexicon_tags_12 },
    { 0xc6e2f660, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf02f664, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd0a2f666, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3b42f668, SUC_lexicon_tags_2 },
    { 0xb142f669, SUC_lexicon_tags_37 },
    { 0xfee2f66a, SUC_lexicon_tags_37 },
    { 0x24c2f66b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5482f67b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac82f681, SUC_lexicon_tags_17 },
    { 0x3202f682, SUC_lexicon_tags_7 },
    { 0x8d62f682, SUC_lexicon_tags_29 },
    { 0x3c62f681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fc2f686, SUC_lexicon_tags_17 },
    { 0x6f62f686, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3a82f689, SUC_lexicon_tags_3 },
    { 0xcd82f68a, SUC_lexicon_tags_15 },
    { 0x43c2f68a, SUC_lexicon_tags_108 },
    { 0xa0c2f689, SUC_lexicon_tags_30 },
    { 0x8022f689, SUC_lexicon_tags_11 },
    { 0x54e2f68e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2f691, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb502f693, SUC_lexicon_tags_23 },
    { 0x3ce2f693, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbcc2f696, SUC_lexicon_tags_11 },
    { 0xfe42f697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b82f699, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2122f69d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xca02f6a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb642f6a3, SUC_lexicon_tags_2 },
    { 0xd4a2f6a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d82f6a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf22f6ac, SUC_lexicon_tags_45 },
    { 0x8fc2f6ad, SUC_lexicon_tags_87 },
    { 0x7c62f6ae, SUC_lexicon_tags_37 },
    { 0x7b02f6af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a2f6b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe402f6b7, SUC_lexicon_tags_8 },
    { 0x5042f6b8, SUC_lexicon_tags_43 },
    { 0xfe62f6b8, SUC_lexicon_tags_1 },
    { 0xd1e2f6b7, SUC_lexicon_tags_37 },
    { 0x8e42f6b8, SUC_lexicon_tags_23 },
    { 0xc1a2f6bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6082f6c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe082f6cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9602f6cd, SUC_lexicon_tags_33 },
    { 0x93a2f6ce, SUC_lexicon_tags_24 },
    { 0x2622f6ce, SUC_lexicon_tags_15 },
    { 0x47e2f6d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d42f6d4, SUC_lexicon_tags_30 },
    { 0x3c02f6d5, SUC_lexicon_tags_77 },
    { 0xb2a2f6d4, SUC_lexicon_tags_38 },
    { 0x5862f6d5, SUC_lexicon_tags_1 },
    { 0x40e2f6d8, SUC_lexicon_tags_6 },
    { 0x9562f6d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe2f6e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de2f6e3, SUC_lexicon_tags_60 },
    { 0xc962f6e3, SUC_lexicon_tags_0 },
    { 0xeb42f6e5, SUC_lexicon_tags_26 },
    { 0x962f6e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc2f6ea, SUC_lexicon_tags_43 },
    { 0x8362f6eb, SUC_lexicon_tags_0 },
    { 0xa6e2f6eb, SUC_lexicon_tags_45 },
    { 0x6222f6ed, SUC_lexicon_tags_4 },
    { 0x6e82f6ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdee2f6f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d02f6f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9042f6fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b22f6ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b42f701, SUC_lexicon_tags_13 },
    { 0xd5c2f702, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec2f705, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb942f707, SUC_lexicon_tags_14 },
    { 0x7e02f707, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec2f70b, SUC_lexicon_tags_48 },
    { 0x92e2f70c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb2a2f70e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa282f714, SUC_lexicon_tags_37 },
    { 0x2f42f715, SUC_lexicon_tags_31 },
    { 0xf022f716, SUC_lexicon_tags_2 },
    { 0x3ec2f716, SUC_lexicon_tags_26 },
    { 0xc142f715, SUC_lexicon_tags_60 },
    { 0xb022f716, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c2f71e, SUC_lexicon_tags_7 },
    { 0xbcc2f71f, SUC_lexicon_tags_47 },
    { 0x2862f720, SUC_lexicon_tags_10 },
    { 0x46c2f71f, SUC_lexicon_tags_1 },
    { 0x8de2f722, SUC_lexicon_tags_17 },
    { 0xfdc2f71e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa642f728, SUC_lexicon_tags_17 },
    { 0xc322f729, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c2f72e, SUC_lexicon_tags_13 },
    { 0x1402f72f, SUC_lexicon_tags_37 },
    { 0x3e82f730, SUC_lexicon_tags_7 },
    { 0x2b42f730, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7162f737, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeb22f739, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8142f73d, SUC_lexicon_tags_27 },
    { 0xac62f73e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1a62f740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8842f749, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3662f74c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa042f75a, SUC_lexicon_tags_57 },
    { 0xba02f75b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdb62f75d, SUC_lexicon_tags_17 },
    { 0x4402f75d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd142f765, SUC_lexicon_tags_65 },
    { 0x9662f766, SUC_lexicon_tags_9 },
    { 0xe942f767, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c2f76a, SUC_lexicon_tags_17 },
    { 0x1062f76b, SUC_lexicon_tags_7 },
    { 0x5ca2f76a, SUC_lexicon_tags_43 },
    { 0xdc42f76a, SUC_lexicon_tags_6 },
    { 0x4022f76e, SUC_lexicon_tags_58 },
    { 0xdb42f76e, SUC_lexicon_tags_16 },
    { 0x9fa2f76b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2f773, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8f82f775, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3922f77c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc162f77e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4282f784, SUC_lexicon_tags_11 },
    { 0x4982f784, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x75a2f787, SUC_lexicon_tags_10 },
    { 0x9cc2f788, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf482f78d, SUC_lexicon_tags_1 },
    { 0xe5e2f78e, SUC_lexicon_tags_49 },
    { 0x82c2f78f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32c2f791, SUC_lexicon_tags_62 },
    { 0xe362f792, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a2f79a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb4e2f79c, SUC_lexicon_tags_63 },
    { 0x8e22f79d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2f7a0, SUC_lexicon_tags_2 },
    { 0x2dc2f7a0, SUC_lexicon_tags_6 },
    { 0x2902f7a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c2f7a5, SUC_lexicon_tags_66 },
    { 0xaf02f7a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6922f7a8, SUC_lexicon_tags_11 },
    { 0x6402f7a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b42f7ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c2f7b1, SUC_lexicon_tags_14 },
    { 0x1fa2f7b2, SUC_lexicon_tags_23 },
    { 0xa122f7b3, SUC_lexicon_tags_15 },
    { 0x1f62f7b4, SUC_lexicon_tags_33 },
    { 0xf022f7b4, SUC_lexicon_tags_12 },
    { 0xbc62f7b6, SUC_lexicon_tags_15 },
    { 0xba42f7b4, SUC_lexicon_tags_542 },
    { 0xa282f7b5, SUC_lexicon_tags_15 },
    { 0x9822f7b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e2f7bc, SUC_lexicon_tags_77 },
    { 0x7642f7bd, SUC_lexicon_tags_15 },
    { 0x4a42f7bd, SUC_lexicon_tags_25 },
    { 0x7da2f7bc, SUC_lexicon_tags_1 },
    { 0x8e42f7c0, SUC_lexicon_tags_11 },
    { 0x4902f7c1, SUC_lexicon_tags_11 },
    { 0x1442f7c2, SUC_lexicon_tags_2 },
    { 0x9442f7c0, SUC_lexicon_tags_6 },
    { 0x5362f7bf, SUC_lexicon_tags_6 },
    { 0x6f82f7c5, SUC_lexicon_tags_14 },
    { 0x1ce2f7bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f42f7ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d62f7cd, SUC_lexicon_tags_7 },
    { 0x9662f7ce, SUC_lexicon_tags_61 },
    { 0x82a2f7cd, SUC_lexicon_tags_0 },
    { 0xb542f7d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa722f7d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9882f7d5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d02f7d8, SUC_lexicon_tags_12 },
    { 0xdf62f7d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b22f7e0, SUC_lexicon_tags_17 },
    { 0x11a2f7e1, SUC_lexicon_tags_25 },
    { 0x1f42f7e2, SUC_lexicon_tags_48 },
    { 0xa6c2f7e0, SUC_lexicon_tags_29 },
    { 0x4202f7e1, SUC_lexicon_tags_15 },
    { 0xf882f7e5, SUC_lexicon_tags_39 },
    { 0x2d62f7e6, SUC_lexicon_tags_17 },
    { 0xfdc2f7e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4042f7ea, SUC_lexicon_tags_6 },
    { 0xc6e2f7eb, SUC_lexicon_tags_2 },
    { 0xdd62f7eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e22f7f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf402f7f3, SUC_lexicon_tags_15 },
    { 0x782f7f4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3ee2f7f6, SUC_lexicon_tags_60 },
    { 0x8082f7f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2f7fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa2f7fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc002f7ff, SUC_lexicon_tags_65 },
    { 0x5482f800, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3002f802, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa402f804, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce2f80b, SUC_lexicon_tags_7 },
    { 0x3282f80b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe102f810, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf882f812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe462f815, SUC_lexicon_tags_47 },
    { 0xf82f815, SUC_lexicon_tags_45 },
    { 0x8a02f815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7382f81b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8462f81d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe142f826, SUC_lexicon_tags_25 },
    { 0x2182f826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe302f832, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca2f835, SUC_lexicon_tags_12 },
    { 0x52a2f836, SUC_lexicon_tags_25 },
    { 0x5cc2f836, SUC_lexicon_tags_29 },
    { 0x27a2f835, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9902f83b, SUC_lexicon_tags_6 },
    { 0xc582f83b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe782f83f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d22f846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa22f848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bc2f84a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a2f84f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc82f852, SUC_lexicon_tags_28 },
    { 0xf262f853, SUC_lexicon_tags_37 },
    { 0xb862f854, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2f85b, SUC_lexicon_tags_37 },
    { 0xf942f85c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd842f85e, SUC_lexicon_tags_25 },
    { 0x7982f85e, SUC_lexicon_tags_37 },
    { 0x8bc2f85f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddc2f862, SUC_lexicon_tags_30 },
    { 0x42f863, SUC_lexicon_tags_17 },
    { 0x6da2f864, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3e2f866, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da2f86b, SUC_lexicon_tags_33 },
    { 0xc882f86c, SUC_lexicon_tags_23 },
    { 0x6582f86d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2f872, SUC_lexicon_tags_37 },
    { 0x6b82f873, SUC_lexicon_tags_14 },
    { 0x4fe2f874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1522f877, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2742f881, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e2f889, SUC_lexicon_tags_23 },
    { 0xa902f88a, SUC_lexicon_tags_7 },
    { 0x96a2f88b, SUC_lexicon_tags_17 },
    { 0x8362f88b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2622f88e, SUC_lexicon_tags_47 },
    { 0x5162f88e, SUC_lexicon_tags_2 },
    { 0xcc02f88f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e62f892, SUC_lexicon_tags_28 },
    { 0x37a2f893, SUC_lexicon_tags_1 },
    { 0x1122f893, SUC_lexicon_tags_10 },
    { 0xd882f894, SUC_lexicon_tags_43 },
    { 0xfec2f892, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f82f899, SUC_lexicon_tags_26 },
    { 0xa402f89a, SUC_lexicon_tags_3 },
    { 0x7be2f89a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a02f8a1, SUC_lexicon_tags_14 },
    { 0xe82f8a1, SUC_lexicon_tags_6 },
    { 0x3b42f8a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1382f8a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c2f8a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc722f8ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6642f8ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a2f8b2, SUC_lexicon_tags_37 },
    { 0x6a82f8b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3802f8b5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdc02f8b7, SUC_lexicon_tags_83 },
    { 0x4ae2f8b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa2f8bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4862f8bd, SUC_lexicon_tags_14 },
    { 0x2cc2f8bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc2f8c1, SUC_lexicon_tags_45 },
    { 0xa382f8c2, SUC_lexicon_tags_48 },
    { 0x33e2f8c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2942f8c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc42f8ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a2f8cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd1a2f8cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fa2f8d1, SUC_lexicon_tags_30 },
    { 0xc842f8d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7082f8d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4282f8d7, SUC_lexicon_tags_26 },
    { 0xe202f8d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de2f8de, SUC_lexicon_tags_1 },
    { 0x7802f8de, SUC_lexicon_tags_37 },
    { 0xa902f8df, SUC_lexicon_tags_17 },
    { 0x5582f8df, SUC_lexicon_tags_11 },
    { 0x3682f8e2, SUC_lexicon_tags_23 },
    { 0xa6e2f8e2, SUC_lexicon_tags_63 },
    { 0x7de2f8e4, SUC_lexicon_tags_38 },
    { 0x2842f8e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d22f8ea, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be2f8f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc302f900, SUC_lexicon_tags_16 },
    { 0xe402f901, SUC_lexicon_tags_37 },
    { 0x76c2f902, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc42f906, SUC_lexicon_tags_92 },
    { 0x4bc2f907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd002f90c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8322f910, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e2f913, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd622f91b, SUC_lexicon_tags_28 },
    { 0xdf62f91c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc822f920, SUC_lexicon_tags_19 },
    { 0xee62f921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfe2f923, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2f925, SUC_lexicon_tags_11 },
    { 0xda2f926, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc2f92c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2f933, SUC_lexicon_tags_47 },
    { 0x922f934, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcdc2f936, SUC_lexicon_tags_31 },
    { 0xb462f936, SUC_lexicon_tags_43 },
    { 0x5f22f937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c2f93b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3402f93f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82f944, SUC_lexicon_tags_87 },
    { 0x7842f944, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5062f949, SUC_lexicon_tags_17 },
    { 0x84e2f94a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84a2f94c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b42f94f, SUC_lexicon_tags_15 },
    { 0x3382f950, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7e2f952, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda02f958, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xa4e2f95a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb42f95c, SUC_lexicon_tags_18 },
    { 0xb42f95d, SUC_lexicon_tags_82 },
    { 0xbca2f95e, SUC_lexicon_tags_35 },
    { 0x1422f95e, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd102f966, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf222f96a, SUC_lexicon_tags_26 },
    { 0x9ce2f96a, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x2b02f96d, SUC_lexicon_tags_12 },
    { 0xba2f96e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xef02f970, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8822f975, SUC_lexicon_tags_11 },
    { 0xdc2f976, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc082f978, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a2f97b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf382f980, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182f984, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd102f986, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc2f988, SUC_lexicon_tags_9 },
    { 0xd862f989, SUC_lexicon_tags_37 },
    { 0x3962f988, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce2f98e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9942f993, SUC_lexicon_tags_1 },
    { 0x8182f994, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a62f997, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa682f999, SUC_lexicon_tags_14 },
    { 0x722f99a, SUC_lexicon_tags_2 },
    { 0x6e62f99b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8562f99f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c2f9a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4982f9a5, SUC_lexicon_tags_30 },
    { 0x21e2f9a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a2f9aa, SUC_lexicon_tags_17 },
    { 0xa222f9aa, SUC_lexicon_tags_37 },
    { 0x7582f9ac, SUC_lexicon_tags_57 },
    { 0x1422f9ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa2f9b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf82f9b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc2f9b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee62f9c4, SUC_lexicon_tags_6 },
    { 0xdce2f9c5, SUC_lexicon_tags_92 },
    { 0xbbe2f9c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3422f9c9, SUC_lexicon_tags_1 },
    { 0x56c2f9ca, SUC_lexicon_tags_17 },
    { 0x5ae2f9c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe922f9ce, SUC_lexicon_tags_25 },
    { 0xd562f9cf, SUC_lexicon_tags_63 },
    { 0x8962f9d0, SUC_lexicon_tags_29 },
    { 0xa122f9d1, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x2c2f9d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7a62f9d5, SUC_lexicon_tags_10 },
    { 0xe842f9d6, SUC_lexicon_tags_15 },
    { 0xd282f9d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2f9dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb02f9de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8202f9e3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c82f9e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae2f9eb, SUC_lexicon_tags_8 },
    { 0x34e2f9ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e2f9f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x14a2f9f6, SUC_lexicon_tags_28 },
    { 0x2b02f9f7, SUC_lexicon_tags_25 },
    { 0x4bc2f9f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf902f9fa, SUC_lexicon_tags_17 },
    { 0x5b02f9fb, SUC_lexicon_tags_0 },
    { 0x5262f9fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5582f9ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2022fa01, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5462fa03, SUC_lexicon_tags_23 },
    { 0x4c82fa04, SUC_lexicon_tags_45 },
    { 0xc482fa05, SUC_lexicon_tags_12 },
    { 0x802fa06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6862fa09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc242fa12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff62fa17, SUC_lexicon_tags_14 },
    { 0x3ac2fa17, SUC_lexicon_tags_6 },
    { 0xb8e2fa17, SUC_lexicon_tags_7 },
    { 0xcce2fa1a, SUC_lexicon_tags_2 },
    { 0xec62fa17, SUC_lexicon_tags_17 },
    { 0x50a2fa1b, SUC_lexicon_tags_31 },
    { 0xb222fa1d, SUC_lexicon_tags_49 },
    { 0xe122fa1e, SUC_lexicon_tags_56 },
    { 0x5162fa1f, SUC_lexicon_tags_37 },
    { 0xc0e2fa1b, SUC_lexicon_tags_0 },
    { 0xcc62fa21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e2fa2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c02fa30, SUC_lexicon_tags_25 },
    { 0x3f02fa31, SUC_lexicon_tags_68 },
    { 0x4b02fa32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e2fa3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e2fa42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x182fa45, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e2fa49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f62fa4c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e2fa4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1882fa56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8a02fa58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2fa5d, SUC_lexicon_tags_11 },
    { 0xdc22fa5e, SUC_lexicon_tags_10 },
    { 0x8f02fa5f, SUC_lexicon_tags_11 },
    { 0x30c2fa5f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a82fa70, SUC_lexicon_tags_31 },
    { 0x642fa71, SUC_lexicon_tags_380 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c2fa74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xec02fa76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2c02fa78, SUC_lexicon_tags_30 },
    { 0x3ae2fa79, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2762fa80, SUC_lexicon_tags_87 },
    { 0x6342fa81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1682fa83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe2fa86, SUC_lexicon_tags_49 },
    { 0x7c62fa87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc2fa8c, SUC_lexicon_tags_1 },
    { 0xe582fa8c, SUC_lexicon_tags_47 },
    { 0x24c2fa8d, SUC_lexicon_tags_37 },
    { 0x7322fa8d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662fa97, SUC_lexicon_tags_56 },
    { 0xd5c2fa98, SUC_lexicon_tags_26 },
    { 0x182fa98, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e2fa9c, SUC_lexicon_tags_60 },
    { 0x11a2fa9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd862faa2, SUC_lexicon_tags_106 },
    { 0x922faa3, SUC_lexicon_tags_25 },
    { 0x8842faa4, SUC_lexicon_tags_49 },
    { 0xf342faa3, SUC_lexicon_tags_1 },
    { 0xef22faa5, SUC_lexicon_tags_0 },
    { 0x5e22faa3, SUC_lexicon_tags_8 },
    { 0x6262faa8, SUC_lexicon_tags_63 },
    { 0x9d42faa3, SUC_lexicon_tags_37 },
    { 0x6d22faa6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb982faac, SUC_lexicon_tags_47 },
    { 0x73c2faac, SUC_lexicon_tags_11 },
    { 0xc642faac, SUC_lexicon_tags_43 },
    { 0x9642faaf, SUC_lexicon_tags_30 },
    { 0x1842fab0, SUC_lexicon_tags_8 },
    { 0xc8a2fab1, SUC_lexicon_tags_11 },
    { 0x2fab2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7aa2fab4, SUC_lexicon_tags_9 },
    { 0x1042fab5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd702fabd, SUC_lexicon_tags_26 },
    { 0xddc2fabd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdaa2fac0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0a2fac2, SUC_lexicon_tags_63 },
    { 0x8282fac3, SUC_lexicon_tags_87 },
    { 0xcc22fac2, SUC_lexicon_tags_7 },
    { 0x3582fac4, SUC_lexicon_tags_65 },
    { 0xcda2fac3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x12c2fac8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e2facf, SUC_lexicon_tags_6 },
    { 0x6c42fad0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b22fad3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d42fadf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe162fae3, SUC_lexicon_tags_235 },
    { 0xaac2fae4, SUC_lexicon_tags_12 },
    { 0x1262fae3, SUC_lexicon_tags_86 },
    { 0xc962fae6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd182fae9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f82faee, SUC_lexicon_tags_46 },
    { 0xc862faef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3182faf4, SUC_lexicon_tags_11 },
    { 0x4922faf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a2faf8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba2fafe, SUC_lexicon_tags_63 },
    { 0xa2fafe, SUC_lexicon_tags_158 },
    { 0x41a2fb00, SUC_lexicon_tags_25 },
    { 0xce62faff, SUC_lexicon_tags_15 },
    { 0xac02fb02, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x64e2fb04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf662fb06, SUC_lexicon_tags_32 },
    { 0xa882fb07, SUC_lexicon_tags_17 },
    { 0xbdc2fb08, SUC_lexicon_tags_37 },
    { 0x5042fb09, SUC_lexicon_tags_37 },
    { 0x1dc2fb08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a2fb12, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd222fb19, SUC_lexicon_tags_4 },
    { 0x2942fb1a, SUC_lexicon_tags_11 },
    { 0xc742fb1b, SUC_lexicon_tags_37 },
    { 0xc162fb1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1862fb1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c42fb24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc02fb27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc22fb2a, SUC_lexicon_tags_14 },
    { 0x9342fb2b, SUC_lexicon_tags_43 },
    { 0xa9a2fb2c, SUC_lexicon_tags_11 },
    { 0xc162fb2b, SUC_lexicon_tags_7 },
    { 0x6e42fb2d, SUC_lexicon_tags_23 },
    { 0xca82fb2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde2fb34, SUC_lexicon_tags_7 },
    { 0xc282fb34, SUC_lexicon_tags_17 },
    { 0xdf22fb34, SUC_lexicon_tags_37 },
    { 0x1d62fb37, SUC_lexicon_tags_10 },
    { 0x4ec2fb34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42a2fb3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a2fb3f, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xd6c2fb41, SUC_lexicon_tags_140 },
    { 0x97c2fb42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7122fb44, SUC_lexicon_tags_12 },
    { 0x8f02fb45, SUC_lexicon_tags_23 },
    { 0x3882fb45, SUC_lexicon_tags_31 },
    { 0x94a2fb46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa2fb4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb182fb4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad82fb51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9602fb5f, SUC_lexicon_tags_9 },
    { 0xfe2fb5f, SUC_lexicon_tags_104 },
    { 0x3e62fb61, SUC_lexicon_tags_15 },
    { 0x3b82fb62, SUC_lexicon_tags_17 },
    { 0x3062fb63, SUC_lexicon_tags_7 },
    { 0x56a2fb62, SUC_lexicon_tags_33 },
    { 0xd8e2fb64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4fa2fb67, SUC_lexicon_tags_27 },
    { 0x1ba2fb67, SUC_lexicon_tags_11 },
    { 0x6622fb69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x74e2fb6b, SUC_lexicon_tags_49 },
    { 0x2f82fb6c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x70e2fb6e, SUC_lexicon_tags_27 },
    { 0xf4a2fb6e, SUC_lexicon_tags_2 },
    { 0x65c2fb6f, SUC_lexicon_tags_7 },
    { 0x7c42fb6f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc082fb75, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5302fb78, SUC_lexicon_tags_11 },
    { 0x4782fb79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb022fb7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa2fb83, SUC_lexicon_tags_17 },
    { 0x52c2fb84, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb3c2fb86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5382fb89, SUC_lexicon_tags_12 },
    { 0xe922fb89, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf022fb91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee02fb97, SUC_lexicon_tags_19 },
    { 0x9c2fb98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c82fb9b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2822fb9d, SUC_lexicon_tags_23 },
    { 0x6e42fb9e, SUC_lexicon_tags_15 },
    { 0xfa82fb9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e2fba6, SUC_lexicon_tags_11 },
    { 0xa3a2fba7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc082fbac, SUC_lexicon_tags_16 },
    { 0xbb22fbad, SUC_lexicon_tags_1 },
    { 0xda62fbae, SUC_lexicon_tags_11 },
    { 0x23a2fbac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f42fbb1, SUC_lexicon_tags_23 },
    { 0x7402fbb2, SUC_lexicon_tags_38 },
    { 0xb3c2fbb2, SUC_lexicon_tags_1 },
    { 0x48e2fbb3, SUC_lexicon_tags_14 },
    { 0x5862fbb5, SUC_lexicon_tags_63 },
    { 0xda62fbb5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e2fbbd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd462fbc1, SUC_lexicon_tags_9 },
    { 0x9c2fbc2, SUC_lexicon_tags_30 },
    { 0x682fbc2, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f22fbcd, SUC_lexicon_tags_12 },
    { 0x9a82fbce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e82fbd9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xce62fbdb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3fa2fbdd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac2fbe0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a2fbe3, SUC_lexicon_tags_15 },
    { 0x6462fbe4, SUC_lexicon_tags_47 },
    { 0xd762fbe5, SUC_lexicon_tags_19 },
    { 0xb922fbe5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8602fbe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c2fbef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9682fbf6, SUC_lexicon_tags_17 },
    { 0x9542fbf6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6962fbfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd042fbfd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe482fc06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9542fc13, SUC_lexicon_tags_25 },
    { 0x3662fc13, SUC_lexicon_tags_60 },
    { 0x6002fc15, SUC_lexicon_tags_45 },
    { 0xa922fc14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e2fc19, SUC_lexicon_tags_15 },
    { 0x4e2fc1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce2fc1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe182fc1f, SUC_lexicon_tags_29 },
    { 0xf982fc1f, SUC_lexicon_tags_11 },
    { 0x57a2fc1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae62fc2a, SUC_lexicon_tags_84 },
    { 0xe62fc2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42fc2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6182fc31, SUC_lexicon_tags_17 },
    { 0x5722fc31, SUC_lexicon_tags_3 },
    { 0x38a2fc31, SUC_lexicon_tags_11 },
    { 0xc502fc34, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe262fc37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b22fc3d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xefc2fc3f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4282fc43, SUC_lexicon_tags_25 },
    { 0xefc2fc44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d62fc47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc582fc49, SUC_lexicon_tags_65 },
    { 0x7fe2fc4a, SUC_lexicon_tags_47 },
    { 0x5182fc4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x93c2fc4d, SUC_lexicon_tags_43 },
    { 0x9a42fc4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4342fc50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3502fc55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9582fc58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2382fc5c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x37e2fc5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e2fc60, SUC_lexicon_tags_30 },
    { 0x1482fc61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a2fc65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3122fc67, SUC_lexicon_tags_56 },
    { 0x522fc68, SUC_lexicon_tags_7 },
    { 0xab22fc67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6922fc6e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa962fc74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc2fc77, SUC_lexicon_tags_25 },
    { 0x66a2fc77, SUC_lexicon_tags_30 },
    { 0x9d02fc79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb22fc7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2222fc81, SUC_lexicon_tags_61 },
    { 0x9102fc82, SUC_lexicon_tags_49 },
    { 0xa902fc83, SUC_lexicon_tags_32 },
    { 0xf062fc84, SUC_lexicon_tags_17 },
    { 0x4de2fc83, SUC_lexicon_tags_37 },
    { 0xc542fc81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a2fc88, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24e2fc8a, SUC_lexicon_tags_37 },
    { 0xe22fc8a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4102fc8d, SUC_lexicon_tags_11 },
    { 0x2e82fc8d, SUC_lexicon_tags_37 },
    { 0x2b62fc8f, SUC_lexicon_tags_49 },
    { 0xfe42fc90, SUC_lexicon_tags_719 },
    { 0x5f62fc91, SUC_lexicon_tags_38 },
    { 0xd5e2fc8f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5182fc95, SUC_lexicon_tags_85 },
    { 0x6122fc95, SUC_lexicon_tags_29 },
    { 0xd002fc96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f02fc99, SUC_lexicon_tags_12 },
    { 0xf282fc9a, SUC_lexicon_tags_33 },
    { 0xaca2fc9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1da2fc9d, SUC_lexicon_tags_33 },
    { 0xda62fc9e, SUC_lexicon_tags_23 },
    { 0x8f62fc9f, SUC_lexicon_tags_31 },
    { 0x6402fc9e, SUC_lexicon_tags_27 },
    { 0x55e2fca1, SUC_lexicon_tags_25 },
    { 0x4282fca0, SUC_lexicon_tags_7 },
    { 0xc642fca1, SUC_lexicon_tags_17 },
    { 0xf5e2fc9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ca2fca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc522fcaa, SUC_lexicon_tags_17 },
    { 0xdc2fcaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7262fcae, SUC_lexicon_tags_1 },
    { 0xaac2fcaf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb642fcb1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa162fcbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5802fcbd, SUC_lexicon_tags_27 },
    { 0xfda2fcbe, SUC_lexicon_tags_19 },
    { 0xf6c2fcbf, SUC_lexicon_tags_23 },
    { 0x13a2fcc0, SUC_lexicon_tags_66 },
    { 0x3b62fcc1, SUC_lexicon_tags_2 },
    { 0x6ac2fcc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8042fcc8, SUC_lexicon_tags_1 },
    { 0x6fe2fcc9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xf242fccc, SUC_lexicon_tags_25 },
    { 0xc382fccd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9782fcd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a02fcd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a2fcda, SUC_lexicon_tags_2 },
    { 0x8f42fcdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e2fcde, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a2fce4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a2fce9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7982fcf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2fcf6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d42fcfe, SUC_lexicon_tags_25 },
    { 0xfc42fcfe, SUC_lexicon_tags_60 },
    { 0x92c2fcfe, SUC_lexicon_tags_12 },
    { 0xeae2fd01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb822fd03, SUC_lexicon_tags_48 },
    { 0xf3e2fd04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa062fd08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3762fd0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf742fd0e, SUC_lexicon_tags_15 },
    { 0x56a2fd0e, SUC_lexicon_tags_4 },
    { 0x8982fd10, SUC_lexicon_tags_37 },
    { 0xbf22fd10, SUC_lexicon_tags_15 },
    { 0x24e2fd11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xace2fd14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x21a2fd16, SUC_lexicon_tags_7 },
    { 0xaf62fd16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4182fd19, SUC_lexicon_tags_91 },
    { 0xf8c2fd1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb722fd1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc2fd2a, SUC_lexicon_tags_37 },
    { 0x8102fd2b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed02fd2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe2fd31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb62fd35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb322fd3b, SUC_lexicon_tags_15 },
    { 0x1ac2fd3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb662fd3e, SUC_lexicon_tags_9 },
    { 0xf162fd3f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x982fd41, SUC_lexicon_tags_3 },
    { 0x2482fd42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e62fd46, SUC_lexicon_tags_25 },
    { 0xd042fd46, SUC_lexicon_tags_38 },
    { 0x3742fd47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5362fd4a, SUC_lexicon_tags_1 },
    { 0xd922fd4b, SUC_lexicon_tags_21 },
    { 0xf222fd4a, SUC_lexicon_tags_11 },
    { 0xd302fd4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc962fd51, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6182fd55, SUC_lexicon_tags_47 },
    { 0xc6a2fd56, SUC_lexicon_tags_12 },
    { 0x2f02fd56, SUC_lexicon_tags_87 },
    { 0xcc02fd58, SUC_lexicon_tags_38 },
    { 0x5062fd59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6062fd5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x29a2fd5f, SUC_lexicon_tags_45 },
    { 0x9ae2fd5f, SUC_lexicon_tags_11 },
    { 0x68e2fd60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9662fd65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2162fd68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e2fd6b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb742fd6e, SUC_lexicon_tags_2 },
    { 0x5802fd6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf82fd74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefa2fd76, SUC_lexicon_tags_9 },
    { 0xd482fd77, SUC_lexicon_tags_28 },
    { 0x6a42fd78, SUC_lexicon_tags_15 },
    { 0xc22fd79, SUC_lexicon_tags_27 },
    { 0x36c2fd7a, SUC_lexicon_tags_55 },
    { 0xe62fd77, SUC_lexicon_tags_7 },
    { 0x3202fd7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e2fd82, SUC_lexicon_tags_17 },
    { 0x3e62fd83, SUC_lexicon_tags_46 },
    { 0xc0c2fd84, SUC_lexicon_tags_12 },
    { 0x2642fd85, SUC_lexicon_tags_83 },
    { 0x8aa2fd84, SUC_lexicon_tags_47 },
    { 0x6962fd87, SUC_lexicon_tags_12 },
    { 0x7062fd88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1542fd8b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa142fd90, SUC_lexicon_tags_37 },
    { 0xdbc2fd91, SUC_lexicon_tags_60 },
    { 0x7302fd91, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9182fd9d, SUC_lexicon_tags_27 },
    { 0xbd22fd9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2022fda0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc242fda8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f02fdaa, SUC_lexicon_tags_17 },
    { 0x5582fdab, SUC_lexicon_tags_2 },
    { 0x5402fdac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9602fdae, SUC_lexicon_tags_10 },
    { 0xdcc2fdaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd42fdb1, SUC_lexicon_tags_38 },
    { 0x22fdb2, SUC_lexicon_tags_17 },
    { 0x10e2fdb3, SUC_lexicon_tags_46 },
    { 0x522fdb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xee42fdb7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3342fdbd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc02fdd0, SUC_lexicon_tags_23 },
    { 0x8602fdd0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2702fdd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5582fdd9, SUC_lexicon_tags_1 },
    { 0x69c2fdd9, SUC_lexicon_tags_43 },
    { 0xb802fddb, SUC_lexicon_tags_15 },
    { 0x16e2fdd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb482fddf, SUC_lexicon_tags_7 },
    { 0xdec2fde0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba2fde5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc282fde9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc02fdef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c2fdf4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x40c2fdf6, SUC_lexicon_tags_11 },
    { 0xf222fdf7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd462fdfb, SUC_lexicon_tags_17 },
    { 0x77c2fdfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7282fdff, SUC_lexicon_tags_8 },
    { 0x8562fe00, SUC_lexicon_tags_45 },
    { 0x1242fe01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2fe04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a2fe0a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x742fe0c, SUC_lexicon_tags_2 },
    { 0xd4e2fe0d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa22fe13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3422fe15, SUC_lexicon_tags_17 },
    { 0x3782fe16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9ba2fe18, SUC_lexicon_tags_8 },
    { 0x7e02fe19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd482fe1e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a2fe24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd202fe26, SUC_lexicon_tags_23 },
    { 0x3722fe27, SUC_lexicon_tags_55 },
    { 0xd922fe26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xae82fe2b, SUC_lexicon_tags_25 },
    { 0x6702fe2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae42fe2e, SUC_lexicon_tags_37 },
    { 0x2e02fe2f, SUC_lexicon_tags_2 },
    { 0xd422fe2f, SUC_lexicon_tags_26 },
    { 0x6f82fe31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee2fe34, SUC_lexicon_tags_27 },
    { 0x48c2fe35, SUC_lexicon_tags_25 },
    { 0xe242fe34, SUC_lexicon_tags_37 },
    { 0x1962fe37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57e2fe39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd822fe3d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee2fe44, SUC_lexicon_tags_27 },
    { 0x2b42fe45, SUC_lexicon_tags_25 },
    { 0xdbe2fe46, SUC_lexicon_tags_17 },
    { 0xd502fe47, SUC_lexicon_tags_12 },
    { 0x24a2fe48, SUC_lexicon_tags_25 },
    { 0x2c02fe45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe2fe4c, SUC_lexicon_tags_29 },
    { 0x62e2fe4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b42fe50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb782fe54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e82fe59, SUC_lexicon_tags_2 },
    { 0x5c82fe5a, SUC_lexicon_tags_23 },
    { 0x9742fe5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de2fe63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac2fe66, SUC_lexicon_tags_27 },
    { 0xe562fe67, SUC_lexicon_tags_15 },
    { 0x19a2fe66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a62fe6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa102fe6d, SUC_lexicon_tags_25 },
    { 0xd8a2fe6e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9422fe73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fe75, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a2fe7c, SUC_lexicon_tags_2 },
    { 0x90a2fe7c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d82fe80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a2fe84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d82fe87, SUC_lexicon_tags_12 },
    { 0x8be2fe88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c2fe8c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5642fe8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a2fe95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4342fe9b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb522fe9e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2022fea0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd5c2fea2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x20e2fea4, SUC_lexicon_tags_47 },
    { 0xa4c2fea5, SUC_lexicon_tags_11 },
    { 0x9c42fea5, SUC_lexicon_tags_29 },
    { 0xc902fea5, SUC_lexicon_tags_27 },
    { 0xd782fea8, SUC_lexicon_tags_57 },
    { 0x7822fea9, SUC_lexicon_tags_26 },
    { 0x9ec2fea8, SUC_lexicon_tags_38 },
    { 0xbbc2feab, SUC_lexicon_tags_21 },
    { 0x8862feac, SUC_lexicon_tags_30 },
    { 0xca22feab, SUC_lexicon_tags_17 },
    { 0xb4e2feaa, SUC_lexicon_tags_17 },
    { 0x5ea2fea8, SUC_lexicon_tags_57 },
    { 0x1ba2feb0, SUC_lexicon_tags_61 },
    { 0x7382feb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a2feb6, SUC_lexicon_tags_37 },
    { 0x16e2feb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f42feb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e62febc, SUC_lexicon_tags_31 },
    { 0xc562febc, SUC_lexicon_tags_30 },
    { 0x2082febe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f22fec1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec2fec9, SUC_lexicon_tags_1 },
    { 0x3942feca, SUC_lexicon_tags_47 },
    { 0xc42fecb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xc062fece, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c2fed1, SUC_lexicon_tags_9 },
    { 0x7a2fed2, SUC_lexicon_tags_2 },
    { 0x2e42fed1, SUC_lexicon_tags_28 },
    { 0x8ec2fed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef22fed6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e2fed9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3642fedc, SUC_lexicon_tags_37 },
    { 0x4482fedc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2fee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682fee5, SUC_lexicon_tags_12 },
    { 0xa502fee6, SUC_lexicon_tags_62 },
    { 0x6862fee6, SUC_lexicon_tags_27 },
    { 0x84c2fee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c2feea, SUC_lexicon_tags_15 },
    { 0xfa62feeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d02feed, SUC_lexicon_tags_79 },
    { 0xe02feed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b22fef8, SUC_lexicon_tags_49 },
    { 0xe842fef9, SUC_lexicon_tags_38 },
    { 0xab22fef9, SUC_lexicon_tags_1 },
    { 0x5cc2fefb, SUC_lexicon_tags_37 },
    { 0x8ae2fef9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8722ff02, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf42ff04, SUC_lexicon_tags_46 },
    { 0x322ff04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae2ff10, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a2ff17, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe562ff1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe822ff21, SUC_lexicon_tags_211 },
    { 0xa562ff22, SUC_lexicon_tags_25 },
    { 0x4282ff23, SUC_lexicon_tags_37 },
    { 0x8402ff23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd42ff28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec62ff2a, SUC_lexicon_tags_12 },
    { 0x1f22ff2b, SUC_lexicon_tags_32 },
    { 0xbdc2ff2c, SUC_lexicon_tags_656 },
    { 0, NULL },
    { 0xfe2ff2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e2ff32, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x1402ff35, SUC_lexicon_tags_26 },
    { 0xfa02ff36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb82ff38, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c2ff3b, SUC_lexicon_tags_32 },
    { 0x4f82ff3c, SUC_lexicon_tags_47 },
    { 0x94e2ff3b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd682ff3f, SUC_lexicon_tags_25 },
    { 0x9222ff40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfa02ff42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6802ff44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a2ff4a, SUC_lexicon_tags_1175 },
    { 0x3542ff4b, SUC_lexicon_tags_820 },
    { 0x562ff4a, SUC_lexicon_tags_7 },
    { 0x8ae2ff4c, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6002ff51, SUC_lexicon_tags_17 },
    { 0x2802ff51, SUC_lexicon_tags_7 },
    { 0x7702ff53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf042ff58, SUC_lexicon_tags_25 },
    { 0x5bc2ff59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c82ff5c, SUC_lexicon_tags_15 },
    { 0xa242ff5c, SUC_lexicon_tags_12 },
    { 0x822ff5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3442ff67, SUC_lexicon_tags_38 },
    { 0xbc02ff68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa422ff6a, SUC_lexicon_tags_92 },
    { 0x73c2ff6a, SUC_lexicon_tags_25 },
    { 0xf8a2ff6c, SUC_lexicon_tags_21 },
    { 0xb022ff6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff82ff6f, SUC_lexicon_tags_15 },
    { 0xa862ff70, SUC_lexicon_tags_15 },
    { 0x3782ff71, SUC_lexicon_tags_37 },
    { 0xf2e2ff72, SUC_lexicon_tags_17 },
    { 0x8b22ff73, SUC_lexicon_tags_62 },
    { 0x59e2ff70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa2ff78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e2ff7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7602ff7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89c2ff7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec2ff84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2662ff88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e82ff8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2ff92, SUC_lexicon_tags_11 },
    { 0xfc42ff93, SUC_lexicon_tags_23 },
    { 0x8f82ff93, SUC_lexicon_tags_27 },
    { 0x6982ff95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8782ff97, SUC_lexicon_tags_31 },
    { 0x51a2ff98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe022ff9a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xace2ff9c, SUC_lexicon_tags_57 },
    { 0xa862ff9d, SUC_lexicon_tags_37 },
    { 0x16e2ff9e, SUC_lexicon_tags_48 },
    { 0x6c42ff9e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f22ffa2, SUC_lexicon_tags_12 },
    { 0x4a22ffa3, SUC_lexicon_tags_32 },
    { 0x55c2ffa4, SUC_lexicon_tags_37 },
    { 0xd102ffa4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2c2ffa7, SUC_lexicon_tags_33 },
    { 0x3382ffa7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6142ffab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6482ffb3, SUC_lexicon_tags_1 },
    { 0xfea2ffb4, SUC_lexicon_tags_31 },
    { 0x1a22ffb4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xafe2ffb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d82ffb9, SUC_lexicon_tags_237 },
    { 0xdf02ffba, SUC_lexicon_tags_38 },
    { 0x92e2ffb9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb622ffbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc2ffc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b22ffc2, SUC_lexicon_tags_11 },
    { 0xe782ffc2, SUC_lexicon_tags_27 },
    { 0x3282ffc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42ffcd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x11e2ffcf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3142ffd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbd02ffd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6882ffd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb842ffd8, SUC_lexicon_tags_49 },
    { 0x1682ffd8, SUC_lexicon_tags_15 },
    { 0x4c2ffd9, SUC_lexicon_tags_65 },
    { 0xe582ffdb, SUC_lexicon_tags_12 },
    { 0x4942ffdc, SUC_lexicon_tags_46 },
    { 0xeca2ffdc, SUC_lexicon_tags_65 },
    { 0x102ffde, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf062ffe2, SUC_lexicon_tags_56 },
    { 0xeae2ffe3, SUC_lexicon_tags_45 },
    { 0x21a2ffe4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a2ffe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7742ffea, SUC_lexicon_tags_66 },
    { 0xc742ffeb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x78e2ffed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f62fff5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3722fff9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd202ffff, SUC_lexicon_tags_27 },
    { 0x66630000, SUC_lexicon_tags_87 },
    { 0x42630000, SUC_lexicon_tags_17 },
    { 0xcfc30001, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc30005, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0x50230008, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc3000d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc8a3000f, SUC_lexicon_tags_47 },
    { 0x7703000f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ca30012, SUC_lexicon_tags_45 },
    { 0xfa230012, SUC_lexicon_tags_1 },
    { 0xf2630012, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a30019, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc663001d, SUC_lexicon_tags_6 },
    { 0x78a3001e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe30021, SUC_lexicon_tags_47 },
    { 0xa7630021, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49a30024, SUC_lexicon_tags_28 },
    { 0x63430025, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x94c30027, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c3002d, SUC_lexicon_tags_37 },
    { 0xb6a3002e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac630030, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7f230032, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x49a30034, SUC_lexicon_tags_37 },
    { 0x2aa30035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c03003c, SUC_lexicon_tags_68 },
    { 0x4623003c, SUC_lexicon_tags_2 },
    { 0x8243003e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20030040, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58030047, SUC_lexicon_tags_1 },
    { 0xa3a30047, SUC_lexicon_tags_26 },
    { 0xd7c30049, SUC_lexicon_tags_37 },
    { 0xe303004a, SUC_lexicon_tags_0 },
    { 0xdde3004b, SUC_lexicon_tags_25 },
    { 0xf9a3004a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52230051, SUC_lexicon_tags_37 },
    { 0x30430052, SUC_lexicon_tags_63 },
    { 0x74830053, SUC_lexicon_tags_15 },
    { 0xec430053, SUC_lexicon_tags_7 },
    { 0x3ac30054, SUC_lexicon_tags_91 },
    { 0x9630054, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283005c, SUC_lexicon_tags_11 },
    { 0x11a3005c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4843005f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x42a30061, SUC_lexicon_tags_43 },
    { 0x66c30061, SUC_lexicon_tags_15 },
    { 0xb6430063, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0c30065, SUC_lexicon_tags_60 },
    { 0x97230066, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e3006b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31430073, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x97030075, SUC_lexicon_tags_31 },
    { 0xf5430075, SUC_lexicon_tags_87 },
    { 0xce430077, SUC_lexicon_tags_17 },
    { 0x80830076, SUC_lexicon_tags_11 },
    { 0x4b430076, SUC_lexicon_tags_1 },
    { 0x430077, SUC_lexicon_tags_34 },
    { 0xbba3007a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6e3007d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce3008b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6503008e, SUC_lexicon_tags_63 },
    { 0x2683008e, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xce230091, SUC_lexicon_tags_87 },
    { 0xc7c30092, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e30096, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa30099, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x323009c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c300a0, SUC_lexicon_tags_23 },
    { 0x35a300a1, SUC_lexicon_tags_12 },
    { 0x444300a2, SUC_lexicon_tags_9 },
    { 0xe6e300a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2300aa, SUC_lexicon_tags_4 },
    { 0x59e300aa, SUC_lexicon_tags_17 },
    { 0x1b0300ab, SUC_lexicon_tags_14 },
    { 0xfe300ac, SUC_lexicon_tags_37 },
    { 0x20300ae, SUC_lexicon_tags_14 },
    { 0xc32300af, SUC_lexicon_tags_27 },
    { 0x262300af, SUC_lexicon_tags_82 },
    { 0x5cc300ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810300b6, SUC_lexicon_tags_30 },
    { 0x19a300b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e300bc, SUC_lexicon_tags_25 },
    { 0x45a300bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62300c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a300cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0300ce, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc66300d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6300d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe02300d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a300d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe24300db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8c300dd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xe36300df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e300e7, SUC_lexicon_tags_2 },
    { 0xc82300e7, SUC_lexicon_tags_15 },
    { 0x456300e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68300ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4300ef, SUC_lexicon_tags_17 },
    { 0xc84300f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6300f6, SUC_lexicon_tags_65 },
    { 0xda8300f6, SUC_lexicon_tags_26 },
    { 0xf92300f6, SUC_lexicon_tags_14 },
    { 0xd78300f8, SUC_lexicon_tags_25 },
    { 0xa2a300fa, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa300fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x68300ff, SUC_lexicon_tags_37 },
    { 0x12a300ff, SUC_lexicon_tags_38 },
    { 0xe5430101, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2230104, SUC_lexicon_tags_17 },
    { 0x16e30105, SUC_lexicon_tags_37 },
    { 0x55230105, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6e030108, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23010c, SUC_lexicon_tags_15 },
    { 0x8c23010c, SUC_lexicon_tags_37 },
    { 0x5183010c, SUC_lexicon_tags_59 },
    { 0x7f43010f, SUC_lexicon_tags_57 },
    { 0x77030110, SUC_lexicon_tags_2 },
    { 0x7f830110, SUC_lexicon_tags_20 },
    { 0x8903010c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae430114, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c30117, SUC_lexicon_tags_7 },
    { 0x7ae30118, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce3011c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5163011e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b430123, SUC_lexicon_tags_9 },
    { 0xf7c30124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb030128, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd823012a, SUC_lexicon_tags_15 },
    { 0x3183012b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c30130, SUC_lexicon_tags_14 },
    { 0x4d430131, SUC_lexicon_tags_1 },
    { 0xd8230131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37030134, SUC_lexicon_tags_64 },
    { 0xaa830135, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf063013b, SUC_lexicon_tags_37 },
    { 0x3013c, SUC_lexicon_tags_1 },
    { 0x50e3013b, SUC_lexicon_tags_16 },
    { 0x5b23013d, SUC_lexicon_tags_47 },
    { 0x4003013f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e30145, SUC_lexicon_tags_17 },
    { 0xee30146, SUC_lexicon_tags_103 },
    { 0x10a30147, SUC_lexicon_tags_1 },
    { 0x2f830146, SUC_lexicon_tags_27 },
    { 0xb6230147, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a3014d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca30151, SUC_lexicon_tags_38 },
    { 0x8a30152, SUC_lexicon_tags_38 },
    { 0x5d830153, SUC_lexicon_tags_17 },
    { 0x32630152, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e030157, SUC_lexicon_tags_25 },
    { 0x5cc30157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7243015d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8630162, SUC_lexicon_tags_19 },
    { 0x54c30163, SUC_lexicon_tags_28 },
    { 0x65830163, SUC_lexicon_tags_12 },
    { 0x46230165, SUC_lexicon_tags_14 },
    { 0x3b430166, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe723016b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a3016e, SUC_lexicon_tags_23 },
    { 0x17e3016f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95030176, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd030179, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3243017b, SUC_lexicon_tags_29 },
    { 0xd443017b, SUC_lexicon_tags_27 },
    { 0x2c03017b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8630184, SUC_lexicon_tags_49 },
    { 0x41c30185, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9da30187, SUC_lexicon_tags_25 },
    { 0xc5830188, SUC_lexicon_tags_38 },
    { 0x9cc30189, SUC_lexicon_tags_12 },
    { 0xea03018a, SUC_lexicon_tags_10 },
    { 0x83430187, SUC_lexicon_tags_27 },
    { 0x1283018c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc743018e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a30195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3030197, SUC_lexicon_tags_14 },
    { 0x94430197, SUC_lexicon_tags_12 },
    { 0x22630199, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2563019b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7223019d, SUC_lexicon_tags_31 },
    { 0x1ba3019e, SUC_lexicon_tags_45 },
    { 0xaae3019d, SUC_lexicon_tags_9 },
    { 0x2b03019d, SUC_lexicon_tags_13 },
    { 0x6e23019d, SUC_lexicon_tags_37 },
    { 0xc903019e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b4301a4, SUC_lexicon_tags_4 },
    { 0x8cc301a5, SUC_lexicon_tags_91 },
    { 0x4f4301a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd48301a8, SUC_lexicon_tags_25 },
    { 0x82301a8, SUC_lexicon_tags_25 },
    { 0x710301a8, SUC_lexicon_tags_25 },
    { 0x1da301ab, SUC_lexicon_tags_17 },
    { 0x610301ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10301af, SUC_lexicon_tags_27 },
    { 0x660301b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe301b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312301c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9a301c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e301ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34301ce, SUC_lexicon_tags_15 },
    { 0x87e301cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x378301d2, SUC_lexicon_tags_27 },
    { 0x626301d3, SUC_lexicon_tags_1 },
    { 0x384301d4, SUC_lexicon_tags_23 },
    { 0xeb6301d4, SUC_lexicon_tags_10 },
    { 0x6ae301d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdea301d8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x76a301da, SUC_lexicon_tags_14 },
    { 0x5f6301db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6301e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de301e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e301ed, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf50301ef, SUC_lexicon_tags_10 },
    { 0x9f0301ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1c6301f2, SUC_lexicon_tags_34 },
    { 0x866301f3, SUC_lexicon_tags_30 },
    { 0x12e301f4, SUC_lexicon_tags_15 },
    { 0xdca301f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2301f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef8301fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe42301fd, SUC_lexicon_tags_60 },
    { 0xc6c301fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1630207, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1523020a, SUC_lexicon_tags_56 },
    { 0xd143020b, SUC_lexicon_tags_0 },
    { 0xf303020c, SUC_lexicon_tags_0 },
    { 0x5fe3020d, SUC_lexicon_tags_15 },
    { 0xd403020c, SUC_lexicon_tags_8 },
    { 0xde23020a, SUC_lexicon_tags_77 },
    { 0xb4e30210, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb4030212, SUC_lexicon_tags_11 },
    { 0x3f030212, SUC_lexicon_tags_23 },
    { 0x4a630213, SUC_lexicon_tags_74 },
    { 0xbc230215, SUC_lexicon_tags_15 },
    { 0x3ee30216, SUC_lexicon_tags_25 },
    { 0xc1a30217, SUC_lexicon_tags_8 },
    { 0x56630215, SUC_lexicon_tags_29 },
    { 0xc9830219, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d23021d, SUC_lexicon_tags_25 },
    { 0xd0c3021d, SUC_lexicon_tags_6 },
    { 0xfae3021d, SUC_lexicon_tags_129 },
    { 0x11830220, SUC_lexicon_tags_15 },
    { 0x7fc30221, SUC_lexicon_tags_11 },
    { 0x18430222, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xbac30224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9030226, SUC_lexicon_tags_2 },
    { 0x62e30227, SUC_lexicon_tags_15 },
    { 0x70030228, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5323022b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9d43022d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44c3022f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85830232, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6630237, SUC_lexicon_tags_27 },
    { 0x26830238, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x383023a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f230243, SUC_lexicon_tags_37 },
    { 0xb9230244, SUC_lexicon_tags_12 },
    { 0xc8030245, SUC_lexicon_tags_1 },
    { 0x32430246, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e3024a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc83024d, SUC_lexicon_tags_33 },
    { 0x95c3024d, SUC_lexicon_tags_8 },
    { 0x75c3024d, SUC_lexicon_tags_37 },
    { 0xecc3024d, SUC_lexicon_tags_9 },
    { 0x9c3024d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e30255, SUC_lexicon_tags_8 },
    { 0xd5430255, SUC_lexicon_tags_37 },
    { 0xcf830256, SUC_lexicon_tags_25 },
    { 0x94a30258, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x23025b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23230263, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7430266, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3463026d, SUC_lexicon_tags_1 },
    { 0xf423026e, SUC_lexicon_tags_14 },
    { 0xb903026d, SUC_lexicon_tags_1 },
    { 0x91030270, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66430273, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf630278, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb263027a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c3027d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0430283, SUC_lexicon_tags_252 },
    { 0x12030284, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc4c30286, SUC_lexicon_tags_27 },
    { 0xf6430287, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2783028a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c30290, SUC_lexicon_tags_7 },
    { 0x5a430290, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c30294, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc30297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20830299, SUC_lexicon_tags_14 },
    { 0x93c3029a, SUC_lexicon_tags_15 },
    { 0xd8230299, SUC_lexicon_tags_84 },
    { 0x9703029c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8123029f, SUC_lexicon_tags_9 },
    { 0x4c6302a0, SUC_lexicon_tags_7 },
    { 0x4e43029f, SUC_lexicon_tags_23 },
    { 0x586302a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd16302a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6302a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88302aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8302ad, SUC_lexicon_tags_37 },
    { 0xfd8302ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99e302b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4a302b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79e302b4, SUC_lexicon_tags_2 },
    { 0xf98302b5, SUC_lexicon_tags_2 },
    { 0x2be302b6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x42c302b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2c302ba, SUC_lexicon_tags_48 },
    { 0xdb2302bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764302c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac302c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2ea302cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1be302cd, SUC_lexicon_tags_17 },
    { 0xca4302ce, SUC_lexicon_tags_46 },
    { 0x2fa302cf, SUC_lexicon_tags_39 },
    { 0x2be302d0, SUC_lexicon_tags_14 },
    { 0xdc2302d1, SUC_lexicon_tags_66 },
    { 0x81e302d2, SUC_lexicon_tags_2 },
    { 0x926302d3, SUC_lexicon_tags_2 },
    { 0xfbc302d3, SUC_lexicon_tags_37 },
    { 0x176302d2, SUC_lexicon_tags_15 },
    { 0xe5a302d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760302df, SUC_lexicon_tags_19 },
    { 0x9bc302e0, SUC_lexicon_tags_25 },
    { 0x886302e0, SUC_lexicon_tags_172 },
    { 0x56e302e2, SUC_lexicon_tags_47 },
    { 0x7d6302e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2302ea, SUC_lexicon_tags_2 },
    { 0x7c4302eb, SUC_lexicon_tags_11 },
    { 0x1e0302ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e302f6, SUC_lexicon_tags_47 },
    { 0xd78302f7, SUC_lexicon_tags_65 },
    { 0x38302f8, SUC_lexicon_tags_15 },
    { 0xc96302f7, SUC_lexicon_tags_27 },
    { 0x69e302f6, SUC_lexicon_tags_25 },
    { 0x55e302fb, SUC_lexicon_tags_17 },
    { 0x502302fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc30301, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca30308, SUC_lexicon_tags_7 },
    { 0x96230309, SUC_lexicon_tags_49 },
    { 0x92230308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb923030c, SUC_lexicon_tags_6 },
    { 0xfb43030c, SUC_lexicon_tags_48 },
    { 0x60c3030e, SUC_lexicon_tags_25 },
    { 0xb1e3030e, SUC_lexicon_tags_299 },
    { 0xcca3030c, SUC_lexicon_tags_15 },
    { 0x97e3030d, SUC_lexicon_tags_23 },
    { 0x49a3030e, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2830315, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0030318, SUC_lexicon_tags_23 },
    { 0x15830319, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc883031d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x32030320, SUC_lexicon_tags_25 },
    { 0xdf230321, SUC_lexicon_tags_27 },
    { 0x7f830321, SUC_lexicon_tags_32 },
    { 0xfec30322, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9230325, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6030328, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82e3032a, SUC_lexicon_tags_17 },
    { 0xd383032b, SUC_lexicon_tags_100 },
    { 0xfbc3032a, SUC_lexicon_tags_45 },
    { 0xd283032a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace30332, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d030337, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e63033b, SUC_lexicon_tags_16 },
    { 0xa583033c, SUC_lexicon_tags_66 },
    { 0x9863033c, SUC_lexicon_tags_11 },
    { 0xb3e3033b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46030342, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e30345, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283034b, SUC_lexicon_tags_6 },
    { 0xea23034c, SUC_lexicon_tags_2 },
    { 0xb43034c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47630351, SUC_lexicon_tags_211 },
    { 0x22c30352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5830355, SUC_lexicon_tags_29 },
    { 0x3630355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e30359, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf943035d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e30362, SUC_lexicon_tags_27 },
    { 0xf5030363, SUC_lexicon_tags_28 },
    { 0xc3830364, SUC_lexicon_tags_14 },
    { 0xe9430364, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe1030367, SUC_lexicon_tags_7 },
    { 0xb0030368, SUC_lexicon_tags_17 },
    { 0xae430369, SUC_lexicon_tags_44 },
    { 0x94c30368, SUC_lexicon_tags_60 },
    { 0x8903036b, SUC_lexicon_tags_47 },
    { 0xf903036c, SUC_lexicon_tags_25 },
    { 0x3a43036d, SUC_lexicon_tags_5 },
    { 0xada3036e, SUC_lexicon_tags_37 },
    { 0x9383036f, SUC_lexicon_tags_11 },
    { 0x1a03036d, SUC_lexicon_tags_12 },
    { 0x2ac30371, SUC_lexicon_tags_44 },
    { 0xfc030372, SUC_lexicon_tags_45 },
    { 0xb2c30373, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a30376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59630378, SUC_lexicon_tags_17 },
    { 0x7e630379, SUC_lexicon_tags_15 },
    { 0x6803037a, SUC_lexicon_tags_45 },
    { 0x3943037b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc230381, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x86030383, SUC_lexicon_tags_11 },
    { 0xd1e30384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95430386, SUC_lexicon_tags_14 },
    { 0x7fe30386, SUC_lexicon_tags_17 },
    { 0xa1030388, SUC_lexicon_tags_37 },
    { 0xbe630389, SUC_lexicon_tags_28 },
    { 0x19830389, SUC_lexicon_tags_16 },
    { 0x1b630387, SUC_lexicon_tags_27 },
    { 0x74a3038c, SUC_lexicon_tags_108 },
    { 0xff43038c, SUC_lexicon_tags_37 },
    { 0x82e30388, SUC_lexicon_tags_1 },
    { 0xbde30386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77830391, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f630394, SUC_lexicon_tags_38 },
    { 0x82030395, SUC_lexicon_tags_15 },
    { 0x9be30394, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf183039b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3039e, SUC_lexicon_tags_11 },
    { 0xb823039e, SUC_lexicon_tags_15 },
    { 0x7ba3039e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4303a3, SUC_lexicon_tags_15 },
    { 0x538303a3, SUC_lexicon_tags_7 },
    { 0x82303a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa303a9, SUC_lexicon_tags_11 },
    { 0x834303a9, SUC_lexicon_tags_23 },
    { 0x6dc303aa, SUC_lexicon_tags_27 },
    { 0xea4303ab, SUC_lexicon_tags_29 },
    { 0xeb6303ad, SUC_lexicon_tags_27 },
    { 0x836303ae, SUC_lexicon_tags_38 },
    { 0x7cc303ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x772303b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c303b6, SUC_lexicon_tags_35 },
    { 0x3f8303b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa60303b9, SUC_lexicon_tags_1 },
    { 0x572303b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf50303bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8b6303be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8303c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3ee303c3, SUC_lexicon_tags_1 },
    { 0x950303c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda303c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16303d2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x46303d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a303dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x682303df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8303e7, SUC_lexicon_tags_45 },
    { 0x82a303e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9be303ea, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x414303ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34e303ee, SUC_lexicon_tags_69 },
    { 0xdf2303ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134303f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc303fd, SUC_lexicon_tags_25 },
    { 0xc60303fd, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac30407, SUC_lexicon_tags_2 },
    { 0xfb030408, SUC_lexicon_tags_7 },
    { 0x44630408, SUC_lexicon_tags_37 },
    { 0xe403040a, SUC_lexicon_tags_25 },
    { 0x7d030408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6263040d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3030411, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x37430413, SUC_lexicon_tags_25 },
    { 0x37230414, SUC_lexicon_tags_48 },
    { 0x5b430414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3683041b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a30425, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03042a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1723042c, SUC_lexicon_tags_7 },
    { 0xec83042d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1bc3042f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b030432, SUC_lexicon_tags_82 },
    { 0xde430433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50830435, SUC_lexicon_tags_19 },
    { 0xb7a30436, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e3043c, SUC_lexicon_tags_60 },
    { 0x9e43043c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a30442, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ee30444, SUC_lexicon_tags_12 },
    { 0x34630444, SUC_lexicon_tags_45 },
    { 0x48230446, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa843044a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9e83044c, SUC_lexicon_tags_8 },
    { 0x30a3044c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b030451, SUC_lexicon_tags_43 },
    { 0xb8e30452, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98030454, SUC_lexicon_tags_17 },
    { 0xe7830455, SUC_lexicon_tags_9 },
    { 0xea030456, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a230458, SUC_lexicon_tags_25 },
    { 0x17a30459, SUC_lexicon_tags_55 },
    { 0x9743045a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe83045c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae3046a, SUC_lexicon_tags_19 },
    { 0x2f03046a, SUC_lexicon_tags_12 },
    { 0x30e3046b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5383046e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74830472, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9230476, SUC_lexicon_tags_2 },
    { 0x4d030476, SUC_lexicon_tags_47 },
    { 0x77030478, SUC_lexicon_tags_31 },
    { 0xacc30478, SUC_lexicon_tags_11 },
    { 0x1f630479, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91430489, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11630491, SUC_lexicon_tags_26 },
    { 0x1e630492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34c30494, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62430496, SUC_lexicon_tags_21 },
    { 0xbdc30496, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc4430499, SUC_lexicon_tags_2 },
    { 0x7383049a, SUC_lexicon_tags_7 },
    { 0x2fc3049b, SUC_lexicon_tags_55 },
    { 0x783049c, SUC_lexicon_tags_47 },
    { 0x42a3049d, SUC_lexicon_tags_16 },
    { 0x1ea3049c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec304a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd26304a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa36304a5, SUC_lexicon_tags_12 },
    { 0x10a304a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de304aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x840304ac, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5a0304ae, SUC_lexicon_tags_19 },
    { 0x94e304af, SUC_lexicon_tags_31 },
    { 0x4ca304b0, SUC_lexicon_tags_43 },
    { 0x214304b1, SUC_lexicon_tags_12 },
    { 0x86e304ae, SUC_lexicon_tags_2 },
    { 0x4a4304b0, SUC_lexicon_tags_47 },
    { 0x326304b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa6a304b6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x61c304b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff0304ba, SUC_lexicon_tags_25 },
    { 0xfd6304bb, SUC_lexicon_tags_7 },
    { 0x5a0304bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20304c3, SUC_lexicon_tags_25 },
    { 0xbd0304c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6304c7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a304cc, SUC_lexicon_tags_32 },
    { 0x79c304cd, SUC_lexicon_tags_2 },
    { 0x20304cd, SUC_lexicon_tags_30 },
    { 0x196304cd, SUC_lexicon_tags_11 },
    { 0xfe2304d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de304d3, SUC_lexicon_tags_6 },
    { 0xdd0304d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312304d8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8b6304da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac304e0, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a304e3, SUC_lexicon_tags_12 },
    { 0x646304e4, SUC_lexicon_tags_26 },
    { 0x266304e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a304e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x254304eb, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8304ef, SUC_lexicon_tags_13 },
    { 0x3e8304ef, SUC_lexicon_tags_56 },
    { 0x3b2304f1, SUC_lexicon_tags_23 },
    { 0xa46304f2, SUC_lexicon_tags_47 },
    { 0x762304f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd78304f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c304f9, SUC_lexicon_tags_37 },
    { 0x496304fa, SUC_lexicon_tags_17 },
    { 0x750304fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf4304fd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a3050a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c3050f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6903051b, SUC_lexicon_tags_11 },
    { 0x9a43051c, SUC_lexicon_tags_17 },
    { 0xd7a3051c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1430527, SUC_lexicon_tags_1 },
    { 0xade30528, SUC_lexicon_tags_47 },
    { 0x91030529, SUC_lexicon_tags_17 },
    { 0x47c30528, SUC_lexicon_tags_108 },
    { 0x4563052b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb843052d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x243052f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e630534, SUC_lexicon_tags_23 },
    { 0x99430534, SUC_lexicon_tags_37 },
    { 0x88c30536, SUC_lexicon_tags_14 },
    { 0xcd030537, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc783053c, SUC_lexicon_tags_33 },
    { 0x9563053d, SUC_lexicon_tags_26 },
    { 0xba03053e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84830544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece30549, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e3054c, SUC_lexicon_tags_27 },
    { 0x36c3054d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30554, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd230557, SUC_lexicon_tags_66 },
    { 0x44630558, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf03055a, SUC_lexicon_tags_37 },
    { 0x6fe3055a, SUC_lexicon_tags_53 },
    { 0x1a23055c, SUC_lexicon_tags_48 },
    { 0xe7e3055d, SUC_lexicon_tags_23 },
    { 0x1e03055e, SUC_lexicon_tags_11 },
    { 0x8a63055f, SUC_lexicon_tags_6 },
    { 0xbba3055e, SUC_lexicon_tags_82 },
    { 0xa163055d, SUC_lexicon_tags_2 },
    { 0xdce3055c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x81e30564, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf203056e, SUC_lexicon_tags_31 },
    { 0xa3c3056f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee30574, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a3057a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6743057d, SUC_lexicon_tags_18 },
    { 0xb2c3057d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc1e30580, SUC_lexicon_tags_30 },
    { 0xf2030581, SUC_lexicon_tags_11 },
    { 0xd5e30581, SUC_lexicon_tags_7 },
    { 0x55c30583, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35830589, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb663058e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ae30590, SUC_lexicon_tags_0 },
    { 0x74030590, SUC_lexicon_tags_29 },
    { 0x4fc30592, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x18430595, SUC_lexicon_tags_3 },
    { 0xab030596, SUC_lexicon_tags_47 },
    { 0x54a30597, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16e30599, SUC_lexicon_tags_72 },
    { 0x243059a, SUC_lexicon_tags_25 },
    { 0x4623059b, SUC_lexicon_tags_9 },
    { 0xe6a3059c, SUC_lexicon_tags_12 },
    { 0x6ee3059d, SUC_lexicon_tags_11 },
    { 0x2963059e, SUC_lexicon_tags_30 },
    { 0xea03059c, SUC_lexicon_tags_46 },
    { 0xe8e3059b, SUC_lexicon_tags_13 },
    { 0x88e3059b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x78a305a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdaa305a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5d6305a7, SUC_lexicon_tags_9 },
    { 0xcbe305a8, SUC_lexicon_tags_31 },
    { 0xf82305a7, SUC_lexicon_tags_28 },
    { 0xa9e305a7, SUC_lexicon_tags_68 },
    { 0x4e6305a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376305b0, SUC_lexicon_tags_0 },
    { 0x1b4305b1, SUC_lexicon_tags_12 },
    { 0xf26305b2, SUC_lexicon_tags_2 },
    { 0x45c305b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92305b7, SUC_lexicon_tags_9 },
    { 0x486305b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b4305ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x102305bc, SUC_lexicon_tags_33 },
    { 0x318305bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60305c0, SUC_lexicon_tags_15 },
    { 0x218305c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a305c7, SUC_lexicon_tags_0 },
    { 0x930305c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a305ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc305cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32305d2, SUC_lexicon_tags_6 },
    { 0x704305d3, SUC_lexicon_tags_25 },
    { 0x2dc305d4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6ea305d6, SUC_lexicon_tags_30 },
    { 0x7a4305d7, SUC_lexicon_tags_12 },
    { 0x4c305d7, SUC_lexicon_tags_1 },
    { 0x756305d7, SUC_lexicon_tags_10 },
    { 0x170305d7, SUC_lexicon_tags_49 },
    { 0xe84305db, SUC_lexicon_tags_43 },
    { 0x594305dc, SUC_lexicon_tags_30 },
    { 0xc18305d7, SUC_lexicon_tags_2 },
    { 0x1f4305d7, SUC_lexicon_tags_25 },
    { 0x45a305df, SUC_lexicon_tags_48 },
    { 0xb8305e0, SUC_lexicon_tags_17 },
    { 0x660305e1, SUC_lexicon_tags_60 },
    { 0xb7c305e2, SUC_lexicon_tags_12 },
    { 0xc5c305de, SUC_lexicon_tags_26 },
    { 0x368305e4, SUC_lexicon_tags_7 },
    { 0x9bc305df, SUC_lexicon_tags_27 },
    { 0x84a305e6, SUC_lexicon_tags_35 },
    { 0xfac305e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x28305e9, SUC_lexicon_tags_27 },
    { 0xa18305ea, SUC_lexicon_tags_16 },
    { 0x8ae305eb, SUC_lexicon_tags_2 },
    { 0x7f6305ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a305f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9ac305f6, SUC_lexicon_tags_82 },
    { 0x3e4305f7, SUC_lexicon_tags_25 },
    { 0x3ae305f8, SUC_lexicon_tags_11 },
    { 0xc44305f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac305fd, SUC_lexicon_tags_46 },
    { 0x446305fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x70830600, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7430609, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb723060c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca30614, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc8030616, SUC_lexicon_tags_46 },
    { 0x52430616, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8763061b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27e3061d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c30622, SUC_lexicon_tags_17 },
    { 0x60a30622, SUC_lexicon_tags_7 },
    { 0xd8a30622, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a30628, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe30633, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2630636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73e30638, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf403063c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb683063e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9830641, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c30649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c63064c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66830650, SUC_lexicon_tags_43 },
    { 0xf3a30650, SUC_lexicon_tags_15 },
    { 0x15c30650, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x28830655, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda63065c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c3065e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9830661, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde30664, SUC_lexicon_tags_37 },
    { 0x76a30665, SUC_lexicon_tags_19 },
    { 0xeae30666, SUC_lexicon_tags_41 },
    { 0x8030666, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa283066b, SUC_lexicon_tags_31 },
    { 0x3983066c, SUC_lexicon_tags_37 },
    { 0x92a3066d, SUC_lexicon_tags_14 },
    { 0xde83066e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de30673, SUC_lexicon_tags_27 },
    { 0x71e30674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a30677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c3067a, SUC_lexicon_tags_15 },
    { 0x7203067b, SUC_lexicon_tags_17 },
    { 0xc203067a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x87c3067e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6230680, SUC_lexicon_tags_59 },
    { 0x41030681, SUC_lexicon_tags_5 },
    { 0xf7830682, SUC_lexicon_tags_25 },
    { 0x25030681, SUC_lexicon_tags_7 },
    { 0x4e30684, SUC_lexicon_tags_27 },
    { 0x82630685, SUC_lexicon_tags_7 },
    { 0xb7030686, SUC_lexicon_tags_12 },
    { 0x7e830687, SUC_lexicon_tags_16 },
    { 0x71c30683, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743068c, SUC_lexicon_tags_32 },
    { 0x5e03068d, SUC_lexicon_tags_7 },
    { 0xbfc3068d, SUC_lexicon_tags_8 },
    { 0xee43068d, SUC_lexicon_tags_1 },
    { 0x23068d, SUC_lexicon_tags_37 },
    { 0x67830691, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x36030694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d63069b, SUC_lexicon_tags_37 },
    { 0xd4c3069b, SUC_lexicon_tags_25 },
    { 0x2b43069d, SUC_lexicon_tags_31 },
    { 0xdea3069b, SUC_lexicon_tags_11 },
    { 0x8923069f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c306a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e306a6, SUC_lexicon_tags_17 },
    { 0xfea306a7, SUC_lexicon_tags_17 },
    { 0x926306a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde8306aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x14306ac, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da306b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108306ba, SUC_lexicon_tags_7 },
    { 0x62c306ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf40306bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59a306bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202306c3, SUC_lexicon_tags_65 },
    { 0xf6c306c4, SUC_lexicon_tags_15 },
    { 0x5e6306c5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6306c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc6c306ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa306ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8c306d0, SUC_lexicon_tags_1 },
    { 0x232306d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410306d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24306d9, SUC_lexicon_tags_7 },
    { 0x23c306da, SUC_lexicon_tags_38 },
    { 0x45e306db, SUC_lexicon_tags_60 },
    { 0x756306db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0306e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x728306e3, SUC_lexicon_tags_10 },
    { 0x2da306e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x330306e7, SUC_lexicon_tags_49 },
    { 0x29c306e8, SUC_lexicon_tags_7 },
    { 0x542306e9, SUC_lexicon_tags_15 },
    { 0x7b0306ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56306ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x548306ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e306f2, SUC_lexicon_tags_0 },
    { 0x8306f3, SUC_lexicon_tags_3 },
    { 0x4f2306f4, SUC_lexicon_tags_17 },
    { 0x752306f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e306fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe30702, SUC_lexicon_tags_2 },
    { 0x53030703, SUC_lexicon_tags_7 },
    { 0x34a30702, SUC_lexicon_tags_38 },
    { 0xc9c30705, SUC_lexicon_tags_27 },
    { 0x1b430705, SUC_lexicon_tags_15 },
    { 0x4bc30705, SUC_lexicon_tags_25 },
    { 0x57230706, SUC_lexicon_tags_25 },
    { 0x68e30702, SUC_lexicon_tags_12 },
    { 0x99c3070a, SUC_lexicon_tags_28 },
    { 0xa383070b, SUC_lexicon_tags_124 },
    { 0xef83070b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5123070e, SUC_lexicon_tags_63 },
    { 0x3543070f, SUC_lexicon_tags_30 },
    { 0x82c30710, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8430713, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6923071a, SUC_lexicon_tags_8 },
    { 0x503071b, SUC_lexicon_tags_57 },
    { 0x7823071c, SUC_lexicon_tags_12 },
    { 0xbf43071a, SUC_lexicon_tags_31 },
    { 0x223071e, SUC_lexicon_tags_19 },
    { 0x1223071f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa9a30721, SUC_lexicon_tags_6 },
    { 0xbc30722, SUC_lexicon_tags_1 },
    { 0xf7430723, SUC_lexicon_tags_12 },
    { 0x62830724, SUC_lexicon_tags_14 },
    { 0x8aa30725, SUC_lexicon_tags_9 },
    { 0xd1e30726, SUC_lexicon_tags_30 },
    { 0xd1a30726, SUC_lexicon_tags_0 },
    { 0xe7230726, SUC_lexicon_tags_8 },
    { 0xa3a30729, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea3072c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac30739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1863073b, SUC_lexicon_tags_55 },
    { 0xeb23073b, SUC_lexicon_tags_38 },
    { 0xa3c3073c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54230742, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc30745, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f830748, SUC_lexicon_tags_11 },
    { 0x70030748, SUC_lexicon_tags_29 },
    { 0xaba3074a, SUC_lexicon_tags_14 },
    { 0xbbe3074b, SUC_lexicon_tags_17 },
    { 0x10c3074b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e83074e, SUC_lexicon_tags_37 },
    { 0x3e03074f, SUC_lexicon_tags_37 },
    { 0xbd63074f, SUC_lexicon_tags_17 },
    { 0xa543074f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25430758, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2603075b, SUC_lexicon_tags_8 },
    { 0xd3a3075c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e30760, SUC_lexicon_tags_38 },
    { 0xb0230761, SUC_lexicon_tags_68 },
    { 0x7f630760, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f830769, SUC_lexicon_tags_15 },
    { 0xec3076a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc83076c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f630775, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23630780, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7c630782, SUC_lexicon_tags_2 },
    { 0xa8430782, SUC_lexicon_tags_11 },
    { 0xa5e30784, SUC_lexicon_tags_39 },
    { 0xf1030785, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b03078a, SUC_lexicon_tags_7 },
    { 0xa343078b, SUC_lexicon_tags_36 },
    { 0xad83078b, SUC_lexicon_tags_229 },
    { 0xe023078b, SUC_lexicon_tags_0 },
    { 0xc003078e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20430790, SUC_lexicon_tags_38 },
    { 0x59e30791, SUC_lexicon_tags_23 },
    { 0xeb230791, SUC_lexicon_tags_151 },
    { 0xd8830790, SUC_lexicon_tags_10 },
    { 0x2d830794, SUC_lexicon_tags_85 },
    { 0xb5030795, SUC_lexicon_tags_15 },
    { 0x53030796, SUC_lexicon_tags_45 },
    { 0xb7e30795, SUC_lexicon_tags_64 },
    { 0xd6a30798, SUC_lexicon_tags_38 },
    { 0x36630797, SUC_lexicon_tags_7 },
    { 0x7fc3079a, SUC_lexicon_tags_33 },
    { 0xc383079a, SUC_lexicon_tags_17 },
    { 0x85230795, SUC_lexicon_tags_0 },
    { 0x3583079d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1b23079f, SUC_lexicon_tags_49 },
    { 0x2ea307a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0307a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x958307ab, SUC_lexicon_tags_68 },
    { 0xb0c307ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8307af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8307b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe46307b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba307c4, SUC_lexicon_tags_15 },
    { 0x18a307c4, SUC_lexicon_tags_7 },
    { 0x3fa307c5, SUC_lexicon_tags_9 },
    { 0x2a8307c5, SUC_lexicon_tags_66 },
    { 0x270307c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac307cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x820307cd, SUC_lexicon_tags_25 },
    { 0x85a307ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x464307d1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2307dd, SUC_lexicon_tags_12 },
    { 0xa48307de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8307e2, SUC_lexicon_tags_45 },
    { 0xc5e307e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8307e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb8a307e8, SUC_lexicon_tags_38 },
    { 0x702307e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c307f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd6307f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c307fa, SUC_lexicon_tags_25 },
    { 0x12c307fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea30801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2430806, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe630809, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c3080d, SUC_lexicon_tags_15 },
    { 0xc543080e, SUC_lexicon_tags_33 },
    { 0x7f83080f, SUC_lexicon_tags_15 },
    { 0xcea30810, SUC_lexicon_tags_17 },
    { 0xf443080d, SUC_lexicon_tags_2 },
    { 0x22830812, SUC_lexicon_tags_63 },
    { 0xc0a30813, SUC_lexicon_tags_17 },
    { 0x4b030810, SUC_lexicon_tags_19 },
    { 0x6f230815, SUC_lexicon_tags_40 },
    { 0x79230811, SUC_lexicon_tags_7 },
    { 0x6430817, SUC_lexicon_tags_2 },
    { 0xd7430818, SUC_lexicon_tags_38 },
    { 0xd1e30818, SUC_lexicon_tags_25 },
    { 0x4423081a, SUC_lexicon_tags_27 },
    { 0x61030815, SUC_lexicon_tags_11 },
    { 0xd843081c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3263081f, SUC_lexicon_tags_14 },
    { 0xcb230820, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c30823, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd830828, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d03082b, SUC_lexicon_tags_63 },
    { 0x4f23082c, SUC_lexicon_tags_25 },
    { 0x7263082d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xda43082f, SUC_lexicon_tags_14 },
    { 0x4f230830, SUC_lexicon_tags_43 },
    { 0x6ee30831, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a30836, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423083f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9830844, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e30848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c3084c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2030853, SUC_lexicon_tags_68 },
    { 0x93230854, SUC_lexicon_tags_14 },
    { 0x83630855, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b430857, SUC_lexicon_tags_56 },
    { 0x32430858, SUC_lexicon_tags_30 },
    { 0xd8030859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c3085d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9383085f, SUC_lexicon_tags_43 },
    { 0x2343085f, SUC_lexicon_tags_43 },
    { 0xe5430861, SUC_lexicon_tags_29 },
    { 0x62a30861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7630866, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x37630869, SUC_lexicon_tags_19 },
    { 0x65a3086a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a3086f, SUC_lexicon_tags_68 },
    { 0x59030870, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c30874, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e30877, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90430879, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3087e, SUC_lexicon_tags_25 },
    { 0xd2a3087f, SUC_lexicon_tags_0 },
    { 0xca23087e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2030883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51e30885, SUC_lexicon_tags_33 },
    { 0x8c230885, SUC_lexicon_tags_2 },
    { 0xd5630886, SUC_lexicon_tags_37 },
    { 0x1b830886, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2163088c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8430890, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d230897, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5c230899, SUC_lexicon_tags_2 },
    { 0x9dc3089a, SUC_lexicon_tags_19 },
    { 0xcd230899, SUC_lexicon_tags_85 },
    { 0x8d23089c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd903089f, SUC_lexicon_tags_14 },
    { 0xc86308a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3b4308a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c308a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36308a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee308b1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x18308b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90a308b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c308b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7fa308ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e308bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a308c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca308c5, SUC_lexicon_tags_37 },
    { 0x3f2308c6, SUC_lexicon_tags_1 },
    { 0xa0308c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe64308c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6308cc, SUC_lexicon_tags_6 },
    { 0x3ac308cd, SUC_lexicon_tags_14 },
    { 0xa80308cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de308d1, SUC_lexicon_tags_33 },
    { 0xde6308d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ae308d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e308d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94308dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe308e5, SUC_lexicon_tags_15 },
    { 0xf46308e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04308ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdda308ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x224308ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2308f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2308fa, SUC_lexicon_tags_21 },
    { 0x1e8308fb, SUC_lexicon_tags_82 },
    { 0xd6c308fc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc308ff, SUC_lexicon_tags_160 },
    { 0x85230900, SUC_lexicon_tags_25 },
    { 0x75230901, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35230903, SUC_lexicon_tags_62 },
    { 0x32430904, SUC_lexicon_tags_39 },
    { 0xc1630904, SUC_lexicon_tags_42 },
    { 0xbd830903, SUC_lexicon_tags_6 },
    { 0x2b630906, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2830911, SUC_lexicon_tags_14 },
    { 0x26a30911, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86830916, SUC_lexicon_tags_17 },
    { 0x31230917, SUC_lexicon_tags_25 },
    { 0xcdc30918, SUC_lexicon_tags_30 },
    { 0xdc430919, SUC_lexicon_tags_45 },
    { 0x71c3091a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f63091e, SUC_lexicon_tags_7 },
    { 0xcc03091f, SUC_lexicon_tags_7 },
    { 0x69a3091f, SUC_lexicon_tags_46 },
    { 0xf3830921, SUC_lexicon_tags_75 },
    { 0xe7c30922, SUC_lexicon_tags_47 },
    { 0x35630923, SUC_lexicon_tags_55 },
    { 0xadc30922, SUC_lexicon_tags_31 },
    { 0x38c30925, SUC_lexicon_tags_62 },
    { 0x8a630925, SUC_lexicon_tags_3 },
    { 0xc7230925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ee30929, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9363092c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62630932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1430934, SUC_lexicon_tags_1 },
    { 0xb0a30935, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97c30937, SUC_lexicon_tags_2 },
    { 0xa3630938, SUC_lexicon_tags_0 },
    { 0xbcc30939, SUC_lexicon_tags_31 },
    { 0xaa23093a, SUC_lexicon_tags_37 },
    { 0x7823093a, SUC_lexicon_tags_87 },
    { 0x2503093c, SUC_lexicon_tags_17 },
    { 0x56830939, SUC_lexicon_tags_25 },
    { 0x983093d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb230943, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8630947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe343094d, SUC_lexicon_tags_7 },
    { 0x4ca3094e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5030952, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf5430954, SUC_lexicon_tags_0 },
    { 0xba630955, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14430957, SUC_lexicon_tags_37 },
    { 0x1a630958, SUC_lexicon_tags_9 },
    { 0x6b630959, SUC_lexicon_tags_27 },
    { 0x35030957, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f83095e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x81230961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc30967, SUC_lexicon_tags_26 },
    { 0x5da30968, SUC_lexicon_tags_17 },
    { 0x1f830968, SUC_lexicon_tags_11 },
    { 0x2f63096a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f23096d, SUC_lexicon_tags_6 },
    { 0xb523096e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8b230970, SUC_lexicon_tags_11 },
    { 0xc4a30970, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc030977, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23097b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6cc3097d, SUC_lexicon_tags_47 },
    { 0x2943097d, SUC_lexicon_tags_17 },
    { 0xce23097d, SUC_lexicon_tags_2 },
    { 0xcb230980, SUC_lexicon_tags_37 },
    { 0x49a30981, SUC_lexicon_tags_31 },
    { 0x3c83097e, SUC_lexicon_tags_63 },
    { 0x61c3097e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c30988, SUC_lexicon_tags_11 },
    { 0x4830989, SUC_lexicon_tags_65 },
    { 0x3ca3098a, SUC_lexicon_tags_28 },
    { 0x37830989, SUC_lexicon_tags_7 },
    { 0x5783098c, SUC_lexicon_tags_11 },
    { 0x9c3098d, SUC_lexicon_tags_77 },
    { 0x3243098e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b030997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36c30999, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e3099e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86309a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c309a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c309ac, SUC_lexicon_tags_27 },
    { 0xe54309ad, SUC_lexicon_tags_55 },
    { 0x290309ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770309b3, SUC_lexicon_tags_2 },
    { 0x33c309b4, SUC_lexicon_tags_2 },
    { 0x870309b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166309ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a309be, SUC_lexicon_tags_17 },
    { 0x636309bf, SUC_lexicon_tags_11 },
    { 0xcd4309bf, SUC_lexicon_tags_25 },
    { 0x9ba309c0, SUC_lexicon_tags_37 },
    { 0xfe0309c0, SUC_lexicon_tags_9 },
    { 0x1a0309c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x372309c6, SUC_lexicon_tags_80 },
    { 0x7e2309c7, SUC_lexicon_tags_37 },
    { 0xd2e309c8, SUC_lexicon_tags_49 },
    { 0x9c6309c6, SUC_lexicon_tags_25 },
    { 0xa6c309ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0309ce, SUC_lexicon_tags_45 },
    { 0xb38309cf, SUC_lexicon_tags_19 },
    { 0x9f4309cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x33a309d2, SUC_lexicon_tags_37 },
    { 0x5f4309d3, SUC_lexicon_tags_33 },
    { 0x810309d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x108309d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x114309da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34309dc, SUC_lexicon_tags_9 },
    { 0xa46309dc, SUC_lexicon_tags_39 },
    { 0x6dc309dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc309f0, SUC_lexicon_tags_12 },
    { 0xff8309f0, SUC_lexicon_tags_17 },
    { 0x76e309f0, SUC_lexicon_tags_364 },
    { 0xeee309f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x872309f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e309fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50230a03, SUC_lexicon_tags_29 },
    { 0xf0a30a04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c30a06, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x30a30a08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9830a0b, SUC_lexicon_tags_7 },
    { 0x8d630a0c, SUC_lexicon_tags_2 },
    { 0x4230a0c, SUC_lexicon_tags_1 },
    { 0x45a30a0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a830a11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98030a14, SUC_lexicon_tags_17 },
    { 0x34630a14, SUC_lexicon_tags_28 },
    { 0xbb030a16, SUC_lexicon_tags_47 },
    { 0x91430a17, SUC_lexicon_tags_37 },
    { 0xbca30a18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2a30a1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4230a1c, SUC_lexicon_tags_37 },
    { 0x65830a1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x47c30a1f, SUC_lexicon_tags_17 },
    { 0x94430a20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82230a23, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xadc30a25, SUC_lexicon_tags_60 },
    { 0x1f230a25, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c230a2d, SUC_lexicon_tags_844 },
    { 0xe1030a2d, SUC_lexicon_tags_23 },
    { 0xac830a2f, SUC_lexicon_tags_0 },
    { 0xcde30a2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1630a32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b630a35, SUC_lexicon_tags_17 },
    { 0x63830a35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e30a39, SUC_lexicon_tags_7 },
    { 0x58030a39, SUC_lexicon_tags_15 },
    { 0x60e30a3a, SUC_lexicon_tags_23 },
    { 0xa8a30a3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca30a3f, SUC_lexicon_tags_2 },
    { 0xda230a40, SUC_lexicon_tags_14 },
    { 0x6fc30a41, SUC_lexicon_tags_17 },
    { 0xfd430a3f, SUC_lexicon_tags_7 },
    { 0x5da30a3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19030a45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55a30a47, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc30a4a, SUC_lexicon_tags_45 },
    { 0x26a30a4a, SUC_lexicon_tags_37 },
    { 0xe8c30a4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce30a50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c430a53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3b230a55, SUC_lexicon_tags_17 },
    { 0x87430a56, SUC_lexicon_tags_7 },
    { 0x36e30a57, SUC_lexicon_tags_29 },
    { 0xb7c30a58, SUC_lexicon_tags_31 },
    { 0x15e30a59, SUC_lexicon_tags_236 },
    { 0x65230a58, SUC_lexicon_tags_72 },
    { 0x6ac30a5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c30a5e, SUC_lexicon_tags_11 },
    { 0xbb830a5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f030a63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba30a69, SUC_lexicon_tags_63 },
    { 0x87c30a6a, SUC_lexicon_tags_37 },
    { 0xba030a6b, SUC_lexicon_tags_1 },
    { 0x9d430a6a, SUC_lexicon_tags_65 },
    { 0xc7430a6d, SUC_lexicon_tags_7 },
    { 0x3ee30a6e, SUC_lexicon_tags_3 },
    { 0xcc830a6a, SUC_lexicon_tags_25 },
    { 0x91430a70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ac30a72, SUC_lexicon_tags_17 },
    { 0x89630a73, SUC_lexicon_tags_26 },
    { 0x97230a74, SUC_lexicon_tags_60 },
    { 0x46c30a73, SUC_lexicon_tags_29 },
    { 0xb4230a75, SUC_lexicon_tags_11 },
    { 0x49e30a77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52430a7b, SUC_lexicon_tags_11 },
    { 0x96c30a7c, SUC_lexicon_tags_14 },
    { 0x29a30a7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb6230a7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71c30a81, SUC_lexicon_tags_64 },
    { 0xd8e30a82, SUC_lexicon_tags_37 },
    { 0x3ec30a83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcd430a85, SUC_lexicon_tags_7 },
    { 0xc1430a85, SUC_lexicon_tags_51 },
    { 0xf3c30a87, SUC_lexicon_tags_26 },
    { 0xf830a88, SUC_lexicon_tags_17 },
    { 0xcda30a88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e430a8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be30a90, SUC_lexicon_tags_43 },
    { 0xe9630a90, SUC_lexicon_tags_49 },
    { 0xe4030a92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb830a94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a30a9a, SUC_lexicon_tags_10 },
    { 0x45230a9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fa30a9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad630aa1, SUC_lexicon_tags_82 },
    { 0xd6430aa2, SUC_lexicon_tags_37 },
    { 0xe8c30aa3, SUC_lexicon_tags_10 },
    { 0xe0a30aa4, SUC_lexicon_tags_15 },
    { 0xfbe30aa5, SUC_lexicon_tags_43 },
    { 0x39030aa6, SUC_lexicon_tags_14 },
    { 0x7f830aa7, SUC_lexicon_tags_11 },
    { 0x74830aa7, SUC_lexicon_tags_11 },
    { 0xd6030aa9, SUC_lexicon_tags_21 },
    { 0xc2030aa9, SUC_lexicon_tags_19 },
    { 0xf230aab, SUC_lexicon_tags_1 },
    { 0x63a30aa1, SUC_lexicon_tags_25 },
    { 0xfae30aad, SUC_lexicon_tags_37 },
    { 0xe7430aa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2230ab1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35830ab5, SUC_lexicon_tags_25 },
    { 0x9ee30ab6, SUC_lexicon_tags_55 },
    { 0x24830ab7, SUC_lexicon_tags_1 },
    { 0xd6230ab8, SUC_lexicon_tags_37 },
    { 0x56e30ab9, SUC_lexicon_tags_45 },
    { 0xfd630ab7, SUC_lexicon_tags_37 },
    { 0x76230abb, SUC_lexicon_tags_11 },
    { 0x89430aba, SUC_lexicon_tags_10 },
    { 0x1c30abd, SUC_lexicon_tags_47 },
    { 0xa6630ab6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec430ac2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a30ac7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x35430aca, SUC_lexicon_tags_60 },
    { 0xe9430acb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd9c30acd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c830ad0, SUC_lexicon_tags_10 },
    { 0x6cc30ad1, SUC_lexicon_tags_25 },
    { 0x93630ad0, SUC_lexicon_tags_11 },
    { 0xdac30ad3, SUC_lexicon_tags_39 },
    { 0x4830ad1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a30ad7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37230ade, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46830ae9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50630aeb, SUC_lexicon_tags_41 },
    { 0x2630aeb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad830af2, SUC_lexicon_tags_8 },
    { 0x6dc30af2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e30af8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9430afa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a30b02, SUC_lexicon_tags_33 },
    { 0x51630b03, SUC_lexicon_tags_7 },
    { 0x9d430b04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab430b06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae30b0a, SUC_lexicon_tags_10 },
    { 0xe7a30b0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd830b0d, SUC_lexicon_tags_12 },
    { 0xd9e30b0e, SUC_lexicon_tags_7 },
    { 0xae430b0f, SUC_lexicon_tags_56 },
    { 0x1f630b10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc30b14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c430b1e, SUC_lexicon_tags_12 },
    { 0x92830b1f, SUC_lexicon_tags_37 },
    { 0xe9830b20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c30b24, SUC_lexicon_tags_11 },
    { 0x40c30b25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x84230b28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb430b32, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27830b38, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xada30b3b, SUC_lexicon_tags_2 },
    { 0xb3a30b3c, SUC_lexicon_tags_25 },
    { 0xba230b3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22030b40, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58430b4b, SUC_lexicon_tags_25 },
    { 0x2e30b4c, SUC_lexicon_tags_27 },
    { 0x96c30b4d, SUC_lexicon_tags_2 },
    { 0x6ec30b4e, SUC_lexicon_tags_15 },
    { 0x6b230b4f, SUC_lexicon_tags_1 },
    { 0xfaa30b4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xad430b52, SUC_lexicon_tags_8 },
    { 0xb9630b52, SUC_lexicon_tags_1 },
    { 0x2f630b54, SUC_lexicon_tags_55 },
    { 0xf1430b55, SUC_lexicon_tags_2 },
    { 0xf6230b55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a30b5b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80230b65, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xff430b68, SUC_lexicon_tags_6 },
    { 0x7dc30b69, SUC_lexicon_tags_25 },
    { 0x3da30b68, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e30b6d, SUC_lexicon_tags_2 },
    { 0x1230b6e, SUC_lexicon_tags_25 },
    { 0x45230b6e, SUC_lexicon_tags_9 },
    { 0xc8830b6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84430b73, SUC_lexicon_tags_23 },
    { 0xcb430b74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x71030b77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53030b7c, SUC_lexicon_tags_0 },
    { 0xab430b7d, SUC_lexicon_tags_25 },
    { 0x66e30b7c, SUC_lexicon_tags_9 },
    { 0x95e30b7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24630b82, SUC_lexicon_tags_37 },
    { 0xaec30b83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c430b86, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcc230b88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc30b8c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37830b90, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9a230b92, SUC_lexicon_tags_17 },
    { 0x7d230b93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc830b98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a30ba0, SUC_lexicon_tags_43 },
    { 0x13430ba1, SUC_lexicon_tags_11 },
    { 0x88030ba0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e30ba5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3fe30ba7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4630baa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2830bae, SUC_lexicon_tags_2 },
    { 0x33630baf, SUC_lexicon_tags_37 },
    { 0x9d830baf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c30bb3, SUC_lexicon_tags_14 },
    { 0xaa630bb4, SUC_lexicon_tags_28 },
    { 0x8ac30bb5, SUC_lexicon_tags_11 },
    { 0x8ce30bb6, SUC_lexicon_tags_7 },
    { 0x69030bb6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62430bbd, SUC_lexicon_tags_25 },
    { 0x6430bbe, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c30bc5, SUC_lexicon_tags_47 },
    { 0x38e30bc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c30bc9, SUC_lexicon_tags_65 },
    { 0x54e30bca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60e30bcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c30bd1, SUC_lexicon_tags_2 },
    { 0x70630bd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a830bd4, SUC_lexicon_tags_28 },
    { 0xf9c30bd4, SUC_lexicon_tags_43 },
    { 0x44830bd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x90e30bd8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1030bde, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b230be3, SUC_lexicon_tags_28 },
    { 0xcf830be4, SUC_lexicon_tags_29 },
    { 0x85a30be5, SUC_lexicon_tags_17 },
    { 0x9be30be6, SUC_lexicon_tags_37 },
    { 0x2c630be6, SUC_lexicon_tags_15 },
    { 0x5d030be7, SUC_lexicon_tags_15 },
    { 0xc4630be6, SUC_lexicon_tags_45 },
    { 0xe0430be5, SUC_lexicon_tags_12 },
    { 0x63c30beb, SUC_lexicon_tags_87 },
    { 0x97430bec, SUC_lexicon_tags_113 },
    { 0x53630bec, SUC_lexicon_tags_25 },
    { 0x7e030beb, SUC_lexicon_tags_1 },
    { 0x96630bef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e30bf2, SUC_lexicon_tags_1 },
    { 0xd0230bf3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x90230bf5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c30c01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22230c05, SUC_lexicon_tags_60 },
    { 0xe3a30c06, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e30c0a, SUC_lexicon_tags_14 },
    { 0xa8030c0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99030c11, SUC_lexicon_tags_27 },
    { 0x29030c11, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e30c15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c30c18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc30c1b, SUC_lexicon_tags_23 },
    { 0x86030c1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a30c20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf630c24, SUC_lexicon_tags_11 },
    { 0x6f630c25, SUC_lexicon_tags_30 },
    { 0xa6a30c24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67030c29, SUC_lexicon_tags_151 },
    { 0xf5a30c2a, SUC_lexicon_tags_11 },
    { 0xdbe30c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1c30c2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73430c32, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5a630c34, SUC_lexicon_tags_212 },
    { 0xd6830c34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x36230c38, SUC_lexicon_tags_17 },
    { 0x19e30c38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2230c3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea830c40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66030c42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11e30c44, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x2e30c46, SUC_lexicon_tags_7 },
    { 0xf5230c46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79830c4c, SUC_lexicon_tags_27 },
    { 0xf5230c4d, SUC_lexicon_tags_4 },
    { 0x84c30c4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b830c50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30430c55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70830c5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f230c5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84230c63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70630c65, SUC_lexicon_tags_82 },
    { 0x79430c66, SUC_lexicon_tags_37 },
    { 0x67630c67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a230c6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xba030c6d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d430c6f, SUC_lexicon_tags_27 },
    { 0xba030c70, SUC_lexicon_tags_6 },
    { 0x32030c71, SUC_lexicon_tags_37 },
    { 0xb2630c71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15230c74, SUC_lexicon_tags_19 },
    { 0xeec30c75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60230c7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9c30c81, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e30c84, SUC_lexicon_tags_17 },
    { 0xed830c85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7230c87, SUC_lexicon_tags_21 },
    { 0x4ea30c87, SUC_lexicon_tags_1 },
    { 0x71230c89, SUC_lexicon_tags_11 },
    { 0x5c830c8a, SUC_lexicon_tags_14 },
    { 0x65230c8b, SUC_lexicon_tags_15 },
    { 0xc9c30c89, SUC_lexicon_tags_275 },
    { 0xc830c8d, SUC_lexicon_tags_15 },
    { 0x87030c8e, SUC_lexicon_tags_2 },
    { 0x8aa30c8f, SUC_lexicon_tags_11 },
    { 0xaa230c8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c30c93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x81e30c95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14a30c97, SUC_lexicon_tags_7 },
    { 0x6ca30c97, SUC_lexicon_tags_33 },
    { 0xee630c99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e30c9c, SUC_lexicon_tags_31 },
    { 0xfc30c9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ee30c9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c30ca5, SUC_lexicon_tags_37 },
    { 0xde430ca5, SUC_lexicon_tags_1 },
    { 0x41430ca6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd030caa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe30cad, SUC_lexicon_tags_10 },
    { 0xde230cad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f430cb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5430cb6, SUC_lexicon_tags_21 },
    { 0x69230cb7, SUC_lexicon_tags_43 },
    { 0x79030cb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac230cba, SUC_lexicon_tags_46 },
    { 0x3630cba, SUC_lexicon_tags_13 },
    { 0x6f230cbc, SUC_lexicon_tags_2 },
    { 0x80030cbb, SUC_lexicon_tags_63 },
    { 0xfe630cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d430cc6, SUC_lexicon_tags_79 },
    { 0x80e30cc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec30ccb, SUC_lexicon_tags_9 },
    { 0x4da30ccc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51830cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3830cd2, SUC_lexicon_tags_27 },
    { 0x54e30cd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c30cd5, SUC_lexicon_tags_45 },
    { 0x5d430cd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7630cd9, SUC_lexicon_tags_13 },
    { 0x44a30cd9, SUC_lexicon_tags_0 },
    { 0xe5430cdb, SUC_lexicon_tags_343 },
    { 0xc5a30cd9, SUC_lexicon_tags_17 },
    { 0x62830cdd, SUC_lexicon_tags_12 },
    { 0xa0c30cdb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77830ce2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9230ce5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c430ce9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c30cec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59830cef, SUC_lexicon_tags_47 },
    { 0x4ae30cef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60230cf5, SUC_lexicon_tags_19 },
    { 0x38230cf6, SUC_lexicon_tags_11 },
    { 0xd2e30cf6, SUC_lexicon_tags_4 },
    { 0x4d430cf8, SUC_lexicon_tags_46 },
    { 0xdc30cf9, SUC_lexicon_tags_12 },
    { 0x7ee30cf9, SUC_lexicon_tags_25 },
    { 0xfc230cfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3030cff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14a30d01, SUC_lexicon_tags_19 },
    { 0x5bc30d01, SUC_lexicon_tags_11 },
    { 0xcd630d01, SUC_lexicon_tags_8 },
    { 0x7de30d04, SUC_lexicon_tags_0 },
    { 0x13030d05, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9430d0e, SUC_lexicon_tags_14 },
    { 0x90230d0e, SUC_lexicon_tags_15 },
    { 0x8a430d0e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x23830d13, SUC_lexicon_tags_33 },
    { 0x6e630d14, SUC_lexicon_tags_12 },
    { 0xdb830d15, SUC_lexicon_tags_55 },
    { 0x59630d14, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77830d1a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8430d24, SUC_lexicon_tags_30 },
    { 0x1230d25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa3c30d27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f030d2b, SUC_lexicon_tags_12 },
    { 0x30630d2c, SUC_lexicon_tags_15 },
    { 0xc5430d2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x30030d2f, SUC_lexicon_tags_1 },
    { 0xd7030d30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4c30d32, SUC_lexicon_tags_26 },
    { 0xc9e30d32, SUC_lexicon_tags_2 },
    { 0xbf430d34, SUC_lexicon_tags_11 },
    { 0x7fc30d34, SUC_lexicon_tags_37 },
    { 0xac830d36, SUC_lexicon_tags_1 },
    { 0x27a30d34, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x19430d39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f630d3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb230d42, SUC_lexicon_tags_26 },
    { 0x60830d42, SUC_lexicon_tags_11 },
    { 0x1b430d44, SUC_lexicon_tags_11 },
    { 0x5fc30d45, SUC_lexicon_tags_17 },
    { 0x1de30d46, SUC_lexicon_tags_15 },
    { 0x7dc30d47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35230d49, SUC_lexicon_tags_22 },
    { 0xe30d4a, SUC_lexicon_tags_30 },
    { 0xe6830d4b, SUC_lexicon_tags_11 },
    { 0x52430d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb230d4e, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x19430d50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8630d52, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0430d5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf830d5d, SUC_lexicon_tags_13 },
    { 0xbf030d5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e030d60, SUC_lexicon_tags_91 },
    { 0x1c830d61, SUC_lexicon_tags_48 },
    { 0x8f230d62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2830d64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac030d69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87230d6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e830d6e, SUC_lexicon_tags_25 },
    { 0xdd630d6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72830d73, SUC_lexicon_tags_17 },
    { 0xb3030d74, SUC_lexicon_tags_0 },
    { 0x17430d75, SUC_lexicon_tags_8 },
    { 0xe9030d74, SUC_lexicon_tags_14 },
    { 0x7de30d73, SUC_lexicon_tags_25 },
    { 0xf0230d78, SUC_lexicon_tags_9 },
    { 0x6e30d78, SUC_lexicon_tags_549 },
    { 0xae30d73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x86630d7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e30d80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e30d83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44c30d85, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a30d89, SUC_lexicon_tags_6 },
    { 0x30a30d8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac30d92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a30d97, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c430d9d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa430d9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14230da5, SUC_lexicon_tags_16 },
    { 0x80830da6, SUC_lexicon_tags_1 },
    { 0x16830da7, SUC_lexicon_tags_27 },
    { 0x70a30da8, SUC_lexicon_tags_4 },
    { 0xe8e30da5, SUC_lexicon_tags_37 },
    { 0x30c30da9, SUC_lexicon_tags_31 },
    { 0x69630daa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19630db0, SUC_lexicon_tags_39 },
    { 0x2830db0, SUC_lexicon_tags_11 },
    { 0x4e830db2, SUC_lexicon_tags_4 },
    { 0x86e30db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49630db8, SUC_lexicon_tags_7 },
    { 0x4230db9, SUC_lexicon_tags_5 },
    { 0x9c030dba, SUC_lexicon_tags_11 },
    { 0x32630dba, SUC_lexicon_tags_10 },
    { 0xf1830db9, SUC_lexicon_tags_47 },
    { 0xee230db9, SUC_lexicon_tags_15 },
    { 0x80a30dbe, SUC_lexicon_tags_45 },
    { 0xf5030dbf, SUC_lexicon_tags_8 },
    { 0x9a230dbe, SUC_lexicon_tags_0 },
    { 0x1e30dc0, SUC_lexicon_tags_37 },
    { 0xf3230dbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92c30dc4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdd630dc6, SUC_lexicon_tags_38 },
    { 0x18a30dc7, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x68230dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98630dcf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e230dd2, SUC_lexicon_tags_27 },
    { 0x3ec30dd3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc230dd7, SUC_lexicon_tags_13 },
    { 0xfa630dd7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c30ddd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xef830de0, SUC_lexicon_tags_56 },
    { 0x85e30de1, SUC_lexicon_tags_34 },
    { 0x84030de2, SUC_lexicon_tags_35 },
    { 0x5ba30de3, SUC_lexicon_tags_45 },
    { 0x8c230de4, SUC_lexicon_tags_12 },
    { 0xa5230de3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b430de8, SUC_lexicon_tags_15 },
    { 0xe2430de9, SUC_lexicon_tags_42 },
    { 0xf7e30de8, SUC_lexicon_tags_12 },
    { 0x88c30deb, SUC_lexicon_tags_43 },
    { 0x35c30deb, SUC_lexicon_tags_25 },
    { 0x7a630de8, SUC_lexicon_tags_17 },
    { 0xa9a30dec, SUC_lexicon_tags_11 },
    { 0x3c430def, SUC_lexicon_tags_15 },
    { 0x16c30df0, SUC_lexicon_tags_23 },
    { 0xb9030df1, SUC_lexicon_tags_7 },
    { 0xcb230df2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2c30df4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7830df8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e030dfa, SUC_lexicon_tags_11 },
    { 0xd7c30dfa, SUC_lexicon_tags_31 },
    { 0x24230dfb, SUC_lexicon_tags_25 },
    { 0xb7630dfd, SUC_lexicon_tags_27 },
    { 0x49e30dfe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc30e02, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a30e06, SUC_lexicon_tags_12 },
    { 0x3bc30e07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40430e0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x66c30e0c, SUC_lexicon_tags_26 },
    { 0x88230e0d, SUC_lexicon_tags_37 },
    { 0x21830e0e, SUC_lexicon_tags_55 },
    { 0x25030e0f, SUC_lexicon_tags_12 },
    { 0x74430e0f, SUC_lexicon_tags_15 },
    { 0xb5430e11, SUC_lexicon_tags_26 },
    { 0xf3230e11, SUC_lexicon_tags_14 },
    { 0xcf630e10, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0830e18, SUC_lexicon_tags_2 },
    { 0xc1230e19, SUC_lexicon_tags_82 },
    { 0xdb030e1a, SUC_lexicon_tags_25 },
    { 0x21c30e1a, SUC_lexicon_tags_11 },
    { 0x19c30e1a, SUC_lexicon_tags_43 },
    { 0x44430e1d, SUC_lexicon_tags_17 },
    { 0x3e630e1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64a30e20, SUC_lexicon_tags_60 },
    { 0x69630e20, SUC_lexicon_tags_66 },
    { 0x86a30e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfaa30e24, SUC_lexicon_tags_79 },
    { 0xf9a30e25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8430e29, SUC_lexicon_tags_19 },
    { 0x55630e29, SUC_lexicon_tags_34 },
    { 0xd3a30e29, SUC_lexicon_tags_11 },
    { 0xa8230e2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a30e30, SUC_lexicon_tags_37 },
    { 0x79830e31, SUC_lexicon_tags_25 },
    { 0x3c830e32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f830e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40230e3b, SUC_lexicon_tags_25 },
    { 0x8dc30e3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37630e3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70e30e41, SUC_lexicon_tags_38 },
    { 0x38430e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d030e46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3030e4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee30e50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15830e53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef630e57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed630e59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34e30e5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44e30e5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e30e63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a30e69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x78030e6c, SUC_lexicon_tags_23 },
    { 0xee830e6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60830e6f, SUC_lexicon_tags_17 },
    { 0x46230e70, SUC_lexicon_tags_14 },
    { 0xd6230e71, SUC_lexicon_tags_11 },
    { 0x98e30e71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf630e74, SUC_lexicon_tags_37 },
    { 0xec030e75, SUC_lexicon_tags_12 },
    { 0x79230e75, SUC_lexicon_tags_2 },
    { 0xcf030e75, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x14c30e79, SUC_lexicon_tags_15 },
    { 0xb2830e79, SUC_lexicon_tags_0 },
    { 0xaca30e7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73830e7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4c30e7f, SUC_lexicon_tags_11 },
    { 0xf4430e7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62830e82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf230e84, SUC_lexicon_tags_38 },
    { 0x8d230e85, SUC_lexicon_tags_74 },
    { 0x2a30e86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd430e89, SUC_lexicon_tags_37 },
    { 0x32230e89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x46630e8d, SUC_lexicon_tags_2 },
    { 0xbba30e8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a30e91, SUC_lexicon_tags_37 },
    { 0xe5430e92, SUC_lexicon_tags_47 },
    { 0xda30e93, SUC_lexicon_tags_15 },
    { 0x85430e94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x22c30e96, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce30e9a, SUC_lexicon_tags_2 },
    { 0x19030e9b, SUC_lexicon_tags_426 },
    { 0x2030e9b, SUC_lexicon_tags_25 },
    { 0xcee30e9d, SUC_lexicon_tags_15 },
    { 0x12a30e9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5230ea0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa30ea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a30eac, SUC_lexicon_tags_15 },
    { 0xf4230eac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a30eb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55830eb3, SUC_lexicon_tags_26 },
    { 0xe7c30eb3, SUC_lexicon_tags_37 },
    { 0xbd430eb5, SUC_lexicon_tags_31 },
    { 0x75030eb6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x78230eb8, SUC_lexicon_tags_14 },
    { 0x15230eb9, SUC_lexicon_tags_68 },
    { 0x61830eb9, SUC_lexicon_tags_32 },
    { 0x35030eb8, SUC_lexicon_tags_38 },
    { 0x96430ebc, SUC_lexicon_tags_1 },
    { 0xc0830eb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba430ec0, SUC_lexicon_tags_37 },
    { 0x46230ec1, SUC_lexicon_tags_61 },
    { 0xa7e30ec0, SUC_lexicon_tags_47 },
    { 0x91630ec2, SUC_lexicon_tags_26 },
    { 0xc0e30ec0, SUC_lexicon_tags_12 },
    { 0x52030ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c030ec7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x28430ec9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8ae30ecb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e30ed2, SUC_lexicon_tags_11 },
    { 0x89430ed2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc5830ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c30ed9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x46630edb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230edf, SUC_lexicon_tags_47 },
    { 0xa4830ee0, SUC_lexicon_tags_7 },
    { 0xc2230ee1, SUC_lexicon_tags_15 },
    { 0x76a30ee2, SUC_lexicon_tags_1 },
    { 0x86e30edf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b230ee9, SUC_lexicon_tags_87 },
    { 0x9be30ee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65230eef, SUC_lexicon_tags_38 },
    { 0xeda30eef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1830ef3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaf430ef5, SUC_lexicon_tags_33 },
    { 0x62430ef6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64630f03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a430f09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34030f0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf230f10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a430f16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0030f1a, SUC_lexicon_tags_7 },
    { 0xf1e30f1b, SUC_lexicon_tags_23 },
    { 0xd3c30f1b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de30f1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc30f22, SUC_lexicon_tags_37 },
    { 0x64630f22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae30f27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33230f29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4030f2b, SUC_lexicon_tags_12 },
    { 0xfb230f2b, SUC_lexicon_tags_108 },
    { 0x94030f2d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f430f34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda30f3c, SUC_lexicon_tags_11 },
    { 0x7e430f3d, SUC_lexicon_tags_37 },
    { 0xe6230f3c, SUC_lexicon_tags_17 },
    { 0x18230f3c, SUC_lexicon_tags_12 },
    { 0x41230f40, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x93e30f42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab430f48, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a30f4b, SUC_lexicon_tags_19 },
    { 0xdfa30f4c, SUC_lexicon_tags_92 },
    { 0x89830f4b, SUC_lexicon_tags_47 },
    { 0xd1430f4e, SUC_lexicon_tags_14 },
    { 0x6a030f4f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55030f51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a30f56, SUC_lexicon_tags_2 },
    { 0xdde30f57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce30f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4430f5c, SUC_lexicon_tags_11 },
    { 0xc3430f5d, SUC_lexicon_tags_61 },
    { 0xc4230f5e, SUC_lexicon_tags_68 },
    { 0x40430f5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53c30f61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64a30f63, SUC_lexicon_tags_24 },
    { 0xfb630f63, SUC_lexicon_tags_15 },
    { 0x41230f65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x97c30f67, SUC_lexicon_tags_47 },
    { 0x32030f67, SUC_lexicon_tags_9 },
    { 0x7f830f69, SUC_lexicon_tags_34 },
    { 0x86030f6a, SUC_lexicon_tags_15 },
    { 0xf1e30f6b, SUC_lexicon_tags_45 },
    { 0xf8a30f6c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x1fc30f6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1430f70, SUC_lexicon_tags_23 },
    { 0x3ac30f71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64830f73, SUC_lexicon_tags_37 },
    { 0x7be30f74, SUC_lexicon_tags_25 },
    { 0x20e30f73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a30f78, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x42430f7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6030f7c, SUC_lexicon_tags_49 },
    { 0x89030f7c, SUC_lexicon_tags_19 },
    { 0x2e030f7e, SUC_lexicon_tags_17 },
    { 0xbf230f7e, SUC_lexicon_tags_49 },
    { 0x2c830f80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5830f83, SUC_lexicon_tags_11 },
    { 0x5b230f84, SUC_lexicon_tags_68 },
    { 0x4ee30f85, SUC_lexicon_tags_11 },
    { 0x14430f86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce30f8b, SUC_lexicon_tags_30 },
    { 0xe5630f8c, SUC_lexicon_tags_2 },
    { 0x86830f8d, SUC_lexicon_tags_57 },
    { 0x5a30f8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c30f92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e30f95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45430f98, SUC_lexicon_tags_40 },
    { 0x1a630f98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d430f9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2830fa0, SUC_lexicon_tags_87 },
    { 0xc0a30fa1, SUC_lexicon_tags_43 },
    { 0x10e30fa1, SUC_lexicon_tags_17 },
    { 0x6a430fa3, SUC_lexicon_tags_15 },
    { 0xba30fa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da30fab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca430fad, SUC_lexicon_tags_37 },
    { 0x23a30fae, SUC_lexicon_tags_12 },
    { 0x2630faf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fa30fb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9030fb7, SUC_lexicon_tags_87 },
    { 0x3e030fb7, SUC_lexicon_tags_39 },
    { 0x91230fb9, SUC_lexicon_tags_38 },
    { 0xee030fb8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xfe230fbc, SUC_lexicon_tags_15 },
    { 0x1ec30fbd, SUC_lexicon_tags_15 },
    { 0xde630fbc, SUC_lexicon_tags_47 },
    { 0x97830fbf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e30fc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53430fcb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3fe30fcd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d430fd1, SUC_lexicon_tags_25 },
    { 0x51630fd2, SUC_lexicon_tags_27 },
    { 0x99030fd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9c30fd5, SUC_lexicon_tags_33 },
    { 0xc5e30fd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d430fdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d030fe6, SUC_lexicon_tags_32 },
    { 0xae230fe7, SUC_lexicon_tags_8 },
    { 0x4a230fe8, SUC_lexicon_tags_37 },
    { 0xa9e30fe9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2a30feb, SUC_lexicon_tags_30 },
    { 0x90030feb, SUC_lexicon_tags_12 },
    { 0x70c30feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a30ff1, SUC_lexicon_tags_16 },
    { 0x12830ff1, SUC_lexicon_tags_1042 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c30ff5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da31000, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb431002, SUC_lexicon_tags_1326 },
    { 0x1d831003, SUC_lexicon_tags_1438 },
    { 0, NULL },
    { 0, NULL },
    { 0xd431006, SUC_lexicon_tags_37 },
    { 0x71631007, SUC_lexicon_tags_49 },
    { 0x4fc31006, SUC_lexicon_tags_45 },
    { 0xe0c31009, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2843100d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba31010, SUC_lexicon_tags_37 },
    { 0x4c831010, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a31016, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcea31018, SUC_lexicon_tags_4 },
    { 0x92631019, SUC_lexicon_tags_11 },
    { 0x6ba3101a, SUC_lexicon_tags_7 },
    { 0xb7e3101b, SUC_lexicon_tags_0 },
    { 0xe003101c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83831020, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x24c31022, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57231024, SUC_lexicon_tags_2 },
    { 0x9be31024, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa4c31027, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb583102a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9023102e, SUC_lexicon_tags_2 },
    { 0x1ba3102f, SUC_lexicon_tags_7 },
    { 0x1583102e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35631039, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec3103c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda431040, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c031043, SUC_lexicon_tags_63 },
    { 0x33e31044, SUC_lexicon_tags_323 },
    { 0x7e031045, SUC_lexicon_tags_10 },
    { 0x65e31044, SUC_lexicon_tags_25 },
    { 0x50a31047, SUC_lexicon_tags_1 },
    { 0xa7231044, SUC_lexicon_tags_64 },
    { 0xf1c31048, SUC_lexicon_tags_27 },
    { 0x36831044, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d63104d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f631055, SUC_lexicon_tags_19 },
    { 0xc231055, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca3105a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4203105c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68431063, SUC_lexicon_tags_37 },
    { 0x4f231064, SUC_lexicon_tags_2 },
    { 0xab231065, SUC_lexicon_tags_27 },
    { 0x9e631066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf231068, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa3106b, SUC_lexicon_tags_48 },
    { 0xea03106c, SUC_lexicon_tags_63 },
    { 0x44e3106c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x99231070, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7431072, SUC_lexicon_tags_37 },
    { 0x32831073, SUC_lexicon_tags_53 },
    { 0x2d431074, SUC_lexicon_tags_37 },
    { 0xf2e31075, SUC_lexicon_tags_1 },
    { 0x90631075, SUC_lexicon_tags_33 },
    { 0x98631074, SUC_lexicon_tags_33 },
    { 0x3e631077, SUC_lexicon_tags_66 },
    { 0x2b431078, SUC_lexicon_tags_25 },
    { 0xb8631073, SUC_lexicon_tags_17 },
    { 0xab23107b, SUC_lexicon_tags_7 },
    { 0xaf031077, SUC_lexicon_tags_15 },
    { 0x8231079, SUC_lexicon_tags_25 },
    { 0xab83107e, SUC_lexicon_tags_15 },
    { 0x2583107f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdd631081, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8031085, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c231088, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8683108a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf503108c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd831091, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53031095, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c3109c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40310a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae310a5, SUC_lexicon_tags_43 },
    { 0xe34310a6, SUC_lexicon_tags_87 },
    { 0x2a310a7, SUC_lexicon_tags_37 },
    { 0x8a4310a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x54e310aa, SUC_lexicon_tags_12 },
    { 0xb78310aa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824310b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x250310b2, SUC_lexicon_tags_12 },
    { 0xaf8310b3, SUC_lexicon_tags_43 },
    { 0x55a310b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2310b8, SUC_lexicon_tags_55 },
    { 0xe66310b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18310bd, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88310c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe310c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x582310c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x662310c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896310cd, SUC_lexicon_tags_27 },
    { 0x16c310ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce310d3, SUC_lexicon_tags_21 },
    { 0xef8310d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf68310d6, SUC_lexicon_tags_1 },
    { 0xe28310d7, SUC_lexicon_tags_11 },
    { 0xb0e310d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x836310da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12310e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66310e5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x422310e7, SUC_lexicon_tags_27 },
    { 0xba4310e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e310f0, SUC_lexicon_tags_47 },
    { 0x8be310f0, SUC_lexicon_tags_1 },
    { 0xdd0310f1, SUC_lexicon_tags_47 },
    { 0x152310f0, SUC_lexicon_tags_11 },
    { 0x6c2310f3, SUC_lexicon_tags_25 },
    { 0xe5c310f5, SUC_lexicon_tags_47 },
    { 0x520310f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x876310f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae310fc, SUC_lexicon_tags_28 },
    { 0xc62310fc, SUC_lexicon_tags_12 },
    { 0x336310fe, SUC_lexicon_tags_38 },
    { 0xe48310ff, SUC_lexicon_tags_11 },
    { 0x954310ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x64431102, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf463110c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a31110, SUC_lexicon_tags_11 },
    { 0xa8e31111, SUC_lexicon_tags_37 },
    { 0x66631112, SUC_lexicon_tags_7 },
    { 0xf6831111, SUC_lexicon_tags_7 },
    { 0xb3a31114, SUC_lexicon_tags_15 },
    { 0xc7831115, SUC_lexicon_tags_16 },
    { 0x97831116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a3111a, SUC_lexicon_tags_63 },
    { 0x4a63111b, SUC_lexicon_tags_11 },
    { 0x1b43111c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a3111f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24431124, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe963112f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d031132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaea31134, SUC_lexicon_tags_25 },
    { 0x1631134, SUC_lexicon_tags_63 },
    { 0x77231134, SUC_lexicon_tags_29 },
    { 0x4d831137, SUC_lexicon_tags_14 },
    { 0xba431138, SUC_lexicon_tags_1 },
    { 0xb6031139, SUC_lexicon_tags_47 },
    { 0xb0e31139, SUC_lexicon_tags_60 },
    { 0x7323113b, SUC_lexicon_tags_33 },
    { 0xa3631134, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3a43113e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4c231140, SUC_lexicon_tags_17 },
    { 0x7ac31141, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e31144, SUC_lexicon_tags_4 },
    { 0xefe31145, SUC_lexicon_tags_39 },
    { 0x87e31146, SUC_lexicon_tags_45 },
    { 0xda31144, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e3114f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18631151, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38431156, SUC_lexicon_tags_11 },
    { 0x4f031157, SUC_lexicon_tags_22 },
    { 0x59031158, SUC_lexicon_tags_17 },
    { 0xd831159, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a3115c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdca3115e, SUC_lexicon_tags_11 },
    { 0x5803115f, SUC_lexicon_tags_68 },
    { 0xa603115e, SUC_lexicon_tags_0 },
    { 0xb223115e, SUC_lexicon_tags_15 },
    { 0xe7631162, SUC_lexicon_tags_6 },
    { 0x42231163, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e31166, SUC_lexicon_tags_60 },
    { 0x76a31167, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe3116b, SUC_lexicon_tags_45 },
    { 0x6ea3116b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed43116e, SUC_lexicon_tags_45 },
    { 0xd83116e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2831172, SUC_lexicon_tags_12 },
    { 0x45a31172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89631177, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea83117c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c31181, SUC_lexicon_tags_17 },
    { 0xe6031182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53231186, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55631188, SUC_lexicon_tags_19 },
    { 0xbd831188, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf683118d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56031191, SUC_lexicon_tags_25 },
    { 0x41231192, SUC_lexicon_tags_14 },
    { 0xfa831193, SUC_lexicon_tags_19 },
    { 0x79831192, SUC_lexicon_tags_43 },
    { 0x43e31194, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3ca31197, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf603119a, SUC_lexicon_tags_30 },
    { 0x7b03119b, SUC_lexicon_tags_15 },
    { 0xb63119c, SUC_lexicon_tags_37 },
    { 0xf9e3119d, SUC_lexicon_tags_68 },
    { 0x2703119d, SUC_lexicon_tags_47 },
    { 0xa343119f, SUC_lexicon_tags_74 },
    { 0xd363119e, SUC_lexicon_tags_27 },
    { 0xc5e311a1, SUC_lexicon_tags_12 },
    { 0x9ac311a2, SUC_lexicon_tags_0 },
    { 0x89e3119c, SUC_lexicon_tags_12 },
    { 0xa8a311a4, SUC_lexicon_tags_12 },
    { 0x8b2311a5, SUC_lexicon_tags_162 },
    { 0x992311a4, SUC_lexicon_tags_17 },
    { 0x49e311a7, SUC_lexicon_tags_12 },
    { 0x380311a7, SUC_lexicon_tags_38 },
    { 0x4e6311a9, SUC_lexicon_tags_47 },
    { 0x2a2311a6, SUC_lexicon_tags_16 },
    { 0x41a311ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1c8311ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xada311af, SUC_lexicon_tags_1 },
    { 0x382311b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22311b5, SUC_lexicon_tags_42 },
    { 0x66a311b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x928311b8, SUC_lexicon_tags_7 },
    { 0xbb0311b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0311bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4311bf, SUC_lexicon_tags_12 },
    { 0x4f0311bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02311c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14311ce, SUC_lexicon_tags_1 },
    { 0x634311ce, SUC_lexicon_tags_54 },
    { 0xd28311cf, SUC_lexicon_tags_25 },
    { 0xf56311d1, SUC_lexicon_tags_7 },
    { 0xcfc311d2, SUC_lexicon_tags_37 },
    { 0x8f6311d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a311d9, SUC_lexicon_tags_1 },
    { 0x950311da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x912311dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6311e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00311eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2311ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c311f1, SUC_lexicon_tags_33 },
    { 0x596311f2, SUC_lexicon_tags_62 },
    { 0x396311f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4311f6, SUC_lexicon_tags_37 },
    { 0xeaa311f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec311fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x150311fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x198311ff, SUC_lexicon_tags_63 },
    { 0xac6311ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9231202, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f631206, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x66631209, SUC_lexicon_tags_11 },
    { 0xac23120a, SUC_lexicon_tags_37 },
    { 0x30e31209, SUC_lexicon_tags_15 },
    { 0xb0e3120c, SUC_lexicon_tags_45 },
    { 0xe463120d, SUC_lexicon_tags_7 },
    { 0x5c63120d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf831212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa723121a, SUC_lexicon_tags_25 },
    { 0x5b63121b, SUC_lexicon_tags_43 },
    { 0x71a3121b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2631220, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a31223, SUC_lexicon_tags_15 },
    { 0xb7431224, SUC_lexicon_tags_1 },
    { 0x28231225, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x42e31227, SUC_lexicon_tags_23 },
    { 0xefe31227, SUC_lexicon_tags_1 },
    { 0xe6831229, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa83122c, SUC_lexicon_tags_11 },
    { 0xfa3122d, SUC_lexicon_tags_49 },
    { 0x41e3122e, SUC_lexicon_tags_17 },
    { 0x19a3122f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xade31231, SUC_lexicon_tags_34 },
    { 0x8e231232, SUC_lexicon_tags_6 },
    { 0x6b431233, SUC_lexicon_tags_15 },
    { 0x29831234, SUC_lexicon_tags_14 },
    { 0xc5e31235, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae3123b, SUC_lexicon_tags_1 },
    { 0xb003123b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x78c3123e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe631243, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8631247, SUC_lexicon_tags_15 },
    { 0xa4031248, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc483124c, SUC_lexicon_tags_7 },
    { 0x65e3124d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f431250, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ba31252, SUC_lexicon_tags_37 },
    { 0xca231253, SUC_lexicon_tags_1 },
    { 0x88631254, SUC_lexicon_tags_2 },
    { 0x65431255, SUC_lexicon_tags_15 },
    { 0x3cc31253, SUC_lexicon_tags_118 },
    { 0x8b431252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfde31259, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a31260, SUC_lexicon_tags_37 },
    { 0x42e31261, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a31268, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada31270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc431275, SUC_lexicon_tags_37 },
    { 0x23a31276, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8223127c, SUC_lexicon_tags_14 },
    { 0x59e3127d, SUC_lexicon_tags_5 },
    { 0x5c23127c, SUC_lexicon_tags_37 },
    { 0x42c3127e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb631282, SUC_lexicon_tags_11 },
    { 0xc3431283, SUC_lexicon_tags_1 },
    { 0x1b231284, SUC_lexicon_tags_11 },
    { 0x93831285, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b63128c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4163128f, SUC_lexicon_tags_26 },
    { 0x6083128f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfe031292, SUC_lexicon_tags_15 },
    { 0xf4e31292, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9631297, SUC_lexicon_tags_19 },
    { 0xb3c31297, SUC_lexicon_tags_60 },
    { 0x55231298, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4803129b, SUC_lexicon_tags_7 },
    { 0xc463129b, SUC_lexicon_tags_15 },
    { 0xe763129d, SUC_lexicon_tags_1 },
    { 0x9c23129e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce2312a0, SUC_lexicon_tags_23 },
    { 0x7ec312a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x802312a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e312a6, SUC_lexicon_tags_0 },
    { 0x5d4312a6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6d0312a9, SUC_lexicon_tags_10 },
    { 0xc30312aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x580312ac, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6312b2, SUC_lexicon_tags_7 },
    { 0x6b8312b3, SUC_lexicon_tags_11 },
    { 0x8b0312b3, SUC_lexicon_tags_7 },
    { 0x222312b5, SUC_lexicon_tags_15 },
    { 0x180312b6, SUC_lexicon_tags_31 },
    { 0x224312b7, SUC_lexicon_tags_18 },
    { 0x3f6312b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278312bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc42312be, SUC_lexicon_tags_15 },
    { 0x214312bf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x464312c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x33c312c4, SUC_lexicon_tags_60 },
    { 0x8f8312c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10312c9, SUC_lexicon_tags_28 },
    { 0xf50312ca, SUC_lexicon_tags_17 },
    { 0x1da312cb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948312d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x954312d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a0312d6, SUC_lexicon_tags_10 },
    { 0x4f4312d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c312df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46312e4, SUC_lexicon_tags_11 },
    { 0x44e312e4, SUC_lexicon_tags_0 },
    { 0x4c2312e6, SUC_lexicon_tags_6 },
    { 0xbd2312e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x572312e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0312ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da312f5, SUC_lexicon_tags_61 },
    { 0x92e312f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee312f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43a312fb, SUC_lexicon_tags_28 },
    { 0x6a2312fc, SUC_lexicon_tags_37 },
    { 0xa08312fb, SUC_lexicon_tags_3 },
    { 0xc9a312fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe31301, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef431303, SUC_lexicon_tags_69 },
    { 0x1b231304, SUC_lexicon_tags_12 },
    { 0x38831304, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb031309, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf843130d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2883130f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff631313, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6231317, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c63131c, SUC_lexicon_tags_17 },
    { 0xda03131d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36031321, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31431323, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf831325, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x58031328, SUC_lexicon_tags_26 },
    { 0x93031329, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1631331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36431334, SUC_lexicon_tags_25 },
    { 0x27431335, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d431337, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x22831339, SUC_lexicon_tags_25 },
    { 0x7523133a, SUC_lexicon_tags_17 },
    { 0x1343133b, SUC_lexicon_tags_25 },
    { 0x3fe3133c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37231340, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4431344, SUC_lexicon_tags_15 },
    { 0x1b431345, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65031348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57e3134a, SUC_lexicon_tags_15 },
    { 0x5de3134b, SUC_lexicon_tags_23 },
    { 0x1e83134c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1a3134e, SUC_lexicon_tags_33 },
    { 0x1623134f, SUC_lexicon_tags_33 },
    { 0x15a31350, SUC_lexicon_tags_30 },
    { 0x8be31350, SUC_lexicon_tags_11 },
    { 0x81031352, SUC_lexicon_tags_33 },
    { 0x28231351, SUC_lexicon_tags_11 },
    { 0x58a3134e, SUC_lexicon_tags_7 },
    { 0x95c31355, SUC_lexicon_tags_26 },
    { 0x8343134e, SUC_lexicon_tags_27 },
    { 0x5a431357, SUC_lexicon_tags_30 },
    { 0x41c31358, SUC_lexicon_tags_37 },
    { 0x4543134e, SUC_lexicon_tags_9 },
    { 0x81231351, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2a3135c, SUC_lexicon_tags_11 },
    { 0x1e83135c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x8843135f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a31368, SUC_lexicon_tags_7 },
    { 0x79c31368, SUC_lexicon_tags_27 },
    { 0x53e3136a, SUC_lexicon_tags_25 },
    { 0x2b03136b, SUC_lexicon_tags_15 },
    { 0x3543136b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3136f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad831371, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e31375, SUC_lexicon_tags_2 },
    { 0xe1831375, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c3137a, SUC_lexicon_tags_19 },
    { 0x29e3137b, SUC_lexicon_tags_12 },
    { 0x6aa3137b, SUC_lexicon_tags_2 },
    { 0xba3137c, SUC_lexicon_tags_66 },
    { 0x1923137e, SUC_lexicon_tags_23 },
    { 0x86c3137f, SUC_lexicon_tags_16 },
    { 0x84031380, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x31831383, SUC_lexicon_tags_79 },
    { 0x9e031384, SUC_lexicon_tags_47 },
    { 0xb9031385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1231388, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8c3138a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe2c3138c, SUC_lexicon_tags_118 },
    { 0x3663138d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe43138f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xed031392, SUC_lexicon_tags_15 },
    { 0x64231393, SUC_lexicon_tags_63 },
    { 0xa0431394, SUC_lexicon_tags_17 },
    { 0x63a31393, SUC_lexicon_tags_48 },
    { 0x75431396, SUC_lexicon_tags_21 },
    { 0x61231397, SUC_lexicon_tags_32 },
    { 0x13031396, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1063139d, SUC_lexicon_tags_2 },
    { 0x9883139e, SUC_lexicon_tags_45 },
    { 0x2fc3139e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae313a3, SUC_lexicon_tags_47 },
    { 0x66313a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c313ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97a313b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394313b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca313bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0313c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c313c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x402313c6, SUC_lexicon_tags_40 },
    { 0xb7e313c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeea313c9, SUC_lexicon_tags_12 },
    { 0x286313ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c2313cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86313d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00313d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39e313da, SUC_lexicon_tags_55 },
    { 0xb80313db, SUC_lexicon_tags_17 },
    { 0xb0313db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd26313de, SUC_lexicon_tags_2 },
    { 0x1f4313df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c8313e1, SUC_lexicon_tags_60 },
    { 0x856313e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88313e6, SUC_lexicon_tags_45 },
    { 0x5e4313e7, SUC_lexicon_tags_27 },
    { 0x394313e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x440313eb, SUC_lexicon_tags_23 },
    { 0x3c0313eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e313f0, SUC_lexicon_tags_35 },
    { 0xcb0313f1, SUC_lexicon_tags_37 },
    { 0x3ac313f1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac313f7, SUC_lexicon_tags_7 },
    { 0xbde313f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3bc313fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4313fd, SUC_lexicon_tags_7 },
    { 0xf34313fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca031401, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e631404, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a31409, SUC_lexicon_tags_37 },
    { 0x7683140a, SUC_lexicon_tags_14 },
    { 0x69031409, SUC_lexicon_tags_13 },
    { 0x7a631409, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6723140e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69631410, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x53c31412, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0631414, SUC_lexicon_tags_7 },
    { 0xb8031415, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc31419, SUC_lexicon_tags_38 },
    { 0x98431419, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe683141c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f63141f, SUC_lexicon_tags_19 },
    { 0x3bc31420, SUC_lexicon_tags_48 },
    { 0xe2231421, SUC_lexicon_tags_39 },
    { 0x8c831422, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a31425, SUC_lexicon_tags_39 },
    { 0x75e31425, SUC_lexicon_tags_37 },
    { 0xc7231427, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b43142b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a3142f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc231431, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x41831433, SUC_lexicon_tags_14 },
    { 0x97031434, SUC_lexicon_tags_0 },
    { 0x19831433, SUC_lexicon_tags_2 },
    { 0xb2231436, SUC_lexicon_tags_27 },
    { 0x7d831437, SUC_lexicon_tags_13 },
    { 0xf4431436, SUC_lexicon_tags_49 },
    { 0xab431439, SUC_lexicon_tags_0 },
    { 0xbee31436, SUC_lexicon_tags_1 },
    { 0x6f63143b, SUC_lexicon_tags_37 },
    { 0x28a3143c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeae3143e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd831442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb231448, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb943144a, SUC_lexicon_tags_60 },
    { 0x3f03144b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62031458, SUC_lexicon_tags_15 },
    { 0x55a31459, SUC_lexicon_tags_21 },
    { 0x4a3145a, SUC_lexicon_tags_33 },
    { 0xd823145b, SUC_lexicon_tags_63 },
    { 0x5c03145b, SUC_lexicon_tags_17 },
    { 0xb403145d, SUC_lexicon_tags_6 },
    { 0x85231458, SUC_lexicon_tags_23 },
    { 0xc363145f, SUC_lexicon_tags_9 },
    { 0xda63145d, SUC_lexicon_tags_11 },
    { 0x83c3145f, SUC_lexicon_tags_87 },
    { 0x84e31462, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4ce31464, SUC_lexicon_tags_29 },
    { 0xe0631465, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62431467, SUC_lexicon_tags_14 },
    { 0x49431467, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7de3146a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a3146e, SUC_lexicon_tags_1 },
    { 0xf003146e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c31472, SUC_lexicon_tags_25 },
    { 0xe5231473, SUC_lexicon_tags_28 },
    { 0x65631474, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44231478, SUC_lexicon_tags_12 },
    { 0x38431478, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdb43147b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c31482, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24431485, SUC_lexicon_tags_38 },
    { 0xdc231485, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b831488, SUC_lexicon_tags_11 },
    { 0x92431489, SUC_lexicon_tags_0 },
    { 0x2d03148a, SUC_lexicon_tags_37 },
    { 0xa663148b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf003148e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee31492, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc31495, SUC_lexicon_tags_15 },
    { 0x1c031495, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ec31498, SUC_lexicon_tags_23 },
    { 0x55031499, SUC_lexicon_tags_9 },
    { 0xbc231498, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe283149c, SUC_lexicon_tags_37 },
    { 0x5463149c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e314a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64314a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30314a7, SUC_lexicon_tags_43 },
    { 0x6ea314a8, SUC_lexicon_tags_10 },
    { 0x11a314a9, SUC_lexicon_tags_13 },
    { 0x87e314a8, SUC_lexicon_tags_108 },
    { 0x886314ab, SUC_lexicon_tags_0 },
    { 0x2fc314ac, SUC_lexicon_tags_34 },
    { 0xf4e314ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c314b3, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e314b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd8c314b9, SUC_lexicon_tags_7 },
    { 0x3c6314b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4314bf, SUC_lexicon_tags_12 },
    { 0xa5a314bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe5c314c2, SUC_lexicon_tags_12 },
    { 0xb2314c2, SUC_lexicon_tags_25 },
    { 0xb6c314c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546314c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e314cc, SUC_lexicon_tags_1 },
    { 0xaae314cd, SUC_lexicon_tags_14 },
    { 0x7c2314cd, SUC_lexicon_tags_60 },
    { 0x680314cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe314d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x498314d4, SUC_lexicon_tags_16 },
    { 0x358314d4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8314d9, SUC_lexicon_tags_9 },
    { 0x58c314d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c8314dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4314e4, SUC_lexicon_tags_15 },
    { 0xc60314e4, SUC_lexicon_tags_23 },
    { 0x450314e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332314ec, SUC_lexicon_tags_12 },
    { 0xc88314ed, SUC_lexicon_tags_23 },
    { 0xa70314ee, SUC_lexicon_tags_47 },
    { 0x7f8314ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0314f2, SUC_lexicon_tags_12 },
    { 0xcf4314f2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x8c314f5, SUC_lexicon_tags_11 },
    { 0xb72314f6, SUC_lexicon_tags_689 },
    { 0xb38314f6, SUC_lexicon_tags_26 },
    { 0xf1a314f8, SUC_lexicon_tags_23 },
    { 0x842314f8, SUC_lexicon_tags_30 },
    { 0xd58314fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad031500, SUC_lexicon_tags_10 },
    { 0x8b431501, SUC_lexicon_tags_15 },
    { 0xb9231501, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e031504, SUC_lexicon_tags_43 },
    { 0xbf831505, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97831507, SUC_lexicon_tags_23 },
    { 0xfa831508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4e3150a, SUC_lexicon_tags_11 },
    { 0x1fa3150a, SUC_lexicon_tags_7 },
    { 0xf243150c, SUC_lexicon_tags_9 },
    { 0x4dc3150d, SUC_lexicon_tags_37 },
    { 0x2b83150e, SUC_lexicon_tags_65 },
    { 0x9103150c, SUC_lexicon_tags_37 },
    { 0x88431510, SUC_lexicon_tags_37 },
    { 0xde631511, SUC_lexicon_tags_14 },
    { 0x53431512, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c31516, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea3151a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca3151e, SUC_lexicon_tags_15 },
    { 0xd8e3151e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e31524, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e031526, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e3152d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf963152f, SUC_lexicon_tags_37 },
    { 0x68231530, SUC_lexicon_tags_29 },
    { 0x1031531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d431533, SUC_lexicon_tags_11 },
    { 0xcfc31534, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b63153b, SUC_lexicon_tags_26 },
    { 0xe3c3153b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26c3153e, SUC_lexicon_tags_38 },
    { 0xa943153f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a631542, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79231545, SUC_lexicon_tags_7 },
    { 0x7631546, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xee231548, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xd583154b, SUC_lexicon_tags_12 },
    { 0x7e3154c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a23154f, SUC_lexicon_tags_63 },
    { 0x3de31550, SUC_lexicon_tags_17 },
    { 0x8c23154f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c3155a, SUC_lexicon_tags_12 },
    { 0x5c83155b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4823155f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e831564, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18831566, SUC_lexicon_tags_0 },
    { 0x10031566, SUC_lexicon_tags_17 },
    { 0x23631568, SUC_lexicon_tags_13 },
    { 0x3cc31566, SUC_lexicon_tags_47 },
    { 0x24a3156a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30031570, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a631573, SUC_lexicon_tags_37 },
    { 0xc4a31573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e31580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42431583, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xca631585, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7231589, SUC_lexicon_tags_37 },
    { 0x6523158a, SUC_lexicon_tags_25 },
    { 0xfd631589, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x6863158d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a31590, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7431595, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac631597, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9103159c, SUC_lexicon_tags_2 },
    { 0xb123159c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8315a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4da315a2, SUC_lexicon_tags_59 },
    { 0x5fc315a3, SUC_lexicon_tags_11 },
    { 0x3c2315a3, SUC_lexicon_tags_33 },
    { 0x7ea315a4, SUC_lexicon_tags_3 },
    { 0xc5c315a4, SUC_lexicon_tags_77 },
    { 0xf68315a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730315ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238315b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516315b6, SUC_lexicon_tags_14 },
    { 0xde6315b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e315ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49c315bc, SUC_lexicon_tags_45 },
    { 0x936315bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2315c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x622315c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe02315ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200315ce, SUC_lexicon_tags_25 },
    { 0x50315ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a315d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584315de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa78315e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe5c315e2, SUC_lexicon_tags_6 },
    { 0x4e2315e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0a315e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200315ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a315ed, SUC_lexicon_tags_38 },
    { 0x348315ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec315f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc0315f5, SUC_lexicon_tags_0 },
    { 0xcce315f6, SUC_lexicon_tags_37 },
    { 0x272315f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a315fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e31600, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c31607, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce31613, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1523161b, SUC_lexicon_tags_27 },
    { 0xdae3161c, SUC_lexicon_tags_23 },
    { 0x3803161d, SUC_lexicon_tags_39 },
    { 0x42c3161e, SUC_lexicon_tags_12 },
    { 0xc283161f, SUC_lexicon_tags_25 },
    { 0x8f23161d, SUC_lexicon_tags_2 },
    { 0xea231620, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf3831623, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1831629, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6243162e, SUC_lexicon_tags_2 },
    { 0x26a3162f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc031631, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc3e31633, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b63163a, SUC_lexicon_tags_17 },
    { 0xf823163b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5243163e, SUC_lexicon_tags_49 },
    { 0xf5a3163e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd3a31641, SUC_lexicon_tags_27 },
    { 0x9a231642, SUC_lexicon_tags_60 },
    { 0x5ca31643, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5431645, SUC_lexicon_tags_12 },
    { 0x1aa31646, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2ce31648, SUC_lexicon_tags_15 },
    { 0xc9c31649, SUC_lexicon_tags_17 },
    { 0x39e3164a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5631650, SUC_lexicon_tags_15 },
    { 0x35631651, SUC_lexicon_tags_15 },
    { 0xd2831652, SUC_lexicon_tags_25 },
    { 0x67431653, SUC_lexicon_tags_17 },
    { 0x3f631654, SUC_lexicon_tags_30 },
    { 0x35431651, SUC_lexicon_tags_30 },
    { 0x63e31653, SUC_lexicon_tags_341 },
    { 0x45631657, SUC_lexicon_tags_62 },
    { 0x89e31651, SUC_lexicon_tags_7 },
    { 0x82631657, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc163165f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee031662, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f631666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7723166b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c3166e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e31671, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce31674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8783167a, SUC_lexicon_tags_23 },
    { 0x6f83167b, SUC_lexicon_tags_37 },
    { 0x3603167c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce31680, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a31686, SUC_lexicon_tags_15 },
    { 0xb5a31687, SUC_lexicon_tags_12 },
    { 0xe0231688, SUC_lexicon_tags_23 },
    { 0x4d631689, SUC_lexicon_tags_7 },
    { 0xd323168a, SUC_lexicon_tags_23 },
    { 0x1fe3168b, SUC_lexicon_tags_2 },
    { 0x4b43168a, SUC_lexicon_tags_12 },
    { 0xc9a3168d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea3168f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e31693, SUC_lexicon_tags_35 },
    { 0x29831693, SUC_lexicon_tags_39 },
    { 0xd9831695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4943169a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb583169c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe543169e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7e316a0, SUC_lexicon_tags_7 },
    { 0x188316a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8a316a3, SUC_lexicon_tags_28 },
    { 0xc48316a4, SUC_lexicon_tags_11 },
    { 0xd7e316a4, SUC_lexicon_tags_46 },
    { 0xebc316a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4316a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x33e316ab, SUC_lexicon_tags_27 },
    { 0x72c316ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8316b0, SUC_lexicon_tags_37 },
    { 0x1a0316b1, SUC_lexicon_tags_15 },
    { 0x7fa316b2, SUC_lexicon_tags_12 },
    { 0x2bc316b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe22316b5, SUC_lexicon_tags_43 },
    { 0x8a0316b6, SUC_lexicon_tags_65 },
    { 0xf5c316b7, SUC_lexicon_tags_47 },
    { 0x6de316b7, SUC_lexicon_tags_692 },
    { 0x6c2316b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c316bf, SUC_lexicon_tags_33 },
    { 0x170316c0, SUC_lexicon_tags_63 },
    { 0x760316c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a316c4, SUC_lexicon_tags_38 },
    { 0x6a2316c5, SUC_lexicon_tags_53 },
    { 0x14316c6, SUC_lexicon_tags_10 },
    { 0x946316c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5a316c9, SUC_lexicon_tags_25 },
    { 0x9ae316ca, SUC_lexicon_tags_67 },
    { 0xffc316cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4316cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a316d2, SUC_lexicon_tags_39 },
    { 0x9ba316d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x408316d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4316da, SUC_lexicon_tags_25 },
    { 0x2d2316da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc316de, SUC_lexicon_tags_29 },
    { 0xbbe316df, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf0c316e1, SUC_lexicon_tags_45 },
    { 0x746316e1, SUC_lexicon_tags_12 },
    { 0xbb6316e3, SUC_lexicon_tags_49 },
    { 0x738316e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3d8316e6, SUC_lexicon_tags_12 },
    { 0x5fc316e7, SUC_lexicon_tags_66 },
    { 0xa62316e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e316eb, SUC_lexicon_tags_0 },
    { 0x764316ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8316ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a316f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6316f8, SUC_lexicon_tags_77 },
    { 0x79a316f9, SUC_lexicon_tags_37 },
    { 0xd70316f9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x394316fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbe316fe, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1431702, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce031705, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x76831707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03170d, SUC_lexicon_tags_12 },
    { 0xa5e3170e, SUC_lexicon_tags_47 },
    { 0x85a3170e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de31714, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd631716, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc31719, SUC_lexicon_tags_0 },
    { 0xb9c3171a, SUC_lexicon_tags_27 },
    { 0x2a3171b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6383171e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c31725, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f631727, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0831729, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc31731, SUC_lexicon_tags_16 },
    { 0xf9231732, SUC_lexicon_tags_25 },
    { 0x63631731, SUC_lexicon_tags_2 },
    { 0x8b431734, SUC_lexicon_tags_11 },
    { 0xc7c31735, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52231737, SUC_lexicon_tags_407 },
    { 0x7c831737, SUC_lexicon_tags_7 },
    { 0xaba31739, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5231740, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e31742, SUC_lexicon_tags_11 },
    { 0x76031743, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3431747, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7443174a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee31750, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa831752, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x90831755, SUC_lexicon_tags_57 },
    { 0x54831756, SUC_lexicon_tags_47 },
    { 0x54031755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38431759, SUC_lexicon_tags_1 },
    { 0x2903175a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5a3175c, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xe4e3175e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9631761, SUC_lexicon_tags_7 },
    { 0x4ce31762, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b631765, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a31768, SUC_lexicon_tags_25 },
    { 0x5b431769, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa4c3176b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e43176e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c31773, SUC_lexicon_tags_47 },
    { 0x36431773, SUC_lexicon_tags_16 },
    { 0xd6231775, SUC_lexicon_tags_43 },
    { 0xcee31775, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc31779, SUC_lexicon_tags_68 },
    { 0xe2831779, SUC_lexicon_tags_1 },
    { 0x38c3177a, SUC_lexicon_tags_43 },
    { 0xa7431779, SUC_lexicon_tags_65 },
    { 0xd383177d, SUC_lexicon_tags_60 },
    { 0x5043177d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae31781, SUC_lexicon_tags_23 },
    { 0xc8e31781, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e83178d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a31794, SUC_lexicon_tags_37 },
    { 0xf0a31795, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c031798, SUC_lexicon_tags_7 },
    { 0x44231798, SUC_lexicon_tags_1 },
    { 0x5ee31798, SUC_lexicon_tags_30 },
    { 0xa263179a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d63179d, SUC_lexicon_tags_49 },
    { 0xde03179e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x782317a1, SUC_lexicon_tags_17 },
    { 0xf9c317a1, SUC_lexicon_tags_17 },
    { 0xb60317a3, SUC_lexicon_tags_31 },
    { 0xbd8317a4, SUC_lexicon_tags_15 },
    { 0x182317a5, SUC_lexicon_tags_11 },
    { 0x9bc317a5, SUC_lexicon_tags_43 },
    { 0xe06317a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c317ab, SUC_lexicon_tags_1 },
    { 0x44e317ac, SUC_lexicon_tags_25 },
    { 0x498317ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a317b0, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa317b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c317b6, SUC_lexicon_tags_38 },
    { 0xe94317b6, SUC_lexicon_tags_11 },
    { 0x848317b8, SUC_lexicon_tags_2 },
    { 0x830317b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe317bc, SUC_lexicon_tags_30 },
    { 0xb12317bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e317c4, SUC_lexicon_tags_17 },
    { 0x6ba317c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64317ca, SUC_lexicon_tags_49 },
    { 0x57c317ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60e317cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e4317cf, SUC_lexicon_tags_37 },
    { 0xcc6317d0, SUC_lexicon_tags_33 },
    { 0xf66317cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ca317d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f0317d5, SUC_lexicon_tags_25 },
    { 0x7e0317d5, SUC_lexicon_tags_25 },
    { 0x910317d7, SUC_lexicon_tags_38 },
    { 0x1ec317d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe317e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78317e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x690317e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6317e9, SUC_lexicon_tags_48 },
    { 0xdea317e9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76317ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fe317ef, SUC_lexicon_tags_0 },
    { 0x70e317f0, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa317f4, SUC_lexicon_tags_17 },
    { 0xea8317f5, SUC_lexicon_tags_47 },
    { 0x53a317f5, SUC_lexicon_tags_11 },
    { 0xda4317f7, SUC_lexicon_tags_11 },
    { 0x35e317f8, SUC_lexicon_tags_37 },
    { 0xeec317f9, SUC_lexicon_tags_12 },
    { 0x6d0317f7, SUC_lexicon_tags_35 },
    { 0x414317f5, SUC_lexicon_tags_12 },
    { 0x8fc317fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8e317fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91231805, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce31808, SUC_lexicon_tags_7 },
    { 0x47831809, SUC_lexicon_tags_0 },
    { 0x6a83180a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb231811, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa31816, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcc231818, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfca3181a, SUC_lexicon_tags_27 },
    { 0xd563181b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba3181e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d831821, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28e31823, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53431827, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27c31829, SUC_lexicon_tags_19 },
    { 0x31c3182a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a3182d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39431830, SUC_lexicon_tags_12 },
    { 0x35431830, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19831836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5543183e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e31844, SUC_lexicon_tags_15 },
    { 0xd2c31845, SUC_lexicon_tags_15 },
    { 0x93c31845, SUC_lexicon_tags_49 },
    { 0xc0031847, SUC_lexicon_tags_49 },
    { 0x3631848, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x9b23184a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3184d, SUC_lexicon_tags_15 },
    { 0xcc23184e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc31852, SUC_lexicon_tags_39 },
    { 0xbb031853, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce31858, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6443185b, SUC_lexicon_tags_17 },
    { 0xade3185c, SUC_lexicon_tags_15 },
    { 0xdc63185d, SUC_lexicon_tags_37 },
    { 0x49e3185e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x13431860, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e31863, SUC_lexicon_tags_11 },
    { 0x45a31864, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2831866, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a63186c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c431870, SUC_lexicon_tags_1 },
    { 0x59431871, SUC_lexicon_tags_2 },
    { 0x63631872, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11831876, SUC_lexicon_tags_2 },
    { 0x64a31876, SUC_lexicon_tags_38 },
    { 0xb1631878, SUC_lexicon_tags_39 },
    { 0xfbc31879, SUC_lexicon_tags_11 },
    { 0xa3c31878, SUC_lexicon_tags_7 },
    { 0x7bc31878, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a31881, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d031887, SUC_lexicon_tags_54 },
    { 0x7cc31887, SUC_lexicon_tags_23 },
    { 0x99e31888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d63188c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc183188f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x81031891, SUC_lexicon_tags_11 },
    { 0xb4c31892, SUC_lexicon_tags_7 },
    { 0x18431893, SUC_lexicon_tags_25 },
    { 0x9e31894, SUC_lexicon_tags_17 },
    { 0x79031894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60231898, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x76c3189a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd143189c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a3189f, SUC_lexicon_tags_35 },
    { 0x3a6318a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xece318a2, SUC_lexicon_tags_18 },
    { 0x94c318a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77a318a5, SUC_lexicon_tags_14 },
    { 0xbfe318a6, SUC_lexicon_tags_25 },
    { 0xf7e318a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee318aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e318af, SUC_lexicon_tags_29 },
    { 0xc8e318af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf82318b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6318b5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a318b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba318bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946318c4, SUC_lexicon_tags_31 },
    { 0x462318c5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc318ca, SUC_lexicon_tags_55 },
    { 0x552318ca, SUC_lexicon_tags_114 },
    { 0x35a318ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3c4318ce, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x91e318d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c318d3, SUC_lexicon_tags_17 },
    { 0x770318d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08318da, SUC_lexicon_tags_9 },
    { 0x7f2318db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90318df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a318e6, SUC_lexicon_tags_19 },
    { 0x924318e7, SUC_lexicon_tags_14 },
    { 0x20a318e8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea318ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe318f3, SUC_lexicon_tags_2 },
    { 0xbf2318f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x972318f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de318fb, SUC_lexicon_tags_38 },
    { 0x790318fc, SUC_lexicon_tags_47 },
    { 0xe2e318fd, SUC_lexicon_tags_25 },
    { 0xd98318fd, SUC_lexicon_tags_65 },
    { 0x60318ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc31901, SUC_lexicon_tags_8 },
    { 0x82a31901, SUC_lexicon_tags_46 },
    { 0x5c631902, SUC_lexicon_tags_66 },
    { 0x7ee31904, SUC_lexicon_tags_11 },
    { 0xf4431902, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2431908, SUC_lexicon_tags_4 },
    { 0x4ea31909, SUC_lexicon_tags_10 },
    { 0xe2831908, SUC_lexicon_tags_34 },
    { 0xbcc3190b, SUC_lexicon_tags_2 },
    { 0x83c3190c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d83190e, SUC_lexicon_tags_37 },
    { 0xf083190f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4d231911, SUC_lexicon_tags_30 },
    { 0x73a31912, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe183191a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d03191e, SUC_lexicon_tags_11 },
    { 0x31a3191f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38431923, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa6631925, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9e31927, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae83192b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xfe63192d, SUC_lexicon_tags_87 },
    { 0x7e03192d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e031931, SUC_lexicon_tags_33 },
    { 0xc3031932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c31935, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b63193a, SUC_lexicon_tags_12 },
    { 0x8f03193b, SUC_lexicon_tags_63 },
    { 0x45c3193b, SUC_lexicon_tags_11 },
    { 0xb583193d, SUC_lexicon_tags_15 },
    { 0x2963193e, SUC_lexicon_tags_33 },
    { 0xe943193f, SUC_lexicon_tags_14 },
    { 0x40431940, SUC_lexicon_tags_4 },
    { 0x1403193d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa31947, SUC_lexicon_tags_1 },
    { 0xdba31948, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba23194a, SUC_lexicon_tags_12 },
    { 0x843194b, SUC_lexicon_tags_1 },
    { 0x4d43194c, SUC_lexicon_tags_38 },
    { 0x3ec3194d, SUC_lexicon_tags_7 },
    { 0xd1c3194d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2231950, SUC_lexicon_tags_29 },
    { 0x98831951, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa6a31953, SUC_lexicon_tags_17 },
    { 0x76831954, SUC_lexicon_tags_9 },
    { 0xe4631955, SUC_lexicon_tags_14 },
    { 0x89031956, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3c431958, SUC_lexicon_tags_66 },
    { 0x3fa31959, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7583195b, SUC_lexicon_tags_1 },
    { 0xe63195c, SUC_lexicon_tags_49 },
    { 0xe623195d, SUC_lexicon_tags_26 },
    { 0x8083195e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b631962, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x68e31964, SUC_lexicon_tags_7 },
    { 0xa7231964, SUC_lexicon_tags_6 },
    { 0xe6831964, SUC_lexicon_tags_1 },
    { 0xe9a31967, SUC_lexicon_tags_17 },
    { 0xb1e31968, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c23196b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6803196d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb583196f, SUC_lexicon_tags_43 },
    { 0x83831970, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e31974, SUC_lexicon_tags_14 },
    { 0xce231975, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9d231977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd263197a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543197f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7631986, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc3198a, SUC_lexicon_tags_47 },
    { 0x71c3198b, SUC_lexicon_tags_25 },
    { 0x9423198c, SUC_lexicon_tags_16 },
    { 0x14e3198d, SUC_lexicon_tags_7 },
    { 0x9a43198a, SUC_lexicon_tags_37 },
    { 0xd4a3198f, SUC_lexicon_tags_2 },
    { 0xbba3198f, SUC_lexicon_tags_6 },
    { 0x9a43198e, SUC_lexicon_tags_17 },
    { 0x2243198a, SUC_lexicon_tags_61 },
    { 0xbc031993, SUC_lexicon_tags_21 },
    { 0x8e831993, SUC_lexicon_tags_9 },
    { 0xe583198a, SUC_lexicon_tags_33 },
    { 0x79431996, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da3199a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10319a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x854319a2, SUC_lexicon_tags_17 },
    { 0x9c0319a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50319b4, SUC_lexicon_tags_25 },
    { 0x86e319b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0319bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xab2319bf, SUC_lexicon_tags_27 },
    { 0xa58319c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e2319c2, SUC_lexicon_tags_84 },
    { 0x3f6319c2, SUC_lexicon_tags_35 },
    { 0xc38319c4, SUC_lexicon_tags_25 },
    { 0xeea319c5, SUC_lexicon_tags_69 },
    { 0xe46319c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdea319c8, SUC_lexicon_tags_25 },
    { 0x918319c9, SUC_lexicon_tags_27 },
    { 0x9b4319ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc319cf, SUC_lexicon_tags_1 },
    { 0x9c6319d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04319d4, SUC_lexicon_tags_1 },
    { 0xf60319d5, SUC_lexicon_tags_56 },
    { 0x18c319d5, SUC_lexicon_tags_56 },
    { 0xd0319d6, SUC_lexicon_tags_4 },
    { 0x8a2319d5, SUC_lexicon_tags_6 },
    { 0x2da319d8, SUC_lexicon_tags_11 },
    { 0x746319da, SUC_lexicon_tags_8 },
    { 0x38319da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288319e5, SUC_lexicon_tags_15 },
    { 0x66319e5, SUC_lexicon_tags_47 },
    { 0x23e319e7, SUC_lexicon_tags_17 },
    { 0xaae319e5, SUC_lexicon_tags_43 },
    { 0x612319e9, SUC_lexicon_tags_2 },
    { 0xc76319e9, SUC_lexicon_tags_25 },
    { 0x2bc319eb, SUC_lexicon_tags_37 },
    { 0xeac319eb, SUC_lexicon_tags_37 },
    { 0x3a0319eb, SUC_lexicon_tags_1 },
    { 0xf3a319ee, SUC_lexicon_tags_5 },
    { 0x1ce319ef, SUC_lexicon_tags_10 },
    { 0x9b6319ee, SUC_lexicon_tags_30 },
    { 0xf6319f1, SUC_lexicon_tags_151 },
    { 0x280319f2, SUC_lexicon_tags_37 },
    { 0x504319f3, SUC_lexicon_tags_21 },
    { 0xe4a319eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b8319f6, SUC_lexicon_tags_30 },
    { 0xf66319f7, SUC_lexicon_tags_2 },
    { 0xfda319f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a319fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a31a0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60431a0f, SUC_lexicon_tags_1 },
    { 0x91431a0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x14431a12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d231a19, SUC_lexicon_tags_15 },
    { 0xf5e31a19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd1031a1c, SUC_lexicon_tags_45 },
    { 0xea431a1d, SUC_lexicon_tags_45 },
    { 0x6f231a1c, SUC_lexicon_tags_16 },
    { 0x31031a1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94831a21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a31a25, SUC_lexicon_tags_16 },
    { 0x12631a26, SUC_lexicon_tags_65 },
    { 0xb2631a27, SUC_lexicon_tags_25 },
    { 0x82831a28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c31a2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadc31a2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe31a31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e31a35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0831a3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a31a3e, SUC_lexicon_tags_15 },
    { 0x81431a3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e231a41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38831a43, SUC_lexicon_tags_37 },
    { 0xd1631a44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc5c31a46, SUC_lexicon_tags_9 },
    { 0x70831a47, SUC_lexicon_tags_16 },
    { 0xcf031a48, SUC_lexicon_tags_45 },
    { 0x34c31a49, SUC_lexicon_tags_17 },
    { 0x2f431a4a, SUC_lexicon_tags_37 },
    { 0x6ae31a4a, SUC_lexicon_tags_9 },
    { 0x6de31a47, SUC_lexicon_tags_25 },
    { 0x69a31a4a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xeb031a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa31a57, SUC_lexicon_tags_23 },
    { 0xb4c31a58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0031a5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe831a61, SUC_lexicon_tags_25 },
    { 0xfcc31a62, SUC_lexicon_tags_46 },
    { 0x57a31a63, SUC_lexicon_tags_71 },
    { 0x70c31a63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea31a68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbc031a6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76431a6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e31a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c31a76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22831a78, SUC_lexicon_tags_8 },
    { 0xb6231a79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e31a7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e031a85, SUC_lexicon_tags_92 },
    { 0xa7e31a85, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8031a8a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da31a8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a631a91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c031a93, SUC_lexicon_tags_1 },
    { 0xbe31a94, SUC_lexicon_tags_7 },
    { 0x3c431a95, SUC_lexicon_tags_1 },
    { 0xd9c31a96, SUC_lexicon_tags_17 },
    { 0x9e631a97, SUC_lexicon_tags_17 },
    { 0xe1431a94, SUC_lexicon_tags_27 },
    { 0x7a31a99, SUC_lexicon_tags_2 },
    { 0x95c31a9a, SUC_lexicon_tags_23 },
    { 0x40831a9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9631a9e, SUC_lexicon_tags_11 },
    { 0xe6c31a9f, SUC_lexicon_tags_29 },
    { 0xa2231aa0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75231aa2, SUC_lexicon_tags_12 },
    { 0x4aa31aa3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a31aa6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6831aaa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xec431aad, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x41c31aaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c31ab7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a31abb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x66c31abd, SUC_lexicon_tags_27 },
    { 0xaa831abd, SUC_lexicon_tags_0 },
    { 0x9431abe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33231ac3, SUC_lexicon_tags_27 },
    { 0xd5a31ac3, SUC_lexicon_tags_69 },
    { 0xa1031ac5, SUC_lexicon_tags_60 },
    { 0x72831ac6, SUC_lexicon_tags_35 },
    { 0x96231ac5, SUC_lexicon_tags_33 },
    { 0xff231ac8, SUC_lexicon_tags_32 },
    { 0x99e31ac9, SUC_lexicon_tags_30 },
    { 0x7e31ac7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f431ace, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78831ad1, SUC_lexicon_tags_27 },
    { 0x61031ad2, SUC_lexicon_tags_8 },
    { 0xe3231ad2, SUC_lexicon_tags_28 },
    { 0xf8431ad4, SUC_lexicon_tags_11 },
    { 0x25c31ad5, SUC_lexicon_tags_37 },
    { 0xb2831ad3, SUC_lexicon_tags_27 },
    { 0x38831ad7, SUC_lexicon_tags_45 },
    { 0xbe631ad5, SUC_lexicon_tags_11 },
    { 0xf0431ad9, SUC_lexicon_tags_12 },
    { 0x95631ad4, SUC_lexicon_tags_43 },
    { 0x3e631adb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7fa31add, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e231ae1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x70031ae4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc2831ae6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9c431ae8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2231aeb, SUC_lexicon_tags_28 },
    { 0x2c231aec, SUC_lexicon_tags_87 },
    { 0x1c231aed, SUC_lexicon_tags_37 },
    { 0x89231aed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a31af1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe431af9, SUC_lexicon_tags_34 },
    { 0xdc031af9, SUC_lexicon_tags_29 },
    { 0x94831afb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c31afe, SUC_lexicon_tags_25 },
    { 0xe4c31aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a031b05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78231b0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea231b10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42231b12, SUC_lexicon_tags_12 },
    { 0xf7431b13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf5431b15, SUC_lexicon_tags_17 },
    { 0xc9431b15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89431b21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46431b2b, SUC_lexicon_tags_8 },
    { 0xe3e31b2c, SUC_lexicon_tags_38 },
    { 0xf2a31b2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa31b34, SUC_lexicon_tags_12 },
    { 0x5ce31b35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15231b37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ae31b39, SUC_lexicon_tags_79 },
    { 0x40031b3a, SUC_lexicon_tags_6 },
    { 0xcc631b3b, SUC_lexicon_tags_12 },
    { 0x96c31b3c, SUC_lexicon_tags_0 },
    { 0xbc631b3d, SUC_lexicon_tags_1 },
    { 0xcf831b3c, SUC_lexicon_tags_296 },
    { 0x90831b3d, SUC_lexicon_tags_1 },
    { 0xc9831b40, SUC_lexicon_tags_8 },
    { 0x1e431b3e, SUC_lexicon_tags_85 },
    { 0x74231b42, SUC_lexicon_tags_27 },
    { 0xa3031b43, SUC_lexicon_tags_25 },
    { 0x83a31b43, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8ac31b46, SUC_lexicon_tags_1 },
    { 0xc1c31b47, SUC_lexicon_tags_3 },
    { 0x14631b47, SUC_lexicon_tags_30 },
    { 0x7b831b49, SUC_lexicon_tags_2 },
    { 0xf0031b4a, SUC_lexicon_tags_62 },
    { 0x91031b46, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8f631b4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ae31b4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fe31b51, SUC_lexicon_tags_25 },
    { 0xc8431b52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc31b55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2831b58, SUC_lexicon_tags_11 },
    { 0x83a31b59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd6031b5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94831b5e, SUC_lexicon_tags_41 },
    { 0xd5431b5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac31b66, SUC_lexicon_tags_63 },
    { 0x33231b66, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8831b69, SUC_lexicon_tags_38 },
    { 0x3c31b6a, SUC_lexicon_tags_5 },
    { 0xbfe31b6a, SUC_lexicon_tags_7 },
    { 0xf2831b6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c231b6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52831b77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27031b7f, SUC_lexicon_tags_11 },
    { 0x6431b80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98231b83, SUC_lexicon_tags_38 },
    { 0x5ee31b83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7431b86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafa31b88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa31b8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c831b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc231b93, SUC_lexicon_tags_6 },
    { 0x78c31b94, SUC_lexicon_tags_55 },
    { 0x35e31b95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e31b98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x48831b9b, SUC_lexicon_tags_12 },
    { 0x75031b9b, SUC_lexicon_tags_17 },
    { 0x4ee31b9d, SUC_lexicon_tags_37 },
    { 0x73031b9e, SUC_lexicon_tags_10 },
    { 0x6f031b9c, SUC_lexicon_tags_0 },
    { 0xaf831ba0, SUC_lexicon_tags_0 },
    { 0x9ac31ba1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b231ba5, SUC_lexicon_tags_2 },
    { 0x93231ba6, SUC_lexicon_tags_2 },
    { 0x48a31ba7, SUC_lexicon_tags_26 },
    { 0x11231ba7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9031bab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4631baf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4431bb7, SUC_lexicon_tags_46 },
    { 0x31431bb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23631bba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d031bbf, SUC_lexicon_tags_21 },
    { 0xe2c31bc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c31bc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c631bc8, SUC_lexicon_tags_37 },
    { 0x22e31bc9, SUC_lexicon_tags_19 },
    { 0x80031bca, SUC_lexicon_tags_71 },
    { 0x89c31bca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdc031bcd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xab031bd0, SUC_lexicon_tags_18 },
    { 0x60c31bd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62231bd3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9031bd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2831bdc, SUC_lexicon_tags_25 },
    { 0xc2631bdd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdfa31bdf, SUC_lexicon_tags_12 },
    { 0x5be31be0, SUC_lexicon_tags_83 },
    { 0xcb631be0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a31be9, SUC_lexicon_tags_1 },
    { 0xee31bea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6e31bec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba31bf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8631bf3, SUC_lexicon_tags_12 },
    { 0x11c31bf4, SUC_lexicon_tags_17 },
    { 0xf4831bf5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a831bfe, SUC_lexicon_tags_33 },
    { 0x54231bfe, SUC_lexicon_tags_4 },
    { 0xaf631bff, SUC_lexicon_tags_17 },
    { 0x63831c01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc831c05, SUC_lexicon_tags_17 },
    { 0x1b831c06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdec31c08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60231c0c, SUC_lexicon_tags_19 },
    { 0x89631c0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4431c10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15831c16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc31c19, SUC_lexicon_tags_49 },
    { 0x6ae31c1a, SUC_lexicon_tags_45 },
    { 0x75631c1b, SUC_lexicon_tags_2 },
    { 0xb1831c1b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a31c1f, SUC_lexicon_tags_1 },
    { 0x73231c20, SUC_lexicon_tags_31 },
    { 0xdb831c20, SUC_lexicon_tags_0 },
    { 0x5b431c20, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e31c26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x84031c28, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf231c2f, SUC_lexicon_tags_12 },
    { 0xeea31c30, SUC_lexicon_tags_22 },
    { 0xc4231c31, SUC_lexicon_tags_27 },
    { 0x79631c30, SUC_lexicon_tags_23 },
    { 0xfb831c33, SUC_lexicon_tags_1 },
    { 0xe4a31c33, SUC_lexicon_tags_8 },
    { 0xdac31c35, SUC_lexicon_tags_11 },
    { 0x87631c30, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d031c3c, SUC_lexicon_tags_8 },
    { 0x9aa31c3d, SUC_lexicon_tags_936 },
    { 0x36431c3e, SUC_lexicon_tags_14 },
    { 0x9d631c3f, SUC_lexicon_tags_25 },
    { 0xf2031c40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1431c42, SUC_lexicon_tags_25 },
    { 0xfa31c43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb231c45, SUC_lexicon_tags_5 },
    { 0x7e431c45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d431c48, SUC_lexicon_tags_1 },
    { 0xdb031c49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3031c4c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x57831c4e, SUC_lexicon_tags_17 },
    { 0x4a031c4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3231c52, SUC_lexicon_tags_91 },
    { 0x6ee31c52, SUC_lexicon_tags_9 },
    { 0xd0831c52, SUC_lexicon_tags_55 },
    { 0xed431c55, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34231c59, SUC_lexicon_tags_60 },
    { 0x74431c5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46a31c5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30431c5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac31c62, SUC_lexicon_tags_2 },
    { 0x38e31c63, SUC_lexicon_tags_13 },
    { 0xbc031c64, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x28831c66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c31c6a, SUC_lexicon_tags_30 },
    { 0xf0231c6b, SUC_lexicon_tags_19 },
    { 0xba31c6b, SUC_lexicon_tags_49 },
    { 0x1e31c6d, SUC_lexicon_tags_37 },
    { 0x1c431c6d, SUC_lexicon_tags_11 },
    { 0xaaa31c6f, SUC_lexicon_tags_1 },
    { 0xb7231c6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea031c73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e31c76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x27031c78, SUC_lexicon_tags_37 },
    { 0x8431c78, SUC_lexicon_tags_61 },
    { 0xffa31c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92a31c7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f631c81, SUC_lexicon_tags_14 },
    { 0x2fa31c82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xff631c85, SUC_lexicon_tags_49 },
    { 0x4031c86, SUC_lexicon_tags_5 },
    { 0x7ac31c87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcda31c89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d031c90, SUC_lexicon_tags_31 },
    { 0x15c31c91, SUC_lexicon_tags_27 },
    { 0x53431c92, SUC_lexicon_tags_33 },
    { 0xb8831c92, SUC_lexicon_tags_47 },
    { 0xfa231c91, SUC_lexicon_tags_17 },
    { 0x6ea31c93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e231c97, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x71431c9a, SUC_lexicon_tags_33 },
    { 0x6ba31c9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2031c9e, SUC_lexicon_tags_8 },
    { 0x19831c9f, SUC_lexicon_tags_65 },
    { 0x2fc31c9e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x22431ca2, SUC_lexicon_tags_37 },
    { 0x19631ca3, SUC_lexicon_tags_108 },
    { 0x80c31ca3, SUC_lexicon_tags_2 },
    { 0xbd631ca5, SUC_lexicon_tags_38 },
    { 0x8fe31ca6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5d831ca8, SUC_lexicon_tags_77 },
    { 0x9f431ca9, SUC_lexicon_tags_25 },
    { 0xb0831ca9, SUC_lexicon_tags_7 },
    { 0x6b631cab, SUC_lexicon_tags_15 },
    { 0x4e831cac, SUC_lexicon_tags_37 },
    { 0xa0e31cad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe8c31caf, SUC_lexicon_tags_34 },
    { 0xdd431caf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe0231cb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fa31cb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94831cb6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53831cbf, SUC_lexicon_tags_25 },
    { 0xfc631cc0, SUC_lexicon_tags_2 },
    { 0x39c31cc0, SUC_lexicon_tags_12 },
    { 0x86c31cc2, SUC_lexicon_tags_7 },
    { 0x3a431cc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c31cc7, SUC_lexicon_tags_7 },
    { 0x63631cc8, SUC_lexicon_tags_7 },
    { 0x45a31cc9, SUC_lexicon_tags_11 },
    { 0xcb431cc8, SUC_lexicon_tags_47 },
    { 0xe7c31ccb, SUC_lexicon_tags_2 },
    { 0xcfa31cc9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7231cd0, SUC_lexicon_tags_7 },
    { 0x7bc31cd1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a31cd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd831cd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca831cde, SUC_lexicon_tags_17 },
    { 0x1e631cdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29431ce6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x85431ce9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e31cec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6031cee, SUC_lexicon_tags_32 },
    { 0xd2c31cef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c31cf5, SUC_lexicon_tags_25 },
    { 0x41831cf5, SUC_lexicon_tags_46 },
    { 0xaf431cf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a31cfc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd0231cfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c431d03, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32631d09, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xafe31d0b, SUC_lexicon_tags_33 },
    { 0xb5831d0c, SUC_lexicon_tags_12 },
    { 0xebc31d0b, SUC_lexicon_tags_66 },
    { 0x8c831d0d, SUC_lexicon_tags_10 },
    { 0x5ea31d0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a431d13, SUC_lexicon_tags_43 },
    { 0x27231d13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb631d19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab031d1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23e31d1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9031d20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9231d23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95231d27, SUC_lexicon_tags_68 },
    { 0x44231d28, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c31d2d, SUC_lexicon_tags_4 },
    { 0x5a831d2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0431d3d, SUC_lexicon_tags_7 },
    { 0xdc031d3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x81631d41, SUC_lexicon_tags_18 },
    { 0x16631d42, SUC_lexicon_tags_34 },
    { 0x7a431d43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58031d49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97431d4b, SUC_lexicon_tags_25 },
    { 0xed031d4b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55031d51, SUC_lexicon_tags_27 },
    { 0x35031d52, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf3a31d54, SUC_lexicon_tags_25 },
    { 0x41a31d54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ae31d57, SUC_lexicon_tags_25 },
    { 0x51431d58, SUC_lexicon_tags_23 },
    { 0xcc431d59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27c31d5b, SUC_lexicon_tags_17 },
    { 0xed031d5c, SUC_lexicon_tags_15 },
    { 0xc4231d5c, SUC_lexicon_tags_7 },
    { 0xabe31d5e, SUC_lexicon_tags_11 },
    { 0x9ac31d5e, SUC_lexicon_tags_11 },
    { 0x96631d60, SUC_lexicon_tags_27 },
    { 0x7a631d5f, SUC_lexicon_tags_7 },
    { 0x95831d60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf831d66, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92031d6c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6831d6e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0831d73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74631d75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6031d77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4e31d79, SUC_lexicon_tags_87 },
    { 0x99831d7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33431d7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73a31d7e, SUC_lexicon_tags_11 },
    { 0x29031d7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4431d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83831d87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a31d8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ca31d8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c31d91, SUC_lexicon_tags_6 },
    { 0x8c031d92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2631d94, SUC_lexicon_tags_39 },
    { 0x2d831d94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a31d9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a31da0, SUC_lexicon_tags_68 },
    { 0x21431da1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc031da3, SUC_lexicon_tags_11 },
    { 0x231da3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30231daa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc031db8, SUC_lexicon_tags_160 },
    { 0x9d831db8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7831dbc, SUC_lexicon_tags_43 },
    { 0x83a31dbd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32231dc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f831dc5, SUC_lexicon_tags_37 },
    { 0xcc31dc5, SUC_lexicon_tags_15 },
    { 0x53431dc6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e831dcd, SUC_lexicon_tags_12 },
    { 0xe0031dcd, SUC_lexicon_tags_17 },
    { 0x84a31dcf, SUC_lexicon_tags_46 },
    { 0xb4c31dcf, SUC_lexicon_tags_37 },
    { 0x5fa31dcf, SUC_lexicon_tags_12 },
    { 0xb8c31dd2, SUC_lexicon_tags_15 },
    { 0xb6831dd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a31ddb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x56431dde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae31de2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4c31de4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f231de6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9031de8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ec31dea, SUC_lexicon_tags_9 },
    { 0xd7a31deb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c631dee, SUC_lexicon_tags_32 },
    { 0x21a31dee, SUC_lexicon_tags_12 },
    { 0x32231df0, SUC_lexicon_tags_55 },
    { 0xabe31dee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9fe31df3, SUC_lexicon_tags_7 },
    { 0x9ea31df4, SUC_lexicon_tags_49 },
    { 0x88a31df4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e31df8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c031dfb, SUC_lexicon_tags_59 },
    { 0x98231dfc, SUC_lexicon_tags_27 },
    { 0xc3431dfd, SUC_lexicon_tags_0 },
    { 0x7b431dfd, SUC_lexicon_tags_11 },
    { 0x54031dff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb831e01, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e31e05, SUC_lexicon_tags_37 },
    { 0x44e31e05, SUC_lexicon_tags_34 },
    { 0xed231e07, SUC_lexicon_tags_0 },
    { 0x12831e08, SUC_lexicon_tags_74 },
    { 0xb9831e08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a31e0d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x3f431e0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29631e15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2031e18, SUC_lexicon_tags_37 },
    { 0xc1a31e19, SUC_lexicon_tags_15 },
    { 0x10631e1a, SUC_lexicon_tags_65 },
    { 0xb1631e19, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc4a31e1d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7831e20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x70631e22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b831e25, SUC_lexicon_tags_11 },
    { 0x7fa31e26, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x13031e28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27831e2a, SUC_lexicon_tags_60 },
    { 0xb0031e2b, SUC_lexicon_tags_7 },
    { 0x48831e2c, SUC_lexicon_tags_60 },
    { 0xba631e2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa1231e2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11831e37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6031e3b, SUC_lexicon_tags_113 },
    { 0xe0631e3c, SUC_lexicon_tags_65 },
    { 0xe9831e3d, SUC_lexicon_tags_19 },
    { 0x9d631e3e, SUC_lexicon_tags_34 },
    { 0xb1c31e3d, SUC_lexicon_tags_17 },
    { 0x65431e3d, SUC_lexicon_tags_7 },
    { 0x47231e41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaae31e43, SUC_lexicon_tags_28 },
    { 0x6bc31e44, SUC_lexicon_tags_37 },
    { 0xe6431e45, SUC_lexicon_tags_14 },
    { 0x46c31e44, SUC_lexicon_tags_19 },
    { 0xee31e47, SUC_lexicon_tags_29 },
    { 0x8ea31e48, SUC_lexicon_tags_17 },
    { 0xfa31e48, SUC_lexicon_tags_17 },
    { 0xad231e4a, SUC_lexicon_tags_7 },
    { 0x5ae31e48, SUC_lexicon_tags_38 },
    { 0x73e31e48, SUC_lexicon_tags_0 },
    { 0x3b831e4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae31e53, SUC_lexicon_tags_11 },
    { 0x30231e53, SUC_lexicon_tags_7 },
    { 0x45031e55, SUC_lexicon_tags_10 },
    { 0x32631e53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2031e5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0431e5d, SUC_lexicon_tags_29 },
    { 0x63031e5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbdc31e60, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x26431e62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3831e68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea31e74, SUC_lexicon_tags_23 },
    { 0x86631e75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c431e77, SUC_lexicon_tags_47 },
    { 0x35c31e77, SUC_lexicon_tags_11 },
    { 0xf2a31e77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9031e7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c231e84, SUC_lexicon_tags_15 },
    { 0x4d431e85, SUC_lexicon_tags_26 },
    { 0xe9231e85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25031e88, SUC_lexicon_tags_27 },
    { 0xeb631e89, SUC_lexicon_tags_40 },
    { 0x1a831e88, SUC_lexicon_tags_29 },
    { 0x7b031e89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e31e8f, SUC_lexicon_tags_7 },
    { 0x1a831e8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xce31e92, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x18031e94, SUC_lexicon_tags_17 },
    { 0xaf031e95, SUC_lexicon_tags_45 },
    { 0x23231e95, SUC_lexicon_tags_23 },
    { 0xc7e31e97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d431e9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98231ea0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea31eab, SUC_lexicon_tags_12 },
    { 0x2c31eac, SUC_lexicon_tags_7 },
    { 0xc2031eab, SUC_lexicon_tags_9 },
    { 0x49831eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91831eb1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd5c31eb3, SUC_lexicon_tags_56 },
    { 0xfc831eb4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e31eb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac031ebd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x83831ec0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4631ec2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c831ec4, SUC_lexicon_tags_17 },
    { 0xf3c31ec5, SUC_lexicon_tags_30 },
    { 0x54c31ec5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x98431ec9, SUC_lexicon_tags_66 },
    { 0x7ee31eca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95231ed3, SUC_lexicon_tags_14 },
    { 0xef031ed3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d231eda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a031edf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7431ee1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9de31ee3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a31ee6, SUC_lexicon_tags_25 },
    { 0xae831ee6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c431eef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8031ef2, SUC_lexicon_tags_8 },
    { 0xf9431ef3, SUC_lexicon_tags_7 },
    { 0x3cc31ef2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd231efc, SUC_lexicon_tags_35 },
    { 0x5ae31efd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc431f04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8831f07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcac31f09, SUC_lexicon_tags_34 },
    { 0xd5831f0a, SUC_lexicon_tags_12 },
    { 0xca031f0a, SUC_lexicon_tags_15 },
    { 0xbc431f0c, SUC_lexicon_tags_28 },
    { 0x20e31f0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b431f0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca31f12, SUC_lexicon_tags_2 },
    { 0x5f431f13, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e31f18, SUC_lexicon_tags_37 },
    { 0x63a31f19, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19831f1f, SUC_lexicon_tags_2 },
    { 0xcd831f20, SUC_lexicon_tags_12 },
    { 0xc5831f21, SUC_lexicon_tags_1 },
    { 0x1a831f22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe0231f24, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2431f2c, SUC_lexicon_tags_81 },
    { 0x9c431f2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc0e31f2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee31f34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabc31f36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c831f38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34431f3e, SUC_lexicon_tags_37 },
    { 0x6da31f3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce31f46, SUC_lexicon_tags_15 },
    { 0x57c31f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14031f4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c431f50, SUC_lexicon_tags_11 },
    { 0x1be31f51, SUC_lexicon_tags_7 },
    { 0xcfc31f52, SUC_lexicon_tags_1 },
    { 0xc5031f53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36431f59, SUC_lexicon_tags_15 },
    { 0xaf631f5a, SUC_lexicon_tags_43 },
    { 0xc8431f5b, SUC_lexicon_tags_47 },
    { 0xb9431f5b, SUC_lexicon_tags_17 },
    { 0x2d431f59, SUC_lexicon_tags_28 },
    { 0xe4e31f5e, SUC_lexicon_tags_2 },
    { 0x81c31f5f, SUC_lexicon_tags_17 },
    { 0x81c31f59, SUC_lexicon_tags_2 },
    { 0xb8431f60, SUC_lexicon_tags_12 },
    { 0xc3031f5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf631f67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x28231f69, SUC_lexicon_tags_17 },
    { 0x74e31f6a, SUC_lexicon_tags_31 },
    { 0xb8431f6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c31f6e, SUC_lexicon_tags_70 },
    { 0xb7431f6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e431f71, SUC_lexicon_tags_17 },
    { 0xdf631f71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a631f75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37831f7a, SUC_lexicon_tags_25 },
    { 0x1f231f7b, SUC_lexicon_tags_11 },
    { 0xca231f7b, SUC_lexicon_tags_11 },
    { 0x55631f7b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3231f80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3431f87, SUC_lexicon_tags_28 },
    { 0x79631f88, SUC_lexicon_tags_37 },
    { 0x43231f88, SUC_lexicon_tags_0 },
    { 0x39231f89, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdd631f8c, SUC_lexicon_tags_7 },
    { 0xe5631f8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43231f91, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa4431f93, SUC_lexicon_tags_17 },
    { 0x93831f93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a31f98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13031f9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc31fa3, SUC_lexicon_tags_37 },
    { 0xd6031fa4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26231fa6, SUC_lexicon_tags_7 },
    { 0x6ee31fa7, SUC_lexicon_tags_17 },
    { 0xa5231fa8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbd631faa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a31fad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71431fb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21631fb5, SUC_lexicon_tags_15 },
    { 0x5f831fb6, SUC_lexicon_tags_47 },
    { 0x6d231fb7, SUC_lexicon_tags_8 },
    { 0x95031fb8, SUC_lexicon_tags_4 },
    { 0xe31fb9, SUC_lexicon_tags_39 },
    { 0x35a31fba, SUC_lexicon_tags_77 },
    { 0x1c631fb6, SUC_lexicon_tags_60 },
    { 0x2f831fbc, SUC_lexicon_tags_136 },
    { 0x63231fbc, SUC_lexicon_tags_7 },
    { 0x19831fbe, SUC_lexicon_tags_21 },
    { 0x7ea31fbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a31fc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc031fc5, SUC_lexicon_tags_38 },
    { 0xb3231fc6, SUC_lexicon_tags_46 },
    { 0x3ac31fc5, SUC_lexicon_tags_12 },
    { 0x5f831fc6, SUC_lexicon_tags_45 },
    { 0xc6831fc9, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46031fcf, SUC_lexicon_tags_49 },
    { 0xea631fcf, SUC_lexicon_tags_2 },
    { 0x61231fd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada31fd5, SUC_lexicon_tags_0 },
    { 0x77e31fd5, SUC_lexicon_tags_27 },
    { 0xb2e31fd6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf231fda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde31fdc, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf631fe3, SUC_lexicon_tags_17 },
    { 0xe3431fe3, SUC_lexicon_tags_1 },
    { 0x17031fe4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c231fe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7a31fea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e31ff2, SUC_lexicon_tags_6 },
    { 0xf1c31ff3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf031ff5, SUC_lexicon_tags_45 },
    { 0x9e31ff5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b631ff8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xad231ffb, SUC_lexicon_tags_11 },
    { 0x99a31ffb, SUC_lexicon_tags_10 },
    { 0x20831ffd, SUC_lexicon_tags_17 },
    { 0xc3c31ffd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde232003, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f832005, SUC_lexicon_tags_38 },
    { 0xf5432006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c32009, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9232014, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8232016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a32019, SUC_lexicon_tags_17 },
    { 0x3183201a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23432020, SUC_lexicon_tags_27 },
    { 0xaea32021, SUC_lexicon_tags_49 },
    { 0xe9232022, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x56a32024, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a32029, SUC_lexicon_tags_7 },
    { 0x2f432029, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf63202e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1ac32030, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e32033, SUC_lexicon_tags_21 },
    { 0x9d632034, SUC_lexicon_tags_17 },
    { 0x4bc32034, SUC_lexicon_tags_12 },
    { 0x69a32036, SUC_lexicon_tags_45 },
    { 0x75e32036, SUC_lexicon_tags_37 },
    { 0xf6032038, SUC_lexicon_tags_47 },
    { 0x11632039, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e03203e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbc32040, SUC_lexicon_tags_56 },
    { 0x4d232040, SUC_lexicon_tags_15 },
    { 0x42632042, SUC_lexicon_tags_12 },
    { 0x1c232042, SUC_lexicon_tags_12 },
    { 0x6ba32043, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca03204a, SUC_lexicon_tags_9 },
    { 0xe043204b, SUC_lexicon_tags_1 },
    { 0x3ba3204c, SUC_lexicon_tags_14 },
    { 0xad43204d, SUC_lexicon_tags_37 },
    { 0xf383204a, SUC_lexicon_tags_11 },
    { 0x68a3204f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec32054, SUC_lexicon_tags_23 },
    { 0xeec32054, SUC_lexicon_tags_12 },
    { 0x80032056, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b63205a, SUC_lexicon_tags_2 },
    { 0xa5c3205a, SUC_lexicon_tags_25 },
    { 0x5ca3205b, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f232060, SUC_lexicon_tags_43 },
    { 0x4a032061, SUC_lexicon_tags_15 },
    { 0xe1432062, SUC_lexicon_tags_46 },
    { 0x41432062, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca32069, SUC_lexicon_tags_7 },
    { 0xf7e32069, SUC_lexicon_tags_25 },
    { 0x2b23206b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1032072, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a32076, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68a32078, SUC_lexicon_tags_37 },
    { 0xfe32079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9403207b, SUC_lexicon_tags_2 },
    { 0x1543207c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4063207f, SUC_lexicon_tags_37 },
    { 0x7e632080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35832086, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63208d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1623208f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda832093, SUC_lexicon_tags_37 },
    { 0xc8632093, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e32097, SUC_lexicon_tags_2 },
    { 0x16832098, SUC_lexicon_tags_12 },
    { 0xa0832099, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd263209b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354320a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x90320a6, SUC_lexicon_tags_55 },
    { 0x196320a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30320aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808320ae, SUC_lexicon_tags_1 },
    { 0x9c2320af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0320b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42320b9, SUC_lexicon_tags_15 },
    { 0x2ba320ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54320bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86c320c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae320c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83c320ca, SUC_lexicon_tags_28 },
    { 0xf5e320cb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e320cf, SUC_lexicon_tags_7 },
    { 0x45c320d0, SUC_lexicon_tags_49 },
    { 0xfbe320d1, SUC_lexicon_tags_27 },
    { 0x552320cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f0320d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338320db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14320de, SUC_lexicon_tags_45 },
    { 0xee2320df, SUC_lexicon_tags_28 },
    { 0x1ee320de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7f6320e2, SUC_lexicon_tags_43 },
    { 0x204320e3, SUC_lexicon_tags_37 },
    { 0xd98320e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c320e7, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66320ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x480320ec, SUC_lexicon_tags_17 },
    { 0x2d8320ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342320f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8320f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc320fe, SUC_lexicon_tags_46 },
    { 0x1a8320ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96432101, SUC_lexicon_tags_64 },
    { 0x4032101, SUC_lexicon_tags_17 },
    { 0x6d432102, SUC_lexicon_tags_28 },
    { 0x1ae32104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a3210b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa32112, SUC_lexicon_tags_25 },
    { 0x7632113, SUC_lexicon_tags_2 },
    { 0x40632114, SUC_lexicon_tags_33 },
    { 0xb1a32113, SUC_lexicon_tags_9 },
    { 0xc9632116, SUC_lexicon_tags_16 },
    { 0x4dc32116, SUC_lexicon_tags_7 },
    { 0x3ce32117, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x5643211b, SUC_lexicon_tags_8 },
    { 0xf823211c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63211f, SUC_lexicon_tags_27 },
    { 0xd663211f, SUC_lexicon_tags_12 },
    { 0xd0232121, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e432124, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f232127, SUC_lexicon_tags_25 },
    { 0x64232128, SUC_lexicon_tags_26 },
    { 0xf8a32128, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923212d, SUC_lexicon_tags_65 },
    { 0xb803212e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf0232130, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x11632132, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52232135, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77c32137, SUC_lexicon_tags_37 },
    { 0x32032138, SUC_lexicon_tags_15 },
    { 0xad832139, SUC_lexicon_tags_15 },
    { 0x6963213a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a3213d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5832141, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde632145, SUC_lexicon_tags_45 },
    { 0x76632145, SUC_lexicon_tags_7 },
    { 0xafe32146, SUC_lexicon_tags_29 },
    { 0xccc32147, SUC_lexicon_tags_49 },
    { 0x82e32149, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7823214b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6663214f, SUC_lexicon_tags_15 },
    { 0x41c3214f, SUC_lexicon_tags_37 },
    { 0x6d032151, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d232157, SUC_lexicon_tags_14 },
    { 0xdbc32158, SUC_lexicon_tags_27 },
    { 0xd5e32159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe32161, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c32164, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fc32166, SUC_lexicon_tags_15 },
    { 0xa2c32167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc23216b, SUC_lexicon_tags_6 },
    { 0x8d23216c, SUC_lexicon_tags_10 },
    { 0xbcc3216d, SUC_lexicon_tags_8 },
    { 0xf23216d, SUC_lexicon_tags_47 },
    { 0x7a03216d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69832176, SUC_lexicon_tags_15 },
    { 0x88c32177, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x69c32179, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda63217d, SUC_lexicon_tags_1 },
    { 0x7f43217e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfce32180, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x62832182, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61032187, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf643218b, SUC_lexicon_tags_66 },
    { 0x7383218c, SUC_lexicon_tags_45 },
    { 0xa2a3218d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e32193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45a32195, SUC_lexicon_tags_17 },
    { 0x3c432195, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd923219a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8321a1, SUC_lexicon_tags_30 },
    { 0x8321a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a321a6, SUC_lexicon_tags_30 },
    { 0x914321a6, SUC_lexicon_tags_48 },
    { 0xf38321a6, SUC_lexicon_tags_0 },
    { 0x420321a7, SUC_lexicon_tags_33 },
    { 0x2fa321a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a321ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x660321b1, SUC_lexicon_tags_15 },
    { 0xcc321b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e321b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2321ba, SUC_lexicon_tags_14 },
    { 0x990321bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174321c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2321c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e321c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0321cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbfe321cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x938321d0, SUC_lexicon_tags_118 },
    { 0xa62321d1, SUC_lexicon_tags_60 },
    { 0x5da321d2, SUC_lexicon_tags_7 },
    { 0xf2c321d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6321d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c321da, SUC_lexicon_tags_2 },
    { 0x2a321da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34321de, SUC_lexicon_tags_33 },
    { 0x528321df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd8321e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da321e4, SUC_lexicon_tags_2 },
    { 0xbd6321e4, SUC_lexicon_tags_40 },
    { 0x9ee321e6, SUC_lexicon_tags_7 },
    { 0x99a321e6, SUC_lexicon_tags_1 },
    { 0xb96321e7, SUC_lexicon_tags_47 },
    { 0x2be321e7, SUC_lexicon_tags_53 },
    { 0xedc321ea, SUC_lexicon_tags_15 },
    { 0x62e321ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd32321ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236321f2, SUC_lexicon_tags_15 },
    { 0x456321f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda321f6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4432201, SUC_lexicon_tags_62 },
    { 0xcfc32201, SUC_lexicon_tags_1 },
    { 0xf9232203, SUC_lexicon_tags_6 },
    { 0x1ec32202, SUC_lexicon_tags_8 },
    { 0x71632204, SUC_lexicon_tags_23 },
    { 0x20e32206, SUC_lexicon_tags_27 },
    { 0x18a32207, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc83220a, SUC_lexicon_tags_15 },
    { 0x6a03220b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe003220d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a32212, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c32218, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8383221b, SUC_lexicon_tags_7 },
    { 0x6543221b, SUC_lexicon_tags_27 },
    { 0x99e3221c, SUC_lexicon_tags_42 },
    { 0x19e3221e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede32223, SUC_lexicon_tags_1087 },
    { 0xf4232224, SUC_lexicon_tags_234 },
    { 0x2ba32225, SUC_lexicon_tags_17 },
    { 0xf5832225, SUC_lexicon_tags_1 },
    { 0x6f632227, SUC_lexicon_tags_31 },
    { 0x8c232227, SUC_lexicon_tags_28 },
    { 0x1d432229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b232231, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b032235, SUC_lexicon_tags_8 },
    { 0x67232235, SUC_lexicon_tags_12 },
    { 0x4a32237, SUC_lexicon_tags_37 },
    { 0xeba32238, SUC_lexicon_tags_1 },
    { 0x84632239, SUC_lexicon_tags_7 },
    { 0xc32238, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa943223c, SUC_lexicon_tags_31 },
    { 0x92a3223d, SUC_lexicon_tags_2 },
    { 0x4063223e, SUC_lexicon_tags_7 },
    { 0xf603223f, SUC_lexicon_tags_19 },
    { 0xd643223f, SUC_lexicon_tags_11 },
    { 0x2dc32241, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79632245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b032248, SUC_lexicon_tags_37 },
    { 0xcea32248, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a3224c, SUC_lexicon_tags_37 },
    { 0x3a03224d, SUC_lexicon_tags_68 },
    { 0x6983224d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4432250, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea432252, SUC_lexicon_tags_7 },
    { 0x1a632253, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80632255, SUC_lexicon_tags_7 },
    { 0xfc232256, SUC_lexicon_tags_16 },
    { 0x25832256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e3225c, SUC_lexicon_tags_25 },
    { 0x3543225d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb103225f, SUC_lexicon_tags_25 },
    { 0xed432260, SUC_lexicon_tags_60 },
    { 0x8a032260, SUC_lexicon_tags_11 },
    { 0x3d832262, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd2a32264, SUC_lexicon_tags_26 },
    { 0x26632265, SUC_lexicon_tags_37 },
    { 0xeb632266, SUC_lexicon_tags_42 },
    { 0x38032267, SUC_lexicon_tags_48 },
    { 0xce632268, SUC_lexicon_tags_23 },
    { 0x80032269, SUC_lexicon_tags_45 },
    { 0x52832266, SUC_lexicon_tags_14 },
    { 0xa2632269, SUC_lexicon_tags_47 },
    { 0x3083226c, SUC_lexicon_tags_15 },
    { 0x2483226d, SUC_lexicon_tags_65 },
    { 0xf723226d, SUC_lexicon_tags_46 },
    { 0x2443226f, SUC_lexicon_tags_14 },
    { 0x90e3226e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70432278, SUC_lexicon_tags_33 },
    { 0x9032278, SUC_lexicon_tags_15 },
    { 0x7003227a, SUC_lexicon_tags_7 },
    { 0x9e3227b, SUC_lexicon_tags_7 },
    { 0x7883227a, SUC_lexicon_tags_2 },
    { 0x5703227b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1e3227f, SUC_lexicon_tags_8 },
    { 0xe083227f, SUC_lexicon_tags_63 },
    { 0x32281, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e32285, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfc32287, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2f232289, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd663228c, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b63228f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0232291, SUC_lexicon_tags_45 },
    { 0xf232292, SUC_lexicon_tags_0 },
    { 0xc7632293, SUC_lexicon_tags_15 },
    { 0xc0232294, SUC_lexicon_tags_32 },
    { 0x37032291, SUC_lexicon_tags_38 },
    { 0x54032292, SUC_lexicon_tags_4 },
    { 0x83632297, SUC_lexicon_tags_56 },
    { 0xccc32297, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc43229d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2322a0, SUC_lexicon_tags_1 },
    { 0xbd2322a1, SUC_lexicon_tags_1 },
    { 0x35a322a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x89e322a4, SUC_lexicon_tags_4 },
    { 0xf3c322a5, SUC_lexicon_tags_12 },
    { 0xc1a322a5, SUC_lexicon_tags_27 },
    { 0xa24322a5, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4322ab, SUC_lexicon_tags_1 },
    { 0xf72322ac, SUC_lexicon_tags_8 },
    { 0xa6e322ab, SUC_lexicon_tags_7 },
    { 0xa00322ae, SUC_lexicon_tags_30 },
    { 0xd2e322af, SUC_lexicon_tags_14 },
    { 0x368322ac, SUC_lexicon_tags_11 },
    { 0x4c6322b1, SUC_lexicon_tags_28 },
    { 0x582322ac, SUC_lexicon_tags_11 },
    { 0x9a8322b3, SUC_lexicon_tags_25 },
    { 0x984322af, SUC_lexicon_tags_54 },
    { 0xdda322af, SUC_lexicon_tags_49 },
    { 0x8f6322b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe322b9, SUC_lexicon_tags_2 },
    { 0xbd6322ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4322bc, SUC_lexicon_tags_31 },
    { 0xf56322bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x324322bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0322c5, SUC_lexicon_tags_11 },
    { 0x5ea322c6, SUC_lexicon_tags_7 },
    { 0x590322c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8322cc, SUC_lexicon_tags_49 },
    { 0xde4322cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x388322cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf2322d1, SUC_lexicon_tags_435 },
    { 0xfe2322d1, SUC_lexicon_tags_42 },
    { 0x6dc322d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2322d6, SUC_lexicon_tags_45 },
    { 0x49e322d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc322da, SUC_lexicon_tags_11 },
    { 0xfbc322db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee322dd, SUC_lexicon_tags_17 },
    { 0x30e322de, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xce322e0, SUC_lexicon_tags_99 },
    { 0xbee322e1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580322eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c322f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x51c322f5, SUC_lexicon_tags_47 },
    { 0x174322f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a322f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a322fc, SUC_lexicon_tags_29 },
    { 0x9cc322fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21632302, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea32306, SUC_lexicon_tags_133 },
    { 0x9d432307, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a3230a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd23230c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d03230f, SUC_lexicon_tags_7 },
    { 0xb4432310, SUC_lexicon_tags_43 },
    { 0xbc32310, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x67432314, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8232317, SUC_lexicon_tags_17 },
    { 0x2c432318, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc32320, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa32324, SUC_lexicon_tags_25 },
    { 0x8fa32325, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0xe7832327, SUC_lexicon_tags_1 },
    { 0x40a32327, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x203232a, SUC_lexicon_tags_0 },
    { 0x6aa3232b, SUC_lexicon_tags_29 },
    { 0x7fc3232c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae3232f, SUC_lexicon_tags_15 },
    { 0x23032330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9e32332, SUC_lexicon_tags_2 },
    { 0x76a32333, SUC_lexicon_tags_15 },
    { 0x22c32334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba232336, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d63233a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe943233c, SUC_lexicon_tags_37 },
    { 0x1263233d, SUC_lexicon_tags_19 },
    { 0x8463233c, SUC_lexicon_tags_9 },
    { 0x3a43233f, SUC_lexicon_tags_69 },
    { 0x2543233f, SUC_lexicon_tags_28 },
    { 0x2fe3233c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd632343, SUC_lexicon_tags_11 },
    { 0x96832343, SUC_lexicon_tags_11 },
    { 0x8bc32345, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a232349, SUC_lexicon_tags_49 },
    { 0xa483234a, SUC_lexicon_tags_1 },
    { 0xc7c32349, SUC_lexicon_tags_37 },
    { 0x4903234c, SUC_lexicon_tags_87 },
    { 0x5b63234c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c032351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd232353, SUC_lexicon_tags_7 },
    { 0x4d232354, SUC_lexicon_tags_11 },
    { 0xc832353, SUC_lexicon_tags_37 },
    { 0x7cc32353, SUC_lexicon_tags_37 },
    { 0xe0032354, SUC_lexicon_tags_43 },
    { 0xa3632354, SUC_lexicon_tags_13 },
    { 0x18c32359, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc983235d, SUC_lexicon_tags_48 },
    { 0x1543235d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e32361, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a032363, SUC_lexicon_tags_14 },
    { 0x61a32363, SUC_lexicon_tags_7 },
    { 0x52432365, SUC_lexicon_tags_25 },
    { 0x3ec32366, SUC_lexicon_tags_118 },
    { 0xed632365, SUC_lexicon_tags_11 },
    { 0x91232368, SUC_lexicon_tags_29 },
    { 0x30c32369, SUC_lexicon_tags_15 },
    { 0xd143236a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe663236c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37432374, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b03237e, SUC_lexicon_tags_6 },
    { 0x63237f, SUC_lexicon_tags_17 },
    { 0x59632380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33032383, SUC_lexicon_tags_343 },
    { 0xaa832384, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4232387, SUC_lexicon_tags_16 },
    { 0xffc32388, SUC_lexicon_tags_9 },
    { 0x12832389, SUC_lexicon_tags_32 },
    { 0x54a3238a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac32393, SUC_lexicon_tags_37 },
    { 0x44632393, SUC_lexicon_tags_12 },
    { 0xe5432395, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28e32397, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce3239b, SUC_lexicon_tags_2 },
    { 0xade3239c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766323a1, SUC_lexicon_tags_17 },
    { 0xed2323a1, SUC_lexicon_tags_15 },
    { 0x97c323a3, SUC_lexicon_tags_49 },
    { 0x9fa323a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe68323a6, SUC_lexicon_tags_2 },
    { 0x900323a7, SUC_lexicon_tags_14 },
    { 0x1e6323a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x864323aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18323b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30323b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c323b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c323bd, SUC_lexicon_tags_11 },
    { 0x660323be, SUC_lexicon_tags_11 },
    { 0x208323bf, SUC_lexicon_tags_37 },
    { 0x864323c0, SUC_lexicon_tags_33 },
    { 0xa86323bd, SUC_lexicon_tags_47 },
    { 0x4a6323bf, SUC_lexicon_tags_21 },
    { 0x37e323c3, SUC_lexicon_tags_7 },
    { 0xd2c323c4, SUC_lexicon_tags_49 },
    { 0xb18323c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x36e323c7, SUC_lexicon_tags_12 },
    { 0x1a8323c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd52323ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5a323cc, SUC_lexicon_tags_17 },
    { 0x296323cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c323d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8323d4, SUC_lexicon_tags_68 },
    { 0xb9e323d5, SUC_lexicon_tags_38 },
    { 0x506323d5, SUC_lexicon_tags_1 },
    { 0x14e323d7, SUC_lexicon_tags_0 },
    { 0xd24323d8, SUC_lexicon_tags_29 },
    { 0xcf2323d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6323dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16323e0, SUC_lexicon_tags_57 },
    { 0x5ca323e1, SUC_lexicon_tags_15 },
    { 0x2d2323e2, SUC_lexicon_tags_17 },
    { 0x8dc323e3, SUC_lexicon_tags_37 },
    { 0x3f6323e2, SUC_lexicon_tags_11 },
    { 0x864323e5, SUC_lexicon_tags_26 },
    { 0x232323e6, SUC_lexicon_tags_25 },
    { 0x4fc323e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a323ea, SUC_lexicon_tags_7 },
    { 0x4d6323eb, SUC_lexicon_tags_1 },
    { 0xbe4323ec, SUC_lexicon_tags_79 },
    { 0xbe323ec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf88323ef, SUC_lexicon_tags_15 },
    { 0x450323ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20323f4, SUC_lexicon_tags_15 },
    { 0x126323f5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c323f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c323fb, SUC_lexicon_tags_17 },
    { 0x1bc323fc, SUC_lexicon_tags_2 },
    { 0xe50323fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fc323ff, SUC_lexicon_tags_27 },
    { 0xaa832400, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c32405, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1803240f, SUC_lexicon_tags_0 },
    { 0x53632410, SUC_lexicon_tags_23 },
    { 0x37a32411, SUC_lexicon_tags_11 },
    { 0xb123240f, SUC_lexicon_tags_37 },
    { 0x1fa32411, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a3241a, SUC_lexicon_tags_982 },
    { 0xd7a3241b, SUC_lexicon_tags_1 },
    { 0x443241c, SUC_lexicon_tags_12 },
    { 0xe063241d, SUC_lexicon_tags_11 },
    { 0x1103241e, SUC_lexicon_tags_60 },
    { 0x9063241e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9032426, SUC_lexicon_tags_127 },
    { 0x31632426, SUC_lexicon_tags_87 },
    { 0x8ae32428, SUC_lexicon_tags_7 },
    { 0xcfe32428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8203242e, SUC_lexicon_tags_87 },
    { 0xf123242f, SUC_lexicon_tags_15 },
    { 0xd7832430, SUC_lexicon_tags_7 },
    { 0x35a3242f, SUC_lexicon_tags_45 },
    { 0x1403242f, SUC_lexicon_tags_11 },
    { 0x13032433, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1432436, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8032438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec23243c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9a03243e, SUC_lexicon_tags_1 },
    { 0x1083243e, SUC_lexicon_tags_120 },
    { 0x53632440, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a3244b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84632452, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x62c32454, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa232457, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd8e32459, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd803245b, SUC_lexicon_tags_23 },
    { 0xa343245b, SUC_lexicon_tags_1 },
    { 0x7323245d, SUC_lexicon_tags_26 },
    { 0xec83245e, SUC_lexicon_tags_65 },
    { 0x3323245f, SUC_lexicon_tags_17 },
    { 0x3943245c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b832463, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x7032466, SUC_lexicon_tags_65 },
    { 0xbea32467, SUC_lexicon_tags_39 },
    { 0xc6e32468, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e32470, SUC_lexicon_tags_15 },
    { 0x5f032471, SUC_lexicon_tags_47 },
    { 0x1a032470, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ce32474, SUC_lexicon_tags_7 },
    { 0x5c032475, SUC_lexicon_tags_15 },
    { 0xada32475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3247a, SUC_lexicon_tags_387 },
    { 0x4443247b, SUC_lexicon_tags_37 },
    { 0xaaa3247c, SUC_lexicon_tags_74 },
    { 0xa103247b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c32484, SUC_lexicon_tags_10 },
    { 0x6bc32485, SUC_lexicon_tags_10 },
    { 0x4c232486, SUC_lexicon_tags_2 },
    { 0x26232485, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7043248d, SUC_lexicon_tags_35 },
    { 0x9623248e, SUC_lexicon_tags_43 },
    { 0xf7e3248f, SUC_lexicon_tags_77 },
    { 0xe863248d, SUC_lexicon_tags_47 },
    { 0xab83248d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36232497, SUC_lexicon_tags_1 },
    { 0xf0a32498, SUC_lexicon_tags_11 },
    { 0x9e032498, SUC_lexicon_tags_33 },
    { 0xd632498, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c43249c, SUC_lexicon_tags_56 },
    { 0x6503249d, SUC_lexicon_tags_15 },
    { 0xf243249e, SUC_lexicon_tags_27 },
    { 0xd383249d, SUC_lexicon_tags_27 },
    { 0x90e324a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134324a4, SUC_lexicon_tags_7 },
    { 0x28324a5, SUC_lexicon_tags_2 },
    { 0x53c324a6, SUC_lexicon_tags_37 },
    { 0x57e324a7, SUC_lexicon_tags_15 },
    { 0xad4324a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e324ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62324b0, SUC_lexicon_tags_1 },
    { 0xc5c324b1, SUC_lexicon_tags_28 },
    { 0x436324b2, SUC_lexicon_tags_230 },
    { 0x89c324b2, SUC_lexicon_tags_2 },
    { 0xd08324b4, SUC_lexicon_tags_17 },
    { 0x174324b5, SUC_lexicon_tags_37 },
    { 0x85c324b5, SUC_lexicon_tags_151 },
    { 0x45e324b5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x358324b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4324bc, SUC_lexicon_tags_37 },
    { 0x570324bd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22324c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c324c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8324cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc28324cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c324d6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x57a324d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x780324da, SUC_lexicon_tags_1 },
    { 0xb96324db, SUC_lexicon_tags_94 },
    { 0x8324da, SUC_lexicon_tags_0 },
    { 0x866324dc, SUC_lexicon_tags_11 },
    { 0x67c324de, SUC_lexicon_tags_14 },
    { 0xeec324df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960324e3, SUC_lexicon_tags_17 },
    { 0xe02324e4, SUC_lexicon_tags_37 },
    { 0xd9c324e4, SUC_lexicon_tags_3 },
    { 0x34e324e6, SUC_lexicon_tags_25 },
    { 0x888324e6, SUC_lexicon_tags_25 },
    { 0x158324e6, SUC_lexicon_tags_38 },
    { 0x712324e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xddc324eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460324f0, SUC_lexicon_tags_2 },
    { 0xe34324f1, SUC_lexicon_tags_8 },
    { 0x46a324f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea324f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0324fe, SUC_lexicon_tags_2 },
    { 0x330324ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89432504, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa1832506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22032508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd083250f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92432513, SUC_lexicon_tags_1 },
    { 0xec632514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9432519, SUC_lexicon_tags_37 },
    { 0x9e32519, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe023251c, SUC_lexicon_tags_64 },
    { 0x26a3251c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63432521, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x23032524, SUC_lexicon_tags_25 },
    { 0xe2032525, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7e32527, SUC_lexicon_tags_2 },
    { 0x29432527, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8423252a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdc83252c, SUC_lexicon_tags_10 },
    { 0xdb03252d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb832532, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd9e32534, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x12e32536, SUC_lexicon_tags_2 },
    { 0xc1632537, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42e32539, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x343253b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2de3253d, SUC_lexicon_tags_31 },
    { 0xf2e3253d, SUC_lexicon_tags_49 },
    { 0x52a3253f, SUC_lexicon_tags_37 },
    { 0xa3e3253f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe32545, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x67832547, SUC_lexicon_tags_17 },
    { 0x3a832548, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae3254b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66e3254d, SUC_lexicon_tags_11 },
    { 0xbc03254e, SUC_lexicon_tags_113 },
    { 0xb1a3254f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98c32551, SUC_lexicon_tags_15 },
    { 0x9ec32551, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c32555, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39632557, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa3255b, SUC_lexicon_tags_38 },
    { 0x8903255c, SUC_lexicon_tags_37 },
    { 0xb7e3255b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xebc3255f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd632566, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0c32568, SUC_lexicon_tags_12 },
    { 0xd5c32568, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d03256c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe43256e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c432572, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45232577, SUC_lexicon_tags_23 },
    { 0x2e432577, SUC_lexicon_tags_43 },
    { 0x8f632579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf963257b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223257f, SUC_lexicon_tags_47 },
    { 0x4b032580, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31432582, SUC_lexicon_tags_25 },
    { 0xe5032582, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e432587, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4743258a, SUC_lexicon_tags_29 },
    { 0x2243258b, SUC_lexicon_tags_25 },
    { 0x9043258c, SUC_lexicon_tags_17 },
    { 0xd9c3258c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8832590, SUC_lexicon_tags_17 },
    { 0xca432591, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd443259c, SUC_lexicon_tags_39 },
    { 0x5163259d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794325a1, SUC_lexicon_tags_7 },
    { 0x2f2325a2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98325a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x398325a7, SUC_lexicon_tags_82 },
    { 0xafc325a8, SUC_lexicon_tags_33 },
    { 0x6e8325a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x994325ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6325b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf42325b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84325b7, SUC_lexicon_tags_16 },
    { 0x74e325b8, SUC_lexicon_tags_10 },
    { 0xc48325b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c325bc, SUC_lexicon_tags_17 },
    { 0xc9c325bd, SUC_lexicon_tags_11 },
    { 0x896325bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x790325c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x282325c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260325c7, SUC_lexicon_tags_6 },
    { 0x854325c8, SUC_lexicon_tags_17 },
    { 0xe9e325c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636325cd, SUC_lexicon_tags_19 },
    { 0xe00325ce, SUC_lexicon_tags_38 },
    { 0x6325cf, SUC_lexicon_tags_34 },
    { 0x18325cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee325d3, SUC_lexicon_tags_7 },
    { 0x4a2325d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160325d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x12325db, SUC_lexicon_tags_0 },
    { 0x38c325dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa325df, SUC_lexicon_tags_30 },
    { 0x748325e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x330325e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c325e6, SUC_lexicon_tags_49 },
    { 0xbb4325e7, SUC_lexicon_tags_13 },
    { 0xb66325e8, SUC_lexicon_tags_45 },
    { 0x860325e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912325ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8325f2, SUC_lexicon_tags_38 },
    { 0x89e325f3, SUC_lexicon_tags_0 },
    { 0xcee325f4, SUC_lexicon_tags_26 },
    { 0xcce325f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a325fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60a325fe, SUC_lexicon_tags_37 },
    { 0x434325ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc032607, SUC_lexicon_tags_37 },
    { 0x57a32608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfaa3260a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a3260e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24232611, SUC_lexicon_tags_37 },
    { 0xa5832612, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a32615, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48032617, SUC_lexicon_tags_0 },
    { 0x1e432618, SUC_lexicon_tags_0 },
    { 0xa3832619, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa583261e, SUC_lexicon_tags_7 },
    { 0x8ce3261f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b432624, SUC_lexicon_tags_17 },
    { 0xe2e32625, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3632628, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1503262b, SUC_lexicon_tags_7 },
    { 0x2f83262b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59232630, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc32633, SUC_lexicon_tags_29 },
    { 0x57832634, SUC_lexicon_tags_15 },
    { 0x30832634, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8632638, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c03263a, SUC_lexicon_tags_2 },
    { 0xf863263b, SUC_lexicon_tags_65 },
    { 0x83c3263b, SUC_lexicon_tags_2 },
    { 0xe23263b, SUC_lexicon_tags_37 },
    { 0xf383263e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc432641, SUC_lexicon_tags_72 },
    { 0x86232642, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xefe32644, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67232649, SUC_lexicon_tags_7 },
    { 0x7e23264a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca83264c, SUC_lexicon_tags_6 },
    { 0xda03264d, SUC_lexicon_tags_22 },
    { 0x18c3264d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fa32650, SUC_lexicon_tags_104 },
    { 0xa5232651, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x40432654, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc32658, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee23265a, SUC_lexicon_tags_31 },
    { 0xb6a3265a, SUC_lexicon_tags_53 },
    { 0x3b63265c, SUC_lexicon_tags_11 },
    { 0x89e3265d, SUC_lexicon_tags_49 },
    { 0xeb23265a, SUC_lexicon_tags_7 },
    { 0xd883265c, SUC_lexicon_tags_627 },
    { 0x52432660, SUC_lexicon_tags_2 },
    { 0xcce32660, SUC_lexicon_tags_7 },
    { 0x8be3265c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca32668, SUC_lexicon_tags_15 },
    { 0x48632668, SUC_lexicon_tags_17 },
    { 0xe503266a, SUC_lexicon_tags_14 },
    { 0x3b032669, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6632670, SUC_lexicon_tags_17 },
    { 0x3ce32671, SUC_lexicon_tags_9 },
    { 0x39432672, SUC_lexicon_tags_49 },
    { 0xe2432670, SUC_lexicon_tags_15 },
    { 0x98e32671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe23267a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a63267d, SUC_lexicon_tags_37 },
    { 0x2be3267e, SUC_lexicon_tags_7 },
    { 0x4003267f, SUC_lexicon_tags_15 },
    { 0x3d23267d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e43268c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeee3268e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe232695, SUC_lexicon_tags_25 },
    { 0x34832696, SUC_lexicon_tags_37 },
    { 0xa0632695, SUC_lexicon_tags_37 },
    { 0x3a32698, SUC_lexicon_tags_17 },
    { 0xb5e32696, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6283269d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac83269f, SUC_lexicon_tags_2 },
    { 0x5ba326a0, SUC_lexicon_tags_1 },
    { 0xd06326a1, SUC_lexicon_tags_15 },
    { 0xe9e326a0, SUC_lexicon_tags_33 },
    { 0x6ac326a1, SUC_lexicon_tags_31 },
    { 0x980326a4, SUC_lexicon_tags_27 },
    { 0x9dc326a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x73a326a7, SUC_lexicon_tags_25 },
    { 0x400326a8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a326ac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe12326ae, SUC_lexicon_tags_60 },
    { 0xcdc326af, SUC_lexicon_tags_68 },
    { 0x6cc326b0, SUC_lexicon_tags_33 },
    { 0xb54326b1, SUC_lexicon_tags_7 },
    { 0xb70326b0, SUC_lexicon_tags_10 },
    { 0xc04326b3, SUC_lexicon_tags_25 },
    { 0xc6a326b4, SUC_lexicon_tags_15 },
    { 0x9f6326b5, SUC_lexicon_tags_37 },
    { 0x9d4326b6, SUC_lexicon_tags_25 },
    { 0x4ce326b1, SUC_lexicon_tags_37 },
    { 0xd44326b8, SUC_lexicon_tags_30 },
    { 0x5e4326b6, SUC_lexicon_tags_15 },
    { 0x8d4326ba, SUC_lexicon_tags_7 },
    { 0x1fe326ba, SUC_lexicon_tags_25 },
    { 0xfd0326b8, SUC_lexicon_tags_17 },
    { 0x9ac326bd, SUC_lexicon_tags_39 },
    { 0xa68326bd, SUC_lexicon_tags_6 },
    { 0x836326bf, SUC_lexicon_tags_12 },
    { 0x486326b0, SUC_lexicon_tags_37 },
    { 0x72e326bb, SUC_lexicon_tags_46 },
    { 0x282326be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00326c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a2326ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x19e326cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x968326cf, SUC_lexicon_tags_27 },
    { 0xd90326d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2326d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea326db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534326e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424326ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7c326ec, SUC_lexicon_tags_1 },
    { 0x57c326ed, SUC_lexicon_tags_12 },
    { 0x95e326ed, SUC_lexicon_tags_7 },
    { 0xbbc326ef, SUC_lexicon_tags_63 },
    { 0xa40326ed, SUC_lexicon_tags_46 },
    { 0x6d0326f1, SUC_lexicon_tags_66 },
    { 0x534326f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a326f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8326f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590326fe, SUC_lexicon_tags_27 },
    { 0x1e6326fe, SUC_lexicon_tags_21 },
    { 0xabe32700, SUC_lexicon_tags_15 },
    { 0x7ea326ff, SUC_lexicon_tags_4 },
    { 0x4cc326ff, SUC_lexicon_tags_1 },
    { 0xf2832700, SUC_lexicon_tags_29 },
    { 0x56632700, SUC_lexicon_tags_27 },
    { 0x15a32705, SUC_lexicon_tags_60 },
    { 0x7ac32701, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8e32708, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa583270b, SUC_lexicon_tags_5 },
    { 0xcde3270c, SUC_lexicon_tags_0 },
    { 0x1403270d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58232711, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x90432714, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ba32716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e32719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6e3271b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3ae3271d, SUC_lexicon_tags_15 },
    { 0x3fa3271e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6ca32720, SUC_lexicon_tags_1 },
    { 0xa6c32721, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93232724, SUC_lexicon_tags_19 },
    { 0xf8e32724, SUC_lexicon_tags_19 },
    { 0xe4032726, SUC_lexicon_tags_37 },
    { 0x78832727, SUC_lexicon_tags_37 },
    { 0x2c432728, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8483272b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f63272d, SUC_lexicon_tags_1 },
    { 0x6243272d, SUC_lexicon_tags_17 },
    { 0x3863272f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d232733, SUC_lexicon_tags_68 },
    { 0xb7432734, SUC_lexicon_tags_17 },
    { 0xe3c32733, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x16832737, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7343273a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4243273e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ee32740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c432743, SUC_lexicon_tags_2 },
    { 0xa4a32744, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e32747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd383274c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1f03274e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb032751, SUC_lexicon_tags_17 },
    { 0xb3632751, SUC_lexicon_tags_7 },
    { 0x42632753, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7b832755, SUC_lexicon_tags_20 },
    { 0xc6c32756, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c3275a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c43275e, SUC_lexicon_tags_38 },
    { 0x4a43275f, SUC_lexicon_tags_12 },
    { 0x23632760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2232762, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23232766, SUC_lexicon_tags_84 },
    { 0x5c232767, SUC_lexicon_tags_41 },
    { 0xfe232766, SUC_lexicon_tags_35 },
    { 0x26232768, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78632771, SUC_lexicon_tags_37 },
    { 0xd2232772, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea832777, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de3277b, SUC_lexicon_tags_15 },
    { 0x4b23277c, SUC_lexicon_tags_55 },
    { 0x2c63277c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d43277f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a32785, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9483278d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3832791, SUC_lexicon_tags_31 },
    { 0x84032792, SUC_lexicon_tags_14 },
    { 0x90232793, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29232795, SUC_lexicon_tags_12 },
    { 0x96232796, SUC_lexicon_tags_46 },
    { 0x56a32797, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788327a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x872327a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52327a6, SUC_lexicon_tags_37 },
    { 0xc9a327a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee327b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6327b2, SUC_lexicon_tags_12 },
    { 0xda6327b2, SUC_lexicon_tags_57 },
    { 0x9e6327b2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8327b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66327ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c327bd, SUC_lexicon_tags_91 },
    { 0xff6327be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144327c4, SUC_lexicon_tags_7 },
    { 0x2fc327c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc327d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326327da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a327dd, SUC_lexicon_tags_12 },
    { 0x388327dd, SUC_lexicon_tags_47 },
    { 0xc78327df, SUC_lexicon_tags_12 },
    { 0x934327e0, SUC_lexicon_tags_27 },
    { 0x898327e1, SUC_lexicon_tags_8 },
    { 0x418327e2, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x514327e4, SUC_lexicon_tags_1 },
    { 0x48e327e4, SUC_lexicon_tags_17 },
    { 0x944327e4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac327ed, SUC_lexicon_tags_37 },
    { 0xb26327ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe74327f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc32803, SUC_lexicon_tags_1 },
    { 0xa6632804, SUC_lexicon_tags_12 },
    { 0x43432805, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe9a32807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba3280c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe32810, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x40232813, SUC_lexicon_tags_30 },
    { 0x69e32814, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe323281a, SUC_lexicon_tags_56 },
    { 0x8e83281b, SUC_lexicon_tags_25 },
    { 0xf003281c, SUC_lexicon_tags_48 },
    { 0x38e3281d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89a3281f, SUC_lexicon_tags_1 },
    { 0x39e32820, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c432823, SUC_lexicon_tags_17 },
    { 0xbea32824, SUC_lexicon_tags_0 },
    { 0xb5c32824, SUC_lexicon_tags_112 },
    { 0xa6832826, SUC_lexicon_tags_27 },
    { 0x87632825, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd963282f, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0xf8c32831, SUC_lexicon_tags_11 },
    { 0xc3232832, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc32838, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbbe3283a, SUC_lexicon_tags_17 },
    { 0x8f83283b, SUC_lexicon_tags_25 },
    { 0x23c3283b, SUC_lexicon_tags_49 },
    { 0x3823283d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40632841, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde32845, SUC_lexicon_tags_43 },
    { 0xb8a32846, SUC_lexicon_tags_11 },
    { 0x74e32846, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3284b, SUC_lexicon_tags_14 },
    { 0xc423284b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4032852, SUC_lexicon_tags_1 },
    { 0x55832852, SUC_lexicon_tags_1 },
    { 0x2c832853, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b832856, SUC_lexicon_tags_33 },
    { 0xdbe32856, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xaaa32859, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e3285d, SUC_lexicon_tags_11 },
    { 0x6e43285e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e32861, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe2a32863, SUC_lexicon_tags_2 },
    { 0x22432864, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc32869, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f832871, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ae32873, SUC_lexicon_tags_11 },
    { 0x71a32873, SUC_lexicon_tags_37 },
    { 0xd1232875, SUC_lexicon_tags_2 },
    { 0x91832876, SUC_lexicon_tags_8 },
    { 0xa2432877, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5232880, SUC_lexicon_tags_25 },
    { 0xe1432881, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb232883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c432887, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c3288b, SUC_lexicon_tags_1 },
    { 0x8143288c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec32890, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e32896, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3289c, SUC_lexicon_tags_25 },
    { 0xf903289c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac328a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b6328a3, SUC_lexicon_tags_9 },
    { 0x7b8328a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4328a9, SUC_lexicon_tags_0 },
    { 0xb02328aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304328b1, SUC_lexicon_tags_17 },
    { 0xda328b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e328bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x168328bd, SUC_lexicon_tags_25 },
    { 0x46e328be, SUC_lexicon_tags_15 },
    { 0xe8a328bf, SUC_lexicon_tags_15 },
    { 0x86328bf, SUC_lexicon_tags_26 },
    { 0x8e2328c1, SUC_lexicon_tags_25 },
    { 0x86e328c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96328c5, SUC_lexicon_tags_7 },
    { 0x5a0328c6, SUC_lexicon_tags_37 },
    { 0xa32328c6, SUC_lexicon_tags_25 },
    { 0xee2328c8, SUC_lexicon_tags_27 },
    { 0x124328c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14328d2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0328d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a328db, SUC_lexicon_tags_0 },
    { 0xe2e328dc, SUC_lexicon_tags_43 },
    { 0xb20328dd, SUC_lexicon_tags_14 },
    { 0xe56328de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfe6328e0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x648328e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcde328e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c328f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14328f5, SUC_lexicon_tags_11 },
    { 0x4b6328f5, SUC_lexicon_tags_46 },
    { 0x98328f7, SUC_lexicon_tags_17 },
    { 0xe4c328f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x840328fa, SUC_lexicon_tags_38 },
    { 0x5a2328fa, SUC_lexicon_tags_25 },
    { 0x924328fa, SUC_lexicon_tags_1 },
    { 0x50a328fd, SUC_lexicon_tags_29 },
    { 0xd0c328fe, SUC_lexicon_tags_15 },
    { 0x112328fe, SUC_lexicon_tags_87 },
    { 0xe90328fe, SUC_lexicon_tags_47 },
    { 0xc3032901, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8ec32903, SUC_lexicon_tags_15 },
    { 0x72c32904, SUC_lexicon_tags_47 },
    { 0x3de32905, SUC_lexicon_tags_0 },
    { 0x53c32906, SUC_lexicon_tags_51 },
    { 0x94832907, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7832909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e632911, SUC_lexicon_tags_12 },
    { 0xa032912, SUC_lexicon_tags_38 },
    { 0xd8032913, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f832915, SUC_lexicon_tags_68 },
    { 0x79632916, SUC_lexicon_tags_60 },
    { 0xac032917, SUC_lexicon_tags_12 },
    { 0xec032916, SUC_lexicon_tags_1 },
    { 0x7f432916, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3643291d, SUC_lexicon_tags_45 },
    { 0x5f83291e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4032921, SUC_lexicon_tags_7 },
    { 0x8f432922, SUC_lexicon_tags_13 },
    { 0x9232923, SUC_lexicon_tags_27 },
    { 0x31c32922, SUC_lexicon_tags_30 },
    { 0x34232925, SUC_lexicon_tags_38 },
    { 0x91432926, SUC_lexicon_tags_0 },
    { 0xfc432927, SUC_lexicon_tags_63 },
    { 0xdce32924, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x33c3292a, SUC_lexicon_tags_7 },
    { 0xa623292b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb283292d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52632931, SUC_lexicon_tags_27 },
    { 0xda032932, SUC_lexicon_tags_9 },
    { 0x38a32932, SUC_lexicon_tags_7 },
    { 0x7c832934, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87432936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaac32938, SUC_lexicon_tags_87 },
    { 0x1d432938, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8a63293b, SUC_lexicon_tags_29 },
    { 0x583293b, SUC_lexicon_tags_10 },
    { 0x7423293b, SUC_lexicon_tags_1 },
    { 0xd843293d, SUC_lexicon_tags_17 },
    { 0x9523293f, SUC_lexicon_tags_14 },
    { 0x6f032940, SUC_lexicon_tags_8 },
    { 0x6e032941, SUC_lexicon_tags_0 },
    { 0x8ee32941, SUC_lexicon_tags_93 },
    { 0x82432943, SUC_lexicon_tags_28 },
    { 0xc3e3293e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8032947, SUC_lexicon_tags_38 },
    { 0x6c232948, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe743294b, SUC_lexicon_tags_15 },
    { 0x8ea3294b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d232954, SUC_lexicon_tags_14 },
    { 0x20632955, SUC_lexicon_tags_27 },
    { 0xbee32956, SUC_lexicon_tags_15 },
    { 0xa32957, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8e32959, SUC_lexicon_tags_23 },
    { 0x59632959, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca3295e, SUC_lexicon_tags_21 },
    { 0x1283295f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb603296a, SUC_lexicon_tags_37 },
    { 0x46c3296b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa3296f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d432972, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d632974, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac32978, SUC_lexicon_tags_91 },
    { 0xd0032978, SUC_lexicon_tags_14 },
    { 0x60e3297a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53a3297c, SUC_lexicon_tags_7 },
    { 0x1be3297d, SUC_lexicon_tags_33 },
    { 0x6e83297e, SUC_lexicon_tags_7 },
    { 0x5123297e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17c32981, SUC_lexicon_tags_0 },
    { 0x73032982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7203298a, SUC_lexicon_tags_25 },
    { 0x44e3298b, SUC_lexicon_tags_129 },
    { 0x59a3298a, SUC_lexicon_tags_17 },
    { 0xf8c3298d, SUC_lexicon_tags_79 },
    { 0x1a43298c, SUC_lexicon_tags_6 },
    { 0x3d83298e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38032994, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66632998, SUC_lexicon_tags_28 },
    { 0x6d032998, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc483299c, SUC_lexicon_tags_5 },
    { 0xc7e3299d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564329a1, SUC_lexicon_tags_27 },
    { 0x908329a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a329a5, SUC_lexicon_tags_14 },
    { 0x34a329a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30a329a8, SUC_lexicon_tags_29 },
    { 0x8b0329a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x322329ab, SUC_lexicon_tags_14 },
    { 0xbdc329ab, SUC_lexicon_tags_47 },
    { 0xaec329ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8329b1, SUC_lexicon_tags_12 },
    { 0x6f0329b2, SUC_lexicon_tags_33 },
    { 0x9d2329b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96329c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358329c6, SUC_lexicon_tags_69 },
    { 0x2cc329c7, SUC_lexicon_tags_37 },
    { 0x884329c8, SUC_lexicon_tags_47 },
    { 0x53a329c6, SUC_lexicon_tags_37 },
    { 0x646329ca, SUC_lexicon_tags_21 },
    { 0xfc2329cb, SUC_lexicon_tags_32 },
    { 0x9b8329cc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8ba329ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2e329d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82329d2, SUC_lexicon_tags_12 },
    { 0xa36329d3, SUC_lexicon_tags_13 },
    { 0x38329d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52329da, SUC_lexicon_tags_7 },
    { 0xf4329db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x798329dd, SUC_lexicon_tags_55 },
    { 0xf58329dd, SUC_lexicon_tags_28 },
    { 0x6e4329df, SUC_lexicon_tags_61 },
    { 0x664329e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676329e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2329ed, SUC_lexicon_tags_9 },
    { 0xeca329ee, SUC_lexicon_tags_47 },
    { 0xb0329ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe54329f1, SUC_lexicon_tags_2 },
    { 0x12e329f1, SUC_lexicon_tags_2 },
    { 0x716329f3, SUC_lexicon_tags_5 },
    { 0x6f6329f4, SUC_lexicon_tags_37 },
    { 0x47e329f5, SUC_lexicon_tags_43 },
    { 0x236329f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970329fa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa329fd, SUC_lexicon_tags_12 },
    { 0x9a6329fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27232a00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79e32a02, SUC_lexicon_tags_60 },
    { 0xbd232a02, SUC_lexicon_tags_2 },
    { 0x31632a04, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc5232a06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c832a08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c32a0a, SUC_lexicon_tags_170 },
    { 0x12432a0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7d232a0d, SUC_lexicon_tags_37 },
    { 0x6a832a0e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xac32a11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaac32a13, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x62632a16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15c32a18, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32a1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c32a1f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9432a21, SUC_lexicon_tags_14 },
    { 0xd2632a22, SUC_lexicon_tags_11 },
    { 0x2f432a22, SUC_lexicon_tags_1 },
    { 0xf1432a23, SUC_lexicon_tags_71 },
    { 0x2432a25, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x85632a28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76232a2b, SUC_lexicon_tags_35 },
    { 0x66432a2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba832a2e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x56232a31, SUC_lexicon_tags_12 },
    { 0x7fc32a31, SUC_lexicon_tags_49 },
    { 0x2f832a32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce032a35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14232a39, SUC_lexicon_tags_47 },
    { 0x4f232a3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd032a3c, SUC_lexicon_tags_47 },
    { 0x22e32a3c, SUC_lexicon_tags_7 },
    { 0xfe632a3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2832a41, SUC_lexicon_tags_1 },
    { 0x6fa32a42, SUC_lexicon_tags_2 },
    { 0xfc632a42, SUC_lexicon_tags_40 },
    { 0xf7832a42, SUC_lexicon_tags_33 },
    { 0x98632a44, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e32a48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85432a4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5832a4f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x60232a52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd832a55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74432a59, SUC_lexicon_tags_15 },
    { 0xd8832a59, SUC_lexicon_tags_33 },
    { 0x45832a5b, SUC_lexicon_tags_25 },
    { 0x34c32a5b, SUC_lexicon_tags_23 },
    { 0x9fe32a59, SUC_lexicon_tags_11 },
    { 0x36a32a59, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde232a65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62c32a67, SUC_lexicon_tags_1 },
    { 0x9b032a68, SUC_lexicon_tags_8 },
    { 0x3b632a69, SUC_lexicon_tags_2 },
    { 0xd4032a67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd832a6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x80432a71, SUC_lexicon_tags_33 },
    { 0x12632a72, SUC_lexicon_tags_46 },
    { 0xd8232a71, SUC_lexicon_tags_21 },
    { 0x3a632a71, SUC_lexicon_tags_13 },
    { 0x6ca32a75, SUC_lexicon_tags_85 },
    { 0x21a32a74, SUC_lexicon_tags_160 },
    { 0x94632a73, SUC_lexicon_tags_14 },
    { 0x29032a78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea32a7b, SUC_lexicon_tags_9 },
    { 0x82432a7c, SUC_lexicon_tags_25 },
    { 0x61232a7d, SUC_lexicon_tags_43 },
    { 0x82832a7d, SUC_lexicon_tags_49 },
    { 0xbd832a7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec832a84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b432a87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a32a91, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8f832a93, SUC_lexicon_tags_56 },
    { 0x2bc32a94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a32a99, SUC_lexicon_tags_12 },
    { 0xa7a32a99, SUC_lexicon_tags_27 },
    { 0xb8232a99, SUC_lexicon_tags_23 },
    { 0x82e32a9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0432a9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c32aa4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23032aae, SUC_lexicon_tags_20 },
    { 0x94632aae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c632ab1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f432ab5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a32ab8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd232aba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63232ac1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf8432ac3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12232ac6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a32acd, SUC_lexicon_tags_0 },
    { 0x35632ace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9032ad0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea32ad3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e32ad7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c32ada, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0032ade, SUC_lexicon_tags_6 },
    { 0x6ae32adf, SUC_lexicon_tags_6 },
    { 0xe7a32ae0, SUC_lexicon_tags_7 },
    { 0x8a032adf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c32ae6, SUC_lexicon_tags_47 },
    { 0x6c832ae7, SUC_lexicon_tags_26 },
    { 0xdb632ae8, SUC_lexicon_tags_38 },
    { 0x2aa32ae9, SUC_lexicon_tags_27 },
    { 0xfbe32ae9, SUC_lexicon_tags_25 },
    { 0x42832aeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32432aed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5032af5, SUC_lexicon_tags_62 },
    { 0xe2e32af5, SUC_lexicon_tags_26 },
    { 0x35c32af6, SUC_lexicon_tags_38 },
    { 0x75432af8, SUC_lexicon_tags_6 },
    { 0x6e032af8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9c32afb, SUC_lexicon_tags_25 },
    { 0x93a32afb, SUC_lexicon_tags_33 },
    { 0xb2832afd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2632b01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2a432b03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11432b08, SUC_lexicon_tags_38 },
    { 0x68432b08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e32b0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa32b0f, SUC_lexicon_tags_62 },
    { 0x4aa32b0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f232b16, SUC_lexicon_tags_387 },
    { 0x93032b16, SUC_lexicon_tags_38 },
    { 0xcee32b16, SUC_lexicon_tags_23 },
    { 0x47832b18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1632b1d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c32b22, SUC_lexicon_tags_46 },
    { 0x14032b23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9632b25, SUC_lexicon_tags_7 },
    { 0x2e032b26, SUC_lexicon_tags_17 },
    { 0xe1432b25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a32b2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82032b2e, SUC_lexicon_tags_30 },
    { 0x44232b2f, SUC_lexicon_tags_11 },
    { 0xea032b2f, SUC_lexicon_tags_33 },
    { 0x66832b2f, SUC_lexicon_tags_17 },
    { 0x8232b32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37632b35, SUC_lexicon_tags_23 },
    { 0x8632b36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x66432b39, SUC_lexicon_tags_19 },
    { 0x3fa32b3a, SUC_lexicon_tags_11 },
    { 0x6ee32b3a, SUC_lexicon_tags_60 },
    { 0xfc232b3a, SUC_lexicon_tags_15 },
    { 0x6a032b3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65832b42, SUC_lexicon_tags_129 },
    { 0xf9c32b43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb832b46, SUC_lexicon_tags_12 },
    { 0xad232b47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80032b49, SUC_lexicon_tags_19 },
    { 0x73832b49, SUC_lexicon_tags_12 },
    { 0x1f232b4b, SUC_lexicon_tags_1 },
    { 0x5232b4c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c832b52, SUC_lexicon_tags_11 },
    { 0x7432b53, SUC_lexicon_tags_12 },
    { 0xdc432b54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9832b5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3c32b5f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4432b6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4632b74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f632b7a, SUC_lexicon_tags_6 },
    { 0xbac32b7a, SUC_lexicon_tags_1 },
    { 0x8c232b7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44032b7f, SUC_lexicon_tags_2 },
    { 0x98432b80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e032b83, SUC_lexicon_tags_16 },
    { 0x8fc32b83, SUC_lexicon_tags_37 },
    { 0x5a232b85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee632b89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c232b8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e32b90, SUC_lexicon_tags_38 },
    { 0xe5832b90, SUC_lexicon_tags_172 },
    { 0x4c632b91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1a32b94, SUC_lexicon_tags_8 },
    { 0x87c32b94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31032b9a, SUC_lexicon_tags_11 },
    { 0xcac32b9a, SUC_lexicon_tags_7 },
    { 0x74a32b9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c32ba3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c232ba6, SUC_lexicon_tags_37 },
    { 0x3e832ba7, SUC_lexicon_tags_11 },
    { 0xdc832ba8, SUC_lexicon_tags_5 },
    { 0x3da32ba6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e32bad, SUC_lexicon_tags_12 },
    { 0x84a32bae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77832bb0, SUC_lexicon_tags_30 },
    { 0x32032bb0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4b632bb3, SUC_lexicon_tags_0 },
    { 0xdd032bb3, SUC_lexicon_tags_11 },
    { 0x6bc32bb5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb4432bb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f832bba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2be32bbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26832bc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c32bc8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67232bcc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8e432bce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e432bd1, SUC_lexicon_tags_26 },
    { 0x92a32bd1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c32bd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54232bdf, SUC_lexicon_tags_151 },
    { 0x24832be0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a32be4, SUC_lexicon_tags_15 },
    { 0xaa232be5, SUC_lexicon_tags_11 },
    { 0x91032be6, SUC_lexicon_tags_11 },
    { 0x14432be7, SUC_lexicon_tags_11 },
    { 0xad832be8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d632beb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf0c32bed, SUC_lexicon_tags_15 },
    { 0x65632bee, SUC_lexicon_tags_1 },
    { 0xa1032bee, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e432bf2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1032bf4, SUC_lexicon_tags_29 },
    { 0x62832bf4, SUC_lexicon_tags_2 },
    { 0x49232bf6, SUC_lexicon_tags_1 },
    { 0x63832bf7, SUC_lexicon_tags_0 },
    { 0x97432bf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe032bfe, SUC_lexicon_tags_15 },
    { 0xece32bff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a032c03, SUC_lexicon_tags_13 },
    { 0x3e232c04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a32c0d, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78232c11, SUC_lexicon_tags_47 },
    { 0xe8232c12, SUC_lexicon_tags_0 },
    { 0x29432c13, SUC_lexicon_tags_33 },
    { 0x64032c14, SUC_lexicon_tags_65 },
    { 0xb6c32c13, SUC_lexicon_tags_45 },
    { 0xd4032c16, SUC_lexicon_tags_39 },
    { 0x4aa32c17, SUC_lexicon_tags_27 },
    { 0xb432c17, SUC_lexicon_tags_29 },
    { 0xb6432c14, SUC_lexicon_tags_39 },
    { 0xf3432c1a, SUC_lexicon_tags_11 },
    { 0xa9632c1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e32c20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb432c25, SUC_lexicon_tags_46 },
    { 0xc3e32c25, SUC_lexicon_tags_30 },
    { 0xc7a32c25, SUC_lexicon_tags_2 },
    { 0xe4832c28, SUC_lexicon_tags_13 },
    { 0x8d232c29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x87632c2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a32c30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e232c34, SUC_lexicon_tags_19 },
    { 0x7d432c35, SUC_lexicon_tags_6 },
    { 0x5a432c34, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x34e32c38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e32c3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14632c3d, SUC_lexicon_tags_39 },
    { 0xd6032c3d, SUC_lexicon_tags_12 },
    { 0x19032c3f, SUC_lexicon_tags_11 },
    { 0x7e632c40, SUC_lexicon_tags_37 },
    { 0xf8832c41, SUC_lexicon_tags_60 },
    { 0xece32c41, SUC_lexicon_tags_12 },
    { 0x84032c43, SUC_lexicon_tags_1 },
    { 0x92c32c3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35632c4d, SUC_lexicon_tags_31 },
    { 0x6b232c4e, SUC_lexicon_tags_11 },
    { 0x29032c4f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa32c56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b832c58, SUC_lexicon_tags_25 },
    { 0xfaa32c59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2632c5d, SUC_lexicon_tags_38 },
    { 0xd0c32c5d, SUC_lexicon_tags_79 },
    { 0xb4832c5f, SUC_lexicon_tags_27 },
    { 0xf3632c60, SUC_lexicon_tags_0 },
    { 0xb432c61, SUC_lexicon_tags_23 },
    { 0xb5c32c61, SUC_lexicon_tags_49 },
    { 0xd4c32c5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5632c69, SUC_lexicon_tags_1 },
    { 0x8c432c6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa432c6c, SUC_lexicon_tags_49 },
    { 0x83a32c6d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x98432c70, SUC_lexicon_tags_5 },
    { 0xe4632c71, SUC_lexicon_tags_11 },
    { 0xe2632c72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3232c76, SUC_lexicon_tags_14 },
    { 0x3d032c77, SUC_lexicon_tags_11 },
    { 0x9de32c78, SUC_lexicon_tags_60 },
    { 0xcd232c78, SUC_lexicon_tags_12 },
    { 0x37632c7a, SUC_lexicon_tags_23 },
    { 0x9032c7b, SUC_lexicon_tags_25 },
    { 0xa5e32c7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbe32c7e, SUC_lexicon_tags_11 },
    { 0xdb432c7f, SUC_lexicon_tags_19 },
    { 0x86832c80, SUC_lexicon_tags_11 },
    { 0xb0832c7e, SUC_lexicon_tags_19 },
    { 0x3f032c82, SUC_lexicon_tags_37 },
    { 0x8032c82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69832c85, SUC_lexicon_tags_19 },
    { 0xc2a32c85, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d832c8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86432c8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48432c8e, SUC_lexicon_tags_66 },
    { 0xa8432c8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e032c95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8632c9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d432ca0, SUC_lexicon_tags_27 },
    { 0x5cc32ca0, SUC_lexicon_tags_1 },
    { 0x73032ca2, SUC_lexicon_tags_25 },
    { 0xdda32ca1, SUC_lexicon_tags_17 },
    { 0x52e32ca4, SUC_lexicon_tags_17 },
    { 0x9d832ca0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb0632ca7, SUC_lexicon_tags_26 },
    { 0xfde32ca8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a32cac, SUC_lexicon_tags_25 },
    { 0xc3232cac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x99a32caf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad832cb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f832cb7, SUC_lexicon_tags_1470 },
    { 0, NULL },
    { 0xde032cb9, SUC_lexicon_tags_97 },
    { 0x27632cba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc6e32cbc, SUC_lexicon_tags_17 },
    { 0x74632cbd, SUC_lexicon_tags_1 },
    { 0x68432cbc, SUC_lexicon_tags_47 },
    { 0xa2232cbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6232cc4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e32cc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10232cca, SUC_lexicon_tags_11 },
    { 0x3de32cca, SUC_lexicon_tags_17 },
    { 0x79032ccb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a32cd0, SUC_lexicon_tags_1 },
    { 0xda432cd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x98032cd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1232cd8, SUC_lexicon_tags_60 },
    { 0xd9032cd9, SUC_lexicon_tags_33 },
    { 0x75832cda, SUC_lexicon_tags_15 },
    { 0x8d032cda, SUC_lexicon_tags_21 },
    { 0xc0232cda, SUC_lexicon_tags_11 },
    { 0xf4e32cda, SUC_lexicon_tags_2 },
    { 0xa9432cde, SUC_lexicon_tags_27 },
    { 0xc4232cdc, SUC_lexicon_tags_1 },
    { 0x7f432ce0, SUC_lexicon_tags_14 },
    { 0x61c32ce0, SUC_lexicon_tags_21 },
    { 0x11032ce2, SUC_lexicon_tags_11 },
    { 0x4e32ce1, SUC_lexicon_tags_13 },
    { 0xa7c32ce1, SUC_lexicon_tags_29 },
    { 0xfb232cdf, SUC_lexicon_tags_47 },
    { 0x2d232cdd, SUC_lexicon_tags_15 },
    { 0x5632ce7, SUC_lexicon_tags_38 },
    { 0x22632ce7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2ac32cea, SUC_lexicon_tags_25 },
    { 0x7832ceb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74a32ced, SUC_lexicon_tags_26 },
    { 0xd8032ced, SUC_lexicon_tags_17 },
    { 0x60632cee, SUC_lexicon_tags_37 },
    { 0xda432cf0, SUC_lexicon_tags_63 },
    { 0xb8032cf1, SUC_lexicon_tags_45 },
    { 0x23832cf0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb232cf5, SUC_lexicon_tags_26 },
    { 0xfb032cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1a32cf8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd632cfb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb5e32cfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcc32cff, SUC_lexicon_tags_15 },
    { 0x8b832d00, SUC_lexicon_tags_29 },
    { 0x33a32d00, SUC_lexicon_tags_17 },
    { 0xdac32d02, SUC_lexicon_tags_17 },
    { 0xb1e32d03, SUC_lexicon_tags_23 },
    { 0xe7e32d03, SUC_lexicon_tags_60 },
    { 0x83432d05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13c32d07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92432d09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf6c32d0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce32d0e, SUC_lexicon_tags_3 },
    { 0x45032d0f, SUC_lexicon_tags_0 },
    { 0x26e32d0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c32d16, SUC_lexicon_tags_45 },
    { 0x9d432d16, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xb5e32d19, SUC_lexicon_tags_45 },
    { 0xd8632d1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c432d1e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4232d23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e32d2f, SUC_lexicon_tags_2 },
    { 0xa4a32d30, SUC_lexicon_tags_27 },
    { 0x68432d30, SUC_lexicon_tags_60 },
    { 0xc6e32d32, SUC_lexicon_tags_17 },
    { 0x38832d33, SUC_lexicon_tags_25 },
    { 0xfc032d31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c32d3c, SUC_lexicon_tags_31 },
    { 0xa4032d3d, SUC_lexicon_tags_17 },
    { 0xe4632d3d, SUC_lexicon_tags_2 },
    { 0xb7632d3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf232d45, SUC_lexicon_tags_37 },
    { 0xce32d46, SUC_lexicon_tags_47 },
    { 0x7d632d46, SUC_lexicon_tags_1 },
    { 0x72632d46, SUC_lexicon_tags_12 },
    { 0xa5232d49, SUC_lexicon_tags_63 },
    { 0x64c32d45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3632d4f, SUC_lexicon_tags_88 },
    { 0x4c632d50, SUC_lexicon_tags_11 },
    { 0xbe632d4f, SUC_lexicon_tags_37 },
    { 0xae632d52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e32d55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x81432d58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e32d60, SUC_lexicon_tags_0 },
    { 0x5f632d60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e032d66, SUC_lexicon_tags_8 },
    { 0x21832d67, SUC_lexicon_tags_17 },
    { 0x71a32d67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91632d6a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x10e32d6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada32d70, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6632d74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91c32d76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2232d7c, SUC_lexicon_tags_12 },
    { 0x48832d7d, SUC_lexicon_tags_37 },
    { 0x58232d7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a632d81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f032d85, SUC_lexicon_tags_33 },
    { 0xcd232d86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38432d8a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc9432d8c, SUC_lexicon_tags_0 },
    { 0xaa632d8d, SUC_lexicon_tags_69 },
    { 0x7f832d8e, SUC_lexicon_tags_2 },
    { 0xf4832d8c, SUC_lexicon_tags_19 },
    { 0x1c832d90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b832d9d, SUC_lexicon_tags_17 },
    { 0xb5632d9e, SUC_lexicon_tags_37 },
    { 0xe4632d9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6832da1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf2632da3, SUC_lexicon_tags_7 },
    { 0x29c32da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5632da7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e32daa, SUC_lexicon_tags_37 },
    { 0x85c32dab, SUC_lexicon_tags_34 },
    { 0x40432dac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2c32dae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d432db0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e432db2, SUC_lexicon_tags_7 },
    { 0xd4232db3, SUC_lexicon_tags_49 },
    { 0xa5832db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e32dbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6232dbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fe32dbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea32dc6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc32dc9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xac032dcb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb432dd1, SUC_lexicon_tags_1 },
    { 0x57a32dd2, SUC_lexicon_tags_29 },
    { 0xdd632dd3, SUC_lexicon_tags_8 },
    { 0x54232dd4, SUC_lexicon_tags_2 },
    { 0xeae32dd4, SUC_lexicon_tags_30 },
    { 0x94232dd6, SUC_lexicon_tags_2 },
    { 0xdd632dd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda32dda, SUC_lexicon_tags_210 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a32ddd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0632de6, SUC_lexicon_tags_8 },
    { 0x88432de7, SUC_lexicon_tags_25 },
    { 0xb0032de8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x44432dea, SUC_lexicon_tags_5 },
    { 0x2c032deb, SUC_lexicon_tags_43 },
    { 0xa3832dea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74432dee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4232df6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc632df8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x95832dfa, SUC_lexicon_tags_8 },
    { 0x63832dfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31632dfe, SUC_lexicon_tags_55 },
    { 0xd9032dfe, SUC_lexicon_tags_15 },
    { 0x1ae32e00, SUC_lexicon_tags_8 },
    { 0x1d432dff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a32e05, SUC_lexicon_tags_7 },
    { 0xfe832e05, SUC_lexicon_tags_68 },
    { 0xc5632e06, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf1c32e09, SUC_lexicon_tags_29 },
    { 0xebe32e0a, SUC_lexicon_tags_11 },
    { 0x79032e0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83632e0d, SUC_lexicon_tags_31 },
    { 0xf7a32e0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c432e12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7032e16, SUC_lexicon_tags_14 },
    { 0xf6e32e17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9032e19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x66232e1c, SUC_lexicon_tags_17 },
    { 0x16232e1d, SUC_lexicon_tags_15 },
    { 0x6ba32e1d, SUC_lexicon_tags_20 },
    { 0x4fa32e1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17632e23, SUC_lexicon_tags_0 },
    { 0x3ca32e24, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9be32e26, SUC_lexicon_tags_7 },
    { 0xaf232e27, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1032e2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53232e30, SUC_lexicon_tags_31 },
    { 0x59832e31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7832e33, SUC_lexicon_tags_5 },
    { 0xe9e32e34, SUC_lexicon_tags_25 },
    { 0xc3e32e35, SUC_lexicon_tags_49 },
    { 0x21632e35, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0832e39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65432e3c, SUC_lexicon_tags_60 },
    { 0x4e832e3d, SUC_lexicon_tags_39 },
    { 0xaa832e3c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29432e42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4a832e44, SUC_lexicon_tags_13 },
    { 0x25c32e45, SUC_lexicon_tags_16 },
    { 0xf4632e45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec432e4a, SUC_lexicon_tags_12 },
    { 0xe7232e4b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8832e4e, SUC_lexicon_tags_201 },
    { 0x1ce32e4f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc2832e51, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa6c32e53, SUC_lexicon_tags_162 },
    { 0x2a32e54, SUC_lexicon_tags_47 },
    { 0x6da32e54, SUC_lexicon_tags_11 },
    { 0xde632e54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16232e58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56a32e5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e32e5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6232e60, SUC_lexicon_tags_1 },
    { 0x75432e61, SUC_lexicon_tags_10 },
    { 0x57832e62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98832e68, SUC_lexicon_tags_38 },
    { 0xcb032e68, SUC_lexicon_tags_23 },
    { 0xace32e68, SUC_lexicon_tags_15 },
    { 0xb3a32e68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e32e6e, SUC_lexicon_tags_7 },
    { 0x81e32e6e, SUC_lexicon_tags_2 },
    { 0xc4a32e70, SUC_lexicon_tags_55 },
    { 0x6e032e71, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9632e76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x232e79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a632e7f, SUC_lexicon_tags_2 },
    { 0xc6632e7f, SUC_lexicon_tags_56 },
    { 0x65232e81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c32e88, SUC_lexicon_tags_10 },
    { 0x11c32e89, SUC_lexicon_tags_25 },
    { 0xdea32e8a, SUC_lexicon_tags_14 },
    { 0x94c32e8b, SUC_lexicon_tags_39 },
    { 0xf8432e8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c32e8f, SUC_lexicon_tags_37 },
    { 0xe8232e90, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46232e94, SUC_lexicon_tags_14 },
    { 0xd432e95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c032e99, SUC_lexicon_tags_10 },
    { 0x19832e9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a32ea3, SUC_lexicon_tags_65 },
    { 0x3be32ea3, SUC_lexicon_tags_37 },
    { 0xae232ea5, SUC_lexicon_tags_28 },
    { 0x7c432ea3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1432eac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4ee32eae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17432eb0, SUC_lexicon_tags_38 },
    { 0xbac32eb1, SUC_lexicon_tags_60 },
    { 0x84e32eb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3232eb5, SUC_lexicon_tags_7 },
    { 0xfe032eb6, SUC_lexicon_tags_7 },
    { 0x47e32eb6, SUC_lexicon_tags_63 },
    { 0x5b832eb8, SUC_lexicon_tags_37 },
    { 0xae032eb9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60632ec0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e32ec4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f632ec9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b432ece, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fa32ed0, SUC_lexicon_tags_2 },
    { 0x38a32ed1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87032ed4, SUC_lexicon_tags_0 },
    { 0x93c32ed5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d032eda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff432edd, SUC_lexicon_tags_68 },
    { 0x10632ede, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x39e32ee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53632ee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66032ee7, SUC_lexicon_tags_2 },
    { 0xf6632ee8, SUC_lexicon_tags_27 },
    { 0xc1432ee8, SUC_lexicon_tags_65 },
    { 0x7b232eea, SUC_lexicon_tags_39 },
    { 0x92232ee9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c32eee, SUC_lexicon_tags_45 },
    { 0x75e32eef, SUC_lexicon_tags_7 },
    { 0x6c32eee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69232ef4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58232ef6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef232eff, SUC_lexicon_tags_38 },
    { 0xc9a32f00, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5c432f02, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a32f08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9432f0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x92c32f11, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae32f14, SUC_lexicon_tags_48 },
    { 0x76032f15, SUC_lexicon_tags_1 },
    { 0xa1232f15, SUC_lexicon_tags_7 },
    { 0xa6a32f17, SUC_lexicon_tags_136 },
    { 0x7cc32f17, SUC_lexicon_tags_37 },
    { 0x81e32f19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0032f1f, SUC_lexicon_tags_40 },
    { 0xf6432f20, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x11032f23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae32f26, SUC_lexicon_tags_29 },
    { 0xb2a32f26, SUC_lexicon_tags_68 },
    { 0xef032f28, SUC_lexicon_tags_17 },
    { 0x70e32f28, SUC_lexicon_tags_63 },
    { 0xee432f27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f432f2c, SUC_lexicon_tags_30 },
    { 0xd9232f2d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x12432f30, SUC_lexicon_tags_28 },
    { 0x87e32f31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c32f35, SUC_lexicon_tags_38 },
    { 0x22832f36, SUC_lexicon_tags_2 },
    { 0xe1832f37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6632f3c, SUC_lexicon_tags_25 },
    { 0xf4232f3c, SUC_lexicon_tags_49 },
    { 0x96632f3c, SUC_lexicon_tags_0 },
    { 0x4cc32f3e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x74632f41, SUC_lexicon_tags_38 },
    { 0x53c32f42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae32f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde32f4d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe0232f4f, SUC_lexicon_tags_2 },
    { 0x63632f4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a32f53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea032f55, SUC_lexicon_tags_15 },
    { 0xbea32f56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89832f5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58632f5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fe32f61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e632f63, SUC_lexicon_tags_25 },
    { 0x7a632f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a32f6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e32f6d, SUC_lexicon_tags_14 },
    { 0x98c32f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c32f72, SUC_lexicon_tags_0 },
    { 0x52c32f72, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x80c32f75, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0632f7b, SUC_lexicon_tags_1 },
    { 0xb4e32f7b, SUC_lexicon_tags_30 },
    { 0xae32f7d, SUC_lexicon_tags_38 },
    { 0x4c632f7d, SUC_lexicon_tags_19 },
    { 0xa6c32f7f, SUC_lexicon_tags_16 },
    { 0xb1c32f7f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf8432f82, SUC_lexicon_tags_23 },
    { 0x9a432f83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc32f88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcac32f8a, SUC_lexicon_tags_2 },
    { 0xf2a32f8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c32f8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ea32f90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59632f92, SUC_lexicon_tags_12 },
    { 0x24632f93, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xa5432f95, SUC_lexicon_tags_2 },
    { 0x8a432f96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c032f9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c32fa2, SUC_lexicon_tags_30 },
    { 0x65432fa2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe0a32fa5, SUC_lexicon_tags_37 },
    { 0x74a32fa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e32fac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36232fb1, SUC_lexicon_tags_29 },
    { 0xd1a32fb1, SUC_lexicon_tags_27 },
    { 0x5632fb3, SUC_lexicon_tags_61 },
    { 0x2aa32fb2, SUC_lexicon_tags_49 },
    { 0x5fe32fb2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x56232fb7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x94c32fb9, SUC_lexicon_tags_17 },
    { 0x28032fb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88432fc0, SUC_lexicon_tags_29 },
    { 0x5ca32fc0, SUC_lexicon_tags_45 },
    { 0xe832fc2, SUC_lexicon_tags_43 },
    { 0x78c32fc3, SUC_lexicon_tags_16 },
    { 0xa6a32fc4, SUC_lexicon_tags_8 },
    { 0xde032fc5, SUC_lexicon_tags_47 },
    { 0xcdc32fc0, SUC_lexicon_tags_23 },
    { 0xdc032fc7, SUC_lexicon_tags_100 },
    { 0xda232fc7, SUC_lexicon_tags_17 },
    { 0x39432fc7, SUC_lexicon_tags_118 },
    { 0x39832fc9, SUC_lexicon_tags_2 },
    { 0x38032fcb, SUC_lexicon_tags_4 },
    { 0x82e32fc2, SUC_lexicon_tags_37 },
    { 0x8ae32fcd, SUC_lexicon_tags_30 },
    { 0x21832fcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf232fd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4432fd2, SUC_lexicon_tags_1 },
    { 0x87032fd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde032fd5, SUC_lexicon_tags_29 },
    { 0x39032fd5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e32fda, SUC_lexicon_tags_11 },
    { 0xca32fda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6432fdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6232fe0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc3e32fe2, SUC_lexicon_tags_48 },
    { 0x59632fe3, SUC_lexicon_tags_17 },
    { 0xe5e32fe4, SUC_lexicon_tags_29 },
    { 0xe032fe5, SUC_lexicon_tags_45 },
    { 0x24632fe6, SUC_lexicon_tags_38 },
    { 0x50232fe7, SUC_lexicon_tags_27 },
    { 0x21432fe8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee432feb, SUC_lexicon_tags_14 },
    { 0x77232fec, SUC_lexicon_tags_15 },
    { 0x48e32fec, SUC_lexicon_tags_60 },
    { 0x5232fee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ec32ff0, SUC_lexicon_tags_7 },
    { 0xef032ff1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeea32ff3, SUC_lexicon_tags_17 },
    { 0x28032ff3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78032ff8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1432ffa, SUC_lexicon_tags_118 },
    { 0x2ba32ffb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9033002, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36233004, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff83300d, SUC_lexicon_tags_14 },
    { 0x57e3300e, SUC_lexicon_tags_8 },
    { 0x61c3300f, SUC_lexicon_tags_66 },
    { 0xdf833010, SUC_lexicon_tags_26 },
    { 0xad033011, SUC_lexicon_tags_45 },
    { 0xf5c3300e, SUC_lexicon_tags_11 },
    { 0x58e33013, SUC_lexicon_tags_11 },
    { 0x55e33013, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb033016, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa3301a, SUC_lexicon_tags_37 },
    { 0xb323301b, SUC_lexicon_tags_27 },
    { 0x8b43301c, SUC_lexicon_tags_37 },
    { 0x9e83301d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a33020, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd433022, SUC_lexicon_tags_13 },
    { 0xb7c33022, SUC_lexicon_tags_1412 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77433027, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd83302f, SUC_lexicon_tags_35 },
    { 0x8a833030, SUC_lexicon_tags_25 },
    { 0xa823302f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c3303b, SUC_lexicon_tags_47 },
    { 0x3ba3303c, SUC_lexicon_tags_12 },
    { 0x2043303d, SUC_lexicon_tags_50 },
    { 0xbd63303e, SUC_lexicon_tags_43 },
    { 0x2ea3303c, SUC_lexicon_tags_12 },
    { 0x1d63303d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe33045, SUC_lexicon_tags_188 },
    { 0xdfa33046, SUC_lexicon_tags_29 },
    { 0x4833046, SUC_lexicon_tags_8 },
    { 0x35633047, SUC_lexicon_tags_11 },
    { 0x8be33045, SUC_lexicon_tags_17 },
    { 0x3c3304a, SUC_lexicon_tags_17 },
    { 0x89c3304b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9043304d, SUC_lexicon_tags_45 },
    { 0x9823304d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e33053, SUC_lexicon_tags_87 },
    { 0xe5c33054, SUC_lexicon_tags_17 },
    { 0x8a33054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c33058, SUC_lexicon_tags_25 },
    { 0x29633059, SUC_lexicon_tags_11 },
    { 0x3a33059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45e3305c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc143305e, SUC_lexicon_tags_17 },
    { 0x6503305f, SUC_lexicon_tags_19 },
    { 0xd0233060, SUC_lexicon_tags_12 },
    { 0x80e33061, SUC_lexicon_tags_37 },
    { 0x76c33062, SUC_lexicon_tags_66 },
    { 0x43033063, SUC_lexicon_tags_25 },
    { 0x99a33062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c33067, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403306b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e33073, SUC_lexicon_tags_1 },
    { 0x9c433074, SUC_lexicon_tags_26 },
    { 0x8dc33074, SUC_lexicon_tags_17 },
    { 0xfb233076, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c3307e, SUC_lexicon_tags_1 },
    { 0x1723307e, SUC_lexicon_tags_48 },
    { 0x80e3307e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c3308b, SUC_lexicon_tags_47 },
    { 0xe003308b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc803308e, SUC_lexicon_tags_25 },
    { 0x21e3308f, SUC_lexicon_tags_12 },
    { 0xa43308f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c33095, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1633098, SUC_lexicon_tags_37 },
    { 0xc1233099, SUC_lexicon_tags_14 },
    { 0xd7a3309a, SUC_lexicon_tags_17 },
    { 0x4703309a, SUC_lexicon_tags_1 },
    { 0x90233098, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa83309e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf40330a0, SUC_lexicon_tags_38 },
    { 0xcde330a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c330a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8330a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51e330ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2330b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6330b3, SUC_lexicon_tags_75 },
    { 0xe12330b3, SUC_lexicon_tags_10 },
    { 0x9ce330b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6330b9, SUC_lexicon_tags_37 },
    { 0x96c330ba, SUC_lexicon_tags_6 },
    { 0x46c330ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4330c1, SUC_lexicon_tags_17 },
    { 0x320330c2, SUC_lexicon_tags_1 },
    { 0xa0330c2, SUC_lexicon_tags_17 },
    { 0xc88330c4, SUC_lexicon_tags_37 },
    { 0x992330c2, SUC_lexicon_tags_12 },
    { 0xbea330c6, SUC_lexicon_tags_13 },
    { 0x6a6330c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3fc330c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4330cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38330cf, SUC_lexicon_tags_25 },
    { 0x808330d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x182330d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8330d7, SUC_lexicon_tags_19 },
    { 0x306330d7, SUC_lexicon_tags_14 },
    { 0xfa4330d7, SUC_lexicon_tags_25 },
    { 0xf5c330da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a330e1, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0x38e330e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0330ed, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xca2330ef, SUC_lexicon_tags_47 },
    { 0x716330f0, SUC_lexicon_tags_61 },
    { 0xdf4330ef, SUC_lexicon_tags_44 },
    { 0x70c330f2, SUC_lexicon_tags_31 },
    { 0xe40330f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce330f8, SUC_lexicon_tags_12 },
    { 0x44a330f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96330ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c33102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec3310b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1c3310d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x703310f, SUC_lexicon_tags_62 },
    { 0x6523310f, SUC_lexicon_tags_49 },
    { 0x2b23310f, SUC_lexicon_tags_12 },
    { 0x5743310f, SUC_lexicon_tags_37 },
    { 0xfe33111, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67233115, SUC_lexicon_tags_37 },
    { 0x2233115, SUC_lexicon_tags_12 },
    { 0x1a433117, SUC_lexicon_tags_87 },
    { 0x35a33117, SUC_lexicon_tags_38 },
    { 0x80033119, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa5e3311b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3423311e, SUC_lexicon_tags_25 },
    { 0x81a3311f, SUC_lexicon_tags_48 },
    { 0x86433120, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba33123, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a3312d, SUC_lexicon_tags_45 },
    { 0x2663312e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x96033130, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba033134, SUC_lexicon_tags_31 },
    { 0x37a33135, SUC_lexicon_tags_17 },
    { 0x57433136, SUC_lexicon_tags_38 },
    { 0x6f433137, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c033139, SUC_lexicon_tags_151 },
    { 0xd4433139, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3383313d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e33142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e33146, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6ee33148, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4e3314a, SUC_lexicon_tags_19 },
    { 0x9e03314b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf83314d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24e3314f, SUC_lexicon_tags_25 },
    { 0x53c3314f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea33153, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a033156, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c833159, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xca83315c, SUC_lexicon_tags_1 },
    { 0xf843315d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf143315f, SUC_lexicon_tags_11 },
    { 0x5f233160, SUC_lexicon_tags_25 },
    { 0x5e033160, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62233167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11033169, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xac63316c, SUC_lexicon_tags_17 },
    { 0x4103316c, SUC_lexicon_tags_15 },
    { 0xa143316e, SUC_lexicon_tags_60 },
    { 0x7763316f, SUC_lexicon_tags_19 },
    { 0xe3233170, SUC_lexicon_tags_47 },
    { 0x9b033170, SUC_lexicon_tags_15 },
    { 0x28433172, SUC_lexicon_tags_17 },
    { 0x7a3316e, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xe2e33175, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c33178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a3317b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x71e3317d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x67233180, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcbe33182, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a33185, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdae33187, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a3318b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97033194, SUC_lexicon_tags_43 },
    { 0x2fc33195, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1703319a, SUC_lexicon_tags_70 },
    { 0xa743319a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370331a4, SUC_lexicon_tags_4 },
    { 0xa84331a4, SUC_lexicon_tags_27 },
    { 0xf7a331a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd2331a8, SUC_lexicon_tags_5 },
    { 0xbee331a8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe2331ab, SUC_lexicon_tags_17 },
    { 0xb4c331ac, SUC_lexicon_tags_63 },
    { 0xbbc331ad, SUC_lexicon_tags_2 },
    { 0x326331ae, SUC_lexicon_tags_193 },
    { 0xc74331af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf6331b1, SUC_lexicon_tags_12 },
    { 0xc52331b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8331b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e331b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6de331bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810331bf, SUC_lexicon_tags_28 },
    { 0xdea331bf, SUC_lexicon_tags_30 },
    { 0xa18331c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x700331c4, SUC_lexicon_tags_9 },
    { 0x956331c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a331c9, SUC_lexicon_tags_7 },
    { 0x574331ca, SUC_lexicon_tags_6 },
    { 0x50c331cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308331d2, SUC_lexicon_tags_11 },
    { 0x9e8331d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0331d6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x39a331d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a331dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4331df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60331e2, SUC_lexicon_tags_79 },
    { 0xb7c331e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2331e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0331ec, SUC_lexicon_tags_17 },
    { 0xdfc331ed, SUC_lexicon_tags_25 },
    { 0xb02331ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71a331f0, SUC_lexicon_tags_0 },
    { 0x77c331f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36e331f3, SUC_lexicon_tags_12 },
    { 0x7f6331f4, SUC_lexicon_tags_7 },
    { 0x1f0331f5, SUC_lexicon_tags_47 },
    { 0xb2c331f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20233200, SUC_lexicon_tags_19 },
    { 0x71633200, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c33204, SUC_lexicon_tags_25 },
    { 0x37433204, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a3320c, SUC_lexicon_tags_15 },
    { 0xbaa3320d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa4c3320f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1433216, SUC_lexicon_tags_14 },
    { 0xb2633217, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa3321b, SUC_lexicon_tags_8 },
    { 0xb463321c, SUC_lexicon_tags_11 },
    { 0xec3321b, SUC_lexicon_tags_23 },
    { 0x583321e, SUC_lexicon_tags_11 },
    { 0xbde3321e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46633224, SUC_lexicon_tags_113 },
    { 0x1bc33225, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c33229, SUC_lexicon_tags_45 },
    { 0xacc3322a, SUC_lexicon_tags_23 },
    { 0xa483322b, SUC_lexicon_tags_15 },
    { 0xd9a33229, SUC_lexicon_tags_32 },
    { 0x2a83322a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c833231, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c33235, SUC_lexicon_tags_37 },
    { 0x60e33236, SUC_lexicon_tags_11 },
    { 0x20c33237, SUC_lexicon_tags_17 },
    { 0xd7433235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9883323c, SUC_lexicon_tags_7 },
    { 0x6ac3323c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d033240, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0033243, SUC_lexicon_tags_7 },
    { 0x2f833243, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac433246, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c63324a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9033253, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf063325b, SUC_lexicon_tags_27 },
    { 0x8503325c, SUC_lexicon_tags_17 },
    { 0xc643325b, SUC_lexicon_tags_46 },
    { 0x34e3325d, SUC_lexicon_tags_1 },
    { 0xe683325c, SUC_lexicon_tags_31 },
    { 0x34a33260, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x24033263, SUC_lexicon_tags_8 },
    { 0xc9c33263, SUC_lexicon_tags_17 },
    { 0x6da33265, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc33268, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e03326c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d833270, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x68633272, SUC_lexicon_tags_6 },
    { 0xdfc33273, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x66233275, SUC_lexicon_tags_12 },
    { 0xacc33276, SUC_lexicon_tags_11 },
    { 0x89433276, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03327b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5043327f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67833285, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52833289, SUC_lexicon_tags_17 },
    { 0x9a833289, SUC_lexicon_tags_30 },
    { 0x4ec3328b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xae23328e, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e33296, SUC_lexicon_tags_11 },
    { 0x38633297, SUC_lexicon_tags_15 },
    { 0x6433297, SUC_lexicon_tags_1 },
    { 0xfe633299, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e332a2, SUC_lexicon_tags_14 },
    { 0x100332a3, SUC_lexicon_tags_15 },
    { 0xd82332a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4332a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8332ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xefe332af, SUC_lexicon_tags_17 },
    { 0x1ce332b0, SUC_lexicon_tags_17 },
    { 0x22a332b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4332b5, SUC_lexicon_tags_0 },
    { 0x55c332b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9aa332b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8332bc, SUC_lexicon_tags_37 },
    { 0x602332bd, SUC_lexicon_tags_37 },
    { 0xfde332bc, SUC_lexicon_tags_25 },
    { 0x58e332bf, SUC_lexicon_tags_2 },
    { 0xc68332bf, SUC_lexicon_tags_19 },
    { 0x32332c1, SUC_lexicon_tags_2 },
    { 0xa92332bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42332cc, SUC_lexicon_tags_21 },
    { 0x770332cc, SUC_lexicon_tags_11 },
    { 0xf1c332ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x510332d1, SUC_lexicon_tags_37 },
    { 0x44e332d2, SUC_lexicon_tags_43 },
    { 0x86e332d3, SUC_lexicon_tags_2 },
    { 0x1f0332d4, SUC_lexicon_tags_11 },
    { 0x22a332d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbfe332d7, SUC_lexicon_tags_17 },
    { 0xc7a332d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2fc332da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e332de, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358332e2, SUC_lexicon_tags_15 },
    { 0x9a6332e3, SUC_lexicon_tags_37 },
    { 0xcde332e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac332e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x788332ec, SUC_lexicon_tags_1 },
    { 0x8fe332ec, SUC_lexicon_tags_31 },
    { 0x1b2332ed, SUC_lexicon_tags_6 },
    { 0x4a6332ed, SUC_lexicon_tags_25 },
    { 0x142332ed, SUC_lexicon_tags_13 },
    { 0xdfa332ed, SUC_lexicon_tags_11 },
    { 0x2be332f2, SUC_lexicon_tags_17 },
    { 0x404332f2, SUC_lexicon_tags_0 },
    { 0x716332f1, SUC_lexicon_tags_12 },
    { 0xde8332ef, SUC_lexicon_tags_3 },
    { 0x8b6332f6, SUC_lexicon_tags_1 },
    { 0xd2a332f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984332fb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0332fe, SUC_lexicon_tags_11 },
    { 0xcc8332ff, SUC_lexicon_tags_220 },
    { 0xaca332ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1433302, SUC_lexicon_tags_3 },
    { 0xbb233302, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52833307, SUC_lexicon_tags_47 },
    { 0x6633308, SUC_lexicon_tags_23 },
    { 0x4d633308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe3330d, SUC_lexicon_tags_43 },
    { 0xba23330d, SUC_lexicon_tags_66 },
    { 0x8f43330f, SUC_lexicon_tags_2 },
    { 0xb0e3330f, SUC_lexicon_tags_47 },
    { 0x3c433311, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x60e33313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8033316, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0x30833318, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b63331d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de33323, SUC_lexicon_tags_35 },
    { 0x37633323, SUC_lexicon_tags_38 },
    { 0x8bc33325, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x25633328, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x7483332b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81233331, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97a33333, SUC_lexicon_tags_45 },
    { 0xbbc33334, SUC_lexicon_tags_1 },
    { 0x4a433334, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf833337, SUC_lexicon_tags_25 },
    { 0x18833338, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7f43333a, SUC_lexicon_tags_103 },
    { 0x1603333b, SUC_lexicon_tags_9 },
    { 0x69c3333c, SUC_lexicon_tags_55 },
    { 0xab03333a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x62e3333f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafe33341, SUC_lexicon_tags_30 },
    { 0x77c33342, SUC_lexicon_tags_23 },
    { 0x28a33343, SUC_lexicon_tags_2 },
    { 0xaf833342, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8633346, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaee33348, SUC_lexicon_tags_14 },
    { 0xb7a33348, SUC_lexicon_tags_18 },
    { 0xed233349, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee633351, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae33354, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcc833356, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3c633358, SUC_lexicon_tags_6 },
    { 0x8e33359, SUC_lexicon_tags_37 },
    { 0x38a33358, SUC_lexicon_tags_47 },
    { 0xd5c33358, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8d43335d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xece3335f, SUC_lexicon_tags_8 },
    { 0x96833360, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x8033363, SUC_lexicon_tags_0 },
    { 0x31233364, SUC_lexicon_tags_7 },
    { 0xe7c33365, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d633369, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7e3336b, SUC_lexicon_tags_25 },
    { 0xf303336b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6683336f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7c33371, SUC_lexicon_tags_17 },
    { 0xdd233372, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4be33374, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd633376, SUC_lexicon_tags_365 },
    { 0x12c33376, SUC_lexicon_tags_16 },
    { 0x9f433378, SUC_lexicon_tags_46 },
    { 0xd8033379, SUC_lexicon_tags_87 },
    { 0x9cc3337a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e3337c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f633380, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x77033382, SUC_lexicon_tags_69 },
    { 0xc8433383, SUC_lexicon_tags_30 },
    { 0x7d233384, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe2033386, SUC_lexicon_tags_19 },
    { 0x51433386, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fc33389, SUC_lexicon_tags_29 },
    { 0x57633389, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd63338d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d833391, SUC_lexicon_tags_39 },
    { 0xc5833392, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e33396, SUC_lexicon_tags_7 },
    { 0x1ec33397, SUC_lexicon_tags_11 },
    { 0x42633398, SUC_lexicon_tags_46 },
    { 0xc433399, SUC_lexicon_tags_2 },
    { 0xa863339a, SUC_lexicon_tags_15 },
    { 0xe923339b, SUC_lexicon_tags_7 },
    { 0x42a3339c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804333a0, SUC_lexicon_tags_11 },
    { 0x248333a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2aa333a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa36333a5, SUC_lexicon_tags_31 },
    { 0x38e333a6, SUC_lexicon_tags_45 },
    { 0x444333a7, SUC_lexicon_tags_23 },
    { 0xa1c333a8, SUC_lexicon_tags_14 },
    { 0x2c4333a7, SUC_lexicon_tags_17 },
    { 0x6c0333a8, SUC_lexicon_tags_15 },
    { 0xf32333ab, SUC_lexicon_tags_43 },
    { 0x762333ab, SUC_lexicon_tags_16 },
    { 0x1f6333ad, SUC_lexicon_tags_25 },
    { 0x236333ab, SUC_lexicon_tags_15 },
    { 0x70333af, SUC_lexicon_tags_47 },
    { 0x796333b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8333b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a333bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe72333be, SUC_lexicon_tags_63 },
    { 0x2333bf, SUC_lexicon_tags_7 },
    { 0x924333c0, SUC_lexicon_tags_65 },
    { 0xdc4333c1, SUC_lexicon_tags_47 },
    { 0xaf2333c2, SUC_lexicon_tags_25 },
    { 0x4bc333c0, SUC_lexicon_tags_2 },
    { 0x8d2333c4, SUC_lexicon_tags_37 },
    { 0x28c333c5, SUC_lexicon_tags_21 },
    { 0xf5e333c1, SUC_lexicon_tags_38 },
    { 0x18c333c7, SUC_lexicon_tags_25 },
    { 0xa7c333c7, SUC_lexicon_tags_2 },
    { 0xf5c333c4, SUC_lexicon_tags_66 },
    { 0xd32333ca, SUC_lexicon_tags_63 },
    { 0xfe6333cb, SUC_lexicon_tags_13 },
    { 0xbc2333ca, SUC_lexicon_tags_25 },
    { 0xe42333cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4333d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae333d5, SUC_lexicon_tags_17 },
    { 0xe72333d6, SUC_lexicon_tags_37 },
    { 0x9b2333d7, SUC_lexicon_tags_15 },
    { 0xbf2333d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384333dc, SUC_lexicon_tags_26 },
    { 0x5ec333dc, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xb06333df, SUC_lexicon_tags_39 },
    { 0x3fa333e0, SUC_lexicon_tags_11 },
    { 0x5ee333e1, SUC_lexicon_tags_47 },
    { 0xf40333e2, SUC_lexicon_tags_17 },
    { 0x81a333e3, SUC_lexicon_tags_38 },
    { 0x6dc333e2, SUC_lexicon_tags_17 },
    { 0xa2333e4, SUC_lexicon_tags_45 },
    { 0xaf8333e6, SUC_lexicon_tags_2 },
    { 0x43e333e2, SUC_lexicon_tags_8 },
    { 0xe74333e0, SUC_lexicon_tags_39 },
    { 0x51c333e8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf96333eb, SUC_lexicon_tags_47 },
    { 0xa2333eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6333f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x260333f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e333f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6333fd, SUC_lexicon_tags_35 },
    { 0xfc8333fe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e33402, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea233405, SUC_lexicon_tags_15 },
    { 0x61033405, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97233408, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e3340f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14233411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51a33413, SUC_lexicon_tags_2 },
    { 0x8a33413, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb9233416, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23341a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1a3341c, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xa623341e, SUC_lexicon_tags_12 },
    { 0xe163341f, SUC_lexicon_tags_42 },
    { 0xaf83341f, SUC_lexicon_tags_7 },
    { 0x5e83341e, SUC_lexicon_tags_63 },
    { 0x33421, SUC_lexicon_tags_47 },
    { 0xbdc33423, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x73233425, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e233427, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6c833429, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba3342c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2463342e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24233431, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee433433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81033435, SUC_lexicon_tags_108 },
    { 0x38833436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9c33438, SUC_lexicon_tags_46 },
    { 0xffc33439, SUC_lexicon_tags_25 },
    { 0xcc33439, SUC_lexicon_tags_1 },
    { 0xd743343b, SUC_lexicon_tags_43 },
    { 0x94833438, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc3343f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a833441, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c33444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa433446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21433448, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2543344b, SUC_lexicon_tags_25 },
    { 0x30e3344c, SUC_lexicon_tags_23 },
    { 0x2863344b, SUC_lexicon_tags_17 },
    { 0x3923344e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa33453, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a433456, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba3345c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7243345f, SUC_lexicon_tags_12 },
    { 0xc1c3345f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2633462, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41433466, SUC_lexicon_tags_1 },
    { 0xbb233466, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c03346a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a23346d, SUC_lexicon_tags_37 },
    { 0x4923346e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7633472, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a233478, SUC_lexicon_tags_60 },
    { 0xe4e33479, SUC_lexicon_tags_30 },
    { 0xec833478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa3347f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea633482, SUC_lexicon_tags_29 },
    { 0xffe33483, SUC_lexicon_tags_77 },
    { 0xe33483, SUC_lexicon_tags_37 },
    { 0x43233485, SUC_lexicon_tags_2 },
    { 0x3de33485, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0a33488, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4183348a, SUC_lexicon_tags_26 },
    { 0x87a3348a, SUC_lexicon_tags_7 },
    { 0x8da3348c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35633492, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3633496, SUC_lexicon_tags_17 },
    { 0x17c33497, SUC_lexicon_tags_27 },
    { 0x74233498, SUC_lexicon_tags_27 },
    { 0x1ac33498, SUC_lexicon_tags_11 },
    { 0x56c3349a, SUC_lexicon_tags_2 },
    { 0x7703349b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1423349e, SUC_lexicon_tags_27 },
    { 0x1c23349e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x936334a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558334af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c334b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc334b8, SUC_lexicon_tags_7 },
    { 0x65c334b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb4334bb, SUC_lexicon_tags_28 },
    { 0xbde334bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a334cc, SUC_lexicon_tags_47 },
    { 0xbd6334cc, SUC_lexicon_tags_9 },
    { 0x51e334ce, SUC_lexicon_tags_28 },
    { 0x236334ce, SUC_lexicon_tags_0 },
    { 0x170334d0, SUC_lexicon_tags_1 },
    { 0x840334d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c334d6, SUC_lexicon_tags_7 },
    { 0xe1a334d7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2334da, SUC_lexicon_tags_113 },
    { 0xd90334db, SUC_lexicon_tags_55 },
    { 0xc6e334dc, SUC_lexicon_tags_63 },
    { 0x3e334dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a4334df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be334e3, SUC_lexicon_tags_49 },
    { 0x9c334e3, SUC_lexicon_tags_37 },
    { 0xd88334e3, SUC_lexicon_tags_1 },
    { 0x5f4334e6, SUC_lexicon_tags_7 },
    { 0xa46334e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0334eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec334f0, SUC_lexicon_tags_1 },
    { 0x3ec334f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58334f4, SUC_lexicon_tags_88 },
    { 0xe88334f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5c334f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d6334f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x180334fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5e334fd, SUC_lexicon_tags_14 },
    { 0x824334fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa33505, SUC_lexicon_tags_15 },
    { 0xf5233506, SUC_lexicon_tags_69 },
    { 0x5c833506, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c3350a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc23350e, SUC_lexicon_tags_39 },
    { 0x8a23350f, SUC_lexicon_tags_45 },
    { 0xb633510, SUC_lexicon_tags_12 },
    { 0x2bc33510, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2833513, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa233519, SUC_lexicon_tags_21 },
    { 0xcce3351a, SUC_lexicon_tags_105 },
    { 0x2743351a, SUC_lexicon_tags_33 },
    { 0x71e3351c, SUC_lexicon_tags_12 },
    { 0xc423351c, SUC_lexicon_tags_12 },
    { 0x2503351c, SUC_lexicon_tags_6 },
    { 0x56c3351f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a33524, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28833528, SUC_lexicon_tags_28 },
    { 0x7aa33529, SUC_lexicon_tags_11 },
    { 0xa2833529, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd423352c, SUC_lexicon_tags_27 },
    { 0x7fc3352c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6883352f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a33535, SUC_lexicon_tags_39 },
    { 0x45c33536, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe483353a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5803353d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7583353f, SUC_lexicon_tags_11 },
    { 0xf563353f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x19e33542, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40833546, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83354c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc963354e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57033550, SUC_lexicon_tags_38 },
    { 0x8833551, SUC_lexicon_tags_8 },
    { 0x14233552, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47033554, SUC_lexicon_tags_15 },
    { 0x9a833555, SUC_lexicon_tags_47 },
    { 0x28a33556, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x72233559, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3303355c, SUC_lexicon_tags_2 },
    { 0x4b23355c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf403355f, SUC_lexicon_tags_32 },
    { 0x6123355f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c433563, SUC_lexicon_tags_29 },
    { 0xc6633564, SUC_lexicon_tags_145 },
    { 0x48033564, SUC_lexicon_tags_2 },
    { 0x1f233565, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x77633569, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd403356c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe383356e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x50633570, SUC_lexicon_tags_136 },
    { 0x31a33571, SUC_lexicon_tags_0 },
    { 0xce633572, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93c33574, SUC_lexicon_tags_23 },
    { 0x59633574, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa633578, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c633583, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a3358c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe033590, SUC_lexicon_tags_74 },
    { 0x7fc33591, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e33594, SUC_lexicon_tags_15 },
    { 0x8ce33594, SUC_lexicon_tags_11 },
    { 0x5ea33596, SUC_lexicon_tags_17 },
    { 0x5f033595, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1043359c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d23359f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94335a2, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8335a6, SUC_lexicon_tags_19 },
    { 0xb20335a6, SUC_lexicon_tags_47 },
    { 0x312335a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78335af, SUC_lexicon_tags_43 },
    { 0xffe335af, SUC_lexicon_tags_34 },
    { 0xbc0335b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe08335b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x656335b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc6335b7, SUC_lexicon_tags_43 },
    { 0x4de335b8, SUC_lexicon_tags_38 },
    { 0x4fe335b8, SUC_lexicon_tags_38 },
    { 0x852335b9, SUC_lexicon_tags_33 },
    { 0xda8335bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x356335bd, SUC_lexicon_tags_30 },
    { 0x42a335bd, SUC_lexicon_tags_29 },
    { 0x4b6335bf, SUC_lexicon_tags_25 },
    { 0xc34335be, SUC_lexicon_tags_23 },
    { 0x7fc335c1, SUC_lexicon_tags_87 },
    { 0xf64335bd, SUC_lexicon_tags_74 },
    { 0x8e6335c0, SUC_lexicon_tags_25 },
    { 0xd48335c4, SUC_lexicon_tags_2 },
    { 0x4e6335c5, SUC_lexicon_tags_26 },
    { 0x762335c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98335d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe335da, SUC_lexicon_tags_11 },
    { 0x430335db, SUC_lexicon_tags_12 },
    { 0xbcc335db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc20335de, SUC_lexicon_tags_113 },
    { 0x8fa335de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a335e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c335e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0335e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c335ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x772335f0, SUC_lexicon_tags_32 },
    { 0x224335f1, SUC_lexicon_tags_45 },
    { 0x182335f1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6335f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54335fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe233601, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf833603, SUC_lexicon_tags_4 },
    { 0x85233604, SUC_lexicon_tags_21 },
    { 0x31833604, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa33608, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb63360b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd563360f, SUC_lexicon_tags_17 },
    { 0x60433610, SUC_lexicon_tags_65 },
    { 0x7b833610, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd5033613, SUC_lexicon_tags_7 },
    { 0x33614, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2233619, SUC_lexicon_tags_37 },
    { 0x68c3361a, SUC_lexicon_tags_8 },
    { 0x76a3361a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3743361e, SUC_lexicon_tags_402 },
    { 0x1c3361e, SUC_lexicon_tags_38 },
    { 0x5e633620, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a33625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72233628, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2b63362a, SUC_lexicon_tags_25 },
    { 0x2123362b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d63362e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50633630, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87833634, SUC_lexicon_tags_27 },
    { 0x75e33634, SUC_lexicon_tags_33 },
    { 0xefc33635, SUC_lexicon_tags_152 },
    { 0xcd633637, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e43363c, SUC_lexicon_tags_7 },
    { 0x8763363d, SUC_lexicon_tags_19 },
    { 0x2d63363e, SUC_lexicon_tags_8 },
    { 0x61c3363f, SUC_lexicon_tags_12 },
    { 0xb2a33640, SUC_lexicon_tags_37 },
    { 0x97c33641, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3e33643, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2433650, SUC_lexicon_tags_179 },
    { 0xc5433650, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd0233653, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e3365b, SUC_lexicon_tags_7 },
    { 0xe423365c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb03365f, SUC_lexicon_tags_45 },
    { 0xa843365f, SUC_lexicon_tags_29 },
    { 0xb8e33660, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c33668, SUC_lexicon_tags_37 },
    { 0x15633669, SUC_lexicon_tags_25 },
    { 0xfe033669, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe603366d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8a03366f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee433672, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e033676, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c233678, SUC_lexicon_tags_88 },
    { 0x7a33679, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5063367b, SUC_lexicon_tags_12 },
    { 0x9d83367b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd833682, SUC_lexicon_tags_1 },
    { 0x5ec33683, SUC_lexicon_tags_32 },
    { 0x3f633684, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72033687, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x57833689, SUC_lexicon_tags_25 },
    { 0x8ae3368a, SUC_lexicon_tags_37 },
    { 0xea3368b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11633693, SUC_lexicon_tags_13 },
    { 0x67433694, SUC_lexicon_tags_65 },
    { 0x3c633695, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a633697, SUC_lexicon_tags_7 },
    { 0x25a33698, SUC_lexicon_tags_27 },
    { 0x2e833697, SUC_lexicon_tags_38 },
    { 0xbb633698, SUC_lexicon_tags_47 },
    { 0xd633697, SUC_lexicon_tags_6 },
    { 0xc433698, SUC_lexicon_tags_45 },
    { 0xbbe33699, SUC_lexicon_tags_17 },
    { 0xf7a3369e, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x4ac336a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308336a5, SUC_lexicon_tags_27 },
    { 0xa52336a6, SUC_lexicon_tags_17 },
    { 0x3b0336a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb2336a9, SUC_lexicon_tags_3 },
    { 0x2e4336aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x604336ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20336b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3c336b2, SUC_lexicon_tags_57 },
    { 0xd34336b3, SUC_lexicon_tags_9 },
    { 0x988336b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964336bb, SUC_lexicon_tags_11 },
    { 0x3ec336bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc336bf, SUC_lexicon_tags_6 },
    { 0xc7a336c0, SUC_lexicon_tags_37 },
    { 0x52336c1, SUC_lexicon_tags_3 },
    { 0x5ce336c1, SUC_lexicon_tags_17 },
    { 0x61c336c1, SUC_lexicon_tags_11 },
    { 0xfc2336c4, SUC_lexicon_tags_66 },
    { 0x28c336c5, SUC_lexicon_tags_63 },
    { 0x86336c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a336c9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0336ce, SUC_lexicon_tags_21 },
    { 0xc78336cf, SUC_lexicon_tags_8 },
    { 0x1fe336ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c336d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d6336d6, SUC_lexicon_tags_23 },
    { 0x2c8336d7, SUC_lexicon_tags_38 },
    { 0xd0c336d7, SUC_lexicon_tags_11 },
    { 0xf8e336d9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500336dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce336e2, SUC_lexicon_tags_47 },
    { 0xb28336e3, SUC_lexicon_tags_7 },
    { 0x7a6336e4, SUC_lexicon_tags_39 },
    { 0x462336e5, SUC_lexicon_tags_25 },
    { 0x84a336e6, SUC_lexicon_tags_11 },
    { 0xa72336e7, SUC_lexicon_tags_1 },
    { 0x8f2336e8, SUC_lexicon_tags_1 },
    { 0x854336e2, SUC_lexicon_tags_7 },
    { 0x3c6336ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82336ec, SUC_lexicon_tags_15 },
    { 0x6a336ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68336f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x640336f9, SUC_lexicon_tags_38 },
    { 0xccc336f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x822336fc, SUC_lexicon_tags_28 },
    { 0x204336fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1633707, SUC_lexicon_tags_80 },
    { 0xdc833707, SUC_lexicon_tags_45 },
    { 0x39233709, SUC_lexicon_tags_11 },
    { 0xe6033709, SUC_lexicon_tags_16 },
    { 0xfa23370b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x243370e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7833710, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade33719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7263371b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe83371f, SUC_lexicon_tags_25 },
    { 0xb4c3371f, SUC_lexicon_tags_17 },
    { 0x44c33721, SUC_lexicon_tags_12 },
    { 0x84c33720, SUC_lexicon_tags_2 },
    { 0xb233723, SUC_lexicon_tags_23 },
    { 0x8b233724, SUC_lexicon_tags_7 },
    { 0xb4433725, SUC_lexicon_tags_11 },
    { 0xeae33720, SUC_lexicon_tags_14 },
    { 0x4ec33727, SUC_lexicon_tags_10 },
    { 0x7ae33728, SUC_lexicon_tags_1 },
    { 0x1d233729, SUC_lexicon_tags_60 },
    { 0xcb03372a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xda43372c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a3372f, SUC_lexicon_tags_37 },
    { 0xc783372f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf4833732, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66233739, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d43373b, SUC_lexicon_tags_11 },
    { 0xe503373b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb83373f, SUC_lexicon_tags_2 },
    { 0x54c33740, SUC_lexicon_tags_15 },
    { 0x4c633741, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce633747, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x2f633749, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb63374e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33a33750, SUC_lexicon_tags_11 },
    { 0x1ee33750, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59433756, SUC_lexicon_tags_25 },
    { 0x38e33757, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x57033759, SUC_lexicon_tags_19 },
    { 0x79c33759, SUC_lexicon_tags_17 },
    { 0x1c03375a, SUC_lexicon_tags_11 },
    { 0xc783375c, SUC_lexicon_tags_7 },
    { 0xa063375d, SUC_lexicon_tags_39 },
    { 0x2fa3375e, SUC_lexicon_tags_29 },
    { 0x5d83375f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83833765, SUC_lexicon_tags_46 },
    { 0xa0033765, SUC_lexicon_tags_12 },
    { 0xf4033767, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a03376a, SUC_lexicon_tags_38 },
    { 0x6323376b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f23376e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a33772, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcc33774, SUC_lexicon_tags_23 },
    { 0xd0233774, SUC_lexicon_tags_45 },
    { 0x4bc33774, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc323377c, SUC_lexicon_tags_28 },
    { 0x1f03377d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f633783, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfa433785, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2833789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43378f, SUC_lexicon_tags_12 },
    { 0x8ac33790, SUC_lexicon_tags_45 },
    { 0x10433790, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c33794, SUC_lexicon_tags_17 },
    { 0x92833795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3e33797, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa0033799, SUC_lexicon_tags_2 },
    { 0xd4c33799, SUC_lexicon_tags_19 },
    { 0x743379b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0337a5, SUC_lexicon_tags_11 },
    { 0xde8337a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c337ab, SUC_lexicon_tags_2 },
    { 0xcda337ac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x54c337ae, SUC_lexicon_tags_45 },
    { 0x150337ae, SUC_lexicon_tags_11 },
    { 0xdca337b0, SUC_lexicon_tags_2 },
    { 0x270337b1, SUC_lexicon_tags_11 },
    { 0x12a337b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2337b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe88337b8, SUC_lexicon_tags_31 },
    { 0x6ec337b9, SUC_lexicon_tags_23 },
    { 0x728337ba, SUC_lexicon_tags_17 },
    { 0x69c337b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2337bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be337c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c337c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a337cc, SUC_lexicon_tags_17 },
    { 0x928337cd, SUC_lexicon_tags_146 },
    { 0x9ce337cd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0337d4, SUC_lexicon_tags_47 },
    { 0x8d2337d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a4337d7, SUC_lexicon_tags_1 },
    { 0xe60337d8, SUC_lexicon_tags_9 },
    { 0xe26337d9, SUC_lexicon_tags_25 },
    { 0xa9e337da, SUC_lexicon_tags_17 },
    { 0xb7c337db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae337e2, SUC_lexicon_tags_33 },
    { 0xb9c337e2, SUC_lexicon_tags_55 },
    { 0x34e337e4, SUC_lexicon_tags_2 },
    { 0x398337e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x70a337e7, SUC_lexicon_tags_38 },
    { 0xac0337e8, SUC_lexicon_tags_82 },
    { 0xf24337e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x804337ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf4337ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x440337f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46337f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9f2337f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea337fc, SUC_lexicon_tags_1 },
    { 0x312337fc, SUC_lexicon_tags_7 },
    { 0xffc337fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4633800, SUC_lexicon_tags_11 },
    { 0x9a633800, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7aa33803, SUC_lexicon_tags_38 },
    { 0x4233804, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da33807, SUC_lexicon_tags_30 },
    { 0xfba33807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4c3380a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6123380f, SUC_lexicon_tags_14 },
    { 0x3bc33810, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf4633812, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32633817, SUC_lexicon_tags_0 },
    { 0xf6c33818, SUC_lexicon_tags_14 },
    { 0x3b033819, SUC_lexicon_tags_15 },
    { 0x93c33819, SUC_lexicon_tags_11 },
    { 0xba43381b, SUC_lexicon_tags_37 },
    { 0x5903381c, SUC_lexicon_tags_2 },
    { 0xc303381d, SUC_lexicon_tags_17 },
    { 0x9643381b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c33821, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1033825, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e33829, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f43382b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3143382e, SUC_lexicon_tags_2 },
    { 0x5e3382f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b233831, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32233836, SUC_lexicon_tags_17 },
    { 0x30633836, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a3383a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd343383d, SUC_lexicon_tags_17 },
    { 0x2ee3383e, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb633841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1033844, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6433847, SUC_lexicon_tags_23 },
    { 0xe4833848, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac3384c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a633851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b833854, SUC_lexicon_tags_6 },
    { 0x1c833854, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0e33857, SUC_lexicon_tags_7 },
    { 0xca633858, SUC_lexicon_tags_2 },
    { 0x83433859, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1743385b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e33867, SUC_lexicon_tags_224 },
    { 0x94a33868, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba3386e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62233870, SUC_lexicon_tags_17 },
    { 0x3c633871, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21833873, SUC_lexicon_tags_25 },
    { 0xc4833874, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x28633876, SUC_lexicon_tags_38 },
    { 0x4de33877, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7143387a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e83387d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9633880, SUC_lexicon_tags_9 },
    { 0x1233881, SUC_lexicon_tags_11 },
    { 0xb5633882, SUC_lexicon_tags_55 },
    { 0x2c833882, SUC_lexicon_tags_11 },
    { 0x41433883, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1033887, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94e33889, SUC_lexicon_tags_1 },
    { 0xf5e33889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43388f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c33894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d233896, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d23389a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e338a1, SUC_lexicon_tags_37 },
    { 0x26a338a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88338a5, SUC_lexicon_tags_206 },
    { 0x2d0338a6, SUC_lexicon_tags_27 },
    { 0x8ba338a6, SUC_lexicon_tags_17 },
    { 0x24c338a8, SUC_lexicon_tags_1 },
    { 0x962338a9, SUC_lexicon_tags_15 },
    { 0x7cc338a9, SUC_lexicon_tags_41 },
    { 0xa44338a9, SUC_lexicon_tags_17 },
    { 0xbb4338a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2338af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12338b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0338b5, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44338bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2338c0, SUC_lexicon_tags_33 },
    { 0xe40338c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2338c4, SUC_lexicon_tags_17 },
    { 0x980338c4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa6e338c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806338cb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca338d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716338d9, SUC_lexicon_tags_63 },
    { 0xfbe338d9, SUC_lexicon_tags_27 },
    { 0x378338db, SUC_lexicon_tags_37 },
    { 0x73a338da, SUC_lexicon_tags_3 },
    { 0xda0338db, SUC_lexicon_tags_18 },
    { 0xe84338de, SUC_lexicon_tags_39 },
    { 0x16e338df, SUC_lexicon_tags_55 },
    { 0xd96338d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a338e2, SUC_lexicon_tags_12 },
    { 0xb08338e2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8338e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92338ed, SUC_lexicon_tags_17 },
    { 0x6f8338ee, SUC_lexicon_tags_4 },
    { 0x45e338ee, SUC_lexicon_tags_14 },
    { 0x69a338ee, SUC_lexicon_tags_43 },
    { 0x28338f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x592338f4, SUC_lexicon_tags_37 },
    { 0x7dc338f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6338f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4338fb, SUC_lexicon_tags_2 },
    { 0x744338fc, SUC_lexicon_tags_17 },
    { 0xa92338fd, SUC_lexicon_tags_0 },
    { 0x440338fd, SUC_lexicon_tags_6 },
    { 0xfbc338fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe833907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b633909, SUC_lexicon_tags_25 },
    { 0xf3c33909, SUC_lexicon_tags_31 },
    { 0x96833909, SUC_lexicon_tags_14 },
    { 0xb7833909, SUC_lexicon_tags_49 },
    { 0xfd83390a, SUC_lexicon_tags_15 },
    { 0x5703390e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8a33910, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x62633912, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a33915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa433917, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58833919, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc33920, SUC_lexicon_tags_19 },
    { 0x56a33921, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76633925, SUC_lexicon_tags_31 },
    { 0x39833925, SUC_lexicon_tags_38 },
    { 0x84a33925, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c43392c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc33934, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883393a, SUC_lexicon_tags_49 },
    { 0x2c63393b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5263393d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8033943, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6633949, SUC_lexicon_tags_27 },
    { 0x4e03394a, SUC_lexicon_tags_29 },
    { 0xf603394b, SUC_lexicon_tags_17 },
    { 0x7ac3394c, SUC_lexicon_tags_26 },
    { 0xf9c3394d, SUC_lexicon_tags_32 },
    { 0x37e3394c, SUC_lexicon_tags_60 },
    { 0x1363394e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf433954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ec33956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15e33958, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c63395c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2033960, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18033963, SUC_lexicon_tags_35 },
    { 0xb4833964, SUC_lexicon_tags_26 },
    { 0xe6433965, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea03396a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1633970, SUC_lexicon_tags_7 },
    { 0x55a33971, SUC_lexicon_tags_11 },
    { 0x4e33972, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc33977, SUC_lexicon_tags_17 },
    { 0xed633977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d23397a, SUC_lexicon_tags_49 },
    { 0x9e03397b, SUC_lexicon_tags_19 },
    { 0x15e3397c, SUC_lexicon_tags_7 },
    { 0xc3397c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c33982, SUC_lexicon_tags_33 },
    { 0x66633983, SUC_lexicon_tags_7 },
    { 0x9cc33983, SUC_lexicon_tags_25 },
    { 0x71633985, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac33988, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7983398a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e3398d, SUC_lexicon_tags_12 },
    { 0x5c43398e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8ec33990, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c33997, SUC_lexicon_tags_1 },
    { 0x9e833998, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3399b, SUC_lexicon_tags_26 },
    { 0x15e3399b, SUC_lexicon_tags_17 },
    { 0x7363399b, SUC_lexicon_tags_15 },
    { 0x9143399d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a339a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d6339a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e339aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a339af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf22339b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba339b9, SUC_lexicon_tags_60 },
    { 0xe82339ba, SUC_lexicon_tags_48 },
    { 0xa6c339bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e4339bd, SUC_lexicon_tags_15 },
    { 0x99c339be, SUC_lexicon_tags_10 },
    { 0xf0c339bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc339c4, SUC_lexicon_tags_30 },
    { 0x2d8339c5, SUC_lexicon_tags_11 },
    { 0x900339c6, SUC_lexicon_tags_17 },
    { 0x8a0339c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x196339ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6339cd, SUC_lexicon_tags_12 },
    { 0xea0339ce, SUC_lexicon_tags_42 },
    { 0x87a339ce, SUC_lexicon_tags_7 },
    { 0x41c339cf, SUC_lexicon_tags_33 },
    { 0x188339d1, SUC_lexicon_tags_31 },
    { 0xd3c339d2, SUC_lexicon_tags_49 },
    { 0x5e6339d3, SUC_lexicon_tags_23 },
    { 0xc02339d4, SUC_lexicon_tags_3 },
    { 0x322339d5, SUC_lexicon_tags_2 },
    { 0x4bc339d5, SUC_lexicon_tags_17 },
    { 0xfce339d3, SUC_lexicon_tags_12 },
    { 0x436339cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x688339db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1d0339dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6339e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd1c339e4, SUC_lexicon_tags_9 },
    { 0xc64339e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce4339e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a339ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16339f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256339f5, SUC_lexicon_tags_11 },
    { 0x79e339f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28339fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51433a01, SUC_lexicon_tags_7 },
    { 0x75c33a01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74a33a04, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a633a0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11433a0d, SUC_lexicon_tags_1 },
    { 0x3bc33a0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81e33a10, SUC_lexicon_tags_26 },
    { 0xfc233a11, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x91833a14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa33a19, SUC_lexicon_tags_57 },
    { 0xf9a33a1a, SUC_lexicon_tags_49 },
    { 0x73c33a1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a33a24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a33a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a433a2a, SUC_lexicon_tags_1358 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc633a2d, SUC_lexicon_tags_17 },
    { 0xc8833a2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a33a31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6fc33a33, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a33a39, SUC_lexicon_tags_14 },
    { 0xb3a33a3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea33a3d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfa033a3f, SUC_lexicon_tags_37 },
    { 0xbbe33a40, SUC_lexicon_tags_63 },
    { 0xe0233a40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c633a45, SUC_lexicon_tags_212 },
    { 0x50c33a46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7033a49, SUC_lexicon_tags_7 },
    { 0xcc33a4a, SUC_lexicon_tags_9 },
    { 0xfc233a4a, SUC_lexicon_tags_33 },
    { 0xe0633a49, SUC_lexicon_tags_63 },
    { 0xe0033a4d, SUC_lexicon_tags_53 },
    { 0x3ce33a4d, SUC_lexicon_tags_37 },
    { 0x8ee33a4e, SUC_lexicon_tags_25 },
    { 0xdfc33a4b, SUC_lexicon_tags_17 },
    { 0x45633a4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdfc33a53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c33a56, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f833a5b, SUC_lexicon_tags_34 },
    { 0xa5233a5c, SUC_lexicon_tags_11 },
    { 0xd2233a5d, SUC_lexicon_tags_17 },
    { 0xfd233a5e, SUC_lexicon_tags_37 },
    { 0x85033a5f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a33a65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb833a69, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbe233a6b, SUC_lexicon_tags_63 },
    { 0x8a233a6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x40433a6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc233a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea33a7e, SUC_lexicon_tags_1 },
    { 0x75a33a7f, SUC_lexicon_tags_7 },
    { 0x2f033a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5833a82, SUC_lexicon_tags_19 },
    { 0xecc33a83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb5233a85, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x87c33a87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52833a8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9033a92, SUC_lexicon_tags_25 },
    { 0x57633a93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe9833a95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e33a9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1433aa0, SUC_lexicon_tags_179 },
    { 0x30633aa1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc633aa5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e33aa8, SUC_lexicon_tags_55 },
    { 0x85233aa9, SUC_lexicon_tags_11 },
    { 0xb6433aa8, SUC_lexicon_tags_0 },
    { 0xe0e33aab, SUC_lexicon_tags_37 },
    { 0xe5c33aac, SUC_lexicon_tags_17 },
    { 0x5ba33aac, SUC_lexicon_tags_39 },
    { 0x9ee33aae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3233ab0, SUC_lexicon_tags_2 },
    { 0x73033ab1, SUC_lexicon_tags_26 },
    { 0x93c33ab2, SUC_lexicon_tags_2 },
    { 0xc3c33ab0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61833ab7, SUC_lexicon_tags_17 },
    { 0xd5e33ab7, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43433ac0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3b833ac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ba33ac4, SUC_lexicon_tags_2 },
    { 0xd4033ac5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf833aca, SUC_lexicon_tags_12 },
    { 0x2f633acb, SUC_lexicon_tags_27 },
    { 0xc9833acc, SUC_lexicon_tags_48 },
    { 0xea33acb, SUC_lexicon_tags_33 },
    { 0x94033ace, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac33ad3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e33ad7, SUC_lexicon_tags_3 },
    { 0x63633ad7, SUC_lexicon_tags_43 },
    { 0x34833ad9, SUC_lexicon_tags_17 },
    { 0x5c633ada, SUC_lexicon_tags_7 },
    { 0xfe433ada, SUC_lexicon_tags_31 },
    { 0xa2433adc, SUC_lexicon_tags_8 },
    { 0x3a233add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8033ae2, SUC_lexicon_tags_7 },
    { 0x41233ae3, SUC_lexicon_tags_28 },
    { 0x40e33ae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3833ae7, SUC_lexicon_tags_47 },
    { 0x6e633ae8, SUC_lexicon_tags_31 },
    { 0x9e833ae9, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xdce33aeb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c33aee, SUC_lexicon_tags_11 },
    { 0x78033aef, SUC_lexicon_tags_12 },
    { 0xe2c33aef, SUC_lexicon_tags_47 },
    { 0x2b033af1, SUC_lexicon_tags_50 },
    { 0x86e33af2, SUC_lexicon_tags_15 },
    { 0xba233af2, SUC_lexicon_tags_1 },
    { 0x22433af4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e33aff, SUC_lexicon_tags_31 },
    { 0x6e233b00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2433b02, SUC_lexicon_tags_33 },
    { 0x19233b03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce33b05, SUC_lexicon_tags_11 },
    { 0x49833b06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e633b0a, SUC_lexicon_tags_30 },
    { 0x6e433b0b, SUC_lexicon_tags_1 },
    { 0xd1433b0c, SUC_lexicon_tags_11 },
    { 0xf5033b0c, SUC_lexicon_tags_12 },
    { 0x51a33b0c, SUC_lexicon_tags_63 },
    { 0x24a33b0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79633b17, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4ee33b19, SUC_lexicon_tags_28 },
    { 0x3d833b19, SUC_lexicon_tags_38 },
    { 0x38033b1b, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c33b1e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe2433b20, SUC_lexicon_tags_37 },
    { 0xa233b21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62c33b23, SUC_lexicon_tags_7 },
    { 0xa1033b24, SUC_lexicon_tags_2 },
    { 0x40033b23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc833b2b, SUC_lexicon_tags_29 },
    { 0x59e33b2c, SUC_lexicon_tags_3 },
    { 0x70c33b2d, SUC_lexicon_tags_2 },
    { 0x4a033b2e, SUC_lexicon_tags_10 },
    { 0x86833b2f, SUC_lexicon_tags_60 },
    { 0x3ac33b2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a433b33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77a33b35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4033b38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xda233b3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a33b40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39633b44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa33b48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e33b4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9433b4f, SUC_lexicon_tags_14 },
    { 0xfe633b50, SUC_lexicon_tags_4 },
    { 0x3e433b4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa33b55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c633b58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e33b5c, SUC_lexicon_tags_15 },
    { 0xfc833b5d, SUC_lexicon_tags_6 },
    { 0x8ea33b5e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa433b66, SUC_lexicon_tags_25 },
    { 0x86433b67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc233b6b, SUC_lexicon_tags_49 },
    { 0x33b6c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88033b7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee33b7f, SUC_lexicon_tags_47 },
    { 0x8c833b7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x54433b82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e33b85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c33b88, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x9a633b8a, SUC_lexicon_tags_12 },
    { 0xd3633b8b, SUC_lexicon_tags_8 },
    { 0x31433b8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a33b91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ae33b93, SUC_lexicon_tags_11 },
    { 0xfb433b94, SUC_lexicon_tags_25 },
    { 0xef633b93, SUC_lexicon_tags_71 },
    { 0xdba33b96, SUC_lexicon_tags_45 },
    { 0xf6233b97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e33b9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc9a33b9e, SUC_lexicon_tags_105 },
    { 0x21233b9f, SUC_lexicon_tags_63 },
    { 0xcac33b9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa433ba3, SUC_lexicon_tags_87 },
    { 0x62033ba4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc33ba9, SUC_lexicon_tags_37 },
    { 0xb8c33baa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec33bb1, SUC_lexicon_tags_25 },
    { 0x72833bb2, SUC_lexicon_tags_4 },
    { 0x54433bb1, SUC_lexicon_tags_12 },
    { 0xfd633bb4, SUC_lexicon_tags_43 },
    { 0x91433bb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x41a33bb7, SUC_lexicon_tags_15 },
    { 0x48633bb8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a33bbc, SUC_lexicon_tags_12 },
    { 0x52433bbc, SUC_lexicon_tags_27 },
    { 0xbc033bbe, SUC_lexicon_tags_17 },
    { 0xfb033bbf, SUC_lexicon_tags_8 },
    { 0x68a33bc0, SUC_lexicon_tags_17 },
    { 0xbb033bc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe233bc8, SUC_lexicon_tags_15 },
    { 0xdd233bc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5fc33bcb, SUC_lexicon_tags_37 },
    { 0xd1433bcc, SUC_lexicon_tags_2 },
    { 0x3b833bcd, SUC_lexicon_tags_14 },
    { 0x5b833bce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe33bd1, SUC_lexicon_tags_17 },
    { 0x11a33bd2, SUC_lexicon_tags_2 },
    { 0x42433bd3, SUC_lexicon_tags_641 },
    { 0x1a233bd4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe33bd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee33bda, SUC_lexicon_tags_31 },
    { 0x57e33bdb, SUC_lexicon_tags_13 },
    { 0xdc833bdc, SUC_lexicon_tags_25 },
    { 0x73633bdc, SUC_lexicon_tags_77 },
    { 0x4ce33bde, SUC_lexicon_tags_17 },
    { 0x3e233bda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e33bed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc433bf4, SUC_lexicon_tags_23 },
    { 0x21633bf5, SUC_lexicon_tags_17 },
    { 0x1e33bf6, SUC_lexicon_tags_198 },
    { 0xdc833bf5, SUC_lexicon_tags_38 },
    { 0x22c33bf8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf233bfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e33c03, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e433c06, SUC_lexicon_tags_40 },
    { 0xea033c06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10433c09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3a33c0b, SUC_lexicon_tags_37 },
    { 0x5e233c0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa33c0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d633c15, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x94433c17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40233c1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x82633c1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84633c1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x12433c21, SUC_lexicon_tags_15 },
    { 0x99833c21, SUC_lexicon_tags_25 },
    { 0xe3c33c21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c33c26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d633c28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88433c2a, SUC_lexicon_tags_87 },
    { 0x45c33c2b, SUC_lexicon_tags_29 },
    { 0x21c33c2b, SUC_lexicon_tags_28 },
    { 0xa0833c2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a233c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9033c3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xba233c3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe033c41, SUC_lexicon_tags_25 },
    { 0x1fc33c42, SUC_lexicon_tags_16 },
    { 0x2ec33c43, SUC_lexicon_tags_15 },
    { 0x6be33c44, SUC_lexicon_tags_29 },
    { 0x6dc33c44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e633c47, SUC_lexicon_tags_17 },
    { 0x7a033c48, SUC_lexicon_tags_25 },
    { 0xd8e33c49, SUC_lexicon_tags_29 },
    { 0x33233c4a, SUC_lexicon_tags_17 },
    { 0x5f833c4a, SUC_lexicon_tags_37 },
    { 0xf5833c48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e33c4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e33c52, SUC_lexicon_tags_25 },
    { 0xa3033c53, SUC_lexicon_tags_11 },
    { 0x15033c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33e33c56, SUC_lexicon_tags_11 },
    { 0xdd033c56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98233c5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c33c60, SUC_lexicon_tags_1 },
    { 0x39a33c61, SUC_lexicon_tags_13 },
    { 0x7b233c61, SUC_lexicon_tags_12 },
    { 0x88a33c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39c33c65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16633c72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc33c77, SUC_lexicon_tags_15 },
    { 0x9fe33c77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8e33c7a, SUC_lexicon_tags_6 },
    { 0x1e633c7b, SUC_lexicon_tags_2 },
    { 0xcea33c7c, SUC_lexicon_tags_0 },
    { 0xbd433c7c, SUC_lexicon_tags_19 },
    { 0x69e33c7c, SUC_lexicon_tags_55 },
    { 0x14633c7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c33c82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c33c85, SUC_lexicon_tags_2 },
    { 0xfd433c86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e33c89, SUC_lexicon_tags_47 },
    { 0x5c833c89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a33c8d, SUC_lexicon_tags_32 },
    { 0x57033c8d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa33c92, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea233c96, SUC_lexicon_tags_17 },
    { 0x6f233c97, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd0633c99, SUC_lexicon_tags_14 },
    { 0xd5233c9a, SUC_lexicon_tags_91 },
    { 0xcf233c99, SUC_lexicon_tags_63 },
    { 0x95833c9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60e33c9e, SUC_lexicon_tags_17 },
    { 0x52033c9f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2433ca1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77a33ca3, SUC_lexicon_tags_0 },
    { 0x67633ca4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2833ca7, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x42233caa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c833cac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x63033cae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6633cb6, SUC_lexicon_tags_30 },
    { 0x92633cb7, SUC_lexicon_tags_7 },
    { 0x83233cb8, SUC_lexicon_tags_29 },
    { 0x65a33cb7, SUC_lexicon_tags_49 },
    { 0x84e33cb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a33cbf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47233cc3, SUC_lexicon_tags_66 },
    { 0x1fa33cc4, SUC_lexicon_tags_55 },
    { 0x7a633cc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3833cc7, SUC_lexicon_tags_65 },
    { 0xb5433cc7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e33ccb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5833ccd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65033cd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32233cd4, SUC_lexicon_tags_87 },
    { 0xa4c33cd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37433cdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a33ce0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2233ce4, SUC_lexicon_tags_38 },
    { 0xe6033ce5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa0e33ce7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b633cec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a33cf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8233cf2, SUC_lexicon_tags_37 },
    { 0xbc33cf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7633cf7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac33cfc, SUC_lexicon_tags_25 },
    { 0x72233cfd, SUC_lexicon_tags_32 },
    { 0x3e433cfd, SUC_lexicon_tags_33 },
    { 0x97633cff, SUC_lexicon_tags_37 },
    { 0x16a33d00, SUC_lexicon_tags_7 },
    { 0xeb633d01, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c33d04, SUC_lexicon_tags_16 },
    { 0x56833d05, SUC_lexicon_tags_12 },
    { 0xdb233d06, SUC_lexicon_tags_25 },
    { 0xbd833d05, SUC_lexicon_tags_25 },
    { 0xa8433d06, SUC_lexicon_tags_9 },
    { 0x5ac33d09, SUC_lexicon_tags_2 },
    { 0x43833d07, SUC_lexicon_tags_35 },
    { 0xe7633d0b, SUC_lexicon_tags_27 },
    { 0x52033d0c, SUC_lexicon_tags_2 },
    { 0xfde33d0c, SUC_lexicon_tags_15 },
    { 0x1b233d0e, SUC_lexicon_tags_12 },
    { 0x3f033d0d, SUC_lexicon_tags_11 },
    { 0x58833d10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac33d13, SUC_lexicon_tags_0 },
    { 0x3d633d14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e633d16, SUC_lexicon_tags_82 },
    { 0xba233d17, SUC_lexicon_tags_47 },
    { 0xeec33d18, SUC_lexicon_tags_25 },
    { 0x67c33d19, SUC_lexicon_tags_16 },
    { 0x1a833d17, SUC_lexicon_tags_33 },
    { 0x36c33d18, SUC_lexicon_tags_17 },
    { 0x5fe33d18, SUC_lexicon_tags_31 },
    { 0xf5033d19, SUC_lexicon_tags_14 },
    { 0xe4c33d1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3033d22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1433d24, SUC_lexicon_tags_6 },
    { 0x7fc33d25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c33d29, SUC_lexicon_tags_14 },
    { 0xa0433d2a, SUC_lexicon_tags_14 },
    { 0x22633d2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93833d30, SUC_lexicon_tags_3 },
    { 0xc6e33d31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc33d36, SUC_lexicon_tags_25 },
    { 0x84633d37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75433d42, SUC_lexicon_tags_19 },
    { 0xcc33d43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3033d4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a33d51, SUC_lexicon_tags_1 },
    { 0x7a33d51, SUC_lexicon_tags_60 },
    { 0x5f833d52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23633d55, SUC_lexicon_tags_5 },
    { 0x42c33d55, SUC_lexicon_tags_67 },
    { 0xeba33d57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28a33d59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52e33d5b, SUC_lexicon_tags_57 },
    { 0x9f233d5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e833d63, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd633d66, SUC_lexicon_tags_37 },
    { 0xd9433d67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29033d6e, SUC_lexicon_tags_25 },
    { 0x11033d6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b633d72, SUC_lexicon_tags_37 },
    { 0x3a233d72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x69833d75, SUC_lexicon_tags_29 },
    { 0x78233d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3e33d78, SUC_lexicon_tags_82 },
    { 0x50633d78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9633d7f, SUC_lexicon_tags_1 },
    { 0xd9a33d7f, SUC_lexicon_tags_15 },
    { 0xfdc33d80, SUC_lexicon_tags_45 },
    { 0x3aa33d80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdc233d84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9d233d86, SUC_lexicon_tags_45 },
    { 0x32c33d86, SUC_lexicon_tags_37 },
    { 0xef033d87, SUC_lexicon_tags_15 },
    { 0xf3433d88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x77633d8b, SUC_lexicon_tags_15 },
    { 0x57c33d8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11633d8e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x73e33d90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70233d98, SUC_lexicon_tags_9 },
    { 0x78633d99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x55c33d9b, SUC_lexicon_tags_33 },
    { 0x37c33d9b, SUC_lexicon_tags_25 },
    { 0x15633d9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c33da1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b033da9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85033dae, SUC_lexicon_tags_25 },
    { 0x3d233dae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98233db5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe33db8, SUC_lexicon_tags_0 },
    { 0xfc033db8, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x9033dbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e33dc6, SUC_lexicon_tags_7 },
    { 0x42433dc7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e33dca, SUC_lexicon_tags_6 },
    { 0x52c33dca, SUC_lexicon_tags_16 },
    { 0xae233dcc, SUC_lexicon_tags_1 },
    { 0x37033dcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e33dd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a33dd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e33ddb, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4433de1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x76633de4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd433de7, SUC_lexicon_tags_27 },
    { 0xf7033de8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe033dec, SUC_lexicon_tags_26 },
    { 0xe8433ded, SUC_lexicon_tags_17 },
    { 0x3233ded, SUC_lexicon_tags_15 },
    { 0x26833dee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43033df4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x50e33df6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51033dfb, SUC_lexicon_tags_37 },
    { 0xe0e33dfc, SUC_lexicon_tags_15 },
    { 0xc0a33dfd, SUC_lexicon_tags_39 },
    { 0xdc033dfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40033e09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc33e0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c233e10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1433e17, SUC_lexicon_tags_11 },
    { 0x55a33e18, SUC_lexicon_tags_19 },
    { 0xb9a33e19, SUC_lexicon_tags_12 },
    { 0x35033e1a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe0233e1c, SUC_lexicon_tags_25 },
    { 0xe9a33e1d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66833e21, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c33e25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2c833e27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69233e2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34033e2c, SUC_lexicon_tags_0 },
    { 0x5ba33e2d, SUC_lexicon_tags_12 },
    { 0x84c33e2e, SUC_lexicon_tags_47 },
    { 0xe8633e2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38633e35, SUC_lexicon_tags_4 },
    { 0x70a33e36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94633e38, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea33e3c, SUC_lexicon_tags_1 },
    { 0xab033e3c, SUC_lexicon_tags_53 },
    { 0xfdc33e3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd833e40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8e33e42, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb1c33e44, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58433e4b, SUC_lexicon_tags_17 },
    { 0xe0433e4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4233e52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2633e54, SUC_lexicon_tags_2 },
    { 0x54433e55, SUC_lexicon_tags_12 },
    { 0x81a33e56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57833e5a, SUC_lexicon_tags_25 },
    { 0x38633e5b, SUC_lexicon_tags_23 },
    { 0xe0033e5c, SUC_lexicon_tags_27 },
    { 0xd6833e5c, SUC_lexicon_tags_27 },
    { 0xf4833e5c, SUC_lexicon_tags_1 },
    { 0x81833e5f, SUC_lexicon_tags_6 },
    { 0xb7c33e5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9833e65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5233e67, SUC_lexicon_tags_30 },
    { 0x83833e67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee33e70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc833e73, SUC_lexicon_tags_0 },
    { 0x7dc33e74, SUC_lexicon_tags_23 },
    { 0xee033e75, SUC_lexicon_tags_85 },
    { 0x52c33e75, SUC_lexicon_tags_77 },
    { 0xfd633e75, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3033e7b, SUC_lexicon_tags_29 },
    { 0x12a33e7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7633e7e, SUC_lexicon_tags_11 },
    { 0x12233e7f, SUC_lexicon_tags_17 },
    { 0xd1a33e80, SUC_lexicon_tags_27 },
    { 0x9dc33e81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9633e85, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcf633e87, SUC_lexicon_tags_15 },
    { 0xa3233e88, SUC_lexicon_tags_2 },
    { 0x5e033e87, SUC_lexicon_tags_23 },
    { 0x72e33e87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9633e8c, SUC_lexicon_tags_8 },
    { 0xeba33e8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81833e91, SUC_lexicon_tags_12 },
    { 0xf0c33e91, SUC_lexicon_tags_1 },
    { 0xb8a33e92, SUC_lexicon_tags_37 },
    { 0x6fa33e94, SUC_lexicon_tags_25 },
    { 0x81a33e94, SUC_lexicon_tags_25 },
    { 0x67633e93, SUC_lexicon_tags_15 },
    { 0xff433e94, SUC_lexicon_tags_1 },
    { 0xe3233e92, SUC_lexicon_tags_2 },
    { 0x83c33e99, SUC_lexicon_tags_40 },
    { 0xc3833e9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92833e9e, SUC_lexicon_tags_37 },
    { 0xee033e9e, SUC_lexicon_tags_27 },
    { 0x55c33ea0, SUC_lexicon_tags_31 },
    { 0x833ea1, SUC_lexicon_tags_17 },
    { 0xa1c33e9e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x8c433ea4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c33eaa, SUC_lexicon_tags_1 },
    { 0xdf233eaa, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96633eb0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec833eb4, SUC_lexicon_tags_47 },
    { 0x3e433eb4, SUC_lexicon_tags_25 },
    { 0xc1233eb6, SUC_lexicon_tags_68 },
    { 0x56a33eb7, SUC_lexicon_tags_17 },
    { 0x12633eb7, SUC_lexicon_tags_49 },
    { 0x53833eb5, SUC_lexicon_tags_28 },
    { 0x96e33eba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d033ebc, SUC_lexicon_tags_45 },
    { 0x40833ebc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x53433ebf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14633ec1, SUC_lexicon_tags_38 },
    { 0x7b633ec2, SUC_lexicon_tags_1 },
    { 0x87e33ec2, SUC_lexicon_tags_19 },
    { 0x59233ec2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31633ec8, SUC_lexicon_tags_45 },
    { 0xaf833ec8, SUC_lexicon_tags_7 },
    { 0xb3233eca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c33ecd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79633ed4, SUC_lexicon_tags_79 },
    { 0x4c433ed5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdf233ed7, SUC_lexicon_tags_1 },
    { 0x92433ed7, SUC_lexicon_tags_37 },
    { 0x3e633ed7, SUC_lexicon_tags_47 },
    { 0x7ea33ed9, SUC_lexicon_tags_11 },
    { 0xc2833edb, SUC_lexicon_tags_19 },
    { 0x9d233edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c33edf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3433ee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b433ee6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1033eea, SUC_lexicon_tags_37 },
    { 0x3fe33eeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xac833eee, SUC_lexicon_tags_11 },
    { 0x34c33eef, SUC_lexicon_tags_4 },
    { 0x15033eee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf633ef2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa33ef8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c33efc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3233f03, SUC_lexicon_tags_3 },
    { 0x87633f04, SUC_lexicon_tags_19 },
    { 0xba433f05, SUC_lexicon_tags_37 },
    { 0x72433f06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc33f0b, SUC_lexicon_tags_8 },
    { 0x70233f0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c33f10, SUC_lexicon_tags_37 },
    { 0x26833f11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a33f1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5833f1f, SUC_lexicon_tags_11 },
    { 0x69033f20, SUC_lexicon_tags_68 },
    { 0xde033f1f, SUC_lexicon_tags_25 },
    { 0x57433f20, SUC_lexicon_tags_17 },
    { 0x16c33f1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb233f25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae33f2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x71633f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba33f34, SUC_lexicon_tags_17 },
    { 0xc3e33f34, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f433f3a, SUC_lexicon_tags_47 },
    { 0xeac33f3b, SUC_lexicon_tags_17 },
    { 0x4ba33f3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29033f3e, SUC_lexicon_tags_37 },
    { 0x66233f3f, SUC_lexicon_tags_48 },
    { 0x5633f40, SUC_lexicon_tags_28 },
    { 0xf9433f40, SUC_lexicon_tags_63 },
    { 0x10e33f42, SUC_lexicon_tags_37 },
    { 0xfa033f43, SUC_lexicon_tags_63 },
    { 0xd5233f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5633f47, SUC_lexicon_tags_1 },
    { 0xfc633f48, SUC_lexicon_tags_12 },
    { 0xf1633f47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a433f4d, SUC_lexicon_tags_17 },
    { 0xc9833f4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdea33f50, SUC_lexicon_tags_23 },
    { 0x3ce33f51, SUC_lexicon_tags_48 },
    { 0xdd833f52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e33f55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa633f5b, SUC_lexicon_tags_17 },
    { 0x71c33f5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51033f5f, SUC_lexicon_tags_7 },
    { 0x9f233f5f, SUC_lexicon_tags_38 },
    { 0x73233f61, SUC_lexicon_tags_33 },
    { 0x7a633f61, SUC_lexicon_tags_12 },
    { 0xf9433f63, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x50233f65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a33f6a, SUC_lexicon_tags_55 },
    { 0xc9233f6b, SUC_lexicon_tags_7 },
    { 0xff833f6b, SUC_lexicon_tags_17 },
    { 0x49e33f6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5233f70, SUC_lexicon_tags_37 },
    { 0x1dc33f71, SUC_lexicon_tags_19 },
    { 0x4c833f72, SUC_lexicon_tags_37 },
    { 0x17633f70, SUC_lexicon_tags_6 },
    { 0x58e33f74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98233f79, SUC_lexicon_tags_12 },
    { 0x80a33f79, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e33f7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9a33f81, SUC_lexicon_tags_37 },
    { 0x26433f82, SUC_lexicon_tags_12 },
    { 0xffc33f83, SUC_lexicon_tags_47 },
    { 0x5b633f83, SUC_lexicon_tags_37 },
    { 0x4fc33f85, SUC_lexicon_tags_15 },
    { 0x4fc33f81, SUC_lexicon_tags_17 },
    { 0x9e833f87, SUC_lexicon_tags_12 },
    { 0xe8833f88, SUC_lexicon_tags_37 },
    { 0x23633f89, SUC_lexicon_tags_7 },
    { 0x7c233f89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f433f8d, SUC_lexicon_tags_25 },
    { 0x69233f8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e33f90, SUC_lexicon_tags_7 },
    { 0xf5633f91, SUC_lexicon_tags_30 },
    { 0xa7033f92, SUC_lexicon_tags_11 },
    { 0xf9833f92, SUC_lexicon_tags_13 },
    { 0x4be33f94, SUC_lexicon_tags_11 },
    { 0x57e33f95, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x61233f97, SUC_lexicon_tags_61 },
    { 0xdfe33f98, SUC_lexicon_tags_9 },
    { 0xaca33f99, SUC_lexicon_tags_25 },
    { 0x1cc33f9a, SUC_lexicon_tags_7 },
    { 0x6d033f9a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb233f9d, SUC_lexicon_tags_0 },
    { 0x93e33f9e, SUC_lexicon_tags_30 },
    { 0xe4e33f9f, SUC_lexicon_tags_37 },
    { 0x92033f9f, SUC_lexicon_tags_49 },
    { 0x8b033f9f, SUC_lexicon_tags_27 },
    { 0x74633f9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5033fab, SUC_lexicon_tags_30 },
    { 0xeee33fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1233fb9, SUC_lexicon_tags_2 },
    { 0xb0233fba, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x7033fbc, SUC_lexicon_tags_25 },
    { 0xe9633fbc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd5833fbf, SUC_lexicon_tags_46 },
    { 0xf3e33fc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a33fc4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a33fcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xec433fd1, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91833fd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6033fd9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e633fde, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3c33fe0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8a33fe2, SUC_lexicon_tags_1291 },
    { 0x1c033fe2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39e33fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc33fe8, SUC_lexicon_tags_7 },
    { 0xa7a33fe9, SUC_lexicon_tags_11 },
    { 0x7ce33fe9, SUC_lexicon_tags_10 },
    { 0x5c833feb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf233fee, SUC_lexicon_tags_33 },
    { 0x76c33fef, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x2b833ff1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x69833ff3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e33ff6, SUC_lexicon_tags_55 },
    { 0x3e233ff7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfea33ff9, SUC_lexicon_tags_31 },
    { 0x87a33ffa, SUC_lexicon_tags_37 },
    { 0x4c633ffb, SUC_lexicon_tags_23 },
    { 0xe033ffc, SUC_lexicon_tags_37 },
    { 0xc9633ffd, SUC_lexicon_tags_29 },
    { 0x7fe33ff9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6d434000, SUC_lexicon_tags_19 },
    { 0x95234001, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce34005, SUC_lexicon_tags_32 },
    { 0x96434006, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7034008, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4703400b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf63400d, SUC_lexicon_tags_37 },
    { 0x6283400e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54234010, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc34014, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb234016, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c3401d, SUC_lexicon_tags_37 },
    { 0x9cc3401d, SUC_lexicon_tags_48 },
    { 0x5803401e, SUC_lexicon_tags_17 },
    { 0x84a34020, SUC_lexicon_tags_0 },
    { 0xcc234020, SUC_lexicon_tags_7 },
    { 0x5a834021, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da34028, SUC_lexicon_tags_9 },
    { 0xe0834028, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce34031, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x93234034, SUC_lexicon_tags_15 },
    { 0x94a34035, SUC_lexicon_tags_15 },
    { 0x97634035, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8b834038, SUC_lexicon_tags_17 },
    { 0xeec34039, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c3403c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a3403f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc834042, SUC_lexicon_tags_14 },
    { 0x9f634043, SUC_lexicon_tags_37 },
    { 0xbb234044, SUC_lexicon_tags_21 },
    { 0xc9434045, SUC_lexicon_tags_63 },
    { 0xc2e34046, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xabe34048, SUC_lexicon_tags_37 },
    { 0x18e34049, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee3404d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc1c3404f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2634051, SUC_lexicon_tags_25 },
    { 0xa8034052, SUC_lexicon_tags_29 },
    { 0x3ac34053, SUC_lexicon_tags_37 },
    { 0x3cc34054, SUC_lexicon_tags_12 },
    { 0x95034054, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc7034057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12834059, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae3405c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c34060, SUC_lexicon_tags_15 },
    { 0xe2c34060, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc34064, SUC_lexicon_tags_11 },
    { 0xbea34065, SUC_lexicon_tags_38 },
    { 0x50c34066, SUC_lexicon_tags_42 },
    { 0x45434064, SUC_lexicon_tags_37 },
    { 0xb3a34066, SUC_lexicon_tags_17 },
    { 0xeae34069, SUC_lexicon_tags_37 },
    { 0x7c634069, SUC_lexicon_tags_11 },
    { 0xbd03406a, SUC_lexicon_tags_38 },
    { 0xec3406c, SUC_lexicon_tags_60 },
    { 0x2823406d, SUC_lexicon_tags_25 },
    { 0xf4434069, SUC_lexicon_tags_17 },
    { 0xb023406b, SUC_lexicon_tags_28 },
    { 0xc4c34070, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x24c34072, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2234076, SUC_lexicon_tags_12 },
    { 0x51c34077, SUC_lexicon_tags_55 },
    { 0x75034078, SUC_lexicon_tags_25 },
    { 0x75a34077, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde83407b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x763407d, SUC_lexicon_tags_3 },
    { 0xfa43407d, SUC_lexicon_tags_37 },
    { 0xdaa3407f, SUC_lexicon_tags_21 },
    { 0x3003407f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a34085, SUC_lexicon_tags_26 },
    { 0xc3e34086, SUC_lexicon_tags_32 },
    { 0x92434087, SUC_lexicon_tags_11 },
    { 0xffa34088, SUC_lexicon_tags_1 },
    { 0xc4a34088, SUC_lexicon_tags_12 },
    { 0x1ca34087, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3563408c, SUC_lexicon_tags_9 },
    { 0x6883408d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38834092, SUC_lexicon_tags_1 },
    { 0x1f434092, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e034096, SUC_lexicon_tags_0 },
    { 0x2b634097, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe063409d, SUC_lexicon_tags_969 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206340a3, SUC_lexicon_tags_25 },
    { 0xaf4340a4, SUC_lexicon_tags_61 },
    { 0x98a340a4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce340a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8340ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x584340b1, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e340b4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560340ba, SUC_lexicon_tags_37 },
    { 0x35a340bb, SUC_lexicon_tags_27 },
    { 0x280340bc, SUC_lexicon_tags_45 },
    { 0xcd8340bc, SUC_lexicon_tags_31 },
    { 0x5c8340bd, SUC_lexicon_tags_45 },
    { 0xd1a340bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca340c2, SUC_lexicon_tags_11 },
    { 0xda340c2, SUC_lexicon_tags_6 },
    { 0x782340c4, SUC_lexicon_tags_17 },
    { 0x6ce340c4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38340ca, SUC_lexicon_tags_29 },
    { 0xc44340cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96340ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xba4340d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a340d4, SUC_lexicon_tags_79 },
    { 0xe1e340d5, SUC_lexicon_tags_22 },
    { 0x914340d6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa340d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970340dd, SUC_lexicon_tags_6 },
    { 0xa7e340de, SUC_lexicon_tags_30 },
    { 0xe82340df, SUC_lexicon_tags_17 },
    { 0xf7c340e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e340e3, SUC_lexicon_tags_11 },
    { 0x702340e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ca340e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a340ed, SUC_lexicon_tags_27 },
    { 0x50c340ee, SUC_lexicon_tags_11 },
    { 0x97e340ed, SUC_lexicon_tags_46 },
    { 0xdfc340f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x668340f2, SUC_lexicon_tags_47 },
    { 0x9f6340f3, SUC_lexicon_tags_17 },
    { 0xb2c340f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ac340f6, SUC_lexicon_tags_12 },
    { 0x898340f6, SUC_lexicon_tags_37 },
    { 0x398340f6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986340ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e34102, SUC_lexicon_tags_12 },
    { 0xcb434102, SUC_lexicon_tags_2 },
    { 0x2ec34102, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e34108, SUC_lexicon_tags_15 },
    { 0x6b634109, SUC_lexicon_tags_11 },
    { 0xa963410a, SUC_lexicon_tags_129 },
    { 0x9d83410b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4463410d, SUC_lexicon_tags_37 },
    { 0xbde3410e, SUC_lexicon_tags_23 },
    { 0x9563410d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0c34111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a834114, SUC_lexicon_tags_33 },
    { 0xe0034115, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c34117, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4863411a, SUC_lexicon_tags_4 },
    { 0x5d83411a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b634123, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3634126, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b03412b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfea3412d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1b63412f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81634131, SUC_lexicon_tags_37 },
    { 0xc1234132, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1034139, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f63413f, SUC_lexicon_tags_11 },
    { 0x2703413f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7e34142, SUC_lexicon_tags_15 },
    { 0xc8234143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe834145, SUC_lexicon_tags_23 },
    { 0xada34146, SUC_lexicon_tags_7 },
    { 0x48634146, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43234151, SUC_lexicon_tags_25 },
    { 0x81a34152, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40434159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5383415d, SUC_lexicon_tags_25 },
    { 0xd43415e, SUC_lexicon_tags_17 },
    { 0x3c43415f, SUC_lexicon_tags_12 },
    { 0xdc43415e, SUC_lexicon_tags_64 },
    { 0x9f43415d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4834164, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45834166, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc83416a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb234170, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x95a34172, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe34175, SUC_lexicon_tags_25 },
    { 0x2d234176, SUC_lexicon_tags_7 },
    { 0xfaa34177, SUC_lexicon_tags_31 },
    { 0x73234178, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2e3417a, SUC_lexicon_tags_1 },
    { 0x16e3417b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9783417e, SUC_lexicon_tags_37 },
    { 0x2a63417f, SUC_lexicon_tags_25 },
    { 0x38a3417f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39034187, SUC_lexicon_tags_66 },
    { 0x49e34188, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9823418b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a3418f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc034192, SUC_lexicon_tags_14 },
    { 0xe9c34193, SUC_lexicon_tags_8 },
    { 0x4a234193, SUC_lexicon_tags_32 },
    { 0x70c34195, SUC_lexicon_tags_63 },
    { 0x31034196, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x83234198, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa323419a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe4c3419c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f03419f, SUC_lexicon_tags_45 },
    { 0x7e03419f, SUC_lexicon_tags_60 },
    { 0xada341a1, SUC_lexicon_tags_9 },
    { 0xc24341a1, SUC_lexicon_tags_25 },
    { 0x54341a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba341ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x490341af, SUC_lexicon_tags_25 },
    { 0xd80341af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632341b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6341b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63a341bb, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a341bf, SUC_lexicon_tags_11 },
    { 0x896341c0, SUC_lexicon_tags_66 },
    { 0xe30341bf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172341c6, SUC_lexicon_tags_10 },
    { 0xdf0341c7, SUC_lexicon_tags_18 },
    { 0x8d0341c7, SUC_lexicon_tags_11 },
    { 0xfe0341c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30341d1, SUC_lexicon_tags_37 },
    { 0x1de341d2, SUC_lexicon_tags_47 },
    { 0xa5a341d3, SUC_lexicon_tags_17 },
    { 0xb48341d4, SUC_lexicon_tags_46 },
    { 0xdd4341d3, SUC_lexicon_tags_17 },
    { 0xc8e341d4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x44341d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4341e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7fa341e2, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x7c0341e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd88341e6, SUC_lexicon_tags_14 },
    { 0x1ec341e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf30341e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978341f1, SUC_lexicon_tags_31 },
    { 0xec341f2, SUC_lexicon_tags_137 },
    { 0x6f0341f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf0a341f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c341f9, SUC_lexicon_tags_5 },
    { 0x956341fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a34201, SUC_lexicon_tags_11 },
    { 0xcc434202, SUC_lexicon_tags_7 },
    { 0x5e034203, SUC_lexicon_tags_25 },
    { 0xeb434201, SUC_lexicon_tags_15 },
    { 0x69034204, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa4234207, SUC_lexicon_tags_17 },
    { 0x14834208, SUC_lexicon_tags_37 },
    { 0x38434209, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa843420b, SUC_lexicon_tags_14 },
    { 0x6ec3420c, SUC_lexicon_tags_4 },
    { 0xb823420c, SUC_lexicon_tags_81 },
    { 0x7423420d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea34215, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c834218, SUC_lexicon_tags_45 },
    { 0xd8834219, SUC_lexicon_tags_31 },
    { 0x2983421a, SUC_lexicon_tags_12 },
    { 0x6d83421b, SUC_lexicon_tags_2 },
    { 0xf183421b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4323421e, SUC_lexicon_tags_2 },
    { 0x8f03421f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71434222, SUC_lexicon_tags_103 },
    { 0x20834223, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78834225, SUC_lexicon_tags_273 },
    { 0, NULL },
    { 0xafc34227, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8583422a, SUC_lexicon_tags_17 },
    { 0xe363422a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc3422f, SUC_lexicon_tags_7 },
    { 0x5e834230, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6034234, SUC_lexicon_tags_11 },
    { 0x52834235, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x61a34237, SUC_lexicon_tags_8 },
    { 0xfbc34238, SUC_lexicon_tags_47 },
    { 0x24234239, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8e3423b, SUC_lexicon_tags_37 },
    { 0xcdc3423c, SUC_lexicon_tags_9 },
    { 0x51a3423d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c034241, SUC_lexicon_tags_14 },
    { 0x42634242, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x90034244, SUC_lexicon_tags_11 },
    { 0xa034244, SUC_lexicon_tags_7 },
    { 0x85e34246, SUC_lexicon_tags_0 },
    { 0x6f634246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85034249, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf023424c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa3424f, SUC_lexicon_tags_17 },
    { 0x5c834250, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x52434252, SUC_lexicon_tags_55 },
    { 0x26c34252, SUC_lexicon_tags_15 },
    { 0xd0034254, SUC_lexicon_tags_34 },
    { 0x96434255, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75434259, SUC_lexicon_tags_45 },
    { 0xc863425a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb943425c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa663425f, SUC_lexicon_tags_11 },
    { 0xfa23425f, SUC_lexicon_tags_0 },
    { 0x95034261, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e34264, SUC_lexicon_tags_15 },
    { 0xd1e34265, SUC_lexicon_tags_42 },
    { 0x35634265, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c34271, SUC_lexicon_tags_2 },
    { 0xa7e34272, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6f634274, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8234276, SUC_lexicon_tags_7 },
    { 0x51434276, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42a34279, SUC_lexicon_tags_140 },
    { 0xe9a3427a, SUC_lexicon_tags_8 },
    { 0xe5234279, SUC_lexicon_tags_8 },
    { 0x2423427c, SUC_lexicon_tags_45 },
    { 0x8ea3427d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c34283, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53634287, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a3428a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xae63428d, SUC_lexicon_tags_11 },
    { 0x61a3428d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e34293, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6963429a, SUC_lexicon_tags_37 },
    { 0xd083429a, SUC_lexicon_tags_37 },
    { 0x86c3429c, SUC_lexicon_tags_8 },
    { 0xdda3429c, SUC_lexicon_tags_1 },
    { 0x5023429e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x888342a1, SUC_lexicon_tags_49 },
    { 0x8b8342a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24342a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4be342a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea342ae, SUC_lexicon_tags_12 },
    { 0x38342af, SUC_lexicon_tags_7 },
    { 0x184342af, SUC_lexicon_tags_25 },
    { 0x32a342b1, SUC_lexicon_tags_45 },
    { 0xb16342b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0342b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0342ba, SUC_lexicon_tags_7 },
    { 0x7fe342bb, SUC_lexicon_tags_1 },
    { 0xca0342ba, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8342bf, SUC_lexicon_tags_31 },
    { 0x464342c0, SUC_lexicon_tags_45 },
    { 0xd16342c0, SUC_lexicon_tags_37 },
    { 0x290342c0, SUC_lexicon_tags_1569 },
    { 0, NULL },
    { 0xf52342c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x554342c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6342ca, SUC_lexicon_tags_8 },
    { 0xbb6342cb, SUC_lexicon_tags_12 },
    { 0xcbe342cc, SUC_lexicon_tags_7 },
    { 0x744342cd, SUC_lexicon_tags_25 },
    { 0x38342cc, SUC_lexicon_tags_17 },
    { 0x374342cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x954342d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba342d6, SUC_lexicon_tags_11 },
    { 0x118342d7, SUC_lexicon_tags_38 },
    { 0x176342d8, SUC_lexicon_tags_33 },
    { 0xc04342d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec342dd, SUC_lexicon_tags_37 },
    { 0x40c342de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714342e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6342e8, SUC_lexicon_tags_65 },
    { 0x494342e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664342ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76342f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c342f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e342fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9c342fc, SUC_lexicon_tags_47 },
    { 0xed0342fd, SUC_lexicon_tags_11 },
    { 0x9b4342fd, SUC_lexicon_tags_23 },
    { 0xc02342fd, SUC_lexicon_tags_43 },
    { 0x81834300, SUC_lexicon_tags_75 },
    { 0x2a034301, SUC_lexicon_tags_25 },
    { 0xb6434301, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde434304, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcc34306, SUC_lexicon_tags_25 },
    { 0xcee34307, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc483430a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d03430e, SUC_lexicon_tags_49 },
    { 0xe083430f, SUC_lexicon_tags_48 },
    { 0x18634310, SUC_lexicon_tags_49 },
    { 0x92034310, SUC_lexicon_tags_12 },
    { 0xe323430f, SUC_lexicon_tags_27 },
    { 0x58634312, SUC_lexicon_tags_1 },
    { 0x25634314, SUC_lexicon_tags_37 },
    { 0x47e34310, SUC_lexicon_tags_82 },
    { 0x95c34316, SUC_lexicon_tags_2 },
    { 0xfa234317, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a034319, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a3431c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6083431e, SUC_lexicon_tags_15 },
    { 0xa9e3431f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x60e34321, SUC_lexicon_tags_87 },
    { 0x61e34321, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65834328, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc3432b, SUC_lexicon_tags_65 },
    { 0xa4c3432c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d43432e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34834331, SUC_lexicon_tags_49 },
    { 0xbbc34331, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c034335, SUC_lexicon_tags_83 },
    { 0x1c434336, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de34339, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac3433c, SUC_lexicon_tags_1 },
    { 0x4743433d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd123433f, SUC_lexicon_tags_25 },
    { 0x50234340, SUC_lexicon_tags_70 },
    { 0x77a34341, SUC_lexicon_tags_33 },
    { 0x7ac34342, SUC_lexicon_tags_27 },
    { 0xf1634343, SUC_lexicon_tags_11 },
    { 0xa9834341, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaa634346, SUC_lexicon_tags_49 },
    { 0x22834347, SUC_lexicon_tags_35 },
    { 0x23e34348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b03434f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a34352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e34359, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb034360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e34364, SUC_lexicon_tags_37 },
    { 0xd434364, SUC_lexicon_tags_10 },
    { 0x86c34366, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3234372, SUC_lexicon_tags_33 },
    { 0x81034373, SUC_lexicon_tags_14 },
    { 0x24c34372, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4ee34376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc563437e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87c34380, SUC_lexicon_tags_14 },
    { 0xfc234381, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d834386, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7d634388, SUC_lexicon_tags_34 },
    { 0x40a34389, SUC_lexicon_tags_19 },
    { 0x2c43438a, SUC_lexicon_tags_116 },
    { 0x6463438b, SUC_lexicon_tags_14 },
    { 0x4e83438a, SUC_lexicon_tags_7 },
    { 0xaa434389, SUC_lexicon_tags_84 },
    { 0xa223438c, SUC_lexicon_tags_13 },
    { 0x6863438f, SUC_lexicon_tags_1 },
    { 0xee234390, SUC_lexicon_tags_56 },
    { 0x9e83438b, SUC_lexicon_tags_37 },
    { 0x44c3438b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e3439a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd03439e, SUC_lexicon_tags_12 },
    { 0x4183439e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64343a3, SUC_lexicon_tags_28 },
    { 0xd54343a4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782343a8, SUC_lexicon_tags_30 },
    { 0x192343a9, SUC_lexicon_tags_37 },
    { 0xe42343aa, SUC_lexicon_tags_33 },
    { 0x62c343aa, SUC_lexicon_tags_282 },
    { 0xcb2343ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc343b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34343b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8343b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x932343b9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558343bd, SUC_lexicon_tags_0 },
    { 0x5e6343bd, SUC_lexicon_tags_10 },
    { 0xd86343bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70343c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92343c7, SUC_lexicon_tags_47 },
    { 0x2e6343c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274343cc, SUC_lexicon_tags_114 },
    { 0x7a6343cd, SUC_lexicon_tags_12 },
    { 0xf88343ce, SUC_lexicon_tags_11 },
    { 0x19e343ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddc343d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58343d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5f8343d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ae343d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc8343d9, SUC_lexicon_tags_29 },
    { 0x2d2343da, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3f2343dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a343df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x422343e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa0343e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeac343e6, SUC_lexicon_tags_1557 },
    { 0x41c343e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x624343e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93c343eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44343ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a0343f1, SUC_lexicon_tags_12 },
    { 0xb06343f2, SUC_lexicon_tags_64 },
    { 0x6b0343f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4ea343f5, SUC_lexicon_tags_8 },
    { 0x7f2343f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x480343f9, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0x88343fb, SUC_lexicon_tags_25 },
    { 0x124343fb, SUC_lexicon_tags_11 },
    { 0xa04343fb, SUC_lexicon_tags_26 },
    { 0x7ae343fe, SUC_lexicon_tags_82 },
    { 0x428343ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23834401, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3834405, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa34409, SUC_lexicon_tags_10 },
    { 0xe4834409, SUC_lexicon_tags_12 },
    { 0x6e23440b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9a3440d, SUC_lexicon_tags_43 },
    { 0xc383440e, SUC_lexicon_tags_15 },
    { 0x983440d, SUC_lexicon_tags_53 },
    { 0x5b634410, SUC_lexicon_tags_8 },
    { 0x1f63440e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f434415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d234419, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c43441c, SUC_lexicon_tags_8 },
    { 0x2a03441d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78e3441f, SUC_lexicon_tags_8 },
    { 0x52434420, SUC_lexicon_tags_12 },
    { 0x51c34421, SUC_lexicon_tags_21 },
    { 0xaa034422, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c34425, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3442c, SUC_lexicon_tags_17 },
    { 0x3ce3442d, SUC_lexicon_tags_12 },
    { 0xc763442e, SUC_lexicon_tags_45 },
    { 0x1923442f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c034432, SUC_lexicon_tags_33 },
    { 0xe8034433, SUC_lexicon_tags_17 },
    { 0xb4c34433, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7603443a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac834445, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43834447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e23444c, SUC_lexicon_tags_37 },
    { 0x10e3444c, SUC_lexicon_tags_40 },
    { 0x1e03444e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e34455, SUC_lexicon_tags_25 },
    { 0x22e34456, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfc634458, SUC_lexicon_tags_4 },
    { 0xea834459, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a3445c, SUC_lexicon_tags_25 },
    { 0xfca3445c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c34460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf034462, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92434466, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8783446b, SUC_lexicon_tags_17 },
    { 0x89e3446b, SUC_lexicon_tags_55 },
    { 0x7623446d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e34470, SUC_lexicon_tags_68 },
    { 0x30634471, SUC_lexicon_tags_10 },
    { 0x9b034471, SUC_lexicon_tags_17 },
    { 0x83234470, SUC_lexicon_tags_37 },
    { 0x70c34473, SUC_lexicon_tags_26 },
    { 0x8c634473, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf834477, SUC_lexicon_tags_7 },
    { 0x234478, SUC_lexicon_tags_82 },
    { 0xc1634479, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae43447e, SUC_lexicon_tags_11 },
    { 0xcaa3447f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb234482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75434485, SUC_lexicon_tags_11 },
    { 0xf6e34485, SUC_lexicon_tags_46 },
    { 0x60e34487, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be3448c, SUC_lexicon_tags_37 },
    { 0x7b03448d, SUC_lexicon_tags_10 },
    { 0x6703448e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96834492, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda34495, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa34498, SUC_lexicon_tags_47 },
    { 0xf8834498, SUC_lexicon_tags_43 },
    { 0xf463449a, SUC_lexicon_tags_12 },
    { 0x29e3449b, SUC_lexicon_tags_37 },
    { 0x65a3449b, SUC_lexicon_tags_4 },
    { 0xd63449d, SUC_lexicon_tags_1 },
    { 0xe3c3449a, SUC_lexicon_tags_2 },
    { 0x34e3449e, SUC_lexicon_tags_25 },
    { 0x9fa34499, SUC_lexicon_tags_33 },
    { 0x7a0344a1, SUC_lexicon_tags_28 },
    { 0x16a344a2, SUC_lexicon_tags_17 },
    { 0x106344a3, SUC_lexicon_tags_16 },
    { 0xba8344a3, SUC_lexicon_tags_1 },
    { 0xe0c344a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac344aa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x93a344ac, SUC_lexicon_tags_27 },
    { 0x298344ac, SUC_lexicon_tags_11 },
    { 0x990344ae, SUC_lexicon_tags_31 },
    { 0x7fe344af, SUC_lexicon_tags_30 },
    { 0x9be344af, SUC_lexicon_tags_42 },
    { 0x310344ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c344b4, SUC_lexicon_tags_38 },
    { 0x44e344b5, SUC_lexicon_tags_2 },
    { 0x1de344b5, SUC_lexicon_tags_28 },
    { 0x33a344b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100344bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a344be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c344c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe344cd, SUC_lexicon_tags_26 },
    { 0x1a0344ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x256344d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16344d4, SUC_lexicon_tags_7 },
    { 0xba8344d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638344dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc344df, SUC_lexicon_tags_14 },
    { 0xd3e344df, SUC_lexicon_tags_17 },
    { 0x1d4344e0, SUC_lexicon_tags_68 },
    { 0xa04344e2, SUC_lexicon_tags_11 },
    { 0x4c4344e0, SUC_lexicon_tags_37 },
    { 0x16344e0, SUC_lexicon_tags_33 },
    { 0x28e344e4, SUC_lexicon_tags_4 },
    { 0xe46344e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424344ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x238344ec, SUC_lexicon_tags_2 },
    { 0x5e4344ec, SUC_lexicon_tags_13 },
    { 0x60344ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a344f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694344fb, SUC_lexicon_tags_42 },
    { 0xfb2344fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a34503, SUC_lexicon_tags_29 },
    { 0x31034504, SUC_lexicon_tags_17 },
    { 0xa8234505, SUC_lexicon_tags_46 },
    { 0xe3634505, SUC_lexicon_tags_31 },
    { 0xe0a34507, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a3450a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d63450d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6034510, SUC_lexicon_tags_1 },
    { 0xbf834511, SUC_lexicon_tags_31 },
    { 0x8c634512, SUC_lexicon_tags_53 },
    { 0x76634511, SUC_lexicon_tags_1 },
    { 0xe0e34514, SUC_lexicon_tags_19 },
    { 0x29234514, SUC_lexicon_tags_14 },
    { 0x7fc34515, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd7234518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2223451a, SUC_lexicon_tags_17 },
    { 0x6583451b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5103451d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8e3451f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe634522, SUC_lexicon_tags_17 },
    { 0x89834523, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f034529, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d634534, SUC_lexicon_tags_17 },
    { 0xb2c34534, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f434537, SUC_lexicon_tags_38 },
    { 0x8de34538, SUC_lexicon_tags_6 },
    { 0xb8a34539, SUC_lexicon_tags_14 },
    { 0x2c3453a, SUC_lexicon_tags_51 },
    { 0xc834538, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0234545, SUC_lexicon_tags_55 },
    { 0x90e34545, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec34549, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1383454b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c3454d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dc3454f, SUC_lexicon_tags_8 },
    { 0x5a634550, SUC_lexicon_tags_15 },
    { 0xb7034551, SUC_lexicon_tags_41 },
    { 0x77834550, SUC_lexicon_tags_17 },
    { 0x20834553, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8d834555, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc34559, SUC_lexicon_tags_11 },
    { 0xbc43455a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ae3455c, SUC_lexicon_tags_12 },
    { 0xeec3455d, SUC_lexicon_tags_15 },
    { 0x1823455e, SUC_lexicon_tags_12 },
    { 0xc423455e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e34562, SUC_lexicon_tags_1 },
    { 0x1b434563, SUC_lexicon_tags_1 },
    { 0x13e34564, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d634569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e3456f, SUC_lexicon_tags_52 },
    { 0x8d034570, SUC_lexicon_tags_11 },
    { 0x68834570, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0x44834573, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be34576, SUC_lexicon_tags_1 },
    { 0xf9834577, SUC_lexicon_tags_27 },
    { 0xda234576, SUC_lexicon_tags_158 },
    { 0x29034579, SUC_lexicon_tags_2 },
    { 0x84e3457a, SUC_lexicon_tags_47 },
    { 0x31c3457b, SUC_lexicon_tags_65 },
    { 0xa163457c, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xf2e3457e, SUC_lexicon_tags_37 },
    { 0xef63457e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68034582, SUC_lexicon_tags_87 },
    { 0x72634583, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a34589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae3458e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c434593, SUC_lexicon_tags_7 },
    { 0xc6834594, SUC_lexicon_tags_91 },
    { 0x84034593, SUC_lexicon_tags_17 },
    { 0x1e634596, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x62a34598, SUC_lexicon_tags_38 },
    { 0xd5434598, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c3459d, SUC_lexicon_tags_2 },
    { 0x8b23459d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea345a1, SUC_lexicon_tags_30 },
    { 0x31e345a2, SUC_lexicon_tags_19 },
    { 0x9e0345a3, SUC_lexicon_tags_11 },
    { 0xeba345a3, SUC_lexicon_tags_65 },
    { 0x9bc345a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ca345a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a345ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b4345ad, SUC_lexicon_tags_27 },
    { 0xf10345ad, SUC_lexicon_tags_15 },
    { 0x658345af, SUC_lexicon_tags_14 },
    { 0x5ca345af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6c345b2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6345ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfee345bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe04345be, SUC_lexicon_tags_37 },
    { 0x856345be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe2345c1, SUC_lexicon_tags_9 },
    { 0xce4345c2, SUC_lexicon_tags_23 },
    { 0x144345c3, SUC_lexicon_tags_12 },
    { 0x804345c2, SUC_lexicon_tags_12 },
    { 0x69c345c3, SUC_lexicon_tags_9 },
    { 0x42c345c6, SUC_lexicon_tags_0 },
    { 0x720345c6, SUC_lexicon_tags_63 },
    { 0x5c2345c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88c345ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136345d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xebc345da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346345de, SUC_lexicon_tags_2 },
    { 0xc56345de, SUC_lexicon_tags_30 },
    { 0xbc0345e0, SUC_lexicon_tags_12 },
    { 0x560345e1, SUC_lexicon_tags_21 },
    { 0xfea345e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38345e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6345eb, SUC_lexicon_tags_6 },
    { 0xb44345ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x298345ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dc345f0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08345f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56345f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22a345f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x422345fb, SUC_lexicon_tags_10 },
    { 0x8d8345fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2034603, SUC_lexicon_tags_27 },
    { 0xa2e34603, SUC_lexicon_tags_46 },
    { 0xdc834605, SUC_lexicon_tags_49 },
    { 0x46a34606, SUC_lexicon_tags_36 },
    { 0xa7234607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ec34609, SUC_lexicon_tags_31 },
    { 0x9d63460a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97634616, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e03461b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d434620, SUC_lexicon_tags_12 },
    { 0x41034620, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x62a34623, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x53634625, SUC_lexicon_tags_0 },
    { 0x73034626, SUC_lexicon_tags_5 },
    { 0x55a34627, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7823462a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8e3462c, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57434630, SUC_lexicon_tags_25 },
    { 0x23434631, SUC_lexicon_tags_19 },
    { 0x21c34632, SUC_lexicon_tags_14 },
    { 0xd434633, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x16034635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe34639, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa923463d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa34644, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8234649, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33434651, SUC_lexicon_tags_60 },
    { 0x90034651, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69e34654, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf663465f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x4a634661, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe234665, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2034669, SUC_lexicon_tags_7 },
    { 0x62e3466a, SUC_lexicon_tags_56 },
    { 0x23c3466b, SUC_lexicon_tags_63 },
    { 0xb7234669, SUC_lexicon_tags_0 },
    { 0x2243466b, SUC_lexicon_tags_37 },
    { 0x4f03466e, SUC_lexicon_tags_7 },
    { 0xd243466f, SUC_lexicon_tags_17 },
    { 0x52e3466d, SUC_lexicon_tags_1 },
    { 0xf2234671, SUC_lexicon_tags_7 },
    { 0x2ca34671, SUC_lexicon_tags_65 },
    { 0xb3a34673, SUC_lexicon_tags_31 },
    { 0x53e3466d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x13434676, SUC_lexicon_tags_60 },
    { 0x54834676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c23467a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e23467c, SUC_lexicon_tags_27 },
    { 0xafe3467d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1103467f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc34682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a3468a, SUC_lexicon_tags_14 },
    { 0x1623468b, SUC_lexicon_tags_24 },
    { 0xa243468b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4863468e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a34694, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x92e34696, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5943469b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d83469f, SUC_lexicon_tags_82 },
    { 0x582346a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c346a3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420346a8, SUC_lexicon_tags_25 },
    { 0xa2e346a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdcc346ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x306346ae, SUC_lexicon_tags_17 },
    { 0x7e6346af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdaa346b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06346b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd90346b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14346c3, SUC_lexicon_tags_47 },
    { 0xdce346c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x536346c7, SUC_lexicon_tags_37 },
    { 0x300346c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420346cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264346d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd32346d2, SUC_lexicon_tags_17 },
    { 0xd74346d3, SUC_lexicon_tags_37 },
    { 0x9c4346d3, SUC_lexicon_tags_6 },
    { 0x638346d3, SUC_lexicon_tags_28 },
    { 0xca4346d6, SUC_lexicon_tags_11 },
    { 0xed0346d3, SUC_lexicon_tags_60 },
    { 0xa34346d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa60346da, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0346e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa346e5, SUC_lexicon_tags_17 },
    { 0x7ea346e5, SUC_lexicon_tags_16 },
    { 0x9f2346e5, SUC_lexicon_tags_246 },
    { 0xf72346e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c346ed, SUC_lexicon_tags_63 },
    { 0xe22346ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2346f3, SUC_lexicon_tags_11 },
    { 0x370346f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e346f9, SUC_lexicon_tags_38 },
    { 0x130346f9, SUC_lexicon_tags_47 },
    { 0x97c346fb, SUC_lexicon_tags_14 },
    { 0x566346fc, SUC_lexicon_tags_252 },
    { 0x58a346fc, SUC_lexicon_tags_15 },
    { 0x856346f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a34702, SUC_lexicon_tags_4 },
    { 0x40634703, SUC_lexicon_tags_11 },
    { 0x24434703, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80034707, SUC_lexicon_tags_1 },
    { 0x73634707, SUC_lexicon_tags_23 },
    { 0x75a34709, SUC_lexicon_tags_2 },
    { 0x5663470a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c3470f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22e34711, SUC_lexicon_tags_33 },
    { 0xb2034712, SUC_lexicon_tags_31 },
    { 0xc2434711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e034719, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e3471e, SUC_lexicon_tags_60 },
    { 0x9523471f, SUC_lexicon_tags_0 },
    { 0x48034720, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69c34722, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb434725, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61834728, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf403472a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffa3472c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3634732, SUC_lexicon_tags_113 },
    { 0x71434733, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba34737, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3be34739, SUC_lexicon_tags_37 },
    { 0x3163473a, SUC_lexicon_tags_25 },
    { 0x1de3473b, SUC_lexicon_tags_2 },
    { 0xe063473c, SUC_lexicon_tags_47 },
    { 0x3703473d, SUC_lexicon_tags_33 },
    { 0xd2a3473e, SUC_lexicon_tags_7 },
    { 0xeea3473f, SUC_lexicon_tags_1 },
    { 0xbc034740, SUC_lexicon_tags_30 },
    { 0x33434740, SUC_lexicon_tags_1 },
    { 0xf7c3473b, SUC_lexicon_tags_63 },
    { 0xd2e34743, SUC_lexicon_tags_29 },
    { 0xdc43473f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1fe34746, SUC_lexicon_tags_18 },
    { 0x5a634747, SUC_lexicon_tags_23 },
    { 0x1ee34748, SUC_lexicon_tags_37 },
    { 0xf2434747, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x95a3474b, SUC_lexicon_tags_15 },
    { 0xb6c3474b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f23474e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x78834750, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf1634752, SUC_lexicon_tags_17 },
    { 0x60434752, SUC_lexicon_tags_12 },
    { 0xb7434754, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e34757, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c43475e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5434760, SUC_lexicon_tags_61 },
    { 0x45634761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e434764, SUC_lexicon_tags_29 },
    { 0x8834765, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x18634768, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4203476a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd623476d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6434777, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa983477a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe23477c, SUC_lexicon_tags_7 },
    { 0x17e3477d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c234781, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96e34783, SUC_lexicon_tags_10 },
    { 0xade34784, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68c34786, SUC_lexicon_tags_91 },
    { 0x8634787, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3234789, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c3478c, SUC_lexicon_tags_88 },
    { 0xfae3478d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa283478f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26a34791, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72834794, SUC_lexicon_tags_37 },
    { 0xee234795, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90034798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b43479a, SUC_lexicon_tags_23 },
    { 0x923479b, SUC_lexicon_tags_38 },
    { 0xd6a3479c, SUC_lexicon_tags_48 },
    { 0x6b03479d, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6347a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0347a2, SUC_lexicon_tags_8 },
    { 0x668347a3, SUC_lexicon_tags_25 },
    { 0xaee347a4, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876347a8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46347ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704347b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740347b9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x818347bb, SUC_lexicon_tags_62 },
    { 0xa52347bc, SUC_lexicon_tags_7 },
    { 0x8c8347bd, SUC_lexicon_tags_12 },
    { 0xb18347be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x690347c1, SUC_lexicon_tags_31 },
    { 0xae6347c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x70347c5, SUC_lexicon_tags_723 },
    { 0x48347c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf66347c8, SUC_lexicon_tags_26 },
    { 0x854347c9, SUC_lexicon_tags_16 },
    { 0xaee347ca, SUC_lexicon_tags_75 },
    { 0xe46347cb, SUC_lexicon_tags_82 },
    { 0xc5e347cc, SUC_lexicon_tags_21 },
    { 0x64e347cc, SUC_lexicon_tags_1566 },
    { 0x992347ce, SUC_lexicon_tags_81 },
    { 0x44a347cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46347d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6347da, SUC_lexicon_tags_23 },
    { 0xd10347db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8347df, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82347e4, SUC_lexicon_tags_1 },
    { 0xa70347e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0347e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7ea347ea, SUC_lexicon_tags_149 },
    { 0xfe0347eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4347f2, SUC_lexicon_tags_87 },
    { 0xff2347f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2347f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996347fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61034801, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec34805, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8034808, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac3480b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a34814, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e34817, SUC_lexicon_tags_45 },
    { 0xdaa34818, SUC_lexicon_tags_96 },
    { 0x8cc34818, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x13c3481b, SUC_lexicon_tags_23 },
    { 0x5603481b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e34823, SUC_lexicon_tags_17 },
    { 0x7de34824, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x34234826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5343482a, SUC_lexicon_tags_37 },
    { 0x2803482b, SUC_lexicon_tags_27 },
    { 0x8143482c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c34830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c634833, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11634836, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc803483a, SUC_lexicon_tags_33 },
    { 0xfdc3483b, SUC_lexicon_tags_19 },
    { 0x34e3483c, SUC_lexicon_tags_11 },
    { 0xc9c3483d, SUC_lexicon_tags_43 },
    { 0xaee3483d, SUC_lexicon_tags_15 },
    { 0xf723483f, SUC_lexicon_tags_1 },
    { 0x33234840, SUC_lexicon_tags_12 },
    { 0xac34841, SUC_lexicon_tags_37 },
    { 0xc3e34842, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x65e34844, SUC_lexicon_tags_25 },
    { 0xffe34845, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc34848, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9ea3484a, SUC_lexicon_tags_49 },
    { 0x303484a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd403484d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63234851, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc834863, SUC_lexicon_tags_65 },
    { 0x75434864, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffa34866, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x25434869, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce3486c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb603486e, SUC_lexicon_tags_179 },
    { 0xbaa3486f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a34872, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a23487a, SUC_lexicon_tags_29 },
    { 0x7ee3487b, SUC_lexicon_tags_26 },
    { 0xe543487c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a3487f, SUC_lexicon_tags_25 },
    { 0x56434880, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb403488d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a34891, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e34896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea234899, SUC_lexicon_tags_47 },
    { 0x4b234899, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94c3489c, SUC_lexicon_tags_13 },
    { 0xa943489d, SUC_lexicon_tags_1 },
    { 0xedc3489d, SUC_lexicon_tags_23 },
    { 0xeba3489c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xd30348a1, SUC_lexicon_tags_26 },
    { 0x6bc348a1, SUC_lexicon_tags_7 },
    { 0x5f6348a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a348a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x534348a8, SUC_lexicon_tags_87 },
    { 0x6b8348a8, SUC_lexicon_tags_33 },
    { 0x762348aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6348ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c348b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8348b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42348b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58348ba, SUC_lexicon_tags_25 },
    { 0xbb4348bb, SUC_lexicon_tags_1 },
    { 0x876348bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10348c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae348c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8348cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2f2348cd, SUC_lexicon_tags_15 },
    { 0xe94348cd, SUC_lexicon_tags_60 },
    { 0x408348cf, SUC_lexicon_tags_30 },
    { 0xbc4348cf, SUC_lexicon_tags_21 },
    { 0x6d8348d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148348d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x848348d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6e348db, SUC_lexicon_tags_7 },
    { 0x26348dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x558348de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39c348e0, SUC_lexicon_tags_25 },
    { 0x868348e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea348e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc50348ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2348ef, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x3b0348f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344348f7, SUC_lexicon_tags_91 },
    { 0xe3c348f7, SUC_lexicon_tags_1 },
    { 0x340348f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa348fc, SUC_lexicon_tags_27 },
    { 0x70c348fd, SUC_lexicon_tags_65 },
    { 0x5f4348fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67a34900, SUC_lexicon_tags_9 },
    { 0x5ca34901, SUC_lexicon_tags_63 },
    { 0x59c34902, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88634904, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e34906, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e434908, SUC_lexicon_tags_19 },
    { 0x1b634909, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda3490c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xec23490f, SUC_lexicon_tags_25 },
    { 0x3ea3490f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x70634913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f834915, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c34918, SUC_lexicon_tags_2 },
    { 0x26834919, SUC_lexicon_tags_14 },
    { 0xd8e3491a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4503491e, SUC_lexicon_tags_25 },
    { 0x1263491e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a34922, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf803492a, SUC_lexicon_tags_37 },
    { 0x9323492b, SUC_lexicon_tags_57 },
    { 0x363492a, SUC_lexicon_tags_1 },
    { 0x7b83492d, SUC_lexicon_tags_21 },
    { 0x9f63492e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a34932, SUC_lexicon_tags_12 },
    { 0xb9634932, SUC_lexicon_tags_38 },
    { 0x71a34933, SUC_lexicon_tags_2 },
    { 0x3da34935, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbe834937, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f83493b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c23493e, SUC_lexicon_tags_0 },
    { 0x30e3493e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8834941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a434943, SUC_lexicon_tags_14 },
    { 0x33e34944, SUC_lexicon_tags_29 },
    { 0x7ae34945, SUC_lexicon_tags_47 },
    { 0x7f834944, SUC_lexicon_tags_4 },
    { 0xb9634944, SUC_lexicon_tags_7 },
    { 0x8c34944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86e3494a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf023494d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee23494f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a34956, SUC_lexicon_tags_35 },
    { 0x8e34957, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64a34959, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb23495b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8234967, SUC_lexicon_tags_8 },
    { 0xc3634968, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3496b, SUC_lexicon_tags_19 },
    { 0x3083496c, SUC_lexicon_tags_42 },
    { 0x29c3496c, SUC_lexicon_tags_12 },
    { 0x9ee3496c, SUC_lexicon_tags_10 },
    { 0x3823496f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8e34971, SUC_lexicon_tags_17 },
    { 0x2d634972, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda34978, SUC_lexicon_tags_27 },
    { 0xbe034978, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc34982, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece34986, SUC_lexicon_tags_11 },
    { 0x9234986, SUC_lexicon_tags_2 },
    { 0xe9834986, SUC_lexicon_tags_17 },
    { 0x92034986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa3498d, SUC_lexicon_tags_47 },
    { 0xd1e3498d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb634995, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5e834997, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x86c34999, SUC_lexicon_tags_33 },
    { 0x27a3499a, SUC_lexicon_tags_64 },
    { 0x42a3499b, SUC_lexicon_tags_31 },
    { 0x3663499c, SUC_lexicon_tags_17 },
    { 0xee3499a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5083499f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850349a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xede349a6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3b8349a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x64a349aa, SUC_lexicon_tags_25 },
    { 0x78a349ab, SUC_lexicon_tags_26 },
    { 0x39a349aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe56349ae, SUC_lexicon_tags_43 },
    { 0x130349ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92349b7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xaf4349b9, SUC_lexicon_tags_46 },
    { 0xd92349ba, SUC_lexicon_tags_60 },
    { 0x742349b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610349bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea349c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a349c8, SUC_lexicon_tags_39 },
    { 0x92349c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4c0349cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4349d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66a349d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0349df, SUC_lexicon_tags_7 },
    { 0x810349e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd6349e2, SUC_lexicon_tags_31 },
    { 0xc86349e3, SUC_lexicon_tags_13 },
    { 0x8349e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae6349e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a349ea, SUC_lexicon_tags_15 },
    { 0xe38349eb, SUC_lexicon_tags_37 },
    { 0xac6349ec, SUC_lexicon_tags_11 },
    { 0xc9e349ed, SUC_lexicon_tags_12 },
    { 0xd34349ee, SUC_lexicon_tags_2 },
    { 0x238349ef, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814349f4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe94349f6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c349f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c349fc, SUC_lexicon_tags_12 },
    { 0xd24349fd, SUC_lexicon_tags_1 },
    { 0x3d4349fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c834a00, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e34a08, SUC_lexicon_tags_1 },
    { 0x86234a09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d834a0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de34a11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f234a14, SUC_lexicon_tags_12 },
    { 0xa5a34a15, SUC_lexicon_tags_0 },
    { 0x73a34a15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f834a19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f234a1c, SUC_lexicon_tags_12 },
    { 0xa34a1d, SUC_lexicon_tags_25 },
    { 0x1634a1c, SUC_lexicon_tags_38 },
    { 0xc2434a1f, SUC_lexicon_tags_39 },
    { 0xef434a20, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xda634a22, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c34a25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e34a28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb834a2c, SUC_lexicon_tags_7 },
    { 0xaba34a2d, SUC_lexicon_tags_45 },
    { 0xd3a34a2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e834a33, SUC_lexicon_tags_25 },
    { 0x93e34a34, SUC_lexicon_tags_37 },
    { 0xc8434a35, SUC_lexicon_tags_1 },
    { 0x4fc34a33, SUC_lexicon_tags_138 },
    { 0xd0834a37, SUC_lexicon_tags_37 },
    { 0x9e034a37, SUC_lexicon_tags_63 },
    { 0xb5034a39, SUC_lexicon_tags_12 },
    { 0x81a34a3a, SUC_lexicon_tags_0 },
    { 0xace34a3b, SUC_lexicon_tags_47 },
    { 0x30434a3a, SUC_lexicon_tags_15 },
    { 0x8ee34a3c, SUC_lexicon_tags_63 },
    { 0x32c34a39, SUC_lexicon_tags_11 },
    { 0x85234a3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c34a42, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd9634a44, SUC_lexicon_tags_1 },
    { 0x41c34a45, SUC_lexicon_tags_17 },
    { 0xe4034a45, SUC_lexicon_tags_0 },
    { 0x42a34a45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e34a4a, SUC_lexicon_tags_6 },
    { 0xdb634a4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe34a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5634a51, SUC_lexicon_tags_38 },
    { 0x8ce34a52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c234a60, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xe7034a62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3234a65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xec234a67, SUC_lexicon_tags_45 },
    { 0xb0a34a68, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e34a6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x33834a6e, SUC_lexicon_tags_9 },
    { 0x25434a6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde434a72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a634a79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c234a7d, SUC_lexicon_tags_39 },
    { 0xeb434a7e, SUC_lexicon_tags_82 },
    { 0xe4834a7d, SUC_lexicon_tags_37 },
    { 0xbe34a7e, SUC_lexicon_tags_12 },
    { 0x1fa34a81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37034a90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x28834a92, SUC_lexicon_tags_37 },
    { 0x9d634a93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8234a95, SUC_lexicon_tags_16 },
    { 0x88434a96, SUC_lexicon_tags_47 },
    { 0xb7634a97, SUC_lexicon_tags_8 },
    { 0xe2c34a98, SUC_lexicon_tags_37 },
    { 0x8434a98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87a34a9b, SUC_lexicon_tags_17 },
    { 0x5aa34a9c, SUC_lexicon_tags_11 },
    { 0x54834a9c, SUC_lexicon_tags_14 },
    { 0xede34a9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85034aa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d234aa4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20634aab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee34aae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4e234ab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44834ab9, SUC_lexicon_tags_22 },
    { 0xf0634aba, SUC_lexicon_tags_7 },
    { 0x76e34aba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a34aca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c34ad3, SUC_lexicon_tags_11 },
    { 0x57234ad3, SUC_lexicon_tags_11 },
    { 0xfca34ad5, SUC_lexicon_tags_37 },
    { 0x7fc34ad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27834ad8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c834adc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x96834ade, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac34ae2, SUC_lexicon_tags_37 },
    { 0xaf034ae2, SUC_lexicon_tags_12 },
    { 0xc834ae3, SUC_lexicon_tags_43 },
    { 0xf0834ae4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb434ae7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xefc34ae9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63834aec, SUC_lexicon_tags_12 },
    { 0x17634aec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce434aef, SUC_lexicon_tags_110 },
    { 0xb0434af0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81034af2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec234af7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdc34af9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xef034afc, SUC_lexicon_tags_79 },
    { 0x52834afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc34b02, SUC_lexicon_tags_79 },
    { 0x8f034b02, SUC_lexicon_tags_37 },
    { 0xbca34b04, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2a834b06, SUC_lexicon_tags_21 },
    { 0x59434b07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe34b0e, SUC_lexicon_tags_10 },
    { 0xcae34b0f, SUC_lexicon_tags_30 },
    { 0x54e34b10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e034b13, SUC_lexicon_tags_108 },
    { 0x63e34b14, SUC_lexicon_tags_14 },
    { 0xdc834b15, SUC_lexicon_tags_7 },
    { 0x84234b16, SUC_lexicon_tags_79 },
    { 0xeb434b14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49034b21, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95234b26, SUC_lexicon_tags_29 },
    { 0x3d034b27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf634b29, SUC_lexicon_tags_9 },
    { 0xf4234b2a, SUC_lexicon_tags_9 },
    { 0x6a34b2b, SUC_lexicon_tags_25 },
    { 0x4aa34b2b, SUC_lexicon_tags_61 },
    { 0xe7834b2d, SUC_lexicon_tags_80 },
    { 0xdfa34b2d, SUC_lexicon_tags_14 },
    { 0x5ec34b2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c34b32, SUC_lexicon_tags_37 },
    { 0xb0c34b33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c34b37, SUC_lexicon_tags_23 },
    { 0xdf634b37, SUC_lexicon_tags_15 },
    { 0x91634b38, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca34b3e, SUC_lexicon_tags_48 },
    { 0xd6234b3f, SUC_lexicon_tags_45 },
    { 0x9da34b3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8c34b42, SUC_lexicon_tags_17 },
    { 0x36634b42, SUC_lexicon_tags_49 },
    { 0xd7634b44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41e34b46, SUC_lexicon_tags_43 },
    { 0x85e34b46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc34b4a, SUC_lexicon_tags_39 },
    { 0x68234b4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62a34b4d, SUC_lexicon_tags_11 },
    { 0x10a34b4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec434b52, SUC_lexicon_tags_49 },
    { 0x86c34b53, SUC_lexicon_tags_29 },
    { 0x56034b53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a34b58, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c634b5b, SUC_lexicon_tags_26 },
    { 0x9aa34b5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f634b61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c34b66, SUC_lexicon_tags_14 },
    { 0x51034b67, SUC_lexicon_tags_11 },
    { 0x1d834b68, SUC_lexicon_tags_17 },
    { 0x16e34b69, SUC_lexicon_tags_63 },
    { 0xd234b68, SUC_lexicon_tags_47 },
    { 0x1ca34b6b, SUC_lexicon_tags_4 },
    { 0x1c34b6a, SUC_lexicon_tags_56 },
    { 0x8f234b69, SUC_lexicon_tags_1 },
    { 0x2f034b6b, SUC_lexicon_tags_9 },
    { 0xbf034b6f, SUC_lexicon_tags_7 },
    { 0x62634b66, SUC_lexicon_tags_82 },
    { 0xe3034b6d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x57234b74, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf6234b76, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe634b7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37834b83, SUC_lexicon_tags_60 },
    { 0xe634b83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec34b8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e034b8e, SUC_lexicon_tags_47 },
    { 0x53234b8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c434b98, SUC_lexicon_tags_47 },
    { 0xbec34b98, SUC_lexicon_tags_58 },
    { 0x72034b9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac034b9f, SUC_lexicon_tags_37 },
    { 0x5b634b9f, SUC_lexicon_tags_15 },
    { 0x53834ba1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b434ba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11634ba6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb6234ba8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x63834bab, SUC_lexicon_tags_37 },
    { 0x49634bab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xab234bae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd834bb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf834bb3, SUC_lexicon_tags_1 },
    { 0x16634bb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc34bb7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2034bbd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca34bc5, SUC_lexicon_tags_11 },
    { 0xdf234bc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcea34bc8, SUC_lexicon_tags_12 },
    { 0x2f634bc8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed634bcb, SUC_lexicon_tags_63 },
    { 0x45434bcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26634bd0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e34bd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc34bd9, SUC_lexicon_tags_23 },
    { 0x7c634bd9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca34be1, SUC_lexicon_tags_43 },
    { 0x1f034be2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c234be6, SUC_lexicon_tags_6 },
    { 0xb0634be7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb8034be9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5634beb, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0xa4634bed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8434bf2, SUC_lexicon_tags_37 },
    { 0x29c34bf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d834c00, SUC_lexicon_tags_1 },
    { 0xd5434c00, SUC_lexicon_tags_47 },
    { 0xa4e34c01, SUC_lexicon_tags_11 },
    { 0x71434c03, SUC_lexicon_tags_23 },
    { 0xfae34c03, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e34c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a34c0e, SUC_lexicon_tags_15 },
    { 0x2be34c0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c34c16, SUC_lexicon_tags_33 },
    { 0x9ca34c17, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a34c19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75434c1d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc34c23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeda34c25, SUC_lexicon_tags_25 },
    { 0x2c634c25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81434c28, SUC_lexicon_tags_12 },
    { 0xb2634c29, SUC_lexicon_tags_1 },
    { 0xee434c28, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x52034c2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ca34c2e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9634c31, SUC_lexicon_tags_0 },
    { 0xc9434c31, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4e34c34, SUC_lexicon_tags_17 },
    { 0xcaa34c34, SUC_lexicon_tags_14 },
    { 0x1e834c36, SUC_lexicon_tags_27 },
    { 0x3ce34c36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x93434c3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c634c3d, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de34c40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88234c48, SUC_lexicon_tags_37 },
    { 0x1a034c49, SUC_lexicon_tags_83 },
    { 0xc0434c49, SUC_lexicon_tags_17 },
    { 0xd3834c4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c34c54, SUC_lexicon_tags_15 },
    { 0x7b034c55, SUC_lexicon_tags_25 },
    { 0xe6034c56, SUC_lexicon_tags_15 },
    { 0x7cc34c57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c634c5f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40834c65, SUC_lexicon_tags_7 },
    { 0x3a634c66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83234c6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd034c6c, SUC_lexicon_tags_15 },
    { 0x10434c6d, SUC_lexicon_tags_37 },
    { 0x80c34c6d, SUC_lexicon_tags_15 },
    { 0xc1634c6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74834c73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2a234c75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b234c79, SUC_lexicon_tags_0 },
    { 0x77834c7a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c34c7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x25e34c7f, SUC_lexicon_tags_7 },
    { 0xb6a34c80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c34c84, SUC_lexicon_tags_19 },
    { 0xdde34c85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c34c8c, SUC_lexicon_tags_55 },
    { 0x22034c8c, SUC_lexicon_tags_37 },
    { 0xd7634c8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9034c91, SUC_lexicon_tags_82 },
    { 0xfd034c92, SUC_lexicon_tags_25 },
    { 0xb2234c93, SUC_lexicon_tags_0 },
    { 0xb1434c94, SUC_lexicon_tags_6 },
    { 0x2234c95, SUC_lexicon_tags_25 },
    { 0x9e034c95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2434c98, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91434c9d, SUC_lexicon_tags_39 },
    { 0x6cc34c9e, SUC_lexicon_tags_0 },
    { 0x8834c9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4634ca4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff234ca6, SUC_lexicon_tags_1 },
    { 0x52034ca7, SUC_lexicon_tags_84 },
    { 0x76834ca8, SUC_lexicon_tags_7 },
    { 0x18434ca9, SUC_lexicon_tags_15 },
    { 0x95634ca9, SUC_lexicon_tags_23 },
    { 0xbc34caa, SUC_lexicon_tags_23 },
    { 0x8b234ca7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46634cb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac34cb6, SUC_lexicon_tags_10 },
    { 0x95834cb6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5be34cb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c34cbf, SUC_lexicon_tags_14 },
    { 0x1b234cc0, SUC_lexicon_tags_30 },
    { 0xba234cc1, SUC_lexicon_tags_47 },
    { 0x29834cc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a234cc4, SUC_lexicon_tags_0 },
    { 0xb1634cc5, SUC_lexicon_tags_25 },
    { 0xe9434cc5, SUC_lexicon_tags_26 },
    { 0x6aa34cc5, SUC_lexicon_tags_12 },
    { 0xaba34cc6, SUC_lexicon_tags_91 },
    { 0x4a234cc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e234ccc, SUC_lexicon_tags_15 },
    { 0x56e34ccd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45234ccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e34cd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12034cdc, SUC_lexicon_tags_25 },
    { 0x77434cdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56834cdf, SUC_lexicon_tags_25 },
    { 0x22634ce0, SUC_lexicon_tags_7 },
    { 0xaae34ce1, SUC_lexicon_tags_35 },
    { 0x31834ce2, SUC_lexicon_tags_1 },
    { 0xde34ce3, SUC_lexicon_tags_37 },
    { 0x55434ce4, SUC_lexicon_tags_60 },
    { 0x47234ce0, SUC_lexicon_tags_1 },
    { 0xd3634cdf, SUC_lexicon_tags_1 },
    { 0x9834ce7, SUC_lexicon_tags_15 },
    { 0xb2434ce7, SUC_lexicon_tags_29 },
    { 0x39e34ce9, SUC_lexicon_tags_5 },
    { 0x33a34cea, SUC_lexicon_tags_60 },
    { 0x1c434ce9, SUC_lexicon_tags_14 },
    { 0x87234cec, SUC_lexicon_tags_37 },
    { 0xddc34ced, SUC_lexicon_tags_15 },
    { 0xaae34cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc34cf5, SUC_lexicon_tags_1 },
    { 0xb3634cf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0034cf8, SUC_lexicon_tags_37 },
    { 0xcfe34cf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece34cfd, SUC_lexicon_tags_87 },
    { 0x73a34cfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92834d00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f434d02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e34d05, SUC_lexicon_tags_11 },
    { 0xed034d05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe7c34d08, SUC_lexicon_tags_84 },
    { 0x7a634d09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9634d0c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaa034d0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa234d18, SUC_lexicon_tags_212 },
    { 0x7ae34d19, SUC_lexicon_tags_37 },
    { 0xc3434d1a, SUC_lexicon_tags_37 },
    { 0x38c34d18, SUC_lexicon_tags_7 },
    { 0x30834d1c, SUC_lexicon_tags_25 },
    { 0xd9634d18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x59634d1f, SUC_lexicon_tags_37 },
    { 0x71a34d20, SUC_lexicon_tags_0 },
    { 0xa9634d21, SUC_lexicon_tags_49 },
    { 0x40834d1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed234d26, SUC_lexicon_tags_11 },
    { 0x4a834d27, SUC_lexicon_tags_46 },
    { 0x68a34d28, SUC_lexicon_tags_4 },
    { 0x45434d27, SUC_lexicon_tags_27 },
    { 0xa7034d28, SUC_lexicon_tags_17 },
    { 0x5e634d28, SUC_lexicon_tags_17 },
    { 0xe9834d27, SUC_lexicon_tags_15 },
    { 0x6c34d2d, SUC_lexicon_tags_65 },
    { 0xfac34d2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ae34d30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf034d37, SUC_lexicon_tags_17 },
    { 0x91434d38, SUC_lexicon_tags_9 },
    { 0x8c34d38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a434d3b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84234d40, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55834d44, SUC_lexicon_tags_15 },
    { 0x34a34d45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e34d4a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31834d4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4034d50, SUC_lexicon_tags_2 },
    { 0x90c34d50, SUC_lexicon_tags_25 },
    { 0x48c34d52, SUC_lexicon_tags_2 },
    { 0x5be34d51, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x77434d56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29034d58, SUC_lexicon_tags_17 },
    { 0x8a634d59, SUC_lexicon_tags_38 },
    { 0xe6034d5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81034d5c, SUC_lexicon_tags_66 },
    { 0x77034d5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc8234d5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xca834d61, SUC_lexicon_tags_2 },
    { 0xeb234d61, SUC_lexicon_tags_27 },
    { 0x67634d62, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb4434d65, SUC_lexicon_tags_11 },
    { 0xfb034d65, SUC_lexicon_tags_17 },
    { 0x51434d65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa034d69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56234d6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a034d73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x42634d76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac34d79, SUC_lexicon_tags_15 },
    { 0x39834d79, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xafa34d7c, SUC_lexicon_tags_45 },
    { 0xeec34d7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42034d81, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75234d86, SUC_lexicon_tags_25 },
    { 0x98a34d86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa234d8a, SUC_lexicon_tags_25 },
    { 0x90a34d8b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67434d90, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x34634d93, SUC_lexicon_tags_11 },
    { 0xa834d93, SUC_lexicon_tags_37 },
    { 0xab634d93, SUC_lexicon_tags_15 },
    { 0xb2434d96, SUC_lexicon_tags_38 },
    { 0x47c34d95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c834d99, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a34d9c, SUC_lexicon_tags_16 },
    { 0x68634d9c, SUC_lexicon_tags_63 },
    { 0x4f434d9e, SUC_lexicon_tags_43 },
    { 0x64634d9f, SUC_lexicon_tags_7 },
    { 0xa834da0, SUC_lexicon_tags_12 },
    { 0xb9234da1, SUC_lexicon_tags_25 },
    { 0x68634da2, SUC_lexicon_tags_2 },
    { 0x2e634da0, SUC_lexicon_tags_77 },
    { 0xf8634da4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73634da7, SUC_lexicon_tags_47 },
    { 0x86834da7, SUC_lexicon_tags_12 },
    { 0xe4434da9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcce34dab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d834dad, SUC_lexicon_tags_7 },
    { 0x9b034dae, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1034db2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49434db6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26c34db8, SUC_lexicon_tags_33 },
    { 0x4aa34db9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b034dbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13634dbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf234dc7, SUC_lexicon_tags_37 },
    { 0xbca34dc7, SUC_lexicon_tags_12 },
    { 0x40834dc9, SUC_lexicon_tags_37 },
    { 0x7dc34dca, SUC_lexicon_tags_0 },
    { 0x98034dca, SUC_lexicon_tags_47 },
    { 0x6fa34dcb, SUC_lexicon_tags_11 },
    { 0x91c34dcd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf634dd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6234dd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89e34dd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6c34dd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4634dd9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6834ddc, SUC_lexicon_tags_2 },
    { 0x1d634ddd, SUC_lexicon_tags_184 },
    { 0xacc34dde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2634de5, SUC_lexicon_tags_28 },
    { 0xb3434de6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3234de8, SUC_lexicon_tags_17 },
    { 0xb234de9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa634deb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8834dee, SUC_lexicon_tags_11 },
    { 0x67c34def, SUC_lexicon_tags_1 },
    { 0xa0434df0, SUC_lexicon_tags_11 },
    { 0xbbc34def, SUC_lexicon_tags_17 },
    { 0xe8e34df0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d034df7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a434dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a34dfe, SUC_lexicon_tags_87 },
    { 0x78e34dff, SUC_lexicon_tags_12 },
    { 0x89034e00, SUC_lexicon_tags_17 },
    { 0xb0234dfe, SUC_lexicon_tags_35 },
    { 0x6bc34e02, SUC_lexicon_tags_37 },
    { 0xd4434e02, SUC_lexicon_tags_57 },
    { 0x3c34e02, SUC_lexicon_tags_8 },
    { 0xa4e34dff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92834e07, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa34e11, SUC_lexicon_tags_38 },
    { 0xe1834e11, SUC_lexicon_tags_4 },
    { 0xfd034e13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a34e17, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc34e1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x89434e1c, SUC_lexicon_tags_29 },
    { 0x95234e1d, SUC_lexicon_tags_37 },
    { 0x76834e1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82434e21, SUC_lexicon_tags_15 },
    { 0x8b434e22, SUC_lexicon_tags_1 },
    { 0x4a634e23, SUC_lexicon_tags_63 },
    { 0x32c34e24, SUC_lexicon_tags_49 },
    { 0xc034e24, SUC_lexicon_tags_38 },
    { 0x36434e24, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe9e34e28, SUC_lexicon_tags_34 },
    { 0xf034e28, SUC_lexicon_tags_15 },
    { 0xd0834e2a, SUC_lexicon_tags_10 },
    { 0x80a34e2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a34e2e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e34e34, SUC_lexicon_tags_21 },
    { 0x15e34e35, SUC_lexicon_tags_2 },
    { 0xd6834e34, SUC_lexicon_tags_12 },
    { 0x6a234e37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a34e45, SUC_lexicon_tags_29 },
    { 0x67834e45, SUC_lexicon_tags_37 },
    { 0x84234e47, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba34e4a, SUC_lexicon_tags_194 },
    { 0xcf034e4a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x49234e4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6034e50, SUC_lexicon_tags_0 },
    { 0x6fc34e51, SUC_lexicon_tags_1 },
    { 0x8ae34e51, SUC_lexicon_tags_7 },
    { 0x63234e53, SUC_lexicon_tags_12 },
    { 0xe6434e54, SUC_lexicon_tags_25 },
    { 0xd8c34e50, SUC_lexicon_tags_32 },
    { 0xb9e34e56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2634e58, SUC_lexicon_tags_17 },
    { 0xb034e59, SUC_lexicon_tags_25 },
    { 0x82e34e59, SUC_lexicon_tags_4 },
    { 0xb5634e5b, SUC_lexicon_tags_33 },
    { 0xf4834e5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada34e61, SUC_lexicon_tags_2 },
    { 0x69e34e62, SUC_lexicon_tags_24 },
    { 0xb4434e63, SUC_lexicon_tags_8 },
    { 0x7b434e64, SUC_lexicon_tags_15 },
    { 0x39434e63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x434e68, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c34e6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd834e72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c434e7b, SUC_lexicon_tags_25 },
    { 0xca34e7b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e034e80, SUC_lexicon_tags_14 },
    { 0xa9c34e80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20834e85, SUC_lexicon_tags_39 },
    { 0x21234e85, SUC_lexicon_tags_1 },
    { 0x7b834e87, SUC_lexicon_tags_31 },
    { 0x7d834e88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56634e8e, SUC_lexicon_tags_0 },
    { 0x3c34e8e, SUC_lexicon_tags_47 },
    { 0x6fa34e8f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5834e94, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e34e97, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe2234e99, SUC_lexicon_tags_7 },
    { 0x98e34e99, SUC_lexicon_tags_0 },
    { 0xecc34e9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f234e9f, SUC_lexicon_tags_39 },
    { 0x4de34ea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e34ea4, SUC_lexicon_tags_31 },
    { 0x76e34ea5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f634ea9, SUC_lexicon_tags_25 },
    { 0x1e034eaa, SUC_lexicon_tags_27 },
    { 0x80834ea9, SUC_lexicon_tags_12 },
    { 0x6e434eac, SUC_lexicon_tags_37 },
    { 0xaf634ea9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c34eb2, SUC_lexicon_tags_17 },
    { 0x62634eb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4634eb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee34ebb, SUC_lexicon_tags_17 },
    { 0x62834ebc, SUC_lexicon_tags_45 },
    { 0x28a34ebd, SUC_lexicon_tags_17 },
    { 0xfde34ebe, SUC_lexicon_tags_6 },
    { 0xd7834ebd, SUC_lexicon_tags_12 },
    { 0x2de34ec0, SUC_lexicon_tags_2 },
    { 0x68c34ebd, SUC_lexicon_tags_30 },
    { 0xe7634ec2, SUC_lexicon_tags_17 },
    { 0x10434ec3, SUC_lexicon_tags_11 },
    { 0x80c34ec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a234ec8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a34ecc, SUC_lexicon_tags_8 },
    { 0xb1c34ecd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad834ed2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59034ed8, SUC_lexicon_tags_31 },
    { 0xf9c34ed9, SUC_lexicon_tags_15 },
    { 0x9d634eda, SUC_lexicon_tags_47 },
    { 0xf1a34ed8, SUC_lexicon_tags_6 },
    { 0xd2034ed8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c34ee1, SUC_lexicon_tags_34 },
    { 0x3d834ee2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x26434ee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f634eed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe8234eef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3434ef1, SUC_lexicon_tags_12 },
    { 0xfcc34ef2, SUC_lexicon_tags_25 },
    { 0xe7434ef3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x82e34ef5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b034ef7, SUC_lexicon_tags_17 },
    { 0xcbc34ef8, SUC_lexicon_tags_15 },
    { 0xfc434ef7, SUC_lexicon_tags_37 },
    { 0x86834ef9, SUC_lexicon_tags_17 },
    { 0x82c34ef7, SUC_lexicon_tags_5 },
    { 0xbd034efa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11434efe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39834f02, SUC_lexicon_tags_15 },
    { 0x8fe34f03, SUC_lexicon_tags_29 },
    { 0x16834f02, SUC_lexicon_tags_15 },
    { 0x29e34f05, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae34f0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e34f11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e834f13, SUC_lexicon_tags_0 },
    { 0xbec34f14, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2834f18, SUC_lexicon_tags_17 },
    { 0x7fc34f19, SUC_lexicon_tags_47 },
    { 0x40e34f19, SUC_lexicon_tags_15 },
    { 0x87e34f1b, SUC_lexicon_tags_16 },
    { 0x6e034f1c, SUC_lexicon_tags_9 },
    { 0x2434f1d, SUC_lexicon_tags_37 },
    { 0xc7c34f1d, SUC_lexicon_tags_29 },
    { 0xf2c34f1f, SUC_lexicon_tags_6 },
    { 0x8bc34f19, SUC_lexicon_tags_47 },
    { 0x5d234f18, SUC_lexicon_tags_15 },
    { 0x6e834f22, SUC_lexicon_tags_23 },
    { 0x6c34f23, SUC_lexicon_tags_79 },
    { 0x3634f23, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3434f2a, SUC_lexicon_tags_1 },
    { 0xb1a34f2b, SUC_lexicon_tags_14 },
    { 0x90234f2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46434f2e, SUC_lexicon_tags_31 },
    { 0xbda34f2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c034f32, SUC_lexicon_tags_17 },
    { 0xb3834f32, SUC_lexicon_tags_25 },
    { 0x83c34f34, SUC_lexicon_tags_13 },
    { 0x22634f35, SUC_lexicon_tags_7 },
    { 0x38e34f36, SUC_lexicon_tags_30 },
    { 0x6f034f32, SUC_lexicon_tags_13 },
    { 0xe8c34f32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac34f41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc34f43, SUC_lexicon_tags_25 },
    { 0x8a034f43, SUC_lexicon_tags_60 },
    { 0x89434f45, SUC_lexicon_tags_15 },
    { 0x7cc34f46, SUC_lexicon_tags_30 },
    { 0x75034f47, SUC_lexicon_tags_87 },
    { 0x72434f48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9034f4a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x41834f4d, SUC_lexicon_tags_23 },
    { 0xf3a34f4e, SUC_lexicon_tags_11 },
    { 0xa4434f4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x10234f52, SUC_lexicon_tags_25 },
    { 0x1a234f53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56434f56, SUC_lexicon_tags_39 },
    { 0x36034f57, SUC_lexicon_tags_10 },
    { 0x45a34f58, SUC_lexicon_tags_27 },
    { 0x91034f56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e34f63, SUC_lexicon_tags_19 },
    { 0xaf434f63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c34f6a, SUC_lexicon_tags_1 },
    { 0x56634f6a, SUC_lexicon_tags_13 },
    { 0x47e34f6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13434f70, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4634f74, SUC_lexicon_tags_28 },
    { 0xf1434f75, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xde234f77, SUC_lexicon_tags_11 },
    { 0x81c34f77, SUC_lexicon_tags_1 },
    { 0xe5a34f79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f434f7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8b634f7f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce34f82, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e34f88, SUC_lexicon_tags_32 },
    { 0xc2434f88, SUC_lexicon_tags_13 },
    { 0x10834f89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16834f8e, SUC_lexicon_tags_37 },
    { 0x9fa34f8f, SUC_lexicon_tags_45 },
    { 0x57034f90, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6434f94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c234f97, SUC_lexicon_tags_45 },
    { 0x6c234f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c634f9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc7434f9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc7c34f9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8634fa1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x63834fa4, SUC_lexicon_tags_60 },
    { 0xaa034fa5, SUC_lexicon_tags_15 },
    { 0x42a34fa6, SUC_lexicon_tags_68 },
    { 0x90a34fa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c34fac, SUC_lexicon_tags_44 },
    { 0xb7034fad, SUC_lexicon_tags_11 },
    { 0xcfc34fae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96634fb0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88634fb6, SUC_lexicon_tags_27 },
    { 0x8fc34fb7, SUC_lexicon_tags_37 },
    { 0xb4434fb8, SUC_lexicon_tags_165 },
    { 0xa0634fb9, SUC_lexicon_tags_11 },
    { 0xb8634fb9, SUC_lexicon_tags_25 },
    { 0x55834fbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2234fbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee34fc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0434fcb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcda34fcd, SUC_lexicon_tags_12 },
    { 0x86a34fcd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7ce34fd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61834fd2, SUC_lexicon_tags_1 },
    { 0x29834fd2, SUC_lexicon_tags_25 },
    { 0x9c834fd4, SUC_lexicon_tags_10 },
    { 0xfac34fd5, SUC_lexicon_tags_68 },
    { 0xd0e34fd4, SUC_lexicon_tags_31 },
    { 0x66234fd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f634fdb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41634fe6, SUC_lexicon_tags_0 },
    { 0x92e34fe6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38034feb, SUC_lexicon_tags_27 },
    { 0xba234fec, SUC_lexicon_tags_11 },
    { 0xaa434fed, SUC_lexicon_tags_38 },
    { 0x6ae34fec, SUC_lexicon_tags_7 },
    { 0x5e634fef, SUC_lexicon_tags_30 },
    { 0x95834fee, SUC_lexicon_tags_13 },
    { 0x7a034ff1, SUC_lexicon_tags_26 },
    { 0x32834ff2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa6c34ff4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb634ff6, SUC_lexicon_tags_25 },
    { 0x77634ff7, SUC_lexicon_tags_34 },
    { 0x53a34ff8, SUC_lexicon_tags_40 },
    { 0x35634ff9, SUC_lexicon_tags_6 },
    { 0x80634ff6, SUC_lexicon_tags_8 },
    { 0xcac34ffb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x37834ffd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88434fff, SUC_lexicon_tags_17 },
    { 0x41434fff, SUC_lexicon_tags_17 },
    { 0xc2835001, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e35004, SUC_lexicon_tags_39 },
    { 0xb3235004, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xef035008, SUC_lexicon_tags_15 },
    { 0xb8035008, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e3500c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9cc3500e, SUC_lexicon_tags_25 },
    { 0xce63500e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a35012, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7d235014, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b635017, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xca35019, SUC_lexicon_tags_19 },
    { 0xf063501a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c83501d, SUC_lexicon_tags_91 },
    { 0x69e3501d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xefc35020, SUC_lexicon_tags_7 },
    { 0x74035020, SUC_lexicon_tags_2 },
    { 0xdb835022, SUC_lexicon_tags_1 },
    { 0xdb035023, SUC_lexicon_tags_49 },
    { 0xa9635023, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ce35026, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10035028, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37835038, SUC_lexicon_tags_17 },
    { 0xd2e35039, SUC_lexicon_tags_12 },
    { 0x7e63503a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa003503c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3a63503e, SUC_lexicon_tags_23 },
    { 0xb383503e, SUC_lexicon_tags_23 },
    { 0x74e3503f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdb235042, SUC_lexicon_tags_25 },
    { 0xb0235043, SUC_lexicon_tags_65 },
    { 0x1fe35042, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97835048, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1203504e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f435051, SUC_lexicon_tags_57 },
    { 0xecc35051, SUC_lexicon_tags_11 },
    { 0xe6635053, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe35058, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4d83505a, SUC_lexicon_tags_11 },
    { 0x3ae3505a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba83505f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4183506d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x53435070, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60035073, SUC_lexicon_tags_25 },
    { 0x49635074, SUC_lexicon_tags_63 },
    { 0x66e35073, SUC_lexicon_tags_7 },
    { 0x94835076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a35079, SUC_lexicon_tags_0 },
    { 0xcee3507a, SUC_lexicon_tags_12 },
    { 0xe43507a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf643507d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a43507f, SUC_lexicon_tags_33 },
    { 0x70435080, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x83835083, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c35086, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1263508c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8783508e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58635091, SUC_lexicon_tags_12 },
    { 0xc7435092, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66635097, SUC_lexicon_tags_25 },
    { 0x9a235098, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb303509c, SUC_lexicon_tags_12 },
    { 0xf443509c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986350a1, SUC_lexicon_tags_26 },
    { 0x8d2350a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c350a8, SUC_lexicon_tags_26 },
    { 0x344350a8, SUC_lexicon_tags_161 },
    { 0x866350aa, SUC_lexicon_tags_1 },
    { 0xfe2350ab, SUC_lexicon_tags_21 },
    { 0x324350ac, SUC_lexicon_tags_48 },
    { 0x48350aa, SUC_lexicon_tags_47 },
    { 0x416350ae, SUC_lexicon_tags_12 },
    { 0x6da350ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a350b3, SUC_lexicon_tags_37 },
    { 0xbb4350b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34350b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcc2350b8, SUC_lexicon_tags_25 },
    { 0xb14350b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f2350bb, SUC_lexicon_tags_27 },
    { 0xbc6350bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c350c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeec350c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124350c8, SUC_lexicon_tags_43 },
    { 0x88350c9, SUC_lexicon_tags_16 },
    { 0xa82350ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb58350cc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12350d0, SUC_lexicon_tags_84 },
    { 0xede350d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c350d8, SUC_lexicon_tags_8 },
    { 0xa3a350d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4350df, SUC_lexicon_tags_47 },
    { 0xf4c350df, SUC_lexicon_tags_25 },
    { 0xae0350e1, SUC_lexicon_tags_43 },
    { 0x774350e2, SUC_lexicon_tags_43 },
    { 0xef8350e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a350ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14350ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc350f2, SUC_lexicon_tags_34 },
    { 0xb5a350f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902350f7, SUC_lexicon_tags_15 },
    { 0xc64350f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94c350fa, SUC_lexicon_tags_63 },
    { 0xa88350fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c35104, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1635107, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f23510c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fa3510e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed835117, SUC_lexicon_tags_32 },
    { 0x7da35118, SUC_lexicon_tags_37 },
    { 0xaae35118, SUC_lexicon_tags_1 },
    { 0x9e63511a, SUC_lexicon_tags_12 },
    { 0x8ac3511a, SUC_lexicon_tags_25 },
    { 0xd3c3511c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc023511e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe35121, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c35127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10835129, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9fa3512b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e35130, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f235133, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa2a35135, SUC_lexicon_tags_11 },
    { 0x4a035135, SUC_lexicon_tags_7 },
    { 0x1a35137, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x883513a, SUC_lexicon_tags_11 },
    { 0xa403513b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c3513e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61835145, SUC_lexicon_tags_47 },
    { 0x7ba35146, SUC_lexicon_tags_29 },
    { 0x48c35145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62035149, SUC_lexicon_tags_92 },
    { 0x73435149, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d63514e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a635152, SUC_lexicon_tags_15 },
    { 0x32435153, SUC_lexicon_tags_47 },
    { 0x48e35152, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de35159, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c3515c, SUC_lexicon_tags_47 },
    { 0xe9e3515d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a03515f, SUC_lexicon_tags_38 },
    { 0x77235160, SUC_lexicon_tags_17 },
    { 0x83235161, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1635165, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8b635167, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23516f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0635172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e35176, SUC_lexicon_tags_35 },
    { 0xc9635177, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b43517b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53235180, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x80235183, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37835187, SUC_lexicon_tags_25 },
    { 0xde635188, SUC_lexicon_tags_7 },
    { 0xa3c35189, SUC_lexicon_tags_17 },
    { 0xe3235189, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e035193, SUC_lexicon_tags_11 },
    { 0xb4a35194, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56235199, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9923519f, SUC_lexicon_tags_25 },
    { 0xcd63519f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4351a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568351a9, SUC_lexicon_tags_9 },
    { 0x560351aa, SUC_lexicon_tags_9 },
    { 0x67e351ab, SUC_lexicon_tags_8 },
    { 0x79c351a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e351b1, SUC_lexicon_tags_17 },
    { 0x868351b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a351b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6351bb, SUC_lexicon_tags_37 },
    { 0x39a351bb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4351c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee351c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x798351c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e351ca, SUC_lexicon_tags_11 },
    { 0x8c6351ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x820351ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2351d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c6351d5, SUC_lexicon_tags_26 },
    { 0x71e351d6, SUC_lexicon_tags_49 },
    { 0xabe351d7, SUC_lexicon_tags_23 },
    { 0x5c8351d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a351db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90351de, SUC_lexicon_tags_15 },
    { 0x6ae351de, SUC_lexicon_tags_26 },
    { 0xea4351e0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x288351e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2351e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec351ee, SUC_lexicon_tags_1 },
    { 0x594351ee, SUC_lexicon_tags_11 },
    { 0x246351f0, SUC_lexicon_tags_47 },
    { 0x114351ef, SUC_lexicon_tags_26 },
    { 0x506351f2, SUC_lexicon_tags_1 },
    { 0x2d2351f2, SUC_lexicon_tags_87 },
    { 0xa48351f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x14351f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd88351f8, SUC_lexicon_tags_62 },
    { 0x8bc351f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588351fd, SUC_lexicon_tags_19 },
    { 0xfb6351fd, SUC_lexicon_tags_17 },
    { 0xa9c351fe, SUC_lexicon_tags_25 },
    { 0x9f435200, SUC_lexicon_tags_12 },
    { 0x4fc35200, SUC_lexicon_tags_17 },
    { 0xd70351fd, SUC_lexicon_tags_11 },
    { 0xed035203, SUC_lexicon_tags_40 },
    { 0xf4035203, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb635208, SUC_lexicon_tags_68 },
    { 0x38635209, SUC_lexicon_tags_1 },
    { 0x4c3520a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e435210, SUC_lexicon_tags_7 },
    { 0xf5635210, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x73035213, SUC_lexicon_tags_11 },
    { 0xc3435214, SUC_lexicon_tags_60 },
    { 0x78635215, SUC_lexicon_tags_65 },
    { 0xb8035214, SUC_lexicon_tags_11 },
    { 0x235215, SUC_lexicon_tags_57 },
    { 0x3a235215, SUC_lexicon_tags_25 },
    { 0x98e35219, SUC_lexicon_tags_2 },
    { 0x66a3521a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc35221, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa435228, SUC_lexicon_tags_81 },
    { 0x31635228, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4835230, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4835232, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b835234, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c3523a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac35241, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde235243, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29035245, SUC_lexicon_tags_37 },
    { 0x44635246, SUC_lexicon_tags_8 },
    { 0x6d835247, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c3524c, SUC_lexicon_tags_15 },
    { 0xa363524d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x28435250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6c35252, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a3525b, SUC_lexicon_tags_17 },
    { 0x86a3525c, SUC_lexicon_tags_33 },
    { 0x3ce3525d, SUC_lexicon_tags_15 },
    { 0x6583525d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45235260, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x69635263, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71835265, SUC_lexicon_tags_12 },
    { 0x21435266, SUC_lexicon_tags_8 },
    { 0xc5035267, SUC_lexicon_tags_46 },
    { 0x66e35267, SUC_lexicon_tags_15 },
    { 0x9d835267, SUC_lexicon_tags_17 },
    { 0x9c235267, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x55a3526c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6743526f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe2435271, SUC_lexicon_tags_179 },
    { 0xf7835271, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa263527d, SUC_lexicon_tags_45 },
    { 0x2c63527e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e035280, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5035288, SUC_lexicon_tags_4 },
    { 0xcca35289, SUC_lexicon_tags_2 },
    { 0x47435289, SUC_lexicon_tags_12 },
    { 0x1c83528a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x923528e, SUC_lexicon_tags_17 },
    { 0x6303528e, SUC_lexicon_tags_25 },
    { 0xe6c35290, SUC_lexicon_tags_12 },
    { 0x2bc35291, SUC_lexicon_tags_60 },
    { 0x67e35292, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12035296, SUC_lexicon_tags_60 },
    { 0xb9e35297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b63529b, SUC_lexicon_tags_7 },
    { 0x4cc3529c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96352a1, SUC_lexicon_tags_1 },
    { 0xd12352a2, SUC_lexicon_tags_43 },
    { 0xc76352a3, SUC_lexicon_tags_3 },
    { 0x13a352a4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x298352a7, SUC_lexicon_tags_45 },
    { 0xb72352a8, SUC_lexicon_tags_17 },
    { 0x8ba352a9, SUC_lexicon_tags_45 },
    { 0x196352a9, SUC_lexicon_tags_43 },
    { 0x24e352ab, SUC_lexicon_tags_11 },
    { 0x942352ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c352b0, SUC_lexicon_tags_8 },
    { 0xa06352b1, SUC_lexicon_tags_11 },
    { 0xdda352b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522352bc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498352c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8352c7, SUC_lexicon_tags_33 },
    { 0xe76352c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38352cb, SUC_lexicon_tags_7 },
    { 0xeec352cc, SUC_lexicon_tags_47 },
    { 0x67e352cd, SUC_lexicon_tags_7 },
    { 0xed0352ce, SUC_lexicon_tags_17 },
    { 0xbc8352ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a352d2, SUC_lexicon_tags_46 },
    { 0xbd0352d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0352d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a352d9, SUC_lexicon_tags_17 },
    { 0xa0a352da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x96a352dc, SUC_lexicon_tags_38 },
    { 0x2d2352dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574352e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6352e7, SUC_lexicon_tags_44 },
    { 0xde8352e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe352ed, SUC_lexicon_tags_15 },
    { 0x448352ee, SUC_lexicon_tags_4 },
    { 0x926352ef, SUC_lexicon_tags_0 },
    { 0xd72352ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x364352f3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6352f6, SUC_lexicon_tags_106 },
    { 0x50c352f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128352fc, SUC_lexicon_tags_8 },
    { 0x21a352fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a35302, SUC_lexicon_tags_14 },
    { 0x4b035303, SUC_lexicon_tags_21 },
    { 0xa8835302, SUC_lexicon_tags_2 },
    { 0x51835305, SUC_lexicon_tags_17 },
    { 0xf8435306, SUC_lexicon_tags_25 },
    { 0xea035306, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1c635309, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x79e3530b, SUC_lexicon_tags_12 },
    { 0x98a3530c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x84c3530e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2e35310, SUC_lexicon_tags_25 },
    { 0x73235311, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36c35313, SUC_lexicon_tags_27 },
    { 0x1b435314, SUC_lexicon_tags_63 },
    { 0x87a35313, SUC_lexicon_tags_17 },
    { 0x6235316, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d83531a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb123531c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf635321, SUC_lexicon_tags_47 },
    { 0x11635322, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44435328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8083532e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8435330, SUC_lexicon_tags_31 },
    { 0xeae35331, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e35334, SUC_lexicon_tags_7 },
    { 0xa7835335, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee3533a, SUC_lexicon_tags_21 },
    { 0x8fc3533b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c3533f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x51c35341, SUC_lexicon_tags_7 },
    { 0x8b435342, SUC_lexicon_tags_37 },
    { 0xf4635343, SUC_lexicon_tags_28 },
    { 0xee635344, SUC_lexicon_tags_60 },
    { 0xe2435344, SUC_lexicon_tags_7 },
    { 0xd635346, SUC_lexicon_tags_1 },
    { 0x9f835346, SUC_lexicon_tags_48 },
    { 0xf5835344, SUC_lexicon_tags_26 },
    { 0x88235346, SUC_lexicon_tags_7 },
    { 0x1123534a, SUC_lexicon_tags_37 },
    { 0x59a35349, SUC_lexicon_tags_29 },
    { 0xffe3534c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a435350, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54035354, SUC_lexicon_tags_27 },
    { 0x8a435354, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa3535a, SUC_lexicon_tags_1134 },
    { 0x9da3535b, SUC_lexicon_tags_12 },
    { 0x8843535a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e35361, SUC_lexicon_tags_56 },
    { 0x1ec35362, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ba35364, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85e35366, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58035369, SUC_lexicon_tags_0 },
    { 0x6ce35369, SUC_lexicon_tags_45 },
    { 0x2783536a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c3536f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89235374, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c35377, SUC_lexicon_tags_2 },
    { 0x6c835378, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x98a3537a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40035380, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7c835382, SUC_lexicon_tags_25 },
    { 0x59435382, SUC_lexicon_tags_15 },
    { 0xd635384, SUC_lexicon_tags_17 },
    { 0xc9c35385, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe603538a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75235391, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e3539a, SUC_lexicon_tags_15 },
    { 0x7c63539b, SUC_lexicon_tags_11 },
    { 0x40a3539c, SUC_lexicon_tags_10 },
    { 0xd163539d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6353a1, SUC_lexicon_tags_11 },
    { 0xf82353a2, SUC_lexicon_tags_11 },
    { 0x8da353a3, SUC_lexicon_tags_11 },
    { 0xbf6353a4, SUC_lexicon_tags_30 },
    { 0x8f4353a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70353a8, SUC_lexicon_tags_70 },
    { 0xad4353a8, SUC_lexicon_tags_37 },
    { 0x18c353aa, SUC_lexicon_tags_47 },
    { 0x5ba353a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e353b3, SUC_lexicon_tags_15 },
    { 0xf6353b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52353b7, SUC_lexicon_tags_27 },
    { 0xc06353b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e353bb, SUC_lexicon_tags_31 },
    { 0xf0353bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7be353be, SUC_lexicon_tags_19 },
    { 0x924353bf, SUC_lexicon_tags_7 },
    { 0x9f2353c0, SUC_lexicon_tags_49 },
    { 0x78e353c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648353c5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06353c8, SUC_lexicon_tags_108 },
    { 0xcc2353c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e353cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4353d1, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc353d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x182353d9, SUC_lexicon_tags_17 },
    { 0x1e4353da, SUC_lexicon_tags_37 },
    { 0xcf2353d9, SUC_lexicon_tags_29 },
    { 0x3e353dc, SUC_lexicon_tags_30 },
    { 0x678353d9, SUC_lexicon_tags_7 },
    { 0x6c6353de, SUC_lexicon_tags_77 },
    { 0x3c4353da, SUC_lexicon_tags_49 },
    { 0xf4c353e0, SUC_lexicon_tags_12 },
    { 0xffe353e1, SUC_lexicon_tags_12 },
    { 0xd94353e2, SUC_lexicon_tags_69 },
    { 0x7c6353e3, SUC_lexicon_tags_7 },
    { 0xefc353e4, SUC_lexicon_tags_77 },
    { 0xfd6353e5, SUC_lexicon_tags_25 },
    { 0x63c353e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2353ea, SUC_lexicon_tags_37 },
    { 0x2b6353ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c353f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e353f9, SUC_lexicon_tags_47 },
    { 0x4a6353fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ca353fc, SUC_lexicon_tags_30 },
    { 0xe2c353fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x18835400, SUC_lexicon_tags_38 },
    { 0x5c435400, SUC_lexicon_tags_25 },
    { 0x1c635402, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5c35404, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3035408, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5703540a, SUC_lexicon_tags_17 },
    { 0xa1e3540b, SUC_lexicon_tags_7 },
    { 0x8723540a, SUC_lexicon_tags_12 },
    { 0x4d23540d, SUC_lexicon_tags_12 },
    { 0xa1a3540a, SUC_lexicon_tags_48 },
    { 0x5f63540e, SUC_lexicon_tags_160 },
    { 0x80235410, SUC_lexicon_tags_15 },
    { 0xbe635411, SUC_lexicon_tags_1 },
    { 0xef435412, SUC_lexicon_tags_11 },
    { 0x27635412, SUC_lexicon_tags_8 },
    { 0x77c35414, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b835418, SUC_lexicon_tags_25 },
    { 0x57835418, SUC_lexicon_tags_7 },
    { 0xad235418, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45835421, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a35424, SUC_lexicon_tags_11 },
    { 0x54c35425, SUC_lexicon_tags_70 },
    { 0x3435426, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe3542b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8183542d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba35430, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a35439, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc9c3543b, SUC_lexicon_tags_15 },
    { 0x43c3543c, SUC_lexicon_tags_68 },
    { 0x8fc3543d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4ba3543f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e35442, SUC_lexicon_tags_25 },
    { 0xbc035442, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x72435445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae635448, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1923544c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf623544f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c835451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50635454, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xad035457, SUC_lexicon_tags_25 },
    { 0x8fe35457, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5de3545a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4ca3545c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6983545e, SUC_lexicon_tags_25 },
    { 0x1a63545f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2da35461, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a35468, SUC_lexicon_tags_37 },
    { 0xb7035469, SUC_lexicon_tags_11 },
    { 0xc5c35468, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd723546c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda835471, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x50635474, SUC_lexicon_tags_8 },
    { 0x7b035475, SUC_lexicon_tags_63 },
    { 0x81835476, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb263547d, SUC_lexicon_tags_15 },
    { 0xd363547d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc35486, SUC_lexicon_tags_28 },
    { 0x8c035487, SUC_lexicon_tags_27 },
    { 0xcda35488, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9835492, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8bc35494, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf5035496, SUC_lexicon_tags_27 },
    { 0x6c035497, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab43549f, SUC_lexicon_tags_12 },
    { 0x76a3549f, SUC_lexicon_tags_11 },
    { 0x2323549f, SUC_lexicon_tags_7 },
    { 0x87c354a2, SUC_lexicon_tags_47 },
    { 0x15a354a3, SUC_lexicon_tags_85 },
    { 0xb9e354a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36c354a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110354ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78354ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c354b1, SUC_lexicon_tags_31 },
    { 0x61a354b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02354bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e354c6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2354c9, SUC_lexicon_tags_79 },
    { 0xae2354c9, SUC_lexicon_tags_14 },
    { 0xbe6354cb, SUC_lexicon_tags_15 },
    { 0xca4354cb, SUC_lexicon_tags_1 },
    { 0x5a0354cd, SUC_lexicon_tags_49 },
    { 0x6dc354cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e354d4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6354d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee354db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d2354dd, SUC_lexicon_tags_105 },
    { 0x4a8354de, SUC_lexicon_tags_21 },
    { 0xecc354df, SUC_lexicon_tags_31 },
    { 0x47c354e0, SUC_lexicon_tags_12 },
    { 0x1a4354e0, SUC_lexicon_tags_27 },
    { 0x6c4354e2, SUC_lexicon_tags_8 },
    { 0xc0354e0, SUC_lexicon_tags_17 },
    { 0x4ec354de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02354e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e2354ea, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x6a2354ec, SUC_lexicon_tags_1 },
    { 0x904354ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c6354ef, SUC_lexicon_tags_66 },
    { 0xfae354ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2c354f2, SUC_lexicon_tags_28 },
    { 0x4a2354f3, SUC_lexicon_tags_1 },
    { 0x850354f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65c354f6, SUC_lexicon_tags_15 },
    { 0x364354f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbae354f9, SUC_lexicon_tags_15 },
    { 0xbb0354fa, SUC_lexicon_tags_32 },
    { 0x40e354fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7c2354fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa35502, SUC_lexicon_tags_0 },
    { 0x78c35502, SUC_lexicon_tags_11 },
    { 0xcf235503, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f835507, SUC_lexicon_tags_16 },
    { 0x18835508, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1943550d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1035511, SUC_lexicon_tags_23 },
    { 0x43435512, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x65635515, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef23551b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ea3551d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee35520, SUC_lexicon_tags_160 },
    { 0x42835521, SUC_lexicon_tags_7 },
    { 0x6ba35522, SUC_lexicon_tags_0 },
    { 0xdc35521, SUC_lexicon_tags_5 },
    { 0x40e35524, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed035528, SUC_lexicon_tags_67 },
    { 0x5b035529, SUC_lexicon_tags_15 },
    { 0x5f43552a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4223552c, SUC_lexicon_tags_12 },
    { 0x72e3552c, SUC_lexicon_tags_7 },
    { 0x9023552e, SUC_lexicon_tags_26 },
    { 0x47c3552f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae35533, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x47035535, SUC_lexicon_tags_37 },
    { 0xa8835536, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6303553b, SUC_lexicon_tags_0 },
    { 0xff03553c, SUC_lexicon_tags_14 },
    { 0x9f23553c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e35541, SUC_lexicon_tags_23 },
    { 0x61c35542, SUC_lexicon_tags_43 },
    { 0xdde35541, SUC_lexicon_tags_65 },
    { 0x48435541, SUC_lexicon_tags_9 },
    { 0xe9a35542, SUC_lexicon_tags_11 },
    { 0xcce35546, SUC_lexicon_tags_11 },
    { 0xec835547, SUC_lexicon_tags_9 },
    { 0x6ce35548, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a3554c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e3554f, SUC_lexicon_tags_25 },
    { 0xbe435550, SUC_lexicon_tags_17 },
    { 0xcfe35551, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbe35553, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d435557, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x58e35559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7323555c, SUC_lexicon_tags_7 },
    { 0x6483555d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa35562, SUC_lexicon_tags_0 },
    { 0x42035563, SUC_lexicon_tags_4 },
    { 0x94435563, SUC_lexicon_tags_17 },
    { 0x50035565, SUC_lexicon_tags_35 },
    { 0xde435563, SUC_lexicon_tags_6 },
    { 0xa9c35567, SUC_lexicon_tags_15 },
    { 0x63035566, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc843556b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed835576, SUC_lexicon_tags_47 },
    { 0xb035576, SUC_lexicon_tags_38 },
    { 0x46435578, SUC_lexicon_tags_14 },
    { 0x8d835576, SUC_lexicon_tags_129 },
    { 0x6f635577, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2383557d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc235585, SUC_lexicon_tags_7 },
    { 0x92035586, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85635588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae23558c, SUC_lexicon_tags_0 },
    { 0x3d03558c, SUC_lexicon_tags_11 },
    { 0xb503558e, SUC_lexicon_tags_11 },
    { 0x7b43558f, SUC_lexicon_tags_7 },
    { 0x61a3558c, SUC_lexicon_tags_25 },
    { 0x30835591, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xc7c35593, SUC_lexicon_tags_37 },
    { 0xb0835594, SUC_lexicon_tags_87 },
    { 0x41435595, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe823559a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa103559d, SUC_lexicon_tags_27 },
    { 0x2923559d, SUC_lexicon_tags_37 },
    { 0xe763559f, SUC_lexicon_tags_1 },
    { 0xae83559e, SUC_lexicon_tags_27 },
    { 0x71c355a1, SUC_lexicon_tags_26 },
    { 0x412355a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2355a7, SUC_lexicon_tags_0 },
    { 0xb8355a8, SUC_lexicon_tags_2 },
    { 0x6d8355a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x910355ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38e355ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742355b2, SUC_lexicon_tags_15 },
    { 0xce4355b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0355b9, SUC_lexicon_tags_1 },
    { 0xfcc355b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd6355bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6355bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf36355c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa355c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe355c8, SUC_lexicon_tags_37 },
    { 0xfd4355c8, SUC_lexicon_tags_13 },
    { 0x9a0355ca, SUC_lexicon_tags_15 },
    { 0x5c8355c9, SUC_lexicon_tags_56 },
    { 0x538355cc, SUC_lexicon_tags_26 },
    { 0xf12355c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11e355cf, SUC_lexicon_tags_179 },
    { 0x460355d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8e355d2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x54355d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4355d8, SUC_lexicon_tags_5 },
    { 0x740355d9, SUC_lexicon_tags_1 },
    { 0x9f2355da, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514355df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x918355e1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xa58355e3, SUC_lexicon_tags_3 },
    { 0x19e355e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf30355e6, SUC_lexicon_tags_37 },
    { 0x29a355e7, SUC_lexicon_tags_46 },
    { 0xf80355e8, SUC_lexicon_tags_46 },
    { 0x770355e9, SUC_lexicon_tags_47 },
    { 0xac355e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc355ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x798355f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370355f6, SUC_lexicon_tags_17 },
    { 0x82a355f6, SUC_lexicon_tags_25 },
    { 0x5c2355f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6355fc, SUC_lexicon_tags_15 },
    { 0x818355fd, SUC_lexicon_tags_2 },
    { 0x2b8355fe, SUC_lexicon_tags_30 },
    { 0x45c355ff, SUC_lexicon_tags_1 },
    { 0x146355ff, SUC_lexicon_tags_12 },
    { 0xe9035601, SUC_lexicon_tags_15 },
    { 0x2d2355ff, SUC_lexicon_tags_47 },
    { 0xa54355fd, SUC_lexicon_tags_27 },
    { 0x2c235604, SUC_lexicon_tags_15 },
    { 0x510355ff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b635608, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa863560a, SUC_lexicon_tags_7 },
    { 0x4b43560b, SUC_lexicon_tags_11 },
    { 0x3423560b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfee3560e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca435612, SUC_lexicon_tags_47 },
    { 0x64435613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x44435616, SUC_lexicon_tags_48 },
    { 0x7435617, SUC_lexicon_tags_15 },
    { 0x66435618, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e35621, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc35625, SUC_lexicon_tags_201 },
    { 0x8e235626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8235629, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da3562d, SUC_lexicon_tags_38 },
    { 0x6483562d, SUC_lexicon_tags_27 },
    { 0x4503562e, SUC_lexicon_tags_1 },
    { 0x78435630, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62235634, SUC_lexicon_tags_11 },
    { 0x3b435634, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd283563e, SUC_lexicon_tags_12 },
    { 0x7383563f, SUC_lexicon_tags_64 },
    { 0x46c3563e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c35646, SUC_lexicon_tags_34 },
    { 0x1fe35646, SUC_lexicon_tags_0 },
    { 0xf9e35648, SUC_lexicon_tags_116 },
    { 0xafe35649, SUC_lexicon_tags_29 },
    { 0x3a03564a, SUC_lexicon_tags_11 },
    { 0x64835648, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10835651, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba83565b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c3566c, SUC_lexicon_tags_7 },
    { 0x2943566c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xda03566f, SUC_lexicon_tags_11 },
    { 0xf4435670, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea35674, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61a35676, SUC_lexicon_tags_11 },
    { 0xe0035677, SUC_lexicon_tags_14 },
    { 0x4ba35678, SUC_lexicon_tags_8 },
    { 0xd8435679, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac3567e, SUC_lexicon_tags_9 },
    { 0x9f63567f, SUC_lexicon_tags_11 },
    { 0xc963567e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e435683, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd035685, SUC_lexicon_tags_16 },
    { 0xaac35686, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf23568b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea235693, SUC_lexicon_tags_26 },
    { 0x31435694, SUC_lexicon_tags_12 },
    { 0x21035695, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e356a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be356a5, SUC_lexicon_tags_23 },
    { 0x9d4356a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x37c356a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae356ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252356b2, SUC_lexicon_tags_17 },
    { 0x2ae356b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a356ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48356be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6356c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeba356c6, SUC_lexicon_tags_23 },
    { 0x5c356c6, SUC_lexicon_tags_37 },
    { 0x1de356c6, SUC_lexicon_tags_61 },
    { 0x7e6356c6, SUC_lexicon_tags_14 },
    { 0xf9e356c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdba356cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb4356ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77a356d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4356d3, SUC_lexicon_tags_1 },
    { 0xdc6356d3, SUC_lexicon_tags_12 },
    { 0x342356d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8356d8, SUC_lexicon_tags_47 },
    { 0xf40356d9, SUC_lexicon_tags_39 },
    { 0x9e4356d9, SUC_lexicon_tags_25 },
    { 0x430356db, SUC_lexicon_tags_33 },
    { 0x1c4356dc, SUC_lexicon_tags_16 },
    { 0x544356dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa356df, SUC_lexicon_tags_82 },
    { 0x8b4356e0, SUC_lexicon_tags_9 },
    { 0x6d0356e1, SUC_lexicon_tags_11 },
    { 0x766356e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294356eb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a356ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718356f6, SUC_lexicon_tags_38 },
    { 0xbc356f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea356ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa35702, SUC_lexicon_tags_12 },
    { 0x77c35702, SUC_lexicon_tags_47 },
    { 0x27035704, SUC_lexicon_tags_25 },
    { 0xe35705, SUC_lexicon_tags_15 },
    { 0xd0c35704, SUC_lexicon_tags_33 },
    { 0xa2835702, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1943570b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3783570d, SUC_lexicon_tags_25 },
    { 0x5c3570d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec35715, SUC_lexicon_tags_17 },
    { 0xf4835715, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8635718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec3571c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa923571e, SUC_lexicon_tags_23 },
    { 0x13c3571e, SUC_lexicon_tags_4 },
    { 0x5a63571f, SUC_lexicon_tags_16 },
    { 0xf4e35721, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50035724, SUC_lexicon_tags_25 },
    { 0xa9a35724, SUC_lexicon_tags_37 },
    { 0x83e35726, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5943572c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5035732, SUC_lexicon_tags_17 },
    { 0x79035733, SUC_lexicon_tags_229 },
    { 0x4cc35732, SUC_lexicon_tags_62 },
    { 0x30235732, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32235737, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac3573f, SUC_lexicon_tags_112 },
    { 0x1d235740, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x93835742, SUC_lexicon_tags_25 },
    { 0xeda35742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18e35745, SUC_lexicon_tags_11 },
    { 0x12a35745, SUC_lexicon_tags_37 },
    { 0xdd635745, SUC_lexicon_tags_30 },
    { 0x60835748, SUC_lexicon_tags_15 },
    { 0x15e35748, SUC_lexicon_tags_0 },
    { 0x60435749, SUC_lexicon_tags_14 },
    { 0x70435747, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d635750, SUC_lexicon_tags_6 },
    { 0x1b435750, SUC_lexicon_tags_47 },
    { 0xb7635751, SUC_lexicon_tags_30 },
    { 0x16635752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa35756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4035758, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a3575c, SUC_lexicon_tags_23 },
    { 0x6f63575d, SUC_lexicon_tags_3 },
    { 0xe363575d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb035764, SUC_lexicon_tags_11 },
    { 0xe1035765, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee3576d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa123576f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b635774, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba635779, SUC_lexicon_tags_11 },
    { 0xce63577a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x643577c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a235780, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8c35782, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c35785, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8435787, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93035789, SUC_lexicon_tags_37 },
    { 0xf0c35789, SUC_lexicon_tags_15 },
    { 0x1423578a, SUC_lexicon_tags_61 },
    { 0xf1c3578a, SUC_lexicon_tags_9 },
    { 0xdd43578d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e35790, SUC_lexicon_tags_7 },
    { 0xe6c35791, SUC_lexicon_tags_1 },
    { 0xd9e35792, SUC_lexicon_tags_43 },
    { 0xb9635793, SUC_lexicon_tags_31 },
    { 0xf3435794, SUC_lexicon_tags_7 },
    { 0x85235793, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f23579b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b83579d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdde3579f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec357a9, SUC_lexicon_tags_2 },
    { 0xa3e357a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c357b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456357b6, SUC_lexicon_tags_11 },
    { 0xe9c357b6, SUC_lexicon_tags_1 },
    { 0xefe357b8, SUC_lexicon_tags_55 },
    { 0x16c357b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x71e357bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xede357be, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x382357c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18c357c2, SUC_lexicon_tags_46 },
    { 0xfba357c3, SUC_lexicon_tags_37 },
    { 0x106357c2, SUC_lexicon_tags_32 },
    { 0xdec357c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x714357c8, SUC_lexicon_tags_17 },
    { 0x3be357c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x346357cc, SUC_lexicon_tags_28 },
    { 0x1fe357cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84357d1, SUC_lexicon_tags_14 },
    { 0xe94357d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2357d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe30357d7, SUC_lexicon_tags_15 },
    { 0x71e357d7, SUC_lexicon_tags_60 },
    { 0xbee357d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2357dc, SUC_lexicon_tags_17 },
    { 0xb8a357dc, SUC_lexicon_tags_25 },
    { 0x154357dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4357e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da357e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70357ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x14a357ec, SUC_lexicon_tags_42 },
    { 0x2fa357ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ae357ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76357f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd80357f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00357fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0357fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58235802, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa983580b, SUC_lexicon_tags_14 },
    { 0x69e3580b, SUC_lexicon_tags_30 },
    { 0x6143580b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e35815, SUC_lexicon_tags_12 },
    { 0x24835815, SUC_lexicon_tags_33 },
    { 0xb7435815, SUC_lexicon_tags_15 },
    { 0xbe35817, SUC_lexicon_tags_31 },
    { 0xdb235815, SUC_lexicon_tags_37 },
    { 0x2ca3581a, SUC_lexicon_tags_11 },
    { 0x6fe3581b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3581e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc35821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36435823, SUC_lexicon_tags_55 },
    { 0x58235824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfac35826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d83582a, SUC_lexicon_tags_1 },
    { 0xf0c3582a, SUC_lexicon_tags_11 },
    { 0x7423582c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xca63582f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c35835, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xad235838, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b63583b, SUC_lexicon_tags_49 },
    { 0x4863583c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea3583f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e35842, SUC_lexicon_tags_37 },
    { 0xdda35843, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6e35845, SUC_lexicon_tags_61 },
    { 0xb1035846, SUC_lexicon_tags_12 },
    { 0x2fc35847, SUC_lexicon_tags_7 },
    { 0xcae35846, SUC_lexicon_tags_23 },
    { 0x4f035849, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7c3584b, SUC_lexicon_tags_44 },
    { 0x74a3584b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1463584f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f235853, SUC_lexicon_tags_33 },
    { 0x78c35854, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71035859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b43585b, SUC_lexicon_tags_29 },
    { 0xca03585b, SUC_lexicon_tags_2 },
    { 0xfd83585d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae035860, SUC_lexicon_tags_26 },
    { 0xee835861, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff435863, SUC_lexicon_tags_32 },
    { 0x46235863, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a35866, SUC_lexicon_tags_43 },
    { 0x57435866, SUC_lexicon_tags_29 },
    { 0xdc35867, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7463586c, SUC_lexicon_tags_25 },
    { 0xd383586d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e83586f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83435875, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd2035877, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b435879, SUC_lexicon_tags_17 },
    { 0xff835879, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13835880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x83e35882, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x57635884, SUC_lexicon_tags_9 },
    { 0x7b035885, SUC_lexicon_tags_2 },
    { 0x79e35885, SUC_lexicon_tags_37 },
    { 0xb3c35884, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd2c35889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf703588d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e35890, SUC_lexicon_tags_25 },
    { 0xc1c35891, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34435898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc23589c, SUC_lexicon_tags_66 },
    { 0x6943589d, SUC_lexicon_tags_37 },
    { 0xaa03589d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdea358a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e358a4, SUC_lexicon_tags_11 },
    { 0x7f4358a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x620358a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52358aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b4358ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac0358ae, SUC_lexicon_tags_38 },
    { 0x6ca358af, SUC_lexicon_tags_37 },
    { 0x56a358af, SUC_lexicon_tags_84 },
    { 0xd5e358ae, SUC_lexicon_tags_1 },
    { 0x860358b2, SUC_lexicon_tags_19 },
    { 0xab4358b2, SUC_lexicon_tags_31 },
    { 0x54a358b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee358ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4358be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a358c1, SUC_lexicon_tags_0 },
    { 0xb66358c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x82358c4, SUC_lexicon_tags_39 },
    { 0x764358c5, SUC_lexicon_tags_37 },
    { 0x4fe358c4, SUC_lexicon_tags_37 },
    { 0xde358c4, SUC_lexicon_tags_20 },
    { 0x5f8358c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104358cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde358cf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x328358d2, SUC_lexicon_tags_25 },
    { 0x514358d2, SUC_lexicon_tags_6 },
    { 0xb82358d2, SUC_lexicon_tags_12 },
    { 0x2bc358d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x240358d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51e358d9, SUC_lexicon_tags_61 },
    { 0x210358da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x984358dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38358e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e358e3, SUC_lexicon_tags_14 },
    { 0xcda358e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa358e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa4c358e8, SUC_lexicon_tags_60 },
    { 0x1f0358e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10358ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614358f3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02358f7, SUC_lexicon_tags_11 },
    { 0xe2e358f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0358fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17835903, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x61235905, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x78c35907, SUC_lexicon_tags_29 },
    { 0xb0a35908, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c3590d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8223590f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6435912, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e3591a, SUC_lexicon_tags_45 },
    { 0x80c3591a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x84a3591d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd383591f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a35926, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x93835929, SUC_lexicon_tags_37 },
    { 0x9c3592a, SUC_lexicon_tags_11 },
    { 0x8063592b, SUC_lexicon_tags_6 },
    { 0x27e3592b, SUC_lexicon_tags_45 },
    { 0xeba3592b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7e3592f, SUC_lexicon_tags_66 },
    { 0x83635930, SUC_lexicon_tags_26 },
    { 0x3e235931, SUC_lexicon_tags_1 },
    { 0x5463592f, SUC_lexicon_tags_2 },
    { 0x7d435933, SUC_lexicon_tags_23 },
    { 0xcf035934, SUC_lexicon_tags_65 },
    { 0x69635934, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae3593b, SUC_lexicon_tags_28 },
    { 0x14a3593c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58035941, SUC_lexicon_tags_60 },
    { 0x7fc35942, SUC_lexicon_tags_37 },
    { 0x92835943, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc35947, SUC_lexicon_tags_38 },
    { 0x56635948, SUC_lexicon_tags_27 },
    { 0xc2235949, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c3594d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbda3594f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7fa35951, SUC_lexicon_tags_45 },
    { 0xeb235952, SUC_lexicon_tags_25 },
    { 0xa9c35953, SUC_lexicon_tags_9 },
    { 0xa1e35951, SUC_lexicon_tags_37 },
    { 0xf6c35955, SUC_lexicon_tags_15 },
    { 0xfc035956, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5103595d, SUC_lexicon_tags_55 },
    { 0x1303595d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9235960, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc35964, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a835969, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6503596c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe743596e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bc35970, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c35974, SUC_lexicon_tags_11 },
    { 0x42a35974, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e35978, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc3597c, SUC_lexicon_tags_31 },
    { 0x84c3597c, SUC_lexicon_tags_141 },
    { 0x6dc3597d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c35981, SUC_lexicon_tags_1 },
    { 0x4e035981, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca35985, SUC_lexicon_tags_25 },
    { 0x13435986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6623598c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc035990, SUC_lexicon_tags_8 },
    { 0x2cc35990, SUC_lexicon_tags_7 },
    { 0xd0c35992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66635996, SUC_lexicon_tags_13 },
    { 0x54e35996, SUC_lexicon_tags_32 },
    { 0xf1e35998, SUC_lexicon_tags_31 },
    { 0x68635999, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620359a1, SUC_lexicon_tags_38 },
    { 0xa8e359a2, SUC_lexicon_tags_37 },
    { 0x722359a3, SUC_lexicon_tags_25 },
    { 0x406359a4, SUC_lexicon_tags_12 },
    { 0x7ae359a5, SUC_lexicon_tags_17 },
    { 0xa2c359a3, SUC_lexicon_tags_38 },
    { 0xeca359a7, SUC_lexicon_tags_11 },
    { 0xe90359a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238359ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2359b1, SUC_lexicon_tags_7 },
    { 0x30359b2, SUC_lexicon_tags_23 },
    { 0x830359b3, SUC_lexicon_tags_15 },
    { 0x6d0359b3, SUC_lexicon_tags_11 },
    { 0x16359b5, SUC_lexicon_tags_42 },
    { 0x6a0359b3, SUC_lexicon_tags_47 },
    { 0xe3c359b7, SUC_lexicon_tags_37 },
    { 0x990359b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd72359ba, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xbb8359bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaca359be, SUC_lexicon_tags_1 },
    { 0xa52359be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc18359c1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4359c9, SUC_lexicon_tags_23 },
    { 0x76a359ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ec359cc, SUC_lexicon_tags_87 },
    { 0xbac359cd, SUC_lexicon_tags_47 },
    { 0xc30359ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68359d8, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0359db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84e359dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ae359df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410359e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5a359e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c359e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde4359ec, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04359f0, SUC_lexicon_tags_0 },
    { 0xcda359f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a359f4, SUC_lexicon_tags_54 },
    { 0x3ac359f4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0359f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456359fc, SUC_lexicon_tags_17 },
    { 0x900359fd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce35a01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2835a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c235a0b, SUC_lexicon_tags_15 },
    { 0x1c635a0c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a35a12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a235a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47635a1c, SUC_lexicon_tags_163 },
    { 0x29235a1d, SUC_lexicon_tags_47 },
    { 0x3b835a1d, SUC_lexicon_tags_37 },
    { 0x60035a1f, SUC_lexicon_tags_1 },
    { 0x2d435a1c, SUC_lexicon_tags_39 },
    { 0x85e35a21, SUC_lexicon_tags_30 },
    { 0x1e635a22, SUC_lexicon_tags_38 },
    { 0x41635a23, SUC_lexicon_tags_23 },
    { 0xc1835a24, SUC_lexicon_tags_6 },
    { 0xfb235a23, SUC_lexicon_tags_27 },
    { 0x99e35a26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2035a28, SUC_lexicon_tags_15 },
    { 0x2d235a29, SUC_lexicon_tags_12 },
    { 0xdc435a2a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b635a2e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea35a34, SUC_lexicon_tags_25 },
    { 0xa8635a35, SUC_lexicon_tags_1 },
    { 0xee635a35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce35a3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d635a3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc35a41, SUC_lexicon_tags_43 },
    { 0xcda35a41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc8835a44, SUC_lexicon_tags_17 },
    { 0x5e435a45, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7835a49, SUC_lexicon_tags_25 },
    { 0x97235a49, SUC_lexicon_tags_7 },
    { 0xfb635a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a35a4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c35a50, SUC_lexicon_tags_12 },
    { 0x62635a51, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e35a55, SUC_lexicon_tags_12 },
    { 0x2b035a56, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcc035a58, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed035a5c, SUC_lexicon_tags_65 },
    { 0xc0235a5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67635a5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xae235a62, SUC_lexicon_tags_15 },
    { 0x9835a62, SUC_lexicon_tags_38 },
    { 0x18835a64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c35a6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0035a6e, SUC_lexicon_tags_17 },
    { 0xe8835a6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c35a72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c35a78, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c35a80, SUC_lexicon_tags_14 },
    { 0x9a835a81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2035a83, SUC_lexicon_tags_1004 },
    { 0x85435a84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac35a88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e035a8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a35a90, SUC_lexicon_tags_30 },
    { 0x6e635a91, SUC_lexicon_tags_60 },
    { 0xad235a92, SUC_lexicon_tags_65 },
    { 0xe6235a93, SUC_lexicon_tags_45 },
    { 0xcde35a94, SUC_lexicon_tags_25 },
    { 0xf7c35a95, SUC_lexicon_tags_25 },
    { 0x99435a96, SUC_lexicon_tags_25 },
    { 0xd8835a90, SUC_lexicon_tags_63 },
    { 0x94a35a96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x28835a9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5035aa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54235aa6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c35ab0, SUC_lexicon_tags_46 },
    { 0x57a35ab0, SUC_lexicon_tags_56 },
    { 0x1435ab0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33235ab4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe835ab9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5435abf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b035ac2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3635ac4, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e835ac7, SUC_lexicon_tags_11 },
    { 0xd9435ac8, SUC_lexicon_tags_25 },
    { 0xb1035ac9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x89c35acb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c35acf, SUC_lexicon_tags_25 },
    { 0x6f235ad0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed235ad2, SUC_lexicon_tags_15 },
    { 0x9a635ad2, SUC_lexicon_tags_1 },
    { 0xa1635ad4, SUC_lexicon_tags_15 },
    { 0xb9035ad5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e35ad9, SUC_lexicon_tags_87 },
    { 0x54835ad9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd635adc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98635ade, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xeb235ae0, SUC_lexicon_tags_12 },
    { 0xda835ae1, SUC_lexicon_tags_1 },
    { 0xe5035ae2, SUC_lexicon_tags_38 },
    { 0xec435ae3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa1c35ae5, SUC_lexicon_tags_12 },
    { 0x8b635ae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12035ae8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade35af1, SUC_lexicon_tags_12 },
    { 0x40435af2, SUC_lexicon_tags_29 },
    { 0x3b435af3, SUC_lexicon_tags_60 },
    { 0xc3e35af1, SUC_lexicon_tags_15 },
    { 0x84235af5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a235afa, SUC_lexicon_tags_14 },
    { 0xa9635afb, SUC_lexicon_tags_15 },
    { 0x22435afc, SUC_lexicon_tags_32 },
    { 0xbca35afb, SUC_lexicon_tags_37 },
    { 0x46035afc, SUC_lexicon_tags_4 },
    { 0x15a35aff, SUC_lexicon_tags_6 },
    { 0xcd435afc, SUC_lexicon_tags_37 },
    { 0x6aa35aff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaec35b03, SUC_lexicon_tags_17 },
    { 0x84835b03, SUC_lexicon_tags_30 },
    { 0xfce35b04, SUC_lexicon_tags_28 },
    { 0x85e35b05, SUC_lexicon_tags_12 },
    { 0xce235b07, SUC_lexicon_tags_15 },
    { 0x13c35b08, SUC_lexicon_tags_43 },
    { 0x56c35b08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9f435b0b, SUC_lexicon_tags_17 },
    { 0xd1a35b0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6235b10, SUC_lexicon_tags_27 },
    { 0xd7a35b11, SUC_lexicon_tags_1 },
    { 0x66a35b12, SUC_lexicon_tags_1 },
    { 0x36a35b13, SUC_lexicon_tags_27 },
    { 0xcf835b14, SUC_lexicon_tags_79 },
    { 0x6f035b15, SUC_lexicon_tags_14 },
    { 0xf2c35b16, SUC_lexicon_tags_63 },
    { 0xed035b17, SUC_lexicon_tags_1 },
    { 0x8835b18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe35b1f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36035b21, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdb235b23, SUC_lexicon_tags_17 },
    { 0x79a35b24, SUC_lexicon_tags_2 },
    { 0x58035b24, SUC_lexicon_tags_1042 },
    { 0, NULL },
    { 0x46035b27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c635b2a, SUC_lexicon_tags_11 },
    { 0xe0e35b2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x42835b2e, SUC_lexicon_tags_37 },
    { 0xd3e35b2f, SUC_lexicon_tags_11 },
    { 0x73635b2f, SUC_lexicon_tags_19 },
    { 0x3e635b31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2635b34, SUC_lexicon_tags_2 },
    { 0x99635b34, SUC_lexicon_tags_8 },
    { 0xa8a35b36, SUC_lexicon_tags_1 },
    { 0xe635b37, SUC_lexicon_tags_1 },
    { 0x50435b36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb8635b3a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x54635b3d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2835b41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc35b46, SUC_lexicon_tags_45 },
    { 0x24c35b47, SUC_lexicon_tags_6 },
    { 0x3e235b48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed635b51, SUC_lexicon_tags_0 },
    { 0x63835b52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48035b58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a35b5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6435b5e, SUC_lexicon_tags_11 },
    { 0xd7835b5f, SUC_lexicon_tags_1 },
    { 0x6f635b5f, SUC_lexicon_tags_65 },
    { 0x39a35b61, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e35b64, SUC_lexicon_tags_7 },
    { 0xe5a35b65, SUC_lexicon_tags_48 },
    { 0xed635b64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a635b69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c35b6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9635b72, SUC_lexicon_tags_14 },
    { 0xee235b73, SUC_lexicon_tags_17 },
    { 0xdea35b74, SUC_lexicon_tags_1 },
    { 0xfb035b73, SUC_lexicon_tags_8 },
    { 0x22235b76, SUC_lexicon_tags_14 },
    { 0xb7035b73, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8635b7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f435b7d, SUC_lexicon_tags_37 },
    { 0x6ae35b7e, SUC_lexicon_tags_31 },
    { 0x80235b7f, SUC_lexicon_tags_33 },
    { 0xd2035b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfa35b82, SUC_lexicon_tags_46 },
    { 0x5ba35b82, SUC_lexicon_tags_47 },
    { 0x9f235b84, SUC_lexicon_tags_0 },
    { 0x18035b85, SUC_lexicon_tags_5 },
    { 0xaba35b86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6435b8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12835b8d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x69235b8f, SUC_lexicon_tags_10 },
    { 0x8bc35b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc35b95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc0e35b97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2635b9c, SUC_lexicon_tags_43 },
    { 0xce635b9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca035ba3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec35ba7, SUC_lexicon_tags_0 },
    { 0xa7035ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fe35baa, SUC_lexicon_tags_7 },
    { 0x86c35baa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e35bb0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe3a35bb2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e35bb6, SUC_lexicon_tags_45 },
    { 0x59635bb6, SUC_lexicon_tags_63 },
    { 0xa2e35bb8, SUC_lexicon_tags_12 },
    { 0x98e35bb9, SUC_lexicon_tags_17 },
    { 0x71e35bb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10435bbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6835bbe, SUC_lexicon_tags_12 },
    { 0x77835bbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6835bc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd635bc4, SUC_lexicon_tags_1 },
    { 0x8e835bc4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x37a35bc7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x54435bca, SUC_lexicon_tags_15 },
    { 0x4d235bcb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x20235bce, SUC_lexicon_tags_45 },
    { 0x8de35bcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69035bd8, SUC_lexicon_tags_42 },
    { 0x22035bd9, SUC_lexicon_tags_37 },
    { 0xbde35bd8, SUC_lexicon_tags_1 },
    { 0xd8c35bdb, SUC_lexicon_tags_6 },
    { 0x62e35bda, SUC_lexicon_tags_25 },
    { 0xd1235bdd, SUC_lexicon_tags_30 },
    { 0x8c35bd9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee35be5, SUC_lexicon_tags_15 },
    { 0x5e835be6, SUC_lexicon_tags_116 },
    { 0x3f235be6, SUC_lexicon_tags_37 },
    { 0x23a35be8, SUC_lexicon_tags_28 },
    { 0x6f435be6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81235bef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee35bf2, SUC_lexicon_tags_60 },
    { 0x85e35bf3, SUC_lexicon_tags_33 },
    { 0x8f035bf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22635bfb, SUC_lexicon_tags_15 },
    { 0x65035bfc, SUC_lexicon_tags_43 },
    { 0x24635bfd, SUC_lexicon_tags_6 },
    { 0xbc635bfe, SUC_lexicon_tags_11 },
    { 0x63435bfc, SUC_lexicon_tags_65 },
    { 0x28a35c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e435c02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8435c04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ae35c06, SUC_lexicon_tags_27 },
    { 0xa235c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc35c0b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a635c10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80435c15, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x88835c17, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc235c1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11035c30, SUC_lexicon_tags_9 },
    { 0x5835c30, SUC_lexicon_tags_17 },
    { 0x93c35c32, SUC_lexicon_tags_49 },
    { 0xaee35c33, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x99a35c35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe35c38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8635c3a, SUC_lexicon_tags_7 },
    { 0x12e35c3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60835c3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3235c40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56635c44, SUC_lexicon_tags_14 },
    { 0x25035c45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c235c49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42635c4c, SUC_lexicon_tags_15 },
    { 0xdc635c4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x14635c4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5635c58, SUC_lexicon_tags_24 },
    { 0x27635c58, SUC_lexicon_tags_60 },
    { 0x91835c5a, SUC_lexicon_tags_74 },
    { 0x55c35c5a, SUC_lexicon_tags_31 },
    { 0xd035c5c, SUC_lexicon_tags_87 },
    { 0x76035c5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x50035c5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a35c66, SUC_lexicon_tags_55 },
    { 0x8a235c67, SUC_lexicon_tags_25 },
    { 0xf6635c68, SUC_lexicon_tags_39 },
    { 0xdf635c69, SUC_lexicon_tags_47 },
    { 0x70035c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ac35c6c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x48e35c6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61235c70, SUC_lexicon_tags_10 },
    { 0x15e35c71, SUC_lexicon_tags_7 },
    { 0x85435c70, SUC_lexicon_tags_7 },
    { 0xd7e35c72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b635c75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21435c79, SUC_lexicon_tags_542 },
    { 0x2c435c79, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9f235c7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce635c7f, SUC_lexicon_tags_68 },
    { 0xaf235c80, SUC_lexicon_tags_25 },
    { 0xa4035c81, SUC_lexicon_tags_19 },
    { 0xb9435c82, SUC_lexicon_tags_171 },
    { 0x40035c80, SUC_lexicon_tags_37 },
    { 0x6fc35c84, SUC_lexicon_tags_52 },
    { 0x83635c84, SUC_lexicon_tags_11 },
    { 0x4b835c86, SUC_lexicon_tags_27 },
    { 0x32435c87, SUC_lexicon_tags_0 },
    { 0x10c35c88, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf235c8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6235c90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3035c93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77435c95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a635c98, SUC_lexicon_tags_32 },
    { 0x9d635c99, SUC_lexicon_tags_2 },
    { 0xf0835c9a, SUC_lexicon_tags_273 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c9f, SUC_lexicon_tags_49 },
    { 0x72835ca0, SUC_lexicon_tags_39 },
    { 0xfe235ca1, SUC_lexicon_tags_27 },
    { 0x5835ca2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6435ca4, SUC_lexicon_tags_37 },
    { 0xa3a35ca4, SUC_lexicon_tags_35 },
    { 0x6235ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68035ca8, SUC_lexicon_tags_0 },
    { 0x13235ca9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x24a35cab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6a35cad, SUC_lexicon_tags_27 },
    { 0x6a35cad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85635cb0, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x43235cb3, SUC_lexicon_tags_28 },
    { 0x69035cb3, SUC_lexicon_tags_1 },
    { 0xf4c35cb4, SUC_lexicon_tags_43 },
    { 0x22e35cb6, SUC_lexicon_tags_9 },
    { 0x64435cb7, SUC_lexicon_tags_61 },
    { 0xdee35cb6, SUC_lexicon_tags_49 },
    { 0x4f435cb9, SUC_lexicon_tags_15 },
    { 0x4de35cba, SUC_lexicon_tags_15 },
    { 0x8da35cbb, SUC_lexicon_tags_27 },
    { 0x44035cb9, SUC_lexicon_tags_27 },
    { 0x89835cbc, SUC_lexicon_tags_19 },
    { 0xe4c35cb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91035cc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8835cc8, SUC_lexicon_tags_65 },
    { 0x34e35cc9, SUC_lexicon_tags_11 },
    { 0x63435cc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1035ccc, SUC_lexicon_tags_8 },
    { 0x60e35ccd, SUC_lexicon_tags_25 },
    { 0x2ee35ccd, SUC_lexicon_tags_15 },
    { 0x19a35ccf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2e35cd1, SUC_lexicon_tags_41 },
    { 0xa0035cd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4435cd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80035cd9, SUC_lexicon_tags_25 },
    { 0x62235cda, SUC_lexicon_tags_28 },
    { 0x15035cdb, SUC_lexicon_tags_39 },
    { 0xf7035cdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1435cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5835ce1, SUC_lexicon_tags_8 },
    { 0xf1835ce1, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5835ce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb035ce8, SUC_lexicon_tags_8 },
    { 0xc7035ce9, SUC_lexicon_tags_7 },
    { 0x2e35ce8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a35cee, SUC_lexicon_tags_25 },
    { 0x1035cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad835cf2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd2a35cf4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42035cfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2a35cfc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a35d01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e235d04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16635d08, SUC_lexicon_tags_68 },
    { 0xbde35d09, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f235d11, SUC_lexicon_tags_14 },
    { 0x71435d11, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa835d15, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35835d1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b635d1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9435d20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c35d25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a635d29, SUC_lexicon_tags_63 },
    { 0x42035d29, SUC_lexicon_tags_1 },
    { 0x68435d2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xae35d2e, SUC_lexicon_tags_15 },
    { 0x30c35d2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x10035d31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7235d35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43e35d37, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x12835d3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a35d3e, SUC_lexicon_tags_13 },
    { 0x3de35d3f, SUC_lexicon_tags_47 },
    { 0x65035d3f, SUC_lexicon_tags_1 },
    { 0x31c35d41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89635d46, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c35d49, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x50035d4b, SUC_lexicon_tags_37 },
    { 0x98c35d4b, SUC_lexicon_tags_2 },
    { 0x8c435d4c, SUC_lexicon_tags_12 },
    { 0xa3a35d4e, SUC_lexicon_tags_11 },
    { 0x3d435d4b, SUC_lexicon_tags_11 },
    { 0x44435d50, SUC_lexicon_tags_66 },
    { 0xe3a35d51, SUC_lexicon_tags_17 },
    { 0xee435d51, SUC_lexicon_tags_47 },
    { 0xcea35d4b, SUC_lexicon_tags_38 },
    { 0x7b435d54, SUC_lexicon_tags_1 },
    { 0x93835d4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75035d57, SUC_lexicon_tags_1 },
    { 0x9f835d58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f235d5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c35d62, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2635d6a, SUC_lexicon_tags_16 },
    { 0xad435d6b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1ac35d6d, SUC_lexicon_tags_2 },
    { 0x4e835d6d, SUC_lexicon_tags_29 },
    { 0xfe35d6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6ba35d71, SUC_lexicon_tags_63 },
    { 0x3a235d72, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc35d77, SUC_lexicon_tags_9 },
    { 0x6b635d78, SUC_lexicon_tags_1 },
    { 0x87235d79, SUC_lexicon_tags_25 },
    { 0xfa835d7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45635d7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1cc35d7e, SUC_lexicon_tags_1 },
    { 0x17a35d7f, SUC_lexicon_tags_1 },
    { 0x30435d80, SUC_lexicon_tags_1 },
    { 0x1a835d81, SUC_lexicon_tags_2 },
    { 0x74c35d7f, SUC_lexicon_tags_25 },
    { 0x29435d80, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x9f235d85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4635d87, SUC_lexicon_tags_27 },
    { 0x6f835d87, SUC_lexicon_tags_17 },
    { 0x58a35d89, SUC_lexicon_tags_29 },
    { 0xe35d8a, SUC_lexicon_tags_37 },
    { 0x5f035d89, SUC_lexicon_tags_8 },
    { 0xb635d8a, SUC_lexicon_tags_25 },
    { 0x66035d8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5235d8f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c35d92, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc35d95, SUC_lexicon_tags_44 },
    { 0xbba35d96, SUC_lexicon_tags_43 },
    { 0xf2035d97, SUC_lexicon_tags_31 },
    { 0xdf435d98, SUC_lexicon_tags_37 },
    { 0x94435d97, SUC_lexicon_tags_25 },
    { 0x72c35d96, SUC_lexicon_tags_230 },
    { 0x36435d9b, SUC_lexicon_tags_7 },
    { 0x83235d95, SUC_lexicon_tags_7 },
    { 0xa3635d9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e35da1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf835da4, SUC_lexicon_tags_61 },
    { 0x40835da4, SUC_lexicon_tags_69 },
    { 0xc2e35da6, SUC_lexicon_tags_17 },
    { 0xe2835da7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde235da9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28635db1, SUC_lexicon_tags_1 },
    { 0x67c35db2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec35dba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b835dbd, SUC_lexicon_tags_68 },
    { 0xe0c35dbd, SUC_lexicon_tags_31 },
    { 0xd1c35dbf, SUC_lexicon_tags_8 },
    { 0xc8035dbf, SUC_lexicon_tags_32 },
    { 0x4dc35dbf, SUC_lexicon_tags_25 },
    { 0xd9635dc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89635dc5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8235dc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e635dcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda235dd1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e635dd9, SUC_lexicon_tags_1 },
    { 0xf6635dd9, SUC_lexicon_tags_6 },
    { 0x79e35dd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7435de5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6435dea, SUC_lexicon_tags_47 },
    { 0xe8a35deb, SUC_lexicon_tags_12 },
    { 0xc2035deb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b235df5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe35df8, SUC_lexicon_tags_45 },
    { 0x1a035df9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e35dfc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3fe35dfe, SUC_lexicon_tags_17 },
    { 0x2c035dff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5e635e01, SUC_lexicon_tags_11 },
    { 0x57c35e01, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xca235e04, SUC_lexicon_tags_21 },
    { 0x25435e05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfb235e07, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc635e09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4435e0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54435e12, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbd235e14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e35e17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf6835e19, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70235e1e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4d435e20, SUC_lexicon_tags_2 },
    { 0x70235e20, SUC_lexicon_tags_12 },
    { 0x2ac35e22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52835e2b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd7435e2d, SUC_lexicon_tags_55 },
    { 0x2635e2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a35e32, SUC_lexicon_tags_25 },
    { 0xf3435e33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a35e36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a35e39, SUC_lexicon_tags_19 },
    { 0x88235e3a, SUC_lexicon_tags_27 },
    { 0xca435e3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71a35e3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6035e40, SUC_lexicon_tags_859 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95435e47, SUC_lexicon_tags_20 },
    { 0x13635e48, SUC_lexicon_tags_0 },
    { 0x1ee35e47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b635e4f, SUC_lexicon_tags_11 },
    { 0x6ea35e50, SUC_lexicon_tags_84 },
    { 0x6e835e51, SUC_lexicon_tags_33 },
    { 0xcec35e51, SUC_lexicon_tags_27 },
    { 0xd5835e51, SUC_lexicon_tags_17 },
    { 0xac035e54, SUC_lexicon_tags_17 },
    { 0x7b635e55, SUC_lexicon_tags_7 },
    { 0x7fc35e56, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51435e5a, SUC_lexicon_tags_27 },
    { 0x67035e5b, SUC_lexicon_tags_11 },
    { 0x9e835e5b, SUC_lexicon_tags_11 },
    { 0xf4c35e5d, SUC_lexicon_tags_11 },
    { 0x7d635e5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7235e66, SUC_lexicon_tags_33 },
    { 0xc6435e67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xde35e69, SUC_lexicon_tags_17 },
    { 0x8b235e6a, SUC_lexicon_tags_25 },
    { 0x48835e6a, SUC_lexicon_tags_43 },
    { 0xf6835e69, SUC_lexicon_tags_0 },
    { 0x58a35e6d, SUC_lexicon_tags_45 },
    { 0xf5a35e6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95235e72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35835e74, SUC_lexicon_tags_8 },
    { 0xfd635e75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a235e78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x84e35e7a, SUC_lexicon_tags_1 },
    { 0x57e35e7b, SUC_lexicon_tags_7 },
    { 0xaba35e7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73435e84, SUC_lexicon_tags_63 },
    { 0x8cc35e85, SUC_lexicon_tags_17 },
    { 0x26235e84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7235e89, SUC_lexicon_tags_0 },
    { 0xe635e89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf035e90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62035e96, SUC_lexicon_tags_12 },
    { 0x7a235e97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ee35e99, SUC_lexicon_tags_45 },
    { 0x7b635e99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcaa35e9c, SUC_lexicon_tags_38 },
    { 0x27235e9c, SUC_lexicon_tags_12 },
    { 0x50035e9c, SUC_lexicon_tags_26 },
    { 0x48635e9f, SUC_lexicon_tags_1 },
    { 0x8d435ea0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a235ea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15035ea9, SUC_lexicon_tags_25 },
    { 0xe3e35ea9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc35eae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a435eb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e35eb5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2435eba, SUC_lexicon_tags_14 },
    { 0x55235eba, SUC_lexicon_tags_11 },
    { 0xaec35ebb, SUC_lexicon_tags_15 },
    { 0xcea35ebc, SUC_lexicon_tags_2 },
    { 0x9e035ebe, SUC_lexicon_tags_17 },
    { 0xbe035ebc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca035ec2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a35ec5, SUC_lexicon_tags_30 },
    { 0xfa835ec6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef235ecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a35ed4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5235ed8, SUC_lexicon_tags_12 },
    { 0x6a835ed9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc35ee3, SUC_lexicon_tags_12 },
    { 0x9ea35ee4, SUC_lexicon_tags_11 },
    { 0x44c35ee3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a835eea, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x68635eec, SUC_lexicon_tags_49 },
    { 0xe7435eec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71435eef, SUC_lexicon_tags_11 },
    { 0x3f835ef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a635ef2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e35ef8, SUC_lexicon_tags_56 },
    { 0xe4435ef9, SUC_lexicon_tags_15 },
    { 0x69a35efa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x43835efd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x63035eff, SUC_lexicon_tags_9 },
    { 0x45e35f00, SUC_lexicon_tags_14 },
    { 0xd5635f00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f035f0c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86635f10, SUC_lexicon_tags_22 },
    { 0x1d435f11, SUC_lexicon_tags_7 },
    { 0x96e35f12, SUC_lexicon_tags_17 },
    { 0xd235f13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab035f15, SUC_lexicon_tags_25 },
    { 0xaa435f16, SUC_lexicon_tags_25 },
    { 0x1dc35f17, SUC_lexicon_tags_15 },
    { 0xf9635f16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58835f1c, SUC_lexicon_tags_61 },
    { 0xb3e35f1d, SUC_lexicon_tags_14 },
    { 0x54c35f1e, SUC_lexicon_tags_8 },
    { 0x58c35f1f, SUC_lexicon_tags_37 },
    { 0x30835f1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50435f22, SUC_lexicon_tags_17 },
    { 0x85a35f22, SUC_lexicon_tags_37 },
    { 0xb9e35f24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa35f27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x51235f2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae635f2f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b035f39, SUC_lexicon_tags_8 },
    { 0x7be35f3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x435f3d, SUC_lexicon_tags_15 },
    { 0xadc35f3e, SUC_lexicon_tags_1 },
    { 0x9fe35f3e, SUC_lexicon_tags_2 },
    { 0xe8635f40, SUC_lexicon_tags_11 },
    { 0x2ac35f3e, SUC_lexicon_tags_9 },
    { 0x60435f42, SUC_lexicon_tags_38 },
    { 0x91a35f43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43635f49, SUC_lexicon_tags_87 },
    { 0x4c035f4a, SUC_lexicon_tags_84 },
    { 0x20035f4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x61035f4e, SUC_lexicon_tags_23 },
    { 0x3e35f4e, SUC_lexicon_tags_15 },
    { 0xa7a35f50, SUC_lexicon_tags_2 },
    { 0xf6635f51, SUC_lexicon_tags_49 },
    { 0xe3e35f51, SUC_lexicon_tags_47 },
    { 0x9a635f52, SUC_lexicon_tags_2 },
    { 0x52c35f54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c35f57, SUC_lexicon_tags_9 },
    { 0x45c35f57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64c35f5a, SUC_lexicon_tags_25 },
    { 0x2d435f5a, SUC_lexicon_tags_45 },
    { 0xf0a35f5c, SUC_lexicon_tags_14 },
    { 0xfb235f5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x14835f60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2035f66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61635f6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb035f71, SUC_lexicon_tags_45 },
    { 0x63235f72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64635f74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4835f78, SUC_lexicon_tags_23 },
    { 0xec035f79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4635f7b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6635f7f, SUC_lexicon_tags_77 },
    { 0xfbc35f80, SUC_lexicon_tags_28 },
    { 0x38c35f81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc35f83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28035f85, SUC_lexicon_tags_37 },
    { 0x59a35f86, SUC_lexicon_tags_2 },
    { 0xd7a35f87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde235f89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe835f8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cc35f8d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e235f91, SUC_lexicon_tags_37 },
    { 0x9f635f92, SUC_lexicon_tags_21 },
    { 0xa6235f93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0435f99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6c35f9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34435fa7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8635fad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe035faf, SUC_lexicon_tags_12 },
    { 0xb035fb0, SUC_lexicon_tags_4 },
    { 0x86835faf, SUC_lexicon_tags_63 },
    { 0x3bc35fb0, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a435fb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1635fba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e35fbf, SUC_lexicon_tags_80 },
    { 0xb4235fbf, SUC_lexicon_tags_60 },
    { 0xe6a35fbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe1235fc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e35fc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda35fcd, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c635fd0, SUC_lexicon_tags_254 },
    { 0x9235fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d235fd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62435fd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6635fdb, SUC_lexicon_tags_8 },
    { 0xc9435fdc, SUC_lexicon_tags_0 },
    { 0x4d435fdb, SUC_lexicon_tags_37 },
    { 0x77035fdb, SUC_lexicon_tags_15 },
    { 0xa1635fdf, SUC_lexicon_tags_11 },
    { 0x1235fde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe235fe4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a35fe7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c035fef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c35ff2, SUC_lexicon_tags_25 },
    { 0xf6235ff3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa35ffc, SUC_lexicon_tags_11 },
    { 0xa3235ffd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1435fff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfc636001, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb036004, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96e36006, SUC_lexicon_tags_37 },
    { 0xd2836007, SUC_lexicon_tags_27 },
    { 0x3ca36008, SUC_lexicon_tags_11 },
    { 0x6a036006, SUC_lexicon_tags_55 },
    { 0x5c3600a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8de3600c, SUC_lexicon_tags_10 },
    { 0x7c23600c, SUC_lexicon_tags_12 },
    { 0xf423600e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2036010, SUC_lexicon_tags_25 },
    { 0x58836011, SUC_lexicon_tags_0 },
    { 0xf7a36012, SUC_lexicon_tags_2 },
    { 0xea636013, SUC_lexicon_tags_25 },
    { 0xa9436014, SUC_lexicon_tags_20 },
    { 0x15836015, SUC_lexicon_tags_63 },
    { 0xb3e36016, SUC_lexicon_tags_7 },
    { 0xee636016, SUC_lexicon_tags_53 },
    { 0xda436013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7dc3601a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5036024, SUC_lexicon_tags_29 },
    { 0xcfe36025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56636027, SUC_lexicon_tags_12 },
    { 0x35236028, SUC_lexicon_tags_11 },
    { 0x43436029, SUC_lexicon_tags_13 },
    { 0x9e03602a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9103602f, SUC_lexicon_tags_12 },
    { 0xb436030, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e36034, SUC_lexicon_tags_23 },
    { 0x7de36034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4c36037, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c3603b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc236040, SUC_lexicon_tags_2 },
    { 0xb2436041, SUC_lexicon_tags_25 },
    { 0x7c36041, SUC_lexicon_tags_15 },
    { 0x80c36043, SUC_lexicon_tags_37 },
    { 0xa7a36043, SUC_lexicon_tags_39 },
    { 0x28636045, SUC_lexicon_tags_106 },
    { 0x99e36044, SUC_lexicon_tags_45 },
    { 0x9de36041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47836049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc3604d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43636055, SUC_lexicon_tags_33 },
    { 0xe4636056, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c3605b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc3605e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c36062, SUC_lexicon_tags_28 },
    { 0x32236063, SUC_lexicon_tags_37 },
    { 0x82a36064, SUC_lexicon_tags_27 },
    { 0x1b236064, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c36069, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa423606b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de36071, SUC_lexicon_tags_25 },
    { 0x1ce36072, SUC_lexicon_tags_1 },
    { 0x4a236073, SUC_lexicon_tags_28 },
    { 0xc4e36074, SUC_lexicon_tags_15 },
    { 0xdbc36075, SUC_lexicon_tags_25 },
    { 0xd0636071, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e03607c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd236081, SUC_lexicon_tags_1 },
    { 0x6236082, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a236086, SUC_lexicon_tags_9 },
    { 0x32a36086, SUC_lexicon_tags_2 },
    { 0x2036088, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff23608c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3036091, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2836094, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8de36096, SUC_lexicon_tags_37 },
    { 0x67e36097, SUC_lexicon_tags_37 },
    { 0x9a36096, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1363609a, SUC_lexicon_tags_15 },
    { 0xf2e3609b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ac3609d, SUC_lexicon_tags_7 },
    { 0xf223609e, SUC_lexicon_tags_33 },
    { 0xb803609e, SUC_lexicon_tags_45 },
    { 0xca2360a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e360a3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee360a7, SUC_lexicon_tags_13 },
    { 0x996360a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf0a360aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28360ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c360b0, SUC_lexicon_tags_0 },
    { 0x108360b0, SUC_lexicon_tags_55 },
    { 0xbf0360b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e6360b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478360bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96360bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aa360c1, SUC_lexicon_tags_87 },
    { 0x8c4360c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10360c7, SUC_lexicon_tags_60 },
    { 0x530360c8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xccc360ca, SUC_lexicon_tags_27 },
    { 0xd68360cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x900360cd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5be360cf, SUC_lexicon_tags_19 },
    { 0xef2360d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e360d5, SUC_lexicon_tags_12 },
    { 0xa98360d6, SUC_lexicon_tags_1 },
    { 0x284360d7, SUC_lexicon_tags_4 },
    { 0xe9a360d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc2a360da, SUC_lexicon_tags_138 },
    { 0xd02360db, SUC_lexicon_tags_12 },
    { 0x2a4360dc, SUC_lexicon_tags_2 },
    { 0x8ae360dc, SUC_lexicon_tags_63 },
    { 0x430360de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda360e1, SUC_lexicon_tags_17 },
    { 0xf8e360e2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x25c360e4, SUC_lexicon_tags_11 },
    { 0x5b4360e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x79a360e7, SUC_lexicon_tags_6 },
    { 0xe38360e7, SUC_lexicon_tags_37 },
    { 0x326360e8, SUC_lexicon_tags_60 },
    { 0x994360ea, SUC_lexicon_tags_23 },
    { 0xfb4360eb, SUC_lexicon_tags_37 },
    { 0x7ec360eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4360ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb0360f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x716360f3, SUC_lexicon_tags_2 },
    { 0x196360f4, SUC_lexicon_tags_1 },
    { 0xf50360f5, SUC_lexicon_tags_37 },
    { 0x1e6360f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38360f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda6360fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7e360fd, SUC_lexicon_tags_11 },
    { 0xa36360fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b036106, SUC_lexicon_tags_82 },
    { 0x14036107, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0036109, SUC_lexicon_tags_0 },
    { 0xe636109, SUC_lexicon_tags_7 },
    { 0xc03610b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6e3610d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xca63610f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da36113, SUC_lexicon_tags_27 },
    { 0x6ca36114, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda636119, SUC_lexicon_tags_15 },
    { 0xc9c36119, SUC_lexicon_tags_16 },
    { 0xbe23611b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe203611d, SUC_lexicon_tags_17 },
    { 0x50c3611d, SUC_lexicon_tags_19 },
    { 0xba23611e, SUC_lexicon_tags_66 },
    { 0x3ea36120, SUC_lexicon_tags_1 },
    { 0xdf236121, SUC_lexicon_tags_11 },
    { 0xfe836122, SUC_lexicon_tags_7 },
    { 0xbe3611d, SUC_lexicon_tags_32 },
    { 0xa8636124, SUC_lexicon_tags_11 },
    { 0xf5036125, SUC_lexicon_tags_47 },
    { 0x18e36126, SUC_lexicon_tags_47 },
    { 0x4e636127, SUC_lexicon_tags_38 },
    { 0xee836124, SUC_lexicon_tags_7 },
    { 0xbe36129, SUC_lexicon_tags_4 },
    { 0x6aa3612a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a3612e, SUC_lexicon_tags_19 },
    { 0x6e63612f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd436131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc236136, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3263613d, SUC_lexicon_tags_2 },
    { 0x3783613e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26636143, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc1036145, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a83614c, SUC_lexicon_tags_0 },
    { 0xd4e3614c, SUC_lexicon_tags_0 },
    { 0x42e3614e, SUC_lexicon_tags_57 },
    { 0x32a3614e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3e36151, SUC_lexicon_tags_49 },
    { 0x71836151, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37036158, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3823615a, SUC_lexicon_tags_7 },
    { 0x84c3615b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95e3615d, SUC_lexicon_tags_31 },
    { 0x1a23615e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5636163, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29836169, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa3616d, SUC_lexicon_tags_49 },
    { 0xfa43616e, SUC_lexicon_tags_16 },
    { 0xf423616e, SUC_lexicon_tags_63 },
    { 0x9483616e, SUC_lexicon_tags_15 },
    { 0x3f036171, SUC_lexicon_tags_15 },
    { 0x89c3616f, SUC_lexicon_tags_17 },
    { 0x4a836173, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5836176, SUC_lexicon_tags_12 },
    { 0x98636177, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2803617b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc3617e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3036181, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a36188, SUC_lexicon_tags_11 },
    { 0x60236188, SUC_lexicon_tags_3 },
    { 0xc43618a, SUC_lexicon_tags_12 },
    { 0x8783618b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4943618d, SUC_lexicon_tags_14 },
    { 0xd203618e, SUC_lexicon_tags_17 },
    { 0x1d43618e, SUC_lexicon_tags_27 },
    { 0x8f236190, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3836199, SUC_lexicon_tags_7 },
    { 0xf1036199, SUC_lexicon_tags_9 },
    { 0xf3036199, SUC_lexicon_tags_32 },
    { 0xb883619c, SUC_lexicon_tags_63 },
    { 0xe203619b, SUC_lexicon_tags_1 },
    { 0x17e3619b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994361a3, SUC_lexicon_tags_14 },
    { 0x6b4361a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb58361a6, SUC_lexicon_tags_61 },
    { 0x38c361a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x574361a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6361ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ae361b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396361b6, SUC_lexicon_tags_158 },
    { 0x6de361b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x834361b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198361c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818361c6, SUC_lexicon_tags_11 },
    { 0x452361c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6361cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844361d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58361d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x750361d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4361db, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c361de, SUC_lexicon_tags_9 },
    { 0xc3c361de, SUC_lexicon_tags_14 },
    { 0xca8361e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2361e8, SUC_lexicon_tags_43 },
    { 0x980361e9, SUC_lexicon_tags_29 },
    { 0x372361ea, SUC_lexicon_tags_15 },
    { 0x706361e9, SUC_lexicon_tags_17 },
    { 0x770361ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526361f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0e361f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14361f9, SUC_lexicon_tags_12 },
    { 0xf32361f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f0361fc, SUC_lexicon_tags_7 },
    { 0xde8361fd, SUC_lexicon_tags_39 },
    { 0xc6c361fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86836201, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec36207, SUC_lexicon_tags_12 },
    { 0x53a36207, SUC_lexicon_tags_28 },
    { 0x94836207, SUC_lexicon_tags_65 },
    { 0x67636207, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xc823620c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92636210, SUC_lexicon_tags_60 },
    { 0x7c036210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78436213, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8036215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b636217, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7183621a, SUC_lexicon_tags_14 },
    { 0x51e3621a, SUC_lexicon_tags_46 },
    { 0x8fa3621c, SUC_lexicon_tags_1 },
    { 0x1a63621d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46836220, SUC_lexicon_tags_12 },
    { 0x3de36221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a36228, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e236230, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9836232, SUC_lexicon_tags_17 },
    { 0xcde36233, SUC_lexicon_tags_14 },
    { 0x43036234, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25836239, SUC_lexicon_tags_63 },
    { 0x3763623a, SUC_lexicon_tags_59 },
    { 0x9e43623a, SUC_lexicon_tags_12 },
    { 0xe8c3623c, SUC_lexicon_tags_84 },
    { 0x3c63623d, SUC_lexicon_tags_34 },
    { 0x67c3623e, SUC_lexicon_tags_8 },
    { 0x823623c, SUC_lexicon_tags_10 },
    { 0xfc036240, SUC_lexicon_tags_35 },
    { 0xe4c3623e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa36244, SUC_lexicon_tags_12 },
    { 0xb7436245, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b43624a, SUC_lexicon_tags_7 },
    { 0xe963624a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8703624e, SUC_lexicon_tags_23 },
    { 0xbac3624e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37a36251, SUC_lexicon_tags_11 },
    { 0x32436251, SUC_lexicon_tags_7 },
    { 0x59836252, SUC_lexicon_tags_19 },
    { 0xcc436252, SUC_lexicon_tags_23 },
    { 0xbb636255, SUC_lexicon_tags_11 },
    { 0xd6c36256, SUC_lexicon_tags_26 },
    { 0x34436257, SUC_lexicon_tags_11 },
    { 0xce836255, SUC_lexicon_tags_60 },
    { 0x91836257, SUC_lexicon_tags_17 },
    { 0x18836252, SUC_lexicon_tags_12 },
    { 0x5d23625b, SUC_lexicon_tags_7 },
    { 0xd923625c, SUC_lexicon_tags_25 },
    { 0xeb236259, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3343625f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2e636261, SUC_lexicon_tags_25 },
    { 0x70036262, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb5436264, SUC_lexicon_tags_65 },
    { 0x12e36265, SUC_lexicon_tags_0 },
    { 0x5c236265, SUC_lexicon_tags_40 },
    { 0x1d036265, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b43626b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe203626d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c36271, SUC_lexicon_tags_30 },
    { 0x36836272, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1a36274, SUC_lexicon_tags_63 },
    { 0xad636275, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9b836277, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee3627b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3923627d, SUC_lexicon_tags_65 },
    { 0x7da3627e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x73c36280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae836285, SUC_lexicon_tags_12 },
    { 0xb4236286, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec3628a, SUC_lexicon_tags_3 },
    { 0x7183628b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd3e3628d, SUC_lexicon_tags_23 },
    { 0xf223628e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0236290, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36436293, SUC_lexicon_tags_39 },
    { 0xe7036294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a36298, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba3629c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40362a5, SUC_lexicon_tags_32 },
    { 0x384362a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54362a9, SUC_lexicon_tags_8 },
    { 0x21c362a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de362ae, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x352362b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818362b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90362bc, SUC_lexicon_tags_23 },
    { 0xf48362bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87a362bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8362c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c362cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8362ce, SUC_lexicon_tags_47 },
    { 0xbde362cf, SUC_lexicon_tags_11 },
    { 0x87a362d0, SUC_lexicon_tags_11 },
    { 0x216362d1, SUC_lexicon_tags_38 },
    { 0x826362d1, SUC_lexicon_tags_2 },
    { 0x51a362d3, SUC_lexicon_tags_1 },
    { 0xaba362d4, SUC_lexicon_tags_31 },
    { 0x366362d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac362d9, SUC_lexicon_tags_11 },
    { 0x83a362d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234362de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6362e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658362ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcba362f0, SUC_lexicon_tags_21 },
    { 0xeee362f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204362f7, SUC_lexicon_tags_33 },
    { 0x212362f7, SUC_lexicon_tags_17 },
    { 0x862362f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e362fc, SUC_lexicon_tags_19 },
    { 0x1c0362fd, SUC_lexicon_tags_12 },
    { 0xd16362fd, SUC_lexicon_tags_45 },
    { 0x9e8362fe, SUC_lexicon_tags_21 },
    { 0xe6436300, SUC_lexicon_tags_5 },
    { 0xc3636301, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1836304, SUC_lexicon_tags_18 },
    { 0xe4e36305, SUC_lexicon_tags_21 },
    { 0xb4036304, SUC_lexicon_tags_17 },
    { 0xba236306, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43630d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a36313, SUC_lexicon_tags_27 },
    { 0x50e36314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe36317, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b83631c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c36323, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b836325, SUC_lexicon_tags_17 },
    { 0x6de36325, SUC_lexicon_tags_61 },
    { 0x9d836327, SUC_lexicon_tags_1 },
    { 0xf5636325, SUC_lexicon_tags_11 },
    { 0x43836325, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e3632e, SUC_lexicon_tags_15 },
    { 0x7023632f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27c36331, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a36334, SUC_lexicon_tags_31 },
    { 0x20c36335, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03633f, SUC_lexicon_tags_12 },
    { 0x34636340, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d436347, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e3634f, SUC_lexicon_tags_1 },
    { 0xc1036350, SUC_lexicon_tags_12 },
    { 0x8b236350, SUC_lexicon_tags_7 },
    { 0x10e3634f, SUC_lexicon_tags_37 },
    { 0x7fa36353, SUC_lexicon_tags_37 },
    { 0xa0036354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37a36356, SUC_lexicon_tags_11 },
    { 0x1c036357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe103635a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b23635c, SUC_lexicon_tags_25 },
    { 0x1363635d, SUC_lexicon_tags_2 },
    { 0xfb83635d, SUC_lexicon_tags_12 },
    { 0x8603635e, SUC_lexicon_tags_37 },
    { 0x3e3635d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde436363, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a436367, SUC_lexicon_tags_2 },
    { 0xfc836368, SUC_lexicon_tags_14 },
    { 0x21e36369, SUC_lexicon_tags_39 },
    { 0x33e3636a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c3636e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec36371, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f236373, SUC_lexicon_tags_47 },
    { 0x78236374, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a3637c, SUC_lexicon_tags_7 },
    { 0xc9c3637d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe83637f, SUC_lexicon_tags_11 },
    { 0xfd436380, SUC_lexicon_tags_37 },
    { 0xaf636381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d236386, SUC_lexicon_tags_29 },
    { 0xe2036386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1436389, SUC_lexicon_tags_47 },
    { 0x33a3638a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54236390, SUC_lexicon_tags_17 },
    { 0xfba36391, SUC_lexicon_tags_1 },
    { 0xeea36392, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82036394, SUC_lexicon_tags_28 },
    { 0x45636395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0436399, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf023639b, SUC_lexicon_tags_161 },
    { 0x23e3639b, SUC_lexicon_tags_15 },
    { 0x1443639c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8363a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c363aa, SUC_lexicon_tags_9 },
    { 0xde0363ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a363b0, SUC_lexicon_tags_15 },
    { 0x266363b1, SUC_lexicon_tags_12 },
    { 0xaec363b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a363b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152363bd, SUC_lexicon_tags_95 },
    { 0x84363bd, SUC_lexicon_tags_37 },
    { 0xcd6363bf, SUC_lexicon_tags_12 },
    { 0xf98363bf, SUC_lexicon_tags_9 },
    { 0x4ee363c1, SUC_lexicon_tags_11 },
    { 0xbda363c1, SUC_lexicon_tags_27 },
    { 0x410363c3, SUC_lexicon_tags_23 },
    { 0x56e363c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866363c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4a363ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c363d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2363db, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x730363de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8363e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72363e4, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00363e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec2363e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c363ec, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce363ef, SUC_lexicon_tags_47 },
    { 0x216363f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba363f4, SUC_lexicon_tags_23 },
    { 0x346363f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a363fb, SUC_lexicon_tags_1 },
    { 0x884363fc, SUC_lexicon_tags_7 },
    { 0xea0363fc, SUC_lexicon_tags_1 },
    { 0x156363fe, SUC_lexicon_tags_38 },
    { 0x30363fe, SUC_lexicon_tags_12 },
    { 0x2f836400, SUC_lexicon_tags_17 },
    { 0x2e636401, SUC_lexicon_tags_17 },
    { 0xe7436402, SUC_lexicon_tags_11 },
    { 0xd1836400, SUC_lexicon_tags_25 },
    { 0xb80363fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xefc36406, SUC_lexicon_tags_47 },
    { 0x4ea36407, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce3640a, SUC_lexicon_tags_37 },
    { 0x17e3640a, SUC_lexicon_tags_25 },
    { 0x5923640c, SUC_lexicon_tags_0 },
    { 0xd9c3640a, SUC_lexicon_tags_7 },
    { 0x7823640e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2be36410, SUC_lexicon_tags_49 },
    { 0xeda36411, SUC_lexicon_tags_15 },
    { 0xd9e36412, SUC_lexicon_tags_10 },
    { 0x6e436413, SUC_lexicon_tags_45 },
    { 0x3e636414, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc36419, SUC_lexicon_tags_230 },
    { 0x8803641a, SUC_lexicon_tags_35 },
    { 0x5a836419, SUC_lexicon_tags_7 },
    { 0xc2a3641b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95436424, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2636426, SUC_lexicon_tags_77 },
    { 0x80a36427, SUC_lexicon_tags_7 },
    { 0xf4436426, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6e3642a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9503642d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee436430, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce36435, SUC_lexicon_tags_1 },
    { 0x7f036436, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfc36438, SUC_lexicon_tags_15 },
    { 0x95836438, SUC_lexicon_tags_37 },
    { 0x8d63643a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbba3643c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2963643e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e36442, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1436445, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e036449, SUC_lexicon_tags_2 },
    { 0x4303644a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7403644e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e236450, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x48236453, SUC_lexicon_tags_2 },
    { 0x92036454, SUC_lexicon_tags_3 },
    { 0x45236454, SUC_lexicon_tags_19 },
    { 0xac836456, SUC_lexicon_tags_11 },
    { 0x3b036453, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e36460, SUC_lexicon_tags_15 },
    { 0xb7e36461, SUC_lexicon_tags_33 },
    { 0xbc036462, SUC_lexicon_tags_17 },
    { 0x84a36463, SUC_lexicon_tags_7 },
    { 0xae836461, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4836466, SUC_lexicon_tags_17 },
    { 0xb8e36467, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4323646a, SUC_lexicon_tags_7 },
    { 0x3e23646b, SUC_lexicon_tags_1 },
    { 0xe7e3646c, SUC_lexicon_tags_45 },
    { 0x52e3646d, SUC_lexicon_tags_2 },
    { 0x31c3646e, SUC_lexicon_tags_33 },
    { 0x9c03646a, SUC_lexicon_tags_7 },
    { 0x4e43646a, SUC_lexicon_tags_23 },
    { 0xc1e36471, SUC_lexicon_tags_56 },
    { 0xb2836472, SUC_lexicon_tags_30 },
    { 0x60e36472, SUC_lexicon_tags_2 },
    { 0xdea3646e, SUC_lexicon_tags_49 },
    { 0x69636475, SUC_lexicon_tags_11 },
    { 0xf0c36471, SUC_lexicon_tags_55 },
    { 0xf9c3646f, SUC_lexicon_tags_66 },
    { 0x8f436478, SUC_lexicon_tags_15 },
    { 0x9d036479, SUC_lexicon_tags_14 },
    { 0x3fc36478, SUC_lexicon_tags_14 },
    { 0x4663647b, SUC_lexicon_tags_45 },
    { 0x6fa3647b, SUC_lexicon_tags_63 },
    { 0x8e03647d, SUC_lexicon_tags_33 },
    { 0xae03647e, SUC_lexicon_tags_12 },
    { 0xeac3647f, SUC_lexicon_tags_12 },
    { 0xae43647f, SUC_lexicon_tags_17 },
    { 0x78c36479, SUC_lexicon_tags_10 },
    { 0x8d036482, SUC_lexicon_tags_37 },
    { 0x9cc36483, SUC_lexicon_tags_37 },
    { 0x94236484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e36489, SUC_lexicon_tags_69 },
    { 0xf036489, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b43648f, SUC_lexicon_tags_11 },
    { 0x3ac36490, SUC_lexicon_tags_81 },
    { 0x51436491, SUC_lexicon_tags_11 },
    { 0x52a36492, SUC_lexicon_tags_8 },
    { 0x1c36493, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3236495, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672364a1, SUC_lexicon_tags_26 },
    { 0xfaa364a1, SUC_lexicon_tags_25 },
    { 0x6fa364a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcac364a5, SUC_lexicon_tags_25 },
    { 0xb9e364a5, SUC_lexicon_tags_2 },
    { 0x868364a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394364ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8364b1, SUC_lexicon_tags_19 },
    { 0x71a364b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246364c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17e364c5, SUC_lexicon_tags_30 },
    { 0xa38364c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2364ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e364d0, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516364d4, SUC_lexicon_tags_17 },
    { 0x8c0364d5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0364d8, SUC_lexicon_tags_37 },
    { 0xb64364d8, SUC_lexicon_tags_8 },
    { 0xc6e364da, SUC_lexicon_tags_42 },
    { 0x170364db, SUC_lexicon_tags_23 },
    { 0xdbe364da, SUC_lexicon_tags_26 },
    { 0x91a364dd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec364e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8364e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x800364eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be364ee, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x844364f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa364f3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528364fe, SUC_lexicon_tags_1 },
    { 0xcfa364ff, SUC_lexicon_tags_2 },
    { 0x4e4364ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be36504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63036507, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4723650e, SUC_lexicon_tags_31 },
    { 0xdea3650f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb436513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c36516, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7703651b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7de3651d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a36523, SUC_lexicon_tags_33 },
    { 0xe1436524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x23036527, SUC_lexicon_tags_37 },
    { 0xb0c36527, SUC_lexicon_tags_25 },
    { 0xc7e36528, SUC_lexicon_tags_25 },
    { 0xcb63652a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc3652f, SUC_lexicon_tags_30 },
    { 0xc323652f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf636537, SUC_lexicon_tags_29 },
    { 0x14036537, SUC_lexicon_tags_31 },
    { 0xfee36539, SUC_lexicon_tags_12 },
    { 0xe4a3653a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce3653e, SUC_lexicon_tags_45 },
    { 0x4063653f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85436541, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66036543, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcee36545, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85836549, SUC_lexicon_tags_61 },
    { 0xee43654a, SUC_lexicon_tags_7 },
    { 0x1703654b, SUC_lexicon_tags_65 },
    { 0xde43654c, SUC_lexicon_tags_23 },
    { 0x9b83654b, SUC_lexicon_tags_2 },
    { 0x52a3654e, SUC_lexicon_tags_11 },
    { 0xe5c3654e, SUC_lexicon_tags_27 },
    { 0xbd03654e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3623655a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9283655c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30436563, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b036568, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2663656c, SUC_lexicon_tags_17 },
    { 0x6ba3656d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5203656f, SUC_lexicon_tags_15 },
    { 0x5ac36570, SUC_lexicon_tags_17 },
    { 0x5c36571, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x92a36573, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be3657b, SUC_lexicon_tags_47 },
    { 0x9ba3657c, SUC_lexicon_tags_77 },
    { 0x73a3657d, SUC_lexicon_tags_9 },
    { 0x9803657d, SUC_lexicon_tags_11 },
    { 0xfe3657e, SUC_lexicon_tags_27 },
    { 0x37c3657f, SUC_lexicon_tags_48 },
    { 0x72e36581, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66836586, SUC_lexicon_tags_37 },
    { 0x3ce36587, SUC_lexicon_tags_11 },
    { 0x3bc36587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a3658b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e3658e, SUC_lexicon_tags_30 },
    { 0x4603658f, SUC_lexicon_tags_43 },
    { 0x73c3658e, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c36593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51636597, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc3659e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae365a1, SUC_lexicon_tags_15 },
    { 0x9da365a2, SUC_lexicon_tags_43 },
    { 0x654365a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0365a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c365ad, SUC_lexicon_tags_33 },
    { 0xe2c365ae, SUC_lexicon_tags_9 },
    { 0xd90365af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e365b8, SUC_lexicon_tags_7 },
    { 0x76c365b9, SUC_lexicon_tags_84 },
    { 0xe78365b9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3f0365bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe365c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e365c8, SUC_lexicon_tags_47 },
    { 0x630365c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78a365cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c6365cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x63a365cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x462365d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a365d5, SUC_lexicon_tags_43 },
    { 0x9c4365d6, SUC_lexicon_tags_68 },
    { 0x22365d6, SUC_lexicon_tags_32 },
    { 0x3ca365d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa96365da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2365dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8365e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76365e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe12365eb, SUC_lexicon_tags_25 },
    { 0xbb2365eb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1be365ee, SUC_lexicon_tags_17 },
    { 0xc2a365ee, SUC_lexicon_tags_0 },
    { 0x72365f0, SUC_lexicon_tags_15 },
    { 0x7c6365f1, SUC_lexicon_tags_33 },
    { 0xb5c365f2, SUC_lexicon_tags_27 },
    { 0xef4365ee, SUC_lexicon_tags_30 },
    { 0x266365f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b4365f6, SUC_lexicon_tags_17 },
    { 0x350365f7, SUC_lexicon_tags_19 },
    { 0x2d0365f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34365fb, SUC_lexicon_tags_17 },
    { 0x782365fc, SUC_lexicon_tags_79 },
    { 0x56365fd, SUC_lexicon_tags_15 },
    { 0x49a365fd, SUC_lexicon_tags_14 },
    { 0x826365ff, SUC_lexicon_tags_37 },
    { 0x832365ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa36604, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac36608, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14c3660a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec3660e, SUC_lexicon_tags_37 },
    { 0xb03660e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb836612, SUC_lexicon_tags_77 },
    { 0x30636613, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea3661a, SUC_lexicon_tags_23 },
    { 0x1483661b, SUC_lexicon_tags_11 },
    { 0x1383661c, SUC_lexicon_tags_23 },
    { 0x523661d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c36623, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf036625, SUC_lexicon_tags_37 },
    { 0x5be36626, SUC_lexicon_tags_26 },
    { 0x10036625, SUC_lexicon_tags_53 },
    { 0x51436626, SUC_lexicon_tags_11 },
    { 0x16636629, SUC_lexicon_tags_43 },
    { 0xff436628, SUC_lexicon_tags_13 },
    { 0x7d83662b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0236630, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c36634, SUC_lexicon_tags_43 },
    { 0xa5a36635, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3663b, SUC_lexicon_tags_17 },
    { 0xd263663c, SUC_lexicon_tags_15 },
    { 0x34e3663d, SUC_lexicon_tags_49 },
    { 0xed03663e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87236642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae236644, SUC_lexicon_tags_27 },
    { 0xe5a36645, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a43664c, SUC_lexicon_tags_15 },
    { 0x42c3664c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10636654, SUC_lexicon_tags_11 },
    { 0xd4236654, SUC_lexicon_tags_1330 },
    { 0xeae36656, SUC_lexicon_tags_26 },
    { 0x5e236657, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe383665d, SUC_lexicon_tags_15 },
    { 0x67c3665d, SUC_lexicon_tags_12 },
    { 0x763665f, SUC_lexicon_tags_77 },
    { 0xaf63665e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0636665, SUC_lexicon_tags_37 },
    { 0x36036666, SUC_lexicon_tags_4 },
    { 0xea036667, SUC_lexicon_tags_15 },
    { 0x50e36668, SUC_lexicon_tags_45 },
    { 0x2ee36669, SUC_lexicon_tags_34 },
    { 0xa7c3666a, SUC_lexicon_tags_10 },
    { 0x9d036667, SUC_lexicon_tags_28 },
    { 0x9423666c, SUC_lexicon_tags_34 },
    { 0x72c3666d, SUC_lexicon_tags_6 },
    { 0xed03666a, SUC_lexicon_tags_47 },
    { 0x1d236666, SUC_lexicon_tags_114 },
    { 0x27436670, SUC_lexicon_tags_7 },
    { 0x8ea36671, SUC_lexicon_tags_47 },
    { 0x5fe36670, SUC_lexicon_tags_37 },
    { 0x40236673, SUC_lexicon_tags_1 },
    { 0x51e36667, SUC_lexicon_tags_39 },
    { 0x9d036667, SUC_lexicon_tags_2 },
    { 0x64e36669, SUC_lexicon_tags_17 },
    { 0xcf236677, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c83667a, SUC_lexicon_tags_17 },
    { 0x1243667b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb243667e, SUC_lexicon_tags_7 },
    { 0xd843667e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae436686, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfca36688, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x683668b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc383668e, SUC_lexicon_tags_68 },
    { 0x9d23668f, SUC_lexicon_tags_79 },
    { 0xaf43668f, SUC_lexicon_tags_42 },
    { 0xd5e36691, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd236696, SUC_lexicon_tags_43 },
    { 0x96036697, SUC_lexicon_tags_0 },
    { 0x7f436697, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcec3669a, SUC_lexicon_tags_82 },
    { 0xde63669a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf643669d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0366a0, SUC_lexicon_tags_2 },
    { 0x400366a0, SUC_lexicon_tags_2 },
    { 0x9d0366a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb4366a4, SUC_lexicon_tags_37 },
    { 0xad0366a5, SUC_lexicon_tags_12 },
    { 0x22c366a5, SUC_lexicon_tags_7 },
    { 0x9b4366a7, SUC_lexicon_tags_1 },
    { 0x964366a8, SUC_lexicon_tags_15 },
    { 0x73a366a9, SUC_lexicon_tags_32 },
    { 0x5a0366a9, SUC_lexicon_tags_15 },
    { 0x9de366ab, SUC_lexicon_tags_35 },
    { 0x2f2366ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2366b1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08366b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52366bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee366c1, SUC_lexicon_tags_17 },
    { 0xba4366c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16366c8, SUC_lexicon_tags_1 },
    { 0xad6366c9, SUC_lexicon_tags_38 },
    { 0xa3e366c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e366cd, SUC_lexicon_tags_11 },
    { 0xc76366ce, SUC_lexicon_tags_70 },
    { 0xfde366cf, SUC_lexicon_tags_38 },
    { 0x464366cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482366d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc4366d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0366df, SUC_lexicon_tags_21 },
    { 0x478366e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e366e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40366e8, SUC_lexicon_tags_48 },
    { 0xe66366e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd3e366eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0366ef, SUC_lexicon_tags_17 },
    { 0xf30366ef, SUC_lexicon_tags_0 },
    { 0xea8366f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2366f6, SUC_lexicon_tags_15 },
    { 0x69a366f7, SUC_lexicon_tags_2 },
    { 0xe48366f8, SUC_lexicon_tags_60 },
    { 0x626366f7, SUC_lexicon_tags_17 },
    { 0x7c6366f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8a366fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x546366fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55836700, SUC_lexicon_tags_6 },
    { 0x80c36701, SUC_lexicon_tags_7 },
    { 0xa6a36701, SUC_lexicon_tags_23 },
    { 0x43236703, SUC_lexicon_tags_11 },
    { 0x19e36703, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a36707, SUC_lexicon_tags_7 },
    { 0x2a036707, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc3670e, SUC_lexicon_tags_16 },
    { 0x8523670f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b636711, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a236717, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3683671b, SUC_lexicon_tags_2 },
    { 0xcb43671c, SUC_lexicon_tags_14 },
    { 0x84e3671d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c836721, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b636723, SUC_lexicon_tags_38 },
    { 0xf8e36724, SUC_lexicon_tags_12 },
    { 0x93636724, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e636729, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf423672d, SUC_lexicon_tags_39 },
    { 0x3f23672e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee436730, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a836736, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d036738, SUC_lexicon_tags_12 },
    { 0x92236739, SUC_lexicon_tags_23 },
    { 0x49236739, SUC_lexicon_tags_30 },
    { 0x40a3673b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c3673e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x19c36740, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e36746, SUC_lexicon_tags_6 },
    { 0x86836746, SUC_lexicon_tags_15 },
    { 0x49036747, SUC_lexicon_tags_114 },
    { 0xb3036749, SUC_lexicon_tags_4 },
    { 0x2ae36749, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e43674c, SUC_lexicon_tags_42 },
    { 0xbd03674d, SUC_lexicon_tags_2 },
    { 0xae63674c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e36756, SUC_lexicon_tags_66 },
    { 0x77a36757, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e3675c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce36762, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8fc36764, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc36767, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a3676d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20036770, SUC_lexicon_tags_37 },
    { 0x61e36771, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d236773, SUC_lexicon_tags_33 },
    { 0x1436774, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c3677d, SUC_lexicon_tags_83 },
    { 0xe583677e, SUC_lexicon_tags_55 },
    { 0xcfe3677d, SUC_lexicon_tags_25 },
    { 0xd3a36780, SUC_lexicon_tags_30 },
    { 0xfaa36781, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad836787, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43678b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29636791, SUC_lexicon_tags_12 },
    { 0xf1e36792, SUC_lexicon_tags_8 },
    { 0x51836793, SUC_lexicon_tags_26 },
    { 0xe7e36791, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e3679d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfe3679f, SUC_lexicon_tags_1 },
    { 0x54a3679f, SUC_lexicon_tags_14 },
    { 0x490367a1, SUC_lexicon_tags_12 },
    { 0xb30367a0, SUC_lexicon_tags_47 },
    { 0x2763679f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcc4367a5, SUC_lexicon_tags_19 },
    { 0x6bc367a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x708367a9, SUC_lexicon_tags_26 },
    { 0x7c2367a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e367ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2e367b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc367b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e367bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x962367bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50367c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0367cb, SUC_lexicon_tags_82 },
    { 0xb84367cc, SUC_lexicon_tags_33 },
    { 0x566367cd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764367d2, SUC_lexicon_tags_28 },
    { 0xb20367d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc367d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0a367d9, SUC_lexicon_tags_7 },
    { 0x6ac367da, SUC_lexicon_tags_11 },
    { 0xb24367db, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44367e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b6367e8, SUC_lexicon_tags_11 },
    { 0x37e367e9, SUC_lexicon_tags_2 },
    { 0xb08367e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa367ed, SUC_lexicon_tags_2 },
    { 0x2e367ed, SUC_lexicon_tags_11 },
    { 0x97a367ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20367f2, SUC_lexicon_tags_15 },
    { 0xf9e367f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa367f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x798367f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6367fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7436803, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44236807, SUC_lexicon_tags_27 },
    { 0xc2a36808, SUC_lexicon_tags_38 },
    { 0xf7036809, SUC_lexicon_tags_9 },
    { 0x6243680a, SUC_lexicon_tags_47 },
    { 0x7f836807, SUC_lexicon_tags_25 },
    { 0x1e03680c, SUC_lexicon_tags_66 },
    { 0xdd43680c, SUC_lexicon_tags_12 },
    { 0x9223680c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48636814, SUC_lexicon_tags_18 },
    { 0x7e836815, SUC_lexicon_tags_9 },
    { 0x97436816, SUC_lexicon_tags_11 },
    { 0xbee36817, SUC_lexicon_tags_7 },
    { 0x7fc36815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e3681b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70436820, SUC_lexicon_tags_11 },
    { 0x41c36820, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf836824, SUC_lexicon_tags_15 },
    { 0xb636825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a36828, SUC_lexicon_tags_6 },
    { 0xda036829, SUC_lexicon_tags_87 },
    { 0x9883682a, SUC_lexicon_tags_37 },
    { 0xeea3682b, SUC_lexicon_tags_11 },
    { 0xae23682c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58e3682e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c36833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xacc36835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2463683d, SUC_lexicon_tags_37 },
    { 0xa523683d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab436842, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3684d, SUC_lexicon_tags_12 },
    { 0x44e3684e, SUC_lexicon_tags_1 },
    { 0xee63684f, SUC_lexicon_tags_15 },
    { 0xf6036850, SUC_lexicon_tags_17 },
    { 0x1d63684e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece36859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdee3685b, SUC_lexicon_tags_12 },
    { 0x6a83685b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e3685f, SUC_lexicon_tags_57 },
    { 0x6cc36860, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c836863, SUC_lexicon_tags_208 },
    { 0x89236864, SUC_lexicon_tags_30 },
    { 0xb1236863, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4636868, SUC_lexicon_tags_9 },
    { 0x8d636869, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e3686c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5836871, SUC_lexicon_tags_312 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6436876, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81236878, SUC_lexicon_tags_0 },
    { 0x6636878, SUC_lexicon_tags_21 },
    { 0xd103687a, SUC_lexicon_tags_35 },
    { 0xd2e3687a, SUC_lexicon_tags_37 },
    { 0x7543687b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46836888, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e23688a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc63688c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x14c3688e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c636890, SUC_lexicon_tags_6 },
    { 0x75c36891, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce436893, SUC_lexicon_tags_31 },
    { 0xfc36893, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfda36896, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7236899, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9903689c, SUC_lexicon_tags_47 },
    { 0x1e03689c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca3689f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea368a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa0368a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88368a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91e368aa, SUC_lexicon_tags_11 },
    { 0xab0368ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28a368ad, SUC_lexicon_tags_11 },
    { 0x34368ad, SUC_lexicon_tags_9 },
    { 0xfdc368ad, SUC_lexicon_tags_47 },
    { 0x696368b0, SUC_lexicon_tags_65 },
    { 0x29a368b1, SUC_lexicon_tags_32 },
    { 0x8be368b0, SUC_lexicon_tags_25 },
    { 0xe88368b3, SUC_lexicon_tags_8 },
    { 0x8fa368b3, SUC_lexicon_tags_11 },
    { 0x36a368b4, SUC_lexicon_tags_1 },
    { 0x316368b6, SUC_lexicon_tags_45 },
    { 0xe94368b7, SUC_lexicon_tags_33 },
    { 0xbb8368b8, SUC_lexicon_tags_37 },
    { 0x8d6368b6, SUC_lexicon_tags_45 },
    { 0x7ba368ae, SUC_lexicon_tags_1 },
    { 0xed0368bb, SUC_lexicon_tags_63 },
    { 0x132368bc, SUC_lexicon_tags_15 },
    { 0x392368bc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842368c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0368c9, SUC_lexicon_tags_6 },
    { 0x418368ca, SUC_lexicon_tags_25 },
    { 0xc00368cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44368cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x126368d2, SUC_lexicon_tags_9 },
    { 0x902368d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x866368d5, SUC_lexicon_tags_2 },
    { 0xb3e368d5, SUC_lexicon_tags_32 },
    { 0x222368d7, SUC_lexicon_tags_3 },
    { 0x1b2368d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36368db, SUC_lexicon_tags_37 },
    { 0x41e368dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xabc368de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x356368e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e368e3, SUC_lexicon_tags_1 },
    { 0x708368e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a368e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x652368ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0368ef, SUC_lexicon_tags_37 },
    { 0xa0a368f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6368f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a368fa, SUC_lexicon_tags_15 },
    { 0x7a0368fa, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf636903, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc36908, SUC_lexicon_tags_17 },
    { 0xc9636909, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2036910, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836919, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c3691e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1436920, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde36923, SUC_lexicon_tags_7 },
    { 0x92c36923, SUC_lexicon_tags_25 },
    { 0x87636925, SUC_lexicon_tags_17 },
    { 0x29c36925, SUC_lexicon_tags_146 },
    { 0x31836927, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da3692b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7636933, SUC_lexicon_tags_7 },
    { 0x14036934, SUC_lexicon_tags_49 },
    { 0x13e36934, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf743693c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xea23693f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c36944, SUC_lexicon_tags_11 },
    { 0x87436945, SUC_lexicon_tags_7 },
    { 0xb1a36946, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a36949, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd963694f, SUC_lexicon_tags_38 },
    { 0x10436950, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92436954, SUC_lexicon_tags_17 },
    { 0xfa36954, SUC_lexicon_tags_2 },
    { 0xc2c36956, SUC_lexicon_tags_9 },
    { 0x6f236957, SUC_lexicon_tags_47 },
    { 0x5a636956, SUC_lexicon_tags_56 },
    { 0x29036955, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2903695b, SUC_lexicon_tags_32 },
    { 0x4083695b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4236961, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94836969, SUC_lexicon_tags_17 },
    { 0xf7a3696a, SUC_lexicon_tags_0 },
    { 0x3ae3696b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5fc3696d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe36972, SUC_lexicon_tags_12 },
    { 0x57436972, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x31036976, SUC_lexicon_tags_12 },
    { 0x99a36976, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x36636979, SUC_lexicon_tags_11 },
    { 0xc363697a, SUC_lexicon_tags_15 },
    { 0xe0036979, SUC_lexicon_tags_12 },
    { 0x46836979, SUC_lexicon_tags_16 },
    { 0x94c3697d, SUC_lexicon_tags_15 },
    { 0x1e03697a, SUC_lexicon_tags_17 },
    { 0x9fc3697f, SUC_lexicon_tags_7 },
    { 0xf4036980, SUC_lexicon_tags_25 },
    { 0xd0e3697c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42836983, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88636986, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b036988, SUC_lexicon_tags_37 },
    { 0x88236989, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d63698c, SUC_lexicon_tags_8 },
    { 0x1643698c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd436990, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4636994, SUC_lexicon_tags_12 },
    { 0xd8636994, SUC_lexicon_tags_66 },
    { 0x12c36995, SUC_lexicon_tags_11 },
    { 0xe4236997, SUC_lexicon_tags_63 },
    { 0xbf636995, SUC_lexicon_tags_11 },
    { 0xb2236995, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb43699b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6443699e, SUC_lexicon_tags_1 },
    { 0xbea3699f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36a369a1, SUC_lexicon_tags_27 },
    { 0x608369a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76369b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x302369ba, SUC_lexicon_tags_37 },
    { 0x12e369ba, SUC_lexicon_tags_12 },
    { 0x624369bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x802369bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca369c2, SUC_lexicon_tags_2 },
    { 0x13a369c3, SUC_lexicon_tags_104 },
    { 0x2ae369c3, SUC_lexicon_tags_37 },
    { 0xcba369c5, SUC_lexicon_tags_12 },
    { 0xaee369c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6369ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6369ce, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576369d3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x38c369d5, SUC_lexicon_tags_7 },
    { 0xeee369d6, SUC_lexicon_tags_45 },
    { 0x720369d5, SUC_lexicon_tags_79 },
    { 0x2bc369d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848369dc, SUC_lexicon_tags_12 },
    { 0x3ea369dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb2369df, SUC_lexicon_tags_82 },
    { 0xf64369e0, SUC_lexicon_tags_48 },
    { 0x890369e1, SUC_lexicon_tags_87 },
    { 0x96a369e2, SUC_lexicon_tags_37 },
    { 0x910369e3, SUC_lexicon_tags_11 },
    { 0x32e369df, SUC_lexicon_tags_9 },
    { 0x932369e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e369e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe36369ea, SUC_lexicon_tags_25 },
    { 0x8ec369eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2369ee, SUC_lexicon_tags_27 },
    { 0x32369ee, SUC_lexicon_tags_37 },
    { 0xcbe369f0, SUC_lexicon_tags_7 },
    { 0x744369f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed2369f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a369f6, SUC_lexicon_tags_37 },
    { 0xc90369f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8369fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a36a03, SUC_lexicon_tags_23 },
    { 0x62036a04, SUC_lexicon_tags_38 },
    { 0x9ac36a05, SUC_lexicon_tags_27 },
    { 0xa6036a05, SUC_lexicon_tags_37 },
    { 0xe9c36a07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5636a0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d836a0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a36a17, SUC_lexicon_tags_10 },
    { 0x47236a18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7e36a1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d636a1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf4e36a1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59436a21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37636a24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x39236a27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7836a2a, SUC_lexicon_tags_31 },
    { 0x9e836a2b, SUC_lexicon_tags_35 },
    { 0xb7c36a2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdde36a2e, SUC_lexicon_tags_48 },
    { 0x1a436a2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee636a33, SUC_lexicon_tags_17 },
    { 0x5b836a34, SUC_lexicon_tags_43 },
    { 0xe036a33, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x36636a38, SUC_lexicon_tags_158 },
    { 0x41e36a38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc36a3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0636a41, SUC_lexicon_tags_37 },
    { 0x12436a41, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be36a45, SUC_lexicon_tags_19 },
    { 0xcc36a45, SUC_lexicon_tags_92 },
    { 0xc9636a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4436a4a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd5836a4c, SUC_lexicon_tags_25 },
    { 0x93836a4d, SUC_lexicon_tags_25 },
    { 0x79836a4e, SUC_lexicon_tags_27 },
    { 0x3d636a4f, SUC_lexicon_tags_1 },
    { 0xbf036a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18636a52, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52836a56, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x7c236a58, SUC_lexicon_tags_37 },
    { 0xe2e36a59, SUC_lexicon_tags_12 },
    { 0xf5436a58, SUC_lexicon_tags_32 },
    { 0x6e36a58, SUC_lexicon_tags_31 },
    { 0xca36a5c, SUC_lexicon_tags_55 },
    { 0xa6636a5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a36a61, SUC_lexicon_tags_14 },
    { 0xe7a36a62, SUC_lexicon_tags_60 },
    { 0x40c36a63, SUC_lexicon_tags_2 },
    { 0x79836a64, SUC_lexicon_tags_27 },
    { 0xcb436a65, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e836a6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d636a6f, SUC_lexicon_tags_17 },
    { 0x93236a70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7036a75, SUC_lexicon_tags_12 },
    { 0x8fe36a76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e36a7a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x11236a7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14236a7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa36a82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51436a84, SUC_lexicon_tags_49 },
    { 0x4436a85, SUC_lexicon_tags_17 },
    { 0x4d436a86, SUC_lexicon_tags_37 },
    { 0xe6436a85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x54436a89, SUC_lexicon_tags_79 },
    { 0xb8c36a89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4236a8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc436a94, SUC_lexicon_tags_37 },
    { 0x23836a95, SUC_lexicon_tags_37 },
    { 0x75a36a94, SUC_lexicon_tags_49 },
    { 0xbd236a95, SUC_lexicon_tags_37 },
    { 0x5c836a95, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc36a9b, SUC_lexicon_tags_15 },
    { 0x60636a9b, SUC_lexicon_tags_17 },
    { 0xafa36a9b, SUC_lexicon_tags_7 },
    { 0x60636a9e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1836aa1, SUC_lexicon_tags_27 },
    { 0x6e636aa1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33236aa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cc36aa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c36aad, SUC_lexicon_tags_17 },
    { 0xf7a36aae, SUC_lexicon_tags_37 },
    { 0x22a36aaf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8836ab1, SUC_lexicon_tags_56 },
    { 0x84a36ab2, SUC_lexicon_tags_66 },
    { 0xa6c36ab2, SUC_lexicon_tags_15 },
    { 0x61c36ab4, SUC_lexicon_tags_17 },
    { 0xd4636ab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e36abc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58636abe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da36ac2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13636acd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee36ad2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c36ad8, SUC_lexicon_tags_23 },
    { 0x9ce36ad8, SUC_lexicon_tags_10 },
    { 0x34e36ada, SUC_lexicon_tags_6 },
    { 0xc6036ad9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a036ae2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f836ae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96236ae7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e36aeb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb036aed, SUC_lexicon_tags_27 },
    { 0x82c36aed, SUC_lexicon_tags_17 },
    { 0x7be36aed, SUC_lexicon_tags_45 },
    { 0x2ca36af0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc36af4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa6c36af6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8036afd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0236b00, SUC_lexicon_tags_0 },
    { 0xf8236b01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc36b04, SUC_lexicon_tags_7 },
    { 0xf0836b05, SUC_lexicon_tags_63 },
    { 0x2ae36b05, SUC_lexicon_tags_2 },
    { 0xa3c36b04, SUC_lexicon_tags_77 },
    { 0x80836b05, SUC_lexicon_tags_64 },
    { 0xb9036b08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe0036b0b, SUC_lexicon_tags_41 },
    { 0x9f836b0c, SUC_lexicon_tags_17 },
    { 0xdda36b0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce36b11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27636b14, SUC_lexicon_tags_26 },
    { 0x8e636b15, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4636b18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb436b1a, SUC_lexicon_tags_11 },
    { 0x4e836b1b, SUC_lexicon_tags_26 },
    { 0x5dc36b1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad636b1e, SUC_lexicon_tags_36 },
    { 0x6a636b1e, SUC_lexicon_tags_31 },
    { 0x2e236b1e, SUC_lexicon_tags_0 },
    { 0x4ba36b21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1de36b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15436b25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26036b27, SUC_lexicon_tags_1 },
    { 0x67636b28, SUC_lexicon_tags_4 },
    { 0x6c236b29, SUC_lexicon_tags_7 },
    { 0x95c36b29, SUC_lexicon_tags_30 },
    { 0x4a236b2b, SUC_lexicon_tags_15 },
    { 0x7c236b29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e36b31, SUC_lexicon_tags_25 },
    { 0xdc36b32, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f836b35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39036b3a, SUC_lexicon_tags_47 },
    { 0x17e36b3a, SUC_lexicon_tags_1 },
    { 0x4ea36b3c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa36b42, SUC_lexicon_tags_107 },
    { 0x64a36b43, SUC_lexicon_tags_13 },
    { 0x87036b43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x97e36b46, SUC_lexicon_tags_6 },
    { 0x98236b47, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x89636b49, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f836b51, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0236b56, SUC_lexicon_tags_37 },
    { 0x9f636b57, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x8e836b59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40e36b5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43436b5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34036b63, SUC_lexicon_tags_234 },
    { 0x11836b64, SUC_lexicon_tags_23 },
    { 0xfc236b64, SUC_lexicon_tags_7 },
    { 0x26e36b66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x74e36b68, SUC_lexicon_tags_0 },
    { 0xefa36b68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e36b6d, SUC_lexicon_tags_98 },
    { 0x63636b6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a36b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed436b7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c36b82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5836b87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23036b89, SUC_lexicon_tags_30 },
    { 0x61e36b8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xde436b8d, SUC_lexicon_tags_2 },
    { 0x5c436b8d, SUC_lexicon_tags_6 },
    { 0x5da36b8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3636b98, SUC_lexicon_tags_44 },
    { 0x49436b99, SUC_lexicon_tags_31 },
    { 0x43a36b99, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a36b9d, SUC_lexicon_tags_25 },
    { 0x13436b9d, SUC_lexicon_tags_6 },
    { 0x39836b9f, SUC_lexicon_tags_31 },
    { 0xbec36ba0, SUC_lexicon_tags_17 },
    { 0x89636ba1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49436ba7, SUC_lexicon_tags_14 },
    { 0x8e836ba8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce436bac, SUC_lexicon_tags_8 },
    { 0xf5836bad, SUC_lexicon_tags_77 },
    { 0xd0e36bae, SUC_lexicon_tags_8 },
    { 0x6b236baf, SUC_lexicon_tags_14 },
    { 0xc7e36bae, SUC_lexicon_tags_66 },
    { 0x41a36bb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf036bb4, SUC_lexicon_tags_56 },
    { 0x64636bb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba036bb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30436bba, SUC_lexicon_tags_63 },
    { 0xc4636bbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23236bc3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e36bc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e36bcc, SUC_lexicon_tags_39 },
    { 0xa1036bcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61c36bcf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x48036bd1, SUC_lexicon_tags_11 },
    { 0x6ea36bd2, SUC_lexicon_tags_31 },
    { 0xa3036bd3, SUC_lexicon_tags_14 },
    { 0x2de36bd1, SUC_lexicon_tags_7 },
    { 0xe836bd5, SUC_lexicon_tags_12 },
    { 0xfc836bd5, SUC_lexicon_tags_60 },
    { 0x7e436bd7, SUC_lexicon_tags_17 },
    { 0x77e36bd8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfaa36bda, SUC_lexicon_tags_25 },
    { 0x65836bdb, SUC_lexicon_tags_12 },
    { 0x5c236bdb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7236bde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50436be2, SUC_lexicon_tags_4 },
    { 0x93e36be2, SUC_lexicon_tags_16 },
    { 0x1ba36be4, SUC_lexicon_tags_1 },
    { 0x4b436be5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce36beb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5fe36bed, SUC_lexicon_tags_26 },
    { 0x80636bee, SUC_lexicon_tags_37 },
    { 0xe8036bef, SUC_lexicon_tags_7 },
    { 0xfe236bef, SUC_lexicon_tags_25 },
    { 0x12a36bf0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac636bf5, SUC_lexicon_tags_6 },
    { 0xa7236bf6, SUC_lexicon_tags_12 },
    { 0xc3e36bf5, SUC_lexicon_tags_82 },
    { 0x5c236bf8, SUC_lexicon_tags_25 },
    { 0x6d436bf9, SUC_lexicon_tags_63 },
    { 0xafc36bf9, SUC_lexicon_tags_17 },
    { 0x27a36bfb, SUC_lexicon_tags_12 },
    { 0x12c36bfb, SUC_lexicon_tags_15 },
    { 0xd5836bfa, SUC_lexicon_tags_15 },
    { 0xe7c36bf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73836c04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc436c06, SUC_lexicon_tags_25 },
    { 0xfc036c07, SUC_lexicon_tags_11 },
    { 0x8aa36c06, SUC_lexicon_tags_14 },
    { 0x89e36c06, SUC_lexicon_tags_4 },
    { 0x14436c0a, SUC_lexicon_tags_21 },
    { 0xd2e36c07, SUC_lexicon_tags_11 },
    { 0x42636c0c, SUC_lexicon_tags_7 },
    { 0xf4436c0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb236c0f, SUC_lexicon_tags_32 },
    { 0xacc36c10, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24636c14, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x44836c17, SUC_lexicon_tags_48 },
    { 0xfb036c18, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93436c1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17036c21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cc36c23, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e836c29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8436c2c, SUC_lexicon_tags_11 },
    { 0x9a36c2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0436c2f, SUC_lexicon_tags_1 },
    { 0x55a36c30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa36c33, SUC_lexicon_tags_7 },
    { 0x63436c33, SUC_lexicon_tags_11 },
    { 0x8b436c35, SUC_lexicon_tags_47 },
    { 0x1de36c33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20036c3b, SUC_lexicon_tags_57 },
    { 0xa9a36c3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ac36c3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58a36c40, SUC_lexicon_tags_10 },
    { 0x62036c41, SUC_lexicon_tags_15 },
    { 0xdee36c42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c036c45, SUC_lexicon_tags_11 },
    { 0x22636c45, SUC_lexicon_tags_15 },
    { 0x1ce36c47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e436c4b, SUC_lexicon_tags_31 },
    { 0xeee36c4c, SUC_lexicon_tags_4 },
    { 0xafa36c4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79236c4f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc836c55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636c59, SUC_lexicon_tags_39 },
    { 0xc7636c5a, SUC_lexicon_tags_714 },
    { 0x36e36c5a, SUC_lexicon_tags_2 },
    { 0xc5436c5b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3cc36c5e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x69636c61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfc36c63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc36c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89236c73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x78436c76, SUC_lexicon_tags_14 },
    { 0xdcc36c77, SUC_lexicon_tags_87 },
    { 0xf7236c78, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x82436c7b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c36c80, SUC_lexicon_tags_66 },
    { 0x2636c81, SUC_lexicon_tags_7 },
    { 0x57836c82, SUC_lexicon_tags_93 },
    { 0xf8036c83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b036c88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41036c8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f436c8d, SUC_lexicon_tags_12 },
    { 0x8be36c8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x67636c90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d036c92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e36c95, SUC_lexicon_tags_27 },
    { 0xa4836c96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4636c9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3a36c9e, SUC_lexicon_tags_37 },
    { 0x1b436c9f, SUC_lexicon_tags_15 },
    { 0x49836ca0, SUC_lexicon_tags_9 },
    { 0xcee36ca1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65636ca3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4636ca6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a436caa, SUC_lexicon_tags_27 },
    { 0x73436cab, SUC_lexicon_tags_28 },
    { 0xea036cab, SUC_lexicon_tags_43 },
    { 0xddc36cad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c36cb0, SUC_lexicon_tags_25 },
    { 0xdb036cb0, SUC_lexicon_tags_11 },
    { 0x8cc36cb2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10236cb6, SUC_lexicon_tags_34 },
    { 0xe5836cb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c36cc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68436cc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22436ccb, SUC_lexicon_tags_65 },
    { 0x1c436ccc, SUC_lexicon_tags_11 },
    { 0xd8236ccd, SUC_lexicon_tags_7 },
    { 0x50836cce, SUC_lexicon_tags_37 },
    { 0x93a36ccd, SUC_lexicon_tags_68 },
    { 0x6e436cd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6236cd6, SUC_lexicon_tags_36 },
    { 0x16e36cd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d636cda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75436cde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a36ce3, SUC_lexicon_tags_48 },
    { 0x90e36ce4, SUC_lexicon_tags_79 },
    { 0xfca36ce3, SUC_lexicon_tags_37 },
    { 0xfc836ce6, SUC_lexicon_tags_28 },
    { 0xac036ce7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c436cea, SUC_lexicon_tags_7 },
    { 0xf9436cea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a36cef, SUC_lexicon_tags_11 },
    { 0x7636cf0, SUC_lexicon_tags_47 },
    { 0x11636cf1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45436cf5, SUC_lexicon_tags_11 },
    { 0x13636cf6, SUC_lexicon_tags_39 },
    { 0x8e836cf5, SUC_lexicon_tags_37 },
    { 0x74236cf8, SUC_lexicon_tags_2 },
    { 0x80c36cf9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x85836cfc, SUC_lexicon_tags_11 },
    { 0xfe236cfd, SUC_lexicon_tags_45 },
    { 0x12a36cfc, SUC_lexicon_tags_56 },
    { 0xcc636cfd, SUC_lexicon_tags_43 },
    { 0xcd036cff, SUC_lexicon_tags_25 },
    { 0x7c36cff, SUC_lexicon_tags_11 },
    { 0xe2636d00, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda236d06, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x21836d08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2636d0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33236d12, SUC_lexicon_tags_15 },
    { 0x68636d13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a36d16, SUC_lexicon_tags_26 },
    { 0xa6236d17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e436d19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17e36d1b, SUC_lexicon_tags_49 },
    { 0xfbe36d1c, SUC_lexicon_tags_64 },
    { 0x85036d1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0236d2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5836d2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d636d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da36d3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60636d42, SUC_lexicon_tags_19 },
    { 0x36d42, SUC_lexicon_tags_11 },
    { 0xc8e36d44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7c36d46, SUC_lexicon_tags_43 },
    { 0xbe836d47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea36d4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbc236d4c, SUC_lexicon_tags_14 },
    { 0x9ae36d4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb836d50, SUC_lexicon_tags_60 },
    { 0xc7a36d50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc36d54, SUC_lexicon_tags_25 },
    { 0xb3636d55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a436d58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe6436d5a, SUC_lexicon_tags_15 },
    { 0x6d436d5a, SUC_lexicon_tags_63 },
    { 0xa8e36d5c, SUC_lexicon_tags_29 },
    { 0x53636d5c, SUC_lexicon_tags_38 },
    { 0x7be36d5d, SUC_lexicon_tags_19 },
    { 0xbac36d5e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e36d62, SUC_lexicon_tags_49 },
    { 0x8dc36d62, SUC_lexicon_tags_37 },
    { 0xac236d62, SUC_lexicon_tags_8 },
    { 0xeae36d65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85436d68, SUC_lexicon_tags_12 },
    { 0x95c36d69, SUC_lexicon_tags_25 },
    { 0x51a36d68, SUC_lexicon_tags_2 },
    { 0xbac36d6b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e236d6f, SUC_lexicon_tags_11 },
    { 0xc9a36d70, SUC_lexicon_tags_56 },
    { 0x38c36d71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63436d75, SUC_lexicon_tags_25 },
    { 0xea36d76, SUC_lexicon_tags_37 },
    { 0x9d836d76, SUC_lexicon_tags_47 },
    { 0x92836d77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4836d7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4c236d7e, SUC_lexicon_tags_25 },
    { 0x37436d7f, SUC_lexicon_tags_9 },
    { 0xa8e36d80, SUC_lexicon_tags_11 },
    { 0x12e36d81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1e36d83, SUC_lexicon_tags_46 },
    { 0x1d836d84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56236d86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe3836d88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26236d8a, SUC_lexicon_tags_93 },
    { 0x22436d8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8636d8e, SUC_lexicon_tags_25 },
    { 0xa36d8e, SUC_lexicon_tags_46 },
    { 0xac36d90, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e36d97, SUC_lexicon_tags_17 },
    { 0x6ba36d97, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xec636d9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88036d9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e36da1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6e436da3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e36dab, SUC_lexicon_tags_23 },
    { 0x5d236dac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x61636dae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79636db5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfa436db7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c36dbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de36dc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc236dc2, SUC_lexicon_tags_38 },
    { 0x60e36dc2, SUC_lexicon_tags_47 },
    { 0x8ac36dc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f236dc6, SUC_lexicon_tags_45 },
    { 0x5f836dc6, SUC_lexicon_tags_64 },
    { 0xc3a36dc7, SUC_lexicon_tags_25 },
    { 0x6c636dc9, SUC_lexicon_tags_27 },
    { 0x51836dca, SUC_lexicon_tags_2 },
    { 0xcc436dc8, SUC_lexicon_tags_12 },
    { 0xf8e36dcc, SUC_lexicon_tags_2 },
    { 0xde436dcd, SUC_lexicon_tags_15 },
    { 0x74236dce, SUC_lexicon_tags_48 },
    { 0x14236dcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41036dd2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11636dd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae836dd7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a436ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6036de0, SUC_lexicon_tags_11 },
    { 0x52036de1, SUC_lexicon_tags_25 },
    { 0x7a836de2, SUC_lexicon_tags_66 },
    { 0xef236de3, SUC_lexicon_tags_33 },
    { 0x1b836de4, SUC_lexicon_tags_47 },
    { 0xf9a36de5, SUC_lexicon_tags_15 },
    { 0xc5636de5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e36deb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae36df0, SUC_lexicon_tags_15 },
    { 0xd5836df1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x28e36df3, SUC_lexicon_tags_2 },
    { 0x84e36df3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfc36df6, SUC_lexicon_tags_8 },
    { 0xe8436df7, SUC_lexicon_tags_31 },
    { 0xd2c36df8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x15a36dfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a36dff, SUC_lexicon_tags_47 },
    { 0xeda36e00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa2c36e02, SUC_lexicon_tags_29 },
    { 0x23636e03, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e836e06, SUC_lexicon_tags_17 },
    { 0x9e436e06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d836e09, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e36e0c, SUC_lexicon_tags_27 },
    { 0x81c36e0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1236e0f, SUC_lexicon_tags_37 },
    { 0x8a236e0f, SUC_lexicon_tags_17 },
    { 0x79c36e11, SUC_lexicon_tags_289 },
    { 0xa9636e11, SUC_lexicon_tags_1 },
    { 0xf1636e10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7036e15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ea36e17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c636e1a, SUC_lexicon_tags_2 },
    { 0x80036e1b, SUC_lexicon_tags_31 },
    { 0xacc36e1a, SUC_lexicon_tags_28 },
    { 0x31236e1b, SUC_lexicon_tags_1 },
    { 0xa3436e1c, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x61836e20, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e236e27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e36e2b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x11c36e2d, SUC_lexicon_tags_46 },
    { 0xdec36e2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65836e36, SUC_lexicon_tags_100 },
    { 0x7ee36e37, SUC_lexicon_tags_49 },
    { 0xa4e36e37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e36e3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x66436e3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79c36e3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f636e48, SUC_lexicon_tags_41 },
    { 0xb6836e49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8036e4f, SUC_lexicon_tags_17 },
    { 0xde436e50, SUC_lexicon_tags_1 },
    { 0x8dc36e50, SUC_lexicon_tags_15 },
    { 0xa6e36e52, SUC_lexicon_tags_63 },
    { 0x52c36e50, SUC_lexicon_tags_56 },
    { 0xc2036e54, SUC_lexicon_tags_45 },
    { 0xa7036e53, SUC_lexicon_tags_37 },
    { 0x30436e55, SUC_lexicon_tags_38 },
    { 0xc2e36e57, SUC_lexicon_tags_82 },
    { 0x14636e58, SUC_lexicon_tags_1 },
    { 0xc6636e58, SUC_lexicon_tags_32 },
    { 0x6fc36e58, SUC_lexicon_tags_47 },
    { 0xc7836e5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa636e5d, SUC_lexicon_tags_11 },
    { 0xfc036e5e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53836e60, SUC_lexicon_tags_12 },
    { 0x23e36e61, SUC_lexicon_tags_12 },
    { 0x53236e61, SUC_lexicon_tags_11 },
    { 0xf436e63, SUC_lexicon_tags_25 },
    { 0xf7436e64, SUC_lexicon_tags_15 },
    { 0xdc436e63, SUC_lexicon_tags_12 },
    { 0x19236e65, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb5636e68, SUC_lexicon_tags_49 },
    { 0x16636e68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11436e6d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6236e70, SUC_lexicon_tags_29 },
    { 0x78c36e71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e36e76, SUC_lexicon_tags_14 },
    { 0x36c36e77, SUC_lexicon_tags_25 },
    { 0x5cc36e78, SUC_lexicon_tags_0 },
    { 0xb1c36e77, SUC_lexicon_tags_14 },
    { 0x14636e78, SUC_lexicon_tags_14 },
    { 0x8e636e78, SUC_lexicon_tags_27 },
    { 0x1de36e7c, SUC_lexicon_tags_92 },
    { 0x77636e78, SUC_lexicon_tags_15 },
    { 0x47a36e76, SUC_lexicon_tags_14 },
    { 0x41636e7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19436e83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd436e86, SUC_lexicon_tags_25 },
    { 0x5ac36e86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5236e89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5836e8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89036e8d, SUC_lexicon_tags_17 },
    { 0x14436e8e, SUC_lexicon_tags_1 },
    { 0x5b636e8d, SUC_lexicon_tags_17 },
    { 0x4a236e90, SUC_lexicon_tags_37 },
    { 0x3836e8f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c36e9a, SUC_lexicon_tags_60 },
    { 0x29036e9b, SUC_lexicon_tags_38 },
    { 0xf7c36e9b, SUC_lexicon_tags_14 },
    { 0xa5c36e9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e36ea0, SUC_lexicon_tags_12 },
    { 0x5c636ea1, SUC_lexicon_tags_26 },
    { 0x10a36ea2, SUC_lexicon_tags_37 },
    { 0x51c36ea2, SUC_lexicon_tags_1 },
    { 0x6da36ea4, SUC_lexicon_tags_23 },
    { 0xea836ea5, SUC_lexicon_tags_17 },
    { 0x41636ea5, SUC_lexicon_tags_38 },
    { 0xc8c36ea5, SUC_lexicon_tags_28 },
    { 0xb6236ea8, SUC_lexicon_tags_37 },
    { 0x4e836ea9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb8036eab, SUC_lexicon_tags_112 },
    { 0xae236eac, SUC_lexicon_tags_49 },
    { 0xfa036eac, SUC_lexicon_tags_27 },
    { 0xfb836ead, SUC_lexicon_tags_37 },
    { 0xc0836eaf, SUC_lexicon_tags_7 },
    { 0x3e836eaf, SUC_lexicon_tags_25 },
    { 0x76a36eab, SUC_lexicon_tags_10 },
    { 0x20436eaf, SUC_lexicon_tags_43 },
    { 0xb236eb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e36eb6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc0c36eb8, SUC_lexicon_tags_15 },
    { 0xc6a36eb9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb836ebb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c36ebe, SUC_lexicon_tags_47 },
    { 0x27036ebf, SUC_lexicon_tags_26 },
    { 0xe9e36ec0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42236ec2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e436ec4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe36ec7, SUC_lexicon_tags_43 },
    { 0x78036ec8, SUC_lexicon_tags_349 },
    { 0xed236ec9, SUC_lexicon_tags_17 },
    { 0xcc836ec7, SUC_lexicon_tags_21 },
    { 0xbfe36ecb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2636ecd, SUC_lexicon_tags_11 },
    { 0x37e36ecd, SUC_lexicon_tags_15 },
    { 0x30a36ecf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4436eda, SUC_lexicon_tags_0 },
    { 0x67e36edb, SUC_lexicon_tags_2 },
    { 0x8e36edc, SUC_lexicon_tags_10 },
    { 0xeb636edd, SUC_lexicon_tags_31 },
    { 0x7b636edd, SUC_lexicon_tags_77 },
    { 0xf5436edb, SUC_lexicon_tags_65 },
    { 0x7a36edf, SUC_lexicon_tags_87 },
    { 0x75a36edd, SUC_lexicon_tags_0 },
    { 0xf6436ee0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c36ee5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90436ee7, SUC_lexicon_tags_201 },
    { 0x9da36ee7, SUC_lexicon_tags_43 },
    { 0xb436ee9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdce36eeb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77436ef1, SUC_lexicon_tags_7 },
    { 0x26436ef1, SUC_lexicon_tags_28 },
    { 0xf4636ef3, SUC_lexicon_tags_8 },
    { 0xf0a36ef4, SUC_lexicon_tags_14 },
    { 0x20636ef5, SUC_lexicon_tags_1 },
    { 0xa0636ef6, SUC_lexicon_tags_37 },
    { 0xa0036ef2, SUC_lexicon_tags_15 },
    { 0xee836ef6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33236f02, SUC_lexicon_tags_12 },
    { 0x11636f02, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4036f07, SUC_lexicon_tags_8 },
    { 0x83236f07, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a36f0c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc1c36f0e, SUC_lexicon_tags_118 },
    { 0xede36f0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95036f15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc7836f17, SUC_lexicon_tags_1 },
    { 0xcea36f17, SUC_lexicon_tags_26 },
    { 0x88e36f19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc636f1b, SUC_lexicon_tags_31 },
    { 0xc5036f1c, SUC_lexicon_tags_2 },
    { 0x90436f1b, SUC_lexicon_tags_25 },
    { 0xf9236f1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6036f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c36f25, SUC_lexicon_tags_17 },
    { 0xb836f26, SUC_lexicon_tags_15 },
    { 0x20636f26, SUC_lexicon_tags_14 },
    { 0x40a36f26, SUC_lexicon_tags_26 },
    { 0x88436f26, SUC_lexicon_tags_7 },
    { 0x2e836f2a, SUC_lexicon_tags_65 },
    { 0x88e36f28, SUC_lexicon_tags_12 },
    { 0xaba36f28, SUC_lexicon_tags_2 },
    { 0x51636f2d, SUC_lexicon_tags_11 },
    { 0xabc36f2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9236f30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95036f35, SUC_lexicon_tags_10 },
    { 0x4836f36, SUC_lexicon_tags_17 },
    { 0xd8236f35, SUC_lexicon_tags_25 },
    { 0x16e36f38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf636f3a, SUC_lexicon_tags_27 },
    { 0xe4636f3b, SUC_lexicon_tags_38 },
    { 0x11e36f3b, SUC_lexicon_tags_2 },
    { 0xb7236f3d, SUC_lexicon_tags_35 },
    { 0xe5636f3d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8636f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe236f4b, SUC_lexicon_tags_26 },
    { 0x30636f4b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba36f51, SUC_lexicon_tags_35 },
    { 0x4ac36f52, SUC_lexicon_tags_15 },
    { 0x1a236f52, SUC_lexicon_tags_6 },
    { 0x2a436f54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7236f57, SUC_lexicon_tags_37 },
    { 0x83236f57, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x71236f5a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a36f5e, SUC_lexicon_tags_59 },
    { 0xf7836f5f, SUC_lexicon_tags_38 },
    { 0x53a36f5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a36f6a, SUC_lexicon_tags_15 },
    { 0xb8036f6a, SUC_lexicon_tags_31 },
    { 0xf6236f6c, SUC_lexicon_tags_26 },
    { 0x73036f6d, SUC_lexicon_tags_45 },
    { 0xbb436f6d, SUC_lexicon_tags_48 },
    { 0x9ba36f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1836f72, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf636f77, SUC_lexicon_tags_39 },
    { 0x9436f78, SUC_lexicon_tags_14 },
    { 0x8836f78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa36f81, SUC_lexicon_tags_42 },
    { 0x77a36f81, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b436f85, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e636f88, SUC_lexicon_tags_26 },
    { 0x29836f88, SUC_lexicon_tags_47 },
    { 0x21a36f89, SUC_lexicon_tags_7 },
    { 0x63c36f8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94436f8e, SUC_lexicon_tags_1 },
    { 0xa1836f8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x63836f92, SUC_lexicon_tags_12 },
    { 0xe3236f93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f636f96, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa6836f98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a236f9a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22436fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c36fa8, SUC_lexicon_tags_712 },
    { 0x4236fa9, SUC_lexicon_tags_7 },
    { 0x6a236fa8, SUC_lexicon_tags_82 },
    { 0x1e036fab, SUC_lexicon_tags_38 },
    { 0x6ec36faa, SUC_lexicon_tags_38 },
    { 0x59036fab, SUC_lexicon_tags_47 },
    { 0x7ae36fae, SUC_lexicon_tags_25 },
    { 0x2ac36faf, SUC_lexicon_tags_15 },
    { 0x69636fb0, SUC_lexicon_tags_11 },
    { 0x7a636faf, SUC_lexicon_tags_2 },
    { 0x6e436fb2, SUC_lexicon_tags_37 },
    { 0x26036fb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x75036fb6, SUC_lexicon_tags_9 },
    { 0x4a636fb7, SUC_lexicon_tags_15 },
    { 0x87436fb6, SUC_lexicon_tags_47 },
    { 0xe8e36fb6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4236fbd, SUC_lexicon_tags_23 },
    { 0x9e436fbd, SUC_lexicon_tags_37 },
    { 0x42836fbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c36fc2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a036fc7, SUC_lexicon_tags_30 },
    { 0x29236fc8, SUC_lexicon_tags_37 },
    { 0x7c436fc9, SUC_lexicon_tags_17 },
    { 0xbee36fca, SUC_lexicon_tags_29 },
    { 0xe8e36fc8, SUC_lexicon_tags_4 },
    { 0xbb036fcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee36fd2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2ee36fd4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8c236fd6, SUC_lexicon_tags_12 },
    { 0x9e836fd7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a36fdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1236fde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a36fe1, SUC_lexicon_tags_17 },
    { 0x85836fe2, SUC_lexicon_tags_14 },
    { 0x96436fe1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c36feb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39236ff6, SUC_lexicon_tags_37 },
    { 0xe2436ff7, SUC_lexicon_tags_8 },
    { 0xf5036ff8, SUC_lexicon_tags_7 },
    { 0x56836ff9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6836fff, SUC_lexicon_tags_17 },
    { 0x23636fff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1037002, SUC_lexicon_tags_27 },
    { 0xb7a37002, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c37008, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3003700a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd443700c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf363700e, SUC_lexicon_tags_18 },
    { 0x7ca3700f, SUC_lexicon_tags_9 },
    { 0xb0a37010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96437012, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c437014, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e037017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7437019, SUC_lexicon_tags_45 },
    { 0x6303701a, SUC_lexicon_tags_7 },
    { 0xb5c37019, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d03701d, SUC_lexicon_tags_57 },
    { 0x5d63701d, SUC_lexicon_tags_37 },
    { 0x4c43701f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc0037021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49237025, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c23702b, SUC_lexicon_tags_45 },
    { 0x90c3702c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf923702e, SUC_lexicon_tags_26 },
    { 0xeb83702e, SUC_lexicon_tags_15 },
    { 0x9a83702e, SUC_lexicon_tags_15 },
    { 0xeb437031, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5143703b, SUC_lexicon_tags_8 },
    { 0x1c63703b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a37040, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x12437042, SUC_lexicon_tags_16 },
    { 0x23e37042, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e37046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe37048, SUC_lexicon_tags_21 },
    { 0xf7437049, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94637050, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e37053, SUC_lexicon_tags_27 },
    { 0x1fa37054, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce837058, SUC_lexicon_tags_11 },
    { 0xcca37059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4943705b, SUC_lexicon_tags_17 },
    { 0x8ec3705c, SUC_lexicon_tags_11 },
    { 0x8e43705d, SUC_lexicon_tags_11 },
    { 0xdee3705d, SUC_lexicon_tags_11 },
    { 0xa9c3705c, SUC_lexicon_tags_29 },
    { 0x78c37060, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e37063, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a37068, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73837070, SUC_lexicon_tags_15 },
    { 0x57437071, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x69637073, SUC_lexicon_tags_60 },
    { 0x75c37074, SUC_lexicon_tags_63 },
    { 0xb4237075, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43637077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be3707b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3603707e, SUC_lexicon_tags_47 },
    { 0x5983707f, SUC_lexicon_tags_25 },
    { 0x3463707f, SUC_lexicon_tags_37 },
    { 0xc183707f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c37085, SUC_lexicon_tags_17 },
    { 0x35437086, SUC_lexicon_tags_23 },
    { 0xff437086, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xce43708a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5083708d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e37092, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6037096, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e3709a, SUC_lexicon_tags_15 },
    { 0xf403709b, SUC_lexicon_tags_34 },
    { 0xcb43709c, SUC_lexicon_tags_8 },
    { 0x2d63709c, SUC_lexicon_tags_43 },
    { 0xb263709a, SUC_lexicon_tags_25 },
    { 0x70c3709a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74370a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8370ab, SUC_lexicon_tags_56 },
    { 0x660370ac, SUC_lexicon_tags_17 },
    { 0x9e6370ab, SUC_lexicon_tags_47 },
    { 0xe2370ab, SUC_lexicon_tags_1 },
    { 0xe2370ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf38370b1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14370b4, SUC_lexicon_tags_65 },
    { 0xdbe370b5, SUC_lexicon_tags_37 },
    { 0x6be370b6, SUC_lexicon_tags_66 },
    { 0x3a370b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xca8370b9, SUC_lexicon_tags_15 },
    { 0xe46370b9, SUC_lexicon_tags_11 },
    { 0x896370bb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a370c1, SUC_lexicon_tags_116 },
    { 0xbe370c2, SUC_lexicon_tags_0 },
    { 0xf94370c1, SUC_lexicon_tags_15 },
    { 0x4a4370c4, SUC_lexicon_tags_12 },
    { 0x92a370c4, SUC_lexicon_tags_56 },
    { 0x326370c6, SUC_lexicon_tags_47 },
    { 0x22a370c7, SUC_lexicon_tags_47 },
    { 0x86a370c8, SUC_lexicon_tags_25 },
    { 0x3b0370c9, SUC_lexicon_tags_2 },
    { 0x356370c6, SUC_lexicon_tags_45 },
    { 0x69c370c7, SUC_lexicon_tags_11 },
    { 0xb6e370c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb14370ce, SUC_lexicon_tags_37 },
    { 0x992370cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6e370d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0370d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89e370d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a370d9, SUC_lexicon_tags_23 },
    { 0x6a370da, SUC_lexicon_tags_16 },
    { 0x4c0370db, SUC_lexicon_tags_25 },
    { 0xdee370d9, SUC_lexicon_tags_23 },
    { 0x23c370dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3fc370df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a370e3, SUC_lexicon_tags_11 },
    { 0x84c370e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e370ea, SUC_lexicon_tags_10 },
    { 0xef0370eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2370f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfd4370f2, SUC_lexicon_tags_28 },
    { 0x348370f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf2370f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee370f8, SUC_lexicon_tags_37 },
    { 0xeaa370f8, SUC_lexicon_tags_12 },
    { 0xac2370fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca370ff, SUC_lexicon_tags_12 },
    { 0xa2037100, SUC_lexicon_tags_29 },
    { 0xc037101, SUC_lexicon_tags_49 },
    { 0x29437102, SUC_lexicon_tags_19 },
    { 0x64e37101, SUC_lexicon_tags_11 },
    { 0x4ba37102, SUC_lexicon_tags_63 },
    { 0x4e637102, SUC_lexicon_tags_7 },
    { 0xe437105, SUC_lexicon_tags_17 },
    { 0x5a637105, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37237109, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de3710d, SUC_lexicon_tags_15 },
    { 0xf863710e, SUC_lexicon_tags_7 },
    { 0x9ca3710e, SUC_lexicon_tags_45 },
    { 0xc9c37110, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e37115, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d23711c, SUC_lexicon_tags_80 },
    { 0xff03711d, SUC_lexicon_tags_13 },
    { 0x8203711d, SUC_lexicon_tags_61 },
    { 0x5983711e, SUC_lexicon_tags_262 },
    { 0x69437120, SUC_lexicon_tags_7 },
    { 0x91637120, SUC_lexicon_tags_55 },
    { 0x84837122, SUC_lexicon_tags_23 },
    { 0xe643711c, SUC_lexicon_tags_11 },
    { 0xc6437124, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52237127, SUC_lexicon_tags_10 },
    { 0x9a437128, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea3712b, SUC_lexicon_tags_17 },
    { 0x3a83712b, SUC_lexicon_tags_25 },
    { 0x3ba3712c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a37131, SUC_lexicon_tags_0 },
    { 0x54a37131, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f837136, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923713a, SUC_lexicon_tags_25 },
    { 0xd8a3713a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9423713d, SUC_lexicon_tags_17 },
    { 0x43a3713e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a37142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4437144, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e37147, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3183714a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce37153, SUC_lexicon_tags_38 },
    { 0x6f637154, SUC_lexicon_tags_12 },
    { 0x34237153, SUC_lexicon_tags_12 },
    { 0x9fc37155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e23715a, SUC_lexicon_tags_17 },
    { 0x3bc3715b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb0e3715d, SUC_lexicon_tags_45 },
    { 0x3b03715d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e37162, SUC_lexicon_tags_37 },
    { 0x62a37163, SUC_lexicon_tags_17 },
    { 0xb1e37164, SUC_lexicon_tags_33 },
    { 0xecc37163, SUC_lexicon_tags_37 },
    { 0x11237165, SUC_lexicon_tags_14 },
    { 0x11c37165, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a3716b, SUC_lexicon_tags_35 },
    { 0x9183716b, SUC_lexicon_tags_33 },
    { 0xcc03716d, SUC_lexicon_tags_2 },
    { 0xeda3716b, SUC_lexicon_tags_1 },
    { 0x2be3716f, SUC_lexicon_tags_23 },
    { 0x9de37170, SUC_lexicon_tags_25 },
    { 0x6ac3716d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x24437173, SUC_lexicon_tags_31 },
    { 0x63c37173, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7037176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f437179, SUC_lexicon_tags_2 },
    { 0x5123717a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb237180, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e637182, SUC_lexicon_tags_11 },
    { 0x3d437182, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c237186, SUC_lexicon_tags_46 },
    { 0x5a037187, SUC_lexicon_tags_15 },
    { 0xe6e37187, SUC_lexicon_tags_25 },
    { 0xe7a37189, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe3718c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5be3718e, SUC_lexicon_tags_60 },
    { 0xa603718e, SUC_lexicon_tags_23 },
    { 0xb8437190, SUC_lexicon_tags_11 },
    { 0x88437191, SUC_lexicon_tags_25 },
    { 0x21237191, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe237195, SUC_lexicon_tags_17 },
    { 0x24a37196, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8583719c, SUC_lexicon_tags_1 },
    { 0x2f83719d, SUC_lexicon_tags_47 },
    { 0xa263719c, SUC_lexicon_tags_37 },
    { 0x8ee3719c, SUC_lexicon_tags_463 },
    { 0xf2e3719f, SUC_lexicon_tags_37 },
    { 0x938371a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a371a6, SUC_lexicon_tags_27 },
    { 0xc58371a7, SUC_lexicon_tags_33 },
    { 0x734371a8, SUC_lexicon_tags_15 },
    { 0x8be371a9, SUC_lexicon_tags_14 },
    { 0xb54371aa, SUC_lexicon_tags_95 },
    { 0x4ba371ab, SUC_lexicon_tags_48 },
    { 0x98c371a7, SUC_lexicon_tags_28 },
    { 0x470371ad, SUC_lexicon_tags_16 },
    { 0xae6371ad, SUC_lexicon_tags_23 },
    { 0x948371ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6371b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc0371b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x482371ba, SUC_lexicon_tags_61 },
    { 0x1b4371bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c371c5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16371c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a371cc, SUC_lexicon_tags_59 },
    { 0x430371cd, SUC_lexicon_tags_34 },
    { 0x202371ce, SUC_lexicon_tags_8 },
    { 0x188371ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64371d2, SUC_lexicon_tags_10 },
    { 0xf36371d2, SUC_lexicon_tags_12 },
    { 0x3fe371d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50371dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20371de, SUC_lexicon_tags_31 },
    { 0xa1a371de, SUC_lexicon_tags_2 },
    { 0xe7c371de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812371e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc371e7, SUC_lexicon_tags_37 },
    { 0xb20371e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x860371ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94371f2, SUC_lexicon_tags_33 },
    { 0xe6a371f3, SUC_lexicon_tags_11 },
    { 0x62e371f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4371f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17e371fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab6371fc, SUC_lexicon_tags_43 },
    { 0xe4371fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69237200, SUC_lexicon_tags_37 },
    { 0x83a37200, SUC_lexicon_tags_25 },
    { 0xfda37201, SUC_lexicon_tags_2 },
    { 0xa9e37201, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9a37205, SUC_lexicon_tags_37 },
    { 0xec437206, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9ba37208, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae3720f, SUC_lexicon_tags_1 },
    { 0x9383720f, SUC_lexicon_tags_11 },
    { 0xfc837211, SUC_lexicon_tags_47 },
    { 0xfe437212, SUC_lexicon_tags_12 },
    { 0xa5a37213, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a37218, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1063721f, SUC_lexicon_tags_37 },
    { 0x1263721f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73a37222, SUC_lexicon_tags_15 },
    { 0x9e437222, SUC_lexicon_tags_70 },
    { 0xc0e37224, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea037226, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf03722f, SUC_lexicon_tags_25 },
    { 0x71637230, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32637235, SUC_lexicon_tags_25 },
    { 0xdfe37236, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94637239, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3fc3723b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e3723d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93c3723f, SUC_lexicon_tags_33 },
    { 0x90c37240, SUC_lexicon_tags_25 },
    { 0xdda37241, SUC_lexicon_tags_26 },
    { 0x55637241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8a37244, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf437246, SUC_lexicon_tags_29 },
    { 0x4fe37247, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4023724d, SUC_lexicon_tags_37 },
    { 0xfca3724e, SUC_lexicon_tags_33 },
    { 0x59a3724e, SUC_lexicon_tags_0 },
    { 0x2063724e, SUC_lexicon_tags_2 },
    { 0x9f037251, SUC_lexicon_tags_25 },
    { 0xab037252, SUC_lexicon_tags_1 },
    { 0x92a37251, SUC_lexicon_tags_17 },
    { 0x4fc37254, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa243725f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67c37261, SUC_lexicon_tags_34 },
    { 0x68637261, SUC_lexicon_tags_33 },
    { 0x8ee37261, SUC_lexicon_tags_63 },
    { 0x92837261, SUC_lexicon_tags_146 },
    { 0x2ce37265, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea3726c, SUC_lexicon_tags_45 },
    { 0x3ae3726c, SUC_lexicon_tags_27 },
    { 0xb8a3726c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe37271, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd7037273, SUC_lexicon_tags_47 },
    { 0x45837273, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3727c, SUC_lexicon_tags_15 },
    { 0x5423727d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a37280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda03728a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc743728d, SUC_lexicon_tags_14 },
    { 0xe863728d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcc837290, SUC_lexicon_tags_49 },
    { 0x97e37290, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55837298, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb803729b, SUC_lexicon_tags_12 },
    { 0x6343729b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6372a1, SUC_lexicon_tags_7 },
    { 0x890372a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08372ab, SUC_lexicon_tags_17 },
    { 0x846372ab, SUC_lexicon_tags_17 },
    { 0x2fc372ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e6372af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf2e372b1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8372b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x396372b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be372be, SUC_lexicon_tags_47 },
    { 0x750372be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf38372c1, SUC_lexicon_tags_17 },
    { 0xdb6372c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28372c4, SUC_lexicon_tags_7 },
    { 0xb46372c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4cc372c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc372ca, SUC_lexicon_tags_17 },
    { 0x36372cb, SUC_lexicon_tags_17 },
    { 0xcc6372cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202372d4, SUC_lexicon_tags_23 },
    { 0x4a4372d5, SUC_lexicon_tags_10 },
    { 0x930372d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbe372d8, SUC_lexicon_tags_11 },
    { 0xec2372d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa40372db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2372e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea372e6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688372ec, SUC_lexicon_tags_38 },
    { 0xcd0372ed, SUC_lexicon_tags_47 },
    { 0x2e2372ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc16372f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf62372f2, SUC_lexicon_tags_37 },
    { 0xa16372f3, SUC_lexicon_tags_57 },
    { 0xd0c372f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa372f9, SUC_lexicon_tags_12 },
    { 0x6b8372fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48372ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8037302, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x39437304, SUC_lexicon_tags_25 },
    { 0x47a37305, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab43730e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5f637310, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28837315, SUC_lexicon_tags_11 },
    { 0x1da37316, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x81837319, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2563731b, SUC_lexicon_tags_7 },
    { 0xc763731c, SUC_lexicon_tags_17 },
    { 0xcf23731b, SUC_lexicon_tags_19 },
    { 0x2183731e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0037324, SUC_lexicon_tags_1061 },
    { 0, NULL },
    { 0x21e37326, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc3732e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x78637331, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c3733b, SUC_lexicon_tags_140 },
    { 0xf903733c, SUC_lexicon_tags_27 },
    { 0xa783733c, SUC_lexicon_tags_17 },
    { 0x6563733c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea37342, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a37346, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa143734b, SUC_lexicon_tags_25 },
    { 0x13c3734c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa83734e, SUC_lexicon_tags_12 },
    { 0xe183734e, SUC_lexicon_tags_14 },
    { 0x5e037350, SUC_lexicon_tags_7 },
    { 0x5b83734e, SUC_lexicon_tags_11 },
    { 0xe837352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4c37354, SUC_lexicon_tags_43 },
    { 0x67e37355, SUC_lexicon_tags_7 },
    { 0x3cc37354, SUC_lexicon_tags_77 },
    { 0x29e37357, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4437359, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc923735c, SUC_lexicon_tags_1 },
    { 0x63a3735d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ba3735f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd237363, SUC_lexicon_tags_87 },
    { 0xfb437364, SUC_lexicon_tags_14 },
    { 0xd3237364, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d837368, SUC_lexicon_tags_26 },
    { 0xb3c37369, SUC_lexicon_tags_18 },
    { 0x16a3736a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca3736f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d237375, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c37378, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1837385, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a237388, SUC_lexicon_tags_32 },
    { 0x5b037389, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d43738c, SUC_lexicon_tags_26 },
    { 0x2a03738c, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e37391, SUC_lexicon_tags_3 },
    { 0x7fa37391, SUC_lexicon_tags_17 },
    { 0xefc37391, SUC_lexicon_tags_17 },
    { 0xea837394, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e837396, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb437399, SUC_lexicon_tags_11 },
    { 0x9783739a, SUC_lexicon_tags_48 },
    { 0x22e37399, SUC_lexicon_tags_2 },
    { 0x2a43739c, SUC_lexicon_tags_2 },
    { 0x2463739c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a373a0, SUC_lexicon_tags_17 },
    { 0xd3e373a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0373a4, SUC_lexicon_tags_43 },
    { 0xc9c373a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac373a9, SUC_lexicon_tags_17 },
    { 0x936373aa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2ae373ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x660373ae, SUC_lexicon_tags_68 },
    { 0x6ea373af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ba373b1, SUC_lexicon_tags_55 },
    { 0x674373b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae373b5, SUC_lexicon_tags_3 },
    { 0xca6373b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x510373b8, SUC_lexicon_tags_27 },
    { 0x9f8373b9, SUC_lexicon_tags_33 },
    { 0x4b2373ba, SUC_lexicon_tags_0 },
    { 0xa98373bb, SUC_lexicon_tags_9 },
    { 0x78373bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x304373bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4373c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xada373c7, SUC_lexicon_tags_92 },
    { 0x680373c8, SUC_lexicon_tags_284 },
    { 0x232373c9, SUC_lexicon_tags_38 },
    { 0xd16373ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08373cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0373d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ac373d6, SUC_lexicon_tags_45 },
    { 0xbec373d7, SUC_lexicon_tags_63 },
    { 0x956373d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x560373db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6373e1, SUC_lexicon_tags_11 },
    { 0x9d6373e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700373ee, SUC_lexicon_tags_17 },
    { 0x27e373ef, SUC_lexicon_tags_7 },
    { 0x5f8373ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51a373f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x292373f4, SUC_lexicon_tags_40 },
    { 0x3ec373f4, SUC_lexicon_tags_33 },
    { 0xb2e373f6, SUC_lexicon_tags_25 },
    { 0xcd4373f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6373fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe46373fc, SUC_lexicon_tags_38 },
    { 0xc6a373fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4437401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7437403, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4e37405, SUC_lexicon_tags_25 },
    { 0xaa237406, SUC_lexicon_tags_66 },
    { 0x70637407, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a3740b, SUC_lexicon_tags_7 },
    { 0x9623740b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9c3740e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76437410, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9037412, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe37415, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee37419, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x30a3741b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7ec3741d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb283741f, SUC_lexicon_tags_2 },
    { 0xed437420, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e37426, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e37429, SUC_lexicon_tags_30 },
    { 0x2d037429, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c3742e, SUC_lexicon_tags_46 },
    { 0x91e3742f, SUC_lexicon_tags_48 },
    { 0x2a03742f, SUC_lexicon_tags_27 },
    { 0x30a3742f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a37438, SUC_lexicon_tags_30 },
    { 0x4f237439, SUC_lexicon_tags_82 },
    { 0x96a37438, SUC_lexicon_tags_8 },
    { 0xade3743b, SUC_lexicon_tags_17 },
    { 0x2fa37438, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18237443, SUC_lexicon_tags_7 },
    { 0x81837444, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e37447, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63637449, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a83744c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a37451, SUC_lexicon_tags_37 },
    { 0xeb637452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a37455, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c37458, SUC_lexicon_tags_25 },
    { 0xb0837458, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e37462, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7237465, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfde37467, SUC_lexicon_tags_11 },
    { 0x7ca37467, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa543746b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd23746f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f237474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b237478, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47637481, SUC_lexicon_tags_7 },
    { 0xd837482, SUC_lexicon_tags_38 },
    { 0x5a37482, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84637485, SUC_lexicon_tags_27 },
    { 0x77e37486, SUC_lexicon_tags_7 },
    { 0x8e837486, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e3748b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6123748e, SUC_lexicon_tags_30 },
    { 0x89e3748f, SUC_lexicon_tags_25 },
    { 0x9b837490, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95037492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e037494, SUC_lexicon_tags_11 },
    { 0x47c37494, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23749e, SUC_lexicon_tags_7 },
    { 0xb9e3749f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe8374a1, SUC_lexicon_tags_7 },
    { 0x976374a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16374a6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c374ad, SUC_lexicon_tags_7 },
    { 0x814374ae, SUC_lexicon_tags_7 },
    { 0xa6374ae, SUC_lexicon_tags_14 },
    { 0xbda374ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb36374b2, SUC_lexicon_tags_60 },
    { 0x560374b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856374b7, SUC_lexicon_tags_25 },
    { 0xaec374b8, SUC_lexicon_tags_23 },
    { 0x64e374b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386374bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9c374bf, SUC_lexicon_tags_10 },
    { 0xed4374bf, SUC_lexicon_tags_15 },
    { 0x3a2374c1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba374c6, SUC_lexicon_tags_9 },
    { 0xae4374c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6374cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58374cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xba374d1, SUC_lexicon_tags_8 },
    { 0x894374d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e374d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x430374d9, SUC_lexicon_tags_2 },
    { 0x7a8374d9, SUC_lexicon_tags_47 },
    { 0x966374db, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc374e0, SUC_lexicon_tags_25 },
    { 0xf68374e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216374e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93a374e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6374ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c374f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x992374f3, SUC_lexicon_tags_46 },
    { 0x7dc374f4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a374fa, SUC_lexicon_tags_7 },
    { 0x328374fb, SUC_lexicon_tags_29 },
    { 0xe66374fb, SUC_lexicon_tags_35 },
    { 0x668374fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25837500, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44437502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a37506, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9963750a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b83750c, SUC_lexicon_tags_29 },
    { 0xd743750c, SUC_lexicon_tags_33 },
    { 0xa7e3750d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e037511, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x4437513, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8437515, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a37521, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x61c37523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f43752a, SUC_lexicon_tags_17 },
    { 0xb2e3752b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b83752f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa037532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57837538, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8de3753a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de3753f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x35837541, SUC_lexicon_tags_2 },
    { 0xae237542, SUC_lexicon_tags_25 },
    { 0xfc437542, SUC_lexicon_tags_38 },
    { 0x97e37544, SUC_lexicon_tags_15 },
    { 0xf7637542, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e37548, SUC_lexicon_tags_25 },
    { 0x10c37549, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da3754e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2037551, SUC_lexicon_tags_12 },
    { 0x59237551, SUC_lexicon_tags_43 },
    { 0xa7a37553, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa37557, SUC_lexicon_tags_8 },
    { 0x5a837558, SUC_lexicon_tags_15 },
    { 0x7c637557, SUC_lexicon_tags_29 },
    { 0xb583755a, SUC_lexicon_tags_39 },
    { 0xf063755b, SUC_lexicon_tags_27 },
    { 0xee23755c, SUC_lexicon_tags_65 },
    { 0x7e3755d, SUC_lexicon_tags_11 },
    { 0xdd03755d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94037560, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc37563, SUC_lexicon_tags_37 },
    { 0xe237563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec37567, SUC_lexicon_tags_11 },
    { 0xf9e37568, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a37579, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xef43757b, SUC_lexicon_tags_39 },
    { 0x6183757c, SUC_lexicon_tags_12 },
    { 0x91a3757c, SUC_lexicon_tags_26 },
    { 0x3843757c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5dc37580, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x13e37582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60637589, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b63758f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34837592, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d637594, SUC_lexicon_tags_7 },
    { 0x29237595, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93637599, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f63759c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa043759e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x466375a0, SUC_lexicon_tags_56 },
    { 0xef4375a0, SUC_lexicon_tags_17 },
    { 0x25a375a2, SUC_lexicon_tags_55 },
    { 0x124375a3, SUC_lexicon_tags_7 },
    { 0x15e375a4, SUC_lexicon_tags_11 },
    { 0xce8375a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94375a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d8375ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a375ad, SUC_lexicon_tags_37 },
    { 0xe2e375ae, SUC_lexicon_tags_15 },
    { 0x94e375af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300375b3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a375b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec375ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce375bf, SUC_lexicon_tags_15 },
    { 0x36c375bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0375c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6375ce, SUC_lexicon_tags_32 },
    { 0x6fc375ce, SUC_lexicon_tags_25 },
    { 0xf38375d0, SUC_lexicon_tags_25 },
    { 0x924375ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf30375d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfdc375d5, SUC_lexicon_tags_8 },
    { 0x716375d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc375da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x612375dc, SUC_lexicon_tags_49 },
    { 0x5f6375dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcb0375df, SUC_lexicon_tags_45 },
    { 0xb76375df, SUC_lexicon_tags_29 },
    { 0x9d6375e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456375e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa375ea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98375ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c375f3, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xcee375f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe08375f7, SUC_lexicon_tags_39 },
    { 0x1e4375f8, SUC_lexicon_tags_19 },
    { 0xd1e375f9, SUC_lexicon_tags_23 },
    { 0xd8375f7, SUC_lexicon_tags_87 },
    { 0x98375fb, SUC_lexicon_tags_30 },
    { 0x620375fc, SUC_lexicon_tags_1 },
    { 0x2be375fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x528375ff, SUC_lexicon_tags_47 },
    { 0x682375ff, SUC_lexicon_tags_15 },
    { 0x348375ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30a37603, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb03760c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa543760e, SUC_lexicon_tags_7 },
    { 0x8163760f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf5a37611, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x25037614, SUC_lexicon_tags_37 },
    { 0xe0437615, SUC_lexicon_tags_9 },
    { 0x4f037616, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a37619, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b237629, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f83762f, SUC_lexicon_tags_23 },
    { 0x5e3762f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97837632, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3dc37634, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14237638, SUC_lexicon_tags_25 },
    { 0xf6037639, SUC_lexicon_tags_7 },
    { 0x9563763a, SUC_lexicon_tags_1 },
    { 0xa423763a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc943763d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e37643, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x13437646, SUC_lexicon_tags_6 },
    { 0xd7a37647, SUC_lexicon_tags_12 },
    { 0xce437648, SUC_lexicon_tags_16 },
    { 0xf8037648, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e3764d, SUC_lexicon_tags_68 },
    { 0xdd03764e, SUC_lexicon_tags_47 },
    { 0x1a23764d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe37652, SUC_lexicon_tags_14 },
    { 0xad037653, SUC_lexicon_tags_18 },
    { 0x75837654, SUC_lexicon_tags_11 },
    { 0x6437655, SUC_lexicon_tags_11 },
    { 0xad237656, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a3765a, SUC_lexicon_tags_0 },
    { 0xc303765b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d83765f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c37664, SUC_lexicon_tags_16 },
    { 0x1fc37665, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc3037667, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3766b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a3766e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x22837671, SUC_lexicon_tags_12 },
    { 0x63a37672, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a37682, SUC_lexicon_tags_7 },
    { 0xd4837682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de37686, SUC_lexicon_tags_17 },
    { 0x2be37687, SUC_lexicon_tags_2 },
    { 0x9c637688, SUC_lexicon_tags_26 },
    { 0x2a637689, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5783768f, SUC_lexicon_tags_20 },
    { 0x2f83768f, SUC_lexicon_tags_30 },
    { 0x5343768f, SUC_lexicon_tags_37 },
    { 0x48c37692, SUC_lexicon_tags_1 },
    { 0xf8a37692, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7037695, SUC_lexicon_tags_6 },
    { 0x76437696, SUC_lexicon_tags_8 },
    { 0x30037697, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4523769b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10e3769d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce03769f, SUC_lexicon_tags_6 },
    { 0xf363769f, SUC_lexicon_tags_15 },
    { 0xb2376a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xae8376a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x382376a5, SUC_lexicon_tags_2 },
    { 0x6fe376a5, SUC_lexicon_tags_69 },
    { 0x5f4376a6, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310376ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd72376af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6376b3, SUC_lexicon_tags_2 },
    { 0x792376b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a376bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16376c2, SUC_lexicon_tags_37 },
    { 0xb02376c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322376c7, SUC_lexicon_tags_1 },
    { 0xd06376c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116376d0, SUC_lexicon_tags_25 },
    { 0xdae376d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a376d4, SUC_lexicon_tags_1 },
    { 0xcc2376d4, SUC_lexicon_tags_23 },
    { 0xec376d4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182376e3, SUC_lexicon_tags_12 },
    { 0x9be376e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x42e376e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84376f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83a376f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x706376f4, SUC_lexicon_tags_11 },
    { 0x8c376f4, SUC_lexicon_tags_37 },
    { 0x394376f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82376fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc37702, SUC_lexicon_tags_1 },
    { 0xb1837703, SUC_lexicon_tags_2 },
    { 0x84837702, SUC_lexicon_tags_0 },
    { 0x2d037705, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc6437707, SUC_lexicon_tags_7 },
    { 0x97837708, SUC_lexicon_tags_60 },
    { 0x3a637709, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1483770b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d23770d, SUC_lexicon_tags_27 },
    { 0xb4a3770e, SUC_lexicon_tags_39 },
    { 0x6f43770f, SUC_lexicon_tags_30 },
    { 0x58037710, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaa037712, SUC_lexicon_tags_70 },
    { 0x30c37713, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf003771c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da37720, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36637722, SUC_lexicon_tags_25 },
    { 0xd6037723, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdac37725, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee37729, SUC_lexicon_tags_37 },
    { 0x31e3772a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x94c3772c, SUC_lexicon_tags_33 },
    { 0x9d83772c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c837730, SUC_lexicon_tags_25 },
    { 0x42a37731, SUC_lexicon_tags_49 },
    { 0xfd037732, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa783773b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62837740, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c37743, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xf2e37745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea37747, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd2a37749, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c37754, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e637757, SUC_lexicon_tags_26 },
    { 0x38037757, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcda3775a, SUC_lexicon_tags_7 },
    { 0x3f03775a, SUC_lexicon_tags_11 },
    { 0x2fc3775a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e83775f, SUC_lexicon_tags_91 },
    { 0x7ba37760, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb437766, SUC_lexicon_tags_16 },
    { 0x77037766, SUC_lexicon_tags_39 },
    { 0xa2037768, SUC_lexicon_tags_37 },
    { 0x9ae37769, SUC_lexicon_tags_12 },
    { 0x3c23776a, SUC_lexicon_tags_12 },
    { 0x2c837769, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8243776e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8237771, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x11437774, SUC_lexicon_tags_27 },
    { 0x1f037774, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a37779, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c3777e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f637785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5437787, SUC_lexicon_tags_27 },
    { 0x32237788, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d83778c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4643778f, SUC_lexicon_tags_63 },
    { 0x2e3778f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c37794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5437796, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc3779a, SUC_lexicon_tags_33 },
    { 0x7f83779b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1da3779d, SUC_lexicon_tags_23 },
    { 0x8d43779d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae377a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc377a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8377ae, SUC_lexicon_tags_0 },
    { 0x2fa377af, SUC_lexicon_tags_53 },
    { 0x66c377ae, SUC_lexicon_tags_17 },
    { 0xf54377b1, SUC_lexicon_tags_12 },
    { 0x726377b2, SUC_lexicon_tags_49 },
    { 0x5fc377b3, SUC_lexicon_tags_7 },
    { 0x128377b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaba377b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992377ba, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84377bd, SUC_lexicon_tags_10 },
    { 0x25e377be, SUC_lexicon_tags_1 },
    { 0x7ac377bf, SUC_lexicon_tags_1 },
    { 0x7fc377c0, SUC_lexicon_tags_1 },
    { 0xb4c377c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc2377c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x740377c6, SUC_lexicon_tags_47 },
    { 0x7b6377c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc377cb, SUC_lexicon_tags_2 },
    { 0x604377cc, SUC_lexicon_tags_46 },
    { 0x630377cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8377d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36377d4, SUC_lexicon_tags_47 },
    { 0xf32377d5, SUC_lexicon_tags_1 },
    { 0x200377d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47e377d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6e377da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a377e1, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e377e8, SUC_lexicon_tags_2 },
    { 0xf8e377e8, SUC_lexicon_tags_1 },
    { 0x41e377ea, SUC_lexicon_tags_11 },
    { 0xb0a377ea, SUC_lexicon_tags_38 },
    { 0x3fa377ec, SUC_lexicon_tags_15 },
    { 0xd82377ea, SUC_lexicon_tags_38 },
    { 0x9a8377ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6377f1, SUC_lexicon_tags_38 },
    { 0xed8377f2, SUC_lexicon_tags_56 },
    { 0x64a377f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42377f6, SUC_lexicon_tags_26 },
    { 0x10a377f7, SUC_lexicon_tags_7 },
    { 0x46c377f8, SUC_lexicon_tags_49 },
    { 0x824377f9, SUC_lexicon_tags_1 },
    { 0x72c377f8, SUC_lexicon_tags_11 },
    { 0x90c377fb, SUC_lexicon_tags_30 },
    { 0x20c377fb, SUC_lexicon_tags_6 },
    { 0x5bc377fa, SUC_lexicon_tags_15 },
    { 0x540377f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a37803, SUC_lexicon_tags_7 },
    { 0xd5a37803, SUC_lexicon_tags_49 },
    { 0x20237805, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5103780a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4743780d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c37814, SUC_lexicon_tags_17 },
    { 0x93037814, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3237819, SUC_lexicon_tags_25 },
    { 0x4b43781a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2e3781c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e37821, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa637825, SUC_lexicon_tags_68 },
    { 0x9fa37826, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61837828, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x123782a, SUC_lexicon_tags_0 },
    { 0x54e3782b, SUC_lexicon_tags_17 },
    { 0x10c3782b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1483782f, SUC_lexicon_tags_25 },
    { 0x64837830, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x54637832, SUC_lexicon_tags_45 },
    { 0x9e437833, SUC_lexicon_tags_23 },
    { 0x1a837834, SUC_lexicon_tags_10 },
    { 0xbce37834, SUC_lexicon_tags_37 },
    { 0xc6e37836, SUC_lexicon_tags_31 },
    { 0xc8a37836, SUC_lexicon_tags_11 },
    { 0x1f637836, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c83783f, SUC_lexicon_tags_2 },
    { 0x1ae37840, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc37846, SUC_lexicon_tags_49 },
    { 0x52037847, SUC_lexicon_tags_37 },
    { 0xd4037848, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x543784b, SUC_lexicon_tags_15 },
    { 0x65a3784b, SUC_lexicon_tags_1 },
    { 0x1723784d, SUC_lexicon_tags_38 },
    { 0x4143784e, SUC_lexicon_tags_25 },
    { 0x8463784f, SUC_lexicon_tags_31 },
    { 0xb9e3784d, SUC_lexicon_tags_1526 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c437857, SUC_lexicon_tags_26 },
    { 0xc2437858, SUC_lexicon_tags_1 },
    { 0x38837859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e3785c, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c03785f, SUC_lexicon_tags_37 },
    { 0xa783785f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd837868, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a3786b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70237871, SUC_lexicon_tags_9 },
    { 0xa1e37872, SUC_lexicon_tags_27 },
    { 0xe7437873, SUC_lexicon_tags_17 },
    { 0x72637874, SUC_lexicon_tags_8 },
    { 0x6d437875, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4403787a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c3787e, SUC_lexicon_tags_33 },
    { 0xb883787e, SUC_lexicon_tags_17 },
    { 0xbd637880, SUC_lexicon_tags_47 },
    { 0xbe637880, SUC_lexicon_tags_43 },
    { 0xc0637881, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca37887, SUC_lexicon_tags_43 },
    { 0x6f037887, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x723788a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x27c3788c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae37890, SUC_lexicon_tags_29 },
    { 0xcba37891, SUC_lexicon_tags_17 },
    { 0x27c37891, SUC_lexicon_tags_12 },
    { 0x75037893, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1437896, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a637899, SUC_lexicon_tags_17 },
    { 0x3803789a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe3789d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52378a0, SUC_lexicon_tags_25 },
    { 0x560378a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2378a8, SUC_lexicon_tags_21 },
    { 0x47c378a8, SUC_lexicon_tags_33 },
    { 0xb24378aa, SUC_lexicon_tags_27 },
    { 0xc3e378a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2378b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4378b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c378c3, SUC_lexicon_tags_31 },
    { 0x694378c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0378c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb2a378cb, SUC_lexicon_tags_1 },
    { 0x5f0378cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbba378ce, SUC_lexicon_tags_17 },
    { 0xde0378cf, SUC_lexicon_tags_37 },
    { 0xf8c378d0, SUC_lexicon_tags_31 },
    { 0xd1a378d1, SUC_lexicon_tags_9 },
    { 0x690378d1, SUC_lexicon_tags_31 },
    { 0xaf2378d1, SUC_lexicon_tags_1 },
    { 0xdd0378d4, SUC_lexicon_tags_12 },
    { 0x6da378d4, SUC_lexicon_tags_43 },
    { 0xee6378d4, SUC_lexicon_tags_12 },
    { 0x52e378d7, SUC_lexicon_tags_25 },
    { 0x4ee378d5, SUC_lexicon_tags_47 },
    { 0x710378d9, SUC_lexicon_tags_17 },
    { 0x38e378d9, SUC_lexicon_tags_31 },
    { 0x24e378d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e378e2, SUC_lexicon_tags_2 },
    { 0xb2a378e3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468378e8, SUC_lexicon_tags_7 },
    { 0x786378e8, SUC_lexicon_tags_63 },
    { 0xade378e9, SUC_lexicon_tags_2 },
    { 0x886378ea, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8da378ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340378f1, SUC_lexicon_tags_37 },
    { 0xd90378f1, SUC_lexicon_tags_19 },
    { 0xe42378f2, SUC_lexicon_tags_30 },
    { 0xa6378f4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716378fb, SUC_lexicon_tags_11 },
    { 0xe92378fc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd52378fe, SUC_lexicon_tags_165 },
    { 0x9d8378ff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c37903, SUC_lexicon_tags_26 },
    { 0x9c37904, SUC_lexicon_tags_2 },
    { 0x29037905, SUC_lexicon_tags_30 },
    { 0x5237906, SUC_lexicon_tags_29 },
    { 0xf637907, SUC_lexicon_tags_25 },
    { 0x22237905, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57a3790a, SUC_lexicon_tags_38 },
    { 0x58a3790a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6237917, SUC_lexicon_tags_19 },
    { 0x36c37918, SUC_lexicon_tags_2 },
    { 0xc2437919, SUC_lexicon_tags_30 },
    { 0xe383791a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88a3791c, SUC_lexicon_tags_49 },
    { 0xe23791d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8e43791f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e37924, SUC_lexicon_tags_25 },
    { 0x32237925, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac637929, SUC_lexicon_tags_25 },
    { 0xad437929, SUC_lexicon_tags_65 },
    { 0xa3437929, SUC_lexicon_tags_54 },
    { 0x91c3792c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d63792e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76837932, SUC_lexicon_tags_47 },
    { 0xcf237932, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5c837935, SUC_lexicon_tags_7 },
    { 0x2aa37936, SUC_lexicon_tags_22 },
    { 0x80c37936, SUC_lexicon_tags_28 },
    { 0x36c37938, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea43793d, SUC_lexicon_tags_37 },
    { 0xff83793d, SUC_lexicon_tags_12 },
    { 0xbbe3793f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63837946, SUC_lexicon_tags_37 },
    { 0x9037947, SUC_lexicon_tags_23 },
    { 0x8fe37946, SUC_lexicon_tags_47 },
    { 0x18c37949, SUC_lexicon_tags_61 },
    { 0x5be37949, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65437951, SUC_lexicon_tags_7 },
    { 0xcb837951, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1437954, SUC_lexicon_tags_38 },
    { 0x73a37954, SUC_lexicon_tags_7 },
    { 0x1ac37956, SUC_lexicon_tags_17 },
    { 0x2c637955, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd83795a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2263795f, SUC_lexicon_tags_19 },
    { 0xd9a3795f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2037962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff237964, SUC_lexicon_tags_7 },
    { 0xbda37965, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81637968, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a3796b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa37973, SUC_lexicon_tags_7 },
    { 0x63637974, SUC_lexicon_tags_26 },
    { 0xd4237975, SUC_lexicon_tags_27 },
    { 0x81837974, SUC_lexicon_tags_7 },
    { 0x94837973, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e637979, SUC_lexicon_tags_17 },
    { 0x2203797a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b83797e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x74237980, SUC_lexicon_tags_9 },
    { 0xa837981, SUC_lexicon_tags_33 },
    { 0x86037982, SUC_lexicon_tags_15 },
    { 0xda37983, SUC_lexicon_tags_28 },
    { 0x3f237984, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x18437987, SUC_lexicon_tags_14 },
    { 0xd9e37988, SUC_lexicon_tags_7 },
    { 0xfb237988, SUC_lexicon_tags_49 },
    { 0xa923798a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b23798d, SUC_lexicon_tags_6 },
    { 0xc5e3798e, SUC_lexicon_tags_2 },
    { 0x4143798f, SUC_lexicon_tags_47 },
    { 0xef23798f, SUC_lexicon_tags_23 },
    { 0x38e37991, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e237994, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6c037996, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d037999, SUC_lexicon_tags_87 },
    { 0x4323799a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63799d, SUC_lexicon_tags_45 },
    { 0xc9c3799e, SUC_lexicon_tags_65 },
    { 0xbb63799f, SUC_lexicon_tags_29 },
    { 0x940379a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf02379a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x118379a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de379a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e379b1, SUC_lexicon_tags_7 },
    { 0xae0379b2, SUC_lexicon_tags_29 },
    { 0x426379b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e0379b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x190379b7, SUC_lexicon_tags_8 },
    { 0x78379b8, SUC_lexicon_tags_27 },
    { 0xc0a379b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf58379bb, SUC_lexicon_tags_3 },
    { 0xe60379bc, SUC_lexicon_tags_56 },
    { 0x6b8379bc, SUC_lexicon_tags_43 },
    { 0x700379be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb4379c0, SUC_lexicon_tags_23 },
    { 0xba2379c0, SUC_lexicon_tags_1 },
    { 0x6f4379c1, SUC_lexicon_tags_1 },
    { 0x6c0379c3, SUC_lexicon_tags_23 },
    { 0x244379c4, SUC_lexicon_tags_25 },
    { 0xb92379c5, SUC_lexicon_tags_19 },
    { 0x24c379c6, SUC_lexicon_tags_37 },
    { 0x822379c2, SUC_lexicon_tags_1 },
    { 0x8ae379c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba379cb, SUC_lexicon_tags_2 },
    { 0xcd6379cb, SUC_lexicon_tags_14 },
    { 0x388379cd, SUC_lexicon_tags_16 },
    { 0xf84379ce, SUC_lexicon_tags_38 },
    { 0x6fc379cd, SUC_lexicon_tags_12 },
    { 0xee4379d0, SUC_lexicon_tags_25 },
    { 0x7bc379d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e379d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca379d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xba0379db, SUC_lexicon_tags_11 },
    { 0x62c379dc, SUC_lexicon_tags_32 },
    { 0xbb8379db, SUC_lexicon_tags_63 },
    { 0x726379de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4d4379e0, SUC_lexicon_tags_14 },
    { 0x35c379e0, SUC_lexicon_tags_37 },
    { 0x10e379e2, SUC_lexicon_tags_146 },
    { 0xa8e379e3, SUC_lexicon_tags_13 },
    { 0xd04379e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x836379e7, SUC_lexicon_tags_11 },
    { 0x400379e7, SUC_lexicon_tags_35 },
    { 0x94a379e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd96379eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc36379ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a379f1, SUC_lexicon_tags_27 },
    { 0x904379f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fa379f4, SUC_lexicon_tags_11 },
    { 0xd84379f5, SUC_lexicon_tags_12 },
    { 0x364379f6, SUC_lexicon_tags_60 },
    { 0x99a379f5, SUC_lexicon_tags_15 },
    { 0xe00379f8, SUC_lexicon_tags_17 },
    { 0xa44379f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2379fc, SUC_lexicon_tags_25 },
    { 0xa2c379fc, SUC_lexicon_tags_55 },
    { 0xd14379fe, SUC_lexicon_tags_12 },
    { 0xfee379ff, SUC_lexicon_tags_714 },
    { 0xa3c37a00, SUC_lexicon_tags_13 },
    { 0x582379ff, SUC_lexicon_tags_10 },
    { 0x95a37a02, SUC_lexicon_tags_1 },
    { 0x4e6379ff, SUC_lexicon_tags_37 },
    { 0xa9a379fe, SUC_lexicon_tags_25 },
    { 0x77837a05, SUC_lexicon_tags_47 },
    { 0xfee37a06, SUC_lexicon_tags_33 },
    { 0x6ac37a07, SUC_lexicon_tags_32 },
    { 0x95a37a08, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8237a0a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xac037a0c, SUC_lexicon_tags_82 },
    { 0x99c37a0c, SUC_lexicon_tags_17 },
    { 0x94c37a0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e37a18, SUC_lexicon_tags_15 },
    { 0x67e37a18, SUC_lexicon_tags_178 },
    { 0x14237a1a, SUC_lexicon_tags_11 },
    { 0x96e37a1a, SUC_lexicon_tags_45 },
    { 0x80237a1c, SUC_lexicon_tags_11 },
    { 0xcca37a1d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbe237a1f, SUC_lexicon_tags_27 },
    { 0xff837a20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2837a24, SUC_lexicon_tags_38 },
    { 0xc3037a25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77837a28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73837a2a, SUC_lexicon_tags_63 },
    { 0x70837a2b, SUC_lexicon_tags_8 },
    { 0xc0c37a2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45837a30, SUC_lexicon_tags_25 },
    { 0x14e37a31, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1e437a33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c237a38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xff037a3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac37a3f, SUC_lexicon_tags_35 },
    { 0x6d237a40, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca37a44, SUC_lexicon_tags_31 },
    { 0x44c37a45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfae37a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c37a4b, SUC_lexicon_tags_5 },
    { 0x69237a4c, SUC_lexicon_tags_35 },
    { 0x2fe37a4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a37a50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92837a58, SUC_lexicon_tags_2 },
    { 0x31437a59, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd3837a5b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d637a60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb037a62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98837a6a, SUC_lexicon_tags_1 },
    { 0x6d037a6b, SUC_lexicon_tags_15 },
    { 0xb9437a6b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7637a74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea37a77, SUC_lexicon_tags_30 },
    { 0x10237a78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1437a80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6237a83, SUC_lexicon_tags_9 },
    { 0xbcc37a83, SUC_lexicon_tags_33 },
    { 0x2c237a84, SUC_lexicon_tags_17 },
    { 0xca037a86, SUC_lexicon_tags_47 },
    { 0x2e037a87, SUC_lexicon_tags_11 },
    { 0x91437a88, SUC_lexicon_tags_60 },
    { 0xbec37a89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b437a8d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e37a91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac37a94, SUC_lexicon_tags_2 },
    { 0x4ea37a95, SUC_lexicon_tags_25 },
    { 0x58437a94, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa9a37a98, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfc637a9a, SUC_lexicon_tags_31 },
    { 0xcb437a9a, SUC_lexicon_tags_37 },
    { 0x30037a9a, SUC_lexicon_tags_37 },
    { 0xf7437a9c, SUC_lexicon_tags_10 },
    { 0x2d637a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf637aa0, SUC_lexicon_tags_14 },
    { 0x87c37aa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeea37aa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbec37aa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f237aa7, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62437aac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2037aae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17a37ab0, SUC_lexicon_tags_21 },
    { 0xca837ab1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f437ab3, SUC_lexicon_tags_4 },
    { 0xc8c37ab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5237ab6, SUC_lexicon_tags_27 },
    { 0x15637ab7, SUC_lexicon_tags_37 },
    { 0xdaa37ab8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a37abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa037abd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96637ac0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc37ac3, SUC_lexicon_tags_9 },
    { 0xe8a37ac3, SUC_lexicon_tags_14 },
    { 0x8d037ac4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21837acd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a437ad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x237ad3, SUC_lexicon_tags_11 },
    { 0x7dc37ad4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd437adb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d437ae4, SUC_lexicon_tags_25 },
    { 0xbca37ae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaee37ae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91837aed, SUC_lexicon_tags_25 },
    { 0xcc437aee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2237af1, SUC_lexicon_tags_68 },
    { 0x9a37af2, SUC_lexicon_tags_63 },
    { 0xfa837af2, SUC_lexicon_tags_9 },
    { 0x1a837af4, SUC_lexicon_tags_12 },
    { 0xb7237af5, SUC_lexicon_tags_47 },
    { 0xfd237af6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c37af9, SUC_lexicon_tags_7 },
    { 0x6f037afa, SUC_lexicon_tags_11 },
    { 0x82837afa, SUC_lexicon_tags_1 },
    { 0x1be37afc, SUC_lexicon_tags_2 },
    { 0x46237af9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd6437aff, SUC_lexicon_tags_17 },
    { 0xe4437aff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90237b04, SUC_lexicon_tags_43 },
    { 0x40637b05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77037b08, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e37b0f, SUC_lexicon_tags_23 },
    { 0x5a637b10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7de37b12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12237b14, SUC_lexicon_tags_63 },
    { 0x7ae37b14, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x60e37b17, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae37b1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c37b21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6637b23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x26637b26, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9637b29, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b037b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be37b33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7ac37b35, SUC_lexicon_tags_23 },
    { 0x9637b35, SUC_lexicon_tags_25 },
    { 0x12c37b36, SUC_lexicon_tags_15 },
    { 0x9fe37b38, SUC_lexicon_tags_60 },
    { 0xf3e37b39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc037b3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e37b42, SUC_lexicon_tags_48 },
    { 0x47237b43, SUC_lexicon_tags_38 },
    { 0x17037b44, SUC_lexicon_tags_42 },
    { 0x4bc37b43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee37b49, SUC_lexicon_tags_33 },
    { 0x84037b49, SUC_lexicon_tags_6 },
    { 0xb8437b4b, SUC_lexicon_tags_11 },
    { 0x5cc37b4c, SUC_lexicon_tags_27 },
    { 0x7ea37b49, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce37b50, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9437b54, SUC_lexicon_tags_45 },
    { 0x5d837b54, SUC_lexicon_tags_63 },
    { 0xcd037b56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be37b59, SUC_lexicon_tags_9 },
    { 0xce237b5a, SUC_lexicon_tags_12 },
    { 0x4b237b5a, SUC_lexicon_tags_1 },
    { 0x18c37b5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e37b60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b437b64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a37b67, SUC_lexicon_tags_9 },
    { 0x55837b67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e37b6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a37b72, SUC_lexicon_tags_8 },
    { 0xc3237b73, SUC_lexicon_tags_11 },
    { 0x94037b74, SUC_lexicon_tags_12 },
    { 0x84837b75, SUC_lexicon_tags_48 },
    { 0x10a37b76, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a637b7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d437b85, SUC_lexicon_tags_27 },
    { 0xc2837b85, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xd4a37b88, SUC_lexicon_tags_48 },
    { 0x4e237b89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a437b8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a37b8e, SUC_lexicon_tags_25 },
    { 0xd1a37b8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5837b92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xffa37b94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9237b98, SUC_lexicon_tags_1 },
    { 0x35c37b98, SUC_lexicon_tags_12 },
    { 0xf8637b9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb237ba3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3437ba8, SUC_lexicon_tags_0 },
    { 0x2b237ba9, SUC_lexicon_tags_4 },
    { 0xd8c37ba9, SUC_lexicon_tags_7 },
    { 0xc9a37bab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97c37bad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c37bb2, SUC_lexicon_tags_37 },
    { 0x7ee37bb3, SUC_lexicon_tags_1 },
    { 0xe6037bb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36037bc1, SUC_lexicon_tags_35 },
    { 0xd2a37bc1, SUC_lexicon_tags_9 },
    { 0x6a837bc3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x27237bc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc837bc9, SUC_lexicon_tags_19 },
    { 0xf7e37bc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce37bcf, SUC_lexicon_tags_12 },
    { 0xce837bd0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e37bda, SUC_lexicon_tags_15 },
    { 0x44437bda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ba37bdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0637bdf, SUC_lexicon_tags_49 },
    { 0xe4e37be0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafe37be2, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d037be5, SUC_lexicon_tags_18 },
    { 0x94837be5, SUC_lexicon_tags_38 },
    { 0xc5637be6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f837be9, SUC_lexicon_tags_41 },
    { 0x42837bea, SUC_lexicon_tags_79 },
    { 0x65237bea, SUC_lexicon_tags_36 },
    { 0xb5a37beb, SUC_lexicon_tags_11 },
    { 0x95437bea, SUC_lexicon_tags_12 },
    { 0x86437bed, SUC_lexicon_tags_46 },
    { 0x91237bef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfce37bf1, SUC_lexicon_tags_31 },
    { 0x8f437bf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e37bf6, SUC_lexicon_tags_34 },
    { 0xc5c37bf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28e37bf9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d637bfd, SUC_lexicon_tags_30 },
    { 0xcb837bfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e437c01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c37c05, SUC_lexicon_tags_33 },
    { 0x9237c05, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c037c11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec37c14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda237c1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40037c1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea37c23, SUC_lexicon_tags_25 },
    { 0xe1837c24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a37c27, SUC_lexicon_tags_22 },
    { 0x7cc37c27, SUC_lexicon_tags_48 },
    { 0x43a37c29, SUC_lexicon_tags_45 },
    { 0x30637c2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85837c2c, SUC_lexicon_tags_19 },
    { 0xae037c2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a037c32, SUC_lexicon_tags_23 },
    { 0x56037c33, SUC_lexicon_tags_28 },
    { 0x2e637c34, SUC_lexicon_tags_38 },
    { 0xd8837c35, SUC_lexicon_tags_23 },
    { 0x1fc37c33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x38437c38, SUC_lexicon_tags_68 },
    { 0xf8e37c39, SUC_lexicon_tags_1 },
    { 0xf4437c3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b037c3d, SUC_lexicon_tags_77 },
    { 0x90c37c3e, SUC_lexicon_tags_0 },
    { 0xbe437c3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc37c42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30637c47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a37c4c, SUC_lexicon_tags_15 },
    { 0xd3437c4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b637c4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34037c54, SUC_lexicon_tags_15 },
    { 0x7d237c54, SUC_lexicon_tags_7 },
    { 0x47237c56, SUC_lexicon_tags_1 },
    { 0x96a37c56, SUC_lexicon_tags_23 },
    { 0x5d237c56, SUC_lexicon_tags_37 },
    { 0x9b637c59, SUC_lexicon_tags_11 },
    { 0xc5237c5a, SUC_lexicon_tags_1 },
    { 0x23e37c5b, SUC_lexicon_tags_29 },
    { 0xd8037c59, SUC_lexicon_tags_12 },
    { 0x63437c54, SUC_lexicon_tags_26 },
    { 0x5b037c5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x17237c61, SUC_lexicon_tags_25 },
    { 0x84637c61, SUC_lexicon_tags_26 },
    { 0x32437c61, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e37c67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c37c6a, SUC_lexicon_tags_18 },
    { 0xa2437c6b, SUC_lexicon_tags_17 },
    { 0x1a637c6a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd0437c6e, SUC_lexicon_tags_30 },
    { 0xe7a37c6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c37c72, SUC_lexicon_tags_62 },
    { 0xc1437c73, SUC_lexicon_tags_17 },
    { 0x39c37c74, SUC_lexicon_tags_25 },
    { 0x99037c74, SUC_lexicon_tags_15 },
    { 0xb4437c76, SUC_lexicon_tags_29 },
    { 0x93237c76, SUC_lexicon_tags_47 },
    { 0x32e37c78, SUC_lexicon_tags_2 },
    { 0x7ca37c78, SUC_lexicon_tags_70 },
    { 0x65237c7a, SUC_lexicon_tags_12 },
    { 0x7ce37c77, SUC_lexicon_tags_31 },
    { 0x26437c78, SUC_lexicon_tags_14 },
    { 0xa4637c72, SUC_lexicon_tags_60 },
    { 0x80c37c7e, SUC_lexicon_tags_61 },
    { 0xd5437c76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb5037c81, SUC_lexicon_tags_47 },
    { 0x49e37c82, SUC_lexicon_tags_25 },
    { 0x7dc37c82, SUC_lexicon_tags_28 },
    { 0x4d837c84, SUC_lexicon_tags_108 },
    { 0x84e37c85, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec37c8c, SUC_lexicon_tags_47 },
    { 0x84437c8c, SUC_lexicon_tags_8 },
    { 0x41a37c8e, SUC_lexicon_tags_37 },
    { 0xcb037c8f, SUC_lexicon_tags_45 },
    { 0x3f837c8f, SUC_lexicon_tags_47 },
    { 0xec837c91, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9837c96, SUC_lexicon_tags_37 },
    { 0xca437c97, SUC_lexicon_tags_15 },
    { 0xa37c98, SUC_lexicon_tags_17 },
    { 0x85a37c98, SUC_lexicon_tags_37 },
    { 0x6ec37c9a, SUC_lexicon_tags_14 },
    { 0xfdc37c9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc37ca0, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d837ca5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd637ca8, SUC_lexicon_tags_1 },
    { 0x26e37ca9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5a37cab, SUC_lexicon_tags_38 },
    { 0xe3637cac, SUC_lexicon_tags_669 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c37caf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4fc37cb1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5ca37cb3, SUC_lexicon_tags_11 },
    { 0x6b237cb4, SUC_lexicon_tags_17 },
    { 0x17437cb5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a237cb8, SUC_lexicon_tags_19 },
    { 0xc4a37cb8, SUC_lexicon_tags_25 },
    { 0x5b437cba, SUC_lexicon_tags_17 },
    { 0xeb037cba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b837cc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78637cc3, SUC_lexicon_tags_8 },
    { 0x49237cc4, SUC_lexicon_tags_34 },
    { 0x9cc37cc3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9ca37cc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55437cc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a37cd0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e37cd5, SUC_lexicon_tags_11 },
    { 0x58c37cd6, SUC_lexicon_tags_165 },
    { 0xf6e37cd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a37cdd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0237ce5, SUC_lexicon_tags_17 },
    { 0xa7c37ce6, SUC_lexicon_tags_46 },
    { 0x28c37ce7, SUC_lexicon_tags_49 },
    { 0xf2437ce8, SUC_lexicon_tags_55 },
    { 0xde637ce9, SUC_lexicon_tags_14 },
    { 0x59c37ce8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3a37cec, SUC_lexicon_tags_87 },
    { 0x95a37ced, SUC_lexicon_tags_17 },
    { 0x11437ced, SUC_lexicon_tags_15 },
    { 0x87037ced, SUC_lexicon_tags_82 },
    { 0x8e037cf0, SUC_lexicon_tags_15 },
    { 0x37037cf1, SUC_lexicon_tags_45 },
    { 0xf4837cf2, SUC_lexicon_tags_28 },
    { 0xfec37cf1, SUC_lexicon_tags_61 },
    { 0xe3e37cf4, SUC_lexicon_tags_45 },
    { 0xd1637cf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17837cfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe2a37cfd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a637cff, SUC_lexicon_tags_39 },
    { 0xb1e37d00, SUC_lexicon_tags_2 },
    { 0x69a37d01, SUC_lexicon_tags_31 },
    { 0x3da37d01, SUC_lexicon_tags_19 },
    { 0xece37d03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3637d06, SUC_lexicon_tags_87 },
    { 0x44037d07, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x87c37d09, SUC_lexicon_tags_136 },
    { 0xd4437d0a, SUC_lexicon_tags_47 },
    { 0xd6e37d0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e37d10, SUC_lexicon_tags_25 },
    { 0x2e037d10, SUC_lexicon_tags_15 },
    { 0xe1e37d12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba637d17, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x5e037d19, SUC_lexicon_tags_1 },
    { 0xc8e37d1a, SUC_lexicon_tags_15 },
    { 0x44037d1a, SUC_lexicon_tags_15 },
    { 0x1437d1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a37d1f, SUC_lexicon_tags_39 },
    { 0xea437d1f, SUC_lexicon_tags_12 },
    { 0xa6037d21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc37d23, SUC_lexicon_tags_16 },
    { 0xf9237d24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1a437d26, SUC_lexicon_tags_27 },
    { 0xe3e37d27, SUC_lexicon_tags_7 },
    { 0xd9837d28, SUC_lexicon_tags_6 },
    { 0xffe37d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e37d2b, SUC_lexicon_tags_68 },
    { 0x5837d2c, SUC_lexicon_tags_12 },
    { 0x73a37d2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c837d34, SUC_lexicon_tags_1 },
    { 0xb6c37d35, SUC_lexicon_tags_87 },
    { 0xdd437d35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd837d38, SUC_lexicon_tags_15 },
    { 0x1fc37d39, SUC_lexicon_tags_14 },
    { 0xa237d39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7037d42, SUC_lexicon_tags_42 },
    { 0x91e37d43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2437d46, SUC_lexicon_tags_15 },
    { 0xe6037d46, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa37d4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec37d51, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c37d57, SUC_lexicon_tags_11 },
    { 0x10437d58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22037d5a, SUC_lexicon_tags_29 },
    { 0x6e237d5a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae37d5e, SUC_lexicon_tags_0 },
    { 0x5e637d5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2237d63, SUC_lexicon_tags_15 },
    { 0xe2e37d64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc37d6b, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc037d6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1837d75, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98637d79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x71037d7c, SUC_lexicon_tags_12 },
    { 0x83e37d7d, SUC_lexicon_tags_30 },
    { 0x78437d7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea37d81, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f637d83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b437d8a, SUC_lexicon_tags_25 },
    { 0xa8a37d8b, SUC_lexicon_tags_60 },
    { 0x38637d8c, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57837d91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90037d93, SUC_lexicon_tags_13 },
    { 0x67a37d94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf837d96, SUC_lexicon_tags_81 },
    { 0x52237d96, SUC_lexicon_tags_31 },
    { 0xe3837d98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x77e37d9a, SUC_lexicon_tags_0 },
    { 0xbc437d9b, SUC_lexicon_tags_15 },
    { 0x9e037d9c, SUC_lexicon_tags_27 },
    { 0xb237d9c, SUC_lexicon_tags_191 },
    { 0x24437d9c, SUC_lexicon_tags_6 },
    { 0x9e437d9c, SUC_lexicon_tags_29 },
    { 0xd237d9d, SUC_lexicon_tags_15 },
    { 0x2a437da1, SUC_lexicon_tags_37 },
    { 0x3e837da1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x37237da4, SUC_lexicon_tags_9 },
    { 0xf7837da5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95c37da7, SUC_lexicon_tags_1 },
    { 0xd9c37da7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x19637daa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa8c37dac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d837dae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb0037db0, SUC_lexicon_tags_66 },
    { 0x7ec37db0, SUC_lexicon_tags_33 },
    { 0x5a37db0, SUC_lexicon_tags_37 },
    { 0xc4e37db3, SUC_lexicon_tags_17 },
    { 0x68237db0, SUC_lexicon_tags_15 },
    { 0x34e37db3, SUC_lexicon_tags_28 },
    { 0xb5837db6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac37dbb, SUC_lexicon_tags_46 },
    { 0xb5437dbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbaa37dbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b837dc2, SUC_lexicon_tags_21 },
    { 0xbd237dc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c037dc8, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xe8e37dca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a637dce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7837dd7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x85837dda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb837ddd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a37de1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe237de4, SUC_lexicon_tags_7 },
    { 0x70a37de5, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35637dea, SUC_lexicon_tags_19 },
    { 0x1a837dea, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e37df1, SUC_lexicon_tags_49 },
    { 0x6f237df2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb237df6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x89037df9, SUC_lexicon_tags_25 },
    { 0x90a37dfa, SUC_lexicon_tags_63 },
    { 0x66037dfb, SUC_lexicon_tags_17 },
    { 0xc8a37dfc, SUC_lexicon_tags_20 },
    { 0xbc037dfd, SUC_lexicon_tags_47 },
    { 0xaba37dfd, SUC_lexicon_tags_2 },
    { 0x9837dfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab037e02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a37e05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda37e08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53237e10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa837e16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4237e1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95837e1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x82a37e21, SUC_lexicon_tags_37 },
    { 0xb9e37e22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a237e24, SUC_lexicon_tags_31 },
    { 0xe5e37e25, SUC_lexicon_tags_1 },
    { 0xc037e25, SUC_lexicon_tags_47 },
    { 0x13637e25, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3f837e29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9437e2c, SUC_lexicon_tags_85 },
    { 0xc8237e2c, SUC_lexicon_tags_57 },
    { 0x76437e2e, SUC_lexicon_tags_47 },
    { 0xd6437e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93637e32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5c37e34, SUC_lexicon_tags_16 },
    { 0x93a37e35, SUC_lexicon_tags_37 },
    { 0x6b437e36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde637e38, SUC_lexicon_tags_47 },
    { 0x95e37e39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec37e3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a37e41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60037e47, SUC_lexicon_tags_7 },
    { 0x2ca37e47, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf037e4b, SUC_lexicon_tags_63 },
    { 0x4c37e4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a237e4e, SUC_lexicon_tags_1 },
    { 0x4fe37e4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3237e52, SUC_lexicon_tags_21 },
    { 0x4ba37e53, SUC_lexicon_tags_37 },
    { 0x5f237e53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc437e58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec37e5d, SUC_lexicon_tags_17 },
    { 0xcea37e5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec437e61, SUC_lexicon_tags_17 },
    { 0x39c37e61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fe37e64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68437e66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c37e69, SUC_lexicon_tags_1 },
    { 0xde037e6a, SUC_lexicon_tags_17 },
    { 0x8dc37e6b, SUC_lexicon_tags_15 },
    { 0xec437e6c, SUC_lexicon_tags_39 },
    { 0x38037e6a, SUC_lexicon_tags_27 },
    { 0xa2237e6e, SUC_lexicon_tags_11 },
    { 0xc2837e6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7437e75, SUC_lexicon_tags_55 },
    { 0x7c37e76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb437e7d, SUC_lexicon_tags_11 },
    { 0x39637e7d, SUC_lexicon_tags_7 },
    { 0xbfa37e7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa37e82, SUC_lexicon_tags_27 },
    { 0x92437e83, SUC_lexicon_tags_2 },
    { 0x7ae37e84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xedc37e86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba37e8e, SUC_lexicon_tags_2 },
    { 0x3c837e8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23037e91, SUC_lexicon_tags_1 },
    { 0x5d437e92, SUC_lexicon_tags_9 },
    { 0x78037e91, SUC_lexicon_tags_31 },
    { 0xc8e37e94, SUC_lexicon_tags_15 },
    { 0x62a37e92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30837e9a, SUC_lexicon_tags_1 },
    { 0x98e37e9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f037e9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f037ea2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e37ea6, SUC_lexicon_tags_48 },
    { 0x16c37ea7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd237ea9, SUC_lexicon_tags_19 },
    { 0xe8037eaa, SUC_lexicon_tags_11 },
    { 0xbda37eab, SUC_lexicon_tags_37 },
    { 0x93a37eac, SUC_lexicon_tags_2 },
    { 0xe8c37eab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x76837eaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x65837eb2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x19037eb5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf1037eb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e37ec0, SUC_lexicon_tags_43 },
    { 0xae637ec1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa6237ec3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e37ec7, SUC_lexicon_tags_23 },
    { 0x38c37ec7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55037eca, SUC_lexicon_tags_178 },
    { 0x50e37ecb, SUC_lexicon_tags_46 },
    { 0x79837ecb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a37ecf, SUC_lexicon_tags_11 },
    { 0x42437ecf, SUC_lexicon_tags_38 },
    { 0x5d237ed1, SUC_lexicon_tags_87 },
    { 0xd0c37ed2, SUC_lexicon_tags_1 },
    { 0x54037ed3, SUC_lexicon_tags_8 },
    { 0xd8e37ed4, SUC_lexicon_tags_15 },
    { 0x1ce37ed5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26237ed8, SUC_lexicon_tags_1 },
    { 0x65437ed8, SUC_lexicon_tags_17 },
    { 0x68837eda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a37edf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b837ee2, SUC_lexicon_tags_17 },
    { 0xb4837ee3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d237ee5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37eeb, SUC_lexicon_tags_21 },
    { 0x95237eec, SUC_lexicon_tags_14 },
    { 0x4ac37eeb, SUC_lexicon_tags_11 },
    { 0x1aa37eee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e37ef2, SUC_lexicon_tags_17 },
    { 0xe8637ef3, SUC_lexicon_tags_4 },
    { 0x18237ef4, SUC_lexicon_tags_0 },
    { 0x15037ef4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x21c37ef7, SUC_lexicon_tags_56 },
    { 0xf9237ef8, SUC_lexicon_tags_7 },
    { 0x1c037ef7, SUC_lexicon_tags_37 },
    { 0xc7c37efa, SUC_lexicon_tags_2 },
    { 0x75e37efa, SUC_lexicon_tags_45 },
    { 0x9dc37efc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb637eff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a37f0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6d237f0f, SUC_lexicon_tags_82 },
    { 0xf5237f10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb6437f12, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe37f15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc437f17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27437f1a, SUC_lexicon_tags_10 },
    { 0xd5c37f1b, SUC_lexicon_tags_82 },
    { 0x89237f1c, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x3037f1e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c837f21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a837f29, SUC_lexicon_tags_49 },
    { 0xaf437f29, SUC_lexicon_tags_47 },
    { 0x2237f29, SUC_lexicon_tags_18 },
    { 0xde437f2c, SUC_lexicon_tags_15 },
    { 0xe2837f2d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf0a37f2f, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71037f35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf437f37, SUC_lexicon_tags_25 },
    { 0x4a837f38, SUC_lexicon_tags_63 },
    { 0xe9e37f37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82437f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3637f41, SUC_lexicon_tags_41 },
    { 0xd1437f42, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdc637f44, SUC_lexicon_tags_25 },
    { 0x4437f45, SUC_lexicon_tags_17 },
    { 0x41037f45, SUC_lexicon_tags_2 },
    { 0x53237f44, SUC_lexicon_tags_7 },
    { 0xf8237f48, SUC_lexicon_tags_44 },
    { 0xd7c37f49, SUC_lexicon_tags_15 },
    { 0x89837f4a, SUC_lexicon_tags_10 },
    { 0x53237f46, SUC_lexicon_tags_47 },
    { 0x71837f47, SUC_lexicon_tags_25 },
    { 0x83037f48, SUC_lexicon_tags_8 },
    { 0x95c37f4e, SUC_lexicon_tags_7 },
    { 0x84237f4f, SUC_lexicon_tags_33 },
    { 0x69437f4e, SUC_lexicon_tags_33 },
    { 0xf2437f49, SUC_lexicon_tags_15 },
    { 0xb8c37f49, SUC_lexicon_tags_2 },
    { 0xb6c37f53, SUC_lexicon_tags_23 },
    { 0xe1837f54, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d237f56, SUC_lexicon_tags_11 },
    { 0x7f437f57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba37f5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa37f5d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41837f61, SUC_lexicon_tags_25 },
    { 0xf6637f62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d037f66, SUC_lexicon_tags_66 },
    { 0x87437f67, SUC_lexicon_tags_65 },
    { 0xbbe37f68, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x4f837f6a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x42e37f6c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a37f6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a637f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93a37f75, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22437f79, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e37f7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x34237f7f, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0x9a37f81, SUC_lexicon_tags_38 },
    { 0xbc437f81, SUC_lexicon_tags_25 },
    { 0x54637f83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2037f89, SUC_lexicon_tags_17 },
    { 0xa0637f8a, SUC_lexicon_tags_33 },
    { 0x10c37f89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f437f90, SUC_lexicon_tags_42 },
    { 0xd0037f91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c237f97, SUC_lexicon_tags_2 },
    { 0xf5437f97, SUC_lexicon_tags_17 },
    { 0x4aa37f98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcf237f9b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e437f9f, SUC_lexicon_tags_11 },
    { 0x3b837f9f, SUC_lexicon_tags_43 },
    { 0x6e837f9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b837fa3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe37fa6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc8237fa8, SUC_lexicon_tags_7 },
    { 0xc5437fa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab437fad, SUC_lexicon_tags_16 },
    { 0x32e37fae, SUC_lexicon_tags_25 },
    { 0xf7037fae, SUC_lexicon_tags_23 },
    { 0x9be37fb0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x42637fb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48837fbd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa0037fbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a37fc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e37fc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40037fc8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a637fcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c37fcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9437fd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94237fd4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xda637fd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b437fd8, SUC_lexicon_tags_25 },
    { 0x27637fd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5637fdb, SUC_lexicon_tags_13 },
    { 0x79237fdc, SUC_lexicon_tags_15 },
    { 0xefc37fdd, SUC_lexicon_tags_59 },
    { 0x51437fde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb9c37fe0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x39c37fe2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d037fe5, SUC_lexicon_tags_15 },
    { 0xe4237fe6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe9a37fe8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb237feb, SUC_lexicon_tags_4 },
    { 0x50837fec, SUC_lexicon_tags_26 },
    { 0x96237fed, SUC_lexicon_tags_13 },
    { 0xae437fec, SUC_lexicon_tags_7 },
    { 0x34837fec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd637ff1, SUC_lexicon_tags_8 },
    { 0xc2a37ff2, SUC_lexicon_tags_17 },
    { 0x7f837ff3, SUC_lexicon_tags_4 },
    { 0x6e037ff4, SUC_lexicon_tags_1 },
    { 0x5b437ff2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x17437ff8, SUC_lexicon_tags_37 },
    { 0x1c437ff9, SUC_lexicon_tags_15 },
    { 0x4f437ffa, SUC_lexicon_tags_2 },
    { 0xc2037ffb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90437ffd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x16638000, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a438002, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd683800a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cc3800c, SUC_lexicon_tags_15 },
    { 0x2003800d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6a23800f, SUC_lexicon_tags_11 },
    { 0x2b838010, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6838014, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x75238016, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97638018, SUC_lexicon_tags_1 },
    { 0xcd838019, SUC_lexicon_tags_37 },
    { 0x5103801a, SUC_lexicon_tags_37 },
    { 0x4643801b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a3801f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94838028, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23802d, SUC_lexicon_tags_17 },
    { 0x7bc3802d, SUC_lexicon_tags_37 },
    { 0x8e43802f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4838036, SUC_lexicon_tags_1 },
    { 0x4ea38036, SUC_lexicon_tags_15 },
    { 0x42a38038, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc3803c, SUC_lexicon_tags_12 },
    { 0x1c63803d, SUC_lexicon_tags_7 },
    { 0x1ec3803c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x27638040, SUC_lexicon_tags_19 },
    { 0x1a438041, SUC_lexicon_tags_14 },
    { 0x9b438041, SUC_lexicon_tags_17 },
    { 0x8bc38043, SUC_lexicon_tags_17 },
    { 0x43e38043, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a38047, SUC_lexicon_tags_17 },
    { 0xd2038048, SUC_lexicon_tags_11 },
    { 0xa8438048, SUC_lexicon_tags_37 },
    { 0xb583804a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba3804e, SUC_lexicon_tags_24 },
    { 0xf603804e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x85238051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38438054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc38058, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6263805b, SUC_lexicon_tags_2 },
    { 0x4183805c, SUC_lexicon_tags_37 },
    { 0x3043805d, SUC_lexicon_tags_37 },
    { 0xb23805e, SUC_lexicon_tags_68 },
    { 0x2103805f, SUC_lexicon_tags_12 },
    { 0x5983805e, SUC_lexicon_tags_32 },
    { 0x9883805e, SUC_lexicon_tags_7 },
    { 0x52c38060, SUC_lexicon_tags_2 },
    { 0x4c83805d, SUC_lexicon_tags_7 },
    { 0x8f03805f, SUC_lexicon_tags_33 },
    { 0xaa38063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03806e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5da38070, SUC_lexicon_tags_47 },
    { 0x22838071, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49a38073, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c38079, SUC_lexicon_tags_17 },
    { 0x39c3807a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb63807c, SUC_lexicon_tags_23 },
    { 0x7aa3807d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1a3807f, SUC_lexicon_tags_17 },
    { 0x41038080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaa38082, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a38085, SUC_lexicon_tags_29 },
    { 0x8be38085, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x35838088, SUC_lexicon_tags_13 },
    { 0x21238089, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a3808c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f23808f, SUC_lexicon_tags_29 },
    { 0xcbe3808f, SUC_lexicon_tags_25 },
    { 0x6ec38091, SUC_lexicon_tags_11 },
    { 0xf3e3808f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d638096, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa3809b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a380a0, SUC_lexicon_tags_43 },
    { 0x6e6380a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbec380a3, SUC_lexicon_tags_1 },
    { 0x25c380a4, SUC_lexicon_tags_12 },
    { 0x7b6380a5, SUC_lexicon_tags_15 },
    { 0x8e0380a4, SUC_lexicon_tags_66 },
    { 0x4b8380a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60380ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc380b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80380b6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae380bd, SUC_lexicon_tags_37 },
    { 0x1a4380be, SUC_lexicon_tags_12 },
    { 0x554380bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e380c2, SUC_lexicon_tags_7 },
    { 0x154380c3, SUC_lexicon_tags_27 },
    { 0x4fa380c4, SUC_lexicon_tags_66 },
    { 0xc2e380c5, SUC_lexicon_tags_25 },
    { 0xb4380c2, SUC_lexicon_tags_10 },
    { 0x9a8380c7, SUC_lexicon_tags_37 },
    { 0x88c380c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa80380ca, SUC_lexicon_tags_35 },
    { 0x412380ca, SUC_lexicon_tags_15 },
    { 0xdec380ca, SUC_lexicon_tags_17 },
    { 0x2de380cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e380d0, SUC_lexicon_tags_37 },
    { 0x240380d1, SUC_lexicon_tags_2 },
    { 0x10a380d2, SUC_lexicon_tags_29 },
    { 0xbfa380d3, SUC_lexicon_tags_23 },
    { 0x4b6380d4, SUC_lexicon_tags_7 },
    { 0x28e380d3, SUC_lexicon_tags_2 },
    { 0x6ce380d6, SUC_lexicon_tags_37 },
    { 0xfb4380d7, SUC_lexicon_tags_17 },
    { 0x7ce380d2, SUC_lexicon_tags_148 },
    { 0x25c380d6, SUC_lexicon_tags_11 },
    { 0x5c4380da, SUC_lexicon_tags_27 },
    { 0x3d6380d1, SUC_lexicon_tags_37 },
    { 0x4d4380dc, SUC_lexicon_tags_37 },
    { 0x48a380d2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0380e6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04380ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa380f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa28380f3, SUC_lexicon_tags_19 },
    { 0xb8e380f3, SUC_lexicon_tags_19 },
    { 0xa34380f5, SUC_lexicon_tags_126 },
    { 0x370380f4, SUC_lexicon_tags_11 },
    { 0x1c6380f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc8380f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb6380fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x84e380fd, SUC_lexicon_tags_11 },
    { 0xa14380fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71438100, SUC_lexicon_tags_17 },
    { 0x6f438101, SUC_lexicon_tags_34 },
    { 0xcd038102, SUC_lexicon_tags_32 },
    { 0xbec38103, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea38106, SUC_lexicon_tags_10 },
    { 0x95838107, SUC_lexicon_tags_60 },
    { 0x37838108, SUC_lexicon_tags_8 },
    { 0x2ea38107, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce038111, SUC_lexicon_tags_11 },
    { 0xce38112, SUC_lexicon_tags_17 },
    { 0xa8038113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21038118, SUC_lexicon_tags_7 },
    { 0xc8e38118, SUC_lexicon_tags_17 },
    { 0x1f238118, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa143811f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59638126, SUC_lexicon_tags_1 },
    { 0xd4238127, SUC_lexicon_tags_37 },
    { 0xe6038128, SUC_lexicon_tags_17 },
    { 0xf6c38129, SUC_lexicon_tags_17 },
    { 0x88838129, SUC_lexicon_tags_27 },
    { 0xc923812a, SUC_lexicon_tags_12 },
    { 0x963812c, SUC_lexicon_tags_17 },
    { 0x22638128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8923812f, SUC_lexicon_tags_12 },
    { 0x2e43812f, SUC_lexicon_tags_15 },
    { 0x52e38131, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89438133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a38138, SUC_lexicon_tags_1 },
    { 0x19438139, SUC_lexicon_tags_1 },
    { 0x6ae3813a, SUC_lexicon_tags_1 },
    { 0xfa43813b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb238142, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf8438144, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1438148, SUC_lexicon_tags_40 },
    { 0xdbe38149, SUC_lexicon_tags_27 },
    { 0x98c38149, SUC_lexicon_tags_14 },
    { 0x7763814b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23038153, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a38157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfde38159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfa3815b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a3815e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b238161, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a38165, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f83816a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e38170, SUC_lexicon_tags_23 },
    { 0x70c38171, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8638177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5343817d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16038180, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ca38182, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b038184, SUC_lexicon_tags_12 },
    { 0x90238185, SUC_lexicon_tags_13 },
    { 0x7c438186, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xea438189, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a3818e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa38195, SUC_lexicon_tags_11 },
    { 0x30838196, SUC_lexicon_tags_30 },
    { 0x2c838195, SUC_lexicon_tags_15 },
    { 0x9de38195, SUC_lexicon_tags_15 },
    { 0x42238195, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x583819b, SUC_lexicon_tags_30 },
    { 0xb423819c, SUC_lexicon_tags_16 },
    { 0x4d23819c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca381a1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x878381a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc64381a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a381ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e381b1, SUC_lexicon_tags_15 },
    { 0x3a4381b1, SUC_lexicon_tags_48 },
    { 0xe62381b3, SUC_lexicon_tags_37 },
    { 0x342381b1, SUC_lexicon_tags_2 },
    { 0x768381b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4381b9, SUC_lexicon_tags_60 },
    { 0x53c381b9, SUC_lexicon_tags_28 },
    { 0xd2a381bb, SUC_lexicon_tags_45 },
    { 0x952381bb, SUC_lexicon_tags_19 },
    { 0x60c381b9, SUC_lexicon_tags_1 },
    { 0xff6381be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c381c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20381c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e381ce, SUC_lexicon_tags_25 },
    { 0x728381cf, SUC_lexicon_tags_1 },
    { 0x5fe381d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336381d6, SUC_lexicon_tags_14 },
    { 0x4b4381d7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50381dd, SUC_lexicon_tags_17 },
    { 0xa0e381de, SUC_lexicon_tags_15 },
    { 0x930381df, SUC_lexicon_tags_12 },
    { 0x760381dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x676381e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x692381e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8381ea, SUC_lexicon_tags_82 },
    { 0xa84381eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14381f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x648381f2, SUC_lexicon_tags_12 },
    { 0xc8c381f3, SUC_lexicon_tags_4 },
    { 0x708381f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9ba381f6, SUC_lexicon_tags_1 },
    { 0xf5e381f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4381fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59038202, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca3820a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a3820d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a238210, SUC_lexicon_tags_14 },
    { 0x92238211, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c38214, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf543821e, SUC_lexicon_tags_14 },
    { 0xfde3821e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8038227, SUC_lexicon_tags_17 },
    { 0xcac38228, SUC_lexicon_tags_88 },
    { 0x4b238229, SUC_lexicon_tags_25 },
    { 0xaba3822a, SUC_lexicon_tags_33 },
    { 0x86a3822b, SUC_lexicon_tags_63 },
    { 0x8463822c, SUC_lexicon_tags_17 },
    { 0xde3822d, SUC_lexicon_tags_0 },
    { 0xc783822c, SUC_lexicon_tags_25 },
    { 0x73638229, SUC_lexicon_tags_49 },
    { 0xb8c3822e, SUC_lexicon_tags_15 },
    { 0xbe3822c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf638237, SUC_lexicon_tags_82 },
    { 0xa5438237, SUC_lexicon_tags_1 },
    { 0x70038239, SUC_lexicon_tags_6 },
    { 0xb243823a, SUC_lexicon_tags_7 },
    { 0x5783823b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa383823e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7a238240, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9038242, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2638247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e63824a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1ec3824c, SUC_lexicon_tags_2 },
    { 0x6043824d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c38255, SUC_lexicon_tags_30 },
    { 0x7e038256, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25238258, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa23825d, SUC_lexicon_tags_1 },
    { 0xafe3825e, SUC_lexicon_tags_2 },
    { 0x3a63825e, SUC_lexicon_tags_17 },
    { 0x6663825f, SUC_lexicon_tags_2 },
    { 0xda638261, SUC_lexicon_tags_12 },
    { 0xe5838261, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da3826a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc38272, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc838274, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbde38276, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c3827a, SUC_lexicon_tags_45 },
    { 0x66e3827b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55038281, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e38285, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7438287, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3423828a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c43828f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xca238292, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b038294, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e43829c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9963829f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x594382a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804382a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e382aa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x79c382ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe382af, SUC_lexicon_tags_32 },
    { 0x7ca382b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x912382b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbce382b4, SUC_lexicon_tags_26 },
    { 0xbda382b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6382b8, SUC_lexicon_tags_0 },
    { 0xd88382b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x474382bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be382bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd44382c1, SUC_lexicon_tags_10 },
    { 0xbb4382c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x56a382c4, SUC_lexicon_tags_17 },
    { 0xdd6382c5, SUC_lexicon_tags_43 },
    { 0xa2a382c5, SUC_lexicon_tags_17 },
    { 0x626382c7, SUC_lexicon_tags_2 },
    { 0x9b8382c7, SUC_lexicon_tags_14 },
    { 0x200382c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc382cf, SUC_lexicon_tags_47 },
    { 0x596382d0, SUC_lexicon_tags_33 },
    { 0x626382d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a382d7, SUC_lexicon_tags_45 },
    { 0x70c382d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x300382da, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x912382dc, SUC_lexicon_tags_27 },
    { 0xa2e382dd, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a382e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60382e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4382ea, SUC_lexicon_tags_17 },
    { 0x2382eb, SUC_lexicon_tags_49 },
    { 0xd5e382ec, SUC_lexicon_tags_17 },
    { 0x674382ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x918382f0, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70382f5, SUC_lexicon_tags_11 },
    { 0xa2c382f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6a382f8, SUC_lexicon_tags_6 },
    { 0xdda382f9, SUC_lexicon_tags_31 },
    { 0x242382f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d8382fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35438301, SUC_lexicon_tags_11 },
    { 0xd2038302, SUC_lexicon_tags_8 },
    { 0xa438303, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9e438305, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45a38307, SUC_lexicon_tags_26 },
    { 0xcaa38307, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac38317, SUC_lexicon_tags_2 },
    { 0x7b838318, SUC_lexicon_tags_63 },
    { 0xa6038319, SUC_lexicon_tags_2 },
    { 0xace38319, SUC_lexicon_tags_0 },
    { 0x1ea3831a, SUC_lexicon_tags_6 },
    { 0x6783831c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c38321, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e438323, SUC_lexicon_tags_17 },
    { 0xa9a38324, SUC_lexicon_tags_29 },
    { 0x17638325, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8903832b, SUC_lexicon_tags_37 },
    { 0xb223832c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84838332, SUC_lexicon_tags_65 },
    { 0xc5438333, SUC_lexicon_tags_2 },
    { 0x12638334, SUC_lexicon_tags_25 },
    { 0x2d438335, SUC_lexicon_tags_25 },
    { 0xa0238336, SUC_lexicon_tags_27 },
    { 0x67838337, SUC_lexicon_tags_85 },
    { 0x4ba38338, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf03833b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc883833e, SUC_lexicon_tags_36 },
    { 0x5483833e, SUC_lexicon_tags_11 },
    { 0xe2e3833f, SUC_lexicon_tags_27 },
    { 0xde83833f, SUC_lexicon_tags_7 },
    { 0x7b238341, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7038344, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69a38346, SUC_lexicon_tags_38 },
    { 0x63438347, SUC_lexicon_tags_7 },
    { 0x50a38346, SUC_lexicon_tags_26 },
    { 0x69438347, SUC_lexicon_tags_77 },
    { 0x73a3834a, SUC_lexicon_tags_84 },
    { 0x7c23834b, SUC_lexicon_tags_103 },
    { 0xd343834c, SUC_lexicon_tags_47 },
    { 0x1803834b, SUC_lexicon_tags_25 },
    { 0x83a38347, SUC_lexicon_tags_14 },
    { 0xa023834f, SUC_lexicon_tags_17 },
    { 0xf103834f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0038353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b43835d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a38360, SUC_lexicon_tags_11 },
    { 0xc4e38361, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61a38363, SUC_lexicon_tags_12 },
    { 0x5d238363, SUC_lexicon_tags_19 },
    { 0x12638363, SUC_lexicon_tags_11 },
    { 0x61838364, SUC_lexicon_tags_47 },
    { 0xe238367, SUC_lexicon_tags_0 },
    { 0xf6c38364, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde23836e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a38373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4323837a, SUC_lexicon_tags_47 },
    { 0x2143837a, SUC_lexicon_tags_12 },
    { 0xd743837c, SUC_lexicon_tags_25 },
    { 0x7163837d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e38381, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xccc38383, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x38238386, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c3838a, SUC_lexicon_tags_65 },
    { 0xb4c3838a, SUC_lexicon_tags_37 },
    { 0xb7a3838a, SUC_lexicon_tags_17 },
    { 0x9b83838d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86038390, SUC_lexicon_tags_37 },
    { 0xb7238391, SUC_lexicon_tags_37 },
    { 0xb6e38390, SUC_lexicon_tags_21 },
    { 0x89e38391, SUC_lexicon_tags_37 },
    { 0x99238394, SUC_lexicon_tags_19 },
    { 0xb1c38395, SUC_lexicon_tags_1 },
    { 0xc7238396, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c3839b, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce383a1, SUC_lexicon_tags_2 },
    { 0x726383a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0a383a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c383a8, SUC_lexicon_tags_40 },
    { 0xc4383a8, SUC_lexicon_tags_55 },
    { 0x390383a8, SUC_lexicon_tags_8 },
    { 0x970383ab, SUC_lexicon_tags_37 },
    { 0x23e383ab, SUC_lexicon_tags_4 },
    { 0x672383a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02383b2, SUC_lexicon_tags_19 },
    { 0x4bc383b2, SUC_lexicon_tags_17 },
    { 0xc7c383b3, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac383b7, SUC_lexicon_tags_50 },
    { 0xec0383b8, SUC_lexicon_tags_42 },
    { 0x59e383b9, SUC_lexicon_tags_11 },
    { 0x830383ba, SUC_lexicon_tags_12 },
    { 0x87a383bb, SUC_lexicon_tags_61 },
    { 0xcaa383b9, SUC_lexicon_tags_11 },
    { 0x458383bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a0383bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c8383c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24c383c3, SUC_lexicon_tags_1 },
    { 0xd6383c4, SUC_lexicon_tags_44 },
    { 0x622383c5, SUC_lexicon_tags_1 },
    { 0xc4383c6, SUC_lexicon_tags_60 },
    { 0xcdc383c6, SUC_lexicon_tags_11 },
    { 0xbda383c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36c383ca, SUC_lexicon_tags_37 },
    { 0x6c6383ca, SUC_lexicon_tags_11 },
    { 0x6e8383cc, SUC_lexicon_tags_37 },
    { 0xc7a383cd, SUC_lexicon_tags_38 },
    { 0xf8c383cd, SUC_lexicon_tags_38 },
    { 0xe383cf, SUC_lexicon_tags_33 },
    { 0xa76383cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17e383d2, SUC_lexicon_tags_25 },
    { 0xd58383d3, SUC_lexicon_tags_25 },
    { 0x6be383d4, SUC_lexicon_tags_5 },
    { 0x75e383d5, SUC_lexicon_tags_56 },
    { 0xaac383d6, SUC_lexicon_tags_25 },
    { 0x828383d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358383e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0383e5, SUC_lexicon_tags_37 },
    { 0x16c383e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c383ec, SUC_lexicon_tags_15 },
    { 0x358383ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfe383ef, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4383f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc8a383f4, SUC_lexicon_tags_37 },
    { 0x4ae383f4, SUC_lexicon_tags_79 },
    { 0x1a8383f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04383f9, SUC_lexicon_tags_25 },
    { 0x2c6383f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88383fd, SUC_lexicon_tags_64 },
    { 0x384383fe, SUC_lexicon_tags_17 },
    { 0xf0383ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc38406, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc3840b, SUC_lexicon_tags_17 },
    { 0x7d23840c, SUC_lexicon_tags_1 },
    { 0x7ae3840b, SUC_lexicon_tags_17 },
    { 0xea03840b, SUC_lexicon_tags_47 },
    { 0xfce3840e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8438411, SUC_lexicon_tags_63 },
    { 0x43838412, SUC_lexicon_tags_17 },
    { 0x1c438413, SUC_lexicon_tags_27 },
    { 0xeb838413, SUC_lexicon_tags_28 },
    { 0x58838412, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff038417, SUC_lexicon_tags_97 },
    { 0x88838418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e3841b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a3841f, SUC_lexicon_tags_1 },
    { 0xa4238420, SUC_lexicon_tags_10 },
    { 0x93a38421, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f838423, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae38426, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6983842b, SUC_lexicon_tags_12 },
    { 0x8023842b, SUC_lexicon_tags_37 },
    { 0x53a3842d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec38432, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643843a, SUC_lexicon_tags_37 },
    { 0xbbe3843a, SUC_lexicon_tags_26 },
    { 0x67c3843a, SUC_lexicon_tags_37 },
    { 0xcba3843d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1823843f, SUC_lexicon_tags_26 },
    { 0x18e38440, SUC_lexicon_tags_7 },
    { 0xea638441, SUC_lexicon_tags_30 },
    { 0x80e38441, SUC_lexicon_tags_7 },
    { 0x5d838443, SUC_lexicon_tags_11 },
    { 0xcbe38444, SUC_lexicon_tags_2 },
    { 0x35a38445, SUC_lexicon_tags_9 },
    { 0xff038446, SUC_lexicon_tags_21 },
    { 0x1e638445, SUC_lexicon_tags_25 },
    { 0x45438448, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3683844f, SUC_lexicon_tags_27 },
    { 0x3323844f, SUC_lexicon_tags_27 },
    { 0x66238450, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x75e38453, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf83845e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0438463, SUC_lexicon_tags_25 },
    { 0x6638463, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e38469, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc923846d, SUC_lexicon_tags_63 },
    { 0xa483846d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e38476, SUC_lexicon_tags_1 },
    { 0x58238476, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa438486, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80638492, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x68238495, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac3849b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ce3849d, SUC_lexicon_tags_2 },
    { 0xd183849e, SUC_lexicon_tags_25 },
    { 0x9483849f, SUC_lexicon_tags_27 },
    { 0x1de384a0, SUC_lexicon_tags_2 },
    { 0x8d43849f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd36384a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312384a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72384ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72384b0, SUC_lexicon_tags_17 },
    { 0x806384b1, SUC_lexicon_tags_17 },
    { 0xc40384b2, SUC_lexicon_tags_66 },
    { 0xdde384b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4384ba, SUC_lexicon_tags_79 },
    { 0xcac384ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2384bf, SUC_lexicon_tags_19 },
    { 0x440384c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e384c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x81c384c5, SUC_lexicon_tags_26 },
    { 0x510384c6, SUC_lexicon_tags_15 },
    { 0x848384c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x88384ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x696384cc, SUC_lexicon_tags_6 },
    { 0xe04384cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee384d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4384d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x712384db, SUC_lexicon_tags_0 },
    { 0xa6c384db, SUC_lexicon_tags_37 },
    { 0x510384dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e384e2, SUC_lexicon_tags_17 },
    { 0xcba384e3, SUC_lexicon_tags_23 },
    { 0x816384e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa50384e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532384eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730384f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc24384f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa384fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd80384fc, SUC_lexicon_tags_105 },
    { 0xbc384fd, SUC_lexicon_tags_7 },
    { 0x256384fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17038500, SUC_lexicon_tags_68 },
    { 0x8e038501, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f83850b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7823850d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf238510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d438512, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3a38514, SUC_lexicon_tags_0 },
    { 0x5c238515, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e3851b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc3851f, SUC_lexicon_tags_77 },
    { 0x6f238520, SUC_lexicon_tags_60 },
    { 0x19238521, SUC_lexicon_tags_57 },
    { 0x3a38521, SUC_lexicon_tags_17 },
    { 0x83e38523, SUC_lexicon_tags_65 },
    { 0xefe38524, SUC_lexicon_tags_17 },
    { 0x23438524, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13038529, SUC_lexicon_tags_2 },
    { 0x7d43852a, SUC_lexicon_tags_31 },
    { 0x29e3852b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e38530, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58838532, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c38536, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e838538, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb043853b, SUC_lexicon_tags_33 },
    { 0xafa3853c, SUC_lexicon_tags_15 },
    { 0xa563853d, SUC_lexicon_tags_63 },
    { 0x8723853e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab038544, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb3e38546, SUC_lexicon_tags_45 },
    { 0x7c38547, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a3854b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc2a3854d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6783854f, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92238555, SUC_lexicon_tags_33 },
    { 0xae38555, SUC_lexicon_tags_17 },
    { 0x7de38557, SUC_lexicon_tags_11 },
    { 0xcc38556, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x843855b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe8e3855d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c38561, SUC_lexicon_tags_19 },
    { 0x6f438562, SUC_lexicon_tags_17 },
    { 0xaf638563, SUC_lexicon_tags_39 },
    { 0xe9638564, SUC_lexicon_tags_2 },
    { 0x56a38561, SUC_lexicon_tags_14 },
    { 0x1ea38566, SUC_lexicon_tags_48 },
    { 0xfc638563, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fe38569, SUC_lexicon_tags_47 },
    { 0x4423856a, SUC_lexicon_tags_33 },
    { 0xcaa3856a, SUC_lexicon_tags_7 },
    { 0xdba3856c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c38571, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed038574, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4838576, SUC_lexicon_tags_7 },
    { 0xd3e38577, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4303857c, SUC_lexicon_tags_12 },
    { 0xc983857c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24438580, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3f638582, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4838586, SUC_lexicon_tags_1 },
    { 0x92a38586, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5943858f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x61038592, SUC_lexicon_tags_49 },
    { 0xe2a38592, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27438598, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe523859e, SUC_lexicon_tags_37 },
    { 0x87a3859f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268385a5, SUC_lexicon_tags_37 },
    { 0x86c385a6, SUC_lexicon_tags_49 },
    { 0x8a2385a5, SUC_lexicon_tags_11 },
    { 0x46e385a5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e385ab, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaa6385ad, SUC_lexicon_tags_7 },
    { 0xd80385ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f4385b0, SUC_lexicon_tags_15 },
    { 0x99e385b1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2c385b3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x8dc385b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x606385b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc34385b9, SUC_lexicon_tags_8 },
    { 0x70a385ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a385be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86c385c0, SUC_lexicon_tags_2 },
    { 0xd20385c0, SUC_lexicon_tags_17 },
    { 0xab8385c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64385c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f6385c8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf2e385ca, SUC_lexicon_tags_47 },
    { 0xefa385ca, SUC_lexicon_tags_8 },
    { 0x682385cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810385d0, SUC_lexicon_tags_60 },
    { 0x36385d1, SUC_lexicon_tags_43 },
    { 0x13c385d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e385d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6385d7, SUC_lexicon_tags_11 },
    { 0xf44385d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x418385da, SUC_lexicon_tags_8 },
    { 0x842385db, SUC_lexicon_tags_8 },
    { 0x10e385dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c385e0, SUC_lexicon_tags_25 },
    { 0xd66385e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec385e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x350385ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c385f0, SUC_lexicon_tags_7 },
    { 0xcec385f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe385f5, SUC_lexicon_tags_381 },
    { 0x3c2385f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e385f9, SUC_lexicon_tags_19 },
    { 0xf06385f9, SUC_lexicon_tags_1 },
    { 0x972385fa, SUC_lexicon_tags_8 },
    { 0x5a6385fc, SUC_lexicon_tags_84 },
    { 0xdb0385fd, SUC_lexicon_tags_29 },
    { 0x6c4385fe, SUC_lexicon_tags_4 },
    { 0x16e385ff, SUC_lexicon_tags_6 },
    { 0x6fe385fc, SUC_lexicon_tags_82 },
    { 0xa8a38601, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xac38604, SUC_lexicon_tags_45 },
    { 0x50c38604, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58638609, SUC_lexicon_tags_14 },
    { 0xac43860a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb63860d, SUC_lexicon_tags_21 },
    { 0x7f63860e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee38612, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9fe38614, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c38618, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa663861d, SUC_lexicon_tags_74 },
    { 0xd403861d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a838622, SUC_lexicon_tags_12 },
    { 0xc4038622, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24038625, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa9e38627, SUC_lexicon_tags_18 },
    { 0x29c38627, SUC_lexicon_tags_30 },
    { 0xc2838629, SUC_lexicon_tags_0 },
    { 0x7b238629, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64638630, SUC_lexicon_tags_37 },
    { 0xf5238631, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74c38633, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a3863a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11c3863c, SUC_lexicon_tags_68 },
    { 0x9203863d, SUC_lexicon_tags_17 },
    { 0xb363863e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e38643, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60838645, SUC_lexicon_tags_23 },
    { 0x43038646, SUC_lexicon_tags_46 },
    { 0x93c38647, SUC_lexicon_tags_2 },
    { 0xf2038647, SUC_lexicon_tags_1 },
    { 0x24838649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf403864d, SUC_lexicon_tags_29 },
    { 0xc263864d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b038650, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade38655, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c83865a, SUC_lexicon_tags_1 },
    { 0xb363865b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1c83865d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e38661, SUC_lexicon_tags_26 },
    { 0x97638662, SUC_lexicon_tags_15 },
    { 0xbfa38663, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4ca38665, SUC_lexicon_tags_19 },
    { 0x7d238665, SUC_lexicon_tags_11 },
    { 0x2b438665, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a3866a, SUC_lexicon_tags_12 },
    { 0x3803866b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62838671, SUC_lexicon_tags_28 },
    { 0xd038672, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c38675, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53038677, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xf8038679, SUC_lexicon_tags_35 },
    { 0x20e3867a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x843867d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe963867f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x61238681, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d438683, SUC_lexicon_tags_25 },
    { 0x41e38683, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e38687, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5103868a, SUC_lexicon_tags_1 },
    { 0x38c3868a, SUC_lexicon_tags_6 },
    { 0xce23868b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb638691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55038695, SUC_lexicon_tags_8 },
    { 0xada38696, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0038699, SUC_lexicon_tags_45 },
    { 0x283869a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6403869d, SUC_lexicon_tags_17 },
    { 0xcfc3869d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3a386a0, SUC_lexicon_tags_13 },
    { 0x760386a0, SUC_lexicon_tags_3 },
    { 0xefa386a1, SUC_lexicon_tags_25 },
    { 0x520386a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca386a7, SUC_lexicon_tags_265 },
    { 0x7be386a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262386ae, SUC_lexicon_tags_17 },
    { 0x2cc386af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0386b7, SUC_lexicon_tags_33 },
    { 0x5ee386b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x440386ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be386bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa386c2, SUC_lexicon_tags_10 },
    { 0xfd0386c2, SUC_lexicon_tags_158 },
    { 0x33a386c4, SUC_lexicon_tags_12 },
    { 0x13a386c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e386d1, SUC_lexicon_tags_59 },
    { 0x7a2386d1, SUC_lexicon_tags_11 },
    { 0x6e4386d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x564386d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a386db, SUC_lexicon_tags_419 },
    { 0x88a386dc, SUC_lexicon_tags_11 },
    { 0x2f8386db, SUC_lexicon_tags_27 },
    { 0x662386de, SUC_lexicon_tags_27 },
    { 0x778386db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a386e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x766386e8, SUC_lexicon_tags_17 },
    { 0x59e386e8, SUC_lexicon_tags_106 },
    { 0x6d6386ea, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x9a6386ec, SUC_lexicon_tags_37 },
    { 0x696386ec, SUC_lexicon_tags_64 },
    { 0x63c386ee, SUC_lexicon_tags_30 },
    { 0xf4c386ee, SUC_lexicon_tags_29 },
    { 0x364386ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e386f3, SUC_lexicon_tags_29 },
    { 0x644386f3, SUC_lexicon_tags_79 },
    { 0xbc8386f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc4386f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0386fa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a386fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda838700, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fa38702, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x32438704, SUC_lexicon_tags_7 },
    { 0x6b038704, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b638709, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9723870b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e438710, SUC_lexicon_tags_15 },
    { 0x27238711, SUC_lexicon_tags_7 },
    { 0x73438712, SUC_lexicon_tags_30 },
    { 0x41238711, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x84638715, SUC_lexicon_tags_8 },
    { 0xa8a38716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x88238719, SUC_lexicon_tags_47 },
    { 0xd663871a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa323871f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c38726, SUC_lexicon_tags_7 },
    { 0xca838727, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc3872a, SUC_lexicon_tags_15 },
    { 0xb403872a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e3872f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55038735, SUC_lexicon_tags_55 },
    { 0x31238736, SUC_lexicon_tags_11 },
    { 0xb438736, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e3873a, SUC_lexicon_tags_11 },
    { 0x5523873b, SUC_lexicon_tags_15 },
    { 0x84a3873c, SUC_lexicon_tags_1 },
    { 0x783873d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xea38740, SUC_lexicon_tags_30 },
    { 0x6e038740, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7dc38743, SUC_lexicon_tags_11 },
    { 0x70c38744, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe2838746, SUC_lexicon_tags_60 },
    { 0xd2238747, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e38752, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8c38754, SUC_lexicon_tags_1022 },
    { 0x6e38754, SUC_lexicon_tags_8 },
    { 0x54238756, SUC_lexicon_tags_15 },
    { 0xf0838757, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce3875a, SUC_lexicon_tags_15 },
    { 0x3be3875a, SUC_lexicon_tags_12 },
    { 0xec23875b, SUC_lexicon_tags_19 },
    { 0x7563875b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c238765, SUC_lexicon_tags_6 },
    { 0xdec38766, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4638768, SUC_lexicon_tags_1 },
    { 0xb6438769, SUC_lexicon_tags_15 },
    { 0x3003876a, SUC_lexicon_tags_7 },
    { 0xe203876b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23876e, SUC_lexicon_tags_29 },
    { 0xee83876f, SUC_lexicon_tags_7 },
    { 0xd443876e, SUC_lexicon_tags_25 },
    { 0x2283876e, SUC_lexicon_tags_14 },
    { 0x28038771, SUC_lexicon_tags_69 },
    { 0xcb238771, SUC_lexicon_tags_43 },
    { 0x53638774, SUC_lexicon_tags_19 },
    { 0xfb238775, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe38778, SUC_lexicon_tags_11 },
    { 0x39438779, SUC_lexicon_tags_38 },
    { 0xe403877a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8243877e, SUC_lexicon_tags_11 },
    { 0x5083877f, SUC_lexicon_tags_25 },
    { 0xe0e3877f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91a38782, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57038786, SUC_lexicon_tags_65 },
    { 0x31c38786, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11838789, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da3878c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x96c3878e, SUC_lexicon_tags_11 },
    { 0x4263878f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae38796, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be3879a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a43879f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0387a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x982387a4, SUC_lexicon_tags_15 },
    { 0x1ce387a5, SUC_lexicon_tags_12 },
    { 0x732387a6, SUC_lexicon_tags_63 },
    { 0xd50387a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x418387a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2387ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e387af, SUC_lexicon_tags_1 },
    { 0x6d4387af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8387b8, SUC_lexicon_tags_37 },
    { 0x3ec387b8, SUC_lexicon_tags_38 },
    { 0xad6387b9, SUC_lexicon_tags_42 },
    { 0x33a387bb, SUC_lexicon_tags_33 },
    { 0x906387bc, SUC_lexicon_tags_11 },
    { 0x756387bd, SUC_lexicon_tags_17 },
    { 0x40a387be, SUC_lexicon_tags_43 },
    { 0x420387bc, SUC_lexicon_tags_12 },
    { 0x4387c0, SUC_lexicon_tags_99 },
    { 0xc7c387c1, SUC_lexicon_tags_59 },
    { 0x38a387b9, SUC_lexicon_tags_11 },
    { 0xf76387c3, SUC_lexicon_tags_44 },
    { 0x32a387bc, SUC_lexicon_tags_68 },
    { 0x1c4387c5, SUC_lexicon_tags_12 },
    { 0x766387c1, SUC_lexicon_tags_47 },
    { 0x150387c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe86387c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8387cc, SUC_lexicon_tags_28 },
    { 0x1cc387cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x128387cf, SUC_lexicon_tags_10 },
    { 0xade387d0, SUC_lexicon_tags_10 },
    { 0xb66387d1, SUC_lexicon_tags_16 },
    { 0x540387d2, SUC_lexicon_tags_11 },
    { 0x580387d2, SUC_lexicon_tags_7 },
    { 0x44c387d3, SUC_lexicon_tags_11 },
    { 0x462387d1, SUC_lexicon_tags_64 },
    { 0x1fc387d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1e387d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda0387da, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xab4387dc, SUC_lexicon_tags_12 },
    { 0x384387dd, SUC_lexicon_tags_25 },
    { 0x7de387de, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba387e2, SUC_lexicon_tags_17 },
    { 0xf56387e3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc387e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x47a387ea, SUC_lexicon_tags_45 },
    { 0x870387eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64387f0, SUC_lexicon_tags_47 },
    { 0x6c2387f1, SUC_lexicon_tags_10 },
    { 0xab8387f0, SUC_lexicon_tags_70 },
    { 0x5a4387f3, SUC_lexicon_tags_12 },
    { 0xb12387f1, SUC_lexicon_tags_57 },
    { 0x244387f3, SUC_lexicon_tags_27 },
    { 0xb94387f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a387fc, SUC_lexicon_tags_64 },
    { 0xc82387fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc238800, SUC_lexicon_tags_113 },
    { 0xab038801, SUC_lexicon_tags_12 },
    { 0x8fa38801, SUC_lexicon_tags_10 },
    { 0x85a38803, SUC_lexicon_tags_1 },
    { 0x56238804, SUC_lexicon_tags_17 },
    { 0xab438804, SUC_lexicon_tags_28 },
    { 0x65238806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d43880d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70038814, SUC_lexicon_tags_31 },
    { 0xd5a38815, SUC_lexicon_tags_37 },
    { 0x41a38814, SUC_lexicon_tags_17 },
    { 0x42e38817, SUC_lexicon_tags_17 },
    { 0x1ca38818, SUC_lexicon_tags_7 },
    { 0xb038819, SUC_lexicon_tags_64 },
    { 0x2583881a, SUC_lexicon_tags_59 },
    { 0xd1838814, SUC_lexicon_tags_1 },
    { 0x5523881c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e38822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e38825, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8038828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a38831, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x35038833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1038837, SUC_lexicon_tags_0 },
    { 0xdb038838, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be3883b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc3883e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9038841, SUC_lexicon_tags_47 },
    { 0xa6e38842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24838845, SUC_lexicon_tags_27 },
    { 0xaa238846, SUC_lexicon_tags_56 },
    { 0xf3238847, SUC_lexicon_tags_80 },
    { 0x29c38846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee83884a, SUC_lexicon_tags_7 },
    { 0xf5c3884a, SUC_lexicon_tags_25 },
    { 0x3a3884a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41838851, SUC_lexicon_tags_2 },
    { 0xf438851, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22438854, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4038857, SUC_lexicon_tags_31 },
    { 0xe8e38858, SUC_lexicon_tags_19 },
    { 0x2e038859, SUC_lexicon_tags_57 },
    { 0xe2438859, SUC_lexicon_tags_29 },
    { 0x1b03885a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6563885d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca38860, SUC_lexicon_tags_48 },
    { 0x36638860, SUC_lexicon_tags_14 },
    { 0x72e38860, SUC_lexicon_tags_28 },
    { 0x87638863, SUC_lexicon_tags_2 },
    { 0x2b238864, SUC_lexicon_tags_10 },
    { 0x85638864, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa38870, SUC_lexicon_tags_27 },
    { 0xfb838871, SUC_lexicon_tags_29 },
    { 0x23838872, SUC_lexicon_tags_63 },
    { 0x39c38872, SUC_lexicon_tags_2 },
    { 0xeb838874, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c3887a, SUC_lexicon_tags_17 },
    { 0xacc3887b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e38884, SUC_lexicon_tags_25 },
    { 0x74638885, SUC_lexicon_tags_37 },
    { 0x16c38885, SUC_lexicon_tags_6 },
    { 0xeda38887, SUC_lexicon_tags_38 },
    { 0xd3a38887, SUC_lexicon_tags_37 },
    { 0x20438889, SUC_lexicon_tags_28 },
    { 0xc4c38885, SUC_lexicon_tags_38 },
    { 0x67e3888b, SUC_lexicon_tags_28 },
    { 0xaee38887, SUC_lexicon_tags_49 },
    { 0x75a38885, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75038892, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae38895, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c43889c, SUC_lexicon_tags_2 },
    { 0x1a83889c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4388a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0388a4, SUC_lexicon_tags_25 },
    { 0xd12388a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca4388a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x436388aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0388ad, SUC_lexicon_tags_28 },
    { 0xac6388ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c388b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd4388b5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x298388b8, SUC_lexicon_tags_47 },
    { 0x7c0388b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a0388bb, SUC_lexicon_tags_2 },
    { 0x7a2388bc, SUC_lexicon_tags_49 },
    { 0xc92388bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6388c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2c0388c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b4388c4, SUC_lexicon_tags_82 },
    { 0x146388c5, SUC_lexicon_tags_27 },
    { 0xcba388c5, SUC_lexicon_tags_47 },
    { 0x806388c7, SUC_lexicon_tags_14 },
    { 0xf40388c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76388cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2bc388cd, SUC_lexicon_tags_12 },
    { 0x532388ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x790388d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e388d6, SUC_lexicon_tags_49 },
    { 0x22e388d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6388da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad2388dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x818388de, SUC_lexicon_tags_5 },
    { 0x122388df, SUC_lexicon_tags_25 },
    { 0x1ec388df, SUC_lexicon_tags_25 },
    { 0x4e2388e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c388e4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x788388e6, SUC_lexicon_tags_43 },
    { 0x602388e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d4388e9, SUC_lexicon_tags_9 },
    { 0x1aa388ea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4388ec, SUC_lexicon_tags_37 },
    { 0x320388ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8388f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756388f9, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50388fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e38900, SUC_lexicon_tags_25 },
    { 0x65238900, SUC_lexicon_tags_60 },
    { 0x7e038902, SUC_lexicon_tags_15 },
    { 0x6be38903, SUC_lexicon_tags_7 },
    { 0x32c38902, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xea238906, SUC_lexicon_tags_25 },
    { 0x89238906, SUC_lexicon_tags_37 },
    { 0x39438908, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e3890b, SUC_lexicon_tags_31 },
    { 0xa183890c, SUC_lexicon_tags_343 },
    { 0x7503890d, SUC_lexicon_tags_26 },
    { 0x7603890e, SUC_lexicon_tags_30 },
    { 0xe983890f, SUC_lexicon_tags_30 },
    { 0x26038910, SUC_lexicon_tags_2 },
    { 0x31a38910, SUC_lexicon_tags_17 },
    { 0x3983890e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb438915, SUC_lexicon_tags_17 },
    { 0x2b838915, SUC_lexicon_tags_1 },
    { 0xe5e38917, SUC_lexicon_tags_1 },
    { 0x86c38918, SUC_lexicon_tags_63 },
    { 0x40838919, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6643891d, SUC_lexicon_tags_34 },
    { 0x4583891d, SUC_lexicon_tags_1 },
    { 0xb3c3891f, SUC_lexicon_tags_61 },
    { 0x16e38920, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a38923, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5038927, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7838929, SUC_lexicon_tags_11 },
    { 0xd923892a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42e3892c, SUC_lexicon_tags_29 },
    { 0x9b83892d, SUC_lexicon_tags_17 },
    { 0x1003892e, SUC_lexicon_tags_63 },
    { 0x2ea3892f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a38934, SUC_lexicon_tags_60 },
    { 0x59038935, SUC_lexicon_tags_0 },
    { 0x51038936, SUC_lexicon_tags_35 },
    { 0x24238937, SUC_lexicon_tags_1 },
    { 0xd7238938, SUC_lexicon_tags_16 },
    { 0x5638935, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce3893d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1838940, SUC_lexicon_tags_17 },
    { 0x15838940, SUC_lexicon_tags_30 },
    { 0x58e38942, SUC_lexicon_tags_1 },
    { 0x96038940, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb403894d, SUC_lexicon_tags_124 },
    { 0xf723894e, SUC_lexicon_tags_29 },
    { 0xb683894f, SUC_lexicon_tags_16 },
    { 0x3b43894f, SUC_lexicon_tags_17 },
    { 0xfae38951, SUC_lexicon_tags_43 },
    { 0x2e83894f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a38958, SUC_lexicon_tags_27 },
    { 0xe2038959, SUC_lexicon_tags_30 },
    { 0xc2838958, SUC_lexicon_tags_54 },
    { 0x96a3895a, SUC_lexicon_tags_39 },
    { 0xdbe3895c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa3895f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea38962, SUC_lexicon_tags_1 },
    { 0xc8e38963, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3443896a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26c3896c, SUC_lexicon_tags_14 },
    { 0xd783896c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9063896f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb238972, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf438974, SUC_lexicon_tags_14 },
    { 0x42a38975, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76638977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8763897b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c3897e, SUC_lexicon_tags_7 },
    { 0x1763897e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c38984, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d838986, SUC_lexicon_tags_15 },
    { 0x85a38987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb63898c, SUC_lexicon_tags_45 },
    { 0x5d23898d, SUC_lexicon_tags_7 },
    { 0xfe3898e, SUC_lexicon_tags_11 },
    { 0x363898d, SUC_lexicon_tags_8 },
    { 0x8d638990, SUC_lexicon_tags_1 },
    { 0x7c3898d, SUC_lexicon_tags_25 },
    { 0x20238992, SUC_lexicon_tags_15 },
    { 0xa7038993, SUC_lexicon_tags_60 },
    { 0xdb638994, SUC_lexicon_tags_15 },
    { 0xc1e38995, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a38998, SUC_lexicon_tags_336 },
    { 0x7be38998, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03899c, SUC_lexicon_tags_27 },
    { 0x93a3899d, SUC_lexicon_tags_68 },
    { 0x2e63899e, SUC_lexicon_tags_12 },
    { 0x6443899f, SUC_lexicon_tags_0 },
    { 0xed23899d, SUC_lexicon_tags_3 },
    { 0xc46389a1, SUC_lexicon_tags_12 },
    { 0x222389a2, SUC_lexicon_tags_17 },
    { 0xc4e3899c, SUC_lexicon_tags_37 },
    { 0x5ca389a4, SUC_lexicon_tags_30 },
    { 0xec0389a4, SUC_lexicon_tags_17 },
    { 0x186389a6, SUC_lexicon_tags_28 },
    { 0x7de389a6, SUC_lexicon_tags_64 },
    { 0x9de389a8, SUC_lexicon_tags_25 },
    { 0x18e389a4, SUC_lexicon_tags_42 },
    { 0x254389aa, SUC_lexicon_tags_49 },
    { 0xd0a389ab, SUC_lexicon_tags_26 },
    { 0x9aa389ac, SUC_lexicon_tags_17 },
    { 0x14a389ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe389b1, SUC_lexicon_tags_11 },
    { 0x440389b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2389b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa16389b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80389c1, SUC_lexicon_tags_4 },
    { 0xbdc389c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce389c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe54389c8, SUC_lexicon_tags_88 },
    { 0x426389c9, SUC_lexicon_tags_9 },
    { 0xf20389ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa389d6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x422389d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464389de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x334389e1, SUC_lexicon_tags_9 },
    { 0xe3a389e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e389e6, SUC_lexicon_tags_37 },
    { 0xf94389e7, SUC_lexicon_tags_25 },
    { 0x83c389e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5ce389ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ba389ec, SUC_lexicon_tags_31 },
    { 0xba8389ec, SUC_lexicon_tags_15 },
    { 0xc64389ee, SUC_lexicon_tags_17 },
    { 0xe78389ef, SUC_lexicon_tags_33 },
    { 0xc96389ef, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x766389f3, SUC_lexicon_tags_31 },
    { 0xaec389f4, SUC_lexicon_tags_47 },
    { 0x88e389f5, SUC_lexicon_tags_69 },
    { 0xd90389f3, SUC_lexicon_tags_15 },
    { 0x212389f7, SUC_lexicon_tags_11 },
    { 0x5c8389f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2389fd, SUC_lexicon_tags_17 },
    { 0x926389fd, SUC_lexicon_tags_49 },
    { 0xdd2389ff, SUC_lexicon_tags_8 },
    { 0xaee38a00, SUC_lexicon_tags_72 },
    { 0xf9838a01, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e38a08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98638a0a, SUC_lexicon_tags_69 },
    { 0xc4e38a0b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a38a0e, SUC_lexicon_tags_25 },
    { 0x39438a0f, SUC_lexicon_tags_27 },
    { 0xfc038a0f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfae38a12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xade38a14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6038a1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a38a24, SUC_lexicon_tags_30 },
    { 0xc38a24, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c38a29, SUC_lexicon_tags_37 },
    { 0x93c38a2a, SUC_lexicon_tags_7 },
    { 0xa1438a2a, SUC_lexicon_tags_47 },
    { 0x73438a2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e38a30, SUC_lexicon_tags_12 },
    { 0x6ec38a31, SUC_lexicon_tags_69 },
    { 0x65a38a30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x94438a35, SUC_lexicon_tags_12 },
    { 0x32e38a36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c38a39, SUC_lexicon_tags_37 },
    { 0x69a38a3a, SUC_lexicon_tags_17 },
    { 0xe8838a3a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a38a42, SUC_lexicon_tags_65 },
    { 0xb6838a43, SUC_lexicon_tags_37 },
    { 0x91c38a44, SUC_lexicon_tags_63 },
    { 0xd638a43, SUC_lexicon_tags_3 },
    { 0xb6638a46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4838a4d, SUC_lexicon_tags_27 },
    { 0xb8238a4e, SUC_lexicon_tags_31 },
    { 0x6c238a4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58838a54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5438a57, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32038a5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2038a5d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc38a66, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8fa38a68, SUC_lexicon_tags_37 },
    { 0xa6838a68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe38a6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91638a76, SUC_lexicon_tags_38 },
    { 0xfae38a76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a38a7b, SUC_lexicon_tags_7 },
    { 0x42838a7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23038a7f, SUC_lexicon_tags_43 },
    { 0x80038a7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70838a83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63638a86, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xba638a88, SUC_lexicon_tags_17 },
    { 0xe4c38a89, SUC_lexicon_tags_49 },
    { 0x84838a8a, SUC_lexicon_tags_37 },
    { 0x74238a8b, SUC_lexicon_tags_11 },
    { 0x18c38a8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb038a8e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2638a91, SUC_lexicon_tags_0 },
    { 0x96e38a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3438a95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a38a98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6c38a9a, SUC_lexicon_tags_30 },
    { 0x6ea38a9b, SUC_lexicon_tags_11 },
    { 0x63c38a9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4638aae, SUC_lexicon_tags_19 },
    { 0xc7a38aae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a38ab2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c38ab5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a838ab7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfe238ab9, SUC_lexicon_tags_37 },
    { 0x32238ab9, SUC_lexicon_tags_63 },
    { 0xa2038abb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa838abf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe38ac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a38ac5, SUC_lexicon_tags_0 },
    { 0x91038ac6, SUC_lexicon_tags_11 },
    { 0xf9638ac7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e38ace, SUC_lexicon_tags_2 },
    { 0xc6438acf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6238ad3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a38ad6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5838ad9, SUC_lexicon_tags_2 },
    { 0xa3c38ad9, SUC_lexicon_tags_65 },
    { 0x93838ada, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8438ade, SUC_lexicon_tags_27 },
    { 0x34838adf, SUC_lexicon_tags_11 },
    { 0x6d638ae0, SUC_lexicon_tags_37 },
    { 0xe2438adf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a438ae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6238aea, SUC_lexicon_tags_7 },
    { 0x28238aeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14238af0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe38af3, SUC_lexicon_tags_12 },
    { 0xa5038af3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2aa38af6, SUC_lexicon_tags_4 },
    { 0xa4838af7, SUC_lexicon_tags_43 },
    { 0xd8238af8, SUC_lexicon_tags_27 },
    { 0xed038af9, SUC_lexicon_tags_38 },
    { 0x7ee38afa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e038afd, SUC_lexicon_tags_31 },
    { 0x32038afe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73438b04, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe2838b06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x79038b08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fe38b0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc38b0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c38b15, SUC_lexicon_tags_7 },
    { 0x5fe38b16, SUC_lexicon_tags_1 },
    { 0x22838b16, SUC_lexicon_tags_56 },
    { 0x30638b18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd0638b1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41238b21, SUC_lexicon_tags_31 },
    { 0x9aa38b22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a38b28, SUC_lexicon_tags_1 },
    { 0xe8638b29, SUC_lexicon_tags_47 },
    { 0xe2238b2a, SUC_lexicon_tags_9 },
    { 0xfaa38b28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a38b2e, SUC_lexicon_tags_19 },
    { 0x3038b2e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea638b35, SUC_lexicon_tags_39 },
    { 0xc8e38b36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc238b38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7438b3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8438b3d, SUC_lexicon_tags_26 },
    { 0xe238b3e, SUC_lexicon_tags_16 },
    { 0x6ac38b3f, SUC_lexicon_tags_17 },
    { 0xfae38b40, SUC_lexicon_tags_22 },
    { 0xc5638b3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8638b46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x61a38b48, SUC_lexicon_tags_37 },
    { 0x8cc38b49, SUC_lexicon_tags_49 },
    { 0x3f238b4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1238b4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fc38b4f, SUC_lexicon_tags_25 },
    { 0x6c238b50, SUC_lexicon_tags_27 },
    { 0x44638b51, SUC_lexicon_tags_12 },
    { 0xefe38b51, SUC_lexicon_tags_15 },
    { 0x82838b51, SUC_lexicon_tags_11 },
    { 0x9bc38b54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5ce38b56, SUC_lexicon_tags_23 },
    { 0xf4238b56, SUC_lexicon_tags_5 },
    { 0x2a038b58, SUC_lexicon_tags_17 },
    { 0xf1638b59, SUC_lexicon_tags_56 },
    { 0xb038b57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63638b60, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb438b63, SUC_lexicon_tags_15 },
    { 0x66238b64, SUC_lexicon_tags_37 },
    { 0xba838b65, SUC_lexicon_tags_33 },
    { 0x50438b66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9e038b68, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xfc638b6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b038b6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4238b74, SUC_lexicon_tags_1 },
    { 0xcb438b75, SUC_lexicon_tags_29 },
    { 0x74038b75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee38b79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x90e38b7b, SUC_lexicon_tags_82 },
    { 0x11e38b7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b638b7e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x77438b80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3038b88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea38b8e, SUC_lexicon_tags_28 },
    { 0x99a38b8e, SUC_lexicon_tags_63 },
    { 0x44c38b90, SUC_lexicon_tags_27 },
    { 0x9e38b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0e38b93, SUC_lexicon_tags_25 },
    { 0x44438b93, SUC_lexicon_tags_11 },
    { 0xf5438b95, SUC_lexicon_tags_11 },
    { 0x42238b95, SUC_lexicon_tags_37 },
    { 0x3ca38b94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c038b9b, SUC_lexicon_tags_28 },
    { 0x2dc38b9c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x75438b9e, SUC_lexicon_tags_43 },
    { 0x9c038b9e, SUC_lexicon_tags_17 },
    { 0xd5438ba0, SUC_lexicon_tags_37 },
    { 0xbb638ba1, SUC_lexicon_tags_7 },
    { 0xf2c38b9f, SUC_lexicon_tags_38 },
    { 0xd5238b9e, SUC_lexicon_tags_31 },
    { 0x81e38ba4, SUC_lexicon_tags_39 },
    { 0x38238b9f, SUC_lexicon_tags_12 },
    { 0xe9838ba6, SUC_lexicon_tags_38 },
    { 0x33e38ba4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6238baa, SUC_lexicon_tags_37 },
    { 0x28038bab, SUC_lexicon_tags_1 },
    { 0x6e238bac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55438bb4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e38bb8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9638bbb, SUC_lexicon_tags_79 },
    { 0x61038bbb, SUC_lexicon_tags_2 },
    { 0x15c38bbc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6038bc1, SUC_lexicon_tags_11 },
    { 0x45838bc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82438bc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3038bca, SUC_lexicon_tags_116 },
    { 0xee838bcb, SUC_lexicon_tags_27 },
    { 0x70838bcc, SUC_lexicon_tags_15 },
    { 0xfd838bcd, SUC_lexicon_tags_25 },
    { 0x58038bce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c38bd4, SUC_lexicon_tags_17 },
    { 0x36838bd5, SUC_lexicon_tags_140 },
    { 0xce838bd6, SUC_lexicon_tags_25 },
    { 0x14238bd4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de38bdc, SUC_lexicon_tags_12 },
    { 0xa1438bdc, SUC_lexicon_tags_15 },
    { 0x5638bde, SUC_lexicon_tags_15 },
    { 0x71a38bde, SUC_lexicon_tags_25 },
    { 0xb6438bde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82238be6, SUC_lexicon_tags_14 },
    { 0x42e38be7, SUC_lexicon_tags_47 },
    { 0x7438be8, SUC_lexicon_tags_37 },
    { 0x23e38be7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71e38beb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee38bf1, SUC_lexicon_tags_7 },
    { 0x7ce38bf1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d238bf5, SUC_lexicon_tags_35 },
    { 0x64238bf6, SUC_lexicon_tags_37 },
    { 0xa3c38bf6, SUC_lexicon_tags_14 },
    { 0x74c38bf6, SUC_lexicon_tags_35 },
    { 0x41038bf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a38bfd, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a38c00, SUC_lexicon_tags_15 },
    { 0x71c38c01, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a38c04, SUC_lexicon_tags_7 },
    { 0x18638c04, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfd638c07, SUC_lexicon_tags_5 },
    { 0x43438c07, SUC_lexicon_tags_27 },
    { 0xf8838c09, SUC_lexicon_tags_55 },
    { 0x4dc38c07, SUC_lexicon_tags_12 },
    { 0x16e38c07, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e38c12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb238c15, SUC_lexicon_tags_61 },
    { 0x4ae38c16, SUC_lexicon_tags_38 },
    { 0xd0a38c16, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f838c1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76838c21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba38c27, SUC_lexicon_tags_14 },
    { 0x42038c27, SUC_lexicon_tags_37 },
    { 0x4aa38c28, SUC_lexicon_tags_25 },
    { 0xb8638c2a, SUC_lexicon_tags_11 },
    { 0x2ce38c28, SUC_lexicon_tags_14 },
    { 0xae438c29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe38c32, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f038c34, SUC_lexicon_tags_2 },
    { 0xbc038c35, SUC_lexicon_tags_25 },
    { 0xa7c38c34, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a238c39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75238c3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c438c42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c38c45, SUC_lexicon_tags_49 },
    { 0xcba38c46, SUC_lexicon_tags_2 },
    { 0xe4438c46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53c38c49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f438c4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b838c50, SUC_lexicon_tags_4 },
    { 0x9a638c50, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc638c55, SUC_lexicon_tags_61 },
    { 0x58a38c56, SUC_lexicon_tags_11 },
    { 0xf0e38c56, SUC_lexicon_tags_9 },
    { 0xae38c58, SUC_lexicon_tags_25 },
    { 0xde38c57, SUC_lexicon_tags_10 },
    { 0x3b638c57, SUC_lexicon_tags_17 },
    { 0x11c38c5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27838c5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2c38c61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b838c63, SUC_lexicon_tags_60 },
    { 0xd8438c63, SUC_lexicon_tags_17 },
    { 0x5be38c65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80638c6a, SUC_lexicon_tags_38 },
    { 0x2c238c6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7238c6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac838c75, SUC_lexicon_tags_37 },
    { 0x56038c76, SUC_lexicon_tags_2 },
    { 0x54c38c77, SUC_lexicon_tags_30 },
    { 0xd0238c78, SUC_lexicon_tags_15 },
    { 0xa4038c75, SUC_lexicon_tags_12 },
    { 0x1d438c7a, SUC_lexicon_tags_17 },
    { 0x68a38c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23c38c7d, SUC_lexicon_tags_45 },
    { 0x71e38c7e, SUC_lexicon_tags_46 },
    { 0xc238c7f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c38c84, SUC_lexicon_tags_8 },
    { 0x81638c85, SUC_lexicon_tags_1 },
    { 0x27838c86, SUC_lexicon_tags_2 },
    { 0xa6a38c86, SUC_lexicon_tags_2 },
    { 0x4f238c88, SUC_lexicon_tags_28 },
    { 0xe9c38c89, SUC_lexicon_tags_3 },
    { 0xc4038c8a, SUC_lexicon_tags_25 },
    { 0x89838c86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf638c90, SUC_lexicon_tags_30 },
    { 0xb9038c90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd438c93, SUC_lexicon_tags_25 },
    { 0x7e638c94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e38c99, SUC_lexicon_tags_284 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b638ca0, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x4e38ca2, SUC_lexicon_tags_25 },
    { 0x56438ca3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3a38ca5, SUC_lexicon_tags_25 },
    { 0x43638ca6, SUC_lexicon_tags_27 },
    { 0x39438ca7, SUC_lexicon_tags_15 },
    { 0x1ba38ca7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10038cac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6838cae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce38cb2, SUC_lexicon_tags_84 },
    { 0xbf638cb3, SUC_lexicon_tags_26 },
    { 0x4d438cb2, SUC_lexicon_tags_17 },
    { 0xd9638cb5, SUC_lexicon_tags_69 },
    { 0xf0638cb4, SUC_lexicon_tags_6 },
    { 0x5c838cb4, SUC_lexicon_tags_12 },
    { 0x9ca38cb8, SUC_lexicon_tags_4 },
    { 0x90638cb9, SUC_lexicon_tags_37 },
    { 0xfac38cb8, SUC_lexicon_tags_1 },
    { 0x5a638cb7, SUC_lexicon_tags_37 },
    { 0xe3a38cb6, SUC_lexicon_tags_33 },
    { 0xe2e38cbd, SUC_lexicon_tags_11 },
    { 0x75a38cbe, SUC_lexicon_tags_34 },
    { 0xf5838cbf, SUC_lexicon_tags_11 },
    { 0xa1638cb9, SUC_lexicon_tags_49 },
    { 0xb7438cb3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8038cc5, SUC_lexicon_tags_81 },
    { 0x50238cc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46838cc8, SUC_lexicon_tags_6 },
    { 0xd5a38cc9, SUC_lexicon_tags_25 },
    { 0xdb438cca, SUC_lexicon_tags_2 },
    { 0x7be38cc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4e38ccd, SUC_lexicon_tags_84 },
    { 0xc7238cce, SUC_lexicon_tags_49 },
    { 0xdfe38ccd, SUC_lexicon_tags_45 },
    { 0xafe38cd0, SUC_lexicon_tags_15 },
    { 0xfaa38cce, SUC_lexicon_tags_31 },
    { 0x39638cd2, SUC_lexicon_tags_9 },
    { 0xf5438cd3, SUC_lexicon_tags_9 },
    { 0xb7438cd3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5238cd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ba38cdb, SUC_lexicon_tags_87 },
    { 0x6c838cdc, SUC_lexicon_tags_17 },
    { 0xe9c38cdd, SUC_lexicon_tags_7 },
    { 0xff838cde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a38ce1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd438ce3, SUC_lexicon_tags_12 },
    { 0xfe838ce4, SUC_lexicon_tags_77 },
    { 0x12438ce3, SUC_lexicon_tags_57 },
    { 0x29838ce6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c38ce9, SUC_lexicon_tags_0 },
    { 0x60c38ce9, SUC_lexicon_tags_19 },
    { 0xf7e38cea, SUC_lexicon_tags_65 },
    { 0x5a838ce9, SUC_lexicon_tags_37 },
    { 0x4f038ced, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f438cf0, SUC_lexicon_tags_7 },
    { 0x23438cf1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x74638cf4, SUC_lexicon_tags_74 },
    { 0xa6838cf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a438cfd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25638d01, SUC_lexicon_tags_1 },
    { 0xb3438d02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61838d14, SUC_lexicon_tags_2 },
    { 0xd4a38d15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c38d18, SUC_lexicon_tags_37 },
    { 0x54438d19, SUC_lexicon_tags_31 },
    { 0xb2438d19, SUC_lexicon_tags_4 },
    { 0xf8438d1b, SUC_lexicon_tags_2 },
    { 0x8638d1c, SUC_lexicon_tags_27 },
    { 0x7d238d1d, SUC_lexicon_tags_30 },
    { 0xfba38d1b, SUC_lexicon_tags_1 },
    { 0x31638d1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9838d26, SUC_lexicon_tags_11 },
    { 0x30038d26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa38d2b, SUC_lexicon_tags_38 },
    { 0x48238d2b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf0c38d2e, SUC_lexicon_tags_37 },
    { 0xd1038d2f, SUC_lexicon_tags_12 },
    { 0xba638d30, SUC_lexicon_tags_60 },
    { 0x43a38d31, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x71438d33, SUC_lexicon_tags_15 },
    { 0xe9e38d34, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad438d3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x70438d3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e38d48, SUC_lexicon_tags_10 },
    { 0x17238d49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45c38d4b, SUC_lexicon_tags_25 },
    { 0x8638d4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe38d54, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd1438d56, SUC_lexicon_tags_64 },
    { 0x10c38d57, SUC_lexicon_tags_17 },
    { 0x5e638d58, SUC_lexicon_tags_43 },
    { 0x12e38d59, SUC_lexicon_tags_17 },
    { 0x6ec38d5a, SUC_lexicon_tags_28 },
    { 0xaf838d5a, SUC_lexicon_tags_11 },
    { 0x50038d5b, SUC_lexicon_tags_11 },
    { 0x2f438d56, SUC_lexicon_tags_39 },
    { 0x3638d5e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9038d65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a38d69, SUC_lexicon_tags_25 },
    { 0x21638d6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21038d76, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x87e38d78, SUC_lexicon_tags_17 },
    { 0x92638d79, SUC_lexicon_tags_17 },
    { 0x2f438d79, SUC_lexicon_tags_14 },
    { 0xd6838d7a, SUC_lexicon_tags_14 },
    { 0xe9238d7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c38d80, SUC_lexicon_tags_46 },
    { 0x8fe38d80, SUC_lexicon_tags_19 },
    { 0x9a638d82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37638d86, SUC_lexicon_tags_2 },
    { 0xef238d86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x63038d89, SUC_lexicon_tags_37 },
    { 0x75e38d8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x28038d8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65638d90, SUC_lexicon_tags_30 },
    { 0x1c38d90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1438d94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f238d96, SUC_lexicon_tags_41 },
    { 0x88838d97, SUC_lexicon_tags_1 },
    { 0xa5e38d98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa38d9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d838dab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6638dae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88638db3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfd438db5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c38dba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb638dbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26a38dbf, SUC_lexicon_tags_25 },
    { 0xe2638dc0, SUC_lexicon_tags_7 },
    { 0xc2038dc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc38dc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e38dc9, SUC_lexicon_tags_17 },
    { 0x14438dca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1238dcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc238dd0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x24438dd2, SUC_lexicon_tags_12 },
    { 0x72838dd3, SUC_lexicon_tags_47 },
    { 0xccc38dd4, SUC_lexicon_tags_17 },
    { 0xc3c38dd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77238dd8, SUC_lexicon_tags_43 },
    { 0x69a38dd9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7838ddf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e38de9, SUC_lexicon_tags_7 },
    { 0xa2638dea, SUC_lexicon_tags_11 },
    { 0x7ec38de9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e38dee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4438df3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47438df7, SUC_lexicon_tags_31 },
    { 0x8ec38df8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe38dff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a238e02, SUC_lexicon_tags_43 },
    { 0x26438e03, SUC_lexicon_tags_19 },
    { 0x7b838e04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19238e06, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe7c38e08, SUC_lexicon_tags_2 },
    { 0x55038e08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9238e0b, SUC_lexicon_tags_37 },
    { 0x97c38e0c, SUC_lexicon_tags_43 },
    { 0xad238e0c, SUC_lexicon_tags_48 },
    { 0x6a438e0d, SUC_lexicon_tags_0 },
    { 0x56838e0c, SUC_lexicon_tags_12 },
    { 0x11a38e10, SUC_lexicon_tags_11 },
    { 0x9ac38e11, SUC_lexicon_tags_63 },
    { 0xc4638e12, SUC_lexicon_tags_29 },
    { 0x2b238e13, SUC_lexicon_tags_25 },
    { 0x36e38e12, SUC_lexicon_tags_2 },
    { 0xf3038e15, SUC_lexicon_tags_11 },
    { 0x67238e16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe38e19, SUC_lexicon_tags_49 },
    { 0x95638e1a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4038e1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95638e21, SUC_lexicon_tags_179 },
    { 0x1e38e22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a38e29, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x64e38e2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb6e38e2d, SUC_lexicon_tags_25 },
    { 0xe3a38e2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30038e30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5038e32, SUC_lexicon_tags_61 },
    { 0xa8438e33, SUC_lexicon_tags_35 },
    { 0x74e38e32, SUC_lexicon_tags_999 },
    { 0, NULL },
    { 0xa1238e36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c638e39, SUC_lexicon_tags_18 },
    { 0xeea38e3a, SUC_lexicon_tags_43 },
    { 0x2ac38e3b, SUC_lexicon_tags_10 },
    { 0x2f038e3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12838e3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42438e40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80238e42, SUC_lexicon_tags_12 },
    { 0xb1838e43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x70438e46, SUC_lexicon_tags_25 },
    { 0xf8638e46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94438e4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31838e4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae438e56, SUC_lexicon_tags_12 },
    { 0x10238e57, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e638e5c, SUC_lexicon_tags_23 },
    { 0x51838e5d, SUC_lexicon_tags_27 },
    { 0x5aa38e5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee38e61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14438e65, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa38e68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39238e6a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca838e6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d638e70, SUC_lexicon_tags_37 },
    { 0xebc38e71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e38e74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca838e7a, SUC_lexicon_tags_37 },
    { 0xccc38e7b, SUC_lexicon_tags_46 },
    { 0xec838e7a, SUC_lexicon_tags_14 },
    { 0x3d038e7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa38e80, SUC_lexicon_tags_27 },
    { 0x25a38e81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e38e83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e38e87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ac38e89, SUC_lexicon_tags_61 },
    { 0xd4c38e89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37038e8c, SUC_lexicon_tags_151 },
    { 0x64e38e8d, SUC_lexicon_tags_45 },
    { 0xe9038e8e, SUC_lexicon_tags_47 },
    { 0xf7a38e8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4438e99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50038e9d, SUC_lexicon_tags_40 },
    { 0xe3638e9e, SUC_lexicon_tags_7 },
    { 0x3e238e9f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23238ea5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a38eac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39038eae, SUC_lexicon_tags_0 },
    { 0xe8238eaf, SUC_lexicon_tags_15 },
    { 0x89c38eae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e38eb3, SUC_lexicon_tags_21 },
    { 0x30638eb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4438eb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee038ebd, SUC_lexicon_tags_14 },
    { 0x98038ebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea638ec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9838ec6, SUC_lexicon_tags_46 },
    { 0xdfc38ec6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x25a38ec9, SUC_lexicon_tags_33 },
    { 0xafc38eca, SUC_lexicon_tags_9 },
    { 0xebc38ecb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa38ed2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98438ed4, SUC_lexicon_tags_87 },
    { 0x2f038ed4, SUC_lexicon_tags_23 },
    { 0x8b238ed5, SUC_lexicon_tags_4 },
    { 0xaf238ed7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc38eda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59e38edc, SUC_lexicon_tags_1 },
    { 0x34438edd, SUC_lexicon_tags_33 },
    { 0x1b038ede, SUC_lexicon_tags_38 },
    { 0x7b838edf, SUC_lexicon_tags_65 },
    { 0x64a38ee0, SUC_lexicon_tags_14 },
    { 0xca238ee1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8838ee4, SUC_lexicon_tags_28 },
    { 0x53838ee5, SUC_lexicon_tags_77 },
    { 0x3a38ee6, SUC_lexicon_tags_45 },
    { 0xee638ee7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x94838eea, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba38eed, SUC_lexicon_tags_37 },
    { 0x1f438eee, SUC_lexicon_tags_1 },
    { 0x17e38eef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e838ef3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f638efb, SUC_lexicon_tags_2 },
    { 0xce038efc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80038efe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae38f04, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec438f08, SUC_lexicon_tags_23 },
    { 0x5a438f08, SUC_lexicon_tags_54 },
    { 0x3d638f0a, SUC_lexicon_tags_6 },
    { 0xe3c38f0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c638f0d, SUC_lexicon_tags_8 },
    { 0xa1c38f0d, SUC_lexicon_tags_15 },
    { 0xdce38f0f, SUC_lexicon_tags_48 },
    { 0xc4838f0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35838f12, SUC_lexicon_tags_43 },
    { 0x3f838f13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf238f16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3238f18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13438f1c, SUC_lexicon_tags_7 },
    { 0x20838f1c, SUC_lexicon_tags_77 },
    { 0x99438f1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a438f23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc3038f25, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf0038f27, SUC_lexicon_tags_44 },
    { 0x2a438f28, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6838f2b, SUC_lexicon_tags_282 },
    { 0x76038f2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc38f30, SUC_lexicon_tags_11 },
    { 0x7a838f31, SUC_lexicon_tags_14 },
    { 0xbd838f32, SUC_lexicon_tags_12 },
    { 0xa9838f33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4438f35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff638f37, SUC_lexicon_tags_25 },
    { 0x40838f37, SUC_lexicon_tags_64 },
    { 0x36838f39, SUC_lexicon_tags_28 },
    { 0xc4e38f3a, SUC_lexicon_tags_37 },
    { 0x3b038f38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38f3d, SUC_lexicon_tags_12 },
    { 0x75238f3d, SUC_lexicon_tags_1 },
    { 0x14038f3f, SUC_lexicon_tags_37 },
    { 0x99e38f40, SUC_lexicon_tags_2 },
    { 0xae438f3f, SUC_lexicon_tags_15 },
    { 0x65e38f41, SUC_lexicon_tags_11 },
    { 0x65a38f3f, SUC_lexicon_tags_151 },
    { 0x35638f44, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2a638f46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1838f48, SUC_lexicon_tags_15 },
    { 0x7a038f49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c38f4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x438f50, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa7038f52, SUC_lexicon_tags_23 },
    { 0x56438f53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a38f5d, SUC_lexicon_tags_46 },
    { 0xe7238f5e, SUC_lexicon_tags_26 },
    { 0xf9c38f5d, SUC_lexicon_tags_17 },
    { 0xd2838f60, SUC_lexicon_tags_19 },
    { 0x74e38f61, SUC_lexicon_tags_12 },
    { 0xd038f5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c38f68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c38f6c, SUC_lexicon_tags_19 },
    { 0xe9438f6d, SUC_lexicon_tags_26 },
    { 0xbdc38f6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43a38f70, SUC_lexicon_tags_1 },
    { 0x71838f71, SUC_lexicon_tags_33 },
    { 0x5e638f71, SUC_lexicon_tags_25 },
    { 0x4d238f73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x50a38f75, SUC_lexicon_tags_9 },
    { 0xcbc38f76, SUC_lexicon_tags_11 },
    { 0x74838f76, SUC_lexicon_tags_27 },
    { 0xc0c38f75, SUC_lexicon_tags_36 },
    { 0x9ee38f75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x70038f7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc838f82, SUC_lexicon_tags_2 },
    { 0x48c38f83, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50038f88, SUC_lexicon_tags_11 },
    { 0x8a38f89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34838f8d, SUC_lexicon_tags_27 },
    { 0xe0838f8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ec38f90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaca38f92, SUC_lexicon_tags_12 },
    { 0x9f438f93, SUC_lexicon_tags_14 },
    { 0x3238f93, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71438f96, SUC_lexicon_tags_38 },
    { 0x19c38f97, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec38f9d, SUC_lexicon_tags_15 },
    { 0x6aa38f9e, SUC_lexicon_tags_2 },
    { 0xb7a38f9f, SUC_lexicon_tags_63 },
    { 0xc6638fa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8038fa6, SUC_lexicon_tags_11 },
    { 0x4f838fa6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8438faa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc38fad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23038fb3, SUC_lexicon_tags_8 },
    { 0x98438fb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a038fb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48238fba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be38fbd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce38fc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8038fc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba638fc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d038fd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23038fd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaaa38fd8, SUC_lexicon_tags_1 },
    { 0x34038fd8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x35838fdc, SUC_lexicon_tags_7 },
    { 0x35c38fdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45638fe0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc38fe5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd438fe8, SUC_lexicon_tags_53 },
    { 0x1ce38fe8, SUC_lexicon_tags_47 },
    { 0xf0238fe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49e38fec, SUC_lexicon_tags_45 },
    { 0x50238fec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2438ff2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50838ff5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd6838ff7, SUC_lexicon_tags_49 },
    { 0x8be38ff8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde238fff, SUC_lexicon_tags_1211 },
    { 0, NULL },
    { 0, NULL },
    { 0x67439002, SUC_lexicon_tags_33 },
    { 0xb3c39003, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x40839006, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a3900f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c39012, SUC_lexicon_tags_10 },
    { 0x74c39012, SUC_lexicon_tags_2 },
    { 0x72639014, SUC_lexicon_tags_15 },
    { 0x47439015, SUC_lexicon_tags_0 },
    { 0x81239015, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc039019, SUC_lexicon_tags_14 },
    { 0x94439019, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b63901c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7c63901e, SUC_lexicon_tags_37 },
    { 0x54a3901e, SUC_lexicon_tags_2 },
    { 0x543901e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc783902b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe3a3902d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e39030, SUC_lexicon_tags_23 },
    { 0x1d439031, SUC_lexicon_tags_43 },
    { 0x6239032, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7323903b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc683903f, SUC_lexicon_tags_52 },
    { 0x29c3903f, SUC_lexicon_tags_60 },
    { 0xfe39041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca239045, SUC_lexicon_tags_47 },
    { 0x92a39045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103904b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe439052, SUC_lexicon_tags_45 },
    { 0xdf639053, SUC_lexicon_tags_12 },
    { 0x2039054, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c639058, SUC_lexicon_tags_12 },
    { 0xc0039059, SUC_lexicon_tags_15 },
    { 0x7ce3905a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a3905e, SUC_lexicon_tags_6 },
    { 0x9c03905e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb1c39061, SUC_lexicon_tags_11 },
    { 0x48439062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47a39064, SUC_lexicon_tags_7 },
    { 0x32639065, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x70439067, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x79c39069, SUC_lexicon_tags_25 },
    { 0x47a39069, SUC_lexicon_tags_14 },
    { 0xb563906b, SUC_lexicon_tags_14 },
    { 0x3043906a, SUC_lexicon_tags_17 },
    { 0x3183906d, SUC_lexicon_tags_12 },
    { 0x9763906e, SUC_lexicon_tags_87 },
    { 0x66c3906f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca639076, SUC_lexicon_tags_6 },
    { 0xfe439076, SUC_lexicon_tags_53 },
    { 0xeb239077, SUC_lexicon_tags_17 },
    { 0x49439079, SUC_lexicon_tags_4 },
    { 0x1143907a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xff03907c, SUC_lexicon_tags_14 },
    { 0x3e83907d, SUC_lexicon_tags_2 },
    { 0x4cc3907e, SUC_lexicon_tags_7 },
    { 0xbe43907f, SUC_lexicon_tags_14 },
    { 0x6d439080, SUC_lexicon_tags_15 },
    { 0xcf039081, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa239083, SUC_lexicon_tags_87 },
    { 0x18039084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe123908f, SUC_lexicon_tags_13 },
    { 0x4da3908f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa439094, SUC_lexicon_tags_66 },
    { 0x7a439094, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88390a6, SUC_lexicon_tags_7 },
    { 0x6f4390a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a0390a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a390ad, SUC_lexicon_tags_43 },
    { 0x7fa390ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6a390b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9b4390b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0390ba, SUC_lexicon_tags_11 },
    { 0x95c390bb, SUC_lexicon_tags_27 },
    { 0xd78390bb, SUC_lexicon_tags_85 },
    { 0xd14390bc, SUC_lexicon_tags_60 },
    { 0xaae390be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc8390c0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8390cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79e390ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdac390d0, SUC_lexicon_tags_15 },
    { 0x922390d1, SUC_lexicon_tags_6 },
    { 0x2a0390d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c390d7, SUC_lexicon_tags_7 },
    { 0x120390d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e390db, SUC_lexicon_tags_282 },
    { 0x6f6390dc, SUC_lexicon_tags_12 },
    { 0x4a6390dc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664390e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e390e5, SUC_lexicon_tags_33 },
    { 0x9e2390e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2390ec, SUC_lexicon_tags_21 },
    { 0x8c2390ec, SUC_lexicon_tags_1 },
    { 0xefe390ec, SUC_lexicon_tags_26 },
    { 0xc44390ef, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x438390f1, SUC_lexicon_tags_25 },
    { 0xa62390f2, SUC_lexicon_tags_23 },
    { 0x902390f3, SUC_lexicon_tags_193 },
    { 0x7e0390f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc44390f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3c390f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e390fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35039100, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19039109, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa3910c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b43910f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38c39111, SUC_lexicon_tags_15 },
    { 0x68039112, SUC_lexicon_tags_37 },
    { 0x17439112, SUC_lexicon_tags_37 },
    { 0x1c039113, SUC_lexicon_tags_2 },
    { 0x28439111, SUC_lexicon_tags_66 },
    { 0x3ee39116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3943911a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x103911d, SUC_lexicon_tags_63 },
    { 0xcc83911e, SUC_lexicon_tags_37 },
    { 0x8483911f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e839122, SUC_lexicon_tags_15 },
    { 0x26439123, SUC_lexicon_tags_23 },
    { 0xf0439123, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x41239127, SUC_lexicon_tags_11 },
    { 0x40439127, SUC_lexicon_tags_47 },
    { 0xc9c39129, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40e3912b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6883912e, SUC_lexicon_tags_34 },
    { 0x7de3912e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x40039132, SUC_lexicon_tags_62 },
    { 0xcd239133, SUC_lexicon_tags_63 },
    { 0x58439132, SUC_lexicon_tags_17 },
    { 0x29239134, SUC_lexicon_tags_1 },
    { 0xd4e39136, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec39139, SUC_lexicon_tags_23 },
    { 0xad43913a, SUC_lexicon_tags_47 },
    { 0x55a3913b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf63913d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80239142, SUC_lexicon_tags_27 },
    { 0xddc39142, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9e839145, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e3914a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f039151, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66039154, SUC_lexicon_tags_12 },
    { 0x86039155, SUC_lexicon_tags_17 },
    { 0x9c839156, SUC_lexicon_tags_14 },
    { 0x6e239157, SUC_lexicon_tags_17 },
    { 0xa1639156, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd083915b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd243915e, SUC_lexicon_tags_15 },
    { 0xe043915e, SUC_lexicon_tags_25 },
    { 0xa4439160, SUC_lexicon_tags_127 },
    { 0xec839161, SUC_lexicon_tags_55 },
    { 0x94439161, SUC_lexicon_tags_37 },
    { 0xe383915e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26439166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53039168, SUC_lexicon_tags_45 },
    { 0x5a439169, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc03916e, SUC_lexicon_tags_0 },
    { 0x983916e, SUC_lexicon_tags_7 },
    { 0x54a39170, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea39174, SUC_lexicon_tags_15 },
    { 0xa8a39174, SUC_lexicon_tags_9 },
    { 0xa7a39176, SUC_lexicon_tags_11 },
    { 0x23839177, SUC_lexicon_tags_65 },
    { 0xb1639178, SUC_lexicon_tags_2 },
    { 0xf8c39178, SUC_lexicon_tags_15 },
    { 0xa963917a, SUC_lexicon_tags_25 },
    { 0xc7639178, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc443917d, SUC_lexicon_tags_79 },
    { 0xb503917e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a39183, SUC_lexicon_tags_25 },
    { 0x10639183, SUC_lexicon_tags_1 },
    { 0x5ee39185, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663918c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc323918e, SUC_lexicon_tags_23 },
    { 0xf523918f, SUC_lexicon_tags_47 },
    { 0x82a39190, SUC_lexicon_tags_37 },
    { 0xb803918e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a39195, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee39198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9c3919a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4391a0, SUC_lexicon_tags_15 },
    { 0x9c4391a1, SUC_lexicon_tags_1 },
    { 0x842391a2, SUC_lexicon_tags_7 },
    { 0x674391a3, SUC_lexicon_tags_2 },
    { 0xd58391a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2391a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8391ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5a391ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa391b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c391b7, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0x656391b9, SUC_lexicon_tags_17 },
    { 0x642391b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x702391bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93a391be, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf26391c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb6391c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4391c5, SUC_lexicon_tags_25 },
    { 0x42a391c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258391ca, SUC_lexicon_tags_7 },
    { 0x550391cb, SUC_lexicon_tags_15 },
    { 0xc2391cc, SUC_lexicon_tags_68 },
    { 0x998391cb, SUC_lexicon_tags_0 },
    { 0x3e8391ce, SUC_lexicon_tags_37 },
    { 0xe1e391cf, SUC_lexicon_tags_46 },
    { 0x391d0, SUC_lexicon_tags_23 },
    { 0x640391d1, SUC_lexicon_tags_0 },
    { 0xfa391d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2391de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e391e1, SUC_lexicon_tags_47 },
    { 0x60a391e2, SUC_lexicon_tags_37 },
    { 0xee2391e3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c391e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04391ed, SUC_lexicon_tags_45 },
    { 0xc72391ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf60391f0, SUC_lexicon_tags_32 },
    { 0x47e391f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652391f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc12391f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fc391fb, SUC_lexicon_tags_17 },
    { 0x2b0391fb, SUC_lexicon_tags_27 },
    { 0x1be391fd, SUC_lexicon_tags_1 },
    { 0x9e391fe, SUC_lexicon_tags_44 },
    { 0x98a391fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50639202, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75439207, SUC_lexicon_tags_17 },
    { 0x63039207, SUC_lexicon_tags_23 },
    { 0x4c439209, SUC_lexicon_tags_32 },
    { 0x1ba39209, SUC_lexicon_tags_32 },
    { 0xb063920b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8483920d, SUC_lexicon_tags_14 },
    { 0x5c63920e, SUC_lexicon_tags_12 },
    { 0x96a3920e, SUC_lexicon_tags_7 },
    { 0x2b03920f, SUC_lexicon_tags_17 },
    { 0xe6639211, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5439214, SUC_lexicon_tags_31 },
    { 0x76239215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff239218, SUC_lexicon_tags_1 },
    { 0x7f239218, SUC_lexicon_tags_41 },
    { 0x23439218, SUC_lexicon_tags_27 },
    { 0x9923921b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe639226, SUC_lexicon_tags_12 },
    { 0xb6c39227, SUC_lexicon_tags_23 },
    { 0xe2239227, SUC_lexicon_tags_56 },
    { 0x70c39229, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4823922b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3922e, SUC_lexicon_tags_1 },
    { 0xc983922f, SUC_lexicon_tags_26 },
    { 0xd0a39230, SUC_lexicon_tags_27 },
    { 0xc8239231, SUC_lexicon_tags_68 },
    { 0x16639232, SUC_lexicon_tags_15 },
    { 0x76e39233, SUC_lexicon_tags_28 },
    { 0x9c439232, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c39239, SUC_lexicon_tags_65 },
    { 0x3543923a, SUC_lexicon_tags_79 },
    { 0x1a83923a, SUC_lexicon_tags_28 },
    { 0xef03923c, SUC_lexicon_tags_17 },
    { 0x4b43923b, SUC_lexicon_tags_60 },
    { 0xbe23923c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c39243, SUC_lexicon_tags_26 },
    { 0x82a39244, SUC_lexicon_tags_46 },
    { 0xa8a39245, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce39252, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e39257, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17239262, SUC_lexicon_tags_4 },
    { 0x34039263, SUC_lexicon_tags_85 },
    { 0x6bc39264, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8039266, SUC_lexicon_tags_7 },
    { 0x3ea39266, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e3926d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c039270, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1839276, SUC_lexicon_tags_15 },
    { 0xf7a39277, SUC_lexicon_tags_25 },
    { 0x3839276, SUC_lexicon_tags_37 },
    { 0x9da39279, SUC_lexicon_tags_105 },
    { 0x8e239278, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9763927c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4863927e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c39281, SUC_lexicon_tags_27 },
    { 0xcec39282, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x46639285, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20039289, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1743928f, SUC_lexicon_tags_32 },
    { 0x41a39290, SUC_lexicon_tags_23 },
    { 0x76a39291, SUC_lexicon_tags_37 },
    { 0xa2439291, SUC_lexicon_tags_14 },
    { 0x3fe39293, SUC_lexicon_tags_60 },
    { 0x25a39290, SUC_lexicon_tags_33 },
    { 0x89839295, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38439298, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9c23929a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5543929c, SUC_lexicon_tags_7 },
    { 0x9603929d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894392a3, SUC_lexicon_tags_2 },
    { 0x98c392a3, SUC_lexicon_tags_24 },
    { 0x3d8392a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x298392a7, SUC_lexicon_tags_13 },
    { 0xce4392a8, SUC_lexicon_tags_43 },
    { 0xfc2392a9, SUC_lexicon_tags_14 },
    { 0x8b6392a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be392ad, SUC_lexicon_tags_12 },
    { 0xce4392ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x16392b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a392b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe4a392b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1b4392b7, SUC_lexicon_tags_17 },
    { 0xe5a392b8, SUC_lexicon_tags_11 },
    { 0x248392b9, SUC_lexicon_tags_34 },
    { 0xe8e392ba, SUC_lexicon_tags_1 },
    { 0x118392ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e392c0, SUC_lexicon_tags_15 },
    { 0xc36392c1, SUC_lexicon_tags_33 },
    { 0x6b4392c2, SUC_lexicon_tags_269 },
    { 0x3e6392c0, SUC_lexicon_tags_63 },
    { 0xbf4392c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4392cd, SUC_lexicon_tags_2 },
    { 0xa18392ce, SUC_lexicon_tags_55 },
    { 0x546392cf, SUC_lexicon_tags_80 },
    { 0xf54392d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26392d9, SUC_lexicon_tags_15 },
    { 0xa7c392da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38392dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84392e3, SUC_lexicon_tags_2 },
    { 0x78e392e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x308392e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1e2392e8, SUC_lexicon_tags_23 },
    { 0x7ee392e9, SUC_lexicon_tags_40 },
    { 0x410392ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2392ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x56a392f1, SUC_lexicon_tags_13 },
    { 0x118392f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc22392f4, SUC_lexicon_tags_37 },
    { 0xe0a392f5, SUC_lexicon_tags_37 },
    { 0x37e392f6, SUC_lexicon_tags_2 },
    { 0xa58392f7, SUC_lexicon_tags_77 },
    { 0x7e4392f8, SUC_lexicon_tags_11 },
    { 0x388392f6, SUC_lexicon_tags_65 },
    { 0xa70392f9, SUC_lexicon_tags_26 },
    { 0x392f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab6392fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x130392ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f239301, SUC_lexicon_tags_26 },
    { 0x9aa39302, SUC_lexicon_tags_60 },
    { 0x9ee39301, SUC_lexicon_tags_1 },
    { 0x6de39303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc39306, SUC_lexicon_tags_12 },
    { 0xe8239307, SUC_lexicon_tags_15 },
    { 0xd7c39308, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe623930b, SUC_lexicon_tags_23 },
    { 0x7aa3930c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e3930f, SUC_lexicon_tags_12 },
    { 0x9fa39310, SUC_lexicon_tags_11 },
    { 0x8143930f, SUC_lexicon_tags_26 },
    { 0xf8439312, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0039317, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc83931a, SUC_lexicon_tags_7 },
    { 0xd8a3931b, SUC_lexicon_tags_37 },
    { 0xa163931b, SUC_lexicon_tags_2 },
    { 0xa6a3931d, SUC_lexicon_tags_47 },
    { 0x1283931d, SUC_lexicon_tags_40 },
    { 0x9823931b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79839323, SUC_lexicon_tags_26 },
    { 0x79639324, SUC_lexicon_tags_25 },
    { 0x96c39325, SUC_lexicon_tags_4 },
    { 0xc1039326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb303932b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9563932e, SUC_lexicon_tags_23 },
    { 0xec83932e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee39332, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7239334, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85439338, SUC_lexicon_tags_133 },
    { 0x60439338, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8143933f, SUC_lexicon_tags_9 },
    { 0xdd639340, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa439342, SUC_lexicon_tags_12 },
    { 0x49839343, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7639347, SUC_lexicon_tags_1 },
    { 0x69639348, SUC_lexicon_tags_30 },
    { 0x22a39348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede3934e, SUC_lexicon_tags_37 },
    { 0x65c3934f, SUC_lexicon_tags_26 },
    { 0xcc639350, SUC_lexicon_tags_37 },
    { 0x68439351, SUC_lexicon_tags_60 },
    { 0x86439350, SUC_lexicon_tags_74 },
    { 0x45a39353, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5039355, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87439359, SUC_lexicon_tags_35 },
    { 0xd7e39359, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0xd643935d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a39363, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c39366, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xff439369, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1a3936b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e39371, SUC_lexicon_tags_8 },
    { 0xc0e39372, SUC_lexicon_tags_19 },
    { 0x7c439373, SUC_lexicon_tags_7 },
    { 0xe639374, SUC_lexicon_tags_27 },
    { 0xc5839375, SUC_lexicon_tags_15 },
    { 0x66039374, SUC_lexicon_tags_11 },
    { 0xdae39371, SUC_lexicon_tags_15 },
    { 0x34c39378, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3843937c, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d239382, SUC_lexicon_tags_17 },
    { 0xdb239383, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1839385, SUC_lexicon_tags_17 },
    { 0x86c39385, SUC_lexicon_tags_63 },
    { 0x56439385, SUC_lexicon_tags_8 },
    { 0x28e39388, SUC_lexicon_tags_17 },
    { 0x48839388, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b23938c, SUC_lexicon_tags_11 },
    { 0x62c3938d, SUC_lexicon_tags_17 },
    { 0x31c3938c, SUC_lexicon_tags_1 },
    { 0xc523938f, SUC_lexicon_tags_25 },
    { 0x30a39390, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81639394, SUC_lexicon_tags_6 },
    { 0xf2639395, SUC_lexicon_tags_1 },
    { 0x9ac39396, SUC_lexicon_tags_1 },
    { 0x8b439397, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4d839399, SUC_lexicon_tags_27 },
    { 0xdfc3939a, SUC_lexicon_tags_12 },
    { 0x75e3939b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02393a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x370393a8, SUC_lexicon_tags_37 },
    { 0x78a393a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96393af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8393b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c393b9, SUC_lexicon_tags_48 },
    { 0x1f4393ba, SUC_lexicon_tags_2 },
    { 0xf90393bb, SUC_lexicon_tags_19 },
    { 0xbc393bc, SUC_lexicon_tags_17 },
    { 0xe74393bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x44a393bf, SUC_lexicon_tags_1 },
    { 0x390393c0, SUC_lexicon_tags_31 },
    { 0x58393c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd90393c4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558393c8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8393cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104393e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e393e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a393e8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2393eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe94393ed, SUC_lexicon_tags_60 },
    { 0x400393ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0393f1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x738393f3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a393f6, SUC_lexicon_tags_1 },
    { 0xe6c393f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0393fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d239400, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde39403, SUC_lexicon_tags_351 },
    { 0xc839404, SUC_lexicon_tags_11 },
    { 0x1a39405, SUC_lexicon_tags_37 },
    { 0xe6c39404, SUC_lexicon_tags_43 },
    { 0xe4439403, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x7583940a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a3940d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32439418, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6383941b, SUC_lexicon_tags_47 },
    { 0x2263941c, SUC_lexicon_tags_16 },
    { 0x8723941c, SUC_lexicon_tags_37 },
    { 0x8cc3941e, SUC_lexicon_tags_12 },
    { 0x5a43941f, SUC_lexicon_tags_35 },
    { 0xd7e3941f, SUC_lexicon_tags_1 },
    { 0xf083941c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe39425, SUC_lexicon_tags_63 },
    { 0x69a39426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c39429, SUC_lexicon_tags_39 },
    { 0xa0e3942a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc263942d, SUC_lexicon_tags_25 },
    { 0x1163942d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb839431, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad239436, SUC_lexicon_tags_15 },
    { 0xa1039437, SUC_lexicon_tags_14 },
    { 0x32e39438, SUC_lexicon_tags_8 },
    { 0x27839439, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5803943e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a39442, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x91239445, SUC_lexicon_tags_2 },
    { 0x7a239446, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x88439448, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be39450, SUC_lexicon_tags_15 },
    { 0x43039451, SUC_lexicon_tags_9 },
    { 0x43a39452, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa39458, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8763945b, SUC_lexicon_tags_33 },
    { 0x5de3945c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1843945f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d439462, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80639464, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd5e39466, SUC_lexicon_tags_11 },
    { 0x56639466, SUC_lexicon_tags_43 },
    { 0x23639468, SUC_lexicon_tags_82 },
    { 0x32c39469, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3946d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf163946f, SUC_lexicon_tags_45 },
    { 0xcd639470, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x68639472, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x77c39474, SUC_lexicon_tags_55 },
    { 0x6f039474, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad839478, SUC_lexicon_tags_15 },
    { 0xc839479, SUC_lexicon_tags_25 },
    { 0x5523947a, SUC_lexicon_tags_17 },
    { 0xcc23947b, SUC_lexicon_tags_25 },
    { 0xea23947c, SUC_lexicon_tags_15 },
    { 0xde83947b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a39481, SUC_lexicon_tags_1 },
    { 0x45e39481, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d039484, SUC_lexicon_tags_27 },
    { 0x94a39484, SUC_lexicon_tags_15 },
    { 0x44e39485, SUC_lexicon_tags_15 },
    { 0x88439487, SUC_lexicon_tags_7 },
    { 0xf6639485, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e39491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba439496, SUC_lexicon_tags_1 },
    { 0x23039496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36239499, SUC_lexicon_tags_38 },
    { 0x11c3949a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e3949c, SUC_lexicon_tags_19 },
    { 0x8823949d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4394a0, SUC_lexicon_tags_17 },
    { 0xd9c394a1, SUC_lexicon_tags_49 },
    { 0xa4394a1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac394a9, SUC_lexicon_tags_11 },
    { 0xbb0394a9, SUC_lexicon_tags_35 },
    { 0x5dc394ab, SUC_lexicon_tags_21 },
    { 0xfe8394aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee394b0, SUC_lexicon_tags_14 },
    { 0x530394b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a394bf, SUC_lexicon_tags_31 },
    { 0xd70394c0, SUC_lexicon_tags_23 },
    { 0x23e394bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186394ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0394cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518394d5, SUC_lexicon_tags_29 },
    { 0xb1c394d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78394d9, SUC_lexicon_tags_25 },
    { 0x568394da, SUC_lexicon_tags_0 },
    { 0x122394db, SUC_lexicon_tags_15 },
    { 0x6ee394db, SUC_lexicon_tags_23 },
    { 0xe0394dd, SUC_lexicon_tags_15 },
    { 0x6bc394de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50394e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c394e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2394e7, SUC_lexicon_tags_66 },
    { 0x6a2394e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x634394ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50394ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8394f0, SUC_lexicon_tags_19 },
    { 0x2d4394f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06394f7, SUC_lexicon_tags_21 },
    { 0x2a8394f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e394ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x42439502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1639504, SUC_lexicon_tags_12 },
    { 0x76039505, SUC_lexicon_tags_14 },
    { 0x62239504, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc439508, SUC_lexicon_tags_15 },
    { 0xe3039509, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e3950d, SUC_lexicon_tags_60 },
    { 0x3b03950d, SUC_lexicon_tags_11 },
    { 0x9c3950d, SUC_lexicon_tags_14 },
    { 0xfa03950f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x45839512, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a39515, SUC_lexicon_tags_30 },
    { 0xfe639516, SUC_lexicon_tags_7 },
    { 0x95239517, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd3c39519, SUC_lexicon_tags_37 },
    { 0xf223951a, SUC_lexicon_tags_7 },
    { 0x9203951b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e43951d, SUC_lexicon_tags_17 },
    { 0x2523951e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21039526, SUC_lexicon_tags_38 },
    { 0xc4439527, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa43952b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6be3952d, SUC_lexicon_tags_2 },
    { 0xc183952e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e039530, SUC_lexicon_tags_14 },
    { 0x10a39531, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f039537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e3953b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71239540, SUC_lexicon_tags_7 },
    { 0x69e39541, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2239547, SUC_lexicon_tags_37 },
    { 0xfde39548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc3954e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e39554, SUC_lexicon_tags_60 },
    { 0x14639554, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc39558, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e3955b, SUC_lexicon_tags_2 },
    { 0xd3e3955c, SUC_lexicon_tags_2 },
    { 0xf803955d, SUC_lexicon_tags_81 },
    { 0x55e3955e, SUC_lexicon_tags_57 },
    { 0xa323955f, SUC_lexicon_tags_63 },
    { 0xfc23955f, SUC_lexicon_tags_33 },
    { 0xf703955f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6a39563, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfda39565, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4e639567, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e239569, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7783956b, SUC_lexicon_tags_11 },
    { 0x8183956b, SUC_lexicon_tags_15 },
    { 0xe7a3956b, SUC_lexicon_tags_215 },
    { 0x9c03956e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe1c39570, SUC_lexicon_tags_12 },
    { 0x1e39571, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e39575, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb439578, SUC_lexicon_tags_38 },
    { 0x15239578, SUC_lexicon_tags_12 },
    { 0xbf439578, SUC_lexicon_tags_11 },
    { 0xbf23957b, SUC_lexicon_tags_38 },
    { 0xfb83957c, SUC_lexicon_tags_91 },
    { 0xd963957c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd839580, SUC_lexicon_tags_13 },
    { 0xc0639580, SUC_lexicon_tags_7 },
    { 0xc6a39582, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1439584, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b839589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6303958b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c639594, SUC_lexicon_tags_8 },
    { 0xf9839595, SUC_lexicon_tags_27 },
    { 0xaa639595, SUC_lexicon_tags_1 },
    { 0xd7a39597, SUC_lexicon_tags_11 },
    { 0xc1a39596, SUC_lexicon_tags_27 },
    { 0x82a39594, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c3959c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4283959f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310395a5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x366395a7, SUC_lexicon_tags_33 },
    { 0xa72395a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6395ac, SUC_lexicon_tags_884 },
    { 0, NULL },
    { 0xbce395ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68395b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84395b8, SUC_lexicon_tags_12 },
    { 0x344395b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca395bd, SUC_lexicon_tags_43 },
    { 0x60c395be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90395c3, SUC_lexicon_tags_11 },
    { 0x20a395c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a395ca, SUC_lexicon_tags_14 },
    { 0xc52395ca, SUC_lexicon_tags_11 },
    { 0xfd2395cc, SUC_lexicon_tags_25 },
    { 0x2f8395cc, SUC_lexicon_tags_39 },
    { 0xbe395cd, SUC_lexicon_tags_47 },
    { 0x124395cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x266395d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1fc395d3, SUC_lexicon_tags_33 },
    { 0x114395d4, SUC_lexicon_tags_45 },
    { 0x732395d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430395da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9c395dc, SUC_lexicon_tags_63 },
    { 0x88395dd, SUC_lexicon_tags_45 },
    { 0x508395dd, SUC_lexicon_tags_23 },
    { 0x94a395df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672395e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180395ea, SUC_lexicon_tags_11 },
    { 0xdc6395ea, SUC_lexicon_tags_37 },
    { 0x7ec395ec, SUC_lexicon_tags_55 },
    { 0xdc8395ea, SUC_lexicon_tags_37 },
    { 0x856395ea, SUC_lexicon_tags_87 },
    { 0x78e395ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf02395f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68395f8, SUC_lexicon_tags_11 },
    { 0x578395f8, SUC_lexicon_tags_14 },
    { 0xfa395f9, SUC_lexicon_tags_39 },
    { 0x5ce395f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x69e395fd, SUC_lexicon_tags_28 },
    { 0xe70395fd, SUC_lexicon_tags_12 },
    { 0x7e0395ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c39604, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34639606, SUC_lexicon_tags_19 },
    { 0xd5839607, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x18c39609, SUC_lexicon_tags_14 },
    { 0x9963960a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc3960e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c39613, SUC_lexicon_tags_2 },
    { 0x3b239614, SUC_lexicon_tags_47 },
    { 0xe5e39614, SUC_lexicon_tags_19 },
    { 0xc7a39616, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb039618, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x19c3961a, SUC_lexicon_tags_30 },
    { 0xf043961a, SUC_lexicon_tags_15 },
    { 0x4643961c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a3961f, SUC_lexicon_tags_0 },
    { 0xa7639620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19e39622, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc039625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79439627, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3603962a, SUC_lexicon_tags_25 },
    { 0xee23962b, SUC_lexicon_tags_38 },
    { 0xa663962a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ba3962e, SUC_lexicon_tags_12 },
    { 0x7803962f, SUC_lexicon_tags_19 },
    { 0xf1a3962e, SUC_lexicon_tags_13 },
    { 0x14a39630, SUC_lexicon_tags_25 },
    { 0x4bc3962e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4e839634, SUC_lexicon_tags_17 },
    { 0x18e39635, SUC_lexicon_tags_82 },
    { 0x4c839636, SUC_lexicon_tags_0 },
    { 0xbfe39637, SUC_lexicon_tags_61 },
    { 0xc1639638, SUC_lexicon_tags_27 },
    { 0x78c39635, SUC_lexicon_tags_10 },
    { 0x3de3963a, SUC_lexicon_tags_17 },
    { 0xb923963b, SUC_lexicon_tags_19 },
    { 0x54e3963a, SUC_lexicon_tags_15 },
    { 0xc223963d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1683963f, SUC_lexicon_tags_37 },
    { 0xc923963f, SUC_lexicon_tags_37 },
    { 0x9039641, SUC_lexicon_tags_37 },
    { 0xc2839640, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23239644, SUC_lexicon_tags_77 },
    { 0xc9a39644, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41439649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e3964b, SUC_lexicon_tags_12 },
    { 0x3043964c, SUC_lexicon_tags_37 },
    { 0x2023964d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3de3964f, SUC_lexicon_tags_17 },
    { 0x82439650, SUC_lexicon_tags_25 },
    { 0xc9c3964f, SUC_lexicon_tags_13 },
    { 0x5fe39652, SUC_lexicon_tags_60 },
    { 0xe9039653, SUC_lexicon_tags_19 },
    { 0x87239654, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4439656, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6943965b, SUC_lexicon_tags_129 },
    { 0xc683965b, SUC_lexicon_tags_8 },
    { 0xe2e3965d, SUC_lexicon_tags_12 },
    { 0xd963965c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea39661, SUC_lexicon_tags_129 },
    { 0x93a39662, SUC_lexicon_tags_7 },
    { 0xece39663, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4639665, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8be39667, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a83966c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e39672, SUC_lexicon_tags_21 },
    { 0xcfe39673, SUC_lexicon_tags_2 },
    { 0x7c439672, SUC_lexicon_tags_30 },
    { 0x29239673, SUC_lexicon_tags_37 },
    { 0x29c39674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f439678, SUC_lexicon_tags_17 },
    { 0x46639679, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a3967c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b839680, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5039683, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a39687, SUC_lexicon_tags_33 },
    { 0x89a39687, SUC_lexicon_tags_74 },
    { 0xeda39689, SUC_lexicon_tags_12 },
    { 0xa7c39689, SUC_lexicon_tags_14 },
    { 0x56e3968a, SUC_lexicon_tags_23 },
    { 0x52a39689, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c39691, SUC_lexicon_tags_14 },
    { 0x8ac39692, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7239696, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b83969a, SUC_lexicon_tags_38 },
    { 0x6d63969a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2396a1, SUC_lexicon_tags_47 },
    { 0x5c8396a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda396a5, SUC_lexicon_tags_27 },
    { 0x668396a6, SUC_lexicon_tags_8 },
    { 0x340396a7, SUC_lexicon_tags_60 },
    { 0xcd6396a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962396b3, SUC_lexicon_tags_104 },
    { 0xc54396b3, SUC_lexicon_tags_0 },
    { 0x65e396b5, SUC_lexicon_tags_33 },
    { 0xa58396b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66396bd, SUC_lexicon_tags_12 },
    { 0x402396bd, SUC_lexicon_tags_2 },
    { 0x1aa396bf, SUC_lexicon_tags_12 },
    { 0x438396bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc24396c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14396c4, SUC_lexicon_tags_31 },
    { 0x804396c5, SUC_lexicon_tags_14 },
    { 0x902396c6, SUC_lexicon_tags_11 },
    { 0x7a4396c4, SUC_lexicon_tags_6 },
    { 0x8a4396c8, SUC_lexicon_tags_8 },
    { 0x42a396c8, SUC_lexicon_tags_68 },
    { 0x6d8396ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x826396cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae4396cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86396d3, SUC_lexicon_tags_23 },
    { 0xd9e396d4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x29e396d6, SUC_lexicon_tags_25 },
    { 0x180396d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776396db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6396e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2396eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68396f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x874396f5, SUC_lexicon_tags_37 },
    { 0x440396f5, SUC_lexicon_tags_27 },
    { 0x81e396f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d8396f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaac396fb, SUC_lexicon_tags_23 },
    { 0xc64396fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e396ff, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94239704, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4439707, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x1b239709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4239714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a39719, SUC_lexicon_tags_47 },
    { 0x3563971a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x703971c, SUC_lexicon_tags_25 },
    { 0x8ac3971d, SUC_lexicon_tags_27 },
    { 0xa3c3971e, SUC_lexicon_tags_9 },
    { 0x5ca3971f, SUC_lexicon_tags_12 },
    { 0xf723971f, SUC_lexicon_tags_12 },
    { 0x8b239721, SUC_lexicon_tags_28 },
    { 0xeea39722, SUC_lexicon_tags_12 },
    { 0x39239723, SUC_lexicon_tags_15 },
    { 0x5443971e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa39729, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b03972e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaee39730, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa39735, SUC_lexicon_tags_12 },
    { 0x7e39736, SUC_lexicon_tags_2 },
    { 0xc0639735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd263973c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb039743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51439745, SUC_lexicon_tags_7 },
    { 0x62e39746, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52239749, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4da3974b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de3974f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x37839751, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc39754, SUC_lexicon_tags_65 },
    { 0xd3439754, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0239758, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a3975f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9839761, SUC_lexicon_tags_77 },
    { 0xf6839762, SUC_lexicon_tags_14 },
    { 0x7be39763, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c39766, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed43976b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4fe3976d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c39773, SUC_lexicon_tags_25 },
    { 0x5039774, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3ee39776, SUC_lexicon_tags_107 },
    { 0x43839776, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc39780, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64839784, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a39787, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c3978d, SUC_lexicon_tags_8 },
    { 0x8243978e, SUC_lexicon_tags_8 },
    { 0xb8e3978e, SUC_lexicon_tags_14 },
    { 0x9423978e, SUC_lexicon_tags_82 },
    { 0x85e39791, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a39794, SUC_lexicon_tags_29 },
    { 0xfaa39795, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f839798, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe3979b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf343979f, SUC_lexicon_tags_31 },
    { 0x3fe397a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a397a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb28397a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a8397a7, SUC_lexicon_tags_2 },
    { 0xe44397a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76397b3, SUC_lexicon_tags_12 },
    { 0x6be397b3, SUC_lexicon_tags_12 },
    { 0x8d6397b4, SUC_lexicon_tags_14 },
    { 0x7b6397b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x900397b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ac397ba, SUC_lexicon_tags_12 },
    { 0x526397ba, SUC_lexicon_tags_15 },
    { 0x1b2397bb, SUC_lexicon_tags_1 },
    { 0xfe6397ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2397c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80397ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a397cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb22397d1, SUC_lexicon_tags_4 },
    { 0x946397d2, SUC_lexicon_tags_37 },
    { 0x472397d1, SUC_lexicon_tags_47 },
    { 0x7a6397d3, SUC_lexicon_tags_25 },
    { 0x5a4397d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718397dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa6397df, SUC_lexicon_tags_12 },
    { 0xca397df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e397e4, SUC_lexicon_tags_37 },
    { 0xc08397e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a397e8, SUC_lexicon_tags_12 },
    { 0x630397e8, SUC_lexicon_tags_12 },
    { 0x70e397ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6397f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92397f6, SUC_lexicon_tags_6 },
    { 0x93e397f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14397fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca397fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74439800, SUC_lexicon_tags_25 },
    { 0xd5639801, SUC_lexicon_tags_3 },
    { 0xfa839802, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a3980a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf783980d, SUC_lexicon_tags_731 },
    { 0x8a83980e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c239810, SUC_lexicon_tags_7 },
    { 0x55a39811, SUC_lexicon_tags_1 },
    { 0xcf439811, SUC_lexicon_tags_11 },
    { 0xf039811, SUC_lexicon_tags_21 },
    { 0x9b839810, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e39817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763981b, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0x3ea3981d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c39823, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1e039825, SUC_lexicon_tags_33 },
    { 0xd9e39825, SUC_lexicon_tags_123 },
    { 0x62239826, SUC_lexicon_tags_65 },
    { 0xa7e39828, SUC_lexicon_tags_2 },
    { 0xaa639829, SUC_lexicon_tags_14 },
    { 0xd8239828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5343982e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2c39830, SUC_lexicon_tags_25 },
    { 0xd8a39830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4039833, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48039837, SUC_lexicon_tags_7 },
    { 0x5aa39837, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb863983c, SUC_lexicon_tags_37 },
    { 0x9003983d, SUC_lexicon_tags_49 },
    { 0x3a83983c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c39841, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb7639843, SUC_lexicon_tags_30 },
    { 0xa1439844, SUC_lexicon_tags_31 },
    { 0xb2639844, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52439847, SUC_lexicon_tags_15 },
    { 0x97039848, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b23984a, SUC_lexicon_tags_25 },
    { 0xd883984b, SUC_lexicon_tags_25 },
    { 0x6803984a, SUC_lexicon_tags_37 },
    { 0x3a03984d, SUC_lexicon_tags_376 },
    { 0x8083984b, SUC_lexicon_tags_27 },
    { 0x60a3984f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34e39851, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a039854, SUC_lexicon_tags_143 },
    { 0x21639855, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a03985f, SUC_lexicon_tags_816 },
    { 0x33c39860, SUC_lexicon_tags_29 },
    { 0x13a39861, SUC_lexicon_tags_38 },
    { 0xd8e39862, SUC_lexicon_tags_46 },
    { 0x54c39863, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc6e39865, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca039869, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e03986f, SUC_lexicon_tags_17 },
    { 0xf2c39870, SUC_lexicon_tags_47 },
    { 0x2e439870, SUC_lexicon_tags_43 },
    { 0xae039870, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b239874, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa39879, SUC_lexicon_tags_114 },
    { 0xcbe39879, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63987d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2039881, SUC_lexicon_tags_2 },
    { 0x95c39882, SUC_lexicon_tags_37 },
    { 0xc6439882, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a39886, SUC_lexicon_tags_49 },
    { 0x5439887, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e3988d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc39890, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a39893, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc39898, SUC_lexicon_tags_12 },
    { 0xc1639898, SUC_lexicon_tags_229 },
    { 0x7d239899, SUC_lexicon_tags_17 },
    { 0x78a3989a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7583989f, SUC_lexicon_tags_3 },
    { 0xd8a398a0, SUC_lexicon_tags_9 },
    { 0x16e3989f, SUC_lexicon_tags_82 },
    { 0xf0398a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe398a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x15c398a7, SUC_lexicon_tags_37 },
    { 0x526398a8, SUC_lexicon_tags_25 },
    { 0x7a6398a9, SUC_lexicon_tags_60 },
    { 0xa0398aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e398b1, SUC_lexicon_tags_87 },
    { 0xaac398b1, SUC_lexicon_tags_7 },
    { 0xe86398b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60398bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x950398be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d2398c0, SUC_lexicon_tags_11 },
    { 0xb16398c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854398c5, SUC_lexicon_tags_23 },
    { 0xc18398c6, SUC_lexicon_tags_25 },
    { 0xd72398c7, SUC_lexicon_tags_37 },
    { 0x30e398c7, SUC_lexicon_tags_15 },
    { 0x9f4398c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1e398cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x202398cd, SUC_lexicon_tags_11 },
    { 0x4cc398ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56398d2, SUC_lexicon_tags_0 },
    { 0x260398d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44e398d5, SUC_lexicon_tags_1 },
    { 0xe0a398d6, SUC_lexicon_tags_14 },
    { 0x31a398d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a398da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf16398dc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90398e4, SUC_lexicon_tags_8 },
    { 0x66398e5, SUC_lexicon_tags_47 },
    { 0x88a398e6, SUC_lexicon_tags_37 },
    { 0xeda398e4, SUC_lexicon_tags_47 },
    { 0x68c398e8, SUC_lexicon_tags_8 },
    { 0x332398e9, SUC_lexicon_tags_2 },
    { 0x4f2398ea, SUC_lexicon_tags_79 },
    { 0xb8398ea, SUC_lexicon_tags_55 },
    { 0xb1c398ec, SUC_lexicon_tags_2 },
    { 0xe8c398eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9c398ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3e398f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68398fb, SUC_lexicon_tags_72 },
    { 0x23e398fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97039907, SUC_lexicon_tags_12 },
    { 0x26439908, SUC_lexicon_tags_2 },
    { 0xa5439907, SUC_lexicon_tags_61 },
    { 0x4123990a, SUC_lexicon_tags_0 },
    { 0x5c03990b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c39912, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33439916, SUC_lexicon_tags_27 },
    { 0xcd639916, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d03991a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be39922, SUC_lexicon_tags_37 },
    { 0xff239923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c39929, SUC_lexicon_tags_31 },
    { 0xdb63992a, SUC_lexicon_tags_9 },
    { 0x2103992a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10039933, SUC_lexicon_tags_28 },
    { 0xfbc39934, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79439939, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e3993c, SUC_lexicon_tags_2 },
    { 0xfe63993c, SUC_lexicon_tags_60 },
    { 0xac63993d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58439943, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb9039945, SUC_lexicon_tags_17 },
    { 0x9639945, SUC_lexicon_tags_138 },
    { 0x89639947, SUC_lexicon_tags_44 },
    { 0x8c039947, SUC_lexicon_tags_9 },
    { 0x92039947, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee63994b, SUC_lexicon_tags_79 },
    { 0x30c3994c, SUC_lexicon_tags_49 },
    { 0xd603994c, SUC_lexicon_tags_26 },
    { 0xcc23994b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7439951, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9063995a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba3995d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae39965, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7839968, SUC_lexicon_tags_9 },
    { 0x97c39969, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0x1e23996b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a3996f, SUC_lexicon_tags_30 },
    { 0xe5a3996f, SUC_lexicon_tags_30 },
    { 0xea239971, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x10239973, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47a39975, SUC_lexicon_tags_87 },
    { 0xa1439976, SUC_lexicon_tags_18 },
    { 0x91e39977, SUC_lexicon_tags_17 },
    { 0x59239976, SUC_lexicon_tags_15 },
    { 0xc7239977, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a3997d, SUC_lexicon_tags_37 },
    { 0x1803997e, SUC_lexicon_tags_37 },
    { 0x783997f, SUC_lexicon_tags_19 },
    { 0xdec39980, SUC_lexicon_tags_9 },
    { 0xc7a39981, SUC_lexicon_tags_11 },
    { 0xd1639981, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d239985, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb439989, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe483998c, SUC_lexicon_tags_38 },
    { 0x4383998c, SUC_lexicon_tags_0 },
    { 0x63e3998e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e39996, SUC_lexicon_tags_25 },
    { 0x34e39996, SUC_lexicon_tags_32 },
    { 0x39998, SUC_lexicon_tags_65 },
    { 0xbf439999, SUC_lexicon_tags_49 },
    { 0xb183999a, SUC_lexicon_tags_38 },
    { 0xc123999a, SUC_lexicon_tags_17 },
    { 0x1ee39999, SUC_lexicon_tags_63 },
    { 0x3763999b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x918399a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822399a4, SUC_lexicon_tags_0 },
    { 0xee4399a5, SUC_lexicon_tags_37 },
    { 0xf98399a6, SUC_lexicon_tags_207 },
    { 0xede399a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946399ac, SUC_lexicon_tags_825 },
    { 0x68e399ac, SUC_lexicon_tags_55 },
    { 0x5d2399ae, SUC_lexicon_tags_37 },
    { 0xbe6399ae, SUC_lexicon_tags_11 },
    { 0x720399b0, SUC_lexicon_tags_37 },
    { 0x27c399b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x352399b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38399b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c399bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce399c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14399c6, SUC_lexicon_tags_47 },
    { 0x2c6399c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60399cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcec399cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec399d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba399d8, SUC_lexicon_tags_17 },
    { 0x3c4399d9, SUC_lexicon_tags_11 },
    { 0x544399d9, SUC_lexicon_tags_9 },
    { 0x514399db, SUC_lexicon_tags_55 },
    { 0x438399d9, SUC_lexicon_tags_60 },
    { 0x8f4399db, SUC_lexicon_tags_30 },
    { 0xf8a399de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x668399e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a399e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd2399e6, SUC_lexicon_tags_27 },
    { 0x182399e6, SUC_lexicon_tags_23 },
    { 0x6b6399e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c399ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa04399ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c399f3, SUC_lexicon_tags_9 },
    { 0xf92399f4, SUC_lexicon_tags_14 },
    { 0x800399f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108399fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e399fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec839a01, SUC_lexicon_tags_55 },
    { 0x56e39a02, SUC_lexicon_tags_28 },
    { 0xe4039a02, SUC_lexicon_tags_37 },
    { 0x3d439a04, SUC_lexicon_tags_6 },
    { 0xaae39a05, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75839a0e, SUC_lexicon_tags_38 },
    { 0x49039a0f, SUC_lexicon_tags_17 },
    { 0x92039a10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb839a13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x30439a15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda39a17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f439a19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2c39a1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c039a20, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x83c39a22, SUC_lexicon_tags_27 },
    { 0xade39a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de39a28, SUC_lexicon_tags_26 },
    { 0x8be39a28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27039a2b, SUC_lexicon_tags_17 },
    { 0x70839a2c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a39a2f, SUC_lexicon_tags_23 },
    { 0xb6839a2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4fa39a32, SUC_lexicon_tags_25 },
    { 0xac239a33, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a439a36, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd639a3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c39a41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c39a46, SUC_lexicon_tags_63 },
    { 0xf839a46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c239a49, SUC_lexicon_tags_7 },
    { 0x4039a4a, SUC_lexicon_tags_17 },
    { 0x57439a4b, SUC_lexicon_tags_25 },
    { 0x5ee39a4a, SUC_lexicon_tags_25 },
    { 0x16e39a4a, SUC_lexicon_tags_87 },
    { 0x8cc39a4e, SUC_lexicon_tags_1 },
    { 0xd4839a4d, SUC_lexicon_tags_15 },
    { 0x7fc39a4a, SUC_lexicon_tags_11 },
    { 0x19c39a51, SUC_lexicon_tags_84 },
    { 0x4aa39a51, SUC_lexicon_tags_22 },
    { 0x2dc39a53, SUC_lexicon_tags_6 },
    { 0xdb039a54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25a39a56, SUC_lexicon_tags_37 },
    { 0xb4639a57, SUC_lexicon_tags_11 },
    { 0x31839a58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36639a5a, SUC_lexicon_tags_6 },
    { 0xa3c39a5a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e39a5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8639a61, SUC_lexicon_tags_34 },
    { 0x70639a61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11c39a64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75639a66, SUC_lexicon_tags_7 },
    { 0x9439a66, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4e039a69, SUC_lexicon_tags_15 },
    { 0xd1439a6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21839a6c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7839a70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b239a76, SUC_lexicon_tags_11 },
    { 0x6e239a77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfa039a79, SUC_lexicon_tags_27 },
    { 0x66a39a7a, SUC_lexicon_tags_15 },
    { 0x95039a7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1aa39a7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c39a82, SUC_lexicon_tags_47 },
    { 0x39039a83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e39a88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38439a8b, SUC_lexicon_tags_1 },
    { 0x65e39a8c, SUC_lexicon_tags_1 },
    { 0x95639a8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3039a92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9839a96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe39a99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60039aa7, SUC_lexicon_tags_37 },
    { 0x76239aa8, SUC_lexicon_tags_37 },
    { 0x19439aa8, SUC_lexicon_tags_42 },
    { 0x8c039aa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37639aad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c39ab4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9f639ab6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38839aba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f239abc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4439abf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcec39ac1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0839ac5, SUC_lexicon_tags_57 },
    { 0x2a839ac5, SUC_lexicon_tags_1 },
    { 0x3fa39ac7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86439acd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa39acf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78239ad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33439ad7, SUC_lexicon_tags_40 },
    { 0xcfe39ad8, SUC_lexicon_tags_7 },
    { 0xe8839ad9, SUC_lexicon_tags_23 },
    { 0x35239ad7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5ea39adc, SUC_lexicon_tags_10 },
    { 0x9439add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0639ae6, SUC_lexicon_tags_60 },
    { 0x21c39ae7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2c39ae9, SUC_lexicon_tags_11 },
    { 0x34039ae9, SUC_lexicon_tags_25 },
    { 0x4b239aeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49239aed, SUC_lexicon_tags_65 },
    { 0x35a39aed, SUC_lexicon_tags_38 },
    { 0xdfc39aee, SUC_lexicon_tags_1 },
    { 0x80839af0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x86839af3, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x27639af6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20039af8, SUC_lexicon_tags_738 },
    { 0x2e839af8, SUC_lexicon_tags_87 },
    { 0x28239afa, SUC_lexicon_tags_16 },
    { 0x2a439afa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf239afd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d639aff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc5e39b01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e39b04, SUC_lexicon_tags_43 },
    { 0xb2839b05, SUC_lexicon_tags_14 },
    { 0x47639b06, SUC_lexicon_tags_68 },
    { 0x9239b07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99039b0c, SUC_lexicon_tags_12 },
    { 0x5c239b0d, SUC_lexicon_tags_46 },
    { 0x85839b0d, SUC_lexicon_tags_38 },
    { 0xcc239b0f, SUC_lexicon_tags_34 },
    { 0x92639b0d, SUC_lexicon_tags_2 },
    { 0xca239b11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc39b19, SUC_lexicon_tags_25 },
    { 0x7fe39b1a, SUC_lexicon_tags_38 },
    { 0x60a39b1b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a39b1f, SUC_lexicon_tags_35 },
    { 0xa6c39b20, SUC_lexicon_tags_15 },
    { 0xdc39b21, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0239b28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8239b34, SUC_lexicon_tags_37 },
    { 0x35639b34, SUC_lexicon_tags_0 },
    { 0xcb239b36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x68039b39, SUC_lexicon_tags_15 },
    { 0x8f239b39, SUC_lexicon_tags_9 },
    { 0x56639b3b, SUC_lexicon_tags_17 },
    { 0x5c639b39, SUC_lexicon_tags_17 },
    { 0xc4039b3d, SUC_lexicon_tags_12 },
    { 0xe6a39b3c, SUC_lexicon_tags_87 },
    { 0x62639b3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf039b42, SUC_lexicon_tags_49 },
    { 0x7cc39b43, SUC_lexicon_tags_45 },
    { 0x69c39b43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e639b4b, SUC_lexicon_tags_7 },
    { 0x40039b4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a39b53, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c39b57, SUC_lexicon_tags_68 },
    { 0x46c39b58, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdc239b5a, SUC_lexicon_tags_23 },
    { 0xf0839b5a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xff639b5e, SUC_lexicon_tags_45 },
    { 0xaf439b5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8de39b61, SUC_lexicon_tags_28 },
    { 0x27239b62, SUC_lexicon_tags_11 },
    { 0x77439b62, SUC_lexicon_tags_12 },
    { 0x26a39b64, SUC_lexicon_tags_12 },
    { 0x39239b63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83839b68, SUC_lexicon_tags_8 },
    { 0xf4839b69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a639b70, SUC_lexicon_tags_49 },
    { 0xb8839b71, SUC_lexicon_tags_34 },
    { 0x5fa39b72, SUC_lexicon_tags_31 },
    { 0x7b239b73, SUC_lexicon_tags_60 },
    { 0xa0e39b74, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a39b77, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea839b7e, SUC_lexicon_tags_47 },
    { 0x4f039b7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c039b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e39b8a, SUC_lexicon_tags_38 },
    { 0xb2039b8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e439b8d, SUC_lexicon_tags_7 },
    { 0xc4039b8e, SUC_lexicon_tags_29 },
    { 0x6f839b8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4439b92, SUC_lexicon_tags_25 },
    { 0x23639b93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22039b97, SUC_lexicon_tags_17 },
    { 0xf039b98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97039b9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9039ba1, SUC_lexicon_tags_12 },
    { 0xe7c39ba2, SUC_lexicon_tags_4 },
    { 0x61839ba2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x37439ba5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a39ba8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d239baf, SUC_lexicon_tags_27 },
    { 0xf0e39bb0, SUC_lexicon_tags_15 },
    { 0x47239bb1, SUC_lexicon_tags_15 },
    { 0xe4239bb2, SUC_lexicon_tags_23 },
    { 0x75639bb0, SUC_lexicon_tags_2 },
    { 0x5aa39baf, SUC_lexicon_tags_25 },
    { 0xd2039baf, SUC_lexicon_tags_15 },
    { 0x11c39bb6, SUC_lexicon_tags_31 },
    { 0xbe639bb0, SUC_lexicon_tags_13 },
    { 0x49639bb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45639bba, SUC_lexicon_tags_37 },
    { 0xc1639bbb, SUC_lexicon_tags_7 },
    { 0x11839bbc, SUC_lexicon_tags_25 },
    { 0xbae39bba, SUC_lexicon_tags_33 },
    { 0xe0839bba, SUC_lexicon_tags_21 },
    { 0x95239bbc, SUC_lexicon_tags_25 },
    { 0x82239bc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b039bc3, SUC_lexicon_tags_60 },
    { 0x86639bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39bc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec39bca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51a39bcc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf639bcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa39bd4, SUC_lexicon_tags_87 },
    { 0x54a39bd5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71439bd7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62639bdf, SUC_lexicon_tags_29 },
    { 0x35039be0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9439be4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c39be7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe39bea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e39bf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94c39bf3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c39bfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32439bfd, SUC_lexicon_tags_17 },
    { 0xaf439bfe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3039c03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a39c06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d839c09, SUC_lexicon_tags_37 },
    { 0x1a439c0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae39c0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe439c1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60839c1c, SUC_lexicon_tags_37 },
    { 0x67a39c1d, SUC_lexicon_tags_25 },
    { 0x18439c1e, SUC_lexicon_tags_23 },
    { 0x61239c1e, SUC_lexicon_tags_16 },
    { 0xf8239c1e, SUC_lexicon_tags_25 },
    { 0xf1839c21, SUC_lexicon_tags_56 },
    { 0x95c39c22, SUC_lexicon_tags_31 },
    { 0x1e839c23, SUC_lexicon_tags_37 },
    { 0x1d639c24, SUC_lexicon_tags_33 },
    { 0x9c639c25, SUC_lexicon_tags_17 },
    { 0xb2a39c26, SUC_lexicon_tags_1 },
    { 0x48439c1e, SUC_lexicon_tags_29 },
    { 0x79a39c22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaee39c2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9239c2d, SUC_lexicon_tags_17 },
    { 0x32e39c2d, SUC_lexicon_tags_37 },
    { 0xaa39c2f, SUC_lexicon_tags_15 },
    { 0xbea39c2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf839c3d, SUC_lexicon_tags_25 },
    { 0x84839c3e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x38639c41, SUC_lexicon_tags_37 },
    { 0x50439c42, SUC_lexicon_tags_12 },
    { 0x94239c42, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb9039c45, SUC_lexicon_tags_17 },
    { 0xc8239c46, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x45c39c48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0439c4c, SUC_lexicon_tags_31 },
    { 0x57439c4d, SUC_lexicon_tags_12 },
    { 0xefa39c4d, SUC_lexicon_tags_66 },
    { 0xaf239c4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e39c53, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9839c59, SUC_lexicon_tags_31 },
    { 0xb5a39c5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d839c60, SUC_lexicon_tags_16 },
    { 0x11639c60, SUC_lexicon_tags_25 },
    { 0xb2c39c60, SUC_lexicon_tags_15 },
    { 0x7f239c63, SUC_lexicon_tags_27 },
    { 0xefe39c64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75639c66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6039c68, SUC_lexicon_tags_82 },
    { 0x6aa39c68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5839c6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9639c6f, SUC_lexicon_tags_28 },
    { 0x90a39c70, SUC_lexicon_tags_11 },
    { 0xb2839c71, SUC_lexicon_tags_29 },
    { 0x1aa39c72, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xefa39c74, SUC_lexicon_tags_80 },
    { 0xa6239c75, SUC_lexicon_tags_4 },
    { 0x22039c76, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xecc39c78, SUC_lexicon_tags_19 },
    { 0x21439c79, SUC_lexicon_tags_45 },
    { 0x43c39c78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfa39c7c, SUC_lexicon_tags_9 },
    { 0x42a39c7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a39c81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xae39c84, SUC_lexicon_tags_25 },
    { 0x6dc39c85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbae39c87, SUC_lexicon_tags_10 },
    { 0x93039c88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad839c8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc39c8d, SUC_lexicon_tags_17 },
    { 0xc3a39c8e, SUC_lexicon_tags_46 },
    { 0x26e39c8f, SUC_lexicon_tags_33 },
    { 0x2e439c8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b239c96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4639c9b, SUC_lexicon_tags_17 },
    { 0xbe39c9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65239ca0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88639ca8, SUC_lexicon_tags_27 },
    { 0x7d439ca9, SUC_lexicon_tags_15 },
    { 0x7c639caa, SUC_lexicon_tags_63 },
    { 0xe5e39caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba39cb4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e39cb7, SUC_lexicon_tags_65 },
    { 0x22439cb8, SUC_lexicon_tags_37 },
    { 0x2b039cb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22439cbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3039cbd, SUC_lexicon_tags_9 },
    { 0x68039cbd, SUC_lexicon_tags_13 },
    { 0x87639cbf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0a39cc1, SUC_lexicon_tags_25 },
    { 0x7a839cc2, SUC_lexicon_tags_6 },
    { 0x80a39cc2, SUC_lexicon_tags_61 },
    { 0xc1839cc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2039cc9, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x12839ccb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3639cd0, SUC_lexicon_tags_87 },
    { 0x1b439cd0, SUC_lexicon_tags_44 },
    { 0x8ee39cd1, SUC_lexicon_tags_12 },
    { 0xfb039cd3, SUC_lexicon_tags_23 },
    { 0x8d839cd3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1639cd9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3c439cdb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2639cdf, SUC_lexicon_tags_38 },
    { 0x36a39cdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4039ce5, SUC_lexicon_tags_33 },
    { 0xd3039ce6, SUC_lexicon_tags_37 },
    { 0xd7239ce7, SUC_lexicon_tags_25 },
    { 0xef239ce8, SUC_lexicon_tags_14 },
    { 0x42639ce8, SUC_lexicon_tags_46 },
    { 0x88639ce8, SUC_lexicon_tags_37 },
    { 0xe4c39ceb, SUC_lexicon_tags_23 },
    { 0x2ae39cec, SUC_lexicon_tags_37 },
    { 0xdc239ced, SUC_lexicon_tags_12 },
    { 0x73a39ced, SUC_lexicon_tags_65 },
    { 0xa4839ced, SUC_lexicon_tags_60 },
    { 0x31439cee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c439cf2, SUC_lexicon_tags_17 },
    { 0x34c39cf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b239cf7, SUC_lexicon_tags_33 },
    { 0xd2c39cf7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xedc39cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41039cfc, SUC_lexicon_tags_25 },
    { 0xfa039cfd, SUC_lexicon_tags_87 },
    { 0xc8a39cfe, SUC_lexicon_tags_28 },
    { 0xe8639cfe, SUC_lexicon_tags_37 },
    { 0x3d039cfe, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe7839d02, SUC_lexicon_tags_29 },
    { 0x13839d02, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89639d0a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c39d0d, SUC_lexicon_tags_38 },
    { 0x1a439d0e, SUC_lexicon_tags_15 },
    { 0xa8839d0d, SUC_lexicon_tags_54 },
    { 0xc39d10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac439d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75839d16, SUC_lexicon_tags_30 },
    { 0x57639d16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0439d1c, SUC_lexicon_tags_22 },
    { 0xd5239d1c, SUC_lexicon_tags_23 },
    { 0xd7039d1e, SUC_lexicon_tags_37 },
    { 0x77439d1f, SUC_lexicon_tags_15 },
    { 0xf6239d20, SUC_lexicon_tags_14 },
    { 0x67a39d21, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc439d24, SUC_lexicon_tags_2 },
    { 0xea039d25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee439d28, SUC_lexicon_tags_11 },
    { 0x1639d29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec439d2e, SUC_lexicon_tags_1 },
    { 0xf839d2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49839d31, SUC_lexicon_tags_1 },
    { 0xfb639d31, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c39d36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8439d3b, SUC_lexicon_tags_11 },
    { 0x3e439d3b, SUC_lexicon_tags_1365 },
    { 0x90e39d3d, SUC_lexicon_tags_15 },
    { 0xed639d3e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c39d41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62839d44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80039d4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b839d4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa39d50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3839d55, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x30a39d57, SUC_lexicon_tags_15 },
    { 0xffa39d58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8039d5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac839d5d, SUC_lexicon_tags_55 },
    { 0x5e439d5e, SUC_lexicon_tags_63 },
    { 0x7a839d5f, SUC_lexicon_tags_37 },
    { 0x30439d5e, SUC_lexicon_tags_1102 },
    { 0x13e39d61, SUC_lexicon_tags_65 },
    { 0xfe239d62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2039d64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0439d66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75839d6c, SUC_lexicon_tags_29 },
    { 0x2ec39d6c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec39d70, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77639d74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e39d79, SUC_lexicon_tags_11 },
    { 0x90039d7a, SUC_lexicon_tags_15 },
    { 0x9d039d7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41c39d7d, SUC_lexicon_tags_14 },
    { 0xe4e39d7e, SUC_lexicon_tags_17 },
    { 0x5839d7f, SUC_lexicon_tags_25 },
    { 0xb5039d7f, SUC_lexicon_tags_8 },
    { 0xc5839d7f, SUC_lexicon_tags_2 },
    { 0xfb839d82, SUC_lexicon_tags_14 },
    { 0x38839d83, SUC_lexicon_tags_7 },
    { 0x2d839d84, SUC_lexicon_tags_12 },
    { 0x13839d81, SUC_lexicon_tags_46 },
    { 0x92e39d7e, SUC_lexicon_tags_15 },
    { 0x60a39d83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c39d8a, SUC_lexicon_tags_11 },
    { 0x65a39d8a, SUC_lexicon_tags_25 },
    { 0x94639d8b, SUC_lexicon_tags_27 },
    { 0x1ee39d8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9639d92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce39d9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a39d9f, SUC_lexicon_tags_47 },
    { 0x51a39da0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1439da4, SUC_lexicon_tags_39 },
    { 0xd6639da5, SUC_lexicon_tags_11 },
    { 0xbde39da6, SUC_lexicon_tags_10 },
    { 0xeb239da7, SUC_lexicon_tags_1 },
    { 0x6f039da6, SUC_lexicon_tags_25 },
    { 0xed239da8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95839daf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89639db3, SUC_lexicon_tags_6 },
    { 0x99a39db4, SUC_lexicon_tags_47 },
    { 0x86439db5, SUC_lexicon_tags_46 },
    { 0x71839db6, SUC_lexicon_tags_15 },
    { 0xdc839db6, SUC_lexicon_tags_2 },
    { 0xa9a39db8, SUC_lexicon_tags_274 },
    { 0x82c39db4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde39dbc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc39dc3, SUC_lexicon_tags_38 },
    { 0x7f439dc4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x28c39dc6, SUC_lexicon_tags_31 },
    { 0x1f839dc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b839dcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa3839dcd, SUC_lexicon_tags_47 },
    { 0x8f839dce, SUC_lexicon_tags_17 },
    { 0x50239dcf, SUC_lexicon_tags_37 },
    { 0x25a39dcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44439dd4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb4439dd6, SUC_lexicon_tags_11 },
    { 0x3a239dd7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c39ddb, SUC_lexicon_tags_63 },
    { 0x66e39ddc, SUC_lexicon_tags_15 },
    { 0x50c39ddd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5439de0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf439de2, SUC_lexicon_tags_25 },
    { 0x3ea39de3, SUC_lexicon_tags_38 },
    { 0x4e39de4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd2439de6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19a39de8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e39deb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65639def, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e39df3, SUC_lexicon_tags_39 },
    { 0xe2c39df4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce039df6, SUC_lexicon_tags_26 },
    { 0xfea39df7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1839dfc, SUC_lexicon_tags_23 },
    { 0x84a39dfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13439e03, SUC_lexicon_tags_55 },
    { 0x65a39e04, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e639e08, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1bc39e0a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c39e0d, SUC_lexicon_tags_11 },
    { 0x66c39e0d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9239e13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8839e16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec039e18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae39e1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfba39e1d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf1239e1f, SUC_lexicon_tags_82 },
    { 0xa4639e1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6a039e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56439e25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe39e29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbc39e2b, SUC_lexicon_tags_9 },
    { 0x75039e2c, SUC_lexicon_tags_37 },
    { 0xede39e2d, SUC_lexicon_tags_12 },
    { 0x8439e2e, SUC_lexicon_tags_39 },
    { 0x40a39e2e, SUC_lexicon_tags_7 },
    { 0x17039e30, SUC_lexicon_tags_249 },
    { 0xd6c39e2c, SUC_lexicon_tags_33 },
    { 0xd0e39e32, SUC_lexicon_tags_42 },
    { 0x50239e32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a39e3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a39e42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1439e46, SUC_lexicon_tags_31 },
    { 0xf5239e47, SUC_lexicon_tags_27 },
    { 0xf0639e47, SUC_lexicon_tags_17 },
    { 0xac639e46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9839e4e, SUC_lexicon_tags_37 },
    { 0x38839e4f, SUC_lexicon_tags_32 },
    { 0x9ee39e4f, SUC_lexicon_tags_2 },
    { 0xe039e4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x23e39e53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf1e39e55, SUC_lexicon_tags_17 },
    { 0x40c39e56, SUC_lexicon_tags_39 },
    { 0xaca39e55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92039e59, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xd4639e5b, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90039e64, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82639e6b, SUC_lexicon_tags_11 },
    { 0x10839e6c, SUC_lexicon_tags_39 },
    { 0x88639e6d, SUC_lexicon_tags_12 },
    { 0x9c439e6d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e39e72, SUC_lexicon_tags_14 },
    { 0xbac39e73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14439e82, SUC_lexicon_tags_6 },
    { 0x32639e82, SUC_lexicon_tags_25 },
    { 0x59e39e82, SUC_lexicon_tags_0 },
    { 0x72839e83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde39e89, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc39e8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c39e91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5239e94, SUC_lexicon_tags_2 },
    { 0x11c39e95, SUC_lexicon_tags_63 },
    { 0x7c439e96, SUC_lexicon_tags_38 },
    { 0xb5839e97, SUC_lexicon_tags_48 },
    { 0xb4839e98, SUC_lexicon_tags_31 },
    { 0x6f839e97, SUC_lexicon_tags_12 },
    { 0x4b039e9a, SUC_lexicon_tags_11 },
    { 0xdd639e9b, SUC_lexicon_tags_16 },
    { 0xfa239e9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd839e9f, SUC_lexicon_tags_17 },
    { 0xbe39ea0, SUC_lexicon_tags_17 },
    { 0xd3839ea1, SUC_lexicon_tags_43 },
    { 0x3da39ea2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c39ea6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x20c39ea8, SUC_lexicon_tags_47 },
    { 0x5f639ea9, SUC_lexicon_tags_17 },
    { 0x44439eaa, SUC_lexicon_tags_47 },
    { 0x85839eaa, SUC_lexicon_tags_11 },
    { 0xcfa39eab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae39eaf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd4639eb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6439eb5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55239eb9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43039ebd, SUC_lexicon_tags_30 },
    { 0xb3c39ebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc39ec3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb639ec7, SUC_lexicon_tags_25 },
    { 0xea439ec8, SUC_lexicon_tags_37 },
    { 0x42a39ec9, SUC_lexicon_tags_11 },
    { 0xedc39ec9, SUC_lexicon_tags_14 },
    { 0x66239ec9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5f239ecd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe239ed4, SUC_lexicon_tags_54 },
    { 0x27639ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf039ed9, SUC_lexicon_tags_2 },
    { 0xf4439ed9, SUC_lexicon_tags_17 },
    { 0x2fa39edb, SUC_lexicon_tags_71 },
    { 0x9c439edc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc39ee1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6da39ee3, SUC_lexicon_tags_56 },
    { 0x5ba39ee3, SUC_lexicon_tags_133 },
    { 0x24839ee3, SUC_lexicon_tags_49 },
    { 0x4cc39ee6, SUC_lexicon_tags_6 },
    { 0x9039ee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1639ee9, SUC_lexicon_tags_47 },
    { 0xbb839eea, SUC_lexicon_tags_1 },
    { 0xd5439ee9, SUC_lexicon_tags_14 },
    { 0xe3239eec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de39ef0, SUC_lexicon_tags_11 },
    { 0xe639ef1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2e839ef3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2839ef8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece39eff, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27039f08, SUC_lexicon_tags_19 },
    { 0xe9639f09, SUC_lexicon_tags_2 },
    { 0x27839f0a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xcc239f0c, SUC_lexicon_tags_23 },
    { 0x94e39f0d, SUC_lexicon_tags_43 },
    { 0xb7639f0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f12, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1e39f14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1439f1d, SUC_lexicon_tags_45 },
    { 0x4f639f1e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x30639f21, SUC_lexicon_tags_17 },
    { 0x60439f22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d639f25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c839f2a, SUC_lexicon_tags_8 },
    { 0x4ce39f2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa39f32, SUC_lexicon_tags_37 },
    { 0xa3039f33, SUC_lexicon_tags_17 },
    { 0xf4839f34, SUC_lexicon_tags_6 },
    { 0x30439f34, SUC_lexicon_tags_29 },
    { 0xccc39f36, SUC_lexicon_tags_8 },
    { 0x4c439f37, SUC_lexicon_tags_39 },
    { 0xde239f38, SUC_lexicon_tags_68 },
    { 0x57439f39, SUC_lexicon_tags_46 },
    { 0x1f039f3a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba39f42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa39f48, SUC_lexicon_tags_39 },
    { 0xa8239f49, SUC_lexicon_tags_17 },
    { 0xe4a39f48, SUC_lexicon_tags_48 },
    { 0x16e39f48, SUC_lexicon_tags_56 },
    { 0xee39f4c, SUC_lexicon_tags_2 },
    { 0x2c439f4c, SUC_lexicon_tags_68 },
    { 0xab039f4e, SUC_lexicon_tags_17 },
    { 0xb5239f4e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e39f52, SUC_lexicon_tags_62 },
    { 0xf3239f52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d439f57, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f439f5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da39f60, SUC_lexicon_tags_7 },
    { 0x4d239f61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e639f63, SUC_lexicon_tags_25 },
    { 0x77e39f64, SUC_lexicon_tags_23 },
    { 0x44039f64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d639f67, SUC_lexicon_tags_17 },
    { 0x1c39f68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e39f6d, SUC_lexicon_tags_11 },
    { 0xa1c39f6e, SUC_lexicon_tags_47 },
    { 0xfe839f6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba39f76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5039f7e, SUC_lexicon_tags_37 },
    { 0x9b039f7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3639f82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc39f8b, SUC_lexicon_tags_238 },
    { 0x42639f8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec039f8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3a39f90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde39f95, SUC_lexicon_tags_27 },
    { 0x18a39f96, SUC_lexicon_tags_12 },
    { 0xf6839f97, SUC_lexicon_tags_65 },
    { 0x27839f98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec39f9d, SUC_lexicon_tags_15 },
    { 0x3ec39f9e, SUC_lexicon_tags_11 },
    { 0x75239f9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcb439fa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5439fa8, SUC_lexicon_tags_47 },
    { 0x2e39fa9, SUC_lexicon_tags_56 },
    { 0xaa39fa8, SUC_lexicon_tags_7 },
    { 0xe9839fab, SUC_lexicon_tags_77 },
    { 0x97639fab, SUC_lexicon_tags_19 },
    { 0xb4039fac, SUC_lexicon_tags_1 },
    { 0x43a39fae, SUC_lexicon_tags_15 },
    { 0xec039faf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a039fb3, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0xe6c39fb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x45839fb8, SUC_lexicon_tags_33 },
    { 0xec839fb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73a39fbb, SUC_lexicon_tags_25 },
    { 0xdb439fbc, SUC_lexicon_tags_12 },
    { 0x49439fbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf6a39fbf, SUC_lexicon_tags_12 },
    { 0x3e439fc0, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x7ae39fc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21239fc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa439fca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1439fcc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x13e39fce, SUC_lexicon_tags_4 },
    { 0x2639fcf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76839fd3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1a839fd5, SUC_lexicon_tags_63 },
    { 0x93e39fd5, SUC_lexicon_tags_1 },
    { 0x2439fd5, SUC_lexicon_tags_37 },
    { 0xb0c39fd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe39fde, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e39fe5, SUC_lexicon_tags_82 },
    { 0x57839fe6, SUC_lexicon_tags_74 },
    { 0xd5239fe6, SUC_lexicon_tags_15 },
    { 0x56c39fe8, SUC_lexicon_tags_38 },
    { 0x79639fe6, SUC_lexicon_tags_3 },
    { 0xba439fea, SUC_lexicon_tags_77 },
    { 0x4b239feb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdac39fed, SUC_lexicon_tags_23 },
    { 0x7da39fed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xda239ff0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22a39ff2, SUC_lexicon_tags_55 },
    { 0x2ea39ff3, SUC_lexicon_tags_29 },
    { 0xfd239ff3, SUC_lexicon_tags_15 },
    { 0xdfc39ff5, SUC_lexicon_tags_33 },
    { 0xeac39ff6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a39ff9, SUC_lexicon_tags_15 },
    { 0x78a39ffa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e39ffd, SUC_lexicon_tags_47 },
    { 0x1439ffe, SUC_lexicon_tags_37 },
    { 0xbb039fff, SUC_lexicon_tags_65 },
    { 0x5003a000, SUC_lexicon_tags_47 },
    { 0x3283a001, SUC_lexicon_tags_37 },
    { 0xf8639fff, SUC_lexicon_tags_29 },
    { 0x56639fff, SUC_lexicon_tags_17 },
    { 0x47239ffd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c83a007, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de3a00d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5a3a00f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36e3a011, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da3a017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf363a01a, SUC_lexicon_tags_8 },
    { 0x4843a01a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x14a3a01d, SUC_lexicon_tags_6 },
    { 0xd543a01e, SUC_lexicon_tags_55 },
    { 0x30c3a01f, SUC_lexicon_tags_26 },
    { 0xe023a01e, SUC_lexicon_tags_37 },
    { 0xb6e3a021, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe3a024, SUC_lexicon_tags_14 },
    { 0xc943a025, SUC_lexicon_tags_7 },
    { 0xf2a3a025, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5003a029, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a3a02e, SUC_lexicon_tags_1 },
    { 0x1883a02f, SUC_lexicon_tags_15 },
    { 0xc143a02f, SUC_lexicon_tags_11 },
    { 0x27c3a030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc283a033, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e63a036, SUC_lexicon_tags_1 },
    { 0x7803a037, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4543a039, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf583a03c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2963a03f, SUC_lexicon_tags_26 },
    { 0x37a3a040, SUC_lexicon_tags_1 },
    { 0x8083a040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad63a049, SUC_lexicon_tags_7 },
    { 0xe5e3a04a, SUC_lexicon_tags_61 },
    { 0xc4a3a04b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7563a04d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723a055, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd543a057, SUC_lexicon_tags_12 },
    { 0x75a3a058, SUC_lexicon_tags_8 },
    { 0xee83a059, SUC_lexicon_tags_46 },
    { 0xba83a05a, SUC_lexicon_tags_38 },
    { 0x6423a058, SUC_lexicon_tags_19 },
    { 0xcd23a059, SUC_lexicon_tags_23 },
    { 0x74a3a05d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe403a05f, SUC_lexicon_tags_9 },
    { 0x2103a060, SUC_lexicon_tags_1 },
    { 0xba43a060, SUC_lexicon_tags_11 },
    { 0x1d63a060, SUC_lexicon_tags_17 },
    { 0xec3a063, SUC_lexicon_tags_25 },
    { 0xb7c3a064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a83a066, SUC_lexicon_tags_19 },
    { 0xbee3a067, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3943a06b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e3a06f, SUC_lexicon_tags_17 },
    { 0xcde3a06f, SUC_lexicon_tags_1296 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1123a077, SUC_lexicon_tags_23 },
    { 0xa863a077, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4823a07e, SUC_lexicon_tags_15 },
    { 0x79c3a07e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa183a081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc3a084, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f03a087, SUC_lexicon_tags_17 },
    { 0xe403a087, SUC_lexicon_tags_7 },
    { 0x6043a088, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1803a08d, SUC_lexicon_tags_17 },
    { 0x39a3a08e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd0a3a090, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5243a092, SUC_lexicon_tags_18 },
    { 0x1fc3a093, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x90c3a095, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d23a09c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2223a0a2, SUC_lexicon_tags_30 },
    { 0x1463a0a3, SUC_lexicon_tags_0 },
    { 0x2343a0a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e3a0a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c3a0ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c3a0b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7763a0b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a3a0b8, SUC_lexicon_tags_0 },
    { 0xab23a0b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb23a0bd, SUC_lexicon_tags_43 },
    { 0xb843a0be, SUC_lexicon_tags_33 },
    { 0x13c3a0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb23a0c7, SUC_lexicon_tags_27 },
    { 0x16a3a0c8, SUC_lexicon_tags_15 },
    { 0xdf23a0c9, SUC_lexicon_tags_37 },
    { 0x53c3a0ca, SUC_lexicon_tags_25 },
    { 0x7e3a0cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3643a0ce, SUC_lexicon_tags_25 },
    { 0xada3a0cf, SUC_lexicon_tags_51 },
    { 0xcb03a0d0, SUC_lexicon_tags_11 },
    { 0x81c3a0d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf43a0d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff83a0db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa303a0df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1943a0e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe163a0e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2223a0e7, SUC_lexicon_tags_11 },
    { 0xdec3a0e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee3a0ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d63a0f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd223a0f2, SUC_lexicon_tags_19 },
    { 0xce3a0f2, SUC_lexicon_tags_37 },
    { 0xe4c3a0f2, SUC_lexicon_tags_68 },
    { 0xb963a0f2, SUC_lexicon_tags_2 },
    { 0xd3a3a0f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce3a0fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfdc3a0fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb743a103, SUC_lexicon_tags_4 },
    { 0x2f83a104, SUC_lexicon_tags_244 },
    { 0xf83a103, SUC_lexicon_tags_37 },
    { 0x2aa3a106, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9063a10b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c3a10f, SUC_lexicon_tags_5 },
    { 0x35e3a10f, SUC_lexicon_tags_28 },
    { 0xf1a3a10f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe103a115, SUC_lexicon_tags_11 },
    { 0xa123a116, SUC_lexicon_tags_36 },
    { 0x5c83a116, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe3a11b, SUC_lexicon_tags_38 },
    { 0xbc63a11b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa43a11e, SUC_lexicon_tags_8 },
    { 0x3ac3a11e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19c3a121, SUC_lexicon_tags_11 },
    { 0x943a121, SUC_lexicon_tags_1 },
    { 0xfe43a122, SUC_lexicon_tags_15 },
    { 0x4b03a124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd943a126, SUC_lexicon_tags_57 },
    { 0x2fc3a127, SUC_lexicon_tags_7 },
    { 0x3583a127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf603a12f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6743a131, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10a3a133, SUC_lexicon_tags_7 },
    { 0x2503a133, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x51e3a136, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e3a138, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d23a13a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d83a13d, SUC_lexicon_tags_17 },
    { 0x2343a13e, SUC_lexicon_tags_2 },
    { 0xb7c3a13f, SUC_lexicon_tags_26 },
    { 0xfaa3a140, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4343a142, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2483a144, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa3a149, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe263a14f, SUC_lexicon_tags_85 },
    { 0x9da3a150, SUC_lexicon_tags_37 },
    { 0xbe23a14f, SUC_lexicon_tags_38 },
    { 0xa923a151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc83a155, SUC_lexicon_tags_12 },
    { 0x7363a155, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc963a15c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e3a161, SUC_lexicon_tags_30 },
    { 0x57e3a162, SUC_lexicon_tags_25 },
    { 0x83a163, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xac63a166, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17c3a168, SUC_lexicon_tags_87 },
    { 0x8743a169, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa523a16f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa3a17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e3a182, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1dc3a184, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c3a187, SUC_lexicon_tags_63 },
    { 0x4e23a187, SUC_lexicon_tags_38 },
    { 0x70a3a189, SUC_lexicon_tags_18 },
    { 0x76e3a189, SUC_lexicon_tags_15 },
    { 0x2983a18b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9643a190, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa723a192, SUC_lexicon_tags_8 },
    { 0x5943a193, SUC_lexicon_tags_37 },
    { 0xae43a194, SUC_lexicon_tags_2 },
    { 0xb403a195, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6e3a197, SUC_lexicon_tags_15 },
    { 0xe2a3a197, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc843a19b, SUC_lexicon_tags_14 },
    { 0x12c3a19c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae3a1a2, SUC_lexicon_tags_15 },
    { 0x263a1a2, SUC_lexicon_tags_28 },
    { 0x9323a1a4, SUC_lexicon_tags_23 },
    { 0xe923a1a5, SUC_lexicon_tags_6 },
    { 0x4e43a1a6, SUC_lexicon_tags_25 },
    { 0xd3e3a1a7, SUC_lexicon_tags_47 },
    { 0xe83a1a8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e3a1ac, SUC_lexicon_tags_12 },
    { 0x76a3a1ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a03a1b9, SUC_lexicon_tags_7 },
    { 0xb783a1ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc3a1c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c3a1c3, SUC_lexicon_tags_37 },
    { 0x9f43a1c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7e3a1c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc843a1c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9983a1cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca3a1d6, SUC_lexicon_tags_25 },
    { 0xd083a1d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce43a1dc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd803a1e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf243a1ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc023a1ee, SUC_lexicon_tags_37 },
    { 0x7903a1ee, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0xd883a1f1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c03a1f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4963a1f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90a3a1fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c3a1fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ce3a1ff, SUC_lexicon_tags_55 },
    { 0xc5c3a200, SUC_lexicon_tags_11 },
    { 0xbb63a1ff, SUC_lexicon_tags_15 },
    { 0x1de3a202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9343a205, SUC_lexicon_tags_85 },
    { 0x2fc3a206, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9143a208, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8983a20b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a3a213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcac3a215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe03a217, SUC_lexicon_tags_11 },
    { 0xa183a218, SUC_lexicon_tags_7 },
    { 0xb3e3a219, SUC_lexicon_tags_60 },
    { 0xca03a219, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c3a21d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de3a221, SUC_lexicon_tags_12 },
    { 0x46c3a222, SUC_lexicon_tags_11 },
    { 0x17c3a223, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3783a225, SUC_lexicon_tags_63 },
    { 0xca03a225, SUC_lexicon_tags_11 },
    { 0xa9c3a227, SUC_lexicon_tags_60 },
    { 0x4da3a228, SUC_lexicon_tags_47 },
    { 0xb3c3a228, SUC_lexicon_tags_12 },
    { 0x9ec3a225, SUC_lexicon_tags_37 },
    { 0x9503a22b, SUC_lexicon_tags_8 },
    { 0x563a22b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e3a237, SUC_lexicon_tags_7 },
    { 0x7823a238, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae3a23c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf23a23e, SUC_lexicon_tags_82 },
    { 0x55c3a23e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2143a244, SUC_lexicon_tags_32 },
    { 0x4fc3a244, SUC_lexicon_tags_37 },
    { 0x3ce3a246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3283a24e, SUC_lexicon_tags_49 },
    { 0x5c3a24e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a3a255, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f23a258, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c3a25b, SUC_lexicon_tags_2 },
    { 0x923a25c, SUC_lexicon_tags_11 },
    { 0xd323a25c, SUC_lexicon_tags_27 },
    { 0x4ce3a25b, SUC_lexicon_tags_37 },
    { 0xac43a25f, SUC_lexicon_tags_65 },
    { 0x68a3a25f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe703a267, SUC_lexicon_tags_26 },
    { 0xdd63a268, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6da3a26a, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x1923a26d, SUC_lexicon_tags_0 },
    { 0xd263a26e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe863a272, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf823a275, SUC_lexicon_tags_27 },
    { 0x7903a276, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa863a27a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99e3a27c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7283a282, SUC_lexicon_tags_64 },
    { 0xad03a283, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2743a28a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd403a28c, SUC_lexicon_tags_8 },
    { 0xf23a28d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1d03a28f, SUC_lexicon_tags_27 },
    { 0x5463a290, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e3a293, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4263a296, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc163a298, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d43a29a, SUC_lexicon_tags_25 },
    { 0x8063a29a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e3a2a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1863a2aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf283a2ac, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d63a2b0, SUC_lexicon_tags_37 },
    { 0x2d43a2b1, SUC_lexicon_tags_1 },
    { 0xd03a2b1, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7803a2b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa623a2c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf943a2c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf03a2ca, SUC_lexicon_tags_2 },
    { 0x11c3a2ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5943a2ce, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c3a2d2, SUC_lexicon_tags_4 },
    { 0xa123a2d3, SUC_lexicon_tags_4 },
    { 0x54a3a2d3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x80e3a2d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b43a2d8, SUC_lexicon_tags_63 },
    { 0xfce3a2d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a3a2dd, SUC_lexicon_tags_25 },
    { 0x3a23a2de, SUC_lexicon_tags_25 },
    { 0xd5a3a2df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf83a2e5, SUC_lexicon_tags_29 },
    { 0xce63a2e6, SUC_lexicon_tags_28 },
    { 0xb7c3a2e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da3a2ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5143a2f2, SUC_lexicon_tags_6 },
    { 0x7083a2f3, SUC_lexicon_tags_11 },
    { 0xe143a2f4, SUC_lexicon_tags_7 },
    { 0x3da3a2f5, SUC_lexicon_tags_17 },
    { 0x203a2f5, SUC_lexicon_tags_47 },
    { 0x8223a2f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb023a2f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5343a2fc, SUC_lexicon_tags_33 },
    { 0x4fe3a2fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee63a301, SUC_lexicon_tags_15 },
    { 0xe1c3a301, SUC_lexicon_tags_28 },
    { 0x1183a303, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8963a306, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5623a308, SUC_lexicon_tags_12 },
    { 0xcbe3a309, SUC_lexicon_tags_30 },
    { 0x3c03a30a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e3a30d, SUC_lexicon_tags_7 },
    { 0xbdc3a30e, SUC_lexicon_tags_7 },
    { 0xb543a30e, SUC_lexicon_tags_5 },
    { 0x9823a30e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7023a317, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfac3a319, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc43a31c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x84e3a31e, SUC_lexicon_tags_60 },
    { 0x8523a31e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3623a321, SUC_lexicon_tags_31 },
    { 0x27e3a322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b63a328, SUC_lexicon_tags_25 },
    { 0x7323a328, SUC_lexicon_tags_14 },
    { 0x3e63a32a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c3a32e, SUC_lexicon_tags_14 },
    { 0x9ae3a32f, SUC_lexicon_tags_37 },
    { 0xd783a32e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea43a334, SUC_lexicon_tags_11 },
    { 0x3183a335, SUC_lexicon_tags_31 },
    { 0x48e3a335, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3a339, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc3a33c, SUC_lexicon_tags_19 },
    { 0xace3a33d, SUC_lexicon_tags_43 },
    { 0x6d63a33e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5123a340, SUC_lexicon_tags_15 },
    { 0x10c3a341, SUC_lexicon_tags_28 },
    { 0xd223a342, SUC_lexicon_tags_15 },
    { 0x6c43a343, SUC_lexicon_tags_45 },
    { 0xbbe3a341, SUC_lexicon_tags_55 },
    { 0x65e3a344, SUC_lexicon_tags_11 },
    { 0xb4e3a343, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb543a34a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf003a34e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa83a350, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da3a353, SUC_lexicon_tags_2 },
    { 0x6963a354, SUC_lexicon_tags_15 },
    { 0x53e3a355, SUC_lexicon_tags_14 },
    { 0xecc3a353, SUC_lexicon_tags_3 },
    { 0x8ca3a353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba3a35b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x69c3a35d, SUC_lexicon_tags_63 },
    { 0xc823a35e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43a360, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8243a364, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc3a367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf543a369, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6703a36d, SUC_lexicon_tags_148 },
    { 0x4d23a36d, SUC_lexicon_tags_85 },
    { 0x2143a36d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff63a371, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a3a375, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb23a378, SUC_lexicon_tags_12 },
    { 0x8483a378, SUC_lexicon_tags_25 },
    { 0x1583a37a, SUC_lexicon_tags_33 },
    { 0x4fc3a37b, SUC_lexicon_tags_40 },
    { 0x4ce3a37c, SUC_lexicon_tags_61 },
    { 0x3563a37d, SUC_lexicon_tags_14 },
    { 0xc983a37b, SUC_lexicon_tags_37 },
    { 0x20a3a37f, SUC_lexicon_tags_15 },
    { 0xb6e3a380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4303a383, SUC_lexicon_tags_12 },
    { 0x6be3a384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbba3a386, SUC_lexicon_tags_146 },
    { 0xad23a387, SUC_lexicon_tags_11 },
    { 0x5f43a388, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe3a38b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e3a38e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3aa3a390, SUC_lexicon_tags_37 },
    { 0xc903a391, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa043a398, SUC_lexicon_tags_17 },
    { 0xce83a398, SUC_lexicon_tags_84 },
    { 0x1383a39a, SUC_lexicon_tags_77 },
    { 0x2e43a398, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4263a3a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d23a3a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9923a3ae, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6763a3b3, SUC_lexicon_tags_1 },
    { 0x163a3b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3a3b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c3a3bb, SUC_lexicon_tags_11 },
    { 0xcf43a3bc, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xf603a3be, SUC_lexicon_tags_17 },
    { 0x223a3be, SUC_lexicon_tags_11 },
    { 0xbc83a3c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3a3c5, SUC_lexicon_tags_1 },
    { 0xa263a3c6, SUC_lexicon_tags_60 },
    { 0x7163a3c5, SUC_lexicon_tags_28 },
    { 0xff43a3c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec3a3cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d43a3cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8e3a3d1, SUC_lexicon_tags_21 },
    { 0xfce3a3d1, SUC_lexicon_tags_37 },
    { 0x8863a3d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c43a3d7, SUC_lexicon_tags_23 },
    { 0x69e3a3d8, SUC_lexicon_tags_11 },
    { 0x82a3a3d7, SUC_lexicon_tags_27 },
    { 0xcec3a3da, SUC_lexicon_tags_17 },
    { 0x7e23a3da, SUC_lexicon_tags_19 },
    { 0x7203a3dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbce3a3de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3883a3e0, SUC_lexicon_tags_23 },
    { 0x1e23a3e1, SUC_lexicon_tags_37 },
    { 0x3de3a3e2, SUC_lexicon_tags_37 },
    { 0x4583a3e3, SUC_lexicon_tags_0 },
    { 0xd4a3a3e4, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2423a3ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f63a3f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9a3a3f7, SUC_lexicon_tags_17 },
    { 0x9043a3f8, SUC_lexicon_tags_7 },
    { 0xcf63a3f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6523a403, SUC_lexicon_tags_38 },
    { 0x74a3a403, SUC_lexicon_tags_61 },
    { 0x7d83a405, SUC_lexicon_tags_16 },
    { 0xffc3a405, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa4c3a408, SUC_lexicon_tags_48 },
    { 0xf563a408, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a3a413, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec43a415, SUC_lexicon_tags_19 },
    { 0x4743a415, SUC_lexicon_tags_26 },
    { 0x61a3a417, SUC_lexicon_tags_43 },
    { 0x7c83a416, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4543a41d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223a422, SUC_lexicon_tags_33 },
    { 0xbc03a423, SUC_lexicon_tags_29 },
    { 0x25a3a424, SUC_lexicon_tags_26 },
    { 0x7a03a425, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fe3a427, SUC_lexicon_tags_37 },
    { 0x2583a428, SUC_lexicon_tags_33 },
    { 0x4443a427, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e63a42d, SUC_lexicon_tags_45 },
    { 0x2583a42d, SUC_lexicon_tags_2 },
    { 0x19a3a42f, SUC_lexicon_tags_10 },
    { 0x5823a430, SUC_lexicon_tags_2 },
    { 0x9103a431, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3a435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca3a43b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b23a441, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1443a443, SUC_lexicon_tags_1 },
    { 0x5143a444, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c3a447, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef03a44b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb143a44d, SUC_lexicon_tags_1 },
    { 0xd483a44e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4823a450, SUC_lexicon_tags_12 },
    { 0x98c3a451, SUC_lexicon_tags_394 },
    { 0x5723a451, SUC_lexicon_tags_11 },
    { 0x4763a453, SUC_lexicon_tags_113 },
    { 0x6b43a454, SUC_lexicon_tags_47 },
    { 0x7363a453, SUC_lexicon_tags_27 },
    { 0xd683a450, SUC_lexicon_tags_0 },
    { 0x6f83a457, SUC_lexicon_tags_127 },
    { 0x32a3a457, SUC_lexicon_tags_47 },
    { 0xd563a458, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a23a45f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x84c3a461, SUC_lexicon_tags_11 },
    { 0xd903a462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc43a464, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3083a46a, SUC_lexicon_tags_25 },
    { 0x6703a46b, SUC_lexicon_tags_87 },
    { 0x14a3a46b, SUC_lexicon_tags_23 },
    { 0x26e3a46c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7643a46f, SUC_lexicon_tags_30 },
    { 0xc143a470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf763a472, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec03a476, SUC_lexicon_tags_19 },
    { 0x5da3a477, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323a47b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de3a47e, SUC_lexicon_tags_12 },
    { 0x19c3a47f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb43a481, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa943a487, SUC_lexicon_tags_11 },
    { 0x2023a488, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x78e3a48a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb83a48c, SUC_lexicon_tags_14 },
    { 0x1b23a48c, SUC_lexicon_tags_11 },
    { 0x37a3a48e, SUC_lexicon_tags_32 },
    { 0x1de3a48e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e3a493, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa63a496, SUC_lexicon_tags_63 },
    { 0x59e3a497, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae3a49a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca3a4a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed63a4a3, SUC_lexicon_tags_2 },
    { 0x2b63a4a4, SUC_lexicon_tags_14 },
    { 0xf83a4a4, SUC_lexicon_tags_10 },
    { 0x943a4a4, SUC_lexicon_tags_17 },
    { 0x4503a4a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3a4ab, SUC_lexicon_tags_151 },
    { 0xbd23a4ab, SUC_lexicon_tags_57 },
    { 0x1443a4ad, SUC_lexicon_tags_27 },
    { 0x1043a4ac, SUC_lexicon_tags_29 },
    { 0x2fe3a4af, SUC_lexicon_tags_56 },
    { 0x26a3a4b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63a4b8, SUC_lexicon_tags_45 },
    { 0x8da3a4b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe263a4bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4443a4bd, SUC_lexicon_tags_7 },
    { 0x9f23a4be, SUC_lexicon_tags_25 },
    { 0xca63a4bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c83a4c3, SUC_lexicon_tags_13 },
    { 0x6e43a4c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c3a4c9, SUC_lexicon_tags_39 },
    { 0x4a03a4c9, SUC_lexicon_tags_26 },
    { 0x7ee3a4cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xea23a4ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc723a4d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c3a4d6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd63a4da, SUC_lexicon_tags_37 },
    { 0x36e3a4da, SUC_lexicon_tags_11 },
    { 0x5083a4dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7883a4e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e3a4e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xac83a4e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b83a4eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4703a4ed, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c3a4f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb063a4f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6583a4fa, SUC_lexicon_tags_61 },
    { 0x2503a4fa, SUC_lexicon_tags_11 },
    { 0x3343a4fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf403a4ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa503a502, SUC_lexicon_tags_19 },
    { 0x17a3a503, SUC_lexicon_tags_35 },
    { 0x8ec3a504, SUC_lexicon_tags_2 },
    { 0x1583a505, SUC_lexicon_tags_15 },
    { 0x48c3a506, SUC_lexicon_tags_14 },
    { 0x3063a507, SUC_lexicon_tags_7 },
    { 0xd123a508, SUC_lexicon_tags_1 },
    { 0x19e3a509, SUC_lexicon_tags_33 },
    { 0x62a3a50a, SUC_lexicon_tags_57 },
    { 0xa9a3a503, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d63a50d, SUC_lexicon_tags_47 },
    { 0x2a43a50e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e43a511, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xea23a514, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf603a517, SUC_lexicon_tags_17 },
    { 0xc423a517, SUC_lexicon_tags_11 },
    { 0x5703a519, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde3a51f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec83a523, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f43a52b, SUC_lexicon_tags_6 },
    { 0x4d43a52c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x583a52e, SUC_lexicon_tags_11 },
    { 0x5d43a52f, SUC_lexicon_tags_0 },
    { 0xd983a52f, SUC_lexicon_tags_126 },
    { 0xe223a531, SUC_lexicon_tags_26 },
    { 0x7e03a532, SUC_lexicon_tags_15 },
    { 0x42e3a532, SUC_lexicon_tags_8 },
    { 0x6743a534, SUC_lexicon_tags_1 },
    { 0xd383a533, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc3a53e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d43a541, SUC_lexicon_tags_12 },
    { 0x3be3a541, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6023a546, SUC_lexicon_tags_15 },
    { 0xd763a547, SUC_lexicon_tags_27 },
    { 0xe5a3a548, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2063a54e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3a555, SUC_lexicon_tags_28 },
    { 0xb163a555, SUC_lexicon_tags_1 },
    { 0x1f03a556, SUC_lexicon_tags_17 },
    { 0x2d23a558, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7363a55b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6be3a55d, SUC_lexicon_tags_28 },
    { 0xca43a55e, SUC_lexicon_tags_17 },
    { 0x3e3a55f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a3a562, SUC_lexicon_tags_15 },
    { 0xc03a563, SUC_lexicon_tags_12 },
    { 0xe923a563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c63a566, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8783a568, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f03a56c, SUC_lexicon_tags_49 },
    { 0x3de3a56c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd783a570, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c23a572, SUC_lexicon_tags_2 },
    { 0x25e3a573, SUC_lexicon_tags_23 },
    { 0x8263a572, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7563a576, SUC_lexicon_tags_63 },
    { 0x1863a577, SUC_lexicon_tags_63 },
    { 0x86c3a578, SUC_lexicon_tags_25 },
    { 0x2763a579, SUC_lexicon_tags_37 },
    { 0x3a57a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7423a57d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe683a585, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1a3a587, SUC_lexicon_tags_12 },
    { 0x6783a587, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2c3a58a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x31a3a58c, SUC_lexicon_tags_31 },
    { 0xdac3a58c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb983a590, SUC_lexicon_tags_15 },
    { 0xfc23a591, SUC_lexicon_tags_9 },
    { 0x5743a590, SUC_lexicon_tags_35 },
    { 0xc803a591, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a63a599, SUC_lexicon_tags_2 },
    { 0x7d83a599, SUC_lexicon_tags_1 },
    { 0x57c3a59b, SUC_lexicon_tags_14 },
    { 0xa9a3a59a, SUC_lexicon_tags_12 },
    { 0x8983a599, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad83a5a2, SUC_lexicon_tags_70 },
    { 0x303a5a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7123a5a6, SUC_lexicon_tags_19 },
    { 0xa503a5a6, SUC_lexicon_tags_10 },
    { 0xfee3a5a6, SUC_lexicon_tags_63 },
    { 0xee63a5a9, SUC_lexicon_tags_11 },
    { 0x6b43a5aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff83a5ae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a3a5b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa963a5b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3043a5b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2c3a5b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e3a5ba, SUC_lexicon_tags_17 },
    { 0xdf83a5bb, SUC_lexicon_tags_25 },
    { 0xfc63a5bb, SUC_lexicon_tags_25 },
    { 0xdd43a5bc, SUC_lexicon_tags_11 },
    { 0x9963a5bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b83a5c1, SUC_lexicon_tags_49 },
    { 0x84e3a5c2, SUC_lexicon_tags_14 },
    { 0x6063a5c3, SUC_lexicon_tags_1 },
    { 0x3423a5c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7743a5c7, SUC_lexicon_tags_11 },
    { 0x5ba3a5c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb63a5cb, SUC_lexicon_tags_10 },
    { 0xd8c3a5cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce23a5ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9283a5d0, SUC_lexicon_tags_43 },
    { 0xeda3a5d0, SUC_lexicon_tags_47 },
    { 0xa163a5d0, SUC_lexicon_tags_11 },
    { 0xc503a5d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa63a5d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec3a5da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f63a5dc, SUC_lexicon_tags_7 },
    { 0x7763a5dd, SUC_lexicon_tags_2 },
    { 0x9c3a5de, SUC_lexicon_tags_15 },
    { 0xff23a5df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc3a5e4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c3a5e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe23a5ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d03a5f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x80c3a5f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e63a5f6, SUC_lexicon_tags_17 },
    { 0x1563a5f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c3a5fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a03a5ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83a3a601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc03a605, SUC_lexicon_tags_45 },
    { 0x5e83a605, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6103a609, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba3a60d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa3a610, SUC_lexicon_tags_15 },
    { 0xe563a611, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa423a613, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc83a619, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c3a628, SUC_lexicon_tags_38 },
    { 0x4fe3a629, SUC_lexicon_tags_37 },
    { 0xe5e3a62a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x963a62c, SUC_lexicon_tags_7 },
    { 0xe623a62d, SUC_lexicon_tags_2 },
    { 0x5923a62c, SUC_lexicon_tags_1 },
    { 0xbc23a62c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4e83a631, SUC_lexicon_tags_61 },
    { 0xa5e3a632, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b23a634, SUC_lexicon_tags_28 },
    { 0x50a3a635, SUC_lexicon_tags_66 },
    { 0x84c3a636, SUC_lexicon_tags_2 },
    { 0xa9e3a637, SUC_lexicon_tags_25 },
    { 0xbaa3a638, SUC_lexicon_tags_1 },
    { 0x24a3a637, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2623a63c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2243a63e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd543a640, SUC_lexicon_tags_17 },
    { 0x9dc3a640, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac3a645, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6923a649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc963a64b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a3a651, SUC_lexicon_tags_12 },
    { 0xc2e3a651, SUC_lexicon_tags_37 },
    { 0x1f23a653, SUC_lexicon_tags_25 },
    { 0xf063a654, SUC_lexicon_tags_48 },
    { 0x6f83a651, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e03a65a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a3a65e, SUC_lexicon_tags_4 },
    { 0x81c3a65e, SUC_lexicon_tags_19 },
    { 0xc23a660, SUC_lexicon_tags_29 },
    { 0xfea3a660, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3de3a663, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd603a666, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xafc3a668, SUC_lexicon_tags_37 },
    { 0x8263a669, SUC_lexicon_tags_26 },
    { 0xc043a66a, SUC_lexicon_tags_68 },
    { 0xc803a66b, SUC_lexicon_tags_26 },
    { 0x9e63a66a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f43a66f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5de3a671, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c03a675, SUC_lexicon_tags_20 },
    { 0xf903a675, SUC_lexicon_tags_43 },
    { 0x5283a677, SUC_lexicon_tags_25 },
    { 0x8e23a678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac3a67b, SUC_lexicon_tags_2 },
    { 0x51e3a67c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda3a67f, SUC_lexicon_tags_15 },
    { 0xd983a680, SUC_lexicon_tags_38 },
    { 0x1ca3a681, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb783a686, SUC_lexicon_tags_28 },
    { 0xdc3a686, SUC_lexicon_tags_44 },
    { 0x3ba3a686, SUC_lexicon_tags_38 },
    { 0x9e63a689, SUC_lexicon_tags_1 },
    { 0xa6e3a68a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63a68d, SUC_lexicon_tags_39 },
    { 0x5243a68e, SUC_lexicon_tags_13 },
    { 0xb543a68d, SUC_lexicon_tags_3 },
    { 0xff43a68e, SUC_lexicon_tags_6 },
    { 0x5fa3a691, SUC_lexicon_tags_1 },
    { 0x8903a691, SUC_lexicon_tags_25 },
    { 0xc363a693, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ea3a695, SUC_lexicon_tags_7 },
    { 0xee03a696, SUC_lexicon_tags_60 },
    { 0x90a3a696, SUC_lexicon_tags_37 },
    { 0x9d43a695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f03a69b, SUC_lexicon_tags_33 },
    { 0xe6e3a69c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc843a69e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc3a6a1, SUC_lexicon_tags_49 },
    { 0xd123a6a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2023a6a4, SUC_lexicon_tags_12 },
    { 0xf1a3a6a4, SUC_lexicon_tags_60 },
    { 0xd0e3a6a6, SUC_lexicon_tags_28 },
    { 0xd0e3a6a7, SUC_lexicon_tags_64 },
    { 0x87e3a6a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a3a6ab, SUC_lexicon_tags_37 },
    { 0xd023a6ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1863a6af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4463a6b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543a6b9, SUC_lexicon_tags_10 },
    { 0x6e43a6ba, SUC_lexicon_tags_33 },
    { 0xafc3a6bb, SUC_lexicon_tags_47 },
    { 0x9103a6bc, SUC_lexicon_tags_26 },
    { 0xc843a6bd, SUC_lexicon_tags_3 },
    { 0x9143a6be, SUC_lexicon_tags_9 },
    { 0x7003a6bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d63a6c2, SUC_lexicon_tags_65 },
    { 0x9943a6c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e3a6cc, SUC_lexicon_tags_2 },
    { 0xf663a6cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783a6d3, SUC_lexicon_tags_25 },
    { 0xa263a6d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd303a6d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29c3a6da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb843a6de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9063a6e0, SUC_lexicon_tags_27 },
    { 0x1aa3a6e0, SUC_lexicon_tags_15 },
    { 0xb703a6e2, SUC_lexicon_tags_11 },
    { 0x5063a6e2, SUC_lexicon_tags_12 },
    { 0xa883a6e2, SUC_lexicon_tags_80 },
    { 0x4743a6e4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c3a6ea, SUC_lexicon_tags_7 },
    { 0x71a3a6eb, SUC_lexicon_tags_32 },
    { 0x2263a6ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c3a6f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1623a6f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2443a6fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8683a6fd, SUC_lexicon_tags_2 },
    { 0x69a3a6fe, SUC_lexicon_tags_7 },
    { 0x81c3a6fe, SUC_lexicon_tags_84 },
    { 0xa2e3a700, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce3a70a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec03a710, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf423a714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac3a718, SUC_lexicon_tags_33 },
    { 0xe823a719, SUC_lexicon_tags_39 },
    { 0x4b03a71a, SUC_lexicon_tags_79 },
    { 0x3f83a719, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4023a721, SUC_lexicon_tags_63 },
    { 0xadc3a721, SUC_lexicon_tags_11 },
    { 0x4dc3a723, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xfac3a725, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7583a72d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd283a731, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad63a735, SUC_lexicon_tags_2 },
    { 0x1c43a735, SUC_lexicon_tags_11 },
    { 0x1e63a737, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c43a740, SUC_lexicon_tags_43 },
    { 0x11a3a741, SUC_lexicon_tags_17 },
    { 0x9583a742, SUC_lexicon_tags_43 },
    { 0x8683a742, SUC_lexicon_tags_20 },
    { 0xdae3a743, SUC_lexicon_tags_30 },
    { 0x5de3a745, SUC_lexicon_tags_17 },
    { 0xbec3a745, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf663a748, SUC_lexicon_tags_47 },
    { 0x4dc3a749, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc3a74b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a3a74f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c83a751, SUC_lexicon_tags_31 },
    { 0x7303a752, SUC_lexicon_tags_15 },
    { 0xd903a752, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a3a759, SUC_lexicon_tags_11 },
    { 0x8e03a75a, SUC_lexicon_tags_35 },
    { 0xea63a75b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7983a75d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6983a75f, SUC_lexicon_tags_38 },
    { 0xe6c3a75f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa83a765, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d23a767, SUC_lexicon_tags_32 },
    { 0x3a03a767, SUC_lexicon_tags_56 },
    { 0x4e63a767, SUC_lexicon_tags_15 },
    { 0x2a3a76a, SUC_lexicon_tags_11 },
    { 0xc603a769, SUC_lexicon_tags_1 },
    { 0x97e3a768, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e3a772, SUC_lexicon_tags_14 },
    { 0x33a3a772, SUC_lexicon_tags_68 },
    { 0xcba3a772, SUC_lexicon_tags_31 },
    { 0xfb43a774, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae63a77c, SUC_lexicon_tags_60 },
    { 0xa7e3a77d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x70c3a77f, SUC_lexicon_tags_1 },
    { 0x9543a780, SUC_lexicon_tags_2 },
    { 0xa523a781, SUC_lexicon_tags_11 },
    { 0x6b23a782, SUC_lexicon_tags_8 },
    { 0xcc63a783, SUC_lexicon_tags_1 },
    { 0x12c3a783, SUC_lexicon_tags_4 },
    { 0x9fa3a785, SUC_lexicon_tags_37 },
    { 0x1083a786, SUC_lexicon_tags_25 },
    { 0xecc3a782, SUC_lexicon_tags_45 },
    { 0x2143a788, SUC_lexicon_tags_0 },
    { 0x7a83a786, SUC_lexicon_tags_15 },
    { 0x1923a78a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8343a78d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea63a78f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e3a792, SUC_lexicon_tags_25 },
    { 0x4e43a792, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xd163a795, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b43a798, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e3a79b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a3a79e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2183a7a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e3a7a6, SUC_lexicon_tags_30 },
    { 0xff83a7a7, SUC_lexicon_tags_7 },
    { 0xbae3a7a8, SUC_lexicon_tags_25 },
    { 0x7f23a7a9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf363a7b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd803a7b8, SUC_lexicon_tags_14 },
    { 0xc7e3a7b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3243a7bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc843a7bf, SUC_lexicon_tags_47 },
    { 0x6a23a7c0, SUC_lexicon_tags_12 },
    { 0x6d43a7c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb3c3a7c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7743a7c5, SUC_lexicon_tags_14 },
    { 0xab43a7c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea3a7c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f63a7cd, SUC_lexicon_tags_25 },
    { 0xcf83a7ce, SUC_lexicon_tags_1 },
    { 0x40e3a7cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd683a7d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2ea3a7d4, SUC_lexicon_tags_2 },
    { 0xe943a7d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6743a7d9, SUC_lexicon_tags_19 },
    { 0x42c3a7d9, SUC_lexicon_tags_27 },
    { 0x33c3a7da, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5563a7e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb603a7e8, SUC_lexicon_tags_783 },
    { 0x23a7e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e3a7ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e3a7f0, SUC_lexicon_tags_7 },
    { 0xd303a7f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf223a7f3, SUC_lexicon_tags_463 },
    { 0x8063a7f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a3a7ff, SUC_lexicon_tags_17 },
    { 0x10a3a800, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3a803, SUC_lexicon_tags_47 },
    { 0xac03a804, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9703a806, SUC_lexicon_tags_28 },
    { 0xe863a807, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d63a809, SUC_lexicon_tags_7 },
    { 0x9143a80a, SUC_lexicon_tags_1 },
    { 0x9ba3a809, SUC_lexicon_tags_4 },
    { 0xfaa3a80a, SUC_lexicon_tags_37 },
    { 0x22e3a80d, SUC_lexicon_tags_11 },
    { 0xe663a80e, SUC_lexicon_tags_69 },
    { 0x8fc3a80e, SUC_lexicon_tags_7 },
    { 0xfcc3a80b, SUC_lexicon_tags_29 },
    { 0xc723a80a, SUC_lexicon_tags_70 },
    { 0x92a3a810, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ac3a814, SUC_lexicon_tags_2 },
    { 0x8f83a815, SUC_lexicon_tags_25 },
    { 0x8bc3a815, SUC_lexicon_tags_26 },
    { 0x67e3a816, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2963a81c, SUC_lexicon_tags_17 },
    { 0x3943a81d, SUC_lexicon_tags_60 },
    { 0xed83a81e, SUC_lexicon_tags_28 },
    { 0x3d03a81d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a826, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d03a829, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa383a82b, SUC_lexicon_tags_49 },
    { 0xeb23a82c, SUC_lexicon_tags_29 },
    { 0x3a83a82d, SUC_lexicon_tags_151 },
    { 0x6dc3a82c, SUC_lexicon_tags_0 },
    { 0x1183a82f, SUC_lexicon_tags_17 },
    { 0xfae3a830, SUC_lexicon_tags_29 },
    { 0x7023a82b, SUC_lexicon_tags_17 },
    { 0x5ec3a82e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b63a838, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x923a83b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb403a83d, SUC_lexicon_tags_31 },
    { 0x4c03a83e, SUC_lexicon_tags_11 },
    { 0xa803a83f, SUC_lexicon_tags_27 },
    { 0xcbe3a83e, SUC_lexicon_tags_65 },
    { 0x4ca3a83f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba83a843, SUC_lexicon_tags_11 },
    { 0xb423a844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea3a847, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc23a851, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xff23a854, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe643a857, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x53a3a859, SUC_lexicon_tags_2 },
    { 0x6363a85a, SUC_lexicon_tags_17 },
    { 0x2783a859, SUC_lexicon_tags_26 },
    { 0x1283a85c, SUC_lexicon_tags_95 },
    { 0x73c3a85d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace3a863, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c3a866, SUC_lexicon_tags_11 },
    { 0xf623a866, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xea23a869, SUC_lexicon_tags_7 },
    { 0x7063a869, SUC_lexicon_tags_23 },
    { 0xc9e3a86b, SUC_lexicon_tags_38 },
    { 0xb903a86b, SUC_lexicon_tags_32 },
    { 0xde3a86d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a3a872, SUC_lexicon_tags_17 },
    { 0x3723a873, SUC_lexicon_tags_43 },
    { 0x8023a872, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xbbe3a876, SUC_lexicon_tags_37 },
    { 0x8443a876, SUC_lexicon_tags_38 },
    { 0x723a878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc23a87a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a3a87f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe383a884, SUC_lexicon_tags_1 },
    { 0x70c3a884, SUC_lexicon_tags_15 },
    { 0xed23a886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f03a889, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc3a88d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a3a890, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da3a893, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44e3a895, SUC_lexicon_tags_843 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee3a898, SUC_lexicon_tags_51 },
    { 0xc463a898, SUC_lexicon_tags_56 },
    { 0xf43a89a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a3a89e, SUC_lexicon_tags_30 },
    { 0x1663a89f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a8a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca3a8a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c3a8b0, SUC_lexicon_tags_14 },
    { 0x3da3a8b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc003a8b9, SUC_lexicon_tags_39 },
    { 0xe503a8ba, SUC_lexicon_tags_13 },
    { 0x4c83a8bb, SUC_lexicon_tags_10 },
    { 0x43e3a8ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x12c3a8be, SUC_lexicon_tags_12 },
    { 0x69a3a8be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92e3a8c1, SUC_lexicon_tags_2 },
    { 0x13a3a8c2, SUC_lexicon_tags_1 },
    { 0x27e3a8c3, SUC_lexicon_tags_15 },
    { 0x3363a8c4, SUC_lexicon_tags_7 },
    { 0x5c3a8c5, SUC_lexicon_tags_23 },
    { 0xb523a8c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8423a8c8, SUC_lexicon_tags_6 },
    { 0x2723a8c9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3b03a8cb, SUC_lexicon_tags_21 },
    { 0xf703a8cb, SUC_lexicon_tags_17 },
    { 0xe623a8cd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xca83a8d0, SUC_lexicon_tags_25 },
    { 0x2c23a8d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2043a8d3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde23a8d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe423a8db, SUC_lexicon_tags_15 },
    { 0x85a3a8db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f63a8e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2083a8f6, SUC_lexicon_tags_15 },
    { 0x72a3a8f7, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x8443a8fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7103a8fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5023a901, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3223a904, SUC_lexicon_tags_60 },
    { 0xad83a905, SUC_lexicon_tags_43 },
    { 0xb183a906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a3a90b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb343a90f, SUC_lexicon_tags_31 },
    { 0xd263a910, SUC_lexicon_tags_37 },
    { 0xc763a910, SUC_lexicon_tags_28 },
    { 0x9ee3a912, SUC_lexicon_tags_17 },
    { 0x68c3a911, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa903a91a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3503a91c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9783a91e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a3a922, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c3a92a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee3a932, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe763a934, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8e3a936, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda3a93f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa323a941, SUC_lexicon_tags_12 },
    { 0xd0a3a942, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4a63a944, SUC_lexicon_tags_7 },
    { 0x2a83a945, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7263a949, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf23a94e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7043a950, SUC_lexicon_tags_33 },
    { 0x8a43a951, SUC_lexicon_tags_30 },
    { 0x2363a952, SUC_lexicon_tags_0 },
    { 0x35e3a952, SUC_lexicon_tags_15 },
    { 0x7663a954, SUC_lexicon_tags_37 },
    { 0x6183a954, SUC_lexicon_tags_11 },
    { 0xff23a954, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa03a958, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec43a95c, SUC_lexicon_tags_7 },
    { 0x863a95c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a3a967, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8943a96a, SUC_lexicon_tags_23 },
    { 0xd943a96b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f83a971, SUC_lexicon_tags_47 },
    { 0x21a3a972, SUC_lexicon_tags_37 },
    { 0x2483a972, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd423a976, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x463a978, SUC_lexicon_tags_61 },
    { 0xfb03a979, SUC_lexicon_tags_21 },
    { 0xb823a979, SUC_lexicon_tags_23 },
    { 0xb7e3a97b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1e3a97d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d43a986, SUC_lexicon_tags_37 },
    { 0xc8e3a987, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c63a98a, SUC_lexicon_tags_26 },
    { 0x29e3a98b, SUC_lexicon_tags_6 },
    { 0x7fc3a98b, SUC_lexicon_tags_33 },
    { 0xb983a98d, SUC_lexicon_tags_37 },
    { 0x7463a98e, SUC_lexicon_tags_11 },
    { 0xb223a98e, SUC_lexicon_tags_11 },
    { 0x4343a98d, SUC_lexicon_tags_17 },
    { 0x5a43a98b, SUC_lexicon_tags_37 },
    { 0x7783a992, SUC_lexicon_tags_23 },
    { 0xb03a993, SUC_lexicon_tags_45 },
    { 0xe5e3a994, SUC_lexicon_tags_37 },
    { 0x7cc3a993, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe363a998, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4263a99a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23a9a6, SUC_lexicon_tags_43 },
    { 0xfa63a9a7, SUC_lexicon_tags_39 },
    { 0xdb43a9a6, SUC_lexicon_tags_17 },
    { 0x3ac3a9a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca3a9ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e3a9b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9903a9b8, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd43a9be, SUC_lexicon_tags_4 },
    { 0xafe3a9bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe43a9c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc3a9c7, SUC_lexicon_tags_33 },
    { 0x1703a9c8, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x5c3a9ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d03a9cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe923a9ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa743a9d0, SUC_lexicon_tags_25 },
    { 0x74a3a9d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe0a3a9d3, SUC_lexicon_tags_33 },
    { 0x97e3a9d4, SUC_lexicon_tags_14 },
    { 0xf023a9d5, SUC_lexicon_tags_65 },
    { 0xb883a9d3, SUC_lexicon_tags_9 },
    { 0x2ec3a9d7, SUC_lexicon_tags_3 },
    { 0x65e3a9d5, SUC_lexicon_tags_37 },
    { 0xfb03a9d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16e3a9db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b63a9df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce23a9e4, SUC_lexicon_tags_7 },
    { 0x9363a9e5, SUC_lexicon_tags_37 },
    { 0x78e3a9e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe243a9ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a3a9ef, SUC_lexicon_tags_61 },
    { 0x4923a9ef, SUC_lexicon_tags_14 },
    { 0x63a3a9f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf23a9f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6943a9f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea3a9fd, SUC_lexicon_tags_91 },
    { 0x6c43a9fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9f83aa00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63aa0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ce3aa0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6423aa13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c3aa17, SUC_lexicon_tags_27 },
    { 0x1963aa17, SUC_lexicon_tags_25 },
    { 0xbac3aa19, SUC_lexicon_tags_14 },
    { 0x4063aa1a, SUC_lexicon_tags_17 },
    { 0xf1a3aa1b, SUC_lexicon_tags_33 },
    { 0x4fe3aa1b, SUC_lexicon_tags_1 },
    { 0xc943aa1a, SUC_lexicon_tags_65 },
    { 0x6043aa1d, SUC_lexicon_tags_17 },
    { 0x6d83aa1f, SUC_lexicon_tags_7 },
    { 0xe8c3aa20, SUC_lexicon_tags_1 },
    { 0x6e63aa1a, SUC_lexicon_tags_19 },
    { 0x8c23aa22, SUC_lexicon_tags_7 },
    { 0x1c43aa1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea3aa26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c3aa29, SUC_lexicon_tags_37 },
    { 0xf43aa29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac3aa2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1343aa30, SUC_lexicon_tags_16 },
    { 0xe7a3aa31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38c3aa33, SUC_lexicon_tags_37 },
    { 0x9403aa34, SUC_lexicon_tags_55 },
    { 0x7fc3aa33, SUC_lexicon_tags_11 },
    { 0x7dc3aa33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a3aa3b, SUC_lexicon_tags_37 },
    { 0x3243aa3c, SUC_lexicon_tags_87 },
    { 0xc9a3aa3d, SUC_lexicon_tags_10 },
    { 0x4163aa3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd503aa46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbe3aa48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c3aa4b, SUC_lexicon_tags_0 },
    { 0x4903aa4b, SUC_lexicon_tags_11 },
    { 0xde43aa4d, SUC_lexicon_tags_11 },
    { 0x45e3aa4c, SUC_lexicon_tags_7 },
    { 0x1803aa4c, SUC_lexicon_tags_17 },
    { 0xe003aa4b, SUC_lexicon_tags_911 },
    { 0xe9a3aa51, SUC_lexicon_tags_28 },
    { 0xa8a3aa52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b83aa54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea3aa59, SUC_lexicon_tags_4 },
    { 0x4683aa5a, SUC_lexicon_tags_2 },
    { 0x3943aa5b, SUC_lexicon_tags_38 },
    { 0x3aa5c, SUC_lexicon_tags_2 },
    { 0xe083aa5d, SUC_lexicon_tags_37 },
    { 0x6dc3aa5e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa843aa63, SUC_lexicon_tags_12 },
    { 0x7e83aa64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42e3aa66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a83aa68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fa3aa6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc043aa6d, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d23aa74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe83aa76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb223aa78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda23aa7d, SUC_lexicon_tags_17 },
    { 0x77a3aa7e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf03aa85, SUC_lexicon_tags_17 },
    { 0x21c3aa86, SUC_lexicon_tags_82 },
    { 0xbf43aa86, SUC_lexicon_tags_11 },
    { 0xec03aa86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63aa8f, SUC_lexicon_tags_25 },
    { 0x36e3aa90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb23aa92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c23aa98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a3aa9b, SUC_lexicon_tags_43 },
    { 0x7163aa9b, SUC_lexicon_tags_11 },
    { 0xe5a3aa9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7cc3aa9f, SUC_lexicon_tags_12 },
    { 0x8f83aa9f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf703aaa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc3aaa7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3aaaa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd283aaac, SUC_lexicon_tags_74 },
    { 0xffa3aaac, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c3aab2, SUC_lexicon_tags_1 },
    { 0x56a3aab3, SUC_lexicon_tags_54 },
    { 0xa2e3aab4, SUC_lexicon_tags_111 },
    { 0xe863aab2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2103aab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2543aabc, SUC_lexicon_tags_2 },
    { 0xf8a3aabd, SUC_lexicon_tags_14 },
    { 0xf003aabc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfe3aac0, SUC_lexicon_tags_1 },
    { 0x6e83aac0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb23aac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a43aac9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec3aacd, SUC_lexicon_tags_6 },
    { 0x2b23aace, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf363aad0, SUC_lexicon_tags_47 },
    { 0x28e3aad1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a3aad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dc3aad7, SUC_lexicon_tags_11 },
    { 0xab03aad8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fc3aada, SUC_lexicon_tags_37 },
    { 0xeda3aadb, SUC_lexicon_tags_48 },
    { 0x7ca3aadc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee3aae2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d43aae6, SUC_lexicon_tags_47 },
    { 0x1343aae7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2543aae9, SUC_lexicon_tags_95 },
    { 0xd383aaea, SUC_lexicon_tags_12 },
    { 0xdc03aaeb, SUC_lexicon_tags_57 },
    { 0xb2e3aaec, SUC_lexicon_tags_25 },
    { 0x5d83aaec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb43aaf1, SUC_lexicon_tags_37 },
    { 0x68a3aaf2, SUC_lexicon_tags_25 },
    { 0x6683aaf2, SUC_lexicon_tags_60 },
    { 0xc223aaf4, SUC_lexicon_tags_17 },
    { 0x6dc3aaf3, SUC_lexicon_tags_7 },
    { 0xbf43aaf6, SUC_lexicon_tags_27 },
    { 0x8783aaf6, SUC_lexicon_tags_10 },
    { 0xd0a3aaf2, SUC_lexicon_tags_7 },
    { 0x2ac3aaf8, SUC_lexicon_tags_14 },
    { 0xdde3aafa, SUC_lexicon_tags_68 },
    { 0xd3a3aafb, SUC_lexicon_tags_17 },
    { 0x49e3aafb, SUC_lexicon_tags_12 },
    { 0x5503aafd, SUC_lexicon_tags_2 },
    { 0xa063aafe, SUC_lexicon_tags_1 },
    { 0x9e03aaff, SUC_lexicon_tags_33 },
    { 0x6943ab00, SUC_lexicon_tags_1 },
    { 0x6903ab01, SUC_lexicon_tags_1 },
    { 0xf63ab02, SUC_lexicon_tags_0 },
    { 0x5063ab02, SUC_lexicon_tags_11 },
    { 0xd423aafa, SUC_lexicon_tags_19 },
    { 0x6363ab01, SUC_lexicon_tags_37 },
    { 0xe003ab06, SUC_lexicon_tags_17 },
    { 0x923ab06, SUC_lexicon_tags_25 },
    { 0x783ab08, SUC_lexicon_tags_15 },
    { 0xec03ab09, SUC_lexicon_tags_30 },
    { 0xf923ab03, SUC_lexicon_tags_0 },
    { 0x9de3ab0b, SUC_lexicon_tags_48 },
    { 0xd703aaf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c3ab12, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e3ab16, SUC_lexicon_tags_15 },
    { 0x67e3ab17, SUC_lexicon_tags_25 },
    { 0x2103ab17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c3ab1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a3ab1d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b63ab21, SUC_lexicon_tags_63 },
    { 0x1a3ab22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7983ab24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c3ab2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d23ab2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9483ab30, SUC_lexicon_tags_224 },
    { 0x7883ab31, SUC_lexicon_tags_39 },
    { 0x6cc3ab31, SUC_lexicon_tags_1 },
    { 0x3323ab33, SUC_lexicon_tags_1 },
    { 0xdee3ab32, SUC_lexicon_tags_37 },
    { 0xac63ab31, SUC_lexicon_tags_19 },
    { 0x4fa3ab36, SUC_lexicon_tags_66 },
    { 0xa4e3ab37, SUC_lexicon_tags_1 },
    { 0xfd23ab38, SUC_lexicon_tags_14 },
    { 0x7783ab39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x64a3ab3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e3ab44, SUC_lexicon_tags_359 },
    { 0x4ac3ab45, SUC_lexicon_tags_23 },
    { 0xf683ab45, SUC_lexicon_tags_224 },
    { 0x7983ab45, SUC_lexicon_tags_25 },
    { 0xe1a3ab45, SUC_lexicon_tags_28 },
    { 0x5663ab49, SUC_lexicon_tags_4 },
    { 0x5143ab46, SUC_lexicon_tags_37 },
    { 0xbde3ab4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e3ab50, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd443ab58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c63ab63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe763ab65, SUC_lexicon_tags_13 },
    { 0x93c3ab66, SUC_lexicon_tags_1 },
    { 0xdb23ab67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3103ab6a, SUC_lexicon_tags_43 },
    { 0x7443ab6b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe943ab6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee43ab72, SUC_lexicon_tags_69 },
    { 0x8d63ab72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd83ab75, SUC_lexicon_tags_37 },
    { 0x4303ab75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec3ab7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc983ab7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a03ab81, SUC_lexicon_tags_17 },
    { 0x9683ab82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1323ab84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883ab89, SUC_lexicon_tags_1 },
    { 0xa643ab8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3fc3ab8c, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x7c43ab8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3ab91, SUC_lexicon_tags_38 },
    { 0x8c63ab91, SUC_lexicon_tags_70 },
    { 0x8763ab92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a23ab95, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf063ab99, SUC_lexicon_tags_23 },
    { 0x1ea3ab9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5883ab9e, SUC_lexicon_tags_37 },
    { 0xef23ab9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cc3aba1, SUC_lexicon_tags_37 },
    { 0x1c23aba2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa03aba6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb23aba9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a03abab, SUC_lexicon_tags_65 },
    { 0x6ec3abac, SUC_lexicon_tags_4 },
    { 0x8983abad, SUC_lexicon_tags_63 },
    { 0x8e3abad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a3abb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63a3abb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf023abbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c3abc3, SUC_lexicon_tags_11 },
    { 0x4403abc4, SUC_lexicon_tags_15 },
    { 0x35c3abc4, SUC_lexicon_tags_1 },
    { 0xc1c3abc4, SUC_lexicon_tags_37 },
    { 0x2ac3abc7, SUC_lexicon_tags_536 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5323abce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c23abd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3603abd9, SUC_lexicon_tags_16 },
    { 0xfda3abda, SUC_lexicon_tags_63 },
    { 0x6443abd9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a3abe1, SUC_lexicon_tags_8 },
    { 0x1023abe1, SUC_lexicon_tags_0 },
    { 0xe263abe1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b83abe5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b83abe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa843abe9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc183abf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1543abf5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c43abf9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc103abfb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c83ac00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66e3ac02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9463ac08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1943ac0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf523ac0e, SUC_lexicon_tags_23 },
    { 0x2e23ac0f, SUC_lexicon_tags_2 },
    { 0x4043ac0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa143ac14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd383ac17, SUC_lexicon_tags_11 },
    { 0x2423ac18, SUC_lexicon_tags_2 },
    { 0xe23ac18, SUC_lexicon_tags_23 },
    { 0xbec3ac1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae83ac1c, SUC_lexicon_tags_0 },
    { 0x3923ac1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa283ac21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a3ac25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4563ac2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe23ac2d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6343ac32, SUC_lexicon_tags_7 },
    { 0x3e03ac33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x303ac36, SUC_lexicon_tags_39 },
    { 0x6123ac36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e83ac39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8443ac3b, SUC_lexicon_tags_46 },
    { 0xdbc3ac3c, SUC_lexicon_tags_17 },
    { 0xb0a3ac3d, SUC_lexicon_tags_11 },
    { 0x8523ac3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f63ac41, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a3ac4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7103ac4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf623ac57, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee3ac5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2723ac61, SUC_lexicon_tags_27 },
    { 0xb043ac62, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a3ac6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xace3ac6c, SUC_lexicon_tags_25 },
    { 0xa6e3ac6d, SUC_lexicon_tags_2 },
    { 0x3683ac6d, SUC_lexicon_tags_23 },
    { 0xdbc3ac6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80c3ac71, SUC_lexicon_tags_11 },
    { 0x7f43ac71, SUC_lexicon_tags_7 },
    { 0x8a43ac72, SUC_lexicon_tags_11 },
    { 0x4723ac74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea3ac77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9963ac7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e23ac7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d23ac83, SUC_lexicon_tags_47 },
    { 0x4a3ac83, SUC_lexicon_tags_12 },
    { 0x8dc3ac84, SUC_lexicon_tags_191 },
    { 0xf9c3ac86, SUC_lexicon_tags_0 },
    { 0x8ac3ac87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03ac8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d03ac91, SUC_lexicon_tags_11 },
    { 0xb483ac92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3ac9b, SUC_lexicon_tags_25 },
    { 0xf9c3ac9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf83aca2, SUC_lexicon_tags_25 },
    { 0x33a3aca2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a03aca6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf43acac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1243acaf, SUC_lexicon_tags_6 },
    { 0x4563acb0, SUC_lexicon_tags_72 },
    { 0x2443acb1, SUC_lexicon_tags_7 },
    { 0x9223acaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb23acb9, SUC_lexicon_tags_54 },
    { 0x5ca3acb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1503acbf, SUC_lexicon_tags_11 },
    { 0xac63acc0, SUC_lexicon_tags_12 },
    { 0x123acbf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb883acc3, SUC_lexicon_tags_37 },
    { 0x8463acc4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ec3acc6, SUC_lexicon_tags_15 },
    { 0x2143acc7, SUC_lexicon_tags_1 },
    { 0xd603acc7, SUC_lexicon_tags_37 },
    { 0x8283acc9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6783acce, SUC_lexicon_tags_17 },
    { 0x7a83accf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a3acd9, SUC_lexicon_tags_7 },
    { 0x60c3acda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc303acdd, SUC_lexicon_tags_7 },
    { 0x4f43acde, SUC_lexicon_tags_61 },
    { 0x2863acdf, SUC_lexicon_tags_8 },
    { 0x7603acdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa303ace4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf683ace6, SUC_lexicon_tags_7 },
    { 0xde03ace6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e23acea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3acef, SUC_lexicon_tags_147 },
    { 0, NULL },
    { 0xc123acf1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a3acf6, SUC_lexicon_tags_19 },
    { 0x8b63acf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2303acf9, SUC_lexicon_tags_31 },
    { 0x4843acf9, SUC_lexicon_tags_17 },
    { 0x6783acfb, SUC_lexicon_tags_25 },
    { 0xe0e3acfc, SUC_lexicon_tags_37 },
    { 0x1723acfd, SUC_lexicon_tags_17 },
    { 0x4b63acfb, SUC_lexicon_tags_17 },
    { 0xe323acfc, SUC_lexicon_tags_30 },
    { 0xfd43ad00, SUC_lexicon_tags_8 },
    { 0x9663ad01, SUC_lexicon_tags_28 },
    { 0x7523ad00, SUC_lexicon_tags_14 },
    { 0x21e3ad02, SUC_lexicon_tags_25 },
    { 0x1a63ad04, SUC_lexicon_tags_47 },
    { 0x5c03ad05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3da3ad07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9183ad0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a43ad10, SUC_lexicon_tags_1 },
    { 0xb063ad10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb703ad13, SUC_lexicon_tags_15 },
    { 0xc923ad14, SUC_lexicon_tags_12 },
    { 0xaa23ad15, SUC_lexicon_tags_11 },
    { 0xe363ad14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc3ad1e, SUC_lexicon_tags_1 },
    { 0xa403ad1f, SUC_lexicon_tags_2 },
    { 0x8f03ad20, SUC_lexicon_tags_25 },
    { 0x8b63ad20, SUC_lexicon_tags_29 },
    { 0x6b23ad20, SUC_lexicon_tags_34 },
    { 0xd1c3ad23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8183ad25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59e3ad27, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf763ad2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb8c3ad2e, SUC_lexicon_tags_0 },
    { 0x54e3ad2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd663ad36, SUC_lexicon_tags_7 },
    { 0x2503ad36, SUC_lexicon_tags_1 },
    { 0x6643ad36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe3ad3e, SUC_lexicon_tags_85 },
    { 0x1da3ad3e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x51c3ad41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc3ad46, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6943ad48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5943ad4e, SUC_lexicon_tags_10 },
    { 0xd403ad4e, SUC_lexicon_tags_47 },
    { 0xe583ad50, SUC_lexicon_tags_14 },
    { 0x8ce3ad50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7643ad5f, SUC_lexicon_tags_13 },
    { 0x5de3ad5f, SUC_lexicon_tags_1 },
    { 0x3663ad61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1283ad64, SUC_lexicon_tags_37 },
    { 0x1323ad65, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3ad68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83c3ad6a, SUC_lexicon_tags_47 },
    { 0x963ad6b, SUC_lexicon_tags_45 },
    { 0xb2c3ad6b, SUC_lexicon_tags_1 },
    { 0xb063ad6d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe23ad74, SUC_lexicon_tags_7 },
    { 0xdea3ad75, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9663ad77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfba3ad79, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb23ad7e, SUC_lexicon_tags_60 },
    { 0x8fc3ad7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1e3ad81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1583ad84, SUC_lexicon_tags_23 },
    { 0xaf03ad85, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6003ad87, SUC_lexicon_tags_4 },
    { 0x5de3ad88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a23ad8d, SUC_lexicon_tags_48 },
    { 0x7543ad8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf243ad92, SUC_lexicon_tags_39 },
    { 0x7d23ad93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf243ad96, SUC_lexicon_tags_4 },
    { 0x9803ad97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b63ad9c, SUC_lexicon_tags_28 },
    { 0xe983ad9d, SUC_lexicon_tags_43 },
    { 0xd743ad9e, SUC_lexicon_tags_12 },
    { 0xb843ad9f, SUC_lexicon_tags_138 },
    { 0x9f03ad9f, SUC_lexicon_tags_1 },
    { 0x8703ada1, SUC_lexicon_tags_25 },
    { 0xdb63ada1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x63e3ada4, SUC_lexicon_tags_25 },
    { 0xee03ada4, SUC_lexicon_tags_2 },
    { 0x7da3ada6, SUC_lexicon_tags_27 },
    { 0xb943ada7, SUC_lexicon_tags_65 },
    { 0x60e3ada8, SUC_lexicon_tags_27 },
    { 0x2283ada4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d23adab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1243adb2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4443adb4, SUC_lexicon_tags_23 },
    { 0x20e3adb5, SUC_lexicon_tags_25 },
    { 0x2823adb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee3adb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3adbd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9503adc2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6443adc5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1223adc9, SUC_lexicon_tags_30 },
    { 0x2843adca, SUC_lexicon_tags_33 },
    { 0x6e63adcb, SUC_lexicon_tags_25 },
    { 0x48e3adcb, SUC_lexicon_tags_11 },
    { 0x7a3adcd, SUC_lexicon_tags_298 },
    { 0x6f03adce, SUC_lexicon_tags_12 },
    { 0xa8c3adcf, SUC_lexicon_tags_279 },
    { 0x9bc3adcf, SUC_lexicon_tags_1 },
    { 0x69a3add1, SUC_lexicon_tags_11 },
    { 0x1f03adcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe843add5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e3ade8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8323adec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9483adee, SUC_lexicon_tags_79 },
    { 0x4623adee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x143adf1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc403adf5, SUC_lexicon_tags_17 },
    { 0x5263adf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac3adf9, SUC_lexicon_tags_7 },
    { 0x3883adfa, SUC_lexicon_tags_8 },
    { 0x9ca3adfa, SUC_lexicon_tags_7 },
    { 0x19a3adfc, SUC_lexicon_tags_70 },
    { 0xc983adfa, SUC_lexicon_tags_21 },
    { 0x32c3adf9, SUC_lexicon_tags_13 },
    { 0x4e03adfe, SUC_lexicon_tags_15 },
    { 0x7c43adf9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdbc3ae02, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde3ae05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de3ae0c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1ea3ae0e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca3ae17, SUC_lexicon_tags_27 },
    { 0xb103ae17, SUC_lexicon_tags_47 },
    { 0x1ac3ae17, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x71c3ae1b, SUC_lexicon_tags_37 },
    { 0x9ca3ae1b, SUC_lexicon_tags_38 },
    { 0x6fa3ae1d, SUC_lexicon_tags_28 },
    { 0x9083ae1e, SUC_lexicon_tags_37 },
    { 0x7483ae1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb683ae22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae3ae26, SUC_lexicon_tags_2 },
    { 0xe283ae27, SUC_lexicon_tags_65 },
    { 0xbe43ae28, SUC_lexicon_tags_15 },
    { 0x6843ae29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd803ae2b, SUC_lexicon_tags_18 },
    { 0x64e3ae2c, SUC_lexicon_tags_2 },
    { 0x45c3ae2d, SUC_lexicon_tags_14 },
    { 0x3ba3ae2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe503ae32, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b43ae38, SUC_lexicon_tags_12 },
    { 0xe643ae39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc03ae3c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a23ae3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9043ae43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6b63ae45, SUC_lexicon_tags_9 },
    { 0xca3ae46, SUC_lexicon_tags_55 },
    { 0xf003ae47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada3ae4d, SUC_lexicon_tags_15 },
    { 0xa223ae4e, SUC_lexicon_tags_11 },
    { 0x4983ae4e, SUC_lexicon_tags_12 },
    { 0x6c83ae50, SUC_lexicon_tags_11 },
    { 0x2263ae4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c3ae60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e83ae63, SUC_lexicon_tags_60 },
    { 0xfa43ae64, SUC_lexicon_tags_8 },
    { 0x8b23ae64, SUC_lexicon_tags_37 },
    { 0x4523ae64, SUC_lexicon_tags_0 },
    { 0xd323ae65, SUC_lexicon_tags_45 },
    { 0xc283ae65, SUC_lexicon_tags_15 },
    { 0xd4a3ae66, SUC_lexicon_tags_17 },
    { 0xc943ae68, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3ae6e, SUC_lexicon_tags_57 },
    { 0x7e23ae6f, SUC_lexicon_tags_11 },
    { 0xc623ae6f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2183ae77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1843ae79, SUC_lexicon_tags_45 },
    { 0xdbe3ae7a, SUC_lexicon_tags_15 },
    { 0x9643ae7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6e3ae7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea3ae7f, SUC_lexicon_tags_11 },
    { 0x7923ae7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b83ae85, SUC_lexicon_tags_141 },
    { 0x7463ae86, SUC_lexicon_tags_9 },
    { 0x70c3ae87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc823ae8c, SUC_lexicon_tags_1 },
    { 0xeda3ae8d, SUC_lexicon_tags_38 },
    { 0xfe03ae8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c43ae95, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b43ae99, SUC_lexicon_tags_30 },
    { 0x3983ae99, SUC_lexicon_tags_25 },
    { 0x8463ae9b, SUC_lexicon_tags_11 },
    { 0x2a3ae9b, SUC_lexicon_tags_2 },
    { 0x623ae9d, SUC_lexicon_tags_5 },
    { 0xfa83ae99, SUC_lexicon_tags_25 },
    { 0xa1a3ae9e, SUC_lexicon_tags_1 },
    { 0x53c3aea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73a3aea2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a3aea6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf363aead, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x383aeaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe3aeb2, SUC_lexicon_tags_3 },
    { 0xa0a3aeb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f23aebb, SUC_lexicon_tags_15 },
    { 0xca83aebc, SUC_lexicon_tags_12 },
    { 0x2823aebd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2103aebf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c3aec2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe503aec4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9183aecc, SUC_lexicon_tags_0 },
    { 0xfe3aecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b83aed4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa603aed6, SUC_lexicon_tags_2 },
    { 0xeaa3aed7, SUC_lexicon_tags_21 },
    { 0xb863aed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa163aee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8723aee2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc3aee7, SUC_lexicon_tags_15 },
    { 0xda63aee7, SUC_lexicon_tags_1 },
    { 0x143aee9, SUC_lexicon_tags_37 },
    { 0xe963aeea, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe643aef3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba3aef6, SUC_lexicon_tags_55 },
    { 0xe123aef6, SUC_lexicon_tags_16 },
    { 0xc523aef8, SUC_lexicon_tags_60 },
    { 0x1be3aef6, SUC_lexicon_tags_63 },
    { 0xbba3aef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c3aefd, SUC_lexicon_tags_35 },
    { 0x9a3aefe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe363af00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c3af03, SUC_lexicon_tags_9 },
    { 0x4983af04, SUC_lexicon_tags_14 },
    { 0x9163af05, SUC_lexicon_tags_43 },
    { 0xc0e3af05, SUC_lexicon_tags_39 },
    { 0xfb83af03, SUC_lexicon_tags_7 },
    { 0x1e3af05, SUC_lexicon_tags_29 },
    { 0xfb63af05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96a3af0b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc3af0e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc63af12, SUC_lexicon_tags_17 },
    { 0x803af13, SUC_lexicon_tags_25 },
    { 0xf083af14, SUC_lexicon_tags_11 },
    { 0x34a3af12, SUC_lexicon_tags_0 },
    { 0xca03af16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb43af19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8203af1e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x77c3af20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d63af22, SUC_lexicon_tags_2 },
    { 0xfe03af23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9c3af25, SUC_lexicon_tags_25 },
    { 0x5843af26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea3af2a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xdce3af2c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c03af2f, SUC_lexicon_tags_14 },
    { 0xa623af2f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7123af37, SUC_lexicon_tags_140 },
    { 0x3ba3af38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf223af3a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda3af3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee3af43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe23af46, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8183af48, SUC_lexicon_tags_31 },
    { 0x2a63af48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa383af4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163af50, SUC_lexicon_tags_1 },
    { 0x2443af51, SUC_lexicon_tags_11 },
    { 0x2fa3af51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa263af57, SUC_lexicon_tags_2 },
    { 0x77a3af58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b63af5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd03af5c, SUC_lexicon_tags_30 },
    { 0x84c3af5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc23af61, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3603af63, SUC_lexicon_tags_236 },
    { 0xfa03af64, SUC_lexicon_tags_49 },
    { 0x14c3af63, SUC_lexicon_tags_13 },
    { 0xe23af66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5063af70, SUC_lexicon_tags_1 },
    { 0xec23af70, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9c43af73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c63af75, SUC_lexicon_tags_16 },
    { 0x5723af76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d83af7d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e3af82, SUC_lexicon_tags_15 },
    { 0x57a3af83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c63af87, SUC_lexicon_tags_27 },
    { 0x8283af88, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa3af8d, SUC_lexicon_tags_30 },
    { 0xb563af8d, SUC_lexicon_tags_1 },
    { 0x86c3af8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1443af92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7583af95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca3af9a, SUC_lexicon_tags_19 },
    { 0xb63af9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c3afa1, SUC_lexicon_tags_57 },
    { 0x1703afa1, SUC_lexicon_tags_29 },
    { 0x11c3afa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c3afa7, SUC_lexicon_tags_37 },
    { 0xd503afa8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a3afae, SUC_lexicon_tags_17 },
    { 0xfcc3afaf, SUC_lexicon_tags_8 },
    { 0x2a43afae, SUC_lexicon_tags_37 },
    { 0x8e83afaf, SUC_lexicon_tags_43 },
    { 0x223afb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf383afb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2803afba, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc983afbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc83afbe, SUC_lexicon_tags_66 },
    { 0x4823afbe, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xb703afc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd823afc7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x943afca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a03afcc, SUC_lexicon_tags_60 },
    { 0xe003afcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74e3afcf, SUC_lexicon_tags_32 },
    { 0xcf83afcf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd843afd4, SUC_lexicon_tags_15 },
    { 0x7023afd4, SUC_lexicon_tags_5 },
    { 0xcbe3afd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3843afd8, SUC_lexicon_tags_47 },
    { 0x3a83afd9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9e43afdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2223afdd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe603afdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe823afe1, SUC_lexicon_tags_17 },
    { 0xfc63afe2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd263afe4, SUC_lexicon_tags_2 },
    { 0x5c63afe5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa403afea, SUC_lexicon_tags_84 },
    { 0x2c03afeb, SUC_lexicon_tags_34 },
    { 0xea3afec, SUC_lexicon_tags_11 },
    { 0xb8e3afec, SUC_lexicon_tags_2 },
    { 0xf323afed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64e3aff0, SUC_lexicon_tags_63 },
    { 0x82c3aff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa803aff4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b43affa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5903affc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdbe3affe, SUC_lexicon_tags_60 },
    { 0x46a3affe, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8003b009, SUC_lexicon_tags_17 },
    { 0x2323b009, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e3b00e, SUC_lexicon_tags_7 },
    { 0xbe83b00e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d23b016, SUC_lexicon_tags_8 },
    { 0x8da3b017, SUC_lexicon_tags_27 },
    { 0x703b018, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4623b020, SUC_lexicon_tags_0 },
    { 0x7d03b020, SUC_lexicon_tags_25 },
    { 0x5323b022, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe783b028, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a3b02d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8f63b02f, SUC_lexicon_tags_17 },
    { 0x423b030, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2c3b032, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b43b034, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2343b036, SUC_lexicon_tags_7 },
    { 0xccc3b037, SUC_lexicon_tags_15 },
    { 0xef43b038, SUC_lexicon_tags_37 },
    { 0xed03b039, SUC_lexicon_tags_16 },
    { 0x12c3b039, SUC_lexicon_tags_9 },
    { 0x24e3b038, SUC_lexicon_tags_19 },
    { 0x7523b03b, SUC_lexicon_tags_63 },
    { 0x4883b03d, SUC_lexicon_tags_39 },
    { 0xeba3b03e, SUC_lexicon_tags_26 },
    { 0x7ac3b03e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa643b041, SUC_lexicon_tags_38 },
    { 0xe323b042, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xec03b044, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5323b048, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fa3b04a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8163b04f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c3b052, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe103b056, SUC_lexicon_tags_1 },
    { 0x7b03b057, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81a3b059, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1643b05d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b63b061, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e3b065, SUC_lexicon_tags_43 },
    { 0x8583b065, SUC_lexicon_tags_2 },
    { 0x7403b067, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda03b06c, SUC_lexicon_tags_16 },
    { 0x73a3b06c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb283b06f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9ec3b071, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a3b074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc243b07b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xde23b07e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb563b082, SUC_lexicon_tags_11 },
    { 0x3cc3b083, SUC_lexicon_tags_10 },
    { 0xfcc3b084, SUC_lexicon_tags_49 },
    { 0xd223b085, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe963b088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa3b08e, SUC_lexicon_tags_10 },
    { 0x5a63b08f, SUC_lexicon_tags_18 },
    { 0xb923b090, SUC_lexicon_tags_2 },
    { 0x5a83b090, SUC_lexicon_tags_17 },
    { 0xe3b08f, SUC_lexicon_tags_11 },
    { 0xda63b090, SUC_lexicon_tags_38 },
    { 0xf243b094, SUC_lexicon_tags_12 },
    { 0xda23b094, SUC_lexicon_tags_27 },
    { 0x4a43b092, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9463b098, SUC_lexicon_tags_16 },
    { 0x2ea3b098, SUC_lexicon_tags_47 },
    { 0xc483b099, SUC_lexicon_tags_1 },
    { 0x1bc3b09b, SUC_lexicon_tags_9 },
    { 0x50c3b09b, SUC_lexicon_tags_109 },
    { 0x643b09b, SUC_lexicon_tags_14 },
    { 0x75a3b09d, SUC_lexicon_tags_2 },
    { 0xac43b09f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a3b0a3, SUC_lexicon_tags_69 },
    { 0x18c3b0a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x48e3b0a6, SUC_lexicon_tags_39 },
    { 0x8603b0a7, SUC_lexicon_tags_38 },
    { 0x1ce3b0a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f83b0ab, SUC_lexicon_tags_14 },
    { 0x6083b0ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a3b0b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c3b0b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca43b0bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b23b0bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8263b0c0, SUC_lexicon_tags_19 },
    { 0xc1c3b0c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x163b0c3, SUC_lexicon_tags_40 },
    { 0xb643b0c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0e3b0c6, SUC_lexicon_tags_2 },
    { 0x1fe3b0c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f83b0c9, SUC_lexicon_tags_27 },
    { 0xbf43b0ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5583b0cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a3b0d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c3b0d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d63b0d5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4ac3b0d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e3b0dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe203b0df, SUC_lexicon_tags_7 },
    { 0x6f03b0e0, SUC_lexicon_tags_45 },
    { 0x26a3b0e1, SUC_lexicon_tags_12 },
    { 0x60e3b0e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e3b0e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa863b0f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd543b0f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa3b0f6, SUC_lexicon_tags_11 },
    { 0x61c3b0f7, SUC_lexicon_tags_38 },
    { 0xc3c3b0f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e3b0fd, SUC_lexicon_tags_23 },
    { 0x8bc3b0fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c03b106, SUC_lexicon_tags_63 },
    { 0x8323b107, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6243b10e, SUC_lexicon_tags_60 },
    { 0x4363b10f, SUC_lexicon_tags_14 },
    { 0x1e23b110, SUC_lexicon_tags_23 },
    { 0x14e3b111, SUC_lexicon_tags_38 },
    { 0x3303b110, SUC_lexicon_tags_7 },
    { 0xe1a3b110, SUC_lexicon_tags_43 },
    { 0x97c3b114, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x68c3b116, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c3b11a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc83b11d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe3b124, SUC_lexicon_tags_11 },
    { 0x9623b125, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a3b128, SUC_lexicon_tags_18 },
    { 0x6e3b128, SUC_lexicon_tags_11 },
    { 0xc1c3b12a, SUC_lexicon_tags_14 },
    { 0x2b43b12b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7803b12e, SUC_lexicon_tags_2 },
    { 0xe623b12e, SUC_lexicon_tags_12 },
    { 0x3a43b130, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e83b132, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb463b136, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9503b139, SUC_lexicon_tags_37 },
    { 0xa163b13a, SUC_lexicon_tags_9 },
    { 0x28e3b13b, SUC_lexicon_tags_65 },
    { 0x5103b13b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee03b144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x803b146, SUC_lexicon_tags_61 },
    { 0x1e83b147, SUC_lexicon_tags_49 },
    { 0x1a83b148, SUC_lexicon_tags_1 },
    { 0x62a3b149, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf0a3b14b, SUC_lexicon_tags_1 },
    { 0x3ca3b14c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a3b14f, SUC_lexicon_tags_55 },
    { 0x2163b150, SUC_lexicon_tags_84 },
    { 0xd743b14f, SUC_lexicon_tags_17 },
    { 0x1fc3b14f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1603b158, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6fe3b15a, SUC_lexicon_tags_9 },
    { 0x9903b15b, SUC_lexicon_tags_15 },
    { 0xf9e3b15a, SUC_lexicon_tags_30 },
    { 0x3a83b15b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc503b160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1803b162, SUC_lexicon_tags_17 },
    { 0x2fc3b163, SUC_lexicon_tags_37 },
    { 0x343b163, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4863b168, SUC_lexicon_tags_11 },
    { 0xa143b168, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e83b16c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f43b170, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb4c3b172, SUC_lexicon_tags_8 },
    { 0x5b63b173, SUC_lexicon_tags_55 },
    { 0x15a3b172, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f63b179, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8383b17b, SUC_lexicon_tags_30 },
    { 0xe503b17c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e3b180, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca63b187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc3b18c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83a3b18e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f43b190, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1543b192, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e43b197, SUC_lexicon_tags_15 },
    { 0x9683b197, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7683b1a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38c3b1a8, SUC_lexicon_tags_6 },
    { 0x94a3b1a8, SUC_lexicon_tags_11 },
    { 0xf7c3b1aa, SUC_lexicon_tags_7 },
    { 0x9863b1aa, SUC_lexicon_tags_12 },
    { 0x183b1ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3b1af, SUC_lexicon_tags_37 },
    { 0xfc83b1af, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7283b1b2, SUC_lexicon_tags_17 },
    { 0x9c23b1b2, SUC_lexicon_tags_60 },
    { 0x2603b1b2, SUC_lexicon_tags_9 },
    { 0xf223b1b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x443b1b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8603b1ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be3b1bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca23b1c3, SUC_lexicon_tags_0 },
    { 0x7a63b1c4, SUC_lexicon_tags_26 },
    { 0xe5c3b1c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe403b1c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3b1cd, SUC_lexicon_tags_17 },
    { 0x2de3b1ce, SUC_lexicon_tags_19 },
    { 0x60c3b1cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb843b1d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa983b1d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2943b1db, SUC_lexicon_tags_12 },
    { 0xc663b1dc, SUC_lexicon_tags_14 },
    { 0x9583b1dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e3b1e0, SUC_lexicon_tags_65 },
    { 0xf283b1e1, SUC_lexicon_tags_19 },
    { 0x8fe3b1e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a3b1e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43b1ec, SUC_lexicon_tags_38 },
    { 0xb603b1ed, SUC_lexicon_tags_8 },
    { 0x3463b1ee, SUC_lexicon_tags_17 },
    { 0x3003b1ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5143b1f3, SUC_lexicon_tags_17 },
    { 0xda23b1f4, SUC_lexicon_tags_6 },
    { 0xae3b1f5, SUC_lexicon_tags_56 },
    { 0xdd83b1f6, SUC_lexicon_tags_37 },
    { 0x62e3b1f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4443b1fc, SUC_lexicon_tags_19 },
    { 0x6683b1fd, SUC_lexicon_tags_65 },
    { 0xcc43b1fe, SUC_lexicon_tags_11 },
    { 0x5223b1fc, SUC_lexicon_tags_53 },
    { 0x4a3b200, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3363b204, SUC_lexicon_tags_37 },
    { 0x2183b205, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1003b208, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5143b218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6603b21c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c3b220, SUC_lexicon_tags_84 },
    { 0xfda3b220, SUC_lexicon_tags_16 },
    { 0x11a3b221, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc483b228, SUC_lexicon_tags_17 },
    { 0x2903b229, SUC_lexicon_tags_65 },
    { 0xcaa3b22a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4ce3b22c, SUC_lexicon_tags_31 },
    { 0x4ae3b22c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x83b22f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7a3b231, SUC_lexicon_tags_60 },
    { 0x64e3b232, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f83b237, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e3b240, SUC_lexicon_tags_1 },
    { 0x39e3b240, SUC_lexicon_tags_33 },
    { 0x12a3b242, SUC_lexicon_tags_17 },
    { 0x2103b243, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8363b245, SUC_lexicon_tags_9 },
    { 0xa083b245, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf23b248, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d23b24d, SUC_lexicon_tags_14 },
    { 0x5823b24e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36e3b250, SUC_lexicon_tags_17 },
    { 0x1163b251, SUC_lexicon_tags_19 },
    { 0x24c3b252, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa63b25b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f03b25e, SUC_lexicon_tags_45 },
    { 0xcac3b25e, SUC_lexicon_tags_31 },
    { 0x21e3b25e, SUC_lexicon_tags_11 },
    { 0xcaa3b261, SUC_lexicon_tags_60 },
    { 0x403b262, SUC_lexicon_tags_39 },
    { 0xdda3b263, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x183b265, SUC_lexicon_tags_0 },
    { 0xb143b266, SUC_lexicon_tags_16 },
    { 0xcd63b265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc3b26a, SUC_lexicon_tags_62 },
    { 0x1c83b26b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa363b26d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1403b274, SUC_lexicon_tags_49 },
    { 0x9903b275, SUC_lexicon_tags_262 },
    { 0xddc3b276, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1363b278, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3b27b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea3b27e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca3b281, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2183b284, SUC_lexicon_tags_25 },
    { 0xf6a3b285, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x203b287, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e3b28c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbea3b28e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a3b291, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc3b295, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3303b298, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb323b29a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee83b29d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3be3b29f, SUC_lexicon_tags_12 },
    { 0xe303b29f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xa7e3b2a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3943b2a4, SUC_lexicon_tags_8 },
    { 0x683b2a5, SUC_lexicon_tags_7 },
    { 0xd7e3b2a6, SUC_lexicon_tags_7 },
    { 0xa5c3b2a7, SUC_lexicon_tags_11 },
    { 0x9a43b2a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10c3b2aa, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x7463b2ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6883b2b2, SUC_lexicon_tags_6 },
    { 0x6803b2b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7003b2b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea23b2b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac3b2bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4723b2c1, SUC_lexicon_tags_7 },
    { 0xcd43b2c2, SUC_lexicon_tags_46 },
    { 0x46e3b2c3, SUC_lexicon_tags_19 },
    { 0xe063b2c4, SUC_lexicon_tags_38 },
    { 0x7903b2c5, SUC_lexicon_tags_37 },
    { 0x29e3b2c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e3b2c9, SUC_lexicon_tags_45 },
    { 0x3963b2c9, SUC_lexicon_tags_27 },
    { 0xeee3b2cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9603b2cf, SUC_lexicon_tags_2 },
    { 0x6e03b2d0, SUC_lexicon_tags_27 },
    { 0xc483b2d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb63b2d5, SUC_lexicon_tags_26 },
    { 0xcea3b2d6, SUC_lexicon_tags_25 },
    { 0x2983b2d6, SUC_lexicon_tags_25 },
    { 0xbee3b2d8, SUC_lexicon_tags_10 },
    { 0x45a3b2d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc623b2db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8243b2dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe643b2df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe523b2e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a3b2e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b43b2ed, SUC_lexicon_tags_38 },
    { 0xe623b2ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce3b2f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4403b2f8, SUC_lexicon_tags_29 },
    { 0x3f83b2f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f43b2fc, SUC_lexicon_tags_17 },
    { 0x5603b2fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f03b305, SUC_lexicon_tags_13 },
    { 0x2b83b306, SUC_lexicon_tags_67 },
    { 0x6c03b306, SUC_lexicon_tags_12 },
    { 0xae03b308, SUC_lexicon_tags_60 },
    { 0xaa03b305, SUC_lexicon_tags_37 },
    { 0xa443b30a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe323b30e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc63b310, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8743b313, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7583b319, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1743b31c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa3b31f, SUC_lexicon_tags_29 },
    { 0xea3b320, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda3b324, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf563b326, SUC_lexicon_tags_17 },
    { 0x5643b327, SUC_lexicon_tags_14 },
    { 0x7e63b327, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb83b32e, SUC_lexicon_tags_12 },
    { 0x203b32e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7343b335, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c3b338, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6903b33a, SUC_lexicon_tags_37 },
    { 0xc383b33b, SUC_lexicon_tags_11 },
    { 0xbe23b33a, SUC_lexicon_tags_65 },
    { 0x9503b33b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5503b345, SUC_lexicon_tags_16 },
    { 0xe083b346, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe03b349, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d63b34b, SUC_lexicon_tags_10 },
    { 0xf803b34b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3383b34e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa943b351, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee3b354, SUC_lexicon_tags_63 },
    { 0x7ea3b355, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3883b357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c63b35d, SUC_lexicon_tags_8 },
    { 0x95c3b35e, SUC_lexicon_tags_2 },
    { 0xc3e3b35e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c3b363, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783b368, SUC_lexicon_tags_11 },
    { 0x43e3b369, SUC_lexicon_tags_12 },
    { 0xb663b36a, SUC_lexicon_tags_17 },
    { 0xf3c3b36b, SUC_lexicon_tags_23 },
    { 0xab43b36a, SUC_lexicon_tags_9 },
    { 0x3243b36d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b03b372, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6403b375, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc423b377, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aa3b379, SUC_lexicon_tags_17 },
    { 0x5263b37a, SUC_lexicon_tags_47 },
    { 0xb9c3b37b, SUC_lexicon_tags_37 },
    { 0xe843b37b, SUC_lexicon_tags_13 },
    { 0x19e3b37c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7403b37f, SUC_lexicon_tags_23 },
    { 0x47a3b380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3943b382, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63b389, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a3b38e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3043b391, SUC_lexicon_tags_11 },
    { 0xe9a3b392, SUC_lexicon_tags_11 },
    { 0x7003b393, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec83b399, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c3b3a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6203b3a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd8e3b3a5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe6a3b3a7, SUC_lexicon_tags_68 },
    { 0x8c43b3a8, SUC_lexicon_tags_25 },
    { 0x7f03b3a9, SUC_lexicon_tags_1 },
    { 0x7483b3a8, SUC_lexicon_tags_38 },
    { 0x6983b3ab, SUC_lexicon_tags_37 },
    { 0x5fc3b3ac, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e3b3b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8d63b3b4, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc3b3b7, SUC_lexicon_tags_42 },
    { 0x8923b3b7, SUC_lexicon_tags_43 },
    { 0x1cc3b3b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e3b3be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xabc3b3c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54a3b3c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81c3b3c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4683b3c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa83b3ca, SUC_lexicon_tags_33 },
    { 0x3e43b3cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c43b3cf, SUC_lexicon_tags_9 },
    { 0x83a3b3d0, SUC_lexicon_tags_11 },
    { 0x3063b3d0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd43b3d3, SUC_lexicon_tags_11 },
    { 0x78c3b3d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce83b3d6, SUC_lexicon_tags_63 },
    { 0xbe3b3d7, SUC_lexicon_tags_27 },
    { 0xe183b3d7, SUC_lexicon_tags_63 },
    { 0x77a3b3d9, SUC_lexicon_tags_4 },
    { 0xfd43b3da, SUC_lexicon_tags_33 },
    { 0xed63b3d6, SUC_lexicon_tags_46 },
    { 0x8603b3da, SUC_lexicon_tags_2 },
    { 0xbbc3b3dd, SUC_lexicon_tags_27 },
    { 0x4c63b3de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc303b3e1, SUC_lexicon_tags_17 },
    { 0x4e63b3e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8e83b3e4, SUC_lexicon_tags_8 },
    { 0x2883b3e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8063b3e7, SUC_lexicon_tags_27 },
    { 0x4323b3e8, SUC_lexicon_tags_17 },
    { 0x7203b3e7, SUC_lexicon_tags_21 },
    { 0x2223b3e9, SUC_lexicon_tags_37 },
    { 0x71c3b3eb, SUC_lexicon_tags_60 },
    { 0xcb23b3e9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe423b3f4, SUC_lexicon_tags_27 },
    { 0x2e43b3f5, SUC_lexicon_tags_39 },
    { 0x8223b3f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b03b3fa, SUC_lexicon_tags_37 },
    { 0x25c3b3fb, SUC_lexicon_tags_17 },
    { 0x8043b3fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a43b3ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf63b40a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ac3b40c, SUC_lexicon_tags_1 },
    { 0x1e43b40d, SUC_lexicon_tags_2 },
    { 0x5b83b40c, SUC_lexicon_tags_12 },
    { 0xec43b40d, SUC_lexicon_tags_79 },
    { 0x4ca3b410, SUC_lexicon_tags_7 },
    { 0x1643b411, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9de3b413, SUC_lexicon_tags_77 },
    { 0xa723b414, SUC_lexicon_tags_52 },
    { 0xa663b415, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b03b41b, SUC_lexicon_tags_29 },
    { 0xda63b41b, SUC_lexicon_tags_8 },
    { 0x5543b41c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5083b420, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc3b424, SUC_lexicon_tags_1 },
    { 0x8283b425, SUC_lexicon_tags_25 },
    { 0x9303b425, SUC_lexicon_tags_77 },
    { 0xb743b427, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa3b42d, SUC_lexicon_tags_27 },
    { 0x18a3b42d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a3b432, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15a3b434, SUC_lexicon_tags_14 },
    { 0xd263b435, SUC_lexicon_tags_23 },
    { 0x6163b436, SUC_lexicon_tags_27 },
    { 0x63c3b437, SUC_lexicon_tags_55 },
    { 0x7ce3b438, SUC_lexicon_tags_17 },
    { 0x3643b439, SUC_lexicon_tags_30 },
    { 0xd7e3b43a, SUC_lexicon_tags_11 },
    { 0xb0e3b439, SUC_lexicon_tags_17 },
    { 0x4d83b43b, SUC_lexicon_tags_8 },
    { 0xff23b43d, SUC_lexicon_tags_6 },
    { 0x81c3b43e, SUC_lexicon_tags_25 },
    { 0x7b23b43f, SUC_lexicon_tags_38 },
    { 0xee3b440, SUC_lexicon_tags_11 },
    { 0x97e3b441, SUC_lexicon_tags_243 },
    { 0x8963b441, SUC_lexicon_tags_11 },
    { 0xe443b443, SUC_lexicon_tags_37 },
    { 0x6363b441, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ee3b446, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03b449, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3b44d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc283b44f, SUC_lexicon_tags_11 },
    { 0x9c83b450, SUC_lexicon_tags_60 },
    { 0x64e3b450, SUC_lexicon_tags_34 },
    { 0x563b452, SUC_lexicon_tags_25 },
    { 0x2343b453, SUC_lexicon_tags_7 },
    { 0xe263b454, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d23b45b, SUC_lexicon_tags_1 },
    { 0x5b63b45c, SUC_lexicon_tags_59 },
    { 0xa63b45d, SUC_lexicon_tags_25 },
    { 0xeb63b45c, SUC_lexicon_tags_2 },
    { 0xa483b45f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f03b462, SUC_lexicon_tags_7 },
    { 0x1023b462, SUC_lexicon_tags_71 },
    { 0xe1c3b462, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c3b468, SUC_lexicon_tags_15 },
    { 0x40a3b469, SUC_lexicon_tags_47 },
    { 0x6f03b469, SUC_lexicon_tags_12 },
    { 0xb03b46b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5c3b46d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3bc3b46f, SUC_lexicon_tags_9 },
    { 0xe643b470, SUC_lexicon_tags_60 },
    { 0xd43b470, SUC_lexicon_tags_15 },
    { 0x4763b46f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb23b475, SUC_lexicon_tags_19 },
    { 0x13a3b476, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf863b47d, SUC_lexicon_tags_11 },
    { 0x1183b47d, SUC_lexicon_tags_234 },
    { 0x3ca3b47f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8663b483, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d63b487, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe903b48c, SUC_lexicon_tags_17 },
    { 0x3f83b48c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23b48f, SUC_lexicon_tags_9 },
    { 0x9163b48f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5143b494, SUC_lexicon_tags_6 },
    { 0xbf23b495, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd543b497, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4463b49b, SUC_lexicon_tags_47 },
    { 0x1b43b49c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeaa3b49e, SUC_lexicon_tags_30 },
    { 0xa0a3b49f, SUC_lexicon_tags_17 },
    { 0xebc3b4a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c3b4a5, SUC_lexicon_tags_15 },
    { 0x9563b4a6, SUC_lexicon_tags_38 },
    { 0xd7a3b4a7, SUC_lexicon_tags_25 },
    { 0x9c03b4a8, SUC_lexicon_tags_60 },
    { 0x5263b4a5, SUC_lexicon_tags_79 },
    { 0xa3a3b4a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb083b4ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343b4b2, SUC_lexicon_tags_43 },
    { 0x4e03b4b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a3b4b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a3b4bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff43b4bf, SUC_lexicon_tags_14 },
    { 0x5383b4c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb43b4cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e3b4d0, SUC_lexicon_tags_6 },
    { 0x72a3b4d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9d23b4d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03b4d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4c3b4db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd023b4dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf23b4df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee83b4e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e3b4e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7563b4eb, SUC_lexicon_tags_224 },
    { 0x7bc3b4eb, SUC_lexicon_tags_37 },
    { 0x71a3b4ed, SUC_lexicon_tags_49 },
    { 0xc583b4ee, SUC_lexicon_tags_23 },
    { 0xfe23b4ec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8103b4f4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1e63b4f6, SUC_lexicon_tags_45 },
    { 0x2323b4f7, SUC_lexicon_tags_14 },
    { 0x6223b4f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d83b4fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb63b508, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xea03b50a, SUC_lexicon_tags_7 },
    { 0x5ae3b50b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e3b510, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc743b517, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa3b51c, SUC_lexicon_tags_11 },
    { 0x7ac3b51d, SUC_lexicon_tags_25 },
    { 0xf163b51c, SUC_lexicon_tags_65 },
    { 0x4be3b51f, SUC_lexicon_tags_9 },
    { 0x6843b520, SUC_lexicon_tags_31 },
    { 0x4943b521, SUC_lexicon_tags_37 },
    { 0x87c3b522, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18a3b524, SUC_lexicon_tags_37 },
    { 0x9d43b525, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea3b52a, SUC_lexicon_tags_33 },
    { 0xe6e3b52b, SUC_lexicon_tags_25 },
    { 0x5cc3b52c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e3b52f, SUC_lexicon_tags_11 },
    { 0x5be3b52f, SUC_lexicon_tags_17 },
    { 0x9003b531, SUC_lexicon_tags_60 },
    { 0x4bc3b532, SUC_lexicon_tags_33 },
    { 0x9ac3b533, SUC_lexicon_tags_17 },
    { 0x36e3b534, SUC_lexicon_tags_27 },
    { 0xd803b535, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca3b539, SUC_lexicon_tags_23 },
    { 0xba03b53a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c3b53e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb723b545, SUC_lexicon_tags_21 },
    { 0x443b546, SUC_lexicon_tags_2 },
    { 0x7d03b547, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcde3b549, SUC_lexicon_tags_15 },
    { 0x223b549, SUC_lexicon_tags_37 },
    { 0xd423b54a, SUC_lexicon_tags_15 },
    { 0x5203b54c, SUC_lexicon_tags_11 },
    { 0x73e3b54a, SUC_lexicon_tags_7 },
    { 0x3a63b54a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c3b551, SUC_lexicon_tags_27 },
    { 0xcd03b551, SUC_lexicon_tags_1 },
    { 0xf63b552, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce23b556, SUC_lexicon_tags_11 },
    { 0xc703b557, SUC_lexicon_tags_7 },
    { 0x5803b558, SUC_lexicon_tags_25 },
    { 0xd263b559, SUC_lexicon_tags_17 },
    { 0xdd43b55a, SUC_lexicon_tags_26 },
    { 0x8763b557, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63b55e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa863b563, SUC_lexicon_tags_48 },
    { 0xbc23b563, SUC_lexicon_tags_23 },
    { 0x55a3b564, SUC_lexicon_tags_15 },
    { 0x3623b566, SUC_lexicon_tags_45 },
    { 0x3e63b564, SUC_lexicon_tags_1 },
    { 0xf5e3b568, SUC_lexicon_tags_2 },
    { 0x63c3b569, SUC_lexicon_tags_2 },
    { 0x5743b56a, SUC_lexicon_tags_6 },
    { 0xd383b56a, SUC_lexicon_tags_7 },
    { 0x8a3b564, SUC_lexicon_tags_35 },
    { 0x5083b565, SUC_lexicon_tags_63 },
    { 0x16e3b56a, SUC_lexicon_tags_7 },
    { 0x7843b56f, SUC_lexicon_tags_2 },
    { 0x67a3b570, SUC_lexicon_tags_23 },
    { 0x9443b571, SUC_lexicon_tags_17 },
    { 0xa3a3b572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a3b57b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73c3b57d, SUC_lexicon_tags_39 },
    { 0x2083b57e, SUC_lexicon_tags_38 },
    { 0x4cc3b57d, SUC_lexicon_tags_48 },
    { 0xae63b580, SUC_lexicon_tags_33 },
    { 0x8783b581, SUC_lexicon_tags_1 },
    { 0xd803b582, SUC_lexicon_tags_11 },
    { 0x9cc3b583, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a3b589, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e23b594, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae43b59c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8203b59e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc643b5a0, SUC_lexicon_tags_37 },
    { 0x9ac3b5a1, SUC_lexicon_tags_6 },
    { 0xfb63b5a2, SUC_lexicon_tags_17 },
    { 0x5583b5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6723b5a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xffa3b5a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f83b5aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d03b5af, SUC_lexicon_tags_38 },
    { 0x9f83b5b0, SUC_lexicon_tags_27 },
    { 0x7283b5b0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4da3b5b3, SUC_lexicon_tags_38 },
    { 0x8b83b5b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1743b5b8, SUC_lexicon_tags_12 },
    { 0x1ae3b5b9, SUC_lexicon_tags_69 },
    { 0x1723b5ba, SUC_lexicon_tags_25 },
    { 0xbf03b5bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf103b5bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3263b5c4, SUC_lexicon_tags_2 },
    { 0xf723b5c4, SUC_lexicon_tags_9 },
    { 0xac03b5c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe3b5ce, SUC_lexicon_tags_37 },
    { 0x84a3b5ce, SUC_lexicon_tags_37 },
    { 0x5363b5ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4683b5d2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc6a3b5d4, SUC_lexicon_tags_25 },
    { 0xba83b5d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e43b5d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda3b5df, SUC_lexicon_tags_1 },
    { 0x9063b5e0, SUC_lexicon_tags_35 },
    { 0xbe83b5e1, SUC_lexicon_tags_37 },
    { 0x983b5e2, SUC_lexicon_tags_15 },
    { 0x9aa3b5e3, SUC_lexicon_tags_0 },
    { 0xbca3b5e4, SUC_lexicon_tags_15 },
    { 0xab83b5e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ee3b5e7, SUC_lexicon_tags_33 },
    { 0xae83b5e8, SUC_lexicon_tags_37 },
    { 0x4ba3b5e9, SUC_lexicon_tags_151 },
    { 0x9e23b5ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6823b5ee, SUC_lexicon_tags_46 },
    { 0xb423b5ef, SUC_lexicon_tags_25 },
    { 0x1563b5f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce3b5f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2123b5f6, SUC_lexicon_tags_2 },
    { 0xc323b5f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef03b5fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10a3b5ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23b605, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe743b60b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2163b611, SUC_lexicon_tags_47 },
    { 0x9fe3b612, SUC_lexicon_tags_60 },
    { 0x863b613, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e3b616, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6723b625, SUC_lexicon_tags_28 },
    { 0x52e3b626, SUC_lexicon_tags_63 },
    { 0xefe3b627, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa83b62c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe903b62e, SUC_lexicon_tags_38 },
    { 0xcca3b62f, SUC_lexicon_tags_11 },
    { 0xfd63b62e, SUC_lexicon_tags_17 },
    { 0x763b631, SUC_lexicon_tags_19 },
    { 0xaee3b632, SUC_lexicon_tags_49 },
    { 0x1483b633, SUC_lexicon_tags_38 },
    { 0xebc3b631, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc63b636, SUC_lexicon_tags_0 },
    { 0x483b637, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe3b63f, SUC_lexicon_tags_14 },
    { 0xada3b63f, SUC_lexicon_tags_25 },
    { 0x9263b641, SUC_lexicon_tags_12 },
    { 0x7b83b642, SUC_lexicon_tags_87 },
    { 0xaa03b641, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7343b646, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0e3b648, SUC_lexicon_tags_4 },
    { 0x1a03b648, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c3b64e, SUC_lexicon_tags_15 },
    { 0xe63b64e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a3b654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa3b65b, SUC_lexicon_tags_25 },
    { 0xbdc3b65c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4703b661, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb663b664, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3b66b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5423b672, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b43b677, SUC_lexicon_tags_40 },
    { 0xa5c3b678, SUC_lexicon_tags_37 },
    { 0x9c3b679, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2263b681, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e3b684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd123b68a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac3b68d, SUC_lexicon_tags_17 },
    { 0xa1a3b68e, SUC_lexicon_tags_21 },
    { 0x1683b68f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4243b691, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e3b695, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6863b698, SUC_lexicon_tags_10 },
    { 0xfcc3b699, SUC_lexicon_tags_15 },
    { 0x4d43b698, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ee3b69c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c23b69f, SUC_lexicon_tags_6 },
    { 0x1a03b6a0, SUC_lexicon_tags_2 },
    { 0x9283b6a1, SUC_lexicon_tags_12 },
    { 0x2603b6a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b83b6a5, SUC_lexicon_tags_37 },
    { 0x85c3b6a6, SUC_lexicon_tags_49 },
    { 0x81a3b6a7, SUC_lexicon_tags_10 },
    { 0xed63b6a8, SUC_lexicon_tags_12 },
    { 0x43a3b6a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d43b6ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1483b6b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd563b6b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f63b6ba, SUC_lexicon_tags_23 },
    { 0x6863b6bb, SUC_lexicon_tags_6 },
    { 0x2943b6bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x83e3b6be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c3b6c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc383b6c3, SUC_lexicon_tags_27 },
    { 0x26e3b6c4, SUC_lexicon_tags_14 },
    { 0x7a23b6c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3b6c8, SUC_lexicon_tags_60 },
    { 0x50e3b6c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c3b6cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d83b6d5, SUC_lexicon_tags_39 },
    { 0x3943b6d5, SUC_lexicon_tags_19 },
    { 0x4583b6d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f23b6db, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a3b6e0, SUC_lexicon_tags_7 },
    { 0x9263b6e1, SUC_lexicon_tags_5 },
    { 0xaf03b6e0, SUC_lexicon_tags_27 },
    { 0x2043b6e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4503b6e7, SUC_lexicon_tags_63 },
    { 0xcaa3b6e8, SUC_lexicon_tags_17 },
    { 0x8343b6e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7443b6f0, SUC_lexicon_tags_23 },
    { 0x8983b6f1, SUC_lexicon_tags_19 },
    { 0xb183b6f0, SUC_lexicon_tags_12 },
    { 0x7823b6f2, SUC_lexicon_tags_17 },
    { 0x4ce3b6f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5723b6f6, SUC_lexicon_tags_25 },
    { 0x9b83b6f7, SUC_lexicon_tags_38 },
    { 0xeb83b6f8, SUC_lexicon_tags_1 },
    { 0xcd63b6f9, SUC_lexicon_tags_61 },
    { 0xe903b6f9, SUC_lexicon_tags_2 },
    { 0xef03b6f8, SUC_lexicon_tags_31 },
    { 0x6643b6f6, SUC_lexicon_tags_206 },
    { 0xc783b6fd, SUC_lexicon_tags_16 },
    { 0x8203b6fe, SUC_lexicon_tags_45 },
    { 0xd843b6ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69a3b701, SUC_lexicon_tags_6 },
    { 0x8583b701, SUC_lexicon_tags_68 },
    { 0xd3a3b703, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9883b70b, SUC_lexicon_tags_11 },
    { 0x3123b70c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3b70e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de3b716, SUC_lexicon_tags_15 },
    { 0xd663b716, SUC_lexicon_tags_47 },
    { 0xdcc3b718, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7603b71d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe623b722, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e3b72c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9903b72e, SUC_lexicon_tags_25 },
    { 0x1b83b72e, SUC_lexicon_tags_2 },
    { 0xa643b730, SUC_lexicon_tags_15 },
    { 0xc163b730, SUC_lexicon_tags_37 },
    { 0x223b730, SUC_lexicon_tags_49 },
    { 0x3223b731, SUC_lexicon_tags_31 },
    { 0xb5a3b734, SUC_lexicon_tags_47 },
    { 0x4c3b734, SUC_lexicon_tags_19 },
    { 0xac83b732, SUC_lexicon_tags_25 },
    { 0x8043b737, SUC_lexicon_tags_11 },
    { 0xef43b733, SUC_lexicon_tags_25 },
    { 0x8283b734, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc403b73d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e3b743, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa763b745, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9243b748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe163b74a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd2a3b74c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf783b74e, SUC_lexicon_tags_37 },
    { 0x463b74e, SUC_lexicon_tags_17 },
    { 0x9663b750, SUC_lexicon_tags_7 },
    { 0x9643b751, SUC_lexicon_tags_26 },
    { 0x4d43b752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd583b759, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e23b75c, SUC_lexicon_tags_15 },
    { 0x84a3b75c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3b760, SUC_lexicon_tags_15 },
    { 0xdaa3b761, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba3b764, SUC_lexicon_tags_100 },
    { 0xa2e3b765, SUC_lexicon_tags_15 },
    { 0xd263b766, SUC_lexicon_tags_38 },
    { 0x1663b767, SUC_lexicon_tags_11 },
    { 0x3bc3b768, SUC_lexicon_tags_38 },
    { 0xf983b767, SUC_lexicon_tags_7 },
    { 0xdd63b76a, SUC_lexicon_tags_11 },
    { 0x7203b767, SUC_lexicon_tags_37 },
    { 0xbe63b76c, SUC_lexicon_tags_7 },
    { 0x56a3b76b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea3b775, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b83b77b, SUC_lexicon_tags_47 },
    { 0x61a3b77c, SUC_lexicon_tags_6 },
    { 0x2f63b77c, SUC_lexicon_tags_104 },
    { 0xbe23b77e, SUC_lexicon_tags_7 },
    { 0xb343b77e, SUC_lexicon_tags_2 },
    { 0x3a03b77d, SUC_lexicon_tags_3 },
    { 0xe0e3b77e, SUC_lexicon_tags_2 },
    { 0x1da3b77d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe643b784, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c03b787, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c3b78b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca03b790, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1203b795, SUC_lexicon_tags_15 },
    { 0xf403b795, SUC_lexicon_tags_15 },
    { 0x4ac3b795, SUC_lexicon_tags_12 },
    { 0x6223b798, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf003b79a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c3b79e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e3b7a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x643b7a5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb703b7aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc3b7b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8803b7b6, SUC_lexicon_tags_25 },
    { 0x75a3b7b6, SUC_lexicon_tags_0 },
    { 0xd803b7b6, SUC_lexicon_tags_7 },
    { 0xeda3b7b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3b7bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xafe3b7be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5263b7c1, SUC_lexicon_tags_172 },
    { 0xf863b7c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c63b7c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d83b7c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc83b7d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdde3b7d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7843b7d9, SUC_lexicon_tags_47 },
    { 0x3763b7da, SUC_lexicon_tags_17 },
    { 0xce3b7db, SUC_lexicon_tags_15 },
    { 0xfa83b7dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5003b7df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63e3b7e1, SUC_lexicon_tags_31 },
    { 0xd443b7e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c3b7e5, SUC_lexicon_tags_24 },
    { 0xb263b7e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203b7ea, SUC_lexicon_tags_65 },
    { 0x2423b7eb, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e3b7ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7663b7f5, SUC_lexicon_tags_17 },
    { 0xd403b7f5, SUC_lexicon_tags_15 },
    { 0x2e43b7f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc923b7fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a3b800, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d43b807, SUC_lexicon_tags_23 },
    { 0x3243b808, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc763b80a, SUC_lexicon_tags_15 },
    { 0xcfc3b80a, SUC_lexicon_tags_25 },
    { 0xe943b80b, SUC_lexicon_tags_7 },
    { 0xed03b80d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8183b810, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a43b817, SUC_lexicon_tags_1 },
    { 0x1da3b818, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbe03b81a, SUC_lexicon_tags_45 },
    { 0x98c3b81a, SUC_lexicon_tags_151 },
    { 0x10a3b81c, SUC_lexicon_tags_15 },
    { 0xb043b81c, SUC_lexicon_tags_11 },
    { 0xe863b81e, SUC_lexicon_tags_2 },
    { 0xec03b81d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd383b822, SUC_lexicon_tags_793 },
    { 0x2943b822, SUC_lexicon_tags_37 },
    { 0xa8c3b824, SUC_lexicon_tags_1 },
    { 0x403b823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4323b827, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd083b82c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c3b82f, SUC_lexicon_tags_29 },
    { 0x4663b82f, SUC_lexicon_tags_4 },
    { 0x7183b82f, SUC_lexicon_tags_282 },
    { 0xa9a3b832, SUC_lexicon_tags_14 },
    { 0xa9a3b833, SUC_lexicon_tags_25 },
    { 0x4e43b834, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3b839, SUC_lexicon_tags_37 },
    { 0x3523b83a, SUC_lexicon_tags_1033 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd763b842, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1c3b844, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xae43b847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a3b84d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe3b850, SUC_lexicon_tags_15 },
    { 0x3803b851, SUC_lexicon_tags_19 },
    { 0xdd83b851, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83b855, SUC_lexicon_tags_1 },
    { 0x8203b856, SUC_lexicon_tags_43 },
    { 0xbbc3b857, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xffa3b859, SUC_lexicon_tags_34 },
    { 0x6e03b85a, SUC_lexicon_tags_49 },
    { 0x3663b85b, SUC_lexicon_tags_0 },
    { 0xb6c3b85c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e03b85f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf803b861, SUC_lexicon_tags_48 },
    { 0x2c63b861, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c23b86d, SUC_lexicon_tags_11 },
    { 0x23c3b86e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683b876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a43b879, SUC_lexicon_tags_16 },
    { 0x6303b87a, SUC_lexicon_tags_27 },
    { 0xaea3b87b, SUC_lexicon_tags_30 },
    { 0x6563b87c, SUC_lexicon_tags_31 },
    { 0x7d03b87d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe23b87f, SUC_lexicon_tags_2 },
    { 0x9ee3b87f, SUC_lexicon_tags_49 },
    { 0x51c3b87f, SUC_lexicon_tags_17 },
    { 0x63b881, SUC_lexicon_tags_33 },
    { 0xea43b883, SUC_lexicon_tags_12 },
    { 0xaf23b883, SUC_lexicon_tags_6 },
    { 0x2c83b885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9883b888, SUC_lexicon_tags_39 },
    { 0x1ba3b889, SUC_lexicon_tags_45 },
    { 0xddc3b88a, SUC_lexicon_tags_7 },
    { 0xbd23b888, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5663b88f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5d63b891, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x603b894, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c3b898, SUC_lexicon_tags_7 },
    { 0xdf83b899, SUC_lexicon_tags_26 },
    { 0x9c23b89a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b63b89f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf963b8a1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa3b8a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb243b8ad, SUC_lexicon_tags_7 },
    { 0x3203b8ae, SUC_lexicon_tags_17 },
    { 0xe3b8ad, SUC_lexicon_tags_12 },
    { 0x23b8ae, SUC_lexicon_tags_1 },
    { 0x24a3b8b1, SUC_lexicon_tags_37 },
    { 0x26c3b8af, SUC_lexicon_tags_82 },
    { 0xac3b8b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7d63b8b5, SUC_lexicon_tags_33 },
    { 0xfe43b8b6, SUC_lexicon_tags_7 },
    { 0x4b43b8b7, SUC_lexicon_tags_2 },
    { 0x3b63b8b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc803b8bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc083b8be, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a83b8c1, SUC_lexicon_tags_27 },
    { 0x17a3b8c2, SUC_lexicon_tags_15 },
    { 0x87c3b8c3, SUC_lexicon_tags_26 },
    { 0x7d83b8c3, SUC_lexicon_tags_47 },
    { 0x1343b8c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d23b8c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xce83b8c9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c3b8cd, SUC_lexicon_tags_7 },
    { 0xe383b8ce, SUC_lexicon_tags_45 },
    { 0xd63b8ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2103b8d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc3b8da, SUC_lexicon_tags_27 },
    { 0x10c3b8db, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe183b8df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb63b8e2, SUC_lexicon_tags_49 },
    { 0x1fc3b8e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e3b8e9, SUC_lexicon_tags_29 },
    { 0x263b8ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b43b8ef, SUC_lexicon_tags_1 },
    { 0xca23b8ef, SUC_lexicon_tags_65 },
    { 0xba63b8f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4843b8f7, SUC_lexicon_tags_28 },
    { 0x2763b8f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743b8fc, SUC_lexicon_tags_25 },
    { 0x2343b8fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb63b900, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c23b903, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf3a3b905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be3b908, SUC_lexicon_tags_31 },
    { 0xe603b908, SUC_lexicon_tags_2 },
    { 0xdfe3b908, SUC_lexicon_tags_7 },
    { 0x32c3b90b, SUC_lexicon_tags_3 },
    { 0x5283b90c, SUC_lexicon_tags_0 },
    { 0x9483b90b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc023b910, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfd63b912, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4083b917, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c43b91a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c3b91d, SUC_lexicon_tags_29 },
    { 0xc603b91e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf523b921, SUC_lexicon_tags_45 },
    { 0x43c3b922, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2783b924, SUC_lexicon_tags_16 },
    { 0xdc3b924, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d03b929, SUC_lexicon_tags_11 },
    { 0xe43b929, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda3b92e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xea83b931, SUC_lexicon_tags_14 },
    { 0x8303b931, SUC_lexicon_tags_45 },
    { 0x14c3b932, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3023b938, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf403b942, SUC_lexicon_tags_87 },
    { 0x9b83b942, SUC_lexicon_tags_27 },
    { 0xcb23b942, SUC_lexicon_tags_17 },
    { 0xea03b944, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d03b947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c3b94b, SUC_lexicon_tags_68 },
    { 0x4d83b94b, SUC_lexicon_tags_7 },
    { 0xf3c3b94d, SUC_lexicon_tags_16 },
    { 0x41a3b94c, SUC_lexicon_tags_45 },
    { 0x8d83b94b, SUC_lexicon_tags_26 },
    { 0xbf23b94c, SUC_lexicon_tags_29 },
    { 0x18c3b951, SUC_lexicon_tags_17 },
    { 0x3d23b952, SUC_lexicon_tags_2 },
    { 0x7ce3b953, SUC_lexicon_tags_11 },
    { 0x4c23b953, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa3b95f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a3b963, SUC_lexicon_tags_68 },
    { 0xbfe3b964, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c23b96b, SUC_lexicon_tags_7 },
    { 0x5703b96c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde43b96f, SUC_lexicon_tags_68 },
    { 0xd7e3b96f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c3b977, SUC_lexicon_tags_37 },
    { 0x5743b978, SUC_lexicon_tags_1 },
    { 0xc5c3b979, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3003b97c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c3b983, SUC_lexicon_tags_38 },
    { 0x8823b984, SUC_lexicon_tags_49 },
    { 0x1103b985, SUC_lexicon_tags_121 },
    { 0xd2e3b986, SUC_lexicon_tags_6 },
    { 0xea03b983, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1023b989, SUC_lexicon_tags_43 },
    { 0xf5e3b98a, SUC_lexicon_tags_16 },
    { 0x1763b98b, SUC_lexicon_tags_17 },
    { 0xa3a3b98b, SUC_lexicon_tags_25 },
    { 0xde03b98d, SUC_lexicon_tags_6 },
    { 0x2fc3b98c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x2643b991, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d23b996, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d23b999, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x4963b99c, SUC_lexicon_tags_9 },
    { 0xcc03b99c, SUC_lexicon_tags_37 },
    { 0x6aa3b99e, SUC_lexicon_tags_12 },
    { 0xec03b99d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb003b9a1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8923b9a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce23b9ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfa23b9ae, SUC_lexicon_tags_63 },
    { 0x7d83b9ae, SUC_lexicon_tags_12 },
    { 0x8063b9ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cc3b9b2, SUC_lexicon_tags_29 },
    { 0x5123b9b3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7343b9b5, SUC_lexicon_tags_8 },
    { 0x4803b9b6, SUC_lexicon_tags_17 },
    { 0x32e3b9b6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4343b9ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e3b9be, SUC_lexicon_tags_1 },
    { 0x4b23b9bf, SUC_lexicon_tags_49 },
    { 0xe23b9be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2043b9c8, SUC_lexicon_tags_21 },
    { 0x2da3b9c9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d43b9d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e3b9d3, SUC_lexicon_tags_49 },
    { 0x8b63b9d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc363b9d6, SUC_lexicon_tags_37 },
    { 0xb463b9d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc363b9db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb523b9dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd723b9df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2283b9e3, SUC_lexicon_tags_33 },
    { 0x18a3b9e3, SUC_lexicon_tags_19 },
    { 0x34a3b9e5, SUC_lexicon_tags_11 },
    { 0x8d43b9e5, SUC_lexicon_tags_7 },
    { 0xd303b9e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x93c3b9e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d03b9f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e43b9f6, SUC_lexicon_tags_12 },
    { 0xf163b9f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3b9fb, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8343b9ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba3ba02, SUC_lexicon_tags_5 },
    { 0x8b03ba02, SUC_lexicon_tags_9 },
    { 0xc5e3ba03, SUC_lexicon_tags_12 },
    { 0x7f23ba02, SUC_lexicon_tags_2 },
    { 0x46c3ba02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x243ba08, SUC_lexicon_tags_0 },
    { 0x4063ba08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f83ba0f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe003ba12, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3243ba17, SUC_lexicon_tags_23 },
    { 0x1743ba17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2103ba1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4443ba25, SUC_lexicon_tags_43 },
    { 0x2003ba26, SUC_lexicon_tags_1 },
    { 0x6f63ba26, SUC_lexicon_tags_26 },
    { 0xa7a3ba28, SUC_lexicon_tags_15 },
    { 0x5de3ba28, SUC_lexicon_tags_3 },
    { 0x72e3ba26, SUC_lexicon_tags_25 },
    { 0xf423ba25, SUC_lexicon_tags_43 },
    { 0xf9a3ba2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6883ba2f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2163ba33, SUC_lexicon_tags_11 },
    { 0x7063ba33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fe3ba36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea43ba38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c3ba3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf243ba3d, SUC_lexicon_tags_1 },
    { 0x4643ba3d, SUC_lexicon_tags_0 },
    { 0x68a3ba3f, SUC_lexicon_tags_31 },
    { 0x56c3ba40, SUC_lexicon_tags_17 },
    { 0xd363ba3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb423ba54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc83ba5b, SUC_lexicon_tags_37 },
    { 0xe803ba5b, SUC_lexicon_tags_12 },
    { 0x1863ba5d, SUC_lexicon_tags_14 },
    { 0x3383ba5e, SUC_lexicon_tags_2 },
    { 0xf6e3ba5f, SUC_lexicon_tags_14 },
    { 0xa3c3ba5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea3ba68, SUC_lexicon_tags_46 },
    { 0x8423ba69, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643ba71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1903ba73, SUC_lexicon_tags_38 },
    { 0x5d03ba74, SUC_lexicon_tags_17 },
    { 0x54e3ba75, SUC_lexicon_tags_37 },
    { 0x52a3ba76, SUC_lexicon_tags_7 },
    { 0xb963ba75, SUC_lexicon_tags_38 },
    { 0x2e43ba77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa463ba7a, SUC_lexicon_tags_37 },
    { 0x9163ba7b, SUC_lexicon_tags_37 },
    { 0x2783ba7a, SUC_lexicon_tags_55 },
    { 0x51a3ba7d, SUC_lexicon_tags_25 },
    { 0xec3ba7d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa403ba80, SUC_lexicon_tags_47 },
    { 0x3943ba81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a3ba84, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3923ba86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x63a3ba88, SUC_lexicon_tags_37 },
    { 0x7c63ba89, SUC_lexicon_tags_28 },
    { 0x5963ba8a, SUC_lexicon_tags_9 },
    { 0x3c43ba8b, SUC_lexicon_tags_2 },
    { 0x1c83ba8b, SUC_lexicon_tags_49 },
    { 0x9483ba8d, SUC_lexicon_tags_23 },
    { 0xd243ba8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd543ba92, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2023ba97, SUC_lexicon_tags_28 },
    { 0x5f03ba98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9663ba9a, SUC_lexicon_tags_26 },
    { 0xe743ba9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1123ba9e, SUC_lexicon_tags_55 },
    { 0x1103ba9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd563baa7, SUC_lexicon_tags_49 },
    { 0x4f23baa8, SUC_lexicon_tags_7 },
    { 0xdb03baa9, SUC_lexicon_tags_38 },
    { 0x5d23baa9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c3baad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a3bab4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe43bab6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4863bab8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe083babf, SUC_lexicon_tags_25 },
    { 0xd163babf, SUC_lexicon_tags_12 },
    { 0xbbc3bac1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1243bac3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7743bac5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83baca, SUC_lexicon_tags_77 },
    { 0x8f63bacb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9143bace, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7623bad0, SUC_lexicon_tags_11 },
    { 0xb4a3bad0, SUC_lexicon_tags_17 },
    { 0x9fa3bad1, SUC_lexicon_tags_3 },
    { 0x783bad0, SUC_lexicon_tags_1 },
    { 0x7703bad3, SUC_lexicon_tags_35 },
    { 0x5463bad5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdde3bad7, SUC_lexicon_tags_0 },
    { 0xa1c3bad8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc3badc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c3badf, SUC_lexicon_tags_68 },
    { 0x8663bae0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4143bae2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5943bae5, SUC_lexicon_tags_37 },
    { 0xcb43bae5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf763baec, SUC_lexicon_tags_7 },
    { 0x9b03baed, SUC_lexicon_tags_27 },
    { 0xb503baee, SUC_lexicon_tags_74 },
    { 0x6f03baef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd583baf1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4c03baf3, SUC_lexicon_tags_60 },
    { 0x93a3baf4, SUC_lexicon_tags_71 },
    { 0x12e3baf5, SUC_lexicon_tags_48 },
    { 0x2d83baf6, SUC_lexicon_tags_14 },
    { 0x16a3baf7, SUC_lexicon_tags_27 },
    { 0xa563baf5, SUC_lexicon_tags_46 },
    { 0xe8a3baf5, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e3bafc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd403baff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1723bb03, SUC_lexicon_tags_17 },
    { 0x55e3bb04, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa3bb07, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa223bb0d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x30e3bb0f, SUC_lexicon_tags_12 },
    { 0xd263bb10, SUC_lexicon_tags_952 },
    { 0x3903bb11, SUC_lexicon_tags_22 },
    { 0x5863bb11, SUC_lexicon_tags_64 },
    { 0x5163bb13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa3bb18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc3bb1c, SUC_lexicon_tags_6 },
    { 0xbd03bb1d, SUC_lexicon_tags_9 },
    { 0x11e3bb1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b43bb21, SUC_lexicon_tags_33 },
    { 0x4b23bb21, SUC_lexicon_tags_11 },
    { 0x1323bb23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1983bb26, SUC_lexicon_tags_15 },
    { 0xc0c3bb26, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5143bb29, SUC_lexicon_tags_49 },
    { 0x1763bb2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaca3bb2c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa603bb2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda3bb31, SUC_lexicon_tags_12 },
    { 0xd123bb32, SUC_lexicon_tags_607 },
    { 0, NULL },
    { 0, NULL },
    { 0xa123bb35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c3bb37, SUC_lexicon_tags_31 },
    { 0x5d43bb38, SUC_lexicon_tags_63 },
    { 0x5823bb38, SUC_lexicon_tags_27 },
    { 0x4e03bb3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xca43bb3d, SUC_lexicon_tags_17 },
    { 0x7223bb3e, SUC_lexicon_tags_37 },
    { 0xdd03bb3f, SUC_lexicon_tags_61 },
    { 0xf1a3bb3e, SUC_lexicon_tags_17 },
    { 0xaba3bb3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe923bb44, SUC_lexicon_tags_36 },
    { 0x74e3bb44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa3bb48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e3bb50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d43bb54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa163bb56, SUC_lexicon_tags_7 },
    { 0x33c3bb56, SUC_lexicon_tags_48 },
    { 0x263bb58, SUC_lexicon_tags_2 },
    { 0x2563bb56, SUC_lexicon_tags_7 },
    { 0x3f23bb5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3263bb5f, SUC_lexicon_tags_63 },
    { 0x9fc3bb60, SUC_lexicon_tags_17 },
    { 0x1d63bb61, SUC_lexicon_tags_37 },
    { 0xeaa3bb60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b63bb65, SUC_lexicon_tags_25 },
    { 0xea83bb65, SUC_lexicon_tags_47 },
    { 0x2d63bb65, SUC_lexicon_tags_77 },
    { 0xc3e3bb67, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa943bb6b, SUC_lexicon_tags_7 },
    { 0x7083bb6b, SUC_lexicon_tags_36 },
    { 0xce83bb6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c3bb71, SUC_lexicon_tags_12 },
    { 0xf483bb72, SUC_lexicon_tags_40 },
    { 0x95a3bb73, SUC_lexicon_tags_14 },
    { 0xe0c3bb73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f43bb77, SUC_lexicon_tags_14 },
    { 0x34c3bb78, SUC_lexicon_tags_6 },
    { 0xf3a3bb78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2043bb7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ee3bb80, SUC_lexicon_tags_7 },
    { 0xb743bb81, SUC_lexicon_tags_37 },
    { 0xb8e3bb82, SUC_lexicon_tags_7 },
    { 0xac43bb83, SUC_lexicon_tags_6 },
    { 0xe3a3bb83, SUC_lexicon_tags_37 },
    { 0xba63bb85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e43bb87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd4c3bb89, SUC_lexicon_tags_12 },
    { 0xdaa3bb8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c3bb8d, SUC_lexicon_tags_82 },
    { 0xa9c3bb8e, SUC_lexicon_tags_21 },
    { 0x2cc3bb8d, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0, NULL },
    { 0x9063bb92, SUC_lexicon_tags_14 },
    { 0x7663bb93, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2243bb95, SUC_lexicon_tags_18 },
    { 0xf3e3bb96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f23bb9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5123bb9f, SUC_lexicon_tags_14 },
    { 0x1f03bba0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8163bba3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc43bba5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d43bba8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc943bbaa, SUC_lexicon_tags_47 },
    { 0x5843bbab, SUC_lexicon_tags_23 },
    { 0xbe83bbab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4323bbb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a3bbb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3643bbb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe243bbba, SUC_lexicon_tags_12 },
    { 0x9783bbbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca3bbbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3bbc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c23bbc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaca3bbc5, SUC_lexicon_tags_2 },
    { 0x3463bbc5, SUC_lexicon_tags_17 },
    { 0xd2c3bbc7, SUC_lexicon_tags_7 },
    { 0xe6a3bbc8, SUC_lexicon_tags_4 },
    { 0xe2a3bbc9, SUC_lexicon_tags_19 },
    { 0xafc3bbc7, SUC_lexicon_tags_108 },
    { 0xd523bbc6, SUC_lexicon_tags_7 },
    { 0x4323bbcc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1683bbd3, SUC_lexicon_tags_65 },
    { 0x3823bbd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf123bbd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe143bbd8, SUC_lexicon_tags_34 },
    { 0x2d83bbd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6843bbdf, SUC_lexicon_tags_14 },
    { 0x6f63bbe0, SUC_lexicon_tags_25 },
    { 0x4463bbe1, SUC_lexicon_tags_2 },
    { 0xfce3bbe2, SUC_lexicon_tags_56 },
    { 0x763bbe3, SUC_lexicon_tags_14 },
    { 0x5083bbe0, SUC_lexicon_tags_12 },
    { 0x2823bbe4, SUC_lexicon_tags_66 },
    { 0xa0e3bbe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf363bbe8, SUC_lexicon_tags_14 },
    { 0xfa23bbe9, SUC_lexicon_tags_34 },
    { 0xe863bbe8, SUC_lexicon_tags_30 },
    { 0x8863bbeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde3bbef, SUC_lexicon_tags_26 },
    { 0x35c3bbef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd43bbf3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3bbf8, SUC_lexicon_tags_26 },
    { 0xd203bbf8, SUC_lexicon_tags_60 },
    { 0xea63bbfa, SUC_lexicon_tags_25 },
    { 0x1063bbfb, SUC_lexicon_tags_43 },
    { 0x9b23bbfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c3bc02, SUC_lexicon_tags_37 },
    { 0x57e3bc03, SUC_lexicon_tags_12 },
    { 0x5e83bc04, SUC_lexicon_tags_47 },
    { 0x3583bc05, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e3bc0a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9263bc0e, SUC_lexicon_tags_87 },
    { 0xab23bc0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9363bc12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb83bc16, SUC_lexicon_tags_2 },
    { 0xe283bc17, SUC_lexicon_tags_410 },
    { 0x57e3bc18, SUC_lexicon_tags_38 },
    { 0x3b03bc19, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc43bc1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa523bc1f, SUC_lexicon_tags_37 },
    { 0x4283bc1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbfe3bc22, SUC_lexicon_tags_7 },
    { 0xa263bc22, SUC_lexicon_tags_38 },
    { 0x5703bc23, SUC_lexicon_tags_0 },
    { 0x7163bc25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5743bc2e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a3bc31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43bc34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8943bc39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c3bc3c, SUC_lexicon_tags_68 },
    { 0x4ea3bc3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a83bc41, SUC_lexicon_tags_1 },
    { 0xc143bc42, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d63bc45, SUC_lexicon_tags_33 },
    { 0x3363bc45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd983bc48, SUC_lexicon_tags_25 },
    { 0x4b83bc49, SUC_lexicon_tags_23 },
    { 0x4323bc4a, SUC_lexicon_tags_14 },
    { 0xb663bc48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1143bc55, SUC_lexicon_tags_0 },
    { 0x36e3bc56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe3bc59, SUC_lexicon_tags_25 },
    { 0x5c3bc59, SUC_lexicon_tags_12 },
    { 0x12e3bc59, SUC_lexicon_tags_19 },
    { 0x5123bc5c, SUC_lexicon_tags_26 },
    { 0x32a3bc5d, SUC_lexicon_tags_66 },
    { 0xfa63bc5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1683bc61, SUC_lexicon_tags_17 },
    { 0xf983bc62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e3bc66, SUC_lexicon_tags_15 },
    { 0xe163bc66, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e3bc6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5243bc6c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb2a3bc6e, SUC_lexicon_tags_26 },
    { 0x1003bc6f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5023bc74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3bc77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c3bc7c, SUC_lexicon_tags_17 },
    { 0x21a3bc7d, SUC_lexicon_tags_1 },
    { 0x5ac3bc7e, SUC_lexicon_tags_1 },
    { 0x3b23bc7c, SUC_lexicon_tags_55 },
    { 0x5123bc80, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc3bc88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e3bc8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3bc94, SUC_lexicon_tags_7 },
    { 0xf903bc95, SUC_lexicon_tags_38 },
    { 0x2ec3bc96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb143bc9a, SUC_lexicon_tags_37 },
    { 0x3ca3bc9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd83bc9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7883bca7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c3bcaa, SUC_lexicon_tags_22 },
    { 0x57a3bcab, SUC_lexicon_tags_7 },
    { 0xb8a3bcac, SUC_lexicon_tags_47 },
    { 0xea23bcad, SUC_lexicon_tags_7 },
    { 0x9163bcad, SUC_lexicon_tags_8 },
    { 0x89a3bcaf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2663bcb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe683bcb6, SUC_lexicon_tags_44 },
    { 0xda03bcb7, SUC_lexicon_tags_31 },
    { 0x89e3bcb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3063bcba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x43bcbd, SUC_lexicon_tags_25 },
    { 0xbbc3bcbd, SUC_lexicon_tags_28 },
    { 0x93c3bcbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcac3bcc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc963bcc3, SUC_lexicon_tags_37 },
    { 0x6a23bcc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23bcc7, SUC_lexicon_tags_61 },
    { 0x2343bcc8, SUC_lexicon_tags_17 },
    { 0xe0c3bcc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e3bcce, SUC_lexicon_tags_137 },
    { 0, NULL },
    { 0xdda3bcd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6923bcd4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b23bcd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac43bcdd, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a3bce1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2ac3bce3, SUC_lexicon_tags_1 },
    { 0x3843bce4, SUC_lexicon_tags_60 },
    { 0xa563bce5, SUC_lexicon_tags_7 },
    { 0x3b63bce3, SUC_lexicon_tags_13 },
    { 0xfda3bce5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc023bcea, SUC_lexicon_tags_23 },
    { 0x48c3bceb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5843bcee, SUC_lexicon_tags_57 },
    { 0x5223bcee, SUC_lexicon_tags_37 },
    { 0x2863bcee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3943bcf5, SUC_lexicon_tags_49 },
    { 0xb0c3bcf6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a63bcfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6443bd01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6023bd0c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa0a3bd0e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4643bd10, SUC_lexicon_tags_21 },
    { 0x10c3bd11, SUC_lexicon_tags_61 },
    { 0x3543bd12, SUC_lexicon_tags_8 },
    { 0xc163bd10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e3bd16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3bd1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9a3bd1c, SUC_lexicon_tags_37 },
    { 0x2683bd1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3623bd22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65e3bd24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc23bd26, SUC_lexicon_tags_12 },
    { 0x1423bd26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e3bd2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef03bd30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1603bd37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5aa3bd39, SUC_lexicon_tags_4 },
    { 0xee43bd3a, SUC_lexicon_tags_11 },
    { 0x2c43bd3b, SUC_lexicon_tags_12 },
    { 0x89e3bd3a, SUC_lexicon_tags_21 },
    { 0x2ae3bd39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb663bd3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe963bd44, SUC_lexicon_tags_25 },
    { 0x6a63bd44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3c3bd47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8703bd4a, SUC_lexicon_tags_37 },
    { 0x4643bd4b, SUC_lexicon_tags_23 },
    { 0x3fa3bd4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeac3bd4e, SUC_lexicon_tags_47 },
    { 0x3963bd4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f63bd51, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae3bd54, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x4a63bd56, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3903bd62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc723bd67, SUC_lexicon_tags_8 },
    { 0x6203bd68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7103bd6d, SUC_lexicon_tags_70 },
    { 0x37c3bd6e, SUC_lexicon_tags_39 },
    { 0x5a83bd6f, SUC_lexicon_tags_25 },
    { 0xee3bd70, SUC_lexicon_tags_0 },
    { 0x85c3bd70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a3bd74, SUC_lexicon_tags_14 },
    { 0xc543bd75, SUC_lexicon_tags_17 },
    { 0x2103bd75, SUC_lexicon_tags_25 },
    { 0x343bd77, SUC_lexicon_tags_34 },
    { 0xb443bd78, SUC_lexicon_tags_10 },
    { 0xa243bd78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfae3bd7b, SUC_lexicon_tags_11 },
    { 0xeb23bd7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7063bd7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba63bd88, SUC_lexicon_tags_9 },
    { 0xd7e3bd89, SUC_lexicon_tags_11 },
    { 0x6ae3bd89, SUC_lexicon_tags_10 },
    { 0xa203bd8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a83bd8e, SUC_lexicon_tags_10 },
    { 0x19a3bd8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba3bd92, SUC_lexicon_tags_1016 },
    { 0, NULL },
    { 0x6ee3bd94, SUC_lexicon_tags_40 },
    { 0x6843bd95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d83bd9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd663bd9e, SUC_lexicon_tags_151 },
    { 0x3ca3bd9e, SUC_lexicon_tags_17 },
    { 0xe583bd9e, SUC_lexicon_tags_7 },
    { 0x50c3bda1, SUC_lexicon_tags_14 },
    { 0xbfe3bd9f, SUC_lexicon_tags_28 },
    { 0xeba3bda1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c3bda6, SUC_lexicon_tags_47 },
    { 0x3ce3bda7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a23bdaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47e3bdb1, SUC_lexicon_tags_65 },
    { 0x4c43bdb2, SUC_lexicon_tags_5 },
    { 0xc663bdb3, SUC_lexicon_tags_25 },
    { 0xb123bdb3, SUC_lexicon_tags_17 },
    { 0xa743bdb3, SUC_lexicon_tags_37 },
    { 0x4383bdb6, SUC_lexicon_tags_29 },
    { 0x4983bdb6, SUC_lexicon_tags_12 },
    { 0x7303bdb3, SUC_lexicon_tags_37 },
    { 0x5ea3bdb9, SUC_lexicon_tags_14 },
    { 0x3f43bdba, SUC_lexicon_tags_37 },
    { 0x3903bdbb, SUC_lexicon_tags_273 },
    { 0x2ae3bdb7, SUC_lexicon_tags_38 },
    { 0x7d63bdbd, SUC_lexicon_tags_15 },
    { 0x5023bdbc, SUC_lexicon_tags_11 },
    { 0xf4c3bdb8, SUC_lexicon_tags_4 },
    { 0x7d83bdb2, SUC_lexicon_tags_9 },
    { 0xb003bdc1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6c23bdc3, SUC_lexicon_tags_37 },
    { 0x9143bdc4, SUC_lexicon_tags_7 },
    { 0xc803bdc4, SUC_lexicon_tags_2 },
    { 0x1623bdc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c43bdc8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa03bdcb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcde3bdcd, SUC_lexicon_tags_27 },
    { 0xfbc3bdce, SUC_lexicon_tags_25 },
    { 0x80c3bdcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c83bdd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a83bddb, SUC_lexicon_tags_0 },
    { 0x7a3bddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1303bddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd183bde3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbf03bde5, SUC_lexicon_tags_9 },
    { 0x9223bde6, SUC_lexicon_tags_1 },
    { 0xa0c3bde7, SUC_lexicon_tags_43 },
    { 0xba63bde7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc963bdec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5323bdee, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x7663bdf1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8323bdf4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2983bdf8, SUC_lexicon_tags_17 },
    { 0x7c3bdf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a83bdfb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9543be01, SUC_lexicon_tags_6 },
    { 0xd723be02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ae3be04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf203be08, SUC_lexicon_tags_47 },
    { 0x48a3be08, SUC_lexicon_tags_8 },
    { 0x89c3be08, SUC_lexicon_tags_1 },
    { 0xc0a3be0b, SUC_lexicon_tags_17 },
    { 0x10e3be08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf563be0f, SUC_lexicon_tags_37 },
    { 0x22a3be0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb43be12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21e3be14, SUC_lexicon_tags_69 },
    { 0x9ee3be15, SUC_lexicon_tags_29 },
    { 0x9843be16, SUC_lexicon_tags_12 },
    { 0xd463be16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a3be1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26c3be1f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba3be26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc063be29, SUC_lexicon_tags_11 },
    { 0xdce3be29, SUC_lexicon_tags_17 },
    { 0xd163be2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9663be2d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8103be30, SUC_lexicon_tags_16 },
    { 0xf2a3be31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8483be36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6423be3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a23be3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d83be40, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5523be42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4803be44, SUC_lexicon_tags_47 },
    { 0xbcc3be45, SUC_lexicon_tags_2 },
    { 0x9f43be44, SUC_lexicon_tags_37 },
    { 0x9f83be44, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7623be4c, SUC_lexicon_tags_26 },
    { 0x3c63be4c, SUC_lexicon_tags_43 },
    { 0x85a3be4d, SUC_lexicon_tags_124 },
    { 0xfa3be4f, SUC_lexicon_tags_1 },
    { 0x7223be50, SUC_lexicon_tags_63 },
    { 0x8b43be51, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03be55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5da3be57, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe3be5c, SUC_lexicon_tags_17 },
    { 0x9143be5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee3be62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8103be69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x63be6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa063be6f, SUC_lexicon_tags_33 },
    { 0x8b43be6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923be77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c03be79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a3be7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e3be80, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac3be84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdce3be86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78e3be88, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x22c3be8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c03be8c, SUC_lexicon_tags_2 },
    { 0x6203be8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f23be90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd323be93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a23be96, SUC_lexicon_tags_12 },
    { 0xe8e3be96, SUC_lexicon_tags_60 },
    { 0xe9e3be97, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783be9f, SUC_lexicon_tags_1 },
    { 0xcd03bea0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1003bea2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423beab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2743beae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6043beb0, SUC_lexicon_tags_1 },
    { 0x73a3beb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c3beba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa683bebc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffe3bebe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb03bec2, SUC_lexicon_tags_38 },
    { 0x2403bec3, SUC_lexicon_tags_33 },
    { 0xef83bec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7543becc, SUC_lexicon_tags_60 },
    { 0x9e3becc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e83bed1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5a3bed3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x51e3bed5, SUC_lexicon_tags_4 },
    { 0xcee3bed6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x23bed8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd63bedc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c3bedf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad63bee1, SUC_lexicon_tags_6 },
    { 0x2e63bee1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bee5, SUC_lexicon_tags_18 },
    { 0x8803bee6, SUC_lexicon_tags_11 },
    { 0xb3e3bee7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xddc3bee9, SUC_lexicon_tags_17 },
    { 0xbec3beea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4083beed, SUC_lexicon_tags_14 },
    { 0x6f63beee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5483bef1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e03bef4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9443bef7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7423befa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e3befe, SUC_lexicon_tags_11 },
    { 0x42c3beff, SUC_lexicon_tags_43 },
    { 0x1ea3befe, SUC_lexicon_tags_45 },
    { 0x99e3bf01, SUC_lexicon_tags_63 },
    { 0xfe83bf02, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe723bf05, SUC_lexicon_tags_32 },
    { 0x90c3bf06, SUC_lexicon_tags_2 },
    { 0x67c3bf07, SUC_lexicon_tags_3 },
    { 0xcb43bf08, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7343bf0a, SUC_lexicon_tags_39 },
    { 0xf303bf0b, SUC_lexicon_tags_31 },
    { 0x3fc3bf0a, SUC_lexicon_tags_2 },
    { 0x8303bf0b, SUC_lexicon_tags_47 },
    { 0x93e3bf0d, SUC_lexicon_tags_23 },
    { 0xfe03bf0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e3bf12, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xec23bf15, SUC_lexicon_tags_9 },
    { 0xea43bf16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5783bf19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e03bf1c, SUC_lexicon_tags_29 },
    { 0xcd03bf1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c3bf21, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae3bf27, SUC_lexicon_tags_16 },
    { 0x62a3bf28, SUC_lexicon_tags_45 },
    { 0x4863bf29, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7c23bf2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503bf31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc163bf34, SUC_lexicon_tags_11 },
    { 0xce63bf35, SUC_lexicon_tags_66 },
    { 0xc2e3bf36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa3bf38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1963bf3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3683bf42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e3bf46, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x61c3bf48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43bf4c, SUC_lexicon_tags_37 },
    { 0x86c3bf4c, SUC_lexicon_tags_61 },
    { 0xd243bf4e, SUC_lexicon_tags_6 },
    { 0xc2e3bf4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e23bf53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9883bf55, SUC_lexicon_tags_25 },
    { 0xdd43bf55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6e3bf58, SUC_lexicon_tags_45 },
    { 0x6083bf58, SUC_lexicon_tags_65 },
    { 0x8983bf5a, SUC_lexicon_tags_1 },
    { 0xa363bf5a, SUC_lexicon_tags_23 },
    { 0xc1c3bf5c, SUC_lexicon_tags_25 },
    { 0xa3bf58, SUC_lexicon_tags_27 },
    { 0x8f63bf5e, SUC_lexicon_tags_11 },
    { 0x43e3bf5e, SUC_lexicon_tags_11 },
    { 0xe303bf60, SUC_lexicon_tags_11 },
    { 0x2983bf60, SUC_lexicon_tags_37 },
    { 0x29c3bf61, SUC_lexicon_tags_42 },
    { 0x2fe3bf5d, SUC_lexicon_tags_7 },
    { 0xa983bf64, SUC_lexicon_tags_14 },
    { 0x5fa3bf65, SUC_lexicon_tags_27 },
    { 0xbb63bf66, SUC_lexicon_tags_0 },
    { 0x1fe3bf65, SUC_lexicon_tags_27 },
    { 0xb823bf68, SUC_lexicon_tags_25 },
    { 0x563bf69, SUC_lexicon_tags_7 },
    { 0xe343bf69, SUC_lexicon_tags_7 },
    { 0x4c03bf69, SUC_lexicon_tags_17 },
    { 0xa983bf67, SUC_lexicon_tags_37 },
    { 0x343bf5b, SUC_lexicon_tags_11 },
    { 0x1f03bf6e, SUC_lexicon_tags_31 },
    { 0x3b83bf6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da3bf72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd063bf74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6843bf76, SUC_lexicon_tags_25 },
    { 0x8d43bf77, SUC_lexicon_tags_28 },
    { 0xaec3bf78, SUC_lexicon_tags_12 },
    { 0x76c3bf77, SUC_lexicon_tags_17 },
    { 0x1543bf77, SUC_lexicon_tags_11 },
    { 0x63bf79, SUC_lexicon_tags_11 },
    { 0xbda3bf7c, SUC_lexicon_tags_60 },
    { 0xa83bf7b, SUC_lexicon_tags_7 },
    { 0xca43bf7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd003bf83, SUC_lexicon_tags_17 },
    { 0x9c83bf84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc3bf8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf803bf90, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6643bf96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc503bf99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d43bf9c, SUC_lexicon_tags_39 },
    { 0xf803bf9d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8183bf9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc423bfa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a3bfa5, SUC_lexicon_tags_38 },
    { 0xa1a3bfa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa043bfac, SUC_lexicon_tags_220 },
    { 0xab83bfac, SUC_lexicon_tags_43 },
    { 0x6ca3bfad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c83bfb0, SUC_lexicon_tags_47 },
    { 0xd523bfb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb603bfb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e23bfb6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23bfba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd723bfbd, SUC_lexicon_tags_7 },
    { 0x7a83bfbe, SUC_lexicon_tags_49 },
    { 0xe103bfbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b03bfc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c3bfc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d43bfc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ce3bfca, SUC_lexicon_tags_2 },
    { 0x80e3bfcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc3bfce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703bfd2, SUC_lexicon_tags_28 },
    { 0xad23bfd3, SUC_lexicon_tags_29 },
    { 0xb5e3bfd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e3bfd8, SUC_lexicon_tags_37 },
    { 0x19c3bfd9, SUC_lexicon_tags_37 },
    { 0x3e3bfda, SUC_lexicon_tags_19 },
    { 0xd423bfd9, SUC_lexicon_tags_23 },
    { 0x4fa3bfda, SUC_lexicon_tags_59 },
    { 0xe783bfd8, SUC_lexicon_tags_65 },
    { 0xfe03bfdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7983bfe0, SUC_lexicon_tags_2 },
    { 0xb7a3bfe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcae3bfe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1403bfe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d03bfef, SUC_lexicon_tags_7 },
    { 0xb7a3bff0, SUC_lexicon_tags_1 },
    { 0xb2a3bfef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5043bffa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba3bffe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3c004, SUC_lexicon_tags_7 },
    { 0xae23c005, SUC_lexicon_tags_61 },
    { 0x63c3c006, SUC_lexicon_tags_63 },
    { 0xa6c3c004, SUC_lexicon_tags_9 },
    { 0x1de3c008, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0xf5c3c00a, SUC_lexicon_tags_1 },
    { 0x1b23c00b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9da3c00d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e3c015, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf63c017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5963c01b, SUC_lexicon_tags_17 },
    { 0x6943c01c, SUC_lexicon_tags_2 },
    { 0xbde3c01c, SUC_lexicon_tags_34 },
    { 0x1e3c01b, SUC_lexicon_tags_13 },
    { 0x1843c01f, SUC_lexicon_tags_37 },
    { 0x8f63c020, SUC_lexicon_tags_74 },
    { 0x1a43c021, SUC_lexicon_tags_4 },
    { 0x70c3c022, SUC_lexicon_tags_37 },
    { 0x5703c023, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x99a3c025, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd2e3c027, SUC_lexicon_tags_39 },
    { 0xdc83c028, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d03c02a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c63c02e, SUC_lexicon_tags_7 },
    { 0xbc3c02f, SUC_lexicon_tags_12 },
    { 0xa2a3c030, SUC_lexicon_tags_31 },
    { 0x2623c031, SUC_lexicon_tags_38 },
    { 0xf083c031, SUC_lexicon_tags_12 },
    { 0x683c033, SUC_lexicon_tags_28 },
    { 0x9163c032, SUC_lexicon_tags_7 },
    { 0x6a3c031, SUC_lexicon_tags_16 },
    { 0x6023c036, SUC_lexicon_tags_1 },
    { 0xa2c3c035, SUC_lexicon_tags_7 },
    { 0x2943c038, SUC_lexicon_tags_1 },
    { 0xa463c039, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf43c03b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3243c03d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab63c044, SUC_lexicon_tags_13 },
    { 0xae03c045, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1163c048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4563c04a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8583c04e, SUC_lexicon_tags_71 },
    { 0xbec3c04f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183c054, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c3c059, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa83c05b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8103c05d, SUC_lexicon_tags_12 },
    { 0x69e3c05e, SUC_lexicon_tags_15 },
    { 0xaf43c05f, SUC_lexicon_tags_35 },
    { 0x75a3c05f, SUC_lexicon_tags_17 },
    { 0xc043c061, SUC_lexicon_tags_37 },
    { 0x2cc3c062, SUC_lexicon_tags_17 },
    { 0x7d83c05f, SUC_lexicon_tags_35 },
    { 0x5da3c063, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3be3c066, SUC_lexicon_tags_12 },
    { 0x5a83c067, SUC_lexicon_tags_26 },
    { 0xea3c068, SUC_lexicon_tags_11 },
    { 0xa883c069, SUC_lexicon_tags_29 },
    { 0x4443c06a, SUC_lexicon_tags_7 },
    { 0xcc03c06b, SUC_lexicon_tags_26 },
    { 0x6ee3c067, SUC_lexicon_tags_25 },
    { 0xc263c06c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c3c071, SUC_lexicon_tags_19 },
    { 0xbfa3c072, SUC_lexicon_tags_70 },
    { 0x9e03c072, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb543c07a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2103c07d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f43c083, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c3c086, SUC_lexicon_tags_14 },
    { 0xdf83c087, SUC_lexicon_tags_14 },
    { 0x54e3c088, SUC_lexicon_tags_30 },
    { 0x5e23c087, SUC_lexicon_tags_15 },
    { 0x9643c08a, SUC_lexicon_tags_9 },
    { 0xe183c08b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae3c08f, SUC_lexicon_tags_34 },
    { 0x28e3c090, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd103c092, SUC_lexicon_tags_37 },
    { 0x8dc3c092, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1103c097, SUC_lexicon_tags_39 },
    { 0xbca3c098, SUC_lexicon_tags_37 },
    { 0x6223c099, SUC_lexicon_tags_38 },
    { 0xa1a3c097, SUC_lexicon_tags_7 },
    { 0xfbc3c09b, SUC_lexicon_tags_7 },
    { 0x1363c09c, SUC_lexicon_tags_59 },
    { 0xc963c097, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a3c0a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a43c0a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa103c0a6, SUC_lexicon_tags_11 },
    { 0xc9e3c0a7, SUC_lexicon_tags_8 },
    { 0xc3a3c0a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8083c0ac, SUC_lexicon_tags_0 },
    { 0xeaa3c0ac, SUC_lexicon_tags_65 },
    { 0x3723c0ae, SUC_lexicon_tags_23 },
    { 0x8763c0ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3c0b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8423c0b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd823c0b8, SUC_lexicon_tags_25 },
    { 0xac63c0b8, SUC_lexicon_tags_17 },
    { 0xb323c0ba, SUC_lexicon_tags_7 },
    { 0x8203c0bb, SUC_lexicon_tags_4 },
    { 0xa003c0bc, SUC_lexicon_tags_2 },
    { 0x8043c0bd, SUC_lexicon_tags_6 },
    { 0xd963c0bd, SUC_lexicon_tags_1 },
    { 0x2343c0bf, SUC_lexicon_tags_12 },
    { 0xdbe3c0bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb363c0c3, SUC_lexicon_tags_12 },
    { 0xe1e3c0c4, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x1003c0c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a3c0cc, SUC_lexicon_tags_11 },
    { 0x21a3c0cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c3c0d0, SUC_lexicon_tags_27 },
    { 0x6743c0d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7423c0d3, SUC_lexicon_tags_49 },
    { 0xd0e3c0d4, SUC_lexicon_tags_15 },
    { 0x5623c0d5, SUC_lexicon_tags_30 },
    { 0x1903c0d6, SUC_lexicon_tags_19 },
    { 0xc203c0d7, SUC_lexicon_tags_32 },
    { 0x6d43c0d7, SUC_lexicon_tags_7 },
    { 0xfe23c0d9, SUC_lexicon_tags_17 },
    { 0xf903c0d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe723c0dd, SUC_lexicon_tags_7 },
    { 0x28a3c0de, SUC_lexicon_tags_0 },
    { 0xfa63c0df, SUC_lexicon_tags_15 },
    { 0xb2c3c0df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c3c0e3, SUC_lexicon_tags_2 },
    { 0x7f83c0e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea3c0eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e3c0f2, SUC_lexicon_tags_47 },
    { 0x6f83c0f3, SUC_lexicon_tags_47 },
    { 0xe363c0f4, SUC_lexicon_tags_25 },
    { 0xe5c3c0f5, SUC_lexicon_tags_12 },
    { 0xa543c0f5, SUC_lexicon_tags_136 },
    { 0x4fa3c0f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3323c0fa, SUC_lexicon_tags_11 },
    { 0x8b83c0fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa3c102, SUC_lexicon_tags_15 },
    { 0xf023c102, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3143c106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7cc3c108, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6963c10d, SUC_lexicon_tags_47 },
    { 0xe9e3c10d, SUC_lexicon_tags_2 },
    { 0x8563c10f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf523c112, SUC_lexicon_tags_60 },
    { 0x9603c113, SUC_lexicon_tags_37 },
    { 0xc683c112, SUC_lexicon_tags_37 },
    { 0x8843c115, SUC_lexicon_tags_47 },
    { 0x93a3c116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf43c118, SUC_lexicon_tags_148 },
    { 0xbc23c119, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7c3c11b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xad63c11d, SUC_lexicon_tags_7 },
    { 0xfc83c11e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7b23c120, SUC_lexicon_tags_10 },
    { 0x2a23c121, SUC_lexicon_tags_68 },
    { 0xf3c3c121, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd03c124, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3043c12b, SUC_lexicon_tags_23 },
    { 0x1d83c12c, SUC_lexicon_tags_1 },
    { 0x8ea3c12c, SUC_lexicon_tags_47 },
    { 0xea83c12e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c3c133, SUC_lexicon_tags_27 },
    { 0xcd83c134, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17c3c136, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e3c139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7c3c13b, SUC_lexicon_tags_7 },
    { 0x9bc3c13c, SUC_lexicon_tags_6 },
    { 0x4043c13d, SUC_lexicon_tags_21 },
    { 0x22a3c13e, SUC_lexicon_tags_27 },
    { 0x77c3c13f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd103c142, SUC_lexicon_tags_13 },
    { 0x83e3c143, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcea3c146, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe63c148, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97e3c14a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf83c153, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5e3c155, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x48a3c157, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce3c15a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c3c15d, SUC_lexicon_tags_37 },
    { 0xf1e3c15d, SUC_lexicon_tags_37 },
    { 0xfc83c15f, SUC_lexicon_tags_11 },
    { 0x7063c15f, SUC_lexicon_tags_1 },
    { 0xa683c161, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4de3c163, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc3c166, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51a3c168, SUC_lexicon_tags_17 },
    { 0xc843c168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe123c16c, SUC_lexicon_tags_7 },
    { 0xbd43c16c, SUC_lexicon_tags_17 },
    { 0x6663c16e, SUC_lexicon_tags_45 },
    { 0x4e43c16c, SUC_lexicon_tags_37 },
    { 0xc6a3c170, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x583c173, SUC_lexicon_tags_15 },
    { 0x843c174, SUC_lexicon_tags_49 },
    { 0x4523c174, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a43c178, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4a3c17a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b03c180, SUC_lexicon_tags_1 },
    { 0xaae3c180, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a43c186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd943c18c, SUC_lexicon_tags_4 },
    { 0x3c18d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f83c18f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4c3c191, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a3c194, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdfe3c196, SUC_lexicon_tags_11 },
    { 0xafa3c196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa043c199, SUC_lexicon_tags_2 },
    { 0xb963c19a, SUC_lexicon_tags_29 },
    { 0xa2c3c19b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c3c19e, SUC_lexicon_tags_0 },
    { 0xb0e3c19f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda3c1a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee3c1a5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3043c1a7, SUC_lexicon_tags_38 },
    { 0x8463c1a8, SUC_lexicon_tags_1 },
    { 0x5f03c1a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f03c1b3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa243c1b9, SUC_lexicon_tags_49 },
    { 0x2de3c1ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb063c1bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xffc3c1be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7963c1c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac3c1c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3c1ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46a3c1d0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xec63c1d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e63c1d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce3c1dd, SUC_lexicon_tags_66 },
    { 0xcfa3c1de, SUC_lexicon_tags_66 },
    { 0xea83c1dd, SUC_lexicon_tags_27 },
    { 0xbec3c1e0, SUC_lexicon_tags_28 },
    { 0xda23c1de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1c3c1e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc3c1e6, SUC_lexicon_tags_25 },
    { 0x66a3c1e7, SUC_lexicon_tags_7 },
    { 0x2d43c1e6, SUC_lexicon_tags_10 },
    { 0xac43c1e6, SUC_lexicon_tags_2 },
    { 0xfc03c1e7, SUC_lexicon_tags_9 },
    { 0x7143c1eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e3c1f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4123c1f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2623c1f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23c1fa, SUC_lexicon_tags_0 },
    { 0xe03c1fa, SUC_lexicon_tags_17 },
    { 0x33c3c1fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc03c201, SUC_lexicon_tags_30 },
    { 0x9a63c201, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xf563c204, SUC_lexicon_tags_23 },
    { 0x4c63c205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8743c207, SUC_lexicon_tags_19 },
    { 0xf963c208, SUC_lexicon_tags_37 },
    { 0xf5a3c208, SUC_lexicon_tags_30 },
    { 0x1003c20a, SUC_lexicon_tags_15 },
    { 0x9303c20b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfc23c20d, SUC_lexicon_tags_1 },
    { 0x6023c20d, SUC_lexicon_tags_7 },
    { 0x7703c20f, SUC_lexicon_tags_43 },
    { 0x78c3c210, SUC_lexicon_tags_2 },
    { 0xc283c211, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe83c214, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e3c217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4943c21a, SUC_lexicon_tags_26 },
    { 0x8e43c21a, SUC_lexicon_tags_158 },
    { 0x2883c21b, SUC_lexicon_tags_26 },
    { 0x1483c21c, SUC_lexicon_tags_7 },
    { 0xa103c21e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff43c221, SUC_lexicon_tags_31 },
    { 0x74c3c222, SUC_lexicon_tags_47 },
    { 0xd023c221, SUC_lexicon_tags_21 },
    { 0x9de3c221, SUC_lexicon_tags_63 },
    { 0xc503c222, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8483c229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3c22c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec3c232, SUC_lexicon_tags_11 },
    { 0xacc3c233, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9483c237, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xe123c239, SUC_lexicon_tags_37 },
    { 0x1623c23a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f43c23d, SUC_lexicon_tags_33 },
    { 0x843c23d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6823c241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7143c244, SUC_lexicon_tags_27 },
    { 0xe0a3c244, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec3c248, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0x7943c24b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a3c255, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12e3c257, SUC_lexicon_tags_19 },
    { 0x4183c257, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3c25d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe743c265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c3c271, SUC_lexicon_tags_33 },
    { 0x1a3c272, SUC_lexicon_tags_15 },
    { 0x52a3c273, SUC_lexicon_tags_45 },
    { 0x8403c271, SUC_lexicon_tags_17 },
    { 0xeec3c272, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd163c277, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbb23c279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f83c27d, SUC_lexicon_tags_65 },
    { 0x49e3c27e, SUC_lexicon_tags_25 },
    { 0x26c3c27f, SUC_lexicon_tags_37 },
    { 0xb923c280, SUC_lexicon_tags_16 },
    { 0x6f63c280, SUC_lexicon_tags_7 },
    { 0x4003c280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a3c288, SUC_lexicon_tags_48 },
    { 0x7683c289, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa343c28e, SUC_lexicon_tags_1 },
    { 0xb583c28e, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0xd203c292, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63c296, SUC_lexicon_tags_35 },
    { 0xb803c297, SUC_lexicon_tags_26 },
    { 0x5203c298, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5a03c29a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf83c2a1, SUC_lexicon_tags_45 },
    { 0x2183c2a1, SUC_lexicon_tags_23 },
    { 0x59e3c2a3, SUC_lexicon_tags_33 },
    { 0x4a63c2a4, SUC_lexicon_tags_29 },
    { 0xbc43c2a5, SUC_lexicon_tags_12 },
    { 0xb8c3c2a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb9c3c2a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7383c2aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc3c2ae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae3c2b1, SUC_lexicon_tags_25 },
    { 0x7ba3c2b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa663c2b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe943c2b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4d23c2ba, SUC_lexicon_tags_27 },
    { 0x4f63c2bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6303c2c0, SUC_lexicon_tags_37 },
    { 0x1b03c2c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee3c2c5, SUC_lexicon_tags_37 },
    { 0x8343c2c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d03c2c8, SUC_lexicon_tags_16 },
    { 0x2403c2c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ba3c2cb, SUC_lexicon_tags_25 },
    { 0x9b63c2cc, SUC_lexicon_tags_7 },
    { 0x3803c2cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa3c2d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e3c2d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c3c2d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9983c2da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23c2de, SUC_lexicon_tags_11 },
    { 0x7ca3c2df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6283c2e1, SUC_lexicon_tags_17 },
    { 0x81c3c2e1, SUC_lexicon_tags_44 },
    { 0x50c3c2e3, SUC_lexicon_tags_1 },
    { 0x6243c2e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de3c2e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe3c2ef, SUC_lexicon_tags_13 },
    { 0xfba3c2f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8f83c2f2, SUC_lexicon_tags_991 },
    { 0, NULL },
    { 0x3d83c2f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc083c2fd, SUC_lexicon_tags_15 },
    { 0xd543c2fe, SUC_lexicon_tags_27 },
    { 0xc943c2ff, SUC_lexicon_tags_11 },
    { 0xabe3c300, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e3c305, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8723c307, SUC_lexicon_tags_11 },
    { 0xada3c307, SUC_lexicon_tags_37 },
    { 0x6503c309, SUC_lexicon_tags_12 },
    { 0xd0e3c307, SUC_lexicon_tags_38 },
    { 0x9843c30b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4843c30d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x803c30f, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x3be3c311, SUC_lexicon_tags_15 },
    { 0xa63c312, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3c3c314, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b63c318, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5083c31e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c3c321, SUC_lexicon_tags_7 },
    { 0x6943c322, SUC_lexicon_tags_7 },
    { 0x3823c323, SUC_lexicon_tags_15 },
    { 0x4fc3c323, SUC_lexicon_tags_17 },
    { 0xf463c323, SUC_lexicon_tags_37 },
    { 0x4a63c326, SUC_lexicon_tags_7 },
    { 0x4a23c327, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x78c3c329, SUC_lexicon_tags_16 },
    { 0xf523c329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51c3c32c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd4e3c32e, SUC_lexicon_tags_7 },
    { 0xb623c32e, SUC_lexicon_tags_60 },
    { 0x4663c330, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c83c333, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c03c338, SUC_lexicon_tags_17 },
    { 0x2d83c339, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb043c33c, SUC_lexicon_tags_37 },
    { 0x46c3c33c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e3c340, SUC_lexicon_tags_25 },
    { 0xd43c341, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5103c343, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3e3c345, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e3c349, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0c3c34b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb703c352, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e3c357, SUC_lexicon_tags_25 },
    { 0xb83c358, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba3c35d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae3c363, SUC_lexicon_tags_2 },
    { 0xa2a3c364, SUC_lexicon_tags_16 },
    { 0x6603c365, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1ae3c367, SUC_lexicon_tags_27 },
    { 0x11e3c368, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8583c36f, SUC_lexicon_tags_66 },
    { 0x5b83c36f, SUC_lexicon_tags_4 },
    { 0x3c3c371, SUC_lexicon_tags_32 },
    { 0xdac3c36f, SUC_lexicon_tags_25 },
    { 0xefc3c370, SUC_lexicon_tags_94 },
    { 0xc0a3c36f, SUC_lexicon_tags_17 },
    { 0x4103c36f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c23c37a, SUC_lexicon_tags_31 },
    { 0xd263c37b, SUC_lexicon_tags_16 },
    { 0x9643c37b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc43c37f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc043c381, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede3c385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae3c391, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfca3c393, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe0a3c395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1c3c397, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23c39c, SUC_lexicon_tags_1 },
    { 0x1f23c39d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf483c3a2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6383c3a5, SUC_lexicon_tags_25 },
    { 0xb043c3a6, SUC_lexicon_tags_29 },
    { 0x73a3c3a7, SUC_lexicon_tags_7 },
    { 0x6503c3a8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x51e3c3aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb03c3af, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb63c3b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8523c3c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87a3c3c3, SUC_lexicon_tags_30 },
    { 0x983c3c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f63c3c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf263c3c8, SUC_lexicon_tags_6 },
    { 0xf903c3c9, SUC_lexicon_tags_43 },
    { 0xe463c3ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb223c3cd, SUC_lexicon_tags_30 },
    { 0x8883c3ce, SUC_lexicon_tags_46 },
    { 0xac23c3cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1943c3d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8803c3d3, SUC_lexicon_tags_11 },
    { 0xe383c3d4, SUC_lexicon_tags_37 },
    { 0x2c63c3d5, SUC_lexicon_tags_16 },
    { 0x5903c3d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e3c3dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7783c3e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e23c3e6, SUC_lexicon_tags_11 },
    { 0x4d63c3e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6283c3e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e23c3eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c3c3ee, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be3c3f3, SUC_lexicon_tags_15 },
    { 0x3423c3f4, SUC_lexicon_tags_8 },
    { 0x86a3c3f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c3fa, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3c3fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8923c408, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd303c40b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9603c40f, SUC_lexicon_tags_19 },
    { 0x59e3c410, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce23c416, SUC_lexicon_tags_21 },
    { 0xa2c3c416, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae3c41e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7403c421, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a23c423, SUC_lexicon_tags_22 },
    { 0xa5a3c423, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x12e3c426, SUC_lexicon_tags_1 },
    { 0x4e03c426, SUC_lexicon_tags_8 },
    { 0x3923c428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5363c42f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa3c431, SUC_lexicon_tags_7 },
    { 0x5a23c431, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa3c447, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x763c449, SUC_lexicon_tags_7 },
    { 0xd143c449, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5523c44f, SUC_lexicon_tags_12 },
    { 0x4803c44f, SUC_lexicon_tags_14 },
    { 0xf183c451, SUC_lexicon_tags_27 },
    { 0xba3c452, SUC_lexicon_tags_25 },
    { 0xbca3c452, SUC_lexicon_tags_23 },
    { 0xbc03c450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e3c45a, SUC_lexicon_tags_26 },
    { 0x2423c45b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5b03c45d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd383c460, SUC_lexicon_tags_11 },
    { 0x2583c461, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7403c463, SUC_lexicon_tags_29 },
    { 0x85a3c463, SUC_lexicon_tags_26 },
    { 0xdc3c465, SUC_lexicon_tags_40 },
    { 0xefc3c466, SUC_lexicon_tags_19 },
    { 0x1f43c467, SUC_lexicon_tags_19 },
    { 0x7223c468, SUC_lexicon_tags_11 },
    { 0xdfa3c468, SUC_lexicon_tags_10 },
    { 0x9a63c466, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc63c46c, SUC_lexicon_tags_92 },
    { 0x6e3c46d, SUC_lexicon_tags_15 },
    { 0x4ea3c46e, SUC_lexicon_tags_197 },
    { 0x8b43c46e, SUC_lexicon_tags_17 },
    { 0x8f43c46f, SUC_lexicon_tags_34 },
    { 0x5e3c46c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfda3c473, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd63c47c, SUC_lexicon_tags_63 },
    { 0xbfa3c47d, SUC_lexicon_tags_12 },
    { 0x8f23c47d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c3c480, SUC_lexicon_tags_5 },
    { 0xeaa3c480, SUC_lexicon_tags_80 },
    { 0x56c3c480, SUC_lexicon_tags_43 },
    { 0x6783c483, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4363c486, SUC_lexicon_tags_1 },
    { 0x32a3c487, SUC_lexicon_tags_29 },
    { 0x5c83c488, SUC_lexicon_tags_1 },
    { 0x5a63c488, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c3c48d, SUC_lexicon_tags_17 },
    { 0x7723c48d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e3c491, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4cc3c493, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f63c495, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e3c49d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe023c49f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d43c4a6, SUC_lexicon_tags_11 },
    { 0x5303c4a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe423c4ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfa3c4ae, SUC_lexicon_tags_37 },
    { 0xf123c4ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf963c4b2, SUC_lexicon_tags_1 },
    { 0xbaa3c4b2, SUC_lexicon_tags_15 },
    { 0x2683c4b4, SUC_lexicon_tags_91 },
    { 0x62e3c4b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283c4ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1083c4bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6103c4c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3263c4c3, SUC_lexicon_tags_12 },
    { 0xd2e3c4c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e63c4ca, SUC_lexicon_tags_1476 },
    { 0xaf43c4ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf703c4cf, SUC_lexicon_tags_31 },
    { 0x4dc3c4d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03c4d5, SUC_lexicon_tags_47 },
    { 0xbb43c4d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf583c4d8, SUC_lexicon_tags_95 },
    { 0xc5a3c4d9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d43c4dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9863c4de, SUC_lexicon_tags_11 },
    { 0x9223c4df, SUC_lexicon_tags_2 },
    { 0x5d03c4e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3c4e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe1c3c4e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc3c4ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8003c4ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa063c4ef, SUC_lexicon_tags_23 },
    { 0x9963c4f0, SUC_lexicon_tags_11 },
    { 0x8463c4f1, SUC_lexicon_tags_23 },
    { 0x8fa3c4f2, SUC_lexicon_tags_21 },
    { 0x79e3c4f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a3c4f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe183c4f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c3c4fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a3c501, SUC_lexicon_tags_12 },
    { 0xff03c501, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3c505, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e3c508, SUC_lexicon_tags_59 },
    { 0x5b43c509, SUC_lexicon_tags_14 },
    { 0x903c509, SUC_lexicon_tags_0 },
    { 0x2323c50b, SUC_lexicon_tags_37 },
    { 0xc323c50c, SUC_lexicon_tags_25 },
    { 0xd703c50c, SUC_lexicon_tags_37 },
    { 0xb383c50e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be3c513, SUC_lexicon_tags_33 },
    { 0x3403c513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49e3c516, SUC_lexicon_tags_3 },
    { 0x17e3c516, SUC_lexicon_tags_352 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9203c51d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a23c51f, SUC_lexicon_tags_34 },
    { 0xc0e3c520, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5343c526, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xebe3c528, SUC_lexicon_tags_38 },
    { 0x32a3c529, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c3c52c, SUC_lexicon_tags_37 },
    { 0x13e3c52d, SUC_lexicon_tags_37 },
    { 0x2a83c52d, SUC_lexicon_tags_5 },
    { 0x37e3c52f, SUC_lexicon_tags_37 },
    { 0x44e3c530, SUC_lexicon_tags_56 },
    { 0xdbe3c531, SUC_lexicon_tags_19 },
    { 0xda63c532, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac3c536, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4743c53a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e3c53d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe643c543, SUC_lexicon_tags_43 },
    { 0x4543c543, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xee43c546, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9603c549, SUC_lexicon_tags_7 },
    { 0x93a3c54a, SUC_lexicon_tags_29 },
    { 0x6bc3c549, SUC_lexicon_tags_38 },
    { 0x3da3c54c, SUC_lexicon_tags_17 },
    { 0xc283c549, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c3c553, SUC_lexicon_tags_15 },
    { 0x46e3c554, SUC_lexicon_tags_2 },
    { 0xaac3c553, SUC_lexicon_tags_25 },
    { 0x8183c556, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb703c559, SUC_lexicon_tags_10 },
    { 0x7f03c559, SUC_lexicon_tags_7 },
    { 0x7e3c55b, SUC_lexicon_tags_84 },
    { 0x6ce3c55b, SUC_lexicon_tags_148 },
    { 0xda3c55c, SUC_lexicon_tags_12 },
    { 0xdfc3c55e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad43c567, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbae3c569, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d43c56b, SUC_lexicon_tags_0 },
    { 0x4743c56b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c3c56f, SUC_lexicon_tags_15 },
    { 0xb703c56f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec3c574, SUC_lexicon_tags_7 },
    { 0x8943c575, SUC_lexicon_tags_0 },
    { 0x4ca3c576, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd843c578, SUC_lexicon_tags_6 },
    { 0x9ac3c579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf343c57d, SUC_lexicon_tags_26 },
    { 0xeaa3c57e, SUC_lexicon_tags_25 },
    { 0x1003c57e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1403c581, SUC_lexicon_tags_30 },
    { 0xe8c3c582, SUC_lexicon_tags_45 },
    { 0x9703c583, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c23c589, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8963c58b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1843c592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc3c598, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe3c59b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6643c5a2, SUC_lexicon_tags_224 },
    { 0x8c23c5a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe83c5a5, SUC_lexicon_tags_33 },
    { 0x4e03c5a6, SUC_lexicon_tags_15 },
    { 0x9643c5a6, SUC_lexicon_tags_9 },
    { 0x8da3c5a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb9a3c5aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9143c5ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2323c5b0, SUC_lexicon_tags_11 },
    { 0xe683c5b1, SUC_lexicon_tags_17 },
    { 0x5d63c5b1, SUC_lexicon_tags_29 },
    { 0x2743c5b3, SUC_lexicon_tags_47 },
    { 0x1b43c5b4, SUC_lexicon_tags_23 },
    { 0x59e3c5b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf823c5b7, SUC_lexicon_tags_2 },
    { 0xfe23c5b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27a3c5ba, SUC_lexicon_tags_1 },
    { 0x6743c5ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c3c5bd, SUC_lexicon_tags_25 },
    { 0x3363c5be, SUC_lexicon_tags_38 },
    { 0xb343c5be, SUC_lexicon_tags_30 },
    { 0xcee3c5c0, SUC_lexicon_tags_25 },
    { 0x1603c5c1, SUC_lexicon_tags_11 },
    { 0x8a63c5c2, SUC_lexicon_tags_17 },
    { 0x6803c5c3, SUC_lexicon_tags_18 },
    { 0x48e3c5c3, SUC_lexicon_tags_25 },
    { 0xb683c5c4, SUC_lexicon_tags_31 },
    { 0xdb83c5bd, SUC_lexicon_tags_11 },
    { 0xb023c5c0, SUC_lexicon_tags_69 },
    { 0xe7c3c5c8, SUC_lexicon_tags_80 },
    { 0xbea3c5c9, SUC_lexicon_tags_7 },
    { 0x2fc3c5ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd203c5cd, SUC_lexicon_tags_37 },
    { 0x1ec3c5cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e3c5d1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe883c5d8, SUC_lexicon_tags_35 },
    { 0x31e3c5d9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a3c5dd, SUC_lexicon_tags_16 },
    { 0xaca3c5dd, SUC_lexicon_tags_19 },
    { 0x9383c5df, SUC_lexicon_tags_27 },
    { 0x2843c5e0, SUC_lexicon_tags_23 },
    { 0xa7a3c5e0, SUC_lexicon_tags_12 },
    { 0x9ea3c5e0, SUC_lexicon_tags_37 },
    { 0xf603c5e3, SUC_lexicon_tags_11 },
    { 0xac23c5e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503c5ea, SUC_lexicon_tags_14 },
    { 0x3243c5eb, SUC_lexicon_tags_11 },
    { 0xad63c5ec, SUC_lexicon_tags_6 },
    { 0xd463c5ed, SUC_lexicon_tags_45 },
    { 0x4463c5ec, SUC_lexicon_tags_33 },
    { 0x33a3c5ef, SUC_lexicon_tags_38 },
    { 0x2303c5f0, SUC_lexicon_tags_15 },
    { 0x8a83c5f1, SUC_lexicon_tags_15 },
    { 0xf403c5f0, SUC_lexicon_tags_77 },
    { 0x2983c5f3, SUC_lexicon_tags_25 },
    { 0xbfa3c5f4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6963c5ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43c603, SUC_lexicon_tags_17 },
    { 0x8723c604, SUC_lexicon_tags_63 },
    { 0x9483c605, SUC_lexicon_tags_6 },
    { 0xc3c3c605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c3c60a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5463c60e, SUC_lexicon_tags_7 },
    { 0x7f83c60e, SUC_lexicon_tags_26 },
    { 0xe203c60e, SUC_lexicon_tags_65 },
    { 0x8ae3c611, SUC_lexicon_tags_38 },
    { 0xd3a3c612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603c616, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec3c61a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc803c61e, SUC_lexicon_tags_33 },
    { 0xb703c61f, SUC_lexicon_tags_7 },
    { 0x96a3c620, SUC_lexicon_tags_49 },
    { 0xa183c620, SUC_lexicon_tags_11 },
    { 0xb763c622, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf723c624, SUC_lexicon_tags_23 },
    { 0xb343c625, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9583c62a, SUC_lexicon_tags_27 },
    { 0x9583c62a, SUC_lexicon_tags_38 },
    { 0xbee3c62c, SUC_lexicon_tags_11 },
    { 0xf9c3c62c, SUC_lexicon_tags_38 },
    { 0x3ec3c62c, SUC_lexicon_tags_12 },
    { 0x1fe3c62f, SUC_lexicon_tags_1 },
    { 0x19a3c630, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3723c633, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18e3c635, SUC_lexicon_tags_27 },
    { 0x20a3c636, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7163c63a, SUC_lexicon_tags_38 },
    { 0xa443c63b, SUC_lexicon_tags_14 },
    { 0x8ba3c63b, SUC_lexicon_tags_15 },
    { 0x1843c63d, SUC_lexicon_tags_11 },
    { 0xcb83c63b, SUC_lexicon_tags_27 },
    { 0x83c63b, SUC_lexicon_tags_56 },
    { 0x5743c63c, SUC_lexicon_tags_84 },
    { 0xf143c63d, SUC_lexicon_tags_33 },
    { 0x2b83c642, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4263c644, SUC_lexicon_tags_7 },
    { 0x8f43c644, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4983c648, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83c64b, SUC_lexicon_tags_6 },
    { 0x76c3c64b, SUC_lexicon_tags_15 },
    { 0x67e3c64c, SUC_lexicon_tags_62 },
    { 0xa843c64e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7663c653, SUC_lexicon_tags_13 },
    { 0x4a63c653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1243c656, SUC_lexicon_tags_56 },
    { 0x50e3c657, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9903c659, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5043c65b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee23c65d, SUC_lexicon_tags_13 },
    { 0xa223c65e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e3c662, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4003c664, SUC_lexicon_tags_43 },
    { 0xaae3c664, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9523c66b, SUC_lexicon_tags_21 },
    { 0x4a3c66b, SUC_lexicon_tags_57 },
    { 0x6e43c66d, SUC_lexicon_tags_27 },
    { 0xdc83c66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd83c671, SUC_lexicon_tags_65 },
    { 0xf383c672, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xaaa3c674, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a3c677, SUC_lexicon_tags_45 },
    { 0x9123c678, SUC_lexicon_tags_23 },
    { 0x1de3c678, SUC_lexicon_tags_1 },
    { 0xf1a3c677, SUC_lexicon_tags_14 },
    { 0x7f63c67b, SUC_lexicon_tags_37 },
    { 0xb4c3c67c, SUC_lexicon_tags_7 },
    { 0xc943c677, SUC_lexicon_tags_26 },
    { 0x1043c67e, SUC_lexicon_tags_15 },
    { 0x8803c67d, SUC_lexicon_tags_49 },
    { 0xd663c680, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c3c686, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe403c68b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a3c691, SUC_lexicon_tags_3 },
    { 0xd443c692, SUC_lexicon_tags_32 },
    { 0x5f03c693, SUC_lexicon_tags_7 },
    { 0xfe03c694, SUC_lexicon_tags_7 },
    { 0x9ca3c693, SUC_lexicon_tags_66 },
    { 0x97e3c696, SUC_lexicon_tags_33 },
    { 0xcf03c697, SUC_lexicon_tags_40 },
    { 0xcc03c694, SUC_lexicon_tags_25 },
    { 0x6c63c693, SUC_lexicon_tags_7 },
    { 0x6723c69a, SUC_lexicon_tags_30 },
    { 0x2b23c69b, SUC_lexicon_tags_82 },
    { 0xeda3c69c, SUC_lexicon_tags_29 },
    { 0xd523c697, SUC_lexicon_tags_98 },
    { 0x9a43c69e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad63c6a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1fc3c6a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9503c6ab, SUC_lexicon_tags_1 },
    { 0x4263c6ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee3c6af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f03c6b2, SUC_lexicon_tags_27 },
    { 0x5343c6b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4dc3c6b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2803c6b7, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e3c6bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3083c6be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3c6c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d23c6ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd43c6ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d43c6d2, SUC_lexicon_tags_1 },
    { 0xdb43c6d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1923c6da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2703c6dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3c6df, SUC_lexicon_tags_37 },
    { 0x55a3c6e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5503c6e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e03c6e5, SUC_lexicon_tags_2 },
    { 0xf9a3c6e6, SUC_lexicon_tags_13 },
    { 0x7f23c6e7, SUC_lexicon_tags_19 },
    { 0x6423c6e8, SUC_lexicon_tags_2 },
    { 0x5e83c6e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d43c6f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfe3c6f2, SUC_lexicon_tags_2 },
    { 0x22c3c6f3, SUC_lexicon_tags_13 },
    { 0xbc03c6f3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1283c6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a3c6fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb63c6fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9343c705, SUC_lexicon_tags_17 },
    { 0xb963c705, SUC_lexicon_tags_23 },
    { 0x5563c707, SUC_lexicon_tags_19 },
    { 0x5863c708, SUC_lexicon_tags_2 },
    { 0xbe63c709, SUC_lexicon_tags_82 },
    { 0xab43c709, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c3c710, SUC_lexicon_tags_7 },
    { 0x4d43c710, SUC_lexicon_tags_6 },
    { 0x29c3c710, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5643c715, SUC_lexicon_tags_2 },
    { 0xf183c716, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b63c719, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9763c71c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c3c71f, SUC_lexicon_tags_28 },
    { 0xcb63c71f, SUC_lexicon_tags_21 },
    { 0x3e83c721, SUC_lexicon_tags_25 },
    { 0xed23c721, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61e3c724, SUC_lexicon_tags_7 },
    { 0xe5e3c724, SUC_lexicon_tags_2 },
    { 0x51e3c726, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x54c3c728, SUC_lexicon_tags_0 },
    { 0x8dc3c728, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa3c72d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb443c730, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf0c3c732, SUC_lexicon_tags_32 },
    { 0x7043c733, SUC_lexicon_tags_5 },
    { 0x39c3c732, SUC_lexicon_tags_61 },
    { 0x11e3c735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9943c737, SUC_lexicon_tags_23 },
    { 0xb9e3c738, SUC_lexicon_tags_39 },
    { 0x56c3c737, SUC_lexicon_tags_63 },
    { 0xae23c73a, SUC_lexicon_tags_25 },
    { 0x163c73a, SUC_lexicon_tags_63 },
    { 0x8a63c73c, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x9e03c73e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7ae3c740, SUC_lexicon_tags_37 },
    { 0x3a3c741, SUC_lexicon_tags_43 },
    { 0x85e3c742, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf023c745, SUC_lexicon_tags_37 },
    { 0xbc43c746, SUC_lexicon_tags_30 },
    { 0xdac3c747, SUC_lexicon_tags_57 },
    { 0xa2a3c747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e3c74b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe963c74e, SUC_lexicon_tags_17 },
    { 0x32c3c74f, SUC_lexicon_tags_25 },
    { 0xb723c750, SUC_lexicon_tags_146 },
    { 0x1503c750, SUC_lexicon_tags_49 },
    { 0x68e3c750, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c3c757, SUC_lexicon_tags_7 },
    { 0xc483c758, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe2a3c75a, SUC_lexicon_tags_37 },
    { 0x1343c75b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1003c75f, SUC_lexicon_tags_1 },
    { 0xa003c760, SUC_lexicon_tags_37 },
    { 0xcfa3c761, SUC_lexicon_tags_17 },
    { 0x923c760, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a83c765, SUC_lexicon_tags_15 },
    { 0x5443c766, SUC_lexicon_tags_0 },
    { 0x40a3c767, SUC_lexicon_tags_25 },
    { 0xd583c766, SUC_lexicon_tags_17 },
    { 0x8a23c769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb503c76b, SUC_lexicon_tags_49 },
    { 0x3ea3c76c, SUC_lexicon_tags_60 },
    { 0x4f63c76d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba43c76f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe123c774, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a43c778, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c3c77c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6bc3c77e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd783c780, SUC_lexicon_tags_3 },
    { 0x6f03c780, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4a3c783, SUC_lexicon_tags_207 },
    { 0x3903c784, SUC_lexicon_tags_37 },
    { 0x4003c785, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2ae3c787, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcf83c789, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa843c78b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x57a3c78d, SUC_lexicon_tags_7 },
    { 0x43a3c78e, SUC_lexicon_tags_1 },
    { 0xfcc3c78e, SUC_lexicon_tags_0 },
    { 0x15a3c78f, SUC_lexicon_tags_7 },
    { 0x80a3c78e, SUC_lexicon_tags_17 },
    { 0x4983c78d, SUC_lexicon_tags_37 },
    { 0x10a3c792, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac63c79a, SUC_lexicon_tags_26 },
    { 0x9a83c79b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1563c79e, SUC_lexicon_tags_47 },
    { 0x5183c79f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9203c7a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f63c7a7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a63c7aa, SUC_lexicon_tags_17 },
    { 0x4d83c7ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b63c7ae, SUC_lexicon_tags_15 },
    { 0x9823c7ae, SUC_lexicon_tags_44 },
    { 0xd9a3c7af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e3c7b4, SUC_lexicon_tags_11 },
    { 0xd6a3c7b5, SUC_lexicon_tags_18 },
    { 0x5943c7b6, SUC_lexicon_tags_43 },
    { 0x9183c7b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c3c7bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xffa3c7bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee3c7c2, SUC_lexicon_tags_33 },
    { 0x39e3c7c2, SUC_lexicon_tags_11 },
    { 0xcd83c7c4, SUC_lexicon_tags_27 },
    { 0x2663c7c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb223c7c7, SUC_lexicon_tags_27 },
    { 0xc123c7c7, SUC_lexicon_tags_15 },
    { 0x6843c7c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9da3c7cb, SUC_lexicon_tags_0 },
    { 0x53a3c7cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f23c7ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3c7d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc3c7d7, SUC_lexicon_tags_11 },
    { 0x2963c7d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2723c7da, SUC_lexicon_tags_47 },
    { 0x24c3c7db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e3c7df, SUC_lexicon_tags_62 },
    { 0x3443c7e0, SUC_lexicon_tags_12 },
    { 0xf183c7df, SUC_lexicon_tags_10 },
    { 0x5323c7e2, SUC_lexicon_tags_17 },
    { 0x3263c7e3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e3c7ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3143c7f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf523c7fa, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x9d63c7fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7763c7fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e03c804, SUC_lexicon_tags_17 },
    { 0xf1e3c805, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f63c809, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e43c80f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa83c811, SUC_lexicon_tags_31 },
    { 0xfc03c812, SUC_lexicon_tags_63 },
    { 0x2303c812, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf303c817, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd083c823, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe043c829, SUC_lexicon_tags_11 },
    { 0x4fe3c82a, SUC_lexicon_tags_49 },
    { 0x1363c82a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x363c82e, SUC_lexicon_tags_11 },
    { 0x6ac3c82e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a3c832, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x26c3c834, SUC_lexicon_tags_6 },
    { 0x96e3c835, SUC_lexicon_tags_2 },
    { 0xe143c836, SUC_lexicon_tags_17 },
    { 0xbac3c837, SUC_lexicon_tags_17 },
    { 0x1e43c835, SUC_lexicon_tags_12 },
    { 0x7883c839, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d43c840, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5363c847, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e23c84d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xce43c84f, SUC_lexicon_tags_37 },
    { 0x2fe3c850, SUC_lexicon_tags_9 },
    { 0xae3c851, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3c855, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab03c858, SUC_lexicon_tags_60 },
    { 0x9fc3c859, SUC_lexicon_tags_30 },
    { 0xa723c85a, SUC_lexicon_tags_25 },
    { 0x65c3c85a, SUC_lexicon_tags_14 },
    { 0x4d83c85b, SUC_lexicon_tags_38 },
    { 0x4123c859, SUC_lexicon_tags_52 },
    { 0x8803c85e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6503c863, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803c867, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc103c86d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a3c870, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb543c872, SUC_lexicon_tags_17 },
    { 0x6ea3c873, SUC_lexicon_tags_27 },
    { 0xec63c873, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf03c877, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1643c87f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1903c882, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6263c884, SUC_lexicon_tags_13 },
    { 0x6c03c884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3583c888, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda83c88b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5e3c88d, SUC_lexicon_tags_49 },
    { 0xb803c88d, SUC_lexicon_tags_2 },
    { 0xd0e3c88f, SUC_lexicon_tags_1 },
    { 0xd623c88f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe763c892, SUC_lexicon_tags_37 },
    { 0xc663c892, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e3c899, SUC_lexicon_tags_47 },
    { 0x3e83c89a, SUC_lexicon_tags_10 },
    { 0x77c3c89a, SUC_lexicon_tags_11 },
    { 0x8223c89c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63e3c89e, SUC_lexicon_tags_32 },
    { 0x2e43c89f, SUC_lexicon_tags_769 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03c8a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c3c8a7, SUC_lexicon_tags_17 },
    { 0xac63c8a8, SUC_lexicon_tags_9 },
    { 0xf923c8a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e3c8ad, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c3c8b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc583c8b4, SUC_lexicon_tags_6 },
    { 0x4743c8b5, SUC_lexicon_tags_35 },
    { 0x51a3c8b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5663c8c0, SUC_lexicon_tags_0 },
    { 0x503c8c1, SUC_lexicon_tags_46 },
    { 0xe303c8c2, SUC_lexicon_tags_15 },
    { 0xf323c8c1, SUC_lexicon_tags_75 },
    { 0xfd23c8c4, SUC_lexicon_tags_7 },
    { 0x6ec3c8c5, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x28c3c8c7, SUC_lexicon_tags_117 },
    { 0x42e3c8c8, SUC_lexicon_tags_26 },
    { 0x86c3c8c9, SUC_lexicon_tags_7 },
    { 0x7223c8c9, SUC_lexicon_tags_8 },
    { 0xaf83c8c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7343c8cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee3c8da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48c3c8dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f83c8e1, SUC_lexicon_tags_12 },
    { 0x6d63c8e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a43c8e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xda03c8e6, SUC_lexicon_tags_11 },
    { 0x89a3c8e7, SUC_lexicon_tags_21 },
    { 0xbe63c8e8, SUC_lexicon_tags_11 },
    { 0x48c3c8e9, SUC_lexicon_tags_27 },
    { 0xe283c8ea, SUC_lexicon_tags_37 },
    { 0x16a3c8e9, SUC_lexicon_tags_47 },
    { 0xed43c8ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcd03c8ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc763c8f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c3c8fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c3c902, SUC_lexicon_tags_15 },
    { 0x86e3c903, SUC_lexicon_tags_15 },
    { 0xf923c903, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623c909, SUC_lexicon_tags_27 },
    { 0x20c3c90a, SUC_lexicon_tags_60 },
    { 0x61c3c90b, SUC_lexicon_tags_2 },
    { 0x1e63c90b, SUC_lexicon_tags_23 },
    { 0x4043c90d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce43c90f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb303c915, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5663c91c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc3c920, SUC_lexicon_tags_19 },
    { 0x1f43c921, SUC_lexicon_tags_37 },
    { 0x7b03c921, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb43c925, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb9e3c927, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a3c92b, SUC_lexicon_tags_25 },
    { 0xad03c92c, SUC_lexicon_tags_65 },
    { 0xabe3c92b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f03c932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4c3c934, SUC_lexicon_tags_6 },
    { 0x5ae3c935, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c3c939, SUC_lexicon_tags_1 },
    { 0xc823c939, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4603c940, SUC_lexicon_tags_84 },
    { 0x2c23c941, SUC_lexicon_tags_43 },
    { 0x2a03c941, SUC_lexicon_tags_11 },
    { 0xe23c941, SUC_lexicon_tags_87 },
    { 0x1043c941, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5b43c946, SUC_lexicon_tags_25 },
    { 0x4583c946, SUC_lexicon_tags_55 },
    { 0xcc43c946, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2283c94a, SUC_lexicon_tags_37 },
    { 0x4b83c94a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x263c94d, SUC_lexicon_tags_37 },
    { 0xe543c94e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe183c956, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2703c958, SUC_lexicon_tags_26 },
    { 0xc5e3c959, SUC_lexicon_tags_29 },
    { 0xce23c95a, SUC_lexicon_tags_37 },
    { 0x2263c95b, SUC_lexicon_tags_88 },
    { 0xc4c3c958, SUC_lexicon_tags_21 },
    { 0xab63c95c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc083c966, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e3c969, SUC_lexicon_tags_7 },
    { 0xe8e3c96a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb543c96e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d43c971, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e3c977, SUC_lexicon_tags_1 },
    { 0x6663c977, SUC_lexicon_tags_65 },
    { 0xe203c979, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde3c984, SUC_lexicon_tags_133 },
    { 0x7d43c985, SUC_lexicon_tags_12 },
    { 0xf643c986, SUC_lexicon_tags_12 },
    { 0xd503c987, SUC_lexicon_tags_2 },
    { 0x8403c986, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8cc3c98a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7de3c98c, SUC_lexicon_tags_17 },
    { 0x4343c98d, SUC_lexicon_tags_12 },
    { 0x8643c98d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc743c994, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c3c999, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc603c99b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b43c9a0, SUC_lexicon_tags_38 },
    { 0xf823c9a1, SUC_lexicon_tags_27 },
    { 0x3743c9a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36a3c9a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c3c9a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c3c9ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4703c9af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf03c9b1, SUC_lexicon_tags_25 },
    { 0xbe3c9b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb323c9bb, SUC_lexicon_tags_42 },
    { 0xece3c9bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a3c9bf, SUC_lexicon_tags_7 },
    { 0x5003c9c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be3c9c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2383c9cb, SUC_lexicon_tags_40 },
    { 0x9ca3c9cc, SUC_lexicon_tags_15 },
    { 0xcfa3c9cd, SUC_lexicon_tags_204 },
    { 0x2103c9cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc023c9d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd843c9d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b83c9d5, SUC_lexicon_tags_19 },
    { 0xb203c9d6, SUC_lexicon_tags_19 },
    { 0x1c03c9d6, SUC_lexicon_tags_55 },
    { 0x3383c9d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6883c9da, SUC_lexicon_tags_47 },
    { 0x5503c9db, SUC_lexicon_tags_60 },
    { 0xd123c9dc, SUC_lexicon_tags_92 },
    { 0xe9e3c9dd, SUC_lexicon_tags_15 },
    { 0x4163c9de, SUC_lexicon_tags_27 },
    { 0xbe23c9dd, SUC_lexicon_tags_25 },
    { 0x8e03c9e0, SUC_lexicon_tags_17 },
    { 0xe943c9e1, SUC_lexicon_tags_23 },
    { 0x3ee3c9e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a63c9e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e3c9ef, SUC_lexicon_tags_31 },
    { 0xdf43c9f0, SUC_lexicon_tags_49 },
    { 0x4863c9f1, SUC_lexicon_tags_14 },
    { 0xd5e3c9f2, SUC_lexicon_tags_7 },
    { 0x1bc3c9f3, SUC_lexicon_tags_37 },
    { 0xa3a3c9f4, SUC_lexicon_tags_37 },
    { 0x7f23c9f2, SUC_lexicon_tags_2 },
    { 0xa0e3c9f6, SUC_lexicon_tags_25 },
    { 0x8303c9f5, SUC_lexicon_tags_14 },
    { 0xfbe3c9f8, SUC_lexicon_tags_61 },
    { 0x6583c9f8, SUC_lexicon_tags_11 },
    { 0x4fa3c9fa, SUC_lexicon_tags_55 },
    { 0xa123c9fa, SUC_lexicon_tags_39 },
    { 0x65e3c9f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf843c9ff, SUC_lexicon_tags_1167 },
    { 0, NULL },
    { 0, NULL },
    { 0xae83ca02, SUC_lexicon_tags_43 },
    { 0x3103ca02, SUC_lexicon_tags_60 },
    { 0xe543ca03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4743ca09, SUC_lexicon_tags_95 },
    { 0x4a3ca0a, SUC_lexicon_tags_63 },
    { 0x72e3ca0a, SUC_lexicon_tags_17 },
    { 0x3bc3ca0c, SUC_lexicon_tags_28 },
    { 0xe303ca0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa243ca0f, SUC_lexicon_tags_37 },
    { 0xabe3ca0f, SUC_lexicon_tags_45 },
    { 0x5a3ca11, SUC_lexicon_tags_6 },
    { 0xcc43ca12, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c03ca15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea3ca19, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x743ca1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe763ca22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb663ca26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeba3ca28, SUC_lexicon_tags_37 },
    { 0x2c3ca29, SUC_lexicon_tags_49 },
    { 0xe223ca2a, SUC_lexicon_tags_39 },
    { 0xca43ca2a, SUC_lexicon_tags_151 },
    { 0xe903ca2c, SUC_lexicon_tags_19 },
    { 0x2b83ca2a, SUC_lexicon_tags_8 },
    { 0x2983ca2e, SUC_lexicon_tags_25 },
    { 0x1c23ca2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a23ca35, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc9c3ca37, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe983ca3b, SUC_lexicon_tags_0 },
    { 0x42c3ca3b, SUC_lexicon_tags_45 },
    { 0xbbc3ca3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3ca41, SUC_lexicon_tags_17 },
    { 0x8603ca41, SUC_lexicon_tags_7 },
    { 0x5ba3ca41, SUC_lexicon_tags_14 },
    { 0xd8e3ca44, SUC_lexicon_tags_33 },
    { 0xc4c3ca41, SUC_lexicon_tags_48 },
    { 0xdb23ca46, SUC_lexicon_tags_29 },
    { 0x2703ca47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c3ca4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da3ca56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fa3ca58, SUC_lexicon_tags_15 },
    { 0x2443ca59, SUC_lexicon_tags_35 },
    { 0xeb03ca5a, SUC_lexicon_tags_12 },
    { 0x2743ca5b, SUC_lexicon_tags_25 },
    { 0xee63ca5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa843ca5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a3ca65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a3ca68, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x19a3ca6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf603ca6d, SUC_lexicon_tags_49 },
    { 0xf4c3ca6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c3ca71, SUC_lexicon_tags_2 },
    { 0xb4e3ca72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb03ca74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce23ca79, SUC_lexicon_tags_28 },
    { 0xd7c3ca7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8223ca7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e3ca81, SUC_lexicon_tags_12 },
    { 0x3ec3ca82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe3ca86, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa3ca89, SUC_lexicon_tags_7 },
    { 0xde63ca89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54e3ca8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6e3ca8e, SUC_lexicon_tags_2 },
    { 0xfb63ca8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f83ca93, SUC_lexicon_tags_0 },
    { 0x8023ca93, SUC_lexicon_tags_27 },
    { 0x9fc3ca94, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ca9c, SUC_lexicon_tags_49 },
    { 0x7103ca9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc203ca9f, SUC_lexicon_tags_11 },
    { 0x8be3ca9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf563caa3, SUC_lexicon_tags_63 },
    { 0xcd23caa4, SUC_lexicon_tags_1447 },
    { 0, NULL },
    { 0, NULL },
    { 0x2583caa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a3caab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca3cab2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9003cab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc003cab7, SUC_lexicon_tags_13 },
    { 0xd5e3cab7, SUC_lexicon_tags_0 },
    { 0x67a3cab7, SUC_lexicon_tags_14 },
    { 0x9163caba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb463cabc, SUC_lexicon_tags_15 },
    { 0x4e83cabd, SUC_lexicon_tags_47 },
    { 0x56a3cabd, SUC_lexicon_tags_12 },
    { 0xfea3cabf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa223cac1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cac4, SUC_lexicon_tags_77 },
    { 0x5a3cac5, SUC_lexicon_tags_44 },
    { 0xc2c3cac4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb3c3cac8, SUC_lexicon_tags_12 },
    { 0x73a3cac9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2723cacb, SUC_lexicon_tags_10 },
    { 0x7d83cacc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd03cace, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63c3cad0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xee63cad3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb763cad6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6783cad9, SUC_lexicon_tags_11 },
    { 0x7a23cada, SUC_lexicon_tags_17 },
    { 0xd563cad9, SUC_lexicon_tags_0 },
    { 0xbce3cadb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf303cae1, SUC_lexicon_tags_16 },
    { 0xc283cae2, SUC_lexicon_tags_11 },
    { 0xa6e3cae2, SUC_lexicon_tags_4 },
    { 0xb003cae2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7143cae6, SUC_lexicon_tags_55 },
    { 0x983cae6, SUC_lexicon_tags_0 },
    { 0x8f83cae8, SUC_lexicon_tags_28 },
    { 0xc23cae9, SUC_lexicon_tags_37 },
    { 0xf683cae9, SUC_lexicon_tags_63 },
    { 0xef83caeb, SUC_lexicon_tags_27 },
    { 0x2443caec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd863caef, SUC_lexicon_tags_60 },
    { 0x9743caf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe763caf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec3caf7, SUC_lexicon_tags_11 },
    { 0x3e23caf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c3cafb, SUC_lexicon_tags_37 },
    { 0x6963cafc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7903cafe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb243cb00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a3cb03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd103cb05, SUC_lexicon_tags_37 },
    { 0xee43cb05, SUC_lexicon_tags_54 },
    { 0x4d23cb05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e43cb09, SUC_lexicon_tags_11 },
    { 0x3d43cb0a, SUC_lexicon_tags_44 },
    { 0x2a3cb0b, SUC_lexicon_tags_7 },
    { 0x8b43cb0c, SUC_lexicon_tags_37 },
    { 0x6fc3cb0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe303cb10, SUC_lexicon_tags_2 },
    { 0xc003cb11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3743cb13, SUC_lexicon_tags_23 },
    { 0x69e3cb14, SUC_lexicon_tags_19 },
    { 0x9d63cb15, SUC_lexicon_tags_35 },
    { 0x9643cb15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4663cb18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0e3cb1a, SUC_lexicon_tags_7 },
    { 0x7e23cb1b, SUC_lexicon_tags_35 },
    { 0x1e83cb1a, SUC_lexicon_tags_61 },
    { 0x943cb1d, SUC_lexicon_tags_11 },
    { 0x403cb1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd23cb24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea03cb2e, SUC_lexicon_tags_9 },
    { 0x1f23cb2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a3cb35, SUC_lexicon_tags_5 },
    { 0xbda3cb36, SUC_lexicon_tags_37 },
    { 0xda23cb36, SUC_lexicon_tags_19 },
    { 0xaaa3cb38, SUC_lexicon_tags_49 },
    { 0xec83cb39, SUC_lexicon_tags_39 },
    { 0xbd63cb38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d03cb46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe043cb48, SUC_lexicon_tags_32 },
    { 0x7683cb49, SUC_lexicon_tags_4 },
    { 0xe9a3cb49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae3cb4e, SUC_lexicon_tags_2 },
    { 0xd123cb4e, SUC_lexicon_tags_2 },
    { 0xfe03cb50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6123cb53, SUC_lexicon_tags_104 },
    { 0x4d23cb54, SUC_lexicon_tags_7 },
    { 0x1ca3cb54, SUC_lexicon_tags_37 },
    { 0xd223cb55, SUC_lexicon_tags_17 },
    { 0xe63cb56, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde43cb5e, SUC_lexicon_tags_31 },
    { 0xd343cb5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2aa3cb61, SUC_lexicon_tags_27 },
    { 0x8a63cb62, SUC_lexicon_tags_38 },
    { 0xaa23cb61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a3cb68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa3cb72, SUC_lexicon_tags_12 },
    { 0xaaa3cb73, SUC_lexicon_tags_29 },
    { 0x9203cb74, SUC_lexicon_tags_27 },
    { 0x5963cb74, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa383cb7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc3cb7d, SUC_lexicon_tags_87 },
    { 0xae3cb7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e3cb83, SUC_lexicon_tags_37 },
    { 0x7ee3cb84, SUC_lexicon_tags_27 },
    { 0xc4e3cb84, SUC_lexicon_tags_77 },
    { 0xfb03cb84, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a3cb8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf003cb8c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb723cb94, SUC_lexicon_tags_17 },
    { 0x9983cb94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4943cb9b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee3cba0, SUC_lexicon_tags_17 },
    { 0xb383cba1, SUC_lexicon_tags_9 },
    { 0x27a3cba0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9343cba4, SUC_lexicon_tags_34 },
    { 0x9d83cba5, SUC_lexicon_tags_11 },
    { 0x9a63cba5, SUC_lexicon_tags_56 },
    { 0xf583cba7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee3cbaa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd6a3cbac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe123cbae, SUC_lexicon_tags_37 },
    { 0xf223cbaf, SUC_lexicon_tags_48 },
    { 0x14e3cbae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c3cbb9, SUC_lexicon_tags_17 },
    { 0xd5a3cbba, SUC_lexicon_tags_2 },
    { 0xfdc3cbbb, SUC_lexicon_tags_7 },
    { 0xac83cbbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e3cbc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc3cbc4, SUC_lexicon_tags_80 },
    { 0x61e3cbc5, SUC_lexicon_tags_38 },
    { 0xb483cbc6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4663cbcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6523cbce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c03cbd8, SUC_lexicon_tags_15 },
    { 0xedc3cbd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe723cbdb, SUC_lexicon_tags_32 },
    { 0x3b23cbdc, SUC_lexicon_tags_66 },
    { 0xb543cbdd, SUC_lexicon_tags_9 },
    { 0xeec3cbde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6803cbe3, SUC_lexicon_tags_12 },
    { 0xd0c3cbe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f23cbe7, SUC_lexicon_tags_17 },
    { 0x33c3cbe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6263cbeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38a3cbed, SUC_lexicon_tags_30 },
    { 0x6b23cbee, SUC_lexicon_tags_4 },
    { 0x17c3cbed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f03cbf2, SUC_lexicon_tags_1 },
    { 0x8763cbf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6283cbf5, SUC_lexicon_tags_33 },
    { 0x9943cbf5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1963cbf8, SUC_lexicon_tags_37 },
    { 0xa263cbf9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb863cbfb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb03cbff, SUC_lexicon_tags_17 },
    { 0x5ea3cc00, SUC_lexicon_tags_49 },
    { 0x443cc01, SUC_lexicon_tags_25 },
    { 0x3663cc01, SUC_lexicon_tags_38 },
    { 0x3883cc02, SUC_lexicon_tags_7 },
    { 0x3e23cc03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe063cc06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4743cc09, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3fe3cc0b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xf403cc0d, SUC_lexicon_tags_0 },
    { 0xed63cc0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x99c3cc10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3843cc13, SUC_lexicon_tags_1 },
    { 0x9983cc14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c03cc16, SUC_lexicon_tags_0 },
    { 0xdaa3cc17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5043cc22, SUC_lexicon_tags_2 },
    { 0xad83cc22, SUC_lexicon_tags_33 },
    { 0x9a83cc24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103cc28, SUC_lexicon_tags_27 },
    { 0xd0c3cc29, SUC_lexicon_tags_45 },
    { 0x5963cc29, SUC_lexicon_tags_2 },
    { 0xfec3cc2b, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9723cc2f, SUC_lexicon_tags_28 },
    { 0x82c3cc30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7863cc32, SUC_lexicon_tags_12 },
    { 0x6f03cc32, SUC_lexicon_tags_25 },
    { 0xd483cc33, SUC_lexicon_tags_14 },
    { 0x9103cc32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3cc3c, SUC_lexicon_tags_32 },
    { 0x6e63cc3c, SUC_lexicon_tags_83 },
    { 0x5023cc3e, SUC_lexicon_tags_11 },
    { 0x9603cc3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6243cc48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd43cc4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc63cc51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa723cc54, SUC_lexicon_tags_27 },
    { 0xf243cc55, SUC_lexicon_tags_15 },
    { 0xd403cc56, SUC_lexicon_tags_43 },
    { 0xd483cc57, SUC_lexicon_tags_29 },
    { 0x3323cc55, SUC_lexicon_tags_57 },
    { 0x57a3cc59, SUC_lexicon_tags_26 },
    { 0x5b23cc59, SUC_lexicon_tags_49 },
    { 0xd163cc56, SUC_lexicon_tags_56 },
    { 0xca63cc5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7983cc5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae3cc61, SUC_lexicon_tags_0 },
    { 0x44e3cc62, SUC_lexicon_tags_55 },
    { 0x9943cc62, SUC_lexicon_tags_12 },
    { 0xa643cc64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6443cc66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcc83cc68, SUC_lexicon_tags_25 },
    { 0x7883cc69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a3cc6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc63cc71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc3cc76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda3cc79, SUC_lexicon_tags_11 },
    { 0x39a3cc7a, SUC_lexicon_tags_25 },
    { 0x4383cc7b, SUC_lexicon_tags_15 },
    { 0x64e3cc7c, SUC_lexicon_tags_60 },
    { 0x99a3cc7c, SUC_lexicon_tags_39 },
    { 0x6be3cc7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb83cc80, SUC_lexicon_tags_27 },
    { 0xfbc3cc80, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1023cc87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b03cc8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1803cc92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd363cc95, SUC_lexicon_tags_1 },
    { 0x3443cc96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1283cc99, SUC_lexicon_tags_19 },
    { 0xaaa3cc9a, SUC_lexicon_tags_25 },
    { 0xff63cc99, SUC_lexicon_tags_26 },
    { 0xa4a3cc9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8923cc9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc3cca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6683cca5, SUC_lexicon_tags_17 },
    { 0x6143cca6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a3ccae, SUC_lexicon_tags_17 },
    { 0xb2a3ccaf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd263ccb7, SUC_lexicon_tags_43 },
    { 0xdf03ccb8, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a3ccbb, SUC_lexicon_tags_37 },
    { 0x95a3ccbc, SUC_lexicon_tags_17 },
    { 0x46c3ccbd, SUC_lexicon_tags_66 },
    { 0xaca3ccbb, SUC_lexicon_tags_25 },
    { 0xfb83ccbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x743ccc1, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2483ccc5, SUC_lexicon_tags_12 },
    { 0x4183ccc6, SUC_lexicon_tags_17 },
    { 0xe9e3ccc7, SUC_lexicon_tags_25 },
    { 0x1ec3ccc8, SUC_lexicon_tags_3 },
    { 0x2b23ccc7, SUC_lexicon_tags_27 },
    { 0x923ccc6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c3ccd8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xc023ccdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c3cce7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf103cce9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee3ccf2, SUC_lexicon_tags_31 },
    { 0x9303ccf2, SUC_lexicon_tags_85 },
    { 0x7f63ccf3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe3a3ccf6, SUC_lexicon_tags_28 },
    { 0x2083ccf7, SUC_lexicon_tags_17 },
    { 0x3aa3ccf8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b43ccfc, SUC_lexicon_tags_5 },
    { 0xaa63ccfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c3cd02, SUC_lexicon_tags_12 },
    { 0x8703cd02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e43cd07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d83cd09, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1ee3cd0b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a63cd14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2623cd16, SUC_lexicon_tags_32 },
    { 0xb623cd17, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9d63cd19, SUC_lexicon_tags_15 },
    { 0xcbe3cd19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed43cd1d, SUC_lexicon_tags_1 },
    { 0xf583cd1e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2263cd22, SUC_lexicon_tags_60 },
    { 0xad43cd23, SUC_lexicon_tags_7 },
    { 0x2523cd23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6a3cd26, SUC_lexicon_tags_11 },
    { 0x96a3cd27, SUC_lexicon_tags_17 },
    { 0xd543cd28, SUC_lexicon_tags_87 },
    { 0x73e3cd28, SUC_lexicon_tags_15 },
    { 0x58c3cd2a, SUC_lexicon_tags_11 },
    { 0x4263cd26, SUC_lexicon_tags_171 },
    { 0x2323cd2c, SUC_lexicon_tags_9 },
    { 0x73a3cd2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x75e3cd2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3563cd35, SUC_lexicon_tags_25 },
    { 0xf303cd35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc763cd3a, SUC_lexicon_tags_21 },
    { 0xace3cd3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3103cd3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c3cd43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a3cd47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b63cd49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc4a3cd4b, SUC_lexicon_tags_491 },
    { 0x3de3cd4c, SUC_lexicon_tags_197 },
    { 0xb943cd4c, SUC_lexicon_tags_1 },
    { 0xd663cd4e, SUC_lexicon_tags_2 },
    { 0xf303cd4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed83cd51, SUC_lexicon_tags_33 },
    { 0xf0e3cd51, SUC_lexicon_tags_38 },
    { 0x6d03cd53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9043cd55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa983cd59, SUC_lexicon_tags_77 },
    { 0x7fe3cd5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe3cd67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff83cd6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee3cd6d, SUC_lexicon_tags_2 },
    { 0x40c3cd6e, SUC_lexicon_tags_43 },
    { 0x4543cd6e, SUC_lexicon_tags_32 },
    { 0x5223cd70, SUC_lexicon_tags_37 },
    { 0xc2e3cd71, SUC_lexicon_tags_2 },
    { 0x38c3cd70, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e3cd75, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2783cd78, SUC_lexicon_tags_28 },
    { 0x63e3cd78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e43cd7f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f23cd82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c3cd87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80a3cd89, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe43cd8e, SUC_lexicon_tags_7 },
    { 0xff83cd8f, SUC_lexicon_tags_16 },
    { 0xb7a3cd8f, SUC_lexicon_tags_47 },
    { 0x1743cd8f, SUC_lexicon_tags_2 },
    { 0x3cd92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe2c3cd94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3543cd97, SUC_lexicon_tags_9 },
    { 0xf583cd98, SUC_lexicon_tags_25 },
    { 0x5aa3cd99, SUC_lexicon_tags_12 },
    { 0xf043cd99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf383cd9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be3cd9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1343cda2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8403cda4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2143cda7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36c3cda9, SUC_lexicon_tags_37 },
    { 0xb0e3cdaa, SUC_lexicon_tags_14 },
    { 0x7f83cdab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4703cdb0, SUC_lexicon_tags_40 },
    { 0xf43cdb1, SUC_lexicon_tags_7 },
    { 0x6483cdb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba63cdb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa823cdb9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a3cdbd, SUC_lexicon_tags_16 },
    { 0x47e3cdbe, SUC_lexicon_tags_68 },
    { 0x203cdbf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe483cdc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb663cdc6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3823cdc8, SUC_lexicon_tags_85 },
    { 0xc183cdc9, SUC_lexicon_tags_77 },
    { 0xf543cdc8, SUC_lexicon_tags_11 },
    { 0x5fa3cdcb, SUC_lexicon_tags_37 },
    { 0xa283cdcb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e3cdcf, SUC_lexicon_tags_13 },
    { 0xdde3cdcf, SUC_lexicon_tags_17 },
    { 0xb023cdd0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63cddd, SUC_lexicon_tags_60 },
    { 0x71c3cdde, SUC_lexicon_tags_11 },
    { 0x5fe3cdde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fe3cde1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x843cde3, SUC_lexicon_tags_47 },
    { 0x10e3cde4, SUC_lexicon_tags_26 },
    { 0xd083cde5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf683cde7, SUC_lexicon_tags_17 },
    { 0x7f83cde7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c3cdef, SUC_lexicon_tags_37 },
    { 0xfd63cdf0, SUC_lexicon_tags_170 },
    { 0xafa3cdf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83cdf4, SUC_lexicon_tags_17 },
    { 0x91c3cdf5, SUC_lexicon_tags_21 },
    { 0xb63cdf6, SUC_lexicon_tags_37 },
    { 0xf423cdf6, SUC_lexicon_tags_37 },
    { 0xbfc3cdf7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca3ce00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc563ce05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf443ce07, SUC_lexicon_tags_37 },
    { 0x4943ce07, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9523ce12, SUC_lexicon_tags_15 },
    { 0xb923ce12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9203ce15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa23ce17, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6103ce1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc103ce1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a3ce23, SUC_lexicon_tags_16 },
    { 0xa0c3ce24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a3ce28, SUC_lexicon_tags_46 },
    { 0x14a3ce28, SUC_lexicon_tags_37 },
    { 0xd0c3ce2a, SUC_lexicon_tags_12 },
    { 0x3563ce29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c23ce2f, SUC_lexicon_tags_14 },
    { 0x7463ce30, SUC_lexicon_tags_32 },
    { 0x4063ce31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b43ce36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2223ce39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4603ce3f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5e63ce41, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5503ce43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41c3ce45, SUC_lexicon_tags_0 },
    { 0xd683ce46, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d43ce4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc3ce4d, SUC_lexicon_tags_1 },
    { 0xa463ce4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc3ce55, SUC_lexicon_tags_27 },
    { 0x14e3ce55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c23ce5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4363ce63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ae3ce65, SUC_lexicon_tags_1 },
    { 0xcc03ce66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23ce69, SUC_lexicon_tags_11 },
    { 0x45e3ce6a, SUC_lexicon_tags_17 },
    { 0x7723ce6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c83ce6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6643ce6f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca3ce7b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa423ce7d, SUC_lexicon_tags_37 },
    { 0x16c3ce7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30c3ce80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8663ce88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c3ce8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a3ce93, SUC_lexicon_tags_12 },
    { 0xafe3ce94, SUC_lexicon_tags_11 },
    { 0x7cc3ce95, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe23ce98, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf343ce9c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a83ce9f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a3cea2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c3cea4, SUC_lexicon_tags_897 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da3cea7, SUC_lexicon_tags_17 },
    { 0xdd03cea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf723ceb0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6e3ceb2, SUC_lexicon_tags_47 },
    { 0xcb83ceb3, SUC_lexicon_tags_26 },
    { 0xb3e3ceb4, SUC_lexicon_tags_1 },
    { 0xcd23ceb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ceb8, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f83cebb, SUC_lexicon_tags_37 },
    { 0xe0a3cebc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad63cec1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa03cec4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1363cec7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xecc3cec9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe0a3cecb, SUC_lexicon_tags_31 },
    { 0xada3cecc, SUC_lexicon_tags_37 },
    { 0x1d23cecd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa3ced0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce3cedb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae3cede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81e3cee0, SUC_lexicon_tags_25 },
    { 0x93c3cee1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a3ceec, SUC_lexicon_tags_0 },
    { 0x7983ceec, SUC_lexicon_tags_92 },
    { 0xcce3ceee, SUC_lexicon_tags_43 },
    { 0x41a3ceed, SUC_lexicon_tags_15 },
    { 0xcae3ceef, SUC_lexicon_tags_0 },
    { 0xcf03cef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8203cef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8083cef7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c3cefa, SUC_lexicon_tags_12 },
    { 0xd543cefb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca3cf02, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x21a3cf04, SUC_lexicon_tags_11 },
    { 0x3b23cf05, SUC_lexicon_tags_62 },
    { 0x89e3cf05, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa843cf08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7dc3cf0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b83cf0c, SUC_lexicon_tags_17 },
    { 0x36e3cf0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee3cf12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8403cf14, SUC_lexicon_tags_77 },
    { 0x8b03cf15, SUC_lexicon_tags_1707 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c3cf18, SUC_lexicon_tags_32 },
    { 0x4403cf19, SUC_lexicon_tags_21 },
    { 0x2b23cf1a, SUC_lexicon_tags_32 },
    { 0x9a23cf1b, SUC_lexicon_tags_33 },
    { 0x3643cf1c, SUC_lexicon_tags_1 },
    { 0x4f03cf19, SUC_lexicon_tags_7 },
    { 0xc363cf1a, SUC_lexicon_tags_7 },
    { 0xc343cf1f, SUC_lexicon_tags_63 },
    { 0xa703cf1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafe3cf22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b03cf24, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a3cf28, SUC_lexicon_tags_82 },
    { 0xbf63cf28, SUC_lexicon_tags_29 },
    { 0x95c3cf28, SUC_lexicon_tags_11 },
    { 0xf223cf2a, SUC_lexicon_tags_2 },
    { 0xf83cf29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa643cf2f, SUC_lexicon_tags_47 },
    { 0x3ae3cf2f, SUC_lexicon_tags_7 },
    { 0x9f23cf31, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc503cf34, SUC_lexicon_tags_11 },
    { 0xb383cf35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1603cf39, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e3cf3d, SUC_lexicon_tags_26 },
    { 0x9f23cf3e, SUC_lexicon_tags_1 },
    { 0x18e3cf3e, SUC_lexicon_tags_27 },
    { 0xb03cf40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33a3cf42, SUC_lexicon_tags_63 },
    { 0x55a3cf43, SUC_lexicon_tags_14 },
    { 0x5e43cf44, SUC_lexicon_tags_15 },
    { 0xda63cf44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7c3cf47, SUC_lexicon_tags_11 },
    { 0x5223cf48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9123cf4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c3cf50, SUC_lexicon_tags_2 },
    { 0xdc23cf51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb163cf54, SUC_lexicon_tags_8 },
    { 0x6e63cf54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1643cf58, SUC_lexicon_tags_17 },
    { 0x7fc3cf59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa03cf5b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda3cf5e, SUC_lexicon_tags_15 },
    { 0xa323cf5f, SUC_lexicon_tags_11 },
    { 0x3da3cf60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3cf63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1523cf65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69e3cf67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada3cf6b, SUC_lexicon_tags_44 },
    { 0xf8e3cf6b, SUC_lexicon_tags_37 },
    { 0xe303cf6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c3cf71, SUC_lexicon_tags_29 },
    { 0x843cf72, SUC_lexicon_tags_25 },
    { 0x7003cf73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b03cf7b, SUC_lexicon_tags_15 },
    { 0x26a3cf7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a3cf7f, SUC_lexicon_tags_49 },
    { 0x2f23cf7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea3cf83, SUC_lexicon_tags_38 },
    { 0x3e83cf84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb03cf89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63cf8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a3cf8f, SUC_lexicon_tags_63 },
    { 0x17a3cf8f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5d43cf92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7583cf97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd623cf99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2dc3cf9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x88a3cf9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c3cfa4, SUC_lexicon_tags_45 },
    { 0x2563cfa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae3cfa9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac3cfb1, SUC_lexicon_tags_45 },
    { 0xe983cfb2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfec3cfb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c3cfb7, SUC_lexicon_tags_15 },
    { 0xefe3cfb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea83cfbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5863cfbe, SUC_lexicon_tags_29 },
    { 0x8c83cfbf, SUC_lexicon_tags_25 },
    { 0xefc3cfbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83cfc4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe823cfc8, SUC_lexicon_tags_25 },
    { 0x7da3cfc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a3cfcb, SUC_lexicon_tags_17 },
    { 0xd4c3cfcc, SUC_lexicon_tags_63 },
    { 0xf6c3cfcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3cfd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe43cfdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf703cfdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4023cfe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c3cfe6, SUC_lexicon_tags_5 },
    { 0x3de3cfe6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d43cfeb, SUC_lexicon_tags_77 },
    { 0xe043cfec, SUC_lexicon_tags_60 },
    { 0x3cc3cfec, SUC_lexicon_tags_25 },
    { 0x98a3cfee, SUC_lexicon_tags_26 },
    { 0xaec3cfed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84c3cff1, SUC_lexicon_tags_13 },
    { 0xa443cff1, SUC_lexicon_tags_12 },
    { 0xf4a3cff3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a3cff7, SUC_lexicon_tags_0 },
    { 0x8463cff8, SUC_lexicon_tags_35 },
    { 0x1943cff9, SUC_lexicon_tags_1 },
    { 0xa4e3cffa, SUC_lexicon_tags_32 },
    { 0x20e3cff8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc3d000, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a63d003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b63d007, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3a3d009, SUC_lexicon_tags_17 },
    { 0x18e3d009, SUC_lexicon_tags_37 },
    { 0x4d83d00b, SUC_lexicon_tags_7 },
    { 0xbfe3d00a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c3d00e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4563d010, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe43d012, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a3d015, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7963d019, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe3d022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d63d024, SUC_lexicon_tags_0 },
    { 0xd503d025, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf503d027, SUC_lexicon_tags_25 },
    { 0x8ae3d028, SUC_lexicon_tags_60 },
    { 0xf823d029, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd123d02f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a3d035, SUC_lexicon_tags_0 },
    { 0x7063d035, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x71c3d038, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc43d03d, SUC_lexicon_tags_14 },
    { 0x6483d03e, SUC_lexicon_tags_38 },
    { 0x9903d03f, SUC_lexicon_tags_17 },
    { 0x6ea3d040, SUC_lexicon_tags_37 },
    { 0xeca3d03e, SUC_lexicon_tags_25 },
    { 0x4763d040, SUC_lexicon_tags_6 },
    { 0xd4a3d03d, SUC_lexicon_tags_37 },
    { 0x4ce3d041, SUC_lexicon_tags_37 },
    { 0xd7e3d045, SUC_lexicon_tags_43 },
    { 0xe8c3d046, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a3d050, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcee3d052, SUC_lexicon_tags_7 },
    { 0xa8c3d053, SUC_lexicon_tags_11 },
    { 0xa503d053, SUC_lexicon_tags_9 },
    { 0xbfc3d055, SUC_lexicon_tags_11 },
    { 0x3c23d054, SUC_lexicon_tags_69 },
    { 0xeb63d054, SUC_lexicon_tags_7 },
    { 0x6283d058, SUC_lexicon_tags_11 },
    { 0xb83d052, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d83d05c, SUC_lexicon_tags_33 },
    { 0x3c23d05d, SUC_lexicon_tags_37 },
    { 0xbee3d05e, SUC_lexicon_tags_14 },
    { 0xf803d05f, SUC_lexicon_tags_63 },
    { 0xfb63d060, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4403d063, SUC_lexicon_tags_33 },
    { 0x4e83d064, SUC_lexicon_tags_30 },
    { 0xbf43d064, SUC_lexicon_tags_7 },
    { 0xf223d066, SUC_lexicon_tags_14 },
    { 0x4a63d067, SUC_lexicon_tags_104 },
    { 0xc163d068, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf183d071, SUC_lexicon_tags_18 },
    { 0x64e3d072, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7563d07a, SUC_lexicon_tags_77 },
    { 0x1683d07b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c3d083, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x44c3d085, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe383d08b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8043d08d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c3d090, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8963d094, SUC_lexicon_tags_386 },
    { 0x7543d095, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa743d09a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb623d0a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc3d0a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6883d0a7, SUC_lexicon_tags_11 },
    { 0x5783d0a7, SUC_lexicon_tags_15 },
    { 0xa523d0a9, SUC_lexicon_tags_698 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e3d0ad, SUC_lexicon_tags_23 },
    { 0x2843d0ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc3d0b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3543d0b5, SUC_lexicon_tags_15 },
    { 0x6823d0b5, SUC_lexicon_tags_17 },
    { 0x4c83d0b6, SUC_lexicon_tags_7 },
    { 0xc183d0b8, SUC_lexicon_tags_25 },
    { 0xcd23d0b9, SUC_lexicon_tags_37 },
    { 0x72c3d0b8, SUC_lexicon_tags_23 },
    { 0xfca3d0bb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1e63d0bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa3d0c0, SUC_lexicon_tags_47 },
    { 0x4ac3d0c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9563d0c3, SUC_lexicon_tags_38 },
    { 0x6263d0c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe823d0c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e3d0ce, SUC_lexicon_tags_26 },
    { 0x9f23d0ce, SUC_lexicon_tags_37 },
    { 0xc043d0ce, SUC_lexicon_tags_45 },
    { 0x6bc3d0d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be3d0d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d63d0d9, SUC_lexicon_tags_2 },
    { 0x8043d0da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5763d0dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c3d0e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13e3d0e7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf023d0e9, SUC_lexicon_tags_15 },
    { 0x7ee3d0e9, SUC_lexicon_tags_12 },
    { 0xc723d0eb, SUC_lexicon_tags_37 },
    { 0x2cc3d0eb, SUC_lexicon_tags_1 },
    { 0xe203d0ec, SUC_lexicon_tags_16 },
    { 0x2b63d0eb, SUC_lexicon_tags_56 },
    { 0x24a3d0ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8343d0f3, SUC_lexicon_tags_28 },
    { 0x7f23d0f3, SUC_lexicon_tags_57 },
    { 0xd843d0f4, SUC_lexicon_tags_32 },
    { 0xfe3d0f5, SUC_lexicon_tags_7 },
    { 0xdd63d0f4, SUC_lexicon_tags_14 },
    { 0x583d0f5, SUC_lexicon_tags_8 },
    { 0xda3d0f7, SUC_lexicon_tags_4 },
    { 0x1603d0f5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b03d0ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3d102, SUC_lexicon_tags_63 },
    { 0x73e3d103, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1323d106, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3cc3d108, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e3d10c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e3d111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea3d114, SUC_lexicon_tags_11 },
    { 0x4423d114, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd183d124, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe283d12d, SUC_lexicon_tags_15 },
    { 0x5363d12e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6ae3d130, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x583d132, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf143d138, SUC_lexicon_tags_17 },
    { 0x3ba3d139, SUC_lexicon_tags_28 },
    { 0x8903d139, SUC_lexicon_tags_25 },
    { 0x4ee3d139, SUC_lexicon_tags_43 },
    { 0xa583d13c, SUC_lexicon_tags_25 },
    { 0x6143d13d, SUC_lexicon_tags_11 },
    { 0xc7c3d13e, SUC_lexicon_tags_30 },
    { 0xe683d13f, SUC_lexicon_tags_10 },
    { 0x5a83d13e, SUC_lexicon_tags_12 },
    { 0x70e3d141, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9163d14d, SUC_lexicon_tags_2 },
    { 0x2a23d14d, SUC_lexicon_tags_11 },
    { 0x6543d14f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a83d152, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3963d157, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7903d159, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba3d15c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda23d162, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee23d164, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x463d166, SUC_lexicon_tags_7 },
    { 0x4423d167, SUC_lexicon_tags_13 },
    { 0x4223d166, SUC_lexicon_tags_9 },
    { 0x5e03d168, SUC_lexicon_tags_201 },
    { 0xcea3d16a, SUC_lexicon_tags_66 },
    { 0xdc23d16b, SUC_lexicon_tags_1 },
    { 0xb243d16b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x50a3d16e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa43d170, SUC_lexicon_tags_11 },
    { 0x3363d170, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa1c3d173, SUC_lexicon_tags_2 },
    { 0xa743d174, SUC_lexicon_tags_37 },
    { 0xd4a3d175, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc3d179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5063d17b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e3d182, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa583d184, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa3c3d186, SUC_lexicon_tags_37 },
    { 0xae63d187, SUC_lexicon_tags_45 },
    { 0xcda3d187, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37c3d18a, SUC_lexicon_tags_45 },
    { 0xbd43d18b, SUC_lexicon_tags_55 },
    { 0xe7e3d18c, SUC_lexicon_tags_6 },
    { 0x36e3d18c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbb03d18f, SUC_lexicon_tags_68 },
    { 0xeb03d18f, SUC_lexicon_tags_37 },
    { 0x6aa3d191, SUC_lexicon_tags_11 },
    { 0x1343d192, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5a3d194, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e3d198, SUC_lexicon_tags_47 },
    { 0xa9c3d199, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3963d19b, SUC_lexicon_tags_37 },
    { 0x6263d19c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe0e3d19e, SUC_lexicon_tags_14 },
    { 0xff23d19e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c43d1a1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c43d1a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc3d1aa, SUC_lexicon_tags_37 },
    { 0xfd83d1ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc3d1b1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa3d1b6, SUC_lexicon_tags_12 },
    { 0x3403d1b6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a3d1bb, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x6883d1bd, SUC_lexicon_tags_29 },
    { 0xc003d1bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4383d1c1, SUC_lexicon_tags_2 },
    { 0xc83d1c2, SUC_lexicon_tags_23 },
    { 0xae43d1c2, SUC_lexicon_tags_19 },
    { 0x3223d1c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc3d1c7, SUC_lexicon_tags_23 },
    { 0x5dc3d1c7, SUC_lexicon_tags_8 },
    { 0x9c03d1c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc3d1cd, SUC_lexicon_tags_17 },
    { 0xfb23d1ce, SUC_lexicon_tags_15 },
    { 0x1443d1ce, SUC_lexicon_tags_22 },
    { 0x5623d1ce, SUC_lexicon_tags_30 },
    { 0x9023d1d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd83d1d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6803d1d6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd163d1da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1043d1dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6923d1de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf23d1e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7023d1e2, SUC_lexicon_tags_19 },
    { 0xa743d1e3, SUC_lexicon_tags_37 },
    { 0x23d1e4, SUC_lexicon_tags_7 },
    { 0xedc3d1e4, SUC_lexicon_tags_11 },
    { 0xadc3d1e6, SUC_lexicon_tags_15 },
    { 0x5fc3d1e7, SUC_lexicon_tags_33 },
    { 0xcea3d1e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9bc3d1ea, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e63d1ef, SUC_lexicon_tags_15 },
    { 0xe3c3d1f0, SUC_lexicon_tags_57 },
    { 0xa8c3d1f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2623d1f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9163d1f7, SUC_lexicon_tags_1 },
    { 0xd963d1f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a3d1ff, SUC_lexicon_tags_9 },
    { 0x4503d200, SUC_lexicon_tags_8 },
    { 0x3ea3d200, SUC_lexicon_tags_31 },
    { 0xda43d201, SUC_lexicon_tags_27 },
    { 0x6803d202, SUC_lexicon_tags_2 },
    { 0xd843d1ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc3d206, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf463d20c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa163d20e, SUC_lexicon_tags_8 },
    { 0x4ae3d20e, SUC_lexicon_tags_17 },
    { 0xdd63d20e, SUC_lexicon_tags_11 },
    { 0xcd03d210, SUC_lexicon_tags_13 },
    { 0x95c3d20e, SUC_lexicon_tags_25 },
    { 0xd003d20f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03d216, SUC_lexicon_tags_151 },
    { 0xe603d217, SUC_lexicon_tags_77 },
    { 0x1083d216, SUC_lexicon_tags_33 },
    { 0x9e43d219, SUC_lexicon_tags_47 },
    { 0x2983d21a, SUC_lexicon_tags_8 },
    { 0x7903d21b, SUC_lexicon_tags_68 },
    { 0xbb23d21c, SUC_lexicon_tags_60 },
    { 0xcda3d21d, SUC_lexicon_tags_11 },
    { 0xdd43d21c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xc5c3d220, SUC_lexicon_tags_29 },
    { 0x9dc3d220, SUC_lexicon_tags_66 },
    { 0x1303d222, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf823d226, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e3d229, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c3d22c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443d231, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba43d233, SUC_lexicon_tags_0 },
    { 0x5863d234, SUC_lexicon_tags_11 },
    { 0x8703d234, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x92e3d237, SUC_lexicon_tags_10 },
    { 0x4e3d238, SUC_lexicon_tags_2 },
    { 0x4423d239, SUC_lexicon_tags_37 },
    { 0x5ee3d23a, SUC_lexicon_tags_8 },
    { 0x3483d23b, SUC_lexicon_tags_23 },
    { 0x5383d23b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a3d23f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e3d245, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa43d247, SUC_lexicon_tags_37 },
    { 0xe463d248, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xce63d24b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd03d24d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9783d250, SUC_lexicon_tags_37 },
    { 0x5363d251, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1843d253, SUC_lexicon_tags_33 },
    { 0xe963d253, SUC_lexicon_tags_1 },
    { 0xd663d255, SUC_lexicon_tags_17 },
    { 0x8e63d256, SUC_lexicon_tags_31 },
    { 0xa863d257, SUC_lexicon_tags_61 },
    { 0x9bc3d258, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3fe3d25a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c43d25e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9683d263, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9823d266, SUC_lexicon_tags_113 },
    { 0xdb83d267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9043d269, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c43d271, SUC_lexicon_tags_7 },
    { 0x5283d271, SUC_lexicon_tags_39 },
    { 0x6923d273, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43a3d275, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe3d27a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x99a3d27c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa063d27e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb243d286, SUC_lexicon_tags_47 },
    { 0xca43d287, SUC_lexicon_tags_60 },
    { 0xf123d286, SUC_lexicon_tags_21 },
    { 0x823d289, SUC_lexicon_tags_37 },
    { 0x5d23d28a, SUC_lexicon_tags_60 },
    { 0x8ae3d288, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e3d291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf823d293, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e3d29a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3283d2a0, SUC_lexicon_tags_108 },
    { 0x9323d2a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6823d2a3, SUC_lexicon_tags_7 },
    { 0x3003d2a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce63d2a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d43d2a9, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x76e3d2ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e3d2ae, SUC_lexicon_tags_15 },
    { 0x7aa3d2af, SUC_lexicon_tags_12 },
    { 0x81c3d2b0, SUC_lexicon_tags_30 },
    { 0xf603d2af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1ba3d2b3, SUC_lexicon_tags_1 },
    { 0x7243d2b4, SUC_lexicon_tags_17 },
    { 0xd2e3d2b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a43d2b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe903d2bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ba3d2bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc3d2c3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e3d2c7, SUC_lexicon_tags_15 },
    { 0x63c3d2c7, SUC_lexicon_tags_38 },
    { 0x15e3d2c9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6e23d2cb, SUC_lexicon_tags_38 },
    { 0x7763d2cc, SUC_lexicon_tags_37 },
    { 0xf763d2cd, SUC_lexicon_tags_25 },
    { 0x47e3d2ce, SUC_lexicon_tags_11 },
    { 0x18a3d2cf, SUC_lexicon_tags_37 },
    { 0xb903d2d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcac3d2d2, SUC_lexicon_tags_11 },
    { 0xe143d2d2, SUC_lexicon_tags_91 },
    { 0xf6e3d2d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9963d2d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1283d2dc, SUC_lexicon_tags_8 },
    { 0x4b63d2dd, SUC_lexicon_tags_1 },
    { 0x68e3d2de, SUC_lexicon_tags_17 },
    { 0x4423d2de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c3d2e2, SUC_lexicon_tags_34 },
    { 0xa143d2e3, SUC_lexicon_tags_17 },
    { 0x58e3d2e4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe3d2e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8543d2ec, SUC_lexicon_tags_17 },
    { 0x60a3d2ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc3d2f2, SUC_lexicon_tags_23 },
    { 0xa23d2f3, SUC_lexicon_tags_37 },
    { 0x13a3d2f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30c3d2f6, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x2743d2f8, SUC_lexicon_tags_17 },
    { 0x1c3d2f9, SUC_lexicon_tags_33 },
    { 0x28c3d2fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1803d2fe, SUC_lexicon_tags_17 },
    { 0x1443d2ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf103d304, SUC_lexicon_tags_57 },
    { 0x9da3d305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf63d309, SUC_lexicon_tags_30 },
    { 0xbf43d309, SUC_lexicon_tags_26 },
    { 0x8383d30b, SUC_lexicon_tags_17 },
    { 0x4ce3d309, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1a43d30e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2883d310, SUC_lexicon_tags_37 },
    { 0x57a3d310, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeca3d313, SUC_lexicon_tags_49 },
    { 0x2a03d313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9783d319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa523d31b, SUC_lexicon_tags_37 },
    { 0x9063d31b, SUC_lexicon_tags_14 },
    { 0xb803d31d, SUC_lexicon_tags_8 },
    { 0xa583d31b, SUC_lexicon_tags_37 },
    { 0x5a3d31f, SUC_lexicon_tags_1 },
    { 0xd803d320, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a3d327, SUC_lexicon_tags_37 },
    { 0xc2c3d328, SUC_lexicon_tags_26 },
    { 0x9c23d328, SUC_lexicon_tags_47 },
    { 0xd5e3d32a, SUC_lexicon_tags_15 },
    { 0x2543d32b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8883d32e, SUC_lexicon_tags_48 },
    { 0xbea3d32f, SUC_lexicon_tags_10 },
    { 0x3e83d330, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d43d334, SUC_lexicon_tags_7 },
    { 0xb63d335, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf043d338, SUC_lexicon_tags_15 },
    { 0xfca3d339, SUC_lexicon_tags_48 },
    { 0xdf23d33a, SUC_lexicon_tags_17 },
    { 0x6ea3d33b, SUC_lexicon_tags_0 },
    { 0x6343d33c, SUC_lexicon_tags_26 },
    { 0x8663d33b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e83d340, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3d345, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43d349, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1243d34d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e3d352, SUC_lexicon_tags_19 },
    { 0x7283d352, SUC_lexicon_tags_1 },
    { 0x38e3d354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d357, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9183d359, SUC_lexicon_tags_23 },
    { 0x3ba3d35a, SUC_lexicon_tags_15 },
    { 0xfda3d35b, SUC_lexicon_tags_11 },
    { 0xa1c3d35c, SUC_lexicon_tags_37 },
    { 0xf0c3d35c, SUC_lexicon_tags_25 },
    { 0x2f83d35d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4843d360, SUC_lexicon_tags_2 },
    { 0x4b63d361, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba3d364, SUC_lexicon_tags_31 },
    { 0x1d63d365, SUC_lexicon_tags_16 },
    { 0x93e3d364, SUC_lexicon_tags_37 },
    { 0x3a3d367, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c3d36b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e43d36d, SUC_lexicon_tags_33 },
    { 0xadc3d36e, SUC_lexicon_tags_0 },
    { 0x7583d36e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c3d372, SUC_lexicon_tags_1 },
    { 0x363d373, SUC_lexicon_tags_12 },
    { 0x1fc3d372, SUC_lexicon_tags_6 },
    { 0x99c3d374, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e83d37a, SUC_lexicon_tags_12 },
    { 0xd8a3d37b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x88a3d37d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dc3d37f, SUC_lexicon_tags_23 },
    { 0x2863d380, SUC_lexicon_tags_25 },
    { 0x1c43d381, SUC_lexicon_tags_37 },
    { 0x5f43d381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1623d385, SUC_lexicon_tags_31 },
    { 0x82e3d386, SUC_lexicon_tags_11 },
    { 0x3c3d386, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4c3d389, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3723d38b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x363d38d, SUC_lexicon_tags_2 },
    { 0xc1e3d38e, SUC_lexicon_tags_49 },
    { 0xac83d38f, SUC_lexicon_tags_78 },
    { 0xf663d38f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a03d395, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5143d397, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a3d39a, SUC_lexicon_tags_19 },
    { 0xba3d39b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbda3d39d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5dc3d39f, SUC_lexicon_tags_12 },
    { 0xcae3d3a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8643d3a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc663d3a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x30c3d3a9, SUC_lexicon_tags_8 },
    { 0x2183d3aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba3d3b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8763d3b6, SUC_lexicon_tags_47 },
    { 0x1843d3b7, SUC_lexicon_tags_29 },
    { 0x19e3d3b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a83d3bc, SUC_lexicon_tags_137 },
    { 0x6303d3bd, SUC_lexicon_tags_43 },
    { 0xbc3d3bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bc3d3c0, SUC_lexicon_tags_26 },
    { 0x90a3d3c1, SUC_lexicon_tags_37 },
    { 0x8403d3c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3d3c7, SUC_lexicon_tags_14 },
    { 0xec23d3c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83d3cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a3d3d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c3d3d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9903d3d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe163d3db, SUC_lexicon_tags_0 },
    { 0x82e3d3dc, SUC_lexicon_tags_0 },
    { 0x1623d3dc, SUC_lexicon_tags_19 },
    { 0xcca3d3de, SUC_lexicon_tags_7 },
    { 0xfc03d3dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae3d3e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f63d3e6, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e63d3e9, SUC_lexicon_tags_29 },
    { 0xd83d3ea, SUC_lexicon_tags_41 },
    { 0x4d23d3e9, SUC_lexicon_tags_289 },
    { 0xbb83d3ec, SUC_lexicon_tags_11 },
    { 0xaee3d3ed, SUC_lexicon_tags_21 },
    { 0x9303d3ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4c03d3f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e3d3f3, SUC_lexicon_tags_17 },
    { 0x37e3d3f4, SUC_lexicon_tags_3 },
    { 0x3d43d3f4, SUC_lexicon_tags_8 },
    { 0xb043d3f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e3d3fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88e3d3fd, SUC_lexicon_tags_64 },
    { 0xa4a3d3fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7c3d400, SUC_lexicon_tags_17 },
    { 0x3e63d401, SUC_lexicon_tags_38 },
    { 0x1423d401, SUC_lexicon_tags_55 },
    { 0x3843d401, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda43d407, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9be3d409, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xee03d40b, SUC_lexicon_tags_113 },
    { 0xed83d40c, SUC_lexicon_tags_68 },
    { 0xe163d40d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9063d40f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03d412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63d417, SUC_lexicon_tags_81 },
    { 0x4863d418, SUC_lexicon_tags_11 },
    { 0x33c3d417, SUC_lexicon_tags_15 },
    { 0xd2a3d41a, SUC_lexicon_tags_23 },
    { 0xc843d41b, SUC_lexicon_tags_7 },
    { 0xac83d41b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57a3d41e, SUC_lexicon_tags_15 },
    { 0x9e83d41f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a83d423, SUC_lexicon_tags_37 },
    { 0x2683d423, SUC_lexicon_tags_12 },
    { 0x77c3d424, SUC_lexicon_tags_33 },
    { 0x7bc3d426, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25a3d428, SUC_lexicon_tags_17 },
    { 0x583d429, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5043d42b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3563d42e, SUC_lexicon_tags_15 },
    { 0x7dc3d42e, SUC_lexicon_tags_1 },
    { 0x1203d430, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3d436, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc023d43a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2123d43c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a3d445, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2f23d447, SUC_lexicon_tags_14 },
    { 0x9423d448, SUC_lexicon_tags_104 },
    { 0x7a03d448, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3903d44b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3983d44d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8a3d44f, SUC_lexicon_tags_65 },
    { 0x103d44f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a3d456, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x703d459, SUC_lexicon_tags_12 },
    { 0x7e03d459, SUC_lexicon_tags_48 },
    { 0xc563d45b, SUC_lexicon_tags_12 },
    { 0xfae3d45b, SUC_lexicon_tags_15 },
    { 0xe363d45c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e3d460, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8083d462, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeae3d464, SUC_lexicon_tags_38 },
    { 0xc2a3d465, SUC_lexicon_tags_39 },
    { 0x10e3d464, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3443d468, SUC_lexicon_tags_11 },
    { 0x4343d469, SUC_lexicon_tags_37 },
    { 0x55a3d468, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5123d46d, SUC_lexicon_tags_48 },
    { 0xbf23d46e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4663d471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x543d473, SUC_lexicon_tags_37 },
    { 0x81a3d474, SUC_lexicon_tags_25 },
    { 0xf023d473, SUC_lexicon_tags_33 },
    { 0x4523d474, SUC_lexicon_tags_7 },
    { 0xda83d477, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa23d479, SUC_lexicon_tags_14 },
    { 0x31e3d47a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb683d47d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d23d47f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a3d483, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e3d487, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5043d489, SUC_lexicon_tags_1 },
    { 0x7c3d48a, SUC_lexicon_tags_63 },
    { 0x6363d489, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x683d48d, SUC_lexicon_tags_38 },
    { 0xc183d48e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe703d495, SUC_lexicon_tags_11 },
    { 0x2a23d496, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2923d498, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b43d49b, SUC_lexicon_tags_25 },
    { 0x5e83d49c, SUC_lexicon_tags_29 },
    { 0x2123d49d, SUC_lexicon_tags_14 },
    { 0xb5e3d49c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20a3d4a0, SUC_lexicon_tags_27 },
    { 0x70e3d4a1, SUC_lexicon_tags_12 },
    { 0x8543d4a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3d4ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc583d4b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17a3d4b3, SUC_lexicon_tags_30 },
    { 0x6723d4b4, SUC_lexicon_tags_17 },
    { 0x5bc3d4b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a83d4b7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x88c3d4b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9983d4bc, SUC_lexicon_tags_2 },
    { 0xd6a3d4bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc83d4c4, SUC_lexicon_tags_26 },
    { 0x25e3d4c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7163d4c7, SUC_lexicon_tags_1 },
    { 0xdfc3d4c8, SUC_lexicon_tags_11 },
    { 0x483d4c8, SUC_lexicon_tags_7 },
    { 0xd243d4c7, SUC_lexicon_tags_47 },
    { 0x7263d4cb, SUC_lexicon_tags_47 },
    { 0x9b83d4cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde43d4cf, SUC_lexicon_tags_1 },
    { 0xbfc3d4d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a3d4d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe703d4d8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2463d4da, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf863d4dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb43d4e1, SUC_lexicon_tags_12 },
    { 0x7ae3d4e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c43d4e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca63d4ec, SUC_lexicon_tags_12 },
    { 0xb983d4ed, SUC_lexicon_tags_1 },
    { 0xec43d4ee, SUC_lexicon_tags_12 },
    { 0xd103d4ed, SUC_lexicon_tags_17 },
    { 0x86e3d4ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03d4f4, SUC_lexicon_tags_11 },
    { 0x8a83d4f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e3d4f9, SUC_lexicon_tags_11 },
    { 0x8cc3d4f9, SUC_lexicon_tags_33 },
    { 0xe8c3d4f9, SUC_lexicon_tags_2 },
    { 0x64a3d4fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5c83d4fe, SUC_lexicon_tags_7 },
    { 0x9c23d4ff, SUC_lexicon_tags_25 },
    { 0x60a3d4ff, SUC_lexicon_tags_7 },
    { 0xd143d501, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e03d504, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x183d507, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb03d50a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5543d511, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc3d516, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf23d51a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe283d51d, SUC_lexicon_tags_17 },
    { 0xcf23d51e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff03d520, SUC_lexicon_tags_4 },
    { 0x9603d521, SUC_lexicon_tags_35 },
    { 0xa563d521, SUC_lexicon_tags_19 },
    { 0x69c3d521, SUC_lexicon_tags_7 },
    { 0x1943d521, SUC_lexicon_tags_25 },
    { 0x69a3d520, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x143d527, SUC_lexicon_tags_47 },
    { 0xb3e3d528, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeae3d52a, SUC_lexicon_tags_7 },
    { 0xf7c3d52b, SUC_lexicon_tags_17 },
    { 0x3fe3d52c, SUC_lexicon_tags_17 },
    { 0x42c3d52c, SUC_lexicon_tags_2 },
    { 0x2783d52e, SUC_lexicon_tags_14 },
    { 0xe423d52d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa3d535, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3143d53d, SUC_lexicon_tags_63 },
    { 0xc203d53e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3063d543, SUC_lexicon_tags_1 },
    { 0x9863d544, SUC_lexicon_tags_2 },
    { 0x8003d545, SUC_lexicon_tags_27 },
    { 0x3003d545, SUC_lexicon_tags_34 },
    { 0x8c63d547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3183d54b, SUC_lexicon_tags_23 },
    { 0x8883d54c, SUC_lexicon_tags_49 },
    { 0xd283d54d, SUC_lexicon_tags_37 },
    { 0xca63d54e, SUC_lexicon_tags_9 },
    { 0x5cc3d54e, SUC_lexicon_tags_17 },
    { 0xfc63d550, SUC_lexicon_tags_49 },
    { 0xad03d551, SUC_lexicon_tags_63 },
    { 0x5d23d54d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa423d554, SUC_lexicon_tags_47 },
    { 0xc4e3d554, SUC_lexicon_tags_17 },
    { 0xb363d556, SUC_lexicon_tags_19 },
    { 0xfe43d556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76a3d559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc283d55c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a03d561, SUC_lexicon_tags_5 },
    { 0xffa3d561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c3d565, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e03d567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd483d56b, SUC_lexicon_tags_38 },
    { 0xfdc3d56c, SUC_lexicon_tags_12 },
    { 0x16e3d56d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5183d574, SUC_lexicon_tags_15 },
    { 0xea23d575, SUC_lexicon_tags_14 },
    { 0x42e3d576, SUC_lexicon_tags_12 },
    { 0x4163d576, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2183d57d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf503d57f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e83d584, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa3d589, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8063d58d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f43d590, SUC_lexicon_tags_16 },
    { 0xf4e3d590, SUC_lexicon_tags_32 },
    { 0xfdc3d591, SUC_lexicon_tags_11 },
    { 0xd823d593, SUC_lexicon_tags_62 },
    { 0x27e3d593, SUC_lexicon_tags_19 },
    { 0x47e3d595, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3303d597, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8583d599, SUC_lexicon_tags_17 },
    { 0x3aa3d59a, SUC_lexicon_tags_30 },
    { 0x823d59a, SUC_lexicon_tags_11 },
    { 0x4a83d59a, SUC_lexicon_tags_25 },
    { 0xf143d59a, SUC_lexicon_tags_77 },
    { 0x1643d59a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2c3d5a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1ba3d5a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1343d5a4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc83d5ac, SUC_lexicon_tags_27 },
    { 0x4f23d5ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d5af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xecc3d5b1, SUC_lexicon_tags_37 },
    { 0x3c03d5b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac23d5b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x923d5b7, SUC_lexicon_tags_15 },
    { 0xb063d5b8, SUC_lexicon_tags_31 },
    { 0x32e3d5b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a3d5bd, SUC_lexicon_tags_82 },
    { 0x2683d5be, SUC_lexicon_tags_33 },
    { 0x6ea3d5bf, SUC_lexicon_tags_37 },
    { 0xbbe3d5c0, SUC_lexicon_tags_17 },
    { 0xe5e3d5bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac3d5c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ea3d5c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc123d5c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6fa3d5cb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c3d5ce, SUC_lexicon_tags_33 },
    { 0x2183d5ce, SUC_lexicon_tags_17 },
    { 0x55a3d5cf, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c3d5d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec3d5d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc343d5de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc763d5e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c83d5e3, SUC_lexicon_tags_0 },
    { 0xc0c3d5e4, SUC_lexicon_tags_17 },
    { 0x50c3d5e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a3d5ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8763d5ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae3d5f3, SUC_lexicon_tags_117 },
    { 0xef63d5f4, SUC_lexicon_tags_33 },
    { 0xad23d5f4, SUC_lexicon_tags_27 },
    { 0xe83d5f6, SUC_lexicon_tags_6 },
    { 0xb303d5f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd303d5f9, SUC_lexicon_tags_48 },
    { 0x39c3d5fa, SUC_lexicon_tags_45 },
    { 0xaa63d5fb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da3d600, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7363d605, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7503d60a, SUC_lexicon_tags_61 },
    { 0xa623d60a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2203d60d, SUC_lexicon_tags_15 },
    { 0xb0e3d60e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae3d611, SUC_lexicon_tags_16 },
    { 0x81c3d611, SUC_lexicon_tags_15 },
    { 0x1ee3d613, SUC_lexicon_tags_40 },
    { 0x9d23d614, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c3d617, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c43d61e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5fa3d620, SUC_lexicon_tags_33 },
    { 0xcd23d621, SUC_lexicon_tags_26 },
    { 0x99c3d621, SUC_lexicon_tags_17 },
    { 0x9e23d622, SUC_lexicon_tags_7 },
    { 0x62a3d624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc083d628, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7343d62d, SUC_lexicon_tags_38 },
    { 0x6bc3d62e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc43d634, SUC_lexicon_tags_123 },
    { 0x14a3d635, SUC_lexicon_tags_37 },
    { 0xdae3d636, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83d63b, SUC_lexicon_tags_14 },
    { 0x1603d63c, SUC_lexicon_tags_7 },
    { 0xa1e3d63c, SUC_lexicon_tags_25 },
    { 0xe3d63e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e3d642, SUC_lexicon_tags_47 },
    { 0x5de3d643, SUC_lexicon_tags_25 },
    { 0x3ca3d644, SUC_lexicon_tags_19 },
    { 0xd643d645, SUC_lexicon_tags_15 },
    { 0x61a3d646, SUC_lexicon_tags_15 },
    { 0x9a83d647, SUC_lexicon_tags_15 },
    { 0x8bc3d648, SUC_lexicon_tags_6 },
    { 0x93e3d647, SUC_lexicon_tags_30 },
    { 0x1f83d647, SUC_lexicon_tags_11 },
    { 0x75e3d649, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba3d64e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1103d653, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd83d655, SUC_lexicon_tags_28 },
    { 0x38a3d656, SUC_lexicon_tags_25 },
    { 0x3803d657, SUC_lexicon_tags_15 },
    { 0x5383d657, SUC_lexicon_tags_12 },
    { 0x2b83d659, SUC_lexicon_tags_38 },
    { 0x83d65a, SUC_lexicon_tags_17 },
    { 0xbee3d657, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d43d65e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5e3d660, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77e3d662, SUC_lexicon_tags_17 },
    { 0x8d83d662, SUC_lexicon_tags_63 },
    { 0xba43d662, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6a23d666, SUC_lexicon_tags_66 },
    { 0x1563d667, SUC_lexicon_tags_7 },
    { 0x2d23d667, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e3d66c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8dc3d66e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce83d671, SUC_lexicon_tags_33 },
    { 0xbe63d672, SUC_lexicon_tags_25 },
    { 0xbaa3d673, SUC_lexicon_tags_17 },
    { 0x5bc3d671, SUC_lexicon_tags_43 },
    { 0x28e3d672, SUC_lexicon_tags_87 },
    { 0xed43d676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3683d67b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f63d67d, SUC_lexicon_tags_27 },
    { 0xbd03d67e, SUC_lexicon_tags_1 },
    { 0x1863d67f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c63d68b, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xd243d68e, SUC_lexicon_tags_1 },
    { 0xa043d68f, SUC_lexicon_tags_49 },
    { 0xc803d68f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e3d695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf803d698, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ae3d69a, SUC_lexicon_tags_27 },
    { 0x1ac3d69a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8b63d69d, SUC_lexicon_tags_12 },
    { 0x2603d69e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3d6a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3783d6a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3803d6ad, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae3d6b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5a3d6b6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe403d6ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb63d6bc, SUC_lexicon_tags_11 },
    { 0xf563d6bd, SUC_lexicon_tags_61 },
    { 0x70c3d6be, SUC_lexicon_tags_160 },
    { 0x1383d6be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52c3d6c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e3d6cf, SUC_lexicon_tags_60 },
    { 0x5be3d6d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd1c3d6d2, SUC_lexicon_tags_15 },
    { 0x26c3d6d2, SUC_lexicon_tags_10 },
    { 0x12c3d6d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6203d6d7, SUC_lexicon_tags_27 },
    { 0xf7c3d6d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0a3d6da, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a43d6e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a3d6e6, SUC_lexicon_tags_37 },
    { 0x943d6e7, SUC_lexicon_tags_38 },
    { 0x8303d6e7, SUC_lexicon_tags_43 },
    { 0x14a3d6e9, SUC_lexicon_tags_25 },
    { 0x91e3d6ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6a3d6ec, SUC_lexicon_tags_17 },
    { 0x29a3d6ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd63d6ef, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x91c3d6f1, SUC_lexicon_tags_18 },
    { 0xcb83d6f2, SUC_lexicon_tags_49 },
    { 0xe663d6f2, SUC_lexicon_tags_37 },
    { 0xef83d6f3, SUC_lexicon_tags_11 },
    { 0x91a3d6f5, SUC_lexicon_tags_63 },
    { 0x1263d6f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9303d6f9, SUC_lexicon_tags_11 },
    { 0xd943d6fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e3d6fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xab03d701, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2ac3d703, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6623d706, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3d70a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x183d70d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3263d70f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa903d713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2a3d715, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a63d71c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c3d71f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b23d723, SUC_lexicon_tags_15 },
    { 0xea3d723, SUC_lexicon_tags_47 },
    { 0x6923d725, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da3d728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce83d72b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c3d72f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c3d732, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c23d736, SUC_lexicon_tags_87 },
    { 0x1023d737, SUC_lexicon_tags_11 },
    { 0x3fa3d736, SUC_lexicon_tags_11 },
    { 0x2823d739, SUC_lexicon_tags_1 },
    { 0x4de3d73a, SUC_lexicon_tags_27 },
    { 0x6e63d73b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26e3d73d, SUC_lexicon_tags_31 },
    { 0x6803d73e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c83d741, SUC_lexicon_tags_55 },
    { 0x4223d742, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc223d745, SUC_lexicon_tags_14 },
    { 0xb183d746, SUC_lexicon_tags_40 },
    { 0xbf43d745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c3d74e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa963d752, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8683d759, SUC_lexicon_tags_0 },
    { 0xe083d75a, SUC_lexicon_tags_5 },
    { 0x5463d75a, SUC_lexicon_tags_33 },
    { 0xc0e3d759, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c3d75f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe63d767, SUC_lexicon_tags_26 },
    { 0x35a3d768, SUC_lexicon_tags_36 },
    { 0x243d769, SUC_lexicon_tags_56 },
    { 0x5443d769, SUC_lexicon_tags_8 },
    { 0x4f23d767, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x3d76d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a3d772, SUC_lexicon_tags_1 },
    { 0x6e83d773, SUC_lexicon_tags_13 },
    { 0x49a3d773, SUC_lexicon_tags_87 },
    { 0x7423d772, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2443d779, SUC_lexicon_tags_40 },
    { 0x6543d77a, SUC_lexicon_tags_43 },
    { 0x3323d77b, SUC_lexicon_tags_47 },
    { 0x5d83d77c, SUC_lexicon_tags_43 },
    { 0x6c83d77d, SUC_lexicon_tags_17 },
    { 0xa523d77e, SUC_lexicon_tags_37 },
    { 0xc923d77e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c3d782, SUC_lexicon_tags_179 },
    { 0x2743d783, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe9e3d785, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb003d789, SUC_lexicon_tags_65 },
    { 0x7863d789, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e83d78c, SUC_lexicon_tags_123 },
    { 0xa1c3d78d, SUC_lexicon_tags_27 },
    { 0x3543d78d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a3d792, SUC_lexicon_tags_39 },
    { 0x9a83d793, SUC_lexicon_tags_15 },
    { 0xab23d794, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1103d797, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e3d79c, SUC_lexicon_tags_17 },
    { 0x11e3d79d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e3d7a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83d7a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3423d7a8, SUC_lexicon_tags_17 },
    { 0xe383d7a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d83d7ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2a43d7ae, SUC_lexicon_tags_66 },
    { 0x3023d7af, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe823d7b9, SUC_lexicon_tags_434 },
    { 0x83c3d7ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba03d7bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6163d7c5, SUC_lexicon_tags_29 },
    { 0xb083d7c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a3d7cd, SUC_lexicon_tags_15 },
    { 0x3963d7cd, SUC_lexicon_tags_25 },
    { 0x6203d7cf, SUC_lexicon_tags_0 },
    { 0xa323d7cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd603d7d4, SUC_lexicon_tags_15 },
    { 0x5123d7d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5443d7d7, SUC_lexicon_tags_25 },
    { 0x40e3d7d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d83d7df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a63d7e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3d7e5, SUC_lexicon_tags_23 },
    { 0x7203d7e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ac3d7e8, SUC_lexicon_tags_37 },
    { 0x61a3d7e9, SUC_lexicon_tags_25 },
    { 0x6863d7ea, SUC_lexicon_tags_15 },
    { 0x4c43d7e9, SUC_lexicon_tags_113 },
    { 0x3ce3d7e9, SUC_lexicon_tags_3 },
    { 0x7383d7ed, SUC_lexicon_tags_1 },
    { 0x6d43d7ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c23d7f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4043d7f4, SUC_lexicon_tags_8 },
    { 0x563d7f5, SUC_lexicon_tags_28 },
    { 0xe0e3d7f5, SUC_lexicon_tags_0 },
    { 0x97a3d7f7, SUC_lexicon_tags_87 },
    { 0x4e63d7f8, SUC_lexicon_tags_48 },
    { 0x683d7f9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd03d7fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b43d802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1c3d804, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe723d80b, SUC_lexicon_tags_17 },
    { 0x6b63d80b, SUC_lexicon_tags_7 },
    { 0x72e3d80d, SUC_lexicon_tags_12 },
    { 0xb23d80e, SUC_lexicon_tags_15 },
    { 0xf863d80f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1563d811, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe103d813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c3d816, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x15e3d818, SUC_lexicon_tags_37 },
    { 0xe43d818, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9423d81d, SUC_lexicon_tags_15 },
    { 0xd443d81e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc483d820, SUC_lexicon_tags_47 },
    { 0xd263d820, SUC_lexicon_tags_12 },
    { 0x5d03d822, SUC_lexicon_tags_14 },
    { 0x2b03d823, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e3d826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6183d829, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc983d82f, SUC_lexicon_tags_25 },
    { 0x2623d830, SUC_lexicon_tags_11 },
    { 0x1163d830, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c3d834, SUC_lexicon_tags_25 },
    { 0xdc03d835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e3d83d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc543d841, SUC_lexicon_tags_84 },
    { 0xeac3d842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803d846, SUC_lexicon_tags_29 },
    { 0x9fe3d847, SUC_lexicon_tags_38 },
    { 0xde23d848, SUC_lexicon_tags_37 },
    { 0x22a3d848, SUC_lexicon_tags_85 },
    { 0x2263d847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03d84f, SUC_lexicon_tags_45 },
    { 0x4963d84f, SUC_lexicon_tags_4 },
    { 0xc3d851, SUC_lexicon_tags_2 },
    { 0xd503d84f, SUC_lexicon_tags_12 },
    { 0x8a3d852, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f03d855, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a3d85a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62a3d85c, SUC_lexicon_tags_79 },
    { 0x8443d85d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7623d862, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e3d866, SUC_lexicon_tags_15 },
    { 0xc723d867, SUC_lexicon_tags_8 },
    { 0x77e3d868, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7163d86a, SUC_lexicon_tags_14 },
    { 0x3a03d86b, SUC_lexicon_tags_37 },
    { 0xa483d86b, SUC_lexicon_tags_114 },
    { 0x15a3d86b, SUC_lexicon_tags_2 },
    { 0x7603d86e, SUC_lexicon_tags_66 },
    { 0xeb43d86f, SUC_lexicon_tags_25 },
    { 0xcbc3d86f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8823d872, SUC_lexicon_tags_37 },
    { 0x3ba3d873, SUC_lexicon_tags_14 },
    { 0xb183d873, SUC_lexicon_tags_17 },
    { 0x2443d875, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb723d877, SUC_lexicon_tags_38 },
    { 0x623d877, SUC_lexicon_tags_15 },
    { 0x6863d879, SUC_lexicon_tags_17 },
    { 0xe43d879, SUC_lexicon_tags_23 },
    { 0x4bc3d87b, SUC_lexicon_tags_10 },
    { 0x8023d879, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a3d880, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e63d884, SUC_lexicon_tags_15 },
    { 0x5b43d884, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5043d888, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec3d88d, SUC_lexicon_tags_2 },
    { 0x32e3d88e, SUC_lexicon_tags_10 },
    { 0x6103d88e, SUC_lexicon_tags_49 },
    { 0x4c3d88e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd83d897, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe3d89a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c3d89e, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xaa43d8a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3d8a3, SUC_lexicon_tags_0 },
    { 0x76e3d8a4, SUC_lexicon_tags_11 },
    { 0x6603d8a5, SUC_lexicon_tags_17 },
    { 0xeae3d8a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x743d8a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e63d8ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e3d8b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f63d8b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc63d8b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7143d8ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2aa3d8bc, SUC_lexicon_tags_27 },
    { 0xd443d8bd, SUC_lexicon_tags_23 },
    { 0xc023d8be, SUC_lexicon_tags_68 },
    { 0xe663d8bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x443d8c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac3d8c7, SUC_lexicon_tags_9 },
    { 0xbb03d8c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1903d8cb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303d8d1, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xa4e3d8d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8d43d8d5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9463d8d9, SUC_lexicon_tags_17 },
    { 0x4bc3d8da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c23d8de, SUC_lexicon_tags_2 },
    { 0x3f23d8df, SUC_lexicon_tags_17 },
    { 0xfb23d8e0, SUC_lexicon_tags_1 },
    { 0x8c03d8e1, SUC_lexicon_tags_11 },
    { 0x8f63d8e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3163d8e6, SUC_lexicon_tags_21 },
    { 0x6b43d8e7, SUC_lexicon_tags_2 },
    { 0xfaa3d8e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9103d8ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1503d8ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e3d8f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc63d8f4, SUC_lexicon_tags_33 },
    { 0xa123d8f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa63d8f7, SUC_lexicon_tags_7 },
    { 0x9043d8f7, SUC_lexicon_tags_79 },
    { 0x7c43d8f9, SUC_lexicon_tags_43 },
    { 0xdea3d8fa, SUC_lexicon_tags_29 },
    { 0xf003d8fb, SUC_lexicon_tags_21 },
    { 0xe523d8fc, SUC_lexicon_tags_15 },
    { 0x8ee3d8fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2e83d8ff, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5f63d901, SUC_lexicon_tags_2 },
    { 0x19e3d901, SUC_lexicon_tags_2 },
    { 0xf143d903, SUC_lexicon_tags_7 },
    { 0xb43d904, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2183d906, SUC_lexicon_tags_15 },
    { 0xd6c3d907, SUC_lexicon_tags_11 },
    { 0xb6a3d908, SUC_lexicon_tags_264 },
    { 0x6d43d909, SUC_lexicon_tags_5 },
    { 0x6063d90a, SUC_lexicon_tags_30 },
    { 0x6a23d907, SUC_lexicon_tags_25 },
    { 0xdf03d909, SUC_lexicon_tags_19 },
    { 0xf843d90d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6183d90f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5603d913, SUC_lexicon_tags_2 },
    { 0xb1e3d914, SUC_lexicon_tags_68 },
    { 0xcba3d915, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3d91a, SUC_lexicon_tags_87 },
    { 0xc23d91b, SUC_lexicon_tags_60 },
    { 0xa5e3d91b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5443d91e, SUC_lexicon_tags_43 },
    { 0x2ac3d91f, SUC_lexicon_tags_37 },
    { 0x2783d91e, SUC_lexicon_tags_16 },
    { 0xda3d920, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf243d924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae3d927, SUC_lexicon_tags_48 },
    { 0x1d23d928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e3d92b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3323d92d, SUC_lexicon_tags_48 },
    { 0xeda3d92e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc123d930, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a3d934, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6363d936, SUC_lexicon_tags_7 },
    { 0x2023d937, SUC_lexicon_tags_77 },
    { 0x5963d938, SUC_lexicon_tags_63 },
    { 0x5203d938, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff03d93b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x963d93d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7c3d93f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6423d941, SUC_lexicon_tags_1 },
    { 0x15a3d942, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7463d948, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4903d94a, SUC_lexicon_tags_46 },
    { 0xfe03d94a, SUC_lexicon_tags_27 },
    { 0xdb83d94c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe523d94e, SUC_lexicon_tags_27 },
    { 0x1e03d94e, SUC_lexicon_tags_19 },
    { 0xb23d950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd223d953, SUC_lexicon_tags_14 },
    { 0xd163d953, SUC_lexicon_tags_23 },
    { 0xf223d955, SUC_lexicon_tags_25 },
    { 0x2f03d953, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab43d959, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f43d95b, SUC_lexicon_tags_2 },
    { 0x7fe3d95b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43d962, SUC_lexicon_tags_11 },
    { 0x3843d963, SUC_lexicon_tags_63 },
    { 0x4783d964, SUC_lexicon_tags_11 },
    { 0x9e23d965, SUC_lexicon_tags_17 },
    { 0xe23d966, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x87a3d968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba3d96d, SUC_lexicon_tags_31 },
    { 0x5d23d96e, SUC_lexicon_tags_120 },
    { 0xafc3d96d, SUC_lexicon_tags_17 },
    { 0xaf83d970, SUC_lexicon_tags_63 },
    { 0x2dc3d971, SUC_lexicon_tags_33 },
    { 0xf323d972, SUC_lexicon_tags_19 },
    { 0x4ce3d971, SUC_lexicon_tags_23 },
    { 0x2a3d974, SUC_lexicon_tags_12 },
    { 0xee3d974, SUC_lexicon_tags_15 },
    { 0xb5c3d976, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c03d97b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c3d97f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcb03d981, SUC_lexicon_tags_37 },
    { 0xf443d982, SUC_lexicon_tags_10 },
    { 0xfa3d983, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb463d985, SUC_lexicon_tags_64 },
    { 0x8323d986, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03d98c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f83d995, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e3d998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe3d99c, SUC_lexicon_tags_37 },
    { 0xdc3d99d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7803d9a2, SUC_lexicon_tags_25 },
    { 0xa703d9a3, SUC_lexicon_tags_7 },
    { 0x64a3d9a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc443d9a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf783d9ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2903d9ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a3d9b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8e3d9b5, SUC_lexicon_tags_12 },
    { 0xecc3d9b6, SUC_lexicon_tags_11 },
    { 0xff83d9b5, SUC_lexicon_tags_48 },
    { 0x6743d9b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb423d9ba, SUC_lexicon_tags_25 },
    { 0xdee3d9ba, SUC_lexicon_tags_66 },
    { 0x8603d9bc, SUC_lexicon_tags_14 },
    { 0xd303d9bd, SUC_lexicon_tags_33 },
    { 0x9d63d9ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1ea3d9c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb763d9c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d83d9c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd343d9d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x18c3d9d2, SUC_lexicon_tags_17 },
    { 0xcf63d9d2, SUC_lexicon_tags_39 },
    { 0x6ba3d9d2, SUC_lexicon_tags_47 },
    { 0x63a3d9d5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c3d9da, SUC_lexicon_tags_7 },
    { 0x78a3d9db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9383d9e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbce3d9e3, SUC_lexicon_tags_7 },
    { 0xfce3d9e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e63d9e9, SUC_lexicon_tags_43 },
    { 0x85a3d9ea, SUC_lexicon_tags_25 },
    { 0x3363d9eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c63d9ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc223d9fc, SUC_lexicon_tags_7 },
    { 0x8923d9fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1943d9ff, SUC_lexicon_tags_7 },
    { 0xfdc3d9ff, SUC_lexicon_tags_60 },
    { 0xfea3d9ff, SUC_lexicon_tags_11 },
    { 0xb6c3da02, SUC_lexicon_tags_1 },
    { 0x543da02, SUC_lexicon_tags_38 },
    { 0x103da02, SUC_lexicon_tags_1589 },
    { 0x2303da05, SUC_lexicon_tags_66 },
    { 0xf823da04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1843da0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f83da0e, SUC_lexicon_tags_12 },
    { 0xe083da0e, SUC_lexicon_tags_7 },
    { 0xdb83da10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe823da14, SUC_lexicon_tags_38 },
    { 0x1a23da15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7fc3da17, SUC_lexicon_tags_37 },
    { 0x3b63da17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf23da1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea3da1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a3da23, SUC_lexicon_tags_11 },
    { 0x2583da24, SUC_lexicon_tags_12 },
    { 0x4883da24, SUC_lexicon_tags_8 },
    { 0xfd43da26, SUC_lexicon_tags_74 },
    { 0x8683da26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec3da2a, SUC_lexicon_tags_21 },
    { 0x9543da2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d83da2e, SUC_lexicon_tags_28 },
    { 0xb543da2f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c3da32, SUC_lexicon_tags_2 },
    { 0xc023da33, SUC_lexicon_tags_11 },
    { 0xc6e3da33, SUC_lexicon_tags_9 },
    { 0xfc03da34, SUC_lexicon_tags_7 },
    { 0x203da36, SUC_lexicon_tags_14 },
    { 0x70e3da35, SUC_lexicon_tags_11 },
    { 0x3083da34, SUC_lexicon_tags_7 },
    { 0xbf23da38, SUC_lexicon_tags_15 },
    { 0xf303da33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa03da3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a3da43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbbc3da45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c3da4c, SUC_lexicon_tags_12 },
    { 0x25e3da4d, SUC_lexicon_tags_15 },
    { 0x69a3da4e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc3da59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3223da5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3da63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf63da65, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x1ee3da67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43da6a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xeec3da6c, SUC_lexicon_tags_168 },
    { 0x75e3da6d, SUC_lexicon_tags_11 },
    { 0xbae3da6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a3da70, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a3da75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e3da79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc683da7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e83da80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c3da8a, SUC_lexicon_tags_25 },
    { 0x7283da8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2163da8f, SUC_lexicon_tags_32 },
    { 0xca3da90, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3da94, SUC_lexicon_tags_12 },
    { 0x8323da95, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd283da97, SUC_lexicon_tags_1061 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc3daa1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c3daa5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e83daa8, SUC_lexicon_tags_37 },
    { 0xeca3daa9, SUC_lexicon_tags_8 },
    { 0xf963daa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x203daac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c3dab5, SUC_lexicon_tags_15 },
    { 0xf583dab5, SUC_lexicon_tags_45 },
    { 0x9583dab7, SUC_lexicon_tags_37 },
    { 0x3aa3dab6, SUC_lexicon_tags_17 },
    { 0x9e43dab9, SUC_lexicon_tags_1 },
    { 0xdf83daba, SUC_lexicon_tags_55 },
    { 0xf6a3dabb, SUC_lexicon_tags_25 },
    { 0xe123dabb, SUC_lexicon_tags_14 },
    { 0xdc23dab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b83dabf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd523dac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf23dac5, SUC_lexicon_tags_9 },
    { 0x8463dac6, SUC_lexicon_tags_45 },
    { 0xa063dac7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d63daca, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xbd83dacc, SUC_lexicon_tags_11 },
    { 0x46a3dacd, SUC_lexicon_tags_64 },
    { 0xb283dace, SUC_lexicon_tags_7 },
    { 0xd03dacf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7423dad3, SUC_lexicon_tags_49 },
    { 0x5043dad4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03dad7, SUC_lexicon_tags_65 },
    { 0x1183dad7, SUC_lexicon_tags_17 },
    { 0x3183dad9, SUC_lexicon_tags_17 },
    { 0x9943dada, SUC_lexicon_tags_30 },
    { 0x1563dadb, SUC_lexicon_tags_63 },
    { 0xeac3dadc, SUC_lexicon_tags_37 },
    { 0xa43dadb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc623dae0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a3dae3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc3dae6, SUC_lexicon_tags_7 },
    { 0xdfe3dae7, SUC_lexicon_tags_38 },
    { 0x2de3dae8, SUC_lexicon_tags_30 },
    { 0xb203dae8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e23daed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd603daf2, SUC_lexicon_tags_113 },
    { 0x5fe3daf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4883daf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e3db02, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7403db04, SUC_lexicon_tags_7 },
    { 0xb323db05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8763db07, SUC_lexicon_tags_60 },
    { 0xef23db08, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c03db0b, SUC_lexicon_tags_47 },
    { 0xad63db0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe283db0e, SUC_lexicon_tags_8 },
    { 0xb3e3db0f, SUC_lexicon_tags_45 },
    { 0x3403db10, SUC_lexicon_tags_39 },
    { 0x9603db11, SUC_lexicon_tags_27 },
    { 0x9c43db0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc163db16, SUC_lexicon_tags_12 },
    { 0xa943db17, SUC_lexicon_tags_11 },
    { 0x80a3db18, SUC_lexicon_tags_65 },
    { 0xb23db19, SUC_lexicon_tags_31 },
    { 0xec83db19, SUC_lexicon_tags_15 },
    { 0x50e3db1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e23db1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3db22, SUC_lexicon_tags_48 },
    { 0x3943db23, SUC_lexicon_tags_47 },
    { 0xb1e3db24, SUC_lexicon_tags_19 },
    { 0x7bc3db24, SUC_lexicon_tags_7 },
    { 0xa423db25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3db2c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1763db32, SUC_lexicon_tags_77 },
    { 0x90c3db33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9483db35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f03db37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f23db3d, SUC_lexicon_tags_12 },
    { 0x5183db3e, SUC_lexicon_tags_1 },
    { 0x2ea3db3f, SUC_lexicon_tags_47 },
    { 0xd223db3f, SUC_lexicon_tags_37 },
    { 0xde03db41, SUC_lexicon_tags_25 },
    { 0xa823db3e, SUC_lexicon_tags_63 },
    { 0x2b03db43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f03db45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2883db48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c03db4a, SUC_lexicon_tags_30 },
    { 0x6663db4b, SUC_lexicon_tags_43 },
    { 0x1923db4b, SUC_lexicon_tags_31 },
    { 0x1603db4b, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xe723db50, SUC_lexicon_tags_17 },
    { 0x3cc3db51, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8b63db53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4103db5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5d43db5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81c3db5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9023db62, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6063db65, SUC_lexicon_tags_25 },
    { 0xa7a3db66, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa43db69, SUC_lexicon_tags_45 },
    { 0xca43db69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de3db6f, SUC_lexicon_tags_62 },
    { 0xed03db6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca3db74, SUC_lexicon_tags_17 },
    { 0xd3a3db75, SUC_lexicon_tags_2 },
    { 0x5ec3db76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6683db7c, SUC_lexicon_tags_2 },
    { 0x22a3db7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dc3db7f, SUC_lexicon_tags_7 },
    { 0x5ae3db7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa983db83, SUC_lexicon_tags_191 },
    { 0x2223db84, SUC_lexicon_tags_63 },
    { 0x2b63db85, SUC_lexicon_tags_12 },
    { 0x3c83db85, SUC_lexicon_tags_28 },
    { 0x53e3db83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf203db8a, SUC_lexicon_tags_11 },
    { 0x7663db8a, SUC_lexicon_tags_17 },
    { 0x5543db8c, SUC_lexicon_tags_23 },
    { 0x46a3db8d, SUC_lexicon_tags_1 },
    { 0xf5a3db8e, SUC_lexicon_tags_14 },
    { 0x24a3db8a, SUC_lexicon_tags_0 },
    { 0x1e23db90, SUC_lexicon_tags_33 },
    { 0x8c63db8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf03db93, SUC_lexicon_tags_7 },
    { 0x2ba3db93, SUC_lexicon_tags_17 },
    { 0x5ba3db95, SUC_lexicon_tags_9 },
    { 0xe563db96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a63db9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6403db9c, SUC_lexicon_tags_47 },
    { 0x6d23db9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a3dba0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a3dba8, SUC_lexicon_tags_1234 },
    { 0x1b63dba8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9463dbab, SUC_lexicon_tags_15 },
    { 0x20a3dbac, SUC_lexicon_tags_49 },
    { 0x2de3dbac, SUC_lexicon_tags_2 },
    { 0x7c63dbad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3e3dbb0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xea83dbb3, SUC_lexicon_tags_11 },
    { 0xc7a3dbb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2143dbba, SUC_lexicon_tags_2 },
    { 0x6fc3dbba, SUC_lexicon_tags_30 },
    { 0xdc83dbbc, SUC_lexicon_tags_33 },
    { 0xb443dbbc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe643dbbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3923dbc5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3dbcc, SUC_lexicon_tags_12 },
    { 0x7083dbcd, SUC_lexicon_tags_29 },
    { 0x3343dbcd, SUC_lexicon_tags_4 },
    { 0xd523dbce, SUC_lexicon_tags_66 },
    { 0x1da3dbce, SUC_lexicon_tags_1 },
    { 0xab83dbd1, SUC_lexicon_tags_6 },
    { 0xbc43dbd2, SUC_lexicon_tags_11 },
    { 0xbe03dbd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2a3dbd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed83dbd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e3dbdb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9163dbdd, SUC_lexicon_tags_42 },
    { 0x5d83dbde, SUC_lexicon_tags_17 },
    { 0xcb63dbdd, SUC_lexicon_tags_2 },
    { 0xb9a3dbde, SUC_lexicon_tags_1 },
    { 0x7983dbe1, SUC_lexicon_tags_27 },
    { 0xb203dbe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3a3dbe4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f83dbeb, SUC_lexicon_tags_1 },
    { 0x38c3dbec, SUC_lexicon_tags_19 },
    { 0xea3dbed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea3dbf0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5b63dbf2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3dbfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1c83dbfd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf223dc00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x62a3dc02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6283dc06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ca3dc08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf03dc0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1983dc11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1363dc15, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c83dc18, SUC_lexicon_tags_2 },
    { 0x3da3dc19, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e3dc1c, SUC_lexicon_tags_65 },
    { 0xb943dc1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a23dc1f, SUC_lexicon_tags_84 },
    { 0x23dc20, SUC_lexicon_tags_19 },
    { 0x6023dc21, SUC_lexicon_tags_11 },
    { 0xa403dc22, SUC_lexicon_tags_23 },
    { 0x5443dc22, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x5e63dc25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8043dc2b, SUC_lexicon_tags_61 },
    { 0x7843dc2b, SUC_lexicon_tags_43 },
    { 0x28c3dc2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2083dc33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede3dc38, SUC_lexicon_tags_26 },
    { 0xbd03dc39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe423dc3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2903dc41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd883dc44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae03dc4b, SUC_lexicon_tags_23 },
    { 0x2583dc4c, SUC_lexicon_tags_27 },
    { 0x663dc4d, SUC_lexicon_tags_28 },
    { 0x1563dc4e, SUC_lexicon_tags_82 },
    { 0xe6c3dc4f, SUC_lexicon_tags_32 },
    { 0x59a3dc4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863dc54, SUC_lexicon_tags_17 },
    { 0x9903dc55, SUC_lexicon_tags_196 },
    { 0xd9a3dc56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1803dc5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5b23dc5c, SUC_lexicon_tags_33 },
    { 0x64a3dc5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29e3dc5f, SUC_lexicon_tags_7 },
    { 0xd8a3dc60, SUC_lexicon_tags_29 },
    { 0xcf63dc61, SUC_lexicon_tags_47 },
    { 0xa3a3dc62, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x1263dc64, SUC_lexicon_tags_136 },
    { 0xab63dc65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e3dc69, SUC_lexicon_tags_30 },
    { 0x8c03dc6a, SUC_lexicon_tags_15 },
    { 0x4d23dc69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf923dc71, SUC_lexicon_tags_14 },
    { 0xf03dc72, SUC_lexicon_tags_9 },
    { 0x9823dc73, SUC_lexicon_tags_63 },
    { 0x38e3dc73, SUC_lexicon_tags_11 },
    { 0xb563dc73, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8303dc7b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a3dc81, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3583dc84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a3dc87, SUC_lexicon_tags_14 },
    { 0x9fe3dc88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c3dc8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac03dc8f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e3dc92, SUC_lexicon_tags_38 },
    { 0x9ca3dc93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c3dc96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc3dc99, SUC_lexicon_tags_7 },
    { 0x7783dc9a, SUC_lexicon_tags_1 },
    { 0x743dc9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e3dc9e, SUC_lexicon_tags_30 },
    { 0x2e83dc9e, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c63dca2, SUC_lexicon_tags_11 },
    { 0xa1e3dca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa783dca6, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b03dcad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6143dcaf, SUC_lexicon_tags_47 },
    { 0xb243dcaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a3dcb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f43dcbc, SUC_lexicon_tags_11 },
    { 0x6923dcbc, SUC_lexicon_tags_45 },
    { 0xb743dcbe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbce3dcc0, SUC_lexicon_tags_30 },
    { 0x5ae3dcc1, SUC_lexicon_tags_25 },
    { 0xff43dcc2, SUC_lexicon_tags_11 },
    { 0x7c43dcc3, SUC_lexicon_tags_11 },
    { 0xdb43dcc4, SUC_lexicon_tags_68 },
    { 0xe423dcc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf223dccf, SUC_lexicon_tags_4 },
    { 0x9223dccf, SUC_lexicon_tags_6 },
    { 0x6623dcd1, SUC_lexicon_tags_17 },
    { 0x8fa3dcd0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e3dcd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcda3dcd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e3dcdb, SUC_lexicon_tags_39 },
    { 0x4243dcdc, SUC_lexicon_tags_46 },
    { 0x77e3dcdd, SUC_lexicon_tags_25 },
    { 0x1ee3dcdb, SUC_lexicon_tags_19 },
    { 0xf223dcdf, SUC_lexicon_tags_17 },
    { 0xdfe3dce0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47e3dce2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3743dce6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6e3dce8, SUC_lexicon_tags_17 },
    { 0x1da3dce9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8723dced, SUC_lexicon_tags_1 },
    { 0x77c3dcee, SUC_lexicon_tags_53 },
    { 0x4ae3dcef, SUC_lexicon_tags_7 },
    { 0x3f23dcf0, SUC_lexicon_tags_49 },
    { 0x8d23dcf1, SUC_lexicon_tags_26 },
    { 0x7ec3dcf2, SUC_lexicon_tags_28 },
    { 0xafc3dcf2, SUC_lexicon_tags_17 },
    { 0x60a3dcee, SUC_lexicon_tags_17 },
    { 0x5743dcef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc3dcfc, SUC_lexicon_tags_43 },
    { 0xb403dcfd, SUC_lexicon_tags_15 },
    { 0x65a3dcfc, SUC_lexicon_tags_4 },
    { 0x75e3dcff, SUC_lexicon_tags_12 },
    { 0x70c3dd00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe3dd03, SUC_lexicon_tags_15 },
    { 0xb743dd04, SUC_lexicon_tags_47 },
    { 0x9923dd05, SUC_lexicon_tags_17 },
    { 0x4b23dd06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x143dd08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6003dd0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4223dd0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8bc3dd0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd703dd13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7263dd15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2123dd18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf6e3dd1a, SUC_lexicon_tags_17 },
    { 0xba23dd1b, SUC_lexicon_tags_1 },
    { 0xc1a3dd1b, SUC_lexicon_tags_17 },
    { 0xe403dd1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a3dd23, SUC_lexicon_tags_25 },
    { 0x61c3dd24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7063dd2c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b43dd35, SUC_lexicon_tags_25 },
    { 0x4de3dd36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8803dd3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa063dd40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee3dd43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb83dd45, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe243dd49, SUC_lexicon_tags_55 },
    { 0x6b83dd4a, SUC_lexicon_tags_38 },
    { 0x703dd4b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6063dd4d, SUC_lexicon_tags_38 },
    { 0x15c3dd4e, SUC_lexicon_tags_14 },
    { 0x17e3dd4f, SUC_lexicon_tags_37 },
    { 0x46c3dd50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe203dd56, SUC_lexicon_tags_2 },
    { 0xf423dd57, SUC_lexicon_tags_13 },
    { 0xcf63dd58, SUC_lexicon_tags_25 },
    { 0x6243dd58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa3dd5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f63dd61, SUC_lexicon_tags_25 },
    { 0x6483dd62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6143dd64, SUC_lexicon_tags_105 },
    { 0x8b03dd64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce83dd68, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc83dd71, SUC_lexicon_tags_7 },
    { 0x4c43dd72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa3dd78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd223dd7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9643dd84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c03dd8b, SUC_lexicon_tags_2 },
    { 0xdf23dd8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb143dd8f, SUC_lexicon_tags_39 },
    { 0xa423dd90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd363dd96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c3dd9a, SUC_lexicon_tags_47 },
    { 0x6203dd9a, SUC_lexicon_tags_17 },
    { 0x4483dd9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7863dd9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e23dda2, SUC_lexicon_tags_11 },
    { 0xb0e3dda3, SUC_lexicon_tags_31 },
    { 0x8123dda3, SUC_lexicon_tags_28 },
    { 0xb943dda2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f83dda7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce23ddac, SUC_lexicon_tags_2 },
    { 0x8843ddac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66c3ddaf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee3ddb2, SUC_lexicon_tags_47 },
    { 0x3283ddb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc803ddb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03ddbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e43ddbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e63ddc3, SUC_lexicon_tags_68 },
    { 0x6b83ddc3, SUC_lexicon_tags_11 },
    { 0xc103ddc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6543ddc8, SUC_lexicon_tags_7 },
    { 0xc7c3ddc9, SUC_lexicon_tags_31 },
    { 0x61e3ddc9, SUC_lexicon_tags_2 },
    { 0xbce3ddcb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6b03ddcd, SUC_lexicon_tags_12 },
    { 0xe3ddce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c43ddd0, SUC_lexicon_tags_30 },
    { 0x9ae3ddd1, SUC_lexicon_tags_33 },
    { 0xa543ddd2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8143ddd4, SUC_lexicon_tags_0 },
    { 0xc643ddd5, SUC_lexicon_tags_25 },
    { 0x9e03ddd6, SUC_lexicon_tags_1 },
    { 0x4903ddd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb643ddd9, SUC_lexicon_tags_29 },
    { 0xa283ddda, SUC_lexicon_tags_25 },
    { 0x3a3dddb, SUC_lexicon_tags_13 },
    { 0xdae3dddb, SUC_lexicon_tags_2 },
    { 0x4b03dddd, SUC_lexicon_tags_66 },
    { 0x1f23dddb, SUC_lexicon_tags_45 },
    { 0xa8c3dddd, SUC_lexicon_tags_49 },
    { 0x7763dde0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c3ddee, SUC_lexicon_tags_22 },
    { 0xea43ddee, SUC_lexicon_tags_45 },
    { 0x8fa3ddef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95e3ddf2, SUC_lexicon_tags_26 },
    { 0x34c3ddf3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83ddf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc3ddfd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec3de04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a3de07, SUC_lexicon_tags_7 },
    { 0x2c83de08, SUC_lexicon_tags_28 },
    { 0x7fe3de09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5183de0b, SUC_lexicon_tags_0 },
    { 0x1283de0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f43de0e, SUC_lexicon_tags_0 },
    { 0xe23de0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a3de12, SUC_lexicon_tags_32 },
    { 0x1ae3de13, SUC_lexicon_tags_37 },
    { 0x1963de14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c83de16, SUC_lexicon_tags_38 },
    { 0xdde3de17, SUC_lexicon_tags_2 },
    { 0x1243de18, SUC_lexicon_tags_8 },
    { 0xbc43de19, SUC_lexicon_tags_47 },
    { 0xd2e3de19, SUC_lexicon_tags_47 },
    { 0xb03de16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed63de1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3de23, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfd63de25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada3de2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8203de34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef83de39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a3de3c, SUC_lexicon_tags_47 },
    { 0xce23de3d, SUC_lexicon_tags_2 },
    { 0x84c3de3e, SUC_lexicon_tags_37 },
    { 0xf143de3f, SUC_lexicon_tags_2 },
    { 0x1863de3f, SUC_lexicon_tags_6 },
    { 0xa583de3f, SUC_lexicon_tags_14 },
    { 0x1d83de3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd403de44, SUC_lexicon_tags_37 },
    { 0x29e3de44, SUC_lexicon_tags_14 },
    { 0x1383de44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea3de4a, SUC_lexicon_tags_37 },
    { 0x4663de4a, SUC_lexicon_tags_21 },
    { 0x30a3de4c, SUC_lexicon_tags_11 },
    { 0xb0e3de4c, SUC_lexicon_tags_43 },
    { 0xd843de4d, SUC_lexicon_tags_55 },
    { 0xf123de4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e3de53, SUC_lexicon_tags_7 },
    { 0x8383de54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8303de56, SUC_lexicon_tags_31 },
    { 0x4523de57, SUC_lexicon_tags_160 },
    { 0x983de58, SUC_lexicon_tags_17 },
    { 0x6c03de59, SUC_lexicon_tags_15 },
    { 0xffc3de5a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a3de67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa103de69, SUC_lexicon_tags_416 },
    { 0x5163de6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a3de6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb203de70, SUC_lexicon_tags_11 },
    { 0x8803de71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e3de74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e3de78, SUC_lexicon_tags_49 },
    { 0x3523de79, SUC_lexicon_tags_1 },
    { 0x88c3de79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2343de7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5a3de7f, SUC_lexicon_tags_47 },
    { 0x7ee3de80, SUC_lexicon_tags_43 },
    { 0xb0c3de81, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb283de84, SUC_lexicon_tags_7 },
    { 0xe8e3de85, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd943de87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e3de89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9823de8e, SUC_lexicon_tags_14 },
    { 0x62e3de8e, SUC_lexicon_tags_69 },
    { 0x7ca3de90, SUC_lexicon_tags_63 },
    { 0xc9c3de91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c3de94, SUC_lexicon_tags_26 },
    { 0xe763de94, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x8263de98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc3dea1, SUC_lexicon_tags_32 },
    { 0x55e3dea2, SUC_lexicon_tags_17 },
    { 0x3243dea3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c3dea7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7083deac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a3deaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9763deb1, SUC_lexicon_tags_14 },
    { 0x5483deb2, SUC_lexicon_tags_2 },
    { 0x57a3deb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4483deb5, SUC_lexicon_tags_15 },
    { 0x1ce3deb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd903deb8, SUC_lexicon_tags_25 },
    { 0x3043deb8, SUC_lexicon_tags_39 },
    { 0x1483deba, SUC_lexicon_tags_23 },
    { 0x43e3debb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4f63debd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3823dec0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a43dec2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc63dec4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b03dec7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf843dec9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9523decb, SUC_lexicon_tags_43 },
    { 0xa2c3decc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec3decf, SUC_lexicon_tags_25 },
    { 0x9b43decf, SUC_lexicon_tags_1 },
    { 0xbb23ded1, SUC_lexicon_tags_96 },
    { 0xf303ded2, SUC_lexicon_tags_2 },
    { 0x7923ded0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3ded6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc7e3ded8, SUC_lexicon_tags_43 },
    { 0xa163ded8, SUC_lexicon_tags_8 },
    { 0x5603deda, SUC_lexicon_tags_7 },
    { 0x5163dedb, SUC_lexicon_tags_336 },
    { 0x3ac3deda, SUC_lexicon_tags_45 },
    { 0xab83dedd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c3dee1, SUC_lexicon_tags_55 },
    { 0xd943dee1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3dee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef63dee7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf343deea, SUC_lexicon_tags_7 },
    { 0x5463deea, SUC_lexicon_tags_29 },
    { 0x83c3deeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1623def0, SUC_lexicon_tags_12 },
    { 0x2063def1, SUC_lexicon_tags_15 },
    { 0x7f03def2, SUC_lexicon_tags_14 },
    { 0x4ec3def3, SUC_lexicon_tags_3 },
    { 0x3e43def4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc323defc, SUC_lexicon_tags_9 },
    { 0x723defc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76c3deff, SUC_lexicon_tags_17 },
    { 0xfa83deff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2283df06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea3df09, SUC_lexicon_tags_21 },
    { 0xb883df0a, SUC_lexicon_tags_37 },
    { 0x1243df0b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba03df0f, SUC_lexicon_tags_7 },
    { 0x71a3df0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7743df12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba3df15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf543df17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d03df1d, SUC_lexicon_tags_2 },
    { 0xc23df1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e3df21, SUC_lexicon_tags_15 },
    { 0xf623df22, SUC_lexicon_tags_45 },
    { 0x2be3df23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe3df26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3063df2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7363df32, SUC_lexicon_tags_27 },
    { 0xfca3df33, SUC_lexicon_tags_25 },
    { 0xe23df34, SUC_lexicon_tags_4 },
    { 0xaf83df35, SUC_lexicon_tags_43 },
    { 0x85c3df34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0c3df38, SUC_lexicon_tags_45 },
    { 0x80e3df38, SUC_lexicon_tags_1 },
    { 0x4323df39, SUC_lexicon_tags_17 },
    { 0x7dc3df3b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d43df3e, SUC_lexicon_tags_11 },
    { 0x55a3df3e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc323df41, SUC_lexicon_tags_37 },
    { 0x6463df42, SUC_lexicon_tags_68 },
    { 0x6823df41, SUC_lexicon_tags_63 },
    { 0x88c3df42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3063df48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23df54, SUC_lexicon_tags_2 },
    { 0x2d63df55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce3df59, SUC_lexicon_tags_63 },
    { 0x7363df5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6923df5d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8323df67, SUC_lexicon_tags_55 },
    { 0x5683df68, SUC_lexicon_tags_25 },
    { 0xeaa3df69, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa663df6b, SUC_lexicon_tags_1 },
    { 0x53c3df6c, SUC_lexicon_tags_19 },
    { 0xd9a3df6b, SUC_lexicon_tags_87 },
    { 0xc343df6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf8a3df70, SUC_lexicon_tags_48 },
    { 0xe4c3df70, SUC_lexicon_tags_2 },
    { 0x3303df70, SUC_lexicon_tags_25 },
    { 0x4103df73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe8c3df75, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x42a3df77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf823df7a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x9f43df7c, SUC_lexicon_tags_43 },
    { 0x5883df7d, SUC_lexicon_tags_2 },
    { 0x1363df7e, SUC_lexicon_tags_27 },
    { 0x1903df7e, SUC_lexicon_tags_37 },
    { 0x7ea3df7d, SUC_lexicon_tags_59 },
    { 0x5e63df81, SUC_lexicon_tags_15 },
    { 0xaf03df82, SUC_lexicon_tags_0 },
    { 0x6583df82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a3df87, SUC_lexicon_tags_43 },
    { 0xf0c3df88, SUC_lexicon_tags_60 },
    { 0x5a23df87, SUC_lexicon_tags_43 },
    { 0x8b43df8a, SUC_lexicon_tags_37 },
    { 0x67c3df87, SUC_lexicon_tags_7 },
    { 0xe463df8c, SUC_lexicon_tags_12 },
    { 0x4903df8c, SUC_lexicon_tags_46 },
    { 0x1da3df87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a3df90, SUC_lexicon_tags_7 },
    { 0xe4e3df90, SUC_lexicon_tags_57 },
    { 0xf643df92, SUC_lexicon_tags_47 },
    { 0xcea3df93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8943df96, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa143df9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc3df9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c3dfa4, SUC_lexicon_tags_26 },
    { 0xb403dfa5, SUC_lexicon_tags_17 },
    { 0x1443dfa6, SUC_lexicon_tags_11 },
    { 0xdf03dfa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e03dfaa, SUC_lexicon_tags_55 },
    { 0xa03dfab, SUC_lexicon_tags_37 },
    { 0xc683dfac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d83dfb0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203dfb5, SUC_lexicon_tags_17 },
    { 0xb5e3dfb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c3dfb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23dfbf, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a3dfc4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba3dfc9, SUC_lexicon_tags_11 },
    { 0xf423dfca, SUC_lexicon_tags_34 },
    { 0x6043dfcb, SUC_lexicon_tags_16 },
    { 0x20c3dfcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xca43dfce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa3dfd4, SUC_lexicon_tags_17 },
    { 0xb9e3dfd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd43dfd9, SUC_lexicon_tags_49 },
    { 0x5d63dfda, SUC_lexicon_tags_174 },
    { 0x4b03dfdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c83dfdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac3dfe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb43dfea, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x1ba3dfec, SUC_lexicon_tags_182 },
    { 0x3983dfed, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9543dff3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd663dff5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba3dff9, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63e000, SUC_lexicon_tags_15 },
    { 0xc0c3e001, SUC_lexicon_tags_8 },
    { 0x65e3e002, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c3e00a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6323e00e, SUC_lexicon_tags_43 },
    { 0xf203e00e, SUC_lexicon_tags_30 },
    { 0xc923e00f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7183e012, SUC_lexicon_tags_17 },
    { 0x3a63e012, SUC_lexicon_tags_39 },
    { 0x7423e014, SUC_lexicon_tags_39 },
    { 0xb9a3e014, SUC_lexicon_tags_1 },
    { 0x1703e013, SUC_lexicon_tags_31 },
    { 0xcfe3e017, SUC_lexicon_tags_21 },
    { 0x6d03e018, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e03e01f, SUC_lexicon_tags_47 },
    { 0x54c3e020, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e3e02a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7103e02c, SUC_lexicon_tags_87 },
    { 0x5d23e02c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6683e035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3e03b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff03e03f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x52e3e041, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff83e044, SUC_lexicon_tags_967 },
    { 0x3183e045, SUC_lexicon_tags_1 },
    { 0x8ba3e046, SUC_lexicon_tags_12 },
    { 0xa43e046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d63e04e, SUC_lexicon_tags_32 },
    { 0x90a3e04e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8543e051, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4bc3e053, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb943e055, SUC_lexicon_tags_25 },
    { 0x9d83e056, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc3e05b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8423e05d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43e068, SUC_lexicon_tags_11 },
    { 0xb403e069, SUC_lexicon_tags_1 },
    { 0xa203e068, SUC_lexicon_tags_15 },
    { 0xbe23e06b, SUC_lexicon_tags_17 },
    { 0xb603e068, SUC_lexicon_tags_12 },
    { 0x3043e06a, SUC_lexicon_tags_17 },
    { 0x20e3e06e, SUC_lexicon_tags_3 },
    { 0x5da3e06f, SUC_lexicon_tags_7 },
    { 0x1763e070, SUC_lexicon_tags_37 },
    { 0xe283e06e, SUC_lexicon_tags_37 },
    { 0xabe3e06e, SUC_lexicon_tags_37 },
    { 0x39c3e06f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f03e077, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xeb43e079, SUC_lexicon_tags_11 },
    { 0xd863e07a, SUC_lexicon_tags_2 },
    { 0x7ca3e07b, SUC_lexicon_tags_45 },
    { 0x8303e07c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae3e07f, SUC_lexicon_tags_23 },
    { 0xe423e07f, SUC_lexicon_tags_12 },
    { 0x7803e080, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5083e087, SUC_lexicon_tags_12 },
    { 0x6e83e087, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7623e08b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6fe3e08d, SUC_lexicon_tags_7 },
    { 0x3063e08e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce3e095, SUC_lexicon_tags_7 },
    { 0xefa3e096, SUC_lexicon_tags_37 },
    { 0x7f43e096, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a43e09c, SUC_lexicon_tags_27 },
    { 0x6fa3e09d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12c3e09f, SUC_lexicon_tags_21 },
    { 0xd243e0a0, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5723e0a6, SUC_lexicon_tags_61 },
    { 0x4663e0a7, SUC_lexicon_tags_43 },
    { 0x3c03e0a8, SUC_lexicon_tags_11 },
    { 0x1d63e0a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbfe3e0ab, SUC_lexicon_tags_25 },
    { 0xf9c3e0ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3e0b1, SUC_lexicon_tags_27 },
    { 0x1823e0b1, SUC_lexicon_tags_17 },
    { 0x8883e0b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf843e0b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8943e0b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48a3e0b9, SUC_lexicon_tags_11 },
    { 0x1563e0ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a3e0c3, SUC_lexicon_tags_17 },
    { 0x6b23e0c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2a3e0c6, SUC_lexicon_tags_43 },
    { 0xeda3e0c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9823e0c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8083e0cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf383e0ce, SUC_lexicon_tags_11 },
    { 0x2043e0cf, SUC_lexicon_tags_47 },
    { 0x49c3e0d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e3e0d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da3e0db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf23e0dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a3e0e0, SUC_lexicon_tags_2 },
    { 0x2243e0e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da3e0e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc3e0ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e3e0f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5803e0f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c3e0f6, SUC_lexicon_tags_40 },
    { 0x1083e0f7, SUC_lexicon_tags_37 },
    { 0xa763e0f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5643e0fc, SUC_lexicon_tags_27 },
    { 0x2683e0fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a3e100, SUC_lexicon_tags_12 },
    { 0x2243e100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57c3e103, SUC_lexicon_tags_45 },
    { 0x5483e103, SUC_lexicon_tags_45 },
    { 0x7743e104, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e3e10d, SUC_lexicon_tags_124 },
    { 0xc663e10e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe323e110, SUC_lexicon_tags_15 },
    { 0x9443e111, SUC_lexicon_tags_38 },
    { 0xdd23e112, SUC_lexicon_tags_66 },
    { 0x9ec3e113, SUC_lexicon_tags_25 },
    { 0xf323e114, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a3e117, SUC_lexicon_tags_25 },
    { 0x41e3e118, SUC_lexicon_tags_37 },
    { 0xaa43e119, SUC_lexicon_tags_14 },
    { 0x9d43e118, SUC_lexicon_tags_7 },
    { 0x1843e11b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823e11f, SUC_lexicon_tags_48 },
    { 0xbfa3e120, SUC_lexicon_tags_83 },
    { 0xbd83e120, SUC_lexicon_tags_43 },
    { 0x4de3e11f, SUC_lexicon_tags_7 },
    { 0xd303e11f, SUC_lexicon_tags_26 },
    { 0x1fc3e11f, SUC_lexicon_tags_46 },
    { 0xb8e3e124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe203e127, SUC_lexicon_tags_5 },
    { 0xe343e127, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26c3e12a, SUC_lexicon_tags_14 },
    { 0xfee3e12b, SUC_lexicon_tags_15 },
    { 0x8103e12c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf63e12e, SUC_lexicon_tags_43 },
    { 0x8703e12f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad23e135, SUC_lexicon_tags_2 },
    { 0x2a63e136, SUC_lexicon_tags_4 },
    { 0x4003e137, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf03e139, SUC_lexicon_tags_3 },
    { 0xf503e13a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9163e13f, SUC_lexicon_tags_7 },
    { 0x723e140, SUC_lexicon_tags_8 },
    { 0xe463e141, SUC_lexicon_tags_1 },
    { 0x82e3e13f, SUC_lexicon_tags_26 },
    { 0xbd03e13f, SUC_lexicon_tags_4 },
    { 0x99e3e144, SUC_lexicon_tags_63 },
    { 0xfd63e145, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7303e148, SUC_lexicon_tags_19 },
    { 0xcc23e149, SUC_lexicon_tags_14 },
    { 0x7a3e14a, SUC_lexicon_tags_39 },
    { 0xd123e148, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3e14f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e3e156, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3263e158, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5a3e15a, SUC_lexicon_tags_53 },
    { 0xeee3e15a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4023e15e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae3e163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea3e166, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c3e169, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3e16e, SUC_lexicon_tags_14 },
    { 0xd683e16e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x823e171, SUC_lexicon_tags_37 },
    { 0x8443e171, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f63e174, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8543e181, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1483e186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c03e18b, SUC_lexicon_tags_32 },
    { 0xe483e18b, SUC_lexicon_tags_16 },
    { 0xf83e18d, SUC_lexicon_tags_29 },
    { 0xa6e3e18d, SUC_lexicon_tags_34 },
    { 0x66c3e18f, SUC_lexicon_tags_3 },
    { 0xb223e18f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa763e193, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c3e196, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc303e198, SUC_lexicon_tags_32 },
    { 0x99e3e199, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf043e1a0, SUC_lexicon_tags_28 },
    { 0xd9a3e1a0, SUC_lexicon_tags_37 },
    { 0xa783e1a1, SUC_lexicon_tags_45 },
    { 0x5503e1a3, SUC_lexicon_tags_29 },
    { 0x2543e1a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c43e1aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a23e1ad, SUC_lexicon_tags_11 },
    { 0xb243e1ae, SUC_lexicon_tags_47 },
    { 0x1fe3e1af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf103e1b7, SUC_lexicon_tags_30 },
    { 0xe3e3e1b8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5683e1bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c63e1bf, SUC_lexicon_tags_7 },
    { 0x24c3e1c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd543e1c3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c3e1c9, SUC_lexicon_tags_12 },
    { 0x57e3e1ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa443e1d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e3e1d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfae3e1da, SUC_lexicon_tags_15 },
    { 0x6203e1db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce3e1df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd343e1e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd23e1eb, SUC_lexicon_tags_1 },
    { 0xa683e1ec, SUC_lexicon_tags_30 },
    { 0x6303e1ec, SUC_lexicon_tags_27 },
    { 0x5123e1ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e3e1f1, SUC_lexicon_tags_84 },
    { 0x4ea3e1f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1e63e1f4, SUC_lexicon_tags_37 },
    { 0xeea3e1f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b43e1f9, SUC_lexicon_tags_91 },
    { 0x70c3e1fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2443e1fd, SUC_lexicon_tags_7 },
    { 0xbac3e1fe, SUC_lexicon_tags_37 },
    { 0xc6a3e1ff, SUC_lexicon_tags_38 },
    { 0xf783e200, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2183e203, SUC_lexicon_tags_12 },
    { 0x4d63e203, SUC_lexicon_tags_25 },
    { 0x6f83e205, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb83e207, SUC_lexicon_tags_60 },
    { 0xcb63e207, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa523e20b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c3e20e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf223e211, SUC_lexicon_tags_11 },
    { 0x8983e211, SUC_lexicon_tags_68 },
    { 0x89c3e213, SUC_lexicon_tags_14 },
    { 0xa03e211, SUC_lexicon_tags_28 },
    { 0x1723e212, SUC_lexicon_tags_25 },
    { 0x61a3e216, SUC_lexicon_tags_196 },
    { 0, NULL },
    { 0x1783e218, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e3e21b, SUC_lexicon_tags_115 },
    { 0x40a3e21b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6c3e21e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2803e220, SUC_lexicon_tags_17 },
    { 0x84c3e220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce3e225, SUC_lexicon_tags_43 },
    { 0xe163e226, SUC_lexicon_tags_620 },
    { 0x9de3e227, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ec3e229, SUC_lexicon_tags_17 },
    { 0x2be3e22a, SUC_lexicon_tags_25 },
    { 0x27c3e22b, SUC_lexicon_tags_19 },
    { 0x9503e22a, SUC_lexicon_tags_15 },
    { 0x75c3e22d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de3e231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e3e235, SUC_lexicon_tags_13 },
    { 0x8543e236, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda83e238, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90e3e23a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x90e3e23c, SUC_lexicon_tags_23 },
    { 0x9a43e23d, SUC_lexicon_tags_1 },
    { 0xfd43e23d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9d03e240, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb843e244, SUC_lexicon_tags_8 },
    { 0x70a3e245, SUC_lexicon_tags_31 },
    { 0x6323e246, SUC_lexicon_tags_15 },
    { 0x4143e247, SUC_lexicon_tags_25 },
    { 0x17a3e246, SUC_lexicon_tags_12 },
    { 0x4d23e249, SUC_lexicon_tags_12 },
    { 0xa063e24a, SUC_lexicon_tags_37 },
    { 0xd403e247, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7b83e24d, SUC_lexicon_tags_12 },
    { 0xae43e24e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5243e255, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb283e257, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x9143e25a, SUC_lexicon_tags_56 },
    { 0x3c03e25a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c3e265, SUC_lexicon_tags_40 },
    { 0x6de3e266, SUC_lexicon_tags_1000 },
    { 0x7ac3e267, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e3e26c, SUC_lexicon_tags_14 },
    { 0x35c3e26d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce3e274, SUC_lexicon_tags_19 },
    { 0xe003e275, SUC_lexicon_tags_60 },
    { 0x4643e276, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x4823e279, SUC_lexicon_tags_38 },
    { 0xa683e279, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3803e281, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea63e285, SUC_lexicon_tags_27 },
    { 0x6ce3e286, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23e28a, SUC_lexicon_tags_55 },
    { 0xe8a3e28b, SUC_lexicon_tags_15 },
    { 0x5e63e28b, SUC_lexicon_tags_49 },
    { 0xa103e28c, SUC_lexicon_tags_15 },
    { 0xf583e28b, SUC_lexicon_tags_47 },
    { 0x1063e28b, SUC_lexicon_tags_25 },
    { 0x21a3e28e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf683e296, SUC_lexicon_tags_77 },
    { 0xfd43e296, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a3e29a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda43e2a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c3e2aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf403e2ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f23e2b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf843e2b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb443e2c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb103e2c7, SUC_lexicon_tags_37 },
    { 0x5e43e2c7, SUC_lexicon_tags_12 },
    { 0xf5e3e2c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecc3e2cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d43e2cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1743e2d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4523e2d3, SUC_lexicon_tags_21 },
    { 0x9263e2d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf823e2d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec43e2dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa3e2e3, SUC_lexicon_tags_26 },
    { 0x5cc3e2e4, SUC_lexicon_tags_12 },
    { 0x9563e2e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a3e2e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa463e2ec, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e3e2f2, SUC_lexicon_tags_49 },
    { 0x2b43e2f2, SUC_lexicon_tags_17 },
    { 0x26e3e2f3, SUC_lexicon_tags_25 },
    { 0x8c43e2f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc063e2f7, SUC_lexicon_tags_6 },
    { 0x9143e2f8, SUC_lexicon_tags_63 },
    { 0x163e2f9, SUC_lexicon_tags_0 },
    { 0x20a3e2f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac3e301, SUC_lexicon_tags_11 },
    { 0xb243e302, SUC_lexicon_tags_25 },
    { 0xa383e302, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c03e305, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e63e309, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec03e311, SUC_lexicon_tags_7 },
    { 0xe43e312, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be3e317, SUC_lexicon_tags_10 },
    { 0x7fe3e317, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2823e31a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x53e3e31c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a3e327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb583e329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc3e32c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x45a3e32e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27e3e330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb403e332, SUC_lexicon_tags_15 },
    { 0xc6e3e333, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5503e335, SUC_lexicon_tags_9 },
    { 0x9563e335, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa3e339, SUC_lexicon_tags_28 },
    { 0x6663e33a, SUC_lexicon_tags_21 },
    { 0x4883e33a, SUC_lexicon_tags_26 },
    { 0x6543e33c, SUC_lexicon_tags_21 },
    { 0x9a3e339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf423e33f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf43e344, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2763e349, SUC_lexicon_tags_11 },
    { 0x9583e34a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaae3e34c, SUC_lexicon_tags_56 },
    { 0x29a3e34c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf723e350, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd003e352, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e3e355, SUC_lexicon_tags_8 },
    { 0x2103e356, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9863e358, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x5383e35a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa523e35f, SUC_lexicon_tags_21 },
    { 0x4403e360, SUC_lexicon_tags_46 },
    { 0x7463e361, SUC_lexicon_tags_37 },
    { 0x60a3e360, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa763e367, SUC_lexicon_tags_47 },
    { 0x6463e367, SUC_lexicon_tags_12 },
    { 0x19e3e369, SUC_lexicon_tags_2 },
    { 0xa83e367, SUC_lexicon_tags_31 },
    { 0x9323e36b, SUC_lexicon_tags_23 },
    { 0xe323e36c, SUC_lexicon_tags_11 },
    { 0xe6c3e36d, SUC_lexicon_tags_7 },
    { 0x86e3e36e, SUC_lexicon_tags_7 },
    { 0xdf83e36f, SUC_lexicon_tags_25 },
    { 0x503e370, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d23e374, SUC_lexicon_tags_45 },
    { 0xa0e3e375, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b03e377, SUC_lexicon_tags_14 },
    { 0xec43e378, SUC_lexicon_tags_25 },
    { 0xb143e379, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a3e37c, SUC_lexicon_tags_37 },
    { 0xfea3e37d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec3e380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be3e383, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1403e385, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3683e387, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c63e38a, SUC_lexicon_tags_0 },
    { 0x79e3e38a, SUC_lexicon_tags_15 },
    { 0x1763e38b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c23e390, SUC_lexicon_tags_1 },
    { 0x2a03e390, SUC_lexicon_tags_25 },
    { 0xace3e391, SUC_lexicon_tags_14 },
    { 0x4443e393, SUC_lexicon_tags_2 },
    { 0xab63e393, SUC_lexicon_tags_70 },
    { 0xaf43e391, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a3e39b, SUC_lexicon_tags_25 },
    { 0xb323e39c, SUC_lexicon_tags_33 },
    { 0xf303e39d, SUC_lexicon_tags_47 },
    { 0xd843e39e, SUC_lexicon_tags_15 },
    { 0xe743e39e, SUC_lexicon_tags_19 },
    { 0xa7e3e39c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa043e3a3, SUC_lexicon_tags_46 },
    { 0x48c3e3a4, SUC_lexicon_tags_55 },
    { 0xc5c3e3a5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2803e3ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac03e3b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f23e3b2, SUC_lexicon_tags_60 },
    { 0xd6e3e3b3, SUC_lexicon_tags_2 },
    { 0xac03e3b2, SUC_lexicon_tags_7 },
    { 0x8ac3e3b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c03e3bc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b03e3c6, SUC_lexicon_tags_37 },
    { 0x7a83e3c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1003e3ca, SUC_lexicon_tags_19 },
    { 0x9963e3ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d83e3cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6cc3e3cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4803e3d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc3e3d9, SUC_lexicon_tags_7 },
    { 0xdfa3e3da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd623e3df, SUC_lexicon_tags_33 },
    { 0x9ac3e3e0, SUC_lexicon_tags_37 },
    { 0x8103e3e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7583e3e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x41c3e3e6, SUC_lexicon_tags_1 },
    { 0x6883e3e7, SUC_lexicon_tags_49 },
    { 0x5c03e3e6, SUC_lexicon_tags_23 },
    { 0xfca3e3e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbd23e3eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9723e3ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c03e3f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x283e3f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a3e3fa, SUC_lexicon_tags_1 },
    { 0x823e3fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2903e3fd, SUC_lexicon_tags_16 },
    { 0xda23e3fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da3e404, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6603e406, SUC_lexicon_tags_25 },
    { 0x73c3e406, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5643e40b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e3e40e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1063e419, SUC_lexicon_tags_29 },
    { 0x8f43e41a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b43e41c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f43e41e, SUC_lexicon_tags_47 },
    { 0x3943e41f, SUC_lexicon_tags_47 },
    { 0x9303e420, SUC_lexicon_tags_15 },
    { 0x2523e420, SUC_lexicon_tags_26 },
    { 0xa8e3e421, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6443e424, SUC_lexicon_tags_11 },
    { 0xdb03e425, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3223e428, SUC_lexicon_tags_25 },
    { 0x9423e428, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce3e42c, SUC_lexicon_tags_26 },
    { 0xa563e42d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc903e42f, SUC_lexicon_tags_30 },
    { 0xc0a3e430, SUC_lexicon_tags_25 },
    { 0xbee3e431, SUC_lexicon_tags_5 },
    { 0x4623e432, SUC_lexicon_tags_28 },
    { 0x8763e433, SUC_lexicon_tags_33 },
    { 0xab03e434, SUC_lexicon_tags_7 },
    { 0xa6c3e435, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a83e43a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03e440, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa23e443, SUC_lexicon_tags_32 },
    { 0x5643e443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x683e447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa823e45b, SUC_lexicon_tags_0 },
    { 0xbb23e45c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe3e45f, SUC_lexicon_tags_7 },
    { 0xa163e460, SUC_lexicon_tags_17 },
    { 0xcd23e45f, SUC_lexicon_tags_26 },
    { 0x8e23e462, SUC_lexicon_tags_12 },
    { 0x1f63e463, SUC_lexicon_tags_37 },
    { 0x26c3e464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b43e467, SUC_lexicon_tags_37 },
    { 0x7443e467, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8703e46a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf723e46e, SUC_lexicon_tags_15 },
    { 0xa1a3e46f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc3e478, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad43e47d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e3e482, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e83e486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ec3e488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c3e48b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e63e48d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8083e48f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0c3e491, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc063e493, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe23e497, SUC_lexicon_tags_0 },
    { 0x8783e498, SUC_lexicon_tags_29 },
    { 0x683e499, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3503e4a1, SUC_lexicon_tags_17 },
    { 0xabc3e4a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3783e4a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c23e4a8, SUC_lexicon_tags_12 },
    { 0x2603e4a8, SUC_lexicon_tags_23 },
    { 0xcc3e4aa, SUC_lexicon_tags_1 },
    { 0xe3c3e4ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa703e4ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf823e4b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa3e4b5, SUC_lexicon_tags_9 },
    { 0x1a23e4b6, SUC_lexicon_tags_87 },
    { 0xad03e4b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x383e4b9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c3e4bc, SUC_lexicon_tags_7 },
    { 0xf843e4bc, SUC_lexicon_tags_37 },
    { 0x9ec3e4be, SUC_lexicon_tags_11 },
    { 0xd083e4be, SUC_lexicon_tags_49 },
    { 0xea3e4c0, SUC_lexicon_tags_19 },
    { 0x8fe3e4c1, SUC_lexicon_tags_2 },
    { 0x5883e4c2, SUC_lexicon_tags_6 },
    { 0x9a3e4c1, SUC_lexicon_tags_47 },
    { 0xf143e4c4, SUC_lexicon_tags_11 },
    { 0x5683e4c5, SUC_lexicon_tags_26 },
    { 0xbec3e4c5, SUC_lexicon_tags_33 },
    { 0x9783e4c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8703e4c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c43e4cd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbda3e4cf, SUC_lexicon_tags_45 },
    { 0x8c63e4cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa3e4d6, SUC_lexicon_tags_0 },
    { 0x43c3e4d6, SUC_lexicon_tags_152 },
    { 0xae03e4d6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x603e4da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa143e4dc, SUC_lexicon_tags_17 },
    { 0xc9e3e4dd, SUC_lexicon_tags_12 },
    { 0x37a3e4de, SUC_lexicon_tags_17 },
    { 0x3e3e4dc, SUC_lexicon_tags_47 },
    { 0xf1e3e4e0, SUC_lexicon_tags_7 },
    { 0x5703e4e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7263e4e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x40e3e4e5, SUC_lexicon_tags_92 },
    { 0xbba3e4e6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xbb83e4e8, SUC_lexicon_tags_12 },
    { 0x97c3e4e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe323e4ec, SUC_lexicon_tags_17 },
    { 0xf943e4ed, SUC_lexicon_tags_31 },
    { 0x2d23e4ed, SUC_lexicon_tags_10 },
    { 0xc943e4ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa183e4f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7843e4f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fc3e4fa, SUC_lexicon_tags_16 },
    { 0x9e23e4fb, SUC_lexicon_tags_11 },
    { 0x1423e4fc, SUC_lexicon_tags_29 },
    { 0x8343e4fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52a3e4ff, SUC_lexicon_tags_54 },
    { 0xd9c3e500, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2763e503, SUC_lexicon_tags_11 },
    { 0x4a63e504, SUC_lexicon_tags_37 },
    { 0x883e505, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8283e507, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3243e509, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca43e50b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff63e50d, SUC_lexicon_tags_6 },
    { 0x2543e50e, SUC_lexicon_tags_0 },
    { 0x6223e50f, SUC_lexicon_tags_14 },
    { 0x26a3e50f, SUC_lexicon_tags_43 },
    { 0x3b03e511, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f23e514, SUC_lexicon_tags_12 },
    { 0x1343e514, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc03e517, SUC_lexicon_tags_10 },
    { 0x6503e518, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa3e51b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e03e51d, SUC_lexicon_tags_151 },
    { 0x9f03e51d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca03e524, SUC_lexicon_tags_9 },
    { 0xf663e525, SUC_lexicon_tags_40 },
    { 0x6783e526, SUC_lexicon_tags_27 },
    { 0x3c3e524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4003e52b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc523e52e, SUC_lexicon_tags_63 },
    { 0x22e3e52f, SUC_lexicon_tags_33 },
    { 0x9823e52f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa3e536, SUC_lexicon_tags_31 },
    { 0xb8e3e537, SUC_lexicon_tags_1 },
    { 0x3f43e538, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc8e3e53a, SUC_lexicon_tags_0 },
    { 0xd1c3e53b, SUC_lexicon_tags_25 },
    { 0x6e43e53b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a3e53f, SUC_lexicon_tags_61 },
    { 0x56e3e540, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a23e542, SUC_lexicon_tags_55 },
    { 0xf83e542, SUC_lexicon_tags_25 },
    { 0x3703e544, SUC_lexicon_tags_87 },
    { 0x9263e543, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf063e549, SUC_lexicon_tags_37 },
    { 0xc1a3e54a, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7543e54e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a3e552, SUC_lexicon_tags_31 },
    { 0xeaa3e553, SUC_lexicon_tags_37 },
    { 0x37e3e554, SUC_lexicon_tags_26 },
    { 0x1a83e555, SUC_lexicon_tags_38 },
    { 0xf063e554, SUC_lexicon_tags_7 },
    { 0xbfa3e553, SUC_lexicon_tags_45 },
    { 0x43e3e558, SUC_lexicon_tags_60 },
    { 0x9563e558, SUC_lexicon_tags_17 },
    { 0xa103e55a, SUC_lexicon_tags_27 },
    { 0x4503e55b, SUC_lexicon_tags_7 },
    { 0xcba3e559, SUC_lexicon_tags_11 },
    { 0x8403e557, SUC_lexicon_tags_14 },
    { 0xd4e3e55c, SUC_lexicon_tags_2 },
    { 0xfbe3e55a, SUC_lexicon_tags_15 },
    { 0xf523e55d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f03e563, SUC_lexicon_tags_4 },
    { 0x2c3e564, SUC_lexicon_tags_14 },
    { 0xec3e565, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x423e568, SUC_lexicon_tags_2 },
    { 0xc583e569, SUC_lexicon_tags_33 },
    { 0xd343e56a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xece3e56d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8143e571, SUC_lexicon_tags_11 },
    { 0x4323e572, SUC_lexicon_tags_11 },
    { 0x4e23e573, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e3e57a, SUC_lexicon_tags_25 },
    { 0xd0c3e57b, SUC_lexicon_tags_45 },
    { 0x1e83e57c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa423e580, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4943e587, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a83e589, SUC_lexicon_tags_31 },
    { 0xf1a3e58a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x363e58d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a3e590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a3e595, SUC_lexicon_tags_37 },
    { 0xb883e595, SUC_lexicon_tags_11 },
    { 0xdd63e597, SUC_lexicon_tags_19 },
    { 0xafc3e595, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2103e59b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7a3e59d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6823e59f, SUC_lexicon_tags_45 },
    { 0x6963e5a0, SUC_lexicon_tags_60 },
    { 0xf703e5a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543e5a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b43e5a7, SUC_lexicon_tags_61 },
    { 0x143e5a7, SUC_lexicon_tags_8 },
    { 0xeee3e5a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d03e5ab, SUC_lexicon_tags_21 },
    { 0x7303e5ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3e5b0, SUC_lexicon_tags_57 },
    { 0x4303e5b1, SUC_lexicon_tags_12 },
    { 0xc6a3e5b0, SUC_lexicon_tags_25 },
    { 0x5f63e5b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc3e5b7, SUC_lexicon_tags_57 },
    { 0xe443e5b8, SUC_lexicon_tags_6 },
    { 0xa063e5b9, SUC_lexicon_tags_0 },
    { 0x1b03e5b8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e3e5c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa883e5c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc63e5c5, SUC_lexicon_tags_38 },
    { 0x56e3e5c5, SUC_lexicon_tags_31 },
    { 0xd323e5c7, SUC_lexicon_tags_37 },
    { 0xd703e5c7, SUC_lexicon_tags_37 },
    { 0x9e03e5c8, SUC_lexicon_tags_3 },
    { 0xd2a3e5c7, SUC_lexicon_tags_17 },
    { 0xb423e5c9, SUC_lexicon_tags_35 },
    { 0xe723e5cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4783e5d0, SUC_lexicon_tags_14 },
    { 0xc883e5d1, SUC_lexicon_tags_12 },
    { 0x6223e5d1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3243e5d6, SUC_lexicon_tags_37 },
    { 0x4d03e5d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc663e5db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a3e5de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa843e5e1, SUC_lexicon_tags_37 },
    { 0xa523e5e1, SUC_lexicon_tags_9 },
    { 0xb423e5e3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xc303e5e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac3e5ed, SUC_lexicon_tags_40 },
    { 0x25e3e5ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1783e5f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a3e5f6, SUC_lexicon_tags_38 },
    { 0x5283e5f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5603e5fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8623e5fc, SUC_lexicon_tags_1 },
    { 0x8323e5fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a3e603, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3c3e605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6923e60c, SUC_lexicon_tags_23 },
    { 0xad63e60d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a3e610, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a3e613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d43e620, SUC_lexicon_tags_25 },
    { 0x10e3e620, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee3e625, SUC_lexicon_tags_12 },
    { 0xd603e626, SUC_lexicon_tags_37 },
    { 0xd983e627, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8423e629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde03e62b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c03e630, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef63e633, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea43e635, SUC_lexicon_tags_7 },
    { 0x68e3e635, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa3e639, SUC_lexicon_tags_27 },
    { 0x32a3e63a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa563e640, SUC_lexicon_tags_63 },
    { 0x6f43e641, SUC_lexicon_tags_38 },
    { 0xf2a3e642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb743e644, SUC_lexicon_tags_9 },
    { 0xf603e645, SUC_lexicon_tags_7 },
    { 0x9743e646, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d23e648, SUC_lexicon_tags_2 },
    { 0x6ce3e649, SUC_lexicon_tags_7 },
    { 0xf343e64a, SUC_lexicon_tags_30 },
    { 0xf2a3e64b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8683e650, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2223e653, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7563e655, SUC_lexicon_tags_15 },
    { 0x9f43e656, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9663e658, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a3e65c, SUC_lexicon_tags_45 },
    { 0xfc63e65d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae43e661, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e3e664, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc223e66f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x96a3e671, SUC_lexicon_tags_2 },
    { 0x1f43e672, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8783e675, SUC_lexicon_tags_29 },
    { 0x3c43e676, SUC_lexicon_tags_87 },
    { 0xf743e677, SUC_lexicon_tags_12 },
    { 0xd463e675, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x70a3e67a, SUC_lexicon_tags_39 },
    { 0x83c3e67a, SUC_lexicon_tags_11 },
    { 0xa683e67b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd743e67e, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be3e682, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8403e684, SUC_lexicon_tags_47 },
    { 0x2483e685, SUC_lexicon_tags_63 },
    { 0x79e3e685, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e03e68d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1483e692, SUC_lexicon_tags_17 },
    { 0xf7e3e693, SUC_lexicon_tags_7 },
    { 0x7623e693, SUC_lexicon_tags_8 },
    { 0xa363e694, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa603e698, SUC_lexicon_tags_25 },
    { 0x1c63e699, SUC_lexicon_tags_17 },
    { 0xd983e69a, SUC_lexicon_tags_1 },
    { 0x9ca3e69b, SUC_lexicon_tags_61 },
    { 0x98e3e69c, SUC_lexicon_tags_3 },
    { 0xce63e69d, SUC_lexicon_tags_15 },
    { 0x1243e69d, SUC_lexicon_tags_33 },
    { 0xf783e69f, SUC_lexicon_tags_0 },
    { 0x7603e69e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x143e6a3, SUC_lexicon_tags_15 },
    { 0xda03e6a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb23e6a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc583e6ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e3e6b6, SUC_lexicon_tags_27 },
    { 0x35a3e6b6, SUC_lexicon_tags_39 },
    { 0xf2e3e6b7, SUC_lexicon_tags_25 },
    { 0xf063e6b6, SUC_lexicon_tags_17 },
    { 0xaba3e6b8, SUC_lexicon_tags_2 },
    { 0x8983e6bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c3e6c0, SUC_lexicon_tags_16 },
    { 0xa0c3e6c1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade3e6c6, SUC_lexicon_tags_17 },
    { 0x80c3e6c6, SUC_lexicon_tags_47 },
    { 0x2723e6c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1f83e6ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec03e6d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d83e6d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb283e6d5, SUC_lexicon_tags_15 },
    { 0xb863e6d5, SUC_lexicon_tags_14 },
    { 0xe403e6d7, SUC_lexicon_tags_3 },
    { 0xbe3e6d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9363e6da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x223e6dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5283e6e3, SUC_lexicon_tags_19 },
    { 0x9483e6e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e43e6e9, SUC_lexicon_tags_12 },
    { 0xc803e6ea, SUC_lexicon_tags_57 },
    { 0x5723e6ea, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x6823e6ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d03e6f0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e3e6f5, SUC_lexicon_tags_31 },
    { 0xea3e6f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x56a3e6f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x42e3e6fa, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0x3603e6fc, SUC_lexicon_tags_8 },
    { 0x39e3e6fd, SUC_lexicon_tags_14 },
    { 0xe723e6fe, SUC_lexicon_tags_7 },
    { 0x6e3e6ff, SUC_lexicon_tags_43 },
    { 0x8883e700, SUC_lexicon_tags_15 },
    { 0x8a3e6ff, SUC_lexicon_tags_25 },
    { 0xc03e700, SUC_lexicon_tags_82 },
    { 0x4e63e6ff, SUC_lexicon_tags_25 },
    { 0x55a3e703, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8523e706, SUC_lexicon_tags_17 },
    { 0xd003e707, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e23e70b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6703e70d, SUC_lexicon_tags_47 },
    { 0xb1a3e70d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3e715, SUC_lexicon_tags_63 },
    { 0xcd63e715, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa983e718, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e23e71e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b63e721, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c83e723, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa23e728, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e3e72a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf523e72c, SUC_lexicon_tags_26 },
    { 0x6c03e72d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x123e72f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d63e731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce3e734, SUC_lexicon_tags_17 },
    { 0xed63e735, SUC_lexicon_tags_11 },
    { 0x44a3e736, SUC_lexicon_tags_63 },
    { 0x3e3e737, SUC_lexicon_tags_38 },
    { 0x1243e736, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee63e73e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2343e743, SUC_lexicon_tags_35 },
    { 0xeee3e744, SUC_lexicon_tags_1 },
    { 0xb003e745, SUC_lexicon_tags_51 },
    { 0xe4c3e746, SUC_lexicon_tags_37 },
    { 0xeb23e747, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafa3e749, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea3e74c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7343e74f, SUC_lexicon_tags_6 },
    { 0x943e750, SUC_lexicon_tags_25 },
    { 0x3cc3e751, SUC_lexicon_tags_2 },
    { 0xff63e752, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba63e754, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de3e759, SUC_lexicon_tags_30 },
    { 0x1623e75a, SUC_lexicon_tags_60 },
    { 0xed83e75b, SUC_lexicon_tags_2 },
    { 0x7d63e75c, SUC_lexicon_tags_47 },
    { 0x59c3e759, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7563e75f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc983e762, SUC_lexicon_tags_27 },
    { 0x2163e763, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xab63e766, SUC_lexicon_tags_23 },
    { 0xc03e767, SUC_lexicon_tags_1 },
    { 0x62e3e766, SUC_lexicon_tags_10 },
    { 0x4283e767, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc423e76c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc03e771, SUC_lexicon_tags_25 },
    { 0x4103e772, SUC_lexicon_tags_35 },
    { 0xc143e773, SUC_lexicon_tags_28 },
    { 0x6ac3e773, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc1e3e776, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a3e77a, SUC_lexicon_tags_12 },
    { 0xcf63e77b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40a3e77d, SUC_lexicon_tags_55 },
    { 0xe263e77d, SUC_lexicon_tags_116 },
    { 0x3d23e77f, SUC_lexicon_tags_9 },
    { 0x2463e780, SUC_lexicon_tags_11 },
    { 0xe6c3e781, SUC_lexicon_tags_37 },
    { 0x9463e77e, SUC_lexicon_tags_0 },
    { 0xb5a3e783, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd423e789, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8163e78b, SUC_lexicon_tags_31 },
    { 0x6d43e78b, SUC_lexicon_tags_19 },
    { 0x7943e78d, SUC_lexicon_tags_11 },
    { 0xc8c3e78e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab03e792, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf43e795, SUC_lexicon_tags_44 },
    { 0xac3e796, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5643e79a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e3e7a0, SUC_lexicon_tags_1 },
    { 0x5583e7a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e3e7a8, SUC_lexicon_tags_51 },
    { 0xafc3e7a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8163e7ab, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7de3e7ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4523e7b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe183e7b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7503e7b6, SUC_lexicon_tags_8 },
    { 0xecc3e7b7, SUC_lexicon_tags_55 },
    { 0xba3e7b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc3e7be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc3e7c3, SUC_lexicon_tags_201 },
    { 0x7663e7c4, SUC_lexicon_tags_47 },
    { 0xca43e7c3, SUC_lexicon_tags_900 },
    { 0xf403e7c6, SUC_lexicon_tags_0 },
    { 0xd143e7c7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3e7cb, SUC_lexicon_tags_41 },
    { 0x1a3e7cb, SUC_lexicon_tags_15 },
    { 0x4f83e7cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea43e7d3, SUC_lexicon_tags_14 },
    { 0xa623e7d4, SUC_lexicon_tags_0 },
    { 0x5943e7d3, SUC_lexicon_tags_14 },
    { 0xe623e7d6, SUC_lexicon_tags_19 },
    { 0xc3e3e7d7, SUC_lexicon_tags_37 },
    { 0x43c3e7d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf63e7da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c3e7dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e3e7e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c83e7e9, SUC_lexicon_tags_31 },
    { 0x3de3e7ea, SUC_lexicon_tags_16 },
    { 0x9e03e7e9, SUC_lexicon_tags_25 },
    { 0x8503e7ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7863e7ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b63e7f4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa3e7f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe843e7fb, SUC_lexicon_tags_17 },
    { 0x1243e7fc, SUC_lexicon_tags_7 },
    { 0x5ae3e7fc, SUC_lexicon_tags_17 },
    { 0xaba3e7fb, SUC_lexicon_tags_7 },
    { 0xeca3e7fd, SUC_lexicon_tags_55 },
    { 0x7b23e800, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec3e802, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb943e806, SUC_lexicon_tags_55 },
    { 0x54e3e807, SUC_lexicon_tags_39 },
    { 0xbc3e808, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x52a3e80a, SUC_lexicon_tags_1 },
    { 0xcda3e80b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b03e80f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa823e813, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78e3e815, SUC_lexicon_tags_136 },
    { 0xe2c3e815, SUC_lexicon_tags_11 },
    { 0x783e817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23c3e819, SUC_lexicon_tags_11 },
    { 0xe5c3e819, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8643e821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f23e825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2163e831, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc303e837, SUC_lexicon_tags_91 },
    { 0x1283e838, SUC_lexicon_tags_38 },
    { 0xbf23e837, SUC_lexicon_tags_28 },
    { 0x76e3e838, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4163e840, SUC_lexicon_tags_17 },
    { 0x6f83e841, SUC_lexicon_tags_45 },
    { 0x3f63e840, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa623e844, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a3e849, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f83e84b, SUC_lexicon_tags_66 },
    { 0x7ec3e84b, SUC_lexicon_tags_47 },
    { 0xc2c3e84d, SUC_lexicon_tags_27 },
    { 0xcd23e84e, SUC_lexicon_tags_11 },
    { 0x5be3e84e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e3e851, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc83e856, SUC_lexicon_tags_11 },
    { 0x4963e857, SUC_lexicon_tags_49 },
    { 0xba03e856, SUC_lexicon_tags_127 },
    { 0x9703e858, SUC_lexicon_tags_14 },
    { 0x8523e857, SUC_lexicon_tags_7 },
    { 0x7c63e85b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a3e85d, SUC_lexicon_tags_10 },
    { 0xb143e85e, SUC_lexicon_tags_48 },
    { 0xa7a3e85f, SUC_lexicon_tags_83 },
    { 0x6c3e85d, SUC_lexicon_tags_2 },
    { 0xb6c3e85e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2963e864, SUC_lexicon_tags_11 },
    { 0xc1c3e865, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a3e86c, SUC_lexicon_tags_2 },
    { 0xada3e86c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb483e873, SUC_lexicon_tags_47 },
    { 0x90e3e873, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7123e877, SUC_lexicon_tags_45 },
    { 0x3203e877, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb543e87e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc843e884, SUC_lexicon_tags_4 },
    { 0x64a3e884, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9003e892, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca3e896, SUC_lexicon_tags_2 },
    { 0x8183e896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da3e89a, SUC_lexicon_tags_28 },
    { 0xb203e89a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a3e89e, SUC_lexicon_tags_25 },
    { 0x8903e89e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a3e8a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x503e8a5, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583e8b0, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea83e8b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe03e8bc, SUC_lexicon_tags_12 },
    { 0xa9c3e8bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e3e8c1, SUC_lexicon_tags_60 },
    { 0x67c3e8c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43e8c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8a3e8c7, SUC_lexicon_tags_37 },
    { 0xdea3e8c8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x67e3e8ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2203e8cd, SUC_lexicon_tags_1 },
    { 0xa523e8ce, SUC_lexicon_tags_2 },
    { 0x70c3e8cf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1743e8d6, SUC_lexicon_tags_15 },
    { 0xbfa3e8d7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeda3e8d9, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8143e8de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5be3e8e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x14a3e8e3, SUC_lexicon_tags_2 },
    { 0xca63e8e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603e8e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a3e8ec, SUC_lexicon_tags_38 },
    { 0x94a3e8ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa543e8f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2523e8f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x863e8f6, SUC_lexicon_tags_23 },
    { 0x3ea3e8f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5063e8f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea3e8fd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xff63e900, SUC_lexicon_tags_63 },
    { 0x1b83e901, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xca03e903, SUC_lexicon_tags_63 },
    { 0x3323e903, SUC_lexicon_tags_15 },
    { 0x36a3e905, SUC_lexicon_tags_60 },
    { 0x6b63e903, SUC_lexicon_tags_21 },
    { 0x7e03e907, SUC_lexicon_tags_47 },
    { 0x6943e908, SUC_lexicon_tags_4 },
    { 0x4743e908, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbfc3e90b, SUC_lexicon_tags_1 },
    { 0xa063e90c, SUC_lexicon_tags_25 },
    { 0x2ec3e90c, SUC_lexicon_tags_85 },
    { 0x6983e90e, SUC_lexicon_tags_47 },
    { 0x3d03e90b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8523e912, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7843e91a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8823e91c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2163e921, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3e927, SUC_lexicon_tags_32 },
    { 0x32a3e927, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1783e92b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8a83e92d, SUC_lexicon_tags_31 },
    { 0xf543e92e, SUC_lexicon_tags_1 },
    { 0x7083e92f, SUC_lexicon_tags_15 },
    { 0x6fa3e930, SUC_lexicon_tags_37 },
    { 0x71c3e930, SUC_lexicon_tags_8 },
    { 0x21c3e932, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebe3e934, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe743e939, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1143e93f, SUC_lexicon_tags_55 },
    { 0x6263e940, SUC_lexicon_tags_63 },
    { 0x7be3e940, SUC_lexicon_tags_2 },
    { 0xf23e942, SUC_lexicon_tags_11 },
    { 0xe4c3e941, SUC_lexicon_tags_25 },
    { 0xda03e943, SUC_lexicon_tags_15 },
    { 0x1383e942, SUC_lexicon_tags_6 },
    { 0x2f23e93f, SUC_lexicon_tags_11 },
    { 0xbe3e947, SUC_lexicon_tags_7 },
    { 0x7f63e948, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa523e94d, SUC_lexicon_tags_37 },
    { 0x2263e94d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf203e952, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x1ca3e954, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4383e957, SUC_lexicon_tags_17 },
    { 0xaa23e957, SUC_lexicon_tags_12 },
    { 0x94a3e959, SUC_lexicon_tags_31 },
    { 0xf3c3e95a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be3e95d, SUC_lexicon_tags_33 },
    { 0x6d63e95e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa583e962, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd563e964, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23e96a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xad23e96c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4983e977, SUC_lexicon_tags_49 },
    { 0xb3a3e978, SUC_lexicon_tags_39 },
    { 0xa8a3e977, SUC_lexicon_tags_11 },
    { 0x64c3e977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a3e97d, SUC_lexicon_tags_15 },
    { 0xfba3e97e, SUC_lexicon_tags_2 },
    { 0x76c3e97e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc3e982, SUC_lexicon_tags_7 },
    { 0xf463e983, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac03e988, SUC_lexicon_tags_17 },
    { 0xa603e988, SUC_lexicon_tags_27 },
    { 0x6343e98a, SUC_lexicon_tags_7 },
    { 0x12a3e98a, SUC_lexicon_tags_0 },
    { 0x71a3e98a, SUC_lexicon_tags_48 },
    { 0x8a43e98c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b63e991, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1443e993, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea3e99a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde83e99d, SUC_lexicon_tags_17 },
    { 0xcf63e99d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2c03e9a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa23e9a5, SUC_lexicon_tags_31 },
    { 0xdc03e9a6, SUC_lexicon_tags_8 },
    { 0x7ca3e9a5, SUC_lexicon_tags_43 },
    { 0x5d23e9a7, SUC_lexicon_tags_39 },
    { 0x5c43e9a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8483e9ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8003e9b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c3e9b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e3e9b7, SUC_lexicon_tags_11 },
    { 0xa223e9b8, SUC_lexicon_tags_28 },
    { 0xd2a3e9b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3083e9bb, SUC_lexicon_tags_7 },
    { 0x4063e9bb, SUC_lexicon_tags_12 },
    { 0x403e9bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd663e9bf, SUC_lexicon_tags_1 },
    { 0x4643e9c0, SUC_lexicon_tags_37 },
    { 0xfa83e9c1, SUC_lexicon_tags_17 },
    { 0x2ca3e9c1, SUC_lexicon_tags_15 },
    { 0x9363e9c3, SUC_lexicon_tags_29 },
    { 0x5803e9c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb683e9c7, SUC_lexicon_tags_33 },
    { 0x5d23e9c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e9cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4623e9d1, SUC_lexicon_tags_47 },
    { 0x1e63e9d2, SUC_lexicon_tags_37 },
    { 0xd83e9d3, SUC_lexicon_tags_6 },
    { 0xad23e9d4, SUC_lexicon_tags_61 },
    { 0x79a3e9d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa183e9da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd243e9df, SUC_lexicon_tags_11 },
    { 0x1c83e9e0, SUC_lexicon_tags_17 },
    { 0x4203e9df, SUC_lexicon_tags_43 },
    { 0x8863e9df, SUC_lexicon_tags_60 },
    { 0xa5a3e9e3, SUC_lexicon_tags_167 },
    { 0x77e3e9e3, SUC_lexicon_tags_31 },
    { 0xf243e9e0, SUC_lexicon_tags_117 },
    { 0xd23e9e6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf1c3e9e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9983e9eb, SUC_lexicon_tags_17 },
    { 0x11c3e9ec, SUC_lexicon_tags_82 },
    { 0xec83e9ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5463e9f0, SUC_lexicon_tags_11 },
    { 0x4a03e9f1, SUC_lexicon_tags_250 },
    { 0xea03e9f1, SUC_lexicon_tags_38 },
    { 0x84e3e9f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7343e9f6, SUC_lexicon_tags_2 },
    { 0x9383e9f7, SUC_lexicon_tags_2 },
    { 0xdf03e9f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf63e9fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643ea03, SUC_lexicon_tags_25 },
    { 0xb883ea04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e3ea0a, SUC_lexicon_tags_47 },
    { 0xb543ea0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea63ea0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be3ea10, SUC_lexicon_tags_25 },
    { 0x26e3ea11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9363ea14, SUC_lexicon_tags_15 },
    { 0x89a3ea15, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e3ea1e, SUC_lexicon_tags_35 },
    { 0xa843ea1e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e3ea22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdea3ea24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3ea27, SUC_lexicon_tags_27 },
    { 0x3343ea28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14c3ea2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb483ea2e, SUC_lexicon_tags_12 },
    { 0xa963ea2e, SUC_lexicon_tags_37 },
    { 0x5d03ea2e, SUC_lexicon_tags_25 },
    { 0x1983ea2f, SUC_lexicon_tags_11 },
    { 0x5e63ea32, SUC_lexicon_tags_6 },
    { 0x11c3ea32, SUC_lexicon_tags_11 },
    { 0xcd23ea33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9763ea38, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1903ea3d, SUC_lexicon_tags_25 },
    { 0x4f23ea3d, SUC_lexicon_tags_15 },
    { 0xa1a3ea3e, SUC_lexicon_tags_3 },
    { 0xa163ea40, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a03ea45, SUC_lexicon_tags_7 },
    { 0xcb43ea45, SUC_lexicon_tags_84 },
    { 0xce83ea47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae3ea4a, SUC_lexicon_tags_11 },
    { 0x1763ea4b, SUC_lexicon_tags_37 },
    { 0x2c63ea4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e3ea4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce03ea51, SUC_lexicon_tags_11 },
    { 0xc563ea52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fe3ea54, SUC_lexicon_tags_55 },
    { 0xc783ea55, SUC_lexicon_tags_37 },
    { 0xf223ea56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69c3ea58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb63ea5a, SUC_lexicon_tags_1 },
    { 0x883ea5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc843ea5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1183ea62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x90e3ea64, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xda83ea67, SUC_lexicon_tags_19 },
    { 0x3d03ea67, SUC_lexicon_tags_37 },
    { 0xbf23ea69, SUC_lexicon_tags_15 },
    { 0x9a83ea6a, SUC_lexicon_tags_37 },
    { 0x6e03ea69, SUC_lexicon_tags_22 },
    { 0xbee3ea69, SUC_lexicon_tags_12 },
    { 0xbd83ea6d, SUC_lexicon_tags_17 },
    { 0x5063ea6e, SUC_lexicon_tags_60 },
    { 0xa4e3ea6e, SUC_lexicon_tags_29 },
    { 0x5c83ea6b, SUC_lexicon_tags_30 },
    { 0xc4c3ea6f, SUC_lexicon_tags_14 },
    { 0x5c43ea72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e3ea76, SUC_lexicon_tags_17 },
    { 0x7b43ea77, SUC_lexicon_tags_17 },
    { 0x3083ea77, SUC_lexicon_tags_30 },
    { 0xca83ea79, SUC_lexicon_tags_29 },
    { 0xa403ea79, SUC_lexicon_tags_25 },
    { 0xb1a3ea7b, SUC_lexicon_tags_15 },
    { 0xf623ea7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1823ea7e, SUC_lexicon_tags_38 },
    { 0x6003ea7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa3ea84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbb63ea86, SUC_lexicon_tags_5 },
    { 0x6223ea86, SUC_lexicon_tags_2 },
    { 0xd243ea88, SUC_lexicon_tags_10 },
    { 0x3d43ea89, SUC_lexicon_tags_53 },
    { 0xe203ea8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe643ea8c, SUC_lexicon_tags_8 },
    { 0x3923ea8c, SUC_lexicon_tags_33 },
    { 0x6a63ea8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8a3ea90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2843ea95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafe3ea97, SUC_lexicon_tags_7 },
    { 0xe743ea97, SUC_lexicon_tags_40 },
    { 0xf9c3ea98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf403ea9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa243eaa4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c83eaa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea03eaac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7be3eaae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd043eab0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7983eab3, SUC_lexicon_tags_7 },
    { 0x6123eab3, SUC_lexicon_tags_16 },
    { 0x31c3eab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e83eab8, SUC_lexicon_tags_8 },
    { 0x3383eab8, SUC_lexicon_tags_25 },
    { 0x5343eaba, SUC_lexicon_tags_12 },
    { 0x3b03eab9, SUC_lexicon_tags_12 },
    { 0xb223eabc, SUC_lexicon_tags_17 },
    { 0xe103eabd, SUC_lexicon_tags_12 },
    { 0xc183eabe, SUC_lexicon_tags_11 },
    { 0x7163eabe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb743eac1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x72a3eac3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaec3eac5, SUC_lexicon_tags_11 },
    { 0x1ca3eac6, SUC_lexicon_tags_14 },
    { 0x8e03eac6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e3eace, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa83ead0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6763ead3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7583ead7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd283eadc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5183eade, SUC_lexicon_tags_7 },
    { 0x1703eadf, SUC_lexicon_tags_15 },
    { 0xdce3eae0, SUC_lexicon_tags_23 },
    { 0x6303eae1, SUC_lexicon_tags_15 },
    { 0x94a3eae2, SUC_lexicon_tags_47 },
    { 0x4a23eae2, SUC_lexicon_tags_37 },
    { 0xd5a3eae4, SUC_lexicon_tags_11 },
    { 0xab63eae4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfde3eae7, SUC_lexicon_tags_11 },
    { 0x9cc3eae7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff43eaea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3eaef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd263eaf1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2223eaf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e3eafa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7a3eafc, SUC_lexicon_tags_39 },
    { 0x743eafd, SUC_lexicon_tags_31 },
    { 0x7a03eafe, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xcf43eb00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3eb05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b63eb07, SUC_lexicon_tags_47 },
    { 0x8c63eb08, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7c83eb0a, SUC_lexicon_tags_25 },
    { 0xeae3eb0b, SUC_lexicon_tags_30 },
    { 0xdc23eb0c, SUC_lexicon_tags_229 },
    { 0x483eb0d, SUC_lexicon_tags_15 },
    { 0xab03eb0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ca3eb10, SUC_lexicon_tags_28 },
    { 0xf743eb11, SUC_lexicon_tags_15 },
    { 0xf9e3eb11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9363eb16, SUC_lexicon_tags_112 },
    { 0x9ce3eb16, SUC_lexicon_tags_27 },
    { 0x8d23eb18, SUC_lexicon_tags_15 },
    { 0xf2c3eb19, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x4123eb1b, SUC_lexicon_tags_19 },
    { 0x3b03eb1c, SUC_lexicon_tags_13 },
    { 0xab43eb1d, SUC_lexicon_tags_17 },
    { 0xa683eb1e, SUC_lexicon_tags_25 },
    { 0x2e23eb1f, SUC_lexicon_tags_33 },
    { 0xbd43eb1f, SUC_lexicon_tags_39 },
    { 0xc403eb1e, SUC_lexicon_tags_2 },
    { 0xfb43eb1c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd623eb2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca3eb30, SUC_lexicon_tags_60 },
    { 0x4aa3eb31, SUC_lexicon_tags_2 },
    { 0x2663eb32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa3eb35, SUC_lexicon_tags_65 },
    { 0xe1c3eb36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3eb43, SUC_lexicon_tags_1 },
    { 0xdaa3eb43, SUC_lexicon_tags_37 },
    { 0x59a3eb45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea63eb48, SUC_lexicon_tags_28 },
    { 0x3c83eb49, SUC_lexicon_tags_227 },
    { 0x4483eb4a, SUC_lexicon_tags_37 },
    { 0x1ac3eb4b, SUC_lexicon_tags_25 },
    { 0xe803eb4b, SUC_lexicon_tags_63 },
    { 0xed43eb4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b63eb57, SUC_lexicon_tags_11 },
    { 0xd5c3eb58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e3eb5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e3eb60, SUC_lexicon_tags_30 },
    { 0xc983eb60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd23eb63, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb363eb67, SUC_lexicon_tags_63 },
    { 0xcde3eb68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c3eb6d, SUC_lexicon_tags_31 },
    { 0xe643eb6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa1a3eb70, SUC_lexicon_tags_17 },
    { 0x4323eb71, SUC_lexicon_tags_7 },
    { 0x203eb71, SUC_lexicon_tags_12 },
    { 0x3783eb70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1903eb7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf863eb81, SUC_lexicon_tags_11 },
    { 0xcec3eb82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a3eb8a, SUC_lexicon_tags_10 },
    { 0x8d83eb8b, SUC_lexicon_tags_37 },
    { 0x27a3eb8a, SUC_lexicon_tags_28 },
    { 0x7fa3eb8d, SUC_lexicon_tags_212 },
    { 0x11a3eb8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de3eb97, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43eb9b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3643eb9d, SUC_lexicon_tags_2 },
    { 0xeba3eb9e, SUC_lexicon_tags_12 },
    { 0x4623eb9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c83eba3, SUC_lexicon_tags_8 },
    { 0xd823eba3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c3eba7, SUC_lexicon_tags_2 },
    { 0x3443eba8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6f43ebaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe3ebae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc863ebb1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd03ebb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb143ebb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1503ebc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3ebc7, SUC_lexicon_tags_46 },
    { 0x6f63ebc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3ebcc, SUC_lexicon_tags_3 },
    { 0x7563ebcd, SUC_lexicon_tags_17 },
    { 0x20e3ebce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xece3ebd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e43ebd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa863ebdc, SUC_lexicon_tags_39 },
    { 0xd063ebdd, SUC_lexicon_tags_12 },
    { 0x5883ebdc, SUC_lexicon_tags_12 },
    { 0x5b43ebdf, SUC_lexicon_tags_37 },
    { 0x9683ebe0, SUC_lexicon_tags_25 },
    { 0x683ebe1, SUC_lexicon_tags_25 },
    { 0x4363ebe2, SUC_lexicon_tags_4 },
    { 0x9623ebe3, SUC_lexicon_tags_17 },
    { 0xffe3ebe0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc63ebe6, SUC_lexicon_tags_37 },
    { 0xd8c3ebe7, SUC_lexicon_tags_9 },
    { 0xaee3ebe8, SUC_lexicon_tags_56 },
    { 0xca43ebe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee3ebed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e3ebf0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23ebf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1243ebf6, SUC_lexicon_tags_12 },
    { 0xec83ebf7, SUC_lexicon_tags_4 },
    { 0x7ea3ebf8, SUC_lexicon_tags_17 },
    { 0x4c23ebf9, SUC_lexicon_tags_17 },
    { 0xf63ebfa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a3ec02, SUC_lexicon_tags_7 },
    { 0x51a3ec03, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe423ec07, SUC_lexicon_tags_48 },
    { 0x2dc3ec08, SUC_lexicon_tags_61 },
    { 0x1943ec09, SUC_lexicon_tags_11 },
    { 0xac43ec08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1923ec0d, SUC_lexicon_tags_7 },
    { 0x6f03ec0e, SUC_lexicon_tags_7 },
    { 0x5283ec0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a3ec14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e3ec17, SUC_lexicon_tags_17 },
    { 0xb7e3ec17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb923ec1a, SUC_lexicon_tags_0 },
    { 0x2d63ec1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf83ec1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f83ec28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4a3ec2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2343ec2e, SUC_lexicon_tags_14 },
    { 0xd7c3ec2f, SUC_lexicon_tags_3 },
    { 0x5d43ec2f, SUC_lexicon_tags_33 },
    { 0xb03ec31, SUC_lexicon_tags_17 },
    { 0x8163ec32, SUC_lexicon_tags_26 },
    { 0x28e3ec33, SUC_lexicon_tags_12 },
    { 0x3be3ec2f, SUC_lexicon_tags_25 },
    { 0x3f43ec30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3ba3ec37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6583ec3a, SUC_lexicon_tags_14 },
    { 0xcb23ec3b, SUC_lexicon_tags_13 },
    { 0x5603ec3c, SUC_lexicon_tags_2 },
    { 0xadc3ec3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe3ec41, SUC_lexicon_tags_17 },
    { 0xd123ec42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf043ec49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd183ec4c, SUC_lexicon_tags_25 },
    { 0x2de3ec4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb043ec51, SUC_lexicon_tags_39 },
    { 0x4463ec52, SUC_lexicon_tags_9 },
    { 0xd5e3ec51, SUC_lexicon_tags_15 },
    { 0xda83ec54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5263ec56, SUC_lexicon_tags_17 },
    { 0xde23ec56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf03ec5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeda3ec5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a3ec61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9643ec69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44a3ec6b, SUC_lexicon_tags_2 },
    { 0xa9c3ec6c, SUC_lexicon_tags_15 },
    { 0xf963ec6d, SUC_lexicon_tags_11 },
    { 0x7a03ec6b, SUC_lexicon_tags_201 },
    { 0xe5a3ec6f, SUC_lexicon_tags_17 },
    { 0x13a3ec70, SUC_lexicon_tags_33 },
    { 0xc503ec70, SUC_lexicon_tags_11 },
    { 0xd803ec6b, SUC_lexicon_tags_31 },
    { 0xc023ec73, SUC_lexicon_tags_37 },
    { 0xf463ec74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c3ec79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa423ec7b, SUC_lexicon_tags_49 },
    { 0x5723ec7c, SUC_lexicon_tags_138 },
    { 0x3b43ec7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03ec88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1a3ec8a, SUC_lexicon_tags_12 },
    { 0xc983ec8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa443ec94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc3ec9c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x32e3ec9e, SUC_lexicon_tags_7 },
    { 0xa943ec9e, SUC_lexicon_tags_11 },
    { 0xfc63eca0, SUC_lexicon_tags_13 },
    { 0x4423eca0, SUC_lexicon_tags_12 },
    { 0xc963eca2, SUC_lexicon_tags_61 },
    { 0x3123eca1, SUC_lexicon_tags_0 },
    { 0x12e3ec9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2423eca6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7863eca9, SUC_lexicon_tags_35 },
    { 0xa783ecaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7283ecac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3ecaf, SUC_lexicon_tags_28 },
    { 0xd7a3ecb0, SUC_lexicon_tags_47 },
    { 0x6683ecaf, SUC_lexicon_tags_15 },
    { 0xd7a3ecb2, SUC_lexicon_tags_39 },
    { 0x2ec3ecb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc3ecb9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3783ecbd, SUC_lexicon_tags_37 },
    { 0x1b23ecbd, SUC_lexicon_tags_1 },
    { 0xefe3ecbe, SUC_lexicon_tags_2 },
    { 0x27e3ecc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf63ecc2, SUC_lexicon_tags_30 },
    { 0x4ec3ecc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf743ecc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa3ecc8, SUC_lexicon_tags_17 },
    { 0xd7c3ecc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd83eccb, SUC_lexicon_tags_17 },
    { 0x3283eccb, SUC_lexicon_tags_8 },
    { 0x6e3eccd, SUC_lexicon_tags_15 },
    { 0x5a63eccb, SUC_lexicon_tags_49 },
    { 0xdbe3ecce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xada3ecd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3ecd8, SUC_lexicon_tags_4 },
    { 0x703ecd9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd23ecdd, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0xcf83ecdf, SUC_lexicon_tags_12 },
    { 0x3283ece0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1483ece2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd83ece6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe203ecea, SUC_lexicon_tags_1 },
    { 0x56a3ecea, SUC_lexicon_tags_28 },
    { 0xf603ecec, SUC_lexicon_tags_16 },
    { 0x80e3ecec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c3ecf1, SUC_lexicon_tags_39 },
    { 0xb283ecf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e3ecf5, SUC_lexicon_tags_15 },
    { 0xa1c3ecf6, SUC_lexicon_tags_2 },
    { 0x30c3ecf5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b03ecf9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8ba3ecfb, SUC_lexicon_tags_27 },
    { 0xc423ecfc, SUC_lexicon_tags_17 },
    { 0x6563ecfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6083ed00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3423ed02, SUC_lexicon_tags_37 },
    { 0x5263ed03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6a3ed05, SUC_lexicon_tags_14 },
    { 0xf7a3ed06, SUC_lexicon_tags_30 },
    { 0xd963ed07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d43ed09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c43ed11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac3ed18, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a3ed1c, SUC_lexicon_tags_19 },
    { 0x99a3ed1d, SUC_lexicon_tags_17 },
    { 0xf8e3ed1e, SUC_lexicon_tags_15 },
    { 0x4323ed1e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9d23ed21, SUC_lexicon_tags_43 },
    { 0x9103ed22, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xc3c3ed24, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5643ed27, SUC_lexicon_tags_1 },
    { 0xcf43ed28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec3ed2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7883ed32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6483ed36, SUC_lexicon_tags_11 },
    { 0x32c3ed37, SUC_lexicon_tags_32 },
    { 0x13c3ed38, SUC_lexicon_tags_2 },
    { 0x9303ed39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc3ed3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3ed42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e43ed45, SUC_lexicon_tags_15 },
    { 0xefe3ed45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7083ed48, SUC_lexicon_tags_1 },
    { 0x3a23ed49, SUC_lexicon_tags_15 },
    { 0x56a3ed49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c3ed4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x42c3ed4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x45e3ed51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae3ed55, SUC_lexicon_tags_37 },
    { 0x5c03ed55, SUC_lexicon_tags_21 },
    { 0x3023ed55, SUC_lexicon_tags_21 },
    { 0x6b63ed58, SUC_lexicon_tags_7 },
    { 0x6e23ed59, SUC_lexicon_tags_28 },
    { 0xaaa3ed5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde3ed5f, SUC_lexicon_tags_25 },
    { 0x6ae3ed60, SUC_lexicon_tags_37 },
    { 0x4d23ed60, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x5543ed63, SUC_lexicon_tags_30 },
    { 0x2843ed63, SUC_lexicon_tags_37 },
    { 0xc1c3ed63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b83ed71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace3ed75, SUC_lexicon_tags_63 },
    { 0x5b63ed75, SUC_lexicon_tags_11 },
    { 0xa383ed77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce03ed7e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf523ed85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc3ed8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea3ed92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x383ed94, SUC_lexicon_tags_9 },
    { 0xa163ed94, SUC_lexicon_tags_6 },
    { 0x76c3ed94, SUC_lexicon_tags_7 },
    { 0x343ed97, SUC_lexicon_tags_30 },
    { 0xee3ed98, SUC_lexicon_tags_23 },
    { 0x6323ed98, SUC_lexicon_tags_37 },
    { 0x6483ed9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9343ed9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d83eda2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe83eda5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f63eda8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e3edab, SUC_lexicon_tags_63 },
    { 0xa983edac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c3edb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e3edb7, SUC_lexicon_tags_85 },
    { 0x3d43edb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6a3edba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4043edc0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e3edc4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa283edc8, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc03edcc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x69e3edce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc903edd0, SUC_lexicon_tags_47 },
    { 0x4f23edd0, SUC_lexicon_tags_4 },
    { 0x6223edd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f63edd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1743edd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8163eddb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6103ede1, SUC_lexicon_tags_12 },
    { 0xda83ede2, SUC_lexicon_tags_27 },
    { 0xae23ede3, SUC_lexicon_tags_9 },
    { 0x2a03ede3, SUC_lexicon_tags_25 },
    { 0x6bc3ede5, SUC_lexicon_tags_25 },
    { 0xddc3ede6, SUC_lexicon_tags_63 },
    { 0x9da3ede1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63eded, SUC_lexicon_tags_2 },
    { 0x6c3edee, SUC_lexicon_tags_66 },
    { 0x5303edef, SUC_lexicon_tags_13 },
    { 0x3103edf0, SUC_lexicon_tags_2 },
    { 0x9f03edf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c3edff, SUC_lexicon_tags_23 },
    { 0x6f83ee00, SUC_lexicon_tags_15 },
    { 0xafe3ee00, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xdec3ee04, SUC_lexicon_tags_63 },
    { 0xd883ee04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c3ee10, SUC_lexicon_tags_2 },
    { 0x8a83ee11, SUC_lexicon_tags_17 },
    { 0x9283ee12, SUC_lexicon_tags_27 },
    { 0xe803ee13, SUC_lexicon_tags_37 },
    { 0x9723ee13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53c3ee16, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c3ee1b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7883ee1e, SUC_lexicon_tags_7 },
    { 0x4f63ee1f, SUC_lexicon_tags_29 },
    { 0x8ba3ee20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9003ee23, SUC_lexicon_tags_97 },
    { 0xd263ee24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a03ee2e, SUC_lexicon_tags_27 },
    { 0x56e3ee2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80c3ee31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb63ee33, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8563ee39, SUC_lexicon_tags_12 },
    { 0x3863ee39, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c23ee3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e3ee45, SUC_lexicon_tags_12 },
    { 0xfd83ee45, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e43ee4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9983ee52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e3ee56, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8e63ee58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3ee5d, SUC_lexicon_tags_79 },
    { 0xeb03ee5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c23ee60, SUC_lexicon_tags_45 },
    { 0x5463ee61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec43ee63, SUC_lexicon_tags_65 },
    { 0x8363ee64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9283ee66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c63ee68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf523ee6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e3ee71, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe5a3ee73, SUC_lexicon_tags_1 },
    { 0xc763ee73, SUC_lexicon_tags_98 },
    { 0xa7c3ee74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce23ee79, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xef83ee7c, SUC_lexicon_tags_25 },
    { 0x1b83ee7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c03ee7f, SUC_lexicon_tags_694 },
    { 0xdd43ee80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa3ee85, SUC_lexicon_tags_26 },
    { 0x7e03ee85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x443ee88, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x183ee8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3c43ee8d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e3ee98, SUC_lexicon_tags_2 },
    { 0xed43ee99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2a3ee9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9903eea0, SUC_lexicon_tags_91 },
    { 0x6e03eea1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe3eea6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c3eeaf, SUC_lexicon_tags_30 },
    { 0x3823eeaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3a3eeb2, SUC_lexicon_tags_37 },
    { 0x5bc3eeb3, SUC_lexicon_tags_1 },
    { 0xe323eeb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac3eeb8, SUC_lexicon_tags_15 },
    { 0xe6a3eeb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c3eebe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a3eec9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca03eece, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb443eed2, SUC_lexicon_tags_79 },
    { 0xdf03eed3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6c3eed5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb763eed7, SUC_lexicon_tags_47 },
    { 0x5ba3eed8, SUC_lexicon_tags_32 },
    { 0x95e3eed9, SUC_lexicon_tags_15 },
    { 0x2283eeda, SUC_lexicon_tags_8 },
    { 0xe663eed7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c03eede, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9463eee2, SUC_lexicon_tags_62 },
    { 0xf783eee2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc3eee7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c3eeeb, SUC_lexicon_tags_43 },
    { 0x6083eeec, SUC_lexicon_tags_11 },
    { 0xc4c3eeed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1583eeef, SUC_lexicon_tags_46 },
    { 0x8623eef0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x8963eef2, SUC_lexicon_tags_2 },
    { 0xf4c3eef2, SUC_lexicon_tags_2 },
    { 0xa3c3eef4, SUC_lexicon_tags_12 },
    { 0x56c3eef5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc3eefa, SUC_lexicon_tags_11 },
    { 0x8da3eefb, SUC_lexicon_tags_12 },
    { 0x1823eefb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18a3eefe, SUC_lexicon_tags_7 },
    { 0x3bc3eeff, SUC_lexicon_tags_17 },
    { 0xb903eefe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4763ef06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2003ef0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40c3ef0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc83ef14, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e3ef1c, SUC_lexicon_tags_37 },
    { 0x9b83ef1d, SUC_lexicon_tags_62 },
    { 0x4863ef1e, SUC_lexicon_tags_0 },
    { 0x92e3ef1d, SUC_lexicon_tags_63 },
    { 0x3343ef1e, SUC_lexicon_tags_1 },
    { 0x5483ef21, SUC_lexicon_tags_22 },
    { 0x40a3ef1c, SUC_lexicon_tags_17 },
    { 0xdaa3ef23, SUC_lexicon_tags_37 },
    { 0x3ee3ef1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5623ef2b, SUC_lexicon_tags_13 },
    { 0x5163ef2c, SUC_lexicon_tags_14 },
    { 0xba23ef2c, SUC_lexicon_tags_79 },
    { 0xce23ef2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee3ef36, SUC_lexicon_tags_42 },
    { 0xd263ef37, SUC_lexicon_tags_28 },
    { 0xf8e3ef38, SUC_lexicon_tags_790 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de3ef3d, SUC_lexicon_tags_63 },
    { 0x16a3ef3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc803ef40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe903ef42, SUC_lexicon_tags_49 },
    { 0xd943ef42, SUC_lexicon_tags_7 },
    { 0x2503ef42, SUC_lexicon_tags_12 },
    { 0xb763ef45, SUC_lexicon_tags_43 },
    { 0xd523ef46, SUC_lexicon_tags_31 },
    { 0x303ef45, SUC_lexicon_tags_45 },
    { 0x9be3ef48, SUC_lexicon_tags_17 },
    { 0xea03ef44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7703ef4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6e3ef4e, SUC_lexicon_tags_47 },
    { 0x3823ef4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa203ef53, SUC_lexicon_tags_10 },
    { 0x5ba3ef54, SUC_lexicon_tags_1 },
    { 0x7a63ef55, SUC_lexicon_tags_7 },
    { 0x71c3ef56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44c3ef58, SUC_lexicon_tags_38 },
    { 0xdbe3ef58, SUC_lexicon_tags_15 },
    { 0x8623ef5a, SUC_lexicon_tags_47 },
    { 0x7683ef5b, SUC_lexicon_tags_4 },
    { 0xd943ef5b, SUC_lexicon_tags_1 },
    { 0xbaa3ef5a, SUC_lexicon_tags_66 },
    { 0x38e3ef5d, SUC_lexicon_tags_17 },
    { 0xd543ef5f, SUC_lexicon_tags_17 },
    { 0xeb03ef58, SUC_lexicon_tags_0 },
    { 0xafc3ef59, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x2a23ef63, SUC_lexicon_tags_60 },
    { 0x5c23ef64, SUC_lexicon_tags_26 },
    { 0xd9c3ef64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c63ef68, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e3ef6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9383ef74, SUC_lexicon_tags_57 },
    { 0x2bc3ef74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe623ef7c, SUC_lexicon_tags_38 },
    { 0xa703ef7d, SUC_lexicon_tags_60 },
    { 0x74c3ef7d, SUC_lexicon_tags_37 },
    { 0x4fe3ef7f, SUC_lexicon_tags_11 },
    { 0x2863ef7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a3ef83, SUC_lexicon_tags_27 },
    { 0x143ef84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf83ef88, SUC_lexicon_tags_27 },
    { 0x8d03ef89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2163ef90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdae3ef92, SUC_lexicon_tags_25 },
    { 0x4aa3ef93, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d23ef97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf203ef9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f83efa0, SUC_lexicon_tags_25 },
    { 0x88a3efa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde43efa5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a3efa9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd4e3efab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc263efb0, SUC_lexicon_tags_2 },
    { 0x4c23efb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe603efbe, SUC_lexicon_tags_12 },
    { 0xe863efbf, SUC_lexicon_tags_15 },
    { 0x9b83efc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc43efc2, SUC_lexicon_tags_26 },
    { 0x7c23efc3, SUC_lexicon_tags_17 },
    { 0x7bc3efc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5743efc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe3efcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a23efd4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1603efd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8a3efd9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2c43efdb, SUC_lexicon_tags_7 },
    { 0xa7a3efdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee3efdf, SUC_lexicon_tags_66 },
    { 0x2303efdf, SUC_lexicon_tags_47 },
    { 0xa783efe0, SUC_lexicon_tags_56 },
    { 0x5823efe0, SUC_lexicon_tags_11 },
    { 0x51e3efe3, SUC_lexicon_tags_11 },
    { 0x8783efe4, SUC_lexicon_tags_47 },
    { 0x7bc3efe5, SUC_lexicon_tags_26 },
    { 0x4fa3efe4, SUC_lexicon_tags_66 },
    { 0xfbe3efe4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c3efeb, SUC_lexicon_tags_0 },
    { 0x5f63efec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5543efee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e3eff2, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7503eff7, SUC_lexicon_tags_12 },
    { 0x903eff8, SUC_lexicon_tags_3 },
    { 0x3a23eff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf423efff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a3f003, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6003f008, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2903f00c, SUC_lexicon_tags_45 },
    { 0x98a3f00c, SUC_lexicon_tags_7 },
    { 0x3283f00e, SUC_lexicon_tags_48 },
    { 0x6ec3f00e, SUC_lexicon_tags_11 },
    { 0xc623f010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8183f013, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa903f017, SUC_lexicon_tags_2 },
    { 0x8d03f017, SUC_lexicon_tags_82 },
    { 0xeb63f019, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8863f01d, SUC_lexicon_tags_9 },
    { 0x52e3f01d, SUC_lexicon_tags_14 },
    { 0xd383f01d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef83f023, SUC_lexicon_tags_16 },
    { 0x7f03f024, SUC_lexicon_tags_2 },
    { 0xca23f025, SUC_lexicon_tags_11 },
    { 0x3923f026, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5183f028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd503f02d, SUC_lexicon_tags_14 },
    { 0x1c23f02d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa3f031, SUC_lexicon_tags_29 },
    { 0x9303f032, SUC_lexicon_tags_2 },
    { 0x9e03f033, SUC_lexicon_tags_17 },
    { 0xc683f034, SUC_lexicon_tags_63 },
    { 0x80e3f035, SUC_lexicon_tags_2 },
    { 0x46c3f036, SUC_lexicon_tags_83 },
    { 0x7483f037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a3f03b, SUC_lexicon_tags_87 },
    { 0x4383f03c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7563f03f, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xe8c3f041, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd03f047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb763f050, SUC_lexicon_tags_7 },
    { 0xd303f051, SUC_lexicon_tags_37 },
    { 0x2ce3f050, SUC_lexicon_tags_11 },
    { 0xa723f053, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf063f055, SUC_lexicon_tags_60 },
    { 0x1103f055, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5183f05c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2923f061, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42a3f063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a3f068, SUC_lexicon_tags_17 },
    { 0x8443f069, SUC_lexicon_tags_33 },
    { 0xd7c3f068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b23f070, SUC_lexicon_tags_15 },
    { 0x28c3f071, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf483f078, SUC_lexicon_tags_88 },
    { 0x3ee3f079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf143f07b, SUC_lexicon_tags_21 },
    { 0x5fa3f07c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b63f07e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e3f083, SUC_lexicon_tags_1 },
    { 0xe383f084, SUC_lexicon_tags_15 },
    { 0xe823f085, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c3f088, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x203f08a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c3f08c, SUC_lexicon_tags_11 },
    { 0x41c3f08d, SUC_lexicon_tags_7 },
    { 0x3e3f08e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2c23f090, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e3f095, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdba3f097, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a3f09b, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x5ce3f09d, SUC_lexicon_tags_45 },
    { 0x4b03f09e, SUC_lexicon_tags_25 },
    { 0xcf03f09e, SUC_lexicon_tags_29 },
    { 0xa0a3f0a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb43f0a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c3f0a5, SUC_lexicon_tags_14 },
    { 0x1803f0a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb143f0a8, SUC_lexicon_tags_87 },
    { 0x503f0a9, SUC_lexicon_tags_2 },
    { 0xdc43f0a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd583f0ad, SUC_lexicon_tags_31 },
    { 0x18e3f0ae, SUC_lexicon_tags_0 },
    { 0x3803f0af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d43f0b3, SUC_lexicon_tags_25 },
    { 0xf9a3f0b3, SUC_lexicon_tags_29 },
    { 0x9e43f0b5, SUC_lexicon_tags_15 },
    { 0x5683f0b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb63f0bb, SUC_lexicon_tags_14 },
    { 0x1103f0bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbde3f0be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc163f0c0, SUC_lexicon_tags_68 },
    { 0x53c3f0c0, SUC_lexicon_tags_28 },
    { 0xca23f0c2, SUC_lexicon_tags_13 },
    { 0x3ca3f0c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73c3f0c5, SUC_lexicon_tags_25 },
    { 0xcd23f0c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a3f0c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c83f0d1, SUC_lexicon_tags_37 },
    { 0xcde3f0d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca3f0d5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e3f0d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda43f0da, SUC_lexicon_tags_26 },
    { 0x61a3f0db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x73e3f0dd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8023f0e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c3f0e8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9823f0ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd243f0f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a3f0fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c3f101, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8443f105, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e3f10e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd003f110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe003f113, SUC_lexicon_tags_33 },
    { 0xd0a3f113, SUC_lexicon_tags_0 },
    { 0x3523f115, SUC_lexicon_tags_2 },
    { 0x31c3f115, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2663f118, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ca3f11a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee3f11d, SUC_lexicon_tags_17 },
    { 0xba43f11e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1883f122, SUC_lexicon_tags_1 },
    { 0xfd03f123, SUC_lexicon_tags_37 },
    { 0x3e3f122, SUC_lexicon_tags_48 },
    { 0xa163f125, SUC_lexicon_tags_4 },
    { 0x4903f125, SUC_lexicon_tags_45 },
    { 0x18c3f127, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e03f129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4c3f12b, SUC_lexicon_tags_28 },
    { 0x6fc3f12c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e83f12f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63f132, SUC_lexicon_tags_30 },
    { 0xe23f133, SUC_lexicon_tags_4 },
    { 0x8263f133, SUC_lexicon_tags_25 },
    { 0xa203f135, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6563f138, SUC_lexicon_tags_33 },
    { 0xf023f139, SUC_lexicon_tags_23 },
    { 0x7ea3f138, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f43f13e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb23f141, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd9c3f143, SUC_lexicon_tags_68 },
    { 0x6da3f143, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a3f148, SUC_lexicon_tags_68 },
    { 0x36e3f149, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb023f152, SUC_lexicon_tags_33 },
    { 0x1223f152, SUC_lexicon_tags_47 },
    { 0x5ca3f154, SUC_lexicon_tags_594 },
    { 0x2543f154, SUC_lexicon_tags_37 },
    { 0xfc43f153, SUC_lexicon_tags_21 },
    { 0xbaa3f155, SUC_lexicon_tags_11 },
    { 0x58a3f158, SUC_lexicon_tags_23 },
    { 0xd223f159, SUC_lexicon_tags_17 },
    { 0xae83f15a, SUC_lexicon_tags_23 },
    { 0x3ac3f15b, SUC_lexicon_tags_12 },
    { 0x5ee3f15c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa3f163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbe3f165, SUC_lexicon_tags_56 },
    { 0xc503f166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc203f168, SUC_lexicon_tags_15 },
    { 0x4583f169, SUC_lexicon_tags_42 },
    { 0x8683f16a, SUC_lexicon_tags_47 },
    { 0x6b63f16b, SUC_lexicon_tags_49 },
    { 0xa63f16b, SUC_lexicon_tags_12 },
    { 0x3163f16c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae63f171, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa3f174, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f43f177, SUC_lexicon_tags_25 },
    { 0x43a3f178, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0e3f17a, SUC_lexicon_tags_37 },
    { 0xbb63f17a, SUC_lexicon_tags_37 },
    { 0x6363f17b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb883f182, SUC_lexicon_tags_2 },
    { 0x5583f183, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa3f18a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4303f18c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3f190, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2643f194, SUC_lexicon_tags_34 },
    { 0xf0c3f195, SUC_lexicon_tags_49 },
    { 0xe7c3f196, SUC_lexicon_tags_12 },
    { 0x4da3f197, SUC_lexicon_tags_37 },
    { 0xf043f195, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7803f19b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2863f1a0, SUC_lexicon_tags_9 },
    { 0xc043f1a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8363f1a5, SUC_lexicon_tags_61 },
    { 0x9203f1a6, SUC_lexicon_tags_1 },
    { 0x9d83f1a7, SUC_lexicon_tags_14 },
    { 0x5ea3f1a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd363f1ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23f1b2, SUC_lexicon_tags_1 },
    { 0xf9a3f1b3, SUC_lexicon_tags_46 },
    { 0xadc3f1b4, SUC_lexicon_tags_63 },
    { 0xe803f1b5, SUC_lexicon_tags_37 },
    { 0x3ea3f1b3, SUC_lexicon_tags_26 },
    { 0x6fc3f1b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e3f1bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc263f1bd, SUC_lexicon_tags_11 },
    { 0x18c3f1be, SUC_lexicon_tags_27 },
    { 0x8c43f1bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe883f1c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a3f1cc, SUC_lexicon_tags_11 },
    { 0x70e3f1cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1da3f1cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea3f1d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc543f1d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe063f1d9, SUC_lexicon_tags_2 },
    { 0xc483f1da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6403f1df, SUC_lexicon_tags_84 },
    { 0x21c3f1e0, SUC_lexicon_tags_17 },
    { 0x7d23f1e1, SUC_lexicon_tags_18 },
    { 0x8e03f1e2, SUC_lexicon_tags_17 },
    { 0x1143f1e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4883f1e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d43f1ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xea03f1ec, SUC_lexicon_tags_19 },
    { 0x5d63f1ec, SUC_lexicon_tags_4 },
    { 0xa423f1ee, SUC_lexicon_tags_15 },
    { 0x69a3f1ef, SUC_lexicon_tags_2 },
    { 0xd763f1ef, SUC_lexicon_tags_17 },
    { 0x7a63f1f1, SUC_lexicon_tags_12 },
    { 0xf163f1ec, SUC_lexicon_tags_9 },
    { 0x80e3f1f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe783f1f8, SUC_lexicon_tags_7 },
    { 0x9623f1f9, SUC_lexicon_tags_37 },
    { 0x1fc3f1f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e3f1fd, SUC_lexicon_tags_43 },
    { 0xf643f1fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5463f203, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x21c3f205, SUC_lexicon_tags_28 },
    { 0xf123f206, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3f20a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8703f213, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60a3f215, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcae3f217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a23f219, SUC_lexicon_tags_17 },
    { 0xd783f21a, SUC_lexicon_tags_23 },
    { 0x1883f21b, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x4fa3f21d, SUC_lexicon_tags_2 },
    { 0x8d63f21e, SUC_lexicon_tags_12 },
    { 0xae43f21f, SUC_lexicon_tags_27 },
    { 0xf603f220, SUC_lexicon_tags_25 },
    { 0x9cc3f21f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd943f224, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e23f228, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8083f22a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4303f22c, SUC_lexicon_tags_1 },
    { 0xdc83f22d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e3f230, SUC_lexicon_tags_2 },
    { 0x2e83f230, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d03f233, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfa63f235, SUC_lexicon_tags_37 },
    { 0xfc03f236, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4963f238, SUC_lexicon_tags_12 },
    { 0x2d03f238, SUC_lexicon_tags_26 },
    { 0x7c3f23a, SUC_lexicon_tags_1 },
    { 0x4b03f23b, SUC_lexicon_tags_14 },
    { 0x22c3f23b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e3f23f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c03f246, SUC_lexicon_tags_17 },
    { 0x1b63f246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa583f24b, SUC_lexicon_tags_13 },
    { 0xdfe3f24c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee3f24f, SUC_lexicon_tags_2 },
    { 0xd043f250, SUC_lexicon_tags_1 },
    { 0xcb83f251, SUC_lexicon_tags_1 },
    { 0x8cc3f251, SUC_lexicon_tags_68 },
    { 0xb923f251, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea3f256, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa3f259, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe3f25b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcdc3f25d, SUC_lexicon_tags_12 },
    { 0x8783f25d, SUC_lexicon_tags_1 },
    { 0x3523f25f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd823f262, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1de3f264, SUC_lexicon_tags_25 },
    { 0x7163f264, SUC_lexicon_tags_6 },
    { 0x6863f266, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c3f269, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6ba3f26b, SUC_lexicon_tags_7 },
    { 0xb9a3f26c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2de3f26e, SUC_lexicon_tags_27 },
    { 0xb1a3f26f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7783f278, SUC_lexicon_tags_25 },
    { 0xc543f279, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7183f27e, SUC_lexicon_tags_0 },
    { 0x5503f27f, SUC_lexicon_tags_5 },
    { 0xc4c3f27e, SUC_lexicon_tags_1 },
    { 0x1a3f281, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2163f283, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x24e3f285, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x5263f287, SUC_lexicon_tags_11 },
    { 0x96c3f288, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f23f290, SUC_lexicon_tags_38 },
    { 0x4503f291, SUC_lexicon_tags_34 },
    { 0xee03f290, SUC_lexicon_tags_46 },
    { 0x3b03f293, SUC_lexicon_tags_56 },
    { 0x3f83f294, SUC_lexicon_tags_11 },
    { 0xae63f293, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa7e3f297, SUC_lexicon_tags_11 },
    { 0xf343f298, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a3f29e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7483f2a0, SUC_lexicon_tags_37 },
    { 0xdb83f2a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3623f2a3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec3f2b2, SUC_lexicon_tags_79 },
    { 0xcde3f2b3, SUC_lexicon_tags_30 },
    { 0x26a3f2b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b23f2b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7883f2ba, SUC_lexicon_tags_37 },
    { 0x8823f2ba, SUC_lexicon_tags_0 },
    { 0x943f2bc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23f2c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xde63f2c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfba3f2c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2623f2c7, SUC_lexicon_tags_12 },
    { 0x8a63f2c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be3f2cd, SUC_lexicon_tags_46 },
    { 0x8363f2cd, SUC_lexicon_tags_12 },
    { 0xe963f2cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f43f2d1, SUC_lexicon_tags_17 },
    { 0x1c83f2d2, SUC_lexicon_tags_14 },
    { 0x4b83f2d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec3f2d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e3f2da, SUC_lexicon_tags_2 },
    { 0x2903f2db, SUC_lexicon_tags_31 },
    { 0xcc3f2da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3f2df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6283f2e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f2ec, SUC_lexicon_tags_2 },
    { 0x65c3f2ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa63f2ef, SUC_lexicon_tags_2 },
    { 0x4e43f2f0, SUC_lexicon_tags_37 },
    { 0x4423f2f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd863f2f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe403f2f6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8863f2f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe843f2fb, SUC_lexicon_tags_77 },
    { 0xa823f2fb, SUC_lexicon_tags_63 },
    { 0x8903f2fc, SUC_lexicon_tags_30 },
    { 0xe7a3f2fe, SUC_lexicon_tags_27 },
    { 0x7203f2ff, SUC_lexicon_tags_47 },
    { 0x9d63f2fe, SUC_lexicon_tags_32 },
    { 0x52c3f301, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b23f307, SUC_lexicon_tags_19 },
    { 0x9b23f308, SUC_lexicon_tags_21 },
    { 0xf863f308, SUC_lexicon_tags_15 },
    { 0xa23f30a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a63f30c, SUC_lexicon_tags_15 },
    { 0x4ce3f30d, SUC_lexicon_tags_37 },
    { 0xa03f30d, SUC_lexicon_tags_59 },
    { 0x5e63f30d, SUC_lexicon_tags_63 },
    { 0x3543f30c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb143f312, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa63f315, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4063f318, SUC_lexicon_tags_6 },
    { 0x7143f319, SUC_lexicon_tags_84 },
    { 0x3d63f31a, SUC_lexicon_tags_33 },
    { 0x4e43f318, SUC_lexicon_tags_19 },
    { 0xd403f31c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d83f321, SUC_lexicon_tags_85 },
    { 0xeb43f322, SUC_lexicon_tags_19 },
    { 0xc6e3f321, SUC_lexicon_tags_37 },
    { 0x8ea3f324, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee3f327, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7383f32a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd303f330, SUC_lexicon_tags_19 },
    { 0xfda3f331, SUC_lexicon_tags_37 },
    { 0xf2c3f332, SUC_lexicon_tags_7 },
    { 0x75c3f333, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b23f336, SUC_lexicon_tags_27 },
    { 0x5303f337, SUC_lexicon_tags_2 },
    { 0x6b43f338, SUC_lexicon_tags_49 },
    { 0x1883f337, SUC_lexicon_tags_25 },
    { 0x7e03f33a, SUC_lexicon_tags_12 },
    { 0x8be3f33b, SUC_lexicon_tags_40 },
    { 0xe7a3f33c, SUC_lexicon_tags_12 },
    { 0x4d63f33d, SUC_lexicon_tags_49 },
    { 0x263f33a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6823f340, SUC_lexicon_tags_17 },
    { 0x7d83f340, SUC_lexicon_tags_7 },
    { 0x7123f341, SUC_lexicon_tags_10 },
    { 0xb7c3f340, SUC_lexicon_tags_63 },
    { 0xa43f340, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5103f346, SUC_lexicon_tags_37 },
    { 0x1803f347, SUC_lexicon_tags_11 },
    { 0x9dc3f348, SUC_lexicon_tags_1 },
    { 0x6643f349, SUC_lexicon_tags_7 },
    { 0xa243f34a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5803f352, SUC_lexicon_tags_85 },
    { 0xd5c3f353, SUC_lexicon_tags_15 },
    { 0xd3e3f354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dc3f356, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a3f359, SUC_lexicon_tags_30 },
    { 0x12e3f359, SUC_lexicon_tags_17 },
    { 0x7823f359, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea3f363, SUC_lexicon_tags_7 },
    { 0xd3c3f364, SUC_lexicon_tags_47 },
    { 0xb63f364, SUC_lexicon_tags_9 },
    { 0x1183f366, SUC_lexicon_tags_79 },
    { 0x1343f364, SUC_lexicon_tags_27 },
    { 0x9ac3f368, SUC_lexicon_tags_14 },
    { 0xfe23f366, SUC_lexicon_tags_32 },
    { 0x3983f364, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c3f36d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6343f374, SUC_lexicon_tags_0 },
    { 0xab63f375, SUC_lexicon_tags_22 },
    { 0xbe83f375, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8f83f378, SUC_lexicon_tags_12 },
    { 0x58a3f378, SUC_lexicon_tags_20 },
    { 0x683f37a, SUC_lexicon_tags_25 },
    { 0xe643f37a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6623f37e, SUC_lexicon_tags_8 },
    { 0xd1a3f37e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7b83f381, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd783f388, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa3f38c, SUC_lexicon_tags_43 },
    { 0x60e3f38c, SUC_lexicon_tags_11 },
    { 0xd2a3f38e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd5a3f390, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x31a3f392, SUC_lexicon_tags_25 },
    { 0x8ea3f393, SUC_lexicon_tags_29 },
    { 0x5a43f393, SUC_lexicon_tags_14 },
    { 0x9383f395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc483f397, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e3f39e, SUC_lexicon_tags_25 },
    { 0xeb43f39f, SUC_lexicon_tags_25 },
    { 0x143f39e, SUC_lexicon_tags_44 },
    { 0x36a3f39f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3c3f3a3, SUC_lexicon_tags_25 },
    { 0x32e3f3a4, SUC_lexicon_tags_47 },
    { 0x9fc3f3a4, SUC_lexicon_tags_45 },
    { 0x72a3f3a6, SUC_lexicon_tags_45 },
    { 0x7d03f3a4, SUC_lexicon_tags_10 },
    { 0xb1e3f3a8, SUC_lexicon_tags_7 },
    { 0xa023f3a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4183f3ad, SUC_lexicon_tags_11 },
    { 0x62e3f3ae, SUC_lexicon_tags_45 },
    { 0x75e3f3ae, SUC_lexicon_tags_25 },
    { 0x1ac3f3b0, SUC_lexicon_tags_45 },
    { 0xbea3f3b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99c3f3b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd683f3b5, SUC_lexicon_tags_38 },
    { 0x8b63f3b5, SUC_lexicon_tags_17 },
    { 0x62c3f3b7, SUC_lexicon_tags_10 },
    { 0x45a3f3b7, SUC_lexicon_tags_34 },
    { 0xa983f3b9, SUC_lexicon_tags_15 },
    { 0x7263f3ba, SUC_lexicon_tags_11 },
    { 0x5c23f3bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b23f3c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f23f3c9, SUC_lexicon_tags_47 },
    { 0x6e83f3ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xab83f3cd, SUC_lexicon_tags_2 },
    { 0x2fe3f3ce, SUC_lexicon_tags_5 },
    { 0x1523f3cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb03f3d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf523f3d6, SUC_lexicon_tags_37 },
    { 0x7623f3d7, SUC_lexicon_tags_136 },
    { 0xef03f3d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1443f3da, SUC_lexicon_tags_49 },
    { 0x9863f3db, SUC_lexicon_tags_17 },
    { 0xe423f3da, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcde3f3de, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9663f3e0, SUC_lexicon_tags_14 },
    { 0x9823f3e1, SUC_lexicon_tags_25 },
    { 0xa1c3f3e2, SUC_lexicon_tags_17 },
    { 0x2403f3e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea23f3e5, SUC_lexicon_tags_14 },
    { 0x11c3f3e5, SUC_lexicon_tags_60 },
    { 0x71c3f3e7, SUC_lexicon_tags_1 },
    { 0x8483f3e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8283f3ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee3f3f9, SUC_lexicon_tags_33 },
    { 0x183f3fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde03f404, SUC_lexicon_tags_88 },
    { 0xd483f404, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb63f409, SUC_lexicon_tags_70 },
    { 0xa663f409, SUC_lexicon_tags_1 },
    { 0xa143f409, SUC_lexicon_tags_43 },
    { 0x83c3f40c, SUC_lexicon_tags_11 },
    { 0x43c3f40a, SUC_lexicon_tags_15 },
    { 0x7a83f40e, SUC_lexicon_tags_0 },
    { 0xb383f40f, SUC_lexicon_tags_12 },
    { 0xd403f410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e3f413, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce03f417, SUC_lexicon_tags_9 },
    { 0x8543f417, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e03f41e, SUC_lexicon_tags_6 },
    { 0x42a3f41f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa3f424, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe703f426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e3f42c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c23f42e, SUC_lexicon_tags_23 },
    { 0x5523f42e, SUC_lexicon_tags_15 },
    { 0x6323f430, SUC_lexicon_tags_82 },
    { 0x5ec3f431, SUC_lexicon_tags_27 },
    { 0xbca3f432, SUC_lexicon_tags_19 },
    { 0x5463f432, SUC_lexicon_tags_25 },
    { 0x18c3f431, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8383f439, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf983f43c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c3f442, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa123f446, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c3f449, SUC_lexicon_tags_15 },
    { 0x3f449, SUC_lexicon_tags_14 },
    { 0x6fe3f449, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdc3f44d, SUC_lexicon_tags_7 },
    { 0xb663f44e, SUC_lexicon_tags_54 },
    { 0xf5e3f44d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe843f452, SUC_lexicon_tags_15 },
    { 0x283f452, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c83f459, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4483f45e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe2a3f460, SUC_lexicon_tags_25 },
    { 0xffa3f461, SUC_lexicon_tags_17 },
    { 0xa623f462, SUC_lexicon_tags_14 },
    { 0x1003f462, SUC_lexicon_tags_656 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c3f469, SUC_lexicon_tags_7 },
    { 0xcca3f46a, SUC_lexicon_tags_2 },
    { 0x9fa3f46a, SUC_lexicon_tags_17 },
    { 0x5223f46c, SUC_lexicon_tags_7 },
    { 0xabe3f46d, SUC_lexicon_tags_43 },
    { 0x26c3f46d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6243f470, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a3f473, SUC_lexicon_tags_61 },
    { 0xe863f474, SUC_lexicon_tags_17 },
    { 0xf723f474, SUC_lexicon_tags_14 },
    { 0x6523f476, SUC_lexicon_tags_46 },
    { 0x8ae3f477, SUC_lexicon_tags_170 },
    { 0x45c3f477, SUC_lexicon_tags_4 },
    { 0xd83f478, SUC_lexicon_tags_1 },
    { 0x2403f47a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3f47e, SUC_lexicon_tags_37 },
    { 0xb4a3f47e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa343f484, SUC_lexicon_tags_12 },
    { 0x1d23f485, SUC_lexicon_tags_0 },
    { 0xad23f486, SUC_lexicon_tags_11 },
    { 0xb203f485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f43f48a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc783f48d, SUC_lexicon_tags_11 },
    { 0x5883f48e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa763f491, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5723f498, SUC_lexicon_tags_0 },
    { 0xa963f498, SUC_lexicon_tags_11 },
    { 0x2d23f498, SUC_lexicon_tags_47 },
    { 0xd9e3f49b, SUC_lexicon_tags_82 },
    { 0x6563f49b, SUC_lexicon_tags_47 },
    { 0x9723f49d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a3f4a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d23f4a9, SUC_lexicon_tags_23 },
    { 0xbbc3f4aa, SUC_lexicon_tags_66 },
    { 0x18e3f4ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be3f4b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c3f4b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e3f4ba, SUC_lexicon_tags_44 },
    { 0x3603f4bb, SUC_lexicon_tags_2 },
    { 0xdb23f4bc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdbe3f4be, SUC_lexicon_tags_25 },
    { 0x2ce3f4be, SUC_lexicon_tags_63 },
    { 0x4423f4c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8163f4c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16e3f4c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfe43f4c8, SUC_lexicon_tags_120 },
    { 0x7cc3f4c9, SUC_lexicon_tags_66 },
    { 0x1803f4ca, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac3f4ce, SUC_lexicon_tags_21 },
    { 0x7a43f4cf, SUC_lexicon_tags_1 },
    { 0x5743f4cf, SUC_lexicon_tags_25 },
    { 0x8a3f4d1, SUC_lexicon_tags_31 },
    { 0xe23f4d1, SUC_lexicon_tags_761 },
    { 0xa4e3f4d3, SUC_lexicon_tags_33 },
    { 0xe003f4d4, SUC_lexicon_tags_17 },
    { 0x7a03f4d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xefe3f4d7, SUC_lexicon_tags_0 },
    { 0x44a3f4d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa0c3f4da, SUC_lexicon_tags_2 },
    { 0xc823f4db, SUC_lexicon_tags_37 },
    { 0xd0e3f4db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2443f4de, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0x4383f4e1, SUC_lexicon_tags_33 },
    { 0xda23f4e2, SUC_lexicon_tags_6 },
    { 0x3943f4e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d83f4e6, SUC_lexicon_tags_25 },
    { 0xd303f4e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2943f4ef, SUC_lexicon_tags_13 },
    { 0xb023f4f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4c63f4f2, SUC_lexicon_tags_2 },
    { 0xad83f4f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd83f4f6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5103f4fa, SUC_lexicon_tags_8 },
    { 0xb843f4fb, SUC_lexicon_tags_37 },
    { 0xfb23f4fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f43f4ff, SUC_lexicon_tags_87 },
    { 0x93e3f500, SUC_lexicon_tags_15 },
    { 0xa243f501, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1543f504, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1743f506, SUC_lexicon_tags_42 },
    { 0x51e3f507, SUC_lexicon_tags_17 },
    { 0x3aa3f508, SUC_lexicon_tags_44 },
    { 0x4de3f508, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1163f50b, SUC_lexicon_tags_14 },
    { 0x43f50b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6a3f50e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a3f511, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e3f514, SUC_lexicon_tags_12 },
    { 0x6e43f515, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7483f51a, SUC_lexicon_tags_0 },
    { 0xe9e3f51b, SUC_lexicon_tags_2 },
    { 0x13a3f51b, SUC_lexicon_tags_17 },
    { 0x8b63f51c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a3f522, SUC_lexicon_tags_32 },
    { 0xc563f523, SUC_lexicon_tags_70 },
    { 0x7a23f524, SUC_lexicon_tags_4 },
    { 0xf23f525, SUC_lexicon_tags_9 },
    { 0x3283f523, SUC_lexicon_tags_60 },
    { 0x7ea3f527, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xb4c3f529, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb23f52b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac83f52d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8f83f52f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5203f531, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa063f533, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9683f537, SUC_lexicon_tags_56 },
    { 0x8703f538, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62c3f53a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7943f540, SUC_lexicon_tags_25 },
    { 0x33e3f540, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3723f546, SUC_lexicon_tags_124 },
    { 0x9a83f547, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a3f54a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e3f54e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e83f555, SUC_lexicon_tags_11 },
    { 0x9363f555, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e3f559, SUC_lexicon_tags_48 },
    { 0x1003f55a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23f562, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29a3f564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a43f569, SUC_lexicon_tags_9 },
    { 0xe103f569, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3283f570, SUC_lexicon_tags_25 },
    { 0xb263f570, SUC_lexicon_tags_27 },
    { 0x6323f571, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce3f577, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe3f57b, SUC_lexicon_tags_17 },
    { 0xb8e3f57b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc403f57e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1143f580, SUC_lexicon_tags_14 },
    { 0x5783f580, SUC_lexicon_tags_15 },
    { 0x183f581, SUC_lexicon_tags_37 },
    { 0x1ec3f583, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2023f588, SUC_lexicon_tags_17 },
    { 0x51a3f588, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da3f58c, SUC_lexicon_tags_11 },
    { 0x2ea3f58c, SUC_lexicon_tags_8 },
    { 0x1f03f58e, SUC_lexicon_tags_6 },
    { 0xa343f58f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4743f592, SUC_lexicon_tags_12 },
    { 0x4843f592, SUC_lexicon_tags_46 },
    { 0xb663f592, SUC_lexicon_tags_12 },
    { 0x6743f595, SUC_lexicon_tags_43 },
    { 0x6603f596, SUC_lexicon_tags_23 },
    { 0xa43f597, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a3f59a, SUC_lexicon_tags_8 },
    { 0xae83f59a, SUC_lexicon_tags_68 },
    { 0xaca3f59a, SUC_lexicon_tags_19 },
    { 0x5a3f59d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9d83f59f, SUC_lexicon_tags_15 },
    { 0x143f5a0, SUC_lexicon_tags_11 },
    { 0xa2c3f59f, SUC_lexicon_tags_28 },
    { 0xa943f5a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee3f5a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2d43f5ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c3f5b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1923f5b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6103f5b5, SUC_lexicon_tags_16 },
    { 0xf2a3f5b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba3f5b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7663f5c0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7d83f5c2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5983f5c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb763f5c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ee3f5c8, SUC_lexicon_tags_57 },
    { 0x8883f5c8, SUC_lexicon_tags_27 },
    { 0xe063f5ca, SUC_lexicon_tags_15 },
    { 0x9063f5c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1483f5cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9cc3f5d1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3f5d4, SUC_lexicon_tags_26 },
    { 0x3f43f5d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c3f5d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f63f5da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xec3f5dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a63f5e1, SUC_lexicon_tags_0 },
    { 0xce43f5e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba3f5e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1083f5eb, SUC_lexicon_tags_7 },
    { 0x3983f5ec, SUC_lexicon_tags_9 },
    { 0xf983f5eb, SUC_lexicon_tags_53 },
    { 0x9943f5ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e83f5fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c83f5fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3103f600, SUC_lexicon_tags_19 },
    { 0x1283f600, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x24a3f603, SUC_lexicon_tags_8 },
    { 0xd983f603, SUC_lexicon_tags_18 },
    { 0xefa3f605, SUC_lexicon_tags_32 },
    { 0x5983f606, SUC_lexicon_tags_27 },
    { 0x6103f607, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd643f60a, SUC_lexicon_tags_43 },
    { 0x2c43f60b, SUC_lexicon_tags_23 },
    { 0x8743f60c, SUC_lexicon_tags_44 },
    { 0x3603f60d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc3f612, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1ca3f614, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e3f61a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf123f61e, SUC_lexicon_tags_87 },
    { 0x21e3f61e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c3f622, SUC_lexicon_tags_17 },
    { 0x4a23f623, SUC_lexicon_tags_11 },
    { 0x7ba3f624, SUC_lexicon_tags_85 },
    { 0x64a3f624, SUC_lexicon_tags_1 },
    { 0x7a23f626, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6703f628, SUC_lexicon_tags_15 },
    { 0xf663f628, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3863f635, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3f638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe583f63d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6883f640, SUC_lexicon_tags_18 },
    { 0x24a3f641, SUC_lexicon_tags_38 },
    { 0x67c3f642, SUC_lexicon_tags_43 },
    { 0xad23f642, SUC_lexicon_tags_7 },
    { 0x22c3f643, SUC_lexicon_tags_43 },
    { 0x2cc3f645, SUC_lexicon_tags_69 },
    { 0xae83f646, SUC_lexicon_tags_65 },
    { 0x8203f644, SUC_lexicon_tags_11 },
    { 0xd1c3f645, SUC_lexicon_tags_1 },
    { 0xb863f649, SUC_lexicon_tags_17 },
    { 0xca23f649, SUC_lexicon_tags_75 },
    { 0x34a3f64b, SUC_lexicon_tags_18 },
    { 0xbce3f644, SUC_lexicon_tags_38 },
    { 0xc6c3f64d, SUC_lexicon_tags_106 },
    { 0x3123f643, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4703f651, SUC_lexicon_tags_27 },
    { 0x5583f652, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c3f654, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdc03f656, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a43f659, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc663f65c, SUC_lexicon_tags_2 },
    { 0x39e3f65d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3f661, SUC_lexicon_tags_37 },
    { 0x3e43f661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe683f666, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe543f66a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x363f66c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e3f66f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc3f678, SUC_lexicon_tags_11 },
    { 0x2e83f679, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5963f67b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6c3f67d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b03f67f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3f683, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc3f686, SUC_lexicon_tags_49 },
    { 0x45e3f686, SUC_lexicon_tags_28 },
    { 0xb903f688, SUC_lexicon_tags_8 },
    { 0x6d23f687, SUC_lexicon_tags_6 },
    { 0x75c3f68a, SUC_lexicon_tags_7 },
    { 0xeae3f689, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c43f68e, SUC_lexicon_tags_64 },
    { 0x1c3f68e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe523f691, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be3f695, SUC_lexicon_tags_13 },
    { 0x4de3f696, SUC_lexicon_tags_141 },
    { 0x6503f697, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe3c3f699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5683f69c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab03f69e, SUC_lexicon_tags_37 },
    { 0x6983f69f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe583f6af, SUC_lexicon_tags_25 },
    { 0x8163f6af, SUC_lexicon_tags_27 },
    { 0x7263f6b0, SUC_lexicon_tags_14 },
    { 0x6a63f6b2, SUC_lexicon_tags_45 },
    { 0xb343f6b3, SUC_lexicon_tags_3 },
    { 0x3ae3f6b4, SUC_lexicon_tags_77 },
    { 0xf6c3f6b5, SUC_lexicon_tags_11 },
    { 0xdb43f6b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba3f6bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba3f6c2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9323f6d1, SUC_lexicon_tags_37 },
    { 0x2923f6d1, SUC_lexicon_tags_45 },
    { 0xb283f6d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a03f6db, SUC_lexicon_tags_37 },
    { 0x68e3f6db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3f6df, SUC_lexicon_tags_7 },
    { 0x67e3f6e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd023f6e4, SUC_lexicon_tags_6 },
    { 0x9e83f6e5, SUC_lexicon_tags_19 },
    { 0xfa43f6e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a3f6ec, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf23f6f2, SUC_lexicon_tags_12 },
    { 0xb723f6f3, SUC_lexicon_tags_66 },
    { 0x1c03f6f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad23f700, SUC_lexicon_tags_12 },
    { 0xc783f701, SUC_lexicon_tags_25 },
    { 0x5a63f701, SUC_lexicon_tags_12 },
    { 0x1703f701, SUC_lexicon_tags_37 },
    { 0x9063f701, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba63f708, SUC_lexicon_tags_1 },
    { 0x28e3f708, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e3f711, SUC_lexicon_tags_1 },
    { 0x1483f712, SUC_lexicon_tags_2 },
    { 0x64c3f713, SUC_lexicon_tags_9 },
    { 0x9c63f714, SUC_lexicon_tags_46 },
    { 0xd0a3f714, SUC_lexicon_tags_25 },
    { 0xd1a3f711, SUC_lexicon_tags_65 },
    { 0xbd63f717, SUC_lexicon_tags_39 },
    { 0x61a3f718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf383f71a, SUC_lexicon_tags_17 },
    { 0x2cc3f71b, SUC_lexicon_tags_485 },
    { 0x4703f71a, SUC_lexicon_tags_12 },
    { 0x8063f71d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7423f720, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef43f724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa3f72a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb63f72d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4923f734, SUC_lexicon_tags_7 },
    { 0xbd03f735, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6763f738, SUC_lexicon_tags_33 },
    { 0x5c63f739, SUC_lexicon_tags_16 },
    { 0x1b23f73a, SUC_lexicon_tags_17 },
    { 0x10a3f73b, SUC_lexicon_tags_33 },
    { 0x41e3f73c, SUC_lexicon_tags_14 },
    { 0x1f43f73b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6243f73f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7703f742, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9363f744, SUC_lexicon_tags_25 },
    { 0xe783f745, SUC_lexicon_tags_63 },
    { 0xb683f746, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6843f749, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe3f74d, SUC_lexicon_tags_15 },
    { 0xbb63f74e, SUC_lexicon_tags_15 },
    { 0x4aa3f74e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd343f751, SUC_lexicon_tags_12 },
    { 0x5963f752, SUC_lexicon_tags_31 },
    { 0xd2e3f753, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x363f756, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17c3f758, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a3f75e, SUC_lexicon_tags_401 },
    { 0x2b43f75e, SUC_lexicon_tags_11 },
    { 0x3943f760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2823f763, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea3f765, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a3f769, SUC_lexicon_tags_7 },
    { 0xb763f769, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4a43f76c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a3f778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21a3f77a, SUC_lexicon_tags_17 },
    { 0x8663f77b, SUC_lexicon_tags_1 },
    { 0xd983f77b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d03f781, SUC_lexicon_tags_14 },
    { 0xf2a3f782, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb63f78a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x76a3f78c, SUC_lexicon_tags_334 },
    { 0, NULL },
    { 0xdb43f78e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a3f791, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc663f794, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5063f798, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1683f79e, SUC_lexicon_tags_17 },
    { 0xff23f79e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa063f7a2, SUC_lexicon_tags_19 },
    { 0xff83f7a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5623f7a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd143f7aa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a3f7ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c3f7b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4523f7b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3f7bb, SUC_lexicon_tags_15 },
    { 0x3a3f7bc, SUC_lexicon_tags_11 },
    { 0xa403f7bd, SUC_lexicon_tags_47 },
    { 0xa3e3f7be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9623f7c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f03f7c7, SUC_lexicon_tags_2 },
    { 0x7923f7c8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac3f7cd, SUC_lexicon_tags_262 },
    { 0xea23f7ce, SUC_lexicon_tags_11 },
    { 0x9d03f7cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f63f7d6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6183f7dd, SUC_lexicon_tags_37 },
    { 0xc23f7de, SUC_lexicon_tags_15 },
    { 0x8c43f7df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3763f7e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a43f7e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a3f7e9, SUC_lexicon_tags_25 },
    { 0x1c63f7e9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a03f7ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4583f7ef, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a3f7f4, SUC_lexicon_tags_15 },
    { 0xd1a3f7f5, SUC_lexicon_tags_2 },
    { 0x3ae3f7f4, SUC_lexicon_tags_17 },
    { 0x62a3f7f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc163f7fb, SUC_lexicon_tags_15 },
    { 0xf583f7fb, SUC_lexicon_tags_1 },
    { 0x70c3f7fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc3f801, SUC_lexicon_tags_1 },
    { 0xd63f801, SUC_lexicon_tags_37 },
    { 0x8e43f803, SUC_lexicon_tags_77 },
    { 0x45e3f803, SUC_lexicon_tags_34 },
    { 0x7623f803, SUC_lexicon_tags_3 },
    { 0xe83f806, SUC_lexicon_tags_15 },
    { 0xf043f807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd983f80b, SUC_lexicon_tags_12 },
    { 0x87c3f80c, SUC_lexicon_tags_26 },
    { 0xe303f80d, SUC_lexicon_tags_68 },
    { 0xb463f80e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d03f815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa43f817, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2fc3f819, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa763f81b, SUC_lexicon_tags_12 },
    { 0xa2e3f81b, SUC_lexicon_tags_106 },
    { 0x5923f81d, SUC_lexicon_tags_62 },
    { 0x5323f81d, SUC_lexicon_tags_2 },
    { 0x8ac3f81f, SUC_lexicon_tags_31 },
    { 0xa43f81f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe763f822, SUC_lexicon_tags_10 },
    { 0x7923f823, SUC_lexicon_tags_43 },
    { 0x34e3f824, SUC_lexicon_tags_2 },
    { 0x4003f822, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd603f82f, SUC_lexicon_tags_11 },
    { 0x1823f830, SUC_lexicon_tags_27 },
    { 0xeba3f831, SUC_lexicon_tags_1 },
    { 0x4563f831, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9363f835, SUC_lexicon_tags_12 },
    { 0xe823f835, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc3a3f838, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d83f83d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a3f840, SUC_lexicon_tags_25 },
    { 0xbfe3f840, SUC_lexicon_tags_25 },
    { 0x1403f842, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd503f844, SUC_lexicon_tags_15 },
    { 0xc63f845, SUC_lexicon_tags_27 },
    { 0x3543f846, SUC_lexicon_tags_27 },
    { 0x6383f846, SUC_lexicon_tags_1 },
    { 0x9303f845, SUC_lexicon_tags_29 },
    { 0x9da3f844, SUC_lexicon_tags_68 },
    { 0x5ee3f84a, SUC_lexicon_tags_12 },
    { 0x38c3f84b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9943f84f, SUC_lexicon_tags_11 },
    { 0xce23f850, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e83f856, SUC_lexicon_tags_37 },
    { 0xe083f857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x323f859, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf43f85e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8163f860, SUC_lexicon_tags_17 },
    { 0x5763f860, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa463f864, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee3f86a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d83f86c, SUC_lexicon_tags_37 },
    { 0x69c3f86d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9983f86f, SUC_lexicon_tags_29 },
    { 0x77a3f86f, SUC_lexicon_tags_15 },
    { 0xbb43f871, SUC_lexicon_tags_7 },
    { 0x1f03f870, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd663f879, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da3f87e, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e3f881, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d03f885, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa3f888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb503f890, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa843f892, SUC_lexicon_tags_11 },
    { 0x8743f893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6223f897, SUC_lexicon_tags_17 },
    { 0x6f03f897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaea3f89a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9023f8a1, SUC_lexicon_tags_8 },
    { 0x4a03f8a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe2a3f8a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e3f8a7, SUC_lexicon_tags_12 },
    { 0xb823f8a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe3f8ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe743f8ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3323f8af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3ec3f8b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d43f8b4, SUC_lexicon_tags_30 },
    { 0x43e3f8b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf883f8c1, SUC_lexicon_tags_7 },
    { 0x3063f8c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8303f8c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a3f8c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863f8d2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6623f8d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe363f8d9, SUC_lexicon_tags_32 },
    { 0x21c3f8d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4783f8dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc3f8e3, SUC_lexicon_tags_9 },
    { 0xe23f8e4, SUC_lexicon_tags_13 },
    { 0x36c3f8e5, SUC_lexicon_tags_37 },
    { 0x9263f8e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2683f8f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7683f8f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfca3f8f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42e3f8f6, SUC_lexicon_tags_17 },
    { 0x5d43f8f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e3f8ff, SUC_lexicon_tags_46 },
    { 0x75c3f900, SUC_lexicon_tags_77 },
    { 0x55c3f901, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe623f903, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb803f908, SUC_lexicon_tags_27 },
    { 0xa863f909, SUC_lexicon_tags_30 },
    { 0xdf43f90a, SUC_lexicon_tags_7 },
    { 0xae83f90b, SUC_lexicon_tags_65 },
    { 0xd723f90b, SUC_lexicon_tags_17 },
    { 0x88c3f909, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c3f910, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e3f914, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8483f916, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc383f919, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x19c3f91b, SUC_lexicon_tags_11 },
    { 0xebe3f91c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2183f91f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd803f922, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4863f924, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e43f928, SUC_lexicon_tags_2 },
    { 0x4023f929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c3f92d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef83f931, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc3f934, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb943f937, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd23f939, SUC_lexicon_tags_12 },
    { 0xbe03f93a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x5783f93d, SUC_lexicon_tags_7 },
    { 0xca63f93e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfde3f940, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8923f947, SUC_lexicon_tags_106 },
    { 0xec83f947, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa143f94b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b43f94d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe3f950, SUC_lexicon_tags_13 },
    { 0xbec3f951, SUC_lexicon_tags_23 },
    { 0xd63f950, SUC_lexicon_tags_2 },
    { 0x7943f953, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c3f956, SUC_lexicon_tags_49 },
    { 0xe643f957, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c63f95a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x723f95c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f23f961, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3f965, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a3f96d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e43f970, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c3f973, SUC_lexicon_tags_17 },
    { 0xff43f974, SUC_lexicon_tags_25 },
    { 0xea3f975, SUC_lexicon_tags_46 },
    { 0xc43f976, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xef23f978, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6903f97c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9883f981, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe563f983, SUC_lexicon_tags_7 },
    { 0xf6e3f983, SUC_lexicon_tags_5 },
    { 0x37e3f984, SUC_lexicon_tags_64 },
    { 0x3803f986, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x523f988, SUC_lexicon_tags_63 },
    { 0xe8c3f989, SUC_lexicon_tags_37 },
    { 0x2803f98a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e3f992, SUC_lexicon_tags_17 },
    { 0x45c3f993, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e3f998, SUC_lexicon_tags_6 },
    { 0x24c3f998, SUC_lexicon_tags_11 },
    { 0xdbc3f99a, SUC_lexicon_tags_29 },
    { 0xb083f99a, SUC_lexicon_tags_15 },
    { 0x2a23f99c, SUC_lexicon_tags_11 },
    { 0xfb23f99c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a3f9a2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb83f9ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23f9af, SUC_lexicon_tags_55 },
    { 0x1f03f9b0, SUC_lexicon_tags_113 },
    { 0xf103f9b0, SUC_lexicon_tags_11 },
    { 0xade3f9b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6203f9b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce83f9b7, SUC_lexicon_tags_16 },
    { 0xfca3f9b8, SUC_lexicon_tags_33 },
    { 0xc3a3f9b9, SUC_lexicon_tags_11 },
    { 0x8be3f9b8, SUC_lexicon_tags_43 },
    { 0x91a3f9bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1403f9c0, SUC_lexicon_tags_45 },
    { 0x87a3f9c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe03f9c3, SUC_lexicon_tags_21 },
    { 0x83f9c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa23f9c6, SUC_lexicon_tags_12 },
    { 0xf403f9c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd303f9c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x16c3f9cb, SUC_lexicon_tags_10 },
    { 0x90a3f9cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e63f9cf, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x6963f9d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9903f9d4, SUC_lexicon_tags_15 },
    { 0xfca3f9d4, SUC_lexicon_tags_1 },
    { 0xe43f9d6, SUC_lexicon_tags_66 },
    { 0x8563f9d6, SUC_lexicon_tags_15 },
    { 0xfbc3f9d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7523f9dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce23f9df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c3f9e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9443f9e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba83f9e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c63f9eb, SUC_lexicon_tags_2 },
    { 0x9d23f9eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c3f9ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc703f9fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce63f9fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1ea3fa00, SUC_lexicon_tags_7 },
    { 0x5723fa01, SUC_lexicon_tags_10 },
    { 0x7923fa02, SUC_lexicon_tags_1 },
    { 0x3fc3fa03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xba83fa06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f03fa0b, SUC_lexicon_tags_14 },
    { 0x8303fa0c, SUC_lexicon_tags_18 },
    { 0x903fa0d, SUC_lexicon_tags_11 },
    { 0xc7a3fa0e, SUC_lexicon_tags_105 },
    { 0x52c3fa0e, SUC_lexicon_tags_0 },
    { 0xdc3fa0d, SUC_lexicon_tags_45 },
    { 0xbde3fa0e, SUC_lexicon_tags_8 },
    { 0xa1e3fa12, SUC_lexicon_tags_11 },
    { 0x7b63fa0f, SUC_lexicon_tags_17 },
    { 0x3e23fa14, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4da3fa16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6603fa18, SUC_lexicon_tags_30 },
    { 0xcd23fa18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa883fa22, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe663fa24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe363fa26, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc6e3fa28, SUC_lexicon_tags_28 },
    { 0x33c3fa29, SUC_lexicon_tags_63 },
    { 0x6c43fa29, SUC_lexicon_tags_14 },
    { 0x2283fa29, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3fa2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5e3fa31, SUC_lexicon_tags_60 },
    { 0xb8a3fa32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5c3fa34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9f83fa36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e23fa38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6363fa3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa03fa3f, SUC_lexicon_tags_46 },
    { 0xc6e3fa40, SUC_lexicon_tags_25 },
    { 0x7c43fa41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1143fa46, SUC_lexicon_tags_15 },
    { 0x5d03fa47, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c3fa4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e3fa52, SUC_lexicon_tags_17 },
    { 0xccc3fa52, SUC_lexicon_tags_0 },
    { 0xd683fa53, SUC_lexicon_tags_12 },
    { 0x7e03fa54, SUC_lexicon_tags_32 },
    { 0xaa03fa56, SUC_lexicon_tags_31 },
    { 0xb9e3fa57, SUC_lexicon_tags_39 },
    { 0x6083fa54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5fe3fa5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ea3fa5c, SUC_lexicon_tags_15 },
    { 0xa0e3fa5d, SUC_lexicon_tags_63 },
    { 0x8aa3fa5d, SUC_lexicon_tags_6 },
    { 0xd523fa5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde83fa65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb283fa6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2123fa6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe583fa71, SUC_lexicon_tags_30 },
    { 0x5aa3fa71, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb163fa75, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a3fa79, SUC_lexicon_tags_11 },
    { 0x7ac3fa7a, SUC_lexicon_tags_17 },
    { 0xfa83fa7a, SUC_lexicon_tags_7 },
    { 0xe503fa7c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa743fa80, SUC_lexicon_tags_17 },
    { 0xc383fa81, SUC_lexicon_tags_43 },
    { 0xbc43fa82, SUC_lexicon_tags_26 },
    { 0x5863fa81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb03fa88, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x4483fa8a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2963fa8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9543fa90, SUC_lexicon_tags_0 },
    { 0x51a3fa91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723fa9a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc3fa9e, SUC_lexicon_tags_10 },
    { 0x81c3fa9f, SUC_lexicon_tags_2 },
    { 0x9b43faa0, SUC_lexicon_tags_55 },
    { 0xa3a3faa0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecc3faa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3543faab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243fab1, SUC_lexicon_tags_201 },
    { 0xcba3fab2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfc3fab4, SUC_lexicon_tags_15 },
    { 0x17a3fab4, SUC_lexicon_tags_16 },
    { 0x9663fab6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c3fab9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7783fabf, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xd483fac2, SUC_lexicon_tags_12 },
    { 0x2c83fac3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5c63fac5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd263facd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5e3facf, SUC_lexicon_tags_17 },
    { 0xb443fad0, SUC_lexicon_tags_43 },
    { 0xb963fad1, SUC_lexicon_tags_33 },
    { 0x5423fad1, SUC_lexicon_tags_15 },
    { 0x5e23fad3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3363fad7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe663fada, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2a3fadc, SUC_lexicon_tags_35 },
    { 0xe203fadd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c3fae4, SUC_lexicon_tags_2 },
    { 0xaf03fae5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2243fae9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf263faec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3faef, SUC_lexicon_tags_37 },
    { 0xc9e3faf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3e3faf2, SUC_lexicon_tags_17 },
    { 0x35a3faf3, SUC_lexicon_tags_12 },
    { 0x1163faf4, SUC_lexicon_tags_27 },
    { 0x36c3faf5, SUC_lexicon_tags_71 },
    { 0xbb43faf4, SUC_lexicon_tags_48 },
    { 0x37a3faf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4603fafe, SUC_lexicon_tags_7 },
    { 0xc243fafe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c3fb04, SUC_lexicon_tags_38 },
    { 0xa1e3fb05, SUC_lexicon_tags_15 },
    { 0x6863fb05, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6723fb0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda3fb10, SUC_lexicon_tags_23 },
    { 0xf8e3fb10, SUC_lexicon_tags_65 },
    { 0x8843fb10, SUC_lexicon_tags_11 },
    { 0x80e3fb12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a3fb17, SUC_lexicon_tags_10 },
    { 0xe103fb18, SUC_lexicon_tags_30 },
    { 0x1303fb17, SUC_lexicon_tags_18 },
    { 0xc343fb1a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd983fb1f, SUC_lexicon_tags_60 },
    { 0x3063fb20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbba3fb22, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e3fb2c, SUC_lexicon_tags_15 },
    { 0xb923fb2c, SUC_lexicon_tags_25 },
    { 0x8a03fb2c, SUC_lexicon_tags_44 },
    { 0x6383fb2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1363fb31, SUC_lexicon_tags_37 },
    { 0x89a3fb32, SUC_lexicon_tags_25 },
    { 0x74e3fb32, SUC_lexicon_tags_77 },
    { 0x8f83fb34, SUC_lexicon_tags_129 },
    { 0x5a23fb35, SUC_lexicon_tags_29 },
    { 0xa103fb32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa63fb38, SUC_lexicon_tags_12 },
    { 0x18e3fb39, SUC_lexicon_tags_1 },
    { 0xe923fb3a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d83fb3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3943fb40, SUC_lexicon_tags_49 },
    { 0xe403fb41, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x1683fb44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd603fb46, SUC_lexicon_tags_47 },
    { 0xf4c3fb46, SUC_lexicon_tags_82 },
    { 0x80c3fb48, SUC_lexicon_tags_25 },
    { 0xaa43fb46, SUC_lexicon_tags_87 },
    { 0x8a23fb48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4563fb4c, SUC_lexicon_tags_11 },
    { 0x7983fb4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x523fb4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5063fb54, SUC_lexicon_tags_55 },
    { 0xd6a3fb55, SUC_lexicon_tags_15 },
    { 0xb023fb56, SUC_lexicon_tags_15 },
    { 0x19a3fb57, SUC_lexicon_tags_1 },
    { 0xd0e3fb57, SUC_lexicon_tags_11 },
    { 0x7183fb58, SUC_lexicon_tags_25 },
    { 0x49e3fb5a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4ee3fb5c, SUC_lexicon_tags_1 },
    { 0x49c3fb5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x65c3fb5f, SUC_lexicon_tags_3 },
    { 0x2cc3fb5f, SUC_lexicon_tags_2 },
    { 0xce03fb61, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0xb5c3fb63, SUC_lexicon_tags_43 },
    { 0x1423fb63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6643fb67, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa883fb69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e3fb6f, SUC_lexicon_tags_1 },
    { 0x4643fb6f, SUC_lexicon_tags_14 },
    { 0x2263fb71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab03fb75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8083fb78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf683fb7c, SUC_lexicon_tags_1 },
    { 0xf4a3fb7d, SUC_lexicon_tags_221 },
    { 0x6703fb7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4343fb83, SUC_lexicon_tags_15 },
    { 0x6943fb83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e3fb89, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be3fb8f, SUC_lexicon_tags_31 },
    { 0x8043fb90, SUC_lexicon_tags_26 },
    { 0x4363fb91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623fb95, SUC_lexicon_tags_10 },
    { 0x1a3fb95, SUC_lexicon_tags_16 },
    { 0x8643fb97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a3fb9b, SUC_lexicon_tags_11 },
    { 0x5043fb9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa83fb9e, SUC_lexicon_tags_57 },
    { 0x6543fb9f, SUC_lexicon_tags_13 },
    { 0x3423fba0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf43fbaa, SUC_lexicon_tags_56 },
    { 0x83fbaa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd23fbad, SUC_lexicon_tags_12 },
    { 0x7e83fbae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61a3fbb0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe3fbb3, SUC_lexicon_tags_12 },
    { 0x97c3fbb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c3fbb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb523fbbd, SUC_lexicon_tags_28 },
    { 0xc223fbbe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf883fbc8, SUC_lexicon_tags_60 },
    { 0xc903fbc8, SUC_lexicon_tags_17 },
    { 0x1043fbc9, SUC_lexicon_tags_55 },
    { 0xec03fbc8, SUC_lexicon_tags_21 },
    { 0x3983fbcc, SUC_lexicon_tags_7 },
    { 0x7fa3fbcd, SUC_lexicon_tags_57 },
    { 0x7c3fbcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c3fbdd, SUC_lexicon_tags_63 },
    { 0x4423fbdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b23fbe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7483fbe3, SUC_lexicon_tags_37 },
    { 0x10c3fbe4, SUC_lexicon_tags_14 },
    { 0xc483fbe4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c03fbe9, SUC_lexicon_tags_17 },
    { 0xec23fbe9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa303fbef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xca83fbf2, SUC_lexicon_tags_2 },
    { 0xd4a3fbf2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x37a3fbf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6963fbfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b43fbfd, SUC_lexicon_tags_11 },
    { 0x9563fbfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be3fc05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9563fc07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3fc0a, SUC_lexicon_tags_6 },
    { 0x3223fc0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae3fc0f, SUC_lexicon_tags_87 },
    { 0x1143fc10, SUC_lexicon_tags_15 },
    { 0x73c3fc11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac3fc16, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6323fc1b, SUC_lexicon_tags_29 },
    { 0xb303fc1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe3fc21, SUC_lexicon_tags_37 },
    { 0x8603fc22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7243fc26, SUC_lexicon_tags_6 },
    { 0x6c23fc27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c3fc2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b23fc36, SUC_lexicon_tags_30 },
    { 0x6c63fc37, SUC_lexicon_tags_23 },
    { 0xdfa3fc38, SUC_lexicon_tags_15 },
    { 0x4403fc37, SUC_lexicon_tags_2 },
    { 0xc123fc36, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a3fc40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec83fc42, SUC_lexicon_tags_37 },
    { 0x9023fc43, SUC_lexicon_tags_11 },
    { 0x4ee3fc43, SUC_lexicon_tags_27 },
    { 0xd2e3fc45, SUC_lexicon_tags_0 },
    { 0x7643fc43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c3fc4a, SUC_lexicon_tags_11 },
    { 0x3da3fc4b, SUC_lexicon_tags_15 },
    { 0x4a23fc4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf483fc4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb703fc51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c3fc54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d83fc56, SUC_lexicon_tags_25 },
    { 0xe1c3fc57, SUC_lexicon_tags_15 },
    { 0x9683fc57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3603fc5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2c3fc5c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e3fc5f, SUC_lexicon_tags_29 },
    { 0xd943fc60, SUC_lexicon_tags_17 },
    { 0x9383fc60, SUC_lexicon_tags_7 },
    { 0x1c3fc62, SUC_lexicon_tags_11 },
    { 0x1d83fc63, SUC_lexicon_tags_8 },
    { 0xaae3fc64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1483fc66, SUC_lexicon_tags_1 },
    { 0x55c3fc67, SUC_lexicon_tags_30 },
    { 0x6843fc66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe3fc6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03fc72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe23fc75, SUC_lexicon_tags_14 },
    { 0x3563fc76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e03fc7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe63fc7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3b83fc7f, SUC_lexicon_tags_1 },
    { 0x5da3fc80, SUC_lexicon_tags_11 },
    { 0x3b63fc80, SUC_lexicon_tags_55 },
    { 0x1c3fc82, SUC_lexicon_tags_17 },
    { 0x4a23fc83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe7a3fc85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3003fc8c, SUC_lexicon_tags_23 },
    { 0x5383fc8d, SUC_lexicon_tags_27 },
    { 0xa383fc8e, SUC_lexicon_tags_29 },
    { 0x5203fc8f, SUC_lexicon_tags_43 },
    { 0x9fe3fc90, SUC_lexicon_tags_25 },
    { 0xe823fc8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8703fc93, SUC_lexicon_tags_37 },
    { 0x5a63fc93, SUC_lexicon_tags_23 },
    { 0xa503fc93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1883fc97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c3fc9b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8683fca2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b63fca7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6983fcaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1963fcac, SUC_lexicon_tags_31 },
    { 0xacc3fcad, SUC_lexicon_tags_49 },
    { 0xb903fcae, SUC_lexicon_tags_4 },
    { 0x8f63fcad, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8683fcbc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3983fcbe, SUC_lexicon_tags_10 },
    { 0x1f63fcbe, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5223fcc5, SUC_lexicon_tags_15 },
    { 0x8ba3fcc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7343fcca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6923fccc, SUC_lexicon_tags_1 },
    { 0xa403fccd, SUC_lexicon_tags_37 },
    { 0x7843fccd, SUC_lexicon_tags_25 },
    { 0xd63fccf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a3fcd2, SUC_lexicon_tags_17 },
    { 0x7fa3fcd3, SUC_lexicon_tags_15 },
    { 0x5b63fcd4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c83fcda, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc3fce2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc83fce8, SUC_lexicon_tags_82 },
    { 0xcc83fce8, SUC_lexicon_tags_31 },
    { 0xe123fce8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcb83fcec, SUC_lexicon_tags_37 },
    { 0x71e3fced, SUC_lexicon_tags_12 },
    { 0x6b03fced, SUC_lexicon_tags_60 },
    { 0xc5a3fcef, SUC_lexicon_tags_12 },
    { 0xde23fcf0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae3fcf6, SUC_lexicon_tags_1 },
    { 0x9f23fcf6, SUC_lexicon_tags_11 },
    { 0xeb63fcf7, SUC_lexicon_tags_55 },
    { 0xb9e3fcf9, SUC_lexicon_tags_47 },
    { 0xd903fcf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb863fcff, SUC_lexicon_tags_37 },
    { 0xe963fd00, SUC_lexicon_tags_45 },
    { 0x95c3fd01, SUC_lexicon_tags_0 },
    { 0xb843fd02, SUC_lexicon_tags_56 },
    { 0xfd03fd00, SUC_lexicon_tags_79 },
    { 0x4a43fd02, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3543fd07, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x9583fd09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa443fd0b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d03fd0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a3fd17, SUC_lexicon_tags_17 },
    { 0xd003fd18, SUC_lexicon_tags_84 },
    { 0x43c3fd19, SUC_lexicon_tags_84 },
    { 0xd5c3fd1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf43fd1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc883fd1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a3fd2d, SUC_lexicon_tags_17 },
    { 0xab03fd2e, SUC_lexicon_tags_25 },
    { 0xdf83fd2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x963fd31, SUC_lexicon_tags_11 },
    { 0x6303fd32, SUC_lexicon_tags_62 },
    { 0x163fd33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1783fd38, SUC_lexicon_tags_1 },
    { 0xd423fd39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9543fd45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xab83fd48, SUC_lexicon_tags_0 },
    { 0x9003fd48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea3fd4c, SUC_lexicon_tags_13 },
    { 0x4243fd4d, SUC_lexicon_tags_141 },
    { 0xa5a3fd4d, SUC_lexicon_tags_38 },
    { 0x4383fd4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xad43fd52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2943fd54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba23fd5d, SUC_lexicon_tags_8 },
    { 0x76c3fd5e, SUC_lexicon_tags_60 },
    { 0xff63fd5f, SUC_lexicon_tags_2 },
    { 0x7203fd60, SUC_lexicon_tags_45 },
    { 0x3923fd61, SUC_lexicon_tags_28 },
    { 0xf6c3fd62, SUC_lexicon_tags_0 },
    { 0xfde3fd63, SUC_lexicon_tags_63 },
    { 0x8483fd64, SUC_lexicon_tags_2 },
    { 0xb1a3fd60, SUC_lexicon_tags_15 },
    { 0xdda3fd61, SUC_lexicon_tags_15 },
    { 0x5ba3fd67, SUC_lexicon_tags_37 },
    { 0xa1a3fd62, SUC_lexicon_tags_25 },
    { 0x5ea3fd69, SUC_lexicon_tags_11 },
    { 0x6f63fd69, SUC_lexicon_tags_7 },
    { 0x5843fd68, SUC_lexicon_tags_11 },
    { 0x4fc3fd6c, SUC_lexicon_tags_38 },
    { 0xc7c3fd6c, SUC_lexicon_tags_11 },
    { 0xbca3fd6e, SUC_lexicon_tags_4 },
    { 0xfd83fd62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd163fd71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f23fd73, SUC_lexicon_tags_56 },
    { 0xf5a3fd74, SUC_lexicon_tags_11 },
    { 0xb6e3fd74, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec03fd7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3063fd80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a43fd84, SUC_lexicon_tags_7 },
    { 0x5003fd85, SUC_lexicon_tags_10 },
    { 0xb03fd85, SUC_lexicon_tags_35 },
    { 0xe683fd87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e3fd8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb663fd8d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543fd92, SUC_lexicon_tags_38 },
    { 0xd1a3fd93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb103fd97, SUC_lexicon_tags_158 },
    { 0xc4a3fd98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783fd9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c3fda1, SUC_lexicon_tags_32 },
    { 0xa423fda2, SUC_lexicon_tags_57 },
    { 0x7a83fda3, SUC_lexicon_tags_31 },
    { 0x7d23fda4, SUC_lexicon_tags_15 },
    { 0xa403fda2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b83fda7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6323fdaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e3fdb1, SUC_lexicon_tags_11 },
    { 0x3283fdb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf803fdbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc3fdbe, SUC_lexicon_tags_47 },
    { 0xe783fdbf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc003fdc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e3fdcb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5443fdcd, SUC_lexicon_tags_17 },
    { 0x4983fdce, SUC_lexicon_tags_12 },
    { 0x8ba3fdcf, SUC_lexicon_tags_29 },
    { 0x5a43fdd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a43fdd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee3fddd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc143fde2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xac63fde5, SUC_lexicon_tags_53 },
    { 0xeac3fde5, SUC_lexicon_tags_37 },
    { 0xcc83fde7, SUC_lexicon_tags_17 },
    { 0xa43fde7, SUC_lexicon_tags_11 },
    { 0x8ac3fde9, SUC_lexicon_tags_49 },
    { 0xbca3fde7, SUC_lexicon_tags_25 },
    { 0xbe3fde6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfa43fded, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7403fdef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c3fdf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a3fe02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f03fe06, SUC_lexicon_tags_17 },
    { 0x3443fe06, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a3fe0c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3fe12, SUC_lexicon_tags_36 },
    { 0xd463fe13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc3fe17, SUC_lexicon_tags_37 },
    { 0x7123fe18, SUC_lexicon_tags_17 },
    { 0xef83fe19, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x3203fe1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c3fe22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf03fe27, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4103fe2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e3fe31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b03fe34, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd083fe36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43fe3a, SUC_lexicon_tags_49 },
    { 0xdd63fe3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x943fe3d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x7903fe3f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd823fe41, SUC_lexicon_tags_7 },
    { 0xb123fe42, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1543fe44, SUC_lexicon_tags_55 },
    { 0xcc23fe45, SUC_lexicon_tags_25 },
    { 0xd243fe46, SUC_lexicon_tags_14 },
    { 0x3943fe46, SUC_lexicon_tags_33 },
    { 0xd1a3fe46, SUC_lexicon_tags_25 },
    { 0x5823fe46, SUC_lexicon_tags_26 },
    { 0x8bc3fe49, SUC_lexicon_tags_7 },
    { 0xff03fe4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x51c3fe4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2923fe50, SUC_lexicon_tags_27 },
    { 0x3143fe51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c03fe5a, SUC_lexicon_tags_10 },
    { 0xb463fe5b, SUC_lexicon_tags_33 },
    { 0x6663fe5a, SUC_lexicon_tags_1 },
    { 0xbc23fe5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbec3fe5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd43fe63, SUC_lexicon_tags_39 },
    { 0xe8e3fe64, SUC_lexicon_tags_1 },
    { 0x4f83fe63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8623fe67, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf543fe6a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a3fe6e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x94c3fe70, SUC_lexicon_tags_25 },
    { 0x5063fe71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7623fe73, SUC_lexicon_tags_33 },
    { 0xca03fe73, SUC_lexicon_tags_62 },
    { 0x1223fe75, SUC_lexicon_tags_41 },
    { 0x6803fe76, SUC_lexicon_tags_11 },
    { 0x8063fe76, SUC_lexicon_tags_14 },
    { 0x2983fe78, SUC_lexicon_tags_10 },
    { 0xc983fe79, SUC_lexicon_tags_55 },
    { 0x71a3fe75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e03fe7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6423fe7f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd5a3fe81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef83fe85, SUC_lexicon_tags_65 },
    { 0x8083fe86, SUC_lexicon_tags_48 },
    { 0xf463fe87, SUC_lexicon_tags_2 },
    { 0x47c3fe86, SUC_lexicon_tags_42 },
    { 0x3263fe89, SUC_lexicon_tags_30 },
    { 0xd4e3fe89, SUC_lexicon_tags_25 },
    { 0x3983fe88, SUC_lexicon_tags_47 },
    { 0x8dc3fe8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4803fe8e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1263fe92, SUC_lexicon_tags_14 },
    { 0x91e3fe93, SUC_lexicon_tags_25 },
    { 0xea63fe94, SUC_lexicon_tags_60 },
    { 0x5d83fe95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf563fe99, SUC_lexicon_tags_60 },
    { 0x4903fe9a, SUC_lexicon_tags_10 },
    { 0x20a3fe9a, SUC_lexicon_tags_60 },
    { 0x1da3fe9c, SUC_lexicon_tags_2 },
    { 0x6b43fe9c, SUC_lexicon_tags_63 },
    { 0x5e23fe9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e63fea5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8063fea9, SUC_lexicon_tags_14 },
    { 0x6003feaa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1503feb0, SUC_lexicon_tags_16 },
    { 0x7043feb1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa943feb3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a3feba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8483febc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c3febe, SUC_lexicon_tags_77 },
    { 0xb1c3febe, SUC_lexicon_tags_56 },
    { 0x7803febe, SUC_lexicon_tags_6 },
    { 0x7c03fec1, SUC_lexicon_tags_40 },
    { 0xa8c3fec1, SUC_lexicon_tags_61 },
    { 0x4ee3fec3, SUC_lexicon_tags_11 },
    { 0x9d23febe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4943fec6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9063fec8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9863feca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac3fece, SUC_lexicon_tags_19 },
    { 0x1a3fece, SUC_lexicon_tags_7 },
    { 0xbe3fed0, SUC_lexicon_tags_30 },
    { 0x61c3fecf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4403fed4, SUC_lexicon_tags_17 },
    { 0x2203fed5, SUC_lexicon_tags_29 },
    { 0x7323fed6, SUC_lexicon_tags_18 },
    { 0xa1a3fed7, SUC_lexicon_tags_13 },
    { 0x2e03fed8, SUC_lexicon_tags_1 },
    { 0x41c3fed4, SUC_lexicon_tags_7 },
    { 0x1c23fed7, SUC_lexicon_tags_1 },
    { 0x9583fed9, SUC_lexicon_tags_30 },
    { 0x9703fedc, SUC_lexicon_tags_28 },
    { 0x85e3fed6, SUC_lexicon_tags_45 },
    { 0xcf83fede, SUC_lexicon_tags_47 },
    { 0x8283fede, SUC_lexicon_tags_7 },
    { 0x8ac3fed6, SUC_lexicon_tags_23 },
    { 0xcc03fed6, SUC_lexicon_tags_60 },
    { 0x5383fedf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd763fee5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2203fee7, SUC_lexicon_tags_37 },
    { 0x81e3fee7, SUC_lexicon_tags_17 },
    { 0xe6e3fee8, SUC_lexicon_tags_17 },
    { 0x36a3fee7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5903feec, SUC_lexicon_tags_15 },
    { 0x3c83feec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa623fef0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc163fef3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb63fef6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c3fefd, SUC_lexicon_tags_14 },
    { 0xd403fefd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1f03ff00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe323ff07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e3ff0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb83ff10, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e3ff14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e3ff1a, SUC_lexicon_tags_17 },
    { 0x243ff1a, SUC_lexicon_tags_17 },
    { 0xea43ff1b, SUC_lexicon_tags_1 },
    { 0x6b63ff1d, SUC_lexicon_tags_87 },
    { 0x2e63ff1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ff21, SUC_lexicon_tags_63 },
    { 0x7de3ff21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8283ff26, SUC_lexicon_tags_1 },
    { 0x5d63ff27, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe643ff2b, SUC_lexicon_tags_14 },
    { 0x2ee3ff2c, SUC_lexicon_tags_5 },
    { 0x9c63ff2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c3ff30, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x91a3ff32, SUC_lexicon_tags_1 },
    { 0xfb43ff33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc863ff3c, SUC_lexicon_tags_43 },
    { 0x55c3ff3d, SUC_lexicon_tags_46 },
    { 0xee23ff3e, SUC_lexicon_tags_28 },
    { 0x6703ff3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2a3ff41, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9643ff45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe423ff4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc363ff51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf683ff57, SUC_lexicon_tags_11 },
    { 0xe763ff58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3723ff5b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa3ff5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49e3ff60, SUC_lexicon_tags_7 },
    { 0x5263ff61, SUC_lexicon_tags_10 },
    { 0x2ac3ff62, SUC_lexicon_tags_26 },
    { 0x45a3ff61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb63ff6b, SUC_lexicon_tags_7 },
    { 0xb543ff6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb683ff70, SUC_lexicon_tags_14 },
    { 0x1c03ff71, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2e23ff73, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c3ff77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x15c3ff79, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c63ff7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa3ff82, SUC_lexicon_tags_37 },
    { 0xe0e3ff83, SUC_lexicon_tags_38 },
    { 0xe9a3ff84, SUC_lexicon_tags_77 },
    { 0x15a3ff85, SUC_lexicon_tags_2 },
    { 0xc5e3ff82, SUC_lexicon_tags_17 },
    { 0x363ff83, SUC_lexicon_tags_84 },
    { 0x2cc3ff87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a83ff8b, SUC_lexicon_tags_63 },
    { 0xed03ff8c, SUC_lexicon_tags_27 },
    { 0xbb03ff8c, SUC_lexicon_tags_11 },
    { 0x9263ff8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c23ff94, SUC_lexicon_tags_15 },
    { 0xa283ff95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf803ff9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9c3ff9d, SUC_lexicon_tags_27 },
    { 0x3e3ff9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2003ffa9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2023ffae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcb63ffb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3ffb3, SUC_lexicon_tags_4 },
    { 0x6fa3ffb4, SUC_lexicon_tags_4 },
    { 0x83ffb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a3ffbe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7c43ffc0, SUC_lexicon_tags_25 },
    { 0x5923ffc1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xba63ffc4, SUC_lexicon_tags_17 },
    { 0x7c63ffc4, SUC_lexicon_tags_57 },
    { 0x9ae3ffc6, SUC_lexicon_tags_37 },
    { 0x8a43ffc7, SUC_lexicon_tags_12 },
    { 0x1223ffc7, SUC_lexicon_tags_11 },
    { 0x2a23ffc5, SUC_lexicon_tags_27 },
    { 0xb8a3ffca, SUC_lexicon_tags_11 },
    { 0xb603ffca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9523ffce, SUC_lexicon_tags_2 },
    { 0x5663ffce, SUC_lexicon_tags_45 },
    { 0x1803ffcf, SUC_lexicon_tags_47 },
    { 0x49c3ffd1, SUC_lexicon_tags_77 },
    { 0xcdc3ffd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b83ffd6, SUC_lexicon_tags_10 },
    { 0xdd23ffd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a3ffdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c03ffe1, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c3ffea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc363ffec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd603ffee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f43fff9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9a63fffb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2643ffff, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc40002, SUC_lexicon_tags_49 },
    { 0xf4040003, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e40006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28640009, SUC_lexicon_tags_106 },
    { 0xfd44000a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7464000c, SUC_lexicon_tags_11 },
    { 0xeba4000d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe40016, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e40019, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x204001b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a4001e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc40024, SUC_lexicon_tags_17 },
    { 0x7f440025, SUC_lexicon_tags_1 },
    { 0x56e40026, SUC_lexicon_tags_14 },
    { 0x64c40026, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x96840029, SUC_lexicon_tags_17 },
    { 0xf064002a, SUC_lexicon_tags_60 },
    { 0x2c24002b, SUC_lexicon_tags_172 },
    { 0xd964002b, SUC_lexicon_tags_2 },
    { 0xd5a4002d, SUC_lexicon_tags_17 },
    { 0x8dc4002e, SUC_lexicon_tags_45 },
    { 0x1b24002a, SUC_lexicon_tags_39 },
    { 0xc40030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f440036, SUC_lexicon_tags_14 },
    { 0x68840037, SUC_lexicon_tags_27 },
    { 0xc5440036, SUC_lexicon_tags_19 },
    { 0xec640036, SUC_lexicon_tags_45 },
    { 0x9584003a, SUC_lexicon_tags_23 },
    { 0x72c4003b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a40040, SUC_lexicon_tags_7 },
    { 0xc6640041, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xba40044, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60040046, SUC_lexicon_tags_3 },
    { 0xedc40047, SUC_lexicon_tags_14 },
    { 0x40240046, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9344004a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e4004d, SUC_lexicon_tags_53 },
    { 0xf1a4004e, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a40053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2640055, SUC_lexicon_tags_25 },
    { 0xdbe40055, SUC_lexicon_tags_2 },
    { 0x4f840057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff240060, SUC_lexicon_tags_9 },
    { 0x61440061, SUC_lexicon_tags_23 },
    { 0x35440061, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x58840064, SUC_lexicon_tags_43 },
    { 0x2ea40064, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcde40067, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb2c40069, SUC_lexicon_tags_14 },
    { 0xa4840069, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11e4006c, SUC_lexicon_tags_7 },
    { 0x5944006d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd1a4006f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9da40071, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a40075, SUC_lexicon_tags_17 },
    { 0xe0a40076, SUC_lexicon_tags_63 },
    { 0x3b240077, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc04007a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16240084, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4664008a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1fa4008c, SUC_lexicon_tags_0 },
    { 0x2fe4008c, SUC_lexicon_tags_11 },
    { 0x6984008d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2440091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1a40093, SUC_lexicon_tags_15 },
    { 0xd440094, SUC_lexicon_tags_0 },
    { 0xd6640095, SUC_lexicon_tags_1 },
    { 0x48a40096, SUC_lexicon_tags_12 },
    { 0x9ec40096, SUC_lexicon_tags_7 },
    { 0x8ac40098, SUC_lexicon_tags_1 },
    { 0x42040097, SUC_lexicon_tags_1 },
    { 0xf7040096, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98400a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ca400a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e400a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8400ac, SUC_lexicon_tags_33 },
    { 0xe4a400ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282400b2, SUC_lexicon_tags_33 },
    { 0x4f6400b2, SUC_lexicon_tags_12 },
    { 0xa6c400b2, SUC_lexicon_tags_37 },
    { 0xba4400b5, SUC_lexicon_tags_37 },
    { 0xd78400b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50400ba, SUC_lexicon_tags_7 },
    { 0x790400bb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7e400bd, SUC_lexicon_tags_14 },
    { 0x320400be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x398400c1, SUC_lexicon_tags_11 },
    { 0x202400c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de400c5, SUC_lexicon_tags_12 },
    { 0x302400c5, SUC_lexicon_tags_9 },
    { 0x5de400c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750400d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0400d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246400dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2400df, SUC_lexicon_tags_37 },
    { 0xb84400e0, SUC_lexicon_tags_182 },
    { 0xf3c400df, SUC_lexicon_tags_2 },
    { 0xa5e400e1, SUC_lexicon_tags_25 },
    { 0x680400e3, SUC_lexicon_tags_11 },
    { 0xaf2400e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb36400e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a400eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18400f0, SUC_lexicon_tags_0 },
    { 0x504400f0, SUC_lexicon_tags_43 },
    { 0x972400f2, SUC_lexicon_tags_38 },
    { 0x646400f3, SUC_lexicon_tags_2 },
    { 0x58a400f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6400fc, SUC_lexicon_tags_4 },
    { 0xbce400fd, SUC_lexicon_tags_14 },
    { 0x93e400fd, SUC_lexicon_tags_26 },
    { 0xd82400ff, SUC_lexicon_tags_14 },
    { 0xfa4400ff, SUC_lexicon_tags_23 },
    { 0xc82400fd, SUC_lexicon_tags_25 },
    { 0x26240102, SUC_lexicon_tags_14 },
    { 0xd5040100, SUC_lexicon_tags_12 },
    { 0x38e400ff, SUC_lexicon_tags_17 },
    { 0x112400fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x86c40107, SUC_lexicon_tags_8 },
    { 0xeaa40108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c64010d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9440110, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6040112, SUC_lexicon_tags_38 },
    { 0x63440112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec640117, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc984011c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74640129, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44440132, SUC_lexicon_tags_11 },
    { 0x2ca40133, SUC_lexicon_tags_31 },
    { 0xa2240134, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce4013a, SUC_lexicon_tags_37 },
    { 0x2ea4013a, SUC_lexicon_tags_23 },
    { 0x4784013c, SUC_lexicon_tags_7 },
    { 0x8504013c, SUC_lexicon_tags_15 },
    { 0x5e44013d, SUC_lexicon_tags_33 },
    { 0x7d84013e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9840141, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc240147, SUC_lexicon_tags_2 },
    { 0x4d840148, SUC_lexicon_tags_60 },
    { 0x93040149, SUC_lexicon_tags_1 },
    { 0xfca4014a, SUC_lexicon_tags_25 },
    { 0xc9a4014b, SUC_lexicon_tags_8 },
    { 0x7944014c, SUC_lexicon_tags_12 },
    { 0x37e4014d, SUC_lexicon_tags_37 },
    { 0x7c4014b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a40152, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4240156, SUC_lexicon_tags_63 },
    { 0x3ee40157, SUC_lexicon_tags_49 },
    { 0xf7e40158, SUC_lexicon_tags_45 },
    { 0x96640159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c4015c, SUC_lexicon_tags_11 },
    { 0x6b24015d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c440164, SUC_lexicon_tags_17 },
    { 0x3e040165, SUC_lexicon_tags_37 },
    { 0x6b040166, SUC_lexicon_tags_108 },
    { 0x7bc40165, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc4016a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a4016d, SUC_lexicon_tags_62 },
    { 0x55c4016e, SUC_lexicon_tags_43 },
    { 0xd64016d, SUC_lexicon_tags_65 },
    { 0x2f84016d, SUC_lexicon_tags_11 },
    { 0x21640171, SUC_lexicon_tags_25 },
    { 0xbec4016e, SUC_lexicon_tags_11 },
    { 0x11840172, SUC_lexicon_tags_47 },
    { 0x11640174, SUC_lexicon_tags_14 },
    { 0xc7a40174, SUC_lexicon_tags_2 },
    { 0x3040176, SUC_lexicon_tags_14 },
    { 0x14a4016d, SUC_lexicon_tags_26 },
    { 0xefa40178, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7944017a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae4017d, SUC_lexicon_tags_4 },
    { 0xa084017e, SUC_lexicon_tags_37 },
    { 0xd9c4017e, SUC_lexicon_tags_37 },
    { 0x6c4017e, SUC_lexicon_tags_2 },
    { 0x33a4017e, SUC_lexicon_tags_8 },
    { 0x4844017d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a40185, SUC_lexicon_tags_1 },
    { 0x93040185, SUC_lexicon_tags_37 },
    { 0xf5240185, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9184018c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xed84018f, SUC_lexicon_tags_8 },
    { 0xa6240190, SUC_lexicon_tags_26 },
    { 0x3f040191, SUC_lexicon_tags_11 },
    { 0xbca40191, SUC_lexicon_tags_15 },
    { 0x85240190, SUC_lexicon_tags_2 },
    { 0xc5e40193, SUC_lexicon_tags_12 },
    { 0x65a40195, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5440198, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c4019b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c4019f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2401a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea401a6, SUC_lexicon_tags_12 },
    { 0xdd4401a6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa3e401a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732401b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b0401b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38c401b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc0401b7, SUC_lexicon_tags_36 },
    { 0x892401b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564401be, SUC_lexicon_tags_37 },
    { 0x600401bf, SUC_lexicon_tags_9 },
    { 0x3e8401be, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7a6401c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d0401c4, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c401c7, SUC_lexicon_tags_31 },
    { 0x5a4401c7, SUC_lexicon_tags_224 },
    { 0xb2401c8, SUC_lexicon_tags_47 },
    { 0x5f6401c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x968401cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798401d0, SUC_lexicon_tags_17 },
    { 0x401d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8401d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x658401d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c401da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96401e2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe4e401e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0401e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x310401ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32401ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48401f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb52401f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62401fb, SUC_lexicon_tags_19 },
    { 0xe64401fb, SUC_lexicon_tags_25 },
    { 0x7d6401fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a40200, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca40204, SUC_lexicon_tags_38 },
    { 0x12c40205, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf4040207, SUC_lexicon_tags_16 },
    { 0x6640208, SUC_lexicon_tags_21 },
    { 0x16040209, SUC_lexicon_tags_1 },
    { 0x4544020a, SUC_lexicon_tags_37 },
    { 0x2a04020b, SUC_lexicon_tags_5 },
    { 0x7ce40209, SUC_lexicon_tags_45 },
    { 0xd9e4020a, SUC_lexicon_tags_7 },
    { 0xdf64020e, SUC_lexicon_tags_11 },
    { 0x184020d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87e40211, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce40217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c4021a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5404021d, SUC_lexicon_tags_31 },
    { 0x7dc4021e, SUC_lexicon_tags_17 },
    { 0x8a44021e, SUC_lexicon_tags_6 },
    { 0xa944021d, SUC_lexicon_tags_23 },
    { 0x2ce4021f, SUC_lexicon_tags_252 },
    { 0xf3e4021e, SUC_lexicon_tags_43 },
    { 0x59440223, SUC_lexicon_tags_12 },
    { 0xbf840224, SUC_lexicon_tags_37 },
    { 0xda840223, SUC_lexicon_tags_1 },
    { 0x79840223, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc9a40228, SUC_lexicon_tags_60 },
    { 0xbba40229, SUC_lexicon_tags_27 },
    { 0xdaa4022a, SUC_lexicon_tags_14 },
    { 0x22040228, SUC_lexicon_tags_25 },
    { 0x4be4022a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2c4022e, SUC_lexicon_tags_43 },
    { 0xa944022f, SUC_lexicon_tags_46 },
    { 0xdb840230, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x26040232, SUC_lexicon_tags_12 },
    { 0x51440233, SUC_lexicon_tags_25 },
    { 0x23e40234, SUC_lexicon_tags_55 },
    { 0xb3240232, SUC_lexicon_tags_10 },
    { 0xe8840235, SUC_lexicon_tags_25 },
    { 0x1ba40237, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3640239, SUC_lexicon_tags_19 },
    { 0x2104023a, SUC_lexicon_tags_33 },
    { 0xce84023a, SUC_lexicon_tags_36 },
    { 0xfb64023c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x45c4023e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x77840240, SUC_lexicon_tags_7 },
    { 0x15640240, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc40246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78440249, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeb64024b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb764024f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x45040252, SUC_lexicon_tags_19 },
    { 0xada40252, SUC_lexicon_tags_15 },
    { 0xc6640254, SUC_lexicon_tags_28 },
    { 0xd4e40253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf240258, SUC_lexicon_tags_31 },
    { 0xcc040259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a4025c, SUC_lexicon_tags_70 },
    { 0xe624025d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa040262, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd440268, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3544026a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d040272, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x7da40274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c240276, SUC_lexicon_tags_9 },
    { 0x36e40277, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b640279, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe244027d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8504027f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1440282, SUC_lexicon_tags_33 },
    { 0x38c40283, SUC_lexicon_tags_11 },
    { 0x20a40284, SUC_lexicon_tags_9 },
    { 0xc1c40283, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7a440287, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8e40289, SUC_lexicon_tags_21 },
    { 0x8be4028a, SUC_lexicon_tags_6 },
    { 0xef04028a, SUC_lexicon_tags_11 },
    { 0x4d24028c, SUC_lexicon_tags_25 },
    { 0xc964028c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27640292, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f840295, SUC_lexicon_tags_0 },
    { 0x9c240296, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244029d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a402a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x812402a6, SUC_lexicon_tags_1 },
    { 0x132402a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6402aa, SUC_lexicon_tags_9 },
    { 0x78c402aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6402b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d2402b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8402b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x706402b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xace402b8, SUC_lexicon_tags_107 },
    { 0x5e6402b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x762402bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4402bf, SUC_lexicon_tags_27 },
    { 0x876402bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102402cb, SUC_lexicon_tags_19 },
    { 0xac8402cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce402cf, SUC_lexicon_tags_11 },
    { 0xa50402d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550402d9, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e402dc, SUC_lexicon_tags_61 },
    { 0x678402dc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x464402e0, SUC_lexicon_tags_7 },
    { 0xa0a402e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e402e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76402ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c402f3, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640402f9, SUC_lexicon_tags_0 },
    { 0xac2402fa, SUC_lexicon_tags_25 },
    { 0x902402fb, SUC_lexicon_tags_49 },
    { 0x784402fc, SUC_lexicon_tags_7 },
    { 0x75e402fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5640300, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xf5040302, SUC_lexicon_tags_57 },
    { 0x24840302, SUC_lexicon_tags_14 },
    { 0x2640304, SUC_lexicon_tags_28 },
    { 0x4e40303, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15040309, SUC_lexicon_tags_6 },
    { 0xbc84030a, SUC_lexicon_tags_65 },
    { 0x4564030b, SUC_lexicon_tags_11 },
    { 0x3c84030c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c840311, SUC_lexicon_tags_27 },
    { 0x7d840312, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c4031b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fa4031d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2904031f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x24240321, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b840324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaae40326, SUC_lexicon_tags_30 },
    { 0xa6240326, SUC_lexicon_tags_7 },
    { 0x2d640326, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4604032d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xab04032f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91240333, SUC_lexicon_tags_2 },
    { 0x67040334, SUC_lexicon_tags_17 },
    { 0xf8840335, SUC_lexicon_tags_37 },
    { 0x77a40336, SUC_lexicon_tags_37 },
    { 0x64440336, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x90240339, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac64033b, SUC_lexicon_tags_33 },
    { 0x96a4033c, SUC_lexicon_tags_56 },
    { 0xd884033c, SUC_lexicon_tags_38 },
    { 0x4ee4033e, SUC_lexicon_tags_25 },
    { 0xcb24033c, SUC_lexicon_tags_32 },
    { 0x18840340, SUC_lexicon_tags_37 },
    { 0x32640341, SUC_lexicon_tags_28 },
    { 0xc4640341, SUC_lexicon_tags_15 },
    { 0xcac40343, SUC_lexicon_tags_45 },
    { 0x18a40342, SUC_lexicon_tags_25 },
    { 0x5f040341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23240347, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54440355, SUC_lexicon_tags_14 },
    { 0x8e040355, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62a40358, SUC_lexicon_tags_8 },
    { 0x68240358, SUC_lexicon_tags_23 },
    { 0xdd840358, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a64035d, SUC_lexicon_tags_6 },
    { 0x98a4035d, SUC_lexicon_tags_11 },
    { 0x9d64035d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43840363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a64036a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c4036f, SUC_lexicon_tags_2 },
    { 0x8b040370, SUC_lexicon_tags_47 },
    { 0xc1840371, SUC_lexicon_tags_37 },
    { 0xf7c4036f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c40375, SUC_lexicon_tags_15 },
    { 0xc6840376, SUC_lexicon_tags_37 },
    { 0x2bc40377, SUC_lexicon_tags_23 },
    { 0xfe240378, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa4037d, SUC_lexicon_tags_14 },
    { 0xed84037e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e40384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb240386, SUC_lexicon_tags_37 },
    { 0x7f240387, SUC_lexicon_tags_25 },
    { 0x63440386, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3344038b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea84038d, SUC_lexicon_tags_25 },
    { 0x2404038d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c40393, SUC_lexicon_tags_7 },
    { 0xc40393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e40397, SUC_lexicon_tags_15 },
    { 0x93e40397, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17a4039a, SUC_lexicon_tags_7 },
    { 0x98c4039a, SUC_lexicon_tags_11 },
    { 0x3bc4039c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe64039e, SUC_lexicon_tags_16 },
    { 0xec04039e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20a403a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf68403a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc403ab, SUC_lexicon_tags_25 },
    { 0x7a0403ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e403af, SUC_lexicon_tags_15 },
    { 0x3e0403b0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc6e403b2, SUC_lexicon_tags_48 },
    { 0xb7c403b3, SUC_lexicon_tags_4 },
    { 0x7ce403b2, SUC_lexicon_tags_10 },
    { 0xa60403b2, SUC_lexicon_tags_0 },
    { 0x6a4403b4, SUC_lexicon_tags_9 },
    { 0xd06403b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc403c1, SUC_lexicon_tags_25 },
    { 0x96403c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366403cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92c403cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x880403d0, SUC_lexicon_tags_37 },
    { 0xaa0403d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2c403d3, SUC_lexicon_tags_37 },
    { 0x496403d4, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c403dd, SUC_lexicon_tags_182 },
    { 0x6cc403de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e403e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298403e6, SUC_lexicon_tags_2 },
    { 0x242403e7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8d6403e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x638403eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1cc403ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70403f0, SUC_lexicon_tags_17 },
    { 0x33a403f1, SUC_lexicon_tags_43 },
    { 0x6dc403f0, SUC_lexicon_tags_7 },
    { 0xe6403f3, SUC_lexicon_tags_61 },
    { 0xc8e403f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2403f7, SUC_lexicon_tags_7 },
    { 0xcdc403f8, SUC_lexicon_tags_11 },
    { 0x98a403f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4d8403fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26040401, SUC_lexicon_tags_19 },
    { 0xe9840401, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d840406, SUC_lexicon_tags_23 },
    { 0x12440407, SUC_lexicon_tags_18 },
    { 0x1d840408, SUC_lexicon_tags_13 },
    { 0x5240408, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc4040b, SUC_lexicon_tags_25 },
    { 0xc8e4040b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76040412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d840415, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c40418, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f840420, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a840429, SUC_lexicon_tags_46 },
    { 0x36e40429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd304042c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe640430, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc240435, SUC_lexicon_tags_16 },
    { 0x840435, SUC_lexicon_tags_25 },
    { 0xc7e40437, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a4043b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12240442, SUC_lexicon_tags_15 },
    { 0x13440443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc840445, SUC_lexicon_tags_31 },
    { 0xbe640446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e4044d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a40451, SUC_lexicon_tags_17 },
    { 0xec40451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc840455, SUC_lexicon_tags_2 },
    { 0x52e40456, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f04045d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1240462, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7240466, SUC_lexicon_tags_47 },
    { 0x43440466, SUC_lexicon_tags_80 },
    { 0x7f640468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e24046e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97240474, SUC_lexicon_tags_7 },
    { 0xcac40475, SUC_lexicon_tags_37 },
    { 0x3ce40475, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a4047a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a4047e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdd440480, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40040482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be40485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ac40487, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c440489, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2884048d, SUC_lexicon_tags_56 },
    { 0xcc84048e, SUC_lexicon_tags_95 },
    { 0x27a4048e, SUC_lexicon_tags_29 },
    { 0xf3e4048e, SUC_lexicon_tags_15 },
    { 0xa544048e, SUC_lexicon_tags_17 },
    { 0xaa640492, SUC_lexicon_tags_37 },
    { 0x9ca4048f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x29c40495, SUC_lexicon_tags_12 },
    { 0x4f240496, SUC_lexicon_tags_15 },
    { 0x34c40495, SUC_lexicon_tags_28 },
    { 0xdea40498, SUC_lexicon_tags_17 },
    { 0x65840498, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8024049b, SUC_lexicon_tags_12 },
    { 0x11a4049c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510404a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32404a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c404aa, SUC_lexicon_tags_33 },
    { 0x84a404ab, SUC_lexicon_tags_45 },
    { 0x6c404ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec404ae, SUC_lexicon_tags_15 },
    { 0x4a6404af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6404b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8c404b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de404ba, SUC_lexicon_tags_7 },
    { 0x93e404bb, SUC_lexicon_tags_65 },
    { 0x112404bc, SUC_lexicon_tags_65 },
    { 0x524404bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688404c2, SUC_lexicon_tags_15 },
    { 0x5ec404c3, SUC_lexicon_tags_60 },
    { 0x57e404c4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a404c7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10404ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee6404cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368404d0, SUC_lexicon_tags_1 },
    { 0x6f8404d1, SUC_lexicon_tags_7 },
    { 0xc84404d1, SUC_lexicon_tags_17 },
    { 0xc3c404d1, SUC_lexicon_tags_5 },
    { 0xab6404d3, SUC_lexicon_tags_1 },
    { 0xea8404d5, SUC_lexicon_tags_7 },
    { 0x40404d6, SUC_lexicon_tags_12 },
    { 0xa2404d4, SUC_lexicon_tags_25 },
    { 0x448404d3, SUC_lexicon_tags_14 },
    { 0x402404d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2e404db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462404e3, SUC_lexicon_tags_15 },
    { 0x2c8404e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae404e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de404ec, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6404ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c404f2, SUC_lexicon_tags_3 },
    { 0x888404f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46404fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe40500, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5640504, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e40508, SUC_lexicon_tags_87 },
    { 0x6ee40509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e84050c, SUC_lexicon_tags_61 },
    { 0xba24050d, SUC_lexicon_tags_30 },
    { 0x1e04050e, SUC_lexicon_tags_15 },
    { 0x54a4050f, SUC_lexicon_tags_25 },
    { 0x8ce40510, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a840519, SUC_lexicon_tags_15 },
    { 0x3e640519, SUC_lexicon_tags_124 },
    { 0xfbc4051b, SUC_lexicon_tags_30 },
    { 0x74a4051b, SUC_lexicon_tags_1 },
    { 0x87c4051b, SUC_lexicon_tags_1 },
    { 0x3c04051e, SUC_lexicon_tags_15 },
    { 0x2304051e, SUC_lexicon_tags_4 },
    { 0xe544051e, SUC_lexicon_tags_25 },
    { 0x1f040521, SUC_lexicon_tags_37 },
    { 0x7b840521, SUC_lexicon_tags_12 },
    { 0xbce40522, SUC_lexicon_tags_1 },
    { 0x14440521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a40527, SUC_lexicon_tags_87 },
    { 0x8a040528, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e4052b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a040530, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a40534, SUC_lexicon_tags_32 },
    { 0xd6240534, SUC_lexicon_tags_53 },
    { 0x9e440536, SUC_lexicon_tags_27 },
    { 0x29c40535, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e4053b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d04053d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac40542, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11040546, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee40549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa724054e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d840551, SUC_lexicon_tags_12 },
    { 0x62e40552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc040554, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8640558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27e4055a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e84055d, SUC_lexicon_tags_28 },
    { 0x2bc4055d, SUC_lexicon_tags_2 },
    { 0x8424055f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7240561, SUC_lexicon_tags_1 },
    { 0x4d640562, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37a40564, SUC_lexicon_tags_14 },
    { 0x14840564, SUC_lexicon_tags_37 },
    { 0x71e40566, SUC_lexicon_tags_11 },
    { 0x7f640567, SUC_lexicon_tags_12 },
    { 0xa840565, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb744056d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb84056f, SUC_lexicon_tags_37 },
    { 0x6a4056f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcee40572, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15640577, SUC_lexicon_tags_48 },
    { 0x25840578, SUC_lexicon_tags_19 },
    { 0x62e40577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7964057b, SUC_lexicon_tags_23 },
    { 0xc9e4057c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c40580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e040582, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfb640584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa040588, SUC_lexicon_tags_15 },
    { 0x93c40589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c04058d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeee4058f, SUC_lexicon_tags_21 },
    { 0x8040590, SUC_lexicon_tags_39 },
    { 0x4ee40591, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78640596, SUC_lexicon_tags_45 },
    { 0xef440597, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x16640599, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d84059b, SUC_lexicon_tags_43 },
    { 0xba64059c, SUC_lexicon_tags_26 },
    { 0xbbc4059b, SUC_lexicon_tags_2 },
    { 0x6084059b, SUC_lexicon_tags_2 },
    { 0x6a04059f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4405a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c405ad, SUC_lexicon_tags_2 },
    { 0xa405ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c405b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4405bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50405bf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc1e405c1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x322405c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02405c8, SUC_lexicon_tags_19 },
    { 0x62405c9, SUC_lexicon_tags_24 },
    { 0x84e405ca, SUC_lexicon_tags_66 },
    { 0x254405c9, SUC_lexicon_tags_1 },
    { 0xfd0405cc, SUC_lexicon_tags_25 },
    { 0x218405cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16a405cf, SUC_lexicon_tags_1 },
    { 0x178405d0, SUC_lexicon_tags_9 },
    { 0xf3c405d0, SUC_lexicon_tags_37 },
    { 0xd72405d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98405d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf68405d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04405db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc405e2, SUC_lexicon_tags_17 },
    { 0x8c0405e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c405e8, SUC_lexicon_tags_2 },
    { 0xaf8405e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x724405ec, SUC_lexicon_tags_12 },
    { 0x538405ed, SUC_lexicon_tags_9 },
    { 0xa80405ee, SUC_lexicon_tags_25 },
    { 0xf08405ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x740405f1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52405f6, SUC_lexicon_tags_31 },
    { 0x2ee405f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x160405f9, SUC_lexicon_tags_10 },
    { 0xc1e405fa, SUC_lexicon_tags_38 },
    { 0xf04405fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e240600, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec40606, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x17240609, SUC_lexicon_tags_12 },
    { 0xbc04060a, SUC_lexicon_tags_29 },
    { 0xede4060b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3440611, SUC_lexicon_tags_33 },
    { 0x57e40612, SUC_lexicon_tags_28 },
    { 0x33c40613, SUC_lexicon_tags_14 },
    { 0xb0440614, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb240617, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44061a, SUC_lexicon_tags_5 },
    { 0x5904061b, SUC_lexicon_tags_0 },
    { 0xa824061c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe524061f, SUC_lexicon_tags_26 },
    { 0x56e40620, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa40625, SUC_lexicon_tags_13 },
    { 0xdf840626, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x99040628, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5864062f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad840631, SUC_lexicon_tags_15 },
    { 0x36440632, SUC_lexicon_tags_23 },
    { 0x60840632, SUC_lexicon_tags_15 },
    { 0x4e640634, SUC_lexicon_tags_43 },
    { 0xc3e40635, SUC_lexicon_tags_27 },
    { 0xa6040635, SUC_lexicon_tags_55 },
    { 0x7cc40637, SUC_lexicon_tags_2 },
    { 0x7ee40633, SUC_lexicon_tags_83 },
    { 0x69640639, SUC_lexicon_tags_47 },
    { 0x6c840636, SUC_lexicon_tags_25 },
    { 0xa544063b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede40641, SUC_lexicon_tags_37 },
    { 0x56840642, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b640645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3240648, SUC_lexicon_tags_34 },
    { 0x1d240649, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f04065a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x384065c, SUC_lexicon_tags_65 },
    { 0xabe4065c, SUC_lexicon_tags_84 },
    { 0x14c4065d, SUC_lexicon_tags_43 },
    { 0xe1c4065f, SUC_lexicon_tags_17 },
    { 0x8a04065f, SUC_lexicon_tags_23 },
    { 0xb9040661, SUC_lexicon_tags_37 },
    { 0x1c44065d, SUC_lexicon_tags_23 },
    { 0x86440663, SUC_lexicon_tags_12 },
    { 0x9640661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea240666, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e4066c, SUC_lexicon_tags_25 },
    { 0x3664066d, SUC_lexicon_tags_25 },
    { 0xdd64066d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa40671, SUC_lexicon_tags_25 },
    { 0xc5840672, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc6c40674, SUC_lexicon_tags_27 },
    { 0xd6c40674, SUC_lexicon_tags_12 },
    { 0x98440675, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a4067a, SUC_lexicon_tags_1 },
    { 0x83c4067a, SUC_lexicon_tags_17 },
    { 0x1e4067c, SUC_lexicon_tags_25 },
    { 0x4cc4067c, SUC_lexicon_tags_15 },
    { 0xf044067e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c40681, SUC_lexicon_tags_63 },
    { 0x1840682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38a40684, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c240689, SUC_lexicon_tags_66 },
    { 0x7b04068a, SUC_lexicon_tags_29 },
    { 0xfee4068b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5004068f, SUC_lexicon_tags_25 },
    { 0xdf440690, SUC_lexicon_tags_0 },
    { 0xe2040691, SUC_lexicon_tags_29 },
    { 0xf424068f, SUC_lexicon_tags_30 },
    { 0xabe40690, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f240699, SUC_lexicon_tags_6 },
    { 0xe6240699, SUC_lexicon_tags_1 },
    { 0x16e4069a, SUC_lexicon_tags_26 },
    { 0x3904069c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12406a1, SUC_lexicon_tags_25 },
    { 0xe02406a1, SUC_lexicon_tags_17 },
    { 0xf16406a3, SUC_lexicon_tags_7 },
    { 0xbf8406a4, SUC_lexicon_tags_27 },
    { 0x44e406a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70406aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa56406ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f6406ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4406b1, SUC_lexicon_tags_67 },
    { 0x974406b2, SUC_lexicon_tags_28 },
    { 0xf76406b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c406b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04406c2, SUC_lexicon_tags_11 },
    { 0xcb2406c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46406c6, SUC_lexicon_tags_204 },
    { 0x38406c6, SUC_lexicon_tags_47 },
    { 0x722406c8, SUC_lexicon_tags_39 },
    { 0x20e406c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa406cd, SUC_lexicon_tags_2 },
    { 0x7e2406cd, SUC_lexicon_tags_14 },
    { 0x7fe406cd, SUC_lexicon_tags_35 },
    { 0xe52406d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9aa406d2, SUC_lexicon_tags_12 },
    { 0xf4c406d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xefe406d5, SUC_lexicon_tags_87 },
    { 0x910406d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x206406d8, SUC_lexicon_tags_2 },
    { 0xb14406d9, SUC_lexicon_tags_83 },
    { 0xbee406da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6406df, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c406e7, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74406ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4406f0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c406f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12406fd, SUC_lexicon_tags_27 },
    { 0x76406fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87040700, SUC_lexicon_tags_136 },
    { 0xc3240700, SUC_lexicon_tags_1 },
    { 0x92240702, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x33840705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea4070b, SUC_lexicon_tags_6 },
    { 0x6ee4070c, SUC_lexicon_tags_28 },
    { 0xe844070c, SUC_lexicon_tags_46 },
    { 0xf6c4070e, SUC_lexicon_tags_8 },
    { 0x7d64070d, SUC_lexicon_tags_70 },
    { 0x4564070e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c40715, SUC_lexicon_tags_7 },
    { 0x5640716, SUC_lexicon_tags_7 },
    { 0x5d240717, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98440719, SUC_lexicon_tags_1 },
    { 0xf1e4071a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c40722, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0x6ae40724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8240727, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73440729, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b24072b, SUC_lexicon_tags_37 },
    { 0x80e4072c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8464072f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d440733, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4040736, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5204073c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea40740, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd204074b, SUC_lexicon_tags_15 },
    { 0x44a4074c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc40758, SUC_lexicon_tags_17 },
    { 0xe040759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa924075c, SUC_lexicon_tags_37 },
    { 0xd0c4075c, SUC_lexicon_tags_28 },
    { 0x6384075c, SUC_lexicon_tags_15 },
    { 0xa3e4075f, SUC_lexicon_tags_43 },
    { 0xa5c40760, SUC_lexicon_tags_237 },
    { 0x5c040761, SUC_lexicon_tags_37 },
    { 0x68a40762, SUC_lexicon_tags_25 },
    { 0xde4075c, SUC_lexicon_tags_2 },
    { 0x2e440764, SUC_lexicon_tags_52 },
    { 0xe1440765, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c240769, SUC_lexicon_tags_46 },
    { 0x54a40769, SUC_lexicon_tags_15 },
    { 0x5404076a, SUC_lexicon_tags_48 },
    { 0x7144076a, SUC_lexicon_tags_45 },
    { 0xb884076d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c40772, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c40778, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104077d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec40780, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f840782, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae240788, SUC_lexicon_tags_19 },
    { 0x61e40788, SUC_lexicon_tags_11 },
    { 0x4fc4078a, SUC_lexicon_tags_25 },
    { 0xf8a4078a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44240791, SUC_lexicon_tags_40 },
    { 0x80840792, SUC_lexicon_tags_69 },
    { 0x6b040793, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1fe40795, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e24079a, SUC_lexicon_tags_37 },
    { 0x3e4079b, SUC_lexicon_tags_65 },
    { 0x4184079c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5164079f, SUC_lexicon_tags_17 },
    { 0x2c6407a0, SUC_lexicon_tags_40 },
    { 0xb7c407a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0407a8, SUC_lexicon_tags_45 },
    { 0xd0e407a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x746407ab, SUC_lexicon_tags_47 },
    { 0xb32407ac, SUC_lexicon_tags_11 },
    { 0xfe0407ad, SUC_lexicon_tags_47 },
    { 0x458407ae, SUC_lexicon_tags_34 },
    { 0x68a407ac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4407b2, SUC_lexicon_tags_7 },
    { 0xfc6407b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcdc407b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x690407b8, SUC_lexicon_tags_87 },
    { 0x8dc407b9, SUC_lexicon_tags_14 },
    { 0x750407ba, SUC_lexicon_tags_12 },
    { 0x72a407bb, SUC_lexicon_tags_28 },
    { 0x25e407bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x440407be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da407c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348407c7, SUC_lexicon_tags_37 },
    { 0xc02407c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4407cb, SUC_lexicon_tags_82 },
    { 0x886407cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1e407ce, SUC_lexicon_tags_1 },
    { 0x5ea407ce, SUC_lexicon_tags_17 },
    { 0x682407d0, SUC_lexicon_tags_15 },
    { 0x31a407ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c407d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29e407d8, SUC_lexicon_tags_12 },
    { 0x918407d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x312407dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174407e1, SUC_lexicon_tags_47 },
    { 0x7a2407e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00407e5, SUC_lexicon_tags_9 },
    { 0x3c8407e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70407ed, SUC_lexicon_tags_38 },
    { 0xee8407ee, SUC_lexicon_tags_7 },
    { 0xbd2407ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6aa407f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8407f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x834407f6, SUC_lexicon_tags_17 },
    { 0xe78407f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8407fd, SUC_lexicon_tags_63 },
    { 0x612407fd, SUC_lexicon_tags_26 },
    { 0x390407ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1fa40801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16040803, SUC_lexicon_tags_87 },
    { 0x94040803, SUC_lexicon_tags_27 },
    { 0xfd440805, SUC_lexicon_tags_7 },
    { 0xb8840804, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c40812, SUC_lexicon_tags_33 },
    { 0x3a640813, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56e40815, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4204081b, SUC_lexicon_tags_29 },
    { 0xf5e4081b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb384081f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ce40821, SUC_lexicon_tags_35 },
    { 0x48640821, SUC_lexicon_tags_12 },
    { 0x40040823, SUC_lexicon_tags_16 },
    { 0xc2840823, SUC_lexicon_tags_25 },
    { 0x82040823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a40829, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e4082c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51640832, SUC_lexicon_tags_32 },
    { 0xd440833, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a40836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1840838, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea40840, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea40843, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdc440845, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea64084d, SUC_lexicon_tags_11 },
    { 0xa4c4084e, SUC_lexicon_tags_11 },
    { 0x69a4084f, SUC_lexicon_tags_27 },
    { 0x52040850, SUC_lexicon_tags_10 },
    { 0xa7640851, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e40857, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x13c40859, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb104085b, SUC_lexicon_tags_15 },
    { 0xeda4085c, SUC_lexicon_tags_11 },
    { 0x7864085d, SUC_lexicon_tags_1 },
    { 0xb084085b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x26640860, SUC_lexicon_tags_1 },
    { 0xb8240861, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f440866, SUC_lexicon_tags_38 },
    { 0x94040866, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaca40869, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b24086b, SUC_lexicon_tags_69 },
    { 0xdc4086c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f84086f, SUC_lexicon_tags_32 },
    { 0x2a04086f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7c840872, SUC_lexicon_tags_60 },
    { 0x8a40873, SUC_lexicon_tags_25 },
    { 0xc4a40873, SUC_lexicon_tags_15 },
    { 0xfec40874, SUC_lexicon_tags_1 },
    { 0x640873, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e40880, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a40883, SUC_lexicon_tags_23 },
    { 0x5e40883, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a4088b, SUC_lexicon_tags_29 },
    { 0x4e04088b, SUC_lexicon_tags_34 },
    { 0x62c4088d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6240891, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d240893, SUC_lexicon_tags_25 },
    { 0x1c440894, SUC_lexicon_tags_23 },
    { 0x93440894, SUC_lexicon_tags_26 },
    { 0xea440896, SUC_lexicon_tags_17 },
    { 0x73440897, SUC_lexicon_tags_14 },
    { 0xe0040897, SUC_lexicon_tags_17 },
    { 0xe9e40899, SUC_lexicon_tags_26 },
    { 0xf304089a, SUC_lexicon_tags_2 },
    { 0xd44089b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a408a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x678408a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8408a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x688408ab, SUC_lexicon_tags_11 },
    { 0xa70408ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418408b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x354408b2, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98408b7, SUC_lexicon_tags_2 },
    { 0xfe6408b8, SUC_lexicon_tags_11 },
    { 0x740408b9, SUC_lexicon_tags_43 },
    { 0x456408ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4408be, SUC_lexicon_tags_23 },
    { 0x880408be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a408c5, SUC_lexicon_tags_8 },
    { 0x51a408c6, SUC_lexicon_tags_37 },
    { 0x4f4408c7, SUC_lexicon_tags_31 },
    { 0x172408c8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8408ce, SUC_lexicon_tags_1 },
    { 0x3c2408ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce408d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a408de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8408e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba408e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2408ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a408f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x352408f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4408f8, SUC_lexicon_tags_60 },
    { 0xdc6408f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43040904, SUC_lexicon_tags_15 },
    { 0xebe40905, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e4090a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c4090d, SUC_lexicon_tags_16 },
    { 0xf864090e, SUC_lexicon_tags_37 },
    { 0x2064090e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23640913, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54240915, SUC_lexicon_tags_25 },
    { 0x58840916, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32640918, SUC_lexicon_tags_47 },
    { 0x32a40919, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7844091b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13640922, SUC_lexicon_tags_43 },
    { 0xc8640922, SUC_lexicon_tags_2 },
    { 0xc8e40922, SUC_lexicon_tags_10 },
    { 0x9d440925, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c40928, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac4092c, SUC_lexicon_tags_7 },
    { 0xd924092d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5240931, SUC_lexicon_tags_65 },
    { 0xf6c40931, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3440936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdba40938, SUC_lexicon_tags_14 },
    { 0x7fe40939, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a40940, SUC_lexicon_tags_37 },
    { 0x21040940, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c640944, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0240948, SUC_lexicon_tags_8 },
    { 0x13a40949, SUC_lexicon_tags_46 },
    { 0x59c4094a, SUC_lexicon_tags_4 },
    { 0xd6a4094a, SUC_lexicon_tags_14 },
    { 0xb424094c, SUC_lexicon_tags_15 },
    { 0x1904094c, SUC_lexicon_tags_47 },
    { 0x65a4094d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf0a40950, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11440955, SUC_lexicon_tags_61 },
    { 0xd8640956, SUC_lexicon_tags_27 },
    { 0x12440957, SUC_lexicon_tags_68 },
    { 0x4a040958, SUC_lexicon_tags_0 },
    { 0x7b640959, SUC_lexicon_tags_37 },
    { 0xda40959, SUC_lexicon_tags_14 },
    { 0xc924095a, SUC_lexicon_tags_37 },
    { 0x67c4095c, SUC_lexicon_tags_10 },
    { 0xa844095c, SUC_lexicon_tags_15 },
    { 0x6ea40959, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7440965, SUC_lexicon_tags_23 },
    { 0x5e240966, SUC_lexicon_tags_25 },
    { 0x9de40966, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ee40969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1584096c, SUC_lexicon_tags_47 },
    { 0x1bc4096d, SUC_lexicon_tags_17 },
    { 0x1e04096e, SUC_lexicon_tags_12 },
    { 0x4096d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac40973, SUC_lexicon_tags_7 },
    { 0x41240974, SUC_lexicon_tags_6 },
    { 0x16040974, SUC_lexicon_tags_63 },
    { 0x50a40974, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c24097a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6840980, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x95840982, SUC_lexicon_tags_1 },
    { 0xbc640983, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58840988, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c4098f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x11840991, SUC_lexicon_tags_21 },
    { 0x2b440991, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdbe40994, SUC_lexicon_tags_38 },
    { 0x39e40995, SUC_lexicon_tags_47 },
    { 0x82840995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b840999, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c4099c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578409a0, SUC_lexicon_tags_60 },
    { 0x98c409a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532409a5, SUC_lexicon_tags_31 },
    { 0x54a409a6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974409ac, SUC_lexicon_tags_25 },
    { 0x29e409ac, SUC_lexicon_tags_2 },
    { 0x6a6409ae, SUC_lexicon_tags_55 },
    { 0x5f6409af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6409b5, SUC_lexicon_tags_0 },
    { 0xfae409b6, SUC_lexicon_tags_146 },
    { 0xd92409b6, SUC_lexicon_tags_15 },
    { 0xae8409b6, SUC_lexicon_tags_28 },
    { 0xca0409b9, SUC_lexicon_tags_47 },
    { 0x920409b6, SUC_lexicon_tags_18 },
    { 0x8f6409b8, SUC_lexicon_tags_16 },
    { 0x960409bc, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4409bf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772409c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2409c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c8409c9, SUC_lexicon_tags_25 },
    { 0xa0c409ca, SUC_lexicon_tags_37 },
    { 0x3ce409ca, SUC_lexicon_tags_11 },
    { 0x39c409cc, SUC_lexicon_tags_8 },
    { 0xb18409ca, SUC_lexicon_tags_63 },
    { 0x580409ce, SUC_lexicon_tags_0 },
    { 0xea4409cc, SUC_lexicon_tags_28 },
    { 0x364409d0, SUC_lexicon_tags_82 },
    { 0x5a409cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6409d5, SUC_lexicon_tags_27 },
    { 0x50c409d6, SUC_lexicon_tags_2 },
    { 0xa00409d7, SUC_lexicon_tags_15 },
    { 0x422409d8, SUC_lexicon_tags_25 },
    { 0x1fc409d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65c409db, SUC_lexicon_tags_1 },
    { 0x428409dc, SUC_lexicon_tags_32 },
    { 0x4c4409dc, SUC_lexicon_tags_1 },
    { 0xdf8409de, SUC_lexicon_tags_38 },
    { 0x6ee409df, SUC_lexicon_tags_25 },
    { 0xf4e409de, SUC_lexicon_tags_59 },
    { 0xf4a409e0, SUC_lexicon_tags_2 },
    { 0x21a409dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee409e5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e409e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x644409eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0409f1, SUC_lexicon_tags_1 },
    { 0x6d4409f1, SUC_lexicon_tags_13 },
    { 0x654409f2, SUC_lexicon_tags_19 },
    { 0xdb2409f2, SUC_lexicon_tags_33 },
    { 0x2c409f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e409f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x660409fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3e6409fd, SUC_lexicon_tags_17 },
    { 0xb04409fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac40a06, SUC_lexicon_tags_12 },
    { 0x12640a06, SUC_lexicon_tags_0 },
    { 0xb5e40a07, SUC_lexicon_tags_13 },
    { 0x4f640a09, SUC_lexicon_tags_48 },
    { 0xf5440a07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e40a11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb240a13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf840a16, SUC_lexicon_tags_11 },
    { 0xa9840a17, SUC_lexicon_tags_15 },
    { 0x1c040a18, SUC_lexicon_tags_15 },
    { 0xd3640a19, SUC_lexicon_tags_17 },
    { 0xdd440a1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1840a1e, SUC_lexicon_tags_1 },
    { 0x8bc40a1f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbe840a21, SUC_lexicon_tags_37 },
    { 0x76440a22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4240a25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43840a27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab240a2b, SUC_lexicon_tags_14 },
    { 0x5c40a2c, SUC_lexicon_tags_9 },
    { 0xca840a2b, SUC_lexicon_tags_17 },
    { 0xc0440a2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71840a31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed440a34, SUC_lexicon_tags_40 },
    { 0xa840a35, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7840a38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61e40a3a, SUC_lexicon_tags_7 },
    { 0x63040a3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d440a3d, SUC_lexicon_tags_26 },
    { 0x5f840a3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e40a41, SUC_lexicon_tags_27 },
    { 0x1a440a41, SUC_lexicon_tags_38 },
    { 0x55040a43, SUC_lexicon_tags_47 },
    { 0xb6440a43, SUC_lexicon_tags_17 },
    { 0x2bc40a45, SUC_lexicon_tags_15 },
    { 0x36040a42, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3440a49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9640a4b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3840a4f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7840a54, SUC_lexicon_tags_0 },
    { 0x4040a54, SUC_lexicon_tags_25 },
    { 0x3cc40a56, SUC_lexicon_tags_37 },
    { 0xdfe40a57, SUC_lexicon_tags_30 },
    { 0x67c40a56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe440a5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb040a5d, SUC_lexicon_tags_8 },
    { 0xdba40a5e, SUC_lexicon_tags_19 },
    { 0xcc640a5f, SUC_lexicon_tags_1 },
    { 0x66a40a5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcc40a62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce40a65, SUC_lexicon_tags_39 },
    { 0xcd440a66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed640a68, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcec40a6a, SUC_lexicon_tags_7 },
    { 0xbca40a6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbac40a6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b840a73, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c40a7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b440a84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6040a89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9240a8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a40a92, SUC_lexicon_tags_7 },
    { 0x75e40a93, SUC_lexicon_tags_38 },
    { 0x9c840a92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e40a9a, SUC_lexicon_tags_63 },
    { 0x9aa40a9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x62c40a9d, SUC_lexicon_tags_60 },
    { 0xd1640a9e, SUC_lexicon_tags_28 },
    { 0xd7e40a9e, SUC_lexicon_tags_25 },
    { 0x81c40a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88840aa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a40aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8640aa6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x76640aa9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca40aad, SUC_lexicon_tags_17 },
    { 0xf0c40aad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6040ab0, SUC_lexicon_tags_25 },
    { 0xc6040ab1, SUC_lexicon_tags_60 },
    { 0x39c40ab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b440ab4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70840ab6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1240aba, SUC_lexicon_tags_12 },
    { 0x9640abb, SUC_lexicon_tags_1 },
    { 0xd9c40abc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b040abf, SUC_lexicon_tags_19 },
    { 0x70c40ac0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee40ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2040ac7, SUC_lexicon_tags_8 },
    { 0x88840ac7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1040acb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8640ace, SUC_lexicon_tags_14 },
    { 0xc3c40acf, SUC_lexicon_tags_12 },
    { 0xa2040ad0, SUC_lexicon_tags_12 },
    { 0x1ce40ad1, SUC_lexicon_tags_57 },
    { 0xb1c40ace, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21840ad6, SUC_lexicon_tags_38 },
    { 0x51e40ad7, SUC_lexicon_tags_4 },
    { 0x3b240ad6, SUC_lexicon_tags_47 },
    { 0xa040ad9, SUC_lexicon_tags_29 },
    { 0xd5a40ad6, SUC_lexicon_tags_25 },
    { 0x80640adb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea40adf, SUC_lexicon_tags_37 },
    { 0xace40ae0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6640ae4, SUC_lexicon_tags_37 },
    { 0xe3840ae5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91640ae9, SUC_lexicon_tags_7 },
    { 0x20e40ae9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51040aed, SUC_lexicon_tags_7 },
    { 0xf5640aee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a840af3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c40af7, SUC_lexicon_tags_39 },
    { 0xcf440af7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a240afa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93240afe, SUC_lexicon_tags_47 },
    { 0x29e40aff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2840b05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a40b08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb840b0c, SUC_lexicon_tags_61 },
    { 0x1f240b0c, SUC_lexicon_tags_151 },
    { 0xf4440b0e, SUC_lexicon_tags_15 },
    { 0xff440b0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca40b12, SUC_lexicon_tags_11 },
    { 0x38c40b13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46440b18, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb840b1a, SUC_lexicon_tags_1 },
    { 0x9b640b1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e40b1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c40b22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c40b25, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d640b2a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75840b33, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c40b3a, SUC_lexicon_tags_56 },
    { 0xe1c40b3a, SUC_lexicon_tags_37 },
    { 0xb4c40b3b, SUC_lexicon_tags_15 },
    { 0xdce40b3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf240b3f, SUC_lexicon_tags_28 },
    { 0x91240b40, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf1e40b42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc040b4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c40b53, SUC_lexicon_tags_7 },
    { 0xbcc40b53, SUC_lexicon_tags_25 },
    { 0xb8240b54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea40b59, SUC_lexicon_tags_30 },
    { 0x88a40b5a, SUC_lexicon_tags_34 },
    { 0x61440b5b, SUC_lexicon_tags_25 },
    { 0xa3e40b5c, SUC_lexicon_tags_48 },
    { 0xc8640b5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c640b60, SUC_lexicon_tags_15 },
    { 0x5f040b61, SUC_lexicon_tags_1 },
    { 0x9fc40b61, SUC_lexicon_tags_23 },
    { 0x51c40b62, SUC_lexicon_tags_3 },
    { 0xda040b64, SUC_lexicon_tags_38 },
    { 0xf5240b65, SUC_lexicon_tags_11 },
    { 0xb9e40b63, SUC_lexicon_tags_87 },
    { 0xe0c40b67, SUC_lexicon_tags_45 },
    { 0x89a40b68, SUC_lexicon_tags_11 },
    { 0xc2040b68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5c40b6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a640b6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42640b78, SUC_lexicon_tags_16 },
    { 0xcb040b78, SUC_lexicon_tags_1 },
    { 0xaa40b7a, SUC_lexicon_tags_17 },
    { 0x1e040b7b, SUC_lexicon_tags_47 },
    { 0xdd040b7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4f640b7e, SUC_lexicon_tags_11 },
    { 0xdd240b7f, SUC_lexicon_tags_17 },
    { 0x1fa40b7f, SUC_lexicon_tags_26 },
    { 0xd4a40b7f, SUC_lexicon_tags_15 },
    { 0xddc40b7f, SUC_lexicon_tags_37 },
    { 0x86e40b83, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86440b88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae40b8a, SUC_lexicon_tags_8 },
    { 0x8ee40b8a, SUC_lexicon_tags_47 },
    { 0x5ea40b8b, SUC_lexicon_tags_25 },
    { 0xf8a40b8d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de40b93, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfea40b95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90240b99, SUC_lexicon_tags_11 },
    { 0x6f640b99, SUC_lexicon_tags_4 },
    { 0x6ea40b9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1640b9e, SUC_lexicon_tags_1 },
    { 0x72240b9e, SUC_lexicon_tags_29 },
    { 0xaee40ba0, SUC_lexicon_tags_14 },
    { 0xac040ba0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb440ba3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78440baa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51840bad, SUC_lexicon_tags_42 },
    { 0xe0e40bae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b040bb8, SUC_lexicon_tags_12 },
    { 0xe5040bb8, SUC_lexicon_tags_0 },
    { 0x2fe40bba, SUC_lexicon_tags_37 },
    { 0xbc240bbb, SUC_lexicon_tags_2 },
    { 0x90840bbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47240bc0, SUC_lexicon_tags_37 },
    { 0xd5a40bc1, SUC_lexicon_tags_2 },
    { 0xd3a40bc2, SUC_lexicon_tags_17 },
    { 0x3d840bc2, SUC_lexicon_tags_12 },
    { 0xcc240bc4, SUC_lexicon_tags_30 },
    { 0x86240bc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68240bc7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba40bcf, SUC_lexicon_tags_8 },
    { 0xed240bd0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a40bd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e40bda, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a40be2, SUC_lexicon_tags_7 },
    { 0x20240be3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56a40be5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f240be8, SUC_lexicon_tags_39 },
    { 0xab440be9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c40bed, SUC_lexicon_tags_32 },
    { 0xb3640bee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63a40bf0, SUC_lexicon_tags_55 },
    { 0x87640bf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36e40bf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f040bf6, SUC_lexicon_tags_92 },
    { 0x9ca40bf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ea40bf9, SUC_lexicon_tags_11 },
    { 0x55a40bfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39640c00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c40c07, SUC_lexicon_tags_28 },
    { 0xa0c40c07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a840c0d, SUC_lexicon_tags_11 },
    { 0x73e40c0d, SUC_lexicon_tags_11 },
    { 0xa5a40c0f, SUC_lexicon_tags_33 },
    { 0x6b240c10, SUC_lexicon_tags_11 },
    { 0xfd840c0f, SUC_lexicon_tags_6 },
    { 0x54040c11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54040c15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e40c18, SUC_lexicon_tags_17 },
    { 0x38640c19, SUC_lexicon_tags_14 },
    { 0xa3440c19, SUC_lexicon_tags_37 },
    { 0x54240c1b, SUC_lexicon_tags_34 },
    { 0x1ce40c1c, SUC_lexicon_tags_37 },
    { 0xbfa40c1d, SUC_lexicon_tags_37 },
    { 0x9c640c1e, SUC_lexicon_tags_11 },
    { 0x8c840c1b, SUC_lexicon_tags_40 },
    { 0xf6e40c1f, SUC_lexicon_tags_230 },
    { 0x25040c21, SUC_lexicon_tags_27 },
    { 0x5ea40c21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a40c25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71840c2b, SUC_lexicon_tags_37 },
    { 0x96640c2c, SUC_lexicon_tags_2 },
    { 0x7c440c2b, SUC_lexicon_tags_57 },
    { 0xec640c2e, SUC_lexicon_tags_21 },
    { 0x70240c2b, SUC_lexicon_tags_1 },
    { 0x8ee40c30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ae40c32, SUC_lexicon_tags_33 },
    { 0x5ae40c33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6840c39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c40c3f, SUC_lexicon_tags_38 },
    { 0x4aa40c40, SUC_lexicon_tags_66 },
    { 0x8a640c40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28840c43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86240c45, SUC_lexicon_tags_45 },
    { 0x52a40c45, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd040c4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f240c53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32240c57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84240c59, SUC_lexicon_tags_7 },
    { 0x7ea40c5a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c40c5d, SUC_lexicon_tags_26 },
    { 0xaaa40c5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a40c64, SUC_lexicon_tags_14 },
    { 0xb3640c65, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x63a40c67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb9640c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55840c6b, SUC_lexicon_tags_7 },
    { 0x64a40c6c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe040c6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48040c73, SUC_lexicon_tags_60 },
    { 0x16040c73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42240c78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b840c7b, SUC_lexicon_tags_63 },
    { 0xbe040c7b, SUC_lexicon_tags_21 },
    { 0xc040c7c, SUC_lexicon_tags_17 },
    { 0xdb640c7c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd3e40c80, SUC_lexicon_tags_1 },
    { 0x9d040c80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6440c89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa640c8c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed440c91, SUC_lexicon_tags_1 },
    { 0xa8a40c91, SUC_lexicon_tags_11 },
    { 0xa1e40c93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24840c95, SUC_lexicon_tags_46 },
    { 0x79c40c96, SUC_lexicon_tags_1 },
    { 0xf2040c97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa240c99, SUC_lexicon_tags_9 },
    { 0xf1240c9a, SUC_lexicon_tags_34 },
    { 0x6c40c9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe40c9e, SUC_lexicon_tags_23 },
    { 0x13840c9f, SUC_lexicon_tags_25 },
    { 0xf4c40ca0, SUC_lexicon_tags_13 },
    { 0x10440ca0, SUC_lexicon_tags_33 },
    { 0x6b040ca2, SUC_lexicon_tags_11 },
    { 0xf1640ca3, SUC_lexicon_tags_37 },
    { 0xb6640ca3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13840ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c640ca8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f240cab, SUC_lexicon_tags_28 },
    { 0x7e40cab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ba40cae, SUC_lexicon_tags_68 },
    { 0xcf840cae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7040cb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a40cb4, SUC_lexicon_tags_4 },
    { 0x33c40cb5, SUC_lexicon_tags_0 },
    { 0xc3c40cb5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e240cbc, SUC_lexicon_tags_23 },
    { 0xc5e40cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b840cbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc40cc2, SUC_lexicon_tags_38 },
    { 0xe8240cc2, SUC_lexicon_tags_42 },
    { 0xefc40cc3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xb840cc6, SUC_lexicon_tags_26 },
    { 0x12240cc7, SUC_lexicon_tags_60 },
    { 0xcb040cc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6040cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51240cd2, SUC_lexicon_tags_37 },
    { 0xbe840cd3, SUC_lexicon_tags_1 },
    { 0x85240cd4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe0240cd6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x15440cd9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x10040cdc, SUC_lexicon_tags_1 },
    { 0x68240cdc, SUC_lexicon_tags_27 },
    { 0xa7040cde, SUC_lexicon_tags_7 },
    { 0x4ca40cde, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2ee40ce1, SUC_lexicon_tags_7 },
    { 0xaf040ce2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7e40ce4, SUC_lexicon_tags_8 },
    { 0xf8440ce5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58840ce7, SUC_lexicon_tags_28 },
    { 0x55240ce8, SUC_lexicon_tags_84 },
    { 0xec240ce9, SUC_lexicon_tags_2 },
    { 0xfcc40ce7, SUC_lexicon_tags_11 },
    { 0xa2e40ce7, SUC_lexicon_tags_17 },
    { 0x4d640ceb, SUC_lexicon_tags_25 },
    { 0xc4440cec, SUC_lexicon_tags_28 },
    { 0x9d040cec, SUC_lexicon_tags_17 },
    { 0xcb440cef, SUC_lexicon_tags_27 },
    { 0x22640ced, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4840cf6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc40cfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee440d01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e640d03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a240d07, SUC_lexicon_tags_63 },
    { 0x9e240d08, SUC_lexicon_tags_1 },
    { 0xd9840d09, SUC_lexicon_tags_37 },
    { 0x25840d09, SUC_lexicon_tags_4 },
    { 0x60640d08, SUC_lexicon_tags_23 },
    { 0x58c40d0a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcfc40d0e, SUC_lexicon_tags_17 },
    { 0x23040d0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4640d11, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdb240d13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcbc40d15, SUC_lexicon_tags_31 },
    { 0xb5840d16, SUC_lexicon_tags_27 },
    { 0x3f040d17, SUC_lexicon_tags_1 },
    { 0xeba40d15, SUC_lexicon_tags_11 },
    { 0x57840d19, SUC_lexicon_tags_42 },
    { 0x7e240d1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6440d1c, SUC_lexicon_tags_25 },
    { 0x5b840d1d, SUC_lexicon_tags_12 },
    { 0x67640d1d, SUC_lexicon_tags_11 },
    { 0x6c640d1f, SUC_lexicon_tags_43 },
    { 0x36040d1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e040d24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d040d27, SUC_lexicon_tags_31 },
    { 0x5840d28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc40d2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b840d2e, SUC_lexicon_tags_1 },
    { 0x8ba40d2f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0840d34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x35640d36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5be40d38, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc640d41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcb040d43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a840d4b, SUC_lexicon_tags_54 },
    { 0x6a40d4c, SUC_lexicon_tags_45 },
    { 0x72240d4d, SUC_lexicon_tags_47 },
    { 0x47240d4e, SUC_lexicon_tags_37 },
    { 0x9640d4f, SUC_lexicon_tags_7 },
    { 0xea440d4f, SUC_lexicon_tags_1 },
    { 0xf2c40d51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8240d53, SUC_lexicon_tags_25 },
    { 0x3b040d54, SUC_lexicon_tags_47 },
    { 0xdf840d53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8840d5c, SUC_lexicon_tags_37 },
    { 0x14440d5d, SUC_lexicon_tags_19 },
    { 0x8aa40d5e, SUC_lexicon_tags_49 },
    { 0x5c040d5f, SUC_lexicon_tags_6 },
    { 0x9ec40d5f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1d240d62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1240d64, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xa4840d66, SUC_lexicon_tags_25 },
    { 0x8ca40d66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca040d6c, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e40d72, SUC_lexicon_tags_32 },
    { 0x14c40d72, SUC_lexicon_tags_49 },
    { 0xaf240d73, SUC_lexicon_tags_66 },
    { 0x5a40d75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42440d79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85e40d7b, SUC_lexicon_tags_17 },
    { 0x1a240d7b, SUC_lexicon_tags_10 },
    { 0x3aa40d7c, SUC_lexicon_tags_1 },
    { 0x10e40d7e, SUC_lexicon_tags_19 },
    { 0xfc840d7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x39040d81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91240d85, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed040d8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5840d94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed040d97, SUC_lexicon_tags_13 },
    { 0xe3e40d97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21640d9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4440d9e, SUC_lexicon_tags_14 },
    { 0xb7440d9f, SUC_lexicon_tags_15 },
    { 0xa4040da0, SUC_lexicon_tags_21 },
    { 0xe6a40da1, SUC_lexicon_tags_9 },
    { 0xe5a40da1, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x2c40da4, SUC_lexicon_tags_11 },
    { 0xd1040da4, SUC_lexicon_tags_37 },
    { 0x1d840da6, SUC_lexicon_tags_33 },
    { 0xf8040da7, SUC_lexicon_tags_55 },
    { 0x49040da6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40240dad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf0840daf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x83440db1, SUC_lexicon_tags_54 },
    { 0xc4c40db2, SUC_lexicon_tags_30 },
    { 0x62840db1, SUC_lexicon_tags_2 },
    { 0x1ea40db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7c40db6, SUC_lexicon_tags_16 },
    { 0x2cc40db7, SUC_lexicon_tags_25 },
    { 0x10e40db8, SUC_lexicon_tags_63 },
    { 0x35e40db9, SUC_lexicon_tags_30 },
    { 0x59040db9, SUC_lexicon_tags_7 },
    { 0xc8240dbb, SUC_lexicon_tags_17 },
    { 0xe2240dbc, SUC_lexicon_tags_1 },
    { 0x35040dbd, SUC_lexicon_tags_25 },
    { 0xab240db7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa8440dc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7240dc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9840dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c40dcd, SUC_lexicon_tags_17 },
    { 0x54e40dce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1040dd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23640dd6, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1040dde, SUC_lexicon_tags_33 },
    { 0x8c240dde, SUC_lexicon_tags_17 },
    { 0x93c40de0, SUC_lexicon_tags_0 },
    { 0xb6040de1, SUC_lexicon_tags_27 },
    { 0x6c440ddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d840de5, SUC_lexicon_tags_25 },
    { 0xc2c40de6, SUC_lexicon_tags_2 },
    { 0xa40de7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa0e40de9, SUC_lexicon_tags_11 },
    { 0xb8a40dea, SUC_lexicon_tags_2 },
    { 0x52840deb, SUC_lexicon_tags_33 },
    { 0xf5440deb, SUC_lexicon_tags_37 },
    { 0x22e40de9, SUC_lexicon_tags_11 },
    { 0xa2840deb, SUC_lexicon_tags_77 },
    { 0x85240def, SUC_lexicon_tags_15 },
    { 0xc1040de9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe40df4, SUC_lexicon_tags_7 },
    { 0xac840df5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c040df7, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x27240dfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9040e01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6840e04, SUC_lexicon_tags_7 },
    { 0xe6a40e04, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa7c40e07, SUC_lexicon_tags_1 },
    { 0x5a240e08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67440e0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55040e10, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x57840e12, SUC_lexicon_tags_15 },
    { 0x89640e13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92640e15, SUC_lexicon_tags_71 },
    { 0xc6240e16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9240e1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfe40e1d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84440e21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a40e24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4840e27, SUC_lexicon_tags_39 },
    { 0x9fe40e27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xae240e2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5ee40e2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf440e2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8840e32, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb6640e34, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c040e3b, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0xe1640e3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e240e44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e40e48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c40e52, SUC_lexicon_tags_1322 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da40e55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6840e57, SUC_lexicon_tags_17 },
    { 0x55840e58, SUC_lexicon_tags_17 },
    { 0x7e040e59, SUC_lexicon_tags_39 },
    { 0xa2840e58, SUC_lexicon_tags_47 },
    { 0x58a40e5b, SUC_lexicon_tags_29 },
    { 0x96440e57, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e40e61, SUC_lexicon_tags_23 },
    { 0x92a40e61, SUC_lexicon_tags_7 },
    { 0xb2640e62, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62040e67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdf440e69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1440e72, SUC_lexicon_tags_47 },
    { 0xec640e73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xafa40e75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1a40e77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da40e7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a40e7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee40e80, SUC_lexicon_tags_44 },
    { 0xb8c40e81, SUC_lexicon_tags_27 },
    { 0xe2440e80, SUC_lexicon_tags_15 },
    { 0xd640e81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcee40e85, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x78440e87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f240e8d, SUC_lexicon_tags_1 },
    { 0x9f640e8d, SUC_lexicon_tags_17 },
    { 0xa0840e8f, SUC_lexicon_tags_62 },
    { 0xd7640e90, SUC_lexicon_tags_23 },
    { 0x5c440e8f, SUC_lexicon_tags_9 },
    { 0xe0e40e92, SUC_lexicon_tags_37 },
    { 0x6640e93, SUC_lexicon_tags_19 },
    { 0xd5a40e93, SUC_lexicon_tags_37 },
    { 0xb6240e92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31e40e97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea40e9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e40e9f, SUC_lexicon_tags_25 },
    { 0xc040ea0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x61640ea2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaa040ea4, SUC_lexicon_tags_2 },
    { 0xb5640ea5, SUC_lexicon_tags_49 },
    { 0x43a40ea6, SUC_lexicon_tags_15 },
    { 0x37240ea6, SUC_lexicon_tags_27 },
    { 0x2ba40ea6, SUC_lexicon_tags_37 },
    { 0x57e40ea9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe040eaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84640eb1, SUC_lexicon_tags_1 },
    { 0x8f640eb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f640eb6, SUC_lexicon_tags_19 },
    { 0x7b440eb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5e40eb9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6840ebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97040ec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26640ec3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30240ec7, SUC_lexicon_tags_57 },
    { 0xa040ec7, SUC_lexicon_tags_87 },
    { 0x9be40ec9, SUC_lexicon_tags_12 },
    { 0x5a840eca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a40ece, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a40ed1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a40ed4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdfe40ed6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e40ede, SUC_lexicon_tags_49 },
    { 0x79240edf, SUC_lexicon_tags_17 },
    { 0xb9c40ede, SUC_lexicon_tags_37 },
    { 0x39840ee1, SUC_lexicon_tags_31 },
    { 0x89a40ee0, SUC_lexicon_tags_7 },
    { 0x97e40ede, SUC_lexicon_tags_25 },
    { 0x5d440ee4, SUC_lexicon_tags_152 },
    { 0xb2440ee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c40ee9, SUC_lexicon_tags_92 },
    { 0x35c40eea, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f840eed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be40ef2, SUC_lexicon_tags_12 },
    { 0xb3c40ef3, SUC_lexicon_tags_47 },
    { 0xbec40ef3, SUC_lexicon_tags_9 },
    { 0x6ec40ef4, SUC_lexicon_tags_47 },
    { 0xc8640ef2, SUC_lexicon_tags_12 },
    { 0x54c40ef5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82640f04, SUC_lexicon_tags_439 },
    { 0xce440f04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61040f08, SUC_lexicon_tags_10 },
    { 0x98e40f09, SUC_lexicon_tags_82 },
    { 0xe4240f08, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e440f0e, SUC_lexicon_tags_37 },
    { 0x9d640f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81840f17, SUC_lexicon_tags_15 },
    { 0xf8e40f17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2240f1d, SUC_lexicon_tags_31 },
    { 0x32640f1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c840f24, SUC_lexicon_tags_824 },
    { 0x8b240f24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce840f2b, SUC_lexicon_tags_33 },
    { 0xb1440f2b, SUC_lexicon_tags_35 },
    { 0x3a440f2c, SUC_lexicon_tags_15 },
    { 0x93a40f2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf440f30, SUC_lexicon_tags_55 },
    { 0xeee40f31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x34840f34, SUC_lexicon_tags_48 },
    { 0x7a440f34, SUC_lexicon_tags_7 },
    { 0x67a40f34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c840f3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c440f41, SUC_lexicon_tags_21 },
    { 0x57e40f42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc40f47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a40f4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4440f4e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79040f52, SUC_lexicon_tags_30 },
    { 0x7c840f52, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x58040f56, SUC_lexicon_tags_30 },
    { 0xde640f57, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae40f5a, SUC_lexicon_tags_1 },
    { 0x2a640f5a, SUC_lexicon_tags_63 },
    { 0x7e440f5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5640f5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x41040f60, SUC_lexicon_tags_17 },
    { 0x7c040f60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc640f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe240f69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x48a40f6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18640f71, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x44c40f73, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c40f77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3440f7c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x10a40f7e, SUC_lexicon_tags_29 },
    { 0x240f7f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c440f82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4240f87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50240f8e, SUC_lexicon_tags_11 },
    { 0x78840f8e, SUC_lexicon_tags_15 },
    { 0xa640f90, SUC_lexicon_tags_0 },
    { 0x58440f90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e40f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2840f96, SUC_lexicon_tags_87 },
    { 0x2c040f97, SUC_lexicon_tags_70 },
    { 0xfca40f98, SUC_lexicon_tags_0 },
    { 0xa9640f98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c40f9c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54440fa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57840fa2, SUC_lexicon_tags_25 },
    { 0x28c40fa2, SUC_lexicon_tags_45 },
    { 0xbea40fa2, SUC_lexicon_tags_63 },
    { 0x4c640fa5, SUC_lexicon_tags_47 },
    { 0xeea40fa4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe0240fa8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7240fab, SUC_lexicon_tags_35 },
    { 0x67240fac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ca40fae, SUC_lexicon_tags_106 },
    { 0x22040faf, SUC_lexicon_tags_37 },
    { 0x57640faf, SUC_lexicon_tags_19 },
    { 0xa0240fb1, SUC_lexicon_tags_55 },
    { 0x13840fae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa9a40fb4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9fc40fb6, SUC_lexicon_tags_47 },
    { 0xa4240fb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca40fbd, SUC_lexicon_tags_14 },
    { 0xd3040fbd, SUC_lexicon_tags_1 },
    { 0x2e840fbf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e40fc5, SUC_lexicon_tags_33 },
    { 0x8ba40fc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49c40fc8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce40fcc, SUC_lexicon_tags_8 },
    { 0x41240fcd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41c40fcf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c40fd2, SUC_lexicon_tags_71 },
    { 0xbd240fd2, SUC_lexicon_tags_15 },
    { 0x4c40fd4, SUC_lexicon_tags_60 },
    { 0x60640fd5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c40fde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6640fe4, SUC_lexicon_tags_3 },
    { 0x42e40fe4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a40ff1, SUC_lexicon_tags_16 },
    { 0x83240ff2, SUC_lexicon_tags_63 },
    { 0xd7840ff3, SUC_lexicon_tags_37 },
    { 0xdc240ff2, SUC_lexicon_tags_11 },
    { 0xaec40ff5, SUC_lexicon_tags_27 },
    { 0x89440ff6, SUC_lexicon_tags_25 },
    { 0xa4a40ff6, SUC_lexicon_tags_16 },
    { 0xbf240ff7, SUC_lexicon_tags_11 },
    { 0x29840ff9, SUC_lexicon_tags_27 },
    { 0xa7640ff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7241001, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24841005, SUC_lexicon_tags_48 },
    { 0x23841006, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c24100a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xce64100c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec24100f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a041011, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c41017, SUC_lexicon_tags_55 },
    { 0xa3841018, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e4101b, SUC_lexicon_tags_12 },
    { 0x6544101b, SUC_lexicon_tags_37 },
    { 0x20c4101d, SUC_lexicon_tags_33 },
    { 0xd964101e, SUC_lexicon_tags_2 },
    { 0xb7a4101f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa41025, SUC_lexicon_tags_49 },
    { 0xb5c41026, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba4102b, SUC_lexicon_tags_38 },
    { 0xfb24102b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf04102f, SUC_lexicon_tags_140 },
    { 0x96e4102f, SUC_lexicon_tags_25 },
    { 0x5aa41030, SUC_lexicon_tags_45 },
    { 0x6584102f, SUC_lexicon_tags_43 },
    { 0x77041033, SUC_lexicon_tags_37 },
    { 0xa641034, SUC_lexicon_tags_33 },
    { 0xacc41035, SUC_lexicon_tags_11 },
    { 0xbce41035, SUC_lexicon_tags_37 },
    { 0x3be41037, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf544103c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbec4103e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a41042, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a641044, SUC_lexicon_tags_37 },
    { 0x56041044, SUC_lexicon_tags_1 },
    { 0xac41046, SUC_lexicon_tags_39 },
    { 0xe7641047, SUC_lexicon_tags_0 },
    { 0xe4041048, SUC_lexicon_tags_129 },
    { 0xd2041048, SUC_lexicon_tags_1 },
    { 0xd3641047, SUC_lexicon_tags_1639 },
    { 0, NULL },
    { 0xae04104c, SUC_lexicon_tags_16 },
    { 0xe4e4104c, SUC_lexicon_tags_2 },
    { 0xa044104d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc41054, SUC_lexicon_tags_11 },
    { 0x5f641055, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a41057, SUC_lexicon_tags_7 },
    { 0x32e41058, SUC_lexicon_tags_12 },
    { 0xd8641057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1224105b, SUC_lexicon_tags_19 },
    { 0xbe4105c, SUC_lexicon_tags_63 },
    { 0xdcc4105c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe884105f, SUC_lexicon_tags_170 },
    { 0xd6c4105f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e41067, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe6e41069, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed44106e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe41077, SUC_lexicon_tags_11 },
    { 0x2ae41078, SUC_lexicon_tags_14 },
    { 0xbf041079, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a4107c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa41083, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3284108b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bc4108d, SUC_lexicon_tags_1177 },
    { 0x1444108d, SUC_lexicon_tags_19 },
    { 0xfca4108f, SUC_lexicon_tags_45 },
    { 0x86c41090, SUC_lexicon_tags_1 },
    { 0x3e041090, SUC_lexicon_tags_25 },
    { 0x9ec41092, SUC_lexicon_tags_37 },
    { 0x9a441092, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36441097, SUC_lexicon_tags_37 },
    { 0xb2441097, SUC_lexicon_tags_1 },
    { 0xb8041099, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf8a4109b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5504109d, SUC_lexicon_tags_23 },
    { 0x86c4109e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x848410a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2410a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a410a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd34410aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19e410ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac410b1, SUC_lexicon_tags_40 },
    { 0xf20410b1, SUC_lexicon_tags_12 },
    { 0x4ce410b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52410b8, SUC_lexicon_tags_12 },
    { 0x2b2410b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82410be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf60410c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad0410c2, SUC_lexicon_tags_68 },
    { 0xef4410c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4410c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de410cb, SUC_lexicon_tags_45 },
    { 0x1fc410cb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a410d7, SUC_lexicon_tags_17 },
    { 0x284410d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff6410da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96410e0, SUC_lexicon_tags_106 },
    { 0x256410e1, SUC_lexicon_tags_1 },
    { 0xbda410e2, SUC_lexicon_tags_56 },
    { 0x92410e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4410e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8410eb, SUC_lexicon_tags_39 },
    { 0xc00410eb, SUC_lexicon_tags_14 },
    { 0xfe410eb, SUC_lexicon_tags_63 },
    { 0xf38410ee, SUC_lexicon_tags_1 },
    { 0x28410ef, SUC_lexicon_tags_23 },
    { 0x412410ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a410f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x110410f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93e410f9, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e410ff, SUC_lexicon_tags_57 },
    { 0xb4a41100, SUC_lexicon_tags_17 },
    { 0x8c41100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce41106, SUC_lexicon_tags_17 },
    { 0x3b441107, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x7f041109, SUC_lexicon_tags_68 },
    { 0x6884110a, SUC_lexicon_tags_47 },
    { 0xe24110b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8304110e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c41111, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf9041113, SUC_lexicon_tags_2 },
    { 0x6c041113, SUC_lexicon_tags_2 },
    { 0xac641115, SUC_lexicon_tags_14 },
    { 0xba641116, SUC_lexicon_tags_13 },
    { 0xc7a41115, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad84111c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8a4111e, SUC_lexicon_tags_14 },
    { 0x1aa4111f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x51841122, SUC_lexicon_tags_30 },
    { 0x6d441123, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24c41125, SUC_lexicon_tags_33 },
    { 0x6f041125, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec4112a, SUC_lexicon_tags_109 },
    { 0x36c4112b, SUC_lexicon_tags_29 },
    { 0x24112c, SUC_lexicon_tags_1 },
    { 0xfda4112c, SUC_lexicon_tags_17 },
    { 0xa8c4112b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20841132, SUC_lexicon_tags_38 },
    { 0xc6041133, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd841135, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d44113a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae41140, SUC_lexicon_tags_6 },
    { 0xa0441141, SUC_lexicon_tags_14 },
    { 0x87a41141, SUC_lexicon_tags_17 },
    { 0x98041143, SUC_lexicon_tags_45 },
    { 0xea41144, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a41148, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba24114c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5241152, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a41156, SUC_lexicon_tags_37 },
    { 0x75a41156, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7324115a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4604115d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f64115f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7841163, SUC_lexicon_tags_8 },
    { 0x7641163, SUC_lexicon_tags_37 },
    { 0x4b641163, SUC_lexicon_tags_26 },
    { 0xf8e41166, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c4116a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee4116e, SUC_lexicon_tags_19 },
    { 0x7b64116f, SUC_lexicon_tags_54 },
    { 0xf144116f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10841173, SUC_lexicon_tags_8 },
    { 0x5ae41173, SUC_lexicon_tags_17 },
    { 0xc4041175, SUC_lexicon_tags_1 },
    { 0x51441176, SUC_lexicon_tags_19 },
    { 0xed841177, SUC_lexicon_tags_49 },
    { 0x86641178, SUC_lexicon_tags_63 },
    { 0x87a41179, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc404117f, SUC_lexicon_tags_1 },
    { 0xdfc4117f, SUC_lexicon_tags_6 },
    { 0x2841181, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99641186, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8204118a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e4118d, SUC_lexicon_tags_0 },
    { 0x3964118e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c41192, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee41198, SUC_lexicon_tags_27 },
    { 0xe6641198, SUC_lexicon_tags_12 },
    { 0x51041198, SUC_lexicon_tags_25 },
    { 0xff241198, SUC_lexicon_tags_7 },
    { 0x12a4119b, SUC_lexicon_tags_1 },
    { 0xa2a4119d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8411a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e411ab, SUC_lexicon_tags_29 },
    { 0x548411ac, SUC_lexicon_tags_0 },
    { 0x39c411ad, SUC_lexicon_tags_1 },
    { 0xc88411ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176411b2, SUC_lexicon_tags_25 },
    { 0x98a411b2, SUC_lexicon_tags_17 },
    { 0xda4411b3, SUC_lexicon_tags_15 },
    { 0xfda411b5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16411b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x852411ba, SUC_lexicon_tags_45 },
    { 0xede411ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec0411bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb86411bf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfd8411c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a411c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36411d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac411da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x608411dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4411df, SUC_lexicon_tags_17 },
    { 0xb54411e0, SUC_lexicon_tags_6 },
    { 0xa46411e1, SUC_lexicon_tags_17 },
    { 0xeb8411e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728411e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x846411e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a411ec, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x408411ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4411f4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x440411f6, SUC_lexicon_tags_15 },
    { 0xae411f6, SUC_lexicon_tags_11 },
    { 0xe0411f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a8411fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e411fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc41211, SUC_lexicon_tags_1 },
    { 0xb7041212, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704121d, SUC_lexicon_tags_9 },
    { 0xa04121e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e41222, SUC_lexicon_tags_46 },
    { 0x5a641222, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c4122c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4304122e, SUC_lexicon_tags_26 },
    { 0x4344122e, SUC_lexicon_tags_12 },
    { 0x75241230, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46841234, SUC_lexicon_tags_47 },
    { 0x99041234, SUC_lexicon_tags_7 },
    { 0x1a441236, SUC_lexicon_tags_11 },
    { 0xc641237, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f44123a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7504123d, SUC_lexicon_tags_8 },
    { 0x3de4123e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e41241, SUC_lexicon_tags_8 },
    { 0xc0a41241, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a041247, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c84124b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70841255, SUC_lexicon_tags_3 },
    { 0xaba41255, SUC_lexicon_tags_38 },
    { 0xbdc41257, SUC_lexicon_tags_2 },
    { 0xd6441258, SUC_lexicon_tags_11 },
    { 0x67641258, SUC_lexicon_tags_55 },
    { 0x33c4125a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc404125d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3a4125f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x79a41261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb241265, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd5241267, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2224126c, SUC_lexicon_tags_11 },
    { 0x4e24126d, SUC_lexicon_tags_17 },
    { 0x9524126e, SUC_lexicon_tags_27 },
    { 0x11e4126f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e241274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc41279, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e24127b, SUC_lexicon_tags_12 },
    { 0xcd84127c, SUC_lexicon_tags_15 },
    { 0xc224127d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56841282, SUC_lexicon_tags_12 },
    { 0x53641283, SUC_lexicon_tags_25 },
    { 0x5a641284, SUC_lexicon_tags_603 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e41288, SUC_lexicon_tags_1 },
    { 0x35241288, SUC_lexicon_tags_1 },
    { 0xf1c41288, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1904128d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c41291, SUC_lexicon_tags_1611 },
    { 0x26a41292, SUC_lexicon_tags_37 },
    { 0x88041292, SUC_lexicon_tags_37 },
    { 0x38c41294, SUC_lexicon_tags_11 },
    { 0xeb641295, SUC_lexicon_tags_19 },
    { 0xdac41296, SUC_lexicon_tags_14 },
    { 0x1ba41294, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a4129b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a4129d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63e4129f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x728412a1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae412a4, SUC_lexicon_tags_15 },
    { 0xc20412a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x998412a7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18412ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06412b0, SUC_lexicon_tags_82 },
    { 0x944412b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396412b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4412bc, SUC_lexicon_tags_1 },
    { 0x272412bd, SUC_lexicon_tags_43 },
    { 0xefe412bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e412c8, SUC_lexicon_tags_2 },
    { 0xf92412c8, SUC_lexicon_tags_25 },
    { 0x462412ca, SUC_lexicon_tags_7 },
    { 0xc4c412c9, SUC_lexicon_tags_1 },
    { 0xfe2412c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe412d0, SUC_lexicon_tags_91 },
    { 0x548412d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f0412d3, SUC_lexicon_tags_26 },
    { 0x2a0412d3, SUC_lexicon_tags_11 },
    { 0xea412d4, SUC_lexicon_tags_12 },
    { 0xf88412d3, SUC_lexicon_tags_17 },
    { 0x79c412d7, SUC_lexicon_tags_38 },
    { 0xacc412d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e4412da, SUC_lexicon_tags_15 },
    { 0x6aa412da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388412e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c412e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x984412e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75c412e8, SUC_lexicon_tags_77 },
    { 0xc84412e9, SUC_lexicon_tags_1 },
    { 0xdd2412ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b8412ec, SUC_lexicon_tags_32 },
    { 0xd22412ed, SUC_lexicon_tags_63 },
    { 0x93c412ec, SUC_lexicon_tags_12 },
    { 0xa46412ee, SUC_lexicon_tags_14 },
    { 0xd2e412ef, SUC_lexicon_tags_25 },
    { 0x14a412f1, SUC_lexicon_tags_65 },
    { 0x174412f2, SUC_lexicon_tags_1 },
    { 0x380412f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a412f7, SUC_lexicon_tags_33 },
    { 0x5e8412f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e412fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0041302, SUC_lexicon_tags_7 },
    { 0x83241303, SUC_lexicon_tags_25 },
    { 0xcba41304, SUC_lexicon_tags_1 },
    { 0xe6e41305, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed441307, SUC_lexicon_tags_29 },
    { 0x1a841308, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe424130b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c4130e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a41311, SUC_lexicon_tags_6 },
    { 0x63841312, SUC_lexicon_tags_17 },
    { 0x7a441313, SUC_lexicon_tags_49 },
    { 0x241314, SUC_lexicon_tags_12 },
    { 0x34841315, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac4131d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51a4131f, SUC_lexicon_tags_15 },
    { 0x4fc41320, SUC_lexicon_tags_60 },
    { 0x3f441320, SUC_lexicon_tags_29 },
    { 0x8f241321, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a41326, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4041328, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2884132c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7904132f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb841331, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea241337, SUC_lexicon_tags_30 },
    { 0xe1e41338, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3364133a, SUC_lexicon_tags_38 },
    { 0xf624133a, SUC_lexicon_tags_6 },
    { 0x2d04133c, SUC_lexicon_tags_38 },
    { 0x6484133b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa41344, SUC_lexicon_tags_7 },
    { 0xccc41345, SUC_lexicon_tags_26 },
    { 0xe6e41345, SUC_lexicon_tags_47 },
    { 0xa4641347, SUC_lexicon_tags_27 },
    { 0xa8c41347, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc4134c, SUC_lexicon_tags_49 },
    { 0x9224134c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2184134f, SUC_lexicon_tags_25 },
    { 0xaf241350, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3a41352, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63041356, SUC_lexicon_tags_15 },
    { 0x36041357, SUC_lexicon_tags_38 },
    { 0x48441358, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3744135a, SUC_lexicon_tags_23 },
    { 0x90c4135a, SUC_lexicon_tags_16 },
    { 0x164135b, SUC_lexicon_tags_27 },
    { 0xd8a4135d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc41364, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a41367, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b441369, SUC_lexicon_tags_11 },
    { 0xe5c41369, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a04136d, SUC_lexicon_tags_26 },
    { 0x9784136e, SUC_lexicon_tags_66 },
    { 0x8d44136f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x91041372, SUC_lexicon_tags_6 },
    { 0xc9c41372, SUC_lexicon_tags_15 },
    { 0x20a41374, SUC_lexicon_tags_14 },
    { 0xb4441375, SUC_lexicon_tags_11 },
    { 0x97441374, SUC_lexicon_tags_7 },
    { 0xf8c41377, SUC_lexicon_tags_37 },
    { 0x11c41378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e24137b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f64137d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc41380, SUC_lexicon_tags_21 },
    { 0x22041380, SUC_lexicon_tags_11 },
    { 0x25641382, SUC_lexicon_tags_17 },
    { 0x83641382, SUC_lexicon_tags_0 },
    { 0x41384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbe41386, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e241389, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf04138e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1641390, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c41393, SUC_lexicon_tags_11 },
    { 0xa3241394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab241396, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a4139c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866413a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82413a4, SUC_lexicon_tags_28 },
    { 0x610413a4, SUC_lexicon_tags_25 },
    { 0xf70413a6, SUC_lexicon_tags_45 },
    { 0x88413a7, SUC_lexicon_tags_15 },
    { 0xf74413a8, SUC_lexicon_tags_14 },
    { 0xf90413a4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x290413ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45e413ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4413b1, SUC_lexicon_tags_15 },
    { 0xe6c413b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc413b7, SUC_lexicon_tags_25 },
    { 0x9a413b8, SUC_lexicon_tags_22 },
    { 0xfde413b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xada413bb, SUC_lexicon_tags_17 },
    { 0xa0a413bc, SUC_lexicon_tags_7 },
    { 0xf20413bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x218413bf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc413c4, SUC_lexicon_tags_0 },
    { 0xf86413c4, SUC_lexicon_tags_19 },
    { 0xc5c413c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e413c9, SUC_lexicon_tags_27 },
    { 0xf28413ca, SUC_lexicon_tags_37 },
    { 0x74c413ca, SUC_lexicon_tags_6 },
    { 0xf58413cc, SUC_lexicon_tags_49 },
    { 0xb76413cb, SUC_lexicon_tags_17 },
    { 0xf96413ce, SUC_lexicon_tags_11 },
    { 0xd1a413cf, SUC_lexicon_tags_40 },
    { 0x884413ce, SUC_lexicon_tags_12 },
    { 0xcf6413d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06413d8, SUC_lexicon_tags_13 },
    { 0xb12413d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c413dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc413df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276413e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbce413e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e413ea, SUC_lexicon_tags_25 },
    { 0x594413eb, SUC_lexicon_tags_23 },
    { 0xc48413eb, SUC_lexicon_tags_82 },
    { 0xeb6413ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c413f3, SUC_lexicon_tags_1 },
    { 0x58e413f4, SUC_lexicon_tags_14 },
    { 0x64c413f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c413f9, SUC_lexicon_tags_124 },
    { 0xe78413fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4e413fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2413ff, SUC_lexicon_tags_23 },
    { 0x2f041400, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d441403, SUC_lexicon_tags_12 },
    { 0x21e41404, SUC_lexicon_tags_12 },
    { 0xa0641405, SUC_lexicon_tags_11 },
    { 0x1b441405, SUC_lexicon_tags_42 },
    { 0xbb641405, SUC_lexicon_tags_2 },
    { 0xfc241407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf484140b, SUC_lexicon_tags_56 },
    { 0xe984140c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x444140e, SUC_lexicon_tags_7 },
    { 0x7924140f, SUC_lexicon_tags_37 },
    { 0x70c4140f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e41413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2604141c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdce4141e, SUC_lexicon_tags_27 },
    { 0x8e64141f, SUC_lexicon_tags_4 },
    { 0xb3e41420, SUC_lexicon_tags_7 },
    { 0x97241420, SUC_lexicon_tags_1 },
    { 0xd5a41422, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c41426, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a4142b, SUC_lexicon_tags_37 },
    { 0xf544142c, SUC_lexicon_tags_17 },
    { 0x9924142b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0441433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e641435, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe84143d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c041440, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34241444, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e4144a, SUC_lexicon_tags_15 },
    { 0x5164144b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee41456, SUC_lexicon_tags_48 },
    { 0xda41457, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45a41459, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbae4145b, SUC_lexicon_tags_46 },
    { 0xa824145c, SUC_lexicon_tags_178 },
    { 0x1d44145d, SUC_lexicon_tags_68 },
    { 0x3ee4145e, SUC_lexicon_tags_12 },
    { 0x7644145f, SUC_lexicon_tags_45 },
    { 0xa9e4145b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3a41462, SUC_lexicon_tags_15 },
    { 0xeaa41462, SUC_lexicon_tags_38 },
    { 0x64c41462, SUC_lexicon_tags_14 },
    { 0xbc841463, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0c41467, SUC_lexicon_tags_15 },
    { 0xb241468, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a4146b, SUC_lexicon_tags_17 },
    { 0xaf24146c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa344146e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73041471, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x94c41473, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59641476, SUC_lexicon_tags_42 },
    { 0xd0a41477, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b24147a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae4147d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87441480, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2641484, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3564148b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8784148d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5241491, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed641495, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x45041498, SUC_lexicon_tags_19 },
    { 0x4b241499, SUC_lexicon_tags_15 },
    { 0x75441499, SUC_lexicon_tags_152 },
    { 0x3dc4149b, SUC_lexicon_tags_32 },
    { 0xec441498, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaf04149e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x10e414a0, SUC_lexicon_tags_0 },
    { 0xfce414a1, SUC_lexicon_tags_37 },
    { 0x4a8414a2, SUC_lexicon_tags_37 },
    { 0xf3e414a3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x582414a6, SUC_lexicon_tags_37 },
    { 0xffe414a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67a414a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e414ac, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x298414af, SUC_lexicon_tags_37 },
    { 0xc74414b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212414b5, SUC_lexicon_tags_37 },
    { 0xcf6414b5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc414b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbdc414bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3a414bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6414c1, SUC_lexicon_tags_12 },
    { 0x7c0414c2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e414c7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb02414c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62414cf, SUC_lexicon_tags_47 },
    { 0xb8414d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80c414d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e8414d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e414d8, SUC_lexicon_tags_33 },
    { 0x3b0414d9, SUC_lexicon_tags_19 },
    { 0x39a414d9, SUC_lexicon_tags_66 },
    { 0x600414db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x946414dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22a414df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x656414e1, SUC_lexicon_tags_48 },
    { 0xba414e2, SUC_lexicon_tags_25 },
    { 0x1c4414e3, SUC_lexicon_tags_26 },
    { 0x960414e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x610414e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc66414e9, SUC_lexicon_tags_207 },
    { 0x27a414e9, SUC_lexicon_tags_12 },
    { 0x606414eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e414ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x370414f2, SUC_lexicon_tags_37 },
    { 0xd08414f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c414fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92414ff, SUC_lexicon_tags_17 },
    { 0xcdc41500, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e41503, SUC_lexicon_tags_2 },
    { 0xdb041504, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea41507, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8444150a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0241511, SUC_lexicon_tags_1 },
    { 0x5d241512, SUC_lexicon_tags_3 },
    { 0xdec41513, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c641518, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf64151b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97c4151d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5041520, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x84641522, SUC_lexicon_tags_2 },
    { 0xb5c41523, SUC_lexicon_tags_15 },
    { 0xc8241522, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xf4041526, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81041528, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c44152a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc484152d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1041530, SUC_lexicon_tags_25 },
    { 0x33a41531, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc241536, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a41544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c641547, SUC_lexicon_tags_37 },
    { 0x85241548, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac4154d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a41552, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x44641554, SUC_lexicon_tags_14 },
    { 0x62041555, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a4155c, SUC_lexicon_tags_47 },
    { 0x4ac4155d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32841561, SUC_lexicon_tags_27 },
    { 0xdce41562, SUC_lexicon_tags_25 },
    { 0x75a41561, SUC_lexicon_tags_49 },
    { 0x4441562, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x46841567, SUC_lexicon_tags_1 },
    { 0xe3e41568, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe6a4156a, SUC_lexicon_tags_7 },
    { 0x1404156b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a4156e, SUC_lexicon_tags_25 },
    { 0x3784156f, SUC_lexicon_tags_27 },
    { 0x43041570, SUC_lexicon_tags_31 },
    { 0x3d84156f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c41579, SUC_lexicon_tags_30 },
    { 0x32c41579, SUC_lexicon_tags_23 },
    { 0x7c4157b, SUC_lexicon_tags_61 },
    { 0x4a64157c, SUC_lexicon_tags_23 },
    { 0x7ee4157b, SUC_lexicon_tags_39 },
    { 0x2184157e, SUC_lexicon_tags_27 },
    { 0x1f24157e, SUC_lexicon_tags_37 },
    { 0x1be41580, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19641589, SUC_lexicon_tags_17 },
    { 0x6944158a, SUC_lexicon_tags_12 },
    { 0x5b241589, SUC_lexicon_tags_37 },
    { 0x4244158b, SUC_lexicon_tags_60 },
    { 0x8a24158d, SUC_lexicon_tags_15 },
    { 0xc284158e, SUC_lexicon_tags_35 },
    { 0x9f24158f, SUC_lexicon_tags_12 },
    { 0x6344158b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x45841593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdc41595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa41599, SUC_lexicon_tags_72 },
    { 0x3cc41599, SUC_lexicon_tags_22 },
    { 0xb104159b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc4159f, SUC_lexicon_tags_27 },
    { 0x53e415a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8415a6, SUC_lexicon_tags_11 },
    { 0x59c415a6, SUC_lexicon_tags_31 },
    { 0x9a8415a8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c415ad, SUC_lexicon_tags_23 },
    { 0x334415ad, SUC_lexicon_tags_7 },
    { 0x31e415ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0415b3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98415be, SUC_lexicon_tags_63 },
    { 0x8a0415be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe415c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8dc415c5, SUC_lexicon_tags_17 },
    { 0x3f4415c5, SUC_lexicon_tags_17 },
    { 0xdce415c6, SUC_lexicon_tags_1 },
    { 0x878415c8, SUC_lexicon_tags_17 },
    { 0xec0415c9, SUC_lexicon_tags_1 },
    { 0x7b4415c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x964415cd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee415d0, SUC_lexicon_tags_38 },
    { 0x702415d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeee415d3, SUC_lexicon_tags_12 },
    { 0x568415d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02415d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30415de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x398415e1, SUC_lexicon_tags_15 },
    { 0x210415e2, SUC_lexicon_tags_8 },
    { 0x594415e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1e0415e5, SUC_lexicon_tags_17 },
    { 0x2cc415e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c415eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c415ee, SUC_lexicon_tags_83 },
    { 0xf98415ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760415f7, SUC_lexicon_tags_6 },
    { 0x6f4415f7, SUC_lexicon_tags_17 },
    { 0x742415f9, SUC_lexicon_tags_70 },
    { 0x32c415f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee41600, SUC_lexicon_tags_25 },
    { 0xc241601, SUC_lexicon_tags_27 },
    { 0x62e41602, SUC_lexicon_tags_68 },
    { 0x3ba41603, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee41606, SUC_lexicon_tags_27 },
    { 0x27e41607, SUC_lexicon_tags_0 },
    { 0x44041608, SUC_lexicon_tags_79 },
    { 0xc5e41609, SUC_lexicon_tags_12 },
    { 0xf0841609, SUC_lexicon_tags_5 },
    { 0x75641607, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce44160f, SUC_lexicon_tags_66 },
    { 0x9f241610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4841613, SUC_lexicon_tags_68 },
    { 0xdfc41614, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40841617, SUC_lexicon_tags_14 },
    { 0xdb241618, SUC_lexicon_tags_46 },
    { 0xfda41618, SUC_lexicon_tags_11 },
    { 0x44641617, SUC_lexicon_tags_2 },
    { 0xb4241617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c41621, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21641627, SUC_lexicon_tags_37 },
    { 0x7bc41628, SUC_lexicon_tags_7 },
    { 0x9ca41629, SUC_lexicon_tags_29 },
    { 0x1e4162a, SUC_lexicon_tags_12 },
    { 0xde4162a, SUC_lexicon_tags_37 },
    { 0x14241627, SUC_lexicon_tags_0 },
    { 0xfdc4162b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a41630, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdbc41632, SUC_lexicon_tags_17 },
    { 0xb4241633, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99041635, SUC_lexicon_tags_31 },
    { 0x10441635, SUC_lexicon_tags_37 },
    { 0xc8e41637, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa4163a, SUC_lexicon_tags_17 },
    { 0x26e4163b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e41640, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac841645, SUC_lexicon_tags_48 },
    { 0x28441646, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x91a41648, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x7704164a, SUC_lexicon_tags_6 },
    { 0x3024164b, SUC_lexicon_tags_7 },
    { 0x31a4164c, SUC_lexicon_tags_18 },
    { 0xd824164c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7a4164f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0841652, SUC_lexicon_tags_21 },
    { 0x36641652, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f641655, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b241657, SUC_lexicon_tags_216 },
    { 0x84241658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x864165b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a41662, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a4166b, SUC_lexicon_tags_7 },
    { 0xa84166c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa964166e, SUC_lexicon_tags_26 },
    { 0x4184166f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a41672, SUC_lexicon_tags_64 },
    { 0x5dc41672, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12c41675, SUC_lexicon_tags_7 },
    { 0xf8c41676, SUC_lexicon_tags_65 },
    { 0xda841676, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2704167a, SUC_lexicon_tags_39 },
    { 0x1284167b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca24167d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5841681, SUC_lexicon_tags_11 },
    { 0x4d641682, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfb441684, SUC_lexicon_tags_8 },
    { 0x18241685, SUC_lexicon_tags_7 },
    { 0x3a041686, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x57441689, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c41693, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac4169d, SUC_lexicon_tags_0 },
    { 0xf524169e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x544416a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc416a7, SUC_lexicon_tags_23 },
    { 0x254416a8, SUC_lexicon_tags_11 },
    { 0xd58416a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968416ad, SUC_lexicon_tags_17 },
    { 0x424416ae, SUC_lexicon_tags_11 },
    { 0xc88416af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x884416b1, SUC_lexicon_tags_33 },
    { 0xbc8416b2, SUC_lexicon_tags_13 },
    { 0x78a416b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c416bb, SUC_lexicon_tags_12 },
    { 0x93e416bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc416c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e416cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1c4416cd, SUC_lexicon_tags_11 },
    { 0xd2c416ce, SUC_lexicon_tags_113 },
    { 0x362416cf, SUC_lexicon_tags_9 },
    { 0x886416d0, SUC_lexicon_tags_27 },
    { 0x5c4416d0, SUC_lexicon_tags_17 },
    { 0x294416cf, SUC_lexicon_tags_23 },
    { 0x14a416d3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x256416d5, SUC_lexicon_tags_7 },
    { 0xda8416d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2416d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64416e6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x984416e9, SUC_lexicon_tags_1 },
    { 0xb32416ea, SUC_lexicon_tags_37 },
    { 0x2b0416ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e416f0, SUC_lexicon_tags_27 },
    { 0xa5a416f1, SUC_lexicon_tags_10 },
    { 0x3bc416f2, SUC_lexicon_tags_60 },
    { 0x600416f2, SUC_lexicon_tags_37 },
    { 0x666416f3, SUC_lexicon_tags_9 },
    { 0xf60416f5, SUC_lexicon_tags_33 },
    { 0x758416f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926416fa, SUC_lexicon_tags_45 },
    { 0xa72416fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c6416fd, SUC_lexicon_tags_77 },
    { 0x5ec416fe, SUC_lexicon_tags_37 },
    { 0xb10416ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0641701, SUC_lexicon_tags_47 },
    { 0x20841701, SUC_lexicon_tags_16 },
    { 0xf6e41703, SUC_lexicon_tags_11 },
    { 0xc7a41704, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2c041706, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc744170b, SUC_lexicon_tags_38 },
    { 0xfb64170c, SUC_lexicon_tags_55 },
    { 0x3884170d, SUC_lexicon_tags_15 },
    { 0xc664170e, SUC_lexicon_tags_12 },
    { 0xf2e4170f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd241713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f241716, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8641719, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55841720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52e41722, SUC_lexicon_tags_25 },
    { 0x2641723, SUC_lexicon_tags_23 },
    { 0x4ce41724, SUC_lexicon_tags_15 },
    { 0xdee41725, SUC_lexicon_tags_60 },
    { 0xbf841725, SUC_lexicon_tags_11 },
    { 0x4b441727, SUC_lexicon_tags_4 },
    { 0x99441728, SUC_lexicon_tags_40 },
    { 0x23441728, SUC_lexicon_tags_8 },
    { 0xc3441723, SUC_lexicon_tags_7 },
    { 0xe5641724, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d04172e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a41730, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb641734, SUC_lexicon_tags_27 },
    { 0x33241735, SUC_lexicon_tags_14 },
    { 0x82c41736, SUC_lexicon_tags_0 },
    { 0x67841736, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea4173a, SUC_lexicon_tags_48 },
    { 0xe904173a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e4173d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de41740, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd6e41742, SUC_lexicon_tags_1 },
    { 0x2fa41743, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x90641746, SUC_lexicon_tags_107 },
    { 0xb6e41747, SUC_lexicon_tags_936 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb204174b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75a4174d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c41751, SUC_lexicon_tags_46 },
    { 0x2b641751, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e441754, SUC_lexicon_tags_33 },
    { 0xf3641755, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d241759, SUC_lexicon_tags_39 },
    { 0x9a4175a, SUC_lexicon_tags_29 },
    { 0xb6a4175b, SUC_lexicon_tags_17 },
    { 0x28641759, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa764175e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b041761, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccc41763, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7641767, SUC_lexicon_tags_30 },
    { 0x6041768, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c24176c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e4176e, SUC_lexicon_tags_12 },
    { 0xd824176e, SUC_lexicon_tags_47 },
    { 0x1c441770, SUC_lexicon_tags_14 },
    { 0xbc841770, SUC_lexicon_tags_2 },
    { 0x15841772, SUC_lexicon_tags_30 },
    { 0x4be41773, SUC_lexicon_tags_15 },
    { 0xa6241773, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7324177e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x39641781, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0041783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7841788, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b24178d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66641790, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa41794, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26041799, SUC_lexicon_tags_16 },
    { 0xe0a4179a, SUC_lexicon_tags_17 },
    { 0x2b44179b, SUC_lexicon_tags_17 },
    { 0xf7a4179c, SUC_lexicon_tags_66 },
    { 0xbe24179d, SUC_lexicon_tags_12 },
    { 0x9ce4179e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96417a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52a417aa, SUC_lexicon_tags_36 },
    { 0xb6e417aa, SUC_lexicon_tags_17 },
    { 0x766417ac, SUC_lexicon_tags_79 },
    { 0x6c417ac, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x996417af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e417b3, SUC_lexicon_tags_15 },
    { 0x77a417b3, SUC_lexicon_tags_15 },
    { 0xb56417b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72417b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x528417ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0417be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8417c1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e417c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x936417c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2417ca, SUC_lexicon_tags_11 },
    { 0x342417ca, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xcaa417cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x158417cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8417df, SUC_lexicon_tags_66 },
    { 0x46417e0, SUC_lexicon_tags_26 },
    { 0x9c6417e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x436417e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6417e8, SUC_lexicon_tags_2 },
    { 0xdda417e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf30417eb, SUC_lexicon_tags_63 },
    { 0x906417ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc14417ee, SUC_lexicon_tags_37 },
    { 0x49c417ee, SUC_lexicon_tags_39 },
    { 0xa22417f0, SUC_lexicon_tags_15 },
    { 0xf4e417f1, SUC_lexicon_tags_15 },
    { 0xf5e417f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbea417f4, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x2bc417f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe56417f8, SUC_lexicon_tags_17 },
    { 0x548417f9, SUC_lexicon_tags_79 },
    { 0xcba417f9, SUC_lexicon_tags_0 },
    { 0x83c417fb, SUC_lexicon_tags_12 },
    { 0x630417f9, SUC_lexicon_tags_80 },
    { 0x4a0417fd, SUC_lexicon_tags_37 },
    { 0x144417fd, SUC_lexicon_tags_87 },
    { 0x29a417ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc41804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31241808, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7e4180a, SUC_lexicon_tags_33 },
    { 0x7004180b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe764180d, SUC_lexicon_tags_329 },
    { 0x4f64180d, SUC_lexicon_tags_1 },
    { 0x41e4180f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4841812, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xace41814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33441817, SUC_lexicon_tags_22 },
    { 0x1b841817, SUC_lexicon_tags_7 },
    { 0xadc41818, SUC_lexicon_tags_14 },
    { 0x45a41817, SUC_lexicon_tags_37 },
    { 0x57e4181b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e4181e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16241826, SUC_lexicon_tags_21 },
    { 0x13e41827, SUC_lexicon_tags_37 },
    { 0x5d641828, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcaa4182a, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a4182d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee41833, SUC_lexicon_tags_65 },
    { 0x1241833, SUC_lexicon_tags_43 },
    { 0x77841835, SUC_lexicon_tags_17 },
    { 0xc5441836, SUC_lexicon_tags_11 },
    { 0x53a41837, SUC_lexicon_tags_53 },
    { 0xac841838, SUC_lexicon_tags_15 },
    { 0xe6e41834, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6644183c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c24183e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c41842, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e41848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d44184a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e4184f, SUC_lexicon_tags_31 },
    { 0x84a41850, SUC_lexicon_tags_45 },
    { 0xbe41851, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf641854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb041858, SUC_lexicon_tags_151 },
    { 0xb5041858, SUC_lexicon_tags_49 },
    { 0xb5c4185a, SUC_lexicon_tags_27 },
    { 0x19c41859, SUC_lexicon_tags_37 },
    { 0x67a41859, SUC_lexicon_tags_63 },
    { 0xe1e41859, SUC_lexicon_tags_27 },
    { 0xc8441858, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c41861, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36041869, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ca4186b, SUC_lexicon_tags_11 },
    { 0x36a4186b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a44186f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42241871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c41874, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75c41876, SUC_lexicon_tags_32 },
    { 0xc0241877, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4187b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c241883, SUC_lexicon_tags_15 },
    { 0x10e41884, SUC_lexicon_tags_25 },
    { 0x1d641885, SUC_lexicon_tags_25 },
    { 0xff841886, SUC_lexicon_tags_28 },
    { 0x54241884, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3241889, SUC_lexicon_tags_25 },
    { 0x1c04188a, SUC_lexicon_tags_15 },
    { 0x5664188b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f84188f, SUC_lexicon_tags_2 },
    { 0xc7841890, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x24241893, SUC_lexicon_tags_25 },
    { 0x64241893, SUC_lexicon_tags_49 },
    { 0xb5a41893, SUC_lexicon_tags_40 },
    { 0x5be41896, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d641899, SUC_lexicon_tags_37 },
    { 0x5904189a, SUC_lexicon_tags_1 },
    { 0x5c4189b, SUC_lexicon_tags_15 },
    { 0x1ca4189b, SUC_lexicon_tags_10 },
    { 0x4f64189c, SUC_lexicon_tags_17 },
    { 0xf7c4189a, SUC_lexicon_tags_44 },
    { 0xae04189a, SUC_lexicon_tags_19 },
    { 0x350418a0, SUC_lexicon_tags_28 },
    { 0xff44189b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7a6418a3, SUC_lexicon_tags_66 },
    { 0xa4418a4, SUC_lexicon_tags_37 },
    { 0x646418a3, SUC_lexicon_tags_45 },
    { 0xba6418a6, SUC_lexicon_tags_8 },
    { 0xed6418a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda418aa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e418b1, SUC_lexicon_tags_79 },
    { 0x89a418b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6418b8, SUC_lexicon_tags_56 },
    { 0xb8e418b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e418c4, SUC_lexicon_tags_33 },
    { 0xf38418c5, SUC_lexicon_tags_32 },
    { 0x918418c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c418cc, SUC_lexicon_tags_15 },
    { 0x2f8418cc, SUC_lexicon_tags_37 },
    { 0x784418ce, SUC_lexicon_tags_7 },
    { 0x6fc418cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954418d5, SUC_lexicon_tags_0 },
    { 0xf80418d5, SUC_lexicon_tags_43 },
    { 0xe1e418d7, SUC_lexicon_tags_1 },
    { 0xac6418d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x80e418da, SUC_lexicon_tags_37 },
    { 0x226418da, SUC_lexicon_tags_11 },
    { 0xd66418dc, SUC_lexicon_tags_29 },
    { 0x5d4418dc, SUC_lexicon_tags_17 },
    { 0x802418dc, SUC_lexicon_tags_11 },
    { 0xa70418de, SUC_lexicon_tags_7 },
    { 0x954418e0, SUC_lexicon_tags_27 },
    { 0x50a418de, SUC_lexicon_tags_19 },
    { 0xf2c418e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf2418e4, SUC_lexicon_tags_14 },
    { 0x18418e5, SUC_lexicon_tags_7 },
    { 0xba0418e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e418e9, SUC_lexicon_tags_37 },
    { 0xf60418ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e418ee, SUC_lexicon_tags_85 },
    { 0xee0418ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb8a418f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca418f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16418f7, SUC_lexicon_tags_28 },
    { 0x670418f8, SUC_lexicon_tags_9 },
    { 0x89e418f7, SUC_lexicon_tags_12 },
    { 0xa34418f9, SUC_lexicon_tags_20 },
    { 0x168418fb, SUC_lexicon_tags_129 },
    { 0x4c418fc, SUC_lexicon_tags_15 },
    { 0xea8418fd, SUC_lexicon_tags_1 },
    { 0xdae418fb, SUC_lexicon_tags_17 },
    { 0x63e418ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc841902, SUC_lexicon_tags_38 },
    { 0x85e41902, SUC_lexicon_tags_17 },
    { 0xb6641904, SUC_lexicon_tags_28 },
    { 0x63041902, SUC_lexicon_tags_17 },
    { 0x81e41906, SUC_lexicon_tags_45 },
    { 0x8e441907, SUC_lexicon_tags_25 },
    { 0x98a41908, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4190d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc41910, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec41914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1441916, SUC_lexicon_tags_1 },
    { 0xc9c41917, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf64191d, SUC_lexicon_tags_1 },
    { 0x6024191e, SUC_lexicon_tags_15 },
    { 0xed44191e, SUC_lexicon_tags_31 },
    { 0x6484191f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e4192a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff24192c, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc4192f, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc41933, SUC_lexicon_tags_65 },
    { 0xb7a41933, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x68c41936, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x86041938, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a4193f, SUC_lexicon_tags_47 },
    { 0x6864193f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf641942, SUC_lexicon_tags_23 },
    { 0x2f041943, SUC_lexicon_tags_7 },
    { 0x90441944, SUC_lexicon_tags_63 },
    { 0xa041945, SUC_lexicon_tags_37 },
    { 0xaac41946, SUC_lexicon_tags_1 },
    { 0x1ce41946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee4194a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6144194c, SUC_lexicon_tags_38 },
    { 0x8704194d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0c4194f, SUC_lexicon_tags_37 },
    { 0x98041950, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c41955, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c4195b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa4195e, SUC_lexicon_tags_1 },
    { 0x3e4195e, SUC_lexicon_tags_66 },
    { 0xcb04195f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40641967, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e4196b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c44196f, SUC_lexicon_tags_15 },
    { 0x1c841970, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a41974, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4041979, SUC_lexicon_tags_42 },
    { 0x7e64197a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b441980, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33041986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5044198a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b04198d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x18841990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x78841993, SUC_lexicon_tags_15 },
    { 0x92841993, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0041997, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe624199c, SUC_lexicon_tags_12 },
    { 0x8764199c, SUC_lexicon_tags_60 },
    { 0x7a24199c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c419a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66419a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6bc419a8, SUC_lexicon_tags_49 },
    { 0xbe2419a9, SUC_lexicon_tags_25 },
    { 0xa28419aa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60419ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x704419b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb6419b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a419b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c419bd, SUC_lexicon_tags_60 },
    { 0x80a419bd, SUC_lexicon_tags_5 },
    { 0x408419bf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6419c3, SUC_lexicon_tags_15 },
    { 0xa7a419c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e419c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x934419ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca419d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x508419d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e419d5, SUC_lexicon_tags_7 },
    { 0xaca419d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398419da, SUC_lexicon_tags_103 },
    { 0xede419db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472419e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x300419e3, SUC_lexicon_tags_2 },
    { 0x730419e4, SUC_lexicon_tags_25 },
    { 0x472419e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e419e8, SUC_lexicon_tags_60 },
    { 0xf54419e9, SUC_lexicon_tags_201 },
    { 0xf3e419ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204419ef, SUC_lexicon_tags_11 },
    { 0xfb8419f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2419f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x524419f6, SUC_lexicon_tags_27 },
    { 0x2f4419f6, SUC_lexicon_tags_47 },
    { 0x3b0419f6, SUC_lexicon_tags_47 },
    { 0x898419f9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x39c419fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc58419fd, SUC_lexicon_tags_0 },
    { 0xd48419fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc41a03, SUC_lexicon_tags_49 },
    { 0xf7241a04, SUC_lexicon_tags_3 },
    { 0xff841a03, SUC_lexicon_tags_4 },
    { 0x30041a06, SUC_lexicon_tags_61 },
    { 0xcf841a05, SUC_lexicon_tags_37 },
    { 0xe3641a08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9241a0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49441a0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e41a13, SUC_lexicon_tags_29 },
    { 0x9a41a14, SUC_lexicon_tags_25 },
    { 0xec41a15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39841a19, SUC_lexicon_tags_23 },
    { 0x56e41a1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d441a20, SUC_lexicon_tags_12 },
    { 0x25a41a20, SUC_lexicon_tags_2 },
    { 0x51e41a22, SUC_lexicon_tags_37 },
    { 0xdbe41a23, SUC_lexicon_tags_29 },
    { 0xf1841a24, SUC_lexicon_tags_37 },
    { 0x95841a25, SUC_lexicon_tags_37 },
    { 0xe3441a26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8b841a28, SUC_lexicon_tags_14 },
    { 0x1441a29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4441a30, SUC_lexicon_tags_8 },
    { 0x71c41a31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2841a34, SUC_lexicon_tags_42 },
    { 0x42a41a34, SUC_lexicon_tags_39 },
    { 0x8e441a36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc41a38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a041a3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86441a3f, SUC_lexicon_tags_63 },
    { 0x2a641a3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4e41a42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7441a46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee41a4a, SUC_lexicon_tags_37 },
    { 0xbb641a4a, SUC_lexicon_tags_7 },
    { 0x92041a4c, SUC_lexicon_tags_39 },
    { 0x76241a4c, SUC_lexicon_tags_37 },
    { 0xbcc41a4e, SUC_lexicon_tags_16 },
    { 0x72041a4f, SUC_lexicon_tags_12 },
    { 0x7da41a50, SUC_lexicon_tags_27 },
    { 0x27241a51, SUC_lexicon_tags_190 },
    { 0x88041a4d, SUC_lexicon_tags_27 },
    { 0xc4e41a53, SUC_lexicon_tags_66 },
    { 0xc0e41a51, SUC_lexicon_tags_43 },
    { 0xcd441a55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71841a5a, SUC_lexicon_tags_21 },
    { 0x7f441a5a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xde641a5d, SUC_lexicon_tags_27 },
    { 0x71841a5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde41a61, SUC_lexicon_tags_127 },
    { 0xe7441a62, SUC_lexicon_tags_27 },
    { 0xbf441a63, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26441a67, SUC_lexicon_tags_37 },
    { 0x93c41a67, SUC_lexicon_tags_46 },
    { 0x31041a69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe641a6c, SUC_lexicon_tags_19 },
    { 0x29c41a6d, SUC_lexicon_tags_0 },
    { 0x9ea41a6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x67041a70, SUC_lexicon_tags_16 },
    { 0x17c41a71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e41a77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a41a7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62c41a80, SUC_lexicon_tags_27 },
    { 0x98041a80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61241a88, SUC_lexicon_tags_40 },
    { 0xfde41a89, SUC_lexicon_tags_7 },
    { 0xacc41a89, SUC_lexicon_tags_26 },
    { 0x93e41a89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x89241a8e, SUC_lexicon_tags_32 },
    { 0x28841a8f, SUC_lexicon_tags_27 },
    { 0x8c641a8f, SUC_lexicon_tags_37 },
    { 0xbd241a8f, SUC_lexicon_tags_17 },
    { 0x2a041a91, SUC_lexicon_tags_25 },
    { 0x9a641a93, SUC_lexicon_tags_84 },
    { 0x4ca41a91, SUC_lexicon_tags_53 },
    { 0xb4641a95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9641a98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9641a9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba41aa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72441aa6, SUC_lexicon_tags_17 },
    { 0xf3641aa6, SUC_lexicon_tags_7 },
    { 0xe3841aa7, SUC_lexicon_tags_31 },
    { 0x6ec41aa9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x14841aab, SUC_lexicon_tags_14 },
    { 0xaee41aac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c441aaf, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f041ab2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d041ab5, SUC_lexicon_tags_33 },
    { 0xaf641ab6, SUC_lexicon_tags_37 },
    { 0xabe41ab7, SUC_lexicon_tags_9 },
    { 0xe5441ab5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1841aba, SUC_lexicon_tags_12 },
    { 0x89041abb, SUC_lexicon_tags_40 },
    { 0xbfa41abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63841ac0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45441ac5, SUC_lexicon_tags_29 },
    { 0x75e41ac5, SUC_lexicon_tags_17 },
    { 0x84e41ac6, SUC_lexicon_tags_37 },
    { 0xc5e41ac8, SUC_lexicon_tags_37 },
    { 0xa0641ac8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37241acb, SUC_lexicon_tags_25 },
    { 0x13241acb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e41acf, SUC_lexicon_tags_38 },
    { 0x88841ad0, SUC_lexicon_tags_234 },
    { 0x59a41ad0, SUC_lexicon_tags_15 },
    { 0xe7041acf, SUC_lexicon_tags_11 },
    { 0x2ec41ad3, SUC_lexicon_tags_37 },
    { 0xc41ad4, SUC_lexicon_tags_12 },
    { 0x6a841ad5, SUC_lexicon_tags_49 },
    { 0x72a41ad5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdee41ad8, SUC_lexicon_tags_37 },
    { 0x33c41ad8, SUC_lexicon_tags_29 },
    { 0xa5241ada, SUC_lexicon_tags_25 },
    { 0x60c41ad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0241add, SUC_lexicon_tags_29 },
    { 0x8ba41add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc241ae1, SUC_lexicon_tags_6 },
    { 0x77241ae2, SUC_lexicon_tags_37 },
    { 0x18a41ae3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e41ae6, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc41aec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c41af1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50641af8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9841afd, SUC_lexicon_tags_2 },
    { 0x74a41afe, SUC_lexicon_tags_43 },
    { 0xc2041afd, SUC_lexicon_tags_33 },
    { 0x6ce41b00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f041b03, SUC_lexicon_tags_1 },
    { 0x7f041b04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c41b08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc41b13, SUC_lexicon_tags_2 },
    { 0x3cc41b14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ca41b16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b241b1b, SUC_lexicon_tags_282 },
    { 0x14a41b1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c41b20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3a41b22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c41b25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x45041b27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2441b2d, SUC_lexicon_tags_23 },
    { 0x67841b2e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece41b32, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d441b3a, SUC_lexicon_tags_23 },
    { 0xe4a41b3a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb0c41b3d, SUC_lexicon_tags_17 },
    { 0x9ae41b3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9c41b40, SUC_lexicon_tags_91 },
    { 0x2f841b41, SUC_lexicon_tags_12 },
    { 0xc5441b42, SUC_lexicon_tags_10 },
    { 0x19c41b43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc1441b45, SUC_lexicon_tags_37 },
    { 0x15641b46, SUC_lexicon_tags_65 },
    { 0xa2e41b45, SUC_lexicon_tags_31 },
    { 0x43041b46, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb6c41b4a, SUC_lexicon_tags_48 },
    { 0xee41b4a, SUC_lexicon_tags_28 },
    { 0x66641b4b, SUC_lexicon_tags_37 },
    { 0x82e41b4d, SUC_lexicon_tags_11 },
    { 0x41a41b4d, SUC_lexicon_tags_2 },
    { 0x90041b4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce41b56, SUC_lexicon_tags_85 },
    { 0xed641b57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c41b5c, SUC_lexicon_tags_1 },
    { 0x94641b5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a41b68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec41b70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81a41b72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x11641b75, SUC_lexicon_tags_95 },
    { 0x21a41b76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61441b79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92841b7c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34641b81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5e41b83, SUC_lexicon_tags_19 },
    { 0xcae41b84, SUC_lexicon_tags_12 },
    { 0xfd641b85, SUC_lexicon_tags_12 },
    { 0xed241b84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8841b8e, SUC_lexicon_tags_9 },
    { 0xa841b8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67241b94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d041b99, SUC_lexicon_tags_27 },
    { 0x86641b9a, SUC_lexicon_tags_12 },
    { 0xe441b99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8441ba1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32441ba5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc641ba7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e241baa, SUC_lexicon_tags_15 },
    { 0xfae41bab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee41baf, SUC_lexicon_tags_5 },
    { 0x6f841bb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a441bb2, SUC_lexicon_tags_9 },
    { 0x82841bb2, SUC_lexicon_tags_37 },
    { 0x4b841bb4, SUC_lexicon_tags_87 },
    { 0xd9041bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc41bb8, SUC_lexicon_tags_17 },
    { 0x93441bb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ae41bbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e41bc0, SUC_lexicon_tags_0 },
    { 0xebe41bc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x57441bc4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x90841bc7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x37e41bc9, SUC_lexicon_tags_71 },
    { 0xd5c41bca, SUC_lexicon_tags_11 },
    { 0x76041bcb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24841bd5, SUC_lexicon_tags_63 },
    { 0x3aa41bd6, SUC_lexicon_tags_11 },
    { 0xd2841bd7, SUC_lexicon_tags_43 },
    { 0x35841bd8, SUC_lexicon_tags_23 },
    { 0xe1a41bd8, SUC_lexicon_tags_31 },
    { 0xc4641bda, SUC_lexicon_tags_25 },
    { 0xe641bdb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x11241bde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9de41be0, SUC_lexicon_tags_27 },
    { 0x55041be0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43041be5, SUC_lexicon_tags_7 },
    { 0xee241be5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ee41be8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd441beb, SUC_lexicon_tags_43 },
    { 0x5f641bec, SUC_lexicon_tags_924 },
    { 0x7e641bed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de41bf2, SUC_lexicon_tags_26 },
    { 0xd1841bf3, SUC_lexicon_tags_48 },
    { 0x68641bf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a41bff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39241c07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61041c10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e41c15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39a41c17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86641c19, SUC_lexicon_tags_82 },
    { 0x86a41c19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37441c26, SUC_lexicon_tags_35 },
    { 0x53641c26, SUC_lexicon_tags_2 },
    { 0x5a441c28, SUC_lexicon_tags_38 },
    { 0xcc241c26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a41c2c, SUC_lexicon_tags_30 },
    { 0x51641c2d, SUC_lexicon_tags_12 },
    { 0x6f841c2c, SUC_lexicon_tags_11 },
    { 0xd0841c2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe41c32, SUC_lexicon_tags_25 },
    { 0xd4841c33, SUC_lexicon_tags_63 },
    { 0x93c41c34, SUC_lexicon_tags_12 },
    { 0x31a41c34, SUC_lexicon_tags_25 },
    { 0xce241c33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb041c3a, SUC_lexicon_tags_1 },
    { 0xecc41c3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x82841c3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fe41c3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2041c41, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xe4241c43, SUC_lexicon_tags_126 },
    { 0xeb841c43, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd241c4b, SUC_lexicon_tags_63 },
    { 0xe9841c4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3641c4f, SUC_lexicon_tags_45 },
    { 0x4c041c50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37841c53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33441c56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb441c58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1b441c5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97041c5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93441c60, SUC_lexicon_tags_7 },
    { 0x82a41c61, SUC_lexicon_tags_28 },
    { 0x9041c61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac41c65, SUC_lexicon_tags_1 },
    { 0xc0c41c66, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0641c69, SUC_lexicon_tags_11 },
    { 0x6ac41c6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c41c6d, SUC_lexicon_tags_25 },
    { 0x25c41c6d, SUC_lexicon_tags_1 },
    { 0xda41c6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd641c74, SUC_lexicon_tags_11 },
    { 0x69e41c74, SUC_lexicon_tags_1 },
    { 0xdf841c76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83241c7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65841c81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e641c87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e41c8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5441c93, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0841c97, SUC_lexicon_tags_2 },
    { 0x68e41c97, SUC_lexicon_tags_11 },
    { 0x62241c99, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c41c9e, SUC_lexicon_tags_2 },
    { 0x14241c9f, SUC_lexicon_tags_37 },
    { 0x73841c9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce41ca9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x65841cab, SUC_lexicon_tags_48 },
    { 0xb7041cac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae41cb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42c41cb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c41cb7, SUC_lexicon_tags_43 },
    { 0x2a241cb7, SUC_lexicon_tags_87 },
    { 0x84241cb8, SUC_lexicon_tags_27 },
    { 0x7a441cba, SUC_lexicon_tags_15 },
    { 0xeae41cb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0041cc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb041cc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae41cce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c41cd3, SUC_lexicon_tags_19 },
    { 0x93441cd4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xea41cd7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a41cdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc841cdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf041cdf, SUC_lexicon_tags_10 },
    { 0x8c841ce0, SUC_lexicon_tags_10 },
    { 0xf4241ce1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xefe41ce3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea41ce8, SUC_lexicon_tags_1 },
    { 0x1be41ce9, SUC_lexicon_tags_25 },
    { 0x51441ce8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xade41cec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x18441cef, SUC_lexicon_tags_38 },
    { 0x19c41cef, SUC_lexicon_tags_57 },
    { 0x3c641cef, SUC_lexicon_tags_37 },
    { 0xf7041cf1, SUC_lexicon_tags_37 },
    { 0x80041cf3, SUC_lexicon_tags_45 },
    { 0xdb841cf3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a41cfe, SUC_lexicon_tags_75 },
    { 0x7c41cff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e41d03, SUC_lexicon_tags_15 },
    { 0xa6a41d04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8641d09, SUC_lexicon_tags_30 },
    { 0x7cc41d0a, SUC_lexicon_tags_8 },
    { 0x36041d0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c41d0d, SUC_lexicon_tags_108 },
    { 0x47a41d0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8441d12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdde41d14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e41d17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea41d20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24041d25, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea41d29, SUC_lexicon_tags_6 },
    { 0xee041d2a, SUC_lexicon_tags_26 },
    { 0x38c41d2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e841d2d, SUC_lexicon_tags_82 },
    { 0x31e41d2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71041d30, SUC_lexicon_tags_37 },
    { 0xf441d31, SUC_lexicon_tags_1498 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e041d35, SUC_lexicon_tags_83 },
    { 0x65c41d36, SUC_lexicon_tags_32 },
    { 0x8d841d36, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38641d3b, SUC_lexicon_tags_37 },
    { 0xcdc41d3c, SUC_lexicon_tags_63 },
    { 0x70e41d3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdbc41d3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x57c41d41, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11641d4e, SUC_lexicon_tags_10 },
    { 0xfbe41d4f, SUC_lexicon_tags_12 },
    { 0xfc041d4e, SUC_lexicon_tags_28 },
    { 0x3b441d51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a41d55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa41d58, SUC_lexicon_tags_28 },
    { 0x6441d58, SUC_lexicon_tags_16 },
    { 0xca41d59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3441d5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e41d60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64441d67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9e41d69, SUC_lexicon_tags_39 },
    { 0xb5c41d6a, SUC_lexicon_tags_19 },
    { 0xb3a41d69, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xb2841d6d, SUC_lexicon_tags_0 },
    { 0x7be41d6e, SUC_lexicon_tags_7 },
    { 0x33841d6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c841d71, SUC_lexicon_tags_17 },
    { 0x3b641d71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x21241d74, SUC_lexicon_tags_26 },
    { 0xd7641d75, SUC_lexicon_tags_37 },
    { 0x66241d74, SUC_lexicon_tags_9 },
    { 0x59a41d75, SUC_lexicon_tags_17 },
    { 0xf441d76, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfc441d7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdd441d7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc41d7f, SUC_lexicon_tags_37 },
    { 0xccc41d80, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe4e41d82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd841d92, SUC_lexicon_tags_45 },
    { 0xbbe41d93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1041d95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2241d97, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a641d9b, SUC_lexicon_tags_7 },
    { 0xa1e41d9b, SUC_lexicon_tags_85 },
    { 0x25041d9b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7441da2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a41da5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc1041da7, SUC_lexicon_tags_37 },
    { 0x26c41da8, SUC_lexicon_tags_66 },
    { 0x9e41da9, SUC_lexicon_tags_29 },
    { 0x63241daa, SUC_lexicon_tags_46 },
    { 0xe9c41dab, SUC_lexicon_tags_1 },
    { 0x9ce41da7, SUC_lexicon_tags_63 },
    { 0x37241da9, SUC_lexicon_tags_9 },
    { 0x1d241dae, SUC_lexicon_tags_60 },
    { 0x42a41daf, SUC_lexicon_tags_60 },
    { 0xaf641db0, SUC_lexicon_tags_32 },
    { 0x56441db1, SUC_lexicon_tags_27 },
    { 0xc3641db2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4641db6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e41dbb, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xce841dbd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x47e41dbf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc41dc3, SUC_lexicon_tags_16 },
    { 0xc9441dc3, SUC_lexicon_tags_7 },
    { 0xa1241dc4, SUC_lexicon_tags_40 },
    { 0xb4c41dc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e641dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10441dce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecc41dd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69241dd4, SUC_lexicon_tags_7 },
    { 0x6da41dd5, SUC_lexicon_tags_11 },
    { 0x57e41dd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28241dd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a841ddb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68241ddd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e41de6, SUC_lexicon_tags_11 },
    { 0x6ea41de6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41e41de9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea41def, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe641df1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf641df6, SUC_lexicon_tags_2 },
    { 0x5b641df7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc641dfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9be41dfd, SUC_lexicon_tags_16 },
    { 0xe5241dfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98241e01, SUC_lexicon_tags_39 },
    { 0xce241e02, SUC_lexicon_tags_27 },
    { 0x11e41e03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca41e07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33c41e09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e441e0b, SUC_lexicon_tags_27 },
    { 0x2ac41e0c, SUC_lexicon_tags_26 },
    { 0x49841e0d, SUC_lexicon_tags_7 },
    { 0x77241e0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88841e14, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e41e17, SUC_lexicon_tags_1 },
    { 0xa2041e18, SUC_lexicon_tags_45 },
    { 0xa6e41e19, SUC_lexicon_tags_9 },
    { 0x1b641e1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4641e22, SUC_lexicon_tags_54 },
    { 0x26c41e23, SUC_lexicon_tags_12 },
    { 0xa441e23, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0841e27, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2841e2c, SUC_lexicon_tags_37 },
    { 0x84e41e2d, SUC_lexicon_tags_15 },
    { 0xb1e41e2d, SUC_lexicon_tags_21 },
    { 0x36641e2c, SUC_lexicon_tags_17 },
    { 0x4be41e30, SUC_lexicon_tags_37 },
    { 0x1da41e31, SUC_lexicon_tags_25 },
    { 0xf3641e32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad041e38, SUC_lexicon_tags_33 },
    { 0x8b241e39, SUC_lexicon_tags_11 },
    { 0xc9c41e3a, SUC_lexicon_tags_12 },
    { 0xf8641e3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83841e41, SUC_lexicon_tags_17 },
    { 0x4b441e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8441e44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c041e47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75041e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84c41e4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a41e51, SUC_lexicon_tags_11 },
    { 0x29c41e52, SUC_lexicon_tags_84 },
    { 0x3b041e52, SUC_lexicon_tags_0 },
    { 0x85241e52, SUC_lexicon_tags_57 },
    { 0x2e641e54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe41e59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4841e5b, SUC_lexicon_tags_2 },
    { 0x87041e5c, SUC_lexicon_tags_28 },
    { 0xcaa41e5c, SUC_lexicon_tags_23 },
    { 0x57041e5e, SUC_lexicon_tags_37 },
    { 0x41041e5e, SUC_lexicon_tags_66 },
    { 0xe2e41e60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9e41e62, SUC_lexicon_tags_47 },
    { 0xdbe41e62, SUC_lexicon_tags_37 },
    { 0x7b641e63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d641e68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d641e6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x95841e72, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d041e74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78041e76, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a41e7e, SUC_lexicon_tags_10 },
    { 0x1aa41e7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97241e82, SUC_lexicon_tags_7 },
    { 0x8e41e82, SUC_lexicon_tags_23 },
    { 0x55a41e84, SUC_lexicon_tags_6 },
    { 0x92641e85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2441e87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37241e92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e41e97, SUC_lexicon_tags_25 },
    { 0x16441e98, SUC_lexicon_tags_26 },
    { 0x32841e99, SUC_lexicon_tags_68 },
    { 0xecc41e99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55241ea4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x8fa41ea6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5841ead, SUC_lexicon_tags_27 },
    { 0x56841eae, SUC_lexicon_tags_11 },
    { 0x4ee41eaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e41eb2, SUC_lexicon_tags_1 },
    { 0x2ac41eb3, SUC_lexicon_tags_14 },
    { 0x55c41eb3, SUC_lexicon_tags_11 },
    { 0x6e241eb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe641eb7, SUC_lexicon_tags_15 },
    { 0xb9041eb8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a41ebc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45041ebe, SUC_lexicon_tags_72 },
    { 0xa41ebe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e41ec4, SUC_lexicon_tags_37 },
    { 0x70441ec5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5441ec7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca041ec9, SUC_lexicon_tags_33 },
    { 0x27c41eca, SUC_lexicon_tags_116 },
    { 0xd041eca, SUC_lexicon_tags_0 },
    { 0xad641ec9, SUC_lexicon_tags_16 },
    { 0xd1c41ecd, SUC_lexicon_tags_17 },
    { 0x9e41ec9, SUC_lexicon_tags_19 },
    { 0x71a41ecd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e41ed2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8441ed4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61841ed6, SUC_lexicon_tags_60 },
    { 0xc4641ed6, SUC_lexicon_tags_0 },
    { 0x45641ed8, SUC_lexicon_tags_39 },
    { 0xdc041ed8, SUC_lexicon_tags_30 },
    { 0xd9841eda, SUC_lexicon_tags_31 },
    { 0xe641edb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3641edd, SUC_lexicon_tags_37 },
    { 0xad841edd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x17a41ee0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0641ee4, SUC_lexicon_tags_11 },
    { 0xab041ee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f641eee, SUC_lexicon_tags_12 },
    { 0xf8a41eef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a41ef3, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2041ef8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ac41efa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98841f01, SUC_lexicon_tags_23 },
    { 0xe4a41f02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97641f04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac41f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26241f0d, SUC_lexicon_tags_15 },
    { 0xe2241f0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x12c41f10, SUC_lexicon_tags_1 },
    { 0xb4e41f11, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3841f19, SUC_lexicon_tags_84 },
    { 0x37241f1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e841f1c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c41f20, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a41f28, SUC_lexicon_tags_46 },
    { 0x8841f29, SUC_lexicon_tags_15 },
    { 0x95841f2a, SUC_lexicon_tags_62 },
    { 0x16e41f29, SUC_lexicon_tags_23 },
    { 0x2ec41f28, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54441f30, SUC_lexicon_tags_11 },
    { 0x3d441f31, SUC_lexicon_tags_45 },
    { 0xcae41f31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27641f3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1ac41f3c, SUC_lexicon_tags_37 },
    { 0x1841f3d, SUC_lexicon_tags_11 },
    { 0x34841f3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae41f41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7241f46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26441f48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c41f4b, SUC_lexicon_tags_61 },
    { 0xed841f4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73441f4e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c41f56, SUC_lexicon_tags_7 },
    { 0xfe441f56, SUC_lexicon_tags_15 },
    { 0x85241f56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4441f5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc41f60, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x64c41f62, SUC_lexicon_tags_27 },
    { 0x53441f63, SUC_lexicon_tags_37 },
    { 0x4c041f64, SUC_lexicon_tags_57 },
    { 0x6c641f64, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x40041f68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0641f6e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf3241f70, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x42241f73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd641f77, SUC_lexicon_tags_25 },
    { 0x72641f78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a41f7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd841f7f, SUC_lexicon_tags_47 },
    { 0x6ba41f80, SUC_lexicon_tags_12 },
    { 0x9a241f81, SUC_lexicon_tags_25 },
    { 0xe0e41f82, SUC_lexicon_tags_103 },
    { 0xa1641f83, SUC_lexicon_tags_25 },
    { 0x55c41f83, SUC_lexicon_tags_9 },
    { 0x7b241f85, SUC_lexicon_tags_54 },
    { 0x21241f86, SUC_lexicon_tags_2 },
    { 0xede41f86, SUC_lexicon_tags_11 },
    { 0xf7041f88, SUC_lexicon_tags_37 },
    { 0xfbe41f89, SUC_lexicon_tags_57 },
    { 0x9fc41f8a, SUC_lexicon_tags_12 },
    { 0xbcc41f8b, SUC_lexicon_tags_38 },
    { 0x93c41f88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc41f90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0041f92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc041f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66641f9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4841f9e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x33e41fa0, SUC_lexicon_tags_155 },
    { 0, NULL },
    { 0x4841fa2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x40c41fa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5641faa, SUC_lexicon_tags_123 },
    { 0x9be41fab, SUC_lexicon_tags_16 },
    { 0xbde41fab, SUC_lexicon_tags_27 },
    { 0x4e041fad, SUC_lexicon_tags_3 },
    { 0xfa41fad, SUC_lexicon_tags_38 },
    { 0xe7c41fae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb041fb2, SUC_lexicon_tags_55 },
    { 0x9c441fb3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x61041fb6, SUC_lexicon_tags_6 },
    { 0x59e41fb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba041fbd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5841fc1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22441fca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38241fcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29241fce, SUC_lexicon_tags_11 },
    { 0xa9841fce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ec41fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa41fe5, SUC_lexicon_tags_486 },
    { 0x8a241fe6, SUC_lexicon_tags_21 },
    { 0x96a41fe7, SUC_lexicon_tags_15 },
    { 0x87241fe8, SUC_lexicon_tags_79 },
    { 0x8a041fe9, SUC_lexicon_tags_26 },
    { 0xcb241fe7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8441fed, SUC_lexicon_tags_1 },
    { 0x36841fee, SUC_lexicon_tags_37 },
    { 0xe0e41fef, SUC_lexicon_tags_37 },
    { 0xcd041ff0, SUC_lexicon_tags_11 },
    { 0x4b641ff0, SUC_lexicon_tags_29 },
    { 0xd9a41fed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc41ff6, SUC_lexicon_tags_13 },
    { 0x95041ff7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b641ff9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7441ffb, SUC_lexicon_tags_12 },
    { 0xe9e41ffc, SUC_lexicon_tags_2 },
    { 0x79641ffd, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c42007, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4200c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc42012, SUC_lexicon_tags_37 },
    { 0x67642013, SUC_lexicon_tags_45 },
    { 0x21642012, SUC_lexicon_tags_60 },
    { 0xe3a42015, SUC_lexicon_tags_53 },
    { 0x7d842016, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x63642019, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8044201d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a842026, SUC_lexicon_tags_45 },
    { 0x59c42027, SUC_lexicon_tags_17 },
    { 0x3be42027, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x89e4202a, SUC_lexicon_tags_47 },
    { 0x6524202b, SUC_lexicon_tags_32 },
    { 0x4c64202b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c42034, SUC_lexicon_tags_72 },
    { 0xbbc42035, SUC_lexicon_tags_1 },
    { 0xb8a42036, SUC_lexicon_tags_15 },
    { 0xc8a42036, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc884203a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95442048, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb84204e, SUC_lexicon_tags_2 },
    { 0x18c4204e, SUC_lexicon_tags_27 },
    { 0x8dc42050, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c42053, SUC_lexicon_tags_45 },
    { 0x6c842054, SUC_lexicon_tags_14 },
    { 0x27042054, SUC_lexicon_tags_11 },
    { 0x38e42056, SUC_lexicon_tags_17 },
    { 0xcf242054, SUC_lexicon_tags_15 },
    { 0x38c42053, SUC_lexicon_tags_7 },
    { 0x8a442059, SUC_lexicon_tags_87 },
    { 0x544205a, SUC_lexicon_tags_15 },
    { 0xa9842054, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f442060, SUC_lexicon_tags_0 },
    { 0xf1c42061, SUC_lexicon_tags_37 },
    { 0xa9c42060, SUC_lexicon_tags_17 },
    { 0x77c42063, SUC_lexicon_tags_60 },
    { 0x3c042062, SUC_lexicon_tags_31 },
    { 0xbfa42061, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38842067, SUC_lexicon_tags_60 },
    { 0x30c42068, SUC_lexicon_tags_17 },
    { 0x41242068, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed64206d, SUC_lexicon_tags_7 },
    { 0xe824206e, SUC_lexicon_tags_26 },
    { 0xed84206f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40042073, SUC_lexicon_tags_1 },
    { 0xb9042074, SUC_lexicon_tags_26 },
    { 0x81842074, SUC_lexicon_tags_43 },
    { 0xea842074, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe84207a, SUC_lexicon_tags_26 },
    { 0xe564207b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc642080, SUC_lexicon_tags_25 },
    { 0x42442081, SUC_lexicon_tags_2 },
    { 0x40842082, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e42086, SUC_lexicon_tags_46 },
    { 0x50a42086, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28442089, SUC_lexicon_tags_17 },
    { 0x5b64208a, SUC_lexicon_tags_2 },
    { 0x1984208b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa4208e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83442092, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3842097, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2384209c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe24209e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaac420a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a420a2, SUC_lexicon_tags_25 },
    { 0xc08420a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74420a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8420a9, SUC_lexicon_tags_12 },
    { 0xd56420aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x470420ad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0420b0, SUC_lexicon_tags_14 },
    { 0x98e420b1, SUC_lexicon_tags_49 },
    { 0xb88420b0, SUC_lexicon_tags_37 },
    { 0xd4e420b2, SUC_lexicon_tags_100 },
    { 0x6c4420b3, SUC_lexicon_tags_11 },
    { 0x934420b0, SUC_lexicon_tags_11 },
    { 0xdfe420b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6420b9, SUC_lexicon_tags_25 },
    { 0x988420ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36420bf, SUC_lexicon_tags_38 },
    { 0x722420c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe5c420c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb10420c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe420d3, SUC_lexicon_tags_17 },
    { 0xe62420d3, SUC_lexicon_tags_9 },
    { 0xb34420d5, SUC_lexicon_tags_28 },
    { 0x47a420d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe48420d8, SUC_lexicon_tags_28 },
    { 0xde420d8, SUC_lexicon_tags_15 },
    { 0x686420d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe24420dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a420df, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x608420e1, SUC_lexicon_tags_43 },
    { 0x50c420e2, SUC_lexicon_tags_37 },
    { 0xd8c420e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84420e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae420ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffa420ef, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2420f2, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0x1c2420f4, SUC_lexicon_tags_2 },
    { 0x772420f5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa9a420f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8b4420f9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38420fd, SUC_lexicon_tags_65 },
    { 0x4da420fd, SUC_lexicon_tags_2 },
    { 0x2e2420fe, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a42106, SUC_lexicon_tags_377 },
    { 0xdb642107, SUC_lexicon_tags_7 },
    { 0x36a42107, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b24210d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3242110, SUC_lexicon_tags_1 },
    { 0x97442110, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc684211b, SUC_lexicon_tags_15 },
    { 0x5c84211c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb042123, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e42126, SUC_lexicon_tags_25 },
    { 0xd3442127, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29042129, SUC_lexicon_tags_12 },
    { 0xe804212a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7904212c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6442139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c4213e, SUC_lexicon_tags_7 },
    { 0x7604213f, SUC_lexicon_tags_35 },
    { 0x80442140, SUC_lexicon_tags_23 },
    { 0x7a842141, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be42144, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9de42146, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75a42148, SUC_lexicon_tags_33 },
    { 0x4e42149, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e4214c, SUC_lexicon_tags_1 },
    { 0xf184214c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f842152, SUC_lexicon_tags_14 },
    { 0x35a42153, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2144215a, SUC_lexicon_tags_45 },
    { 0xd2a4215a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80842163, SUC_lexicon_tags_12 },
    { 0x98a42164, SUC_lexicon_tags_27 },
    { 0xeba42165, SUC_lexicon_tags_25 },
    { 0xbce42166, SUC_lexicon_tags_11 },
    { 0xee042167, SUC_lexicon_tags_10 },
    { 0x13c42164, SUC_lexicon_tags_1 },
    { 0x1f442165, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x61a4216b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfee4216d, SUC_lexicon_tags_47 },
    { 0xc184216e, SUC_lexicon_tags_7 },
    { 0x65e4216f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1042174, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a42178, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc44217c, SUC_lexicon_tags_63 },
    { 0x6904217d, SUC_lexicon_tags_47 },
    { 0xbc84217c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1d042180, SUC_lexicon_tags_17 },
    { 0xb2a42180, SUC_lexicon_tags_46 },
    { 0x35a42182, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbba42184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcca42186, SUC_lexicon_tags_25 },
    { 0x2c42187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21842189, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c4218e, SUC_lexicon_tags_60 },
    { 0xb664218f, SUC_lexicon_tags_37 },
    { 0x44c42190, SUC_lexicon_tags_7 },
    { 0x8cc4218f, SUC_lexicon_tags_0 },
    { 0x3c42191, SUC_lexicon_tags_47 },
    { 0x93c42192, SUC_lexicon_tags_14 },
    { 0xe42194, SUC_lexicon_tags_39 },
    { 0x6464218f, SUC_lexicon_tags_0 },
    { 0xd9c42195, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2e42198, SUC_lexicon_tags_0 },
    { 0xd0242199, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4564219f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4a421a1, SUC_lexicon_tags_1 },
    { 0xddc421a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x296421a4, SUC_lexicon_tags_15 },
    { 0x4b0421a5, SUC_lexicon_tags_27 },
    { 0xafe421a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf4e421a8, SUC_lexicon_tags_2 },
    { 0xb7e421a9, SUC_lexicon_tags_25 },
    { 0x588421a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb26421ac, SUC_lexicon_tags_63 },
    { 0xebc421ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a8421af, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd30421b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c421b7, SUC_lexicon_tags_17 },
    { 0x8e8421b8, SUC_lexicon_tags_99 },
    { 0xe8421b9, SUC_lexicon_tags_31 },
    { 0x5e6421b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260421be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298421c2, SUC_lexicon_tags_7 },
    { 0x6b4421c2, SUC_lexicon_tags_17 },
    { 0x272421c4, SUC_lexicon_tags_33 },
    { 0x386421c5, SUC_lexicon_tags_38 },
    { 0xbea421c6, SUC_lexicon_tags_12 },
    { 0x304421c4, SUC_lexicon_tags_1 },
    { 0x30a421c3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64421d1, SUC_lexicon_tags_46 },
    { 0xdc4421d2, SUC_lexicon_tags_15 },
    { 0x2ac421d3, SUC_lexicon_tags_37 },
    { 0x3d0421d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec421d8, SUC_lexicon_tags_37 },
    { 0x5a421d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa1e421db, SUC_lexicon_tags_55 },
    { 0x28a421dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a421e0, SUC_lexicon_tags_37 },
    { 0x40e421e0, SUC_lexicon_tags_39 },
    { 0xf86421e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0421e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20c421e7, SUC_lexicon_tags_37 },
    { 0x1d4421e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162421ee, SUC_lexicon_tags_32 },
    { 0xcf6421ef, SUC_lexicon_tags_6 },
    { 0x5fe421ef, SUC_lexicon_tags_1 },
    { 0x14a421f1, SUC_lexicon_tags_11 },
    { 0x63c421f2, SUC_lexicon_tags_238 },
    { 0xcf2421f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad8421f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c421f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa442200, SUC_lexicon_tags_26 },
    { 0x59c42200, SUC_lexicon_tags_33 },
    { 0x98842202, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7842204, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa042206, SUC_lexicon_tags_27 },
    { 0xaf442207, SUC_lexicon_tags_48 },
    { 0x642206, SUC_lexicon_tags_61 },
    { 0xeb242206, SUC_lexicon_tags_2 },
    { 0xf784220a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe944220c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5442211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da42216, SUC_lexicon_tags_4 },
    { 0xcbe42216, SUC_lexicon_tags_17 },
    { 0x1fa42216, SUC_lexicon_tags_2 },
    { 0xf5642219, SUC_lexicon_tags_2 },
    { 0x38642219, SUC_lexicon_tags_11 },
    { 0xe264221b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe0c4221d, SUC_lexicon_tags_2 },
    { 0x8aa4221e, SUC_lexicon_tags_25 },
    { 0x7c44221f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e42224, SUC_lexicon_tags_3 },
    { 0x96242225, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xee042228, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50c4222a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fa4222c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2284222f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8a42231, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da42234, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe42238, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3904223a, SUC_lexicon_tags_17 },
    { 0x364223a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de4223e, SUC_lexicon_tags_11 },
    { 0xa1a4223f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f042242, SUC_lexicon_tags_61 },
    { 0x57042243, SUC_lexicon_tags_7 },
    { 0x2be42244, SUC_lexicon_tags_46 },
    { 0x1de42243, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11a42247, SUC_lexicon_tags_12 },
    { 0x7c642248, SUC_lexicon_tags_23 },
    { 0xe4a42248, SUC_lexicon_tags_1 },
    { 0xe2642247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5624224d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba42251, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fc42253, SUC_lexicon_tags_37 },
    { 0x43e42253, SUC_lexicon_tags_32 },
    { 0x2c842253, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18642260, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40042265, SUC_lexicon_tags_14 },
    { 0x9b042266, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d04226b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xac44226d, SUC_lexicon_tags_17 },
    { 0xaec4226e, SUC_lexicon_tags_15 },
    { 0x6e4226f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc242271, SUC_lexicon_tags_31 },
    { 0x36c42272, SUC_lexicon_tags_19 },
    { 0x96842273, SUC_lexicon_tags_0 },
    { 0x6dc42271, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88842277, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2884227b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc4227e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70442282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a42290, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c042292, SUC_lexicon_tags_60 },
    { 0xa6a42292, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8042296, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae4229b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9524229f, SUC_lexicon_tags_31 },
    { 0xd7c4229f, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8422a7, SUC_lexicon_tags_25 },
    { 0x844422a7, SUC_lexicon_tags_37 },
    { 0x5b0422a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a422ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0422af, SUC_lexicon_tags_31 },
    { 0xfc6422b0, SUC_lexicon_tags_63 },
    { 0x5e4422b1, SUC_lexicon_tags_23 },
    { 0xd6e422b2, SUC_lexicon_tags_11 },
    { 0x9ee422b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x10a422b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca0422b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd64422b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04422bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc6422bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x272422c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8da422c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ac422c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc422c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4422cb, SUC_lexicon_tags_25 },
    { 0x920422cc, SUC_lexicon_tags_30 },
    { 0x756422cc, SUC_lexicon_tags_49 },
    { 0x1bc422ce, SUC_lexicon_tags_31 },
    { 0xc98422ce, SUC_lexicon_tags_29 },
    { 0x154422ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb2422d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0422d7, SUC_lexicon_tags_21 },
    { 0x13e422d7, SUC_lexicon_tags_15 },
    { 0x4f0422d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c422dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc422e0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3fc422e2, SUC_lexicon_tags_43 },
    { 0xd8c422e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392422ee, SUC_lexicon_tags_7 },
    { 0x8a4422ee, SUC_lexicon_tags_16 },
    { 0x536422f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeba422f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748422f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40e422fa, SUC_lexicon_tags_16 },
    { 0xb4a422fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c422ff, SUC_lexicon_tags_0 },
    { 0x43042300, SUC_lexicon_tags_11 },
    { 0xb3a422ff, SUC_lexicon_tags_17 },
    { 0x3b842302, SUC_lexicon_tags_47 },
    { 0x7d642300, SUC_lexicon_tags_160 },
    { 0x208422ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c4230f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c42312, SUC_lexicon_tags_160 },
    { 0xbb842313, SUC_lexicon_tags_11 },
    { 0xec42313, SUC_lexicon_tags_49 },
    { 0x89042314, SUC_lexicon_tags_12 },
    { 0xe0e42315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39842318, SUC_lexicon_tags_11 },
    { 0xa3a42318, SUC_lexicon_tags_30 },
    { 0xee24231a, SUC_lexicon_tags_19 },
    { 0x8304231a, SUC_lexicon_tags_17 },
    { 0xa6e4231a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb504231e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2842320, SUC_lexicon_tags_223 },
    { 0xabc42321, SUC_lexicon_tags_26 },
    { 0xef242322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93642327, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b84232a, SUC_lexicon_tags_27 },
    { 0xc264232b, SUC_lexicon_tags_15 },
    { 0x32c4232a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a4232f, SUC_lexicon_tags_12 },
    { 0xe8e4232f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8042336, SUC_lexicon_tags_37 },
    { 0x39042336, SUC_lexicon_tags_7 },
    { 0xe4842338, SUC_lexicon_tags_12 },
    { 0x6c842338, SUC_lexicon_tags_0 },
    { 0x24042338, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3cc4233c, SUC_lexicon_tags_1 },
    { 0x2564233d, SUC_lexicon_tags_23 },
    { 0xed84233e, SUC_lexicon_tags_61 },
    { 0x3664233d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a42342, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54842345, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf442349, SUC_lexicon_tags_220 },
    { 0xaf84234a, SUC_lexicon_tags_25 },
    { 0xc704234b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86e4234d, SUC_lexicon_tags_37 },
    { 0xa6c4234e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7442352, SUC_lexicon_tags_25 },
    { 0x9e842353, SUC_lexicon_tags_2 },
    { 0xfa042353, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c42357, SUC_lexicon_tags_19 },
    { 0xb6842358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c42363, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c42366, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce42369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9664236b, SUC_lexicon_tags_43 },
    { 0xed24236c, SUC_lexicon_tags_43 },
    { 0x6b44236c, SUC_lexicon_tags_11 },
    { 0x3824236e, SUC_lexicon_tags_47 },
    { 0x4a24236f, SUC_lexicon_tags_11 },
    { 0x54a42370, SUC_lexicon_tags_60 },
    { 0x51e4236f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a42375, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1324237b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7504237f, SUC_lexicon_tags_17 },
    { 0x2044237f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b042383, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1842386, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed642392, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe42399, SUC_lexicon_tags_11 },
    { 0xe5c4239a, SUC_lexicon_tags_11 },
    { 0x34a4239b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c423a1, SUC_lexicon_tags_43 },
    { 0x300423a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde423a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec423ad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84423b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea423be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b6423c0, SUC_lexicon_tags_7 },
    { 0xd00423c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe4423c3, SUC_lexicon_tags_30 },
    { 0xa40423c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd2423c6, SUC_lexicon_tags_29 },
    { 0x3b0423c6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14423ca, SUC_lexicon_tags_15 },
    { 0x980423cb, SUC_lexicon_tags_77 },
    { 0x1a2423cc, SUC_lexicon_tags_1 },
    { 0xc70423cc, SUC_lexicon_tags_11 },
    { 0x6f8423ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa2423d0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x88c423d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922423db, SUC_lexicon_tags_144 },
    { 0x1fe423db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54e423de, SUC_lexicon_tags_47 },
    { 0xec0423de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284423e4, SUC_lexicon_tags_65 },
    { 0xaf2423e5, SUC_lexicon_tags_27 },
    { 0x940423e6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfa8423e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x260423ea, SUC_lexicon_tags_4 },
    { 0xa423eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5c423ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2423f7, SUC_lexicon_tags_37 },
    { 0x3aa423f8, SUC_lexicon_tags_37 },
    { 0x1ac423f9, SUC_lexicon_tags_47 },
    { 0xcac423f8, SUC_lexicon_tags_25 },
    { 0x91a423f9, SUC_lexicon_tags_34 },
    { 0x48423f7, SUC_lexicon_tags_2 },
    { 0xe48423f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x60442400, SUC_lexicon_tags_17 },
    { 0x60a42401, SUC_lexicon_tags_7 },
    { 0x3d042402, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e42405, SUC_lexicon_tags_60 },
    { 0xaa042406, SUC_lexicon_tags_40 },
    { 0x34e42406, SUC_lexicon_tags_1 },
    { 0x56042406, SUC_lexicon_tags_14 },
    { 0xbe242409, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2864240c, SUC_lexicon_tags_12 },
    { 0x6e24240c, SUC_lexicon_tags_46 },
    { 0x3f84240e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66c42410, SUC_lexicon_tags_63 },
    { 0xa4e42410, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xae842413, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55042416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c442419, SUC_lexicon_tags_17 },
    { 0x58a4241a, SUC_lexicon_tags_1 },
    { 0x9884241a, SUC_lexicon_tags_11 },
    { 0x6884241c, SUC_lexicon_tags_39 },
    { 0x15e4241b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeee4241f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39c42421, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57c42423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab042425, SUC_lexicon_tags_47 },
    { 0x94242426, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89842429, SUC_lexicon_tags_16 },
    { 0xdb842429, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x824242c, SUC_lexicon_tags_6 },
    { 0x7c4242c, SUC_lexicon_tags_61 },
    { 0xdd84242c, SUC_lexicon_tags_15 },
    { 0xf3c4242d, SUC_lexicon_tags_25 },
    { 0xb9c42430, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f842434, SUC_lexicon_tags_10 },
    { 0x8e442435, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c4243a, SUC_lexicon_tags_15 },
    { 0x1564243b, SUC_lexicon_tags_15 },
    { 0x9564243c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc042440, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a842442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95642447, SUC_lexicon_tags_31 },
    { 0x60242448, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19242450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f84245c, SUC_lexicon_tags_145 },
    { 0x2284245c, SUC_lexicon_tags_23 },
    { 0xc344245d, SUC_lexicon_tags_45 },
    { 0xca84245f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e242463, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x6442466, SUC_lexicon_tags_15 },
    { 0x2ee42467, SUC_lexicon_tags_45 },
    { 0xba842466, SUC_lexicon_tags_82 },
    { 0x63642469, SUC_lexicon_tags_16 },
    { 0x7b642469, SUC_lexicon_tags_9 },
    { 0x72e4246b, SUC_lexicon_tags_12 },
    { 0x4be42469, SUC_lexicon_tags_19 },
    { 0x3da42467, SUC_lexicon_tags_1 },
    { 0xd3442468, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9242472, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1442475, SUC_lexicon_tags_43 },
    { 0x7d842476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb04247c, SUC_lexicon_tags_25 },
    { 0x7624247d, SUC_lexicon_tags_868 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3242483, SUC_lexicon_tags_25 },
    { 0x2b442484, SUC_lexicon_tags_15 },
    { 0xb5442484, SUC_lexicon_tags_47 },
    { 0x85042485, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3964248a, SUC_lexicon_tags_37 },
    { 0x1164248b, SUC_lexicon_tags_25 },
    { 0xfda4248b, SUC_lexicon_tags_38 },
    { 0x2024248d, SUC_lexicon_tags_8 },
    { 0xe964248e, SUC_lexicon_tags_47 },
    { 0x1424248b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b842493, SUC_lexicon_tags_0 },
    { 0x1ae42494, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a42498, SUC_lexicon_tags_17 },
    { 0xc8e42499, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b64249b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3484249e, SUC_lexicon_tags_37 },
    { 0x204249f, SUC_lexicon_tags_17 },
    { 0x7804249e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e424a3, SUC_lexicon_tags_61 },
    { 0x6b4424a4, SUC_lexicon_tags_9 },
    { 0x2e0424a5, SUC_lexicon_tags_104 },
    { 0x1a4424a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660424ab, SUC_lexicon_tags_62 },
    { 0xd48424ac, SUC_lexicon_tags_1 },
    { 0xe4e424ad, SUC_lexicon_tags_33 },
    { 0x796424ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4424b4, SUC_lexicon_tags_47 },
    { 0xf00424b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc424b8, SUC_lexicon_tags_34 },
    { 0x1ba424b9, SUC_lexicon_tags_35 },
    { 0x790424b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2424bd, SUC_lexicon_tags_47 },
    { 0xbac424be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508424c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e424cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338424d0, SUC_lexicon_tags_27 },
    { 0x932424d1, SUC_lexicon_tags_12 },
    { 0x4d2424d2, SUC_lexicon_tags_15 },
    { 0x980424d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd94424d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeea424d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754424db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2da424dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450424e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8424e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a424eb, SUC_lexicon_tags_2 },
    { 0x6b6424ec, SUC_lexicon_tags_50 },
    { 0x306424ed, SUC_lexicon_tags_15 },
    { 0xe60424eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14424f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6424f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5a4424f8, SUC_lexicon_tags_25 },
    { 0x5f6424f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6424fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfbe424ff, SUC_lexicon_tags_3 },
    { 0x60e42500, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39242505, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20842507, SUC_lexicon_tags_33 },
    { 0xd4842508, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4250e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c842513, SUC_lexicon_tags_45 },
    { 0x2dc42514, SUC_lexicon_tags_14 },
    { 0x9242514, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe42519, SUC_lexicon_tags_15 },
    { 0x6c44251a, SUC_lexicon_tags_19 },
    { 0xe204251a, SUC_lexicon_tags_23 },
    { 0x4d24251b, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c4251f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb442522, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1124252a, SUC_lexicon_tags_12 },
    { 0xf264252b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x38a4252d, SUC_lexicon_tags_12 },
    { 0x8b84252d, SUC_lexicon_tags_37 },
    { 0xe2c4252f, SUC_lexicon_tags_46 },
    { 0xda442530, SUC_lexicon_tags_7 },
    { 0x7f842531, SUC_lexicon_tags_16 },
    { 0xeca42530, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25042535, SUC_lexicon_tags_11 },
    { 0xdaa42536, SUC_lexicon_tags_16 },
    { 0xb5e42537, SUC_lexicon_tags_7 },
    { 0x8b242535, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2124253a, SUC_lexicon_tags_11 },
    { 0x8684253a, SUC_lexicon_tags_47 },
    { 0x164253c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb144253e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20242545, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6642549, SUC_lexicon_tags_11 },
    { 0x9dc42549, SUC_lexicon_tags_39 },
    { 0x9e64254b, SUC_lexicon_tags_1 },
    { 0x4804254a, SUC_lexicon_tags_68 },
    { 0x68e4254b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2e4254f, SUC_lexicon_tags_11 },
    { 0x98642550, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x70442552, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1fa42554, SUC_lexicon_tags_29 },
    { 0xbb042555, SUC_lexicon_tags_1 },
    { 0xdec42555, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa784255d, SUC_lexicon_tags_30 },
    { 0xc744255d, SUC_lexicon_tags_9 },
    { 0xbd64255d, SUC_lexicon_tags_43 },
    { 0x2ee42560, SUC_lexicon_tags_30 },
    { 0xd0e42560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63842566, SUC_lexicon_tags_25 },
    { 0x21c42567, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bc42569, SUC_lexicon_tags_29 },
    { 0x71a42569, SUC_lexicon_tags_17 },
    { 0xa2c42569, SUC_lexicon_tags_12 },
    { 0xca24256c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x64256e, SUC_lexicon_tags_60 },
    { 0xb9c4256f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4442572, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfa42574, SUC_lexicon_tags_11 },
    { 0x64842575, SUC_lexicon_tags_37 },
    { 0x43c42576, SUC_lexicon_tags_47 },
    { 0xb0842576, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5664257a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa284257c, SUC_lexicon_tags_25 },
    { 0x3ba4257d, SUC_lexicon_tags_25 },
    { 0xbf64257e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3242583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2242585, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6242588, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b04258b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba84258d, SUC_lexicon_tags_43 },
    { 0x53c4258e, SUC_lexicon_tags_6 },
    { 0x7d64258f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc642591, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a642595, SUC_lexicon_tags_48 },
    { 0x69e42595, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbb442598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c4259b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4244259d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e425a0, SUC_lexicon_tags_15 },
    { 0xb68425a1, SUC_lexicon_tags_14 },
    { 0xd68425a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482425a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c425ac, SUC_lexicon_tags_15 },
    { 0xf8e425ad, SUC_lexicon_tags_13 },
    { 0x87a425ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c425b2, SUC_lexicon_tags_11 },
    { 0xdaa425b2, SUC_lexicon_tags_42 },
    { 0xf24425b3, SUC_lexicon_tags_17 },
    { 0xbce425b5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86425b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c425bc, SUC_lexicon_tags_14 },
    { 0xe88425bd, SUC_lexicon_tags_23 },
    { 0x726425bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa84425c0, SUC_lexicon_tags_29 },
    { 0x5fa425c0, SUC_lexicon_tags_7 },
    { 0xc88425c2, SUC_lexicon_tags_11 },
    { 0x202425c1, SUC_lexicon_tags_17 },
    { 0xfe2425c4, SUC_lexicon_tags_37 },
    { 0x700425c4, SUC_lexicon_tags_61 },
    { 0xf5a425c4, SUC_lexicon_tags_28 },
    { 0xe9a425c7, SUC_lexicon_tags_25 },
    { 0x894425c7, SUC_lexicon_tags_7 },
    { 0xace425c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c425d1, SUC_lexicon_tags_15 },
    { 0xdde425d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc425d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc425da, SUC_lexicon_tags_84 },
    { 0x214425db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca4425dd, SUC_lexicon_tags_79 },
    { 0xc70425dd, SUC_lexicon_tags_15 },
    { 0xe18425df, SUC_lexicon_tags_46 },
    { 0x534425df, SUC_lexicon_tags_12 },
    { 0xbe8425e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x180425e3, SUC_lexicon_tags_1 },
    { 0xb10425e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a425ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306425f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668425f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8425fc, SUC_lexicon_tags_160 },
    { 0xca2425fc, SUC_lexicon_tags_28 },
    { 0xe08425fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e42601, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8642607, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a4260a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc4260d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e42610, SUC_lexicon_tags_15 },
    { 0x34842611, SUC_lexicon_tags_9 },
    { 0xe7a42612, SUC_lexicon_tags_17 },
    { 0x55e42611, SUC_lexicon_tags_7 },
    { 0xb6e42611, SUC_lexicon_tags_37 },
    { 0x73e42615, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4442617, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xff24261a, SUC_lexicon_tags_25 },
    { 0x4b04261b, SUC_lexicon_tags_55 },
    { 0x1524261b, SUC_lexicon_tags_14 },
    { 0xf384261d, SUC_lexicon_tags_38 },
    { 0xf0e4261e, SUC_lexicon_tags_11 },
    { 0x9d44261d, SUC_lexicon_tags_8 },
    { 0x63e42620, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaba42622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54c42624, SUC_lexicon_tags_37 },
    { 0x6aa42625, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c42629, SUC_lexicon_tags_15 },
    { 0x72a42629, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe644262c, SUC_lexicon_tags_8 },
    { 0x164262c, SUC_lexicon_tags_10 },
    { 0x30e4262d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae42634, SUC_lexicon_tags_204 },
    { 0xde042635, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x20442637, SUC_lexicon_tags_16 },
    { 0xfb042638, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d242641, SUC_lexicon_tags_47 },
    { 0x61642642, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a642644, SUC_lexicon_tags_1 },
    { 0xe2a42645, SUC_lexicon_tags_238 },
    { 0xa8c42644, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24264c, SUC_lexicon_tags_7 },
    { 0x7ae4264c, SUC_lexicon_tags_17 },
    { 0xaaa4264e, SUC_lexicon_tags_60 },
    { 0x78e4264f, SUC_lexicon_tags_66 },
    { 0x1c24264f, SUC_lexicon_tags_170 },
    { 0x23a42651, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a42655, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca4265a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a42660, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e42663, SUC_lexicon_tags_1 },
    { 0x63842664, SUC_lexicon_tags_7 },
    { 0x19c42665, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0442667, SUC_lexicon_tags_2 },
    { 0x5d642668, SUC_lexicon_tags_15 },
    { 0xe0042669, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2984266b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5024266d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94842671, SUC_lexicon_tags_1 },
    { 0x5c642671, SUC_lexicon_tags_15 },
    { 0x50242672, SUC_lexicon_tags_15 },
    { 0x68242674, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e42677, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62242679, SUC_lexicon_tags_2 },
    { 0x9204267a, SUC_lexicon_tags_29 },
    { 0x3a64267b, SUC_lexicon_tags_2 },
    { 0x9cc4267c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6744267f, SUC_lexicon_tags_25 },
    { 0xdd64267f, SUC_lexicon_tags_43 },
    { 0xb4e42681, SUC_lexicon_tags_58 },
    { 0x72442682, SUC_lexicon_tags_12 },
    { 0xa4442683, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfae42685, SUC_lexicon_tags_1 },
    { 0x2242686, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c4268a, SUC_lexicon_tags_65 },
    { 0x1ae4268b, SUC_lexicon_tags_19 },
    { 0xa4a4268b, SUC_lexicon_tags_49 },
    { 0x2084268b, SUC_lexicon_tags_25 },
    { 0xc1c4268e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76242693, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae42697, SUC_lexicon_tags_11 },
    { 0xde442698, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36426a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd2426a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdea426a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x304426ab, SUC_lexicon_tags_2 },
    { 0xe82426ac, SUC_lexicon_tags_2 },
    { 0xd64426ad, SUC_lexicon_tags_37 },
    { 0x4de426ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf70426b0, SUC_lexicon_tags_77 },
    { 0x24426b0, SUC_lexicon_tags_26 },
    { 0x13e426b2, SUC_lexicon_tags_1 },
    { 0x656426b2, SUC_lexicon_tags_45 },
    { 0x684426b4, SUC_lexicon_tags_21 },
    { 0xb58426b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x28a426b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac426bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee426bf, SUC_lexicon_tags_37 },
    { 0xe3e426c0, SUC_lexicon_tags_15 },
    { 0x1a8426c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb68426c3, SUC_lexicon_tags_85 },
    { 0xbae426c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2426c6, SUC_lexicon_tags_37 },
    { 0x5cc426c7, SUC_lexicon_tags_33 },
    { 0xe68426c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970426ce, SUC_lexicon_tags_7 },
    { 0x9d4426ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390426d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x80c426d6, SUC_lexicon_tags_15 },
    { 0x70e426d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27c426d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd14426db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2426de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a426e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c426e8, SUC_lexicon_tags_50 },
    { 0xeca426e9, SUC_lexicon_tags_56 },
    { 0x8d2426ea, SUC_lexicon_tags_11 },
    { 0x644426eb, SUC_lexicon_tags_11 },
    { 0x484426eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4426f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c426fc, SUC_lexicon_tags_31 },
    { 0xa2a426fc, SUC_lexicon_tags_37 },
    { 0xd8426fe, SUC_lexicon_tags_7 },
    { 0xe08426ff, SUC_lexicon_tags_11 },
    { 0xf6a42700, SUC_lexicon_tags_43 },
    { 0xf46426ff, SUC_lexicon_tags_77 },
    { 0xa8642702, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c4270b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe584270f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe42712, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e42715, SUC_lexicon_tags_21 },
    { 0xc6642715, SUC_lexicon_tags_79 },
    { 0xc1242715, SUC_lexicon_tags_47 },
    { 0xde442718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd484271b, SUC_lexicon_tags_63 },
    { 0x6c24271c, SUC_lexicon_tags_87 },
    { 0x2b04271d, SUC_lexicon_tags_2 },
    { 0x5044271e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e42721, SUC_lexicon_tags_66 },
    { 0x96042722, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c242725, SUC_lexicon_tags_68 },
    { 0x3e042726, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f042729, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae4272d, SUC_lexicon_tags_87 },
    { 0x3c4272e, SUC_lexicon_tags_103 },
    { 0x3e64272f, SUC_lexicon_tags_1 },
    { 0xc384272f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5da42732, SUC_lexicon_tags_131 },
    { 0x20042732, SUC_lexicon_tags_11 },
    { 0xb9e42733, SUC_lexicon_tags_1 },
    { 0x4d042735, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c242739, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb264273c, SUC_lexicon_tags_37 },
    { 0xa244273c, SUC_lexicon_tags_38 },
    { 0x4e44273e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60642744, SUC_lexicon_tags_15 },
    { 0x33642744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2242747, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1984274b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda4274f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe2a42751, SUC_lexicon_tags_12 },
    { 0x2b442752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56842755, SUC_lexicon_tags_8 },
    { 0xb6042755, SUC_lexicon_tags_69 },
    { 0x77c42756, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e4275d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf524275f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x86042761, SUC_lexicon_tags_11 },
    { 0x6d242762, SUC_lexicon_tags_66 },
    { 0x92242763, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe42766, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30c42768, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4964276a, SUC_lexicon_tags_23 },
    { 0x93a4276b, SUC_lexicon_tags_65 },
    { 0xdc04276c, SUC_lexicon_tags_11 },
    { 0x5204276a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa6a4276f, SUC_lexicon_tags_87 },
    { 0x1aa42770, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5442772, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c42775, SUC_lexicon_tags_37 },
    { 0x47042776, SUC_lexicon_tags_13 },
    { 0xc3c42776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3e42779, SUC_lexicon_tags_60 },
    { 0x44c4277a, SUC_lexicon_tags_15 },
    { 0xc0442779, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d042785, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42c42787, SUC_lexicon_tags_15 },
    { 0x7dc42787, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd524278a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc204278c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8264278e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c42792, SUC_lexicon_tags_32 },
    { 0x51242793, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9242798, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee4279f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86427a5, SUC_lexicon_tags_7 },
    { 0x774427a6, SUC_lexicon_tags_1 },
    { 0xd40427a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370427ac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e427b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2427b3, SUC_lexicon_tags_14 },
    { 0x7aa427b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350427bb, SUC_lexicon_tags_10 },
    { 0x3fe427bc, SUC_lexicon_tags_27 },
    { 0x216427bc, SUC_lexicon_tags_30 },
    { 0x730427bc, SUC_lexicon_tags_4 },
    { 0x408427bf, SUC_lexicon_tags_2 },
    { 0xdf4427bf, SUC_lexicon_tags_45 },
    { 0xc10427c1, SUC_lexicon_tags_37 },
    { 0xbbc427c2, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa427c7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2427cb, SUC_lexicon_tags_37 },
    { 0xb7e427cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594427d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6427db, SUC_lexicon_tags_14 },
    { 0xee427db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608427e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x736427e2, SUC_lexicon_tags_39 },
    { 0xfe8427e3, SUC_lexicon_tags_8 },
    { 0xb0427e4, SUC_lexicon_tags_14 },
    { 0x1d6427e5, SUC_lexicon_tags_43 },
    { 0x7c427e6, SUC_lexicon_tags_1 },
    { 0x1a6427e4, SUC_lexicon_tags_47 },
    { 0xa6427e8, SUC_lexicon_tags_11 },
    { 0x572427e8, SUC_lexicon_tags_48 },
    { 0x30e427e9, SUC_lexicon_tags_30 },
    { 0x190427ea, SUC_lexicon_tags_1 },
    { 0x97a427e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade427f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea427f4, SUC_lexicon_tags_33 },
    { 0x2c8427f5, SUC_lexicon_tags_33 },
    { 0xb3e427f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xde2427f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x954427fb, SUC_lexicon_tags_25 },
    { 0x772427fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44842800, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10242805, SUC_lexicon_tags_17 },
    { 0xa7042806, SUC_lexicon_tags_151 },
    { 0x87642806, SUC_lexicon_tags_6 },
    { 0x56042807, SUC_lexicon_tags_10 },
    { 0xc842809, SUC_lexicon_tags_15 },
    { 0xd7a4280a, SUC_lexicon_tags_26 },
    { 0xf84280b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe44280d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb642813, SUC_lexicon_tags_38 },
    { 0x18c42814, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e642816, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a24281c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5242823, SUC_lexicon_tags_13 },
    { 0xb5442824, SUC_lexicon_tags_85 },
    { 0x3f042823, SUC_lexicon_tags_15 },
    { 0x81e42826, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1964282a, SUC_lexicon_tags_37 },
    { 0xaaa4282a, SUC_lexicon_tags_38 },
    { 0x6924282b, SUC_lexicon_tags_17 },
    { 0xd264282d, SUC_lexicon_tags_83 },
    { 0x1c4282e, SUC_lexicon_tags_11 },
    { 0x7804282d, SUC_lexicon_tags_37 },
    { 0xe9242830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb642834, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x97c42836, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9484283a, SUC_lexicon_tags_25 },
    { 0x2144283b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5604283e, SUC_lexicon_tags_7 },
    { 0xa904283f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfbe42841, SUC_lexicon_tags_11 },
    { 0xa0842842, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e42849, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c4284c, SUC_lexicon_tags_48 },
    { 0x62e4284d, SUC_lexicon_tags_8 },
    { 0x40c4284c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94442852, SUC_lexicon_tags_25 },
    { 0x11642852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62242855, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3042859, SUC_lexicon_tags_10 },
    { 0x3e242859, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e4285c, SUC_lexicon_tags_21 },
    { 0xd884285d, SUC_lexicon_tags_81 },
    { 0x22e4285e, SUC_lexicon_tags_29 },
    { 0xf164285e, SUC_lexicon_tags_63 },
    { 0xb9e4285e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e42868, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a4286c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x50a4286e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d842874, SUC_lexicon_tags_12 },
    { 0x75642874, SUC_lexicon_tags_7 },
    { 0xa9442876, SUC_lexicon_tags_2 },
    { 0x2a042877, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc4287f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec42885, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79242889, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44288c, SUC_lexicon_tags_11 },
    { 0x86a4288d, SUC_lexicon_tags_25 },
    { 0x5824288c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c642895, SUC_lexicon_tags_17 },
    { 0xa4442895, SUC_lexicon_tags_11 },
    { 0xf4442897, SUC_lexicon_tags_12 },
    { 0xee842896, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3444289f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a428a2, SUC_lexicon_tags_17 },
    { 0x358428a3, SUC_lexicon_tags_61 },
    { 0x80c428a4, SUC_lexicon_tags_37 },
    { 0xe0428a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89c428a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ae428a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4428ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0428af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c428b4, SUC_lexicon_tags_31 },
    { 0x4b2428b5, SUC_lexicon_tags_31 },
    { 0xe2e428b5, SUC_lexicon_tags_60 },
    { 0x692428b7, SUC_lexicon_tags_37 },
    { 0x42a428b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42428bd, SUC_lexicon_tags_66 },
    { 0x350428bd, SUC_lexicon_tags_7 },
    { 0xff4428bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c428c3, SUC_lexicon_tags_13 },
    { 0x96c428c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc4e428c6, SUC_lexicon_tags_66 },
    { 0xf98428c7, SUC_lexicon_tags_2 },
    { 0x18e428c6, SUC_lexicon_tags_31 },
    { 0x5ce428c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2428cc, SUC_lexicon_tags_1 },
    { 0x6cc428cd, SUC_lexicon_tags_15 },
    { 0x1b2428cc, SUC_lexicon_tags_53 },
    { 0x1f6428cc, SUC_lexicon_tags_1 },
    { 0x1e4428d0, SUC_lexicon_tags_11 },
    { 0x3e8428d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48c428d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a428d7, SUC_lexicon_tags_47 },
    { 0xd62428d8, SUC_lexicon_tags_15 },
    { 0xd50428d9, SUC_lexicon_tags_14 },
    { 0xb6428d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28428df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a428e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd98428e4, SUC_lexicon_tags_42 },
    { 0x496428e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4da428e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cc428e9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x876428ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe428ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da428f2, SUC_lexicon_tags_1 },
    { 0xe44428f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c428f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd38428fb, SUC_lexicon_tags_0 },
    { 0x2da428fc, SUC_lexicon_tags_15 },
    { 0x8a0428fd, SUC_lexicon_tags_7 },
    { 0x2da428fe, SUC_lexicon_tags_66 },
    { 0xfae428fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd0a42901, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x12c42903, SUC_lexicon_tags_28 },
    { 0xcfe42903, SUC_lexicon_tags_11 },
    { 0x2ce42903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da42908, SUC_lexicon_tags_35 },
    { 0x82c42909, SUC_lexicon_tags_77 },
    { 0x92642909, SUC_lexicon_tags_16 },
    { 0x1644290b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3a4290d, SUC_lexicon_tags_12 },
    { 0xfe4290d, SUC_lexicon_tags_28 },
    { 0x2844290d, SUC_lexicon_tags_11 },
    { 0xde24290f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89642914, SUC_lexicon_tags_60 },
    { 0x91642914, SUC_lexicon_tags_15 },
    { 0x1442916, SUC_lexicon_tags_12 },
    { 0xb9842914, SUC_lexicon_tags_0 },
    { 0x2e042918, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8c04291a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf844291c, SUC_lexicon_tags_25 },
    { 0xee04291d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf642922, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e42925, SUC_lexicon_tags_45 },
    { 0xc9842925, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0442929, SUC_lexicon_tags_63 },
    { 0x27042929, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e42930, SUC_lexicon_tags_91 },
    { 0x4f842931, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe8e42933, SUC_lexicon_tags_12 },
    { 0x65c42934, SUC_lexicon_tags_19 },
    { 0x38642935, SUC_lexicon_tags_5 },
    { 0xe0842936, SUC_lexicon_tags_0 },
    { 0x24642937, SUC_lexicon_tags_15 },
    { 0xd2842937, SUC_lexicon_tags_45 },
    { 0x50842939, SUC_lexicon_tags_23 },
    { 0x1da42937, SUC_lexicon_tags_11 },
    { 0x83a4293b, SUC_lexicon_tags_11 },
    { 0x9144293c, SUC_lexicon_tags_27 },
    { 0xac84293a, SUC_lexicon_tags_2 },
    { 0xaac42934, SUC_lexicon_tags_27 },
    { 0x67e4293f, SUC_lexicon_tags_17 },
    { 0xbac42940, SUC_lexicon_tags_25 },
    { 0xc8642940, SUC_lexicon_tags_15 },
    { 0x8c42940, SUC_lexicon_tags_17 },
    { 0x35442939, SUC_lexicon_tags_63 },
    { 0x62e4293f, SUC_lexicon_tags_1 },
    { 0xbe642945, SUC_lexicon_tags_27 },
    { 0xfbc4293b, SUC_lexicon_tags_25 },
    { 0x97842944, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e64294c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa824294f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec42955, SUC_lexicon_tags_28 },
    { 0xb0442956, SUC_lexicon_tags_25 },
    { 0x73a42957, SUC_lexicon_tags_15 },
    { 0x48c42958, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1484295a, SUC_lexicon_tags_2 },
    { 0xe944295b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e42961, SUC_lexicon_tags_11 },
    { 0xb642962, SUC_lexicon_tags_2 },
    { 0x99642963, SUC_lexicon_tags_45 },
    { 0x26c42964, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64242968, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e4296c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf204296e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x55842971, SUC_lexicon_tags_28 },
    { 0xc9242972, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3242979, SUC_lexicon_tags_37 },
    { 0x1d24297a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe764297d, SUC_lexicon_tags_11 },
    { 0xbaa4297d, SUC_lexicon_tags_1 },
    { 0x6ae4297f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1924298a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x49a4298c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c42991, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4299b, SUC_lexicon_tags_11 },
    { 0x9344299b, SUC_lexicon_tags_14 },
    { 0xca4299c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e429a1, SUC_lexicon_tags_27 },
    { 0xf08429a2, SUC_lexicon_tags_23 },
    { 0x958429a2, SUC_lexicon_tags_39 },
    { 0xd72429a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xab4429a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x834429a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9e429aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x442429ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e429ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c0429b0, SUC_lexicon_tags_633 },
    { 0x5e8429b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d8429b3, SUC_lexicon_tags_11 },
    { 0xedc429b4, SUC_lexicon_tags_7 },
    { 0x700429b5, SUC_lexicon_tags_7 },
    { 0x174429b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x940429b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8429be, SUC_lexicon_tags_43 },
    { 0x188429be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79a429c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac429c6, SUC_lexicon_tags_63 },
    { 0x78429c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x716429c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e429ce, SUC_lexicon_tags_0 },
    { 0x5e6429ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x260429d2, SUC_lexicon_tags_48 },
    { 0xc32429d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24429d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56429de, SUC_lexicon_tags_25 },
    { 0x150429de, SUC_lexicon_tags_26 },
    { 0x528429e0, SUC_lexicon_tags_37 },
    { 0xba8429e1, SUC_lexicon_tags_26 },
    { 0x718429e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c429e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x480429e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6429eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84429f2, SUC_lexicon_tags_25 },
    { 0x8f0429f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6429f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2429fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2842a00, SUC_lexicon_tags_15 },
    { 0x2c842a01, SUC_lexicon_tags_11 },
    { 0xf8c42a02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a642a09, SUC_lexicon_tags_23 },
    { 0x28042a0a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4042a14, SUC_lexicon_tags_19 },
    { 0x60842a15, SUC_lexicon_tags_82 },
    { 0x3d842a16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1442a19, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7842a1f, SUC_lexicon_tags_17 },
    { 0x8aa42a20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48e42a22, SUC_lexicon_tags_45 },
    { 0xebe42a23, SUC_lexicon_tags_43 },
    { 0x2b842a23, SUC_lexicon_tags_63 },
    { 0xae842a22, SUC_lexicon_tags_2 },
    { 0xb1c42a26, SUC_lexicon_tags_17 },
    { 0x3e442a27, SUC_lexicon_tags_14 },
    { 0xa4c42a28, SUC_lexicon_tags_37 },
    { 0x27c42a28, SUC_lexicon_tags_34 },
    { 0x15c42a24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22242a2c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4042a2e, SUC_lexicon_tags_15 },
    { 0xa4242a2f, SUC_lexicon_tags_29 },
    { 0xa8842a2f, SUC_lexicon_tags_25 },
    { 0x5de42a30, SUC_lexicon_tags_25 },
    { 0x8dc42a32, SUC_lexicon_tags_16 },
    { 0x31842a31, SUC_lexicon_tags_2 },
    { 0x7f242a34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e42a3a, SUC_lexicon_tags_15 },
    { 0x55442a3b, SUC_lexicon_tags_0 },
    { 0xad642a3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6042a40, SUC_lexicon_tags_37 },
    { 0xada42a41, SUC_lexicon_tags_27 },
    { 0x3e842a42, SUC_lexicon_tags_1 },
    { 0x40442a41, SUC_lexicon_tags_174 },
    { 0x1a042a44, SUC_lexicon_tags_35 },
    { 0x41a42a45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c42a48, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca42a4e, SUC_lexicon_tags_29 },
    { 0xeb042a4f, SUC_lexicon_tags_44 },
    { 0xe6c42a50, SUC_lexicon_tags_38 },
    { 0xc1e42a4f, SUC_lexicon_tags_25 },
    { 0xd1e42a52, SUC_lexicon_tags_2 },
    { 0xd8c42a52, SUC_lexicon_tags_15 },
    { 0xf3642a52, SUC_lexicon_tags_11 },
    { 0x14a42a55, SUC_lexicon_tags_198 },
    { 0xb3842a52, SUC_lexicon_tags_57 },
    { 0x8c42a53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f042a59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e842a60, SUC_lexicon_tags_33 },
    { 0x8e842a61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28a42a63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4442a66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b642a68, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca42a6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x44042a6e, SUC_lexicon_tags_17 },
    { 0xd7a42a6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea42a73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac42a76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86642a7b, SUC_lexicon_tags_1 },
    { 0x14642a7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7c42a7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x19e42a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d042a85, SUC_lexicon_tags_0 },
    { 0x9f242a86, SUC_lexicon_tags_2 },
    { 0xf242a86, SUC_lexicon_tags_1 },
    { 0xfd242a86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x79842a8a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6842a90, SUC_lexicon_tags_15 },
    { 0x6a842a91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83842a94, SUC_lexicon_tags_13 },
    { 0x2f442a95, SUC_lexicon_tags_7 },
    { 0xbae42a96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1442a99, SUC_lexicon_tags_0 },
    { 0x54a42a99, SUC_lexicon_tags_1 },
    { 0x58e42a9b, SUC_lexicon_tags_33 },
    { 0x2f242a9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4042a9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a642aa3, SUC_lexicon_tags_2 },
    { 0x21042aa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a42aaf, SUC_lexicon_tags_32 },
    { 0xbf642aaf, SUC_lexicon_tags_15 },
    { 0xf042ab1, SUC_lexicon_tags_1 },
    { 0x17442ab2, SUC_lexicon_tags_35 },
    { 0x8ac42ab2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a42ab6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8642ab9, SUC_lexicon_tags_14 },
    { 0x77a42aba, SUC_lexicon_tags_37 },
    { 0x80c42abb, SUC_lexicon_tags_1 },
    { 0x1c242aba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4e42abe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d042ac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35242ac3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82642ac7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e42aca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5642ad0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e042ad5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e42ada, SUC_lexicon_tags_37 },
    { 0xbfe42adb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7642ae0, SUC_lexicon_tags_37 },
    { 0xdf642ae1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1642ae6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52042ae8, SUC_lexicon_tags_17 },
    { 0x29642ae9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb8a42aeb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31842af3, SUC_lexicon_tags_47 },
    { 0x22042af4, SUC_lexicon_tags_9 },
    { 0x91642af3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4042af9, SUC_lexicon_tags_37 },
    { 0x3b242af9, SUC_lexicon_tags_7 },
    { 0x86e42afb, SUC_lexicon_tags_25 },
    { 0x72c42afc, SUC_lexicon_tags_7 },
    { 0xb6a42afd, SUC_lexicon_tags_2 },
    { 0x91042afd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x77842b01, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2e42b03, SUC_lexicon_tags_87 },
    { 0x1de42b04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8e42b06, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x86842b09, SUC_lexicon_tags_25 },
    { 0xc7042b0a, SUC_lexicon_tags_47 },
    { 0x63642b0a, SUC_lexicon_tags_7 },
    { 0x9b242b0a, SUC_lexicon_tags_23 },
    { 0xf2042b0d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbee42b0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c42b14, SUC_lexicon_tags_14 },
    { 0x59642b15, SUC_lexicon_tags_7 },
    { 0xca42b16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9242b18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80c42b1a, SUC_lexicon_tags_17 },
    { 0x3cc42b1b, SUC_lexicon_tags_10 },
    { 0xf8842b1c, SUC_lexicon_tags_15 },
    { 0x1a42b1a, SUC_lexicon_tags_15 },
    { 0x27842b1d, SUC_lexicon_tags_47 },
    { 0x19242b1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59042b22, SUC_lexicon_tags_1572 },
    { 0, NULL },
    { 0, NULL },
    { 0x85842b25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb242b2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4a42b2f, SUC_lexicon_tags_8 },
    { 0xab842b30, SUC_lexicon_tags_22 },
    { 0x3642b30, SUC_lexicon_tags_45 },
    { 0x9dc42b32, SUC_lexicon_tags_12 },
    { 0x25442b33, SUC_lexicon_tags_1 },
    { 0xd3042b34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xefa42b36, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae42b40, SUC_lexicon_tags_19 },
    { 0x90842b41, SUC_lexicon_tags_32 },
    { 0xa2c42b42, SUC_lexicon_tags_25 },
    { 0x98042b40, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb5042b45, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e42b48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c42b4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe42b4e, SUC_lexicon_tags_31 },
    { 0xe7242b4e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2de42b51, SUC_lexicon_tags_56 },
    { 0xaea42b52, SUC_lexicon_tags_27 },
    { 0x33842b52, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8642b56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x67c42b58, SUC_lexicon_tags_38 },
    { 0x44c42b59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c42b66, SUC_lexicon_tags_15 },
    { 0x7fc42b66, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae42b6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x442b6e, SUC_lexicon_tags_87 },
    { 0xb2c42b6f, SUC_lexicon_tags_60 },
    { 0x79442b70, SUC_lexicon_tags_19 },
    { 0x3c42b70, SUC_lexicon_tags_37 },
    { 0x61842b71, SUC_lexicon_tags_27 },
    { 0x8d242b70, SUC_lexicon_tags_37 },
    { 0x16242b74, SUC_lexicon_tags_57 },
    { 0x32442b75, SUC_lexicon_tags_15 },
    { 0x8e642b70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe0c42b78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee842b7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9b842b7e, SUC_lexicon_tags_37 },
    { 0x1e042b7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcda42b81, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb842b8a, SUC_lexicon_tags_2 },
    { 0xc0a42b8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a42b8f, SUC_lexicon_tags_25 },
    { 0xb9c42b8f, SUC_lexicon_tags_25 },
    { 0x5f442b91, SUC_lexicon_tags_37 },
    { 0xcec42b92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c42b98, SUC_lexicon_tags_56 },
    { 0x24842b99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f642b9f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x8b642ba1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4242ba3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a42bab, SUC_lexicon_tags_7 },
    { 0x7c042bab, SUC_lexicon_tags_43 },
    { 0x55c42bad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8f042baf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9042bb1, SUC_lexicon_tags_21 },
    { 0xf5642bb2, SUC_lexicon_tags_26 },
    { 0x83c42bb3, SUC_lexicon_tags_16 },
    { 0x14842bb3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x40042bb6, SUC_lexicon_tags_37 },
    { 0x69a42bb7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d442bba, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1ec42bbc, SUC_lexicon_tags_5 },
    { 0xabc42bbd, SUC_lexicon_tags_46 },
    { 0x9f042bbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d442bc3, SUC_lexicon_tags_21 },
    { 0x26642bc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79042bc6, SUC_lexicon_tags_17 },
    { 0x4b842bc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46242bcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c642bd0, SUC_lexicon_tags_60 },
    { 0xe7842bd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a42bd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8642bd9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x34442bdc, SUC_lexicon_tags_2 },
    { 0x9dc42bdc, SUC_lexicon_tags_12 },
    { 0x21a42bde, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4242be2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc42be5, SUC_lexicon_tags_49 },
    { 0x77042be6, SUC_lexicon_tags_60 },
    { 0xf3042be7, SUC_lexicon_tags_6 },
    { 0x17642be8, SUC_lexicon_tags_79 },
    { 0x74442be9, SUC_lexicon_tags_43 },
    { 0xa7642be9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3e42bec, SUC_lexicon_tags_9 },
    { 0x5b842bed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabe42bef, SUC_lexicon_tags_2 },
    { 0xca642bef, SUC_lexicon_tags_17 },
    { 0xc9c42bf1, SUC_lexicon_tags_47 },
    { 0xb042bf2, SUC_lexicon_tags_46 },
    { 0x47642bef, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac42bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43642bfc, SUC_lexicon_tags_35 },
    { 0x7b842bfd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd2042bff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4ce42c01, SUC_lexicon_tags_37 },
    { 0x1d842c02, SUC_lexicon_tags_37 },
    { 0x7e442c03, SUC_lexicon_tags_19 },
    { 0xf9c42c04, SUC_lexicon_tags_7 },
    { 0x63242c05, SUC_lexicon_tags_0 },
    { 0xfc642c06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc42c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e42c0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13242c0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf7042c10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc42c13, SUC_lexicon_tags_43 },
    { 0xf3e42c13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd842c17, SUC_lexicon_tags_25 },
    { 0x19642c18, SUC_lexicon_tags_37 },
    { 0xe3a42c19, SUC_lexicon_tags_0 },
    { 0x7c642c19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4b842c1c, SUC_lexicon_tags_15 },
    { 0x4ea42c1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee442c22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb042c25, SUC_lexicon_tags_14 },
    { 0xbd242c26, SUC_lexicon_tags_13 },
    { 0x9d442c27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5842c29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbe042c2b, SUC_lexicon_tags_19 },
    { 0x58042c2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c42c2f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x77e42c31, SUC_lexicon_tags_17 },
    { 0x43c42c31, SUC_lexicon_tags_17 },
    { 0x98e42c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50242c37, SUC_lexicon_tags_6 },
    { 0x14442c37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x26042c3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73042c3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50a42c3f, SUC_lexicon_tags_15 },
    { 0x58042c40, SUC_lexicon_tags_17 },
    { 0xe7842c41, SUC_lexicon_tags_63 },
    { 0xc3c42c42, SUC_lexicon_tags_12 },
    { 0x91e42c43, SUC_lexicon_tags_12 },
    { 0xa1a42c44, SUC_lexicon_tags_27 },
    { 0xd4042c41, SUC_lexicon_tags_19 },
    { 0x38e42c40, SUC_lexicon_tags_27 },
    { 0xbc842c44, SUC_lexicon_tags_33 },
    { 0x4042c40, SUC_lexicon_tags_6 },
    { 0xd9e42c44, SUC_lexicon_tags_12 },
    { 0xd0e42c42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6c042c4c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4cc42c4e, SUC_lexicon_tags_49 },
    { 0x9e42c4f, SUC_lexicon_tags_17 },
    { 0xd8042c50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3e42c52, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5cc42c54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0c42c56, SUC_lexicon_tags_25 },
    { 0xc0242c57, SUC_lexicon_tags_17 },
    { 0x8d442c57, SUC_lexicon_tags_62 },
    { 0xc8c42c56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa442c5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcc042c61, SUC_lexicon_tags_23 },
    { 0xe7642c61, SUC_lexicon_tags_37 },
    { 0x9fe42c63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x57842c66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a42c6a, SUC_lexicon_tags_37 },
    { 0xc1242c6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a042c6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99242c73, SUC_lexicon_tags_25 },
    { 0xa0642c73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5442c78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4442c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c242c87, SUC_lexicon_tags_31 },
    { 0xcc442c87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23a42c8a, SUC_lexicon_tags_49 },
    { 0x4642c8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e42c8e, SUC_lexicon_tags_28 },
    { 0x44242c8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2f242c91, SUC_lexicon_tags_45 },
    { 0xe2042c92, SUC_lexicon_tags_1 },
    { 0xdb242c93, SUC_lexicon_tags_45 },
    { 0x9a842c94, SUC_lexicon_tags_21 },
    { 0x35e42c91, SUC_lexicon_tags_3 },
    { 0x94c42c91, SUC_lexicon_tags_20 },
    { 0x9d842c97, SUC_lexicon_tags_55 },
    { 0x59442c94, SUC_lexicon_tags_2 },
    { 0x78842c99, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc42c9d, SUC_lexicon_tags_34 },
    { 0xcd042c9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc42ca1, SUC_lexicon_tags_17 },
    { 0x66a42ca1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x86242ca4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x34842ca6, SUC_lexicon_tags_25 },
    { 0x52842ca6, SUC_lexicon_tags_1 },
    { 0x34a42ca6, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0x8c842caa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c842cad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54e42caf, SUC_lexicon_tags_37 },
    { 0xbb042cb0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x60442cb3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2042cb8, SUC_lexicon_tags_87 },
    { 0x3dc42cb8, SUC_lexicon_tags_108 },
    { 0xa5a42cba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea42cbd, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0xe242cbf, SUC_lexicon_tags_8 },
    { 0x8ce42cc0, SUC_lexicon_tags_1 },
    { 0x83e42cbf, SUC_lexicon_tags_23 },
    { 0x99042cbf, SUC_lexicon_tags_55 },
    { 0x9ae42cc0, SUC_lexicon_tags_47 },
    { 0x85842cc2, SUC_lexicon_tags_19 },
    { 0x8da42cc0, SUC_lexicon_tags_37 },
    { 0xa4a42cc2, SUC_lexicon_tags_51 },
    { 0x58242cc7, SUC_lexicon_tags_7 },
    { 0x80c42cc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46442ccc, SUC_lexicon_tags_31 },
    { 0x4e842ccd, SUC_lexicon_tags_37 },
    { 0xf9c42cce, SUC_lexicon_tags_151 },
    { 0xd5a42ccd, SUC_lexicon_tags_17 },
    { 0xcae42cd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9642cd4, SUC_lexicon_tags_18 },
    { 0xdf042cd5, SUC_lexicon_tags_19 },
    { 0xefe42cd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78442cda, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8442cdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7642ce2, SUC_lexicon_tags_38 },
    { 0xf0442ce3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc42ce9, SUC_lexicon_tags_38 },
    { 0x64842cea, SUC_lexicon_tags_14 },
    { 0x8a842ceb, SUC_lexicon_tags_37 },
    { 0x40e42ceb, SUC_lexicon_tags_17 },
    { 0x45e42cea, SUC_lexicon_tags_6 },
    { 0x95842cee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e42cf2, SUC_lexicon_tags_8 },
    { 0x47442cf2, SUC_lexicon_tags_7 },
    { 0xa3a42cf4, SUC_lexicon_tags_37 },
    { 0xfe042cf4, SUC_lexicon_tags_15 },
    { 0x14842cf5, SUC_lexicon_tags_37 },
    { 0xd6c42cf7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa42cf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7642cfd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3842d02, SUC_lexicon_tags_1 },
    { 0x21442d03, SUC_lexicon_tags_28 },
    { 0x85642d02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e42d07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22042d0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9442d11, SUC_lexicon_tags_49 },
    { 0x10c42d12, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e642d15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x39a42d17, SUC_lexicon_tags_45 },
    { 0x1d242d18, SUC_lexicon_tags_37 },
    { 0x26642d19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a642d1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d442d20, SUC_lexicon_tags_45 },
    { 0x4ec42d20, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c42d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5242d27, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb642d2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e42d2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9642d31, SUC_lexicon_tags_11 },
    { 0xb7642d32, SUC_lexicon_tags_14 },
    { 0x61042d33, SUC_lexicon_tags_26 },
    { 0xc042d34, SUC_lexicon_tags_11 },
    { 0xcf642d35, SUC_lexicon_tags_37 },
    { 0xcc42d35, SUC_lexicon_tags_7 },
    { 0x6cc42d36, SUC_lexicon_tags_25 },
    { 0x4f642d35, SUC_lexicon_tags_33 },
    { 0x8e842d36, SUC_lexicon_tags_47 },
    { 0x80a42d32, SUC_lexicon_tags_37 },
    { 0x26442d39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce42d3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48242d49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60c42d4b, SUC_lexicon_tags_38 },
    { 0x3ee42d4c, SUC_lexicon_tags_15 },
    { 0x1ec42d4d, SUC_lexicon_tags_14 },
    { 0xcd242d4e, SUC_lexicon_tags_65 },
    { 0x6b842d4f, SUC_lexicon_tags_16 },
    { 0xdda42d50, SUC_lexicon_tags_15 },
    { 0x3e442d4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xca042d53, SUC_lexicon_tags_38 },
    { 0xea842d54, SUC_lexicon_tags_25 },
    { 0xedc42d54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa42d61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30842d66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12642d6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c442d72, SUC_lexicon_tags_63 },
    { 0xbca42d73, SUC_lexicon_tags_15 },
    { 0x1ea42d73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35442d77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e042d7a, SUC_lexicon_tags_65 },
    { 0x42e42d7a, SUC_lexicon_tags_24 },
    { 0x38442d7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3042d80, SUC_lexicon_tags_17 },
    { 0x95c42d80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40e42d83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6242d8a, SUC_lexicon_tags_45 },
    { 0x35242d8b, SUC_lexicon_tags_25 },
    { 0x93a42d8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcd042d8e, SUC_lexicon_tags_42 },
    { 0x2d442d8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa42d93, SUC_lexicon_tags_10 },
    { 0xe8e42d94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97642d96, SUC_lexicon_tags_39 },
    { 0xc5642d96, SUC_lexicon_tags_28 },
    { 0xcac42d98, SUC_lexicon_tags_66 },
    { 0x5f442d99, SUC_lexicon_tags_7 },
    { 0xf5a42d9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4642d9c, SUC_lexicon_tags_30 },
    { 0x6a642d9c, SUC_lexicon_tags_46 },
    { 0x14242d9e, SUC_lexicon_tags_66 },
    { 0xe6e42d9f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9fa42da1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee042da3, SUC_lexicon_tags_11 },
    { 0x92a42da3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30842da6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x19642da8, SUC_lexicon_tags_82 },
    { 0xed242da9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c42dac, SUC_lexicon_tags_43 },
    { 0xb7642dad, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c642db0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32042dba, SUC_lexicon_tags_27 },
    { 0x85242dbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4242dc1, SUC_lexicon_tags_39 },
    { 0x4ae42dc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3242dca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e42dcd, SUC_lexicon_tags_60 },
    { 0x3ca42dcd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c42dd1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a642dd9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a842de2, SUC_lexicon_tags_5 },
    { 0x29442de3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc6c42de5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb1c42de7, SUC_lexicon_tags_84 },
    { 0x51242de8, SUC_lexicon_tags_37 },
    { 0xe4042de9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63442dee, SUC_lexicon_tags_17 },
    { 0x86a42def, SUC_lexicon_tags_47 },
    { 0xb3842dee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7a42df2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a42df5, SUC_lexicon_tags_15 },
    { 0xf2442df6, SUC_lexicon_tags_12 },
    { 0xb3642df7, SUC_lexicon_tags_2 },
    { 0x4fc42df8, SUC_lexicon_tags_0 },
    { 0x7b042df8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x23c42dfb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d242dfe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3fa42e00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c042e09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e42e0c, SUC_lexicon_tags_37 },
    { 0x7f042e0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a42e12, SUC_lexicon_tags_63 },
    { 0x4642e13, SUC_lexicon_tags_38 },
    { 0xd0a42e14, SUC_lexicon_tags_12 },
    { 0x88442e12, SUC_lexicon_tags_1 },
    { 0xb842e15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4642e1a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa42e1f, SUC_lexicon_tags_7 },
    { 0x2d842e1f, SUC_lexicon_tags_15 },
    { 0xc2242e20, SUC_lexicon_tags_63 },
    { 0xc7842e1f, SUC_lexicon_tags_27 },
    { 0xe6642e1f, SUC_lexicon_tags_17 },
    { 0x2a842e23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc42e2c, SUC_lexicon_tags_19 },
    { 0x13442e2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60642e2f, SUC_lexicon_tags_40 },
    { 0x35042e2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e642e38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1e42e3a, SUC_lexicon_tags_10 },
    { 0x95642e3b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa442e3e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa42e41, SUC_lexicon_tags_44 },
    { 0x71642e41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5442e47, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da42e4a, SUC_lexicon_tags_30 },
    { 0x78c42e4b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e42e51, SUC_lexicon_tags_12 },
    { 0x2f242e52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93042e54, SUC_lexicon_tags_43 },
    { 0x35442e55, SUC_lexicon_tags_7 },
    { 0xea842e55, SUC_lexicon_tags_30 },
    { 0x97842e57, SUC_lexicon_tags_15 },
    { 0x19442e58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a042e5e, SUC_lexicon_tags_25 },
    { 0x93a42e5f, SUC_lexicon_tags_8 },
    { 0x65e42e60, SUC_lexicon_tags_11 },
    { 0xeae42e61, SUC_lexicon_tags_15 },
    { 0x2e442e60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae042e64, SUC_lexicon_tags_9 },
    { 0x4ba42e65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f442e68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41842e6b, SUC_lexicon_tags_12 },
    { 0x10a42e6c, SUC_lexicon_tags_38 },
    { 0xb1042e6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a642e6f, SUC_lexicon_tags_38 },
    { 0x1ce42e70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ca42e72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73a42e74, SUC_lexicon_tags_27 },
    { 0xa4242e74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xae642e78, SUC_lexicon_tags_23 },
    { 0xda242e79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e42e7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4442e7e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa1842e80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13042e83, SUC_lexicon_tags_0 },
    { 0xeca42e84, SUC_lexicon_tags_558 },
    { 0x65442e84, SUC_lexicon_tags_87 },
    { 0x2e42e86, SUC_lexicon_tags_9 },
    { 0x46c42e87, SUC_lexicon_tags_60 },
    { 0x47642e87, SUC_lexicon_tags_1 },
    { 0x76242e89, SUC_lexicon_tags_47 },
    { 0x33642e8a, SUC_lexicon_tags_12 },
    { 0xa6e42e87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe442e8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9042e94, SUC_lexicon_tags_65 },
    { 0xf3c42e95, SUC_lexicon_tags_57 },
    { 0xf6c42e96, SUC_lexicon_tags_1 },
    { 0x17c42e96, SUC_lexicon_tags_25 },
    { 0x8c42e96, SUC_lexicon_tags_56 },
    { 0x92c42e99, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c442ea4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2a442ea6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf242ea8, SUC_lexicon_tags_8 },
    { 0xe3442ea9, SUC_lexicon_tags_136 },
    { 0xccc42eaa, SUC_lexicon_tags_17 },
    { 0x12642ea9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xea442ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67042eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6842eb1, SUC_lexicon_tags_14 },
    { 0x89442eb2, SUC_lexicon_tags_63 },
    { 0x8042eb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3842eb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2042eb9, SUC_lexicon_tags_49 },
    { 0x18242eba, SUC_lexicon_tags_27 },
    { 0x5ea42eb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e42ec0, SUC_lexicon_tags_11 },
    { 0xcc442ec1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36242eca, SUC_lexicon_tags_11 },
    { 0x4d842ecb, SUC_lexicon_tags_10 },
    { 0x29a42ecc, SUC_lexicon_tags_15 },
    { 0x2242ecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77042ed3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a42ed6, SUC_lexicon_tags_47 },
    { 0x44e42ed6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0042ed9, SUC_lexicon_tags_14 },
    { 0x5d042ed9, SUC_lexicon_tags_38 },
    { 0x89842eda, SUC_lexicon_tags_61 },
    { 0x4bc42edc, SUC_lexicon_tags_1 },
    { 0x1ec42edb, SUC_lexicon_tags_63 },
    { 0x46a42eda, SUC_lexicon_tags_28 },
    { 0x2d042edb, SUC_lexicon_tags_12 },
    { 0x6ec42ee0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x9ec42ee2, SUC_lexicon_tags_37 },
    { 0x36642ee3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e42ee6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e42ee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x71442eec, SUC_lexicon_tags_27 },
    { 0xb4042eed, SUC_lexicon_tags_10 },
    { 0xe8a42eee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4442ef2, SUC_lexicon_tags_8 },
    { 0x5842ef3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4642ef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6442eff, SUC_lexicon_tags_15 },
    { 0x3642f00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8c42f02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e42f05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78242f07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21242f0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb642f0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4642f11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7642f13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e42f18, SUC_lexicon_tags_27 },
    { 0xd3842f18, SUC_lexicon_tags_133 },
    { 0x68842f1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3cc42f1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x77a42f1e, SUC_lexicon_tags_37 },
    { 0x63042f1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0c42f21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10842f27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c442f2d, SUC_lexicon_tags_25 },
    { 0x89a42f2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadc42f30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ce42f32, SUC_lexicon_tags_37 },
    { 0x45c42f33, SUC_lexicon_tags_1 },
    { 0xf1442f32, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce42f37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13842f39, SUC_lexicon_tags_11 },
    { 0x3842f3a, SUC_lexicon_tags_25 },
    { 0x8bc42f3a, SUC_lexicon_tags_11 },
    { 0xa8c42f39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42242f43, SUC_lexicon_tags_17 },
    { 0x87a42f43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4242f49, SUC_lexicon_tags_1 },
    { 0x6dc42f4a, SUC_lexicon_tags_12 },
    { 0x68a42f4b, SUC_lexicon_tags_44 },
    { 0xb3042f4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12042f4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26642f54, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xcca42f56, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53842f5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f442f5c, SUC_lexicon_tags_30 },
    { 0x4c042f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e42f67, SUC_lexicon_tags_24 },
    { 0xb1a42f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13042f6c, SUC_lexicon_tags_13 },
    { 0xc1a42f6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c42f6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27642f75, SUC_lexicon_tags_30 },
    { 0x44242f76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e42f7a, SUC_lexicon_tags_11 },
    { 0xa7e42f7b, SUC_lexicon_tags_12 },
    { 0x42c42f7b, SUC_lexicon_tags_65 },
    { 0xe4a42f7d, SUC_lexicon_tags_10 },
    { 0xc9042f7d, SUC_lexicon_tags_12 },
    { 0x92a42f7f, SUC_lexicon_tags_11 },
    { 0x55e42f7c, SUC_lexicon_tags_65 },
    { 0x6e242f81, SUC_lexicon_tags_7 },
    { 0x5e242f7a, SUC_lexicon_tags_84 },
    { 0xd5442f82, SUC_lexicon_tags_12 },
    { 0x79442f84, SUC_lexicon_tags_28 },
    { 0xf7042f85, SUC_lexicon_tags_45 },
    { 0x4a442f86, SUC_lexicon_tags_23 },
    { 0xe0c42f7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3bc42f89, SUC_lexicon_tags_37 },
    { 0xe7842f8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1042f8c, SUC_lexicon_tags_17 },
    { 0x43642f8d, SUC_lexicon_tags_17 },
    { 0x61442f8e, SUC_lexicon_tags_11 },
    { 0x6c842f8f, SUC_lexicon_tags_45 },
    { 0xc6c42f90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1242f96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc42f9d, SUC_lexicon_tags_2 },
    { 0xcf842f9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17042fa0, SUC_lexicon_tags_37 },
    { 0xaae42fa1, SUC_lexicon_tags_7 },
    { 0x59442fa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef442fa4, SUC_lexicon_tags_19 },
    { 0xdc42fa4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc42fa8, SUC_lexicon_tags_15 },
    { 0xa1042fa9, SUC_lexicon_tags_14 },
    { 0x21442faa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x39642fac, SUC_lexicon_tags_44 },
    { 0xf9442fad, SUC_lexicon_tags_7 },
    { 0x14e42fae, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d442fb3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x35842fb6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d042fbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e42fc1, SUC_lexicon_tags_13 },
    { 0xa6042fc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda042fc7, SUC_lexicon_tags_37 },
    { 0x90e42fc7, SUC_lexicon_tags_17 },
    { 0xe3a42fc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac442fd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74042fd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb442fd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c42fd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80e42fd9, SUC_lexicon_tags_37 },
    { 0x72e42fda, SUC_lexicon_tags_11 },
    { 0x87042fda, SUC_lexicon_tags_2 },
    { 0xa2642fdc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b442fe2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac42fe6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a42fe9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b842feb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5842ff2, SUC_lexicon_tags_101 },
    { 0xba042ff2, SUC_lexicon_tags_10 },
    { 0xcc642ff3, SUC_lexicon_tags_12 },
    { 0x30042ff5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0642ffe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a43001, SUC_lexicon_tags_38 },
    { 0x97a43002, SUC_lexicon_tags_15 },
    { 0x84a43002, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1243005, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56643008, SUC_lexicon_tags_21 },
    { 0x7de43008, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb24300b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaea4300d, SUC_lexicon_tags_0 },
    { 0x4b44300d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x38243010, SUC_lexicon_tags_12 },
    { 0xc9843011, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37043016, SUC_lexicon_tags_45 },
    { 0x72c43017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49c43019, SUC_lexicon_tags_12 },
    { 0x2e243019, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a24301f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba43022, SUC_lexicon_tags_37 },
    { 0x7d843022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11843025, SUC_lexicon_tags_39 },
    { 0xff043026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38e43028, SUC_lexicon_tags_1 },
    { 0x8bc43028, SUC_lexicon_tags_25 },
    { 0x8ae4302a, SUC_lexicon_tags_37 },
    { 0x7ca4302b, SUC_lexicon_tags_3 },
    { 0x68843029, SUC_lexicon_tags_15 },
    { 0x3e44302d, SUC_lexicon_tags_15 },
    { 0x5b843028, SUC_lexicon_tags_29 },
    { 0x6464302f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e043034, SUC_lexicon_tags_37 },
    { 0x44243034, SUC_lexicon_tags_11 },
    { 0xff043036, SUC_lexicon_tags_68 },
    { 0xfbe43037, SUC_lexicon_tags_17 },
    { 0xd8443038, SUC_lexicon_tags_7 },
    { 0xbf443039, SUC_lexicon_tags_30 },
    { 0x6a44303a, SUC_lexicon_tags_11 },
    { 0xa1243036, SUC_lexicon_tags_25 },
    { 0x2a44303c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e43043, SUC_lexicon_tags_6 },
    { 0x83243043, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61443046, SUC_lexicon_tags_7 },
    { 0xb643046, SUC_lexicon_tags_33 },
    { 0x5bc43048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd04304a, SUC_lexicon_tags_8 },
    { 0xe6e4304b, SUC_lexicon_tags_63 },
    { 0xd144304b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2643051, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0843057, SUC_lexicon_tags_26 },
    { 0x3d443058, SUC_lexicon_tags_15 },
    { 0x58243059, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x37c4305b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a4305f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6443065, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5c43067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78a43069, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2864306c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2924306e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a243070, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd8c43072, SUC_lexicon_tags_25 },
    { 0xc0043073, SUC_lexicon_tags_12 },
    { 0x71243074, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc4307e, SUC_lexicon_tags_19 },
    { 0x44a4307f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39843084, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77643086, SUC_lexicon_tags_21 },
    { 0x7e043086, SUC_lexicon_tags_12 },
    { 0xe2643088, SUC_lexicon_tags_0 },
    { 0xb2e43089, SUC_lexicon_tags_25 },
    { 0x3e04308a, SUC_lexicon_tags_11 },
    { 0xb7c4308b, SUC_lexicon_tags_37 },
    { 0x31e4308b, SUC_lexicon_tags_33 },
    { 0xd084308b, SUC_lexicon_tags_7 },
    { 0x5c64308e, SUC_lexicon_tags_17 },
    { 0x1d04308f, SUC_lexicon_tags_32 },
    { 0xa0643090, SUC_lexicon_tags_37 },
    { 0x7604308e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x26643094, SUC_lexicon_tags_7 },
    { 0x36443094, SUC_lexicon_tags_1 },
    { 0xcec43096, SUC_lexicon_tags_17 },
    { 0xdfe43096, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c4309d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca430a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6c0430a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0430ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xade430ae, SUC_lexicon_tags_65 },
    { 0xb90430af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x598430b1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4430b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa4c430b8, SUC_lexicon_tags_82 },
    { 0x328430b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a430bc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x146430bf, SUC_lexicon_tags_84 },
    { 0x6b0430c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x624430c3, SUC_lexicon_tags_37 },
    { 0x66c430c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc430c7, SUC_lexicon_tags_1 },
    { 0xe4e430c8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x32430ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba430cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38c430cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x122430d2, SUC_lexicon_tags_11 },
    { 0x3c430d2, SUC_lexicon_tags_0 },
    { 0x11e430d4, SUC_lexicon_tags_1 },
    { 0xf86430d3, SUC_lexicon_tags_1 },
    { 0x854430d6, SUC_lexicon_tags_15 },
    { 0x994430d7, SUC_lexicon_tags_9 },
    { 0xef6430d2, SUC_lexicon_tags_1 },
    { 0xa60430d9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12430dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x528430de, SUC_lexicon_tags_0 },
    { 0xbe0430de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2e4430e1, SUC_lexicon_tags_38 },
    { 0x7fc430e2, SUC_lexicon_tags_17 },
    { 0x45e430e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52430e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6430ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714430f3, SUC_lexicon_tags_25 },
    { 0xb5c430f4, SUC_lexicon_tags_0 },
    { 0x8e8430f3, SUC_lexicon_tags_10 },
    { 0x5b0430f6, SUC_lexicon_tags_30 },
    { 0x65a430f7, SUC_lexicon_tags_25 },
    { 0xf9c430f6, SUC_lexicon_tags_17 },
    { 0xf34430f9, SUC_lexicon_tags_11 },
    { 0xb78430fa, SUC_lexicon_tags_7 },
    { 0xf64430f7, SUC_lexicon_tags_1 },
    { 0xc8c430fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6f2430fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e243100, SUC_lexicon_tags_39 },
    { 0x52243100, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aa43103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fe43105, SUC_lexicon_tags_26 },
    { 0x4b443106, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf443109, SUC_lexicon_tags_14 },
    { 0x95e43109, SUC_lexicon_tags_38 },
    { 0xd3243109, SUC_lexicon_tags_104 },
    { 0x2be4310b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec843115, SUC_lexicon_tags_9 },
    { 0x44e43116, SUC_lexicon_tags_17 },
    { 0x74c43117, SUC_lexicon_tags_0 },
    { 0x96043118, SUC_lexicon_tags_37 },
    { 0x83a43119, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c4311d, SUC_lexicon_tags_11 },
    { 0xd024311e, SUC_lexicon_tags_23 },
    { 0xd244311e, SUC_lexicon_tags_3 },
    { 0xc2a43120, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6f043122, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf544312a, SUC_lexicon_tags_21 },
    { 0x8744312a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2244312e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66043130, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d643134, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65243139, SUC_lexicon_tags_2 },
    { 0xb824313a, SUC_lexicon_tags_87 },
    { 0xb424313b, SUC_lexicon_tags_17 },
    { 0xe244313c, SUC_lexicon_tags_38 },
    { 0x1dc4313d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab64313f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x72843141, SUC_lexicon_tags_49 },
    { 0xc8243142, SUC_lexicon_tags_23 },
    { 0x2d443142, SUC_lexicon_tags_17 },
    { 0xf7043144, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2643146, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c4314a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e24314c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa43151, SUC_lexicon_tags_16 },
    { 0xa0443152, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e43157, SUC_lexicon_tags_24 },
    { 0xfe643157, SUC_lexicon_tags_61 },
    { 0x24043158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e4315c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2c84315e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2443166, SUC_lexicon_tags_33 },
    { 0xa7c43166, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd8043169, SUC_lexicon_tags_47 },
    { 0xc4e4316a, SUC_lexicon_tags_7 },
    { 0x5d043169, SUC_lexicon_tags_37 },
    { 0xf0c4316a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d443170, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c843178, SUC_lexicon_tags_63 },
    { 0x6c443179, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2f44317b, SUC_lexicon_tags_25 },
    { 0xb744317c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x65c4317e, SUC_lexicon_tags_84 },
    { 0xd924317e, SUC_lexicon_tags_27 },
    { 0x2c643180, SUC_lexicon_tags_7 },
    { 0x98643181, SUC_lexicon_tags_20 },
    { 0x23443180, SUC_lexicon_tags_16 },
    { 0x43643183, SUC_lexicon_tags_35 },
    { 0xc4a43181, SUC_lexicon_tags_28 },
    { 0xb6a43185, SUC_lexicon_tags_37 },
    { 0x8d643183, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8243189, SUC_lexicon_tags_27 },
    { 0x5e84318a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a44318d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e43192, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49843197, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9a043199, SUC_lexicon_tags_0 },
    { 0x6f643199, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xed04319c, SUC_lexicon_tags_10 },
    { 0xe824319c, SUC_lexicon_tags_37 },
    { 0x7f04319e, SUC_lexicon_tags_2 },
    { 0xd664319e, SUC_lexicon_tags_11 },
    { 0xe74431a0, SUC_lexicon_tags_1 },
    { 0x3f6431a0, SUC_lexicon_tags_14 },
    { 0x614431a2, SUC_lexicon_tags_25 },
    { 0x492431a2, SUC_lexicon_tags_11 },
    { 0x840431a0, SUC_lexicon_tags_37 },
    { 0xb36431a5, SUC_lexicon_tags_30 },
    { 0x288431a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0431aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0e431ac, SUC_lexicon_tags_27 },
    { 0xd08431ac, SUC_lexicon_tags_4 },
    { 0xc0c431ac, SUC_lexicon_tags_19 },
    { 0xd28431ac, SUC_lexicon_tags_38 },
    { 0x3c0431ac, SUC_lexicon_tags_19 },
    { 0xe431ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a431b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8431c2, SUC_lexicon_tags_37 },
    { 0x808431c3, SUC_lexicon_tags_37 },
    { 0xc80431c4, SUC_lexicon_tags_0 },
    { 0xfc2431c5, SUC_lexicon_tags_1 },
    { 0xb48431c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e431c9, SUC_lexicon_tags_12 },
    { 0x60c431c9, SUC_lexicon_tags_39 },
    { 0x184431cb, SUC_lexicon_tags_17 },
    { 0xa8431cc, SUC_lexicon_tags_43 },
    { 0xed4431cc, SUC_lexicon_tags_26 },
    { 0x36c431c9, SUC_lexicon_tags_40 },
    { 0x9d8431cd, SUC_lexicon_tags_11 },
    { 0xb58431d0, SUC_lexicon_tags_17 },
    { 0x3e6431ca, SUC_lexicon_tags_17 },
    { 0x78a431d2, SUC_lexicon_tags_17 },
    { 0xa12431d3, SUC_lexicon_tags_47 },
    { 0x724431d2, SUC_lexicon_tags_17 },
    { 0xdca431d2, SUC_lexicon_tags_47 },
    { 0x43a431d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74431dd, SUC_lexicon_tags_37 },
    { 0x714431dd, SUC_lexicon_tags_37 },
    { 0x250431de, SUC_lexicon_tags_25 },
    { 0x166431e0, SUC_lexicon_tags_26 },
    { 0xa9a431de, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658431e5, SUC_lexicon_tags_0 },
    { 0xcbc431e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x582431e9, SUC_lexicon_tags_10 },
    { 0x6ea431ea, SUC_lexicon_tags_3 },
    { 0x21c431eb, SUC_lexicon_tags_37 },
    { 0x482431e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea431f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46e431f3, SUC_lexicon_tags_32 },
    { 0x400431f4, SUC_lexicon_tags_8 },
    { 0xd96431f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c0431f7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2431fd, SUC_lexicon_tags_7 },
    { 0xe32431fe, SUC_lexicon_tags_70 },
    { 0x1d0431fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a43207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce4320d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc43215, SUC_lexicon_tags_15 },
    { 0x5ae43216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94643218, SUC_lexicon_tags_17 },
    { 0xa9643219, SUC_lexicon_tags_25 },
    { 0x8ac43218, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1c24321c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43220, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e43229, SUC_lexicon_tags_12 },
    { 0x7964322a, SUC_lexicon_tags_33 },
    { 0xc9e43229, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be4322f, SUC_lexicon_tags_0 },
    { 0x3a43230, SUC_lexicon_tags_17 },
    { 0xe2a4322f, SUC_lexicon_tags_15 },
    { 0x3484322f, SUC_lexicon_tags_17 },
    { 0x18a43232, SUC_lexicon_tags_15 },
    { 0x9ca4322f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a643238, SUC_lexicon_tags_60 },
    { 0xbc43238, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5843241, SUC_lexicon_tags_2 },
    { 0x3f043241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a43248, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a43251, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18643257, SUC_lexicon_tags_10 },
    { 0x3443258, SUC_lexicon_tags_27 },
    { 0xcc443259, SUC_lexicon_tags_12 },
    { 0x9b44325a, SUC_lexicon_tags_14 },
    { 0xfa04325b, SUC_lexicon_tags_27 },
    { 0x5e64325c, SUC_lexicon_tags_25 },
    { 0xd1c4325c, SUC_lexicon_tags_17 },
    { 0x3bc4325e, SUC_lexicon_tags_28 },
    { 0x3384325f, SUC_lexicon_tags_21 },
    { 0xce43260, SUC_lexicon_tags_29 },
    { 0x4a043261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4243264, SUC_lexicon_tags_11 },
    { 0x9ea43264, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20843275, SUC_lexicon_tags_1 },
    { 0x43a43276, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c43279, SUC_lexicon_tags_1 },
    { 0xd324327a, SUC_lexicon_tags_11 },
    { 0x2164327b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb404327d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8244327f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72043282, SUC_lexicon_tags_7 },
    { 0x3dc43283, SUC_lexicon_tags_37 },
    { 0xffc43284, SUC_lexicon_tags_77 },
    { 0x99e43285, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa044328a, SUC_lexicon_tags_12 },
    { 0x9e04328b, SUC_lexicon_tags_32 },
    { 0xc164328a, SUC_lexicon_tags_29 },
    { 0x6664328c, SUC_lexicon_tags_12 },
    { 0xa984328c, SUC_lexicon_tags_14 },
    { 0x20a4328f, SUC_lexicon_tags_25 },
    { 0x5704328b, SUC_lexicon_tags_26 },
    { 0x51c43291, SUC_lexicon_tags_28 },
    { 0xa6243292, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a4329a, SUC_lexicon_tags_37 },
    { 0x1784329a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8432a0, SUC_lexicon_tags_25 },
    { 0xd22432a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c432a4, SUC_lexicon_tags_63 },
    { 0x574432a4, SUC_lexicon_tags_10 },
    { 0xfe432a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9a432a8, SUC_lexicon_tags_17 },
    { 0xd2a432a9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6be432ab, SUC_lexicon_tags_37 },
    { 0x384432ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0432bf, SUC_lexicon_tags_17 },
    { 0x48a432c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96432c5, SUC_lexicon_tags_15 },
    { 0x10432c6, SUC_lexicon_tags_11 },
    { 0x8dc432c7, SUC_lexicon_tags_31 },
    { 0xf24432c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7fc432ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28432cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe432cf, SUC_lexicon_tags_1 },
    { 0x324432d0, SUC_lexicon_tags_47 },
    { 0xb12432d0, SUC_lexicon_tags_26 },
    { 0xe14432d2, SUC_lexicon_tags_45 },
    { 0xe28432d3, SUC_lexicon_tags_49 },
    { 0x210432d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0432d7, SUC_lexicon_tags_47 },
    { 0x2e6432d8, SUC_lexicon_tags_38 },
    { 0xac2432d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58a432db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda432de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a432e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0432e5, SUC_lexicon_tags_37 },
    { 0x53c432e6, SUC_lexicon_tags_25 },
    { 0x344432e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a432ea, SUC_lexicon_tags_17 },
    { 0xb90432ea, SUC_lexicon_tags_37 },
    { 0x5be432ea, SUC_lexicon_tags_13 },
    { 0xf66432ed, SUC_lexicon_tags_43 },
    { 0xdb0432ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57e432f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c432f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac432fe, SUC_lexicon_tags_33 },
    { 0xf4e432fe, SUC_lexicon_tags_15 },
    { 0x98e432fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c43304, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29443307, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de4330a, SUC_lexicon_tags_65 },
    { 0x8da4330b, SUC_lexicon_tags_11 },
    { 0x93a4330a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe04330e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec43311, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a43314, SUC_lexicon_tags_17 },
    { 0x2ca43315, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe43318, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f243322, SUC_lexicon_tags_11 },
    { 0xdc843323, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcca43325, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c24332d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c43331, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e43334, SUC_lexicon_tags_60 },
    { 0x16843334, SUC_lexicon_tags_14 },
    { 0x52643335, SUC_lexicon_tags_19 },
    { 0x8b643335, SUC_lexicon_tags_17 },
    { 0xc8c43338, SUC_lexicon_tags_19 },
    { 0x3b243339, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e4333e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f443345, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a4334a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e4334d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e43351, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84243355, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4944335a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a04335e, SUC_lexicon_tags_19 },
    { 0x6424335e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a43364, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb443368, SUC_lexicon_tags_17 },
    { 0x57a43369, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf84336e, SUC_lexicon_tags_43 },
    { 0x824336f, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca43378, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f44337e, SUC_lexicon_tags_37 },
    { 0xaac4337f, SUC_lexicon_tags_12 },
    { 0xf7c43380, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79643387, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da4338a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3364338d, SUC_lexicon_tags_25 },
    { 0x94a4338e, SUC_lexicon_tags_93 },
    { 0xf2a4338e, SUC_lexicon_tags_32 },
    { 0x7b243390, SUC_lexicon_tags_46 },
    { 0x9ac43390, SUC_lexicon_tags_25 },
    { 0xb444338f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d243394, SUC_lexicon_tags_37 },
    { 0x1b843395, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58643399, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7904339e, SUC_lexicon_tags_47 },
    { 0x8324339f, SUC_lexicon_tags_91 },
    { 0xb48433a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb0433a2, SUC_lexicon_tags_15 },
    { 0xd8e433a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c433a7, SUC_lexicon_tags_14 },
    { 0xd6c433a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16433ab, SUC_lexicon_tags_17 },
    { 0xf4433ac, SUC_lexicon_tags_1 },
    { 0x72433ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c433b0, SUC_lexicon_tags_12 },
    { 0xa7c433b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36433b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c433be, SUC_lexicon_tags_42 },
    { 0xcdc433bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6433c4, SUC_lexicon_tags_3 },
    { 0x45e433c4, SUC_lexicon_tags_21 },
    { 0x92e433c6, SUC_lexicon_tags_7 },
    { 0xaf2433c7, SUC_lexicon_tags_10 },
    { 0xdd2433c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc54433ca, SUC_lexicon_tags_11 },
    { 0xf00433ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae433d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x678433d2, SUC_lexicon_tags_39 },
    { 0x5b8433d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8fc433d5, SUC_lexicon_tags_8 },
    { 0x60e433d6, SUC_lexicon_tags_15 },
    { 0x958433d7, SUC_lexicon_tags_47 },
    { 0x7dc433d8, SUC_lexicon_tags_15 },
    { 0x354433d9, SUC_lexicon_tags_12 },
    { 0x7a0433d8, SUC_lexicon_tags_7 },
    { 0x6f8433db, SUC_lexicon_tags_1 },
    { 0x5b4433db, SUC_lexicon_tags_33 },
    { 0x562433db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe68433df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4433e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf7a433e5, SUC_lexicon_tags_47 },
    { 0x700433e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2433e9, SUC_lexicon_tags_11 },
    { 0x60433e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30c433ec, SUC_lexicon_tags_64 },
    { 0xa96433ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da433f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfc433f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1c6433f5, SUC_lexicon_tags_12 },
    { 0x8f2433f5, SUC_lexicon_tags_45 },
    { 0xa7c433f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da433fb, SUC_lexicon_tags_15 },
    { 0xfec433fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2433ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde43405, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584340e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e43412, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a4341f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x40043422, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65643426, SUC_lexicon_tags_60 },
    { 0x6e443427, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c4342e, SUC_lexicon_tags_129 },
    { 0x1c24342f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e43432, SUC_lexicon_tags_9 },
    { 0xdd043433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa443439, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x464343b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a43440, SUC_lexicon_tags_28 },
    { 0x2ec43441, SUC_lexicon_tags_96 },
    { 0xe4c43442, SUC_lexicon_tags_39 },
    { 0x5fc43441, SUC_lexicon_tags_12 },
    { 0x6da43444, SUC_lexicon_tags_37 },
    { 0x77643443, SUC_lexicon_tags_43 },
    { 0xd2843443, SUC_lexicon_tags_35 },
    { 0xc3c43447, SUC_lexicon_tags_15 },
    { 0x25a43448, SUC_lexicon_tags_33 },
    { 0x60843449, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb644344b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb404344f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc43455, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc4345b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf604345d, SUC_lexicon_tags_11 },
    { 0xff04345e, SUC_lexicon_tags_1 },
    { 0x5944345e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9b443461, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7b243463, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d24346d, SUC_lexicon_tags_40 },
    { 0xe24346e, SUC_lexicon_tags_7 },
    { 0x45a4346f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdc243471, SUC_lexicon_tags_11 },
    { 0x79a43472, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84e43474, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e84347a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaf64347c, SUC_lexicon_tags_12 },
    { 0x26e4347d, SUC_lexicon_tags_15 },
    { 0xae84347e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa9643480, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee43485, SUC_lexicon_tags_15 },
    { 0x62a43485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9884348e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe7e43490, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce643495, SUC_lexicon_tags_25 },
    { 0x86443496, SUC_lexicon_tags_40 },
    { 0x8ee43496, SUC_lexicon_tags_25 },
    { 0xca443498, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xb364349a, SUC_lexicon_tags_37 },
    { 0x3964349b, SUC_lexicon_tags_33 },
    { 0xaae4349b, SUC_lexicon_tags_55 },
    { 0xba24349d, SUC_lexicon_tags_65 },
    { 0x63e4349a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92434a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a6434a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a434a8, SUC_lexicon_tags_17 },
    { 0x86434a9, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e434ad, SUC_lexicon_tags_46 },
    { 0x182434ad, SUC_lexicon_tags_11 },
    { 0x6c4434af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e434b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596434bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0434be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa434c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x348434c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ea434c5, SUC_lexicon_tags_60 },
    { 0xe70434c6, SUC_lexicon_tags_46 },
    { 0x550434c7, SUC_lexicon_tags_27 },
    { 0x600434c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec434cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8434d5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384434d9, SUC_lexicon_tags_12 },
    { 0x9e0434d9, SUC_lexicon_tags_25 },
    { 0xc6e434db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa434e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18434e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x824434e7, SUC_lexicon_tags_43 },
    { 0xd88434e7, SUC_lexicon_tags_25 },
    { 0xf8434e8, SUC_lexicon_tags_11 },
    { 0x612434e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8b2434ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18434ef, SUC_lexicon_tags_2 },
    { 0xf10434f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8434f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac8434f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30434ff, SUC_lexicon_tags_55 },
    { 0x95a434ff, SUC_lexicon_tags_47 },
    { 0x5e443500, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b443504, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe843506, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce43509, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1064350d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e43510, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61843516, SUC_lexicon_tags_9 },
    { 0xe1843516, SUC_lexicon_tags_38 },
    { 0x24043518, SUC_lexicon_tags_29 },
    { 0x1f843518, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe544351c, SUC_lexicon_tags_55 },
    { 0x6864351d, SUC_lexicon_tags_2 },
    { 0xa004351e, SUC_lexicon_tags_19 },
    { 0xbb84351c, SUC_lexicon_tags_15 },
    { 0xe5a43520, SUC_lexicon_tags_19 },
    { 0x9ee43521, SUC_lexicon_tags_13 },
    { 0xcf443522, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c4352c, SUC_lexicon_tags_25 },
    { 0x17c4352d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8bc4352f, SUC_lexicon_tags_17 },
    { 0xd5c4352f, SUC_lexicon_tags_0 },
    { 0x8e43531, SUC_lexicon_tags_27 },
    { 0xece43532, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99843534, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb984353a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e43540, SUC_lexicon_tags_7 },
    { 0x443541, SUC_lexicon_tags_47 },
    { 0x87643541, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbfc43544, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x1443547, SUC_lexicon_tags_31 },
    { 0x89643548, SUC_lexicon_tags_15 },
    { 0x30043549, SUC_lexicon_tags_43 },
    { 0xfa44354a, SUC_lexicon_tags_37 },
    { 0x5964354a, SUC_lexicon_tags_7 },
    { 0xe2c4354c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77843554, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa443558, SUC_lexicon_tags_0 },
    { 0xbca43559, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e243566, SUC_lexicon_tags_27 },
    { 0x17e43566, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a04356b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0e4356d, SUC_lexicon_tags_17 },
    { 0xba04356e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac43571, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x52043574, SUC_lexicon_tags_31 },
    { 0xda43574, SUC_lexicon_tags_2 },
    { 0x9c43575, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbd443578, SUC_lexicon_tags_15 },
    { 0xfca43579, SUC_lexicon_tags_55 },
    { 0x41c4357a, SUC_lexicon_tags_37 },
    { 0x4184357b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab84357f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33243581, SUC_lexicon_tags_12 },
    { 0xf1e43582, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x42643584, SUC_lexicon_tags_23 },
    { 0xed843585, SUC_lexicon_tags_17 },
    { 0xb6e43584, SUC_lexicon_tags_6 },
    { 0x2ba43585, SUC_lexicon_tags_11 },
    { 0xba643588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0443591, SUC_lexicon_tags_28 },
    { 0x50643591, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a43595, SUC_lexicon_tags_77 },
    { 0x80443596, SUC_lexicon_tags_44 },
    { 0xca843596, SUC_lexicon_tags_2 },
    { 0x9b043598, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x224359b, SUC_lexicon_tags_38 },
    { 0xf424359c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a435a3, SUC_lexicon_tags_11 },
    { 0x472435a4, SUC_lexicon_tags_63 },
    { 0xa36435a3, SUC_lexicon_tags_17 },
    { 0x112435a5, SUC_lexicon_tags_2 },
    { 0xf98435a7, SUC_lexicon_tags_87 },
    { 0x846435a8, SUC_lexicon_tags_27 },
    { 0xd16435a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x182435ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32435ae, SUC_lexicon_tags_23 },
    { 0xcd4435ae, SUC_lexicon_tags_11 },
    { 0xfde435af, SUC_lexicon_tags_27 },
    { 0x54c435b1, SUC_lexicon_tags_15 },
    { 0x450435ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a435b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a435b9, SUC_lexicon_tags_35 },
    { 0xb96435ba, SUC_lexicon_tags_12 },
    { 0x53e435bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20435bf, SUC_lexicon_tags_37 },
    { 0xab6435c0, SUC_lexicon_tags_17 },
    { 0x6f2435bf, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc435c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x104435ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a435ce, SUC_lexicon_tags_8 },
    { 0x230435cf, SUC_lexicon_tags_37 },
    { 0xfea435d0, SUC_lexicon_tags_47 },
    { 0xfb0435d1, SUC_lexicon_tags_10 },
    { 0x580435d0, SUC_lexicon_tags_37 },
    { 0x3e435cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334435d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a435dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6435e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9b0435e4, SUC_lexicon_tags_8 },
    { 0x39e435e5, SUC_lexicon_tags_43 },
    { 0x46a435e6, SUC_lexicon_tags_17 },
    { 0x50c435e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24c435e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0435ee, SUC_lexicon_tags_47 },
    { 0x564435ef, SUC_lexicon_tags_87 },
    { 0x4c2435ee, SUC_lexicon_tags_49 },
    { 0x2e6435ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e435f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea435fb, SUC_lexicon_tags_15 },
    { 0x7a4435fb, SUC_lexicon_tags_1 },
    { 0x3ea435fc, SUC_lexicon_tags_23 },
    { 0xda0435fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e43602, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcb843604, SUC_lexicon_tags_32 },
    { 0xf443604, SUC_lexicon_tags_11 },
    { 0xbe443606, SUC_lexicon_tags_7 },
    { 0xaac43607, SUC_lexicon_tags_28 },
    { 0x6b843608, SUC_lexicon_tags_87 },
    { 0x47043609, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc44360c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x75a4360e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c43613, SUC_lexicon_tags_23 },
    { 0x5be43614, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a43617, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c443619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb124361b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f24361e, SUC_lexicon_tags_25 },
    { 0x8844361f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f843622, SUC_lexicon_tags_15 },
    { 0x87c43622, SUC_lexicon_tags_37 },
    { 0x9ae43624, SUC_lexicon_tags_37 },
    { 0x54043622, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda843628, SUC_lexicon_tags_15 },
    { 0x18043629, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xce44362c, SUC_lexicon_tags_15 },
    { 0x6544362c, SUC_lexicon_tags_11 },
    { 0xf944362e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28c43630, SUC_lexicon_tags_12 },
    { 0xd9843631, SUC_lexicon_tags_7 },
    { 0xad643632, SUC_lexicon_tags_0 },
    { 0x52c43633, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde4363b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6644363d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a43643, SUC_lexicon_tags_49 },
    { 0xc8e43643, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9444364a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a43650, SUC_lexicon_tags_3 },
    { 0x61e43651, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78443656, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x80843659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d84365d, SUC_lexicon_tags_6 },
    { 0x7984365e, SUC_lexicon_tags_21 },
    { 0xd9a4365f, SUC_lexicon_tags_7 },
    { 0x1c24365f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3843665, SUC_lexicon_tags_33 },
    { 0x3b643666, SUC_lexicon_tags_160 },
    { 0x7243666, SUC_lexicon_tags_10 },
    { 0x14843667, SUC_lexicon_tags_14 },
    { 0x91043669, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf184366f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde843674, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfe243676, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d84367b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b64367d, SUC_lexicon_tags_68 },
    { 0xef24367e, SUC_lexicon_tags_29 },
    { 0x4b24367f, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x26043682, SUC_lexicon_tags_85 },
    { 0xcac43683, SUC_lexicon_tags_12 },
    { 0xfc443684, SUC_lexicon_tags_60 },
    { 0x60e43683, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7043687, SUC_lexicon_tags_30 },
    { 0x32843688, SUC_lexicon_tags_63 },
    { 0x59643688, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc4368e, SUC_lexicon_tags_38 },
    { 0x1f64368f, SUC_lexicon_tags_65 },
    { 0x7b04368e, SUC_lexicon_tags_37 },
    { 0xb64368e, SUC_lexicon_tags_43 },
    { 0xcce4368e, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x2643694, SUC_lexicon_tags_46 },
    { 0x6c843695, SUC_lexicon_tags_0 },
    { 0xf5643696, SUC_lexicon_tags_37 },
    { 0xb1a43697, SUC_lexicon_tags_32 },
    { 0x9c243698, SUC_lexicon_tags_25 },
    { 0x77443699, SUC_lexicon_tags_17 },
    { 0xd3643699, SUC_lexicon_tags_15 },
    { 0x10443697, SUC_lexicon_tags_38 },
    { 0xc364369c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6464369e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xaa4436a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xff6436a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c436ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x630436ae, SUC_lexicon_tags_2 },
    { 0x178436af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8436b2, SUC_lexicon_tags_25 },
    { 0xe42436b3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec436bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc436bf, SUC_lexicon_tags_69 },
    { 0x74c436c0, SUC_lexicon_tags_17 },
    { 0x774436bf, SUC_lexicon_tags_49 },
    { 0xdca436c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd8436c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e436c7, SUC_lexicon_tags_17 },
    { 0xf5c436c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5a436ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc436ce, SUC_lexicon_tags_43 },
    { 0xb7e436cf, SUC_lexicon_tags_77 },
    { 0xa3a436d0, SUC_lexicon_tags_47 },
    { 0x46e436d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x202436d3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2436d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6436db, SUC_lexicon_tags_29 },
    { 0x702436dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0436e6, SUC_lexicon_tags_19 },
    { 0xb12436e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0436eb, SUC_lexicon_tags_15 },
    { 0x17e436ec, SUC_lexicon_tags_10 },
    { 0xd32436ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x64436f0, SUC_lexicon_tags_48 },
    { 0x298436f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x652436f3, SUC_lexicon_tags_60 },
    { 0x7ba436f3, SUC_lexicon_tags_15 },
    { 0xfa4436f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x388436f7, SUC_lexicon_tags_229 },
    { 0xc1a436f8, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x94e436fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86e436fc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c43704, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7c43706, SUC_lexicon_tags_17 },
    { 0xd2c43707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e4370c, SUC_lexicon_tags_25 },
    { 0x784370d, SUC_lexicon_tags_6 },
    { 0x9444370e, SUC_lexicon_tags_17 },
    { 0x5084370e, SUC_lexicon_tags_1 },
    { 0x35843710, SUC_lexicon_tags_8 },
    { 0x5ba43710, SUC_lexicon_tags_17 },
    { 0x51a4370e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a43717, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec243719, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c4371e, SUC_lexicon_tags_14 },
    { 0x2644371f, SUC_lexicon_tags_1 },
    { 0xe9443720, SUC_lexicon_tags_3 },
    { 0xe004371e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c443724, SUC_lexicon_tags_11 },
    { 0x33843725, SUC_lexicon_tags_8 },
    { 0x2b643726, SUC_lexicon_tags_28 },
    { 0x7fe43726, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c643730, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c43733, SUC_lexicon_tags_38 },
    { 0xc5443734, SUC_lexicon_tags_7 },
    { 0x36a43734, SUC_lexicon_tags_17 },
    { 0x84c43734, SUC_lexicon_tags_44 },
    { 0xdc443734, SUC_lexicon_tags_30 },
    { 0x7c443737, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8443740, SUC_lexicon_tags_2 },
    { 0xbbc43741, SUC_lexicon_tags_2 },
    { 0xafa43742, SUC_lexicon_tags_15 },
    { 0x7e643741, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29443748, SUC_lexicon_tags_16 },
    { 0x4cc43749, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xc9e4374b, SUC_lexicon_tags_11 },
    { 0xdd64374b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e43751, SUC_lexicon_tags_11 },
    { 0x4ee43752, SUC_lexicon_tags_28 },
    { 0x44443753, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b443755, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5844375d, SUC_lexicon_tags_37 },
    { 0xdcc4375d, SUC_lexicon_tags_33 },
    { 0x524375f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc43765, SUC_lexicon_tags_14 },
    { 0x50443765, SUC_lexicon_tags_17 },
    { 0xd0e43767, SUC_lexicon_tags_17 },
    { 0xe843768, SUC_lexicon_tags_11 },
    { 0xd2243769, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb564376b, SUC_lexicon_tags_21 },
    { 0x9e64376c, SUC_lexicon_tags_49 },
    { 0x8824376c, SUC_lexicon_tags_25 },
    { 0x8364376e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48843770, SUC_lexicon_tags_0 },
    { 0x10443770, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bc43773, SUC_lexicon_tags_351 },
    { 0x97a43774, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb2643776, SUC_lexicon_tags_1 },
    { 0xef843777, SUC_lexicon_tags_33 },
    { 0xf443778, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1d24377a, SUC_lexicon_tags_82 },
    { 0x84c4377a, SUC_lexicon_tags_1 },
    { 0x85e4377c, SUC_lexicon_tags_133 },
    { 0x68c4377d, SUC_lexicon_tags_49 },
    { 0x93c4377e, SUC_lexicon_tags_13 },
    { 0x3264377f, SUC_lexicon_tags_1 },
    { 0xb644377b, SUC_lexicon_tags_25 },
    { 0xd664377f, SUC_lexicon_tags_77 },
    { 0xf2843782, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7443784, SUC_lexicon_tags_7 },
    { 0x76043785, SUC_lexicon_tags_25 },
    { 0x8de43786, SUC_lexicon_tags_23 },
    { 0x5ae43787, SUC_lexicon_tags_1 },
    { 0x24243787, SUC_lexicon_tags_12 },
    { 0xcd243789, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b84378b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4204378d, SUC_lexicon_tags_12 },
    { 0xdf24378e, SUC_lexicon_tags_221 },
    { 0xf124378f, SUC_lexicon_tags_12 },
    { 0xf384378e, SUC_lexicon_tags_25 },
    { 0x5c843791, SUC_lexicon_tags_8 },
    { 0x5f843792, SUC_lexicon_tags_0 },
    { 0xf4843791, SUC_lexicon_tags_49 },
    { 0xf764378e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e4379f, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xea437a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c437a6, SUC_lexicon_tags_61 },
    { 0x3f8437a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6437ad, SUC_lexicon_tags_25 },
    { 0x16e437ad, SUC_lexicon_tags_15 },
    { 0x914437ad, SUC_lexicon_tags_7 },
    { 0x396437ad, SUC_lexicon_tags_17 },
    { 0x716437af, SUC_lexicon_tags_14 },
    { 0x566437b2, SUC_lexicon_tags_2 },
    { 0xed0437b3, SUC_lexicon_tags_47 },
    { 0xafc437b4, SUC_lexicon_tags_0 },
    { 0x6f2437af, SUC_lexicon_tags_25 },
    { 0x428437b6, SUC_lexicon_tags_17 },
    { 0x6d2437b6, SUC_lexicon_tags_19 },
    { 0x2b8437b8, SUC_lexicon_tags_234 },
    { 0x9aa437b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34437be, SUC_lexicon_tags_25 },
    { 0x67a437bf, SUC_lexicon_tags_9 },
    { 0x5be437c0, SUC_lexicon_tags_57 },
    { 0x698437c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520437c6, SUC_lexicon_tags_11 },
    { 0x848437c6, SUC_lexicon_tags_33 },
    { 0xfe6437c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea437cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3de437cd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e437d0, SUC_lexicon_tags_17 },
    { 0x9e2437d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd32437d3, SUC_lexicon_tags_12 },
    { 0x4f0437d3, SUC_lexicon_tags_37 },
    { 0xc50437d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c437e4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84437e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac437eb, SUC_lexicon_tags_7 },
    { 0x388437eb, SUC_lexicon_tags_25 },
    { 0xfa437eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504437f2, SUC_lexicon_tags_43 },
    { 0x3f2437f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e437f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e437f9, SUC_lexicon_tags_28 },
    { 0x3ba437fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc437ff, SUC_lexicon_tags_55 },
    { 0xdd443800, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f443802, SUC_lexicon_tags_25 },
    { 0x36843802, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x27643806, SUC_lexicon_tags_31 },
    { 0x9a443807, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x83a43809, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a4380d, SUC_lexicon_tags_66 },
    { 0x4144380d, SUC_lexicon_tags_66 },
    { 0xa1e4380f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xdb243811, SUC_lexicon_tags_11 },
    { 0xc3c43811, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e443814, SUC_lexicon_tags_37 },
    { 0x27643815, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2043818, SUC_lexicon_tags_33 },
    { 0x8e043819, SUC_lexicon_tags_31 },
    { 0xd6a4381a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf644381e, SUC_lexicon_tags_43 },
    { 0x81e4381e, SUC_lexicon_tags_12 },
    { 0x3c843820, SUC_lexicon_tags_4 },
    { 0xe4a4381f, SUC_lexicon_tags_63 },
    { 0x24443822, SUC_lexicon_tags_11 },
    { 0xc6c43823, SUC_lexicon_tags_15 },
    { 0xb9c43822, SUC_lexicon_tags_23 },
    { 0x2843824, SUC_lexicon_tags_25 },
    { 0xf6a43824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2443828, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf224382a, SUC_lexicon_tags_37 },
    { 0x684382a, SUC_lexicon_tags_10 },
    { 0xa24382c, SUC_lexicon_tags_49 },
    { 0xd464382a, SUC_lexicon_tags_11 },
    { 0xfd84382e, SUC_lexicon_tags_11 },
    { 0xdc24382f, SUC_lexicon_tags_16 },
    { 0x2c843830, SUC_lexicon_tags_2 },
    { 0xf5843831, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3043836, SUC_lexicon_tags_63 },
    { 0xa2843836, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e4383b, SUC_lexicon_tags_84 },
    { 0x7a84383c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8be4383e, SUC_lexicon_tags_17 },
    { 0x2864383e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62443841, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e643846, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc44384b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c43850, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x68443852, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d443855, SUC_lexicon_tags_45 },
    { 0x28643856, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4264385b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1824385d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37243862, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2843868, SUC_lexicon_tags_41 },
    { 0x2ce43869, SUC_lexicon_tags_11 },
    { 0xcfa4386a, SUC_lexicon_tags_6 },
    { 0x54e4386b, SUC_lexicon_tags_2 },
    { 0xd9e4386c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa784386e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56243872, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9643874, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa5243876, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac243878, SUC_lexicon_tags_17 },
    { 0xc1443878, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4387f, SUC_lexicon_tags_2 },
    { 0x1043880, SUC_lexicon_tags_49 },
    { 0x3fe43880, SUC_lexicon_tags_12 },
    { 0xb1243882, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x12843885, SUC_lexicon_tags_68 },
    { 0xf5443885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3c43888, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1204388b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac43890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b643893, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa1043895, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe043898, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x7324389a, SUC_lexicon_tags_47 },
    { 0x1784389b, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0x6304389d, SUC_lexicon_tags_15 },
    { 0x9ca4389d, SUC_lexicon_tags_26 },
    { 0x7604389f, SUC_lexicon_tags_61 },
    { 0x5364389f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4438a4, SUC_lexicon_tags_27 },
    { 0xa6c438a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c438ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc438b3, SUC_lexicon_tags_37 },
    { 0x12438b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8438b8, SUC_lexicon_tags_17 },
    { 0x7fe438b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2438bc, SUC_lexicon_tags_26 },
    { 0x5e2438bd, SUC_lexicon_tags_77 },
    { 0xc4438bc, SUC_lexicon_tags_37 },
    { 0x14e438bf, SUC_lexicon_tags_43 },
    { 0x18e438be, SUC_lexicon_tags_11 },
    { 0x9a4438c1, SUC_lexicon_tags_2 },
    { 0x22c438c2, SUC_lexicon_tags_26 },
    { 0x6e6438c3, SUC_lexicon_tags_47 },
    { 0x438c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da438cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x684438cd, SUC_lexicon_tags_61 },
    { 0x86c438ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb8438d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4a438d2, SUC_lexicon_tags_49 },
    { 0x470438d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1b2438d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e6438d7, SUC_lexicon_tags_38 },
    { 0xec0438d8, SUC_lexicon_tags_12 },
    { 0x5ac438d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e438df, SUC_lexicon_tags_6 },
    { 0xefc438e0, SUC_lexicon_tags_15 },
    { 0x9d8438e1, SUC_lexicon_tags_25 },
    { 0x2aa438df, SUC_lexicon_tags_21 },
    { 0x5c6438e3, SUC_lexicon_tags_28 },
    { 0xa2438e4, SUC_lexicon_tags_63 },
    { 0x608438df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c438ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a438ee, SUC_lexicon_tags_6 },
    { 0xac0438ef, SUC_lexicon_tags_57 },
    { 0x38e438ee, SUC_lexicon_tags_10 },
    { 0xe80438f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7cc438f3, SUC_lexicon_tags_1 },
    { 0xd2e438f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82438f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2438fd, SUC_lexicon_tags_136 },
    { 0x5dc438fd, SUC_lexicon_tags_1 },
    { 0xf06438ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba43902, SUC_lexicon_tags_6 },
    { 0xf5a43903, SUC_lexicon_tags_65 },
    { 0x86843904, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x65643906, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2243909, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4344390c, SUC_lexicon_tags_9 },
    { 0x8bc4390d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0643912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb043914, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c43918, SUC_lexicon_tags_15 },
    { 0x84243919, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c4391c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6843922, SUC_lexicon_tags_15 },
    { 0xfe643923, SUC_lexicon_tags_38 },
    { 0x60443923, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace43930, SUC_lexicon_tags_45 },
    { 0x1ec43931, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4ba43933, SUC_lexicon_tags_2 },
    { 0xe1e43933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e43937, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2444393d, SUC_lexicon_tags_28 },
    { 0x8604393e, SUC_lexicon_tags_32 },
    { 0xcd44393f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3243944, SUC_lexicon_tags_49 },
    { 0x91443945, SUC_lexicon_tags_17 },
    { 0x25843945, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4643948, SUC_lexicon_tags_21 },
    { 0x60443948, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a43951, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x84643953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a643955, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0443957, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba4395e, SUC_lexicon_tags_25 },
    { 0xcd24395f, SUC_lexicon_tags_2 },
    { 0x404395f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe43964, SUC_lexicon_tags_77 },
    { 0x4f243964, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a43968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a4396b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da43970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24243974, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d04397a, SUC_lexicon_tags_2 },
    { 0xb84397b, SUC_lexicon_tags_17 },
    { 0xd1a4397b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c4397f, SUC_lexicon_tags_45 },
    { 0x10643980, SUC_lexicon_tags_17 },
    { 0xf7443980, SUC_lexicon_tags_27 },
    { 0xf3843981, SUC_lexicon_tags_9 },
    { 0x9a843983, SUC_lexicon_tags_47 },
    { 0xa9a43983, SUC_lexicon_tags_26 },
    { 0x45c43980, SUC_lexicon_tags_55 },
    { 0x53043986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c4398f, SUC_lexicon_tags_47 },
    { 0x89e43990, SUC_lexicon_tags_8 },
    { 0xa0643991, SUC_lexicon_tags_10 },
    { 0x84e43991, SUC_lexicon_tags_11 },
    { 0x21243992, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x47843995, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404399a, SUC_lexicon_tags_48 },
    { 0xc824399b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718439a4, SUC_lexicon_tags_17 },
    { 0xf6439a4, SUC_lexicon_tags_2 },
    { 0x408439a6, SUC_lexicon_tags_11 },
    { 0x66e439a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c439ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x212439ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e439b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e439b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90439ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974439c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da439c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36439c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea439ca, SUC_lexicon_tags_37 },
    { 0xe28439ca, SUC_lexicon_tags_60 },
    { 0xb26439cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa32439ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a439d1, SUC_lexicon_tags_12 },
    { 0xe26439d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4439d5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x552439d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee6439d9, SUC_lexicon_tags_37 },
    { 0xba4439da, SUC_lexicon_tags_21 },
    { 0x1e2439db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf82439dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56439e0, SUC_lexicon_tags_30 },
    { 0x1b8439e1, SUC_lexicon_tags_32 },
    { 0xe28439e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0439e7, SUC_lexicon_tags_4 },
    { 0x5b8439e8, SUC_lexicon_tags_12 },
    { 0xb80439e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6439ec, SUC_lexicon_tags_1 },
    { 0xa5c439ed, SUC_lexicon_tags_0 },
    { 0x4bc439ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e439f2, SUC_lexicon_tags_7 },
    { 0x1ba439f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a439fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24439ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5043a03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ca43a05, SUC_lexicon_tags_25 },
    { 0x61a43a06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde43a0b, SUC_lexicon_tags_28 },
    { 0xf1c43a0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41a43a0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff843a10, SUC_lexicon_tags_15 },
    { 0x27043a10, SUC_lexicon_tags_65 },
    { 0x6fa43a12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a43a15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c43a18, SUC_lexicon_tags_12 },
    { 0xe5443a18, SUC_lexicon_tags_28 },
    { 0x3a643a1a, SUC_lexicon_tags_37 },
    { 0xc0843a19, SUC_lexicon_tags_48 },
    { 0x70c43a19, SUC_lexicon_tags_7 },
    { 0x46e43a1d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c43a23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc9e43a25, SUC_lexicon_tags_7 },
    { 0xb8043a25, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf3e43a28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb6843a2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a843a2d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9243a33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15443a37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76243a3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x25e43a3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0243a41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7443a46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3bc43a48, SUC_lexicon_tags_2 },
    { 0x3c843a49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x68643a4b, SUC_lexicon_tags_12 },
    { 0x43a43a4c, SUC_lexicon_tags_17 },
    { 0x5ce43a4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c43a50, SUC_lexicon_tags_37 },
    { 0xc843a50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a43a55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f643a58, SUC_lexicon_tags_5 },
    { 0x2fe43a58, SUC_lexicon_tags_19 },
    { 0x75843a5a, SUC_lexicon_tags_17 },
    { 0xd6e43a5b, SUC_lexicon_tags_68 },
    { 0xa2043a5a, SUC_lexicon_tags_0 },
    { 0x2a043a5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a43a60, SUC_lexicon_tags_22 },
    { 0xa1043a61, SUC_lexicon_tags_25 },
    { 0x11243a62, SUC_lexicon_tags_14 },
    { 0x28643a61, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e43a66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f043a69, SUC_lexicon_tags_191 },
    { 0xabe43a6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1d443a6c, SUC_lexicon_tags_1 },
    { 0x8d043a6d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2243a71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24443a79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae43a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e43a82, SUC_lexicon_tags_7 },
    { 0xcde43a83, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa5043a85, SUC_lexicon_tags_17 },
    { 0x9aa43a86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3843a90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb643a95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a443a98, SUC_lexicon_tags_1632 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc643a9b, SUC_lexicon_tags_12 },
    { 0x9e243a9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8643aa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb443aa6, SUC_lexicon_tags_17 },
    { 0xf6843aa7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5643aaa, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe043aad, SUC_lexicon_tags_15 },
    { 0xd8243aae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a643ab1, SUC_lexicon_tags_30 },
    { 0xa0c43ab1, SUC_lexicon_tags_69 },
    { 0x1243ab3, SUC_lexicon_tags_12 },
    { 0x22e43ab1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x29c43ab6, SUC_lexicon_tags_8 },
    { 0x14643ab7, SUC_lexicon_tags_48 },
    { 0xfa043ab8, SUC_lexicon_tags_62 },
    { 0x33a43ab9, SUC_lexicon_tags_25 },
    { 0xf5443aba, SUC_lexicon_tags_83 },
    { 0x86443abb, SUC_lexicon_tags_11 },
    { 0x5ac43ab8, SUC_lexicon_tags_25 },
    { 0xc7243abb, SUC_lexicon_tags_2 },
    { 0x99043abe, SUC_lexicon_tags_19 },
    { 0xefc43abf, SUC_lexicon_tags_23 },
    { 0x13a43ab7, SUC_lexicon_tags_47 },
    { 0xac443abc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61043ac6, SUC_lexicon_tags_16 },
    { 0x3ac43ac6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab643ace, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x12443ad1, SUC_lexicon_tags_38 },
    { 0x14643ad2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33243ad6, SUC_lexicon_tags_25 },
    { 0xe6843ad6, SUC_lexicon_tags_6 },
    { 0xeae43ad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d443ada, SUC_lexicon_tags_63 },
    { 0xab043adb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2243ade, SUC_lexicon_tags_9 },
    { 0xf3a43adf, SUC_lexicon_tags_25 },
    { 0x1e243ae0, SUC_lexicon_tags_11 },
    { 0x54a43ae1, SUC_lexicon_tags_11 },
    { 0x61043ae0, SUC_lexicon_tags_8 },
    { 0x4da43ae3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c443ae6, SUC_lexicon_tags_25 },
    { 0xa243ae7, SUC_lexicon_tags_29 },
    { 0x2fe43ae8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8f843aea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a43af5, SUC_lexicon_tags_1 },
    { 0x1e043af6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafe43af8, SUC_lexicon_tags_0 },
    { 0xcc043af8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a43afc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcc443afe, SUC_lexicon_tags_12 },
    { 0x95a43afe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69643b01, SUC_lexicon_tags_85 },
    { 0x36643b02, SUC_lexicon_tags_14 },
    { 0x47843b03, SUC_lexicon_tags_1 },
    { 0x21443b02, SUC_lexicon_tags_11 },
    { 0x9a643b04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x15e43b07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0a43b09, SUC_lexicon_tags_1 },
    { 0x4043b0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e43b0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c443b10, SUC_lexicon_tags_60 },
    { 0x4243b11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86243b14, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f843b19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a843b1c, SUC_lexicon_tags_34 },
    { 0x54843b1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee043b1f, SUC_lexicon_tags_0 },
    { 0x94243b20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3243b23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc443b27, SUC_lexicon_tags_7 },
    { 0x6c443b27, SUC_lexicon_tags_9 },
    { 0xf8843b29, SUC_lexicon_tags_39 },
    { 0x7f643b2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa243b2d, SUC_lexicon_tags_17 },
    { 0x77a43b2e, SUC_lexicon_tags_56 },
    { 0xc7c43b2d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6843b38, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7443b3c, SUC_lexicon_tags_15 },
    { 0x8b843b3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7843b3f, SUC_lexicon_tags_84 },
    { 0x9f043b40, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe43b44, SUC_lexicon_tags_28 },
    { 0xbbc43b45, SUC_lexicon_tags_12 },
    { 0xafc43b46, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6443b49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7043b4b, SUC_lexicon_tags_17 },
    { 0xb0a43b4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46043b52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88843b54, SUC_lexicon_tags_77 },
    { 0x78443b54, SUC_lexicon_tags_1 },
    { 0x5043b56, SUC_lexicon_tags_37 },
    { 0x843b57, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xef443b59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc5043b5b, SUC_lexicon_tags_15 },
    { 0x39c43b5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e43b61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86443b64, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1ce43b66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a43b69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45643b6b, SUC_lexicon_tags_74 },
    { 0x61043b6c, SUC_lexicon_tags_2 },
    { 0xb1e43b6c, SUC_lexicon_tags_48 },
    { 0x73c43b6e, SUC_lexicon_tags_46 },
    { 0xc9843b6f, SUC_lexicon_tags_46 },
    { 0x18843b6e, SUC_lexicon_tags_39 },
    { 0x62043b6e, SUC_lexicon_tags_43 },
    { 0x5f043b72, SUC_lexicon_tags_17 },
    { 0x50843b6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf843b76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x23443b78, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be43b7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75443b7f, SUC_lexicon_tags_19 },
    { 0x43b80, SUC_lexicon_tags_15 },
    { 0x22043b80, SUC_lexicon_tags_7 },
    { 0x23e43b82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87043b84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a43b89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc43b91, SUC_lexicon_tags_8 },
    { 0x94e43b92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83243b94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4443b9e, SUC_lexicon_tags_6 },
    { 0x5e243b9f, SUC_lexicon_tags_17 },
    { 0x80443ba0, SUC_lexicon_tags_11 },
    { 0xc3843b9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b443ba4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a43bae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe43bb2, SUC_lexicon_tags_11 },
    { 0xbb443bb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92643bb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2443bb7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe5643bb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1843bbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a43bc0, SUC_lexicon_tags_25 },
    { 0x1d843bc1, SUC_lexicon_tags_8 },
    { 0xc2443bc0, SUC_lexicon_tags_2 },
    { 0xe2a43bc3, SUC_lexicon_tags_8 },
    { 0x1f243bc1, SUC_lexicon_tags_33 },
    { 0x2a843bc4, SUC_lexicon_tags_7 },
    { 0x27e43bc3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12443bca, SUC_lexicon_tags_48 },
    { 0x49a43bcb, SUC_lexicon_tags_29 },
    { 0x9e043bcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0043bce, SUC_lexicon_tags_4 },
    { 0xa8e43bcf, SUC_lexicon_tags_37 },
    { 0x32843bd0, SUC_lexicon_tags_12 },
    { 0x15243bcf, SUC_lexicon_tags_27 },
    { 0x95043bd2, SUC_lexicon_tags_25 },
    { 0xae443bd1, SUC_lexicon_tags_11 },
    { 0x2a643bd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3243bd6, SUC_lexicon_tags_37 },
    { 0x2e43bd7, SUC_lexicon_tags_33 },
    { 0x4be43bd8, SUC_lexicon_tags_31 },
    { 0x34c43bd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b443bde, SUC_lexicon_tags_11 },
    { 0x40643bdf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc3e43be1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b443be7, SUC_lexicon_tags_11 },
    { 0xbda43be7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x21a43bea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45043bf1, SUC_lexicon_tags_11 },
    { 0xe7c43bf1, SUC_lexicon_tags_12 },
    { 0x47643bf2, SUC_lexicon_tags_33 },
    { 0xf8443bf4, SUC_lexicon_tags_37 },
    { 0x8e243bf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece43bfa, SUC_lexicon_tags_14 },
    { 0x61e43bfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a43c00, SUC_lexicon_tags_69 },
    { 0xce843c01, SUC_lexicon_tags_1 },
    { 0xba843c00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15643c07, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a43c0f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7243c16, SUC_lexicon_tags_62 },
    { 0x78c43c17, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c43c1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ae43c1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d643c1e, SUC_lexicon_tags_25 },
    { 0x35a43c1f, SUC_lexicon_tags_15 },
    { 0x5ca43c1f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8843c24, SUC_lexicon_tags_15 },
    { 0x41043c25, SUC_lexicon_tags_6 },
    { 0x87643c24, SUC_lexicon_tags_14 },
    { 0x6f443c24, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1843c2b, SUC_lexicon_tags_33 },
    { 0xa2643c2c, SUC_lexicon_tags_171 },
    { 0xef243c2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8fa43c2f, SUC_lexicon_tags_33 },
    { 0xe0243c2f, SUC_lexicon_tags_25 },
    { 0xdd043c31, SUC_lexicon_tags_28 },
    { 0x40843c32, SUC_lexicon_tags_423 },
    { 0xdbc43c33, SUC_lexicon_tags_15 },
    { 0x62e43c34, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb643c39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48443c3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87843c3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x64a43c3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe43c42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e43c45, SUC_lexicon_tags_25 },
    { 0x3b843c46, SUC_lexicon_tags_32 },
    { 0x2b843c47, SUC_lexicon_tags_14 },
    { 0xd1243c48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc43c4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4643c4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da43c53, SUC_lexicon_tags_29 },
    { 0x6ee43c54, SUC_lexicon_tags_14 },
    { 0xf1e43c53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f443c59, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e43c5c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x17843c5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83043c63, SUC_lexicon_tags_16 },
    { 0x92643c63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27043c66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43843c6a, SUC_lexicon_tags_17 },
    { 0x25243c6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c843c70, SUC_lexicon_tags_14 },
    { 0x8e643c71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38443c73, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22643c79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c43c7d, SUC_lexicon_tags_45 },
    { 0xe43c7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e43c83, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa4e43c85, SUC_lexicon_tags_16 },
    { 0x2de43c86, SUC_lexicon_tags_48 },
    { 0x95243c87, SUC_lexicon_tags_43 },
    { 0x6c843c87, SUC_lexicon_tags_43 },
    { 0x1ca43c87, SUC_lexicon_tags_63 },
    { 0x37c43c8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c43c90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52843c92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea43c95, SUC_lexicon_tags_12 },
    { 0x60043c96, SUC_lexicon_tags_7 },
    { 0x3c043c96, SUC_lexicon_tags_37 },
    { 0xb3043c98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6643c9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x16843c9c, SUC_lexicon_tags_17 },
    { 0x19e43c9c, SUC_lexicon_tags_63 },
    { 0xc643c9c, SUC_lexicon_tags_17 },
    { 0x50a43c9e, SUC_lexicon_tags_14 },
    { 0xbce43c9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5443ca4, SUC_lexicon_tags_45 },
    { 0xaf043ca4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97a43ca7, SUC_lexicon_tags_37 },
    { 0xd443ca7, SUC_lexicon_tags_38 },
    { 0x92043ca7, SUC_lexicon_tags_14 },
    { 0x1de43ca9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6e43cac, SUC_lexicon_tags_17 },
    { 0xd4243cad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43cb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeec43cb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d443cb5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x38243cb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e43cbd, SUC_lexicon_tags_37 },
    { 0x11643cbe, SUC_lexicon_tags_6 },
    { 0x1443cbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc43cc3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4a43cc5, SUC_lexicon_tags_15 },
    { 0x7de43cc6, SUC_lexicon_tags_506 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec43cca, SUC_lexicon_tags_60 },
    { 0x94a43cca, SUC_lexicon_tags_1 },
    { 0xa9843cca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f443cd0, SUC_lexicon_tags_11 },
    { 0x3f243cd1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4043cd3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x41043cd5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a043cd9, SUC_lexicon_tags_63 },
    { 0x22643cda, SUC_lexicon_tags_63 },
    { 0xe9c43cd9, SUC_lexicon_tags_41 },
    { 0x5e843cdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51643ce1, SUC_lexicon_tags_47 },
    { 0xec843ce2, SUC_lexicon_tags_38 },
    { 0x44243ce2, SUC_lexicon_tags_1 },
    { 0x74a43ce4, SUC_lexicon_tags_1 },
    { 0xcb043ce4, SUC_lexicon_tags_2 },
    { 0xbd043ce6, SUC_lexicon_tags_11 },
    { 0xf7a43ce7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2643cea, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xff043cec, SUC_lexicon_tags_2 },
    { 0x96843ced, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92443cef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x92a43cf1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e43cf4, SUC_lexicon_tags_32 },
    { 0x33643cf5, SUC_lexicon_tags_8 },
    { 0xf643cf5, SUC_lexicon_tags_12 },
    { 0xc0c43cf7, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50643cfe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda43d02, SUC_lexicon_tags_50 },
    { 0xe2e43d03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36443d05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1c43d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c43d13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9ee43d15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d043d18, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e43d1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93443d1f, SUC_lexicon_tags_1 },
    { 0xcfe43d20, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e43d24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc0e43d26, SUC_lexicon_tags_57 },
    { 0x9a443d27, SUC_lexicon_tags_25 },
    { 0x6a443d28, SUC_lexicon_tags_56 },
    { 0xeea43d28, SUC_lexicon_tags_38 },
    { 0xee443d2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443d2e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66243d37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58843d39, SUC_lexicon_tags_84 },
    { 0x7fe43d39, SUC_lexicon_tags_46 },
    { 0x19643d3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23643d3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab443d42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4443d49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e43d4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e243d53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c643d56, SUC_lexicon_tags_63 },
    { 0x6a043d56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3d043d59, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e43d5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46243d5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c43d64, SUC_lexicon_tags_14 },
    { 0xce843d64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d843d69, SUC_lexicon_tags_9 },
    { 0x2b443d6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6643d6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e243d72, SUC_lexicon_tags_1 },
    { 0xe1e43d73, SUC_lexicon_tags_17 },
    { 0xf9c43d74, SUC_lexicon_tags_2 },
    { 0x78843d75, SUC_lexicon_tags_38 },
    { 0x61843d72, SUC_lexicon_tags_37 },
    { 0x76a43d76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54043d7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0443d7e, SUC_lexicon_tags_11 },
    { 0x4cc43d7f, SUC_lexicon_tags_32 },
    { 0xd3843d80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8dc43d82, SUC_lexicon_tags_63 },
    { 0xbbe43d83, SUC_lexicon_tags_9 },
    { 0x16043d83, SUC_lexicon_tags_14 },
    { 0x9be43d83, SUC_lexicon_tags_2 },
    { 0x53e43d86, SUC_lexicon_tags_11 },
    { 0xdf443d86, SUC_lexicon_tags_45 },
    { 0x69a43d86, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc6643d8a, SUC_lexicon_tags_25 },
    { 0x24843d8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xba243d8e, SUC_lexicon_tags_29 },
    { 0xa43d8f, SUC_lexicon_tags_17 },
    { 0x14843d8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca443d95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a43d99, SUC_lexicon_tags_15 },
    { 0xbf043d9a, SUC_lexicon_tags_11 },
    { 0x69843d9a, SUC_lexicon_tags_27 },
    { 0x8b843d9b, SUC_lexicon_tags_12 },
    { 0x80c43d9d, SUC_lexicon_tags_187 },
    { 0xdd443d9e, SUC_lexicon_tags_10 },
    { 0x9de43d99, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xabe43da1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62e43da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3643da5, SUC_lexicon_tags_1 },
    { 0x63243da6, SUC_lexicon_tags_8 },
    { 0xbe843da7, SUC_lexicon_tags_53 },
    { 0xd1e43da7, SUC_lexicon_tags_7 },
    { 0x6ec43da5, SUC_lexicon_tags_7 },
    { 0xcd043daa, SUC_lexicon_tags_17 },
    { 0x2c443da8, SUC_lexicon_tags_11 },
    { 0xf2043dab, SUC_lexicon_tags_17 },
    { 0xc9243dac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe643db1, SUC_lexicon_tags_25 },
    { 0x3c243db2, SUC_lexicon_tags_25 },
    { 0xf0c43db3, SUC_lexicon_tags_63 },
    { 0x48243db1, SUC_lexicon_tags_0 },
    { 0xeea43db5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c443db9, SUC_lexicon_tags_15 },
    { 0x98c43dba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1243dbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43dc0, SUC_lexicon_tags_1 },
    { 0x7ca43dc1, SUC_lexicon_tags_15 },
    { 0x32643dc2, SUC_lexicon_tags_11 },
    { 0x46a43dc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1fe43dc5, SUC_lexicon_tags_10 },
    { 0x1e443dc6, SUC_lexicon_tags_66 },
    { 0x90243dc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e43dcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0443dcf, SUC_lexicon_tags_33 },
    { 0xe9a43dcf, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec43dd8, SUC_lexicon_tags_11 },
    { 0x20843dd9, SUC_lexicon_tags_25 },
    { 0x4e443dd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7043dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74443de0, SUC_lexicon_tags_11 },
    { 0x7c443de1, SUC_lexicon_tags_15 },
    { 0x62843de1, SUC_lexicon_tags_29 },
    { 0x6a043de2, SUC_lexicon_tags_92 },
    { 0x1243de2, SUC_lexicon_tags_12 },
    { 0x87843de3, SUC_lexicon_tags_12 },
    { 0x5fa43de6, SUC_lexicon_tags_26 },
    { 0xf6a43de2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da43dea, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc43ded, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c643df1, SUC_lexicon_tags_39 },
    { 0xc0a43df1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e43df8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a43e01, SUC_lexicon_tags_7 },
    { 0xb0043e01, SUC_lexicon_tags_49 },
    { 0x6b643e03, SUC_lexicon_tags_37 },
    { 0xc5643e04, SUC_lexicon_tags_27 },
    { 0x57e43e05, SUC_lexicon_tags_7 },
    { 0xf7843e05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e43e0b, SUC_lexicon_tags_8 },
    { 0xb7a43e0b, SUC_lexicon_tags_2 },
    { 0xf8443e0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2e43e0f, SUC_lexicon_tags_27 },
    { 0x6be43e0f, SUC_lexicon_tags_45 },
    { 0xa9a43e11, SUC_lexicon_tags_58 },
    { 0xaa443e12, SUC_lexicon_tags_63 },
    { 0x9c43e11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c43e17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3443e1a, SUC_lexicon_tags_8 },
    { 0xb0843e1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e643e1f, SUC_lexicon_tags_17 },
    { 0xa5443e20, SUC_lexicon_tags_30 },
    { 0x31243e21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39243e27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37243e2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c43e39, SUC_lexicon_tags_1 },
    { 0xc2a43e3a, SUC_lexicon_tags_12 },
    { 0x16043e3a, SUC_lexicon_tags_56 },
    { 0xf8a43e39, SUC_lexicon_tags_37 },
    { 0x51a43e3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae443e44, SUC_lexicon_tags_2 },
    { 0x7a43e45, SUC_lexicon_tags_917 },
    { 0, NULL },
    { 0xed643e47, SUC_lexicon_tags_14 },
    { 0xcf843e48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c843e4b, SUC_lexicon_tags_151 },
    { 0xc6243e4c, SUC_lexicon_tags_55 },
    { 0x95a43e4d, SUC_lexicon_tags_17 },
    { 0xaca43e4e, SUC_lexicon_tags_33 },
    { 0xab443e4f, SUC_lexicon_tags_77 },
    { 0x35a43e50, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x73643e52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a43e58, SUC_lexicon_tags_37 },
    { 0x86e43e59, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44843e5b, SUC_lexicon_tags_17 },
    { 0xfda43e5b, SUC_lexicon_tags_0 },
    { 0x6b243e5c, SUC_lexicon_tags_25 },
    { 0x9843e5e, SUC_lexicon_tags_37 },
    { 0x55843e5e, SUC_lexicon_tags_37 },
    { 0xaa643e5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x11643e63, SUC_lexicon_tags_31 },
    { 0xfba43e64, SUC_lexicon_tags_113 },
    { 0xab443e65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e43e68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d643e70, SUC_lexicon_tags_0 },
    { 0x25043e71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd043e74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe43e77, SUC_lexicon_tags_83 },
    { 0xae643e78, SUC_lexicon_tags_37 },
    { 0x49c43e77, SUC_lexicon_tags_60 },
    { 0xfd843e7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c43e7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e43e82, SUC_lexicon_tags_63 },
    { 0x5d843e82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8443e88, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be43e8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf243e8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee43e95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71043e9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfe43ea1, SUC_lexicon_tags_29 },
    { 0xe3c43ea2, SUC_lexicon_tags_29 },
    { 0xe3c43ea1, SUC_lexicon_tags_13 },
    { 0x8f043ea4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x77643ea7, SUC_lexicon_tags_12 },
    { 0x86043ea8, SUC_lexicon_tags_45 },
    { 0xf5a43ea8, SUC_lexicon_tags_17 },
    { 0xd0043eaa, SUC_lexicon_tags_1 },
    { 0xcf843eab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdba43ead, SUC_lexicon_tags_41 },
    { 0x68e43ead, SUC_lexicon_tags_37 },
    { 0x20443ead, SUC_lexicon_tags_18 },
    { 0x47843eb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe43eb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57a43eb6, SUC_lexicon_tags_28 },
    { 0x43443eb6, SUC_lexicon_tags_26 },
    { 0xa8643eb8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xafc43eba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e43ebd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8243ec3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdca43ec5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c43ecd, SUC_lexicon_tags_0 },
    { 0xf1c43ecd, SUC_lexicon_tags_23 },
    { 0x4f043ecd, SUC_lexicon_tags_28 },
    { 0xd3843ecd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd1a43ed2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42243ede, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51443ee3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf0443ee5, SUC_lexicon_tags_11 },
    { 0x76043ee6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ce43ee8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82243eed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30643ef1, SUC_lexicon_tags_25 },
    { 0xf2243ef1, SUC_lexicon_tags_2 },
    { 0xb2643ef3, SUC_lexicon_tags_9 },
    { 0x4f443ef4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63243ef7, SUC_lexicon_tags_38 },
    { 0xb1c43ef8, SUC_lexicon_tags_7 },
    { 0x7a243ef9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39843efb, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9243f03, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e43f07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a43f0a, SUC_lexicon_tags_27 },
    { 0x9ea43f0b, SUC_lexicon_tags_37 },
    { 0xc1043f0c, SUC_lexicon_tags_63 },
    { 0x36243f0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c43f11, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa43f16, SUC_lexicon_tags_30 },
    { 0x24843f16, SUC_lexicon_tags_25 },
    { 0xf9643f16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d643f1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x79043f22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68e43f24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c43f27, SUC_lexicon_tags_17 },
    { 0xeb843f28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1443f2e, SUC_lexicon_tags_37 },
    { 0xde643f2f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfd843f31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4643f35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b843f38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd843f3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e43f41, SUC_lexicon_tags_15 },
    { 0x99a43f42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee243f45, SUC_lexicon_tags_46 },
    { 0x30a43f46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15e43f48, SUC_lexicon_tags_15 },
    { 0x1e243f48, SUC_lexicon_tags_49 },
    { 0x70c43f49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95643f4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5443f54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f443f56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bc43f58, SUC_lexicon_tags_33 },
    { 0xbb643f59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9a643f5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a843f60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a43f67, SUC_lexicon_tags_7 },
    { 0xa6843f68, SUC_lexicon_tags_43 },
    { 0xddc43f68, SUC_lexicon_tags_65 },
    { 0x51043f6a, SUC_lexicon_tags_29 },
    { 0x78643f6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42043f70, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e43f74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5043f76, SUC_lexicon_tags_27 },
    { 0x59043f76, SUC_lexicon_tags_7 },
    { 0xd2443f77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6243f7a, SUC_lexicon_tags_11 },
    { 0xc043f7a, SUC_lexicon_tags_17 },
    { 0xf6e43f7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26843f7f, SUC_lexicon_tags_63 },
    { 0x7de43f7f, SUC_lexicon_tags_2 },
    { 0xce443f81, SUC_lexicon_tags_7 },
    { 0xd7e43f7f, SUC_lexicon_tags_45 },
    { 0xff443f83, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae43f86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8643f88, SUC_lexicon_tags_12 },
    { 0x6fe43f89, SUC_lexicon_tags_56 },
    { 0xb5243f8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb843f8e, SUC_lexicon_tags_0 },
    { 0xce243f8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c43f92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d243f96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1043f98, SUC_lexicon_tags_25 },
    { 0x54a43f98, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb3243f9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4843f9e, SUC_lexicon_tags_4 },
    { 0x20c43f9f, SUC_lexicon_tags_48 },
    { 0x67843f9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65243fa3, SUC_lexicon_tags_72 },
    { 0xeb443fa3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c043fa6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb243fb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0443fb3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc2243fb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9643fbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a43fc3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x16e43fc5, SUC_lexicon_tags_56 },
    { 0xea43fc6, SUC_lexicon_tags_2 },
    { 0xe8243fc5, SUC_lexicon_tags_10 },
    { 0x92243fc7, SUC_lexicon_tags_9 },
    { 0xad843fc6, SUC_lexicon_tags_11 },
    { 0x53243fca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x18e43fcc, SUC_lexicon_tags_30 },
    { 0x84643fcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1043fd0, SUC_lexicon_tags_4 },
    { 0x44043fd0, SUC_lexicon_tags_23 },
    { 0x71a43fd0, SUC_lexicon_tags_7 },
    { 0xa5c43fd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc43fd6, SUC_lexicon_tags_11 },
    { 0x78c43fd6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9843fdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe243fe4, SUC_lexicon_tags_29 },
    { 0x3c243fe5, SUC_lexicon_tags_28 },
    { 0x61843fe6, SUC_lexicon_tags_17 },
    { 0xf7043fe7, SUC_lexicon_tags_19 },
    { 0x1d843fe8, SUC_lexicon_tags_15 },
    { 0x86043fe9, SUC_lexicon_tags_27 },
    { 0xfb843fe8, SUC_lexicon_tags_37 },
    { 0x63c43feb, SUC_lexicon_tags_55 },
    { 0x70643fec, SUC_lexicon_tags_7 },
    { 0x53043fea, SUC_lexicon_tags_11 },
    { 0xf0e43fe7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea43ff6, SUC_lexicon_tags_47 },
    { 0x39243ff7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3a43ff9, SUC_lexicon_tags_43 },
    { 0x4ae43ffa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1443ffc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92843ffe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d844003, SUC_lexicon_tags_10 },
    { 0x6c44004, SUC_lexicon_tags_43 },
    { 0x2844003, SUC_lexicon_tags_4 },
    { 0x28044005, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e444008, SUC_lexicon_tags_42 },
    { 0xbd044009, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa524400e, SUC_lexicon_tags_29 },
    { 0x1e44400e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca844011, SUC_lexicon_tags_19 },
    { 0x8da44011, SUC_lexicon_tags_11 },
    { 0x60244013, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb844015, SUC_lexicon_tags_65 },
    { 0xdcc44016, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc4401c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2204401e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6244021, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8044027, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5a44029, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8384402c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca44031, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18044034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50044039, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2e04403b, SUC_lexicon_tags_45 },
    { 0xa344403c, SUC_lexicon_tags_15 },
    { 0x4e84403d, SUC_lexicon_tags_25 },
    { 0xa684403e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a044041, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c44045, SUC_lexicon_tags_1 },
    { 0xc3444045, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3844050, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45844052, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30844056, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fe44058, SUC_lexicon_tags_27 },
    { 0xe7c44059, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3424405d, SUC_lexicon_tags_7 },
    { 0x6f04405d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f644060, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8244062, SUC_lexicon_tags_17 },
    { 0x3ac44062, SUC_lexicon_tags_37 },
    { 0x60444064, SUC_lexicon_tags_31 },
    { 0xd5444065, SUC_lexicon_tags_46 },
    { 0xab244066, SUC_lexicon_tags_35 },
    { 0xb8444065, SUC_lexicon_tags_4 },
    { 0xffc44068, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5ba4406a, SUC_lexicon_tags_1 },
    { 0x4744406b, SUC_lexicon_tags_11 },
    { 0x94a4406b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe44071, SUC_lexicon_tags_2 },
    { 0xcc44072, SUC_lexicon_tags_27 },
    { 0xaaa44073, SUC_lexicon_tags_17 },
    { 0x98844073, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xc9a44076, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed64407f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7844086, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c4408b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c44090, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10644092, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1c44094, SUC_lexicon_tags_29 },
    { 0x9e444095, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f24409a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e440a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xed0440a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb8440a4, SUC_lexicon_tags_11 },
    { 0xc06440a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b0440a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88440ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe440b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24440b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c440b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c440bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328440c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x874440c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x55a440c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45a440c8, SUC_lexicon_tags_47 },
    { 0x6fa440c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x358440cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322440dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x198440de, SUC_lexicon_tags_0 },
    { 0x590440df, SUC_lexicon_tags_38 },
    { 0x7d4440df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6440e3, SUC_lexicon_tags_38 },
    { 0x618440e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x246440e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a440eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e440ef, SUC_lexicon_tags_12 },
    { 0x448440f0, SUC_lexicon_tags_11 },
    { 0x38440f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9d6440f3, SUC_lexicon_tags_19 },
    { 0x1da440f4, SUC_lexicon_tags_7 },
    { 0x68e440f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa440f9, SUC_lexicon_tags_17 },
    { 0x404440fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc440fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xff044100, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43244106, SUC_lexicon_tags_65 },
    { 0x7b244107, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x43044109, SUC_lexicon_tags_25 },
    { 0x4f244109, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4764410c, SUC_lexicon_tags_12 },
    { 0xe864410d, SUC_lexicon_tags_37 },
    { 0xd84410e, SUC_lexicon_tags_37 },
    { 0x9824410d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c44112, SUC_lexicon_tags_60 },
    { 0x20244112, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84e44115, SUC_lexicon_tags_0 },
    { 0x5a844116, SUC_lexicon_tags_37 },
    { 0x50c44117, SUC_lexicon_tags_63 },
    { 0xb6644118, SUC_lexicon_tags_77 },
    { 0x18044116, SUC_lexicon_tags_11 },
    { 0x1644411a, SUC_lexicon_tags_55 },
    { 0x41c4411b, SUC_lexicon_tags_87 },
    { 0xa424411b, SUC_lexicon_tags_16 },
    { 0x3d84411d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8084411f, SUC_lexicon_tags_77 },
    { 0xfc444120, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7dc44122, SUC_lexicon_tags_33 },
    { 0xf1e44123, SUC_lexicon_tags_43 },
    { 0xdc044123, SUC_lexicon_tags_25 },
    { 0x84244125, SUC_lexicon_tags_43 },
    { 0xa5644126, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4e44128, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x8084412b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6884412e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9644131, SUC_lexicon_tags_7 },
    { 0x6e644132, SUC_lexicon_tags_7 },
    { 0x2ba44133, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7424413d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34c4413f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5044143, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x18244145, SUC_lexicon_tags_25 },
    { 0x9a844146, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb44414b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c44414f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a44158, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c4415c, SUC_lexicon_tags_3 },
    { 0x2fe4415c, SUC_lexicon_tags_9 },
    { 0x3f04415e, SUC_lexicon_tags_18 },
    { 0x1d04415e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0444162, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x82244164, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x30044166, SUC_lexicon_tags_12 },
    { 0xac044166, SUC_lexicon_tags_49 },
    { 0x28c44168, SUC_lexicon_tags_47 },
    { 0xca244169, SUC_lexicon_tags_25 },
    { 0x8c044167, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfd64416c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30844170, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98444173, SUC_lexicon_tags_63 },
    { 0x2c044174, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde64417f, SUC_lexicon_tags_25 },
    { 0xae644180, SUC_lexicon_tags_11 },
    { 0xa9c44180, SUC_lexicon_tags_15 },
    { 0x58e44181, SUC_lexicon_tags_60 },
    { 0xe4c44183, SUC_lexicon_tags_11 },
    { 0x28c44182, SUC_lexicon_tags_15 },
    { 0xaec44185, SUC_lexicon_tags_37 },
    { 0x31244180, SUC_lexicon_tags_63 },
    { 0xa44186, SUC_lexicon_tags_6 },
    { 0x5a044180, SUC_lexicon_tags_11 },
    { 0xf5a44186, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fe4418b, SUC_lexicon_tags_12 },
    { 0x4984418c, SUC_lexicon_tags_45 },
    { 0x4fe4418d, SUC_lexicon_tags_31 },
    { 0xa9e4418d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e44195, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4441aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932441b1, SUC_lexicon_tags_42 },
    { 0xdee441b2, SUC_lexicon_tags_14 },
    { 0xa3a441b2, SUC_lexicon_tags_7 },
    { 0x4c2441b4, SUC_lexicon_tags_23 },
    { 0x7a4441b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8441b8, SUC_lexicon_tags_38 },
    { 0x1ee441b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb50441bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2441c1, SUC_lexicon_tags_25 },
    { 0xfc441c2, SUC_lexicon_tags_37 },
    { 0xa9c441c3, SUC_lexicon_tags_15 },
    { 0x6c441c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6441c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x448441ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a441cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75e441cf, SUC_lexicon_tags_23 },
    { 0x4a6441cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba441d9, SUC_lexicon_tags_3 },
    { 0xd64441da, SUC_lexicon_tags_37 },
    { 0x8c8441d9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb4c441dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x142441df, SUC_lexicon_tags_7 },
    { 0x15a441e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x378441e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222441e7, SUC_lexicon_tags_16 },
    { 0xea4441e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3bc441ea, SUC_lexicon_tags_30 },
    { 0xf60441ea, SUC_lexicon_tags_11 },
    { 0xe8441eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00441f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6441f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6441f6, SUC_lexicon_tags_91 },
    { 0x37e441f7, SUC_lexicon_tags_12 },
    { 0xf94441f8, SUC_lexicon_tags_30 },
    { 0xba6441f9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0441fc, SUC_lexicon_tags_25 },
    { 0xf82441fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d244208, SUC_lexicon_tags_127 },
    { 0xb5a44209, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37e4420b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50c4420d, SUC_lexicon_tags_7 },
    { 0x8fe4420e, SUC_lexicon_tags_25 },
    { 0xb444420d, SUC_lexicon_tags_12 },
    { 0x54444210, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8844212, SUC_lexicon_tags_21 },
    { 0xea844213, SUC_lexicon_tags_55 },
    { 0xef444214, SUC_lexicon_tags_29 },
    { 0xd4844214, SUC_lexicon_tags_33 },
    { 0x95a44215, SUC_lexicon_tags_23 },
    { 0x79444215, SUC_lexicon_tags_2 },
    { 0x10844215, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a4421b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x91a4421d, SUC_lexicon_tags_33 },
    { 0xe164421d, SUC_lexicon_tags_47 },
    { 0xdc44421f, SUC_lexicon_tags_0 },
    { 0xe6244220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39844227, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc284422e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec44232, SUC_lexicon_tags_1 },
    { 0x6dc44233, SUC_lexicon_tags_22 },
    { 0x7e244232, SUC_lexicon_tags_7 },
    { 0xc6844235, SUC_lexicon_tags_31 },
    { 0xad644236, SUC_lexicon_tags_14 },
    { 0xd2844237, SUC_lexicon_tags_87 },
    { 0xc4a44235, SUC_lexicon_tags_31 },
    { 0x28444238, SUC_lexicon_tags_15 },
    { 0x6664423a, SUC_lexicon_tags_38 },
    { 0x15a44237, SUC_lexicon_tags_6 },
    { 0xd24423c, SUC_lexicon_tags_1 },
    { 0xc524423d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fe4423f, SUC_lexicon_tags_2 },
    { 0x2a844240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5cc44242, SUC_lexicon_tags_37 },
    { 0x3dc44243, SUC_lexicon_tags_25 },
    { 0x36844244, SUC_lexicon_tags_17 },
    { 0x9ee44245, SUC_lexicon_tags_8 },
    { 0x1fc44244, SUC_lexicon_tags_105 },
    { 0xa9244247, SUC_lexicon_tags_29 },
    { 0x95244248, SUC_lexicon_tags_14 },
    { 0x29644248, SUC_lexicon_tags_49 },
    { 0x4e044248, SUC_lexicon_tags_32 },
    { 0xfb444244, SUC_lexicon_tags_42 },
    { 0xfc444243, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f64424f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea44251, SUC_lexicon_tags_37 },
    { 0x37244252, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x47e44254, SUC_lexicon_tags_14 },
    { 0x61444254, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0x60444257, SUC_lexicon_tags_7 },
    { 0xefe44258, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9044425b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae844261, SUC_lexicon_tags_19 },
    { 0x14a44261, SUC_lexicon_tags_10 },
    { 0xf0a44261, SUC_lexicon_tags_11 },
    { 0xa44264, SUC_lexicon_tags_15 },
    { 0x90844262, SUC_lexicon_tags_45 },
    { 0x63e44266, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde4426a, SUC_lexicon_tags_31 },
    { 0xf9e4426a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9b84426d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8724426f, SUC_lexicon_tags_7 },
    { 0xd6a4426f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c244273, SUC_lexicon_tags_29 },
    { 0xd9644274, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a44278, SUC_lexicon_tags_23 },
    { 0xb1644279, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e4427d, SUC_lexicon_tags_17 },
    { 0x7d04427d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d644280, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x72644282, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1a244284, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5444287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee444289, SUC_lexicon_tags_2 },
    { 0xe6e4428a, SUC_lexicon_tags_6 },
    { 0x8724428b, SUC_lexicon_tags_38 },
    { 0x7744428a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf644428f, SUC_lexicon_tags_30 },
    { 0xfdc44290, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae44293, SUC_lexicon_tags_37 },
    { 0x47844294, SUC_lexicon_tags_63 },
    { 0x30644294, SUC_lexicon_tags_43 },
    { 0x9a444296, SUC_lexicon_tags_46 },
    { 0x67844294, SUC_lexicon_tags_7 },
    { 0xe044298, SUC_lexicon_tags_12 },
    { 0xd5c44298, SUC_lexicon_tags_33 },
    { 0xdac44297, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7524429f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa9e442a1, SUC_lexicon_tags_1 },
    { 0xce4442a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24a442a4, SUC_lexicon_tags_17 },
    { 0x58e442a5, SUC_lexicon_tags_27 },
    { 0x4e2442a5, SUC_lexicon_tags_15 },
    { 0x6a442a7, SUC_lexicon_tags_19 },
    { 0x932442a4, SUC_lexicon_tags_13 },
    { 0xaf8442a9, SUC_lexicon_tags_4 },
    { 0xc40442a9, SUC_lexicon_tags_0 },
    { 0x4be442ab, SUC_lexicon_tags_10 },
    { 0x190442a5, SUC_lexicon_tags_12 },
    { 0xa02442ad, SUC_lexicon_tags_297 },
    { 0x334442ae, SUC_lexicon_tags_37 },
    { 0xf06442a4, SUC_lexicon_tags_27 },
    { 0xef8442b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40442b4, SUC_lexicon_tags_1300 },
    { 0, NULL },
    { 0x8ea442b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc442b9, SUC_lexicon_tags_30 },
    { 0x884442ba, SUC_lexicon_tags_8 },
    { 0x8d4442bb, SUC_lexicon_tags_7 },
    { 0xed8442bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36e442be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x66442c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238442ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4442cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba442d4, SUC_lexicon_tags_41 },
    { 0x80a442d5, SUC_lexicon_tags_64 },
    { 0x960442d5, SUC_lexicon_tags_11 },
    { 0x2a8442d7, SUC_lexicon_tags_26 },
    { 0x744442d7, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x7ba442da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88442dd, SUC_lexicon_tags_37 },
    { 0x70c442de, SUC_lexicon_tags_23 },
    { 0x5ca442dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890442e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec442e9, SUC_lexicon_tags_7 },
    { 0xe9c442ea, SUC_lexicon_tags_39 },
    { 0xddc442eb, SUC_lexicon_tags_11 },
    { 0xbb4442ec, SUC_lexicon_tags_0 },
    { 0xda2442ed, SUC_lexicon_tags_15 },
    { 0xab0442ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e4442f0, SUC_lexicon_tags_28 },
    { 0xe50442f1, SUC_lexicon_tags_7 },
    { 0xd28442f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a442f9, SUC_lexicon_tags_12 },
    { 0xaae442fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e442fe, SUC_lexicon_tags_45 },
    { 0xaf0442ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbe44301, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x30844303, SUC_lexicon_tags_11 },
    { 0x14444304, SUC_lexicon_tags_28 },
    { 0x4f444305, SUC_lexicon_tags_1 },
    { 0xc0644305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45044308, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4764430d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8044310, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be44315, SUC_lexicon_tags_3 },
    { 0x7a444316, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x94044318, SUC_lexicon_tags_7 },
    { 0x54444319, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9624431b, SUC_lexicon_tags_37 },
    { 0xede4431b, SUC_lexicon_tags_1 },
    { 0xb124431d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8644324, SUC_lexicon_tags_27 },
    { 0xca44325, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc44328, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb964432b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c4432e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7844333, SUC_lexicon_tags_15 },
    { 0x3a444334, SUC_lexicon_tags_31 },
    { 0xe1244335, SUC_lexicon_tags_27 },
    { 0xd6844336, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x20844338, SUC_lexicon_tags_7 },
    { 0x1ba44339, SUC_lexicon_tags_37 },
    { 0xb724433a, SUC_lexicon_tags_19 },
    { 0xda24433a, SUC_lexicon_tags_2 },
    { 0xfcc4433c, SUC_lexicon_tags_0 },
    { 0xed044339, SUC_lexicon_tags_25 },
    { 0xe6c4433e, SUC_lexicon_tags_60 },
    { 0x25c4433e, SUC_lexicon_tags_27 },
    { 0x644340, SUC_lexicon_tags_15 },
    { 0x13a4433e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73444345, SUC_lexicon_tags_92 },
    { 0x22a44345, SUC_lexicon_tags_2 },
    { 0x64e44345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c4434b, SUC_lexicon_tags_49 },
    { 0x3ce4434c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b244350, SUC_lexicon_tags_27 },
    { 0xb6c44351, SUC_lexicon_tags_77 },
    { 0x22a44351, SUC_lexicon_tags_7 },
    { 0x43444353, SUC_lexicon_tags_12 },
    { 0xe6044354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca44359, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc84435e, SUC_lexicon_tags_32 },
    { 0x56e4435e, SUC_lexicon_tags_25 },
    { 0x4584435f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b644363, SUC_lexicon_tags_87 },
    { 0x49c44364, SUC_lexicon_tags_25 },
    { 0xad444365, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a44369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a4436d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40444372, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba644376, SUC_lexicon_tags_295 },
    { 0xb2e44377, SUC_lexicon_tags_37 },
    { 0x9a44377, SUC_lexicon_tags_37 },
    { 0x75a44377, SUC_lexicon_tags_63 },
    { 0x1664437a, SUC_lexicon_tags_29 },
    { 0x6584437b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf464437e, SUC_lexicon_tags_33 },
    { 0x4924437f, SUC_lexicon_tags_47 },
    { 0x2284437f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7644385, SUC_lexicon_tags_49 },
    { 0x64e44386, SUC_lexicon_tags_1 },
    { 0xa3c44386, SUC_lexicon_tags_27 },
    { 0x80244387, SUC_lexicon_tags_60 },
    { 0x49844389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe784438e, SUC_lexicon_tags_141 },
    { 0x6744438e, SUC_lexicon_tags_55 },
    { 0xa2a44390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc44398, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2504439a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472443a0, SUC_lexicon_tags_0 },
    { 0xf56443a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c443a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe20443a6, SUC_lexicon_tags_47 },
    { 0x278443a7, SUC_lexicon_tags_2 },
    { 0xbe6443a8, SUC_lexicon_tags_25 },
    { 0xef6443a9, SUC_lexicon_tags_12 },
    { 0xef2443aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c6443ac, SUC_lexicon_tags_23 },
    { 0x142443ac, SUC_lexicon_tags_10 },
    { 0x2e443ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208443b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660443ba, SUC_lexicon_tags_30 },
    { 0x124443ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x63c443bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc443c3, SUC_lexicon_tags_25 },
    { 0xe00443c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3a443c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6443cd, SUC_lexicon_tags_13 },
    { 0x422443cd, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xfd8443d0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf34443d2, SUC_lexicon_tags_47 },
    { 0x2ee443d2, SUC_lexicon_tags_69 },
    { 0x74c443d4, SUC_lexicon_tags_17 },
    { 0xed2443d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472443db, SUC_lexicon_tags_1 },
    { 0x6d6443db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a443df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba443e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4443e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x856443ea, SUC_lexicon_tags_15 },
    { 0xfc4443eb, SUC_lexicon_tags_38 },
    { 0x53c443ec, SUC_lexicon_tags_7 },
    { 0x94e443eb, SUC_lexicon_tags_4 },
    { 0x72c443ee, SUC_lexicon_tags_45 },
    { 0x11c443ef, SUC_lexicon_tags_43 },
    { 0x7ca443f0, SUC_lexicon_tags_35 },
    { 0x2fa443f1, SUC_lexicon_tags_8 },
    { 0xda8443ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2443f5, SUC_lexicon_tags_17 },
    { 0xf4e443f6, SUC_lexicon_tags_26 },
    { 0xc1e443f7, SUC_lexicon_tags_12 },
    { 0x19a443f7, SUC_lexicon_tags_17 },
    { 0x450443f8, SUC_lexicon_tags_21 },
    { 0xcac443f5, SUC_lexicon_tags_1 },
    { 0xb68443f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4443ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9044401, SUC_lexicon_tags_229 },
    { 0x56244402, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47444404, SUC_lexicon_tags_1 },
    { 0x16a44405, SUC_lexicon_tags_1 },
    { 0xea044406, SUC_lexicon_tags_14 },
    { 0x35244406, SUC_lexicon_tags_37 },
    { 0x42e44406, SUC_lexicon_tags_8 },
    { 0xff644409, SUC_lexicon_tags_2 },
    { 0xc8844404, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x41e4440c, SUC_lexicon_tags_12 },
    { 0x3004440d, SUC_lexicon_tags_1 },
    { 0x67c4440d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa044410, SUC_lexicon_tags_33 },
    { 0xcfc44411, SUC_lexicon_tags_61 },
    { 0x8e244411, SUC_lexicon_tags_57 },
    { 0x76e44411, SUC_lexicon_tags_70 },
    { 0xb8244414, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27444419, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf24441c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7044424, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f044427, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xab244429, SUC_lexicon_tags_2 },
    { 0x66c4442a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd244433, SUC_lexicon_tags_1112 },
    { 0x7c644434, SUC_lexicon_tags_0 },
    { 0x1da44435, SUC_lexicon_tags_61 },
    { 0x47c44436, SUC_lexicon_tags_0 },
    { 0xb0044437, SUC_lexicon_tags_32 },
    { 0x28244435, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa044443b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc4443f, SUC_lexicon_tags_45 },
    { 0x58644440, SUC_lexicon_tags_35 },
    { 0xdba4443f, SUC_lexicon_tags_10 },
    { 0x8b044441, SUC_lexicon_tags_11 },
    { 0x97044442, SUC_lexicon_tags_45 },
    { 0x8f044444, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e244449, SUC_lexicon_tags_15 },
    { 0x75c4444a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c4444e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2844451, SUC_lexicon_tags_25 },
    { 0xafa44452, SUC_lexicon_tags_47 },
    { 0x60c44453, SUC_lexicon_tags_19 },
    { 0x8244454, SUC_lexicon_tags_1 },
    { 0xf8e44451, SUC_lexicon_tags_12 },
    { 0x36244453, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3824445a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf84445c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a444462, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x56844465, SUC_lexicon_tags_6 },
    { 0x26e44466, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd4a44468, SUC_lexicon_tags_60 },
    { 0x7e644469, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd244471, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfaa44473, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x55e44475, SUC_lexicon_tags_10 },
    { 0x73e44475, SUC_lexicon_tags_7 },
    { 0x2f444477, SUC_lexicon_tags_45 },
    { 0xbbc44475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d44447c, SUC_lexicon_tags_8 },
    { 0x7924447d, SUC_lexicon_tags_43 },
    { 0xbd64447e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d244483, SUC_lexicon_tags_17 },
    { 0x12844484, SUC_lexicon_tags_11 },
    { 0x61c44485, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xada44487, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c4448e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21644498, SUC_lexicon_tags_0 },
    { 0xcb044498, SUC_lexicon_tags_12 },
    { 0xd884449a, SUC_lexicon_tags_12 },
    { 0xcbe44499, SUC_lexicon_tags_45 },
    { 0x1d44449c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8124449e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd8444a0, SUC_lexicon_tags_1 },
    { 0xd7a444a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x124444a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a444a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c444ad, SUC_lexicon_tags_35 },
    { 0xe96444ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x496444b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd8a444b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca444b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a444bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x636444bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe20444c1, SUC_lexicon_tags_37 },
    { 0x930444c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8444c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e444cc, SUC_lexicon_tags_25 },
    { 0x81a444cd, SUC_lexicon_tags_25 },
    { 0xfd8444ce, SUC_lexicon_tags_66 },
    { 0xd24444ce, SUC_lexicon_tags_37 },
    { 0xc6444d0, SUC_lexicon_tags_38 },
    { 0x9ac444d1, SUC_lexicon_tags_12 },
    { 0x3e444d2, SUC_lexicon_tags_9 },
    { 0x94c444d3, SUC_lexicon_tags_29 },
    { 0x12444d3, SUC_lexicon_tags_17 },
    { 0xee0444d5, SUC_lexicon_tags_37 },
    { 0xd20444d6, SUC_lexicon_tags_11 },
    { 0x340444d7, SUC_lexicon_tags_12 },
    { 0xf12444d4, SUC_lexicon_tags_15 },
    { 0x11a444d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a444df, SUC_lexicon_tags_17 },
    { 0xffe444e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x61c444e2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4444e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea444eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02444f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c6444f2, SUC_lexicon_tags_63 },
    { 0x8a6444f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72444f7, SUC_lexicon_tags_29 },
    { 0xbd2444f8, SUC_lexicon_tags_25 },
    { 0x41e444f9, SUC_lexicon_tags_2 },
    { 0xe5e444f8, SUC_lexicon_tags_1 },
    { 0x6e6444f7, SUC_lexicon_tags_11 },
    { 0x1a0444fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92444ff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x57844501, SUC_lexicon_tags_17 },
    { 0xa2e44502, SUC_lexicon_tags_22 },
    { 0x72844501, SUC_lexicon_tags_38 },
    { 0x1ba44504, SUC_lexicon_tags_63 },
    { 0xe5644505, SUC_lexicon_tags_13 },
    { 0xe3444504, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44450d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1244511, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44515, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee244517, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a24451b, SUC_lexicon_tags_15 },
    { 0xad84451c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe184451e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x78844520, SUC_lexicon_tags_45 },
    { 0x35c44520, SUC_lexicon_tags_66 },
    { 0xd5a44522, SUC_lexicon_tags_11 },
    { 0xd5044523, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x48044525, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e44528, SUC_lexicon_tags_27 },
    { 0x2f244529, SUC_lexicon_tags_17 },
    { 0x3f044529, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7264452c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2884452f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb844532, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd244534, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c44539, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4453c, SUC_lexicon_tags_23 },
    { 0xca44453d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d244547, SUC_lexicon_tags_15 },
    { 0x9de44548, SUC_lexicon_tags_0 },
    { 0x7c044548, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70244555, SUC_lexicon_tags_87 },
    { 0x1a844556, SUC_lexicon_tags_11 },
    { 0xb6c44556, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c4455c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3264455f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41844563, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0844567, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd524456a, SUC_lexicon_tags_38 },
    { 0xedc4456b, SUC_lexicon_tags_0 },
    { 0x184456c, SUC_lexicon_tags_37 },
    { 0x74a4456c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x34a4456f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82644576, SUC_lexicon_tags_7 },
    { 0x17e44577, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa704457c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab04457e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbb444580, SUC_lexicon_tags_17 },
    { 0x53844581, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20044583, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e244586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc44589, SUC_lexicon_tags_31 },
    { 0x31644589, SUC_lexicon_tags_17 },
    { 0xe924458b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed444599, SUC_lexicon_tags_17 },
    { 0xce24459a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec4459d, SUC_lexicon_tags_8 },
    { 0x7cc4459e, SUC_lexicon_tags_11 },
    { 0xaf44459f, SUC_lexicon_tags_8 },
    { 0x41c4459f, SUC_lexicon_tags_7 },
    { 0x210445a1, SUC_lexicon_tags_7 },
    { 0xa0c445a2, SUC_lexicon_tags_38 },
    { 0xeac445a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb0445a5, SUC_lexicon_tags_25 },
    { 0x8e6445a5, SUC_lexicon_tags_25 },
    { 0x886445a7, SUC_lexicon_tags_65 },
    { 0xc00445a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186445ae, SUC_lexicon_tags_15 },
    { 0x82c445af, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf72445b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24445b3, SUC_lexicon_tags_16 },
    { 0xc3c445b4, SUC_lexicon_tags_47 },
    { 0xbf8445b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x250445b8, SUC_lexicon_tags_172 },
    { 0xb1c445b9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x248445bb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x66e445bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x294445bf, SUC_lexicon_tags_87 },
    { 0x4ca445c0, SUC_lexicon_tags_7 },
    { 0xb62445c1, SUC_lexicon_tags_7 },
    { 0x5ce445c0, SUC_lexicon_tags_34 },
    { 0x63e445c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e445c6, SUC_lexicon_tags_7 },
    { 0x6f8445c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa0e445c9, SUC_lexicon_tags_48 },
    { 0xefe445c9, SUC_lexicon_tags_1 },
    { 0x914445c9, SUC_lexicon_tags_25 },
    { 0x664445ca, SUC_lexicon_tags_28 },
    { 0x482445cd, SUC_lexicon_tags_53 },
    { 0x610445ce, SUC_lexicon_tags_37 },
    { 0x5f2445cd, SUC_lexicon_tags_17 },
    { 0xdf4445d0, SUC_lexicon_tags_27 },
    { 0xdc445d1, SUC_lexicon_tags_38 },
    { 0x454445cf, SUC_lexicon_tags_25 },
    { 0x826445ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a445d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348445df, SUC_lexicon_tags_12 },
    { 0x5b4445e0, SUC_lexicon_tags_17 },
    { 0xf18445e1, SUC_lexicon_tags_26 },
    { 0x1fa445e1, SUC_lexicon_tags_17 },
    { 0xcae445e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd54445e5, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x466445e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd2445ea, SUC_lexicon_tags_23 },
    { 0xe7c445eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c445ef, SUC_lexicon_tags_8 },
    { 0x778445f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c445f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4445f9, SUC_lexicon_tags_29 },
    { 0xf84445fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec44601, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4244605, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e44460a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a44615, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4a644617, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e44619, SUC_lexicon_tags_108 },
    { 0x18c4461a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c44461c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2544461e, SUC_lexicon_tags_33 },
    { 0x3e04461f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e844622, SUC_lexicon_tags_15 },
    { 0xb6844623, SUC_lexicon_tags_25 },
    { 0xb1644623, SUC_lexicon_tags_15 },
    { 0xac244622, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e44628, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x30e4462a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60a4462c, SUC_lexicon_tags_37 },
    { 0xa184462d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7204462f, SUC_lexicon_tags_25 },
    { 0xa04462f, SUC_lexicon_tags_39 },
    { 0xb5a44631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc644633, SUC_lexicon_tags_49 },
    { 0xb0444634, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c44637, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9944463d, SUC_lexicon_tags_60 },
    { 0x3664463e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa44643, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf244645, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe644649, SUC_lexicon_tags_25 },
    { 0x8024464a, SUC_lexicon_tags_0 },
    { 0xc424464b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf84464d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0244650, SUC_lexicon_tags_33 },
    { 0x96e44650, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c44654, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45444659, SUC_lexicon_tags_31 },
    { 0xe7e4465a, SUC_lexicon_tags_17 },
    { 0xaa644659, SUC_lexicon_tags_22 },
    { 0xec24465a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4044465e, SUC_lexicon_tags_63 },
    { 0x5e84465f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3c244661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e44666, SUC_lexicon_tags_37 },
    { 0x1244667, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1464466a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5104466d, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a44671, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc44674, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb444678, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x964467a, SUC_lexicon_tags_26 },
    { 0xd2e4467b, SUC_lexicon_tags_2 },
    { 0xfb84467c, SUC_lexicon_tags_49 },
    { 0xa4467c, SUC_lexicon_tags_236 },
    { 0x76c4467e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c44682, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1644686, SUC_lexicon_tags_10 },
    { 0x33a44686, SUC_lexicon_tags_37 },
    { 0x3c844688, SUC_lexicon_tags_54 },
    { 0xce444688, SUC_lexicon_tags_14 },
    { 0x61244689, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be4468d, SUC_lexicon_tags_19 },
    { 0x2284468e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x72644690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21c44692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99244695, SUC_lexicon_tags_31 },
    { 0x1ca44696, SUC_lexicon_tags_16 },
    { 0x1aa44696, SUC_lexicon_tags_11 },
    { 0x47c44698, SUC_lexicon_tags_14 },
    { 0xbc244696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac4469e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c446a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a446a7, SUC_lexicon_tags_8 },
    { 0xbb8446a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a446ad, SUC_lexicon_tags_11 },
    { 0x1b2446ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c446b1, SUC_lexicon_tags_37 },
    { 0x81c446b2, SUC_lexicon_tags_37 },
    { 0x61c446b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158446bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6446c2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xece446c4, SUC_lexicon_tags_2 },
    { 0x492446c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x574446c7, SUC_lexicon_tags_23 },
    { 0xeee446c7, SUC_lexicon_tags_12 },
    { 0x686446c7, SUC_lexicon_tags_1 },
    { 0xebc446c9, SUC_lexicon_tags_5 },
    { 0xbc0446c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa446d5, SUC_lexicon_tags_47 },
    { 0x6446d6, SUC_lexicon_tags_64 },
    { 0xc76446d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72446db, SUC_lexicon_tags_25 },
    { 0x946446dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x342446de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x54446e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x710446e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78446e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1e446e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf32446ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8446ed, SUC_lexicon_tags_17 },
    { 0x436446ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4446f1, SUC_lexicon_tags_3 },
    { 0xa80446f2, SUC_lexicon_tags_0 },
    { 0x922446f3, SUC_lexicon_tags_33 },
    { 0x42c446f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x282446f7, SUC_lexicon_tags_43 },
    { 0x7e0446f8, SUC_lexicon_tags_15 },
    { 0x2ae446f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104446fd, SUC_lexicon_tags_8 },
    { 0x256446fe, SUC_lexicon_tags_25 },
    { 0x22e446ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb0644701, SUC_lexicon_tags_37 },
    { 0xbea44702, SUC_lexicon_tags_15 },
    { 0x70844701, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4c44705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa244714, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xffe44716, SUC_lexicon_tags_80 },
    { 0x82844717, SUC_lexicon_tags_11 },
    { 0xe7a44718, SUC_lexicon_tags_47 },
    { 0x67844719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5044471c, SUC_lexicon_tags_31 },
    { 0x4d04471d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc44720, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89a44722, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4844724, SUC_lexicon_tags_17 },
    { 0xe1644725, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd844727, SUC_lexicon_tags_55 },
    { 0xa7a44728, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6da4472a, SUC_lexicon_tags_11 },
    { 0x5e84472b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc4472e, SUC_lexicon_tags_11 },
    { 0xd644472e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f844732, SUC_lexicon_tags_17 },
    { 0xbee44732, SUC_lexicon_tags_33 },
    { 0xa2244732, SUC_lexicon_tags_25 },
    { 0xa9e44734, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xef844737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac4473b, SUC_lexicon_tags_46 },
    { 0x9324473c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b84473e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce44742, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9244745, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde24474b, SUC_lexicon_tags_23 },
    { 0x4664474b, SUC_lexicon_tags_60 },
    { 0xe2a4474d, SUC_lexicon_tags_55 },
    { 0x2224474e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2844753, SUC_lexicon_tags_17 },
    { 0x9da44754, SUC_lexicon_tags_45 },
    { 0x8f644754, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3904475a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6644760, SUC_lexicon_tags_160 },
    { 0x9e44761, SUC_lexicon_tags_28 },
    { 0xaa44761, SUC_lexicon_tags_53 },
    { 0x80244761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5844765, SUC_lexicon_tags_13 },
    { 0xf4a44766, SUC_lexicon_tags_28 },
    { 0x47044766, SUC_lexicon_tags_56 },
    { 0x9b244768, SUC_lexicon_tags_25 },
    { 0x10c44769, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf844476b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5964476d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f044772, SUC_lexicon_tags_15 },
    { 0xc2044772, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac44776, SUC_lexicon_tags_29 },
    { 0xed644777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a4477a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6644477d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5844789, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8444792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95844795, SUC_lexicon_tags_1 },
    { 0xfd644795, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a4479a, SUC_lexicon_tags_65 },
    { 0x4a24479a, SUC_lexicon_tags_47 },
    { 0x9ec4479c, SUC_lexicon_tags_17 },
    { 0xbe84479a, SUC_lexicon_tags_15 },
    { 0x9084479a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4447a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a447a8, SUC_lexicon_tags_16 },
    { 0xb26447a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e447ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8447b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x362447b3, SUC_lexicon_tags_9 },
    { 0xb30447b4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x824447b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4447bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a447c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830447c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80447cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e447d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xce6447d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40447dc, SUC_lexicon_tags_65 },
    { 0x302447dd, SUC_lexicon_tags_11 },
    { 0x856447dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8447e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x104447e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56447e9, SUC_lexicon_tags_614 },
    { 0x856447ea, SUC_lexicon_tags_17 },
    { 0xff2447eb, SUC_lexicon_tags_8 },
    { 0x47a447ec, SUC_lexicon_tags_62 },
    { 0xbc0447ec, SUC_lexicon_tags_25 },
    { 0x3cc447ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7da447f0, SUC_lexicon_tags_17 },
    { 0xf08447f1, SUC_lexicon_tags_2 },
    { 0x3fe447f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd46447f4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x920447f6, SUC_lexicon_tags_2 },
    { 0xcdc447f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e447fa, SUC_lexicon_tags_63 },
    { 0x5dc447fb, SUC_lexicon_tags_1 },
    { 0xb26447fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8a8447fe, SUC_lexicon_tags_32 },
    { 0xc86447ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc44802, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xdee44804, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88444809, SUC_lexicon_tags_1 },
    { 0xfa44480a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd0a4480c, SUC_lexicon_tags_39 },
    { 0xd484480d, SUC_lexicon_tags_12 },
    { 0x18a4480d, SUC_lexicon_tags_17 },
    { 0x124480f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5044822, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c44826, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6164482a, SUC_lexicon_tags_12 },
    { 0x6ee4482a, SUC_lexicon_tags_2 },
    { 0xb104482c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90844833, SUC_lexicon_tags_23 },
    { 0x33a44834, SUC_lexicon_tags_8 },
    { 0xf7e44835, SUC_lexicon_tags_37 },
    { 0x88a44836, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28844838, SUC_lexicon_tags_47 },
    { 0x60044839, SUC_lexicon_tags_7 },
    { 0x66444838, SUC_lexicon_tags_11 },
    { 0x7984483b, SUC_lexicon_tags_43 },
    { 0x1cc44839, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20044841, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c044847, SUC_lexicon_tags_21 },
    { 0xf3c44847, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f24484a, SUC_lexicon_tags_16 },
    { 0x69e4484a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86844850, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x9d444852, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c44858, SUC_lexicon_tags_11 },
    { 0x4fe44859, SUC_lexicon_tags_11 },
    { 0xe2a4485a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc84485d, SUC_lexicon_tags_0 },
    { 0xafa4485e, SUC_lexicon_tags_19 },
    { 0x6784485d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c44862, SUC_lexicon_tags_27 },
    { 0xcfa44863, SUC_lexicon_tags_2 },
    { 0x5c044864, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a4486d, SUC_lexicon_tags_77 },
    { 0xae4486e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d844871, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7c44873, SUC_lexicon_tags_25 },
    { 0x99e44874, SUC_lexicon_tags_30 },
    { 0xc9244875, SUC_lexicon_tags_25 },
    { 0xdf444875, SUC_lexicon_tags_26 },
    { 0xd2e44876, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da4487b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xa2a4487d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de44883, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de4488d, SUC_lexicon_tags_56 },
    { 0xb144488d, SUC_lexicon_tags_49 },
    { 0x81a4488f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42244897, SUC_lexicon_tags_7 },
    { 0xbee44897, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfcc4489a, SUC_lexicon_tags_114 },
    { 0xbe4489a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0448a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9d4448a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc448a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04448a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba448ac, SUC_lexicon_tags_26 },
    { 0x5d2448ad, SUC_lexicon_tags_63 },
    { 0x8be448ad, SUC_lexicon_tags_37 },
    { 0x796448ae, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90448b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4448b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40448bd, SUC_lexicon_tags_17 },
    { 0x540448be, SUC_lexicon_tags_47 },
    { 0xc54448be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x204448c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86448c5, SUC_lexicon_tags_55 },
    { 0xc3c448c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x914448c9, SUC_lexicon_tags_14 },
    { 0x450448ca, SUC_lexicon_tags_34 },
    { 0x3ec448cb, SUC_lexicon_tags_29 },
    { 0x1ba448cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78448d1, SUC_lexicon_tags_63 },
    { 0x7f2448d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8448dc, SUC_lexicon_tags_13 },
    { 0x4d6448dd, SUC_lexicon_tags_27 },
    { 0xf82448de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2a448e0, SUC_lexicon_tags_29 },
    { 0xf56448e0, SUC_lexicon_tags_35 },
    { 0x834448e2, SUC_lexicon_tags_1 },
    { 0x31e448e3, SUC_lexicon_tags_4 },
    { 0x90c448e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a0448e6, SUC_lexicon_tags_6 },
    { 0x44448e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a448eb, SUC_lexicon_tags_151 },
    { 0xa08448ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e448f4, SUC_lexicon_tags_60 },
    { 0xb2448f5, SUC_lexicon_tags_33 },
    { 0x3be448f4, SUC_lexicon_tags_17 },
    { 0x5f8448f7, SUC_lexicon_tags_28 },
    { 0x786448f7, SUC_lexicon_tags_719 },
    { 0, NULL },
    { 0x670448fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6448fd, SUC_lexicon_tags_2 },
    { 0xb84448fe, SUC_lexicon_tags_491 },
    { 0xe44448fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa44902, SUC_lexicon_tags_25 },
    { 0xe6c44903, SUC_lexicon_tags_1 },
    { 0x5ee44904, SUC_lexicon_tags_234 },
    { 0x22e44905, SUC_lexicon_tags_12 },
    { 0x84c44905, SUC_lexicon_tags_49 },
    { 0x9c044904, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x54c44909, SUC_lexicon_tags_37 },
    { 0x68244909, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc324490e, SUC_lexicon_tags_49 },
    { 0x67a4490e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a444912, SUC_lexicon_tags_77 },
    { 0x9ac44912, SUC_lexicon_tags_12 },
    { 0x30844914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee44917, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c4491b, SUC_lexicon_tags_25 },
    { 0x71a4491b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x21e4491e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb644922, SUC_lexicon_tags_7 },
    { 0x96044923, SUC_lexicon_tags_64 },
    { 0x6e444923, SUC_lexicon_tags_17 },
    { 0xdd644924, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbcc44927, SUC_lexicon_tags_19 },
    { 0x65044927, SUC_lexicon_tags_33 },
    { 0x74444929, SUC_lexicon_tags_8 },
    { 0x97e4492a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e4492f, SUC_lexicon_tags_2 },
    { 0x5924492f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe44934, SUC_lexicon_tags_74 },
    { 0xec444935, SUC_lexicon_tags_1 },
    { 0xa2644936, SUC_lexicon_tags_27 },
    { 0xab644936, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x38444939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd744493c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4a44493e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7ce44940, SUC_lexicon_tags_33 },
    { 0x71a44941, SUC_lexicon_tags_116 },
    { 0x1a044942, SUC_lexicon_tags_45 },
    { 0xa2e44942, SUC_lexicon_tags_45 },
    { 0x92a44944, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb0444946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8844948, SUC_lexicon_tags_16 },
    { 0x76444948, SUC_lexicon_tags_7 },
    { 0x4904494a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77244952, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e44957, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f644959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2124495b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5f04495d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf844963, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb644968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a84496d, SUC_lexicon_tags_17 },
    { 0xfce4496e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x85644970, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2844973, SUC_lexicon_tags_12 },
    { 0x78a44974, SUC_lexicon_tags_23 },
    { 0x9dc44974, SUC_lexicon_tags_65 },
    { 0x3fe44973, SUC_lexicon_tags_11 },
    { 0x63244976, SUC_lexicon_tags_25 },
    { 0xa1044978, SUC_lexicon_tags_30 },
    { 0x1d844976, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2e4497b, SUC_lexicon_tags_31 },
    { 0x9f44497c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdec4497e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30844982, SUC_lexicon_tags_10 },
    { 0x5d644983, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2044988, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa4498d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdca4498f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b444992, SUC_lexicon_tags_15 },
    { 0x83c44993, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e44998, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84499b, SUC_lexicon_tags_19 },
    { 0x8524499c, SUC_lexicon_tags_14 },
    { 0x1e64499d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae449a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0449a6, SUC_lexicon_tags_31 },
    { 0x7e449a6, SUC_lexicon_tags_60 },
    { 0xb36449a7, SUC_lexicon_tags_37 },
    { 0xbc6449a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c449ad, SUC_lexicon_tags_17 },
    { 0x41c449ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xec449b1, SUC_lexicon_tags_11 },
    { 0xcf4449b2, SUC_lexicon_tags_26 },
    { 0xc2c449b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798449b8, SUC_lexicon_tags_2 },
    { 0x4ae449b8, SUC_lexicon_tags_57 },
    { 0xef4449b9, SUC_lexicon_tags_0 },
    { 0x20e449bb, SUC_lexicon_tags_49 },
    { 0x26c449ba, SUC_lexicon_tags_12 },
    { 0xcb4449bd, SUC_lexicon_tags_0 },
    { 0x72e449be, SUC_lexicon_tags_17 },
    { 0x530449bf, SUC_lexicon_tags_27 },
    { 0xea4449be, SUC_lexicon_tags_10 },
    { 0x682449b8, SUC_lexicon_tags_3 },
    { 0xdd4449c2, SUC_lexicon_tags_549 },
    { 0xaaa449c2, SUC_lexicon_tags_17 },
    { 0xa80449c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a449c7, SUC_lexicon_tags_0 },
    { 0x6a2449c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x708449ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978449cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec449d4, SUC_lexicon_tags_30 },
    { 0x3ec449d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0c449d7, SUC_lexicon_tags_66 },
    { 0xeb6449d7, SUC_lexicon_tags_0 },
    { 0xb3e449d9, SUC_lexicon_tags_17 },
    { 0x71c449d7, SUC_lexicon_tags_66 },
    { 0x6e8449d7, SUC_lexicon_tags_47 },
    { 0xd92449dc, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758449e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa62449e2, SUC_lexicon_tags_17 },
    { 0xba6449e2, SUC_lexicon_tags_25 },
    { 0xd6a449e2, SUC_lexicon_tags_23 },
    { 0xadc449e5, SUC_lexicon_tags_116 },
    { 0xb34449e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae449e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2449f0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x770449f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144449f9, SUC_lexicon_tags_37 },
    { 0x68449f9, SUC_lexicon_tags_19 },
    { 0xbc2449fa, SUC_lexicon_tags_23 },
    { 0x412449fc, SUC_lexicon_tags_60 },
    { 0xcfc449fd, SUC_lexicon_tags_35 },
    { 0x44449fe, SUC_lexicon_tags_62 },
    { 0xc98449ff, SUC_lexicon_tags_9 },
    { 0xefe44a00, SUC_lexicon_tags_26 },
    { 0xdc044a01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb244a08, SUC_lexicon_tags_21 },
    { 0xe0644a09, SUC_lexicon_tags_26 },
    { 0xba244a08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24644a0d, SUC_lexicon_tags_0 },
    { 0x37044a0d, SUC_lexicon_tags_16 },
    { 0xbb644a0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c44a12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a44a15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e44a1b, SUC_lexicon_tags_15 },
    { 0x6f844a1b, SUC_lexicon_tags_25 },
    { 0xf2a44a1d, SUC_lexicon_tags_23 },
    { 0x7f444a1d, SUC_lexicon_tags_25 },
    { 0xfd644a1e, SUC_lexicon_tags_26 },
    { 0xfdc44a1e, SUC_lexicon_tags_37 },
    { 0xc4244a1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66844a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2844a25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb644a30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13244a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4e44a38, SUC_lexicon_tags_151 },
    { 0xeaa44a38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87044a3d, SUC_lexicon_tags_11 },
    { 0xe1644a3e, SUC_lexicon_tags_37 },
    { 0x3ac44a3e, SUC_lexicon_tags_2 },
    { 0x56844a40, SUC_lexicon_tags_63 },
    { 0x23244a3e, SUC_lexicon_tags_38 },
    { 0xc5a44a42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8ce44a44, SUC_lexicon_tags_25 },
    { 0x5f444a45, SUC_lexicon_tags_151 },
    { 0x17644a45, SUC_lexicon_tags_1 },
    { 0xcaa44a47, SUC_lexicon_tags_55 },
    { 0xa0444a47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee44a4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b644a52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc7644a54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c44a57, SUC_lexicon_tags_17 },
    { 0x79044a57, SUC_lexicon_tags_11 },
    { 0x4be44a57, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x30844a5b, SUC_lexicon_tags_25 },
    { 0xc5444a5c, SUC_lexicon_tags_37 },
    { 0xb9644a5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e44a68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c44a6c, SUC_lexicon_tags_583 },
    { 0xb9c44a6d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a044a73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34e44a75, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46444a7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77644a7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f844a84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc44a88, SUC_lexicon_tags_29 },
    { 0x28c44a89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a44a8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf1044a8e, SUC_lexicon_tags_11 },
    { 0x32644a8f, SUC_lexicon_tags_12 },
    { 0x8d644a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a444a92, SUC_lexicon_tags_30 },
    { 0xc0844a93, SUC_lexicon_tags_23 },
    { 0xd2444a94, SUC_lexicon_tags_17 },
    { 0x61444a95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c44a9b, SUC_lexicon_tags_2 },
    { 0xb1644a9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d644a9e, SUC_lexicon_tags_12 },
    { 0x1044a9e, SUC_lexicon_tags_39 },
    { 0xfd844aa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb444aa3, SUC_lexicon_tags_1 },
    { 0xb244aa4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c44aa7, SUC_lexicon_tags_15 },
    { 0x4f444aa8, SUC_lexicon_tags_37 },
    { 0x2b844aa9, SUC_lexicon_tags_63 },
    { 0x3fa44aaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82244aaf, SUC_lexicon_tags_25 },
    { 0x5c44ab0, SUC_lexicon_tags_17 },
    { 0xc7244aaf, SUC_lexicon_tags_1 },
    { 0x76a44ab2, SUC_lexicon_tags_6 },
    { 0x72044ab3, SUC_lexicon_tags_16 },
    { 0xc4444ab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c44abc, SUC_lexicon_tags_47 },
    { 0xf1044abd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7444ac1, SUC_lexicon_tags_64 },
    { 0x6b444ac2, SUC_lexicon_tags_19 },
    { 0xeae44ac1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc44ac6, SUC_lexicon_tags_43 },
    { 0xaa844ac7, SUC_lexicon_tags_65 },
    { 0x94444ac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ca44aca, SUC_lexicon_tags_27 },
    { 0x8c644acb, SUC_lexicon_tags_35 },
    { 0x3b244acc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c244ace, SUC_lexicon_tags_7 },
    { 0x34844acf, SUC_lexicon_tags_31 },
    { 0x87044ad0, SUC_lexicon_tags_37 },
    { 0xa8844ad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57044ad7, SUC_lexicon_tags_37 },
    { 0x32c44ad7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92044adb, SUC_lexicon_tags_14 },
    { 0x73644adc, SUC_lexicon_tags_56 },
    { 0x27244add, SUC_lexicon_tags_0 },
    { 0xece44adb, SUC_lexicon_tags_66 },
    { 0xfbe44adc, SUC_lexicon_tags_11 },
    { 0x87444adf, SUC_lexicon_tags_37 },
    { 0xade44ae0, SUC_lexicon_tags_11 },
    { 0x85c44adf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x98644ae4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x48844ae6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2f044ae8, SUC_lexicon_tags_79 },
    { 0xada44ae9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9844aed, SUC_lexicon_tags_27 },
    { 0x58644aee, SUC_lexicon_tags_6 },
    { 0xfc644aef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e044af1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e44af4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad444af9, SUC_lexicon_tags_49 },
    { 0xe6e44afa, SUC_lexicon_tags_43 },
    { 0x25e44af9, SUC_lexicon_tags_27 },
    { 0x8dc44afb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c44aff, SUC_lexicon_tags_13 },
    { 0x50c44aff, SUC_lexicon_tags_1 },
    { 0xce244b00, SUC_lexicon_tags_15 },
    { 0x93c44b02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e44b06, SUC_lexicon_tags_11 },
    { 0x14044b07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e444b09, SUC_lexicon_tags_7 },
    { 0xbe044b0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92844b0e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da44b18, SUC_lexicon_tags_7 },
    { 0x67a44b19, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0044b1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f444b1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe44b26, SUC_lexicon_tags_29 },
    { 0xcbc44b26, SUC_lexicon_tags_37 },
    { 0x6ce44b28, SUC_lexicon_tags_12 },
    { 0x64444b29, SUC_lexicon_tags_27 },
    { 0xb8c44b2a, SUC_lexicon_tags_25 },
    { 0x84244b2a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x70644b2e, SUC_lexicon_tags_49 },
    { 0x6a844b2e, SUC_lexicon_tags_10 },
    { 0x57e44b30, SUC_lexicon_tags_11 },
    { 0xeb444b30, SUC_lexicon_tags_7 },
    { 0x7ae44b32, SUC_lexicon_tags_34 },
    { 0xb2a44b33, SUC_lexicon_tags_14 },
    { 0x2f044b32, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e44b39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c44b41, SUC_lexicon_tags_2 },
    { 0xaa644b42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf6844b44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1644b47, SUC_lexicon_tags_98 },
    { 0x48644b48, SUC_lexicon_tags_64 },
    { 0x97e44b49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8444b4b, SUC_lexicon_tags_37 },
    { 0xfc644b4c, SUC_lexicon_tags_17 },
    { 0xfe244b4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c244b56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82844b58, SUC_lexicon_tags_15 },
    { 0xe3644b59, SUC_lexicon_tags_53 },
    { 0xe2e44b5a, SUC_lexicon_tags_60 },
    { 0x40a44b5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74844b5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c44b61, SUC_lexicon_tags_11 },
    { 0x29c44b62, SUC_lexicon_tags_25 },
    { 0x4d844b63, SUC_lexicon_tags_11 },
    { 0x7f644b63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x79444b66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29444b6a, SUC_lexicon_tags_29 },
    { 0x15844b6b, SUC_lexicon_tags_37 },
    { 0x1ac44b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c44b6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e44b72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e44b75, SUC_lexicon_tags_60 },
    { 0xc9444b76, SUC_lexicon_tags_26 },
    { 0x16e44b77, SUC_lexicon_tags_11 },
    { 0x3ca44b78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4c44b7a, SUC_lexicon_tags_2 },
    { 0x56844b7b, SUC_lexicon_tags_14 },
    { 0x13244b7c, SUC_lexicon_tags_28 },
    { 0x78044b7d, SUC_lexicon_tags_15 },
    { 0x61244b7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa44b84, SUC_lexicon_tags_15 },
    { 0xea44b84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x69644b87, SUC_lexicon_tags_1 },
    { 0x91c44b88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a44b8b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xae844b8d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea44b90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c044b97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e844b9a, SUC_lexicon_tags_31 },
    { 0x9d044b9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87444b9e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa844ba1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8244ba5, SUC_lexicon_tags_23 },
    { 0xf1e44ba6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9244bac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9a44bae, SUC_lexicon_tags_11 },
    { 0xd644baf, SUC_lexicon_tags_28 },
    { 0xd8844bb0, SUC_lexicon_tags_15 },
    { 0x9244baf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf044bb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x18844bb5, SUC_lexicon_tags_63 },
    { 0x1d644bb5, SUC_lexicon_tags_39 },
    { 0xec844bb6, SUC_lexicon_tags_37 },
    { 0xa8a44bb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a44bbb, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0x4ba44bbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb0c44bbf, SUC_lexicon_tags_49 },
    { 0xef444bc0, SUC_lexicon_tags_17 },
    { 0x6a244bc0, SUC_lexicon_tags_306 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc844bc4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88844bc9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca44bcd, SUC_lexicon_tags_26 },
    { 0x24a44bce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c044bd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e44bd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc44bd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb644bdd, SUC_lexicon_tags_117 },
    { 0xd1844bde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2044be3, SUC_lexicon_tags_25 },
    { 0xee244be3, SUC_lexicon_tags_17 },
    { 0x2a044be5, SUC_lexicon_tags_19 },
    { 0x41444be4, SUC_lexicon_tags_37 },
    { 0x27e44be7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5244bef, SUC_lexicon_tags_38 },
    { 0x7f844bf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e44bf3, SUC_lexicon_tags_27 },
    { 0x63044bf4, SUC_lexicon_tags_14 },
    { 0x21244bf5, SUC_lexicon_tags_56 },
    { 0x31a44bf5, SUC_lexicon_tags_14 },
    { 0xe2044bf5, SUC_lexicon_tags_42 },
    { 0x5f244bf7, SUC_lexicon_tags_45 },
    { 0xa9a44bf9, SUC_lexicon_tags_41 },
    { 0xeea44bf8, SUC_lexicon_tags_11 },
    { 0xff644bfb, SUC_lexicon_tags_12 },
    { 0x23644bfc, SUC_lexicon_tags_12 },
    { 0x92a44bfd, SUC_lexicon_tags_21 },
    { 0x6b244bf7, SUC_lexicon_tags_6 },
    { 0x66c44bf7, SUC_lexicon_tags_25 },
    { 0xaa444c00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2444c04, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de44c09, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xc6244c0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14644c11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac44c15, SUC_lexicon_tags_38 },
    { 0x8644c16, SUC_lexicon_tags_43 },
    { 0x43244c15, SUC_lexicon_tags_48 },
    { 0x4d044c18, SUC_lexicon_tags_15 },
    { 0x48244c17, SUC_lexicon_tags_38 },
    { 0x64244c1a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe044c1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcc44c1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xda044c20, SUC_lexicon_tags_11 },
    { 0x47044c21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9644c24, SUC_lexicon_tags_7 },
    { 0x92c44c25, SUC_lexicon_tags_17 },
    { 0x40044c26, SUC_lexicon_tags_0 },
    { 0xe0844c25, SUC_lexicon_tags_4 },
    { 0xfce44c26, SUC_lexicon_tags_12 },
    { 0xd2c44c28, SUC_lexicon_tags_23 },
    { 0xf444c2a, SUC_lexicon_tags_47 },
    { 0x5f244c2b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x15044c2d, SUC_lexicon_tags_9 },
    { 0x644c2e, SUC_lexicon_tags_129 },
    { 0xa4644c2e, SUC_lexicon_tags_29 },
    { 0xdde44c2d, SUC_lexicon_tags_31 },
    { 0xf4444c31, SUC_lexicon_tags_37 },
    { 0x6ea44c31, SUC_lexicon_tags_56 },
    { 0x50244c33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1644c3b, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0x68044c3d, SUC_lexicon_tags_31 },
    { 0xdde44c3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a44c49, SUC_lexicon_tags_39 },
    { 0xbe444c4a, SUC_lexicon_tags_23 },
    { 0x2f044c4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76644c4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd3c44c51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a844c55, SUC_lexicon_tags_15 },
    { 0xbee44c56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf1a44c58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c44c5b, SUC_lexicon_tags_15 },
    { 0x9cc44c5b, SUC_lexicon_tags_9 },
    { 0xac244c5d, SUC_lexicon_tags_30 },
    { 0x57c44c5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96e44c60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x54244c63, SUC_lexicon_tags_17 },
    { 0xf844c64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c44c68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd0a44c6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a44c76, SUC_lexicon_tags_8 },
    { 0x59044c77, SUC_lexicon_tags_19 },
    { 0x8aa44c77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8244c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60844c7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e44c82, SUC_lexicon_tags_2 },
    { 0x62a44c82, SUC_lexicon_tags_48 },
    { 0xc444c82, SUC_lexicon_tags_47 },
    { 0x78044c85, SUC_lexicon_tags_37 },
    { 0xd5244c86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbee44c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa044c8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c44c8d, SUC_lexicon_tags_18 },
    { 0x4d244c8e, SUC_lexicon_tags_11 },
    { 0x3244c8f, SUC_lexicon_tags_45 },
    { 0xb1644c8f, SUC_lexicon_tags_1 },
    { 0xdc044c91, SUC_lexicon_tags_29 },
    { 0xf7444c8f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x55a44c94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe44c97, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x99044c99, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3b444c9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1644c9f, SUC_lexicon_tags_106 },
    { 0xee644c9f, SUC_lexicon_tags_65 },
    { 0xfa644ca0, SUC_lexicon_tags_27 },
    { 0x93444c9f, SUC_lexicon_tags_37 },
    { 0x57044ca3, SUC_lexicon_tags_8 },
    { 0xea44ca4, SUC_lexicon_tags_15 },
    { 0xd8a44ca5, SUC_lexicon_tags_25 },
    { 0x58c44ca6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x56644ca9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x34c44cab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61a44cad, SUC_lexicon_tags_2 },
    { 0xa7e44cae, SUC_lexicon_tags_37 },
    { 0xbd644cae, SUC_lexicon_tags_17 },
    { 0xa7844cb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e44cb6, SUC_lexicon_tags_37 },
    { 0xe7644cb7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca44cbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40c44cbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11244cbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d044cc1, SUC_lexicon_tags_13 },
    { 0xd1a44cc1, SUC_lexicon_tags_37 },
    { 0x6644cc3, SUC_lexicon_tags_25 },
    { 0xdbe44cc2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17444cc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a644ccc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb4e44cce, SUC_lexicon_tags_2 },
    { 0x38844ccf, SUC_lexicon_tags_45 },
    { 0xba44cce, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x95644cd2, SUC_lexicon_tags_7 },
    { 0xda44cd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf844cd6, SUC_lexicon_tags_25 },
    { 0x56244cd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc444cd9, SUC_lexicon_tags_29 },
    { 0x32244cd9, SUC_lexicon_tags_7 },
    { 0x5c44cdb, SUC_lexicon_tags_7 },
    { 0x6bc44cdb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e244ce1, SUC_lexicon_tags_11 },
    { 0x4fa44ce2, SUC_lexicon_tags_23 },
    { 0x88c44ce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62244cea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x89044cec, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e44cf0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73244cf4, SUC_lexicon_tags_27 },
    { 0x9aa44cf5, SUC_lexicon_tags_15 },
    { 0xb4244cf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe44cf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58644cfc, SUC_lexicon_tags_23 },
    { 0xd3a44cfc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7444d00, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x28044d02, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e44d05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2a44d07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a44d0d, SUC_lexicon_tags_37 },
    { 0x8be44d0d, SUC_lexicon_tags_2 },
    { 0x50a44d0f, SUC_lexicon_tags_95 },
    { 0xe4c44d10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e44d13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96244d18, SUC_lexicon_tags_1 },
    { 0x83844d19, SUC_lexicon_tags_35 },
    { 0xb6644d1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9044d21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1244d23, SUC_lexicon_tags_17 },
    { 0x24c44d24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3c44d26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a44d2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd844d2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e44d33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7844d3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5244d3f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb2a44d41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8244d46, SUC_lexicon_tags_66 },
    { 0x80044d47, SUC_lexicon_tags_7 },
    { 0x56444d47, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a44d4b, SUC_lexicon_tags_12 },
    { 0xfd844d4c, SUC_lexicon_tags_19 },
    { 0xc4244d4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a844d50, SUC_lexicon_tags_25 },
    { 0xc4c44d50, SUC_lexicon_tags_11 },
    { 0x1a044d52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x34444d54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e44d57, SUC_lexicon_tags_11 },
    { 0x35244d58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a44d5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba44d66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb644d69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf644d6b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96844d71, SUC_lexicon_tags_60 },
    { 0x83644d72, SUC_lexicon_tags_17 },
    { 0x9a44d73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x15044d75, SUC_lexicon_tags_49 },
    { 0x78244d75, SUC_lexicon_tags_47 },
    { 0xf7244d77, SUC_lexicon_tags_43 },
    { 0x34444d78, SUC_lexicon_tags_17 },
    { 0xf3e44d78, SUC_lexicon_tags_27 },
    { 0xd8044d7a, SUC_lexicon_tags_15 },
    { 0xc7844d75, SUC_lexicon_tags_182 },
    { 0x8ca44d79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb244d80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14644d85, SUC_lexicon_tags_25 },
    { 0x82c44d85, SUC_lexicon_tags_9 },
    { 0xb8244d87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8cc44d89, SUC_lexicon_tags_31 },
    { 0x83a44d89, SUC_lexicon_tags_17 },
    { 0x11644d8b, SUC_lexicon_tags_19 },
    { 0x4ee44d8b, SUC_lexicon_tags_15 },
    { 0x7d244d8d, SUC_lexicon_tags_7 },
    { 0xdce44d8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56a44d90, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58444d98, SUC_lexicon_tags_25 },
    { 0xf3e44d99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e44d9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b644da0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58844da9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ba44dab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f844db0, SUC_lexicon_tags_6 },
    { 0x63e44db1, SUC_lexicon_tags_2 },
    { 0x1ce44db0, SUC_lexicon_tags_33 },
    { 0xa2644db0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f844db9, SUC_lexicon_tags_26 },
    { 0xf8e44dba, SUC_lexicon_tags_27 },
    { 0x61444db9, SUC_lexicon_tags_13 },
    { 0xb0e44dbc, SUC_lexicon_tags_2 },
    { 0x8a44dbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5044dc0, SUC_lexicon_tags_1 },
    { 0x2e244dc1, SUC_lexicon_tags_1 },
    { 0xaf444dc0, SUC_lexicon_tags_7 },
    { 0xfea44dc3, SUC_lexicon_tags_25 },
    { 0x39c44dc0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac644dc9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f844dd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc44dda, SUC_lexicon_tags_23 },
    { 0xd2c44ddb, SUC_lexicon_tags_9 },
    { 0xdbe44ddc, SUC_lexicon_tags_2 },
    { 0x66244ddd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91444de5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72c44de7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4644ded, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44c44def, SUC_lexicon_tags_87 },
    { 0xb2044df0, SUC_lexicon_tags_11 },
    { 0xf4844df1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf5844df3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a444df7, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x5644dfa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2844dfc, SUC_lexicon_tags_39 },
    { 0x59244dfd, SUC_lexicon_tags_16 },
    { 0xe244dfe, SUC_lexicon_tags_35 },
    { 0xa7a44dff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c44e07, SUC_lexicon_tags_1 },
    { 0xc244e07, SUC_lexicon_tags_21 },
    { 0xdd444e09, SUC_lexicon_tags_16 },
    { 0xfde44e0a, SUC_lexicon_tags_57 },
    { 0x5c044e0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa444e11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78044e15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4644e1b, SUC_lexicon_tags_25 },
    { 0xdbc44e1c, SUC_lexicon_tags_37 },
    { 0x37a44e1b, SUC_lexicon_tags_47 },
    { 0xee444e1e, SUC_lexicon_tags_68 },
    { 0xc2e44e1f, SUC_lexicon_tags_45 },
    { 0x71c44e1d, SUC_lexicon_tags_35 },
    { 0x7f444e21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a444e25, SUC_lexicon_tags_68 },
    { 0x4f244e26, SUC_lexicon_tags_36 },
    { 0x79044e27, SUC_lexicon_tags_17 },
    { 0x92a44e28, SUC_lexicon_tags_1 },
    { 0xb7a44e27, SUC_lexicon_tags_11 },
    { 0xce44e2a, SUC_lexicon_tags_34 },
    { 0xfd244e2b, SUC_lexicon_tags_11 },
    { 0x5c044e2a, SUC_lexicon_tags_6 },
    { 0x21844e29, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa6444e2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d244e3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7444e41, SUC_lexicon_tags_3 },
    { 0x69a44e42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8644e47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36a44e49, SUC_lexicon_tags_11 },
    { 0x59c44e49, SUC_lexicon_tags_0 },
    { 0xa8444e4a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d644e52, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b644e55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ee44e57, SUC_lexicon_tags_6 },
    { 0xd9e44e57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a44e5b, SUC_lexicon_tags_23 },
    { 0xc0244e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a44e60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce044e68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f844e6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20c44e6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba044e73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdfa44e75, SUC_lexicon_tags_15 },
    { 0x28844e76, SUC_lexicon_tags_37 },
    { 0x8644e76, SUC_lexicon_tags_30 },
    { 0xc3a44e78, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc644e7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3844e81, SUC_lexicon_tags_5 },
    { 0x10044e82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be44e87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c244e8a, SUC_lexicon_tags_38 },
    { 0xfa644e8a, SUC_lexicon_tags_37 },
    { 0xcf244e8c, SUC_lexicon_tags_25 },
    { 0x93044e8d, SUC_lexicon_tags_17 },
    { 0x8a644e8e, SUC_lexicon_tags_2 },
    { 0xa7644e8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed644e97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1444e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b444e9d, SUC_lexicon_tags_14 },
    { 0x7bc44e9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18044ea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa644ea2, SUC_lexicon_tags_15 },
    { 0x67644ea3, SUC_lexicon_tags_1 },
    { 0x73644ea4, SUC_lexicon_tags_30 },
    { 0x6ac44ea2, SUC_lexicon_tags_7 },
    { 0xfb044ea2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6444eab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4244ead, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc44eb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea844eb5, SUC_lexicon_tags_43 },
    { 0x9c244eb5, SUC_lexicon_tags_21 },
    { 0x72444eb7, SUC_lexicon_tags_26 },
    { 0x73644eb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a44ebc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x76044ebf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a244ec1, SUC_lexicon_tags_11 },
    { 0xb8844ec2, SUC_lexicon_tags_12 },
    { 0xa6044ec3, SUC_lexicon_tags_7 },
    { 0xc9244ec4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x47244ec7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42444ecd, SUC_lexicon_tags_14 },
    { 0x29e44ece, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51844ed0, SUC_lexicon_tags_12 },
    { 0x3b444ed1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22644ed6, SUC_lexicon_tags_6 },
    { 0xd9844ed6, SUC_lexicon_tags_11 },
    { 0xa5a44ed8, SUC_lexicon_tags_12 },
    { 0x3e444ed8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a44edc, SUC_lexicon_tags_7 },
    { 0x82444edc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce44ee2, SUC_lexicon_tags_2 },
    { 0xb6644ee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4444ee5, SUC_lexicon_tags_4 },
    { 0xbbc44ee6, SUC_lexicon_tags_37 },
    { 0x2c244ee7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe44eed, SUC_lexicon_tags_17 },
    { 0x92c44eee, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b844ef2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfa644ef4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfcc44ef6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc044ef9, SUC_lexicon_tags_37 },
    { 0x7d844efa, SUC_lexicon_tags_14 },
    { 0xc9c44efa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94644efd, SUC_lexicon_tags_4 },
    { 0xe2244efe, SUC_lexicon_tags_55 },
    { 0x52244efe, SUC_lexicon_tags_23 },
    { 0x60c44f00, SUC_lexicon_tags_15 },
    { 0x95644f00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c44f05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde44f0f, SUC_lexicon_tags_43 },
    { 0xac444f10, SUC_lexicon_tags_11 },
    { 0x99e44f0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee44f13, SUC_lexicon_tags_60 },
    { 0xa3644f14, SUC_lexicon_tags_11 },
    { 0xc5e44f14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71244f17, SUC_lexicon_tags_8 },
    { 0x74644f17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a244f1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa44f1f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9e244f21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf844f23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe5a44f25, SUC_lexicon_tags_17 },
    { 0xf7244f25, SUC_lexicon_tags_30 },
    { 0xb7e44f26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e844f2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6044f32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca44f35, SUC_lexicon_tags_7 },
    { 0x24c44f35, SUC_lexicon_tags_17 },
    { 0xbb244f37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a44f3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xabc44f3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a44f40, SUC_lexicon_tags_22 },
    { 0xa8244f41, SUC_lexicon_tags_17 },
    { 0x32444f42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8044f47, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e44f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b844f50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcc44f52, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc844f55, SUC_lexicon_tags_51 },
    { 0xb8444f55, SUC_lexicon_tags_29 },
    { 0x2e644f57, SUC_lexicon_tags_25 },
    { 0xc8c44f58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52644f5d, SUC_lexicon_tags_21 },
    { 0xb4044f5e, SUC_lexicon_tags_17 },
    { 0x30444f5f, SUC_lexicon_tags_17 },
    { 0x5ae44f60, SUC_lexicon_tags_7 },
    { 0x99044f61, SUC_lexicon_tags_47 },
    { 0x35644f61, SUC_lexicon_tags_84 },
    { 0x9aa44f63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a44f66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34044f69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcaa44f6b, SUC_lexicon_tags_1 },
    { 0xda244f6c, SUC_lexicon_tags_0 },
    { 0x4ca44f6c, SUC_lexicon_tags_43 },
    { 0x6de44f6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57044f70, SUC_lexicon_tags_26 },
    { 0xdb044f71, SUC_lexicon_tags_8 },
    { 0x76444f72, SUC_lexicon_tags_84 },
    { 0x11644f72, SUC_lexicon_tags_61 },
    { 0xb0644f72, SUC_lexicon_tags_15 },
    { 0xc4a44f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7844f7e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c44f84, SUC_lexicon_tags_46 },
    { 0x6de44f85, SUC_lexicon_tags_33 },
    { 0x87a44f86, SUC_lexicon_tags_11 },
    { 0x3c844f87, SUC_lexicon_tags_15 },
    { 0x76044f85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8044f8d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xad244f8f, SUC_lexicon_tags_14 },
    { 0xd1644f90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7be44f92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c844f96, SUC_lexicon_tags_15 },
    { 0x36044f97, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe3844f99, SUC_lexicon_tags_1 },
    { 0x40c44f99, SUC_lexicon_tags_39 },
    { 0x59444f99, SUC_lexicon_tags_24 },
    { 0x2b244f9c, SUC_lexicon_tags_60 },
    { 0xf9a44f9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20a44f9f, SUC_lexicon_tags_6 },
    { 0x6fa44fa0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec44fa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9844fae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x49e44fb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cc44fb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61c44fb4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e644fbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7244fc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30444fc5, SUC_lexicon_tags_60 },
    { 0xe1444fc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac44fca, SUC_lexicon_tags_12 },
    { 0x43644fcb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25644fd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da44fd4, SUC_lexicon_tags_16 },
    { 0x74a44fd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9a044fd7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc844fd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc44fdc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50244fe6, SUC_lexicon_tags_0 },
    { 0xbac44fe7, SUC_lexicon_tags_11 },
    { 0x39644fe8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x65044feb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11c44fed, SUC_lexicon_tags_12 },
    { 0xae844fed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce44ff3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e44ff8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4c844ffa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f644ffc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4444ffe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78445001, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba845003, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ac45005, SUC_lexicon_tags_25 },
    { 0x92645006, SUC_lexicon_tags_37 },
    { 0xc3e45007, SUC_lexicon_tags_15 },
    { 0x6d245008, SUC_lexicon_tags_2 },
    { 0x29c45005, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae45011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdaa45013, SUC_lexicon_tags_29 },
    { 0xa8845013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a845017, SUC_lexicon_tags_0 },
    { 0x42845017, SUC_lexicon_tags_23 },
    { 0x39c45019, SUC_lexicon_tags_15 },
    { 0x1404501a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85845020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2e45022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68245024, SUC_lexicon_tags_38 },
    { 0x11445025, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e4502f, SUC_lexicon_tags_43 },
    { 0x3c645030, SUC_lexicon_tags_28 },
    { 0xa2a45030, SUC_lexicon_tags_14 },
    { 0x7d645032, SUC_lexicon_tags_47 },
    { 0x56845032, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b045036, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a4503a, SUC_lexicon_tags_37 },
    { 0x7a84503b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x99c4503d, SUC_lexicon_tags_11 },
    { 0x7da4503e, SUC_lexicon_tags_23 },
    { 0x3dc4503d, SUC_lexicon_tags_15 },
    { 0x2da4503f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1f045042, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec45045, SUC_lexicon_tags_15 },
    { 0x76a45046, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e4504b, SUC_lexicon_tags_37 },
    { 0x8a44504b, SUC_lexicon_tags_17 },
    { 0xb1e4504c, SUC_lexicon_tags_2 },
    { 0x3fc4504e, SUC_lexicon_tags_15 },
    { 0xa4c4504f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57845053, SUC_lexicon_tags_32 },
    { 0x33045054, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d245056, SUC_lexicon_tags_1 },
    { 0xca45056, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9e45059, SUC_lexicon_tags_7 },
    { 0x5c845059, SUC_lexicon_tags_63 },
    { 0x3984505a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae45060, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa645066, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c245069, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb724506f, SUC_lexicon_tags_29 },
    { 0xea645070, SUC_lexicon_tags_11 },
    { 0xeb645071, SUC_lexicon_tags_9 },
    { 0x76045072, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4645075, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22245078, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbaa4507a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb864507f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e245081, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda45085, SUC_lexicon_tags_6 },
    { 0x30445086, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9c45088, SUC_lexicon_tags_11 },
    { 0x35645088, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba245092, SUC_lexicon_tags_17 },
    { 0x9fa45093, SUC_lexicon_tags_0 },
    { 0x71e45092, SUC_lexicon_tags_15 },
    { 0xfb245095, SUC_lexicon_tags_37 },
    { 0x93a45096, SUC_lexicon_tags_37 },
    { 0x1fe45097, SUC_lexicon_tags_11 },
    { 0xc0045098, SUC_lexicon_tags_17 },
    { 0xbf845098, SUC_lexicon_tags_2 },
    { 0x6ac45092, SUC_lexicon_tags_31 },
    { 0xf724509b, SUC_lexicon_tags_0 },
    { 0xf624509a, SUC_lexicon_tags_31 },
    { 0xa124509c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56450a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c450a4, SUC_lexicon_tags_8 },
    { 0x7ee450a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304450a9, SUC_lexicon_tags_2 },
    { 0xe1c450a9, SUC_lexicon_tags_25 },
    { 0x75e450ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84450b0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04450b3, SUC_lexicon_tags_25 },
    { 0x234450b4, SUC_lexicon_tags_79 },
    { 0x190450b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd58450b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x436450ba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x672450bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c450c0, SUC_lexicon_tags_38 },
    { 0xedc450c1, SUC_lexicon_tags_8 },
    { 0x994450c2, SUC_lexicon_tags_30 },
    { 0xcb0450c1, SUC_lexicon_tags_1 },
    { 0x6ac450c4, SUC_lexicon_tags_4 },
    { 0xd28450c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0450c9, SUC_lexicon_tags_6 },
    { 0xd1a450ca, SUC_lexicon_tags_25 },
    { 0xe24450cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace450cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a450d2, SUC_lexicon_tags_17 },
    { 0x4f0450d2, SUC_lexicon_tags_87 },
    { 0x438450d2, SUC_lexicon_tags_26 },
    { 0xf88450d5, SUC_lexicon_tags_15 },
    { 0xb8c450d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca450db, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5be450dd, SUC_lexicon_tags_9 },
    { 0xe74450de, SUC_lexicon_tags_14 },
    { 0x34c450dd, SUC_lexicon_tags_63 },
    { 0x172450e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82450e4, SUC_lexicon_tags_13 },
    { 0x6dc450e4, SUC_lexicon_tags_17 },
    { 0x9e6450e6, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x336450e9, SUC_lexicon_tags_12 },
    { 0x86c450ea, SUC_lexicon_tags_217 },
    { 0x9fc450eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52e450ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56a450ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4450f3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3d6450f5, SUC_lexicon_tags_37 },
    { 0xd6c450f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0450f9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2450fe, SUC_lexicon_tags_31 },
    { 0x55e450ff, SUC_lexicon_tags_45 },
    { 0x41645100, SUC_lexicon_tags_9 },
    { 0xe5c450fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84845103, SUC_lexicon_tags_49 },
    { 0xd9c45104, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1ae45106, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9304510c, SUC_lexicon_tags_1 },
    { 0x4a44510d, SUC_lexicon_tags_26 },
    { 0xa0c4510c, SUC_lexicon_tags_33 },
    { 0xec4510c, SUC_lexicon_tags_92 },
    { 0xc764510e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae45113, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9245117, SUC_lexicon_tags_9 },
    { 0xf0045118, SUC_lexicon_tags_12 },
    { 0x22645119, SUC_lexicon_tags_37 },
    { 0x3ea45118, SUC_lexicon_tags_14 },
    { 0xabc45119, SUC_lexicon_tags_2 },
    { 0xc3a4511a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a45135, SUC_lexicon_tags_0 },
    { 0x7a845135, SUC_lexicon_tags_10 },
    { 0x2ca45137, SUC_lexicon_tags_14 },
    { 0x68645136, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa484513b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8164513d, SUC_lexicon_tags_1 },
    { 0xcac4513e, SUC_lexicon_tags_17 },
    { 0x7184513e, SUC_lexicon_tags_25 },
    { 0x2be4513f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a45143, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c45147, SUC_lexicon_tags_1 },
    { 0x55645148, SUC_lexicon_tags_11 },
    { 0x9a045148, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c4514c, SUC_lexicon_tags_35 },
    { 0x9e64514d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba645154, SUC_lexicon_tags_38 },
    { 0xaf645154, SUC_lexicon_tags_12 },
    { 0x71645156, SUC_lexicon_tags_47 },
    { 0x64e45154, SUC_lexicon_tags_8 },
    { 0xf1645154, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc224515d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8045160, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74845165, SUC_lexicon_tags_33 },
    { 0xe2245165, SUC_lexicon_tags_17 },
    { 0xd4445167, SUC_lexicon_tags_26 },
    { 0x3da45167, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164516e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29845175, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8445179, SUC_lexicon_tags_9 },
    { 0x62c45179, SUC_lexicon_tags_79 },
    { 0x7aa4517b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c84517d, SUC_lexicon_tags_37 },
    { 0x91e4517e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x17645181, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69445186, SUC_lexicon_tags_42 },
    { 0x6a045187, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf784518e, SUC_lexicon_tags_37 },
    { 0x9fe4518e, SUC_lexicon_tags_2 },
    { 0xa0e4518e, SUC_lexicon_tags_85 },
    { 0xace45190, SUC_lexicon_tags_65 },
    { 0x8e445191, SUC_lexicon_tags_13 },
    { 0x53a45190, SUC_lexicon_tags_14 },
    { 0xca45194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c045196, SUC_lexicon_tags_38 },
    { 0x66e45196, SUC_lexicon_tags_7 },
    { 0xdcc45196, SUC_lexicon_tags_15 },
    { 0xf2045196, SUC_lexicon_tags_25 },
    { 0xa04519a, SUC_lexicon_tags_29 },
    { 0x48c45199, SUC_lexicon_tags_14 },
    { 0x5de4519a, SUC_lexicon_tags_45 },
    { 0xd2a4519d, SUC_lexicon_tags_7 },
    { 0xb645198, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec2451a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e451a2, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x652451a4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x1b4451a6, SUC_lexicon_tags_7 },
    { 0x3e451a7, SUC_lexicon_tags_25 },
    { 0xcfe451a7, SUC_lexicon_tags_66 },
    { 0x630451a9, SUC_lexicon_tags_25 },
    { 0x234451aa, SUC_lexicon_tags_96 },
    { 0x684451ab, SUC_lexicon_tags_37 },
    { 0x82e451aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf6451ae, SUC_lexicon_tags_57 },
    { 0x1c8451af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa3a451b1, SUC_lexicon_tags_27 },
    { 0x310451b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49e451b4, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528451bc, SUC_lexicon_tags_19 },
    { 0x968451bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeee451bf, SUC_lexicon_tags_126 },
    { 0xe5c451bf, SUC_lexicon_tags_15 },
    { 0x338451c1, SUC_lexicon_tags_2 },
    { 0x8ae451c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8451c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x914451c9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a451cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a451cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e451d4, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0xb1e451d6, SUC_lexicon_tags_48 },
    { 0xfb6451d7, SUC_lexicon_tags_31 },
    { 0xda8451d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c0451da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae451df, SUC_lexicon_tags_11 },
    { 0x330451df, SUC_lexicon_tags_15 },
    { 0x58c451e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c451e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe32451eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98a451ed, SUC_lexicon_tags_47 },
    { 0xf62451ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e451f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c451f5, SUC_lexicon_tags_607 },
    { 0x6c0451f6, SUC_lexicon_tags_17 },
    { 0x61e451f6, SUC_lexicon_tags_7 },
    { 0xeec451f8, SUC_lexicon_tags_31 },
    { 0xc0a451f9, SUC_lexicon_tags_21 },
    { 0x53e451fa, SUC_lexicon_tags_62 },
    { 0xd2e451fb, SUC_lexicon_tags_12 },
    { 0xe7a451fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c4520a, SUC_lexicon_tags_86 },
    { 0xaee4520b, SUC_lexicon_tags_72 },
    { 0xf6e4520c, SUC_lexicon_tags_11 },
    { 0x1b04520d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4644520f, SUC_lexicon_tags_23 },
    { 0xeb445210, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3245212, SUC_lexicon_tags_14 },
    { 0x4ca45213, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea4521a, SUC_lexicon_tags_19 },
    { 0xdc04521b, SUC_lexicon_tags_56 },
    { 0x9b04521c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c45222, SUC_lexicon_tags_0 },
    { 0x9ae45223, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d845226, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8045228, SUC_lexicon_tags_0 },
    { 0x51445229, SUC_lexicon_tags_49 },
    { 0xf3e45229, SUC_lexicon_tags_11 },
    { 0xa24522a, SUC_lexicon_tags_7 },
    { 0x11645228, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec45230, SUC_lexicon_tags_6 },
    { 0x3a645231, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1645233, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53645239, SUC_lexicon_tags_9 },
    { 0xb9245239, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a4523e, SUC_lexicon_tags_37 },
    { 0x1124523e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66045243, SUC_lexicon_tags_21 },
    { 0xe9845244, SUC_lexicon_tags_15 },
    { 0x83445244, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fa45247, SUC_lexicon_tags_15 },
    { 0x7d045247, SUC_lexicon_tags_11 },
    { 0x58c45247, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e45250, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e45256, SUC_lexicon_tags_14 },
    { 0x6e245257, SUC_lexicon_tags_26 },
    { 0xad645257, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea4525a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7644525c, SUC_lexicon_tags_14 },
    { 0xb524525d, SUC_lexicon_tags_57 },
    { 0x2dc4525e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f445260, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de45264, SUC_lexicon_tags_7 },
    { 0xa4645265, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfde45267, SUC_lexicon_tags_55 },
    { 0xec45268, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5384526d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9845270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d445272, SUC_lexicon_tags_1723 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a645277, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd204527a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e45281, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfe845283, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e44528a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd045290, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8445292, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5645299, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86c4529b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa204529f, SUC_lexicon_tags_37 },
    { 0xe744529f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8452a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae452aa, SUC_lexicon_tags_75 },
    { 0x602452aa, SUC_lexicon_tags_11 },
    { 0x34452ac, SUC_lexicon_tags_57 },
    { 0xb1e452aa, SUC_lexicon_tags_2 },
    { 0xf78452ae, SUC_lexicon_tags_47 },
    { 0xb7e452ae, SUC_lexicon_tags_43 },
    { 0xe8a452af, SUC_lexicon_tags_15 },
    { 0xd72452ae, SUC_lexicon_tags_25 },
    { 0x6ba452b2, SUC_lexicon_tags_49 },
    { 0x638452b3, SUC_lexicon_tags_9 },
    { 0xe62452b4, SUC_lexicon_tags_14 },
    { 0x2ca452b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12452ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56452bc, SUC_lexicon_tags_65 },
    { 0x530452bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f6452bf, SUC_lexicon_tags_25 },
    { 0x8bc452c0, SUC_lexicon_tags_3 },
    { 0xfe0452c1, SUC_lexicon_tags_11 },
    { 0x36a452c2, SUC_lexicon_tags_37 },
    { 0xce8452c1, SUC_lexicon_tags_14 },
    { 0x70e452c1, SUC_lexicon_tags_31 },
    { 0x4ae452c5, SUC_lexicon_tags_27 },
    { 0xeba452c5, SUC_lexicon_tags_17 },
    { 0x100452c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6452ca, SUC_lexicon_tags_15 },
    { 0x76452cb, SUC_lexicon_tags_12 },
    { 0xd58452cb, SUC_lexicon_tags_27 },
    { 0xb56452cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15a452cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae452d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34452d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84452da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50452dd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb7e452df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da452e4, SUC_lexicon_tags_19 },
    { 0x56452e5, SUC_lexicon_tags_1 },
    { 0x306452e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad8452e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x10452ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34452ee, SUC_lexicon_tags_39 },
    { 0xfe8452ef, SUC_lexicon_tags_0 },
    { 0x6ec452ee, SUC_lexicon_tags_49 },
    { 0xc4c452ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14452f6, SUC_lexicon_tags_11 },
    { 0x6ba452f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0452fb, SUC_lexicon_tags_38 },
    { 0xcba452fc, SUC_lexicon_tags_60 },
    { 0x650452fb, SUC_lexicon_tags_0 },
    { 0x9e2452fd, SUC_lexicon_tags_29 },
    { 0xefa452fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc45303, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9a45305, SUC_lexicon_tags_63 },
    { 0x8c045306, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e4530a, SUC_lexicon_tags_17 },
    { 0x25a4530b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd84530f, SUC_lexicon_tags_34 },
    { 0x5b845310, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb245316, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec445318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7084531b, SUC_lexicon_tags_12 },
    { 0xd604531c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f645322, SUC_lexicon_tags_6 },
    { 0xf5445322, SUC_lexicon_tags_29 },
    { 0x27245324, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e245328, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8564532b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30045330, SUC_lexicon_tags_25 },
    { 0xc3a45331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a245338, SUC_lexicon_tags_17 },
    { 0x40245338, SUC_lexicon_tags_25 },
    { 0x18045339, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc44533c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e45340, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x54245342, SUC_lexicon_tags_12 },
    { 0xca45343, SUC_lexicon_tags_63 },
    { 0x9e45344, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43e45346, SUC_lexicon_tags_31 },
    { 0xe9645347, SUC_lexicon_tags_11 },
    { 0xb5645348, SUC_lexicon_tags_47 },
    { 0xbaa45349, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e4534e, SUC_lexicon_tags_20 },
    { 0x5a4534f, SUC_lexicon_tags_11 },
    { 0x8e045350, SUC_lexicon_tags_19 },
    { 0xde245351, SUC_lexicon_tags_15 },
    { 0x44534f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x92645354, SUC_lexicon_tags_12 },
    { 0x5ee45355, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d645360, SUC_lexicon_tags_60 },
    { 0xef645361, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x55245363, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1845369, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc4536c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2884536f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x73045372, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf645376, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2524537a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4904537d, SUC_lexicon_tags_25 },
    { 0x3c64537d, SUC_lexicon_tags_15 },
    { 0x8504537f, SUC_lexicon_tags_63 },
    { 0xdde4537f, SUC_lexicon_tags_2 },
    { 0x7584537f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87445389, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3764538f, SUC_lexicon_tags_46 },
    { 0x1264538f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4504539b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8453a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222453a4, SUC_lexicon_tags_4 },
    { 0x7ec453a4, SUC_lexicon_tags_17 },
    { 0xd4453a6, SUC_lexicon_tags_26 },
    { 0x6f6453a5, SUC_lexicon_tags_7 },
    { 0x952453a8, SUC_lexicon_tags_40 },
    { 0x962453a8, SUC_lexicon_tags_60 },
    { 0xec0453aa, SUC_lexicon_tags_19 },
    { 0xbe453a9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x806453ad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd44453af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2453b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfb8453b5, SUC_lexicon_tags_4 },
    { 0xabe453b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae453bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2453c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2453c7, SUC_lexicon_tags_25 },
    { 0x72453c8, SUC_lexicon_tags_2 },
    { 0x6d8453c9, SUC_lexicon_tags_136 },
    { 0x4a6453ca, SUC_lexicon_tags_38 },
    { 0x460453cb, SUC_lexicon_tags_46 },
    { 0xa72453cc, SUC_lexicon_tags_66 },
    { 0x8f6453cb, SUC_lexicon_tags_37 },
    { 0x538453cb, SUC_lexicon_tags_37 },
    { 0xc22453cf, SUC_lexicon_tags_9 },
    { 0xce0453c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1aa453d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972453d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcea453da, SUC_lexicon_tags_1 },
    { 0x77a453db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900453df, SUC_lexicon_tags_37 },
    { 0x15a453df, SUC_lexicon_tags_17 },
    { 0x7a0453e1, SUC_lexicon_tags_60 },
    { 0x7c6453e2, SUC_lexicon_tags_24 },
    { 0x164453e1, SUC_lexicon_tags_8 },
    { 0xd0c453e4, SUC_lexicon_tags_7 },
    { 0xf2e453e2, SUC_lexicon_tags_63 },
    { 0xbe2453e1, SUC_lexicon_tags_30 },
    { 0xc0e453e7, SUC_lexicon_tags_15 },
    { 0x1c4453e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x400453eb, SUC_lexicon_tags_16 },
    { 0x2c453ec, SUC_lexicon_tags_15 },
    { 0x6f4453ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x240453ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20453f2, SUC_lexicon_tags_1 },
    { 0xe7e453f2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe06453f5, SUC_lexicon_tags_11 },
    { 0x4f0453f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86453fa, SUC_lexicon_tags_28 },
    { 0x3a4453fb, SUC_lexicon_tags_25 },
    { 0xe6a453fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4fc453fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa845401, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9245403, SUC_lexicon_tags_23 },
    { 0xf0445404, SUC_lexicon_tags_10 },
    { 0x1f445405, SUC_lexicon_tags_23 },
    { 0x75645405, SUC_lexicon_tags_14 },
    { 0x48a45407, SUC_lexicon_tags_62 },
    { 0xc9445408, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc64540a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf24540f, SUC_lexicon_tags_17 },
    { 0xee4540f, SUC_lexicon_tags_17 },
    { 0x62a45411, SUC_lexicon_tags_37 },
    { 0x41a45412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c845416, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xca045419, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x6ae4541b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a4541e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe445422, SUC_lexicon_tags_33 },
    { 0x88645422, SUC_lexicon_tags_11 },
    { 0xbea45424, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda4542c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa45433, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43c45435, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xa5a45437, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e4543c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa4543f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcca45441, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x47445444, SUC_lexicon_tags_33 },
    { 0x3a245445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaee45447, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2045450, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a45457, SUC_lexicon_tags_15 },
    { 0xaf045457, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb44545d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c45461, SUC_lexicon_tags_29 },
    { 0x60c45462, SUC_lexicon_tags_17 },
    { 0x4c445463, SUC_lexicon_tags_3 },
    { 0xcda45462, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb645466, SUC_lexicon_tags_2 },
    { 0x24245467, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9be45469, SUC_lexicon_tags_4 },
    { 0xc3e4546a, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xca4546c, SUC_lexicon_tags_60 },
    { 0x2244546d, SUC_lexicon_tags_17 },
    { 0xc4a4546c, SUC_lexicon_tags_87 },
    { 0x1e4546e, SUC_lexicon_tags_17 },
    { 0x7e44546e, SUC_lexicon_tags_6 },
    { 0xfaa45471, SUC_lexicon_tags_140 },
    { 0xebe45470, SUC_lexicon_tags_11 },
    { 0x9be45473, SUC_lexicon_tags_17 },
    { 0x3ea4546e, SUC_lexicon_tags_25 },
    { 0x1f84546d, SUC_lexicon_tags_25 },
    { 0x7b045476, SUC_lexicon_tags_11 },
    { 0xe1c45470, SUC_lexicon_tags_8 },
    { 0xa2245478, SUC_lexicon_tags_34 },
    { 0x84c45479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4364547b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1424547f, SUC_lexicon_tags_14 },
    { 0x6e24547f, SUC_lexicon_tags_1 },
    { 0x4da45481, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d645486, SUC_lexicon_tags_37 },
    { 0x2c445487, SUC_lexicon_tags_30 },
    { 0xc5845488, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7504548d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21245491, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80645498, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b44549b, SUC_lexicon_tags_28 },
    { 0x8c44549c, SUC_lexicon_tags_17 },
    { 0x1ce4549d, SUC_lexicon_tags_7 },
    { 0x3e44549e, SUC_lexicon_tags_25 },
    { 0xa8a4549f, SUC_lexicon_tags_47 },
    { 0xa00454a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2c454a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x652454a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xef8454a6, SUC_lexicon_tags_14 },
    { 0xdea454a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf62454a9, SUC_lexicon_tags_39 },
    { 0x2d4454aa, SUC_lexicon_tags_38 },
    { 0x16a454ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420454b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0454b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468454bc, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc454bf, SUC_lexicon_tags_37 },
    { 0xf1c454bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1fc454c2, SUC_lexicon_tags_15 },
    { 0x7b6454c3, SUC_lexicon_tags_275 },
    { 0xb24454c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c454c8, SUC_lexicon_tags_1 },
    { 0x6ce454c8, SUC_lexicon_tags_26 },
    { 0x9e4454c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf34454cc, SUC_lexicon_tags_8 },
    { 0x59c454cc, SUC_lexicon_tags_146 },
    { 0x1ec454cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90454d3, SUC_lexicon_tags_60 },
    { 0x24c454d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4454d9, SUC_lexicon_tags_25 },
    { 0x472454da, SUC_lexicon_tags_7 },
    { 0x5d6454db, SUC_lexicon_tags_63 },
    { 0x8b6454da, SUC_lexicon_tags_25 },
    { 0xa3e454dd, SUC_lexicon_tags_46 },
    { 0x328454dd, SUC_lexicon_tags_12 },
    { 0xd56454dc, SUC_lexicon_tags_15 },
    { 0xb5a454e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06454e6, SUC_lexicon_tags_26 },
    { 0xfe454e7, SUC_lexicon_tags_23 },
    { 0xcc4454e6, SUC_lexicon_tags_12 },
    { 0x1e8454e6, SUC_lexicon_tags_1 },
    { 0xd50454e6, SUC_lexicon_tags_49 },
    { 0x7c0454ea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96454ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x324454f1, SUC_lexicon_tags_1 },
    { 0xc60454f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a454f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b0454fa, SUC_lexicon_tags_37 },
    { 0x4d8454fb, SUC_lexicon_tags_11 },
    { 0x6454fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec45501, SUC_lexicon_tags_43 },
    { 0x23645502, SUC_lexicon_tags_25 },
    { 0x36a45503, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcec45505, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a45509, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a45514, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c4551b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65e4551d, SUC_lexicon_tags_12 },
    { 0xda44551e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c045521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4045525, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c245529, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x79c4552b, SUC_lexicon_tags_27 },
    { 0x7184552c, SUC_lexicon_tags_12 },
    { 0x7ac4552d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f845532, SUC_lexicon_tags_9 },
    { 0x13045533, SUC_lexicon_tags_12 },
    { 0x18245534, SUC_lexicon_tags_35 },
    { 0x20845535, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd645539, SUC_lexicon_tags_46 },
    { 0xc244553a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56845541, SUC_lexicon_tags_12 },
    { 0xa4245541, SUC_lexicon_tags_17 },
    { 0x38e45542, SUC_lexicon_tags_60 },
    { 0x58a45544, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x30845546, SUC_lexicon_tags_9 },
    { 0xa4045547, SUC_lexicon_tags_7 },
    { 0x56645546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49c4554a, SUC_lexicon_tags_10 },
    { 0xc3c4554b, SUC_lexicon_tags_29 },
    { 0x6024554a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa104554e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9245553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c645556, SUC_lexicon_tags_23 },
    { 0x42045557, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc4555b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e4555e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde645561, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x53c45563, SUC_lexicon_tags_26 },
    { 0x90c45564, SUC_lexicon_tags_17 },
    { 0x4e445563, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb784556a, SUC_lexicon_tags_17 },
    { 0xcf44556b, SUC_lexicon_tags_34 },
    { 0x5b04556c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d04556e, SUC_lexicon_tags_14 },
    { 0x7ea4556f, SUC_lexicon_tags_14 },
    { 0x18245570, SUC_lexicon_tags_17 },
    { 0xdae45571, SUC_lexicon_tags_17 },
    { 0xd7a45572, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x38e45574, SUC_lexicon_tags_1 },
    { 0x44845574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e4557d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9664557f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c45582, SUC_lexicon_tags_15 },
    { 0xd0e45583, SUC_lexicon_tags_12 },
    { 0x42845582, SUC_lexicon_tags_2 },
    { 0xe8645585, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e4558b, SUC_lexicon_tags_1 },
    { 0x5d04558c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c4558f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8245591, SUC_lexicon_tags_87 },
    { 0x4ea45591, SUC_lexicon_tags_59 },
    { 0x43e45593, SUC_lexicon_tags_14 },
    { 0x66645591, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d645598, SUC_lexicon_tags_66 },
    { 0xc4645599, SUC_lexicon_tags_25 },
    { 0xc2645599, SUC_lexicon_tags_9 },
    { 0xa04559b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c455a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c455a4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba455a8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x53c455aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34e455ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c455af, SUC_lexicon_tags_11 },
    { 0x690455b0, SUC_lexicon_tags_37 },
    { 0xbb0455b1, SUC_lexicon_tags_11 },
    { 0x9d6455b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x444455b4, SUC_lexicon_tags_63 },
    { 0xe20455b5, SUC_lexicon_tags_17 },
    { 0x36c455b4, SUC_lexicon_tags_15 },
    { 0xa2a455b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26455bc, SUC_lexicon_tags_151 },
    { 0x4aa455bc, SUC_lexicon_tags_15 },
    { 0x458455bd, SUC_lexicon_tags_27 },
    { 0xca4455bd, SUC_lexicon_tags_8 },
    { 0x69a455c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8455c5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x594455c8, SUC_lexicon_tags_0 },
    { 0x438455c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xac2455cb, SUC_lexicon_tags_27 },
    { 0x2c6455cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e455d1, SUC_lexicon_tags_25 },
    { 0x688455d2, SUC_lexicon_tags_9 },
    { 0xa04455d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18455d7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8455e1, SUC_lexicon_tags_2 },
    { 0xeb0455e2, SUC_lexicon_tags_37 },
    { 0x32455e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x646455e6, SUC_lexicon_tags_2 },
    { 0xbfa455e7, SUC_lexicon_tags_37 },
    { 0x9aa455e7, SUC_lexicon_tags_45 },
    { 0x390455e9, SUC_lexicon_tags_48 },
    { 0xed6455e8, SUC_lexicon_tags_39 },
    { 0x5e455ea, SUC_lexicon_tags_1147 },
    { 0x7ee455eb, SUC_lexicon_tags_37 },
    { 0xce4455e7, SUC_lexicon_tags_12 },
    { 0xc86455ee, SUC_lexicon_tags_10 },
    { 0x982455ef, SUC_lexicon_tags_37 },
    { 0x470455ee, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e455f4, SUC_lexicon_tags_9 },
    { 0xec4455f4, SUC_lexicon_tags_7 },
    { 0x416455f6, SUC_lexicon_tags_17 },
    { 0x41e455f7, SUC_lexicon_tags_8 },
    { 0xea6455f4, SUC_lexicon_tags_30 },
    { 0x18e455f5, SUC_lexicon_tags_17 },
    { 0x5c4455f4, SUC_lexicon_tags_2 },
    { 0x4b8455fb, SUC_lexicon_tags_138 },
    { 0x89a455fb, SUC_lexicon_tags_31 },
    { 0x570455fd, SUC_lexicon_tags_32 },
    { 0xbf4455fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc45603, SUC_lexicon_tags_26 },
    { 0xb2245604, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2445608, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c44560b, SUC_lexicon_tags_17 },
    { 0x3dc4560b, SUC_lexicon_tags_31 },
    { 0x4204560d, SUC_lexicon_tags_11 },
    { 0x4864560d, SUC_lexicon_tags_37 },
    { 0x9704560f, SUC_lexicon_tags_0 },
    { 0xf6a45610, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29645615, SUC_lexicon_tags_12 },
    { 0xcdc45616, SUC_lexicon_tags_39 },
    { 0xf6645617, SUC_lexicon_tags_503 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e4561c, SUC_lexicon_tags_87 },
    { 0xfa84561c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3845622, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4a45624, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72a45626, SUC_lexicon_tags_37 },
    { 0x13e45626, SUC_lexicon_tags_2 },
    { 0x42c45628, SUC_lexicon_tags_33 },
    { 0x54a45629, SUC_lexicon_tags_37 },
    { 0x83c45627, SUC_lexicon_tags_11 },
    { 0x5fe45628, SUC_lexicon_tags_12 },
    { 0xae245629, SUC_lexicon_tags_8 },
    { 0x30e4562b, SUC_lexicon_tags_3 },
    { 0xac845629, SUC_lexicon_tags_7 },
    { 0x9f24562f, SUC_lexicon_tags_60 },
    { 0xc904562f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30645632, SUC_lexicon_tags_13 },
    { 0x38845633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39045635, SUC_lexicon_tags_28 },
    { 0x29c45636, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda845638, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d64563c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e45642, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab845645, SUC_lexicon_tags_25 },
    { 0x18a45646, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364564b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba24564d, SUC_lexicon_tags_17 },
    { 0x32e4564e, SUC_lexicon_tags_14 },
    { 0xe624564f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c845660, SUC_lexicon_tags_25 },
    { 0xb3e45660, SUC_lexicon_tags_11 },
    { 0x1ea45662, SUC_lexicon_tags_1 },
    { 0x3a045663, SUC_lexicon_tags_12 },
    { 0x30a45664, SUC_lexicon_tags_14 },
    { 0xa5a45664, SUC_lexicon_tags_11 },
    { 0xd245665, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c4566b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5a4566d, SUC_lexicon_tags_1 },
    { 0x2784566e, SUC_lexicon_tags_30 },
    { 0xb24566f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c45672, SUC_lexicon_tags_25 },
    { 0xf3c45673, SUC_lexicon_tags_14 },
    { 0xb6445674, SUC_lexicon_tags_10 },
    { 0xbfe45675, SUC_lexicon_tags_15 },
    { 0xd7a45676, SUC_lexicon_tags_25 },
    { 0x46a45676, SUC_lexicon_tags_47 },
    { 0xe445678, SUC_lexicon_tags_37 },
    { 0x74445679, SUC_lexicon_tags_39 },
    { 0xb924567a, SUC_lexicon_tags_2 },
    { 0x4e44567b, SUC_lexicon_tags_1 },
    { 0x544567a, SUC_lexicon_tags_11 },
    { 0xf445676, SUC_lexicon_tags_48 },
    { 0x6c4567e, SUC_lexicon_tags_1 },
    { 0x28045677, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x92445682, SUC_lexicon_tags_1203 },
    { 0x88245683, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70e45685, SUC_lexicon_tags_15 },
    { 0x2dc45686, SUC_lexicon_tags_7 },
    { 0xeee45687, SUC_lexicon_tags_7 },
    { 0x95a45688, SUC_lexicon_tags_12 },
    { 0xb8c45689, SUC_lexicon_tags_2 },
    { 0x43e45688, SUC_lexicon_tags_26 },
    { 0x16845688, SUC_lexicon_tags_11 },
    { 0x99c4568c, SUC_lexicon_tags_11 },
    { 0x28645688, SUC_lexicon_tags_39 },
    { 0x9584568e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19245691, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f045695, SUC_lexicon_tags_10 },
    { 0xc5845695, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8764569a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6624569d, SUC_lexicon_tags_14 },
    { 0xbd04569e, SUC_lexicon_tags_43 },
    { 0x4964569f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8456a3, SUC_lexicon_tags_2 },
    { 0xe36456a3, SUC_lexicon_tags_21 },
    { 0xc0c456a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x810456a8, SUC_lexicon_tags_25 },
    { 0x3d0456a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664456ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20456b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14456b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e456b9, SUC_lexicon_tags_37 },
    { 0xdc6456ba, SUC_lexicon_tags_26 },
    { 0x33e456b9, SUC_lexicon_tags_33 },
    { 0x684456bc, SUC_lexicon_tags_7 },
    { 0xb42456bd, SUC_lexicon_tags_2 },
    { 0x938456be, SUC_lexicon_tags_64 },
    { 0xc4e456bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3c456c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa456c6, SUC_lexicon_tags_7 },
    { 0xaf6456c7, SUC_lexicon_tags_7 },
    { 0xa64456c6, SUC_lexicon_tags_6 },
    { 0x69e456c7, SUC_lexicon_tags_2 },
    { 0x2b6456ca, SUC_lexicon_tags_1 },
    { 0x60456cb, SUC_lexicon_tags_55 },
    { 0xce8456cc, SUC_lexicon_tags_12 },
    { 0x2da456cd, SUC_lexicon_tags_19 },
    { 0x542456ce, SUC_lexicon_tags_17 },
    { 0xa60456cd, SUC_lexicon_tags_17 },
    { 0xefa456cd, SUC_lexicon_tags_25 },
    { 0xe76456d1, SUC_lexicon_tags_2 },
    { 0x170456c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2da456d4, SUC_lexicon_tags_1 },
    { 0x134456d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c456d8, SUC_lexicon_tags_60 },
    { 0x8a8456d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6456dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2456df, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x846456e1, SUC_lexicon_tags_25 },
    { 0x298456e2, SUC_lexicon_tags_23 },
    { 0x188456e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x752456e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0456ec, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0456ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4456f5, SUC_lexicon_tags_19 },
    { 0xc12456f5, SUC_lexicon_tags_19 },
    { 0x2c0456f7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678456fb, SUC_lexicon_tags_61 },
    { 0xba6456fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c2456fe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a45702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c45709, SUC_lexicon_tags_32 },
    { 0xa0445709, SUC_lexicon_tags_9 },
    { 0x3fe4570b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c44570d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5b44570f, SUC_lexicon_tags_9 },
    { 0x4ba45710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a45713, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea45717, SUC_lexicon_tags_2 },
    { 0x68245718, SUC_lexicon_tags_19 },
    { 0x69845718, SUC_lexicon_tags_33 },
    { 0xfbe4571a, SUC_lexicon_tags_38 },
    { 0xe9c4571a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da45722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf845724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47245726, SUC_lexicon_tags_23 },
    { 0x60845727, SUC_lexicon_tags_23 },
    { 0x38c45727, SUC_lexicon_tags_15 },
    { 0xeec45727, SUC_lexicon_tags_30 },
    { 0x3045728, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5804572f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d845731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a45735, SUC_lexicon_tags_21 },
    { 0x77a45736, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x30845738, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58c4573a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e45741, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c45744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a4574a, SUC_lexicon_tags_16 },
    { 0x5a04574b, SUC_lexicon_tags_11 },
    { 0xcfc4574c, SUC_lexicon_tags_27 },
    { 0x5404574a, SUC_lexicon_tags_17 },
    { 0x2b64574c, SUC_lexicon_tags_27 },
    { 0x364574f, SUC_lexicon_tags_15 },
    { 0x16445750, SUC_lexicon_tags_37 },
    { 0xf1c45750, SUC_lexicon_tags_2 },
    { 0xc6c45752, SUC_lexicon_tags_19 },
    { 0xfee45750, SUC_lexicon_tags_63 },
    { 0x1c845754, SUC_lexicon_tags_17 },
    { 0x73c45755, SUC_lexicon_tags_29 },
    { 0xa8245751, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c45759, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a24575c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c45761, SUC_lexicon_tags_26 },
    { 0xda645761, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be45765, SUC_lexicon_tags_65 },
    { 0x61e45765, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde24576c, SUC_lexicon_tags_15 },
    { 0x40e4576d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6be4576f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1845771, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5645774, SUC_lexicon_tags_7 },
    { 0x54645775, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c45781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f245784, SUC_lexicon_tags_47 },
    { 0x1c445784, SUC_lexicon_tags_26 },
    { 0x61645786, SUC_lexicon_tags_33 },
    { 0x4da45786, SUC_lexicon_tags_39 },
    { 0xa7845787, SUC_lexicon_tags_11 },
    { 0x4f445786, SUC_lexicon_tags_87 },
    { 0xb644578a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc424578c, SUC_lexicon_tags_47 },
    { 0x7b04578c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d84578f, SUC_lexicon_tags_65 },
    { 0x78245790, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41445798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc504579e, SUC_lexicon_tags_31 },
    { 0xf084579f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28457a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca457aa, SUC_lexicon_tags_68 },
    { 0x234457aa, SUC_lexicon_tags_1 },
    { 0xac457ac, SUC_lexicon_tags_37 },
    { 0x43c457aa, SUC_lexicon_tags_82 },
    { 0x348457ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x300457b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0a457b2, SUC_lexicon_tags_1154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e457b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2457be, SUC_lexicon_tags_6 },
    { 0x8e457bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8457c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca457c7, SUC_lexicon_tags_26 },
    { 0x64457c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6e457ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x528457cc, SUC_lexicon_tags_2 },
    { 0xf5e457cd, SUC_lexicon_tags_25 },
    { 0xfb8457cd, SUC_lexicon_tags_43 },
    { 0x6f8457cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c6457d1, SUC_lexicon_tags_25 },
    { 0x90c457d2, SUC_lexicon_tags_15 },
    { 0xcf0457d3, SUC_lexicon_tags_30 },
    { 0x6f4457d1, SUC_lexicon_tags_37 },
    { 0x74457d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194457dd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x144457df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28457e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c457e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196457f1, SUC_lexicon_tags_26 },
    { 0x3ec457f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x440457f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec457f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe80457fa, SUC_lexicon_tags_25 },
    { 0x450457fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x902457fd, SUC_lexicon_tags_15 },
    { 0xef6457fe, SUC_lexicon_tags_15 },
    { 0x20457fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1445801, SUC_lexicon_tags_12 },
    { 0x73845801, SUC_lexicon_tags_31 },
    { 0x2845801, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbc45805, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47645808, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x444580a, SUC_lexicon_tags_13 },
    { 0x8644580b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40245815, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa3c45817, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xec04581a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2224581c, SUC_lexicon_tags_39 },
    { 0xcfa4581d, SUC_lexicon_tags_33 },
    { 0x5884581d, SUC_lexicon_tags_87 },
    { 0xb1e4581d, SUC_lexicon_tags_2 },
    { 0xe6845820, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce45823, SUC_lexicon_tags_27 },
    { 0x9e645824, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec645826, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa45829, SUC_lexicon_tags_27 },
    { 0xe2e45829, SUC_lexicon_tags_27 },
    { 0x9dc4582a, SUC_lexicon_tags_1 },
    { 0xc4c4582b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7864582f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x21045832, SUC_lexicon_tags_46 },
    { 0xcc645833, SUC_lexicon_tags_12 },
    { 0x73645834, SUC_lexicon_tags_2 },
    { 0x5e645833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee24583a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b44583f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5c45841, SUC_lexicon_tags_37 },
    { 0x91645841, SUC_lexicon_tags_25 },
    { 0xa3845843, SUC_lexicon_tags_21 },
    { 0x95645844, SUC_lexicon_tags_7 },
    { 0xc5245845, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x16445848, SUC_lexicon_tags_14 },
    { 0xdda45848, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec4584d, SUC_lexicon_tags_14 },
    { 0xe584584e, SUC_lexicon_tags_7 },
    { 0xf544584f, SUC_lexicon_tags_2 },
    { 0x6064584f, SUC_lexicon_tags_8 },
    { 0x69a45851, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c45855, SUC_lexicon_tags_47 },
    { 0xd5845856, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7845859, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x864585b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c645861, SUC_lexicon_tags_0 },
    { 0x46045861, SUC_lexicon_tags_3 },
    { 0xce45863, SUC_lexicon_tags_77 },
    { 0x72a45862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e4586a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd24586d, SUC_lexicon_tags_47 },
    { 0xf044586e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed645871, SUC_lexicon_tags_35 },
    { 0x4ac45872, SUC_lexicon_tags_9 },
    { 0xdc245873, SUC_lexicon_tags_25 },
    { 0xcea45871, SUC_lexicon_tags_9 },
    { 0x75c45875, SUC_lexicon_tags_17 },
    { 0x19a45875, SUC_lexicon_tags_11 },
    { 0x68045877, SUC_lexicon_tags_14 },
    { 0xbfe45878, SUC_lexicon_tags_6 },
    { 0x86845878, SUC_lexicon_tags_0 },
    { 0xac64587a, SUC_lexicon_tags_25 },
    { 0xc4845874, SUC_lexicon_tags_57 },
    { 0xada45871, SUC_lexicon_tags_12 },
    { 0xd1045871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41845881, SUC_lexicon_tags_11 },
    { 0x50645881, SUC_lexicon_tags_12 },
    { 0x69e45881, SUC_lexicon_tags_25 },
    { 0x4a045884, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6244588d, SUC_lexicon_tags_11 },
    { 0xc084588e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c45892, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8845894, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x21645896, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x28845898, SUC_lexicon_tags_88 },
    { 0xbae45898, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8684589b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b04589e, SUC_lexicon_tags_35 },
    { 0x73a4589f, SUC_lexicon_tags_43 },
    { 0x5cc458a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608458a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x776458a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6e458a9, SUC_lexicon_tags_23 },
    { 0x81a458a9, SUC_lexicon_tags_47 },
    { 0xce2458ab, SUC_lexicon_tags_15 },
    { 0x964458ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a458b7, SUC_lexicon_tags_46 },
    { 0x1e2458b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x852458ba, SUC_lexicon_tags_11 },
    { 0x9ac458bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6458be, SUC_lexicon_tags_39 },
    { 0x630458bf, SUC_lexicon_tags_29 },
    { 0xc68458bf, SUC_lexicon_tags_6 },
    { 0xd8c458c1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd8c458c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00458c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2458ce, SUC_lexicon_tags_9 },
    { 0x85e458ce, SUC_lexicon_tags_25 },
    { 0xb1e458d0, SUC_lexicon_tags_10 },
    { 0xd7e458d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444458d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66458dd, SUC_lexicon_tags_54 },
    { 0xea8458dd, SUC_lexicon_tags_12 },
    { 0x2da458df, SUC_lexicon_tags_12 },
    { 0x60e458de, SUC_lexicon_tags_25 },
    { 0x87c458df, SUC_lexicon_tags_12 },
    { 0xb88458e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae458e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6458eb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e458ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a458f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3f4458f5, SUC_lexicon_tags_15 },
    { 0xdcc458f6, SUC_lexicon_tags_25 },
    { 0x7b8458f7, SUC_lexicon_tags_56 },
    { 0xa2458f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100458ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4445902, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27445904, SUC_lexicon_tags_17 },
    { 0xc0645905, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x61045907, SUC_lexicon_tags_25 },
    { 0x86645907, SUC_lexicon_tags_17 },
    { 0xec045909, SUC_lexicon_tags_0 },
    { 0xd5245908, SUC_lexicon_tags_39 },
    { 0xf964590b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9284590f, SUC_lexicon_tags_26 },
    { 0x4324590f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x76e45912, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda45915, SUC_lexicon_tags_17 },
    { 0xe6a45916, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa24591e, SUC_lexicon_tags_543 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce45922, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf445926, SUC_lexicon_tags_25 },
    { 0x8ee45926, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e24592b, SUC_lexicon_tags_27 },
    { 0xdf04592c, SUC_lexicon_tags_26 },
    { 0xa264592c, SUC_lexicon_tags_15 },
    { 0x70e4592e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16445935, SUC_lexicon_tags_33 },
    { 0x86645936, SUC_lexicon_tags_38 },
    { 0x28e45936, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x63645939, SUC_lexicon_tags_19 },
    { 0xc124593a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa6c4593c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9444593f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e45943, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc4594a, SUC_lexicon_tags_55 },
    { 0xd344594b, SUC_lexicon_tags_12 },
    { 0x80e4594c, SUC_lexicon_tags_37 },
    { 0x9784594b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84645951, SUC_lexicon_tags_6 },
    { 0x9845952, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae45956, SUC_lexicon_tags_14 },
    { 0x35245957, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfa45959, SUC_lexicon_tags_39 },
    { 0xc024595a, SUC_lexicon_tags_37 },
    { 0xf804595b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6964595e, SUC_lexicon_tags_37 },
    { 0xf4a4595f, SUC_lexicon_tags_27 },
    { 0x57445960, SUC_lexicon_tags_11 },
    { 0xc645961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c245963, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85645968, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc64596c, SUC_lexicon_tags_17 },
    { 0x4fc4596d, SUC_lexicon_tags_69 },
    { 0xa484596e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2845971, SUC_lexicon_tags_45 },
    { 0x7b045971, SUC_lexicon_tags_17 },
    { 0xa1a45972, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c45977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d24597a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6164597e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9c845980, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x9845983, SUC_lexicon_tags_15 },
    { 0x2d445983, SUC_lexicon_tags_30 },
    { 0x23845985, SUC_lexicon_tags_0 },
    { 0x7a445986, SUC_lexicon_tags_15 },
    { 0x8dc45987, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad44598a, SUC_lexicon_tags_26 },
    { 0xc9c4598b, SUC_lexicon_tags_39 },
    { 0x6c84598c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc445992, SUC_lexicon_tags_7 },
    { 0x93245993, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d04599c, SUC_lexicon_tags_2 },
    { 0x9e04599d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee24599f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34459a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbc8459a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e459ae, SUC_lexicon_tags_17 },
    { 0xcea459af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x32459b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c6459b3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x85e459b5, SUC_lexicon_tags_30 },
    { 0x230459b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc36459b8, SUC_lexicon_tags_17 },
    { 0xd22459b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e459c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x70459c5, SUC_lexicon_tags_31 },
    { 0x520459c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2459ca, SUC_lexicon_tags_12 },
    { 0xda0459cb, SUC_lexicon_tags_30 },
    { 0x8e0459cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6459cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x236459d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6459d6, SUC_lexicon_tags_12 },
    { 0xaae459d7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe26459d9, SUC_lexicon_tags_6 },
    { 0x794459d9, SUC_lexicon_tags_37 },
    { 0xfe2459da, SUC_lexicon_tags_16 },
    { 0x2c8459dc, SUC_lexicon_tags_30 },
    { 0x2a0459dd, SUC_lexicon_tags_46 },
    { 0x820459de, SUC_lexicon_tags_23 },
    { 0xa8a459df, SUC_lexicon_tags_17 },
    { 0x1fe459df, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90459e3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a459ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0459f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6459f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22459f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8459fc, SUC_lexicon_tags_8 },
    { 0x7c8459fd, SUC_lexicon_tags_17 },
    { 0x5f8459fe, SUC_lexicon_tags_17 },
    { 0xd1e459ff, SUC_lexicon_tags_14 },
    { 0x6ae459ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f845a02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74445a07, SUC_lexicon_tags_19 },
    { 0x7e045a07, SUC_lexicon_tags_27 },
    { 0xb8a45a08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb445a15, SUC_lexicon_tags_43 },
    { 0x4045a16, SUC_lexicon_tags_2 },
    { 0xdc645a15, SUC_lexicon_tags_7 },
    { 0x5e845a18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x24045a1a, SUC_lexicon_tags_45 },
    { 0xed845a1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2045a1d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x40845a1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54845a24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a45a29, SUC_lexicon_tags_21 },
    { 0x3e045a2a, SUC_lexicon_tags_1 },
    { 0xbe245a2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1645a2d, SUC_lexicon_tags_37 },
    { 0xe3845a2e, SUC_lexicon_tags_33 },
    { 0x35445a2e, SUC_lexicon_tags_63 },
    { 0x76845a30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x16445a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94a45a34, SUC_lexicon_tags_11 },
    { 0xe8c45a35, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1ba45a37, SUC_lexicon_tags_141 },
    { 0x1f645a38, SUC_lexicon_tags_33 },
    { 0xd6c45a39, SUC_lexicon_tags_23 },
    { 0x27245a3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f445a3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x38a45a40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c245a43, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79645a47, SUC_lexicon_tags_27 },
    { 0xb6e45a48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75c45a4a, SUC_lexicon_tags_10 },
    { 0x3c845a4a, SUC_lexicon_tags_47 },
    { 0xf9a45a4c, SUC_lexicon_tags_11 },
    { 0xcd845a4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72845a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e45a52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4645a54, SUC_lexicon_tags_2 },
    { 0x2c645a55, SUC_lexicon_tags_8 },
    { 0xb5c45a56, SUC_lexicon_tags_65 },
    { 0xf1a45a56, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3645a5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf045a5f, SUC_lexicon_tags_26 },
    { 0x89c45a60, SUC_lexicon_tags_19 },
    { 0xea845a61, SUC_lexicon_tags_31 },
    { 0xf6645a62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a845a6c, SUC_lexicon_tags_40 },
    { 0x94645a6d, SUC_lexicon_tags_1 },
    { 0xe9c45a6e, SUC_lexicon_tags_14 },
    { 0x85445a6e, SUC_lexicon_tags_47 },
    { 0x81445a70, SUC_lexicon_tags_7 },
    { 0x1445a70, SUC_lexicon_tags_9 },
    { 0x77845a70, SUC_lexicon_tags_25 },
    { 0xdb445a71, SUC_lexicon_tags_25 },
    { 0x77a45a72, SUC_lexicon_tags_31 },
    { 0x6dc45a6e, SUC_lexicon_tags_38 },
    { 0xd4845a76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e45a7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e245a7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x81245a7e, SUC_lexicon_tags_17 },
    { 0x3b045a7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3de45a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf645a84, SUC_lexicon_tags_37 },
    { 0x31e45a85, SUC_lexicon_tags_7 },
    { 0x4cc45a84, SUC_lexicon_tags_16 },
    { 0xd6845a87, SUC_lexicon_tags_12 },
    { 0xabe45a88, SUC_lexicon_tags_15 },
    { 0x56c45a89, SUC_lexicon_tags_14 },
    { 0x65445a8a, SUC_lexicon_tags_11 },
    { 0xf7a45a8b, SUC_lexicon_tags_39 },
    { 0x1a445a8a, SUC_lexicon_tags_37 },
    { 0xf9a45a8d, SUC_lexicon_tags_15 },
    { 0x70a45a85, SUC_lexicon_tags_7 },
    { 0x32045a8d, SUC_lexicon_tags_17 },
    { 0x73045a90, SUC_lexicon_tags_27 },
    { 0x7aa45a91, SUC_lexicon_tags_25 },
    { 0x26045a92, SUC_lexicon_tags_46 },
    { 0x53245a91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c45a97, SUC_lexicon_tags_6 },
    { 0x26645a98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3845a9b, SUC_lexicon_tags_719 },
    { 0x86045a9c, SUC_lexicon_tags_30 },
    { 0x7e645a9d, SUC_lexicon_tags_47 },
    { 0xab845a9e, SUC_lexicon_tags_17 },
    { 0xbda45a9f, SUC_lexicon_tags_15 },
    { 0x54845aa0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19045aa2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7045aaa, SUC_lexicon_tags_1 },
    { 0xe7e45aab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c645ab4, SUC_lexicon_tags_15 },
    { 0x34245ab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a645ab8, SUC_lexicon_tags_7 },
    { 0x10845ab9, SUC_lexicon_tags_23 },
    { 0xe2645aba, SUC_lexicon_tags_2 },
    { 0xcb845ab9, SUC_lexicon_tags_27 },
    { 0xf1a45abb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa845ac1, SUC_lexicon_tags_7 },
    { 0xc9845ac1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48a45ac4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd2045ac6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa0645ac8, SUC_lexicon_tags_20 },
    { 0x9d645ac9, SUC_lexicon_tags_27 },
    { 0x24e45ac9, SUC_lexicon_tags_11 },
    { 0x24e45acb, SUC_lexicon_tags_3 },
    { 0xf7a45aca, SUC_lexicon_tags_29 },
    { 0x94e45aca, SUC_lexicon_tags_37 },
    { 0xb5a45ac9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2245ad1, SUC_lexicon_tags_12 },
    { 0x51c45ad2, SUC_lexicon_tags_0 },
    { 0x3245ad3, SUC_lexicon_tags_7 },
    { 0xf6845ad4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2245ad6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3445ad8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba45ade, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e245ae4, SUC_lexicon_tags_6 },
    { 0x17a45ae5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdd245ae7, SUC_lexicon_tags_1 },
    { 0x36e45ae7, SUC_lexicon_tags_11 },
    { 0xb6a45ae8, SUC_lexicon_tags_7 },
    { 0x89645ae9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f845aed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee45af1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c45af5, SUC_lexicon_tags_26 },
    { 0x8e245af6, SUC_lexicon_tags_46 },
    { 0xe3445af7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38845afa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56045afe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc45b02, SUC_lexicon_tags_11 },
    { 0x1f445b03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde845b08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7fc45b0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee45b0e, SUC_lexicon_tags_11 },
    { 0x59845b0e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f245b14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d245b16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcc45b18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6445b1a, SUC_lexicon_tags_15 },
    { 0x55e45b1b, SUC_lexicon_tags_63 },
    { 0x36045b1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48645b1e, SUC_lexicon_tags_63 },
    { 0x8e445b1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece45b23, SUC_lexicon_tags_15 },
    { 0xe2445b24, SUC_lexicon_tags_3 },
    { 0x92845b23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be45b2a, SUC_lexicon_tags_33 },
    { 0x87c45b2b, SUC_lexicon_tags_30 },
    { 0x34245b2c, SUC_lexicon_tags_0 },
    { 0xae845b2d, SUC_lexicon_tags_85 },
    { 0xcf845b2d, SUC_lexicon_tags_37 },
    { 0xabe45b2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xede45b32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb045b35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72c45b37, SUC_lexicon_tags_17 },
    { 0xf9845b38, SUC_lexicon_tags_7 },
    { 0xa6e45b39, SUC_lexicon_tags_15 },
    { 0x6d245b3a, SUC_lexicon_tags_11 },
    { 0x72045b3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7a45b3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf5845b3f, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x6b645b41, SUC_lexicon_tags_1 },
    { 0xf0845b42, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6b445b44, SUC_lexicon_tags_17 },
    { 0x3f845b45, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc45b4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x81645b51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca845b54, SUC_lexicon_tags_2 },
    { 0xd3645b54, SUC_lexicon_tags_14 },
    { 0x3a45b56, SUC_lexicon_tags_14 },
    { 0x53e45b56, SUC_lexicon_tags_0 },
    { 0xe6845b58, SUC_lexicon_tags_25 },
    { 0x77045b59, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a45b5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec45b5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e45b63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84e45b65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9845b67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e45b6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c645b78, SUC_lexicon_tags_47 },
    { 0xe8445b78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a045b7f, SUC_lexicon_tags_37 },
    { 0x51845b7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc9045b82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e45b85, SUC_lexicon_tags_32 },
    { 0x38645b86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c45b89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd245b8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83e45b8f, SUC_lexicon_tags_60 },
    { 0x37845b90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50045b93, SUC_lexicon_tags_25 },
    { 0xa3045b94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e645b9b, SUC_lexicon_tags_28 },
    { 0x98845b9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe445ba1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ba45ba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe45ba5, SUC_lexicon_tags_63 },
    { 0xee045ba6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69045ba9, SUC_lexicon_tags_9 },
    { 0x68e45baa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc45bae, SUC_lexicon_tags_12 },
    { 0xa4a45baf, SUC_lexicon_tags_39 },
    { 0x5cc45baf, SUC_lexicon_tags_38 },
    { 0x48c45bb1, SUC_lexicon_tags_1 },
    { 0x8c245bae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e445bb4, SUC_lexicon_tags_68 },
    { 0x8a845bb5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15645bbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78c45bbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c45bc2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa45bc5, SUC_lexicon_tags_47 },
    { 0xc645bc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x72045bc9, SUC_lexicon_tags_47 },
    { 0xef245bca, SUC_lexicon_tags_47 },
    { 0x2cc45bc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e245bcd, SUC_lexicon_tags_38 },
    { 0xb0445bce, SUC_lexicon_tags_2 },
    { 0x56645bce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e45bd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf045bd4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x47845bd6, SUC_lexicon_tags_1 },
    { 0xd5645bd7, SUC_lexicon_tags_39 },
    { 0x98045bd6, SUC_lexicon_tags_43 },
    { 0xb0245bd9, SUC_lexicon_tags_15 },
    { 0xa245bd9, SUC_lexicon_tags_12 },
    { 0x26c45bda, SUC_lexicon_tags_17 },
    { 0x4c245bdc, SUC_lexicon_tags_25 },
    { 0x2ac45bdb, SUC_lexicon_tags_1 },
    { 0x2c445bd9, SUC_lexicon_tags_15 },
    { 0x7c045bdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae45be2, SUC_lexicon_tags_11 },
    { 0xfec45be3, SUC_lexicon_tags_60 },
    { 0x93a45be4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x27245be6, SUC_lexicon_tags_15 },
    { 0x10a45be7, SUC_lexicon_tags_68 },
    { 0xabc45be7, SUC_lexicon_tags_37 },
    { 0xc7245be9, SUC_lexicon_tags_31 },
    { 0x48245be9, SUC_lexicon_tags_81 },
    { 0x7aa45be8, SUC_lexicon_tags_1 },
    { 0x87a45bec, SUC_lexicon_tags_27 },
    { 0x8de45bec, SUC_lexicon_tags_14 },
    { 0xece45bee, SUC_lexicon_tags_19 },
    { 0xde245beb, SUC_lexicon_tags_0 },
    { 0x77245bf0, SUC_lexicon_tags_92 },
    { 0x7d245bf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad245bf3, SUC_lexicon_tags_17 },
    { 0x29645bf3, SUC_lexicon_tags_10 },
    { 0x9445bf5, SUC_lexicon_tags_27 },
    { 0xb7845bf6, SUC_lexicon_tags_25 },
    { 0x22245bf5, SUC_lexicon_tags_47 },
    { 0x88e45bf5, SUC_lexicon_tags_15 },
    { 0x6aa45bf9, SUC_lexicon_tags_28 },
    { 0xcaa45bfa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4fe45bfc, SUC_lexicon_tags_7 },
    { 0xc3845bfd, SUC_lexicon_tags_106 },
    { 0x10a45bfe, SUC_lexicon_tags_12 },
    { 0x51045bfe, SUC_lexicon_tags_63 },
    { 0x88845c00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d045c0c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x44c45c0e, SUC_lexicon_tags_6 },
    { 0x42a45c0f, SUC_lexicon_tags_19 },
    { 0xa1045c0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd645c12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb045c15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc45c18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e45c1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c045c20, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a45c23, SUC_lexicon_tags_8 },
    { 0x45845c23, SUC_lexicon_tags_16 },
    { 0x37645c23, SUC_lexicon_tags_48 },
    { 0x4dc45c26, SUC_lexicon_tags_37 },
    { 0x22845c25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35845c29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb045c2b, SUC_lexicon_tags_17 },
    { 0x20c45c2c, SUC_lexicon_tags_9 },
    { 0x82e45c2c, SUC_lexicon_tags_31 },
    { 0x82445c2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb045c30, SUC_lexicon_tags_19 },
    { 0xce445c31, SUC_lexicon_tags_47 },
    { 0xe2245c32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cc45c34, SUC_lexicon_tags_178 },
    { 0xc0245c35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a45c3b, SUC_lexicon_tags_43 },
    { 0x3f245c3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e45c41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x89e45c43, SUC_lexicon_tags_15 },
    { 0x5e445c43, SUC_lexicon_tags_45 },
    { 0x11045c43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf445c47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90045c4a, SUC_lexicon_tags_17 },
    { 0x5fa45c4a, SUC_lexicon_tags_17 },
    { 0xc4a45c4c, SUC_lexicon_tags_12 },
    { 0x41845c4d, SUC_lexicon_tags_3 },
    { 0xd6645c4d, SUC_lexicon_tags_11 },
    { 0x22845c4f, SUC_lexicon_tags_11 },
    { 0xdaa45c4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57845c52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb845c54, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae45c57, SUC_lexicon_tags_1 },
    { 0xa7845c58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38845c5a, SUC_lexicon_tags_30 },
    { 0xe1045c5b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x58445c5e, SUC_lexicon_tags_47 },
    { 0x40245c5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7845c61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1e45c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20045c65, SUC_lexicon_tags_125 },
    { 0x84045c65, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x82645c68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73045c6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae45c74, SUC_lexicon_tags_46 },
    { 0xe1645c74, SUC_lexicon_tags_17 },
    { 0x4f845c76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d845c79, SUC_lexicon_tags_16 },
    { 0xfbe45c7a, SUC_lexicon_tags_7 },
    { 0x7e245c7b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1645c7e, SUC_lexicon_tags_60 },
    { 0xd1245c7f, SUC_lexicon_tags_33 },
    { 0xb4445c80, SUC_lexicon_tags_27 },
    { 0x67045c81, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9f245c83, SUC_lexicon_tags_25 },
    { 0x29045c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa45c87, SUC_lexicon_tags_14 },
    { 0x1da45c88, SUC_lexicon_tags_11 },
    { 0x88c45c89, SUC_lexicon_tags_15 },
    { 0x4ec45c8a, SUC_lexicon_tags_2 },
    { 0xa45c88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe845c8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6845c92, SUC_lexicon_tags_13 },
    { 0x6de45c92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae45c97, SUC_lexicon_tags_9 },
    { 0xda245c98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8245c9a, SUC_lexicon_tags_37 },
    { 0x5c45c9a, SUC_lexicon_tags_60 },
    { 0xabe45c9b, SUC_lexicon_tags_26 },
    { 0xb7e45c9d, SUC_lexicon_tags_15 },
    { 0xaa245c9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e45ca5, SUC_lexicon_tags_7 },
    { 0x4d845ca6, SUC_lexicon_tags_25 },
    { 0x1ce45ca5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a45cab, SUC_lexicon_tags_31 },
    { 0x45c45cac, SUC_lexicon_tags_31 },
    { 0xd7245cad, SUC_lexicon_tags_37 },
    { 0x50845cad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea645cb4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89845cbd, SUC_lexicon_tags_1 },
    { 0xefc45cbe, SUC_lexicon_tags_27 },
    { 0xd5245cbd, SUC_lexicon_tags_25 },
    { 0xf4c45cc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7645cc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f445cc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48245cc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe045ccb, SUC_lexicon_tags_1 },
    { 0xa1245ccc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c845cd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c445cd7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x16845cd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4445cdb, SUC_lexicon_tags_11 },
    { 0x22445cdc, SUC_lexicon_tags_46 },
    { 0x81c45cdb, SUC_lexicon_tags_33 },
    { 0x32245cdb, SUC_lexicon_tags_30 },
    { 0xa7645cdf, SUC_lexicon_tags_31 },
    { 0x4e645ce0, SUC_lexicon_tags_15 },
    { 0x2b445cde, SUC_lexicon_tags_23 },
    { 0xdce45cdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f445ce4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1645ce6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x14645ce8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac45ceb, SUC_lexicon_tags_65 },
    { 0xf8c45ceb, SUC_lexicon_tags_25 },
    { 0x45045ced, SUC_lexicon_tags_79 },
    { 0xb0645cee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5445cf3, SUC_lexicon_tags_25 },
    { 0x445cf3, SUC_lexicon_tags_6 },
    { 0xea445cf4, SUC_lexicon_tags_25 },
    { 0x85445cf3, SUC_lexicon_tags_11 },
    { 0xc2645cf7, SUC_lexicon_tags_7 },
    { 0x94c45cf8, SUC_lexicon_tags_60 },
    { 0xb7e45cf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36845cff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a45d02, SUC_lexicon_tags_1 },
    { 0xc3845d03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc445d07, SUC_lexicon_tags_7 },
    { 0xbd645d07, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e45d11, SUC_lexicon_tags_7 },
    { 0xeac45d12, SUC_lexicon_tags_65 },
    { 0xe4045d13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf1645d15, SUC_lexicon_tags_17 },
    { 0x3fe45d16, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3de45d18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e45d1c, SUC_lexicon_tags_7 },
    { 0xa3845d1c, SUC_lexicon_tags_63 },
    { 0xda045d1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde645d21, SUC_lexicon_tags_39 },
    { 0x24c45d22, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e45d25, SUC_lexicon_tags_47 },
    { 0xe1445d26, SUC_lexicon_tags_37 },
    { 0x8c245d27, SUC_lexicon_tags_23 },
    { 0x84645d26, SUC_lexicon_tags_17 },
    { 0x58645d28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46845d3d, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x2de45d3f, SUC_lexicon_tags_17 },
    { 0xb3c45d40, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e45d43, SUC_lexicon_tags_0 },
    { 0x7b645d44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8a45d46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc045d48, SUC_lexicon_tags_65 },
    { 0x7445d49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15045d4f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x92845d52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17845d55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42845d57, SUC_lexicon_tags_35 },
    { 0x1d245d57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2645d5b, SUC_lexicon_tags_11 },
    { 0x2dc45d5c, SUC_lexicon_tags_47 },
    { 0x6be45d5d, SUC_lexicon_tags_13 },
    { 0x67845d5e, SUC_lexicon_tags_25 },
    { 0x90245d5f, SUC_lexicon_tags_11 },
    { 0xc9645d5f, SUC_lexicon_tags_27 },
    { 0xbda45d61, SUC_lexicon_tags_68 },
    { 0xd4a45d62, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc245d67, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x57245d69, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f445d72, SUC_lexicon_tags_25 },
    { 0xd9245d73, SUC_lexicon_tags_0 },
    { 0x4fc45d74, SUC_lexicon_tags_29 },
    { 0x16a45d75, SUC_lexicon_tags_49 },
    { 0xf1845d76, SUC_lexicon_tags_19 },
    { 0x3e245d77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d445d79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14445d7d, SUC_lexicon_tags_68 },
    { 0x4c245d7e, SUC_lexicon_tags_23 },
    { 0xf8245d7e, SUC_lexicon_tags_35 },
    { 0xe9845d7d, SUC_lexicon_tags_37 },
    { 0x32845d7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17645d84, SUC_lexicon_tags_11 },
    { 0x8ac45d85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xec445d88, SUC_lexicon_tags_13 },
    { 0x2c445d89, SUC_lexicon_tags_12 },
    { 0xe7845d89, SUC_lexicon_tags_47 },
    { 0xe8045d89, SUC_lexicon_tags_28 },
    { 0xcde45d8a, SUC_lexicon_tags_47 },
    { 0xc6e45d8d, SUC_lexicon_tags_15 },
    { 0xf8445d8b, SUC_lexicon_tags_65 },
    { 0x2c045d8f, SUC_lexicon_tags_55 },
    { 0xb2845d8f, SUC_lexicon_tags_7 },
    { 0xfd645d90, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5245d95, SUC_lexicon_tags_23 },
    { 0x9ca45d96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde45d9b, SUC_lexicon_tags_17 },
    { 0x80c45d9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7445d9f, SUC_lexicon_tags_2 },
    { 0xab445da0, SUC_lexicon_tags_34 },
    { 0x70e45da1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc45da4, SUC_lexicon_tags_9 },
    { 0x34a45da5, SUC_lexicon_tags_4 },
    { 0xf0645da5, SUC_lexicon_tags_25 },
    { 0xd3e45da7, SUC_lexicon_tags_47 },
    { 0xfbc45da8, SUC_lexicon_tags_23 },
    { 0x6b045da7, SUC_lexicon_tags_8 },
    { 0x61845daa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3a45dac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4045daf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x46845db2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbde45db4, SUC_lexicon_tags_9 },
    { 0x6a045db5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29045dba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a45dc1, SUC_lexicon_tags_11 },
    { 0x9f245dc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a45dc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa845dca, SUC_lexicon_tags_15 },
    { 0x23645dcb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e45dce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a45dd1, SUC_lexicon_tags_15 },
    { 0xcb245dd2, SUC_lexicon_tags_2 },
    { 0x52845dd3, SUC_lexicon_tags_17 },
    { 0x14245dd4, SUC_lexicon_tags_12 },
    { 0xab445dd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85245dd8, SUC_lexicon_tags_7 },
    { 0x21245dd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5245ddc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa45de1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e45dea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e45df1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa645df6, SUC_lexicon_tags_7 },
    { 0x58645df7, SUC_lexicon_tags_37 },
    { 0x37c45df7, SUC_lexicon_tags_12 },
    { 0x9f045df6, SUC_lexicon_tags_26 },
    { 0x80445dfa, SUC_lexicon_tags_11 },
    { 0x72245dfb, SUC_lexicon_tags_21 },
    { 0xa3045df7, SUC_lexicon_tags_7 },
    { 0x3ca45dfd, SUC_lexicon_tags_65 },
    { 0xe6645dfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a45e03, SUC_lexicon_tags_17 },
    { 0x94045e04, SUC_lexicon_tags_28 },
    { 0x61e45e05, SUC_lexicon_tags_9 },
    { 0x42045e06, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3245e0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac45e0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6845e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ca45e15, SUC_lexicon_tags_26 },
    { 0xc3445e15, SUC_lexicon_tags_43 },
    { 0x1f645e17, SUC_lexicon_tags_19 },
    { 0x37a45e18, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x96245e1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a45e1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7645e22, SUC_lexicon_tags_25 },
    { 0xd7045e23, SUC_lexicon_tags_35 },
    { 0x61245e24, SUC_lexicon_tags_37 },
    { 0xa045e24, SUC_lexicon_tags_33 },
    { 0x80a45e26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee45e2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xefa45e2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93845e30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8645e36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c45e3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xda45e3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48645e3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b445e41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a45e4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba45e51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaea45e53, SUC_lexicon_tags_37 },
    { 0x4f445e54, SUC_lexicon_tags_17 },
    { 0xd045e54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e45e59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4245e62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32245e66, SUC_lexicon_tags_47 },
    { 0x80645e67, SUC_lexicon_tags_7 },
    { 0xe4645e68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4f445e6a, SUC_lexicon_tags_45 },
    { 0xcac45e6b, SUC_lexicon_tags_1 },
    { 0xfde45e6c, SUC_lexicon_tags_38 },
    { 0x8b245e6b, SUC_lexicon_tags_62 },
    { 0xcee45e6e, SUC_lexicon_tags_19 },
    { 0x6f245e6e, SUC_lexicon_tags_9 },
    { 0x70645e70, SUC_lexicon_tags_57 },
    { 0x7f845e6f, SUC_lexicon_tags_30 },
    { 0x12c45e72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd045e75, SUC_lexicon_tags_56 },
    { 0x8245e76, SUC_lexicon_tags_1 },
    { 0xcc445e77, SUC_lexicon_tags_17 },
    { 0xe6a45e78, SUC_lexicon_tags_11 },
    { 0x92045e77, SUC_lexicon_tags_43 },
    { 0x1a445e7a, SUC_lexicon_tags_33 },
    { 0x1fa45e76, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61245e81, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb045e87, SUC_lexicon_tags_0 },
    { 0x9c45e87, SUC_lexicon_tags_8 },
    { 0xb3c45e89, SUC_lexicon_tags_15 },
    { 0xd045e8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc845e8e, SUC_lexicon_tags_49 },
    { 0x34045e8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4045e95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe45e97, SUC_lexicon_tags_25 },
    { 0x75045e98, SUC_lexicon_tags_7 },
    { 0xcc045e98, SUC_lexicon_tags_14 },
    { 0x99645e98, SUC_lexicon_tags_27 },
    { 0xde245e98, SUC_lexicon_tags_15 },
    { 0xfd045e99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0245e9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1645ea1, SUC_lexicon_tags_4 },
    { 0x8e245ea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99245ea5, SUC_lexicon_tags_1 },
    { 0x5045ea6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74245ea9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x42245eab, SUC_lexicon_tags_28 },
    { 0x10e45eac, SUC_lexicon_tags_37 },
    { 0xf8245eac, SUC_lexicon_tags_17 },
    { 0xdba45eae, SUC_lexicon_tags_11 },
    { 0x63e45eaf, SUC_lexicon_tags_1 },
    { 0x30a45eb0, SUC_lexicon_tags_9 },
    { 0x93c45eb0, SUC_lexicon_tags_26 },
    { 0xe7a45ead, SUC_lexicon_tags_7 },
    { 0x4fa45eb3, SUC_lexicon_tags_31 },
    { 0xcc445eb0, SUC_lexicon_tags_6 },
    { 0x75645eac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48045eba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a45ebd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b845ec2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26245ec6, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c45ec9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xed245ecb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37045ed6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4a45ed8, SUC_lexicon_tags_17 },
    { 0xafe45ed9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a845edd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1c45edf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35245ee1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x39845ee3, SUC_lexicon_tags_28 },
    { 0x27a45ee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27245eec, SUC_lexicon_tags_5 },
    { 0x6a845eec, SUC_lexicon_tags_2 },
    { 0x75a45eec, SUC_lexicon_tags_17 },
    { 0xa6245eee, SUC_lexicon_tags_45 },
    { 0x64e45ef0, SUC_lexicon_tags_17 },
    { 0x7c645ef1, SUC_lexicon_tags_34 },
    { 0x81e45ef0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x63245ef4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3445ef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d245efb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e45f00, SUC_lexicon_tags_47 },
    { 0xe045f01, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a45f04, SUC_lexicon_tags_17 },
    { 0xb8845f04, SUC_lexicon_tags_59 },
    { 0x6b645f06, SUC_lexicon_tags_7 },
    { 0x91845f06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c845f0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4045f11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x82845f13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1645f16, SUC_lexicon_tags_37 },
    { 0xaaa45f17, SUC_lexicon_tags_25 },
    { 0x68445f16, SUC_lexicon_tags_27 },
    { 0x66045f19, SUC_lexicon_tags_6 },
    { 0xfba45f19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c45f1f, SUC_lexicon_tags_9 },
    { 0xb7c45f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4de45f22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4045f24, SUC_lexicon_tags_37 },
    { 0xe6645f24, SUC_lexicon_tags_21 },
    { 0xd9845f24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b045f2b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee45f2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac45f32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2645f34, SUC_lexicon_tags_37 },
    { 0x31c45f35, SUC_lexicon_tags_17 },
    { 0x37a45f34, SUC_lexicon_tags_1 },
    { 0x7e45f37, SUC_lexicon_tags_11 },
    { 0x76a45f38, SUC_lexicon_tags_28 },
    { 0x6b645f39, SUC_lexicon_tags_17 },
    { 0x74845f39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58645f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e45f43, SUC_lexicon_tags_6 },
    { 0xd5245f44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f045f48, SUC_lexicon_tags_32 },
    { 0xa5445f49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a845f4b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x2f445f4d, SUC_lexicon_tags_2 },
    { 0x42445f4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c45f52, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac45f58, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c45f5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4045f5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e45f62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e45f65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e645f68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd445f6b, SUC_lexicon_tags_38 },
    { 0xb8c45f6c, SUC_lexicon_tags_11 },
    { 0xdec45f6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23045f70, SUC_lexicon_tags_6 },
    { 0x67c45f71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17645f75, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee45f7a, SUC_lexicon_tags_42 },
    { 0x82a45f7b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5245f7d, SUC_lexicon_tags_14 },
    { 0x44e45f7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa45f81, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa2e45f83, SUC_lexicon_tags_7 },
    { 0x75c45f84, SUC_lexicon_tags_6 },
    { 0x4dc45f85, SUC_lexicon_tags_25 },
    { 0xc0645f85, SUC_lexicon_tags_57 },
    { 0x3aa45f84, SUC_lexicon_tags_12 },
    { 0x6ae45f88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a845f8d, SUC_lexicon_tags_27 },
    { 0x24645f8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a45f91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a45f98, SUC_lexicon_tags_11 },
    { 0x9de45f99, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e45f9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e45fa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc45fa4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc045fab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb045fae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11045fb0, SUC_lexicon_tags_27 },
    { 0x11245fb1, SUC_lexicon_tags_8 },
    { 0x75245fb2, SUC_lexicon_tags_82 },
    { 0x8a645fb3, SUC_lexicon_tags_158 },
    { 0xbf445fb4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c245fb6, SUC_lexicon_tags_33 },
    { 0x95e45fb7, SUC_lexicon_tags_64 },
    { 0x4b845fb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a45fbc, SUC_lexicon_tags_37 },
    { 0x7a245fbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7245fbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94445fc2, SUC_lexicon_tags_64 },
    { 0x2045fc2, SUC_lexicon_tags_23 },
    { 0xee645fc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b845fc7, SUC_lexicon_tags_11 },
    { 0x73e45fc8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x30245fca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e45fcd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x47445fcf, SUC_lexicon_tags_27 },
    { 0x9fa45fcf, SUC_lexicon_tags_25 },
    { 0x37245fd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7c45fd3, SUC_lexicon_tags_12 },
    { 0xf4245fd4, SUC_lexicon_tags_11 },
    { 0xe9a45fd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb045fda, SUC_lexicon_tags_23 },
    { 0xd4645fda, SUC_lexicon_tags_47 },
    { 0x41a45fdc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d245fde, SUC_lexicon_tags_7 },
    { 0x41045fde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37445fe1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6a845fe3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x27245fe5, SUC_lexicon_tags_60 },
    { 0x44845fe6, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0x26645fe9, SUC_lexicon_tags_2 },
    { 0x96c45fe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d045fed, SUC_lexicon_tags_45 },
    { 0x4ec45fed, SUC_lexicon_tags_17 },
    { 0x3ba45fed, SUC_lexicon_tags_12 },
    { 0x14c45ff0, SUC_lexicon_tags_7 },
    { 0x5a245ff1, SUC_lexicon_tags_65 },
    { 0x38645ff2, SUC_lexicon_tags_37 },
    { 0xf645ff3, SUC_lexicon_tags_11 },
    { 0xcac45fed, SUC_lexicon_tags_29 },
    { 0xe6e45ff5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3a45ff7, SUC_lexicon_tags_8 },
    { 0x86445ff8, SUC_lexicon_tags_19 },
    { 0x63045ff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c46007, SUC_lexicon_tags_17 },
    { 0x15646008, SUC_lexicon_tags_0 },
    { 0x53446007, SUC_lexicon_tags_28 },
    { 0x2924600a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d846011, SUC_lexicon_tags_37 },
    { 0x50246012, SUC_lexicon_tags_16 },
    { 0x27a46012, SUC_lexicon_tags_1 },
    { 0x60446014, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50446021, SUC_lexicon_tags_17 },
    { 0xd5a46022, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c46025, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc246028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c64602b, SUC_lexicon_tags_82 },
    { 0xf0e4602c, SUC_lexicon_tags_14 },
    { 0x1664602c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x62446030, SUC_lexicon_tags_15 },
    { 0x33e46030, SUC_lexicon_tags_47 },
    { 0x81846032, SUC_lexicon_tags_13 },
    { 0x12c46031, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60246037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe84603a, SUC_lexicon_tags_45 },
    { 0x2324603b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2464603d, SUC_lexicon_tags_0 },
    { 0xb1a4603e, SUC_lexicon_tags_12 },
    { 0x1ca4603f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b64604f, SUC_lexicon_tags_0 },
    { 0xf964604f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5446052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47046056, SUC_lexicon_tags_17 },
    { 0x65446057, SUC_lexicon_tags_14 },
    { 0xb5a46057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee4605b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x72a4605d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdf84605f, SUC_lexicon_tags_15 },
    { 0x2a04605f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64246063, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x80846066, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f84606c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a4606f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba46072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee846074, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x54246076, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae4607a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27046080, SUC_lexicon_tags_7 },
    { 0x4dc46080, SUC_lexicon_tags_63 },
    { 0x6fe46080, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e4608a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xace4608c, SUC_lexicon_tags_17 },
    { 0x82e4608d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17e4608f, SUC_lexicon_tags_95 },
    { 0xac46090, SUC_lexicon_tags_17 },
    { 0x8fa46091, SUC_lexicon_tags_27 },
    { 0x10246092, SUC_lexicon_tags_27 },
    { 0x70e46090, SUC_lexicon_tags_23 },
    { 0x2a246094, SUC_lexicon_tags_11 },
    { 0x69046095, SUC_lexicon_tags_15 },
    { 0xe9c46095, SUC_lexicon_tags_17 },
    { 0xcfc46097, SUC_lexicon_tags_114 },
    { 0x8c46098, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4b04609a, SUC_lexicon_tags_14 },
    { 0x1b64609a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe004609e, SUC_lexicon_tags_37 },
    { 0x9204609f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x132460a2, SUC_lexicon_tags_12 },
    { 0x652460a2, SUC_lexicon_tags_31 },
    { 0x654460a4, SUC_lexicon_tags_1 },
    { 0x674460a4, SUC_lexicon_tags_43 },
    { 0x104460a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18460a9, SUC_lexicon_tags_17 },
    { 0x76460aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a460b7, SUC_lexicon_tags_30 },
    { 0xf7a460b8, SUC_lexicon_tags_25 },
    { 0x6460b9, SUC_lexicon_tags_25 },
    { 0xa88460b9, SUC_lexicon_tags_18 },
    { 0x7a2460ba, SUC_lexicon_tags_37 },
    { 0x666460bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc460bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24460c2, SUC_lexicon_tags_38 },
    { 0x332460c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e460ca, SUC_lexicon_tags_29 },
    { 0xd66460cb, SUC_lexicon_tags_32 },
    { 0xd72460cb, SUC_lexicon_tags_37 },
    { 0xa18460ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7f0460cf, SUC_lexicon_tags_7 },
    { 0xb96460d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682460d4, SUC_lexicon_tags_11 },
    { 0xfa460d5, SUC_lexicon_tags_1 },
    { 0xd1a460d5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a460dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae460e0, SUC_lexicon_tags_7 },
    { 0x28a460e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9aa460e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4460e8, SUC_lexicon_tags_11 },
    { 0xa2c460e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492460ed, SUC_lexicon_tags_25 },
    { 0x794460ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98c460f0, SUC_lexicon_tags_17 },
    { 0xa8460f0, SUC_lexicon_tags_47 },
    { 0xbc8460f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a460f5, SUC_lexicon_tags_17 },
    { 0xf36460f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52460fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4460fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87446100, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c46103, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a46106, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5ba46108, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8204610b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e64610e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc646113, SUC_lexicon_tags_8 },
    { 0x3ac46113, SUC_lexicon_tags_57 },
    { 0x62046115, SUC_lexicon_tags_17 },
    { 0x55046114, SUC_lexicon_tags_847 },
    { 0xf9a46114, SUC_lexicon_tags_21 },
    { 0x60046118, SUC_lexicon_tags_0 },
    { 0x60446117, SUC_lexicon_tags_12 },
    { 0x3e24611a, SUC_lexicon_tags_45 },
    { 0x224611b, SUC_lexicon_tags_11 },
    { 0x2484611c, SUC_lexicon_tags_23 },
    { 0x6604611c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a846120, SUC_lexicon_tags_47 },
    { 0x8c846121, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c46127, SUC_lexicon_tags_31 },
    { 0xeea46127, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e246130, SUC_lexicon_tags_23 },
    { 0x31046131, SUC_lexicon_tags_55 },
    { 0xe7846130, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d646134, SUC_lexicon_tags_1 },
    { 0x52a46135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a846137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a4613d, SUC_lexicon_tags_37 },
    { 0xe7a4613d, SUC_lexicon_tags_43 },
    { 0x3e64613e, SUC_lexicon_tags_11 },
    { 0xb984613d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e46146, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa3046148, SUC_lexicon_tags_65 },
    { 0x1c046149, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa544614e, SUC_lexicon_tags_68 },
    { 0x7904614f, SUC_lexicon_tags_25 },
    { 0x4be4614f, SUC_lexicon_tags_14 },
    { 0xfd44614e, SUC_lexicon_tags_19 },
    { 0x24e46152, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea46155, SUC_lexicon_tags_25 },
    { 0x29a46156, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c4615c, SUC_lexicon_tags_11 },
    { 0xfb64615c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab646165, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87046167, SUC_lexicon_tags_17 },
    { 0xb8a46167, SUC_lexicon_tags_33 },
    { 0x55646169, SUC_lexicon_tags_87 },
    { 0x2b84616a, SUC_lexicon_tags_47 },
    { 0xb544616b, SUC_lexicon_tags_37 },
    { 0xc8a4616a, SUC_lexicon_tags_25 },
    { 0x5924616d, SUC_lexicon_tags_23 },
    { 0x66c4616e, SUC_lexicon_tags_66 },
    { 0x3624616f, SUC_lexicon_tags_37 },
    { 0xee46168, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51046172, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada46176, SUC_lexicon_tags_7 },
    { 0x73646177, SUC_lexicon_tags_12 },
    { 0xd5846178, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd984617a, SUC_lexicon_tags_37 },
    { 0xbe84617b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x584617e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8446181, SUC_lexicon_tags_37 },
    { 0x22846181, SUC_lexicon_tags_11 },
    { 0xf5246183, SUC_lexicon_tags_11 },
    { 0x50e46184, SUC_lexicon_tags_1 },
    { 0x32c46183, SUC_lexicon_tags_11 },
    { 0x62646183, SUC_lexicon_tags_10 },
    { 0x4ae46186, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f64618a, SUC_lexicon_tags_17 },
    { 0x2e4618b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b246195, SUC_lexicon_tags_23 },
    { 0xe6646196, SUC_lexicon_tags_19 },
    { 0xcb446197, SUC_lexicon_tags_8 },
    { 0x22646198, SUC_lexicon_tags_27 },
    { 0xfc46198, SUC_lexicon_tags_14 },
    { 0x10846196, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e4619d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa484619f, SUC_lexicon_tags_16 },
    { 0xbe8461a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4461a4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa56461a6, SUC_lexicon_tags_12 },
    { 0x42a461a6, SUC_lexicon_tags_6 },
    { 0xbb6461a8, SUC_lexicon_tags_63 },
    { 0xea4461a8, SUC_lexicon_tags_19 },
    { 0x704461aa, SUC_lexicon_tags_0 },
    { 0x76c461aa, SUC_lexicon_tags_45 },
    { 0x186461ac, SUC_lexicon_tags_12 },
    { 0xb20461ad, SUC_lexicon_tags_51 },
    { 0xbbe461ad, SUC_lexicon_tags_0 },
    { 0x744461ad, SUC_lexicon_tags_27 },
    { 0x396461b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c461bc, SUC_lexicon_tags_107 },
    { 0x394461bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62a461bf, SUC_lexicon_tags_28 },
    { 0x7aa461c0, SUC_lexicon_tags_1 },
    { 0x958461c1, SUC_lexicon_tags_2 },
    { 0xec6461c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc28461c4, SUC_lexicon_tags_2 },
    { 0xba0461c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x922461c7, SUC_lexicon_tags_25 },
    { 0xe8461c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfe461ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736461d0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x450461d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac461d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x90461d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12461dc, SUC_lexicon_tags_40 },
    { 0x828461dd, SUC_lexicon_tags_23 },
    { 0xeb6461dd, SUC_lexicon_tags_38 },
    { 0xa8e461dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ae461e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x252461e3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x36e461e5, SUC_lexicon_tags_43 },
    { 0xdcc461e5, SUC_lexicon_tags_11 },
    { 0x4a4461e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c461eb, SUC_lexicon_tags_15 },
    { 0xd08461ec, SUC_lexicon_tags_7 },
    { 0x1be461ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e461f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80461f5, SUC_lexicon_tags_0 },
    { 0x40a461f5, SUC_lexicon_tags_65 },
    { 0x83a461f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d6461f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ea461fb, SUC_lexicon_tags_0 },
    { 0xfdc461fc, SUC_lexicon_tags_77 },
    { 0x88461fd, SUC_lexicon_tags_23 },
    { 0x684461fe, SUC_lexicon_tags_9 },
    { 0x43a461fd, SUC_lexicon_tags_4 },
    { 0x44c46200, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee46208, SUC_lexicon_tags_37 },
    { 0x64446209, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a4620e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb246211, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf246217, SUC_lexicon_tags_70 },
    { 0x11646217, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c46220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd446223, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7446227, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e04622a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba04622d, SUC_lexicon_tags_51 },
    { 0x1b24622e, SUC_lexicon_tags_84 },
    { 0x79a4622f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82046231, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5846235, SUC_lexicon_tags_11 },
    { 0x10846236, SUC_lexicon_tags_28 },
    { 0x4a46235, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc4623a, SUC_lexicon_tags_25 },
    { 0x9144623b, SUC_lexicon_tags_21 },
    { 0xce84623c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e46241, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x12c46243, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e446247, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6a46249, SUC_lexicon_tags_27 },
    { 0xe284624a, SUC_lexicon_tags_30 },
    { 0x1ee4624b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a24624d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba64624f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72046251, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1246253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f046255, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e46259, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a84625c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a046260, SUC_lexicon_tags_27 },
    { 0x71446261, SUC_lexicon_tags_0 },
    { 0x11846260, SUC_lexicon_tags_30 },
    { 0x2d446263, SUC_lexicon_tags_46 },
    { 0xc7246264, SUC_lexicon_tags_43 },
    { 0x89e46265, SUC_lexicon_tags_19 },
    { 0x18846266, SUC_lexicon_tags_26 },
    { 0xefc46263, SUC_lexicon_tags_47 },
    { 0xdb446268, SUC_lexicon_tags_65 },
    { 0x20846269, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc04626f, SUC_lexicon_tags_79 },
    { 0x7e24626f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c46276, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x11246279, SUC_lexicon_tags_32 },
    { 0xc4446279, SUC_lexicon_tags_12 },
    { 0x4544627a, SUC_lexicon_tags_16 },
    { 0x6a44627a, SUC_lexicon_tags_17 },
    { 0xbae4627b, SUC_lexicon_tags_0 },
    { 0x5c84627e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a46282, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb0246284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ec46286, SUC_lexicon_tags_47 },
    { 0xbf646286, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeec46289, SUC_lexicon_tags_15 },
    { 0x20a4628a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5044628d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c46292, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b646295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc46299, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa204629b, SUC_lexicon_tags_19 },
    { 0xcb24629c, SUC_lexicon_tags_13 },
    { 0x61a4629b, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2462a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66a462a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d4462a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0462a7, SUC_lexicon_tags_17 },
    { 0x80c462a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x95a462aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e462ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x960462b1, SUC_lexicon_tags_43 },
    { 0x2e4462b2, SUC_lexicon_tags_68 },
    { 0xbcc462b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc462b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be462bb, SUC_lexicon_tags_17 },
    { 0xa10462bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4462c2, SUC_lexicon_tags_21 },
    { 0x21a462c3, SUC_lexicon_tags_29 },
    { 0x6c4462c4, SUC_lexicon_tags_29 },
    { 0xdc462c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe52462c7, SUC_lexicon_tags_27 },
    { 0x74c462c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2c462ca, SUC_lexicon_tags_9 },
    { 0x3ac462cb, SUC_lexicon_tags_37 },
    { 0x802462cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34462d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e462d5, SUC_lexicon_tags_1 },
    { 0xf04462d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222462db, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba462df, SUC_lexicon_tags_12 },
    { 0x12462e0, SUC_lexicon_tags_10 },
    { 0xa80462e1, SUC_lexicon_tags_11 },
    { 0xf64462e2, SUC_lexicon_tags_37 },
    { 0x1e4462e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0462e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8462eb, SUC_lexicon_tags_37 },
    { 0x60462ec, SUC_lexicon_tags_15 },
    { 0xec2462ec, SUC_lexicon_tags_15 },
    { 0xa7c462ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5de462f0, SUC_lexicon_tags_19 },
    { 0x336462f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c2462f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e462f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4462fb, SUC_lexicon_tags_92 },
    { 0x37a462fb, SUC_lexicon_tags_60 },
    { 0xd0c462fc, SUC_lexicon_tags_35 },
    { 0x1d4462fd, SUC_lexicon_tags_37 },
    { 0xe34462ff, SUC_lexicon_tags_25 },
    { 0x9fa46300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71646302, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84646307, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a4630b, SUC_lexicon_tags_23 },
    { 0x5cc4630c, SUC_lexicon_tags_17 },
    { 0x2c84630b, SUC_lexicon_tags_87 },
    { 0x5924630b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea846316, SUC_lexicon_tags_29 },
    { 0xfb646317, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a4631a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb884631c, SUC_lexicon_tags_45 },
    { 0xe144631d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1446321, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9646325, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc46329, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8084632c, SUC_lexicon_tags_12 },
    { 0x6b64632c, SUC_lexicon_tags_23 },
    { 0x2404632d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72a46330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1c46332, SUC_lexicon_tags_37 },
    { 0x6bc46332, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca446336, SUC_lexicon_tags_11 },
    { 0x66a46337, SUC_lexicon_tags_65 },
    { 0x4c846338, SUC_lexicon_tags_17 },
    { 0xc7046339, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf564633d, SUC_lexicon_tags_47 },
    { 0x3224633e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c46341, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a46343, SUC_lexicon_tags_40 },
    { 0x20246343, SUC_lexicon_tags_23 },
    { 0xfa846345, SUC_lexicon_tags_37 },
    { 0xf1446346, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc4634b, SUC_lexicon_tags_48 },
    { 0xbde4634c, SUC_lexicon_tags_1 },
    { 0x3864634b, SUC_lexicon_tags_0 },
    { 0x304634e, SUC_lexicon_tags_33 },
    { 0xecc4634b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc246354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e46358, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa764635a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4635e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27c46360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15846365, SUC_lexicon_tags_47 },
    { 0xd6246365, SUC_lexicon_tags_9 },
    { 0xee846367, SUC_lexicon_tags_45 },
    { 0x4e046366, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x29a4636a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab44636c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf04636f, SUC_lexicon_tags_17 },
    { 0x14e4636f, SUC_lexicon_tags_84 },
    { 0xce846371, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30846378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb64637a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba246380, SUC_lexicon_tags_15 },
    { 0xc0646380, SUC_lexicon_tags_37 },
    { 0x67e46382, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd9846384, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0246389, SUC_lexicon_tags_29 },
    { 0xb884638a, SUC_lexicon_tags_25 },
    { 0x6d046389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae46393, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6904639a, SUC_lexicon_tags_26 },
    { 0x1e04639a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee4639e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30463a1, SUC_lexicon_tags_11 },
    { 0x9e463a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48c463a4, SUC_lexicon_tags_66 },
    { 0xba2463a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d2463a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe463ab, SUC_lexicon_tags_25 },
    { 0xf28463ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb48463ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x234463b1, SUC_lexicon_tags_11 },
    { 0xdd0463b1, SUC_lexicon_tags_37 },
    { 0xdd2463b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x784463b6, SUC_lexicon_tags_25 },
    { 0xf92463b7, SUC_lexicon_tags_37 },
    { 0x2c4463b8, SUC_lexicon_tags_63 },
    { 0x2c2463b9, SUC_lexicon_tags_26 },
    { 0xa90463b7, SUC_lexicon_tags_37 },
    { 0x73c463b8, SUC_lexicon_tags_49 },
    { 0x158463b9, SUC_lexicon_tags_10 },
    { 0xa84463bd, SUC_lexicon_tags_11 },
    { 0x56a463bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64463c1, SUC_lexicon_tags_47 },
    { 0xf04463c2, SUC_lexicon_tags_17 },
    { 0x190463c3, SUC_lexicon_tags_140 },
    { 0x366463c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x722463c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50463d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70463d3, SUC_lexicon_tags_43 },
    { 0xaa463d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e463d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x518463d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c463de, SUC_lexicon_tags_15 },
    { 0x99c463de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x352463e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb2a463e4, SUC_lexicon_tags_12 },
    { 0x842463e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcd8463e7, SUC_lexicon_tags_26 },
    { 0x106463e8, SUC_lexicon_tags_15 },
    { 0x4e2463e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a463ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786463f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6463f9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xddc463fb, SUC_lexicon_tags_7 },
    { 0xe70463fb, SUC_lexicon_tags_2 },
    { 0x1b8463fb, SUC_lexicon_tags_65 },
    { 0x19c463fd, SUC_lexicon_tags_17 },
    { 0x66c463ff, SUC_lexicon_tags_77 },
    { 0x638463fc, SUC_lexicon_tags_45 },
    { 0xe66463ff, SUC_lexicon_tags_30 },
    { 0xc56463fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x32a46404, SUC_lexicon_tags_2 },
    { 0xa6846405, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b846410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3246414, SUC_lexicon_tags_71 },
    { 0xca46415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf246417, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a44641d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3646423, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd8646425, SUC_lexicon_tags_33 },
    { 0xf3646425, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c04642b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c24642e, SUC_lexicon_tags_19 },
    { 0x9664642e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63046431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8046433, SUC_lexicon_tags_26 },
    { 0x21a46433, SUC_lexicon_tags_9 },
    { 0x67846433, SUC_lexicon_tags_11 },
    { 0x69846436, SUC_lexicon_tags_14 },
    { 0x4e246434, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f446439, SUC_lexicon_tags_12 },
    { 0x9644643a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x964643d, SUC_lexicon_tags_2 },
    { 0xc984643e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdc46440, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8446442, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e46448, SUC_lexicon_tags_59 },
    { 0xa046448, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x19e4644b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61646458, SUC_lexicon_tags_0 },
    { 0xe4e46459, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e4645e, SUC_lexicon_tags_19 },
    { 0x1b24645f, SUC_lexicon_tags_23 },
    { 0x59646460, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e646465, SUC_lexicon_tags_45 },
    { 0xf8e46466, SUC_lexicon_tags_1 },
    { 0x30046467, SUC_lexicon_tags_28 },
    { 0xe2a46468, SUC_lexicon_tags_12 },
    { 0x3fa46468, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2046470, SUC_lexicon_tags_25 },
    { 0x1646471, SUC_lexicon_tags_2 },
    { 0x6e046472, SUC_lexicon_tags_16 },
    { 0x73046471, SUC_lexicon_tags_27 },
    { 0x80c46471, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17046476, SUC_lexicon_tags_8 },
    { 0x18446477, SUC_lexicon_tags_17 },
    { 0xda646478, SUC_lexicon_tags_37 },
    { 0x63e46479, SUC_lexicon_tags_38 },
    { 0xb6a4647a, SUC_lexicon_tags_7 },
    { 0x76a46478, SUC_lexicon_tags_12 },
    { 0xd2046479, SUC_lexicon_tags_13 },
    { 0xba64647d, SUC_lexicon_tags_11 },
    { 0xc84647e, SUC_lexicon_tags_11 },
    { 0x384647f, SUC_lexicon_tags_45 },
    { 0x84c46480, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e046482, SUC_lexicon_tags_29 },
    { 0x49846482, SUC_lexicon_tags_7 },
    { 0xb8646482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1046486, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x65246489, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac4648d, SUC_lexicon_tags_2 },
    { 0xf824648e, SUC_lexicon_tags_1 },
    { 0x9584648f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaf446491, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27046493, SUC_lexicon_tags_12 },
    { 0x48246494, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff646498, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c464a0, SUC_lexicon_tags_43 },
    { 0xb66464a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x162464a3, SUC_lexicon_tags_12 },
    { 0x6b4464a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x954464a7, SUC_lexicon_tags_1 },
    { 0xf7a464a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c464b1, SUC_lexicon_tags_77 },
    { 0x84e464b1, SUC_lexicon_tags_17 },
    { 0x3fa464b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf36464b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8464b8, SUC_lexicon_tags_4 },
    { 0xf10464b9, SUC_lexicon_tags_1 },
    { 0x3c2464b8, SUC_lexicon_tags_2 },
    { 0xb2464ba, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0x50a464bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482464c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28464c5, SUC_lexicon_tags_46 },
    { 0x6d0464c6, SUC_lexicon_tags_37 },
    { 0x63c464c7, SUC_lexicon_tags_37 },
    { 0xfb4464c8, SUC_lexicon_tags_14 },
    { 0xf3a464c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x10a464cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c464ce, SUC_lexicon_tags_60 },
    { 0xe0464ce, SUC_lexicon_tags_2 },
    { 0xaa6464d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce464d7, SUC_lexicon_tags_68 },
    { 0x42a464d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x666464da, SUC_lexicon_tags_11 },
    { 0x138464db, SUC_lexicon_tags_37 },
    { 0x9e6464dc, SUC_lexicon_tags_8 },
    { 0x37e464dd, SUC_lexicon_tags_11 },
    { 0xe86464de, SUC_lexicon_tags_138 },
    { 0xe58464de, SUC_lexicon_tags_15 },
    { 0xb12464e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x51a464e2, SUC_lexicon_tags_37 },
    { 0x33a464e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe6464e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e464ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd22464ef, SUC_lexicon_tags_63 },
    { 0x374464ef, SUC_lexicon_tags_1 },
    { 0xe6a464ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x598464f3, SUC_lexicon_tags_16 },
    { 0xd76464f3, SUC_lexicon_tags_15 },
    { 0x35c464f5, SUC_lexicon_tags_30 },
    { 0x2de464f5, SUC_lexicon_tags_33 },
    { 0x462464f6, SUC_lexicon_tags_1 },
    { 0x5fe464f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x672464fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4464fd, SUC_lexicon_tags_0 },
    { 0x6cc464fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e246501, SUC_lexicon_tags_17 },
    { 0xba446502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c46507, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa924650a, SUC_lexicon_tags_17 },
    { 0xa164650a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42e4650d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a46512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a646514, SUC_lexicon_tags_0 },
    { 0xfd646515, SUC_lexicon_tags_992 },
    { 0xedc46515, SUC_lexicon_tags_60 },
    { 0x4446515, SUC_lexicon_tags_14 },
    { 0xdea46515, SUC_lexicon_tags_1 },
    { 0xb9446519, SUC_lexicon_tags_2 },
    { 0xfa646519, SUC_lexicon_tags_11 },
    { 0x23846519, SUC_lexicon_tags_17 },
    { 0x35646519, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5704651f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43e46521, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a46528, SUC_lexicon_tags_45 },
    { 0xbf646529, SUC_lexicon_tags_7 },
    { 0x2ca4652a, SUC_lexicon_tags_45 },
    { 0x9a246529, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e4652f, SUC_lexicon_tags_25 },
    { 0xf7846530, SUC_lexicon_tags_124 },
    { 0x57246531, SUC_lexicon_tags_12 },
    { 0x27a46532, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe46535, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7944653b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5046542, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e46546, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x72c46548, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e4654b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61446550, SUC_lexicon_tags_37 },
    { 0x8be46551, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc46554, SUC_lexicon_tags_25 },
    { 0x23a46555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a4655b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f64655f, SUC_lexicon_tags_23 },
    { 0xd0446560, SUC_lexicon_tags_46 },
    { 0x4784655f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32046565, SUC_lexicon_tags_13 },
    { 0x7c646566, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe6846568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4656e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa46571, SUC_lexicon_tags_12 },
    { 0xc4446571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c46576, SUC_lexicon_tags_30 },
    { 0xf0c46576, SUC_lexicon_tags_1 },
    { 0xdb246578, SUC_lexicon_tags_11 },
    { 0xb7446579, SUC_lexicon_tags_34 },
    { 0x3204657a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc84657f, SUC_lexicon_tags_437 },
    { 0xb2a46580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80846582, SUC_lexicon_tags_2 },
    { 0x5f046582, SUC_lexicon_tags_14 },
    { 0xf8c46583, SUC_lexicon_tags_9 },
    { 0xe0a46585, SUC_lexicon_tags_1 },
    { 0x59646586, SUC_lexicon_tags_44 },
    { 0x76846584, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7646589, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1846590, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x44846592, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc46596, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1646598, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58a4659a, SUC_lexicon_tags_2 },
    { 0x3264659b, SUC_lexicon_tags_178 },
    { 0x7c64659c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394465a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40465ab, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996465af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2465b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6465bc, SUC_lexicon_tags_36 },
    { 0xfbe465bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a465c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2465c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87e465cb, SUC_lexicon_tags_0 },
    { 0xd98465cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e465cf, SUC_lexicon_tags_32 },
    { 0x9fc465d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x326465d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca465dd, SUC_lexicon_tags_23 },
    { 0x3cc465de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6465e5, SUC_lexicon_tags_45 },
    { 0x3a2465e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2a465e8, SUC_lexicon_tags_11 },
    { 0x774465e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1be465eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58c465ed, SUC_lexicon_tags_25 },
    { 0xbec465ed, SUC_lexicon_tags_37 },
    { 0x8ea465ef, SUC_lexicon_tags_2 },
    { 0xb32465f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a465f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192465f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x880465f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4465fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c46601, SUC_lexicon_tags_19 },
    { 0x1ea46602, SUC_lexicon_tags_27 },
    { 0x77046602, SUC_lexicon_tags_14 },
    { 0xf4646602, SUC_lexicon_tags_11 },
    { 0xbe446605, SUC_lexicon_tags_15 },
    { 0x92246606, SUC_lexicon_tags_33 },
    { 0x5b846606, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a4660c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd04660e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce46613, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0e46615, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e04661d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5446626, SUC_lexicon_tags_17 },
    { 0x51c46627, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe864662b, SUC_lexicon_tags_47 },
    { 0xe724662c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da46630, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e46634, SUC_lexicon_tags_31 },
    { 0x2b646635, SUC_lexicon_tags_12 },
    { 0x15846636, SUC_lexicon_tags_47 },
    { 0x25046637, SUC_lexicon_tags_37 },
    { 0x4b846638, SUC_lexicon_tags_32 },
    { 0xb3846639, SUC_lexicon_tags_25 },
    { 0x5904663a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb644663d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e46640, SUC_lexicon_tags_12 },
    { 0x50a46641, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xff646643, SUC_lexicon_tags_0 },
    { 0x5f646644, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a46649, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4184664e, SUC_lexicon_tags_39 },
    { 0xb464664f, SUC_lexicon_tags_1 },
    { 0xb6846650, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x14e46652, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb9e46654, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xffe46656, SUC_lexicon_tags_15 },
    { 0x49446657, SUC_lexicon_tags_23 },
    { 0x9246658, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43c4665a, SUC_lexicon_tags_6 },
    { 0x8704665b, SUC_lexicon_tags_1 },
    { 0xa544665a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a46662, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14446664, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd924666d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe46670, SUC_lexicon_tags_3 },
    { 0x68246671, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e46674, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a846679, SUC_lexicon_tags_1 },
    { 0x644667a, SUC_lexicon_tags_15 },
    { 0xf24667b, SUC_lexicon_tags_45 },
    { 0x2dc4667a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8584667f, SUC_lexicon_tags_60 },
    { 0xbf84667f, SUC_lexicon_tags_0 },
    { 0x9f646681, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78646683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18446686, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e46689, SUC_lexicon_tags_11 },
    { 0xf804668a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c64668d, SUC_lexicon_tags_48 },
    { 0xe984668e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27846690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a646692, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33646697, SUC_lexicon_tags_66 },
    { 0x69846698, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02466a1, SUC_lexicon_tags_64 },
    { 0xd16466a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0466a5, SUC_lexicon_tags_47 },
    { 0xdde466a6, SUC_lexicon_tags_27 },
    { 0xe7a466a7, SUC_lexicon_tags_13 },
    { 0x324466a8, SUC_lexicon_tags_17 },
    { 0x4f8466a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0466ac, SUC_lexicon_tags_38 },
    { 0x4bc466ad, SUC_lexicon_tags_17 },
    { 0x2ac466ae, SUC_lexicon_tags_33 },
    { 0x6e4466af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x384466b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0466b3, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88466bd, SUC_lexicon_tags_27 },
    { 0x4be466be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x834466c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de466c6, SUC_lexicon_tags_9 },
    { 0x810466c7, SUC_lexicon_tags_25 },
    { 0x104466c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68466cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0466d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6466dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a6466de, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x892466e0, SUC_lexicon_tags_7 },
    { 0xa06466e1, SUC_lexicon_tags_12 },
    { 0x512466e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x58466e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0466eb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2466ee, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xdb6466f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c466fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86466fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7646701, SUC_lexicon_tags_70 },
    { 0xeb846702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7246705, SUC_lexicon_tags_11 },
    { 0xebc46706, SUC_lexicon_tags_45 },
    { 0x81246706, SUC_lexicon_tags_16 },
    { 0xb3446705, SUC_lexicon_tags_23 },
    { 0x99646709, SUC_lexicon_tags_129 },
    { 0x1724670a, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc284670f, SUC_lexicon_tags_19 },
    { 0x87446710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46846716, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8646719, SUC_lexicon_tags_47 },
    { 0xbc64671a, SUC_lexicon_tags_12 },
    { 0x5da4671b, SUC_lexicon_tags_2 },
    { 0x5b64671c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a46720, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x7246722, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d846725, SUC_lexicon_tags_14 },
    { 0x40246725, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68a46728, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf44672b, SUC_lexicon_tags_33 },
    { 0xf744672b, SUC_lexicon_tags_26 },
    { 0x6144672d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c46735, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c646739, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x3544673c, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xe264673e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b446742, SUC_lexicon_tags_38 },
    { 0x3a246743, SUC_lexicon_tags_47 },
    { 0x60846742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a4674a, SUC_lexicon_tags_52 },
    { 0x43e4674b, SUC_lexicon_tags_118 },
    { 0xa4c4674c, SUC_lexicon_tags_8 },
    { 0x8784674a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x76646750, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba46753, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a646756, SUC_lexicon_tags_25 },
    { 0xf6846756, SUC_lexicon_tags_1214 },
    { 0, NULL },
    { 0xa0846759, SUC_lexicon_tags_1 },
    { 0x84a4675a, SUC_lexicon_tags_38 },
    { 0x8264675b, SUC_lexicon_tags_31 },
    { 0x6984675c, SUC_lexicon_tags_17 },
    { 0x5544675c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9446760, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0xfc046762, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x52446764, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a46769, SUC_lexicon_tags_49 },
    { 0x2b64676a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48246770, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf046774, SUC_lexicon_tags_4 },
    { 0x1de46775, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d446779, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc644677d, SUC_lexicon_tags_27 },
    { 0xbd64677e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c46783, SUC_lexicon_tags_7 },
    { 0xf2446784, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8646786, SUC_lexicon_tags_64 },
    { 0x4b646787, SUC_lexicon_tags_28 },
    { 0x53246786, SUC_lexicon_tags_25 },
    { 0xfc846789, SUC_lexicon_tags_29 },
    { 0xad446787, SUC_lexicon_tags_11 },
    { 0x71a4678b, SUC_lexicon_tags_33 },
    { 0x32646789, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e46792, SUC_lexicon_tags_6 },
    { 0xb2846793, SUC_lexicon_tags_27 },
    { 0xd8c46794, SUC_lexicon_tags_17 },
    { 0x40e46793, SUC_lexicon_tags_27 },
    { 0x9ca46793, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e4679a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca24679c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe467a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc40467a2, SUC_lexicon_tags_63 },
    { 0x14a467a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce467ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x914467ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e467b1, SUC_lexicon_tags_9 },
    { 0xdf0467b2, SUC_lexicon_tags_19 },
    { 0x900467b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356467b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4467be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe467c1, SUC_lexicon_tags_25 },
    { 0x4de467c2, SUC_lexicon_tags_12 },
    { 0xb22467c2, SUC_lexicon_tags_43 },
    { 0x980467c4, SUC_lexicon_tags_15 },
    { 0xf14467c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb02467c7, SUC_lexicon_tags_191 },
    { 0xa3a467c8, SUC_lexicon_tags_49 },
    { 0xe28467c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f6467cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2467d0, SUC_lexicon_tags_14 },
    { 0x1e467d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc467d3, SUC_lexicon_tags_47 },
    { 0xfdc467d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e467d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32467e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf3e467e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x28c467ea, SUC_lexicon_tags_1 },
    { 0x138467ea, SUC_lexicon_tags_15 },
    { 0x242467ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf54467ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00467f1, SUC_lexicon_tags_25 },
    { 0x1e4467f2, SUC_lexicon_tags_14 },
    { 0xd74467f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42467f6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c467fa, SUC_lexicon_tags_22 },
    { 0xab8467fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c046806, SUC_lexicon_tags_37 },
    { 0xae446807, SUC_lexicon_tags_81 },
    { 0xb0c46807, SUC_lexicon_tags_12 },
    { 0x69846809, SUC_lexicon_tags_1 },
    { 0x64a4680a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a446815, SUC_lexicon_tags_19 },
    { 0x7dc46815, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44681a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe4681c, SUC_lexicon_tags_113 },
    { 0xe744681d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8204681f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9046822, SUC_lexicon_tags_28 },
    { 0x8e446822, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee46828, SUC_lexicon_tags_17 },
    { 0x90846829, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8504682c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3044682e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48046830, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c46838, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4683d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54846841, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe46843, SUC_lexicon_tags_25 },
    { 0xf4046844, SUC_lexicon_tags_47 },
    { 0x5a46844, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16046849, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29c4684b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55e4684d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe604684f, SUC_lexicon_tags_12 },
    { 0xbc846850, SUC_lexicon_tags_27 },
    { 0x8d246850, SUC_lexicon_tags_47 },
    { 0xe24684f, SUC_lexicon_tags_63 },
    { 0x2ca46853, SUC_lexicon_tags_47 },
    { 0x5fc46854, SUC_lexicon_tags_1 },
    { 0xc9046852, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x48246858, SUC_lexicon_tags_12 },
    { 0x3da46859, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9e04685b, SUC_lexicon_tags_17 },
    { 0x4da4685c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14246861, SUC_lexicon_tags_2 },
    { 0x13c46862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c446865, SUC_lexicon_tags_39 },
    { 0x32c46865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1944686a, SUC_lexicon_tags_2 },
    { 0xacc4686b, SUC_lexicon_tags_8 },
    { 0xd9e4686c, SUC_lexicon_tags_3 },
    { 0x464686b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7646870, SUC_lexicon_tags_23 },
    { 0xb9a46870, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67446874, SUC_lexicon_tags_37 },
    { 0x46875, SUC_lexicon_tags_55 },
    { 0x3a446876, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2264687d, SUC_lexicon_tags_11 },
    { 0x23c4687e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5446881, SUC_lexicon_tags_39 },
    { 0x7da46882, SUC_lexicon_tags_2 },
    { 0x3e446883, SUC_lexicon_tags_11 },
    { 0x61c46884, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85646886, SUC_lexicon_tags_15 },
    { 0xadc46887, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15a46889, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50e4688b, SUC_lexicon_tags_432 },
    { 0x36c4688b, SUC_lexicon_tags_207 },
    { 0x8944688b, SUC_lexicon_tags_37 },
    { 0x9a4688e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a46894, SUC_lexicon_tags_2 },
    { 0x446895, SUC_lexicon_tags_2 },
    { 0xb2e46895, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118468a0, SUC_lexicon_tags_7 },
    { 0x67c468a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0468a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4468a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78468ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x250468af, SUC_lexicon_tags_48 },
    { 0x98e468b0, SUC_lexicon_tags_37 },
    { 0xf4e468b1, SUC_lexicon_tags_46 },
    { 0x948468af, SUC_lexicon_tags_39 },
    { 0x6468b3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae468b6, SUC_lexicon_tags_70 },
    { 0x30c468b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848468bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2fc468bd, SUC_lexicon_tags_68 },
    { 0xed4468be, SUC_lexicon_tags_30 },
    { 0x854468bf, SUC_lexicon_tags_15 },
    { 0x1dc468c0, SUC_lexicon_tags_48 },
    { 0x71a468bf, SUC_lexicon_tags_2 },
    { 0x866468c2, SUC_lexicon_tags_31 },
    { 0x378468c3, SUC_lexicon_tags_19 },
    { 0xc06468c3, SUC_lexicon_tags_25 },
    { 0x57a468c4, SUC_lexicon_tags_17 },
    { 0x14e468c2, SUC_lexicon_tags_17 },
    { 0xb68468c6, SUC_lexicon_tags_19 },
    { 0x5ee468c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc50468ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16468cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18468d2, SUC_lexicon_tags_39 },
    { 0x538468d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22468d7, SUC_lexicon_tags_17 },
    { 0xe9e468d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c468db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x77e468dd, SUC_lexicon_tags_0 },
    { 0xff2468dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6ee468e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1e468e2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa5e468e4, SUC_lexicon_tags_12 },
    { 0x160468e5, SUC_lexicon_tags_49 },
    { 0x3a4468e5, SUC_lexicon_tags_45 },
    { 0x176468e7, SUC_lexicon_tags_1 },
    { 0xbca468e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494468ed, SUC_lexicon_tags_136 },
    { 0x4aa468ee, SUC_lexicon_tags_35 },
    { 0xa9a468ed, SUC_lexicon_tags_77 },
    { 0x360468ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x356468f2, SUC_lexicon_tags_14 },
    { 0x63c468f3, SUC_lexicon_tags_17 },
    { 0xba468f4, SUC_lexicon_tags_6 },
    { 0xbe8468f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x62a468f7, SUC_lexicon_tags_10 },
    { 0xca468f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c468fb, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xcfe468fd, SUC_lexicon_tags_17 },
    { 0x87e468fd, SUC_lexicon_tags_11 },
    { 0xdb0468ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72246905, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26046907, SUC_lexicon_tags_15 },
    { 0xdb246907, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x6704690b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda44690f, SUC_lexicon_tags_12 },
    { 0xdb846910, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46046916, SUC_lexicon_tags_25 },
    { 0xbce46917, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c4691a, SUC_lexicon_tags_17 },
    { 0xea44691a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c4691f, SUC_lexicon_tags_55 },
    { 0xd8246920, SUC_lexicon_tags_30 },
    { 0x3a446921, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9846928, SUC_lexicon_tags_15 },
    { 0xdc446928, SUC_lexicon_tags_29 },
    { 0xd1846929, SUC_lexicon_tags_15 },
    { 0xb04692a, SUC_lexicon_tags_2 },
    { 0xa8446929, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfca4692e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d046931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d246933, SUC_lexicon_tags_14 },
    { 0x29446934, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a46938, SUC_lexicon_tags_23 },
    { 0x83646939, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9084693c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f24693e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1f446940, SUC_lexicon_tags_2 },
    { 0xf0046941, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c46945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2046948, SUC_lexicon_tags_38 },
    { 0x3ae46948, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a44694e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38446957, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a4695a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e4695e, SUC_lexicon_tags_37 },
    { 0x2be4695e, SUC_lexicon_tags_14 },
    { 0x8644695e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6bc46962, SUC_lexicon_tags_60 },
    { 0xad446963, SUC_lexicon_tags_47 },
    { 0xad046963, SUC_lexicon_tags_17 },
    { 0x41c46965, SUC_lexicon_tags_25 },
    { 0xc8246966, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc846968, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee4696d, SUC_lexicon_tags_16 },
    { 0x9004696d, SUC_lexicon_tags_17 },
    { 0xb0a4696f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3846975, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e46978, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c44698d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22046990, SUC_lexicon_tags_15 },
    { 0xa7246990, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb024699a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcb64699c, SUC_lexicon_tags_4 },
    { 0x2784699d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66469a0, SUC_lexicon_tags_38 },
    { 0x3cc469a0, SUC_lexicon_tags_43 },
    { 0x58c469a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae469a7, SUC_lexicon_tags_19 },
    { 0xffe469a8, SUC_lexicon_tags_17 },
    { 0x214469a8, SUC_lexicon_tags_66 },
    { 0x3a2469aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c469ae, SUC_lexicon_tags_17 },
    { 0xd8a469af, SUC_lexicon_tags_7 },
    { 0x31e469ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8469b4, SUC_lexicon_tags_10 },
    { 0xa74469b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8b0469b7, SUC_lexicon_tags_25 },
    { 0x776469b8, SUC_lexicon_tags_18 },
    { 0x3a6469b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32469bd, SUC_lexicon_tags_2 },
    { 0x918469be, SUC_lexicon_tags_65 },
    { 0x88e469bd, SUC_lexicon_tags_30 },
    { 0x69a469c0, SUC_lexicon_tags_60 },
    { 0x82c469c1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e469c4, SUC_lexicon_tags_37 },
    { 0x5ce469c5, SUC_lexicon_tags_3 },
    { 0xbb8469c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x688469c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe40469ca, SUC_lexicon_tags_17 },
    { 0x1f0469cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x902469cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea6469cf, SUC_lexicon_tags_48 },
    { 0xd5a469d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e469d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc469d8, SUC_lexicon_tags_10 },
    { 0xc4c469d9, SUC_lexicon_tags_25 },
    { 0xd72469d9, SUC_lexicon_tags_27 },
    { 0xa0469d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80469de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x528469e0, SUC_lexicon_tags_46 },
    { 0x858469e0, SUC_lexicon_tags_63 },
    { 0x6ce469e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da469e7, SUC_lexicon_tags_7 },
    { 0xd8469e7, SUC_lexicon_tags_2 },
    { 0x7e8469e8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158469ef, SUC_lexicon_tags_35 },
    { 0x576469f0, SUC_lexicon_tags_17 },
    { 0xcc0469f1, SUC_lexicon_tags_82 },
    { 0x1ec469f1, SUC_lexicon_tags_7 },
    { 0x292469f2, SUC_lexicon_tags_6 },
    { 0xe5c469f4, SUC_lexicon_tags_6 },
    { 0x9b6469f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa469f8, SUC_lexicon_tags_25 },
    { 0xa02469f9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x14469fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2469fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf246a00, SUC_lexicon_tags_47 },
    { 0x6c646a00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44a46a03, SUC_lexicon_tags_15 },
    { 0xb3246a04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfec46a06, SUC_lexicon_tags_7 },
    { 0x88a46a07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e46a0a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b446a0d, SUC_lexicon_tags_30 },
    { 0x50c46a0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7646a12, SUC_lexicon_tags_7 },
    { 0xd5646a12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3446a17, SUC_lexicon_tags_37 },
    { 0x50246a18, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67846a1c, SUC_lexicon_tags_3 },
    { 0xb4e46a1c, SUC_lexicon_tags_27 },
    { 0x3a446a1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe446a20, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7646a23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc46a29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c46a2d, SUC_lexicon_tags_7 },
    { 0x31a46a2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18646a30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67046a33, SUC_lexicon_tags_26 },
    { 0xe2c46a34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc646a38, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x43246a3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71246a3c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa046a40, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x1ba46a42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a046a4b, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0xe4646a4d, SUC_lexicon_tags_7 },
    { 0xa0046a4e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb246a54, SUC_lexicon_tags_14 },
    { 0xffa46a54, SUC_lexicon_tags_37 },
    { 0xfca46a55, SUC_lexicon_tags_45 },
    { 0xdca46a57, SUC_lexicon_tags_32 },
    { 0xb6e46a58, SUC_lexicon_tags_19 },
    { 0xd1446a56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c46a60, SUC_lexicon_tags_1 },
    { 0xdda46a60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f846a64, SUC_lexicon_tags_0 },
    { 0x39e46a65, SUC_lexicon_tags_25 },
    { 0x71846a66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e46a6a, SUC_lexicon_tags_63 },
    { 0xf8846a6b, SUC_lexicon_tags_1 },
    { 0x68046a6c, SUC_lexicon_tags_37 },
    { 0x87446a6d, SUC_lexicon_tags_37 },
    { 0x8ca46a6d, SUC_lexicon_tags_47 },
    { 0x40646a6c, SUC_lexicon_tags_31 },
    { 0x32a46a70, SUC_lexicon_tags_37 },
    { 0xdc46a71, SUC_lexicon_tags_27 },
    { 0x60e46a6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fa46a74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e046a77, SUC_lexicon_tags_65 },
    { 0x85c46a77, SUC_lexicon_tags_63 },
    { 0xaa046a78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd646a87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33e46a89, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84046a90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77a46a92, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a46a95, SUC_lexicon_tags_1 },
    { 0x8e246a96, SUC_lexicon_tags_37 },
    { 0x21246a97, SUC_lexicon_tags_17 },
    { 0x23646a98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72246a9b, SUC_lexicon_tags_0 },
    { 0xd8046a9b, SUC_lexicon_tags_0 },
    { 0x2b446a9d, SUC_lexicon_tags_15 },
    { 0xd0846a9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a46aa1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x22846aa4, SUC_lexicon_tags_2 },
    { 0xb6846aa5, SUC_lexicon_tags_19 },
    { 0xb6a46aa6, SUC_lexicon_tags_108 },
    { 0x4a046aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac46aab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0646ab1, SUC_lexicon_tags_39 },
    { 0xf3246ab2, SUC_lexicon_tags_349 },
    { 0xf0646ab1, SUC_lexicon_tags_2 },
    { 0xdae46ab3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9be46ab6, SUC_lexicon_tags_2 },
    { 0x4646ab6, SUC_lexicon_tags_6 },
    { 0x15046ab7, SUC_lexicon_tags_14 },
    { 0x20046ab9, SUC_lexicon_tags_66 },
    { 0xc2646aba, SUC_lexicon_tags_7 },
    { 0x83246ab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b246abf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe246ac3, SUC_lexicon_tags_28 },
    { 0xe1246ac4, SUC_lexicon_tags_45 },
    { 0xf7c46ac5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x94046ac7, SUC_lexicon_tags_3 },
    { 0x7f846ac7, SUC_lexicon_tags_15 },
    { 0x35c46ac7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7846acd, SUC_lexicon_tags_7 },
    { 0xf9246ace, SUC_lexicon_tags_37 },
    { 0xe3646acf, SUC_lexicon_tags_37 },
    { 0xf7046acf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78a46ad2, SUC_lexicon_tags_14 },
    { 0x57646ad3, SUC_lexicon_tags_2 },
    { 0xde646ad4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2646ad7, SUC_lexicon_tags_107 },
    { 0xbfe46ad8, SUC_lexicon_tags_17 },
    { 0x1c446ad8, SUC_lexicon_tags_13 },
    { 0x4d246ada, SUC_lexicon_tags_17 },
    { 0x6046adb, SUC_lexicon_tags_37 },
    { 0x85c46ad9, SUC_lexicon_tags_15 },
    { 0x3d046add, SUC_lexicon_tags_17 },
    { 0x95e46ade, SUC_lexicon_tags_7 },
    { 0x846ade, SUC_lexicon_tags_87 },
    { 0x5a46ae0, SUC_lexicon_tags_17 },
    { 0xb3e46ad7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc46ae8, SUC_lexicon_tags_13 },
    { 0x35e46ae9, SUC_lexicon_tags_38 },
    { 0x4aa46ae9, SUC_lexicon_tags_4 },
    { 0xe646aea, SUC_lexicon_tags_376 },
    { 0xee846aec, SUC_lexicon_tags_8 },
    { 0x52846aed, SUC_lexicon_tags_9 },
    { 0xe7446aee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdac46af0, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80446af4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6646af8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37446afd, SUC_lexicon_tags_49 },
    { 0xc5046afe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba46b01, SUC_lexicon_tags_2 },
    { 0x34e46b02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1446b04, SUC_lexicon_tags_42 },
    { 0xe3046b05, SUC_lexicon_tags_19 },
    { 0x54846b06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38a46b08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31246b0a, SUC_lexicon_tags_37 },
    { 0x98c46b0b, SUC_lexicon_tags_25 },
    { 0x78446b0a, SUC_lexicon_tags_13 },
    { 0x6a846b0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdc46b0f, SUC_lexicon_tags_1 },
    { 0x29446b0f, SUC_lexicon_tags_27 },
    { 0xb1e46b11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25846b13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1446b16, SUC_lexicon_tags_37 },
    { 0x67846b17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18846b19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb046b1d, SUC_lexicon_tags_44 },
    { 0xc3446b1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85046b20, SUC_lexicon_tags_46 },
    { 0xf9c46b21, SUC_lexicon_tags_47 },
    { 0x75c46b21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69246b24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde046b28, SUC_lexicon_tags_9 },
    { 0xd4246b29, SUC_lexicon_tags_17 },
    { 0xecc46b2a, SUC_lexicon_tags_2 },
    { 0x65846b2b, SUC_lexicon_tags_23 },
    { 0x79046b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd846b2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c46b31, SUC_lexicon_tags_1238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0446b36, SUC_lexicon_tags_8 },
    { 0xf9a46b37, SUC_lexicon_tags_47 },
    { 0x96246b36, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba46b3b, SUC_lexicon_tags_12 },
    { 0x39246b3c, SUC_lexicon_tags_2 },
    { 0xeb246b3d, SUC_lexicon_tags_37 },
    { 0xfe646b3d, SUC_lexicon_tags_6 },
    { 0xd846b3f, SUC_lexicon_tags_4 },
    { 0xf1646b40, SUC_lexicon_tags_0 },
    { 0xba046b40, SUC_lexicon_tags_12 },
    { 0xd3446b41, SUC_lexicon_tags_60 },
    { 0x64846b42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd446b46, SUC_lexicon_tags_17 },
    { 0x23846b46, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc46b4a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb6846b4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc446b50, SUC_lexicon_tags_62 },
    { 0x97046b50, SUC_lexicon_tags_0 },
    { 0x10046b52, SUC_lexicon_tags_2 },
    { 0xfbc46b53, SUC_lexicon_tags_37 },
    { 0xa7a46b53, SUC_lexicon_tags_16 },
    { 0x86246b52, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a46b5b, SUC_lexicon_tags_6 },
    { 0x1646b5c, SUC_lexicon_tags_11 },
    { 0x5b046b5b, SUC_lexicon_tags_12 },
    { 0x58646b5c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9c246b60, SUC_lexicon_tags_46 },
    { 0x47246b61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xed846b64, SUC_lexicon_tags_53 },
    { 0xd1646b65, SUC_lexicon_tags_19 },
    { 0x96246b65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a46b6b, SUC_lexicon_tags_31 },
    { 0x3c46b6c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92846b72, SUC_lexicon_tags_4 },
    { 0x72846b73, SUC_lexicon_tags_10 },
    { 0xe1a46b73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c46b78, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c046b7b, SUC_lexicon_tags_44 },
    { 0x1f846b7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0246b7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0846b80, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd6046b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b246b85, SUC_lexicon_tags_25 },
    { 0xc5646b86, SUC_lexicon_tags_25 },
    { 0x7fa46b86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2446b89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee46b8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81046b90, SUC_lexicon_tags_32 },
    { 0x43a46b90, SUC_lexicon_tags_0 },
    { 0x79846b92, SUC_lexicon_tags_11 },
    { 0xcb646b92, SUC_lexicon_tags_19 },
    { 0xf9246b94, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x44246b96, SUC_lexicon_tags_25 },
    { 0xaea46b96, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe46b9b, SUC_lexicon_tags_13 },
    { 0x7a46b9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6c646b9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a46ba2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xce846ba4, SUC_lexicon_tags_37 },
    { 0x9bc46ba4, SUC_lexicon_tags_7 },
    { 0xf9a46ba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae46bb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2a46bb2, SUC_lexicon_tags_2 },
    { 0x9c446bb3, SUC_lexicon_tags_27 },
    { 0x5d646bb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e46bb9, SUC_lexicon_tags_37 },
    { 0x12c46bb9, SUC_lexicon_tags_33 },
    { 0xe246bbb, SUC_lexicon_tags_1 },
    { 0x75046bbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78246bc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e846bc2, SUC_lexicon_tags_7 },
    { 0xc4a46bc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c046bc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9a46bc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95246bc9, SUC_lexicon_tags_11 },
    { 0x35646bc9, SUC_lexicon_tags_15 },
    { 0xe3446bcb, SUC_lexicon_tags_37 },
    { 0xf0646bcc, SUC_lexicon_tags_49 },
    { 0x67446bcd, SUC_lexicon_tags_0 },
    { 0x1ee46bcd, SUC_lexicon_tags_38 },
    { 0x14846bca, SUC_lexicon_tags_48 },
    { 0x54646bd0, SUC_lexicon_tags_25 },
    { 0xd5c46bd1, SUC_lexicon_tags_33 },
    { 0xd2a46bd2, SUC_lexicon_tags_1 },
    { 0x41c46bd3, SUC_lexicon_tags_4 },
    { 0xc1646bd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca46bda, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc46bde, SUC_lexicon_tags_6 },
    { 0x27646bdf, SUC_lexicon_tags_31 },
    { 0x11646be0, SUC_lexicon_tags_11 },
    { 0x53e46bdf, SUC_lexicon_tags_60 },
    { 0x4e446bde, SUC_lexicon_tags_45 },
    { 0x48646be3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc846be6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9a46be8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec046bee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc1a46bf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc46bf3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c46bf6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x50046bf8, SUC_lexicon_tags_53 },
    { 0xb0846bf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c446c01, SUC_lexicon_tags_36 },
    { 0x9d846c01, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a46c09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa46c11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd446c16, SUC_lexicon_tags_14 },
    { 0x6ae46c17, SUC_lexicon_tags_30 },
    { 0x70046c18, SUC_lexicon_tags_15 },
    { 0x36446c18, SUC_lexicon_tags_9 },
    { 0x73846c1a, SUC_lexicon_tags_17 },
    { 0x8fa46c16, SUC_lexicon_tags_7 },
    { 0xc8046c1c, SUC_lexicon_tags_7 },
    { 0xfb846c18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4c46c1f, SUC_lexicon_tags_17 },
    { 0xf0c46c20, SUC_lexicon_tags_25 },
    { 0x14446c21, SUC_lexicon_tags_39 },
    { 0x65a46c21, SUC_lexicon_tags_9 },
    { 0xa7a46c21, SUC_lexicon_tags_14 },
    { 0xc2c46c20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0446c27, SUC_lexicon_tags_37 },
    { 0x14246c27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b646c2b, SUC_lexicon_tags_37 },
    { 0x69446c2c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74646c34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6846c37, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c46c44, SUC_lexicon_tags_14 },
    { 0xf3246c45, SUC_lexicon_tags_60 },
    { 0x73646c45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4646c51, SUC_lexicon_tags_2 },
    { 0x2bc46c52, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3ae46c54, SUC_lexicon_tags_33 },
    { 0x1ee46c54, SUC_lexicon_tags_0 },
    { 0x8246c56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a46c5d, SUC_lexicon_tags_15 },
    { 0xf5a46c5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e46c61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3246c64, SUC_lexicon_tags_25 },
    { 0xa6446c64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc46c68, SUC_lexicon_tags_26 },
    { 0xf6a46c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36e46c6b, SUC_lexicon_tags_47 },
    { 0xd8e46c6c, SUC_lexicon_tags_0 },
    { 0xf3646c6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15646c72, SUC_lexicon_tags_32 },
    { 0x3e046c73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5a46c75, SUC_lexicon_tags_7 },
    { 0x3246c75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42a46c78, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc46c7e, SUC_lexicon_tags_17 },
    { 0x21646c7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c846c81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea46c86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37646c8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f846c95, SUC_lexicon_tags_11 },
    { 0xe0046c95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c46c9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95046ca8, SUC_lexicon_tags_25 },
    { 0x12246ca8, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a46cae, SUC_lexicon_tags_10 },
    { 0x27646caf, SUC_lexicon_tags_1 },
    { 0x87846caf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4846cb7, SUC_lexicon_tags_19 },
    { 0x93246cb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58846cba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3846cbe, SUC_lexicon_tags_37 },
    { 0x5be46cbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f846cc1, SUC_lexicon_tags_12 },
    { 0x95846cc2, SUC_lexicon_tags_23 },
    { 0x72c46cc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46c46cc5, SUC_lexicon_tags_8 },
    { 0xca846cc5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc46cc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca46ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c46cce, SUC_lexicon_tags_19 },
    { 0x6c246ccf, SUC_lexicon_tags_15 },
    { 0x20246ccf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6a46cd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42a46cd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e246cd6, SUC_lexicon_tags_16 },
    { 0x8c846cd7, SUC_lexicon_tags_14 },
    { 0x1aa46cd8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x11a46cda, SUC_lexicon_tags_21 },
    { 0xd4e46cdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f846cde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46ce0, SUC_lexicon_tags_82 },
    { 0xd6046ce1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa4646ce3, SUC_lexicon_tags_11 },
    { 0x29c46ce3, SUC_lexicon_tags_1 },
    { 0x4ba46ce3, SUC_lexicon_tags_6 },
    { 0x3ac46ce6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf246cf2, SUC_lexicon_tags_57 },
    { 0x60c46cf2, SUC_lexicon_tags_14 },
    { 0x33846cf4, SUC_lexicon_tags_76 },
    { 0x6d646cf5, SUC_lexicon_tags_23 },
    { 0xa8446cf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c046cf9, SUC_lexicon_tags_0 },
    { 0x71446cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e46d01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b646d03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee46d07, SUC_lexicon_tags_3 },
    { 0xa2c46d07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a446d11, SUC_lexicon_tags_7 },
    { 0x2d246d12, SUC_lexicon_tags_23 },
    { 0xf5246d11, SUC_lexicon_tags_60 },
    { 0x62c46d14, SUC_lexicon_tags_28 },
    { 0xd4c46d12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fa46d17, SUC_lexicon_tags_3 },
    { 0x8c646d17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e46d1c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c46d21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82046d23, SUC_lexicon_tags_15 },
    { 0x80646d23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d046d27, SUC_lexicon_tags_15 },
    { 0x36c46d27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb446d30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c46d33, SUC_lexicon_tags_29 },
    { 0x1c046d33, SUC_lexicon_tags_47 },
    { 0xcf246d35, SUC_lexicon_tags_1 },
    { 0x33c46d36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79246d3b, SUC_lexicon_tags_49 },
    { 0xf5c46d3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca846d3e, SUC_lexicon_tags_7 },
    { 0x97846d3e, SUC_lexicon_tags_11 },
    { 0xfac46d40, SUC_lexicon_tags_17 },
    { 0x4b846d41, SUC_lexicon_tags_25 },
    { 0xa7446d42, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96846d46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a446d48, SUC_lexicon_tags_37 },
    { 0x5de46d49, SUC_lexicon_tags_37 },
    { 0x4fc46d4a, SUC_lexicon_tags_10 },
    { 0x2c246d4b, SUC_lexicon_tags_25 },
    { 0x3ec46d4c, SUC_lexicon_tags_37 },
    { 0x14846d4c, SUC_lexicon_tags_7 },
    { 0x50a46d4e, SUC_lexicon_tags_15 },
    { 0x24446d4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52446d56, SUC_lexicon_tags_30 },
    { 0xffa46d56, SUC_lexicon_tags_47 },
    { 0x91246d57, SUC_lexicon_tags_30 },
    { 0x72246d56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58446d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97046d5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec46d60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae46d66, SUC_lexicon_tags_33 },
    { 0x6c646d66, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c46d6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c46d6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b046d72, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca46d76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e046d78, SUC_lexicon_tags_46 },
    { 0x64a46d79, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7b246d7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1646d80, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0xf6046d82, SUC_lexicon_tags_17 },
    { 0x68846d83, SUC_lexicon_tags_1 },
    { 0x16c46d83, SUC_lexicon_tags_37 },
    { 0x16046d85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8246d88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7046d8a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x846d8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0046d8f, SUC_lexicon_tags_37 },
    { 0x1d246d90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2646d93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a246d9f, SUC_lexicon_tags_17 },
    { 0xdfc46da0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee46da5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x41846da8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ae46daa, SUC_lexicon_tags_7 },
    { 0x4e046daa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7046daf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa646db1, SUC_lexicon_tags_11 },
    { 0xcd046db2, SUC_lexicon_tags_34 },
    { 0x44246db3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58446db5, SUC_lexicon_tags_7 },
    { 0x25246db6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdc46db8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c46dbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88646dbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43e46dbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2246dc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47246dc7, SUC_lexicon_tags_31 },
    { 0x7c246dc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1246dcb, SUC_lexicon_tags_577 },
    { 0xc9c46dcb, SUC_lexicon_tags_11 },
    { 0xa1c46dcd, SUC_lexicon_tags_17 },
    { 0xd2246dce, SUC_lexicon_tags_92 },
    { 0x8e646dcd, SUC_lexicon_tags_25 },
    { 0x80446dcf, SUC_lexicon_tags_37 },
    { 0xb5446dce, SUC_lexicon_tags_45 },
    { 0x1f646dd2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdaa46dd4, SUC_lexicon_tags_71 },
    { 0xd0446dd5, SUC_lexicon_tags_0 },
    { 0x15e46dd6, SUC_lexicon_tags_207 },
    { 0x2f646dd5, SUC_lexicon_tags_1 },
    { 0x83e46dd8, SUC_lexicon_tags_27 },
    { 0x7ee46dd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b246ddc, SUC_lexicon_tags_57 },
    { 0xce846ddd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc46de0, SUC_lexicon_tags_48 },
    { 0x99c46de1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a46de7, SUC_lexicon_tags_7 },
    { 0xda246de8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e46df3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2846df7, SUC_lexicon_tags_28 },
    { 0x84846df8, SUC_lexicon_tags_38 },
    { 0x74046df9, SUC_lexicon_tags_10 },
    { 0x2e846dfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc46e00, SUC_lexicon_tags_2 },
    { 0x8d046e00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52046e05, SUC_lexicon_tags_6 },
    { 0x1e46e06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce646e09, SUC_lexicon_tags_43 },
    { 0x6d646e0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71246e11, SUC_lexicon_tags_1 },
    { 0x16c46e11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30e46e14, SUC_lexicon_tags_29 },
    { 0x44e46e15, SUC_lexicon_tags_60 },
    { 0x97646e15, SUC_lexicon_tags_11 },
    { 0x29e46e17, SUC_lexicon_tags_15 },
    { 0x3ca46e18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x71a46e1a, SUC_lexicon_tags_19 },
    { 0x56246e1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ae46e1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde446e1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x33046e22, SUC_lexicon_tags_87 },
    { 0xbb846e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4e46e25, SUC_lexicon_tags_17 },
    { 0x88e46e26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3846e28, SUC_lexicon_tags_15 },
    { 0x5e046e29, SUC_lexicon_tags_12 },
    { 0xbb246e2a, SUC_lexicon_tags_37 },
    { 0x71e46e29, SUC_lexicon_tags_23 },
    { 0xafa46e2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c646e31, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x88646e34, SUC_lexicon_tags_2 },
    { 0x52846e34, SUC_lexicon_tags_17 },
    { 0x5c646e35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3846e3b, SUC_lexicon_tags_17 },
    { 0x3fa46e3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71246e3e, SUC_lexicon_tags_15 },
    { 0x3c46e3f, SUC_lexicon_tags_87 },
    { 0x16c46e3f, SUC_lexicon_tags_11 },
    { 0xce046e41, SUC_lexicon_tags_10 },
    { 0x5ce46e42, SUC_lexicon_tags_8 },
    { 0xa8a46e42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab846e46, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7846e49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3446e4b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a46e50, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79846e55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4a46e57, SUC_lexicon_tags_55 },
    { 0x9aa46e57, SUC_lexicon_tags_1 },
    { 0x32c46e58, SUC_lexicon_tags_10 },
    { 0xd0e46e59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78246e5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83a46e5f, SUC_lexicon_tags_55 },
    { 0x75246e60, SUC_lexicon_tags_38 },
    { 0x29446e60, SUC_lexicon_tags_17 },
    { 0x2a446e62, SUC_lexicon_tags_69 },
    { 0x74c46e63, SUC_lexicon_tags_2 },
    { 0xe8c46e61, SUC_lexicon_tags_14 },
    { 0x81046e5f, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a246e6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c446e6e, SUC_lexicon_tags_37 },
    { 0x13c46e6e, SUC_lexicon_tags_25 },
    { 0xc646e70, SUC_lexicon_tags_15 },
    { 0x29e46e6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe46e78, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e46e7d, SUC_lexicon_tags_29 },
    { 0x99a46e7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe646e85, SUC_lexicon_tags_38 },
    { 0x5846e86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c046e88, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8246e8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18646e92, SUC_lexicon_tags_41 },
    { 0x59846e93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c46e96, SUC_lexicon_tags_75 },
    { 0x83e46e97, SUC_lexicon_tags_12 },
    { 0x5046e98, SUC_lexicon_tags_11 },
    { 0x70c46e97, SUC_lexicon_tags_12 },
    { 0x9fe46e9a, SUC_lexicon_tags_38 },
    { 0x49246e9a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x61246e9d, SUC_lexicon_tags_45 },
    { 0xa6a46e9e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x88046ea0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73846ea2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76046ea4, SUC_lexicon_tags_68 },
    { 0x94c46ea5, SUC_lexicon_tags_84 },
    { 0x7e246ea6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8646ea9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc46eae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe6846eb0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x95246eb3, SUC_lexicon_tags_17 },
    { 0xa2846eb4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4e646eb6, SUC_lexicon_tags_29 },
    { 0xab246eb7, SUC_lexicon_tags_56 },
    { 0xdd446eb8, SUC_lexicon_tags_47 },
    { 0x15a46eb9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6246ebc, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b846ec1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74446ec3, SUC_lexicon_tags_45 },
    { 0xdc846ec4, SUC_lexicon_tags_19 },
    { 0x4c046ec5, SUC_lexicon_tags_12 },
    { 0x62246ec5, SUC_lexicon_tags_1 },
    { 0x57a46ec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c46ec9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e646ecc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90246ece, SUC_lexicon_tags_7 },
    { 0xa9246ecf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4fc46ed1, SUC_lexicon_tags_33 },
    { 0x49846ed2, SUC_lexicon_tags_9 },
    { 0x33446ed3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a46ed6, SUC_lexicon_tags_1 },
    { 0x8b446ed7, SUC_lexicon_tags_58 },
    { 0x69246ed6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27646ee1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a246ee6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9646ee9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6246eeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ea46eed, SUC_lexicon_tags_77 },
    { 0x93246eee, SUC_lexicon_tags_11 },
    { 0xe9a46eed, SUC_lexicon_tags_1 },
    { 0xd3446ef0, SUC_lexicon_tags_7 },
    { 0xd1e46ef1, SUC_lexicon_tags_60 },
    { 0xf0046ef0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb046ef4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa46ef7, SUC_lexicon_tags_27 },
    { 0x1ea46ef8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf6246efa, SUC_lexicon_tags_2 },
    { 0x60046efb, SUC_lexicon_tags_40 },
    { 0xde446efb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5046f00, SUC_lexicon_tags_17 },
    { 0xe6046f00, SUC_lexicon_tags_33 },
    { 0xd6e46f02, SUC_lexicon_tags_26 },
    { 0x45446f03, SUC_lexicon_tags_15 },
    { 0x2d246f03, SUC_lexicon_tags_50 },
    { 0x3fc46f05, SUC_lexicon_tags_65 },
    { 0x2f446f06, SUC_lexicon_tags_37 },
    { 0xba446f07, SUC_lexicon_tags_2 },
    { 0xa5a46f08, SUC_lexicon_tags_37 },
    { 0x20846f06, SUC_lexicon_tags_25 },
    { 0x86a46f08, SUC_lexicon_tags_17 },
    { 0xa6246f0b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe646f0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e46f13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4246f1a, SUC_lexicon_tags_537 },
    { 0x8e446f1a, SUC_lexicon_tags_1 },
    { 0x5ac46f1c, SUC_lexicon_tags_85 },
    { 0x7ae46f1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69c46f1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca46f25, SUC_lexicon_tags_11 },
    { 0x77846f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67246f28, SUC_lexicon_tags_12 },
    { 0x1bc46f29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa46f2c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d846f2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87646f31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3246f3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc1246f3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe5846f3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7046f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc246f48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e046f4b, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x6c846f4d, SUC_lexicon_tags_16 },
    { 0x2a646f4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee46f55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c46f59, SUC_lexicon_tags_17 },
    { 0xaa446f5a, SUC_lexicon_tags_9 },
    { 0xa0446f5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59246f60, SUC_lexicon_tags_27 },
    { 0xd4c46f61, SUC_lexicon_tags_28 },
    { 0x90c46f62, SUC_lexicon_tags_43 },
    { 0xd3846f63, SUC_lexicon_tags_2 },
    { 0xaa246f61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x37646f67, SUC_lexicon_tags_37 },
    { 0x72c46f68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc46f6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7446f70, SUC_lexicon_tags_12 },
    { 0x93e46f71, SUC_lexicon_tags_0 },
    { 0x53446f70, SUC_lexicon_tags_37 },
    { 0xb5646f73, SUC_lexicon_tags_33 },
    { 0x2be46f73, SUC_lexicon_tags_19 },
    { 0xc8646f75, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9446f79, SUC_lexicon_tags_201 },
    { 0xb0646f7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70446f80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6846f86, SUC_lexicon_tags_11 },
    { 0xfe046f86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c46f8b, SUC_lexicon_tags_37 },
    { 0x65a46f8b, SUC_lexicon_tags_15 },
    { 0x66a46f8d, SUC_lexicon_tags_14 },
    { 0x15046f8d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72846f92, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1ba46f94, SUC_lexicon_tags_43 },
    { 0x5b846f95, SUC_lexicon_tags_27 },
    { 0x6e846f96, SUC_lexicon_tags_11 },
    { 0x60446f95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e446f9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97446fa1, SUC_lexicon_tags_68 },
    { 0xdf646fa2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee446fa4, SUC_lexicon_tags_20 },
    { 0x52846fa5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23646faa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31246fb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5046fb4, SUC_lexicon_tags_25 },
    { 0xc0246fb5, SUC_lexicon_tags_7 },
    { 0x72646fb5, SUC_lexicon_tags_559 },
    { 0x26046fb5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e246fba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c46fbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c46fc0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e46fc3, SUC_lexicon_tags_10 },
    { 0xf9e46fc4, SUC_lexicon_tags_45 },
    { 0x68c46fc3, SUC_lexicon_tags_2 },
    { 0x3c046fc4, SUC_lexicon_tags_11 },
    { 0x13c46fc7, SUC_lexicon_tags_26 },
    { 0x80a46fc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1ca46fca, SUC_lexicon_tags_30 },
    { 0xaa46fcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac46fd1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x20446fd4, SUC_lexicon_tags_37 },
    { 0xf046fd5, SUC_lexicon_tags_37 },
    { 0x14c46fd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f446fd8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2fe46fda, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x14e46fdc, SUC_lexicon_tags_25 },
    { 0x8b646fdd, SUC_lexicon_tags_11 },
    { 0xcd446fde, SUC_lexicon_tags_46 },
    { 0xb2246fdf, SUC_lexicon_tags_87 },
    { 0x7dc46fdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6846fe4, SUC_lexicon_tags_27 },
    { 0xd5646fe5, SUC_lexicon_tags_14 },
    { 0x2046fe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0846fe9, SUC_lexicon_tags_11 },
    { 0xa7046fea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee46fed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a46ff0, SUC_lexicon_tags_55 },
    { 0x35646ff1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x24846ff4, SUC_lexicon_tags_4 },
    { 0xef646ff5, SUC_lexicon_tags_3 },
    { 0x9ea46ff5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51046ff8, SUC_lexicon_tags_19 },
    { 0xbb846ff9, SUC_lexicon_tags_27 },
    { 0xf5246ff8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c46ffe, SUC_lexicon_tags_37 },
    { 0x3ee46fff, SUC_lexicon_tags_12 },
    { 0x82846fff, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba47003, SUC_lexicon_tags_11 },
    { 0xdf647004, SUC_lexicon_tags_25 },
    { 0x5e247005, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe647008, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a44700a, SUC_lexicon_tags_341 },
    { 0x5a84700b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1047010, SUC_lexicon_tags_7 },
    { 0xdae47011, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0047014, SUC_lexicon_tags_11 },
    { 0x5d247014, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2047017, SUC_lexicon_tags_87 },
    { 0x70047018, SUC_lexicon_tags_0 },
    { 0x4e847019, SUC_lexicon_tags_4 },
    { 0x5aa4701a, SUC_lexicon_tags_7 },
    { 0xf284701b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6804701e, SUC_lexicon_tags_118 },
    { 0x70c4701f, SUC_lexicon_tags_87 },
    { 0xb7847020, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e047025, SUC_lexicon_tags_11 },
    { 0xe1c47026, SUC_lexicon_tags_25 },
    { 0x12c47027, SUC_lexicon_tags_18 },
    { 0x8dc47028, SUC_lexicon_tags_37 },
    { 0x7c047028, SUC_lexicon_tags_77 },
    { 0x27847027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a4702d, SUC_lexicon_tags_25 },
    { 0x9ac4702e, SUC_lexicon_tags_92 },
    { 0x55c4702f, SUC_lexicon_tags_27 },
    { 0x3c4702e, SUC_lexicon_tags_7 },
    { 0x20e47031, SUC_lexicon_tags_26 },
    { 0x6d647032, SUC_lexicon_tags_26 },
    { 0xf04702e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6e047035, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4647039, SUC_lexicon_tags_16 },
    { 0xa1c4703a, SUC_lexicon_tags_45 },
    { 0x7824703b, SUC_lexicon_tags_23 },
    { 0x5044703c, SUC_lexicon_tags_31 },
    { 0x96c4703d, SUC_lexicon_tags_35 },
    { 0xd1e4703e, SUC_lexicon_tags_8 },
    { 0x96a4703f, SUC_lexicon_tags_1 },
    { 0xb6a4703d, SUC_lexicon_tags_1 },
    { 0x29647041, SUC_lexicon_tags_25 },
    { 0xd4047041, SUC_lexicon_tags_25 },
    { 0x7504703e, SUC_lexicon_tags_57 },
    { 0xb884703f, SUC_lexicon_tags_9 },
    { 0xc5a47045, SUC_lexicon_tags_2 },
    { 0xde247045, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b447049, SUC_lexicon_tags_46 },
    { 0x14a4704a, SUC_lexicon_tags_27 },
    { 0x4f24704b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xace4704d, SUC_lexicon_tags_37 },
    { 0xde04704e, SUC_lexicon_tags_35 },
    { 0x3864704e, SUC_lexicon_tags_12 },
    { 0xece47050, SUC_lexicon_tags_28 },
    { 0xf2447050, SUC_lexicon_tags_37 },
    { 0xdfe47052, SUC_lexicon_tags_37 },
    { 0xec247051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c4705c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5984705e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x14e47060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee47067, SUC_lexicon_tags_14 },
    { 0xcca47068, SUC_lexicon_tags_18 },
    { 0xca047069, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44706c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92047074, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdde47076, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x50647079, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c44707c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbec4707e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9447080, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf2c47082, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3047084, SUC_lexicon_tags_2 },
    { 0xee847084, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e447087, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x47447089, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7924708d, SUC_lexicon_tags_14 },
    { 0x4404708e, SUC_lexicon_tags_49 },
    { 0x1324708f, SUC_lexicon_tags_17 },
    { 0x28c4708d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab647098, SUC_lexicon_tags_38 },
    { 0x59247099, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8624709c, SUC_lexicon_tags_37 },
    { 0x5c24709c, SUC_lexicon_tags_21 },
    { 0xce64709e, SUC_lexicon_tags_12 },
    { 0x1164709f, SUC_lexicon_tags_37 },
    { 0xe6470a0, SUC_lexicon_tags_37 },
    { 0x89a470a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892470ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520470b7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374470bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8470c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c470c6, SUC_lexicon_tags_15 },
    { 0xdb6470c7, SUC_lexicon_tags_23 },
    { 0xdbe470c6, SUC_lexicon_tags_7 },
    { 0xc470c9, SUC_lexicon_tags_12 },
    { 0xd88470c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08470d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x694470d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x660470d8, SUC_lexicon_tags_9 },
    { 0xb1e470d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x644470db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc470e2, SUC_lexicon_tags_37 },
    { 0x74470e3, SUC_lexicon_tags_11 },
    { 0x9b8470e2, SUC_lexicon_tags_29 },
    { 0x1c4470e5, SUC_lexicon_tags_7 },
    { 0xa3a470e6, SUC_lexicon_tags_11 },
    { 0x622470e3, SUC_lexicon_tags_37 },
    { 0xd16470e8, SUC_lexicon_tags_17 },
    { 0x11e470e3, SUC_lexicon_tags_15 },
    { 0x52a470e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0470ed, SUC_lexicon_tags_158 },
    { 0xa10470ee, SUC_lexicon_tags_37 },
    { 0xeaa470ee, SUC_lexicon_tags_23 },
    { 0xe50470ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348470f4, SUC_lexicon_tags_958 },
    { 0xc6e470f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0470f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a47100, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x93247102, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e647104, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2647106, SUC_lexicon_tags_27 },
    { 0x92047107, SUC_lexicon_tags_43 },
    { 0x4ba47107, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3904710b, SUC_lexicon_tags_27 },
    { 0x9444710c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5604711a, SUC_lexicon_tags_1 },
    { 0xd24711b, SUC_lexicon_tags_15 },
    { 0x7ca4711b, SUC_lexicon_tags_57 },
    { 0xd4c4711c, SUC_lexicon_tags_27 },
    { 0x37a4711e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc0a47120, SUC_lexicon_tags_45 },
    { 0x17447121, SUC_lexicon_tags_38 },
    { 0x34e47120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b047126, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73447128, SUC_lexicon_tags_1 },
    { 0x97847128, SUC_lexicon_tags_12 },
    { 0x3124712a, SUC_lexicon_tags_77 },
    { 0xf684712a, SUC_lexicon_tags_47 },
    { 0x26c47129, SUC_lexicon_tags_2 },
    { 0x84c4712d, SUC_lexicon_tags_15 },
    { 0xe784712e, SUC_lexicon_tags_7 },
    { 0xfc24712f, SUC_lexicon_tags_12 },
    { 0xbfa4712d, SUC_lexicon_tags_37 },
    { 0x8de47130, SUC_lexicon_tags_11 },
    { 0xb8447131, SUC_lexicon_tags_15 },
    { 0x86a47128, SUC_lexicon_tags_107 },
    { 0xda247134, SUC_lexicon_tags_10 },
    { 0x73847135, SUC_lexicon_tags_17 },
    { 0xd9647136, SUC_lexicon_tags_31 },
    { 0x66647135, SUC_lexicon_tags_43 },
    { 0x24247137, SUC_lexicon_tags_1 },
    { 0xbca47139, SUC_lexicon_tags_9 },
    { 0x26e4713a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e447144, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e24714d, SUC_lexicon_tags_37 },
    { 0x48e4714d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a24715c, SUC_lexicon_tags_37 },
    { 0x5684715d, SUC_lexicon_tags_37 },
    { 0xe7a4715d, SUC_lexicon_tags_1 },
    { 0x8164715c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54c47161, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d047165, SUC_lexicon_tags_9 },
    { 0x9c047166, SUC_lexicon_tags_37 },
    { 0x2f247167, SUC_lexicon_tags_4 },
    { 0xa047167, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed04716c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd44716e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec47171, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x44647174, SUC_lexicon_tags_17 },
    { 0x19447175, SUC_lexicon_tags_49 },
    { 0x33e47176, SUC_lexicon_tags_32 },
    { 0x84447177, SUC_lexicon_tags_23 },
    { 0x95247178, SUC_lexicon_tags_9 },
    { 0xc8c47177, SUC_lexicon_tags_15 },
    { 0x27e47179, SUC_lexicon_tags_29 },
    { 0x8be47178, SUC_lexicon_tags_17 },
    { 0x984717b, SUC_lexicon_tags_22 },
    { 0xdb84717a, SUC_lexicon_tags_46 },
    { 0x56c4717a, SUC_lexicon_tags_11 },
    { 0x7da4717f, SUC_lexicon_tags_38 },
    { 0x75e47180, SUC_lexicon_tags_17 },
    { 0x32047181, SUC_lexicon_tags_39 },
    { 0x8a047180, SUC_lexicon_tags_17 },
    { 0xeb04717b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e647186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a647189, SUC_lexicon_tags_37 },
    { 0x284718a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa64718c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbea4718e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79447199, SUC_lexicon_tags_69 },
    { 0x33e4719a, SUC_lexicon_tags_37 },
    { 0xe4247199, SUC_lexicon_tags_8 },
    { 0xe044719c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac471a0, SUC_lexicon_tags_7 },
    { 0x536471a1, SUC_lexicon_tags_37 },
    { 0xdd6471a0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xcbe471a4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb1c471a6, SUC_lexicon_tags_37 },
    { 0xfe4471a6, SUC_lexicon_tags_37 },
    { 0x27c471a8, SUC_lexicon_tags_65 },
    { 0x424471a8, SUC_lexicon_tags_25 },
    { 0x748471a7, SUC_lexicon_tags_77 },
    { 0x1cc471ab, SUC_lexicon_tags_2 },
    { 0x9c6471aa, SUC_lexicon_tags_9 },
    { 0x4b6471ad, SUC_lexicon_tags_12 },
    { 0x360471ac, SUC_lexicon_tags_43 },
    { 0xfd2471af, SUC_lexicon_tags_48 },
    { 0x23a471aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x654471b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6471b9, SUC_lexicon_tags_25 },
    { 0x3f8471ba, SUC_lexicon_tags_5 },
    { 0xea0471ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144471bf, SUC_lexicon_tags_1 },
    { 0x624471bf, SUC_lexicon_tags_33 },
    { 0x676471c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0471c5, SUC_lexicon_tags_63 },
    { 0x9e0471c6, SUC_lexicon_tags_25 },
    { 0x456471c7, SUC_lexicon_tags_37 },
    { 0x35a471c6, SUC_lexicon_tags_47 },
    { 0xbec471c5, SUC_lexicon_tags_25 },
    { 0x9c2471ca, SUC_lexicon_tags_17 },
    { 0x100471cb, SUC_lexicon_tags_8 },
    { 0x530471cc, SUC_lexicon_tags_17 },
    { 0x63a471ca, SUC_lexicon_tags_47 },
    { 0xf86471ca, SUC_lexicon_tags_2 },
    { 0xa42471cf, SUC_lexicon_tags_19 },
    { 0x906471cf, SUC_lexicon_tags_12 },
    { 0x434471d1, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x1e0471d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce471d7, SUC_lexicon_tags_14 },
    { 0xf62471d8, SUC_lexicon_tags_65 },
    { 0xa22471d8, SUC_lexicon_tags_7 },
    { 0x77c471d9, SUC_lexicon_tags_11 },
    { 0xac471d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfda471dd, SUC_lexicon_tags_49 },
    { 0x88a471de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4471e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4471e8, SUC_lexicon_tags_30 },
    { 0xab4471e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd36471eb, SUC_lexicon_tags_11 },
    { 0x6ac471ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa471ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18471f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65a471f4, SUC_lexicon_tags_4 },
    { 0x494471f5, SUC_lexicon_tags_12 },
    { 0x152471f6, SUC_lexicon_tags_25 },
    { 0x59c471f7, SUC_lexicon_tags_12 },
    { 0x86c471f8, SUC_lexicon_tags_37 },
    { 0xcd6471f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae471fc, SUC_lexicon_tags_7 },
    { 0xbc6471fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34a471ff, SUC_lexicon_tags_37 },
    { 0xa7e47200, SUC_lexicon_tags_17 },
    { 0x9c247201, SUC_lexicon_tags_27 },
    { 0x73e47202, SUC_lexicon_tags_1 },
    { 0x64c47202, SUC_lexicon_tags_65 },
    { 0xc0247204, SUC_lexicon_tags_15 },
    { 0x2a647202, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c4720a, SUC_lexicon_tags_1 },
    { 0xccc4720a, SUC_lexicon_tags_11 },
    { 0x34e4720c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1984720e, SUC_lexicon_tags_16 },
    { 0x6d84720f, SUC_lexicon_tags_37 },
    { 0x92e4720f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6647215, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca4721b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2944721d, SUC_lexicon_tags_6 },
    { 0x3ca4721e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c447223, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x83c47225, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e4722a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x724722d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25a4722f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11647233, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c47236, SUC_lexicon_tags_1 },
    { 0x3ac47236, SUC_lexicon_tags_29 },
    { 0xe6c47238, SUC_lexicon_tags_11 },
    { 0x11a47236, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d84723d, SUC_lexicon_tags_25 },
    { 0x6e4723e, SUC_lexicon_tags_33 },
    { 0x8de4723f, SUC_lexicon_tags_0 },
    { 0xe6e47240, SUC_lexicon_tags_11 },
    { 0xfa84723f, SUC_lexicon_tags_6 },
    { 0x9cc47242, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xad847244, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82647246, SUC_lexicon_tags_38 },
    { 0xf7c47247, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f84724b, SUC_lexicon_tags_63 },
    { 0x8604724b, SUC_lexicon_tags_45 },
    { 0xb144724b, SUC_lexicon_tags_16 },
    { 0x43a4724e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87447254, SUC_lexicon_tags_39 },
    { 0x17a47255, SUC_lexicon_tags_19 },
    { 0xcd047254, SUC_lexicon_tags_17 },
    { 0x6ac47257, SUC_lexicon_tags_55 },
    { 0x3c047254, SUC_lexicon_tags_1 },
    { 0xc4c47259, SUC_lexicon_tags_38 },
    { 0x4d847255, SUC_lexicon_tags_16 },
    { 0xeec47254, SUC_lexicon_tags_12 },
    { 0x8e64725c, SUC_lexicon_tags_4 },
    { 0x4d04725d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f047260, SUC_lexicon_tags_33 },
    { 0x20a47260, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d847264, SUC_lexicon_tags_68 },
    { 0x82447265, SUC_lexicon_tags_14 },
    { 0x8d247265, SUC_lexicon_tags_19 },
    { 0xd4647264, SUC_lexicon_tags_11 },
    { 0x8e47264, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a4726e, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5047273, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a47276, SUC_lexicon_tags_46 },
    { 0x85647276, SUC_lexicon_tags_92 },
    { 0x86a47278, SUC_lexicon_tags_11 },
    { 0xa2a47278, SUC_lexicon_tags_14 },
    { 0xee04727a, SUC_lexicon_tags_7 },
    { 0xdde4727b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed64727d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da47288, SUC_lexicon_tags_17 },
    { 0xfe247289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d44728d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x824728f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a47298, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x15c4729a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1c04729c, SUC_lexicon_tags_1 },
    { 0xc7c4729d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x29a4729f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x284472a1, SUC_lexicon_tags_17 },
    { 0x800472a1, SUC_lexicon_tags_9 },
    { 0x276472a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x156472a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda472ad, SUC_lexicon_tags_4 },
    { 0xc12472ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb66472b0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc472b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c472b8, SUC_lexicon_tags_23 },
    { 0xda6472b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6472bb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x104472be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabc472c0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4472c3, SUC_lexicon_tags_45 },
    { 0xe78472c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152472ca, SUC_lexicon_tags_12 },
    { 0x88e472cb, SUC_lexicon_tags_15 },
    { 0xb8472cc, SUC_lexicon_tags_4 },
    { 0x8c4472cb, SUC_lexicon_tags_10 },
    { 0xefe472cb, SUC_lexicon_tags_33 },
    { 0xfa8472cd, SUC_lexicon_tags_65 },
    { 0xf62472d0, SUC_lexicon_tags_45 },
    { 0x68a472d1, SUC_lexicon_tags_46 },
    { 0xc50472cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a472d7, SUC_lexicon_tags_18 },
    { 0x7a4472d8, SUC_lexicon_tags_39 },
    { 0x10c472d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe00472db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c472df, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x464472e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2a472e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c472e7, SUC_lexicon_tags_12 },
    { 0x688472e8, SUC_lexicon_tags_37 },
    { 0x67a472e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a472ec, SUC_lexicon_tags_11 },
    { 0x87c472ed, SUC_lexicon_tags_37 },
    { 0x4c8472ec, SUC_lexicon_tags_63 },
    { 0xf9a472ef, SUC_lexicon_tags_25 },
    { 0x758472f0, SUC_lexicon_tags_47 },
    { 0x690472ef, SUC_lexicon_tags_16 },
    { 0x494472ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4472f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b0472f9, SUC_lexicon_tags_25 },
    { 0x21a472fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x848472fc, SUC_lexicon_tags_14 },
    { 0xdd6472fd, SUC_lexicon_tags_15 },
    { 0x68a472fe, SUC_lexicon_tags_37 },
    { 0xb72472ff, SUC_lexicon_tags_47 },
    { 0xb04472fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9847304, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe2447306, SUC_lexicon_tags_65 },
    { 0x8d447306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e4730a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc4730e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8247310, SUC_lexicon_tags_45 },
    { 0x22847311, SUC_lexicon_tags_37 },
    { 0xd1447311, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e047315, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a47319, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e4731c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b64731e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88247324, SUC_lexicon_tags_25 },
    { 0xc2847324, SUC_lexicon_tags_7 },
    { 0xeee47324, SUC_lexicon_tags_7 },
    { 0xc9a47327, SUC_lexicon_tags_17 },
    { 0xb6847328, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a4732d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c4732f, SUC_lexicon_tags_148 },
    { 0x9be47330, SUC_lexicon_tags_6 },
    { 0x8d647330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf447334, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e4733d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6704733f, SUC_lexicon_tags_49 },
    { 0x4ce47340, SUC_lexicon_tags_53 },
    { 0x5e047341, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10a47343, SUC_lexicon_tags_25 },
    { 0x33047344, SUC_lexicon_tags_25 },
    { 0xe5847345, SUC_lexicon_tags_37 },
    { 0x93447346, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae47351, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba47358, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x90e4735a, SUC_lexicon_tags_21 },
    { 0xf184735a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc424735f, SUC_lexicon_tags_12 },
    { 0x5ce47360, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d247364, SUC_lexicon_tags_1 },
    { 0x3847365, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1864736e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70247371, SUC_lexicon_tags_23 },
    { 0x70e47372, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb247374, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xad047376, SUC_lexicon_tags_13 },
    { 0x9fe47376, SUC_lexicon_tags_37 },
    { 0xa9647377, SUC_lexicon_tags_15 },
    { 0xc6247379, SUC_lexicon_tags_38 },
    { 0xadc47378, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa764737d, SUC_lexicon_tags_39 },
    { 0xb684737d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20247383, SUC_lexicon_tags_11 },
    { 0xf2447384, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8047387, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c4738a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e447390, SUC_lexicon_tags_82 },
    { 0x2b047391, SUC_lexicon_tags_1 },
    { 0x1ba47390, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a47398, SUC_lexicon_tags_39 },
    { 0x64e47398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a4739d, SUC_lexicon_tags_37 },
    { 0xcf64739e, SUC_lexicon_tags_32 },
    { 0xffc4739f, SUC_lexicon_tags_8 },
    { 0xb564739d, SUC_lexicon_tags_23 },
    { 0xe58473a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80473a3, SUC_lexicon_tags_15 },
    { 0x6a8473a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a473a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c473ae, SUC_lexicon_tags_15 },
    { 0xaa2473af, SUC_lexicon_tags_19 },
    { 0x7a0473ae, SUC_lexicon_tags_25 },
    { 0x98a473ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a473bc, SUC_lexicon_tags_15 },
    { 0xebe473bd, SUC_lexicon_tags_15 },
    { 0x784473be, SUC_lexicon_tags_37 },
    { 0xf2e473bf, SUC_lexicon_tags_3 },
    { 0xaa4473c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e473c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe54473c5, SUC_lexicon_tags_265 },
    { 0xeca473c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56473ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958473ce, SUC_lexicon_tags_11 },
    { 0xadc473ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e4473d1, SUC_lexicon_tags_37 },
    { 0xd98473d2, SUC_lexicon_tags_11 },
    { 0x2dc473d3, SUC_lexicon_tags_38 },
    { 0xf3c473d3, SUC_lexicon_tags_2 },
    { 0x430473d5, SUC_lexicon_tags_47 },
    { 0xc18473d6, SUC_lexicon_tags_29 },
    { 0x9f0473d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a2473d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e473dd, SUC_lexicon_tags_17 },
    { 0x94c473de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de473e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52473e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe473ed, SUC_lexicon_tags_2 },
    { 0xc46473ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62473f3, SUC_lexicon_tags_63 },
    { 0x9aa473f4, SUC_lexicon_tags_6 },
    { 0xb28473f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4473f8, SUC_lexicon_tags_23 },
    { 0x38473f9, SUC_lexicon_tags_56 },
    { 0xf28473fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42473ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec47406, SUC_lexicon_tags_14 },
    { 0x25447407, SUC_lexicon_tags_25 },
    { 0x2847408, SUC_lexicon_tags_2 },
    { 0x62a47409, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a04740f, SUC_lexicon_tags_29 },
    { 0xd7c47410, SUC_lexicon_tags_105 },
    { 0x66647410, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3447415, SUC_lexicon_tags_3 },
    { 0x56047416, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x79447419, SUC_lexicon_tags_6 },
    { 0x1de4741a, SUC_lexicon_tags_21 },
    { 0xee64741a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9084741d, SUC_lexicon_tags_8 },
    { 0x6064741d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44647420, SUC_lexicon_tags_27 },
    { 0xfe447421, SUC_lexicon_tags_37 },
    { 0x88a47420, SUC_lexicon_tags_49 },
    { 0x52647423, SUC_lexicon_tags_68 },
    { 0x72847423, SUC_lexicon_tags_18 },
    { 0xea47422, SUC_lexicon_tags_30 },
    { 0x23447426, SUC_lexicon_tags_30 },
    { 0xed247427, SUC_lexicon_tags_2 },
    { 0x37447425, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce4742c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1047434, SUC_lexicon_tags_6 },
    { 0xe1247435, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e4743e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeca47440, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc47444, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafe47446, SUC_lexicon_tags_21 },
    { 0xfe47447, SUC_lexicon_tags_10 },
    { 0xed047448, SUC_lexicon_tags_57 },
    { 0xdbc47449, SUC_lexicon_tags_37 },
    { 0x84a47449, SUC_lexicon_tags_69 },
    { 0x68a4744b, SUC_lexicon_tags_63 },
    { 0x28647446, SUC_lexicon_tags_1 },
    { 0xb004744d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x74847450, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86847454, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1847456, SUC_lexicon_tags_32 },
    { 0x3ca47457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18e47459, SUC_lexicon_tags_1 },
    { 0x9bc4745a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d647460, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37847464, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1647467, SUC_lexicon_tags_17 },
    { 0xa2e47468, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb04746a, SUC_lexicon_tags_25 },
    { 0x7fa4746b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5604746d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce47472, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c47475, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64747a, SUC_lexicon_tags_37 },
    { 0x5544747a, SUC_lexicon_tags_45 },
    { 0xcb64747c, SUC_lexicon_tags_57 },
    { 0x1384747c, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0xdc84747f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c47483, SUC_lexicon_tags_47 },
    { 0x33c47483, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe047486, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6c47488, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fc4748a, SUC_lexicon_tags_38 },
    { 0x2344748a, SUC_lexicon_tags_17 },
    { 0xe1c4748c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c4748f, SUC_lexicon_tags_40 },
    { 0x33247490, SUC_lexicon_tags_61 },
    { 0x47647491, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26847494, SUC_lexicon_tags_0 },
    { 0xeba47495, SUC_lexicon_tags_7 },
    { 0x4f247495, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcac47498, SUC_lexicon_tags_104 },
    { 0x42247499, SUC_lexicon_tags_37 },
    { 0x2e64749a, SUC_lexicon_tags_46 },
    { 0xcee4749b, SUC_lexicon_tags_12 },
    { 0xdde4749c, SUC_lexicon_tags_66 },
    { 0x8904749d, SUC_lexicon_tags_11 },
    { 0x6684749b, SUC_lexicon_tags_16 },
    { 0x9084749d, SUC_lexicon_tags_37 },
    { 0xda2474a0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea474a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a474af, SUC_lexicon_tags_8 },
    { 0x488474b0, SUC_lexicon_tags_8 },
    { 0x26a474b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf18474b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2a474b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61e474b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b0474b9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e474bc, SUC_lexicon_tags_23 },
    { 0x1be474bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2474c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a474c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c474ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70474cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ba474d1, SUC_lexicon_tags_9 },
    { 0xc32474d2, SUC_lexicon_tags_16 },
    { 0x1c474d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84474d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x362474d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd08474da, SUC_lexicon_tags_33 },
    { 0x416474da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4474e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x218474e3, SUC_lexicon_tags_25 },
    { 0x340474e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a474e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55a474e9, SUC_lexicon_tags_47 },
    { 0xbea474ea, SUC_lexicon_tags_25 },
    { 0xd1a474eb, SUC_lexicon_tags_37 },
    { 0x2ce474ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2e474ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac474f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6474f7, SUC_lexicon_tags_62 },
    { 0x220474f8, SUC_lexicon_tags_23 },
    { 0x3ea474f9, SUC_lexicon_tags_38 },
    { 0xc7c474f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33a474fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x256474fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94647500, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb4847502, SUC_lexicon_tags_43 },
    { 0xf5e47503, SUC_lexicon_tags_0 },
    { 0x5c47504, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfde47506, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5884750a, SUC_lexicon_tags_37 },
    { 0xde84750a, SUC_lexicon_tags_31 },
    { 0x764750c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee47510, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b847512, SUC_lexicon_tags_14 },
    { 0x64c47512, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x28447515, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x92047517, SUC_lexicon_tags_37 },
    { 0x33647518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x48a4751a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a4751d, SUC_lexicon_tags_49 },
    { 0x3664751e, SUC_lexicon_tags_11 },
    { 0xbc84751f, SUC_lexicon_tags_37 },
    { 0x1204751e, SUC_lexicon_tags_17 },
    { 0x6b447521, SUC_lexicon_tags_38 },
    { 0x9d847522, SUC_lexicon_tags_12 },
    { 0xd1447520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9aa47525, SUC_lexicon_tags_0 },
    { 0x3be47526, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe847528, SUC_lexicon_tags_66 },
    { 0xbb047529, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x184752b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5004752e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4ce47530, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x98247532, SUC_lexicon_tags_6 },
    { 0x61c47532, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc804753b, SUC_lexicon_tags_14 },
    { 0xb1a4753c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1704753e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe47543, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a4754a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c24754c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c47552, SUC_lexicon_tags_2 },
    { 0x75647553, SUC_lexicon_tags_17 },
    { 0x99a47554, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a247557, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe344755c, SUC_lexicon_tags_7 },
    { 0x6204755c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb447561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a47564, SUC_lexicon_tags_45 },
    { 0xad247565, SUC_lexicon_tags_15 },
    { 0xb2047564, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde647569, SUC_lexicon_tags_38 },
    { 0x1c84756a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9644756c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c47570, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0047572, SUC_lexicon_tags_6 },
    { 0x69447573, SUC_lexicon_tags_2 },
    { 0x8f247574, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae4757b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47580, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x26247583, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e47589, SUC_lexicon_tags_47 },
    { 0x7b04758a, SUC_lexicon_tags_37 },
    { 0x8e84758b, SUC_lexicon_tags_15 },
    { 0xfa24758c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe47590, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de47593, SUC_lexicon_tags_11 },
    { 0x1f047594, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x44047596, SUC_lexicon_tags_2 },
    { 0x42847596, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x58047599, SUC_lexicon_tags_11 },
    { 0x21e47599, SUC_lexicon_tags_23 },
    { 0xaf44759b, SUC_lexicon_tags_15 },
    { 0x3d44759c, SUC_lexicon_tags_3 },
    { 0x7104759d, SUC_lexicon_tags_16 },
    { 0x93a4759e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e475a1, SUC_lexicon_tags_39 },
    { 0xbf4475a2, SUC_lexicon_tags_43 },
    { 0xa1e475a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee475aa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a475ad, SUC_lexicon_tags_38 },
    { 0xd80475ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdfa475b0, SUC_lexicon_tags_66 },
    { 0xc90475b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x670475b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60475b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202475ba, SUC_lexicon_tags_37 },
    { 0x4fa475bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e475be, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdf4475c0, SUC_lexicon_tags_4 },
    { 0xf88475c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8475c4, SUC_lexicon_tags_33 },
    { 0x4c475c4, SUC_lexicon_tags_26 },
    { 0x3e6475c4, SUC_lexicon_tags_7 },
    { 0x8fa475c7, SUC_lexicon_tags_60 },
    { 0x76a475c5, SUC_lexicon_tags_25 },
    { 0xc38475c8, SUC_lexicon_tags_15 },
    { 0xe5e475ca, SUC_lexicon_tags_25 },
    { 0x9ac475cb, SUC_lexicon_tags_7 },
    { 0x2b2475c7, SUC_lexicon_tags_11 },
    { 0xbae475cb, SUC_lexicon_tags_38 },
    { 0x152475c6, SUC_lexicon_tags_38 },
    { 0xa2e475cf, SUC_lexicon_tags_37 },
    { 0x282475ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6475d3, SUC_lexicon_tags_71 },
    { 0x6b8475d4, SUC_lexicon_tags_37 },
    { 0xb3e475d5, SUC_lexicon_tags_201 },
    { 0xa94475d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482475db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c475de, SUC_lexicon_tags_4 },
    { 0x69c475df, SUC_lexicon_tags_28 },
    { 0xcfc475e0, SUC_lexicon_tags_8 },
    { 0x9b4475de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5f2475e3, SUC_lexicon_tags_91 },
    { 0x3dc475e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c475e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88475ec, SUC_lexicon_tags_25 },
    { 0x67e475ed, SUC_lexicon_tags_12 },
    { 0xf82475ee, SUC_lexicon_tags_15 },
    { 0x862475ee, SUC_lexicon_tags_12 },
    { 0x286475f0, SUC_lexicon_tags_47 },
    { 0xcd4475f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c475f5, SUC_lexicon_tags_6 },
    { 0x22475f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2475f9, SUC_lexicon_tags_7 },
    { 0xf9a475f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb5c475fc, SUC_lexicon_tags_7 },
    { 0x2c475fd, SUC_lexicon_tags_1 },
    { 0x66a475fe, SUC_lexicon_tags_29 },
    { 0xf4a475ff, SUC_lexicon_tags_15 },
    { 0x148475fd, SUC_lexicon_tags_11 },
    { 0x760475fd, SUC_lexicon_tags_11 },
    { 0x2fc47602, SUC_lexicon_tags_2 },
    { 0x82c47603, SUC_lexicon_tags_48 },
    { 0xc6447604, SUC_lexicon_tags_64 },
    { 0x748475fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x76647608, SUC_lexicon_tags_47 },
    { 0x88647608, SUC_lexicon_tags_1090 },
    { 0x47047609, SUC_lexicon_tags_47 },
    { 0x51c4760b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe84760d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb684760f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe047614, SUC_lexicon_tags_2 },
    { 0x21c47615, SUC_lexicon_tags_17 },
    { 0x97047616, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5647618, SUC_lexicon_tags_11 },
    { 0xbd047619, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2e4761b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9804761e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec447625, SUC_lexicon_tags_37 },
    { 0xeda47626, SUC_lexicon_tags_2 },
    { 0x8047626, SUC_lexicon_tags_37 },
    { 0xde447628, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e4762e, SUC_lexicon_tags_15 },
    { 0x2fc4762f, SUC_lexicon_tags_23 },
    { 0x2544762e, SUC_lexicon_tags_15 },
    { 0xc0e47631, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfc447633, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc47636, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf84763d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a47641, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x56647644, SUC_lexicon_tags_15 },
    { 0x6a847644, SUC_lexicon_tags_33 },
    { 0x94247646, SUC_lexicon_tags_11 },
    { 0xec647646, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e4764a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be4764d, SUC_lexicon_tags_15 },
    { 0xf264764e, SUC_lexicon_tags_49 },
    { 0x3ce4764d, SUC_lexicon_tags_14 },
    { 0x1544764e, SUC_lexicon_tags_23 },
    { 0x6f24764f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d447657, SUC_lexicon_tags_343 },
    { 0xa1447658, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b04765a, SUC_lexicon_tags_1 },
    { 0x7084765b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9104765d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b047664, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x25647666, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684766d, SUC_lexicon_tags_105 },
    { 0x9e04766e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff247670, SUC_lexicon_tags_15 },
    { 0x45647671, SUC_lexicon_tags_14 },
    { 0xb3a47672, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8684767a, SUC_lexicon_tags_0 },
    { 0x824767b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xad04767d, SUC_lexicon_tags_15 },
    { 0x1f44767d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb3847680, SUC_lexicon_tags_22 },
    { 0x83c47680, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e47686, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x50047689, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7324768c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34047690, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16047698, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4769c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2476a7, SUC_lexicon_tags_25 },
    { 0x3ae476a7, SUC_lexicon_tags_124 },
    { 0x41a476a9, SUC_lexicon_tags_15 },
    { 0xf98476aa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e476af, SUC_lexicon_tags_11 },
    { 0xe14476b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc38476b2, SUC_lexicon_tags_46 },
    { 0xc18476b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32476b6, SUC_lexicon_tags_11 },
    { 0x5a6476b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4476bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x10c476bd, SUC_lexicon_tags_0 },
    { 0xd52476be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36476c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca4476c3, SUC_lexicon_tags_32 },
    { 0x5a4476c3, SUC_lexicon_tags_25 },
    { 0xe0c476c5, SUC_lexicon_tags_15 },
    { 0xc06476c6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0476c9, SUC_lexicon_tags_11 },
    { 0x39c476ca, SUC_lexicon_tags_25 },
    { 0xcc476c9, SUC_lexicon_tags_65 },
    { 0x2c4476ca, SUC_lexicon_tags_37 },
    { 0x610476cd, SUC_lexicon_tags_21 },
    { 0x538476ce, SUC_lexicon_tags_6 },
    { 0x402476ce, SUC_lexicon_tags_8 },
    { 0x40e476d0, SUC_lexicon_tags_12 },
    { 0x810476ce, SUC_lexicon_tags_15 },
    { 0x8ce476ce, SUC_lexicon_tags_17 },
    { 0x45c476d3, SUC_lexicon_tags_17 },
    { 0xba8476cf, SUC_lexicon_tags_144 },
    { 0xfc6476d5, SUC_lexicon_tags_25 },
    { 0xf80476d5, SUC_lexicon_tags_15 },
    { 0xfda476d7, SUC_lexicon_tags_61 },
    { 0xd5e476d7, SUC_lexicon_tags_37 },
    { 0x6fe476d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc476de, SUC_lexicon_tags_12 },
    { 0xf9e476df, SUC_lexicon_tags_17 },
    { 0x45e476e0, SUC_lexicon_tags_45 },
    { 0xe78476e0, SUC_lexicon_tags_31 },
    { 0xda8476e0, SUC_lexicon_tags_37 },
    { 0xa5a476e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0476eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2a6476ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6476f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a476f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc00476f9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4476ff, SUC_lexicon_tags_14 },
    { 0x96247700, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a647705, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0847708, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd604770a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff04770c, SUC_lexicon_tags_31 },
    { 0x7064770d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e64770f, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0xfec47711, SUC_lexicon_tags_25 },
    { 0x66e47712, SUC_lexicon_tags_12 },
    { 0xf9047713, SUC_lexicon_tags_14 },
    { 0x17a47712, SUC_lexicon_tags_37 },
    { 0xb0847714, SUC_lexicon_tags_84 },
    { 0xd6847712, SUC_lexicon_tags_32 },
    { 0xff247717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2447719, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a4771e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x35c47720, SUC_lexicon_tags_63 },
    { 0x4ea47720, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x27847723, SUC_lexicon_tags_47 },
    { 0x2f047723, SUC_lexicon_tags_2 },
    { 0x89c47725, SUC_lexicon_tags_10 },
    { 0xa6047725, SUC_lexicon_tags_32 },
    { 0x5d247726, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe2c47729, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a44772b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea24772f, SUC_lexicon_tags_70 },
    { 0x27847730, SUC_lexicon_tags_25 },
    { 0x6e047731, SUC_lexicon_tags_47 },
    { 0x7a04772f, SUC_lexicon_tags_15 },
    { 0x29447730, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75e47735, SUC_lexicon_tags_1 },
    { 0x48647736, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf6847738, SUC_lexicon_tags_1 },
    { 0xf3647738, SUC_lexicon_tags_7 },
    { 0xd8647738, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a47740, SUC_lexicon_tags_26 },
    { 0x35a47740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8447743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37247745, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93447749, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4124774e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34447751, SUC_lexicon_tags_7 },
    { 0x45447751, SUC_lexicon_tags_37 },
    { 0xede47753, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17847757, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7904775b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8b84775d, SUC_lexicon_tags_61 },
    { 0x4d44775e, SUC_lexicon_tags_45 },
    { 0x9504775f, SUC_lexicon_tags_29 },
    { 0xe0a4775f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f447762, SUC_lexicon_tags_38 },
    { 0xcf447763, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76847770, SUC_lexicon_tags_1 },
    { 0x12e47771, SUC_lexicon_tags_45 },
    { 0xe0c47772, SUC_lexicon_tags_45 },
    { 0x5b447771, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e47776, SUC_lexicon_tags_165 },
    { 0xb8647777, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b847779, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a4777c, SUC_lexicon_tags_29 },
    { 0x5c24777d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8247786, SUC_lexicon_tags_54 },
    { 0x85647787, SUC_lexicon_tags_2 },
    { 0xf1847786, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4424778a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93247790, SUC_lexicon_tags_0 },
    { 0x11047790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97247794, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe2247796, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e47799, SUC_lexicon_tags_23 },
    { 0xc424779a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc144779e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e2477a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x610477a2, SUC_lexicon_tags_60 },
    { 0xe48477a3, SUC_lexicon_tags_33 },
    { 0xf3a477a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76477a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba477ab, SUC_lexicon_tags_25 },
    { 0xa6a477ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc6477ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa477b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f8477b9, SUC_lexicon_tags_10 },
    { 0x71e477ba, SUC_lexicon_tags_11 },
    { 0x4d6477bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84477c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e477cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd88477cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104477d1, SUC_lexicon_tags_27 },
    { 0xbf6477d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e8477d4, SUC_lexicon_tags_21 },
    { 0x6f8477d5, SUC_lexicon_tags_1 },
    { 0x954477d6, SUC_lexicon_tags_55 },
    { 0x646477d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2477da, SUC_lexicon_tags_23 },
    { 0x49a477db, SUC_lexicon_tags_11 },
    { 0x2a2477dc, SUC_lexicon_tags_25 },
    { 0xc2c477dc, SUC_lexicon_tags_37 },
    { 0x68c477dd, SUC_lexicon_tags_11 },
    { 0x2c4477df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x746477e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c477e5, SUC_lexicon_tags_12 },
    { 0x276477e6, SUC_lexicon_tags_17 },
    { 0xb7e477e7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6477ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x374477ef, SUC_lexicon_tags_8 },
    { 0xae0477f0, SUC_lexicon_tags_0 },
    { 0x590477f1, SUC_lexicon_tags_5 },
    { 0x404477f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x650477f4, SUC_lexicon_tags_53 },
    { 0xefa477f5, SUC_lexicon_tags_1 },
    { 0xb7a477f5, SUC_lexicon_tags_0 },
    { 0x54a477f7, SUC_lexicon_tags_25 },
    { 0x85e477f7, SUC_lexicon_tags_40 },
    { 0xd80477f9, SUC_lexicon_tags_11 },
    { 0xb06477f9, SUC_lexicon_tags_4 },
    { 0xa48477fb, SUC_lexicon_tags_17 },
    { 0xc36477f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba47801, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88447808, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6ba4780a, SUC_lexicon_tags_37 },
    { 0xe004780a, SUC_lexicon_tags_235 },
    { 0x38e4780c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b64780f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d647811, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c47815, SUC_lexicon_tags_17 },
    { 0x2ec47815, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x91247818, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb124781a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf64781d, SUC_lexicon_tags_80 },
    { 0xa664781e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24047820, SUC_lexicon_tags_417 },
    { 0xb3447820, SUC_lexicon_tags_82 },
    { 0x18447821, SUC_lexicon_tags_28 },
    { 0x39647821, SUC_lexicon_tags_62 },
    { 0x65a47824, SUC_lexicon_tags_37 },
    { 0x7f247825, SUC_lexicon_tags_43 },
    { 0x22a47826, SUC_lexicon_tags_37 },
    { 0x98847826, SUC_lexicon_tags_15 },
    { 0xe9a47822, SUC_lexicon_tags_45 },
    { 0x4a647829, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd6e4782b, SUC_lexicon_tags_7 },
    { 0xe384782c, SUC_lexicon_tags_42 },
    { 0x71a4782b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d847832, SUC_lexicon_tags_25 },
    { 0xd0047833, SUC_lexicon_tags_37 },
    { 0x6a647833, SUC_lexicon_tags_16 },
    { 0xeb047834, SUC_lexicon_tags_49 },
    { 0x6ba47836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9247840, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd247844, SUC_lexicon_tags_1056 },
    { 0x93447845, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be4784a, SUC_lexicon_tags_33 },
    { 0x1304784a, SUC_lexicon_tags_4 },
    { 0x39e4784c, SUC_lexicon_tags_15 },
    { 0xd164784d, SUC_lexicon_tags_15 },
    { 0x7164784e, SUC_lexicon_tags_14 },
    { 0x4344784f, SUC_lexicon_tags_12 },
    { 0x7e84784b, SUC_lexicon_tags_26 },
    { 0xbf84784a, SUC_lexicon_tags_17 },
    { 0xeb84784d, SUC_lexicon_tags_0 },
    { 0x54447853, SUC_lexicon_tags_0 },
    { 0xe3c47854, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e47857, SUC_lexicon_tags_1 },
    { 0x87247858, SUC_lexicon_tags_37 },
    { 0x68447859, SUC_lexicon_tags_15 },
    { 0x6d647859, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8664785f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94247862, SUC_lexicon_tags_29 },
    { 0x84247863, SUC_lexicon_tags_19 },
    { 0x26647864, SUC_lexicon_tags_25 },
    { 0x4e647864, SUC_lexicon_tags_7 },
    { 0x89047862, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62247868, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c4786b, SUC_lexicon_tags_12 },
    { 0xf144786c, SUC_lexicon_tags_17 },
    { 0xba84786d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xa9a4786f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a47872, SUC_lexicon_tags_423 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c47875, SUC_lexicon_tags_27 },
    { 0x10a47876, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd647881, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd447883, SUC_lexicon_tags_28 },
    { 0x8047884, SUC_lexicon_tags_12 },
    { 0xdfe47885, SUC_lexicon_tags_37 },
    { 0x89647886, SUC_lexicon_tags_35 },
    { 0xd0447885, SUC_lexicon_tags_61 },
    { 0x5b047888, SUC_lexicon_tags_25 },
    { 0xbda47888, SUC_lexicon_tags_14 },
    { 0x3f04788a, SUC_lexicon_tags_6 },
    { 0x8684788b, SUC_lexicon_tags_38 },
    { 0x7e44788b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4447890, SUC_lexicon_tags_30 },
    { 0xfae47891, SUC_lexicon_tags_7 },
    { 0x9c47892, SUC_lexicon_tags_37 },
    { 0x78047892, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6247896, SUC_lexicon_tags_16 },
    { 0xa8847896, SUC_lexicon_tags_37 },
    { 0xdd247898, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab24789b, SUC_lexicon_tags_1 },
    { 0x5404789b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x39e4789e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xee8478a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82478a5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea478a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8478b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x36478b3, SUC_lexicon_tags_34 },
    { 0xafc478b4, SUC_lexicon_tags_154 },
    { 0x334478b5, SUC_lexicon_tags_113 },
    { 0xfe0478b6, SUC_lexicon_tags_19 },
    { 0x69c478b5, SUC_lexicon_tags_17 },
    { 0x3a8478b8, SUC_lexicon_tags_49 },
    { 0xe2478b4, SUC_lexicon_tags_65 },
    { 0x966478b3, SUC_lexicon_tags_25 },
    { 0xc64478bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8478c4, SUC_lexicon_tags_11 },
    { 0x152478c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x934478c7, SUC_lexicon_tags_108 },
    { 0x232478c8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c478cc, SUC_lexicon_tags_12 },
    { 0xf50478cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x87a478cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x920478d2, SUC_lexicon_tags_17 },
    { 0xb46478d3, SUC_lexicon_tags_109 },
    { 0x122478d4, SUC_lexicon_tags_37 },
    { 0x9de478d2, SUC_lexicon_tags_33 },
    { 0xf40478d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82478da, SUC_lexicon_tags_39 },
    { 0xa46478db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2f2478dd, SUC_lexicon_tags_37 },
    { 0x554478dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4478e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08478f2, SUC_lexicon_tags_11 },
    { 0xdc8478f3, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8478f6, SUC_lexicon_tags_27 },
    { 0x8b0478f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36478fa, SUC_lexicon_tags_2 },
    { 0xfe2478fb, SUC_lexicon_tags_46 },
    { 0xc36478fc, SUC_lexicon_tags_12 },
    { 0x800478fc, SUC_lexicon_tags_37 },
    { 0xb94478fe, SUC_lexicon_tags_68 },
    { 0x9ea478ff, SUC_lexicon_tags_26 },
    { 0x506478fb, SUC_lexicon_tags_15 },
    { 0xdec478fa, SUC_lexicon_tags_33 },
    { 0xc5c47901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3647904, SUC_lexicon_tags_8 },
    { 0xa5647905, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x57a47907, SUC_lexicon_tags_6 },
    { 0x68847908, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb944790a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c4790d, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c47911, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x62647914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5847917, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac4791b, SUC_lexicon_tags_12 },
    { 0x58c4791c, SUC_lexicon_tags_7 },
    { 0x7124791c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec47921, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63647927, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e44792a, SUC_lexicon_tags_15 },
    { 0xa1e4792a, SUC_lexicon_tags_31 },
    { 0x73a4792c, SUC_lexicon_tags_14 },
    { 0x16e4792d, SUC_lexicon_tags_46 },
    { 0x7624792e, SUC_lexicon_tags_4 },
    { 0x1d64792e, SUC_lexicon_tags_9 },
    { 0x96e4792e, SUC_lexicon_tags_23 },
    { 0x54647931, SUC_lexicon_tags_60 },
    { 0x2ba47932, SUC_lexicon_tags_15 },
    { 0xa4847933, SUC_lexicon_tags_7 },
    { 0x5ac47934, SUC_lexicon_tags_17 },
    { 0xbb647932, SUC_lexicon_tags_34 },
    { 0x60e47936, SUC_lexicon_tags_29 },
    { 0xf4e47937, SUC_lexicon_tags_2 },
    { 0x75047938, SUC_lexicon_tags_15 },
    { 0x12c47938, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1484793f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8447943, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c84794a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6847952, SUC_lexicon_tags_1 },
    { 0x26c47953, SUC_lexicon_tags_9 },
    { 0xbc047953, SUC_lexicon_tags_63 },
    { 0x8f447955, SUC_lexicon_tags_10 },
    { 0x6e447955, SUC_lexicon_tags_47 },
    { 0xb3e47957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16047959, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe424795b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec4795f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1de47961, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e47964, SUC_lexicon_tags_21 },
    { 0x63847964, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17847969, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd847970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d647973, SUC_lexicon_tags_37 },
    { 0x78247973, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa47977, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8847980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96647982, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a84798d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8447991, SUC_lexicon_tags_7 },
    { 0xe1447992, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e47996, SUC_lexicon_tags_27 },
    { 0x83647997, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc4799e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8479a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c479aa, SUC_lexicon_tags_11 },
    { 0xcee479ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x122479ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x634479b0, SUC_lexicon_tags_26 },
    { 0xb0c479b0, SUC_lexicon_tags_11 },
    { 0xe7c479b2, SUC_lexicon_tags_37 },
    { 0xba2479b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2479b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4479be, SUC_lexicon_tags_17 },
    { 0x512479bf, SUC_lexicon_tags_37 },
    { 0x98a479bf, SUC_lexicon_tags_15 },
    { 0xc7a479c1, SUC_lexicon_tags_63 },
    { 0x92c479c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2479c6, SUC_lexicon_tags_25 },
    { 0x128479c6, SUC_lexicon_tags_4 },
    { 0x362479c8, SUC_lexicon_tags_17 },
    { 0x442479c9, SUC_lexicon_tags_11 },
    { 0x62e479c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe58479cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e479cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3bc479d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a6479d3, SUC_lexicon_tags_719 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba479d7, SUC_lexicon_tags_34 },
    { 0x6f8479d8, SUC_lexicon_tags_63 },
    { 0xa6479d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6479df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x250479e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a479e4, SUC_lexicon_tags_49 },
    { 0x360479e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492479eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0479f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a479f5, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xd76479f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84479fd, SUC_lexicon_tags_1 },
    { 0x510479fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e047a01, SUC_lexicon_tags_12 },
    { 0x82a47a01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91047a06, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xae647a08, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc47a0b, SUC_lexicon_tags_35 },
    { 0x9f447a0c, SUC_lexicon_tags_6 },
    { 0xdd247a0c, SUC_lexicon_tags_9 },
    { 0x56247a0d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x7fa47a10, SUC_lexicon_tags_23 },
    { 0xc847a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e47a15, SUC_lexicon_tags_82 },
    { 0x8647a16, SUC_lexicon_tags_25 },
    { 0x78047a17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a47a1a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc0247a1c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60647a20, SUC_lexicon_tags_27 },
    { 0x25447a20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15047a27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec47a2a, SUC_lexicon_tags_25 },
    { 0xa4247a2a, SUC_lexicon_tags_8 },
    { 0x5f047a2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c47a36, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b047a39, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xf5c47a3b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d847a3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0247a41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10847a43, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x46047a46, SUC_lexicon_tags_30 },
    { 0xe9047a47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae47a4c, SUC_lexicon_tags_37 },
    { 0xda647a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21647a4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26447a51, SUC_lexicon_tags_63 },
    { 0x5e47a52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34247a57, SUC_lexicon_tags_47 },
    { 0x8b847a58, SUC_lexicon_tags_11 },
    { 0xa0247a59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb647a5b, SUC_lexicon_tags_61 },
    { 0x9c047a5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d447a5f, SUC_lexicon_tags_23 },
    { 0xf9c47a5f, SUC_lexicon_tags_29 },
    { 0xa0647a61, SUC_lexicon_tags_47 },
    { 0x88c47a62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc47a66, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3f447a68, SUC_lexicon_tags_4 },
    { 0x11047a69, SUC_lexicon_tags_11 },
    { 0xa247a69, SUC_lexicon_tags_238 },
    { 0x3c247a6b, SUC_lexicon_tags_34 },
    { 0xff047a6c, SUC_lexicon_tags_11 },
    { 0x97247a6c, SUC_lexicon_tags_39 },
    { 0x21447a6a, SUC_lexicon_tags_2 },
    { 0x54a47a6f, SUC_lexicon_tags_19 },
    { 0x7b647a70, SUC_lexicon_tags_7 },
    { 0x3d447a6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847a7a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xab247a7c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee647a82, SUC_lexicon_tags_60 },
    { 0xb247a82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea47a86, SUC_lexicon_tags_151 },
    { 0x69e47a87, SUC_lexicon_tags_25 },
    { 0x30047a87, SUC_lexicon_tags_39 },
    { 0x42447a87, SUC_lexicon_tags_27 },
    { 0x63247a88, SUC_lexicon_tags_39 },
    { 0x76047a8b, SUC_lexicon_tags_14 },
    { 0xaf047a8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda47a95, SUC_lexicon_tags_19 },
    { 0x81a47a96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce047a99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf047a9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e47aa0, SUC_lexicon_tags_7 },
    { 0x6a47aa1, SUC_lexicon_tags_27 },
    { 0x77e47aa2, SUC_lexicon_tags_11 },
    { 0xae847aa1, SUC_lexicon_tags_37 },
    { 0x65c47aa4, SUC_lexicon_tags_21 },
    { 0xb3e47aa4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef447aae, SUC_lexicon_tags_87 },
    { 0xc3847aaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa247ab7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b847aba, SUC_lexicon_tags_16 },
    { 0xdc647abb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc6a47abd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe47ac1, SUC_lexicon_tags_37 },
    { 0x9a247ac1, SUC_lexicon_tags_21 },
    { 0xe8847ac2, SUC_lexicon_tags_32 },
    { 0x71047ac3, SUC_lexicon_tags_27 },
    { 0xd6047ac4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8447ac7, SUC_lexicon_tags_37 },
    { 0xf447ac8, SUC_lexicon_tags_2 },
    { 0xc6847ac9, SUC_lexicon_tags_201 },
    { 0x5f847ac9, SUC_lexicon_tags_44 },
    { 0xf2247acb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x82447acd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e447ad0, SUC_lexicon_tags_7 },
    { 0xc7247ad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77447ad3, SUC_lexicon_tags_7 },
    { 0x87647ad4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fa47ad6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18847ad8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5447ada, SUC_lexicon_tags_37 },
    { 0x8d447adb, SUC_lexicon_tags_2 },
    { 0x8f647ada, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x55247adf, SUC_lexicon_tags_32 },
    { 0xdbc47ae0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a47ae6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed447ae9, SUC_lexicon_tags_68 },
    { 0x4d647ae9, SUC_lexicon_tags_47 },
    { 0xa2a47aeb, SUC_lexicon_tags_27 },
    { 0x99247ae9, SUC_lexicon_tags_37 },
    { 0x8b847aeb, SUC_lexicon_tags_15 },
    { 0x3e647aee, SUC_lexicon_tags_6 },
    { 0xfc847aef, SUC_lexicon_tags_13 },
    { 0x2a447af0, SUC_lexicon_tags_151 },
    { 0x52647aea, SUC_lexicon_tags_39 },
    { 0x80047af0, SUC_lexicon_tags_17 },
    { 0x24c47af3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea847afa, SUC_lexicon_tags_30 },
    { 0x37047afb, SUC_lexicon_tags_37 },
    { 0x45c47afc, SUC_lexicon_tags_25 },
    { 0xbd247afd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a47b02, SUC_lexicon_tags_43 },
    { 0xc6a47b02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e47b10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31647b17, SUC_lexicon_tags_4 },
    { 0x93a47b18, SUC_lexicon_tags_37 },
    { 0xdd647b19, SUC_lexicon_tags_11 },
    { 0xd2247b18, SUC_lexicon_tags_84 },
    { 0xb4847b1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80047b1d, SUC_lexicon_tags_46 },
    { 0x53647b1d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a47b24, SUC_lexicon_tags_47 },
    { 0x25647b25, SUC_lexicon_tags_25 },
    { 0x94e47b25, SUC_lexicon_tags_15 },
    { 0x86847b27, SUC_lexicon_tags_6 },
    { 0x78247b26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba47b2a, SUC_lexicon_tags_31 },
    { 0x5e47b2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36a47b2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf447b2f, SUC_lexicon_tags_19 },
    { 0x1647b2f, SUC_lexicon_tags_47 },
    { 0xc4c47b31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c47b36, SUC_lexicon_tags_17 },
    { 0x65c47b36, SUC_lexicon_tags_15 },
    { 0xbe047b37, SUC_lexicon_tags_12 },
    { 0x46247b39, SUC_lexicon_tags_8 },
    { 0x47447b3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c47b3f, SUC_lexicon_tags_8 },
    { 0xae847b3f, SUC_lexicon_tags_15 },
    { 0x24e47b41, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x33247b43, SUC_lexicon_tags_8 },
    { 0x51647b44, SUC_lexicon_tags_45 },
    { 0xeaa47b43, SUC_lexicon_tags_37 },
    { 0x18c47b46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57e47b48, SUC_lexicon_tags_9 },
    { 0xcde47b49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf5247b4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7647b54, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x77047b57, SUC_lexicon_tags_26 },
    { 0x1a47b58, SUC_lexicon_tags_29 },
    { 0x9c47b58, SUC_lexicon_tags_3 },
    { 0x4f647b5a, SUC_lexicon_tags_63 },
    { 0xd9447b5b, SUC_lexicon_tags_17 },
    { 0x56c47b5b, SUC_lexicon_tags_11 },
    { 0x54247b5c, SUC_lexicon_tags_2 },
    { 0xdd047b58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3247b61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3a47b63, SUC_lexicon_tags_65 },
    { 0xe3247b64, SUC_lexicon_tags_49 },
    { 0xba47b63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5647b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc47b6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2ce47b6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd847b72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c47b7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9847b80, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc47b84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc647b87, SUC_lexicon_tags_36 },
    { 0xc5247b88, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a47b8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30647b8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5247b92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca847b9c, SUC_lexicon_tags_37 },
    { 0x87a47b9d, SUC_lexicon_tags_12 },
    { 0xf1c47b9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60847ba2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a47ba9, SUC_lexicon_tags_12 },
    { 0x84647baa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e47bad, SUC_lexicon_tags_80 },
    { 0x59047bae, SUC_lexicon_tags_15 },
    { 0x5b047baf, SUC_lexicon_tags_25 },
    { 0xeba47bb0, SUC_lexicon_tags_7 },
    { 0xa3a47bae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1047bb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1be47bb6, SUC_lexicon_tags_12 },
    { 0xfcc47bb7, SUC_lexicon_tags_11 },
    { 0x64847bb7, SUC_lexicon_tags_31 },
    { 0xd7a47bb7, SUC_lexicon_tags_31 },
    { 0x5de47bba, SUC_lexicon_tags_15 },
    { 0xac247bb7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec647bbd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa1447bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e447bc2, SUC_lexicon_tags_31 },
    { 0xe6447bc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39047bc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30047bcd, SUC_lexicon_tags_17 },
    { 0x3aa47bce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea47bd3, SUC_lexicon_tags_19 },
    { 0x6da47bd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36047bd8, SUC_lexicon_tags_25 },
    { 0xbcc47bd9, SUC_lexicon_tags_31 },
    { 0x38047bd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa47bde, SUC_lexicon_tags_25 },
    { 0x14c47bde, SUC_lexicon_tags_37 },
    { 0x78c47bde, SUC_lexicon_tags_15 },
    { 0xf8c47be1, SUC_lexicon_tags_364 },
    { 0xafa47be2, SUC_lexicon_tags_19 },
    { 0x9fa47be0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x16047be5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a47beb, SUC_lexicon_tags_1 },
    { 0x2ee47bec, SUC_lexicon_tags_47 },
    { 0x99847bec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6247bf2, SUC_lexicon_tags_7 },
    { 0x83a47bf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2de47bf5, SUC_lexicon_tags_7 },
    { 0xd5647bf6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c47bfa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e47bfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab247c00, SUC_lexicon_tags_31 },
    { 0x84e47c01, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9a47c03, SUC_lexicon_tags_2 },
    { 0x25e47c04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a47c07, SUC_lexicon_tags_11 },
    { 0x57247c07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc47c0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdd647c0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7447c11, SUC_lexicon_tags_151 },
    { 0xf5047c12, SUC_lexicon_tags_16 },
    { 0xe0c47c11, SUC_lexicon_tags_23 },
    { 0x65c47c12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce247c1b, SUC_lexicon_tags_37 },
    { 0xa8047c1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e47c23, SUC_lexicon_tags_4 },
    { 0xc5e47c23, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xe3847c26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e47c29, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe7047c2b, SUC_lexicon_tags_65 },
    { 0x85a47c2c, SUC_lexicon_tags_12 },
    { 0xe3a47c2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9647c2f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e47c32, SUC_lexicon_tags_37 },
    { 0x97e47c33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21047c37, SUC_lexicon_tags_7 },
    { 0x1e047c38, SUC_lexicon_tags_25 },
    { 0xf5447c39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef447c40, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf9647c42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe47c45, SUC_lexicon_tags_7 },
    { 0xef247c45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9847c4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e47c4e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x55447c50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x85847c53, SUC_lexicon_tags_17 },
    { 0x5da47c54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247c5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac47c6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12047c6e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e47c72, SUC_lexicon_tags_38 },
    { 0xeb047c73, SUC_lexicon_tags_49 },
    { 0x13e47c74, SUC_lexicon_tags_17 },
    { 0xec847c74, SUC_lexicon_tags_37 },
    { 0xd9447c73, SUC_lexicon_tags_37 },
    { 0x6fa47c77, SUC_lexicon_tags_9 },
    { 0x1f647c73, SUC_lexicon_tags_19 },
    { 0xd6c47c79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d847c7b, SUC_lexicon_tags_11 },
    { 0x8a47c7b, SUC_lexicon_tags_120 },
    { 0x29a47c7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1c647c7f, SUC_lexicon_tags_11 },
    { 0xdbe47c80, SUC_lexicon_tags_1 },
    { 0xcc047c81, SUC_lexicon_tags_33 },
    { 0x17247c82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce647c85, SUC_lexicon_tags_63 },
    { 0xca247c86, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea47c8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x20247c8d, SUC_lexicon_tags_32 },
    { 0x94847c8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac47c93, SUC_lexicon_tags_17 },
    { 0x24847c94, SUC_lexicon_tags_17 },
    { 0x61e47c95, SUC_lexicon_tags_63 },
    { 0x4c047c96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc247c99, SUC_lexicon_tags_0 },
    { 0x36247c9a, SUC_lexicon_tags_17 },
    { 0x13a47c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72047ca0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x38e47ca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7047ca4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x13847ca7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1247ca9, SUC_lexicon_tags_37 },
    { 0x53447ca9, SUC_lexicon_tags_15 },
    { 0xa7047caa, SUC_lexicon_tags_17 },
    { 0x78e47cac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f247cae, SUC_lexicon_tags_33 },
    { 0xab647caf, SUC_lexicon_tags_82 },
    { 0xee847cb0, SUC_lexicon_tags_33 },
    { 0x13447cb1, SUC_lexicon_tags_29 },
    { 0xa6a47cb2, SUC_lexicon_tags_7 },
    { 0xd5e47cb3, SUC_lexicon_tags_25 },
    { 0x26e47cb2, SUC_lexicon_tags_23 },
    { 0xf2247cb5, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x21847cb7, SUC_lexicon_tags_37 },
    { 0x22647cb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23847cbc, SUC_lexicon_tags_15 },
    { 0x24c47cbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee47cc5, SUC_lexicon_tags_6 },
    { 0x6aa47cc6, SUC_lexicon_tags_2 },
    { 0x8ca47cc7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xa447cc9, SUC_lexicon_tags_12 },
    { 0xa1247cc9, SUC_lexicon_tags_12 },
    { 0x64447cc9, SUC_lexicon_tags_234 },
    { 0x8c047ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1047cce, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a47cd1, SUC_lexicon_tags_19 },
    { 0x72c47cd1, SUC_lexicon_tags_1 },
    { 0x8a247cd1, SUC_lexicon_tags_87 },
    { 0xa8047cd4, SUC_lexicon_tags_37 },
    { 0x49847cd1, SUC_lexicon_tags_32 },
    { 0x6f247cd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd847cda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa47cdd, SUC_lexicon_tags_31 },
    { 0x59847cdd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba47ce4, SUC_lexicon_tags_12 },
    { 0x8c47ce4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55247cea, SUC_lexicon_tags_55 },
    { 0x60647ceb, SUC_lexicon_tags_7 },
    { 0x44447cea, SUC_lexicon_tags_17 },
    { 0xd9247ced, SUC_lexicon_tags_7 },
    { 0xcf647cee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42247cf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde247cf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a47d03, SUC_lexicon_tags_87 },
    { 0xcc447d04, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0447d07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce47d0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd647d0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb047d15, SUC_lexicon_tags_6 },
    { 0x2be47d16, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1847d1b, SUC_lexicon_tags_17 },
    { 0xbcc47d1c, SUC_lexicon_tags_2 },
    { 0x20c47d1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47847d28, SUC_lexicon_tags_2 },
    { 0x93847d28, SUC_lexicon_tags_2 },
    { 0x2d047d2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc47d2f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6f847d31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65847d33, SUC_lexicon_tags_29 },
    { 0xa8447d34, SUC_lexicon_tags_15 },
    { 0x4c647d35, SUC_lexicon_tags_16 },
    { 0xf2247d36, SUC_lexicon_tags_21 },
    { 0x36a47d34, SUC_lexicon_tags_25 },
    { 0x50e47d38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5447d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaca47d3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3847d44, SUC_lexicon_tags_1 },
    { 0xa9247d44, SUC_lexicon_tags_19 },
    { 0xd4c47d46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3847d4e, SUC_lexicon_tags_45 },
    { 0xf1c47d4e, SUC_lexicon_tags_16 },
    { 0xf5c47d4e, SUC_lexicon_tags_0 },
    { 0x89847d51, SUC_lexicon_tags_6 },
    { 0x7b047d52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a47d56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65647d58, SUC_lexicon_tags_27 },
    { 0x1b647d58, SUC_lexicon_tags_2 },
    { 0x70647d58, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31247d65, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x42e47d67, SUC_lexicon_tags_15 },
    { 0x7f647d68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8847d6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a247d6c, SUC_lexicon_tags_15 },
    { 0x55847d6d, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa47d71, SUC_lexicon_tags_1 },
    { 0xb7447d72, SUC_lexicon_tags_31 },
    { 0x1a847d73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77847d76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c47d7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99647d8b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc47d91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec47d94, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce47d9f, SUC_lexicon_tags_17 },
    { 0x33647d9f, SUC_lexicon_tags_33 },
    { 0x6f647da1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e047da7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57247dab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1e47dad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79e47daf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c47db2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15047db7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72647dba, SUC_lexicon_tags_33 },
    { 0xfb447dbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda47dc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4847dc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe47dca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a647dce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x38047dd1, SUC_lexicon_tags_28 },
    { 0x2ea47dd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38047dd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c47dd8, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x39047dda, SUC_lexicon_tags_38 },
    { 0x5447ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f447ddf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda47de4, SUC_lexicon_tags_12 },
    { 0xf447de5, SUC_lexicon_tags_17 },
    { 0x91247de4, SUC_lexicon_tags_8 },
    { 0xe8447de7, SUC_lexicon_tags_19 },
    { 0xd5c47de4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99c47dea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a47df5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9447dfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e47dfe, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa47e01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d047e05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a47e08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x55447e0a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74647e0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0847e15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c047e18, SUC_lexicon_tags_12 },
    { 0xfde47e19, SUC_lexicon_tags_25 },
    { 0x59847e18, SUC_lexicon_tags_12 },
    { 0x3a647e1a, SUC_lexicon_tags_48 },
    { 0x8ee47e1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e47e22, SUC_lexicon_tags_11 },
    { 0x4e847e23, SUC_lexicon_tags_5 },
    { 0x5c047e22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa4647e26, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x73447e28, SUC_lexicon_tags_11 },
    { 0xf7e47e29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaae47e2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c47e2f, SUC_lexicon_tags_1 },
    { 0x65447e30, SUC_lexicon_tags_10 },
    { 0x6f847e31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e47e36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce47e39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47e3d, SUC_lexicon_tags_17 },
    { 0xd6a47e3e, SUC_lexicon_tags_1 },
    { 0x32a47e3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7647e41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x52847e43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f047e47, SUC_lexicon_tags_17 },
    { 0xa4047e47, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc47e4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x86647e50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a847e53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9fa47e55, SUC_lexicon_tags_138 },
    { 0x2de47e56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc647e59, SUC_lexicon_tags_1 },
    { 0x19247e59, SUC_lexicon_tags_12 },
    { 0x4647e5a, SUC_lexicon_tags_37 },
    { 0xfe847e5a, SUC_lexicon_tags_37 },
    { 0x2f647e59, SUC_lexicon_tags_37 },
    { 0x21c47e5e, SUC_lexicon_tags_25 },
    { 0xd3847e5a, SUC_lexicon_tags_60 },
    { 0xf4e47e60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0e47e62, SUC_lexicon_tags_25 },
    { 0x3c447e62, SUC_lexicon_tags_25 },
    { 0x69647e62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c247e66, SUC_lexicon_tags_17 },
    { 0xa4247e67, SUC_lexicon_tags_21 },
    { 0x47c47e68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce47e6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb247e6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14047e72, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99447e77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a047e7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf447e7d, SUC_lexicon_tags_9 },
    { 0x5ae47e7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb247e81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d847e86, SUC_lexicon_tags_1 },
    { 0x89447e86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b047e89, SUC_lexicon_tags_19 },
    { 0xaa47e89, SUC_lexicon_tags_29 },
    { 0x18e47e89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x75247e8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91047e90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc247e92, SUC_lexicon_tags_17 },
    { 0x37c47e92, SUC_lexicon_tags_2 },
    { 0x81c47e94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b847e98, SUC_lexicon_tags_7 },
    { 0x52847e99, SUC_lexicon_tags_45 },
    { 0xc3a47e9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a247ea1, SUC_lexicon_tags_12 },
    { 0x76a47ea1, SUC_lexicon_tags_17 },
    { 0x1a847ea2, SUC_lexicon_tags_11 },
    { 0x4f247ea1, SUC_lexicon_tags_37 },
    { 0xfb847ea5, SUC_lexicon_tags_15 },
    { 0x17047ea5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76047ea8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf647eab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2d247ead, SUC_lexicon_tags_28 },
    { 0xf6247eae, SUC_lexicon_tags_11 },
    { 0xac247eaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e247eb6, SUC_lexicon_tags_14 },
    { 0x35e47eb6, SUC_lexicon_tags_63 },
    { 0x17247eb8, SUC_lexicon_tags_7 },
    { 0x76c47eb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a47ebf, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x20447ec2, SUC_lexicon_tags_15 },
    { 0x6e047ec2, SUC_lexicon_tags_2 },
    { 0x60847ec2, SUC_lexicon_tags_2 },
    { 0x16047ec5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x44e47ec7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf247ec9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa047ecf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6c47ed1, SUC_lexicon_tags_25 },
    { 0xa2447ed2, SUC_lexicon_tags_17 },
    { 0x99047ed3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb047ed7, SUC_lexicon_tags_56 },
    { 0x86247ed7, SUC_lexicon_tags_37 },
    { 0xb3c47ed9, SUC_lexicon_tags_37 },
    { 0x87447ed8, SUC_lexicon_tags_17 },
    { 0xaac47ed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a247ede, SUC_lexicon_tags_82 },
    { 0xaf247edf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a847ee3, SUC_lexicon_tags_31 },
    { 0xbc847ee4, SUC_lexicon_tags_7 },
    { 0xe4a47ee3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0447ee8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x66a47eea, SUC_lexicon_tags_1 },
    { 0x5f647eeb, SUC_lexicon_tags_17 },
    { 0x95847eeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e447ef2, SUC_lexicon_tags_87 },
    { 0xc8c47ef3, SUC_lexicon_tags_10 },
    { 0xf8247ef4, SUC_lexicon_tags_12 },
    { 0x73047ef5, SUC_lexicon_tags_0 },
    { 0x5c647ef4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e847ef8, SUC_lexicon_tags_10 },
    { 0x61047ef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7247efd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc47f02, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18647f09, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c47f0f, SUC_lexicon_tags_68 },
    { 0x3b847f10, SUC_lexicon_tags_7 },
    { 0xca047f10, SUC_lexicon_tags_30 },
    { 0xe3647f12, SUC_lexicon_tags_47 },
    { 0xdc847f13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64247f16, SUC_lexicon_tags_17 },
    { 0x2f047f17, SUC_lexicon_tags_28 },
    { 0x32247f17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36647f1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e47f20, SUC_lexicon_tags_16 },
    { 0xf847f21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b047f24, SUC_lexicon_tags_46 },
    { 0xd4c47f24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb847f27, SUC_lexicon_tags_58 },
    { 0xb0047f28, SUC_lexicon_tags_37 },
    { 0xc8247f28, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e47f2d, SUC_lexicon_tags_34 },
    { 0xd3047f2e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4c247f30, SUC_lexicon_tags_2 },
    { 0x74647f31, SUC_lexicon_tags_43 },
    { 0xc5e47f31, SUC_lexicon_tags_47 },
    { 0x2ce47f32, SUC_lexicon_tags_47 },
    { 0x65a47f34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7447f37, SUC_lexicon_tags_1680 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63447f3e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe5c47f40, SUC_lexicon_tags_130 },
    { 0xf8847f41, SUC_lexicon_tags_45 },
    { 0x1ca47f40, SUC_lexicon_tags_87 },
    { 0x5b047f43, SUC_lexicon_tags_63 },
    { 0xb9047f43, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4847f49, SUC_lexicon_tags_179 },
    { 0xafa47f4a, SUC_lexicon_tags_31 },
    { 0x70c47f4b, SUC_lexicon_tags_17 },
    { 0x33647f4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d847f50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45447f54, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc9a47f56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x79a47f58, SUC_lexicon_tags_57 },
    { 0xdae47f59, SUC_lexicon_tags_17 },
    { 0xd0447f5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5447f5c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24847f60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b247f62, SUC_lexicon_tags_25 },
    { 0xf7647f62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x72447f65, SUC_lexicon_tags_60 },
    { 0xd8647f65, SUC_lexicon_tags_37 },
    { 0xefe47f65, SUC_lexicon_tags_47 },
    { 0xed047f68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x65247f6b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee847f6d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12e47f6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17047f72, SUC_lexicon_tags_15 },
    { 0xbe247f72, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b847f76, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6047f79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x70247f7b, SUC_lexicon_tags_174 },
    { 0xab847f7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40247f80, SUC_lexicon_tags_27 },
    { 0x5c247f81, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c47f84, SUC_lexicon_tags_33 },
    { 0xd7c47f84, SUC_lexicon_tags_37 },
    { 0xa8647f86, SUC_lexicon_tags_25 },
    { 0x1b847f84, SUC_lexicon_tags_37 },
    { 0xe5e47f84, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x67047f8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x13447f8d, SUC_lexicon_tags_27 },
    { 0x9ec47f8e, SUC_lexicon_tags_25 },
    { 0xeae47f8f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6647f94, SUC_lexicon_tags_37 },
    { 0xeda47f95, SUC_lexicon_tags_19 },
    { 0x17e47f96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d447f9b, SUC_lexicon_tags_194 },
    { 0xe8447f9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe047f9f, SUC_lexicon_tags_56 },
    { 0x71847fa0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc47fa7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c47fab, SUC_lexicon_tags_19 },
    { 0x95847fac, SUC_lexicon_tags_65 },
    { 0xf5647fac, SUC_lexicon_tags_2 },
    { 0xcfe47fad, SUC_lexicon_tags_2 },
    { 0xe0247faf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba47fb5, SUC_lexicon_tags_25 },
    { 0x24847fb5, SUC_lexicon_tags_23 },
    { 0xb1e47fb5, SUC_lexicon_tags_1 },
    { 0x1a247fb8, SUC_lexicon_tags_55 },
    { 0x59e47fb9, SUC_lexicon_tags_55 },
    { 0xabc47fba, SUC_lexicon_tags_85 },
    { 0x6cc47fbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3247fc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8aa47fc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3847fc5, SUC_lexicon_tags_19 },
    { 0x8ae47fc6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x43647fc8, SUC_lexicon_tags_17 },
    { 0x8447fc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88a47fcb, SUC_lexicon_tags_1 },
    { 0x1f847fcc, SUC_lexicon_tags_47 },
    { 0xc1447fcd, SUC_lexicon_tags_5 },
    { 0x4047fcd, SUC_lexicon_tags_15 },
    { 0xebc47fcc, SUC_lexicon_tags_1 },
    { 0x26e47fd0, SUC_lexicon_tags_10 },
    { 0xb8847fcf, SUC_lexicon_tags_9 },
    { 0x3e847fd2, SUC_lexicon_tags_14 },
    { 0xd7647fd2, SUC_lexicon_tags_14 },
    { 0x6e047fd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da47fd8, SUC_lexicon_tags_64 },
    { 0x82a47fd9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4447fdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f047fde, SUC_lexicon_tags_37 },
    { 0xc2847fdf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1447fe2, SUC_lexicon_tags_8 },
    { 0x12847fe2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7047fe7, SUC_lexicon_tags_7 },
    { 0xec447fe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a47fef, SUC_lexicon_tags_63 },
    { 0x1ee47fef, SUC_lexicon_tags_11 },
    { 0xa2447fef, SUC_lexicon_tags_11 },
    { 0xb5647ff2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd247ff7, SUC_lexicon_tags_37 },
    { 0xd047ff8, SUC_lexicon_tags_11 },
    { 0xbac47ff7, SUC_lexicon_tags_37 },
    { 0x32047ff8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x63c47ffc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce247fff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42648001, SUC_lexicon_tags_15 },
    { 0x57a48002, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea48006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3024800f, SUC_lexicon_tags_15 },
    { 0xdec48010, SUC_lexicon_tags_39 },
    { 0xcc648011, SUC_lexicon_tags_170 },
    { 0x3da48012, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe0248014, SUC_lexicon_tags_1518 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c48017, SUC_lexicon_tags_1 },
    { 0x6ee48018, SUC_lexicon_tags_15 },
    { 0xf6648017, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c4801c, SUC_lexicon_tags_17 },
    { 0xf304801d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2124801f, SUC_lexicon_tags_114 },
    { 0xea648020, SUC_lexicon_tags_2 },
    { 0x47048020, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a48024, SUC_lexicon_tags_17 },
    { 0xb4a48025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63048027, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb964802c, SUC_lexicon_tags_19 },
    { 0xdb84802d, SUC_lexicon_tags_15 },
    { 0xcac4802e, SUC_lexicon_tags_26 },
    { 0xa284802e, SUC_lexicon_tags_37 },
    { 0xd944802c, SUC_lexicon_tags_38 },
    { 0x51048031, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa48033, SUC_lexicon_tags_38 },
    { 0xee048034, SUC_lexicon_tags_49 },
    { 0xeb848035, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x65648038, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe344803d, SUC_lexicon_tags_107 },
    { 0x7a44803e, SUC_lexicon_tags_15 },
    { 0xdfe4803f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8648043, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc448049, SUC_lexicon_tags_1686 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3644804e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1ee48050, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29248055, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd584805b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa264805d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3248060, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7648062, SUC_lexicon_tags_11 },
    { 0x1848062, SUC_lexicon_tags_33 },
    { 0x2848064, SUC_lexicon_tags_61 },
    { 0x68a48063, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca4806a, SUC_lexicon_tags_1 },
    { 0x95c4806a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe4806f, SUC_lexicon_tags_29 },
    { 0xf9c48070, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a48076, SUC_lexicon_tags_56 },
    { 0xecc48076, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x41248079, SUC_lexicon_tags_84 },
    { 0xb5448079, SUC_lexicon_tags_16 },
    { 0xdd84807b, SUC_lexicon_tags_5 },
    { 0x5964807c, SUC_lexicon_tags_38 },
    { 0xd124807d, SUC_lexicon_tags_2 },
    { 0x2044807e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa9e48080, SUC_lexicon_tags_16 },
    { 0x5ba48081, SUC_lexicon_tags_25 },
    { 0x4a48082, SUC_lexicon_tags_37 },
    { 0x91848083, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x70648086, SUC_lexicon_tags_2 },
    { 0xdae48086, SUC_lexicon_tags_2 },
    { 0xabe48088, SUC_lexicon_tags_112 },
    { 0xb6248089, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc48092, SUC_lexicon_tags_31 },
    { 0xa9a48093, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92448097, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde4809a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e480a1, SUC_lexicon_tags_27 },
    { 0x514480a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x542480a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194480aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba480ad, SUC_lexicon_tags_158 },
    { 0x348480ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6480b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802480bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e480c0, SUC_lexicon_tags_2 },
    { 0x2d8480c0, SUC_lexicon_tags_68 },
    { 0x6d8480c0, SUC_lexicon_tags_29 },
    { 0x92480c3, SUC_lexicon_tags_37 },
    { 0x606480c4, SUC_lexicon_tags_7 },
    { 0xe38480c5, SUC_lexicon_tags_7 },
    { 0x344480c4, SUC_lexicon_tags_66 },
    { 0xe36480c7, SUC_lexicon_tags_60 },
    { 0x502480c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x426480cb, SUC_lexicon_tags_30 },
    { 0x216480cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342480d1, SUC_lexicon_tags_32 },
    { 0xbb8480d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa480d7, SUC_lexicon_tags_69 },
    { 0xff4480d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6480df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x890480e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2480ec, SUC_lexicon_tags_7 },
    { 0x9be480ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0480f1, SUC_lexicon_tags_45 },
    { 0xc90480f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0480f7, SUC_lexicon_tags_9 },
    { 0x1a2480f8, SUC_lexicon_tags_15 },
    { 0x272480f8, SUC_lexicon_tags_17 },
    { 0x1ea480f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f648107, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9544810b, SUC_lexicon_tags_2 },
    { 0x49e4810c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9448112, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x61248115, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a4811b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb44811e, SUC_lexicon_tags_23 },
    { 0xb884811f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6c48121, SUC_lexicon_tags_27 },
    { 0x11c48121, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc6c48124, SUC_lexicon_tags_27 },
    { 0x8fe48124, SUC_lexicon_tags_11 },
    { 0x71248126, SUC_lexicon_tags_15 },
    { 0xf0648127, SUC_lexicon_tags_15 },
    { 0xe5a48128, SUC_lexicon_tags_28 },
    { 0x37448129, SUC_lexicon_tags_65 },
    { 0x86848128, SUC_lexicon_tags_49 },
    { 0xeb048125, SUC_lexicon_tags_10 },
    { 0x13c4812c, SUC_lexicon_tags_35 },
    { 0x42648125, SUC_lexicon_tags_65 },
    { 0x5e848126, SUC_lexicon_tags_11 },
    { 0x97848129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34848133, SUC_lexicon_tags_45 },
    { 0x2fc48133, SUC_lexicon_tags_2 },
    { 0xe5248135, SUC_lexicon_tags_29 },
    { 0xf4048136, SUC_lexicon_tags_13 },
    { 0x9e48137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2448139, SUC_lexicon_tags_68 },
    { 0x20248139, SUC_lexicon_tags_17 },
    { 0xfee4813b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae48142, SUC_lexicon_tags_17 },
    { 0xf7848143, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e48149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a4814d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50248150, SUC_lexicon_tags_17 },
    { 0x4fc48151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3648154, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xeda48156, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e4815c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59048163, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16048165, SUC_lexicon_tags_49 },
    { 0xf5a48166, SUC_lexicon_tags_25 },
    { 0xe6e48167, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe744816a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfce4816c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3048174, SUC_lexicon_tags_23 },
    { 0x32448175, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb048178, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe4817b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x64e4817d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a4818f, SUC_lexicon_tags_34 },
    { 0x3e248190, SUC_lexicon_tags_37 },
    { 0x29e48191, SUC_lexicon_tags_761 },
    { 0xcb04818f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb648194, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x20c48196, SUC_lexicon_tags_34 },
    { 0x87248196, SUC_lexicon_tags_12 },
    { 0x7b648198, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa804819c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a481a0, SUC_lexicon_tags_23 },
    { 0x37c481a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c481a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14481a8, SUC_lexicon_tags_4 },
    { 0xcd4481a9, SUC_lexicon_tags_15 },
    { 0x506481aa, SUC_lexicon_tags_15 },
    { 0x4c8481ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x964481ae, SUC_lexicon_tags_25 },
    { 0x352481af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a2481b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa481b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a481bb, SUC_lexicon_tags_15 },
    { 0x702481bb, SUC_lexicon_tags_6 },
    { 0xc68481bd, SUC_lexicon_tags_1 },
    { 0x64a481bd, SUC_lexicon_tags_14 },
    { 0x7e8481bc, SUC_lexicon_tags_26 },
    { 0xbe2481bb, SUC_lexicon_tags_60 },
    { 0x1b6481c1, SUC_lexicon_tags_11 },
    { 0xf66481c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6fa481c4, SUC_lexicon_tags_2 },
    { 0x752481c5, SUC_lexicon_tags_23 },
    { 0x4d2481c6, SUC_lexicon_tags_49 },
    { 0x3c4481c6, SUC_lexicon_tags_31 },
    { 0xa1a481c7, SUC_lexicon_tags_63 },
    { 0x1a6481c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6481d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8481d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e8481d9, SUC_lexicon_tags_12 },
    { 0x984481d9, SUC_lexicon_tags_42 },
    { 0x72481da, SUC_lexicon_tags_17 },
    { 0x668481dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c481e1, SUC_lexicon_tags_10 },
    { 0x82481e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8481e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92481e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e481ec, SUC_lexicon_tags_43 },
    { 0x294481ec, SUC_lexicon_tags_57 },
    { 0x178481ec, SUC_lexicon_tags_1 },
    { 0xb5a481ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8481f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e481f6, SUC_lexicon_tags_17 },
    { 0x4d8481f7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8481fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14048200, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x48e48202, SUC_lexicon_tags_31 },
    { 0x8be48203, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6048209, SUC_lexicon_tags_16 },
    { 0x4a24820a, SUC_lexicon_tags_11 },
    { 0x8184820b, SUC_lexicon_tags_1 },
    { 0x6dc4820c, SUC_lexicon_tags_17 },
    { 0xd0c4820c, SUC_lexicon_tags_12 },
    { 0x3944820d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee648216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef248218, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6848221, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4048224, SUC_lexicon_tags_6 },
    { 0xa0c48224, SUC_lexicon_tags_14 },
    { 0x53048226, SUC_lexicon_tags_91 },
    { 0xae248227, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85848229, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a64822c, SUC_lexicon_tags_33 },
    { 0x4924822d, SUC_lexicon_tags_87 },
    { 0xe3e4822e, SUC_lexicon_tags_25 },
    { 0x2ca4822f, SUC_lexicon_tags_15 },
    { 0xe024822f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5c48232, SUC_lexicon_tags_1 },
    { 0xcea48232, SUC_lexicon_tags_65 },
    { 0x81c48234, SUC_lexicon_tags_8 },
    { 0x7d448234, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6304823e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6848246, SUC_lexicon_tags_68 },
    { 0xd7248246, SUC_lexicon_tags_5 },
    { 0x67848248, SUC_lexicon_tags_25 },
    { 0xd048248, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9484824c, SUC_lexicon_tags_34 },
    { 0xd704824c, SUC_lexicon_tags_25 },
    { 0xe724824c, SUC_lexicon_tags_17 },
    { 0xf484824c, SUC_lexicon_tags_23 },
    { 0x11848250, SUC_lexicon_tags_7 },
    { 0xc7248251, SUC_lexicon_tags_0 },
    { 0x7d048252, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae648257, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c4825f, SUC_lexicon_tags_47 },
    { 0xc464825f, SUC_lexicon_tags_16 },
    { 0x1da48261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba48266, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e48269, SUC_lexicon_tags_23 },
    { 0xe744826a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x51e4826c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c4826f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3048273, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb448277, SUC_lexicon_tags_1 },
    { 0x8248277, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf84827a, SUC_lexicon_tags_7 },
    { 0xd544827b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52048282, SUC_lexicon_tags_23 },
    { 0xf0048283, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c48287, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x384828a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc204828c, SUC_lexicon_tags_38 },
    { 0xed44828d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc284828f, SUC_lexicon_tags_108 },
    { 0xa624828f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb448298, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe164829b, SUC_lexicon_tags_19 },
    { 0x5a44829c, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e4829f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992482a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84482a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0482ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x526482af, SUC_lexicon_tags_4 },
    { 0x3b6482af, SUC_lexicon_tags_25 },
    { 0x940482b1, SUC_lexicon_tags_1 },
    { 0xa8c482b0, SUC_lexicon_tags_32 },
    { 0x916482b3, SUC_lexicon_tags_2 },
    { 0x1fc482b2, SUC_lexicon_tags_17 },
    { 0xa42482b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x868482b7, SUC_lexicon_tags_17 },
    { 0x138482b7, SUC_lexicon_tags_40 },
    { 0x4fe482b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5ae482bb, SUC_lexicon_tags_77 },
    { 0xf3c482bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c482bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8482c3, SUC_lexicon_tags_37 },
    { 0xf32482c4, SUC_lexicon_tags_496 },
    { 0xbac482c4, SUC_lexicon_tags_12 },
    { 0x334482c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ec482c8, SUC_lexicon_tags_27 },
    { 0x72482c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a482cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb82482d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc482d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe50482d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c482da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82482e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb20482e2, SUC_lexicon_tags_9 },
    { 0x166482e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc482e6, SUC_lexicon_tags_30 },
    { 0x86482e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c482ec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04482f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6dc482f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da482f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62482fa, SUC_lexicon_tags_4 },
    { 0x6d2482fb, SUC_lexicon_tags_33 },
    { 0x31a482fc, SUC_lexicon_tags_7 },
    { 0x440482fd, SUC_lexicon_tags_9 },
    { 0x1b6482fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x46248300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63048306, SUC_lexicon_tags_2 },
    { 0x40c48306, SUC_lexicon_tags_34 },
    { 0x8848308, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2304830c, SUC_lexicon_tags_49 },
    { 0xf3e4830c, SUC_lexicon_tags_43 },
    { 0x4064830e, SUC_lexicon_tags_1 },
    { 0xfd44830f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda48312, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x76448314, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec48318, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a48320, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6048326, SUC_lexicon_tags_4 },
    { 0x49a48327, SUC_lexicon_tags_19 },
    { 0x51e48328, SUC_lexicon_tags_29 },
    { 0x94648326, SUC_lexicon_tags_15 },
    { 0xe2248326, SUC_lexicon_tags_33 },
    { 0xa004832b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7564832e, SUC_lexicon_tags_25 },
    { 0x904832f, SUC_lexicon_tags_15 },
    { 0xa2a48330, SUC_lexicon_tags_38 },
    { 0x34c48331, SUC_lexicon_tags_11 },
    { 0x9dc48332, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c048337, SUC_lexicon_tags_47 },
    { 0xd4648338, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e4833a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd604833c, SUC_lexicon_tags_1 },
    { 0xb464833d, SUC_lexicon_tags_37 },
    { 0x3224833d, SUC_lexicon_tags_43 },
    { 0x99a4833f, SUC_lexicon_tags_39 },
    { 0x57c4833d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76248347, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe04834a, SUC_lexicon_tags_27 },
    { 0xdbc4834b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc224834f, SUC_lexicon_tags_15 },
    { 0x28c48350, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b64835d, SUC_lexicon_tags_43 },
    { 0x2444835e, SUC_lexicon_tags_12 },
    { 0xd624835d, SUC_lexicon_tags_1 },
    { 0x6744835f, SUC_lexicon_tags_1 },
    { 0x2944835e, SUC_lexicon_tags_19 },
    { 0x44e48360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f848369, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5848371, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb048376, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e48379, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c648380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39248382, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc48386, SUC_lexicon_tags_38 },
    { 0x27a48386, SUC_lexicon_tags_10 },
    { 0x15848388, SUC_lexicon_tags_2 },
    { 0xe2c48388, SUC_lexicon_tags_27 },
    { 0x6584838a, SUC_lexicon_tags_87 },
    { 0xd5e48388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0448390, SUC_lexicon_tags_9 },
    { 0x77848391, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35448396, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20848399, SUC_lexicon_tags_3 },
    { 0x65048399, SUC_lexicon_tags_15 },
    { 0x8a648399, SUC_lexicon_tags_46 },
    { 0xc064839c, SUC_lexicon_tags_1 },
    { 0x7c04839d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8483a1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70483a4, SUC_lexicon_tags_25 },
    { 0xf38483a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x410483a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490483ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0483b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x946483b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa483b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894483bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x150483c1, SUC_lexicon_tags_14 },
    { 0x800483c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52483c5, SUC_lexicon_tags_37 },
    { 0xb68483c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848483cb, SUC_lexicon_tags_21 },
    { 0x52a483cc, SUC_lexicon_tags_4 },
    { 0x74e483cd, SUC_lexicon_tags_14 },
    { 0x8fc483cc, SUC_lexicon_tags_11 },
    { 0xf62483cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc483d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2b6483d4, SUC_lexicon_tags_7 },
    { 0x724483d4, SUC_lexicon_tags_15 },
    { 0xbe8483d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2dc483d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c483db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x622483de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa72483e0, SUC_lexicon_tags_39 },
    { 0x332483e0, SUC_lexicon_tags_34 },
    { 0x290483e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6483e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x87e483e8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x774483ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7de483ec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x42c483ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c483f2, SUC_lexicon_tags_11 },
    { 0xfc4483f3, SUC_lexicon_tags_25 },
    { 0x5b0483f4, SUC_lexicon_tags_7 },
    { 0xe1c483f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x37e483f8, SUC_lexicon_tags_15 },
    { 0x35c483f9, SUC_lexicon_tags_23 },
    { 0x4e8483fa, SUC_lexicon_tags_37 },
    { 0x628483fb, SUC_lexicon_tags_11 },
    { 0x142483fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x188483fe, SUC_lexicon_tags_15 },
    { 0x444483fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf3a48401, SUC_lexicon_tags_19 },
    { 0x84848401, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49a48404, SUC_lexicon_tags_12 },
    { 0xd9648405, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x18e48407, SUC_lexicon_tags_1 },
    { 0xfd248408, SUC_lexicon_tags_48 },
    { 0x12a48408, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c84840b, SUC_lexicon_tags_37 },
    { 0xc904840c, SUC_lexicon_tags_0 },
    { 0x284840b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8a64840f, SUC_lexicon_tags_37 },
    { 0x3fa48410, SUC_lexicon_tags_46 },
    { 0x43a48411, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f048414, SUC_lexicon_tags_47 },
    { 0xc048415, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9c48417, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb448421, SUC_lexicon_tags_6 },
    { 0xa3e48422, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x74048424, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e48428, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe764842c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75848430, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a48436, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35a48438, SUC_lexicon_tags_47 },
    { 0x11048439, SUC_lexicon_tags_9 },
    { 0xc844843a, SUC_lexicon_tags_47 },
    { 0xcec4843b, SUC_lexicon_tags_9 },
    { 0x5d848438, SUC_lexicon_tags_15 },
    { 0x51c4843d, SUC_lexicon_tags_12 },
    { 0x6d44843d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaa48440, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2aa48442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e48445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f248447, SUC_lexicon_tags_28 },
    { 0xc7848448, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd444844b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c4844e, SUC_lexicon_tags_6 },
    { 0xefc4844f, SUC_lexicon_tags_2 },
    { 0xf9c48450, SUC_lexicon_tags_15 },
    { 0xa8648450, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a048454, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c48457, SUC_lexicon_tags_26 },
    { 0x4f448458, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2784845c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8648460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d848463, SUC_lexicon_tags_112 },
    { 0x8f248464, SUC_lexicon_tags_2 },
    { 0xdec48464, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8c448467, SUC_lexicon_tags_23 },
    { 0xc5e48468, SUC_lexicon_tags_17 },
    { 0xf8248467, SUC_lexicon_tags_11 },
    { 0xaea4846a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc4846d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0048472, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a448474, SUC_lexicon_tags_39 },
    { 0xbe048474, SUC_lexicon_tags_21 },
    { 0xce048476, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c4847a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a4847e, SUC_lexicon_tags_2 },
    { 0x61a4847f, SUC_lexicon_tags_43 },
    { 0xca648480, SUC_lexicon_tags_12 },
    { 0x36e48481, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3e48483, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c48489, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5744848d, SUC_lexicon_tags_42 },
    { 0xaf44848e, SUC_lexicon_tags_21 },
    { 0x1fa4848f, SUC_lexicon_tags_32 },
    { 0x8aa4848d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c048493, SUC_lexicon_tags_17 },
    { 0x47c48494, SUC_lexicon_tags_12 },
    { 0xae48495, SUC_lexicon_tags_25 },
    { 0xe7c48496, SUC_lexicon_tags_17 },
    { 0x2c848497, SUC_lexicon_tags_13 },
    { 0xedc48496, SUC_lexicon_tags_19 },
    { 0x48248499, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2984849b, SUC_lexicon_tags_17 },
    { 0xab44849c, SUC_lexicon_tags_8 },
    { 0x49e4849b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576484a2, SUC_lexicon_tags_7 },
    { 0x3e0484a2, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x62a484a5, SUC_lexicon_tags_25 },
    { 0x90e484a6, SUC_lexicon_tags_43 },
    { 0xa40484a7, SUC_lexicon_tags_234 },
    { 0x19e484a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde484ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8484ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x68e484b0, SUC_lexicon_tags_12 },
    { 0xd38484b1, SUC_lexicon_tags_45 },
    { 0xfb2484b2, SUC_lexicon_tags_33 },
    { 0xaf8484b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18484b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704484bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x640484c0, SUC_lexicon_tags_95 },
    { 0x386484c1, SUC_lexicon_tags_17 },
    { 0x200484c2, SUC_lexicon_tags_47 },
    { 0x616484c3, SUC_lexicon_tags_14 },
    { 0xd0484c4, SUC_lexicon_tags_7 },
    { 0x400484c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa26484c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4c2484c9, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xc4c484cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x672484ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x962484d0, SUC_lexicon_tags_61 },
    { 0x184484d1, SUC_lexicon_tags_8 },
    { 0x1ea484d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244484da, SUC_lexicon_tags_11 },
    { 0xbf0484db, SUC_lexicon_tags_47 },
    { 0xf54484db, SUC_lexicon_tags_11 },
    { 0xbf8484dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe484df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394484e7, SUC_lexicon_tags_16 },
    { 0x9ec484e8, SUC_lexicon_tags_25 },
    { 0x4f8484e7, SUC_lexicon_tags_743 },
    { 0, NULL },
    { 0xed2484eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e484f0, SUC_lexicon_tags_25 },
    { 0x392484f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae484fb, SUC_lexicon_tags_11 },
    { 0xd3e484fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x902484fe, SUC_lexicon_tags_25 },
    { 0xaa6484ff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16448504, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a48507, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xc1a48509, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7dc4850b, SUC_lexicon_tags_21 },
    { 0x2384850c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27648513, SUC_lexicon_tags_47 },
    { 0xdba48513, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b448519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc48523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5e48525, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6848528, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad04852a, SUC_lexicon_tags_7 },
    { 0xf0e4852b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd24852e, SUC_lexicon_tags_37 },
    { 0x14c4852f, SUC_lexicon_tags_33 },
    { 0xc4248530, SUC_lexicon_tags_34 },
    { 0x77048531, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb4848533, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0048536, SUC_lexicon_tags_33 },
    { 0x6a48537, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e4853e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82e48540, SUC_lexicon_tags_60 },
    { 0x61448540, SUC_lexicon_tags_28 },
    { 0x6de48542, SUC_lexicon_tags_10 },
    { 0xbfc48543, SUC_lexicon_tags_29 },
    { 0x3848544, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf7448546, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4aa48548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a4854b, SUC_lexicon_tags_15 },
    { 0xd924854c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1848554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26248557, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a24855b, SUC_lexicon_tags_31 },
    { 0x6964855b, SUC_lexicon_tags_12 },
    { 0xf4e4855d, SUC_lexicon_tags_1 },
    { 0xe824855d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xea248560, SUC_lexicon_tags_15 },
    { 0x37e48560, SUC_lexicon_tags_81 },
    { 0xd1248561, SUC_lexicon_tags_11 },
    { 0xf5648562, SUC_lexicon_tags_0 },
    { 0x29c48562, SUC_lexicon_tags_2 },
    { 0x47448565, SUC_lexicon_tags_45 },
    { 0x73c48566, SUC_lexicon_tags_38 },
    { 0xdee48565, SUC_lexicon_tags_0 },
    { 0x65c48568, SUC_lexicon_tags_43 },
    { 0x1c48569, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5964856b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b848578, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeb84857a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7244857d, SUC_lexicon_tags_11 },
    { 0x4aa4857e, SUC_lexicon_tags_15 },
    { 0x3584857f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab248581, SUC_lexicon_tags_17 },
    { 0x25e48581, SUC_lexicon_tags_7 },
    { 0x7de48583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4848587, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7244858a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf144858f, SUC_lexicon_tags_69 },
    { 0x8b648590, SUC_lexicon_tags_37 },
    { 0x3a448591, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbaa48593, SUC_lexicon_tags_2 },
    { 0x40648593, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x4a048596, SUC_lexicon_tags_79 },
    { 0x5e48597, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1344859a, SUC_lexicon_tags_26 },
    { 0x14a4859b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc684859d, SUC_lexicon_tags_33 },
    { 0x2ea4859d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x512485a1, SUC_lexicon_tags_14 },
    { 0x8ca485a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2485a8, SUC_lexicon_tags_1 },
    { 0xc52485a8, SUC_lexicon_tags_43 },
    { 0x7f0485aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8485ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392485b1, SUC_lexicon_tags_25 },
    { 0xb66485b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146485b7, SUC_lexicon_tags_17 },
    { 0x8a2485b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd06485ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2485bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x130485c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2485c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c485ca, SUC_lexicon_tags_25 },
    { 0x668485cb, SUC_lexicon_tags_25 },
    { 0x822485cc, SUC_lexicon_tags_6 },
    { 0x5d4485cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0485d5, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50485dd, SUC_lexicon_tags_37 },
    { 0x92a485de, SUC_lexicon_tags_25 },
    { 0x7c2485df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x996485e1, SUC_lexicon_tags_0 },
    { 0x148485e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c485e6, SUC_lexicon_tags_25 },
    { 0xce0485e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea6485e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332485ed, SUC_lexicon_tags_59 },
    { 0x964485ee, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x146485f0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcae485f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf36485f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820485f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb44485fa, SUC_lexicon_tags_45 },
    { 0xcf6485fa, SUC_lexicon_tags_7 },
    { 0xa5a485fc, SUC_lexicon_tags_1 },
    { 0xbfa485fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8648600, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c48603, SUC_lexicon_tags_45 },
    { 0x65248604, SUC_lexicon_tags_57 },
    { 0xbde48605, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca48608, SUC_lexicon_tags_25 },
    { 0xb3a48608, SUC_lexicon_tags_31 },
    { 0xf544860a, SUC_lexicon_tags_4 },
    { 0xac4860b, SUC_lexicon_tags_37 },
    { 0x2724860b, SUC_lexicon_tags_15 },
    { 0x5d44860d, SUC_lexicon_tags_11 },
    { 0xca24860e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e48612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c48619, SUC_lexicon_tags_25 },
    { 0xa3a4861a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e4861d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35248620, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9248623, SUC_lexicon_tags_37 },
    { 0x54848624, SUC_lexicon_tags_1 },
    { 0xdd048625, SUC_lexicon_tags_21 },
    { 0xb6648625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea048628, SUC_lexicon_tags_28 },
    { 0x67c48629, SUC_lexicon_tags_7 },
    { 0x2a448628, SUC_lexicon_tags_32 },
    { 0xbfe4862b, SUC_lexicon_tags_25 },
    { 0xc484862a, SUC_lexicon_tags_60 },
    { 0xa24862a, SUC_lexicon_tags_46 },
    { 0xb844862e, SUC_lexicon_tags_7 },
    { 0xf6c4862f, SUC_lexicon_tags_19 },
    { 0xdf04862f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a48635, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64848638, SUC_lexicon_tags_29 },
    { 0x96848639, SUC_lexicon_tags_9 },
    { 0x64848638, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb144863e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad848640, SUC_lexicon_tags_4 },
    { 0x67048641, SUC_lexicon_tags_25 },
    { 0xd1648642, SUC_lexicon_tags_12 },
    { 0xd9248642, SUC_lexicon_tags_1 },
    { 0x21a48644, SUC_lexicon_tags_1 },
    { 0xaa848645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e48648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe84864b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04864e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8848653, SUC_lexicon_tags_4 },
    { 0x16a48653, SUC_lexicon_tags_13 },
    { 0xd2048654, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4724865e, SUC_lexicon_tags_2 },
    { 0xa724865f, SUC_lexicon_tags_43 },
    { 0xfca48660, SUC_lexicon_tags_2 },
    { 0x2b248661, SUC_lexicon_tags_28 },
    { 0xd5848662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c48667, SUC_lexicon_tags_17 },
    { 0xc2448668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e4866d, SUC_lexicon_tags_29 },
    { 0x13e4866d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8048670, SUC_lexicon_tags_29 },
    { 0xcd648671, SUC_lexicon_tags_194 },
    { 0x11448671, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a648675, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7984867b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcf44867d, SUC_lexicon_tags_54 },
    { 0x7a24867d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x77248680, SUC_lexicon_tags_37 },
    { 0xd0e48680, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84248684, SUC_lexicon_tags_33 },
    { 0x20448685, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c4868a, SUC_lexicon_tags_19 },
    { 0xaca4868b, SUC_lexicon_tags_30 },
    { 0x9a24868b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c44868f, SUC_lexicon_tags_29 },
    { 0x20c4868f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f248697, SUC_lexicon_tags_7 },
    { 0xace48698, SUC_lexicon_tags_38 },
    { 0x10048699, SUC_lexicon_tags_7 },
    { 0xbae4869a, SUC_lexicon_tags_30 },
    { 0xb844869b, SUC_lexicon_tags_11 },
    { 0xc844869b, SUC_lexicon_tags_37 },
    { 0x5d848698, SUC_lexicon_tags_26 },
    { 0xb1e48699, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4486a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8486a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40486b5, SUC_lexicon_tags_23 },
    { 0xefc486b5, SUC_lexicon_tags_3 },
    { 0x112486b7, SUC_lexicon_tags_65 },
    { 0xf12486b8, SUC_lexicon_tags_11 },
    { 0x53e486b9, SUC_lexicon_tags_43 },
    { 0x74e486ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e486c1, SUC_lexicon_tags_30 },
    { 0x2e6486c2, SUC_lexicon_tags_84 },
    { 0x304486c2, SUC_lexicon_tags_1 },
    { 0xa74486c4, SUC_lexicon_tags_25 },
    { 0x306486c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6486c8, SUC_lexicon_tags_23 },
    { 0x660486c9, SUC_lexicon_tags_6 },
    { 0x308486ca, SUC_lexicon_tags_27 },
    { 0xf76486c9, SUC_lexicon_tags_48 },
    { 0xbc486cc, SUC_lexicon_tags_12 },
    { 0x632486cd, SUC_lexicon_tags_2 },
    { 0xa24486ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc72486d0, SUC_lexicon_tags_15 },
    { 0x388486d0, SUC_lexicon_tags_11 },
    { 0x6f6486d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc486d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92486da, SUC_lexicon_tags_26 },
    { 0xbd6486db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac486df, SUC_lexicon_tags_45 },
    { 0x90c486e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26486e5, SUC_lexicon_tags_37 },
    { 0xb3c486e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x76c486e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686486ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8486f0, SUC_lexicon_tags_45 },
    { 0xe7a486f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ec486f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0e486f5, SUC_lexicon_tags_29 },
    { 0x83a486f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2e486f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8486fb, SUC_lexicon_tags_15 },
    { 0x224486fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65848701, SUC_lexicon_tags_1037 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a48707, SUC_lexicon_tags_38 },
    { 0x62e48708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a4870c, SUC_lexicon_tags_11 },
    { 0x3aa4870d, SUC_lexicon_tags_29 },
    { 0x57c4870c, SUC_lexicon_tags_10 },
    { 0xb6e4870e, SUC_lexicon_tags_29 },
    { 0xf384870c, SUC_lexicon_tags_647 },
    { 0x25448711, SUC_lexicon_tags_425 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a448717, SUC_lexicon_tags_2 },
    { 0x8e448718, SUC_lexicon_tags_21 },
    { 0xc5c48719, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da4871e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42848720, SUC_lexicon_tags_15 },
    { 0xd0848720, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20848725, SUC_lexicon_tags_11 },
    { 0x6be48726, SUC_lexicon_tags_37 },
    { 0x648725, SUC_lexicon_tags_2 },
    { 0x9ca48728, SUC_lexicon_tags_0 },
    { 0x1f448729, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a848731, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a48734, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x85648736, SUC_lexicon_tags_47 },
    { 0xc5048737, SUC_lexicon_tags_1 },
    { 0x67248738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd24873a, SUC_lexicon_tags_29 },
    { 0x1ac4873b, SUC_lexicon_tags_2 },
    { 0xb244873b, SUC_lexicon_tags_60 },
    { 0xf724873d, SUC_lexicon_tags_30 },
    { 0xa24873e, SUC_lexicon_tags_37 },
    { 0xa0e4873e, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc48743, SUC_lexicon_tags_35 },
    { 0x80a48744, SUC_lexicon_tags_79 },
    { 0xfd648744, SUC_lexicon_tags_23 },
    { 0x61a48746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa48749, SUC_lexicon_tags_11 },
    { 0xcd84874a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe44874d, SUC_lexicon_tags_33 },
    { 0xb2a4874e, SUC_lexicon_tags_46 },
    { 0xfb44874f, SUC_lexicon_tags_138 },
    { 0x4004874f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a248752, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98248757, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da4875f, SUC_lexicon_tags_7 },
    { 0x14e4875f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e648762, SUC_lexicon_tags_26 },
    { 0x93a48763, SUC_lexicon_tags_11 },
    { 0x8fe48764, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a648769, SUC_lexicon_tags_25 },
    { 0xd9e4876a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1a4876c, SUC_lexicon_tags_19 },
    { 0x3544876c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3848770, SUC_lexicon_tags_37 },
    { 0xc448770, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56848776, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e4877b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xecc4877d, SUC_lexicon_tags_25 },
    { 0x7a64877e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca48782, SUC_lexicon_tags_28 },
    { 0xe848783, SUC_lexicon_tags_49 },
    { 0xdce48784, SUC_lexicon_tags_15 },
    { 0xf0a48785, SUC_lexicon_tags_7 },
    { 0xb3848785, SUC_lexicon_tags_7 },
    { 0x31a48782, SUC_lexicon_tags_7 },
    { 0xae848787, SUC_lexicon_tags_37 },
    { 0x3248789, SUC_lexicon_tags_14 },
    { 0xd4848783, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe584878d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c48790, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56a48792, SUC_lexicon_tags_15 },
    { 0xacc48793, SUC_lexicon_tags_11 },
    { 0xea448793, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c4879b, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca487a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58487a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbe2487a6, SUC_lexicon_tags_45 },
    { 0x9c0487a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996487ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0487b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71a487b3, SUC_lexicon_tags_2 },
    { 0x81e487b4, SUC_lexicon_tags_28 },
    { 0xf2e487b5, SUC_lexicon_tags_7 },
    { 0x8a8487b6, SUC_lexicon_tags_47 },
    { 0x70487b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a487ba, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x34e487bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0487bf, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x56487c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642487ca, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38487d0, SUC_lexicon_tags_37 },
    { 0x5a8487d0, SUC_lexicon_tags_31 },
    { 0x2b8487d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a487d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0487da, SUC_lexicon_tags_46 },
    { 0xf94487da, SUC_lexicon_tags_25 },
    { 0x848487dc, SUC_lexicon_tags_25 },
    { 0x948487dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ae487df, SUC_lexicon_tags_326 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8487e8, SUC_lexicon_tags_15 },
    { 0x466487e9, SUC_lexicon_tags_11 },
    { 0x3de487ea, SUC_lexicon_tags_25 },
    { 0x892487e9, SUC_lexicon_tags_12 },
    { 0xe9c487ea, SUC_lexicon_tags_33 },
    { 0xe2c487eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x704487f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2487f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcae487f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61a487f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8487fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a48800, SUC_lexicon_tags_15 },
    { 0x69248801, SUC_lexicon_tags_15 },
    { 0x92448802, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc048807, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1e048809, SUC_lexicon_tags_25 },
    { 0x1fc4880a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a64880d, SUC_lexicon_tags_7 },
    { 0xbaa4880e, SUC_lexicon_tags_48 },
    { 0x4a84880f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb448811, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28848813, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a048818, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe4881f, SUC_lexicon_tags_15 },
    { 0x2284881f, SUC_lexicon_tags_7 },
    { 0x63248821, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c48826, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d048829, SUC_lexicon_tags_23 },
    { 0xe424882a, SUC_lexicon_tags_43 },
    { 0x3084882b, SUC_lexicon_tags_17 },
    { 0xff64882b, SUC_lexicon_tags_27 },
    { 0x59c4882d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc64883a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d44883d, SUC_lexicon_tags_14 },
    { 0x2964883e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda48843, SUC_lexicon_tags_47 },
    { 0xdd248844, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71248846, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x86648848, SUC_lexicon_tags_7 },
    { 0xff448849, SUC_lexicon_tags_4 },
    { 0xb2448849, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa4884e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a48852, SUC_lexicon_tags_11 },
    { 0x52848853, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c48859, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b84885b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4744885f, SUC_lexicon_tags_45 },
    { 0xe3e48860, SUC_lexicon_tags_60 },
    { 0xe7448860, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee648865, SUC_lexicon_tags_15 },
    { 0x89c48865, SUC_lexicon_tags_0 },
    { 0x64c48867, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7784886c, SUC_lexicon_tags_11 },
    { 0x3ce4886d, SUC_lexicon_tags_17 },
    { 0xec44886d, SUC_lexicon_tags_1 },
    { 0x9344886c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69848873, SUC_lexicon_tags_1 },
    { 0xe6e48873, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca48877, SUC_lexicon_tags_2 },
    { 0xb8c48878, SUC_lexicon_tags_63 },
    { 0xcfe48877, SUC_lexicon_tags_17 },
    { 0xbf24887a, SUC_lexicon_tags_60 },
    { 0xa1e4887a, SUC_lexicon_tags_0 },
    { 0x5024887c, SUC_lexicon_tags_66 },
    { 0x32c48879, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x35e4887f, SUC_lexicon_tags_11 },
    { 0xa544887f, SUC_lexicon_tags_48 },
    { 0xc1048881, SUC_lexicon_tags_11 },
    { 0xa4a4887f, SUC_lexicon_tags_42 },
    { 0x96e48880, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88848889, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc824888d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2648895, SUC_lexicon_tags_4 },
    { 0x8ba48895, SUC_lexicon_tags_2 },
    { 0xcc448897, SUC_lexicon_tags_1 },
    { 0x6a248897, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8e84889a, SUC_lexicon_tags_2 },
    { 0x5d44889a, SUC_lexicon_tags_37 },
    { 0x9324889c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3424889e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50488a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x266488a8, SUC_lexicon_tags_32 },
    { 0x5d0488a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c488ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706488b0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2488b4, SUC_lexicon_tags_11 },
    { 0xef6488b5, SUC_lexicon_tags_14 },
    { 0xb9c488b6, SUC_lexicon_tags_68 },
    { 0xe3e488b7, SUC_lexicon_tags_1 },
    { 0x300488b7, SUC_lexicon_tags_38 },
    { 0xa32488b9, SUC_lexicon_tags_23 },
    { 0x152488b9, SUC_lexicon_tags_37 },
    { 0xdfe488b9, SUC_lexicon_tags_25 },
    { 0x64c488b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214488c1, SUC_lexicon_tags_4 },
    { 0x3f2488c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x176488c4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd56488c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d2488c8, SUC_lexicon_tags_27 },
    { 0x1a8488c8, SUC_lexicon_tags_17 },
    { 0x9e488ca, SUC_lexicon_tags_39 },
    { 0xae488ca, SUC_lexicon_tags_11 },
    { 0x196488cb, SUC_lexicon_tags_15 },
    { 0xdd4488c9, SUC_lexicon_tags_1 },
    { 0x70e488ce, SUC_lexicon_tags_45 },
    { 0x2d0488ce, SUC_lexicon_tags_27 },
    { 0xd42488d0, SUC_lexicon_tags_49 },
    { 0x224488c9, SUC_lexicon_tags_30 },
    { 0x3da488d2, SUC_lexicon_tags_77 },
    { 0x642488d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980488d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6be488da, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcbe488dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384488e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac488ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8488ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2cc488f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30e488f2, SUC_lexicon_tags_11 },
    { 0x646488f3, SUC_lexicon_tags_2 },
    { 0xbbc488f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e488f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e48901, SUC_lexicon_tags_17 },
    { 0x29a48902, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20048906, SUC_lexicon_tags_0 },
    { 0x34048907, SUC_lexicon_tags_27 },
    { 0x72448907, SUC_lexicon_tags_60 },
    { 0x79a48909, SUC_lexicon_tags_4 },
    { 0xf4048908, SUC_lexicon_tags_11 },
    { 0x5a24890b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc4890f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b648912, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba48915, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x19648917, SUC_lexicon_tags_6 },
    { 0xbc048918, SUC_lexicon_tags_23 },
    { 0x3da48919, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8064891c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42848921, SUC_lexicon_tags_17 },
    { 0x9d648921, SUC_lexicon_tags_11 },
    { 0x85048923, SUC_lexicon_tags_4 },
    { 0x8bc48924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2324892e, SUC_lexicon_tags_30 },
    { 0xde84892f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c48932, SUC_lexicon_tags_7 },
    { 0x66048932, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c4893b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda48943, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c4894c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba4894f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68648954, SUC_lexicon_tags_2 },
    { 0x10a48955, SUC_lexicon_tags_12 },
    { 0x3dc48956, SUC_lexicon_tags_15 },
    { 0xfe48954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2084895b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6984895d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7804895f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x23c48961, SUC_lexicon_tags_1 },
    { 0xfc648961, SUC_lexicon_tags_37 },
    { 0x35c48963, SUC_lexicon_tags_40 },
    { 0x89448964, SUC_lexicon_tags_12 },
    { 0x46248965, SUC_lexicon_tags_25 },
    { 0x2e48966, SUC_lexicon_tags_0 },
    { 0x85c48961, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x81a48969, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8664896b, SUC_lexicon_tags_46 },
    { 0x7524896c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb964896f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc48973, SUC_lexicon_tags_32 },
    { 0x8b048974, SUC_lexicon_tags_104 },
    { 0x55048975, SUC_lexicon_tags_56 },
    { 0x84a48973, SUC_lexicon_tags_23 },
    { 0x9a648977, SUC_lexicon_tags_82 },
    { 0xe4a48977, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c44897f, SUC_lexicon_tags_49 },
    { 0x54648980, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba48985, SUC_lexicon_tags_37 },
    { 0x29e48986, SUC_lexicon_tags_0 },
    { 0xcbe48986, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda4898b, SUC_lexicon_tags_25 },
    { 0xdc04898b, SUC_lexicon_tags_15 },
    { 0x7284898d, SUC_lexicon_tags_25 },
    { 0x4504898e, SUC_lexicon_tags_7 },
    { 0xa144898d, SUC_lexicon_tags_33 },
    { 0x94248990, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42e48992, SUC_lexicon_tags_26 },
    { 0x8c048993, SUC_lexicon_tags_37 },
    { 0xdee48992, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c248997, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8ae48999, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6084899b, SUC_lexicon_tags_25 },
    { 0x8924899c, SUC_lexicon_tags_25 },
    { 0x4ac4899c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x410489a0, SUC_lexicon_tags_17 },
    { 0xfba489a0, SUC_lexicon_tags_37 },
    { 0xca489a0, SUC_lexicon_tags_5 },
    { 0xcb2489a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192489a9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc489ac, SUC_lexicon_tags_19 },
    { 0x52a489ac, SUC_lexicon_tags_19 },
    { 0xfc4489ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdda489b0, SUC_lexicon_tags_1 },
    { 0x6e6489b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e489b7, SUC_lexicon_tags_2 },
    { 0xf32489b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe54489ba, SUC_lexicon_tags_66 },
    { 0x68a489bb, SUC_lexicon_tags_117 },
    { 0x56e489bc, SUC_lexicon_tags_17 },
    { 0xbc8489bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352489c3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6489c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4a489c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996489cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a2489cf, SUC_lexicon_tags_17 },
    { 0xf96489cf, SUC_lexicon_tags_77 },
    { 0xa8a489d1, SUC_lexicon_tags_25 },
    { 0x97c489d1, SUC_lexicon_tags_57 },
    { 0xb44489d3, SUC_lexicon_tags_2 },
    { 0x6c4489d4, SUC_lexicon_tags_17 },
    { 0x1f6489d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26489d8, SUC_lexicon_tags_27 },
    { 0x73a489d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768489e0, SUC_lexicon_tags_65 },
    { 0x25c489e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xade489e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6f0489e5, SUC_lexicon_tags_14 },
    { 0xfea489e6, SUC_lexicon_tags_29 },
    { 0x3bc489e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x892489e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa66489eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27e489ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fa489ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a489f2, SUC_lexicon_tags_9 },
    { 0xca2489f3, SUC_lexicon_tags_14 },
    { 0x4dc489f3, SUC_lexicon_tags_80 },
    { 0x4e6489f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6489f9, SUC_lexicon_tags_37 },
    { 0x832489f9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a489fe, SUC_lexicon_tags_8 },
    { 0xeb6489ff, SUC_lexicon_tags_37 },
    { 0xd5248a00, SUC_lexicon_tags_25 },
    { 0xe9848a01, SUC_lexicon_tags_12 },
    { 0x48a02, SUC_lexicon_tags_17 },
    { 0x9e48a02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81048a0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18048a0d, SUC_lexicon_tags_37 },
    { 0xa0c48a0e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87848a14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f648a18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80848a20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f048a24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e48a27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48a2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8848a2d, SUC_lexicon_tags_11 },
    { 0xa7e48a2e, SUC_lexicon_tags_17 },
    { 0xc848a2f, SUC_lexicon_tags_25 },
    { 0x1c448a30, SUC_lexicon_tags_25 },
    { 0xa1a48a30, SUC_lexicon_tags_1 },
    { 0x31648a30, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a48a38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea48a3d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x10e48a3f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77448a43, SUC_lexicon_tags_7 },
    { 0xc1e48a44, SUC_lexicon_tags_27 },
    { 0xd5048a43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfa48a47, SUC_lexicon_tags_17 },
    { 0x9da48a48, SUC_lexicon_tags_28 },
    { 0xbda48a49, SUC_lexicon_tags_30 },
    { 0x3d048a49, SUC_lexicon_tags_14 },
    { 0xbf248a48, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa48a4e, SUC_lexicon_tags_85 },
    { 0x7e248a4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e48a53, SUC_lexicon_tags_15 },
    { 0xd6648a54, SUC_lexicon_tags_1 },
    { 0x31a48a55, SUC_lexicon_tags_11 },
    { 0x30648a56, SUC_lexicon_tags_16 },
    { 0x23c48a57, SUC_lexicon_tags_129 },
    { 0xed048a58, SUC_lexicon_tags_103 },
    { 0xc5648a54, SUC_lexicon_tags_84 },
    { 0xce848a55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72c48a5c, SUC_lexicon_tags_43 },
    { 0xd2a48a5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x46648a5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55448a61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20248a63, SUC_lexicon_tags_1 },
    { 0xa2048a64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc248a67, SUC_lexicon_tags_37 },
    { 0xea048a67, SUC_lexicon_tags_22 },
    { 0xa6848a69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c48a6c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c48a70, SUC_lexicon_tags_7 },
    { 0x64848a71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4048a76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49648a78, SUC_lexicon_tags_1 },
    { 0x7dc48a78, SUC_lexicon_tags_28 },
    { 0xce648a7a, SUC_lexicon_tags_47 },
    { 0x7448a7b, SUC_lexicon_tags_11 },
    { 0xe5c48a7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5448a7f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x22a48a81, SUC_lexicon_tags_37 },
    { 0xc4a48a82, SUC_lexicon_tags_7 },
    { 0x4ea48a83, SUC_lexicon_tags_2 },
    { 0x2648a81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9248a8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d648a8f, SUC_lexicon_tags_1 },
    { 0xd7e48a90, SUC_lexicon_tags_9 },
    { 0x48448a91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb048a95, SUC_lexicon_tags_37 },
    { 0x8a48a96, SUC_lexicon_tags_15 },
    { 0x32448a97, SUC_lexicon_tags_28 },
    { 0xade48a98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc48aa4, SUC_lexicon_tags_29 },
    { 0x4e248aa4, SUC_lexicon_tags_17 },
    { 0xa1e48aa6, SUC_lexicon_tags_17 },
    { 0x81e48aa6, SUC_lexicon_tags_25 },
    { 0xf2048aa5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x76848aaa, SUC_lexicon_tags_37 },
    { 0x81448aaa, SUC_lexicon_tags_11 },
    { 0x60248aac, SUC_lexicon_tags_17 },
    { 0xdd048aad, SUC_lexicon_tags_17 },
    { 0xaf248aaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44048ab0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71248ab2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x53848ab4, SUC_lexicon_tags_56 },
    { 0xed648ab5, SUC_lexicon_tags_1 },
    { 0xee048ab6, SUC_lexicon_tags_37 },
    { 0xb1048ab7, SUC_lexicon_tags_34 },
    { 0x51448ab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1448abd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67848ac1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de48ac4, SUC_lexicon_tags_49 },
    { 0x5dc48ac5, SUC_lexicon_tags_37 },
    { 0xd4248ac4, SUC_lexicon_tags_8 },
    { 0x33a48ac7, SUC_lexicon_tags_37 },
    { 0xab648ac8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37048acf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2fc48ad1, SUC_lexicon_tags_17 },
    { 0x87a48ad2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5248ad5, SUC_lexicon_tags_7 },
    { 0xbf648ad6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd8c48ad8, SUC_lexicon_tags_17 },
    { 0xe6048ad9, SUC_lexicon_tags_37 },
    { 0x31248ad9, SUC_lexicon_tags_7 },
    { 0xb6a48ada, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9848ade, SUC_lexicon_tags_12 },
    { 0x3b248adf, SUC_lexicon_tags_2 },
    { 0xc848ae0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x22c48ae2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4d848ae4, SUC_lexicon_tags_26 },
    { 0x9a648ae4, SUC_lexicon_tags_29 },
    { 0x6248ae4, SUC_lexicon_tags_15 },
    { 0x3da48ae6, SUC_lexicon_tags_31 },
    { 0xb1448ae5, SUC_lexicon_tags_32 },
    { 0x1c848ae7, SUC_lexicon_tags_17 },
    { 0x39648aea, SUC_lexicon_tags_63 },
    { 0x1d648aeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c48aef, SUC_lexicon_tags_17 },
    { 0x73448af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc048af2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5e248af4, SUC_lexicon_tags_27 },
    { 0xb1848af4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x49448af7, SUC_lexicon_tags_12 },
    { 0xd8e48af7, SUC_lexicon_tags_0 },
    { 0x97848af9, SUC_lexicon_tags_38 },
    { 0x58a48af8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63048aff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd8048b01, SUC_lexicon_tags_7 },
    { 0x4c248b02, SUC_lexicon_tags_230 },
    { 0x52a48b03, SUC_lexicon_tags_23 },
    { 0xbf848b03, SUC_lexicon_tags_43 },
    { 0x14848b04, SUC_lexicon_tags_25 },
    { 0xdc248b06, SUC_lexicon_tags_60 },
    { 0x3b048b07, SUC_lexicon_tags_37 },
    { 0x9e648b05, SUC_lexicon_tags_40 },
    { 0xf4848b09, SUC_lexicon_tags_37 },
    { 0x84c48b08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x94448b0c, SUC_lexicon_tags_30 },
    { 0x29048b0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9848b11, SUC_lexicon_tags_25 },
    { 0xe7c48b12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a48b17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae648b19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a48b21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69848b23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x32848b26, SUC_lexicon_tags_27 },
    { 0x98c48b26, SUC_lexicon_tags_13 },
    { 0x5d248b26, SUC_lexicon_tags_27 },
    { 0xe0a48b29, SUC_lexicon_tags_37 },
    { 0x41648b2a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x17448b2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85e48b2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5048b34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28248b37, SUC_lexicon_tags_31 },
    { 0x3ac48b37, SUC_lexicon_tags_17 },
    { 0x19048b38, SUC_lexicon_tags_17 },
    { 0x8b448b38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4a48b3c, SUC_lexicon_tags_37 },
    { 0xb9648b3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0648b3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b448b41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbe48b43, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x45448b45, SUC_lexicon_tags_10 },
    { 0x5fe48b46, SUC_lexicon_tags_14 },
    { 0x62648b46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde48b4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e48b50, SUC_lexicon_tags_51 },
    { 0xd6848b51, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x36c48b53, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4648b56, SUC_lexicon_tags_26 },
    { 0xa0448b57, SUC_lexicon_tags_17 },
    { 0x84248b57, SUC_lexicon_tags_11 },
    { 0xec448b59, SUC_lexicon_tags_17 },
    { 0x10e48b57, SUC_lexicon_tags_33 },
    { 0x18448b5b, SUC_lexicon_tags_26 },
    { 0x74448b5c, SUC_lexicon_tags_47 },
    { 0x9ac48b5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb248b5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a48b65, SUC_lexicon_tags_17 },
    { 0xbda48b65, SUC_lexicon_tags_0 },
    { 0xf6e48b67, SUC_lexicon_tags_23 },
    { 0x63e48b68, SUC_lexicon_tags_7 },
    { 0x57648b67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76248b6b, SUC_lexicon_tags_17 },
    { 0x31448b6c, SUC_lexicon_tags_49 },
    { 0x9d648b6c, SUC_lexicon_tags_2 },
    { 0x96a48b6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe048b70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64048b72, SUC_lexicon_tags_11 },
    { 0x70048b72, SUC_lexicon_tags_2 },
    { 0x61848b72, SUC_lexicon_tags_17 },
    { 0x7a48b75, SUC_lexicon_tags_11 },
    { 0xdda48b76, SUC_lexicon_tags_27 },
    { 0x85c48b73, SUC_lexicon_tags_7 },
    { 0xb9c48b74, SUC_lexicon_tags_7 },
    { 0x5e48b79, SUC_lexicon_tags_82 },
    { 0x55c48b79, SUC_lexicon_tags_15 },
    { 0x15248b76, SUC_lexicon_tags_35 },
    { 0x36a48b7c, SUC_lexicon_tags_82 },
    { 0xc5648b7d, SUC_lexicon_tags_12 },
    { 0x59c48b7e, SUC_lexicon_tags_42 },
    { 0x1bc48b7f, SUC_lexicon_tags_15 },
    { 0x61848b7d, SUC_lexicon_tags_29 },
    { 0x1f248b81, SUC_lexicon_tags_47 },
    { 0x3ba48b82, SUC_lexicon_tags_30 },
    { 0x94848b83, SUC_lexicon_tags_17 },
    { 0xfe648b76, SUC_lexicon_tags_37 },
    { 0x6e248b77, SUC_lexicon_tags_34 },
    { 0x20848b86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15648b8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69048b8d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xec448b8f, SUC_lexicon_tags_1 },
    { 0x81848b90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d248b94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea848b9c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c48ba1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e448ba7, SUC_lexicon_tags_47 },
    { 0x2c048ba8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x63e48baa, SUC_lexicon_tags_7 },
    { 0xaf448bab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e48baf, SUC_lexicon_tags_43 },
    { 0xbc648baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7648bb2, SUC_lexicon_tags_4 },
    { 0xfec48bb2, SUC_lexicon_tags_8 },
    { 0x93848bb4, SUC_lexicon_tags_25 },
    { 0xd3048bb5, SUC_lexicon_tags_37 },
    { 0x57c48bb4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4ce48bb8, SUC_lexicon_tags_4 },
    { 0x89448bb9, SUC_lexicon_tags_25 },
    { 0xbbe48bba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee648bbc, SUC_lexicon_tags_27 },
    { 0x77c48bbd, SUC_lexicon_tags_27 },
    { 0x57a48bbc, SUC_lexicon_tags_37 },
    { 0x55648bbe, SUC_lexicon_tags_46 },
    { 0xba248bc0, SUC_lexicon_tags_19 },
    { 0xf6448bbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76048bc3, SUC_lexicon_tags_7 },
    { 0x5de48bc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc48bca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68448bd0, SUC_lexicon_tags_11 },
    { 0xca048bd1, SUC_lexicon_tags_31 },
    { 0xe248bd2, SUC_lexicon_tags_0 },
    { 0xae048bd0, SUC_lexicon_tags_37 },
    { 0x7e448bd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e48bd7, SUC_lexicon_tags_27 },
    { 0x56848bd8, SUC_lexicon_tags_29 },
    { 0x6ba48bd9, SUC_lexicon_tags_29 },
    { 0x39248bda, SUC_lexicon_tags_43 },
    { 0x30e48bdb, SUC_lexicon_tags_17 },
    { 0x98c48bda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcde48bde, SUC_lexicon_tags_28 },
    { 0x3c848bdf, SUC_lexicon_tags_43 },
    { 0x92e48be0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac48be4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c648bec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2048bee, SUC_lexicon_tags_37 },
    { 0x23a48bef, SUC_lexicon_tags_11 },
    { 0x94c48bee, SUC_lexicon_tags_0 },
    { 0x2ec48bef, SUC_lexicon_tags_8 },
    { 0xd1c48bf2, SUC_lexicon_tags_37 },
    { 0x59448bf2, SUC_lexicon_tags_19 },
    { 0xb5648bf4, SUC_lexicon_tags_19 },
    { 0xa3a48bf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfac48bf7, SUC_lexicon_tags_25 },
    { 0x65248bf8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e648bfd, SUC_lexicon_tags_30 },
    { 0xacc48bfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d848c00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5448c02, SUC_lexicon_tags_34 },
    { 0x7dc48c03, SUC_lexicon_tags_11 },
    { 0x22448c04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98048c09, SUC_lexicon_tags_39 },
    { 0x34648c0a, SUC_lexicon_tags_29 },
    { 0x75e48c0b, SUC_lexicon_tags_17 },
    { 0x1248c0b, SUC_lexicon_tags_48 },
    { 0x47648c0d, SUC_lexicon_tags_21 },
    { 0xa2448c0e, SUC_lexicon_tags_96 },
    { 0x3048c0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c48c12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3c48c14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a848c1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76a48c1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe48c21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11648c25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1448c29, SUC_lexicon_tags_13 },
    { 0xdbc48c2a, SUC_lexicon_tags_7 },
    { 0xd7648c2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac448c2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e48c33, SUC_lexicon_tags_15 },
    { 0x41248c34, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe48c3a, SUC_lexicon_tags_12 },
    { 0xee48c3a, SUC_lexicon_tags_3 },
    { 0x5c248c3c, SUC_lexicon_tags_6 },
    { 0xe248c3c, SUC_lexicon_tags_43 },
    { 0x59848c3e, SUC_lexicon_tags_11 },
    { 0xaa048c3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a48c45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80648c49, SUC_lexicon_tags_43 },
    { 0x66648c4a, SUC_lexicon_tags_30 },
    { 0xcea48c4b, SUC_lexicon_tags_14 },
    { 0x2bc48c4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfca48c4e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a48c54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74448c56, SUC_lexicon_tags_19 },
    { 0x1c248c57, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfc48c59, SUC_lexicon_tags_11 },
    { 0xe4248c5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a48c5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc48c64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa48c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee48c6e, SUC_lexicon_tags_25 },
    { 0x3da48c6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc48c73, SUC_lexicon_tags_17 },
    { 0x96248c74, SUC_lexicon_tags_104 },
    { 0x8ae48c75, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x85048c78, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7248c7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5da48c7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79e48c80, SUC_lexicon_tags_43 },
    { 0x45c48c81, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb248c84, SUC_lexicon_tags_12 },
    { 0x96e48c85, SUC_lexicon_tags_63 },
    { 0x99448c86, SUC_lexicon_tags_1 },
    { 0x6b648c87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe48c8e, SUC_lexicon_tags_0 },
    { 0xccc48c8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x35448c91, SUC_lexicon_tags_47 },
    { 0x62048c92, SUC_lexicon_tags_7 },
    { 0x23e48c93, SUC_lexicon_tags_63 },
    { 0xc7248c91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1648c99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xad448c9b, SUC_lexicon_tags_46 },
    { 0x85848c9b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3848c9e, SUC_lexicon_tags_12 },
    { 0x94e48c9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6648ca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38648ca4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xee248ca7, SUC_lexicon_tags_103 },
    { 0x51248ca7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc248cad, SUC_lexicon_tags_12 },
    { 0xf3a48cad, SUC_lexicon_tags_29 },
    { 0x24e48cae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a48cb7, SUC_lexicon_tags_63 },
    { 0x46a48cb8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7248cbe, SUC_lexicon_tags_37 },
    { 0x5a648cbe, SUC_lexicon_tags_14 },
    { 0x58248cc0, SUC_lexicon_tags_45 },
    { 0xf9048cc0, SUC_lexicon_tags_46 },
    { 0x33c48cc2, SUC_lexicon_tags_17 },
    { 0x4b848cc2, SUC_lexicon_tags_19 },
    { 0xfd048cc4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xea848cc6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c48cca, SUC_lexicon_tags_31 },
    { 0x83848ccb, SUC_lexicon_tags_37 },
    { 0x8bc48ccc, SUC_lexicon_tags_25 },
    { 0x8b048ccb, SUC_lexicon_tags_27 },
    { 0x56a48cce, SUC_lexicon_tags_37 },
    { 0x7e648cce, SUC_lexicon_tags_30 },
    { 0xdf448cd0, SUC_lexicon_tags_37 },
    { 0x30c48ccd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448cd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81048cda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf248cdd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a48ce4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b848ce8, SUC_lexicon_tags_17 },
    { 0x39648ce9, SUC_lexicon_tags_11 },
    { 0x2fe48cea, SUC_lexicon_tags_28 },
    { 0x9ec48cea, SUC_lexicon_tags_45 },
    { 0xde448cec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8848cef, SUC_lexicon_tags_17 },
    { 0x70648cef, SUC_lexicon_tags_39 },
    { 0xf3048cf1, SUC_lexicon_tags_14 },
    { 0xaa848cf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e48cf5, SUC_lexicon_tags_25 },
    { 0x17648cf6, SUC_lexicon_tags_43 },
    { 0x1a048cf7, SUC_lexicon_tags_29 },
    { 0x19a48cf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x54648cfb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9c48cfd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8aa48cff, SUC_lexicon_tags_15 },
    { 0x60448cff, SUC_lexicon_tags_17 },
    { 0x28048d00, SUC_lexicon_tags_7 },
    { 0x23248d02, SUC_lexicon_tags_27 },
    { 0x2da48d03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa6448d05, SUC_lexicon_tags_6 },
    { 0xe4e48d06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7648d0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x77848d0d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc648d12, SUC_lexicon_tags_44 },
    { 0x2048d13, SUC_lexicon_tags_31 },
    { 0x9b848d13, SUC_lexicon_tags_19 },
    { 0x5a448d15, SUC_lexicon_tags_28 },
    { 0xb4e48d16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19248d18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe48d1d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12448d20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf248d26, SUC_lexicon_tags_26 },
    { 0xe7a48d27, SUC_lexicon_tags_44 },
    { 0xeda48d27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28048d2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3248d32, SUC_lexicon_tags_7 },
    { 0x5a248d33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc848d36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0448d3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80c48d3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29048d3f, SUC_lexicon_tags_2 },
    { 0xbac48d40, SUC_lexicon_tags_30 },
    { 0x3a248d41, SUC_lexicon_tags_2 },
    { 0x1b248d41, SUC_lexicon_tags_15 },
    { 0x4248d41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec48d47, SUC_lexicon_tags_47 },
    { 0xb4848d47, SUC_lexicon_tags_65 },
    { 0x86a48d49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf248d4b, SUC_lexicon_tags_49 },
    { 0x34248d4c, SUC_lexicon_tags_16 },
    { 0x66048d4c, SUC_lexicon_tags_47 },
    { 0x26248d4e, SUC_lexicon_tags_12 },
    { 0xd6e48d4e, SUC_lexicon_tags_23 },
    { 0x6d248d50, SUC_lexicon_tags_12 },
    { 0xb8248d51, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e648d56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87048d5b, SUC_lexicon_tags_14 },
    { 0x8e448d5b, SUC_lexicon_tags_33 },
    { 0xab248d5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73248d67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40248d69, SUC_lexicon_tags_30 },
    { 0x76a48d6a, SUC_lexicon_tags_8 },
    { 0x79848d69, SUC_lexicon_tags_12 },
    { 0x49248d6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a48d72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73248d74, SUC_lexicon_tags_25 },
    { 0xedc48d75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbbc48d77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51248d7b, SUC_lexicon_tags_2 },
    { 0x36648d7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb448d80, SUC_lexicon_tags_37 },
    { 0xeb048d80, SUC_lexicon_tags_25 },
    { 0xd4c48d81, SUC_lexicon_tags_37 },
    { 0x6b248d82, SUC_lexicon_tags_45 },
    { 0xc8248d81, SUC_lexicon_tags_7 },
    { 0x6b048d83, SUC_lexicon_tags_6 },
    { 0xf5c48d86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe48d93, SUC_lexicon_tags_47 },
    { 0xac648d94, SUC_lexicon_tags_56 },
    { 0xcf048d95, SUC_lexicon_tags_45 },
    { 0x7ec48d96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b448d9b, SUC_lexicon_tags_9 },
    { 0x13e48d9b, SUC_lexicon_tags_25 },
    { 0xe2848d9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a648da1, SUC_lexicon_tags_17 },
    { 0x88e48da2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e48da5, SUC_lexicon_tags_11 },
    { 0xe6248da6, SUC_lexicon_tags_25 },
    { 0xf248da7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de48dae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2848db1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x45a48db3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c48db9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb1248dbb, SUC_lexicon_tags_63 },
    { 0x7f848dbc, SUC_lexicon_tags_60 },
    { 0x2be48dbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c48dc7, SUC_lexicon_tags_17 },
    { 0x9f648dc8, SUC_lexicon_tags_3 },
    { 0x70448dc8, SUC_lexicon_tags_19 },
    { 0xc648dca, SUC_lexicon_tags_1 },
    { 0x2de48dc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c48dce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e648dd8, SUC_lexicon_tags_63 },
    { 0x25648dd9, SUC_lexicon_tags_26 },
    { 0x51a48dd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa48de1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84248de6, SUC_lexicon_tags_63 },
    { 0xc8248de7, SUC_lexicon_tags_1 },
    { 0xe7e48de8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26848ded, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f848df0, SUC_lexicon_tags_253 },
    { 0x4cc48df0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2248df4, SUC_lexicon_tags_87 },
    { 0x8de48df5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94848df9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1248dfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfa48dfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74648e00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xba48e02, SUC_lexicon_tags_25 },
    { 0x9e248e03, SUC_lexicon_tags_7 },
    { 0xc7c48e02, SUC_lexicon_tags_1 },
    { 0x6a848e05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38648e08, SUC_lexicon_tags_25 },
    { 0xd8048e09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9648e0e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85048e13, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xf048e15, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba48e18, SUC_lexicon_tags_25 },
    { 0xa0048e18, SUC_lexicon_tags_37 },
    { 0xb0848e1a, SUC_lexicon_tags_29 },
    { 0xd4e48e1a, SUC_lexicon_tags_39 },
    { 0x3be48e1a, SUC_lexicon_tags_37 },
    { 0x52448e18, SUC_lexicon_tags_15 },
    { 0x7c248e1e, SUC_lexicon_tags_26 },
    { 0x1a848e19, SUC_lexicon_tags_38 },
    { 0x1048e1b, SUC_lexicon_tags_10 },
    { 0xaa48e21, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb448e27, SUC_lexicon_tags_1 },
    { 0x2da48e28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef248e2c, SUC_lexicon_tags_2 },
    { 0x40048e2d, SUC_lexicon_tags_14 },
    { 0x2ae48e2e, SUC_lexicon_tags_60 },
    { 0x45448e2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50248e31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c48e34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe248e3c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x91a48e3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7448e46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94248e4a, SUC_lexicon_tags_26 },
    { 0x66848e4b, SUC_lexicon_tags_7 },
    { 0x75e48e4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde448e4e, SUC_lexicon_tags_25 },
    { 0x54848e4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c648e52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a48e57, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3ba48e59, SUC_lexicon_tags_48 },
    { 0xa248e5a, SUC_lexicon_tags_0 },
    { 0xccc48e5a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x46848e5e, SUC_lexicon_tags_201 },
    { 0xe7c48e5f, SUC_lexicon_tags_28 },
    { 0x6fc48e60, SUC_lexicon_tags_19 },
    { 0xe3c48e5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xede48e64, SUC_lexicon_tags_17 },
    { 0x6b848e65, SUC_lexicon_tags_17 },
    { 0x7648e66, SUC_lexicon_tags_15 },
    { 0x87248e67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29848e6d, SUC_lexicon_tags_2 },
    { 0x87a48e6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78648e79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3648e7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe248e7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a248e81, SUC_lexicon_tags_11 },
    { 0xdfc48e82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9f848e84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f848e87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63c48e89, SUC_lexicon_tags_14 },
    { 0xcfc48e8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9048e8d, SUC_lexicon_tags_12 },
    { 0xb1e48e8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee648e96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc48ea2, SUC_lexicon_tags_15 },
    { 0x47648ea3, SUC_lexicon_tags_27 },
    { 0xdf248ea3, SUC_lexicon_tags_2 },
    { 0x61e48ea5, SUC_lexicon_tags_27 },
    { 0xee648ea6, SUC_lexicon_tags_30 },
    { 0xda048ea7, SUC_lexicon_tags_10 },
    { 0x21a48ea2, SUC_lexicon_tags_1 },
    { 0x64e48ea5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9448eae, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43e48eb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x93448eb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a48eb5, SUC_lexicon_tags_11 },
    { 0xca848eb5, SUC_lexicon_tags_1324 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca48ebc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a848ebe, SUC_lexicon_tags_17 },
    { 0x8c248ebf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x29a48ec1, SUC_lexicon_tags_0 },
    { 0xb1848ec2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18a48ec4, SUC_lexicon_tags_49 },
    { 0xbfc48ec5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95448eca, SUC_lexicon_tags_12 },
    { 0xe3248ecb, SUC_lexicon_tags_12 },
    { 0xd5248ecb, SUC_lexicon_tags_19 },
    { 0x7e248ecd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x39648ecf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc48ed1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6448ed4, SUC_lexicon_tags_24 },
    { 0x7c048ed5, SUC_lexicon_tags_25 },
    { 0x15848ed4, SUC_lexicon_tags_29 },
    { 0x2248ed7, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x24248ed9, SUC_lexicon_tags_15 },
    { 0xd8248eda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8c48edc, SUC_lexicon_tags_7 },
    { 0xfde48edd, SUC_lexicon_tags_25 },
    { 0xee448edd, SUC_lexicon_tags_14 },
    { 0x3dc48ede, SUC_lexicon_tags_1 },
    { 0xfec48ee0, SUC_lexicon_tags_2 },
    { 0x85048ee0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65848ee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a48ee7, SUC_lexicon_tags_7 },
    { 0x5e048ee8, SUC_lexicon_tags_15 },
    { 0xf9448ee9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba48eec, SUC_lexicon_tags_153 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce48ef2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa048ef6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x32648ef9, SUC_lexicon_tags_37 },
    { 0x84248efa, SUC_lexicon_tags_33 },
    { 0x7fa48efb, SUC_lexicon_tags_1 },
    { 0x6f648efc, SUC_lexicon_tags_15 },
    { 0x71e48efd, SUC_lexicon_tags_43 },
    { 0x79a48efd, SUC_lexicon_tags_1 },
    { 0x47848eff, SUC_lexicon_tags_2 },
    { 0x38848efb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be48f08, SUC_lexicon_tags_6 },
    { 0x3c848f08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9248f0e, SUC_lexicon_tags_37 },
    { 0xdbe48f0f, SUC_lexicon_tags_11 },
    { 0x4be48f10, SUC_lexicon_tags_47 },
    { 0x13648f11, SUC_lexicon_tags_11 },
    { 0xd8048f10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc48f18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfee48f1a, SUC_lexicon_tags_14 },
    { 0x53448f1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6648f1f, SUC_lexicon_tags_23 },
    { 0xbcc48f1f, SUC_lexicon_tags_27 },
    { 0x5ee48f21, SUC_lexicon_tags_12 },
    { 0x74048f22, SUC_lexicon_tags_7 },
    { 0xf448f22, SUC_lexicon_tags_35 },
    { 0x26448f23, SUC_lexicon_tags_7 },
    { 0x35448f22, SUC_lexicon_tags_11 },
    { 0xc0a48f1f, SUC_lexicon_tags_1 },
    { 0x7be48f27, SUC_lexicon_tags_15 },
    { 0x9ba48f28, SUC_lexicon_tags_25 },
    { 0x29a48f26, SUC_lexicon_tags_97 },
    { 0x22c48f2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e48f2d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee48f31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa48f36, SUC_lexicon_tags_38 },
    { 0x9ba48f37, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b648f3b, SUC_lexicon_tags_47 },
    { 0x8f848f3b, SUC_lexicon_tags_56 },
    { 0xc0648f3d, SUC_lexicon_tags_2 },
    { 0x26648f3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1848f40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7648f42, SUC_lexicon_tags_7 },
    { 0x27648f43, SUC_lexicon_tags_9 },
    { 0x5a248f44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f848f49, SUC_lexicon_tags_25 },
    { 0x4648f4a, SUC_lexicon_tags_6 },
    { 0x5a248f4b, SUC_lexicon_tags_12 },
    { 0x1de48f4c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xac448f4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3c48f51, SUC_lexicon_tags_1480 },
    { 0x27248f52, SUC_lexicon_tags_2 },
    { 0xeb248f52, SUC_lexicon_tags_6 },
    { 0xb8848f54, SUC_lexicon_tags_2 },
    { 0x4648f55, SUC_lexicon_tags_28 },
    { 0x45448f53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed048f58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c848f5b, SUC_lexicon_tags_17 },
    { 0x13048f5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36648f5e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb848f61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93648f66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc48f69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e48f6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21248f70, SUC_lexicon_tags_96 },
    { 0x10448f70, SUC_lexicon_tags_14 },
    { 0x75448f72, SUC_lexicon_tags_14 },
    { 0x9bc48f72, SUC_lexicon_tags_32 },
    { 0x5f448f74, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a48f77, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x90448f7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82048f7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c48f7f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca648f85, SUC_lexicon_tags_1 },
    { 0x72c48f86, SUC_lexicon_tags_49 },
    { 0x72048f86, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac48f8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa848f8e, SUC_lexicon_tags_4 },
    { 0x45448f8f, SUC_lexicon_tags_19 },
    { 0xe6c48f8f, SUC_lexicon_tags_49 },
    { 0x97e48f8f, SUC_lexicon_tags_60 },
    { 0x46a48f8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57a48f94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf248f97, SUC_lexicon_tags_16 },
    { 0x6ca48f97, SUC_lexicon_tags_9 },
    { 0xacc48f99, SUC_lexicon_tags_37 },
    { 0xe7e48f99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x66448f9d, SUC_lexicon_tags_63 },
    { 0xf1c48f9e, SUC_lexicon_tags_60 },
    { 0x3d048f9d, SUC_lexicon_tags_14 },
    { 0xda248fa0, SUC_lexicon_tags_16 },
    { 0xc4e48f9f, SUC_lexicon_tags_60 },
    { 0x23048fa2, SUC_lexicon_tags_27 },
    { 0xdea48fa0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8048fa7, SUC_lexicon_tags_17 },
    { 0x7ca48fa8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada48fae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88848fb6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6be48fb8, SUC_lexicon_tags_0 },
    { 0x6fc48fb9, SUC_lexicon_tags_37 },
    { 0x39248fba, SUC_lexicon_tags_7 },
    { 0xa8648fba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec48fc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e48fc9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a848fce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb248fd3, SUC_lexicon_tags_33 },
    { 0x448fd4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9248fdb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7648fdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50648fe1, SUC_lexicon_tags_7 },
    { 0xda048fe1, SUC_lexicon_tags_7 },
    { 0xdba48fe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a48fe7, SUC_lexicon_tags_7 },
    { 0xa7248fe8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b448fee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7f648ff0, SUC_lexicon_tags_60 },
    { 0xb4048ff1, SUC_lexicon_tags_11 },
    { 0x30848ff2, SUC_lexicon_tags_60 },
    { 0x57c48ff3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b248ff6, SUC_lexicon_tags_26 },
    { 0xbe48ff7, SUC_lexicon_tags_65 },
    { 0x22648ff8, SUC_lexicon_tags_6 },
    { 0x7ee48ff9, SUC_lexicon_tags_29 },
    { 0x53c48ffa, SUC_lexicon_tags_15 },
    { 0xf3a48ffa, SUC_lexicon_tags_55 },
    { 0x82c48ffc, SUC_lexicon_tags_12 },
    { 0x40848ffc, SUC_lexicon_tags_46 },
    { 0x1dc48ffe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x82a49000, SUC_lexicon_tags_23 },
    { 0x50249001, SUC_lexicon_tags_25 },
    { 0x57649002, SUC_lexicon_tags_49 },
    { 0x18449000, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f449006, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3444900a, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x36c4900c, SUC_lexicon_tags_11 },
    { 0xa144900c, SUC_lexicon_tags_33 },
    { 0x2164900e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfea49010, SUC_lexicon_tags_15 },
    { 0xdc49011, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc4901b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xeb44901d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf484901f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xba049021, SUC_lexicon_tags_30 },
    { 0x80449022, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb7049024, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22849027, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c4902a, SUC_lexicon_tags_19 },
    { 0x6d44902a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8a4902d, SUC_lexicon_tags_66 },
    { 0xc024902e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a049030, SUC_lexicon_tags_15 },
    { 0xf4a49031, SUC_lexicon_tags_1 },
    { 0x89449030, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c649034, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae4903b, SUC_lexicon_tags_30 },
    { 0x3384903c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c49040, SUC_lexicon_tags_31 },
    { 0x96049040, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5fa49043, SUC_lexicon_tags_68 },
    { 0x37c49044, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9f649046, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x16049049, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea04904d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a49050, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec49053, SUC_lexicon_tags_65 },
    { 0x81649054, SUC_lexicon_tags_32 },
    { 0x1f449055, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58249058, SUC_lexicon_tags_45 },
    { 0x98a49058, SUC_lexicon_tags_45 },
    { 0xbfe49058, SUC_lexicon_tags_2 },
    { 0xa6849059, SUC_lexicon_tags_60 },
    { 0x7884905c, SUC_lexicon_tags_8 },
    { 0x9684905c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6624905f, SUC_lexicon_tags_0 },
    { 0xc0e4905f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8849064, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a4906e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2649072, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc49078, SUC_lexicon_tags_49 },
    { 0x24249078, SUC_lexicon_tags_1 },
    { 0xc6849079, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe604907d, SUC_lexicon_tags_13 },
    { 0x6d64907e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x49a49080, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9049082, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x34c49084, SUC_lexicon_tags_33 },
    { 0x99649085, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea49089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa4908c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd324908e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3449090, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe049095, SUC_lexicon_tags_7 },
    { 0xed849096, SUC_lexicon_tags_15 },
    { 0xf1e49096, SUC_lexicon_tags_19 },
    { 0xfc049098, SUC_lexicon_tags_63 },
    { 0xb449099, SUC_lexicon_tags_9 },
    { 0x9804909a, SUC_lexicon_tags_12 },
    { 0xd5049099, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc4909e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c490a5, SUC_lexicon_tags_82 },
    { 0xece490a6, SUC_lexicon_tags_47 },
    { 0xdcc490a5, SUC_lexicon_tags_39 },
    { 0xf8e490a8, SUC_lexicon_tags_19 },
    { 0xb62490a8, SUC_lexicon_tags_1 },
    { 0xd2490a9, SUC_lexicon_tags_17 },
    { 0x258490aa, SUC_lexicon_tags_19 },
    { 0x924490ac, SUC_lexicon_tags_17 },
    { 0xcec490ad, SUC_lexicon_tags_64 },
    { 0x818490a9, SUC_lexicon_tags_25 },
    { 0x26e490ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x85a490b1, SUC_lexicon_tags_37 },
    { 0xb8490b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4490b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a490b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a490bc, SUC_lexicon_tags_34 },
    { 0xc8a490bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24490c0, SUC_lexicon_tags_25 },
    { 0x40c490c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4490c7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86490ca, SUC_lexicon_tags_33 },
    { 0x750490cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe20490cd, SUC_lexicon_tags_14 },
    { 0x65c490cd, SUC_lexicon_tags_43 },
    { 0xc70490cd, SUC_lexicon_tags_17 },
    { 0x7da490d0, SUC_lexicon_tags_17 },
    { 0x328490ce, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70490d6, SUC_lexicon_tags_0 },
    { 0x786490d7, SUC_lexicon_tags_16 },
    { 0xf6c490d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc490db, SUC_lexicon_tags_60 },
    { 0x94c490dc, SUC_lexicon_tags_12 },
    { 0xdb6490dd, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c490e0, SUC_lexicon_tags_34 },
    { 0xa60490e1, SUC_lexicon_tags_0 },
    { 0x430490e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a490e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be490ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x388490f2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e490f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a490fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf0e490fc, SUC_lexicon_tags_7 },
    { 0x2ac490fc, SUC_lexicon_tags_17 },
    { 0xc70490fe, SUC_lexicon_tags_44 },
    { 0x662490ff, SUC_lexicon_tags_29 },
    { 0x4a0490ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae49103, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa9649105, SUC_lexicon_tags_1 },
    { 0xfba49106, SUC_lexicon_tags_30 },
    { 0x32c49105, SUC_lexicon_tags_1 },
    { 0x93c49106, SUC_lexicon_tags_37 },
    { 0x65a49109, SUC_lexicon_tags_7 },
    { 0xae84910a, SUC_lexicon_tags_63 },
    { 0x4424910b, SUC_lexicon_tags_0 },
    { 0x87449107, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3249115, SUC_lexicon_tags_60 },
    { 0xba449116, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5fc49118, SUC_lexicon_tags_11 },
    { 0x31449119, SUC_lexicon_tags_55 },
    { 0x83e4911a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd544911c, SUC_lexicon_tags_2 },
    { 0xdaa4911c, SUC_lexicon_tags_37 },
    { 0xf164911e, SUC_lexicon_tags_11 },
    { 0xaca4911f, SUC_lexicon_tags_48 },
    { 0xb6249120, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5b249122, SUC_lexicon_tags_30 },
    { 0xf2c49123, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e49127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e4912b, SUC_lexicon_tags_87 },
    { 0xef04912b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2249135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd844913a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a849140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26449143, SUC_lexicon_tags_84 },
    { 0x49e49144, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x25249146, SUC_lexicon_tags_15 },
    { 0x3449147, SUC_lexicon_tags_7 },
    { 0xc5449146, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ce4914a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac64914e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a49151, SUC_lexicon_tags_10 },
    { 0xb3849151, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x59249154, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d049158, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb204915d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec49162, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c49165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf049167, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9564916e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc49171, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb249179, SUC_lexicon_tags_45 },
    { 0xc2e4917a, SUC_lexicon_tags_27 },
    { 0x4a04917b, SUC_lexicon_tags_60 },
    { 0x4f24917c, SUC_lexicon_tags_23 },
    { 0x23649179, SUC_lexicon_tags_47 },
    { 0xd464917e, SUC_lexicon_tags_0 },
    { 0xc884917f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e49182, SUC_lexicon_tags_17 },
    { 0xa9049183, SUC_lexicon_tags_14 },
    { 0xa7e49182, SUC_lexicon_tags_25 },
    { 0x28c49185, SUC_lexicon_tags_165 },
    { 0x3a049185, SUC_lexicon_tags_47 },
    { 0x7249184, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae4918c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa204918f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99849199, SUC_lexicon_tags_37 },
    { 0xd6049199, SUC_lexicon_tags_56 },
    { 0xae449199, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd2e4919d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6491a4, SUC_lexicon_tags_16 },
    { 0xf56491a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54491a9, SUC_lexicon_tags_15 },
    { 0xce491aa, SUC_lexicon_tags_4 },
    { 0xa70491ab, SUC_lexicon_tags_23 },
    { 0xdc4491ac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcee491ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb6e491b0, SUC_lexicon_tags_2 },
    { 0x5ba491b0, SUC_lexicon_tags_16 },
    { 0x82491b0, SUC_lexicon_tags_25 },
    { 0x77e491b0, SUC_lexicon_tags_11 },
    { 0x11c491b4, SUC_lexicon_tags_11 },
    { 0x9f0491b0, SUC_lexicon_tags_19 },
    { 0xe72491b6, SUC_lexicon_tags_35 },
    { 0x4a491b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a491bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22c491bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea491c3, SUC_lexicon_tags_1 },
    { 0x48c491c4, SUC_lexicon_tags_37 },
    { 0x9d2491c3, SUC_lexicon_tags_49 },
    { 0x72491c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde491ce, SUC_lexicon_tags_1 },
    { 0xec2491cf, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48491d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94491dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a491e0, SUC_lexicon_tags_15 },
    { 0xc4e491e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc491e7, SUC_lexicon_tags_37 },
    { 0x1fa491e8, SUC_lexicon_tags_19 },
    { 0xe02491e9, SUC_lexicon_tags_37 },
    { 0x8cc491e9, SUC_lexicon_tags_13 },
    { 0x308491ea, SUC_lexicon_tags_12 },
    { 0xc46491e7, SUC_lexicon_tags_60 },
    { 0x1c6491ed, SUC_lexicon_tags_12 },
    { 0x2fe491ed, SUC_lexicon_tags_2 },
    { 0xc7a491ef, SUC_lexicon_tags_29 },
    { 0xcf2491ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270491f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68491fa, SUC_lexicon_tags_12 },
    { 0x8a491fa, SUC_lexicon_tags_12 },
    { 0xf9a491fc, SUC_lexicon_tags_11 },
    { 0x148491fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3249202, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a849204, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5424920d, SUC_lexicon_tags_7 },
    { 0x50a4920d, SUC_lexicon_tags_12 },
    { 0x24a4920f, SUC_lexicon_tags_29 },
    { 0x16049210, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1be49212, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x38449215, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe8249217, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa4921b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a84921e, SUC_lexicon_tags_1 },
    { 0xe924921f, SUC_lexicon_tags_1 },
    { 0x39849220, SUC_lexicon_tags_15 },
    { 0xb9449221, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52249225, SUC_lexicon_tags_37 },
    { 0x8be49226, SUC_lexicon_tags_42 },
    { 0x9ae49226, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a44922b, SUC_lexicon_tags_1 },
    { 0x1204922b, SUC_lexicon_tags_15 },
    { 0x4dc4922d, SUC_lexicon_tags_120 },
    { 0xd484922e, SUC_lexicon_tags_0 },
    { 0x4a24922b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe49232, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xab649235, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf024923b, SUC_lexicon_tags_37 },
    { 0x9084923c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd244923e, SUC_lexicon_tags_140 },
    { 0x35a4923f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3e49241, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f84924b, SUC_lexicon_tags_7 },
    { 0xd64924c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5449251, SUC_lexicon_tags_37 },
    { 0xb8049252, SUC_lexicon_tags_19 },
    { 0x6e49253, SUC_lexicon_tags_17 },
    { 0xdd849254, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa49259, SUC_lexicon_tags_55 },
    { 0xa384925a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8004925e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4c49260, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76c49262, SUC_lexicon_tags_32 },
    { 0x54649263, SUC_lexicon_tags_1 },
    { 0xde449264, SUC_lexicon_tags_1 },
    { 0xfc449264, SUC_lexicon_tags_7 },
    { 0x7e449265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be4926a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec04926d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf64926f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38249273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa249277, SUC_lexicon_tags_8 },
    { 0x7fc49277, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12c4927a, SUC_lexicon_tags_37 },
    { 0xa2a4927b, SUC_lexicon_tags_37 },
    { 0x8e24927c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb649284, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xca649287, SUC_lexicon_tags_11 },
    { 0x87c49287, SUC_lexicon_tags_11 },
    { 0x1f449289, SUC_lexicon_tags_23 },
    { 0x99849289, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4928f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x80849291, SUC_lexicon_tags_25 },
    { 0x9fc49292, SUC_lexicon_tags_213 },
    { 0x8f049292, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e49296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b049298, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8484929b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1784929d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6492a5, SUC_lexicon_tags_17 },
    { 0x1a6492a5, SUC_lexicon_tags_12 },
    { 0xfaa492a6, SUC_lexicon_tags_47 },
    { 0x4c6492a5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e492b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x120492b2, SUC_lexicon_tags_12 },
    { 0x788492b3, SUC_lexicon_tags_12 },
    { 0xf42492b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x268492b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2492b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60492bc, SUC_lexicon_tags_27 },
    { 0x7e2492bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc492c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec492cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c6492cd, SUC_lexicon_tags_25 },
    { 0x490492ce, SUC_lexicon_tags_38 },
    { 0x1a6492ce, SUC_lexicon_tags_37 },
    { 0xb8c492d0, SUC_lexicon_tags_11 },
    { 0x8d4492d0, SUC_lexicon_tags_174 },
    { 0x78c492d2, SUC_lexicon_tags_37 },
    { 0x576492d2, SUC_lexicon_tags_47 },
    { 0xa0a492d3, SUC_lexicon_tags_9 },
    { 0x14a492d5, SUC_lexicon_tags_28 },
    { 0xfb2492d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572492dc, SUC_lexicon_tags_38 },
    { 0x5ac492dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce0492df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8492e2, SUC_lexicon_tags_37 },
    { 0xd6492e3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4492e7, SUC_lexicon_tags_37 },
    { 0xf18492e8, SUC_lexicon_tags_7 },
    { 0x422492e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca492ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c492ee, SUC_lexicon_tags_3 },
    { 0x8c6492ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd86492f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80492f4, SUC_lexicon_tags_6 },
    { 0xe5e492f4, SUC_lexicon_tags_47 },
    { 0xcea492f4, SUC_lexicon_tags_37 },
    { 0x838492f7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c492fd, SUC_lexicon_tags_116 },
    { 0x900492fe, SUC_lexicon_tags_26 },
    { 0x57a492fe, SUC_lexicon_tags_2 },
    { 0x6e4492fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e49303, SUC_lexicon_tags_30 },
    { 0xb1449303, SUC_lexicon_tags_14 },
    { 0x51a49305, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c04930b, SUC_lexicon_tags_29 },
    { 0x23a4930c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7e4930e, SUC_lexicon_tags_8 },
    { 0x2484930f, SUC_lexicon_tags_37 },
    { 0x9264930f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x74e49312, SUC_lexicon_tags_1 },
    { 0xd8449312, SUC_lexicon_tags_82 },
    { 0x6249314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4e49316, SUC_lexicon_tags_11 },
    { 0xf649317, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a24931b, SUC_lexicon_tags_66 },
    { 0x2e64931b, SUC_lexicon_tags_46 },
    { 0x97a4931d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a49321, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e49324, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac49328, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c4932e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52849332, SUC_lexicon_tags_11 },
    { 0xc2649333, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a49337, SUC_lexicon_tags_2 },
    { 0x8c849338, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9284933b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e24933f, SUC_lexicon_tags_49 },
    { 0xc0c49340, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa49347, SUC_lexicon_tags_38 },
    { 0x39a49348, SUC_lexicon_tags_12 },
    { 0x5c849349, SUC_lexicon_tags_61 },
    { 0x4aa49349, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7024934c, SUC_lexicon_tags_17 },
    { 0x4a24934d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e49350, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x37c49352, SUC_lexicon_tags_47 },
    { 0x88249353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a449355, SUC_lexicon_tags_2 },
    { 0xbc649356, SUC_lexicon_tags_17 },
    { 0xd5849356, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde4935f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46449366, SUC_lexicon_tags_47 },
    { 0x40c49367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8249369, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1804936c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3624936e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44249370, SUC_lexicon_tags_21 },
    { 0x37849371, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae449374, SUC_lexicon_tags_79 },
    { 0x5a449374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21249378, SUC_lexicon_tags_1 },
    { 0x64849378, SUC_lexicon_tags_15 },
    { 0xada49378, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2284937f, SUC_lexicon_tags_60 },
    { 0xb244937f, SUC_lexicon_tags_25 },
    { 0x15649381, SUC_lexicon_tags_21 },
    { 0x8f84937f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14049387, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc0649389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8144938b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9144938d, SUC_lexicon_tags_1 },
    { 0x8444938e, SUC_lexicon_tags_15 },
    { 0x9b44938f, SUC_lexicon_tags_27 },
    { 0xa004938f, SUC_lexicon_tags_37 },
    { 0x6ba4938f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x79049394, SUC_lexicon_tags_12 },
    { 0x8249394, SUC_lexicon_tags_43 },
    { 0xcec49396, SUC_lexicon_tags_1 },
    { 0xcfc49396, SUC_lexicon_tags_106 },
    { 0x52649396, SUC_lexicon_tags_11 },
    { 0xe3c49394, SUC_lexicon_tags_37 },
    { 0x73c49399, SUC_lexicon_tags_0 },
    { 0xe1a4939b, SUC_lexicon_tags_0 },
    { 0x9c44939a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa64939e, SUC_lexicon_tags_8 },
    { 0x8424939f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28493a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa493a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6493aa, SUC_lexicon_tags_158 },
    { 0x712493ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4493af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2493b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8493b9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x136493bb, SUC_lexicon_tags_12 },
    { 0x5d0493bc, SUC_lexicon_tags_7 },
    { 0xd58493bd, SUC_lexicon_tags_66 },
    { 0x81a493be, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8493c1, SUC_lexicon_tags_54 },
    { 0xcd6493c2, SUC_lexicon_tags_7 },
    { 0xe44493c3, SUC_lexicon_tags_34 },
    { 0x69c493c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0493c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942493cc, SUC_lexicon_tags_15 },
    { 0xaf0493cc, SUC_lexicon_tags_25 },
    { 0xa44493cd, SUC_lexicon_tags_25 },
    { 0x92e493cf, SUC_lexicon_tags_17 },
    { 0x11c493d0, SUC_lexicon_tags_45 },
    { 0x61c493d0, SUC_lexicon_tags_41 },
    { 0x590493d1, SUC_lexicon_tags_10 },
    { 0xc64493d3, SUC_lexicon_tags_15 },
    { 0x2e6493d2, SUC_lexicon_tags_12 },
    { 0xb90493d3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x828493d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72493da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x16e493dc, SUC_lexicon_tags_48 },
    { 0xb3a493dd, SUC_lexicon_tags_0 },
    { 0x566493de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6493e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64c493e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582493ed, SUC_lexicon_tags_37 },
    { 0xff2493ed, SUC_lexicon_tags_104 },
    { 0x7a493ed, SUC_lexicon_tags_20 },
    { 0x85c493f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a449400, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5449403, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de49407, SUC_lexicon_tags_49 },
    { 0x41849407, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c04940b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de49411, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x56649413, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c49422, SUC_lexicon_tags_7 },
    { 0x5a49422, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3844942d, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a49435, SUC_lexicon_tags_25 },
    { 0x4bc49435, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e4943b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6da4943d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31c4943f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4c49441, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c49445, SUC_lexicon_tags_108 },
    { 0x65e49446, SUC_lexicon_tags_38 },
    { 0x4aa49447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c649449, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a24944b, SUC_lexicon_tags_0 },
    { 0x1ac4944c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24849456, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c4945d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88449461, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1c649463, SUC_lexicon_tags_54 },
    { 0xa3449464, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49c49466, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xbaa49468, SUC_lexicon_tags_25 },
    { 0xea49468, SUC_lexicon_tags_23 },
    { 0x30849469, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e49474, SUC_lexicon_tags_38 },
    { 0x3e249475, SUC_lexicon_tags_37 },
    { 0xf4c49475, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65049478, SUC_lexicon_tags_33 },
    { 0x1b249479, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xedc4947b, SUC_lexicon_tags_2 },
    { 0xeee4947c, SUC_lexicon_tags_17 },
    { 0x8084947d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3fa4947f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d449483, SUC_lexicon_tags_11 },
    { 0x43c49484, SUC_lexicon_tags_12 },
    { 0x33e49485, SUC_lexicon_tags_0 },
    { 0xd3c49485, SUC_lexicon_tags_0 },
    { 0x53449487, SUC_lexicon_tags_37 },
    { 0x66849485, SUC_lexicon_tags_65 },
    { 0x4d249488, SUC_lexicon_tags_2 },
    { 0xfb84948a, SUC_lexicon_tags_60 },
    { 0xb024948b, SUC_lexicon_tags_0 },
    { 0xfea49489, SUC_lexicon_tags_17 },
    { 0xfe49486, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6e4948f, SUC_lexicon_tags_39 },
    { 0x43849490, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e049495, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb1449497, SUC_lexicon_tags_439 },
    { 0xb1849498, SUC_lexicon_tags_11 },
    { 0x3b849497, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe494a0, SUC_lexicon_tags_5 },
    { 0x438494a1, SUC_lexicon_tags_11 },
    { 0x4ba494a2, SUC_lexicon_tags_3 },
    { 0x1fe494a2, SUC_lexicon_tags_25 },
    { 0x6ac494a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8494ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36494ae, SUC_lexicon_tags_32 },
    { 0x9de494ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e494b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee494b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80494bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e494c1, SUC_lexicon_tags_1 },
    { 0x3c4494c2, SUC_lexicon_tags_107 },
    { 0x4ae494c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8e494c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7d0494c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x52a494c9, SUC_lexicon_tags_14 },
    { 0x41e494ca, SUC_lexicon_tags_14 },
    { 0xba6494cb, SUC_lexicon_tags_38 },
    { 0x3ea494cc, SUC_lexicon_tags_2 },
    { 0x1ac494cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9c6494cf, SUC_lexicon_tags_7 },
    { 0x356494cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57e494d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x910494d4, SUC_lexicon_tags_12 },
    { 0xf50494d4, SUC_lexicon_tags_25 },
    { 0xedc494d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a494d9, SUC_lexicon_tags_29 },
    { 0x974494d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d8494dc, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x874494df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c494e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfa8494e9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa494f0, SUC_lexicon_tags_32 },
    { 0x47c494f0, SUC_lexicon_tags_23 },
    { 0x2fe494f2, SUC_lexicon_tags_69 },
    { 0xe8e494f2, SUC_lexicon_tags_1 },
    { 0xa7a494f4, SUC_lexicon_tags_15 },
    { 0xc44494f5, SUC_lexicon_tags_30 },
    { 0x34e494f6, SUC_lexicon_tags_1 },
    { 0x826494f5, SUC_lexicon_tags_12 },
    { 0xf94494f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be494fb, SUC_lexicon_tags_44 },
    { 0xcac494fb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x918494fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c49503, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde249505, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55049508, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x26a4950a, SUC_lexicon_tags_14 },
    { 0x6864950b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4950e, SUC_lexicon_tags_11 },
    { 0xa4c4950e, SUC_lexicon_tags_26 },
    { 0xa884950f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa49515, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x41849518, SUC_lexicon_tags_12 },
    { 0x16049519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b049525, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x47849527, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa764952a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96249533, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7124953d, SUC_lexicon_tags_79 },
    { 0x4044953e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c49543, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc49546, SUC_lexicon_tags_3 },
    { 0xebc49546, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c44954d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50049551, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e49556, SUC_lexicon_tags_12 },
    { 0xe9c49557, SUC_lexicon_tags_11 },
    { 0x59449558, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e4955c, SUC_lexicon_tags_43 },
    { 0xc0e4955d, SUC_lexicon_tags_7 },
    { 0x7be4955e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1f849560, SUC_lexicon_tags_15 },
    { 0x28849561, SUC_lexicon_tags_29 },
    { 0x18049560, SUC_lexicon_tags_15 },
    { 0x13249563, SUC_lexicon_tags_25 },
    { 0x5c849564, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3049567, SUC_lexicon_tags_7 },
    { 0xa8449568, SUC_lexicon_tags_25 },
    { 0x55649569, SUC_lexicon_tags_113 },
    { 0x57e49569, SUC_lexicon_tags_49 },
    { 0x5364956b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x1704956e, SUC_lexicon_tags_25 },
    { 0x9484956e, SUC_lexicon_tags_37 },
    { 0xc4a4956f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc049572, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b849574, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe49577, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc144957c, SUC_lexicon_tags_54 },
    { 0xd004957d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4644957f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7da49581, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a649584, SUC_lexicon_tags_16 },
    { 0x9b649585, SUC_lexicon_tags_1 },
    { 0x98e49586, SUC_lexicon_tags_43 },
    { 0xf0449587, SUC_lexicon_tags_17 },
    { 0x65049588, SUC_lexicon_tags_15 },
    { 0xa6449589, SUC_lexicon_tags_13 },
    { 0x8b049589, SUC_lexicon_tags_23 },
    { 0xb5449588, SUC_lexicon_tags_25 },
    { 0x7784958c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x16a4958e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x56a49590, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc49594, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c449596, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83a49598, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5864959d, SUC_lexicon_tags_21 },
    { 0x8864959d, SUC_lexicon_tags_11 },
    { 0x2624959f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76a495a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0495a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c495a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a0495a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8495af, SUC_lexicon_tags_66 },
    { 0xf2c495b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4495b4, SUC_lexicon_tags_1 },
    { 0x41a495b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2495ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974495be, SUC_lexicon_tags_133 },
    { 0xbbe495be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf80495c1, SUC_lexicon_tags_47 },
    { 0x630495c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x962495c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0495c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3c6495c9, SUC_lexicon_tags_14 },
    { 0xc5c495ca, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x504495cd, SUC_lexicon_tags_15 },
    { 0x762495ce, SUC_lexicon_tags_23 },
    { 0x622495cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc495d5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc60495d7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x230495d9, SUC_lexicon_tags_17 },
    { 0x62c495d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8495de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47a495e0, SUC_lexicon_tags_64 },
    { 0x700495e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34495e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x25e495e8, SUC_lexicon_tags_15 },
    { 0x502495e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726495ef, SUC_lexicon_tags_57 },
    { 0x31a495f0, SUC_lexicon_tags_37 },
    { 0xd7a495f1, SUC_lexicon_tags_6 },
    { 0xdbc495f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb48495f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0495f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4495fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44849600, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb249604, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3504960a, SUC_lexicon_tags_47 },
    { 0x2964960b, SUC_lexicon_tags_17 },
    { 0x8e04960a, SUC_lexicon_tags_17 },
    { 0xde4960d, SUC_lexicon_tags_21 },
    { 0xb944960e, SUC_lexicon_tags_25 },
    { 0x6564960c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f249611, SUC_lexicon_tags_1116 },
    { 0xaf049611, SUC_lexicon_tags_17 },
    { 0x85249613, SUC_lexicon_tags_17 },
    { 0x98649611, SUC_lexicon_tags_79 },
    { 0x83c49611, SUC_lexicon_tags_19 },
    { 0x5b649616, SUC_lexicon_tags_8 },
    { 0x20649617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada4961c, SUC_lexicon_tags_25 },
    { 0xffe4961d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5de4961f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48049624, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c449627, SUC_lexicon_tags_66 },
    { 0x51649628, SUC_lexicon_tags_31 },
    { 0x12649629, SUC_lexicon_tags_79 },
    { 0x9d64962a, SUC_lexicon_tags_25 },
    { 0xd304962b, SUC_lexicon_tags_12 },
    { 0xb5049629, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5649634, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde649636, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4ee49638, SUC_lexicon_tags_38 },
    { 0x9c49639, SUC_lexicon_tags_33 },
    { 0xde249639, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf524963d, SUC_lexicon_tags_32 },
    { 0x7784963e, SUC_lexicon_tags_7 },
    { 0xa9a4963f, SUC_lexicon_tags_3 },
    { 0x98449640, SUC_lexicon_tags_16 },
    { 0x2de4963f, SUC_lexicon_tags_31 },
    { 0x6c44963e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x17849644, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6849646, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd464964a, SUC_lexicon_tags_33 },
    { 0x91c4964a, SUC_lexicon_tags_47 },
    { 0x5e4964b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf24964e, SUC_lexicon_tags_30 },
    { 0x5804964e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c649652, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x27449654, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcaa49656, SUC_lexicon_tags_38 },
    { 0x1249657, SUC_lexicon_tags_194 },
    { 0xfc049658, SUC_lexicon_tags_25 },
    { 0x21c49658, SUC_lexicon_tags_37 },
    { 0xcb049659, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8449664, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c49667, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba84966b, SUC_lexicon_tags_63 },
    { 0xc2e4966c, SUC_lexicon_tags_43 },
    { 0xbb64966b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab84966f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74449673, SUC_lexicon_tags_25 },
    { 0x9be49674, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc49678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a4967c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa504967f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ee49681, SUC_lexicon_tags_797 },
    { 0, NULL },
    { 0xc449683, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78849688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa404968f, SUC_lexicon_tags_33 },
    { 0x29849690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x30049693, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c49696, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb1e49698, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4204969a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4284969c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b84969e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84a496a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2496a4, SUC_lexicon_tags_25 },
    { 0xf48496a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6496ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8496b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb82496b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe26496b5, SUC_lexicon_tags_23 },
    { 0x132496b6, SUC_lexicon_tags_23 },
    { 0xe4e496b6, SUC_lexicon_tags_63 },
    { 0x884496b8, SUC_lexicon_tags_26 },
    { 0x784496b9, SUC_lexicon_tags_45 },
    { 0xcc496b7, SUC_lexicon_tags_1 },
    { 0x188496bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd38496bd, SUC_lexicon_tags_15 },
    { 0x6a496be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e496c2, SUC_lexicon_tags_27 },
    { 0xa60496c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c496c6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e496c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x668496cc, SUC_lexicon_tags_605 },
    { 0x1be496cc, SUC_lexicon_tags_74 },
    { 0x296496cc, SUC_lexicon_tags_15 },
    { 0x328496cf, SUC_lexicon_tags_60 },
    { 0x6dc496d0, SUC_lexicon_tags_43 },
    { 0x21e496d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfda496d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa496d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896496db, SUC_lexicon_tags_10 },
    { 0xd98496dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdc496de, SUC_lexicon_tags_15 },
    { 0xc66496df, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6496e4, SUC_lexicon_tags_6 },
    { 0xfaa496e5, SUC_lexicon_tags_28 },
    { 0x7c0496e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf2496e8, SUC_lexicon_tags_11 },
    { 0xaf2496e8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c496ec, SUC_lexicon_tags_1 },
    { 0x6c6496ed, SUC_lexicon_tags_15 },
    { 0x340496ec, SUC_lexicon_tags_47 },
    { 0x30496ee, SUC_lexicon_tags_780 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0496f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e496f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x360496f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x862496fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0496fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcce49700, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc249702, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ea49704, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89049707, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c64970b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64849710, SUC_lexicon_tags_17 },
    { 0xdfc49711, SUC_lexicon_tags_37 },
    { 0xc2249712, SUC_lexicon_tags_17 },
    { 0x74249712, SUC_lexicon_tags_0 },
    { 0x26449713, SUC_lexicon_tags_7 },
    { 0x61449715, SUC_lexicon_tags_2 },
    { 0xd0249716, SUC_lexicon_tags_14 },
    { 0xf2449716, SUC_lexicon_tags_11 },
    { 0x13c49712, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a44971b, SUC_lexicon_tags_48 },
    { 0xaec4971b, SUC_lexicon_tags_15 },
    { 0x85e4971d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x81e4971f, SUC_lexicon_tags_12 },
    { 0xfd44971f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaec49722, SUC_lexicon_tags_66 },
    { 0x51c49723, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb084972a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c4972d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe49732, SUC_lexicon_tags_66 },
    { 0x54e49733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c49736, SUC_lexicon_tags_23 },
    { 0x6e649737, SUC_lexicon_tags_11 },
    { 0xae849738, SUC_lexicon_tags_45 },
    { 0xe4849738, SUC_lexicon_tags_27 },
    { 0x70649737, SUC_lexicon_tags_25 },
    { 0x5224973b, SUC_lexicon_tags_1 },
    { 0x6ce49737, SUC_lexicon_tags_7 },
    { 0xdea4973b, SUC_lexicon_tags_3 },
    { 0x6c04973e, SUC_lexicon_tags_12 },
    { 0x8fa4973e, SUC_lexicon_tags_17 },
    { 0x77849739, SUC_lexicon_tags_2 },
    { 0xe5449741, SUC_lexicon_tags_43 },
    { 0xd2e4973e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb049747, SUC_lexicon_tags_47 },
    { 0x82849747, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbe44974a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e64974c, SUC_lexicon_tags_11 },
    { 0x384974d, SUC_lexicon_tags_17 },
    { 0x4604974c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449756, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe204975b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a4975e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c49761, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde649765, SUC_lexicon_tags_9 },
    { 0x48449765, SUC_lexicon_tags_37 },
    { 0x64e49767, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe564976a, SUC_lexicon_tags_12 },
    { 0x24e4976a, SUC_lexicon_tags_16 },
    { 0x9884976b, SUC_lexicon_tags_25 },
    { 0x264976c, SUC_lexicon_tags_17 },
    { 0x6124976e, SUC_lexicon_tags_38 },
    { 0x2c64976f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61a49771, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65449774, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92249776, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e4977b, SUC_lexicon_tags_37 },
    { 0x3d44977c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44849783, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc49787, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e4978b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7164978d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2504978f, SUC_lexicon_tags_15 },
    { 0x9a449790, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc4979c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe4a4979e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x92e497a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6497a7, SUC_lexicon_tags_55 },
    { 0x18497a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe5e497aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28e497ac, SUC_lexicon_tags_0 },
    { 0x802497ad, SUC_lexicon_tags_16 },
    { 0xfe0497ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c497b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfce497b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a497b6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x62a497b8, SUC_lexicon_tags_32 },
    { 0x316497b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8497bc, SUC_lexicon_tags_21 },
    { 0x5ce497bc, SUC_lexicon_tags_12 },
    { 0x10c497bc, SUC_lexicon_tags_23 },
    { 0x948497bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598497c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c497ca, SUC_lexicon_tags_31 },
    { 0x3e0497cb, SUC_lexicon_tags_85 },
    { 0xec2497cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda497d1, SUC_lexicon_tags_11 },
    { 0x988497d2, SUC_lexicon_tags_85 },
    { 0xd16497d1, SUC_lexicon_tags_27 },
    { 0xed8497d4, SUC_lexicon_tags_12 },
    { 0xdce497d4, SUC_lexicon_tags_2 },
    { 0xd3c497d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4497d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc497dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab0497de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57e497e0, SUC_lexicon_tags_27 },
    { 0xc54497e1, SUC_lexicon_tags_60 },
    { 0x6b8497e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce497ec, SUC_lexicon_tags_49 },
    { 0x1ec497ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd28497ef, SUC_lexicon_tags_45 },
    { 0xdc2497f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcfc497f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06497f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6497fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd88497fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd049802, SUC_lexicon_tags_15 },
    { 0x6fe49803, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac49806, SUC_lexicon_tags_31 },
    { 0xaec49806, SUC_lexicon_tags_38 },
    { 0xa3049808, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x22a4980a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5564980d, SUC_lexicon_tags_17 },
    { 0x51a4980e, SUC_lexicon_tags_37 },
    { 0x8da4980e, SUC_lexicon_tags_15 },
    { 0x93649810, SUC_lexicon_tags_37 },
    { 0x17849810, SUC_lexicon_tags_2 },
    { 0x14849812, SUC_lexicon_tags_17 },
    { 0xab249813, SUC_lexicon_tags_87 },
    { 0xe49813, SUC_lexicon_tags_33 },
    { 0xa9e49813, SUC_lexicon_tags_17 },
    { 0xa649815, SUC_lexicon_tags_230 },
    { 0x9449817, SUC_lexicon_tags_25 },
    { 0xf2049818, SUC_lexicon_tags_0 },
    { 0xd5049819, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3ce4981b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa49820, SUC_lexicon_tags_112 },
    { 0xdf649821, SUC_lexicon_tags_47 },
    { 0x3e649821, SUC_lexicon_tags_29 },
    { 0xa5249823, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xbc449825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2404982b, SUC_lexicon_tags_13 },
    { 0xb8a4982c, SUC_lexicon_tags_7 },
    { 0xc024982c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26849830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70249832, SUC_lexicon_tags_30 },
    { 0x849833, SUC_lexicon_tags_65 },
    { 0x2f649832, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4b849836, SUC_lexicon_tags_37 },
    { 0x2b049837, SUC_lexicon_tags_26 },
    { 0x57449837, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb664983a, SUC_lexicon_tags_113 },
    { 0x2744983a, SUC_lexicon_tags_60 },
    { 0x68c4983c, SUC_lexicon_tags_14 },
    { 0x3b64983d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf844983f, SUC_lexicon_tags_6 },
    { 0x2b449840, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5049844, SUC_lexicon_tags_6 },
    { 0xf1449845, SUC_lexicon_tags_25 },
    { 0x80e49846, SUC_lexicon_tags_1 },
    { 0xab049845, SUC_lexicon_tags_28 },
    { 0xdde49847, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c4984e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69c49850, SUC_lexicon_tags_27 },
    { 0x249851, SUC_lexicon_tags_46 },
    { 0x4fc49852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4aa49854, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe4c49856, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344985e, SUC_lexicon_tags_11 },
    { 0x4924985e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6649866, SUC_lexicon_tags_37 },
    { 0x55849866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11449869, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1164986b, SUC_lexicon_tags_31 },
    { 0x5864986b, SUC_lexicon_tags_12 },
    { 0xee24986d, SUC_lexicon_tags_6 },
    { 0xcfc4986e, SUC_lexicon_tags_37 },
    { 0x75e4986d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f249874, SUC_lexicon_tags_15 },
    { 0x29e49875, SUC_lexicon_tags_57 },
    { 0xec049876, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53249878, SUC_lexicon_tags_28 },
    { 0x64e49878, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e49882, SUC_lexicon_tags_4 },
    { 0x9049882, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca49886, SUC_lexicon_tags_15 },
    { 0xcbe49887, SUC_lexicon_tags_38 },
    { 0x3ca49886, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c4988c, SUC_lexicon_tags_15 },
    { 0x1824988d, SUC_lexicon_tags_25 },
    { 0x3464988e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c49892, SUC_lexicon_tags_8 },
    { 0x70249893, SUC_lexicon_tags_2 },
    { 0x3de49894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c49898, SUC_lexicon_tags_9 },
    { 0x98049898, SUC_lexicon_tags_45 },
    { 0x81a49899, SUC_lexicon_tags_7 },
    { 0x1f44989b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5324989e, SUC_lexicon_tags_37 },
    { 0xfb64989f, SUC_lexicon_tags_37 },
    { 0x884989e, SUC_lexicon_tags_19 },
    { 0xcee498a1, SUC_lexicon_tags_14 },
    { 0x54e498a2, SUC_lexicon_tags_24 },
    { 0x404498a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e498a6, SUC_lexicon_tags_20 },
    { 0xcb0498a7, SUC_lexicon_tags_11 },
    { 0x9e2498a6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe90498aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c498af, SUC_lexicon_tags_17 },
    { 0xb70498b0, SUC_lexicon_tags_87 },
    { 0x12a498b1, SUC_lexicon_tags_7 },
    { 0xc4c498b2, SUC_lexicon_tags_25 },
    { 0x312498b0, SUC_lexicon_tags_16 },
    { 0xda498b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36498b7, SUC_lexicon_tags_29 },
    { 0x750498b8, SUC_lexicon_tags_0 },
    { 0x62a498b9, SUC_lexicon_tags_158 },
    { 0xa9e498ba, SUC_lexicon_tags_17 },
    { 0xec498bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd1e498bd, SUC_lexicon_tags_37 },
    { 0x212498be, SUC_lexicon_tags_15 },
    { 0x4ce498bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8498c4, SUC_lexicon_tags_37 },
    { 0xa9a498c4, SUC_lexicon_tags_12 },
    { 0xd44498c6, SUC_lexicon_tags_0 },
    { 0x624498c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x226498ca, SUC_lexicon_tags_1 },
    { 0xe16498cb, SUC_lexicon_tags_0 },
    { 0x5c2498cb, SUC_lexicon_tags_51 },
    { 0x53a498cb, SUC_lexicon_tags_61 },
    { 0x91c498ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x132498d0, SUC_lexicon_tags_68 },
    { 0x9c6498d1, SUC_lexicon_tags_0 },
    { 0xd48498d2, SUC_lexicon_tags_23 },
    { 0x66498d2, SUC_lexicon_tags_37 },
    { 0xe86498d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbfe498d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c498d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be498dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dc498de, SUC_lexicon_tags_15 },
    { 0x65c498df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2498e2, SUC_lexicon_tags_49 },
    { 0x43e498e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e498f1, SUC_lexicon_tags_25 },
    { 0x1b8498f2, SUC_lexicon_tags_27 },
    { 0xeb8498f1, SUC_lexicon_tags_43 },
    { 0xe20498f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42e498f6, SUC_lexicon_tags_17 },
    { 0xe76498f7, SUC_lexicon_tags_37 },
    { 0xeae498f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2498fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82498fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb249902, SUC_lexicon_tags_26 },
    { 0x13a49903, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d249906, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a49909, SUC_lexicon_tags_31 },
    { 0x4b44990a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc6e4990c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa84990e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a49913, SUC_lexicon_tags_77 },
    { 0x8ea49913, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21449918, SUC_lexicon_tags_33 },
    { 0xa2249919, SUC_lexicon_tags_19 },
    { 0x6e44991a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf124991e, SUC_lexicon_tags_4 },
    { 0x6dc4991f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x34249922, SUC_lexicon_tags_43 },
    { 0x73e49922, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6c49925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12649928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c84992c, SUC_lexicon_tags_63 },
    { 0x3744992d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e49930, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa9c49932, SUC_lexicon_tags_47 },
    { 0x92c49932, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee49938, SUC_lexicon_tags_17 },
    { 0x5a249939, SUC_lexicon_tags_7 },
    { 0xe384993a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd144993d, SUC_lexicon_tags_65 },
    { 0x2b04993e, SUC_lexicon_tags_14 },
    { 0xa804993e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9849943, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1049949, SUC_lexicon_tags_65 },
    { 0x8684994a, SUC_lexicon_tags_29 },
    { 0x4ba4994a, SUC_lexicon_tags_47 },
    { 0x4c24994c, SUC_lexicon_tags_37 },
    { 0xd504994d, SUC_lexicon_tags_60 },
    { 0xbae4994b, SUC_lexicon_tags_17 },
    { 0x71a4994d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafa49951, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1049954, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9ae49956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3249959, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb04995c, SUC_lexicon_tags_9 },
    { 0xf4a4995d, SUC_lexicon_tags_37 },
    { 0x86c4995e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44849963, SUC_lexicon_tags_1 },
    { 0x6f249963, SUC_lexicon_tags_17 },
    { 0xc0a49964, SUC_lexicon_tags_42 },
    { 0x1aa49966, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa04996a, SUC_lexicon_tags_35 },
    { 0x604996a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2664996d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1784996f, SUC_lexicon_tags_15 },
    { 0x1e049970, SUC_lexicon_tags_37 },
    { 0xfda49970, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13849974, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfd649976, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x69649978, SUC_lexicon_tags_17 },
    { 0x71249979, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb144997e, SUC_lexicon_tags_47 },
    { 0xaa64997e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96049981, SUC_lexicon_tags_37 },
    { 0xdfc49981, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe984998e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee49991, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e49997, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba84999d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbbe4999f, SUC_lexicon_tags_2 },
    { 0xd84999f, SUC_lexicon_tags_47 },
    { 0xda84999f, SUC_lexicon_tags_27 },
    { 0x7e4499a2, SUC_lexicon_tags_12 },
    { 0x21c499a1, SUC_lexicon_tags_45 },
    { 0x14a499a0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328499a8, SUC_lexicon_tags_15 },
    { 0xfba499a9, SUC_lexicon_tags_2 },
    { 0xcfe499aa, SUC_lexicon_tags_17 },
    { 0x82a499a9, SUC_lexicon_tags_37 },
    { 0x1b0499aa, SUC_lexicon_tags_60 },
    { 0xc0a499ad, SUC_lexicon_tags_47 },
    { 0x5e8499ae, SUC_lexicon_tags_25 },
    { 0xb08499a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50499b4, SUC_lexicon_tags_4 },
    { 0x19c499b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x198499b8, SUC_lexicon_tags_2 },
    { 0x366499b8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x714499bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e2499be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e499c4, SUC_lexicon_tags_37 },
    { 0x150499c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66499c8, SUC_lexicon_tags_15 },
    { 0xc94499c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba499cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662499d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6499da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x962499dc, SUC_lexicon_tags_11 },
    { 0x6f2499dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x998499df, SUC_lexicon_tags_17 },
    { 0x21e499df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e499e3, SUC_lexicon_tags_12 },
    { 0xe8a499e4, SUC_lexicon_tags_15 },
    { 0xb86499e5, SUC_lexicon_tags_2 },
    { 0xe86499e5, SUC_lexicon_tags_39 },
    { 0x548499e5, SUC_lexicon_tags_6 },
    { 0xd6499e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c499eb, SUC_lexicon_tags_0 },
    { 0x8ea499ec, SUC_lexicon_tags_19 },
    { 0x8c4499ed, SUC_lexicon_tags_25 },
    { 0xe72499ee, SUC_lexicon_tags_65 },
    { 0xd3c499eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2499f7, SUC_lexicon_tags_31 },
    { 0xc86499f8, SUC_lexicon_tags_37 },
    { 0xfce499f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e499fc, SUC_lexicon_tags_37 },
    { 0x344499fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17049a00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d449a02, SUC_lexicon_tags_27 },
    { 0x3ae49a02, SUC_lexicon_tags_27 },
    { 0xf0249a04, SUC_lexicon_tags_25 },
    { 0xd1c49a05, SUC_lexicon_tags_8 },
    { 0xc7c49a06, SUC_lexicon_tags_34 },
    { 0x3ce49a06, SUC_lexicon_tags_1 },
    { 0xaca49a05, SUC_lexicon_tags_30 },
    { 0xab049a06, SUC_lexicon_tags_7 },
    { 0xba049a03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e49a0f, SUC_lexicon_tags_8 },
    { 0x7d649a10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7049a14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca849a16, SUC_lexicon_tags_21 },
    { 0xaa449a17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c49a1d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a49a20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce649a2c, SUC_lexicon_tags_17 },
    { 0xdc49a2d, SUC_lexicon_tags_25 },
    { 0x7f649a2e, SUC_lexicon_tags_60 },
    { 0x21c49a2f, SUC_lexicon_tags_1 },
    { 0x7da49a30, SUC_lexicon_tags_66 },
    { 0xea649a31, SUC_lexicon_tags_15 },
    { 0x58449a2d, SUC_lexicon_tags_17 },
    { 0x5c249a33, SUC_lexicon_tags_43 },
    { 0x649a34, SUC_lexicon_tags_14 },
    { 0xa249a35, SUC_lexicon_tags_59 },
    { 0x33249a31, SUC_lexicon_tags_45 },
    { 0x66449a37, SUC_lexicon_tags_55 },
    { 0xeb849a37, SUC_lexicon_tags_25 },
    { 0x5f849a37, SUC_lexicon_tags_6 },
    { 0x5449a3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6249a3c, SUC_lexicon_tags_11 },
    { 0xf7449a3d, SUC_lexicon_tags_46 },
    { 0xf3e49a3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29649a42, SUC_lexicon_tags_15 },
    { 0xad049a42, SUC_lexicon_tags_2 },
    { 0x7f649a43, SUC_lexicon_tags_79 },
    { 0xd6249a42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1649a4a, SUC_lexicon_tags_3 },
    { 0x8ae49a4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57249a4e, SUC_lexicon_tags_54 },
    { 0x78449a4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4049a53, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f849a5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea49a60, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xeb849a62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14049a67, SUC_lexicon_tags_4 },
    { 0xebe49a68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca649a6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc49a6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c49a73, SUC_lexicon_tags_7 },
    { 0x43049a73, SUC_lexicon_tags_4 },
    { 0x38049a75, SUC_lexicon_tags_7 },
    { 0x49049a74, SUC_lexicon_tags_1 },
    { 0x39249a76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbda49a79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x22249a7b, SUC_lexicon_tags_15 },
    { 0x52649a7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe49a82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e449a84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c849a8b, SUC_lexicon_tags_47 },
    { 0x5ae49a8c, SUC_lexicon_tags_27 },
    { 0x44649a8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c49a92, SUC_lexicon_tags_29 },
    { 0x92649a92, SUC_lexicon_tags_9 },
    { 0x72a49a92, SUC_lexicon_tags_60 },
    { 0x8ac49a92, SUC_lexicon_tags_65 },
    { 0x5b449a95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44449a98, SUC_lexicon_tags_29 },
    { 0xd1e49a99, SUC_lexicon_tags_17 },
    { 0x15c49a9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78049a9e, SUC_lexicon_tags_19 },
    { 0x5c049a9e, SUC_lexicon_tags_6 },
    { 0xd5c49aa0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea049aa4, SUC_lexicon_tags_7 },
    { 0x14449aa5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58249aa9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b049aad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4a49aaf, SUC_lexicon_tags_57 },
    { 0xe5449ab0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed449ab2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc249ab4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c849ab6, SUC_lexicon_tags_11 },
    { 0xeda49ab7, SUC_lexicon_tags_171 },
    { 0xb3249ab8, SUC_lexicon_tags_6 },
    { 0x7b449ab9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde49abc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a849abf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89649ac7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec49acd, SUC_lexicon_tags_6 },
    { 0x90049ace, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x24049ad1, SUC_lexicon_tags_11 },
    { 0x73249ad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ca49ad4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb449ad8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74449ada, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x27849adc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10249adf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7649ae4, SUC_lexicon_tags_48 },
    { 0xeae49ae4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1449aea, SUC_lexicon_tags_2 },
    { 0x14849aeb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4f449aed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x54a49aef, SUC_lexicon_tags_31 },
    { 0xb0049af0, SUC_lexicon_tags_15 },
    { 0xc8649af1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e49af4, SUC_lexicon_tags_151 },
    { 0x44e49af4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xae249af7, SUC_lexicon_tags_14 },
    { 0x11449af7, SUC_lexicon_tags_7 },
    { 0x89a49af8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe49afc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc49aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37849b01, SUC_lexicon_tags_11 },
    { 0xfa849b02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c649b05, SUC_lexicon_tags_17 },
    { 0xfce49b06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49b0a, SUC_lexicon_tags_1 },
    { 0xfee49b0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad649b0e, SUC_lexicon_tags_49 },
    { 0x19a49b0f, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6849b12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x54049b14, SUC_lexicon_tags_37 },
    { 0xe049b14, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc049b19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d849b21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe49b24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5849b29, SUC_lexicon_tags_37 },
    { 0x55849b29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb649b2c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba49b30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea49b33, SUC_lexicon_tags_9 },
    { 0x2dc49b34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5449b37, SUC_lexicon_tags_9 },
    { 0xe9449b38, SUC_lexicon_tags_185 },
    { 0xb5e49b38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48249b42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec49b45, SUC_lexicon_tags_23 },
    { 0x16e49b46, SUC_lexicon_tags_7 },
    { 0xd3249b47, SUC_lexicon_tags_17 },
    { 0x38249b48, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8e849b4a, SUC_lexicon_tags_47 },
    { 0x16649b4b, SUC_lexicon_tags_27 },
    { 0x5fe49b4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2849b4e, SUC_lexicon_tags_10 },
    { 0x5f449b4e, SUC_lexicon_tags_17 },
    { 0xdf649b50, SUC_lexicon_tags_37 },
    { 0x32a49b51, SUC_lexicon_tags_65 },
    { 0x40449b51, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe49b56, SUC_lexicon_tags_33 },
    { 0x7f249b57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x88449b59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63849b5d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8649b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66849b63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73249b66, SUC_lexicon_tags_4 },
    { 0x6b849b66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3249b6b, SUC_lexicon_tags_8 },
    { 0xb3449b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a649b6f, SUC_lexicon_tags_28 },
    { 0x54e49b6f, SUC_lexicon_tags_38 },
    { 0x2b849b70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48249b74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63849b76, SUC_lexicon_tags_49 },
    { 0xb7849b77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e649b7a, SUC_lexicon_tags_23 },
    { 0xc3049b7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c49b80, SUC_lexicon_tags_15 },
    { 0x11a49b81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb649b84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0c49b86, SUC_lexicon_tags_106 },
    { 0xfd849b87, SUC_lexicon_tags_11 },
    { 0x8ae49b87, SUC_lexicon_tags_25 },
    { 0x25649b87, SUC_lexicon_tags_29 },
    { 0xd9449b8a, SUC_lexicon_tags_28 },
    { 0x7aa49b8a, SUC_lexicon_tags_19 },
    { 0x92849b8c, SUC_lexicon_tags_1 },
    { 0xa2c49b8d, SUC_lexicon_tags_27 },
    { 0x13849b8c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81849b92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91449b96, SUC_lexicon_tags_37 },
    { 0x8dc49b97, SUC_lexicon_tags_14 },
    { 0xad249b96, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x31449b9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d249ba1, SUC_lexicon_tags_7 },
    { 0x4a449ba1, SUC_lexicon_tags_17 },
    { 0xdf049ba1, SUC_lexicon_tags_7 },
    { 0x87249ba2, SUC_lexicon_tags_9 },
    { 0x83049ba2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x47049ba8, SUC_lexicon_tags_14 },
    { 0xa8e49ba8, SUC_lexicon_tags_27 },
    { 0x8ac49ba8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc9249bac, SUC_lexicon_tags_15 },
    { 0xb9c49bad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d649baf, SUC_lexicon_tags_17 },
    { 0xe7a49baf, SUC_lexicon_tags_27 },
    { 0xaaa49baf, SUC_lexicon_tags_27 },
    { 0xb6849bb0, SUC_lexicon_tags_17 },
    { 0x52449bb3, SUC_lexicon_tags_77 },
    { 0x39c49bb3, SUC_lexicon_tags_2 },
    { 0x12e49bb0, SUC_lexicon_tags_2 },
    { 0xf7a49bb6, SUC_lexicon_tags_11 },
    { 0x4a249bb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81449bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a649bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1249bc0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d449bc4, SUC_lexicon_tags_23 },
    { 0x29449bc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe49bc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8c49bca, SUC_lexicon_tags_26 },
    { 0x7a049bca, SUC_lexicon_tags_23 },
    { 0x2e649bcc, SUC_lexicon_tags_31 },
    { 0x43649bcd, SUC_lexicon_tags_2 },
    { 0x99a49bcb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x16449bd0, SUC_lexicon_tags_87 },
    { 0xf6649bd1, SUC_lexicon_tags_37 },
    { 0x9a249bd1, SUC_lexicon_tags_8 },
    { 0xfcc49bd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec449bd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10049bd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99049bda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda49bdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a49be2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa849be5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7849be9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x64e49beb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x50249bed, SUC_lexicon_tags_7 },
    { 0x59c49bed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e449bf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x36449bf5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa49bfc, SUC_lexicon_tags_9 },
    { 0x9a249bfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c49c02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d249c08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36e49c0a, SUC_lexicon_tags_34 },
    { 0x32e49c0a, SUC_lexicon_tags_92 },
    { 0x24e49c0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9249c0e, SUC_lexicon_tags_77 },
    { 0x93e49c0f, SUC_lexicon_tags_63 },
    { 0x4a49c0e, SUC_lexicon_tags_91 },
    { 0x67a49c0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc049c13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9e49c15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1c49c17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a249c1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2f249c1d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac49c20, SUC_lexicon_tags_9 },
    { 0x30449c21, SUC_lexicon_tags_2 },
    { 0x71c49c21, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60049c27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a449c29, SUC_lexicon_tags_47 },
    { 0x94c49c2a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4249c2d, SUC_lexicon_tags_1660 },
    { 0, NULL },
    { 0x11249c2f, SUC_lexicon_tags_1246 },
    { 0x2f049c30, SUC_lexicon_tags_16 },
    { 0x5849c31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d049c34, SUC_lexicon_tags_17 },
    { 0x33849c35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6449c3a, SUC_lexicon_tags_25 },
    { 0x35e49c3b, SUC_lexicon_tags_27 },
    { 0x5449c3b, SUC_lexicon_tags_31 },
    { 0x11049c3b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9849c42, SUC_lexicon_tags_49 },
    { 0xec449c42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e49c47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba49c4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed049c51, SUC_lexicon_tags_65 },
    { 0xc4249c51, SUC_lexicon_tags_7 },
    { 0xb3449c53, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5b049c55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee449c57, SUC_lexicon_tags_10 },
    { 0x78c49c58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33449c62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a449c65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7049c6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3649c70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1be49c72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc849c75, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf449c7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc049c7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4a49c81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b449c86, SUC_lexicon_tags_25 },
    { 0xcea49c87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe0449c89, SUC_lexicon_tags_31 },
    { 0x25e49c8a, SUC_lexicon_tags_37 },
    { 0xae49c8b, SUC_lexicon_tags_17 },
    { 0x96249c8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74a49c8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb049c96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c49ca2, SUC_lexicon_tags_15 },
    { 0xf0849ca3, SUC_lexicon_tags_25 },
    { 0x6b849ca3, SUC_lexicon_tags_15 },
    { 0x7ee49ca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85849ca8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8449cb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee249cb5, SUC_lexicon_tags_10 },
    { 0x85c49cb6, SUC_lexicon_tags_12 },
    { 0xe8c49cb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21249cb9, SUC_lexicon_tags_23 },
    { 0x70249cba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca49cc0, SUC_lexicon_tags_12 },
    { 0x35c49cc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fe49cc3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2c249cc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d449ccc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c49cd2, SUC_lexicon_tags_33 },
    { 0x97049cd2, SUC_lexicon_tags_33 },
    { 0xd249cd4, SUC_lexicon_tags_28 },
    { 0x2e649cd5, SUC_lexicon_tags_32 },
    { 0x3a449cd6, SUC_lexicon_tags_38 },
    { 0xf2649cd3, SUC_lexicon_tags_32 },
    { 0xf049cd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x47049cda, SUC_lexicon_tags_55 },
    { 0x9b649cdb, SUC_lexicon_tags_87 },
    { 0x3a449cdb, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x8a649cde, SUC_lexicon_tags_7 },
    { 0x6b049cdf, SUC_lexicon_tags_11 },
    { 0x5d249ce0, SUC_lexicon_tags_28 },
    { 0x4f649ce0, SUC_lexicon_tags_31 },
    { 0x2c49ce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39649ce8, SUC_lexicon_tags_17 },
    { 0xf5049ce8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c49cec, SUC_lexicon_tags_15 },
    { 0x8de49cec, SUC_lexicon_tags_1 },
    { 0x5ac49cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae49cf2, SUC_lexicon_tags_2 },
    { 0xeb449cf3, SUC_lexicon_tags_9 },
    { 0x3a249cf3, SUC_lexicon_tags_43 },
    { 0x36449cf5, SUC_lexicon_tags_38 },
    { 0x9d049cf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60049cfb, SUC_lexicon_tags_37 },
    { 0xfa249cfb, SUC_lexicon_tags_15 },
    { 0xcc049cfc, SUC_lexicon_tags_33 },
    { 0xa5049cfe, SUC_lexicon_tags_13 },
    { 0x69449cfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4249d01, SUC_lexicon_tags_37 },
    { 0x8c049d02, SUC_lexicon_tags_19 },
    { 0x27a49d03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe49d07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74249d11, SUC_lexicon_tags_100 },
    { 0x48449d11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e49d16, SUC_lexicon_tags_31 },
    { 0x76449d17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c49d1a, SUC_lexicon_tags_46 },
    { 0xeb249d1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c449d1e, SUC_lexicon_tags_15 },
    { 0xed649d1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb049d24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8449d26, SUC_lexicon_tags_28 },
    { 0x3b049d27, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x49e49d29, SUC_lexicon_tags_32 },
    { 0x24249d2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa49d2d, SUC_lexicon_tags_49 },
    { 0x25449d2e, SUC_lexicon_tags_15 },
    { 0x63c49d2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb649d32, SUC_lexicon_tags_47 },
    { 0xece49d32, SUC_lexicon_tags_1 },
    { 0xcd249d33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x65849d37, SUC_lexicon_tags_34 },
    { 0x75c49d38, SUC_lexicon_tags_12 },
    { 0x48a49d39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48c49d3b, SUC_lexicon_tags_506 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2449d3f, SUC_lexicon_tags_11 },
    { 0x3ac49d40, SUC_lexicon_tags_15 },
    { 0xcfc49d41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4f049d43, SUC_lexicon_tags_49 },
    { 0xbd849d44, SUC_lexicon_tags_14 },
    { 0xca249d45, SUC_lexicon_tags_25 },
    { 0x97049d44, SUC_lexicon_tags_63 },
    { 0x5b849d44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3049d4b, SUC_lexicon_tags_8 },
    { 0xe1e49d4c, SUC_lexicon_tags_25 },
    { 0xdf249d4d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e049d50, SUC_lexicon_tags_11 },
    { 0x52e49d50, SUC_lexicon_tags_4 },
    { 0xce49d50, SUC_lexicon_tags_7 },
    { 0x5b049d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6449d57, SUC_lexicon_tags_2 },
    { 0x5ee49d57, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x57849d5a, SUC_lexicon_tags_14 },
    { 0x93a49d5a, SUC_lexicon_tags_29 },
    { 0xb9049d5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca49d5f, SUC_lexicon_tags_68 },
    { 0xed849d5f, SUC_lexicon_tags_27 },
    { 0xcd649d60, SUC_lexicon_tags_1 },
    { 0x83449d62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42449d69, SUC_lexicon_tags_7 },
    { 0x41049d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1249d6d, SUC_lexicon_tags_11 },
    { 0x65449d6d, SUC_lexicon_tags_30 },
    { 0xdd649d6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e49d76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c049d79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde649d7e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b249d83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a49d89, SUC_lexicon_tags_47 },
    { 0x9ac49d8a, SUC_lexicon_tags_37 },
    { 0x28649d8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe49d8d, SUC_lexicon_tags_47 },
    { 0xfee49d8e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a49d92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c49d99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2049d9b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc49da0, SUC_lexicon_tags_45 },
    { 0xfae49da1, SUC_lexicon_tags_48 },
    { 0x4ea49da2, SUC_lexicon_tags_1 },
    { 0x77849da3, SUC_lexicon_tags_23 },
    { 0xd4449da1, SUC_lexicon_tags_37 },
    { 0xc1249da3, SUC_lexicon_tags_29 },
    { 0x52249da6, SUC_lexicon_tags_3 },
    { 0xb3049da7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42849dab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3e49dad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11e49daf, SUC_lexicon_tags_7 },
    { 0xdc049daf, SUC_lexicon_tags_17 },
    { 0x9a449db1, SUC_lexicon_tags_47 },
    { 0x8ac49db0, SUC_lexicon_tags_11 },
    { 0x39c49db3, SUC_lexicon_tags_25 },
    { 0x33e49db4, SUC_lexicon_tags_7 },
    { 0xdf849db5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5649dbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2be49dbe, SUC_lexicon_tags_243 },
    { 0x28049dbe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54249dc1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe1849dc3, SUC_lexicon_tags_15 },
    { 0xc4e49dc3, SUC_lexicon_tags_63 },
    { 0x2ca49dc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x66849dc8, SUC_lexicon_tags_29 },
    { 0x8ec49dc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x70849dcc, SUC_lexicon_tags_37 },
    { 0x94c49dcd, SUC_lexicon_tags_47 },
    { 0x64249dcd, SUC_lexicon_tags_45 },
    { 0x76249dcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ca49dd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc849dd7, SUC_lexicon_tags_11 },
    { 0x8d249dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11c49dda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50449ddd, SUC_lexicon_tags_7 },
    { 0x3ce49dde, SUC_lexicon_tags_17 },
    { 0x5a849ddf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48449dea, SUC_lexicon_tags_25 },
    { 0x44a49deb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f049ded, SUC_lexicon_tags_17 },
    { 0xeac49dee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c49df3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f649df5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b649dfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c849dfd, SUC_lexicon_tags_70 },
    { 0x8c449dfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b049e00, SUC_lexicon_tags_37 },
    { 0x27849e01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9049e03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d849e05, SUC_lexicon_tags_2 },
    { 0x91049e06, SUC_lexicon_tags_77 },
    { 0x9b449e07, SUC_lexicon_tags_11 },
    { 0x20849e08, SUC_lexicon_tags_38 },
    { 0xc3c49e05, SUC_lexicon_tags_45 },
    { 0x3e049e08, SUC_lexicon_tags_37 },
    { 0x2ba49e08, SUC_lexicon_tags_19 },
    { 0x22649e0c, SUC_lexicon_tags_21 },
    { 0x13649e0d, SUC_lexicon_tags_14 },
    { 0xae449e0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a449e1a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x9b649e1c, SUC_lexicon_tags_17 },
    { 0x27a49e1d, SUC_lexicon_tags_11 },
    { 0x55649e1e, SUC_lexicon_tags_1 },
    { 0x66849e1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26449e26, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x53c49e28, SUC_lexicon_tags_43 },
    { 0xda49e29, SUC_lexicon_tags_63 },
    { 0xe3a49e28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab849e2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6649e34, SUC_lexicon_tags_37 },
    { 0xdf449e34, SUC_lexicon_tags_47 },
    { 0xb9a49e36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xec449e39, SUC_lexicon_tags_37 },
    { 0x74a49e3a, SUC_lexicon_tags_1266 },
    { 0x7a49e3b, SUC_lexicon_tags_12 },
    { 0xdca49e3b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce49e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba49e46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a649e49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c449e51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de49e55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a249e57, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a49e5a, SUC_lexicon_tags_46 },
    { 0x13049e5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e49e5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa3c49e60, SUC_lexicon_tags_12 },
    { 0xb5e49e61, SUC_lexicon_tags_22 },
    { 0x6c049e62, SUC_lexicon_tags_0 },
    { 0xbd249e63, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e49e66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca49e6e, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x97e49e70, SUC_lexicon_tags_7 },
    { 0x20449e71, SUC_lexicon_tags_39 },
    { 0xb8249e71, SUC_lexicon_tags_447 },
    { 0x19449e73, SUC_lexicon_tags_43 },
    { 0x63c49e70, SUC_lexicon_tags_8 },
    { 0xcdc49e75, SUC_lexicon_tags_29 },
    { 0x2be49e70, SUC_lexicon_tags_151 },
    { 0xbaa49e76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee49e7b, SUC_lexicon_tags_37 },
    { 0xcaa49e7c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1249e7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6249e82, SUC_lexicon_tags_29 },
    { 0xe5649e82, SUC_lexicon_tags_25 },
    { 0x7dc49e82, SUC_lexicon_tags_84 },
    { 0xd7c49e83, SUC_lexicon_tags_0 },
    { 0x81649e86, SUC_lexicon_tags_0 },
    { 0x98c49e87, SUC_lexicon_tags_45 },
    { 0x78649e88, SUC_lexicon_tags_38 },
    { 0x57249e84, SUC_lexicon_tags_60 },
    { 0xeba49e8a, SUC_lexicon_tags_33 },
    { 0x69a49e8b, SUC_lexicon_tags_28 },
    { 0x2b249e8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c649e90, SUC_lexicon_tags_19 },
    { 0xcc49e91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede49e95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88849e98, SUC_lexicon_tags_11 },
    { 0xb9049e99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c49e9e, SUC_lexicon_tags_9 },
    { 0xab249e9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5249ea6, SUC_lexicon_tags_37 },
    { 0xd9449ea6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d449eb3, SUC_lexicon_tags_27 },
    { 0xeea49eb3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc49eb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43049ebf, SUC_lexicon_tags_49 },
    { 0xbd449ec0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91049ec4, SUC_lexicon_tags_28 },
    { 0xe4649ec5, SUC_lexicon_tags_17 },
    { 0xdd849ec5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5249ecc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfc49ece, SUC_lexicon_tags_29 },
    { 0x90c49ece, SUC_lexicon_tags_12 },
    { 0x37249ece, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5049ed7, SUC_lexicon_tags_49 },
    { 0x92c49ed8, SUC_lexicon_tags_11 },
    { 0x73e49ed9, SUC_lexicon_tags_0 },
    { 0x19a49ed8, SUC_lexicon_tags_10 },
    { 0x50e49edb, SUC_lexicon_tags_12 },
    { 0x5249edc, SUC_lexicon_tags_27 },
    { 0xa1a49edd, SUC_lexicon_tags_150 },
    { 0x2ae49ede, SUC_lexicon_tags_63 },
    { 0x6be49edc, SUC_lexicon_tags_7 },
    { 0x50e49ee0, SUC_lexicon_tags_14 },
    { 0x64a49edd, SUC_lexicon_tags_12 },
    { 0xa49edb, SUC_lexicon_tags_19 },
    { 0x2f249ee3, SUC_lexicon_tags_25 },
    { 0x39649ee4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57449ee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba849eeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e849eee, SUC_lexicon_tags_0 },
    { 0x51c49eef, SUC_lexicon_tags_33 },
    { 0x6a249eee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6249ef4, SUC_lexicon_tags_31 },
    { 0x54a49ef4, SUC_lexicon_tags_15 },
    { 0xa3449ef5, SUC_lexicon_tags_38 },
    { 0x1c249ef7, SUC_lexicon_tags_31 },
    { 0x7b849ef8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35c49efa, SUC_lexicon_tags_7 },
    { 0x4649efa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f849f00, SUC_lexicon_tags_12 },
    { 0xc3249f01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0849f05, SUC_lexicon_tags_38 },
    { 0x9e049f05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be49f0a, SUC_lexicon_tags_33 },
    { 0xa3049f0a, SUC_lexicon_tags_12 },
    { 0x73249f0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c49f10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x89849f12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c49f15, SUC_lexicon_tags_47 },
    { 0xc3e49f16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9dc49f18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x89049f1b, SUC_lexicon_tags_37 },
    { 0xd1e49f1c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c49f20, SUC_lexicon_tags_2 },
    { 0x9dc49f21, SUC_lexicon_tags_12 },
    { 0xf249f22, SUC_lexicon_tags_12 },
    { 0x2c249f23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca49f26, SUC_lexicon_tags_12 },
    { 0x3be49f27, SUC_lexicon_tags_17 },
    { 0xa0649f28, SUC_lexicon_tags_2 },
    { 0xd9249f29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96649f31, SUC_lexicon_tags_17 },
    { 0x16849f32, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e49f35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a249f37, SUC_lexicon_tags_43 },
    { 0x2de49f37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad849f3e, SUC_lexicon_tags_6 },
    { 0xe5e49f3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1b649f41, SUC_lexicon_tags_43 },
    { 0xea849f42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8249f45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e49f48, SUC_lexicon_tags_17 },
    { 0x83449f49, SUC_lexicon_tags_17 },
    { 0xc849f48, SUC_lexicon_tags_27 },
    { 0xb8049f4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a49f4e, SUC_lexicon_tags_15 },
    { 0xb9849f4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41649f54, SUC_lexicon_tags_17 },
    { 0x6da49f55, SUC_lexicon_tags_33 },
    { 0x9f249f54, SUC_lexicon_tags_33 },
    { 0x41a49f57, SUC_lexicon_tags_19 },
    { 0x8a849f57, SUC_lexicon_tags_6 },
    { 0x8d849f59, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2449f5c, SUC_lexicon_tags_17 },
    { 0xed449f5d, SUC_lexicon_tags_27 },
    { 0x1c49f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de49f65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe49f6a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e49f6f, SUC_lexicon_tags_23 },
    { 0x2ba49f70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef249f72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a49f78, SUC_lexicon_tags_11 },
    { 0x6ae49f79, SUC_lexicon_tags_23 },
    { 0xf4849f7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe49f7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7049f86, SUC_lexicon_tags_65 },
    { 0x17849f86, SUC_lexicon_tags_37 },
    { 0xb6849f88, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x84049f8a, SUC_lexicon_tags_54 },
    { 0x16049f8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c49f90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25849f97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x28249f99, SUC_lexicon_tags_17 },
    { 0xc0c49f9a, SUC_lexicon_tags_1 },
    { 0xc1449f9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ba49f9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a49fa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9449fa5, SUC_lexicon_tags_37 },
    { 0xae449fa5, SUC_lexicon_tags_23 },
    { 0xf649fa5, SUC_lexicon_tags_82 },
    { 0xf8049fa8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59049fac, SUC_lexicon_tags_34 },
    { 0x72649fad, SUC_lexicon_tags_10 },
    { 0x1bc49fac, SUC_lexicon_tags_17 },
    { 0xcba49fad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40249fb1, SUC_lexicon_tags_33 },
    { 0x1c249fb2, SUC_lexicon_tags_23 },
    { 0xa3a49fb3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79649fb8, SUC_lexicon_tags_110 },
    { 0x35449fb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7449fbd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x59849fbf, SUC_lexicon_tags_55 },
    { 0x4f849fc0, SUC_lexicon_tags_17 },
    { 0x50849fc0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae49fc4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e49fc7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75849fce, SUC_lexicon_tags_28 },
    { 0x32c49fce, SUC_lexicon_tags_9 },
    { 0x5fa49fd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92c49fd2, SUC_lexicon_tags_32 },
    { 0x50049fd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91049fd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x76049fd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86449fda, SUC_lexicon_tags_14 },
    { 0xe1449fdb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8a49fdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2fa49fdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e49fe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x77249fe3, SUC_lexicon_tags_11 },
    { 0x27a49fe4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d249fe7, SUC_lexicon_tags_25 },
    { 0xe3c49fe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a49fec, SUC_lexicon_tags_37 },
    { 0x79049fed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67449ff1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21049ff3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54c49ff5, SUC_lexicon_tags_45 },
    { 0xda249ff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9a49ff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2849ffd, SUC_lexicon_tags_49 },
    { 0x41649ffe, SUC_lexicon_tags_17 },
    { 0xa7049fff, SUC_lexicon_tags_11 },
    { 0x62049ffe, SUC_lexicon_tags_17 },
    { 0xab049ffe, SUC_lexicon_tags_31 },
    { 0x1a4a002, SUC_lexicon_tags_38 },
    { 0xe124a003, SUC_lexicon_tags_37 },
    { 0xf444a004, SUC_lexicon_tags_30 },
    { 0x12e4a002, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6064a007, SUC_lexicon_tags_47 },
    { 0x8fa4a008, SUC_lexicon_tags_2 },
    { 0xaee4a009, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e4a00e, SUC_lexicon_tags_81 },
    { 0xc804a00e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e4a017, SUC_lexicon_tags_19 },
    { 0xb544a017, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe4a01c, SUC_lexicon_tags_15 },
    { 0x5ac4a01d, SUC_lexicon_tags_17 },
    { 0x5224a01e, SUC_lexicon_tags_15 },
    { 0xf5e4a01f, SUC_lexicon_tags_17 },
    { 0x4644a01f, SUC_lexicon_tags_26 },
    { 0xb624a021, SUC_lexicon_tags_26 },
    { 0xc504a022, SUC_lexicon_tags_43 },
    { 0x8b84a023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4684a025, SUC_lexicon_tags_11 },
    { 0x5904a026, SUC_lexicon_tags_1 },
    { 0xfc24a027, SUC_lexicon_tags_68 },
    { 0xda4a027, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da4a02f, SUC_lexicon_tags_31 },
    { 0xafa4a030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c04a033, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a44a036, SUC_lexicon_tags_6 },
    { 0x7a04a037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e4a03c, SUC_lexicon_tags_25 },
    { 0x3844a03d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da4a042, SUC_lexicon_tags_37 },
    { 0x184a043, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf864a046, SUC_lexicon_tags_23 },
    { 0x5664a047, SUC_lexicon_tags_45 },
    { 0x6f84a048, SUC_lexicon_tags_1 },
    { 0xfc64a049, SUC_lexicon_tags_45 },
    { 0x7764a046, SUC_lexicon_tags_1 },
    { 0xa164a047, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc084a050, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2104a052, SUC_lexicon_tags_1 },
    { 0xfa24a053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a44a055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1a4a057, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f84a05a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa824a05d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff84a061, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26e4a063, SUC_lexicon_tags_12 },
    { 0xc744a064, SUC_lexicon_tags_45 },
    { 0xc744a065, SUC_lexicon_tags_1 },
    { 0x3a64a066, SUC_lexicon_tags_38 },
    { 0x9884a067, SUC_lexicon_tags_37 },
    { 0x7104a068, SUC_lexicon_tags_37 },
    { 0x5644a069, SUC_lexicon_tags_37 },
    { 0xd424a065, SUC_lexicon_tags_27 },
    { 0xc764a06b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa084a06e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa544a072, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe4a075, SUC_lexicon_tags_37 },
    { 0xd704a075, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae4a07e, SUC_lexicon_tags_17 },
    { 0xb7a4a07e, SUC_lexicon_tags_1 },
    { 0x5b24a07e, SUC_lexicon_tags_45 },
    { 0xfa44a081, SUC_lexicon_tags_16 },
    { 0x8be4a082, SUC_lexicon_tags_63 },
    { 0x91a4a082, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba4a086, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f64a089, SUC_lexicon_tags_55 },
    { 0x2ce4a08a, SUC_lexicon_tags_42 },
    { 0xa3c4a08b, SUC_lexicon_tags_108 },
    { 0x4ae4a08b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a4a092, SUC_lexicon_tags_15 },
    { 0x7124a093, SUC_lexicon_tags_9 },
    { 0x9e4a094, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb984a097, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1004a09a, SUC_lexicon_tags_2 },
    { 0x7aa4a09b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x93e4a09d, SUC_lexicon_tags_38 },
    { 0x4284a09e, SUC_lexicon_tags_49 },
    { 0xac24a09e, SUC_lexicon_tags_17 },
    { 0xeb84a0a0, SUC_lexicon_tags_2 },
    { 0x7444a0a0, SUC_lexicon_tags_82 },
    { 0x9a4a0a0, SUC_lexicon_tags_12 },
    { 0xa244a0a3, SUC_lexicon_tags_98 },
    { 0x2b44a0a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c64a0a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1b04a0a9, SUC_lexicon_tags_2 },
    { 0x3fc4a0aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe364a0ad, SUC_lexicon_tags_57 },
    { 0xdf44a0ad, SUC_lexicon_tags_7 },
    { 0x4a84a0af, SUC_lexicon_tags_8 },
    { 0xb8c4a0b0, SUC_lexicon_tags_26 },
    { 0x6584a0ad, SUC_lexicon_tags_7 },
    { 0x9244a0b2, SUC_lexicon_tags_2 },
    { 0xbe64a0b2, SUC_lexicon_tags_17 },
    { 0xbec4a0b0, SUC_lexicon_tags_15 },
    { 0xba04a0b5, SUC_lexicon_tags_12 },
    { 0x4184a0b6, SUC_lexicon_tags_23 },
    { 0x88e4a0b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47a4a0b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b44a0bd, SUC_lexicon_tags_15 },
    { 0x5384a0bd, SUC_lexicon_tags_12 },
    { 0x5004a0bf, SUC_lexicon_tags_37 },
    { 0x91a4a0be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa964a0c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc264a0c7, SUC_lexicon_tags_68 },
    { 0x53a4a0c7, SUC_lexicon_tags_31 },
    { 0x9604a0c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4784a0d2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x68e4a0d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e4a0d8, SUC_lexicon_tags_25 },
    { 0x8aa4a0d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4144a0dc, SUC_lexicon_tags_17 },
    { 0xf84a0dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4644a0df, SUC_lexicon_tags_0 },
    { 0xc44a0e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5164a0e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb904a0e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e4a0e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b04a0ea, SUC_lexicon_tags_25 },
    { 0xf004a0eb, SUC_lexicon_tags_6 },
    { 0xcd44a0ec, SUC_lexicon_tags_6 },
    { 0x8424a0ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e4a0f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a24a0fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e4a0fe, SUC_lexicon_tags_21 },
    { 0xb944a0fe, SUC_lexicon_tags_24 },
    { 0x9a04a100, SUC_lexicon_tags_154 },
    { 0x464a100, SUC_lexicon_tags_14 },
    { 0x9884a102, SUC_lexicon_tags_43 },
    { 0xf7c4a0fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a4a10c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1104a10e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe0c4a110, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c64a113, SUC_lexicon_tags_15 },
    { 0x9ba4a114, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7d84a116, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec4a121, SUC_lexicon_tags_7 },
    { 0x6784a121, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b24a128, SUC_lexicon_tags_17 },
    { 0x6fe4a129, SUC_lexicon_tags_28 },
    { 0xcc04a12a, SUC_lexicon_tags_63 },
    { 0x82c4a12b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef84a12e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc4a132, SUC_lexicon_tags_104 },
    { 0x844a133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc44a137, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c4a142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3584a14a, SUC_lexicon_tags_38 },
    { 0xf464a14a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc224a14f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ca4a151, SUC_lexicon_tags_56 },
    { 0x99a4a151, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb04a155, SUC_lexicon_tags_25 },
    { 0xf64a156, SUC_lexicon_tags_7 },
    { 0x3c04a157, SUC_lexicon_tags_29 },
    { 0x7684a157, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x1a4a15a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c4a15d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb084a161, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8124a163, SUC_lexicon_tags_29 },
    { 0x8dc4a163, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a167, SUC_lexicon_tags_7 },
    { 0xcd84a168, SUC_lexicon_tags_23 },
    { 0x5b04a169, SUC_lexicon_tags_45 },
    { 0xd724a168, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x25e4a16c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c4a170, SUC_lexicon_tags_36 },
    { 0x884a171, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7484a174, SUC_lexicon_tags_624 },
    { 0x2f04a175, SUC_lexicon_tags_58 },
    { 0x87a4a175, SUC_lexicon_tags_25 },
    { 0x7c64a174, SUC_lexicon_tags_35 },
    { 0xae64a178, SUC_lexicon_tags_0 },
    { 0x4364a178, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2684a17d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf24a181, SUC_lexicon_tags_25 },
    { 0xeee4a182, SUC_lexicon_tags_63 },
    { 0x6f64a183, SUC_lexicon_tags_61 },
    { 0x7f24a184, SUC_lexicon_tags_31 },
    { 0x7c4a185, SUC_lexicon_tags_27 },
    { 0x1984a186, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2264a18f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a4a192, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd244a197, SUC_lexicon_tags_13 },
    { 0xef24a198, SUC_lexicon_tags_11 },
    { 0xaa04a199, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6324a19c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2824a19e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa64a1a2, SUC_lexicon_tags_7 },
    { 0x2724a1a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7924a1aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3944a1ae, SUC_lexicon_tags_28 },
    { 0xa7e4a1ae, SUC_lexicon_tags_47 },
    { 0xb5e4a1b0, SUC_lexicon_tags_17 },
    { 0x7604a1b1, SUC_lexicon_tags_2 },
    { 0xb4c4a1b0, SUC_lexicon_tags_37 },
    { 0xf6e4a1b3, SUC_lexicon_tags_38 },
    { 0x4344a1b3, SUC_lexicon_tags_65 },
    { 0x1284a1b2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d84a1b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaec4a1bb, SUC_lexicon_tags_1 },
    { 0x9ae4a1bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2724a1be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2a4a1c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c4a1c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfe4a1c8, SUC_lexicon_tags_45 },
    { 0xb864a1c9, SUC_lexicon_tags_38 },
    { 0x8964a1c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e4a1ce, SUC_lexicon_tags_25 },
    { 0xbb44a1cf, SUC_lexicon_tags_0 },
    { 0x1744a1cf, SUC_lexicon_tags_19 },
    { 0x4ec4a1d1, SUC_lexicon_tags_17 },
    { 0xd364a1d2, SUC_lexicon_tags_1 },
    { 0xef64a1d2, SUC_lexicon_tags_14 },
    { 0x4944a1d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4a1d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4a1da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7184a1dd, SUC_lexicon_tags_2 },
    { 0x6e64a1de, SUC_lexicon_tags_38 },
    { 0xe5c4a1df, SUC_lexicon_tags_7 },
    { 0xae84a1df, SUC_lexicon_tags_12 },
    { 0x6d64a1e1, SUC_lexicon_tags_62 },
    { 0xa084a1e0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef84a1ea, SUC_lexicon_tags_15 },
    { 0x44a1eb, SUC_lexicon_tags_19 },
    { 0x7184a1eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc684a1ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc544a1f5, SUC_lexicon_tags_26 },
    { 0x1f44a1f5, SUC_lexicon_tags_46 },
    { 0x5844a1f5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5504a1fb, SUC_lexicon_tags_1 },
    { 0xde44a1fb, SUC_lexicon_tags_37 },
    { 0x7ae4a1fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6964a1ff, SUC_lexicon_tags_15 },
    { 0xfda4a200, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4a203, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa4a207, SUC_lexicon_tags_61 },
    { 0x9684a208, SUC_lexicon_tags_2 },
    { 0xa644a209, SUC_lexicon_tags_23 },
    { 0xf464a20a, SUC_lexicon_tags_47 },
    { 0x3f44a20b, SUC_lexicon_tags_82 },
    { 0xd2a4a20c, SUC_lexicon_tags_33 },
    { 0x97e4a208, SUC_lexicon_tags_2 },
    { 0x8824a20e, SUC_lexicon_tags_42 },
    { 0x2b24a20e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644a214, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8024a217, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8444a21d, SUC_lexicon_tags_32 },
    { 0x1124a21d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a4a221, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea4a225, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb264a227, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe964a22d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6764a22f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1144a231, SUC_lexicon_tags_83 },
    { 0xec4a232, SUC_lexicon_tags_2 },
    { 0x86a4a232, SUC_lexicon_tags_11 },
    { 0xed44a232, SUC_lexicon_tags_17 },
    { 0x15a4a235, SUC_lexicon_tags_2 },
    { 0x1de4a236, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe864a238, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4864a23d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa324a23f, SUC_lexicon_tags_17 },
    { 0x2004a240, SUC_lexicon_tags_2 },
    { 0x71e4a241, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac84a243, SUC_lexicon_tags_15 },
    { 0xd9c4a244, SUC_lexicon_tags_1 },
    { 0xe6c4a244, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6284a248, SUC_lexicon_tags_49 },
    { 0x56e4a249, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77a4a24b, SUC_lexicon_tags_12 },
    { 0x95c4a24b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcb84a24e, SUC_lexicon_tags_38 },
    { 0x8584a24f, SUC_lexicon_tags_14 },
    { 0xc324a250, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54e4a252, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f04a255, SUC_lexicon_tags_0 },
    { 0x5644a256, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf104a25a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf284a265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c84a267, SUC_lexicon_tags_85 },
    { 0x3d24a267, SUC_lexicon_tags_7 },
    { 0x8944a269, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72a4a26b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7a24a26d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a4a274, SUC_lexicon_tags_11 },
    { 0x4304a274, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd64a278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85c4a27a, SUC_lexicon_tags_15 },
    { 0x9f64a27b, SUC_lexicon_tags_12 },
    { 0x8e4a27a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1e24a27e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe4a281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c4a284, SUC_lexicon_tags_57 },
    { 0x3f44a285, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x424a287, SUC_lexicon_tags_7 },
    { 0x1c44a287, SUC_lexicon_tags_8 },
    { 0x6064a289, SUC_lexicon_tags_44 },
    { 0xa084a288, SUC_lexicon_tags_87 },
    { 0x9564a28b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd324a28d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff04a291, SUC_lexicon_tags_29 },
    { 0x95c4a292, SUC_lexicon_tags_25 },
    { 0x4584a293, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa4a29b, SUC_lexicon_tags_12 },
    { 0xf924a29c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b84a29f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6744a2a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b44a2ab, SUC_lexicon_tags_7 },
    { 0x9a84a2ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde84a2ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1384a2b2, SUC_lexicon_tags_15 },
    { 0x86a4a2b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a4a2b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbba4a2ba, SUC_lexicon_tags_27 },
    { 0x7a04a2ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8da4a2bd, SUC_lexicon_tags_87 },
    { 0x4504a2be, SUC_lexicon_tags_17 },
    { 0xb324a2bd, SUC_lexicon_tags_11 },
    { 0x83a4a2c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b64a2c3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64a2c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb064a2c9, SUC_lexicon_tags_37 },
    { 0x55a4a2ca, SUC_lexicon_tags_19 },
    { 0xb084a2cb, SUC_lexicon_tags_9 },
    { 0x9344a2cc, SUC_lexicon_tags_47 },
    { 0xd4c4a2cd, SUC_lexicon_tags_49 },
    { 0x9104a2ce, SUC_lexicon_tags_215 },
    { 0x7044a2ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea04a2d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaae4a2d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb104a2d9, SUC_lexicon_tags_7 },
    { 0x3b84a2da, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa804a2dc, SUC_lexicon_tags_25 },
    { 0xdf44a2dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bc4a2df, SUC_lexicon_tags_17 },
    { 0xd6c4a2e0, SUC_lexicon_tags_37 },
    { 0xd5c4a2e1, SUC_lexicon_tags_7 },
    { 0xac24a2e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da4a2ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x484a2ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa164a2f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd244a2f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d24a2f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b44a2f8, SUC_lexicon_tags_11 },
    { 0xc3a4a2f9, SUC_lexicon_tags_30 },
    { 0xf044a2fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a4a2fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa124a2ff, SUC_lexicon_tags_15 },
    { 0xdae4a300, SUC_lexicon_tags_11 },
    { 0xa8a4a301, SUC_lexicon_tags_3 },
    { 0xeb04a300, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7244a305, SUC_lexicon_tags_25 },
    { 0xf684a306, SUC_lexicon_tags_17 },
    { 0x8364a306, SUC_lexicon_tags_25 },
    { 0x1ba4a308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c44a30a, SUC_lexicon_tags_7 },
    { 0x3084a30b, SUC_lexicon_tags_11 },
    { 0x86e4a30b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce64a30f, SUC_lexicon_tags_7 },
    { 0xc0e4a310, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5244a315, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c4a31b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5284a31e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1064a321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1dc4a323, SUC_lexicon_tags_28 },
    { 0x5e84a324, SUC_lexicon_tags_17 },
    { 0x9224a325, SUC_lexicon_tags_25 },
    { 0x7124a325, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e4a329, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd684a32e, SUC_lexicon_tags_11 },
    { 0xd9a4a32f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba4a332, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3ea4a334, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd924a336, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0x55c4a338, SUC_lexicon_tags_17 },
    { 0xa0a4a339, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x664a33c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1524a33f, SUC_lexicon_tags_35 },
    { 0xd7e4a33f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xabe4a342, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e4a344, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc04a346, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e4a34a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4644a34c, SUC_lexicon_tags_48 },
    { 0x9e04a34d, SUC_lexicon_tags_37 },
    { 0x20a4a34e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1784a353, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6704a355, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6344a358, SUC_lexicon_tags_87 },
    { 0xa984a359, SUC_lexicon_tags_33 },
    { 0xd884a35a, SUC_lexicon_tags_11 },
    { 0x384a358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc204a35f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b24a366, SUC_lexicon_tags_17 },
    { 0x5664a367, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb584a36b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c4a36e, SUC_lexicon_tags_7 },
    { 0x66e4a36f, SUC_lexicon_tags_11 },
    { 0xa4c4a36e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc4a37a, SUC_lexicon_tags_1 },
    { 0x9804a37b, SUC_lexicon_tags_7 },
    { 0xc2a4a37c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x93e4a37e, SUC_lexicon_tags_46 },
    { 0x89e4a37f, SUC_lexicon_tags_11 },
    { 0x6584a380, SUC_lexicon_tags_26 },
    { 0xd4e4a381, SUC_lexicon_tags_38 },
    { 0x37a4a381, SUC_lexicon_tags_45 },
    { 0x67e4a381, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xef24a386, SUC_lexicon_tags_37 },
    { 0x4804a387, SUC_lexicon_tags_11 },
    { 0x64a388, SUC_lexicon_tags_32 },
    { 0x9004a389, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a04a38d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b24a393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd724a395, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff24a398, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2504a39d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac4a3a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4a3a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4944a3ad, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x5624a3af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7624a3b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d44a3b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe724a3bb, SUC_lexicon_tags_3 },
    { 0xd624a3bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82c4a3be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7184a3c0, SUC_lexicon_tags_11 },
    { 0xa64a3c0, SUC_lexicon_tags_33 },
    { 0xfec4a3c2, SUC_lexicon_tags_29 },
    { 0xcf24a3c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e24a3c5, SUC_lexicon_tags_37 },
    { 0xc2e4a3c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7564a3c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae4a3cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce84a3ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e4a3d1, SUC_lexicon_tags_38 },
    { 0xa3e4a3d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x464a3d5, SUC_lexicon_tags_25 },
    { 0x6be4a3d6, SUC_lexicon_tags_25 },
    { 0x8444a3d7, SUC_lexicon_tags_2 },
    { 0xd4c4a3d5, SUC_lexicon_tags_26 },
    { 0x4e64a3d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a4a3df, SUC_lexicon_tags_27 },
    { 0xbbc4a3df, SUC_lexicon_tags_14 },
    { 0xff84a3df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf204a3e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7244a3ec, SUC_lexicon_tags_37 },
    { 0xa804a3ed, SUC_lexicon_tags_27 },
    { 0xa4a3ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f04a3f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e84a3f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5964a3f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe804a3f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c4a3fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7384a3fe, SUC_lexicon_tags_46 },
    { 0x5804a3ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6564a401, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe144a404, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7304a407, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67e4a409, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e84a415, SUC_lexicon_tags_49 },
    { 0xa624a416, SUC_lexicon_tags_7 },
    { 0xf164a416, SUC_lexicon_tags_45 },
    { 0x8ee4a416, SUC_lexicon_tags_25 },
    { 0x6f44a419, SUC_lexicon_tags_11 },
    { 0xbc64a41a, SUC_lexicon_tags_14 },
    { 0xbf64a41b, SUC_lexicon_tags_17 },
    { 0x4c04a415, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea04a41f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79e4a421, SUC_lexicon_tags_16 },
    { 0x5c24a422, SUC_lexicon_tags_29 },
    { 0x1204a421, SUC_lexicon_tags_29 },
    { 0x1e64a424, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda44a42a, SUC_lexicon_tags_12 },
    { 0x6764a42b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a4a431, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa4a434, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ec4a436, SUC_lexicon_tags_7 },
    { 0x3e44a436, SUC_lexicon_tags_1 },
    { 0x4004a437, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc4a43c, SUC_lexicon_tags_30 },
    { 0x8fa4a43c, SUC_lexicon_tags_29 },
    { 0xc204a43e, SUC_lexicon_tags_33 },
    { 0x7444a43f, SUC_lexicon_tags_44 },
    { 0xcf64a43f, SUC_lexicon_tags_2 },
    { 0xe7a4a441, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd844a444, SUC_lexicon_tags_696 },
    { 0x4dc4a445, SUC_lexicon_tags_25 },
    { 0x43a4a445, SUC_lexicon_tags_33 },
    { 0xcb84a445, SUC_lexicon_tags_63 },
    { 0xede4a448, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c4a44c, SUC_lexicon_tags_17 },
    { 0x6f24a44d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaa4a44f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4364a451, SUC_lexicon_tags_17 },
    { 0x7364a451, SUC_lexicon_tags_29 },
    { 0x5d44a453, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e4a45c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7644a45e, SUC_lexicon_tags_48 },
    { 0x1344a45f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa124a46a, SUC_lexicon_tags_26 },
    { 0xe964a46b, SUC_lexicon_tags_55 },
    { 0x5644a46b, SUC_lexicon_tags_14 },
    { 0xd4c4a46d, SUC_lexicon_tags_47 },
    { 0x2444a46d, SUC_lexicon_tags_17 },
    { 0x5104a46c, SUC_lexicon_tags_23 },
    { 0x8b24a470, SUC_lexicon_tags_11 },
    { 0xca4a46a, SUC_lexicon_tags_85 },
    { 0x3a44a46d, SUC_lexicon_tags_17 },
    { 0x1d44a46f, SUC_lexicon_tags_82 },
    { 0x6804a474, SUC_lexicon_tags_12 },
    { 0x9a44a474, SUC_lexicon_tags_6 },
    { 0xca24a475, SUC_lexicon_tags_14 },
    { 0xd864a46b, SUC_lexicon_tags_17 },
    { 0xbec4a478, SUC_lexicon_tags_15 },
    { 0xcee4a479, SUC_lexicon_tags_37 },
    { 0xd2a4a479, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1324a480, SUC_lexicon_tags_25 },
    { 0x52a4a481, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba4a486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe764a489, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a4a498, SUC_lexicon_tags_11 },
    { 0xde64a499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc84a49f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c24a4a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e4a4a6, SUC_lexicon_tags_17 },
    { 0xc64a4a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea4a4ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da4a4b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaa4a4b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de4a4b8, SUC_lexicon_tags_17 },
    { 0x2b24a4b9, SUC_lexicon_tags_11 },
    { 0x3744a4ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e4a4c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2964a4c2, SUC_lexicon_tags_8 },
    { 0xa864a4c3, SUC_lexicon_tags_30 },
    { 0xdd84a4c2, SUC_lexicon_tags_37 },
    { 0xcb44a4c4, SUC_lexicon_tags_30 },
    { 0x2ec4a4c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4a4ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e4a4cd, SUC_lexicon_tags_11 },
    { 0x3c04a4cd, SUC_lexicon_tags_6 },
    { 0xee64a4cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac4a4d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e4a4de, SUC_lexicon_tags_46 },
    { 0x9c64a4de, SUC_lexicon_tags_7 },
    { 0x51c4a4e0, SUC_lexicon_tags_66 },
    { 0xbf24a4e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124a4e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0c4a4e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc284a4e9, SUC_lexicon_tags_7 },
    { 0x1be4a4e9, SUC_lexicon_tags_49 },
    { 0x5764a4eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4a4ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf844a4f0, SUC_lexicon_tags_39 },
    { 0x6724a4f1, SUC_lexicon_tags_49 },
    { 0xad24a4f2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a4a4f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be4a501, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe844a503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x864a505, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce64a50a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda64a50c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4484a512, SUC_lexicon_tags_12 },
    { 0xbc04a513, SUC_lexicon_tags_49 },
    { 0xf9c4a514, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde84a51e, SUC_lexicon_tags_37 },
    { 0xc984a51f, SUC_lexicon_tags_14 },
    { 0x1d44a520, SUC_lexicon_tags_37 },
    { 0xba24a520, SUC_lexicon_tags_1 },
    { 0x5464a522, SUC_lexicon_tags_17 },
    { 0xc324a51f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf424a525, SUC_lexicon_tags_27 },
    { 0xc104a525, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb964a52a, SUC_lexicon_tags_1 },
    { 0xb584a52b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e4a530, SUC_lexicon_tags_15 },
    { 0x7a64a531, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x224a533, SUC_lexicon_tags_84 },
    { 0xa684a533, SUC_lexicon_tags_37 },
    { 0x5a4a534, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8724a537, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7224a53a, SUC_lexicon_tags_31 },
    { 0x75c4a53b, SUC_lexicon_tags_28 },
    { 0x3624a53c, SUC_lexicon_tags_26 },
    { 0x1064a53d, SUC_lexicon_tags_36 },
    { 0x7fe4a53e, SUC_lexicon_tags_25 },
    { 0xa924a53e, SUC_lexicon_tags_28 },
    { 0x7004a53f, SUC_lexicon_tags_17 },
    { 0x5c64a53f, SUC_lexicon_tags_1 },
    { 0x2ca4a53d, SUC_lexicon_tags_49 },
    { 0x6324a543, SUC_lexicon_tags_25 },
    { 0x8ca4a543, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2a4a546, SUC_lexicon_tags_12 },
    { 0x264a547, SUC_lexicon_tags_12 },
    { 0xc3a4a547, SUC_lexicon_tags_37 },
    { 0xbd04a549, SUC_lexicon_tags_37 },
    { 0x2ba4a54a, SUC_lexicon_tags_8 },
    { 0x91e4a547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7944a556, SUC_lexicon_tags_23 },
    { 0xf8a4a557, SUC_lexicon_tags_1 },
    { 0x4fa4a558, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa4a560, SUC_lexicon_tags_3 },
    { 0x7bc4a561, SUC_lexicon_tags_31 },
    { 0x9864a561, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf944a564, SUC_lexicon_tags_79 },
    { 0xabe4a564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c64a567, SUC_lexicon_tags_47 },
    { 0xf844a568, SUC_lexicon_tags_7 },
    { 0x79a4a569, SUC_lexicon_tags_10 },
    { 0x4ce4a56a, SUC_lexicon_tags_11 },
    { 0xfbe4a568, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7a4a56d, SUC_lexicon_tags_11 },
    { 0x5a84a56e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d04a572, SUC_lexicon_tags_21 },
    { 0x56e4a573, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa724a579, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd04a57c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x644a57e, SUC_lexicon_tags_25 },
    { 0xf304a57e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec4a583, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71a4a585, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4644a589, SUC_lexicon_tags_25 },
    { 0xee64a589, SUC_lexicon_tags_10 },
    { 0xde84a58b, SUC_lexicon_tags_63 },
    { 0x6e64a58b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e84a594, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c4a598, SUC_lexicon_tags_25 },
    { 0x1ba4a599, SUC_lexicon_tags_15 },
    { 0x6984a59a, SUC_lexicon_tags_22 },
    { 0x9884a599, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa04a59d, SUC_lexicon_tags_37 },
    { 0x4004a59e, SUC_lexicon_tags_15 },
    { 0x3e44a59e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e84a5a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd844a5a4, SUC_lexicon_tags_15 },
    { 0x7784a5a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea84a5a7, SUC_lexicon_tags_17 },
    { 0x3a64a5a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb424a5aa, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c4a5ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28e4a5b0, SUC_lexicon_tags_46 },
    { 0x2164a5b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a4a5b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2324a5ba, SUC_lexicon_tags_49 },
    { 0xc4e4a5bb, SUC_lexicon_tags_2 },
    { 0x8304a5bc, SUC_lexicon_tags_15 },
    { 0x5ae4a5ba, SUC_lexicon_tags_87 },
    { 0x4864a5ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb784a5c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb84a5c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27c4a5c4, SUC_lexicon_tags_4 },
    { 0xe124a5c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e64a5c9, SUC_lexicon_tags_60 },
    { 0x1484a5ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e4a5ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a4a5d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd44a5d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6984a5e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec64a5e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a4a5ea, SUC_lexicon_tags_6 },
    { 0xf2a4a5eb, SUC_lexicon_tags_7 },
    { 0x4384a5ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a84a5ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f84a5f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9264a601, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e4a605, SUC_lexicon_tags_15 },
    { 0x50e4a606, SUC_lexicon_tags_11 },
    { 0x3e04a607, SUC_lexicon_tags_15 },
    { 0x8aa4a606, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a60b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c84a60d, SUC_lexicon_tags_37 },
    { 0x2be4a60e, SUC_lexicon_tags_15 },
    { 0xc904a60e, SUC_lexicon_tags_25 },
    { 0xa064a60d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6824a615, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3844a61d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x87a4a61f, SUC_lexicon_tags_25 },
    { 0x6a84a620, SUC_lexicon_tags_8 },
    { 0x7a04a620, SUC_lexicon_tags_27 },
    { 0xee84a621, SUC_lexicon_tags_15 },
    { 0xdb64a623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a4a627, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe764a629, SUC_lexicon_tags_11 },
    { 0x3ca4a62a, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a04a631, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2ec4a633, SUC_lexicon_tags_15 },
    { 0x6064a634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fc4a636, SUC_lexicon_tags_7 },
    { 0xc8e4a636, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a04a63b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce4a63f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb004a643, SUC_lexicon_tags_37 },
    { 0x39c4a643, SUC_lexicon_tags_11 },
    { 0x69e4a645, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd264a64a, SUC_lexicon_tags_2 },
    { 0xbd04a64b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9924a650, SUC_lexicon_tags_17 },
    { 0x6964a650, SUC_lexicon_tags_5 },
    { 0x6544a652, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0e4a654, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e84a657, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe4a65a, SUC_lexicon_tags_17 },
    { 0xed64a65b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5504a65e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de4a661, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xab24a663, SUC_lexicon_tags_23 },
    { 0x61a4a664, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e84a66c, SUC_lexicon_tags_17 },
    { 0x17e4a66d, SUC_lexicon_tags_27 },
    { 0xd0e4a66c, SUC_lexicon_tags_30 },
    { 0x32e4a66d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc984a671, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324a679, SUC_lexicon_tags_17 },
    { 0x91c4a67a, SUC_lexicon_tags_17 },
    { 0x15e4a67b, SUC_lexicon_tags_11 },
    { 0xa5a4a67c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7544a67e, SUC_lexicon_tags_33 },
    { 0xd5c4a67f, SUC_lexicon_tags_49 },
    { 0xfd64a680, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c4a685, SUC_lexicon_tags_15 },
    { 0xab24a685, SUC_lexicon_tags_33 },
    { 0x2644a687, SUC_lexicon_tags_1 },
    { 0x28c4a688, SUC_lexicon_tags_7 },
    { 0xd444a685, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c4a692, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a64a696, SUC_lexicon_tags_11 },
    { 0x6884a696, SUC_lexicon_tags_60 },
    { 0x3fc4a696, SUC_lexicon_tags_32 },
    { 0xfac4a697, SUC_lexicon_tags_82 },
    { 0x4a44a697, SUC_lexicon_tags_37 },
    { 0x364a698, SUC_lexicon_tags_38 },
    { 0xa504a696, SUC_lexicon_tags_37 },
    { 0x9504a69d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9904a69f, SUC_lexicon_tags_1118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf444a6a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53c4a6a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c4a6a8, SUC_lexicon_tags_63 },
    { 0x6244a6a9, SUC_lexicon_tags_29 },
    { 0x3c64a6a9, SUC_lexicon_tags_27 },
    { 0xf5a4a6ab, SUC_lexicon_tags_26 },
    { 0x3a84a6ab, SUC_lexicon_tags_12 },
    { 0x1584a6ac, SUC_lexicon_tags_53 },
    { 0xe844a6ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c4a6b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c4a6b6, SUC_lexicon_tags_46 },
    { 0xd764a6b7, SUC_lexicon_tags_1 },
    { 0xa244a6b7, SUC_lexicon_tags_12 },
    { 0x7fe4a6b9, SUC_lexicon_tags_11 },
    { 0x8a84a6ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd824a6bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ce4a6be, SUC_lexicon_tags_85 },
    { 0x5cc4a6bf, SUC_lexicon_tags_17 },
    { 0x9fc4a6c0, SUC_lexicon_tags_600 },
    { 0x9fa4a6c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1884a6c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3104a6c9, SUC_lexicon_tags_10 },
    { 0xe544a6ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e4a6cf, SUC_lexicon_tags_12 },
    { 0xc444a6d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e4a6d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5fc4a6d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc4a6d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e84a6dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2144a6e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4604a6e6, SUC_lexicon_tags_6 },
    { 0xc304a6e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e4a6ec, SUC_lexicon_tags_38 },
    { 0x67c4a6ed, SUC_lexicon_tags_7 },
    { 0x4e64a6ed, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1024a6f1, SUC_lexicon_tags_3 },
    { 0x5d04a6f1, SUC_lexicon_tags_3 },
    { 0xaf24a6f1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc764a6f5, SUC_lexicon_tags_23 },
    { 0x1e64a6f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e84a6fb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2224a6fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4704a6ff, SUC_lexicon_tags_25 },
    { 0x49c4a6ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb84a70c, SUC_lexicon_tags_92 },
    { 0x23e4a70d, SUC_lexicon_tags_35 },
    { 0x7b44a70d, SUC_lexicon_tags_38 },
    { 0x7864a70c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea4a712, SUC_lexicon_tags_15 },
    { 0xca04a712, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc4a716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf04a719, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43e4a71b, SUC_lexicon_tags_77 },
    { 0xfce4a71c, SUC_lexicon_tags_23 },
    { 0x1304a71d, SUC_lexicon_tags_6 },
    { 0xfd24a71e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9624a724, SUC_lexicon_tags_12 },
    { 0x4864a724, SUC_lexicon_tags_60 },
    { 0x1044a726, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784a72a, SUC_lexicon_tags_8 },
    { 0x6764a72b, SUC_lexicon_tags_39 },
    { 0x2d44a72c, SUC_lexicon_tags_21 },
    { 0x80e4a72b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c4a732, SUC_lexicon_tags_37 },
    { 0x9c64a733, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea4a739, SUC_lexicon_tags_15 },
    { 0x7644a73a, SUC_lexicon_tags_8 },
    { 0xee4a73b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xefe4a73d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd924a742, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8624a745, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc4a74c, SUC_lexicon_tags_2 },
    { 0x3764a74d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c4a753, SUC_lexicon_tags_27 },
    { 0x6c64a754, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d24a757, SUC_lexicon_tags_11 },
    { 0x29e4a758, SUC_lexicon_tags_98 },
    { 0xe6a4a759, SUC_lexicon_tags_15 },
    { 0xc1e4a75a, SUC_lexicon_tags_36 },
    { 0xbc04a758, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea4a763, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e4a766, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe4a76b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x944a76e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb84a771, SUC_lexicon_tags_47 },
    { 0x2ca4a772, SUC_lexicon_tags_26 },
    { 0xf944a772, SUC_lexicon_tags_6 },
    { 0x4ee4a774, SUC_lexicon_tags_29 },
    { 0xba4a772, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x8524a777, SUC_lexicon_tags_179 },
    { 0xb344a778, SUC_lexicon_tags_2 },
    { 0x18e4a778, SUC_lexicon_tags_28 },
    { 0x7ce4a77a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7224a780, SUC_lexicon_tags_0 },
    { 0x6c24a781, SUC_lexicon_tags_37 },
    { 0x6b44a781, SUC_lexicon_tags_2 },
    { 0x57c4a783, SUC_lexicon_tags_35 },
    { 0x7f64a781, SUC_lexicon_tags_34 },
    { 0x4544a783, SUC_lexicon_tags_44 },
    { 0x8604a783, SUC_lexicon_tags_48 },
    { 0x4a24a787, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1084a78e, SUC_lexicon_tags_29 },
    { 0x3504a78f, SUC_lexicon_tags_0 },
    { 0x1964a790, SUC_lexicon_tags_20 },
    { 0x79a4a791, SUC_lexicon_tags_15 },
    { 0x6544a78f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344a796, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6204a79c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a4a79f, SUC_lexicon_tags_68 },
    { 0xcf84a79f, SUC_lexicon_tags_2 },
    { 0x5984a79f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f44a7a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5164a7a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce4a7ac, SUC_lexicon_tags_37 },
    { 0x5d84a7ad, SUC_lexicon_tags_11 },
    { 0x5b84a7ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6804a7b3, SUC_lexicon_tags_66 },
    { 0xc864a7b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa224a7b6, SUC_lexicon_tags_7 },
    { 0xe9e4a7b7, SUC_lexicon_tags_38 },
    { 0xd5e4a7b8, SUC_lexicon_tags_0 },
    { 0x3be4a7b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9664a7c1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4444a7c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee04a7c5, SUC_lexicon_tags_23 },
    { 0x99e4a7c6, SUC_lexicon_tags_87 },
    { 0x8a4a7c5, SUC_lexicon_tags_7 },
    { 0x4e04a7c8, SUC_lexicon_tags_116 },
    { 0xfce4a7c9, SUC_lexicon_tags_39 },
    { 0x2904a7c9, SUC_lexicon_tags_32 },
    { 0x3b04a7cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1384a7d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb484a7d2, SUC_lexicon_tags_15 },
    { 0xfa24a7d3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x90e4a7d5, SUC_lexicon_tags_49 },
    { 0x6c64a7d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4064a7d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd704a7dc, SUC_lexicon_tags_17 },
    { 0xc704a7dd, SUC_lexicon_tags_38 },
    { 0xa604a7de, SUC_lexicon_tags_97 },
    { 0x87a4a7dc, SUC_lexicon_tags_12 },
    { 0x3364a7e0, SUC_lexicon_tags_33 },
    { 0xc7a4a7e0, SUC_lexicon_tags_9 },
    { 0x1f64a7e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf664a7e4, SUC_lexicon_tags_79 },
    { 0x9c24a7e5, SUC_lexicon_tags_25 },
    { 0x2144a7e6, SUC_lexicon_tags_29 },
    { 0x91c4a7e7, SUC_lexicon_tags_11 },
    { 0x8d64a7e7, SUC_lexicon_tags_15 },
    { 0x6364a7e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea4a7ec, SUC_lexicon_tags_21 },
    { 0xe524a7ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd884a7f0, SUC_lexicon_tags_33 },
    { 0xd1a4a7f1, SUC_lexicon_tags_27 },
    { 0x2384a7f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc364a7f5, SUC_lexicon_tags_19 },
    { 0x7724a7f6, SUC_lexicon_tags_1 },
    { 0x3304a7f7, SUC_lexicon_tags_39 },
    { 0xba84a7f8, SUC_lexicon_tags_16 },
    { 0xe884a7f8, SUC_lexicon_tags_30 },
    { 0x1f84a7fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4fe4a7fc, SUC_lexicon_tags_41 },
    { 0x8a84a7fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ae4a7ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c4a805, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e4a809, SUC_lexicon_tags_70 },
    { 0x2864a80a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6b64a80c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c4a80f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e64a813, SUC_lexicon_tags_37 },
    { 0x5144a814, SUC_lexicon_tags_25 },
    { 0xb844a815, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f44a818, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3684a81a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x184a81d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb324a820, SUC_lexicon_tags_30 },
    { 0x644a821, SUC_lexicon_tags_25 },
    { 0xcac4a821, SUC_lexicon_tags_1683 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5004a82e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e4a832, SUC_lexicon_tags_15 },
    { 0xc384a833, SUC_lexicon_tags_1 },
    { 0xc684a834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4a837, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c4a83a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e4a83d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8864a83f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd0c4a841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a845, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a4a848, SUC_lexicon_tags_364 },
    { 0x83c4a849, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3904a84c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5c4a84e, SUC_lexicon_tags_37 },
    { 0x8dc4a84f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f04a852, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab64a857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5684a85a, SUC_lexicon_tags_3 },
    { 0xc084a85b, SUC_lexicon_tags_31 },
    { 0xa0a4a85c, SUC_lexicon_tags_23 },
    { 0xf204a85d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb9c4a85f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf84a861, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3264a863, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc4a866, SUC_lexicon_tags_17 },
    { 0x6764a867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1424a86a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c4a86f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf6e4a871, SUC_lexicon_tags_25 },
    { 0xba64a872, SUC_lexicon_tags_118 },
    { 0x5a24a873, SUC_lexicon_tags_23 },
    { 0xf184a872, SUC_lexicon_tags_12 },
    { 0xbc64a873, SUC_lexicon_tags_60 },
    { 0x504a876, SUC_lexicon_tags_7 },
    { 0x3644a874, SUC_lexicon_tags_23 },
    { 0xa7c4a878, SUC_lexicon_tags_3 },
    { 0x2784a877, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9724a87f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e84a883, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc664a888, SUC_lexicon_tags_28 },
    { 0x2704a889, SUC_lexicon_tags_82 },
    { 0xd1e4a88a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c4a88d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe764a890, SUC_lexicon_tags_45 },
    { 0x2ea4a891, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e4a894, SUC_lexicon_tags_37 },
    { 0x7244a894, SUC_lexicon_tags_12 },
    { 0xb3e4a896, SUC_lexicon_tags_23 },
    { 0x7204a896, SUC_lexicon_tags_19 },
    { 0x48a4a898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcac4a89a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c4a89f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a4a8a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab84a8a5, SUC_lexicon_tags_49 },
    { 0x66e4a8a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce44a8ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c4a8b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4044a8b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a44a8ba, SUC_lexicon_tags_60 },
    { 0x2044a8bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa464a8be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7364a8c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ae4a8c3, SUC_lexicon_tags_19 },
    { 0x9824a8c4, SUC_lexicon_tags_82 },
    { 0x1ea4a8c4, SUC_lexicon_tags_2 },
    { 0x25e4a8c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e4a8ca, SUC_lexicon_tags_15 },
    { 0xffc4a8cb, SUC_lexicon_tags_2 },
    { 0xea44a8cb, SUC_lexicon_tags_17 },
    { 0x5524a8cd, SUC_lexicon_tags_12 },
    { 0xe364a8cd, SUC_lexicon_tags_14 },
    { 0x664a8cc, SUC_lexicon_tags_12 },
    { 0xf004a8d0, SUC_lexicon_tags_37 },
    { 0x4dc4a8d1, SUC_lexicon_tags_21 },
    { 0xe324a8d2, SUC_lexicon_tags_43 },
    { 0x4264a8d2, SUC_lexicon_tags_12 },
    { 0x2a44a8d4, SUC_lexicon_tags_11 },
    { 0x6b04a8d1, SUC_lexicon_tags_12 },
    { 0x2204a8d6, SUC_lexicon_tags_25 },
    { 0x5124a8d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c4a8da, SUC_lexicon_tags_7 },
    { 0xad44a8db, SUC_lexicon_tags_7 },
    { 0xa824a8dc, SUC_lexicon_tags_6 },
    { 0xe784a8dd, SUC_lexicon_tags_12 },
    { 0x7fe4a8dd, SUC_lexicon_tags_16 },
    { 0x8f24a8da, SUC_lexicon_tags_12 },
    { 0x55c4a8e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a4a8e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc784a8ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa204a8f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf44a8f7, SUC_lexicon_tags_11 },
    { 0x3504a8f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac4a8fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe04a8fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e4a901, SUC_lexicon_tags_27 },
    { 0x3c64a901, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e4a906, SUC_lexicon_tags_32 },
    { 0xede4a907, SUC_lexicon_tags_25 },
    { 0x7ec4a906, SUC_lexicon_tags_23 },
    { 0xcd64a909, SUC_lexicon_tags_47 },
    { 0xe844a90a, SUC_lexicon_tags_7 },
    { 0x3364a90b, SUC_lexicon_tags_15 },
    { 0x5a24a90c, SUC_lexicon_tags_43 },
    { 0xe784a90d, SUC_lexicon_tags_45 },
    { 0xc2c4a90d, SUC_lexicon_tags_47 },
    { 0x1a84a90e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e4a913, SUC_lexicon_tags_17 },
    { 0xa1a4a914, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c44a919, SUC_lexicon_tags_2 },
    { 0xafe4a919, SUC_lexicon_tags_2 },
    { 0x26e4a919, SUC_lexicon_tags_4 },
    { 0xc004a91c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x184a91f, SUC_lexicon_tags_39 },
    { 0x444a91f, SUC_lexicon_tags_25 },
    { 0xf2c4a921, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1a4a923, SUC_lexicon_tags_47 },
    { 0xa5a4a923, SUC_lexicon_tags_12 },
    { 0x78c4a925, SUC_lexicon_tags_65 },
    { 0x64e4a926, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4be4a928, SUC_lexicon_tags_37 },
    { 0xe904a928, SUC_lexicon_tags_6 },
    { 0x5764a92a, SUC_lexicon_tags_2 },
    { 0xfc64a928, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c24a92f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8584a934, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea44a936, SUC_lexicon_tags_5 },
    { 0xfea4a936, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a24a93c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b84a940, SUC_lexicon_tags_11 },
    { 0x4524a941, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb524a943, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0xa004a946, SUC_lexicon_tags_7 },
    { 0x7a4a946, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa284a949, SUC_lexicon_tags_4 },
    { 0x8684a94a, SUC_lexicon_tags_1 },
    { 0x9f04a94b, SUC_lexicon_tags_25 },
    { 0x6a4a94c, SUC_lexicon_tags_11 },
    { 0xd464a94d, SUC_lexicon_tags_15 },
    { 0x5f24a94c, SUC_lexicon_tags_37 },
    { 0xa644a94f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f24a955, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9784a95a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c4a95d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8e4a95f, SUC_lexicon_tags_49 },
    { 0x5ac4a960, SUC_lexicon_tags_65 },
    { 0x3d04a95f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe4a966, SUC_lexicon_tags_12 },
    { 0xe4a967, SUC_lexicon_tags_7 },
    { 0x7144a968, SUC_lexicon_tags_14 },
    { 0x6ca4a969, SUC_lexicon_tags_12 },
    { 0x9024a969, SUC_lexicon_tags_40 },
    { 0x64c4a96a, SUC_lexicon_tags_2 },
    { 0xe8c4a969, SUC_lexicon_tags_15 },
    { 0x9944a96d, SUC_lexicon_tags_1 },
    { 0xc944a96c, SUC_lexicon_tags_65 },
    { 0xeae4a968, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xee24a971, SUC_lexicon_tags_11 },
    { 0x58a4a972, SUC_lexicon_tags_16 },
    { 0xfe4a973, SUC_lexicon_tags_23 },
    { 0xfe64a973, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c4a977, SUC_lexicon_tags_32 },
    { 0x6f04a977, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca4a97b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5264a982, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe4a984, SUC_lexicon_tags_58 },
    { 0x24e4a984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7984a98d, SUC_lexicon_tags_19 },
    { 0x9a24a98d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e4a991, SUC_lexicon_tags_9 },
    { 0x3ae4a991, SUC_lexicon_tags_28 },
    { 0xff24a993, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe4a9a2, SUC_lexicon_tags_32 },
    { 0xbc4a9a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3964a9a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e4a9ab, SUC_lexicon_tags_7 },
    { 0x36a4a9ac, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6104a9b0, SUC_lexicon_tags_11 },
    { 0x8344a9b1, SUC_lexicon_tags_17 },
    { 0x2684a9b2, SUC_lexicon_tags_43 },
    { 0x5084a9b3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9544a9b5, SUC_lexicon_tags_27 },
    { 0x8ae4a9b6, SUC_lexicon_tags_106 },
    { 0x6dc4a9b7, SUC_lexicon_tags_25 },
    { 0x1d84a9b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc4a9be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca4a9c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9684a9d1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5264a9d5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6cc4a9d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdd44a9d9, SUC_lexicon_tags_2 },
    { 0x9364a9d9, SUC_lexicon_tags_0 },
    { 0x4e64a9d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd344a9e3, SUC_lexicon_tags_60 },
    { 0xd064a9e4, SUC_lexicon_tags_6 },
    { 0xb404a9e5, SUC_lexicon_tags_151 },
    { 0x54a4a9e5, SUC_lexicon_tags_12 },
    { 0x88c4a9e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb044a9eb, SUC_lexicon_tags_11 },
    { 0x48e4a9ec, SUC_lexicon_tags_23 },
    { 0xd604a9ed, SUC_lexicon_tags_22 },
    { 0xf724a9eb, SUC_lexicon_tags_25 },
    { 0xae64a9ef, SUC_lexicon_tags_11 },
    { 0x5d64a9f0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1da4a9f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae4a9f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3324a9f9, SUC_lexicon_tags_2 },
    { 0x4744a9fa, SUC_lexicon_tags_12 },
    { 0x4464a9fa, SUC_lexicon_tags_25 },
    { 0x55a4a9fc, SUC_lexicon_tags_37 },
    { 0x6dc4a9fd, SUC_lexicon_tags_17 },
    { 0x9a44a9fe, SUC_lexicon_tags_7 },
    { 0x24c4a9fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb784aa01, SUC_lexicon_tags_23 },
    { 0xac64aa01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ca4aa04, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c4aa09, SUC_lexicon_tags_809 },
    { 0xe524aa0a, SUC_lexicon_tags_17 },
    { 0xc864aa0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc104aa0d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd204aa11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e04aa16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea4aa20, SUC_lexicon_tags_25 },
    { 0x9a44aa20, SUC_lexicon_tags_25 },
    { 0xff84aa20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2104aa25, SUC_lexicon_tags_68 },
    { 0xac84aa25, SUC_lexicon_tags_7 },
    { 0xd784aa27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe204aa29, SUC_lexicon_tags_7 },
    { 0xf044aa2a, SUC_lexicon_tags_19 },
    { 0x2bc4aa29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c04aa31, SUC_lexicon_tags_25 },
    { 0xff04aa32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd704aa38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be4aa3e, SUC_lexicon_tags_37 },
    { 0xb584aa3f, SUC_lexicon_tags_2 },
    { 0xab44aa3f, SUC_lexicon_tags_43 },
    { 0x7624aa41, SUC_lexicon_tags_26 },
    { 0xa504aa3e, SUC_lexicon_tags_69 },
    { 0xbee4aa3f, SUC_lexicon_tags_11 },
    { 0xc924aa44, SUC_lexicon_tags_63 },
    { 0xc4aa44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a44aa47, SUC_lexicon_tags_37 },
    { 0x8244aa48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xca24aa4a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8604aa52, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1c4aa54, SUC_lexicon_tags_4 },
    { 0x2a64aa55, SUC_lexicon_tags_19 },
    { 0xc504aa56, SUC_lexicon_tags_29 },
    { 0x1d84aa55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d84aa5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf244aa5f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3364aa64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe684aa6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2424aa71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeae4aa73, SUC_lexicon_tags_27 },
    { 0xfce4aa73, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4824aa78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8344aa7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf6e4aa7c, SUC_lexicon_tags_15 },
    { 0x1d04aa7d, SUC_lexicon_tags_19 },
    { 0xeac4aa7e, SUC_lexicon_tags_37 },
    { 0x3a64aa7f, SUC_lexicon_tags_60 },
    { 0x7f64aa7e, SUC_lexicon_tags_6 },
    { 0x13c4aa81, SUC_lexicon_tags_7 },
    { 0x9884aa81, SUC_lexicon_tags_37 },
    { 0x5844aa83, SUC_lexicon_tags_17 },
    { 0xa6a4aa84, SUC_lexicon_tags_12 },
    { 0x6c04aa85, SUC_lexicon_tags_12 },
    { 0xd244aa84, SUC_lexicon_tags_47 },
    { 0xcb24aa87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x184aa89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8624aa8f, SUC_lexicon_tags_37 },
    { 0xcc4aa90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9604aa93, SUC_lexicon_tags_63 },
    { 0x10a4aa94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e4aa99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c4aaa1, SUC_lexicon_tags_1 },
    { 0xeaa4aaa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6644aaa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e4aaa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c64aaaf, SUC_lexicon_tags_7 },
    { 0x6784aaaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c4aab3, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0x1144aab5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd124aabd, SUC_lexicon_tags_16 },
    { 0x7e04aabe, SUC_lexicon_tags_0 },
    { 0x63c4aabf, SUC_lexicon_tags_22 },
    { 0x8584aabf, SUC_lexicon_tags_11 },
    { 0x61e4aabd, SUC_lexicon_tags_37 },
    { 0x2424aac2, SUC_lexicon_tags_1 },
    { 0xaa64aac3, SUC_lexicon_tags_23 },
    { 0x7924aac4, SUC_lexicon_tags_6 },
    { 0x3b64aac4, SUC_lexicon_tags_37 },
    { 0x1304aac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e4aaca, SUC_lexicon_tags_7 },
    { 0x6f44aacb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2c4aacd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc4aad1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6b64aad3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b64aad6, SUC_lexicon_tags_25 },
    { 0x6f84aad6, SUC_lexicon_tags_27 },
    { 0xa7a4aad6, SUC_lexicon_tags_15 },
    { 0x2e4aad6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc544aadb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa4aadf, SUC_lexicon_tags_15 },
    { 0x444aadf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a4aae3, SUC_lexicon_tags_65 },
    { 0x2564aae3, SUC_lexicon_tags_65 },
    { 0xa1a4aae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264aaea, SUC_lexicon_tags_8 },
    { 0x2a04aaea, SUC_lexicon_tags_25 },
    { 0x35c4aaea, SUC_lexicon_tags_2 },
    { 0xc064aaed, SUC_lexicon_tags_63 },
    { 0xdea4aaee, SUC_lexicon_tags_7 },
    { 0xe844aaea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2984aaf3, SUC_lexicon_tags_182 },
    { 0xa9c4aaf4, SUC_lexicon_tags_37 },
    { 0x21e4aaf5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e04aaf8, SUC_lexicon_tags_104 },
    { 0x9564aaf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc4aafc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc84aaff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e4ab02, SUC_lexicon_tags_25 },
    { 0xd2c4ab03, SUC_lexicon_tags_17 },
    { 0x1184ab04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf984ab07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e4ab0c, SUC_lexicon_tags_20 },
    { 0xe704ab0c, SUC_lexicon_tags_23 },
    { 0xa744ab0e, SUC_lexicon_tags_4 },
    { 0xed84ab0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7424ab12, SUC_lexicon_tags_11 },
    { 0xd404ab13, SUC_lexicon_tags_43 },
    { 0xeee4ab14, SUC_lexicon_tags_25 },
    { 0xff24ab13, SUC_lexicon_tags_2 },
    { 0x4aa4ab16, SUC_lexicon_tags_567 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f64ab20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04ab25, SUC_lexicon_tags_27 },
    { 0x8144ab25, SUC_lexicon_tags_57 },
    { 0x584ab27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa184ab2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc84ab2e, SUC_lexicon_tags_26 },
    { 0xdf24ab2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc4ab32, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd04ab36, SUC_lexicon_tags_23 },
    { 0xeee4ab37, SUC_lexicon_tags_4 },
    { 0x9564ab37, SUC_lexicon_tags_49 },
    { 0x31a4ab39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b24ab40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca04ab42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x86a4ab44, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb04ab47, SUC_lexicon_tags_25 },
    { 0x7a44ab48, SUC_lexicon_tags_14 },
    { 0x2944ab49, SUC_lexicon_tags_77 },
    { 0x8b84ab4a, SUC_lexicon_tags_25 },
    { 0x5f84ab4b, SUC_lexicon_tags_17 },
    { 0xe4e4ab4b, SUC_lexicon_tags_15 },
    { 0xe4a4ab49, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc4ab50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc4ab53, SUC_lexicon_tags_7 },
    { 0x2084ab53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6964ab57, SUC_lexicon_tags_29 },
    { 0xffc4ab58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e4ab5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e4ab62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7da4ab64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc4ab67, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xa244ab6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf504ab6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e4ab70, SUC_lexicon_tags_61 },
    { 0x9464ab71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf964ab75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ec4ab77, SUC_lexicon_tags_19 },
    { 0x3884ab78, SUC_lexicon_tags_12 },
    { 0xa624ab77, SUC_lexicon_tags_7 },
    { 0xbe4ab7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xde24ab7d, SUC_lexicon_tags_7 },
    { 0x1b04ab7d, SUC_lexicon_tags_19 },
    { 0x4524ab7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee84ab81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc4ab87, SUC_lexicon_tags_37 },
    { 0x6484ab88, SUC_lexicon_tags_27 },
    { 0x4e84ab88, SUC_lexicon_tags_60 },
    { 0xbae4ab87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f24ab8e, SUC_lexicon_tags_12 },
    { 0xb884ab8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c4ab94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc4ab96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e4ab98, SUC_lexicon_tags_1 },
    { 0x2744ab99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a4ab9f, SUC_lexicon_tags_17 },
    { 0x3d84ab9f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a84aba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd204aba5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb04aba7, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c4abb7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8284abba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5264abbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f24abc0, SUC_lexicon_tags_27 },
    { 0x6264abc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a4abc6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba4abc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64abce, SUC_lexicon_tags_33 },
    { 0x1664abcf, SUC_lexicon_tags_25 },
    { 0x3b44abd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e4abd3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xe084abd6, SUC_lexicon_tags_61 },
    { 0x904abd7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a4abdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xabc4abde, SUC_lexicon_tags_37 },
    { 0xf804abdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba4abe3, SUC_lexicon_tags_7 },
    { 0xe1c4abe3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x29a4abe6, SUC_lexicon_tags_25 },
    { 0x8684abe7, SUC_lexicon_tags_2 },
    { 0xa1c4abe8, SUC_lexicon_tags_15 },
    { 0x8504abe9, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d44abf0, SUC_lexicon_tags_11 },
    { 0xd504abf0, SUC_lexicon_tags_29 },
    { 0xc604abf2, SUC_lexicon_tags_60 },
    { 0x57c4abf3, SUC_lexicon_tags_9 },
    { 0xf844abf0, SUC_lexicon_tags_2 },
    { 0x6384abf4, SUC_lexicon_tags_14 },
    { 0xdb04abf6, SUC_lexicon_tags_45 },
    { 0x2424abf6, SUC_lexicon_tags_15 },
    { 0x46a4abf8, SUC_lexicon_tags_27 },
    { 0xbc24abf9, SUC_lexicon_tags_17 },
    { 0xa2c4abfa, SUC_lexicon_tags_12 },
    { 0xa6a4abf9, SUC_lexicon_tags_30 },
    { 0x2cc4abfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe04abff, SUC_lexicon_tags_28 },
    { 0x69e4abff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c4ac03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a04ac05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3384ac07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd024ac0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3704ac1a, SUC_lexicon_tags_17 },
    { 0xcea4ac1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x89a4ac1d, SUC_lexicon_tags_19 },
    { 0x3784ac1e, SUC_lexicon_tags_7 },
    { 0xc344ac1f, SUC_lexicon_tags_11 },
    { 0xed24ac1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa4ac24, SUC_lexicon_tags_17 },
    { 0xea84ac25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf64ac27, SUC_lexicon_tags_14 },
    { 0x92a4ac27, SUC_lexicon_tags_14 },
    { 0x2ce4ac29, SUC_lexicon_tags_19 },
    { 0x5384ac2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6524ac30, SUC_lexicon_tags_61 },
    { 0x4c64ac30, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4de4ac33, SUC_lexicon_tags_46 },
    { 0x78a4ac34, SUC_lexicon_tags_31 },
    { 0x4a4ac34, SUC_lexicon_tags_7 },
    { 0x2ba4ac35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f44ac39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2c4ac3b, SUC_lexicon_tags_14 },
    { 0x3d44ac3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7084ac40, SUC_lexicon_tags_17 },
    { 0x3664ac41, SUC_lexicon_tags_6 },
    { 0x9b24ac42, SUC_lexicon_tags_28 },
    { 0x2ca4ac43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04ac47, SUC_lexicon_tags_21 },
    { 0xbd04ac48, SUC_lexicon_tags_29 },
    { 0xfc04ac49, SUC_lexicon_tags_46 },
    { 0xfe44ac49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x52a4ac4c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e4ac4f, SUC_lexicon_tags_17 },
    { 0x4924ac50, SUC_lexicon_tags_45 },
    { 0x5b84ac51, SUC_lexicon_tags_12 },
    { 0x5d44ac52, SUC_lexicon_tags_12 },
    { 0x14a4ac53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79a4ac55, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc04ac58, SUC_lexicon_tags_35 },
    { 0x6f84ac58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e4ac5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x58e4ac5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3a4ac5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6bc4ac61, SUC_lexicon_tags_306 },
    { 0x3864ac62, SUC_lexicon_tags_31 },
    { 0xc4c4ac61, SUC_lexicon_tags_27 },
    { 0x5e84ac63, SUC_lexicon_tags_11 },
    { 0xdc4ac63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd004ac6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4384ac6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c4ac72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7184ac74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4184ac77, SUC_lexicon_tags_37 },
    { 0xa7a4ac78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9884ac7c, SUC_lexicon_tags_16 },
    { 0xc024ac7c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x144ac80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b44ac88, SUC_lexicon_tags_7 },
    { 0x7fa4ac88, SUC_lexicon_tags_38 },
    { 0x8ce4ac89, SUC_lexicon_tags_3 },
    { 0x8f44ac8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4784ac94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea64ac96, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444aca1, SUC_lexicon_tags_27 },
    { 0x1b24aca2, SUC_lexicon_tags_2 },
    { 0x9ca4aca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee84aca7, SUC_lexicon_tags_12 },
    { 0x7ba4aca8, SUC_lexicon_tags_12 },
    { 0x2d24aca8, SUC_lexicon_tags_2 },
    { 0x7b24aca8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4324acb0, SUC_lexicon_tags_25 },
    { 0xc1a4acb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf24acb3, SUC_lexicon_tags_1 },
    { 0xd664acb4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7884acbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2484acbe, SUC_lexicon_tags_27 },
    { 0x1bc4acbf, SUC_lexicon_tags_15 },
    { 0xce44acc0, SUC_lexicon_tags_11 },
    { 0xe704acbf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4acc7, SUC_lexicon_tags_17 },
    { 0xa644acc8, SUC_lexicon_tags_11 },
    { 0xd624acc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e4accd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d64acd0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa4acd7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad04acdb, SUC_lexicon_tags_11 },
    { 0x2a04acdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7944acdf, SUC_lexicon_tags_63 },
    { 0xc8e4ace0, SUC_lexicon_tags_11 },
    { 0x684ace0, SUC_lexicon_tags_38 },
    { 0x4564ace2, SUC_lexicon_tags_11 },
    { 0x13a4acdf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa44acec, SUC_lexicon_tags_28 },
    { 0xe904aced, SUC_lexicon_tags_46 },
    { 0x5ae4acee, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c4acf1, SUC_lexicon_tags_64 },
    { 0x3a4acf1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee4acf6, SUC_lexicon_tags_15 },
    { 0xdb64acf7, SUC_lexicon_tags_17 },
    { 0x9004acf6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd304acfa, SUC_lexicon_tags_37 },
    { 0x4444acfb, SUC_lexicon_tags_15 },
    { 0xc704acfc, SUC_lexicon_tags_11 },
    { 0x5004acfd, SUC_lexicon_tags_12 },
    { 0xe824acfe, SUC_lexicon_tags_25 },
    { 0xf944acff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa4ad03, SUC_lexicon_tags_4 },
    { 0x4b24ad04, SUC_lexicon_tags_17 },
    { 0xb324ad03, SUC_lexicon_tags_31 },
    { 0xefe4ad06, SUC_lexicon_tags_11 },
    { 0x71c4ad04, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3984ad0d, SUC_lexicon_tags_6 },
    { 0xb424ad0d, SUC_lexicon_tags_11 },
    { 0x11a4ad0f, SUC_lexicon_tags_19 },
    { 0xd624ad10, SUC_lexicon_tags_999 },
    { 0, NULL },
    { 0x1044ad12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea4ad15, SUC_lexicon_tags_14 },
    { 0xbb44ad15, SUC_lexicon_tags_2 },
    { 0x3a64ad17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe704ad1c, SUC_lexicon_tags_37 },
    { 0x444ad1d, SUC_lexicon_tags_49 },
    { 0x3484ad1d, SUC_lexicon_tags_46 },
    { 0x4dc4ad1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf64ad21, SUC_lexicon_tags_26 },
    { 0xb924ad22, SUC_lexicon_tags_26 },
    { 0xb3c4ad23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce4ad26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec4ad2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6464ad33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1304ad36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1084ad38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4ad3d, SUC_lexicon_tags_11 },
    { 0xdce4ad3e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba4ad42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe844ad45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f64ad48, SUC_lexicon_tags_17 },
    { 0xfe24ad48, SUC_lexicon_tags_37 },
    { 0xb804ad4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd04ad4c, SUC_lexicon_tags_25 },
    { 0x2f84ad4d, SUC_lexicon_tags_15 },
    { 0xac24ad4e, SUC_lexicon_tags_9 },
    { 0xebc4ad4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea84ad53, SUC_lexicon_tags_1 },
    { 0x75c4ad53, SUC_lexicon_tags_17 },
    { 0x2964ad53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e4ad5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf884ad5e, SUC_lexicon_tags_61 },
    { 0xe44ad5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ea4ad61, SUC_lexicon_tags_8 },
    { 0x3924ad62, SUC_lexicon_tags_2 },
    { 0xe764ad62, SUC_lexicon_tags_29 },
    { 0x4dc4ad64, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x5384ad67, SUC_lexicon_tags_27 },
    { 0x59c4ad68, SUC_lexicon_tags_11 },
    { 0x77c4ad69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xede4ad6c, SUC_lexicon_tags_65 },
    { 0x25c4ad6d, SUC_lexicon_tags_12 },
    { 0x4e4ad6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee4ad72, SUC_lexicon_tags_8 },
    { 0x52a4ad72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ec4ad75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bc4ad77, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a44ad7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb64ad7f, SUC_lexicon_tags_26 },
    { 0xdc84ad80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4004ad88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c84ad8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc4ad94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5644ad99, SUC_lexicon_tags_26 },
    { 0xd184ad9a, SUC_lexicon_tags_30 },
    { 0xbf44ad9b, SUC_lexicon_tags_47 },
    { 0x84ad9a, SUC_lexicon_tags_14 },
    { 0x4644ad9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c4ada4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1244ada9, SUC_lexicon_tags_6 },
    { 0xa2e4adaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee64adae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e4adb4, SUC_lexicon_tags_21 },
    { 0xdd24adb4, SUC_lexicon_tags_2 },
    { 0xb6e4adb4, SUC_lexicon_tags_77 },
    { 0x2224adb5, SUC_lexicon_tags_14 },
    { 0x3664adb8, SUC_lexicon_tags_17 },
    { 0x4ac4adb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe4adc0, SUC_lexicon_tags_49 },
    { 0x8124adc1, SUC_lexicon_tags_4 },
    { 0xae04adc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf464adc4, SUC_lexicon_tags_15 },
    { 0x72e4adc4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a4adca, SUC_lexicon_tags_17 },
    { 0xe344adcb, SUC_lexicon_tags_7 },
    { 0x4ee4adcc, SUC_lexicon_tags_30 },
    { 0xb724adcd, SUC_lexicon_tags_12 },
    { 0x18a4adce, SUC_lexicon_tags_47 },
    { 0x5e84adcf, SUC_lexicon_tags_1 },
    { 0xaa4add0, SUC_lexicon_tags_22 },
    { 0x9524add1, SUC_lexicon_tags_17 },
    { 0x3ce4adce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd644add7, SUC_lexicon_tags_15 },
    { 0xce4add8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4addb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3e4addd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c4ade3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5164ade5, SUC_lexicon_tags_2 },
    { 0x90c4ade6, SUC_lexicon_tags_37 },
    { 0xb1e4ade7, SUC_lexicon_tags_48 },
    { 0xfe24ade8, SUC_lexicon_tags_37 },
    { 0xae24ade8, SUC_lexicon_tags_178 },
    { 0x9ca4ade9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1cc4adec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a4adf1, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf124adf8, SUC_lexicon_tags_23 },
    { 0x1404adf9, SUC_lexicon_tags_23 },
    { 0xefe4adf9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3144adfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c4ae01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xabe4ae03, SUC_lexicon_tags_12 },
    { 0x6fc4ae04, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x75e4ae06, SUC_lexicon_tags_11 },
    { 0xb344ae07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e44ae09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc184ae10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7764ae13, SUC_lexicon_tags_15 },
    { 0xc464ae14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c44ae1b, SUC_lexicon_tags_11 },
    { 0xd944ae1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4444ae1e, SUC_lexicon_tags_57 },
    { 0xa024ae1e, SUC_lexicon_tags_0 },
    { 0xdc44ae20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc64ae26, SUC_lexicon_tags_26 },
    { 0x43c4ae27, SUC_lexicon_tags_476 },
    { 0x3c84ae28, SUC_lexicon_tags_25 },
    { 0xfee4ae29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a4ae2f, SUC_lexicon_tags_25 },
    { 0xdfe4ae2f, SUC_lexicon_tags_16 },
    { 0x89c4ae31, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x12e4ae33, SUC_lexicon_tags_11 },
    { 0x1264ae33, SUC_lexicon_tags_7 },
    { 0x9de4ae35, SUC_lexicon_tags_5 },
    { 0x3b04ae36, SUC_lexicon_tags_51 },
    { 0x8ae4ae37, SUC_lexicon_tags_28 },
    { 0xd6e4ae37, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0xb064ae3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee4ae3f, SUC_lexicon_tags_31 },
    { 0xf5c4ae3f, SUC_lexicon_tags_25 },
    { 0xe764ae41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf944ae45, SUC_lexicon_tags_137 },
    { 0x64c4ae46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93e4ae48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6804ae4a, SUC_lexicon_tags_63 },
    { 0x5744ae4a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c4ae52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x404ae54, SUC_lexicon_tags_7 },
    { 0xf0c4ae55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7804ae59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c4ae5b, SUC_lexicon_tags_23 },
    { 0xe984ae5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb24ae5e, SUC_lexicon_tags_7 },
    { 0x58e4ae5e, SUC_lexicon_tags_10 },
    { 0x67c4ae60, SUC_lexicon_tags_0 },
    { 0xd684ae61, SUC_lexicon_tags_33 },
    { 0xc684ae61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd424ae6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f64ae70, SUC_lexicon_tags_25 },
    { 0x5404ae71, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca04ae79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a4ae7c, SUC_lexicon_tags_2 },
    { 0xa84ae7d, SUC_lexicon_tags_15 },
    { 0x59c4ae7d, SUC_lexicon_tags_54 },
    { 0x4424ae7c, SUC_lexicon_tags_7 },
    { 0x6804ae7e, SUC_lexicon_tags_2 },
    { 0xe2c4ae80, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba4ae88, SUC_lexicon_tags_0 },
    { 0xc944ae88, SUC_lexicon_tags_45 },
    { 0xc824ae8a, SUC_lexicon_tags_38 },
    { 0x1944ae8b, SUC_lexicon_tags_37 },
    { 0xcd64ae8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c4ae8f, SUC_lexicon_tags_61 },
    { 0x52e4ae90, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784ae98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e4ae9f, SUC_lexicon_tags_12 },
    { 0x38e4ae9f, SUC_lexicon_tags_34 },
    { 0x8164aea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5344aea4, SUC_lexicon_tags_17 },
    { 0x6184aea4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1284aea7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea4aead, SUC_lexicon_tags_37 },
    { 0xcd44aead, SUC_lexicon_tags_60 },
    { 0x1f24aeaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb344aeb2, SUC_lexicon_tags_11 },
    { 0x5ee4aeb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x944aeb5, SUC_lexicon_tags_31 },
    { 0x3004aeb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee64aeb9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7ce4aebb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6604aec3, SUC_lexicon_tags_27 },
    { 0x9d84aec4, SUC_lexicon_tags_37 },
    { 0xbbc4aec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8564aec7, SUC_lexicon_tags_30 },
    { 0xf464aec8, SUC_lexicon_tags_12 },
    { 0x4a4aec9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc04aecc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e4aed0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2864aed2, SUC_lexicon_tags_216 },
    { 0xd064aed3, SUC_lexicon_tags_46 },
    { 0x18c4aed4, SUC_lexicon_tags_5 },
    { 0x2bc4aed3, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e4aed8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc04aedd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e04aedf, SUC_lexicon_tags_31 },
    { 0x4aa4aedf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e4aee6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc744aeea, SUC_lexicon_tags_0 },
    { 0xd1c4aeea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b24aeed, SUC_lexicon_tags_7 },
    { 0x9e44aeee, SUC_lexicon_tags_7 },
    { 0xe644aeed, SUC_lexicon_tags_51 },
    { 0x5884aef0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x12a4aef2, SUC_lexicon_tags_7 },
    { 0x7504aef2, SUC_lexicon_tags_0 },
    { 0xd224aef4, SUC_lexicon_tags_37 },
    { 0xc8c4aef5, SUC_lexicon_tags_3 },
    { 0x8084aef6, SUC_lexicon_tags_1 },
    { 0x1484aef6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c4aefa, SUC_lexicon_tags_103 },
    { 0x8904aefb, SUC_lexicon_tags_11 },
    { 0x9e04aefb, SUC_lexicon_tags_47 },
    { 0x25e4aefd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf844af02, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca4af05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab64af07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a4af0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6924af0c, SUC_lexicon_tags_7 },
    { 0x4b24af0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa144af0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e24af11, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca4af1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c4af20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4af24, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcac4af26, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8344af2c, SUC_lexicon_tags_15 },
    { 0xb8e4af2d, SUC_lexicon_tags_0 },
    { 0x6de4af2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324af34, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a4af38, SUC_lexicon_tags_27 },
    { 0x3f84af39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa924af3b, SUC_lexicon_tags_7 },
    { 0x60c4af3c, SUC_lexicon_tags_82 },
    { 0x5cc4af3d, SUC_lexicon_tags_37 },
    { 0x1bc4af3e, SUC_lexicon_tags_17 },
    { 0x2704af3f, SUC_lexicon_tags_1 },
    { 0x4e64af40, SUC_lexicon_tags_2 },
    { 0x2864af41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84af44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1464af46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdc4af48, SUC_lexicon_tags_0 },
    { 0xbee4af49, SUC_lexicon_tags_32 },
    { 0x26a4af4a, SUC_lexicon_tags_2 },
    { 0xbf64af4b, SUC_lexicon_tags_18 },
    { 0x9be4af4c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5504af4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c24af55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc004af5d, SUC_lexicon_tags_19 },
    { 0x3464af5e, SUC_lexicon_tags_27 },
    { 0x2ae4af5e, SUC_lexicon_tags_28 },
    { 0x8d44af5f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x20e4af62, SUC_lexicon_tags_11 },
    { 0x4ba4af63, SUC_lexicon_tags_0 },
    { 0x5ae4af64, SUC_lexicon_tags_48 },
    { 0xb604af64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604af69, SUC_lexicon_tags_17 },
    { 0x724af6a, SUC_lexicon_tags_49 },
    { 0xe464af69, SUC_lexicon_tags_37 },
    { 0x7d24af6a, SUC_lexicon_tags_32 },
    { 0x4c24af6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e24af73, SUC_lexicon_tags_60 },
    { 0x8004af74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a4af78, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c4af7b, SUC_lexicon_tags_11 },
    { 0x9ac4af7c, SUC_lexicon_tags_17 },
    { 0x5224af7d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a24af85, SUC_lexicon_tags_13 },
    { 0xc344af86, SUC_lexicon_tags_2 },
    { 0x3a04af85, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4444af92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff24af96, SUC_lexicon_tags_17 },
    { 0xd764af97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe724af9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xedc4af9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3964afa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2424afa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4c4afa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b24afa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd964afaa, SUC_lexicon_tags_25 },
    { 0x83e4afab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4624afad, SUC_lexicon_tags_178 },
    { 0xc4e4afae, SUC_lexicon_tags_23 },
    { 0xabe4afae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664afb3, SUC_lexicon_tags_14 },
    { 0xd244afb4, SUC_lexicon_tags_0 },
    { 0xa764afb5, SUC_lexicon_tags_15 },
    { 0x4ba4afb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe504afb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c4afbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f64afc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b24afc2, SUC_lexicon_tags_63 },
    { 0xc764afc3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7664afc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8fe4afc8, SUC_lexicon_tags_32 },
    { 0xcb24afc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64afce, SUC_lexicon_tags_15 },
    { 0x57e4afcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7744afd2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xac64afd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf364afd9, SUC_lexicon_tags_11 },
    { 0xf884afda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7144afe1, SUC_lexicon_tags_15 },
    { 0xfe84afe2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa3e4afe4, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x8584afe6, SUC_lexicon_tags_37 },
    { 0x34a4afe7, SUC_lexicon_tags_12 },
    { 0x9004afe6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea84afeb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb804afee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a4aff3, SUC_lexicon_tags_72 },
    { 0x45c4aff3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf644aff8, SUC_lexicon_tags_6 },
    { 0x7bc4aff9, SUC_lexicon_tags_25 },
    { 0xdf84affa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e4affc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8104afff, SUC_lexicon_tags_28 },
    { 0xf844afff, SUC_lexicon_tags_12 },
    { 0x8444b001, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe9a4b003, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58e4b005, SUC_lexicon_tags_2 },
    { 0x33e4b006, SUC_lexicon_tags_30 },
    { 0x7a84b005, SUC_lexicon_tags_1 },
    { 0x4ea4b006, SUC_lexicon_tags_28 },
    { 0x69c4b009, SUC_lexicon_tags_12 },
    { 0xab04b006, SUC_lexicon_tags_17 },
    { 0x5da4b007, SUC_lexicon_tags_0 },
    { 0x1724b00a, SUC_lexicon_tags_8 },
    { 0xe384b007, SUC_lexicon_tags_37 },
    { 0x7324b00a, SUC_lexicon_tags_15 },
    { 0x7e44b009, SUC_lexicon_tags_47 },
    { 0x224b010, SUC_lexicon_tags_17 },
    { 0x31a4b010, SUC_lexicon_tags_10 },
    { 0xae44b010, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b24b019, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb64b01b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7164b020, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2084b022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa244b024, SUC_lexicon_tags_15 },
    { 0x9c84b025, SUC_lexicon_tags_2 },
    { 0x9de4b024, SUC_lexicon_tags_55 },
    { 0x74e4b027, SUC_lexicon_tags_37 },
    { 0xd104b028, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e4b02a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd184b02c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a4b034, SUC_lexicon_tags_23 },
    { 0x1984b035, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x1064b037, SUC_lexicon_tags_0 },
    { 0x1ca4b037, SUC_lexicon_tags_11 },
    { 0x4324b039, SUC_lexicon_tags_12 },
    { 0x3e44b039, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e84b03f, SUC_lexicon_tags_45 },
    { 0xbce4b03f, SUC_lexicon_tags_29 },
    { 0x9404b041, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x5c04b043, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec4b04a, SUC_lexicon_tags_49 },
    { 0x54a4b04b, SUC_lexicon_tags_108 },
    { 0x3824b04c, SUC_lexicon_tags_37 },
    { 0x6a84b04d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5964b050, SUC_lexicon_tags_82 },
    { 0x3ba4b051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfe4b053, SUC_lexicon_tags_56 },
    { 0x81c4b053, SUC_lexicon_tags_11 },
    { 0xb6c4b054, SUC_lexicon_tags_43 },
    { 0x56a4b056, SUC_lexicon_tags_37 },
    { 0x63a4b056, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c4b05e, SUC_lexicon_tags_62 },
    { 0xd8c4b05f, SUC_lexicon_tags_11 },
    { 0x80e4b060, SUC_lexicon_tags_47 },
    { 0x4464b061, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf24b064, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd64b066, SUC_lexicon_tags_9 },
    { 0xf664b067, SUC_lexicon_tags_17 },
    { 0x7da4b068, SUC_lexicon_tags_7 },
    { 0x9ca4b069, SUC_lexicon_tags_55 },
    { 0xa324b069, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa124b071, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf364b073, SUC_lexicon_tags_14 },
    { 0xf684b073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e4b078, SUC_lexicon_tags_37 },
    { 0xc1e4b079, SUC_lexicon_tags_17 },
    { 0xada4b07a, SUC_lexicon_tags_21 },
    { 0xed04b07b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b84b082, SUC_lexicon_tags_13 },
    { 0x1b84b083, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c4b088, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9064b08c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e4b08f, SUC_lexicon_tags_2 },
    { 0xbac4b090, SUC_lexicon_tags_11 },
    { 0x4664b091, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5804b094, SUC_lexicon_tags_15 },
    { 0x2f04b094, SUC_lexicon_tags_8 },
    { 0xf024b096, SUC_lexicon_tags_29 },
    { 0x7e4b097, SUC_lexicon_tags_87 },
    { 0x8804b098, SUC_lexicon_tags_48 },
    { 0x8dc4b099, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1004b09b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xabc4b09d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdbc4b09f, SUC_lexicon_tags_9 },
    { 0xee84b0a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4b0a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdde4b0a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xed84b0a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf924b0ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d44b0af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4584b0b2, SUC_lexicon_tags_6 },
    { 0x1624b0b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x704b0b5, SUC_lexicon_tags_25 },
    { 0xf244b0b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb944b0bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x484b0bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3fc4b0c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe784b0c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3344b0c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1984b0cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64b0d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc4b0d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e84b0db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb564b0dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc4b0e3, SUC_lexicon_tags_17 },
    { 0xde4b0e4, SUC_lexicon_tags_11 },
    { 0x7aa4b0e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xda04b0e7, SUC_lexicon_tags_12 },
    { 0x4ae4b0e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd404b0ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c64b0f0, SUC_lexicon_tags_43 },
    { 0x744b0f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac4b0f4, SUC_lexicon_tags_79 },
    { 0xfac4b0f5, SUC_lexicon_tags_33 },
    { 0x2f64b0f6, SUC_lexicon_tags_47 },
    { 0xb364b0f7, SUC_lexicon_tags_28 },
    { 0x15e4b0f4, SUC_lexicon_tags_63 },
    { 0xa0a4b0f9, SUC_lexicon_tags_37 },
    { 0xac4b0f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3404b0fe, SUC_lexicon_tags_25 },
    { 0x2964b0ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe284b106, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8304b108, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e4b10b, SUC_lexicon_tags_17 },
    { 0x3824b10c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da4b111, SUC_lexicon_tags_16 },
    { 0x3d64b112, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8084b115, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a4b11a, SUC_lexicon_tags_237 },
    { 0xecc4b11a, SUC_lexicon_tags_27 },
    { 0x8564b11a, SUC_lexicon_tags_27 },
    { 0x95c4b11d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b24b11f, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x1404b122, SUC_lexicon_tags_17 },
    { 0x5dc4b122, SUC_lexicon_tags_33 },
    { 0x384b123, SUC_lexicon_tags_31 },
    { 0x5144b125, SUC_lexicon_tags_1 },
    { 0x9964b126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca4b128, SUC_lexicon_tags_39 },
    { 0xf664b129, SUC_lexicon_tags_37 },
    { 0x30c4b12a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6564b12e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e4b133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2644b135, SUC_lexicon_tags_38 },
    { 0xce64b136, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e4b138, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a4b13c, SUC_lexicon_tags_12 },
    { 0x1f84b13d, SUC_lexicon_tags_48 },
    { 0x1584b13e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a4b141, SUC_lexicon_tags_17 },
    { 0xba04b142, SUC_lexicon_tags_17 },
    { 0x6d24b143, SUC_lexicon_tags_11 },
    { 0x2404b144, SUC_lexicon_tags_47 },
    { 0x57c4b142, SUC_lexicon_tags_60 },
    { 0x8de4b146, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83e4b148, SUC_lexicon_tags_37 },
    { 0xa8c4b148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4304b14c, SUC_lexicon_tags_0 },
    { 0x4ea4b14d, SUC_lexicon_tags_27 },
    { 0xb344b14e, SUC_lexicon_tags_45 },
    { 0x1064b14f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe44b153, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac4b157, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a4b15e, SUC_lexicon_tags_87 },
    { 0xb7a4b15e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce4b166, SUC_lexicon_tags_37 },
    { 0xd844b167, SUC_lexicon_tags_17 },
    { 0xf004b168, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac4b16c, SUC_lexicon_tags_28 },
    { 0x7204b16d, SUC_lexicon_tags_1 },
    { 0xc3a4b16e, SUC_lexicon_tags_1 },
    { 0xad24b16e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6544b172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9084b175, SUC_lexicon_tags_39 },
    { 0x75e4b176, SUC_lexicon_tags_7 },
    { 0xd024b176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fe4b179, SUC_lexicon_tags_1 },
    { 0xef44b17a, SUC_lexicon_tags_6 },
    { 0x53a4b17b, SUC_lexicon_tags_26 },
    { 0x7e84b17b, SUC_lexicon_tags_4 },
    { 0x41a4b17d, SUC_lexicon_tags_7 },
    { 0x2924b17e, SUC_lexicon_tags_15 },
    { 0x1364b17f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbba4b181, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8c64b183, SUC_lexicon_tags_16 },
    { 0x7d64b184, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51c4b186, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb464b188, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8cc4b18a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdda4b18c, SUC_lexicon_tags_12 },
    { 0xfbc4b18d, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xb6a4b18f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcce4b191, SUC_lexicon_tags_7 },
    { 0x75c4b192, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7a4b194, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdae4b196, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1dc4b198, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc4b19c, SUC_lexicon_tags_12 },
    { 0x40e4b19d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa4b1a0, SUC_lexicon_tags_23 },
    { 0x29c4b1a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6164b1a6, SUC_lexicon_tags_1 },
    { 0x9a84b1a7, SUC_lexicon_tags_45 },
    { 0xa424b1a7, SUC_lexicon_tags_68 },
    { 0xf204b1a8, SUC_lexicon_tags_25 },
    { 0x11c4b1a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea4b1b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f44b1b8, SUC_lexicon_tags_25 },
    { 0x3a64b1b9, SUC_lexicon_tags_2 },
    { 0xcc24b1ba, SUC_lexicon_tags_7 },
    { 0x78c4b1bb, SUC_lexicon_tags_12 },
    { 0xce84b1bc, SUC_lexicon_tags_2 },
    { 0x6a84b1bc, SUC_lexicon_tags_22 },
    { 0xa904b1bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc4b1c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b1c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a4b1c8, SUC_lexicon_tags_9 },
    { 0x6fe4b1c9, SUC_lexicon_tags_1 },
    { 0xba4b1ca, SUC_lexicon_tags_43 },
    { 0xa804b1c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c4b1d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a4b1d6, SUC_lexicon_tags_0 },
    { 0x5964b1d6, SUC_lexicon_tags_9 },
    { 0x7844b1d8, SUC_lexicon_tags_4 },
    { 0x3584b1d7, SUC_lexicon_tags_38 },
    { 0x77e4b1da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9864b1dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee4b1df, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x2ba4b1e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae4b1e6, SUC_lexicon_tags_45 },
    { 0xd504b1e7, SUC_lexicon_tags_46 },
    { 0x7564b1e6, SUC_lexicon_tags_2 },
    { 0xd404b1e9, SUC_lexicon_tags_37 },
    { 0xfc4b1ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3144b1f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d24b1f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce4b1f8, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb844b1fc, SUC_lexicon_tags_8 },
    { 0xd3a4b1fd, SUC_lexicon_tags_12 },
    { 0x74a4b1fe, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x8a04b200, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4744b202, SUC_lexicon_tags_37 },
    { 0xd424b203, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x66c4b205, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6524b20d, SUC_lexicon_tags_47 },
    { 0x10a4b20e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d24b211, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa024b214, SUC_lexicon_tags_23 },
    { 0xa1a4b214, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xce84b218, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6344b220, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf804b222, SUC_lexicon_tags_23 },
    { 0x7644b223, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a4b225, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf044b227, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b04b22b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc244b22e, SUC_lexicon_tags_28 },
    { 0x5444b22f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e4b232, SUC_lexicon_tags_25 },
    { 0xec04b233, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x39e4b235, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a4b238, SUC_lexicon_tags_42 },
    { 0x6764b239, SUC_lexicon_tags_5 },
    { 0x4024b238, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e4b240, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e04b246, SUC_lexicon_tags_30 },
    { 0xccc4b246, SUC_lexicon_tags_28 },
    { 0x3044b248, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9564b24b, SUC_lexicon_tags_27 },
    { 0x8e64b24c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc204b252, SUC_lexicon_tags_27 },
    { 0xf304b252, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca4b256, SUC_lexicon_tags_7 },
    { 0x74c4b257, SUC_lexicon_tags_63 },
    { 0x68a4b258, SUC_lexicon_tags_21 },
    { 0x5744b258, SUC_lexicon_tags_53 },
    { 0xaa44b25a, SUC_lexicon_tags_38 },
    { 0x6124b25b, SUC_lexicon_tags_7 },
    { 0x7c84b25b, SUC_lexicon_tags_25 },
    { 0xa804b25b, SUC_lexicon_tags_25 },
    { 0x1364b256, SUC_lexicon_tags_79 },
    { 0xdfa4b25b, SUC_lexicon_tags_1 },
    { 0xfe24b259, SUC_lexicon_tags_37 },
    { 0xad24b25c, SUC_lexicon_tags_27 },
    { 0xbcc4b262, SUC_lexicon_tags_15 },
    { 0x8f84b263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d44b269, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd584b26d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4b273, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe404b276, SUC_lexicon_tags_17 },
    { 0x7ac4b277, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2684b27b, SUC_lexicon_tags_15 },
    { 0x9944b27b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3404b27f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e4b282, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8cc4b284, SUC_lexicon_tags_25 },
    { 0xd4c4b284, SUC_lexicon_tags_19 },
    { 0xd464b284, SUC_lexicon_tags_11 },
    { 0xb364b287, SUC_lexicon_tags_2 },
    { 0x7844b288, SUC_lexicon_tags_23 },
    { 0x1f64b289, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd724b28b, SUC_lexicon_tags_27 },
    { 0x224b28c, SUC_lexicon_tags_37 },
    { 0xa264b28d, SUC_lexicon_tags_17 },
    { 0xeae4b28e, SUC_lexicon_tags_66 },
    { 0xf2e4b28f, SUC_lexicon_tags_17 },
    { 0xaf44b290, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31a4b292, SUC_lexicon_tags_0 },
    { 0x5384b292, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a4b297, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9644b299, SUC_lexicon_tags_32 },
    { 0xe6c4b29a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b04b29d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb044b2a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee4b2a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a04b2a8, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xede4b2aa, SUC_lexicon_tags_37 },
    { 0x6124b2aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9744b2af, SUC_lexicon_tags_15 },
    { 0x2344b2af, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4804b2b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c04b2b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2884b2bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e4b2bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17c4b2c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe064b2c4, SUC_lexicon_tags_12 },
    { 0xa164b2c5, SUC_lexicon_tags_60 },
    { 0x87a4b2c6, SUC_lexicon_tags_42 },
    { 0xb2c4b2c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb404b2c9, SUC_lexicon_tags_15 },
    { 0xeb64b2ca, SUC_lexicon_tags_63 },
    { 0xaec4b2ca, SUC_lexicon_tags_1 },
    { 0x3be4b2cb, SUC_lexicon_tags_47 },
    { 0x1444b2cd, SUC_lexicon_tags_2 },
    { 0xcea4b2ca, SUC_lexicon_tags_31 },
    { 0xab44b2cf, SUC_lexicon_tags_23 },
    { 0xf2c4b2c9, SUC_lexicon_tags_13 },
    { 0x7b44b2d1, SUC_lexicon_tags_32 },
    { 0x5604b2d2, SUC_lexicon_tags_11 },
    { 0xf8a4b2d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1964b2d9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa4b2e2, SUC_lexicon_tags_23 },
    { 0x76a4b2e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e4b2e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa4b2ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7364b2f2, SUC_lexicon_tags_0 },
    { 0xbe04b2f2, SUC_lexicon_tags_2 },
    { 0x2b64b2f3, SUC_lexicon_tags_23 },
    { 0x89c4b2f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1984b2f7, SUC_lexicon_tags_43 },
    { 0xdb24b2f8, SUC_lexicon_tags_15 },
    { 0xec4b2f9, SUC_lexicon_tags_33 },
    { 0xca4b2f8, SUC_lexicon_tags_15 },
    { 0x95e4b2f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4d24b2fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee44b303, SUC_lexicon_tags_17 },
    { 0xc444b304, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a4b30e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaca4b310, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8024b312, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2124b316, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3584b31b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1624b326, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa9a4b328, SUC_lexicon_tags_37 },
    { 0x2be4b329, SUC_lexicon_tags_29 },
    { 0xd9c4b32a, SUC_lexicon_tags_4 },
    { 0x4ca4b32b, SUC_lexicon_tags_27 },
    { 0x2104b32b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40a4b32e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf484b330, SUC_lexicon_tags_25 },
    { 0x4384b331, SUC_lexicon_tags_27 },
    { 0x4504b332, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7c24b334, SUC_lexicon_tags_30 },
    { 0xec24b335, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc4b338, SUC_lexicon_tags_15 },
    { 0x76e4b339, SUC_lexicon_tags_1 },
    { 0x8a24b33a, SUC_lexicon_tags_17 },
    { 0x6964b33a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1a4b33d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c4b340, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de4b347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b34a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb44b34d, SUC_lexicon_tags_25 },
    { 0xed44b34e, SUC_lexicon_tags_15 },
    { 0xd7c4b34f, SUC_lexicon_tags_25 },
    { 0x9c4b350, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc4b353, SUC_lexicon_tags_37 },
    { 0x4544b353, SUC_lexicon_tags_37 },
    { 0x26a4b355, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e84b35b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee4b365, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b44b368, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2804b36b, SUC_lexicon_tags_17 },
    { 0x484b36c, SUC_lexicon_tags_36 },
    { 0x6a84b36b, SUC_lexicon_tags_15 },
    { 0x2884b36b, SUC_lexicon_tags_21 },
    { 0xf264b36f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd384b373, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30c4b375, SUC_lexicon_tags_65 },
    { 0x3b04b375, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e24b378, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac4b37d, SUC_lexicon_tags_2 },
    { 0xcd24b37e, SUC_lexicon_tags_11 },
    { 0x76c4b37f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a84b385, SUC_lexicon_tags_37 },
    { 0x5284b386, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c4b38f, SUC_lexicon_tags_7 },
    { 0xbc4b38f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c4b395, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1464b39c, SUC_lexicon_tags_33 },
    { 0x9d84b39c, SUC_lexicon_tags_16 },
    { 0x4c24b39e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4944b3a3, SUC_lexicon_tags_23 },
    { 0xd1c4b3a4, SUC_lexicon_tags_2 },
    { 0x1784b3a5, SUC_lexicon_tags_14 },
    { 0x6be4b3a6, SUC_lexicon_tags_25 },
    { 0x4ac4b3a4, SUC_lexicon_tags_15 },
    { 0x4e24b3a8, SUC_lexicon_tags_17 },
    { 0x7e44b3a9, SUC_lexicon_tags_12 },
    { 0xf564b3a8, SUC_lexicon_tags_11 },
    { 0x6124b3ab, SUC_lexicon_tags_15 },
    { 0xcdc4b3a6, SUC_lexicon_tags_7 },
    { 0xe524b3ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6864b3b6, SUC_lexicon_tags_19 },
    { 0x1164b3b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2884b3bc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4b3c0, SUC_lexicon_tags_7 },
    { 0x84c4b3c1, SUC_lexicon_tags_3 },
    { 0x40a4b3c0, SUC_lexicon_tags_17 },
    { 0x8de4b3c3, SUC_lexicon_tags_26 },
    { 0x444b3c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44b3cf, SUC_lexicon_tags_60 },
    { 0x16e4b3cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4b3d3, SUC_lexicon_tags_12 },
    { 0x18c4b3d4, SUC_lexicon_tags_14 },
    { 0x1e24b3d5, SUC_lexicon_tags_7 },
    { 0x42a4b3d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7144b3d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3124b3dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b04b3de, SUC_lexicon_tags_55 },
    { 0xc544b3df, SUC_lexicon_tags_1 },
    { 0xe364b3df, SUC_lexicon_tags_43 },
    { 0x6484b3e1, SUC_lexicon_tags_11 },
    { 0xc484b3e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee84b3eb, SUC_lexicon_tags_74 },
    { 0x9344b3ec, SUC_lexicon_tags_19 },
    { 0x2544b3eb, SUC_lexicon_tags_33 },
    { 0xe084b3ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb064b3f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3204b3f7, SUC_lexicon_tags_17 },
    { 0xd5e4b3f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b84b3fc, SUC_lexicon_tags_27 },
    { 0x61a4b3fc, SUC_lexicon_tags_2 },
    { 0x2e84b3fd, SUC_lexicon_tags_29 },
    { 0xf824b3ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6364b402, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd084b407, SUC_lexicon_tags_27 },
    { 0x4a24b408, SUC_lexicon_tags_39 },
    { 0xd7c4b409, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee4b40d, SUC_lexicon_tags_78 },
    { 0xddc4b40e, SUC_lexicon_tags_23 },
    { 0x4664b40f, SUC_lexicon_tags_38 },
    { 0x6004b40d, SUC_lexicon_tags_7 },
    { 0x6be4b411, SUC_lexicon_tags_1 },
    { 0xf9c4b40d, SUC_lexicon_tags_37 },
    { 0xeac4b412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e4b416, SUC_lexicon_tags_19 },
    { 0x7064b417, SUC_lexicon_tags_59 },
    { 0x32c4b418, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac4b41b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca4b41e, SUC_lexicon_tags_37 },
    { 0x9164b41f, SUC_lexicon_tags_37 },
    { 0x7924b420, SUC_lexicon_tags_0 },
    { 0xba24b420, SUC_lexicon_tags_25 },
    { 0x2804b422, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa4b425, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc484b42b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2244b42d, SUC_lexicon_tags_2 },
    { 0x164b42e, SUC_lexicon_tags_7 },
    { 0xf144b42d, SUC_lexicon_tags_11 },
    { 0x4d24b42d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93c4b432, SUC_lexicon_tags_17 },
    { 0x1184b433, SUC_lexicon_tags_28 },
    { 0xdde4b434, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e44b439, SUC_lexicon_tags_63 },
    { 0x1e44b43a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a4b43d, SUC_lexicon_tags_23 },
    { 0x6464b43e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c4b445, SUC_lexicon_tags_7 },
    { 0xe744b446, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1564b448, SUC_lexicon_tags_47 },
    { 0x4ba4b449, SUC_lexicon_tags_11 },
    { 0xd5c4b44a, SUC_lexicon_tags_17 },
    { 0x9844b449, SUC_lexicon_tags_65 },
    { 0xc224b44b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa4b450, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfc64b452, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6264b454, SUC_lexicon_tags_23 },
    { 0x6064b455, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1504b45a, SUC_lexicon_tags_65 },
    { 0xb1c4b45a, SUC_lexicon_tags_33 },
    { 0x87a4b45c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa04b45f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc4b463, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbce4b465, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c64b469, SUC_lexicon_tags_23 },
    { 0x3b44b46a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a4b46e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc4b472, SUC_lexicon_tags_12 },
    { 0x85a4b472, SUC_lexicon_tags_25 },
    { 0x7624b472, SUC_lexicon_tags_14 },
    { 0xf2e4b474, SUC_lexicon_tags_15 },
    { 0x7e24b476, SUC_lexicon_tags_79 },
    { 0x98e4b473, SUC_lexicon_tags_0 },
    { 0x3b64b478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b64b47d, SUC_lexicon_tags_63 },
    { 0xd924b47e, SUC_lexicon_tags_15 },
    { 0xe3c4b47d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x644b481, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20e4b483, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe964b485, SUC_lexicon_tags_14 },
    { 0x7b04b486, SUC_lexicon_tags_15 },
    { 0x6b44b487, SUC_lexicon_tags_12 },
    { 0xcd44b487, SUC_lexicon_tags_11 },
    { 0x1a84b489, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x124b48b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84b491, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e4b494, SUC_lexicon_tags_1 },
    { 0x484b494, SUC_lexicon_tags_37 },
    { 0x3ac4b494, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc024b49c, SUC_lexicon_tags_77 },
    { 0x9ec4b49d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4b4a0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa4b4a5, SUC_lexicon_tags_8 },
    { 0x984b4a6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe64b4a8, SUC_lexicon_tags_7 },
    { 0xfce4b4a9, SUC_lexicon_tags_32 },
    { 0xb6e4b4aa, SUC_lexicon_tags_82 },
    { 0xb224b4ab, SUC_lexicon_tags_37 },
    { 0x1a84b4ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xebe4b4ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f04b4b0, SUC_lexicon_tags_37 },
    { 0x9784b4b0, SUC_lexicon_tags_37 },
    { 0x7d84b4b0, SUC_lexicon_tags_16 },
    { 0xb684b4b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x404b4b5, SUC_lexicon_tags_47 },
    { 0xae64b4b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf184b4b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2464b4bb, SUC_lexicon_tags_28 },
    { 0x7744b4bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d84b4c3, SUC_lexicon_tags_27 },
    { 0x63e4b4c4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x93c4b4c6, SUC_lexicon_tags_37 },
    { 0x81e4b4c6, SUC_lexicon_tags_65 },
    { 0x72e4b4c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c4b4cc, SUC_lexicon_tags_4 },
    { 0xb664b4cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc4b4d1, SUC_lexicon_tags_2 },
    { 0x8544b4d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b84b4dc, SUC_lexicon_tags_68 },
    { 0xa224b4dd, SUC_lexicon_tags_2 },
    { 0x7a4b4de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b64b4e6, SUC_lexicon_tags_28 },
    { 0x1e4b4e7, SUC_lexicon_tags_37 },
    { 0xac44b4e8, SUC_lexicon_tags_12 },
    { 0xab24b4e7, SUC_lexicon_tags_47 },
    { 0x864b4ea, SUC_lexicon_tags_12 },
    { 0xc904b4ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e4b4ee, SUC_lexicon_tags_32 },
    { 0x3204b4ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca4b4fa, SUC_lexicon_tags_17 },
    { 0xebe4b4fa, SUC_lexicon_tags_12 },
    { 0x3e44b4fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe24b507, SUC_lexicon_tags_61 },
    { 0x3804b507, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0xca84b50b, SUC_lexicon_tags_66 },
    { 0xd164b50b, SUC_lexicon_tags_12 },
    { 0xa244b50b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224b51b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8224b524, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7124b526, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9a4b528, SUC_lexicon_tags_15 },
    { 0xcc44b529, SUC_lexicon_tags_82 },
    { 0x78a4b529, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc064b52e, SUC_lexicon_tags_12 },
    { 0xc604b52e, SUC_lexicon_tags_47 },
    { 0xc104b52e, SUC_lexicon_tags_28 },
    { 0xf9c4b531, SUC_lexicon_tags_37 },
    { 0x79c4b532, SUC_lexicon_tags_17 },
    { 0xeca4b533, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc124b536, SUC_lexicon_tags_19 },
    { 0x4204b536, SUC_lexicon_tags_37 },
    { 0x6da4b538, SUC_lexicon_tags_63 },
    { 0x2c4b537, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3544b53e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb64b541, SUC_lexicon_tags_33 },
    { 0x25e4b542, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a44b546, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de4b549, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4304b553, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb344b556, SUC_lexicon_tags_37 },
    { 0x8f64b557, SUC_lexicon_tags_6 },
    { 0xbe24b558, SUC_lexicon_tags_30 },
    { 0xd9e4b558, SUC_lexicon_tags_17 },
    { 0xa944b556, SUC_lexicon_tags_37 },
    { 0x9084b559, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a4b55e, SUC_lexicon_tags_7 },
    { 0xe3c4b55f, SUC_lexicon_tags_14 },
    { 0xc7c4b55f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e4b563, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c84b566, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c44b569, SUC_lexicon_tags_11 },
    { 0x75a4b56a, SUC_lexicon_tags_29 },
    { 0xaf64b56a, SUC_lexicon_tags_98 },
    { 0x43e4b56b, SUC_lexicon_tags_7 },
    { 0x84b56d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb1a4b56f, SUC_lexicon_tags_40 },
    { 0xdda4b570, SUC_lexicon_tags_14 },
    { 0x9064b571, SUC_lexicon_tags_19 },
    { 0xbf64b56f, SUC_lexicon_tags_2 },
    { 0x5f84b573, SUC_lexicon_tags_9 },
    { 0x2604b574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1704b579, SUC_lexicon_tags_2 },
    { 0xd9e4b579, SUC_lexicon_tags_17 },
    { 0xfb04b57b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3224b580, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8664b582, SUC_lexicon_tags_10 },
    { 0xd244b582, SUC_lexicon_tags_6 },
    { 0x2b44b584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfec4b586, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d04b588, SUC_lexicon_tags_25 },
    { 0x9ee4b589, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc724b58b, SUC_lexicon_tags_26 },
    { 0xe7c4b58c, SUC_lexicon_tags_2 },
    { 0xede4b58b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7384b58f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3244b591, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a4b595, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95c4b597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7864b59b, SUC_lexicon_tags_0 },
    { 0x5984b59b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae4b59f, SUC_lexicon_tags_1 },
    { 0x5284b59f, SUC_lexicon_tags_11 },
    { 0xeea4b5a1, SUC_lexicon_tags_43 },
    { 0x8984b5a2, SUC_lexicon_tags_0 },
    { 0x9b04b5a1, SUC_lexicon_tags_11 },
    { 0xd7e4b5a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18e4b5a6, SUC_lexicon_tags_12 },
    { 0x56e4b5a7, SUC_lexicon_tags_2 },
    { 0x4904b5a7, SUC_lexicon_tags_47 },
    { 0xb3e4b5a7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6424b5ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa604b5b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24b5bb, SUC_lexicon_tags_0 },
    { 0x4224b5bc, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e4b5c1, SUC_lexicon_tags_35 },
    { 0xd2a4b5c1, SUC_lexicon_tags_45 },
    { 0xd264b5c3, SUC_lexicon_tags_47 },
    { 0x49c4b5c3, SUC_lexicon_tags_15 },
    { 0x284b5c5, SUC_lexicon_tags_61 },
    { 0x1744b5c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5b24b5c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a24b5cc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc724b5d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6844b5d4, SUC_lexicon_tags_11 },
    { 0x9ec4b5d5, SUC_lexicon_tags_1 },
    { 0xc764b5d6, SUC_lexicon_tags_39 },
    { 0x5e4b5d7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe004b5d9, SUC_lexicon_tags_104 },
    { 0x7584b5da, SUC_lexicon_tags_11 },
    { 0x4824b5db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a4b5df, SUC_lexicon_tags_14 },
    { 0xf084b5e0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1444b5eb, SUC_lexicon_tags_33 },
    { 0xadc4b5ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e44b5fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9924b5fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb864b602, SUC_lexicon_tags_2 },
    { 0x7704b602, SUC_lexicon_tags_609 },
    { 0xdb84b603, SUC_lexicon_tags_23 },
    { 0xf5e4b603, SUC_lexicon_tags_9 },
    { 0x3484b605, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64b609, SUC_lexicon_tags_37 },
    { 0x9764b60a, SUC_lexicon_tags_37 },
    { 0xf1a4b609, SUC_lexicon_tags_1 },
    { 0x3104b60c, SUC_lexicon_tags_12 },
    { 0x7d64b60d, SUC_lexicon_tags_19 },
    { 0xee04b60c, SUC_lexicon_tags_33 },
    { 0x3bc4b60f, SUC_lexicon_tags_0 },
    { 0xdf84b610, SUC_lexicon_tags_12 },
    { 0xb3e4b611, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca4b614, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba4b619, SUC_lexicon_tags_1 },
    { 0x8b44b61a, SUC_lexicon_tags_12 },
    { 0xcc24b61a, SUC_lexicon_tags_33 },
    { 0x66c4b61c, SUC_lexicon_tags_12 },
    { 0xd544b61c, SUC_lexicon_tags_1 },
    { 0xbc84b61e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd384b620, SUC_lexicon_tags_17 },
    { 0x7904b620, SUC_lexicon_tags_1 },
    { 0x2524b622, SUC_lexicon_tags_6 },
    { 0xa964b621, SUC_lexicon_tags_61 },
    { 0x1da4b622, SUC_lexicon_tags_31 },
    { 0xcda4b625, SUC_lexicon_tags_42 },
    { 0xdc4b626, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd624b628, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7204b62a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad64b630, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4864b635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d64b63c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9d04b63e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc24b641, SUC_lexicon_tags_60 },
    { 0xa9c4b642, SUC_lexicon_tags_21 },
    { 0x5ba4b643, SUC_lexicon_tags_19 },
    { 0xeca4b642, SUC_lexicon_tags_25 },
    { 0x6d24b645, SUC_lexicon_tags_1 },
    { 0x60a4b646, SUC_lexicon_tags_12 },
    { 0xad84b647, SUC_lexicon_tags_65 },
    { 0x3444b648, SUC_lexicon_tags_30 },
    { 0xbbe4b647, SUC_lexicon_tags_30 },
    { 0x7f64b648, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a4b651, SUC_lexicon_tags_17 },
    { 0x8144b652, SUC_lexicon_tags_17 },
    { 0x2644b653, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4c4b655, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c4b658, SUC_lexicon_tags_25 },
    { 0xda04b659, SUC_lexicon_tags_63 },
    { 0xcec4b65a, SUC_lexicon_tags_15 },
    { 0x4ac4b65b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d24b65e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c4b661, SUC_lexicon_tags_47 },
    { 0xbd04b662, SUC_lexicon_tags_26 },
    { 0xe124b662, SUC_lexicon_tags_31 },
    { 0x5d64b663, SUC_lexicon_tags_26 },
    { 0xf504b662, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5224b667, SUC_lexicon_tags_15 },
    { 0xa9e4b667, SUC_lexicon_tags_1461 },
    { 0xa384b669, SUC_lexicon_tags_1 },
    { 0x7764b669, SUC_lexicon_tags_37 },
    { 0x8024b66b, SUC_lexicon_tags_27 },
    { 0x9f44b667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4904b670, SUC_lexicon_tags_2 },
    { 0x7aa4b671, SUC_lexicon_tags_33 },
    { 0x6a4b672, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9c44b674, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f64b676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf204b678, SUC_lexicon_tags_25 },
    { 0x16a4b678, SUC_lexicon_tags_25 },
    { 0x2bc4b679, SUC_lexicon_tags_1 },
    { 0x6ee4b679, SUC_lexicon_tags_47 },
    { 0xc944b67c, SUC_lexicon_tags_11 },
    { 0xa6a4b67d, SUC_lexicon_tags_47 },
    { 0xcc84b67e, SUC_lexicon_tags_15 },
    { 0xeb64b67e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa404b681, SUC_lexicon_tags_49 },
    { 0xe804b682, SUC_lexicon_tags_14 },
    { 0x3704b681, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53c4b685, SUC_lexicon_tags_10 },
    { 0x80a4b686, SUC_lexicon_tags_19 },
    { 0xa624b687, SUC_lexicon_tags_7 },
    { 0x4364b688, SUC_lexicon_tags_21 },
    { 0x31a4b688, SUC_lexicon_tags_60 },
    { 0x77e4b68a, SUC_lexicon_tags_14 },
    { 0xc9c4b68a, SUC_lexicon_tags_7 },
    { 0x9244b68c, SUC_lexicon_tags_79 },
    { 0xccc4b68d, SUC_lexicon_tags_33 },
    { 0x1ac4b68d, SUC_lexicon_tags_37 },
    { 0x55a4b68f, SUC_lexicon_tags_11 },
    { 0x2c24b68f, SUC_lexicon_tags_35 },
    { 0x6124b688, SUC_lexicon_tags_0 },
    { 0x3584b68c, SUC_lexicon_tags_17 },
    { 0x464b693, SUC_lexicon_tags_27 },
    { 0x3824b693, SUC_lexicon_tags_42 },
    { 0x1144b691, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9564b697, SUC_lexicon_tags_7 },
    { 0xb964b697, SUC_lexicon_tags_2 },
    { 0x5f64b697, SUC_lexicon_tags_9 },
    { 0x88a4b69a, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac4b6a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb084b6a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x324b6a5, SUC_lexicon_tags_26 },
    { 0x4cc4b6a5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6424b6a8, SUC_lexicon_tags_29 },
    { 0xf024b6a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c4b6af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe344b6b1, SUC_lexicon_tags_53 },
    { 0xe844b6b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e4b6b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe164b6bb, SUC_lexicon_tags_27 },
    { 0x1424b6bc, SUC_lexicon_tags_7 },
    { 0x6dc4b6bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2904b6c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8564b6c4, SUC_lexicon_tags_90 },
    { 0xca24b6c5, SUC_lexicon_tags_27 },
    { 0x5304b6c6, SUC_lexicon_tags_17 },
    { 0x9184b6c4, SUC_lexicon_tags_45 },
    { 0x78e4b6c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf984b6ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa904b6cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb604b6d0, SUC_lexicon_tags_2 },
    { 0x8f24b6d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6004b6d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de4b6d7, SUC_lexicon_tags_37 },
    { 0xc2c4b6d8, SUC_lexicon_tags_47 },
    { 0xf9e4b6d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2724b6dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc4b6e0, SUC_lexicon_tags_28 },
    { 0xa5c4b6e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b44b6e3, SUC_lexicon_tags_37 },
    { 0x31c4b6e4, SUC_lexicon_tags_43 },
    { 0x4744b6e5, SUC_lexicon_tags_17 },
    { 0xd6c4b6e6, SUC_lexicon_tags_17 },
    { 0x5224b6e5, SUC_lexicon_tags_141 },
    { 0x58a4b6e6, SUC_lexicon_tags_47 },
    { 0x6544b6e5, SUC_lexicon_tags_17 },
    { 0x6a24b6ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20e4b6ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4b6ef, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a44b6f2, SUC_lexicon_tags_12 },
    { 0x3a64b6f2, SUC_lexicon_tags_11 },
    { 0x93e4b6f3, SUC_lexicon_tags_26 },
    { 0xf204b6f5, SUC_lexicon_tags_7 },
    { 0xd104b6f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be4b703, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1884b707, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44b70b, SUC_lexicon_tags_38 },
    { 0x144b70c, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf04b712, SUC_lexicon_tags_21 },
    { 0x1544b713, SUC_lexicon_tags_0 },
    { 0x1ec4b714, SUC_lexicon_tags_37 },
    { 0xcb24b715, SUC_lexicon_tags_17 },
    { 0xb084b716, SUC_lexicon_tags_39 },
    { 0x7084b717, SUC_lexicon_tags_25 },
    { 0x28c4b717, SUC_lexicon_tags_12 },
    { 0x6344b716, SUC_lexicon_tags_11 },
    { 0x70c4b71a, SUC_lexicon_tags_1 },
    { 0xdc64b71a, SUC_lexicon_tags_17 },
    { 0x5584b718, SUC_lexicon_tags_1 },
    { 0xb004b71d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa144b71f, SUC_lexicon_tags_17 },
    { 0x6424b720, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c4b723, SUC_lexicon_tags_15 },
    { 0x8224b724, SUC_lexicon_tags_11 },
    { 0x2b04b725, SUC_lexicon_tags_14 },
    { 0x7084b724, SUC_lexicon_tags_11 },
    { 0xd1a4b723, SUC_lexicon_tags_25 },
    { 0x7404b728, SUC_lexicon_tags_11 },
    { 0xce04b723, SUC_lexicon_tags_0 },
    { 0x8944b72a, SUC_lexicon_tags_38 },
    { 0x5704b72a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9364b731, SUC_lexicon_tags_37 },
    { 0x6dc4b732, SUC_lexicon_tags_2 },
    { 0x8a24b733, SUC_lexicon_tags_7 },
    { 0x1304b733, SUC_lexicon_tags_26 },
    { 0x2d44b733, SUC_lexicon_tags_11 },
    { 0xb4a4b732, SUC_lexicon_tags_1 },
    { 0x4104b731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7404b73a, SUC_lexicon_tags_25 },
    { 0xf5c4b73b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1884b73e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e4b742, SUC_lexicon_tags_224 },
    { 0x4cc4b743, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62a4b745, SUC_lexicon_tags_17 },
    { 0xc524b745, SUC_lexicon_tags_2 },
    { 0xeee4b747, SUC_lexicon_tags_97 },
    { 0xf2e4b748, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e4b74a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1164b74f, SUC_lexicon_tags_25 },
    { 0xe04b750, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224b756, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d24b75d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9724b761, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384b765, SUC_lexicon_tags_29 },
    { 0x44e4b766, SUC_lexicon_tags_24 },
    { 0x1504b767, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x13c4b769, SUC_lexicon_tags_12 },
    { 0x404b769, SUC_lexicon_tags_17 },
    { 0x2e04b76a, SUC_lexicon_tags_18 },
    { 0x9404b769, SUC_lexicon_tags_17 },
    { 0x1b04b76c, SUC_lexicon_tags_6 },
    { 0x3cc4b76e, SUC_lexicon_tags_11 },
    { 0xa404b76f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf184b771, SUC_lexicon_tags_63 },
    { 0xdbe4b772, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc4b779, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe3c4b77b, SUC_lexicon_tags_6 },
    { 0x78c4b77c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa4b781, SUC_lexicon_tags_17 },
    { 0xa3e4b782, SUC_lexicon_tags_17 },
    { 0x67c4b783, SUC_lexicon_tags_27 },
    { 0x6ae4b781, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2be4b786, SUC_lexicon_tags_9 },
    { 0x72e4b787, SUC_lexicon_tags_38 },
    { 0x2864b788, SUC_lexicon_tags_17 },
    { 0xcf84b789, SUC_lexicon_tags_1 },
    { 0x60a4b788, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4804b78e, SUC_lexicon_tags_1 },
    { 0xaf64b78e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3404b792, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c24b796, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95e4b798, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c4b79c, SUC_lexicon_tags_38 },
    { 0x37e4b79d, SUC_lexicon_tags_60 },
    { 0xadc4b79e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17c4b7a0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2ca4b7a2, SUC_lexicon_tags_27 },
    { 0x75c4b7a3, SUC_lexicon_tags_23 },
    { 0xace4b7a4, SUC_lexicon_tags_37 },
    { 0x1404b7a5, SUC_lexicon_tags_27 },
    { 0x50c4b7a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1444b7aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a4b7ad, SUC_lexicon_tags_0 },
    { 0x3bc4b7ad, SUC_lexicon_tags_37 },
    { 0xd964b7af, SUC_lexicon_tags_0 },
    { 0x7884b7b0, SUC_lexicon_tags_49 },
    { 0x41a4b7af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90e4b7b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c4b7b5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb004b7b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd64b7bb, SUC_lexicon_tags_25 },
    { 0x1aa4b7bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e4b7c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae4b7c3, SUC_lexicon_tags_2 },
    { 0x5364b7c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e4b7c8, SUC_lexicon_tags_8 },
    { 0x4464b7c9, SUC_lexicon_tags_12 },
    { 0x43e4b7ca, SUC_lexicon_tags_1 },
    { 0x4784b7ca, SUC_lexicon_tags_11 },
    { 0xc144b7cb, SUC_lexicon_tags_7 },
    { 0x1ea4b7cc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xde04b7d0, SUC_lexicon_tags_15 },
    { 0xb9a4b7d1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce4b7d4, SUC_lexicon_tags_61 },
    { 0xcfc4b7d5, SUC_lexicon_tags_15 },
    { 0x5ee4b7d4, SUC_lexicon_tags_49 },
    { 0xe464b7d7, SUC_lexicon_tags_9 },
    { 0x8d24b7d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc724b7db, SUC_lexicon_tags_8 },
    { 0x5884b7dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0c4b7de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x124b7e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x704b7e2, SUC_lexicon_tags_16 },
    { 0xd844b7e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a4b7e7, SUC_lexicon_tags_43 },
    { 0xac64b7e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde24b7ef, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9084b7f2, SUC_lexicon_tags_1 },
    { 0xef44b7f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9444b7f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd284b7fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a4b806, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9184b808, SUC_lexicon_tags_25 },
    { 0x2944b809, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984b80d, SUC_lexicon_tags_25 },
    { 0x8444b80d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa64b811, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a4b817, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e4b825, SUC_lexicon_tags_14 },
    { 0xacc4b826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb964b82b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcbe4b82d, SUC_lexicon_tags_23 },
    { 0x3d24b82d, SUC_lexicon_tags_476 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b04b833, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9bc4b835, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc084b83f, SUC_lexicon_tags_32 },
    { 0x5644b840, SUC_lexicon_tags_11 },
    { 0xa704b841, SUC_lexicon_tags_37 },
    { 0x1004b840, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc644b846, SUC_lexicon_tags_6 },
    { 0xd024b847, SUC_lexicon_tags_45 },
    { 0xdbe4b848, SUC_lexicon_tags_60 },
    { 0x50e4b849, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8364b84b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4624b84e, SUC_lexicon_tags_140 },
    { 0x22c4b84f, SUC_lexicon_tags_25 },
    { 0xd004b84e, SUC_lexicon_tags_4 },
    { 0xf004b851, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4b85e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e4b863, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e4b866, SUC_lexicon_tags_25 },
    { 0xa1c4b867, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe4b86f, SUC_lexicon_tags_65 },
    { 0x1d44b870, SUC_lexicon_tags_25 },
    { 0xa984b86f, SUC_lexicon_tags_37 },
    { 0x9924b872, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba4b876, SUC_lexicon_tags_15 },
    { 0x8de4b877, SUC_lexicon_tags_29 },
    { 0x2e24b878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68c4b87a, SUC_lexicon_tags_2 },
    { 0x6f44b87a, SUC_lexicon_tags_33 },
    { 0x74a4b87c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c24b87e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a44b88b, SUC_lexicon_tags_25 },
    { 0x7344b88c, SUC_lexicon_tags_27 },
    { 0xe3c4b88d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b64b891, SUC_lexicon_tags_49 },
    { 0x48e4b892, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e64b894, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83a4b896, SUC_lexicon_tags_13 },
    { 0xa1a4b896, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a4b89e, SUC_lexicon_tags_27 },
    { 0x7be4b89f, SUC_lexicon_tags_12 },
    { 0x3364b89f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcc4b8a2, SUC_lexicon_tags_1 },
    { 0x2a4b8a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e4b8a5, SUC_lexicon_tags_2 },
    { 0xb844b8a6, SUC_lexicon_tags_1 },
    { 0xaba4b8a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb444b8a9, SUC_lexicon_tags_1 },
    { 0xe164b8aa, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6444b8ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3224b8b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda44b8b7, SUC_lexicon_tags_19 },
    { 0x28e4b8b8, SUC_lexicon_tags_34 },
    { 0xb84b8b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3524b8bb, SUC_lexicon_tags_0 },
    { 0x91e4b8bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1564b8c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7424b8c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c4b8c5, SUC_lexicon_tags_37 },
    { 0x40e4b8c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564b8ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d84b8cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac64b8d3, SUC_lexicon_tags_38 },
    { 0x7344b8d4, SUC_lexicon_tags_40 },
    { 0x8f24b8d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e4b8da, SUC_lexicon_tags_11 },
    { 0x1604b8db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7044b8dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c4b8ee, SUC_lexicon_tags_14 },
    { 0x64c4b8ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fa4b8f1, SUC_lexicon_tags_2 },
    { 0x6b24b8f2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x2824b8f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4524b8f9, SUC_lexicon_tags_6 },
    { 0xa4a4b8f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4b8fd, SUC_lexicon_tags_17 },
    { 0x4584b8fe, SUC_lexicon_tags_28 },
    { 0xeec4b8ff, SUC_lexicon_tags_37 },
    { 0xf144b900, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa364b903, SUC_lexicon_tags_37 },
    { 0x9c4b904, SUC_lexicon_tags_1 },
    { 0x8d84b905, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664b90e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf64b915, SUC_lexicon_tags_63 },
    { 0xdb04b915, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c4b91b, SUC_lexicon_tags_2 },
    { 0x80a4b91b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4b91f, SUC_lexicon_tags_66 },
    { 0xf244b920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5224b923, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24b92b, SUC_lexicon_tags_30 },
    { 0x4f44b92b, SUC_lexicon_tags_12 },
    { 0x8744b92d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8464b931, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e4b93a, SUC_lexicon_tags_17 },
    { 0x2aa4b93a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2524b93d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe284b93f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc44b948, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a84b94b, SUC_lexicon_tags_46 },
    { 0xc0a4b94c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44b950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9404b952, SUC_lexicon_tags_4 },
    { 0x7924b952, SUC_lexicon_tags_15 },
    { 0x9404b953, SUC_lexicon_tags_14 },
    { 0xd1e4b952, SUC_lexicon_tags_40 },
    { 0x1944b956, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a4b959, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b24b95b, SUC_lexicon_tags_30 },
    { 0xf8e4b95b, SUC_lexicon_tags_19 },
    { 0x8684b95b, SUC_lexicon_tags_44 },
    { 0x2b24b95b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc24b96e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd564b970, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1a44b972, SUC_lexicon_tags_37 },
    { 0x58a4b973, SUC_lexicon_tags_10 },
    { 0x5c24b973, SUC_lexicon_tags_39 },
    { 0x284b975, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1844b977, SUC_lexicon_tags_27 },
    { 0x1e64b977, SUC_lexicon_tags_25 },
    { 0xaf84b979, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd084b97b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a4b980, SUC_lexicon_tags_11 },
    { 0xa824b981, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3804b983, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca24b985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff24b987, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeaa4b989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c24b98f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6704b991, SUC_lexicon_tags_5 },
    { 0xfc64b991, SUC_lexicon_tags_46 },
    { 0xb44b993, SUC_lexicon_tags_0 },
    { 0xfde4b991, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd524b996, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc544b999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d04b9aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e4b9ad, SUC_lexicon_tags_37 },
    { 0x3224b9ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc24b9b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae4b9b6, SUC_lexicon_tags_17 },
    { 0xe7e4b9b6, SUC_lexicon_tags_7 },
    { 0xbe4b9b7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa404b9bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c4b9c3, SUC_lexicon_tags_45 },
    { 0x3824b9c3, SUC_lexicon_tags_15 },
    { 0x26e4b9c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c64b9c8, SUC_lexicon_tags_11 },
    { 0xe3a4b9c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944b9d2, SUC_lexicon_tags_68 },
    { 0x75a4b9d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe924b9d5, SUC_lexicon_tags_11 },
    { 0xb444b9d6, SUC_lexicon_tags_13 },
    { 0x4784b9d6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a84b9dd, SUC_lexicon_tags_35 },
    { 0xd144b9de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c4b9e1, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d84b9e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4204b9e8, SUC_lexicon_tags_65 },
    { 0x3864b9e9, SUC_lexicon_tags_16 },
    { 0xd364b9ea, SUC_lexicon_tags_2 },
    { 0x3164b9ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd684b9ed, SUC_lexicon_tags_11 },
    { 0x38a4b9ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d64b9f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a44b9f3, SUC_lexicon_tags_38 },
    { 0x65c4b9f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a4b9f6, SUC_lexicon_tags_25 },
    { 0x3664b9f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8644b9fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9e4b9fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a4ba00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae4ba03, SUC_lexicon_tags_47 },
    { 0xb324ba04, SUC_lexicon_tags_21 },
    { 0x9c24ba05, SUC_lexicon_tags_37 },
    { 0x7304ba06, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7284ba0c, SUC_lexicon_tags_1 },
    { 0xf844ba0d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a4ba16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4ba18, SUC_lexicon_tags_38 },
    { 0x6744ba18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca84ba1d, SUC_lexicon_tags_17 },
    { 0x5464ba1d, SUC_lexicon_tags_0 },
    { 0x43c4ba1f, SUC_lexicon_tags_14 },
    { 0x83e4ba20, SUC_lexicon_tags_1 },
    { 0xdce4ba1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e4ba26, SUC_lexicon_tags_27 },
    { 0x73a4ba26, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8224ba31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7e4ba33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe524ba36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe704ba38, SUC_lexicon_tags_7 },
    { 0x36c4ba39, SUC_lexicon_tags_35 },
    { 0xf384ba3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa4ba3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace4ba45, SUC_lexicon_tags_12 },
    { 0xdc24ba46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c4ba4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7844ba4d, SUC_lexicon_tags_13 },
    { 0x91c4ba4e, SUC_lexicon_tags_27 },
    { 0x4024ba4e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea4ba52, SUC_lexicon_tags_30 },
    { 0x8604ba53, SUC_lexicon_tags_14 },
    { 0xde84ba54, SUC_lexicon_tags_1 },
    { 0x6aa4ba55, SUC_lexicon_tags_17 },
    { 0xa544ba53, SUC_lexicon_tags_23 },
    { 0x83e4ba53, SUC_lexicon_tags_66 },
    { 0xf384ba58, SUC_lexicon_tags_11 },
    { 0x9ac4ba58, SUC_lexicon_tags_12 },
    { 0xbe4ba53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2a4ba5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc824ba5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a4ba63, SUC_lexicon_tags_7 },
    { 0xdc84ba63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaca4ba66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa124ba68, SUC_lexicon_tags_63 },
    { 0x7024ba69, SUC_lexicon_tags_95 },
    { 0x2204ba69, SUC_lexicon_tags_1 },
    { 0x3724ba6b, SUC_lexicon_tags_21 },
    { 0xdd04ba6b, SUC_lexicon_tags_12 },
    { 0xd404ba6c, SUC_lexicon_tags_34 },
    { 0x66c4ba6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa4ba71, SUC_lexicon_tags_37 },
    { 0x49e4ba71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c4ba77, SUC_lexicon_tags_17 },
    { 0xafe4ba77, SUC_lexicon_tags_13 },
    { 0x7d64ba78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc4ba7c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a24ba82, SUC_lexicon_tags_23 },
    { 0x5fc4ba83, SUC_lexicon_tags_11 },
    { 0x92e4ba84, SUC_lexicon_tags_35 },
    { 0xbb84ba85, SUC_lexicon_tags_45 },
    { 0xb404ba84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f84ba89, SUC_lexicon_tags_15 },
    { 0x3544ba89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e44ba8c, SUC_lexicon_tags_12 },
    { 0x1ba4ba8c, SUC_lexicon_tags_2 },
    { 0xa2c4ba8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d04ba93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a4ba98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa24ba9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd64baa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf344baa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5964baa9, SUC_lexicon_tags_68 },
    { 0x9004baaa, SUC_lexicon_tags_7 },
    { 0xa244baab, SUC_lexicon_tags_17 },
    { 0x6724baac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a4baae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c4bab4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39c4bab6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4684bab8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaee4baba, SUC_lexicon_tags_15 },
    { 0xcc24babb, SUC_lexicon_tags_38 },
    { 0xc7e4babc, SUC_lexicon_tags_27 },
    { 0x1c4babc, SUC_lexicon_tags_7 },
    { 0xad84babe, SUC_lexicon_tags_1 },
    { 0x13e4babf, SUC_lexicon_tags_9 },
    { 0x8c84babe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5884bac2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc4bac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a4bac9, SUC_lexicon_tags_8 },
    { 0xb584bac9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bacd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62c4bacf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbde4bad1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2924bad6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x3084bad9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5484badc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5a4bade, SUC_lexicon_tags_37 },
    { 0xac84bade, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c4bae4, SUC_lexicon_tags_46 },
    { 0x2f24bae4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c4bae8, SUC_lexicon_tags_9 },
    { 0x1424bae9, SUC_lexicon_tags_15 },
    { 0x4924bae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9944baee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b24baf0, SUC_lexicon_tags_17 },
    { 0x55c4baf1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf504baf3, SUC_lexicon_tags_25 },
    { 0xc5a4baf4, SUC_lexicon_tags_25 },
    { 0x84e4baf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9e4baf7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4bafb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9284bafe, SUC_lexicon_tags_17 },
    { 0xb724baff, SUC_lexicon_tags_38 },
    { 0x2564baff, SUC_lexicon_tags_16 },
    { 0x6ba4bb01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4504bb04, SUC_lexicon_tags_10 },
    { 0x21a4bb05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d64bb08, SUC_lexicon_tags_8 },
    { 0x5da4bb09, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x62e4bb0b, SUC_lexicon_tags_19 },
    { 0x2384bb0c, SUC_lexicon_tags_12 },
    { 0x8a44bb0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4124bb0f, SUC_lexicon_tags_17 },
    { 0x284bb0f, SUC_lexicon_tags_25 },
    { 0x1744bb11, SUC_lexicon_tags_32 },
    { 0x9684bb12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1a64bb14, SUC_lexicon_tags_35 },
    { 0x4f44bb15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6fa4bb17, SUC_lexicon_tags_39 },
    { 0xad64bb18, SUC_lexicon_tags_14 },
    { 0x7f84bb19, SUC_lexicon_tags_12 },
    { 0x77e4bb1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xace4bb1d, SUC_lexicon_tags_2 },
    { 0xd024bb1e, SUC_lexicon_tags_69 },
    { 0x2364bb1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5544bb22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7aa4bb24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x95a4bb26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c4bb2a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba4bb2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab04bb34, SUC_lexicon_tags_74 },
    { 0x2c24bb35, SUC_lexicon_tags_43 },
    { 0x1344bb35, SUC_lexicon_tags_2 },
    { 0x80e4bb37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2844bb3c, SUC_lexicon_tags_47 },
    { 0xe384bb3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c64bb40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb64bb42, SUC_lexicon_tags_63 },
    { 0x2a44bb43, SUC_lexicon_tags_37 },
    { 0x4cc4bb43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4304bb49, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e4bb52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6124bb54, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7864bb56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c4bb5a, SUC_lexicon_tags_11 },
    { 0xc284bb5a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec4bb5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6724bb60, SUC_lexicon_tags_12 },
    { 0x2e84bb61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e4bb66, SUC_lexicon_tags_14 },
    { 0x7e4bb67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa4bb6a, SUC_lexicon_tags_47 },
    { 0xb384bb6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa64bb6f, SUC_lexicon_tags_11 },
    { 0x1464bb70, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9784bb74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9044bb7c, SUC_lexicon_tags_6 },
    { 0xac4bb7c, SUC_lexicon_tags_17 },
    { 0x1d44bb7e, SUC_lexicon_tags_37 },
    { 0x94e4bb7e, SUC_lexicon_tags_234 },
    { 0xc464bb7f, SUC_lexicon_tags_43 },
    { 0x7464bb80, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4bb85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4124bb87, SUC_lexicon_tags_60 },
    { 0x1204bb88, SUC_lexicon_tags_25 },
    { 0x9664bb87, SUC_lexicon_tags_19 },
    { 0x95c4bb89, SUC_lexicon_tags_7 },
    { 0x4364bb8b, SUC_lexicon_tags_7 },
    { 0x23a4bb8c, SUC_lexicon_tags_6 },
    { 0xbaa4bb8c, SUC_lexicon_tags_38 },
    { 0xfec4bb8e, SUC_lexicon_tags_37 },
    { 0x7504bb8f, SUC_lexicon_tags_17 },
    { 0x1184bb8f, SUC_lexicon_tags_37 },
    { 0x3e4bb8f, SUC_lexicon_tags_47 },
    { 0x5564bb92, SUC_lexicon_tags_2 },
    { 0xfc24bb93, SUC_lexicon_tags_27 },
    { 0x85e4bb94, SUC_lexicon_tags_7 },
    { 0x5c44bb8f, SUC_lexicon_tags_0 },
    { 0xbc44bb96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1c64bb98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca4bb9c, SUC_lexicon_tags_11 },
    { 0x3de4bb9d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc484bba1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf564bba4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4384bba6, SUC_lexicon_tags_28 },
    { 0xc824bba6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c4bbaa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c4bbad, SUC_lexicon_tags_7 },
    { 0x4b44bbae, SUC_lexicon_tags_4 },
    { 0xdfc4bbae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3144bbb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304bbb8, SUC_lexicon_tags_12 },
    { 0xdba4bbb9, SUC_lexicon_tags_29 },
    { 0xc724bbb9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2784bbc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc304bbc3, SUC_lexicon_tags_25 },
    { 0xbb64bbc3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcce4bbc6, SUC_lexicon_tags_17 },
    { 0x5524bbc7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8884bbcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d44bbd0, SUC_lexicon_tags_77 },
    { 0xb9e4bbd0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6404bbd4, SUC_lexicon_tags_39 },
    { 0xc424bbd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea4bbd8, SUC_lexicon_tags_8 },
    { 0xfdc4bbd9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5724bbdb, SUC_lexicon_tags_1 },
    { 0xe504bbdb, SUC_lexicon_tags_17 },
    { 0x6844bbdd, SUC_lexicon_tags_15 },
    { 0x3284bbde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4e4bbe0, SUC_lexicon_tags_49 },
    { 0x6a4bbe1, SUC_lexicon_tags_4 },
    { 0xe284bbe0, SUC_lexicon_tags_26 },
    { 0x7104bbe0, SUC_lexicon_tags_6 },
    { 0x8c04bbe4, SUC_lexicon_tags_45 },
    { 0xd4c4bbe5, SUC_lexicon_tags_55 },
    { 0x8104bbe6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3364bbe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a4bbf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea84bbf5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c4bbf8, SUC_lexicon_tags_57 },
    { 0xde4bbf9, SUC_lexicon_tags_38 },
    { 0x6a04bbfa, SUC_lexicon_tags_6 },
    { 0x2ca4bbf9, SUC_lexicon_tags_19 },
    { 0x9144bbfb, SUC_lexicon_tags_14 },
    { 0x8464bbfd, SUC_lexicon_tags_11 },
    { 0xf4e4bbfb, SUC_lexicon_tags_25 },
    { 0xfc44bbfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3684bc03, SUC_lexicon_tags_25 },
    { 0x6004bc03, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7764bc06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3e4bc08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de4bc0c, SUC_lexicon_tags_11 },
    { 0x7564bc0d, SUC_lexicon_tags_38 },
    { 0x59e4bc0e, SUC_lexicon_tags_30 },
    { 0x3c44bc0e, SUC_lexicon_tags_15 },
    { 0xa684bc0f, SUC_lexicon_tags_2 },
    { 0x5a44bc11, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7384bc13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5e04bc15, SUC_lexicon_tags_17 },
    { 0xf044bc16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf44bc1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe964bc1f, SUC_lexicon_tags_19 },
    { 0x4824bc20, SUC_lexicon_tags_25 },
    { 0x1484bc21, SUC_lexicon_tags_33 },
    { 0xa3c4bc22, SUC_lexicon_tags_38 },
    { 0x4464bc23, SUC_lexicon_tags_48 },
    { 0xb444bc21, SUC_lexicon_tags_14 },
    { 0xf524bc22, SUC_lexicon_tags_17 },
    { 0x1de4bc26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e4bc2a, SUC_lexicon_tags_12 },
    { 0x45e4bc2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd8a4bc2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x36e4bc2f, SUC_lexicon_tags_26 },
    { 0x9c84bc30, SUC_lexicon_tags_28 },
    { 0x1944bc31, SUC_lexicon_tags_84 },
    { 0x5e44bc30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c4bc35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe444bc37, SUC_lexicon_tags_38 },
    { 0xcd04bc37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc344bc3a, SUC_lexicon_tags_63 },
    { 0xad44bc3b, SUC_lexicon_tags_16 },
    { 0xfb44bc3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f64bc3f, SUC_lexicon_tags_28 },
    { 0x1ae4bc3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfe4bc45, SUC_lexicon_tags_8 },
    { 0x4da4bc46, SUC_lexicon_tags_37 },
    { 0xf3a4bc47, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3304bc49, SUC_lexicon_tags_4 },
    { 0x3804bc4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb44bc4c, SUC_lexicon_tags_62 },
    { 0x5aa4bc4d, SUC_lexicon_tags_7 },
    { 0x73a4bc4e, SUC_lexicon_tags_8 },
    { 0xea4bc4d, SUC_lexicon_tags_33 },
    { 0x7064bc50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a04bc54, SUC_lexicon_tags_7 },
    { 0x24bc55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4344bc5c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee84bc60, SUC_lexicon_tags_287 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f24bc67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2184bc69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62a4bc6b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a84bc6e, SUC_lexicon_tags_23 },
    { 0x5524bc6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff04bc74, SUC_lexicon_tags_17 },
    { 0xf44bc74, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c4bc78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5f04bc7a, SUC_lexicon_tags_2 },
    { 0x90a4bc7a, SUC_lexicon_tags_11 },
    { 0xe8e4bc7a, SUC_lexicon_tags_22 },
    { 0x2384bc7d, SUC_lexicon_tags_293 },
    { 0x1b84bc7d, SUC_lexicon_tags_7 },
    { 0xd084bc7f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2344bc81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144bc86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xda4bc88, SUC_lexicon_tags_37 },
    { 0x4644bc89, SUC_lexicon_tags_17 },
    { 0xd224bc8a, SUC_lexicon_tags_8 },
    { 0xb144bc8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91a4bc8d, SUC_lexicon_tags_17 },
    { 0xa804bc8e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c4bc92, SUC_lexicon_tags_63 },
    { 0x5984bc93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5944bc95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a4bc9c, SUC_lexicon_tags_23 },
    { 0x7384bc9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8884bc9f, SUC_lexicon_tags_25 },
    { 0xede4bca0, SUC_lexicon_tags_2 },
    { 0x624bc9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb44bca5, SUC_lexicon_tags_27 },
    { 0x6bc4bca6, SUC_lexicon_tags_2 },
    { 0x59c4bca6, SUC_lexicon_tags_25 },
    { 0x4d44bca8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe4bcaf, SUC_lexicon_tags_48 },
    { 0x2fc4bcb0, SUC_lexicon_tags_179 },
    { 0x3e24bcb0, SUC_lexicon_tags_0 },
    { 0xd5a4bcaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6724bcb4, SUC_lexicon_tags_17 },
    { 0x91a4bcb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6324bcb7, SUC_lexicon_tags_47 },
    { 0xf7c4bcb8, SUC_lexicon_tags_35 },
    { 0x2104bcb9, SUC_lexicon_tags_2 },
    { 0x1b64bcba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be4bcc2, SUC_lexicon_tags_0 },
    { 0x544bcc2, SUC_lexicon_tags_17 },
    { 0x43c4bcc4, SUC_lexicon_tags_37 },
    { 0x2f04bcc5, SUC_lexicon_tags_37 },
    { 0xe4bcc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac44bccb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc4bcce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c4bcd1, SUC_lexicon_tags_15 },
    { 0xb3c4bcd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a4bcd5, SUC_lexicon_tags_23 },
    { 0x184bcd6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2664bcdb, SUC_lexicon_tags_17 },
    { 0xa04bcdc, SUC_lexicon_tags_37 },
    { 0xc4a4bcdb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x164bce0, SUC_lexicon_tags_56 },
    { 0x99c4bce1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a4bce4, SUC_lexicon_tags_508 },
    { 0xa944bce4, SUC_lexicon_tags_11 },
    { 0xc284bce4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bce9, SUC_lexicon_tags_7 },
    { 0x5d84bcea, SUC_lexicon_tags_8 },
    { 0xd4c4bcea, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c04bcf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c4bcf8, SUC_lexicon_tags_37 },
    { 0xca64bcf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444bcfe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x40a4bd00, SUC_lexicon_tags_49 },
    { 0xb2c4bd01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde4bd03, SUC_lexicon_tags_12 },
    { 0xaba4bd04, SUC_lexicon_tags_29 },
    { 0xf124bd05, SUC_lexicon_tags_29 },
    { 0xbe4bd06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa964bd09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5444bd0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3084bd0f, SUC_lexicon_tags_12 },
    { 0xef04bd0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4304bd13, SUC_lexicon_tags_14 },
    { 0xb004bd14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3104bd17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a4bd19, SUC_lexicon_tags_11 },
    { 0x8f44bd19, SUC_lexicon_tags_17 },
    { 0x9284bd1b, SUC_lexicon_tags_16 },
    { 0xdd44bd1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6784bd21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xae84bd24, SUC_lexicon_tags_1 },
    { 0xd4e4bd25, SUC_lexicon_tags_12 },
    { 0xf3e4bd25, SUC_lexicon_tags_46 },
    { 0xc6a4bd25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c4bd2d, SUC_lexicon_tags_31 },
    { 0x5864bd2e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc4bd3a, SUC_lexicon_tags_37 },
    { 0xd4c4bd3b, SUC_lexicon_tags_32 },
    { 0xb964bd3a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xad84bd3e, SUC_lexicon_tags_1 },
    { 0x7964bd3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba44bd45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5724bd51, SUC_lexicon_tags_16 },
    { 0x1104bd52, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2204bd55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5324bd58, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52c4bd5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ea4bd5c, SUC_lexicon_tags_11 },
    { 0x7804bd5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4bd60, SUC_lexicon_tags_15 },
    { 0x2d84bd61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade4bd68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d64bd6d, SUC_lexicon_tags_7 },
    { 0xfee4bd6e, SUC_lexicon_tags_7 },
    { 0x2304bd6e, SUC_lexicon_tags_11 },
    { 0x8a04bd6d, SUC_lexicon_tags_63 },
    { 0x30c4bd6d, SUC_lexicon_tags_12 },
    { 0x8664bd72, SUC_lexicon_tags_7 },
    { 0xcc24bd73, SUC_lexicon_tags_12 },
    { 0xe3e4bd74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe984bd77, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7784bd79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6724bd7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a64bd81, SUC_lexicon_tags_29 },
    { 0x4884bd82, SUC_lexicon_tags_26 },
    { 0x2284bd83, SUC_lexicon_tags_17 },
    { 0x4464bd84, SUC_lexicon_tags_47 },
    { 0xb084bd84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e4bd89, SUC_lexicon_tags_12 },
    { 0x86e4bd89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7744bd94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c64bd97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6204bd99, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x9104bd9c, SUC_lexicon_tags_15 },
    { 0x8404bd9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a44bda3, SUC_lexicon_tags_65 },
    { 0xf544bda4, SUC_lexicon_tags_2 },
    { 0x8924bda4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x844bda7, SUC_lexicon_tags_1 },
    { 0x2a24bda8, SUC_lexicon_tags_12 },
    { 0x9484bda9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf724bdac, SUC_lexicon_tags_23 },
    { 0xb5c4bdac, SUC_lexicon_tags_125 },
    { 0xece4bdac, SUC_lexicon_tags_2 },
    { 0x6c04bdaf, SUC_lexicon_tags_1 },
    { 0x9e4bdb0, SUC_lexicon_tags_37 },
    { 0xa944bdae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea4bdb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a4bdbd, SUC_lexicon_tags_12 },
    { 0x8684bdbe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e04bdc2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e44bdc4, SUC_lexicon_tags_37 },
    { 0xb1a4bdc4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6444bdc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c4bdcc, SUC_lexicon_tags_50 },
    { 0x8524bdcc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc364bdd2, SUC_lexicon_tags_37 },
    { 0x13e4bdd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea84bdda, SUC_lexicon_tags_25 },
    { 0x1764bddb, SUC_lexicon_tags_29 },
    { 0x97e4bddc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a64bde3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7784bdea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9524bded, SUC_lexicon_tags_15 },
    { 0xb564bdee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc4bdf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc204bdf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4bdf7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf244bdfb, SUC_lexicon_tags_68 },
    { 0xe9e4bdfb, SUC_lexicon_tags_29 },
    { 0x54c4bdfd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c4be02, SUC_lexicon_tags_7 },
    { 0x5224be03, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbbc4be05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e4be08, SUC_lexicon_tags_25 },
    { 0xc6c4be09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a04be0c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd104be0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f84be12, SUC_lexicon_tags_49 },
    { 0xd4e4be13, SUC_lexicon_tags_37 },
    { 0xcfc4be14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f24be16, SUC_lexicon_tags_63 },
    { 0x2244be17, SUC_lexicon_tags_17 },
    { 0x4c4be17, SUC_lexicon_tags_46 },
    { 0x4484be19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c64be1b, SUC_lexicon_tags_2 },
    { 0x7aa4be1c, SUC_lexicon_tags_15 },
    { 0x7c84be1d, SUC_lexicon_tags_37 },
    { 0xeb64be1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c64be22, SUC_lexicon_tags_47 },
    { 0x80a4be23, SUC_lexicon_tags_17 },
    { 0xa384be24, SUC_lexicon_tags_12 },
    { 0xa04be25, SUC_lexicon_tags_6 },
    { 0xef84be22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e4be2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a64be2d, SUC_lexicon_tags_47 },
    { 0x64a4be2d, SUC_lexicon_tags_43 },
    { 0xff84be2f, SUC_lexicon_tags_87 },
    { 0x80e4be30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84e4be32, SUC_lexicon_tags_45 },
    { 0x3d84be33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e44be3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde4be3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9f84be3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c04be41, SUC_lexicon_tags_33 },
    { 0x1ac4be41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7744be44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e4be48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2504be4c, SUC_lexicon_tags_6 },
    { 0x3c64be4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4fa4be4f, SUC_lexicon_tags_43 },
    { 0x7584be50, SUC_lexicon_tags_26 },
    { 0xca84be4f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf964be54, SUC_lexicon_tags_27 },
    { 0x1104be55, SUC_lexicon_tags_0 },
    { 0xcc44be56, SUC_lexicon_tags_63 },
    { 0xc4a4be54, SUC_lexicon_tags_27 },
    { 0x6de4be58, SUC_lexicon_tags_11 },
    { 0xbe64be57, SUC_lexicon_tags_33 },
    { 0x9344be54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3644be63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe5e4be65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2c4be67, SUC_lexicon_tags_12 },
    { 0xc244be68, SUC_lexicon_tags_62 },
    { 0x5ca4be69, SUC_lexicon_tags_25 },
    { 0xf004be67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba4be6d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e64be76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de4be7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5904be7f, SUC_lexicon_tags_27 },
    { 0x7924be80, SUC_lexicon_tags_2 },
    { 0xac84be81, SUC_lexicon_tags_10 },
    { 0x40a4be81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0c4be84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7164be86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c4be8a, SUC_lexicon_tags_25 },
    { 0xc384be8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x824be8e, SUC_lexicon_tags_12 },
    { 0x3084be8f, SUC_lexicon_tags_49 },
    { 0xc644be8f, SUC_lexicon_tags_1 },
    { 0xe6e4be91, SUC_lexicon_tags_57 },
    { 0xa644be92, SUC_lexicon_tags_25 },
    { 0xb464be93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47e4be95, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb024be97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb404be9b, SUC_lexicon_tags_25 },
    { 0x39a4be9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f04be9e, SUC_lexicon_tags_49 },
    { 0x9e04be9e, SUC_lexicon_tags_11 },
    { 0xc024bea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6c4bea2, SUC_lexicon_tags_30 },
    { 0x97a4bea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa064beaa, SUC_lexicon_tags_25 },
    { 0x4144beab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a4beb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce4beb2, SUC_lexicon_tags_0 },
    { 0x81a4beb3, SUC_lexicon_tags_43 },
    { 0x5644beb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e4beb8, SUC_lexicon_tags_37 },
    { 0x48e4beb9, SUC_lexicon_tags_1 },
    { 0x7f04beba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844bebf, SUC_lexicon_tags_77 },
    { 0xc04bec0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d04bec6, SUC_lexicon_tags_37 },
    { 0x3964bec7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6004bec9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2744becd, SUC_lexicon_tags_17 },
    { 0x5364bece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4644bed0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x49e4bed2, SUC_lexicon_tags_48 },
    { 0x104bed3, SUC_lexicon_tags_60 },
    { 0xea64bed2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a44bed6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc04bedc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeec4bede, SUC_lexicon_tags_37 },
    { 0xf244bedf, SUC_lexicon_tags_11 },
    { 0xe5e4bedf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e24bee2, SUC_lexicon_tags_29 },
    { 0xc724bee3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa584beea, SUC_lexicon_tags_47 },
    { 0x1e44beea, SUC_lexicon_tags_25 },
    { 0x8724beec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d84beee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5704bef2, SUC_lexicon_tags_37 },
    { 0x4044bef3, SUC_lexicon_tags_11 },
    { 0x2f44bef4, SUC_lexicon_tags_98 },
    { 0x9d84bef4, SUC_lexicon_tags_19 },
    { 0x2844bef4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe0a4bef8, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0x8d24befa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6a4befc, SUC_lexicon_tags_25 },
    { 0xb3e4befd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8784bf06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a04bf0b, SUC_lexicon_tags_12 },
    { 0xc064bf0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf664bf0e, SUC_lexicon_tags_29 },
    { 0x5724bf0e, SUC_lexicon_tags_46 },
    { 0xf704bf10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd704bf13, SUC_lexicon_tags_118 },
    { 0x43c4bf13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea4bf17, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5324bf1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a4bf1e, SUC_lexicon_tags_37 },
    { 0x46e4bf1f, SUC_lexicon_tags_101 },
    { 0x82c4bf1f, SUC_lexicon_tags_7 },
    { 0xf7e4bf21, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c4bf24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd24bf28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c4bf2c, SUC_lexicon_tags_25 },
    { 0x6be4bf2c, SUC_lexicon_tags_39 },
    { 0x9904bf2e, SUC_lexicon_tags_16 },
    { 0x184bf2f, SUC_lexicon_tags_3 },
    { 0xefc4bf2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc04bf32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc04bf34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e4bf37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4c4bf39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7aa4bf3b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e4bf3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b64bf40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x95a4bf42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb704bf44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae4bf47, SUC_lexicon_tags_33 },
    { 0x5104bf48, SUC_lexicon_tags_55 },
    { 0x544bf49, SUC_lexicon_tags_0 },
    { 0x5604bf4a, SUC_lexicon_tags_11 },
    { 0x3944bf49, SUC_lexicon_tags_23 },
    { 0xb584bf49, SUC_lexicon_tags_1 },
    { 0xd0e4bf48, SUC_lexicon_tags_12 },
    { 0x74c4bf4e, SUC_lexicon_tags_80 },
    { 0x25a4bf4f, SUC_lexicon_tags_2 },
    { 0x9a04bf49, SUC_lexicon_tags_22 },
    { 0x53c4bf51, SUC_lexicon_tags_3 },
    { 0x7e4bf52, SUC_lexicon_tags_1148 },
    { 0x8324bf50, SUC_lexicon_tags_11 },
    { 0x9da4bf4c, SUC_lexicon_tags_15 },
    { 0x4f64bf55, SUC_lexicon_tags_37 },
    { 0x7d44bf47, SUC_lexicon_tags_12 },
    { 0x53e4bf57, SUC_lexicon_tags_33 },
    { 0xb204bf58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04bf5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafe4bf5d, SUC_lexicon_tags_71 },
    { 0x84e4bf5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc84bf61, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x7544bf63, SUC_lexicon_tags_37 },
    { 0x8184bf64, SUC_lexicon_tags_47 },
    { 0x25a4bf65, SUC_lexicon_tags_15 },
    { 0x8d64bf65, SUC_lexicon_tags_68 },
    { 0x1c84bf67, SUC_lexicon_tags_11 },
    { 0x664bf64, SUC_lexicon_tags_12 },
    { 0x25a4bf67, SUC_lexicon_tags_25 },
    { 0x8c84bf63, SUC_lexicon_tags_38 },
    { 0x7a4bf6b, SUC_lexicon_tags_7 },
    { 0xde84bf6b, SUC_lexicon_tags_39 },
    { 0x7144bf6b, SUC_lexicon_tags_19 },
    { 0x1da4bf68, SUC_lexicon_tags_303 },
    { 0xd544bf6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d44bf71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59a4bf73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c4bf77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e4bf7b, SUC_lexicon_tags_68 },
    { 0x5004bf7c, SUC_lexicon_tags_38 },
    { 0x40c4bf7c, SUC_lexicon_tags_25 },
    { 0x9004bf7b, SUC_lexicon_tags_14 },
    { 0x74c4bf7f, SUC_lexicon_tags_37 },
    { 0xbcc4bf80, SUC_lexicon_tags_12 },
    { 0x9d84bf81, SUC_lexicon_tags_61 },
    { 0x8d64bf80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe4bf86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c4bf89, SUC_lexicon_tags_37 },
    { 0x2e4bf8a, SUC_lexicon_tags_11 },
    { 0x3044bf8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae84bf8e, SUC_lexicon_tags_29 },
    { 0x5164bf8f, SUC_lexicon_tags_38 },
    { 0x3024bf90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c4bf93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6884bf95, SUC_lexicon_tags_31 },
    { 0xfdc4bf95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9844bf9a, SUC_lexicon_tags_2 },
    { 0x8884bf9a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3064bf9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b04bfa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a4bfa7, SUC_lexicon_tags_15 },
    { 0xdce4bfa7, SUC_lexicon_tags_33 },
    { 0x31e4bfa9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c4bfae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5444bfb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a44bfb8, SUC_lexicon_tags_29 },
    { 0x1884bfb9, SUC_lexicon_tags_7 },
    { 0x5764bfba, SUC_lexicon_tags_19 },
    { 0xbba4bfb9, SUC_lexicon_tags_12 },
    { 0xa9c4bfbc, SUC_lexicon_tags_37 },
    { 0x7a64bfbd, SUC_lexicon_tags_2 },
    { 0x5964bfb9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2624bfc1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x89c4bfc3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6324bfc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabc4bfc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e4bfcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c4bfd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41c4bfd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ee4bfd5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x664bfd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5c4bfd9, SUC_lexicon_tags_19 },
    { 0x8104bfda, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc4bfdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9104bfe1, SUC_lexicon_tags_7 },
    { 0x8244bfe1, SUC_lexicon_tags_25 },
    { 0xac24bfe3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4844bfe9, SUC_lexicon_tags_30 },
    { 0xf884bfea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c4bff3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf44bff5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80c4bff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb604bffa, SUC_lexicon_tags_66 },
    { 0x57c4bffa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e24bffe, SUC_lexicon_tags_7 },
    { 0xd024bffe, SUC_lexicon_tags_7 },
    { 0x7044c000, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x74e4c002, SUC_lexicon_tags_33 },
    { 0x23e4c003, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x63a4c005, SUC_lexicon_tags_25 },
    { 0x76c4c005, SUC_lexicon_tags_12 },
    { 0xe6c4c007, SUC_lexicon_tags_11 },
    { 0x89e4c008, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e4c00b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xade4c00d, SUC_lexicon_tags_38 },
    { 0x3b44c00d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1344c012, SUC_lexicon_tags_17 },
    { 0xb3a4c013, SUC_lexicon_tags_112 },
    { 0x9c24c012, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1364c018, SUC_lexicon_tags_8 },
    { 0x164c018, SUC_lexicon_tags_57 },
    { 0x4804c01a, SUC_lexicon_tags_65 },
    { 0x4b84c01b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab44c01d, SUC_lexicon_tags_6 },
    { 0x7504c01d, SUC_lexicon_tags_29 },
    { 0xe504c01f, SUC_lexicon_tags_38 },
    { 0x7b44c01f, SUC_lexicon_tags_17 },
    { 0x85c4c021, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc4c035, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3624c039, SUC_lexicon_tags_25 },
    { 0x4584c03a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf24c03d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e4c043, SUC_lexicon_tags_28 },
    { 0x2704c044, SUC_lexicon_tags_47 },
    { 0x2fc4c045, SUC_lexicon_tags_36 },
    { 0x49e4c045, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc64c049, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdb04c04b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3284c04e, SUC_lexicon_tags_17 },
    { 0x4644c04f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4c053, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9324c056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1904c05b, SUC_lexicon_tags_11 },
    { 0xd8c4c05c, SUC_lexicon_tags_9 },
    { 0x25a4c05b, SUC_lexicon_tags_7 },
    { 0x3c84c05e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a4c065, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x6224c067, SUC_lexicon_tags_25 },
    { 0x3024c068, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d64c06f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x82c4c071, SUC_lexicon_tags_23 },
    { 0x7cc4c072, SUC_lexicon_tags_7 },
    { 0xe4a4c073, SUC_lexicon_tags_7 },
    { 0xda64c073, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3c4c076, SUC_lexicon_tags_17 },
    { 0xfec4c077, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e04c07d, SUC_lexicon_tags_15 },
    { 0xde84c07e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb064c080, SUC_lexicon_tags_259 },
    { 0x8c4c081, SUC_lexicon_tags_7 },
    { 0xb1c4c081, SUC_lexicon_tags_33 },
    { 0xbfe4c083, SUC_lexicon_tags_12 },
    { 0xba24c083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e4c08a, SUC_lexicon_tags_31 },
    { 0x2fc4c08a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce64c08e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a4c092, SUC_lexicon_tags_0 },
    { 0xfee4c092, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2164c096, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1364c09a, SUC_lexicon_tags_25 },
    { 0x8104c09b, SUC_lexicon_tags_25 },
    { 0xc0c4c09b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd044c09e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d24c0a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcc24c0a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd264c0a8, SUC_lexicon_tags_141 },
    { 0xaae4c0a9, SUC_lexicon_tags_29 },
    { 0x9c24c0aa, SUC_lexicon_tags_68 },
    { 0x2bc4c0ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4024c0ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f64c0b1, SUC_lexicon_tags_5 },
    { 0x21a4c0b2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcd44c0b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd724c0b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf764c0b9, SUC_lexicon_tags_12 },
    { 0x8944c0ba, SUC_lexicon_tags_8 },
    { 0x73a4c0bb, SUC_lexicon_tags_11 },
    { 0x5184c0bc, SUC_lexicon_tags_31 },
    { 0xc544c0b9, SUC_lexicon_tags_37 },
    { 0xfca4c0be, SUC_lexicon_tags_14 },
    { 0x1ae4c0bf, SUC_lexicon_tags_63 },
    { 0x5504c0bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca84c0c9, SUC_lexicon_tags_6 },
    { 0x4764c0ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa164c0ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5664c0d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c4c0d7, SUC_lexicon_tags_17 },
    { 0x26e4c0d8, SUC_lexicon_tags_7 },
    { 0x9ec4c0d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c4c0de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2a4c0e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e04c0e2, SUC_lexicon_tags_49 },
    { 0x26c4c0e3, SUC_lexicon_tags_56 },
    { 0x9424c0e3, SUC_lexicon_tags_18 },
    { 0x1224c0e3, SUC_lexicon_tags_14 },
    { 0x2e04c0e6, SUC_lexicon_tags_1 },
    { 0x1e64c0e7, SUC_lexicon_tags_25 },
    { 0x9c24c0e3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xc404c0ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb084c0ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x64a4c0ef, SUC_lexicon_tags_43 },
    { 0x2344c0ef, SUC_lexicon_tags_12 },
    { 0xa4a4c0f1, SUC_lexicon_tags_2 },
    { 0x6c44c0f2, SUC_lexicon_tags_7 },
    { 0x7b24c0f3, SUC_lexicon_tags_47 },
    { 0x4644c0f4, SUC_lexicon_tags_17 },
    { 0x2844c0f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe024c0f8, SUC_lexicon_tags_2 },
    { 0x4384c0f8, SUC_lexicon_tags_14 },
    { 0x11a4c0f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e24c0ff, SUC_lexicon_tags_47 },
    { 0x9604c0ff, SUC_lexicon_tags_1 },
    { 0x2424c0ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc244c104, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc624c108, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac4c10d, SUC_lexicon_tags_15 },
    { 0x1504c10e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x22a4c110, SUC_lexicon_tags_17 },
    { 0x84c110, SUC_lexicon_tags_49 },
    { 0x91e4c110, SUC_lexicon_tags_7 },
    { 0xe304c111, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4724c116, SUC_lexicon_tags_2 },
    { 0x8e04c116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5164c119, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5244c11b, SUC_lexicon_tags_37 },
    { 0xf24c11c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c4c121, SUC_lexicon_tags_26 },
    { 0x4c24c121, SUC_lexicon_tags_1 },
    { 0x45e4c123, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf064c126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd524c128, SUC_lexicon_tags_11 },
    { 0xb184c129, SUC_lexicon_tags_49 },
    { 0xe624c129, SUC_lexicon_tags_63 },
    { 0x6184c128, SUC_lexicon_tags_12 },
    { 0xb144c129, SUC_lexicon_tags_49 },
    { 0x3ae4c12c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a4c131, SUC_lexicon_tags_1 },
    { 0xfde4c132, SUC_lexicon_tags_77 },
    { 0xe9e4c132, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9664c138, SUC_lexicon_tags_87 },
    { 0x93c4c139, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca4c13c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6404c13e, SUC_lexicon_tags_129 },
    { 0xbe44c13f, SUC_lexicon_tags_19 },
    { 0xb784c140, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb664c142, SUC_lexicon_tags_27 },
    { 0x6a24c143, SUC_lexicon_tags_49 },
    { 0x1d84c143, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4164c146, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb604c14b, SUC_lexicon_tags_63 },
    { 0x1c64c14c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad24c14f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe204c152, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf364c157, SUC_lexicon_tags_37 },
    { 0x86a4c158, SUC_lexicon_tags_64 },
    { 0x7344c158, SUC_lexicon_tags_8 },
    { 0x4944c157, SUC_lexicon_tags_6 },
    { 0x5844c15b, SUC_lexicon_tags_12 },
    { 0x6f44c15c, SUC_lexicon_tags_12 },
    { 0x1624c15d, SUC_lexicon_tags_0 },
    { 0x7284c15b, SUC_lexicon_tags_39 },
    { 0x39e4c15c, SUC_lexicon_tags_30 },
    { 0x2824c15d, SUC_lexicon_tags_10 },
    { 0x5ae4c15f, SUC_lexicon_tags_1 },
    { 0xc624c158, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe4c169, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a4c16c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaee4c16e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd304c174, SUC_lexicon_tags_23 },
    { 0x8ac4c175, SUC_lexicon_tags_23 },
    { 0xe9a4c176, SUC_lexicon_tags_1493 },
    { 0, NULL },
    { 0xa224c178, SUC_lexicon_tags_27 },
    { 0xd844c179, SUC_lexicon_tags_15 },
    { 0x46e4c179, SUC_lexicon_tags_1 },
    { 0x6984c178, SUC_lexicon_tags_38 },
    { 0xa764c17b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd24c184, SUC_lexicon_tags_37 },
    { 0x94c4c185, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1324c187, SUC_lexicon_tags_19 },
    { 0x76c4c188, SUC_lexicon_tags_64 },
    { 0x4da4c189, SUC_lexicon_tags_44 },
    { 0x6424c18a, SUC_lexicon_tags_25 },
    { 0xce84c18b, SUC_lexicon_tags_7 },
    { 0x99c4c18a, SUC_lexicon_tags_28 },
    { 0x4ee4c18d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ae4c18f, SUC_lexicon_tags_11 },
    { 0x96e4c190, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda4c196, SUC_lexicon_tags_64 },
    { 0x1404c197, SUC_lexicon_tags_15 },
    { 0xec64c198, SUC_lexicon_tags_33 },
    { 0x9ce4c199, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c4c1a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3124c1a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1364c1a5, SUC_lexicon_tags_7 },
    { 0x6584c1a6, SUC_lexicon_tags_93 },
    { 0x9524c1a7, SUC_lexicon_tags_15 },
    { 0xb04c1a7, SUC_lexicon_tags_60 },
    { 0x6ba4c1a9, SUC_lexicon_tags_43 },
    { 0xce4c1aa, SUC_lexicon_tags_25 },
    { 0xb324c1a7, SUC_lexicon_tags_79 },
    { 0x1c4c1ac, SUC_lexicon_tags_49 },
    { 0x3da4c1ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa964c1b0, SUC_lexicon_tags_4 },
    { 0x6944c1b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf04c1bd, SUC_lexicon_tags_10 },
    { 0x4384c1be, SUC_lexicon_tags_1 },
    { 0x46e4c1bf, SUC_lexicon_tags_37 },
    { 0x5da4c1c0, SUC_lexicon_tags_8 },
    { 0xf64c1c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a4c1c5, SUC_lexicon_tags_12 },
    { 0xb104c1c5, SUC_lexicon_tags_17 },
    { 0xb5c4c1c7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1744c1cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae4c1d3, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b24c1d8, SUC_lexicon_tags_8 },
    { 0x1b84c1d8, SUC_lexicon_tags_37 },
    { 0x1b04c1da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe244c1dc, SUC_lexicon_tags_12 },
    { 0xae4c1dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d64c1df, SUC_lexicon_tags_31 },
    { 0x6c44c1e0, SUC_lexicon_tags_11 },
    { 0xf224c1e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7124c1e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9284c1e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a4c1e8, SUC_lexicon_tags_7 },
    { 0xc4c4c1e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c4c1ed, SUC_lexicon_tags_17 },
    { 0xa8a4c1ee, SUC_lexicon_tags_28 },
    { 0x19a4c1ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5fe4c1f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e4c1f7, SUC_lexicon_tags_49 },
    { 0xeac4c1f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1d04c1fa, SUC_lexicon_tags_17 },
    { 0xb8a4c1fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da4c200, SUC_lexicon_tags_60 },
    { 0xeee4c201, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e64c203, SUC_lexicon_tags_17 },
    { 0xcfc4c204, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x65e4c206, SUC_lexicon_tags_17 },
    { 0xca24c206, SUC_lexicon_tags_3 },
    { 0x98e4c208, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd724c20b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6784c20d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc4c210, SUC_lexicon_tags_35 },
    { 0xe544c210, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c64c216, SUC_lexicon_tags_27 },
    { 0x1a4c216, SUC_lexicon_tags_48 },
    { 0x76e4c218, SUC_lexicon_tags_0 },
    { 0xa044c217, SUC_lexicon_tags_62 },
    { 0x77e4c21a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x804c21c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1104c21e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc24c220, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a24c224, SUC_lexicon_tags_25 },
    { 0xc784c224, SUC_lexicon_tags_84 },
    { 0x6a44c224, SUC_lexicon_tags_25 },
    { 0x24e4c225, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc944c229, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee84c230, SUC_lexicon_tags_25 },
    { 0x96a4c231, SUC_lexicon_tags_48 },
    { 0x90c4c230, SUC_lexicon_tags_14 },
    { 0x9024c233, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9264c235, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8944c237, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a4c23c, SUC_lexicon_tags_13 },
    { 0x80a4c23c, SUC_lexicon_tags_6 },
    { 0x8524c23e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3384c241, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1be4c243, SUC_lexicon_tags_31 },
    { 0xa1e4c243, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e04c246, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c84c249, SUC_lexicon_tags_25 },
    { 0xe9e4c24a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7384c251, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da4c256, SUC_lexicon_tags_39 },
    { 0xeda4c256, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2424c25a, SUC_lexicon_tags_33 },
    { 0x58e4c25b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c4c25e, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0xf3a4c260, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf04c262, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3144c264, SUC_lexicon_tags_25 },
    { 0xa924c265, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb544c268, SUC_lexicon_tags_17 },
    { 0xde84c268, SUC_lexicon_tags_25 },
    { 0x88e4c26a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc4c26f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x664c271, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2664c276, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a4c27d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c4c282, SUC_lexicon_tags_7 },
    { 0xd264c282, SUC_lexicon_tags_3 },
    { 0x7384c284, SUC_lexicon_tags_191 },
    { 0xac24c285, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb984c288, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c4c28c, SUC_lexicon_tags_37 },
    { 0xef44c28c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4884c28f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3484c291, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf224c293, SUC_lexicon_tags_25 },
    { 0x7c44c294, SUC_lexicon_tags_12 },
    { 0x8904c295, SUC_lexicon_tags_2 },
    { 0x3684c296, SUC_lexicon_tags_17 },
    { 0x6f84c297, SUC_lexicon_tags_27 },
    { 0xc4a4c298, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3e4c29a, SUC_lexicon_tags_63 },
    { 0xd744c29b, SUC_lexicon_tags_11 },
    { 0x2f44c29c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc384c29e, SUC_lexicon_tags_9 },
    { 0xf2c4c29f, SUC_lexicon_tags_68 },
    { 0xbea4c29f, SUC_lexicon_tags_49 },
    { 0x6744c2a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3844c2a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e4c2ab, SUC_lexicon_tags_1 },
    { 0xf784c2ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc4c2b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb464c2b6, SUC_lexicon_tags_39 },
    { 0xd164c2b7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x22c4c2b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b04c2c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd904c2c7, SUC_lexicon_tags_17 },
    { 0xc144c2c8, SUC_lexicon_tags_15 },
    { 0x1724c2c9, SUC_lexicon_tags_7 },
    { 0x3704c2c7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x7664c2cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf464c2cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de4c2d2, SUC_lexicon_tags_30 },
    { 0x2ca4c2d2, SUC_lexicon_tags_7 },
    { 0x5a44c2d3, SUC_lexicon_tags_6 },
    { 0x78c4c2d5, SUC_lexicon_tags_17 },
    { 0xf444c2d4, SUC_lexicon_tags_37 },
    { 0x1dc4c2d4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc224c2dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4924c2df, SUC_lexicon_tags_7 },
    { 0xc2e4c2e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8884c2e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8704c2e5, SUC_lexicon_tags_63 },
    { 0xaf84c2e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc4c2e9, SUC_lexicon_tags_7 },
    { 0x4c24c2e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec4c2ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde44c2f3, SUC_lexicon_tags_37 },
    { 0x8ac4c2f3, SUC_lexicon_tags_86 },
    { 0x6ca4c2f5, SUC_lexicon_tags_2 },
    { 0x37e4c2f3, SUC_lexicon_tags_72 },
    { 0x4304c2f3, SUC_lexicon_tags_1 },
    { 0x90c4c2f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a4c2fd, SUC_lexicon_tags_11 },
    { 0xd2c4c2fe, SUC_lexicon_tags_23 },
    { 0x66c4c2fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4c304, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e84c308, SUC_lexicon_tags_39 },
    { 0xc744c309, SUC_lexicon_tags_17 },
    { 0xab04c309, SUC_lexicon_tags_23 },
    { 0x98e4c30b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e04c30f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e4c312, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc084c315, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c64c319, SUC_lexicon_tags_23 },
    { 0xd224c31a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04c31e, SUC_lexicon_tags_25 },
    { 0x18c4c31e, SUC_lexicon_tags_11 },
    { 0xc6a4c320, SUC_lexicon_tags_11 },
    { 0x9be4c321, SUC_lexicon_tags_12 },
    { 0x1d24c322, SUC_lexicon_tags_79 },
    { 0xd184c323, SUC_lexicon_tags_37 },
    { 0x2dc4c323, SUC_lexicon_tags_26 },
    { 0x3704c324, SUC_lexicon_tags_37 },
    { 0x3184c326, SUC_lexicon_tags_17 },
    { 0x7fe4c326, SUC_lexicon_tags_17 },
    { 0x9684c328, SUC_lexicon_tags_10 },
    { 0xf904c321, SUC_lexicon_tags_30 },
    { 0x3964c32a, SUC_lexicon_tags_16 },
    { 0xb7c4c32b, SUC_lexicon_tags_3 },
    { 0xbe4c323, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c4c32f, SUC_lexicon_tags_12 },
    { 0xbb44c330, SUC_lexicon_tags_60 },
    { 0x58a4c331, SUC_lexicon_tags_7 },
    { 0x2584c331, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5924c334, SUC_lexicon_tags_17 },
    { 0x3f84c335, SUC_lexicon_tags_23 },
    { 0x2624c336, SUC_lexicon_tags_13 },
    { 0x10a4c337, SUC_lexicon_tags_35 },
    { 0x5724c338, SUC_lexicon_tags_27 },
    { 0xd5a4c339, SUC_lexicon_tags_15 },
    { 0xcb44c337, SUC_lexicon_tags_7 },
    { 0x8224c335, SUC_lexicon_tags_11 },
    { 0x64c33c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe004c342, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde44c346, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a4c349, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb04c34c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe24c34e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a4c354, SUC_lexicon_tags_49 },
    { 0x9964c355, SUC_lexicon_tags_2 },
    { 0x57a4c356, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe084c362, SUC_lexicon_tags_34 },
    { 0x17c4c362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6324c365, SUC_lexicon_tags_1 },
    { 0x7f24c366, SUC_lexicon_tags_2 },
    { 0x7044c367, SUC_lexicon_tags_12 },
    { 0x2044c368, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa884c36f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8524c371, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2224c374, SUC_lexicon_tags_6 },
    { 0x484c374, SUC_lexicon_tags_17 },
    { 0x1c44c375, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9244c378, SUC_lexicon_tags_38 },
    { 0x6cc4c379, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e44c37b, SUC_lexicon_tags_1 },
    { 0xe484c37b, SUC_lexicon_tags_2 },
    { 0x3de4c37d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1c4c37f, SUC_lexicon_tags_15 },
    { 0x1d64c380, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc4c384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dc4c386, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2264c389, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa864c38f, SUC_lexicon_tags_37 },
    { 0x1c64c390, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1304c392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec24c399, SUC_lexicon_tags_39 },
    { 0xb6c4c39a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6aa4c39c, SUC_lexicon_tags_14 },
    { 0x4964c39c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2d24c39f, SUC_lexicon_tags_2 },
    { 0x95e4c39f, SUC_lexicon_tags_37 },
    { 0xdb44c39f, SUC_lexicon_tags_1655 },
    { 0xcbe4c3a2, SUC_lexicon_tags_17 },
    { 0x62a4c3a3, SUC_lexicon_tags_1 },
    { 0xd3e4c3a4, SUC_lexicon_tags_23 },
    { 0x7bc4c3a4, SUC_lexicon_tags_4 },
    { 0xb684c3a6, SUC_lexicon_tags_9 },
    { 0xee84c3a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c44c3ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4c3af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e4c3b1, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e44c3b5, SUC_lexicon_tags_32 },
    { 0xba64c3b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc064c3b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9524c3bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1464c3c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x68e4c3c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a4c3ca, SUC_lexicon_tags_14 },
    { 0x8964c3cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba4c3cd, SUC_lexicon_tags_8 },
    { 0xf984c3ce, SUC_lexicon_tags_48 },
    { 0xe4c4c3cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f04c3d2, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x5424c3d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2324c3d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x57e4c3db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d44c3df, SUC_lexicon_tags_224 },
    { 0x9f84c3e0, SUC_lexicon_tags_7 },
    { 0xa104c3e1, SUC_lexicon_tags_1 },
    { 0xf924c3df, SUC_lexicon_tags_55 },
    { 0x74c4c3e3, SUC_lexicon_tags_49 },
    { 0x2844c3e4, SUC_lexicon_tags_11 },
    { 0xf1e4c3e5, SUC_lexicon_tags_25 },
    { 0x5044c3e4, SUC_lexicon_tags_17 },
    { 0x2824c3e7, SUC_lexicon_tags_15 },
    { 0xcc84c3e8, SUC_lexicon_tags_25 },
    { 0x20a4c3e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd04c3ed, SUC_lexicon_tags_1 },
    { 0x38c4c3ee, SUC_lexicon_tags_45 },
    { 0x30e4c3ee, SUC_lexicon_tags_19 },
    { 0xcf44c3f0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a4c3f4, SUC_lexicon_tags_2 },
    { 0x1e04c3f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd04c401, SUC_lexicon_tags_23 },
    { 0x20a4c402, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6804c406, SUC_lexicon_tags_7 },
    { 0xbae4c407, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c4c40c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc04c40e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1444c410, SUC_lexicon_tags_15 },
    { 0x6644c410, SUC_lexicon_tags_25 },
    { 0x2c04c412, SUC_lexicon_tags_17 },
    { 0xd584c413, SUC_lexicon_tags_17 },
    { 0xe0e4c414, SUC_lexicon_tags_11 },
    { 0x4504c415, SUC_lexicon_tags_11 },
    { 0x4364c414, SUC_lexicon_tags_2 },
    { 0x7f24c414, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564c41b, SUC_lexicon_tags_26 },
    { 0xd864c41b, SUC_lexicon_tags_17 },
    { 0x3964c41b, SUC_lexicon_tags_2 },
    { 0xcea4c41c, SUC_lexicon_tags_3 },
    { 0xde4c41d, SUC_lexicon_tags_37 },
    { 0x6f84c41c, SUC_lexicon_tags_27 },
    { 0x1be4c41e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d64c426, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c4c42a, SUC_lexicon_tags_11 },
    { 0x74c4c42b, SUC_lexicon_tags_47 },
    { 0x1184c42c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7604c437, SUC_lexicon_tags_87 },
    { 0x7564c438, SUC_lexicon_tags_11 },
    { 0x8d44c437, SUC_lexicon_tags_9 },
    { 0xd7e4c43a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d04c43c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f64c43e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f64c442, SUC_lexicon_tags_33 },
    { 0xb0c4c443, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2644c445, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a4c44b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe024c450, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2864c457, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb64c45c, SUC_lexicon_tags_7 },
    { 0xfbc4c45d, SUC_lexicon_tags_57 },
    { 0xace4c45e, SUC_lexicon_tags_439 },
    { 0xaea4c45e, SUC_lexicon_tags_25 },
    { 0x80c4c460, SUC_lexicon_tags_37 },
    { 0x1ac4c460, SUC_lexicon_tags_21 },
    { 0x82c4c45e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c4c465, SUC_lexicon_tags_25 },
    { 0xbbe4c465, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea4c478, SUC_lexicon_tags_37 },
    { 0xb7e4c479, SUC_lexicon_tags_15 },
    { 0xffc4c479, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c4c47e, SUC_lexicon_tags_7 },
    { 0x9124c47f, SUC_lexicon_tags_57 },
    { 0xd204c480, SUC_lexicon_tags_47 },
    { 0x8ee4c47e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7984c483, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4f84c485, SUC_lexicon_tags_4 },
    { 0xabe4c486, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f84c488, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a4c48f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5084c491, SUC_lexicon_tags_62 },
    { 0x6184c492, SUC_lexicon_tags_160 },
    { 0xc844c492, SUC_lexicon_tags_15 },
    { 0x58e4c494, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c498, SUC_lexicon_tags_17 },
    { 0x3ce4c499, SUC_lexicon_tags_37 },
    { 0x22c4c49a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a4c49e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c4c4a3, SUC_lexicon_tags_15 },
    { 0xc4a4c4a3, SUC_lexicon_tags_34 },
    { 0x8b04c4a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba4c4a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5804c4ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e4c4ae, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304c4b2, SUC_lexicon_tags_11 },
    { 0xfe44c4b2, SUC_lexicon_tags_30 },
    { 0x5ea4c4b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcac4c4b6, SUC_lexicon_tags_27 },
    { 0x77c4c4b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e4c4bf, SUC_lexicon_tags_2 },
    { 0x4f04c4c0, SUC_lexicon_tags_15 },
    { 0x1844c4c0, SUC_lexicon_tags_37 },
    { 0xc484c4c2, SUC_lexicon_tags_17 },
    { 0x324c4c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbe4c4c5, SUC_lexicon_tags_17 },
    { 0x4824c4c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd44c4cb, SUC_lexicon_tags_66 },
    { 0x5e4c4cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6484c4cf, SUC_lexicon_tags_68 },
    { 0x20e4c4d0, SUC_lexicon_tags_1 },
    { 0x6a44c4d1, SUC_lexicon_tags_15 },
    { 0x9de4c4cf, SUC_lexicon_tags_15 },
    { 0x9d64c4d3, SUC_lexicon_tags_25 },
    { 0xc9a4c4d4, SUC_lexicon_tags_37 },
    { 0xc184c4d5, SUC_lexicon_tags_37 },
    { 0xcd44c4d5, SUC_lexicon_tags_15 },
    { 0xe9e4c4d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7024c4db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0a4c4dd, SUC_lexicon_tags_17 },
    { 0xc424c4de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac4c4e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c4c4e4, SUC_lexicon_tags_10 },
    { 0x1ba4c4e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a4c4e8, SUC_lexicon_tags_12 },
    { 0x2b24c4e9, SUC_lexicon_tags_8 },
    { 0xe824c4e9, SUC_lexicon_tags_0 },
    { 0x6504c4e9, SUC_lexicon_tags_63 },
    { 0xf464c4ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaee4c4ee, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x1a4c4f0, SUC_lexicon_tags_17 },
    { 0x23a4c4f1, SUC_lexicon_tags_23 },
    { 0x98e4c4f2, SUC_lexicon_tags_25 },
    { 0xfdc4c4f3, SUC_lexicon_tags_47 },
    { 0x3a64c4f4, SUC_lexicon_tags_12 },
    { 0x7e64c4f4, SUC_lexicon_tags_34 },
    { 0x38c4c4f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe064c4f8, SUC_lexicon_tags_79 },
    { 0x17e4c4f8, SUC_lexicon_tags_0 },
    { 0x96e4c4fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3024c4ff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbe64c501, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2744c503, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe4c506, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5804c508, SUC_lexicon_tags_15 },
    { 0x3a24c509, SUC_lexicon_tags_7 },
    { 0x8de4c50a, SUC_lexicon_tags_14 },
    { 0x60c4c50b, SUC_lexicon_tags_35 },
    { 0x8e44c50a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xbb24c50e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4c515, SUC_lexicon_tags_17 },
    { 0xa864c515, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8f24c518, SUC_lexicon_tags_79 },
    { 0xe504c519, SUC_lexicon_tags_1 },
    { 0x7da4c518, SUC_lexicon_tags_59 },
    { 0x8924c51b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5344c51d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a84c521, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3984c523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e4c526, SUC_lexicon_tags_2 },
    { 0xf104c526, SUC_lexicon_tags_37 },
    { 0xdc4c527, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd164c52c, SUC_lexicon_tags_19 },
    { 0xaf64c52d, SUC_lexicon_tags_55 },
    { 0x5ce4c52e, SUC_lexicon_tags_0 },
    { 0xbbe4c52e, SUC_lexicon_tags_1 },
    { 0xc084c530, SUC_lexicon_tags_25 },
    { 0xa604c531, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a64c536, SUC_lexicon_tags_7 },
    { 0x31a4c537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9244c53b, SUC_lexicon_tags_7 },
    { 0x2e44c53b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ec4c53e, SUC_lexicon_tags_1 },
    { 0xfae4c53f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b44c541, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe04c544, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e4c549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a4c54f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3e4c551, SUC_lexicon_tags_55 },
    { 0x2624c552, SUC_lexicon_tags_8 },
    { 0x23a4c552, SUC_lexicon_tags_68 },
    { 0xc204c554, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c4c557, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea4c55e, SUC_lexicon_tags_45 },
    { 0x1444c55e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a24c561, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c4c564, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb684c569, SUC_lexicon_tags_25 },
    { 0x6a4c569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85c4c56c, SUC_lexicon_tags_101 },
    { 0x9b64c56d, SUC_lexicon_tags_15 },
    { 0xe004c56e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644c572, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdf24c574, SUC_lexicon_tags_1 },
    { 0x1504c574, SUC_lexicon_tags_45 },
    { 0x5864c576, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ae4c578, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x94e4c57a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd244c57d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4c584, SUC_lexicon_tags_7 },
    { 0x4a84c585, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e4c589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c24c58b, SUC_lexicon_tags_19 },
    { 0x1dc4c58b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4984c58e, SUC_lexicon_tags_10 },
    { 0x8a44c58f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x42c4c591, SUC_lexicon_tags_1 },
    { 0xa8c4c592, SUC_lexicon_tags_2 },
    { 0xa544c592, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb924c595, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b04c59a, SUC_lexicon_tags_37 },
    { 0x5a4c59a, SUC_lexicon_tags_17 },
    { 0xb0e4c59a, SUC_lexicon_tags_66 },
    { 0x2a04c59d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e4c5a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b24c5a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1844c5a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9144c5a8, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb24c5ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xec44c5ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeae4c5af, SUC_lexicon_tags_49 },
    { 0xb364c5b0, SUC_lexicon_tags_47 },
    { 0xf64c5b1, SUC_lexicon_tags_29 },
    { 0x6104c5b1, SUC_lexicon_tags_37 },
    { 0xab84c5b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784c5b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2464c5ba, SUC_lexicon_tags_11 },
    { 0x6024c5bb, SUC_lexicon_tags_38 },
    { 0x75e4c5ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce4c5be, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb704c5c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1544c5c8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4c5cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ca4c5cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c4c5d2, SUC_lexicon_tags_15 },
    { 0x7404c5d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904c5d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf144c5de, SUC_lexicon_tags_29 },
    { 0x7e4c5de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd124c5e6, SUC_lexicon_tags_0 },
    { 0x5d44c5e7, SUC_lexicon_tags_2 },
    { 0x20a4c5e8, SUC_lexicon_tags_1 },
    { 0xeb44c5e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c4c5ec, SUC_lexicon_tags_53 },
    { 0x2f44c5ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a64c5f0, SUC_lexicon_tags_10 },
    { 0xb624c5f0, SUC_lexicon_tags_16 },
    { 0xb9c4c5f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec4c5f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1464c5f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x75c4c5f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2844c5fb, SUC_lexicon_tags_7 },
    { 0xe3a4c5fb, SUC_lexicon_tags_60 },
    { 0x35c4c5fc, SUC_lexicon_tags_1 },
    { 0x85a4c5fe, SUC_lexicon_tags_95 },
    { 0xc9c4c5fb, SUC_lexicon_tags_23 },
    { 0xae44c5fe, SUC_lexicon_tags_27 },
    { 0x7e4c5fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c04c604, SUC_lexicon_tags_37 },
    { 0xcee4c604, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e4c609, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5964c60f, SUC_lexicon_tags_4 },
    { 0x7da4c60f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6304c617, SUC_lexicon_tags_25 },
    { 0x78a4c618, SUC_lexicon_tags_12 },
    { 0x9dc4c619, SUC_lexicon_tags_265 },
    { 0x9204c61a, SUC_lexicon_tags_19 },
    { 0x2864c61b, SUC_lexicon_tags_17 },
    { 0x31a4c619, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8444c621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc64c624, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb484c627, SUC_lexicon_tags_17 },
    { 0x49a4c628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde24c62a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a4c62d, SUC_lexicon_tags_2 },
    { 0xb4c4c62e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2744c632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f64c634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67c4c636, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6464c63a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3e64c63c, SUC_lexicon_tags_49 },
    { 0x2024c63c, SUC_lexicon_tags_26 },
    { 0xe744c63e, SUC_lexicon_tags_14 },
    { 0x8e84c63e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2724c644, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe4c647, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x6124c64a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4024c650, SUC_lexicon_tags_7 },
    { 0xb564c651, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa4c654, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1964c656, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7444c659, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8644c65c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c65f, SUC_lexicon_tags_63 },
    { 0x7944c660, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6244c666, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8044c669, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a4c66c, SUC_lexicon_tags_15 },
    { 0xea64c66c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x11a4c66f, SUC_lexicon_tags_1 },
    { 0x4fe4c670, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7944c676, SUC_lexicon_tags_17 },
    { 0x1964c676, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x844c67a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c44c67c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4104c67f, SUC_lexicon_tags_6 },
    { 0x29a4c680, SUC_lexicon_tags_8 },
    { 0x66c4c681, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf924c684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed24c686, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8984c688, SUC_lexicon_tags_33 },
    { 0x5a64c688, SUC_lexicon_tags_11 },
    { 0x3244c68a, SUC_lexicon_tags_15 },
    { 0x93c4c68b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae4c694, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a4c697, SUC_lexicon_tags_2 },
    { 0xfd04c697, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae4c69d, SUC_lexicon_tags_12 },
    { 0xa5e4c69e, SUC_lexicon_tags_486 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb424c6a2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe324c6a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9084c6a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e44c6ab, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd84c6ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb44c6b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e4c6b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a4c6bc, SUC_lexicon_tags_2 },
    { 0x5964c6bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2604c6bf, SUC_lexicon_tags_53 },
    { 0xd764c6bf, SUC_lexicon_tags_30 },
    { 0xb164c6c1, SUC_lexicon_tags_12 },
    { 0xa804c6c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c4c6c5, SUC_lexicon_tags_26 },
    { 0x2864c6c6, SUC_lexicon_tags_23 },
    { 0x4084c6c7, SUC_lexicon_tags_12 },
    { 0x8484c6c7, SUC_lexicon_tags_22 },
    { 0xa44c6c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4a4c6cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3484c6d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5764c6df, SUC_lexicon_tags_7 },
    { 0xb9e4c6e0, SUC_lexicon_tags_5 },
    { 0x1c04c6e1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae4c6ea, SUC_lexicon_tags_0 },
    { 0xc0c4c6eb, SUC_lexicon_tags_27 },
    { 0xf864c6ec, SUC_lexicon_tags_47 },
    { 0x6244c6eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc4c6f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6d44c6f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a44c6f4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf504c6f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece4c704, SUC_lexicon_tags_25 },
    { 0xd624c705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c4c708, SUC_lexicon_tags_37 },
    { 0x8324c709, SUC_lexicon_tags_8 },
    { 0xe064c70a, SUC_lexicon_tags_17 },
    { 0xb224c70b, SUC_lexicon_tags_11 },
    { 0xeae4c70c, SUC_lexicon_tags_26 },
    { 0x5084c70d, SUC_lexicon_tags_16 },
    { 0x7404c70e, SUC_lexicon_tags_14 },
    { 0xbf44c70c, SUC_lexicon_tags_17 },
    { 0x73a4c70d, SUC_lexicon_tags_11 },
    { 0x6724c70c, SUC_lexicon_tags_6 },
    { 0x55c4c712, SUC_lexicon_tags_43 },
    { 0xfb44c709, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c4c719, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf44c71c, SUC_lexicon_tags_14 },
    { 0x5a4c71d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x21a4c71f, SUC_lexicon_tags_2 },
    { 0xe184c720, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1864c723, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3404c729, SUC_lexicon_tags_25 },
    { 0xec64c72a, SUC_lexicon_tags_65 },
    { 0xc64c72b, SUC_lexicon_tags_37 },
    { 0xdf24c72c, SUC_lexicon_tags_25 },
    { 0xb004c72d, SUC_lexicon_tags_12 },
    { 0x34c4c72c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec4c731, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3004c736, SUC_lexicon_tags_85 },
    { 0x6524c736, SUC_lexicon_tags_39 },
    { 0xa4a4c738, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a4c73b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc4c73e, SUC_lexicon_tags_25 },
    { 0xe524c73e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb784c742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1084c744, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a4c74f, SUC_lexicon_tags_1 },
    { 0xee64c750, SUC_lexicon_tags_59 },
    { 0xfee4c750, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e4c760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2284c762, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1724c766, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4de4c768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c4c76b, SUC_lexicon_tags_8 },
    { 0xb564c76b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba4c776, SUC_lexicon_tags_84 },
    { 0xe264c776, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e4c77b, SUC_lexicon_tags_37 },
    { 0x7144c77c, SUC_lexicon_tags_26 },
    { 0x4dc4c77d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e44c782, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8224c786, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b04c78b, SUC_lexicon_tags_11 },
    { 0xf544c78c, SUC_lexicon_tags_37 },
    { 0xf9e4c78d, SUC_lexicon_tags_14 },
    { 0x8284c78e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa104c792, SUC_lexicon_tags_38 },
    { 0xe8e4c793, SUC_lexicon_tags_11 },
    { 0x7364c794, SUC_lexicon_tags_19 },
    { 0x1e44c795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f04c798, SUC_lexicon_tags_12 },
    { 0x5a64c798, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c4c79e, SUC_lexicon_tags_60 },
    { 0x98c4c79f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe4c7a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa324c7a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf784c7ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c4c7ae, SUC_lexicon_tags_23 },
    { 0xd404c7af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7824c7b1, SUC_lexicon_tags_7 },
    { 0xd9a4c7b1, SUC_lexicon_tags_27 },
    { 0xf9c4c7b3, SUC_lexicon_tags_25 },
    { 0xda4c7b4, SUC_lexicon_tags_15 },
    { 0xd224c7b4, SUC_lexicon_tags_10 },
    { 0xf944c7b6, SUC_lexicon_tags_2 },
    { 0xdb64c7b7, SUC_lexicon_tags_17 },
    { 0x404c7b4, SUC_lexicon_tags_87 },
    { 0x3124c7b9, SUC_lexicon_tags_11 },
    { 0x13e4c7ba, SUC_lexicon_tags_25 },
    { 0x6fa4c7bb, SUC_lexicon_tags_47 },
    { 0x4d64c7bb, SUC_lexicon_tags_2 },
    { 0xbd64c7bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4c7c6, SUC_lexicon_tags_27 },
    { 0x9444c7c7, SUC_lexicon_tags_39 },
    { 0xc84c7c8, SUC_lexicon_tags_28 },
    { 0xaf64c7c7, SUC_lexicon_tags_37 },
    { 0x6344c7c8, SUC_lexicon_tags_1 },
    { 0xd7a4c7c6, SUC_lexicon_tags_5 },
    { 0x6ca4c7cc, SUC_lexicon_tags_33 },
    { 0x17c4c7c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4744c7d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec4c7d6, SUC_lexicon_tags_81 },
    { 0x1404c7d7, SUC_lexicon_tags_15 },
    { 0x17c4c7d8, SUC_lexicon_tags_29 },
    { 0x4724c7d9, SUC_lexicon_tags_12 },
    { 0xdbc4c7da, SUC_lexicon_tags_8 },
    { 0xab44c7d7, SUC_lexicon_tags_37 },
    { 0xc144c7db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa464c7e1, SUC_lexicon_tags_37 },
    { 0x8844c7e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e4c7e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bc4c7ea, SUC_lexicon_tags_14 },
    { 0xaa44c7eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d64c7ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6704c7f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e4c7fc, SUC_lexicon_tags_1 },
    { 0x4fc4c7fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd684c801, SUC_lexicon_tags_61 },
    { 0xb4c4c801, SUC_lexicon_tags_29 },
    { 0xb704c803, SUC_lexicon_tags_10 },
    { 0x5024c804, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc4c808, SUC_lexicon_tags_37 },
    { 0x5884c809, SUC_lexicon_tags_27 },
    { 0x9ba4c80a, SUC_lexicon_tags_49 },
    { 0xb144c80a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5064c80d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8484c811, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe784c819, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e4c81d, SUC_lexicon_tags_45 },
    { 0x39c4c81e, SUC_lexicon_tags_65 },
    { 0x41a4c81d, SUC_lexicon_tags_1 },
    { 0xf8c4c81e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca4c829, SUC_lexicon_tags_37 },
    { 0x3cc4c82a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd004c82d, SUC_lexicon_tags_11 },
    { 0xc064c82e, SUC_lexicon_tags_1 },
    { 0x4ac4c82e, SUC_lexicon_tags_1 },
    { 0xedc4c830, SUC_lexicon_tags_45 },
    { 0x4a04c831, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46a4c833, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9864c838, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf304c83f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xede4c841, SUC_lexicon_tags_17 },
    { 0xd8c4c842, SUC_lexicon_tags_10 },
    { 0x1da4c843, SUC_lexicon_tags_2 },
    { 0x9b44c841, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c4c849, SUC_lexicon_tags_15 },
    { 0xbaa4c84a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e4c84f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a4c854, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8884c85c, SUC_lexicon_tags_37 },
    { 0xd164c85d, SUC_lexicon_tags_39 },
    { 0x344c85d, SUC_lexicon_tags_65 },
    { 0x7c44c85c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4984c861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe464c865, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa624c86b, SUC_lexicon_tags_12 },
    { 0x8284c86c, SUC_lexicon_tags_10 },
    { 0x8e04c86d, SUC_lexicon_tags_25 },
    { 0x9784c86c, SUC_lexicon_tags_27 },
    { 0xbd44c86f, SUC_lexicon_tags_12 },
    { 0xbf44c86f, SUC_lexicon_tags_37 },
    { 0xdfa4c870, SUC_lexicon_tags_37 },
    { 0x1c04c872, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3724c874, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x51c4c876, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8844c879, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3744c87c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a4c881, SUC_lexicon_tags_7 },
    { 0x5e4c881, SUC_lexicon_tags_56 },
    { 0x3d44c882, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc784c887, SUC_lexicon_tags_19 },
    { 0x62c4c888, SUC_lexicon_tags_39 },
    { 0xd64c889, SUC_lexicon_tags_33 },
    { 0x1964c889, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e24c88c, SUC_lexicon_tags_2 },
    { 0xc7c4c88d, SUC_lexicon_tags_11 },
    { 0x3784c88d, SUC_lexicon_tags_14 },
    { 0xb8c4c88e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c4c895, SUC_lexicon_tags_11 },
    { 0x40e4c895, SUC_lexicon_tags_7 },
    { 0x8c24c897, SUC_lexicon_tags_15 },
    { 0xa8a4c896, SUC_lexicon_tags_39 },
    { 0x3f04c895, SUC_lexicon_tags_19 },
    { 0x5304c895, SUC_lexicon_tags_31 },
    { 0x2fe4c899, SUC_lexicon_tags_37 },
    { 0x5504c89c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe4c8a2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa704c8a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc544c8a8, SUC_lexicon_tags_7 },
    { 0x5c24c8a9, SUC_lexicon_tags_1 },
    { 0x7184c8aa, SUC_lexicon_tags_25 },
    { 0xbfc4c8a8, SUC_lexicon_tags_63 },
    { 0xbda4c8ac, SUC_lexicon_tags_1 },
    { 0x244c8ac, SUC_lexicon_tags_23 },
    { 0x2104c8ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f64c8b1, SUC_lexicon_tags_15 },
    { 0x3f44c8b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e4c8b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4da4c8b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd684c8bb, SUC_lexicon_tags_44 },
    { 0xdc84c8bc, SUC_lexicon_tags_12 },
    { 0xb6c4c8bd, SUC_lexicon_tags_23 },
    { 0x8404c8bd, SUC_lexicon_tags_37 },
    { 0xdfc4c8bf, SUC_lexicon_tags_2 },
    { 0x8144c8bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a4c8c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3444c8c6, SUC_lexicon_tags_25 },
    { 0xb84c8c6, SUC_lexicon_tags_23 },
    { 0xfe4c8c8, SUC_lexicon_tags_1 },
    { 0x7ee4c8c9, SUC_lexicon_tags_39 },
    { 0x7544c8c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a4c8cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc784c8d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa0e4c8d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc4c8dd, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xc64c8df, SUC_lexicon_tags_83 },
    { 0x76e4c8e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e4c8e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d84c8ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x124c8f0, SUC_lexicon_tags_46 },
    { 0x1a4c8f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6be4c8f3, SUC_lexicon_tags_43 },
    { 0x2924c8f3, SUC_lexicon_tags_25 },
    { 0xc144c8f3, SUC_lexicon_tags_45 },
    { 0x1d24c8f4, SUC_lexicon_tags_28 },
    { 0x6544c8f5, SUC_lexicon_tags_14 },
    { 0x58a4c8f3, SUC_lexicon_tags_15 },
    { 0x59e4c8f4, SUC_lexicon_tags_12 },
    { 0xce44c8fa, SUC_lexicon_tags_60 },
    { 0x964c8fa, SUC_lexicon_tags_2 },
    { 0x8924c8fb, SUC_lexicon_tags_2 },
    { 0xe044c8fd, SUC_lexicon_tags_28 },
    { 0x84e4c8fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3e4c900, SUC_lexicon_tags_7 },
    { 0xa4e4c900, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37a4c903, SUC_lexicon_tags_1 },
    { 0x50a4c903, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x604c907, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5144c909, SUC_lexicon_tags_63 },
    { 0xd0a4c90a, SUC_lexicon_tags_35 },
    { 0xd104c90a, SUC_lexicon_tags_0 },
    { 0x63a4c90a, SUC_lexicon_tags_29 },
    { 0xa684c90d, SUC_lexicon_tags_25 },
    { 0xda84c90d, SUC_lexicon_tags_37 },
    { 0xcd44c90b, SUC_lexicon_tags_38 },
    { 0x80c4c910, SUC_lexicon_tags_25 },
    { 0x4904c911, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x65c4c913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x624c916, SUC_lexicon_tags_11 },
    { 0x6aa4c917, SUC_lexicon_tags_49 },
    { 0x35e4c918, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd504c91a, SUC_lexicon_tags_29 },
    { 0xaa24c91a, SUC_lexicon_tags_739 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba4c91e, SUC_lexicon_tags_40 },
    { 0xc4c4c91e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e24c923, SUC_lexicon_tags_25 },
    { 0xe1a4c924, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e4c927, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11e4c929, SUC_lexicon_tags_33 },
    { 0xed84c92a, SUC_lexicon_tags_23 },
    { 0xae04c92b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc4c92f, SUC_lexicon_tags_66 },
    { 0xce84c92f, SUC_lexicon_tags_26 },
    { 0x3224c930, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b24c935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd64c939, SUC_lexicon_tags_87 },
    { 0x9e24c939, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6044c93d, SUC_lexicon_tags_47 },
    { 0xc344c93e, SUC_lexicon_tags_46 },
    { 0x8384c93d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc364c941, SUC_lexicon_tags_23 },
    { 0xe4a4c942, SUC_lexicon_tags_45 },
    { 0xe124c943, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a4c947, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c44c94b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1284c94d, SUC_lexicon_tags_60 },
    { 0xfe64c94e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde4c950, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc4c953, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbc4c955, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x54e4c957, SUC_lexicon_tags_1 },
    { 0x8424c958, SUC_lexicon_tags_15 },
    { 0x6704c959, SUC_lexicon_tags_37 },
    { 0xcfe4c958, SUC_lexicon_tags_37 },
    { 0xf84c95b, SUC_lexicon_tags_103 },
    { 0x24c4c95c, SUC_lexicon_tags_83 },
    { 0x9cc4c95c, SUC_lexicon_tags_46 },
    { 0x7a44c95e, SUC_lexicon_tags_14 },
    { 0x9904c95d, SUC_lexicon_tags_45 },
    { 0x2a84c95f, SUC_lexicon_tags_60 },
    { 0xc024c95d, SUC_lexicon_tags_84 },
    { 0x4ca4c959, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8964c965, SUC_lexicon_tags_15 },
    { 0xec44c965, SUC_lexicon_tags_15 },
    { 0x8d04c967, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f84c969, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e4c970, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e4c978, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31e4c97a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8164c983, SUC_lexicon_tags_23 },
    { 0x4ca4c984, SUC_lexicon_tags_43 },
    { 0xd164c984, SUC_lexicon_tags_83 },
    { 0x9c4c986, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0x9304c988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e4c98e, SUC_lexicon_tags_17 },
    { 0x4fa4c98f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb444c993, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c4c9a0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcee4c9a2, SUC_lexicon_tags_27 },
    { 0x17a4c9a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc344c9a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8364c9a9, SUC_lexicon_tags_6 },
    { 0x5be4c9a9, SUC_lexicon_tags_2 },
    { 0x5784c9a9, SUC_lexicon_tags_39 },
    { 0x8924c9ac, SUC_lexicon_tags_11 },
    { 0x6c4c9ad, SUC_lexicon_tags_8 },
    { 0xc984c9ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a4c9b1, SUC_lexicon_tags_16 },
    { 0xcc4c9b2, SUC_lexicon_tags_22 },
    { 0xdfa4c9b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6124c9b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1c4c9b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb784c9bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e4c9be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7464c9c0, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae4c9c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65a4c9c6, SUC_lexicon_tags_17 },
    { 0x1064c9c7, SUC_lexicon_tags_2 },
    { 0xd684c9c7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xfa04c9ca, SUC_lexicon_tags_1 },
    { 0x59a4c9cb, SUC_lexicon_tags_34 },
    { 0x94c4c9ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe684c9d8, SUC_lexicon_tags_7 },
    { 0x8604c9d9, SUC_lexicon_tags_17 },
    { 0x3724c9d9, SUC_lexicon_tags_31 },
    { 0xa204c9da, SUC_lexicon_tags_12 },
    { 0x5ee4c9d9, SUC_lexicon_tags_27 },
    { 0x4c44c9da, SUC_lexicon_tags_2 },
    { 0xbde4c9de, SUC_lexicon_tags_37 },
    { 0x82e4c9de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c4c9e2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd184c9e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a4c9eb, SUC_lexicon_tags_37 },
    { 0xa64c9ec, SUC_lexicon_tags_49 },
    { 0x9a04c9ed, SUC_lexicon_tags_55 },
    { 0xc184c9ee, SUC_lexicon_tags_68 },
    { 0xfd04c9ef, SUC_lexicon_tags_30 },
    { 0x4d44c9ee, SUC_lexicon_tags_18 },
    { 0x7b44c9f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc4c9f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6384c9fa, SUC_lexicon_tags_7 },
    { 0xde64c9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8724c9ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c4ca03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa304ca08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9604ca0a, SUC_lexicon_tags_27 },
    { 0xe9a4ca0a, SUC_lexicon_tags_15 },
    { 0xba84ca0c, SUC_lexicon_tags_47 },
    { 0xc6a4ca0b, SUC_lexicon_tags_15 },
    { 0x4224ca0e, SUC_lexicon_tags_17 },
    { 0x5c44ca0f, SUC_lexicon_tags_14 },
    { 0xdaa4ca10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8304ca14, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc124ca19, SUC_lexicon_tags_37 },
    { 0x6624ca19, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa4ca20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e4ca25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb364ca27, SUC_lexicon_tags_2 },
    { 0xa404ca28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcfe4ca2a, SUC_lexicon_tags_45 },
    { 0x1f44ca2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a4ca33, SUC_lexicon_tags_37 },
    { 0x9cc4ca34, SUC_lexicon_tags_25 },
    { 0xfc24ca35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e84ca39, SUC_lexicon_tags_92 },
    { 0x18a4ca39, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea84ca3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf424ca40, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ca43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc4ca47, SUC_lexicon_tags_17 },
    { 0x8bc4ca48, SUC_lexicon_tags_38 },
    { 0x21a4ca49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c4ca4f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6284ca54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x50a4ca56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaba4ca58, SUC_lexicon_tags_62 },
    { 0xfd64ca58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe804ca5b, SUC_lexicon_tags_11 },
    { 0xc6a4ca5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd284ca63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6104ca67, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9564ca6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb04ca6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4484ca6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e4ca72, SUC_lexicon_tags_31 },
    { 0x44e4ca72, SUC_lexicon_tags_24 },
    { 0x8884ca74, SUC_lexicon_tags_7 },
    { 0x2764ca72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7cc4ca77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2224ca7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc4ca7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2144ca7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0c4ca81, SUC_lexicon_tags_25 },
    { 0x66c4ca81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc604ca84, SUC_lexicon_tags_79 },
    { 0xa084ca84, SUC_lexicon_tags_351 },
    { 0x7244ca86, SUC_lexicon_tags_37 },
    { 0xd0e4ca86, SUC_lexicon_tags_17 },
    { 0x984ca85, SUC_lexicon_tags_6 },
    { 0xa0a4ca87, SUC_lexicon_tags_45 },
    { 0x124ca86, SUC_lexicon_tags_15 },
    { 0xe0a4ca8b, SUC_lexicon_tags_37 },
    { 0x7e84ca87, SUC_lexicon_tags_25 },
    { 0x3784ca8d, SUC_lexicon_tags_12 },
    { 0xffa4ca8a, SUC_lexicon_tags_11 },
    { 0x5104ca85, SUC_lexicon_tags_67 },
    { 0x2364ca8c, SUC_lexicon_tags_14 },
    { 0x57c4ca91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c4ca95, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9844ca98, SUC_lexicon_tags_8 },
    { 0xb0a4ca99, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcc84ca9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb24ca9e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf104caa3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a64caa5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9124caa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc984caad, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4ac4caaf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c4cab4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc324cab8, SUC_lexicon_tags_538 },
    { 0, NULL },
    { 0x5f84caba, SUC_lexicon_tags_116 },
    { 0x7404cabb, SUC_lexicon_tags_45 },
    { 0x1ea4cabc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a64cac0, SUC_lexicon_tags_37 },
    { 0xc544cac0, SUC_lexicon_tags_14 },
    { 0xc304cac2, SUC_lexicon_tags_47 },
    { 0x3364cac3, SUC_lexicon_tags_14 },
    { 0xc184cac4, SUC_lexicon_tags_4 },
    { 0xe7a4cac4, SUC_lexicon_tags_37 },
    { 0x84cac1, SUC_lexicon_tags_10 },
    { 0xe024cac2, SUC_lexicon_tags_37 },
    { 0x3a64cac0, SUC_lexicon_tags_27 },
    { 0xea44cac0, SUC_lexicon_tags_84 },
    { 0xc464caca, SUC_lexicon_tags_28 },
    { 0xfb04cac1, SUC_lexicon_tags_14 },
    { 0x1e4cac1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c4cacf, SUC_lexicon_tags_25 },
    { 0x66e4cacf, SUC_lexicon_tags_84 },
    { 0x35c4cad1, SUC_lexicon_tags_45 },
    { 0x7744cad1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe084cad5, SUC_lexicon_tags_36 },
    { 0xd164cad5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe64cada, SUC_lexicon_tags_6 },
    { 0x75c4cada, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e84cadf, SUC_lexicon_tags_2 },
    { 0x65a4cae0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3204cae2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb544caea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd04caf1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x27e4caf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7fa4caf5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd84caff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a04cb02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c4cb07, SUC_lexicon_tags_47 },
    { 0x1744cb08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4cb0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb624cb0f, SUC_lexicon_tags_28 },
    { 0x804cb0f, SUC_lexicon_tags_174 },
    { 0xebc4cb0f, SUC_lexicon_tags_43 },
    { 0x25c4cb12, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd404cb14, SUC_lexicon_tags_43 },
    { 0x8a84cb14, SUC_lexicon_tags_0 },
    { 0x2de4cb14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124cb1a, SUC_lexicon_tags_1 },
    { 0xeee4cb1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce4cb1e, SUC_lexicon_tags_38 },
    { 0x3f84cb1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab64cb22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28c4cb24, SUC_lexicon_tags_7 },
    { 0x99e4cb25, SUC_lexicon_tags_9 },
    { 0x4d84cb26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2584cb28, SUC_lexicon_tags_10 },
    { 0x22e4cb29, SUC_lexicon_tags_53 },
    { 0x66e4cb28, SUC_lexicon_tags_25 },
    { 0x86c4cb2b, SUC_lexicon_tags_91 },
    { 0x904cb2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804cb30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf24cb32, SUC_lexicon_tags_12 },
    { 0x3d04cb33, SUC_lexicon_tags_1 },
    { 0xf84cb33, SUC_lexicon_tags_37 },
    { 0x7e44cb33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa44cb39, SUC_lexicon_tags_2 },
    { 0xd7a4cb39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21c4cb3c, SUC_lexicon_tags_2 },
    { 0x2164cb3c, SUC_lexicon_tags_19 },
    { 0xad04cb3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3164cb4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7564cb54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc4cb59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6c4cb5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee64cb5f, SUC_lexicon_tags_47 },
    { 0xfc4cb60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0e4cb62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8a4cb64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4764cb69, SUC_lexicon_tags_60 },
    { 0x33c4cb69, SUC_lexicon_tags_2 },
    { 0x7e24cb6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4c4cb6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf124cb70, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5b44cb72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e44cb74, SUC_lexicon_tags_87 },
    { 0x4924cb74, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d04cb79, SUC_lexicon_tags_17 },
    { 0xd2c4cb7a, SUC_lexicon_tags_12 },
    { 0xa4e4cb7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee4cb7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd604cb81, SUC_lexicon_tags_220 },
    { 0x3ec4cb81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a4cb85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa424cb89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd64cb8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5824cb8e, SUC_lexicon_tags_23 },
    { 0x4f44cb8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44e4cb91, SUC_lexicon_tags_14 },
    { 0x6b44cb92, SUC_lexicon_tags_1 },
    { 0x3d84cb93, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9384cb95, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2024cb9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9784cba0, SUC_lexicon_tags_25 },
    { 0x40a4cba1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa84cba3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc4cba7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde4cbaa, SUC_lexicon_tags_2 },
    { 0x55a4cbaa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe124cbaf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4cbb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe844cbb9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4cbbf, SUC_lexicon_tags_1 },
    { 0x11a4cbc0, SUC_lexicon_tags_11 },
    { 0xeb84cbc1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2704cbc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc4cbc9, SUC_lexicon_tags_23 },
    { 0x2904cbca, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x89c4cbcc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c4cbd0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9724cbd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbe4cbd4, SUC_lexicon_tags_1 },
    { 0xab24cbd5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd404cbd7, SUC_lexicon_tags_11 },
    { 0x22a4cbd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3de4cbda, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2044cbe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e24cbe8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6664cbf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3404cbf2, SUC_lexicon_tags_17 },
    { 0xa164cbf3, SUC_lexicon_tags_12 },
    { 0xe4e4cbf4, SUC_lexicon_tags_26 },
    { 0x3104cbf5, SUC_lexicon_tags_46 },
    { 0xaa84cbf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9424cbfa, SUC_lexicon_tags_55 },
    { 0xef04cbfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5884cc01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf084cc03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac4cc08, SUC_lexicon_tags_11 },
    { 0x2be4cc09, SUC_lexicon_tags_0 },
    { 0xf3a4cc0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a44cc0f, SUC_lexicon_tags_27 },
    { 0x2104cc0f, SUC_lexicon_tags_9 },
    { 0x8884cc0f, SUC_lexicon_tags_33 },
    { 0x9e24cc12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x884cc14, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ec4cc16, SUC_lexicon_tags_37 },
    { 0x8164cc16, SUC_lexicon_tags_1 },
    { 0xc3e4cc18, SUC_lexicon_tags_66 },
    { 0x6f04cc19, SUC_lexicon_tags_11 },
    { 0x20c4cc1a, SUC_lexicon_tags_12 },
    { 0x8b24cc1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec24cc22, SUC_lexicon_tags_55 },
    { 0x11e4cc22, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf844cc29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a4cc30, SUC_lexicon_tags_19 },
    { 0xe2a4cc31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8124cc35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa4cc3a, SUC_lexicon_tags_43 },
    { 0xdda4cc3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee44cc3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1344cc3f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xad84cc41, SUC_lexicon_tags_2 },
    { 0xad04cc41, SUC_lexicon_tags_118 },
    { 0x2984cc43, SUC_lexicon_tags_0 },
    { 0x3444cc43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee04cc47, SUC_lexicon_tags_47 },
    { 0x2fe4cc48, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2f44cc4a, SUC_lexicon_tags_63 },
    { 0x2284cc4a, SUC_lexicon_tags_82 },
    { 0xc4c4cc4c, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe4cc50, SUC_lexicon_tags_11 },
    { 0x4744cc51, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a64cc55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e64cc57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e4cc60, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5004cc62, SUC_lexicon_tags_28 },
    { 0x6504cc62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad44cc6b, SUC_lexicon_tags_3 },
    { 0x1024cc6c, SUC_lexicon_tags_11 },
    { 0xada4cc6d, SUC_lexicon_tags_15 },
    { 0x7624cc6d, SUC_lexicon_tags_23 },
    { 0x6144cc6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7444cc71, SUC_lexicon_tags_11 },
    { 0x5764cc72, SUC_lexicon_tags_151 },
    { 0x10a4cc72, SUC_lexicon_tags_45 },
    { 0xf464cc73, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da4cc78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc04cc7c, SUC_lexicon_tags_11 },
    { 0xdce4cc7d, SUC_lexicon_tags_7 },
    { 0xb24cc7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38e4cc80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d24cc84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1504cc86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1564cc8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf704cc94, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a04cc97, SUC_lexicon_tags_207 },
    { 0xd164cc98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1864cc9b, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0xd364cc9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90a4cca0, SUC_lexicon_tags_161 },
    { 0xff04cca1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe704cca4, SUC_lexicon_tags_26 },
    { 0x8a24cca4, SUC_lexicon_tags_14 },
    { 0xe6e4cca4, SUC_lexicon_tags_7 },
    { 0x4c84cca6, SUC_lexicon_tags_25 },
    { 0x91e4cca8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a4ccab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f04ccb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf944ccb5, SUC_lexicon_tags_17 },
    { 0xdd44ccb6, SUC_lexicon_tags_25 },
    { 0xf104ccb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa284ccb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac04ccbd, SUC_lexicon_tags_141 },
    { 0xf7a4ccbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ccc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x624ccc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e64ccc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e4ccce, SUC_lexicon_tags_0 },
    { 0xbbe4cccf, SUC_lexicon_tags_0 },
    { 0x6fa4cccf, SUC_lexicon_tags_29 },
    { 0xeba4ccd1, SUC_lexicon_tags_70 },
    { 0x7c24ccd0, SUC_lexicon_tags_28 },
    { 0x96e4ccd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba4ccd6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7744cce4, SUC_lexicon_tags_31 },
    { 0xd024cce5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f24cce9, SUC_lexicon_tags_12 },
    { 0x8a04ccea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed64ccf5, SUC_lexicon_tags_60 },
    { 0x3304ccf5, SUC_lexicon_tags_184 },
    { 0xe8a4ccf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca44ccf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2844ccfb, SUC_lexicon_tags_25 },
    { 0x8da4ccfc, SUC_lexicon_tags_63 },
    { 0xcf04ccfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1624ccff, SUC_lexicon_tags_48 },
    { 0x9ac4ccff, SUC_lexicon_tags_7 },
    { 0xf984cd01, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac4cd04, SUC_lexicon_tags_8 },
    { 0xa2e4cd05, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7624cd07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf344cd09, SUC_lexicon_tags_25 },
    { 0xaaa4cd0a, SUC_lexicon_tags_19 },
    { 0x7ae4cd09, SUC_lexicon_tags_28 },
    { 0x3944cd0c, SUC_lexicon_tags_7 },
    { 0x8a04cd0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x164cd0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc4cd12, SUC_lexicon_tags_7 },
    { 0x8044cd13, SUC_lexicon_tags_77 },
    { 0xcaa4cd14, SUC_lexicon_tags_11 },
    { 0xca04cd15, SUC_lexicon_tags_230 },
    { 0x7a24cd16, SUC_lexicon_tags_2 },
    { 0x19e4cd16, SUC_lexicon_tags_8 },
    { 0xc8a4cd14, SUC_lexicon_tags_2 },
    { 0x8004cd19, SUC_lexicon_tags_9 },
    { 0x9364cd1a, SUC_lexicon_tags_14 },
    { 0x984cd13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba4cd21, SUC_lexicon_tags_2 },
    { 0xe1e4cd21, SUC_lexicon_tags_1 },
    { 0xb024cd23, SUC_lexicon_tags_23 },
    { 0x6684cd24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e4cd26, SUC_lexicon_tags_7 },
    { 0x7444cd27, SUC_lexicon_tags_60 },
    { 0xcba4cd28, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3004cd2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c4cd2f, SUC_lexicon_tags_33 },
    { 0xe7c4cd30, SUC_lexicon_tags_60 },
    { 0x21e4cd31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8164cd34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc4cd3a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda44cd44, SUC_lexicon_tags_29 },
    { 0x9d64cd45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf624cd47, SUC_lexicon_tags_30 },
    { 0x92a4cd48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5164cd52, SUC_lexicon_tags_37 },
    { 0x4d24cd53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9184cd57, SUC_lexicon_tags_25 },
    { 0x20e4cd58, SUC_lexicon_tags_12 },
    { 0x1d44cd59, SUC_lexicon_tags_10 },
    { 0xf44cd58, SUC_lexicon_tags_23 },
    { 0x82e4cd5a, SUC_lexicon_tags_2 },
    { 0x9944cd5c, SUC_lexicon_tags_8 },
    { 0xab04cd5d, SUC_lexicon_tags_11 },
    { 0xe5c4cd5e, SUC_lexicon_tags_37 },
    { 0xbc04cd5c, SUC_lexicon_tags_15 },
    { 0x5ca4cd60, SUC_lexicon_tags_7 },
    { 0xe8e4cd5d, SUC_lexicon_tags_31 },
    { 0x9844cd62, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xa584cd64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c64cd66, SUC_lexicon_tags_63 },
    { 0x9e24cd66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a4cd6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd5a4cd6d, SUC_lexicon_tags_65 },
    { 0xe244cd6e, SUC_lexicon_tags_31 },
    { 0xc864cd6f, SUC_lexicon_tags_19 },
    { 0xbf04cd70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1344cd75, SUC_lexicon_tags_23 },
    { 0xfb84cd76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc404cd78, SUC_lexicon_tags_82 },
    { 0x72e4cd79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf144cd7f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x6424cd81, SUC_lexicon_tags_65 },
    { 0xa804cd82, SUC_lexicon_tags_25 },
    { 0xb704cd82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d44cd85, SUC_lexicon_tags_38 },
    { 0xf284cd86, SUC_lexicon_tags_30 },
    { 0xfe64cd87, SUC_lexicon_tags_38 },
    { 0x7fa4cd88, SUC_lexicon_tags_30 },
    { 0x7404cd85, SUC_lexicon_tags_64 },
    { 0xbde4cd8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6824cd8e, SUC_lexicon_tags_39 },
    { 0xd24cd8f, SUC_lexicon_tags_87 },
    { 0x5bc4cd8e, SUC_lexicon_tags_11 },
    { 0x3044cd91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8624cd93, SUC_lexicon_tags_11 },
    { 0x3944cd93, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a4cd97, SUC_lexicon_tags_7 },
    { 0x1e4cd98, SUC_lexicon_tags_27 },
    { 0x6f64cd98, SUC_lexicon_tags_1 },
    { 0x1784cd9a, SUC_lexicon_tags_17 },
    { 0x8344cd9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc584cd9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e4cda0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b44cda4, SUC_lexicon_tags_25 },
    { 0xd564cda5, SUC_lexicon_tags_25 },
    { 0x6e84cda5, SUC_lexicon_tags_2 },
    { 0x40c4cda5, SUC_lexicon_tags_11 },
    { 0xae4cda8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d24cdab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7224cdb4, SUC_lexicon_tags_2 },
    { 0x4a84cdb4, SUC_lexicon_tags_11 },
    { 0x3e24cdb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa944cdb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a4cdbc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d64cdbf, SUC_lexicon_tags_14 },
    { 0xb984cdbf, SUC_lexicon_tags_27 },
    { 0x3804cdc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a04cdc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7644cdc5, SUC_lexicon_tags_28 },
    { 0x94e4cdc5, SUC_lexicon_tags_15 },
    { 0x8de4cdc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa104cdce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59e4cdd0, SUC_lexicon_tags_26 },
    { 0x84cdd1, SUC_lexicon_tags_15 },
    { 0x7424cdd2, SUC_lexicon_tags_38 },
    { 0xbba4cdd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f24cdd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab44cddc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fc4cdde, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf884cde3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb624cde5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8064cdea, SUC_lexicon_tags_6 },
    { 0x7e84cdeb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2804cdf3, SUC_lexicon_tags_7 },
    { 0x404cdf3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8dc4cdf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ec4cdf8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbd04cdfa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa484cdfe, SUC_lexicon_tags_33 },
    { 0xc404cdfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c64ce05, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e4ce0d, SUC_lexicon_tags_0 },
    { 0xa264ce0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ae4ce10, SUC_lexicon_tags_3 },
    { 0xbe24ce11, SUC_lexicon_tags_12 },
    { 0x6444ce12, SUC_lexicon_tags_63 },
    { 0x9f44ce13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d84ce16, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a4ce19, SUC_lexicon_tags_65 },
    { 0x9c24ce1a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2204ce1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf524ce22, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe284ce24, SUC_lexicon_tags_77 },
    { 0x6e04ce25, SUC_lexicon_tags_57 },
    { 0x3da4ce24, SUC_lexicon_tags_27 },
    { 0x7fc4ce24, SUC_lexicon_tags_1 },
    { 0x6ac4ce25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac4ce2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac84ce2f, SUC_lexicon_tags_37 },
    { 0xc404ce30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c4ce33, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d84ce3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe144ce3c, SUC_lexicon_tags_37 },
    { 0x6ae4ce3c, SUC_lexicon_tags_1 },
    { 0xbf84ce3e, SUC_lexicon_tags_25 },
    { 0x94e4ce3f, SUC_lexicon_tags_11 },
    { 0xe24ce3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf64ce44, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a4ce48, SUC_lexicon_tags_17 },
    { 0xfd64ce48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1704ce4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf444ce54, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824ce58, SUC_lexicon_tags_14 },
    { 0x1904ce59, SUC_lexicon_tags_31 },
    { 0x2b44ce59, SUC_lexicon_tags_15 },
    { 0x4c44ce5b, SUC_lexicon_tags_49 },
    { 0x51a4ce58, SUC_lexicon_tags_16 },
    { 0xf0a4ce5d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb064ce61, SUC_lexicon_tags_1 },
    { 0xe044ce62, SUC_lexicon_tags_39 },
    { 0x1964ce61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c4ce6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa824ce71, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa4ce73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84ce76, SUC_lexicon_tags_43 },
    { 0xc6a4ce77, SUC_lexicon_tags_7 },
    { 0xe124ce78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe4ce7a, SUC_lexicon_tags_45 },
    { 0x89c4ce7b, SUC_lexicon_tags_2 },
    { 0xe704ce7a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf324ce80, SUC_lexicon_tags_11 },
    { 0xe4c4ce81, SUC_lexicon_tags_31 },
    { 0xc1e4ce82, SUC_lexicon_tags_29 },
    { 0xe8e4ce83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc604ce85, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc4ce88, SUC_lexicon_tags_32 },
    { 0x32e4ce88, SUC_lexicon_tags_29 },
    { 0x9f64ce8a, SUC_lexicon_tags_15 },
    { 0x9464ce8b, SUC_lexicon_tags_19 },
    { 0xd664ce8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba4ce90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7604ce92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe4ce95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7944ce97, SUC_lexicon_tags_1418 },
    { 0x1384ce98, SUC_lexicon_tags_19 },
    { 0xb524ce99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4064ce9b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc5e4ce9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9f84ce9f, SUC_lexicon_tags_79 },
    { 0x8604cea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe84cea3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c4cea7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3c4cea9, SUC_lexicon_tags_37 },
    { 0xd604ceaa, SUC_lexicon_tags_47 },
    { 0x6d24cea9, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8184ceaf, SUC_lexicon_tags_17 },
    { 0xa104ceb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5664ceb2, SUC_lexicon_tags_28 },
    { 0x3044ceb3, SUC_lexicon_tags_7 },
    { 0xf044ceb4, SUC_lexicon_tags_25 },
    { 0x8624ceb5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc184ceb7, SUC_lexicon_tags_27 },
    { 0x4284ceb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6884ceba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2564cec6, SUC_lexicon_tags_17 },
    { 0x9684cec7, SUC_lexicon_tags_47 },
    { 0xbe24cec8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb524cecb, SUC_lexicon_tags_66 },
    { 0x3864cecc, SUC_lexicon_tags_25 },
    { 0x56e4cecd, SUC_lexicon_tags_84 },
    { 0x1d04cecd, SUC_lexicon_tags_37 },
    { 0xa5e4cecf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c4ced9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc344cedd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa684cee0, SUC_lexicon_tags_37 },
    { 0x56a4cee1, SUC_lexicon_tags_25 },
    { 0xae64cee1, SUC_lexicon_tags_26 },
    { 0xd324cee2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a44cee7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8244ceea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef84ceec, SUC_lexicon_tags_31 },
    { 0xaba4ceed, SUC_lexicon_tags_12 },
    { 0xdd44ceee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8184cef6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3b64cef8, SUC_lexicon_tags_25 },
    { 0x2204cef9, SUC_lexicon_tags_68 },
    { 0x6784cefa, SUC_lexicon_tags_60 },
    { 0x58c4cef9, SUC_lexicon_tags_38 },
    { 0xb024cefb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e4ceff, SUC_lexicon_tags_23 },
    { 0xfaa4ceff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97a4cf02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e84cf05, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a64cf08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa324cf0a, SUC_lexicon_tags_31 },
    { 0x3804cf0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82c4cf0d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9064cf11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc4cf19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ba4cf1b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa5a4cf1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7e4cf1f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a44cf22, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3124cf27, SUC_lexicon_tags_60 },
    { 0x1bc4cf28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee4cf2b, SUC_lexicon_tags_0 },
    { 0x3f04cf2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d44cf31, SUC_lexicon_tags_12 },
    { 0xca24cf31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804cf36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc324cf3a, SUC_lexicon_tags_10 },
    { 0xf044cf3b, SUC_lexicon_tags_17 },
    { 0xb5a4cf3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa284cf42, SUC_lexicon_tags_19 },
    { 0x4a04cf43, SUC_lexicon_tags_23 },
    { 0x4864cf43, SUC_lexicon_tags_12 },
    { 0x1344cf44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc4cf48, SUC_lexicon_tags_191 },
    { 0x50a4cf48, SUC_lexicon_tags_11 },
    { 0x26c4cf48, SUC_lexicon_tags_17 },
    { 0x1a4cf4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f84cf4d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae4cf50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae4cf56, SUC_lexicon_tags_1 },
    { 0x7544cf56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46c4cf59, SUC_lexicon_tags_38 },
    { 0x8e4cf59, SUC_lexicon_tags_27 },
    { 0x91e4cf59, SUC_lexicon_tags_0 },
    { 0x844cf5c, SUC_lexicon_tags_25 },
    { 0x6b04cf5b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2604cf5f, SUC_lexicon_tags_1 },
    { 0xe204cf60, SUC_lexicon_tags_19 },
    { 0xea04cf61, SUC_lexicon_tags_7 },
    { 0xd984cf62, SUC_lexicon_tags_87 },
    { 0x1024cf63, SUC_lexicon_tags_72 },
    { 0xd884cf64, SUC_lexicon_tags_37 },
    { 0x3fa4cf65, SUC_lexicon_tags_31 },
    { 0xa5e4cf66, SUC_lexicon_tags_29 },
    { 0x1664cf66, SUC_lexicon_tags_17 },
    { 0xffc4cf68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade4cf6f, SUC_lexicon_tags_10 },
    { 0x1424cf6f, SUC_lexicon_tags_63 },
    { 0xa6e4cf71, SUC_lexicon_tags_25 },
    { 0x81e4cf6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9164cf7a, SUC_lexicon_tags_14 },
    { 0x2844cf7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8664cf7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace4cf83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf604cf86, SUC_lexicon_tags_37 },
    { 0x4e84cf87, SUC_lexicon_tags_63 },
    { 0x8ac4cf88, SUC_lexicon_tags_7 },
    { 0x5ec4cf86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc44cf8d, SUC_lexicon_tags_32 },
    { 0x4d44cf8d, SUC_lexicon_tags_15 },
    { 0x4a44cf8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9204cf94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da4cf98, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf764cf9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4cfa0, SUC_lexicon_tags_2 },
    { 0xb824cfa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e44cfaa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa584cfac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44cfb3, SUC_lexicon_tags_11 },
    { 0x184cfb3, SUC_lexicon_tags_13 },
    { 0x21e4cfb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec4cfb8, SUC_lexicon_tags_37 },
    { 0xe6e4cfb9, SUC_lexicon_tags_17 },
    { 0x76e4cfb9, SUC_lexicon_tags_12 },
    { 0xb8c4cfbb, SUC_lexicon_tags_17 },
    { 0x1b44cfbc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b24cfbf, SUC_lexicon_tags_27 },
    { 0xd004cfc0, SUC_lexicon_tags_25 },
    { 0x304cfc1, SUC_lexicon_tags_3 },
    { 0xd04cfc2, SUC_lexicon_tags_68 },
    { 0x18a4cfc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2824cfca, SUC_lexicon_tags_33 },
    { 0x8744cfcb, SUC_lexicon_tags_43 },
    { 0x2e64cfcc, SUC_lexicon_tags_19 },
    { 0x5664cfcb, SUC_lexicon_tags_2 },
    { 0x27c4cfce, SUC_lexicon_tags_17 },
    { 0x53c4cfcf, SUC_lexicon_tags_30 },
    { 0xc1a4cfd0, SUC_lexicon_tags_104 },
    { 0x7564cfd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa4cfd6, SUC_lexicon_tags_79 },
    { 0x8204cfd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35e4cfd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d84cfe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf324cfe5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4cc4cfe7, SUC_lexicon_tags_25 },
    { 0x5444cfe8, SUC_lexicon_tags_31 },
    { 0xa4cfe9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd384cfeb, SUC_lexicon_tags_37 },
    { 0x95c4cfec, SUC_lexicon_tags_12 },
    { 0xe724cfed, SUC_lexicon_tags_7 },
    { 0xd024cfee, SUC_lexicon_tags_35 },
    { 0x43e4cfed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf804cff1, SUC_lexicon_tags_2 },
    { 0xacc4cff2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5504cff5, SUC_lexicon_tags_7 },
    { 0x3ec4cff6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1d24cff8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5804d004, SUC_lexicon_tags_43 },
    { 0x3924d005, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a4d00a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04d00d, SUC_lexicon_tags_27 },
    { 0xd764d00e, SUC_lexicon_tags_1 },
    { 0xe824d00d, SUC_lexicon_tags_9 },
    { 0xbde4d00f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9044d013, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4d019, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae4d01c, SUC_lexicon_tags_63 },
    { 0xc0c4d01d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xec64d01f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3824d026, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba44d028, SUC_lexicon_tags_77 },
    { 0x4f04d028, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5704d02c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2064d02e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1864d032, SUC_lexicon_tags_32 },
    { 0x1664d033, SUC_lexicon_tags_39 },
    { 0xe504d034, SUC_lexicon_tags_15 },
    { 0x50a4d034, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e04d03c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c4d045, SUC_lexicon_tags_7 },
    { 0x1e84d046, SUC_lexicon_tags_631 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa364d04a, SUC_lexicon_tags_47 },
    { 0xf284d04a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18c4d04d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d24d050, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4984d053, SUC_lexicon_tags_10 },
    { 0x84d054, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5984d056, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcda4d058, SUC_lexicon_tags_77 },
    { 0x5704d059, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2c4d05b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc64d05d, SUC_lexicon_tags_27 },
    { 0xaf24d05e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4004d062, SUC_lexicon_tags_45 },
    { 0xa964d063, SUC_lexicon_tags_15 },
    { 0xbfa4d063, SUC_lexicon_tags_63 },
    { 0xb464d062, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf904d068, SUC_lexicon_tags_17 },
    { 0x33c4d069, SUC_lexicon_tags_8 },
    { 0x9504d06a, SUC_lexicon_tags_0 },
    { 0x36c4d06b, SUC_lexicon_tags_25 },
    { 0x7e44d068, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a4d06f, SUC_lexicon_tags_23 },
    { 0x3164d06f, SUC_lexicon_tags_16 },
    { 0xc3e4d071, SUC_lexicon_tags_47 },
    { 0xbfc4d072, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa564d076, SUC_lexicon_tags_38 },
    { 0x61c4d077, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa4d07a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e4d07e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb904d080, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c4d084, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf9e4d086, SUC_lexicon_tags_19 },
    { 0xeb44d087, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3744d08b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8e4d08d, SUC_lexicon_tags_32 },
    { 0xe624d08e, SUC_lexicon_tags_27 },
    { 0xd924d08f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe4d093, SUC_lexicon_tags_38 },
    { 0xa8c4d094, SUC_lexicon_tags_15 },
    { 0xa244d095, SUC_lexicon_tags_11 },
    { 0xf544d096, SUC_lexicon_tags_11 },
    { 0x84a4d097, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca4d09b, SUC_lexicon_tags_7 },
    { 0xc8a4d09c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc1c4d09e, SUC_lexicon_tags_6 },
    { 0x6f24d09e, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x4944d0a1, SUC_lexicon_tags_38 },
    { 0x47a4d0a2, SUC_lexicon_tags_30 },
    { 0xc844d0a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6164d0a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe444d0b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a4d0b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c4d0b8, SUC_lexicon_tags_12 },
    { 0xcae4d0b8, SUC_lexicon_tags_65 },
    { 0xe2e4d0b9, SUC_lexicon_tags_45 },
    { 0x8ce4d0bb, SUC_lexicon_tags_43 },
    { 0xe2e4d0b8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a4d0bf, SUC_lexicon_tags_62 },
    { 0xc004d0c0, SUC_lexicon_tags_47 },
    { 0x8f64d0c1, SUC_lexicon_tags_18 },
    { 0x8984d0c1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e4d0c5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe864d0c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7c4d0c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5384d0d0, SUC_lexicon_tags_19 },
    { 0x9564d0d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69a4d0d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x484d0d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c4d0d8, SUC_lexicon_tags_87 },
    { 0x2104d0d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc4d0dd, SUC_lexicon_tags_2 },
    { 0x2324d0dd, SUC_lexicon_tags_38 },
    { 0x8ba4d0de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7684d0e1, SUC_lexicon_tags_30 },
    { 0x76a4d0e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce44d0e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c4d0ef, SUC_lexicon_tags_2 },
    { 0x844d0f0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc4d0f5, SUC_lexicon_tags_35 },
    { 0x7aa4d0f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ce4d0f8, SUC_lexicon_tags_27 },
    { 0xd384d0f9, SUC_lexicon_tags_27 },
    { 0xe4e4d0f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3884d0fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde44d101, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc344d103, SUC_lexicon_tags_28 },
    { 0x8be4d103, SUC_lexicon_tags_6 },
    { 0xb1c4d104, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xc684d107, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2384d109, SUC_lexicon_tags_7 },
    { 0x76a4d109, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f84d10e, SUC_lexicon_tags_1 },
    { 0x5864d10f, SUC_lexicon_tags_17 },
    { 0xc864d110, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa64d112, SUC_lexicon_tags_17 },
    { 0x8ba4d112, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb2e4d115, SUC_lexicon_tags_25 },
    { 0x1fc4d115, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6084d11b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf944d11d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4944d120, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf344d122, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde64d124, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7964d126, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1084d129, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56e4d12b, SUC_lexicon_tags_38 },
    { 0xbe44d12b, SUC_lexicon_tags_2 },
    { 0xedc4d12d, SUC_lexicon_tags_68 },
    { 0x9584d12e, SUC_lexicon_tags_4 },
    { 0xe4c4d12f, SUC_lexicon_tags_37 },
    { 0xcbc4d12f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e24d132, SUC_lexicon_tags_55 },
    { 0x4064d133, SUC_lexicon_tags_38 },
    { 0xf7c4d134, SUC_lexicon_tags_15 },
    { 0x6a4d133, SUC_lexicon_tags_47 },
    { 0x3424d135, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1764d139, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c4d13c, SUC_lexicon_tags_6 },
    { 0xf004d13d, SUC_lexicon_tags_1 },
    { 0x2fc4d13e, SUC_lexicon_tags_45 },
    { 0x2f64d13f, SUC_lexicon_tags_66 },
    { 0x23a4d140, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca24d142, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e44d144, SUC_lexicon_tags_2 },
    { 0xc0c4d144, SUC_lexicon_tags_12 },
    { 0x8f64d145, SUC_lexicon_tags_37 },
    { 0x3424d147, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6604d14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66c4d14d, SUC_lexicon_tags_8 },
    { 0x2b24d14d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9804d156, SUC_lexicon_tags_31 },
    { 0xdae4d157, SUC_lexicon_tags_14 },
    { 0x8e4d157, SUC_lexicon_tags_65 },
    { 0x3a84d156, SUC_lexicon_tags_17 },
    { 0xd7a4d15a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe404d15d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6aa4d15f, SUC_lexicon_tags_2 },
    { 0xbba4d15f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb24d162, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6944d165, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a4d169, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa564d16c, SUC_lexicon_tags_30 },
    { 0x27e4d16d, SUC_lexicon_tags_29 },
    { 0x3ce4d16e, SUC_lexicon_tags_46 },
    { 0x6904d16f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5204d173, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8364d17c, SUC_lexicon_tags_46 },
    { 0xdfe4d17c, SUC_lexicon_tags_26 },
    { 0x6724d17e, SUC_lexicon_tags_44 },
    { 0xe904d17f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc384d181, SUC_lexicon_tags_43 },
    { 0xcec4d182, SUC_lexicon_tags_37 },
    { 0xd784d183, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e24d187, SUC_lexicon_tags_30 },
    { 0x24a4d188, SUC_lexicon_tags_17 },
    { 0x4a84d189, SUC_lexicon_tags_7 },
    { 0xe04d18a, SUC_lexicon_tags_87 },
    { 0x5c4d18b, SUC_lexicon_tags_25 },
    { 0x524d18c, SUC_lexicon_tags_37 },
    { 0xd04d189, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8444d192, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc384d195, SUC_lexicon_tags_12 },
    { 0xed44d196, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1324d198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e4d19e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c04d1a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xde64d1a4, SUC_lexicon_tags_37 },
    { 0x1ae4d1a5, SUC_lexicon_tags_61 },
    { 0x1f24d1a5, SUC_lexicon_tags_1 },
    { 0x9ea4d1a5, SUC_lexicon_tags_77 },
    { 0x8be4d1a8, SUC_lexicon_tags_38 },
    { 0x38a4d1a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1164d1ad, SUC_lexicon_tags_37 },
    { 0x504d1ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c4d1b3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb24d1b6, SUC_lexicon_tags_37 },
    { 0x4ae4d1b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed84d1be, SUC_lexicon_tags_63 },
    { 0x7d24d1bf, SUC_lexicon_tags_26 },
    { 0xf44d1c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44d1c4, SUC_lexicon_tags_92 },
    { 0x9f04d1c5, SUC_lexicon_tags_37 },
    { 0xf7e4d1c4, SUC_lexicon_tags_207 },
    { 0x9704d1c4, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc864d1cb, SUC_lexicon_tags_17 },
    { 0x8844d1cb, SUC_lexicon_tags_25 },
    { 0xda84d1cd, SUC_lexicon_tags_12 },
    { 0x9224d1ce, SUC_lexicon_tags_2 },
    { 0xca4d1cf, SUC_lexicon_tags_29 },
    { 0x7584d1ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e4d1d6, SUC_lexicon_tags_6 },
    { 0xe4d1d7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce4d1db, SUC_lexicon_tags_25 },
    { 0xc5e4d1dc, SUC_lexicon_tags_7 },
    { 0x8204d1dd, SUC_lexicon_tags_19 },
    { 0x6284d1dd, SUC_lexicon_tags_3 },
    { 0x9684d1dc, SUC_lexicon_tags_37 },
    { 0x2de4d1e0, SUC_lexicon_tags_1 },
    { 0xa1c4d1de, SUC_lexicon_tags_12 },
    { 0x70a4d1e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xff24d1e5, SUC_lexicon_tags_66 },
    { 0x19a4d1e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9be4d1e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d24d1ea, SUC_lexicon_tags_15 },
    { 0xb3c4d1eb, SUC_lexicon_tags_15 },
    { 0x8784d1ec, SUC_lexicon_tags_148 },
    { 0xc4c4d1ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a4d1f1, SUC_lexicon_tags_65 },
    { 0x7b44d1f2, SUC_lexicon_tags_37 },
    { 0x2ec4d1f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6304d1f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0a4d1f7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdbe4d1f9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d64d1fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9844d200, SUC_lexicon_tags_14 },
    { 0x42e4d201, SUC_lexicon_tags_68 },
    { 0xd904d202, SUC_lexicon_tags_7 },
    { 0x3f24d200, SUC_lexicon_tags_12 },
    { 0xcee4d203, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf344d208, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3304d20a, SUC_lexicon_tags_7 },
    { 0xefa4d20a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x664d20d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x82c4d20f, SUC_lexicon_tags_7 },
    { 0xc084d210, SUC_lexicon_tags_25 },
    { 0x6ce4d211, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a4d214, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9924d217, SUC_lexicon_tags_61 },
    { 0xe644d218, SUC_lexicon_tags_15 },
    { 0x41e4d219, SUC_lexicon_tags_17 },
    { 0x7f44d219, SUC_lexicon_tags_10 },
    { 0x22a4d21b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc4d21e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c4d221, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x4504d223, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba4d227, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d24d229, SUC_lexicon_tags_47 },
    { 0xaa24d22a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1404d231, SUC_lexicon_tags_21 },
    { 0xf0a4d232, SUC_lexicon_tags_42 },
    { 0xb244d233, SUC_lexicon_tags_2 },
    { 0x7a24d234, SUC_lexicon_tags_7 },
    { 0x7344d235, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7004d23c, SUC_lexicon_tags_5 },
    { 0x2de4d23c, SUC_lexicon_tags_47 },
    { 0x26c4d23e, SUC_lexicon_tags_7 },
    { 0xa84d23f, SUC_lexicon_tags_17 },
    { 0x1d64d23f, SUC_lexicon_tags_24 },
    { 0xd944d241, SUC_lexicon_tags_29 },
    { 0x264d23c, SUC_lexicon_tags_38 },
    { 0xc1c4d242, SUC_lexicon_tags_21 },
    { 0x4b24d242, SUC_lexicon_tags_12 },
    { 0x90e4d23d, SUC_lexicon_tags_68 },
    { 0xb8e4d240, SUC_lexicon_tags_38 },
    { 0xe2a4d247, SUC_lexicon_tags_17 },
    { 0xeec4d248, SUC_lexicon_tags_17 },
    { 0x6904d249, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb64d24c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8b44d24e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4d251, SUC_lexicon_tags_0 },
    { 0x204d251, SUC_lexicon_tags_343 },
    { 0x61e4d253, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a4d259, SUC_lexicon_tags_7 },
    { 0x7d44d259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade4d25f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a4d264, SUC_lexicon_tags_30 },
    { 0x7fa4d265, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa64d269, SUC_lexicon_tags_19 },
    { 0xce64d26a, SUC_lexicon_tags_30 },
    { 0x86c4d269, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3064d26d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac4d272, SUC_lexicon_tags_38 },
    { 0x35e4d273, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7444d277, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda84d279, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e04d27e, SUC_lexicon_tags_235 },
    { 0xbbc4d27f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c4d282, SUC_lexicon_tags_70 },
    { 0xf904d283, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6424d286, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x8804d288, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3704d290, SUC_lexicon_tags_7 },
    { 0x61a4d291, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc344d297, SUC_lexicon_tags_25 },
    { 0x1fe4d297, SUC_lexicon_tags_11 },
    { 0x524d299, SUC_lexicon_tags_1 },
    { 0x584d299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb04d29d, SUC_lexicon_tags_1 },
    { 0xb184d29d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7c4d2a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c64d2a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71c4d2a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2404d2a9, SUC_lexicon_tags_19 },
    { 0xd844d2aa, SUC_lexicon_tags_29 },
    { 0x4c64d2ab, SUC_lexicon_tags_11 },
    { 0xfd64d2a9, SUC_lexicon_tags_29 },
    { 0xafa4d2a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8144d2af, SUC_lexicon_tags_8 },
    { 0xdf44d2b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2384d2b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e24d2bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804d2bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc4d2c2, SUC_lexicon_tags_17 },
    { 0x8264d2c3, SUC_lexicon_tags_7 },
    { 0x4c04d2c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf44d2ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2284d2cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x51e4d2ce, SUC_lexicon_tags_39 },
    { 0x2484d2cf, SUC_lexicon_tags_26 },
    { 0x76a4d2cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea4d2d3, SUC_lexicon_tags_29 },
    { 0x7c44d2d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b24d2d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8c4d2d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb6a4d2db, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e64d2e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a4d2ea, SUC_lexicon_tags_60 },
    { 0x25e4d2eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c4d2ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb44d2f4, SUC_lexicon_tags_17 },
    { 0x3ba4d2f4, SUC_lexicon_tags_68 },
    { 0x11e4d2f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc904d2ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3304d301, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb744d303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5e4d305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384d309, SUC_lexicon_tags_37 },
    { 0xd504d30a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe124d314, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c04d317, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de4d31b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a04d31e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f04d320, SUC_lexicon_tags_33 },
    { 0x3124d320, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe664d324, SUC_lexicon_tags_23 },
    { 0x2b04d324, SUC_lexicon_tags_14 },
    { 0xfc24d325, SUC_lexicon_tags_43 },
    { 0x4aa4d327, SUC_lexicon_tags_11 },
    { 0xc044d328, SUC_lexicon_tags_15 },
    { 0x56c4d326, SUC_lexicon_tags_2 },
    { 0x2144d326, SUC_lexicon_tags_62 },
    { 0x9024d32b, SUC_lexicon_tags_4 },
    { 0x57c4d32c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f04d32f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b84d333, SUC_lexicon_tags_47 },
    { 0xe384d334, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f24d337, SUC_lexicon_tags_49 },
    { 0x2fc4d338, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x51c4d33a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6464d341, SUC_lexicon_tags_29 },
    { 0x3a04d341, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f24d345, SUC_lexicon_tags_17 },
    { 0x7b84d346, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7a4d348, SUC_lexicon_tags_37 },
    { 0x4464d349, SUC_lexicon_tags_37 },
    { 0xda84d34a, SUC_lexicon_tags_33 },
    { 0x6084d34a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e4d352, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcda4d354, SUC_lexicon_tags_17 },
    { 0x57e4d354, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfce4d357, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2804d35c, SUC_lexicon_tags_15 },
    { 0xe644d35c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b84d364, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5244d368, SUC_lexicon_tags_17 },
    { 0x8624d369, SUC_lexicon_tags_17 },
    { 0x3284d36a, SUC_lexicon_tags_47 },
    { 0x9a64d369, SUC_lexicon_tags_17 },
    { 0x52c4d36c, SUC_lexicon_tags_12 },
    { 0x95c4d36a, SUC_lexicon_tags_11 },
    { 0x2e24d36c, SUC_lexicon_tags_35 },
    { 0x31c4d36f, SUC_lexicon_tags_6 },
    { 0x2fe4d370, SUC_lexicon_tags_113 },
    { 0x67c4d371, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x664d373, SUC_lexicon_tags_27 },
    { 0xe564d373, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c4d378, SUC_lexicon_tags_4 },
    { 0x6184d379, SUC_lexicon_tags_37 },
    { 0xfc84d37a, SUC_lexicon_tags_10 },
    { 0x4124d379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc4d37e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fa4d380, SUC_lexicon_tags_14 },
    { 0xff04d381, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7784d384, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4d38c, SUC_lexicon_tags_32 },
    { 0x2c4d38d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a4d396, SUC_lexicon_tags_10 },
    { 0x5744d397, SUC_lexicon_tags_7 },
    { 0x4024d396, SUC_lexicon_tags_17 },
    { 0x88a4d399, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d3a1, SUC_lexicon_tags_32 },
    { 0x5584d3a1, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xfb64d3a4, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xc264d3a6, SUC_lexicon_tags_27 },
    { 0x57e4d3a7, SUC_lexicon_tags_27 },
    { 0x75e4d3a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9c4d3aa, SUC_lexicon_tags_61 },
    { 0xc224d3aa, SUC_lexicon_tags_7 },
    { 0x9e44d3ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8444d3b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3c4d3b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb224d3b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c4d3c0, SUC_lexicon_tags_0 },
    { 0xf284d3c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c4d3c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a4d3c8, SUC_lexicon_tags_37 },
    { 0xb004d3c9, SUC_lexicon_tags_14 },
    { 0xd9c4d3ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e4d3d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc744d3d2, SUC_lexicon_tags_12 },
    { 0x71a4d3d3, SUC_lexicon_tags_43 },
    { 0xa6a4d3d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1444d3d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f24d3db, SUC_lexicon_tags_37 },
    { 0xe2a4d3dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a4d3e1, SUC_lexicon_tags_7 },
    { 0x5ea4d3e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5544d3e4, SUC_lexicon_tags_47 },
    { 0xd584d3e5, SUC_lexicon_tags_15 },
    { 0xfaa4d3e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d44d3e9, SUC_lexicon_tags_23 },
    { 0x5984d3ea, SUC_lexicon_tags_15 },
    { 0xcd24d3eb, SUC_lexicon_tags_2 },
    { 0x35a4d3eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d24d3ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd984d3f0, SUC_lexicon_tags_29 },
    { 0xcd24d3f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d44d3fa, SUC_lexicon_tags_26 },
    { 0x1924d3fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee4d401, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44d405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c04d407, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a4d40c, SUC_lexicon_tags_2 },
    { 0x67e4d40d, SUC_lexicon_tags_12 },
    { 0x56a4d40c, SUC_lexicon_tags_85 },
    { 0x4dc4d40e, SUC_lexicon_tags_82 },
    { 0x1aa4d40d, SUC_lexicon_tags_25 },
    { 0xfec4d40f, SUC_lexicon_tags_25 },
    { 0x8084d412, SUC_lexicon_tags_14 },
    { 0xd704d413, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae84d418, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9684d423, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f64d428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f44d430, SUC_lexicon_tags_151 },
    { 0x7624d431, SUC_lexicon_tags_15 },
    { 0x3964d432, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf904d436, SUC_lexicon_tags_7 },
    { 0x7fc4d437, SUC_lexicon_tags_66 },
    { 0x1b44d438, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd004d43c, SUC_lexicon_tags_63 },
    { 0x1764d43d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c4d449, SUC_lexicon_tags_11 },
    { 0x6f24d44a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd064d44c, SUC_lexicon_tags_1 },
    { 0x984d44d, SUC_lexicon_tags_25 },
    { 0x58e4d44e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c4d457, SUC_lexicon_tags_8 },
    { 0xa984d457, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4d45a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a4d45e, SUC_lexicon_tags_9 },
    { 0x9724d45f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d84d461, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f64d46c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7024d471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa544d474, SUC_lexicon_tags_8 },
    { 0x1ac4d475, SUC_lexicon_tags_8 },
    { 0x1284d475, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e4d47c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c4d480, SUC_lexicon_tags_15 },
    { 0x91c4d481, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8364d48a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc4d48e, SUC_lexicon_tags_45 },
    { 0xef84d48f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a44d493, SUC_lexicon_tags_70 },
    { 0x2c84d494, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe324d498, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1ea4d49a, SUC_lexicon_tags_21 },
    { 0x4564d49b, SUC_lexicon_tags_1 },
    { 0x1884d49c, SUC_lexicon_tags_1 },
    { 0xe204d49c, SUC_lexicon_tags_133 },
    { 0xe3c4d49e, SUC_lexicon_tags_0 },
    { 0xd064d49f, SUC_lexicon_tags_37 },
    { 0x64d49e, SUC_lexicon_tags_37 },
    { 0x12a4d49e, SUC_lexicon_tags_8 },
    { 0x71a4d49c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7604d4a4, SUC_lexicon_tags_23 },
    { 0xcda4d4a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1b64d4a7, SUC_lexicon_tags_1 },
    { 0x1364d4a8, SUC_lexicon_tags_19 },
    { 0x7e84d4a9, SUC_lexicon_tags_47 },
    { 0x6064d4a8, SUC_lexicon_tags_38 },
    { 0xa444d4a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe664d4af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ee4d4b1, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x7e24d4b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b24d4b6, SUC_lexicon_tags_68 },
    { 0xe984d4b7, SUC_lexicon_tags_37 },
    { 0xdcc4d4b7, SUC_lexicon_tags_15 },
    { 0x64e4d4b9, SUC_lexicon_tags_17 },
    { 0x2e64d4b8, SUC_lexicon_tags_15 },
    { 0x9ce4d4bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4524d4be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a4d4c1, SUC_lexicon_tags_15 },
    { 0x8de4d4c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13e4d4c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcbc4d4c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8484d4c9, SUC_lexicon_tags_26 },
    { 0x6de4d4ca, SUC_lexicon_tags_11 },
    { 0x9ac4d4c9, SUC_lexicon_tags_25 },
    { 0x67c4d4cc, SUC_lexicon_tags_45 },
    { 0xdfe4d4cd, SUC_lexicon_tags_23 },
    { 0x4024d4cc, SUC_lexicon_tags_7 },
    { 0xfce4d4cd, SUC_lexicon_tags_37 },
    { 0xc104d4d0, SUC_lexicon_tags_43 },
    { 0x7e44d4d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e4d4d4, SUC_lexicon_tags_17 },
    { 0xe3e4d4d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7564d4d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e4d4db, SUC_lexicon_tags_37 },
    { 0xfea4d4db, SUC_lexicon_tags_55 },
    { 0x9544d4dd, SUC_lexicon_tags_25 },
    { 0x82c4d4de, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e4d4e3, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x1f04d4e5, SUC_lexicon_tags_38 },
    { 0x1784d4e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a4d4eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a4d4ee, SUC_lexicon_tags_0 },
    { 0x47e4d4ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee4d4f2, SUC_lexicon_tags_11 },
    { 0x69c4d4f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a64d4f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f44d4f8, SUC_lexicon_tags_7 },
    { 0x5104d4f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c4d4fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae44d505, SUC_lexicon_tags_4 },
    { 0x6524d506, SUC_lexicon_tags_43 },
    { 0xb2a4d505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1604d50d, SUC_lexicon_tags_47 },
    { 0x6a84d50e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d84d511, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb904d514, SUC_lexicon_tags_69 },
    { 0xc184d515, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf724d51a, SUC_lexicon_tags_63 },
    { 0x20c4d51b, SUC_lexicon_tags_17 },
    { 0x8784d51b, SUC_lexicon_tags_35 },
    { 0x3984d51d, SUC_lexicon_tags_2 },
    { 0x4404d51e, SUC_lexicon_tags_11 },
    { 0xaa64d51f, SUC_lexicon_tags_97 },
    { 0xafa4d51f, SUC_lexicon_tags_8 },
    { 0x70a4d51d, SUC_lexicon_tags_11 },
    { 0xc24d522, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6064d525, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd264d52b, SUC_lexicon_tags_31 },
    { 0x86c4d52c, SUC_lexicon_tags_12 },
    { 0x99c4d52c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xae04d52f, SUC_lexicon_tags_6 },
    { 0x2e24d530, SUC_lexicon_tags_27 },
    { 0xeb44d531, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e64d535, SUC_lexicon_tags_15 },
    { 0xed04d536, SUC_lexicon_tags_87 },
    { 0x9d84d537, SUC_lexicon_tags_60 },
    { 0x4504d538, SUC_lexicon_tags_37 },
    { 0x2624d539, SUC_lexicon_tags_37 },
    { 0xecc4d539, SUC_lexicon_tags_57 },
    { 0x7444d539, SUC_lexicon_tags_29 },
    { 0x2ce4d53c, SUC_lexicon_tags_9 },
    { 0xc3e4d53c, SUC_lexicon_tags_30 },
    { 0xaf84d53c, SUC_lexicon_tags_16 },
    { 0x5544d53b, SUC_lexicon_tags_12 },
    { 0x564d540, SUC_lexicon_tags_39 },
    { 0x4c44d538, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c4d545, SUC_lexicon_tags_37 },
    { 0x7e4d546, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdba4d548, SUC_lexicon_tags_33 },
    { 0xd704d549, SUC_lexicon_tags_25 },
    { 0x61e4d548, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d64d54d, SUC_lexicon_tags_25 },
    { 0xa4c4d54d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea4d553, SUC_lexicon_tags_0 },
    { 0xe684d554, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a4d55b, SUC_lexicon_tags_13 },
    { 0x9ee4d55b, SUC_lexicon_tags_2 },
    { 0xda04d55b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca4d560, SUC_lexicon_tags_47 },
    { 0x6864d561, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704d566, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4d56a, SUC_lexicon_tags_7 },
    { 0xd4e4d56b, SUC_lexicon_tags_25 },
    { 0xb704d56b, SUC_lexicon_tags_33 },
    { 0x1be4d56d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e24d577, SUC_lexicon_tags_37 },
    { 0x3984d577, SUC_lexicon_tags_92 },
    { 0x8384d579, SUC_lexicon_tags_2 },
    { 0x8324d579, SUC_lexicon_tags_80 },
    { 0x2164d579, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5224d57d, SUC_lexicon_tags_11 },
    { 0xdc04d57e, SUC_lexicon_tags_30 },
    { 0x5524d57f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2724d584, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a24d587, SUC_lexicon_tags_75 },
    { 0xca04d588, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5524d58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd04d592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddc4d594, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3624d597, SUC_lexicon_tags_2 },
    { 0x6d84d598, SUC_lexicon_tags_11 },
    { 0xe6c4d599, SUC_lexicon_tags_47 },
    { 0x1724d59a, SUC_lexicon_tags_7 },
    { 0x3aa4d59b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb084d59d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e4d5a2, SUC_lexicon_tags_9 },
    { 0x5624d5a3, SUC_lexicon_tags_14 },
    { 0x5844d5a3, SUC_lexicon_tags_7 },
    { 0x5d84d5a5, SUC_lexicon_tags_18 },
    { 0x3e24d5a6, SUC_lexicon_tags_1 },
    { 0xb2a4d5a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa864d5ab, SUC_lexicon_tags_11 },
    { 0xc5e4d5ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e4d5b3, SUC_lexicon_tags_15 },
    { 0x3904d5b4, SUC_lexicon_tags_12 },
    { 0x2e64d5b5, SUC_lexicon_tags_8 },
    { 0x5384d5b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a4d5b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf984d5bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3004d5be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5664d5c1, SUC_lexicon_tags_6 },
    { 0x3424d5c2, SUC_lexicon_tags_0 },
    { 0x4dc4d5c1, SUC_lexicon_tags_48 },
    { 0x15e4d5c4, SUC_lexicon_tags_47 },
    { 0x6284d5c5, SUC_lexicon_tags_10 },
    { 0xe9a4d5c5, SUC_lexicon_tags_35 },
    { 0x5484d5c7, SUC_lexicon_tags_1 },
    { 0xe6a4d5c7, SUC_lexicon_tags_17 },
    { 0xf6c4d5c3, SUC_lexicon_tags_12 },
    { 0xd0a4d5c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf04d5cd, SUC_lexicon_tags_12 },
    { 0x5ac4d5ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8604d5d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c24d5d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e4d5da, SUC_lexicon_tags_34 },
    { 0xf44d5db, SUC_lexicon_tags_173 },
    { 0xd064d5db, SUC_lexicon_tags_11 },
    { 0x9d64d5da, SUC_lexicon_tags_10 },
    { 0x9644d5dd, SUC_lexicon_tags_141 },
    { 0x324d5df, SUC_lexicon_tags_15 },
    { 0xb24d5e0, SUC_lexicon_tags_65 },
    { 0x624d5e1, SUC_lexicon_tags_37 },
    { 0xaa24d5e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf04d5e4, SUC_lexicon_tags_2 },
    { 0x7e4d5e4, SUC_lexicon_tags_25 },
    { 0x704d5e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb964d5eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e4d5ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99a4d5f1, SUC_lexicon_tags_25 },
    { 0xa5c4d5f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac44d5f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd024d5f8, SUC_lexicon_tags_17 },
    { 0xa1a4d5f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e64d601, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x32a4d603, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de4d60a, SUC_lexicon_tags_30 },
    { 0x3344d60a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee04d614, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa4d61b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4404d61d, SUC_lexicon_tags_59 },
    { 0xf684d61d, SUC_lexicon_tags_7 },
    { 0x1164d61f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb324d622, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x384d624, SUC_lexicon_tags_38 },
    { 0xfd64d625, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xad84d628, SUC_lexicon_tags_61 },
    { 0x2dc4d629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8784d62b, SUC_lexicon_tags_66 },
    { 0xe1e4d62c, SUC_lexicon_tags_87 },
    { 0xdc4d62b, SUC_lexicon_tags_7 },
    { 0xff04d62e, SUC_lexicon_tags_33 },
    { 0x1ac4d62f, SUC_lexicon_tags_14 },
    { 0x87a4d630, SUC_lexicon_tags_2 },
    { 0x97c4d62f, SUC_lexicon_tags_2 },
    { 0x2f64d630, SUC_lexicon_tags_1 },
    { 0xc704d630, SUC_lexicon_tags_17 },
    { 0xad44d634, SUC_lexicon_tags_7 },
    { 0xc364d634, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa424d639, SUC_lexicon_tags_47 },
    { 0x4144d639, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8384d63e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1944d642, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1244d646, SUC_lexicon_tags_68 },
    { 0x7f04d646, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9644d64a, SUC_lexicon_tags_57 },
    { 0x97a4d64a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e4d64e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4d651, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3284d654, SUC_lexicon_tags_17 },
    { 0x2964d655, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c44d657, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab04d65a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42e4d65c, SUC_lexicon_tags_60 },
    { 0xbf84d65d, SUC_lexicon_tags_9 },
    { 0x8564d65d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ce4d660, SUC_lexicon_tags_37 },
    { 0xbee4d661, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee4d667, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd824d66b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9484d66d, SUC_lexicon_tags_14 },
    { 0x7964d66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a24d672, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x4204d675, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e4d679, SUC_lexicon_tags_45 },
    { 0x5244d679, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9824d680, SUC_lexicon_tags_25 },
    { 0x5c84d681, SUC_lexicon_tags_15 },
    { 0x6384d680, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c44d685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f84d689, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7944d68c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b04d692, SUC_lexicon_tags_15 },
    { 0x4304d693, SUC_lexicon_tags_77 },
    { 0xb0c4d692, SUC_lexicon_tags_12 },
    { 0x8c4d695, SUC_lexicon_tags_11 },
    { 0x5f24d696, SUC_lexicon_tags_56 },
    { 0x864d695, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e4d69b, SUC_lexicon_tags_7 },
    { 0x7a4d69b, SUC_lexicon_tags_1 },
    { 0x3dc4d69d, SUC_lexicon_tags_1 },
    { 0xa204d69e, SUC_lexicon_tags_14 },
    { 0x8fc4d69f, SUC_lexicon_tags_2 },
    { 0x2304d69b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3404d6a2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe444d6a4, SUC_lexicon_tags_42 },
    { 0xecc4d6a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x60a4d6a7, SUC_lexicon_tags_12 },
    { 0xe524d6a7, SUC_lexicon_tags_6 },
    { 0x8444d6a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc804d6ac, SUC_lexicon_tags_79 },
    { 0x5364d6ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2404d6af, SUC_lexicon_tags_47 },
    { 0xf264d6b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7a4d6b2, SUC_lexicon_tags_37 },
    { 0x4284d6b3, SUC_lexicon_tags_1 },
    { 0x9984d6b4, SUC_lexicon_tags_7 },
    { 0x72c4d6b4, SUC_lexicon_tags_1 },
    { 0x9a4d6b5, SUC_lexicon_tags_37 },
    { 0x5ae4d6b7, SUC_lexicon_tags_13 },
    { 0x8784d6b3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x8504d6bb, SUC_lexicon_tags_274 },
    { 0xcea4d6bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b64d6c0, SUC_lexicon_tags_10 },
    { 0x6644d6c1, SUC_lexicon_tags_31 },
    { 0xeba4d6c1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3424d6c5, SUC_lexicon_tags_7 },
    { 0x7b84d6c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4344d6c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca4d6cb, SUC_lexicon_tags_12 },
    { 0xab64d6cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1144d6ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f44d6d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f04d6da, SUC_lexicon_tags_12 },
    { 0x2324d6db, SUC_lexicon_tags_2 },
    { 0x5b24d6da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f64d6de, SUC_lexicon_tags_11 },
    { 0xf324d6de, SUC_lexicon_tags_17 },
    { 0x2684d6df, SUC_lexicon_tags_31 },
    { 0xf124d6e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a4d6e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964d6e8, SUC_lexicon_tags_17 },
    { 0x14c4d6e9, SUC_lexicon_tags_2 },
    { 0xed64d6ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc64d6ec, SUC_lexicon_tags_15 },
    { 0x90c4d6ed, SUC_lexicon_tags_47 },
    { 0x10c4d6ee, SUC_lexicon_tags_28 },
    { 0xc4a4d6ec, SUC_lexicon_tags_47 },
    { 0x2144d6ed, SUC_lexicon_tags_11 },
    { 0x8ca4d6f1, SUC_lexicon_tags_29 },
    { 0x6b44d6ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8804d6f5, SUC_lexicon_tags_29 },
    { 0xfb64d6f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be4d6f9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xac4d6fb, SUC_lexicon_tags_30 },
    { 0x33c4d6fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa04d6ff, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x27a4d701, SUC_lexicon_tags_27 },
    { 0x3584d702, SUC_lexicon_tags_16 },
    { 0x2c84d703, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4be4d705, SUC_lexicon_tags_43 },
    { 0xbd64d706, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e4d70b, SUC_lexicon_tags_11 },
    { 0x91e4d70c, SUC_lexicon_tags_11 },
    { 0xea04d70c, SUC_lexicon_tags_7 },
    { 0x48e4d70c, SUC_lexicon_tags_17 },
    { 0x2e84d70f, SUC_lexicon_tags_47 },
    { 0xb604d710, SUC_lexicon_tags_63 },
    { 0x5644d711, SUC_lexicon_tags_37 },
    { 0x4444d711, SUC_lexicon_tags_26 },
    { 0xe544d712, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xee24d715, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c24d71b, SUC_lexicon_tags_17 },
    { 0x6ca4d71c, SUC_lexicon_tags_15 },
    { 0xc064d71c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a4d721, SUC_lexicon_tags_17 },
    { 0x5a64d722, SUC_lexicon_tags_63 },
    { 0xe504d723, SUC_lexicon_tags_12 },
    { 0xec24d724, SUC_lexicon_tags_37 },
    { 0x6024d723, SUC_lexicon_tags_2 },
    { 0x2e4d722, SUC_lexicon_tags_84 },
    { 0x3c44d727, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbfa4d729, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7624d72b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e4d732, SUC_lexicon_tags_6 },
    { 0xd44d732, SUC_lexicon_tags_45 },
    { 0xfb04d734, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e64d73a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca4d73c, SUC_lexicon_tags_55 },
    { 0x3d24d73c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb2e4d73f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1344d743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc64d745, SUC_lexicon_tags_19 },
    { 0xb24d746, SUC_lexicon_tags_45 },
    { 0x4f24d747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf44d74d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e4d750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa64d754, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x224d757, SUC_lexicon_tags_27 },
    { 0x6664d757, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc6e4d75a, SUC_lexicon_tags_15 },
    { 0x6c84d75a, SUC_lexicon_tags_17 },
    { 0x2d24d75c, SUC_lexicon_tags_11 },
    { 0x9044d75c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe284d760, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5864d762, SUC_lexicon_tags_37 },
    { 0x2804d763, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f04d765, SUC_lexicon_tags_98 },
    { 0x2864d765, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1be4d768, SUC_lexicon_tags_37 },
    { 0xe4d769, SUC_lexicon_tags_25 },
    { 0x1184d768, SUC_lexicon_tags_11 },
    { 0x4684d76b, SUC_lexicon_tags_2 },
    { 0xbb64d76c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4e44d76e, SUC_lexicon_tags_47 },
    { 0x384d76f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3164d774, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d778, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb44d77d, SUC_lexicon_tags_17 },
    { 0x8304d77e, SUC_lexicon_tags_7 },
    { 0xc3c4d77f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1de4d781, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf124d783, SUC_lexicon_tags_23 },
    { 0xba24d784, SUC_lexicon_tags_25 },
    { 0xbc24d784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x724d788, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1744d78a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe64d78f, SUC_lexicon_tags_65 },
    { 0x9684d790, SUC_lexicon_tags_60 },
    { 0xe004d791, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c4d794, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe384d797, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e24d79b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc624d79e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2064d7a2, SUC_lexicon_tags_37 },
    { 0xc624d7a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e4d7a6, SUC_lexicon_tags_1 },
    { 0xa304d7a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc4d7ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb704d7b1, SUC_lexicon_tags_37 },
    { 0x62a4d7b1, SUC_lexicon_tags_48 },
    { 0x1e44d7b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2664d7b5, SUC_lexicon_tags_1 },
    { 0x36a4d7b6, SUC_lexicon_tags_23 },
    { 0xb424d7b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c4d7bc, SUC_lexicon_tags_30 },
    { 0x50c4d7bd, SUC_lexicon_tags_13 },
    { 0x5dc4d7be, SUC_lexicon_tags_7 },
    { 0xe564d7bf, SUC_lexicon_tags_7 },
    { 0x1f04d7bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e4d7c7, SUC_lexicon_tags_14 },
    { 0x3ee4d7c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6424d7cc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe424d7ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6864d7d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb024d7d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8404d7e4, SUC_lexicon_tags_19 },
    { 0x6f64d7e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44d7e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf64d7ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd264d7f1, SUC_lexicon_tags_2 },
    { 0xe184d7f2, SUC_lexicon_tags_1 },
    { 0x9c44d7f3, SUC_lexicon_tags_5 },
    { 0x6144d7f4, SUC_lexicon_tags_33 },
    { 0xb3a4d7f3, SUC_lexicon_tags_39 },
    { 0xa2a4d7f6, SUC_lexicon_tags_65 },
    { 0x5ea4d7f7, SUC_lexicon_tags_87 },
    { 0x8c04d7f2, SUC_lexicon_tags_12 },
    { 0xcf44d7f9, SUC_lexicon_tags_37 },
    { 0x94e4d7f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae4d800, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc4d803, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f84d805, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da4d80b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a4d80f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b24d815, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a4d81a, SUC_lexicon_tags_26 },
    { 0xbbe4d81b, SUC_lexicon_tags_27 },
    { 0x8484d81a, SUC_lexicon_tags_40 },
    { 0xacc4d81c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff44d821, SUC_lexicon_tags_63 },
    { 0x2ac4d822, SUC_lexicon_tags_7 },
    { 0x1264d822, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd024d82c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd624d82f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x184d832, SUC_lexicon_tags_0 },
    { 0xe904d833, SUC_lexicon_tags_46 },
    { 0xd3a4d833, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc024d837, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a4d83c, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x5884d83f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8164d841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce4d844, SUC_lexicon_tags_47 },
    { 0xa244d845, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e84d847, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d24d84b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa4d852, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15a4d854, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a24d856, SUC_lexicon_tags_39 },
    { 0x2784d857, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04d85b, SUC_lexicon_tags_174 },
    { 0x1984d85c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb44d85e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da4d861, SUC_lexicon_tags_55 },
    { 0x25e4d861, SUC_lexicon_tags_17 },
    { 0xca64d863, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ca4d865, SUC_lexicon_tags_17 },
    { 0x3764d865, SUC_lexicon_tags_60 },
    { 0xce04d867, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d44d86d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1024d873, SUC_lexicon_tags_46 },
    { 0xb5e4d874, SUC_lexicon_tags_17 },
    { 0xc3e4d875, SUC_lexicon_tags_70 },
    { 0xedc4d876, SUC_lexicon_tags_28 },
    { 0x9724d875, SUC_lexicon_tags_33 },
    { 0x92a4d878, SUC_lexicon_tags_8 },
    { 0x1284d876, SUC_lexicon_tags_2 },
    { 0x244d87a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6624d87c, SUC_lexicon_tags_33 },
    { 0x8584d87d, SUC_lexicon_tags_33 },
    { 0xd044d87e, SUC_lexicon_tags_43 },
    { 0x4684d87e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9604d881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc244d883, SUC_lexicon_tags_11 },
    { 0xc264d883, SUC_lexicon_tags_11 },
    { 0x8de4d885, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1384d887, SUC_lexicon_tags_12 },
    { 0x4984d887, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f04d88b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae4d891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa364d893, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3be4d895, SUC_lexicon_tags_7 },
    { 0x8b44d896, SUC_lexicon_tags_6 },
    { 0x2404d896, SUC_lexicon_tags_12 },
    { 0x25a4d897, SUC_lexicon_tags_1 },
    { 0x1ac4d895, SUC_lexicon_tags_7 },
    { 0x3124d899, SUC_lexicon_tags_14 },
    { 0xc6e4d89b, SUC_lexicon_tags_17 },
    { 0x4864d89a, SUC_lexicon_tags_28 },
    { 0x79a4d89d, SUC_lexicon_tags_37 },
    { 0xb784d89e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf904d8a0, SUC_lexicon_tags_12 },
    { 0xbe44d8a1, SUC_lexicon_tags_37 },
    { 0xdca4d8a1, SUC_lexicon_tags_31 },
    { 0xc4d8a3, SUC_lexicon_tags_11 },
    { 0x5464d8a3, SUC_lexicon_tags_47 },
    { 0x17a4d8a2, SUC_lexicon_tags_68 },
    { 0x5504d8a6, SUC_lexicon_tags_37 },
    { 0xf664d8a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4484d8aa, SUC_lexicon_tags_59 },
    { 0x23a4d8ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7ca4d8ad, SUC_lexicon_tags_15 },
    { 0x5944d8ad, SUC_lexicon_tags_25 },
    { 0xf584d8af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9384d8b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf524d8b8, SUC_lexicon_tags_48 },
    { 0xcda4d8b9, SUC_lexicon_tags_29 },
    { 0x2da4d8ba, SUC_lexicon_tags_15 },
    { 0x4f84d8ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xab44d8be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e84d8c1, SUC_lexicon_tags_68 },
    { 0xd664d8c2, SUC_lexicon_tags_17 },
    { 0x6c44d8c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf044d8c7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde44d8d0, SUC_lexicon_tags_34 },
    { 0x6f44d8d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e84d8d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604d8db, SUC_lexicon_tags_17 },
    { 0x9444d8dc, SUC_lexicon_tags_47 },
    { 0xe544d8dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2124d8df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04d8e2, SUC_lexicon_tags_43 },
    { 0x6f84d8e3, SUC_lexicon_tags_12 },
    { 0x5884d8e4, SUC_lexicon_tags_6 },
    { 0xc504d8e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e4d8e8, SUC_lexicon_tags_21 },
    { 0x5284d8e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8d84d8eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c4d8ee, SUC_lexicon_tags_19 },
    { 0x36c4d8ef, SUC_lexicon_tags_17 },
    { 0x224d8f0, SUC_lexicon_tags_4 },
    { 0x2884d8f0, SUC_lexicon_tags_46 },
    { 0x6fe4d8f2, SUC_lexicon_tags_17 },
    { 0x17e4d8f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4424d8f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c4d903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa904d906, SUC_lexicon_tags_17 },
    { 0xb5a4d907, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c4d90a, SUC_lexicon_tags_6 },
    { 0x3064d90b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e4d916, SUC_lexicon_tags_1 },
    { 0x8b84d917, SUC_lexicon_tags_15 },
    { 0xcc84d917, SUC_lexicon_tags_11 },
    { 0xb504d919, SUC_lexicon_tags_55 },
    { 0xe1c4d91a, SUC_lexicon_tags_12 },
    { 0x3604d91a, SUC_lexicon_tags_7 },
    { 0xd724d91c, SUC_lexicon_tags_11 },
    { 0xb764d91c, SUC_lexicon_tags_26 },
    { 0x2404d91e, SUC_lexicon_tags_43 },
    { 0xc124d91f, SUC_lexicon_tags_6 },
    { 0xbc84d91e, SUC_lexicon_tags_17 },
    { 0x4324d921, SUC_lexicon_tags_84 },
    { 0x8d04d921, SUC_lexicon_tags_27 },
    { 0x6604d91e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3c4d925, SUC_lexicon_tags_16 },
    { 0x5ca4d926, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c84d929, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c4d92c, SUC_lexicon_tags_11 },
    { 0xbe64d92d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5724d92f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xeaa4d931, SUC_lexicon_tags_37 },
    { 0xb824d932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff84d934, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6a04d936, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd444d938, SUC_lexicon_tags_47 },
    { 0x9164d939, SUC_lexicon_tags_33 },
    { 0x2b44d939, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e4d940, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e4d948, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6944d94b, SUC_lexicon_tags_107 },
    { 0xaa44d94b, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x4d44d94e, SUC_lexicon_tags_8 },
    { 0x88c4d94f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d24d953, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3124d965, SUC_lexicon_tags_11 },
    { 0xd944d965, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xdbe4d968, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd84d96a, SUC_lexicon_tags_47 },
    { 0xf324d96b, SUC_lexicon_tags_37 },
    { 0x8464d96a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb24d971, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d64d973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae4d976, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4584d979, SUC_lexicon_tags_19 },
    { 0x6d84d97a, SUC_lexicon_tags_26 },
    { 0x1c04d97b, SUC_lexicon_tags_26 },
    { 0xa464d97b, SUC_lexicon_tags_31 },
    { 0xe604d97d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe64d97f, SUC_lexicon_tags_26 },
    { 0x2284d980, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a84d982, SUC_lexicon_tags_2 },
    { 0x9fa4d982, SUC_lexicon_tags_17 },
    { 0xf504d984, SUC_lexicon_tags_60 },
    { 0x3184d985, SUC_lexicon_tags_11 },
    { 0x6564d986, SUC_lexicon_tags_19 },
    { 0x8164d986, SUC_lexicon_tags_15 },
    { 0x9f44d988, SUC_lexicon_tags_47 },
    { 0x4bc4d989, SUC_lexicon_tags_47 },
    { 0xd1e4d984, SUC_lexicon_tags_47 },
    { 0x58a4d98b, SUC_lexicon_tags_11 },
    { 0x9784d98b, SUC_lexicon_tags_15 },
    { 0x5d24d98d, SUC_lexicon_tags_34 },
    { 0xa8e4d98b, SUC_lexicon_tags_27 },
    { 0x5284d98f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d44d991, SUC_lexicon_tags_38 },
    { 0xf664d991, SUC_lexicon_tags_38 },
    { 0x1164d993, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd824d99d, SUC_lexicon_tags_5 },
    { 0xe3a4d99e, SUC_lexicon_tags_11 },
    { 0x1e84d99f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x344d9a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3264d9a3, SUC_lexicon_tags_60 },
    { 0xd6e4d9a4, SUC_lexicon_tags_15 },
    { 0x6764d9a3, SUC_lexicon_tags_27 },
    { 0x74e4d9a4, SUC_lexicon_tags_1 },
    { 0x95e4d9a7, SUC_lexicon_tags_63 },
    { 0x52a4d9a8, SUC_lexicon_tags_45 },
    { 0xd904d9a8, SUC_lexicon_tags_61 },
    { 0x6c44d9a8, SUC_lexicon_tags_39 },
    { 0x6304d9a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x544d9ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c4d9b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a4d9bb, SUC_lexicon_tags_17 },
    { 0x9264d9bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3744d9be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb924d9c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9784d9c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d64d9c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8bc4d9ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c4d9d1, SUC_lexicon_tags_25 },
    { 0x81a4d9d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c04d9d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8344d9db, SUC_lexicon_tags_47 },
    { 0x96e4d9dc, SUC_lexicon_tags_7 },
    { 0x7f64d9dc, SUC_lexicon_tags_7 },
    { 0x6e44d9de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab04d9e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb264d9e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47c4d9e6, SUC_lexicon_tags_106 },
    { 0x6c24d9e7, SUC_lexicon_tags_31 },
    { 0x8fe4d9e7, SUC_lexicon_tags_17 },
    { 0x4e24d9e7, SUC_lexicon_tags_1 },
    { 0x9fa4d9e6, SUC_lexicon_tags_56 },
    { 0x7b24d9e8, SUC_lexicon_tags_37 },
    { 0xde64d9ea, SUC_lexicon_tags_12 },
    { 0x3d24d9e9, SUC_lexicon_tags_91 },
    { 0x38c4d9ee, SUC_lexicon_tags_47 },
    { 0xbe84d9ef, SUC_lexicon_tags_19 },
    { 0xeea4d9ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e4d9f4, SUC_lexicon_tags_25 },
    { 0x324d9f5, SUC_lexicon_tags_1 },
    { 0x1fc4d9f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a84d9fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcaa4d9fc, SUC_lexicon_tags_62 },
    { 0x7f04d9fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4344da01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc404da09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9c4da0b, SUC_lexicon_tags_37 },
    { 0x9d04da0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2444da11, SUC_lexicon_tags_60 },
    { 0x87a4da12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7844da1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e44da1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1904da20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e4da23, SUC_lexicon_tags_22 },
    { 0xb824da24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70c4da26, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2fa4da28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f64da30, SUC_lexicon_tags_25 },
    { 0xaa24da31, SUC_lexicon_tags_1 },
    { 0xdf04da30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee4da3a, SUC_lexicon_tags_47 },
    { 0xcf24da3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa4da40, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea4da46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4024da49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e4da4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c4da51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe4a4da53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa384da55, SUC_lexicon_tags_11 },
    { 0x1684da56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7684da58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6684da5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf44da60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9c4da62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4c24da64, SUC_lexicon_tags_63 },
    { 0x4324da64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3c4da67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba4da6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae4da6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe424da70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe84da76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e4da7b, SUC_lexicon_tags_11 },
    { 0xad04da7c, SUC_lexicon_tags_27 },
    { 0x1a4da7b, SUC_lexicon_tags_17 },
    { 0x39a4da7d, SUC_lexicon_tags_49 },
    { 0xaca4da7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4584da82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba4da85, SUC_lexicon_tags_2 },
    { 0xc384da86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c4da89, SUC_lexicon_tags_28 },
    { 0xa2c4da8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf24da8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac4da98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa724da9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3484da9d, SUC_lexicon_tags_392 },
    { 0, NULL },
    { 0x32a4da9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd124daa2, SUC_lexicon_tags_27 },
    { 0x9124daa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c4daa9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6e44daab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c4daaf, SUC_lexicon_tags_28 },
    { 0xff04dab0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5224dab6, SUC_lexicon_tags_14 },
    { 0xb5c4dab6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6444dab9, SUC_lexicon_tags_47 },
    { 0x69e4dab9, SUC_lexicon_tags_8 },
    { 0x25e4daba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe204dabd, SUC_lexicon_tags_8 },
    { 0x97e4dabe, SUC_lexicon_tags_17 },
    { 0x7924dabf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe4dac2, SUC_lexicon_tags_1649 },
    { 0, NULL },
    { 0x4684dac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44dacb, SUC_lexicon_tags_82 },
    { 0xcde4dacb, SUC_lexicon_tags_69 },
    { 0xbd64dacd, SUC_lexicon_tags_1 },
    { 0xb324dacc, SUC_lexicon_tags_43 },
    { 0x8c04dacd, SUC_lexicon_tags_45 },
    { 0x9b24dad0, SUC_lexicon_tags_23 },
    { 0x6364dad1, SUC_lexicon_tags_7 },
    { 0x3384dad1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6584dad4, SUC_lexicon_tags_61 },
    { 0x9724dad5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x7504dad8, SUC_lexicon_tags_17 },
    { 0x6d24dad8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5504dadd, SUC_lexicon_tags_7 },
    { 0xf324dade, SUC_lexicon_tags_27 },
    { 0xeec4dadf, SUC_lexicon_tags_30 },
    { 0x5804dadd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b44dae8, SUC_lexicon_tags_33 },
    { 0xe5c4dae9, SUC_lexicon_tags_46 },
    { 0x7064daea, SUC_lexicon_tags_17 },
    { 0xdf64daea, SUC_lexicon_tags_9 },
    { 0x8b84daea, SUC_lexicon_tags_29 },
    { 0xcf84daed, SUC_lexicon_tags_37 },
    { 0xc804daee, SUC_lexicon_tags_10 },
    { 0x52c4daec, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9f24daf1, SUC_lexicon_tags_37 },
    { 0xa944daf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb964daf5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc4daf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe904dafc, SUC_lexicon_tags_11 },
    { 0x9c24dafd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec4db02, SUC_lexicon_tags_7 },
    { 0xa284db03, SUC_lexicon_tags_2 },
    { 0xfc44db03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6a4db06, SUC_lexicon_tags_5 },
    { 0x6ae4db07, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc4db0d, SUC_lexicon_tags_37 },
    { 0xb964db0d, SUC_lexicon_tags_25 },
    { 0x4884db0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e4db12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f04db15, SUC_lexicon_tags_7 },
    { 0xa5a4db15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a4db19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4224db1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3904db21, SUC_lexicon_tags_7 },
    { 0x1c24db22, SUC_lexicon_tags_1 },
    { 0x1744db23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d24db27, SUC_lexicon_tags_21 },
    { 0x2284db28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a4db2b, SUC_lexicon_tags_11 },
    { 0x33a4db2c, SUC_lexicon_tags_7 },
    { 0x1ae4db2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d44db36, SUC_lexicon_tags_87 },
    { 0x9ec4db37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4944db3c, SUC_lexicon_tags_27 },
    { 0x9304db3c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa4db40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8864db43, SUC_lexicon_tags_15 },
    { 0x7464db44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5264db47, SUC_lexicon_tags_11 },
    { 0x3aa4db48, SUC_lexicon_tags_11 },
    { 0x7964db47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8064db4e, SUC_lexicon_tags_28 },
    { 0xe8e4db4f, SUC_lexicon_tags_28 },
    { 0x5964db50, SUC_lexicon_tags_38 },
    { 0x2944db51, SUC_lexicon_tags_57 },
    { 0x5844db52, SUC_lexicon_tags_56 },
    { 0xee04db52, SUC_lexicon_tags_15 },
    { 0x1664db4f, SUC_lexicon_tags_60 },
    { 0x4da4db55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4db59, SUC_lexicon_tags_7 },
    { 0x88c4db5a, SUC_lexicon_tags_107 },
    { 0x6f24db5a, SUC_lexicon_tags_38 },
    { 0xca24db5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd04db61, SUC_lexicon_tags_14 },
    { 0x9ce4db62, SUC_lexicon_tags_39 },
    { 0x84db62, SUC_lexicon_tags_107 },
    { 0xff44db64, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7044db70, SUC_lexicon_tags_11 },
    { 0x8f24db71, SUC_lexicon_tags_17 },
    { 0x6244db70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa524db76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa4db7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe244db7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e4db81, SUC_lexicon_tags_15 },
    { 0xdce4db81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf384db86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaf84db88, SUC_lexicon_tags_12 },
    { 0x7004db89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4db8d, SUC_lexicon_tags_10 },
    { 0x16a4db8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d44db90, SUC_lexicon_tags_48 },
    { 0xbee4db90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2164db93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d24db98, SUC_lexicon_tags_57 },
    { 0x9d24db99, SUC_lexicon_tags_11 },
    { 0xd064db9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a4db9f, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe4dba2, SUC_lexicon_tags_49 },
    { 0x1d24dba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a04dba7, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda24dbac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf984dbae, SUC_lexicon_tags_0 },
    { 0x6764dbaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a4dbb3, SUC_lexicon_tags_47 },
    { 0x98a4dbb3, SUC_lexicon_tags_11 },
    { 0xf304dbb5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x2d24dbb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x864dbba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a4dbbf, SUC_lexicon_tags_43 },
    { 0x3c84dbc0, SUC_lexicon_tags_29 },
    { 0x93e4dbc1, SUC_lexicon_tags_7 },
    { 0x1c84dbc2, SUC_lexicon_tags_7 },
    { 0x8b44dbc1, SUC_lexicon_tags_43 },
    { 0x6444dbc4, SUC_lexicon_tags_160 },
    { 0x8fc4dbc0, SUC_lexicon_tags_11 },
    { 0xcda4dbc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44dbce, SUC_lexicon_tags_39 },
    { 0x7a24dbcf, SUC_lexicon_tags_43 },
    { 0xc5e4dbd0, SUC_lexicon_tags_15 },
    { 0x81a4dbd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8024dbd5, SUC_lexicon_tags_28 },
    { 0x99e4dbd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf564dbd8, SUC_lexicon_tags_17 },
    { 0xdfa4dbd9, SUC_lexicon_tags_1 },
    { 0xee64dbda, SUC_lexicon_tags_17 },
    { 0xe684dbd9, SUC_lexicon_tags_2 },
    { 0x4424dbdc, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c4dbe4, SUC_lexicon_tags_30 },
    { 0xd484dbe4, SUC_lexicon_tags_48 },
    { 0x41a4dbe6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe4dbe9, SUC_lexicon_tags_43 },
    { 0xc384dbe9, SUC_lexicon_tags_26 },
    { 0x8064dbeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4904dbf0, SUC_lexicon_tags_37 },
    { 0x7444dbf0, SUC_lexicon_tags_47 },
    { 0xe3e4dbf2, SUC_lexicon_tags_25 },
    { 0x2a64dbf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a4dbf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf124dbfb, SUC_lexicon_tags_26 },
    { 0xf524dbfc, SUC_lexicon_tags_37 },
    { 0x3b84dbfc, SUC_lexicon_tags_12 },
    { 0x4ca4dbfe, SUC_lexicon_tags_30 },
    { 0xc324dbfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb44dc01, SUC_lexicon_tags_83 },
    { 0xeb64dc02, SUC_lexicon_tags_1 },
    { 0xe864dc01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c4dc06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd704dc0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb264dc0d, SUC_lexicon_tags_65 },
    { 0x93a4dc0e, SUC_lexicon_tags_9 },
    { 0x55e4dc0f, SUC_lexicon_tags_133 },
    { 0xada4dc0d, SUC_lexicon_tags_47 },
    { 0x5544dc0d, SUC_lexicon_tags_2 },
    { 0x93e4dc12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a4dc18, SUC_lexicon_tags_3 },
    { 0x8004dc18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa744dc1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5024dc20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc4dc23, SUC_lexicon_tags_3 },
    { 0xb024dc24, SUC_lexicon_tags_21 },
    { 0x7be4dc25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ce4dc27, SUC_lexicon_tags_14 },
    { 0x1664dc28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4764dc2a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe844dc2e, SUC_lexicon_tags_2 },
    { 0x2a44dc2f, SUC_lexicon_tags_17 },
    { 0xad44dc2e, SUC_lexicon_tags_83 },
    { 0xfc24dc30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e4dc33, SUC_lexicon_tags_106 },
    { 0x5d64dc34, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e4dc37, SUC_lexicon_tags_9 },
    { 0x324dc38, SUC_lexicon_tags_25 },
    { 0xc324dc39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa144dc3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d64dc40, SUC_lexicon_tags_37 },
    { 0xefa4dc41, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x19e4dc43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc584dc45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4dc49, SUC_lexicon_tags_16 },
    { 0x8684dc4a, SUC_lexicon_tags_26 },
    { 0x9d64dc4b, SUC_lexicon_tags_7 },
    { 0x61e4dc4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04dc4f, SUC_lexicon_tags_27 },
    { 0xa2e4dc50, SUC_lexicon_tags_12 },
    { 0x64dc51, SUC_lexicon_tags_19 },
    { 0x9b84dc52, SUC_lexicon_tags_17 },
    { 0x9ec4dc53, SUC_lexicon_tags_25 },
    { 0x6504dc54, SUC_lexicon_tags_7 },
    { 0x6064dc55, SUC_lexicon_tags_38 },
    { 0x17a4dc53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5124dc5f, SUC_lexicon_tags_11 },
    { 0x45e4dc5f, SUC_lexicon_tags_14 },
    { 0x18a4dc60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd244dc64, SUC_lexicon_tags_33 },
    { 0x6104dc65, SUC_lexicon_tags_34 },
    { 0xd544dc66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf144dc68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70e4dc6a, SUC_lexicon_tags_15 },
    { 0x8da4dc6a, SUC_lexicon_tags_87 },
    { 0x6c24dc6b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c4dc6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e4dc72, SUC_lexicon_tags_17 },
    { 0x38c4dc72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a4dc77, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a4dc7b, SUC_lexicon_tags_25 },
    { 0x3204dc7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa4dc80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc24dc83, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf284dc86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3824dc89, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa764dc8c, SUC_lexicon_tags_12 },
    { 0x5ac4dc8d, SUC_lexicon_tags_15 },
    { 0x3924dc8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a4dc91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc4dc94, SUC_lexicon_tags_1 },
    { 0x5764dc95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc4dc98, SUC_lexicon_tags_8 },
    { 0x7a64dc99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc4dc9c, SUC_lexicon_tags_7 },
    { 0xa04dc9d, SUC_lexicon_tags_9 },
    { 0x6784dc9d, SUC_lexicon_tags_25 },
    { 0x7e04dc9f, SUC_lexicon_tags_37 },
    { 0xc6c4dca0, SUC_lexicon_tags_17 },
    { 0x2b24dc9d, SUC_lexicon_tags_19 },
    { 0x55c4dca0, SUC_lexicon_tags_1 },
    { 0xf604dc9c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6524dca5, SUC_lexicon_tags_25 },
    { 0xe404dca5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3444dca8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7324dcaa, SUC_lexicon_tags_17 },
    { 0x15e4dcab, SUC_lexicon_tags_30 },
    { 0xe3e4dcab, SUC_lexicon_tags_45 },
    { 0x2f64dcad, SUC_lexicon_tags_37 },
    { 0x56a4dcac, SUC_lexicon_tags_6 },
    { 0x36c4dcaf, SUC_lexicon_tags_47 },
    { 0xbee4dcb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc4dcb3, SUC_lexicon_tags_64 },
    { 0xcac4dcb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa4dcb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a4dcbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78c4dcbe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9744dcc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf284dcc6, SUC_lexicon_tags_28 },
    { 0xbee4dcc6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb0c4dcc9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e84dccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a4dcd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5584dcd7, SUC_lexicon_tags_25 },
    { 0x1124dcd7, SUC_lexicon_tags_11 },
    { 0x93c4dcd8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2e4dcdb, SUC_lexicon_tags_35 },
    { 0x52a4dcdc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b24dcdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6844dce2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd904dce4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8984dce7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8184dcee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e44dcf0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe404dcf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc624dcf5, SUC_lexicon_tags_19 },
    { 0x35e4dcf5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd984dcfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68e4dcfc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9524dcff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b24dd02, SUC_lexicon_tags_11 },
    { 0xf1c4dd03, SUC_lexicon_tags_63 },
    { 0xfe4dd04, SUC_lexicon_tags_8 },
    { 0x4844dd05, SUC_lexicon_tags_12 },
    { 0xf1e4dd05, SUC_lexicon_tags_31 },
    { 0x81e4dd07, SUC_lexicon_tags_19 },
    { 0xfa04dd08, SUC_lexicon_tags_23 },
    { 0xa7c4dd08, SUC_lexicon_tags_30 },
    { 0x9dc4dd05, SUC_lexicon_tags_15 },
    { 0xde24dd0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7704dd0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d84dd12, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x69a4dd14, SUC_lexicon_tags_11 },
    { 0x81a4dd15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd64dd17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2a04dd19, SUC_lexicon_tags_17 },
    { 0xa084dd1a, SUC_lexicon_tags_7 },
    { 0x7384dd1b, SUC_lexicon_tags_37 },
    { 0xc9e4dd19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xde64dd1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbc84dd21, SUC_lexicon_tags_17 },
    { 0xdfe4dd21, SUC_lexicon_tags_11 },
    { 0xa2c4dd21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa4dd26, SUC_lexicon_tags_23 },
    { 0x50a4dd27, SUC_lexicon_tags_32 },
    { 0x41c4dd28, SUC_lexicon_tags_37 },
    { 0xa7e4dd26, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf44dd2d, SUC_lexicon_tags_40 },
    { 0x55a4dd2d, SUC_lexicon_tags_37 },
    { 0x9e4dd2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a64dd31, SUC_lexicon_tags_37 },
    { 0xf644dd32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc4dd39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c4dd3c, SUC_lexicon_tags_17 },
    { 0x96c4dd3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe244dd41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc4dd45, SUC_lexicon_tags_11 },
    { 0xa484dd46, SUC_lexicon_tags_25 },
    { 0x5224dd47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fe4dd49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc684dd4c, SUC_lexicon_tags_1332 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9204dd51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e4dd55, SUC_lexicon_tags_17 },
    { 0x9404dd55, SUC_lexicon_tags_49 },
    { 0x1244dd57, SUC_lexicon_tags_25 },
    { 0x8704dd58, SUC_lexicon_tags_43 },
    { 0x4304dd57, SUC_lexicon_tags_42 },
    { 0xa484dd5a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x49a4dd5c, SUC_lexicon_tags_17 },
    { 0xbb04dd5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b04dd61, SUC_lexicon_tags_64 },
    { 0xc0e4dd62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c4dd66, SUC_lexicon_tags_12 },
    { 0xe1a4dd67, SUC_lexicon_tags_6 },
    { 0x7ac4dd66, SUC_lexicon_tags_2 },
    { 0x85a4dd66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e84dd6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c04dd73, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1744dd76, SUC_lexicon_tags_15 },
    { 0x1284dd77, SUC_lexicon_tags_2 },
    { 0x4dc4dd77, SUC_lexicon_tags_7 },
    { 0x3304dd79, SUC_lexicon_tags_15 },
    { 0x6544dd79, SUC_lexicon_tags_37 },
    { 0x1324dd7b, SUC_lexicon_tags_45 },
    { 0xf424dd77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74e4dd7e, SUC_lexicon_tags_19 },
    { 0xfe04dd7f, SUC_lexicon_tags_29 },
    { 0x8164dd80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7944dd83, SUC_lexicon_tags_37 },
    { 0x9604dd84, SUC_lexicon_tags_27 },
    { 0xbc64dd85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfa4dd87, SUC_lexicon_tags_0 },
    { 0x1384dd88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe064dd8d, SUC_lexicon_tags_68 },
    { 0x3944dd8d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4744dd92, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9184dd94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa744dd96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf804dd99, SUC_lexicon_tags_12 },
    { 0xbd24dd99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe164dd9f, SUC_lexicon_tags_25 },
    { 0x7304dda0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf144dda7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84ddab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e4ddae, SUC_lexicon_tags_23 },
    { 0x9464ddae, SUC_lexicon_tags_23 },
    { 0xa2e4ddb0, SUC_lexicon_tags_25 },
    { 0x25c4ddb1, SUC_lexicon_tags_87 },
    { 0x3e64ddb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5564ddb4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7164ddb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f84ddba, SUC_lexicon_tags_6 },
    { 0x9dc4ddba, SUC_lexicon_tags_87 },
    { 0x35c4ddbc, SUC_lexicon_tags_12 },
    { 0x99c4ddbc, SUC_lexicon_tags_25 },
    { 0x9de4ddbe, SUC_lexicon_tags_49 },
    { 0x4744ddbf, SUC_lexicon_tags_23 },
    { 0xf664ddbe, SUC_lexicon_tags_8 },
    { 0xeea4ddc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed44ddc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x884ddc9, SUC_lexicon_tags_510 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7444ddcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba4ddd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c24ddd7, SUC_lexicon_tags_1 },
    { 0x2fc4ddd7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5544ddda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85c4dddc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9aa4ddde, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a4dde3, SUC_lexicon_tags_37 },
    { 0x1784dde4, SUC_lexicon_tags_14 },
    { 0xc6e4dde3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7764dde7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6184dde9, SUC_lexicon_tags_12 },
    { 0x1aa4ddea, SUC_lexicon_tags_11 },
    { 0x1c4ddeb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a4ddf0, SUC_lexicon_tags_85 },
    { 0x8784ddf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc24ddf6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf84ddfb, SUC_lexicon_tags_34 },
    { 0x79a4ddfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9544ddfe, SUC_lexicon_tags_0 },
    { 0xdb04ddfe, SUC_lexicon_tags_23 },
    { 0x4664ddfe, SUC_lexicon_tags_26 },
    { 0x3a4ddff, SUC_lexicon_tags_11 },
    { 0x80c4de02, SUC_lexicon_tags_47 },
    { 0x1224de03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47a4de05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3524de07, SUC_lexicon_tags_26 },
    { 0xbda4de07, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1f64de0a, SUC_lexicon_tags_11 },
    { 0x4b04de0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c4de0f, SUC_lexicon_tags_11 },
    { 0x49c4de10, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x5804de13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e4de16, SUC_lexicon_tags_7 },
    { 0x9a84de17, SUC_lexicon_tags_47 },
    { 0x1d64de17, SUC_lexicon_tags_0 },
    { 0x7464de19, SUC_lexicon_tags_17 },
    { 0xcce4de17, SUC_lexicon_tags_19 },
    { 0x3604de1b, SUC_lexicon_tags_28 },
    { 0x7ce4de19, SUC_lexicon_tags_27 },
    { 0x31a4de1c, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0x78e4de1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c4de26, SUC_lexicon_tags_43 },
    { 0x7d64de26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c44de29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2524de2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x14c4de2d, SUC_lexicon_tags_8 },
    { 0xe6c4de2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5c4de30, SUC_lexicon_tags_56 },
    { 0x21c4de31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe684de37, SUC_lexicon_tags_25 },
    { 0x5f04de37, SUC_lexicon_tags_11 },
    { 0x124de37, SUC_lexicon_tags_33 },
    { 0x21a4de3a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a4de3d, SUC_lexicon_tags_37 },
    { 0x2144de3d, SUC_lexicon_tags_11 },
    { 0xc884de3d, SUC_lexicon_tags_7 },
    { 0x6704de40, SUC_lexicon_tags_7 },
    { 0x7ce4de41, SUC_lexicon_tags_11 },
    { 0xf944de41, SUC_lexicon_tags_11 },
    { 0xdd64de41, SUC_lexicon_tags_46 },
    { 0xc684de40, SUC_lexicon_tags_10 },
    { 0x5924de45, SUC_lexicon_tags_27 },
    { 0x2384de46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1684de4a, SUC_lexicon_tags_7 },
    { 0x3764de4b, SUC_lexicon_tags_11 },
    { 0x5844de4b, SUC_lexicon_tags_17 },
    { 0xa304de4d, SUC_lexicon_tags_7 },
    { 0xec24de4d, SUC_lexicon_tags_7 },
    { 0xb884de4f, SUC_lexicon_tags_7 },
    { 0xbc64de50, SUC_lexicon_tags_1 },
    { 0xdda4de51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee4de54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2584de56, SUC_lexicon_tags_31 },
    { 0xe8a4de56, SUC_lexicon_tags_29 },
    { 0xc84de58, SUC_lexicon_tags_45 },
    { 0x4a4de56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc4de5c, SUC_lexicon_tags_7 },
    { 0x484de5c, SUC_lexicon_tags_18 },
    { 0xac04de5e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f24de62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a24de68, SUC_lexicon_tags_30 },
    { 0x5424de69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c4de6e, SUC_lexicon_tags_17 },
    { 0x7764de6e, SUC_lexicon_tags_63 },
    { 0x624de70, SUC_lexicon_tags_17 },
    { 0x7704de6e, SUC_lexicon_tags_47 },
    { 0x2564de70, SUC_lexicon_tags_12 },
    { 0x38a4de73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4384de75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b44de7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4684de81, SUC_lexicon_tags_55 },
    { 0xd044de82, SUC_lexicon_tags_12 },
    { 0x8b24de83, SUC_lexicon_tags_28 },
    { 0x4524de83, SUC_lexicon_tags_28 },
    { 0xc944de85, SUC_lexicon_tags_37 },
    { 0x48e4de85, SUC_lexicon_tags_1639 },
    { 0, NULL },
    { 0x12e4de88, SUC_lexicon_tags_7 },
    { 0xae44de88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb204de8e, SUC_lexicon_tags_14 },
    { 0xa4e4de8f, SUC_lexicon_tags_17 },
    { 0x7784de90, SUC_lexicon_tags_47 },
    { 0x2d84de91, SUC_lexicon_tags_38 },
    { 0xb9e4de90, SUC_lexicon_tags_25 },
    { 0x3c24de93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e64de98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xedc4de9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9424de9d, SUC_lexicon_tags_70 },
    { 0x2e84de9e, SUC_lexicon_tags_10 },
    { 0xfaa4de9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5724dea1, SUC_lexicon_tags_1 },
    { 0x96a4dea2, SUC_lexicon_tags_31 },
    { 0x4d44dea3, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0x3b84dea5, SUC_lexicon_tags_174 },
    { 0x44e4dea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb124deaa, SUC_lexicon_tags_25 },
    { 0x8024deab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a24dead, SUC_lexicon_tags_63 },
    { 0x3344deae, SUC_lexicon_tags_13 },
    { 0x5744deae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3364deb3, SUC_lexicon_tags_33 },
    { 0x73e4deb4, SUC_lexicon_tags_12 },
    { 0x4e64deb5, SUC_lexicon_tags_15 },
    { 0x7004deb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3864deb8, SUC_lexicon_tags_30 },
    { 0xf04deb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc044dec0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49e4dec2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba4decc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba24ded2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede4deda, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec4dee0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a4dee5, SUC_lexicon_tags_15 },
    { 0x7424dee6, SUC_lexicon_tags_0 },
    { 0xf7a4dee6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e4deec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4884deef, SUC_lexicon_tags_11 },
    { 0x7a4deef, SUC_lexicon_tags_37 },
    { 0x8604def1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac4def4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x67e4def6, SUC_lexicon_tags_1 },
    { 0xf364def7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd24def9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c4defc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8384df05, SUC_lexicon_tags_17 },
    { 0x1d44df05, SUC_lexicon_tags_182 },
    { 0x7384df07, SUC_lexicon_tags_11 },
    { 0x9ba4df08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12e4df0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b84df0c, SUC_lexicon_tags_7 },
    { 0x9d84df0c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4504df0f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x96a4df11, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7304df15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7164df18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa84df1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e4df1e, SUC_lexicon_tags_59 },
    { 0x84c4df1f, SUC_lexicon_tags_2 },
    { 0x6844df20, SUC_lexicon_tags_12 },
    { 0x9d64df20, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea24df27, SUC_lexicon_tags_15 },
    { 0xb044df28, SUC_lexicon_tags_0 },
    { 0xac24df28, SUC_lexicon_tags_17 },
    { 0xaee4df28, SUC_lexicon_tags_2 },
    { 0x12e4df27, SUC_lexicon_tags_33 },
    { 0xd3a4df2b, SUC_lexicon_tags_17 },
    { 0xfba4df2d, SUC_lexicon_tags_12 },
    { 0xe9c4df28, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xc704df30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e04df37, SUC_lexicon_tags_11 },
    { 0x9804df38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc224df3b, SUC_lexicon_tags_2 },
    { 0x2a4df3c, SUC_lexicon_tags_7 },
    { 0xa684df3d, SUC_lexicon_tags_37 },
    { 0x6c84df3d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e4df45, SUC_lexicon_tags_4 },
    { 0x2da4df46, SUC_lexicon_tags_19 },
    { 0x8364df47, SUC_lexicon_tags_31 },
    { 0x74c4df47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb824df50, SUC_lexicon_tags_11 },
    { 0x5444df50, SUC_lexicon_tags_14 },
    { 0xf824df52, SUC_lexicon_tags_25 },
    { 0xf64df52, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e4df59, SUC_lexicon_tags_26 },
    { 0x96c4df59, SUC_lexicon_tags_7 },
    { 0x6024df5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5c4df5d, SUC_lexicon_tags_0 },
    { 0x1284df5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c24df64, SUC_lexicon_tags_17 },
    { 0xe7c4df65, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4144df67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe364df6b, SUC_lexicon_tags_25 },
    { 0x5bc4df6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f24df70, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba4df75, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e4df7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe784df7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de4df7f, SUC_lexicon_tags_6 },
    { 0xd2a4df7f, SUC_lexicon_tags_33 },
    { 0xff64df81, SUC_lexicon_tags_14 },
    { 0xa3a4df81, SUC_lexicon_tags_47 },
    { 0x6704df83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2f04df85, SUC_lexicon_tags_7 },
    { 0x4884df86, SUC_lexicon_tags_11 },
    { 0xb304df85, SUC_lexicon_tags_25 },
    { 0xe944df86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb164df8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb864df92, SUC_lexicon_tags_68 },
    { 0x1744df92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a04df96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e64df98, SUC_lexicon_tags_11 },
    { 0x2144df98, SUC_lexicon_tags_34 },
    { 0x57a4df9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e4df9f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f04dfa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7604dfaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1584dfad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a4dfb1, SUC_lexicon_tags_27 },
    { 0x68e4dfb2, SUC_lexicon_tags_26 },
    { 0x2da4dfb3, SUC_lexicon_tags_1 },
    { 0x2004dfb1, SUC_lexicon_tags_10 },
    { 0x7a64dfb1, SUC_lexicon_tags_1 },
    { 0xbbe4dfb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe2a4dfb8, SUC_lexicon_tags_0 },
    { 0x924dfb9, SUC_lexicon_tags_2 },
    { 0xc524dfba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5284dfbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a4dfc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1684dfc5, SUC_lexicon_tags_19 },
    { 0x1364dfc5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5364dfc9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9ba4dfcb, SUC_lexicon_tags_60 },
    { 0x1ce4dfcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xac84dfce, SUC_lexicon_tags_6 },
    { 0xb024dfcf, SUC_lexicon_tags_43 },
    { 0x964dfcf, SUC_lexicon_tags_25 },
    { 0xf884dfd1, SUC_lexicon_tags_1 },
    { 0x3da4dfd2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8304dfd5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e4dfe0, SUC_lexicon_tags_33 },
    { 0xfe84dfe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd304dfe3, SUC_lexicon_tags_6 },
    { 0x5ec4dfe4, SUC_lexicon_tags_9 },
    { 0x59e4dfe5, SUC_lexicon_tags_47 },
    { 0xc8a4dfe4, SUC_lexicon_tags_38 },
    { 0xa1e4dfe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9424dfec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef64dfee, SUC_lexicon_tags_29 },
    { 0x50c4dfef, SUC_lexicon_tags_45 },
    { 0x46a4dfef, SUC_lexicon_tags_13 },
    { 0xe464dff0, SUC_lexicon_tags_37 },
    { 0xf4a4dff2, SUC_lexicon_tags_15 },
    { 0x1924dff2, SUC_lexicon_tags_31 },
    { 0xd384dff4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x104dff6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9184dff9, SUC_lexicon_tags_25 },
    { 0x3e4dffa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe04dffd, SUC_lexicon_tags_8 },
    { 0x55e4dffe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5184e005, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a44e00b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e84e010, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xf804e013, SUC_lexicon_tags_1 },
    { 0xe4e014, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a4e019, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5804e01c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa764e01f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbca4e021, SUC_lexicon_tags_12 },
    { 0x8104e022, SUC_lexicon_tags_0 },
    { 0x26c4e022, SUC_lexicon_tags_11 },
    { 0xb84e024, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc4e02e, SUC_lexicon_tags_7 },
    { 0xc144e02e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f44e031, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2744e033, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a4e037, SUC_lexicon_tags_1 },
    { 0xcd24e038, SUC_lexicon_tags_12 },
    { 0x9fc4e039, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7924e03d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4044e040, SUC_lexicon_tags_151 },
    { 0x5704e040, SUC_lexicon_tags_68 },
    { 0xb144e042, SUC_lexicon_tags_43 },
    { 0x3be4e043, SUC_lexicon_tags_2 },
    { 0x61a4e044, SUC_lexicon_tags_14 },
    { 0xf84e045, SUC_lexicon_tags_31 },
    { 0xe964e041, SUC_lexicon_tags_30 },
    { 0xb1a4e044, SUC_lexicon_tags_11 },
    { 0xc744e048, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf264e04a, SUC_lexicon_tags_0 },
    { 0x1de4e04b, SUC_lexicon_tags_37 },
    { 0x3644e04b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xec84e04e, SUC_lexicon_tags_15 },
    { 0x9f64e04e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xacc4e051, SUC_lexicon_tags_2 },
    { 0x1184e051, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab24e059, SUC_lexicon_tags_25 },
    { 0x7124e05a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe824e064, SUC_lexicon_tags_15 },
    { 0xd484e065, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d64e067, SUC_lexicon_tags_0 },
    { 0xda24e067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd904e06c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6a64e06e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e4e071, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1e4e073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4364e078, SUC_lexicon_tags_8 },
    { 0x1f64e079, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc984e07c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7064e080, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6004e083, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c24e086, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc2c4e088, SUC_lexicon_tags_11 },
    { 0x3864e089, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x364e08c, SUC_lexicon_tags_49 },
    { 0x5304e08d, SUC_lexicon_tags_8 },
    { 0x32e4e08e, SUC_lexicon_tags_31 },
    { 0xcda4e08e, SUC_lexicon_tags_33 },
    { 0xaa64e08e, SUC_lexicon_tags_1 },
    { 0xa724e091, SUC_lexicon_tags_17 },
    { 0xf144e091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd124e094, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1504e096, SUC_lexicon_tags_65 },
    { 0x8ee4e096, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f64e099, SUC_lexicon_tags_37 },
    { 0x53c4e099, SUC_lexicon_tags_39 },
    { 0x4084e099, SUC_lexicon_tags_15 },
    { 0x8204e09c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a4e09f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5aa4e0a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c4e0a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd84e0a7, SUC_lexicon_tags_17 },
    { 0xba04e0a7, SUC_lexicon_tags_94 },
    { 0xefa4e0a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d44e0ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5144e0b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ea4e0b2, SUC_lexicon_tags_60 },
    { 0x1484e0b3, SUC_lexicon_tags_25 },
    { 0x1824e0b4, SUC_lexicon_tags_12 },
    { 0xa064e0b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b24e0bd, SUC_lexicon_tags_17 },
    { 0xc0c4e0bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x98c4e0c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff84e0c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3484e0c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee4e0d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca4e0d5, SUC_lexicon_tags_10 },
    { 0x8504e0d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbec4e0d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e4e0de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc564e0e0, SUC_lexicon_tags_264 },
    { 0x56a4e0e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b24e0e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2764e0eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2704e0f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b24e0f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4464e0f8, SUC_lexicon_tags_174 },
    { 0xfea4e0f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe84e0fc, SUC_lexicon_tags_1 },
    { 0x4ac4e0fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8c4e0ff, SUC_lexicon_tags_15 },
    { 0x6204e100, SUC_lexicon_tags_11 },
    { 0x1b04e101, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab64e105, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c4e109, SUC_lexicon_tags_2 },
    { 0x1dc4e109, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a4e10f, SUC_lexicon_tags_1 },
    { 0xad04e110, SUC_lexicon_tags_48 },
    { 0x9324e10f, SUC_lexicon_tags_9 },
    { 0x2904e112, SUC_lexicon_tags_26 },
    { 0xe8e4e112, SUC_lexicon_tags_39 },
    { 0x1b64e114, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d04e11a, SUC_lexicon_tags_25 },
    { 0x6864e11b, SUC_lexicon_tags_33 },
    { 0x3844e11c, SUC_lexicon_tags_65 },
    { 0x5ec4e11d, SUC_lexicon_tags_1 },
    { 0x53a4e11d, SUC_lexicon_tags_27 },
    { 0xc704e11f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x33e4e121, SUC_lexicon_tags_28 },
    { 0x6924e121, SUC_lexicon_tags_2 },
    { 0xaf04e123, SUC_lexicon_tags_68 },
    { 0xd924e122, SUC_lexicon_tags_12 },
    { 0xd924e125, SUC_lexicon_tags_1 },
    { 0xbfc4e125, SUC_lexicon_tags_2 },
    { 0x5b44e125, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda64e12b, SUC_lexicon_tags_0 },
    { 0xf004e12b, SUC_lexicon_tags_11 },
    { 0x1a64e12c, SUC_lexicon_tags_8 },
    { 0x4164e12e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec84e133, SUC_lexicon_tags_23 },
    { 0x5304e133, SUC_lexicon_tags_0 },
    { 0xe0c4e134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624e13f, SUC_lexicon_tags_70 },
    { 0xcc64e13f, SUC_lexicon_tags_77 },
    { 0xe7e4e141, SUC_lexicon_tags_49 },
    { 0x1f64e13f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd524e144, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a4e14d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x48a4e14f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8684e153, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c4e159, SUC_lexicon_tags_45 },
    { 0x5824e159, SUC_lexicon_tags_11 },
    { 0xfba4e159, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa44e15f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1464e163, SUC_lexicon_tags_25 },
    { 0xf7c4e164, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6124e166, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3744e16a, SUC_lexicon_tags_1 },
    { 0xba84e16b, SUC_lexicon_tags_61 },
    { 0xfe04e16b, SUC_lexicon_tags_25 },
    { 0x8204e16a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x7444e170, SUC_lexicon_tags_37 },
    { 0xfd64e171, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344e179, SUC_lexicon_tags_39 },
    { 0x86e4e17a, SUC_lexicon_tags_37 },
    { 0xd2c4e17a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c4e181, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c4e186, SUC_lexicon_tags_26 },
    { 0xdb84e187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcc4e189, SUC_lexicon_tags_35 },
    { 0x5ce4e18a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb624e190, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3484e192, SUC_lexicon_tags_12 },
    { 0xa0a4e192, SUC_lexicon_tags_47 },
    { 0xd0e4e194, SUC_lexicon_tags_37 },
    { 0x8484e195, SUC_lexicon_tags_17 },
    { 0x9324e196, SUC_lexicon_tags_43 },
    { 0x8c64e197, SUC_lexicon_tags_15 },
    { 0x4c44e198, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13e4e19a, SUC_lexicon_tags_25 },
    { 0xd1e4e19b, SUC_lexicon_tags_26 },
    { 0xc364e19c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c4e19f, SUC_lexicon_tags_33 },
    { 0x644e1a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34e4e1a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c4e1a5, SUC_lexicon_tags_33 },
    { 0xb164e1a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e04e1a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a04e1ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8724e1b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae4e1b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a4e1bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a4e1c3, SUC_lexicon_tags_2 },
    { 0xc0e4e1c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2404e1c7, SUC_lexicon_tags_32 },
    { 0x5764e1c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984e1cc, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3344e1d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb7c4e1d3, SUC_lexicon_tags_83 },
    { 0xa584e1d4, SUC_lexicon_tags_1 },
    { 0x9fc4e1d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc764e1da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb344e1dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a4e1df, SUC_lexicon_tags_38 },
    { 0x8144e1e0, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a4e1e3, SUC_lexicon_tags_19 },
    { 0xce44e1e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b24e1ef, SUC_lexicon_tags_17 },
    { 0xe524e1f0, SUC_lexicon_tags_43 },
    { 0xaa04e1f0, SUC_lexicon_tags_11 },
    { 0x9cc4e1f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4e1f4, SUC_lexicon_tags_11 },
    { 0x9ce4e1f4, SUC_lexicon_tags_31 },
    { 0x7b04e1f6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e4e1f9, SUC_lexicon_tags_7 },
    { 0xbf84e1fa, SUC_lexicon_tags_12 },
    { 0x6424e1fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf804e202, SUC_lexicon_tags_11 },
    { 0xcc84e202, SUC_lexicon_tags_25 },
    { 0xa3a4e204, SUC_lexicon_tags_48 },
    { 0x8524e205, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xc984e207, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa4e20a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa64e210, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2444e212, SUC_lexicon_tags_37 },
    { 0x20a4e212, SUC_lexicon_tags_1 },
    { 0x43e4e213, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe524e218, SUC_lexicon_tags_21 },
    { 0xab64e219, SUC_lexicon_tags_2 },
    { 0xbd24e21a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf164e21c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac4e221, SUC_lexicon_tags_17 },
    { 0x6a4e221, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e4e225, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6184e228, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe4e230, SUC_lexicon_tags_37 },
    { 0x38c4e231, SUC_lexicon_tags_29 },
    { 0xfb44e232, SUC_lexicon_tags_53 },
    { 0x7224e230, SUC_lexicon_tags_23 },
    { 0x2984e230, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c4e237, SUC_lexicon_tags_40 },
    { 0xd184e238, SUC_lexicon_tags_2 },
    { 0x7e84e239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c4e23c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9544e242, SUC_lexicon_tags_23 },
    { 0x7624e243, SUC_lexicon_tags_17 },
    { 0x1f84e243, SUC_lexicon_tags_17 },
    { 0x4aa4e245, SUC_lexicon_tags_14 },
    { 0xa144e245, SUC_lexicon_tags_170 },
    { 0xcf64e245, SUC_lexicon_tags_25 },
    { 0x2304e248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe564e24a, SUC_lexicon_tags_7 },
    { 0x6f24e24b, SUC_lexicon_tags_37 },
    { 0x8024e24a, SUC_lexicon_tags_26 },
    { 0xa144e24c, SUC_lexicon_tags_17 },
    { 0xdb44e24b, SUC_lexicon_tags_19 },
    { 0xe0e4e24f, SUC_lexicon_tags_11 },
    { 0x7624e250, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x7864e253, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc084e257, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9944e25d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x70c4e25f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac4e264, SUC_lexicon_tags_14 },
    { 0xbca4e264, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2ee4e267, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c4e26a, SUC_lexicon_tags_14 },
    { 0xed84e26b, SUC_lexicon_tags_34 },
    { 0x6464e26c, SUC_lexicon_tags_11 },
    { 0xf8c4e26a, SUC_lexicon_tags_29 },
    { 0x2ea4e26e, SUC_lexicon_tags_8 },
    { 0x6024e26d, SUC_lexicon_tags_33 },
    { 0x6164e26e, SUC_lexicon_tags_40 },
    { 0x3da4e270, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4e274, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0xd604e277, SUC_lexicon_tags_45 },
    { 0x2444e278, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xace4e27a, SUC_lexicon_tags_31 },
    { 0x6884e27b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6564e27f, SUC_lexicon_tags_15 },
    { 0xe4e4e27f, SUC_lexicon_tags_30 },
    { 0xbdc4e281, SUC_lexicon_tags_84 },
    { 0xfc64e280, SUC_lexicon_tags_0 },
    { 0xb864e283, SUC_lexicon_tags_11 },
    { 0xa564e281, SUC_lexicon_tags_37 },
    { 0xe764e284, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf264e289, SUC_lexicon_tags_25 },
    { 0x7a4e289, SUC_lexicon_tags_12 },
    { 0xc244e28a, SUC_lexicon_tags_15 },
    { 0x5ee4e28c, SUC_lexicon_tags_84 },
    { 0xea04e28d, SUC_lexicon_tags_68 },
    { 0xcb24e28e, SUC_lexicon_tags_1 },
    { 0x7cc4e28f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384e296, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e4e29b, SUC_lexicon_tags_25 },
    { 0x7c64e29b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6544e29f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1aa4e2a1, SUC_lexicon_tags_11 },
    { 0xf4e4e2a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7844e2a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93a4e2a9, SUC_lexicon_tags_19 },
    { 0x2024e2a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc4e2ae, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3984e2b3, SUC_lexicon_tags_47 },
    { 0x3544e2b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a4e2ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6924e2bc, SUC_lexicon_tags_45 },
    { 0xe5e4e2bd, SUC_lexicon_tags_21 },
    { 0xa2c4e2bc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e4e2c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe4e2c7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d84e2cb, SUC_lexicon_tags_0 },
    { 0x9864e2cc, SUC_lexicon_tags_28 },
    { 0xe804e2cb, SUC_lexicon_tags_49 },
    { 0x43c4e2cb, SUC_lexicon_tags_37 },
    { 0xc3c4e2cc, SUC_lexicon_tags_93 },
    { 0x2c24e2d0, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0xa184e2d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee4e2d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe04e2da, SUC_lexicon_tags_56 },
    { 0xcb44e2db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b44e2df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6364e2e1, SUC_lexicon_tags_25 },
    { 0xf004e2e1, SUC_lexicon_tags_43 },
    { 0x1264e2e2, SUC_lexicon_tags_17 },
    { 0xb344e2e2, SUC_lexicon_tags_11 },
    { 0xf184e2e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cc4e2e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd704e2eb, SUC_lexicon_tags_17 },
    { 0x4c24e2eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe4e2f0, SUC_lexicon_tags_17 },
    { 0x7e04e2f1, SUC_lexicon_tags_17 },
    { 0x7d04e2f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d44e2fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec64e2fd, SUC_lexicon_tags_12 },
    { 0xce64e2fe, SUC_lexicon_tags_17 },
    { 0xa864e2ff, SUC_lexicon_tags_38 },
    { 0x4664e300, SUC_lexicon_tags_23 },
    { 0x5c24e301, SUC_lexicon_tags_23 },
    { 0xdf64e302, SUC_lexicon_tags_42 },
    { 0x9604e303, SUC_lexicon_tags_46 },
    { 0xc244e302, SUC_lexicon_tags_18 },
    { 0xfac4e302, SUC_lexicon_tags_37 },
    { 0xc5e4e2fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2744e30d, SUC_lexicon_tags_34 },
    { 0xb104e30e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a44e310, SUC_lexicon_tags_32 },
    { 0x1d24e310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc404e318, SUC_lexicon_tags_63 },
    { 0xc164e318, SUC_lexicon_tags_2 },
    { 0xeb84e318, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e4e31d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc264e320, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd404e322, SUC_lexicon_tags_25 },
    { 0xc84e323, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c4e328, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2464e32a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a04e331, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde4e336, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf964e33c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd44e33e, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0xe184e340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de4e343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa964e345, SUC_lexicon_tags_12 },
    { 0xe6a4e345, SUC_lexicon_tags_38 },
    { 0x384e347, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a4e34e, SUC_lexicon_tags_15 },
    { 0x5a04e34f, SUC_lexicon_tags_48 },
    { 0x7c64e350, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca4e354, SUC_lexicon_tags_33 },
    { 0x9704e354, SUC_lexicon_tags_63 },
    { 0xff64e355, SUC_lexicon_tags_25 },
    { 0x4a64e357, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa764e35c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8f64e35e, SUC_lexicon_tags_17 },
    { 0xb204e35f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43e4e361, SUC_lexicon_tags_12 },
    { 0x3864e362, SUC_lexicon_tags_26 },
    { 0x6a64e363, SUC_lexicon_tags_19 },
    { 0x31c4e364, SUC_lexicon_tags_11 },
    { 0x85a4e363, SUC_lexicon_tags_44 },
    { 0x71e4e366, SUC_lexicon_tags_28 },
    { 0x52e4e367, SUC_lexicon_tags_25 },
    { 0xcba4e367, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6f44e36a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9344e373, SUC_lexicon_tags_3 },
    { 0x4b44e373, SUC_lexicon_tags_2 },
    { 0x1884e375, SUC_lexicon_tags_58 },
    { 0x65e4e374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e84e379, SUC_lexicon_tags_37 },
    { 0xc964e37a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca44e37f, SUC_lexicon_tags_11 },
    { 0xe8c4e380, SUC_lexicon_tags_25 },
    { 0x2a4e381, SUC_lexicon_tags_17 },
    { 0xff84e382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39a4e384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xede4e387, SUC_lexicon_tags_33 },
    { 0xc624e387, SUC_lexicon_tags_46 },
    { 0x2aa4e389, SUC_lexicon_tags_2 },
    { 0xb324e388, SUC_lexicon_tags_66 },
    { 0xfe84e387, SUC_lexicon_tags_37 },
    { 0x2a84e38c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4484e391, SUC_lexicon_tags_30 },
    { 0xbb84e392, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x9b24e394, SUC_lexicon_tags_12 },
    { 0xaec4e395, SUC_lexicon_tags_37 },
    { 0x58c4e396, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb644e398, SUC_lexicon_tags_0 },
    { 0x31e4e398, SUC_lexicon_tags_37 },
    { 0xc784e399, SUC_lexicon_tags_37 },
    { 0xa3a4e39b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef64e39f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a44e3b0, SUC_lexicon_tags_17 },
    { 0xc0c4e3b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d44e3b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca4e3ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd904e3bc, SUC_lexicon_tags_34 },
    { 0xa504e3bd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x57a4e3bf, SUC_lexicon_tags_7 },
    { 0xf944e3c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc164e3c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e4e3c7, SUC_lexicon_tags_11 },
    { 0x3524e3c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9784e3ca, SUC_lexicon_tags_351 },
    { 0x5964e3cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e4e3d2, SUC_lexicon_tags_43 },
    { 0x8664e3d3, SUC_lexicon_tags_37 },
    { 0x3564e3d3, SUC_lexicon_tags_8 },
    { 0x1a84e3d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8e4e3d7, SUC_lexicon_tags_11 },
    { 0x2ba4e3d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaaa4e3da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8044e3e4, SUC_lexicon_tags_19 },
    { 0xdaa4e3e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce4e3eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae4e3f0, SUC_lexicon_tags_30 },
    { 0xf24e3f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce04e3f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e4e3fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc4e402, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba4e407, SUC_lexicon_tags_66 },
    { 0xc404e408, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e4e40c, SUC_lexicon_tags_37 },
    { 0xbf24e40d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c4e411, SUC_lexicon_tags_1 },
    { 0x8084e411, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a4e41a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90e4e41c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa804e41f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc4e422, SUC_lexicon_tags_60 },
    { 0x6504e422, SUC_lexicon_tags_15 },
    { 0x13c4e424, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6644e426, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde4e42b, SUC_lexicon_tags_26 },
    { 0xc4e4e42c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2884e431, SUC_lexicon_tags_17 },
    { 0xa8a4e432, SUC_lexicon_tags_19 },
    { 0x94e4e433, SUC_lexicon_tags_17 },
    { 0x6a44e434, SUC_lexicon_tags_12 },
    { 0xe0e4e434, SUC_lexicon_tags_1 },
    { 0x9444e436, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a4e43c, SUC_lexicon_tags_6 },
    { 0x37a4e43c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da4e441, SUC_lexicon_tags_15 },
    { 0x3ac4e442, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd404e447, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a04e44f, SUC_lexicon_tags_34 },
    { 0xbb04e450, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f04e457, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da4e464, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d24e466, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c4e469, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb584e46c, SUC_lexicon_tags_33 },
    { 0x9a24e46d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9344e472, SUC_lexicon_tags_48 },
    { 0x2bc4e473, SUC_lexicon_tags_3 },
    { 0x56e4e474, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2384e479, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d24e47d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a4e480, SUC_lexicon_tags_0 },
    { 0x82c4e481, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2e4e483, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3624e487, SUC_lexicon_tags_4 },
    { 0x1cc4e488, SUC_lexicon_tags_17 },
    { 0x26c4e489, SUC_lexicon_tags_2 },
    { 0x81e4e489, SUC_lexicon_tags_37 },
    { 0x9204e48b, SUC_lexicon_tags_45 },
    { 0x1084e48c, SUC_lexicon_tags_21 },
    { 0xbe24e488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf404e490, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce84e492, SUC_lexicon_tags_45 },
    { 0x3e84e493, SUC_lexicon_tags_68 },
    { 0xd184e494, SUC_lexicon_tags_0 },
    { 0xda04e493, SUC_lexicon_tags_25 },
    { 0xd004e496, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee4e49f, SUC_lexicon_tags_47 },
    { 0xf084e4a0, SUC_lexicon_tags_63 },
    { 0xec4e4a1, SUC_lexicon_tags_7 },
    { 0x9504e4a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c4e4a5, SUC_lexicon_tags_23 },
    { 0x424e4a5, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc44e4ab, SUC_lexicon_tags_25 },
    { 0x7444e4ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe524e4b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef24e4b6, SUC_lexicon_tags_16 },
    { 0x6ea4e4b7, SUC_lexicon_tags_7 },
    { 0xd124e4b8, SUC_lexicon_tags_2 },
    { 0xd4a4e4b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8f44e4bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6224e4bf, SUC_lexicon_tags_30 },
    { 0xebc4e4bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ba4e4c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6a4e4c4, SUC_lexicon_tags_11 },
    { 0xe544e4c5, SUC_lexicon_tags_33 },
    { 0x1464e4c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd744e4c8, SUC_lexicon_tags_70 },
    { 0x96e4e4c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb44e4cc, SUC_lexicon_tags_28 },
    { 0x5984e4cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc4e4d2, SUC_lexicon_tags_17 },
    { 0xc324e4d3, SUC_lexicon_tags_16 },
    { 0x3704e4d4, SUC_lexicon_tags_68 },
    { 0xea24e4d5, SUC_lexicon_tags_25 },
    { 0xd604e4d5, SUC_lexicon_tags_10 },
    { 0x80c4e4d5, SUC_lexicon_tags_7 },
    { 0xf224e4d8, SUC_lexicon_tags_47 },
    { 0x8324e4d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee4e4de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x82a4e4e0, SUC_lexicon_tags_7 },
    { 0x43a4e4e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e4e4e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc04e4e6, SUC_lexicon_tags_1 },
    { 0xc204e4e7, SUC_lexicon_tags_15 },
    { 0xff64e4e8, SUC_lexicon_tags_37 },
    { 0xec04e4e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7404e4ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8264e4ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6464e4f1, SUC_lexicon_tags_0 },
    { 0x3504e4f2, SUC_lexicon_tags_79 },
    { 0xe984e4f3, SUC_lexicon_tags_63 },
    { 0x96a4e4f4, SUC_lexicon_tags_17 },
    { 0x5644e4f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c4e4f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd664e500, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c44e503, SUC_lexicon_tags_25 },
    { 0x884e504, SUC_lexicon_tags_25 },
    { 0x87a4e505, SUC_lexicon_tags_11 },
    { 0x4f84e504, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2a4e508, SUC_lexicon_tags_30 },
    { 0xe004e508, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x28e4e50b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74e4e50d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a4e513, SUC_lexicon_tags_17 },
    { 0xf184e514, SUC_lexicon_tags_25 },
    { 0x5464e515, SUC_lexicon_tags_3 },
    { 0xd4a4e514, SUC_lexicon_tags_66 },
    { 0x8784e515, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd584e521, SUC_lexicon_tags_6 },
    { 0xa2c4e521, SUC_lexicon_tags_47 },
    { 0xe4e4e523, SUC_lexicon_tags_11 },
    { 0x1ae4e524, SUC_lexicon_tags_8 },
    { 0xf884e525, SUC_lexicon_tags_15 },
    { 0x9824e526, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab84e531, SUC_lexicon_tags_27 },
    { 0xa4c4e532, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e4e535, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c4e53a, SUC_lexicon_tags_1 },
    { 0x2264e53b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a4e54b, SUC_lexicon_tags_17 },
    { 0x17e4e54c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1064e550, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a4e554, SUC_lexicon_tags_17 },
    { 0xd504e554, SUC_lexicon_tags_92 },
    { 0x7f24e556, SUC_lexicon_tags_82 },
    { 0x35e4e557, SUC_lexicon_tags_11 },
    { 0x9484e558, SUC_lexicon_tags_11 },
    { 0xa544e557, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb924e55b, SUC_lexicon_tags_91 },
    { 0x5ac4e55c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e4e55f, SUC_lexicon_tags_37 },
    { 0x8464e560, SUC_lexicon_tags_15 },
    { 0x83e4e560, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e64e565, SUC_lexicon_tags_23 },
    { 0x7464e566, SUC_lexicon_tags_10 },
    { 0x9424e565, SUC_lexicon_tags_17 },
    { 0xd2e4e566, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc024e56a, SUC_lexicon_tags_15 },
    { 0x92e4e56b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e4e56e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc4e571, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d04e573, SUC_lexicon_tags_17 },
    { 0x22c4e574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ec4e576, SUC_lexicon_tags_19 },
    { 0xed04e577, SUC_lexicon_tags_81 },
    { 0xda4e578, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6344e57d, SUC_lexicon_tags_27 },
    { 0xa6a4e57e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e582, SUC_lexicon_tags_8 },
    { 0x1864e582, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3164e587, SUC_lexicon_tags_30 },
    { 0x37e4e587, SUC_lexicon_tags_56 },
    { 0x7e24e589, SUC_lexicon_tags_45 },
    { 0xec44e58a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57e4e58c, SUC_lexicon_tags_63 },
    { 0x7684e58c, SUC_lexicon_tags_22 },
    { 0x41a4e58d, SUC_lexicon_tags_82 },
    { 0x6024e58f, SUC_lexicon_tags_47 },
    { 0x6f24e590, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe144e592, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c4e595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e4e599, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf64e59c, SUC_lexicon_tags_25 },
    { 0x9544e59c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf44e5a1, SUC_lexicon_tags_37 },
    { 0x9044e5a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17c4e5a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdca4e5a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8484e5af, SUC_lexicon_tags_82 },
    { 0x6784e5b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da4e5b8, SUC_lexicon_tags_7 },
    { 0x4e84e5b9, SUC_lexicon_tags_8 },
    { 0xe664e5b8, SUC_lexicon_tags_30 },
    { 0x2f84e5bb, SUC_lexicon_tags_9 },
    { 0x5224e5b9, SUC_lexicon_tags_11 },
    { 0xd444e5bd, SUC_lexicon_tags_49 },
    { 0x2e64e5bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd864e5c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e4e5c6, SUC_lexicon_tags_14 },
    { 0xcbc4e5c7, SUC_lexicon_tags_8 },
    { 0xfc64e5c8, SUC_lexicon_tags_37 },
    { 0x6b04e5c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a4e5d2, SUC_lexicon_tags_0 },
    { 0x2f04e5d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa84e5d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4564e5db, SUC_lexicon_tags_50 },
    { 0x6484e5dc, SUC_lexicon_tags_11 },
    { 0xc144e5dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e5e1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b64e5e4, SUC_lexicon_tags_45 },
    { 0x4c24e5e5, SUC_lexicon_tags_37 },
    { 0xb1a4e5e6, SUC_lexicon_tags_82 },
    { 0xf144e5e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x43e4e5e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a4e5f2, SUC_lexicon_tags_31 },
    { 0x64a4e5f2, SUC_lexicon_tags_33 },
    { 0xcc4e5f2, SUC_lexicon_tags_11 },
    { 0x7e4e5f4, SUC_lexicon_tags_37 },
    { 0x5fa4e5f6, SUC_lexicon_tags_15 },
    { 0xe6c4e5f7, SUC_lexicon_tags_25 },
    { 0xd944e5f8, SUC_lexicon_tags_29 },
    { 0xb064e5f8, SUC_lexicon_tags_17 },
    { 0xb5a4e5f9, SUC_lexicon_tags_17 },
    { 0xf024e5fb, SUC_lexicon_tags_12 },
    { 0xf624e5fc, SUC_lexicon_tags_7 },
    { 0xbca4e5f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb84e603, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4e608, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd604e60c, SUC_lexicon_tags_38 },
    { 0x1a64e60c, SUC_lexicon_tags_17 },
    { 0x2a4e60c, SUC_lexicon_tags_12 },
    { 0x2de4e60f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe04e613, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x39c4e615, SUC_lexicon_tags_19 },
    { 0x4a04e616, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab64e61b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6644e61f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7244e629, SUC_lexicon_tags_60 },
    { 0x3624e62a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7724e62c, SUC_lexicon_tags_7 },
    { 0x7fa4e62d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d84e631, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2364e637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7444e639, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3704e63d, SUC_lexicon_tags_1 },
    { 0xdd44e63e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a4e642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4de4e644, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x56e4e646, SUC_lexicon_tags_56 },
    { 0x9b84e646, SUC_lexicon_tags_17 },
    { 0xc1c4e648, SUC_lexicon_tags_60 },
    { 0xb8c4e646, SUC_lexicon_tags_61 },
    { 0x21a4e64a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda4e64c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc4e650, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7904e652, SUC_lexicon_tags_37 },
    { 0xfbc4e652, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1404e655, SUC_lexicon_tags_26 },
    { 0x7524e656, SUC_lexicon_tags_2 },
    { 0x2ce4e656, SUC_lexicon_tags_19 },
    { 0x15c4e657, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x104e65a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b64e65c, SUC_lexicon_tags_15 },
    { 0x10e4e65c, SUC_lexicon_tags_2 },
    { 0x4044e65e, SUC_lexicon_tags_12 },
    { 0xeb44e65d, SUC_lexicon_tags_19 },
    { 0x6a64e660, SUC_lexicon_tags_37 },
    { 0xe2c4e65f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a4e66a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a64e66d, SUC_lexicon_tags_19 },
    { 0x8984e66e, SUC_lexicon_tags_11 },
    { 0x4e24e66f, SUC_lexicon_tags_30 },
    { 0xf24e670, SUC_lexicon_tags_37 },
    { 0x5aa4e671, SUC_lexicon_tags_11 },
    { 0x6b44e671, SUC_lexicon_tags_25 },
    { 0x58a4e66f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624e67a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9624e684, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3744e687, SUC_lexicon_tags_75 },
    { 0xd744e688, SUC_lexicon_tags_29 },
    { 0x8fc4e689, SUC_lexicon_tags_8 },
    { 0xe764e68a, SUC_lexicon_tags_33 },
    { 0x93c4e68b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e4e68e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5344e693, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe464e699, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8be4e69b, SUC_lexicon_tags_28 },
    { 0x1b44e69c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6604e69f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be4e6a2, SUC_lexicon_tags_15 },
    { 0x3d64e6a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb384e6a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea4e6b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1484e6b8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc644e6bb, SUC_lexicon_tags_47 },
    { 0xb724e6bb, SUC_lexicon_tags_25 },
    { 0x4164e6bd, SUC_lexicon_tags_46 },
    { 0x8424e6be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d04e6c2, SUC_lexicon_tags_1 },
    { 0x74e4e6c2, SUC_lexicon_tags_37 },
    { 0x41e4e6c4, SUC_lexicon_tags_37 },
    { 0xfb64e6c5, SUC_lexicon_tags_47 },
    { 0x91a4e6c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e4e6ce, SUC_lexicon_tags_25 },
    { 0x8a4e6cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04e6d2, SUC_lexicon_tags_62 },
    { 0x4864e6d3, SUC_lexicon_tags_9 },
    { 0x6024e6d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8744e6d8, SUC_lexicon_tags_61 },
    { 0xe64e6d9, SUC_lexicon_tags_7 },
    { 0x16e4e6d9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1984e6de, SUC_lexicon_tags_42 },
    { 0xe764e6de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa4e6e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbca4e6e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8364e6e9, SUC_lexicon_tags_15 },
    { 0xe4e6ea, SUC_lexicon_tags_29 },
    { 0x4d04e6eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bc4e6ed, SUC_lexicon_tags_12 },
    { 0xe844e6ee, SUC_lexicon_tags_21 },
    { 0x6584e6ee, SUC_lexicon_tags_12 },
    { 0x96e4e6ef, SUC_lexicon_tags_10 },
    { 0xbe64e6f1, SUC_lexicon_tags_37 },
    { 0x3f44e6f2, SUC_lexicon_tags_33 },
    { 0x7104e6f0, SUC_lexicon_tags_68 },
    { 0x40a4e6f0, SUC_lexicon_tags_1 },
    { 0xac4e6ee, SUC_lexicon_tags_17 },
    { 0xe4c4e6f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc4e6fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a4e6fd, SUC_lexicon_tags_11 },
    { 0x1064e6fe, SUC_lexicon_tags_19 },
    { 0xc7a4e6ff, SUC_lexicon_tags_26 },
    { 0xc884e6ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3004e702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e4e705, SUC_lexicon_tags_14 },
    { 0xa5a4e705, SUC_lexicon_tags_14 },
    { 0xa424e707, SUC_lexicon_tags_38 },
    { 0xe5a4e706, SUC_lexicon_tags_37 },
    { 0xbc04e709, SUC_lexicon_tags_0 },
    { 0x6244e705, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e4e70f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a4e715, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc4e718, SUC_lexicon_tags_34 },
    { 0xeb84e718, SUC_lexicon_tags_2 },
    { 0x3884e719, SUC_lexicon_tags_28 },
    { 0x9de4e71b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba4e71e, SUC_lexicon_tags_198 },
    { 0x5064e71f, SUC_lexicon_tags_17 },
    { 0x81e4e71f, SUC_lexicon_tags_25 },
    { 0x2804e71e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4c44e723, SUC_lexicon_tags_28 },
    { 0x2e84e724, SUC_lexicon_tags_17 },
    { 0x5a04e725, SUC_lexicon_tags_61 },
    { 0x7844e726, SUC_lexicon_tags_66 },
    { 0x81a4e724, SUC_lexicon_tags_45 },
    { 0xa424e728, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e4e72b, SUC_lexicon_tags_60 },
    { 0xcee4e72c, SUC_lexicon_tags_11 },
    { 0x5244e72d, SUC_lexicon_tags_13 },
    { 0xa764e72d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e4e731, SUC_lexicon_tags_1 },
    { 0x8544e732, SUC_lexicon_tags_5 },
    { 0x8544e733, SUC_lexicon_tags_6 },
    { 0xf764e734, SUC_lexicon_tags_17 },
    { 0x33a4e732, SUC_lexicon_tags_37 },
    { 0xb644e734, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc4e73e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6724e744, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1364e749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd064e74d, SUC_lexicon_tags_17 },
    { 0x86e4e74e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57c4e750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8584e756, SUC_lexicon_tags_25 },
    { 0xcba4e756, SUC_lexicon_tags_27 },
    { 0xf6a4e757, SUC_lexicon_tags_65 },
    { 0x80c4e759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a4e763, SUC_lexicon_tags_23 },
    { 0xdca4e763, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e767, SUC_lexicon_tags_2 },
    { 0x36e4e767, SUC_lexicon_tags_57 },
    { 0x6c4e767, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b84e76f, SUC_lexicon_tags_43 },
    { 0x3fa4e770, SUC_lexicon_tags_31 },
    { 0xe244e771, SUC_lexicon_tags_7 },
    { 0x45e4e772, SUC_lexicon_tags_7 },
    { 0x1864e771, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4104e775, SUC_lexicon_tags_44 },
    { 0x6844e776, SUC_lexicon_tags_24 },
    { 0xc604e775, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a4e77a, SUC_lexicon_tags_17 },
    { 0x5684e77b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c4e77e, SUC_lexicon_tags_15 },
    { 0xc7e4e77e, SUC_lexicon_tags_49 },
    { 0x6564e77e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4024e782, SUC_lexicon_tags_6 },
    { 0x9c64e783, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b44e785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c84e788, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc4e78b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1804e78d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc024e791, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x32a4e793, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd904e795, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc4e797, SUC_lexicon_tags_37 },
    { 0x22c4e798, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc784e79b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ec4e79d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce4e7a0, SUC_lexicon_tags_15 },
    { 0x12e4e7a1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7164e7a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9324e7af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x26c4e7b1, SUC_lexicon_tags_23 },
    { 0x9024e7b2, SUC_lexicon_tags_117 },
    { 0x1c4e7b2, SUC_lexicon_tags_10 },
    { 0x6084e7b4, SUC_lexicon_tags_49 },
    { 0xc24e7b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c4e7bc, SUC_lexicon_tags_2 },
    { 0x35a4e7bd, SUC_lexicon_tags_25 },
    { 0x6ba4e7bd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf684e7c2, SUC_lexicon_tags_45 },
    { 0x6e04e7c3, SUC_lexicon_tags_26 },
    { 0xed84e7c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa804e7cb, SUC_lexicon_tags_2 },
    { 0xc84e7cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb984e7d0, SUC_lexicon_tags_29 },
    { 0x6344e7d1, SUC_lexicon_tags_38 },
    { 0xc444e7d2, SUC_lexicon_tags_25 },
    { 0x1504e7d2, SUC_lexicon_tags_37 },
    { 0xe6e4e7d4, SUC_lexicon_tags_38 },
    { 0x2464e7d3, SUC_lexicon_tags_68 },
    { 0x5064e7d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a4e7d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde04e7db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504e7df, SUC_lexicon_tags_7 },
    { 0x2184e7df, SUC_lexicon_tags_49 },
    { 0x7084e7e1, SUC_lexicon_tags_8 },
    { 0x5ac4e7df, SUC_lexicon_tags_37 },
    { 0x4f24e7e3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd64e7e5, SUC_lexicon_tags_2 },
    { 0xba64e7e6, SUC_lexicon_tags_43 },
    { 0x2de4e7e7, SUC_lexicon_tags_2 },
    { 0xe6c4e7e5, SUC_lexicon_tags_37 },
    { 0x3464e7e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4564e7ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcc4e7f0, SUC_lexicon_tags_27 },
    { 0xa0c4e7f1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc4e7f4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc824e7f7, SUC_lexicon_tags_60 },
    { 0xa944e7f8, SUC_lexicon_tags_2 },
    { 0x13c4e7f9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe884e7fc, SUC_lexicon_tags_15 },
    { 0xb704e7fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4144e800, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf944e802, SUC_lexicon_tags_37 },
    { 0xb964e803, SUC_lexicon_tags_63 },
    { 0x6304e804, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f04e807, SUC_lexicon_tags_29 },
    { 0x8d04e807, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b44e80c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a4e814, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a64e817, SUC_lexicon_tags_10 },
    { 0x1f44e818, SUC_lexicon_tags_21 },
    { 0xc04e819, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f24e81f, SUC_lexicon_tags_28 },
    { 0x8cc4e820, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff64e822, SUC_lexicon_tags_17 },
    { 0xa2c4e823, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb184e825, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e4e827, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x84c4e829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e4e82d, SUC_lexicon_tags_11 },
    { 0xe64e82e, SUC_lexicon_tags_46 },
    { 0xd6a4e82d, SUC_lexicon_tags_12 },
    { 0x2ce4e82e, SUC_lexicon_tags_21 },
    { 0x33c4e831, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x21a4e833, SUC_lexicon_tags_65 },
    { 0xb4a4e834, SUC_lexicon_tags_15 },
    { 0x1584e835, SUC_lexicon_tags_15 },
    { 0xc964e836, SUC_lexicon_tags_27 },
    { 0xe164e833, SUC_lexicon_tags_12 },
    { 0x1164e838, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1884e83a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef64e83f, SUC_lexicon_tags_30 },
    { 0x5284e840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1484e843, SUC_lexicon_tags_27 },
    { 0xb764e844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c84e848, SUC_lexicon_tags_43 },
    { 0x3ae4e849, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xef44e84b, SUC_lexicon_tags_148 },
    { 0x5704e84c, SUC_lexicon_tags_27 },
    { 0xc1a4e84d, SUC_lexicon_tags_35 },
    { 0x9744e84e, SUC_lexicon_tags_49 },
    { 0x94e4e84f, SUC_lexicon_tags_47 },
    { 0x65c4e84c, SUC_lexicon_tags_17 },
    { 0xf0c4e84b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4e854, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a4e85d, SUC_lexicon_tags_14 },
    { 0xbc44e85e, SUC_lexicon_tags_26 },
    { 0x3ae4e85e, SUC_lexicon_tags_7 },
    { 0x7224e85d, SUC_lexicon_tags_35 },
    { 0x2664e85f, SUC_lexicon_tags_19 },
    { 0xf784e862, SUC_lexicon_tags_31 },
    { 0x81a4e863, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc184e865, SUC_lexicon_tags_23 },
    { 0x96c4e866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b44e869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e4e86c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ac4e86e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc84e871, SUC_lexicon_tags_77 },
    { 0x64e871, SUC_lexicon_tags_12 },
    { 0x6604e873, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8844e876, SUC_lexicon_tags_25 },
    { 0x4ea4e876, SUC_lexicon_tags_4 },
    { 0xaca4e878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c4e87b, SUC_lexicon_tags_37 },
    { 0x84e87b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5a4e87e, SUC_lexicon_tags_25 },
    { 0x48e4e87f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4464e885, SUC_lexicon_tags_49 },
    { 0x3d84e885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8944e889, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5924e88c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a4e892, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c4e895, SUC_lexicon_tags_15 },
    { 0x2444e895, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2944e89e, SUC_lexicon_tags_14 },
    { 0x4564e89e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1764e8a1, SUC_lexicon_tags_11 },
    { 0x9564e8a2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6724e8a4, SUC_lexicon_tags_17 },
    { 0x3be4e8a5, SUC_lexicon_tags_7 },
    { 0x9dc4e8a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf124e8b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb8c4e8b3, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8724e8ba, SUC_lexicon_tags_2 },
    { 0x79a4e8bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba4e8bf, SUC_lexicon_tags_25 },
    { 0xac44e8c0, SUC_lexicon_tags_38 },
    { 0xed24e8bf, SUC_lexicon_tags_21 },
    { 0x1404e8bf, SUC_lexicon_tags_55 },
    { 0xbbc4e8c3, SUC_lexicon_tags_47 },
    { 0x18a4e8c4, SUC_lexicon_tags_8 },
    { 0x9224e8c5, SUC_lexicon_tags_15 },
    { 0xd124e8c6, SUC_lexicon_tags_6 },
    { 0xde24e8c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7ba4e8c9, SUC_lexicon_tags_30 },
    { 0xe7c4e8c9, SUC_lexicon_tags_7 },
    { 0xd344e8c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x54e4e8cd, SUC_lexicon_tags_12 },
    { 0x9ba4e8ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x34e4e8d0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a4e8d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce4e8d9, SUC_lexicon_tags_88 },
    { 0xcf84e8da, SUC_lexicon_tags_25 },
    { 0x3364e8db, SUC_lexicon_tags_23 },
    { 0x3044e8da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa924e8e0, SUC_lexicon_tags_49 },
    { 0x56c4e8e1, SUC_lexicon_tags_17 },
    { 0x8804e8e2, SUC_lexicon_tags_22 },
    { 0x7ca4e8e2, SUC_lexicon_tags_27 },
    { 0xdf84e8e4, SUC_lexicon_tags_138 },
    { 0xf5e4e8e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2424e8e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c04e8ed, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x9004e8ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe724e8f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xde44e8f7, SUC_lexicon_tags_60 },
    { 0xe104e8f8, SUC_lexicon_tags_19 },
    { 0x8444e8f9, SUC_lexicon_tags_32 },
    { 0x2044e8fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5b64e8fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac4e8ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6824e905, SUC_lexicon_tags_64 },
    { 0x19e4e906, SUC_lexicon_tags_146 },
    { 0x4ea4e907, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c24e90e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x55e4e910, SUC_lexicon_tags_25 },
    { 0xfb04e911, SUC_lexicon_tags_25 },
    { 0xc4a4e911, SUC_lexicon_tags_1 },
    { 0xa9e4e913, SUC_lexicon_tags_2 },
    { 0x1484e914, SUC_lexicon_tags_7 },
    { 0xe404e914, SUC_lexicon_tags_2 },
    { 0xb564e910, SUC_lexicon_tags_11 },
    { 0x2504e917, SUC_lexicon_tags_37 },
    { 0x9664e917, SUC_lexicon_tags_6 },
    { 0x864e919, SUC_lexicon_tags_19 },
    { 0xa804e919, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164e921, SUC_lexicon_tags_63 },
    { 0xf0c4e921, SUC_lexicon_tags_30 },
    { 0xc8e4e923, SUC_lexicon_tags_34 },
    { 0x9b84e924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x524e927, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba4e92b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7a44e92d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb584e930, SUC_lexicon_tags_16 },
    { 0x87c4e930, SUC_lexicon_tags_2 },
    { 0x4ea4e932, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1124e939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e4e93b, SUC_lexicon_tags_7 },
    { 0x7564e93b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb484e93e, SUC_lexicon_tags_15 },
    { 0x4a84e93f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4644e943, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x4964e945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ee4e947, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9124e949, SUC_lexicon_tags_7 },
    { 0x2fa4e949, SUC_lexicon_tags_69 },
    { 0x8044e94b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd04e94e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3164e955, SUC_lexicon_tags_38 },
    { 0xdd84e956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd864e960, SUC_lexicon_tags_17 },
    { 0x4224e961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab44e963, SUC_lexicon_tags_25 },
    { 0xf2e4e964, SUC_lexicon_tags_9 },
    { 0xfb44e965, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a4e96c, SUC_lexicon_tags_1 },
    { 0x68c4e96d, SUC_lexicon_tags_7 },
    { 0xda04e96e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc524e970, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26a4e972, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9624e976, SUC_lexicon_tags_7 },
    { 0xe884e977, SUC_lexicon_tags_12 },
    { 0x3224e978, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbfc4e97a, SUC_lexicon_tags_10 },
    { 0x3284e97b, SUC_lexicon_tags_27 },
    { 0xb044e97c, SUC_lexicon_tags_7 },
    { 0x3b24e97d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c4e980, SUC_lexicon_tags_7 },
    { 0xd2c4e981, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8d44e983, SUC_lexicon_tags_17 },
    { 0xbfc4e983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344e988, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd204e98a, SUC_lexicon_tags_38 },
    { 0x1844e98a, SUC_lexicon_tags_23 },
    { 0xecc4e98c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e4e990, SUC_lexicon_tags_17 },
    { 0x3484e991, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6844e994, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4a4e996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8684e998, SUC_lexicon_tags_1 },
    { 0xdd44e999, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf424e99b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf324e9a2, SUC_lexicon_tags_15 },
    { 0x57e4e9a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb784e9a5, SUC_lexicon_tags_37 },
    { 0xff24e9a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2724e9aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b64e9b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf604e9b6, SUC_lexicon_tags_26 },
    { 0x3b44e9b7, SUC_lexicon_tags_47 },
    { 0x54e4e9b8, SUC_lexicon_tags_30 },
    { 0xfb64e9b9, SUC_lexicon_tags_63 },
    { 0xb9e4e9b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764e9be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x67e4e9c0, SUC_lexicon_tags_37 },
    { 0x9f24e9c1, SUC_lexicon_tags_65 },
    { 0x1b04e9c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea64e9c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8e4e9c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a24e9c8, SUC_lexicon_tags_25 },
    { 0x2404e9c8, SUC_lexicon_tags_25 },
    { 0x8b84e9c9, SUC_lexicon_tags_15 },
    { 0x9ca4e9cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd204e9d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe04e9d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e4e9d9, SUC_lexicon_tags_114 },
    { 0x5f84e9da, SUC_lexicon_tags_47 },
    { 0xaca4e9d9, SUC_lexicon_tags_25 },
    { 0xefe4e9dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a4e9e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7964e9e7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd184e9eb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a4e9f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7624e9f2, SUC_lexicon_tags_17 },
    { 0x5864e9f3, SUC_lexicon_tags_0 },
    { 0xac64e9f2, SUC_lexicon_tags_84 },
    { 0xe004e9f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc604e9f9, SUC_lexicon_tags_37 },
    { 0xc744e9f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c4e9fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e84e9ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5444ea02, SUC_lexicon_tags_38 },
    { 0x7604ea03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4284ea08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd44ea0a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c4ea0f, SUC_lexicon_tags_56 },
    { 0x97c4ea10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6024ea14, SUC_lexicon_tags_23 },
    { 0x5664ea15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb04ea19, SUC_lexicon_tags_1 },
    { 0xb124ea1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac4ea1f, SUC_lexicon_tags_39 },
    { 0x3124ea20, SUC_lexicon_tags_63 },
    { 0x6ee4ea21, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc4ea2d, SUC_lexicon_tags_37 },
    { 0x5f44ea2d, SUC_lexicon_tags_2 },
    { 0xb1c4ea2e, SUC_lexicon_tags_37 },
    { 0xe924ea30, SUC_lexicon_tags_9 },
    { 0x38a4ea31, SUC_lexicon_tags_15 },
    { 0x8dc4ea32, SUC_lexicon_tags_37 },
    { 0x91a4ea32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e4ea36, SUC_lexicon_tags_87 },
    { 0xc004ea37, SUC_lexicon_tags_37 },
    { 0xb104ea38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5544ea3d, SUC_lexicon_tags_29 },
    { 0xaa64ea3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96c4ea40, SUC_lexicon_tags_11 },
    { 0xf5e4ea41, SUC_lexicon_tags_43 },
    { 0x3384ea42, SUC_lexicon_tags_38 },
    { 0xa944ea40, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd24ea45, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb684ea48, SUC_lexicon_tags_25 },
    { 0x2184ea49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1e4ea4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa824ea4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x99a4ea50, SUC_lexicon_tags_33 },
    { 0x73c4ea51, SUC_lexicon_tags_82 },
    { 0xae24ea52, SUC_lexicon_tags_5 },
    { 0x2c24ea52, SUC_lexicon_tags_2 },
    { 0xc424ea54, SUC_lexicon_tags_15 },
    { 0x1764ea54, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf64ea59, SUC_lexicon_tags_27 },
    { 0x77a4ea5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4924ea5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1384ea61, SUC_lexicon_tags_11 },
    { 0xd7c4ea62, SUC_lexicon_tags_31 },
    { 0x3484ea62, SUC_lexicon_tags_7 },
    { 0xbf64ea64, SUC_lexicon_tags_45 },
    { 0xaf64ea62, SUC_lexicon_tags_34 },
    { 0x3044ea66, SUC_lexicon_tags_60 },
    { 0x49c4ea67, SUC_lexicon_tags_38 },
    { 0xf544ea64, SUC_lexicon_tags_46 },
    { 0x2584ea65, SUC_lexicon_tags_9 },
    { 0x92c4ea66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea4ea6f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd84ea71, SUC_lexicon_tags_26 },
    { 0x2124ea72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ee4ea74, SUC_lexicon_tags_29 },
    { 0x96a4ea75, SUC_lexicon_tags_19 },
    { 0x2344ea76, SUC_lexicon_tags_6 },
    { 0x9264ea76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4384ea7a, SUC_lexicon_tags_25 },
    { 0x9ec4ea7b, SUC_lexicon_tags_56 },
    { 0x4284ea7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc664ea86, SUC_lexicon_tags_65 },
    { 0x8e84ea87, SUC_lexicon_tags_7 },
    { 0x85a4ea88, SUC_lexicon_tags_31 },
    { 0x73c4ea89, SUC_lexicon_tags_49 },
    { 0x6584ea88, SUC_lexicon_tags_11 },
    { 0x9c44ea8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf04ea97, SUC_lexicon_tags_12 },
    { 0xbcc4ea98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3324ea9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd384ea9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364eaa6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8784eaa8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb24eaaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5464eab1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab64eab3, SUC_lexicon_tags_2 },
    { 0xc024eab4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac4eac0, SUC_lexicon_tags_6 },
    { 0x5b04eac1, SUC_lexicon_tags_10 },
    { 0x1124eac0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5764eac6, SUC_lexicon_tags_37 },
    { 0x3d24eac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8244eac9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4ead1, SUC_lexicon_tags_2 },
    { 0xfb44ead1, SUC_lexicon_tags_11 },
    { 0x6984ead1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3564ead6, SUC_lexicon_tags_132 },
    { 0xc744ead6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c4eadb, SUC_lexicon_tags_1 },
    { 0x3e24eadc, SUC_lexicon_tags_25 },
    { 0xe8c4eadd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf044eadf, SUC_lexicon_tags_14 },
    { 0xa4e4eae0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa564eae2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf984eaef, SUC_lexicon_tags_29 },
    { 0x8964eaef, SUC_lexicon_tags_15 },
    { 0x2fa4eaf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a4eaf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe644eaf9, SUC_lexicon_tags_37 },
    { 0xd764eaf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa44eafc, SUC_lexicon_tags_33 },
    { 0x9044eafc, SUC_lexicon_tags_11 },
    { 0xf8c4eafd, SUC_lexicon_tags_11 },
    { 0x31e4eaff, SUC_lexicon_tags_68 },
    { 0xba04eb00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa4eb08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6284eb0f, SUC_lexicon_tags_15 },
    { 0x47c4eb0f, SUC_lexicon_tags_87 },
    { 0x56e4eb11, SUC_lexicon_tags_1 },
    { 0x5b84eb12, SUC_lexicon_tags_17 },
    { 0x45e4eb13, SUC_lexicon_tags_12 },
    { 0x144eb13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c4eb17, SUC_lexicon_tags_39 },
    { 0xe264eb18, SUC_lexicon_tags_15 },
    { 0xa0a4eb18, SUC_lexicon_tags_19 },
    { 0x17e4eb1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3764eb1c, SUC_lexicon_tags_17 },
    { 0xff24eb1c, SUC_lexicon_tags_29 },
    { 0x6464eb1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e4eb23, SUC_lexicon_tags_11 },
    { 0x5504eb23, SUC_lexicon_tags_8 },
    { 0x62a4eb25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce4eb2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7224eb2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19e4eb31, SUC_lexicon_tags_2 },
    { 0xa9c4eb32, SUC_lexicon_tags_7 },
    { 0x6064eb33, SUC_lexicon_tags_48 },
    { 0xd384eb34, SUC_lexicon_tags_12 },
    { 0x6344eb35, SUC_lexicon_tags_15 },
    { 0xbe64eb36, SUC_lexicon_tags_27 },
    { 0xda84eb37, SUC_lexicon_tags_66 },
    { 0x28e4eb38, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x224eb3b, SUC_lexicon_tags_45 },
    { 0x5ea4eb3b, SUC_lexicon_tags_25 },
    { 0xdfe4eb3c, SUC_lexicon_tags_25 },
    { 0x87a4eb3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf44eb41, SUC_lexicon_tags_141 },
    { 0xf664eb41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae4eb46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde44eb4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d84eb4f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe744eb53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x384eb55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57c4eb57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa44eb59, SUC_lexicon_tags_22 },
    { 0x1244eb5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4664eb5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce44eb5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb424eb63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b44eb69, SUC_lexicon_tags_15 },
    { 0x47c4eb69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0a4eb6c, SUC_lexicon_tags_19 },
    { 0x10e4eb6d, SUC_lexicon_tags_45 },
    { 0x9444eb6c, SUC_lexicon_tags_97 },
    { 0x10e4eb6f, SUC_lexicon_tags_37 },
    { 0xd8e4eb70, SUC_lexicon_tags_39 },
    { 0xa004eb71, SUC_lexicon_tags_12 },
    { 0xfbc4eb72, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd164eb76, SUC_lexicon_tags_7 },
    { 0xea64eb77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee4eb7b, SUC_lexicon_tags_7 },
    { 0x404eb7b, SUC_lexicon_tags_107 },
    { 0xdf84eb7c, SUC_lexicon_tags_11 },
    { 0xb4c4eb7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804eb84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d44eb8a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5a64eb8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2464eb93, SUC_lexicon_tags_31 },
    { 0x19c4eb94, SUC_lexicon_tags_47 },
    { 0xed44eb93, SUC_lexicon_tags_25 },
    { 0xb424eb96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d44eb99, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a4eb9c, SUC_lexicon_tags_2 },
    { 0x79c4eb9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6964eba2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc4ebaa, SUC_lexicon_tags_37 },
    { 0xd544ebab, SUC_lexicon_tags_45 },
    { 0x6224ebac, SUC_lexicon_tags_11 },
    { 0xe5c4ebac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9144ebb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc224ebb3, SUC_lexicon_tags_57 },
    { 0xaa4ebb4, SUC_lexicon_tags_37 },
    { 0x8dc4ebb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe824ebb7, SUC_lexicon_tags_17 },
    { 0x764ebb8, SUC_lexicon_tags_17 },
    { 0x9dc4ebb8, SUC_lexicon_tags_3 },
    { 0xca84ebb7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf04ebc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc344ebc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf324ebce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1c44ebd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38a4ebd2, SUC_lexicon_tags_27 },
    { 0x1724ebd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ac4ebd5, SUC_lexicon_tags_11 },
    { 0x9064ebd5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x64c4ebd8, SUC_lexicon_tags_12 },
    { 0xf004ebd8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa484ebdd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4724ebe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4ebe3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c4ebe8, SUC_lexicon_tags_1 },
    { 0xc124ebe9, SUC_lexicon_tags_11 },
    { 0x99a4ebe9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4064ebf0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x5dc4ebf2, SUC_lexicon_tags_5 },
    { 0xf044ebf3, SUC_lexicon_tags_45 },
    { 0xb2e4ebf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7384ebf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7824ebfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe4ec02, SUC_lexicon_tags_12 },
    { 0xe24ec03, SUC_lexicon_tags_11 },
    { 0x5224ec04, SUC_lexicon_tags_25 },
    { 0xd5a4ec04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1084ec07, SUC_lexicon_tags_6 },
    { 0x96a4ec08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3444ec0f, SUC_lexicon_tags_6 },
    { 0xb284ec10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec4ec15, SUC_lexicon_tags_34 },
    { 0xeee4ec16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2944ec21, SUC_lexicon_tags_79 },
    { 0x7284ec22, SUC_lexicon_tags_12 },
    { 0xed84ec23, SUC_lexicon_tags_1592 },
    { 0xda4ec24, SUC_lexicon_tags_15 },
    { 0x82a4ec25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c4ec2d, SUC_lexicon_tags_15 },
    { 0x96c4ec2d, SUC_lexicon_tags_10 },
    { 0xad64ec2e, SUC_lexicon_tags_12 },
    { 0x4d04ec2e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x31c4ec32, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1224ec39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d84ec3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a24ec3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3304ec42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac4ec48, SUC_lexicon_tags_5 },
    { 0x86e4ec49, SUC_lexicon_tags_2 },
    { 0x8d84ec4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f04ec4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf564ec4e, SUC_lexicon_tags_27 },
    { 0xb764ec4f, SUC_lexicon_tags_17 },
    { 0x3a64ec4f, SUC_lexicon_tags_87 },
    { 0xe344ec51, SUC_lexicon_tags_38 },
    { 0x7584ec52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a4ec55, SUC_lexicon_tags_38 },
    { 0xed44ec56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9244ec5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e4ec5d, SUC_lexicon_tags_26 },
    { 0x4044ec5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7164ec68, SUC_lexicon_tags_25 },
    { 0x2ae4ec69, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a4ec6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc84ec6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e4ec72, SUC_lexicon_tags_8 },
    { 0x51a4ec73, SUC_lexicon_tags_11 },
    { 0xa4a4ec74, SUC_lexicon_tags_19 },
    { 0x82e4ec75, SUC_lexicon_tags_14 },
    { 0x6c04ec75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6684ec7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7be4ec7d, SUC_lexicon_tags_48 },
    { 0xe3a4ec7e, SUC_lexicon_tags_1 },
    { 0xfb44ec7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3484ec82, SUC_lexicon_tags_60 },
    { 0x26e4ec83, SUC_lexicon_tags_15 },
    { 0xa944ec84, SUC_lexicon_tags_27 },
    { 0x7d44ec85, SUC_lexicon_tags_17 },
    { 0xad24ec83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c4ec8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7924ec95, SUC_lexicon_tags_30 },
    { 0xc464ec95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9224ec98, SUC_lexicon_tags_15 },
    { 0x5744ec98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8764ec9b, SUC_lexicon_tags_24 },
    { 0x9304ec9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a4ec9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe784eca1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c84eca4, SUC_lexicon_tags_27 },
    { 0xbc24eca5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5144eca8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe264ecab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe64ecad, SUC_lexicon_tags_12 },
    { 0x9624ecae, SUC_lexicon_tags_49 },
    { 0xc0e4ecad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda44ecb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc944ecb9, SUC_lexicon_tags_45 },
    { 0x9d84ecba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12c4ecbc, SUC_lexicon_tags_32 },
    { 0x764ecbc, SUC_lexicon_tags_28 },
    { 0xba64ecbe, SUC_lexicon_tags_27 },
    { 0x9464ecbf, SUC_lexicon_tags_12 },
    { 0x6084ecbf, SUC_lexicon_tags_27 },
    { 0x4224ecbf, SUC_lexicon_tags_547 },
    { 0x4264ecc0, SUC_lexicon_tags_15 },
    { 0x2c4ecc1, SUC_lexicon_tags_38 },
    { 0xb1c4ecbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a24eccb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8784eccf, SUC_lexicon_tags_17 },
    { 0x6e24ecd0, SUC_lexicon_tags_15 },
    { 0x7064ecd1, SUC_lexicon_tags_6 },
    { 0x6784ecd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe4ecd6, SUC_lexicon_tags_60 },
    { 0xf704ecd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c24ecda, SUC_lexicon_tags_15 },
    { 0xdb24ecdb, SUC_lexicon_tags_12 },
    { 0xf524ecdb, SUC_lexicon_tags_45 },
    { 0x9064ecdd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5164ece0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5e4ece2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d64ece6, SUC_lexicon_tags_12 },
    { 0xf2a4ece6, SUC_lexicon_tags_31 },
    { 0x95a4ece8, SUC_lexicon_tags_11 },
    { 0x2a84ece8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a4ecf4, SUC_lexicon_tags_37 },
    { 0x7b44ecf4, SUC_lexicon_tags_11 },
    { 0x9564ecf6, SUC_lexicon_tags_37 },
    { 0x3704ecf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe44ecfc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e44ed00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e4ed03, SUC_lexicon_tags_25 },
    { 0xbc04ed04, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba4ed07, SUC_lexicon_tags_39 },
    { 0x8264ed07, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44ed0b, SUC_lexicon_tags_47 },
    { 0x9ba4ed0b, SUC_lexicon_tags_4 },
    { 0x124ed0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9044ed10, SUC_lexicon_tags_47 },
    { 0x1184ed10, SUC_lexicon_tags_38 },
    { 0x4ec4ed12, SUC_lexicon_tags_53 },
    { 0x7f04ed13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e4ed16, SUC_lexicon_tags_33 },
    { 0x1244ed16, SUC_lexicon_tags_25 },
    { 0x3c64ed18, SUC_lexicon_tags_74 },
    { 0x6d24ed19, SUC_lexicon_tags_32 },
    { 0x85c4ed1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7784ed1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdb24ed1e, SUC_lexicon_tags_87 },
    { 0x8d24ed1f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b84ed25, SUC_lexicon_tags_12 },
    { 0xc4e4ed26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc84ed28, SUC_lexicon_tags_63 },
    { 0x4704ed29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b84ed2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e4ed33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2744ed37, SUC_lexicon_tags_33 },
    { 0xbe84ed38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e4ed3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a4ed40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a4ed47, SUC_lexicon_tags_14 },
    { 0x37e4ed47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e4ed4b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc4ed4f, SUC_lexicon_tags_38 },
    { 0x8be4ed50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e4ed53, SUC_lexicon_tags_37 },
    { 0xedc4ed54, SUC_lexicon_tags_28 },
    { 0x3fa4ed55, SUC_lexicon_tags_2 },
    { 0xcf24ed53, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xdb64ed58, SUC_lexicon_tags_79 },
    { 0x1b84ed59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd064ed60, SUC_lexicon_tags_17 },
    { 0x5904ed60, SUC_lexicon_tags_12 },
    { 0x5e04ed62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be4ed6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5004ed6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5244ed74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44c4ed76, SUC_lexicon_tags_37 },
    { 0x6dc4ed77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1764ed7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3824ed81, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6104ed83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c4ed89, SUC_lexicon_tags_9 },
    { 0x25a4ed8a, SUC_lexicon_tags_11 },
    { 0x6744ed8b, SUC_lexicon_tags_7 },
    { 0x2324ed89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d24ed8f, SUC_lexicon_tags_25 },
    { 0x39e4ed8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeaa4ed92, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe4ed97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a84ed99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x77c4ed9b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c4ed9f, SUC_lexicon_tags_39 },
    { 0x8704eda0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c4eda7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa584edab, SUC_lexicon_tags_2 },
    { 0x1ea4edac, SUC_lexicon_tags_17 },
    { 0x2904edac, SUC_lexicon_tags_0 },
    { 0x73c4edae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc284edb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2244edb5, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf204edb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a4edbf, SUC_lexicon_tags_154 },
    { 0x49a4edc0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c4edc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc24edc9, SUC_lexicon_tags_4 },
    { 0x3364edca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6864edcc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xad44edcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa4edd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc444edd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe4edd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a4eddc, SUC_lexicon_tags_0 },
    { 0x17a4eddd, SUC_lexicon_tags_7 },
    { 0x38e4edde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x19a4ede0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7304ede9, SUC_lexicon_tags_8 },
    { 0x8344edea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x184eded, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8844edf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1564edf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644edfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca4edff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e4ee03, SUC_lexicon_tags_26 },
    { 0x45a4ee04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5544ee10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc064ee15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a4ee19, SUC_lexicon_tags_37 },
    { 0xfa84ee1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a4ee1f, SUC_lexicon_tags_14 },
    { 0x1264ee20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe484ee22, SUC_lexicon_tags_31 },
    { 0xb584ee23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd584ee27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6be4ee29, SUC_lexicon_tags_10 },
    { 0x3c44ee29, SUC_lexicon_tags_17 },
    { 0xdc64ee2b, SUC_lexicon_tags_38 },
    { 0x6e24ee2b, SUC_lexicon_tags_29 },
    { 0x7124ee2a, SUC_lexicon_tags_17 },
    { 0x2364ee29, SUC_lexicon_tags_27 },
    { 0x4ce4ee29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b04ee31, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3644ee33, SUC_lexicon_tags_11 },
    { 0x4e4ee33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c4ee39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf04ee3b, SUC_lexicon_tags_49 },
    { 0xec4ee3b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa644ee3f, SUC_lexicon_tags_37 },
    { 0x47c4ee40, SUC_lexicon_tags_1 },
    { 0xe664ee40, SUC_lexicon_tags_38 },
    { 0xca24ee3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a64ee44, SUC_lexicon_tags_0 },
    { 0xd424ee45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b64ee4a, SUC_lexicon_tags_63 },
    { 0xaa4ee4b, SUC_lexicon_tags_0 },
    { 0x2384ee4c, SUC_lexicon_tags_11 },
    { 0x3a84ee4d, SUC_lexicon_tags_17 },
    { 0x13c4ee4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c44ee53, SUC_lexicon_tags_8 },
    { 0x5284ee54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a4ee57, SUC_lexicon_tags_7 },
    { 0x4144ee58, SUC_lexicon_tags_12 },
    { 0xac44ee57, SUC_lexicon_tags_16 },
    { 0xbb64ee5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a4ee5d, SUC_lexicon_tags_1 },
    { 0x89e4ee5e, SUC_lexicon_tags_9 },
    { 0x5c44ee5d, SUC_lexicon_tags_33 },
    { 0x1b24ee60, SUC_lexicon_tags_11 },
    { 0x7c44ee61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8944ee66, SUC_lexicon_tags_6 },
    { 0x264ee67, SUC_lexicon_tags_8 },
    { 0x1b84ee68, SUC_lexicon_tags_14 },
    { 0x54e4ee68, SUC_lexicon_tags_25 },
    { 0xdbc4ee68, SUC_lexicon_tags_2 },
    { 0xfaa4ee6b, SUC_lexicon_tags_2 },
    { 0xb584ee6c, SUC_lexicon_tags_11 },
    { 0xa4e4ee67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3204ee70, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb24ee72, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2764ee74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc624ee7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a4ee80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1904ee82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f04ee8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf884ee8f, SUC_lexicon_tags_12 },
    { 0x8e24ee90, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c4ee93, SUC_lexicon_tags_56 },
    { 0x4024ee94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc4ee97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd944ee99, SUC_lexicon_tags_17 },
    { 0xe704ee9a, SUC_lexicon_tags_12 },
    { 0xd384ee99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b24eea1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x8784eea4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfc04eea6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a4eeaa, SUC_lexicon_tags_15 },
    { 0x2dc4eeab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6364eeb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7724eeb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf44eebd, SUC_lexicon_tags_44 },
    { 0x5c44eebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2be4eec0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a64eed4, SUC_lexicon_tags_2 },
    { 0x3a44eed5, SUC_lexicon_tags_68 },
    { 0xb904eed5, SUC_lexicon_tags_63 },
    { 0x69c4eed5, SUC_lexicon_tags_37 },
    { 0xd784eed8, SUC_lexicon_tags_1 },
    { 0x46a4eed7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xece4eedb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4464eee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d44eee3, SUC_lexicon_tags_17 },
    { 0x90e4eee4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd924eee6, SUC_lexicon_tags_6 },
    { 0x5f04eee6, SUC_lexicon_tags_26 },
    { 0x60a4eee8, SUC_lexicon_tags_26 },
    { 0x14a4eee7, SUC_lexicon_tags_49 },
    { 0x1a44eeea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xffa4eeec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbca4eeee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb404eef0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b24eef3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7964eef6, SUC_lexicon_tags_33 },
    { 0x7684eef7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e4eefc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1dc4eefe, SUC_lexicon_tags_0 },
    { 0xb5c4eeff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc4ef02, SUC_lexicon_tags_17 },
    { 0x7484ef03, SUC_lexicon_tags_43 },
    { 0xcc44ef04, SUC_lexicon_tags_9 },
    { 0x2fc4ef05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04ef0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524ef15, SUC_lexicon_tags_1 },
    { 0x844ef16, SUC_lexicon_tags_387 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6684ef1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad84ef22, SUC_lexicon_tags_48 },
    { 0xd624ef23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5824ef26, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca4ef29, SUC_lexicon_tags_17 },
    { 0xc344ef2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a84ef2d, SUC_lexicon_tags_3 },
    { 0xa0a4ef2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0c4ef30, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xac04ef33, SUC_lexicon_tags_17 },
    { 0x31c4ef34, SUC_lexicon_tags_57 },
    { 0x7c4ef33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca4ef3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1584ef3e, SUC_lexicon_tags_12 },
    { 0x6064ef3e, SUC_lexicon_tags_12 },
    { 0xe624ef40, SUC_lexicon_tags_7 },
    { 0x9e04ef41, SUC_lexicon_tags_1 },
    { 0x7724ef40, SUC_lexicon_tags_26 },
    { 0xe7e4ef40, SUC_lexicon_tags_46 },
    { 0xfc04ef3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c4ef48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a4ef4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf584ef4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a4ef52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b44ef54, SUC_lexicon_tags_1 },
    { 0xb284ef55, SUC_lexicon_tags_30 },
    { 0x17a4ef56, SUC_lexicon_tags_7 },
    { 0xf264ef57, SUC_lexicon_tags_25 },
    { 0xf2a4ef54, SUC_lexicon_tags_21 },
    { 0x4584ef59, SUC_lexicon_tags_40 },
    { 0xa304ef5a, SUC_lexicon_tags_55 },
    { 0xe824ef5b, SUC_lexicon_tags_17 },
    { 0x5a84ef5c, SUC_lexicon_tags_21 },
    { 0xd24ef55, SUC_lexicon_tags_7 },
    { 0x6d44ef5e, SUC_lexicon_tags_12 },
    { 0xa784ef5c, SUC_lexicon_tags_68 },
    { 0x16c4ef59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd44ef62, SUC_lexicon_tags_6 },
    { 0x6bc4ef63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde4ef65, SUC_lexicon_tags_15 },
    { 0xa7c4ef66, SUC_lexicon_tags_65 },
    { 0xa624ef67, SUC_lexicon_tags_33 },
    { 0x1004ef68, SUC_lexicon_tags_60 },
    { 0x4604ef68, SUC_lexicon_tags_7 },
    { 0x7924ef6a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa464ef6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4ef7a, SUC_lexicon_tags_38 },
    { 0xf684ef7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea84ef7d, SUC_lexicon_tags_1 },
    { 0x4cc4ef7e, SUC_lexicon_tags_63 },
    { 0x8b24ef7d, SUC_lexicon_tags_23 },
    { 0xf5e4ef7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2ec4ef82, SUC_lexicon_tags_37 },
    { 0xf6e4ef83, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf604ef87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe164ef8a, SUC_lexicon_tags_0 },
    { 0x8ec4ef8a, SUC_lexicon_tags_49 },
    { 0xb3e4ef8c, SUC_lexicon_tags_30 },
    { 0x7264ef8d, SUC_lexicon_tags_14 },
    { 0xf9c4ef8d, SUC_lexicon_tags_39 },
    { 0xf24ef8e, SUC_lexicon_tags_10 },
    { 0x77a4ef8e, SUC_lexicon_tags_15 },
    { 0xc864ef91, SUC_lexicon_tags_12 },
    { 0x11e4ef8e, SUC_lexicon_tags_56 },
    { 0x2aa4ef93, SUC_lexicon_tags_12 },
    { 0x38e4ef8c, SUC_lexicon_tags_11 },
    { 0xdac4ef95, SUC_lexicon_tags_17 },
    { 0x3764ef95, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xefe4ef98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc4ef9b, SUC_lexicon_tags_60 },
    { 0x21a4ef9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a24ef9f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd4c4efa1, SUC_lexicon_tags_17 },
    { 0x54a4efa1, SUC_lexicon_tags_25 },
    { 0xf004efa3, SUC_lexicon_tags_17 },
    { 0x5804efa4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2764efab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x69c4efad, SUC_lexicon_tags_4 },
    { 0x8c04efad, SUC_lexicon_tags_18 },
    { 0x144efaf, SUC_lexicon_tags_31 },
    { 0x8a04efb0, SUC_lexicon_tags_13 },
    { 0xfd44efae, SUC_lexicon_tags_27 },
    { 0x4a4efb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce4efb5, SUC_lexicon_tags_129 },
    { 0x9784efb6, SUC_lexicon_tags_15 },
    { 0xd924efb7, SUC_lexicon_tags_17 },
    { 0x6aa4efb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a24efbc, SUC_lexicon_tags_37 },
    { 0x4884efbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a4efc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6124efc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac4efc7, SUC_lexicon_tags_0 },
    { 0x1604efc8, SUC_lexicon_tags_15 },
    { 0xe284efc9, SUC_lexicon_tags_1 },
    { 0x6d04efc7, SUC_lexicon_tags_26 },
    { 0x5104efcb, SUC_lexicon_tags_45 },
    { 0x3584efcb, SUC_lexicon_tags_25 },
    { 0x1964efc7, SUC_lexicon_tags_49 },
    { 0xf764efce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8824efd0, SUC_lexicon_tags_56 },
    { 0xcd84efd1, SUC_lexicon_tags_25 },
    { 0xaf84efd2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d44efd5, SUC_lexicon_tags_45 },
    { 0xd1a4efd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ce4efd8, SUC_lexicon_tags_12 },
    { 0x1944efd9, SUC_lexicon_tags_68 },
    { 0xa524efd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4104efde, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e4efe1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2644efe9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea4efed, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a4eff0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e4eff4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d84eff7, SUC_lexicon_tags_17 },
    { 0x8044eff8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6484effa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7864effc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2144f000, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1504f002, SUC_lexicon_tags_37 },
    { 0x6344f002, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb624f007, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xd904f009, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ae4f00b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3744f00d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a4f010, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e4f019, SUC_lexicon_tags_1 },
    { 0x7944f019, SUC_lexicon_tags_1 },
    { 0x4544f01b, SUC_lexicon_tags_15 },
    { 0xb724f01b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c64f01e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f84f023, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea24f02a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe4f02d, SUC_lexicon_tags_12 },
    { 0x14e4f02e, SUC_lexicon_tags_40 },
    { 0x1e64f02f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e4f032, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe784f034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2264f037, SUC_lexicon_tags_2 },
    { 0x3ea4f038, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe804f03a, SUC_lexicon_tags_4 },
    { 0xc5c4f03b, SUC_lexicon_tags_45 },
    { 0x9c84f03b, SUC_lexicon_tags_23 },
    { 0x5324f03d, SUC_lexicon_tags_37 },
    { 0x1264f03d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4004f041, SUC_lexicon_tags_11 },
    { 0x2424f042, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2584f048, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x79a4f04a, SUC_lexicon_tags_2 },
    { 0xc044f04b, SUC_lexicon_tags_101 },
    { 0xde44f04a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a4f04f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6cc4f051, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f64f055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9dc4f057, SUC_lexicon_tags_47 },
    { 0x4ca4f058, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a24f05e, SUC_lexicon_tags_11 },
    { 0x1a4f05e, SUC_lexicon_tags_9 },
    { 0xfa44f05f, SUC_lexicon_tags_11 },
    { 0xa9e4f060, SUC_lexicon_tags_1 },
    { 0x7144f05e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4f064, SUC_lexicon_tags_6 },
    { 0x3444f065, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e84f067, SUC_lexicon_tags_17 },
    { 0xbda4f067, SUC_lexicon_tags_11 },
    { 0x7904f069, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x784f06b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c4f073, SUC_lexicon_tags_30 },
    { 0xff44f074, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7884f077, SUC_lexicon_tags_12 },
    { 0xc4e4f078, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c4f07d, SUC_lexicon_tags_1 },
    { 0xbc4f07e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6224f082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c4f088, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f44f08a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd764f08c, SUC_lexicon_tags_29 },
    { 0x49e4f08d, SUC_lexicon_tags_65 },
    { 0x8224f08e, SUC_lexicon_tags_77 },
    { 0xd504f08f, SUC_lexicon_tags_25 },
    { 0xce4f08f, SUC_lexicon_tags_35 },
    { 0xa124f091, SUC_lexicon_tags_45 },
    { 0xf224f08e, SUC_lexicon_tags_21 },
    { 0x7d24f093, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8684f096, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda84f098, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9684f09a, SUC_lexicon_tags_29 },
    { 0x5a04f09a, SUC_lexicon_tags_43 },
    { 0x8144f09c, SUC_lexicon_tags_11 },
    { 0xe9c4f09c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3724f0a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f04f0a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c4f0aa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb124f0af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4264f0b3, SUC_lexicon_tags_10 },
    { 0x1d84f0b4, SUC_lexicon_tags_63 },
    { 0x24a4f0b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd144f0b8, SUC_lexicon_tags_27 },
    { 0x4d04f0b9, SUC_lexicon_tags_37 },
    { 0xbee4f0ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e44f0bc, SUC_lexicon_tags_32 },
    { 0x2244f0bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x97e4f0bf, SUC_lexicon_tags_7 },
    { 0x7944f0c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9924f0c5, SUC_lexicon_tags_8 },
    { 0x1104f0c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b64f0ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf184f0ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe244f0d2, SUC_lexicon_tags_11 },
    { 0xe584f0d3, SUC_lexicon_tags_1 },
    { 0xa424f0d3, SUC_lexicon_tags_37 },
    { 0x8504f0d5, SUC_lexicon_tags_25 },
    { 0x7644f0d6, SUC_lexicon_tags_32 },
    { 0x79e4f0d7, SUC_lexicon_tags_28 },
    { 0x5ae4f0d8, SUC_lexicon_tags_48 },
    { 0x5d24f0d9, SUC_lexicon_tags_60 },
    { 0x7ba4f0da, SUC_lexicon_tags_29 },
    { 0x4d24f0d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7004f0dd, SUC_lexicon_tags_8 },
    { 0x4704f0de, SUC_lexicon_tags_7 },
    { 0xea64f0dd, SUC_lexicon_tags_48 },
    { 0x2684f0df, SUC_lexicon_tags_60 },
    { 0xacc4f0e1, SUC_lexicon_tags_8 },
    { 0x6024f0e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8744f0e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb04f0e8, SUC_lexicon_tags_8 },
    { 0x4664f0e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce4f0ec, SUC_lexicon_tags_2 },
    { 0x5324f0ed, SUC_lexicon_tags_25 },
    { 0x5aa4f0ee, SUC_lexicon_tags_1 },
    { 0x8fc4f0ef, SUC_lexicon_tags_4 },
    { 0xcfc4f0ef, SUC_lexicon_tags_11 },
    { 0xd864f0ee, SUC_lexicon_tags_19 },
    { 0x6aa4f0ef, SUC_lexicon_tags_31 },
    { 0x3a64f0f3, SUC_lexicon_tags_39 },
    { 0x69c4f0f4, SUC_lexicon_tags_17 },
    { 0xd444f0f5, SUC_lexicon_tags_2 },
    { 0xf8c4f0f6, SUC_lexicon_tags_47 },
    { 0x1e64f0f6, SUC_lexicon_tags_55 },
    { 0x6ac4f0f5, SUC_lexicon_tags_37 },
    { 0x4444f0f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f64f0ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc4f102, SUC_lexicon_tags_30 },
    { 0x5424f102, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa244f106, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b84f10d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a24f110, SUC_lexicon_tags_13 },
    { 0xe204f111, SUC_lexicon_tags_77 },
    { 0xc464f111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37a4f114, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a4f117, SUC_lexicon_tags_1 },
    { 0x2f04f118, SUC_lexicon_tags_15 },
    { 0xc624f119, SUC_lexicon_tags_11 },
    { 0x2124f118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa244f11f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca4f126, SUC_lexicon_tags_15 },
    { 0xa264f127, SUC_lexicon_tags_45 },
    { 0x2ae4f128, SUC_lexicon_tags_30 },
    { 0x7824f128, SUC_lexicon_tags_46 },
    { 0xa724f129, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd584f135, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe844f138, SUC_lexicon_tags_25 },
    { 0x8524f139, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe084f13b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa44f140, SUC_lexicon_tags_21 },
    { 0xc1c4f141, SUC_lexicon_tags_37 },
    { 0x564f142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea4f149, SUC_lexicon_tags_32 },
    { 0x3ba4f14a, SUC_lexicon_tags_13 },
    { 0x5664f14a, SUC_lexicon_tags_2 },
    { 0x2e24f14c, SUC_lexicon_tags_15 },
    { 0xd9c4f14a, SUC_lexicon_tags_84 },
    { 0xbdc4f14e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa864f151, SUC_lexicon_tags_6 },
    { 0xaba4f152, SUC_lexicon_tags_63 },
    { 0xb664f151, SUC_lexicon_tags_11 },
    { 0xe1c4f152, SUC_lexicon_tags_25 },
    { 0x4544f155, SUC_lexicon_tags_17 },
    { 0x53a4f155, SUC_lexicon_tags_29 },
    { 0xf164f155, SUC_lexicon_tags_37 },
    { 0x6744f155, SUC_lexicon_tags_27 },
    { 0xf3c4f159, SUC_lexicon_tags_15 },
    { 0x8324f15a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda4f15d, SUC_lexicon_tags_4 },
    { 0x9f64f15d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe424f163, SUC_lexicon_tags_14 },
    { 0x2584f164, SUC_lexicon_tags_27 },
    { 0x5264f165, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a84f16c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7784f16e, SUC_lexicon_tags_25 },
    { 0x1904f16f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc484f172, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a4f175, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c44f179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f44f17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3224f17f, SUC_lexicon_tags_2 },
    { 0x8964f180, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c4f185, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a4f189, SUC_lexicon_tags_17 },
    { 0xf3a4f18a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe604f18e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2804f195, SUC_lexicon_tags_47 },
    { 0x9284f195, SUC_lexicon_tags_57 },
    { 0xd924f197, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeea4f199, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b44f19d, SUC_lexicon_tags_25 },
    { 0xc824f19e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce4f1a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e4f1a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a4f1aa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x84c4f1ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75a4f1ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a4f1b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf244f1b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5084f1b8, SUC_lexicon_tags_23 },
    { 0x30c4f1b9, SUC_lexicon_tags_7 },
    { 0x2884f1b8, SUC_lexicon_tags_29 },
    { 0x7f04f1bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5604f1be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa24f1c1, SUC_lexicon_tags_2 },
    { 0x7bc4f1c2, SUC_lexicon_tags_30 },
    { 0x4264f1c3, SUC_lexicon_tags_14 },
    { 0x9644f1c1, SUC_lexicon_tags_63 },
    { 0xef44f1c5, SUC_lexicon_tags_1 },
    { 0x5724f1c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4f1cb, SUC_lexicon_tags_43 },
    { 0x76c4f1cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe564f1ce, SUC_lexicon_tags_17 },
    { 0x5044f1cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e4f1d4, SUC_lexicon_tags_7 },
    { 0xce4f1d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc924f1d9, SUC_lexicon_tags_60 },
    { 0x7044f1da, SUC_lexicon_tags_27 },
    { 0x4e4f1db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef64f1dd, SUC_lexicon_tags_1 },
    { 0xce04f1de, SUC_lexicon_tags_7 },
    { 0xbea4f1de, SUC_lexicon_tags_37 },
    { 0x4404f1e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e4f1e5, SUC_lexicon_tags_26 },
    { 0x9924f1e6, SUC_lexicon_tags_14 },
    { 0xca84f1e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb844f1e9, SUC_lexicon_tags_33 },
    { 0xa8e4f1ea, SUC_lexicon_tags_17 },
    { 0xc584f1eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd84f1f0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c24f1f6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9c24f1f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7104f1fb, SUC_lexicon_tags_17 },
    { 0xdea4f1fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f24f1fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e44f203, SUC_lexicon_tags_15 },
    { 0x3844f203, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x704f207, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4544f20a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e4f211, SUC_lexicon_tags_11 },
    { 0x3444f212, SUC_lexicon_tags_9 },
    { 0x55c4f213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6a4f215, SUC_lexicon_tags_178 },
    { 0x6da4f216, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a4f21e, SUC_lexicon_tags_3 },
    { 0x80c4f21f, SUC_lexicon_tags_26 },
    { 0x79c4f21e, SUC_lexicon_tags_49 },
    { 0x8584f21f, SUC_lexicon_tags_27 },
    { 0x7a24f220, SUC_lexicon_tags_0 },
    { 0x3ac4f21f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb44f226, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa484f229, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc4f22c, SUC_lexicon_tags_19 },
    { 0x1864f22d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de4f232, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c4f237, SUC_lexicon_tags_2 },
    { 0xace4f237, SUC_lexicon_tags_0 },
    { 0xaec4f237, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4944f23b, SUC_lexicon_tags_55 },
    { 0xac04f23c, SUC_lexicon_tags_27 },
    { 0x75e4f23d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca4f240, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c4f244, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2624f246, SUC_lexicon_tags_37 },
    { 0x5dc4f246, SUC_lexicon_tags_11 },
    { 0x9a04f248, SUC_lexicon_tags_60 },
    { 0xc4f249, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd884f24f, SUC_lexicon_tags_25 },
    { 0xe944f250, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x86a4f252, SUC_lexicon_tags_3 },
    { 0xd844f253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb764f256, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe4f259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9144f260, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8fc4f262, SUC_lexicon_tags_17 },
    { 0xb6c4f263, SUC_lexicon_tags_25 },
    { 0x23c4f263, SUC_lexicon_tags_1 },
    { 0x4b44f262, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7584f267, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca4f26a, SUC_lexicon_tags_44 },
    { 0x8e04f26a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea4f26e, SUC_lexicon_tags_16 },
    { 0x8784f26e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb024f271, SUC_lexicon_tags_25 },
    { 0xf464f272, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf44f274, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd164f277, SUC_lexicon_tags_43 },
    { 0x4344f277, SUC_lexicon_tags_7 },
    { 0x29a4f279, SUC_lexicon_tags_37 },
    { 0x5c24f27a, SUC_lexicon_tags_26 },
    { 0xb704f27a, SUC_lexicon_tags_64 },
    { 0xff04f27b, SUC_lexicon_tags_11 },
    { 0x3364f27c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefc4f27f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4364f282, SUC_lexicon_tags_17 },
    { 0xef64f283, SUC_lexicon_tags_31 },
    { 0x2264f284, SUC_lexicon_tags_21 },
    { 0x16a4f285, SUC_lexicon_tags_25 },
    { 0x4f283, SUC_lexicon_tags_8 },
    { 0xb404f286, SUC_lexicon_tags_27 },
    { 0xa7a4f285, SUC_lexicon_tags_13 },
    { 0x3564f289, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46c4f28b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c4f28e, SUC_lexicon_tags_614 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c4f295, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c4f298, SUC_lexicon_tags_43 },
    { 0x8804f299, SUC_lexicon_tags_4 },
    { 0x8bc4f29a, SUC_lexicon_tags_14 },
    { 0xf8a4f299, SUC_lexicon_tags_15 },
    { 0x6a44f29b, SUC_lexicon_tags_32 },
    { 0xa3e4f29d, SUC_lexicon_tags_43 },
    { 0x7404f29e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d04f2a0, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xebc4f2a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fa4f2a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5904f2ab, SUC_lexicon_tags_194 },
    { 0x77e4f2ac, SUC_lexicon_tags_1169 },
    { 0x6e84f2ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc884f2b1, SUC_lexicon_tags_55 },
    { 0xc4a4f2b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de4f2b5, SUC_lexicon_tags_60 },
    { 0x3384f2b5, SUC_lexicon_tags_14 },
    { 0x604f2b7, SUC_lexicon_tags_25 },
    { 0x4664f2b8, SUC_lexicon_tags_3 },
    { 0x51a4f2b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8be4f2bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2da4f2bd, SUC_lexicon_tags_46 },
    { 0xf6e4f2be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe444f2c0, SUC_lexicon_tags_7 },
    { 0x3bc4f2c1, SUC_lexicon_tags_28 },
    { 0x6b84f2c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a84f2c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3724f2cb, SUC_lexicon_tags_61 },
    { 0xc7e4f2cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b24f2ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9584f2d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfda4f2d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c4f2d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc664f2d8, SUC_lexicon_tags_31 },
    { 0xe604f2d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8044f2dc, SUC_lexicon_tags_21 },
    { 0x8184f2dd, SUC_lexicon_tags_25 },
    { 0x42a4f2de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e4f2e2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a4f2eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6c4f2ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea4f2f7, SUC_lexicon_tags_2 },
    { 0x6284f2f7, SUC_lexicon_tags_48 },
    { 0xa744f2f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d24f2fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1604f306, SUC_lexicon_tags_14 },
    { 0x564f307, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a4f30a, SUC_lexicon_tags_16 },
    { 0xc64f30a, SUC_lexicon_tags_29 },
    { 0xa44f30b, SUC_lexicon_tags_12 },
    { 0x72a4f30d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a4f311, SUC_lexicon_tags_16 },
    { 0x93e4f311, SUC_lexicon_tags_25 },
    { 0x9dc4f311, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6fe4f315, SUC_lexicon_tags_48 },
    { 0x5064f315, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa4f31a, SUC_lexicon_tags_27 },
    { 0x31c4f31a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe404f31e, SUC_lexicon_tags_8 },
    { 0x704f31e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5764f321, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae4f32a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee4f333, SUC_lexicon_tags_12 },
    { 0x5e04f333, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7304f339, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2364f33f, SUC_lexicon_tags_48 },
    { 0xff84f340, SUC_lexicon_tags_2 },
    { 0xd1a4f33f, SUC_lexicon_tags_2 },
    { 0x77a4f341, SUC_lexicon_tags_61 },
    { 0x53a4f341, SUC_lexicon_tags_60 },
    { 0xba24f344, SUC_lexicon_tags_17 },
    { 0xe4e4f345, SUC_lexicon_tags_11 },
    { 0x7bc4f346, SUC_lexicon_tags_7 },
    { 0x1584f347, SUC_lexicon_tags_47 },
    { 0xf404f346, SUC_lexicon_tags_65 },
    { 0xff84f349, SUC_lexicon_tags_114 },
    { 0x22e4f34a, SUC_lexicon_tags_2 },
    { 0xf84f34b, SUC_lexicon_tags_37 },
    { 0x6a84f34c, SUC_lexicon_tags_2 },
    { 0xc984f34d, SUC_lexicon_tags_638 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c64f350, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1664f352, SUC_lexicon_tags_40 },
    { 0xf624f353, SUC_lexicon_tags_17 },
    { 0x88a4f354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e44f357, SUC_lexicon_tags_1 },
    { 0xda44f357, SUC_lexicon_tags_31 },
    { 0x86a4f359, SUC_lexicon_tags_39 },
    { 0x9c44f35a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xba04f35c, SUC_lexicon_tags_32 },
    { 0x38a4f35d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70e4f35f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb844f361, SUC_lexicon_tags_15 },
    { 0xfb84f361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11c4f364, SUC_lexicon_tags_38 },
    { 0x2ba4f364, SUC_lexicon_tags_2 },
    { 0x2ac4f366, SUC_lexicon_tags_15 },
    { 0xb2a4f367, SUC_lexicon_tags_25 },
    { 0x3244f365, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6464f36a, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e4f36d, SUC_lexicon_tags_15 },
    { 0x94e4f36d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec4f372, SUC_lexicon_tags_14 },
    { 0x7dc4f372, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3a04f375, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7444f377, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ee4f379, SUC_lexicon_tags_16 },
    { 0x8fa4f37a, SUC_lexicon_tags_7 },
    { 0x23e4f379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1be4f37d, SUC_lexicon_tags_2 },
    { 0xd704f37e, SUC_lexicon_tags_1147 },
    { 0, NULL },
    { 0, NULL },
    { 0x9264f381, SUC_lexicon_tags_26 },
    { 0xed04f382, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5684f384, SUC_lexicon_tags_38 },
    { 0xd924f385, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e4f388, SUC_lexicon_tags_2 },
    { 0xb144f389, SUC_lexicon_tags_68 },
    { 0x62e4f38a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc384f38c, SUC_lexicon_tags_37 },
    { 0x59c4f38d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb444f391, SUC_lexicon_tags_37 },
    { 0x6364f392, SUC_lexicon_tags_29 },
    { 0x6ea4f393, SUC_lexicon_tags_16 },
    { 0x1f24f394, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb724f396, SUC_lexicon_tags_719 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f64f399, SUC_lexicon_tags_19 },
    { 0x6344f39a, SUC_lexicon_tags_11 },
    { 0x2864f39b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa444f39e, SUC_lexicon_tags_19 },
    { 0xac84f39f, SUC_lexicon_tags_32 },
    { 0xca04f3a0, SUC_lexicon_tags_25 },
    { 0x26c4f3a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3544f3a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7464f3a6, SUC_lexicon_tags_33 },
    { 0xf2a4f3a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49c4f3a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e84f3ae, SUC_lexicon_tags_3 },
    { 0x9124f3af, SUC_lexicon_tags_87 },
    { 0x5264f3b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea24f3b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde4f3b7, SUC_lexicon_tags_45 },
    { 0xff44f3b7, SUC_lexicon_tags_33 },
    { 0x5044f3b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x86c4f3bb, SUC_lexicon_tags_27 },
    { 0x53e4f3bb, SUC_lexicon_tags_12 },
    { 0xe1a4f3bc, SUC_lexicon_tags_4 },
    { 0xa84f3be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324f3c3, SUC_lexicon_tags_1 },
    { 0x6004f3c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe44f3c7, SUC_lexicon_tags_33 },
    { 0x7744f3c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c4f3cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44f3d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7084f3d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e4f3da, SUC_lexicon_tags_15 },
    { 0x1004f3da, SUC_lexicon_tags_38 },
    { 0xa964f3dc, SUC_lexicon_tags_37 },
    { 0xa7a4f3dd, SUC_lexicon_tags_70 },
    { 0x24f3de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2064f3e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dc4f3e3, SUC_lexicon_tags_2 },
    { 0x1184f3e3, SUC_lexicon_tags_77 },
    { 0xbe04f3e5, SUC_lexicon_tags_45 },
    { 0x83a4f3e5, SUC_lexicon_tags_43 },
    { 0xb304f3e3, SUC_lexicon_tags_12 },
    { 0x5124f3e8, SUC_lexicon_tags_1 },
    { 0x1504f3e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3504f3eb, SUC_lexicon_tags_17 },
    { 0x95c4f3ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c4f3ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c84f3f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace4f3f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5644f3f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec24f3fa, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa4f402, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7a4f404, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd24f409, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba4f40d, SUC_lexicon_tags_6 },
    { 0x2ee4f40e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc944f415, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec4f41f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4924f424, SUC_lexicon_tags_2 },
    { 0x3be4f424, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2e24f427, SUC_lexicon_tags_67 },
    { 0x5e64f427, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2324f42a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e04f42c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d04f431, SUC_lexicon_tags_15 },
    { 0x2a24f431, SUC_lexicon_tags_60 },
    { 0x8064f432, SUC_lexicon_tags_25 },
    { 0x3ce4f434, SUC_lexicon_tags_15 },
    { 0x6c24f435, SUC_lexicon_tags_14 },
    { 0x8d24f436, SUC_lexicon_tags_11 },
    { 0x2664f437, SUC_lexicon_tags_63 },
    { 0x7144f435, SUC_lexicon_tags_46 },
    { 0x1c4f439, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2fa4f43b, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4f43e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a4f442, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce4f445, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be4f448, SUC_lexicon_tags_15 },
    { 0x4b24f449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e4f44e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1644f454, SUC_lexicon_tags_10 },
    { 0x7624f455, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6004f458, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc004f45c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a4f45f, SUC_lexicon_tags_63 },
    { 0xc9e4f460, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3644f462, SUC_lexicon_tags_10 },
    { 0xc0a4f463, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8284f468, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d44f46e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c4f478, SUC_lexicon_tags_31 },
    { 0x99e4f479, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4f47d, SUC_lexicon_tags_2 },
    { 0xca64f47e, SUC_lexicon_tags_15 },
    { 0x4a64f47e, SUC_lexicon_tags_10 },
    { 0x4b64f47f, SUC_lexicon_tags_23 },
    { 0xcb64f481, SUC_lexicon_tags_23 },
    { 0xdaa4f482, SUC_lexicon_tags_37 },
    { 0x38a4f483, SUC_lexicon_tags_6 },
    { 0x95e4f484, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d24f486, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c64f489, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4384f48b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e4f48e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4884f492, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb464f4a0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbaa4f4a2, SUC_lexicon_tags_2 },
    { 0xfc24f4a3, SUC_lexicon_tags_17 },
    { 0xa764f4a3, SUC_lexicon_tags_12 },
    { 0xcb04f4a4, SUC_lexicon_tags_37 },
    { 0xfb44f4a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c4f4aa, SUC_lexicon_tags_14 },
    { 0x6bc4f4aa, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c4f4ae, SUC_lexicon_tags_2 },
    { 0x1c64f4af, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d44f4b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5024f4bc, SUC_lexicon_tags_25 },
    { 0x16e4f4bc, SUC_lexicon_tags_49 },
    { 0x75a4f4be, SUC_lexicon_tags_1 },
    { 0x55e4f4be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4544f4c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be4f4c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ea4f4c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb364f4cd, SUC_lexicon_tags_13 },
    { 0x2d24f4ce, SUC_lexicon_tags_74 },
    { 0x5344f4cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4264f4d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83e4f4d4, SUC_lexicon_tags_37 },
    { 0x6f04f4d5, SUC_lexicon_tags_27 },
    { 0x5b84f4d4, SUC_lexicon_tags_5 },
    { 0xe9c4f4d7, SUC_lexicon_tags_30 },
    { 0x1104f4d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7204f4db, SUC_lexicon_tags_23 },
    { 0x1124f4dc, SUC_lexicon_tags_11 },
    { 0xc0e4f4dd, SUC_lexicon_tags_2 },
    { 0x604f4db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2024f4e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c4f4e5, SUC_lexicon_tags_43 },
    { 0xbfa4f4e6, SUC_lexicon_tags_11 },
    { 0xa44f4e7, SUC_lexicon_tags_136 },
    { 0xd1c4f4e6, SUC_lexicon_tags_25 },
    { 0xd564f4e9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1164f4eb, SUC_lexicon_tags_0 },
    { 0xfa04f4ec, SUC_lexicon_tags_43 },
    { 0x1624f4ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa724f4f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe124f4f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x204f4f8, SUC_lexicon_tags_26 },
    { 0x65a4f4f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6284f4fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6924f500, SUC_lexicon_tags_37 },
    { 0xefc4f501, SUC_lexicon_tags_26 },
    { 0x3f24f502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb004f505, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2624f509, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50a4f50b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d04f513, SUC_lexicon_tags_12 },
    { 0x7dc4f513, SUC_lexicon_tags_15 },
    { 0x4bc4f514, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb644f518, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc9e4f51a, SUC_lexicon_tags_45 },
    { 0xd604f51b, SUC_lexicon_tags_55 },
    { 0x42c4f51c, SUC_lexicon_tags_29 },
    { 0xfc44f51a, SUC_lexicon_tags_0 },
    { 0xfa84f51e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f24f525, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4f528, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8284f52a, SUC_lexicon_tags_10 },
    { 0x8324f52b, SUC_lexicon_tags_84 },
    { 0x53a4f52b, SUC_lexicon_tags_11 },
    { 0x6ce4f52d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9ea4f52f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a4f532, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a84f536, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8804f53a, SUC_lexicon_tags_60 },
    { 0x58c4f53b, SUC_lexicon_tags_414 },
    { 0x1484f53a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c44f53f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c4f544, SUC_lexicon_tags_29 },
    { 0xf2c4f545, SUC_lexicon_tags_75 },
    { 0xc5c4f546, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ae4f548, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7684f54a, SUC_lexicon_tags_6 },
    { 0xa2a4f54b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a4f54f, SUC_lexicon_tags_38 },
    { 0xa5c4f550, SUC_lexicon_tags_57 },
    { 0x2da4f551, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x884f553, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7804f557, SUC_lexicon_tags_14 },
    { 0xcd24f558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26e4f55a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72a4f55c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbba4f55e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c4f564, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7224f567, SUC_lexicon_tags_45 },
    { 0xe364f568, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7064f56a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f44f56c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce4f570, SUC_lexicon_tags_82 },
    { 0x8644f571, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2964f573, SUC_lexicon_tags_14 },
    { 0x15c4f574, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xf104f576, SUC_lexicon_tags_39 },
    { 0x1804f577, SUC_lexicon_tags_17 },
    { 0x1244f578, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe184f57a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xda04f57c, SUC_lexicon_tags_7 },
    { 0xaf64f57c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9844f580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf04f586, SUC_lexicon_tags_67 },
    { 0x27c4f586, SUC_lexicon_tags_45 },
    { 0x2ce4f587, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e4f58c, SUC_lexicon_tags_12 },
    { 0xe364f58d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a44f592, SUC_lexicon_tags_12 },
    { 0xb944f593, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d64f599, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a84f59b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7964f59f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e4f5a2, SUC_lexicon_tags_2 },
    { 0xe4e4f5a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba4f5b6, SUC_lexicon_tags_11 },
    { 0xbc24f5b6, SUC_lexicon_tags_48 },
    { 0xbd04f5b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc844f5ba, SUC_lexicon_tags_12 },
    { 0xa164f5bb, SUC_lexicon_tags_2 },
    { 0x6ee4f5bc, SUC_lexicon_tags_11 },
    { 0xa344f5bc, SUC_lexicon_tags_7 },
    { 0x8e64f5be, SUC_lexicon_tags_8 },
    { 0xacc4f5bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4204f5c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd8c4f5c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5524f5c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2064f5cb, SUC_lexicon_tags_28 },
    { 0x2a4f5cc, SUC_lexicon_tags_7 },
    { 0xea44f5cd, SUC_lexicon_tags_3 },
    { 0x6c04f5cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe384f5d1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5724f5d3, SUC_lexicon_tags_30 },
    { 0x5d44f5d4, SUC_lexicon_tags_13 },
    { 0x3e24f5d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd24f5d8, SUC_lexicon_tags_32 },
    { 0x7b24f5d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7364f5dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x204f5de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6484f5e6, SUC_lexicon_tags_7 },
    { 0x3fe4f5e7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc244f5f0, SUC_lexicon_tags_25 },
    { 0xb8a4f5f1, SUC_lexicon_tags_17 },
    { 0x60e4f5f2, SUC_lexicon_tags_7 },
    { 0x85a4f5f0, SUC_lexicon_tags_15 },
    { 0xdd44f5f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce4f5f8, SUC_lexicon_tags_23 },
    { 0xe104f5f9, SUC_lexicon_tags_30 },
    { 0xd344f5f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc864f5ff, SUC_lexicon_tags_16 },
    { 0xfaa4f600, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba4f603, SUC_lexicon_tags_17 },
    { 0x144f604, SUC_lexicon_tags_2 },
    { 0x51a4f605, SUC_lexicon_tags_19 },
    { 0x9e64f606, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf684f60c, SUC_lexicon_tags_11 },
    { 0xd524f60d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8344f610, SUC_lexicon_tags_29 },
    { 0x22a4f611, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b64f615, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a64f618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf484f61b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e4f621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf704f626, SUC_lexicon_tags_17 },
    { 0x8504f627, SUC_lexicon_tags_11 },
    { 0x9ca4f627, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3a4f62a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf804f62f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e4f638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5424f63c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4724f63f, SUC_lexicon_tags_10 },
    { 0x7c84f63f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e4f643, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e04f646, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf364f64b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde64f64e, SUC_lexicon_tags_19 },
    { 0xadc4f64e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x48c4f651, SUC_lexicon_tags_21 },
    { 0x7684f651, SUC_lexicon_tags_82 },
    { 0xc424f653, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a4f65f, SUC_lexicon_tags_43 },
    { 0x92a4f660, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d04f663, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2044f667, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9244f66a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5084f66c, SUC_lexicon_tags_2 },
    { 0xed44f66d, SUC_lexicon_tags_165 },
    { 0x42e4f66c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a4f671, SUC_lexicon_tags_7 },
    { 0x7864f672, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e84f678, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a4f67c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c4f67f, SUC_lexicon_tags_7 },
    { 0x4944f680, SUC_lexicon_tags_17 },
    { 0x96e4f67f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb944f683, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa4f68a, SUC_lexicon_tags_37 },
    { 0xa7a4f68b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd64f68d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaa44f68f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef64f697, SUC_lexicon_tags_53 },
    { 0xdcc4f698, SUC_lexicon_tags_68 },
    { 0x844f699, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb6c4f69b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c4f6a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc744f6a6, SUC_lexicon_tags_17 },
    { 0xa764f6a7, SUC_lexicon_tags_11 },
    { 0x6ac4f6a8, SUC_lexicon_tags_12 },
    { 0x7264f6a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f44f6ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3e4f6b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfb44f6b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e4f6b6, SUC_lexicon_tags_27 },
    { 0x72c4f6b7, SUC_lexicon_tags_11 },
    { 0xa344f6b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324f6bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb984f6c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04f6c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2964f6c8, SUC_lexicon_tags_11 },
    { 0xa6a4f6c9, SUC_lexicon_tags_30 },
    { 0xfb64f6c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c4f6cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac4f6d8, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b44f6db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b84f6e2, SUC_lexicon_tags_11 },
    { 0x63c4f6e2, SUC_lexicon_tags_31 },
    { 0x3cc4f6e2, SUC_lexicon_tags_38 },
    { 0xca4f6e3, SUC_lexicon_tags_12 },
    { 0xf244f6e6, SUC_lexicon_tags_35 },
    { 0x7ce4f6e5, SUC_lexicon_tags_1 },
    { 0xe904f6e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x244f6ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c4f6ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce44f6f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc144f6f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f6f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf184f6fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa184f6ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x604f702, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb344f704, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdde4f706, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5844f708, SUC_lexicon_tags_14 },
    { 0xb924f709, SUC_lexicon_tags_23 },
    { 0x5824f709, SUC_lexicon_tags_1 },
    { 0x6d04f70a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5604f70d, SUC_lexicon_tags_46 },
    { 0x4864f70e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x304f710, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd664f71a, SUC_lexicon_tags_17 },
    { 0xe504f71b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2e4f71d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c84f71f, SUC_lexicon_tags_14 },
    { 0x6c84f720, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3184f723, SUC_lexicon_tags_30 },
    { 0x7e84f724, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0x2e84f726, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe564f72d, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b44f733, SUC_lexicon_tags_30 },
    { 0x24f734, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x3244f737, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee4f73a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e4f73d, SUC_lexicon_tags_15 },
    { 0xd2e4f73d, SUC_lexicon_tags_26 },
    { 0x2e24f73d, SUC_lexicon_tags_31 },
    { 0xfdc4f73f, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc4f743, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad64f748, SUC_lexicon_tags_17 },
    { 0x1cc4f748, SUC_lexicon_tags_17 },
    { 0xefe4f74a, SUC_lexicon_tags_2 },
    { 0x6d04f74a, SUC_lexicon_tags_11 },
    { 0xe544f749, SUC_lexicon_tags_48 },
    { 0xc244f749, SUC_lexicon_tags_8 },
    { 0xa384f74c, SUC_lexicon_tags_5 },
    { 0x2f64f74b, SUC_lexicon_tags_25 },
    { 0xfaa4f750, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee4f755, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x864f757, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc864f75a, SUC_lexicon_tags_87 },
    { 0x9e84f75a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c4f75f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9144f761, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8544f763, SUC_lexicon_tags_11 },
    { 0x4ac4f764, SUC_lexicon_tags_33 },
    { 0x3f84f765, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d44f769, SUC_lexicon_tags_11 },
    { 0x11c4f76a, SUC_lexicon_tags_23 },
    { 0xefe4f769, SUC_lexicon_tags_25 },
    { 0xc0a4f76c, SUC_lexicon_tags_33 },
    { 0x8284f76c, SUC_lexicon_tags_12 },
    { 0x19e4f76e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6504f771, SUC_lexicon_tags_37 },
    { 0x9d24f772, SUC_lexicon_tags_12 },
    { 0xcfa4f773, SUC_lexicon_tags_7 },
    { 0xf244f774, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7864f776, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb244f778, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c4f77b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4144f77e, SUC_lexicon_tags_17 },
    { 0x8604f77f, SUC_lexicon_tags_31 },
    { 0x2fa4f780, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xb884f783, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc704f785, SUC_lexicon_tags_33 },
    { 0x87e4f786, SUC_lexicon_tags_9 },
    { 0xfe4f787, SUC_lexicon_tags_11 },
    { 0xff64f788, SUC_lexicon_tags_17 },
    { 0xc604f789, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5444f78c, SUC_lexicon_tags_2 },
    { 0x7444f78d, SUC_lexicon_tags_18 },
    { 0x1b04f78d, SUC_lexicon_tags_32 },
    { 0xbb04f78e, SUC_lexicon_tags_2 },
    { 0xe884f78d, SUC_lexicon_tags_1576 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf164f794, SUC_lexicon_tags_27 },
    { 0xefa4f795, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7264f79b, SUC_lexicon_tags_25 },
    { 0x8a84f79c, SUC_lexicon_tags_27 },
    { 0xce64f79b, SUC_lexicon_tags_37 },
    { 0x5384f79e, SUC_lexicon_tags_31 },
    { 0xd644f79d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe424f7ad, SUC_lexicon_tags_17 },
    { 0x48c4f7ad, SUC_lexicon_tags_27 },
    { 0x6124f7af, SUC_lexicon_tags_11 },
    { 0x7d64f7af, SUC_lexicon_tags_1 },
    { 0x3024f7b1, SUC_lexicon_tags_5 },
    { 0x3c24f7af, SUC_lexicon_tags_37 },
    { 0x58c4f7b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc524f7b7, SUC_lexicon_tags_11 },
    { 0x7784f7b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb84f7bb, SUC_lexicon_tags_47 },
    { 0x9e04f7bc, SUC_lexicon_tags_25 },
    { 0x8a64f7bb, SUC_lexicon_tags_48 },
    { 0x5564f7be, SUC_lexicon_tags_21 },
    { 0x6444f7bb, SUC_lexicon_tags_30 },
    { 0xbb84f7bc, SUC_lexicon_tags_46 },
    { 0x6a04f7c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb384f7c5, SUC_lexicon_tags_61 },
    { 0xee44f7c5, SUC_lexicon_tags_47 },
    { 0x97c4f7c7, SUC_lexicon_tags_60 },
    { 0x3ba4f7c7, SUC_lexicon_tags_12 },
    { 0xcf84f7c9, SUC_lexicon_tags_19 },
    { 0xcac4f7ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5004f7cf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6b84f7d1, SUC_lexicon_tags_1597 },
    { 0xa464f7d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7c4f7d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcc84f7d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6b44f7d8, SUC_lexicon_tags_2 },
    { 0x4f84f7d9, SUC_lexicon_tags_38 },
    { 0x4224f7da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb464f7dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2004f7de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e84f7e2, SUC_lexicon_tags_37 },
    { 0x6444f7e3, SUC_lexicon_tags_47 },
    { 0x9664f7e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30c4f7e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcae4f7e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04f7ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc64f7f1, SUC_lexicon_tags_7 },
    { 0xb7a4f7f1, SUC_lexicon_tags_37 },
    { 0x7ba4f7f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa544f7f5, SUC_lexicon_tags_12 },
    { 0xb5e4f7f6, SUC_lexicon_tags_43 },
    { 0x224f7f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce44f7f9, SUC_lexicon_tags_60 },
    { 0x4b04f7f9, SUC_lexicon_tags_17 },
    { 0x8444f7fa, SUC_lexicon_tags_6 },
    { 0xbea4f7fc, SUC_lexicon_tags_47 },
    { 0x3c24f7fd, SUC_lexicon_tags_38 },
    { 0x9864f7fc, SUC_lexicon_tags_55 },
    { 0xcc04f7ff, SUC_lexicon_tags_8 },
    { 0x5a04f7ff, SUC_lexicon_tags_17 },
    { 0xec44f7fc, SUC_lexicon_tags_15 },
    { 0x7d24f7ff, SUC_lexicon_tags_25 },
    { 0x1444f803, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa144f805, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfae4f807, SUC_lexicon_tags_0 },
    { 0xa4f807, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d84f80c, SUC_lexicon_tags_11 },
    { 0x964f80d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8644f811, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b64f814, SUC_lexicon_tags_17 },
    { 0x14c4f814, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda44f817, SUC_lexicon_tags_19 },
    { 0x1ce4f818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f84f81b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb404f81f, SUC_lexicon_tags_37 },
    { 0x7324f81f, SUC_lexicon_tags_1 },
    { 0xdb04f821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc024f823, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdec4f825, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea4f829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a4f82e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xebc4f830, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c4f833, SUC_lexicon_tags_15 },
    { 0x9064f834, SUC_lexicon_tags_17 },
    { 0x93e4f835, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x44a4f837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6084f83a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e4f83d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ea4f83f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3064f843, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a4f849, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae4f84c, SUC_lexicon_tags_11 },
    { 0x4d64f84d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa624f851, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2124f85a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed04f85f, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc364f863, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7864f865, SUC_lexicon_tags_47 },
    { 0xd4c4f865, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6644f868, SUC_lexicon_tags_70 },
    { 0x3684f868, SUC_lexicon_tags_46 },
    { 0xafc4f86a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1a4f86c, SUC_lexicon_tags_12 },
    { 0x1d64f86c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6fe4f86f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8944f871, SUC_lexicon_tags_0 },
    { 0x7ae4f871, SUC_lexicon_tags_12 },
    { 0x9b64f873, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4f878, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbb24f87a, SUC_lexicon_tags_10 },
    { 0x3664f87b, SUC_lexicon_tags_37 },
    { 0xef64f87c, SUC_lexicon_tags_15 },
    { 0x4484f87c, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e24f885, SUC_lexicon_tags_10 },
    { 0xd304f886, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8b24f888, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe64f88a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1fa4f88c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd04f88e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae4f896, SUC_lexicon_tags_11 },
    { 0x6884f896, SUC_lexicon_tags_11 },
    { 0x8a84f896, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4f89b, SUC_lexicon_tags_66 },
    { 0x2624f89c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a4f8a6, SUC_lexicon_tags_11 },
    { 0x8e24f8a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd24f8a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce64f8ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4f8b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4e84f8b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa64f8ba, SUC_lexicon_tags_33 },
    { 0xfca4f8bb, SUC_lexicon_tags_27 },
    { 0x30a4f8bc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c4f8bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8184f8c3, SUC_lexicon_tags_37 },
    { 0xbe4f8c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd484f8c6, SUC_lexicon_tags_25 },
    { 0x6564f8c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19c4f8c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4f8cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd604f8d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc4f8d8, SUC_lexicon_tags_63 },
    { 0x3d44f8d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c4f8dd, SUC_lexicon_tags_11 },
    { 0xfde4f8de, SUC_lexicon_tags_15 },
    { 0x39e4f8df, SUC_lexicon_tags_15 },
    { 0x5164f8df, SUC_lexicon_tags_27 },
    { 0x1284f8df, SUC_lexicon_tags_1 },
    { 0x56e4f8e0, SUC_lexicon_tags_7 },
    { 0x3c64f8e3, SUC_lexicon_tags_37 },
    { 0x244f8dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfda4f8e6, SUC_lexicon_tags_48 },
    { 0xdd24f8e6, SUC_lexicon_tags_11 },
    { 0x7ac4f8e8, SUC_lexicon_tags_2 },
    { 0xee4f8e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdde4f8eb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8d84f8ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6244f8f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29c4f8f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d04f8f7, SUC_lexicon_tags_0 },
    { 0x68c4f8f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c04f8fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4224f8fc, SUC_lexicon_tags_43 },
    { 0x3204f8fc, SUC_lexicon_tags_7 },
    { 0x2c24f8fc, SUC_lexicon_tags_49 },
    { 0x4604f8ff, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xaf24f901, SUC_lexicon_tags_25 },
    { 0xd444f902, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea4f906, SUC_lexicon_tags_27 },
    { 0xe7e4f907, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52c4f909, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a84f90d, SUC_lexicon_tags_0 },
    { 0xf444f90d, SUC_lexicon_tags_0 },
    { 0x8624f90f, SUC_lexicon_tags_47 },
    { 0x28a4f90f, SUC_lexicon_tags_31 },
    { 0xfbc4f90d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee4f914, SUC_lexicon_tags_47 },
    { 0xfea4f915, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9a4f917, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfe4f919, SUC_lexicon_tags_33 },
    { 0x1f24f91a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1904f91d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe724f924, SUC_lexicon_tags_37 },
    { 0x1604f925, SUC_lexicon_tags_8 },
    { 0x77e4f926, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c4f92b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb444f92f, SUC_lexicon_tags_1 },
    { 0x7724f92f, SUC_lexicon_tags_43 },
    { 0x50a4f931, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9044f934, SUC_lexicon_tags_32 },
    { 0xcb64f935, SUC_lexicon_tags_19 },
    { 0xb9e4f936, SUC_lexicon_tags_25 },
    { 0xd6a4f936, SUC_lexicon_tags_37 },
    { 0x7b84f934, SUC_lexicon_tags_62 },
    { 0x4d04f939, SUC_lexicon_tags_46 },
    { 0x7bc4f935, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e4f93d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe4f946, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe4c4f948, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7024f94d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa904f951, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4f955, SUC_lexicon_tags_61 },
    { 0xa024f955, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5024f95c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4684f95f, SUC_lexicon_tags_11 },
    { 0x4384f95f, SUC_lexicon_tags_11 },
    { 0x3304f961, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2184f966, SUC_lexicon_tags_38 },
    { 0x7a84f967, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x64f969, SUC_lexicon_tags_56 },
    { 0x45e4f96a, SUC_lexicon_tags_49 },
    { 0xcea4f96b, SUC_lexicon_tags_113 },
    { 0xf484f96c, SUC_lexicon_tags_14 },
    { 0x5204f96d, SUC_lexicon_tags_33 },
    { 0x6a4f96b, SUC_lexicon_tags_54 },
    { 0xbbe4f96b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bc4f971, SUC_lexicon_tags_37 },
    { 0x704f971, SUC_lexicon_tags_63 },
    { 0x6fe4f973, SUC_lexicon_tags_1 },
    { 0xbb04f971, SUC_lexicon_tags_7 },
    { 0xeea4f975, SUC_lexicon_tags_39 },
    { 0x5404f973, SUC_lexicon_tags_63 },
    { 0x1264f977, SUC_lexicon_tags_14 },
    { 0x9784f978, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8544f97a, SUC_lexicon_tags_12 },
    { 0xb7a4f97b, SUC_lexicon_tags_95 },
    { 0xe844f97c, SUC_lexicon_tags_38 },
    { 0xcb24f97d, SUC_lexicon_tags_17 },
    { 0xe04f97e, SUC_lexicon_tags_37 },
    { 0xa4f97f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa984f981, SUC_lexicon_tags_18 },
    { 0xb584f982, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfda4f984, SUC_lexicon_tags_27 },
    { 0xcfa4f985, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904f989, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26a4f98b, SUC_lexicon_tags_15 },
    { 0x2824f98c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba4f98f, SUC_lexicon_tags_45 },
    { 0xa264f990, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2344f993, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a4f998, SUC_lexicon_tags_77 },
    { 0x7c24f999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe764f99c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc004f9a2, SUC_lexicon_tags_37 },
    { 0x6ba4f9a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4004f9a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5384f9a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x504f9a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a4f9ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5104f9af, SUC_lexicon_tags_31 },
    { 0xd5c4f9b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf544f9b2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c24f9b7, SUC_lexicon_tags_15 },
    { 0x57a4f9b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9824f9ba, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c4f9be, SUC_lexicon_tags_33 },
    { 0xfe24f9be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc4f9c3, SUC_lexicon_tags_15 },
    { 0xbea4f9c4, SUC_lexicon_tags_60 },
    { 0x1cc4f9c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce4f9ca, SUC_lexicon_tags_29 },
    { 0xd4a4f9cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2104f9cf, SUC_lexicon_tags_6 },
    { 0xa724f9d0, SUC_lexicon_tags_17 },
    { 0x8624f9d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca4f9d4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a4f9d9, SUC_lexicon_tags_43 },
    { 0xc6c4f9da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e24f9e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8064f9e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584f9e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304f9ee, SUC_lexicon_tags_24 },
    { 0x184f9ef, SUC_lexicon_tags_7 },
    { 0xaf64f9ef, SUC_lexicon_tags_17 },
    { 0xada4f9ef, SUC_lexicon_tags_12 },
    { 0x1e84f9f0, SUC_lexicon_tags_8 },
    { 0x8d24f9f3, SUC_lexicon_tags_1 },
    { 0xec84f9f3, SUC_lexicon_tags_0 },
    { 0x7604f9f5, SUC_lexicon_tags_37 },
    { 0x3504f9f5, SUC_lexicon_tags_1 },
    { 0x4aa4f9f6, SUC_lexicon_tags_7 },
    { 0xd744f9f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa964f9fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdf04f9fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5184f9fe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb1a4fa00, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1e24fa02, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5b44fa04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a4fa07, SUC_lexicon_tags_25 },
    { 0x8484fa08, SUC_lexicon_tags_12 },
    { 0x17e4fa07, SUC_lexicon_tags_39 },
    { 0x6c64fa09, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee4fa0e, SUC_lexicon_tags_12 },
    { 0xbea4fa0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6764fa12, SUC_lexicon_tags_47 },
    { 0x5244fa12, SUC_lexicon_tags_37 },
    { 0xb2c4fa14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca44fa17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4464fa1b, SUC_lexicon_tags_25 },
    { 0x6924fa1c, SUC_lexicon_tags_2 },
    { 0xeac4fa1b, SUC_lexicon_tags_63 },
    { 0x3984fa1e, SUC_lexicon_tags_38 },
    { 0x84c4fa1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c64fa21, SUC_lexicon_tags_29 },
    { 0x6184fa22, SUC_lexicon_tags_1 },
    { 0xf2a4fa23, SUC_lexicon_tags_45 },
    { 0x6424fa22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5464fa2f, SUC_lexicon_tags_63 },
    { 0x58e4fa30, SUC_lexicon_tags_12 },
    { 0xc3a4fa2f, SUC_lexicon_tags_37 },
    { 0x584fa2f, SUC_lexicon_tags_33 },
    { 0xb8a4fa33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce4fa38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdda4fa3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd044fa3c, SUC_lexicon_tags_37 },
    { 0xbc44fa3c, SUC_lexicon_tags_42 },
    { 0x96e4fa3d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaa44fa40, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c64fa44, SUC_lexicon_tags_17 },
    { 0x6ac4fa45, SUC_lexicon_tags_61 },
    { 0x33e4fa46, SUC_lexicon_tags_2 },
    { 0x5164fa45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb744fa49, SUC_lexicon_tags_178 },
    { 0x5884fa49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8a4fa4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4fa4f, SUC_lexicon_tags_48 },
    { 0xbbc4fa50, SUC_lexicon_tags_68 },
    { 0x3e24fa51, SUC_lexicon_tags_55 },
    { 0x2ac4fa51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c4fa58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc4fa5e, SUC_lexicon_tags_44 },
    { 0x34c4fa5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e4fa62, SUC_lexicon_tags_1 },
    { 0xa484fa63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5504fa65, SUC_lexicon_tags_12 },
    { 0x664fa65, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24fa6b, SUC_lexicon_tags_11 },
    { 0x3844fa6c, SUC_lexicon_tags_17 },
    { 0x7204fa6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd344fa6f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x5aa4fa71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c64fa79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc464fa7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a4fa7f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce4fa85, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6604fa87, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8804fa8b, SUC_lexicon_tags_30 },
    { 0x1644fa8c, SUC_lexicon_tags_2 },
    { 0xd864fa8b, SUC_lexicon_tags_64 },
    { 0xeda4fa8e, SUC_lexicon_tags_19 },
    { 0x88c4fa8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf84fa91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae4fa94, SUC_lexicon_tags_17 },
    { 0x3404fa94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7544fa98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9084fa9a, SUC_lexicon_tags_48 },
    { 0x244fa9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5864fa9f, SUC_lexicon_tags_2 },
    { 0xbf04fa9f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xda04faa3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x124faa6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4faac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcc4faae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7a4fab0, SUC_lexicon_tags_8 },
    { 0x1a4fab1, SUC_lexicon_tags_37 },
    { 0x4024fab2, SUC_lexicon_tags_25 },
    { 0xb8c4fab3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c4fab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc64fabd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4884fac2, SUC_lexicon_tags_15 },
    { 0x70a4fac2, SUC_lexicon_tags_18 },
    { 0x9d64fac4, SUC_lexicon_tags_19 },
    { 0xca84fac3, SUC_lexicon_tags_60 },
    { 0x41a4fac4, SUC_lexicon_tags_26 },
    { 0x2e84fac7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb764fac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb844facc, SUC_lexicon_tags_30 },
    { 0xfac4facd, SUC_lexicon_tags_10 },
    { 0x4d84face, SUC_lexicon_tags_7 },
    { 0x58e4facf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b04fad5, SUC_lexicon_tags_27 },
    { 0x5064fad6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13c4fad8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7224fada, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd44fade, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc284fae2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a4fae8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe4faec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49e4faee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe04faf6, SUC_lexicon_tags_15 },
    { 0x5944faf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x65c4faf9, SUC_lexicon_tags_60 },
    { 0x5fa4fafa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7724fafc, SUC_lexicon_tags_7 },
    { 0x5724fafd, SUC_lexicon_tags_29 },
    { 0xa744fafd, SUC_lexicon_tags_37 },
    { 0xdde4fafd, SUC_lexicon_tags_80 },
    { 0xee44fafd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c4fb03, SUC_lexicon_tags_27 },
    { 0x10a4fb04, SUC_lexicon_tags_1 },
    { 0xd524fb05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf624fb07, SUC_lexicon_tags_1 },
    { 0xd804fb07, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2b64fb0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb804fb0d, SUC_lexicon_tags_11 },
    { 0x6d84fb0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c4fb11, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc4fb15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf8c4fb17, SUC_lexicon_tags_25 },
    { 0xb4a4fb17, SUC_lexicon_tags_12 },
    { 0x5fc4fb19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3c4fb1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbc4fb1d, SUC_lexicon_tags_12 },
    { 0x8c24fb1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c24fb23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac4fb27, SUC_lexicon_tags_1 },
    { 0xd444fb28, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbec4fb2a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c4fb33, SUC_lexicon_tags_37 },
    { 0xd9c4fb34, SUC_lexicon_tags_38 },
    { 0x31a4fb35, SUC_lexicon_tags_25 },
    { 0xe1c4fb36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a4fb3c, SUC_lexicon_tags_17 },
    { 0x1f24fb3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc64fb3f, SUC_lexicon_tags_37 },
    { 0x5b84fb40, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac4fb43, SUC_lexicon_tags_39 },
    { 0x1324fb43, SUC_lexicon_tags_160 },
    { 0x3ba4fb45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e84fb47, SUC_lexicon_tags_5 },
    { 0x584fb47, SUC_lexicon_tags_17 },
    { 0x2fc4fb49, SUC_lexicon_tags_14 },
    { 0xbc04fb4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd964fb4c, SUC_lexicon_tags_82 },
    { 0x58c4fb4d, SUC_lexicon_tags_48 },
    { 0x4364fb4e, SUC_lexicon_tags_68 },
    { 0xe364fb4e, SUC_lexicon_tags_15 },
    { 0x7b44fb50, SUC_lexicon_tags_32 },
    { 0x7664fb4c, SUC_lexicon_tags_62 },
    { 0x7e64fb52, SUC_lexicon_tags_31 },
    { 0xe144fb50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c4fb57, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3484fb5b, SUC_lexicon_tags_37 },
    { 0x2604fb5b, SUC_lexicon_tags_3 },
    { 0x4d04fb5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c4fb60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a4fb64, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x9224fb66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e44fb6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a64fb6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf504fb74, SUC_lexicon_tags_6 },
    { 0x2f04fb74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x164fb78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb764fb7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c4fb88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f04fb8e, SUC_lexicon_tags_13 },
    { 0x6624fb8f, SUC_lexicon_tags_37 },
    { 0xcf64fb8f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe704fb98, SUC_lexicon_tags_23 },
    { 0x59c4fb99, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5ca4fb9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc4fba1, SUC_lexicon_tags_13 },
    { 0x1084fba2, SUC_lexicon_tags_8 },
    { 0x7e4fba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef84fba7, SUC_lexicon_tags_7 },
    { 0xaa84fba7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa4fbac, SUC_lexicon_tags_15 },
    { 0xc664fbad, SUC_lexicon_tags_25 },
    { 0x4ba4fbae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4f64fbb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd64fbb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3b04fbb4, SUC_lexicon_tags_37 },
    { 0xb8e4fbb5, SUC_lexicon_tags_28 },
    { 0x6764fbb4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6084fbb8, SUC_lexicon_tags_11 },
    { 0xbee4fbb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x29a4fbbb, SUC_lexicon_tags_49 },
    { 0x5a84fbbb, SUC_lexicon_tags_44 },
    { 0x3844fbbb, SUC_lexicon_tags_47 },
    { 0x6e44fbbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a4fbc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e4fbc8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e4fbce, SUC_lexicon_tags_37 },
    { 0x8a24fbcf, SUC_lexicon_tags_11 },
    { 0xe524fbd0, SUC_lexicon_tags_8 },
    { 0x8b04fbd1, SUC_lexicon_tags_43 },
    { 0x7a84fbd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c4fbd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d44fbda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae04fbdc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa4fbe0, SUC_lexicon_tags_61 },
    { 0x5124fbe0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d44fbf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a4fbf9, SUC_lexicon_tags_7 },
    { 0x8724fbfa, SUC_lexicon_tags_11 },
    { 0x4044fbfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8804fbfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7124fc02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6124fc06, SUC_lexicon_tags_17 },
    { 0x3d84fc06, SUC_lexicon_tags_47 },
    { 0xc084fc08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba4fc0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc24fc11, SUC_lexicon_tags_2 },
    { 0xb3a4fc12, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e4fc17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a4fc1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e4fc21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f44fc23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e04fc27, SUC_lexicon_tags_9 },
    { 0x2f44fc28, SUC_lexicon_tags_37 },
    { 0x34a4fc29, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8184fc2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e44fc31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe84fc33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2224fc37, SUC_lexicon_tags_12 },
    { 0x84c4fc37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3824fc3c, SUC_lexicon_tags_2 },
    { 0x5364fc3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb64fc3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5764fc41, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e4fc44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf404fc46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb644fc49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd84fc4f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c4fc5d, SUC_lexicon_tags_15 },
    { 0xf184fc5d, SUC_lexicon_tags_11 },
    { 0xa6c4fc5f, SUC_lexicon_tags_61 },
    { 0x9704fc5e, SUC_lexicon_tags_11 },
    { 0x4824fc5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a04fc64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a4fc6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf24fc6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94c4fc6f, SUC_lexicon_tags_14 },
    { 0xd884fc6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb064fc72, SUC_lexicon_tags_12 },
    { 0x48e4fc73, SUC_lexicon_tags_25 },
    { 0xd64fc74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c4fc79, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2084fc7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x144fc7d, SUC_lexicon_tags_8 },
    { 0xb9c4fc7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c4fc85, SUC_lexicon_tags_33 },
    { 0x3a24fc86, SUC_lexicon_tags_35 },
    { 0x3264fc85, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e4fc93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0c4fc95, SUC_lexicon_tags_163 },
    { 0x52c4fc96, SUC_lexicon_tags_19 },
    { 0xd784fc96, SUC_lexicon_tags_27 },
    { 0xee84fc96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b04fc9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1544fc9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484fca1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4764fca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1444fca5, SUC_lexicon_tags_25 },
    { 0x24fca6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8484fca8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b64fcaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e4fcad, SUC_lexicon_tags_11 },
    { 0x1224fcae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2544fcb2, SUC_lexicon_tags_37 },
    { 0xac84fcb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc64fcbc, SUC_lexicon_tags_15 },
    { 0xdba4fcbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6504fcc1, SUC_lexicon_tags_33 },
    { 0x5804fcc2, SUC_lexicon_tags_32 },
    { 0x5a44fcc2, SUC_lexicon_tags_6 },
    { 0x1024fcc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe324fcc7, SUC_lexicon_tags_1 },
    { 0x6124fcc8, SUC_lexicon_tags_19 },
    { 0x9784fcc8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb484fccc, SUC_lexicon_tags_37 },
    { 0x9084fccd, SUC_lexicon_tags_15 },
    { 0xe7c4fccd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a04fcd2, SUC_lexicon_tags_1 },
    { 0xb8e4fcd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9a4fcd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7604fcd9, SUC_lexicon_tags_14 },
    { 0x43c4fcd9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x73a4fcdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb64fcde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa944fce1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc4fce7, SUC_lexicon_tags_2 },
    { 0x4f24fce7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e4fcec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5024fcef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb24fcf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7c4fcf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf964fcfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd344fd00, SUC_lexicon_tags_37 },
    { 0xa284fd00, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44fd04, SUC_lexicon_tags_19 },
    { 0x84c4fd05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8884fd07, SUC_lexicon_tags_46 },
    { 0x2184fd08, SUC_lexicon_tags_63 },
    { 0x4f04fd07, SUC_lexicon_tags_30 },
    { 0xf044fd08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca4fd0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9044fd13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc924fd15, SUC_lexicon_tags_79 },
    { 0xd6a4fd15, SUC_lexicon_tags_28 },
    { 0x3b84fd17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c84fd1b, SUC_lexicon_tags_9 },
    { 0xdae4fd1c, SUC_lexicon_tags_7 },
    { 0x1f64fd1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca4fd20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xece4fd22, SUC_lexicon_tags_87 },
    { 0x4c4fd22, SUC_lexicon_tags_4 },
    { 0x91a4fd24, SUC_lexicon_tags_17 },
    { 0xb3c4fd25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d04fd27, SUC_lexicon_tags_56 },
    { 0x8664fd28, SUC_lexicon_tags_12 },
    { 0x2e4fd27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e04fd2d, SUC_lexicon_tags_23 },
    { 0x1104fd2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea4fd32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9484fd38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c4fd3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1144fd40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4fd44, SUC_lexicon_tags_38 },
    { 0xe8c4fd44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb824fd48, SUC_lexicon_tags_542 },
    { 0xc1c4fd49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c4fd52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5384fd55, SUC_lexicon_tags_17 },
    { 0xd8e4fd56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6024fd59, SUC_lexicon_tags_15 },
    { 0x10c4fd5a, SUC_lexicon_tags_12 },
    { 0x22e4fd5b, SUC_lexicon_tags_7 },
    { 0x9344fd5c, SUC_lexicon_tags_32 },
    { 0x4e4fd5a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4dc4fd5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e4fd65, SUC_lexicon_tags_31 },
    { 0xab64fd66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c4fd69, SUC_lexicon_tags_14 },
    { 0xb264fd69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a4fd6f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xda04fd72, SUC_lexicon_tags_37 },
    { 0x7a64fd73, SUC_lexicon_tags_17 },
    { 0x2e84fd74, SUC_lexicon_tags_12 },
    { 0xf524fd75, SUC_lexicon_tags_23 },
    { 0xaea4fd75, SUC_lexicon_tags_16 },
    { 0xdb44fd77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3144fd80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c4fd84, SUC_lexicon_tags_0 },
    { 0xf0a4fd85, SUC_lexicon_tags_11 },
    { 0x5c64fd85, SUC_lexicon_tags_17 },
    { 0xd524fd87, SUC_lexicon_tags_15 },
    { 0x1784fd88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64fd94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e4fd99, SUC_lexicon_tags_10 },
    { 0xacc4fd9a, SUC_lexicon_tags_15 },
    { 0x4684fd9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e84fda2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb04fda5, SUC_lexicon_tags_9 },
    { 0x3d44fda6, SUC_lexicon_tags_12 },
    { 0x8cc4fda7, SUC_lexicon_tags_11 },
    { 0x2b44fda5, SUC_lexicon_tags_25 },
    { 0xd664fda7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba4fdac, SUC_lexicon_tags_17 },
    { 0x1084fdad, SUC_lexicon_tags_6 },
    { 0x5ae4fdae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9784fdb2, SUC_lexicon_tags_37 },
    { 0xf6a4fdb3, SUC_lexicon_tags_14 },
    { 0xa304fdb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924fdb8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe764fdbb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd284fdbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90e4fdc1, SUC_lexicon_tags_11 },
    { 0xbfe4fdc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea4fdca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc4fdce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb44fdd2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9564fdd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4244fdd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe04fddd, SUC_lexicon_tags_103 },
    { 0x7b64fdde, SUC_lexicon_tags_37 },
    { 0x6f64fdde, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e4fde4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c64fde7, SUC_lexicon_tags_12 },
    { 0x49c4fde8, SUC_lexicon_tags_17 },
    { 0x8f24fde8, SUC_lexicon_tags_68 },
    { 0xb4c4fdea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d24fdec, SUC_lexicon_tags_49 },
    { 0xca4fded, SUC_lexicon_tags_39 },
    { 0xd604fdee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964fdf2, SUC_lexicon_tags_45 },
    { 0x5c84fdf3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7624fdf6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3444fdff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa184fe02, SUC_lexicon_tags_7 },
    { 0x73c4fe03, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e4fe08, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2764fe0b, SUC_lexicon_tags_11 },
    { 0x6aa4fe0c, SUC_lexicon_tags_11 },
    { 0xd124fe0b, SUC_lexicon_tags_37 },
    { 0xe824fe0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9184fe12, SUC_lexicon_tags_14 },
    { 0x5444fe13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e84fe19, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3664fe1c, SUC_lexicon_tags_10 },
    { 0x5b44fe1d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e24fe20, SUC_lexicon_tags_2 },
    { 0xdf64fe21, SUC_lexicon_tags_1 },
    { 0xbe44fe22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e44fe24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6544fe28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e24fe2b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5144fe34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe4fe37, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x55e4fe39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4884fe3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe864fe3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd84fe40, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2324fe42, SUC_lexicon_tags_43 },
    { 0xa424fe43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf884fe49, SUC_lexicon_tags_27 },
    { 0xbc84fe4a, SUC_lexicon_tags_16 },
    { 0x2884fe4b, SUC_lexicon_tags_17 },
    { 0x26e4fe4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2064fe51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b64fe53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca4fe58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe4fe63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e4fe67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7fe4fe69, SUC_lexicon_tags_25 },
    { 0xf104fe69, SUC_lexicon_tags_17 },
    { 0x2a04fe69, SUC_lexicon_tags_32 },
    { 0xed04fe6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa364fe6e, SUC_lexicon_tags_49 },
    { 0xf904fe6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e4fe73, SUC_lexicon_tags_0 },
    { 0x9264fe74, SUC_lexicon_tags_4 },
    { 0x3b44fe75, SUC_lexicon_tags_25 },
    { 0xa444fe76, SUC_lexicon_tags_11 },
    { 0x48a4fe75, SUC_lexicon_tags_2 },
    { 0x26c4fe78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf84fe7c, SUC_lexicon_tags_7 },
    { 0x4aa4fe7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xec24fe7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9524fe89, SUC_lexicon_tags_146 },
    { 0x5c04fe8a, SUC_lexicon_tags_0 },
    { 0x2f44fe8b, SUC_lexicon_tags_174 },
    { 0x3664fe8b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba04fe99, SUC_lexicon_tags_4 },
    { 0xdb04fe9a, SUC_lexicon_tags_13 },
    { 0x9fa4fe9b, SUC_lexicon_tags_30 },
    { 0x4d04fe9c, SUC_lexicon_tags_10 },
    { 0x38a4fe9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38e4fe9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3c4fea1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65c4fea3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5864fea5, SUC_lexicon_tags_9 },
    { 0x5d44fea6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc4fea9, SUC_lexicon_tags_7 },
    { 0x9584fea9, SUC_lexicon_tags_2 },
    { 0xd684feaa, SUC_lexicon_tags_60 },
    { 0xa5a4feac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd264feaf, SUC_lexicon_tags_18 },
    { 0x7144feaf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde4feb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb04feb5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc304feb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x37e4feba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc804febf, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x3144fec1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e84fec4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c24feca, SUC_lexicon_tags_77 },
    { 0x3624feca, SUC_lexicon_tags_61 },
    { 0x8d44fecc, SUC_lexicon_tags_15 },
    { 0xf484fecd, SUC_lexicon_tags_1 },
    { 0xfda4fecc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5184fed1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0e4fed3, SUC_lexicon_tags_13 },
    { 0x2764fed4, SUC_lexicon_tags_14 },
    { 0xb604fed5, SUC_lexicon_tags_87 },
    { 0xcd64fed6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfa44fed8, SUC_lexicon_tags_38 },
    { 0xa404fed8, SUC_lexicon_tags_2 },
    { 0xd824feda, SUC_lexicon_tags_21 },
    { 0x4a84feda, SUC_lexicon_tags_6 },
    { 0x1104fedc, SUC_lexicon_tags_62 },
    { 0xb2c4fed9, SUC_lexicon_tags_17 },
    { 0xe6e4fede, SUC_lexicon_tags_138 },
    { 0xaf04fedd, SUC_lexicon_tags_38 },
    { 0x2b84fed9, SUC_lexicon_tags_14 },
    { 0xff24fee1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4d24fee3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e4fee9, SUC_lexicon_tags_11 },
    { 0xd404feea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5284feed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5824fef8, SUC_lexicon_tags_2 },
    { 0x4e24fef9, SUC_lexicon_tags_60 },
    { 0x8364fef9, SUC_lexicon_tags_61 },
    { 0xeca4fef9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1ee4fefd, SUC_lexicon_tags_25 },
    { 0x65c4fefe, SUC_lexicon_tags_10 },
    { 0xa4a4feff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5e44ff01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3884ff05, SUC_lexicon_tags_4 },
    { 0xc6a4ff05, SUC_lexicon_tags_45 },
    { 0x2044ff07, SUC_lexicon_tags_37 },
    { 0x5ea4ff07, SUC_lexicon_tags_60 },
    { 0x9ce4ff08, SUC_lexicon_tags_15 },
    { 0x68e4ff07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f04ff15, SUC_lexicon_tags_31 },
    { 0xd004ff16, SUC_lexicon_tags_63 },
    { 0xec44ff17, SUC_lexicon_tags_37 },
    { 0xb784ff17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c4ff1e, SUC_lexicon_tags_37 },
    { 0x4904ff1f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca4ff23, SUC_lexicon_tags_61 },
    { 0x6dc4ff24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfaa4ff26, SUC_lexicon_tags_1 },
    { 0x9c4ff27, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc404ff2e, SUC_lexicon_tags_1 },
    { 0x58e4ff2f, SUC_lexicon_tags_19 },
    { 0xef44ff2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16a4ff32, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc64ff37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc204ff39, SUC_lexicon_tags_37 },
    { 0xb624ff39, SUC_lexicon_tags_47 },
    { 0xa384ff3a, SUC_lexicon_tags_2 },
    { 0xd964ff3c, SUC_lexicon_tags_12 },
    { 0xff24ff3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5184ff3f, SUC_lexicon_tags_11 },
    { 0x4a84ff40, SUC_lexicon_tags_11 },
    { 0xd4c4ff41, SUC_lexicon_tags_33 },
    { 0x1524ff42, SUC_lexicon_tags_2 },
    { 0x63c4ff43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac4ff47, SUC_lexicon_tags_37 },
    { 0xda64ff48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6444ff4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c4ff52, SUC_lexicon_tags_39 },
    { 0xd9e4ff53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd884ff56, SUC_lexicon_tags_37 },
    { 0x6cc4ff57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324ff5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4524ff5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5104ff63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1564ff65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1744ff68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc4ff6f, SUC_lexicon_tags_49 },
    { 0x29a4ff70, SUC_lexicon_tags_8 },
    { 0x1d04ff6f, SUC_lexicon_tags_17 },
    { 0xe4c4ff72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f04ff76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e4ff7a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2544ff7c, SUC_lexicon_tags_7 },
    { 0x9444ff7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d04ff81, SUC_lexicon_tags_19 },
    { 0x7e4ff82, SUC_lexicon_tags_45 },
    { 0xb804ff82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2964ff88, SUC_lexicon_tags_47 },
    { 0x11a4ff89, SUC_lexicon_tags_17 },
    { 0x5484ff8a, SUC_lexicon_tags_15 },
    { 0x5b44ff8b, SUC_lexicon_tags_1 },
    { 0xd844ff89, SUC_lexicon_tags_14 },
    { 0x3f64ff89, SUC_lexicon_tags_79 },
    { 0xb4e4ff8e, SUC_lexicon_tags_45 },
    { 0xac24ff8b, SUC_lexicon_tags_25 },
    { 0xee64ff90, SUC_lexicon_tags_1 },
    { 0x5fa4ff91, SUC_lexicon_tags_15 },
    { 0x16e4ff91, SUC_lexicon_tags_17 },
    { 0xcf84ff92, SUC_lexicon_tags_15 },
    { 0xc4e4ff94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x80a4ff96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8064ff9a, SUC_lexicon_tags_53 },
    { 0xbd64ff9a, SUC_lexicon_tags_38 },
    { 0x4c04ff9a, SUC_lexicon_tags_6 },
    { 0x2084ff9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04ffa5, SUC_lexicon_tags_25 },
    { 0x90c4ffa6, SUC_lexicon_tags_25 },
    { 0xdc24ffa7, SUC_lexicon_tags_2 },
    { 0x5544ffa8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa704ffaa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xea24ffac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3864ffb0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x444ffb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e04ffb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9744ffb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b64ffbc, SUC_lexicon_tags_11 },
    { 0x5864ffbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7964ffc1, SUC_lexicon_tags_1 },
    { 0x85e4ffc2, SUC_lexicon_tags_77 },
    { 0x8604ffc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2844ffcc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x93e4ffce, SUC_lexicon_tags_17 },
    { 0xd424ffcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2764ffda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44ffe6, SUC_lexicon_tags_15 },
    { 0x5304ffe7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa4ffeb, SUC_lexicon_tags_15 },
    { 0xf5c4ffec, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a4fff1, SUC_lexicon_tags_37 },
    { 0x8c64fff2, SUC_lexicon_tags_16 },
    { 0x7944fff2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7744fff5, SUC_lexicon_tags_2 },
    { 0xd84fff6, SUC_lexicon_tags_34 },
    { 0x1604fff6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x41a4fff9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa724fffb, SUC_lexicon_tags_11 },
    { 0xf104fffc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb450005, SUC_lexicon_tags_75 },
    { 0x27450005, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd065000d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe650014, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42850019, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7285001b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c50020, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a050027, SUC_lexicon_tags_8 },
    { 0x46c50028, SUC_lexicon_tags_21 },
    { 0xed650029, SUC_lexicon_tags_60 },
    { 0x78a5002a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5002e, SUC_lexicon_tags_11 },
    { 0x7c65002f, SUC_lexicon_tags_376 },
    { 0x11250030, SUC_lexicon_tags_12 },
    { 0x34850031, SUC_lexicon_tags_25 },
    { 0x78e50032, SUC_lexicon_tags_2 },
    { 0x84a50030, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c450037, SUC_lexicon_tags_1 },
    { 0x10250037, SUC_lexicon_tags_45 },
    { 0x75e50039, SUC_lexicon_tags_31 },
    { 0xaa050038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffa5003c, SUC_lexicon_tags_64 },
    { 0x16c5003d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c50040, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76850044, SUC_lexicon_tags_9 },
    { 0xe9050045, SUC_lexicon_tags_1 },
    { 0x6a450046, SUC_lexicon_tags_49 },
    { 0x2ec50046, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ba50049, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8a85004b, SUC_lexicon_tags_17 },
    { 0x7865004b, SUC_lexicon_tags_25 },
    { 0xc245004d, SUC_lexicon_tags_21 },
    { 0x6345004e, SUC_lexicon_tags_7 },
    { 0x3985004f, SUC_lexicon_tags_17 },
    { 0x36450050, SUC_lexicon_tags_25 },
    { 0xcd050050, SUC_lexicon_tags_27 },
    { 0x2525004f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa450056, SUC_lexicon_tags_63 },
    { 0x90450057, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69650059, SUC_lexicon_tags_47 },
    { 0x7ca5005a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a50063, SUC_lexicon_tags_19 },
    { 0x87e50064, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a50067, SUC_lexicon_tags_11 },
    { 0xbe850068, SUC_lexicon_tags_19 },
    { 0x17e50069, SUC_lexicon_tags_25 },
    { 0xc8a50068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26c5006c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc225006e, SUC_lexicon_tags_17 },
    { 0x2785006e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4dc50071, SUC_lexicon_tags_27 },
    { 0xf850071, SUC_lexicon_tags_60 },
    { 0x2d650073, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c50076, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa945007f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21450081, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac50085, SUC_lexicon_tags_31 },
    { 0x9f250086, SUC_lexicon_tags_55 },
    { 0x38250087, SUC_lexicon_tags_11 },
    { 0x62250086, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd685008a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e5008e, SUC_lexicon_tags_61 },
    { 0xe05008e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf650091, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac50098, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a5009b, SUC_lexicon_tags_53 },
    { 0xaee5009c, SUC_lexicon_tags_33 },
    { 0xd8c5009c, SUC_lexicon_tags_55 },
    { 0x9aa5009e, SUC_lexicon_tags_80 },
    { 0x5b45009f, SUC_lexicon_tags_0 },
    { 0xc7a5009d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x414500a3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xffa500a5, SUC_lexicon_tags_193 },
    { 0x33e500a6, SUC_lexicon_tags_49 },
    { 0x6ee500a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7a2500a9, SUC_lexicon_tags_34 },
    { 0x432500aa, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x1da500ac, SUC_lexicon_tags_7 },
    { 0xd96500ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66500b0, SUC_lexicon_tags_15 },
    { 0x422500b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb2500b3, SUC_lexicon_tags_47 },
    { 0x1f6500b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2500be, SUC_lexicon_tags_2 },
    { 0xcee500be, SUC_lexicon_tags_51 },
    { 0xbfc500bf, SUC_lexicon_tags_15 },
    { 0xce500bf, SUC_lexicon_tags_17 },
    { 0xc0a500c1, SUC_lexicon_tags_8 },
    { 0x43a500c3, SUC_lexicon_tags_37 },
    { 0x5c500c4, SUC_lexicon_tags_11 },
    { 0xaaa500c5, SUC_lexicon_tags_151 },
    { 0x640500c5, SUC_lexicon_tags_47 },
    { 0x424500c7, SUC_lexicon_tags_12 },
    { 0x136500c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2500cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e500ce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a500d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe500d7, SUC_lexicon_tags_17 },
    { 0x6a0500d8, SUC_lexicon_tags_55 },
    { 0x402500d8, SUC_lexicon_tags_9 },
    { 0xa78500da, SUC_lexicon_tags_45 },
    { 0xf2c500db, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6500df, SUC_lexicon_tags_160 },
    { 0x3ce500e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeec500e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc6500e4, SUC_lexicon_tags_38 },
    { 0xa16500e5, SUC_lexicon_tags_11 },
    { 0x416500e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ba500e8, SUC_lexicon_tags_1 },
    { 0x1a2500e9, SUC_lexicon_tags_7 },
    { 0x68500ea, SUC_lexicon_tags_17 },
    { 0x3aa500eb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90500ee, SUC_lexicon_tags_19 },
    { 0x7a500ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670500f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba500f9, SUC_lexicon_tags_0 },
    { 0xad8500f9, SUC_lexicon_tags_12 },
    { 0x6b6500fb, SUC_lexicon_tags_117 },
    { 0x748500fc, SUC_lexicon_tags_2 },
    { 0x550500fd, SUC_lexicon_tags_47 },
    { 0x88e500fd, SUC_lexicon_tags_0 },
    { 0x2a500ff, SUC_lexicon_tags_2 },
    { 0xc70500ff, SUC_lexicon_tags_23 },
    { 0x30050100, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc50105, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb6c50107, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c50109, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ba5010b, SUC_lexicon_tags_25 },
    { 0x1345010c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c50110, SUC_lexicon_tags_9 },
    { 0x43c50111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6250115, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d650117, SUC_lexicon_tags_17 },
    { 0x4b450118, SUC_lexicon_tags_11 },
    { 0x4a850117, SUC_lexicon_tags_47 },
    { 0x9265011a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ba5011c, SUC_lexicon_tags_27 },
    { 0x6785011c, SUC_lexicon_tags_11 },
    { 0x96e5011e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a050124, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e50128, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d85012b, SUC_lexicon_tags_7 },
    { 0x9de5012b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88650131, SUC_lexicon_tags_9 },
    { 0x30250132, SUC_lexicon_tags_6 },
    { 0xd7c50132, SUC_lexicon_tags_17 },
    { 0x6850134, SUC_lexicon_tags_32 },
    { 0x70c50132, SUC_lexicon_tags_47 },
    { 0xa050131, SUC_lexicon_tags_34 },
    { 0x20450137, SUC_lexicon_tags_65 },
    { 0xc050137, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa905013f, SUC_lexicon_tags_21 },
    { 0x79450140, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a50145, SUC_lexicon_tags_17 },
    { 0x28650146, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe0450148, SUC_lexicon_tags_7 },
    { 0xb2050148, SUC_lexicon_tags_25 },
    { 0xd3650149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab45014c, SUC_lexicon_tags_60 },
    { 0x7925014c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8685014f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4250151, SUC_lexicon_tags_37 },
    { 0xaea50152, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x80c50154, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e50158, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa145015a, SUC_lexicon_tags_9 },
    { 0xc225015b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac50160, SUC_lexicon_tags_2 },
    { 0xa5850160, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41c50163, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e450166, SUC_lexicon_tags_49 },
    { 0x1050166, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee5016a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8305016c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x8d25016e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae50171, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc50176, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe050178, SUC_lexicon_tags_30 },
    { 0x51650179, SUC_lexicon_tags_2 },
    { 0x58250178, SUC_lexicon_tags_23 },
    { 0xfda5017b, SUC_lexicon_tags_43 },
    { 0x7905017c, SUC_lexicon_tags_65 },
    { 0xbcc5017d, SUC_lexicon_tags_9 },
    { 0xf1a50178, SUC_lexicon_tags_17 },
    { 0xbda5017f, SUC_lexicon_tags_47 },
    { 0x88650179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e650184, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e50187, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa450189, SUC_lexicon_tags_37 },
    { 0xd145018a, SUC_lexicon_tags_66 },
    { 0x2745018b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125018f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf9c50191, SUC_lexicon_tags_29 },
    { 0x46050191, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c050197, SUC_lexicon_tags_17 },
    { 0x8f050197, SUC_lexicon_tags_47 },
    { 0x10a50199, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6785019d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112501a2, SUC_lexicon_tags_16 },
    { 0x46c501a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d4501a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e501a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be501aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2501ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5e501af, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xf58501b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00501b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a501b7, SUC_lexicon_tags_19 },
    { 0xcfc501b8, SUC_lexicon_tags_54 },
    { 0x7f8501b7, SUC_lexicon_tags_15 },
    { 0x2b8501ba, SUC_lexicon_tags_23 },
    { 0x1ea501ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e501c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe18501c3, SUC_lexicon_tags_30 },
    { 0x1a6501c3, SUC_lexicon_tags_25 },
    { 0x866501c5, SUC_lexicon_tags_11 },
    { 0xf0e501c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e501c9, SUC_lexicon_tags_12 },
    { 0xedc501ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x330501cd, SUC_lexicon_tags_3 },
    { 0x23a501ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2501d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa501d6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xa96501d8, SUC_lexicon_tags_63 },
    { 0xc18501d9, SUC_lexicon_tags_15 },
    { 0x744501d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4501dd, SUC_lexicon_tags_11 },
    { 0x760501dd, SUC_lexicon_tags_35 },
    { 0x67a501df, SUC_lexicon_tags_49 },
    { 0x7a501df, SUC_lexicon_tags_10 },
    { 0x5e0501df, SUC_lexicon_tags_17 },
    { 0x6b6501e2, SUC_lexicon_tags_6 },
    { 0x66a501e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec501e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c501f3, SUC_lexicon_tags_10 },
    { 0x85a501f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b4501f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e501f9, SUC_lexicon_tags_1 },
    { 0xe30501fa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0501ff, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8650204, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36650208, SUC_lexicon_tags_66 },
    { 0x89850209, SUC_lexicon_tags_14 },
    { 0xc5a50209, SUC_lexicon_tags_63 },
    { 0x7f050209, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2fe5020d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae50214, SUC_lexicon_tags_37 },
    { 0x66a50214, SUC_lexicon_tags_11 },
    { 0xeb050216, SUC_lexicon_tags_11 },
    { 0x54450215, SUC_lexicon_tags_11 },
    { 0x84450218, SUC_lexicon_tags_1 },
    { 0x12450216, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c5021d, SUC_lexicon_tags_46 },
    { 0x3c45021e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5450221, SUC_lexicon_tags_77 },
    { 0xa0450222, SUC_lexicon_tags_1 },
    { 0x9fa50222, SUC_lexicon_tags_4 },
    { 0xc7450224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1250229, SUC_lexicon_tags_14 },
    { 0xaf850229, SUC_lexicon_tags_30 },
    { 0x9f650229, SUC_lexicon_tags_15 },
    { 0x7bc5022c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b850230, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89650232, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee50235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb705023a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce65023c, SUC_lexicon_tags_31 },
    { 0x7d25023d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac250242, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xedc50244, SUC_lexicon_tags_79 },
    { 0x4de50245, SUC_lexicon_tags_3 },
    { 0x22a50245, SUC_lexicon_tags_28 },
    { 0x88050247, SUC_lexicon_tags_17 },
    { 0xbb450244, SUC_lexicon_tags_1541 },
    { 0x18e50247, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede5024e, SUC_lexicon_tags_9 },
    { 0x3345024f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc50252, SUC_lexicon_tags_55 },
    { 0xb4a50252, SUC_lexicon_tags_21 },
    { 0x70a50254, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76050258, SUC_lexicon_tags_0 },
    { 0xbc650258, SUC_lexicon_tags_55 },
    { 0x96850258, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde650261, SUC_lexicon_tags_25 },
    { 0x13e50262, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc5026b, SUC_lexicon_tags_2 },
    { 0xdda5026c, SUC_lexicon_tags_27 },
    { 0x39c5026d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd765027c, SUC_lexicon_tags_55 },
    { 0xd505027d, SUC_lexicon_tags_2 },
    { 0x9de5027d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3250284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3850286, SUC_lexicon_tags_9 },
    { 0xfc250287, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6805028b, SUC_lexicon_tags_2 },
    { 0xaa45028c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fa5028e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x33c50290, SUC_lexicon_tags_17 },
    { 0x9d250290, SUC_lexicon_tags_17 },
    { 0x66a50292, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x12850294, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7050299, SUC_lexicon_tags_0 },
    { 0xb0c5029a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565029f, SUC_lexicon_tags_26 },
    { 0xd82502a0, SUC_lexicon_tags_37 },
    { 0x4dc502a1, SUC_lexicon_tags_14 },
    { 0x778502a2, SUC_lexicon_tags_25 },
    { 0x522502a3, SUC_lexicon_tags_151 },
    { 0x7dc502a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632502a8, SUC_lexicon_tags_19 },
    { 0x672502a9, SUC_lexicon_tags_25 },
    { 0xa40502a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe26502ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72502b6, SUC_lexicon_tags_15 },
    { 0x378502b7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36502bc, SUC_lexicon_tags_37 },
    { 0x488502bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942502c1, SUC_lexicon_tags_7 },
    { 0x9c0502c2, SUC_lexicon_tags_47 },
    { 0xebc502c1, SUC_lexicon_tags_19 },
    { 0x8cc502c4, SUC_lexicon_tags_37 },
    { 0xca502c5, SUC_lexicon_tags_37 },
    { 0x632502c6, SUC_lexicon_tags_17 },
    { 0x666502c2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c502cb, SUC_lexicon_tags_12 },
    { 0xba0502cc, SUC_lexicon_tags_3 },
    { 0x132502cd, SUC_lexicon_tags_11 },
    { 0xce502ce, SUC_lexicon_tags_43 },
    { 0x50e502cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ca502d1, SUC_lexicon_tags_56 },
    { 0x458502d2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2502da, SUC_lexicon_tags_29 },
    { 0x290502db, SUC_lexicon_tags_4 },
    { 0x58502db, SUC_lexicon_tags_28 },
    { 0x6b8502db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8c502df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c502e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da502e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46e502eb, SUC_lexicon_tags_23 },
    { 0xafc502ec, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca502ef, SUC_lexicon_tags_14 },
    { 0x49a502f0, SUC_lexicon_tags_4 },
    { 0xf02502f1, SUC_lexicon_tags_7 },
    { 0x1f0502f2, SUC_lexicon_tags_63 },
    { 0x248502f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4a502f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668502f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46650301, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a50306, SUC_lexicon_tags_28 },
    { 0xedc50307, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa050309, SUC_lexicon_tags_45 },
    { 0x96250309, SUC_lexicon_tags_7 },
    { 0x9925030b, SUC_lexicon_tags_28 },
    { 0x32e5030b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0250311, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x59e50313, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc50317, SUC_lexicon_tags_27 },
    { 0x2aa50317, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a5031f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c50322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc50325, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5050329, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c5032c, SUC_lexicon_tags_37 },
    { 0x1425032d, SUC_lexicon_tags_17 },
    { 0x8aa5032c, SUC_lexicon_tags_14 },
    { 0x2ec5032f, SUC_lexicon_tags_15 },
    { 0x62a5032f, SUC_lexicon_tags_12 },
    { 0x50a50331, SUC_lexicon_tags_2 },
    { 0x97a5032e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a450339, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdcc5033b, SUC_lexicon_tags_12 },
    { 0x13c5033c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b65033e, SUC_lexicon_tags_38 },
    { 0x7a05033e, SUC_lexicon_tags_30 },
    { 0x1c250340, SUC_lexicon_tags_15 },
    { 0x1405033f, SUC_lexicon_tags_37 },
    { 0xe8a50340, SUC_lexicon_tags_10 },
    { 0xa3a50342, SUC_lexicon_tags_12 },
    { 0xff650341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0a50346, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21250349, SUC_lexicon_tags_27 },
    { 0xe9c5034a, SUC_lexicon_tags_23 },
    { 0xd625034a, SUC_lexicon_tags_15 },
    { 0xe9e5034c, SUC_lexicon_tags_27 },
    { 0x77c5034d, SUC_lexicon_tags_28 },
    { 0x6045034e, SUC_lexicon_tags_1 },
    { 0x23e5034e, SUC_lexicon_tags_25 },
    { 0x35a50350, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a50356, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d050359, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5035d, SUC_lexicon_tags_38 },
    { 0x105035e, SUC_lexicon_tags_8 },
    { 0xa765035f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3e50361, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98450364, SUC_lexicon_tags_48 },
    { 0x75c50365, SUC_lexicon_tags_17 },
    { 0x42a50365, SUC_lexicon_tags_2 },
    { 0x22850366, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a5036b, SUC_lexicon_tags_25 },
    { 0xd505036c, SUC_lexicon_tags_49 },
    { 0x2a65036b, SUC_lexicon_tags_15 },
    { 0x45a5036e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cc50370, SUC_lexicon_tags_108 },
    { 0xd1c50371, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81a50373, SUC_lexicon_tags_14 },
    { 0xcb050374, SUC_lexicon_tags_40 },
    { 0xd1050375, SUC_lexicon_tags_57 },
    { 0x18850373, SUC_lexicon_tags_47 },
    { 0x4f850377, SUC_lexicon_tags_23 },
    { 0x34c50373, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e45037b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd405037d, SUC_lexicon_tags_2 },
    { 0x2b25037e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x58450381, SUC_lexicon_tags_635 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36250385, SUC_lexicon_tags_33 },
    { 0xe9250386, SUC_lexicon_tags_33 },
    { 0x34650387, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9285038a, SUC_lexicon_tags_15 },
    { 0xb825038b, SUC_lexicon_tags_45 },
    { 0xfc85038c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd505038e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad650392, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2650398, SUC_lexicon_tags_37 },
    { 0xfb650399, SUC_lexicon_tags_0 },
    { 0x1be5039a, SUC_lexicon_tags_43 },
    { 0xb565039b, SUC_lexicon_tags_66 },
    { 0x5585039b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed45039e, SUC_lexicon_tags_1 },
    { 0xa445039f, SUC_lexicon_tags_17 },
    { 0x4f85039e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4503ab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a503ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x62e503b0, SUC_lexicon_tags_71 },
    { 0xeea503b1, SUC_lexicon_tags_8 },
    { 0x87c503b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c503b6, SUC_lexicon_tags_47 },
    { 0x550503b7, SUC_lexicon_tags_16 },
    { 0x962503b8, SUC_lexicon_tags_15 },
    { 0x18c503b7, SUC_lexicon_tags_2 },
    { 0x866503b9, SUC_lexicon_tags_55 },
    { 0xdd2503b7, SUC_lexicon_tags_48 },
    { 0x140503b9, SUC_lexicon_tags_5 },
    { 0x3f2503bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c6503bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62503c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78503c9, SUC_lexicon_tags_14 },
    { 0x28a503ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8503cd, SUC_lexicon_tags_66 },
    { 0x462503cd, SUC_lexicon_tags_27 },
    { 0x9fe503cf, SUC_lexicon_tags_16 },
    { 0xe7e503d0, SUC_lexicon_tags_37 },
    { 0xfe0503d0, SUC_lexicon_tags_47 },
    { 0x3f6503d1, SUC_lexicon_tags_1 },
    { 0xe98503d3, SUC_lexicon_tags_8 },
    { 0xf66503d3, SUC_lexicon_tags_7 },
    { 0xd78503d5, SUC_lexicon_tags_46 },
    { 0xb82503d0, SUC_lexicon_tags_43 },
    { 0xb4e503d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11c503d9, SUC_lexicon_tags_55 },
    { 0x6c0503da, SUC_lexicon_tags_12 },
    { 0x754503da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc503e0, SUC_lexicon_tags_4 },
    { 0xa52503e0, SUC_lexicon_tags_7 },
    { 0x18503e2, SUC_lexicon_tags_17 },
    { 0x73e503e3, SUC_lexicon_tags_34 },
    { 0x7b8503e4, SUC_lexicon_tags_37 },
    { 0xf6503e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418503ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e503f5, SUC_lexicon_tags_16 },
    { 0xf1a503f6, SUC_lexicon_tags_2 },
    { 0x99c503f7, SUC_lexicon_tags_62 },
    { 0x19c503f7, SUC_lexicon_tags_1 },
    { 0xd54503f6, SUC_lexicon_tags_7 },
    { 0xe1a503fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21450401, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93450403, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a50408, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1825040b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaaa5040d, SUC_lexicon_tags_6 },
    { 0x5e5040d, SUC_lexicon_tags_106 },
    { 0x6045040f, SUC_lexicon_tags_17 },
    { 0x3f650410, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x13c50412, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37050418, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe45041c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1325041f, SUC_lexicon_tags_29 },
    { 0x13450420, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1250426, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x78250428, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e5042b, SUC_lexicon_tags_37 },
    { 0x765042b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a5042f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e50436, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29250438, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a05043d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c50442, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae5044c, SUC_lexicon_tags_17 },
    { 0x9e05044d, SUC_lexicon_tags_19 },
    { 0x5f85044e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc50451, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x83650453, SUC_lexicon_tags_81 },
    { 0x93850453, SUC_lexicon_tags_17 },
    { 0x90c50454, SUC_lexicon_tags_25 },
    { 0x62250456, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd850458, SUC_lexicon_tags_27 },
    { 0x64850458, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b45045b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85045d, SUC_lexicon_tags_63 },
    { 0x9725045d, SUC_lexicon_tags_37 },
    { 0x1c5045f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe450462, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23250466, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x12450469, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c85046f, SUC_lexicon_tags_17 },
    { 0xf6c5046f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x60850472, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee50477, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac5047c, SUC_lexicon_tags_0 },
    { 0xbae5047d, SUC_lexicon_tags_43 },
    { 0x7e85047e, SUC_lexicon_tags_33 },
    { 0x3e85047f, SUC_lexicon_tags_2 },
    { 0x8f65047f, SUC_lexicon_tags_61 },
    { 0x7565047d, SUC_lexicon_tags_6 },
    { 0xd0450482, SUC_lexicon_tags_12 },
    { 0x18650483, SUC_lexicon_tags_15 },
    { 0x2fe50484, SUC_lexicon_tags_17 },
    { 0x31050485, SUC_lexicon_tags_47 },
    { 0x72a50486, SUC_lexicon_tags_1 },
    { 0x89e50485, SUC_lexicon_tags_35 },
    { 0x17450488, SUC_lexicon_tags_66 },
    { 0xb7850482, SUC_lexicon_tags_11 },
    { 0x3805048a, SUC_lexicon_tags_1 },
    { 0xa025047e, SUC_lexicon_tags_12 },
    { 0x3ba5047f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a50490, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f650498, SUC_lexicon_tags_17 },
    { 0xf9650499, SUC_lexicon_tags_38 },
    { 0x9a050499, SUC_lexicon_tags_7 },
    { 0x6825049b, SUC_lexicon_tags_66 },
    { 0xd705049b, SUC_lexicon_tags_6 },
    { 0xdb850498, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc504b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716504b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64504b8, SUC_lexicon_tags_7 },
    { 0xd64504b9, SUC_lexicon_tags_43 },
    { 0x584504ba, SUC_lexicon_tags_32 },
    { 0x53e504bb, SUC_lexicon_tags_19 },
    { 0x17c504bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c504bf, SUC_lexicon_tags_17 },
    { 0x1d0504c0, SUC_lexicon_tags_7 },
    { 0x290504c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728504d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e504da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb60504dc, SUC_lexicon_tags_63 },
    { 0x786504dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x976504df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x858504e2, SUC_lexicon_tags_11 },
    { 0xf0504e2, SUC_lexicon_tags_7 },
    { 0x68c504e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d8504e6, SUC_lexicon_tags_11 },
    { 0x532504e7, SUC_lexicon_tags_11 },
    { 0xfe0504e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa02504ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4504ee, SUC_lexicon_tags_56 },
    { 0x2a8504ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x27c504f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x604504f4, SUC_lexicon_tags_17 },
    { 0x32e504f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee504f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x614504fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd8504fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34650501, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1450504, SUC_lexicon_tags_29 },
    { 0x6bc50504, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c50509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5805050b, SUC_lexicon_tags_31 },
    { 0x11e5050c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce250512, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb650514, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c50519, SUC_lexicon_tags_45 },
    { 0xb145051a, SUC_lexicon_tags_21 },
    { 0x2945051b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9b25051d, SUC_lexicon_tags_23 },
    { 0x6c65051e, SUC_lexicon_tags_66 },
    { 0x19e5051e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2850529, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf945052b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb865052d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e85052f, SUC_lexicon_tags_16 },
    { 0xef65052f, SUC_lexicon_tags_17 },
    { 0x4b850531, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a50535, SUC_lexicon_tags_7 },
    { 0x20850535, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a5053b, SUC_lexicon_tags_11 },
    { 0x92a5053c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85053e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c850540, SUC_lexicon_tags_26 },
    { 0x7c450540, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2850544, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba50546, SUC_lexicon_tags_32 },
    { 0xa1e50547, SUC_lexicon_tags_46 },
    { 0xb0a50548, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e5054c, SUC_lexicon_tags_7 },
    { 0x3945054c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e50551, SUC_lexicon_tags_44 },
    { 0x24850551, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e5055e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9850563, SUC_lexicon_tags_12 },
    { 0x74650563, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3250567, SUC_lexicon_tags_11 },
    { 0xaf050568, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5845056b, SUC_lexicon_tags_37 },
    { 0xf1c5056c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6e5056e, SUC_lexicon_tags_7 },
    { 0x73c5056f, SUC_lexicon_tags_33 },
    { 0x51a5056f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f450575, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd645057d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d450580, SUC_lexicon_tags_37 },
    { 0x4d250581, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64c50583, SUC_lexicon_tags_6 },
    { 0x85450584, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82250587, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x765058a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78850592, SUC_lexicon_tags_4 },
    { 0x42050593, SUC_lexicon_tags_46 },
    { 0xf9250593, SUC_lexicon_tags_23 },
    { 0x55450592, SUC_lexicon_tags_33 },
    { 0xb8050592, SUC_lexicon_tags_23 },
    { 0x56c50597, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5650599, SUC_lexicon_tags_47 },
    { 0x8be5059a, SUC_lexicon_tags_31 },
    { 0xd25059a, SUC_lexicon_tags_21 },
    { 0xbb05059c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70505a0, SUC_lexicon_tags_25 },
    { 0x112505a1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0505a6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9b4505a8, SUC_lexicon_tags_17 },
    { 0xee6505a9, SUC_lexicon_tags_27 },
    { 0xcc2505a8, SUC_lexicon_tags_1 },
    { 0x8c6505a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc505ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92505b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd86505b4, SUC_lexicon_tags_0 },
    { 0xe26505b5, SUC_lexicon_tags_56 },
    { 0x79c505b6, SUC_lexicon_tags_0 },
    { 0xe16505b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca505c2, SUC_lexicon_tags_37 },
    { 0xc12505c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x61c505c5, SUC_lexicon_tags_45 },
    { 0xd6a505c6, SUC_lexicon_tags_37 },
    { 0xb74505c7, SUC_lexicon_tags_7 },
    { 0xefa505c8, SUC_lexicon_tags_25 },
    { 0x1ac505c7, SUC_lexicon_tags_37 },
    { 0x980505c8, SUC_lexicon_tags_29 },
    { 0xfaa505c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c505d7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e505da, SUC_lexicon_tags_3 },
    { 0x24e505db, SUC_lexicon_tags_2 },
    { 0x40505da, SUC_lexicon_tags_6 },
    { 0xe7e505da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf2505df, SUC_lexicon_tags_14 },
    { 0x5c0505e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c505e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12505ed, SUC_lexicon_tags_31 },
    { 0x452505ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee505f2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x452505f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410505f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c450601, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29450603, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3450606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9185060b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d05060f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1de50611, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd205061a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee5061d, SUC_lexicon_tags_37 },
    { 0x9ac5061e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55050620, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x24e50622, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83e50624, SUC_lexicon_tags_63 },
    { 0xe2450625, SUC_lexicon_tags_18 },
    { 0x63250624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5450628, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c5062d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a50632, SUC_lexicon_tags_7 },
    { 0xdae50633, SUC_lexicon_tags_7 },
    { 0x450634, SUC_lexicon_tags_1 },
    { 0x77250632, SUC_lexicon_tags_21 },
    { 0x90a50633, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d050639, SUC_lexicon_tags_46 },
    { 0x70c5063a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bc5063c, SUC_lexicon_tags_66 },
    { 0x11a5063c, SUC_lexicon_tags_25 },
    { 0x9f85063e, SUC_lexicon_tags_11 },
    { 0x4065063f, SUC_lexicon_tags_25 },
    { 0x650640, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca50643, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7fa50645, SUC_lexicon_tags_55 },
    { 0x58850646, SUC_lexicon_tags_2 },
    { 0x2fc50646, SUC_lexicon_tags_0 },
    { 0xd8050648, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9da5064a, SUC_lexicon_tags_43 },
    { 0x3fe5064b, SUC_lexicon_tags_43 },
    { 0x9665064b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a850654, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e50659, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d25065b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef250660, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc50665, SUC_lexicon_tags_30 },
    { 0x4ce50665, SUC_lexicon_tags_68 },
    { 0x11450665, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9225066a, SUC_lexicon_tags_80 },
    { 0x3a25066b, SUC_lexicon_tags_15 },
    { 0x9125066c, SUC_lexicon_tags_9 },
    { 0xd725066d, SUC_lexicon_tags_32 },
    { 0x8125066b, SUC_lexicon_tags_12 },
    { 0x6fc5066b, SUC_lexicon_tags_38 },
    { 0xb425066c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52650676, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22650678, SUC_lexicon_tags_7 },
    { 0xc2c50679, SUC_lexicon_tags_7 },
    { 0xd7450679, SUC_lexicon_tags_23 },
    { 0x5105067b, SUC_lexicon_tags_49 },
    { 0x2fc5067a, SUC_lexicon_tags_7 },
    { 0xcc45067b, SUC_lexicon_tags_15 },
    { 0x5e25067c, SUC_lexicon_tags_11 },
    { 0x8bc50678, SUC_lexicon_tags_31 },
    { 0x6d25067a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75250686, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1050688, SUC_lexicon_tags_7 },
    { 0x55050689, SUC_lexicon_tags_15 },
    { 0x2c650688, SUC_lexicon_tags_4 },
    { 0x4205068a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d450692, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e50697, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd905069a, SUC_lexicon_tags_18 },
    { 0xbe05069a, SUC_lexicon_tags_2 },
    { 0xc065069c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbfc5069e, SUC_lexicon_tags_77 },
    { 0x4105069f, SUC_lexicon_tags_15 },
    { 0x7505069f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d8506a2, SUC_lexicon_tags_63 },
    { 0xdae506a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ba506a5, SUC_lexicon_tags_1 },
    { 0x8ca506a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x370506a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe6506aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8506ae, SUC_lexicon_tags_11 },
    { 0xc8e506af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcc2506b1, SUC_lexicon_tags_37 },
    { 0xf2506b2, SUC_lexicon_tags_48 },
    { 0x25c506b2, SUC_lexicon_tags_15 },
    { 0xa5c506b4, SUC_lexicon_tags_45 },
    { 0x344506b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de506bc, SUC_lexicon_tags_8 },
    { 0xc1e506bd, SUC_lexicon_tags_55 },
    { 0x804506be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6506c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x434506c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x83a506c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ec506c8, SUC_lexicon_tags_16 },
    { 0x8506c9, SUC_lexicon_tags_19 },
    { 0xde6506c8, SUC_lexicon_tags_27 },
    { 0xe0c506cb, SUC_lexicon_tags_11 },
    { 0xf2e506cc, SUC_lexicon_tags_11 },
    { 0xebc506cd, SUC_lexicon_tags_23 },
    { 0x52c506cc, SUC_lexicon_tags_30 },
    { 0xb30506cd, SUC_lexicon_tags_15 },
    { 0x624506d0, SUC_lexicon_tags_9 },
    { 0x94e506cd, SUC_lexicon_tags_43 },
    { 0x848506d0, SUC_lexicon_tags_27 },
    { 0x4e506ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde506d6, SUC_lexicon_tags_43 },
    { 0xb7e506d7, SUC_lexicon_tags_43 },
    { 0x92506d8, SUC_lexicon_tags_38 },
    { 0xa7e506d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46506dc, SUC_lexicon_tags_43 },
    { 0xc0c506dd, SUC_lexicon_tags_25 },
    { 0x2b8506de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc20506e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e506e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4506e7, SUC_lexicon_tags_19 },
    { 0x8c8506e7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450506ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de506f1, SUC_lexicon_tags_49 },
    { 0xcf6506f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220506fa, SUC_lexicon_tags_12 },
    { 0x49a506fb, SUC_lexicon_tags_17 },
    { 0x7c506fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa84506fe, SUC_lexicon_tags_33 },
    { 0x612506fe, SUC_lexicon_tags_57 },
    { 0x17650700, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a50706, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7650708, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c5070c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24a5070f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b250713, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd2e50715, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x31050718, SUC_lexicon_tags_29 },
    { 0xd6650719, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xecc5071b, SUC_lexicon_tags_29 },
    { 0xab45071c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba50724, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xd7450726, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50850729, SUC_lexicon_tags_27 },
    { 0x125072a, SUC_lexicon_tags_31 },
    { 0x3d65072b, SUC_lexicon_tags_15 },
    { 0x4745072c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x78c5072e, SUC_lexicon_tags_14 },
    { 0x7805072f, SUC_lexicon_tags_25 },
    { 0xc1a50730, SUC_lexicon_tags_51 },
    { 0x63650731, SUC_lexicon_tags_7 },
    { 0x8dc50731, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ec50734, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4da50736, SUC_lexicon_tags_87 },
    { 0xade50737, SUC_lexicon_tags_17 },
    { 0x99250738, SUC_lexicon_tags_6 },
    { 0xca50737, SUC_lexicon_tags_27 },
    { 0x56c50739, SUC_lexicon_tags_25 },
    { 0xeda5073b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe50740, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a50744, SUC_lexicon_tags_37 },
    { 0x79250745, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfae50747, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xd3850749, SUC_lexicon_tags_0 },
    { 0x11a5074a, SUC_lexicon_tags_35 },
    { 0xb7e5074b, SUC_lexicon_tags_49 },
    { 0xe0a5074c, SUC_lexicon_tags_17 },
    { 0xa5a5074d, SUC_lexicon_tags_38 },
    { 0x1385074b, SUC_lexicon_tags_44 },
    { 0x1fc5074d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4450751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55050753, SUC_lexicon_tags_27 },
    { 0x1650753, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba50757, SUC_lexicon_tags_15 },
    { 0xac050758, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f850762, SUC_lexicon_tags_1 },
    { 0x5d250762, SUC_lexicon_tags_71 },
    { 0x6dc50764, SUC_lexicon_tags_49 },
    { 0xcfc50764, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45050769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac5076b, SUC_lexicon_tags_6 },
    { 0xcc45076c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c50770, SUC_lexicon_tags_2 },
    { 0x7c250770, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xab850774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb050778, SUC_lexicon_tags_47 },
    { 0x55e50779, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8145077b, SUC_lexicon_tags_85 },
    { 0xeea5077c, SUC_lexicon_tags_27 },
    { 0xa825077d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff05077f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60250787, SUC_lexicon_tags_11 },
    { 0x77450788, SUC_lexicon_tags_11 },
    { 0xe1c50787, SUC_lexicon_tags_103 },
    { 0x93e50788, SUC_lexicon_tags_12 },
    { 0x62850788, SUC_lexicon_tags_66 },
    { 0xef850788, SUC_lexicon_tags_17 },
    { 0xf385078d, SUC_lexicon_tags_12 },
    { 0x3245078d, SUC_lexicon_tags_1540 },
    { 0x89c5078f, SUC_lexicon_tags_15 },
    { 0x4fc50790, SUC_lexicon_tags_12 },
    { 0x6ac50791, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3050794, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a50798, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa5079c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e507a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2507a6, SUC_lexicon_tags_2 },
    { 0xb84507a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140507ab, SUC_lexicon_tags_2 },
    { 0xa6a507ab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x87c507ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4507b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4507b5, SUC_lexicon_tags_1 },
    { 0x6ea507b6, SUC_lexicon_tags_17 },
    { 0x30e507b6, SUC_lexicon_tags_17 },
    { 0x156507b5, SUC_lexicon_tags_23 },
    { 0x3fe507b7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa98507bb, SUC_lexicon_tags_33 },
    { 0x296507bc, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x676507be, SUC_lexicon_tags_15 },
    { 0xd9e507bf, SUC_lexicon_tags_21 },
    { 0x568507c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x230507c2, SUC_lexicon_tags_7 },
    { 0x340507c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c4507c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x244507c7, SUC_lexicon_tags_30 },
    { 0x6c6507c8, SUC_lexicon_tags_25 },
    { 0xd72507c7, SUC_lexicon_tags_12 },
    { 0x78c507c9, SUC_lexicon_tags_17 },
    { 0x2a0507cb, SUC_lexicon_tags_38 },
    { 0x9e0507cc, SUC_lexicon_tags_45 },
    { 0x52c507cd, SUC_lexicon_tags_27 },
    { 0xa08507ce, SUC_lexicon_tags_31 },
    { 0x7cc507cd, SUC_lexicon_tags_25 },
    { 0x1b4507d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdde507d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4507d7, SUC_lexicon_tags_13 },
    { 0xa6507d8, SUC_lexicon_tags_2 },
    { 0x30a507d8, SUC_lexicon_tags_38 },
    { 0xa56507da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96507df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x356507e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x35c507e3, SUC_lexicon_tags_27 },
    { 0x3a6507e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e507e9, SUC_lexicon_tags_37 },
    { 0xefa507ea, SUC_lexicon_tags_2 },
    { 0x188507eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a507ee, SUC_lexicon_tags_25 },
    { 0x3d0507ee, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0507f8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1da507fa, SUC_lexicon_tags_25 },
    { 0xd42507fb, SUC_lexicon_tags_37 },
    { 0xf62507fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e650806, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe545080c, SUC_lexicon_tags_17 },
    { 0x1c65080d, SUC_lexicon_tags_29 },
    { 0xfe65080e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa50811, SUC_lexicon_tags_12 },
    { 0x93c50812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a850816, SUC_lexicon_tags_47 },
    { 0x47250817, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf385081b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a650820, SUC_lexicon_tags_31 },
    { 0x95450820, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29650828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7c5082a, SUC_lexicon_tags_10 },
    { 0xc0e5082b, SUC_lexicon_tags_11 },
    { 0xa065082c, SUC_lexicon_tags_17 },
    { 0x9cc5082d, SUC_lexicon_tags_1 },
    { 0xd745082c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x14c50830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a650835, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6050838, SUC_lexicon_tags_15 },
    { 0xc3250839, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x64e5083b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a50844, SUC_lexicon_tags_54 },
    { 0x50c50844, SUC_lexicon_tags_73 },
    { 0x20850846, SUC_lexicon_tags_77 },
    { 0x15650847, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e5084d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe7c5084f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac50855, SUC_lexicon_tags_1 },
    { 0x32650856, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1005085c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1250862, SUC_lexicon_tags_6 },
    { 0x6b850863, SUC_lexicon_tags_31 },
    { 0xaba50862, SUC_lexicon_tags_68 },
    { 0x75650865, SUC_lexicon_tags_14 },
    { 0x63650864, SUC_lexicon_tags_11 },
    { 0x94a50867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf45086a, SUC_lexicon_tags_25 },
    { 0x17e5086b, SUC_lexicon_tags_11 },
    { 0x54e5086c, SUC_lexicon_tags_5 },
    { 0x6025086a, SUC_lexicon_tags_47 },
    { 0x8005086b, SUC_lexicon_tags_16 },
    { 0x7de5086d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b450877, SUC_lexicon_tags_37 },
    { 0x5b650878, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff45087a, SUC_lexicon_tags_37 },
    { 0x9dc5087b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9450886, SUC_lexicon_tags_63 },
    { 0x40e50887, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa5250889, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e25088c, SUC_lexicon_tags_30 },
    { 0x50c5088c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa45088f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2450893, SUC_lexicon_tags_0 },
    { 0xeae50894, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xce850896, SUC_lexicon_tags_27 },
    { 0x9fe50897, SUC_lexicon_tags_32 },
    { 0x4aa50898, SUC_lexicon_tags_15 },
    { 0x50250897, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf405089b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2105089d, SUC_lexicon_tags_63 },
    { 0x76a5089d, SUC_lexicon_tags_15 },
    { 0xc2e5089f, SUC_lexicon_tags_33 },
    { 0x8605089f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x8be508a2, SUC_lexicon_tags_12 },
    { 0xf80508a3, SUC_lexicon_tags_62 },
    { 0x9e0508a3, SUC_lexicon_tags_33 },
    { 0x50c508a5, SUC_lexicon_tags_37 },
    { 0x128508a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786508ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16508b1, SUC_lexicon_tags_37 },
    { 0x60e508b2, SUC_lexicon_tags_11 },
    { 0x87e508b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6508b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x400508b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54508bc, SUC_lexicon_tags_60 },
    { 0x63a508bd, SUC_lexicon_tags_13 },
    { 0x6b8508be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x958508c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x176508c2, SUC_lexicon_tags_1 },
    { 0x720508c3, SUC_lexicon_tags_65 },
    { 0xf90508c4, SUC_lexicon_tags_1 },
    { 0x88e508c5, SUC_lexicon_tags_179 },
    { 0x9e8508c5, SUC_lexicon_tags_47 },
    { 0xa6a508c7, SUC_lexicon_tags_37 },
    { 0xec508c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0508cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12a508cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8508d1, SUC_lexicon_tags_44 },
    { 0x7b4508d2, SUC_lexicon_tags_37 },
    { 0x396508d1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf48508d5, SUC_lexicon_tags_11 },
    { 0x54508d5, SUC_lexicon_tags_1 },
    { 0x1e2508d7, SUC_lexicon_tags_15 },
    { 0xc46508d5, SUC_lexicon_tags_37 },
    { 0x390508d9, SUC_lexicon_tags_13 },
    { 0x26e508da, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de508dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54508e1, SUC_lexicon_tags_17 },
    { 0xe54508e2, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94508e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc50508eb, SUC_lexicon_tags_11 },
    { 0x862508eb, SUC_lexicon_tags_14 },
    { 0x4de508eb, SUC_lexicon_tags_30 },
    { 0x854508ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16508f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708508fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7450907, SUC_lexicon_tags_43 },
    { 0xc0a50908, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1405090d, SUC_lexicon_tags_41 },
    { 0xd845090d, SUC_lexicon_tags_68 },
    { 0xfc45090d, SUC_lexicon_tags_29 },
    { 0x83c50910, SUC_lexicon_tags_15 },
    { 0x98e50911, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac50915, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x1050918, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d05091a, SUC_lexicon_tags_3 },
    { 0xe545091b, SUC_lexicon_tags_49 },
    { 0x6365091b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf245091e, SUC_lexicon_tags_15 },
    { 0x67c5091e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c850922, SUC_lexicon_tags_1 },
    { 0x34450923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9445092a, SUC_lexicon_tags_23 },
    { 0x5092b, SUC_lexicon_tags_107 },
    { 0x8725092c, SUC_lexicon_tags_25 },
    { 0x9505092d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c50933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c50937, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x485093a, SUC_lexicon_tags_2 },
    { 0xba25093a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c5093f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58250941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d050944, SUC_lexicon_tags_7 },
    { 0xafe50945, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x68050948, SUC_lexicon_tags_7 },
    { 0x6ea50949, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a05094b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabc5094d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b250952, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da5095c, SUC_lexicon_tags_2 },
    { 0x31c5095d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf985095f, SUC_lexicon_tags_1 },
    { 0xd0a50960, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2050963, SUC_lexicon_tags_26 },
    { 0x5d450963, SUC_lexicon_tags_2 },
    { 0x13e50965, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc4c50967, SUC_lexicon_tags_86 },
    { 0x5c050968, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62a5096a, SUC_lexicon_tags_45 },
    { 0xcca5096b, SUC_lexicon_tags_37 },
    { 0xe805096c, SUC_lexicon_tags_2 },
    { 0x9f25096b, SUC_lexicon_tags_23 },
    { 0xa725096e, SUC_lexicon_tags_25 },
    { 0xa1c5096f, SUC_lexicon_tags_12 },
    { 0x70450970, SUC_lexicon_tags_38 },
    { 0xfd850971, SUC_lexicon_tags_1 },
    { 0xdc050972, SUC_lexicon_tags_10 },
    { 0xd8450972, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5050977, SUC_lexicon_tags_7 },
    { 0x7e450978, SUC_lexicon_tags_8 },
    { 0x95250978, SUC_lexicon_tags_33 },
    { 0x5ba5097a, SUC_lexicon_tags_25 },
    { 0xdd85097b, SUC_lexicon_tags_11 },
    { 0x5525097b, SUC_lexicon_tags_65 },
    { 0x63c5097a, SUC_lexicon_tags_63 },
    { 0x4805097e, SUC_lexicon_tags_9 },
    { 0x9ee5097a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81a50981, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce50988, SUC_lexicon_tags_37 },
    { 0x29a50989, SUC_lexicon_tags_214 },
    { 0xe850989, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a850995, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4450999, SUC_lexicon_tags_61 },
    { 0x54050999, SUC_lexicon_tags_17 },
    { 0xda5099b, SUC_lexicon_tags_2 },
    { 0x8065099c, SUC_lexicon_tags_15 },
    { 0xebc50999, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd8c5099f, SUC_lexicon_tags_37 },
    { 0xf1e509a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x406509a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4509a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5fc509a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdce509ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa509b2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6509b5, SUC_lexicon_tags_12 },
    { 0x3e509b5, SUC_lexicon_tags_65 },
    { 0x344509b7, SUC_lexicon_tags_38 },
    { 0x99e509b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70509bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52509c4, SUC_lexicon_tags_10 },
    { 0x6ee509c4, SUC_lexicon_tags_12 },
    { 0x128509c6, SUC_lexicon_tags_14 },
    { 0xeb6509c4, SUC_lexicon_tags_16 },
    { 0x5c6509c8, SUC_lexicon_tags_7 },
    { 0xd0c509c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4509ce, SUC_lexicon_tags_25 },
    { 0x1aa509ce, SUC_lexicon_tags_60 },
    { 0xf46509d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77e509d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c509d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce509da, SUC_lexicon_tags_116 },
    { 0xa1a509db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae0509dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x182509df, SUC_lexicon_tags_1 },
    { 0x588509e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a509e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba509ec, SUC_lexicon_tags_47 },
    { 0x6d2509ed, SUC_lexicon_tags_3 },
    { 0x214509ee, SUC_lexicon_tags_25 },
    { 0x4a2509ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a0509f1, SUC_lexicon_tags_0 },
    { 0x43e509f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f4509f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe509f8, SUC_lexicon_tags_2 },
    { 0xab6509f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x574509fb, SUC_lexicon_tags_7 },
    { 0xcd8509fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5d0509fe, SUC_lexicon_tags_17 },
    { 0x7b0509ff, SUC_lexicon_tags_18 },
    { 0x7ba50a00, SUC_lexicon_tags_0 },
    { 0xcdc50a01, SUC_lexicon_tags_82 },
    { 0x86050a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f450a07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28c50a09, SUC_lexicon_tags_7 },
    { 0xccc50a09, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7250a10, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd5e50a12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7050a14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15a50a16, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x1050a19, SUC_lexicon_tags_38 },
    { 0xda50a1a, SUC_lexicon_tags_9 },
    { 0x2a650a1b, SUC_lexicon_tags_1 },
    { 0xce450a1b, SUC_lexicon_tags_17 },
    { 0xf6a50a1b, SUC_lexicon_tags_12 },
    { 0x45050a1a, SUC_lexicon_tags_8 },
    { 0xaf450a1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15250a23, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8a250a25, SUC_lexicon_tags_12 },
    { 0x92850a25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f450a29, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c50a2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89650a34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87650a36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e50a3a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5250a44, SUC_lexicon_tags_21 },
    { 0x6d250a44, SUC_lexicon_tags_9 },
    { 0xcee50a44, SUC_lexicon_tags_45 },
    { 0xddc50a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c50a4a, SUC_lexicon_tags_27 },
    { 0xb3e50a4b, SUC_lexicon_tags_15 },
    { 0xd6450a4c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xca50a4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51850a50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1650a55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e50a58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be50a5b, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c50a5f, SUC_lexicon_tags_7 },
    { 0x91e50a60, SUC_lexicon_tags_37 },
    { 0x10650a61, SUC_lexicon_tags_48 },
    { 0xe8a50a5f, SUC_lexicon_tags_17 },
    { 0x42c50a62, SUC_lexicon_tags_15 },
    { 0x26a50a62, SUC_lexicon_tags_9 },
    { 0xe3650a65, SUC_lexicon_tags_60 },
    { 0x8be50a65, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe50a6a, SUC_lexicon_tags_10 },
    { 0x86650a6b, SUC_lexicon_tags_4 },
    { 0xfc050a6c, SUC_lexicon_tags_12 },
    { 0x34850a6d, SUC_lexicon_tags_29 },
    { 0xbd450a6a, SUC_lexicon_tags_9 },
    { 0xb9650a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11050a71, SUC_lexicon_tags_11 },
    { 0x27c50a72, SUC_lexicon_tags_28 },
    { 0x4b450a73, SUC_lexicon_tags_0 },
    { 0x26250a74, SUC_lexicon_tags_37 },
    { 0xfbc50a75, SUC_lexicon_tags_13 },
    { 0xd9850a71, SUC_lexicon_tags_47 },
    { 0x32250a74, SUC_lexicon_tags_55 },
    { 0x70050a75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c50a7d, SUC_lexicon_tags_87 },
    { 0xcaa50a7e, SUC_lexicon_tags_17 },
    { 0x79250a7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0850a85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xac050a88, SUC_lexicon_tags_60 },
    { 0x15e50a89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7e650a8b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbe250a8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ee50a8f, SUC_lexicon_tags_15 },
    { 0xbc250a90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0250a93, SUC_lexicon_tags_30 },
    { 0x76a50a94, SUC_lexicon_tags_2 },
    { 0x88450a95, SUC_lexicon_tags_16 },
    { 0xfa450a96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f650a9d, SUC_lexicon_tags_8 },
    { 0x55c50a9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48250aa0, SUC_lexicon_tags_11 },
    { 0x7d250aa1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x99e50aa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68650aa7, SUC_lexicon_tags_12 },
    { 0x75250aa7, SUC_lexicon_tags_77 },
    { 0x20850aa9, SUC_lexicon_tags_47 },
    { 0xfc850aaa, SUC_lexicon_tags_549 },
    { 0x46850aaa, SUC_lexicon_tags_37 },
    { 0x5ae50aa8, SUC_lexicon_tags_37 },
    { 0x6da50aad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a50ab4, SUC_lexicon_tags_31 },
    { 0x97250ab5, SUC_lexicon_tags_6 },
    { 0x17450ab5, SUC_lexicon_tags_7 },
    { 0xff250ab7, SUC_lexicon_tags_37 },
    { 0xfc650ab8, SUC_lexicon_tags_43 },
    { 0x39850ab7, SUC_lexicon_tags_92 },
    { 0xb1650ab7, SUC_lexicon_tags_0 },
    { 0x98c50ab8, SUC_lexicon_tags_25 },
    { 0x7bc50ab7, SUC_lexicon_tags_17 },
    { 0x15a50ab9, SUC_lexicon_tags_12 },
    { 0xcd250ab5, SUC_lexicon_tags_2 },
    { 0x39a50abf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f250ac2, SUC_lexicon_tags_1448 },
    { 0x2a450ac3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4650ac6, SUC_lexicon_tags_45 },
    { 0x62250ac7, SUC_lexicon_tags_14 },
    { 0x4e650ac8, SUC_lexicon_tags_12 },
    { 0x5fa50ac8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc650acc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c50acf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e50ad3, SUC_lexicon_tags_11 },
    { 0xca850ad4, SUC_lexicon_tags_25 },
    { 0x42650ad5, SUC_lexicon_tags_26 },
    { 0xee50ad3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1050adb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76450ae0, SUC_lexicon_tags_26 },
    { 0x74050ae1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x75650ae4, SUC_lexicon_tags_45 },
    { 0x3b850ae4, SUC_lexicon_tags_4 },
    { 0xd7850ae6, SUC_lexicon_tags_37 },
    { 0x41250ae5, SUC_lexicon_tags_146 },
    { 0xec50ae4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e50af1, SUC_lexicon_tags_48 },
    { 0xcb450af2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7850af6, SUC_lexicon_tags_57 },
    { 0x21650af6, SUC_lexicon_tags_15 },
    { 0xeac50af8, SUC_lexicon_tags_77 },
    { 0x95850af8, SUC_lexicon_tags_31 },
    { 0x10850af9, SUC_lexicon_tags_27 },
    { 0x2d850afa, SUC_lexicon_tags_47 },
    { 0xf4650af9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a250afe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49250b03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29850b0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x99250b0c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e50b10, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb7c50b12, SUC_lexicon_tags_39 },
    { 0x1a850b13, SUC_lexicon_tags_17 },
    { 0x21850b14, SUC_lexicon_tags_77 },
    { 0x84450b13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35650b18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec50b20, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4450b24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab850b26, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x98250b29, SUC_lexicon_tags_1457 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c50b2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17050b30, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x51a50b32, SUC_lexicon_tags_38 },
    { 0x4c50b33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2450b36, SUC_lexicon_tags_13 },
    { 0x23050b37, SUC_lexicon_tags_25 },
    { 0xbca50b38, SUC_lexicon_tags_117 },
    { 0x94450b39, SUC_lexicon_tags_60 },
    { 0xf7a50b39, SUC_lexicon_tags_17 },
    { 0xa3250b3a, SUC_lexicon_tags_14 },
    { 0xd9050b3a, SUC_lexicon_tags_45 },
    { 0x75a50b37, SUC_lexicon_tags_15 },
    { 0xd8a50b3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9d250b40, SUC_lexicon_tags_60 },
    { 0x49450b41, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x84650b43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7850b47, SUC_lexicon_tags_9 },
    { 0x96650b48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c50b4d, SUC_lexicon_tags_45 },
    { 0x81c50b4d, SUC_lexicon_tags_17 },
    { 0x5d450b4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69450b51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c50b59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58450b5e, SUC_lexicon_tags_31 },
    { 0x11c50b5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f850b63, SUC_lexicon_tags_37 },
    { 0x32050b63, SUC_lexicon_tags_27 },
    { 0xc6a50b65, SUC_lexicon_tags_29 },
    { 0xe7e50b66, SUC_lexicon_tags_49 },
    { 0xdc450b67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80a50b69, SUC_lexicon_tags_55 },
    { 0xe7050b69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70250b71, SUC_lexicon_tags_66 },
    { 0xb3850b71, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x86250b74, SUC_lexicon_tags_0 },
    { 0xc9050b75, SUC_lexicon_tags_63 },
    { 0x65650b74, SUC_lexicon_tags_15 },
    { 0x15250b74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90050b7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e850b84, SUC_lexicon_tags_25 },
    { 0xd5050b85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c50b8a, SUC_lexicon_tags_60 },
    { 0x2f650b8b, SUC_lexicon_tags_63 },
    { 0x62850b8b, SUC_lexicon_tags_30 },
    { 0x59450b8d, SUC_lexicon_tags_1 },
    { 0x99e50b8c, SUC_lexicon_tags_37 },
    { 0x6ac50b8f, SUC_lexicon_tags_9 },
    { 0x76050b8b, SUC_lexicon_tags_12 },
    { 0xde250b91, SUC_lexicon_tags_2 },
    { 0x7aa50b92, SUC_lexicon_tags_9 },
    { 0x90a50b91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e850b95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1650b99, SUC_lexicon_tags_27 },
    { 0x8ce50b9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29450b9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11850ba4, SUC_lexicon_tags_38 },
    { 0xa5e50ba4, SUC_lexicon_tags_37 },
    { 0x20450ba6, SUC_lexicon_tags_2 },
    { 0x56c50ba7, SUC_lexicon_tags_17 },
    { 0x43050ba6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9b450baa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0050bad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86450baf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ea50bb1, SUC_lexicon_tags_15 },
    { 0x39a50bb2, SUC_lexicon_tags_55 },
    { 0x88650bb1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c850bb6, SUC_lexicon_tags_1 },
    { 0xe450bb6, SUC_lexicon_tags_57 },
    { 0x18450bb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2450bbc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xec450bbe, SUC_lexicon_tags_47 },
    { 0x2b850bbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee50bc6, SUC_lexicon_tags_1 },
    { 0x87050bc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed250bca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa250bcf, SUC_lexicon_tags_37 },
    { 0x23450bcf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7250bd6, SUC_lexicon_tags_37 },
    { 0xe5250bd6, SUC_lexicon_tags_33 },
    { 0x6ee50bd8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x48450bda, SUC_lexicon_tags_38 },
    { 0x7c650bdb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9250be3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd450be5, SUC_lexicon_tags_19 },
    { 0xa8e50be6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec50beb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88050bf1, SUC_lexicon_tags_27 },
    { 0x3d050bf2, SUC_lexicon_tags_82 },
    { 0x6b050bf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d250bfc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x60e50bfe, SUC_lexicon_tags_70 },
    { 0x2d250bff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2d050c01, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x70650c03, SUC_lexicon_tags_37 },
    { 0x99450c04, SUC_lexicon_tags_2 },
    { 0xb7450c05, SUC_lexicon_tags_25 },
    { 0xc0e50c06, SUC_lexicon_tags_0 },
    { 0x9b850c03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e50c0a, SUC_lexicon_tags_19 },
    { 0xb5a50c0a, SUC_lexicon_tags_25 },
    { 0x64450c0b, SUC_lexicon_tags_49 },
    { 0x51650c0a, SUC_lexicon_tags_1 },
    { 0xb650c0b, SUC_lexicon_tags_63 },
    { 0x1a450c0f, SUC_lexicon_tags_56 },
    { 0xffc50c0f, SUC_lexicon_tags_17 },
    { 0x1ec50c10, SUC_lexicon_tags_194 },
    { 0xc0c50c12, SUC_lexicon_tags_21 },
    { 0x88a50c12, SUC_lexicon_tags_1 },
    { 0x10850c14, SUC_lexicon_tags_14 },
    { 0x82450c15, SUC_lexicon_tags_14 },
    { 0xff450c15, SUC_lexicon_tags_25 },
    { 0x6dc50c17, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x70a50c19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25450c1e, SUC_lexicon_tags_49 },
    { 0xb0450c1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f250c26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e50c2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c850c2c, SUC_lexicon_tags_17 },
    { 0xda850c2c, SUC_lexicon_tags_61 },
    { 0x5a850c2e, SUC_lexicon_tags_48 },
    { 0x8d050c2e, SUC_lexicon_tags_158 },
    { 0x8ac50c30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea50c33, SUC_lexicon_tags_0 },
    { 0x85450c34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba50c37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70650c39, SUC_lexicon_tags_23 },
    { 0x13850c3a, SUC_lexicon_tags_16 },
    { 0xb7a50c39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a250c3d, SUC_lexicon_tags_25 },
    { 0xf1c50c3e, SUC_lexicon_tags_23 },
    { 0xdb250c3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32250c44, SUC_lexicon_tags_17 },
    { 0x4c250c44, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a50c4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e50c54, SUC_lexicon_tags_25 },
    { 0x78e50c55, SUC_lexicon_tags_47 },
    { 0xfda50c56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e050c5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc50c62, SUC_lexicon_tags_1 },
    { 0x9de50c63, SUC_lexicon_tags_234 },
    { 0xf2650c64, SUC_lexicon_tags_23 },
    { 0xf7c50c63, SUC_lexicon_tags_23 },
    { 0xdd650c65, SUC_lexicon_tags_80 },
    { 0x29e50c66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c50c6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62450c6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9be50c6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a50c79, SUC_lexicon_tags_23 },
    { 0x3da50c7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde650c7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa50c84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31450c8d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7a450c8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6250c98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a050c9a, SUC_lexicon_tags_2 },
    { 0xde850c9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a50cad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c50cb0, SUC_lexicon_tags_2 },
    { 0x48a50cb1, SUC_lexicon_tags_65 },
    { 0x8de50cb2, SUC_lexicon_tags_28 },
    { 0xcec50cb3, SUC_lexicon_tags_31 },
    { 0x2cc50cb1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c50cb8, SUC_lexicon_tags_3 },
    { 0x65450cb9, SUC_lexicon_tags_12 },
    { 0x2e450cb9, SUC_lexicon_tags_33 },
    { 0x74250cbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c50cc0, SUC_lexicon_tags_13 },
    { 0xab650cc0, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb650cca, SUC_lexicon_tags_45 },
    { 0x5d850cca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7450cd0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76e50cd2, SUC_lexicon_tags_11 },
    { 0x7c450cd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a50cd8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x70450cda, SUC_lexicon_tags_25 },
    { 0x81250cda, SUC_lexicon_tags_19 },
    { 0xbb250cdc, SUC_lexicon_tags_1 },
    { 0x6b850cdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79250cdf, SUC_lexicon_tags_60 },
    { 0xa050cdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec50ce4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9850ce6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x66c50ce8, SUC_lexicon_tags_23 },
    { 0x9e850ce8, SUC_lexicon_tags_17 },
    { 0xc7450cea, SUC_lexicon_tags_2 },
    { 0x4a050ceb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e050cee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb850cf8, SUC_lexicon_tags_25 },
    { 0x42a50cf9, SUC_lexicon_tags_1 },
    { 0x51650cf9, SUC_lexicon_tags_11 },
    { 0xcd850cfb, SUC_lexicon_tags_2 },
    { 0x8ce50cfc, SUC_lexicon_tags_7 },
    { 0xe5450cfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7650d00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36250d02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a250d06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba50d0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1850d0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8650d10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a250d16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e50d19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e50d1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81250d1f, SUC_lexicon_tags_11 },
    { 0xc9450d20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b050d22, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a850d26, SUC_lexicon_tags_29 },
    { 0xbfe50d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9850d2f, SUC_lexicon_tags_69 },
    { 0xb8650d30, SUC_lexicon_tags_12 },
    { 0xea250d31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c850d35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x450d38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x39050d3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6250d40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c450d48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d650d4b, SUC_lexicon_tags_11 },
    { 0x8f250d4b, SUC_lexicon_tags_32 },
    { 0x51650d4b, SUC_lexicon_tags_60 },
    { 0xb650d4d, SUC_lexicon_tags_11 },
    { 0x86c50d4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c50d52, SUC_lexicon_tags_7 },
    { 0x66850d53, SUC_lexicon_tags_25 },
    { 0x6e250d52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f850d57, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc2c50d59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4450d5d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed650d64, SUC_lexicon_tags_14 },
    { 0x9e450d65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16050d6c, SUC_lexicon_tags_15 },
    { 0xd6850d6c, SUC_lexicon_tags_63 },
    { 0x82250d6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec250d77, SUC_lexicon_tags_66 },
    { 0x12250d78, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa50d7b, SUC_lexicon_tags_19 },
    { 0x6c250d7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84250d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf650d85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x48a50d87, SUC_lexicon_tags_82 },
    { 0x4d650d87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7a050d8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e50d96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6650d99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c50d9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51250da0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a50da4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc50daf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x72850db2, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xcfc50db4, SUC_lexicon_tags_33 },
    { 0xec50db5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f250db8, SUC_lexicon_tags_82 },
    { 0x6f650db9, SUC_lexicon_tags_1 },
    { 0x8f250db9, SUC_lexicon_tags_45 },
    { 0x16650dba, SUC_lexicon_tags_37 },
    { 0x91450dbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b050dc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e850dc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea50dca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5e50dcc, SUC_lexicon_tags_4 },
    { 0x7dc50dcd, SUC_lexicon_tags_1 },
    { 0x6da50dce, SUC_lexicon_tags_53 },
    { 0x46450dce, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf2c50dd1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17450dd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d650dd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b850ddd, SUC_lexicon_tags_25 },
    { 0x80850ddd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea50de2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b050deb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa450ded, SUC_lexicon_tags_57 },
    { 0x57650dee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62450df1, SUC_lexicon_tags_11 },
    { 0xecc50df2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65850df4, SUC_lexicon_tags_11 },
    { 0xcfe50df5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2a50df7, SUC_lexicon_tags_10 },
    { 0x6650df8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c50dfc, SUC_lexicon_tags_14 },
    { 0xd6250dfd, SUC_lexicon_tags_0 },
    { 0x73e50dfd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e50e03, SUC_lexicon_tags_37 },
    { 0x45250e03, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x98a50e06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6a50e08, SUC_lexicon_tags_0 },
    { 0xa9250e09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a50e0f, SUC_lexicon_tags_1 },
    { 0x68250e0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c50e14, SUC_lexicon_tags_65 },
    { 0x3ee50e15, SUC_lexicon_tags_39 },
    { 0xa850e15, SUC_lexicon_tags_47 },
    { 0x67450e17, SUC_lexicon_tags_37 },
    { 0x3d050e15, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe050e1c, SUC_lexicon_tags_19 },
    { 0xdc050e1c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b650e21, SUC_lexicon_tags_1 },
    { 0xd9c50e22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6c50e24, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xaae50e26, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9f850e28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a450e2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be50e2e, SUC_lexicon_tags_45 },
    { 0x65a50e2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62250e33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f050e35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e50e38, SUC_lexicon_tags_33 },
    { 0x5ea50e38, SUC_lexicon_tags_31 },
    { 0x33450e39, SUC_lexicon_tags_43 },
    { 0x63c50e3b, SUC_lexicon_tags_8 },
    { 0x19850e38, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f050e42, SUC_lexicon_tags_7 },
    { 0x85a50e42, SUC_lexicon_tags_27 },
    { 0x71e50e43, SUC_lexicon_tags_39 },
    { 0x4fe50e45, SUC_lexicon_tags_10 },
    { 0x9b250e45, SUC_lexicon_tags_17 },
    { 0x96050e47, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe50e4a, SUC_lexicon_tags_23 },
    { 0x83c50e4b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x14050e4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1050e52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc50e58, SUC_lexicon_tags_45 },
    { 0x83a50e59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c50e62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67850e68, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e50e6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a250e6e, SUC_lexicon_tags_14 },
    { 0x5c50e6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bc50e71, SUC_lexicon_tags_11 },
    { 0x7a450e71, SUC_lexicon_tags_63 },
    { 0xfa650e73, SUC_lexicon_tags_1 },
    { 0xf650e74, SUC_lexicon_tags_2 },
    { 0x55c50e75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80a50e77, SUC_lexicon_tags_45 },
    { 0xff450e77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc50e7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b650e7e, SUC_lexicon_tags_7 },
    { 0x3d450e7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52050e87, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xeca50e89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18450e8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6e50e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44250e95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb650e98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a50e9c, SUC_lexicon_tags_29 },
    { 0x87050e9d, SUC_lexicon_tags_37 },
    { 0xb6c50e9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d450ea0, SUC_lexicon_tags_15 },
    { 0x3a850ea1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9450ea3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e050ea5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1050ea7, SUC_lexicon_tags_43 },
    { 0x73850ea8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1650eb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7250eb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c50eb9, SUC_lexicon_tags_15 },
    { 0x3a450eba, SUC_lexicon_tags_37 },
    { 0xda50ebb, SUC_lexicon_tags_17 },
    { 0xacc50ebb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a50ec0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6250ec4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xab650ec6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e850ec9, SUC_lexicon_tags_37 },
    { 0x2da50ec9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x10850ecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc50ed0, SUC_lexicon_tags_17 },
    { 0x1f250ed1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c50ed5, SUC_lexicon_tags_11 },
    { 0x87650ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32e50ed8, SUC_lexicon_tags_55 },
    { 0x1d850ed9, SUC_lexicon_tags_27 },
    { 0xf3a50eda, SUC_lexicon_tags_9 },
    { 0xb2450ed8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb050ee1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40450ee3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c50ee7, SUC_lexicon_tags_11 },
    { 0x59e50ee8, SUC_lexicon_tags_55 },
    { 0xb3a50ee8, SUC_lexicon_tags_25 },
    { 0x76050eea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73450eed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x85850ef0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xea50ef3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x35450ef5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c450ef7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8450ef9, SUC_lexicon_tags_23 },
    { 0xfaa50efa, SUC_lexicon_tags_6 },
    { 0x9650efb, SUC_lexicon_tags_31 },
    { 0x45450efc, SUC_lexicon_tags_47 },
    { 0x87250efd, SUC_lexicon_tags_66 },
    { 0xdcc50efe, SUC_lexicon_tags_14 },
    { 0xf7e50efe, SUC_lexicon_tags_11 },
    { 0x8a650efa, SUC_lexicon_tags_15 },
    { 0x99050efd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c50f05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb250f0b, SUC_lexicon_tags_14 },
    { 0x40250f0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3650f0e, SUC_lexicon_tags_14 },
    { 0x55250f0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7e50f11, SUC_lexicon_tags_25 },
    { 0x3b850f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47850f14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9aa50f16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2850f18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33a50f1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a650f1d, SUC_lexicon_tags_57 },
    { 0x66450f1e, SUC_lexicon_tags_47 },
    { 0x51050f1f, SUC_lexicon_tags_14 },
    { 0x96050f1e, SUC_lexicon_tags_39 },
    { 0x3e250f20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9450f23, SUC_lexicon_tags_27 },
    { 0xc7e50f24, SUC_lexicon_tags_10 },
    { 0xa9c50f23, SUC_lexicon_tags_29 },
    { 0xa4a50f26, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x50650f28, SUC_lexicon_tags_7 },
    { 0x13450f29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc50f2e, SUC_lexicon_tags_38 },
    { 0xcc850f2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ac50f31, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x29450f33, SUC_lexicon_tags_69 },
    { 0x7c50f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28a50f36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e50f3a, SUC_lexicon_tags_49 },
    { 0xf5250f3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9450f3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e50f42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53850f44, SUC_lexicon_tags_37 },
    { 0x2b050f45, SUC_lexicon_tags_25 },
    { 0x2e250f45, SUC_lexicon_tags_48 },
    { 0x80450f44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7850f4a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e50f4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf250f50, SUC_lexicon_tags_15 },
    { 0x3f650f51, SUC_lexicon_tags_37 },
    { 0x8ce50f52, SUC_lexicon_tags_2 },
    { 0xd5050f52, SUC_lexicon_tags_82 },
    { 0x76c50f52, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0x1ca50f56, SUC_lexicon_tags_26 },
    { 0x2de50f56, SUC_lexicon_tags_26 },
    { 0xc450f57, SUC_lexicon_tags_26 },
    { 0x13650f59, SUC_lexicon_tags_1 },
    { 0x26850f5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea50f5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29050f64, SUC_lexicon_tags_2 },
    { 0x73850f64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a050f6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x69850f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdc50f70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x35450f73, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e50f76, SUC_lexicon_tags_87 },
    { 0x28850f77, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a50f7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63450f7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c50f85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2250f88, SUC_lexicon_tags_26 },
    { 0x5b650f89, SUC_lexicon_tags_17 },
    { 0x87a50f8a, SUC_lexicon_tags_2 },
    { 0x5c50f8a, SUC_lexicon_tags_1 },
    { 0x48650f8c, SUC_lexicon_tags_39 },
    { 0x70e50f8d, SUC_lexicon_tags_141 },
    { 0x2c250f8e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x24450f90, SUC_lexicon_tags_0 },
    { 0x5ee50f90, SUC_lexicon_tags_39 },
    { 0xcbc50f90, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e50f96, SUC_lexicon_tags_17 },
    { 0x92e50f96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c050f9b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x77e50f9d, SUC_lexicon_tags_1 },
    { 0x65650f9e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68650fa4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9450fa8, SUC_lexicon_tags_37 },
    { 0xf7650fa9, SUC_lexicon_tags_27 },
    { 0x94650fa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35c50fac, SUC_lexicon_tags_12 },
    { 0x8ca50fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42850faf, SUC_lexicon_tags_37 },
    { 0xd0850faf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x14050fb2, SUC_lexicon_tags_50 },
    { 0x67250fb2, SUC_lexicon_tags_32 },
    { 0x36e50fb4, SUC_lexicon_tags_11 },
    { 0x5250fb5, SUC_lexicon_tags_7 },
    { 0x1c850fb6, SUC_lexicon_tags_40 },
    { 0x27050fb7, SUC_lexicon_tags_2 },
    { 0x62250fb8, SUC_lexicon_tags_25 },
    { 0x12250fb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84850fbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb450fbf, SUC_lexicon_tags_11 },
    { 0xc4c50fbf, SUC_lexicon_tags_37 },
    { 0x73e50fc1, SUC_lexicon_tags_7 },
    { 0x83050fc2, SUC_lexicon_tags_2 },
    { 0x6e250fc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd450fc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c50fc9, SUC_lexicon_tags_30 },
    { 0x63a50fca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe850fcf, SUC_lexicon_tags_17 },
    { 0xf5850fd0, SUC_lexicon_tags_82 },
    { 0xc7c50fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a50fd6, SUC_lexicon_tags_26 },
    { 0xa3c50fd6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33650fd9, SUC_lexicon_tags_15 },
    { 0x79e50fda, SUC_lexicon_tags_7 },
    { 0xe9650fdb, SUC_lexicon_tags_9 },
    { 0xa5850fdb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9050fdf, SUC_lexicon_tags_38 },
    { 0xaee50fdf, SUC_lexicon_tags_26 },
    { 0x1da50fdf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e50fe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a50fed, SUC_lexicon_tags_15 },
    { 0xb3850fee, SUC_lexicon_tags_15 },
    { 0x7a850fef, SUC_lexicon_tags_505 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c50ff2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc50ff9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x77050ffc, SUC_lexicon_tags_23 },
    { 0x86450ffd, SUC_lexicon_tags_7 },
    { 0xe1450ffd, SUC_lexicon_tags_25 },
    { 0x9e250ffc, SUC_lexicon_tags_23 },
    { 0xf0251000, SUC_lexicon_tags_1545 },
    { 0x2a250fff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x80e51003, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86251005, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe185100a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x50c5100c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb025100e, SUC_lexicon_tags_25 },
    { 0xb085100f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a51016, SUC_lexicon_tags_70 },
    { 0x2cc51017, SUC_lexicon_tags_70 },
    { 0x14851016, SUC_lexicon_tags_25 },
    { 0xf1051016, SUC_lexicon_tags_1 },
    { 0x77051018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9851021, SUC_lexicon_tags_4 },
    { 0xfa051022, SUC_lexicon_tags_25 },
    { 0x86051023, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c51026, SUC_lexicon_tags_29 },
    { 0x3b451027, SUC_lexicon_tags_7 },
    { 0x12851027, SUC_lexicon_tags_17 },
    { 0xd7851029, SUC_lexicon_tags_33 },
    { 0x3ac5102a, SUC_lexicon_tags_7 },
    { 0xb845102b, SUC_lexicon_tags_17 },
    { 0x9305102c, SUC_lexicon_tags_45 },
    { 0x7885102d, SUC_lexicon_tags_12 },
    { 0x7945102e, SUC_lexicon_tags_37 },
    { 0x3025102f, SUC_lexicon_tags_45 },
    { 0x2d451030, SUC_lexicon_tags_49 },
    { 0xc1a51030, SUC_lexicon_tags_26 },
    { 0x2b85102f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7ac51034, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1851038, SUC_lexicon_tags_37 },
    { 0xd1c51039, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4a5103b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5025103d, SUC_lexicon_tags_29 },
    { 0x20c5103d, SUC_lexicon_tags_17 },
    { 0x325103f, SUC_lexicon_tags_17 },
    { 0x93a51040, SUC_lexicon_tags_11 },
    { 0xabc51041, SUC_lexicon_tags_11 },
    { 0x88451041, SUC_lexicon_tags_37 },
    { 0x2205103d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac51045, SUC_lexicon_tags_25 },
    { 0x30c51046, SUC_lexicon_tags_17 },
    { 0x96c51047, SUC_lexicon_tags_31 },
    { 0x1c651046, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa925104b, SUC_lexicon_tags_13 },
    { 0xa0e5104b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6251053, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdbc51055, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e5105d, SUC_lexicon_tags_11 },
    { 0x97a5105e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7e051060, SUC_lexicon_tags_27 },
    { 0x58451061, SUC_lexicon_tags_37 },
    { 0x13451062, SUC_lexicon_tags_7 },
    { 0x3851063, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c51067, SUC_lexicon_tags_2 },
    { 0x4da51067, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd305106a, SUC_lexicon_tags_60 },
    { 0x1105106b, SUC_lexicon_tags_11 },
    { 0xb45106c, SUC_lexicon_tags_1 },
    { 0xe705106d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc51070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d451072, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xac51075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a65107c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a51082, SUC_lexicon_tags_26 },
    { 0xff051082, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6851086, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6fa51088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57c5108a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc51090, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdde51092, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa5109d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x830510a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x29a510a2, SUC_lexicon_tags_30 },
    { 0x662510a3, SUC_lexicon_tags_37 },
    { 0x1b6510a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52e510a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64a510a8, SUC_lexicon_tags_11 },
    { 0x3de510a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff0510ab, SUC_lexicon_tags_29 },
    { 0x98a510ab, SUC_lexicon_tags_25 },
    { 0x6dc510ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6aa510af, SUC_lexicon_tags_17 },
    { 0x550510b0, SUC_lexicon_tags_37 },
    { 0xc7e510b1, SUC_lexicon_tags_27 },
    { 0x6fe510b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442510b7, SUC_lexicon_tags_7 },
    { 0x7cc510b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2510bd, SUC_lexicon_tags_39 },
    { 0x560510be, SUC_lexicon_tags_125 },
    { 0x280510be, SUC_lexicon_tags_26 },
    { 0x2c6510bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e510c6, SUC_lexicon_tags_34 },
    { 0xebe510c7, SUC_lexicon_tags_49 },
    { 0xcc4510c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00510cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d4510ce, SUC_lexicon_tags_2 },
    { 0x4f2510cf, SUC_lexicon_tags_45 },
    { 0x3d2510cf, SUC_lexicon_tags_12 },
    { 0x7c2510d0, SUC_lexicon_tags_18 },
    { 0x8c510ce, SUC_lexicon_tags_14 },
    { 0x768510d3, SUC_lexicon_tags_38 },
    { 0xc2c510d4, SUC_lexicon_tags_17 },
    { 0xbde510d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984510da, SUC_lexicon_tags_1 },
    { 0x264510da, SUC_lexicon_tags_2 },
    { 0x95c510dc, SUC_lexicon_tags_45 },
    { 0x708510dd, SUC_lexicon_tags_45 },
    { 0x59e510de, SUC_lexicon_tags_14 },
    { 0x172510de, SUC_lexicon_tags_14 },
    { 0x6ac510dd, SUC_lexicon_tags_47 },
    { 0x754510e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0510e5, SUC_lexicon_tags_7 },
    { 0x4b2510e6, SUC_lexicon_tags_15 },
    { 0x4ee510e7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2510ee, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea510f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x27e510f4, SUC_lexicon_tags_60 },
    { 0x136510f5, SUC_lexicon_tags_79 },
    { 0xb82510f6, SUC_lexicon_tags_49 },
    { 0xc72510f7, SUC_lexicon_tags_14 },
    { 0xb48510f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22510fb, SUC_lexicon_tags_25 },
    { 0xb6510fb, SUC_lexicon_tags_33 },
    { 0xb22510fb, SUC_lexicon_tags_46 },
    { 0x7f6510fe, SUC_lexicon_tags_37 },
    { 0x1ce510fe, SUC_lexicon_tags_87 },
    { 0x41a51100, SUC_lexicon_tags_11 },
    { 0x67051101, SUC_lexicon_tags_37 },
    { 0x4e851100, SUC_lexicon_tags_11 },
    { 0x7d451103, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5f851105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e51109, SUC_lexicon_tags_37 },
    { 0xfc45110a, SUC_lexicon_tags_27 },
    { 0x145110b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9285110d, SUC_lexicon_tags_23 },
    { 0xb205110d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0251115, SUC_lexicon_tags_19 },
    { 0xb3251116, SUC_lexicon_tags_66 },
    { 0xdcc51117, SUC_lexicon_tags_10 },
    { 0x28051116, SUC_lexicon_tags_7 },
    { 0x61851117, SUC_lexicon_tags_27 },
    { 0xec25111a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc745111d, SUC_lexicon_tags_68 },
    { 0x19c5111e, SUC_lexicon_tags_25 },
    { 0xeb85111f, SUC_lexicon_tags_25 },
    { 0xe1e5111d, SUC_lexicon_tags_1 },
    { 0xf525111d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c051126, SUC_lexicon_tags_6 },
    { 0x70851127, SUC_lexicon_tags_12 },
    { 0xbaa51128, SUC_lexicon_tags_14 },
    { 0x29251126, SUC_lexicon_tags_15 },
    { 0xfde51127, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc05112d, SUC_lexicon_tags_16 },
    { 0xb885112e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f451131, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x23651134, SUC_lexicon_tags_15 },
    { 0xbe851135, SUC_lexicon_tags_69 },
    { 0x4e051134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f65113a, SUC_lexicon_tags_15 },
    { 0x5785113a, SUC_lexicon_tags_33 },
    { 0x8ba5113b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3205113f, SUC_lexicon_tags_37 },
    { 0xfee51140, SUC_lexicon_tags_17 },
    { 0xa785113f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e51146, SUC_lexicon_tags_23 },
    { 0x4d051147, SUC_lexicon_tags_49 },
    { 0x21251148, SUC_lexicon_tags_14 },
    { 0xb5851149, SUC_lexicon_tags_25 },
    { 0x1525114a, SUC_lexicon_tags_63 },
    { 0x545114b, SUC_lexicon_tags_1 },
    { 0x6445114c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a51157, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b05115b, SUC_lexicon_tags_11 },
    { 0x6985115c, SUC_lexicon_tags_25 },
    { 0x60c5115d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56651160, SUC_lexicon_tags_3 },
    { 0xa9c51160, SUC_lexicon_tags_31 },
    { 0xba251161, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c851166, SUC_lexicon_tags_68 },
    { 0x7f851166, SUC_lexicon_tags_12 },
    { 0x2f651168, SUC_lexicon_tags_11 },
    { 0xa6651168, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4225116c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda51170, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x24651172, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfe251174, SUC_lexicon_tags_55 },
    { 0x71e51175, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc51179, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc65117b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc5117e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89651180, SUC_lexicon_tags_28 },
    { 0x38c51181, SUC_lexicon_tags_32 },
    { 0xaf051182, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x19251184, SUC_lexicon_tags_17 },
    { 0xa7251185, SUC_lexicon_tags_29 },
    { 0xa2851186, SUC_lexicon_tags_7 },
    { 0x49251186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7251189, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a51194, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fc51196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765119a, SUC_lexicon_tags_2 },
    { 0xaf65119a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52511a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2511a9, SUC_lexicon_tags_160 },
    { 0x8d8511aa, SUC_lexicon_tags_14 },
    { 0x92a511a9, SUC_lexicon_tags_12 },
    { 0x576511ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x780511ae, SUC_lexicon_tags_10 },
    { 0xffe511af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x422511b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396511b6, SUC_lexicon_tags_11 },
    { 0x8a6511b6, SUC_lexicon_tags_8 },
    { 0x63a511b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e8511ba, SUC_lexicon_tags_14 },
    { 0xc06511bb, SUC_lexicon_tags_7 },
    { 0xd9a511bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2511c2, SUC_lexicon_tags_45 },
    { 0x770511c3, SUC_lexicon_tags_25 },
    { 0x914511c4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x2ec511c6, SUC_lexicon_tags_60 },
    { 0xca4511c7, SUC_lexicon_tags_47 },
    { 0x696511c8, SUC_lexicon_tags_15 },
    { 0x918511c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752511cd, SUC_lexicon_tags_18 },
    { 0x68511cd, SUC_lexicon_tags_47 },
    { 0x214511ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc32511d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206511db, SUC_lexicon_tags_15 },
    { 0xde2511dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee511e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d6511e3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8511ec, SUC_lexicon_tags_71 },
    { 0xcb2511ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6511f1, SUC_lexicon_tags_12 },
    { 0x694511f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6511f8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c511fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd051201, SUC_lexicon_tags_12 },
    { 0xcda51202, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c51205, SUC_lexicon_tags_25 },
    { 0x54451206, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe51209, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa5120c, SUC_lexicon_tags_35 },
    { 0x7485120d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5251210, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e651215, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf451218, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c25121f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a51223, SUC_lexicon_tags_27 },
    { 0x2e051224, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2851227, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a5122d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65251233, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd8c51235, SUC_lexicon_tags_8 },
    { 0x1c51235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4525123c, SUC_lexicon_tags_16 },
    { 0x5065123d, SUC_lexicon_tags_306 },
    { 0xee5123e, SUC_lexicon_tags_25 },
    { 0x8265123f, SUC_lexicon_tags_17 },
    { 0xebe5123e, SUC_lexicon_tags_37 },
    { 0x60651241, SUC_lexicon_tags_30 },
    { 0x84251241, SUC_lexicon_tags_9 },
    { 0xdf051242, SUC_lexicon_tags_47 },
    { 0xd3051240, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96851247, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8f451249, SUC_lexicon_tags_14 },
    { 0xc4c5124a, SUC_lexicon_tags_27 },
    { 0xee65124a, SUC_lexicon_tags_29 },
    { 0x6165124a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55251255, SUC_lexicon_tags_2 },
    { 0x2fc51256, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be51259, SUC_lexicon_tags_38 },
    { 0xa5a5125a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbd05125c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a051260, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e251262, SUC_lexicon_tags_2 },
    { 0x62451263, SUC_lexicon_tags_7 },
    { 0xb0451264, SUC_lexicon_tags_15 },
    { 0xa6651264, SUC_lexicon_tags_37 },
    { 0xc9e51263, SUC_lexicon_tags_1 },
    { 0x2251265, SUC_lexicon_tags_1719 },
    { 0, NULL },
    { 0, NULL },
    { 0x4145126a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1851276, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a51279, SUC_lexicon_tags_15 },
    { 0x2745127a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x5885127d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3e5127f, SUC_lexicon_tags_2 },
    { 0x2f45127f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae51283, SUC_lexicon_tags_14 },
    { 0xabc51284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70051286, SUC_lexicon_tags_45 },
    { 0x4451286, SUC_lexicon_tags_1 },
    { 0x66251288, SUC_lexicon_tags_7 },
    { 0x4b651288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa05128b, SUC_lexicon_tags_7 },
    { 0xd6a5128c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98251293, SUC_lexicon_tags_6 },
    { 0xb6451294, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53a51296, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e45129e, SUC_lexicon_tags_11 },
    { 0xa9e5129f, SUC_lexicon_tags_3 },
    { 0x742512a0, SUC_lexicon_tags_1 },
    { 0xaac512a0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x40c512a3, SUC_lexicon_tags_31 },
    { 0x948512a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72512a8, SUC_lexicon_tags_9 },
    { 0xfe4512a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c512af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0512b2, SUC_lexicon_tags_37 },
    { 0x20a512b3, SUC_lexicon_tags_45 },
    { 0xf1e512b4, SUC_lexicon_tags_27 },
    { 0x222512b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe5c512b7, SUC_lexicon_tags_14 },
    { 0x2ee512b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570512bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x374512bf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70512c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae512ca, SUC_lexicon_tags_37 },
    { 0x6b8512cb, SUC_lexicon_tags_47 },
    { 0xa80512ca, SUC_lexicon_tags_2 },
    { 0x81e512cb, SUC_lexicon_tags_31 },
    { 0xa68512cc, SUC_lexicon_tags_138 },
    { 0xdb4512cb, SUC_lexicon_tags_49 },
    { 0xf68512d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26512d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e512da, SUC_lexicon_tags_19 },
    { 0x6e2512db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a512de, SUC_lexicon_tags_47 },
    { 0xf74512df, SUC_lexicon_tags_17 },
    { 0xbb8512e0, SUC_lexicon_tags_79 },
    { 0x50c512e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2512e5, SUC_lexicon_tags_37 },
    { 0x4a512e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd34512e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a512ed, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8512f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2512f4, SUC_lexicon_tags_37 },
    { 0x232512f4, SUC_lexicon_tags_47 },
    { 0xf8a512f5, SUC_lexicon_tags_8 },
    { 0x450512f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c512ff, SUC_lexicon_tags_17 },
    { 0xb1051300, SUC_lexicon_tags_17 },
    { 0x1cc51301, SUC_lexicon_tags_11 },
    { 0x20851300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3e51304, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c51308, SUC_lexicon_tags_11 },
    { 0x5f051309, SUC_lexicon_tags_17 },
    { 0x3051309, SUC_lexicon_tags_15 },
    { 0xb3c51309, SUC_lexicon_tags_12 },
    { 0xbf45130b, SUC_lexicon_tags_28 },
    { 0x6d05130d, SUC_lexicon_tags_17 },
    { 0x7de5130e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0851310, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea651317, SUC_lexicon_tags_11 },
    { 0x61051318, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b25131b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9e5131d, SUC_lexicon_tags_0 },
    { 0x6445131e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xebe51320, SUC_lexicon_tags_1 },
    { 0x46e51321, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x93451324, SUC_lexicon_tags_17 },
    { 0xf2851325, SUC_lexicon_tags_26 },
    { 0x5d451324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7251329, SUC_lexicon_tags_15 },
    { 0xcae51329, SUC_lexicon_tags_97 },
    { 0xa051329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2525132e, SUC_lexicon_tags_17 },
    { 0x3245132f, SUC_lexicon_tags_26 },
    { 0xef851330, SUC_lexicon_tags_27 },
    { 0x2a251331, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfe651333, SUC_lexicon_tags_38 },
    { 0x26251334, SUC_lexicon_tags_47 },
    { 0xe3e51335, SUC_lexicon_tags_13 },
    { 0x8f051336, SUC_lexicon_tags_11 },
    { 0x2b251337, SUC_lexicon_tags_2 },
    { 0x32051338, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f051340, SUC_lexicon_tags_15 },
    { 0xc8251340, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdd851343, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca5134b, SUC_lexicon_tags_43 },
    { 0x1b05134b, SUC_lexicon_tags_11 },
    { 0x99c5134c, SUC_lexicon_tags_40 },
    { 0xeca5134b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee51355, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de51359, SUC_lexicon_tags_43 },
    { 0xc651359, SUC_lexicon_tags_11 },
    { 0x5305135b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc51361, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc4251363, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c651366, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e51369, SUC_lexicon_tags_37 },
    { 0x225136a, SUC_lexicon_tags_14 },
    { 0x1945136b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd965136d, SUC_lexicon_tags_60 },
    { 0xad85136d, SUC_lexicon_tags_16 },
    { 0x6fe5136d, SUC_lexicon_tags_2 },
    { 0x41051370, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7451372, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdca51374, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99451376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61051378, SUC_lexicon_tags_25 },
    { 0x62451379, SUC_lexicon_tags_43 },
    { 0xbea51379, SUC_lexicon_tags_0 },
    { 0x9b251379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc651380, SUC_lexicon_tags_230 },
    { 0x41c51381, SUC_lexicon_tags_23 },
    { 0xd4651381, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40851388, SUC_lexicon_tags_38 },
    { 0x25a51389, SUC_lexicon_tags_37 },
    { 0x2785138a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7851390, SUC_lexicon_tags_27 },
    { 0x4fa51390, SUC_lexicon_tags_45 },
    { 0xdb851390, SUC_lexicon_tags_2 },
    { 0x72e51392, SUC_lexicon_tags_31 },
    { 0x83a51394, SUC_lexicon_tags_60 },
    { 0xdac51394, SUC_lexicon_tags_11 },
    { 0x80051395, SUC_lexicon_tags_55 },
    { 0x16a51397, SUC_lexicon_tags_7 },
    { 0xc6851393, SUC_lexicon_tags_17 },
    { 0xb7c51399, SUC_lexicon_tags_51 },
    { 0x2251390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe305139c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea513a3, SUC_lexicon_tags_39 },
    { 0x6ec513a4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x726513a7, SUC_lexicon_tags_7 },
    { 0x434513a8, SUC_lexicon_tags_1 },
    { 0x782513a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e513ad, SUC_lexicon_tags_12 },
    { 0xe62513ad, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x500513b0, SUC_lexicon_tags_37 },
    { 0xc90513b1, SUC_lexicon_tags_46 },
    { 0xfd0513b2, SUC_lexicon_tags_148 },
    { 0x9f8513b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa513be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa513c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c513c7, SUC_lexicon_tags_10 },
    { 0x8b6513c8, SUC_lexicon_tags_11 },
    { 0xdb8513c9, SUC_lexicon_tags_26 },
    { 0xb78513ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98513d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xde2513d2, SUC_lexicon_tags_9 },
    { 0xf68513d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764513d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710513e0, SUC_lexicon_tags_19 },
    { 0x342513e1, SUC_lexicon_tags_0 },
    { 0x3d2513e1, SUC_lexicon_tags_25 },
    { 0x3e0513e3, SUC_lexicon_tags_0 },
    { 0xdae513e4, SUC_lexicon_tags_2 },
    { 0xd20513e5, SUC_lexicon_tags_45 },
    { 0xd66513e3, SUC_lexicon_tags_7 },
    { 0x146513e7, SUC_lexicon_tags_37 },
    { 0x642513e8, SUC_lexicon_tags_12 },
    { 0x8a8513e4, SUC_lexicon_tags_15 },
    { 0xa4a513e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e513ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c513f1, SUC_lexicon_tags_17 },
    { 0x9c2513f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298513fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4251401, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26051403, SUC_lexicon_tags_11 },
    { 0x20451404, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6251407, SUC_lexicon_tags_26 },
    { 0xde051407, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x68a5140a, SUC_lexicon_tags_11 },
    { 0x1b05140b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2845141b, SUC_lexicon_tags_60 },
    { 0x3905141c, SUC_lexicon_tags_12 },
    { 0x3a05141b, SUC_lexicon_tags_45 },
    { 0x7825141d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e51423, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c51427, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c451429, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a05142d, SUC_lexicon_tags_7 },
    { 0x7ac5142e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28651432, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1851436, SUC_lexicon_tags_2 },
    { 0x9b051436, SUC_lexicon_tags_11 },
    { 0x8ec51438, SUC_lexicon_tags_2 },
    { 0x97051439, SUC_lexicon_tags_11 },
    { 0x74651439, SUC_lexicon_tags_66 },
    { 0x52051439, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6dc5143d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86651444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae51447, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45144b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb25144e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31851450, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a51453, SUC_lexicon_tags_11 },
    { 0x92c51453, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3565145a, SUC_lexicon_tags_37 },
    { 0x39c5145b, SUC_lexicon_tags_30 },
    { 0xe1c5145c, SUC_lexicon_tags_11 },
    { 0x1ca5145a, SUC_lexicon_tags_64 },
    { 0x55a5145e, SUC_lexicon_tags_37 },
    { 0x8745145f, SUC_lexicon_tags_31 },
    { 0xe025145f, SUC_lexicon_tags_37 },
    { 0xd1a51461, SUC_lexicon_tags_33 },
    { 0x7ec51462, SUC_lexicon_tags_33 },
    { 0x4b851462, SUC_lexicon_tags_25 },
    { 0x5ec51464, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x66651466, SUC_lexicon_tags_32 },
    { 0xd2851467, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3251469, SUC_lexicon_tags_37 },
    { 0x8aa5146a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32451473, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc251477, SUC_lexicon_tags_1 },
    { 0x18051477, SUC_lexicon_tags_38 },
    { 0x46451479, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2545147e, SUC_lexicon_tags_38 },
    { 0xef85147e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e51482, SUC_lexicon_tags_17 },
    { 0x3d251482, SUC_lexicon_tags_47 },
    { 0xfc851484, SUC_lexicon_tags_17 },
    { 0x29451485, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e5148a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5c25148c, SUC_lexicon_tags_29 },
    { 0x6265148d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6da5148f, SUC_lexicon_tags_14 },
    { 0x79851490, SUC_lexicon_tags_19 },
    { 0xed251491, SUC_lexicon_tags_39 },
    { 0x5de51492, SUC_lexicon_tags_17 },
    { 0x37251493, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48251498, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e5149d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4514a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x440514a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70514a5, SUC_lexicon_tags_1 },
    { 0x2c4514a5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2514ab, SUC_lexicon_tags_2 },
    { 0xe72514ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700514b0, SUC_lexicon_tags_60 },
    { 0xb6a514b1, SUC_lexicon_tags_32 },
    { 0xc92514b2, SUC_lexicon_tags_27 },
    { 0x640514b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c0514b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a514bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2514bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62c514c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28514c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894514c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xef4514cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c514d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6514de, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a514e2, SUC_lexicon_tags_38 },
    { 0x2b4514e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd0514e5, SUC_lexicon_tags_37 },
    { 0x484514e6, SUC_lexicon_tags_25 },
    { 0x336514e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e514ed, SUC_lexicon_tags_7 },
    { 0x1c514ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00514fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf66514fe, SUC_lexicon_tags_37 },
    { 0x616514ff, SUC_lexicon_tags_35 },
    { 0x50651500, SUC_lexicon_tags_60 },
    { 0x9da51500, SUC_lexicon_tags_23 },
    { 0xfb051501, SUC_lexicon_tags_85 },
    { 0xcb6514ff, SUC_lexicon_tags_1 },
    { 0x824514ff, SUC_lexicon_tags_0 },
    { 0x8fa51502, SUC_lexicon_tags_15 },
    { 0x3ca51505, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x30251508, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xde05150b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf965150d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0451513, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5c51515, SUC_lexicon_tags_15 },
    { 0x30a51516, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7451518, SUC_lexicon_tags_33 },
    { 0xa1851519, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c5151c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x15a5151e, SUC_lexicon_tags_33 },
    { 0xbcc5151f, SUC_lexicon_tags_15 },
    { 0xc4451520, SUC_lexicon_tags_11 },
    { 0x6451520, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf851523, SUC_lexicon_tags_1 },
    { 0xb1451523, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0xb051526, SUC_lexicon_tags_9 },
    { 0xd051527, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x65651529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e5152e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3651531, SUC_lexicon_tags_30 },
    { 0x58e51532, SUC_lexicon_tags_7 },
    { 0x87e51533, SUC_lexicon_tags_33 },
    { 0x8f651533, SUC_lexicon_tags_1 },
    { 0x53251535, SUC_lexicon_tags_12 },
    { 0xc851535, SUC_lexicon_tags_7 },
    { 0xa6a51534, SUC_lexicon_tags_47 },
    { 0x2b051537, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c5153b, SUC_lexicon_tags_2 },
    { 0xb8c5153c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c51542, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fa51544, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0451547, SUC_lexicon_tags_7 },
    { 0x80251548, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x365154b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a05154d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e25154f, SUC_lexicon_tags_63 },
    { 0xb705154f, SUC_lexicon_tags_2 },
    { 0x84c51551, SUC_lexicon_tags_26 },
    { 0xc1651552, SUC_lexicon_tags_28 },
    { 0x2d251551, SUC_lexicon_tags_15 },
    { 0xe3851554, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d251557, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeda51559, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc0a5155b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8fc5155d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b451562, SUC_lexicon_tags_72 },
    { 0x50a51562, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbba51565, SUC_lexicon_tags_14 },
    { 0x47451566, SUC_lexicon_tags_49 },
    { 0x15051566, SUC_lexicon_tags_33 },
    { 0xaae51568, SUC_lexicon_tags_1 },
    { 0x5a251566, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c45156f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc251573, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a51576, SUC_lexicon_tags_77 },
    { 0x3ce51577, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c5157e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x68451581, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf451588, SUC_lexicon_tags_31 },
    { 0x9e051588, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae85158d, SUC_lexicon_tags_45 },
    { 0xd7e5158d, SUC_lexicon_tags_19 },
    { 0x49a5158f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdb651591, SUC_lexicon_tags_14 },
    { 0x6f851591, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e65159b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82515a1, SUC_lexicon_tags_27 },
    { 0xeae515a1, SUC_lexicon_tags_8 },
    { 0x990515a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce515a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696515ac, SUC_lexicon_tags_11 },
    { 0xd5a515ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe515b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb6e515b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a515b9, SUC_lexicon_tags_1 },
    { 0x474515ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a515bd, SUC_lexicon_tags_151 },
    { 0xd54515bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d4515c0, SUC_lexicon_tags_7 },
    { 0x110515c0, SUC_lexicon_tags_15 },
    { 0x376515c2, SUC_lexicon_tags_33 },
    { 0x10c515c2, SUC_lexicon_tags_108 },
    { 0x662515c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfd6515c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x350515c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x422515ca, SUC_lexicon_tags_37 },
    { 0xc48515ca, SUC_lexicon_tags_60 },
    { 0xe58515cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c515d0, SUC_lexicon_tags_25 },
    { 0x7b2515d0, SUC_lexicon_tags_17 },
    { 0x158515d2, SUC_lexicon_tags_33 },
    { 0xf2a515d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242515d8, SUC_lexicon_tags_14 },
    { 0xc5e515d9, SUC_lexicon_tags_37 },
    { 0x940515da, SUC_lexicon_tags_33 },
    { 0x36c515db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe515e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5e0515e3, SUC_lexicon_tags_37 },
    { 0xe30515e4, SUC_lexicon_tags_17 },
    { 0x22515e5, SUC_lexicon_tags_14 },
    { 0xd98515e5, SUC_lexicon_tags_7 },
    { 0x254515e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac515ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x156515ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0515f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4515f9, SUC_lexicon_tags_63 },
    { 0x5c4515fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x740515fd, SUC_lexicon_tags_15 },
    { 0xb8c515fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd5251600, SUC_lexicon_tags_26 },
    { 0xeae51601, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa51604, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x71451606, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1785160e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a51611, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca51615, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e851617, SUC_lexicon_tags_2 },
    { 0x9ac51617, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d05161e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84851624, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea51627, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x77451629, SUC_lexicon_tags_40 },
    { 0xa9c51629, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec251630, SUC_lexicon_tags_68 },
    { 0x72e51631, SUC_lexicon_tags_12 },
    { 0x40851632, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe9e51634, SUC_lexicon_tags_19 },
    { 0xa5c51635, SUC_lexicon_tags_65 },
    { 0x1e051635, SUC_lexicon_tags_11 },
    { 0x17451637, SUC_lexicon_tags_33 },
    { 0xd4251638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc545163a, SUC_lexicon_tags_16 },
    { 0x1445163b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f251642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e51645, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a51649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c45164b, SUC_lexicon_tags_11 },
    { 0x2dc5164c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef851651, SUC_lexicon_tags_25 },
    { 0x9c051651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c51655, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x67451657, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2a51659, SUC_lexicon_tags_1 },
    { 0xb3c5165a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a25165e, SUC_lexicon_tags_31 },
    { 0xc025165e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc51663, SUC_lexicon_tags_63 },
    { 0x3e51664, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7c51666, SUC_lexicon_tags_15 },
    { 0xa1651666, SUC_lexicon_tags_7 },
    { 0x9b251666, SUC_lexicon_tags_53 },
    { 0x83a51669, SUC_lexicon_tags_15 },
    { 0x56251669, SUC_lexicon_tags_15 },
    { 0x1b051668, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4805166d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3651670, SUC_lexicon_tags_33 },
    { 0xd3651671, SUC_lexicon_tags_43 },
    { 0x6be51671, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e05167e, SUC_lexicon_tags_22 },
    { 0xeee5167f, SUC_lexicon_tags_19 },
    { 0xf6c5167f, SUC_lexicon_tags_11 },
    { 0x9825167e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xba651683, SUC_lexicon_tags_2 },
    { 0x1d051684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15651687, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x225168a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c051694, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac51698, SUC_lexicon_tags_3 },
    { 0x85a51698, SUC_lexicon_tags_25 },
    { 0x28051699, SUC_lexicon_tags_21 },
    { 0x9ce5169a, SUC_lexicon_tags_2 },
    { 0xf145169c, SUC_lexicon_tags_8 },
    { 0xc8e5169d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a516a5, SUC_lexicon_tags_1 },
    { 0x398516a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe8c516a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2516ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a516b1, SUC_lexicon_tags_79 },
    { 0xb6516b2, SUC_lexicon_tags_15 },
    { 0xfc516b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c516b6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a516bb, SUC_lexicon_tags_39 },
    { 0xda516bc, SUC_lexicon_tags_15 },
    { 0xf48516bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770516c9, SUC_lexicon_tags_37 },
    { 0x9c2516ca, SUC_lexicon_tags_27 },
    { 0x312516cb, SUC_lexicon_tags_10 },
    { 0x126516cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c516ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932516d8, SUC_lexicon_tags_10 },
    { 0x82516d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd66516db, SUC_lexicon_tags_14 },
    { 0x782516db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92516df, SUC_lexicon_tags_27 },
    { 0x920516e0, SUC_lexicon_tags_68 },
    { 0xd30516e1, SUC_lexicon_tags_12 },
    { 0x98a516e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2516e5, SUC_lexicon_tags_7 },
    { 0xebe516e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b6516e8, SUC_lexicon_tags_63 },
    { 0xc04516e9, SUC_lexicon_tags_19 },
    { 0x8c8516ea, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x924516ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde4516ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38516f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x350516f3, SUC_lexicon_tags_38 },
    { 0xde2516f4, SUC_lexicon_tags_1 },
    { 0xf54516f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a516fb, SUC_lexicon_tags_25 },
    { 0xde516fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a51702, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2851706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7251713, SUC_lexicon_tags_12 },
    { 0x63e51713, SUC_lexicon_tags_7 },
    { 0x65c51714, SUC_lexicon_tags_17 },
    { 0xb3651716, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefe51718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x705171a, SUC_lexicon_tags_6 },
    { 0x9a65171a, SUC_lexicon_tags_12 },
    { 0xaa45171a, SUC_lexicon_tags_43 },
    { 0xcbc5171c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef45171f, SUC_lexicon_tags_15 },
    { 0xb451720, SUC_lexicon_tags_12 },
    { 0xffc51721, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e51725, SUC_lexicon_tags_11 },
    { 0x1a651726, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e5172b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c51731, SUC_lexicon_tags_11 },
    { 0x64851731, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd051735, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xee51738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf85173f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc051744, SUC_lexicon_tags_47 },
    { 0x99851745, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xf1651747, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a5174a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66651750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cc51752, SUC_lexicon_tags_39 },
    { 0x1fe51753, SUC_lexicon_tags_15 },
    { 0x30c51754, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49451756, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8d051758, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf185176a, SUC_lexicon_tags_12 },
    { 0x2d65176b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac51774, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x77651776, SUC_lexicon_tags_13 },
    { 0xcec51777, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a5177c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9745177f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec51783, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2045178a, SUC_lexicon_tags_1 },
    { 0x6d05178a, SUC_lexicon_tags_66 },
    { 0x16e5178c, SUC_lexicon_tags_49 },
    { 0x1505178d, SUC_lexicon_tags_33 },
    { 0x7285178d, SUC_lexicon_tags_37 },
    { 0xa0c5178c, SUC_lexicon_tags_15 },
    { 0x8f65178b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca51793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa51796, SUC_lexicon_tags_26 },
    { 0x86c51797, SUC_lexicon_tags_61 },
    { 0xa0251798, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b85179a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40a5179c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec5179f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x364517a2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x606517a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82517ab, SUC_lexicon_tags_29 },
    { 0x114517ac, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690517b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x196517b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58517b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x190517b8, SUC_lexicon_tags_2 },
    { 0xc3c517b9, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x900517bb, SUC_lexicon_tags_3 },
    { 0xf9e517bc, SUC_lexicon_tags_38 },
    { 0x7ec517bd, SUC_lexicon_tags_5 },
    { 0x162517bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81c517c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb44517c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf04517c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x71c517c6, SUC_lexicon_tags_87 },
    { 0x5f2517c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a517ca, SUC_lexicon_tags_19 },
    { 0x374517cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c517d1, SUC_lexicon_tags_16 },
    { 0x284517d2, SUC_lexicon_tags_15 },
    { 0x15c517d3, SUC_lexicon_tags_11 },
    { 0x1e0517d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c2517d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808517da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf78517dc, SUC_lexicon_tags_11 },
    { 0x832517dd, SUC_lexicon_tags_17 },
    { 0x5cc517dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9ae517e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98517e3, SUC_lexicon_tags_8 },
    { 0x226517e4, SUC_lexicon_tags_15 },
    { 0xa24517e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3a517e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf50517e9, SUC_lexicon_tags_36 },
    { 0x94e517e9, SUC_lexicon_tags_3 },
    { 0x478517eb, SUC_lexicon_tags_14 },
    { 0xb5c517ec, SUC_lexicon_tags_275 },
    { 0x5a6517ed, SUC_lexicon_tags_87 },
    { 0x290517ec, SUC_lexicon_tags_8 },
    { 0x360517e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbc8517f1, SUC_lexicon_tags_17 },
    { 0x754517f2, SUC_lexicon_tags_12 },
    { 0x31c517f1, SUC_lexicon_tags_30 },
    { 0xa6c517f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0517f9, SUC_lexicon_tags_63 },
    { 0xdaa517fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6517fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c51800, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66251802, SUC_lexicon_tags_14 },
    { 0x83a51803, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba651808, SUC_lexicon_tags_12 },
    { 0x77451809, SUC_lexicon_tags_17 },
    { 0x1aa5180a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d25180d, SUC_lexicon_tags_33 },
    { 0x93e5180d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbda51810, SUC_lexicon_tags_15 },
    { 0x95a51811, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99251815, SUC_lexicon_tags_8 },
    { 0x91851816, SUC_lexicon_tags_0 },
    { 0xce051817, SUC_lexicon_tags_19 },
    { 0x51817, SUC_lexicon_tags_14 },
    { 0x70051818, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2845181b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1785181f, SUC_lexicon_tags_39 },
    { 0xce451820, SUC_lexicon_tags_0 },
    { 0x70c51820, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c51824, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da5182f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b051831, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82651833, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xbce51835, SUC_lexicon_tags_27 },
    { 0x37651836, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55051839, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0251841, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a51844, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf025184a, SUC_lexicon_tags_2 },
    { 0x23c5184a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc805184e, SUC_lexicon_tags_138 },
    { 0x18c5184e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd0a51851, SUC_lexicon_tags_2 },
    { 0xc6c51852, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8851855, SUC_lexicon_tags_26 },
    { 0xefc51856, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea5185b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be5185f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8a051861, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c51866, SUC_lexicon_tags_45 },
    { 0x7a51867, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x90a51869, SUC_lexicon_tags_17 },
    { 0x7e05186a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa45186c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91051872, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74051876, SUC_lexicon_tags_0 },
    { 0xd0e51877, SUC_lexicon_tags_37 },
    { 0x5651878, SUC_lexicon_tags_12 },
    { 0x3c251879, SUC_lexicon_tags_37 },
    { 0xc2c5187a, SUC_lexicon_tags_12 },
    { 0xa925187a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c5187e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x80051881, SUC_lexicon_tags_25 },
    { 0x3d651881, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c51889, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4885188b, SUC_lexicon_tags_37 },
    { 0xbba5188b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce05188e, SUC_lexicon_tags_11 },
    { 0x6c25188e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79451891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe51894, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17451896, SUC_lexicon_tags_17 },
    { 0xa6651897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20451899, SUC_lexicon_tags_23 },
    { 0xfc65189a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa25189e, SUC_lexicon_tags_12 },
    { 0x3f65189f, SUC_lexicon_tags_37 },
    { 0xeaa5189f, SUC_lexicon_tags_19 },
    { 0x8b8518a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec518a5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x5d0518a7, SUC_lexicon_tags_2 },
    { 0x8ae518a8, SUC_lexicon_tags_1 },
    { 0x1ac518a7, SUC_lexicon_tags_55 },
    { 0x2f4518aa, SUC_lexicon_tags_14 },
    { 0xabe518ab, SUC_lexicon_tags_2 },
    { 0x1f8518a7, SUC_lexicon_tags_30 },
    { 0xdf0518ac, SUC_lexicon_tags_23 },
    { 0x3d2518ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284518b3, SUC_lexicon_tags_2 },
    { 0x4f6518b4, SUC_lexicon_tags_63 },
    { 0x386518b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99e518b7, SUC_lexicon_tags_37 },
    { 0xdf0518b7, SUC_lexicon_tags_37 },
    { 0x552518b9, SUC_lexicon_tags_37 },
    { 0x42e518ba, SUC_lexicon_tags_47 },
    { 0x674518bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84518be, SUC_lexicon_tags_25 },
    { 0x998518bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a518c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2de518c5, SUC_lexicon_tags_0 },
    { 0x468518c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40518c9, SUC_lexicon_tags_14 },
    { 0xdbe518ca, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6518cf, SUC_lexicon_tags_37 },
    { 0xaf0518d0, SUC_lexicon_tags_33 },
    { 0x870518d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e518d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6518da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca518dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce518e2, SUC_lexicon_tags_25 },
    { 0x35c518e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99c518e5, SUC_lexicon_tags_25 },
    { 0xc10518e6, SUC_lexicon_tags_36 },
    { 0xfde518e7, SUC_lexicon_tags_26 },
    { 0x242518e6, SUC_lexicon_tags_15 },
    { 0x354518e9, SUC_lexicon_tags_12 },
    { 0xcaa518ea, SUC_lexicon_tags_23 },
    { 0x1b0518e8, SUC_lexicon_tags_18 },
    { 0x48518e7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6518ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46518f2, SUC_lexicon_tags_1 },
    { 0x10c518f3, SUC_lexicon_tags_34 },
    { 0x802518f3, SUC_lexicon_tags_12 },
    { 0x5ee518f2, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc518fb, SUC_lexicon_tags_3 },
    { 0xb3a518fc, SUC_lexicon_tags_60 },
    { 0xa26518fb, SUC_lexicon_tags_26 },
    { 0x2ec518fe, SUC_lexicon_tags_82 },
    { 0x3b0518ff, SUC_lexicon_tags_19 },
    { 0xfbc51900, SUC_lexicon_tags_47 },
    { 0xbae518fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61051903, SUC_lexicon_tags_2 },
    { 0x8ba51904, SUC_lexicon_tags_32 },
    { 0x35651904, SUC_lexicon_tags_32 },
    { 0x20451905, SUC_lexicon_tags_14 },
    { 0x41251906, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd165190b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e5190e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc251911, SUC_lexicon_tags_17 },
    { 0x3b051912, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a51915, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f45191b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4651920, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58c51922, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa051924, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4251928, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e65192a, SUC_lexicon_tags_37 },
    { 0x1e65192b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1245192e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde651931, SUC_lexicon_tags_9 },
    { 0xf8e51932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16651935, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29451939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcda5193b, SUC_lexicon_tags_9 },
    { 0xfac5193c, SUC_lexicon_tags_11 },
    { 0xdba5193d, SUC_lexicon_tags_12 },
    { 0xcee5193c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0451940, SUC_lexicon_tags_38 },
    { 0x6fa51940, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc51946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba5194d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9065194f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c451956, SUC_lexicon_tags_17 },
    { 0x6fa51957, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e5195a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece5195e, SUC_lexicon_tags_23 },
    { 0xf165195e, SUC_lexicon_tags_15 },
    { 0xdfc51960, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34c51962, SUC_lexicon_tags_33 },
    { 0x59a51963, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe4651965, SUC_lexicon_tags_0 },
    { 0x11e51966, SUC_lexicon_tags_40 },
    { 0xfa851967, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9865196d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c51970, SUC_lexicon_tags_47 },
    { 0x4e651971, SUC_lexicon_tags_37 },
    { 0x24051972, SUC_lexicon_tags_15 },
    { 0xcb451970, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba51977, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c5197e, SUC_lexicon_tags_29 },
    { 0xff85197f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17051981, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1251983, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d451988, SUC_lexicon_tags_2 },
    { 0xcd851989, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1685198c, SUC_lexicon_tags_17 },
    { 0x3f65198d, SUC_lexicon_tags_45 },
    { 0x1aa5198e, SUC_lexicon_tags_33 },
    { 0x6965198d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a251996, SUC_lexicon_tags_25 },
    { 0xe4e51997, SUC_lexicon_tags_55 },
    { 0x90451997, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb365199d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34519a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4519a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a4519a6, SUC_lexicon_tags_1 },
    { 0xc60519a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94519ab, SUC_lexicon_tags_4 },
    { 0xef2519ab, SUC_lexicon_tags_17 },
    { 0x29e519ad, SUC_lexicon_tags_160 },
    { 0xa74519ac, SUC_lexicon_tags_14 },
    { 0x2f6519ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32519b3, SUC_lexicon_tags_1 },
    { 0x114519b4, SUC_lexicon_tags_19 },
    { 0xbfe519b3, SUC_lexicon_tags_0 },
    { 0xdea519b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98519b9, SUC_lexicon_tags_2 },
    { 0xb22519ba, SUC_lexicon_tags_1 },
    { 0xc58519bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0519c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c519c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x646519cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22519ce, SUC_lexicon_tags_33 },
    { 0x6a2519cf, SUC_lexicon_tags_230 },
    { 0x2fe519ce, SUC_lexicon_tags_15 },
    { 0x526519cf, SUC_lexicon_tags_37 },
    { 0x142519d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6519d5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44519d8, SUC_lexicon_tags_37 },
    { 0x86a519d9, SUC_lexicon_tags_0 },
    { 0xa22519d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194519e2, SUC_lexicon_tags_48 },
    { 0x3fc519e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc02519e5, SUC_lexicon_tags_84 },
    { 0x288519e6, SUC_lexicon_tags_17 },
    { 0x91e519e6, SUC_lexicon_tags_33 },
    { 0x18519e7, SUC_lexicon_tags_17 },
    { 0xe74519e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f2519eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a519f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaee519f3, SUC_lexicon_tags_37 },
    { 0xb52519f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4519f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9e519f9, SUC_lexicon_tags_37 },
    { 0xf12519fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858519fe, SUC_lexicon_tags_27 },
    { 0x80c519ff, SUC_lexicon_tags_7 },
    { 0xb3651a00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23651a05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73251a0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e051a11, SUC_lexicon_tags_48 },
    { 0xe3251a12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee51a18, SUC_lexicon_tags_17 },
    { 0x6f251a19, SUC_lexicon_tags_44 },
    { 0x22c51a19, SUC_lexicon_tags_49 },
    { 0x56251a19, SUC_lexicon_tags_201 },
    { 0x52251a19, SUC_lexicon_tags_1 },
    { 0xfb451a1d, SUC_lexicon_tags_21 },
    { 0x8f051a18, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32851a22, SUC_lexicon_tags_7 },
    { 0xdf651a23, SUC_lexicon_tags_17 },
    { 0xb9c51a23, SUC_lexicon_tags_65 },
    { 0x35651a25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf651a2f, SUC_lexicon_tags_2 },
    { 0x54a51a2f, SUC_lexicon_tags_2 },
    { 0xad851a31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcce51a33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0e51a35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c51a38, SUC_lexicon_tags_32 },
    { 0x43c51a38, SUC_lexicon_tags_63 },
    { 0xf0e51a3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98451a40, SUC_lexicon_tags_45 },
    { 0x35e51a40, SUC_lexicon_tags_2 },
    { 0xa251a42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a51a45, SUC_lexicon_tags_8 },
    { 0x251a46, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12851a4c, SUC_lexicon_tags_87 },
    { 0xdda51a4c, SUC_lexicon_tags_39 },
    { 0xe9c51a4e, SUC_lexicon_tags_12 },
    { 0xc5851a4f, SUC_lexicon_tags_15 },
    { 0xacc51a4d, SUC_lexicon_tags_47 },
    { 0x67451a50, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d051a59, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa51a5c, SUC_lexicon_tags_28 },
    { 0x31c51a5d, SUC_lexicon_tags_64 },
    { 0x5b251a5e, SUC_lexicon_tags_4 },
    { 0x87e51a5d, SUC_lexicon_tags_17 },
    { 0x71251a5d, SUC_lexicon_tags_7 },
    { 0xe7851a61, SUC_lexicon_tags_17 },
    { 0x11651a62, SUC_lexicon_tags_17 },
    { 0xf9651a62, SUC_lexicon_tags_1 },
    { 0x3b451a5f, SUC_lexicon_tags_68 },
    { 0x7d651a5f, SUC_lexicon_tags_26 },
    { 0x7e451a66, SUC_lexicon_tags_38 },
    { 0x94651a66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de51a6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc51a70, SUC_lexicon_tags_27 },
    { 0xc7a51a71, SUC_lexicon_tags_82 },
    { 0x6c851a72, SUC_lexicon_tags_25 },
    { 0xb8051a73, SUC_lexicon_tags_7 },
    { 0xdd451a71, SUC_lexicon_tags_0 },
    { 0x5a51a75, SUC_lexicon_tags_9 },
    { 0xafa51a71, SUC_lexicon_tags_37 },
    { 0x5d851a76, SUC_lexicon_tags_26 },
    { 0x94451a75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251a7d, SUC_lexicon_tags_1 },
    { 0x19a51a7e, SUC_lexicon_tags_7 },
    { 0x3b651a7e, SUC_lexicon_tags_15 },
    { 0x1ca51a7f, SUC_lexicon_tags_15 },
    { 0xefe51a81, SUC_lexicon_tags_37 },
    { 0x5e851a7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9651a84, SUC_lexicon_tags_2 },
    { 0x3a51a85, SUC_lexicon_tags_60 },
    { 0x84651a86, SUC_lexicon_tags_7 },
    { 0x97e51a87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e051a8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb851a92, SUC_lexicon_tags_6 },
    { 0x19451a92, SUC_lexicon_tags_23 },
    { 0x5a851a94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a51a99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5be51a9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbda51a9d, SUC_lexicon_tags_28 },
    { 0x8ea51a9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x651aa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e51aa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18051aa9, SUC_lexicon_tags_29 },
    { 0xcc051aaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c51aaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0051ab1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x74051ab3, SUC_lexicon_tags_47 },
    { 0x7c651ab4, SUC_lexicon_tags_15 },
    { 0xddc51ab3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeb451ab7, SUC_lexicon_tags_7 },
    { 0x3d651ab7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xec851aba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11051ac3, SUC_lexicon_tags_38 },
    { 0xa0651ac4, SUC_lexicon_tags_14 },
    { 0x8e851ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c451ac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10051ac9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdec51acb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8651acd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e051ad0, SUC_lexicon_tags_38 },
    { 0x8ba51ad0, SUC_lexicon_tags_17 },
    { 0xbd651ad2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86851ad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f051ad8, SUC_lexicon_tags_23 },
    { 0xd1451ad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c851adc, SUC_lexicon_tags_31 },
    { 0x59251add, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd3451adf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3051ae1, SUC_lexicon_tags_7 },
    { 0xa1e51ae2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x45451ae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48e51ae7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9251aee, SUC_lexicon_tags_26 },
    { 0x90e51aef, SUC_lexicon_tags_49 },
    { 0x22251af0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xda51af3, SUC_lexicon_tags_7 },
    { 0x52a51af3, SUC_lexicon_tags_47 },
    { 0xe9651af3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xea451af7, SUC_lexicon_tags_37 },
    { 0xd3e51af7, SUC_lexicon_tags_26 },
    { 0x55651af9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a51afc, SUC_lexicon_tags_17 },
    { 0xf0e51afd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7051b03, SUC_lexicon_tags_2 },
    { 0xe1451b04, SUC_lexicon_tags_25 },
    { 0xcc851b05, SUC_lexicon_tags_49 },
    { 0x1851b04, SUC_lexicon_tags_17 },
    { 0x26c51b07, SUC_lexicon_tags_60 },
    { 0x60e51b05, SUC_lexicon_tags_38 },
    { 0x80251b09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f051b0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda651b0e, SUC_lexicon_tags_87 },
    { 0xc3a51b0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e51b12, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7451b15, SUC_lexicon_tags_27 },
    { 0x4b051b16, SUC_lexicon_tags_37 },
    { 0x36e51b17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a51b1c, SUC_lexicon_tags_60 },
    { 0xfce51b1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecc51b1f, SUC_lexicon_tags_7 },
    { 0xbc251b20, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2851b28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c651b2b, SUC_lexicon_tags_14 },
    { 0x89651b2c, SUC_lexicon_tags_1 },
    { 0x47851b2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e51b36, SUC_lexicon_tags_9 },
    { 0x53a51b37, SUC_lexicon_tags_6 },
    { 0x5251b37, SUC_lexicon_tags_25 },
    { 0x25e51b36, SUC_lexicon_tags_7 },
    { 0x7b251b3a, SUC_lexicon_tags_47 },
    { 0x68a51b3a, SUC_lexicon_tags_15 },
    { 0xeea51b38, SUC_lexicon_tags_2 },
    { 0xa4451b3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfe51b3f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9651b43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe51b46, SUC_lexicon_tags_16 },
    { 0x31651b47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c51b4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4051b4f, SUC_lexicon_tags_1 },
    { 0xe1c51b4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa51b54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaa451b56, SUC_lexicon_tags_10 },
    { 0x8f051b57, SUC_lexicon_tags_11 },
    { 0x23851b56, SUC_lexicon_tags_31 },
    { 0xcc651b58, SUC_lexicon_tags_11 },
    { 0xed851b5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be51b5e, SUC_lexicon_tags_45 },
    { 0x95e51b5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef251b61, SUC_lexicon_tags_46 },
    { 0x71451b62, SUC_lexicon_tags_458 },
    { 0x8e651b62, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84251b67, SUC_lexicon_tags_14 },
    { 0x451b68, SUC_lexicon_tags_87 },
    { 0xc6051b69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60851b6d, SUC_lexicon_tags_12 },
    { 0xb3051b6e, SUC_lexicon_tags_17 },
    { 0x6f251b6f, SUC_lexicon_tags_6 },
    { 0xba251b6e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73251b78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0251b7c, SUC_lexicon_tags_29 },
    { 0x27251b7c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c851b7f, SUC_lexicon_tags_68 },
    { 0xe1051b80, SUC_lexicon_tags_25 },
    { 0x60a51b81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e51b84, SUC_lexicon_tags_37 },
    { 0xd8451b84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be51b88, SUC_lexicon_tags_47 },
    { 0x1ea51b89, SUC_lexicon_tags_14 },
    { 0x98c51b8a, SUC_lexicon_tags_55 },
    { 0x9a651b8b, SUC_lexicon_tags_12 },
    { 0xa1051b89, SUC_lexicon_tags_63 },
    { 0x6fc51b8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa51b90, SUC_lexicon_tags_47 },
    { 0x8d851b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8851b95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf0c51b97, SUC_lexicon_tags_15 },
    { 0x82a51b98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be51b9f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9451ba2, SUC_lexicon_tags_39 },
    { 0xbca51ba2, SUC_lexicon_tags_7 },
    { 0x21e51ba4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46251ba6, SUC_lexicon_tags_12 },
    { 0x23a51ba7, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a51baa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c51bae, SUC_lexicon_tags_19 },
    { 0xe0651baf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac51bb3, SUC_lexicon_tags_25 },
    { 0xc9651bb3, SUC_lexicon_tags_1 },
    { 0xcbe51bb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b851bbb, SUC_lexicon_tags_11 },
    { 0x74051bbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81451bbe, SUC_lexicon_tags_12 },
    { 0xa5051bbe, SUC_lexicon_tags_38 },
    { 0x6e651bc0, SUC_lexicon_tags_8 },
    { 0xd6251bc1, SUC_lexicon_tags_33 },
    { 0xa1251bc1, SUC_lexicon_tags_47 },
    { 0x87c51bc3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e251bc6, SUC_lexicon_tags_1306 },
    { 0x36651bc7, SUC_lexicon_tags_37 },
    { 0xea651bc7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d851bce, SUC_lexicon_tags_29 },
    { 0x87451bcf, SUC_lexicon_tags_17 },
    { 0x81051bd0, SUC_lexicon_tags_1 },
    { 0x1d651bd0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4251bd5, SUC_lexicon_tags_17 },
    { 0x5dc51bd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x95251bd9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x43c51bdb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9851be0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53c51be2, SUC_lexicon_tags_10 },
    { 0xe8051be2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6a451be5, SUC_lexicon_tags_1 },
    { 0x44c51be5, SUC_lexicon_tags_25 },
    { 0xfea51be7, SUC_lexicon_tags_17 },
    { 0xb5851be8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc3651bea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69e51bec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8c651bee, SUC_lexicon_tags_15 },
    { 0x63251bef, SUC_lexicon_tags_38 },
    { 0xd0251bef, SUC_lexicon_tags_15 },
    { 0xc7651bef, SUC_lexicon_tags_8 },
    { 0xb6a51bf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a451bf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa51bf6, SUC_lexicon_tags_45 },
    { 0xf4651bf7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xac251bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c51bfc, SUC_lexicon_tags_49 },
    { 0x2b251bfd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9851c00, SUC_lexicon_tags_14 },
    { 0x66851c01, SUC_lexicon_tags_32 },
    { 0xd5c51c00, SUC_lexicon_tags_15 },
    { 0x30c51c03, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4251c06, SUC_lexicon_tags_63 },
    { 0xdb851c07, SUC_lexicon_tags_37 },
    { 0xcfe51c08, SUC_lexicon_tags_26 },
    { 0x89a51c09, SUC_lexicon_tags_15 },
    { 0x46851c0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4451c0c, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x29051c0e, SUC_lexicon_tags_9 },
    { 0xb0651c0e, SUC_lexicon_tags_17 },
    { 0x7a051c10, SUC_lexicon_tags_6 },
    { 0x73c51c11, SUC_lexicon_tags_15 },
    { 0xd0e51c12, SUC_lexicon_tags_14 },
    { 0xbf251c13, SUC_lexicon_tags_77 },
    { 0xc4a51c14, SUC_lexicon_tags_0 },
    { 0xd0251c14, SUC_lexicon_tags_37 },
    { 0xedc51c16, SUC_lexicon_tags_55 },
    { 0xcc651c17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e51c1c, SUC_lexicon_tags_23 },
    { 0x84651c1d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87051c24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99051c26, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e51c2c, SUC_lexicon_tags_1 },
    { 0x97851c2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3651c2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3851c32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a51c38, SUC_lexicon_tags_1 },
    { 0xc2851c38, SUC_lexicon_tags_1 },
    { 0x14051c38, SUC_lexicon_tags_15 },
    { 0x7da51c3b, SUC_lexicon_tags_60 },
    { 0xdf251c3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d251c3f, SUC_lexicon_tags_28 },
    { 0x60651c40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6251c42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22a51c44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a51c47, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5051c4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda451c4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e51c52, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac51c58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3651c60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99251c62, SUC_lexicon_tags_27 },
    { 0x84851c62, SUC_lexicon_tags_26 },
    { 0xcee51c64, SUC_lexicon_tags_16 },
    { 0x19851c65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c651c6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd8251c6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86451c6e, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xef851c71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xabc51c73, SUC_lexicon_tags_49 },
    { 0x59851c74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee051c76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e851c79, SUC_lexicon_tags_68 },
    { 0x88e51c7a, SUC_lexicon_tags_33 },
    { 0xb4251c79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c51c7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b651c82, SUC_lexicon_tags_11 },
    { 0x24851c83, SUC_lexicon_tags_7 },
    { 0xea451c84, SUC_lexicon_tags_1 },
    { 0x66851c84, SUC_lexicon_tags_11 },
    { 0x72451c86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2d051c88, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xda251c8a, SUC_lexicon_tags_17 },
    { 0xae651c8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a51c8e, SUC_lexicon_tags_17 },
    { 0x20c51c8f, SUC_lexicon_tags_28 },
    { 0x48451c90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c851c94, SUC_lexicon_tags_30 },
    { 0xfae51c95, SUC_lexicon_tags_60 },
    { 0x14251c96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6051c99, SUC_lexicon_tags_6 },
    { 0x22251c99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d251c9c, SUC_lexicon_tags_15 },
    { 0x8851c9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9451ca1, SUC_lexicon_tags_37 },
    { 0xb1651ca1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc8e51ca4, SUC_lexicon_tags_0 },
    { 0x42451ca5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57451cac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed851cb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96851cb3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3851cb5, SUC_lexicon_tags_45 },
    { 0xef651cb5, SUC_lexicon_tags_8 },
    { 0x2f051cb7, SUC_lexicon_tags_27 },
    { 0x87651cb7, SUC_lexicon_tags_6 },
    { 0xfea51cb7, SUC_lexicon_tags_11 },
    { 0x83051cb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78451cbc, SUC_lexicon_tags_25 },
    { 0x81c51cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f051cbf, SUC_lexicon_tags_12 },
    { 0xa8851cc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed451cc3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6d251cc5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36051cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b051cce, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x18e51cd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47651cd6, SUC_lexicon_tags_14 },
    { 0xaf851cd7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64851cdc, SUC_lexicon_tags_1 },
    { 0x47e51cdd, SUC_lexicon_tags_7 },
    { 0x48651cdd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2251ce1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66251ce5, SUC_lexicon_tags_7 },
    { 0xdfa51ce6, SUC_lexicon_tags_12 },
    { 0x83e51ce6, SUC_lexicon_tags_7 },
    { 0x5851ce8, SUC_lexicon_tags_11 },
    { 0xc6a51ce9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a51cec, SUC_lexicon_tags_17 },
    { 0x32c51ced, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac051cf4, SUC_lexicon_tags_17 },
    { 0x92251cf5, SUC_lexicon_tags_14 },
    { 0x4a451cf6, SUC_lexicon_tags_37 },
    { 0xcc651cf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c251cfd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e051d00, SUC_lexicon_tags_2 },
    { 0xc7051d01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a51d05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x21451d07, SUC_lexicon_tags_19 },
    { 0x4ec51d08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84051d0e, SUC_lexicon_tags_39 },
    { 0xce651d0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81051d11, SUC_lexicon_tags_7 },
    { 0xf0c51d12, SUC_lexicon_tags_101 },
    { 0x62e51d13, SUC_lexicon_tags_61 },
    { 0xc7851d11, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c651d17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6851d1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba51d1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a51d22, SUC_lexicon_tags_23 },
    { 0x79251d23, SUC_lexicon_tags_27 },
    { 0x3c651d24, SUC_lexicon_tags_37 },
    { 0x15451d23, SUC_lexicon_tags_26 },
    { 0xda451d26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b251d28, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xee651d2a, SUC_lexicon_tags_14 },
    { 0xfee51d2b, SUC_lexicon_tags_87 },
    { 0x4ca51d2a, SUC_lexicon_tags_8 },
    { 0x83c51d2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea51d32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f051d36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea51d3c, SUC_lexicon_tags_65 },
    { 0x6e451d3c, SUC_lexicon_tags_37 },
    { 0x8fc51d3e, SUC_lexicon_tags_27 },
    { 0xcc651d3f, SUC_lexicon_tags_17 },
    { 0xd2a51d40, SUC_lexicon_tags_28 },
    { 0x49a51d41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c51d45, SUC_lexicon_tags_39 },
    { 0x92851d46, SUC_lexicon_tags_2 },
    { 0x22a51d47, SUC_lexicon_tags_23 },
    { 0xca651d45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb051d4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca51d4e, SUC_lexicon_tags_2 },
    { 0x37051d4f, SUC_lexicon_tags_60 },
    { 0xe5a51d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a51d53, SUC_lexicon_tags_16 },
    { 0x16651d54, SUC_lexicon_tags_60 },
    { 0xd5e51d54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1451d59, SUC_lexicon_tags_37 },
    { 0x2dc51d5a, SUC_lexicon_tags_7 },
    { 0xe1651d5b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f851d5e, SUC_lexicon_tags_65 },
    { 0x75c51d5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32451d63, SUC_lexicon_tags_47 },
    { 0x2ec51d64, SUC_lexicon_tags_43 },
    { 0x5ac51d63, SUC_lexicon_tags_2 },
    { 0xff651d66, SUC_lexicon_tags_1 },
    { 0xe0a51d67, SUC_lexicon_tags_33 },
    { 0x31251d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9651d6a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e251d70, SUC_lexicon_tags_2 },
    { 0x4a051d70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e51d74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a251d77, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x62051d7a, SUC_lexicon_tags_15 },
    { 0x19c51d7b, SUC_lexicon_tags_14 },
    { 0x7a51d7b, SUC_lexicon_tags_12 },
    { 0x35a51d7d, SUC_lexicon_tags_183 },
    { 0xa8e51d7d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x64051d81, SUC_lexicon_tags_7 },
    { 0xcae51d82, SUC_lexicon_tags_26 },
    { 0xda451d82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x42c51d85, SUC_lexicon_tags_65 },
    { 0x6fa51d86, SUC_lexicon_tags_12 },
    { 0x86251d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0651d89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc51d91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xed851d93, SUC_lexicon_tags_27 },
    { 0x9dc51d94, SUC_lexicon_tags_28 },
    { 0xe1851d95, SUC_lexicon_tags_194 },
    { 0x95251d94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46c51d98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be51d9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a51d9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc251da1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd651daa, SUC_lexicon_tags_12 },
    { 0xfc451dab, SUC_lexicon_tags_25 },
    { 0xc8051dab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84451daf, SUC_lexicon_tags_46 },
    { 0x3051db0, SUC_lexicon_tags_60 },
    { 0xa8a51db0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e51db5, SUC_lexicon_tags_37 },
    { 0xe0051db6, SUC_lexicon_tags_16 },
    { 0x6a251db7, SUC_lexicon_tags_37 },
    { 0xc1a51db7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa451dbe, SUC_lexicon_tags_46 },
    { 0xa3651dbf, SUC_lexicon_tags_66 },
    { 0x9c451dc0, SUC_lexicon_tags_47 },
    { 0x3051dc1, SUC_lexicon_tags_37 },
    { 0x79051dc2, SUC_lexicon_tags_7 },
    { 0x30c51dc3, SUC_lexicon_tags_82 },
    { 0x41651dc4, SUC_lexicon_tags_7 },
    { 0x16e51dc5, SUC_lexicon_tags_27 },
    { 0x25a51dc1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad051dcd, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2051dd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c251dd5, SUC_lexicon_tags_33 },
    { 0x99051dd6, SUC_lexicon_tags_26 },
    { 0x6dc51dd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4051ddb, SUC_lexicon_tags_25 },
    { 0x3aa51ddc, SUC_lexicon_tags_33 },
    { 0xade51ddc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0451de2, SUC_lexicon_tags_6 },
    { 0xb6c51de2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99c51de5, SUC_lexicon_tags_7 },
    { 0x55e51de6, SUC_lexicon_tags_37 },
    { 0x7be51de6, SUC_lexicon_tags_45 },
    { 0x45451de7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0c51dea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86651df8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7451dfb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a851e00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c51e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d651e07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b451e0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf2251e0d, SUC_lexicon_tags_25 },
    { 0x28651e0e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e51e13, SUC_lexicon_tags_7 },
    { 0x56451e14, SUC_lexicon_tags_25 },
    { 0xd8051e13, SUC_lexicon_tags_12 },
    { 0x27851e15, SUC_lexicon_tags_9 },
    { 0x6f451e17, SUC_lexicon_tags_7 },
    { 0x8b651e18, SUC_lexicon_tags_30 },
    { 0x44451e17, SUC_lexicon_tags_38 },
    { 0x66451e14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0651e1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d851e24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7851e27, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb651e2e, SUC_lexicon_tags_57 },
    { 0x45251e2f, SUC_lexicon_tags_10 },
    { 0x3c51e2f, SUC_lexicon_tags_7 },
    { 0x15e51e31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98051e33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a51e36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd251e3c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8651e3f, SUC_lexicon_tags_49 },
    { 0xc2c51e3f, SUC_lexicon_tags_17 },
    { 0xfce51e41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a51e4c, SUC_lexicon_tags_47 },
    { 0x2b851e4c, SUC_lexicon_tags_120 },
    { 0xbc51e4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdee51e50, SUC_lexicon_tags_63 },
    { 0x8ea51e51, SUC_lexicon_tags_7 },
    { 0x14c51e50, SUC_lexicon_tags_12 },
    { 0xed051e50, SUC_lexicon_tags_12 },
    { 0x8c251e54, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c51e5a, SUC_lexicon_tags_15 },
    { 0x21e51e5b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e51e61, SUC_lexicon_tags_2 },
    { 0x13251e62, SUC_lexicon_tags_2 },
    { 0x7c51e62, SUC_lexicon_tags_15 },
    { 0x7f651e62, SUC_lexicon_tags_124 },
    { 0x2d051e65, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x41651e68, SUC_lexicon_tags_19 },
    { 0xcc51e69, SUC_lexicon_tags_7 },
    { 0x3d651e68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96251e6e, SUC_lexicon_tags_30 },
    { 0x4be51e6f, SUC_lexicon_tags_15 },
    { 0xf1a51e6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34651e75, SUC_lexicon_tags_12 },
    { 0x61e51e76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a51e7c, SUC_lexicon_tags_31 },
    { 0xeac51e7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6051e81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69651e84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16651e8d, SUC_lexicon_tags_54 },
    { 0x49651e8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x59451e91, SUC_lexicon_tags_57 },
    { 0x20851e92, SUC_lexicon_tags_45 },
    { 0xdbe51e91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a51e98, SUC_lexicon_tags_11 },
    { 0x67c51e98, SUC_lexicon_tags_3 },
    { 0xc7651e9a, SUC_lexicon_tags_30 },
    { 0x2fc51e9a, SUC_lexicon_tags_9 },
    { 0x27851e9c, SUC_lexicon_tags_12 },
    { 0x3fc51e9d, SUC_lexicon_tags_11 },
    { 0xb8451e9d, SUC_lexicon_tags_12 },
    { 0x75851e9b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a51ea3, SUC_lexicon_tags_28 },
    { 0x1ec51ea3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2651ea6, SUC_lexicon_tags_12 },
    { 0x6ec51ea7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e51eab, SUC_lexicon_tags_37 },
    { 0x48651eab, SUC_lexicon_tags_1 },
    { 0x4f851eac, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xcd451eaf, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xf6e51eb1, SUC_lexicon_tags_15 },
    { 0x7cc51eb2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9ba51eb4, SUC_lexicon_tags_11 },
    { 0x68251eb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad251eb7, SUC_lexicon_tags_26 },
    { 0x9c651eb8, SUC_lexicon_tags_17 },
    { 0x70051eb9, SUC_lexicon_tags_37 },
    { 0xd651eba, SUC_lexicon_tags_43 },
    { 0x68a51eba, SUC_lexicon_tags_12 },
    { 0x73a51ebc, SUC_lexicon_tags_7 },
    { 0xbae51ebb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45451ebf, SUC_lexicon_tags_43 },
    { 0x86651ec0, SUC_lexicon_tags_43 },
    { 0xa4651ec1, SUC_lexicon_tags_11 },
    { 0xa8051ec2, SUC_lexicon_tags_37 },
    { 0xc4451ec3, SUC_lexicon_tags_25 },
    { 0x2ce51ec4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d451ec7, SUC_lexicon_tags_47 },
    { 0xb4851ec8, SUC_lexicon_tags_25 },
    { 0xf0651ec9, SUC_lexicon_tags_21 },
    { 0xc451ec8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f251ece, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e51ed1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf4051ed3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c451ed7, SUC_lexicon_tags_37 },
    { 0x97051ed8, SUC_lexicon_tags_2 },
    { 0x6c451ed9, SUC_lexicon_tags_12 },
    { 0x95851ed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb851edc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5251ede, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0851ee0, SUC_lexicon_tags_28 },
    { 0x18051ee1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc851ee3, SUC_lexicon_tags_1 },
    { 0x28651ee4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8851eea, SUC_lexicon_tags_63 },
    { 0xd4651eeb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x15c51eed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94651ef7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xef251efa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c051efe, SUC_lexicon_tags_43 },
    { 0x35851eff, SUC_lexicon_tags_47 },
    { 0x84a51f00, SUC_lexicon_tags_9 },
    { 0xa1851f01, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e51f08, SUC_lexicon_tags_25 },
    { 0x34651f09, SUC_lexicon_tags_11 },
    { 0xe9251f09, SUC_lexicon_tags_11 },
    { 0xa1c51f0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeba51f0d, SUC_lexicon_tags_39 },
    { 0xe851f0e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x30651f11, SUC_lexicon_tags_11 },
    { 0xdcc51f12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24251f18, SUC_lexicon_tags_8 },
    { 0xec251f18, SUC_lexicon_tags_33 },
    { 0xc5651f1a, SUC_lexicon_tags_0 },
    { 0xeec51f1b, SUC_lexicon_tags_87 },
    { 0x64451f18, SUC_lexicon_tags_37 },
    { 0x1d651f1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9851f20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d051f26, SUC_lexicon_tags_7 },
    { 0x3f451f27, SUC_lexicon_tags_2 },
    { 0xb7e51f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e51f2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef051f2f, SUC_lexicon_tags_25 },
    { 0x7ee51f30, SUC_lexicon_tags_9 },
    { 0xe5651f2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x17651f33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc51f36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x58a51f38, SUC_lexicon_tags_2 },
    { 0x9d451f39, SUC_lexicon_tags_9 },
    { 0xb0a51f39, SUC_lexicon_tags_19 },
    { 0x6b251f3a, SUC_lexicon_tags_12 },
    { 0x92051f38, SUC_lexicon_tags_40 },
    { 0xe1451f3d, SUC_lexicon_tags_56 },
    { 0x5a51f3e, SUC_lexicon_tags_23 },
    { 0xbd051f3f, SUC_lexicon_tags_32 },
    { 0x46e51f3e, SUC_lexicon_tags_25 },
    { 0xece51f41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc51f45, SUC_lexicon_tags_66 },
    { 0x66051f45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a51f4b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3e051f4d, SUC_lexicon_tags_53 },
    { 0xa2c51f4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7651f56, SUC_lexicon_tags_45 },
    { 0x77c51f57, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca51f5e, SUC_lexicon_tags_11 },
    { 0xa7251f5f, SUC_lexicon_tags_63 },
    { 0x68251f5f, SUC_lexicon_tags_47 },
    { 0x28451f61, SUC_lexicon_tags_63 },
    { 0xc3651f61, SUC_lexicon_tags_10 },
    { 0xe0051f63, SUC_lexicon_tags_15 },
    { 0x78e51f62, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x59c51f66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34651f6c, SUC_lexicon_tags_37 },
    { 0x16051f6d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x73251f70, SUC_lexicon_tags_31 },
    { 0x16851f70, SUC_lexicon_tags_37 },
    { 0xb1651f71, SUC_lexicon_tags_43 },
    { 0x6ee51f73, SUC_lexicon_tags_15 },
    { 0xee651f74, SUC_lexicon_tags_11 },
    { 0x10451f75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x16251f78, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0251f7d, SUC_lexicon_tags_13 },
    { 0x79e51f7d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c51f87, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a51f8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba51f93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5251f99, SUC_lexicon_tags_14 },
    { 0xb6251f99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3aa51f9c, SUC_lexicon_tags_85 },
    { 0x6e451f9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72651fa5, SUC_lexicon_tags_6 },
    { 0xdbc51fa6, SUC_lexicon_tags_9 },
    { 0xddc51fa7, SUC_lexicon_tags_0 },
    { 0xf1e51fa5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x77851faa, SUC_lexicon_tags_37 },
    { 0x3851fab, SUC_lexicon_tags_55 },
    { 0x63a51fac, SUC_lexicon_tags_33 },
    { 0x62851fad, SUC_lexicon_tags_5 },
    { 0xc2051fae, SUC_lexicon_tags_43 },
    { 0xbc51faf, SUC_lexicon_tags_34 },
    { 0x1a251faf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a51fb7, SUC_lexicon_tags_0 },
    { 0xc5051fb8, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19651fbc, SUC_lexicon_tags_55 },
    { 0x96651fbd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c51fc1, SUC_lexicon_tags_29 },
    { 0xc1451fc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd051fc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2451fca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde51fcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24451fd4, SUC_lexicon_tags_7 },
    { 0x62251fd5, SUC_lexicon_tags_49 },
    { 0xe0851fd6, SUC_lexicon_tags_63 },
    { 0xdfe51fd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c51fda, SUC_lexicon_tags_49 },
    { 0xbc051fdb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ce51fdd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5851fe0, SUC_lexicon_tags_39 },
    { 0x63e51fe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5251fe5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71051fee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c051ff0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3451ff7, SUC_lexicon_tags_37 },
    { 0x65851ff7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56251fff, SUC_lexicon_tags_27 },
    { 0xa9052000, SUC_lexicon_tags_216 },
    { 0x1ec52001, SUC_lexicon_tags_28 },
    { 0x5e852001, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdf852004, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a52007, SUC_lexicon_tags_158 },
    { 0xc052007, SUC_lexicon_tags_63 },
    { 0x73252008, SUC_lexicon_tags_19 },
    { 0xb9e5200a, SUC_lexicon_tags_15 },
    { 0xfb45200b, SUC_lexicon_tags_10 },
    { 0xed05200c, SUC_lexicon_tags_26 },
    { 0xe2c5200d, SUC_lexicon_tags_0 },
    { 0xc65200e, SUC_lexicon_tags_40 },
    { 0xe0e5200c, SUC_lexicon_tags_14 },
    { 0x3c452010, SUC_lexicon_tags_1 },
    { 0x8f85200f, SUC_lexicon_tags_27 },
    { 0x92452009, SUC_lexicon_tags_1 },
    { 0x33852013, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x26052015, SUC_lexicon_tags_37 },
    { 0xee252016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64052019, SUC_lexicon_tags_2 },
    { 0x51e5201a, SUC_lexicon_tags_25 },
    { 0xa7e5201b, SUC_lexicon_tags_7 },
    { 0x5a65201c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72852022, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92852025, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a052027, SUC_lexicon_tags_25 },
    { 0x3ee52028, SUC_lexicon_tags_87 },
    { 0x64252029, SUC_lexicon_tags_47 },
    { 0xa6052028, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5052035, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1cc52037, SUC_lexicon_tags_62 },
    { 0x9dc52037, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f45203e, SUC_lexicon_tags_49 },
    { 0xdca5203f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3052042, SUC_lexicon_tags_47 },
    { 0xf4c52043, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee52048, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf225204a, SUC_lexicon_tags_19 },
    { 0x8b85204b, SUC_lexicon_tags_12 },
    { 0x1b65204a, SUC_lexicon_tags_7 },
    { 0x9c5204b, SUC_lexicon_tags_38 },
    { 0x7845204b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8052052, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe52056, SUC_lexicon_tags_12 },
    { 0x88c52057, SUC_lexicon_tags_25 },
    { 0x8f252058, SUC_lexicon_tags_7 },
    { 0xeaa52056, SUC_lexicon_tags_17 },
    { 0x9de5205a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac5205e, SUC_lexicon_tags_11 },
    { 0x4da5205e, SUC_lexicon_tags_69 },
    { 0x82852060, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd052062, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49252066, SUC_lexicon_tags_1337 },
    { 0, NULL },
    { 0, NULL },
    { 0x6052069, SUC_lexicon_tags_17 },
    { 0xeb05206a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3885206c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c52072, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e5207a, SUC_lexicon_tags_9 },
    { 0x1665207a, SUC_lexicon_tags_23 },
    { 0x9ea5207c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8452081, SUC_lexicon_tags_16 },
    { 0xd4c52082, SUC_lexicon_tags_60 },
    { 0xb8252083, SUC_lexicon_tags_2 },
    { 0xeae52083, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4325208b, SUC_lexicon_tags_29 },
    { 0xf585208c, SUC_lexicon_tags_30 },
    { 0xfce5208b, SUC_lexicon_tags_12 },
    { 0xcb85208e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9852092, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41e52094, SUC_lexicon_tags_47 },
    { 0x4b252095, SUC_lexicon_tags_12 },
    { 0xee852096, SUC_lexicon_tags_12 },
    { 0xcb452094, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8c52099, SUC_lexicon_tags_11 },
    { 0xeac52099, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6520a2, SUC_lexicon_tags_8 },
    { 0x498520a3, SUC_lexicon_tags_25 },
    { 0xf5a520a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552520a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1a0520aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a520af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc520b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82520b6, SUC_lexicon_tags_15 },
    { 0xc42520b7, SUC_lexicon_tags_37 },
    { 0x18e520b8, SUC_lexicon_tags_15 },
    { 0x598520b8, SUC_lexicon_tags_15 },
    { 0xe02520b7, SUC_lexicon_tags_11 },
    { 0xbd4520b9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x712520be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4520c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e520c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0a520cb, SUC_lexicon_tags_12 },
    { 0x526520cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c520d0, SUC_lexicon_tags_0 },
    { 0x83e520d0, SUC_lexicon_tags_29 },
    { 0xd06520d2, SUC_lexicon_tags_17 },
    { 0x558520d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4520d9, SUC_lexicon_tags_4 },
    { 0xdfc520d9, SUC_lexicon_tags_60 },
    { 0xa06520db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae520df, SUC_lexicon_tags_14 },
    { 0x860520df, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x196520e2, SUC_lexicon_tags_401 },
    { 0xba0520e3, SUC_lexicon_tags_37 },
    { 0xd78520e4, SUC_lexicon_tags_47 },
    { 0x2ae520e5, SUC_lexicon_tags_4 },
    { 0x3a520e5, SUC_lexicon_tags_37 },
    { 0xe64520e5, SUC_lexicon_tags_25 },
    { 0x4a0520e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56520ef, SUC_lexicon_tags_32 },
    { 0xde2520f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56520f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a520f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178520fb, SUC_lexicon_tags_2 },
    { 0x8da520fc, SUC_lexicon_tags_7 },
    { 0x210520fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf38520ff, SUC_lexicon_tags_12 },
    { 0xf5a52100, SUC_lexicon_tags_19 },
    { 0x51852101, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2452104, SUC_lexicon_tags_55 },
    { 0xe8252104, SUC_lexicon_tags_7 },
    { 0x7f652104, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49652109, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65210c, SUC_lexicon_tags_0 },
    { 0x8b45210d, SUC_lexicon_tags_28 },
    { 0x2745210e, SUC_lexicon_tags_1 },
    { 0x1705210f, SUC_lexicon_tags_7 },
    { 0x1e052110, SUC_lexicon_tags_60 },
    { 0x3f45210e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ae52113, SUC_lexicon_tags_1 },
    { 0xee252114, SUC_lexicon_tags_4 },
    { 0xc2e52114, SUC_lexicon_tags_12 },
    { 0x54c52115, SUC_lexicon_tags_87 },
    { 0xf4a52115, SUC_lexicon_tags_3 },
    { 0x53052118, SUC_lexicon_tags_30 },
    { 0xf452119, SUC_lexicon_tags_1 },
    { 0x6c45211a, SUC_lexicon_tags_11 },
    { 0x25e52117, SUC_lexicon_tags_25 },
    { 0xb7852117, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c52120, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85852128, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ca5212a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3165212f, SUC_lexicon_tags_15 },
    { 0x83e52130, SUC_lexicon_tags_14 },
    { 0x17c52131, SUC_lexicon_tags_27 },
    { 0xe865212f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a052134, SUC_lexicon_tags_15 },
    { 0x49252134, SUC_lexicon_tags_45 },
    { 0x88252134, SUC_lexicon_tags_27 },
    { 0xd9c52135, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9a52139, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98e5213b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad05213f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97652143, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73252145, SUC_lexicon_tags_47 },
    { 0x4e052146, SUC_lexicon_tags_39 },
    { 0x49652145, SUC_lexicon_tags_17 },
    { 0x76e52148, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xab45214a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5065214e, SUC_lexicon_tags_60 },
    { 0xbec5214e, SUC_lexicon_tags_23 },
    { 0xaa452150, SUC_lexicon_tags_14 },
    { 0x9d652150, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a52156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b052159, SUC_lexicon_tags_14 },
    { 0x6a85215a, SUC_lexicon_tags_15 },
    { 0xb65215b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e5215e, SUC_lexicon_tags_1 },
    { 0x8725215f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80452166, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c5216d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb25216f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7652178, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe245217e, SUC_lexicon_tags_8 },
    { 0xf425217f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26452183, SUC_lexicon_tags_17 },
    { 0x41652184, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0252187, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e45218a, SUC_lexicon_tags_40 },
    { 0xa6a5218b, SUC_lexicon_tags_57 },
    { 0x1025218b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e5218f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32252195, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4052198, SUC_lexicon_tags_1 },
    { 0xe8052199, SUC_lexicon_tags_1 },
    { 0x2f05219a, SUC_lexicon_tags_14 },
    { 0x64452198, SUC_lexicon_tags_3 },
    { 0xba65219c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc605219e, SUC_lexicon_tags_37 },
    { 0x5fe5219f, SUC_lexicon_tags_7 },
    { 0x31a5219f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc521a3, SUC_lexicon_tags_2 },
    { 0x3ee521a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6521ab, SUC_lexicon_tags_27 },
    { 0xa2c521ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78c521ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16521b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602521b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0521bc, SUC_lexicon_tags_6 },
    { 0x73c521bd, SUC_lexicon_tags_31 },
    { 0x364521be, SUC_lexicon_tags_12 },
    { 0x1f0521be, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6521c4, SUC_lexicon_tags_61 },
    { 0x4e4521c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44521c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb28521ca, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x674521cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xffc521ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4521d5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4521db, SUC_lexicon_tags_37 },
    { 0xa78521db, SUC_lexicon_tags_46 },
    { 0xca2521db, SUC_lexicon_tags_46 },
    { 0x79a521de, SUC_lexicon_tags_11 },
    { 0x5e0521df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae521e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa84521e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e2521e7, SUC_lexicon_tags_25 },
    { 0x908521e8, SUC_lexicon_tags_60 },
    { 0xb08521e7, SUC_lexicon_tags_65 },
    { 0x25e521e8, SUC_lexicon_tags_17 },
    { 0x1a521e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4521ee, SUC_lexicon_tags_37 },
    { 0x690521ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc521f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c521f8, SUC_lexicon_tags_25 },
    { 0x28521f9, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958521fd, SUC_lexicon_tags_26 },
    { 0x3dc521fd, SUC_lexicon_tags_16 },
    { 0x5f4521ff, SUC_lexicon_tags_47 },
    { 0xa3e521fd, SUC_lexicon_tags_23 },
    { 0x15c521fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa52203, SUC_lexicon_tags_77 },
    { 0x61652203, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e52207, SUC_lexicon_tags_25 },
    { 0xeb452208, SUC_lexicon_tags_7 },
    { 0xb4e52207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf205220b, SUC_lexicon_tags_71 },
    { 0x6b05220c, SUC_lexicon_tags_25 },
    { 0xa605220d, SUC_lexicon_tags_0 },
    { 0x6fc5220c, SUC_lexicon_tags_23 },
    { 0x6ae5220f, SUC_lexicon_tags_27 },
    { 0xbb052210, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9052215, SUC_lexicon_tags_28 },
    { 0x4ce52216, SUC_lexicon_tags_15 },
    { 0x28652217, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e85221c, SUC_lexicon_tags_23 },
    { 0x8c65221d, SUC_lexicon_tags_45 },
    { 0x4ce5221c, SUC_lexicon_tags_37 },
    { 0xb3c5221f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf452227, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345222d, SUC_lexicon_tags_68 },
    { 0xa1c5222d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b252235, SUC_lexicon_tags_25 },
    { 0x49c52236, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb065223c, SUC_lexicon_tags_45 },
    { 0xd585223d, SUC_lexicon_tags_1 },
    { 0x27e5223e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ec52240, SUC_lexicon_tags_2 },
    { 0xefc52240, SUC_lexicon_tags_46 },
    { 0x89852241, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0e52244, SUC_lexicon_tags_29 },
    { 0x55852245, SUC_lexicon_tags_11 },
    { 0xdd452245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a05224b, SUC_lexicon_tags_17 },
    { 0x11a5224c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e5224f, SUC_lexicon_tags_37 },
    { 0xbca52250, SUC_lexicon_tags_15 },
    { 0xad452251, SUC_lexicon_tags_2 },
    { 0x81a52252, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xafc52254, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x15852257, SUC_lexicon_tags_6 },
    { 0x7ac52258, SUC_lexicon_tags_11 },
    { 0xb8c52259, SUC_lexicon_tags_47 },
    { 0x5f85225a, SUC_lexicon_tags_8 },
    { 0x8c85225b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85225e, SUC_lexicon_tags_7 },
    { 0x58e5225f, SUC_lexicon_tags_1 },
    { 0xbfc5225f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb052262, SUC_lexicon_tags_68 },
    { 0x95652263, SUC_lexicon_tags_11 },
    { 0xb6c52264, SUC_lexicon_tags_47 },
    { 0xaa52265, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x31a52267, SUC_lexicon_tags_17 },
    { 0xa452267, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be5226c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e52274, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73252276, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e52279, SUC_lexicon_tags_49 },
    { 0xba25227a, SUC_lexicon_tags_1 },
    { 0xcce5227b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7045227e, SUC_lexicon_tags_1 },
    { 0x9365227e, SUC_lexicon_tags_37 },
    { 0xd4c52280, SUC_lexicon_tags_29 },
    { 0xe3052281, SUC_lexicon_tags_7 },
    { 0x39652282, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9652284, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x39652287, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x29a52289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e25228b, SUC_lexicon_tags_42 },
    { 0x9c5228c, SUC_lexicon_tags_8 },
    { 0xfe85228d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85852295, SUC_lexicon_tags_37 },
    { 0x44852296, SUC_lexicon_tags_1 },
    { 0x40852295, SUC_lexicon_tags_2 },
    { 0x38a52295, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9665229a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7de5229c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2522a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2522a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2522a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a522ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e522b0, SUC_lexicon_tags_1 },
    { 0x63a522b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be522b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x468522b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x718522b8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1a4522ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xce4522bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6aa522be, SUC_lexicon_tags_49 },
    { 0x366522be, SUC_lexicon_tags_17 },
    { 0xe4a522be, SUC_lexicon_tags_61 },
    { 0x910522c1, SUC_lexicon_tags_12 },
    { 0xb00522c1, SUC_lexicon_tags_63 },
    { 0x112522c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758522ca, SUC_lexicon_tags_38 },
    { 0x46522cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x870522ce, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xe58522d0, SUC_lexicon_tags_11 },
    { 0xe9a522d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74522d4, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a522da, SUC_lexicon_tags_30 },
    { 0x9a0522db, SUC_lexicon_tags_43 },
    { 0xaa6522dc, SUC_lexicon_tags_75 },
    { 0x428522dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x320522e0, SUC_lexicon_tags_33 },
    { 0xfd8522e1, SUC_lexicon_tags_37 },
    { 0x79c522e0, SUC_lexicon_tags_17 },
    { 0x96522e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a8522e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150522ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582522ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402522f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3e522f5, SUC_lexicon_tags_77 },
    { 0x67e522f6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbb4522f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x484522fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5852300, SUC_lexicon_tags_23 },
    { 0xca852301, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea5230c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b252311, SUC_lexicon_tags_30 },
    { 0xa5c52311, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaae52314, SUC_lexicon_tags_45 },
    { 0x3ec52314, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc52318, SUC_lexicon_tags_63 },
    { 0x1c252319, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xee25231c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x245231e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc852323, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x25652326, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x11852329, SUC_lexicon_tags_3 },
    { 0x6fa5232a, SUC_lexicon_tags_27 },
    { 0xbea5232a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8785232d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd545232f, SUC_lexicon_tags_15 },
    { 0x3a85232f, SUC_lexicon_tags_1 },
    { 0x42a5232f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a852333, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8452336, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c5233e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9452341, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6a52343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15a52345, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfd252347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59452349, SUC_lexicon_tags_47 },
    { 0x89052349, SUC_lexicon_tags_7 },
    { 0xc5c52349, SUC_lexicon_tags_79 },
    { 0x9a05234b, SUC_lexicon_tags_2 },
    { 0xfd452349, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x63e5234f, SUC_lexicon_tags_14 },
    { 0x98a52350, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98452353, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46052356, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0xa0c52358, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85235d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da52362, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19852364, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e52367, SUC_lexicon_tags_25 },
    { 0xda652367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4065236b, SUC_lexicon_tags_7 },
    { 0xe705236c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5052370, SUC_lexicon_tags_25 },
    { 0xcf652370, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c52374, SUC_lexicon_tags_15 },
    { 0x55652375, SUC_lexicon_tags_11 },
    { 0x4e252376, SUC_lexicon_tags_2 },
    { 0xb8e52374, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e5237a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73e5237c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf165237f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9652384, SUC_lexicon_tags_60 },
    { 0x63452385, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xade52387, SUC_lexicon_tags_19 },
    { 0x9fa52388, SUC_lexicon_tags_27 },
    { 0xc8a52388, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e5238d, SUC_lexicon_tags_12 },
    { 0x9ba5238e, SUC_lexicon_tags_1 },
    { 0xdf05238e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d452394, SUC_lexicon_tags_1 },
    { 0xf3052395, SUC_lexicon_tags_6 },
    { 0x1f652396, SUC_lexicon_tags_17 },
    { 0xbc252396, SUC_lexicon_tags_4 },
    { 0x5d852396, SUC_lexicon_tags_37 },
    { 0xff252396, SUC_lexicon_tags_15 },
    { 0xd265239a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x1ea5239c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c523a0, SUC_lexicon_tags_34 },
    { 0x62e523a1, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e523a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a523aa, SUC_lexicon_tags_15 },
    { 0x3da523ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63e523ad, SUC_lexicon_tags_26 },
    { 0xa72523ae, SUC_lexicon_tags_17 },
    { 0x60a523af, SUC_lexicon_tags_105 },
    { 0xb6e523b0, SUC_lexicon_tags_30 },
    { 0x20a523ad, SUC_lexicon_tags_37 },
    { 0x94e523b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcaa523b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x218523b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84523b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe523bc, SUC_lexicon_tags_27 },
    { 0xabc523bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9aa523bf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce523c4, SUC_lexicon_tags_27 },
    { 0x85c523c5, SUC_lexicon_tags_0 },
    { 0x6c4523c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x920523c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec523cb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xad6523cd, SUC_lexicon_tags_17 },
    { 0x706523ce, SUC_lexicon_tags_8 },
    { 0x12a523cd, SUC_lexicon_tags_118 },
    { 0xf04523cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77a523d2, SUC_lexicon_tags_1 },
    { 0x70c523d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a523d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6523df, SUC_lexicon_tags_10 },
    { 0xdc523e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e523e3, SUC_lexicon_tags_37 },
    { 0xfc4523e4, SUC_lexicon_tags_12 },
    { 0x334523e5, SUC_lexicon_tags_14 },
    { 0xffe523e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe32523e8, SUC_lexicon_tags_17 },
    { 0xed4523e9, SUC_lexicon_tags_10 },
    { 0xbea523ea, SUC_lexicon_tags_28 },
    { 0xbca523eb, SUC_lexicon_tags_6 },
    { 0xde0523ec, SUC_lexicon_tags_29 },
    { 0x9c4523ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e523f2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e523f5, SUC_lexicon_tags_2 },
    { 0xac4523f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe523fa, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca523fd, SUC_lexicon_tags_17 },
    { 0x4e523fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc6852400, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x98252402, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x87252404, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xba452406, SUC_lexicon_tags_22 },
    { 0x34a52406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3452409, SUC_lexicon_tags_26 },
    { 0x23e5240a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3445240d, SUC_lexicon_tags_27 },
    { 0xd945240d, SUC_lexicon_tags_1 },
    { 0xa805240d, SUC_lexicon_tags_47 },
    { 0xed25240d, SUC_lexicon_tags_79 },
    { 0xa252411, SUC_lexicon_tags_160 },
    { 0x40a52411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a05241f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a52424, SUC_lexicon_tags_634 },
    { 0x72e52425, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7005242c, SUC_lexicon_tags_15 },
    { 0x3925242d, SUC_lexicon_tags_6 },
    { 0x1805242d, SUC_lexicon_tags_106 },
    { 0x4b45242f, SUC_lexicon_tags_14 },
    { 0xa1252430, SUC_lexicon_tags_11 },
    { 0x3e652431, SUC_lexicon_tags_37 },
    { 0x60852430, SUC_lexicon_tags_87 },
    { 0x63e52433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c52436, SUC_lexicon_tags_17 },
    { 0x2652437, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe0052439, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe985243b, SUC_lexicon_tags_37 },
    { 0x2165243c, SUC_lexicon_tags_9 },
    { 0x6485243d, SUC_lexicon_tags_11 },
    { 0xd585243d, SUC_lexicon_tags_33 },
    { 0x5d85243f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb052444, SUC_lexicon_tags_7 },
    { 0xcd052445, SUC_lexicon_tags_23 },
    { 0x1ca52445, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x28852448, SUC_lexicon_tags_33 },
    { 0xc5452448, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e45244f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90852454, SUC_lexicon_tags_20 },
    { 0x56452455, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec52458, SUC_lexicon_tags_91 },
    { 0xe8052458, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd025245b, SUC_lexicon_tags_29 },
    { 0x7545245c, SUC_lexicon_tags_15 },
    { 0x6665245b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa25245f, SUC_lexicon_tags_17 },
    { 0x7a452460, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b652466, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d452469, SUC_lexicon_tags_66 },
    { 0xd765246a, SUC_lexicon_tags_12 },
    { 0xb605246b, SUC_lexicon_tags_60 },
    { 0x2545246c, SUC_lexicon_tags_14 },
    { 0xcc25246d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d052472, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x70452474, SUC_lexicon_tags_37 },
    { 0x28e52475, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3252479, SUC_lexicon_tags_37 },
    { 0xe045247a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e25247d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c52480, SUC_lexicon_tags_47 },
    { 0x63a52480, SUC_lexicon_tags_1 },
    { 0xf4852482, SUC_lexicon_tags_118 },
    { 0x67252480, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8305248c, SUC_lexicon_tags_2 },
    { 0x85e5248d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb1e5248f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1052493, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e52496, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca5249b, SUC_lexicon_tags_25 },
    { 0xb1e5249c, SUC_lexicon_tags_2 },
    { 0xd3a5249d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a524a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf16524a2, SUC_lexicon_tags_11 },
    { 0x7a6524a2, SUC_lexicon_tags_15 },
    { 0xadc524a4, SUC_lexicon_tags_2 },
    { 0x7e2524a5, SUC_lexicon_tags_29 },
    { 0x24a524a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996524aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x432524ad, SUC_lexicon_tags_17 },
    { 0xd8524ad, SUC_lexicon_tags_1 },
    { 0xdde524ae, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da524b2, SUC_lexicon_tags_2 },
    { 0x336524b3, SUC_lexicon_tags_23 },
    { 0x472524b4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8524bd, SUC_lexicon_tags_49 },
    { 0x752524bd, SUC_lexicon_tags_15 },
    { 0xadc524bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64524c7, SUC_lexicon_tags_17 },
    { 0xf36524c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c524cc, SUC_lexicon_tags_7 },
    { 0x100524cc, SUC_lexicon_tags_11 },
    { 0x4e2524ce, SUC_lexicon_tags_11 },
    { 0x2ea524cc, SUC_lexicon_tags_7 },
    { 0xc48524d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6524d3, SUC_lexicon_tags_14 },
    { 0x4fc524d4, SUC_lexicon_tags_17 },
    { 0xc26524d3, SUC_lexicon_tags_28 },
    { 0x7de524d6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x690524d8, SUC_lexicon_tags_37 },
    { 0x88e524d9, SUC_lexicon_tags_53 },
    { 0xdcc524d8, SUC_lexicon_tags_91 },
    { 0x614524db, SUC_lexicon_tags_20 },
    { 0xd04524dc, SUC_lexicon_tags_37 },
    { 0xb1c524dd, SUC_lexicon_tags_45 },
    { 0x846524dc, SUC_lexicon_tags_2 },
    { 0xfc4524df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6524e2, SUC_lexicon_tags_12 },
    { 0x998524e3, SUC_lexicon_tags_25 },
    { 0x9d6524e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6524eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ae524ed, SUC_lexicon_tags_19 },
    { 0x636524ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584524f2, SUC_lexicon_tags_2 },
    { 0x64e524f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x926524f6, SUC_lexicon_tags_28 },
    { 0xbb6524f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26524fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x986524ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21052501, SUC_lexicon_tags_60 },
    { 0x7a52502, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe52507, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c5250b, SUC_lexicon_tags_43 },
    { 0xec45250c, SUC_lexicon_tags_23 },
    { 0x3445250d, SUC_lexicon_tags_37 },
    { 0x43c5250b, SUC_lexicon_tags_68 },
    { 0x1bc5250f, SUC_lexicon_tags_6 },
    { 0x84c52510, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c652518, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe45251a, SUC_lexicon_tags_17 },
    { 0x9245251b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf765251d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27052523, SUC_lexicon_tags_25 },
    { 0x88652523, SUC_lexicon_tags_37 },
    { 0xb7252525, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x82852527, SUC_lexicon_tags_1 },
    { 0x4fc52527, SUC_lexicon_tags_1 },
    { 0xaa852529, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e5252e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5e52530, SUC_lexicon_tags_0 },
    { 0xa7e52531, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62252535, SUC_lexicon_tags_0 },
    { 0xe6c52536, SUC_lexicon_tags_38 },
    { 0x3ee52537, SUC_lexicon_tags_12 },
    { 0xb7452537, SUC_lexicon_tags_58 },
    { 0x9ec52535, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4985253e, SUC_lexicon_tags_23 },
    { 0x1ce5253f, SUC_lexicon_tags_295 },
    { 0x4625253e, SUC_lexicon_tags_1 },
    { 0xf5652540, SUC_lexicon_tags_69 },
    { 0xf6252542, SUC_lexicon_tags_19 },
    { 0xa8c52543, SUC_lexicon_tags_15 },
    { 0x71a52542, SUC_lexicon_tags_25 },
    { 0x4e652545, SUC_lexicon_tags_1 },
    { 0x94e52546, SUC_lexicon_tags_8 },
    { 0x54852540, SUC_lexicon_tags_42 },
    { 0xdd052547, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7485254c, SUC_lexicon_tags_31 },
    { 0x4245254c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x66452550, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xed452553, SUC_lexicon_tags_7 },
    { 0xd5052554, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x80e52556, SUC_lexicon_tags_60 },
    { 0xbda52557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c5255a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa625255c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6265255f, SUC_lexicon_tags_8 },
    { 0x5c852560, SUC_lexicon_tags_43 },
    { 0xff052561, SUC_lexicon_tags_2 },
    { 0xcc052562, SUC_lexicon_tags_12 },
    { 0x18252563, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae52566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2da52568, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8905256b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea52572, SUC_lexicon_tags_25 },
    { 0xcea52573, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97052577, SUC_lexicon_tags_17 },
    { 0x26e52578, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e45257b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa45257e, SUC_lexicon_tags_27 },
    { 0xc4e5257e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31252583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6c52585, SUC_lexicon_tags_32 },
    { 0x36252585, SUC_lexicon_tags_11 },
    { 0x87652587, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ba52589, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa445258b, SUC_lexicon_tags_27 },
    { 0xc505258c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e5258f, SUC_lexicon_tags_10 },
    { 0xdfe52590, SUC_lexicon_tags_27 },
    { 0x5852591, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a652597, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a5259c, SUC_lexicon_tags_2 },
    { 0xd045259d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce5259f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x534525a1, SUC_lexicon_tags_887 },
    { 0x2e6525a1, SUC_lexicon_tags_7 },
    { 0xa2c525a3, SUC_lexicon_tags_15 },
    { 0x3ba525a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a525a8, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec525ad, SUC_lexicon_tags_8 },
    { 0x45c525ad, SUC_lexicon_tags_6 },
    { 0x34525ae, SUC_lexicon_tags_1 },
    { 0x33e525b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78525b4, SUC_lexicon_tags_12 },
    { 0x69a525b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50e525b7, SUC_lexicon_tags_62 },
    { 0xa22525b8, SUC_lexicon_tags_37 },
    { 0xbe525b7, SUC_lexicon_tags_43 },
    { 0x1e6525b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c525c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc525c2, SUC_lexicon_tags_3 },
    { 0xe52525c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcb6525c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54525cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c525d6, SUC_lexicon_tags_92 },
    { 0xbc8525d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x574525da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb6525dc, SUC_lexicon_tags_60 },
    { 0xa0525dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x308525df, SUC_lexicon_tags_1 },
    { 0x28c525e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26525e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78525ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2525f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x250525f3, SUC_lexicon_tags_61 },
    { 0x1c6525f3, SUC_lexicon_tags_2 },
    { 0xbd4525f5, SUC_lexicon_tags_37 },
    { 0xa64525f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a52604, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56252606, SUC_lexicon_tags_14 },
    { 0x93c52607, SUC_lexicon_tags_4 },
    { 0xa5652608, SUC_lexicon_tags_34 },
    { 0xd3e52608, SUC_lexicon_tags_9 },
    { 0xd6452608, SUC_lexicon_tags_29 },
    { 0xfda5260b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8652613, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d052616, SUC_lexicon_tags_25 },
    { 0x90c52616, SUC_lexicon_tags_30 },
    { 0xcd252618, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xada5261b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f65261e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa52621, SUC_lexicon_tags_37 },
    { 0x7dc52622, SUC_lexicon_tags_31 },
    { 0x6b252623, SUC_lexicon_tags_31 },
    { 0xed452624, SUC_lexicon_tags_79 },
    { 0xec852621, SUC_lexicon_tags_37 },
    { 0xdaa52621, SUC_lexicon_tags_11 },
    { 0x42c52624, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f05262a, SUC_lexicon_tags_47 },
    { 0x7225262b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c52631, SUC_lexicon_tags_14 },
    { 0xc2852632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10652636, SUC_lexicon_tags_28 },
    { 0x8a452637, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b85263b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3452642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e65264a, SUC_lexicon_tags_2 },
    { 0x58c5264b, SUC_lexicon_tags_7 },
    { 0x44c5264c, SUC_lexicon_tags_7 },
    { 0x80e5264d, SUC_lexicon_tags_30 },
    { 0x6665264d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31c52650, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe52653, SUC_lexicon_tags_15 },
    { 0xa0452654, SUC_lexicon_tags_37 },
    { 0x14652655, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4845265b, SUC_lexicon_tags_17 },
    { 0xd0a5265c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d052660, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa252667, SUC_lexicon_tags_33 },
    { 0xe4c52668, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb805266a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c52673, SUC_lexicon_tags_7 },
    { 0x9e852673, SUC_lexicon_tags_99 },
    { 0x2fe52673, SUC_lexicon_tags_8 },
    { 0x95452676, SUC_lexicon_tags_55 },
    { 0xeea52677, SUC_lexicon_tags_37 },
    { 0x7da52674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5a5267a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc52685, SUC_lexicon_tags_25 },
    { 0xd9a52686, SUC_lexicon_tags_2 },
    { 0x96e52687, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x85452689, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a5268c, SUC_lexicon_tags_1 },
    { 0xae65268d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9dc5268f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6452691, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47852697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80a52699, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5269d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x318526a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94526a3, SUC_lexicon_tags_25 },
    { 0x1e4526a4, SUC_lexicon_tags_13 },
    { 0xd2e526a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e526a8, SUC_lexicon_tags_11 },
    { 0x576526a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55c526ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c526ae, SUC_lexicon_tags_15 },
    { 0x1fe526ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c526b4, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c526bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x514526bd, SUC_lexicon_tags_25 },
    { 0x258526bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380526c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa80526c7, SUC_lexicon_tags_21 },
    { 0xdf0526c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcda526ca, SUC_lexicon_tags_55 },
    { 0xf3a526cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba526d2, SUC_lexicon_tags_37 },
    { 0x4ee526d3, SUC_lexicon_tags_1 },
    { 0xcf8526d3, SUC_lexicon_tags_43 },
    { 0xdf6526d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x760526d8, SUC_lexicon_tags_1 },
    { 0xd86526d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a526e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378526e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8526ec, SUC_lexicon_tags_12 },
    { 0xa16526ed, SUC_lexicon_tags_27 },
    { 0x278526ee, SUC_lexicon_tags_29 },
    { 0x2d8526ef, SUC_lexicon_tags_17 },
    { 0x26c526ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98a526f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x77c526f4, SUC_lexicon_tags_25 },
    { 0xd50526f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a526fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x358526fe, SUC_lexicon_tags_9 },
    { 0x900526ff, SUC_lexicon_tags_12 },
    { 0xa6e526fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e052704, SUC_lexicon_tags_39 },
    { 0xaf852705, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78052709, SUC_lexicon_tags_17 },
    { 0x35e5270a, SUC_lexicon_tags_17 },
    { 0x2da5270b, SUC_lexicon_tags_23 },
    { 0xd325270a, SUC_lexicon_tags_11 },
    { 0x46e5270c, SUC_lexicon_tags_12 },
    { 0xfea5270e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac52711, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0052714, SUC_lexicon_tags_2 },
    { 0x61652715, SUC_lexicon_tags_11 },
    { 0xe9652715, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x54452718, SUC_lexicon_tags_49 },
    { 0x90852719, SUC_lexicon_tags_1 },
    { 0xc3c5271a, SUC_lexicon_tags_7 },
    { 0xb945271b, SUC_lexicon_tags_6 },
    { 0x7ba52719, SUC_lexicon_tags_26 },
    { 0x1805271a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56452726, SUC_lexicon_tags_17 },
    { 0xeba52727, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8da52729, SUC_lexicon_tags_39 },
    { 0x885272a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4645272c, SUC_lexicon_tags_11 },
    { 0x8c85272d, SUC_lexicon_tags_15 },
    { 0xe045272c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a52731, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4252733, SUC_lexicon_tags_37 },
    { 0x74a52733, SUC_lexicon_tags_25 },
    { 0x2ea52735, SUC_lexicon_tags_29 },
    { 0x9fc52736, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x11652738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a85273a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e5273d, SUC_lexicon_tags_51 },
    { 0xc1a5273d, SUC_lexicon_tags_40 },
    { 0x94c5273f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb4c52741, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c52748, SUC_lexicon_tags_34 },
    { 0x3e052748, SUC_lexicon_tags_37 },
    { 0x7b052748, SUC_lexicon_tags_1 },
    { 0x1a052749, SUC_lexicon_tags_25 },
    { 0x8505274b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe452753, SUC_lexicon_tags_12 },
    { 0x78852754, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb0652756, SUC_lexicon_tags_25 },
    { 0xbd652757, SUC_lexicon_tags_40 },
    { 0xcd252756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d65275b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8052762, SUC_lexicon_tags_27 },
    { 0x24052763, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ce52765, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e52768, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcca5276a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5276f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa852771, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a52775, SUC_lexicon_tags_0 },
    { 0x5de52776, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe645277a, SUC_lexicon_tags_62 },
    { 0xabe5277b, SUC_lexicon_tags_28 },
    { 0xd865277a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47452780, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10452785, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcda52787, SUC_lexicon_tags_24 },
    { 0xcea52787, SUC_lexicon_tags_5 },
    { 0x20852789, SUC_lexicon_tags_1 },
    { 0xd4e52788, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd45278d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c52790, SUC_lexicon_tags_11 },
    { 0xf7a52791, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c52796, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc52799, SUC_lexicon_tags_28 },
    { 0xcb252799, SUC_lexicon_tags_15 },
    { 0xc605279a, SUC_lexicon_tags_27 },
    { 0x685279c, SUC_lexicon_tags_37 },
    { 0xc5279d, SUC_lexicon_tags_30 },
    { 0x7925279e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84527a5, SUC_lexicon_tags_16 },
    { 0x494527a6, SUC_lexicon_tags_27 },
    { 0x4a6527a6, SUC_lexicon_tags_37 },
    { 0x568527a8, SUC_lexicon_tags_37 },
    { 0x3d2527a9, SUC_lexicon_tags_30 },
    { 0xa8c527a9, SUC_lexicon_tags_15 },
    { 0xe76527a7, SUC_lexicon_tags_1 },
    { 0xf40527ac, SUC_lexicon_tags_0 },
    { 0x826527a8, SUC_lexicon_tags_17 },
    { 0xa22527ae, SUC_lexicon_tags_31 },
    { 0x730527af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92527b4, SUC_lexicon_tags_0 },
    { 0xb7c527b5, SUC_lexicon_tags_65 },
    { 0xc6527b4, SUC_lexicon_tags_63 },
    { 0x40527b7, SUC_lexicon_tags_11 },
    { 0xb8527b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c527bb, SUC_lexicon_tags_38 },
    { 0x336527bc, SUC_lexicon_tags_11 },
    { 0xcf4527bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e527c4, SUC_lexicon_tags_28 },
    { 0x872527c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4527cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x486527cf, SUC_lexicon_tags_9 },
    { 0xc58527d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0527d3, SUC_lexicon_tags_8 },
    { 0xa4a527d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0527d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec527dc, SUC_lexicon_tags_25 },
    { 0xa86527dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43c527df, SUC_lexicon_tags_35 },
    { 0x37e527df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37a527e2, SUC_lexicon_tags_936 },
    { 0x2c527e3, SUC_lexicon_tags_21 },
    { 0x300527e3, SUC_lexicon_tags_7 },
    { 0xd04527e5, SUC_lexicon_tags_91 },
    { 0x914527e5, SUC_lexicon_tags_21 },
    { 0xc0c527e5, SUC_lexicon_tags_7 },
    { 0xa00527e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf00527ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x100527ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04527f1, SUC_lexicon_tags_49 },
    { 0xc9e527f1, SUC_lexicon_tags_9 },
    { 0xae527f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x624527f5, SUC_lexicon_tags_35 },
    { 0xa3a527f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63e527f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d4527fa, SUC_lexicon_tags_21 },
    { 0xf76527fb, SUC_lexicon_tags_30 },
    { 0x976527fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c527ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6252809, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad45280b, SUC_lexicon_tags_17 },
    { 0x9425280b, SUC_lexicon_tags_42 },
    { 0x9605280d, SUC_lexicon_tags_25 },
    { 0xdb65280d, SUC_lexicon_tags_4 },
    { 0x5325280f, SUC_lexicon_tags_32 },
    { 0x3145280b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11452816, SUC_lexicon_tags_60 },
    { 0xc2a52817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c5281c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec52821, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee452827, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5282b, SUC_lexicon_tags_37 },
    { 0x2d05282c, SUC_lexicon_tags_13 },
    { 0xb2c5282c, SUC_lexicon_tags_37 },
    { 0x8c85282e, SUC_lexicon_tags_0 },
    { 0xa7a5282f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7052833, SUC_lexicon_tags_11 },
    { 0x8852833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7252836, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a5283e, SUC_lexicon_tags_1 },
    { 0xd905283f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45452844, SUC_lexicon_tags_27 },
    { 0xdd652845, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16652848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2205284c, SUC_lexicon_tags_38 },
    { 0x4f05284c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11252853, SUC_lexicon_tags_25 },
    { 0x70252854, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fa52856, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d45285b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xece5285d, SUC_lexicon_tags_15 },
    { 0xfcc5285e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac52862, SUC_lexicon_tags_25 },
    { 0x13252862, SUC_lexicon_tags_37 },
    { 0x17c52864, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38852867, SUC_lexicon_tags_27 },
    { 0x1452867, SUC_lexicon_tags_65 },
    { 0x7e52868, SUC_lexicon_tags_8 },
    { 0xa685286a, SUC_lexicon_tags_82 },
    { 0x6105286b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3325286f, SUC_lexicon_tags_7 },
    { 0xdc652870, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a52874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfe52876, SUC_lexicon_tags_108 },
    { 0xe5c52877, SUC_lexicon_tags_25 },
    { 0xe9452877, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6865287a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2c5287c, SUC_lexicon_tags_7 },
    { 0x8965287d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e52880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e452884, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c65288b, SUC_lexicon_tags_23 },
    { 0xecc5288c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8252891, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf0e52893, SUC_lexicon_tags_39 },
    { 0xd4c52893, SUC_lexicon_tags_7 },
    { 0x60a52895, SUC_lexicon_tags_60 },
    { 0x6ba52896, SUC_lexicon_tags_9 },
    { 0xd7252897, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e5289b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304528a0, SUC_lexicon_tags_37 },
    { 0x394528a1, SUC_lexicon_tags_48 },
    { 0x740528a1, SUC_lexicon_tags_66 },
    { 0x2ba528a1, SUC_lexicon_tags_17 },
    { 0xf12528a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe36528a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x386528a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e528af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc528b1, SUC_lexicon_tags_3 },
    { 0xa26528b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884528b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa528bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x762528be, SUC_lexicon_tags_335 },
    { 0x814528bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0528c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a2528c4, SUC_lexicon_tags_7 },
    { 0x678528c5, SUC_lexicon_tags_31 },
    { 0xfcc528c5, SUC_lexicon_tags_60 },
    { 0x88e528c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc528ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38528d0, SUC_lexicon_tags_415 },
    { 0x6a6528d1, SUC_lexicon_tags_1 },
    { 0x18528d0, SUC_lexicon_tags_1 },
    { 0xf5e528d3, SUC_lexicon_tags_2 },
    { 0x8f8528d0, SUC_lexicon_tags_38 },
    { 0xb76528d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7a528d7, SUC_lexicon_tags_27 },
    { 0x44a528d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb8528da, SUC_lexicon_tags_87 },
    { 0x6dc528da, SUC_lexicon_tags_7 },
    { 0xa84528dc, SUC_lexicon_tags_7 },
    { 0xac0528dd, SUC_lexicon_tags_1 },
    { 0x206528de, SUC_lexicon_tags_37 },
    { 0xfee528df, SUC_lexicon_tags_63 },
    { 0x622528e0, SUC_lexicon_tags_16 },
    { 0x758528e1, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c528e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6528e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x410528e9, SUC_lexicon_tags_61 },
    { 0xfa528e9, SUC_lexicon_tags_37 },
    { 0x7d6528eb, SUC_lexicon_tags_15 },
    { 0x4ce528ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc528f3, SUC_lexicon_tags_25 },
    { 0xfe0528f3, SUC_lexicon_tags_37 },
    { 0xe6e528f5, SUC_lexicon_tags_12 },
    { 0xe4c528f5, SUC_lexicon_tags_6 },
    { 0x50e528f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc70528f9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58528fc, SUC_lexicon_tags_87 },
    { 0x4ea528fd, SUC_lexicon_tags_11 },
    { 0x3c2528fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac52901, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x28052903, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a052906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44652909, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c5290c, SUC_lexicon_tags_65 },
    { 0xa0a5290d, SUC_lexicon_tags_18 },
    { 0xe965290e, SUC_lexicon_tags_17 },
    { 0x7945290f, SUC_lexicon_tags_21 },
    { 0x68e52910, SUC_lexicon_tags_7 },
    { 0xd1652911, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63452914, SUC_lexicon_tags_1 },
    { 0x55452915, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ea52917, SUC_lexicon_tags_37 },
    { 0x74e52917, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11e5291a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe605291d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a52920, SUC_lexicon_tags_37 },
    { 0x50e52921, SUC_lexicon_tags_82 },
    { 0x5852922, SUC_lexicon_tags_37 },
    { 0x8aa52922, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70052927, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e52931, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a52934, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a852936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a5293a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5293d, SUC_lexicon_tags_32 },
    { 0xa665293d, SUC_lexicon_tags_2 },
    { 0xecc5293f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b252942, SUC_lexicon_tags_15 },
    { 0x32a52943, SUC_lexicon_tags_55 },
    { 0x8b852943, SUC_lexicon_tags_25 },
    { 0x1ba52943, SUC_lexicon_tags_46 },
    { 0xaea52946, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c52951, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c52956, SUC_lexicon_tags_30 },
    { 0x59a52956, SUC_lexicon_tags_33 },
    { 0x31852958, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf865295b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa185295d, SUC_lexicon_tags_63 },
    { 0xb1e5295d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a452961, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e52964, SUC_lexicon_tags_7 },
    { 0xe9052965, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3252969, SUC_lexicon_tags_37 },
    { 0xa3052969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e5296d, SUC_lexicon_tags_34 },
    { 0x3765296d, SUC_lexicon_tags_8 },
    { 0x6025296f, SUC_lexicon_tags_2 },
    { 0xb1052970, SUC_lexicon_tags_38 },
    { 0xc3a52971, SUC_lexicon_tags_15 },
    { 0x9fc5296f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xde452974, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa5297b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe845297d, SUC_lexicon_tags_11 },
    { 0xe6e5297e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4de52980, SUC_lexicon_tags_63 },
    { 0x37a52981, SUC_lexicon_tags_0 },
    { 0xf7a52980, SUC_lexicon_tags_12 },
    { 0xed252983, SUC_lexicon_tags_7 },
    { 0x29252984, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64452986, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5425298b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6985298f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11c52991, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13052995, SUC_lexicon_tags_17 },
    { 0x47e52995, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ee52998, SUC_lexicon_tags_17 },
    { 0xcee52999, SUC_lexicon_tags_3 },
    { 0x7545299a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2425299e, SUC_lexicon_tags_16 },
    { 0xa185299e, SUC_lexicon_tags_235 },
    { 0xc6a5299e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe88529a2, SUC_lexicon_tags_7 },
    { 0x10c529a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8b0529a5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666529a9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8529ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338529b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0529b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e529c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9c529c8, SUC_lexicon_tags_17 },
    { 0x3b0529c8, SUC_lexicon_tags_14 },
    { 0x104529ca, SUC_lexicon_tags_68 },
    { 0x81a529ca, SUC_lexicon_tags_33 },
    { 0xf5e529cc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e529d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a529d6, SUC_lexicon_tags_95 },
    { 0x568529d7, SUC_lexicon_tags_11 },
    { 0x3d8529d8, SUC_lexicon_tags_37 },
    { 0xd58529d9, SUC_lexicon_tags_37 },
    { 0xcd8529d6, SUC_lexicon_tags_4 },
    { 0xa60529db, SUC_lexicon_tags_11 },
    { 0x762529dc, SUC_lexicon_tags_25 },
    { 0x13c529db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x858529df, SUC_lexicon_tags_27 },
    { 0x11a529e0, SUC_lexicon_tags_104 },
    { 0xf60529e1, SUC_lexicon_tags_31 },
    { 0x670529e0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62529e5, SUC_lexicon_tags_6 },
    { 0x79c529e5, SUC_lexicon_tags_11 },
    { 0x8f6529e6, SUC_lexicon_tags_9 },
    { 0xd4e529e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe529ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf74529ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06529f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2529f5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x18529f7, SUC_lexicon_tags_27 },
    { 0x672529f8, SUC_lexicon_tags_30 },
    { 0xa5c529f9, SUC_lexicon_tags_30 },
    { 0x702529f9, SUC_lexicon_tags_30 },
    { 0x352529f8, SUC_lexicon_tags_49 },
    { 0x7f6529fc, SUC_lexicon_tags_11 },
    { 0x529f9, SUC_lexicon_tags_15 },
    { 0xf44529fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8452a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8452a03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc52a08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbe52a0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8052a0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e52a10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x27252a12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6252a14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc52a18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e52a1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d652a1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa52a25, SUC_lexicon_tags_17 },
    { 0xf5652a25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61052a28, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c52a2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a52a2e, SUC_lexicon_tags_25 },
    { 0x93e52a2e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e52a36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90652a3a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf852a3c, SUC_lexicon_tags_30 },
    { 0xac852a3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e52a42, SUC_lexicon_tags_27 },
    { 0x27652a43, SUC_lexicon_tags_17 },
    { 0xe9252a43, SUC_lexicon_tags_26 },
    { 0xdfc52a45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f252a48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e52a4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0252a4d, SUC_lexicon_tags_39 },
    { 0x3c52a4e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd5c52a50, SUC_lexicon_tags_17 },
    { 0x80c52a51, SUC_lexicon_tags_47 },
    { 0x13a52a52, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f852a56, SUC_lexicon_tags_17 },
    { 0x40a52a57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84e52a59, SUC_lexicon_tags_87 },
    { 0xab252a5a, SUC_lexicon_tags_45 },
    { 0x71e52a5b, SUC_lexicon_tags_11 },
    { 0xaaa52a5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e52a61, SUC_lexicon_tags_1 },
    { 0xa052a62, SUC_lexicon_tags_37 },
    { 0x88852a63, SUC_lexicon_tags_17 },
    { 0x59852a61, SUC_lexicon_tags_25 },
    { 0x70e52a62, SUC_lexicon_tags_11 },
    { 0x4c452a63, SUC_lexicon_tags_41 },
    { 0x3d252a67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75a52a69, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c52a6c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2252a73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc6652a75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8052a77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa452a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e052a7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x44a52a7f, SUC_lexicon_tags_87 },
    { 0x89a52a80, SUC_lexicon_tags_79 },
    { 0x77052a7f, SUC_lexicon_tags_38 },
    { 0x7b852a82, SUC_lexicon_tags_44 },
    { 0x6852a81, SUC_lexicon_tags_29 },
    { 0x7e52a84, SUC_lexicon_tags_1 },
    { 0xeca52a7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8c52a87, SUC_lexicon_tags_7 },
    { 0xe8a52a88, SUC_lexicon_tags_44 },
    { 0xf2652a89, SUC_lexicon_tags_12 },
    { 0x60652a8a, SUC_lexicon_tags_17 },
    { 0xe5e52a88, SUC_lexicon_tags_14 },
    { 0x42452a8a, SUC_lexicon_tags_68 },
    { 0x7ca52a88, SUC_lexicon_tags_63 },
    { 0xb0052a8e, SUC_lexicon_tags_8 },
    { 0xd4452a89, SUC_lexicon_tags_11 },
    { 0xaa052a90, SUC_lexicon_tags_48 },
    { 0x2ee52a91, SUC_lexicon_tags_62 },
    { 0x8b252a91, SUC_lexicon_tags_55 },
    { 0x43252a93, SUC_lexicon_tags_21 },
    { 0x27252a89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6652a96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74452a9e, SUC_lexicon_tags_12 },
    { 0x5a852a9f, SUC_lexicon_tags_17 },
    { 0xa4052aa0, SUC_lexicon_tags_55 },
    { 0xc652aa0, SUC_lexicon_tags_74 },
    { 0x65452aa2, SUC_lexicon_tags_25 },
    { 0x4b652aa3, SUC_lexicon_tags_8 },
    { 0xdb652aa4, SUC_lexicon_tags_7 },
    { 0xe4452aa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac52aa9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70a52aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91c52aad, SUC_lexicon_tags_27 },
    { 0x95052aad, SUC_lexicon_tags_23 },
    { 0xdd052aad, SUC_lexicon_tags_29 },
    { 0xae452aae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x64252ab3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25252ab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6452aba, SUC_lexicon_tags_63 },
    { 0xda52abb, SUC_lexicon_tags_54 },
    { 0x92c52abc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32c52abe, SUC_lexicon_tags_0 },
    { 0x24052abf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a252aca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x87652acd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9c252acf, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea52ad4, SUC_lexicon_tags_47 },
    { 0xf2452ad4, SUC_lexicon_tags_767 },
    { 0, NULL },
    { 0x53a52ad7, SUC_lexicon_tags_37 },
    { 0x3e052ad8, SUC_lexicon_tags_11 },
    { 0x9f852ad9, SUC_lexicon_tags_0 },
    { 0xaa052ad7, SUC_lexicon_tags_11 },
    { 0x5d052adb, SUC_lexicon_tags_30 },
    { 0x65252adc, SUC_lexicon_tags_39 },
    { 0x6b452add, SUC_lexicon_tags_31 },
    { 0xa4852ade, SUC_lexicon_tags_26 },
    { 0x3d852adf, SUC_lexicon_tags_37 },
    { 0xcb852ae0, SUC_lexicon_tags_4 },
    { 0x6d652ae0, SUC_lexicon_tags_27 },
    { 0x6b652ad9, SUC_lexicon_tags_87 },
    { 0x85c52add, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe4852ae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b452ae7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f452aeb, SUC_lexicon_tags_16 },
    { 0x1ce52aec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0052aef, SUC_lexicon_tags_1 },
    { 0x3ba52aef, SUC_lexicon_tags_25 },
    { 0xb5a52af1, SUC_lexicon_tags_23 },
    { 0x47652af2, SUC_lexicon_tags_23 },
    { 0xa3452af3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd652af9, SUC_lexicon_tags_47 },
    { 0xfb052af9, SUC_lexicon_tags_17 },
    { 0x2ce52afb, SUC_lexicon_tags_65 },
    { 0x7aa52afb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c52b00, SUC_lexicon_tags_39 },
    { 0x81452b00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83852b03, SUC_lexicon_tags_11 },
    { 0xcb452b04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x32252b06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e452b0a, SUC_lexicon_tags_11 },
    { 0x5fc52b0b, SUC_lexicon_tags_7 },
    { 0x3ea52b0b, SUC_lexicon_tags_23 },
    { 0xe8852b0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe7452b0f, SUC_lexicon_tags_15 },
    { 0x88a52b10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50c52b12, SUC_lexicon_tags_14 },
    { 0x15652b13, SUC_lexicon_tags_63 },
    { 0xec852b13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde252b17, SUC_lexicon_tags_27 },
    { 0xf8e52b18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a052b1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6652b1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7a852b21, SUC_lexicon_tags_27 },
    { 0x4b252b22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea052b2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34852b2c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed052b31, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xbd852b33, SUC_lexicon_tags_37 },
    { 0x9e252b34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac652b3c, SUC_lexicon_tags_19 },
    { 0xd4252b3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c52b42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64a52b44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb452b46, SUC_lexicon_tags_108 },
    { 0xd9c52b46, SUC_lexicon_tags_31 },
    { 0xbf052b48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e52b57, SUC_lexicon_tags_9 },
    { 0xe0e52b58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa52b5c, SUC_lexicon_tags_7 },
    { 0x6ca52b5d, SUC_lexicon_tags_17 },
    { 0x4b652b5e, SUC_lexicon_tags_1 },
    { 0x2a052b5f, SUC_lexicon_tags_38 },
    { 0xf0452b5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5852b62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e52b66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71652b69, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb852b6d, SUC_lexicon_tags_30 },
    { 0x9fc52b6e, SUC_lexicon_tags_25 },
    { 0xfa852b6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a52b75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c52b7a, SUC_lexicon_tags_28 },
    { 0x78452b7a, SUC_lexicon_tags_31 },
    { 0xf7e52b7c, SUC_lexicon_tags_14 },
    { 0xec652b7d, SUC_lexicon_tags_45 },
    { 0xdd452b7e, SUC_lexicon_tags_11 },
    { 0x16a52b7e, SUC_lexicon_tags_14 },
    { 0x49a52b7f, SUC_lexicon_tags_35 },
    { 0x96a52b80, SUC_lexicon_tags_31 },
    { 0xc7252b82, SUC_lexicon_tags_79 },
    { 0x21252b82, SUC_lexicon_tags_2 },
    { 0xd0652b84, SUC_lexicon_tags_37 },
    { 0xd5c52b7f, SUC_lexicon_tags_1 },
    { 0xdea52b86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19c52b88, SUC_lexicon_tags_1 },
    { 0xda652b89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c52b8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e52b90, SUC_lexicon_tags_32 },
    { 0x9252b91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7052b98, SUC_lexicon_tags_32 },
    { 0xa1252b98, SUC_lexicon_tags_7 },
    { 0x3f852b99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b852b9c, SUC_lexicon_tags_63 },
    { 0x2fe52b9c, SUC_lexicon_tags_70 },
    { 0x2ee52b9c, SUC_lexicon_tags_7 },
    { 0xfe652b9f, SUC_lexicon_tags_1 },
    { 0x4b852ba0, SUC_lexicon_tags_194 },
    { 0x9aa52ba1, SUC_lexicon_tags_7 },
    { 0x56452ba2, SUC_lexicon_tags_15 },
    { 0xf2252ba3, SUC_lexicon_tags_30 },
    { 0x80252ba0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdd052ba6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7452bab, SUC_lexicon_tags_2 },
    { 0xabe52bab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8652bb1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27852bb9, SUC_lexicon_tags_9 },
    { 0xa8852bb9, SUC_lexicon_tags_302 },
    { 0, NULL },
    { 0, NULL },
    { 0xcea52bbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6c52bbf, SUC_lexicon_tags_17 },
    { 0x57052bc0, SUC_lexicon_tags_68 },
    { 0x7ca52bc1, SUC_lexicon_tags_12 },
    { 0x7a252bc2, SUC_lexicon_tags_19 },
    { 0x3aa52bc3, SUC_lexicon_tags_37 },
    { 0xe9652bc3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x74a52bc6, SUC_lexicon_tags_17 },
    { 0x8052bc6, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80652bcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81852bcf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c52bd5, SUC_lexicon_tags_17 },
    { 0x83852bd6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8b252bd8, SUC_lexicon_tags_79 },
    { 0x48452bd9, SUC_lexicon_tags_12 },
    { 0x30e52bda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf8c52bdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2252be0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x90052be3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32052be7, SUC_lexicon_tags_46 },
    { 0xcc652be8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed652bef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa52bf6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x99252bf8, SUC_lexicon_tags_302 },
    { 0, NULL },
    { 0x92e52bfa, SUC_lexicon_tags_23 },
    { 0x10252bfa, SUC_lexicon_tags_26 },
    { 0x88e52bfc, SUC_lexicon_tags_23 },
    { 0x5ac52bfc, SUC_lexicon_tags_59 },
    { 0x1f252bfe, SUC_lexicon_tags_11 },
    { 0x27c52bfa, SUC_lexicon_tags_45 },
    { 0x66a52c00, SUC_lexicon_tags_18 },
    { 0x96e52c01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d252c04, SUC_lexicon_tags_55 },
    { 0x60252c05, SUC_lexicon_tags_9 },
    { 0xa52c06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3c52c08, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce52c0b, SUC_lexicon_tags_37 },
    { 0x12052c0b, SUC_lexicon_tags_26 },
    { 0x13852c0d, SUC_lexicon_tags_21 },
    { 0x28652c0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6652c11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e52c14, SUC_lexicon_tags_27 },
    { 0x4ce52c15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57452c19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x15a52c1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a052c1d, SUC_lexicon_tags_3 },
    { 0xc9052c1d, SUC_lexicon_tags_17 },
    { 0x63652c1f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c452c23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e52c27, SUC_lexicon_tags_39 },
    { 0x1e252c28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf052c2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x71652c2e, SUC_lexicon_tags_53 },
    { 0x92c52c2f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x16a52c31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3652c33, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4a252c35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb252c37, SUC_lexicon_tags_11 },
    { 0x3e52c37, SUC_lexicon_tags_15 },
    { 0xb1452c39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6652c3c, SUC_lexicon_tags_4 },
    { 0x3d452c3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a52c42, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2652c46, SUC_lexicon_tags_37 },
    { 0x27452c47, SUC_lexicon_tags_17 },
    { 0x75252c46, SUC_lexicon_tags_37 },
    { 0x87852c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6652c4b, SUC_lexicon_tags_46 },
    { 0xc2652c4c, SUC_lexicon_tags_27 },
    { 0x3e052c4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x44852c4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb452c51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a52c5d, SUC_lexicon_tags_8 },
    { 0xd252c5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc9852c60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86652c62, SUC_lexicon_tags_33 },
    { 0xeba52c63, SUC_lexicon_tags_4 },
    { 0x452c64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9452c68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3252c6c, SUC_lexicon_tags_3 },
    { 0x27c52c6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53852c6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac52c75, SUC_lexicon_tags_0 },
    { 0x98052c76, SUC_lexicon_tags_55 },
    { 0x99052c77, SUC_lexicon_tags_37 },
    { 0xdca52c78, SUC_lexicon_tags_32 },
    { 0x77a52c77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16c52c7b, SUC_lexicon_tags_28 },
    { 0x6e452c7c, SUC_lexicon_tags_171 },
    { 0x5f852c7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57652c7f, SUC_lexicon_tags_14 },
    { 0xec52c80, SUC_lexicon_tags_27 },
    { 0x55052c81, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a52c85, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9be52c87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93452c8e, SUC_lexicon_tags_1 },
    { 0x11e52c8e, SUC_lexicon_tags_43 },
    { 0x73e52c8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1652c93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xba852c95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b052c98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0852c9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbd452c9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41c52c9e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xac52ca0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0252ca2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x59c52ca4, SUC_lexicon_tags_38 },
    { 0x87c52ca4, SUC_lexicon_tags_26 },
    { 0x50652ca6, SUC_lexicon_tags_27 },
    { 0xa1e52ca7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5852ca9, SUC_lexicon_tags_63 },
    { 0x90652caa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c452cad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b252cb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25252cb2, SUC_lexicon_tags_33 },
    { 0xe3852cb3, SUC_lexicon_tags_37 },
    { 0x39c52cb4, SUC_lexicon_tags_7 },
    { 0xb6a52cb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39452cbf, SUC_lexicon_tags_68 },
    { 0x96452cc0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe1a52cc2, SUC_lexicon_tags_59 },
    { 0x55252cc2, SUC_lexicon_tags_48 },
    { 0x4de52cc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x10252cc6, SUC_lexicon_tags_66 },
    { 0xd252cc7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1e52cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f252ccd, SUC_lexicon_tags_2 },
    { 0x7252ccd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1252cd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43852cd4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe5e52cd6, SUC_lexicon_tags_27 },
    { 0x7de52cd7, SUC_lexicon_tags_34 },
    { 0xc2e52cd7, SUC_lexicon_tags_7 },
    { 0xa3652cd9, SUC_lexicon_tags_75 },
    { 0x4cc52cda, SUC_lexicon_tags_25 },
    { 0x11052cd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe52cdf, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x51652ce1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91852ce3, SUC_lexicon_tags_4 },
    { 0x6f252ce4, SUC_lexicon_tags_37 },
    { 0xfea52ce5, SUC_lexicon_tags_37 },
    { 0xb4c52ce6, SUC_lexicon_tags_23 },
    { 0x5be52ce7, SUC_lexicon_tags_35 },
    { 0xa5c52ce6, SUC_lexicon_tags_47 },
    { 0xdc52ce5, SUC_lexicon_tags_17 },
    { 0xb5c52cea, SUC_lexicon_tags_3 },
    { 0x9ee52ce5, SUC_lexicon_tags_30 },
    { 0x7652cec, SUC_lexicon_tags_37 },
    { 0x58252ce6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8052cf0, SUC_lexicon_tags_23 },
    { 0xa9e52cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c52cf8, SUC_lexicon_tags_49 },
    { 0x38852cf9, SUC_lexicon_tags_79 },
    { 0xe3252cfa, SUC_lexicon_tags_37 },
    { 0x1ec52cfa, SUC_lexicon_tags_30 },
    { 0xcde52cfc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e52cff, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x27252d02, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1452d11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7252d13, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43452d17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50052d19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d852d1c, SUC_lexicon_tags_28 },
    { 0x6852d1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11a52d1f, SUC_lexicon_tags_1 },
    { 0xfe452d20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9852d29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20252d2b, SUC_lexicon_tags_17 },
    { 0x46a52d2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d052d38, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8452d3b, SUC_lexicon_tags_37 },
    { 0xc2c52d3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1652d3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9852d40, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42452d45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7c452d47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb052d4f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe052d53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae852d55, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x55452d57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa52d5b, SUC_lexicon_tags_55 },
    { 0x97452d5c, SUC_lexicon_tags_7 },
    { 0x9cc52d5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d852d60, SUC_lexicon_tags_1 },
    { 0xe4a52d61, SUC_lexicon_tags_12 },
    { 0x252d62, SUC_lexicon_tags_43 },
    { 0x47852d61, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f852d66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d852d68, SUC_lexicon_tags_23 },
    { 0xf7052d69, SUC_lexicon_tags_2 },
    { 0x7d052d68, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6652d70, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a52d75, SUC_lexicon_tags_49 },
    { 0x6fe52d75, SUC_lexicon_tags_17 },
    { 0xba652d75, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe3252d79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96052d7d, SUC_lexicon_tags_2 },
    { 0x5e652d7e, SUC_lexicon_tags_12 },
    { 0x30a52d7f, SUC_lexicon_tags_15 },
    { 0x6a452d80, SUC_lexicon_tags_1 },
    { 0x8c452d80, SUC_lexicon_tags_37 },
    { 0x37852d82, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd0852d84, SUC_lexicon_tags_37 },
    { 0x78052d85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12852d88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a252d8b, SUC_lexicon_tags_25 },
    { 0xa6452d8c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2ec52d8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe052d90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2652d94, SUC_lexicon_tags_14 },
    { 0x3c852d95, SUC_lexicon_tags_37 },
    { 0xb9452d96, SUC_lexicon_tags_33 },
    { 0x1b852d97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5452d9a, SUC_lexicon_tags_53 },
    { 0xcf852d9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1652d9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x52e52d9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec52da5, SUC_lexicon_tags_17 },
    { 0xe3c52da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b652da8, SUC_lexicon_tags_57 },
    { 0xbb052da8, SUC_lexicon_tags_15 },
    { 0xac652daa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9d052dac, SUC_lexicon_tags_11 },
    { 0xef452dad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46852db1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a52db4, SUC_lexicon_tags_23 },
    { 0xdae52db4, SUC_lexicon_tags_63 },
    { 0xa5052db6, SUC_lexicon_tags_12 },
    { 0x6e652db5, SUC_lexicon_tags_33 },
    { 0x1c452db8, SUC_lexicon_tags_32 },
    { 0x21c52db9, SUC_lexicon_tags_14 },
    { 0xec452db9, SUC_lexicon_tags_43 },
    { 0xf6852dbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27052dc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a52dc6, SUC_lexicon_tags_30 },
    { 0x8de52dc6, SUC_lexicon_tags_1 },
    { 0x13652dc8, SUC_lexicon_tags_37 },
    { 0xd9052dc9, SUC_lexicon_tags_11 },
    { 0x65e52dca, SUC_lexicon_tags_4 },
    { 0x56052dc7, SUC_lexicon_tags_15 },
    { 0x28a52dcc, SUC_lexicon_tags_176 },
    { 0, NULL },
    { 0x45c52dce, SUC_lexicon_tags_27 },
    { 0x7e52dce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18652dd8, SUC_lexicon_tags_2 },
    { 0x4c052dd8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc52de0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf452de2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a52de5, SUC_lexicon_tags_6 },
    { 0x29a52de6, SUC_lexicon_tags_49 },
    { 0xc4852de7, SUC_lexicon_tags_20 },
    { 0x2b452de8, SUC_lexicon_tags_60 },
    { 0xaac52de8, SUC_lexicon_tags_80 },
    { 0xc6652de7, SUC_lexicon_tags_88 },
    { 0x9da52de7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3652df5, SUC_lexicon_tags_12 },
    { 0x91052df5, SUC_lexicon_tags_7 },
    { 0xdca52df6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd852dfb, SUC_lexicon_tags_11 },
    { 0x38e52dfc, SUC_lexicon_tags_21 },
    { 0xf3e52dfd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73852e01, SUC_lexicon_tags_47 },
    { 0x24c52e01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a452e04, SUC_lexicon_tags_62 },
    { 0x4f252e05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c52e10, SUC_lexicon_tags_11 },
    { 0xc7652e10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e052e15, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x86252e17, SUC_lexicon_tags_37 },
    { 0xe052e18, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e52e1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee652e21, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd9c52e23, SUC_lexicon_tags_158 },
    { 0x12452e23, SUC_lexicon_tags_61 },
    { 0x49c52e25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa52e2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e52e2e, SUC_lexicon_tags_9 },
    { 0x72e52e2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10a52e31, SUC_lexicon_tags_17 },
    { 0x22452e31, SUC_lexicon_tags_33 },
    { 0x26052e33, SUC_lexicon_tags_15 },
    { 0x5f052e31, SUC_lexicon_tags_17 },
    { 0x6a252e33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6852e37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e52e3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f052e41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf052e44, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5052e47, SUC_lexicon_tags_45 },
    { 0x84e52e48, SUC_lexicon_tags_9 },
    { 0xd0e52e49, SUC_lexicon_tags_35 },
    { 0xf252e4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3252e55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea52e58, SUC_lexicon_tags_33 },
    { 0x52852e59, SUC_lexicon_tags_2 },
    { 0xdc452e59, SUC_lexicon_tags_25 },
    { 0x3c052e5b, SUC_lexicon_tags_45 },
    { 0x96c52e5c, SUC_lexicon_tags_10 },
    { 0x69a52e5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72052e5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce852e61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9452e66, SUC_lexicon_tags_16 },
    { 0xa6452e66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf0852e69, SUC_lexicon_tags_17 },
    { 0x44252e6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16452e6c, SUC_lexicon_tags_12 },
    { 0x7e252e6c, SUC_lexicon_tags_11 },
    { 0x4d652e6e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91052e73, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d252e76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4452e78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89c52e7a, SUC_lexicon_tags_4 },
    { 0xb2252e7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b452e7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c52e86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b852e8c, SUC_lexicon_tags_10 },
    { 0xfda52e8c, SUC_lexicon_tags_25 },
    { 0x3652e8e, SUC_lexicon_tags_9 },
    { 0xf4252e8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45452e93, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x99852e95, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x57e52e97, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc52e9c, SUC_lexicon_tags_25 },
    { 0x6ae52e9c, SUC_lexicon_tags_25 },
    { 0x82a52e9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a52ea2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59252ea7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5252eab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe652eb4, SUC_lexicon_tags_63 },
    { 0xe7652eb4, SUC_lexicon_tags_25 },
    { 0x15452eb4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de52eb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe852ec1, SUC_lexicon_tags_151 },
    { 0xaee52ec1, SUC_lexicon_tags_11 },
    { 0xa852ec3, SUC_lexicon_tags_14 },
    { 0xb5852ec1, SUC_lexicon_tags_28 },
    { 0xf1c52ec5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c52ec8, SUC_lexicon_tags_60 },
    { 0x6b452ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xace52ecb, SUC_lexicon_tags_15 },
    { 0x5e452ecc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6052ed1, SUC_lexicon_tags_32 },
    { 0x2b052ed2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7f852ed4, SUC_lexicon_tags_11 },
    { 0x7c252ed4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16052ed8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e252eda, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c52ede, SUC_lexicon_tags_27 },
    { 0x7052edf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa452ee5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x77652ee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c52eea, SUC_lexicon_tags_15 },
    { 0xad652eeb, SUC_lexicon_tags_46 },
    { 0x98c52eec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19852ef6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76652ef8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e52efb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee452f00, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e452f05, SUC_lexicon_tags_23 },
    { 0x34a52f06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef252f08, SUC_lexicon_tags_7 },
    { 0x9b852f09, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x52652f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95c52f0d, SUC_lexicon_tags_33 },
    { 0x57a52f0e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74052f14, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0052f17, SUC_lexicon_tags_9 },
    { 0x21c52f18, SUC_lexicon_tags_7 },
    { 0x7b452f19, SUC_lexicon_tags_17 },
    { 0x50252f1a, SUC_lexicon_tags_11 },
    { 0xf9252f1a, SUC_lexicon_tags_15 },
    { 0xc9c52f19, SUC_lexicon_tags_12 },
    { 0xc2852f18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6452f23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed652f28, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x45252f2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89c52f2d, SUC_lexicon_tags_10 },
    { 0xb0852f2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80452f32, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a052f36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe52f39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c52f3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x26852f42, SUC_lexicon_tags_33 },
    { 0x1c252f43, SUC_lexicon_tags_12 },
    { 0x51a52f44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5652f4c, SUC_lexicon_tags_15 },
    { 0x14052f4d, SUC_lexicon_tags_7 },
    { 0xe9852f4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8252f51, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc52f58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a852f5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f052f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1252f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57252f62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93452f64, SUC_lexicon_tags_77 },
    { 0x56452f65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29052f69, SUC_lexicon_tags_33 },
    { 0xb2452f69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcde52f6c, SUC_lexicon_tags_15 },
    { 0x7ac52f6c, SUC_lexicon_tags_61 },
    { 0xc4052f6e, SUC_lexicon_tags_31 },
    { 0xe0252f6c, SUC_lexicon_tags_37 },
    { 0xd3052f70, SUC_lexicon_tags_19 },
    { 0xdce52f71, SUC_lexicon_tags_0 },
    { 0x15652f72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c52f75, SUC_lexicon_tags_32 },
    { 0x11452f75, SUC_lexicon_tags_37 },
    { 0x8d052f77, SUC_lexicon_tags_36 },
    { 0x45652f76, SUC_lexicon_tags_29 },
    { 0x9fc52f76, SUC_lexicon_tags_84 },
    { 0x1d052f75, SUC_lexicon_tags_27 },
    { 0xc4252f7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc52f7f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd7652f81, SUC_lexicon_tags_7 },
    { 0xbaa52f81, SUC_lexicon_tags_37 },
    { 0xb5e52f82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa652f8c, SUC_lexicon_tags_19 },
    { 0xe5852f8d, SUC_lexicon_tags_21 },
    { 0xfc652f8e, SUC_lexicon_tags_47 },
    { 0x11852f8f, SUC_lexicon_tags_25 },
    { 0x28852f8f, SUC_lexicon_tags_21 },
    { 0x69a52f91, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4852f95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0852f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b852f9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e52fa0, SUC_lexicon_tags_43 },
    { 0xa0e52fa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8852fa4, SUC_lexicon_tags_13 },
    { 0x17252fa5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4e452fa7, SUC_lexicon_tags_32 },
    { 0x4c852fa8, SUC_lexicon_tags_15 },
    { 0x18e52fa7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3052faf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7de52fb1, SUC_lexicon_tags_7 },
    { 0x9b652fb1, SUC_lexicon_tags_60 },
    { 0x6b452fb1, SUC_lexicon_tags_47 },
    { 0x31a52fb4, SUC_lexicon_tags_12 },
    { 0xb9652fb1, SUC_lexicon_tags_15 },
    { 0x1f052fb6, SUC_lexicon_tags_43 },
    { 0x61852fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28452fbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc52fc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a52fcc, SUC_lexicon_tags_17 },
    { 0xb3c52fcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7052fd1, SUC_lexicon_tags_25 },
    { 0x27a52fd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e52fd9, SUC_lexicon_tags_1 },
    { 0x46452fd9, SUC_lexicon_tags_17 },
    { 0x3aa52fd9, SUC_lexicon_tags_17 },
    { 0xb1a52fdc, SUC_lexicon_tags_11 },
    { 0x17c52fdc, SUC_lexicon_tags_15 },
    { 0x46c52fde, SUC_lexicon_tags_10 },
    { 0x7f452fdf, SUC_lexicon_tags_87 },
    { 0x22e52fd9, SUC_lexicon_tags_37 },
    { 0x32652fe1, SUC_lexicon_tags_17 },
    { 0x4c52fdd, SUC_lexicon_tags_27 },
    { 0x31652fde, SUC_lexicon_tags_25 },
    { 0x2052fdc, SUC_lexicon_tags_66 },
    { 0x3a052fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a52fec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf252fef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52452ff1, SUC_lexicon_tags_35 },
    { 0xfc052ff2, SUC_lexicon_tags_152 },
    { 0xe1a52ff1, SUC_lexicon_tags_25 },
    { 0x22a52ff3, SUC_lexicon_tags_14 },
    { 0x8a52ff5, SUC_lexicon_tags_47 },
    { 0x1ba52ff6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e52ffa, SUC_lexicon_tags_37 },
    { 0x42a52ffb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a52ffe, SUC_lexicon_tags_8 },
    { 0x1ba52fff, SUC_lexicon_tags_47 },
    { 0xb7453000, SUC_lexicon_tags_12 },
    { 0x86c53000, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc853004, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c25300d, SUC_lexicon_tags_1 },
    { 0xc185300e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9da53010, SUC_lexicon_tags_68 },
    { 0xe3253011, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3853018, SUC_lexicon_tags_1 },
    { 0x39c53018, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc825301c, SUC_lexicon_tags_45 },
    { 0x28a5301d, SUC_lexicon_tags_0 },
    { 0xfde5301d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x64653021, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa653027, SUC_lexicon_tags_37 },
    { 0xb2453028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c85302a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9465302f, SUC_lexicon_tags_17 },
    { 0x47253030, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59053036, SUC_lexicon_tags_30 },
    { 0xde253036, SUC_lexicon_tags_6 },
    { 0xc3a53038, SUC_lexicon_tags_8 },
    { 0xd9c53039, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3e25303b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6253041, SUC_lexicon_tags_27 },
    { 0xc453042, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x45c53044, SUC_lexicon_tags_85 },
    { 0xb0e53045, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x45253047, SUC_lexicon_tags_17 },
    { 0xe9a53048, SUC_lexicon_tags_49 },
    { 0x78453049, SUC_lexicon_tags_39 },
    { 0xb685304a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7385304f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58653054, SUC_lexicon_tags_0 },
    { 0xfa53054, SUC_lexicon_tags_11 },
    { 0x6e053055, SUC_lexicon_tags_1 },
    { 0xd2e53057, SUC_lexicon_tags_32 },
    { 0x7a053057, SUC_lexicon_tags_138 },
    { 0x46253059, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x61c5305b, SUC_lexicon_tags_107 },
    { 0xd685305b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d65305f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c653061, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d853066, SUC_lexicon_tags_15 },
    { 0xcaa53066, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee45306a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac5306d, SUC_lexicon_tags_2 },
    { 0xd05306e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2453073, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68253075, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd705307c, SUC_lexicon_tags_84 },
    { 0xea05307d, SUC_lexicon_tags_25 },
    { 0x5b65307d, SUC_lexicon_tags_17 },
    { 0xba25307f, SUC_lexicon_tags_11 },
    { 0x2d85307e, SUC_lexicon_tags_45 },
    { 0xe9253081, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x27853084, SUC_lexicon_tags_2 },
    { 0x79253085, SUC_lexicon_tags_7 },
    { 0x58853085, SUC_lexicon_tags_25 },
    { 0xfec53087, SUC_lexicon_tags_2 },
    { 0xb8e53088, SUC_lexicon_tags_66 },
    { 0x3ae53084, SUC_lexicon_tags_816 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe5308c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c53095, SUC_lexicon_tags_0 },
    { 0x1d653095, SUC_lexicon_tags_55 },
    { 0x8dc53096, SUC_lexicon_tags_37 },
    { 0x61853098, SUC_lexicon_tags_55 },
    { 0xe1453098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb365309c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee5309f, SUC_lexicon_tags_0 },
    { 0xc56530a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec530a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc16530a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x54530a7, SUC_lexicon_tags_33 },
    { 0x328530a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc530ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d8530ae, SUC_lexicon_tags_25 },
    { 0x582530ae, SUC_lexicon_tags_47 },
    { 0x5ca530b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0530b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x530b8, SUC_lexicon_tags_33 },
    { 0x2d2530b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58530bb, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30530bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92530c2, SUC_lexicon_tags_32 },
    { 0x7fe530c3, SUC_lexicon_tags_63 },
    { 0x34e530c4, SUC_lexicon_tags_7 },
    { 0x3ac530c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698530cd, SUC_lexicon_tags_4 },
    { 0x198530cd, SUC_lexicon_tags_167 },
    { 0x5a8530cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfec530d1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3b4530d3, SUC_lexicon_tags_31 },
    { 0xce8530d4, SUC_lexicon_tags_15 },
    { 0x504530d5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2530da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x988530dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526530e2, SUC_lexicon_tags_15 },
    { 0x876530e3, SUC_lexicon_tags_47 },
    { 0x8f6530e2, SUC_lexicon_tags_45 },
    { 0x730530e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x770530e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110530f0, SUC_lexicon_tags_38 },
    { 0x894530f1, SUC_lexicon_tags_13 },
    { 0xb08530f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9a530f4, SUC_lexicon_tags_85 },
    { 0x6c4530f5, SUC_lexicon_tags_17 },
    { 0x2b4530f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc530fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f453100, SUC_lexicon_tags_7 },
    { 0xac853100, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4c53103, SUC_lexicon_tags_40 },
    { 0x7e053103, SUC_lexicon_tags_17 },
    { 0x1e053105, SUC_lexicon_tags_14 },
    { 0x5ca53104, SUC_lexicon_tags_17 },
    { 0x3ca53107, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1365310a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe325310e, SUC_lexicon_tags_27 },
    { 0xb0a5310e, SUC_lexicon_tags_26 },
    { 0x81c53110, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32853116, SUC_lexicon_tags_0 },
    { 0x83a53117, SUC_lexicon_tags_60 },
    { 0x96853118, SUC_lexicon_tags_5 },
    { 0xe6453118, SUC_lexicon_tags_0 },
    { 0xea45311a, SUC_lexicon_tags_15 },
    { 0x66a53117, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d453120, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b253122, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96453124, SUC_lexicon_tags_14 },
    { 0x33253124, SUC_lexicon_tags_11 },
    { 0x85e53125, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd6a53128, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8e5312a, SUC_lexicon_tags_19 },
    { 0x3825312b, SUC_lexicon_tags_17 },
    { 0x27c5312c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5312f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3453134, SUC_lexicon_tags_1 },
    { 0x31653134, SUC_lexicon_tags_33 },
    { 0x75853134, SUC_lexicon_tags_15 },
    { 0xab453137, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc5313f, SUC_lexicon_tags_0 },
    { 0xbd053140, SUC_lexicon_tags_26 },
    { 0x6b453141, SUC_lexicon_tags_77 },
    { 0x6b85313f, SUC_lexicon_tags_2 },
    { 0xf2453142, SUC_lexicon_tags_5 },
    { 0xae53144, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c53151, SUC_lexicon_tags_27 },
    { 0x7de53152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd453154, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55853158, SUC_lexicon_tags_60 },
    { 0x9fe53158, SUC_lexicon_tags_12 },
    { 0x1845315a, SUC_lexicon_tags_25 },
    { 0xb8653159, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde53160, SUC_lexicon_tags_25 },
    { 0xef853161, SUC_lexicon_tags_11 },
    { 0x11053162, SUC_lexicon_tags_25 },
    { 0x77253163, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8253165, SUC_lexicon_tags_29 },
    { 0x1c053165, SUC_lexicon_tags_85 },
    { 0x51053167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c5316a, SUC_lexicon_tags_15 },
    { 0x6525316a, SUC_lexicon_tags_49 },
    { 0x5545316c, SUC_lexicon_tags_17 },
    { 0xf5a5316d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e53172, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52453176, SUC_lexicon_tags_11 },
    { 0x8aa53177, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3165317a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e5317e, SUC_lexicon_tags_25 },
    { 0x3aa5317f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44e53181, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba53184, SUC_lexicon_tags_37 },
    { 0xe4053185, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a853189, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72e5318b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56253193, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf853198, SUC_lexicon_tags_6 },
    { 0x6a253198, SUC_lexicon_tags_37 },
    { 0x2b053199, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc5319e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2531a1, SUC_lexicon_tags_49 },
    { 0xd66531a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c531a6, SUC_lexicon_tags_11 },
    { 0x664531a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3de531a9, SUC_lexicon_tags_37 },
    { 0xf64531aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x162531ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504531b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ac531b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c531b7, SUC_lexicon_tags_2 },
    { 0x26a531b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae531bc, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2531c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4531c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748531c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab0531cb, SUC_lexicon_tags_25 },
    { 0xe18531cc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e531d1, SUC_lexicon_tags_19 },
    { 0x370531d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8531d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28531db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc86531dd, SUC_lexicon_tags_2 },
    { 0x3b8531de, SUC_lexicon_tags_10 },
    { 0xfee531de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a531e2, SUC_lexicon_tags_15 },
    { 0x308531e3, SUC_lexicon_tags_0 },
    { 0xb48531e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x702531e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6531ec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf64531ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428531f6, SUC_lexicon_tags_37 },
    { 0xa36531f7, SUC_lexicon_tags_66 },
    { 0xbc6531f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce531fe, SUC_lexicon_tags_4 },
    { 0xbb4531ff, SUC_lexicon_tags_56 },
    { 0xd0c53200, SUC_lexicon_tags_49 },
    { 0x92531fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef853205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c53208, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83c5320a, SUC_lexicon_tags_1 },
    { 0xcb65320a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x745320e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f053216, SUC_lexicon_tags_1 },
    { 0x92c53217, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x42a53219, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda5321f, SUC_lexicon_tags_27 },
    { 0x33653220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3453224, SUC_lexicon_tags_181 },
    { 0x22a53225, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2253228, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb525322e, SUC_lexicon_tags_1 },
    { 0xa0a5322f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41453234, SUC_lexicon_tags_25 },
    { 0xee53235, SUC_lexicon_tags_9 },
    { 0x48a53236, SUC_lexicon_tags_17 },
    { 0x32653235, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c05323e, SUC_lexicon_tags_61 },
    { 0xc05323e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbba53241, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa0653243, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b253250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ec53252, SUC_lexicon_tags_28 },
    { 0x4c453252, SUC_lexicon_tags_44 },
    { 0x69653254, SUC_lexicon_tags_8 },
    { 0x7c253255, SUC_lexicon_tags_10 },
    { 0xe9053253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5325b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e5325e, SUC_lexicon_tags_37 },
    { 0x25c5325f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb3c53261, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x72253264, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c253268, SUC_lexicon_tags_10 },
    { 0xd3a53269, SUC_lexicon_tags_63 },
    { 0xbd85326a, SUC_lexicon_tags_25 },
    { 0x3a25326b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16653271, SUC_lexicon_tags_68 },
    { 0x9f253272, SUC_lexicon_tags_17 },
    { 0xb3453273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27853276, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c25327f, SUC_lexicon_tags_2 },
    { 0xd4c53280, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3453282, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2405328c, SUC_lexicon_tags_13 },
    { 0x4f05328d, SUC_lexicon_tags_45 },
    { 0x7825328e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c53291, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x67053293, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22453295, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe653298, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412532a0, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x7da532a2, SUC_lexicon_tags_11 },
    { 0x2ec532a3, SUC_lexicon_tags_10 },
    { 0xa98532a4, SUC_lexicon_tags_57 },
    { 0xf5e532a4, SUC_lexicon_tags_4 },
    { 0x724532a4, SUC_lexicon_tags_15 },
    { 0x66c532a7, SUC_lexicon_tags_7 },
    { 0x5ec532a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca532ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e532b4, SUC_lexicon_tags_46 },
    { 0xe10532b5, SUC_lexicon_tags_27 },
    { 0x2c8532b6, SUC_lexicon_tags_38 },
    { 0x9b4532b5, SUC_lexicon_tags_39 },
    { 0x2c532b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4532be, SUC_lexicon_tags_34 },
    { 0xb40532be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2532c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4532c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe532cc, SUC_lexicon_tags_31 },
    { 0x50a532cd, SUC_lexicon_tags_63 },
    { 0xf9e532ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc64532d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc532d4, SUC_lexicon_tags_11 },
    { 0x7bc532d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe532da, SUC_lexicon_tags_1 },
    { 0xd46532da, SUC_lexicon_tags_45 },
    { 0xe8532dc, SUC_lexicon_tags_29 },
    { 0x976532dd, SUC_lexicon_tags_45 },
    { 0x8e532db, SUC_lexicon_tags_61 },
    { 0x64a532df, SUC_lexicon_tags_27 },
    { 0xaaa532e0, SUC_lexicon_tags_37 },
    { 0x130532df, SUC_lexicon_tags_1204 },
    { 0x344532e2, SUC_lexicon_tags_11 },
    { 0x494532df, SUC_lexicon_tags_17 },
    { 0x686532e2, SUC_lexicon_tags_9 },
    { 0x49a532e4, SUC_lexicon_tags_17 },
    { 0x5a8532de, SUC_lexicon_tags_13 },
    { 0xa94532e7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28532ea, SUC_lexicon_tags_11 },
    { 0x71e532ea, SUC_lexicon_tags_25 },
    { 0xbca532eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41a532ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x502532f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956532f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x664532f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1453300, SUC_lexicon_tags_1 },
    { 0x57453301, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d453305, SUC_lexicon_tags_16 },
    { 0x88653305, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0x97653309, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1545330c, SUC_lexicon_tags_28 },
    { 0x5985330d, SUC_lexicon_tags_34 },
    { 0x3945330e, SUC_lexicon_tags_1 },
    { 0xefa5330e, SUC_lexicon_tags_25 },
    { 0x5e653310, SUC_lexicon_tags_25 },
    { 0xa4853311, SUC_lexicon_tags_37 },
    { 0x74053312, SUC_lexicon_tags_19 },
    { 0xbac53313, SUC_lexicon_tags_31 },
    { 0x5225330c, SUC_lexicon_tags_13 },
    { 0x33853315, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b25331b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa5331e, SUC_lexicon_tags_2 },
    { 0x6625331f, SUC_lexicon_tags_37 },
    { 0xf025331e, SUC_lexicon_tags_25 },
    { 0xb1853321, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb253326, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7a653328, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8a5332a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a53331, SUC_lexicon_tags_37 },
    { 0xdee53332, SUC_lexicon_tags_47 },
    { 0xdae53333, SUC_lexicon_tags_252 },
    { 0x26653334, SUC_lexicon_tags_0 },
    { 0xe1653335, SUC_lexicon_tags_0 },
    { 0x4b453333, SUC_lexicon_tags_10 },
    { 0xe8453337, SUC_lexicon_tags_49 },
    { 0xddc53338, SUC_lexicon_tags_1 },
    { 0x7dc53337, SUC_lexicon_tags_68 },
    { 0x3705333a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdec5333c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d85333f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6c53341, SUC_lexicon_tags_49 },
    { 0x253342, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfb253344, SUC_lexicon_tags_15 },
    { 0xa9053345, SUC_lexicon_tags_33 },
    { 0x45a53346, SUC_lexicon_tags_17 },
    { 0x16a53347, SUC_lexicon_tags_11 },
    { 0x6de53348, SUC_lexicon_tags_15 },
    { 0x96853346, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x59e5334b, SUC_lexicon_tags_26 },
    { 0x7605334c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e53351, SUC_lexicon_tags_25 },
    { 0x6e853352, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc5335b, SUC_lexicon_tags_12 },
    { 0x79a5335c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a253362, SUC_lexicon_tags_8 },
    { 0x5a253362, SUC_lexicon_tags_1 },
    { 0x82453364, SUC_lexicon_tags_37 },
    { 0x7fe53364, SUC_lexicon_tags_25 },
    { 0x89c53364, SUC_lexicon_tags_15 },
    { 0xdf853366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25336d, SUC_lexicon_tags_37 },
    { 0x11c5336e, SUC_lexicon_tags_37 },
    { 0xb545336e, SUC_lexicon_tags_37 },
    { 0x81c5336e, SUC_lexicon_tags_37 },
    { 0xada53371, SUC_lexicon_tags_118 },
    { 0xfde53372, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da5337c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18853381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba53385, SUC_lexicon_tags_31 },
    { 0x253386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e5338b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb945338e, SUC_lexicon_tags_25 },
    { 0x1b45338f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55453393, SUC_lexicon_tags_15 },
    { 0xb2653393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e53398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6665339a, SUC_lexicon_tags_15 },
    { 0x8605339b, SUC_lexicon_tags_60 },
    { 0x2925339a, SUC_lexicon_tags_47 },
    { 0x62e5339d, SUC_lexicon_tags_47 },
    { 0x53a5339a, SUC_lexicon_tags_37 },
    { 0x5085339a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8533a5, SUC_lexicon_tags_12 },
    { 0x454533a6, SUC_lexicon_tags_743 },
    { 0xff0533a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8533ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2533af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce533b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x182533ba, SUC_lexicon_tags_26 },
    { 0x802533bb, SUC_lexicon_tags_38 },
    { 0x744533bb, SUC_lexicon_tags_15 },
    { 0x18e533bd, SUC_lexicon_tags_84 },
    { 0x838533bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de533c6, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c533c9, SUC_lexicon_tags_29 },
    { 0x898533c9, SUC_lexicon_tags_11 },
    { 0x6533ca, SUC_lexicon_tags_19 },
    { 0x6d4533c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8533cf, SUC_lexicon_tags_47 },
    { 0x472533d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e533d3, SUC_lexicon_tags_25 },
    { 0xf8e533d4, SUC_lexicon_tags_418 },
    { 0x560533d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6533d8, SUC_lexicon_tags_45 },
    { 0x5a6533d8, SUC_lexicon_tags_4 },
    { 0xe4533da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae533df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe52533e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182533e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c533eb, SUC_lexicon_tags_37 },
    { 0x334533ec, SUC_lexicon_tags_0 },
    { 0x5ca533ed, SUC_lexicon_tags_43 },
    { 0x6b8533ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x784533f0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc533f3, SUC_lexicon_tags_15 },
    { 0x3c6533f4, SUC_lexicon_tags_17 },
    { 0xe68533f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c0533f7, SUC_lexicon_tags_7 },
    { 0x2c533f8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976533fe, SUC_lexicon_tags_63 },
    { 0x5f4533ff, SUC_lexicon_tags_43 },
    { 0x63c533fe, SUC_lexicon_tags_12 },
    { 0x6e253401, SUC_lexicon_tags_11 },
    { 0x2253402, SUC_lexicon_tags_27 },
    { 0x68653403, SUC_lexicon_tags_11 },
    { 0x96053403, SUC_lexicon_tags_37 },
    { 0xbde53405, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe485340d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f25340f, SUC_lexicon_tags_2 },
    { 0x6f453410, SUC_lexicon_tags_25 },
    { 0x90253411, SUC_lexicon_tags_25 },
    { 0x4a453411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e53416, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa653419, SUC_lexicon_tags_33 },
    { 0x1e45341a, SUC_lexicon_tags_25 },
    { 0x5d45341b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c5342a, SUC_lexicon_tags_2 },
    { 0x365342b, SUC_lexicon_tags_1 },
    { 0x90a5342a, SUC_lexicon_tags_63 },
    { 0x36e5342d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65e5342f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9053432, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee53435, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d853440, SUC_lexicon_tags_15 },
    { 0x3be53440, SUC_lexicon_tags_68 },
    { 0xe2c53442, SUC_lexicon_tags_1 },
    { 0x5ac53440, SUC_lexicon_tags_231 },
    { 0x1fa53444, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f053446, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc805344d, SUC_lexicon_tags_15 },
    { 0x845344e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba53455, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x33253458, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4345345e, SUC_lexicon_tags_2 },
    { 0xa265345f, SUC_lexicon_tags_17 },
    { 0x41c5345e, SUC_lexicon_tags_27 },
    { 0xd6453461, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8453464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3465346e, SUC_lexicon_tags_17 },
    { 0xc225346e, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c53478, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5347c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50653485, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc5348b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xef65348e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b253495, SUC_lexicon_tags_40 },
    { 0xd7053495, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a53499, SUC_lexicon_tags_17 },
    { 0x8dc53499, SUC_lexicon_tags_15 },
    { 0x92a5349b, SUC_lexicon_tags_1 },
    { 0x4ba5349a, SUC_lexicon_tags_82 },
    { 0x92c5349a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414534a1, SUC_lexicon_tags_62 },
    { 0xbb2534a2, SUC_lexicon_tags_39 },
    { 0xb7a534a2, SUC_lexicon_tags_37 },
    { 0x7a2534a4, SUC_lexicon_tags_39 },
    { 0xc534a3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x342534a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8534ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5a534ae, SUC_lexicon_tags_17 },
    { 0xd16534af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd54534b1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6534b8, SUC_lexicon_tags_13 },
    { 0xfe2534b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636534bf, SUC_lexicon_tags_11 },
    { 0x3ca534c0, SUC_lexicon_tags_84 },
    { 0x89e534c1, SUC_lexicon_tags_11 },
    { 0xc68534c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50c534c4, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x5f2534c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6534cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf98534ce, SUC_lexicon_tags_54 },
    { 0xd8a534ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b4534d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da534d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8534dc, SUC_lexicon_tags_49 },
    { 0x724534dd, SUC_lexicon_tags_11 },
    { 0x52534dc, SUC_lexicon_tags_14 },
    { 0xc42534df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb72534e1, SUC_lexicon_tags_15 },
    { 0x806534e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6534e7, SUC_lexicon_tags_9 },
    { 0xfca534e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcee534ea, SUC_lexicon_tags_26 },
    { 0x140534ea, SUC_lexicon_tags_47 },
    { 0xa96534ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2534f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c534f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2534fa, SUC_lexicon_tags_7 },
    { 0x81e534fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c534ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a053502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedc53504, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c453506, SUC_lexicon_tags_15 },
    { 0xf6c53507, SUC_lexicon_tags_8 },
    { 0xd853507, SUC_lexicon_tags_32 },
    { 0xc1453509, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf825350b, SUC_lexicon_tags_21 },
    { 0x3185350c, SUC_lexicon_tags_26 },
    { 0xdc85350b, SUC_lexicon_tags_6 },
    { 0xa225350e, SUC_lexicon_tags_1 },
    { 0x9f05350f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe53511, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22253513, SUC_lexicon_tags_175 },
    { 0x57253514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e53517, SUC_lexicon_tags_15 },
    { 0x6e53517, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c53522, SUC_lexicon_tags_11 },
    { 0x7653523, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb125352a, SUC_lexicon_tags_17 },
    { 0x87a5352b, SUC_lexicon_tags_32 },
    { 0x985352c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a25352e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98853530, SUC_lexicon_tags_6 },
    { 0xb5c53531, SUC_lexicon_tags_35 },
    { 0xcd253532, SUC_lexicon_tags_37 },
    { 0x16a53531, SUC_lexicon_tags_8 },
    { 0x99653530, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c53538, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4925353e, SUC_lexicon_tags_12 },
    { 0x7045353e, SUC_lexicon_tags_2 },
    { 0xa7453540, SUC_lexicon_tags_34 },
    { 0xc4453540, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5853544, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x32653546, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e5354a, SUC_lexicon_tags_40 },
    { 0x365354b, SUC_lexicon_tags_60 },
    { 0x29e5354b, SUC_lexicon_tags_27 },
    { 0xe345354d, SUC_lexicon_tags_12 },
    { 0x7ee5354e, SUC_lexicon_tags_1 },
    { 0xcc05354f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7053552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6c53554, SUC_lexicon_tags_6 },
    { 0x43653555, SUC_lexicon_tags_6 },
    { 0x59653555, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5653558, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2c5355a, SUC_lexicon_tags_11 },
    { 0x8b05355b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xba85355d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7a65355f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1253563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1a53565, SUC_lexicon_tags_5 },
    { 0x44e53566, SUC_lexicon_tags_11 },
    { 0xd6453567, SUC_lexicon_tags_33 },
    { 0xa2253566, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7405356a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e5356d, SUC_lexicon_tags_15 },
    { 0xc825356e, SUC_lexicon_tags_28 },
    { 0x51e5356f, SUC_lexicon_tags_25 },
    { 0xee653570, SUC_lexicon_tags_1 },
    { 0x13a53571, SUC_lexicon_tags_1 },
    { 0x6d453572, SUC_lexicon_tags_8 },
    { 0x48653572, SUC_lexicon_tags_14 },
    { 0xc8453574, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x50253577, SUC_lexicon_tags_7 },
    { 0x28c53578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc253583, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x91453585, SUC_lexicon_tags_28 },
    { 0x20e53585, SUC_lexicon_tags_33 },
    { 0x45c53587, SUC_lexicon_tags_32 },
    { 0x8e453588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd05358a, SUC_lexicon_tags_37 },
    { 0xfe85358a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2925358d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e53590, SUC_lexicon_tags_4 },
    { 0xa4253591, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60653596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40053599, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a5359d, SUC_lexicon_tags_31 },
    { 0xfc45359d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92535a5, SUC_lexicon_tags_92 },
    { 0x63c535a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6535ab, SUC_lexicon_tags_1 },
    { 0x7d8535ac, SUC_lexicon_tags_25 },
    { 0x14e535ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x956535b0, SUC_lexicon_tags_2 },
    { 0x4a535b1, SUC_lexicon_tags_37 },
    { 0xd6c535b0, SUC_lexicon_tags_2 },
    { 0x3ee535b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2535b9, SUC_lexicon_tags_17 },
    { 0x258535ba, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa535bd, SUC_lexicon_tags_49 },
    { 0x404535be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf24535c0, SUC_lexicon_tags_37 },
    { 0x63c535c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7aa535c3, SUC_lexicon_tags_65 },
    { 0xd06535c3, SUC_lexicon_tags_7 },
    { 0x3f2535c5, SUC_lexicon_tags_7 },
    { 0xa84535c5, SUC_lexicon_tags_33 },
    { 0xd8a535c7, SUC_lexicon_tags_27 },
    { 0x472535c8, SUC_lexicon_tags_37 },
    { 0x468535c8, SUC_lexicon_tags_45 },
    { 0x2ba535c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xede535cc, SUC_lexicon_tags_10 },
    { 0x200535cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8535d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe535d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80535dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6535e4, SUC_lexicon_tags_15 },
    { 0xa26535e5, SUC_lexicon_tags_7 },
    { 0x31e535e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2535ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca6535ec, SUC_lexicon_tags_16 },
    { 0x804535ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6535f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e535f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90535fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47653602, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97853605, SUC_lexicon_tags_14 },
    { 0x1e653605, SUC_lexicon_tags_48 },
    { 0x8f453607, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e5360b, SUC_lexicon_tags_6 },
    { 0x4485360c, SUC_lexicon_tags_29 },
    { 0xf2c5360d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e53612, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30453616, SUC_lexicon_tags_27 },
    { 0xc0053617, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a85361a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1905361c, SUC_lexicon_tags_25 },
    { 0x1a45361d, SUC_lexicon_tags_7 },
    { 0x9245361d, SUC_lexicon_tags_39 },
    { 0x1885361d, SUC_lexicon_tags_6 },
    { 0xfc653620, SUC_lexicon_tags_9 },
    { 0x65a53621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde653623, SUC_lexicon_tags_12 },
    { 0x16e53624, SUC_lexicon_tags_45 },
    { 0x75a53625, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x22453628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0a5362a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4785362c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc5362f, SUC_lexicon_tags_49 },
    { 0x653630, SUC_lexicon_tags_4 },
    { 0x70453630, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a5363c, SUC_lexicon_tags_63 },
    { 0x7105363d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xdaa5363f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea53642, SUC_lexicon_tags_65 },
    { 0x2e253642, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3253646, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a5364c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a53651, SUC_lexicon_tags_33 },
    { 0x49453652, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a53656, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf253658, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a5365e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a53668, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7025366b, SUC_lexicon_tags_45 },
    { 0x5d25366b, SUC_lexicon_tags_26 },
    { 0x6a85366d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e53675, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc105367e, SUC_lexicon_tags_63 },
    { 0x1fa5367f, SUC_lexicon_tags_17 },
    { 0x3a53680, SUC_lexicon_tags_2 },
    { 0x3a453681, SUC_lexicon_tags_84 },
    { 0xa0e53682, SUC_lexicon_tags_87 },
    { 0xe0c53682, SUC_lexicon_tags_27 },
    { 0x55853683, SUC_lexicon_tags_1 },
    { 0xc653685, SUC_lexicon_tags_37 },
    { 0xe2a53686, SUC_lexicon_tags_79 },
    { 0x29653687, SUC_lexicon_tags_12 },
    { 0xe4c5367e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb185368b, SUC_lexicon_tags_45 },
    { 0xf165368b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e45368f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac53696, SUC_lexicon_tags_38 },
    { 0x3b453697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d45369e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x286536a1, SUC_lexicon_tags_25 },
    { 0xc32536a2, SUC_lexicon_tags_30 },
    { 0x120536a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38536ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x482536af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6536b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc536b8, SUC_lexicon_tags_17 },
    { 0x7ae536b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x32a536bb, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e536c1, SUC_lexicon_tags_63 },
    { 0x728536c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74536c7, SUC_lexicon_tags_37 },
    { 0x624536c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e536cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4536d0, SUC_lexicon_tags_49 },
    { 0xe6c536d1, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356536d5, SUC_lexicon_tags_6 },
    { 0xcc8536d5, SUC_lexicon_tags_112 },
    { 0xa7e536d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46536e2, SUC_lexicon_tags_48 },
    { 0xd08536e3, SUC_lexicon_tags_44 },
    { 0x32536e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4536f0, SUC_lexicon_tags_1 },
    { 0xf0a536f1, SUC_lexicon_tags_66 },
    { 0x444536f0, SUC_lexicon_tags_25 },
    { 0x22536f3, SUC_lexicon_tags_37 },
    { 0x3c536f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x388536f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96536f9, SUC_lexicon_tags_71 },
    { 0x4b8536f9, SUC_lexicon_tags_49 },
    { 0xad8536fa, SUC_lexicon_tags_60 },
    { 0x56536fb, SUC_lexicon_tags_45 },
    { 0x4a6536fd, SUC_lexicon_tags_11 },
    { 0xe3c536fa, SUC_lexicon_tags_23 },
    { 0x9f4536f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23253701, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e53706, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe25370a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc885370c, SUC_lexicon_tags_43 },
    { 0x12c5370d, SUC_lexicon_tags_43 },
    { 0xf445370c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a53711, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1053715, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa5371a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c5371f, SUC_lexicon_tags_48 },
    { 0xd0453720, SUC_lexicon_tags_28 },
    { 0xfbc53721, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf653726, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd05372b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a5372e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6253737, SUC_lexicon_tags_38 },
    { 0x2c453738, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fc5373a, SUC_lexicon_tags_27 },
    { 0x885373b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63653743, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x7d453745, SUC_lexicon_tags_40 },
    { 0x88c53746, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e5374a, SUC_lexicon_tags_21 },
    { 0xb285374a, SUC_lexicon_tags_49 },
    { 0xac65374c, SUC_lexicon_tags_63 },
    { 0xba85374d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd853750, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x99253752, SUC_lexicon_tags_15 },
    { 0x75253752, SUC_lexicon_tags_1 },
    { 0x30253752, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11253758, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8065375c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79653761, SUC_lexicon_tags_38 },
    { 0xf8453762, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x73c53764, SUC_lexicon_tags_25 },
    { 0x32053765, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23053769, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa905376e, SUC_lexicon_tags_38 },
    { 0xf0a5376e, SUC_lexicon_tags_16 },
    { 0x7605376e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7853773, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76a53775, SUC_lexicon_tags_43 },
    { 0x13a53776, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x97a53778, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x9e45377a, SUC_lexicon_tags_31 },
    { 0xd425377b, SUC_lexicon_tags_6 },
    { 0xcb65377c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb845377e, SUC_lexicon_tags_12 },
    { 0x50e5377e, SUC_lexicon_tags_2 },
    { 0x3b85377e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x39053783, SUC_lexicon_tags_88 },
    { 0x86853784, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19053786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea53789, SUC_lexicon_tags_11 },
    { 0x80853789, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2653790, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac53794, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e5379d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e537a0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748537a4, SUC_lexicon_tags_15 },
    { 0x470537a5, SUC_lexicon_tags_39 },
    { 0x920537a5, SUC_lexicon_tags_33 },
    { 0xefa537a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x556537a9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6537b0, SUC_lexicon_tags_38 },
    { 0xd54537b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x710537b3, SUC_lexicon_tags_687 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206537b7, SUC_lexicon_tags_561 },
    { 0x1be537b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2a537ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x68e537bc, SUC_lexicon_tags_45 },
    { 0xa5c537bc, SUC_lexicon_tags_1 },
    { 0x622537be, SUC_lexicon_tags_2 },
    { 0xa1c537be, SUC_lexicon_tags_49 },
    { 0x57a537c0, SUC_lexicon_tags_31 },
    { 0xa96537be, SUC_lexicon_tags_43 },
    { 0xd8a537c2, SUC_lexicon_tags_23 },
    { 0xb0537c2, SUC_lexicon_tags_49 },
    { 0xacc537c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc537c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x182537ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36537cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8537d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcba537d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca537dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa42537de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x294537e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac537e5, SUC_lexicon_tags_38 },
    { 0x5de537e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x396537e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe537ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c537f0, SUC_lexicon_tags_60 },
    { 0x34537f0, SUC_lexicon_tags_1 },
    { 0x116537f2, SUC_lexicon_tags_12 },
    { 0x764537f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90a537f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4537fc, SUC_lexicon_tags_37 },
    { 0x3ce537fc, SUC_lexicon_tags_17 },
    { 0x9d4537fe, SUC_lexicon_tags_45 },
    { 0xdb6537ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c853801, SUC_lexicon_tags_45 },
    { 0x4ae53802, SUC_lexicon_tags_1 },
    { 0x73253803, SUC_lexicon_tags_87 },
    { 0x8f453804, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1525380c, SUC_lexicon_tags_23 },
    { 0x71e5380d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b853810, SUC_lexicon_tags_15 },
    { 0xedc53811, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e5381a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5381e, SUC_lexicon_tags_37 },
    { 0x5785381f, SUC_lexicon_tags_60 },
    { 0x51c5381f, SUC_lexicon_tags_37 },
    { 0x58a53821, SUC_lexicon_tags_11 },
    { 0x95853822, SUC_lexicon_tags_15 },
    { 0x5f853820, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a853829, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c253831, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3565383f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1853841, SUC_lexicon_tags_1 },
    { 0x1dc53842, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70653847, SUC_lexicon_tags_27 },
    { 0x6fe53847, SUC_lexicon_tags_7 },
    { 0x33253849, SUC_lexicon_tags_25 },
    { 0x50a53847, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c53855, SUC_lexicon_tags_8 },
    { 0xb8053856, SUC_lexicon_tags_17 },
    { 0xb9053857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19c53859, SUC_lexicon_tags_11 },
    { 0xc6c5385a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe505385c, SUC_lexicon_tags_14 },
    { 0xacc5385d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1253860, SUC_lexicon_tags_82 },
    { 0x6653861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7053864, SUC_lexicon_tags_14 },
    { 0xf5c53865, SUC_lexicon_tags_7 },
    { 0xf0053866, SUC_lexicon_tags_11 },
    { 0x3ca53867, SUC_lexicon_tags_19 },
    { 0xacc53868, SUC_lexicon_tags_27 },
    { 0x77853869, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da5386c, SUC_lexicon_tags_37 },
    { 0xf1c5386d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb053872, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x69053874, SUC_lexicon_tags_68 },
    { 0x6253875, SUC_lexicon_tags_27 },
    { 0x77c53876, SUC_lexicon_tags_21 },
    { 0x81253877, SUC_lexicon_tags_15 },
    { 0x88c53878, SUC_lexicon_tags_15 },
    { 0xec853878, SUC_lexicon_tags_25 },
    { 0xf4e5387a, SUC_lexicon_tags_49 },
    { 0xdee5387b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd385387e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e53882, SUC_lexicon_tags_37 },
    { 0xc6c53882, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2653885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b85388a, SUC_lexicon_tags_11 },
    { 0xbc25388b, SUC_lexicon_tags_37 },
    { 0x5bc5388c, SUC_lexicon_tags_15 },
    { 0xfc5388d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c53892, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be53895, SUC_lexicon_tags_17 },
    { 0xabe53895, SUC_lexicon_tags_11 },
    { 0x3d653895, SUC_lexicon_tags_24 },
    { 0x3253898, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xd225389a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a5389d, SUC_lexicon_tags_11 },
    { 0x90a5389e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74538a1, SUC_lexicon_tags_28 },
    { 0x644538a2, SUC_lexicon_tags_80 },
    { 0xbf2538a2, SUC_lexicon_tags_11 },
    { 0xa0538a4, SUC_lexicon_tags_17 },
    { 0x24a538a4, SUC_lexicon_tags_21 },
    { 0x3d0538a5, SUC_lexicon_tags_25 },
    { 0x56c538a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4538ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ae538af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f0538b1, SUC_lexicon_tags_8 },
    { 0x46c538b1, SUC_lexicon_tags_2 },
    { 0xbe4538b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x790538b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57e538b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x126538ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf54538bc, SUC_lexicon_tags_13 },
    { 0x372538bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e538c3, SUC_lexicon_tags_14 },
    { 0xcbc538c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33c538c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26538cb, SUC_lexicon_tags_1 },
    { 0x224538cc, SUC_lexicon_tags_43 },
    { 0xaf4538cd, SUC_lexicon_tags_40 },
    { 0xc66538cb, SUC_lexicon_tags_0 },
    { 0x774538cf, SUC_lexicon_tags_85 },
    { 0x596538d0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x102538d2, SUC_lexicon_tags_63 },
    { 0x770538d2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e538d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x422538d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe538e2, SUC_lexicon_tags_0 },
    { 0xd0c538e3, SUC_lexicon_tags_15 },
    { 0x51e538e4, SUC_lexicon_tags_84 },
    { 0xfa4538e4, SUC_lexicon_tags_63 },
    { 0xc94538e4, SUC_lexicon_tags_17 },
    { 0x7fc538e7, SUC_lexicon_tags_341 },
    { 0x36a538e7, SUC_lexicon_tags_23 },
    { 0x8c6538e4, SUC_lexicon_tags_11 },
    { 0x232538e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6538ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe538f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e538fb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x396538fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a538ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c53904, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb853906, SUC_lexicon_tags_63 },
    { 0x40853907, SUC_lexicon_tags_35 },
    { 0x10653908, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf845390a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a5390f, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0xc0053911, SUC_lexicon_tags_17 },
    { 0xdac53911, SUC_lexicon_tags_1 },
    { 0x29653911, SUC_lexicon_tags_17 },
    { 0xfbc53911, SUC_lexicon_tags_30 },
    { 0xe4a53915, SUC_lexicon_tags_6 },
    { 0xf2653912, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf653918, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba5391b, SUC_lexicon_tags_19 },
    { 0x27e5391c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9985391e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x67a53920, SUC_lexicon_tags_37 },
    { 0xd4253921, SUC_lexicon_tags_15 },
    { 0x96853922, SUC_lexicon_tags_2 },
    { 0x90453922, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52453928, SUC_lexicon_tags_8 },
    { 0xf7253929, SUC_lexicon_tags_12 },
    { 0x1fa53929, SUC_lexicon_tags_9 },
    { 0xe3c5392b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf05392d, SUC_lexicon_tags_11 },
    { 0x8005392e, SUC_lexicon_tags_0 },
    { 0xbbc5392f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe53933, SUC_lexicon_tags_2 },
    { 0x8e253934, SUC_lexicon_tags_47 },
    { 0x37c53935, SUC_lexicon_tags_13 },
    { 0xa9e53936, SUC_lexicon_tags_28 },
    { 0x90c53937, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x4a053939, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2fc5393b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5885393d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3485393f, SUC_lexicon_tags_16 },
    { 0xa5853940, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc453945, SUC_lexicon_tags_37 },
    { 0xb6453946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a5394d, SUC_lexicon_tags_15 },
    { 0x91e5394d, SUC_lexicon_tags_15 },
    { 0x2be5394f, SUC_lexicon_tags_1276 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4653952, SUC_lexicon_tags_12 },
    { 0xf7253953, SUC_lexicon_tags_25 },
    { 0xcfe53954, SUC_lexicon_tags_7 },
    { 0x2e53954, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a53959, SUC_lexicon_tags_12 },
    { 0x61453959, SUC_lexicon_tags_15 },
    { 0x6de5395b, SUC_lexicon_tags_7 },
    { 0x15e53959, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80453963, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1d653965, SUC_lexicon_tags_12 },
    { 0x9253966, SUC_lexicon_tags_79 },
    { 0x99c53965, SUC_lexicon_tags_12 },
    { 0xd253965, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x305396a, SUC_lexicon_tags_11 },
    { 0x49a5396b, SUC_lexicon_tags_2 },
    { 0xbf45396b, SUC_lexicon_tags_2 },
    { 0xf565396b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e53975, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e53979, SUC_lexicon_tags_47 },
    { 0x2585397a, SUC_lexicon_tags_0 },
    { 0xea45397b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe53981, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1453986, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c053989, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6805398b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c5398e, SUC_lexicon_tags_28 },
    { 0xb5c5398e, SUC_lexicon_tags_9 },
    { 0x46a5398f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ee53992, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76053994, SUC_lexicon_tags_8 },
    { 0x85c53994, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa5399b, SUC_lexicon_tags_17 },
    { 0xc765399b, SUC_lexicon_tags_60 },
    { 0x6665399d, SUC_lexicon_tags_26 },
    { 0x6325399e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a6539a0, SUC_lexicon_tags_25 },
    { 0x89c539a1, SUC_lexicon_tags_11 },
    { 0x2e8539a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c539a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc539ab, SUC_lexicon_tags_19 },
    { 0x2d6539ac, SUC_lexicon_tags_26 },
    { 0x4ce539ab, SUC_lexicon_tags_19 },
    { 0x128539ae, SUC_lexicon_tags_26 },
    { 0x46a539ad, SUC_lexicon_tags_33 },
    { 0x9aa539b0, SUC_lexicon_tags_1 },
    { 0x698539ac, SUC_lexicon_tags_5 },
    { 0x8ba539ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0539be, SUC_lexicon_tags_29 },
    { 0x468539bf, SUC_lexicon_tags_15 },
    { 0xdec539be, SUC_lexicon_tags_27 },
    { 0x4e539bf, SUC_lexicon_tags_17 },
    { 0x1ca539c2, SUC_lexicon_tags_17 },
    { 0x3a2539c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7a539c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c539c9, SUC_lexicon_tags_15 },
    { 0xf28539ca, SUC_lexicon_tags_4 },
    { 0xe62539ca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2539cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c539d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc8a539da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6a0539dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x926539df, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10539e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86539e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a539eb, SUC_lexicon_tags_8 },
    { 0x106539eb, SUC_lexicon_tags_65 },
    { 0xde2539ed, SUC_lexicon_tags_37 },
    { 0xd6539ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4a539f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f4539f2, SUC_lexicon_tags_15 },
    { 0xc3e539f3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32539ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3c853a01, SUC_lexicon_tags_38 },
    { 0x6fa53a02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4453a04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84453a0a, SUC_lexicon_tags_37 },
    { 0x60e53a0b, SUC_lexicon_tags_29 },
    { 0x9b253a0c, SUC_lexicon_tags_2 },
    { 0x77853a0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76c53a0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57853a11, SUC_lexicon_tags_179 },
    { 0x1253a11, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xeb653a14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35853a1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x56253a1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2053a20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16453a22, SUC_lexicon_tags_47 },
    { 0xeba53a22, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xf0253a25, SUC_lexicon_tags_43 },
    { 0x7e253a26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1453a28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x66053a2b, SUC_lexicon_tags_37 },
    { 0xb8053a2c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc4a53a2e, SUC_lexicon_tags_37 },
    { 0x19653a2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26c53a31, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x17053a34, SUC_lexicon_tags_37 },
    { 0xb5853a34, SUC_lexicon_tags_27 },
    { 0x70853a36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43053a38, SUC_lexicon_tags_42 },
    { 0x67653a38, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x53653a3b, SUC_lexicon_tags_37 },
    { 0x29a53a3c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20253a40, SUC_lexicon_tags_12 },
    { 0xbb853a40, SUC_lexicon_tags_37 },
    { 0x2c253a42, SUC_lexicon_tags_1 },
    { 0x21e53a43, SUC_lexicon_tags_15 },
    { 0x15453a43, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c53a49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca53a4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4653a58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a653a5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7853a63, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb053a69, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e53a6c, SUC_lexicon_tags_14 },
    { 0x6dc53a6d, SUC_lexicon_tags_27 },
    { 0x1c053a6c, SUC_lexicon_tags_12 },
    { 0x2d253a6d, SUC_lexicon_tags_14 },
    { 0x47c53a70, SUC_lexicon_tags_15 },
    { 0x12c53a70, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a53a74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc53a76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b053a7a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc53a7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a53a81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a53a84, SUC_lexicon_tags_19 },
    { 0x1d253a84, SUC_lexicon_tags_1 },
    { 0x84653a86, SUC_lexicon_tags_22 },
    { 0x81a53a87, SUC_lexicon_tags_31 },
    { 0xcf453a86, SUC_lexicon_tags_79 },
    { 0x65253a89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdd453a8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce53a90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba53a94, SUC_lexicon_tags_49 },
    { 0xe8253a95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39c53a97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77853a9e, SUC_lexicon_tags_15 },
    { 0x6d653a9f, SUC_lexicon_tags_31 },
    { 0x83853a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22453aa5, SUC_lexicon_tags_19 },
    { 0x33253aa5, SUC_lexicon_tags_17 },
    { 0xf0c53aa7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37853aac, SUC_lexicon_tags_6 },
    { 0xba653aad, SUC_lexicon_tags_27 },
    { 0x5d653aad, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e53ab3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29653ab6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77053abb, SUC_lexicon_tags_31 },
    { 0x54253abc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7653abe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e653ac6, SUC_lexicon_tags_45 },
    { 0xa8453ac7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff253ac9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12253ad1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a53ad7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62c53ad9, SUC_lexicon_tags_2 },
    { 0xe0a53ad9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x44653add, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8853ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64453ae2, SUC_lexicon_tags_1 },
    { 0xada53ae2, SUC_lexicon_tags_63 },
    { 0x32053ae4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc453ae6, SUC_lexicon_tags_45 },
    { 0x37e53ae6, SUC_lexicon_tags_35 },
    { 0xdd053ae8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea653aea, SUC_lexicon_tags_28 },
    { 0x2653aeb, SUC_lexicon_tags_14 },
    { 0xd7253aec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12a53aee, SUC_lexicon_tags_12 },
    { 0x90e53aee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba53af3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15e53af5, SUC_lexicon_tags_8 },
    { 0x8e653af5, SUC_lexicon_tags_1 },
    { 0x88c53af7, SUC_lexicon_tags_11 },
    { 0x92853af5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb653afc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15053afe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c53b06, SUC_lexicon_tags_32 },
    { 0xcac53b07, SUC_lexicon_tags_11 },
    { 0x39853b06, SUC_lexicon_tags_63 },
    { 0xffe53b06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e53b0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21653b10, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91253b15, SUC_lexicon_tags_2 },
    { 0x80253b16, SUC_lexicon_tags_45 },
    { 0x6b453b17, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c53b1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48453b1d, SUC_lexicon_tags_39 },
    { 0x75653b1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc853b20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38253b23, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53453b28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64253b2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bc53b30, SUC_lexicon_tags_11 },
    { 0x77453b31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37253b35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0c53b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9853b3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec53b3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a53b40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88453b49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7253b4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbe853b4f, SUC_lexicon_tags_15 },
    { 0xe9c53b50, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85053b57, SUC_lexicon_tags_7 },
    { 0x85653b58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86053b5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf453b5f, SUC_lexicon_tags_25 },
    { 0x41a53b60, SUC_lexicon_tags_7 },
    { 0xf1a53b61, SUC_lexicon_tags_26 },
    { 0xabc53b5f, SUC_lexicon_tags_38 },
    { 0xd2653b63, SUC_lexicon_tags_27 },
    { 0xe3c53b5f, SUC_lexicon_tags_136 },
    { 0xd3c53b65, SUC_lexicon_tags_65 },
    { 0x64653b66, SUC_lexicon_tags_9 },
    { 0x6c53b64, SUC_lexicon_tags_25 },
    { 0xa4e53b61, SUC_lexicon_tags_27 },
    { 0x14453b69, SUC_lexicon_tags_11 },
    { 0x79453b69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e653b6f, SUC_lexicon_tags_14 },
    { 0x93c53b70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c053b77, SUC_lexicon_tags_23 },
    { 0x60653b78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62853b7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe053b7f, SUC_lexicon_tags_6 },
    { 0x3e253b7f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5f253b82, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43853b87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa53b8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x75453b90, SUC_lexicon_tags_17 },
    { 0xc9a53b90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77053b93, SUC_lexicon_tags_11 },
    { 0x62e53b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d853b96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a53b99, SUC_lexicon_tags_21 },
    { 0xdd253b9a, SUC_lexicon_tags_2 },
    { 0xf2e53b9a, SUC_lexicon_tags_13 },
    { 0x8ac53b99, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x16253b9e, SUC_lexicon_tags_12 },
    { 0xade53b9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc53ba2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5aa53ba4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d253ba9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9e53bab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0e53bad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47253bb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x41a53bb3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c53bbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d053bc1, SUC_lexicon_tags_0 },
    { 0x5de53bc2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26253bc7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x37053bc9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xa6653bcb, SUC_lexicon_tags_11 },
    { 0xda053bcc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x94453bce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xef053bd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x31853bd3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94253bd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96653bda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e253bdc, SUC_lexicon_tags_17 },
    { 0x93053bdd, SUC_lexicon_tags_17 },
    { 0xf7a53bde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7a53be0, SUC_lexicon_tags_19 },
    { 0x9be53be1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f053be4, SUC_lexicon_tags_87 },
    { 0x33053be5, SUC_lexicon_tags_33 },
    { 0x3f653be5, SUC_lexicon_tags_22 },
    { 0x2d653be4, SUC_lexicon_tags_27 },
    { 0x2a053be8, SUC_lexicon_tags_1 },
    { 0xcc653be9, SUC_lexicon_tags_38 },
    { 0x12053bea, SUC_lexicon_tags_7 },
    { 0x80e53be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a53bee, SUC_lexicon_tags_37 },
    { 0x8d053bef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba853bf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc53bf5, SUC_lexicon_tags_25 },
    { 0x73e53bf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41453bf8, SUC_lexicon_tags_37 },
    { 0x8d253bf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf853c03, SUC_lexicon_tags_37 },
    { 0x10653c04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86c53c06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bc53c08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c53c0c, SUC_lexicon_tags_38 },
    { 0x95c53c0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e653c13, SUC_lexicon_tags_11 },
    { 0xed253c14, SUC_lexicon_tags_7 },
    { 0xd4c53c14, SUC_lexicon_tags_7 },
    { 0x8bc53c16, SUC_lexicon_tags_26 },
    { 0x5a053c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf253c19, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x17a53c1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e053c1d, SUC_lexicon_tags_55 },
    { 0xfbc53c1e, SUC_lexicon_tags_12 },
    { 0x5653c1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89053c23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39053c25, SUC_lexicon_tags_63 },
    { 0xdf253c26, SUC_lexicon_tags_2 },
    { 0xf4a53c26, SUC_lexicon_tags_11 },
    { 0x7de53c28, SUC_lexicon_tags_11 },
    { 0xfd653c29, SUC_lexicon_tags_37 },
    { 0xbb653c29, SUC_lexicon_tags_30 },
    { 0x15853c2b, SUC_lexicon_tags_29 },
    { 0xa3c53c2c, SUC_lexicon_tags_60 },
    { 0xcc653c2d, SUC_lexicon_tags_1 },
    { 0xba53c27, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef653c30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe453c36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a853c3a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xde453c3c, SUC_lexicon_tags_37 },
    { 0xe2653c3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99653c41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x41453c43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa53c4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be53c4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1453c53, SUC_lexicon_tags_16 },
    { 0x53a53c54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b253c56, SUC_lexicon_tags_37 },
    { 0xf0a53c57, SUC_lexicon_tags_31 },
    { 0xc4453c58, SUC_lexicon_tags_2 },
    { 0x68853c59, SUC_lexicon_tags_17 },
    { 0x3a653c56, SUC_lexicon_tags_15 },
    { 0xd5c53c5b, SUC_lexicon_tags_2 },
    { 0xa7e53c5c, SUC_lexicon_tags_12 },
    { 0x9b853c5b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c53c61, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c53c64, SUC_lexicon_tags_17 },
    { 0x78a53c65, SUC_lexicon_tags_1 },
    { 0x6cc53c65, SUC_lexicon_tags_6 },
    { 0x62a53c67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xea253c6a, SUC_lexicon_tags_22 },
    { 0xdee53c6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x22053c6d, SUC_lexicon_tags_37 },
    { 0xbac53c6e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a53c73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc53c76, SUC_lexicon_tags_27 },
    { 0xd0653c76, SUC_lexicon_tags_29 },
    { 0x54c53c77, SUC_lexicon_tags_8 },
    { 0xe5453c76, SUC_lexicon_tags_7 },
    { 0x18453c7a, SUC_lexicon_tags_32 },
    { 0x51453c77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36853c7e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29453c83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b253c85, SUC_lexicon_tags_9 },
    { 0x37053c86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89c53c88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e653c94, SUC_lexicon_tags_7 },
    { 0x2b853c95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21653c98, SUC_lexicon_tags_63 },
    { 0x79453c99, SUC_lexicon_tags_15 },
    { 0x10653c98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda053c9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe253ca1, SUC_lexicon_tags_47 },
    { 0x8e53ca2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2ea53ca4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc53cab, SUC_lexicon_tags_60 },
    { 0x11253cac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21453cae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb853cb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e53cc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92053cc5, SUC_lexicon_tags_37 },
    { 0x3a253cc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c53cca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf253ccc, SUC_lexicon_tags_11 },
    { 0xf1653ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa853cd4, SUC_lexicon_tags_16 },
    { 0xb253cd4, SUC_lexicon_tags_37 },
    { 0x28453cd4, SUC_lexicon_tags_15 },
    { 0xea053cd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa53cdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x61853ce1, SUC_lexicon_tags_17 },
    { 0xb6253ce2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x66253ce5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e53cef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12653cf3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de53cf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a53cfd, SUC_lexicon_tags_3 },
    { 0xd4253cfd, SUC_lexicon_tags_25 },
    { 0xbf853cff, SUC_lexicon_tags_15 },
    { 0xae453d00, SUC_lexicon_tags_1 },
    { 0xb653cfd, SUC_lexicon_tags_60 },
    { 0xd8653d01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3453d04, SUC_lexicon_tags_9 },
    { 0xd0653d05, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x55e53d07, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d453d0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48253d0f, SUC_lexicon_tags_13 },
    { 0x8e853d10, SUC_lexicon_tags_7 },
    { 0x76253d11, SUC_lexicon_tags_19 },
    { 0x65253d11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95653d14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f853d16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62053d1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c53d22, SUC_lexicon_tags_1 },
    { 0x39e53d23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf1c53d25, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad853d2d, SUC_lexicon_tags_25 },
    { 0x9e053d2d, SUC_lexicon_tags_123 },
    { 0x39453d2f, SUC_lexicon_tags_8 },
    { 0x7c853d30, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xc4c53d32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91053d36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3653d38, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xe6e53d3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d053d3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a53d4b, SUC_lexicon_tags_26 },
    { 0xfaa53d4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c53d51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97a53d53, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbaa53d55, SUC_lexicon_tags_39 },
    { 0x93453d56, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82253d5b, SUC_lexicon_tags_14 },
    { 0xa5253d5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e53d62, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae53d6c, SUC_lexicon_tags_1 },
    { 0xcf453d6c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8ac53d6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74653d72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1de53d74, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xe8e53d76, SUC_lexicon_tags_25 },
    { 0x75c53d77, SUC_lexicon_tags_25 },
    { 0x83853d78, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2653d7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd853d7d, SUC_lexicon_tags_2 },
    { 0x2ce53d7d, SUC_lexicon_tags_11 },
    { 0x22e53d7f, SUC_lexicon_tags_25 },
    { 0xf9c53d7f, SUC_lexicon_tags_27 },
    { 0x63053d81, SUC_lexicon_tags_38 },
    { 0x21653d82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd053d86, SUC_lexicon_tags_37 },
    { 0xfc253d87, SUC_lexicon_tags_37 },
    { 0x39053d87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa53d8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1a53d8d, SUC_lexicon_tags_33 },
    { 0x79053d8e, SUC_lexicon_tags_25 },
    { 0x76c53d8f, SUC_lexicon_tags_25 },
    { 0xc6853d90, SUC_lexicon_tags_14 },
    { 0xbf453d8f, SUC_lexicon_tags_29 },
    { 0xcb053d90, SUC_lexicon_tags_37 },
    { 0x48453d93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf653d95, SUC_lexicon_tags_15 },
    { 0xfaa53d96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2053d9b, SUC_lexicon_tags_28 },
    { 0x2c53d9c, SUC_lexicon_tags_49 },
    { 0x5e253d9d, SUC_lexicon_tags_29 },
    { 0x6a053d9e, SUC_lexicon_tags_38 },
    { 0x98853d9f, SUC_lexicon_tags_15 },
    { 0xea253da0, SUC_lexicon_tags_17 },
    { 0x2cc53d9d, SUC_lexicon_tags_82 },
    { 0xd7453da2, SUC_lexicon_tags_11 },
    { 0x91a53da0, SUC_lexicon_tags_23 },
    { 0x95853d9d, SUC_lexicon_tags_15 },
    { 0x6fc53d9f, SUC_lexicon_tags_33 },
    { 0xfc253da6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1653da8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c253dab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca53dae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a53dbb, SUC_lexicon_tags_37 },
    { 0x20653dbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c53dc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1253dc6, SUC_lexicon_tags_6 },
    { 0x29653dc7, SUC_lexicon_tags_6 },
    { 0x41453dc7, SUC_lexicon_tags_25 },
    { 0xbce53dc9, SUC_lexicon_tags_12 },
    { 0x32053dc9, SUC_lexicon_tags_25 },
    { 0xd0853dca, SUC_lexicon_tags_26 },
    { 0xe5253dc7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9453dd2, SUC_lexicon_tags_0 },
    { 0xc8053dd3, SUC_lexicon_tags_1 },
    { 0x25a53dd4, SUC_lexicon_tags_2 },
    { 0x2e453dd5, SUC_lexicon_tags_3 },
    { 0x54053dd6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xed053dd8, SUC_lexicon_tags_12 },
    { 0xbc853dd9, SUC_lexicon_tags_29 },
    { 0xf1253dda, SUC_lexicon_tags_2 },
    { 0x2d453dda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e53de0, SUC_lexicon_tags_1 },
    { 0x53253de0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd253de4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e53de7, SUC_lexicon_tags_60 },
    { 0xd0a53de8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf453dea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a053ded, SUC_lexicon_tags_31 },
    { 0xf8453dee, SUC_lexicon_tags_12 },
    { 0x7c53def, SUC_lexicon_tags_6 },
    { 0x47653dee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb4653df2, SUC_lexicon_tags_11 },
    { 0xc5253df3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5c53df5, SUC_lexicon_tags_19 },
    { 0x68653df6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1a53df8, SUC_lexicon_tags_108 },
    { 0xb4053df9, SUC_lexicon_tags_56 },
    { 0x5aa53df9, SUC_lexicon_tags_25 },
    { 0x7b253df9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1253e04, SUC_lexicon_tags_12 },
    { 0x2ca53e05, SUC_lexicon_tags_15 },
    { 0xa3653e06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ec53e08, SUC_lexicon_tags_2 },
    { 0x36653e09, SUC_lexicon_tags_1 },
    { 0xa3253e0a, SUC_lexicon_tags_17 },
    { 0x8e053e0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5653e0d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e53e10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d053e13, SUC_lexicon_tags_47 },
    { 0xcda53e13, SUC_lexicon_tags_37 },
    { 0xa0453e15, SUC_lexicon_tags_46 },
    { 0x32e53e15, SUC_lexicon_tags_49 },
    { 0xb1853e15, SUC_lexicon_tags_39 },
    { 0x32253e14, SUC_lexicon_tags_2 },
    { 0x43c53e19, SUC_lexicon_tags_113 },
    { 0xbf053e13, SUC_lexicon_tags_7 },
    { 0xc5c53e1a, SUC_lexicon_tags_60 },
    { 0x75253e1c, SUC_lexicon_tags_2 },
    { 0x79e53e1b, SUC_lexicon_tags_24 },
    { 0x24453e15, SUC_lexicon_tags_46 },
    { 0x2e053e1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5453e21, SUC_lexicon_tags_30 },
    { 0xbdc53e21, SUC_lexicon_tags_7 },
    { 0xa453e23, SUC_lexicon_tags_12 },
    { 0x4ae53e24, SUC_lexicon_tags_68 },
    { 0x60853e25, SUC_lexicon_tags_12 },
    { 0x57a53e23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c53e29, SUC_lexicon_tags_30 },
    { 0x24053e2a, SUC_lexicon_tags_11 },
    { 0xac653e2b, SUC_lexicon_tags_11 },
    { 0x6de53e2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3e53e2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f453e35, SUC_lexicon_tags_60 },
    { 0xeb053e36, SUC_lexicon_tags_26 },
    { 0x6bc53e36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81253e3a, SUC_lexicon_tags_82 },
    { 0x1ea53e3b, SUC_lexicon_tags_19 },
    { 0x51253e3c, SUC_lexicon_tags_19 },
    { 0x48853e3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d853e40, SUC_lexicon_tags_1 },
    { 0x4bc53e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefc53e43, SUC_lexicon_tags_19 },
    { 0x24853e44, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e53e4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b653e50, SUC_lexicon_tags_0 },
    { 0xf1253e50, SUC_lexicon_tags_1 },
    { 0x63a53e52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b453e54, SUC_lexicon_tags_45 },
    { 0x53c53e54, SUC_lexicon_tags_45 },
    { 0xda253e56, SUC_lexicon_tags_48 },
    { 0x14253e54, SUC_lexicon_tags_21 },
    { 0x7a453e54, SUC_lexicon_tags_37 },
    { 0x3253e59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbec53e5b, SUC_lexicon_tags_25 },
    { 0xe0653e5b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x1ee53e5e, SUC_lexicon_tags_87 },
    { 0x2fa53e5f, SUC_lexicon_tags_65 },
    { 0x27e53e60, SUC_lexicon_tags_19 },
    { 0x44a53e61, SUC_lexicon_tags_12 },
    { 0xe9853e5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3653e67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38053e69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c53e6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36253e71, SUC_lexicon_tags_23 },
    { 0xeee53e71, SUC_lexicon_tags_49 },
    { 0xe8c53e73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf0a53e75, SUC_lexicon_tags_11 },
    { 0xd053e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff453e79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c853e7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcd253e80, SUC_lexicon_tags_25 },
    { 0x1ee53e81, SUC_lexicon_tags_17 },
    { 0x12a53e80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf3253e84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2fe53e86, SUC_lexicon_tags_47 },
    { 0x32c53e87, SUC_lexicon_tags_11 },
    { 0xa7a53e88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10653e8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x90453e8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90053e90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e53e97, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x36053e99, SUC_lexicon_tags_25 },
    { 0xc253e9a, SUC_lexicon_tags_26 },
    { 0xdd053e9b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75253ea1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a53ea4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde53ead, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b053eb0, SUC_lexicon_tags_33 },
    { 0x7e853eb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab853eb4, SUC_lexicon_tags_37 },
    { 0xf0253eb5, SUC_lexicon_tags_60 },
    { 0x29853eb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4453ebb, SUC_lexicon_tags_79 },
    { 0x1f453ebc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd253ebf, SUC_lexicon_tags_47 },
    { 0x69653ec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38053ec2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f253ec4, SUC_lexicon_tags_8 },
    { 0x58853ec5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a53eca, SUC_lexicon_tags_63 },
    { 0xd6453eca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb1e53ecd, SUC_lexicon_tags_35 },
    { 0x65e53ece, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e53ed2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14653ed9, SUC_lexicon_tags_47 },
    { 0xa1653eda, SUC_lexicon_tags_11 },
    { 0xae653edb, SUC_lexicon_tags_15 },
    { 0x2f253eda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97253ede, SUC_lexicon_tags_74 },
    { 0xdc53edf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeee53ee1, SUC_lexicon_tags_7 },
    { 0x14653ee1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8e453ee4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c53ee7, SUC_lexicon_tags_45 },
    { 0x4ac53ee8, SUC_lexicon_tags_138 },
    { 0x7ae53ee9, SUC_lexicon_tags_8 },
    { 0x2ec53eea, SUC_lexicon_tags_37 },
    { 0x81253ee9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2853ef0, SUC_lexicon_tags_9 },
    { 0x13453ef0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e53ef4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8b253ef6, SUC_lexicon_tags_27 },
    { 0x6c053ef7, SUC_lexicon_tags_68 },
    { 0x54a53ef8, SUC_lexicon_tags_321 },
    { 0xafe53ef7, SUC_lexicon_tags_2 },
    { 0x86053efa, SUC_lexicon_tags_25 },
    { 0x6ec53efb, SUC_lexicon_tags_7 },
    { 0xcaa53efb, SUC_lexicon_tags_15 },
    { 0x93453ef6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24253f03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e53f07, SUC_lexicon_tags_7 },
    { 0xa2c53f07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be53f0b, SUC_lexicon_tags_14 },
    { 0x31653f0b, SUC_lexicon_tags_9 },
    { 0x7e053f0d, SUC_lexicon_tags_22 },
    { 0x3e853f0d, SUC_lexicon_tags_37 },
    { 0x2d453f0d, SUC_lexicon_tags_47 },
    { 0xa5c53f10, SUC_lexicon_tags_6 },
    { 0xf1c53f0b, SUC_lexicon_tags_6 },
    { 0x28253f0e, SUC_lexicon_tags_28 },
    { 0xb7453f13, SUC_lexicon_tags_17 },
    { 0xe2e53f14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c653f17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x43053f1a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a53f1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da53f26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c253f2f, SUC_lexicon_tags_12 },
    { 0x453f30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4453f32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe53f35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c53f39, SUC_lexicon_tags_25 },
    { 0x74653f3a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a53f3d, SUC_lexicon_tags_10 },
    { 0x3f053f3d, SUC_lexicon_tags_43 },
    { 0xe7c53f3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c53f44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e853f48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c53f4b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9653f4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a53f51, SUC_lexicon_tags_11 },
    { 0x5c653f51, SUC_lexicon_tags_8 },
    { 0xfb053f53, SUC_lexicon_tags_17 },
    { 0xdfa53f53, SUC_lexicon_tags_17 },
    { 0xcac53f55, SUC_lexicon_tags_11 },
    { 0xe8453f56, SUC_lexicon_tags_1 },
    { 0xce853f57, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7b653f59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd653f5b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee53f5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc6a53f61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3853f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50453f66, SUC_lexicon_tags_17 },
    { 0x40453f66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a653f69, SUC_lexicon_tags_1 },
    { 0x16c53f6a, SUC_lexicon_tags_23 },
    { 0xc0853f6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10453f70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfac53f72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ea53f74, SUC_lexicon_tags_25 },
    { 0xbc253f75, SUC_lexicon_tags_47 },
    { 0x7be53f75, SUC_lexicon_tags_10 },
    { 0x30853f75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc853f79, SUC_lexicon_tags_25 },
    { 0x67a53f7a, SUC_lexicon_tags_25 },
    { 0x5bc53f7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x453f7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e53f81, SUC_lexicon_tags_11 },
    { 0x77e53f82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfae53f84, SUC_lexicon_tags_28 },
    { 0x4e653f85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x83653f87, SUC_lexicon_tags_11 },
    { 0x8de53f87, SUC_lexicon_tags_62 },
    { 0x74653f87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a253f8c, SUC_lexicon_tags_25 },
    { 0x4ec53f8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50453f94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9853f9d, SUC_lexicon_tags_11 },
    { 0x11e53f9d, SUC_lexicon_tags_37 },
    { 0x88053f9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa53fa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9653faa, SUC_lexicon_tags_151 },
    { 0xc5453faa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c53fb1, SUC_lexicon_tags_87 },
    { 0x1c053fb2, SUC_lexicon_tags_37 },
    { 0x14853fb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82253fb5, SUC_lexicon_tags_37 },
    { 0x59453fb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6253fb8, SUC_lexicon_tags_31 },
    { 0xf6a53fb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce53fbc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a53fc0, SUC_lexicon_tags_27 },
    { 0x8e053fc0, SUC_lexicon_tags_29 },
    { 0xd3453fc1, SUC_lexicon_tags_37 },
    { 0xee053fc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82453fca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47453fce, SUC_lexicon_tags_25 },
    { 0x85453fcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6453fd2, SUC_lexicon_tags_13 },
    { 0xe0e53fd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57453fd9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5b453fdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac53fdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e53fe1, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xd9e53fe3, SUC_lexicon_tags_2 },
    { 0x88c53fe4, SUC_lexicon_tags_0 },
    { 0x95853fe5, SUC_lexicon_tags_94 },
    { 0x2253fe5, SUC_lexicon_tags_16 },
    { 0x12853fe7, SUC_lexicon_tags_29 },
    { 0x41a53fe3, SUC_lexicon_tags_43 },
    { 0xb2e53fe5, SUC_lexicon_tags_11 },
    { 0xfd053fe8, SUC_lexicon_tags_2 },
    { 0xe7c53feb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50053ff0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda53ff3, SUC_lexicon_tags_19 },
    { 0x26453ff4, SUC_lexicon_tags_47 },
    { 0x6a253ff4, SUC_lexicon_tags_60 },
    { 0x4f653ff6, SUC_lexicon_tags_9 },
    { 0x8b653ff7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa6c53ff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d853ffb, SUC_lexicon_tags_39 },
    { 0xf1053ffc, SUC_lexicon_tags_21 },
    { 0x7e53ffc, SUC_lexicon_tags_12 },
    { 0x1e053ffe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96854002, SUC_lexicon_tags_9 },
    { 0x7b454003, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb025400b, SUC_lexicon_tags_38 },
    { 0x1605400c, SUC_lexicon_tags_2 },
    { 0xd1c5400d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x45654010, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a54014, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa54017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c05401a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6285401d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b054020, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x58c54022, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e654027, SUC_lexicon_tags_29 },
    { 0xc2254027, SUC_lexicon_tags_71 },
    { 0x82e54029, SUC_lexicon_tags_33 },
    { 0x4625402a, SUC_lexicon_tags_0 },
    { 0x2a05402b, SUC_lexicon_tags_29 },
    { 0x16e54029, SUC_lexicon_tags_7 },
    { 0xe605402d, SUC_lexicon_tags_37 },
    { 0xa2054029, SUC_lexicon_tags_7 },
    { 0x2cc54029, SUC_lexicon_tags_15 },
    { 0x7725402b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654034, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89654039, SUC_lexicon_tags_14 },
    { 0xb3254039, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e854041, SUC_lexicon_tags_25 },
    { 0xe2454041, SUC_lexicon_tags_16 },
    { 0xba454042, SUC_lexicon_tags_14 },
    { 0x8d454041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1654056, SUC_lexicon_tags_48 },
    { 0xae454057, SUC_lexicon_tags_11 },
    { 0xb4c54056, SUC_lexicon_tags_43 },
    { 0x62c54058, SUC_lexicon_tags_47 },
    { 0xec854058, SUC_lexicon_tags_38 },
    { 0x2c05405b, SUC_lexicon_tags_17 },
    { 0x3ec5405c, SUC_lexicon_tags_47 },
    { 0x3a45405a, SUC_lexicon_tags_0 },
    { 0xcfa54056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc654060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7654063, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12854065, SUC_lexicon_tags_45 },
    { 0xb9654066, SUC_lexicon_tags_47 },
    { 0x65e54065, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe265406c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a654070, SUC_lexicon_tags_15 },
    { 0x6dc54070, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f654075, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1854078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a54086, SUC_lexicon_tags_92 },
    { 0x6a54087, SUC_lexicon_tags_12 },
    { 0x1f854088, SUC_lexicon_tags_11 },
    { 0x44654086, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e25408b, SUC_lexicon_tags_7 },
    { 0xf385408c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e54090, SUC_lexicon_tags_84 },
    { 0xd6054091, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0254097, SUC_lexicon_tags_133 },
    { 0xa9454098, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c5409c, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2540a0, SUC_lexicon_tags_11 },
    { 0x37e540a1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xa0a540a3, SUC_lexicon_tags_16 },
    { 0x86c540a3, SUC_lexicon_tags_43 },
    { 0xa6540a5, SUC_lexicon_tags_8 },
    { 0xd80540a6, SUC_lexicon_tags_25 },
    { 0xfac540a7, SUC_lexicon_tags_30 },
    { 0x722540a8, SUC_lexicon_tags_17 },
    { 0x9ee540a9, SUC_lexicon_tags_77 },
    { 0x3a6540a6, SUC_lexicon_tags_33 },
    { 0x6ca540a3, SUC_lexicon_tags_10 },
    { 0xed2540ac, SUC_lexicon_tags_0 },
    { 0x278540ac, SUC_lexicon_tags_0 },
    { 0xb88540ad, SUC_lexicon_tags_39 },
    { 0x694540af, SUC_lexicon_tags_14 },
    { 0xbbc540ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388540b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc540bd, SUC_lexicon_tags_17 },
    { 0x63e540bd, SUC_lexicon_tags_4 },
    { 0xaf0540bf, SUC_lexicon_tags_28 },
    { 0xd44540c0, SUC_lexicon_tags_4 },
    { 0x800540c1, SUC_lexicon_tags_7 },
    { 0x9ea540c2, SUC_lexicon_tags_31 },
    { 0xcbc540bf, SUC_lexicon_tags_25 },
    { 0xe54540c3, SUC_lexicon_tags_15 },
    { 0x96e540c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9a4540c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22540cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5ae540cf, SUC_lexicon_tags_37 },
    { 0x35a540d0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf0c540d2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0540d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x762540d8, SUC_lexicon_tags_60 },
    { 0x92c540d8, SUC_lexicon_tags_30 },
    { 0xfea540d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc40540dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a540e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42540e8, SUC_lexicon_tags_1 },
    { 0xf86540e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x728540eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56540ee, SUC_lexicon_tags_17 },
    { 0x2e4540ee, SUC_lexicon_tags_14 },
    { 0x86a540ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe4c540f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414540f6, SUC_lexicon_tags_7 },
    { 0x198540f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c540fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x862540fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x500540ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde054108, SUC_lexicon_tags_10 },
    { 0xd0c54108, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f65410b, SUC_lexicon_tags_43 },
    { 0x81c5410c, SUC_lexicon_tags_15 },
    { 0x8a25410d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90654111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d454118, SUC_lexicon_tags_12 },
    { 0x17e54119, SUC_lexicon_tags_31 },
    { 0x9985411a, SUC_lexicon_tags_79 },
    { 0x3d654119, SUC_lexicon_tags_2 },
    { 0xdac5411a, SUC_lexicon_tags_14 },
    { 0xc25411d, SUC_lexicon_tags_37 },
    { 0x48c5411e, SUC_lexicon_tags_2 },
    { 0x385411e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x72c54121, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1854128, SUC_lexicon_tags_42 },
    { 0xd6454129, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef25412f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc54132, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4054134, SUC_lexicon_tags_25 },
    { 0xf8854135, SUC_lexicon_tags_7 },
    { 0x3c454136, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbe454138, SUC_lexicon_tags_63 },
    { 0xd7a54139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe585413b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x29e5413d, SUC_lexicon_tags_37 },
    { 0x8545413d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa654141, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5854143, SUC_lexicon_tags_0 },
    { 0xaa054144, SUC_lexicon_tags_96 },
    { 0x8d454145, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c5414f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26454153, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a45415d, SUC_lexicon_tags_8 },
    { 0x7fe5415e, SUC_lexicon_tags_49 },
    { 0xdd65415d, SUC_lexicon_tags_7 },
    { 0x7d45415d, SUC_lexicon_tags_17 },
    { 0xdac5415f, SUC_lexicon_tags_17 },
    { 0x3fc5415e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b654165, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a54169, SUC_lexicon_tags_37 },
    { 0x1a65416a, SUC_lexicon_tags_37 },
    { 0x7c65416b, SUC_lexicon_tags_11 },
    { 0x7165416c, SUC_lexicon_tags_179 },
    { 0xe4c5416d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7025416f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61254172, SUC_lexicon_tags_2 },
    { 0x3fe54172, SUC_lexicon_tags_29 },
    { 0x91c54174, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x76054176, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x99c54178, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe185417a, SUC_lexicon_tags_33 },
    { 0xe5a5417b, SUC_lexicon_tags_664 },
    { 0x2a25417c, SUC_lexicon_tags_15 },
    { 0x7985417d, SUC_lexicon_tags_265 },
    { 0x2ac5417e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea54181, SUC_lexicon_tags_40 },
    { 0x3de54182, SUC_lexicon_tags_25 },
    { 0xebc54183, SUC_lexicon_tags_46 },
    { 0xcec54182, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6fc54186, SUC_lexicon_tags_12 },
    { 0xfda54187, SUC_lexicon_tags_29 },
    { 0x40454188, SUC_lexicon_tags_9 },
    { 0xb5454188, SUC_lexicon_tags_15 },
    { 0x5fe5418a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa5419a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f25419e, SUC_lexicon_tags_17 },
    { 0x4065419e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc541a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ee541a5, SUC_lexicon_tags_2 },
    { 0x3bc541a6, SUC_lexicon_tags_23 },
    { 0x584541a7, SUC_lexicon_tags_25 },
    { 0x1d8541a8, SUC_lexicon_tags_2 },
    { 0x3e0541a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4541ad, SUC_lexicon_tags_60 },
    { 0x63c541ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4541b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa80541b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e541b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x484541b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4fe541ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52541bf, SUC_lexicon_tags_9 },
    { 0x6ba541c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc541c6, SUC_lexicon_tags_92 },
    { 0xf18541c6, SUC_lexicon_tags_25 },
    { 0x684541c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c541cb, SUC_lexicon_tags_0 },
    { 0x79e541cc, SUC_lexicon_tags_26 },
    { 0x1c541cd, SUC_lexicon_tags_29 },
    { 0x1ec541ce, SUC_lexicon_tags_12 },
    { 0xe7a541cb, SUC_lexicon_tags_3 },
    { 0x3c4541cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62541d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0541d8, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e541dc, SUC_lexicon_tags_25 },
    { 0x1e8541dc, SUC_lexicon_tags_17 },
    { 0xca8541de, SUC_lexicon_tags_32 },
    { 0xe1c541df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a541e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc541e7, SUC_lexicon_tags_54 },
    { 0x7aa541e8, SUC_lexicon_tags_767 },
    { 0x3ae541e9, SUC_lexicon_tags_31 },
    { 0xc08541e9, SUC_lexicon_tags_17 },
    { 0xe72541e9, SUC_lexicon_tags_11 },
    { 0x340541ec, SUC_lexicon_tags_27 },
    { 0xfd0541ec, SUC_lexicon_tags_207 },
    { 0x38c541ed, SUC_lexicon_tags_49 },
    { 0xb1a541ef, SUC_lexicon_tags_7 },
    { 0x602541ef, SUC_lexicon_tags_15 },
    { 0x756541ef, SUC_lexicon_tags_33 },
    { 0x76a541f2, SUC_lexicon_tags_10 },
    { 0xcc0541f3, SUC_lexicon_tags_17 },
    { 0x6a0541f2, SUC_lexicon_tags_32 },
    { 0x786541f5, SUC_lexicon_tags_48 },
    { 0xbf4541f5, SUC_lexicon_tags_23 },
    { 0xe06541ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c541fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530541ff, SUC_lexicon_tags_11 },
    { 0x13454200, SUC_lexicon_tags_23 },
    { 0xf7854201, SUC_lexicon_tags_11 },
    { 0x74854200, SUC_lexicon_tags_1 },
    { 0x41454203, SUC_lexicon_tags_9 },
    { 0xab654204, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc854207, SUC_lexicon_tags_8 },
    { 0xf0254208, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa505420b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26454212, SUC_lexicon_tags_15 },
    { 0xd6e54213, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x64c54215, SUC_lexicon_tags_10 },
    { 0x88c54216, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c54219, SUC_lexicon_tags_33 },
    { 0x82e5421a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b05421c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xace5421f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57454222, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c54225, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x23a54227, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbb654229, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6425422c, SUC_lexicon_tags_29 },
    { 0x5485422c, SUC_lexicon_tags_14 },
    { 0x5845422d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x85c54230, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89a54232, SUC_lexicon_tags_37 },
    { 0xba254232, SUC_lexicon_tags_49 },
    { 0xf6254234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3805423d, SUC_lexicon_tags_4 },
    { 0x33a5423e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c54241, SUC_lexicon_tags_1 },
    { 0x46854242, SUC_lexicon_tags_25 },
    { 0xcda54243, SUC_lexicon_tags_1 },
    { 0x73454241, SUC_lexicon_tags_37 },
    { 0xc9854245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc65424a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a25424f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe325425b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xdf85425d, SUC_lexicon_tags_28 },
    { 0x5205425d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf905426f, SUC_lexicon_tags_64 },
    { 0xc7654270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e54273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76454277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d65427c, SUC_lexicon_tags_2 },
    { 0x36e5427d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d254280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a854282, SUC_lexicon_tags_28 },
    { 0x96254283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8254285, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e45428a, SUC_lexicon_tags_38 },
    { 0xeb25428b, SUC_lexicon_tags_23 },
    { 0xb9a5428c, SUC_lexicon_tags_2 },
    { 0x70e5428d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9054291, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c54294, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e5429b, SUC_lexicon_tags_43 },
    { 0xe665429c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea542a0, SUC_lexicon_tags_37 },
    { 0x46a542a0, SUC_lexicon_tags_37 },
    { 0xaf2542a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76542a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf6542a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8542af, SUC_lexicon_tags_35 },
    { 0x70542af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82542b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x22c542b5, SUC_lexicon_tags_17 },
    { 0x264542b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44542bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73a542be, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c542c5, SUC_lexicon_tags_14 },
    { 0x484542c6, SUC_lexicon_tags_2 },
    { 0xc2e542c6, SUC_lexicon_tags_37 },
    { 0x128542c8, SUC_lexicon_tags_47 },
    { 0xfea542c9, SUC_lexicon_tags_64 },
    { 0x102542c9, SUC_lexicon_tags_37 },
    { 0xb42542cb, SUC_lexicon_tags_7 },
    { 0x74c542cc, SUC_lexicon_tags_17 },
    { 0x586542cc, SUC_lexicon_tags_12 },
    { 0x4e2542ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee542d3, SUC_lexicon_tags_33 },
    { 0x4f4542d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300542e1, SUC_lexicon_tags_18 },
    { 0x39a542e2, SUC_lexicon_tags_6 },
    { 0x70c542e3, SUC_lexicon_tags_48 },
    { 0x666542e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe542ed, SUC_lexicon_tags_65 },
    { 0x968542ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62542f1, SUC_lexicon_tags_46 },
    { 0x9fa542f2, SUC_lexicon_tags_16 },
    { 0x23e542f2, SUC_lexicon_tags_25 },
    { 0xbca542f1, SUC_lexicon_tags_12 },
    { 0xd6c542f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8542f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x358542fc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3da542fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca54305, SUC_lexicon_tags_47 },
    { 0x20454305, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14854308, SUC_lexicon_tags_17 },
    { 0xd4e54308, SUC_lexicon_tags_7 },
    { 0x23454309, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5365430e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91254312, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12854315, SUC_lexicon_tags_25 },
    { 0xc7a54316, SUC_lexicon_tags_38 },
    { 0x31854315, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb45431d, SUC_lexicon_tags_11 },
    { 0x105431d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9054320, SUC_lexicon_tags_29 },
    { 0xaf854320, SUC_lexicon_tags_17 },
    { 0xa1a54320, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda454329, SUC_lexicon_tags_10 },
    { 0xb1c5432a, SUC_lexicon_tags_45 },
    { 0xbdc5432b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1465432e, SUC_lexicon_tags_45 },
    { 0x3965432e, SUC_lexicon_tags_21 },
    { 0x3a454330, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1ea54332, SUC_lexicon_tags_15 },
    { 0x6654333, SUC_lexicon_tags_11 },
    { 0x26254334, SUC_lexicon_tags_11 },
    { 0xe2254334, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6454338, SUC_lexicon_tags_28 },
    { 0x7ec54338, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x445433b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5433f, SUC_lexicon_tags_61 },
    { 0x46054340, SUC_lexicon_tags_47 },
    { 0x4145433f, SUC_lexicon_tags_37 },
    { 0xbce54341, SUC_lexicon_tags_17 },
    { 0xb8e54343, SUC_lexicon_tags_25 },
    { 0x4a654344, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c54347, SUC_lexicon_tags_27 },
    { 0x15254348, SUC_lexicon_tags_41 },
    { 0xf6e54348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c5434c, SUC_lexicon_tags_49 },
    { 0xe865434c, SUC_lexicon_tags_17 },
    { 0x1765434c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e54353, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6e54355, SUC_lexicon_tags_11 },
    { 0x58854356, SUC_lexicon_tags_0 },
    { 0x1ac54357, SUC_lexicon_tags_105 },
    { 0x52054356, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5435d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16054362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e54365, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e54368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6285436a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35254372, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda854379, SUC_lexicon_tags_26 },
    { 0xfa05437a, SUC_lexicon_tags_1 },
    { 0x7125437a, SUC_lexicon_tags_38 },
    { 0x3dc5437b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5145437e, SUC_lexicon_tags_66 },
    { 0xfa85437f, SUC_lexicon_tags_37 },
    { 0x5765437f, SUC_lexicon_tags_26 },
    { 0xd2654381, SUC_lexicon_tags_101 },
    { 0xc054380, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc5654384, SUC_lexicon_tags_23 },
    { 0x23a54385, SUC_lexicon_tags_9 },
    { 0xc0e54386, SUC_lexicon_tags_47 },
    { 0x58254386, SUC_lexicon_tags_17 },
    { 0xdc654388, SUC_lexicon_tags_129 },
    { 0xfba54384, SUC_lexicon_tags_0 },
    { 0x5c85438a, SUC_lexicon_tags_6 },
    { 0x4a854389, SUC_lexicon_tags_1 },
    { 0x59a54385, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec54391, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5a454393, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e25439c, SUC_lexicon_tags_45 },
    { 0x21a5439d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdaa5439f, SUC_lexicon_tags_7 },
    { 0xd7c5439f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840543a4, SUC_lexicon_tags_37 },
    { 0x3fa543a5, SUC_lexicon_tags_32 },
    { 0x356543a5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb4a543a8, SUC_lexicon_tags_47 },
    { 0x6ae543a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18543ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x226543af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac543b3, SUC_lexicon_tags_8 },
    { 0xfc543b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5fc543b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6543ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e543be, SUC_lexicon_tags_17 },
    { 0xbb8543bf, SUC_lexicon_tags_11 },
    { 0x8a8543be, SUC_lexicon_tags_57 },
    { 0xf00543c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40543c5, SUC_lexicon_tags_38 },
    { 0xee4543c6, SUC_lexicon_tags_12 },
    { 0x9a6543c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a543cb, SUC_lexicon_tags_14 },
    { 0xf8e543cc, SUC_lexicon_tags_0 },
    { 0x180543cd, SUC_lexicon_tags_28 },
    { 0xe5a543cb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184543d3, SUC_lexicon_tags_12 },
    { 0x27e543d3, SUC_lexicon_tags_12 },
    { 0x632543d5, SUC_lexicon_tags_12 },
    { 0x406543d5, SUC_lexicon_tags_2 },
    { 0x93e543d7, SUC_lexicon_tags_45 },
    { 0x846543d8, SUC_lexicon_tags_25 },
    { 0xad2543d4, SUC_lexicon_tags_15 },
    { 0x258543da, SUC_lexicon_tags_14 },
    { 0xa543db, SUC_lexicon_tags_11 },
    { 0xa72543dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd4543de, SUC_lexicon_tags_9 },
    { 0x636543df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6543e2, SUC_lexicon_tags_37 },
    { 0x6ea543e3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x746543e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3fa543e7, SUC_lexicon_tags_29 },
    { 0x9c543e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9a543ea, SUC_lexicon_tags_37 },
    { 0x3c0543eb, SUC_lexicon_tags_27 },
    { 0xb5a543ec, SUC_lexicon_tags_17 },
    { 0xe8c543eb, SUC_lexicon_tags_61 },
    { 0x40e543eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x510543f0, SUC_lexicon_tags_27 },
    { 0x8aa543f1, SUC_lexicon_tags_1 },
    { 0xf0e543f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686543f6, SUC_lexicon_tags_25 },
    { 0xde543f7, SUC_lexicon_tags_19 },
    { 0xe12543f8, SUC_lexicon_tags_47 },
    { 0xca4543f7, SUC_lexicon_tags_17 },
    { 0xd3e543f7, SUC_lexicon_tags_38 },
    { 0xd0c543f9, SUC_lexicon_tags_17 },
    { 0x6a2543f8, SUC_lexicon_tags_70 },
    { 0x78543fd, SUC_lexicon_tags_19 },
    { 0x278543fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x31854401, SUC_lexicon_tags_0 },
    { 0xe7254402, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe105440d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6854411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe054414, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c5441c, SUC_lexicon_tags_14 },
    { 0x84e5441c, SUC_lexicon_tags_1 },
    { 0x12c5441e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf254422, SUC_lexicon_tags_0 },
    { 0xd5654423, SUC_lexicon_tags_17 },
    { 0x37854422, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca54427, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9be54429, SUC_lexicon_tags_31 },
    { 0x885442a, SUC_lexicon_tags_61 },
    { 0x41e5442a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc54431, SUC_lexicon_tags_2 },
    { 0x5a54432, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e54438, SUC_lexicon_tags_27 },
    { 0x1e854438, SUC_lexicon_tags_12 },
    { 0x6825443a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4105443d, SUC_lexicon_tags_55 },
    { 0x9ba5443e, SUC_lexicon_tags_17 },
    { 0x8c45443d, SUC_lexicon_tags_37 },
    { 0xfe254440, SUC_lexicon_tags_11 },
    { 0xbe054441, SUC_lexicon_tags_47 },
    { 0xae854440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d054444, SUC_lexicon_tags_194 },
    { 0x47454445, SUC_lexicon_tags_43 },
    { 0xca054446, SUC_lexicon_tags_25 },
    { 0x5fa54446, SUC_lexicon_tags_1 },
    { 0x35c54448, SUC_lexicon_tags_49 },
    { 0x55c54445, SUC_lexicon_tags_9 },
    { 0xc254444, SUC_lexicon_tags_151 },
    { 0x49c5444b, SUC_lexicon_tags_63 },
    { 0x27a5444c, SUC_lexicon_tags_17 },
    { 0x9ea5444c, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x64a5444f, SUC_lexicon_tags_29 },
    { 0x7aa54450, SUC_lexicon_tags_14 },
    { 0x70c54451, SUC_lexicon_tags_13 },
    { 0x1e054450, SUC_lexicon_tags_25 },
    { 0x1c254453, SUC_lexicon_tags_11 },
    { 0x87a54451, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe54457, SUC_lexicon_tags_25 },
    { 0x7ac54458, SUC_lexicon_tags_9 },
    { 0x5c54459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ba5445b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb725445d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xad654460, SUC_lexicon_tags_33 },
    { 0x63454461, SUC_lexicon_tags_1 },
    { 0xf6e54462, SUC_lexicon_tags_37 },
    { 0x7e54461, SUC_lexicon_tags_17 },
    { 0xe9c54464, SUC_lexicon_tags_33 },
    { 0xa0854464, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0254467, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9454474, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a05447c, SUC_lexicon_tags_29 },
    { 0xa85447d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c54480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc54485, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c5448b, SUC_lexicon_tags_17 },
    { 0x8c65448b, SUC_lexicon_tags_37 },
    { 0x51c5448b, SUC_lexicon_tags_17 },
    { 0xd1e5448d, SUC_lexicon_tags_17 },
    { 0xce5448f, SUC_lexicon_tags_3 },
    { 0x83a5448f, SUC_lexicon_tags_27 },
    { 0x9ac54491, SUC_lexicon_tags_38 },
    { 0xf6254492, SUC_lexicon_tags_30 },
    { 0x2d454492, SUC_lexicon_tags_11 },
    { 0x66e54492, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd0e54496, SUC_lexicon_tags_17 },
    { 0x25c54497, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8f854499, SUC_lexicon_tags_60 },
    { 0x7cc5449a, SUC_lexicon_tags_0 },
    { 0x26e5449b, SUC_lexicon_tags_1 },
    { 0xc05449b, SUC_lexicon_tags_17 },
    { 0x6cc5449d, SUC_lexicon_tags_37 },
    { 0xac85449a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a544a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde544a5, SUC_lexicon_tags_1 },
    { 0x1ea544a6, SUC_lexicon_tags_1038 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90544ae, SUC_lexicon_tags_10 },
    { 0x63e544af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e544b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x254544bb, SUC_lexicon_tags_23 },
    { 0x8aa544bc, SUC_lexicon_tags_47 },
    { 0x94e544bc, SUC_lexicon_tags_37 },
    { 0x78a544bb, SUC_lexicon_tags_7 },
    { 0x9b2544bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0544c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce544c6, SUC_lexicon_tags_1 },
    { 0xfdc544c6, SUC_lexicon_tags_37 },
    { 0xd30544c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14544cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de544cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a544d2, SUC_lexicon_tags_17 },
    { 0x4e4544d2, SUC_lexicon_tags_1 },
    { 0xbb8544d4, SUC_lexicon_tags_8 },
    { 0xb10544d4, SUC_lexicon_tags_82 },
    { 0xf58544d5, SUC_lexicon_tags_12 },
    { 0xee6544d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994544db, SUC_lexicon_tags_55 },
    { 0xd36544dc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712544e0, SUC_lexicon_tags_2 },
    { 0x6c2544e1, SUC_lexicon_tags_27 },
    { 0xa38544e2, SUC_lexicon_tags_2 },
    { 0x89c544e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da544e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa544ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c544f2, SUC_lexicon_tags_25 },
    { 0xb90544f3, SUC_lexicon_tags_10 },
    { 0x83e544f4, SUC_lexicon_tags_38 },
    { 0x582544f5, SUC_lexicon_tags_37 },
    { 0x45c544f2, SUC_lexicon_tags_82 },
    { 0xa0e544f7, SUC_lexicon_tags_60 },
    { 0xb14544f7, SUC_lexicon_tags_18 },
    { 0xf88544f9, SUC_lexicon_tags_18 },
    { 0xbc0544fa, SUC_lexicon_tags_11 },
    { 0x814544f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c544fe, SUC_lexicon_tags_37 },
    { 0xeec544fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e54503, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca5450b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b25450d, SUC_lexicon_tags_25 },
    { 0xfe45450e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e5451c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a54520, SUC_lexicon_tags_2 },
    { 0x8a854521, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0e54523, SUC_lexicon_tags_60 },
    { 0x51c54524, SUC_lexicon_tags_7 },
    { 0xcd454525, SUC_lexicon_tags_37 },
    { 0xa1a54524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b454529, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71c5452b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5452e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f254530, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae54533, SUC_lexicon_tags_45 },
    { 0x7a454533, SUC_lexicon_tags_45 },
    { 0xa3654534, SUC_lexicon_tags_25 },
    { 0xbce54536, SUC_lexicon_tags_32 },
    { 0xd7054533, SUC_lexicon_tags_25 },
    { 0xab454538, SUC_lexicon_tags_8 },
    { 0x20e54537, SUC_lexicon_tags_8 },
    { 0x5825453a, SUC_lexicon_tags_37 },
    { 0x8e25453b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1654541, SUC_lexicon_tags_12 },
    { 0x80654542, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8e054544, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30854548, SUC_lexicon_tags_2 },
    { 0x33a54549, SUC_lexicon_tags_7 },
    { 0x4a5454a, SUC_lexicon_tags_14 },
    { 0x16854548, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd65454e, SUC_lexicon_tags_12 },
    { 0xcd85454f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d254552, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x8cc54554, SUC_lexicon_tags_12 },
    { 0xba854554, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34654557, SUC_lexicon_tags_63 },
    { 0x85254557, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa305455b, SUC_lexicon_tags_11 },
    { 0x7485455c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95e5455e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de54563, SUC_lexicon_tags_6 },
    { 0xc9054563, SUC_lexicon_tags_45 },
    { 0x28054565, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a5456c, SUC_lexicon_tags_37 },
    { 0x1e05456c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9425456f, SUC_lexicon_tags_16 },
    { 0xb4454570, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa54575, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd625457b, SUC_lexicon_tags_25 },
    { 0xfec5457c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf945457e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc3454580, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9c54582, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca854584, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x81054587, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c25458c, SUC_lexicon_tags_38 },
    { 0xba85458c, SUC_lexicon_tags_33 },
    { 0x26c5458e, SUC_lexicon_tags_17 },
    { 0x8c45458f, SUC_lexicon_tags_46 },
    { 0x83054590, SUC_lexicon_tags_1 },
    { 0x22654591, SUC_lexicon_tags_56 },
    { 0xff05458c, SUC_lexicon_tags_0 },
    { 0x2d45458f, SUC_lexicon_tags_30 },
    { 0x9ea54593, SUC_lexicon_tags_68 },
    { 0x2b454595, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf925459a, SUC_lexicon_tags_12 },
    { 0x8b85459a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96545a0, SUC_lexicon_tags_7 },
    { 0x382545a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2545a5, SUC_lexicon_tags_29 },
    { 0xb8e545a6, SUC_lexicon_tags_28 },
    { 0x368545a6, SUC_lexicon_tags_13 },
    { 0x1c8545a8, SUC_lexicon_tags_154 },
    { 0x49e545a8, SUC_lexicon_tags_45 },
    { 0x77e545aa, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x750545ac, SUC_lexicon_tags_31 },
    { 0x5cc545ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x182545af, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x302545b2, SUC_lexicon_tags_0 },
    { 0x65e545b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe08545b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4545ba, SUC_lexicon_tags_37 },
    { 0xcea545bb, SUC_lexicon_tags_63 },
    { 0x8cc545bb, SUC_lexicon_tags_15 },
    { 0xc34545ba, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0xbe6545bf, SUC_lexicon_tags_37 },
    { 0xee6545bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e545c5, SUC_lexicon_tags_33 },
    { 0x7d6545c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2545ce, SUC_lexicon_tags_18 },
    { 0x4c8545cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6545d2, SUC_lexicon_tags_37 },
    { 0x428545d2, SUC_lexicon_tags_7 },
    { 0x848545d4, SUC_lexicon_tags_17 },
    { 0x42c545d3, SUC_lexicon_tags_12 },
    { 0xe0c545d2, SUC_lexicon_tags_7 },
    { 0xd58545d6, SUC_lexicon_tags_11 },
    { 0xc0e545d2, SUC_lexicon_tags_23 },
    { 0xb88545d9, SUC_lexicon_tags_37 },
    { 0xbce545d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884545e3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290545e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e545ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x26e545ef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e545f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1ec545f5, SUC_lexicon_tags_55 },
    { 0xddc545f5, SUC_lexicon_tags_11 },
    { 0x3c545f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcae545f9, SUC_lexicon_tags_38 },
    { 0xb12545fa, SUC_lexicon_tags_87 },
    { 0xa6e545fb, SUC_lexicon_tags_1 },
    { 0x230545fc, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48545ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcac54601, SUC_lexicon_tags_7 },
    { 0xc9c54601, SUC_lexicon_tags_28 },
    { 0xda854602, SUC_lexicon_tags_27 },
    { 0xc4454604, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda854606, SUC_lexicon_tags_49 },
    { 0x3c054607, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc5460c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1365460e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a54617, SUC_lexicon_tags_8 },
    { 0x2e454618, SUC_lexicon_tags_1 },
    { 0x71654619, SUC_lexicon_tags_38 },
    { 0x14c5461a, SUC_lexicon_tags_9 },
    { 0x15654619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8105461d, SUC_lexicon_tags_19 },
    { 0x91a5461d, SUC_lexicon_tags_7 },
    { 0x3e85461f, SUC_lexicon_tags_27 },
    { 0x62c5461e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36454624, SUC_lexicon_tags_61 },
    { 0x3b254625, SUC_lexicon_tags_8 },
    { 0xdb454626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92254628, SUC_lexicon_tags_17 },
    { 0x50454628, SUC_lexicon_tags_25 },
    { 0x2a65462a, SUC_lexicon_tags_0 },
    { 0x6645462b, SUC_lexicon_tags_47 },
    { 0x1965462c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9745462e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xca054630, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0654634, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x29054636, SUC_lexicon_tags_11 },
    { 0xbea54637, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65463a, SUC_lexicon_tags_14 },
    { 0x2685463a, SUC_lexicon_tags_33 },
    { 0xf645463c, SUC_lexicon_tags_14 },
    { 0x6e05463d, SUC_lexicon_tags_37 },
    { 0x2485463b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3aa54640, SUC_lexicon_tags_17 },
    { 0x9dc54641, SUC_lexicon_tags_1 },
    { 0xee654642, SUC_lexicon_tags_33 },
    { 0x91854643, SUC_lexicon_tags_14 },
    { 0x79c54643, SUC_lexicon_tags_39 },
    { 0xc0e54640, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65654648, SUC_lexicon_tags_37 },
    { 0x5b054648, SUC_lexicon_tags_1 },
    { 0xbc054648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a54654, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x50454656, SUC_lexicon_tags_45 },
    { 0xdb254657, SUC_lexicon_tags_17 },
    { 0xd9654658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe45465a, SUC_lexicon_tags_38 },
    { 0x3845465b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d25465f, SUC_lexicon_tags_0 },
    { 0xa854660, SUC_lexicon_tags_32 },
    { 0x93854661, SUC_lexicon_tags_43 },
    { 0xa4454660, SUC_lexicon_tags_15 },
    { 0x78054660, SUC_lexicon_tags_11 },
    { 0x7aa54664, SUC_lexicon_tags_12 },
    { 0x2ea54665, SUC_lexicon_tags_12 },
    { 0x18a54665, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x26c54668, SUC_lexicon_tags_60 },
    { 0xc1254668, SUC_lexicon_tags_30 },
    { 0x9ae5466a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe025466f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42654671, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0454676, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xca254678, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x965467b, SUC_lexicon_tags_47 },
    { 0x1645467c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e54680, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48054686, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1b454688, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac5468c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94054691, SUC_lexicon_tags_56 },
    { 0x85854692, SUC_lexicon_tags_35 },
    { 0xd3454693, SUC_lexicon_tags_13 },
    { 0xd1054694, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x18254696, SUC_lexicon_tags_10 },
    { 0xe2854697, SUC_lexicon_tags_23 },
    { 0x6be54698, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e5469c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4725469f, SUC_lexicon_tags_12 },
    { 0x9c5469f, SUC_lexicon_tags_37 },
    { 0x618546a0, SUC_lexicon_tags_55 },
    { 0x1ea5469f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc546a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426546ad, SUC_lexicon_tags_27 },
    { 0x5e8546ae, SUC_lexicon_tags_14 },
    { 0x194546ae, SUC_lexicon_tags_12 },
    { 0xa8a546b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8546b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x864546b8, SUC_lexicon_tags_25 },
    { 0x6e0546b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8c546bb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84546c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30546c4, SUC_lexicon_tags_17 },
    { 0x160546c5, SUC_lexicon_tags_31 },
    { 0xbde546c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e546cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794546d2, SUC_lexicon_tags_4 },
    { 0x814546d3, SUC_lexicon_tags_17 },
    { 0x68a546d4, SUC_lexicon_tags_26 },
    { 0x4b0546d5, SUC_lexicon_tags_31 },
    { 0x100546d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x836546d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x590546db, SUC_lexicon_tags_6 },
    { 0x8e4546dc, SUC_lexicon_tags_25 },
    { 0x672546dd, SUC_lexicon_tags_7 },
    { 0xaa4546de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb12546e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ba546e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcde546e4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x87e546e6, SUC_lexicon_tags_12 },
    { 0x108546e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x660546e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de546ec, SUC_lexicon_tags_7 },
    { 0xde4546ed, SUC_lexicon_tags_7 },
    { 0x906546ed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf5c546f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c546f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0546f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a546fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0e546ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8454707, SUC_lexicon_tags_57 },
    { 0xfea54708, SUC_lexicon_tags_47 },
    { 0x76654708, SUC_lexicon_tags_15 },
    { 0xf8454709, SUC_lexicon_tags_37 },
    { 0xebc54709, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6245470d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54454712, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc865471c, SUC_lexicon_tags_11 },
    { 0xc425471d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbae5471f, SUC_lexicon_tags_33 },
    { 0xf3454720, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3645472b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1185472e, SUC_lexicon_tags_82 },
    { 0xd345472f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47e54731, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41854733, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdd854735, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe5054737, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xace54739, SUC_lexicon_tags_11 },
    { 0x52c5473a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa685473d, SUC_lexicon_tags_17 },
    { 0x4dc5473d, SUC_lexicon_tags_11 },
    { 0xccc5473f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee254743, SUC_lexicon_tags_25 },
    { 0x30254743, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf0e54746, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9645474a, SUC_lexicon_tags_1 },
    { 0x6ba5474b, SUC_lexicon_tags_25 },
    { 0xd8e5474a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9105474e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8ac54750, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc54753, SUC_lexicon_tags_1 },
    { 0xf3e54753, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c054756, SUC_lexicon_tags_12 },
    { 0xee654757, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e25475a, SUC_lexicon_tags_25 },
    { 0x8d05475b, SUC_lexicon_tags_15 },
    { 0x9d25475c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa025475e, SUC_lexicon_tags_25 },
    { 0xfa5475f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c54767, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea5476c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28054773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c54777, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8185477a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7e5477c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb625477f, SUC_lexicon_tags_25 },
    { 0xc2654780, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e54783, SUC_lexicon_tags_11 },
    { 0x1054783, SUC_lexicon_tags_47 },
    { 0xd1854784, SUC_lexicon_tags_1168 },
    { 0x7e654786, SUC_lexicon_tags_47 },
    { 0xfe854787, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1265478a, SUC_lexicon_tags_12 },
    { 0x2445478a, SUC_lexicon_tags_14 },
    { 0x35a5478b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a5478f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75654795, SUC_lexicon_tags_2 },
    { 0x7ca54796, SUC_lexicon_tags_37 },
    { 0x33854797, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c5479a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146547a0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710547a4, SUC_lexicon_tags_11 },
    { 0xc2c547a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88547aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a547ae, SUC_lexicon_tags_12 },
    { 0x67e547af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12e547b1, SUC_lexicon_tags_15 },
    { 0x626547b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538547b6, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xb14547b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x272547ba, SUC_lexicon_tags_49 },
    { 0x60547bb, SUC_lexicon_tags_5 },
    { 0xf50547ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb2547be, SUC_lexicon_tags_45 },
    { 0xda547be, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334547c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea547c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80547d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c6547d2, SUC_lexicon_tags_27 },
    { 0xff8547d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0547d6, SUC_lexicon_tags_1 },
    { 0xaf8547d6, SUC_lexicon_tags_11 },
    { 0x34e547d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x576547da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2e8547dc, SUC_lexicon_tags_2 },
    { 0x706547dc, SUC_lexicon_tags_17 },
    { 0xf1a547dc, SUC_lexicon_tags_37 },
    { 0x7c4547df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8547e3, SUC_lexicon_tags_63 },
    { 0xe8c547e3, SUC_lexicon_tags_37 },
    { 0x104547e5, SUC_lexicon_tags_28 },
    { 0xc5e547e4, SUC_lexicon_tags_37 },
    { 0x886547e7, SUC_lexicon_tags_15 },
    { 0xea4547e8, SUC_lexicon_tags_15 },
    { 0x7d2547e7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xb7a547eb, SUC_lexicon_tags_1 },
    { 0xb70547ec, SUC_lexicon_tags_2 },
    { 0xff6547ec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858547f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78547f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0547f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa547ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8654801, SUC_lexicon_tags_23 },
    { 0xb5654802, SUC_lexicon_tags_38 },
    { 0x73e54803, SUC_lexicon_tags_11 },
    { 0xa2254803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54854807, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe005480a, SUC_lexicon_tags_30 },
    { 0x6e65480b, SUC_lexicon_tags_18 },
    { 0xea45480a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e54811, SUC_lexicon_tags_17 },
    { 0x68c54812, SUC_lexicon_tags_25 },
    { 0x9254811, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8254819, SUC_lexicon_tags_1 },
    { 0xa1654819, SUC_lexicon_tags_63 },
    { 0x2085481b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1fc5481d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e54823, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x99254826, SUC_lexicon_tags_1 },
    { 0x42654827, SUC_lexicon_tags_9 },
    { 0x99054827, SUC_lexicon_tags_37 },
    { 0x44454828, SUC_lexicon_tags_25 },
    { 0x7dc54828, SUC_lexicon_tags_30 },
    { 0xd305482b, SUC_lexicon_tags_17 },
    { 0x84254828, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30254830, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a54833, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec454835, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a5483b, SUC_lexicon_tags_17 },
    { 0xdbc5483c, SUC_lexicon_tags_12 },
    { 0xf765483d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e54842, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7a54844, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67254849, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a654856, SUC_lexicon_tags_0 },
    { 0xbe254857, SUC_lexicon_tags_28 },
    { 0x64854858, SUC_lexicon_tags_47 },
    { 0x6ec54857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba5485d, SUC_lexicon_tags_12 },
    { 0xbc85485e, SUC_lexicon_tags_15 },
    { 0x4e25485e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51854863, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a854865, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x60454868, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2745486e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9254871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8454878, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7085487b, SUC_lexicon_tags_12 },
    { 0xe085487c, SUC_lexicon_tags_17 },
    { 0xaf05487d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x46c5487f, SUC_lexicon_tags_17 },
    { 0x6d254880, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36c54882, SUC_lexicon_tags_6 },
    { 0xd5e54883, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3254885, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26654888, SUC_lexicon_tags_17 },
    { 0x5b654889, SUC_lexicon_tags_66 },
    { 0x9945488a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85488d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde54894, SUC_lexicon_tags_7 },
    { 0x99654895, SUC_lexicon_tags_8 },
    { 0x5254896, SUC_lexicon_tags_37 },
    { 0xa9e54894, SUC_lexicon_tags_7 },
    { 0xb5a54898, SUC_lexicon_tags_25 },
    { 0x2da54899, SUC_lexicon_tags_37 },
    { 0x90c54899, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf205489d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a548a1, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986548a7, SUC_lexicon_tags_47 },
    { 0xfce548a7, SUC_lexicon_tags_79 },
    { 0x326548a8, SUC_lexicon_tags_113 },
    { 0x4b8548aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68548b0, SUC_lexicon_tags_11 },
    { 0xa6548b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6548b6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604548ba, SUC_lexicon_tags_11 },
    { 0xc84548ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12548cc, SUC_lexicon_tags_9 },
    { 0x7ea548cd, SUC_lexicon_tags_7 },
    { 0x740548ce, SUC_lexicon_tags_26 },
    { 0x670548cf, SUC_lexicon_tags_28 },
    { 0x49a548cd, SUC_lexicon_tags_49 },
    { 0x32c548ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284548d6, SUC_lexicon_tags_25 },
    { 0xc4e548d6, SUC_lexicon_tags_25 },
    { 0xbfe548d8, SUC_lexicon_tags_1 },
    { 0x8ee548d9, SUC_lexicon_tags_7 },
    { 0xac2548da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de548e0, SUC_lexicon_tags_65 },
    { 0x2e4548e1, SUC_lexicon_tags_15 },
    { 0x6d0548e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x202548e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52548ed, SUC_lexicon_tags_47 },
    { 0x4e0548ee, SUC_lexicon_tags_35 },
    { 0x7fc548ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x712548f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8548f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2548fb, SUC_lexicon_tags_14 },
    { 0xfe6548fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a54900, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c054908, SUC_lexicon_tags_45 },
    { 0x89454909, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba25490e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37054910, SUC_lexicon_tags_0 },
    { 0x8ec54911, SUC_lexicon_tags_19 },
    { 0xfe254910, SUC_lexicon_tags_26 },
    { 0x58254913, SUC_lexicon_tags_26 },
    { 0x8ae54912, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x74a54916, SUC_lexicon_tags_87 },
    { 0xdac54917, SUC_lexicon_tags_12 },
    { 0x26654917, SUC_lexicon_tags_23 },
    { 0x85c54918, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a85491b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf205491d, SUC_lexicon_tags_127 },
    { 0xf705491d, SUC_lexicon_tags_28 },
    { 0xec65491f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a54927, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6254929, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f25492d, SUC_lexicon_tags_12 },
    { 0xa485492d, SUC_lexicon_tags_1 },
    { 0xa685492e, SUC_lexicon_tags_31 },
    { 0x9c65492d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa54935, SUC_lexicon_tags_11 },
    { 0x1dc54936, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5685493e, SUC_lexicon_tags_12 },
    { 0xa685493f, SUC_lexicon_tags_26 },
    { 0xfca5493e, SUC_lexicon_tags_11 },
    { 0xa425493e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x70254943, SUC_lexicon_tags_30 },
    { 0x44854944, SUC_lexicon_tags_37 },
    { 0x3f454945, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1705494c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca5494f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde54954, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x72e54956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93c54958, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x91e5495a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e85495d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38854963, SUC_lexicon_tags_17 },
    { 0x80054964, SUC_lexicon_tags_29 },
    { 0x2254965, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8054968, SUC_lexicon_tags_63 },
    { 0x9a654968, SUC_lexicon_tags_11 },
    { 0xe3e54969, SUC_lexicon_tags_160 },
    { 0x9a65496b, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x8dc5496d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c54970, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x36654973, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a854979, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc645497d, SUC_lexicon_tags_12 },
    { 0xee05497e, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a54983, SUC_lexicon_tags_25 },
    { 0xb3e54984, SUC_lexicon_tags_1 },
    { 0xd5e54983, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe54988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e54993, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c54996, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc9054998, SUC_lexicon_tags_47 },
    { 0x1c654998, SUC_lexicon_tags_19 },
    { 0x29c54999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99c5499c, SUC_lexicon_tags_12 },
    { 0x6fc5499d, SUC_lexicon_tags_37 },
    { 0xb605499d, SUC_lexicon_tags_37 },
    { 0x3a65499f, SUC_lexicon_tags_30 },
    { 0xb5c549a0, SUC_lexicon_tags_61 },
    { 0xc8c549a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x972549a4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6549a9, SUC_lexicon_tags_45 },
    { 0xaac549aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60549ad, SUC_lexicon_tags_14 },
    { 0x62e549ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624549b6, SUC_lexicon_tags_8 },
    { 0x3a2549b7, SUC_lexicon_tags_69 },
    { 0x830549b6, SUC_lexicon_tags_0 },
    { 0x9ca549b7, SUC_lexicon_tags_15 },
    { 0xc1a549ba, SUC_lexicon_tags_25 },
    { 0x276549ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57a549bd, SUC_lexicon_tags_2 },
    { 0x740549be, SUC_lexicon_tags_30 },
    { 0x9ce549bf, SUC_lexicon_tags_25 },
    { 0x542549bf, SUC_lexicon_tags_25 },
    { 0x216549c1, SUC_lexicon_tags_151 },
    { 0x36c549c0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x58a549c4, SUC_lexicon_tags_25 },
    { 0xe8c549c5, SUC_lexicon_tags_47 },
    { 0x5f6549c6, SUC_lexicon_tags_11 },
    { 0x1ea549c7, SUC_lexicon_tags_47 },
    { 0x368549c8, SUC_lexicon_tags_10 },
    { 0xba0549c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a549cc, SUC_lexicon_tags_1 },
    { 0x362549cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30549d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60549d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0549da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24549de, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x414549e1, SUC_lexicon_tags_7 },
    { 0xdaa549e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa66549e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d8549e6, SUC_lexicon_tags_7 },
    { 0xcae549e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9e6549e9, SUC_lexicon_tags_0 },
    { 0x3c8549e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x610549ed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0549f6, SUC_lexicon_tags_2 },
    { 0x3f2549f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0549fa, SUC_lexicon_tags_0 },
    { 0x708549fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0549fe, SUC_lexicon_tags_10 },
    { 0x4c549ff, SUC_lexicon_tags_45 },
    { 0xb4c54a00, SUC_lexicon_tags_37 },
    { 0xe654a00, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xcd654a03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec54a08, SUC_lexicon_tags_16 },
    { 0x8a454a09, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xd1254a0b, SUC_lexicon_tags_1150 },
    { 0, NULL },
    { 0x4f654a0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec54a0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a254a16, SUC_lexicon_tags_17 },
    { 0x9c854a17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda054a19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7854a1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30254a1e, SUC_lexicon_tags_5 },
    { 0xc6c54a1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c54a25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5254a28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69654a2b, SUC_lexicon_tags_33 },
    { 0x6454a2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e54a30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27c54a32, SUC_lexicon_tags_2 },
    { 0x6fc54a33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x32054a36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de54a3a, SUC_lexicon_tags_17 },
    { 0x70a54a3a, SUC_lexicon_tags_7 },
    { 0x92c54a3c, SUC_lexicon_tags_10 },
    { 0xeea54a3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51a54a3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c854a42, SUC_lexicon_tags_17 },
    { 0x7f654a43, SUC_lexicon_tags_11 },
    { 0x48454a42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb5854a46, SUC_lexicon_tags_17 },
    { 0x19454a47, SUC_lexicon_tags_47 },
    { 0x92854a48, SUC_lexicon_tags_25 },
    { 0x8b254a47, SUC_lexicon_tags_38 },
    { 0xf8454a4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8ac54a4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11654a4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa254a52, SUC_lexicon_tags_19 },
    { 0x55454a53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c54a56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5654a5d, SUC_lexicon_tags_25 },
    { 0xd9854a5d, SUC_lexicon_tags_6 },
    { 0x93854a5f, SUC_lexicon_tags_1 },
    { 0xdb454a60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa454a62, SUC_lexicon_tags_14 },
    { 0xf1054a62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe6c54a65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5254a68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96254a76, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a54a79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e454a80, SUC_lexicon_tags_31 },
    { 0x82c54a81, SUC_lexicon_tags_12 },
    { 0x23454a82, SUC_lexicon_tags_8 },
    { 0x75c54a83, SUC_lexicon_tags_10 },
    { 0xfee54a80, SUC_lexicon_tags_39 },
    { 0x3654a82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c454a88, SUC_lexicon_tags_27 },
    { 0xb9c54a88, SUC_lexicon_tags_15 },
    { 0xec854a8a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa054a90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7254a95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae54a9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xef454a9f, SUC_lexicon_tags_12 },
    { 0x2e254aa0, SUC_lexicon_tags_37 },
    { 0x91454aa1, SUC_lexicon_tags_17 },
    { 0xc7254aa0, SUC_lexicon_tags_25 },
    { 0x9c854aa3, SUC_lexicon_tags_12 },
    { 0x75c54aa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e854aa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3454aac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc54ab2, SUC_lexicon_tags_20 },
    { 0xdc454ab3, SUC_lexicon_tags_0 },
    { 0xc3454ab3, SUC_lexicon_tags_25 },
    { 0x30454ab4, SUC_lexicon_tags_12 },
    { 0xad254ab5, SUC_lexicon_tags_27 },
    { 0x2c054ab7, SUC_lexicon_tags_47 },
    { 0x9c854ab7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3be54aba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7254abc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xfa54abe, SUC_lexicon_tags_12 },
    { 0xd0854abf, SUC_lexicon_tags_48 },
    { 0x19654abf, SUC_lexicon_tags_6 },
    { 0x5aa54ac1, SUC_lexicon_tags_4 },
    { 0xdf654ac1, SUC_lexicon_tags_28 },
    { 0x5d854ac3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e54ac8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x11e54aca, SUC_lexicon_tags_15 },
    { 0x78a54acb, SUC_lexicon_tags_2 },
    { 0x64654acb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c054acf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49654ad1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8254ad6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a54ae2, SUC_lexicon_tags_7 },
    { 0x1b454ae3, SUC_lexicon_tags_7 },
    { 0x7ec54ae2, SUC_lexicon_tags_19 },
    { 0x32854ae3, SUC_lexicon_tags_45 },
    { 0xb2e54ae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7a54ae8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b654aea, SUC_lexicon_tags_25 },
    { 0x89654aea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe6854aed, SUC_lexicon_tags_25 },
    { 0xaf854aee, SUC_lexicon_tags_4 },
    { 0x25454aef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25454af2, SUC_lexicon_tags_21 },
    { 0x30254af3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c54af7, SUC_lexicon_tags_12 },
    { 0x55c54af8, SUC_lexicon_tags_4 },
    { 0xb5254af9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c54afc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x90854afe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaf054b00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9654b02, SUC_lexicon_tags_9 },
    { 0xbf454b02, SUC_lexicon_tags_12 },
    { 0xbc254b04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4254b06, SUC_lexicon_tags_10 },
    { 0x82654b07, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c54b0d, SUC_lexicon_tags_11 },
    { 0xaa054b0e, SUC_lexicon_tags_15 },
    { 0xd0a54b0f, SUC_lexicon_tags_11 },
    { 0x6e254b10, SUC_lexicon_tags_12 },
    { 0xe8054b11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f054b13, SUC_lexicon_tags_26 },
    { 0xd1c54b14, SUC_lexicon_tags_7 },
    { 0x65c54b15, SUC_lexicon_tags_17 },
    { 0x6d054b16, SUC_lexicon_tags_25 },
    { 0xae054b17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3254b20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc54b25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a54b2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92654b2c, SUC_lexicon_tags_11 },
    { 0x3da54b2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53454b31, SUC_lexicon_tags_65 },
    { 0x89a54b32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf254b35, SUC_lexicon_tags_46 },
    { 0x25054b35, SUC_lexicon_tags_25 },
    { 0x4b854b37, SUC_lexicon_tags_37 },
    { 0x77454b37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35254b3a, SUC_lexicon_tags_68 },
    { 0xbe454b3b, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c54b3e, SUC_lexicon_tags_37 },
    { 0xbe854b3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e54b44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec54b48, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x80c54b4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a254b4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a54b51, SUC_lexicon_tags_10 },
    { 0x8d654b52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e54b58, SUC_lexicon_tags_17 },
    { 0x96054b58, SUC_lexicon_tags_2 },
    { 0xa8a54b58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2c54b5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb8a54b5e, SUC_lexicon_tags_84 },
    { 0x32254b5f, SUC_lexicon_tags_47 },
    { 0xd9454b60, SUC_lexicon_tags_17 },
    { 0x51e54b60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c54b64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f054b66, SUC_lexicon_tags_689 },
    { 0xe5254b66, SUC_lexicon_tags_23 },
    { 0x7f254b66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc654b6b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x85454b6e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf7254b70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e54b73, SUC_lexicon_tags_2 },
    { 0xa9454b74, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c54b79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34c54b7b, SUC_lexicon_tags_9 },
    { 0x69c54b7b, SUC_lexicon_tags_349 },
    { 0x77654b7d, SUC_lexicon_tags_60 },
    { 0x46254b7e, SUC_lexicon_tags_17 },
    { 0x42654b7f, SUC_lexicon_tags_37 },
    { 0x26454b7e, SUC_lexicon_tags_63 },
    { 0x7e54b7f, SUC_lexicon_tags_47 },
    { 0x6ec54b82, SUC_lexicon_tags_19 },
    { 0x15254b83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea254b89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b454b8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3654b8e, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x79854b90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x37c54b92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac54b96, SUC_lexicon_tags_64 },
    { 0x5b054b97, SUC_lexicon_tags_14 },
    { 0xde054b98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed854b9b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x40e54b9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e54ba0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a54ba4, SUC_lexicon_tags_15 },
    { 0x2f454ba5, SUC_lexicon_tags_81 },
    { 0xac054ba5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e54bb0, SUC_lexicon_tags_37 },
    { 0xf2e54bb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20854bb3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac654bbf, SUC_lexicon_tags_2 },
    { 0xd6854bbf, SUC_lexicon_tags_17 },
    { 0x96654bc1, SUC_lexicon_tags_11 },
    { 0x254bbf, SUC_lexicon_tags_31 },
    { 0x4ae54bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e854bc5, SUC_lexicon_tags_11 },
    { 0x70454bc6, SUC_lexicon_tags_37 },
    { 0xdd254bc7, SUC_lexicon_tags_30 },
    { 0x9e054bc7, SUC_lexicon_tags_47 },
    { 0x73254bc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaa654bcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f054bcf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x28254bd1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x43054bd3, SUC_lexicon_tags_17 },
    { 0xc1254bd3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdb254bd6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6454bdc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf054bde, SUC_lexicon_tags_17 },
    { 0xa9654bdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x96654be2, SUC_lexicon_tags_26 },
    { 0x84654be3, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x90054be5, SUC_lexicon_tags_17 },
    { 0xcc054be6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e54bea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2654bec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf054bee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x80854bf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34454bf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30854bf9, SUC_lexicon_tags_0 },
    { 0xdcc54bfa, SUC_lexicon_tags_25 },
    { 0x47a54bf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58854bfe, SUC_lexicon_tags_49 },
    { 0x8b854bff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc654c06, SUC_lexicon_tags_2 },
    { 0x81e54c07, SUC_lexicon_tags_23 },
    { 0x3d654c07, SUC_lexicon_tags_37 },
    { 0x73454c07, SUC_lexicon_tags_46 },
    { 0x16254c09, SUC_lexicon_tags_2 },
    { 0xde054c06, SUC_lexicon_tags_37 },
    { 0xb9254c08, SUC_lexicon_tags_11 },
    { 0xd2a54c0d, SUC_lexicon_tags_39 },
    { 0x2e254c0e, SUC_lexicon_tags_30 },
    { 0xe2c54c0f, SUC_lexicon_tags_37 },
    { 0xeb254c10, SUC_lexicon_tags_7 },
    { 0x44054c10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49e54c13, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c54c16, SUC_lexicon_tags_216 },
    { 0xa9a54c17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee54c1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7ac54c1f, SUC_lexicon_tags_31 },
    { 0x54e54c20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5c854c22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e54c27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffc54c29, SUC_lexicon_tags_14 },
    { 0x8c054c29, SUC_lexicon_tags_7 },
    { 0xd0454c29, SUC_lexicon_tags_68 },
    { 0x43454c2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7054c2e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee654c37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76054c39, SUC_lexicon_tags_37 },
    { 0x9a54c3a, SUC_lexicon_tags_12 },
    { 0xde254c3b, SUC_lexicon_tags_10 },
    { 0xbb854c3b, SUC_lexicon_tags_12 },
    { 0x55854c3b, SUC_lexicon_tags_37 },
    { 0x17c54c3e, SUC_lexicon_tags_1 },
    { 0xe6c54c3f, SUC_lexicon_tags_17 },
    { 0xdce54c3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c54c43, SUC_lexicon_tags_12 },
    { 0xcf854c43, SUC_lexicon_tags_98 },
    { 0xe4454c45, SUC_lexicon_tags_26 },
    { 0xed854c45, SUC_lexicon_tags_14 },
    { 0x7254c44, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e254c4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6854c4d, SUC_lexicon_tags_8 },
    { 0x6be54c4e, SUC_lexicon_tags_45 },
    { 0xed654c4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94254c54, SUC_lexicon_tags_7 },
    { 0x75654c55, SUC_lexicon_tags_37 },
    { 0xce454c56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede54c5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c54c5f, SUC_lexicon_tags_2 },
    { 0x8e454c5f, SUC_lexicon_tags_85 },
    { 0x74054c61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31c54c63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7c54c65, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcb254c67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37654c6b, SUC_lexicon_tags_24 },
    { 0x33c54c6c, SUC_lexicon_tags_28 },
    { 0x58c54c6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b654c76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36254c78, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c54c7e, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31654c82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a54c89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7054c8d, SUC_lexicon_tags_38 },
    { 0x39e54c8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30454c92, SUC_lexicon_tags_547 },
    { 0, NULL },
    { 0x54254c94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2d454c96, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd454c9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2654c9c, SUC_lexicon_tags_38 },
    { 0xbce54c9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab254ca0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e54ca4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2d854ca6, SUC_lexicon_tags_2 },
    { 0x5c054ca6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c654cac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8254cb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47854cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4e54cbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e54cc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39654cc4, SUC_lexicon_tags_11 },
    { 0x43854cc5, SUC_lexicon_tags_12 },
    { 0x41a54cc5, SUC_lexicon_tags_34 },
    { 0xcf254cc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c454cc9, SUC_lexicon_tags_12 },
    { 0xf9654cc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2054cd1, SUC_lexicon_tags_47 },
    { 0xc1854cd2, SUC_lexicon_tags_1 },
    { 0xc0054cd2, SUC_lexicon_tags_3 },
    { 0x32454cd1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x59254cd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96a54cd9, SUC_lexicon_tags_129 },
    { 0xd7254cda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93054cde, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c54ce2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a54ce8, SUC_lexicon_tags_32 },
    { 0xff254ce9, SUC_lexicon_tags_65 },
    { 0x76054cea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfc54cec, SUC_lexicon_tags_0 },
    { 0xb0e54ced, SUC_lexicon_tags_22 },
    { 0x31454cee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36654cf1, SUC_lexicon_tags_11 },
    { 0xf1654cf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a654cf6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xca054cf9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x58854cfc, SUC_lexicon_tags_8 },
    { 0xcd054cfc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcc054cff, SUC_lexicon_tags_31 },
    { 0x73454d00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xec654d03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f454d06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29254d0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc7854d0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90054d13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a54d16, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17854d1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88054d1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c54d21, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c54d25, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70454d30, SUC_lexicon_tags_27 },
    { 0x97654d31, SUC_lexicon_tags_33 },
    { 0x96654d32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36654d36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c54d3a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x99854d3c, SUC_lexicon_tags_28 },
    { 0x33c54d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e854d40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fc54d42, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe654d4a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3e254d4c, SUC_lexicon_tags_38 },
    { 0xe3e54d4c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe454d52, SUC_lexicon_tags_29 },
    { 0x2ea54d53, SUC_lexicon_tags_16 },
    { 0xf0854d53, SUC_lexicon_tags_66 },
    { 0x10254d55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b854d57, SUC_lexicon_tags_23 },
    { 0x4d454d58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1254d5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11054d5e, SUC_lexicon_tags_1 },
    { 0xa9854d5f, SUC_lexicon_tags_45 },
    { 0xc3454d60, SUC_lexicon_tags_37 },
    { 0xe3654d60, SUC_lexicon_tags_37 },
    { 0xc5e54d5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xedc54d64, SUC_lexicon_tags_38 },
    { 0xd5454d65, SUC_lexicon_tags_12 },
    { 0xcde54d65, SUC_lexicon_tags_23 },
    { 0x6ae54d67, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe854d69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a454d6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b054d6d, SUC_lexicon_tags_7 },
    { 0xd3e54d6e, SUC_lexicon_tags_25 },
    { 0x34254d6f, SUC_lexicon_tags_7 },
    { 0x96454d6e, SUC_lexicon_tags_32 },
    { 0x1e854d71, SUC_lexicon_tags_55 },
    { 0xc6054d72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24a54d74, SUC_lexicon_tags_21 },
    { 0xca254d74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf054d78, SUC_lexicon_tags_33 },
    { 0xa7654d79, SUC_lexicon_tags_15 },
    { 0xf3854d7a, SUC_lexicon_tags_27 },
    { 0x1da54d78, SUC_lexicon_tags_28 },
    { 0x7d254d7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2054d7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde54d82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c54d84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf654d88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52254d8d, SUC_lexicon_tags_38 },
    { 0x2c454d8d, SUC_lexicon_tags_39 },
    { 0xbdc54d8e, SUC_lexicon_tags_37 },
    { 0xb6c54d90, SUC_lexicon_tags_29 },
    { 0x5ea54d90, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae54d95, SUC_lexicon_tags_37 },
    { 0x71a54d96, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x20e54d98, SUC_lexicon_tags_43 },
    { 0xd854d99, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78254d9d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f454da0, SUC_lexicon_tags_26 },
    { 0xdae54da0, SUC_lexicon_tags_17 },
    { 0x3c654da0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e454daa, SUC_lexicon_tags_47 },
    { 0xf5454dab, SUC_lexicon_tags_11 },
    { 0xb9a54daa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x99454daf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17c54db1, SUC_lexicon_tags_15 },
    { 0xde854db2, SUC_lexicon_tags_14 },
    { 0x7e254db1, SUC_lexicon_tags_35 },
    { 0xc6254db4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae54db9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed254dbb, SUC_lexicon_tags_8 },
    { 0xe1c54dbc, SUC_lexicon_tags_3 },
    { 0x40454dbd, SUC_lexicon_tags_0 },
    { 0x50054dbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7e54dc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88854dc2, SUC_lexicon_tags_151 },
    { 0x95c54dc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe254dc5, SUC_lexicon_tags_1 },
    { 0xb6a54dc6, SUC_lexicon_tags_66 },
    { 0x5c254dc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae54dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7454dcd, SUC_lexicon_tags_27 },
    { 0xf1a54dcd, SUC_lexicon_tags_37 },
    { 0xc6254dcd, SUC_lexicon_tags_25 },
    { 0x1c54dd0, SUC_lexicon_tags_0 },
    { 0xf3c54dd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49c54dd3, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xf5254dd5, SUC_lexicon_tags_11 },
    { 0xaa854dd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec54dd9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b454ddc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81254de3, SUC_lexicon_tags_25 },
    { 0xd3c54de4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x57e54de6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea654de8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8654dec, SUC_lexicon_tags_7 },
    { 0x7e654ded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94254df1, SUC_lexicon_tags_11 },
    { 0xdc054df1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdee54df4, SUC_lexicon_tags_17 },
    { 0x88054df4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f654e00, SUC_lexicon_tags_27 },
    { 0x73254e01, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf054e05, SUC_lexicon_tags_11 },
    { 0xf7a54e06, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca54e0c, SUC_lexicon_tags_37 },
    { 0xa9e54e0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfee54e0f, SUC_lexicon_tags_62 },
    { 0xf7854e10, SUC_lexicon_tags_37 },
    { 0xadc54e10, SUC_lexicon_tags_11 },
    { 0x35454e11, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa54e15, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65654e1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c54e20, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac54e2a, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87054e30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c54e38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6654e3a, SUC_lexicon_tags_37 },
    { 0x1e654e3a, SUC_lexicon_tags_11 },
    { 0x6854e3c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a54e40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2454e42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43854e44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x26054e46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46854e4e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa54e53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6054e57, SUC_lexicon_tags_57 },
    { 0x60654e57, SUC_lexicon_tags_49 },
    { 0x59854e57, SUC_lexicon_tags_66 },
    { 0xa5254e5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c854e5f, SUC_lexicon_tags_11 },
    { 0x41454e60, SUC_lexicon_tags_63 },
    { 0x21e54e60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6da54e63, SUC_lexicon_tags_37 },
    { 0x81454e64, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e54e68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34254e6a, SUC_lexicon_tags_33 },
    { 0xc5c54e6b, SUC_lexicon_tags_48 },
    { 0x3a054e6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e54e6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9654e71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3c54e73, SUC_lexicon_tags_2 },
    { 0x61c54e73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1254e77, SUC_lexicon_tags_33 },
    { 0xecc54e77, SUC_lexicon_tags_7 },
    { 0xd2854e77, SUC_lexicon_tags_30 },
    { 0x30054e7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16e54e7c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x55c54e7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4d454e80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2fc54e82, SUC_lexicon_tags_151 },
    { 0x36054e83, SUC_lexicon_tags_25 },
    { 0x18254e82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54e8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb254e93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e54e96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c54e99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54e9d, SUC_lexicon_tags_1 },
    { 0xc7e54e9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9c254ea0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x47454ea2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8454eaf, SUC_lexicon_tags_12 },
    { 0xde54eb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79854eb3, SUC_lexicon_tags_60 },
    { 0x4ee54eb3, SUC_lexicon_tags_37 },
    { 0xd9c54eb5, SUC_lexicon_tags_17 },
    { 0x9e854eb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcac54eb8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc54ec4, SUC_lexicon_tags_8 },
    { 0x6c854ec5, SUC_lexicon_tags_30 },
    { 0x7ee54ec6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb654ecb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c54ece, SUC_lexicon_tags_46 },
    { 0x24654ecf, SUC_lexicon_tags_23 },
    { 0x4dc54ed0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b654ed3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37454edc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f654ee1, SUC_lexicon_tags_15 },
    { 0xa5e54ee2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa5854ee4, SUC_lexicon_tags_50 },
    { 0xc9254ee5, SUC_lexicon_tags_82 },
    { 0x4e654ee6, SUC_lexicon_tags_37 },
    { 0x44654ee7, SUC_lexicon_tags_731 },
    { 0x6ea54ee8, SUC_lexicon_tags_23 },
    { 0x91254ee8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa54eeb, SUC_lexicon_tags_2 },
    { 0x9e254eec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b654ef2, SUC_lexicon_tags_10 },
    { 0xbd054ef2, SUC_lexicon_tags_37 },
    { 0x4be54ef4, SUC_lexicon_tags_23 },
    { 0x77254ef5, SUC_lexicon_tags_12 },
    { 0x98e54ef5, SUC_lexicon_tags_12 },
    { 0x27254ef7, SUC_lexicon_tags_12 },
    { 0x86054ef6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e54efc, SUC_lexicon_tags_17 },
    { 0xe3c54efd, SUC_lexicon_tags_13 },
    { 0x50654efe, SUC_lexicon_tags_37 },
    { 0x30e54efd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac654f05, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9cc54f07, SUC_lexicon_tags_15 },
    { 0x19254f08, SUC_lexicon_tags_37 },
    { 0x1b454f09, SUC_lexicon_tags_12 },
    { 0x28a54f09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9a254f0c, SUC_lexicon_tags_25 },
    { 0xaa254f0d, SUC_lexicon_tags_12 },
    { 0x2c654f0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ac54f10, SUC_lexicon_tags_12 },
    { 0x9ec54f11, SUC_lexicon_tags_173 },
    { 0x18654f11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac54f17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa54f1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23854f1e, SUC_lexicon_tags_11 },
    { 0x96454f1e, SUC_lexicon_tags_12 },
    { 0xa9e54f1f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a54f25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1254f29, SUC_lexicon_tags_37 },
    { 0x80654f2a, SUC_lexicon_tags_25 },
    { 0x93a54f2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65254f2d, SUC_lexicon_tags_47 },
    { 0x6b054f2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f854f31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7054f34, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35254f3a, SUC_lexicon_tags_9 },
    { 0x44054f3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee54f3d, SUC_lexicon_tags_37 },
    { 0x11654f3e, SUC_lexicon_tags_25 },
    { 0x39e54f3f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a54f42, SUC_lexicon_tags_37 },
    { 0x3a854f43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea54f46, SUC_lexicon_tags_28 },
    { 0xffc54f46, SUC_lexicon_tags_17 },
    { 0xbc254f48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61654f4c, SUC_lexicon_tags_21 },
    { 0xf054f4d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28054f55, SUC_lexicon_tags_47 },
    { 0xd4254f56, SUC_lexicon_tags_1 },
    { 0x8d054f56, SUC_lexicon_tags_30 },
    { 0x1b854f58, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5ea54f5a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc54f60, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8d054f62, SUC_lexicon_tags_1 },
    { 0xde054f63, SUC_lexicon_tags_17 },
    { 0xfea54f63, SUC_lexicon_tags_23 },
    { 0x84854f64, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x47654f68, SUC_lexicon_tags_47 },
    { 0x5a054f69, SUC_lexicon_tags_71 },
    { 0x28254f6a, SUC_lexicon_tags_2 },
    { 0x14e54f68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c54f71, SUC_lexicon_tags_63 },
    { 0xee454f71, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x25254f74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef054f76, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3054f79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39054f7b, SUC_lexicon_tags_7 },
    { 0x90054f7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xafc54f7e, SUC_lexicon_tags_23 },
    { 0x93a54f7f, SUC_lexicon_tags_1 },
    { 0x78654f80, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e454f83, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x66c54f85, SUC_lexicon_tags_68 },
    { 0xa4454f85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c054f89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa654f8e, SUC_lexicon_tags_17 },
    { 0x71c54f8f, SUC_lexicon_tags_0 },
    { 0x82c54f8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7a54f92, SUC_lexicon_tags_7 },
    { 0xe6e54f92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51654f96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d854f9b, SUC_lexicon_tags_17 },
    { 0x5ce54f9b, SUC_lexicon_tags_17 },
    { 0xae654f9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2054fa1, SUC_lexicon_tags_7 },
    { 0x94854fa2, SUC_lexicon_tags_12 },
    { 0x84e54fa3, SUC_lexicon_tags_11 },
    { 0x83454fa4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4254fb2, SUC_lexicon_tags_15 },
    { 0x55854fb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c54fb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe654fb8, SUC_lexicon_tags_1 },
    { 0x84a54fb9, SUC_lexicon_tags_39 },
    { 0x50054fb9, SUC_lexicon_tags_65 },
    { 0xb6254fbb, SUC_lexicon_tags_45 },
    { 0xd5454fbb, SUC_lexicon_tags_14 },
    { 0x454fbd, SUC_lexicon_tags_14 },
    { 0x1be54fbd, SUC_lexicon_tags_26 },
    { 0x8654fbe, SUC_lexicon_tags_14 },
    { 0x2454fbd, SUC_lexicon_tags_30 },
    { 0x1a054fbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5854fc5, SUC_lexicon_tags_21 },
    { 0x95054fc5, SUC_lexicon_tags_25 },
    { 0x86a54fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e254fc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8854fcc, SUC_lexicon_tags_37 },
    { 0xa2054fcd, SUC_lexicon_tags_17 },
    { 0xa5a54fcd, SUC_lexicon_tags_2 },
    { 0xf054fcd, SUC_lexicon_tags_4 },
    { 0xea254fd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84c54fd2, SUC_lexicon_tags_47 },
    { 0xdba54fd2, SUC_lexicon_tags_0 },
    { 0x2ae54fd4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3fa54fd6, SUC_lexicon_tags_0 },
    { 0xd0454fd6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40854fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xede54fdd, SUC_lexicon_tags_11 },
    { 0x6ca54fdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce54fe5, SUC_lexicon_tags_27 },
    { 0xeca54fe6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5254fe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1254feb, SUC_lexicon_tags_19 },
    { 0xa3654fec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d454ff2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98654ff8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce254ffc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5655000, SUC_lexicon_tags_25 },
    { 0xe4c55001, SUC_lexicon_tags_5 },
    { 0x68255002, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15855004, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc55007, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e85500b, SUC_lexicon_tags_37 },
    { 0xd1c5500b, SUC_lexicon_tags_42 },
    { 0xefc5500d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c5500f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97855012, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xffc55014, SUC_lexicon_tags_21 },
    { 0x8b255015, SUC_lexicon_tags_33 },
    { 0xdb255015, SUC_lexicon_tags_41 },
    { 0x35c55017, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb045501b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9885501e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa7c55020, SUC_lexicon_tags_1 },
    { 0xf5c55021, SUC_lexicon_tags_0 },
    { 0x6ec55021, SUC_lexicon_tags_14 },
    { 0x4ae55023, SUC_lexicon_tags_23 },
    { 0x4e55023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b255027, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1225502f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8255032, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24255035, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f255037, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e5503b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x75e5503d, SUC_lexicon_tags_27 },
    { 0x505503d, SUC_lexicon_tags_7 },
    { 0xae85503f, SUC_lexicon_tags_12 },
    { 0x3a255040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e55044, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a45504c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c55052, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5455054, SUC_lexicon_tags_1 },
    { 0x19855055, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e455057, SUC_lexicon_tags_12 },
    { 0x75655058, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c25505c, SUC_lexicon_tags_15 },
    { 0xbe05505d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26055063, SUC_lexicon_tags_14 },
    { 0x8b655064, SUC_lexicon_tags_11 },
    { 0xdee55065, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba55069, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa5506c, SUC_lexicon_tags_15 },
    { 0x7be5506d, SUC_lexicon_tags_9 },
    { 0xb6a5506d, SUC_lexicon_tags_26 },
    { 0xa905506f, SUC_lexicon_tags_6 },
    { 0x91e5506f, SUC_lexicon_tags_25 },
    { 0x5b85506f, SUC_lexicon_tags_6 },
    { 0xbc055072, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8055079, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f85507f, SUC_lexicon_tags_7 },
    { 0x46455080, SUC_lexicon_tags_25 },
    { 0xbb255081, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c455084, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32455088, SUC_lexicon_tags_63 },
    { 0x7ac55088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7945508d, SUC_lexicon_tags_26 },
    { 0x93e5508e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e855091, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7655094, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24855096, SUC_lexicon_tags_15 },
    { 0xa5655097, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc485509a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a5509e, SUC_lexicon_tags_15 },
    { 0xaa85509f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a550a4, SUC_lexicon_tags_28 },
    { 0x186550a4, SUC_lexicon_tags_8 },
    { 0xbac550a6, SUC_lexicon_tags_14 },
    { 0x1d4550a7, SUC_lexicon_tags_77 },
    { 0xdd6550a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x238550ab, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x76c550ad, SUC_lexicon_tags_37 },
    { 0x62550ad, SUC_lexicon_tags_17 },
    { 0xdd0550af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa550b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e550b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c550ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x30c550bc, SUC_lexicon_tags_7 },
    { 0x468550bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac550c2, SUC_lexicon_tags_75 },
    { 0x9ac550c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98550cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x130550ce, SUC_lexicon_tags_35 },
    { 0xe4c550cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4550d4, SUC_lexicon_tags_12 },
    { 0xce8550d4, SUC_lexicon_tags_14 },
    { 0x992550d4, SUC_lexicon_tags_23 },
    { 0xf04550d7, SUC_lexicon_tags_11 },
    { 0x5f4550d8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x1d6550da, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7da550dc, SUC_lexicon_tags_16 },
    { 0xf32550dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x700550df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x560550e1, SUC_lexicon_tags_25 },
    { 0xf4c550e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704550e7, SUC_lexicon_tags_28 },
    { 0x8fa550e8, SUC_lexicon_tags_100 },
    { 0x370550e7, SUC_lexicon_tags_1 },
    { 0x626550ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x312550ec, SUC_lexicon_tags_30 },
    { 0x540550ed, SUC_lexicon_tags_27 },
    { 0x50e550ee, SUC_lexicon_tags_1375 },
    { 0x462550ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c6550f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396550f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8550fb, SUC_lexicon_tags_15 },
    { 0x620550fc, SUC_lexicon_tags_19 },
    { 0x2c8550fb, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68855102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec255105, SUC_lexicon_tags_171 },
    { 0x58e55106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f25510a, SUC_lexicon_tags_25 },
    { 0x44c5510b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5510f, SUC_lexicon_tags_80 },
    { 0xfc85510f, SUC_lexicon_tags_39 },
    { 0xa1c55111, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec55116, SUC_lexicon_tags_37 },
    { 0x8655116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2725511a, SUC_lexicon_tags_11 },
    { 0x7bc5511b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49055124, SUC_lexicon_tags_42 },
    { 0xacc55124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3455128, SUC_lexicon_tags_26 },
    { 0x8b655128, SUC_lexicon_tags_17 },
    { 0xf3455128, SUC_lexicon_tags_19 },
    { 0xde255129, SUC_lexicon_tags_1 },
    { 0x4425512b, SUC_lexicon_tags_45 },
    { 0xe625512d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbc5512f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87a55131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a55135, SUC_lexicon_tags_49 },
    { 0xfdc55136, SUC_lexicon_tags_0 },
    { 0x34855135, SUC_lexicon_tags_68 },
    { 0xb2455138, SUC_lexicon_tags_33 },
    { 0xa0e55137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32e5513b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a5513e, SUC_lexicon_tags_17 },
    { 0x6ce5513f, SUC_lexicon_tags_47 },
    { 0x67255140, SUC_lexicon_tags_0 },
    { 0xfa055140, SUC_lexicon_tags_17 },
    { 0x9ce5513f, SUC_lexicon_tags_2 },
    { 0xa8a55141, SUC_lexicon_tags_8 },
    { 0xc6455144, SUC_lexicon_tags_2 },
    { 0xc0e55142, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a55148, SUC_lexicon_tags_27 },
    { 0x4c455148, SUC_lexicon_tags_35 },
    { 0x33e55148, SUC_lexicon_tags_17 },
    { 0x6dc55149, SUC_lexicon_tags_45 },
    { 0x63a55148, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3055152, SUC_lexicon_tags_45 },
    { 0x69055152, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd85515d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee55169, SUC_lexicon_tags_0 },
    { 0x5f65516a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d05516d, SUC_lexicon_tags_25 },
    { 0xb245516d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc55175, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5855179, SUC_lexicon_tags_29 },
    { 0x55a5517a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1025517c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa265517e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31255184, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3855187, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5045518d, SUC_lexicon_tags_64 },
    { 0x62e5518e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e55191, SUC_lexicon_tags_46 },
    { 0x64455192, SUC_lexicon_tags_12 },
    { 0xa9e55192, SUC_lexicon_tags_126 },
    { 0xb4255194, SUC_lexicon_tags_56 },
    { 0x7d455195, SUC_lexicon_tags_37 },
    { 0xaaa55192, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c5519a, SUC_lexicon_tags_8 },
    { 0x9985519b, SUC_lexicon_tags_15 },
    { 0xeae5519c, SUC_lexicon_tags_193 },
    { 0x91e5519d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c85519f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8551a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112551a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14551ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82551b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e551b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28551b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e551ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x850551bc, SUC_lexicon_tags_14 },
    { 0xa8551bd, SUC_lexicon_tags_49 },
    { 0x64551be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3a2551c0, SUC_lexicon_tags_23 },
    { 0x13e551c1, SUC_lexicon_tags_109 },
    { 0x82551c1, SUC_lexicon_tags_29 },
    { 0xb0c551c2, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x382551c6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e551ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x60551cd, SUC_lexicon_tags_29 },
    { 0xf0c551ce, SUC_lexicon_tags_60 },
    { 0x39c551ce, SUC_lexicon_tags_2 },
    { 0xda2551ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x952551d2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x76c551d4, SUC_lexicon_tags_27 },
    { 0x23e551d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752551da, SUC_lexicon_tags_65 },
    { 0xbb6551db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe2e551dd, SUC_lexicon_tags_15 },
    { 0xacc551de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ae551e0, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x2f6551e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412551e6, SUC_lexicon_tags_15 },
    { 0xde8551e7, SUC_lexicon_tags_68 },
    { 0x7fa551e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656551ef, SUC_lexicon_tags_45 },
    { 0xb0551f0, SUC_lexicon_tags_48 },
    { 0x590551f1, SUC_lexicon_tags_37 },
    { 0xc7c551f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa68551f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99e551f6, SUC_lexicon_tags_2 },
    { 0xf0a551f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x356551f9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96551fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec55200, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda55205, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12055208, SUC_lexicon_tags_15 },
    { 0x3ce55209, SUC_lexicon_tags_6 },
    { 0x79a55208, SUC_lexicon_tags_6 },
    { 0x5465520b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9925520d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a55211, SUC_lexicon_tags_25 },
    { 0xf9655212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15655215, SUC_lexicon_tags_8 },
    { 0xf0255215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e5521c, SUC_lexicon_tags_84 },
    { 0x54e5521d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8965521f, SUC_lexicon_tags_87 },
    { 0x3d45521f, SUC_lexicon_tags_60 },
    { 0x75255221, SUC_lexicon_tags_43 },
    { 0x31255222, SUC_lexicon_tags_15 },
    { 0x1d655223, SUC_lexicon_tags_15 },
    { 0xcc255222, SUC_lexicon_tags_1 },
    { 0xba055220, SUC_lexicon_tags_74 },
    { 0x61855225, SUC_lexicon_tags_17 },
    { 0x5fe55225, SUC_lexicon_tags_47 },
    { 0xc3255228, SUC_lexicon_tags_11 },
    { 0x4da55224, SUC_lexicon_tags_1 },
    { 0xdc655227, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc055230, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3655234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a855236, SUC_lexicon_tags_33 },
    { 0xdee55236, SUC_lexicon_tags_30 },
    { 0xeba55238, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc525523e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb855240, SUC_lexicon_tags_23 },
    { 0x94255240, SUC_lexicon_tags_32 },
    { 0xdac55240, SUC_lexicon_tags_17 },
    { 0x27255243, SUC_lexicon_tags_27 },
    { 0x67c55240, SUC_lexicon_tags_45 },
    { 0x5c855245, SUC_lexicon_tags_7 },
    { 0x12255245, SUC_lexicon_tags_12 },
    { 0xae655247, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf65524b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd45524d, SUC_lexicon_tags_17 },
    { 0xb985524e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6bc55250, SUC_lexicon_tags_2 },
    { 0x40655250, SUC_lexicon_tags_2 },
    { 0x5e855251, SUC_lexicon_tags_27 },
    { 0xd4655252, SUC_lexicon_tags_25 },
    { 0xf6c55254, SUC_lexicon_tags_15 },
    { 0x7bc55254, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e055257, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8565525a, SUC_lexicon_tags_47 },
    { 0xac85525b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c55261, SUC_lexicon_tags_29 },
    { 0x3e255262, SUC_lexicon_tags_23 },
    { 0x5a455263, SUC_lexicon_tags_13 },
    { 0x7ac55262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0655267, SUC_lexicon_tags_47 },
    { 0xf2e55267, SUC_lexicon_tags_31 },
    { 0xf5255268, SUC_lexicon_tags_7 },
    { 0x89e55268, SUC_lexicon_tags_8 },
    { 0x4fa55269, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x89a5526d, SUC_lexicon_tags_14 },
    { 0x2c45526d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0655270, SUC_lexicon_tags_48 },
    { 0x8b055271, SUC_lexicon_tags_0 },
    { 0x65655272, SUC_lexicon_tags_74 },
    { 0x7c655273, SUC_lexicon_tags_27 },
    { 0x62655270, SUC_lexicon_tags_2 },
    { 0x41655274, SUC_lexicon_tags_19 },
    { 0x4e055276, SUC_lexicon_tags_25 },
    { 0xf4855277, SUC_lexicon_tags_33 },
    { 0x8f655277, SUC_lexicon_tags_7 },
    { 0x5c655278, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc1c5527b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb545527e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a55283, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e455286, SUC_lexicon_tags_17 },
    { 0x87055286, SUC_lexicon_tags_37 },
    { 0xd7e55288, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1325528b, SUC_lexicon_tags_0 },
    { 0x99a5528c, SUC_lexicon_tags_4 },
    { 0xc025528d, SUC_lexicon_tags_235 },
    { 0x1ec5528c, SUC_lexicon_tags_2 },
    { 0x7f25528c, SUC_lexicon_tags_11 },
    { 0x21e5528b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xea455293, SUC_lexicon_tags_11 },
    { 0x29855294, SUC_lexicon_tags_49 },
    { 0xcce55294, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeea55297, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44055299, SUC_lexicon_tags_38 },
    { 0x61e5529a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d05529d, SUC_lexicon_tags_26 },
    { 0x9405529d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32552a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa02552a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae552af, SUC_lexicon_tags_55 },
    { 0x608552af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962552b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a552bd, SUC_lexicon_tags_43 },
    { 0xb74552be, SUC_lexicon_tags_60 },
    { 0x6c2552bf, SUC_lexicon_tags_15 },
    { 0x2f4552c0, SUC_lexicon_tags_74 },
    { 0xf60552bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60552c7, SUC_lexicon_tags_7 },
    { 0xe48552c8, SUC_lexicon_tags_11 },
    { 0xd66552c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6552cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa552ce, SUC_lexicon_tags_65 },
    { 0xe70552ce, SUC_lexicon_tags_11 },
    { 0x126552d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a552d3, SUC_lexicon_tags_7 },
    { 0x2c0552d3, SUC_lexicon_tags_43 },
    { 0x832552d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d2552d7, SUC_lexicon_tags_25 },
    { 0x10e552d8, SUC_lexicon_tags_11 },
    { 0xa64552d8, SUC_lexicon_tags_48 },
    { 0x964552da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e552dc, SUC_lexicon_tags_25 },
    { 0xdba552dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43a552df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6552eb, SUC_lexicon_tags_17 },
    { 0x7ac552eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e552f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x356552f4, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a552f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d8552fa, SUC_lexicon_tags_60 },
    { 0xf3c552fb, SUC_lexicon_tags_68 },
    { 0x204552fc, SUC_lexicon_tags_33 },
    { 0xffa552fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x910552ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xff255302, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e655305, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x63455307, SUC_lexicon_tags_68 },
    { 0x48455307, SUC_lexicon_tags_49 },
    { 0x2ca55309, SUC_lexicon_tags_26 },
    { 0xec05530a, SUC_lexicon_tags_25 },
    { 0xca055309, SUC_lexicon_tags_5 },
    { 0x3125530c, SUC_lexicon_tags_2 },
    { 0x8665530d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3855313, SUC_lexicon_tags_30 },
    { 0xb9055313, SUC_lexicon_tags_60 },
    { 0x4a255315, SUC_lexicon_tags_25 },
    { 0xa1e55313, SUC_lexicon_tags_33 },
    { 0x91c55316, SUC_lexicon_tags_37 },
    { 0xd3655318, SUC_lexicon_tags_68 },
    { 0xf8e55318, SUC_lexicon_tags_38 },
    { 0x4a455313, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2465531d, SUC_lexicon_tags_29 },
    { 0x9c5531e, SUC_lexicon_tags_14 },
    { 0xcda5531f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc55325, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c55328, SUC_lexicon_tags_28 },
    { 0xa9c55329, SUC_lexicon_tags_14 },
    { 0xa3a5532a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3025532d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d45532f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a655333, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9255339, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc685533b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e5533e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xddc55340, SUC_lexicon_tags_12 },
    { 0x5a055340, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc055346, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8255349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae455351, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3055359, SUC_lexicon_tags_13 },
    { 0x57e5535a, SUC_lexicon_tags_63 },
    { 0x1065535b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b45535e, SUC_lexicon_tags_3 },
    { 0x8125535f, SUC_lexicon_tags_7 },
    { 0x4655360, SUC_lexicon_tags_17 },
    { 0xcec5535f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47855363, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xba55365, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9345536a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbc5536c, SUC_lexicon_tags_15 },
    { 0x4825536d, SUC_lexicon_tags_31 },
    { 0x34a5536e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e455373, SUC_lexicon_tags_15 },
    { 0x9855374, SUC_lexicon_tags_23 },
    { 0x2b855374, SUC_lexicon_tags_37 },
    { 0x8be55376, SUC_lexicon_tags_11 },
    { 0x3e255374, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad65537b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ce5537d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc255380, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x64e55382, SUC_lexicon_tags_11 },
    { 0xce255383, SUC_lexicon_tags_25 },
    { 0xb6855384, SUC_lexicon_tags_37 },
    { 0x5a455385, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa6e55387, SUC_lexicon_tags_7 },
    { 0x87a55388, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f055393, SUC_lexicon_tags_1 },
    { 0x48855394, SUC_lexicon_tags_12 },
    { 0xc1255393, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e255399, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde553a2, SUC_lexicon_tags_25 },
    { 0x76a553a3, SUC_lexicon_tags_29 },
    { 0xe50553a4, SUC_lexicon_tags_2 },
    { 0xda0553a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30553ae, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xba0553b0, SUC_lexicon_tags_87 },
    { 0x6fc553b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844553b5, SUC_lexicon_tags_45 },
    { 0xd22553b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612553ba, SUC_lexicon_tags_46 },
    { 0x55c553ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x584553bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a553c6, SUC_lexicon_tags_63 },
    { 0xf0a553c7, SUC_lexicon_tags_12 },
    { 0xcf4553c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc7e553ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa6a553cc, SUC_lexicon_tags_25 },
    { 0x2d8553cd, SUC_lexicon_tags_26 },
    { 0x1b6553cc, SUC_lexicon_tags_62 },
    { 0x5d8553cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe553d2, SUC_lexicon_tags_17 },
    { 0xa00553d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8553d7, SUC_lexicon_tags_613 },
    { 0x566553d7, SUC_lexicon_tags_45 },
    { 0x710553d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810553dd, SUC_lexicon_tags_15 },
    { 0xc6e553de, SUC_lexicon_tags_31 },
    { 0x5d4553de, SUC_lexicon_tags_33 },
    { 0xca6553e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89a553e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0553e6, SUC_lexicon_tags_12 },
    { 0x3d8553e7, SUC_lexicon_tags_25 },
    { 0xcbe553e8, SUC_lexicon_tags_34 },
    { 0x93c553e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce553ed, SUC_lexicon_tags_66 },
    { 0x5c0553ee, SUC_lexicon_tags_17 },
    { 0x7e6553ef, SUC_lexicon_tags_2 },
    { 0x140553ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb84553f2, SUC_lexicon_tags_7 },
    { 0x9e4553f2, SUC_lexicon_tags_7 },
    { 0x164553f4, SUC_lexicon_tags_60 },
    { 0xef0553f2, SUC_lexicon_tags_10 },
    { 0xa8a553f6, SUC_lexicon_tags_32 },
    { 0xc9e553f6, SUC_lexicon_tags_17 },
    { 0x982553f6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684553fc, SUC_lexicon_tags_25 },
    { 0xe7c553fd, SUC_lexicon_tags_7 },
    { 0x1c4553fe, SUC_lexicon_tags_14 },
    { 0x64e553ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8d455401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d655405, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde45540a, SUC_lexicon_tags_14 },
    { 0xd805540a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a5540e, SUC_lexicon_tags_29 },
    { 0xf605540f, SUC_lexicon_tags_60 },
    { 0x5c855410, SUC_lexicon_tags_4 },
    { 0xfca5540f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e55414, SUC_lexicon_tags_48 },
    { 0x3bc55414, SUC_lexicon_tags_7 },
    { 0x85655416, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc455418, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99655420, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee55426, SUC_lexicon_tags_15 },
    { 0x83255427, SUC_lexicon_tags_26 },
    { 0x4ae55427, SUC_lexicon_tags_25 },
    { 0x80a55429, SUC_lexicon_tags_16 },
    { 0x65542a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8185542c, SUC_lexicon_tags_47 },
    { 0xbfa5542d, SUC_lexicon_tags_1 },
    { 0x7dc5542d, SUC_lexicon_tags_30 },
    { 0x9f45542f, SUC_lexicon_tags_19 },
    { 0x72c5542f, SUC_lexicon_tags_12 },
    { 0xdf855431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37255434, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f855437, SUC_lexicon_tags_23 },
    { 0x70255437, SUC_lexicon_tags_12 },
    { 0x2055439, SUC_lexicon_tags_15 },
    { 0x62a55439, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb085543f, SUC_lexicon_tags_38 },
    { 0x88e55440, SUC_lexicon_tags_25 },
    { 0xa7a55441, SUC_lexicon_tags_46 },
    { 0x1b855442, SUC_lexicon_tags_8 },
    { 0xcdc55442, SUC_lexicon_tags_14 },
    { 0x15a5543f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a55449, SUC_lexicon_tags_12 },
    { 0x6665544a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4705544d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x585544f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x28655452, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa255456, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f25545a, SUC_lexicon_tags_37 },
    { 0xe185545b, SUC_lexicon_tags_60 },
    { 0x4685545a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56055460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e55463, SUC_lexicon_tags_17 },
    { 0x25455464, SUC_lexicon_tags_15 },
    { 0xd055465, SUC_lexicon_tags_37 },
    { 0x30055463, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac655470, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd655473, SUC_lexicon_tags_11 },
    { 0x50c55473, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2bc55476, SUC_lexicon_tags_17 },
    { 0xbc255477, SUC_lexicon_tags_23 },
    { 0x42e55477, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4a5547a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd2c5547c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8425547e, SUC_lexicon_tags_1245 },
    { 0x9605547f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c55482, SUC_lexicon_tags_15 },
    { 0xe055482, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x72c55485, SUC_lexicon_tags_38 },
    { 0xe3055486, SUC_lexicon_tags_31 },
    { 0x60255487, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x60855489, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c5548e, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a55492, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7e55494, SUC_lexicon_tags_235 },
    { 0xe7855495, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6be55497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8805549c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaf45549e, SUC_lexicon_tags_8 },
    { 0x75a5549f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf36554a1, SUC_lexicon_tags_15 },
    { 0x82e554a1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x236554a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x242554a7, SUC_lexicon_tags_8 },
    { 0x8ae554a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf88554aa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc0554ac, SUC_lexicon_tags_15 },
    { 0x112554ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1e554af, SUC_lexicon_tags_47 },
    { 0xc66554b0, SUC_lexicon_tags_2 },
    { 0x9ae554af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4c554b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc554b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa554bf, SUC_lexicon_tags_37 },
    { 0x8c2554c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x84554c2, SUC_lexicon_tags_17 },
    { 0x928554c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae554c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4554cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e554d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ae554d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96554db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4554df, SUC_lexicon_tags_77 },
    { 0x63c554e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614554e5, SUC_lexicon_tags_25 },
    { 0x970554e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x938554e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e554eb, SUC_lexicon_tags_25 },
    { 0xdc0554eb, SUC_lexicon_tags_32 },
    { 0x2cc554ed, SUC_lexicon_tags_37 },
    { 0x70a554ed, SUC_lexicon_tags_12 },
    { 0x83e554ed, SUC_lexicon_tags_47 },
    { 0x18c554f0, SUC_lexicon_tags_7 },
    { 0x820554f0, SUC_lexicon_tags_9 },
    { 0xdf6554f2, SUC_lexicon_tags_11 },
    { 0x18a554f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x782554f6, SUC_lexicon_tags_37 },
    { 0xe16554f7, SUC_lexicon_tags_73 },
    { 0xd9a554f7, SUC_lexicon_tags_6 },
    { 0x70554f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a554fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc55502, SUC_lexicon_tags_65 },
    { 0x40255503, SUC_lexicon_tags_26 },
    { 0xd3e55503, SUC_lexicon_tags_39 },
    { 0x7f055505, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd5c55507, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d45550c, SUC_lexicon_tags_15 },
    { 0x9765550c, SUC_lexicon_tags_39 },
    { 0x6dc5550d, SUC_lexicon_tags_45 },
    { 0xe7a5550f, SUC_lexicon_tags_37 },
    { 0x1825550e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18455515, SUC_lexicon_tags_63 },
    { 0x18655516, SUC_lexicon_tags_33 },
    { 0xf8055516, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4285551a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6525551f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca655521, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a25552a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92a5552c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b85552f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe55536, SUC_lexicon_tags_3 },
    { 0x11855536, SUC_lexicon_tags_14 },
    { 0x48455536, SUC_lexicon_tags_14 },
    { 0x1ae55538, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2505553b, SUC_lexicon_tags_87 },
    { 0x2d25553b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb25553f, SUC_lexicon_tags_21 },
    { 0xd0e5553f, SUC_lexicon_tags_55 },
    { 0x3fc55541, SUC_lexicon_tags_13 },
    { 0x2255541, SUC_lexicon_tags_7 },
    { 0xa7e55543, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34855547, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e5554b, SUC_lexicon_tags_12 },
    { 0x3d45554b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96455551, SUC_lexicon_tags_12 },
    { 0x98c55551, SUC_lexicon_tags_12 },
    { 0x99055553, SUC_lexicon_tags_17 },
    { 0x52855554, SUC_lexicon_tags_26 },
    { 0x95855553, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a85555d, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x9905555f, SUC_lexicon_tags_4 },
    { 0xf9655560, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdb255562, SUC_lexicon_tags_27 },
    { 0x74655562, SUC_lexicon_tags_7 },
    { 0xed055564, SUC_lexicon_tags_29 },
    { 0x87055565, SUC_lexicon_tags_23 },
    { 0x6ca55566, SUC_lexicon_tags_25 },
    { 0x5fc55563, SUC_lexicon_tags_33 },
    { 0x8455568, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd6c5556a, SUC_lexicon_tags_17 },
    { 0x3945556b, SUC_lexicon_tags_17 },
    { 0xeea5556c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1345556e, SUC_lexicon_tags_25 },
    { 0x46a5556e, SUC_lexicon_tags_11 },
    { 0x65855570, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a55575, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec5557a, SUC_lexicon_tags_95 },
    { 0x6545557b, SUC_lexicon_tags_11 },
    { 0x3825557c, SUC_lexicon_tags_49 },
    { 0xc905557a, SUC_lexicon_tags_7 },
    { 0x1765557e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc655581, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37655587, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ee55589, SUC_lexicon_tags_47 },
    { 0x71a5558a, SUC_lexicon_tags_63 },
    { 0x8a05558a, SUC_lexicon_tags_63 },
    { 0x28a5558c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e5558f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74855591, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x45e55593, SUC_lexicon_tags_25 },
    { 0xd5455594, SUC_lexicon_tags_69 },
    { 0xa0e55594, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5455597, SUC_lexicon_tags_37 },
    { 0x59455598, SUC_lexicon_tags_17 },
    { 0x78e55599, SUC_lexicon_tags_14 },
    { 0xace5559a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9fe5559c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa805559e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0555a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x414555a7, SUC_lexicon_tags_7 },
    { 0xb8555a8, SUC_lexicon_tags_49 },
    { 0xa8c555a9, SUC_lexicon_tags_27 },
    { 0x356555aa, SUC_lexicon_tags_25 },
    { 0xd56555a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24555af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e555b3, SUC_lexicon_tags_48 },
    { 0x900555b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e555ba, SUC_lexicon_tags_3 },
    { 0xade555bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea555bf, SUC_lexicon_tags_60 },
    { 0xda8555bf, SUC_lexicon_tags_11 },
    { 0xee4555c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a555c6, SUC_lexicon_tags_45 },
    { 0x728555c7, SUC_lexicon_tags_19 },
    { 0x454555c7, SUC_lexicon_tags_55 },
    { 0xd1c555c9, SUC_lexicon_tags_69 },
    { 0x466555ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a555cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102555d1, SUC_lexicon_tags_11 },
    { 0x2c555d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd40555d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08555da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x838555dc, SUC_lexicon_tags_17 },
    { 0x526555dc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa7c555df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a555e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38555e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4555ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40555ef, SUC_lexicon_tags_11 },
    { 0xce0555ef, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x812555f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502555f7, SUC_lexicon_tags_47 },
    { 0x6a4555f8, SUC_lexicon_tags_25 },
    { 0x988555f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae055601, SUC_lexicon_tags_11 },
    { 0x8f255601, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc7255604, SUC_lexicon_tags_47 },
    { 0xae455605, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc1055607, SUC_lexicon_tags_7 },
    { 0x35055607, SUC_lexicon_tags_37 },
    { 0xc9255608, SUC_lexicon_tags_1 },
    { 0x2065560a, SUC_lexicon_tags_28 },
    { 0xdc85560b, SUC_lexicon_tags_25 },
    { 0x9b455607, SUC_lexicon_tags_11 },
    { 0xefe5560d, SUC_lexicon_tags_8 },
    { 0x7645560d, SUC_lexicon_tags_63 },
    { 0x6145560a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c55613, SUC_lexicon_tags_23 },
    { 0x7fe55614, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfe655616, SUC_lexicon_tags_37 },
    { 0xcce55617, SUC_lexicon_tags_66 },
    { 0xff855616, SUC_lexicon_tags_33 },
    { 0x45855619, SUC_lexicon_tags_11 },
    { 0xeda5561a, SUC_lexicon_tags_14 },
    { 0x6da55619, SUC_lexicon_tags_17 },
    { 0xce05561c, SUC_lexicon_tags_28 },
    { 0xe4255619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc55625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeae55627, SUC_lexicon_tags_7 },
    { 0x8ce55628, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb45562d, SUC_lexicon_tags_12 },
    { 0x7825562d, SUC_lexicon_tags_12 },
    { 0xc1e5562f, SUC_lexicon_tags_17 },
    { 0x89255630, SUC_lexicon_tags_7 },
    { 0xd455631, SUC_lexicon_tags_43 },
    { 0x54c55632, SUC_lexicon_tags_19 },
    { 0x6b05562e, SUC_lexicon_tags_9 },
    { 0x70855634, SUC_lexicon_tags_12 },
    { 0xe855634, SUC_lexicon_tags_11 },
    { 0xb3255632, SUC_lexicon_tags_7 },
    { 0xfa655636, SUC_lexicon_tags_11 },
    { 0x79055638, SUC_lexicon_tags_92 },
    { 0x7d655635, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8065563c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a5563f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7855642, SUC_lexicon_tags_2 },
    { 0x40855642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e05564c, SUC_lexicon_tags_25 },
    { 0xa6c5564d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e55651, SUC_lexicon_tags_2 },
    { 0x5f055652, SUC_lexicon_tags_38 },
    { 0x26a55651, SUC_lexicon_tags_31 },
    { 0x26c55653, SUC_lexicon_tags_19 },
    { 0xa3e55655, SUC_lexicon_tags_7 },
    { 0x67a55653, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c85565e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c55661, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1255665, SUC_lexicon_tags_39 },
    { 0x13c55666, SUC_lexicon_tags_11 },
    { 0x9e655666, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a55671, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0855673, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a55676, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81255679, SUC_lexicon_tags_29 },
    { 0x5ca5567a, SUC_lexicon_tags_65 },
    { 0x2a85567b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e5567e, SUC_lexicon_tags_48 },
    { 0xdd25567e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce455683, SUC_lexicon_tags_11 },
    { 0x9e255684, SUC_lexicon_tags_31 },
    { 0xcbc55683, SUC_lexicon_tags_34 },
    { 0xeca55684, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e5568e, SUC_lexicon_tags_11 },
    { 0x50e5568f, SUC_lexicon_tags_12 },
    { 0x4dc55690, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x28055692, SUC_lexicon_tags_14 },
    { 0xc5655692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f655699, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc105569e, SUC_lexicon_tags_17 },
    { 0xf3c5569f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25e556a1, SUC_lexicon_tags_4 },
    { 0xca556a1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22556a7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e556ac, SUC_lexicon_tags_6 },
    { 0xf70556ad, SUC_lexicon_tags_8 },
    { 0x59c556ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb04556b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa556b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0556b9, SUC_lexicon_tags_11 },
    { 0xe96556ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10556bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac556c6, SUC_lexicon_tags_0 },
    { 0x5d8556c7, SUC_lexicon_tags_11 },
    { 0xe52556c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08556cc, SUC_lexicon_tags_31 },
    { 0xf3a556cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe52556cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e556d4, SUC_lexicon_tags_17 },
    { 0xf3a556d5, SUC_lexicon_tags_136 },
    { 0x2e6556d5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea556db, SUC_lexicon_tags_0 },
    { 0x3b6556dc, SUC_lexicon_tags_1 },
    { 0xa18556dd, SUC_lexicon_tags_2 },
    { 0x7b8556dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xda6556e0, SUC_lexicon_tags_230 },
    { 0x5d2556e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66556e7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0556ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16a556f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472556f5, SUC_lexicon_tags_27 },
    { 0x6ce556f5, SUC_lexicon_tags_21 },
    { 0xc12556f5, SUC_lexicon_tags_25 },
    { 0x3b4556f5, SUC_lexicon_tags_37 },
    { 0x5be556f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x574556fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a556ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c55704, SUC_lexicon_tags_43 },
    { 0xaa655705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfe55707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4185570d, SUC_lexicon_tags_11 },
    { 0xe305570d, SUC_lexicon_tags_25 },
    { 0x9225570f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8d655711, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da55717, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c55720, SUC_lexicon_tags_60 },
    { 0xf0255721, SUC_lexicon_tags_25 },
    { 0x8655721, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a655725, SUC_lexicon_tags_2 },
    { 0x66255726, SUC_lexicon_tags_25 },
    { 0xfa055727, SUC_lexicon_tags_12 },
    { 0x4c55727, SUC_lexicon_tags_26 },
    { 0x8e255729, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e65572b, SUC_lexicon_tags_37 },
    { 0x83e5572c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58c5572e, SUC_lexicon_tags_12 },
    { 0xfbe5572f, SUC_lexicon_tags_17 },
    { 0x905572f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd5a55732, SUC_lexicon_tags_29 },
    { 0x89055732, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce55736, SUC_lexicon_tags_38 },
    { 0xf6455737, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a5573b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb245573e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x10255740, SUC_lexicon_tags_37 },
    { 0x31e55740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c55748, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e5574c, SUC_lexicon_tags_68 },
    { 0x1bc5574d, SUC_lexicon_tags_7 },
    { 0xc985574e, SUC_lexicon_tags_15 },
    { 0xa665574e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a255753, SUC_lexicon_tags_25 },
    { 0x62a55753, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a55757, SUC_lexicon_tags_14 },
    { 0xf455757, SUC_lexicon_tags_63 },
    { 0x86a55759, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5185575b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa05575f, SUC_lexicon_tags_65 },
    { 0x7d25575f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0055767, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7be55769, SUC_lexicon_tags_23 },
    { 0x1d65576a, SUC_lexicon_tags_11 },
    { 0xf725576b, SUC_lexicon_tags_57 },
    { 0xf4c5576c, SUC_lexicon_tags_1 },
    { 0x7605576d, SUC_lexicon_tags_3 },
    { 0x5785576c, SUC_lexicon_tags_6 },
    { 0xb045576d, SUC_lexicon_tags_37 },
    { 0x85655770, SUC_lexicon_tags_16 },
    { 0x9aa55770, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4a55773, SUC_lexicon_tags_11 },
    { 0x2fc55774, SUC_lexicon_tags_12 },
    { 0xc6c55775, SUC_lexicon_tags_49 },
    { 0x45c55775, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4dc55778, SUC_lexicon_tags_65 },
    { 0x9be55779, SUC_lexicon_tags_59 },
    { 0xb685577a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a5577f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec055782, SUC_lexicon_tags_30 },
    { 0x2855783, SUC_lexicon_tags_644 },
    { 0x6055784, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x85255787, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1c55789, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec55792, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e5579c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0557a5, SUC_lexicon_tags_35 },
    { 0x346557a6, SUC_lexicon_tags_2 },
    { 0x8c2557a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43a557a9, SUC_lexicon_tags_31 },
    { 0x276557aa, SUC_lexicon_tags_49 },
    { 0x52a557ab, SUC_lexicon_tags_6 },
    { 0x2ae557ac, SUC_lexicon_tags_13 },
    { 0xadc557ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48557b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2557b6, SUC_lexicon_tags_49 },
    { 0xa50557b6, SUC_lexicon_tags_34 },
    { 0xe2c557b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2e557ba, SUC_lexicon_tags_6 },
    { 0xbfa557bb, SUC_lexicon_tags_11 },
    { 0xf22557ba, SUC_lexicon_tags_14 },
    { 0xba6557bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90557c0, SUC_lexicon_tags_11 },
    { 0xc20557c1, SUC_lexicon_tags_25 },
    { 0xc9a557c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x11a557c4, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e557c9, SUC_lexicon_tags_11 },
    { 0xfac557c9, SUC_lexicon_tags_16 },
    { 0x4bc557cb, SUC_lexicon_tags_15 },
    { 0x674557ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160557d0, SUC_lexicon_tags_14 },
    { 0x54c557d1, SUC_lexicon_tags_10 },
    { 0x1f8557d1, SUC_lexicon_tags_14 },
    { 0xf44557d3, SUC_lexicon_tags_11 },
    { 0x36557d4, SUC_lexicon_tags_15 },
    { 0xe3e557d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60557d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79e557da, SUC_lexicon_tags_60 },
    { 0x4b4557db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6557e5, SUC_lexicon_tags_33 },
    { 0x4a2557e6, SUC_lexicon_tags_12 },
    { 0x688557e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620557f0, SUC_lexicon_tags_21 },
    { 0x6d8557f1, SUC_lexicon_tags_32 },
    { 0x7f4557f0, SUC_lexicon_tags_17 },
    { 0x73c557f3, SUC_lexicon_tags_12 },
    { 0x434557f3, SUC_lexicon_tags_17 },
    { 0xcc557f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2557fa, SUC_lexicon_tags_45 },
    { 0x1b6557fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4557ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8255803, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x23855805, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x93455808, SUC_lexicon_tags_15 },
    { 0x59455809, SUC_lexicon_tags_30 },
    { 0xc4455808, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2245580e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3655813, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1805581a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c5581d, SUC_lexicon_tags_19 },
    { 0xf9e5581e, SUC_lexicon_tags_60 },
    { 0xe925581f, SUC_lexicon_tags_47 },
    { 0xcc855820, SUC_lexicon_tags_236 },
    { 0x8b65581f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc8e55823, SUC_lexicon_tags_1 },
    { 0x37255823, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x42055826, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4265582a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3485582e, SUC_lexicon_tags_4 },
    { 0x9485582f, SUC_lexicon_tags_11 },
    { 0xf925582f, SUC_lexicon_tags_19 },
    { 0x2f855831, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73a55833, SUC_lexicon_tags_0 },
    { 0x54855833, SUC_lexicon_tags_32 },
    { 0xff255834, SUC_lexicon_tags_38 },
    { 0xb9255836, SUC_lexicon_tags_15 },
    { 0x5a655837, SUC_lexicon_tags_7 },
    { 0x9ea55836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2565583b, SUC_lexicon_tags_7 },
    { 0x5bc5583c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae55841, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4aa55843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea455847, SUC_lexicon_tags_32 },
    { 0x17855848, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85584b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c55851, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce55856, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be55859, SUC_lexicon_tags_82 },
    { 0xa6455859, SUC_lexicon_tags_28 },
    { 0x8c5585a, SUC_lexicon_tags_45 },
    { 0x325585c, SUC_lexicon_tags_2 },
    { 0xe305585d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb25585f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f255866, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb665586c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa55870, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3655875, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa5587a, SUC_lexicon_tags_33 },
    { 0xe845587b, SUC_lexicon_tags_33 },
    { 0xa625587c, SUC_lexicon_tags_7 },
    { 0x605587d, SUC_lexicon_tags_11 },
    { 0x4885587e, SUC_lexicon_tags_1 },
    { 0xd6c5587d, SUC_lexicon_tags_25 },
    { 0x7a45587e, SUC_lexicon_tags_1 },
    { 0x35c55881, SUC_lexicon_tags_11 },
    { 0xfa65587e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a55888, SUC_lexicon_tags_65 },
    { 0x66e55889, SUC_lexicon_tags_7 },
    { 0xa045588a, SUC_lexicon_tags_12 },
    { 0x76e5588b, SUC_lexicon_tags_11 },
    { 0x4305588c, SUC_lexicon_tags_27 },
    { 0x7245588d, SUC_lexicon_tags_37 },
    { 0x31a5588e, SUC_lexicon_tags_47 },
    { 0xa405588c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc255892, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e55899, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5585589f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b2558a1, SUC_lexicon_tags_68 },
    { 0x3a558a1, SUC_lexicon_tags_2 },
    { 0x910558a3, SUC_lexicon_tags_15 },
    { 0xdbe558a4, SUC_lexicon_tags_19 },
    { 0x600558a3, SUC_lexicon_tags_17 },
    { 0x410558a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776558ab, SUC_lexicon_tags_7 },
    { 0xb94558ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c558af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x122558b2, SUC_lexicon_tags_14 },
    { 0x14c558b3, SUC_lexicon_tags_7 },
    { 0x8c558b3, SUC_lexicon_tags_136 },
    { 0x502558b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30558ba, SUC_lexicon_tags_30 },
    { 0xaa2558bb, SUC_lexicon_tags_30 },
    { 0x8d4558bc, SUC_lexicon_tags_37 },
    { 0x81a558bd, SUC_lexicon_tags_2 },
    { 0x140558bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8558c4, SUC_lexicon_tags_19 },
    { 0xca0558c4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2558c9, SUC_lexicon_tags_46 },
    { 0xf40558ca, SUC_lexicon_tags_0 },
    { 0x8d0558cb, SUC_lexicon_tags_37 },
    { 0xbe0558cb, SUC_lexicon_tags_25 },
    { 0xe7c558cd, SUC_lexicon_tags_37 },
    { 0x4a6558ce, SUC_lexicon_tags_82 },
    { 0x640558cf, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72558d2, SUC_lexicon_tags_47 },
    { 0xc2e558d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0558db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2558e0, SUC_lexicon_tags_32 },
    { 0x846558e0, SUC_lexicon_tags_33 },
    { 0x826558e0, SUC_lexicon_tags_6 },
    { 0x936558e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a558e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5a8558e9, SUC_lexicon_tags_12 },
    { 0x7fa558e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36558ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x326558f1, SUC_lexicon_tags_14 },
    { 0x646558f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496558f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692558fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x840558ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa055901, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0, NULL },
    { 0x10855904, SUC_lexicon_tags_0 },
    { 0x47e55905, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8855908, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7485590d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a55910, SUC_lexicon_tags_15 },
    { 0x31c55911, SUC_lexicon_tags_10 },
    { 0xbae55910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c55918, SUC_lexicon_tags_12 },
    { 0x45855919, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b655921, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c055925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4655927, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca5592a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4925592f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2055933, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7655936, SUC_lexicon_tags_2 },
    { 0x1d855937, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa5593a, SUC_lexicon_tags_37 },
    { 0x59e5593b, SUC_lexicon_tags_3 },
    { 0x2d65593c, SUC_lexicon_tags_27 },
    { 0x3aa5593a, SUC_lexicon_tags_17 },
    { 0x6d05593e, SUC_lexicon_tags_13 },
    { 0xae25593f, SUC_lexicon_tags_201 },
    { 0xd9e55940, SUC_lexicon_tags_91 },
    { 0x66855941, SUC_lexicon_tags_31 },
    { 0x4f855941, SUC_lexicon_tags_23 },
    { 0xbd255941, SUC_lexicon_tags_7 },
    { 0x3b855944, SUC_lexicon_tags_63 },
    { 0x55255945, SUC_lexicon_tags_25 },
    { 0x36e55943, SUC_lexicon_tags_63 },
    { 0x60055946, SUC_lexicon_tags_15 },
    { 0xa6455948, SUC_lexicon_tags_63 },
    { 0x8ba55948, SUC_lexicon_tags_26 },
    { 0xf5c5593c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e25594c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e855950, SUC_lexicon_tags_21 },
    { 0x8e655951, SUC_lexicon_tags_23 },
    { 0xf4255950, SUC_lexicon_tags_15 },
    { 0x94255953, SUC_lexicon_tags_37 },
    { 0xbfe55953, SUC_lexicon_tags_7 },
    { 0x77455955, SUC_lexicon_tags_30 },
    { 0x51255956, SUC_lexicon_tags_4 },
    { 0xbf855956, SUC_lexicon_tags_60 },
    { 0x54a55955, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xba25595a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0e5595c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x40a5595e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x19855961, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fc55963, SUC_lexicon_tags_7 },
    { 0x22455964, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a655968, SUC_lexicon_tags_38 },
    { 0x4dc55968, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb245596b, SUC_lexicon_tags_29 },
    { 0x2c05596c, SUC_lexicon_tags_37 },
    { 0x1925596d, SUC_lexicon_tags_1 },
    { 0x8985596c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde255971, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea055978, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9665597e, SUC_lexicon_tags_59 },
    { 0x3cc5597e, SUC_lexicon_tags_27 },
    { 0x65c5597e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f855987, SUC_lexicon_tags_33 },
    { 0xbe55988, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3405598a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd3a5598c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd385598f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c255999, SUC_lexicon_tags_35 },
    { 0x4785599a, SUC_lexicon_tags_65 },
    { 0x125599a, SUC_lexicon_tags_37 },
    { 0x9b85599b, SUC_lexicon_tags_8 },
    { 0x3b65599b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a559a0, SUC_lexicon_tags_17 },
    { 0x438559a1, SUC_lexicon_tags_37 },
    { 0x3b0559a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a559a6, SUC_lexicon_tags_1 },
    { 0xe0e559a6, SUC_lexicon_tags_47 },
    { 0x56559a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32559ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc559b2, SUC_lexicon_tags_17 },
    { 0xee4559b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8559b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32559ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188559be, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1c559c0, SUC_lexicon_tags_17 },
    { 0xf9a559c0, SUC_lexicon_tags_15 },
    { 0x686559c2, SUC_lexicon_tags_12 },
    { 0x858559c3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x740559c5, SUC_lexicon_tags_19 },
    { 0xa6a559c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a559c9, SUC_lexicon_tags_23 },
    { 0x51a559ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194559d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c559db, SUC_lexicon_tags_27 },
    { 0x648559db, SUC_lexicon_tags_37 },
    { 0xd94559dd, SUC_lexicon_tags_25 },
    { 0xc6e559de, SUC_lexicon_tags_23 },
    { 0x696559df, SUC_lexicon_tags_61 },
    { 0x3b4559dc, SUC_lexicon_tags_19 },
    { 0xfd0559de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x122559e3, SUC_lexicon_tags_17 },
    { 0xf38559e4, SUC_lexicon_tags_25 },
    { 0x294559e4, SUC_lexicon_tags_25 },
    { 0xa6a559e6, SUC_lexicon_tags_25 },
    { 0xfc559e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8559f4, SUC_lexicon_tags_37 },
    { 0x288559f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba559f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x460559fa, SUC_lexicon_tags_11 },
    { 0xcb8559fb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a559ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa055a01, SUC_lexicon_tags_25 },
    { 0x84455a02, SUC_lexicon_tags_47 },
    { 0x35455a01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7055a0c, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0855a10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c855a13, SUC_lexicon_tags_1 },
    { 0x81e55a13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2055a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa55a20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x78455a23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1d655a25, SUC_lexicon_tags_25 },
    { 0x51c55a25, SUC_lexicon_tags_12 },
    { 0x6a655a27, SUC_lexicon_tags_43 },
    { 0xff855a28, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfaa55a2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3655a2d, SUC_lexicon_tags_14 },
    { 0xaf055a2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be55a35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a55a38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87455a40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40c55a42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe855a44, SUC_lexicon_tags_9 },
    { 0x8e55a45, SUC_lexicon_tags_12 },
    { 0xf4e55a44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e655a4c, SUC_lexicon_tags_37 },
    { 0xf0a55a4d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d655a50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c55a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64c55a55, SUC_lexicon_tags_31 },
    { 0x53a55a56, SUC_lexicon_tags_28 },
    { 0x13655a57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaec55a59, SUC_lexicon_tags_55 },
    { 0x4f455a5a, SUC_lexicon_tags_66 },
    { 0x92855a5a, SUC_lexicon_tags_26 },
    { 0x51455a5c, SUC_lexicon_tags_17 },
    { 0xaee55a5c, SUC_lexicon_tags_25 },
    { 0x5e055a5d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x21255a61, SUC_lexicon_tags_48 },
    { 0x8855a62, SUC_lexicon_tags_63 },
    { 0x94e55a61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e255a66, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a455a69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec455a6c, SUC_lexicon_tags_63 },
    { 0x14455a6c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbb255a6f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e55a72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e55a75, SUC_lexicon_tags_47 },
    { 0x63055a75, SUC_lexicon_tags_7 },
    { 0x90a55a75, SUC_lexicon_tags_7 },
    { 0x8f455a75, SUC_lexicon_tags_28 },
    { 0x6fa55a79, SUC_lexicon_tags_25 },
    { 0xa255a7a, SUC_lexicon_tags_12 },
    { 0x74055a75, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4fe55a7d, SUC_lexicon_tags_0 },
    { 0x4655a7e, SUC_lexicon_tags_37 },
    { 0xe055a7e, SUC_lexicon_tags_55 },
    { 0xd1855a7f, SUC_lexicon_tags_53 },
    { 0xb9a55a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8055a83, SUC_lexicon_tags_37 },
    { 0xe7e55a83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f855a86, SUC_lexicon_tags_2 },
    { 0xdd255a86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9855a89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29455a8e, SUC_lexicon_tags_26 },
    { 0x8ca55a8e, SUC_lexicon_tags_33 },
    { 0x88255a8f, SUC_lexicon_tags_30 },
    { 0xb8055a91, SUC_lexicon_tags_502 },
    { 0, NULL },
    { 0x36e55a93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca55a99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc455a9b, SUC_lexicon_tags_37 },
    { 0xb9055a9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c655a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19255aa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94255aa9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xba455aab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a55ab0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e55abb, SUC_lexicon_tags_35 },
    { 0x23e55abc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c55abf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb055ac2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2055ac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e55ac8, SUC_lexicon_tags_15 },
    { 0x77c55ac8, SUC_lexicon_tags_29 },
    { 0xc0055aca, SUC_lexicon_tags_27 },
    { 0x81655aca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da55ad2, SUC_lexicon_tags_11 },
    { 0x6a655ad3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76455ad5, SUC_lexicon_tags_49 },
    { 0x50855ad6, SUC_lexicon_tags_15 },
    { 0xdf055ad7, SUC_lexicon_tags_43 },
    { 0x8fc55ad7, SUC_lexicon_tags_19 },
    { 0x60055ad6, SUC_lexicon_tags_33 },
    { 0xcf455ada, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56855add, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef055ae2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e455ae4, SUC_lexicon_tags_49 },
    { 0x94e55ae4, SUC_lexicon_tags_12 },
    { 0x84c55ae4, SUC_lexicon_tags_63 },
    { 0x66e55ae4, SUC_lexicon_tags_17 },
    { 0x13255ae7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c55aed, SUC_lexicon_tags_65 },
    { 0x82255aee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea55af1, SUC_lexicon_tags_33 },
    { 0x63e55af1, SUC_lexicon_tags_47 },
    { 0xa3255af3, SUC_lexicon_tags_12 },
    { 0x1fa55af4, SUC_lexicon_tags_299 },
    { 0x23a55af1, SUC_lexicon_tags_37 },
    { 0xefe55af3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e55afd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e055b03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc55b06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac55b09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ea55b0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x74655b0d, SUC_lexicon_tags_2 },
    { 0x67455b0e, SUC_lexicon_tags_11 },
    { 0xf8a55b0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94855b12, SUC_lexicon_tags_14 },
    { 0x5be55b13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64a55b15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95855b1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x51e55b1c, SUC_lexicon_tags_25 },
    { 0x25455b1d, SUC_lexicon_tags_14 },
    { 0xd1055b1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e655b22, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x855b24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2be55b26, SUC_lexicon_tags_165 },
    { 0x4d255b27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e255b2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67455b2d, SUC_lexicon_tags_65 },
    { 0x7da55b2d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62655b34, SUC_lexicon_tags_38 },
    { 0x5e855b35, SUC_lexicon_tags_4 },
    { 0x26455b35, SUC_lexicon_tags_35 },
    { 0xaa055b37, SUC_lexicon_tags_25 },
    { 0x31255b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac55b3e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x81055b40, SUC_lexicon_tags_0 },
    { 0x1e55b41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7455b44, SUC_lexicon_tags_37 },
    { 0xafa55b45, SUC_lexicon_tags_32 },
    { 0xf6455b46, SUC_lexicon_tags_25 },
    { 0xa8c55b46, SUC_lexicon_tags_6 },
    { 0x3fa55b45, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c255b4e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49455b52, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x30655b55, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba55b59, SUC_lexicon_tags_15 },
    { 0x21455b5a, SUC_lexicon_tags_12 },
    { 0xa7255b5b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee55b5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa455b60, SUC_lexicon_tags_23 },
    { 0x61655b61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e655b6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe655b6c, SUC_lexicon_tags_60 },
    { 0x3d255b6d, SUC_lexicon_tags_23 },
    { 0x43855b6d, SUC_lexicon_tags_15 },
    { 0x45455b6f, SUC_lexicon_tags_25 },
    { 0x30455b6d, SUC_lexicon_tags_7 },
    { 0xb9455b6f, SUC_lexicon_tags_17 },
    { 0x70655b72, SUC_lexicon_tags_11 },
    { 0x59655b72, SUC_lexicon_tags_29 },
    { 0xa4c55b74, SUC_lexicon_tags_10 },
    { 0xe6655b75, SUC_lexicon_tags_37 },
    { 0x11c55b70, SUC_lexicon_tags_194 },
    { 0xa4655b77, SUC_lexicon_tags_38 },
    { 0xad855b76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc455b7c, SUC_lexicon_tags_20 },
    { 0x9655b7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef055b80, SUC_lexicon_tags_17 },
    { 0x7b255b80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2055b83, SUC_lexicon_tags_19 },
    { 0x8ba55b84, SUC_lexicon_tags_100 },
    { 0x30255b85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f655b8b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x34655b8d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7b855b8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9e55b91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98255b94, SUC_lexicon_tags_14 },
    { 0x12a55b94, SUC_lexicon_tags_106 },
    { 0x8a655b96, SUC_lexicon_tags_1 },
    { 0xc0655b97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec55ba0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0855ba2, SUC_lexicon_tags_72 },
    { 0x9e055ba2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2855ba7, SUC_lexicon_tags_4 },
    { 0xd4e55ba8, SUC_lexicon_tags_25 },
    { 0x9c855ba8, SUC_lexicon_tags_12 },
    { 0x78255baa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x40255bac, SUC_lexicon_tags_13 },
    { 0xaa255bad, SUC_lexicon_tags_2 },
    { 0xaee55bae, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xf5655bb0, SUC_lexicon_tags_37 },
    { 0xad255bb0, SUC_lexicon_tags_10 },
    { 0x14655bb1, SUC_lexicon_tags_12 },
    { 0x5b655bb3, SUC_lexicon_tags_11 },
    { 0x6a55bb3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a55bb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18855bbd, SUC_lexicon_tags_17 },
    { 0xb2a55bbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c255bc6, SUC_lexicon_tags_23 },
    { 0x4c055bc7, SUC_lexicon_tags_7 },
    { 0xa855bc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0855bcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1855bcf, SUC_lexicon_tags_25 },
    { 0xe3055bd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcba55bd2, SUC_lexicon_tags_141 },
    { 0x29e55bd2, SUC_lexicon_tags_2 },
    { 0x1ac55bd2, SUC_lexicon_tags_29 },
    { 0xc2c55bd5, SUC_lexicon_tags_27 },
    { 0x72055bd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13455bdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4dc55bdf, SUC_lexicon_tags_107 },
    { 0x57a55be0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9255be2, SUC_lexicon_tags_45 },
    { 0xca655be2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a55be6, SUC_lexicon_tags_46 },
    { 0x40855be7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a55bf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88855bf7, SUC_lexicon_tags_7 },
    { 0x55855bf8, SUC_lexicon_tags_11 },
    { 0x76455bf8, SUC_lexicon_tags_26 },
    { 0x29855bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c55bfd, SUC_lexicon_tags_47 },
    { 0x9f055bfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e855c01, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba55c09, SUC_lexicon_tags_11 },
    { 0xfb855c0a, SUC_lexicon_tags_17 },
    { 0xe2c55c0b, SUC_lexicon_tags_38 },
    { 0xb4055c0c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x35655c0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11a55c10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20055c12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38055c16, SUC_lexicon_tags_15 },
    { 0x9b455c17, SUC_lexicon_tags_49 },
    { 0xea455c17, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc455c1c, SUC_lexicon_tags_14 },
    { 0xe5e55c1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xda55c20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a55c24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55c2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18055c33, SUC_lexicon_tags_15 },
    { 0xab455c34, SUC_lexicon_tags_11 },
    { 0xc3a55c35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e55c3f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a055c42, SUC_lexicon_tags_12 },
    { 0x49a55c42, SUC_lexicon_tags_1 },
    { 0x64c55c43, SUC_lexicon_tags_31 },
    { 0x7cc55c45, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8455c4a, SUC_lexicon_tags_2 },
    { 0x255c4b, SUC_lexicon_tags_9 },
    { 0x1aa55c4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1a55c4e, SUC_lexicon_tags_15 },
    { 0xeb455c4f, SUC_lexicon_tags_7 },
    { 0xc655c50, SUC_lexicon_tags_19 },
    { 0x65e55c51, SUC_lexicon_tags_87 },
    { 0xa6055c52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15055c55, SUC_lexicon_tags_5 },
    { 0x56255c56, SUC_lexicon_tags_32 },
    { 0x4ba55c57, SUC_lexicon_tags_11 },
    { 0x5d055c58, SUC_lexicon_tags_9 },
    { 0x8c55c59, SUC_lexicon_tags_9 },
    { 0x3de55c57, SUC_lexicon_tags_25 },
    { 0xd5c55c57, SUC_lexicon_tags_27 },
    { 0xc9e55c56, SUC_lexicon_tags_79 },
    { 0xcd255c5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe4255c5f, SUC_lexicon_tags_127 },
    { 0x25c55c5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14e55c62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96855c67, SUC_lexicon_tags_15 },
    { 0x4ec55c68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x43e55c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c055c71, SUC_lexicon_tags_1 },
    { 0xebe55c71, SUC_lexicon_tags_12 },
    { 0x38c55c72, SUC_lexicon_tags_14 },
    { 0x84455c73, SUC_lexicon_tags_45 },
    { 0xce055c75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f855c78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c55c7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a55c82, SUC_lexicon_tags_79 },
    { 0x89855c83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e55c89, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d855c8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93855c8e, SUC_lexicon_tags_14 },
    { 0x83c55c8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e55c92, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd7455c94, SUC_lexicon_tags_47 },
    { 0xc1855c94, SUC_lexicon_tags_25 },
    { 0xc0a55c96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e55c9a, SUC_lexicon_tags_328 },
    { 0xf6855c9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe655c9e, SUC_lexicon_tags_6 },
    { 0xf2a55c9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49855ca2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1655ca7, SUC_lexicon_tags_2 },
    { 0x3c255ca7, SUC_lexicon_tags_2 },
    { 0xd5255ca9, SUC_lexicon_tags_14 },
    { 0xb1855ca7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e55cb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10255cb5, SUC_lexicon_tags_8 },
    { 0xee655cb6, SUC_lexicon_tags_8 },
    { 0x54255cb7, SUC_lexicon_tags_45 },
    { 0xec855cb8, SUC_lexicon_tags_1 },
    { 0x6b055cb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e55cbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc4855cbe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8655cc0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27c55cc2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf655cc4, SUC_lexicon_tags_30 },
    { 0x7b055cc5, SUC_lexicon_tags_27 },
    { 0xb1855cc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61855ccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa55cce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2655cd0, SUC_lexicon_tags_30 },
    { 0x2f255cd1, SUC_lexicon_tags_296 },
    { 0xb5255cd2, SUC_lexicon_tags_26 },
    { 0x3cc55cd2, SUC_lexicon_tags_49 },
    { 0xa1255cd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf055cdd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55c55cdf, SUC_lexicon_tags_2 },
    { 0x255ce0, SUC_lexicon_tags_31 },
    { 0x4da55ce1, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c55ce8, SUC_lexicon_tags_11 },
    { 0x7ea55ce9, SUC_lexicon_tags_45 },
    { 0xba55cea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c55ced, SUC_lexicon_tags_63 },
    { 0xe2455cee, SUC_lexicon_tags_25 },
    { 0x3b055cef, SUC_lexicon_tags_17 },
    { 0x33455ced, SUC_lexicon_tags_49 },
    { 0x43a55cf1, SUC_lexicon_tags_23 },
    { 0xff655cf1, SUC_lexicon_tags_25 },
    { 0xc4a55cf3, SUC_lexicon_tags_2 },
    { 0x6855cf3, SUC_lexicon_tags_28 },
    { 0x24c55cf5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f255cf9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7d655cfb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66255cfd, SUC_lexicon_tags_15 },
    { 0xcac55cfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3855d00, SUC_lexicon_tags_37 },
    { 0x5be55d01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe55d04, SUC_lexicon_tags_11 },
    { 0x26655d05, SUC_lexicon_tags_2 },
    { 0x63a55d06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9255d0b, SUC_lexicon_tags_11 },
    { 0x49e55d0c, SUC_lexicon_tags_14 },
    { 0x1e255d0c, SUC_lexicon_tags_12 },
    { 0xdc455d0e, SUC_lexicon_tags_21 },
    { 0x1ea55d0f, SUC_lexicon_tags_47 },
    { 0x5ee55d0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c055d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9055d18, SUC_lexicon_tags_17 },
    { 0x25255d19, SUC_lexicon_tags_28 },
    { 0x41a55d19, SUC_lexicon_tags_43 },
    { 0x91855d1b, SUC_lexicon_tags_12 },
    { 0xe1655d18, SUC_lexicon_tags_38 },
    { 0x51255d1d, SUC_lexicon_tags_47 },
    { 0x53055d1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81255d27, SUC_lexicon_tags_7 },
    { 0x7d255d28, SUC_lexicon_tags_2 },
    { 0x66055d28, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xdb255d2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71655d2f, SUC_lexicon_tags_33 },
    { 0x55055d2f, SUC_lexicon_tags_158 },
    { 0x99e55d2f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7055d36, SUC_lexicon_tags_36 },
    { 0xbb255d37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54255d3a, SUC_lexicon_tags_26 },
    { 0xf9055d3b, SUC_lexicon_tags_46 },
    { 0x9f655d3b, SUC_lexicon_tags_23 },
    { 0xaae55d3b, SUC_lexicon_tags_25 },
    { 0xf0855d3e, SUC_lexicon_tags_17 },
    { 0x5055d3e, SUC_lexicon_tags_28 },
    { 0x80a55d3e, SUC_lexicon_tags_26 },
    { 0x3be55d40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x255d44, SUC_lexicon_tags_12 },
    { 0xc2c55d45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a55d4b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6455d4e, SUC_lexicon_tags_13 },
    { 0xf1e55d4f, SUC_lexicon_tags_38 },
    { 0xa9e55d4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43855d52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da55d55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfbe55d57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce55d5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f455d5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc55d61, SUC_lexicon_tags_25 },
    { 0x5bc55d62, SUC_lexicon_tags_11 },
    { 0x64255d63, SUC_lexicon_tags_12 },
    { 0x86255d64, SUC_lexicon_tags_30 },
    { 0x97455d65, SUC_lexicon_tags_1 },
    { 0x46e55d65, SUC_lexicon_tags_19 },
    { 0xd5655d66, SUC_lexicon_tags_25 },
    { 0x96655d67, SUC_lexicon_tags_45 },
    { 0xd4e55d65, SUC_lexicon_tags_15 },
    { 0xffa55d69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa55d6d, SUC_lexicon_tags_25 },
    { 0x16055d6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29855d79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57655d7b, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55d7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc8655d81, SUC_lexicon_tags_43 },
    { 0xa7e55d82, SUC_lexicon_tags_1 },
    { 0x6c455d83, SUC_lexicon_tags_37 },
    { 0x7a855d82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f255d86, SUC_lexicon_tags_29 },
    { 0xd7655d87, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b655d8d, SUC_lexicon_tags_25 },
    { 0xf8e55d8e, SUC_lexicon_tags_19 },
    { 0xa5c55d8d, SUC_lexicon_tags_12 },
    { 0x46255d90, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7855d93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c455d97, SUC_lexicon_tags_87 },
    { 0x5a55d98, SUC_lexicon_tags_7 },
    { 0xbf655d98, SUC_lexicon_tags_11 },
    { 0x6fa55d9a, SUC_lexicon_tags_37 },
    { 0xe6455d9a, SUC_lexicon_tags_3 },
    { 0xa5455d9c, SUC_lexicon_tags_22 },
    { 0xd6055d9c, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0xc2055d9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3455da5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c255dac, SUC_lexicon_tags_17 },
    { 0xe7455dad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ca55daf, SUC_lexicon_tags_37 },
    { 0xbb055db0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae255db5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e255db9, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x1da55dbb, SUC_lexicon_tags_37 },
    { 0xaea55dbb, SUC_lexicon_tags_47 },
    { 0x7da55dbc, SUC_lexicon_tags_26 },
    { 0x79055dbe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc655dc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e55dc3, SUC_lexicon_tags_45 },
    { 0xc0055dc3, SUC_lexicon_tags_29 },
    { 0x54855dc5, SUC_lexicon_tags_26 },
    { 0x18455dc3, SUC_lexicon_tags_8 },
    { 0x92455dc7, SUC_lexicon_tags_17 },
    { 0xc1e55dc8, SUC_lexicon_tags_11 },
    { 0x91055dc6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x10455dcc, SUC_lexicon_tags_14 },
    { 0xc5455dcd, SUC_lexicon_tags_38 },
    { 0x89655dcd, SUC_lexicon_tags_11 },
    { 0x6d255dcd, SUC_lexicon_tags_32 },
    { 0x59e55dd0, SUC_lexicon_tags_11 },
    { 0x32c55dd0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x83255dd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81655ddb, SUC_lexicon_tags_23 },
    { 0xe6255ddc, SUC_lexicon_tags_17 },
    { 0xd2455ddc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30655de2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9055de7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da55dea, SUC_lexicon_tags_7 },
    { 0x45255deb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf1055ded, SUC_lexicon_tags_9 },
    { 0x2255dee, SUC_lexicon_tags_49 },
    { 0xc3055def, SUC_lexicon_tags_15 },
    { 0x79e55df0, SUC_lexicon_tags_38 },
    { 0xeb055ded, SUC_lexicon_tags_23 },
    { 0xd7255ded, SUC_lexicon_tags_25 },
    { 0xfba55dee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52455df5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a55dfa, SUC_lexicon_tags_2 },
    { 0x9c055dfb, SUC_lexicon_tags_1 },
    { 0xe8e55dfc, SUC_lexicon_tags_10 },
    { 0x82e55dfc, SUC_lexicon_tags_46 },
    { 0xcdc55dfc, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x60055e01, SUC_lexicon_tags_0 },
    { 0x2455e02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce655e0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x44e55e0d, SUC_lexicon_tags_65 },
    { 0x3b055e0d, SUC_lexicon_tags_25 },
    { 0x47455e0f, SUC_lexicon_tags_43 },
    { 0x8b855e10, SUC_lexicon_tags_14 },
    { 0xffa55e11, SUC_lexicon_tags_66 },
    { 0xb8855e12, SUC_lexicon_tags_7 },
    { 0xdfe55e11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xece55e15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3855e18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa455e1d, SUC_lexicon_tags_11 },
    { 0x8b055e1d, SUC_lexicon_tags_160 },
    { 0xdfe55e1f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9655e21, SUC_lexicon_tags_2 },
    { 0x21055e22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a55e26, SUC_lexicon_tags_14 },
    { 0x45055e27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6ae55e29, SUC_lexicon_tags_3 },
    { 0x2fa55e2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf855e2c, SUC_lexicon_tags_7 },
    { 0x52c55e2d, SUC_lexicon_tags_14 },
    { 0x3c655e2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20055e39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34055e41, SUC_lexicon_tags_35 },
    { 0xee55e42, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1855e45, SUC_lexicon_tags_49 },
    { 0xd2455e46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x51e55e48, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6fe55e4a, SUC_lexicon_tags_0 },
    { 0xdd055e4b, SUC_lexicon_tags_188 },
    { 0xa8c55e4b, SUC_lexicon_tags_28 },
    { 0x89c55e4d, SUC_lexicon_tags_750 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e55e54, SUC_lexicon_tags_235 },
    { 0xd2655e55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94055e57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40455e59, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x57655e5b, SUC_lexicon_tags_7 },
    { 0xc5255e5b, SUC_lexicon_tags_28 },
    { 0x43655e5d, SUC_lexicon_tags_52 },
    { 0xf1255e5d, SUC_lexicon_tags_7 },
    { 0x4f455e5f, SUC_lexicon_tags_12 },
    { 0x3a055e60, SUC_lexicon_tags_16 },
    { 0x8a55e60, SUC_lexicon_tags_15 },
    { 0xbb055e60, SUC_lexicon_tags_187 },
    { 0x8d455e63, SUC_lexicon_tags_4 },
    { 0xcd055e60, SUC_lexicon_tags_27 },
    { 0x6c455e65, SUC_lexicon_tags_61 },
    { 0xa4e55e66, SUC_lexicon_tags_63 },
    { 0x84a55e62, SUC_lexicon_tags_15 },
    { 0xc3255e67, SUC_lexicon_tags_15 },
    { 0xd6455e64, SUC_lexicon_tags_13 },
    { 0xbf255e61, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4c55e6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71255e71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ec55e73, SUC_lexicon_tags_47 },
    { 0x8255e73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x75055e76, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1655e79, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51855e7e, SUC_lexicon_tags_37 },
    { 0xf6e55e7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91055e83, SUC_lexicon_tags_2 },
    { 0x59055e84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb0a55e86, SUC_lexicon_tags_489 },
    { 0x27c55e87, SUC_lexicon_tags_25 },
    { 0x3f055e88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e55e90, SUC_lexicon_tags_45 },
    { 0xb0e55e91, SUC_lexicon_tags_17 },
    { 0xec55e92, SUC_lexicon_tags_37 },
    { 0x7d455e92, SUC_lexicon_tags_91 },
    { 0x3ba55e93, SUC_lexicon_tags_7 },
    { 0x11655e93, SUC_lexicon_tags_25 },
    { 0x98455e92, SUC_lexicon_tags_11 },
    { 0x18e55e95, SUC_lexicon_tags_14 },
    { 0xb4255e98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27855ea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe55ea3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92055ea9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9455eaf, SUC_lexicon_tags_9 },
    { 0xd4655eaf, SUC_lexicon_tags_65 },
    { 0xf1c55eb1, SUC_lexicon_tags_11 },
    { 0xddc55eb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9655eb5, SUC_lexicon_tags_25 },
    { 0x6e055eb6, SUC_lexicon_tags_27 },
    { 0xea255eb6, SUC_lexicon_tags_30 },
    { 0xde255eb6, SUC_lexicon_tags_37 },
    { 0xb4c55eb7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x17855ebc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xafe55ebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5855ec7, SUC_lexicon_tags_12 },
    { 0x85e55ec7, SUC_lexicon_tags_7 },
    { 0x2fe55ec9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ca55ecb, SUC_lexicon_tags_36 },
    { 0x63255ecc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4655ecf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66c55ed1, SUC_lexicon_tags_1 },
    { 0x92055ed2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76455ed5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15455eda, SUC_lexicon_tags_1 },
    { 0xee855edb, SUC_lexicon_tags_57 },
    { 0xfdc55edc, SUC_lexicon_tags_23 },
    { 0xe7a55edd, SUC_lexicon_tags_84 },
    { 0xb8a55edb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c55ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea55eeb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55ef6, SUC_lexicon_tags_87 },
    { 0x4c455ef6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda855ef9, SUC_lexicon_tags_87 },
    { 0xe455efa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6055efd, SUC_lexicon_tags_1 },
    { 0x85455efd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f855f08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xac255f0b, SUC_lexicon_tags_9 },
    { 0xad655f0c, SUC_lexicon_tags_1 },
    { 0x95c55f0c, SUC_lexicon_tags_19 },
    { 0x62055f0d, SUC_lexicon_tags_17 },
    { 0x57855f0f, SUC_lexicon_tags_10 },
    { 0x78e55f0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c55f14, SUC_lexicon_tags_45 },
    { 0x43855f14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x55e55f17, SUC_lexicon_tags_15 },
    { 0x51c55f17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ce55f1a, SUC_lexicon_tags_7 },
    { 0x66255f1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee55f23, SUC_lexicon_tags_15 },
    { 0xb2c55f23, SUC_lexicon_tags_12 },
    { 0x60a55f23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a55f28, SUC_lexicon_tags_45 },
    { 0xbf855f28, SUC_lexicon_tags_56 },
    { 0x4ce55f2a, SUC_lexicon_tags_47 },
    { 0xd6855f2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22455f2f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf2855f31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe55f39, SUC_lexicon_tags_30 },
    { 0xe4655f3a, SUC_lexicon_tags_29 },
    { 0x81855f3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4455f42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e55f46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f655f50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2055f58, SUC_lexicon_tags_116 },
    { 0x2dc55f58, SUC_lexicon_tags_11 },
    { 0x84255f5a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf7c55f5c, SUC_lexicon_tags_2 },
    { 0xad855f5d, SUC_lexicon_tags_11 },
    { 0xd7c55f5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c55f69, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee55f6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc55f71, SUC_lexicon_tags_2 },
    { 0x30a55f72, SUC_lexicon_tags_11 },
    { 0x71855f73, SUC_lexicon_tags_37 },
    { 0x2b055f74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea55f77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31e55f79, SUC_lexicon_tags_37 },
    { 0x96c55f7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f055f82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5655f85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe55f8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1de55f8c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e55f91, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb655f95, SUC_lexicon_tags_39 },
    { 0xdce55f96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1655f99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xef455f9c, SUC_lexicon_tags_47 },
    { 0x5e255f9c, SUC_lexicon_tags_32 },
    { 0x9b455f9c, SUC_lexicon_tags_17 },
    { 0xed055f9d, SUC_lexicon_tags_7 },
    { 0xc8055fa0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0255fa4, SUC_lexicon_tags_17 },
    { 0xc3255fa5, SUC_lexicon_tags_25 },
    { 0x8b055fa6, SUC_lexicon_tags_0 },
    { 0x23a55fa7, SUC_lexicon_tags_160 },
    { 0xf8655fa8, SUC_lexicon_tags_23 },
    { 0x34655fa6, SUC_lexicon_tags_15 },
    { 0x17a55faa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5655fae, SUC_lexicon_tags_7 },
    { 0x57255faf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf5855fb1, SUC_lexicon_tags_5 },
    { 0x5d455fb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73055fb6, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x59855fb9, SUC_lexicon_tags_10 },
    { 0x41055fba, SUC_lexicon_tags_15 },
    { 0xc3655fbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3c255fbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e255fc0, SUC_lexicon_tags_2 },
    { 0x24c55fc1, SUC_lexicon_tags_45 },
    { 0x6e655fc0, SUC_lexicon_tags_87 },
    { 0x8c855fc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e55fc8, SUC_lexicon_tags_44 },
    { 0x49655fc9, SUC_lexicon_tags_27 },
    { 0xb2e55fc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a55fce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94255fd3, SUC_lexicon_tags_43 },
    { 0x10a55fd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57455fdd, SUC_lexicon_tags_11 },
    { 0xa4c55fde, SUC_lexicon_tags_23 },
    { 0x6ea55fdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb055fe4, SUC_lexicon_tags_2 },
    { 0xe7c55fe4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x27455fe7, SUC_lexicon_tags_59 },
    { 0x21655fe7, SUC_lexicon_tags_15 },
    { 0xcd655fe9, SUC_lexicon_tags_8 },
    { 0x29c55fea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48655fee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45255ff1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x76855ff4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b455ff6, SUC_lexicon_tags_1 },
    { 0xba855ff7, SUC_lexicon_tags_25 },
    { 0x41855ff7, SUC_lexicon_tags_25 },
    { 0x6e855ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae655ffd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0855fff, SUC_lexicon_tags_4 },
    { 0xbba55fff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c56004, SUC_lexicon_tags_48 },
    { 0xc5456005, SUC_lexicon_tags_8 },
    { 0xc056006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc5600a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6c5600c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb365600f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76e56011, SUC_lexicon_tags_11 },
    { 0xaec56012, SUC_lexicon_tags_43 },
    { 0x52656013, SUC_lexicon_tags_29 },
    { 0x8b656014, SUC_lexicon_tags_15 },
    { 0x9ce56015, SUC_lexicon_tags_45 },
    { 0xb4856012, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x68a56018, SUC_lexicon_tags_63 },
    { 0x98456018, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59a5601b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2d85601d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d056021, SUC_lexicon_tags_47 },
    { 0xaa056022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9585602b, SUC_lexicon_tags_11 },
    { 0xae05602c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34656030, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a456032, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0056035, SUC_lexicon_tags_30 },
    { 0xdf856035, SUC_lexicon_tags_0 },
    { 0xdea56037, SUC_lexicon_tags_32 },
    { 0xb1656037, SUC_lexicon_tags_1 },
    { 0xaa256037, SUC_lexicon_tags_114 },
    { 0xcd056035, SUC_lexicon_tags_11 },
    { 0x7e05603b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca856041, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b85604a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd85604e, SUC_lexicon_tags_17 },
    { 0xcf05604f, SUC_lexicon_tags_17 },
    { 0xfea56050, SUC_lexicon_tags_15 },
    { 0x3e56051, SUC_lexicon_tags_7 },
    { 0x83656052, SUC_lexicon_tags_2 },
    { 0xe0456053, SUC_lexicon_tags_13 },
    { 0x99856054, SUC_lexicon_tags_11 },
    { 0x44e56053, SUC_lexicon_tags_11 },
    { 0xf8456055, SUC_lexicon_tags_16 },
    { 0x4ba56050, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ac56059, SUC_lexicon_tags_29 },
    { 0xfda56059, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4aa5605c, SUC_lexicon_tags_25 },
    { 0x3965605d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e56063, SUC_lexicon_tags_16 },
    { 0x40656064, SUC_lexicon_tags_17 },
    { 0x9ea56065, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdd656067, SUC_lexicon_tags_7 },
    { 0xdde56068, SUC_lexicon_tags_12 },
    { 0xc4456068, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7965606d, SUC_lexicon_tags_30 },
    { 0x1365606e, SUC_lexicon_tags_32 },
    { 0x12e5606f, SUC_lexicon_tags_37 },
    { 0xed056070, SUC_lexicon_tags_58 },
    { 0x6ac56071, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc56074, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec56079, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6c05607b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7505607f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2656085, SUC_lexicon_tags_66 },
    { 0xd2e56086, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc925608a, SUC_lexicon_tags_84 },
    { 0x725608b, SUC_lexicon_tags_2 },
    { 0x72c5608c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57656092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5456094, SUC_lexicon_tags_27 },
    { 0x60656094, SUC_lexicon_tags_17 },
    { 0x80c56096, SUC_lexicon_tags_30 },
    { 0x1e456097, SUC_lexicon_tags_38 },
    { 0x69656098, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbae5609a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa85609f, SUC_lexicon_tags_87 },
    { 0xa9a560a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c560a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba560aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da560ae, SUC_lexicon_tags_13 },
    { 0x96e560ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a560b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x422560b5, SUC_lexicon_tags_25 },
    { 0xec0560b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x508560b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a560bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fc560bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x406560c0, SUC_lexicon_tags_10 },
    { 0x13c560c1, SUC_lexicon_tags_39 },
    { 0xe60560c1, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a560c6, SUC_lexicon_tags_43 },
    { 0xaa8560c7, SUC_lexicon_tags_12 },
    { 0x8b8560c8, SUC_lexicon_tags_57 },
    { 0xf3c560c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7b4560cb, SUC_lexicon_tags_14 },
    { 0x5fa560cb, SUC_lexicon_tags_17 },
    { 0x4a560cd, SUC_lexicon_tags_9 },
    { 0x862560cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90560d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca4560d2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7ce560d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e560d9, SUC_lexicon_tags_45 },
    { 0x646560da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658560df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88560e4, SUC_lexicon_tags_11 },
    { 0x7c0560e4, SUC_lexicon_tags_14 },
    { 0x58a560e6, SUC_lexicon_tags_14 },
    { 0x788560e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x628560e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x146560ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164560f2, SUC_lexicon_tags_12 },
    { 0x3b8560f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0560f6, SUC_lexicon_tags_45 },
    { 0xc8560f7, SUC_lexicon_tags_2 },
    { 0x5f8560f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2560fd, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56108, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2905610b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xda05610d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e56113, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8056117, SUC_lexicon_tags_11 },
    { 0xcca56118, SUC_lexicon_tags_3 },
    { 0xa0c56117, SUC_lexicon_tags_17 },
    { 0x7565611a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845611e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab656120, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93656126, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c256129, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7905612b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d25612f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44856132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b856135, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac5613b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1405613d, SUC_lexicon_tags_17 },
    { 0xeb45613e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa856142, SUC_lexicon_tags_47 },
    { 0x5e856142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x38256146, SUC_lexicon_tags_7 },
    { 0x10a56146, SUC_lexicon_tags_12 },
    { 0xed056148, SUC_lexicon_tags_11 },
    { 0xd5456149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a56150, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8456152, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa56155, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a56158, SUC_lexicon_tags_0 },
    { 0x29c56159, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8d85615b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63c5615d, SUC_lexicon_tags_38 },
    { 0x59e5615e, SUC_lexicon_tags_11 },
    { 0xb525615d, SUC_lexicon_tags_47 },
    { 0x8125615e, SUC_lexicon_tags_82 },
    { 0xfd056161, SUC_lexicon_tags_2 },
    { 0xa6e56161, SUC_lexicon_tags_17 },
    { 0x52656161, SUC_lexicon_tags_68 },
    { 0xa3e56161, SUC_lexicon_tags_7 },
    { 0x79c56165, SUC_lexicon_tags_43 },
    { 0xf2c56166, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa965616a, SUC_lexicon_tags_12 },
    { 0x3705616b, SUC_lexicon_tags_30 },
    { 0xef85616c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce5616e, SUC_lexicon_tags_19 },
    { 0x23c5616e, SUC_lexicon_tags_23 },
    { 0x605616e, SUC_lexicon_tags_8 },
    { 0x62856171, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd85617b, SUC_lexicon_tags_126 },
    { 0x11a5617c, SUC_lexicon_tags_22 },
    { 0x41e5617b, SUC_lexicon_tags_17 },
    { 0xe525617b, SUC_lexicon_tags_8 },
    { 0x4085617f, SUC_lexicon_tags_37 },
    { 0x2265617f, SUC_lexicon_tags_64 },
    { 0xaec56181, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde56184, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31c56186, SUC_lexicon_tags_37 },
    { 0x8e656187, SUC_lexicon_tags_33 },
    { 0xf5c56186, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb685618b, SUC_lexicon_tags_26 },
    { 0xe105618c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5618f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f856192, SUC_lexicon_tags_31 },
    { 0x12856193, SUC_lexicon_tags_108 },
    { 0x7856194, SUC_lexicon_tags_65 },
    { 0x54056195, SUC_lexicon_tags_107 },
    { 0xd3456196, SUC_lexicon_tags_23 },
    { 0xc0456196, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5965619b, SUC_lexicon_tags_0 },
    { 0x22a5619c, SUC_lexicon_tags_17 },
    { 0xb545619d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c561a3, SUC_lexicon_tags_19 },
    { 0x998561a3, SUC_lexicon_tags_48 },
    { 0x6a561a5, SUC_lexicon_tags_15 },
    { 0xe32561a6, SUC_lexicon_tags_11 },
    { 0x92e561a7, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x510561a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51e561ab, SUC_lexicon_tags_12 },
    { 0x426561ac, SUC_lexicon_tags_7 },
    { 0xb16561ac, SUC_lexicon_tags_218 },
    { 0x262561ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14561b2, SUC_lexicon_tags_27 },
    { 0x88561b3, SUC_lexicon_tags_17 },
    { 0x8fc561b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x188561b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc561bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0561bf, SUC_lexicon_tags_38 },
    { 0xab4561c0, SUC_lexicon_tags_34 },
    { 0x84a561c0, SUC_lexicon_tags_17 },
    { 0x33a561c0, SUC_lexicon_tags_1 },
    { 0xbba561c3, SUC_lexicon_tags_7 },
    { 0x3c4561c4, SUC_lexicon_tags_880 },
    { 0x8c0561c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3fa561c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66a561c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x404561cb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6561d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a561d3, SUC_lexicon_tags_33 },
    { 0x60a561d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b0561d6, SUC_lexicon_tags_2 },
    { 0xf96561d7, SUC_lexicon_tags_2 },
    { 0x33c561d7, SUC_lexicon_tags_16 },
    { 0xd5a561d6, SUC_lexicon_tags_2 },
    { 0x628561da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5cc561dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c561e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf82561e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46561ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746561f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcee561f6, SUC_lexicon_tags_28 },
    { 0xa46561f6, SUC_lexicon_tags_12 },
    { 0xf82561f8, SUC_lexicon_tags_19 },
    { 0x2f4561f9, SUC_lexicon_tags_107 },
    { 0x61a561f9, SUC_lexicon_tags_54 },
    { 0xf3e561fa, SUC_lexicon_tags_21 },
    { 0xf9a561f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e4561fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa256200, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca56204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0a56206, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa56210, SUC_lexicon_tags_45 },
    { 0x64056211, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3745621e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2456221, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11856226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5325622e, SUC_lexicon_tags_25 },
    { 0x6985622f, SUC_lexicon_tags_46 },
    { 0xaa85622f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4656232, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce56236, SUC_lexicon_tags_30 },
    { 0x79856237, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a5623e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xad256240, SUC_lexicon_tags_11 },
    { 0x8a456241, SUC_lexicon_tags_37 },
    { 0x52056242, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a56249, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d45624c, SUC_lexicon_tags_16 },
    { 0x8b65624d, SUC_lexicon_tags_63 },
    { 0x4a5624c, SUC_lexicon_tags_3 },
    { 0xb425624f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece56253, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7456256, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4856259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc445625f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b456264, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9845626e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcee56270, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a656272, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b256276, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x32656278, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e5627f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc256283, SUC_lexicon_tags_7 },
    { 0x1f456284, SUC_lexicon_tags_37 },
    { 0x58c56285, SUC_lexicon_tags_32 },
    { 0xd2456286, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x60056289, SUC_lexicon_tags_8 },
    { 0xa2e56289, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9cc5628c, SUC_lexicon_tags_12 },
    { 0x2105628c, SUC_lexicon_tags_7 },
    { 0xf7c5628c, SUC_lexicon_tags_2 },
    { 0xe6c5628f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31056291, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5656294, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7685629b, SUC_lexicon_tags_14 },
    { 0x9ae5629c, SUC_lexicon_tags_8 },
    { 0xa6e5629d, SUC_lexicon_tags_38 },
    { 0x84c5629e, SUC_lexicon_tags_2 },
    { 0xd465629f, SUC_lexicon_tags_17 },
    { 0xf38562a0, SUC_lexicon_tags_16 },
    { 0xd56562a0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf08562a3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x33e562a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa562a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8562b3, SUC_lexicon_tags_14 },
    { 0x2ac562b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a562b8, SUC_lexicon_tags_434 },
    { 0, NULL },
    { 0xf26562ba, SUC_lexicon_tags_0 },
    { 0xd66562bb, SUC_lexicon_tags_37 },
    { 0x244562bc, SUC_lexicon_tags_66 },
    { 0xa16562bd, SUC_lexicon_tags_25 },
    { 0x84a562be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262562c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x642562c8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea562d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6e562d2, SUC_lexicon_tags_27 },
    { 0xf4a562d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xede562d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x446562d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae562dd, SUC_lexicon_tags_11 },
    { 0x3d4562de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4562e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0562e4, SUC_lexicon_tags_1 },
    { 0x3bc562e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x588562e8, SUC_lexicon_tags_14 },
    { 0x918562e8, SUC_lexicon_tags_37 },
    { 0x806562e9, SUC_lexicon_tags_2 },
    { 0x6d4562eb, SUC_lexicon_tags_7 },
    { 0xc42562ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a0562ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da562f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a562f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2562fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaac562fc, SUC_lexicon_tags_12 },
    { 0x680562fd, SUC_lexicon_tags_25 },
    { 0xbac562fe, SUC_lexicon_tags_2 },
    { 0x80e562fd, SUC_lexicon_tags_2 },
    { 0x3ea562ff, SUC_lexicon_tags_17 },
    { 0x850562fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e56309, SUC_lexicon_tags_12 },
    { 0xc5e56309, SUC_lexicon_tags_25 },
    { 0xb645630a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92a5630d, SUC_lexicon_tags_48 },
    { 0x2765630e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5456311, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a656314, SUC_lexicon_tags_26 },
    { 0xab456315, SUC_lexicon_tags_37 },
    { 0x66656316, SUC_lexicon_tags_15 },
    { 0xc8856315, SUC_lexicon_tags_6 },
    { 0x16656318, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3256321, SUC_lexicon_tags_7 },
    { 0x3dc56321, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94256325, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa856328, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x98a5632a, SUC_lexicon_tags_7 },
    { 0x1e25632b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac56334, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x32656337, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe3c56339, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51e5633b, SUC_lexicon_tags_45 },
    { 0x4b85633b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce656344, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc56347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba5634a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba56350, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb85635f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d856362, SUC_lexicon_tags_15 },
    { 0x7ba56362, SUC_lexicon_tags_26 },
    { 0xbf056364, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a56367, SUC_lexicon_tags_82 },
    { 0xb0656368, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87656370, SUC_lexicon_tags_1 },
    { 0x5ca56370, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3e56373, SUC_lexicon_tags_7 },
    { 0xc8456374, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab056377, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1256379, SUC_lexicon_tags_29 },
    { 0xb4a5637a, SUC_lexicon_tags_0 },
    { 0x5365637b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb1a5637d, SUC_lexicon_tags_6 },
    { 0x80c5637d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e056380, SUC_lexicon_tags_4 },
    { 0x96056381, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x37e56383, SUC_lexicon_tags_11 },
    { 0x3d456384, SUC_lexicon_tags_19 },
    { 0xa7e56385, SUC_lexicon_tags_1076 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b856389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc5638d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34056395, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65639a, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x4b05639c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4445639f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656563ad, SUC_lexicon_tags_8 },
    { 0x238563ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x958563b0, SUC_lexicon_tags_32 },
    { 0x58a563b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfd2563b3, SUC_lexicon_tags_16 },
    { 0x496563b4, SUC_lexicon_tags_16 },
    { 0x834563b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c563b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a563bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x362563bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e563c0, SUC_lexicon_tags_12 },
    { 0x3fe563c0, SUC_lexicon_tags_29 },
    { 0xc80563c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3e563c4, SUC_lexicon_tags_2 },
    { 0xbde563c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0563c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c563cb, SUC_lexicon_tags_55 },
    { 0x224563cb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa32563ce, SUC_lexicon_tags_7 },
    { 0x338563cf, SUC_lexicon_tags_14 },
    { 0x344563d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x954563d2, SUC_lexicon_tags_0 },
    { 0xfe6563d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x562563d5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf24563d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa563e3, SUC_lexicon_tags_47 },
    { 0xe26563e3, SUC_lexicon_tags_49 },
    { 0xc2a563e5, SUC_lexicon_tags_0 },
    { 0xd4563e3, SUC_lexicon_tags_37 },
    { 0x4a6563e6, SUC_lexicon_tags_69 },
    { 0x214563e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6e6563ea, SUC_lexicon_tags_17 },
    { 0x4e0563eb, SUC_lexicon_tags_19 },
    { 0xb52563eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9a563ee, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3a4563f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12a563f2, SUC_lexicon_tags_27 },
    { 0xa22563f2, SUC_lexicon_tags_30 },
    { 0x1bc563f4, SUC_lexicon_tags_60 },
    { 0x64e563f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x284563f7, SUC_lexicon_tags_14 },
    { 0x3e6563f8, SUC_lexicon_tags_29 },
    { 0x53a563f9, SUC_lexicon_tags_1 },
    { 0xc7c563f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbba563fc, SUC_lexicon_tags_27 },
    { 0xf0e563fc, SUC_lexicon_tags_15 },
    { 0xaf0563fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6e656400, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97256403, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x47856406, SUC_lexicon_tags_19 },
    { 0xc7856406, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9025640d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b056410, SUC_lexicon_tags_19 },
    { 0x2bc56411, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c56416, SUC_lexicon_tags_27 },
    { 0xcfe56417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81a56419, SUC_lexicon_tags_12 },
    { 0xe745641a, SUC_lexicon_tags_17 },
    { 0xd05641b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3285641e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c656423, SUC_lexicon_tags_24 },
    { 0xcc256423, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x52a56426, SUC_lexicon_tags_63 },
    { 0x86c56426, SUC_lexicon_tags_7 },
    { 0xc4a56427, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23a5642a, SUC_lexicon_tags_37 },
    { 0xede5642a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c5642f, SUC_lexicon_tags_47 },
    { 0xa4c56430, SUC_lexicon_tags_15 },
    { 0x1f256430, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xece56434, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c56439, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1c45643b, SUC_lexicon_tags_48 },
    { 0x3ca5643b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85643f, SUC_lexicon_tags_7 },
    { 0xb2e5643f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25856446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ea56448, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac56452, SUC_lexicon_tags_47 },
    { 0x3d056453, SUC_lexicon_tags_33 },
    { 0x4c56454, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11456456, SUC_lexicon_tags_0 },
    { 0x1aa56457, SUC_lexicon_tags_48 },
    { 0x3da56458, SUC_lexicon_tags_2 },
    { 0xf3256457, SUC_lexicon_tags_19 },
    { 0x22056456, SUC_lexicon_tags_6 },
    { 0x1625645b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de5645e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c56464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56456469, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2905646c, SUC_lexicon_tags_1 },
    { 0xc8a5646d, SUC_lexicon_tags_14 },
    { 0xa5a5646c, SUC_lexicon_tags_32 },
    { 0xdf65646f, SUC_lexicon_tags_12 },
    { 0xeb256470, SUC_lexicon_tags_38 },
    { 0xd7e5646f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e56474, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91456477, SUC_lexicon_tags_0 },
    { 0x5fc56478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20256486, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61256489, SUC_lexicon_tags_27 },
    { 0xe0056489, SUC_lexicon_tags_19 },
    { 0x69a5648a, SUC_lexicon_tags_17 },
    { 0x40a5648a, SUC_lexicon_tags_0 },
    { 0x505648a, SUC_lexicon_tags_19 },
    { 0x6245648e, SUC_lexicon_tags_39 },
    { 0xcee5648e, SUC_lexicon_tags_1 },
    { 0x48056490, SUC_lexicon_tags_11 },
    { 0xbe5648a, SUC_lexicon_tags_37 },
    { 0x8f656492, SUC_lexicon_tags_17 },
    { 0x7fa56492, SUC_lexicon_tags_9 },
    { 0xc2e56494, SUC_lexicon_tags_1 },
    { 0xa3656495, SUC_lexicon_tags_3 },
    { 0x1a056495, SUC_lexicon_tags_12 },
    { 0xa7456494, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee5649d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738564a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46564a7, SUC_lexicon_tags_11 },
    { 0x16e564a8, SUC_lexicon_tags_12 },
    { 0xb2c564a8, SUC_lexicon_tags_25 },
    { 0x738564aa, SUC_lexicon_tags_16 },
    { 0x7cc564aa, SUC_lexicon_tags_7 },
    { 0x1ee564aa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0564b8, SUC_lexicon_tags_15 },
    { 0x5b6564b9, SUC_lexicon_tags_1 },
    { 0x976564b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b8564bc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4a8564be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeee564c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x848564c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc564c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828564ce, SUC_lexicon_tags_11 },
    { 0x2b8564cf, SUC_lexicon_tags_63 },
    { 0xe36564d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4564d3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7dc564d5, SUC_lexicon_tags_91 },
    { 0x25e564d6, SUC_lexicon_tags_37 },
    { 0xd74564d7, SUC_lexicon_tags_17 },
    { 0xb0a564d8, SUC_lexicon_tags_43 },
    { 0x448564d9, SUC_lexicon_tags_27 },
    { 0xe70564d7, SUC_lexicon_tags_19 },
    { 0xa04564d7, SUC_lexicon_tags_12 },
    { 0x78a564d7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540564e5, SUC_lexicon_tags_15 },
    { 0x986564e6, SUC_lexicon_tags_0 },
    { 0xf58564e7, SUC_lexicon_tags_8 },
    { 0x6cc564e8, SUC_lexicon_tags_17 },
    { 0xe90564e9, SUC_lexicon_tags_17 },
    { 0x5e0564e9, SUC_lexicon_tags_23 },
    { 0x352564eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362564ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa82564f1, SUC_lexicon_tags_17 },
    { 0x5ea564f2, SUC_lexicon_tags_0 },
    { 0xc94564f3, SUC_lexicon_tags_11 },
    { 0x406564f4, SUC_lexicon_tags_1 },
    { 0xe66564f1, SUC_lexicon_tags_60 },
    { 0x970564f6, SUC_lexicon_tags_10 },
    { 0xbf2564f3, SUC_lexicon_tags_25 },
    { 0x188564f8, SUC_lexicon_tags_7 },
    { 0x9b4564f9, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8564ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2c56501, SUC_lexicon_tags_26 },
    { 0x42c56502, SUC_lexicon_tags_45 },
    { 0x97e56503, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14a56505, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x94a56507, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45650a, SUC_lexicon_tags_12 },
    { 0xdcc5650b, SUC_lexicon_tags_15 },
    { 0x4e85650c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c56510, SUC_lexicon_tags_47 },
    { 0xafa56510, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1256517, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59c56519, SUC_lexicon_tags_17 },
    { 0xfee5651a, SUC_lexicon_tags_63 },
    { 0xd8456519, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe125651d, SUC_lexicon_tags_1 },
    { 0xcc45651e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10656526, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0a56528, SUC_lexicon_tags_21 },
    { 0xc2e56529, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8345652b, SUC_lexicon_tags_12 },
    { 0xd885652c, SUC_lexicon_tags_7 },
    { 0x31e5652d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c256531, SUC_lexicon_tags_17 },
    { 0xb7c56531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55a56534, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c5653d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ae5653f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a56542, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5056546, SUC_lexicon_tags_11 },
    { 0x43856546, SUC_lexicon_tags_77 },
    { 0x52856548, SUC_lexicon_tags_70 },
    { 0x22e56546, SUC_lexicon_tags_49 },
    { 0x55c56546, SUC_lexicon_tags_70 },
    { 0x45c5654b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd856552, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb8c56554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2a56556, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c5655e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf656560, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e56563, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78256568, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1065656c, SUC_lexicon_tags_1 },
    { 0x7f05656d, SUC_lexicon_tags_57 },
    { 0xe265656c, SUC_lexicon_tags_19 },
    { 0xebc5656c, SUC_lexicon_tags_10 },
    { 0x6a65656f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73456577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb456579, SUC_lexicon_tags_9 },
    { 0x5905657a, SUC_lexicon_tags_45 },
    { 0x71e5657b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa5657f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa56583, SUC_lexicon_tags_47 },
    { 0xc5a56584, SUC_lexicon_tags_49 },
    { 0x23a56585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86256587, SUC_lexicon_tags_43 },
    { 0x1fe56587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7205658b, SUC_lexicon_tags_25 },
    { 0x8945658b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a5658f, SUC_lexicon_tags_33 },
    { 0x4125658f, SUC_lexicon_tags_33 },
    { 0xe4656590, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e56596, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe56599, SUC_lexicon_tags_38 },
    { 0x5305659a, SUC_lexicon_tags_17 },
    { 0xc485659a, SUC_lexicon_tags_15 },
    { 0x48c5659c, SUC_lexicon_tags_25 },
    { 0xdb85659d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x182565a0, SUC_lexicon_tags_37 },
    { 0x86e565a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4565a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c565af, SUC_lexicon_tags_12 },
    { 0xcce565b0, SUC_lexicon_tags_25 },
    { 0x5d6565b1, SUC_lexicon_tags_11 },
    { 0x8e6565b1, SUC_lexicon_tags_9 },
    { 0x9f2565b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a565b7, SUC_lexicon_tags_15 },
    { 0xf0a565b8, SUC_lexicon_tags_11 },
    { 0x5e6565b8, SUC_lexicon_tags_12 },
    { 0xe2565ba, SUC_lexicon_tags_11 },
    { 0xdd8565bb, SUC_lexicon_tags_31 },
    { 0xa50565bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16565c0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112565c4, SUC_lexicon_tags_26 },
    { 0x6a6565c4, SUC_lexicon_tags_37 },
    { 0xb84565c6, SUC_lexicon_tags_64 },
    { 0x2f4565c7, SUC_lexicon_tags_2 },
    { 0xf32565c8, SUC_lexicon_tags_23 },
    { 0x53a565c7, SUC_lexicon_tags_28 },
    { 0xb2565ca, SUC_lexicon_tags_32 },
    { 0x55a565c7, SUC_lexicon_tags_37 },
    { 0x52565cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6565cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d2565d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c565d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a565dc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6de565de, SUC_lexicon_tags_15 },
    { 0x4a2565df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578565e3, SUC_lexicon_tags_135 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c565f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28565f8, SUC_lexicon_tags_12 },
    { 0x680565f9, SUC_lexicon_tags_7 },
    { 0x3d2565f8, SUC_lexicon_tags_37 },
    { 0x92565f8, SUC_lexicon_tags_17 },
    { 0x762565f8, SUC_lexicon_tags_2 },
    { 0xa12565f8, SUC_lexicon_tags_11 },
    { 0x63c565fe, SUC_lexicon_tags_61 },
    { 0x60565fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36e56601, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x42856603, SUC_lexicon_tags_92 },
    { 0xd1456604, SUC_lexicon_tags_17 },
    { 0x3c256605, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x56456607, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5665660b, SUC_lexicon_tags_17 },
    { 0x9665660c, SUC_lexicon_tags_0 },
    { 0x7105660c, SUC_lexicon_tags_65 },
    { 0x3d25660c, SUC_lexicon_tags_27 },
    { 0x5b65660b, SUC_lexicon_tags_49 },
    { 0xe8456610, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44a56612, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d256615, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x73656617, SUC_lexicon_tags_14 },
    { 0x47656618, SUC_lexicon_tags_35 },
    { 0xe4056619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf505661c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x4005661e, SUC_lexicon_tags_7 },
    { 0x3a45661e, SUC_lexicon_tags_81 },
    { 0xd9456620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10656623, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c456626, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29656628, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85662a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e5662d, SUC_lexicon_tags_11 },
    { 0xf325662d, SUC_lexicon_tags_19 },
    { 0x4465662e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdba56631, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63456635, SUC_lexicon_tags_27 },
    { 0x83c56636, SUC_lexicon_tags_37 },
    { 0xa2656637, SUC_lexicon_tags_7 },
    { 0xc9e56637, SUC_lexicon_tags_16 },
    { 0xc6856637, SUC_lexicon_tags_19 },
    { 0x64456636, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8625663d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2585663f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8656641, SUC_lexicon_tags_15 },
    { 0x7fe56642, SUC_lexicon_tags_37 },
    { 0x10456641, SUC_lexicon_tags_15 },
    { 0x95456644, SUC_lexicon_tags_25 },
    { 0x36856644, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e5664a, SUC_lexicon_tags_117 },
    { 0xdee5664b, SUC_lexicon_tags_10 },
    { 0xf3c5664a, SUC_lexicon_tags_1 },
    { 0x70e5664a, SUC_lexicon_tags_14 },
    { 0x1745664e, SUC_lexicon_tags_17 },
    { 0x6585664f, SUC_lexicon_tags_45 },
    { 0xbb856650, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8856652, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23056655, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce5665e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa56663, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3145666a, SUC_lexicon_tags_27 },
    { 0xeda5666b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5666e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5c56670, SUC_lexicon_tags_29 },
    { 0x9ea56671, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0056675, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e5667b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd465667d, SUC_lexicon_tags_14 },
    { 0x5205667d, SUC_lexicon_tags_7 },
    { 0x105667f, SUC_lexicon_tags_7 },
    { 0x90a56680, SUC_lexicon_tags_1 },
    { 0x53a56681, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78c56683, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc4656685, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5905668c, SUC_lexicon_tags_22 },
    { 0x16e5668d, SUC_lexicon_tags_33 },
    { 0xeb45668e, SUC_lexicon_tags_0 },
    { 0x9745668e, SUC_lexicon_tags_19 },
    { 0xb6256690, SUC_lexicon_tags_37 },
    { 0x7856691, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0656694, SUC_lexicon_tags_25 },
    { 0xa0e56695, SUC_lexicon_tags_2 },
    { 0xcce56695, SUC_lexicon_tags_30 },
    { 0xe4856697, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d05669a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda65669d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e566a0, SUC_lexicon_tags_2 },
    { 0xd2e566a1, SUC_lexicon_tags_9 },
    { 0x8a8566a1, SUC_lexicon_tags_25 },
    { 0x2a566a3, SUC_lexicon_tags_28 },
    { 0x6566a4, SUC_lexicon_tags_39 },
    { 0xacc566a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa566ad, SUC_lexicon_tags_49 },
    { 0x5aa566ad, SUC_lexicon_tags_28 },
    { 0x676566ad, SUC_lexicon_tags_1 },
    { 0xa4c566af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac566b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa74566b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e566b9, SUC_lexicon_tags_17 },
    { 0xb9e566b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36566be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44566cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x270566cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e566d3, SUC_lexicon_tags_10 },
    { 0xc42566d4, SUC_lexicon_tags_30 },
    { 0x66c566d4, SUC_lexicon_tags_17 },
    { 0x2fc566d4, SUC_lexicon_tags_17 },
    { 0xb80566d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x884566d9, SUC_lexicon_tags_69 },
    { 0x3a566da, SUC_lexicon_tags_25 },
    { 0x6c4566da, SUC_lexicon_tags_12 },
    { 0x91e566dc, SUC_lexicon_tags_37 },
    { 0x856566dc, SUC_lexicon_tags_82 },
    { 0xe78566de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8566e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad8566e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c566ef, SUC_lexicon_tags_16 },
    { 0xe0e566f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc566f5, SUC_lexicon_tags_1 },
    { 0x820566f6, SUC_lexicon_tags_2 },
    { 0xc2c566f7, SUC_lexicon_tags_14 },
    { 0xb62566f8, SUC_lexicon_tags_68 },
    { 0xa10566f9, SUC_lexicon_tags_11 },
    { 0x760566fa, SUC_lexicon_tags_10 },
    { 0xb02566fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a566fe, SUC_lexicon_tags_46 },
    { 0x2a2566ff, SUC_lexicon_tags_11 },
    { 0x93456700, SUC_lexicon_tags_12 },
    { 0x6d856700, SUC_lexicon_tags_61 },
    { 0xa3256701, SUC_lexicon_tags_16 },
    { 0x8856703, SUC_lexicon_tags_2 },
    { 0x73e56703, SUC_lexicon_tags_11 },
    { 0x8f2566fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb456708, SUC_lexicon_tags_22 },
    { 0x6a056708, SUC_lexicon_tags_14 },
    { 0x94c56709, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b05670d, SUC_lexicon_tags_17 },
    { 0xdea5670e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c456716, SUC_lexicon_tags_30 },
    { 0xab856716, SUC_lexicon_tags_37 },
    { 0x2e256718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce5671c, SUC_lexicon_tags_37 },
    { 0x2545671d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcb65671f, SUC_lexicon_tags_23 },
    { 0xb5856720, SUC_lexicon_tags_23 },
    { 0x5885671f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c56729, SUC_lexicon_tags_12 },
    { 0x4d856729, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb3a5672c, SUC_lexicon_tags_33 },
    { 0xb4c5672d, SUC_lexicon_tags_9 },
    { 0x4185672e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x93056731, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3056733, SUC_lexicon_tags_7 },
    { 0x82a56733, SUC_lexicon_tags_27 },
    { 0xf4456734, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c85673c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b45673e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14c56740, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7cc56742, SUC_lexicon_tags_1 },
    { 0xbda56743, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xff656745, SUC_lexicon_tags_87 },
    { 0x72856745, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb685674b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc145674e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaa56750, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e56756, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab05675f, SUC_lexicon_tags_68 },
    { 0xb6656760, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e56766, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9025676e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2b256770, SUC_lexicon_tags_37 },
    { 0x4bc56771, SUC_lexicon_tags_2 },
    { 0x50e56772, SUC_lexicon_tags_1 },
    { 0x4e056772, SUC_lexicon_tags_28 },
    { 0x67256773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6e56776, SUC_lexicon_tags_48 },
    { 0x13856777, SUC_lexicon_tags_71 },
    { 0x87c56778, SUC_lexicon_tags_12 },
    { 0x6e056776, SUC_lexicon_tags_1 },
    { 0xe4a5677a, SUC_lexicon_tags_47 },
    { 0xeac5677b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7625677d, SUC_lexicon_tags_37 },
    { 0x9165677e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee56781, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x98456784, SUC_lexicon_tags_15 },
    { 0xf6c56784, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x96456787, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac05678b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9056796, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca05679a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf65679f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a6567a1, SUC_lexicon_tags_38 },
    { 0x720567a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c567a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702567ab, SUC_lexicon_tags_32 },
    { 0x1c6567ab, SUC_lexicon_tags_27 },
    { 0x7d6567ac, SUC_lexicon_tags_15 },
    { 0xcba567ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a567b2, SUC_lexicon_tags_129 },
    { 0x974567b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4567b8, SUC_lexicon_tags_25 },
    { 0xab0567b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x940567bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae567bd, SUC_lexicon_tags_15 },
    { 0x852567be, SUC_lexicon_tags_17 },
    { 0xd86567bf, SUC_lexicon_tags_33 },
    { 0xbc8567c0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x69e567c2, SUC_lexicon_tags_11 },
    { 0x586567c2, SUC_lexicon_tags_82 },
    { 0x1fe567c2, SUC_lexicon_tags_43 },
    { 0x16c567c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x704567c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x374567c9, SUC_lexicon_tags_1 },
    { 0x24a567ca, SUC_lexicon_tags_46 },
    { 0xc10567cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc567ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98a567d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544567d8, SUC_lexicon_tags_27 },
    { 0x45e567d9, SUC_lexicon_tags_30 },
    { 0xb3c567da, SUC_lexicon_tags_37 },
    { 0x99e567d9, SUC_lexicon_tags_84 },
    { 0xb46567dc, SUC_lexicon_tags_77 },
    { 0xec8567db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x384567e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0567e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de567e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88567ee, SUC_lexicon_tags_8 },
    { 0x14c567ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94567f3, SUC_lexicon_tags_49 },
    { 0xe3a567f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd90567f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x776567f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xabc567fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1d6567fc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22056803, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35256805, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4545680b, SUC_lexicon_tags_68 },
    { 0x8d65680c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc5680f, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a56816, SUC_lexicon_tags_25 },
    { 0x75856816, SUC_lexicon_tags_7 },
    { 0x67c56816, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4065681a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb7e5681c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3985681e, SUC_lexicon_tags_33 },
    { 0xe685681e, SUC_lexicon_tags_59 },
    { 0x13a56820, SUC_lexicon_tags_3 },
    { 0x26456821, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda56826, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee05682a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5fc5682c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3056830, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac56834, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e56839, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3025683c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b656844, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa2056846, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12856849, SUC_lexicon_tags_17 },
    { 0x8f456849, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c5684c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca5684f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x74c56851, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1c56853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a56858, SUC_lexicon_tags_37 },
    { 0x68856859, SUC_lexicon_tags_37 },
    { 0xae56859, SUC_lexicon_tags_27 },
    { 0xb245685a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf745685f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xda56861, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac56864, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xff656866, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbba56868, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8365686a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x86e5686c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6345686e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a456874, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7256877, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a25687a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c5687e, SUC_lexicon_tags_1 },
    { 0x54a5687f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3456883, SUC_lexicon_tags_47 },
    { 0x76656884, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc56887, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a5688a, SUC_lexicon_tags_46 },
    { 0x3865688a, SUC_lexicon_tags_37 },
    { 0xf145688c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68256890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce56893, SUC_lexicon_tags_17 },
    { 0xa1256894, SUC_lexicon_tags_55 },
    { 0x50856894, SUC_lexicon_tags_12 },
    { 0x2d656896, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b45689f, SUC_lexicon_tags_26 },
    { 0x16a5689f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318568a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x640568a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x360568aa, SUC_lexicon_tags_1 },
    { 0xeda568ab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x44a568ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5f8568af, SUC_lexicon_tags_108 },
    { 0x74e568af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc568b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c568b6, SUC_lexicon_tags_63 },
    { 0x72568b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e568be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd0568c0, SUC_lexicon_tags_66 },
    { 0x6b0568c1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8a2568c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788568ca, SUC_lexicon_tags_27 },
    { 0x564568cb, SUC_lexicon_tags_47 },
    { 0x214568cc, SUC_lexicon_tags_7 },
    { 0xd64568cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74568d0, SUC_lexicon_tags_11 },
    { 0xb0a568d1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22568d4, SUC_lexicon_tags_11 },
    { 0x5b4568d5, SUC_lexicon_tags_32 },
    { 0xf56568d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdae568d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02568dd, SUC_lexicon_tags_30 },
    { 0xfd2568dd, SUC_lexicon_tags_25 },
    { 0x416568df, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x33a568e1, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0568e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x694568ea, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x64c568ec, SUC_lexicon_tags_8 },
    { 0x3e568ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498568f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6568f4, SUC_lexicon_tags_16 },
    { 0x90e568f5, SUC_lexicon_tags_12 },
    { 0xae6568f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa84568f8, SUC_lexicon_tags_7 },
    { 0x4dc568f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ce568fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2056900, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x22c56902, SUC_lexicon_tags_19 },
    { 0x67656903, SUC_lexicon_tags_63 },
    { 0x29c56903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3256906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c56909, SUC_lexicon_tags_11 },
    { 0xbec56909, SUC_lexicon_tags_2 },
    { 0x20a56909, SUC_lexicon_tags_14 },
    { 0x7425690c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf505690f, SUC_lexicon_tags_17 },
    { 0xe8e56910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b656912, SUC_lexicon_tags_8 },
    { 0x4a656913, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee256918, SUC_lexicon_tags_235 },
    { 0xef256919, SUC_lexicon_tags_35 },
    { 0x28856919, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a5691f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e856923, SUC_lexicon_tags_45 },
    { 0xeb656924, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe2256926, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7656929, SUC_lexicon_tags_12 },
    { 0x2825692a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e5692d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe125692f, SUC_lexicon_tags_25 },
    { 0x60456930, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6256933, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b456935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a56938, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec5693c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34a5693e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c5694e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x66256951, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da56956, SUC_lexicon_tags_27 },
    { 0x4656956, SUC_lexicon_tags_0 },
    { 0xa4856958, SUC_lexicon_tags_27 },
    { 0x7bc56958, SUC_lexicon_tags_11 },
    { 0x32e56958, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda65695f, SUC_lexicon_tags_35 },
    { 0x6da5695f, SUC_lexicon_tags_38 },
    { 0xec456960, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca256963, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee56966, SUC_lexicon_tags_95 },
    { 0xd9a56967, SUC_lexicon_tags_15 },
    { 0x99c56967, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c5696c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3765696e, SUC_lexicon_tags_49 },
    { 0x45696f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76a56971, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1656973, SUC_lexicon_tags_25 },
    { 0xb8656973, SUC_lexicon_tags_17 },
    { 0xed656973, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d256979, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9405697d, SUC_lexicon_tags_45 },
    { 0x9945697e, SUC_lexicon_tags_17 },
    { 0x9d25697d, SUC_lexicon_tags_7 },
    { 0xcbc56980, SUC_lexicon_tags_6 },
    { 0xff456981, SUC_lexicon_tags_10 },
    { 0xfb656982, SUC_lexicon_tags_84 },
    { 0xeb456981, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a456987, SUC_lexicon_tags_61 },
    { 0x7a456988, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8785698b, SUC_lexicon_tags_49 },
    { 0x9f05698c, SUC_lexicon_tags_60 },
    { 0xcd05698d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeee5698f, SUC_lexicon_tags_11 },
    { 0x9d05698f, SUC_lexicon_tags_8 },
    { 0x12456990, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c56994, SUC_lexicon_tags_12 },
    { 0xcec56995, SUC_lexicon_tags_11 },
    { 0x75856996, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9256999, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a5699c, SUC_lexicon_tags_37 },
    { 0xdbc5699d, SUC_lexicon_tags_2 },
    { 0x1f25699e, SUC_lexicon_tags_57 },
    { 0x7425699e, SUC_lexicon_tags_37 },
    { 0xb98569a0, SUC_lexicon_tags_0 },
    { 0x8fa569a0, SUC_lexicon_tags_26 },
    { 0x42a5699c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ae569a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0569a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16569b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30569b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x418569b5, SUC_lexicon_tags_15 },
    { 0x168569b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeda569b8, SUC_lexicon_tags_0 },
    { 0x678569b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x62569bb, SUC_lexicon_tags_47 },
    { 0x232569bb, SUC_lexicon_tags_11 },
    { 0xdba569bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e569c0, SUC_lexicon_tags_48 },
    { 0x642569c0, SUC_lexicon_tags_25 },
    { 0x750569c2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a569c5, SUC_lexicon_tags_60 },
    { 0x88c569c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4569c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e569cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x30569d2, SUC_lexicon_tags_29 },
    { 0x76a569d2, SUC_lexicon_tags_15 },
    { 0xd32569d2, SUC_lexicon_tags_68 },
    { 0x43a569d3, SUC_lexicon_tags_87 },
    { 0x930569d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08569df, SUC_lexicon_tags_18 },
    { 0x292569df, SUC_lexicon_tags_17 },
    { 0x1e6569e0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06569e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb24569e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x508569ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc569ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2569f3, SUC_lexicon_tags_11 },
    { 0x4a6569f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2569fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c569fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b4569ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7656a02, SUC_lexicon_tags_19 },
    { 0xda056a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48256a05, SUC_lexicon_tags_351 },
    { 0x6f456a06, SUC_lexicon_tags_4 },
    { 0x14456a07, SUC_lexicon_tags_11 },
    { 0x31856a08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5656a0f, SUC_lexicon_tags_37 },
    { 0x24e56a0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ca56a12, SUC_lexicon_tags_25 },
    { 0xa8e56a13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9856a15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82256a18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9056a1c, SUC_lexicon_tags_6 },
    { 0x18656a1d, SUC_lexicon_tags_84 },
    { 0xa9056a1e, SUC_lexicon_tags_49 },
    { 0x56a1f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4a656a21, SUC_lexicon_tags_2 },
    { 0xa3856a22, SUC_lexicon_tags_12 },
    { 0x19456a23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc56a28, SUC_lexicon_tags_11 },
    { 0xc3056a29, SUC_lexicon_tags_17 },
    { 0x1c056a2a, SUC_lexicon_tags_49 },
    { 0xff056a2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4056a2d, SUC_lexicon_tags_37 },
    { 0xf2256a2e, SUC_lexicon_tags_29 },
    { 0x6e56a2f, SUC_lexicon_tags_12 },
    { 0x59056a2e, SUC_lexicon_tags_16 },
    { 0x8e456a31, SUC_lexicon_tags_15 },
    { 0x8d856a31, SUC_lexicon_tags_15 },
    { 0xf9656a30, SUC_lexicon_tags_14 },
    { 0xce56a34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b656a36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda56a39, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0x2ae56a3b, SUC_lexicon_tags_14 },
    { 0x47c56a3c, SUC_lexicon_tags_37 },
    { 0xf3a56a3d, SUC_lexicon_tags_8 },
    { 0x11c56a3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61856a44, SUC_lexicon_tags_11 },
    { 0x21e56a45, SUC_lexicon_tags_32 },
    { 0x7d656a45, SUC_lexicon_tags_26 },
    { 0x3b856a47, SUC_lexicon_tags_12 },
    { 0x72056a48, SUC_lexicon_tags_66 },
    { 0x4f256a49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c256a4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6056a54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa056a57, SUC_lexicon_tags_25 },
    { 0xc256a57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a56a5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73456a5f, SUC_lexicon_tags_1 },
    { 0x47a56a60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41456a6c, SUC_lexicon_tags_12 },
    { 0xce856a6d, SUC_lexicon_tags_65 },
    { 0x6da56a6e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa56a75, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43656a7a, SUC_lexicon_tags_11 },
    { 0xe8256a7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c56a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb056a83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81256a86, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e56a8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57c56a8c, SUC_lexicon_tags_0 },
    { 0x84a56a8c, SUC_lexicon_tags_38 },
    { 0x75856a8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a56a92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ae56a94, SUC_lexicon_tags_11 },
    { 0x91656a95, SUC_lexicon_tags_12 },
    { 0x13e56a95, SUC_lexicon_tags_1 },
    { 0xba56a94, SUC_lexicon_tags_49 },
    { 0x51856a94, SUC_lexicon_tags_11 },
    { 0x93a56a99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99656a9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec56aa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9456aa9, SUC_lexicon_tags_1 },
    { 0xfb456aaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9056aad, SUC_lexicon_tags_47 },
    { 0xc5c56aae, SUC_lexicon_tags_8 },
    { 0xb9c56aaf, SUC_lexicon_tags_47 },
    { 0x68c56aad, SUC_lexicon_tags_17 },
    { 0xad056ab1, SUC_lexicon_tags_60 },
    { 0x9a256ab2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0056ab8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e56abe, SUC_lexicon_tags_48 },
    { 0x74a56abf, SUC_lexicon_tags_9 },
    { 0xd9456ac0, SUC_lexicon_tags_45 },
    { 0x2a056ac1, SUC_lexicon_tags_60 },
    { 0x16656ac2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78856ace, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6c456ad0, SUC_lexicon_tags_11 },
    { 0x29a56ad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf456ad4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de56ada, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ba56adc, SUC_lexicon_tags_7 },
    { 0xbd856adc, SUC_lexicon_tags_47 },
    { 0x68a56ade, SUC_lexicon_tags_11 },
    { 0x88056ade, SUC_lexicon_tags_46 },
    { 0xce56ae0, SUC_lexicon_tags_58 },
    { 0x8a856ae1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e56ae7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c56aec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c56af7, SUC_lexicon_tags_38 },
    { 0xab056af7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae56afb, SUC_lexicon_tags_12 },
    { 0x15e56afc, SUC_lexicon_tags_29 },
    { 0x82456afd, SUC_lexicon_tags_35 },
    { 0x2a256afc, SUC_lexicon_tags_7 },
    { 0x25056aff, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xd2a56b01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c56b06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x45856b09, SUC_lexicon_tags_37 },
    { 0xc4e56b09, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1456b11, SUC_lexicon_tags_1 },
    { 0x94c56b12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed856b15, SUC_lexicon_tags_38 },
    { 0x6de56b16, SUC_lexicon_tags_35 },
    { 0x92056b16, SUC_lexicon_tags_22 },
    { 0x13a56b17, SUC_lexicon_tags_38 },
    { 0x76a56b19, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a56b1c, SUC_lexicon_tags_25 },
    { 0x6a256b1d, SUC_lexicon_tags_37 },
    { 0x2e056b1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83856b26, SUC_lexicon_tags_77 },
    { 0x2f056b27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d056b2b, SUC_lexicon_tags_12 },
    { 0xb8056b2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62656b30, SUC_lexicon_tags_7 },
    { 0x18a56b31, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfe056b33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7056b36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2256b44, SUC_lexicon_tags_55 },
    { 0xd2a56b45, SUC_lexicon_tags_46 },
    { 0x8aa56b45, SUC_lexicon_tags_33 },
    { 0xb5a56b45, SUC_lexicon_tags_25 },
    { 0x18056b48, SUC_lexicon_tags_0 },
    { 0xf9656b49, SUC_lexicon_tags_37 },
    { 0xce856b46, SUC_lexicon_tags_2 },
    { 0x86e56b46, SUC_lexicon_tags_2 },
    { 0x3f456b4c, SUC_lexicon_tags_17 },
    { 0xde656b4b, SUC_lexicon_tags_21 },
    { 0xfb256b4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x12a56b50, SUC_lexicon_tags_91 },
    { 0x66e56b51, SUC_lexicon_tags_28 },
    { 0xbe56b52, SUC_lexicon_tags_31 },
    { 0xc8e56b53, SUC_lexicon_tags_25 },
    { 0x58856b53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1856b56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3656b58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13456b62, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x45456b65, SUC_lexicon_tags_15 },
    { 0xa9656b66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7656b6a, SUC_lexicon_tags_17 },
    { 0x45656b6b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e56b6f, SUC_lexicon_tags_63 },
    { 0xdf056b6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdee56b72, SUC_lexicon_tags_0 },
    { 0x24e56b72, SUC_lexicon_tags_27 },
    { 0xd6e56b73, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x62e56b76, SUC_lexicon_tags_16 },
    { 0x6256b77, SUC_lexicon_tags_1 },
    { 0xc0e56b78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x13456b7b, SUC_lexicon_tags_19 },
    { 0xe5e56b7b, SUC_lexicon_tags_47 },
    { 0x49e56b7b, SUC_lexicon_tags_8 },
    { 0xde856b7e, SUC_lexicon_tags_15 },
    { 0xf3056b7f, SUC_lexicon_tags_25 },
    { 0x82c56b80, SUC_lexicon_tags_1 },
    { 0x9f456b81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa656b88, SUC_lexicon_tags_60 },
    { 0x7c056b88, SUC_lexicon_tags_35 },
    { 0xc056b8a, SUC_lexicon_tags_61 },
    { 0xd0a56b88, SUC_lexicon_tags_19 },
    { 0x90256b88, SUC_lexicon_tags_0 },
    { 0x78a56b88, SUC_lexicon_tags_1 },
    { 0xb3e56b8e, SUC_lexicon_tags_17 },
    { 0x50056b8f, SUC_lexicon_tags_25 },
    { 0xcea56b90, SUC_lexicon_tags_30 },
    { 0x57856b8e, SUC_lexicon_tags_179 },
    { 0x66a56b92, SUC_lexicon_tags_2 },
    { 0xb2e56b92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14056b95, SUC_lexicon_tags_11 },
    { 0x12e56b96, SUC_lexicon_tags_7 },
    { 0x6f856b97, SUC_lexicon_tags_11 },
    { 0x4f856b98, SUC_lexicon_tags_21 },
    { 0x29656b95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x51456b9b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba56ba4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75856ba7, SUC_lexicon_tags_57 },
    { 0x64e56ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a56bae, SUC_lexicon_tags_31 },
    { 0x2c456bae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x13256bb2, SUC_lexicon_tags_84 },
    { 0x6f656bb3, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae56bb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a56bbc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93856bbf, SUC_lexicon_tags_1 },
    { 0x7dc56bbf, SUC_lexicon_tags_17 },
    { 0x38656bc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e456bc3, SUC_lexicon_tags_68 },
    { 0x41c56bc3, SUC_lexicon_tags_23 },
    { 0xe1e56bc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81c56bc7, SUC_lexicon_tags_26 },
    { 0xd0256bc8, SUC_lexicon_tags_108 },
    { 0x32056bc8, SUC_lexicon_tags_0 },
    { 0xa8056bc8, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22656bd8, SUC_lexicon_tags_45 },
    { 0x1e856bd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e56bdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56656bde, SUC_lexicon_tags_37 },
    { 0x3da56bdf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c56be2, SUC_lexicon_tags_17 },
    { 0xed856be3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1056be9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ae56beb, SUC_lexicon_tags_12 },
    { 0x49256bec, SUC_lexicon_tags_60 },
    { 0x2d656bed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a56bf0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e56bf8, SUC_lexicon_tags_11 },
    { 0xa2e56bf8, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xbfc56bfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e256bfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x94256c01, SUC_lexicon_tags_63 },
    { 0x74856c02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe56c06, SUC_lexicon_tags_19 },
    { 0x9ca56c07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1856c0d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa056c11, SUC_lexicon_tags_37 },
    { 0xc2656c12, SUC_lexicon_tags_84 },
    { 0xac256c13, SUC_lexicon_tags_14 },
    { 0x20056c11, SUC_lexicon_tags_12 },
    { 0xa8856c11, SUC_lexicon_tags_33 },
    { 0xd1656c14, SUC_lexicon_tags_8 },
    { 0x82e56c17, SUC_lexicon_tags_17 },
    { 0xc2256c18, SUC_lexicon_tags_12 },
    { 0x87856c19, SUC_lexicon_tags_62 },
    { 0x9aa56c18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3056c25, SUC_lexicon_tags_113 },
    { 0x3b256c26, SUC_lexicon_tags_17 },
    { 0x42056c27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66e56c29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3fc56c2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5256c31, SUC_lexicon_tags_25 },
    { 0xfc856c32, SUC_lexicon_tags_26 },
    { 0xeda56c33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87a56c35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x74456c38, SUC_lexicon_tags_4 },
    { 0x8f656c39, SUC_lexicon_tags_47 },
    { 0xb8e56c38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7656c3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6456c43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd656c45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28056c47, SUC_lexicon_tags_15 },
    { 0x87256c48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc456c4b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e56c4e, SUC_lexicon_tags_0 },
    { 0x11656c4f, SUC_lexicon_tags_35 },
    { 0x5c856c4f, SUC_lexicon_tags_17 },
    { 0x81456c51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6456c57, SUC_lexicon_tags_17 },
    { 0x9856c58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78456c5a, SUC_lexicon_tags_37 },
    { 0x83256c5b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f656c5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a656c60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x45056c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ce56c65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a56c6a, SUC_lexicon_tags_43 },
    { 0xcec56c6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe56c75, SUC_lexicon_tags_33 },
    { 0x1fc56c75, SUC_lexicon_tags_9 },
    { 0x65056c77, SUC_lexicon_tags_87 },
    { 0x3a856c78, SUC_lexicon_tags_43 },
    { 0xd4056c79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d056c7c, SUC_lexicon_tags_48 },
    { 0x6ae56c7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec56c80, SUC_lexicon_tags_31 },
    { 0x1da56c81, SUC_lexicon_tags_30 },
    { 0x7e456c81, SUC_lexicon_tags_2 },
    { 0x2d856c83, SUC_lexicon_tags_0 },
    { 0x7c856c84, SUC_lexicon_tags_16 },
    { 0x1ca56c84, SUC_lexicon_tags_25 },
    { 0x18456c83, SUC_lexicon_tags_37 },
    { 0x83c56c87, SUC_lexicon_tags_1085 },
    { 0x20656c88, SUC_lexicon_tags_26 },
    { 0x72a56c85, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e56c8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa56c92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb056c95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b856c98, SUC_lexicon_tags_25 },
    { 0xc8456c98, SUC_lexicon_tags_15 },
    { 0xcb256c99, SUC_lexicon_tags_68 },
    { 0x9256c9b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1f856c9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c056ca1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1bc56ca3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6e456ca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29256ca8, SUC_lexicon_tags_30 },
    { 0x8fe56ca9, SUC_lexicon_tags_45 },
    { 0xcce56ca8, SUC_lexicon_tags_1 },
    { 0xc8c56ca9, SUC_lexicon_tags_12 },
    { 0xf9c56ca9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf856cb1, SUC_lexicon_tags_45 },
    { 0xc056cb2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x44a56cb4, SUC_lexicon_tags_15 },
    { 0x2e56cb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e56cb9, SUC_lexicon_tags_37 },
    { 0x79456cba, SUC_lexicon_tags_11 },
    { 0xbe856cba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf656cbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee56cc1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52256cc5, SUC_lexicon_tags_3 },
    { 0x27a56cc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71056cc9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac56ccc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8256cd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78856cd7, SUC_lexicon_tags_47 },
    { 0x7d856cd8, SUC_lexicon_tags_45 },
    { 0x38e56cd8, SUC_lexicon_tags_8 },
    { 0x84c56cda, SUC_lexicon_tags_2 },
    { 0x40256cdb, SUC_lexicon_tags_25 },
    { 0xf9c56cdc, SUC_lexicon_tags_14 },
    { 0xe56cdd, SUC_lexicon_tags_45 },
    { 0xebe56cde, SUC_lexicon_tags_9 },
    { 0xdae56cda, SUC_lexicon_tags_17 },
    { 0xfa56ce0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2256ce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1056cf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d456cf3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc0256cf5, SUC_lexicon_tags_37 },
    { 0x64e56cf6, SUC_lexicon_tags_17 },
    { 0x68a56cf7, SUC_lexicon_tags_68 },
    { 0x9ec56cf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f456cff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb856d01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38656d03, SUC_lexicon_tags_11 },
    { 0x90c56d04, SUC_lexicon_tags_17 },
    { 0xc6856d05, SUC_lexicon_tags_14 },
    { 0x8a456d06, SUC_lexicon_tags_11 },
    { 0x75256d06, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77056d0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e56d12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7456d14, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5856d19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0056d1c, SUC_lexicon_tags_1 },
    { 0xb5856d1d, SUC_lexicon_tags_43 },
    { 0x8bc56d1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x64456d20, SUC_lexicon_tags_79 },
    { 0x57256d21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5856d25, SUC_lexicon_tags_43 },
    { 0x46e56d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a56d2b, SUC_lexicon_tags_37 },
    { 0x53456d2b, SUC_lexicon_tags_37 },
    { 0x96c56d2b, SUC_lexicon_tags_47 },
    { 0x84656d2e, SUC_lexicon_tags_17 },
    { 0x656d2f, SUC_lexicon_tags_16 },
    { 0x72856d30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2e56d32, SUC_lexicon_tags_27 },
    { 0xb8656d32, SUC_lexicon_tags_3 },
    { 0x1ce56d34, SUC_lexicon_tags_1 },
    { 0xcd856d34, SUC_lexicon_tags_92 },
    { 0x32a56d33, SUC_lexicon_tags_1 },
    { 0xd3e56d36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0056d39, SUC_lexicon_tags_63 },
    { 0x4cc56d3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9c56d3c, SUC_lexicon_tags_7 },
    { 0xee056d3d, SUC_lexicon_tags_12 },
    { 0xf856d3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10c56d40, SUC_lexicon_tags_37 },
    { 0x19856d41, SUC_lexicon_tags_61 },
    { 0x1cc56d41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ec56d44, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x78c56d46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa256d48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa9a56d4a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e656d4d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c56d51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11256d53, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaa456d55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a856d59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce056d5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7a56d60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb456d62, SUC_lexicon_tags_14 },
    { 0xeee56d63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe56d69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90a56d6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc856d6e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x36056d70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e56d77, SUC_lexicon_tags_25 },
    { 0x51456d78, SUC_lexicon_tags_56 },
    { 0xb0256d77, SUC_lexicon_tags_37 },
    { 0x2fe56d7a, SUC_lexicon_tags_37 },
    { 0x3a656d7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b856d81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7456d86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe56d8a, SUC_lexicon_tags_99 },
    { 0x75056d8b, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c56d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1056d91, SUC_lexicon_tags_38 },
    { 0x63456d92, SUC_lexicon_tags_1 },
    { 0x38456d92, SUC_lexicon_tags_9 },
    { 0x18256d94, SUC_lexicon_tags_25 },
    { 0x59256d95, SUC_lexicon_tags_1 },
    { 0xea056d95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa56d99, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x65856d9c, SUC_lexicon_tags_1 },
    { 0x8256d9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b656d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed056da2, SUC_lexicon_tags_37 },
    { 0x5ba56da3, SUC_lexicon_tags_11 },
    { 0x7d656da3, SUC_lexicon_tags_39 },
    { 0x2c56da5, SUC_lexicon_tags_7 },
    { 0x9de56da6, SUC_lexicon_tags_37 },
    { 0x4fa56da7, SUC_lexicon_tags_33 },
    { 0xe6856da8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96c56daa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x89456dac, SUC_lexicon_tags_27 },
    { 0x13c56dad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1056db5, SUC_lexicon_tags_27 },
    { 0xcaa56db6, SUC_lexicon_tags_37 },
    { 0x50c56db6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc656db9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77056dbd, SUC_lexicon_tags_7 },
    { 0xdaa56dbe, SUC_lexicon_tags_37 },
    { 0xf1e56dbf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x98a56dc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6e56dc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c256dcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a56dcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x39456dd2, SUC_lexicon_tags_31 },
    { 0x42256dd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7056dd6, SUC_lexicon_tags_31 },
    { 0xbd656dd7, SUC_lexicon_tags_31 },
    { 0xd1656dd8, SUC_lexicon_tags_27 },
    { 0x86456dd8, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a56de2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456de9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c256ded, SUC_lexicon_tags_23 },
    { 0xe3e56dee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x95e56df0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7256df2, SUC_lexicon_tags_9 },
    { 0x15656df3, SUC_lexicon_tags_98 },
    { 0x1256df3, SUC_lexicon_tags_25 },
    { 0xc7256df5, SUC_lexicon_tags_9 },
    { 0x2b856df6, SUC_lexicon_tags_33 },
    { 0x91e56df5, SUC_lexicon_tags_7 },
    { 0xa7256df8, SUC_lexicon_tags_82 },
    { 0x74c56df9, SUC_lexicon_tags_9 },
    { 0x28856df7, SUC_lexicon_tags_61 },
    { 0xdd456dfb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb5256dfd, SUC_lexicon_tags_95 },
    { 0xfba56dfe, SUC_lexicon_tags_38 },
    { 0xc256dfe, SUC_lexicon_tags_0 },
    { 0x5cc56dff, SUC_lexicon_tags_26 },
    { 0x4a56e01, SUC_lexicon_tags_63 },
    { 0xdfa56dfd, SUC_lexicon_tags_38 },
    { 0x38456e02, SUC_lexicon_tags_23 },
    { 0xdf256e04, SUC_lexicon_tags_17 },
    { 0x1656e05, SUC_lexicon_tags_7 },
    { 0x86e56e05, SUC_lexicon_tags_87 },
    { 0x6fa56e04, SUC_lexicon_tags_7 },
    { 0xeb656e04, SUC_lexicon_tags_11 },
    { 0x7aa56e09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2656e0d, SUC_lexicon_tags_62 },
    { 0x9c456e0d, SUC_lexicon_tags_15 },
    { 0xf9056e0f, SUC_lexicon_tags_4 },
    { 0xdea56e10, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28856e15, SUC_lexicon_tags_14 },
    { 0x5c656e16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65256e1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e856e1e, SUC_lexicon_tags_52 },
    { 0xc1c56e1e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c056e22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x78856e24, SUC_lexicon_tags_23 },
    { 0x98256e25, SUC_lexicon_tags_11 },
    { 0x21056e26, SUC_lexicon_tags_12 },
    { 0x4d656e25, SUC_lexicon_tags_3 },
    { 0xca056e28, SUC_lexicon_tags_8 },
    { 0x36056e28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac56e2f, SUC_lexicon_tags_60 },
    { 0x89056e30, SUC_lexicon_tags_15 },
    { 0x58656e31, SUC_lexicon_tags_549 },
    { 0x5de56e31, SUC_lexicon_tags_15 },
    { 0xd6a56e31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce56e36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be56e3a, SUC_lexicon_tags_37 },
    { 0xbc56e3b, SUC_lexicon_tags_7 },
    { 0xcac56e3c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d056e3f, SUC_lexicon_tags_82 },
    { 0x88456e40, SUC_lexicon_tags_15 },
    { 0x8f056e3f, SUC_lexicon_tags_60 },
    { 0xcb656e40, SUC_lexicon_tags_15 },
    { 0xd4a56e43, SUC_lexicon_tags_2 },
    { 0x3c256e43, SUC_lexicon_tags_1 },
    { 0x40056e45, SUC_lexicon_tags_17 },
    { 0x8d256e45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17656e48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e456e4b, SUC_lexicon_tags_2 },
    { 0xde856e4c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd656e50, SUC_lexicon_tags_23 },
    { 0x6c856e51, SUC_lexicon_tags_1 },
    { 0xf2a56e52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc56e56, SUC_lexicon_tags_6 },
    { 0xa6c56e57, SUC_lexicon_tags_37 },
    { 0xd5c56e58, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x46656e5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd256e5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8456e5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3456e64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9dc56e66, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f056e69, SUC_lexicon_tags_35 },
    { 0x8a656e6a, SUC_lexicon_tags_10 },
    { 0xfe656e6a, SUC_lexicon_tags_82 },
    { 0x5e56e6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da56e6f, SUC_lexicon_tags_63 },
    { 0x9de56e6f, SUC_lexicon_tags_25 },
    { 0x35056e71, SUC_lexicon_tags_13 },
    { 0x1e856e6f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a56e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7856e78, SUC_lexicon_tags_25 },
    { 0x5c656e79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6456e7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c56e83, SUC_lexicon_tags_26 },
    { 0xc1a56e83, SUC_lexicon_tags_70 },
    { 0x31856e85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a56e8a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c56e91, SUC_lexicon_tags_60 },
    { 0x9ac56e92, SUC_lexicon_tags_17 },
    { 0xdd056e92, SUC_lexicon_tags_47 },
    { 0xfe56e94, SUC_lexicon_tags_45 },
    { 0x37256e93, SUC_lexicon_tags_151 },
    { 0xafe56e96, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xb9456e98, SUC_lexicon_tags_25 },
    { 0x36e56e98, SUC_lexicon_tags_17 },
    { 0xb9656e9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c56e9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6c56e9f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x39056ea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c456ea4, SUC_lexicon_tags_25 },
    { 0x92056ea4, SUC_lexicon_tags_2 },
    { 0x81c56ea6, SUC_lexicon_tags_12 },
    { 0xa2c56ea6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2256eb4, SUC_lexicon_tags_70 },
    { 0x3a56eb4, SUC_lexicon_tags_48 },
    { 0x71456eb6, SUC_lexicon_tags_2 },
    { 0x5fa56eb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd456ec4, SUC_lexicon_tags_17 },
    { 0x26256ec5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf456ecb, SUC_lexicon_tags_49 },
    { 0x19656ecc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9056ed0, SUC_lexicon_tags_49 },
    { 0x5f056ed0, SUC_lexicon_tags_21 },
    { 0x19256ed2, SUC_lexicon_tags_22 },
    { 0xc8056ed1, SUC_lexicon_tags_5 },
    { 0xfec56ed3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da56ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c56edb, SUC_lexicon_tags_11 },
    { 0xa3456edc, SUC_lexicon_tags_7 },
    { 0xb0a56edd, SUC_lexicon_tags_14 },
    { 0x1ec56edb, SUC_lexicon_tags_10 },
    { 0x66056edc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65056ee7, SUC_lexicon_tags_2 },
    { 0xcea56ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbae56eea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf456eee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa056ef1, SUC_lexicon_tags_12 },
    { 0x7d256ef1, SUC_lexicon_tags_39 },
    { 0xed256ef3, SUC_lexicon_tags_37 },
    { 0xe7256ef4, SUC_lexicon_tags_151 },
    { 0x3d856ef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98056ef9, SUC_lexicon_tags_25 },
    { 0xc8c56efa, SUC_lexicon_tags_32 },
    { 0x9656efb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba256efd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c56f00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e856f04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13856f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52656f0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71e56f11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4856f17, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xd1456f19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c56f1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35856f26, SUC_lexicon_tags_48 },
    { 0x55a56f27, SUC_lexicon_tags_55 },
    { 0x7d456f26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17056f2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2de56f2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1c56f30, SUC_lexicon_tags_27 },
    { 0xb7856f31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19856f33, SUC_lexicon_tags_0 },
    { 0x2b856f34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a56f3c, SUC_lexicon_tags_37 },
    { 0xc4656f3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3d656f3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbaa56f41, SUC_lexicon_tags_11 },
    { 0xde56f42, SUC_lexicon_tags_2 },
    { 0x72656f43, SUC_lexicon_tags_45 },
    { 0x5ce56f42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac256f49, SUC_lexicon_tags_25 },
    { 0x8a256f4a, SUC_lexicon_tags_3 },
    { 0x46a56f4b, SUC_lexicon_tags_19 },
    { 0xf7456f4b, SUC_lexicon_tags_11 },
    { 0xb8256f49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2a56f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e56f52, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe9056f54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad856f57, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98856f5e, SUC_lexicon_tags_63 },
    { 0x3cc56f5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2f256f61, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa656f68, SUC_lexicon_tags_14 },
    { 0x22256f69, SUC_lexicon_tags_27 },
    { 0x74656f69, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbb856f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59256f6e, SUC_lexicon_tags_10 },
    { 0xa7456f6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65656f71, SUC_lexicon_tags_47 },
    { 0xbe656f72, SUC_lexicon_tags_7 },
    { 0x9d656f73, SUC_lexicon_tags_11 },
    { 0xe0c56f72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a56f77, SUC_lexicon_tags_9 },
    { 0x5a056f78, SUC_lexicon_tags_8 },
    { 0x1a056f79, SUC_lexicon_tags_21 },
    { 0x82656f78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c56f7d, SUC_lexicon_tags_38 },
    { 0x23056f7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a056f86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee56f94, SUC_lexicon_tags_30 },
    { 0x8ce56f95, SUC_lexicon_tags_15 },
    { 0x64856f95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf1a56f98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3256f9e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c56fa1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81456fa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca56fa8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1056fac, SUC_lexicon_tags_7 },
    { 0xa7a56fad, SUC_lexicon_tags_37 },
    { 0xde856fae, SUC_lexicon_tags_29 },
    { 0x75456fae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7656fb2, SUC_lexicon_tags_37 },
    { 0xe5e56fb3, SUC_lexicon_tags_1 },
    { 0xc6256fb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdac56fb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a056fba, SUC_lexicon_tags_56 },
    { 0x7056fbb, SUC_lexicon_tags_49 },
    { 0x4cc56fbc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfae56fbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c056fc4, SUC_lexicon_tags_27 },
    { 0x28656fc5, SUC_lexicon_tags_9 },
    { 0x8b656fc6, SUC_lexicon_tags_35 },
    { 0x78056fc7, SUC_lexicon_tags_19 },
    { 0x10256fc8, SUC_lexicon_tags_2 },
    { 0xfc56fc9, SUC_lexicon_tags_11 },
    { 0x1f256fca, SUC_lexicon_tags_48 },
    { 0x37256fcb, SUC_lexicon_tags_37 },
    { 0x66a56fc8, SUC_lexicon_tags_17 },
    { 0x7d456fc8, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x81256fcf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a456fd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1456fd6, SUC_lexicon_tags_31 },
    { 0x31e56fd7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e56fda, SUC_lexicon_tags_63 },
    { 0x84056fdb, SUC_lexicon_tags_17 },
    { 0xc7256fdc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6d256fde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e56fe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff656fe3, SUC_lexicon_tags_7 },
    { 0x99656fe3, SUC_lexicon_tags_19 },
    { 0x3de56fe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3456fe8, SUC_lexicon_tags_0 },
    { 0x10a56fe8, SUC_lexicon_tags_17 },
    { 0xd8456fe9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e56ff2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7f856ff4, SUC_lexicon_tags_46 },
    { 0x4c456ff4, SUC_lexicon_tags_2 },
    { 0xcdc56ff4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78056ff8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe456ffb, SUC_lexicon_tags_17 },
    { 0x8e256ffc, SUC_lexicon_tags_11 },
    { 0x44656ffc, SUC_lexicon_tags_63 },
    { 0x54a56ffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc57002, SUC_lexicon_tags_37 },
    { 0xa9657002, SUC_lexicon_tags_37 },
    { 0xe3657004, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa9657006, SUC_lexicon_tags_2 },
    { 0xaa057006, SUC_lexicon_tags_15 },
    { 0xaa857006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa5700d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e5700f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57a57011, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21057013, SUC_lexicon_tags_15 },
    { 0xc6857013, SUC_lexicon_tags_17 },
    { 0xf1e57015, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa885701d, SUC_lexicon_tags_11 },
    { 0xfa25701e, SUC_lexicon_tags_49 },
    { 0x4365701e, SUC_lexicon_tags_47 },
    { 0x405701e, SUC_lexicon_tags_25 },
    { 0x91857021, SUC_lexicon_tags_25 },
    { 0x38857022, SUC_lexicon_tags_17 },
    { 0x2657022, SUC_lexicon_tags_25 },
    { 0xa7c57023, SUC_lexicon_tags_14 },
    { 0x93257024, SUC_lexicon_tags_15 },
    { 0xe2e57025, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2a57028, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc105702c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc5702f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97657032, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16057034, SUC_lexicon_tags_178 },
    { 0xfae57035, SUC_lexicon_tags_0 },
    { 0x86057036, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe2e57038, SUC_lexicon_tags_6 },
    { 0xcae57039, SUC_lexicon_tags_31 },
    { 0x5a5703a, SUC_lexicon_tags_25 },
    { 0x35a57039, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x66c5703d, SUC_lexicon_tags_49 },
    { 0x98a5703d, SUC_lexicon_tags_12 },
    { 0x2b05703f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc857049, SUC_lexicon_tags_23 },
    { 0x1ea57049, SUC_lexicon_tags_204 },
    { 0xa945704b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9005704d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e57050, SUC_lexicon_tags_10 },
    { 0xe857051, SUC_lexicon_tags_25 },
    { 0x22457052, SUC_lexicon_tags_49 },
    { 0xe9657053, SUC_lexicon_tags_26 },
    { 0x2da57054, SUC_lexicon_tags_11 },
    { 0x64257051, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x69657058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e5705e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6c857060, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa6657062, SUC_lexicon_tags_5 },
    { 0xc7057063, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4057067, SUC_lexicon_tags_37 },
    { 0x9fa57067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37c5706a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7065706e, SUC_lexicon_tags_25 },
    { 0xd385706e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83857071, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d457076, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d45707b, SUC_lexicon_tags_30 },
    { 0x5725707b, SUC_lexicon_tags_9 },
    { 0x79c5707d, SUC_lexicon_tags_37 },
    { 0x2225707e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee57081, SUC_lexicon_tags_15 },
    { 0x61c57082, SUC_lexicon_tags_2 },
    { 0x8257081, SUC_lexicon_tags_738 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c57086, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf257088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e5708a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd057090, SUC_lexicon_tags_45 },
    { 0x99257091, SUC_lexicon_tags_45 },
    { 0x7f857092, SUC_lexicon_tags_14 },
    { 0x8da57091, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c57097, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be5709a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf8c5709c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3985709f, SUC_lexicon_tags_6 },
    { 0x51e570a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x230570a2, SUC_lexicon_tags_47 },
    { 0xd42570a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39a570a5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee570b0, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0xc30570b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8570b5, SUC_lexicon_tags_29 },
    { 0x47c570b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76570bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a570be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4570c7, SUC_lexicon_tags_6 },
    { 0x28e570c7, SUC_lexicon_tags_6 },
    { 0xf3c570c9, SUC_lexicon_tags_17 },
    { 0x8e8570c7, SUC_lexicon_tags_31 },
    { 0x980570cb, SUC_lexicon_tags_104 },
    { 0x32a570cc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x94c570ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x224570d0, SUC_lexicon_tags_7 },
    { 0x8fe570d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce570d3, SUC_lexicon_tags_25 },
    { 0x92a570d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe3a570d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x450570d8, SUC_lexicon_tags_39 },
    { 0x14a570d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x816570db, SUC_lexicon_tags_45 },
    { 0x24c570dc, SUC_lexicon_tags_11 },
    { 0x79a570dd, SUC_lexicon_tags_45 },
    { 0xc6e570de, SUC_lexicon_tags_2 },
    { 0xa22570df, SUC_lexicon_tags_17 },
    { 0xede570e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x34e570e2, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c570e5, SUC_lexicon_tags_77 },
    { 0x964570e6, SUC_lexicon_tags_24 },
    { 0x6b6570e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e570ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244570ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0570f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8570f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc570f8, SUC_lexicon_tags_37 },
    { 0x30c570f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98570fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xade570ff, SUC_lexicon_tags_96 },
    { 0x28857100, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e57105, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b057108, SUC_lexicon_tags_7 },
    { 0xe7a57109, SUC_lexicon_tags_29 },
    { 0x26e57109, SUC_lexicon_tags_25 },
    { 0xc9e5710b, SUC_lexicon_tags_7 },
    { 0xa225710b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc57114, SUC_lexicon_tags_34 },
    { 0xcb057115, SUC_lexicon_tags_27 },
    { 0xd9257116, SUC_lexicon_tags_1 },
    { 0x9f857115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2c57119, SUC_lexicon_tags_2 },
    { 0xe857119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d65711c, SUC_lexicon_tags_15 },
    { 0x3e05711d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c65711f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca57126, SUC_lexicon_tags_48 },
    { 0x91c57127, SUC_lexicon_tags_17 },
    { 0xaf457127, SUC_lexicon_tags_37 },
    { 0x97057128, SUC_lexicon_tags_14 },
    { 0x46257128, SUC_lexicon_tags_7 },
    { 0xab25712a, SUC_lexicon_tags_25 },
    { 0x52c5712c, SUC_lexicon_tags_1 },
    { 0x86e5712d, SUC_lexicon_tags_6 },
    { 0x1c857127, SUC_lexicon_tags_46 },
    { 0x33e5712d, SUC_lexicon_tags_29 },
    { 0xab257130, SUC_lexicon_tags_14 },
    { 0x9e257131, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x11c57133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe257135, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c857137, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fa57139, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3257140, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xff257143, SUC_lexicon_tags_202 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d657146, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81457148, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb885714a, SUC_lexicon_tags_5 },
    { 0xe3c5714b, SUC_lexicon_tags_1 },
    { 0x3745714c, SUC_lexicon_tags_318 },
    { 0x7b05714a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9257150, SUC_lexicon_tags_15 },
    { 0x29e57151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d457155, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c57159, SUC_lexicon_tags_7 },
    { 0x4a45715a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f25715f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b457162, SUC_lexicon_tags_37 },
    { 0xaee57163, SUC_lexicon_tags_19 },
    { 0xcd457164, SUC_lexicon_tags_25 },
    { 0x83a57163, SUC_lexicon_tags_38 },
    { 0x9e257163, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c57169, SUC_lexicon_tags_7 },
    { 0xfc057169, SUC_lexicon_tags_7 },
    { 0x2ce5716a, SUC_lexicon_tags_38 },
    { 0x3185716a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c5716f, SUC_lexicon_tags_16 },
    { 0x4fa57170, SUC_lexicon_tags_43 },
    { 0x4ce5716f, SUC_lexicon_tags_32 },
    { 0xba257172, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x4a457174, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd857178, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e25717b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a5717e, SUC_lexicon_tags_46 },
    { 0xa8e5717f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15457181, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c57185, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1585718d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9205718f, SUC_lexicon_tags_56 },
    { 0xabe57190, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4857194, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf257197, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c5719d, SUC_lexicon_tags_37 },
    { 0xd105719e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x344571a0, SUC_lexicon_tags_37 },
    { 0xe9a571a1, SUC_lexicon_tags_7 },
    { 0xf92571a2, SUC_lexicon_tags_25 },
    { 0xde4571a3, SUC_lexicon_tags_48 },
    { 0xc40571a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd6571a6, SUC_lexicon_tags_37 },
    { 0xf3e571a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec571aa, SUC_lexicon_tags_12 },
    { 0xb0e571aa, SUC_lexicon_tags_47 },
    { 0x4c8571ac, SUC_lexicon_tags_13 },
    { 0xd62571ad, SUC_lexicon_tags_37 },
    { 0x93e571ab, SUC_lexicon_tags_11 },
    { 0xe38571af, SUC_lexicon_tags_11 },
    { 0x8d8571b0, SUC_lexicon_tags_82 },
    { 0x948571aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18571bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec571c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14e571c2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc571c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a4571c7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf4a571c9, SUC_lexicon_tags_26 },
    { 0x37e571c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea571ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69c571d0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524571da, SUC_lexicon_tags_7 },
    { 0xd22571db, SUC_lexicon_tags_28 },
    { 0x6ee571db, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920571e0, SUC_lexicon_tags_127 },
    { 0xdf6571e1, SUC_lexicon_tags_1 },
    { 0xb12571e2, SUC_lexicon_tags_31 },
    { 0xf8c571e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc571e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458571f4, SUC_lexicon_tags_33 },
    { 0x83c571f5, SUC_lexicon_tags_17 },
    { 0x582571f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9c571f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca571fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b657202, SUC_lexicon_tags_7 },
    { 0x16e57203, SUC_lexicon_tags_25 },
    { 0x1b857204, SUC_lexicon_tags_37 },
    { 0xd5457204, SUC_lexicon_tags_47 },
    { 0x73c57206, SUC_lexicon_tags_21 },
    { 0x4aa57206, SUC_lexicon_tags_29 },
    { 0x52e57208, SUC_lexicon_tags_37 },
    { 0xc3c57209, SUC_lexicon_tags_31 },
    { 0x65457209, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94257211, SUC_lexicon_tags_27 },
    { 0x2e457212, SUC_lexicon_tags_2 },
    { 0x4c57213, SUC_lexicon_tags_17 },
    { 0xe3a57213, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1057216, SUC_lexicon_tags_47 },
    { 0xee57217, SUC_lexicon_tags_30 },
    { 0x3bc57218, SUC_lexicon_tags_31 },
    { 0x88a57219, SUC_lexicon_tags_37 },
    { 0xa45721a, SUC_lexicon_tags_7 },
    { 0xf465721b, SUC_lexicon_tags_12 },
    { 0xa525721b, SUC_lexicon_tags_53 },
    { 0x5725721d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x60c5721f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d257222, SUC_lexicon_tags_37 },
    { 0xc5057223, SUC_lexicon_tags_37 },
    { 0x85a57223, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61a57226, SUC_lexicon_tags_60 },
    { 0xeac57227, SUC_lexicon_tags_14 },
    { 0x53857227, SUC_lexicon_tags_25 },
    { 0x24057229, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6125722b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be5722e, SUC_lexicon_tags_1 },
    { 0xf105722e, SUC_lexicon_tags_7 },
    { 0x9185722e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e5723b, SUC_lexicon_tags_17 },
    { 0xae05723c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c57244, SUC_lexicon_tags_17 },
    { 0xce057245, SUC_lexicon_tags_33 },
    { 0x6b857246, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a5724b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e257251, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa8257253, SUC_lexicon_tags_2 },
    { 0x15c57253, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x24c57256, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86057258, SUC_lexicon_tags_1 },
    { 0x8057258, SUC_lexicon_tags_19 },
    { 0xdb657258, SUC_lexicon_tags_27 },
    { 0x8845725b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0457261, SUC_lexicon_tags_17 },
    { 0x84857261, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce5726c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4cc5726e, SUC_lexicon_tags_17 },
    { 0x8f05726f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf0e57271, SUC_lexicon_tags_25 },
    { 0xb657272, SUC_lexicon_tags_9 },
    { 0x76e57272, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b857275, SUC_lexicon_tags_2 },
    { 0xdfc57276, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a5727d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xffc5727f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d057285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1857289, SUC_lexicon_tags_15 },
    { 0x405728a, SUC_lexicon_tags_28 },
    { 0x6ce5728a, SUC_lexicon_tags_30 },
    { 0xaac5728c, SUC_lexicon_tags_2 },
    { 0x2565728b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa25728f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91657294, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa5729c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c572a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x976572a6, SUC_lexicon_tags_9 },
    { 0xbb8572a7, SUC_lexicon_tags_14 },
    { 0x86a572a8, SUC_lexicon_tags_2 },
    { 0x596572a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x49e572ab, SUC_lexicon_tags_43 },
    { 0x404572ac, SUC_lexicon_tags_12 },
    { 0x28a572ac, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a572b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8572b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca572bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a572c1, SUC_lexicon_tags_11 },
    { 0x5d4572c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x636572c5, SUC_lexicon_tags_1 },
    { 0x9c572c6, SUC_lexicon_tags_63 },
    { 0x82572c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb8572c9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620572ce, SUC_lexicon_tags_47 },
    { 0xa4572cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x92a572d1, SUC_lexicon_tags_17 },
    { 0x334572d1, SUC_lexicon_tags_40 },
    { 0xb32572d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x390572d5, SUC_lexicon_tags_10 },
    { 0x3b0572d6, SUC_lexicon_tags_47 },
    { 0x766572d7, SUC_lexicon_tags_11 },
    { 0x282572d5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf9c572da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc0572dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe572e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4572e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02572e6, SUC_lexicon_tags_17 },
    { 0x4d2572e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x990572e9, SUC_lexicon_tags_2 },
    { 0x216572ea, SUC_lexicon_tags_11 },
    { 0x1b8572eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc74572ed, SUC_lexicon_tags_15 },
    { 0x3be572ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8572f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66572f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x696572f6, SUC_lexicon_tags_14 },
    { 0x97e572f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192572fb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb28572fd, SUC_lexicon_tags_49 },
    { 0xc02572fd, SUC_lexicon_tags_2 },
    { 0x954572ff, SUC_lexicon_tags_11 },
    { 0x886572ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3f457302, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2457305, SUC_lexicon_tags_23 },
    { 0x99857306, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9457309, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa745730d, SUC_lexicon_tags_79 },
    { 0x9e65730e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82057313, SUC_lexicon_tags_27 },
    { 0x33657313, SUC_lexicon_tags_37 },
    { 0x60c57315, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf65731b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec57322, SUC_lexicon_tags_7 },
    { 0x8ca57323, SUC_lexicon_tags_7 },
    { 0xf1a57324, SUC_lexicon_tags_17 },
    { 0x24c57325, SUC_lexicon_tags_1 },
    { 0x30257326, SUC_lexicon_tags_7 },
    { 0x84657326, SUC_lexicon_tags_6 },
    { 0x59457328, SUC_lexicon_tags_12 },
    { 0xb0c57329, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3005732d, SUC_lexicon_tags_28 },
    { 0x3105732e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11457330, SUC_lexicon_tags_27 },
    { 0xc9657331, SUC_lexicon_tags_1 },
    { 0x38857331, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53257334, SUC_lexicon_tags_37 },
    { 0xa7057335, SUC_lexicon_tags_56 },
    { 0x34457335, SUC_lexicon_tags_1 },
    { 0x94657337, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd1c57339, SUC_lexicon_tags_38 },
    { 0xe7c5733a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e57340, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe57343, SUC_lexicon_tags_43 },
    { 0x97e57344, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad657347, SUC_lexicon_tags_1 },
    { 0xf2c57348, SUC_lexicon_tags_14 },
    { 0xb3a57349, SUC_lexicon_tags_22 },
    { 0xdf45734a, SUC_lexicon_tags_9 },
    { 0x69e57349, SUC_lexicon_tags_25 },
    { 0x2025734b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb245734f, SUC_lexicon_tags_12 },
    { 0x54257350, SUC_lexicon_tags_15 },
    { 0xa5257351, SUC_lexicon_tags_37 },
    { 0xece57351, SUC_lexicon_tags_23 },
    { 0x2b257353, SUC_lexicon_tags_25 },
    { 0x9b057350, SUC_lexicon_tags_7 },
    { 0xd8057355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe005735d, SUC_lexicon_tags_1 },
    { 0xc985735e, SUC_lexicon_tags_10 },
    { 0xffe5735e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2657364, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8057366, SUC_lexicon_tags_25 },
    { 0x4f057367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde5736d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3457370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7985737a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce85737c, SUC_lexicon_tags_46 },
    { 0x6065737c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9205737f, SUC_lexicon_tags_48 },
    { 0x56457380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe857383, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80657387, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc657389, SUC_lexicon_tags_13 },
    { 0xef45738a, SUC_lexicon_tags_42 },
    { 0x34e5738b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c65738d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa7a5738f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a57396, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe985739c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e45739e, SUC_lexicon_tags_43 },
    { 0x83a5739f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x68a573a1, SUC_lexicon_tags_60 },
    { 0x432573a1, SUC_lexicon_tags_60 },
    { 0x690573a3, SUC_lexicon_tags_49 },
    { 0xd54573a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee573aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44573ad, SUC_lexicon_tags_63 },
    { 0x1b4573ae, SUC_lexicon_tags_28 },
    { 0x99a573ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd30573b1, SUC_lexicon_tags_7 },
    { 0x3b0573b2, SUC_lexicon_tags_2 },
    { 0xd2c573b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a573b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc0573b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafe573bb, SUC_lexicon_tags_23 },
    { 0x2e573bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc573c2, SUC_lexicon_tags_15 },
    { 0xa8573c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2e4573c5, SUC_lexicon_tags_2 },
    { 0x316573c6, SUC_lexicon_tags_0 },
    { 0xf1e573c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x664573ca, SUC_lexicon_tags_32 },
    { 0xed6573cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c573d1, SUC_lexicon_tags_48 },
    { 0x354573d2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e573d5, SUC_lexicon_tags_8 },
    { 0x52573d6, SUC_lexicon_tags_17 },
    { 0xe22573d6, SUC_lexicon_tags_19 },
    { 0x9c8573d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca573dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce573e2, SUC_lexicon_tags_60 },
    { 0x264573e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a573e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72573ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a573f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54573f6, SUC_lexicon_tags_37 },
    { 0xe2e573f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x98e573f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b4573fb, SUC_lexicon_tags_12 },
    { 0x930573fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc573ff, SUC_lexicon_tags_10 },
    { 0x168573ff, SUC_lexicon_tags_12 },
    { 0xc1657401, SUC_lexicon_tags_423 },
    { 0x55057402, SUC_lexicon_tags_17 },
    { 0xdca57402, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3505740e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27857410, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79057413, SUC_lexicon_tags_25 },
    { 0x457414, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f057417, SUC_lexicon_tags_60 },
    { 0x1457417, SUC_lexicon_tags_47 },
    { 0xf0457418, SUC_lexicon_tags_12 },
    { 0xa7457418, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xde25741d, SUC_lexicon_tags_1 },
    { 0xe765741e, SUC_lexicon_tags_6 },
    { 0xeca5741f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76c57421, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30257426, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1057428, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe065742b, SUC_lexicon_tags_37 },
    { 0xefa5742b, SUC_lexicon_tags_11 },
    { 0x5565742d, SUC_lexicon_tags_12 },
    { 0xae5742e, SUC_lexicon_tags_17 },
    { 0xe385742f, SUC_lexicon_tags_15 },
    { 0xbbc57430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e57433, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a57437, SUC_lexicon_tags_15 },
    { 0x58457437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca57442, SUC_lexicon_tags_31 },
    { 0x23e57443, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb125744d, SUC_lexicon_tags_4 },
    { 0x8465744e, SUC_lexicon_tags_19 },
    { 0xd705744f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4257451, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2857454, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd905745a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e5745d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60257461, SUC_lexicon_tags_61 },
    { 0x96057461, SUC_lexicon_tags_8 },
    { 0x75857463, SUC_lexicon_tags_25 },
    { 0xf7857462, SUC_lexicon_tags_64 },
    { 0x43c57465, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21657469, SUC_lexicon_tags_33 },
    { 0x80e5746a, SUC_lexicon_tags_29 },
    { 0xe625746b, SUC_lexicon_tags_45 },
    { 0x1d25746a, SUC_lexicon_tags_37 },
    { 0xa1a5746b, SUC_lexicon_tags_1 },
    { 0xa605746b, SUC_lexicon_tags_26 },
    { 0xbe05746e, SUC_lexicon_tags_11 },
    { 0x24a5746d, SUC_lexicon_tags_69 },
    { 0xde57471, SUC_lexicon_tags_17 },
    { 0xa565746a, SUC_lexicon_tags_37 },
    { 0xfd45746e, SUC_lexicon_tags_18 },
    { 0x2cc57474, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e057478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee5747d, SUC_lexicon_tags_12 },
    { 0xae45747d, SUC_lexicon_tags_11 },
    { 0xdb65747f, SUC_lexicon_tags_37 },
    { 0x55057480, SUC_lexicon_tags_33 },
    { 0x4f857480, SUC_lexicon_tags_15 },
    { 0x657482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda57488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa265748b, SUC_lexicon_tags_2 },
    { 0x5aa5748c, SUC_lexicon_tags_11 },
    { 0xa285748d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7657491, SUC_lexicon_tags_7 },
    { 0x34057491, SUC_lexicon_tags_17 },
    { 0x85257492, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a57497, SUC_lexicon_tags_26 },
    { 0x5e457497, SUC_lexicon_tags_12 },
    { 0x23c57498, SUC_lexicon_tags_12 },
    { 0xb625749a, SUC_lexicon_tags_7 },
    { 0x265749a, SUC_lexicon_tags_15 },
    { 0x4205749c, SUC_lexicon_tags_87 },
    { 0x68857497, SUC_lexicon_tags_47 },
    { 0x4145749b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x544574a1, SUC_lexicon_tags_49 },
    { 0x96574a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6574a6, SUC_lexicon_tags_7 },
    { 0x83a574a7, SUC_lexicon_tags_27 },
    { 0x686574a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x328574ab, SUC_lexicon_tags_46 },
    { 0xea4574ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x764574ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa72574b0, SUC_lexicon_tags_84 },
    { 0x22e574b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6574b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb8574b7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbf0574b9, SUC_lexicon_tags_91 },
    { 0x55a574ba, SUC_lexicon_tags_1 },
    { 0xd22574bb, SUC_lexicon_tags_14 },
    { 0xb9e574bc, SUC_lexicon_tags_65 },
    { 0x868574bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82574c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990574c9, SUC_lexicon_tags_7 },
    { 0x3dc574c9, SUC_lexicon_tags_138 },
    { 0x8f2574cb, SUC_lexicon_tags_19 },
    { 0x9f6574cc, SUC_lexicon_tags_47 },
    { 0x404574cc, SUC_lexicon_tags_47 },
    { 0xa80574ce, SUC_lexicon_tags_46 },
    { 0x910574cf, SUC_lexicon_tags_7 },
    { 0xc46574cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90574d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e574d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x600574db, SUC_lexicon_tags_25 },
    { 0x94a574db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe32574de, SUC_lexicon_tags_45 },
    { 0xd8a574df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a574e3, SUC_lexicon_tags_8 },
    { 0x898574e4, SUC_lexicon_tags_62 },
    { 0x220574e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e574ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa574f0, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x736574f2, SUC_lexicon_tags_2 },
    { 0xb98574f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x552574f5, SUC_lexicon_tags_25 },
    { 0xbf4574f6, SUC_lexicon_tags_103 },
    { 0xc96574f7, SUC_lexicon_tags_12 },
    { 0xf94574f5, SUC_lexicon_tags_29 },
    { 0xc60574f9, SUC_lexicon_tags_23 },
    { 0xb48574f9, SUC_lexicon_tags_8 },
    { 0x310574fb, SUC_lexicon_tags_129 },
    { 0x798574fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b257501, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f257504, SUC_lexicon_tags_26 },
    { 0xeda57505, SUC_lexicon_tags_63 },
    { 0x24257504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe5750c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7657510, SUC_lexicon_tags_56 },
    { 0x93a57511, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e457516, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a5751a, SUC_lexicon_tags_12 },
    { 0x2f65751b, SUC_lexicon_tags_45 },
    { 0xa785751c, SUC_lexicon_tags_2 },
    { 0xf565751d, SUC_lexicon_tags_11 },
    { 0xe9c5751c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xedc57520, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede57524, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe345752a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc5752f, SUC_lexicon_tags_63 },
    { 0xbf857530, SUC_lexicon_tags_1 },
    { 0xda45752f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a57534, SUC_lexicon_tags_2 },
    { 0xdc257535, SUC_lexicon_tags_60 },
    { 0xb3457536, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c5753a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb25753c, SUC_lexicon_tags_33 },
    { 0x2005753c, SUC_lexicon_tags_32 },
    { 0x4ca5753e, SUC_lexicon_tags_12 },
    { 0x6245753f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b057541, SUC_lexicon_tags_17 },
    { 0x88457542, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76657546, SUC_lexicon_tags_12 },
    { 0x82e57547, SUC_lexicon_tags_11 },
    { 0xfbe57548, SUC_lexicon_tags_14 },
    { 0xecc57548, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61a5754b, SUC_lexicon_tags_37 },
    { 0x8045754c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc5a5754e, SUC_lexicon_tags_38 },
    { 0x5685754f, SUC_lexicon_tags_47 },
    { 0xc6e57550, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9057555, SUC_lexicon_tags_12 },
    { 0xa0857556, SUC_lexicon_tags_9 },
    { 0xa3c57556, SUC_lexicon_tags_66 },
    { 0x93a57557, SUC_lexicon_tags_39 },
    { 0x31457559, SUC_lexicon_tags_18 },
    { 0x58a5755a, SUC_lexicon_tags_8 },
    { 0x72257559, SUC_lexicon_tags_47 },
    { 0x5aa5755c, SUC_lexicon_tags_82 },
    { 0x6145755d, SUC_lexicon_tags_11 },
    { 0x3945755e, SUC_lexicon_tags_235 },
    { 0x1725755b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed457563, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61457566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa57569, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c5756e, SUC_lexicon_tags_7 },
    { 0x49a5756e, SUC_lexicon_tags_23 },
    { 0xfb057570, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97457572, SUC_lexicon_tags_23 },
    { 0xdf057573, SUC_lexicon_tags_1 },
    { 0x96c57574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40057577, SUC_lexicon_tags_37 },
    { 0xa8057578, SUC_lexicon_tags_0 },
    { 0xc0857578, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa405757b, SUC_lexicon_tags_1651 },
    { 0x6f65757c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf0c5757e, SUC_lexicon_tags_11 },
    { 0xac25757f, SUC_lexicon_tags_45 },
    { 0xbce57580, SUC_lexicon_tags_43 },
    { 0xe0457580, SUC_lexicon_tags_31 },
    { 0x14857582, SUC_lexicon_tags_25 },
    { 0x80457583, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0857589, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe965758b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xac5758d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a57595, SUC_lexicon_tags_12 },
    { 0x94a57596, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb057599, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf25759c, SUC_lexicon_tags_87 },
    { 0xcd05759d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc575a1, SUC_lexicon_tags_69 },
    { 0xe4c575a2, SUC_lexicon_tags_2 },
    { 0x47a575a3, SUC_lexicon_tags_17 },
    { 0x7a4575a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18575a7, SUC_lexicon_tags_43 },
    { 0xd80575a8, SUC_lexicon_tags_35 },
    { 0x1b2575a7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x28c575ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52575af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e575b2, SUC_lexicon_tags_29 },
    { 0x6aa575b2, SUC_lexicon_tags_1 },
    { 0x656575b4, SUC_lexicon_tags_17 },
    { 0xb4a575b5, SUC_lexicon_tags_8 },
    { 0x192575b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe575b9, SUC_lexicon_tags_3 },
    { 0xbcc575ba, SUC_lexicon_tags_1 },
    { 0x5ec575bb, SUC_lexicon_tags_27 },
    { 0x2da575bc, SUC_lexicon_tags_12 },
    { 0x3bc575bc, SUC_lexicon_tags_21 },
    { 0xad2575bc, SUC_lexicon_tags_14 },
    { 0x18c575bf, SUC_lexicon_tags_7 },
    { 0x1be575c0, SUC_lexicon_tags_55 },
    { 0x450575c1, SUC_lexicon_tags_25 },
    { 0xd74575bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x240575c4, SUC_lexicon_tags_6 },
    { 0xf8e575c5, SUC_lexicon_tags_17 },
    { 0xc70575c6, SUC_lexicon_tags_65 },
    { 0x538575c7, SUC_lexicon_tags_14 },
    { 0xca4575c7, SUC_lexicon_tags_25 },
    { 0xacc575c9, SUC_lexicon_tags_70 },
    { 0xb74575c9, SUC_lexicon_tags_25 },
    { 0x1b8575cb, SUC_lexicon_tags_37 },
    { 0xf3c575ca, SUC_lexicon_tags_12 },
    { 0x6aa575c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e575d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88575d8, SUC_lexicon_tags_33 },
    { 0xca8575d9, SUC_lexicon_tags_15 },
    { 0xd9e575da, SUC_lexicon_tags_65 },
    { 0xd9c575da, SUC_lexicon_tags_0 },
    { 0xdc4575da, SUC_lexicon_tags_25 },
    { 0xe66575dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc02575df, SUC_lexicon_tags_1 },
    { 0xcb2575df, SUC_lexicon_tags_35 },
    { 0x8da575e1, SUC_lexicon_tags_33 },
    { 0xe30575e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6575e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa575ea, SUC_lexicon_tags_25 },
    { 0x6d4575eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d8575ed, SUC_lexicon_tags_63 },
    { 0xd3c575ed, SUC_lexicon_tags_9 },
    { 0xcec575ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e575f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0575fb, SUC_lexicon_tags_30 },
    { 0x95a575fb, SUC_lexicon_tags_62 },
    { 0x2e4575fd, SUC_lexicon_tags_23 },
    { 0xf2c575fe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e57601, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b057604, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdb857606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a05760c, SUC_lexicon_tags_33 },
    { 0x65a5760d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a57610, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22257614, SUC_lexicon_tags_52 },
    { 0x6857615, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x63657617, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7457619, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c5761c, SUC_lexicon_tags_31 },
    { 0x57c5761d, SUC_lexicon_tags_15 },
    { 0xfe45761e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657623, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea57627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc5762d, SUC_lexicon_tags_17 },
    { 0x8905762d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92857633, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a5763a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6605763d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa57642, SUC_lexicon_tags_43 },
    { 0xa5e57643, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c257647, SUC_lexicon_tags_1 },
    { 0xa7257647, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7445764b, SUC_lexicon_tags_56 },
    { 0x7ca5764c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcc5764e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x12257650, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf457654, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18057659, SUC_lexicon_tags_17 },
    { 0x3305765a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdbc5765c, SUC_lexicon_tags_49 },
    { 0x7665765d, SUC_lexicon_tags_17 },
    { 0x4225765e, SUC_lexicon_tags_11 },
    { 0x3045765e, SUC_lexicon_tags_43 },
    { 0xb925765f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77457665, SUC_lexicon_tags_27 },
    { 0xca457666, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe45766b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2745766e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba657671, SUC_lexicon_tags_53 },
    { 0xa2c57672, SUC_lexicon_tags_11 },
    { 0xb5257672, SUC_lexicon_tags_15 },
    { 0xf1257673, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea5767b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec5767f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a857681, SUC_lexicon_tags_63 },
    { 0xb9857681, SUC_lexicon_tags_15 },
    { 0x31c57683, SUC_lexicon_tags_25 },
    { 0x1e057683, SUC_lexicon_tags_49 },
    { 0x66257685, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab057689, SUC_lexicon_tags_31 },
    { 0x6f05768a, SUC_lexicon_tags_1 },
    { 0xdd65768b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4205768f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x32657691, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x20457694, SUC_lexicon_tags_15 },
    { 0x5e257694, SUC_lexicon_tags_37 },
    { 0xb6257696, SUC_lexicon_tags_25 },
    { 0xa0257697, SUC_lexicon_tags_23 },
    { 0xab657698, SUC_lexicon_tags_2 },
    { 0x66057695, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae576a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64576a7, SUC_lexicon_tags_11 },
    { 0x46a576a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf64576aa, SUC_lexicon_tags_30 },
    { 0x3be576ab, SUC_lexicon_tags_25 },
    { 0x898576ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8576b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x22a576b4, SUC_lexicon_tags_12 },
    { 0xa30576b4, SUC_lexicon_tags_34 },
    { 0x484576b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce576ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c576be, SUC_lexicon_tags_45 },
    { 0xe84576bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0576c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce576c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46576cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462576d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08576d3, SUC_lexicon_tags_19 },
    { 0xcfe576d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e576d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6576dc, SUC_lexicon_tags_6 },
    { 0x79c576dd, SUC_lexicon_tags_5 },
    { 0x178576de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c576e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x664576e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e576e7, SUC_lexicon_tags_25 },
    { 0xe46576e7, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0x680576eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3ce576ed, SUC_lexicon_tags_0 },
    { 0xd60576ed, SUC_lexicon_tags_47 },
    { 0x950576ef, SUC_lexicon_tags_25 },
    { 0x226576f0, SUC_lexicon_tags_14 },
    { 0x4a0576f1, SUC_lexicon_tags_74 },
    { 0x12e576ef, SUC_lexicon_tags_17 },
    { 0xae4576f3, SUC_lexicon_tags_38 },
    { 0x404576f4, SUC_lexicon_tags_65 },
    { 0x1ee576f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0576f8, SUC_lexicon_tags_7 },
    { 0x574576f9, SUC_lexicon_tags_48 },
    { 0x338576fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0576fd, SUC_lexicon_tags_45 },
    { 0xd88576fd, SUC_lexicon_tags_15 },
    { 0x2e2576fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf2657701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10a57703, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4925770a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe6a5770c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5c5770e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea57713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9457715, SUC_lexicon_tags_43 },
    { 0x10257715, SUC_lexicon_tags_37 },
    { 0xcd057715, SUC_lexicon_tags_11 },
    { 0x33457718, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xda65771a, SUC_lexicon_tags_33 },
    { 0x6b85771b, SUC_lexicon_tags_1 },
    { 0x4785771b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda05771f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb457722, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb857728, SUC_lexicon_tags_2 },
    { 0x6da57729, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x29e5772b, SUC_lexicon_tags_87 },
    { 0x7845772b, SUC_lexicon_tags_63 },
    { 0xe045772d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2ea5772f, SUC_lexicon_tags_25 },
    { 0x6dc57730, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae57733, SUC_lexicon_tags_37 },
    { 0xc5457734, SUC_lexicon_tags_43 },
    { 0xf4257734, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22457739, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80c5773b, SUC_lexicon_tags_82 },
    { 0x92a5773c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2165773e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa857742, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57e57744, SUC_lexicon_tags_23 },
    { 0x6b857745, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x64457748, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7a5774a, SUC_lexicon_tags_35 },
    { 0x1225774b, SUC_lexicon_tags_30 },
    { 0xb765774b, SUC_lexicon_tags_37 },
    { 0xdee5774d, SUC_lexicon_tags_25 },
    { 0x9be5774e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e57752, SUC_lexicon_tags_19 },
    { 0xaaa57753, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x41c57755, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x62657757, SUC_lexicon_tags_62 },
    { 0x1a257758, SUC_lexicon_tags_1 },
    { 0x86e57759, SUC_lexicon_tags_1 },
    { 0x5c457758, SUC_lexicon_tags_55 },
    { 0x2f457759, SUC_lexicon_tags_25 },
    { 0xc185775c, SUC_lexicon_tags_19 },
    { 0xf45775d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d257761, SUC_lexicon_tags_7 },
    { 0x49a57762, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e657769, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0xd245776b, SUC_lexicon_tags_17 },
    { 0xe725776b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfac5776e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c57775, SUC_lexicon_tags_3 },
    { 0xfc257775, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc045777b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1465777e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xed257780, SUC_lexicon_tags_11 },
    { 0xf6a57780, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a457784, SUC_lexicon_tags_25 },
    { 0xc2857785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4825778d, SUC_lexicon_tags_2 },
    { 0xcda5778e, SUC_lexicon_tags_28 },
    { 0x6c25778f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0457792, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4657795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd857798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa305779d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x725779f, SUC_lexicon_tags_33 },
    { 0xaa0577a0, SUC_lexicon_tags_9 },
    { 0x470577a1, SUC_lexicon_tags_17 },
    { 0x990577a2, SUC_lexicon_tags_17 },
    { 0xa38577a3, SUC_lexicon_tags_3 },
    { 0xed4577a3, SUC_lexicon_tags_37 },
    { 0x58c577a5, SUC_lexicon_tags_1 },
    { 0x3f8577a6, SUC_lexicon_tags_28 },
    { 0xe62577a6, SUC_lexicon_tags_14 },
    { 0x4a577a8, SUC_lexicon_tags_8 },
    { 0x260577a4, SUC_lexicon_tags_12 },
    { 0x1c8577a7, SUC_lexicon_tags_27 },
    { 0x7ee577ab, SUC_lexicon_tags_12 },
    { 0xe0577a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8577ae, SUC_lexicon_tags_7 },
    { 0x37e577ae, SUC_lexicon_tags_37 },
    { 0xb98577b0, SUC_lexicon_tags_0 },
    { 0x38c577b1, SUC_lexicon_tags_47 },
    { 0x622577b1, SUC_lexicon_tags_0 },
    { 0xd52577b3, SUC_lexicon_tags_19 },
    { 0xc0577b0, SUC_lexicon_tags_11 },
    { 0x894577b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c4577b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x518577ba, SUC_lexicon_tags_25 },
    { 0x30577ba, SUC_lexicon_tags_6 },
    { 0x102577bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb18577be, SUC_lexicon_tags_16 },
    { 0xa0c577bf, SUC_lexicon_tags_8 },
    { 0x3d8577c0, SUC_lexicon_tags_11 },
    { 0x1da577c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4577c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x440577cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f0577cd, SUC_lexicon_tags_11 },
    { 0xd6a577ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28577d9, SUC_lexicon_tags_29 },
    { 0x370577d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810577de, SUC_lexicon_tags_39 },
    { 0x48c577de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c577e3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2577e6, SUC_lexicon_tags_10 },
    { 0x7b4577e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0577ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0577f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x414577f3, SUC_lexicon_tags_49 },
    { 0x2e6577f4, SUC_lexicon_tags_47 },
    { 0x8c2577f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x880577f7, SUC_lexicon_tags_66 },
    { 0xef0577f8, SUC_lexicon_tags_17 },
    { 0x3d8577f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69a577fb, SUC_lexicon_tags_65 },
    { 0xf54577fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8577ff, SUC_lexicon_tags_2 },
    { 0xe6a57800, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57806, SUC_lexicon_tags_4 },
    { 0xc4a57807, SUC_lexicon_tags_55 },
    { 0xffe57807, SUC_lexicon_tags_11 },
    { 0xd9a57807, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd865780e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a57811, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca57816, SUC_lexicon_tags_2 },
    { 0x48057816, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd057819, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5885781f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b057821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc57826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc57829, SUC_lexicon_tags_2 },
    { 0x3005782a, SUC_lexicon_tags_8 },
    { 0xcaa57829, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6005782f, SUC_lexicon_tags_919 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10857835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2c57837, SUC_lexicon_tags_38 },
    { 0x30457838, SUC_lexicon_tags_38 },
    { 0xdc857839, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xab45783c, SUC_lexicon_tags_7 },
    { 0x6aa5783d, SUC_lexicon_tags_77 },
    { 0x1a65783e, SUC_lexicon_tags_2 },
    { 0xfc05783f, SUC_lexicon_tags_25 },
    { 0x9b657840, SUC_lexicon_tags_17 },
    { 0x6565783e, SUC_lexicon_tags_11 },
    { 0x39257842, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc57849, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c5784d, SUC_lexicon_tags_15 },
    { 0x29e5784d, SUC_lexicon_tags_335 },
    { 0x9da5784f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e57852, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda57855, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44c57857, SUC_lexicon_tags_15 },
    { 0xdea57858, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35257860, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e57865, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a57869, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb05786c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3057871, SUC_lexicon_tags_25 },
    { 0x43057872, SUC_lexicon_tags_1 },
    { 0x7a57873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a5787f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c57882, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x32a57884, SUC_lexicon_tags_63 },
    { 0x14e57885, SUC_lexicon_tags_1 },
    { 0x62057886, SUC_lexicon_tags_9 },
    { 0xdd057887, SUC_lexicon_tags_10 },
    { 0xe8857888, SUC_lexicon_tags_12 },
    { 0xc7a57889, SUC_lexicon_tags_12 },
    { 0x86057889, SUC_lexicon_tags_17 },
    { 0xfee5788b, SUC_lexicon_tags_28 },
    { 0xf1a5788c, SUC_lexicon_tags_0 },
    { 0x1ba57889, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5485788f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a57892, SUC_lexicon_tags_13 },
    { 0x2ca57893, SUC_lexicon_tags_60 },
    { 0xa4e57894, SUC_lexicon_tags_7 },
    { 0x47457895, SUC_lexicon_tags_14 },
    { 0x857896, SUC_lexicon_tags_33 },
    { 0x2a257896, SUC_lexicon_tags_10 },
    { 0x2ee57896, SUC_lexicon_tags_9 },
    { 0xc8857896, SUC_lexicon_tags_129 },
    { 0x85e57897, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882578a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6578ad, SUC_lexicon_tags_27 },
    { 0x4cc578ae, SUC_lexicon_tags_25 },
    { 0x808578af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e578b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa578b4, SUC_lexicon_tags_2 },
    { 0xdaa578b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0578bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac578bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a578c2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x32c578c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc86578c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782578d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84578d7, SUC_lexicon_tags_25 },
    { 0xe46578d8, SUC_lexicon_tags_11 },
    { 0x6e0578d8, SUC_lexicon_tags_29 },
    { 0x388578d7, SUC_lexicon_tags_15 },
    { 0xe04578d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152578e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd66578e4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xacc578e6, SUC_lexicon_tags_33 },
    { 0x510578e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a578ea, SUC_lexicon_tags_12 },
    { 0x690578eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224578f1, SUC_lexicon_tags_33 },
    { 0x3d8578f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e578f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcae578f9, SUC_lexicon_tags_45 },
    { 0xf80578fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x856578fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16578ff, SUC_lexicon_tags_30 },
    { 0x7f057900, SUC_lexicon_tags_37 },
    { 0xd5257901, SUC_lexicon_tags_47 },
    { 0xbe257902, SUC_lexicon_tags_63 },
    { 0x90657902, SUC_lexicon_tags_1 },
    { 0x7be57903, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a857909, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab85790e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a57911, SUC_lexicon_tags_1 },
    { 0x8c457911, SUC_lexicon_tags_7 },
    { 0xf2257913, SUC_lexicon_tags_25 },
    { 0x5857914, SUC_lexicon_tags_66 },
    { 0x5de57915, SUC_lexicon_tags_37 },
    { 0x86a57915, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7257918, SUC_lexicon_tags_14 },
    { 0xc7c57919, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbe65791b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5e5791d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce57920, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x12857923, SUC_lexicon_tags_1 },
    { 0xd1457924, SUC_lexicon_tags_12 },
    { 0x53257925, SUC_lexicon_tags_7 },
    { 0x6457926, SUC_lexicon_tags_11 },
    { 0x5e057927, SUC_lexicon_tags_26 },
    { 0x8ca57926, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da5792c, SUC_lexicon_tags_7 },
    { 0x7505792d, SUC_lexicon_tags_15 },
    { 0x91a5792e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6457931, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83657935, SUC_lexicon_tags_11 },
    { 0xd5057936, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a85793b, SUC_lexicon_tags_60 },
    { 0x33e5793b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbc5793e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x45657941, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47857945, SUC_lexicon_tags_49 },
    { 0x79c57946, SUC_lexicon_tags_0 },
    { 0xa9057946, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb65794a, SUC_lexicon_tags_15 },
    { 0xf485794b, SUC_lexicon_tags_293 },
    { 0x7925794c, SUC_lexicon_tags_23 },
    { 0x6225794a, SUC_lexicon_tags_38 },
    { 0xdfa5794b, SUC_lexicon_tags_17 },
    { 0x8dc5794e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e57951, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f257954, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24457958, SUC_lexicon_tags_15 },
    { 0xb8057959, SUC_lexicon_tags_66 },
    { 0x42e5795a, SUC_lexicon_tags_63 },
    { 0xfd05795b, SUC_lexicon_tags_1 },
    { 0xa57958, SUC_lexicon_tags_27 },
    { 0xa3a5795d, SUC_lexicon_tags_33 },
    { 0xf0e5795e, SUC_lexicon_tags_11 },
    { 0xdc5795f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf4057961, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dc57963, SUC_lexicon_tags_1 },
    { 0x94657963, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd785796a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e5796d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71c5796f, SUC_lexicon_tags_2 },
    { 0xf0a57970, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d45797a, SUC_lexicon_tags_15 },
    { 0x2fa5797a, SUC_lexicon_tags_18 },
    { 0xf7a5797c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a57981, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f457986, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2045798b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c05798d, SUC_lexicon_tags_15 },
    { 0xd745798e, SUC_lexicon_tags_30 },
    { 0x4d45798f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec57993, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5799a, SUC_lexicon_tags_15 },
    { 0x5b05799b, SUC_lexicon_tags_8 },
    { 0xb4c5799c, SUC_lexicon_tags_23 },
    { 0xf465799d, SUC_lexicon_tags_7 },
    { 0x78c5799e, SUC_lexicon_tags_37 },
    { 0xd9e5799f, SUC_lexicon_tags_47 },
    { 0x71e579a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e579a3, SUC_lexicon_tags_37 },
    { 0x7cc579a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace579ac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x330579ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8579b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x44a579b5, SUC_lexicon_tags_17 },
    { 0x2f6579b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x916579b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6579bb, SUC_lexicon_tags_14 },
    { 0xde0579bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a579c6, SUC_lexicon_tags_1 },
    { 0x7a6579c7, SUC_lexicon_tags_32 },
    { 0x736579c7, SUC_lexicon_tags_37 },
    { 0x9de579c9, SUC_lexicon_tags_1 },
    { 0xc3c579c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76579cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x952579d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8579d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c579d9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a579df, SUC_lexicon_tags_0 },
    { 0xd4c579e0, SUC_lexicon_tags_12 },
    { 0x8c8579e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40a579e3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128579ea, SUC_lexicon_tags_12 },
    { 0xc4579ea, SUC_lexicon_tags_9 },
    { 0xb98579ec, SUC_lexicon_tags_72 },
    { 0x8b8579ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe20579ef, SUC_lexicon_tags_25 },
    { 0x70a579f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f8579f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e579f4, SUC_lexicon_tags_25 },
    { 0xdba579f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36579f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61a579fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376579ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6257a02, SUC_lexicon_tags_43 },
    { 0x7ae57a03, SUC_lexicon_tags_9 },
    { 0xdba57a04, SUC_lexicon_tags_17 },
    { 0x55657a04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb257a07, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdc857a09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93657a0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7457a0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72457a15, SUC_lexicon_tags_0 },
    { 0x7257a16, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca57a19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc857a1e, SUC_lexicon_tags_61 },
    { 0xcd457a1f, SUC_lexicon_tags_16 },
    { 0x30c57a20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb257a23, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc57a29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3da57a2b, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac57a2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1857a30, SUC_lexicon_tags_8 },
    { 0xc1257a30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6057a3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x49c57a3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56857a40, SUC_lexicon_tags_25 },
    { 0x82057a41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba257a46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d657a4b, SUC_lexicon_tags_2 },
    { 0x59057a4b, SUC_lexicon_tags_27 },
    { 0xf5057a4d, SUC_lexicon_tags_6 },
    { 0xfa57a4e, SUC_lexicon_tags_77 },
    { 0xec057a4f, SUC_lexicon_tags_33 },
    { 0x71457a50, SUC_lexicon_tags_2 },
    { 0xd7657a51, SUC_lexicon_tags_9 },
    { 0xb5657a52, SUC_lexicon_tags_25 },
    { 0x63257a51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf0457a55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31a57a57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae57a5a, SUC_lexicon_tags_17 },
    { 0xf6257a5a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6ce57a5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x73857a60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a457a62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed257a65, SUC_lexicon_tags_2 },
    { 0xa9a57a66, SUC_lexicon_tags_15 },
    { 0x46a57a67, SUC_lexicon_tags_9 },
    { 0x5857a67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc457a6b, SUC_lexicon_tags_60 },
    { 0x28857a6c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43057a6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55057a72, SUC_lexicon_tags_9 },
    { 0x2d657a73, SUC_lexicon_tags_12 },
    { 0x75e57a73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0257a78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd4457a7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58257a87, SUC_lexicon_tags_11 },
    { 0xb7e57a88, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdb857a8a, SUC_lexicon_tags_17 },
    { 0x16457a8a, SUC_lexicon_tags_2 },
    { 0xc0a57a8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4857a8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a57a95, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca57aa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe657aa4, SUC_lexicon_tags_12 },
    { 0x89457aa4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b457aa8, SUC_lexicon_tags_12 },
    { 0x1b857aa9, SUC_lexicon_tags_9 },
    { 0xd1257aaa, SUC_lexicon_tags_17 },
    { 0xb7857aab, SUC_lexicon_tags_65 },
    { 0xd857aa8, SUC_lexicon_tags_17 },
    { 0xb8657aad, SUC_lexicon_tags_48 },
    { 0x2f257aaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae57ab2, SUC_lexicon_tags_0 },
    { 0xe4a57ab3, SUC_lexicon_tags_1 },
    { 0x96257ab3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c57ab7, SUC_lexicon_tags_628 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3257ac1, SUC_lexicon_tags_25 },
    { 0x8f457ac2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e857ac5, SUC_lexicon_tags_11 },
    { 0x38657ac5, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c257ac9, SUC_lexicon_tags_12 },
    { 0x7257aca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad857acc, SUC_lexicon_tags_17 },
    { 0xc2057acd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0857ad0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c57ad6, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba57ade, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f057ae0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ea57ae2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56257ae6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f057ae9, SUC_lexicon_tags_20 },
    { 0x3f457aea, SUC_lexicon_tags_63 },
    { 0xa0257aeb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a857aed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98057af0, SUC_lexicon_tags_8 },
    { 0x9ce57af1, SUC_lexicon_tags_0 },
    { 0xcc657af0, SUC_lexicon_tags_11 },
    { 0xbf657af3, SUC_lexicon_tags_27 },
    { 0xe8657af4, SUC_lexicon_tags_28 },
    { 0xdfc57af5, SUC_lexicon_tags_1 },
    { 0xa2857af5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e57afe, SUC_lexicon_tags_34 },
    { 0xbcc57aff, SUC_lexicon_tags_14 },
    { 0xdc057b00, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa4c57b02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24057b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeba57b06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x91857b09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25657b0c, SUC_lexicon_tags_37 },
    { 0x3e257b0d, SUC_lexicon_tags_11 },
    { 0x3ce57b0e, SUC_lexicon_tags_1 },
    { 0x5257b0f, SUC_lexicon_tags_7 },
    { 0xac457b10, SUC_lexicon_tags_37 },
    { 0x1e57b11, SUC_lexicon_tags_63 },
    { 0xbd057b12, SUC_lexicon_tags_25 },
    { 0x2a057b12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a57b18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa7e57b1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a057b25, SUC_lexicon_tags_37 },
    { 0x8a857b25, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3057b29, SUC_lexicon_tags_15 },
    { 0x9d257b2a, SUC_lexicon_tags_116 },
    { 0xbf257b2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x44a57b2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c57b33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc57b37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8857b47, SUC_lexicon_tags_47 },
    { 0x22657b48, SUC_lexicon_tags_14 },
    { 0xd4e57b48, SUC_lexicon_tags_19 },
    { 0xae257b4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1057b4e, SUC_lexicon_tags_42 },
    { 0x61257b4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea657b57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2ce57b59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3057b5e, SUC_lexicon_tags_12 },
    { 0x4057b5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5857b61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c57b67, SUC_lexicon_tags_47 },
    { 0xee057b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31457b6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8457b6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e57b71, SUC_lexicon_tags_7 },
    { 0x7e657b72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2657b75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a57b78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b257b82, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa4457b84, SUC_lexicon_tags_25 },
    { 0x76057b85, SUC_lexicon_tags_47 },
    { 0xd5657b86, SUC_lexicon_tags_19 },
    { 0x5d057b87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf457b8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71a57b8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73657b95, SUC_lexicon_tags_25 },
    { 0xdd857b95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81e57b98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71057ba0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c57ba4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69057ba7, SUC_lexicon_tags_7 },
    { 0x27c57ba8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57857bb1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66457bb9, SUC_lexicon_tags_12 },
    { 0x3fa57bba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72657bbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0857bc0, SUC_lexicon_tags_33 },
    { 0xa5057bc1, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x55657bc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5e57bc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6657bc8, SUC_lexicon_tags_7 },
    { 0xfba57bc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a57bcc, SUC_lexicon_tags_84 },
    { 0xa1e57bcd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe3057bcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8257bd2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e57bd7, SUC_lexicon_tags_19 },
    { 0xb1057bd8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc57bdb, SUC_lexicon_tags_13 },
    { 0x3b857bdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c57be5, SUC_lexicon_tags_29 },
    { 0x5a57be6, SUC_lexicon_tags_15 },
    { 0x7a257be7, SUC_lexicon_tags_45 },
    { 0xe9457be8, SUC_lexicon_tags_15 },
    { 0x7ee57be8, SUC_lexicon_tags_30 },
    { 0xadc57bea, SUC_lexicon_tags_1 },
    { 0x59057bea, SUC_lexicon_tags_66 },
    { 0x88857be9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4457bf0, SUC_lexicon_tags_37 },
    { 0xa1257bf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3e57bf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93057bf8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd657bff, SUC_lexicon_tags_37 },
    { 0x37057c00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ec57c02, SUC_lexicon_tags_17 },
    { 0x67057c03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1657c09, SUC_lexicon_tags_48 },
    { 0x33a57c0a, SUC_lexicon_tags_17 },
    { 0xe9a57c0b, SUC_lexicon_tags_12 },
    { 0xede57c0a, SUC_lexicon_tags_65 },
    { 0x7a857c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d457c10, SUC_lexicon_tags_15 },
    { 0xf5657c11, SUC_lexicon_tags_11 },
    { 0x5a257c11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3657c16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3d057c18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73c57c1a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc4e57c1c, SUC_lexicon_tags_892 },
    { 0xe5057c1d, SUC_lexicon_tags_17 },
    { 0xea657c1e, SUC_lexicon_tags_2 },
    { 0x78c57c1e, SUC_lexicon_tags_37 },
    { 0x3f057c1e, SUC_lexicon_tags_1 },
    { 0x68c57c1e, SUC_lexicon_tags_27 },
    { 0xd9457c22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a57c25, SUC_lexicon_tags_47 },
    { 0x18657c26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5057c29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc6857c2b, SUC_lexicon_tags_17 },
    { 0x43a57c2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca57c30, SUC_lexicon_tags_28 },
    { 0xc4e57c31, SUC_lexicon_tags_37 },
    { 0x92257c31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8057c3a, SUC_lexicon_tags_10 },
    { 0xa7257c3a, SUC_lexicon_tags_30 },
    { 0x22057c3c, SUC_lexicon_tags_29 },
    { 0x5fa57c3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84857c3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e57c49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c57c4d, SUC_lexicon_tags_57 },
    { 0x6f057c4e, SUC_lexicon_tags_65 },
    { 0x3f657c4e, SUC_lexicon_tags_100 },
    { 0x6ee57c50, SUC_lexicon_tags_32 },
    { 0xce457c4e, SUC_lexicon_tags_29 },
    { 0xea257c52, SUC_lexicon_tags_37 },
    { 0x6cc57c52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13457c55, SUC_lexicon_tags_9 },
    { 0x8d057c56, SUC_lexicon_tags_259 },
    { 0xf2457c57, SUC_lexicon_tags_25 },
    { 0xb3257c57, SUC_lexicon_tags_46 },
    { 0x1a257c59, SUC_lexicon_tags_28 },
    { 0xc3a57c5a, SUC_lexicon_tags_28 },
    { 0xd1057c5a, SUC_lexicon_tags_55 },
    { 0x73057c5b, SUC_lexicon_tags_4 },
    { 0x48857c5d, SUC_lexicon_tags_30 },
    { 0x13657c5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57c62, SUC_lexicon_tags_49 },
    { 0x31c57c63, SUC_lexicon_tags_11 },
    { 0x7b657c63, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x21a57c66, SUC_lexicon_tags_7 },
    { 0x98057c67, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa57c6b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb857c6d, SUC_lexicon_tags_62 },
    { 0x72c57c6e, SUC_lexicon_tags_28 },
    { 0xfa457c6f, SUC_lexicon_tags_37 },
    { 0x97057c6f, SUC_lexicon_tags_7 },
    { 0x63e57c71, SUC_lexicon_tags_2 },
    { 0x2a857c71, SUC_lexicon_tags_10 },
    { 0xf4257c72, SUC_lexicon_tags_12 },
    { 0x3f657c74, SUC_lexicon_tags_1 },
    { 0x1d657c75, SUC_lexicon_tags_2 },
    { 0x16857c71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d057c79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45657c7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e57c85, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57c8b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac57c8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0857c92, SUC_lexicon_tags_45 },
    { 0x78457c93, SUC_lexicon_tags_11 },
    { 0xabc57c92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9857c96, SUC_lexicon_tags_1 },
    { 0x39057c97, SUC_lexicon_tags_8 },
    { 0x81e57c96, SUC_lexicon_tags_1 },
    { 0x79a57c99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a57c9e, SUC_lexicon_tags_49 },
    { 0x25857c9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1457ca4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba57ca7, SUC_lexicon_tags_46 },
    { 0x64e57ca8, SUC_lexicon_tags_816 },
    { 0, NULL },
    { 0, NULL },
    { 0x98857cab, SUC_lexicon_tags_15 },
    { 0x35257cac, SUC_lexicon_tags_37 },
    { 0x32e57cad, SUC_lexicon_tags_16 },
    { 0x3b057cae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85457cb6, SUC_lexicon_tags_25 },
    { 0x9da57cb7, SUC_lexicon_tags_66 },
    { 0x16457cb8, SUC_lexicon_tags_7 },
    { 0x4257cb9, SUC_lexicon_tags_43 },
    { 0xcdc57cba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc7e57cbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e57cbf, SUC_lexicon_tags_15 },
    { 0x6657cc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1ca57cc2, SUC_lexicon_tags_37 },
    { 0x1257cc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd057cc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6657cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42057ccb, SUC_lexicon_tags_1 },
    { 0x8b857ccc, SUC_lexicon_tags_12 },
    { 0x2da57ccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a657ccf, SUC_lexicon_tags_31 },
    { 0x36e57cd0, SUC_lexicon_tags_1 },
    { 0xe0457cd1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x85457cd3, SUC_lexicon_tags_2 },
    { 0x41257cd4, SUC_lexicon_tags_11 },
    { 0x5b057cd5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4aa57cd7, SUC_lexicon_tags_17 },
    { 0x21e57cd8, SUC_lexicon_tags_15 },
    { 0x23a57cd8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3da57cdb, SUC_lexicon_tags_15 },
    { 0xed857cdc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e57cdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30657ce4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96857ce6, SUC_lexicon_tags_2 },
    { 0xb7057ce6, SUC_lexicon_tags_37 },
    { 0x17c57ce8, SUC_lexicon_tags_17 },
    { 0x1d057ce9, SUC_lexicon_tags_12 },
    { 0xe0457ce9, SUC_lexicon_tags_15 },
    { 0x14e57ceb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xce057cee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc57cf1, SUC_lexicon_tags_3 },
    { 0x2f057cf2, SUC_lexicon_tags_37 },
    { 0x52c57cf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64657cf7, SUC_lexicon_tags_11 },
    { 0x28257cf8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x14857cfa, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb457cfe, SUC_lexicon_tags_17 },
    { 0x9e457cfe, SUC_lexicon_tags_33 },
    { 0xb3a57cfe, SUC_lexicon_tags_12 },
    { 0x33857d00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfee57d03, SUC_lexicon_tags_80 },
    { 0x28657d03, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x21a57d06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4657d0a, SUC_lexicon_tags_12 },
    { 0xba257d0b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe4057d0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22e57d0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61857d13, SUC_lexicon_tags_19 },
    { 0x87257d13, SUC_lexicon_tags_45 },
    { 0xe2a57d14, SUC_lexicon_tags_2 },
    { 0x3a057d15, SUC_lexicon_tags_22 },
    { 0xb3657d17, SUC_lexicon_tags_11 },
    { 0xbbc57d17, SUC_lexicon_tags_37 },
    { 0xb3457d19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e57d1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e57d20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfdc57d22, SUC_lexicon_tags_31 },
    { 0x42c57d23, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x65457d25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0457d29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8257d2b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c57d2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79a57d31, SUC_lexicon_tags_39 },
    { 0x93457d32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c57d35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98657d37, SUC_lexicon_tags_23 },
    { 0xd3c57d37, SUC_lexicon_tags_12 },
    { 0xa3257d38, SUC_lexicon_tags_11 },
    { 0x64257d3a, SUC_lexicon_tags_62 },
    { 0x36657d3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68057d3d, SUC_lexicon_tags_64 },
    { 0x4657d3d, SUC_lexicon_tags_21 },
    { 0x5e57d3f, SUC_lexicon_tags_0 },
    { 0x61e57d40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18a57d42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22057d45, SUC_lexicon_tags_37 },
    { 0xdb457d46, SUC_lexicon_tags_12 },
    { 0x86457d45, SUC_lexicon_tags_60 },
    { 0x5f257d48, SUC_lexicon_tags_37 },
    { 0x70a57d48, SUC_lexicon_tags_15 },
    { 0x3b257d4a, SUC_lexicon_tags_12 },
    { 0x1ba57d4a, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a57d4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e57d51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55857d58, SUC_lexicon_tags_7 },
    { 0x90657d59, SUC_lexicon_tags_37 },
    { 0x20057d59, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x33a57d5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47657d5f, SUC_lexicon_tags_25 },
    { 0xd0c57d60, SUC_lexicon_tags_1 },
    { 0x4d657d60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a057d68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca57d6b, SUC_lexicon_tags_16 },
    { 0xd9c57d6b, SUC_lexicon_tags_30 },
    { 0xd9857d6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbc857d6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7657d71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe057d73, SUC_lexicon_tags_12 },
    { 0xa2857d74, SUC_lexicon_tags_46 },
    { 0x4ea57d75, SUC_lexicon_tags_605 },
    { 0x3d857d73, SUC_lexicon_tags_43 },
    { 0xb4457d77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75257d79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be57d7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ea57d7e, SUC_lexicon_tags_4 },
    { 0xa6257d7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d257d81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39857d86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1657d8a, SUC_lexicon_tags_45 },
    { 0x7b057d8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39657d8d, SUC_lexicon_tags_48 },
    { 0x2e657d8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c57d93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c657d95, SUC_lexicon_tags_8 },
    { 0x37457d96, SUC_lexicon_tags_1 },
    { 0x47c57d95, SUC_lexicon_tags_7 },
    { 0x38857d97, SUC_lexicon_tags_32 },
    { 0xfc857d99, SUC_lexicon_tags_7 },
    { 0xfb057d9a, SUC_lexicon_tags_2 },
    { 0xa4c57d97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4657d9e, SUC_lexicon_tags_28 },
    { 0x23e57d9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c57da2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8457da5, SUC_lexicon_tags_55 },
    { 0x87e57da6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed657da9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37657dab, SUC_lexicon_tags_148 },
    { 0x1d857dab, SUC_lexicon_tags_4 },
    { 0x1bc57dad, SUC_lexicon_tags_12 },
    { 0x9c857dae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17a57db0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa3857db2, SUC_lexicon_tags_1 },
    { 0x19457db2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcc57db5, SUC_lexicon_tags_27 },
    { 0x35c57db6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0857db9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a57dbd, SUC_lexicon_tags_17 },
    { 0x68857dbe, SUC_lexicon_tags_27 },
    { 0x10c57dbf, SUC_lexicon_tags_25 },
    { 0xe9857dbe, SUC_lexicon_tags_17 },
    { 0x1e257dbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c057dc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc57dc7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1657dcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c57dd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cc57dd5, SUC_lexicon_tags_63 },
    { 0x5a657dd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49e57dd8, SUC_lexicon_tags_87 },
    { 0xdb857dd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4857ddb, SUC_lexicon_tags_2 },
    { 0x73e57ddb, SUC_lexicon_tags_25 },
    { 0x5c57ddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x257ddf, SUC_lexicon_tags_60 },
    { 0x6e257ddf, SUC_lexicon_tags_7 },
    { 0xb2e57ddf, SUC_lexicon_tags_26 },
    { 0xe4e57ddf, SUC_lexicon_tags_27 },
    { 0xfcc57de3, SUC_lexicon_tags_1 },
    { 0xd2a57de3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e57deb, SUC_lexicon_tags_26 },
    { 0x5f257dec, SUC_lexicon_tags_1 },
    { 0xeb657ded, SUC_lexicon_tags_37 },
    { 0xd4857dee, SUC_lexicon_tags_16 },
    { 0x13c57ded, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6c657df1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4457df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e257dfd, SUC_lexicon_tags_37 },
    { 0x38257dfe, SUC_lexicon_tags_49 },
    { 0x28257dfe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x59657e01, SUC_lexicon_tags_11 },
    { 0x18c57e02, SUC_lexicon_tags_77 },
    { 0xa0457e03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68857e05, SUC_lexicon_tags_23 },
    { 0xc8c57e06, SUC_lexicon_tags_17 },
    { 0x72257e07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e57e0a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc657e10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba657e13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be57e16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb8057e18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6057e26, SUC_lexicon_tags_31 },
    { 0xd7857e27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd7657e29, SUC_lexicon_tags_11 },
    { 0xbd257e2a, SUC_lexicon_tags_25 },
    { 0xad857e2b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59257e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe57e32, SUC_lexicon_tags_15 },
    { 0x78a57e32, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e57e36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8857e3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c657e45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2de57e47, SUC_lexicon_tags_14 },
    { 0x58257e47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27457e4e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd457e52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a57e55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9057e57, SUC_lexicon_tags_31 },
    { 0xbe257e57, SUC_lexicon_tags_9 },
    { 0x17a57e59, SUC_lexicon_tags_49 },
    { 0x8ce57e5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c57e5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb2c57e60, SUC_lexicon_tags_8 },
    { 0x5a57e61, SUC_lexicon_tags_77 },
    { 0x1e657e62, SUC_lexicon_tags_11 },
    { 0x85257e63, SUC_lexicon_tags_14 },
    { 0x59457e64, SUC_lexicon_tags_17 },
    { 0x57e57e65, SUC_lexicon_tags_61 },
    { 0xbc857e66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x77457e68, SUC_lexicon_tags_27 },
    { 0x15c57e69, SUC_lexicon_tags_48 },
    { 0xb8857e6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63857e6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea57e73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbe57e75, SUC_lexicon_tags_37 },
    { 0xed657e76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e57e79, SUC_lexicon_tags_19 },
    { 0xbb457e7a, SUC_lexicon_tags_84 },
    { 0x8ba57e7b, SUC_lexicon_tags_49 },
    { 0x5bc57e79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c57e80, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44657e8a, SUC_lexicon_tags_37 },
    { 0x68657e8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x88257e8e, SUC_lexicon_tags_12 },
    { 0xba57e8e, SUC_lexicon_tags_12 },
    { 0x8d857e8e, SUC_lexicon_tags_0 },
    { 0xdb457e91, SUC_lexicon_tags_2 },
    { 0xb4c57e90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b057e95, SUC_lexicon_tags_60 },
    { 0xdf657e95, SUC_lexicon_tags_46 },
    { 0x75457e97, SUC_lexicon_tags_1 },
    { 0xdd857e97, SUC_lexicon_tags_27 },
    { 0xa4257e97, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66657e9d, SUC_lexicon_tags_7 },
    { 0x50a57e9e, SUC_lexicon_tags_40 },
    { 0xe3457e9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7657ea5, SUC_lexicon_tags_6 },
    { 0xe4657ea6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb057ead, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9f857eaf, SUC_lexicon_tags_27 },
    { 0xb0057eb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2057eb8, SUC_lexicon_tags_7 },
    { 0x93657eb9, SUC_lexicon_tags_26 },
    { 0x43857eb9, SUC_lexicon_tags_11 },
    { 0x23257eb8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54e57ebd, SUC_lexicon_tags_19 },
    { 0xdf857ebe, SUC_lexicon_tags_92 },
    { 0x4c457ebe, SUC_lexicon_tags_7 },
    { 0x13e57ebe, SUC_lexicon_tags_11 },
    { 0xbe57ec1, SUC_lexicon_tags_15 },
    { 0x7ee57ec1, SUC_lexicon_tags_63 },
    { 0x13a57ec2, SUC_lexicon_tags_15 },
    { 0x31457ec4, SUC_lexicon_tags_32 },
    { 0xe9c57ec2, SUC_lexicon_tags_12 },
    { 0xa6857ec3, SUC_lexicon_tags_32 },
    { 0x4f257ec0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a57ecc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb257ed0, SUC_lexicon_tags_27 },
    { 0x65e57ed1, SUC_lexicon_tags_34 },
    { 0x82a57ed0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7657ed5, SUC_lexicon_tags_6 },
    { 0xc9a57ed6, SUC_lexicon_tags_133 },
    { 0xd2657ed6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc57edd, SUC_lexicon_tags_6 },
    { 0x79c57ede, SUC_lexicon_tags_82 },
    { 0x1e457edf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdca57ee1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x46e57ee3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10457ee9, SUC_lexicon_tags_45 },
    { 0x7fa57eea, SUC_lexicon_tags_6 },
    { 0x8dc57eea, SUC_lexicon_tags_12 },
    { 0x7d657eec, SUC_lexicon_tags_7 },
    { 0x23c57eed, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x28457eef, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xa6257ef1, SUC_lexicon_tags_60 },
    { 0x32a57ef2, SUC_lexicon_tags_7 },
    { 0x83e57ef2, SUC_lexicon_tags_3 },
    { 0x96257ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde457efa, SUC_lexicon_tags_17 },
    { 0xbe57efb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x88457efd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa457eff, SUC_lexicon_tags_12 },
    { 0x1a057f00, SUC_lexicon_tags_37 },
    { 0x3c457f01, SUC_lexicon_tags_30 },
    { 0x6c57f02, SUC_lexicon_tags_37 },
    { 0xcea57f00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb857f06, SUC_lexicon_tags_15 },
    { 0xb0a57f07, SUC_lexicon_tags_37 },
    { 0x6fc57f08, SUC_lexicon_tags_25 },
    { 0xd9057f09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e257f0c, SUC_lexicon_tags_27 },
    { 0xf5257f0d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x41e57f0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9ec57f11, SUC_lexicon_tags_1 },
    { 0x2057f12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e57f15, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a57f1b, SUC_lexicon_tags_6 },
    { 0x18a57f1c, SUC_lexicon_tags_43 },
    { 0xf657f1b, SUC_lexicon_tags_103 },
    { 0x1c57f1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51857f22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74457f24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e257f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2457f28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a57f2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6457f31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a657f33, SUC_lexicon_tags_23 },
    { 0x75e57f33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e57f3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f457f3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe257f45, SUC_lexicon_tags_26 },
    { 0xdce57f46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4857f49, SUC_lexicon_tags_26 },
    { 0xa1e57f4a, SUC_lexicon_tags_15 },
    { 0x54257f4b, SUC_lexicon_tags_60 },
    { 0x1457f4c, SUC_lexicon_tags_14 },
    { 0xada57f4a, SUC_lexicon_tags_37 },
    { 0x63657f4e, SUC_lexicon_tags_1 },
    { 0xe5057f4e, SUC_lexicon_tags_45 },
    { 0xe257f4f, SUC_lexicon_tags_37 },
    { 0xa0257f51, SUC_lexicon_tags_38 },
    { 0x88257f50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc0a57f54, SUC_lexicon_tags_17 },
    { 0xcb857f55, SUC_lexicon_tags_60 },
    { 0xbc457f56, SUC_lexicon_tags_174 },
    { 0x32457f54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x73657f59, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce57f5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3ec57f5e, SUC_lexicon_tags_14 },
    { 0x10a57f5f, SUC_lexicon_tags_68 },
    { 0xfc857f60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb857f64, SUC_lexicon_tags_48 },
    { 0xfc057f65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83657f68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26857f6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9657f70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1057f75, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x25057f77, SUC_lexicon_tags_12 },
    { 0x77457f78, SUC_lexicon_tags_7 },
    { 0x55c57f79, SUC_lexicon_tags_15 },
    { 0x39e57f7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95c57f7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f857f7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5657f87, SUC_lexicon_tags_74 },
    { 0xe3e57f87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2057f8a, SUC_lexicon_tags_38 },
    { 0x6e457f8a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea57f8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2257f90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xda57f92, SUC_lexicon_tags_8 },
    { 0x11257f93, SUC_lexicon_tags_39 },
    { 0xb8657f94, SUC_lexicon_tags_2 },
    { 0x93e57f93, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe7257f97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae057f9b, SUC_lexicon_tags_27 },
    { 0x55c57f9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba57f9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e57fa2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9457fa7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9257fab, SUC_lexicon_tags_37 },
    { 0xd6c57fab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89057fae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c857fb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c57fb6, SUC_lexicon_tags_6 },
    { 0xa2857fb6, SUC_lexicon_tags_16 },
    { 0x4ea57fb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac57fbb, SUC_lexicon_tags_37 },
    { 0x11c57fbc, SUC_lexicon_tags_29 },
    { 0x40657fbd, SUC_lexicon_tags_12 },
    { 0xede57fbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda257fc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30057fc8, SUC_lexicon_tags_7 },
    { 0xca57fc9, SUC_lexicon_tags_133 },
    { 0xe8a57fc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3857fcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52a57fce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c257fd1, SUC_lexicon_tags_1 },
    { 0xd0657fd2, SUC_lexicon_tags_14 },
    { 0x11e57fd2, SUC_lexicon_tags_1 },
    { 0x67457fd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6657fd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca57fda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecc57fdc, SUC_lexicon_tags_11 },
    { 0x41a57fdd, SUC_lexicon_tags_2 },
    { 0x50857fde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a57fe3, SUC_lexicon_tags_9 },
    { 0x93257fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da57fe8, SUC_lexicon_tags_11 },
    { 0x89657fe9, SUC_lexicon_tags_8 },
    { 0x38857fe9, SUC_lexicon_tags_32 },
    { 0x9f457fe9, SUC_lexicon_tags_29 },
    { 0x9e457fec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x57657fef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c57ff4, SUC_lexicon_tags_30 },
    { 0xbec57ff5, SUC_lexicon_tags_25 },
    { 0xa257ff6, SUC_lexicon_tags_1 },
    { 0xca457ff7, SUC_lexicon_tags_1 },
    { 0x44657ff7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x71257ffa, SUC_lexicon_tags_2 },
    { 0x4c257ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a57fff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3458004, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd8858006, SUC_lexicon_tags_33 },
    { 0xca258006, SUC_lexicon_tags_2 },
    { 0xab458008, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb725800a, SUC_lexicon_tags_68 },
    { 0x2b65800a, SUC_lexicon_tags_23 },
    { 0x3405800a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d25800f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2b658011, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34258019, SUC_lexicon_tags_20 },
    { 0x5e25801a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de5801d, SUC_lexicon_tags_49 },
    { 0x3c45801e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e58026, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca58029, SUC_lexicon_tags_10 },
    { 0x3f458029, SUC_lexicon_tags_14 },
    { 0xc1e5802a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e25802e, SUC_lexicon_tags_23 },
    { 0x6605802f, SUC_lexicon_tags_60 },
    { 0x8ca58030, SUC_lexicon_tags_80 },
    { 0x88858031, SUC_lexicon_tags_29 },
    { 0x9c258032, SUC_lexicon_tags_30 },
    { 0xef658031, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa58035, SUC_lexicon_tags_25 },
    { 0xb0c58036, SUC_lexicon_tags_45 },
    { 0x9f658035, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1ee58039, SUC_lexicon_tags_12 },
    { 0x96c5803a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac58040, SUC_lexicon_tags_17 },
    { 0xcac58041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b858043, SUC_lexicon_tags_43 },
    { 0xe8c58044, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35e58046, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb1c58048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40458053, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40e58055, SUC_lexicon_tags_37 },
    { 0x42e58056, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f45805a, SUC_lexicon_tags_14 },
    { 0x2165805b, SUC_lexicon_tags_6 },
    { 0x51e5805b, SUC_lexicon_tags_31 },
    { 0xada5805d, SUC_lexicon_tags_16 },
    { 0x3ce5805b, SUC_lexicon_tags_19 },
    { 0xe585805b, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0x67858062, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52458064, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d25806a, SUC_lexicon_tags_43 },
    { 0x9985806b, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa45806f, SUC_lexicon_tags_14 },
    { 0x68e58070, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf3a58072, SUC_lexicon_tags_15 },
    { 0xf4258073, SUC_lexicon_tags_29 },
    { 0xcd258074, SUC_lexicon_tags_1 },
    { 0xa7658074, SUC_lexicon_tags_15 },
    { 0x68458073, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3de58078, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0x2565807a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd225807c, SUC_lexicon_tags_42 },
    { 0x9445807c, SUC_lexicon_tags_27 },
    { 0xa85807e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c58083, SUC_lexicon_tags_6 },
    { 0xd5458084, SUC_lexicon_tags_37 },
    { 0x96258084, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae458087, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be5808a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x225808c, SUC_lexicon_tags_1 },
    { 0xcb65808d, SUC_lexicon_tags_7 },
    { 0xeb85808d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc658090, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5858093, SUC_lexicon_tags_17 },
    { 0xa9458094, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74580a1, SUC_lexicon_tags_37 },
    { 0xd14580a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c580a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a580aa, SUC_lexicon_tags_43 },
    { 0xa3c580aa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c580af, SUC_lexicon_tags_8 },
    { 0x91a580b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2580b6, SUC_lexicon_tags_53 },
    { 0x390580b7, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0x84e580b9, SUC_lexicon_tags_31 },
    { 0x852580b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90580be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x946580c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4580c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x78580c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102580cd, SUC_lexicon_tags_70 },
    { 0x334580cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6580d3, SUC_lexicon_tags_47 },
    { 0x3ee580d4, SUC_lexicon_tags_11 },
    { 0xfb0580d5, SUC_lexicon_tags_2 },
    { 0x8ac580d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x454580d9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0580e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e580e6, SUC_lexicon_tags_103 },
    { 0x68e580e7, SUC_lexicon_tags_17 },
    { 0x6fa580e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c580f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda580f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f4580f5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe580ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1258103, SUC_lexicon_tags_7 },
    { 0xec58104, SUC_lexicon_tags_12 },
    { 0x9be58105, SUC_lexicon_tags_16 },
    { 0xc2a58106, SUC_lexicon_tags_2 },
    { 0xf258103, SUC_lexicon_tags_48 },
    { 0x4458108, SUC_lexicon_tags_57 },
    { 0x3d058109, SUC_lexicon_tags_37 },
    { 0xe7e58106, SUC_lexicon_tags_27 },
    { 0xa7c5810b, SUC_lexicon_tags_1 },
    { 0x2325810c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1645810f, SUC_lexicon_tags_38 },
    { 0x5ac58110, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x37e58112, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec458116, SUC_lexicon_tags_17 },
    { 0xaf058117, SUC_lexicon_tags_37 },
    { 0x45a58118, SUC_lexicon_tags_25 },
    { 0xbf458119, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd945811b, SUC_lexicon_tags_25 },
    { 0xcb45811c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13c5811e, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0xbbe58120, SUC_lexicon_tags_29 },
    { 0x58658121, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c58125, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9658130, SUC_lexicon_tags_35 },
    { 0xa1258131, SUC_lexicon_tags_7 },
    { 0xcd458131, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a58137, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be5813b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc65813e, SUC_lexicon_tags_0 },
    { 0xe5c5813e, SUC_lexicon_tags_35 },
    { 0xf3658140, SUC_lexicon_tags_1 },
    { 0xdfe58140, SUC_lexicon_tags_16 },
    { 0x9aa58142, SUC_lexicon_tags_396 },
    { 0xe1c58143, SUC_lexicon_tags_25 },
    { 0x11258144, SUC_lexicon_tags_11 },
    { 0x9ba58145, SUC_lexicon_tags_1 },
    { 0xb7c58142, SUC_lexicon_tags_1 },
    { 0x34458140, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x41058149, SUC_lexicon_tags_31 },
    { 0xd225814a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa845814e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16458152, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e58155, SUC_lexicon_tags_15 },
    { 0x15658156, SUC_lexicon_tags_12 },
    { 0x37a58157, SUC_lexicon_tags_32 },
    { 0xb8058157, SUC_lexicon_tags_11 },
    { 0xe658159, SUC_lexicon_tags_46 },
    { 0x2e258159, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7485816b, SUC_lexicon_tags_16 },
    { 0xeca5816c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc58171, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3458175, SUC_lexicon_tags_2 },
    { 0xc8258176, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x55258178, SUC_lexicon_tags_14 },
    { 0xec258179, SUC_lexicon_tags_37 },
    { 0xa105817a, SUC_lexicon_tags_6 },
    { 0x6b258178, SUC_lexicon_tags_8 },
    { 0x6f05817c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4505817f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e858182, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3458185, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f058187, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e5818f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7458193, SUC_lexicon_tags_29 },
    { 0xe2858193, SUC_lexicon_tags_31 },
    { 0xcfc58194, SUC_lexicon_tags_37 },
    { 0x1aa58196, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xe8058198, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb85819c, SUC_lexicon_tags_43 },
    { 0x4a85819d, SUC_lexicon_tags_37 },
    { 0x6ba5819c, SUC_lexicon_tags_19 },
    { 0x3a85819f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc581a2, SUC_lexicon_tags_23 },
    { 0xac0581a2, SUC_lexicon_tags_23 },
    { 0x87e581a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e581aa, SUC_lexicon_tags_0 },
    { 0xa1c581aa, SUC_lexicon_tags_17 },
    { 0xbd2581ac, SUC_lexicon_tags_38 },
    { 0xca581aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28581b0, SUC_lexicon_tags_11 },
    { 0x264581b1, SUC_lexicon_tags_19 },
    { 0xbde581b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2581bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0581c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830581c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544581cf, SUC_lexicon_tags_141 },
    { 0x362581d0, SUC_lexicon_tags_17 },
    { 0x8b2581cf, SUC_lexicon_tags_34 },
    { 0xd2a581d2, SUC_lexicon_tags_12 },
    { 0xf22581d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210581db, SUC_lexicon_tags_25 },
    { 0xf42581dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc581df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x102581e1, SUC_lexicon_tags_66 },
    { 0xde4581e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a581e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60581e9, SUC_lexicon_tags_35 },
    { 0x3c0581e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44581ee, SUC_lexicon_tags_1 },
    { 0xada581ef, SUC_lexicon_tags_30 },
    { 0xf56581ef, SUC_lexicon_tags_807 },
    { 0x5a6581ee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2581f5, SUC_lexicon_tags_25 },
    { 0xf2c581f6, SUC_lexicon_tags_17 },
    { 0x62e581f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ba581f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a581fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b6581fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19058201, SUC_lexicon_tags_12 },
    { 0x10c58202, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a58206, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa58209, SUC_lexicon_tags_17 },
    { 0x3045820a, SUC_lexicon_tags_17 },
    { 0x6d658209, SUC_lexicon_tags_1 },
    { 0xb165820c, SUC_lexicon_tags_49 },
    { 0x2fe5820c, SUC_lexicon_tags_158 },
    { 0x6f25820e, SUC_lexicon_tags_11 },
    { 0x30e5820f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc58216, SUC_lexicon_tags_12 },
    { 0xaca58217, SUC_lexicon_tags_12 },
    { 0x55a58218, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae5821c, SUC_lexicon_tags_17 },
    { 0x45c5821d, SUC_lexicon_tags_7 },
    { 0x6ce5821e, SUC_lexicon_tags_12 },
    { 0x4d85821e, SUC_lexicon_tags_38 },
    { 0x18858220, SUC_lexicon_tags_17 },
    { 0x80858221, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce58225, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc5e58227, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38258234, SUC_lexicon_tags_11 },
    { 0xd9258234, SUC_lexicon_tags_0 },
    { 0xa0858234, SUC_lexicon_tags_30 },
    { 0x95658236, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f25823a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae5823e, SUC_lexicon_tags_37 },
    { 0x62c5823f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x24258242, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee58246, SUC_lexicon_tags_66 },
    { 0x3e058247, SUC_lexicon_tags_55 },
    { 0xad858248, SUC_lexicon_tags_18 },
    { 0xa58249, SUC_lexicon_tags_4 },
    { 0x14258248, SUC_lexicon_tags_38 },
    { 0x5d85824a, SUC_lexicon_tags_39 },
    { 0xb645824c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a5824f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6258255, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56258258, SUC_lexicon_tags_18 },
    { 0x9fa58259, SUC_lexicon_tags_25 },
    { 0x3c5825a, SUC_lexicon_tags_12 },
    { 0xdc5825a, SUC_lexicon_tags_15 },
    { 0xddc5825b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6725825f, SUC_lexicon_tags_27 },
    { 0x99c58260, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77a58262, SUC_lexicon_tags_17 },
    { 0x44458263, SUC_lexicon_tags_68 },
    { 0x43e58263, SUC_lexicon_tags_37 },
    { 0x14e58264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85658267, SUC_lexicon_tags_37 },
    { 0x90e58267, SUC_lexicon_tags_37 },
    { 0xdf658267, SUC_lexicon_tags_56 },
    { 0x5245826a, SUC_lexicon_tags_37 },
    { 0x7d258267, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a85826d, SUC_lexicon_tags_12 },
    { 0x2d05826e, SUC_lexicon_tags_48 },
    { 0x5de5826d, SUC_lexicon_tags_7 },
    { 0xa5a5826f, SUC_lexicon_tags_37 },
    { 0x9b058270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c58276, SUC_lexicon_tags_2 },
    { 0x2dc58277, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x98e58279, SUC_lexicon_tags_14 },
    { 0x85827a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d25827d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1458281, SUC_lexicon_tags_37 },
    { 0x9fa58282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e58285, SUC_lexicon_tags_23 },
    { 0xf4658286, SUC_lexicon_tags_28 },
    { 0x34858287, SUC_lexicon_tags_60 },
    { 0xbca58286, SUC_lexicon_tags_25 },
    { 0x35858289, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec5828d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdea5828f, SUC_lexicon_tags_25 },
    { 0xcb458290, SUC_lexicon_tags_32 },
    { 0x2a458291, SUC_lexicon_tags_85 },
    { 0x5cc58292, SUC_lexicon_tags_38 },
    { 0xdc258293, SUC_lexicon_tags_11 },
    { 0x41458291, SUC_lexicon_tags_27 },
    { 0xc0e5828f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa58299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2582a1, SUC_lexicon_tags_19 },
    { 0x842582a2, SUC_lexicon_tags_17 },
    { 0x8a582a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55c582a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e582a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a582ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5a582ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8582b0, SUC_lexicon_tags_16 },
    { 0x302582b0, SUC_lexicon_tags_6 },
    { 0x950582b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6582b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x202582ba, SUC_lexicon_tags_38 },
    { 0x6c0582ba, SUC_lexicon_tags_17 },
    { 0x9b6582bc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee582c1, SUC_lexicon_tags_49 },
    { 0x8a6582c1, SUC_lexicon_tags_28 },
    { 0xb32582c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59a582c5, SUC_lexicon_tags_12 },
    { 0xdc2582c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca4582c8, SUC_lexicon_tags_87 },
    { 0x3e0582c9, SUC_lexicon_tags_43 },
    { 0xf4c582ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc582d0, SUC_lexicon_tags_55 },
    { 0xb02582d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb6582d3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x12c582d5, SUC_lexicon_tags_10 },
    { 0x12582d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e582dc, SUC_lexicon_tags_7 },
    { 0x798582dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d6582df, SUC_lexicon_tags_0 },
    { 0xf2c582e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512582e7, SUC_lexicon_tags_77 },
    { 0x340582e8, SUC_lexicon_tags_63 },
    { 0x956582e9, SUC_lexicon_tags_2 },
    { 0x728582e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x286582ec, SUC_lexicon_tags_30 },
    { 0x8ca582ed, SUC_lexicon_tags_32 },
    { 0x8dc582ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6c582f0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x138582f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06582fa, SUC_lexicon_tags_49 },
    { 0x6ba582fb, SUC_lexicon_tags_12 },
    { 0x89c582fa, SUC_lexicon_tags_33 },
    { 0xd6e582fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e58303, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1458307, SUC_lexicon_tags_11 },
    { 0xa1058308, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea5830b, SUC_lexicon_tags_17 },
    { 0xff05830c, SUC_lexicon_tags_34 },
    { 0x4385830c, SUC_lexicon_tags_7 },
    { 0x1045830e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f058312, SUC_lexicon_tags_7 },
    { 0xfa658313, SUC_lexicon_tags_16 },
    { 0x52a58314, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a58319, SUC_lexicon_tags_11 },
    { 0xfa05831a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc65831e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd858328, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7065832a, SUC_lexicon_tags_12 },
    { 0x105832b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd5e5832d, SUC_lexicon_tags_10 },
    { 0xf5c5832d, SUC_lexicon_tags_14 },
    { 0x8185832f, SUC_lexicon_tags_11 },
    { 0x6665832e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc58335, SUC_lexicon_tags_17 },
    { 0x7b258336, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda258338, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67858340, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83a58342, SUC_lexicon_tags_15 },
    { 0xe7258342, SUC_lexicon_tags_25 },
    { 0xfdc58343, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab658347, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24058349, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac45834e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f258352, SUC_lexicon_tags_17 },
    { 0x68a58353, SUC_lexicon_tags_1 },
    { 0x80e58354, SUC_lexicon_tags_17 },
    { 0x20858352, SUC_lexicon_tags_37 },
    { 0x3ae58356, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e5835b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2658363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc5836b, SUC_lexicon_tags_15 },
    { 0xb745836b, SUC_lexicon_tags_12 },
    { 0xad45836b, SUC_lexicon_tags_19 },
    { 0xc365836e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33458375, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e58378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2aa5837a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2945837c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa58382, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b85838a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c58391, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e58395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce58398, SUC_lexicon_tags_3 },
    { 0xbfc58399, SUC_lexicon_tags_42 },
    { 0x92e58398, SUC_lexicon_tags_8 },
    { 0x2fe58399, SUC_lexicon_tags_33 },
    { 0xe458399, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6e5839e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1ec583a0, SUC_lexicon_tags_37 },
    { 0x3c583a1, SUC_lexicon_tags_37 },
    { 0xec583a2, SUC_lexicon_tags_25 },
    { 0xd48583a2, SUC_lexicon_tags_2 },
    { 0xf10583a4, SUC_lexicon_tags_7 },
    { 0x1ba583a2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xafe583a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c583aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a583b0, SUC_lexicon_tags_17 },
    { 0x926583b0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a583b8, SUC_lexicon_tags_11 },
    { 0x540583b8, SUC_lexicon_tags_26 },
    { 0x350583ba, SUC_lexicon_tags_0 },
    { 0x64e583bb, SUC_lexicon_tags_15 },
    { 0x4e0583ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c583bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e583c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac583cb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c583d0, SUC_lexicon_tags_15 },
    { 0x3a4583d1, SUC_lexicon_tags_12 },
    { 0xa4c583d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112583d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338583e1, SUC_lexicon_tags_79 },
    { 0xd4583e2, SUC_lexicon_tags_1072 },
    { 0x4ea583e3, SUC_lexicon_tags_37 },
    { 0xdde583e4, SUC_lexicon_tags_6 },
    { 0xf60583e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788583ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71c583ef, SUC_lexicon_tags_23 },
    { 0x1c583f0, SUC_lexicon_tags_15 },
    { 0x97c583f1, SUC_lexicon_tags_27 },
    { 0xf22583f1, SUC_lexicon_tags_66 },
    { 0x25c583f1, SUC_lexicon_tags_25 },
    { 0xf96583f4, SUC_lexicon_tags_6 },
    { 0x840583f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ec583f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x520583f9, SUC_lexicon_tags_15 },
    { 0x9b2583fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd84583fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca583ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9658401, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b058403, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33a58405, SUC_lexicon_tags_37 },
    { 0xe7e58406, SUC_lexicon_tags_29 },
    { 0x4ec58407, SUC_lexicon_tags_11 },
    { 0x6f258408, SUC_lexicon_tags_27 },
    { 0x98058407, SUC_lexicon_tags_47 },
    { 0x2605840a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3cc5840c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea58410, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44458412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaaa58414, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde58418, SUC_lexicon_tags_12 },
    { 0xa4058419, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee5841d, SUC_lexicon_tags_48 },
    { 0x2525841d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x73658420, SUC_lexicon_tags_2 },
    { 0x33658420, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfe58423, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105842f, SUC_lexicon_tags_48 },
    { 0xaea5842f, SUC_lexicon_tags_27 },
    { 0xd4458431, SUC_lexicon_tags_1 },
    { 0x72058432, SUC_lexicon_tags_7 },
    { 0x9058432, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f858438, SUC_lexicon_tags_19 },
    { 0xff658439, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda5843d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c58440, SUC_lexicon_tags_17 },
    { 0xc4858441, SUC_lexicon_tags_1 },
    { 0x91058441, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b85844b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd905844e, SUC_lexicon_tags_45 },
    { 0x75e5844e, SUC_lexicon_tags_7 },
    { 0xb725844e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x37258452, SUC_lexicon_tags_25 },
    { 0x76458453, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62858455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2765845c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xaee5845e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b858464, SUC_lexicon_tags_11 },
    { 0x59658464, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5846b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c858471, SUC_lexicon_tags_17 },
    { 0x21c58472, SUC_lexicon_tags_11 },
    { 0x73c58471, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe458478, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4085847d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f258480, SUC_lexicon_tags_121 },
    { 0xb5e58481, SUC_lexicon_tags_23 },
    { 0xc5a58480, SUC_lexicon_tags_103 },
    { 0x75058483, SUC_lexicon_tags_17 },
    { 0x66058484, SUC_lexicon_tags_7 },
    { 0x7a58484, SUC_lexicon_tags_1 },
    { 0x2ac58486, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x76258488, SUC_lexicon_tags_49 },
    { 0x50e58488, SUC_lexicon_tags_1 },
    { 0x1de5848a, SUC_lexicon_tags_17 },
    { 0xd705848b, SUC_lexicon_tags_25 },
    { 0x8a25848c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab25848e, SUC_lexicon_tags_8 },
    { 0xc185848e, SUC_lexicon_tags_63 },
    { 0x4965848e, SUC_lexicon_tags_15 },
    { 0xd265848f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89858495, SUC_lexicon_tags_37 },
    { 0x57258496, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6605849d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4925849f, SUC_lexicon_tags_25 },
    { 0xae5849f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x846584a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x826584a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a584ac, SUC_lexicon_tags_15 },
    { 0x38584ad, SUC_lexicon_tags_2 },
    { 0x4a0584ae, SUC_lexicon_tags_38 },
    { 0xfea584af, SUC_lexicon_tags_15 },
    { 0xc2584af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x168584b2, SUC_lexicon_tags_7 },
    { 0x5e584b2, SUC_lexicon_tags_4 },
    { 0x9aa584b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90e584b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4584bc, SUC_lexicon_tags_0 },
    { 0x830584bd, SUC_lexicon_tags_47 },
    { 0xe76584be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x406584c0, SUC_lexicon_tags_84 },
    { 0x878584c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978584c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a584c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc94584ca, SUC_lexicon_tags_230 },
    { 0xce4584cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6c584cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4584cf, SUC_lexicon_tags_37 },
    { 0x218584d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70584d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc64584d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce584da, SUC_lexicon_tags_25 },
    { 0xef8584da, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1f8584dd, SUC_lexicon_tags_15 },
    { 0xc1c584de, SUC_lexicon_tags_1 },
    { 0xc00584de, SUC_lexicon_tags_14 },
    { 0x188584de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2584e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x968584e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed8584e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e584f3, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x962584f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe584fc, SUC_lexicon_tags_14 },
    { 0xbac584fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdb2584ff, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2a858501, SUC_lexicon_tags_23 },
    { 0x17c58502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee58506, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca5850b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8965850d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a58510, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1e58512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ac58514, SUC_lexicon_tags_37 },
    { 0x3dc58515, SUC_lexicon_tags_0 },
    { 0x6c458516, SUC_lexicon_tags_83 },
    { 0xae058517, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be5851d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee58521, SUC_lexicon_tags_25 },
    { 0xd0658522, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcba58524, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6858529, SUC_lexicon_tags_61 },
    { 0xc5e58529, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e658530, SUC_lexicon_tags_61 },
    { 0xbfa58531, SUC_lexicon_tags_5 },
    { 0xd2a58532, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a58535, SUC_lexicon_tags_26 },
    { 0xf5458536, SUC_lexicon_tags_46 },
    { 0x7a658536, SUC_lexicon_tags_12 },
    { 0x6de58537, SUC_lexicon_tags_49 },
    { 0x1a858539, SUC_lexicon_tags_36 },
    { 0xac45853a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6825853c, SUC_lexicon_tags_1 },
    { 0x3085853d, SUC_lexicon_tags_2 },
    { 0x5845853e, SUC_lexicon_tags_15 },
    { 0xac25853f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3458541, SUC_lexicon_tags_38 },
    { 0x4e458542, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4ba58544, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb85854d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xacc5854f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca58554, SUC_lexicon_tags_12 },
    { 0x64c58554, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac5855e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a58563, SUC_lexicon_tags_12 },
    { 0x30658564, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6005856a, SUC_lexicon_tags_37 },
    { 0x7685856b, SUC_lexicon_tags_87 },
    { 0xbfc5856c, SUC_lexicon_tags_6 },
    { 0x485856d, SUC_lexicon_tags_17 },
    { 0xc1c5856d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d258570, SUC_lexicon_tags_27 },
    { 0xccc58571, SUC_lexicon_tags_25 },
    { 0xc0e58570, SUC_lexicon_tags_6 },
    { 0x86e58570, SUC_lexicon_tags_17 },
    { 0xef058574, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e25857b, SUC_lexicon_tags_37 },
    { 0xe7e5857c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb45857e, SUC_lexicon_tags_17 },
    { 0xf585857e, SUC_lexicon_tags_47 },
    { 0x7c658580, SUC_lexicon_tags_1 },
    { 0x6d258581, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62458583, SUC_lexicon_tags_25 },
    { 0x51a58584, SUC_lexicon_tags_2 },
    { 0xbd258583, SUC_lexicon_tags_32 },
    { 0x89858583, SUC_lexicon_tags_1 },
    { 0x45658587, SUC_lexicon_tags_2 },
    { 0x98458588, SUC_lexicon_tags_15 },
    { 0x92858589, SUC_lexicon_tags_45 },
    { 0x36058586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa258591, SUC_lexicon_tags_68 },
    { 0x6e458591, SUC_lexicon_tags_12 },
    { 0x49658591, SUC_lexicon_tags_60 },
    { 0x4b058592, SUC_lexicon_tags_17 },
    { 0xec58595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1858598, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960585a9, SUC_lexicon_tags_1 },
    { 0xfe6585a9, SUC_lexicon_tags_65 },
    { 0x2e4585ab, SUC_lexicon_tags_19 },
    { 0xc5c585ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4585af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21c585b1, SUC_lexicon_tags_37 },
    { 0xa34585b1, SUC_lexicon_tags_4 },
    { 0xeae585b3, SUC_lexicon_tags_65 },
    { 0xd0585b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4bc585b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde8585b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8585be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6585c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x46e585c4, SUC_lexicon_tags_9 },
    { 0x426585c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x268585c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd9e585c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00585d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4585d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x344585d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8585d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaca585db, SUC_lexicon_tags_14 },
    { 0xdfc585dc, SUC_lexicon_tags_37 },
    { 0xa76585dd, SUC_lexicon_tags_15 },
    { 0x932585dd, SUC_lexicon_tags_22 },
    { 0x208585df, SUC_lexicon_tags_16 },
    { 0x49a585e0, SUC_lexicon_tags_37 },
    { 0xbcc585e1, SUC_lexicon_tags_0 },
    { 0x13c585df, SUC_lexicon_tags_19 },
    { 0xf68585e3, SUC_lexicon_tags_1 },
    { 0xd00585e3, SUC_lexicon_tags_12 },
    { 0x6c4585e5, SUC_lexicon_tags_15 },
    { 0xd0585df, SUC_lexicon_tags_14 },
    { 0x1f0585e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8585ea, SUC_lexicon_tags_12 },
    { 0xff2585eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb8c585ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6585f3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e585f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd4585fb, SUC_lexicon_tags_31 },
    { 0x8e585fb, SUC_lexicon_tags_25 },
    { 0xddc585fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47a585ff, SUC_lexicon_tags_37 },
    { 0xdc0585ff, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0858603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62658605, SUC_lexicon_tags_37 },
    { 0x4dc58606, SUC_lexicon_tags_34 },
    { 0x64858607, SUC_lexicon_tags_35 },
    { 0x33e58607, SUC_lexicon_tags_23 },
    { 0xe1c58607, SUC_lexicon_tags_34 },
    { 0x6b058609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6a5860c, SUC_lexicon_tags_26 },
    { 0x125860d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf458610, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a58613, SUC_lexicon_tags_21 },
    { 0xa3a58613, SUC_lexicon_tags_23 },
    { 0xd8458615, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61658618, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a5861b, SUC_lexicon_tags_23 },
    { 0xa1c5861b, SUC_lexicon_tags_7 },
    { 0xd5e5861b, SUC_lexicon_tags_17 },
    { 0x10a5861d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a58625, SUC_lexicon_tags_49 },
    { 0x86058625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e58631, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x51458633, SUC_lexicon_tags_85 },
    { 0xf5e58634, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd9c58636, SUC_lexicon_tags_11 },
    { 0xa2458637, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c858639, SUC_lexicon_tags_12 },
    { 0xb9c5863a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c58641, SUC_lexicon_tags_33 },
    { 0xdfe58641, SUC_lexicon_tags_14 },
    { 0xf8458641, SUC_lexicon_tags_60 },
    { 0xa8058644, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee58648, SUC_lexicon_tags_25 },
    { 0x5c458648, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5125864d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59858650, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7258654, SUC_lexicon_tags_11 },
    { 0x57c58655, SUC_lexicon_tags_37 },
    { 0x98a58655, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb525865b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c58664, SUC_lexicon_tags_9 },
    { 0xb6a58664, SUC_lexicon_tags_37 },
    { 0xab658665, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b45866d, SUC_lexicon_tags_45 },
    { 0x3225866d, SUC_lexicon_tags_37 },
    { 0xea5866e, SUC_lexicon_tags_38 },
    { 0x80c58670, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8dc58672, SUC_lexicon_tags_17 },
    { 0x7a058673, SUC_lexicon_tags_38 },
    { 0xc858674, SUC_lexicon_tags_31 },
    { 0xc1e58675, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c458678, SUC_lexicon_tags_47 },
    { 0x38258679, SUC_lexicon_tags_25 },
    { 0x385867a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c58683, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca58689, SUC_lexicon_tags_35 },
    { 0x4fa5868a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b45868d, SUC_lexicon_tags_37 },
    { 0x32e5868d, SUC_lexicon_tags_84 },
    { 0xa265868e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a858693, SUC_lexicon_tags_7 },
    { 0xf8458694, SUC_lexicon_tags_2 },
    { 0x65658695, SUC_lexicon_tags_30 },
    { 0xe3e58696, SUC_lexicon_tags_17 },
    { 0xdc658696, SUC_lexicon_tags_35 },
    { 0xbce58698, SUC_lexicon_tags_19 },
    { 0xfea58693, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf585869c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d65869e, SUC_lexicon_tags_1 },
    { 0x4e85869f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da586a3, SUC_lexicon_tags_15 },
    { 0x88c586a4, SUC_lexicon_tags_11 },
    { 0x796586a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x684586a8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa586ac, SUC_lexicon_tags_17 },
    { 0x9ac586ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6586ba, SUC_lexicon_tags_1 },
    { 0x4ba586bb, SUC_lexicon_tags_70 },
    { 0x378586bb, SUC_lexicon_tags_15 },
    { 0x138586ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc96586bf, SUC_lexicon_tags_1 },
    { 0xc12586bf, SUC_lexicon_tags_49 },
    { 0xfca586bf, SUC_lexicon_tags_9 },
    { 0x25a586c1, SUC_lexicon_tags_28 },
    { 0x672586c2, SUC_lexicon_tags_0 },
    { 0x1b2586c4, SUC_lexicon_tags_95 },
    { 0xa24586c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6586ce, SUC_lexicon_tags_26 },
    { 0xf64586cf, SUC_lexicon_tags_43 },
    { 0xafa586ce, SUC_lexicon_tags_33 },
    { 0x80e586d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6586d7, SUC_lexicon_tags_1 },
    { 0x5f6586d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe0a586da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xace586dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d4586de, SUC_lexicon_tags_23 },
    { 0xf92586df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc586e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf7e586e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0586ea, SUC_lexicon_tags_30 },
    { 0x89e586eb, SUC_lexicon_tags_25 },
    { 0xabc586ec, SUC_lexicon_tags_12 },
    { 0x996586ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8586f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6586f9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8265870a, SUC_lexicon_tags_29 },
    { 0xc405870a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e25870e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce058712, SUC_lexicon_tags_0 },
    { 0x12658712, SUC_lexicon_tags_7 },
    { 0x83858714, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d458716, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a58719, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f05871c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7e5871e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9e58720, SUC_lexicon_tags_84 },
    { 0x69658721, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0858724, SUC_lexicon_tags_34 },
    { 0xfbe58725, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec58728, SUC_lexicon_tags_17 },
    { 0x27458728, SUC_lexicon_tags_28 },
    { 0x4345872a, SUC_lexicon_tags_45 },
    { 0xf0c5872b, SUC_lexicon_tags_17 },
    { 0xb2a5872b, SUC_lexicon_tags_17 },
    { 0x3905872d, SUC_lexicon_tags_17 },
    { 0x28e5872e, SUC_lexicon_tags_37 },
    { 0x7e05872f, SUC_lexicon_tags_25 },
    { 0xcae5872d, SUC_lexicon_tags_1 },
    { 0xcce58731, SUC_lexicon_tags_1 },
    { 0xb0e58731, SUC_lexicon_tags_33 },
    { 0xf458733, SUC_lexicon_tags_1 },
    { 0xba58733, SUC_lexicon_tags_30 },
    { 0x8c58734, SUC_lexicon_tags_15 },
    { 0x58a58736, SUC_lexicon_tags_25 },
    { 0x1e05872b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x105873a, SUC_lexicon_tags_32 },
    { 0x60e5873b, SUC_lexicon_tags_7 },
    { 0x9e5873c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd385873e, SUC_lexicon_tags_7 },
    { 0xa165873f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee58742, SUC_lexicon_tags_19 },
    { 0xc6458743, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8ea58745, SUC_lexicon_tags_0 },
    { 0x8ce58746, SUC_lexicon_tags_17 },
    { 0x8ae58746, SUC_lexicon_tags_25 },
    { 0x66c58746, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7a5874a, SUC_lexicon_tags_1 },
    { 0x845874b, SUC_lexicon_tags_2 },
    { 0x7bc5874c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a058753, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f458755, SUC_lexicon_tags_14 },
    { 0xf1a58756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36458758, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa5875c, SUC_lexicon_tags_27 },
    { 0x36a5875c, SUC_lexicon_tags_6 },
    { 0x9b25875e, SUC_lexicon_tags_11 },
    { 0xf645875d, SUC_lexicon_tags_82 },
    { 0xe965875f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c5876b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c5876e, SUC_lexicon_tags_21 },
    { 0x5885876e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a58772, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a58776, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xde258778, SUC_lexicon_tags_34 },
    { 0xb0c58779, SUC_lexicon_tags_416 },
    { 0xe945877a, SUC_lexicon_tags_13 },
    { 0x33e5877b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7965877d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee58780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba858785, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8725878c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd058790, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c58793, SUC_lexicon_tags_32 },
    { 0xb0a58794, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c58797, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf458799, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe65879f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb98587a1, SUC_lexicon_tags_38 },
    { 0xf22587a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x814587a4, SUC_lexicon_tags_27 },
    { 0xf2c587a4, SUC_lexicon_tags_25 },
    { 0xcaa587a6, SUC_lexicon_tags_6 },
    { 0x7fa587a6, SUC_lexicon_tags_2 },
    { 0x578587a4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x526587aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40587b0, SUC_lexicon_tags_25 },
    { 0x682587b1, SUC_lexicon_tags_11 },
    { 0xf18587b1, SUC_lexicon_tags_38 },
    { 0x60587b3, SUC_lexicon_tags_6 },
    { 0x674587b4, SUC_lexicon_tags_14 },
    { 0x194587b5, SUC_lexicon_tags_84 },
    { 0xde0587b6, SUC_lexicon_tags_11 },
    { 0x780587b7, SUC_lexicon_tags_3 },
    { 0xf06587b4, SUC_lexicon_tags_68 },
    { 0x17c587b1, SUC_lexicon_tags_7 },
    { 0x340587ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4587bd, SUC_lexicon_tags_4 },
    { 0x200587be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x902587c0, SUC_lexicon_tags_46 },
    { 0x4be587c1, SUC_lexicon_tags_11 },
    { 0x2a587c2, SUC_lexicon_tags_14 },
    { 0xc6587c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1e587c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac587ca, SUC_lexicon_tags_19 },
    { 0x408587cb, SUC_lexicon_tags_6 },
    { 0x4d8587ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38587cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70587d2, SUC_lexicon_tags_37 },
    { 0xf8587d3, SUC_lexicon_tags_37 },
    { 0x5ca587d4, SUC_lexicon_tags_37 },
    { 0xb8c587d5, SUC_lexicon_tags_48 },
    { 0x644587d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf34587d8, SUC_lexicon_tags_65 },
    { 0x5fa587d9, SUC_lexicon_tags_49 },
    { 0x64a587da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2587df, SUC_lexicon_tags_7 },
    { 0xc42587e0, SUC_lexicon_tags_27 },
    { 0x938587e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304587e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf68587e9, SUC_lexicon_tags_39 },
    { 0x498587ea, SUC_lexicon_tags_29 },
    { 0x160587ea, SUC_lexicon_tags_165 },
    { 0x80a587ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0587f0, SUC_lexicon_tags_37 },
    { 0x7d6587f1, SUC_lexicon_tags_21 },
    { 0xdc6587f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x374587f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe587f9, SUC_lexicon_tags_19 },
    { 0xa94587fa, SUC_lexicon_tags_7 },
    { 0xede587fb, SUC_lexicon_tags_70 },
    { 0x4c4587fc, SUC_lexicon_tags_49 },
    { 0x510587fd, SUC_lexicon_tags_118 },
    { 0x32587fe, SUC_lexicon_tags_6 },
    { 0x29a587ff, SUC_lexicon_tags_46 },
    { 0x9d6587ff, SUC_lexicon_tags_33 },
    { 0x33a587ff, SUC_lexicon_tags_28 },
    { 0x6dc58801, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36a58804, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3858806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885880b, SUC_lexicon_tags_1 },
    { 0xed05880b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde58813, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3258817, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x42258819, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ac5881b, SUC_lexicon_tags_15 },
    { 0x8305881c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e58826, SUC_lexicon_tags_17 },
    { 0x18e58827, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4658829, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x185882c, SUC_lexicon_tags_2 },
    { 0x6ac5882d, SUC_lexicon_tags_65 },
    { 0x36a5882d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66858830, SUC_lexicon_tags_12 },
    { 0xe0a58831, SUC_lexicon_tags_15 },
    { 0x14058832, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66858834, SUC_lexicon_tags_39 },
    { 0x50258835, SUC_lexicon_tags_34 },
    { 0x3058836, SUC_lexicon_tags_160 },
    { 0x36058837, SUC_lexicon_tags_28 },
    { 0x55a58838, SUC_lexicon_tags_11 },
    { 0x48458839, SUC_lexicon_tags_27 },
    { 0x47e5883a, SUC_lexicon_tags_11 },
    { 0x4465883b, SUC_lexicon_tags_62 },
    { 0x2b65883c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c58841, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0458845, SUC_lexicon_tags_15 },
    { 0x86a58846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc285884a, SUC_lexicon_tags_23 },
    { 0xad85884a, SUC_lexicon_tags_11 },
    { 0x9185884c, SUC_lexicon_tags_0 },
    { 0xd1c5884c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb05884f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e58852, SUC_lexicon_tags_2 },
    { 0xb5058853, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b45885c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf165885f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9058861, SUC_lexicon_tags_23 },
    { 0xe4e58862, SUC_lexicon_tags_12 },
    { 0xa2058862, SUC_lexicon_tags_47 },
    { 0x74858863, SUC_lexicon_tags_37 },
    { 0x90658862, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xed058867, SUC_lexicon_tags_12 },
    { 0xd258868, SUC_lexicon_tags_26 },
    { 0x3c58869, SUC_lexicon_tags_17 },
    { 0x74058869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6425886c, SUC_lexicon_tags_7 },
    { 0x8a45886d, SUC_lexicon_tags_11 },
    { 0x8a25886c, SUC_lexicon_tags_11 },
    { 0x6405886d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9258873, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27258875, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1258879, SUC_lexicon_tags_193 },
    { 0xc5c5887a, SUC_lexicon_tags_0 },
    { 0xc6a5887a, SUC_lexicon_tags_57 },
    { 0x5ca5887c, SUC_lexicon_tags_82 },
    { 0xb5c5887d, SUC_lexicon_tags_11 },
    { 0x20c5887e, SUC_lexicon_tags_43 },
    { 0xdcc5887e, SUC_lexicon_tags_11 },
    { 0x1d85887a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x41258882, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35458885, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e5888a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc58890, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc58895, SUC_lexicon_tags_26 },
    { 0x26e58895, SUC_lexicon_tags_17 },
    { 0xaa58897, SUC_lexicon_tags_43 },
    { 0x83a58898, SUC_lexicon_tags_15 },
    { 0x59658899, SUC_lexicon_tags_25 },
    { 0x1625889a, SUC_lexicon_tags_37 },
    { 0x43858896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c5889e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08588a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa26588a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a588ac, SUC_lexicon_tags_18 },
    { 0x4e4588ad, SUC_lexicon_tags_40 },
    { 0xdfa588ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ba588b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x588b2, SUC_lexicon_tags_17 },
    { 0x42e588b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x712588b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee588b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14e588b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc588bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c588bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98c588c1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4588c6, SUC_lexicon_tags_6 },
    { 0x29c588c7, SUC_lexicon_tags_49 },
    { 0xc9c588c7, SUC_lexicon_tags_6 },
    { 0x576588c9, SUC_lexicon_tags_12 },
    { 0xa56588ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e588cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c588d2, SUC_lexicon_tags_11 },
    { 0x1dc588d3, SUC_lexicon_tags_14 },
    { 0x70a588d4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8588d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44588df, SUC_lexicon_tags_27 },
    { 0x820588e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762588e4, SUC_lexicon_tags_2 },
    { 0x400588e5, SUC_lexicon_tags_35 },
    { 0x784588e6, SUC_lexicon_tags_25 },
    { 0xb98588e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212588ef, SUC_lexicon_tags_7 },
    { 0x276588f0, SUC_lexicon_tags_1 },
    { 0xb7c588f0, SUC_lexicon_tags_12 },
    { 0x3ba588f0, SUC_lexicon_tags_12 },
    { 0x580588f1, SUC_lexicon_tags_65 },
    { 0xee2588f4, SUC_lexicon_tags_25 },
    { 0x40e588f0, SUC_lexicon_tags_40 },
    { 0x754588f6, SUC_lexicon_tags_207 },
    { 0xa5a588f3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c588fd, SUC_lexicon_tags_29 },
    { 0x1c8588fe, SUC_lexicon_tags_29 },
    { 0x40a588ff, SUC_lexicon_tags_34 },
    { 0xcea58900, SUC_lexicon_tags_2 },
    { 0xfbe58901, SUC_lexicon_tags_23 },
    { 0xef258902, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0458909, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe645890b, SUC_lexicon_tags_6 },
    { 0x46a5890b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32a5890e, SUC_lexicon_tags_29 },
    { 0x5265890f, SUC_lexicon_tags_25 },
    { 0x43c5890e, SUC_lexicon_tags_17 },
    { 0xc905890f, SUC_lexicon_tags_77 },
    { 0xa8c58912, SUC_lexicon_tags_11 },
    { 0x26e5890f, SUC_lexicon_tags_16 },
    { 0x7ae58914, SUC_lexicon_tags_27 },
    { 0x3b458915, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2825891a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6e05891c, SUC_lexicon_tags_68 },
    { 0x6e25891c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe658921, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x54058923, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xbe58925, SUC_lexicon_tags_37 },
    { 0xee258926, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc625892a, SUC_lexicon_tags_17 },
    { 0xc565892a, SUC_lexicon_tags_60 },
    { 0x4325892c, SUC_lexicon_tags_9 },
    { 0xb8e5892b, SUC_lexicon_tags_68 },
    { 0xd785892e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c58931, SUC_lexicon_tags_7 },
    { 0xb1858932, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e58937, SUC_lexicon_tags_0 },
    { 0x9aa58938, SUC_lexicon_tags_60 },
    { 0xc8858939, SUC_lexicon_tags_11 },
    { 0x75a58938, SUC_lexicon_tags_108 },
    { 0x57c58938, SUC_lexicon_tags_2 },
    { 0x30658938, SUC_lexicon_tags_61 },
    { 0x4b65893d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e058940, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x19258942, SUC_lexicon_tags_230 },
    { 0x61058942, SUC_lexicon_tags_29 },
    { 0xa0458942, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6e58946, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc785894b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c858950, SUC_lexicon_tags_38 },
    { 0x48858950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1dc58953, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69258959, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de58963, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a58966, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc58970, SUC_lexicon_tags_0 },
    { 0x4c858970, SUC_lexicon_tags_17 },
    { 0x1a058972, SUC_lexicon_tags_28 },
    { 0x76258972, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc58976, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xde458979, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa58983, SUC_lexicon_tags_0 },
    { 0xc0458983, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x98658986, SUC_lexicon_tags_6 },
    { 0x2fa58986, SUC_lexicon_tags_9 },
    { 0x37a58988, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13a5898a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d25898f, SUC_lexicon_tags_15 },
    { 0x25a58990, SUC_lexicon_tags_269 },
    { 0x32058990, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x99858994, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e5899e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc28589a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe02589a2, SUC_lexicon_tags_37 },
    { 0xf68589a3, SUC_lexicon_tags_19 },
    { 0x4fc589a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xada589a7, SUC_lexicon_tags_17 },
    { 0x4de589a7, SUC_lexicon_tags_0 },
    { 0x82589a9, SUC_lexicon_tags_7 },
    { 0xa28589aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x540589ad, SUC_lexicon_tags_28 },
    { 0xca6589ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x25c589b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb74589b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b6589b4, SUC_lexicon_tags_7 },
    { 0x62a589b4, SUC_lexicon_tags_17 },
    { 0x83e589b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x784589b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe68589ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44589bf, SUC_lexicon_tags_74 },
    { 0x194589c0, SUC_lexicon_tags_15 },
    { 0xc86589bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x406589c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4589c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92589cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd72589d1, SUC_lexicon_tags_14 },
    { 0xe14589d1, SUC_lexicon_tags_8 },
    { 0xf80589d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e589d9, SUC_lexicon_tags_9 },
    { 0x62589da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672589e0, SUC_lexicon_tags_10 },
    { 0xa36589e0, SUC_lexicon_tags_37 },
    { 0xd44589e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x722589e5, SUC_lexicon_tags_1 },
    { 0x448589e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6589ec, SUC_lexicon_tags_43 },
    { 0xaac589ed, SUC_lexicon_tags_11 },
    { 0xbdc589ed, SUC_lexicon_tags_15 },
    { 0xf94589ef, SUC_lexicon_tags_7 },
    { 0x8ec589ee, SUC_lexicon_tags_14 },
    { 0x2d2589ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e589f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4e6589f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794589fa, SUC_lexicon_tags_6 },
    { 0x8cc589fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e58a03, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x42058a06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8458a0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a58a12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5658a15, SUC_lexicon_tags_6 },
    { 0x9658a16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa7258a18, SUC_lexicon_tags_9 },
    { 0x80c58a19, SUC_lexicon_tags_37 },
    { 0x9ca58a18, SUC_lexicon_tags_37 },
    { 0x3858a19, SUC_lexicon_tags_17 },
    { 0x5c058a1c, SUC_lexicon_tags_25 },
    { 0xc6658a1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x55058a1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19658a21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffc58a23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf1658a25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13e58a27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37258a2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee58a32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c58a38, SUC_lexicon_tags_17 },
    { 0x81058a39, SUC_lexicon_tags_49 },
    { 0xcae58a39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8058a3d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda58a42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x34258a44, SUC_lexicon_tags_39 },
    { 0x2ac58a45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ca58a47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c058a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd258a52, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c58a5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6858a67, SUC_lexicon_tags_17 },
    { 0xa6e58a68, SUC_lexicon_tags_25 },
    { 0x58458a68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87058a6b, SUC_lexicon_tags_23 },
    { 0xe9e58a6c, SUC_lexicon_tags_27 },
    { 0xa858a6b, SUC_lexicon_tags_2 },
    { 0x97858a6e, SUC_lexicon_tags_37 },
    { 0xefe58a6e, SUC_lexicon_tags_12 },
    { 0xb5258a70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb258a72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x81858a75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61458a77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62058a7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x54458a7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e58a7f, SUC_lexicon_tags_33 },
    { 0xdd658a80, SUC_lexicon_tags_9 },
    { 0x62058a81, SUC_lexicon_tags_12 },
    { 0xf9058a82, SUC_lexicon_tags_11 },
    { 0x4ea58a81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e58a88, SUC_lexicon_tags_1 },
    { 0xc6058a89, SUC_lexicon_tags_55 },
    { 0x6d458a8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98658a8e, SUC_lexicon_tags_120 },
    { 0x4a458a8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49e58a91, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x69658a93, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xab258a95, SUC_lexicon_tags_37 },
    { 0xe3a58a96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9f458a98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42258a9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8058a9c, SUC_lexicon_tags_33 },
    { 0x2fa58a9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1858aa0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da58aa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48258aa6, SUC_lexicon_tags_45 },
    { 0x87e58aa7, SUC_lexicon_tags_10 },
    { 0x37258aa8, SUC_lexicon_tags_12 },
    { 0xc4258aa9, SUC_lexicon_tags_12 },
    { 0x7a858aaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6458aad, SUC_lexicon_tags_15 },
    { 0xc0a58aad, SUC_lexicon_tags_17 },
    { 0xce858aad, SUC_lexicon_tags_31 },
    { 0xbec58aaf, SUC_lexicon_tags_11 },
    { 0x53058ab0, SUC_lexicon_tags_4 },
    { 0x2c858ab2, SUC_lexicon_tags_13 },
    { 0xf1658ab3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8458ab5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7458aba, SUC_lexicon_tags_0 },
    { 0x17e58abb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x89e58abd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e58ac9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc258acb, SUC_lexicon_tags_37 },
    { 0xbaa58acc, SUC_lexicon_tags_25 },
    { 0xbc658acc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xba658ad0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81858ad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15658ad8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf258adb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea058aec, SUC_lexicon_tags_12 },
    { 0xdae58aed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c58af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad058af2, SUC_lexicon_tags_11 },
    { 0x28a58af3, SUC_lexicon_tags_8 },
    { 0x4c058af3, SUC_lexicon_tags_66 },
    { 0xaae58af2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e58af8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x60858afb, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a58aff, SUC_lexicon_tags_47 },
    { 0xc5458b00, SUC_lexicon_tags_16 },
    { 0x3d658b01, SUC_lexicon_tags_30 },
    { 0x11c58b02, SUC_lexicon_tags_25 },
    { 0x4ec58b03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xebe58b05, SUC_lexicon_tags_17 },
    { 0x7258b06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61658b0e, SUC_lexicon_tags_2 },
    { 0xc9258b0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70e58b11, SUC_lexicon_tags_70 },
    { 0x60258b11, SUC_lexicon_tags_21 },
    { 0xc3658b12, SUC_lexicon_tags_11 },
    { 0xc258b13, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfbc58b16, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x35058b18, SUC_lexicon_tags_37 },
    { 0x88e58b18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d258b1e, SUC_lexicon_tags_43 },
    { 0xe4c58b1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf058b28, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3458b2f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd4a58b31, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96858b35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf658b3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce58b40, SUC_lexicon_tags_15 },
    { 0xd9258b41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7c58b43, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f058b49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c858b4b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd5a58b4d, SUC_lexicon_tags_29 },
    { 0xdde58b4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1ca58b50, SUC_lexicon_tags_31 },
    { 0xfc58b51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec658b5a, SUC_lexicon_tags_17 },
    { 0x3f258b5b, SUC_lexicon_tags_15 },
    { 0x4258b5a, SUC_lexicon_tags_48 },
    { 0x47258b5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6458b64, SUC_lexicon_tags_25 },
    { 0xb6658b65, SUC_lexicon_tags_25 },
    { 0x4a458b65, SUC_lexicon_tags_19 },
    { 0xca58b66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c58b6a, SUC_lexicon_tags_11 },
    { 0x1a58b6a, SUC_lexicon_tags_23 },
    { 0x15e58b6c, SUC_lexicon_tags_6 },
    { 0xae658b6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12458b74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb258b78, SUC_lexicon_tags_65 },
    { 0x46c58b79, SUC_lexicon_tags_37 },
    { 0x25658b7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c58b81, SUC_lexicon_tags_25 },
    { 0x5f458b82, SUC_lexicon_tags_27 },
    { 0x6b658b83, SUC_lexicon_tags_15 },
    { 0x34a58b84, SUC_lexicon_tags_0 },
    { 0xa0258b84, SUC_lexicon_tags_68 },
    { 0xec58b81, SUC_lexicon_tags_21 },
    { 0x2d058b87, SUC_lexicon_tags_12 },
    { 0xea658b87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcd458b8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71a58b8c, SUC_lexicon_tags_17 },
    { 0x8da58b8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28858b90, SUC_lexicon_tags_47 },
    { 0x62e58b91, SUC_lexicon_tags_8 },
    { 0x69258b92, SUC_lexicon_tags_17 },
    { 0x14e58b93, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa9058b95, SUC_lexicon_tags_32 },
    { 0x4ba58b96, SUC_lexicon_tags_43 },
    { 0xf9e58b97, SUC_lexicon_tags_43 },
    { 0xae458b98, SUC_lexicon_tags_64 },
    { 0x27a58b97, SUC_lexicon_tags_17 },
    { 0xaa658b9a, SUC_lexicon_tags_18 },
    { 0x7b858b9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6258ba1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84858ba3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0458ba8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44258bac, SUC_lexicon_tags_34 },
    { 0xc2658bad, SUC_lexicon_tags_15 },
    { 0x25e58bae, SUC_lexicon_tags_47 },
    { 0xb1c58bad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6258bb1, SUC_lexicon_tags_60 },
    { 0xc4858bb1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e458bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a58bbc, SUC_lexicon_tags_39 },
    { 0xb6e58bbd, SUC_lexicon_tags_25 },
    { 0x80e58bbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd5058bc0, SUC_lexicon_tags_28 },
    { 0xefa58bc1, SUC_lexicon_tags_7 },
    { 0xf858bc2, SUC_lexicon_tags_4 },
    { 0x2bc58bc1, SUC_lexicon_tags_26 },
    { 0x9c58bc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7258bca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32658bd1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2be58bd3, SUC_lexicon_tags_17 },
    { 0xf4a58bd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17e58bd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6458bd9, SUC_lexicon_tags_1 },
    { 0x5f458bda, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35258be1, SUC_lexicon_tags_25 },
    { 0x26258be1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e58be7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3458bea, SUC_lexicon_tags_7 },
    { 0x9d458beb, SUC_lexicon_tags_65 },
    { 0xce058bec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc58bf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7258bf3, SUC_lexicon_tags_92 },
    { 0x95c58bf4, SUC_lexicon_tags_23 },
    { 0x5f658bf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8658bfa, SUC_lexicon_tags_17 },
    { 0xd1c58bfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c58c00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6258c03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a58c08, SUC_lexicon_tags_15 },
    { 0xf5658c09, SUC_lexicon_tags_45 },
    { 0x2a058c09, SUC_lexicon_tags_16 },
    { 0xb9e58c0b, SUC_lexicon_tags_47 },
    { 0x30458c08, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e58c10, SUC_lexicon_tags_65 },
    { 0x17658c10, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c58c14, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9258c16, SUC_lexicon_tags_7 },
    { 0xd2858c17, SUC_lexicon_tags_27 },
    { 0x73e58c16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15258c1c, SUC_lexicon_tags_10 },
    { 0xb0658c1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b458c20, SUC_lexicon_tags_12 },
    { 0x48e58c21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a58c25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb058c27, SUC_lexicon_tags_29 },
    { 0xd2e58c28, SUC_lexicon_tags_39 },
    { 0x67458c27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x88a58c2b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc058c2d, SUC_lexicon_tags_2 },
    { 0x30a58c2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c58c35, SUC_lexicon_tags_1 },
    { 0x59658c35, SUC_lexicon_tags_1 },
    { 0x19458c37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7e58c39, SUC_lexicon_tags_21 },
    { 0xb6a58c3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5058c3e, SUC_lexicon_tags_25 },
    { 0xdfa58c3f, SUC_lexicon_tags_43 },
    { 0xd2458c40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7c58c42, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd058c45, SUC_lexicon_tags_33 },
    { 0xdfc58c45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96458c4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec58c50, SUC_lexicon_tags_85 },
    { 0x78458c51, SUC_lexicon_tags_92 },
    { 0x37058c51, SUC_lexicon_tags_8 },
    { 0xd3a58c53, SUC_lexicon_tags_27 },
    { 0xb6258c50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc58c57, SUC_lexicon_tags_11 },
    { 0xdd858c58, SUC_lexicon_tags_24 },
    { 0x3e658c57, SUC_lexicon_tags_25 },
    { 0x7258c57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac258c5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a58c61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9058c63, SUC_lexicon_tags_43 },
    { 0xd1258c64, SUC_lexicon_tags_1 },
    { 0x89258c65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64258c67, SUC_lexicon_tags_7 },
    { 0xd5258c68, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f058c6b, SUC_lexicon_tags_25 },
    { 0x86c58c6c, SUC_lexicon_tags_48 },
    { 0xfc058c6d, SUC_lexicon_tags_0 },
    { 0xe5658c6e, SUC_lexicon_tags_37 },
    { 0x45a58c6f, SUC_lexicon_tags_38 },
    { 0x74458c6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd458c76, SUC_lexicon_tags_9 },
    { 0xcfc58c76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x94258c79, SUC_lexicon_tags_12 },
    { 0xe3258c7a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe058c7d, SUC_lexicon_tags_28 },
    { 0x81e58c7e, SUC_lexicon_tags_14 },
    { 0xc7e58c7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4058c81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58c84, SUC_lexicon_tags_14 },
    { 0xcf458c85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9458c89, SUC_lexicon_tags_12 },
    { 0xe7e58c89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57658c8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa58c90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe58c94, SUC_lexicon_tags_49 },
    { 0x81e58c95, SUC_lexicon_tags_16 },
    { 0x31458c94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0058c98, SUC_lexicon_tags_0 },
    { 0xe7658c99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e058c9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5058ca0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de58ca8, SUC_lexicon_tags_43 },
    { 0x23658ca8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c58cb1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2858cb7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x74258cba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x31658cbd, SUC_lexicon_tags_46 },
    { 0x61458cbe, SUC_lexicon_tags_1 },
    { 0x69058cbf, SUC_lexicon_tags_12 },
    { 0xc658cbd, SUC_lexicon_tags_33 },
    { 0xb4458cbf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93658cc7, SUC_lexicon_tags_3 },
    { 0xdf258cc8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72258ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42858cd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c058cd8, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7058cdb, SUC_lexicon_tags_15 },
    { 0x5f258cdc, SUC_lexicon_tags_65 },
    { 0xd3c58cdd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a58ce1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1658ce4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c58ceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2258cef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9258cf8, SUC_lexicon_tags_33 },
    { 0x4ce58cf9, SUC_lexicon_tags_31 },
    { 0x83c58cf9, SUC_lexicon_tags_46 },
    { 0x87258cfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19058cfe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x23658d00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c58d03, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46658d0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd1258d0d, SUC_lexicon_tags_12 },
    { 0xdbc58d0e, SUC_lexicon_tags_30 },
    { 0xc1058d0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x11c58d11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae58d16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da58d1d, SUC_lexicon_tags_12 },
    { 0x26a58d1e, SUC_lexicon_tags_12 },
    { 0xb6858d1d, SUC_lexicon_tags_2 },
    { 0x7e058d1e, SUC_lexicon_tags_15 },
    { 0xcc858d20, SUC_lexicon_tags_25 },
    { 0x11c58d22, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4258d2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee858d2e, SUC_lexicon_tags_87 },
    { 0xfcc58d2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63658d3a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36058d42, SUC_lexicon_tags_43 },
    { 0xcdc58d43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba258d46, SUC_lexicon_tags_8 },
    { 0x33058d47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52258d49, SUC_lexicon_tags_167 },
    { 0x6fe58d4a, SUC_lexicon_tags_2 },
    { 0x90658d49, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1058d52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c58d55, SUC_lexicon_tags_15 },
    { 0xda858d56, SUC_lexicon_tags_17 },
    { 0x2d458d55, SUC_lexicon_tags_1 },
    { 0xcb458d58, SUC_lexicon_tags_7 },
    { 0x1c458d59, SUC_lexicon_tags_25 },
    { 0x10458d5a, SUC_lexicon_tags_6 },
    { 0x18058d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9258d62, SUC_lexicon_tags_25 },
    { 0x78a58d63, SUC_lexicon_tags_1 },
    { 0xfaa58d62, SUC_lexicon_tags_15 },
    { 0x76658d62, SUC_lexicon_tags_11 },
    { 0xf9058d66, SUC_lexicon_tags_2 },
    { 0xe258d67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e858d6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79658d6f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56058d75, SUC_lexicon_tags_17 },
    { 0x1458d76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a58d7a, SUC_lexicon_tags_87 },
    { 0x28058d7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9e58d7d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3f658d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa58d82, SUC_lexicon_tags_7 },
    { 0x40258d82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23458d86, SUC_lexicon_tags_5 },
    { 0xd6a58d86, SUC_lexicon_tags_84 },
    { 0x83a58d88, SUC_lexicon_tags_30 },
    { 0x3ce58d88, SUC_lexicon_tags_36 },
    { 0x2ec58d8a, SUC_lexicon_tags_29 },
    { 0x35058d8a, SUC_lexicon_tags_43 },
    { 0x90258d8c, SUC_lexicon_tags_33 },
    { 0xc3e58d8b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8ae58d8f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea58d92, SUC_lexicon_tags_127 },
    { 0x19458d93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a258d98, SUC_lexicon_tags_28 },
    { 0xaee58d99, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1858d9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29258d9f, SUC_lexicon_tags_12 },
    { 0x96858da0, SUC_lexicon_tags_37 },
    { 0xff458da0, SUC_lexicon_tags_46 },
    { 0xe0a58d9f, SUC_lexicon_tags_11 },
    { 0x99658da1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65258dab, SUC_lexicon_tags_2 },
    { 0x6de58dac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c58db0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2cc58db2, SUC_lexicon_tags_61 },
    { 0x72e58db3, SUC_lexicon_tags_17 },
    { 0x76a58db2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb258db7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5858dba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a458dbc, SUC_lexicon_tags_16 },
    { 0x94e58dbd, SUC_lexicon_tags_0 },
    { 0x2d058dbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x50258dc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44058dc4, SUC_lexicon_tags_11 },
    { 0xde658dc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82858dc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f058dcc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x48458dcf, SUC_lexicon_tags_47 },
    { 0x6c58dcf, SUC_lexicon_tags_8 },
    { 0xd9c58dd0, SUC_lexicon_tags_55 },
    { 0x5c058dd2, SUC_lexicon_tags_25 },
    { 0x8f658dd3, SUC_lexicon_tags_15 },
    { 0xfcc58dd4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a58dd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4658ddb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4858ddd, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea58de0, SUC_lexicon_tags_914 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c58de3, SUC_lexicon_tags_7 },
    { 0xae658de4, SUC_lexicon_tags_17 },
    { 0xe0058de5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b458de8, SUC_lexicon_tags_188 },
    { 0xc5058de8, SUC_lexicon_tags_25 },
    { 0x62458dea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xca458ded, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x42658def, SUC_lexicon_tags_15 },
    { 0xe3c58df0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21058df3, SUC_lexicon_tags_37 },
    { 0xb6a58df3, SUC_lexicon_tags_47 },
    { 0x1be58df5, SUC_lexicon_tags_15 },
    { 0xc3058df5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5458dfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbea58dfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da58e02, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde458e0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e858e13, SUC_lexicon_tags_10 },
    { 0xa1c58e13, SUC_lexicon_tags_17 },
    { 0x86858e15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee58e18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6458e1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e058e20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5858e23, SUC_lexicon_tags_8 },
    { 0xf6c58e24, SUC_lexicon_tags_33 },
    { 0xb7c58e25, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd058e28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c58e33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec58e35, SUC_lexicon_tags_25 },
    { 0xc4658e35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a58e3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92258e3c, SUC_lexicon_tags_43 },
    { 0x1fe58e3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24858e3f, SUC_lexicon_tags_25 },
    { 0x19658e40, SUC_lexicon_tags_12 },
    { 0x25a58e3f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde458e45, SUC_lexicon_tags_37 },
    { 0x97c58e46, SUC_lexicon_tags_49 },
    { 0xe5458e46, SUC_lexicon_tags_49 },
    { 0xf4a58e46, SUC_lexicon_tags_30 },
    { 0x75658e49, SUC_lexicon_tags_49 },
    { 0x9de58e49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a258e51, SUC_lexicon_tags_45 },
    { 0x62458e52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d658e56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91258e59, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef658e5f, SUC_lexicon_tags_25 },
    { 0x30e58e60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a58e63, SUC_lexicon_tags_65 },
    { 0x26658e63, SUC_lexicon_tags_37 },
    { 0x55058e65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb858e6b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd8c58e6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xda658e6f, SUC_lexicon_tags_37 },
    { 0x55258e70, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbf858e72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc58e76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5c058e78, SUC_lexicon_tags_11 },
    { 0x26658e79, SUC_lexicon_tags_26 },
    { 0xbcc58e7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea58e7d, SUC_lexicon_tags_23 },
    { 0xc8e58e7e, SUC_lexicon_tags_47 },
    { 0x17658e7e, SUC_lexicon_tags_23 },
    { 0x79e58e80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8e58e82, SUC_lexicon_tags_26 },
    { 0x99458e82, SUC_lexicon_tags_15 },
    { 0x97258e82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18258e87, SUC_lexicon_tags_3 },
    { 0xb9c58e88, SUC_lexicon_tags_77 },
    { 0xfd258e89, SUC_lexicon_tags_11 },
    { 0xfc658e89, SUC_lexicon_tags_33 },
    { 0x1a58e87, SUC_lexicon_tags_2 },
    { 0xacc58e87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x60c58e8e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38658e92, SUC_lexicon_tags_17 },
    { 0xae58e93, SUC_lexicon_tags_12 },
    { 0x18258e92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc58e9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a58ead, SUC_lexicon_tags_65 },
    { 0xe0858eae, SUC_lexicon_tags_27 },
    { 0xdfe58eae, SUC_lexicon_tags_63 },
    { 0x97058eb0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7058eb4, SUC_lexicon_tags_2 },
    { 0xeb658eb5, SUC_lexicon_tags_55 },
    { 0xad858eb6, SUC_lexicon_tags_31 },
    { 0x4058eb4, SUC_lexicon_tags_58 },
    { 0x2658eb8, SUC_lexicon_tags_32 },
    { 0x2b858eb4, SUC_lexicon_tags_47 },
    { 0xddc58eb8, SUC_lexicon_tags_15 },
    { 0x67e58eb9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xfbc58ebd, SUC_lexicon_tags_17 },
    { 0xa5658ebe, SUC_lexicon_tags_2 },
    { 0x9658ebf, SUC_lexicon_tags_15 },
    { 0xdce58ebf, SUC_lexicon_tags_27 },
    { 0xe658ebe, SUC_lexicon_tags_11 },
    { 0x4d458ebe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c58ec7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a58ecd, SUC_lexicon_tags_33 },
    { 0x17258ecd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f458ed0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5458ed5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa458ed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca858ee2, SUC_lexicon_tags_25 },
    { 0xd4458ee2, SUC_lexicon_tags_6 },
    { 0xa4658ee2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf858ee6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c58eeb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e58ef1, SUC_lexicon_tags_37 },
    { 0x42858ef1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85658ef4, SUC_lexicon_tags_1 },
    { 0x7c58ef5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858efb, SUC_lexicon_tags_8 },
    { 0xf1a58efc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xada58efe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab858f02, SUC_lexicon_tags_39 },
    { 0x94e58f03, SUC_lexicon_tags_47 },
    { 0xeee58f04, SUC_lexicon_tags_15 },
    { 0x58458f04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc58f08, SUC_lexicon_tags_17 },
    { 0x8e58f08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa58f0c, SUC_lexicon_tags_1 },
    { 0x7c058f0d, SUC_lexicon_tags_49 },
    { 0x99858f0d, SUC_lexicon_tags_39 },
    { 0x3da58f0d, SUC_lexicon_tags_7 },
    { 0x67458f10, SUC_lexicon_tags_0 },
    { 0x30e58f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16658f14, SUC_lexicon_tags_11 },
    { 0xc9658f15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36258f1a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3ca58f1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a858f1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76458f21, SUC_lexicon_tags_17 },
    { 0x9e658f22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac658f26, SUC_lexicon_tags_25 },
    { 0x4f858f27, SUC_lexicon_tags_0 },
    { 0x8c258f28, SUC_lexicon_tags_57 },
    { 0x9ae58f28, SUC_lexicon_tags_30 },
    { 0xe6a58f29, SUC_lexicon_tags_87 },
    { 0xa3058f2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x20258f2d, SUC_lexicon_tags_17 },
    { 0x5fe58f2e, SUC_lexicon_tags_65 },
    { 0x69458f2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c58f32, SUC_lexicon_tags_82 },
    { 0x45c58f32, SUC_lexicon_tags_45 },
    { 0x91a58f34, SUC_lexicon_tags_60 },
    { 0x9f458f34, SUC_lexicon_tags_9 },
    { 0xda458f34, SUC_lexicon_tags_151 },
    { 0x31e58f36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae258f39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0858f3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc658f43, SUC_lexicon_tags_14 },
    { 0xe858f44, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdf258f46, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa0058f48, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa7858f4a, SUC_lexicon_tags_63 },
    { 0x25658f4b, SUC_lexicon_tags_25 },
    { 0x8b058f4a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7f858f4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f258f54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7858f56, SUC_lexicon_tags_43 },
    { 0xae658f57, SUC_lexicon_tags_25 },
    { 0x14058f58, SUC_lexicon_tags_17 },
    { 0x4a58f59, SUC_lexicon_tags_12 },
    { 0xa7a58f59, SUC_lexicon_tags_2 },
    { 0x9f258f5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x25458f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea258f60, SUC_lexicon_tags_25 },
    { 0xf2058f61, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa58f65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2658f67, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d258f6b, SUC_lexicon_tags_37 },
    { 0xf9858f6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e58f6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e58f73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f258f75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e858f79, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc858f7c, SUC_lexicon_tags_1 },
    { 0x4aa58f7d, SUC_lexicon_tags_0 },
    { 0xe5858f7d, SUC_lexicon_tags_25 },
    { 0xf5058f7e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x76658f82, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x45e58f84, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b258f89, SUC_lexicon_tags_17 },
    { 0x52258f8a, SUC_lexicon_tags_4 },
    { 0x65458f8b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x97858f8d, SUC_lexicon_tags_42 },
    { 0x60c58f8e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0258f93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac058f97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ec58f99, SUC_lexicon_tags_2 },
    { 0xf3458f9a, SUC_lexicon_tags_27 },
    { 0xda458f9b, SUC_lexicon_tags_7 },
    { 0x79258f99, SUC_lexicon_tags_15 },
    { 0xe7858f9c, SUC_lexicon_tags_36 },
    { 0x17c58f9d, SUC_lexicon_tags_37 },
    { 0x41258f9e, SUC_lexicon_tags_37 },
    { 0xe2258fa0, SUC_lexicon_tags_30 },
    { 0x61a58fa1, SUC_lexicon_tags_55 },
    { 0xf8858fa2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7258fa5, SUC_lexicon_tags_47 },
    { 0x6ac58fa6, SUC_lexicon_tags_160 },
    { 0xf9a58fa7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae58fb0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34858fb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0058fb6, SUC_lexicon_tags_25 },
    { 0x83458fb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6858fba, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1458fbf, SUC_lexicon_tags_30 },
    { 0x15e58fc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7058fc6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e058fd2, SUC_lexicon_tags_19 },
    { 0x43258fd2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b658fd7, SUC_lexicon_tags_46 },
    { 0x53c58fd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc58fdb, SUC_lexicon_tags_193 },
    { 0xb7858fdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcc58fde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27858fe3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb7458fe5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9dc58fe7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2658fea, SUC_lexicon_tags_14 },
    { 0x59458fea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac58ff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92458ff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5458ffb, SUC_lexicon_tags_19 },
    { 0x7b658ffc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xec458ffe, SUC_lexicon_tags_2 },
    { 0xd4458fff, SUC_lexicon_tags_63 },
    { 0x34a58fff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30459006, SUC_lexicon_tags_37 },
    { 0x27859007, SUC_lexicon_tags_33 },
    { 0x20059008, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a5900b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b85900e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x55259011, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee59014, SUC_lexicon_tags_11 },
    { 0x34859015, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5385901a, SUC_lexicon_tags_25 },
    { 0xdac5901a, SUC_lexicon_tags_43 },
    { 0x9c85901c, SUC_lexicon_tags_17 },
    { 0x29c5901d, SUC_lexicon_tags_37 },
    { 0x8e25901d, SUC_lexicon_tags_7 },
    { 0x9b85901c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36059021, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc59029, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9505902d, SUC_lexicon_tags_7 },
    { 0x9005902d, SUC_lexicon_tags_11 },
    { 0x8c85902f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a59034, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be5903e, SUC_lexicon_tags_542 },
    { 0x465903e, SUC_lexicon_tags_38 },
    { 0x57059040, SUC_lexicon_tags_478 },
    { 0x94059040, SUC_lexicon_tags_11 },
    { 0x3165903e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0c59044, SUC_lexicon_tags_235 },
    { 0xac059045, SUC_lexicon_tags_7 },
    { 0x1c59046, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79659048, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0e5904a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf2e5904c, SUC_lexicon_tags_1 },
    { 0x4a5904d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a259050, SUC_lexicon_tags_15 },
    { 0x7a659050, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff459058, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2785905e, SUC_lexicon_tags_47 },
    { 0x73a5905f, SUC_lexicon_tags_9 },
    { 0xbb059060, SUC_lexicon_tags_0 },
    { 0x5b85905f, SUC_lexicon_tags_15 },
    { 0x1be59062, SUC_lexicon_tags_17 },
    { 0xc5c59063, SUC_lexicon_tags_15 },
    { 0x30a5905e, SUC_lexicon_tags_19 },
    { 0x4fe59065, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd059067, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd7659069, SUC_lexicon_tags_7 },
    { 0x22a5906a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3685906c, SUC_lexicon_tags_10 },
    { 0x8f65906d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59074, SUC_lexicon_tags_31 },
    { 0x48259075, SUC_lexicon_tags_11 },
    { 0x3a459076, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb859078, SUC_lexicon_tags_65 },
    { 0x7bc59079, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3ba5907b, SUC_lexicon_tags_15 },
    { 0x3885907c, SUC_lexicon_tags_25 },
    { 0xea05907d, SUC_lexicon_tags_85 },
    { 0x7905907e, SUC_lexicon_tags_45 },
    { 0x1805907b, SUC_lexicon_tags_14 },
    { 0xa7a59080, SUC_lexicon_tags_38 },
    { 0xf6059080, SUC_lexicon_tags_17 },
    { 0x50659082, SUC_lexicon_tags_8 },
    { 0xf9a59081, SUC_lexicon_tags_49 },
    { 0x9ec59082, SUC_lexicon_tags_57 },
    { 0xc965907d, SUC_lexicon_tags_15 },
    { 0x99659086, SUC_lexicon_tags_5 },
    { 0xda859087, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd7c59089, SUC_lexicon_tags_11 },
    { 0xa05908a, SUC_lexicon_tags_8 },
    { 0xe9e5908b, SUC_lexicon_tags_17 },
    { 0xc2e5908c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad259093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4659095, SUC_lexicon_tags_12 },
    { 0xb0459096, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a45909f, SUC_lexicon_tags_1 },
    { 0x820590a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8590a3, SUC_lexicon_tags_1 },
    { 0xf6e590a4, SUC_lexicon_tags_12 },
    { 0xde0590a5, SUC_lexicon_tags_12 },
    { 0x574590a6, SUC_lexicon_tags_1 },
    { 0xe0590a3, SUC_lexicon_tags_33 },
    { 0xa54590a8, SUC_lexicon_tags_19 },
    { 0xb38590a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcd4590ab, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x59a590ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe36590af, SUC_lexicon_tags_47 },
    { 0xb82590b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb2590b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92e590b4, SUC_lexicon_tags_27 },
    { 0x51c590b5, SUC_lexicon_tags_63 },
    { 0x370590b6, SUC_lexicon_tags_2 },
    { 0xaa590b7, SUC_lexicon_tags_7 },
    { 0x922590b5, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0xb9a590ba, SUC_lexicon_tags_60 },
    { 0x43c590ba, SUC_lexicon_tags_3 },
    { 0xe6e590bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x324590be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956590c2, SUC_lexicon_tags_25 },
    { 0x8b8590c3, SUC_lexicon_tags_47 },
    { 0x3bc590c2, SUC_lexicon_tags_91 },
    { 0x25e590c4, SUC_lexicon_tags_17 },
    { 0xd04590c6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x92e590c8, SUC_lexicon_tags_16 },
    { 0x332590c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8e590cb, SUC_lexicon_tags_25 },
    { 0x98e590cc, SUC_lexicon_tags_30 },
    { 0x46e590cc, SUC_lexicon_tags_11 },
    { 0xc8a590ce, SUC_lexicon_tags_3 },
    { 0x20e590ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x540590d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12590d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca590d9, SUC_lexicon_tags_17 },
    { 0xf24590da, SUC_lexicon_tags_142 },
    { 0x412590db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x158590de, SUC_lexicon_tags_47 },
    { 0xe82590df, SUC_lexicon_tags_1 },
    { 0xdb0590de, SUC_lexicon_tags_66 },
    { 0xc24590e1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x92e590e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50590e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2590eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x274590ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e6590f0, SUC_lexicon_tags_19 },
    { 0x182590f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccc590f3, SUC_lexicon_tags_37 },
    { 0x152590f3, SUC_lexicon_tags_29 },
    { 0xde0590f3, SUC_lexicon_tags_14 },
    { 0x60e590f5, SUC_lexicon_tags_25 },
    { 0x37c590f5, SUC_lexicon_tags_17 },
    { 0x9d0590f8, SUC_lexicon_tags_11 },
    { 0x442590f9, SUC_lexicon_tags_13 },
    { 0xaac590f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398590fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe59101, SUC_lexicon_tags_28 },
    { 0xdf659102, SUC_lexicon_tags_17 },
    { 0x3b059103, SUC_lexicon_tags_27 },
    { 0xaa659103, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e5910b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e59112, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b259115, SUC_lexicon_tags_49 },
    { 0xf5459115, SUC_lexicon_tags_1 },
    { 0xe8e59117, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e85911d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa259120, SUC_lexicon_tags_9 },
    { 0x95059120, SUC_lexicon_tags_12 },
    { 0x54e59122, SUC_lexicon_tags_7 },
    { 0x2b259122, SUC_lexicon_tags_1 },
    { 0x7dc59121, SUC_lexicon_tags_57 },
    { 0xb7e59125, SUC_lexicon_tags_15 },
    { 0x92659126, SUC_lexicon_tags_63 },
    { 0x18659123, SUC_lexicon_tags_68 },
    { 0x1a859124, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe985912a, SUC_lexicon_tags_3 },
    { 0x54c5912b, SUC_lexicon_tags_37 },
    { 0x8c45912b, SUC_lexicon_tags_55 },
    { 0xe8c5912d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92259131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fa59133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68859135, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77459138, SUC_lexicon_tags_4 },
    { 0x2a859138, SUC_lexicon_tags_63 },
    { 0x89259139, SUC_lexicon_tags_25 },
    { 0x2e659139, SUC_lexicon_tags_37 },
    { 0xf6e5913c, SUC_lexicon_tags_7 },
    { 0xb885913d, SUC_lexicon_tags_82 },
    { 0x38c5913e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63459142, SUC_lexicon_tags_43 },
    { 0xc9c59142, SUC_lexicon_tags_11 },
    { 0x6c859144, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba59148, SUC_lexicon_tags_27 },
    { 0xc8c59149, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a65914f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e659152, SUC_lexicon_tags_7 },
    { 0x5d659153, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x77659155, SUC_lexicon_tags_207 },
    { 0x8259156, SUC_lexicon_tags_25 },
    { 0xf459157, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c25915b, SUC_lexicon_tags_1 },
    { 0xf1a5915c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc785915f, SUC_lexicon_tags_1 },
    { 0xe2e59160, SUC_lexicon_tags_15 },
    { 0xd1459161, SUC_lexicon_tags_25 },
    { 0x17459162, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x80659165, SUC_lexicon_tags_7 },
    { 0x1c59166, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd259168, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5845916c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad25916e, SUC_lexicon_tags_12 },
    { 0x5445916e, SUC_lexicon_tags_32 },
    { 0xc9e5916e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dc59172, SUC_lexicon_tags_43 },
    { 0xbee59173, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a65917a, SUC_lexicon_tags_47 },
    { 0x3bc5917a, SUC_lexicon_tags_70 },
    { 0xdc05917c, SUC_lexicon_tags_39 },
    { 0xaa45917c, SUC_lexicon_tags_12 },
    { 0x5b85917c, SUC_lexicon_tags_46 },
    { 0xdbc5917e, SUC_lexicon_tags_47 },
    { 0x26659180, SUC_lexicon_tags_47 },
    { 0x5fa59181, SUC_lexicon_tags_60 },
    { 0x97e5917d, SUC_lexicon_tags_7 },
    { 0xe3859182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f459185, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5918b, SUC_lexicon_tags_84 },
    { 0x2365918c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e5918f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7259192, SUC_lexicon_tags_2 },
    { 0xc9a59192, SUC_lexicon_tags_113 },
    { 0xa859192, SUC_lexicon_tags_8 },
    { 0x18a59195, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x40859198, SUC_lexicon_tags_11 },
    { 0xc0259199, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf125919c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2645919e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c591a1, SUC_lexicon_tags_27 },
    { 0x80a591a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcac591a4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a591a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8591ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52591b0, SUC_lexicon_tags_1398 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a591b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252591b8, SUC_lexicon_tags_9 },
    { 0xdb8591b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a591bf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4591c2, SUC_lexicon_tags_1 },
    { 0x130591c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768591c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7e591c9, SUC_lexicon_tags_2 },
    { 0x126591ca, SUC_lexicon_tags_19 },
    { 0x1b8591cb, SUC_lexicon_tags_45 },
    { 0xb90591cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e591d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4591d4, SUC_lexicon_tags_47 },
    { 0x6ac591d5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88591d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9e591db, SUC_lexicon_tags_23 },
    { 0x926591dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4591e0, SUC_lexicon_tags_1 },
    { 0xd0e591e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x254591e3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbf6591e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa8c591e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x832591ea, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992591f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8591f6, SUC_lexicon_tags_30 },
    { 0xef2591f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaa0591f9, SUC_lexicon_tags_17 },
    { 0x4c0591f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120591ff, SUC_lexicon_tags_10 },
    { 0xa7459200, SUC_lexicon_tags_60 },
    { 0xf84591ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80259203, SUC_lexicon_tags_23 },
    { 0x6c859204, SUC_lexicon_tags_19 },
    { 0xc8a59203, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5920c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe265920e, SUC_lexicon_tags_15 },
    { 0xef45920f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55e59211, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf4c59213, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c059215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1459217, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9b259219, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe859222, SUC_lexicon_tags_16 },
    { 0xfe059222, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c659226, SUC_lexicon_tags_45 },
    { 0xa3259226, SUC_lexicon_tags_35 },
    { 0x55a59226, SUC_lexicon_tags_33 },
    { 0xd1e59226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee45922d, SUC_lexicon_tags_1 },
    { 0x2265922e, SUC_lexicon_tags_38 },
    { 0x6c85922d, SUC_lexicon_tags_63 },
    { 0x85259230, SUC_lexicon_tags_26 },
    { 0xf025922f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22a59233, SUC_lexicon_tags_3 },
    { 0x96259234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec59238, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe5923c, SUC_lexicon_tags_19 },
    { 0x8125923d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd405923f, SUC_lexicon_tags_15 },
    { 0x7ee5923f, SUC_lexicon_tags_23 },
    { 0x31a59240, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20859243, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e59246, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82259251, SUC_lexicon_tags_17 },
    { 0x8f059252, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b259255, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6059258, SUC_lexicon_tags_57 },
    { 0xa6a59259, SUC_lexicon_tags_10 },
    { 0x9825925a, SUC_lexicon_tags_14 },
    { 0x2f05925a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x205925d, SUC_lexicon_tags_17 },
    { 0x5c85925e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95459262, SUC_lexicon_tags_12 },
    { 0x5b659263, SUC_lexicon_tags_17 },
    { 0x60659262, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8645926c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd925926e, SUC_lexicon_tags_39 },
    { 0x7a5926f, SUC_lexicon_tags_70 },
    { 0x85c5926f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0059274, SUC_lexicon_tags_47 },
    { 0xcb259275, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf459278, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8325927a, SUC_lexicon_tags_15 },
    { 0xede5927b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x83a5927d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e59280, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2259282, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e259288, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c05928a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7445928e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9659293, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4459298, SUC_lexicon_tags_66 },
    { 0x2b059299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce05929d, SUC_lexicon_tags_351 },
    { 0x93e5929e, SUC_lexicon_tags_39 },
    { 0x14a5929f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf42592a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8592ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8592ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc592b4, SUC_lexicon_tags_7 },
    { 0xf4c592b5, SUC_lexicon_tags_14 },
    { 0xfee592b6, SUC_lexicon_tags_26 },
    { 0xdc8592b7, SUC_lexicon_tags_12 },
    { 0x776592b8, SUC_lexicon_tags_12 },
    { 0xc54592b7, SUC_lexicon_tags_25 },
    { 0xb94592ba, SUC_lexicon_tags_63 },
    { 0x414592bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a592c2, SUC_lexicon_tags_1 },
    { 0xd92592c2, SUC_lexicon_tags_15 },
    { 0x904592c4, SUC_lexicon_tags_1 },
    { 0x1ea592c5, SUC_lexicon_tags_6 },
    { 0x36592c6, SUC_lexicon_tags_12 },
    { 0x82592c7, SUC_lexicon_tags_68 },
    { 0x762592c7, SUC_lexicon_tags_1 },
    { 0x526592c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c592cf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x144592d1, SUC_lexicon_tags_15 },
    { 0xc72592d1, SUC_lexicon_tags_26 },
    { 0x730592d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c592d7, SUC_lexicon_tags_15 },
    { 0xdc0592d7, SUC_lexicon_tags_12 },
    { 0x40c592d9, SUC_lexicon_tags_43 },
    { 0xc48592d8, SUC_lexicon_tags_12 },
    { 0x324592d9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xef4592dd, SUC_lexicon_tags_2 },
    { 0xc18592dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e592e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6592e9, SUC_lexicon_tags_4 },
    { 0x9e592ea, SUC_lexicon_tags_124 },
    { 0xec6592eb, SUC_lexicon_tags_11 },
    { 0x44592eb, SUC_lexicon_tags_62 },
    { 0x5cc592eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x966592f0, SUC_lexicon_tags_19 },
    { 0x5b2592f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496592fb, SUC_lexicon_tags_8 },
    { 0x404592fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a592ff, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba59304, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27c59306, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xec59309, SUC_lexicon_tags_15 },
    { 0xc665930a, SUC_lexicon_tags_65 },
    { 0xc305930b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec25930e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e59311, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c59314, SUC_lexicon_tags_11 },
    { 0x30659315, SUC_lexicon_tags_12 },
    { 0x42259315, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25659318, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbbe5931a, SUC_lexicon_tags_1 },
    { 0x7ca5931a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e5931e, SUC_lexicon_tags_28 },
    { 0x1745931e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a259322, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe3c59324, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d85932b, SUC_lexicon_tags_7 },
    { 0x9b05932b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5859330, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7659333, SUC_lexicon_tags_12 },
    { 0xa4859334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe59339, SUC_lexicon_tags_12 },
    { 0x9059339, SUC_lexicon_tags_11 },
    { 0xc225933b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2405933d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2625933f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a059342, SUC_lexicon_tags_8 },
    { 0xaf059342, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc3259345, SUC_lexicon_tags_37 },
    { 0xd8e59346, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x61059349, SUC_lexicon_tags_26 },
    { 0x6445934a, SUC_lexicon_tags_37 },
    { 0xb4e5934a, SUC_lexicon_tags_11 },
    { 0x40c5934b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6265934e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc859356, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a59359, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7305935e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c59364, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a459367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e05936a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c59370, SUC_lexicon_tags_61 },
    { 0x6f459371, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea5937b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e859381, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x43e59383, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda5938e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18c59390, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a259395, SUC_lexicon_tags_25 },
    { 0xb6659395, SUC_lexicon_tags_28 },
    { 0xdd059395, SUC_lexicon_tags_27 },
    { 0x44a59398, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e5939c, SUC_lexicon_tags_47 },
    { 0x1a5939c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef85939f, SUC_lexicon_tags_19 },
    { 0x186593a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a593a8, SUC_lexicon_tags_43 },
    { 0xfc6593a8, SUC_lexicon_tags_25 },
    { 0xcce593aa, SUC_lexicon_tags_9 },
    { 0x180593ab, SUC_lexicon_tags_17 },
    { 0x196593ac, SUC_lexicon_tags_31 },
    { 0x76c593ab, SUC_lexicon_tags_38 },
    { 0xe26593ae, SUC_lexicon_tags_12 },
    { 0x49a593ad, SUC_lexicon_tags_25 },
    { 0xbc593b0, SUC_lexicon_tags_70 },
    { 0x3a4593b1, SUC_lexicon_tags_11 },
    { 0x134593b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82593b5, SUC_lexicon_tags_37 },
    { 0xde0593b5, SUC_lexicon_tags_2 },
    { 0xd80593b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a593bf, SUC_lexicon_tags_47 },
    { 0xe14593bf, SUC_lexicon_tags_87 },
    { 0xef2593c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6593c4, SUC_lexicon_tags_14 },
    { 0x698593c5, SUC_lexicon_tags_12 },
    { 0x596593c5, SUC_lexicon_tags_31 },
    { 0xb5a593c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e593cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6593cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0593d5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2d0593d7, SUC_lexicon_tags_1 },
    { 0x4ea593d7, SUC_lexicon_tags_15 },
    { 0xcb0593d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x436593db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa16593dd, SUC_lexicon_tags_31 },
    { 0x5ae593dd, SUC_lexicon_tags_38 },
    { 0x5ea593de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce593e7, SUC_lexicon_tags_12 },
    { 0xb6c593e8, SUC_lexicon_tags_10 },
    { 0x2d6593e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x43e593eb, SUC_lexicon_tags_0 },
    { 0x3ae593ec, SUC_lexicon_tags_12 },
    { 0xd52593ed, SUC_lexicon_tags_15 },
    { 0x67c593ec, SUC_lexicon_tags_16 },
    { 0x7f4593ef, SUC_lexicon_tags_21 },
    { 0x732593f0, SUC_lexicon_tags_37 },
    { 0x2a4593ed, SUC_lexicon_tags_0 },
    { 0x4b0593f2, SUC_lexicon_tags_7 },
    { 0xd84593f3, SUC_lexicon_tags_4 },
    { 0xafa593f4, SUC_lexicon_tags_12 },
    { 0x406593f4, SUC_lexicon_tags_46 },
    { 0x984593f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c593f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3459405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87459407, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e65940b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a59411, SUC_lexicon_tags_68 },
    { 0xed459412, SUC_lexicon_tags_29 },
    { 0x62059411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e59416, SUC_lexicon_tags_8 },
    { 0x17059417, SUC_lexicon_tags_27 },
    { 0x10059418, SUC_lexicon_tags_34 },
    { 0xafa59418, SUC_lexicon_tags_6 },
    { 0x1a25941a, SUC_lexicon_tags_48 },
    { 0x55e5941b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a59422, SUC_lexicon_tags_23 },
    { 0xb5459423, SUC_lexicon_tags_7 },
    { 0xde59424, SUC_lexicon_tags_4 },
    { 0x2e659422, SUC_lexicon_tags_14 },
    { 0xe6659426, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5859428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75a5942a, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed05942f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xea459432, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6ea59434, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2059437, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a45943e, SUC_lexicon_tags_50 },
    { 0x7d25943f, SUC_lexicon_tags_1 },
    { 0x98a59440, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc259448, SUC_lexicon_tags_30 },
    { 0xe5e59448, SUC_lexicon_tags_92 },
    { 0x77e5944a, SUC_lexicon_tags_65 },
    { 0x66a5944a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5944d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e59450, SUC_lexicon_tags_1 },
    { 0x3f059451, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ec59453, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c659457, SUC_lexicon_tags_0 },
    { 0xb1e59458, SUC_lexicon_tags_28 },
    { 0x2e659459, SUC_lexicon_tags_25 },
    { 0xeb65945a, SUC_lexicon_tags_15 },
    { 0x9c59458, SUC_lexicon_tags_17 },
    { 0x8ee5945b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba5945f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79859467, SUC_lexicon_tags_1 },
    { 0xabc59468, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb045946b, SUC_lexicon_tags_49 },
    { 0x7165946b, SUC_lexicon_tags_19 },
    { 0x2025946d, SUC_lexicon_tags_15 },
    { 0x6485946e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd5e59470, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c59473, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a259479, SUC_lexicon_tags_23 },
    { 0xd105947a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e25947d, SUC_lexicon_tags_11 },
    { 0x67e5947d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4259482, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x33059484, SUC_lexicon_tags_9 },
    { 0x4c59485, SUC_lexicon_tags_55 },
    { 0xe7059486, SUC_lexicon_tags_7 },
    { 0x59859485, SUC_lexicon_tags_25 },
    { 0x1c659484, SUC_lexicon_tags_8 },
    { 0x3ac59489, SUC_lexicon_tags_17 },
    { 0x39e59489, SUC_lexicon_tags_12 },
    { 0x3f65948b, SUC_lexicon_tags_38 },
    { 0xfd059484, SUC_lexicon_tags_46 },
    { 0xac85948d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2e5948f, SUC_lexicon_tags_65 },
    { 0xb5a59490, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac859494, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x32859497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ca59499, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f65949b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d25949d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6594a0, SUC_lexicon_tags_17 },
    { 0xbc4594a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10594a8, SUC_lexicon_tags_38 },
    { 0xd72594a8, SUC_lexicon_tags_12 },
    { 0x2e6594aa, SUC_lexicon_tags_9 },
    { 0x52e594ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42594ae, SUC_lexicon_tags_47 },
    { 0xb2e594ae, SUC_lexicon_tags_31 },
    { 0xe92594af, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2ac594b2, SUC_lexicon_tags_106 },
    { 0x6ca594b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x598594b5, SUC_lexicon_tags_26 },
    { 0xae0594b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc594ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6594be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0594c0, SUC_lexicon_tags_27 },
    { 0x74c594c0, SUC_lexicon_tags_11 },
    { 0x4d0594c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a594cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4594d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c594d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a594db, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe58594dd, SUC_lexicon_tags_11 },
    { 0xdd4594de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e594e4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da594e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c594f0, SUC_lexicon_tags_12 },
    { 0x6bc594f1, SUC_lexicon_tags_29 },
    { 0x118594f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98594fd, SUC_lexicon_tags_6 },
    { 0xefc594fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3459502, SUC_lexicon_tags_31 },
    { 0xdf659502, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad459505, SUC_lexicon_tags_37 },
    { 0x8bc59505, SUC_lexicon_tags_25 },
    { 0x1a659505, SUC_lexicon_tags_31 },
    { 0x75059507, SUC_lexicon_tags_0 },
    { 0xd3c59509, SUC_lexicon_tags_30 },
    { 0x3445950a, SUC_lexicon_tags_49 },
    { 0x21659509, SUC_lexicon_tags_13 },
    { 0x1a5950c, SUC_lexicon_tags_23 },
    { 0x6dc5950d, SUC_lexicon_tags_55 },
    { 0xca859508, SUC_lexicon_tags_5 },
    { 0x12059509, SUC_lexicon_tags_3 },
    { 0x98e59510, SUC_lexicon_tags_31 },
    { 0x50659505, SUC_lexicon_tags_3 },
    { 0x45859512, SUC_lexicon_tags_15 },
    { 0xfae59513, SUC_lexicon_tags_68 },
    { 0x3f259513, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b459518, SUC_lexicon_tags_33 },
    { 0xb5859519, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae45951d, SUC_lexicon_tags_1 },
    { 0x5e45951d, SUC_lexicon_tags_82 },
    { 0x7665951f, SUC_lexicon_tags_25 },
    { 0xefe59520, SUC_lexicon_tags_37 },
    { 0xf0259521, SUC_lexicon_tags_0 },
    { 0xa4659521, SUC_lexicon_tags_25 },
    { 0x4d85951e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe59526, SUC_lexicon_tags_12 },
    { 0xf7659527, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd659529, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a5952c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96c5952e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f659530, SUC_lexicon_tags_33 },
    { 0xec59531, SUC_lexicon_tags_9 },
    { 0xdf459532, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf8459534, SUC_lexicon_tags_11 },
    { 0xf7259535, SUC_lexicon_tags_10 },
    { 0x17859536, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94c59538, SUC_lexicon_tags_8 },
    { 0x9259539, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6e05953b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xac5953d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23859541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c59544, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9859546, SUC_lexicon_tags_66 },
    { 0xf1459547, SUC_lexicon_tags_65 },
    { 0x5a459548, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce5954b, SUC_lexicon_tags_2 },
    { 0xc0e5954c, SUC_lexicon_tags_23 },
    { 0x5c25954d, SUC_lexicon_tags_19 },
    { 0x7705954c, SUC_lexicon_tags_37 },
    { 0x9f25954c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0659555, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72659559, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfa65955b, SUC_lexicon_tags_3 },
    { 0x36e5955c, SUC_lexicon_tags_15 },
    { 0xb065955d, SUC_lexicon_tags_57 },
    { 0x2da5955d, SUC_lexicon_tags_4 },
    { 0x1da5955c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea859562, SUC_lexicon_tags_37 },
    { 0x11a59563, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3e59565, SUC_lexicon_tags_17 },
    { 0xd8a59565, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf745956b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5f25956d, SUC_lexicon_tags_1 },
    { 0x7c25956e, SUC_lexicon_tags_151 },
    { 0xab65956f, SUC_lexicon_tags_16 },
    { 0x3b65956f, SUC_lexicon_tags_17 },
    { 0x82e59571, SUC_lexicon_tags_14 },
    { 0x2c659570, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x56259575, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe5659577, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac5957e, SUC_lexicon_tags_15 },
    { 0xc7a5957f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b259585, SUC_lexicon_tags_2 },
    { 0x3b659586, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c059588, SUC_lexicon_tags_13 },
    { 0x1de59589, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5125958b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58059591, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c59595, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69459598, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x325959a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf965959e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636595a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6595a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4595a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a6595ab, SUC_lexicon_tags_39 },
    { 0x73e595ac, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0595af, SUC_lexicon_tags_39 },
    { 0xbc8595af, SUC_lexicon_tags_28 },
    { 0x5ee595b1, SUC_lexicon_tags_17 },
    { 0x35c595b2, SUC_lexicon_tags_37 },
    { 0x5fe595b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4595b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x996595ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x60595bd, SUC_lexicon_tags_40 },
    { 0x4da595bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x37c595c0, SUC_lexicon_tags_2 },
    { 0x854595c0, SUC_lexicon_tags_17 },
    { 0x2d0595c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5da595c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa595cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e2595d1, SUC_lexicon_tags_33 },
    { 0x74a595d2, SUC_lexicon_tags_17 },
    { 0xe86595d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0595d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x208595d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e595dc, SUC_lexicon_tags_9 },
    { 0xde0595dd, SUC_lexicon_tags_17 },
    { 0x76e595dd, SUC_lexicon_tags_46 },
    { 0x91e595df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c595e8, SUC_lexicon_tags_8 },
    { 0xb5c595e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x468595eb, SUC_lexicon_tags_25 },
    { 0xc6c595ec, SUC_lexicon_tags_26 },
    { 0x1ae595ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32a595ef, SUC_lexicon_tags_27 },
    { 0xd0c595f0, SUC_lexicon_tags_26 },
    { 0x65c595f0, SUC_lexicon_tags_15 },
    { 0x512595f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6595fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb44595fc, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0xd2c595fe, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a59601, SUC_lexicon_tags_37 },
    { 0x93a59602, SUC_lexicon_tags_15 },
    { 0x49659603, SUC_lexicon_tags_4 },
    { 0x52859601, SUC_lexicon_tags_43 },
    { 0x3aa59605, SUC_lexicon_tags_15 },
    { 0xf1c59606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c59609, SUC_lexicon_tags_29 },
    { 0x1565960a, SUC_lexicon_tags_15 },
    { 0xc65960b, SUC_lexicon_tags_37 },
    { 0x185960c, SUC_lexicon_tags_6 },
    { 0x59c5960c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa285960f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5859614, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7c59616, SUC_lexicon_tags_23 },
    { 0x95c59617, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27e59619, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c5961c, SUC_lexicon_tags_11 },
    { 0x8625961c, SUC_lexicon_tags_1 },
    { 0xcba5961d, SUC_lexicon_tags_45 },
    { 0xc785961d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3259625, SUC_lexicon_tags_14 },
    { 0xf3459625, SUC_lexicon_tags_8 },
    { 0xaaa59627, SUC_lexicon_tags_17 },
    { 0x18a59627, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1005962a, SUC_lexicon_tags_46 },
    { 0x6225962a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfcc5962d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfbe5962f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34659633, SUC_lexicon_tags_2 },
    { 0x6c059634, SUC_lexicon_tags_63 },
    { 0xe4459633, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f059638, SUC_lexicon_tags_33 },
    { 0xb6859638, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeba5963b, SUC_lexicon_tags_37 },
    { 0x3225963b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2365963e, SUC_lexicon_tags_45 },
    { 0x665963f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcc59641, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8605964a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xee45964d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b459653, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d659659, SUC_lexicon_tags_6 },
    { 0x2059659, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da5965e, SUC_lexicon_tags_29 },
    { 0x5cc5965e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf459662, SUC_lexicon_tags_27 },
    { 0xbe059663, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c59666, SUC_lexicon_tags_46 },
    { 0x1b859667, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b85966a, SUC_lexicon_tags_48 },
    { 0xa4a5966a, SUC_lexicon_tags_37 },
    { 0x36c5966b, SUC_lexicon_tags_55 },
    { 0x1e25966c, SUC_lexicon_tags_0 },
    { 0x16a5966e, SUC_lexicon_tags_14 },
    { 0xaa45966d, SUC_lexicon_tags_39 },
    { 0xdae59670, SUC_lexicon_tags_8 },
    { 0xbc459671, SUC_lexicon_tags_82 },
    { 0x2ec59672, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2565967a, SUC_lexicon_tags_49 },
    { 0x2a25967b, SUC_lexicon_tags_17 },
    { 0xad65967a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9605967e, SUC_lexicon_tags_31 },
    { 0xc725967e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea59683, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba59686, SUC_lexicon_tags_45 },
    { 0xfea59687, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5305968f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4da59691, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63459695, SUC_lexicon_tags_1 },
    { 0x3d659696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3145969f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf20596a1, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a596a4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8596a7, SUC_lexicon_tags_12 },
    { 0xdde596a8, SUC_lexicon_tags_29 },
    { 0xa46596a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92596ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18596ae, SUC_lexicon_tags_2 },
    { 0xcea596ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e596b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68596b4, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee596c2, SUC_lexicon_tags_7 },
    { 0x396596c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddc596c5, SUC_lexicon_tags_2 },
    { 0x7f8596c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a596c9, SUC_lexicon_tags_1 },
    { 0x90a596ca, SUC_lexicon_tags_31 },
    { 0xa8a596cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66596cf, SUC_lexicon_tags_21 },
    { 0x5f6596d0, SUC_lexicon_tags_11 },
    { 0x544596d1, SUC_lexicon_tags_25 },
    { 0x734596d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3a596d4, SUC_lexicon_tags_8 },
    { 0xf44596d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0596dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c4596de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06596e1, SUC_lexicon_tags_65 },
    { 0x36e596e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83c596e4, SUC_lexicon_tags_26 },
    { 0xa2a596e5, SUC_lexicon_tags_14 },
    { 0xed2596e6, SUC_lexicon_tags_46 },
    { 0x3bc596e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74596eb, SUC_lexicon_tags_11 },
    { 0x94a596eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00596f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a596f3, SUC_lexicon_tags_63 },
    { 0xca8596f4, SUC_lexicon_tags_19 },
    { 0x1bc596f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6596f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e596fc, SUC_lexicon_tags_11 },
    { 0xc596fc, SUC_lexicon_tags_17 },
    { 0xb0c596fe, SUC_lexicon_tags_12 },
    { 0xd26596fe, SUC_lexicon_tags_39 },
    { 0xd16596fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfac59702, SUC_lexicon_tags_17 },
    { 0xea259703, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa59706, SUC_lexicon_tags_26 },
    { 0x90059707, SUC_lexicon_tags_160 },
    { 0xe9059706, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7545970a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c85970d, SUC_lexicon_tags_2 },
    { 0x9185970d, SUC_lexicon_tags_2 },
    { 0x7fe5970f, SUC_lexicon_tags_47 },
    { 0xcf259710, SUC_lexicon_tags_12 },
    { 0x76059711, SUC_lexicon_tags_7 },
    { 0x26459712, SUC_lexicon_tags_17 },
    { 0x62a5970e, SUC_lexicon_tags_68 },
    { 0xd9a59712, SUC_lexicon_tags_7 },
    { 0x76059715, SUC_lexicon_tags_25 },
    { 0x50459716, SUC_lexicon_tags_25 },
    { 0xa2859717, SUC_lexicon_tags_27 },
    { 0xab659715, SUC_lexicon_tags_27 },
    { 0x54e59711, SUC_lexicon_tags_11 },
    { 0x73459710, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf045971f, SUC_lexicon_tags_25 },
    { 0x10e5971f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbaa59722, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd2c59724, SUC_lexicon_tags_25 },
    { 0x6e259725, SUC_lexicon_tags_11 },
    { 0xbd059726, SUC_lexicon_tags_19 },
    { 0xfe459726, SUC_lexicon_tags_37 },
    { 0xf2659728, SUC_lexicon_tags_84 },
    { 0x24259729, SUC_lexicon_tags_12 },
    { 0x8d459725, SUC_lexicon_tags_11 },
    { 0xd185972b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x485972d, SUC_lexicon_tags_25 },
    { 0xc045972e, SUC_lexicon_tags_7 },
    { 0xa85972d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28e59731, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f859736, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x62859738, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb785973d, SUC_lexicon_tags_2 },
    { 0x6fa5973e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46259742, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12059744, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6459749, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefe5974b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7e5974d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd259750, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce5975a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x465975d, SUC_lexicon_tags_0 },
    { 0xf385975d, SUC_lexicon_tags_38 },
    { 0x5b85975f, SUC_lexicon_tags_27 },
    { 0x4c259760, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef259763, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd9259765, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc59769, SUC_lexicon_tags_1 },
    { 0x39c59769, SUC_lexicon_tags_13 },
    { 0xa245976b, SUC_lexicon_tags_2 },
    { 0x8245976c, SUC_lexicon_tags_17 },
    { 0xf105976c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71059771, SUC_lexicon_tags_12 },
    { 0xbd459772, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda59775, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8425977a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x645977c, SUC_lexicon_tags_2 },
    { 0x49c5977d, SUC_lexicon_tags_104 },
    { 0xc025977e, SUC_lexicon_tags_37 },
    { 0x4c65977f, SUC_lexicon_tags_14 },
    { 0xd305977f, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e59785, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa059787, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29459789, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde5978c, SUC_lexicon_tags_37 },
    { 0x45c5978d, SUC_lexicon_tags_7 },
    { 0xfda5978c, SUC_lexicon_tags_35 },
    { 0xb845978f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1059791, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a597a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c597ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7b6597ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7b8597b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x538597b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x748597b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2597b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866597bf, SUC_lexicon_tags_12 },
    { 0xf94597c0, SUC_lexicon_tags_45 },
    { 0x168597c1, SUC_lexicon_tags_879 },
    { 0xb86597c1, SUC_lexicon_tags_39 },
    { 0x4ea597c3, SUC_lexicon_tags_4 },
    { 0x57a597c4, SUC_lexicon_tags_1 },
    { 0xe1e597c1, SUC_lexicon_tags_17 },
    { 0xd6597c0, SUC_lexicon_tags_46 },
    { 0xea597c7, SUC_lexicon_tags_17 },
    { 0x90c597c7, SUC_lexicon_tags_15 },
    { 0xae2597c7, SUC_lexicon_tags_140 },
    { 0xcd6597c8, SUC_lexicon_tags_12 },
    { 0x2d8597cb, SUC_lexicon_tags_17 },
    { 0x4bc597cb, SUC_lexicon_tags_35 },
    { 0x1ce597cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23a597cf, SUC_lexicon_tags_15 },
    { 0x37e597cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2597d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132597d8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xcd2597da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06597df, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00597e8, SUC_lexicon_tags_11 },
    { 0x2e2597e9, SUC_lexicon_tags_47 },
    { 0xdd2597e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96597ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4597f3, SUC_lexicon_tags_94 },
    { 0x9b4597f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2597f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64597fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4597ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8d059801, SUC_lexicon_tags_19 },
    { 0xf859801, SUC_lexicon_tags_0 },
    { 0x41059801, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c459805, SUC_lexicon_tags_25 },
    { 0x26a59806, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce59809, SUC_lexicon_tags_14 },
    { 0xf25980a, SUC_lexicon_tags_27 },
    { 0x9345980a, SUC_lexicon_tags_70 },
    { 0xd605980c, SUC_lexicon_tags_22 },
    { 0x11c5980a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xec659810, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x48259813, SUC_lexicon_tags_25 },
    { 0x40259814, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e259817, SUC_lexicon_tags_1 },
    { 0x63259818, SUC_lexicon_tags_2 },
    { 0x50859817, SUC_lexicon_tags_25 },
    { 0x10459819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe665981c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c65981f, SUC_lexicon_tags_27 },
    { 0x8bc5981f, SUC_lexicon_tags_30 },
    { 0x1c05981f, SUC_lexicon_tags_47 },
    { 0x9ca59822, SUC_lexicon_tags_23 },
    { 0x3d259822, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x77859826, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd859829, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e5982c, SUC_lexicon_tags_2 },
    { 0x2dc5982d, SUC_lexicon_tags_2 },
    { 0x2205982d, SUC_lexicon_tags_37 },
    { 0x2865982d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2059834, SUC_lexicon_tags_30 },
    { 0x3a459835, SUC_lexicon_tags_1 },
    { 0x15a59836, SUC_lexicon_tags_39 },
    { 0x39859837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed05983a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c5983f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87859841, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e59844, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87859846, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3fc59848, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25984c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6659854, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2ba59856, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfb259858, SUC_lexicon_tags_0 },
    { 0x57e59859, SUC_lexicon_tags_12 },
    { 0x6da5985a, SUC_lexicon_tags_7 },
    { 0x7885985a, SUC_lexicon_tags_29 },
    { 0xefa5985c, SUC_lexicon_tags_16 },
    { 0x91e5985d, SUC_lexicon_tags_15 },
    { 0xb345985a, SUC_lexicon_tags_11 },
    { 0xb205985f, SUC_lexicon_tags_26 },
    { 0x585985a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42059866, SUC_lexicon_tags_15 },
    { 0x20e59867, SUC_lexicon_tags_7 },
    { 0x4f259868, SUC_lexicon_tags_7 },
    { 0xdd259869, SUC_lexicon_tags_37 },
    { 0x46a59868, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x20e5986c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd1e5986e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a59874, SUC_lexicon_tags_38 },
    { 0x74259875, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1145987a, SUC_lexicon_tags_12 },
    { 0xfd45987a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf865987f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbe59881, SUC_lexicon_tags_0 },
    { 0x22c59881, SUC_lexicon_tags_12 },
    { 0x14c59883, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9ce59885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e59888, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f65988b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e5988f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8a059891, SUC_lexicon_tags_0 },
    { 0xe6459892, SUC_lexicon_tags_2 },
    { 0xc2a59892, SUC_lexicon_tags_14 },
    { 0xa6259892, SUC_lexicon_tags_63 },
    { 0xf8259894, SUC_lexicon_tags_87 },
    { 0x2b859896, SUC_lexicon_tags_15 },
    { 0xf6e59896, SUC_lexicon_tags_8 },
    { 0x1f859898, SUC_lexicon_tags_1 },
    { 0xf3059899, SUC_lexicon_tags_77 },
    { 0xb0259899, SUC_lexicon_tags_15 },
    { 0xb9c5989b, SUC_lexicon_tags_37 },
    { 0xbae5989c, SUC_lexicon_tags_15 },
    { 0xc065989b, SUC_lexicon_tags_43 },
    { 0x67659899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84e598a0, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x906598a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c598a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c598aa, SUC_lexicon_tags_8 },
    { 0x2d0598ab, SUC_lexicon_tags_4 },
    { 0x614598aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386598b0, SUC_lexicon_tags_8 },
    { 0xa64598b1, SUC_lexicon_tags_12 },
    { 0xaa6598b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4598b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458598b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64598bc, SUC_lexicon_tags_7 },
    { 0x64e598bd, SUC_lexicon_tags_11 },
    { 0xd10598be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a598c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c598c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfae598c6, SUC_lexicon_tags_1 },
    { 0xc598c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe2598c9, SUC_lexicon_tags_1 },
    { 0xe74598ca, SUC_lexicon_tags_14 },
    { 0x330598cb, SUC_lexicon_tags_7 },
    { 0xde4598c9, SUC_lexicon_tags_16 },
    { 0x8d2598cd, SUC_lexicon_tags_37 },
    { 0xcd2598cd, SUC_lexicon_tags_197 },
    { 0x34e598cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6598d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x75c598d4, SUC_lexicon_tags_24 },
    { 0x138598d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c598d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xef0598db, SUC_lexicon_tags_1 },
    { 0x770598dc, SUC_lexicon_tags_39 },
    { 0x524598db, SUC_lexicon_tags_11 },
    { 0xc72598de, SUC_lexicon_tags_31 },
    { 0x622598df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x92e598e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2598ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66598f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66598fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x508598fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e59903, SUC_lexicon_tags_11 },
    { 0xece59904, SUC_lexicon_tags_7 },
    { 0x9f659905, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x56c59907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea5990b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5105990d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x39c5990f, SUC_lexicon_tags_21 },
    { 0x6d659910, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2259917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66459919, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a45991b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e5991f, SUC_lexicon_tags_18 },
    { 0xb0859920, SUC_lexicon_tags_4 },
    { 0xd5659920, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d659928, SUC_lexicon_tags_16 },
    { 0xe3e59929, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdda5992b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac59930, SUC_lexicon_tags_47 },
    { 0x8ca59931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6859934, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9a59936, SUC_lexicon_tags_43 },
    { 0xe2e59937, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae5993a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e659943, SUC_lexicon_tags_31 },
    { 0x9de59944, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14859946, SUC_lexicon_tags_8 },
    { 0x49a59947, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf045994e, SUC_lexicon_tags_1 },
    { 0x9a25994f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a459953, SUC_lexicon_tags_37 },
    { 0x91659954, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a59958, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xac5995a, SUC_lexicon_tags_2 },
    { 0x25c5995a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xee05995e, SUC_lexicon_tags_1 },
    { 0xcf85995f, SUC_lexicon_tags_7 },
    { 0xf985995f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f859964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fc59966, SUC_lexicon_tags_45 },
    { 0x7a659966, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb405996e, SUC_lexicon_tags_37 },
    { 0xafa5996f, SUC_lexicon_tags_11 },
    { 0x20259970, SUC_lexicon_tags_6 },
    { 0xb7659971, SUC_lexicon_tags_16 },
    { 0xc0459972, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6725997e, SUC_lexicon_tags_8 },
    { 0xc9c5997e, SUC_lexicon_tags_34 },
    { 0xb8259980, SUC_lexicon_tags_11 },
    { 0x11659981, SUC_lexicon_tags_25 },
    { 0xb259981, SUC_lexicon_tags_17 },
    { 0xd2259980, SUC_lexicon_tags_38 },
    { 0xc8859984, SUC_lexicon_tags_1 },
    { 0xdfa59985, SUC_lexicon_tags_26 },
    { 0x36a59986, SUC_lexicon_tags_87 },
    { 0x72859987, SUC_lexicon_tags_11 },
    { 0x1645997f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdee5998a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2da5998c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7859994, SUC_lexicon_tags_45 },
    { 0x41e59994, SUC_lexicon_tags_17 },
    { 0x42459995, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6259998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c65999a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc599a0, SUC_lexicon_tags_8 },
    { 0x650599a1, SUC_lexicon_tags_55 },
    { 0xf72599a2, SUC_lexicon_tags_85 },
    { 0x956599a3, SUC_lexicon_tags_65 },
    { 0x2d2599a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8599ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598599b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x98e599b2, SUC_lexicon_tags_8 },
    { 0xf70599b3, SUC_lexicon_tags_118 },
    { 0x538599b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeea599b6, SUC_lexicon_tags_7 },
    { 0x68a599b7, SUC_lexicon_tags_45 },
    { 0xdfc599b8, SUC_lexicon_tags_33 },
    { 0x350599b9, SUC_lexicon_tags_11 },
    { 0xc4599ba, SUC_lexicon_tags_11 },
    { 0xe0a599b7, SUC_lexicon_tags_28 },
    { 0xd98599bc, SUC_lexicon_tags_11 },
    { 0x1da599bd, SUC_lexicon_tags_29 },
    { 0x6e4599bd, SUC_lexicon_tags_60 },
    { 0xd04599be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1e2599c1, SUC_lexicon_tags_2 },
    { 0xaa599c1, SUC_lexicon_tags_43 },
    { 0xa02599c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b6599c5, SUC_lexicon_tags_37 },
    { 0x346599c5, SUC_lexicon_tags_12 },
    { 0x61e599c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x972599ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x384599cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef0599ce, SUC_lexicon_tags_37 },
    { 0xb80599cf, SUC_lexicon_tags_44 },
    { 0x8b6599cf, SUC_lexicon_tags_57 },
    { 0x55a599cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x876599d4, SUC_lexicon_tags_25 },
    { 0x6e599d5, SUC_lexicon_tags_38 },
    { 0xea2599d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x270599d8, SUC_lexicon_tags_17 },
    { 0x7a4599d9, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac599df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x19c599e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66599e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda599ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x248599ed, SUC_lexicon_tags_21 },
    { 0x972599ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x702599f0, SUC_lexicon_tags_33 },
    { 0x7ee599f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x754599f4, SUC_lexicon_tags_0 },
    { 0x876599f5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e59a01, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b259a04, SUC_lexicon_tags_17 },
    { 0xc3059a04, SUC_lexicon_tags_11 },
    { 0xe5259a06, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a59a0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85659a0d, SUC_lexicon_tags_16 },
    { 0xdba59a0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f859a14, SUC_lexicon_tags_45 },
    { 0x7de59a14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3a59a17, SUC_lexicon_tags_43 },
    { 0x9ac59a17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc59a1e, SUC_lexicon_tags_7 },
    { 0x96e59a1e, SUC_lexicon_tags_25 },
    { 0x7b859a1f, SUC_lexicon_tags_17 },
    { 0x9cc59a21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d859a23, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91259a2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae659a2c, SUC_lexicon_tags_11 },
    { 0x68659a2c, SUC_lexicon_tags_1 },
    { 0x22e59a2c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed859a32, SUC_lexicon_tags_11 },
    { 0x70a59a32, SUC_lexicon_tags_2 },
    { 0x22059a33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21459a3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee859a3f, SUC_lexicon_tags_2 },
    { 0xcba59a3f, SUC_lexicon_tags_37 },
    { 0xb2c59a3f, SUC_lexicon_tags_37 },
    { 0x75a59a41, SUC_lexicon_tags_1065 },
    { 0x83259a3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57659a47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c659a4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c659a4f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5459a51, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d659a58, SUC_lexicon_tags_25 },
    { 0x10a59a58, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c59a5e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade59a62, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb7c59a64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7059a6a, SUC_lexicon_tags_33 },
    { 0x4ce59a6a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73059a71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d059a77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf2859a79, SUC_lexicon_tags_17 },
    { 0x1c459a7a, SUC_lexicon_tags_7 },
    { 0x30459a7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92c59a7d, SUC_lexicon_tags_29 },
    { 0xa5459a7d, SUC_lexicon_tags_37 },
    { 0x59859a7f, SUC_lexicon_tags_1 },
    { 0x4d859a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb659a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca59a86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x46459a89, SUC_lexicon_tags_2 },
    { 0x42e59a8a, SUC_lexicon_tags_37 },
    { 0x3ec59a8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f659a8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd259a90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x54459a92, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a59a98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23459a9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22259a9e, SUC_lexicon_tags_11 },
    { 0x25c59a9f, SUC_lexicon_tags_61 },
    { 0xf3259aa0, SUC_lexicon_tags_55 },
    { 0x50059aa0, SUC_lexicon_tags_27 },
    { 0xfd459a9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22659aa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc859aa9, SUC_lexicon_tags_47 },
    { 0xbc259aaa, SUC_lexicon_tags_17 },
    { 0x29659aaa, SUC_lexicon_tags_79 },
    { 0x2b459aac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de59aaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a59ab3, SUC_lexicon_tags_26 },
    { 0xf7059ab3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea459ab6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x22059ab9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c59abc, SUC_lexicon_tags_58 },
    { 0xf6e59abd, SUC_lexicon_tags_1 },
    { 0x2b459abd, SUC_lexicon_tags_37 },
    { 0xfa659abf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd059ac1, SUC_lexicon_tags_63 },
    { 0x8a059ac1, SUC_lexicon_tags_15 },
    { 0x9be59ac3, SUC_lexicon_tags_17 },
    { 0xeee59ac4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x56059ac6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5dc59ac8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdb459aca, SUC_lexicon_tags_17 },
    { 0xc4259acb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c59ace, SUC_lexicon_tags_17 },
    { 0x5fe59ace, SUC_lexicon_tags_10 },
    { 0x4f459acf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf859ad2, SUC_lexicon_tags_37 },
    { 0x48c59ad2, SUC_lexicon_tags_12 },
    { 0xbbe59ad4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e59ad9, SUC_lexicon_tags_37 },
    { 0xaf659ada, SUC_lexicon_tags_61 },
    { 0x5859adb, SUC_lexicon_tags_60 },
    { 0xe0459adb, SUC_lexicon_tags_45 },
    { 0x9c59adc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7659adf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2059ae2, SUC_lexicon_tags_53 },
    { 0x26e59ae3, SUC_lexicon_tags_4 },
    { 0xbf859ae4, SUC_lexicon_tags_12 },
    { 0xc8e59ae5, SUC_lexicon_tags_2 },
    { 0x68059ae6, SUC_lexicon_tags_15 },
    { 0x73859ae5, SUC_lexicon_tags_17 },
    { 0x75059ae4, SUC_lexicon_tags_46 },
    { 0xfa859ae9, SUC_lexicon_tags_63 },
    { 0xbda59aea, SUC_lexicon_tags_15 },
    { 0x2ec59aeb, SUC_lexicon_tags_43 },
    { 0xf2059aec, SUC_lexicon_tags_45 },
    { 0x92459ae8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9b059aef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d059af5, SUC_lexicon_tags_60 },
    { 0x88c59af6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf659af8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e59afc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe459afe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95259b02, SUC_lexicon_tags_11 },
    { 0xc0259b03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec059b07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c59b0e, SUC_lexicon_tags_3 },
    { 0xce259b0f, SUC_lexicon_tags_29 },
    { 0xad859b0f, SUC_lexicon_tags_10 },
    { 0x3659b11, SUC_lexicon_tags_26 },
    { 0xec659b12, SUC_lexicon_tags_63 },
    { 0x2fc59b13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8859b17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b259b19, SUC_lexicon_tags_29 },
    { 0xf9459b1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c59b1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64259b25, SUC_lexicon_tags_6 },
    { 0x19259b26, SUC_lexicon_tags_1 },
    { 0x6d059b26, SUC_lexicon_tags_108 },
    { 0x68459b26, SUC_lexicon_tags_0 },
    { 0x37459b26, SUC_lexicon_tags_47 },
    { 0x3de59b26, SUC_lexicon_tags_25 },
    { 0x42859b2b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a59b2e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a59b31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9659b34, SUC_lexicon_tags_1466 },
    { 0xff659b35, SUC_lexicon_tags_49 },
    { 0x9fa59b36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42659b38, SUC_lexicon_tags_65 },
    { 0x49859b39, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5659b3b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51059b3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e59b42, SUC_lexicon_tags_47 },
    { 0xb3659b43, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x85a59b45, SUC_lexicon_tags_25 },
    { 0xc6659b45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5659b48, SUC_lexicon_tags_1450 },
    { 0, NULL },
    { 0x5a659b4a, SUC_lexicon_tags_17 },
    { 0x61c59b4b, SUC_lexicon_tags_87 },
    { 0xd3659b4c, SUC_lexicon_tags_4 },
    { 0x2e259b4b, SUC_lexicon_tags_61 },
    { 0x62859b4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e059b50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43459b52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca659b56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29659b59, SUC_lexicon_tags_15 },
    { 0x39e59b5a, SUC_lexicon_tags_2 },
    { 0x25c59b5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a59b5f, SUC_lexicon_tags_16 },
    { 0xa5059b60, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37059b69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d059b6b, SUC_lexicon_tags_17 },
    { 0x7e59b6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25e59b6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90659b78, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a59b7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef859b82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d659b84, SUC_lexicon_tags_31 },
    { 0xf3059b84, SUC_lexicon_tags_15 },
    { 0x58459b86, SUC_lexicon_tags_15 },
    { 0x60c59b87, SUC_lexicon_tags_8 },
    { 0x6d859b88, SUC_lexicon_tags_32 },
    { 0xf6459b84, SUC_lexicon_tags_55 },
    { 0x28e59b86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba59b8d, SUC_lexicon_tags_8 },
    { 0x78a59b8e, SUC_lexicon_tags_47 },
    { 0x3b659b8f, SUC_lexicon_tags_37 },
    { 0xf8e59b90, SUC_lexicon_tags_14 },
    { 0xa7459b8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6859b93, SUC_lexicon_tags_25 },
    { 0x90e59b94, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e859b9a, SUC_lexicon_tags_11 },
    { 0x99a59b9a, SUC_lexicon_tags_7 },
    { 0x47459b9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc0859b9e, SUC_lexicon_tags_37 },
    { 0x3de59b9f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f859ba7, SUC_lexicon_tags_15 },
    { 0x18a59ba8, SUC_lexicon_tags_23 },
    { 0xb1459ba8, SUC_lexicon_tags_15 },
    { 0xc459ba9, SUC_lexicon_tags_48 },
    { 0x8c859ba8, SUC_lexicon_tags_17 },
    { 0xa0c59baa, SUC_lexicon_tags_15 },
    { 0x9b259bad, SUC_lexicon_tags_19 },
    { 0x2f859bae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a59bb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x95259bb4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x78659bb6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcb459bb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x93459bbb, SUC_lexicon_tags_65 },
    { 0xaa659bbb, SUC_lexicon_tags_1 },
    { 0x76259bbd, SUC_lexicon_tags_30 },
    { 0x7ce59bbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3859bc0, SUC_lexicon_tags_15 },
    { 0xed459bc1, SUC_lexicon_tags_0 },
    { 0xa5259bc2, SUC_lexicon_tags_25 },
    { 0x4e459bc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71859bc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e59bcc, SUC_lexicon_tags_49 },
    { 0x4e59bcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64459bd1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x13e59bd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec59bd6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44c59bd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d859bdb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc59bde, SUC_lexicon_tags_28 },
    { 0x1a059bdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc659be5, SUC_lexicon_tags_17 },
    { 0x34a59be5, SUC_lexicon_tags_26 },
    { 0x11259be7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c59beb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e59bee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfca59bf0, SUC_lexicon_tags_23 },
    { 0xbca59bf1, SUC_lexicon_tags_37 },
    { 0x92859bf2, SUC_lexicon_tags_23 },
    { 0x93259bf3, SUC_lexicon_tags_12 },
    { 0x11c59bf4, SUC_lexicon_tags_7 },
    { 0x4e059bf5, SUC_lexicon_tags_27 },
    { 0x11859bf6, SUC_lexicon_tags_81 },
    { 0x31a59bf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93259bf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde459bfb, SUC_lexicon_tags_12 },
    { 0x56459bfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48a59bfe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc259c01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33259c05, SUC_lexicon_tags_0 },
    { 0x6e459c06, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a59c0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7659c0d, SUC_lexicon_tags_1 },
    { 0x5c859c0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fc59c10, SUC_lexicon_tags_25 },
    { 0x9ba59c10, SUC_lexicon_tags_761 },
    { 0x46659c12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae059c15, SUC_lexicon_tags_48 },
    { 0xd0a59c16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x72e59c18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a59c1b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda859c23, SUC_lexicon_tags_25 },
    { 0xace59c24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c59c29, SUC_lexicon_tags_61 },
    { 0xe9059c2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28e59c2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e059c2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a859c32, SUC_lexicon_tags_68 },
    { 0xa4259c33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d859c3d, SUC_lexicon_tags_34 },
    { 0xd3659c3e, SUC_lexicon_tags_63 },
    { 0x6059c3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a59c45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84a59c47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c59c49, SUC_lexicon_tags_120 },
    { 0x2dc59c49, SUC_lexicon_tags_37 },
    { 0x35059c4a, SUC_lexicon_tags_37 },
    { 0x88a59c4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46659c50, SUC_lexicon_tags_1 },
    { 0x1c59c51, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x52659c54, SUC_lexicon_tags_1 },
    { 0xe5859c54, SUC_lexicon_tags_17 },
    { 0x3e459c54, SUC_lexicon_tags_17 },
    { 0x74659c57, SUC_lexicon_tags_54 },
    { 0x1f459c56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x80e59c5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa59c5e, SUC_lexicon_tags_1 },
    { 0xc7c59c5f, SUC_lexicon_tags_37 },
    { 0xbcc59c5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca59c63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb059c65, SUC_lexicon_tags_16 },
    { 0x5d059c66, SUC_lexicon_tags_17 },
    { 0x1f659c66, SUC_lexicon_tags_45 },
    { 0xce659c66, SUC_lexicon_tags_61 },
    { 0x59a59c66, SUC_lexicon_tags_33 },
    { 0x1859c6a, SUC_lexicon_tags_1 },
    { 0x66c59c65, SUC_lexicon_tags_63 },
    { 0x2c59c65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec59c70, SUC_lexicon_tags_7 },
    { 0xafa59c71, SUC_lexicon_tags_14 },
    { 0xb3c59c72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17459c75, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d659c78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7259c7a, SUC_lexicon_tags_1 },
    { 0xf5459c7b, SUC_lexicon_tags_15 },
    { 0x83259c7c, SUC_lexicon_tags_1349 },
    { 0x48659c7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe259c83, SUC_lexicon_tags_46 },
    { 0x1d659c84, SUC_lexicon_tags_11 },
    { 0x36459c84, SUC_lexicon_tags_13 },
    { 0xad059c84, SUC_lexicon_tags_32 },
    { 0xa9459c87, SUC_lexicon_tags_37 },
    { 0x94259c87, SUC_lexicon_tags_23 },
    { 0xb0059c85, SUC_lexicon_tags_45 },
    { 0x95c59c8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xede59c8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83859c8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a59c94, SUC_lexicon_tags_12 },
    { 0x32a59c95, SUC_lexicon_tags_2 },
    { 0x8ee59c95, SUC_lexicon_tags_14 },
    { 0xb6259c97, SUC_lexicon_tags_14 },
    { 0x57059c97, SUC_lexicon_tags_45 },
    { 0x40459c99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x92459c9b, SUC_lexicon_tags_12 },
    { 0xf1459c9c, SUC_lexicon_tags_2 },
    { 0x11259c9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9259ca0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x65059ca2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a859ca6, SUC_lexicon_tags_7 },
    { 0x23859ca7, SUC_lexicon_tags_31 },
    { 0x21259ca8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb459cab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd059cad, SUC_lexicon_tags_1241 },
    { 0xb8659cad, SUC_lexicon_tags_45 },
    { 0x11859caf, SUC_lexicon_tags_37 },
    { 0x35c59caf, SUC_lexicon_tags_1 },
    { 0x41259cb0, SUC_lexicon_tags_43 },
    { 0x84e59cb1, SUC_lexicon_tags_11 },
    { 0x5f859cb3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x76259cb5, SUC_lexicon_tags_1 },
    { 0xcba59cb5, SUC_lexicon_tags_65 },
    { 0x19059cb6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xad059cba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a59cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd859cc6, SUC_lexicon_tags_23 },
    { 0xc3a59cc7, SUC_lexicon_tags_29 },
    { 0xa2259cc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c259ccd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b259cd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba459cd2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4859cd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a59cd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60e59cdb, SUC_lexicon_tags_17 },
    { 0x70859cdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3c59cde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e59ce1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e59ce4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e59ced, SUC_lexicon_tags_1 },
    { 0x87a59cee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9459cf4, SUC_lexicon_tags_15 },
    { 0x8fa59cf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c59cf9, SUC_lexicon_tags_1 },
    { 0xd6059cf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60459cfe, SUC_lexicon_tags_6 },
    { 0x70859cff, SUC_lexicon_tags_6 },
    { 0xee59d00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39659d02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40059d0a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf459d0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb259d10, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53459d17, SUC_lexicon_tags_28 },
    { 0xb4859d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca59d1b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c59d1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c59d21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c59d27, SUC_lexicon_tags_25 },
    { 0xa659d28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfc59d2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f459d2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x98659d2e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5659d31, SUC_lexicon_tags_11 },
    { 0x27659d32, SUC_lexicon_tags_0 },
    { 0x82e59d33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa59d38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a59d3b, SUC_lexicon_tags_63 },
    { 0xee059d3b, SUC_lexicon_tags_17 },
    { 0x3bc59d3c, SUC_lexicon_tags_37 },
    { 0xfd259d3b, SUC_lexicon_tags_25 },
    { 0x79859d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5259d41, SUC_lexicon_tags_25 },
    { 0xffe59d41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x92659d44, SUC_lexicon_tags_32 },
    { 0x91c59d45, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0059d4b, SUC_lexicon_tags_0 },
    { 0xf3e59d4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59859d55, SUC_lexicon_tags_77 },
    { 0x77059d56, SUC_lexicon_tags_68 },
    { 0x55659d56, SUC_lexicon_tags_14 },
    { 0x95059d57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90259d5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5059d5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe59d64, SUC_lexicon_tags_37 },
    { 0x98859d65, SUC_lexicon_tags_37 },
    { 0xf2a59d66, SUC_lexicon_tags_15 },
    { 0x2459d66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0459d6b, SUC_lexicon_tags_2 },
    { 0x48c59d6c, SUC_lexicon_tags_33 },
    { 0xd8859d6d, SUC_lexicon_tags_17 },
    { 0x9a059d6e, SUC_lexicon_tags_6 },
    { 0xdd259d6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb059d72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a859d74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93659d7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf459d7c, SUC_lexicon_tags_19 },
    { 0x3b059d7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d459d7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1659d81, SUC_lexicon_tags_14 },
    { 0x9059d82, SUC_lexicon_tags_23 },
    { 0x57659d83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11259d85, SUC_lexicon_tags_15 },
    { 0xaf459d85, SUC_lexicon_tags_68 },
    { 0x42e59d87, SUC_lexicon_tags_15 },
    { 0xc5859d87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f059d90, SUC_lexicon_tags_38 },
    { 0xaa659d91, SUC_lexicon_tags_37 },
    { 0x82e59d90, SUC_lexicon_tags_25 },
    { 0x8de59d93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36a59d95, SUC_lexicon_tags_32 },
    { 0x3f059d96, SUC_lexicon_tags_37 },
    { 0xd8e59d97, SUC_lexicon_tags_63 },
    { 0x65459d98, SUC_lexicon_tags_1 },
    { 0x9459d99, SUC_lexicon_tags_12 },
    { 0x82c59d99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94059d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3859da1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc0659da3, SUC_lexicon_tags_23 },
    { 0x74c59da4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfc259da6, SUC_lexicon_tags_87 },
    { 0x66859da6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5ca59da9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca259dad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659db6, SUC_lexicon_tags_2 },
    { 0xfa459db6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8659dbc, SUC_lexicon_tags_17 },
    { 0x26e59dbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca459dc3, SUC_lexicon_tags_45 },
    { 0xbcc59dc4, SUC_lexicon_tags_38 },
    { 0xaae59dc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a59dc9, SUC_lexicon_tags_1 },
    { 0xbfc59dc9, SUC_lexicon_tags_11 },
    { 0x58259dca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7059dcf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1f059dd1, SUC_lexicon_tags_25 },
    { 0xa3e59dd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x12659dd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2459ddf, SUC_lexicon_tags_37 },
    { 0xa2e59ddf, SUC_lexicon_tags_14 },
    { 0xd6a59ddf, SUC_lexicon_tags_1 },
    { 0x3d259de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c59de5, SUC_lexicon_tags_63 },
    { 0x7c259de6, SUC_lexicon_tags_47 },
    { 0x69059de6, SUC_lexicon_tags_120 },
    { 0x13e59de8, SUC_lexicon_tags_1 },
    { 0x9fc59de9, SUC_lexicon_tags_49 },
    { 0x9fc59dea, SUC_lexicon_tags_23 },
    { 0xa9059de6, SUC_lexicon_tags_2 },
    { 0xc3659de9, SUC_lexicon_tags_25 },
    { 0x33459ded, SUC_lexicon_tags_17 },
    { 0x6aa59ded, SUC_lexicon_tags_7 },
    { 0x28659def, SUC_lexicon_tags_11 },
    { 0x63659df0, SUC_lexicon_tags_14 },
    { 0x859df1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a59df3, SUC_lexicon_tags_37 },
    { 0x78859df4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x15659df7, SUC_lexicon_tags_10 },
    { 0x1fe59df8, SUC_lexicon_tags_12 },
    { 0xe1059df9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e59dfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39459e02, SUC_lexicon_tags_2 },
    { 0xc1659e02, SUC_lexicon_tags_25 },
    { 0x77059e03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e59e08, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e59e0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82859e17, SUC_lexicon_tags_63 },
    { 0xb6459e18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c59e1c, SUC_lexicon_tags_9 },
    { 0x80a59e1d, SUC_lexicon_tags_7 },
    { 0x8ee59e1e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c59e22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80859e26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c59e2a, SUC_lexicon_tags_15 },
    { 0xeda59e2b, SUC_lexicon_tags_100 },
    { 0x5a459e2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16459e2e, SUC_lexicon_tags_4 },
    { 0xe5a59e2f, SUC_lexicon_tags_77 },
    { 0xf1e59e2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de59e35, SUC_lexicon_tags_45 },
    { 0x97259e36, SUC_lexicon_tags_25 },
    { 0xa4a59e37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc59e44, SUC_lexicon_tags_7 },
    { 0x8ee59e45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1459e49, SUC_lexicon_tags_92 },
    { 0xd4659e49, SUC_lexicon_tags_49 },
    { 0xed659e4b, SUC_lexicon_tags_179 },
    { 0x41859e4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f859e4e, SUC_lexicon_tags_6 },
    { 0x80c59e4f, SUC_lexicon_tags_8 },
    { 0x84a59e4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e59e58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd859e64, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e59e69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7d659e6b, SUC_lexicon_tags_253 },
    { 0x75259e6c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1c59e6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0259e71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b659e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d859e77, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc8c59e79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf059e80, SUC_lexicon_tags_17 },
    { 0xba059e81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c859e84, SUC_lexicon_tags_63 },
    { 0x3dc59e84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaec59e87, SUC_lexicon_tags_15 },
    { 0x54459e88, SUC_lexicon_tags_15 },
    { 0x79059e87, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7859e8d, SUC_lexicon_tags_12 },
    { 0x58459e8e, SUC_lexicon_tags_11 },
    { 0x96c59e8f, SUC_lexicon_tags_25 },
    { 0x17c59e8f, SUC_lexicon_tags_4 },
    { 0xcca59e91, SUC_lexicon_tags_30 },
    { 0x8a259e92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3659e95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91659ea1, SUC_lexicon_tags_17 },
    { 0x84259ea2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54059ea7, SUC_lexicon_tags_37 },
    { 0x4f259ea8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2859eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2859eaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e659eb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7e59eb3, SUC_lexicon_tags_15 },
    { 0x88e59eb4, SUC_lexicon_tags_51 },
    { 0x4459eb5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x67259eb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2659ebc, SUC_lexicon_tags_28 },
    { 0xc0659ebd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2e59ebf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf3e59ec1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c59ec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83059ecd, SUC_lexicon_tags_2 },
    { 0x3f059ece, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25459ed0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e59ed5, SUC_lexicon_tags_15 },
    { 0x69259ed5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec459ed8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43259eda, SUC_lexicon_tags_33 },
    { 0x35c59eda, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21859ee3, SUC_lexicon_tags_25 },
    { 0x92659ee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9859ee7, SUC_lexicon_tags_17 },
    { 0xc5259ee8, SUC_lexicon_tags_7 },
    { 0x85659ee9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2459eef, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e459ef3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b259ef6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb6859ef8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48259efa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1059efc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e59eff, SUC_lexicon_tags_57 },
    { 0x86a59f00, SUC_lexicon_tags_100 },
    { 0x40459f01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e59f06, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e59f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9e59f0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9459f14, SUC_lexicon_tags_0 },
    { 0xf1c59f15, SUC_lexicon_tags_14 },
    { 0x88c59f16, SUC_lexicon_tags_25 },
    { 0xa6e59f17, SUC_lexicon_tags_6 },
    { 0x5f459f18, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xacc59f1a, SUC_lexicon_tags_35 },
    { 0x49059f1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76459f1d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfb859f1f, SUC_lexicon_tags_37 },
    { 0xf9859f1f, SUC_lexicon_tags_49 },
    { 0x4ee59f20, SUC_lexicon_tags_33 },
    { 0xeee59f1f, SUC_lexicon_tags_1 },
    { 0x88c59f1f, SUC_lexicon_tags_37 },
    { 0x53a59f23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6fe59f26, SUC_lexicon_tags_14 },
    { 0x9fc59f27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5b659f29, SUC_lexicon_tags_11 },
    { 0x9d059f29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3059f2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ae59f2e, SUC_lexicon_tags_1 },
    { 0xf7a59f2f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e059f32, SUC_lexicon_tags_38 },
    { 0xc4a59f33, SUC_lexicon_tags_8 },
    { 0x33859f34, SUC_lexicon_tags_10 },
    { 0x32e59f35, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a659f39, SUC_lexicon_tags_27 },
    { 0xea259f39, SUC_lexicon_tags_25 },
    { 0xdf059f3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55659f42, SUC_lexicon_tags_8 },
    { 0x3c459f42, SUC_lexicon_tags_201 },
    { 0x78459f43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbac59f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e59f4d, SUC_lexicon_tags_5 },
    { 0x19659f4e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x12059f51, SUC_lexicon_tags_9 },
    { 0x70259f52, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9e059f54, SUC_lexicon_tags_0 },
    { 0x1ee59f54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfac59f57, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8fe59f59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c59f5d, SUC_lexicon_tags_14 },
    { 0x46259f5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64259f60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x61e59f62, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x7e059f64, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x88059f66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4859f69, SUC_lexicon_tags_15 },
    { 0x7dc59f69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3259f70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a59f73, SUC_lexicon_tags_23 },
    { 0x22e59f74, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea59f79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4659f7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84059f85, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc859f8a, SUC_lexicon_tags_13 },
    { 0x4d859f8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e59f90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a59f96, SUC_lexicon_tags_3 },
    { 0xad259f96, SUC_lexicon_tags_2 },
    { 0x93e59f98, SUC_lexicon_tags_63 },
    { 0x92059f97, SUC_lexicon_tags_1 },
    { 0xdd259f97, SUC_lexicon_tags_14 },
    { 0xf1059f9b, SUC_lexicon_tags_11 },
    { 0x25059f9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f859fa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x51259fa3, SUC_lexicon_tags_12 },
    { 0xd3c59fa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e59fa7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb0e59fa9, SUC_lexicon_tags_27 },
    { 0x9f259fa9, SUC_lexicon_tags_27 },
    { 0xda459faa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e59fae, SUC_lexicon_tags_15 },
    { 0xcf859fae, SUC_lexicon_tags_7 },
    { 0xdd059fb0, SUC_lexicon_tags_17 },
    { 0x3fe59fb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb6859fb3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x70a59fb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc59fb8, SUC_lexicon_tags_11 },
    { 0xfce59fb9, SUC_lexicon_tags_47 },
    { 0xf4a59fb8, SUC_lexicon_tags_3 },
    { 0xac659fb9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e59fc1, SUC_lexicon_tags_63 },
    { 0x53859fc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97459fc4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xea59fc7, SUC_lexicon_tags_41 },
    { 0xae459fc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c59fcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c59fd8, SUC_lexicon_tags_47 },
    { 0x67c59fd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f259fdd, SUC_lexicon_tags_37 },
    { 0xb7659fde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42a59fe0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6e59fe2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15259fe6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea59fe9, SUC_lexicon_tags_43 },
    { 0x30a59fea, SUC_lexicon_tags_14 },
    { 0x5a59fea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc59ff4, SUC_lexicon_tags_60 },
    { 0x61859ff5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb4859ff7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48059ffc, SUC_lexicon_tags_15 },
    { 0x5ae59ffd, SUC_lexicon_tags_34 },
    { 0xa9e59ffe, SUC_lexicon_tags_23 },
    { 0x18e59ffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde45a004, SUC_lexicon_tags_7 },
    { 0x62e5a005, SUC_lexicon_tags_47 },
    { 0xe6a5a006, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc025a00a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92e5a00c, SUC_lexicon_tags_2 },
    { 0xc985a00d, SUC_lexicon_tags_1 },
    { 0x23a5a00d, SUC_lexicon_tags_12 },
    { 0xacc5a00f, SUC_lexicon_tags_11 },
    { 0xc085a00e, SUC_lexicon_tags_25 },
    { 0x6565a010, SUC_lexicon_tags_47 },
    { 0x5c65a00e, SUC_lexicon_tags_55 },
    { 0xf265a00c, SUC_lexicon_tags_2 },
    { 0x5ee5a014, SUC_lexicon_tags_63 },
    { 0xbf05a015, SUC_lexicon_tags_63 },
    { 0xc9e5a016, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b05a019, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51c5a01b, SUC_lexicon_tags_60 },
    { 0xc1c5a01c, SUC_lexicon_tags_15 },
    { 0x4a85a01d, SUC_lexicon_tags_38 },
    { 0x1ae5a01c, SUC_lexicon_tags_37 },
    { 0xcc5a01f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a5a025, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3345a027, SUC_lexicon_tags_11 },
    { 0xfce5a028, SUC_lexicon_tags_6 },
    { 0xc9c5a029, SUC_lexicon_tags_12 },
    { 0x8785a02a, SUC_lexicon_tags_47 },
    { 0x9045a02a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe345a030, SUC_lexicon_tags_27 },
    { 0x6be5a030, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9405a033, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa265a036, SUC_lexicon_tags_7 },
    { 0x65e5a037, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa85a03b, SUC_lexicon_tags_43 },
    { 0xb885a03c, SUC_lexicon_tags_2 },
    { 0x725a03c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc425a040, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6265a042, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba25a044, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe65a049, SUC_lexicon_tags_26 },
    { 0xc9e5a049, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3225a04c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb205a052, SUC_lexicon_tags_5 },
    { 0x5645a053, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5665a055, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a5a05c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f25a062, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76a5a064, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e85a066, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765a06e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xae45a071, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5025a07b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a5a07f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae5a082, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3625a085, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef65a08c, SUC_lexicon_tags_12 },
    { 0x4cc5a08d, SUC_lexicon_tags_11 },
    { 0x5225a08e, SUC_lexicon_tags_17 },
    { 0x9245a08e, SUC_lexicon_tags_68 },
    { 0x1d25a08e, SUC_lexicon_tags_12 },
    { 0xfc5a090, SUC_lexicon_tags_9 },
    { 0x3cc5a08f, SUC_lexicon_tags_15 },
    { 0x54e5a093, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd665a096, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc585a09d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9285a0a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab45a0a8, SUC_lexicon_tags_8 },
    { 0x2b45a0a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1d05a0ab, SUC_lexicon_tags_63 },
    { 0x1a05a0ab, SUC_lexicon_tags_25 },
    { 0x9b65a0ad, SUC_lexicon_tags_0 },
    { 0x8965a0ae, SUC_lexicon_tags_66 },
    { 0xbee5a0ad, SUC_lexicon_tags_60 },
    { 0x4285a0af, SUC_lexicon_tags_2 },
    { 0x7225a0b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a5a0b5, SUC_lexicon_tags_38 },
    { 0x6525a0b6, SUC_lexicon_tags_70 },
    { 0x1805a0b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425a0bf, SUC_lexicon_tags_15 },
    { 0x1145a0c0, SUC_lexicon_tags_25 },
    { 0x7405a0c0, SUC_lexicon_tags_37 },
    { 0xcfa5a0c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b45a0ce, SUC_lexicon_tags_0 },
    { 0xc4c5a0cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd585a0d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2725a0d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd6a5a0d9, SUC_lexicon_tags_11 },
    { 0xf845a0d9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x8f65a0dc, SUC_lexicon_tags_1 },
    { 0x1985a0dc, SUC_lexicon_tags_12 },
    { 0x7765a0de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2705a0e2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe505a0e4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6045a0e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a5a0ec, SUC_lexicon_tags_12 },
    { 0xcb45a0ec, SUC_lexicon_tags_12 },
    { 0x7805a0ee, SUC_lexicon_tags_37 },
    { 0xe045a0ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e25a0f5, SUC_lexicon_tags_47 },
    { 0xba5a0f5, SUC_lexicon_tags_0 },
    { 0x1a05a0f7, SUC_lexicon_tags_37 },
    { 0xe485a0f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a5a0fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7265a0ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc5a10a, SUC_lexicon_tags_16 },
    { 0x28a5a10b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe025a114, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x3fe5a116, SUC_lexicon_tags_172 },
    { 0xe9c5a116, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7265a11b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x30e5a11d, SUC_lexicon_tags_12 },
    { 0xdb25a11e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23e5a120, SUC_lexicon_tags_7 },
    { 0xc845a120, SUC_lexicon_tags_11 },
    { 0x5825a120, SUC_lexicon_tags_37 },
    { 0x60c5a123, SUC_lexicon_tags_17 },
    { 0x9b65a121, SUC_lexicon_tags_1330 },
    { 0, NULL },
    { 0x10c5a126, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be5a12b, SUC_lexicon_tags_25 },
    { 0xdce5a12c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1565a12e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc5a134, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a5a138, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec5a141, SUC_lexicon_tags_1 },
    { 0x1e25a142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3405a147, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a5a14b, SUC_lexicon_tags_15 },
    { 0xcce5a14b, SUC_lexicon_tags_27 },
    { 0x1165a14d, SUC_lexicon_tags_26 },
    { 0x425a14e, SUC_lexicon_tags_64 },
    { 0x99e5a14e, SUC_lexicon_tags_7 },
    { 0x42c5a150, SUC_lexicon_tags_12 },
    { 0xa825a14d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae5a155, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4525a15b, SUC_lexicon_tags_25 },
    { 0xa45a15b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c05a15f, SUC_lexicon_tags_187 },
    { 0xb9c5a15f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb25a162, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2885a166, SUC_lexicon_tags_2 },
    { 0xdd85a167, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5605a171, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd985a177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf165a179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac65a17d, SUC_lexicon_tags_48 },
    { 0x81a5a17e, SUC_lexicon_tags_82 },
    { 0xa865a17e, SUC_lexicon_tags_27 },
    { 0x5cc5a17e, SUC_lexicon_tags_38 },
    { 0x35a5a180, SUC_lexicon_tags_12 },
    { 0xbea5a182, SUC_lexicon_tags_66 },
    { 0x5425a17e, SUC_lexicon_tags_9 },
    { 0xc945a183, SUC_lexicon_tags_6 },
    { 0xd125a17f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2565a187, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa865a18f, SUC_lexicon_tags_92 },
    { 0x3525a190, SUC_lexicon_tags_2 },
    { 0x37e5a191, SUC_lexicon_tags_38 },
    { 0x9ae5a190, SUC_lexicon_tags_17 },
    { 0x4725a191, SUC_lexicon_tags_43 },
    { 0x9345a194, SUC_lexicon_tags_15 },
    { 0xc505a195, SUC_lexicon_tags_63 },
    { 0x6845a196, SUC_lexicon_tags_6 },
    { 0x17c5a197, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e5a1a1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa8a5a1a3, SUC_lexicon_tags_63 },
    { 0x7365a1a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a5a1a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb005a1af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xad85a1b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e5a1b5, SUC_lexicon_tags_43 },
    { 0xeb45a1b5, SUC_lexicon_tags_0 },
    { 0xcfe5a1b6, SUC_lexicon_tags_60 },
    { 0x72c5a1b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc245a1ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6725a1bc, SUC_lexicon_tags_8 },
    { 0xe6e5a1bd, SUC_lexicon_tags_2 },
    { 0x205a1be, SUC_lexicon_tags_2 },
    { 0xbe25a1bc, SUC_lexicon_tags_1 },
    { 0xcb05a1c0, SUC_lexicon_tags_10 },
    { 0x84a5a1c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6305a1c5, SUC_lexicon_tags_15 },
    { 0xd0c5a1c6, SUC_lexicon_tags_23 },
    { 0x4925a1c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d85a1c9, SUC_lexicon_tags_797 },
    { 0xb365a1ca, SUC_lexicon_tags_17 },
    { 0x1245a1cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b25a1cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc605a1d1, SUC_lexicon_tags_2 },
    { 0x3da5a1d2, SUC_lexicon_tags_116 },
    { 0xc6e5a1d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaac5a1d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee5a1db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3be5a1dd, SUC_lexicon_tags_0 },
    { 0x44e5a1dd, SUC_lexicon_tags_17 },
    { 0x7c45a1de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5a1e5, SUC_lexicon_tags_23 },
    { 0x525a1e6, SUC_lexicon_tags_11 },
    { 0xb1e5a1e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7825a1e9, SUC_lexicon_tags_37 },
    { 0x4665a1e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdd85a1ec, SUC_lexicon_tags_8 },
    { 0x4f45a1ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f05a1f2, SUC_lexicon_tags_3 },
    { 0x3505a1f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5d25a1f5, SUC_lexicon_tags_66 },
    { 0x1ec5a1f6, SUC_lexicon_tags_15 },
    { 0xa4c5a1f7, SUC_lexicon_tags_27 },
    { 0xb165a1f8, SUC_lexicon_tags_17 },
    { 0xb885a1f9, SUC_lexicon_tags_12 },
    { 0x8225a1fa, SUC_lexicon_tags_29 },
    { 0x1b05a1f7, SUC_lexicon_tags_48 },
    { 0xc145a1f7, SUC_lexicon_tags_7 },
    { 0xc0c5a1fd, SUC_lexicon_tags_30 },
    { 0x3ee5a1fe, SUC_lexicon_tags_11 },
    { 0xec05a1fe, SUC_lexicon_tags_25 },
    { 0xd05a1fe, SUC_lexicon_tags_1 },
    { 0xa545a1f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be5a205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf205a208, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14c5a20a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe25a20c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc25a210, SUC_lexicon_tags_33 },
    { 0xc4a5a210, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7585a214, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245a219, SUC_lexicon_tags_33 },
    { 0xfd25a21a, SUC_lexicon_tags_12 },
    { 0x33a5a21a, SUC_lexicon_tags_17 },
    { 0x1145a21a, SUC_lexicon_tags_35 },
    { 0x1e85a21d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb525a223, SUC_lexicon_tags_2 },
    { 0x23c5a224, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9f85a226, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd165a228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9745a22a, SUC_lexicon_tags_69 },
    { 0xb565a22b, SUC_lexicon_tags_12 },
    { 0xcd45a22b, SUC_lexicon_tags_14 },
    { 0xc9e5a22c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce05a232, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe465a234, SUC_lexicon_tags_43 },
    { 0x1225a235, SUC_lexicon_tags_0 },
    { 0x6825a236, SUC_lexicon_tags_87 },
    { 0x8ae5a237, SUC_lexicon_tags_31 },
    { 0x1585a236, SUC_lexicon_tags_15 },
    { 0xfd85a239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e5a23d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2225a243, SUC_lexicon_tags_25 },
    { 0xf2a5a244, SUC_lexicon_tags_19 },
    { 0xf3c5a245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6565a247, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9485a249, SUC_lexicon_tags_12 },
    { 0xf825a24a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a85a24c, SUC_lexicon_tags_2 },
    { 0x78c5a24d, SUC_lexicon_tags_55 },
    { 0x8165a24d, SUC_lexicon_tags_11 },
    { 0x20a5a24d, SUC_lexicon_tags_25 },
    { 0x6825a250, SUC_lexicon_tags_26 },
    { 0x3fa5a24d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5a257, SUC_lexicon_tags_47 },
    { 0x4c25a258, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x50e5a25a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfc5a25c, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c05a25f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8745a261, SUC_lexicon_tags_25 },
    { 0x8f45a261, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb1e5a264, SUC_lexicon_tags_0 },
    { 0xfd25a264, SUC_lexicon_tags_13 },
    { 0xdda5a266, SUC_lexicon_tags_14 },
    { 0x12e5a267, SUC_lexicon_tags_15 },
    { 0xdb65a264, SUC_lexicon_tags_45 },
    { 0x8785a266, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd485a26b, SUC_lexicon_tags_55 },
    { 0x58c5a26c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e25a26f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185a273, SUC_lexicon_tags_29 },
    { 0x3fe5a274, SUC_lexicon_tags_27 },
    { 0x9585a273, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c5a283, SUC_lexicon_tags_17 },
    { 0xd2c5a284, SUC_lexicon_tags_10 },
    { 0x7f65a285, SUC_lexicon_tags_9 },
    { 0xd665a286, SUC_lexicon_tags_37 },
    { 0x76c5a287, SUC_lexicon_tags_27 },
    { 0x6a25a287, SUC_lexicon_tags_14 },
    { 0xda25a289, SUC_lexicon_tags_12 },
    { 0xd5e5a289, SUC_lexicon_tags_0 },
    { 0x2bc5a28b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa165a28f, SUC_lexicon_tags_11 },
    { 0x6345a290, SUC_lexicon_tags_11 },
    { 0xc5c5a28f, SUC_lexicon_tags_60 },
    { 0x78a5a291, SUC_lexicon_tags_12 },
    { 0x745a28f, SUC_lexicon_tags_33 },
    { 0xc565a294, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab85a297, SUC_lexicon_tags_11 },
    { 0x17c5a297, SUC_lexicon_tags_2 },
    { 0x1bc5a298, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9525a29e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61a5a2a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd865a2a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c5a2a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58a5a2aa, SUC_lexicon_tags_15 },
    { 0x8805a2aa, SUC_lexicon_tags_21 },
    { 0x3e05a2aa, SUC_lexicon_tags_25 },
    { 0xcd25a2ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe785a2b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe225a2b8, SUC_lexicon_tags_37 },
    { 0xa885a2b8, SUC_lexicon_tags_33 },
    { 0x5ec5a2ba, SUC_lexicon_tags_8 },
    { 0x4ee5a2bb, SUC_lexicon_tags_11 },
    { 0xcd25a2b8, SUC_lexicon_tags_25 },
    { 0xe025a2b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a5a2c6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd65a2c8, SUC_lexicon_tags_12 },
    { 0x6525a2c9, SUC_lexicon_tags_17 },
    { 0x7bc5a2ca, SUC_lexicon_tags_45 },
    { 0xa5c5a2ca, SUC_lexicon_tags_37 },
    { 0xeec5a2ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8545a2db, SUC_lexicon_tags_61 },
    { 0x5525a2dc, SUC_lexicon_tags_37 },
    { 0x1745a2dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8085a2e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f45a2e8, SUC_lexicon_tags_25 },
    { 0x4ce5a2e9, SUC_lexicon_tags_11 },
    { 0xd845a2ea, SUC_lexicon_tags_1 },
    { 0xfc5a2eb, SUC_lexicon_tags_0 },
    { 0x3f65a2ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec5a2f1, SUC_lexicon_tags_15 },
    { 0x4f85a2f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa5a2f5, SUC_lexicon_tags_45 },
    { 0x83e5a2f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d25a2fb, SUC_lexicon_tags_15 },
    { 0xd505a2fc, SUC_lexicon_tags_17 },
    { 0x3325a2fd, SUC_lexicon_tags_108 },
    { 0xa525a2fe, SUC_lexicon_tags_17 },
    { 0x2de5a2ff, SUC_lexicon_tags_31 },
    { 0xc3c5a2fd, SUC_lexicon_tags_17 },
    { 0xf6a5a301, SUC_lexicon_tags_2 },
    { 0xa145a302, SUC_lexicon_tags_7 },
    { 0xf4c5a303, SUC_lexicon_tags_15 },
    { 0xd2c5a2fd, SUC_lexicon_tags_9 },
    { 0x6825a305, SUC_lexicon_tags_37 },
    { 0x2525a300, SUC_lexicon_tags_25 },
    { 0xa765a2fd, SUC_lexicon_tags_28 },
    { 0x7b05a308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3805a30e, SUC_lexicon_tags_37 },
    { 0x8b45a30f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c5a312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63e5a314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7645a31b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfd05a31d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4025a31f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf05a324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcde5a326, SUC_lexicon_tags_21 },
    { 0x4ea5a327, SUC_lexicon_tags_47 },
    { 0xcd65a328, SUC_lexicon_tags_28 },
    { 0x13c5a328, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccc5a32b, SUC_lexicon_tags_45 },
    { 0xf245a32c, SUC_lexicon_tags_17 },
    { 0x64e5a32d, SUC_lexicon_tags_37 },
    { 0xde45a32e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6285a335, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4a5a337, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3c5a339, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc585a33b, SUC_lexicon_tags_33 },
    { 0x7fa5a33c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e5a345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8825a348, SUC_lexicon_tags_16 },
    { 0xea65a349, SUC_lexicon_tags_17 },
    { 0x1ce5a34a, SUC_lexicon_tags_7 },
    { 0x1265a34a, SUC_lexicon_tags_4 },
    { 0x4725a34c, SUC_lexicon_tags_0 },
    { 0xcee5a348, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5a354, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e25a357, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b85a35a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365a35e, SUC_lexicon_tags_12 },
    { 0xe2e5a35e, SUC_lexicon_tags_7 },
    { 0xea65a360, SUC_lexicon_tags_63 },
    { 0x2585a35e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd845a366, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4965a368, SUC_lexicon_tags_15 },
    { 0x35a5a368, SUC_lexicon_tags_141 },
    { 0xd365a369, SUC_lexicon_tags_15 },
    { 0x7f25a369, SUC_lexicon_tags_15 },
    { 0xf885a368, SUC_lexicon_tags_37 },
    { 0xffe5a36d, SUC_lexicon_tags_63 },
    { 0xd365a36e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5a376, SUC_lexicon_tags_21 },
    { 0x265a377, SUC_lexicon_tags_37 },
    { 0x4a05a377, SUC_lexicon_tags_28 },
    { 0xd565a379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa05a37b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb625a37f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a5a382, SUC_lexicon_tags_25 },
    { 0x7a85a383, SUC_lexicon_tags_1517 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c5a38c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37e5a38e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865a396, SUC_lexicon_tags_27 },
    { 0x8a65a397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3745a39d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd865a3a3, SUC_lexicon_tags_11 },
    { 0x5865a3a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc485a3a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc5a3a9, SUC_lexicon_tags_25 },
    { 0xa4a5a3aa, SUC_lexicon_tags_11 },
    { 0x9a05a3aa, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7125a3b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd585a3b6, SUC_lexicon_tags_31 },
    { 0x2405a3b6, SUC_lexicon_tags_49 },
    { 0x81e5a3b8, SUC_lexicon_tags_37 },
    { 0xba85a3b9, SUC_lexicon_tags_11 },
    { 0x4a25a3b8, SUC_lexicon_tags_7 },
    { 0x9425a3bb, SUC_lexicon_tags_14 },
    { 0x1745a3b9, SUC_lexicon_tags_0 },
    { 0x2425a3bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d25a3bf, SUC_lexicon_tags_14 },
    { 0x5e25a3c0, SUC_lexicon_tags_47 },
    { 0xbe5a3c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2fe5a3c3, SUC_lexicon_tags_11 },
    { 0xab85a3c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7c5a3c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b05a3ca, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd2a5a3cc, SUC_lexicon_tags_45 },
    { 0x45a5a3cd, SUC_lexicon_tags_10 },
    { 0x2405a3ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x72e5a3d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec5a3d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2365a3d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe965a3da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdce5a3dc, SUC_lexicon_tags_15 },
    { 0xfa5a3dd, SUC_lexicon_tags_49 },
    { 0xf685a3dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x985a3e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1625a3e4, SUC_lexicon_tags_15 },
    { 0x2c65a3e4, SUC_lexicon_tags_7 },
    { 0xa9a5a3e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa205a3eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac5a3f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb1e5a3f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e5a3fc, SUC_lexicon_tags_27 },
    { 0xd025a3fd, SUC_lexicon_tags_15 },
    { 0x9165a3fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b85a405, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4465a40a, SUC_lexicon_tags_15 },
    { 0xb685a40a, SUC_lexicon_tags_25 },
    { 0x5845a40c, SUC_lexicon_tags_0 },
    { 0xb185a40d, SUC_lexicon_tags_27 },
    { 0xa625a40c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfa05a410, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1c5a412, SUC_lexicon_tags_2 },
    { 0x8d45a413, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8205a417, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6885a419, SUC_lexicon_tags_19 },
    { 0x90e5a419, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8705a41e, SUC_lexicon_tags_11 },
    { 0x3885a41f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f65a423, SUC_lexicon_tags_44 },
    { 0x3d85a424, SUC_lexicon_tags_37 },
    { 0x1d85a424, SUC_lexicon_tags_45 },
    { 0x3c5a424, SUC_lexicon_tags_25 },
    { 0xf825a427, SUC_lexicon_tags_25 },
    { 0xf3a5a428, SUC_lexicon_tags_29 },
    { 0xbc65a428, SUC_lexicon_tags_11 },
    { 0x77e5a42a, SUC_lexicon_tags_17 },
    { 0x9bc5a429, SUC_lexicon_tags_33 },
    { 0x8a85a42c, SUC_lexicon_tags_30 },
    { 0xc445a428, SUC_lexicon_tags_28 },
    { 0xdc25a429, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba65a430, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3565a433, SUC_lexicon_tags_17 },
    { 0x9025a434, SUC_lexicon_tags_12 },
    { 0xea85a434, SUC_lexicon_tags_7 },
    { 0x6bc5a436, SUC_lexicon_tags_37 },
    { 0x6e85a437, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83c5a439, SUC_lexicon_tags_45 },
    { 0xe945a439, SUC_lexicon_tags_31 },
    { 0x5125a43b, SUC_lexicon_tags_65 },
    { 0x8e5a439, SUC_lexicon_tags_19 },
    { 0x1245a43b, SUC_lexicon_tags_9 },
    { 0x785a43e, SUC_lexicon_tags_1 },
    { 0x6225a439, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9645a443, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe5a447, SUC_lexicon_tags_206 },
    { 0x4425a448, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x44a5a44a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d05a451, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba65a457, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd845a45c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b05a462, SUC_lexicon_tags_11 },
    { 0x71a5a463, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef65a46c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6425a470, SUC_lexicon_tags_1 },
    { 0x9a65a471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dc5a473, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8165a47b, SUC_lexicon_tags_15 },
    { 0x27c5a47b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd545a481, SUC_lexicon_tags_41 },
    { 0x8aa5a482, SUC_lexicon_tags_7 },
    { 0x7b05a483, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcae5a485, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb85a489, SUC_lexicon_tags_82 },
    { 0x62e5a48a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc845a48c, SUC_lexicon_tags_10 },
    { 0x7225a48c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1865a48f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5a493, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc925a495, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c85a498, SUC_lexicon_tags_37 },
    { 0x26a5a499, SUC_lexicon_tags_6 },
    { 0xc885a49a, SUC_lexicon_tags_33 },
    { 0xd485a49b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c45a49d, SUC_lexicon_tags_2 },
    { 0x8105a49e, SUC_lexicon_tags_16 },
    { 0x9425a49f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e5a4a3, SUC_lexicon_tags_25 },
    { 0xd905a4a4, SUC_lexicon_tags_7 },
    { 0xa9a5a4a4, SUC_lexicon_tags_116 },
    { 0x1e45a4a6, SUC_lexicon_tags_0 },
    { 0x5f45a4a6, SUC_lexicon_tags_30 },
    { 0xbe65a4a8, SUC_lexicon_tags_1 },
    { 0x2a85a4a9, SUC_lexicon_tags_70 },
    { 0x1065a4a7, SUC_lexicon_tags_55 },
    { 0x36c5a4aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a5a4b3, SUC_lexicon_tags_14 },
    { 0x3c05a4b4, SUC_lexicon_tags_33 },
    { 0xe005a4b5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe1c5a4b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5365a4b9, SUC_lexicon_tags_62 },
    { 0xf345a4ba, SUC_lexicon_tags_28 },
    { 0x38a5a4ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8805a4bd, SUC_lexicon_tags_30 },
    { 0xdf25a4bd, SUC_lexicon_tags_45 },
    { 0xb505a4be, SUC_lexicon_tags_27 },
    { 0xcce5a4c0, SUC_lexicon_tags_37 },
    { 0x8225a4c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaea5a4c3, SUC_lexicon_tags_14 },
    { 0xab85a4c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c5a4ca, SUC_lexicon_tags_25 },
    { 0x9b25a4cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2cc5a4cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf645a4d0, SUC_lexicon_tags_35 },
    { 0xd9c5a4d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb65a4d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc5a4d7, SUC_lexicon_tags_118 },
    { 0x4845a4d8, SUC_lexicon_tags_117 },
    { 0x2605a4d7, SUC_lexicon_tags_7 },
    { 0x64a5a4da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a5a4de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe865a4e1, SUC_lexicon_tags_19 },
    { 0x36c5a4e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5a4e7, SUC_lexicon_tags_39 },
    { 0xab25a4e8, SUC_lexicon_tags_37 },
    { 0xe0a5a4e9, SUC_lexicon_tags_17 },
    { 0xab85a4e8, SUC_lexicon_tags_47 },
    { 0xb0e5a4e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d85a4f3, SUC_lexicon_tags_37 },
    { 0xa445a4f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a5a4f9, SUC_lexicon_tags_47 },
    { 0x2745a4f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf405a4fd, SUC_lexicon_tags_10 },
    { 0xe405a4fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a5a502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f45a505, SUC_lexicon_tags_30 },
    { 0x72c5a506, SUC_lexicon_tags_14 },
    { 0x1d05a507, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8a5a509, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe05a50f, SUC_lexicon_tags_25 },
    { 0xa565a50f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b65a514, SUC_lexicon_tags_65 },
    { 0xc3c5a514, SUC_lexicon_tags_17 },
    { 0xb965a516, SUC_lexicon_tags_7 },
    { 0xa645a517, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce5a51b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xef85a51d, SUC_lexicon_tags_25 },
    { 0xe425a51e, SUC_lexicon_tags_19 },
    { 0xbf25a51f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825a525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec5a52d, SUC_lexicon_tags_49 },
    { 0x2c85a52e, SUC_lexicon_tags_10 },
    { 0x6b25a52f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e5a533, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7165a538, SUC_lexicon_tags_31 },
    { 0x1105a539, SUC_lexicon_tags_25 },
    { 0x3e25a53a, SUC_lexicon_tags_37 },
    { 0x5325a53a, SUC_lexicon_tags_11 },
    { 0xbfe5a53a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb85a540, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f45a544, SUC_lexicon_tags_33 },
    { 0x51a5a545, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6145a547, SUC_lexicon_tags_17 },
    { 0xeb85a548, SUC_lexicon_tags_5 },
    { 0xe725a549, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd205a54e, SUC_lexicon_tags_17 },
    { 0xea5a54f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc305a552, SUC_lexicon_tags_25 },
    { 0xfd05a552, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9e5a555, SUC_lexicon_tags_17 },
    { 0x9265a556, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f05a55b, SUC_lexicon_tags_65 },
    { 0xee25a55b, SUC_lexicon_tags_49 },
    { 0x3f85a55d, SUC_lexicon_tags_12 },
    { 0x40e5a55c, SUC_lexicon_tags_38 },
    { 0xeaa5a55f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8645a561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc85a569, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa505a56b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc585a56f, SUC_lexicon_tags_17 },
    { 0x7a85a570, SUC_lexicon_tags_25 },
    { 0xc665a570, SUC_lexicon_tags_22 },
    { 0x2005a570, SUC_lexicon_tags_28 },
    { 0x7725a571, SUC_lexicon_tags_17 },
    { 0x9b25a572, SUC_lexicon_tags_49 },
    { 0xa9e5a575, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a5a578, SUC_lexicon_tags_60 },
    { 0xbb65a579, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7885a588, SUC_lexicon_tags_65 },
    { 0x51c5a589, SUC_lexicon_tags_25 },
    { 0x8105a58a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e25a591, SUC_lexicon_tags_57 },
    { 0x7d25a591, SUC_lexicon_tags_7 },
    { 0x8985a593, SUC_lexicon_tags_15 },
    { 0x2505a594, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7765a596, SUC_lexicon_tags_31 },
    { 0x8325a597, SUC_lexicon_tags_37 },
    { 0x5645a597, SUC_lexicon_tags_27 },
    { 0x2bc5a599, SUC_lexicon_tags_0 },
    { 0x5bc5a597, SUC_lexicon_tags_7 },
    { 0x5785a599, SUC_lexicon_tags_68 },
    { 0xc545a59c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xefa5a59e, SUC_lexicon_tags_9 },
    { 0xf745a59f, SUC_lexicon_tags_7 },
    { 0x4aa5a5a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac5a5a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c5a5ab, SUC_lexicon_tags_15 },
    { 0x7825a5ac, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1125a5b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3d05a5b4, SUC_lexicon_tags_10 },
    { 0x6d05a5b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb645a5b7, SUC_lexicon_tags_1252 },
    { 0x1c45a5b8, SUC_lexicon_tags_82 },
    { 0xbe05a5b8, SUC_lexicon_tags_84 },
    { 0x3605a5ba, SUC_lexicon_tags_17 },
    { 0x9c05a5bb, SUC_lexicon_tags_12 },
    { 0xe2c5a5b9, SUC_lexicon_tags_32 },
    { 0x8da5a5bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fe5a5bf, SUC_lexicon_tags_0 },
    { 0xc005a5c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e05a5c5, SUC_lexicon_tags_15 },
    { 0x4885a5c5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x285a5c8, SUC_lexicon_tags_60 },
    { 0xa6e5a5c8, SUC_lexicon_tags_7 },
    { 0xdac5a5c9, SUC_lexicon_tags_40 },
    { 0xe305a5cb, SUC_lexicon_tags_8 },
    { 0xf8e5a5c9, SUC_lexicon_tags_69 },
    { 0xe525a5c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf2c5a5cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce5a5d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc5a5d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd05a5d7, SUC_lexicon_tags_25 },
    { 0x2045a5d7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x12c5a5da, SUC_lexicon_tags_15 },
    { 0x4325a5db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c5a5e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c5a5e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x545a5e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa5a5ee, SUC_lexicon_tags_15 },
    { 0x9f05a5ef, SUC_lexicon_tags_30 },
    { 0x9145a5f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c5a5f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e5a5f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2785a5fc, SUC_lexicon_tags_7 },
    { 0x7e25a5fd, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xd165a5ff, SUC_lexicon_tags_9 },
    { 0x1c5a600, SUC_lexicon_tags_32 },
    { 0xbfa5a601, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb325a604, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c5a60d, SUC_lexicon_tags_33 },
    { 0x9385a60d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c45a611, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb45a616, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3465a618, SUC_lexicon_tags_60 },
    { 0x125a619, SUC_lexicon_tags_17 },
    { 0xe705a61a, SUC_lexicon_tags_29 },
    { 0x8565a61b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2365a61e, SUC_lexicon_tags_37 },
    { 0x2ae5a61e, SUC_lexicon_tags_48 },
    { 0xd3a5a620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd45a622, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2245a625, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9165a627, SUC_lexicon_tags_17 },
    { 0x96c5a627, SUC_lexicon_tags_129 },
    { 0x9625a629, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb9e5a62b, SUC_lexicon_tags_37 },
    { 0x2405a62c, SUC_lexicon_tags_15 },
    { 0xef65a62b, SUC_lexicon_tags_4 },
    { 0x4825a62c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc225a634, SUC_lexicon_tags_37 },
    { 0x7885a635, SUC_lexicon_tags_37 },
    { 0xe565a635, SUC_lexicon_tags_25 },
    { 0x21e5a637, SUC_lexicon_tags_1 },
    { 0x9b25a637, SUC_lexicon_tags_11 },
    { 0xb1a5a639, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9505a63c, SUC_lexicon_tags_43 },
    { 0x75a5a63d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x55e5a63f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e5a644, SUC_lexicon_tags_15 },
    { 0xdbe5a645, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c5a649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc545a64d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bc5a64f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c25a657, SUC_lexicon_tags_37 },
    { 0x60e5a658, SUC_lexicon_tags_47 },
    { 0x2445a659, SUC_lexicon_tags_25 },
    { 0x9fc5a657, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe005a65c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec5a65f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4865a667, SUC_lexicon_tags_37 },
    { 0x8ae5a668, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b05a66b, SUC_lexicon_tags_2 },
    { 0xc5e5a66c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9965a66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5c5a670, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b45a674, SUC_lexicon_tags_43 },
    { 0x8505a675, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7205a67a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6465a67f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1745a683, SUC_lexicon_tags_127 },
    { 0xa605a684, SUC_lexicon_tags_47 },
    { 0x8425a685, SUC_lexicon_tags_38 },
    { 0xf225a683, SUC_lexicon_tags_6 },
    { 0x41a5a687, SUC_lexicon_tags_23 },
    { 0x9145a687, SUC_lexicon_tags_15 },
    { 0xc1e5a689, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1005a68b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5a690, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9345a693, SUC_lexicon_tags_15 },
    { 0xad85a693, SUC_lexicon_tags_4 },
    { 0xba85a695, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc05a69a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x585a69d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77e5a69f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf725a6a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d45a6a6, SUC_lexicon_tags_27 },
    { 0x3a45a6a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4485a6ac, SUC_lexicon_tags_47 },
    { 0x8b45a6ac, SUC_lexicon_tags_15 },
    { 0x5a65a6ae, SUC_lexicon_tags_7 },
    { 0xa865a6af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc505a6b3, SUC_lexicon_tags_17 },
    { 0xb005a6b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b25a6b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4a5a6ba, SUC_lexicon_tags_27 },
    { 0x785a6ba, SUC_lexicon_tags_38 },
    { 0x1745a6bc, SUC_lexicon_tags_43 },
    { 0x95e5a6bb, SUC_lexicon_tags_252 },
    { 0x465a6bb, SUC_lexicon_tags_12 },
    { 0xaa05a6bf, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf465a6c4, SUC_lexicon_tags_1 },
    { 0x2625a6c5, SUC_lexicon_tags_7 },
    { 0xe805a6c5, SUC_lexicon_tags_12 },
    { 0xaec5a6c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c5a6d2, SUC_lexicon_tags_37 },
    { 0xf965a6d2, SUC_lexicon_tags_4 },
    { 0x9505a6d4, SUC_lexicon_tags_37 },
    { 0xf765a6d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3ae5a6d7, SUC_lexicon_tags_2 },
    { 0xbe85a6d8, SUC_lexicon_tags_15 },
    { 0xfa05a6d9, SUC_lexicon_tags_8 },
    { 0x4a65a6d8, SUC_lexicon_tags_1 },
    { 0x7e25a6d9, SUC_lexicon_tags_25 },
    { 0x2805a6d7, SUC_lexicon_tags_7 },
    { 0x4485a6dd, SUC_lexicon_tags_37 },
    { 0x28c5a6de, SUC_lexicon_tags_12 },
    { 0xc945a6d9, SUC_lexicon_tags_0 },
    { 0x9585a6df, SUC_lexicon_tags_26 },
    { 0x3ac5a6d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec5a6e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1385a6ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5885a6ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb285a6ee, SUC_lexicon_tags_38 },
    { 0xfac5a6ef, SUC_lexicon_tags_17 },
    { 0x26a5a6f0, SUC_lexicon_tags_15 },
    { 0xb765a6f0, SUC_lexicon_tags_11 },
    { 0x9765a6f1, SUC_lexicon_tags_15 },
    { 0xd0c5a6f1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xff45a6f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6945a6f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba25a6fa, SUC_lexicon_tags_17 },
    { 0xd2e5a6fb, SUC_lexicon_tags_8 },
    { 0x5c5a6fa, SUC_lexicon_tags_37 },
    { 0x4085a6fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd85a700, SUC_lexicon_tags_77 },
    { 0x2245a701, SUC_lexicon_tags_151 },
    { 0xc925a701, SUC_lexicon_tags_8 },
    { 0xd445a702, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa185a706, SUC_lexicon_tags_25 },
    { 0x50e5a707, SUC_lexicon_tags_60 },
    { 0x4f85a707, SUC_lexicon_tags_11 },
    { 0xe3e5a706, SUC_lexicon_tags_7 },
    { 0xd465a709, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x14c5a70c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3305a70f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b25a711, SUC_lexicon_tags_27 },
    { 0x8d65a712, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a5a71a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c5a71f, SUC_lexicon_tags_7 },
    { 0x1145a71f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a5a723, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf9c5a725, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd305a728, SUC_lexicon_tags_70 },
    { 0x9225a728, SUC_lexicon_tags_30 },
    { 0xb6c5a72a, SUC_lexicon_tags_26 },
    { 0x2f45a72a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd265a72d, SUC_lexicon_tags_37 },
    { 0x3ec5a72e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2085a732, SUC_lexicon_tags_0 },
    { 0xa525a733, SUC_lexicon_tags_11 },
    { 0xfcc5a734, SUC_lexicon_tags_11 },
    { 0x6565a735, SUC_lexicon_tags_4 },
    { 0x1e45a735, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafc5a738, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7665a73a, SUC_lexicon_tags_7 },
    { 0x9145a73b, SUC_lexicon_tags_18 },
    { 0xd305a73c, SUC_lexicon_tags_828 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d65a740, SUC_lexicon_tags_35 },
    { 0x6f25a741, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x54e5a743, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9365a746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b45a74c, SUC_lexicon_tags_11 },
    { 0x2f65a74d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6625a754, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe5a757, SUC_lexicon_tags_45 },
    { 0x4865a757, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da5a75e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2205a764, SUC_lexicon_tags_25 },
    { 0x41c5a764, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b45a76a, SUC_lexicon_tags_31 },
    { 0xeba5a76b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad05a76e, SUC_lexicon_tags_38 },
    { 0x5f45a76e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e5a775, SUC_lexicon_tags_61 },
    { 0x32c5a776, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99e5a778, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd545a77b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e5a77f, SUC_lexicon_tags_61 },
    { 0xdd65a780, SUC_lexicon_tags_7 },
    { 0xc5e5a77f, SUC_lexicon_tags_3 },
    { 0x6385a780, SUC_lexicon_tags_19 },
    { 0x34a5a783, SUC_lexicon_tags_12 },
    { 0xc5a784, SUC_lexicon_tags_7 },
    { 0x9165a785, SUC_lexicon_tags_43 },
    { 0xe3c5a781, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a5a78a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc05a78d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2525a790, SUC_lexicon_tags_8 },
    { 0x8965a790, SUC_lexicon_tags_2 },
    { 0xd265a790, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5005a797, SUC_lexicon_tags_37 },
    { 0x3b85a798, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e45a79a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d85a79f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9f05a7a1, SUC_lexicon_tags_7 },
    { 0x4585a7a1, SUC_lexicon_tags_13 },
    { 0xa3c5a7a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf025a7a6, SUC_lexicon_tags_37 },
    { 0xf905a7a7, SUC_lexicon_tags_0 },
    { 0x4fc5a7a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5de5a7aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9845a7b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe565a7b8, SUC_lexicon_tags_0 },
    { 0xb925a7b9, SUC_lexicon_tags_1 },
    { 0x9905a7ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8a5a7bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac25a7be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c5a7c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf585a7c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8845a7ca, SUC_lexicon_tags_1 },
    { 0x7965a7cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae5a7d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd385a7d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb145a7dc, SUC_lexicon_tags_16 },
    { 0x545a7dd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcca5a7df, SUC_lexicon_tags_37 },
    { 0x6a25a7e0, SUC_lexicon_tags_15 },
    { 0x2225a7e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c5a7e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd605a7e8, SUC_lexicon_tags_19 },
    { 0xb005a7e9, SUC_lexicon_tags_55 },
    { 0xa925a7ea, SUC_lexicon_tags_19 },
    { 0x725a7ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c5a7ee, SUC_lexicon_tags_63 },
    { 0xd5e5a7ef, SUC_lexicon_tags_65 },
    { 0xda45a7ee, SUC_lexicon_tags_37 },
    { 0xcc45a7f1, SUC_lexicon_tags_1 },
    { 0xea25a7f1, SUC_lexicon_tags_47 },
    { 0x78a5a7f3, SUC_lexicon_tags_33 },
    { 0xf8e5a7f4, SUC_lexicon_tags_19 },
    { 0xd345a7f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc5a7f9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3565a7fc, SUC_lexicon_tags_1 },
    { 0xcf25a7fc, SUC_lexicon_tags_91 },
    { 0xd265a7fe, SUC_lexicon_tags_14 },
    { 0xd2c5a7fe, SUC_lexicon_tags_9 },
    { 0xc5a7ff, SUC_lexicon_tags_37 },
    { 0xfc25a7ff, SUC_lexicon_tags_12 },
    { 0x4fa5a802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ba5a804, SUC_lexicon_tags_37 },
    { 0x1425a805, SUC_lexicon_tags_11 },
    { 0x49e5a804, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c5a80d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9325a810, SUC_lexicon_tags_15 },
    { 0x1025a811, SUC_lexicon_tags_15 },
    { 0xea25a812, SUC_lexicon_tags_6 },
    { 0xa5e5a810, SUC_lexicon_tags_25 },
    { 0xac85a813, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd65a81b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf25a81d, SUC_lexicon_tags_11 },
    { 0x2fa5a81d, SUC_lexicon_tags_25 },
    { 0xb6e5a81f, SUC_lexicon_tags_38 },
    { 0xa925a820, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c5a823, SUC_lexicon_tags_69 },
    { 0x2ca5a823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb685a827, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7305a829, SUC_lexicon_tags_25 },
    { 0x92e5a82a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa5a82d, SUC_lexicon_tags_47 },
    { 0xa565a82d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de5a832, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2065a838, SUC_lexicon_tags_30 },
    { 0x8865a838, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8065a83d, SUC_lexicon_tags_8 },
    { 0xaea5a83e, SUC_lexicon_tags_26 },
    { 0xd145a83f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb605a845, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc5a84a, SUC_lexicon_tags_17 },
    { 0x22a5a84b, SUC_lexicon_tags_47 },
    { 0x4ec5a84a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1345a84f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfa85a851, SUC_lexicon_tags_37 },
    { 0xca65a852, SUC_lexicon_tags_481 },
    { 0x565a853, SUC_lexicon_tags_33 },
    { 0xde65a853, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd845a856, SUC_lexicon_tags_47 },
    { 0x66c5a857, SUC_lexicon_tags_16 },
    { 0x6d65a858, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e5a85d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xea5a85f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c5a862, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca25a869, SUC_lexicon_tags_28 },
    { 0xd505a86a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5a86e, SUC_lexicon_tags_6 },
    { 0x45a86f, SUC_lexicon_tags_14 },
    { 0x6725a870, SUC_lexicon_tags_19 },
    { 0xde05a870, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc5a875, SUC_lexicon_tags_2 },
    { 0xe425a875, SUC_lexicon_tags_29 },
    { 0xf45a877, SUC_lexicon_tags_37 },
    { 0xb045a878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8605a87d, SUC_lexicon_tags_15 },
    { 0x4ba5a87e, SUC_lexicon_tags_2 },
    { 0x9785a87d, SUC_lexicon_tags_24 },
    { 0x7a85a880, SUC_lexicon_tags_84 },
    { 0xa9c5a880, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e5a885, SUC_lexicon_tags_34 },
    { 0xde5a885, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfc5a888, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8465a88d, SUC_lexicon_tags_13 },
    { 0x5765a88d, SUC_lexicon_tags_47 },
    { 0xeaa5a88e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e5a892, SUC_lexicon_tags_12 },
    { 0xe785a893, SUC_lexicon_tags_31 },
    { 0x3785a894, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5a85a896, SUC_lexicon_tags_2 },
    { 0xdf25a897, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd285a89b, SUC_lexicon_tags_63 },
    { 0x3d45a89c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2365a89f, SUC_lexicon_tags_138 },
    { 0x4b05a89f, SUC_lexicon_tags_49 },
    { 0xc4c5a8a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaf25a8a3, SUC_lexicon_tags_47 },
    { 0xd9a5a8a3, SUC_lexicon_tags_49 },
    { 0xde85a8a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc5a8a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbca5a8aa, SUC_lexicon_tags_1 },
    { 0x1085a8aa, SUC_lexicon_tags_10 },
    { 0x88a5a8ac, SUC_lexicon_tags_33 },
    { 0x1685a8ad, SUC_lexicon_tags_3 },
    { 0xa4a5a8ad, SUC_lexicon_tags_1 },
    { 0x49c5a8aa, SUC_lexicon_tags_25 },
    { 0x3945a8aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd65a8b5, SUC_lexicon_tags_25 },
    { 0x8065a8b6, SUC_lexicon_tags_32 },
    { 0xc3a5a8b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe445a8b9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d85a8bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1be5a8be, SUC_lexicon_tags_37 },
    { 0x9e25a8be, SUC_lexicon_tags_1 },
    { 0xed25a8c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87c5a8c2, SUC_lexicon_tags_15 },
    { 0xaf25a8c3, SUC_lexicon_tags_33 },
    { 0x6de5a8c4, SUC_lexicon_tags_31 },
    { 0x865a8c4, SUC_lexicon_tags_37 },
    { 0x9a65a8c6, SUC_lexicon_tags_15 },
    { 0xcb65a8c6, SUC_lexicon_tags_17 },
    { 0x2465a8c8, SUC_lexicon_tags_27 },
    { 0x78a5a8c9, SUC_lexicon_tags_37 },
    { 0x9cc5a8ca, SUC_lexicon_tags_49 },
    { 0x11e5a8ca, SUC_lexicon_tags_11 },
    { 0x965a8cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8185a8ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91c5a8d0, SUC_lexicon_tags_17 },
    { 0xf9a5a8d1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x7185a8d3, SUC_lexicon_tags_15 },
    { 0x9b05a8d4, SUC_lexicon_tags_68 },
    { 0x6485a8d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3565a8d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a5a8dd, SUC_lexicon_tags_12 },
    { 0x6365a8de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e5a8e3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e5a8e7, SUC_lexicon_tags_37 },
    { 0xad05a8e7, SUC_lexicon_tags_60 },
    { 0x9565a8e9, SUC_lexicon_tags_25 },
    { 0x12e5a8ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb125a8ed, SUC_lexicon_tags_14 },
    { 0xaac5a8ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba5a8f3, SUC_lexicon_tags_14 },
    { 0xe0a5a8f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d45a8fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c5a906, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2485a90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945a90e, SUC_lexicon_tags_255 },
    { 0x1fa5a90e, SUC_lexicon_tags_33 },
    { 0x2da5a910, SUC_lexicon_tags_46 },
    { 0x5425a911, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae5a916, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8a5a918, SUC_lexicon_tags_9 },
    { 0x4b65a918, SUC_lexicon_tags_27 },
    { 0xd025a91a, SUC_lexicon_tags_8 },
    { 0x4ee5a91a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c45a91d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8645a91f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea85a921, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de5a924, SUC_lexicon_tags_37 },
    { 0x43c5a924, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e5a928, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e85a92b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e5a92f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f45a931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2545a934, SUC_lexicon_tags_38 },
    { 0xd705a935, SUC_lexicon_tags_2 },
    { 0x3565a935, SUC_lexicon_tags_45 },
    { 0x8a5a937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1465a93d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4465a93f, SUC_lexicon_tags_14 },
    { 0xd825a93f, SUC_lexicon_tags_84 },
    { 0xcc45a93f, SUC_lexicon_tags_39 },
    { 0x51e5a942, SUC_lexicon_tags_37 },
    { 0x3885a943, SUC_lexicon_tags_2 },
    { 0x1ec5a944, SUC_lexicon_tags_15 },
    { 0x6f45a945, SUC_lexicon_tags_2 },
    { 0xf565a945, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb265a949, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e5a94c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c5a950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8105a952, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c5a959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc5a95c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc65a95f, SUC_lexicon_tags_49 },
    { 0x90c5a95f, SUC_lexicon_tags_25 },
    { 0x6085a95f, SUC_lexicon_tags_0 },
    { 0xfe25a962, SUC_lexicon_tags_33 },
    { 0x9e45a962, SUC_lexicon_tags_2 },
    { 0x1485a962, SUC_lexicon_tags_8 },
    { 0x1aa5a965, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2045a967, SUC_lexicon_tags_8 },
    { 0x2e65a967, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcbe5a96a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe145a96e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc5a972, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ce5a974, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xad05a976, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca5a97b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xebc5a97d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5125a981, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1025a984, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc125a989, SUC_lexicon_tags_1 },
    { 0xc1e5a98a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d45a98f, SUC_lexicon_tags_13 },
    { 0x1065a990, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8825a993, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e5a997, SUC_lexicon_tags_37 },
    { 0xc545a998, SUC_lexicon_tags_15 },
    { 0xf045a998, SUC_lexicon_tags_160 },
    { 0xe0c5a99a, SUC_lexicon_tags_118 },
    { 0x9745a997, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c05a9a0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xbb85a9a2, SUC_lexicon_tags_2 },
    { 0x2865a9a3, SUC_lexicon_tags_49 },
    { 0xb125a9a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6325a9a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca5a9ad, SUC_lexicon_tags_17 },
    { 0x82e5a9ae, SUC_lexicon_tags_37 },
    { 0x18c5a9ae, SUC_lexicon_tags_29 },
    { 0x8545a9ae, SUC_lexicon_tags_25 },
    { 0x1f65a9ae, SUC_lexicon_tags_2 },
    { 0x2ae5a9b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2625a9b6, SUC_lexicon_tags_8 },
    { 0xa3e5a9b7, SUC_lexicon_tags_0 },
    { 0x185a9b8, SUC_lexicon_tags_7 },
    { 0x85c5a9b8, SUC_lexicon_tags_12 },
    { 0xed65a9b8, SUC_lexicon_tags_14 },
    { 0x97a5a9b8, SUC_lexicon_tags_11 },
    { 0xf4e5a9bc, SUC_lexicon_tags_11 },
    { 0x6c5a9b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9825a9c1, SUC_lexicon_tags_47 },
    { 0x6165a9c2, SUC_lexicon_tags_25 },
    { 0xde25a9c3, SUC_lexicon_tags_47 },
    { 0x5aa5a9c4, SUC_lexicon_tags_2 },
    { 0xd245a9c4, SUC_lexicon_tags_37 },
    { 0x4905a9c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa845a9cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7605a9cd, SUC_lexicon_tags_28 },
    { 0xb405a9ce, SUC_lexicon_tags_17 },
    { 0xede5a9cf, SUC_lexicon_tags_12 },
    { 0xd5e5a9ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4305a9d2, SUC_lexicon_tags_7 },
    { 0x4f25a9d2, SUC_lexicon_tags_15 },
    { 0x7245a9d2, SUC_lexicon_tags_37 },
    { 0xa745a9d5, SUC_lexicon_tags_55 },
    { 0xeec5a9d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5a9d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e5a9dd, SUC_lexicon_tags_27 },
    { 0x4ce5a9dd, SUC_lexicon_tags_26 },
    { 0xf1c5a9df, SUC_lexicon_tags_11 },
    { 0x9a45a9e0, SUC_lexicon_tags_19 },
    { 0xdc45a9df, SUC_lexicon_tags_25 },
    { 0xbd85a9e2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e5a9e7, SUC_lexicon_tags_1 },
    { 0x56a5a9e8, SUC_lexicon_tags_16 },
    { 0x6685a9e8, SUC_lexicon_tags_46 },
    { 0x8005a9ea, SUC_lexicon_tags_45 },
    { 0xccc5a9e8, SUC_lexicon_tags_1 },
    { 0xa365a9ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8c45a9ee, SUC_lexicon_tags_37 },
    { 0x8be5a9ef, SUC_lexicon_tags_26 },
    { 0x4ca5a9ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f65a9f2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe5a9f5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6e05a9f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2745a9fb, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e5aa01, SUC_lexicon_tags_23 },
    { 0x9385aa02, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7025aa05, SUC_lexicon_tags_25 },
    { 0x85a5aa05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5965aa0b, SUC_lexicon_tags_31 },
    { 0x9aa5aa0c, SUC_lexicon_tags_15 },
    { 0xe5c5aa0d, SUC_lexicon_tags_0 },
    { 0x5565aa0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c5aa12, SUC_lexicon_tags_2 },
    { 0xfc85aa13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3125aa17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f25aa1d, SUC_lexicon_tags_25 },
    { 0x5225aa1e, SUC_lexicon_tags_17 },
    { 0x6945aa1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef85aa21, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0x1965aa23, SUC_lexicon_tags_7 },
    { 0x6b45aa23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e5aa27, SUC_lexicon_tags_68 },
    { 0x5fe5aa27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8325aa2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4465aa2c, SUC_lexicon_tags_2 },
    { 0xf085aa2d, SUC_lexicon_tags_11 },
    { 0x29a5aa2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a65aa30, SUC_lexicon_tags_31 },
    { 0x3a65aa31, SUC_lexicon_tags_37 },
    { 0x6345aa32, SUC_lexicon_tags_6 },
    { 0x3465aa30, SUC_lexicon_tags_12 },
    { 0x8145aa34, SUC_lexicon_tags_1 },
    { 0xf945aa34, SUC_lexicon_tags_25 },
    { 0x1965aa35, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc8a5aa38, SUC_lexicon_tags_38 },
    { 0x97a5aa38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a5aa3d, SUC_lexicon_tags_23 },
    { 0xfaa5aa3d, SUC_lexicon_tags_60 },
    { 0xf4a5aa3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a5aa48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2545aa50, SUC_lexicon_tags_29 },
    { 0xf845aa51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c5aa56, SUC_lexicon_tags_11 },
    { 0x69a5aa57, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8025aa5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd645aa61, SUC_lexicon_tags_6 },
    { 0xb325aa62, SUC_lexicon_tags_87 },
    { 0xe5aa63, SUC_lexicon_tags_60 },
    { 0xc8a5aa63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c25aa6c, SUC_lexicon_tags_1 },
    { 0x74e5aa6d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e5aa71, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x55c5aa73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6e5aa75, SUC_lexicon_tags_25 },
    { 0x8ce5aa76, SUC_lexicon_tags_27 },
    { 0x1165aa76, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a5aa7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6465aa7f, SUC_lexicon_tags_32 },
    { 0xf8a5aa80, SUC_lexicon_tags_26 },
    { 0xd485aa7f, SUC_lexicon_tags_60 },
    { 0x1625aa82, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x805aa85, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be5aa88, SUC_lexicon_tags_27 },
    { 0xad05aa89, SUC_lexicon_tags_47 },
    { 0xa105aa8a, SUC_lexicon_tags_48 },
    { 0x645aa8b, SUC_lexicon_tags_3 },
    { 0xd425aa8b, SUC_lexicon_tags_82 },
    { 0xec25aa8c, SUC_lexicon_tags_31 },
    { 0x8365aa8c, SUC_lexicon_tags_70 },
    { 0x3fc5aa8f, SUC_lexicon_tags_1 },
    { 0xd325aa8c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a5aa96, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb25aa99, SUC_lexicon_tags_27 },
    { 0x6cc5aa9a, SUC_lexicon_tags_25 },
    { 0x5105aa99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe785aa9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce05aaa2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd825aaa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9065aaaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb865aaae, SUC_lexicon_tags_17 },
    { 0xe485aaaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e45aab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe025aabd, SUC_lexicon_tags_47 },
    { 0x9fa5aabe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d85aac4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d25aac6, SUC_lexicon_tags_12 },
    { 0xb9e5aac6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4385aac9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5aacc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c85aace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9285aad5, SUC_lexicon_tags_3 },
    { 0xf3a5aad6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c5aae0, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0x47c5aae2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e5aae4, SUC_lexicon_tags_25 },
    { 0x52c5aae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4105aaec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9e5aaee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc5aaf0, SUC_lexicon_tags_28 },
    { 0xf165aaf0, SUC_lexicon_tags_16 },
    { 0x8fc5aaf1, SUC_lexicon_tags_33 },
    { 0x18a5aaf3, SUC_lexicon_tags_7 },
    { 0x19c5aaf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f85aaf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a85aafb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca05aaff, SUC_lexicon_tags_6 },
    { 0xf8a5aaff, SUC_lexicon_tags_37 },
    { 0x905ab00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa5ab04, SUC_lexicon_tags_25 },
    { 0x6f85ab05, SUC_lexicon_tags_4 },
    { 0xe5ab04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8c5ab08, SUC_lexicon_tags_37 },
    { 0xf65ab09, SUC_lexicon_tags_26 },
    { 0x5925ab08, SUC_lexicon_tags_8 },
    { 0x91a5ab0b, SUC_lexicon_tags_16 },
    { 0x22e5ab08, SUC_lexicon_tags_17 },
    { 0xb025ab0d, SUC_lexicon_tags_45 },
    { 0xc905ab09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc5ab12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a5ab1c, SUC_lexicon_tags_47 },
    { 0x8625ab1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfbc5ab1f, SUC_lexicon_tags_46 },
    { 0x8de5ab20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e5ab24, SUC_lexicon_tags_60 },
    { 0x6a25ab25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc565ab28, SUC_lexicon_tags_5 },
    { 0xc645ab29, SUC_lexicon_tags_19 },
    { 0xa805ab2a, SUC_lexicon_tags_37 },
    { 0x60e5ab2a, SUC_lexicon_tags_13 },
    { 0x6ec5ab2c, SUC_lexicon_tags_38 },
    { 0x9ea5ab2c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4145ab2f, SUC_lexicon_tags_12 },
    { 0x2685ab30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc445ab32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1485ab37, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5f05ab39, SUC_lexicon_tags_31 },
    { 0xc2c5ab3a, SUC_lexicon_tags_17 },
    { 0xc505ab3a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc745ab3e, SUC_lexicon_tags_26 },
    { 0x2565ab3f, SUC_lexicon_tags_30 },
    { 0x3425ab40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7865ab42, SUC_lexicon_tags_23 },
    { 0x3d85ab42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f85ab46, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x34e5ab48, SUC_lexicon_tags_37 },
    { 0x2705ab49, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf45ab4d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b65ab50, SUC_lexicon_tags_7 },
    { 0x2b05ab50, SUC_lexicon_tags_4 },
    { 0xcea5ab52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7805ab56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a5ab5d, SUC_lexicon_tags_12 },
    { 0x7005ab5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa965ab63, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x3805ab65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ea5ab67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc5ab6e, SUC_lexicon_tags_1 },
    { 0x1425ab6f, SUC_lexicon_tags_1236 },
    { 0x4705ab70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee5ab75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf545ab78, SUC_lexicon_tags_141 },
    { 0x6625ab79, SUC_lexicon_tags_65 },
    { 0x8505ab78, SUC_lexicon_tags_17 },
    { 0x2045ab7a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8685ab7e, SUC_lexicon_tags_60 },
    { 0xa05ab7e, SUC_lexicon_tags_26 },
    { 0x445ab7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd605ab83, SUC_lexicon_tags_37 },
    { 0xac5ab83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf165ab8c, SUC_lexicon_tags_16 },
    { 0x4cc5ab8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce5ab90, SUC_lexicon_tags_11 },
    { 0x91a5ab91, SUC_lexicon_tags_37 },
    { 0xdb45ab92, SUC_lexicon_tags_3 },
    { 0x42c5ab91, SUC_lexicon_tags_117 },
    { 0xcf85ab93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90e5ab96, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb225ab9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c5ab9d, SUC_lexicon_tags_25 },
    { 0x63a5ab9d, SUC_lexicon_tags_49 },
    { 0xce05ab9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3745aba2, SUC_lexicon_tags_55 },
    { 0x22e5aba3, SUC_lexicon_tags_7 },
    { 0x6ec5aba3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a5aba9, SUC_lexicon_tags_34 },
    { 0x62a5aba9, SUC_lexicon_tags_12 },
    { 0xa6e5abab, SUC_lexicon_tags_55 },
    { 0xa45abac, SUC_lexicon_tags_9 },
    { 0xb585abad, SUC_lexicon_tags_25 },
    { 0x6d25abae, SUC_lexicon_tags_8 },
    { 0xa7e5abaf, SUC_lexicon_tags_33 },
    { 0x8b45abac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1805abb3, SUC_lexicon_tags_23 },
    { 0xb5c5abb4, SUC_lexicon_tags_12 },
    { 0xc285abb5, SUC_lexicon_tags_37 },
    { 0xb505abb4, SUC_lexicon_tags_6 },
    { 0x36e5abb7, SUC_lexicon_tags_63 },
    { 0xa9c5abb7, SUC_lexicon_tags_32 },
    { 0xb505abb9, SUC_lexicon_tags_17 },
    { 0xba45abba, SUC_lexicon_tags_2 },
    { 0x7d85abbb, SUC_lexicon_tags_37 },
    { 0x7285abbb, SUC_lexicon_tags_2 },
    { 0x4625abbb, SUC_lexicon_tags_42 },
    { 0xa9e5abbe, SUC_lexicon_tags_56 },
    { 0xbde5abbb, SUC_lexicon_tags_26 },
    { 0x7b45abc0, SUC_lexicon_tags_0 },
    { 0xc765abba, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f45abc7, SUC_lexicon_tags_11 },
    { 0x2205abc8, SUC_lexicon_tags_37 },
    { 0x2345abc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1625abcc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f05abd1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9345abd5, SUC_lexicon_tags_7 },
    { 0xc25abd6, SUC_lexicon_tags_28 },
    { 0x8be5abd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6465abde, SUC_lexicon_tags_25 },
    { 0x56a5abdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a5abe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e5abe8, SUC_lexicon_tags_60 },
    { 0x1705abe9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed65abec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a5abf0, SUC_lexicon_tags_27 },
    { 0xec45abf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a5abf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea5abf7, SUC_lexicon_tags_19 },
    { 0xbe25abf8, SUC_lexicon_tags_7 },
    { 0xd325abf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb145abfc, SUC_lexicon_tags_7 },
    { 0x10e5abfc, SUC_lexicon_tags_17 },
    { 0xca65abfe, SUC_lexicon_tags_1 },
    { 0xf545abfe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ac04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ac07, SUC_lexicon_tags_19 },
    { 0x5205ac07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe5ac0b, SUC_lexicon_tags_3 },
    { 0xa265ac0c, SUC_lexicon_tags_23 },
    { 0x6965ac0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdaa5ac0f, SUC_lexicon_tags_33 },
    { 0xb105ac10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5285ac13, SUC_lexicon_tags_63 },
    { 0x5aa5ac14, SUC_lexicon_tags_6 },
    { 0xb205ac14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8765ac17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ce5ac19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3205ac24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6a5ac26, SUC_lexicon_tags_2 },
    { 0x8a85ac27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa665ac2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7845ac2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b45ac30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee5ac33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5405ac38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c85ac3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac85ac3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d45ac41, SUC_lexicon_tags_7 },
    { 0xa725ac41, SUC_lexicon_tags_14 },
    { 0xf65ac43, SUC_lexicon_tags_35 },
    { 0x7085ac44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x565ac46, SUC_lexicon_tags_44 },
    { 0xa5a5ac46, SUC_lexicon_tags_17 },
    { 0x10e5ac48, SUC_lexicon_tags_11 },
    { 0x8d05ac48, SUC_lexicon_tags_187 },
    { 0x3865ac4a, SUC_lexicon_tags_8 },
    { 0x8cc5ac4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3685ac4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd445ac4f, SUC_lexicon_tags_0 },
    { 0x7725ac50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd85ac53, SUC_lexicon_tags_11 },
    { 0xc6c5ac54, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xefe5ac56, SUC_lexicon_tags_17 },
    { 0xafc5ac57, SUC_lexicon_tags_18 },
    { 0xe765ac58, SUC_lexicon_tags_77 },
    { 0x48e5ac59, SUC_lexicon_tags_50 },
    { 0x4b45ac58, SUC_lexicon_tags_6 },
    { 0x9905ac5b, SUC_lexicon_tags_17 },
    { 0xb05ac5c, SUC_lexicon_tags_48 },
    { 0x3cc5ac58, SUC_lexicon_tags_61 },
    { 0x9305ac5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2645ac61, SUC_lexicon_tags_2 },
    { 0x28c5ac62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea5ac6f, SUC_lexicon_tags_62 },
    { 0x7b85ac6f, SUC_lexicon_tags_7 },
    { 0x9e85ac70, SUC_lexicon_tags_37 },
    { 0xbb45ac72, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa45ac76, SUC_lexicon_tags_60 },
    { 0xd865ac76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8865ac7a, SUC_lexicon_tags_96 },
    { 0xba25ac7b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb85ac80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc5ac84, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9c25ac86, SUC_lexicon_tags_28 },
    { 0x6385ac87, SUC_lexicon_tags_63 },
    { 0x3565ac87, SUC_lexicon_tags_37 },
    { 0xa2a5ac88, SUC_lexicon_tags_14 },
    { 0x5ac8a, SUC_lexicon_tags_19 },
    { 0x1c65ac86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a5ac90, SUC_lexicon_tags_14 },
    { 0x105ac91, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x75a5ac93, SUC_lexicon_tags_11 },
    { 0x2245ac94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c5ac9b, SUC_lexicon_tags_14 },
    { 0xb425ac9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c5aca1, SUC_lexicon_tags_16 },
    { 0xbaa5aca1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac5aca6, SUC_lexicon_tags_12 },
    { 0xa645aca7, SUC_lexicon_tags_15 },
    { 0x8ce5aca7, SUC_lexicon_tags_1 },
    { 0x2cc5aca9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a45acae, SUC_lexicon_tags_17 },
    { 0xd305acaf, SUC_lexicon_tags_1 },
    { 0xbec5acae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe625acb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a5acbe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce5acc3, SUC_lexicon_tags_37 },
    { 0xa5e5acc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c5acc7, SUC_lexicon_tags_11 },
    { 0xdd05acc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f25accd, SUC_lexicon_tags_30 },
    { 0xb105acce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c5acd6, SUC_lexicon_tags_11 },
    { 0xa4a5acd7, SUC_lexicon_tags_47 },
    { 0x4ae5acd6, SUC_lexicon_tags_1 },
    { 0x9985acd8, SUC_lexicon_tags_17 },
    { 0x1585acda, SUC_lexicon_tags_40 },
    { 0x1065acd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5525acdf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd25ace1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac65acec, SUC_lexicon_tags_63 },
    { 0xfa25acec, SUC_lexicon_tags_37 },
    { 0x5645acee, SUC_lexicon_tags_14 },
    { 0xa5a5acee, SUC_lexicon_tags_37 },
    { 0x18e5acee, SUC_lexicon_tags_57 },
    { 0x9d05acf1, SUC_lexicon_tags_1 },
    { 0xfe5acef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc5acfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85acff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e5ad02, SUC_lexicon_tags_37 },
    { 0xe165ad03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e5ad08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbee5ad0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa305ad0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf25ad0e, SUC_lexicon_tags_37 },
    { 0x53e5ad0e, SUC_lexicon_tags_37 },
    { 0x73a5ad10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x24c5ad12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65ad17, SUC_lexicon_tags_13 },
    { 0xf765ad18, SUC_lexicon_tags_11 },
    { 0x55e5ad19, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5845ad1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84a5ad1f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe5a5ad21, SUC_lexicon_tags_1 },
    { 0xcd25ad22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x25a5ad24, SUC_lexicon_tags_37 },
    { 0x3885ad25, SUC_lexicon_tags_7 },
    { 0xce85ad26, SUC_lexicon_tags_137 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ad2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x965ad2f, SUC_lexicon_tags_27 },
    { 0x8f45ad30, SUC_lexicon_tags_14 },
    { 0x3f45ad31, SUC_lexicon_tags_15 },
    { 0xb2a5ad31, SUC_lexicon_tags_15 },
    { 0x9165ad33, SUC_lexicon_tags_0 },
    { 0x67c5ad34, SUC_lexicon_tags_16 },
    { 0xc385ad34, SUC_lexicon_tags_38 },
    { 0x67a5ad32, SUC_lexicon_tags_25 },
    { 0x6ba5ad32, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0, NULL },
    { 0x3585ad3a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4465ad44, SUC_lexicon_tags_31 },
    { 0x9fe5ad45, SUC_lexicon_tags_7 },
    { 0x5825ad46, SUC_lexicon_tags_0 },
    { 0x25ad46, SUC_lexicon_tags_6 },
    { 0x8ec5ad46, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc545ad4d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xab45ad50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4345ad52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6125ad55, SUC_lexicon_tags_25 },
    { 0x8425ad55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e65ad58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3025ad5b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be5ad60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e65ad64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc5ad67, SUC_lexicon_tags_37 },
    { 0xc785ad67, SUC_lexicon_tags_17 },
    { 0xb8e5ad69, SUC_lexicon_tags_2 },
    { 0x92c5ad6a, SUC_lexicon_tags_49 },
    { 0xc1a5ad6b, SUC_lexicon_tags_17 },
    { 0x45ad6a, SUC_lexicon_tags_33 },
    { 0x5265ad6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95a5ad6f, SUC_lexicon_tags_26 },
    { 0xcb65ad70, SUC_lexicon_tags_17 },
    { 0xc985ad71, SUC_lexicon_tags_37 },
    { 0x3085ad70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9645ad74, SUC_lexicon_tags_26 },
    { 0x685ad75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc5ad78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f05ad7a, SUC_lexicon_tags_15 },
    { 0xb625ad7b, SUC_lexicon_tags_0 },
    { 0x7445ad7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd25ad7e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x32c5ad80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e5ad82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e5ad86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e5ad8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d65ad92, SUC_lexicon_tags_66 },
    { 0x4865ad92, SUC_lexicon_tags_15 },
    { 0x845ad94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7285ad96, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf225ad99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5185ad9d, SUC_lexicon_tags_49 },
    { 0x1545ad9d, SUC_lexicon_tags_1 },
    { 0xdc65ad9f, SUC_lexicon_tags_12 },
    { 0xa485ada0, SUC_lexicon_tags_2 },
    { 0x6045ada1, SUC_lexicon_tags_47 },
    { 0x1de5ada2, SUC_lexicon_tags_11 },
    { 0x5285ada1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a5ada7, SUC_lexicon_tags_48 },
    { 0x3965ada7, SUC_lexicon_tags_2 },
    { 0x5a45ada9, SUC_lexicon_tags_40 },
    { 0xa385ada9, SUC_lexicon_tags_11 },
    { 0x8865adab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca5adae, SUC_lexicon_tags_12 },
    { 0xe825adaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e65adb1, SUC_lexicon_tags_47 },
    { 0xbce5adb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf65adb6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b65adb9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe065adc3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d85adc7, SUC_lexicon_tags_27 },
    { 0x5825adc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3265adcd, SUC_lexicon_tags_0 },
    { 0xdfa5adce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2545add0, SUC_lexicon_tags_25 },
    { 0xd3a5add1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xecc5add3, SUC_lexicon_tags_4 },
    { 0x2c85add3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae5add9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8f65addb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce5adde, SUC_lexicon_tags_31 },
    { 0x99a5addf, SUC_lexicon_tags_66 },
    { 0x2ce5addf, SUC_lexicon_tags_1 },
    { 0xa465ade1, SUC_lexicon_tags_63 },
    { 0xc845ade0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3145adea, SUC_lexicon_tags_74 },
    { 0x7805adeb, SUC_lexicon_tags_1 },
    { 0x5165adec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6c45adee, SUC_lexicon_tags_2 },
    { 0x8fc5adef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9085adf4, SUC_lexicon_tags_26 },
    { 0xa05adf5, SUC_lexicon_tags_11 },
    { 0xfb05adf4, SUC_lexicon_tags_19 },
    { 0x4605adf7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c5adfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8305ae00, SUC_lexicon_tags_17 },
    { 0x6025ae01, SUC_lexicon_tags_31 },
    { 0x1765ae02, SUC_lexicon_tags_23 },
    { 0x1665ae02, SUC_lexicon_tags_12 },
    { 0x3d25ae04, SUC_lexicon_tags_37 },
    { 0x18e5ae03, SUC_lexicon_tags_37 },
    { 0x4fe5ae06, SUC_lexicon_tags_48 },
    { 0x7d05ae07, SUC_lexicon_tags_39 },
    { 0x7fc5ae00, SUC_lexicon_tags_17 },
    { 0x8345ae06, SUC_lexicon_tags_39 },
    { 0x6d05ae0a, SUC_lexicon_tags_63 },
    { 0xe085ae0a, SUC_lexicon_tags_7 },
    { 0x2fe5ae0c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1305ae0e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x95a5ae10, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5ae15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86a5ae17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9865ae19, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6025ae1b, SUC_lexicon_tags_216 },
    { 0x9965ae1b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1b85ae1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa285ae22, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb945ae28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9065ae2d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6245ae2f, SUC_lexicon_tags_11 },
    { 0xb945ae2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa45ae35, SUC_lexicon_tags_37 },
    { 0xcc85ae35, SUC_lexicon_tags_17 },
    { 0x99e5ae37, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a5ae3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c65ae3f, SUC_lexicon_tags_81 },
    { 0xc85ae40, SUC_lexicon_tags_11 },
    { 0xe5ae3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8265ae45, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5505ae47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a65ae4b, SUC_lexicon_tags_14 },
    { 0x9ac5ae4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf365ae4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b45ae51, SUC_lexicon_tags_17 },
    { 0xd505ae52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1125ae55, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3065ae58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44a5ae5a, SUC_lexicon_tags_37 },
    { 0x8365ae5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf05ae5d, SUC_lexicon_tags_912 },
    { 0x8505ae5d, SUC_lexicon_tags_60 },
    { 0xb405ae5e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8065ae62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d85ae65, SUC_lexicon_tags_11 },
    { 0xaa25ae65, SUC_lexicon_tags_17 },
    { 0x1365ae65, SUC_lexicon_tags_0 },
    { 0xd1a5ae68, SUC_lexicon_tags_11 },
    { 0x1fa5ae65, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ae5ae6b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb045ae6e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xf0a5ae70, SUC_lexicon_tags_7 },
    { 0x1405ae70, SUC_lexicon_tags_31 },
    { 0x5525ae71, SUC_lexicon_tags_65 },
    { 0xfb85ae72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b05ae7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcf25ae7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8445ae7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff25ae81, SUC_lexicon_tags_30 },
    { 0xaea5ae82, SUC_lexicon_tags_25 },
    { 0x2ec5ae83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3a5ae85, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe5ae88, SUC_lexicon_tags_25 },
    { 0xd385ae89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4665ae8f, SUC_lexicon_tags_7 },
    { 0x4245ae90, SUC_lexicon_tags_65 },
    { 0x5585ae8f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf065ae95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec5ae9c, SUC_lexicon_tags_31 },
    { 0x7e5ae9d, SUC_lexicon_tags_1 },
    { 0x43c5ae9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc625aea1, SUC_lexicon_tags_23 },
    { 0xc045aea1, SUC_lexicon_tags_17 },
    { 0x3de5aea1, SUC_lexicon_tags_17 },
    { 0x13e5aea2, SUC_lexicon_tags_1 },
    { 0xe965aea3, SUC_lexicon_tags_77 },
    { 0xec65aea6, SUC_lexicon_tags_16 },
    { 0x81c5aea7, SUC_lexicon_tags_37 },
    { 0xaf25aea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd85aeab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c5aeae, SUC_lexicon_tags_47 },
    { 0xf3c5aeaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c5aeb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb505aeb9, SUC_lexicon_tags_15 },
    { 0x45c5aeb9, SUC_lexicon_tags_0 },
    { 0x3c45aeb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7e5aebd, SUC_lexicon_tags_33 },
    { 0x86c5aebd, SUC_lexicon_tags_12 },
    { 0xcbc5aebe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e85aec2, SUC_lexicon_tags_87 },
    { 0x3245aec3, SUC_lexicon_tags_25 },
    { 0x3d45aec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf665aec6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b65aece, SUC_lexicon_tags_25 },
    { 0x7245aecf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a5aed2, SUC_lexicon_tags_18 },
    { 0x9a45aed3, SUC_lexicon_tags_8 },
    { 0x5bc5aed4, SUC_lexicon_tags_17 },
    { 0x9345aed5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce5aed9, SUC_lexicon_tags_30 },
    { 0x4b25aeda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c85aee3, SUC_lexicon_tags_30 },
    { 0x3c85aee4, SUC_lexicon_tags_11 },
    { 0x4c45aee3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c5aeea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc5aeef, SUC_lexicon_tags_2 },
    { 0xa9e5aef0, SUC_lexicon_tags_1 },
    { 0x1905aef1, SUC_lexicon_tags_17 },
    { 0x1b85aef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75c5aef4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa705aef6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xafa5aef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e5aefc, SUC_lexicon_tags_53 },
    { 0x9e05aefd, SUC_lexicon_tags_63 },
    { 0x6a45aefe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee5af01, SUC_lexicon_tags_37 },
    { 0xfec5af02, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f25af08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a5af0c, SUC_lexicon_tags_182 },
    { 0xd185af0c, SUC_lexicon_tags_10 },
    { 0xaf85af0c, SUC_lexicon_tags_43 },
    { 0xf8c5af0d, SUC_lexicon_tags_4 },
    { 0x865af10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a5af13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb865af16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5505af18, SUC_lexicon_tags_49 },
    { 0x40c5af19, SUC_lexicon_tags_45 },
    { 0x5c05af1a, SUC_lexicon_tags_17 },
    { 0xfe85af1b, SUC_lexicon_tags_6 },
    { 0x7165af1c, SUC_lexicon_tags_28 },
    { 0x99a5af1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd145af20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc405af22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5af26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc765af29, SUC_lexicon_tags_15 },
    { 0x62a5af29, SUC_lexicon_tags_26 },
    { 0x72c5af2b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a5af30, SUC_lexicon_tags_25 },
    { 0xe8e5af30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6805af33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1265af36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26a5af38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1505af3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65af3d, SUC_lexicon_tags_37 },
    { 0x6545af3e, SUC_lexicon_tags_37 },
    { 0x2e05af3e, SUC_lexicon_tags_63 },
    { 0x8d25af3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf45af44, SUC_lexicon_tags_19 },
    { 0xab25af45, SUC_lexicon_tags_37 },
    { 0x5865af45, SUC_lexicon_tags_55 },
    { 0xcfc5af46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9425af4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e5af53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8145af57, SUC_lexicon_tags_30 },
    { 0x9745af58, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2645af5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa1a5af5e, SUC_lexicon_tags_11 },
    { 0xede5af5f, SUC_lexicon_tags_1 },
    { 0x8d25af5f, SUC_lexicon_tags_37 },
    { 0xa065af60, SUC_lexicon_tags_15 },
    { 0x7125af62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d25af66, SUC_lexicon_tags_0 },
    { 0xd185af66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa905af69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee05af6b, SUC_lexicon_tags_8 },
    { 0x45e5af6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb505af6e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9565af71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3125af78, SUC_lexicon_tags_0 },
    { 0x9245af78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee5af7e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c5af82, SUC_lexicon_tags_37 },
    { 0x1245af82, SUC_lexicon_tags_17 },
    { 0x3dc5af84, SUC_lexicon_tags_27 },
    { 0xcc45af85, SUC_lexicon_tags_45 },
    { 0xb905af84, SUC_lexicon_tags_7 },
    { 0xb345af85, SUC_lexicon_tags_17 },
    { 0x7ea5af88, SUC_lexicon_tags_84 },
    { 0xe7a5af88, SUC_lexicon_tags_17 },
    { 0xf805af89, SUC_lexicon_tags_37 },
    { 0x55e5af89, SUC_lexicon_tags_39 },
    { 0xbb45af8c, SUC_lexicon_tags_33 },
    { 0x4fc5af8d, SUC_lexicon_tags_45 },
    { 0x66a5af89, SUC_lexicon_tags_38 },
    { 0x7cc5af8f, SUC_lexicon_tags_23 },
    { 0xbfe5af8b, SUC_lexicon_tags_47 },
    { 0x1005af8a, SUC_lexicon_tags_25 },
    { 0x6ce5af92, SUC_lexicon_tags_47 },
    { 0xd585af93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd05af96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe85af9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e5afa0, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0, NULL },
    { 0x9505afa3, SUC_lexicon_tags_7 },
    { 0x4905afa4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc5afa7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c5afb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc305afb2, SUC_lexicon_tags_28 },
    { 0x5afb2, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xb605afb5, SUC_lexicon_tags_47 },
    { 0x14c5afb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9245afb8, SUC_lexicon_tags_0 },
    { 0x24e5afb9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e5afbc, SUC_lexicon_tags_1043 },
    { 0xfc65afbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c05afbf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6405afc2, SUC_lexicon_tags_2 },
    { 0x50a5afc3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa25afc6, SUC_lexicon_tags_17 },
    { 0x68e5afc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7425afc9, SUC_lexicon_tags_15 },
    { 0x89c5afca, SUC_lexicon_tags_60 },
    { 0x55a5afca, SUC_lexicon_tags_60 },
    { 0x3645afc9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4945afcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5afd2, SUC_lexicon_tags_39 },
    { 0xc005afd2, SUC_lexicon_tags_45 },
    { 0xf4e5afd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e85afda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65afe0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2a5afe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f65afe7, SUC_lexicon_tags_2 },
    { 0xa4e5afe8, SUC_lexicon_tags_56 },
    { 0x90c5afe7, SUC_lexicon_tags_12 },
    { 0xfd25afea, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c5afee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2005aff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe805aff7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c5affc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4aa5affe, SUC_lexicon_tags_68 },
    { 0xd365afff, SUC_lexicon_tags_12 },
    { 0xc8a5afff, SUC_lexicon_tags_19 },
    { 0x8265afff, SUC_lexicon_tags_25 },
    { 0x18a5b002, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff85b004, SUC_lexicon_tags_2 },
    { 0x7965b005, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2745b00c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb325b00f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a85b011, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5465b013, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1945b018, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7c5b01a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x745b01c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae5b022, SUC_lexicon_tags_11 },
    { 0xf165b022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae5b027, SUC_lexicon_tags_6 },
    { 0x72c5b028, SUC_lexicon_tags_167 },
    { 0x9745b029, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a5b02e, SUC_lexicon_tags_39 },
    { 0x2185b02f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8045b032, SUC_lexicon_tags_0 },
    { 0xcfc5b032, SUC_lexicon_tags_17 },
    { 0xc465b034, SUC_lexicon_tags_92 },
    { 0x29a5b033, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05b03c, SUC_lexicon_tags_37 },
    { 0x4265b03d, SUC_lexicon_tags_152 },
    { 0x4985b03d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf625b049, SUC_lexicon_tags_30 },
    { 0x3285b04a, SUC_lexicon_tags_15 },
    { 0xb705b04b, SUC_lexicon_tags_62 },
    { 0x7765b04b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa165b04f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd125b053, SUC_lexicon_tags_37 },
    { 0x57e5b053, SUC_lexicon_tags_49 },
    { 0x6e85b053, SUC_lexicon_tags_31 },
    { 0x9b65b056, SUC_lexicon_tags_17 },
    { 0x5b65b055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa545b059, SUC_lexicon_tags_60 },
    { 0xc1e5b059, SUC_lexicon_tags_25 },
    { 0xb305b05a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee5b05f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc5b062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15c5b064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed65b066, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e5b06c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b05b075, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb725b077, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f45b07a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5785b07d, SUC_lexicon_tags_0 },
    { 0xba05b07e, SUC_lexicon_tags_17 },
    { 0xd0e5b07d, SUC_lexicon_tags_23 },
    { 0x2a65b080, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa905b08b, SUC_lexicon_tags_87 },
    { 0x7fe5b08c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef05b08f, SUC_lexicon_tags_37 },
    { 0x8d45b090, SUC_lexicon_tags_11 },
    { 0xae85b091, SUC_lexicon_tags_38 },
    { 0xfc25b092, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44a5b094, SUC_lexicon_tags_23 },
    { 0x5765b095, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4605b09b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe5b09e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xae45b0a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac5b0a3, SUC_lexicon_tags_1 },
    { 0xc1e5b0a4, SUC_lexicon_tags_23 },
    { 0x8705b0a5, SUC_lexicon_tags_11 },
    { 0xd085b0a6, SUC_lexicon_tags_148 },
    { 0x79a5b0a3, SUC_lexicon_tags_17 },
    { 0x2e85b0a5, SUC_lexicon_tags_37 },
    { 0x7845b0a9, SUC_lexicon_tags_65 },
    { 0x9d65b0a6, SUC_lexicon_tags_11 },
    { 0x7765b0ab, SUC_lexicon_tags_14 },
    { 0xa325b0ab, SUC_lexicon_tags_60 },
    { 0xa9e5b0ac, SUC_lexicon_tags_11 },
    { 0x9745b0ad, SUC_lexicon_tags_10 },
    { 0xb4e5b0a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45b0b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd885b0b6, SUC_lexicon_tags_37 },
    { 0xbf05b0b6, SUC_lexicon_tags_67 },
    { 0xf565b0b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf065b0ba, SUC_lexicon_tags_28 },
    { 0xe5a5b0ba, SUC_lexicon_tags_47 },
    { 0x3725b0bb, SUC_lexicon_tags_23 },
    { 0xdbe5b0bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x38c5b0bf, SUC_lexicon_tags_15 },
    { 0xd005b0bf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05b0c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c45b0c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7445b0ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb65b0ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9165b0d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6685b0d8, SUC_lexicon_tags_23 },
    { 0x8345b0d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1605b0dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa305b0e0, SUC_lexicon_tags_11 },
    { 0xe945b0e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3505b0e5, SUC_lexicon_tags_39 },
    { 0xef45b0e5, SUC_lexicon_tags_631 },
    { 0x1ee5b0e7, SUC_lexicon_tags_12 },
    { 0xa705b0e8, SUC_lexicon_tags_8 },
    { 0x98e5b0e9, SUC_lexicon_tags_60 },
    { 0x8d85b0ea, SUC_lexicon_tags_62 },
    { 0x5a5b0eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba65b0ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf985b0f9, SUC_lexicon_tags_27 },
    { 0x9a25b0fa, SUC_lexicon_tags_48 },
    { 0x2045b0fb, SUC_lexicon_tags_53 },
    { 0x1f25b0fc, SUC_lexicon_tags_48 },
    { 0xe485b0fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc8c5b0ff, SUC_lexicon_tags_31 },
    { 0x5145b100, SUC_lexicon_tags_37 },
    { 0x32a5b101, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2465b103, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc5b10a, SUC_lexicon_tags_61 },
    { 0xd145b10a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb685b10f, SUC_lexicon_tags_47 },
    { 0x4d85b110, SUC_lexicon_tags_84 },
    { 0xeac5b111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3645b116, SUC_lexicon_tags_61 },
    { 0x36e5b117, SUC_lexicon_tags_38 },
    { 0x3385b118, SUC_lexicon_tags_23 },
    { 0xc2e5b118, SUC_lexicon_tags_39 },
    { 0x3e05b117, SUC_lexicon_tags_4 },
    { 0x9945b11b, SUC_lexicon_tags_37 },
    { 0x65c5b119, SUC_lexicon_tags_19 },
    { 0x2225b11d, SUC_lexicon_tags_12 },
    { 0x5045b11e, SUC_lexicon_tags_16 },
    { 0x4ea5b11f, SUC_lexicon_tags_207 },
    { 0xdb25b120, SUC_lexicon_tags_17 },
    { 0xcce5b120, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ea5b123, SUC_lexicon_tags_31 },
    { 0x7d25b124, SUC_lexicon_tags_38 },
    { 0xf7c5b123, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8465b12a, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x9305b12c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a5b130, SUC_lexicon_tags_37 },
    { 0x9065b131, SUC_lexicon_tags_43 },
    { 0x7845b132, SUC_lexicon_tags_11 },
    { 0x8505b132, SUC_lexicon_tags_19 },
    { 0x7505b133, SUC_lexicon_tags_33 },
    { 0x1705b135, SUC_lexicon_tags_38 },
    { 0xcda5b131, SUC_lexicon_tags_49 },
    { 0x9865b137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4265b13a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4405b13c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd7c5b13e, SUC_lexicon_tags_104 },
    { 0xb9e5b13f, SUC_lexicon_tags_25 },
    { 0xb045b13f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5b144, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x83a5b146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17c5b148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec5b14c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ac5b14e, SUC_lexicon_tags_1 },
    { 0x4f65b14f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7cc5b151, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7325b153, SUC_lexicon_tags_295 },
    { 0xd6c5b154, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e5b159, SUC_lexicon_tags_33 },
    { 0xfd25b159, SUC_lexicon_tags_8 },
    { 0x905b15b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3285b164, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd885b16a, SUC_lexicon_tags_25 },
    { 0xe5a5b16b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb1c5b16d, SUC_lexicon_tags_9 },
    { 0x7c5b16e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa5b171, SUC_lexicon_tags_7 },
    { 0x3e45b172, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de5b176, SUC_lexicon_tags_15 },
    { 0xe2e5b177, SUC_lexicon_tags_15 },
    { 0x6125b177, SUC_lexicon_tags_17 },
    { 0x9225b176, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c5b17f, SUC_lexicon_tags_11 },
    { 0x89e5b180, SUC_lexicon_tags_60 },
    { 0xca85b180, SUC_lexicon_tags_39 },
    { 0x3105b180, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa5b18a, SUC_lexicon_tags_43 },
    { 0x19c5b18b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e5b194, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa5b196, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2425b199, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x625b19b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a5b19f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4305b1a1, SUC_lexicon_tags_11 },
    { 0xde5b1a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc225b1a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc5b1a9, SUC_lexicon_tags_19 },
    { 0x9825b1aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe5b1ae, SUC_lexicon_tags_2 },
    { 0xc425b1af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8425b1b2, SUC_lexicon_tags_4 },
    { 0x31c5b1b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6405b1b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5b1ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6745b1bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3485b1c0, SUC_lexicon_tags_112 },
    { 0xa525b1c1, SUC_lexicon_tags_48 },
    { 0x8b85b1c1, SUC_lexicon_tags_8 },
    { 0xd485b1c3, SUC_lexicon_tags_27 },
    { 0x6ba5b1c4, SUC_lexicon_tags_17 },
    { 0x6545b1c4, SUC_lexicon_tags_88 },
    { 0x7fc5b1c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe945b1c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3005b1cb, SUC_lexicon_tags_37 },
    { 0x7305b1cc, SUC_lexicon_tags_1 },
    { 0xc945b1cd, SUC_lexicon_tags_2 },
    { 0x445b1cd, SUC_lexicon_tags_21 },
    { 0xf5e5b1cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab25b1d3, SUC_lexicon_tags_27 },
    { 0x9e85b1d3, SUC_lexicon_tags_1 },
    { 0x6ba5b1d3, SUC_lexicon_tags_2 },
    { 0x8445b1d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8325b1db, SUC_lexicon_tags_15 },
    { 0x6d05b1dc, SUC_lexicon_tags_10 },
    { 0xa4e5b1dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a85b1e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf05b1e6, SUC_lexicon_tags_46 },
    { 0x5c5b1e7, SUC_lexicon_tags_27 },
    { 0xbb25b1e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a5b1ee, SUC_lexicon_tags_87 },
    { 0x9d05b1ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb425b1f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e05b1f5, SUC_lexicon_tags_37 },
    { 0x4765b1f6, SUC_lexicon_tags_14 },
    { 0x7725b1f6, SUC_lexicon_tags_2 },
    { 0x12c5b1f8, SUC_lexicon_tags_30 },
    { 0x8025b1f9, SUC_lexicon_tags_26 },
    { 0xa785b1fa, SUC_lexicon_tags_6 },
    { 0xe985b1fb, SUC_lexicon_tags_92 },
    { 0x8785b1fc, SUC_lexicon_tags_11 },
    { 0x3345b1fd, SUC_lexicon_tags_8 },
    { 0x9305b1fe, SUC_lexicon_tags_17 },
    { 0xa9c5b1fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da5b203, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc985b207, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4f85b209, SUC_lexicon_tags_25 },
    { 0x1025b209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23a5b20c, SUC_lexicon_tags_2 },
    { 0x925b20d, SUC_lexicon_tags_25 },
    { 0x8145b20d, SUC_lexicon_tags_27 },
    { 0x8605b20d, SUC_lexicon_tags_11 },
    { 0x6465b20c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e5b216, SUC_lexicon_tags_37 },
    { 0x83a5b217, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a5b225, SUC_lexicon_tags_38 },
    { 0xa705b226, SUC_lexicon_tags_7 },
    { 0x65c5b227, SUC_lexicon_tags_40 },
    { 0xb4a5b226, SUC_lexicon_tags_1 },
    { 0xb1a5b229, SUC_lexicon_tags_37 },
    { 0x50c5b22a, SUC_lexicon_tags_60 },
    { 0xade5b229, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a25b22e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc585b231, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad65b23f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe905b247, SUC_lexicon_tags_7 },
    { 0x79e5b248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade5b252, SUC_lexicon_tags_610 },
    { 0x31a5b253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc45b259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec5b25d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa545b25f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf985b262, SUC_lexicon_tags_1185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a5b267, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e5b26f, SUC_lexicon_tags_17 },
    { 0x2145b270, SUC_lexicon_tags_2 },
    { 0xb65b271, SUC_lexicon_tags_12 },
    { 0x53a5b272, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f65b27b, SUC_lexicon_tags_12 },
    { 0x4865b27b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xb865b27f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17a5b281, SUC_lexicon_tags_37 },
    { 0xeea5b282, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc765b284, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5b287, SUC_lexicon_tags_17 },
    { 0x86c5b287, SUC_lexicon_tags_28 },
    { 0xcc25b289, SUC_lexicon_tags_37 },
    { 0x61c5b287, SUC_lexicon_tags_66 },
    { 0x6c45b28b, SUC_lexicon_tags_45 },
    { 0x7be5b28b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc45b290, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd785b293, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8445b295, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c25b297, SUC_lexicon_tags_1273 },
    { 0x6965b298, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3385b29f, SUC_lexicon_tags_11 },
    { 0x8365b2a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd025b2a7, SUC_lexicon_tags_26 },
    { 0xa065b2a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d85b2b0, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a5b2b8, SUC_lexicon_tags_37 },
    { 0x245b2b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9265b2bd, SUC_lexicon_tags_7 },
    { 0x65c5b2be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585b2c3, SUC_lexicon_tags_47 },
    { 0x8a25b2c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b85b2c8, SUC_lexicon_tags_37 },
    { 0x8905b2c9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde65b2cd, SUC_lexicon_tags_38 },
    { 0x18e5b2cd, SUC_lexicon_tags_61 },
    { 0x7745b2cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18c5b2d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd605b2d7, SUC_lexicon_tags_12 },
    { 0xc085b2d8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3485b2dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d45b2df, SUC_lexicon_tags_37 },
    { 0x9685b2e0, SUC_lexicon_tags_15 },
    { 0x19c5b2e1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5645b2f2, SUC_lexicon_tags_18 },
    { 0xe2c5b2f2, SUC_lexicon_tags_2 },
    { 0x9105b2f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e5b300, SUC_lexicon_tags_47 },
    { 0x92a5b300, SUC_lexicon_tags_1 },
    { 0x4305b300, SUC_lexicon_tags_45 },
    { 0x21e5b303, SUC_lexicon_tags_11 },
    { 0xda05b303, SUC_lexicon_tags_10 },
    { 0xfa45b305, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc5b308, SUC_lexicon_tags_11 },
    { 0x4925b308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe45b30c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad05b310, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd565b316, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5b31d, SUC_lexicon_tags_5 },
    { 0x7ac5b31d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x465b321, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c05b331, SUC_lexicon_tags_37 },
    { 0x4ca5b332, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a05b334, SUC_lexicon_tags_17 },
    { 0xc025b335, SUC_lexicon_tags_28 },
    { 0xad65b336, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8f25b338, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0a5b33a, SUC_lexicon_tags_63 },
    { 0xc785b33b, SUC_lexicon_tags_37 },
    { 0xdc25b33c, SUC_lexicon_tags_8 },
    { 0x33a5b33b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8585b341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c5b344, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c5b34c, SUC_lexicon_tags_25 },
    { 0xf2c5b34c, SUC_lexicon_tags_55 },
    { 0x9fc5b34d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd725b352, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a5b357, SUC_lexicon_tags_45 },
    { 0x6905b357, SUC_lexicon_tags_7 },
    { 0x2d25b359, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda45b35e, SUC_lexicon_tags_12 },
    { 0x21e5b35f, SUC_lexicon_tags_30 },
    { 0xb885b360, SUC_lexicon_tags_19 },
    { 0xc185b361, SUC_lexicon_tags_37 },
    { 0xc585b362, SUC_lexicon_tags_15 },
    { 0xd405b362, SUC_lexicon_tags_19 },
    { 0x9545b361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec05b366, SUC_lexicon_tags_23 },
    { 0xbce5b366, SUC_lexicon_tags_9 },
    { 0xe565b368, SUC_lexicon_tags_33 },
    { 0xf145b369, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6685b36b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4c5b36d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b372, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d85b378, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec5b37d, SUC_lexicon_tags_11 },
    { 0xc2c5b37e, SUC_lexicon_tags_29 },
    { 0x85b37f, SUC_lexicon_tags_23 },
    { 0x72c5b380, SUC_lexicon_tags_37 },
    { 0xd8a5b381, SUC_lexicon_tags_14 },
    { 0x1d25b381, SUC_lexicon_tags_6 },
    { 0x76e5b381, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xca85b385, SUC_lexicon_tags_68 },
    { 0x9ca5b386, SUC_lexicon_tags_6 },
    { 0x5105b387, SUC_lexicon_tags_32 },
    { 0x5da5b387, SUC_lexicon_tags_34 },
    { 0xb85b389, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb465b38b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb3c5b38d, SUC_lexicon_tags_23 },
    { 0xfce5b38e, SUC_lexicon_tags_9 },
    { 0xd8a5b38e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2165b394, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa185b39a, SUC_lexicon_tags_68 },
    { 0xa245b39a, SUC_lexicon_tags_23 },
    { 0xc85b39c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca5b3a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c5b3a7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e5b3ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb225b3ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7565b3b1, SUC_lexicon_tags_15 },
    { 0xca85b3b1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2f05b3b4, SUC_lexicon_tags_1 },
    { 0xbb25b3b5, SUC_lexicon_tags_16 },
    { 0x6ea5b3b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2b65b3b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e65b3ba, SUC_lexicon_tags_14 },
    { 0x5f05b3bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6265b3c0, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xee25b3c3, SUC_lexicon_tags_15 },
    { 0xf85b3c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8745b3c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc125b3cc, SUC_lexicon_tags_35 },
    { 0xfa25b3cd, SUC_lexicon_tags_96 },
    { 0xbc5b3ce, SUC_lexicon_tags_14 },
    { 0x5a05b3cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e5b3d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac5b3d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9425b3d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa725b3e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a5b3e4, SUC_lexicon_tags_1 },
    { 0xba5b3e5, SUC_lexicon_tags_38 },
    { 0xf745b3e5, SUC_lexicon_tags_17 },
    { 0x74a5b3e7, SUC_lexicon_tags_8 },
    { 0xfdc5b3e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe345b3ea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be5b3ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e5b3f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c5b3f4, SUC_lexicon_tags_37 },
    { 0xeea5b3f5, SUC_lexicon_tags_7 },
    { 0x31c5b3f5, SUC_lexicon_tags_17 },
    { 0xc8e5b3f7, SUC_lexicon_tags_19 },
    { 0xeb25b3f7, SUC_lexicon_tags_80 },
    { 0xe9c5b3f8, SUC_lexicon_tags_17 },
    { 0xca5b3fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe665b3fc, SUC_lexicon_tags_27 },
    { 0x7ac5b3fd, SUC_lexicon_tags_388 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c5b404, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb365b407, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa425b40b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f85b40e, SUC_lexicon_tags_12 },
    { 0xcac5b40f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd045b411, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9785b416, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4885b418, SUC_lexicon_tags_37 },
    { 0xfba5b419, SUC_lexicon_tags_23 },
    { 0xd6a5b419, SUC_lexicon_tags_25 },
    { 0x9585b419, SUC_lexicon_tags_27 },
    { 0x8aa5b41b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc5b420, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5dc5b422, SUC_lexicon_tags_33 },
    { 0xe025b423, SUC_lexicon_tags_0 },
    { 0x5cc5b424, SUC_lexicon_tags_0 },
    { 0x2a5b425, SUC_lexicon_tags_63 },
    { 0x8b25b425, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbe05b428, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd245b42a, SUC_lexicon_tags_49 },
    { 0xa405b42b, SUC_lexicon_tags_47 },
    { 0x8e65b42c, SUC_lexicon_tags_35 },
    { 0x7f05b42d, SUC_lexicon_tags_17 },
    { 0x7865b42a, SUC_lexicon_tags_28 },
    { 0xf245b42f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x645b431, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b05b436, SUC_lexicon_tags_14 },
    { 0xa0c5b437, SUC_lexicon_tags_23 },
    { 0x7685b436, SUC_lexicon_tags_2 },
    { 0xa225b436, SUC_lexicon_tags_37 },
    { 0xf705b43a, SUC_lexicon_tags_47 },
    { 0x40c5b43a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd725b43f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a5b443, SUC_lexicon_tags_23 },
    { 0x4c65b443, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe905b448, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa65b44e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05b454, SUC_lexicon_tags_63 },
    { 0xf885b454, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3625b457, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x32c5b459, SUC_lexicon_tags_17 },
    { 0x4a65b45a, SUC_lexicon_tags_5 },
    { 0xf165b45a, SUC_lexicon_tags_174 },
    { 0x1085b45a, SUC_lexicon_tags_30 },
    { 0xbc5b45d, SUC_lexicon_tags_25 },
    { 0x95e5b459, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed65b460, SUC_lexicon_tags_16 },
    { 0x1a65b461, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a5b465, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9665b46a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4625b46c, SUC_lexicon_tags_29 },
    { 0x8385b46d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c25b475, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x465b477, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa645b479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd725b47b, SUC_lexicon_tags_1 },
    { 0x9fe5b47c, SUC_lexicon_tags_25 },
    { 0x90c5b47d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a5b47f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c85b482, SUC_lexicon_tags_16 },
    { 0x3e25b483, SUC_lexicon_tags_33 },
    { 0x6a25b484, SUC_lexicon_tags_15 },
    { 0xb6a5b485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3285b488, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xac05b48a, SUC_lexicon_tags_12 },
    { 0xdd85b48b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8645b491, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a5b495, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe5b49d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe065b49f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a5b4a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc825b4a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d25b4a7, SUC_lexicon_tags_27 },
    { 0x4305b4a8, SUC_lexicon_tags_2 },
    { 0x91c5b4a8, SUC_lexicon_tags_38 },
    { 0xc585b4aa, SUC_lexicon_tags_25 },
    { 0x2ea5b4ab, SUC_lexicon_tags_12 },
    { 0x2e45b4ac, SUC_lexicon_tags_9 },
    { 0x3765b4ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x39e5b4af, SUC_lexicon_tags_19 },
    { 0xdfc5b4af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3665b4b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e45b4b5, SUC_lexicon_tags_25 },
    { 0x9845b4b5, SUC_lexicon_tags_47 },
    { 0x7ec5b4b6, SUC_lexicon_tags_43 },
    { 0x5c65b4b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76a5b4ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d25b4bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74a5b4be, SUC_lexicon_tags_0 },
    { 0xc785b4bf, SUC_lexicon_tags_37 },
    { 0xb045b4c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe225b4c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74a5b4c4, SUC_lexicon_tags_38 },
    { 0x2d85b4c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b65b4c9, SUC_lexicon_tags_1 },
    { 0x9ae5b4ca, SUC_lexicon_tags_15 },
    { 0xbd05b4cb, SUC_lexicon_tags_3 },
    { 0xb1e5b4ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda5b4ce, SUC_lexicon_tags_11 },
    { 0x7645b4cf, SUC_lexicon_tags_7 },
    { 0xfba5b4cf, SUC_lexicon_tags_79 },
    { 0x9b25b4ce, SUC_lexicon_tags_87 },
    { 0x3c85b4d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1185b4d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c05b4d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc525b4db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e85b4de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4465b4e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa85b4e3, SUC_lexicon_tags_17 },
    { 0xe145b4e4, SUC_lexicon_tags_15 },
    { 0xb0c5b4e5, SUC_lexicon_tags_15 },
    { 0x3365b4e6, SUC_lexicon_tags_2 },
    { 0xa545b4e7, SUC_lexicon_tags_11 },
    { 0x1505b4e7, SUC_lexicon_tags_37 },
    { 0xec65b4e8, SUC_lexicon_tags_26 },
    { 0x8405b4e9, SUC_lexicon_tags_7 },
    { 0x1805b4ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2225b4ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a5b4f0, SUC_lexicon_tags_1 },
    { 0x3685b4f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1265b4f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e65b4f5, SUC_lexicon_tags_114 },
    { 0x5e25b4f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c5b4fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb185b501, SUC_lexicon_tags_12 },
    { 0x9065b501, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf045b509, SUC_lexicon_tags_27 },
    { 0x7ac5b50a, SUC_lexicon_tags_11 },
    { 0xb6e5b50b, SUC_lexicon_tags_25 },
    { 0xeb05b50a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9425b50e, SUC_lexicon_tags_15 },
    { 0x76c5b50f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e45b513, SUC_lexicon_tags_25 },
    { 0x5745b514, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc5b517, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8825b519, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6905b51c, SUC_lexicon_tags_17 },
    { 0xf045b51d, SUC_lexicon_tags_12 },
    { 0xbbc5b51e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea65b525, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8ac5b527, SUC_lexicon_tags_14 },
    { 0x6225b527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e5b52b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa445b531, SUC_lexicon_tags_7 },
    { 0x8905b531, SUC_lexicon_tags_84 },
    { 0xf345b532, SUC_lexicon_tags_9 },
    { 0x3fa5b534, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xd985b536, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x965b538, SUC_lexicon_tags_8 },
    { 0x7405b538, SUC_lexicon_tags_37 },
    { 0xcd65b53a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8485b53e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8f65b540, SUC_lexicon_tags_11 },
    { 0xcc45b541, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e05b545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e5b550, SUC_lexicon_tags_21 },
    { 0x2525b551, SUC_lexicon_tags_1 },
    { 0x4e5b552, SUC_lexicon_tags_12 },
    { 0xe6a5b552, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc245b557, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x80a5b559, SUC_lexicon_tags_31 },
    { 0x2de5b559, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4545b560, SUC_lexicon_tags_2 },
    { 0x8165b561, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe25b563, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe5b565, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc065b56b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a5b56e, SUC_lexicon_tags_1 },
    { 0xe545b56e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb25b572, SUC_lexicon_tags_65 },
    { 0xd905b573, SUC_lexicon_tags_68 },
    { 0x3fa5b574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5585b577, SUC_lexicon_tags_63 },
    { 0x1305b578, SUC_lexicon_tags_7 },
    { 0x3bc5b579, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5565b57b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4225b57d, SUC_lexicon_tags_15 },
    { 0x9e85b57e, SUC_lexicon_tags_37 },
    { 0x55c5b57f, SUC_lexicon_tags_66 },
    { 0x29e5b580, SUC_lexicon_tags_25 },
    { 0x7625b57e, SUC_lexicon_tags_33 },
    { 0x4285b57f, SUC_lexicon_tags_27 },
    { 0x4905b583, SUC_lexicon_tags_25 },
    { 0x8045b584, SUC_lexicon_tags_33 },
    { 0xf705b57f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e05b588, SUC_lexicon_tags_2 },
    { 0x4145b589, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7325b58b, SUC_lexicon_tags_65 },
    { 0x8c85b58c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb545b58f, SUC_lexicon_tags_15 },
    { 0xf765b58f, SUC_lexicon_tags_29 },
    { 0xca25b591, SUC_lexicon_tags_8 },
    { 0x6a5b592, SUC_lexicon_tags_2 },
    { 0xbd25b592, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8865b596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5b59b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc5b5a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5425b5a5, SUC_lexicon_tags_10 },
    { 0x5f65b5a6, SUC_lexicon_tags_15 },
    { 0x1085b5a6, SUC_lexicon_tags_38 },
    { 0xb65b5a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb665b5ab, SUC_lexicon_tags_11 },
    { 0xbfa5b5ac, SUC_lexicon_tags_25 },
    { 0x6565b5ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa645b5af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa945b5bc, SUC_lexicon_tags_43 },
    { 0x42c5b5bc, SUC_lexicon_tags_4 },
    { 0x1725b5be, SUC_lexicon_tags_17 },
    { 0xbfe5b5bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5ec5b5c1, SUC_lexicon_tags_12 },
    { 0x8c5b5c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9485b5c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9065b5cd, SUC_lexicon_tags_293 },
    { 0x2bc5b5cd, SUC_lexicon_tags_17 },
    { 0x8e25b5cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4085b5d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbac5b5d3, SUC_lexicon_tags_37 },
    { 0x6705b5d3, SUC_lexicon_tags_63 },
    { 0x1065b5d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8e5b5d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1625b5da, SUC_lexicon_tags_40 },
    { 0xf465b5db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3645b5dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2505b5df, SUC_lexicon_tags_15 },
    { 0x2645b5e0, SUC_lexicon_tags_12 },
    { 0x785b5df, SUC_lexicon_tags_70 },
    { 0xa145b5e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c45b5e6, SUC_lexicon_tags_38 },
    { 0x8285b5e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa65b5ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a5b5ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa105b5f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea5b5f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce5b5fc, SUC_lexicon_tags_1 },
    { 0x53a5b5fc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f45b600, SUC_lexicon_tags_7 },
    { 0xc505b600, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5005b603, SUC_lexicon_tags_19 },
    { 0xfcc5b603, SUC_lexicon_tags_15 },
    { 0xf225b603, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x58e5b607, SUC_lexicon_tags_1 },
    { 0x3ea5b608, SUC_lexicon_tags_14 },
    { 0x85a5b609, SUC_lexicon_tags_15 },
    { 0x2245b60a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa405b614, SUC_lexicon_tags_7 },
    { 0x3de5b614, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf425b617, SUC_lexicon_tags_218 },
    { 0xcb85b618, SUC_lexicon_tags_26 },
    { 0xf785b619, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ba5b61b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e5b622, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb225b625, SUC_lexicon_tags_2 },
    { 0x7565b626, SUC_lexicon_tags_25 },
    { 0x8c65b627, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2845b62b, SUC_lexicon_tags_12 },
    { 0x15a5b62b, SUC_lexicon_tags_140 },
    { 0xd5c5b62c, SUC_lexicon_tags_25 },
    { 0x4485b62e, SUC_lexicon_tags_68 },
    { 0x5da5b62f, SUC_lexicon_tags_17 },
    { 0x3165b62c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9c5b632, SUC_lexicon_tags_11 },
    { 0x8285b633, SUC_lexicon_tags_26 },
    { 0xb145b634, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8665b638, SUC_lexicon_tags_1 },
    { 0xbaa5b639, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa05b640, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9845b642, SUC_lexicon_tags_282 },
    { 0x3c5b643, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd705b647, SUC_lexicon_tags_11 },
    { 0x1625b648, SUC_lexicon_tags_9 },
    { 0x185b648, SUC_lexicon_tags_8 },
    { 0xa185b648, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf025b64c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda5b651, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc5b656, SUC_lexicon_tags_14 },
    { 0xa3c5b657, SUC_lexicon_tags_1 },
    { 0x68e5b656, SUC_lexicon_tags_27 },
    { 0xfce5b659, SUC_lexicon_tags_37 },
    { 0x7ec5b659, SUC_lexicon_tags_55 },
    { 0x505b65b, SUC_lexicon_tags_25 },
    { 0xd45b65c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6a85b65e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1605b664, SUC_lexicon_tags_11 },
    { 0x2c45b664, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29a5b667, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d45b66b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6625b672, SUC_lexicon_tags_17 },
    { 0x2245b672, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e05b676, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9285b67e, SUC_lexicon_tags_37 },
    { 0xc3e5b67f, SUC_lexicon_tags_32 },
    { 0x4205b67f, SUC_lexicon_tags_17 },
    { 0x1205b67f, SUC_lexicon_tags_11 },
    { 0x1ce5b680, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xab25b684, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd585b686, SUC_lexicon_tags_60 },
    { 0xdd45b686, SUC_lexicon_tags_60 },
    { 0x1fc5b688, SUC_lexicon_tags_68 },
    { 0x9805b686, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5b68e, SUC_lexicon_tags_29 },
    { 0x2a05b68e, SUC_lexicon_tags_28 },
    { 0xc445b690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c65b692, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e05b695, SUC_lexicon_tags_65 },
    { 0x2de5b696, SUC_lexicon_tags_14 },
    { 0x6cc5b696, SUC_lexicon_tags_68 },
    { 0xb7a5b696, SUC_lexicon_tags_10 },
    { 0xbd05b699, SUC_lexicon_tags_35 },
    { 0x6bc5b69a, SUC_lexicon_tags_37 },
    { 0xefe5b697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e5b6a0, SUC_lexicon_tags_12 },
    { 0x6b45b6a1, SUC_lexicon_tags_37 },
    { 0x4a45b6a0, SUC_lexicon_tags_37 },
    { 0xf405b6a0, SUC_lexicon_tags_17 },
    { 0x33c5b6a4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcf85b6a6, SUC_lexicon_tags_37 },
    { 0xac25b6a6, SUC_lexicon_tags_31 },
    { 0x2d05b6a8, SUC_lexicon_tags_8 },
    { 0x5745b6a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf2c5b6ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6465b6af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa485b6b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc1c5b6b4, SUC_lexicon_tags_17 },
    { 0x2585b6b4, SUC_lexicon_tags_2 },
    { 0xed05b6b6, SUC_lexicon_tags_11 },
    { 0x725b6b7, SUC_lexicon_tags_0 },
    { 0x74e5b6b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8745b6c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4a5b6c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc185b6c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7725b6c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e45b6cd, SUC_lexicon_tags_45 },
    { 0x7e5b6ce, SUC_lexicon_tags_60 },
    { 0xd885b6cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b6d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe765b6d3, SUC_lexicon_tags_25 },
    { 0xeb85b6d4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca85b6da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a5b6dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9625b6e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4265b6e9, SUC_lexicon_tags_37 },
    { 0x1185b6ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7225b6ec, SUC_lexicon_tags_63 },
    { 0xc85b6ec, SUC_lexicon_tags_11 },
    { 0x89c5b6ed, SUC_lexicon_tags_11 },
    { 0xe125b6ef, SUC_lexicon_tags_45 },
    { 0xebe5b6ed, SUC_lexicon_tags_10 },
    { 0x4c05b6ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce85b6f3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1325b6ff, SUC_lexicon_tags_25 },
    { 0x1d85b700, SUC_lexicon_tags_43 },
    { 0xb25b701, SUC_lexicon_tags_19 },
    { 0x2845b702, SUC_lexicon_tags_10 },
    { 0xf505b703, SUC_lexicon_tags_11 },
    { 0x3c25b703, SUC_lexicon_tags_10 },
    { 0x1325b701, SUC_lexicon_tags_11 },
    { 0x8f05b706, SUC_lexicon_tags_60 },
    { 0x32c5b707, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb45b70b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc9c5b70d, SUC_lexicon_tags_23 },
    { 0x9fc5b70e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6725b715, SUC_lexicon_tags_17 },
    { 0x8425b716, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c5b71b, SUC_lexicon_tags_31 },
    { 0xc405b71c, SUC_lexicon_tags_29 },
    { 0x2a65b71c, SUC_lexicon_tags_16 },
    { 0x70e5b71e, SUC_lexicon_tags_12 },
    { 0xb725b71f, SUC_lexicon_tags_37 },
    { 0xd845b71e, SUC_lexicon_tags_15 },
    { 0x7405b721, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7185b724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef85b72a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d05b734, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc745b737, SUC_lexicon_tags_45 },
    { 0x6fe5b738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce5b73c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a05b73e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc45b740, SUC_lexicon_tags_60 },
    { 0x565b741, SUC_lexicon_tags_14 },
    { 0xb5e5b742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7885b744, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec5b747, SUC_lexicon_tags_11 },
    { 0x4b25b748, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a85b74d, SUC_lexicon_tags_1 },
    { 0x9cc5b74d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a5b751, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5585b754, SUC_lexicon_tags_17 },
    { 0xfac5b754, SUC_lexicon_tags_34 },
    { 0x15e5b755, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed65b75b, SUC_lexicon_tags_56 },
    { 0x8d45b75c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf405b764, SUC_lexicon_tags_2 },
    { 0x5fa5b765, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe885b768, SUC_lexicon_tags_701 },
    { 0xe25b769, SUC_lexicon_tags_25 },
    { 0x3925b76a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6145b76c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3325b771, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd65b773, SUC_lexicon_tags_58 },
    { 0x4ac5b774, SUC_lexicon_tags_11 },
    { 0xa785b775, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7885b777, SUC_lexicon_tags_77 },
    { 0xc445b778, SUC_lexicon_tags_17 },
    { 0x7145b778, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc05b77d, SUC_lexicon_tags_45 },
    { 0x17c5b77e, SUC_lexicon_tags_15 },
    { 0x545b77f, SUC_lexicon_tags_15 },
    { 0x4e65b77e, SUC_lexicon_tags_1 },
    { 0x20a5b77d, SUC_lexicon_tags_63 },
    { 0xb725b782, SUC_lexicon_tags_68 },
    { 0x6d45b780, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff05b78a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34c5b78c, SUC_lexicon_tags_23 },
    { 0x2805b78c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2785b790, SUC_lexicon_tags_13 },
    { 0xffe5b791, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1225b794, SUC_lexicon_tags_1 },
    { 0xe505b795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b79e, SUC_lexicon_tags_1 },
    { 0x18a5b79f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e5b7a3, SUC_lexicon_tags_30 },
    { 0x6325b7a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd425b7a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc125b7b0, SUC_lexicon_tags_23 },
    { 0x5a85b7b0, SUC_lexicon_tags_37 },
    { 0x73a5b7b2, SUC_lexicon_tags_3 },
    { 0xf985b7b3, SUC_lexicon_tags_8 },
    { 0x585b7b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5f25b7b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2285b7b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7285b7bb, SUC_lexicon_tags_46 },
    { 0x18a5b7bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1885b7c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d05b7c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45b7cc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa885b7d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1865b7d3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd925b7d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1385b7d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8125b7db, SUC_lexicon_tags_46 },
    { 0xad25b7dc, SUC_lexicon_tags_47 },
    { 0x6ba5b7dd, SUC_lexicon_tags_2 },
    { 0x4f05b7dd, SUC_lexicon_tags_37 },
    { 0xd0c5b7de, SUC_lexicon_tags_10 },
    { 0x8c45b7e0, SUC_lexicon_tags_19 },
    { 0xf605b7dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7445b7ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6465b7ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9865b7f0, SUC_lexicon_tags_23 },
    { 0x6645b7f0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec05b7f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7de5b7f7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4be5b7f9, SUC_lexicon_tags_37 },
    { 0x7a5b7fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x865b7fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e5b802, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd425b804, SUC_lexicon_tags_2 },
    { 0x24a5b805, SUC_lexicon_tags_61 },
    { 0xc905b806, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd865b808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd845b80a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa745b80f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafc5b811, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2405b819, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe685b81b, SUC_lexicon_tags_17 },
    { 0x7825b81b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa5b821, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8765b824, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a5b82c, SUC_lexicon_tags_77 },
    { 0x8e65b82d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x865b830, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c65b836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b45b839, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2785b83b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8525b83d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2845b83f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2105b841, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9505b844, SUC_lexicon_tags_29 },
    { 0xb225b844, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x665b847, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x9c25b849, SUC_lexicon_tags_0 },
    { 0xee45b84a, SUC_lexicon_tags_17 },
    { 0x7d65b84a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd025b84f, SUC_lexicon_tags_7 },
    { 0xac65b850, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4405b853, SUC_lexicon_tags_49 },
    { 0xb545b853, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c5b859, SUC_lexicon_tags_2 },
    { 0x4d45b85a, SUC_lexicon_tags_14 },
    { 0x4aa5b85b, SUC_lexicon_tags_65 },
    { 0x1c05b85c, SUC_lexicon_tags_17 },
    { 0x2aa5b85d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc5b863, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b65b866, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc45b86e, SUC_lexicon_tags_11 },
    { 0xfda5b86f, SUC_lexicon_tags_84 },
    { 0x77e5b870, SUC_lexicon_tags_33 },
    { 0xf65b871, SUC_lexicon_tags_7 },
    { 0x4a05b86e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba5b876, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa6a5b878, SUC_lexicon_tags_44 },
    { 0xba65b879, SUC_lexicon_tags_11 },
    { 0xe745b879, SUC_lexicon_tags_1 },
    { 0xa4e5b878, SUC_lexicon_tags_14 },
    { 0xd6c5b879, SUC_lexicon_tags_45 },
    { 0x5905b87d, SUC_lexicon_tags_66 },
    { 0x7e25b87e, SUC_lexicon_tags_60 },
    { 0x5a25b87f, SUC_lexicon_tags_0 },
    { 0xcb05b87f, SUC_lexicon_tags_47 },
    { 0xe725b87f, SUC_lexicon_tags_37 },
    { 0xfe25b87f, SUC_lexicon_tags_27 },
    { 0xff85b883, SUC_lexicon_tags_27 },
    { 0x8225b882, SUC_lexicon_tags_15 },
    { 0xc8c5b884, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc65b887, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a25b88a, SUC_lexicon_tags_10 },
    { 0xfda5b88b, SUC_lexicon_tags_3 },
    { 0x1645b88c, SUC_lexicon_tags_17 },
    { 0xf2a5b88d, SUC_lexicon_tags_7 },
    { 0x5c45b88e, SUC_lexicon_tags_18 },
    { 0x1045b88f, SUC_lexicon_tags_15 },
    { 0xb125b890, SUC_lexicon_tags_11 },
    { 0x4985b891, SUC_lexicon_tags_45 },
    { 0x8265b891, SUC_lexicon_tags_2 },
    { 0x9445b88d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2205b895, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f85b897, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94c5b899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x185b89c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb445b8a3, SUC_lexicon_tags_7 },
    { 0xd3c5b8a4, SUC_lexicon_tags_5 },
    { 0x9ba5b8a4, SUC_lexicon_tags_38 },
    { 0xb805b8a6, SUC_lexicon_tags_15 },
    { 0x59e5b8a7, SUC_lexicon_tags_6 },
    { 0x7605b8a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d45b8ac, SUC_lexicon_tags_33 },
    { 0x4845b8ad, SUC_lexicon_tags_17 },
    { 0x6865b8ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0c5b8b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9d65b8b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e5b8b6, SUC_lexicon_tags_37 },
    { 0x9fa5b8b7, SUC_lexicon_tags_63 },
    { 0x1ae5b8b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd825b8be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5b8c2, SUC_lexicon_tags_37 },
    { 0x2605b8c3, SUC_lexicon_tags_25 },
    { 0x2705b8c3, SUC_lexicon_tags_37 },
    { 0x48a5b8c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1225b8c7, SUC_lexicon_tags_26 },
    { 0x1a05b8c7, SUC_lexicon_tags_23 },
    { 0x8405b8c9, SUC_lexicon_tags_6 },
    { 0x6b25b8ca, SUC_lexicon_tags_25 },
    { 0x5905b8ca, SUC_lexicon_tags_2 },
    { 0x3de5b8cb, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x78a5b8ce, SUC_lexicon_tags_27 },
    { 0xa525b8ce, SUC_lexicon_tags_38 },
    { 0x69e5b8d0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c5b8d3, SUC_lexicon_tags_15 },
    { 0xab25b8d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8925b8d9, SUC_lexicon_tags_38 },
    { 0x6205b8d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc265b8de, SUC_lexicon_tags_2 },
    { 0x9fc5b8de, SUC_lexicon_tags_9 },
    { 0x5a25b8e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7885b8e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9bc5b8e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7f25b8e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e5b8ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6165b8f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e5b900, SUC_lexicon_tags_11 },
    { 0x65b901, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ae5b903, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf385b905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d05b907, SUC_lexicon_tags_2 },
    { 0xab25b908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e85b90a, SUC_lexicon_tags_33 },
    { 0x4705b90b, SUC_lexicon_tags_77 },
    { 0xc565b90c, SUC_lexicon_tags_14 },
    { 0x12a5b90d, SUC_lexicon_tags_1 },
    { 0xefa5b90d, SUC_lexicon_tags_15 },
    { 0xe0c5b90f, SUC_lexicon_tags_1 },
    { 0x3b25b90f, SUC_lexicon_tags_27 },
    { 0x9145b911, SUC_lexicon_tags_26 },
    { 0x3c05b912, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5145b914, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7d25b916, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c5b91a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e5b920, SUC_lexicon_tags_16 },
    { 0xcda5b921, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1745b923, SUC_lexicon_tags_54 },
    { 0xe8a5b924, SUC_lexicon_tags_27 },
    { 0x11e5b925, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc725b932, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce5b935, SUC_lexicon_tags_25 },
    { 0x7645b935, SUC_lexicon_tags_68 },
    { 0xe165b936, SUC_lexicon_tags_14 },
    { 0xbf05b935, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e65b940, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf305b944, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc5b94b, SUC_lexicon_tags_9 },
    { 0xae05b94b, SUC_lexicon_tags_0 },
    { 0x93a5b94d, SUC_lexicon_tags_17 },
    { 0x52a5b94c, SUC_lexicon_tags_11 },
    { 0xbf05b94d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c5b952, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5e45b954, SUC_lexicon_tags_7 },
    { 0x6e45b955, SUC_lexicon_tags_17 },
    { 0x34a5b956, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd405b95d, SUC_lexicon_tags_23 },
    { 0x1b85b95e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6245b960, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf245b963, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5b969, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5765b96e, SUC_lexicon_tags_14 },
    { 0x88e5b96f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1365b971, SUC_lexicon_tags_23 },
    { 0xe145b971, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe485b977, SUC_lexicon_tags_25 },
    { 0xe565b978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c5b97b, SUC_lexicon_tags_29 },
    { 0x5d85b97c, SUC_lexicon_tags_30 },
    { 0xe525b97b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7385b97f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55c5b981, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf85b983, SUC_lexicon_tags_7 },
    { 0xf4c5b983, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc5b988, SUC_lexicon_tags_16 },
    { 0x98a5b988, SUC_lexicon_tags_60 },
    { 0x5dc5b98a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05b98d, SUC_lexicon_tags_4 },
    { 0x4e05b98e, SUC_lexicon_tags_15 },
    { 0x2d25b98f, SUC_lexicon_tags_7 },
    { 0xf2c5b98f, SUC_lexicon_tags_21 },
    { 0xb4e5b991, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf25b994, SUC_lexicon_tags_37 },
    { 0xa345b994, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdba5b997, SUC_lexicon_tags_2 },
    { 0x4a45b998, SUC_lexicon_tags_25 },
    { 0xcd25b999, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda25b99e, SUC_lexicon_tags_1 },
    { 0xf005b99f, SUC_lexicon_tags_12 },
    { 0x5c05b99f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc765b9a2, SUC_lexicon_tags_16 },
    { 0xafe5b9a3, SUC_lexicon_tags_39 },
    { 0xc65b9a4, SUC_lexicon_tags_7 },
    { 0x9025b9a5, SUC_lexicon_tags_136 },
    { 0xd165b9a4, SUC_lexicon_tags_2 },
    { 0xda45b9a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2085b9aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe685b9ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5b9b0, SUC_lexicon_tags_29 },
    { 0xc005b9b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8045b9b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb285b9ba, SUC_lexicon_tags_37 },
    { 0xc505b9ba, SUC_lexicon_tags_15 },
    { 0x6565b9ba, SUC_lexicon_tags_27 },
    { 0xcf65b9bd, SUC_lexicon_tags_12 },
    { 0xa685b9be, SUC_lexicon_tags_25 },
    { 0x6e05b9be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4825b9c1, SUC_lexicon_tags_85 },
    { 0xe885b9c2, SUC_lexicon_tags_38 },
    { 0x3185b9c1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1745b9c5, SUC_lexicon_tags_40 },
    { 0x6925b9c6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa5b9c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe125b9cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6185b9d2, SUC_lexicon_tags_2 },
    { 0xdf05b9d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65b9dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d65b9df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce5b9e5, SUC_lexicon_tags_38 },
    { 0x2305b9e5, SUC_lexicon_tags_7 },
    { 0x82e5b9e7, SUC_lexicon_tags_11 },
    { 0x38a5b9e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb65b9ea, SUC_lexicon_tags_15 },
    { 0xd625b9ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8185b9f7, SUC_lexicon_tags_14 },
    { 0xf745b9f7, SUC_lexicon_tags_29 },
    { 0x6785b9f9, SUC_lexicon_tags_15 },
    { 0xb725b9fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa5b9fe, SUC_lexicon_tags_25 },
    { 0xafe5b9ff, SUC_lexicon_tags_757 },
    { 0x8185ba00, SUC_lexicon_tags_4 },
    { 0x85a5b9fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd385ba03, SUC_lexicon_tags_10 },
    { 0x9bc5ba04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3a5ba06, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7b65ba08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a5ba0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc5ba10, SUC_lexicon_tags_17 },
    { 0x45e5ba11, SUC_lexicon_tags_49 },
    { 0xf8c5ba11, SUC_lexicon_tags_11 },
    { 0xba85ba10, SUC_lexicon_tags_9 },
    { 0x8965ba12, SUC_lexicon_tags_74 },
    { 0x25ba14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf05ba1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf785ba1c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d65ba23, SUC_lexicon_tags_1 },
    { 0x6ec5ba24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc825ba2a, SUC_lexicon_tags_61 },
    { 0x2145ba2b, SUC_lexicon_tags_47 },
    { 0x7c05ba2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf965ba2e, SUC_lexicon_tags_15 },
    { 0xe225ba2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67c5ba31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a5ba35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8565ba38, SUC_lexicon_tags_16 },
    { 0x1505ba38, SUC_lexicon_tags_12 },
    { 0xde45ba39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd45ba3e, SUC_lexicon_tags_37 },
    { 0xc625ba3e, SUC_lexicon_tags_23 },
    { 0x8505ba40, SUC_lexicon_tags_45 },
    { 0x22c5ba3f, SUC_lexicon_tags_27 },
    { 0x845ba42, SUC_lexicon_tags_1 },
    { 0xfd65ba3f, SUC_lexicon_tags_27 },
    { 0x1ae5ba44, SUC_lexicon_tags_61 },
    { 0x645ba3e, SUC_lexicon_tags_28 },
    { 0xf605ba46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec5ba49, SUC_lexicon_tags_1 },
    { 0xb985ba4a, SUC_lexicon_tags_12 },
    { 0xc5ba4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d25ba4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a5ba53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab85ba56, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xabe5ba58, SUC_lexicon_tags_136 },
    { 0x5605ba58, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc45ba5f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x33a5ba61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c25ba67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a5ba6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef45ba6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9625ba73, SUC_lexicon_tags_10 },
    { 0xa0c5ba73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb25ba76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e85ba78, SUC_lexicon_tags_7 },
    { 0x7f65ba79, SUC_lexicon_tags_15 },
    { 0x1605ba7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefe5ba7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ba83, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d45ba88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e5ba8b, SUC_lexicon_tags_27 },
    { 0xb805ba8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a5ba93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6925ba96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9445ba98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5045ba9a, SUC_lexicon_tags_29 },
    { 0x86a5ba9a, SUC_lexicon_tags_21 },
    { 0xeea5ba9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe85baa0, SUC_lexicon_tags_63 },
    { 0xe705baa1, SUC_lexicon_tags_16 },
    { 0xfd45baa2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2865baa4, SUC_lexicon_tags_17 },
    { 0x3605baa4, SUC_lexicon_tags_37 },
    { 0xa285baa6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45baad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc85bab1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2d05bab3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd625babb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x485babd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e5bac0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de5bac4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x425bac6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5725bacd, SUC_lexicon_tags_23 },
    { 0xeac5bacd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c85bad1, SUC_lexicon_tags_5 },
    { 0x2b25bad2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2305bad4, SUC_lexicon_tags_31 },
    { 0x8ee5bad5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1c5bad7, SUC_lexicon_tags_39 },
    { 0x6045bad8, SUC_lexicon_tags_25 },
    { 0xcd45bad9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d65badb, SUC_lexicon_tags_1 },
    { 0x9545badb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e65badf, SUC_lexicon_tags_15 },
    { 0xafc5bae0, SUC_lexicon_tags_8 },
    { 0x4e05bae0, SUC_lexicon_tags_11 },
    { 0xab45bae2, SUC_lexicon_tags_23 },
    { 0x5ce5bae3, SUC_lexicon_tags_14 },
    { 0x37a5bae4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb645bae6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e65baea, SUC_lexicon_tags_6 },
    { 0xe0c5baeb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a5baee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3765baf0, SUC_lexicon_tags_48 },
    { 0x2585baf0, SUC_lexicon_tags_25 },
    { 0x3d85baf1, SUC_lexicon_tags_46 },
    { 0xd105baf3, SUC_lexicon_tags_25 },
    { 0xb205baf1, SUC_lexicon_tags_62 },
    { 0x3c65baf4, SUC_lexicon_tags_9 },
    { 0x545baf3, SUC_lexicon_tags_17 },
    { 0xa6e5baf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b85baf9, SUC_lexicon_tags_37 },
    { 0xe405baf9, SUC_lexicon_tags_37 },
    { 0x3ec5bafb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadc5bafd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac5bb06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4905bb0b, SUC_lexicon_tags_65 },
    { 0xe645bb0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c5bb10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6965bb14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfae5bb16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac5bb1b, SUC_lexicon_tags_23 },
    { 0x2c45bb1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9605bb20, SUC_lexicon_tags_7 },
    { 0x80c5bb20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe645bb2b, SUC_lexicon_tags_9 },
    { 0xf825bb2c, SUC_lexicon_tags_49 },
    { 0xbfc5bb2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee5bb31, SUC_lexicon_tags_60 },
    { 0x4885bb31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4325bb35, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c5bb39, SUC_lexicon_tags_12 },
    { 0x5b85bb3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a85bb3f, SUC_lexicon_tags_81 },
    { 0x6465bb40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2725bb44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9525bb46, SUC_lexicon_tags_7 },
    { 0x3005bb47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de5bb4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5485bb4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce65bb4f, SUC_lexicon_tags_47 },
    { 0xd1c5bb4f, SUC_lexicon_tags_37 },
    { 0x4e25bb4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x965bb54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4825bb57, SUC_lexicon_tags_9 },
    { 0xd745bb58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f65bb5a, SUC_lexicon_tags_2 },
    { 0x19c5bb5b, SUC_lexicon_tags_17 },
    { 0xb845bb5c, SUC_lexicon_tags_103 },
    { 0xc4a5bb5d, SUC_lexicon_tags_2 },
    { 0x5085bb5d, SUC_lexicon_tags_37 },
    { 0xef05bb5f, SUC_lexicon_tags_23 },
    { 0x6ce5bb60, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8a5bb62, SUC_lexicon_tags_11 },
    { 0x3825bb62, SUC_lexicon_tags_25 },
    { 0x2b45bb64, SUC_lexicon_tags_23 },
    { 0x2585bb65, SUC_lexicon_tags_1 },
    { 0x54a5bb66, SUC_lexicon_tags_25 },
    { 0x2b25bb64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x20c5bb69, SUC_lexicon_tags_23 },
    { 0x1ea5bb69, SUC_lexicon_tags_83 },
    { 0x7a5bb6b, SUC_lexicon_tags_19 },
    { 0xaea5bb6c, SUC_lexicon_tags_17 },
    { 0xe165bb6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4125bb6f, SUC_lexicon_tags_1 },
    { 0xee25bb70, SUC_lexicon_tags_87 },
    { 0x32e5bb71, SUC_lexicon_tags_33 },
    { 0x9225bb70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c5bb78, SUC_lexicon_tags_11 },
    { 0x4c25bb78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7785bb7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf445bb80, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc5bb86, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2065bb8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x37a5bb8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4625bb90, SUC_lexicon_tags_17 },
    { 0xe7c5bb91, SUC_lexicon_tags_33 },
    { 0xa25bb91, SUC_lexicon_tags_28 },
    { 0x2125bb92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5bb9f, SUC_lexicon_tags_29 },
    { 0x8005bb9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d85bba3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf2e5bba5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c5bba9, SUC_lexicon_tags_84 },
    { 0xe8a5bbaa, SUC_lexicon_tags_63 },
    { 0x5125bbab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5bbaf, SUC_lexicon_tags_1 },
    { 0xbe45bbb0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e5bbb9, SUC_lexicon_tags_14 },
    { 0xc645bbba, SUC_lexicon_tags_60 },
    { 0x2c65bbbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c5bbc1, SUC_lexicon_tags_30 },
    { 0xfa05bbc2, SUC_lexicon_tags_27 },
    { 0xa3c5bbc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa445bbc7, SUC_lexicon_tags_57 },
    { 0xa3a5bbc8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc65bbcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c5bbd3, SUC_lexicon_tags_63 },
    { 0xa925bbd3, SUC_lexicon_tags_15 },
    { 0xc745bbd5, SUC_lexicon_tags_7 },
    { 0x56a5bbd6, SUC_lexicon_tags_25 },
    { 0x8ae5bbd7, SUC_lexicon_tags_12 },
    { 0x2205bbd8, SUC_lexicon_tags_47 },
    { 0xf085bbd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcde5bbdb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b45bbde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf885bbe6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85bbea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa405bbf3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1765bbf5, SUC_lexicon_tags_92 },
    { 0x39c5bbf5, SUC_lexicon_tags_12 },
    { 0x3ea5bbf7, SUC_lexicon_tags_12 },
    { 0x2965bbf7, SUC_lexicon_tags_0 },
    { 0x1ac5bbf8, SUC_lexicon_tags_2 },
    { 0x5d65bbfa, SUC_lexicon_tags_34 },
    { 0x4ee5bbfa, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc3a5bbfd, SUC_lexicon_tags_7 },
    { 0x32e5bbfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc65bc00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd45bc03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e45bc07, SUC_lexicon_tags_1 },
    { 0xa1e5bc07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e5bc0b, SUC_lexicon_tags_8 },
    { 0xcd25bc0c, SUC_lexicon_tags_38 },
    { 0x7805bc0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5985bc10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd25bc12, SUC_lexicon_tags_8 },
    { 0x5b05bc13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1785bc15, SUC_lexicon_tags_60 },
    { 0xd145bc16, SUC_lexicon_tags_7 },
    { 0xb0a5bc16, SUC_lexicon_tags_65 },
    { 0xc245bc16, SUC_lexicon_tags_7 },
    { 0x98a5bc17, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x67c5bc1b, SUC_lexicon_tags_62 },
    { 0x15a5bc1b, SUC_lexicon_tags_12 },
    { 0x8e85bc1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1a5bc1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7165bc22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab85bc2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5865bc2d, SUC_lexicon_tags_96 },
    { 0xb585bc2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b45bc31, SUC_lexicon_tags_8 },
    { 0x3a45bc31, SUC_lexicon_tags_35 },
    { 0x24e5bc32, SUC_lexicon_tags_37 },
    { 0x6365bc34, SUC_lexicon_tags_8 },
    { 0xba25bc35, SUC_lexicon_tags_2 },
    { 0xa445bc35, SUC_lexicon_tags_2 },
    { 0x8cc5bc37, SUC_lexicon_tags_17 },
    { 0xe5bc37, SUC_lexicon_tags_17 },
    { 0xe0a5bc36, SUC_lexicon_tags_3 },
    { 0x44e5bc35, SUC_lexicon_tags_46 },
    { 0xf8e5bc36, SUC_lexicon_tags_45 },
    { 0xdba5bc3c, SUC_lexicon_tags_37 },
    { 0x26e5bc3c, SUC_lexicon_tags_0 },
    { 0xf85bc3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4465bc40, SUC_lexicon_tags_63 },
    { 0xc345bc41, SUC_lexicon_tags_1 },
    { 0x69a5bc41, SUC_lexicon_tags_79 },
    { 0xbde5bc43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5445bc45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d05bc47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce5bc4b, SUC_lexicon_tags_4 },
    { 0x345bc4c, SUC_lexicon_tags_45 },
    { 0xe0e5bc4d, SUC_lexicon_tags_14 },
    { 0xd705bc4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc545bc51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda5bc54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3205bc59, SUC_lexicon_tags_37 },
    { 0xcbe5bc59, SUC_lexicon_tags_49 },
    { 0xb025bc5b, SUC_lexicon_tags_37 },
    { 0xbb85bc5b, SUC_lexicon_tags_29 },
    { 0x76a5bc5a, SUC_lexicon_tags_37 },
    { 0xa745bc5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x885bc61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a5bc66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe805bc69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a65bc6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de5bc70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a5bc74, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad85bc7b, SUC_lexicon_tags_27 },
    { 0x9265bc7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa5bc7f, SUC_lexicon_tags_11 },
    { 0x4c5bc80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a5bc84, SUC_lexicon_tags_108 },
    { 0x8a65bc85, SUC_lexicon_tags_15 },
    { 0xa9c5bc84, SUC_lexicon_tags_15 },
    { 0xe65bc87, SUC_lexicon_tags_549 },
    { 0x9ac5bc88, SUC_lexicon_tags_1 },
    { 0x6a25bc89, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6985bc8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa465bc90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c5bc93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad05bc96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf465bc99, SUC_lexicon_tags_25 },
    { 0xdfa5bc9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4985bc9c, SUC_lexicon_tags_37 },
    { 0xde65bc9d, SUC_lexicon_tags_25 },
    { 0xad65bc9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe605bca4, SUC_lexicon_tags_1 },
    { 0x5205bca4, SUC_lexicon_tags_18 },
    { 0x4e65bca6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x545bca8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7dc5bcaa, SUC_lexicon_tags_24 },
    { 0xb765bcab, SUC_lexicon_tags_17 },
    { 0xb585bcac, SUC_lexicon_tags_25 },
    { 0x89c5bcad, SUC_lexicon_tags_16 },
    { 0x8165bcae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5bcb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2645bcb8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e5bcbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4e5bcbd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xce45bcc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe685bcc5, SUC_lexicon_tags_6 },
    { 0xd4c5bcc6, SUC_lexicon_tags_15 },
    { 0xbe25bcc6, SUC_lexicon_tags_64 },
    { 0x9fc5bcc8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa05bccb, SUC_lexicon_tags_0 },
    { 0xc785bccb, SUC_lexicon_tags_25 },
    { 0x90e5bccb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1945bcd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9945bcd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33c5bcd6, SUC_lexicon_tags_39 },
    { 0xd2e5bcd7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45bcda, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e5bcdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6e5bcdf, SUC_lexicon_tags_25 },
    { 0x32c5bce0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6245bce4, SUC_lexicon_tags_6 },
    { 0xd3e5bce5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfbc5bce7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25bcea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e5bced, SUC_lexicon_tags_35 },
    { 0x3d25bcee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3265bcf0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d45bcf7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1745bcfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf05bd04, SUC_lexicon_tags_31 },
    { 0xe665bd04, SUC_lexicon_tags_46 },
    { 0xd305bd06, SUC_lexicon_tags_33 },
    { 0x83e5bd07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca5bd0b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc145bd0e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd785bd11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e5bd13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4485bd17, SUC_lexicon_tags_12 },
    { 0xf2e5bd18, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd25bd1a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1245bd1d, SUC_lexicon_tags_91 },
    { 0x6725bd1e, SUC_lexicon_tags_17 },
    { 0x84e5bd1e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c5bd22, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e5bd2d, SUC_lexicon_tags_6 },
    { 0x8a25bd2e, SUC_lexicon_tags_25 },
    { 0xd645bd2e, SUC_lexicon_tags_14 },
    { 0x2ce5bd30, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e5bd33, SUC_lexicon_tags_12 },
    { 0x23a5bd33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc885bd36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab45bd3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9c5bd3c, SUC_lexicon_tags_25 },
    { 0xab25bd3d, SUC_lexicon_tags_19 },
    { 0xe705bd3d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea5bd41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c5bd46, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2765bd4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4925bd4c, SUC_lexicon_tags_9 },
    { 0x5b25bd4d, SUC_lexicon_tags_25 },
    { 0x7605bd4c, SUC_lexicon_tags_2 },
    { 0x8845bd4f, SUC_lexicon_tags_2 },
    { 0xa9a5bd50, SUC_lexicon_tags_12 },
    { 0xdd65bd51, SUC_lexicon_tags_15 },
    { 0x46a5bd52, SUC_lexicon_tags_30 },
    { 0x25c5bd53, SUC_lexicon_tags_55 },
    { 0xe225bd52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c5bd57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa5bd59, SUC_lexicon_tags_25 },
    { 0x3205bd59, SUC_lexicon_tags_2 },
    { 0x63c5bd5b, SUC_lexicon_tags_61 },
    { 0x4f45bd59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc5bd62, SUC_lexicon_tags_11 },
    { 0x8125bd63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda5bd67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25bd6a, SUC_lexicon_tags_12 },
    { 0xc865bd6b, SUC_lexicon_tags_45 },
    { 0x285bd6b, SUC_lexicon_tags_7 },
    { 0x5bd6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf05bd6f, SUC_lexicon_tags_37 },
    { 0x7fa5bd70, SUC_lexicon_tags_11 },
    { 0xbac5bd71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1065bd76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde5bd7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa5bd84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda45bd92, SUC_lexicon_tags_37 },
    { 0xc825bd93, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0x8345bd95, SUC_lexicon_tags_11 },
    { 0xbcc5bd96, SUC_lexicon_tags_17 },
    { 0x505bd96, SUC_lexicon_tags_19 },
    { 0x5325bd97, SUC_lexicon_tags_1 },
    { 0xc225bd99, SUC_lexicon_tags_23 },
    { 0x6525bd99, SUC_lexicon_tags_19 },
    { 0xd2e5bd96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3025bda8, SUC_lexicon_tags_17 },
    { 0xbde5bda9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33e5bdab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8005bdad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda45bdb1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf845bdb8, SUC_lexicon_tags_160 },
    { 0xd145bdb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe925bdbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc765bdc1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2345bdc4, SUC_lexicon_tags_12 },
    { 0x3585bdc5, SUC_lexicon_tags_13 },
    { 0xa685bdc5, SUC_lexicon_tags_8 },
    { 0x8c65bdc5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c5bdcf, SUC_lexicon_tags_47 },
    { 0xea5bdd0, SUC_lexicon_tags_23 },
    { 0x1265bdd0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1585bdd6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4105bdd9, SUC_lexicon_tags_39 },
    { 0xdc85bdda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39e5bddc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec5bddf, SUC_lexicon_tags_60 },
    { 0x7065bde0, SUC_lexicon_tags_29 },
    { 0xd7a5bde1, SUC_lexicon_tags_5 },
    { 0xe2e5bde1, SUC_lexicon_tags_46 },
    { 0x30a5bde3, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0xe005bde6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85bdee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe745bdf0, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x8525bdf2, SUC_lexicon_tags_12 },
    { 0xb845bdf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc785bdf7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a5bdfa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe805bdfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45be01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa5be09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0c5be0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6be5be0d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805be12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc205be14, SUC_lexicon_tags_15 },
    { 0x3a85be15, SUC_lexicon_tags_92 },
    { 0x5045be15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4445be18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e5be1d, SUC_lexicon_tags_17 },
    { 0x4265be1e, SUC_lexicon_tags_15 },
    { 0xdb85be1e, SUC_lexicon_tags_17 },
    { 0xa305be20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec5be23, SUC_lexicon_tags_25 },
    { 0x42a5be24, SUC_lexicon_tags_45 },
    { 0x7245be24, SUC_lexicon_tags_136 },
    { 0xab45be24, SUC_lexicon_tags_103 },
    { 0x9805be27, SUC_lexicon_tags_11 },
    { 0x81c5be28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6125be2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1245be2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5be31, SUC_lexicon_tags_39 },
    { 0x3505be31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e05be34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ea5be36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c45be38, SUC_lexicon_tags_26 },
    { 0xba45be39, SUC_lexicon_tags_2 },
    { 0x7485be3a, SUC_lexicon_tags_7 },
    { 0xe9c5be3b, SUC_lexicon_tags_17 },
    { 0x28e5be3b, SUC_lexicon_tags_15 },
    { 0x6d05be3a, SUC_lexicon_tags_6 },
    { 0x4285be3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d25be40, SUC_lexicon_tags_28 },
    { 0x4665be41, SUC_lexicon_tags_46 },
    { 0x1fe5be40, SUC_lexicon_tags_33 },
    { 0x35e5be43, SUC_lexicon_tags_37 },
    { 0x9365be44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f85be46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9985be4b, SUC_lexicon_tags_9 },
    { 0x8b45be4b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc5be50, SUC_lexicon_tags_26 },
    { 0xdb45be50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d65be56, SUC_lexicon_tags_11 },
    { 0xa805be57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa5be5c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb65be5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a5be62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b85be65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a5be69, SUC_lexicon_tags_10 },
    { 0xa005be69, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x27c5be6c, SUC_lexicon_tags_25 },
    { 0x5465be6c, SUC_lexicon_tags_1 },
    { 0x7305be6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb565be74, SUC_lexicon_tags_7 },
    { 0xa5be74, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba25be80, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e65be89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b05be8c, SUC_lexicon_tags_23 },
    { 0x105be8d, SUC_lexicon_tags_30 },
    { 0x6905be8d, SUC_lexicon_tags_15 },
    { 0xd0a5be8f, SUC_lexicon_tags_30 },
    { 0xc3c5be90, SUC_lexicon_tags_2 },
    { 0xc0c5be91, SUC_lexicon_tags_12 },
    { 0x3885be92, SUC_lexicon_tags_2 },
    { 0x1245be93, SUC_lexicon_tags_23 },
    { 0x7b85be8c, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x38c5be96, SUC_lexicon_tags_7 },
    { 0x7d05be97, SUC_lexicon_tags_30 },
    { 0xa965be96, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee5be9c, SUC_lexicon_tags_26 },
    { 0x6de5be9d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xec5be9f, SUC_lexicon_tags_15 },
    { 0x1c85bea0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce5bea3, SUC_lexicon_tags_7 },
    { 0x5565bea4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8085bea9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc425beac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8965beaf, SUC_lexicon_tags_46 },
    { 0xbd65beb0, SUC_lexicon_tags_17 },
    { 0x58a5beaf, SUC_lexicon_tags_37 },
    { 0xbd85beb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8205beb6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3ca5beb8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa965bebc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4265bebe, SUC_lexicon_tags_16 },
    { 0x9aa5bebf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1065bec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fa5bec3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba5bec6, SUC_lexicon_tags_1 },
    { 0x4905bec7, SUC_lexicon_tags_25 },
    { 0x60a5bec6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac5becc, SUC_lexicon_tags_2 },
    { 0xd885becc, SUC_lexicon_tags_178 },
    { 0x485bece, SUC_lexicon_tags_37 },
    { 0x2ba5bece, SUC_lexicon_tags_1 },
    { 0xfb05bed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e5bed3, SUC_lexicon_tags_37 },
    { 0xff05bed4, SUC_lexicon_tags_38 },
    { 0x905bed5, SUC_lexicon_tags_27 },
    { 0xfc45bed6, SUC_lexicon_tags_17 },
    { 0x60c5bed7, SUC_lexicon_tags_17 },
    { 0x6c05bed7, SUC_lexicon_tags_23 },
    { 0xdac5bed4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2285bedc, SUC_lexicon_tags_37 },
    { 0x3645bedd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe5bee6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xcd85bee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9325beea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da5beee, SUC_lexicon_tags_28 },
    { 0x7505beef, SUC_lexicon_tags_12 },
    { 0xc2e5beee, SUC_lexicon_tags_1 },
    { 0x39a5bef1, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc005bef8, SUC_lexicon_tags_11 },
    { 0xd805bef9, SUC_lexicon_tags_6 },
    { 0x1245befa, SUC_lexicon_tags_68 },
    { 0x90e5bef9, SUC_lexicon_tags_2 },
    { 0xa185bef9, SUC_lexicon_tags_18 },
    { 0x5e05bef8, SUC_lexicon_tags_12 },
    { 0xdfc5befe, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4285bf04, SUC_lexicon_tags_2 },
    { 0x3185bf04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba5bf0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2005bf0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e5bf11, SUC_lexicon_tags_11 },
    { 0x6c65bf11, SUC_lexicon_tags_23 },
    { 0x8625bf12, SUC_lexicon_tags_39 },
    { 0xa985bf14, SUC_lexicon_tags_60 },
    { 0xe145bf13, SUC_lexicon_tags_7 },
    { 0x8625bf16, SUC_lexicon_tags_17 },
    { 0xa1e5bf17, SUC_lexicon_tags_2 },
    { 0x1885bf18, SUC_lexicon_tags_12 },
    { 0x9445bf16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1265bf1c, SUC_lexicon_tags_17 },
    { 0x89e5bf1d, SUC_lexicon_tags_19 },
    { 0xefc5bf1e, SUC_lexicon_tags_12 },
    { 0xe6c5bf1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea5bf22, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f25bf26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa825bf28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdc5bf2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf25bf30, SUC_lexicon_tags_44 },
    { 0x9205bf30, SUC_lexicon_tags_12 },
    { 0x5485bf32, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf505bf36, SUC_lexicon_tags_26 },
    { 0x1e65bf37, SUC_lexicon_tags_22 },
    { 0x9225bf36, SUC_lexicon_tags_37 },
    { 0xf65bf39, SUC_lexicon_tags_2 },
    { 0x5bf3a, SUC_lexicon_tags_390 },
    { 0xeaa5bf3a, SUC_lexicon_tags_17 },
    { 0x6905bf37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2925bf3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9025bf40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2965bf47, SUC_lexicon_tags_25 },
    { 0xc5a5bf48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc85bf4a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad45bf4c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc945bf4e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a5bf52, SUC_lexicon_tags_25 },
    { 0xab85bf53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd25bf55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66e5bf57, SUC_lexicon_tags_172 },
    { 0x55e5bf58, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3085bf5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc05bf64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a5bf68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa5bf6b, SUC_lexicon_tags_0 },
    { 0xb4c5bf6c, SUC_lexicon_tags_17 },
    { 0xb485bf6d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x4fa5bf6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe105bf71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba65bf78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee45bf7c, SUC_lexicon_tags_118 },
    { 0x6ba5bf7c, SUC_lexicon_tags_25 },
    { 0xda5bf7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2365bf81, SUC_lexicon_tags_33 },
    { 0x645bf82, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xad25bf85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc5bf88, SUC_lexicon_tags_10 },
    { 0x2f65bf88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2c5bf8b, SUC_lexicon_tags_17 },
    { 0x65a5bf8c, SUC_lexicon_tags_37 },
    { 0xc745bf8d, SUC_lexicon_tags_37 },
    { 0xb7c5bf8d, SUC_lexicon_tags_49 },
    { 0xf385bf8f, SUC_lexicon_tags_30 },
    { 0xce45bf90, SUC_lexicon_tags_37 },
    { 0xd905bf91, SUC_lexicon_tags_17 },
    { 0xef25bf92, SUC_lexicon_tags_25 },
    { 0xb365bf93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e05bf95, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd905bf99, SUC_lexicon_tags_17 },
    { 0x87e5bf9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfb85bf9c, SUC_lexicon_tags_26 },
    { 0x48a5bf9c, SUC_lexicon_tags_1 },
    { 0xb4c5bf9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc25bfa8, SUC_lexicon_tags_11 },
    { 0x7445bfa9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c5bfac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e5bfb4, SUC_lexicon_tags_667 },
    { 0x4d65bfb5, SUC_lexicon_tags_17 },
    { 0x65a5bfb6, SUC_lexicon_tags_2 },
    { 0x46c5bfb7, SUC_lexicon_tags_33 },
    { 0x4e25bfb4, SUC_lexicon_tags_37 },
    { 0x46a5bfb9, SUC_lexicon_tags_30 },
    { 0x7125bfb5, SUC_lexicon_tags_8 },
    { 0x4c25bfb8, SUC_lexicon_tags_37 },
    { 0xb125bfbc, SUC_lexicon_tags_2 },
    { 0xb825bfbd, SUC_lexicon_tags_23 },
    { 0x2665bfb5, SUC_lexicon_tags_15 },
    { 0x5925bfb7, SUC_lexicon_tags_47 },
    { 0x145bfc0, SUC_lexicon_tags_25 },
    { 0x18e5bfc1, SUC_lexicon_tags_14 },
    { 0x2165bfc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe705bfc9, SUC_lexicon_tags_47 },
    { 0x45c5bfca, SUC_lexicon_tags_246 },
    { 0x7345bfc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1225bfcd, SUC_lexicon_tags_37 },
    { 0xfb65bfce, SUC_lexicon_tags_44 },
    { 0xad25bfcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba5bfd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2225bfd4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e5bfd7, SUC_lexicon_tags_87 },
    { 0x3bc5bfd7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd025bfde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1965bfe0, SUC_lexicon_tags_2 },
    { 0x65c5bfe0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf865bfe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a65bfe5, SUC_lexicon_tags_7 },
    { 0x6385bfe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6125bfea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4885bfec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1ae5bfee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69c5bff0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf685bff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e5bff7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2565bffd, SUC_lexicon_tags_11 },
    { 0xee5bffd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xabe5c000, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2445c004, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9445c007, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5c5c009, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5105c00b, SUC_lexicon_tags_63 },
    { 0xe205c00b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d45c012, SUC_lexicon_tags_15 },
    { 0xfb85c013, SUC_lexicon_tags_5 },
    { 0x5145c014, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb185c01a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x625c01d, SUC_lexicon_tags_85 },
    { 0x3be5c01e, SUC_lexicon_tags_19 },
    { 0x61a5c01d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x45e5c021, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x5465c024, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee5c02b, SUC_lexicon_tags_17 },
    { 0xbfa5c02c, SUC_lexicon_tags_7 },
    { 0x85e5c02d, SUC_lexicon_tags_37 },
    { 0xe6a5c02e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e5c033, SUC_lexicon_tags_25 },
    { 0x92e5c034, SUC_lexicon_tags_37 },
    { 0x74a5c035, SUC_lexicon_tags_8 },
    { 0x4805c036, SUC_lexicon_tags_37 },
    { 0x31e5c036, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6065c03b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e65c03d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd365c03f, SUC_lexicon_tags_1 },
    { 0xd6e5c040, SUC_lexicon_tags_45 },
    { 0x8485c041, SUC_lexicon_tags_11 },
    { 0x5145c042, SUC_lexicon_tags_43 },
    { 0x6845c043, SUC_lexicon_tags_12 },
    { 0x91e5c044, SUC_lexicon_tags_82 },
    { 0xba5c042, SUC_lexicon_tags_14 },
    { 0xb3c5c03f, SUC_lexicon_tags_30 },
    { 0xd2c5c047, SUC_lexicon_tags_49 },
    { 0x3e25c043, SUC_lexicon_tags_14 },
    { 0x8725c045, SUC_lexicon_tags_17 },
    { 0xfc5c04a, SUC_lexicon_tags_27 },
    { 0xd005c04a, SUC_lexicon_tags_13 },
    { 0x52c5c04a, SUC_lexicon_tags_7 },
    { 0xa85c04d, SUC_lexicon_tags_64 },
    { 0xaa85c04e, SUC_lexicon_tags_31 },
    { 0xf1e5c041, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa85c052, SUC_lexicon_tags_12 },
    { 0xb5e5c053, SUC_lexicon_tags_11 },
    { 0xf325c054, SUC_lexicon_tags_62 },
    { 0x50a5c052, SUC_lexicon_tags_2 },
    { 0x36e5c053, SUC_lexicon_tags_12 },
    { 0x5785c056, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f05c05a, SUC_lexicon_tags_54 },
    { 0x5945c05b, SUC_lexicon_tags_38 },
    { 0x6a05c05b, SUC_lexicon_tags_39 },
    { 0x4c05c05d, SUC_lexicon_tags_11 },
    { 0xde65c05e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf45c060, SUC_lexicon_tags_25 },
    { 0xa0e5c060, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe5c068, SUC_lexicon_tags_60 },
    { 0x6945c069, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd825c06b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad05c06e, SUC_lexicon_tags_15 },
    { 0xffe5c06f, SUC_lexicon_tags_23 },
    { 0xd345c070, SUC_lexicon_tags_19 },
    { 0x8c05c071, SUC_lexicon_tags_0 },
    { 0x45a5c071, SUC_lexicon_tags_87 },
    { 0x1625c073, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4665c075, SUC_lexicon_tags_15 },
    { 0xaa65c076, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4465c07d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e5c081, SUC_lexicon_tags_17 },
    { 0x69a5c082, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac5c086, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x605c088, SUC_lexicon_tags_37 },
    { 0xc545c088, SUC_lexicon_tags_45 },
    { 0x72c5c08a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9705c08c, SUC_lexicon_tags_15 },
    { 0x9145c08d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a5c092, SUC_lexicon_tags_96 },
    { 0x4e05c093, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45e5c095, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e5c098, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x65c09a, SUC_lexicon_tags_16 },
    { 0x3405c09a, SUC_lexicon_tags_37 },
    { 0xbf65c09a, SUC_lexicon_tags_35 },
    { 0x66c5c09b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a05c0a0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9ae5c0a2, SUC_lexicon_tags_45 },
    { 0x4225c0a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ba5c0a5, SUC_lexicon_tags_25 },
    { 0x8b05c0a6, SUC_lexicon_tags_1 },
    { 0xb45c0a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e85c0aa, SUC_lexicon_tags_45 },
    { 0xd9e5c0aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e5c0b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe5c0b9, SUC_lexicon_tags_38 },
    { 0x1d85c0ba, SUC_lexicon_tags_2 },
    { 0xcc65c0bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4345c0bd, SUC_lexicon_tags_24 },
    { 0x5085c0be, SUC_lexicon_tags_43 },
    { 0xfe65c0bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec5c0c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9325c0c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe225c0cc, SUC_lexicon_tags_9 },
    { 0xd745c0cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c05c0cf, SUC_lexicon_tags_2 },
    { 0x5d85c0d0, SUC_lexicon_tags_7 },
    { 0x5a65c0d1, SUC_lexicon_tags_12 },
    { 0xa2a5c0d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2aa5c0d4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc5c0d7, SUC_lexicon_tags_17 },
    { 0x73e5c0d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba5c0da, SUC_lexicon_tags_21 },
    { 0x1545c0db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a5c0de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45c0e1, SUC_lexicon_tags_25 },
    { 0xb0a5c0e2, SUC_lexicon_tags_0 },
    { 0x9485c0e3, SUC_lexicon_tags_25 },
    { 0xe25c0e4, SUC_lexicon_tags_9 },
    { 0x9bc5c0e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x185c0e7, SUC_lexicon_tags_1 },
    { 0xb185c0e8, SUC_lexicon_tags_47 },
    { 0x7425c0e9, SUC_lexicon_tags_1 },
    { 0xb285c0e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e5c0ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce5c0f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6225c0f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a5c0f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf85c0fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3785c0fe, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25c101, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe665c104, SUC_lexicon_tags_37 },
    { 0x3045c105, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde5c10d, SUC_lexicon_tags_7 },
    { 0x9a5c10e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda5c116, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc5c11b, SUC_lexicon_tags_16 },
    { 0x9c25c11c, SUC_lexicon_tags_49 },
    { 0xf385c11d, SUC_lexicon_tags_6 },
    { 0x4be5c11e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5a45c120, SUC_lexicon_tags_8 },
    { 0x9ee5c121, SUC_lexicon_tags_9 },
    { 0xa545c121, SUC_lexicon_tags_25 },
    { 0xb265c123, SUC_lexicon_tags_25 },
    { 0xbae5c124, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945c129, SUC_lexicon_tags_37 },
    { 0x77a5c129, SUC_lexicon_tags_49 },
    { 0xf2a5c12a, SUC_lexicon_tags_2 },
    { 0xbe85c12c, SUC_lexicon_tags_0 },
    { 0x5a85c12d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c5c130, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a05c132, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7965c136, SUC_lexicon_tags_25 },
    { 0xc9a5c137, SUC_lexicon_tags_43 },
    { 0x69a5c138, SUC_lexicon_tags_13 },
    { 0x7385c137, SUC_lexicon_tags_63 },
    { 0xef45c136, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b65c13c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6f45c13e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce5c141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x805c143, SUC_lexicon_tags_68 },
    { 0xa765c143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb105c146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b65c148, SUC_lexicon_tags_114 },
    { 0x82a5c149, SUC_lexicon_tags_1 },
    { 0xe325c149, SUC_lexicon_tags_1 },
    { 0x92c5c14b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3645c155, SUC_lexicon_tags_15 },
    { 0x45e5c156, SUC_lexicon_tags_21 },
    { 0x2165c156, SUC_lexicon_tags_37 },
    { 0xcf45c156, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9665c15b, SUC_lexicon_tags_65 },
    { 0xbde5c15c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a5c15f, SUC_lexicon_tags_37 },
    { 0x7865c160, SUC_lexicon_tags_12 },
    { 0x50e5c161, SUC_lexicon_tags_7 },
    { 0xcc05c162, SUC_lexicon_tags_17 },
    { 0xf345c160, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e5c166, SUC_lexicon_tags_26 },
    { 0xd185c167, SUC_lexicon_tags_6 },
    { 0x4e05c168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa345c16e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a5c173, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2125c175, SUC_lexicon_tags_25 },
    { 0xc525c176, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4905c185, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c5c191, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd045c196, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65c19c, SUC_lexicon_tags_27 },
    { 0xf445c19d, SUC_lexicon_tags_38 },
    { 0xc725c19e, SUC_lexicon_tags_38 },
    { 0x9825c19e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe185c1a4, SUC_lexicon_tags_17 },
    { 0x4b45c1a4, SUC_lexicon_tags_100 },
    { 0xc7e5c1a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe065c1a9, SUC_lexicon_tags_2 },
    { 0x7b25c1aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81c5c1ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf3e5c1ae, SUC_lexicon_tags_2 },
    { 0x1045c1ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40c5c1b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef45c1b4, SUC_lexicon_tags_26 },
    { 0x6d65c1b5, SUC_lexicon_tags_4 },
    { 0x5a5c1b6, SUC_lexicon_tags_34 },
    { 0xc9a5c1b7, SUC_lexicon_tags_17 },
    { 0x93e5c1b8, SUC_lexicon_tags_26 },
    { 0x28c5c1b9, SUC_lexicon_tags_11 },
    { 0x5245c1b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa545c1be, SUC_lexicon_tags_7 },
    { 0x965c1bf, SUC_lexicon_tags_11 },
    { 0x7b05c1c0, SUC_lexicon_tags_39 },
    { 0xf8a5c1c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2da5c1c3, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0xfa05c1c5, SUC_lexicon_tags_37 },
    { 0x3385c1c6, SUC_lexicon_tags_68 },
    { 0xd8c5c1c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a5c1ca, SUC_lexicon_tags_1 },
    { 0xae05c1cb, SUC_lexicon_tags_25 },
    { 0x6a85c1ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73e5c1ce, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a5c1d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2145c1d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa505c1db, SUC_lexicon_tags_60 },
    { 0x7de5c1db, SUC_lexicon_tags_12 },
    { 0x33e5c1dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce5c1e0, SUC_lexicon_tags_12 },
    { 0x79a5c1e0, SUC_lexicon_tags_11 },
    { 0x9a5c1e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef05c1e7, SUC_lexicon_tags_15 },
    { 0x4225c1e7, SUC_lexicon_tags_59 },
    { 0xa65c1e9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8c05c1eb, SUC_lexicon_tags_2 },
    { 0x2ce5c1ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3465c1ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9e5c1f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa585c1f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1925c1f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e25c1f7, SUC_lexicon_tags_17 },
    { 0x5245c1f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa405c1fb, SUC_lexicon_tags_11 },
    { 0xdf65c1fc, SUC_lexicon_tags_1 },
    { 0x3c85c1fd, SUC_lexicon_tags_17 },
    { 0x7ca5c1fd, SUC_lexicon_tags_25 },
    { 0xd945c1ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e5c202, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e5c205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc625c208, SUC_lexicon_tags_262 },
    { 0x9745c209, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe785c20d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e5c211, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b65c215, SUC_lexicon_tags_37 },
    { 0x24e5c215, SUC_lexicon_tags_1 },
    { 0x37c5c217, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5c21b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe5c221, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7405c223, SUC_lexicon_tags_47 },
    { 0x63e5c223, SUC_lexicon_tags_33 },
    { 0xcb85c225, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2e65c227, SUC_lexicon_tags_2 },
    { 0xf345c228, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xac5c22a, SUC_lexicon_tags_10 },
    { 0xc285c22b, SUC_lexicon_tags_17 },
    { 0xdc05c22c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7a45c22e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46e5c230, SUC_lexicon_tags_18 },
    { 0xbe25c231, SUC_lexicon_tags_23 },
    { 0x86c5c230, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1345c239, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b85c23c, SUC_lexicon_tags_27 },
    { 0x6e45c23c, SUC_lexicon_tags_28 },
    { 0x6ee5c23e, SUC_lexicon_tags_1 },
    { 0x91c5c23f, SUC_lexicon_tags_11 },
    { 0xee65c23f, SUC_lexicon_tags_19 },
    { 0x91e5c241, SUC_lexicon_tags_15 },
    { 0x9465c23e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe5c248, SUC_lexicon_tags_46 },
    { 0x5a45c249, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7a5c24b, SUC_lexicon_tags_12 },
    { 0xa3e5c24c, SUC_lexicon_tags_1 },
    { 0x93e5c24c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c5c254, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc125c258, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a5c262, SUC_lexicon_tags_37 },
    { 0xab85c263, SUC_lexicon_tags_31 },
    { 0x81c5c262, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2845c267, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a5c26b, SUC_lexicon_tags_15 },
    { 0x95a5c26c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e5c273, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5285c277, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5c27f, SUC_lexicon_tags_27 },
    { 0x8a85c27f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f05c282, SUC_lexicon_tags_21 },
    { 0xb725c283, SUC_lexicon_tags_23 },
    { 0x75a5c284, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5525c289, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8125c293, SUC_lexicon_tags_7 },
    { 0x3ba5c293, SUC_lexicon_tags_29 },
    { 0x5de5c295, SUC_lexicon_tags_37 },
    { 0xbec5c296, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6105c299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8105c29e, SUC_lexicon_tags_28 },
    { 0x3a05c29f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc65c2a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e5c2a9, SUC_lexicon_tags_63 },
    { 0x1585c2aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2745c2ad, SUC_lexicon_tags_14 },
    { 0xc565c2ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3e05c2b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e25c2b4, SUC_lexicon_tags_4 },
    { 0xe725c2b5, SUC_lexicon_tags_63 },
    { 0x7445c2b6, SUC_lexicon_tags_9 },
    { 0x2745c2b7, SUC_lexicon_tags_19 },
    { 0x5f65c2b8, SUC_lexicon_tags_32 },
    { 0xb445c2b6, SUC_lexicon_tags_35 },
    { 0xc305c2ba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5605c2c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe2e5c2c2, SUC_lexicon_tags_60 },
    { 0x1065c2c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e5c2c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa965c2c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c5c2cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1265c2d0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3d85c2d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d65c2d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a65c2d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc5c2db, SUC_lexicon_tags_11 },
    { 0x8985c2dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb825c2e0, SUC_lexicon_tags_16 },
    { 0x71e5c2e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7425c2ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe525c2ef, SUC_lexicon_tags_18 },
    { 0xeaa5c2f0, SUC_lexicon_tags_2 },
    { 0xfa45c2ef, SUC_lexicon_tags_37 },
    { 0x40c5c2f2, SUC_lexicon_tags_12 },
    { 0x7a5c2f3, SUC_lexicon_tags_160 },
    { 0x81c5c2f4, SUC_lexicon_tags_174 },
    { 0xc4a5c2f3, SUC_lexicon_tags_15 },
    { 0xe3e5c2f2, SUC_lexicon_tags_63 },
    { 0xc225c2f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa25c2fb, SUC_lexicon_tags_38 },
    { 0x12c5c2fb, SUC_lexicon_tags_37 },
    { 0xa025c2fb, SUC_lexicon_tags_27 },
    { 0xe545c2fe, SUC_lexicon_tags_17 },
    { 0xbae5c2fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e5c302, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67e5c304, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd365c307, SUC_lexicon_tags_25 },
    { 0x71c5c307, SUC_lexicon_tags_7 },
    { 0xf6e5c307, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8e5c30b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a5c311, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc5c31d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c65c320, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcfa5c322, SUC_lexicon_tags_10 },
    { 0xddc5c323, SUC_lexicon_tags_23 },
    { 0xa465c324, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5305c327, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1585c339, SUC_lexicon_tags_66 },
    { 0x7185c339, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a05c33e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb185c343, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3545c345, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfdc5c347, SUC_lexicon_tags_230 },
    { 0xf645c348, SUC_lexicon_tags_7 },
    { 0x8185c349, SUC_lexicon_tags_26 },
    { 0xc005c34a, SUC_lexicon_tags_17 },
    { 0xb885c34b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a5c352, SUC_lexicon_tags_46 },
    { 0xb005c353, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a45c357, SUC_lexicon_tags_0 },
    { 0x4325c357, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d45c35b, SUC_lexicon_tags_11 },
    { 0xca85c35c, SUC_lexicon_tags_29 },
    { 0x2de5c35d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb725c362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4c5c364, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a5c36a, SUC_lexicon_tags_14 },
    { 0xed25c36b, SUC_lexicon_tags_0 },
    { 0x2a85c36b, SUC_lexicon_tags_11 },
    { 0x5045c36c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85c370, SUC_lexicon_tags_37 },
    { 0x70e5c371, SUC_lexicon_tags_77 },
    { 0x67c5c372, SUC_lexicon_tags_15 },
    { 0xe665c373, SUC_lexicon_tags_12 },
    { 0xd6e5c371, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x98e5c376, SUC_lexicon_tags_25 },
    { 0xa825c377, SUC_lexicon_tags_15 },
    { 0x8685c378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb425c37b, SUC_lexicon_tags_12 },
    { 0xb0e5c37c, SUC_lexicon_tags_1 },
    { 0xba5c37d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a45c381, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9f65c383, SUC_lexicon_tags_29 },
    { 0x1ca5c383, SUC_lexicon_tags_21 },
    { 0xc1a5c385, SUC_lexicon_tags_17 },
    { 0xc545c383, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3925c38e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca5c395, SUC_lexicon_tags_12 },
    { 0x1ca5c396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28c5c398, SUC_lexicon_tags_12 },
    { 0xba45c398, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee5c39c, SUC_lexicon_tags_32 },
    { 0xcac5c39c, SUC_lexicon_tags_11 },
    { 0xfe85c39c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a5c3a3, SUC_lexicon_tags_69 },
    { 0x5fe5c3a4, SUC_lexicon_tags_9 },
    { 0x9005c3a5, SUC_lexicon_tags_45 },
    { 0xb305c3a4, SUC_lexicon_tags_7 },
    { 0xb005c3a6, SUC_lexicon_tags_12 },
    { 0xc8e5c3a8, SUC_lexicon_tags_25 },
    { 0xf825c3a6, SUC_lexicon_tags_1 },
    { 0xba25c3a7, SUC_lexicon_tags_37 },
    { 0xe625c3a5, SUC_lexicon_tags_33 },
    { 0xade5c3ac, SUC_lexicon_tags_12 },
    { 0xd545c3ad, SUC_lexicon_tags_60 },
    { 0x8165c3ae, SUC_lexicon_tags_15 },
    { 0x31c5c3a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc65c3b1, SUC_lexicon_tags_39 },
    { 0x3205c3b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd85c3b8, SUC_lexicon_tags_47 },
    { 0xb545c3b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c5c3bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5c3c1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c5c3c8, SUC_lexicon_tags_6 },
    { 0xbaa5c3c8, SUC_lexicon_tags_49 },
    { 0x5ae5c3c9, SUC_lexicon_tags_26 },
    { 0x78e5c3c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685c3cf, SUC_lexicon_tags_16 },
    { 0x1685c3d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da5c3d8, SUC_lexicon_tags_2 },
    { 0x8b25c3d8, SUC_lexicon_tags_27 },
    { 0xb905c3d9, SUC_lexicon_tags_60 },
    { 0x75c5c3d9, SUC_lexicon_tags_37 },
    { 0xb3c5c3dc, SUC_lexicon_tags_7 },
    { 0x9a05c3dd, SUC_lexicon_tags_27 },
    { 0x5145c3dc, SUC_lexicon_tags_10 },
    { 0x5a25c3df, SUC_lexicon_tags_11 },
    { 0xf2c5c3d8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d45c3e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7205c3e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6665c3ec, SUC_lexicon_tags_32 },
    { 0xa0a5c3ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae5c3f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e5c3f5, SUC_lexicon_tags_25 },
    { 0x3ba5c3f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2425c3f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9345c3fc, SUC_lexicon_tags_47 },
    { 0xf85c3fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6565c401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37e5c403, SUC_lexicon_tags_12 },
    { 0xa3c5c403, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65c40a, SUC_lexicon_tags_15 },
    { 0x9c5c40a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5865c410, SUC_lexicon_tags_2 },
    { 0x62e5c411, SUC_lexicon_tags_12 },
    { 0x2805c411, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c5c414, SUC_lexicon_tags_17 },
    { 0x5405c414, SUC_lexicon_tags_27 },
    { 0xe245c416, SUC_lexicon_tags_0 },
    { 0x5c05c416, SUC_lexicon_tags_57 },
    { 0x6005c418, SUC_lexicon_tags_9 },
    { 0xb7e5c419, SUC_lexicon_tags_107 },
    { 0x5a05c415, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6265c41c, SUC_lexicon_tags_9 },
    { 0x3a45c41d, SUC_lexicon_tags_33 },
    { 0xe1e5c41d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf945c420, SUC_lexicon_tags_27 },
    { 0x72a5c421, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3425c423, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xda25c426, SUC_lexicon_tags_37 },
    { 0x2e5c427, SUC_lexicon_tags_63 },
    { 0xf605c427, SUC_lexicon_tags_11 },
    { 0x4de5c426, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc65c42b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57a5c42d, SUC_lexicon_tags_7 },
    { 0x6c45c42d, SUC_lexicon_tags_68 },
    { 0xb725c42f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f85c435, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae05c43a, SUC_lexicon_tags_11 },
    { 0x32c5c43b, SUC_lexicon_tags_12 },
    { 0x2da5c43b, SUC_lexicon_tags_12 },
    { 0xa6c5c43d, SUC_lexicon_tags_15 },
    { 0x8145c43e, SUC_lexicon_tags_53 },
    { 0x9665c43f, SUC_lexicon_tags_43 },
    { 0x6085c43d, SUC_lexicon_tags_15 },
    { 0x5ca5c43d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c25c44e, SUC_lexicon_tags_49 },
    { 0xd8a5c44e, SUC_lexicon_tags_63 },
    { 0x9605c450, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x265c453, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b05c45b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c5c45f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c5c463, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc25c469, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf65c471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc05c473, SUC_lexicon_tags_25 },
    { 0xc65c473, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xbca5c476, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a65c47d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6085c47f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3205c481, SUC_lexicon_tags_7 },
    { 0x9565c482, SUC_lexicon_tags_10 },
    { 0x75c5c482, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bc5c485, SUC_lexicon_tags_38 },
    { 0xb505c486, SUC_lexicon_tags_17 },
    { 0x26a5c487, SUC_lexicon_tags_49 },
    { 0xb365c488, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26c5c48a, SUC_lexicon_tags_7 },
    { 0x4a65c48b, SUC_lexicon_tags_9 },
    { 0xf125c48c, SUC_lexicon_tags_30 },
    { 0x7a25c48c, SUC_lexicon_tags_17 },
    { 0xfc65c48c, SUC_lexicon_tags_25 },
    { 0xf745c48f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9305c493, SUC_lexicon_tags_11 },
    { 0xcc85c493, SUC_lexicon_tags_39 },
    { 0xf45c495, SUC_lexicon_tags_37 },
    { 0x9905c494, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd25c498, SUC_lexicon_tags_12 },
    { 0x2da5c499, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d25c49d, SUC_lexicon_tags_12 },
    { 0x6e5c49e, SUC_lexicon_tags_25 },
    { 0x5ca5c49e, SUC_lexicon_tags_3 },
    { 0x4ba5c49e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fe5c4a2, SUC_lexicon_tags_31 },
    { 0x2f05c4a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a5c4a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x545c4ab, SUC_lexicon_tags_7 },
    { 0x805c4ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4fa5c4ae, SUC_lexicon_tags_9 },
    { 0xfce5c4af, SUC_lexicon_tags_46 },
    { 0xb805c4b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2c25c4b2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd705c4b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7925c4bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb665c4be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe225c4c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c85c4c6, SUC_lexicon_tags_32 },
    { 0xd6a5c4c6, SUC_lexicon_tags_82 },
    { 0xd2e5c4c8, SUC_lexicon_tags_49 },
    { 0xf305c4c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3265c4cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc105c4d0, SUC_lexicon_tags_19 },
    { 0xcce5c4d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6525c4d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd45c4d7, SUC_lexicon_tags_70 },
    { 0x5445c4d8, SUC_lexicon_tags_47 },
    { 0x1dc5c4d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf565c4dd, SUC_lexicon_tags_26 },
    { 0x925c4dd, SUC_lexicon_tags_7 },
    { 0xe885c4dd, SUC_lexicon_tags_63 },
    { 0xa565c4e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa25c4e2, SUC_lexicon_tags_67 },
    { 0x37c5c4e2, SUC_lexicon_tags_103 },
    { 0x4a05c4e4, SUC_lexicon_tags_40 },
    { 0x7e85c4e5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x75a5c4e7, SUC_lexicon_tags_11 },
    { 0xe365c4e8, SUC_lexicon_tags_34 },
    { 0x63a5c4e9, SUC_lexicon_tags_98 },
    { 0x965c4e9, SUC_lexicon_tags_23 },
    { 0x1f85c4e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8405c4f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8be5c4f8, SUC_lexicon_tags_66 },
    { 0x5325c4f8, SUC_lexicon_tags_427 },
    { 0x45e5c4fa, SUC_lexicon_tags_37 },
    { 0x7485c4fb, SUC_lexicon_tags_4 },
    { 0x2e85c4fb, SUC_lexicon_tags_25 },
    { 0x4925c4fd, SUC_lexicon_tags_12 },
    { 0x3c25c4fb, SUC_lexicon_tags_114 },
    { 0x8be5c4ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2885c502, SUC_lexicon_tags_7 },
    { 0xec5c503, SUC_lexicon_tags_38 },
    { 0x88c5c504, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x2005c506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5205c50a, SUC_lexicon_tags_17 },
    { 0xce65c50a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc5e5c50d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2565c519, SUC_lexicon_tags_37 },
    { 0x8f45c51a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc5c51d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe345c51f, SUC_lexicon_tags_26 },
    { 0x3f85c520, SUC_lexicon_tags_0 },
    { 0x3dc5c521, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xad85c524, SUC_lexicon_tags_69 },
    { 0xd265c525, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c5c52b, SUC_lexicon_tags_37 },
    { 0x405c52c, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b05c531, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b45c534, SUC_lexicon_tags_15 },
    { 0x4145c535, SUC_lexicon_tags_27 },
    { 0x2245c535, SUC_lexicon_tags_7 },
    { 0x80e5c537, SUC_lexicon_tags_25 },
    { 0x1be5c537, SUC_lexicon_tags_11 },
    { 0xbee5c534, SUC_lexicon_tags_47 },
    { 0xd905c53a, SUC_lexicon_tags_7 },
    { 0x6325c53b, SUC_lexicon_tags_11 },
    { 0x4d25c53a, SUC_lexicon_tags_8 },
    { 0x4e65c538, SUC_lexicon_tags_37 },
    { 0x2d85c53e, SUC_lexicon_tags_27 },
    { 0x67c5c53f, SUC_lexicon_tags_14 },
    { 0x9a25c53e, SUC_lexicon_tags_19 },
    { 0xf8a5c541, SUC_lexicon_tags_3 },
    { 0xe185c542, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe4a5c544, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8265c54a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce45c54c, SUC_lexicon_tags_19 },
    { 0xc7e5c54d, SUC_lexicon_tags_25 },
    { 0x8a5c54e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e5c558, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9005c55d, SUC_lexicon_tags_1 },
    { 0x2345c55d, SUC_lexicon_tags_23 },
    { 0xbae5c55d, SUC_lexicon_tags_43 },
    { 0xed05c560, SUC_lexicon_tags_23 },
    { 0x2345c560, SUC_lexicon_tags_14 },
    { 0x80a5c562, SUC_lexicon_tags_27 },
    { 0x87e5c562, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f05c568, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f85c56a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd865c570, SUC_lexicon_tags_14 },
    { 0x5fe5c571, SUC_lexicon_tags_576 },
    { 0xe865c572, SUC_lexicon_tags_17 },
    { 0xf0e5c572, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9225c576, SUC_lexicon_tags_37 },
    { 0x82a5c577, SUC_lexicon_tags_17 },
    { 0x2f85c577, SUC_lexicon_tags_2 },
    { 0x5905c578, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x90e5c57b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe5c57f, SUC_lexicon_tags_45 },
    { 0x7fa5c580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa85c583, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce5c586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc25c588, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e85c58a, SUC_lexicon_tags_11 },
    { 0x21a5c58b, SUC_lexicon_tags_7 },
    { 0xa645c58c, SUC_lexicon_tags_43 },
    { 0x5d25c58c, SUC_lexicon_tags_37 },
    { 0x3f65c58b, SUC_lexicon_tags_21 },
    { 0x6e45c58f, SUC_lexicon_tags_2 },
    { 0x8085c58f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc5c596, SUC_lexicon_tags_11 },
    { 0xa165c597, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ee5c599, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5065c5a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b65c5a3, SUC_lexicon_tags_30 },
    { 0x6e25c5a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c5c5a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a5c5ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1365c5b1, SUC_lexicon_tags_38 },
    { 0x9405c5b1, SUC_lexicon_tags_61 },
    { 0xcc5c5b3, SUC_lexicon_tags_1 },
    { 0x8a85c5b3, SUC_lexicon_tags_1 },
    { 0xa9c5c5b3, SUC_lexicon_tags_12 },
    { 0xb7a5c5b6, SUC_lexicon_tags_60 },
    { 0x40a5c5b6, SUC_lexicon_tags_2 },
    { 0x7225c5b8, SUC_lexicon_tags_25 },
    { 0x5a65c5b8, SUC_lexicon_tags_12 },
    { 0xd085c5b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb85c5bc, SUC_lexicon_tags_5 },
    { 0x78c5c5bc, SUC_lexicon_tags_19 },
    { 0x4245c5bc, SUC_lexicon_tags_32 },
    { 0xc325c5bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf645c5c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e5c5c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb8c5c5c6, SUC_lexicon_tags_234 },
    { 0x8105c5c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a45c5c9, SUC_lexicon_tags_24 },
    { 0x8c25c5ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa05c5d0, SUC_lexicon_tags_31 },
    { 0x6fc5c5d1, SUC_lexicon_tags_23 },
    { 0xe345c5d2, SUC_lexicon_tags_48 },
    { 0xe105c5d2, SUC_lexicon_tags_26 },
    { 0x6d65c5d4, SUC_lexicon_tags_7 },
    { 0xc665c5d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab65c5da, SUC_lexicon_tags_37 },
    { 0xb325c5db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6825c5de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe5c5ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca5c5ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a5c5f2, SUC_lexicon_tags_26 },
    { 0xe85c5f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c5c5f6, SUC_lexicon_tags_79 },
    { 0xdd45c5f7, SUC_lexicon_tags_33 },
    { 0xc0e5c5f7, SUC_lexicon_tags_25 },
    { 0x65c5c5f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b05c5fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5a5c5fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1325c600, SUC_lexicon_tags_29 },
    { 0x60a5c601, SUC_lexicon_tags_28 },
    { 0x6445c602, SUC_lexicon_tags_17 },
    { 0xc465c600, SUC_lexicon_tags_11 },
    { 0xd05c604, SUC_lexicon_tags_1 },
    { 0x94a5c604, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x525c607, SUC_lexicon_tags_32 },
    { 0x49a5c608, SUC_lexicon_tags_37 },
    { 0x3b25c609, SUC_lexicon_tags_55 },
    { 0x1325c60a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf85c60f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca5c615, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb25c617, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe445c619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e45c61b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb725c61d, SUC_lexicon_tags_63 },
    { 0x78a5c61e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e05c624, SUC_lexicon_tags_27 },
    { 0x8da5c625, SUC_lexicon_tags_12 },
    { 0x2545c626, SUC_lexicon_tags_7 },
    { 0x6dc5c627, SUC_lexicon_tags_11 },
    { 0x13e5c628, SUC_lexicon_tags_19 },
    { 0x8e45c628, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae5c62d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf285c62f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5585c632, SUC_lexicon_tags_23 },
    { 0xd9a5c633, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc645c635, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c5c639, SUC_lexicon_tags_38 },
    { 0x1845c63a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5125c63d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac5c641, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1985c644, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8d85c646, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7885c64c, SUC_lexicon_tags_1 },
    { 0xc425c64d, SUC_lexicon_tags_11 },
    { 0xd985c64e, SUC_lexicon_tags_27 },
    { 0x5ae5c64d, SUC_lexicon_tags_64 },
    { 0xf205c650, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e5c653, SUC_lexicon_tags_25 },
    { 0x1745c654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1205c65a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6e25c65c, SUC_lexicon_tags_17 },
    { 0x7a05c65c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4825c660, SUC_lexicon_tags_21 },
    { 0x30e5c661, SUC_lexicon_tags_11 },
    { 0xab45c661, SUC_lexicon_tags_4 },
    { 0x6e05c662, SUC_lexicon_tags_23 },
    { 0x1685c664, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb85c667, SUC_lexicon_tags_17 },
    { 0xcb85c668, SUC_lexicon_tags_17 },
    { 0xcac5c669, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xacc5c66b, SUC_lexicon_tags_7 },
    { 0xe2c5c66c, SUC_lexicon_tags_25 },
    { 0xe805c66d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e5c672, SUC_lexicon_tags_39 },
    { 0x82a5c672, SUC_lexicon_tags_87 },
    { 0xa185c673, SUC_lexicon_tags_60 },
    { 0xd2a5c674, SUC_lexicon_tags_6 },
    { 0x6945c672, SUC_lexicon_tags_179 },
    { 0x4ee5c672, SUC_lexicon_tags_23 },
    { 0x1da5c678, SUC_lexicon_tags_55 },
    { 0x8785c679, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc25c67b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd45c67d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba5c680, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6065c684, SUC_lexicon_tags_11 },
    { 0xe865c684, SUC_lexicon_tags_1 },
    { 0xef85c684, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1565c688, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a5c68b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe365c68d, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x8725c68f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4425c691, SUC_lexicon_tags_12 },
    { 0xf505c692, SUC_lexicon_tags_27 },
    { 0x39a5c692, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8665c69d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4a5c69f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5c6a4, SUC_lexicon_tags_32 },
    { 0xcca5c6a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d65c6a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e5c6ab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee65c6af, SUC_lexicon_tags_17 },
    { 0x9225c6af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb325c6b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa285c6b6, SUC_lexicon_tags_45 },
    { 0xd2e5c6b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c85c6bf, SUC_lexicon_tags_60 },
    { 0xefe5c6c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc5c6c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb385c6c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a5c6cd, SUC_lexicon_tags_34 },
    { 0xd945c6ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d45c6d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff25c6d3, SUC_lexicon_tags_1065 },
    { 0, NULL },
    { 0xb85c6d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2e5c6d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c6da, SUC_lexicon_tags_47 },
    { 0x84a5c6db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb805c6e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e5c6e7, SUC_lexicon_tags_25 },
    { 0xa265c6e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ba5c6ea, SUC_lexicon_tags_17 },
    { 0x5be5c6eb, SUC_lexicon_tags_27 },
    { 0x8805c6ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36e5c6ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e5c6f4, SUC_lexicon_tags_11 },
    { 0x13a5c6f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7e05c6f7, SUC_lexicon_tags_45 },
    { 0xbc45c6f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c45c6fe, SUC_lexicon_tags_17 },
    { 0xf05c6ff, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa4e5c701, SUC_lexicon_tags_28 },
    { 0x4045c702, SUC_lexicon_tags_92 },
    { 0x2905c703, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c5c70d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a5c713, SUC_lexicon_tags_17 },
    { 0x89a5c714, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9025c716, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71a5c718, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x9b45c71a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x785c71d, SUC_lexicon_tags_7 },
    { 0x1485c71d, SUC_lexicon_tags_37 },
    { 0x3405c71d, SUC_lexicon_tags_37 },
    { 0x9705c720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a5c723, SUC_lexicon_tags_17 },
    { 0x97a5c724, SUC_lexicon_tags_7 },
    { 0xfe65c725, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdc5c727, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x46a5c729, SUC_lexicon_tags_55 },
    { 0xd9a5c72a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d85c72f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea5c733, SUC_lexicon_tags_1 },
    { 0x425c733, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6745c738, SUC_lexicon_tags_38 },
    { 0xcf65c739, SUC_lexicon_tags_8 },
    { 0x96a5c739, SUC_lexicon_tags_45 },
    { 0xc7a5c738, SUC_lexicon_tags_87 },
    { 0xaae5c73c, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea5c740, SUC_lexicon_tags_23 },
    { 0xaea5c741, SUC_lexicon_tags_60 },
    { 0x81c5c742, SUC_lexicon_tags_14 },
    { 0x3525c743, SUC_lexicon_tags_46 },
    { 0x3665c740, SUC_lexicon_tags_11 },
    { 0xb0a5c745, SUC_lexicon_tags_33 },
    { 0xb885c745, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a5c749, SUC_lexicon_tags_19 },
    { 0x4e05c749, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1e5c74c, SUC_lexicon_tags_26 },
    { 0x7925c74d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x25c74f, SUC_lexicon_tags_14 },
    { 0xde25c750, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3125c753, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83e5c755, SUC_lexicon_tags_15 },
    { 0x3705c755, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc245c759, SUC_lexicon_tags_1225 },
    { 0x3da5c75a, SUC_lexicon_tags_25 },
    { 0x7b45c75b, SUC_lexicon_tags_47 },
    { 0x2765c75b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a45c760, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xee65c763, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a5c766, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a65c769, SUC_lexicon_tags_21 },
    { 0xebc5c76a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e5c771, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e5c774, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf185c776, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c5c77b, SUC_lexicon_tags_57 },
    { 0xed65c77c, SUC_lexicon_tags_0 },
    { 0x1e45c77d, SUC_lexicon_tags_11 },
    { 0x29c5c77d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca5c786, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74a5c788, SUC_lexicon_tags_12 },
    { 0x1f85c789, SUC_lexicon_tags_4 },
    { 0xfe85c78a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e5c790, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a5c793, SUC_lexicon_tags_49 },
    { 0x5f05c794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb025c79a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc905c7a0, SUC_lexicon_tags_25 },
    { 0x2525c7a1, SUC_lexicon_tags_29 },
    { 0xa345c7a2, SUC_lexicon_tags_7 },
    { 0x6425c7a2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c5c7a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85c7ab, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe665c7b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa85c7b6, SUC_lexicon_tags_15 },
    { 0x6c85c7b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe665c7b9, SUC_lexicon_tags_21 },
    { 0x3005c7ba, SUC_lexicon_tags_27 },
    { 0x4365c7bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a5c7be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ce5c7c0, SUC_lexicon_tags_60 },
    { 0x8565c7c0, SUC_lexicon_tags_25 },
    { 0xb325c7c2, SUC_lexicon_tags_7 },
    { 0x5125c7c2, SUC_lexicon_tags_1 },
    { 0xb665c7c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb05c7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3e5c7c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5c7cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c5c7d2, SUC_lexicon_tags_62 },
    { 0x42c5c7d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb45c7d5, SUC_lexicon_tags_39 },
    { 0xe1a5c7d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8545c7d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fe5c7da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f05c7e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c7e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc5c7ea, SUC_lexicon_tags_6 },
    { 0xba05c7eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c5c7ee, SUC_lexicon_tags_14 },
    { 0x8f25c7ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc5c7f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4125c7f6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a5c7fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa25c802, SUC_lexicon_tags_55 },
    { 0x4d65c803, SUC_lexicon_tags_85 },
    { 0x74a5c803, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1425c807, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2145c80b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da5c80f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdee5c811, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc5c815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf585c818, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc365c81c, SUC_lexicon_tags_15 },
    { 0x6145c81d, SUC_lexicon_tags_15 },
    { 0xe8c5c81d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4985c825, SUC_lexicon_tags_65 },
    { 0x8885c826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e5c829, SUC_lexicon_tags_39 },
    { 0xfe45c82a, SUC_lexicon_tags_33 },
    { 0x66e5c82b, SUC_lexicon_tags_17 },
    { 0xe165c82a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf45c836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd25c83c, SUC_lexicon_tags_18 },
    { 0x2b25c83d, SUC_lexicon_tags_11 },
    { 0xb4a5c83d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7105c841, SUC_lexicon_tags_26 },
    { 0xf2e5c842, SUC_lexicon_tags_11 },
    { 0x1725c842, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab45c847, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf45c84a, SUC_lexicon_tags_14 },
    { 0x7a85c84a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9da5c84d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8045c850, SUC_lexicon_tags_0 },
    { 0x96e5c851, SUC_lexicon_tags_12 },
    { 0x33c5c852, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05c855, SUC_lexicon_tags_2 },
    { 0x9885c856, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e5c859, SUC_lexicon_tags_7 },
    { 0x8725c859, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7845c85c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x41e5c85e, SUC_lexicon_tags_26 },
    { 0x83c5c85e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a5c867, SUC_lexicon_tags_2 },
    { 0xbf25c868, SUC_lexicon_tags_15 },
    { 0x3ce5c869, SUC_lexicon_tags_45 },
    { 0xd05c868, SUC_lexicon_tags_32 },
    { 0x94c5c86a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5c86e, SUC_lexicon_tags_23 },
    { 0xb405c86f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e5c873, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5805c877, SUC_lexicon_tags_23 },
    { 0x4b85c877, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c65c87c, SUC_lexicon_tags_14 },
    { 0x70e5c87d, SUC_lexicon_tags_15 },
    { 0x2185c87c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd385c880, SUC_lexicon_tags_7 },
    { 0x20e5c881, SUC_lexicon_tags_7 },
    { 0xda65c881, SUC_lexicon_tags_15 },
    { 0xd365c883, SUC_lexicon_tags_37 },
    { 0x7745c884, SUC_lexicon_tags_26 },
    { 0x9925c885, SUC_lexicon_tags_25 },
    { 0x6bc5c886, SUC_lexicon_tags_23 },
    { 0x1e05c886, SUC_lexicon_tags_15 },
    { 0xae5c888, SUC_lexicon_tags_77 },
    { 0x9845c883, SUC_lexicon_tags_15 },
    { 0xe965c88a, SUC_lexicon_tags_11 },
    { 0x2625c880, SUC_lexicon_tags_27 },
    { 0xa585c88c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f65c891, SUC_lexicon_tags_2 },
    { 0xc685c892, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e5c897, SUC_lexicon_tags_0 },
    { 0xf3c5c898, SUC_lexicon_tags_25 },
    { 0x42c5c897, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9205c89d, SUC_lexicon_tags_28 },
    { 0xbde5c89e, SUC_lexicon_tags_37 },
    { 0x50e5c89f, SUC_lexicon_tags_14 },
    { 0xeac5c8a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf445c8a5, SUC_lexicon_tags_15 },
    { 0x2585c8a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae5c8ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3785c8b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5805c8b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d05c8ba, SUC_lexicon_tags_17 },
    { 0x2f85c8bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5f45c8bd, SUC_lexicon_tags_230 },
    { 0xce65c8be, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c05c8c1, SUC_lexicon_tags_6 },
    { 0x20c5c8c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38c5c8c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9685c8c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2de5c8c9, SUC_lexicon_tags_87 },
    { 0x1a45c8c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9565c8ce, SUC_lexicon_tags_7 },
    { 0x51e5c8ce, SUC_lexicon_tags_103 },
    { 0x1665c8d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa765c8d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a5c8d8, SUC_lexicon_tags_17 },
    { 0x1a5c8d9, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9465c8e0, SUC_lexicon_tags_35 },
    { 0xa865c8e1, SUC_lexicon_tags_25 },
    { 0xa0e5c8e0, SUC_lexicon_tags_7 },
    { 0x9905c8e3, SUC_lexicon_tags_45 },
    { 0xf065c8e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b25c8e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba45c8e8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9a05c8ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37a5c8ec, SUC_lexicon_tags_30 },
    { 0x41a5c8ed, SUC_lexicon_tags_39 },
    { 0x5de5c8ee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4265c8f0, SUC_lexicon_tags_12 },
    { 0x52a5c8f0, SUC_lexicon_tags_31 },
    { 0x6245c8f1, SUC_lexicon_tags_0 },
    { 0x49e5c8f3, SUC_lexicon_tags_123 },
    { 0x1825c8f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4425c8f6, SUC_lexicon_tags_28 },
    { 0x4ba5c8f7, SUC_lexicon_tags_6 },
    { 0x3e85c8f7, SUC_lexicon_tags_151 },
    { 0x6ea5c8f8, SUC_lexicon_tags_23 },
    { 0xff25c8fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4605c8ff, SUC_lexicon_tags_12 },
    { 0xce85c8ff, SUC_lexicon_tags_26 },
    { 0x85e5c900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9225c906, SUC_lexicon_tags_84 },
    { 0x3085c906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea5c90a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1225c90c, SUC_lexicon_tags_7 },
    { 0x2025c90c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab25c912, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9165c915, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4565c91e, SUC_lexicon_tags_27 },
    { 0x945c91e, SUC_lexicon_tags_25 },
    { 0x5165c91e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1425c923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685c92b, SUC_lexicon_tags_31 },
    { 0xb1a5c92c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8945c931, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55e5c933, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c85c93c, SUC_lexicon_tags_37 },
    { 0xf2a5c93c, SUC_lexicon_tags_19 },
    { 0x6425c93e, SUC_lexicon_tags_49 },
    { 0x5a45c93f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e5c945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe885c947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1225c956, SUC_lexicon_tags_453 },
    { 0xbf85c957, SUC_lexicon_tags_230 },
    { 0x3c45c956, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc465c95d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8745c95f, SUC_lexicon_tags_32 },
    { 0x9ca5c960, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7d65c962, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c5c976, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1045c978, SUC_lexicon_tags_11 },
    { 0x5485c979, SUC_lexicon_tags_11 },
    { 0xc185c97a, SUC_lexicon_tags_43 },
    { 0xb0e5c97a, SUC_lexicon_tags_28 },
    { 0x11a5c97a, SUC_lexicon_tags_14 },
    { 0x25c97d, SUC_lexicon_tags_68 },
    { 0x1305c97c, SUC_lexicon_tags_23 },
    { 0xde5c97f, SUC_lexicon_tags_1 },
    { 0x89a5c980, SUC_lexicon_tags_38 },
    { 0xc665c981, SUC_lexicon_tags_12 },
    { 0x31c5c97d, SUC_lexicon_tags_11 },
    { 0x7c65c97a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2f85c985, SUC_lexicon_tags_15 },
    { 0x1685c985, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a5c989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a45c98c, SUC_lexicon_tags_46 },
    { 0xf685c98d, SUC_lexicon_tags_2 },
    { 0x1565c98e, SUC_lexicon_tags_26 },
    { 0xe9a5c98f, SUC_lexicon_tags_17 },
    { 0xe1a5c990, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe045c994, SUC_lexicon_tags_37 },
    { 0xeb25c994, SUC_lexicon_tags_12 },
    { 0x13e5c995, SUC_lexicon_tags_25 },
    { 0x1f45c997, SUC_lexicon_tags_27 },
    { 0xa745c998, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e5c99d, SUC_lexicon_tags_47 },
    { 0x7d45c99d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba5c9a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a45c9aa, SUC_lexicon_tags_60 },
    { 0xa985c9ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8365c9b0, SUC_lexicon_tags_25 },
    { 0x33c5c9b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a05c9b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcc5c9ba, SUC_lexicon_tags_2 },
    { 0x6aa5c9ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d85c9be, SUC_lexicon_tags_1 },
    { 0x8365c9bf, SUC_lexicon_tags_2 },
    { 0x7825c9c0, SUC_lexicon_tags_17 },
    { 0xb005c9c1, SUC_lexicon_tags_11 },
    { 0xdd85c9c1, SUC_lexicon_tags_12 },
    { 0x9485c9c3, SUC_lexicon_tags_29 },
    { 0x9d85c9c1, SUC_lexicon_tags_301 },
    { 0x1d45c9c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6365c9c7, SUC_lexicon_tags_37 },
    { 0xc4e5c9c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf25c9cb, SUC_lexicon_tags_63 },
    { 0x8325c9cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe365c9cf, SUC_lexicon_tags_25 },
    { 0x5dc5c9cf, SUC_lexicon_tags_48 },
    { 0xe0e5c9d1, SUC_lexicon_tags_49 },
    { 0x6ea5c9d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec5c9d6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4705c9db, SUC_lexicon_tags_55 },
    { 0x6ec5c9db, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7365c9e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc265c9e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a5c9ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeec5c9ee, SUC_lexicon_tags_37 },
    { 0xe325c9ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc745c9f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d85c9f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c65c9fa, SUC_lexicon_tags_45 },
    { 0xd6c5c9fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe65c9fd, SUC_lexicon_tags_15 },
    { 0xc285c9fe, SUC_lexicon_tags_37 },
    { 0x1f25c9ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2145ca02, SUC_lexicon_tags_25 },
    { 0x2185ca02, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcfa5ca05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a25ca09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14a5ca0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa5ca10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9645ca12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26a5ca14, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce5ca17, SUC_lexicon_tags_13 },
    { 0xa525ca18, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c5ca1b, SUC_lexicon_tags_11 },
    { 0x8ec5ca1b, SUC_lexicon_tags_96 },
    { 0x7ec5ca1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c65ca22, SUC_lexicon_tags_32 },
    { 0x4025ca22, SUC_lexicon_tags_16 },
    { 0xcf25ca22, SUC_lexicon_tags_25 },
    { 0x1285ca25, SUC_lexicon_tags_23 },
    { 0xf285ca24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc025ca28, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9885ca2d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf965ca2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9465ca33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a5ca37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6925ca39, SUC_lexicon_tags_37 },
    { 0xff85ca39, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e5ca3d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb425ca3f, SUC_lexicon_tags_17 },
    { 0xd205ca3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf65ca44, SUC_lexicon_tags_62 },
    { 0x9025ca45, SUC_lexicon_tags_29 },
    { 0xa5a5ca46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab05ca49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e85ca4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82e5ca4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45ca51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5265ca57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb625ca5d, SUC_lexicon_tags_46 },
    { 0xf25ca5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b45ca61, SUC_lexicon_tags_336 },
    { 0x3665ca62, SUC_lexicon_tags_32 },
    { 0xc165ca63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd545ca65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c45ca6d, SUC_lexicon_tags_33 },
    { 0x80a5ca6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4dc5ca70, SUC_lexicon_tags_25 },
    { 0xdda5ca71, SUC_lexicon_tags_4 },
    { 0xf785ca72, SUC_lexicon_tags_87 },
    { 0xd9e5ca72, SUC_lexicon_tags_47 },
    { 0xaf65ca71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6a5ca76, SUC_lexicon_tags_32 },
    { 0xb805ca77, SUC_lexicon_tags_15 },
    { 0x9285ca78, SUC_lexicon_tags_29 },
    { 0xbcc5ca78, SUC_lexicon_tags_25 },
    { 0x965ca7a, SUC_lexicon_tags_37 },
    { 0x3465ca7a, SUC_lexicon_tags_11 },
    { 0x8d85ca7b, SUC_lexicon_tags_10 },
    { 0x79e5ca7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf45ca84, SUC_lexicon_tags_25 },
    { 0xa7c5ca84, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf585ca87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x59c5ca89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa65ca8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd825ca8f, SUC_lexicon_tags_34 },
    { 0x26a5ca90, SUC_lexicon_tags_34 },
    { 0x6ee5ca91, SUC_lexicon_tags_31 },
    { 0xafc5ca91, SUC_lexicon_tags_37 },
    { 0x43c5ca90, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1965ca99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd7c5ca9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9da5ca9d, SUC_lexicon_tags_17 },
    { 0x57c5ca9e, SUC_lexicon_tags_15 },
    { 0x52e5ca9d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf325caa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6645caa3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7345caa8, SUC_lexicon_tags_25 },
    { 0x9ba5caa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5305caab, SUC_lexicon_tags_1 },
    { 0x5b45caac, SUC_lexicon_tags_519 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c5cab0, SUC_lexicon_tags_26 },
    { 0x1085cab0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd505cab5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1545cab8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a65cabe, SUC_lexicon_tags_49 },
    { 0xe3a5cabf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb585cac8, SUC_lexicon_tags_12 },
    { 0xd1c5cac9, SUC_lexicon_tags_44 },
    { 0x8dc5caca, SUC_lexicon_tags_14 },
    { 0x9a05cac9, SUC_lexicon_tags_2 },
    { 0xa5e5caca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb385cad3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed45cad7, SUC_lexicon_tags_7 },
    { 0x8aa5cad7, SUC_lexicon_tags_6 },
    { 0x345cad9, SUC_lexicon_tags_48 },
    { 0x58a5cad7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf245cadf, SUC_lexicon_tags_7 },
    { 0x1325cadf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc5cae3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x53c5cae5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5ba5cae7, SUC_lexicon_tags_61 },
    { 0x345cae7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a5caee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28c5caf0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7ea5caf2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d05caf7, SUC_lexicon_tags_8 },
    { 0x7245caf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f25cafc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6805cafe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c5cb0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xadc5cb0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5125cb0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a5cb14, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x3dc5cb16, SUC_lexicon_tags_8 },
    { 0x365cb17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa165cb1f, SUC_lexicon_tags_47 },
    { 0xd105cb20, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc485cb27, SUC_lexicon_tags_63 },
    { 0x2b45cb28, SUC_lexicon_tags_37 },
    { 0x19e5cb28, SUC_lexicon_tags_43 },
    { 0x8085cb2a, SUC_lexicon_tags_29 },
    { 0x3565cb2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa65cb2d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa25cb30, SUC_lexicon_tags_39 },
    { 0x6ca5cb31, SUC_lexicon_tags_17 },
    { 0xc6a5cb30, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb005cb38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c5cb3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf3e5cb3d, SUC_lexicon_tags_56 },
    { 0x9cc5cb3e, SUC_lexicon_tags_2 },
    { 0x64c5cb3f, SUC_lexicon_tags_68 },
    { 0x53c5cb40, SUC_lexicon_tags_33 },
    { 0x5d25cb41, SUC_lexicon_tags_17 },
    { 0xf9c5cb40, SUC_lexicon_tags_45 },
    { 0xf485cb42, SUC_lexicon_tags_91 },
    { 0x8245cb44, SUC_lexicon_tags_30 },
    { 0xefe5cb45, SUC_lexicon_tags_87 },
    { 0x3845cb46, SUC_lexicon_tags_2 },
    { 0x5165cb42, SUC_lexicon_tags_27 },
    { 0x5485cb47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6125cb4a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e85cb4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d65cb50, SUC_lexicon_tags_39 },
    { 0x2825cb51, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe5cb56, SUC_lexicon_tags_29 },
    { 0x2f85cb57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb385cb59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6485cb5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1dc5cb5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2885cb68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8765cb6b, SUC_lexicon_tags_2 },
    { 0x2405cb6c, SUC_lexicon_tags_14 },
    { 0x7b25cb6d, SUC_lexicon_tags_7 },
    { 0x93e5cb6d, SUC_lexicon_tags_67 },
    { 0x84a5cb6f, SUC_lexicon_tags_37 },
    { 0xe5a5cb6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c65cb72, SUC_lexicon_tags_35 },
    { 0x425cb73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc425cb77, SUC_lexicon_tags_26 },
    { 0xb5c5cb78, SUC_lexicon_tags_15 },
    { 0xbec5cb78, SUC_lexicon_tags_2 },
    { 0x80e5cb7a, SUC_lexicon_tags_15 },
    { 0xf865cb77, SUC_lexicon_tags_8 },
    { 0x97e5cb7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb25cb7e, SUC_lexicon_tags_11 },
    { 0xe305cb7f, SUC_lexicon_tags_13 },
    { 0x9ea5cb80, SUC_lexicon_tags_19 },
    { 0xaa85cb80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1c5cb83, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9845cb86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc005cb8b, SUC_lexicon_tags_4 },
    { 0xc5cb8c, SUC_lexicon_tags_84 },
    { 0xb0c5cb8d, SUC_lexicon_tags_25 },
    { 0x2205cb8c, SUC_lexicon_tags_8 },
    { 0x8725cb8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd125cb92, SUC_lexicon_tags_2 },
    { 0x2c65cb92, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xe45cb95, SUC_lexicon_tags_15 },
    { 0x8345cb96, SUC_lexicon_tags_2 },
    { 0xd8a5cb97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea5cb9b, SUC_lexicon_tags_23 },
    { 0xe0a5cb9b, SUC_lexicon_tags_12 },
    { 0xb6c5cb9d, SUC_lexicon_tags_19 },
    { 0x6105cb9c, SUC_lexicon_tags_2 },
    { 0xb1e5cb9f, SUC_lexicon_tags_66 },
    { 0x2e45cb9e, SUC_lexicon_tags_25 },
    { 0xc3e5cba1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c65cba5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e05cba7, SUC_lexicon_tags_8 },
    { 0xfd45cba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9205cbaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7425cbad, SUC_lexicon_tags_8 },
    { 0x29a5cbad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee85cbb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9605cbba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca5cbbf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c5cbc2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb85cbc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a5cbc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c5cbca, SUC_lexicon_tags_473 },
    { 0x8e25cbcb, SUC_lexicon_tags_79 },
    { 0xbde5cbcc, SUC_lexicon_tags_25 },
    { 0x1965cbcc, SUC_lexicon_tags_12 },
    { 0xe825cbce, SUC_lexicon_tags_15 },
    { 0xd245cbcd, SUC_lexicon_tags_17 },
    { 0x2625cbd0, SUC_lexicon_tags_27 },
    { 0xe2a5cbd1, SUC_lexicon_tags_95 },
    { 0x5be5cbd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa85cbd9, SUC_lexicon_tags_1 },
    { 0xa2c5cbda, SUC_lexicon_tags_17 },
    { 0x9aa5cbdb, SUC_lexicon_tags_7 },
    { 0xf8e5cbda, SUC_lexicon_tags_2 },
    { 0x18c5cbdd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8cc5cbdf, SUC_lexicon_tags_7 },
    { 0x9645cbe0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6565cbe3, SUC_lexicon_tags_11 },
    { 0xccc5cbe4, SUC_lexicon_tags_1 },
    { 0x3125cbe5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda45cbed, SUC_lexicon_tags_49 },
    { 0xedc5cbee, SUC_lexicon_tags_33 },
    { 0xd265cbef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2485cbf1, SUC_lexicon_tags_7 },
    { 0x47e5cbf1, SUC_lexicon_tags_1 },
    { 0x3b45cbf3, SUC_lexicon_tags_55 },
    { 0x2a5cbf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafc5cbf6, SUC_lexicon_tags_87 },
    { 0x85a5cbf7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c85cbf9, SUC_lexicon_tags_27 },
    { 0x96a5cbfa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc65cbfe, SUC_lexicon_tags_25 },
    { 0x6ac5cbfe, SUC_lexicon_tags_2 },
    { 0x6645cbfe, SUC_lexicon_tags_31 },
    { 0x7425cbff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e5cc07, SUC_lexicon_tags_27 },
    { 0x4e65cc08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b65cc0c, SUC_lexicon_tags_45 },
    { 0x3e65cc0c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7045cc17, SUC_lexicon_tags_11 },
    { 0xb185cc18, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x54a5cc1a, SUC_lexicon_tags_1 },
    { 0xa765cc1b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c5cc1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4725cc21, SUC_lexicon_tags_157 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b05cc26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e5cc2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa5cc38, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf825cc3f, SUC_lexicon_tags_9 },
    { 0xb905cc40, SUC_lexicon_tags_11 },
    { 0x9c05cc41, SUC_lexicon_tags_4 },
    { 0xeee5cc42, SUC_lexicon_tags_4 },
    { 0xf705cc42, SUC_lexicon_tags_28 },
    { 0x7aa5cc44, SUC_lexicon_tags_19 },
    { 0xeb85cc41, SUC_lexicon_tags_11 },
    { 0xe7e5cc43, SUC_lexicon_tags_29 },
    { 0xa625cc47, SUC_lexicon_tags_14 },
    { 0x6a5cc45, SUC_lexicon_tags_2 },
    { 0x1425cc40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c5cc4f, SUC_lexicon_tags_82 },
    { 0x9625cc50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8445cc52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a5cc5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa565cc5e, SUC_lexicon_tags_178 },
    { 0x2d25cc5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea5cc67, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x84e5cc69, SUC_lexicon_tags_7 },
    { 0xbbc5cc6a, SUC_lexicon_tags_2 },
    { 0xad85cc6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e45cc72, SUC_lexicon_tags_17 },
    { 0xdde5cc73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f85cc76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c5cc7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe085cc7f, SUC_lexicon_tags_0 },
    { 0xe045cc80, SUC_lexicon_tags_19 },
    { 0x1505cc81, SUC_lexicon_tags_1 },
    { 0x1205cc81, SUC_lexicon_tags_33 },
    { 0x6525cc82, SUC_lexicon_tags_2 },
    { 0xe085cc84, SUC_lexicon_tags_87 },
    { 0x87c5cc85, SUC_lexicon_tags_43 },
    { 0x8285cc86, SUC_lexicon_tags_9 },
    { 0x5b85cc85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4465cc89, SUC_lexicon_tags_15 },
    { 0x9945cc89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd445cc8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c5cc90, SUC_lexicon_tags_25 },
    { 0xf805cc91, SUC_lexicon_tags_12 },
    { 0xc4e5cc90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1125cc94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2005cc96, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc5cc99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b85cc9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c05cca2, SUC_lexicon_tags_26 },
    { 0x4485cca3, SUC_lexicon_tags_12 },
    { 0x1ce5cca2, SUC_lexicon_tags_45 },
    { 0x6045cca4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5cca8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5165ccb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5985ccb4, SUC_lexicon_tags_92 },
    { 0x7705ccb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6925ccb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5d85ccbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3245ccbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a65ccc1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9165ccc3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c5ccc7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f25ccc9, SUC_lexicon_tags_19 },
    { 0xc45ccc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff65ccd0, SUC_lexicon_tags_62 },
    { 0x8f25ccd0, SUC_lexicon_tags_17 },
    { 0x8ea5ccd2, SUC_lexicon_tags_17 },
    { 0xd9e5ccd2, SUC_lexicon_tags_66 },
    { 0xa2e5ccd4, SUC_lexicon_tags_25 },
    { 0xb385ccd4, SUC_lexicon_tags_13 },
    { 0x9f05ccd4, SUC_lexicon_tags_33 },
    { 0x4105ccd7, SUC_lexicon_tags_7 },
    { 0xa1e5ccd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf05ccda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9585cce0, SUC_lexicon_tags_25 },
    { 0x28e5cce1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe625cce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65cce6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5cceb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5665ccf1, SUC_lexicon_tags_42 },
    { 0x8745ccf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da5ccf7, SUC_lexicon_tags_38 },
    { 0xae85ccf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc005ccfa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2265ccfc, SUC_lexicon_tags_40 },
    { 0x1385ccfd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e5cd00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5cd06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba5cd0b, SUC_lexicon_tags_1 },
    { 0x95e5cd0c, SUC_lexicon_tags_13 },
    { 0x4a85cd0c, SUC_lexicon_tags_12 },
    { 0x97a5cd0e, SUC_lexicon_tags_26 },
    { 0xed45cd0f, SUC_lexicon_tags_2 },
    { 0xe885cd0d, SUC_lexicon_tags_12 },
    { 0xa405cd11, SUC_lexicon_tags_106 },
    { 0x3e45cd12, SUC_lexicon_tags_13 },
    { 0x57e5cd12, SUC_lexicon_tags_13 },
    { 0xcd25cd12, SUC_lexicon_tags_7 },
    { 0x4505cd13, SUC_lexicon_tags_37 },
    { 0xffc5cd16, SUC_lexicon_tags_11 },
    { 0x6125cd16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e65cd19, SUC_lexicon_tags_2 },
    { 0xa7a5cd1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3625cd1f, SUC_lexicon_tags_43 },
    { 0x1dc5cd20, SUC_lexicon_tags_23 },
    { 0x3fc5cd20, SUC_lexicon_tags_9 },
    { 0xdc05cd22, SUC_lexicon_tags_4 },
    { 0x7aa5cd23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd325cd27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5545cd29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad85cd2b, SUC_lexicon_tags_26 },
    { 0x5e25cd2c, SUC_lexicon_tags_28 },
    { 0x2205cd2b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc605cd2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73a5cd31, SUC_lexicon_tags_57 },
    { 0x9605cd32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4025cd34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda5cd3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd345cd3c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb365cd3f, SUC_lexicon_tags_33 },
    { 0x8b05cd3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60c5cd42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8205cd47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f65cd49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8605cd51, SUC_lexicon_tags_15 },
    { 0x3da5cd51, SUC_lexicon_tags_9 },
    { 0x6a05cd51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7825cd59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc245cd5b, SUC_lexicon_tags_38 },
    { 0x4e5cd5c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c05cd60, SUC_lexicon_tags_15 },
    { 0x15e5cd61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc265cd65, SUC_lexicon_tags_1 },
    { 0xf1c5cd66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9045cd70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd885cd72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20a5cd74, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7225cd7c, SUC_lexicon_tags_2 },
    { 0x87e5cd7c, SUC_lexicon_tags_68 },
    { 0xf525cd7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb945cd81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec45cd85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf505cd87, SUC_lexicon_tags_4 },
    { 0x9e45cd88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef25cd8a, SUC_lexicon_tags_55 },
    { 0x6305cd8a, SUC_lexicon_tags_34 },
    { 0x2a05cd8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d45cd8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd285cd91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf85cd95, SUC_lexicon_tags_25 },
    { 0x1425cd96, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c5cd99, SUC_lexicon_tags_6 },
    { 0x7825cd9a, SUC_lexicon_tags_49 },
    { 0x4805cd9b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea25cda0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf785cda3, SUC_lexicon_tags_37 },
    { 0x40e5cda3, SUC_lexicon_tags_11 },
    { 0xd8e5cda3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd345cda7, SUC_lexicon_tags_31 },
    { 0x95a5cda8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5cdab, SUC_lexicon_tags_9 },
    { 0xfc25cdab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8185cdb0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4905cdb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4285cdb7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a5cdbc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7325cdc2, SUC_lexicon_tags_25 },
    { 0xb4a5cdc2, SUC_lexicon_tags_38 },
    { 0x1d85cdc3, SUC_lexicon_tags_25 },
    { 0x2c85cdc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b45cdca, SUC_lexicon_tags_15 },
    { 0xc8a5cdca, SUC_lexicon_tags_19 },
    { 0xf4e5cdcc, SUC_lexicon_tags_6 },
    { 0xe985cdca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe2e5cdcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d65cdd1, SUC_lexicon_tags_26 },
    { 0x51c5cdd2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e5cdd6, SUC_lexicon_tags_37 },
    { 0x625cdd6, SUC_lexicon_tags_47 },
    { 0xda05cdd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d25cde6, SUC_lexicon_tags_12 },
    { 0x1965cde6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a25cdeb, SUC_lexicon_tags_47 },
    { 0xce05cdeb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9405cdef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc9a5cdf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc65cdf4, SUC_lexicon_tags_11 },
    { 0xfae5cdf5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9665cdfb, SUC_lexicon_tags_148 },
    { 0xb045cdfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc065cdff, SUC_lexicon_tags_15 },
    { 0xbba5ce00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e05ce02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1ac5ce04, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2565ce06, SUC_lexicon_tags_12 },
    { 0x1505ce07, SUC_lexicon_tags_1 },
    { 0x4585ce08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf225ce0b, SUC_lexicon_tags_37 },
    { 0x88a5ce0c, SUC_lexicon_tags_35 },
    { 0x8825ce0d, SUC_lexicon_tags_8 },
    { 0x2c05ce0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de5ce11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d05ce1a, SUC_lexicon_tags_11 },
    { 0xa7a5ce1a, SUC_lexicon_tags_37 },
    { 0x1c5ce1c, SUC_lexicon_tags_1 },
    { 0x9305ce1b, SUC_lexicon_tags_2 },
    { 0x98e5ce1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f25ce21, SUC_lexicon_tags_11 },
    { 0x96a5ce22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfee5ce24, SUC_lexicon_tags_37 },
    { 0x3145ce25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc5ce29, SUC_lexicon_tags_1 },
    { 0x65e5ce29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1365ce2d, SUC_lexicon_tags_1 },
    { 0x95c5ce2e, SUC_lexicon_tags_7 },
    { 0xadc5ce2e, SUC_lexicon_tags_45 },
    { 0xc525ce2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b65ce34, SUC_lexicon_tags_60 },
    { 0x9bc5ce34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c5ce3c, SUC_lexicon_tags_15 },
    { 0x3105ce3c, SUC_lexicon_tags_17 },
    { 0x6dc5ce3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb45ce41, SUC_lexicon_tags_37 },
    { 0x985ce41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c25ce44, SUC_lexicon_tags_4 },
    { 0xe805ce45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a5ce4f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc405ce53, SUC_lexicon_tags_17 },
    { 0xf825ce54, SUC_lexicon_tags_25 },
    { 0x1685ce55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf485ce58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2125ce5d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb65ce61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe865ce65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51e5ce67, SUC_lexicon_tags_17 },
    { 0x9085ce67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa865ce6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4965ce74, SUC_lexicon_tags_38 },
    { 0xb745ce75, SUC_lexicon_tags_22 },
    { 0xb565ce76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee45ce78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5985ce7b, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0xb2c5ce7d, SUC_lexicon_tags_8 },
    { 0xc7a5ce7d, SUC_lexicon_tags_25 },
    { 0xe1e5ce7f, SUC_lexicon_tags_1 },
    { 0xeee5ce7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf685ce85, SUC_lexicon_tags_37 },
    { 0xa725ce86, SUC_lexicon_tags_25 },
    { 0x99c5ce86, SUC_lexicon_tags_25 },
    { 0xad85ce85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde05ce8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc5ce8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f85ce91, SUC_lexicon_tags_0 },
    { 0xd6a5ce92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b25ce94, SUC_lexicon_tags_39 },
    { 0x9d65ce95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3205ce97, SUC_lexicon_tags_66 },
    { 0x65ce98, SUC_lexicon_tags_39 },
    { 0x21c5ce97, SUC_lexicon_tags_43 },
    { 0xeb45ce99, SUC_lexicon_tags_55 },
    { 0x1c5ce97, SUC_lexicon_tags_35 },
    { 0x425ce9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf85ce9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec85cea0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3905cea2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca45cea9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd665ceb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a85ceb8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c5cebc, SUC_lexicon_tags_40 },
    { 0x8005cebc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5cec2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec5cec5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd545cec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfac5cec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8105cece, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf445ced4, SUC_lexicon_tags_37 },
    { 0x1c5ced5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7925ced8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4aa5ceda, SUC_lexicon_tags_11 },
    { 0xf425ceda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c5cee6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a5cee9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6285ceee, SUC_lexicon_tags_0 },
    { 0x1785ceee, SUC_lexicon_tags_17 },
    { 0x3365cef0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5585cef2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3285cef7, SUC_lexicon_tags_58 },
    { 0x6e05cef8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc65cefa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2845cefe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xab25cf01, SUC_lexicon_tags_25 },
    { 0xdfe5cf02, SUC_lexicon_tags_5 },
    { 0xfb25cf01, SUC_lexicon_tags_23 },
    { 0xa665cf02, SUC_lexicon_tags_2 },
    { 0x76e5cf03, SUC_lexicon_tags_16 },
    { 0x71c5cf06, SUC_lexicon_tags_15 },
    { 0xde05cf06, SUC_lexicon_tags_10 },
    { 0xd05cf08, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c85cf0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e25cf0d, SUC_lexicon_tags_19 },
    { 0x67c5cf0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa425cf10, SUC_lexicon_tags_60 },
    { 0xd1a5cf11, SUC_lexicon_tags_46 },
    { 0x6805cf12, SUC_lexicon_tags_15 },
    { 0x7645cf13, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa5cf16, SUC_lexicon_tags_79 },
    { 0xcfa5cf17, SUC_lexicon_tags_3 },
    { 0x6965cf16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab25cf1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3525cf22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa65cf25, SUC_lexicon_tags_11 },
    { 0x4d45cf26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d85cf2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c5cf2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e5cf32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x845cf35, SUC_lexicon_tags_30 },
    { 0x7045cf36, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e5cf39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e5cf3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e5cf3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf65cf43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a5cf47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9525cf49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f85cf4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc45cf4d, SUC_lexicon_tags_37 },
    { 0x6325cf4d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4e05cf50, SUC_lexicon_tags_82 },
    { 0xc465cf51, SUC_lexicon_tags_25 },
    { 0x79c5cf51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd25cf59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb765cf61, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1265cf63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a65cf6b, SUC_lexicon_tags_87 },
    { 0x9365cf6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe225cf6f, SUC_lexicon_tags_25 },
    { 0xf925cf6f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e45cf75, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c05cf84, SUC_lexicon_tags_2 },
    { 0x9265cf84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a5cf88, SUC_lexicon_tags_47 },
    { 0x8085cf88, SUC_lexicon_tags_1 },
    { 0x32a5cf8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c5cf8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5025cf91, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5ca5cf93, SUC_lexicon_tags_14 },
    { 0x8705cf94, SUC_lexicon_tags_43 },
    { 0x4e05cf93, SUC_lexicon_tags_8 },
    { 0xa5a5cf96, SUC_lexicon_tags_37 },
    { 0xba85cf96, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2345cf9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76c5cf9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae05cf9f, SUC_lexicon_tags_33 },
    { 0x9da5cfa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe465cfa2, SUC_lexicon_tags_23 },
    { 0x48a5cfa3, SUC_lexicon_tags_63 },
    { 0xa985cfa2, SUC_lexicon_tags_15 },
    { 0x5c5cfa5, SUC_lexicon_tags_140 },
    { 0x66e5cfa5, SUC_lexicon_tags_10 },
    { 0x3dc5cfa7, SUC_lexicon_tags_37 },
    { 0xfa85cfa8, SUC_lexicon_tags_25 },
    { 0x4ea5cfa9, SUC_lexicon_tags_23 },
    { 0x70a5cfaa, SUC_lexicon_tags_15 },
    { 0x6bc5cfab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e5cfb2, SUC_lexicon_tags_25 },
    { 0xb145cfb3, SUC_lexicon_tags_25 },
    { 0x96c5cfb4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e5cfb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1105cfb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8dc5cfbb, SUC_lexicon_tags_15 },
    { 0x81e5cfbc, SUC_lexicon_tags_17 },
    { 0xf8e5cfbd, SUC_lexicon_tags_23 },
    { 0xfca5cfbe, SUC_lexicon_tags_46 },
    { 0xe3e5cfbe, SUC_lexicon_tags_27 },
    { 0x29e5cfc0, SUC_lexicon_tags_45 },
    { 0xffc5cfbe, SUC_lexicon_tags_11 },
    { 0x4be5cfc2, SUC_lexicon_tags_60 },
    { 0xa7c5cfbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf745cfc6, SUC_lexicon_tags_1 },
    { 0x9e25cfc6, SUC_lexicon_tags_0 },
    { 0x8485cfc6, SUC_lexicon_tags_1 },
    { 0x58a5cfc9, SUC_lexicon_tags_25 },
    { 0x5965cfc9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2965cfcd, SUC_lexicon_tags_52 },
    { 0x8a45cfcd, SUC_lexicon_tags_11 },
    { 0xde25cfcf, SUC_lexicon_tags_12 },
    { 0x20c5cfd0, SUC_lexicon_tags_25 },
    { 0x71c5cfd0, SUC_lexicon_tags_2 },
    { 0x7465cfd2, SUC_lexicon_tags_14 },
    { 0xe665cfd3, SUC_lexicon_tags_27 },
    { 0x5f45cfd4, SUC_lexicon_tags_14 },
    { 0xcda5cfd4, SUC_lexicon_tags_33 },
    { 0x2b45cfce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa5cfdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd325cfe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd65cfe2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae5cfe6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285cfeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6645cff3, SUC_lexicon_tags_25 },
    { 0x74a5cff3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x12c5cff6, SUC_lexicon_tags_45 },
    { 0x5165cff7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x665cff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5605cffd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7365cfff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec25d004, SUC_lexicon_tags_5 },
    { 0xc2a5d005, SUC_lexicon_tags_14 },
    { 0xb845d005, SUC_lexicon_tags_12 },
    { 0x9785d007, SUC_lexicon_tags_37 },
    { 0xe7c5d004, SUC_lexicon_tags_37 },
    { 0x64e5d009, SUC_lexicon_tags_17 },
    { 0x6945d007, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea5d00e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb05d010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb865d015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d05d01c, SUC_lexicon_tags_12 },
    { 0x20c5d01d, SUC_lexicon_tags_25 },
    { 0xd3c5d01e, SUC_lexicon_tags_7 },
    { 0xdc65d01f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb805d024, SUC_lexicon_tags_25 },
    { 0x2365d024, SUC_lexicon_tags_65 },
    { 0x8825d026, SUC_lexicon_tags_95 },
    { 0x1465d027, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6505d02a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9805d032, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd5e5d034, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c5d037, SUC_lexicon_tags_30 },
    { 0xb4e5d038, SUC_lexicon_tags_194 },
    { 0xfa5d037, SUC_lexicon_tags_82 },
    { 0x7a05d038, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f65d03d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc705d048, SUC_lexicon_tags_42 },
    { 0x3405d049, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa145d04e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x2745d050, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5865d054, SUC_lexicon_tags_37 },
    { 0xac5d054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a5d058, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xea85d05a, SUC_lexicon_tags_37 },
    { 0x7665d05a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1265d05f, SUC_lexicon_tags_100 },
    { 0xe765d060, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1a25d062, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3025d064, SUC_lexicon_tags_7 },
    { 0x5365d065, SUC_lexicon_tags_47 },
    { 0x9b25d066, SUC_lexicon_tags_43 },
    { 0xd025d065, SUC_lexicon_tags_15 },
    { 0x83c5d068, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9b05d06a, SUC_lexicon_tags_15 },
    { 0x7425d06a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb165d06e, SUC_lexicon_tags_31 },
    { 0x68e5d06f, SUC_lexicon_tags_47 },
    { 0xa6e5d06e, SUC_lexicon_tags_48 },
    { 0xdd05d071, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc25d07b, SUC_lexicon_tags_148 },
    { 0xa665d07b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf245d082, SUC_lexicon_tags_11 },
    { 0x8ce5d082, SUC_lexicon_tags_63 },
    { 0x3e05d082, SUC_lexicon_tags_26 },
    { 0xd6c5d085, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e05d087, SUC_lexicon_tags_54 },
    { 0xc8c5d087, SUC_lexicon_tags_0 },
    { 0xb0c5d088, SUC_lexicon_tags_38 },
    { 0x3ca5d089, SUC_lexicon_tags_47 },
    { 0x6065d08b, SUC_lexicon_tags_45 },
    { 0x67c5d08c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe705d08f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2985d091, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2925d093, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe785d095, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c5d098, SUC_lexicon_tags_17 },
    { 0x8d05d098, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb25d0a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed65d0a4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf2a5d0a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc925d0ad, SUC_lexicon_tags_47 },
    { 0xb885d0ae, SUC_lexicon_tags_25 },
    { 0x3505d0af, SUC_lexicon_tags_60 },
    { 0x9525d0b0, SUC_lexicon_tags_28 },
    { 0xa545d0b0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc345d0b5, SUC_lexicon_tags_25 },
    { 0xae65d0b6, SUC_lexicon_tags_11 },
    { 0x985d0b7, SUC_lexicon_tags_1 },
    { 0xab25d0b8, SUC_lexicon_tags_47 },
    { 0xde85d0b7, SUC_lexicon_tags_64 },
    { 0x3505d0b7, SUC_lexicon_tags_15 },
    { 0x2a65d0bb, SUC_lexicon_tags_49 },
    { 0x7425d0bc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x57a5d0be, SUC_lexicon_tags_68 },
    { 0xabe5d0bf, SUC_lexicon_tags_0 },
    { 0x2e45d0bf, SUC_lexicon_tags_2 },
    { 0x9f25d0c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x63a5d0c3, SUC_lexicon_tags_17 },
    { 0x3dc5d0c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8285d0c6, SUC_lexicon_tags_0 },
    { 0x85e5d0c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x645d0c9, SUC_lexicon_tags_27 },
    { 0xb965d0ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce5d0d4, SUC_lexicon_tags_35 },
    { 0x5ac5d0d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63e5d0d7, SUC_lexicon_tags_39 },
    { 0x4f25d0d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2585d0da, SUC_lexicon_tags_12 },
    { 0xeb45d0db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad05d0e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb65d0e3, SUC_lexicon_tags_9 },
    { 0xfbe5d0e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd45d0e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5905d0ea, SUC_lexicon_tags_64 },
    { 0x4785d0eb, SUC_lexicon_tags_23 },
    { 0xdec5d0ea, SUC_lexicon_tags_2 },
    { 0x165d0ed, SUC_lexicon_tags_25 },
    { 0xd765d0eb, SUC_lexicon_tags_17 },
    { 0x7445d0ed, SUC_lexicon_tags_38 },
    { 0x8305d0f0, SUC_lexicon_tags_45 },
    { 0x7885d0f1, SUC_lexicon_tags_77 },
    { 0xb525d0f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2865d0f5, SUC_lexicon_tags_12 },
    { 0x3f25d0f5, SUC_lexicon_tags_27 },
    { 0xdf45d0f5, SUC_lexicon_tags_2 },
    { 0x6e5d0f7, SUC_lexicon_tags_37 },
    { 0xd425d0f5, SUC_lexicon_tags_63 },
    { 0xef85d0f7, SUC_lexicon_tags_39 },
    { 0xbae5d0fb, SUC_lexicon_tags_15 },
    { 0x2105d0fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd405d102, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xac85d105, SUC_lexicon_tags_25 },
    { 0x6805d105, SUC_lexicon_tags_8 },
    { 0xb485d106, SUC_lexicon_tags_1 },
    { 0x8185d107, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc5d111, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a5d115, SUC_lexicon_tags_77 },
    { 0x4285d115, SUC_lexicon_tags_27 },
    { 0x1105d117, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb105d11a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xa185d11d, SUC_lexicon_tags_0 },
    { 0xac45d11d, SUC_lexicon_tags_43 },
    { 0x5ca5d11d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a5d123, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb445d129, SUC_lexicon_tags_25 },
    { 0xdba5d12a, SUC_lexicon_tags_66 },
    { 0x75c5d12b, SUC_lexicon_tags_79 },
    { 0x7cc5d12c, SUC_lexicon_tags_17 },
    { 0x85e5d12d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf785d130, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7c5d132, SUC_lexicon_tags_2 },
    { 0x9dc5d133, SUC_lexicon_tags_27 },
    { 0xda5d134, SUC_lexicon_tags_1 },
    { 0xcf05d134, SUC_lexicon_tags_33 },
    { 0x6805d136, SUC_lexicon_tags_37 },
    { 0x53e5d137, SUC_lexicon_tags_17 },
    { 0x1905d133, SUC_lexicon_tags_14 },
    { 0xaa25d135, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf185d13b, SUC_lexicon_tags_55 },
    { 0x6985d13b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe245d141, SUC_lexicon_tags_43 },
    { 0x9b85d141, SUC_lexicon_tags_12 },
    { 0x145d142, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc905d148, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac5d14d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a25d150, SUC_lexicon_tags_12 },
    { 0x3b45d151, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c5d154, SUC_lexicon_tags_38 },
    { 0xcdc5d155, SUC_lexicon_tags_321 },
    { 0x8285d155, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa5d159, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6005d15d, SUC_lexicon_tags_26 },
    { 0x665d15d, SUC_lexicon_tags_39 },
    { 0x2b45d15e, SUC_lexicon_tags_37 },
    { 0x4f45d160, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc965d163, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1945d166, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf805d16e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa605d171, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a5d174, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b25d176, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd865d17a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa345d17c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2585d17e, SUC_lexicon_tags_8 },
    { 0xe785d17f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b25d181, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e65d185, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa6a5d187, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x97c5d189, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe5d18d, SUC_lexicon_tags_12 },
    { 0x6605d18e, SUC_lexicon_tags_45 },
    { 0xc2e5d18f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a5d192, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ba5d194, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd85d199, SUC_lexicon_tags_65 },
    { 0xe5d19a, SUC_lexicon_tags_11 },
    { 0xe065d19b, SUC_lexicon_tags_15 },
    { 0xb8a5d19c, SUC_lexicon_tags_1 },
    { 0x11c5d19d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8105d1a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd605d1a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45d1a7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5d1aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5d1ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd85d1b0, SUC_lexicon_tags_57 },
    { 0xc7e5d1b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f05d1b6, SUC_lexicon_tags_27 },
    { 0x85a5d1b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa5d1bb, SUC_lexicon_tags_68 },
    { 0x2345d1bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2405d1bf, SUC_lexicon_tags_25 },
    { 0x265d1c0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x705d1c2, SUC_lexicon_tags_23 },
    { 0xbda5d1c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x405d1c6, SUC_lexicon_tags_14 },
    { 0x5f05d1c7, SUC_lexicon_tags_37 },
    { 0xad65d1c6, SUC_lexicon_tags_60 },
    { 0x4fa5d1c8, SUC_lexicon_tags_11 },
    { 0x62a5d1ca, SUC_lexicon_tags_37 },
    { 0x7025d1cb, SUC_lexicon_tags_30 },
    { 0x12a5d1c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb45d1d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf65d1d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec5d1d6, SUC_lexicon_tags_87 },
    { 0x3c85d1d7, SUC_lexicon_tags_27 },
    { 0x1de5d1d8, SUC_lexicon_tags_61 },
    { 0x5c25d1d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec85d1e0, SUC_lexicon_tags_35 },
    { 0x2de5d1e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a5d1e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc5d1ee, SUC_lexicon_tags_29 },
    { 0xfe5d1ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a5d1f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e25d1f8, SUC_lexicon_tags_30 },
    { 0xd185d1f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa385d1fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a5d201, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x96a5d203, SUC_lexicon_tags_3 },
    { 0x77e5d204, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a5d208, SUC_lexicon_tags_17 },
    { 0xb2a5d209, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce5d20c, SUC_lexicon_tags_7 },
    { 0x3dc5d20d, SUC_lexicon_tags_8 },
    { 0x4ee5d20e, SUC_lexicon_tags_45 },
    { 0x4445d20f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3625d211, SUC_lexicon_tags_17 },
    { 0xbde5d212, SUC_lexicon_tags_14 },
    { 0xa585d211, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c5d216, SUC_lexicon_tags_25 },
    { 0x5685d217, SUC_lexicon_tags_27 },
    { 0xc505d217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4465d21a, SUC_lexicon_tags_7 },
    { 0xd085d21b, SUC_lexicon_tags_27 },
    { 0x22a5d21b, SUC_lexicon_tags_6 },
    { 0xb325d21d, SUC_lexicon_tags_1 },
    { 0x32e5d21e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c45d22b, SUC_lexicon_tags_42 },
    { 0xdc85d22b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b65d22e, SUC_lexicon_tags_11 },
    { 0xcac5d22f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e5d232, SUC_lexicon_tags_25 },
    { 0x4d65d232, SUC_lexicon_tags_31 },
    { 0xdc5d233, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1885d237, SUC_lexicon_tags_42 },
    { 0x7f85d237, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5425d23a, SUC_lexicon_tags_63 },
    { 0x6fc5d23a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae45d23d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f85d241, SUC_lexicon_tags_38 },
    { 0x8c65d242, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf05d24a, SUC_lexicon_tags_25 },
    { 0xa885d24a, SUC_lexicon_tags_66 },
    { 0xfc85d24c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b05d255, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1565d257, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a25d25a, SUC_lexicon_tags_30 },
    { 0x7405d25b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc0e5d25d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd265d261, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca5d264, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a05d267, SUC_lexicon_tags_1 },
    { 0x3545d268, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88a5d26a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c05d26d, SUC_lexicon_tags_17 },
    { 0xaaa5d26d, SUC_lexicon_tags_34 },
    { 0xd565d26e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b85d274, SUC_lexicon_tags_12 },
    { 0x685d274, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b65d277, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e5d280, SUC_lexicon_tags_37 },
    { 0x1025d281, SUC_lexicon_tags_47 },
    { 0x7685d282, SUC_lexicon_tags_48 },
    { 0x525d280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e5d28b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c5d290, SUC_lexicon_tags_49 },
    { 0x5785d291, SUC_lexicon_tags_25 },
    { 0xb545d291, SUC_lexicon_tags_17 },
    { 0x3d65d292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa925d296, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f65d29e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6065d2a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9805d2a3, SUC_lexicon_tags_32 },
    { 0xaf65d2a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9365d2a6, SUC_lexicon_tags_452 },
    { 0x70e5d2a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f05d2ad, SUC_lexicon_tags_37 },
    { 0x9525d2ae, SUC_lexicon_tags_12 },
    { 0x6d25d2ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe985d2b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4245d2b3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d25d2b6, SUC_lexicon_tags_12 },
    { 0xe485d2b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a5d2bb, SUC_lexicon_tags_25 },
    { 0x10a5d2bc, SUC_lexicon_tags_37 },
    { 0x9aa5d2bb, SUC_lexicon_tags_37 },
    { 0x2e05d2be, SUC_lexicon_tags_15 },
    { 0xe445d2be, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7ac5d2c1, SUC_lexicon_tags_37 },
    { 0xdb65d2c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa5d2c6, SUC_lexicon_tags_17 },
    { 0x9da5d2c6, SUC_lexicon_tags_1 },
    { 0x3a25d2c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f25d2cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcea5d2cd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7705d2d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea5d2d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3e45d2d5, SUC_lexicon_tags_11 },
    { 0xf2c5d2d6, SUC_lexicon_tags_160 },
    { 0x2c25d2d6, SUC_lexicon_tags_0 },
    { 0x98e5d2d8, SUC_lexicon_tags_7 },
    { 0x2245d2d8, SUC_lexicon_tags_12 },
    { 0x285d2d9, SUC_lexicon_tags_15 },
    { 0x37a5d2db, SUC_lexicon_tags_12 },
    { 0x58a5d2d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a65d2e7, SUC_lexicon_tags_29 },
    { 0x7885d2e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe025d2ea, SUC_lexicon_tags_38 },
    { 0x1aa5d2eb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4985d2ee, SUC_lexicon_tags_25 },
    { 0xc4e5d2ee, SUC_lexicon_tags_12 },
    { 0x1765d2ef, SUC_lexicon_tags_17 },
    { 0xbc25d2f1, SUC_lexicon_tags_25 },
    { 0xce45d2f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5925d2f8, SUC_lexicon_tags_1 },
    { 0x3da5d2f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc365d2fb, SUC_lexicon_tags_31 },
    { 0x2825d2fc, SUC_lexicon_tags_1 },
    { 0xe3a5d2fc, SUC_lexicon_tags_16 },
    { 0x2245d2fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a5d304, SUC_lexicon_tags_15 },
    { 0x8145d304, SUC_lexicon_tags_9 },
    { 0x545d304, SUC_lexicon_tags_0 },
    { 0x3fa5d304, SUC_lexicon_tags_11 },
    { 0xb5c5d307, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca5d30b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1445d311, SUC_lexicon_tags_37 },
    { 0x3685d312, SUC_lexicon_tags_10 },
    { 0xf725d313, SUC_lexicon_tags_16 },
    { 0xa425d312, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb7a5d316, SUC_lexicon_tags_0 },
    { 0xd045d317, SUC_lexicon_tags_37 },
    { 0x1a65d318, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb385d31b, SUC_lexicon_tags_11 },
    { 0x20e5d31b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7765d31e, SUC_lexicon_tags_0 },
    { 0x1b05d31e, SUC_lexicon_tags_11 },
    { 0xca45d320, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd165d323, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9185d325, SUC_lexicon_tags_45 },
    { 0xe725d326, SUC_lexicon_tags_2 },
    { 0x5965d327, SUC_lexicon_tags_23 },
    { 0xa9c5d327, SUC_lexicon_tags_66 },
    { 0x2e25d329, SUC_lexicon_tags_26 },
    { 0x41e5d326, SUC_lexicon_tags_14 },
    { 0x4485d326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c5d333, SUC_lexicon_tags_45 },
    { 0x9725d334, SUC_lexicon_tags_15 },
    { 0x6bc5d335, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec5d33d, SUC_lexicon_tags_38 },
    { 0x2325d33e, SUC_lexicon_tags_37 },
    { 0xb8c5d33f, SUC_lexicon_tags_23 },
    { 0xdc65d340, SUC_lexicon_tags_25 },
    { 0x3e85d340, SUC_lexicon_tags_37 },
    { 0x82c5d341, SUC_lexicon_tags_37 },
    { 0xbe25d343, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9865d351, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb565d354, SUC_lexicon_tags_63 },
    { 0x90e5d355, SUC_lexicon_tags_11 },
    { 0xee25d356, SUC_lexicon_tags_30 },
    { 0x3405d357, SUC_lexicon_tags_25 },
    { 0xb8e5d357, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a5d361, SUC_lexicon_tags_25 },
    { 0xb0a5d361, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc65d364, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e5d368, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e05d36c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da5d371, SUC_lexicon_tags_11 },
    { 0xa985d372, SUC_lexicon_tags_27 },
    { 0x9965d371, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc445d375, SUC_lexicon_tags_79 },
    { 0x6ee5d375, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe505d378, SUC_lexicon_tags_11 },
    { 0x12e5d379, SUC_lexicon_tags_7 },
    { 0xbf45d37a, SUC_lexicon_tags_17 },
    { 0x7ea5d379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x985d37e, SUC_lexicon_tags_16 },
    { 0xf9c5d37e, SUC_lexicon_tags_62 },
    { 0x3d45d37e, SUC_lexicon_tags_151 },
    { 0xfe85d381, SUC_lexicon_tags_9 },
    { 0xc545d382, SUC_lexicon_tags_25 },
    { 0xdb85d383, SUC_lexicon_tags_29 },
    { 0xd025d384, SUC_lexicon_tags_0 },
    { 0xc305d383, SUC_lexicon_tags_15 },
    { 0xbca5d386, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3425d38f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c65d394, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b45d396, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e5d399, SUC_lexicon_tags_17 },
    { 0x54c5d39a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed05d39f, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xc445d3a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b65d3a9, SUC_lexicon_tags_13 },
    { 0x1c05d3a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf445d3ac, SUC_lexicon_tags_11 },
    { 0xc885d3ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a5d3b1, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6885d3b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3305d3ba, SUC_lexicon_tags_2 },
    { 0x3585d3bb, SUC_lexicon_tags_25 },
    { 0x61a5d3bb, SUC_lexicon_tags_63 },
    { 0x3e25d3bc, SUC_lexicon_tags_21 },
    { 0x59c5d3be, SUC_lexicon_tags_11 },
    { 0x5285d3bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc5d3c1, SUC_lexicon_tags_23 },
    { 0x3965d3c2, SUC_lexicon_tags_25 },
    { 0x53e5d3c3, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x6245d3c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e5d3cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb965d3cd, SUC_lexicon_tags_26 },
    { 0x3f25d3ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff25d3d0, SUC_lexicon_tags_79 },
    { 0x6225d3d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf525d3d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1565d3da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb605d3dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8485d3e1, SUC_lexicon_tags_16 },
    { 0xe885d3e2, SUC_lexicon_tags_19 },
    { 0xfde5d3e3, SUC_lexicon_tags_49 },
    { 0x7085d3e4, SUC_lexicon_tags_8 },
    { 0x7ea5d3e4, SUC_lexicon_tags_26 },
    { 0xfa5d3e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc265d3ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca5d3f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x75a5d3f3, SUC_lexicon_tags_11 },
    { 0x8ca5d3f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79c5d3f6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc725d3f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6c5d3fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x685d3fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc925d402, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea5d405, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e5d40c, SUC_lexicon_tags_2 },
    { 0x7765d40c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcec5d40f, SUC_lexicon_tags_56 },
    { 0xf865d40f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c5d413, SUC_lexicon_tags_66 },
    { 0xef05d414, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b45d417, SUC_lexicon_tags_54 },
    { 0x1b05d417, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea25d41a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7905d41c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3385d41e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a5d423, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a5d428, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c5d42c, SUC_lexicon_tags_38 },
    { 0xe1a5d42c, SUC_lexicon_tags_23 },
    { 0x5bc5d42d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4765d433, SUC_lexicon_tags_1 },
    { 0x4945d433, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a5d437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b45d43b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3c5d43d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825d443, SUC_lexicon_tags_43 },
    { 0x3c5d443, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c5d447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7985d44e, SUC_lexicon_tags_17 },
    { 0xf805d44e, SUC_lexicon_tags_7 },
    { 0xf6c5d450, SUC_lexicon_tags_45 },
    { 0x63e5d451, SUC_lexicon_tags_1486 },
    { 0x7fa5d452, SUC_lexicon_tags_17 },
    { 0x42e5d453, SUC_lexicon_tags_37 },
    { 0xa085d454, SUC_lexicon_tags_23 },
    { 0x4fc5d455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5085d45b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9cc5d45d, SUC_lexicon_tags_60 },
    { 0x9965d45e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c5d463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5725d468, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c5d46e, SUC_lexicon_tags_12 },
    { 0x2865d46e, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x8f65d471, SUC_lexicon_tags_31 },
    { 0xa2c5d472, SUC_lexicon_tags_8 },
    { 0x1865d473, SUC_lexicon_tags_250 },
    { 0xf385d471, SUC_lexicon_tags_49 },
    { 0x8065d475, SUC_lexicon_tags_25 },
    { 0x1e45d476, SUC_lexicon_tags_27 },
    { 0x29c5d476, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1885d47a, SUC_lexicon_tags_37 },
    { 0xd205d47a, SUC_lexicon_tags_12 },
    { 0x5ca5d47c, SUC_lexicon_tags_26 },
    { 0x73c5d47c, SUC_lexicon_tags_7 },
    { 0xf625d47e, SUC_lexicon_tags_39 },
    { 0xba25d47e, SUC_lexicon_tags_84 },
    { 0xc005d47c, SUC_lexicon_tags_2 },
    { 0xb405d47f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95e5d483, SUC_lexicon_tags_12 },
    { 0xdd85d483, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa785d489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6045d491, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8b85d493, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e65d497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69e5d499, SUC_lexicon_tags_2 },
    { 0xaec5d49a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e45d49c, SUC_lexicon_tags_37 },
    { 0xa9e5d49c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc185d4a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f65d4a6, SUC_lexicon_tags_25 },
    { 0x69c5d4a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5d4ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2625d4af, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6005d4b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7645d4b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545d4ba, SUC_lexicon_tags_37 },
    { 0x8bc5d4bb, SUC_lexicon_tags_63 },
    { 0x7025d4bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe045d4be, SUC_lexicon_tags_25 },
    { 0x505d4be, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e5d4c2, SUC_lexicon_tags_17 },
    { 0x34a5d4c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf365d4c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7485d4ca, SUC_lexicon_tags_1 },
    { 0x5765d4ca, SUC_lexicon_tags_23 },
    { 0xe225d4cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84e5d4ce, SUC_lexicon_tags_47 },
    { 0x84e5d4ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b85d4d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe205d4da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6185d4df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1325d4e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59c5d4e4, SUC_lexicon_tags_4 },
    { 0x4065d4e5, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b45d4e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24c5d4ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeae5d4ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa645d4ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe85d4f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb645d4f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d65d4fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d25d4ff, SUC_lexicon_tags_118 },
    { 0x28a5d500, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4285d502, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a45d505, SUC_lexicon_tags_7 },
    { 0xc445d506, SUC_lexicon_tags_31 },
    { 0x9405d507, SUC_lexicon_tags_4 },
    { 0x1a45d508, SUC_lexicon_tags_37 },
    { 0x9145d505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9925d50d, SUC_lexicon_tags_12 },
    { 0xb7a5d50e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5e5d510, SUC_lexicon_tags_17 },
    { 0x4285d510, SUC_lexicon_tags_0 },
    { 0xd8e5d511, SUC_lexicon_tags_13 },
    { 0x9ea5d512, SUC_lexicon_tags_0 },
    { 0xd825d510, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e5d51d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c65d520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e5d523, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e5d52b, SUC_lexicon_tags_7 },
    { 0x2a85d52b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45d52e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc5d531, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf865d533, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a5d536, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc5d539, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8245d53e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x12e5d540, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2905d548, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf265d54a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f45d54d, SUC_lexicon_tags_17 },
    { 0x6145d54e, SUC_lexicon_tags_15 },
    { 0x6405d54f, SUC_lexicon_tags_46 },
    { 0x6e85d54d, SUC_lexicon_tags_29 },
    { 0x8d65d551, SUC_lexicon_tags_82 },
    { 0xce85d552, SUC_lexicon_tags_60 },
    { 0x1465d550, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b45d55a, SUC_lexicon_tags_23 },
    { 0xa945d55b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e5d55f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc45d562, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a85d565, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf585d56a, SUC_lexicon_tags_37 },
    { 0x7d05d56b, SUC_lexicon_tags_12 },
    { 0x1125d56a, SUC_lexicon_tags_6 },
    { 0xac05d56d, SUC_lexicon_tags_25 },
    { 0x8cc5d56e, SUC_lexicon_tags_25 },
    { 0x5be5d56f, SUC_lexicon_tags_84 },
    { 0xbf25d56e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e25d57b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd5a5d57d, SUC_lexicon_tags_15 },
    { 0xba85d57e, SUC_lexicon_tags_17 },
    { 0x5265d57f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4385d581, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc5d588, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a85d58e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea5d592, SUC_lexicon_tags_25 },
    { 0x88e5d593, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf585d596, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf85d59b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa5d5a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e5d5a3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xce05d5a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb105d5aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc5d5b0, SUC_lexicon_tags_44 },
    { 0x33a5d5b1, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x8925d5b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6ec5d5b6, SUC_lexicon_tags_50 },
    { 0xbc25d5b6, SUC_lexicon_tags_27 },
    { 0x8a25d5b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb25d5bd, SUC_lexicon_tags_19 },
    { 0x2e5d5be, SUC_lexicon_tags_7 },
    { 0x4205d5be, SUC_lexicon_tags_37 },
    { 0x4325d5c0, SUC_lexicon_tags_4 },
    { 0xf6c5d5c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7885d5c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ee5d5c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca85d5cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac05d5d0, SUC_lexicon_tags_37 },
    { 0x2705d5d1, SUC_lexicon_tags_2 },
    { 0x7f05d5d2, SUC_lexicon_tags_15 },
    { 0xbe45d5d3, SUC_lexicon_tags_38 },
    { 0x3525d5d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c85d5d7, SUC_lexicon_tags_16 },
    { 0x2685d5d8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad45d5dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb405d5de, SUC_lexicon_tags_5 },
    { 0x18a5d5de, SUC_lexicon_tags_12 },
    { 0x6045d5df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85a5d5e2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x74c5d5e4, SUC_lexicon_tags_2 },
    { 0x34a5d5e5, SUC_lexicon_tags_35 },
    { 0x5225d5e6, SUC_lexicon_tags_15 },
    { 0x92e5d5e6, SUC_lexicon_tags_17 },
    { 0xed45d5e8, SUC_lexicon_tags_2 },
    { 0x8de5d5e7, SUC_lexicon_tags_19 },
    { 0x7105d5ea, SUC_lexicon_tags_33 },
    { 0xc0e5d5eb, SUC_lexicon_tags_25 },
    { 0x5b05d5e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e5d5ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9685d5f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da5d5f8, SUC_lexicon_tags_196 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a25d5fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6405d602, SUC_lexicon_tags_37 },
    { 0xc605d603, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e5d60a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a5d60d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e5d615, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c05d618, SUC_lexicon_tags_47 },
    { 0x2c05d619, SUC_lexicon_tags_47 },
    { 0xb0a5d619, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3525d61f, SUC_lexicon_tags_37 },
    { 0x97c5d620, SUC_lexicon_tags_14 },
    { 0x1245d61f, SUC_lexicon_tags_1 },
    { 0xf425d622, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x60a5d624, SUC_lexicon_tags_17 },
    { 0x6125d624, SUC_lexicon_tags_37 },
    { 0x15e5d626, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc245d62b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec25d631, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc5d634, SUC_lexicon_tags_39 },
    { 0xb085d635, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2445d63b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a5d63e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec5d641, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6065d643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d646, SUC_lexicon_tags_8 },
    { 0x8b85d647, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6605d64c, SUC_lexicon_tags_25 },
    { 0xa6e5d64d, SUC_lexicon_tags_60 },
    { 0x4e65d64d, SUC_lexicon_tags_2 },
    { 0xb525d64f, SUC_lexicon_tags_68 },
    { 0x7d05d64c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c45d654, SUC_lexicon_tags_7 },
    { 0xa765d655, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5845d658, SUC_lexicon_tags_11 },
    { 0xe2c5d659, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa5d661, SUC_lexicon_tags_15 },
    { 0xe2c5d661, SUC_lexicon_tags_11 },
    { 0xe645d663, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4765d66d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1805d671, SUC_lexicon_tags_37 },
    { 0xf785d672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc45d674, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29a5d676, SUC_lexicon_tags_49 },
    { 0x5465d677, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c5d67c, SUC_lexicon_tags_103 },
    { 0xbde5d67d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c5d682, SUC_lexicon_tags_2 },
    { 0x4e5d683, SUC_lexicon_tags_207 },
    { 0x71c5d684, SUC_lexicon_tags_26 },
    { 0x3685d682, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7705d687, SUC_lexicon_tags_68 },
    { 0xc0e5d687, SUC_lexicon_tags_37 },
    { 0x30c5d689, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e5d68c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c5d694, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa5d69b, SUC_lexicon_tags_11 },
    { 0x5985d69b, SUC_lexicon_tags_7 },
    { 0xb445d69d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc625d6a1, SUC_lexicon_tags_37 },
    { 0xc25d6a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a5d6a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91a5d6ab, SUC_lexicon_tags_15 },
    { 0x9985d6ac, SUC_lexicon_tags_48 },
    { 0xa985d6ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a65d6b0, SUC_lexicon_tags_2 },
    { 0x2e05d6b1, SUC_lexicon_tags_47 },
    { 0xe6e5d6b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc5d6b6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbb65d6b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a5d6bb, SUC_lexicon_tags_26 },
    { 0xc5c5d6bc, SUC_lexicon_tags_12 },
    { 0xc05d6bc, SUC_lexicon_tags_49 },
    { 0x74c5d6bb, SUC_lexicon_tags_6 },
    { 0xea45d6bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e5d6c3, SUC_lexicon_tags_12 },
    { 0xdc05d6c4, SUC_lexicon_tags_37 },
    { 0x5585d6c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc5d6c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe385d6cb, SUC_lexicon_tags_23 },
    { 0x3625d6cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e85d6d1, SUC_lexicon_tags_12 },
    { 0x3965d6d2, SUC_lexicon_tags_23 },
    { 0xfb45d6d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b85d6d7, SUC_lexicon_tags_44 },
    { 0x5385d6d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c05d6dc, SUC_lexicon_tags_31 },
    { 0x52c5d6dc, SUC_lexicon_tags_9 },
    { 0x85d6dc, SUC_lexicon_tags_19 },
    { 0x84e5d6dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e5d6e2, SUC_lexicon_tags_4 },
    { 0xbb45d6e2, SUC_lexicon_tags_43 },
    { 0x8c85d6e4, SUC_lexicon_tags_25 },
    { 0x9f65d6e4, SUC_lexicon_tags_7 },
    { 0x46e5d6e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3765d6e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe925d6f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6745d6f4, SUC_lexicon_tags_17 },
    { 0xaae5d6f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3e5d6f7, SUC_lexicon_tags_7 },
    { 0xe5e5d6f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe885d6fc, SUC_lexicon_tags_46 },
    { 0xbca5d6fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa485d6ff, SUC_lexicon_tags_7 },
    { 0x445d6ff, SUC_lexicon_tags_11 },
    { 0xb4e5d701, SUC_lexicon_tags_27 },
    { 0x405d702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c5d706, SUC_lexicon_tags_10 },
    { 0x26c5d707, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5005d711, SUC_lexicon_tags_25 },
    { 0x2ae5d712, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a5d719, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa945d71d, SUC_lexicon_tags_23 },
    { 0x9b25d71e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc5d722, SUC_lexicon_tags_46 },
    { 0xa445d723, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf585d728, SUC_lexicon_tags_15 },
    { 0x4625d729, SUC_lexicon_tags_4 },
    { 0x8845d72a, SUC_lexicon_tags_112 },
    { 0x5d45d72a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e5d736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6c5d738, SUC_lexicon_tags_170 },
    { 0xdba5d739, SUC_lexicon_tags_49 },
    { 0x5225d73a, SUC_lexicon_tags_49 },
    { 0x9585d73b, SUC_lexicon_tags_9 },
    { 0x8305d73c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc5d742, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4e25d744, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a5d748, SUC_lexicon_tags_7 },
    { 0x9225d748, SUC_lexicon_tags_65 },
    { 0x19c5d748, SUC_lexicon_tags_7 },
    { 0x9125d74b, SUC_lexicon_tags_11 },
    { 0xbf25d74c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefe5d74e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c45d753, SUC_lexicon_tags_27 },
    { 0x5c85d754, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5d758, SUC_lexicon_tags_14 },
    { 0xe065d759, SUC_lexicon_tags_146 },
    { 0x32c5d759, SUC_lexicon_tags_12 },
    { 0x5fe5d75a, SUC_lexicon_tags_11 },
    { 0xda5d75c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1505d75e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ca5d760, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c5d764, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2645d76a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de5d76d, SUC_lexicon_tags_15 },
    { 0x4c05d76d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d65d776, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ba5d778, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e5d77b, SUC_lexicon_tags_27 },
    { 0xaec5d77b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe5d782, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcc65d784, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e5d787, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a45d789, SUC_lexicon_tags_12 },
    { 0xbea5d789, SUC_lexicon_tags_2 },
    { 0xd205d78a, SUC_lexicon_tags_25 },
    { 0x6525d78c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9e5d78e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a5d793, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a65d796, SUC_lexicon_tags_1 },
    { 0xdfa5d796, SUC_lexicon_tags_37 },
    { 0xac5d798, SUC_lexicon_tags_33 },
    { 0xcbc5d799, SUC_lexicon_tags_33 },
    { 0x2405d799, SUC_lexicon_tags_74 },
    { 0x2625d79b, SUC_lexicon_tags_25 },
    { 0x8425d797, SUC_lexicon_tags_17 },
    { 0x2d25d79c, SUC_lexicon_tags_33 },
    { 0x565d797, SUC_lexicon_tags_11 },
    { 0xe425d79f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd525d7a3, SUC_lexicon_tags_30 },
    { 0x9845d7a3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea5d7a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9465d7aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd805d7ac, SUC_lexicon_tags_56 },
    { 0x2325d7ad, SUC_lexicon_tags_27 },
    { 0xc7c5d7ae, SUC_lexicon_tags_47 },
    { 0x7b65d7ac, SUC_lexicon_tags_46 },
    { 0xd8c5d7b0, SUC_lexicon_tags_230 },
    { 0xbf25d7b1, SUC_lexicon_tags_36 },
    { 0xb305d7b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc945d7b5, SUC_lexicon_tags_60 },
    { 0x51e5d7b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa385d7b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3e05d7ba, SUC_lexicon_tags_43 },
    { 0x6c65d7bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5465d7c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05d7c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b05d7c8, SUC_lexicon_tags_37 },
    { 0x7b65d7c8, SUC_lexicon_tags_27 },
    { 0x73e5d7c9, SUC_lexicon_tags_37 },
    { 0x1765d7cb, SUC_lexicon_tags_107 },
    { 0x9645d7cb, SUC_lexicon_tags_4 },
    { 0x12e5d7cd, SUC_lexicon_tags_47 },
    { 0xe465d7cc, SUC_lexicon_tags_43 },
    { 0x18a5d7cc, SUC_lexicon_tags_0 },
    { 0x57a5d7c9, SUC_lexicon_tags_7 },
    { 0x4865d7d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b85d7d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1485d7db, SUC_lexicon_tags_2 },
    { 0xf85d7db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c5d7df, SUC_lexicon_tags_31 },
    { 0x80a5d7df, SUC_lexicon_tags_2 },
    { 0x4e5d7df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x905d7e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925d7e8, SUC_lexicon_tags_8 },
    { 0xe665d7e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe5d7ec, SUC_lexicon_tags_37 },
    { 0x2025d7ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe345d7ef, SUC_lexicon_tags_2 },
    { 0xe065d7f0, SUC_lexicon_tags_44 },
    { 0xba65d7f1, SUC_lexicon_tags_6 },
    { 0xdfe5d7f2, SUC_lexicon_tags_17 },
    { 0xef25d7f0, SUC_lexicon_tags_43 },
    { 0xde5d7f4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4065d7f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4365d7fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa725d7fe, SUC_lexicon_tags_33 },
    { 0x9905d7ff, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1345d803, SUC_lexicon_tags_43 },
    { 0x81c5d804, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a5d807, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4025d80a, SUC_lexicon_tags_14 },
    { 0xd9a5d80b, SUC_lexicon_tags_28 },
    { 0x5ae5d80c, SUC_lexicon_tags_0 },
    { 0x565d80b, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x2305d80f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa825d812, SUC_lexicon_tags_2 },
    { 0x53a5d813, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9e5d815, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea5d818, SUC_lexicon_tags_0 },
    { 0xb9c5d819, SUC_lexicon_tags_37 },
    { 0xec85d819, SUC_lexicon_tags_15 },
    { 0xfa65d81b, SUC_lexicon_tags_33 },
    { 0x66e5d818, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8e85d81e, SUC_lexicon_tags_28 },
    { 0xa865d81f, SUC_lexicon_tags_11 },
    { 0x9b65d820, SUC_lexicon_tags_2 },
    { 0xfee5d820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a65d824, SUC_lexicon_tags_37 },
    { 0xed25d824, SUC_lexicon_tags_17 },
    { 0x94c5d826, SUC_lexicon_tags_12 },
    { 0x38e5d826, SUC_lexicon_tags_19 },
    { 0x29a5d826, SUC_lexicon_tags_0 },
    { 0xe025d829, SUC_lexicon_tags_7 },
    { 0x5705d829, SUC_lexicon_tags_85 },
    { 0xae05d82b, SUC_lexicon_tags_28 },
    { 0x42c5d82c, SUC_lexicon_tags_29 },
    { 0x9c65d824, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e25d833, SUC_lexicon_tags_92 },
    { 0xbd25d834, SUC_lexicon_tags_37 },
    { 0x9565d835, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a45d837, SUC_lexicon_tags_15 },
    { 0x5e45d838, SUC_lexicon_tags_37 },
    { 0x5005d839, SUC_lexicon_tags_17 },
    { 0xcac5d838, SUC_lexicon_tags_7 },
    { 0x9b65d83a, SUC_lexicon_tags_2 },
    { 0x5825d839, SUC_lexicon_tags_2 },
    { 0x77a5d839, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e5d840, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc5d844, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7b85d846, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585d84f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c5d857, SUC_lexicon_tags_1 },
    { 0xdf05d857, SUC_lexicon_tags_12 },
    { 0xaa45d859, SUC_lexicon_tags_79 },
    { 0x38e5d85a, SUC_lexicon_tags_11 },
    { 0x9ee5d85b, SUC_lexicon_tags_26 },
    { 0x26e5d85b, SUC_lexicon_tags_15 },
    { 0x30c5d85d, SUC_lexicon_tags_37 },
    { 0x6225d85d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac5d861, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e5d864, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8985d866, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b25d868, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27c5d86a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6605d86c, SUC_lexicon_tags_17 },
    { 0xfec5d86c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be5d871, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a5d874, SUC_lexicon_tags_19 },
    { 0x1165d875, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a45d879, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb1c5d87b, SUC_lexicon_tags_38 },
    { 0x68a5d87c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c5d87f, SUC_lexicon_tags_1 },
    { 0xffe5d87f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf85d882, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc365d885, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf65d887, SUC_lexicon_tags_28 },
    { 0x6925d887, SUC_lexicon_tags_47 },
    { 0xaaa5d889, SUC_lexicon_tags_7 },
    { 0x6565d88a, SUC_lexicon_tags_2 },
    { 0xbd25d88b, SUC_lexicon_tags_170 },
    { 0x6125d88c, SUC_lexicon_tags_25 },
    { 0x3625d88d, SUC_lexicon_tags_63 },
    { 0x37a5d88e, SUC_lexicon_tags_15 },
    { 0x7845d88d, SUC_lexicon_tags_7 },
    { 0xde65d887, SUC_lexicon_tags_49 },
    { 0x5d85d891, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a65d893, SUC_lexicon_tags_14 },
    { 0x8445d893, SUC_lexicon_tags_1583 },
    { 0x8a45d895, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e85d89c, SUC_lexicon_tags_39 },
    { 0xb1e5d89d, SUC_lexicon_tags_38 },
    { 0x65c5d89e, SUC_lexicon_tags_7 },
    { 0x7a5d89f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a05d8a2, SUC_lexicon_tags_15 },
    { 0xcfa5d8a2, SUC_lexicon_tags_37 },
    { 0x7c45d8a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c5d8a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be5d8b0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2025d8b2, SUC_lexicon_tags_0 },
    { 0xed45d8b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc765d8b7, SUC_lexicon_tags_33 },
    { 0x1605d8b7, SUC_lexicon_tags_47 },
    { 0x2105d8b9, SUC_lexicon_tags_45 },
    { 0x5705d8ba, SUC_lexicon_tags_12 },
    { 0xd2e5d8ba, SUC_lexicon_tags_60 },
    { 0x7425d8b8, SUC_lexicon_tags_17 },
    { 0xea5d8bd, SUC_lexicon_tags_28 },
    { 0x9ee5d8ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaaa5d8c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c65d8c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7da5d8c5, SUC_lexicon_tags_7 },
    { 0x3f45d8c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f05d8c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2f85d8cb, SUC_lexicon_tags_21 },
    { 0x4fa5d8cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5325d8ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a5d8d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e25d8dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9145d8e1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5d8e4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd485d8e6, SUC_lexicon_tags_7 },
    { 0x2385d8e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc585d8ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ee5d8ec, SUC_lexicon_tags_31 },
    { 0x1fc5d8ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3785d8f1, SUC_lexicon_tags_11 },
    { 0x72c5d8f2, SUC_lexicon_tags_182 },
    { 0xc645d8f1, SUC_lexicon_tags_27 },
    { 0x3825d8f4, SUC_lexicon_tags_37 },
    { 0x96c5d8f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3f85d8f7, SUC_lexicon_tags_11 },
    { 0xdf85d8f7, SUC_lexicon_tags_17 },
    { 0xf405d8f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b85d8fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x47a5d8ff, SUC_lexicon_tags_87 },
    { 0xdc45d900, SUC_lexicon_tags_17 },
    { 0x3dc5d901, SUC_lexicon_tags_47 },
    { 0xf5e5d902, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d25d905, SUC_lexicon_tags_38 },
    { 0xf685d906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a05d90b, SUC_lexicon_tags_38 },
    { 0x80e5d90c, SUC_lexicon_tags_15 },
    { 0x4185d90b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e5d914, SUC_lexicon_tags_37 },
    { 0x5e25d915, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc465d918, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f05d91f, SUC_lexicon_tags_12 },
    { 0x2145d920, SUC_lexicon_tags_37 },
    { 0x7e25d921, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c5d926, SUC_lexicon_tags_63 },
    { 0x9c85d927, SUC_lexicon_tags_11 },
    { 0xc745d928, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xefc5d92a, SUC_lexicon_tags_2 },
    { 0x5d85d92a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a5d92e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa345d933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf465d938, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6245d93a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf685d93c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd685d93f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb445d942, SUC_lexicon_tags_37 },
    { 0xc0c5d942, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3105d945, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc5d94d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7845d950, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f65d952, SUC_lexicon_tags_17 },
    { 0x1ee5d953, SUC_lexicon_tags_17 },
    { 0x6465d953, SUC_lexicon_tags_2 },
    { 0x9fa5d955, SUC_lexicon_tags_48 },
    { 0xe8c5d955, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd745d959, SUC_lexicon_tags_11 },
    { 0x3aa5d95a, SUC_lexicon_tags_6 },
    { 0xe445d95b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf6c5d95d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x885d960, SUC_lexicon_tags_27 },
    { 0x1285d960, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e5d966, SUC_lexicon_tags_17 },
    { 0x5885d966, SUC_lexicon_tags_38 },
    { 0x9a5d968, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca25d96c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a5d96f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a65d972, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c5d975, SUC_lexicon_tags_37 },
    { 0x8d25d976, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a5d97a, SUC_lexicon_tags_38 },
    { 0x66e5d97b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a65d983, SUC_lexicon_tags_7 },
    { 0x92c5d984, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa565d98c, SUC_lexicon_tags_43 },
    { 0xcd45d98c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da5d991, SUC_lexicon_tags_7 },
    { 0xb085d992, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc5d996, SUC_lexicon_tags_4 },
    { 0x66c5d996, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0xd525d999, SUC_lexicon_tags_25 },
    { 0x7f25d99a, SUC_lexicon_tags_9 },
    { 0xc2a5d99b, SUC_lexicon_tags_6 },
    { 0x3185d99a, SUC_lexicon_tags_15 },
    { 0x5685d99d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb65d9a0, SUC_lexicon_tags_29 },
    { 0x15c5d9a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c5d9a7, SUC_lexicon_tags_37 },
    { 0xce5d9a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x105d9aa, SUC_lexicon_tags_37 },
    { 0x225d9aa, SUC_lexicon_tags_2 },
    { 0xec25d9ac, SUC_lexicon_tags_39 },
    { 0x4c5d9ad, SUC_lexicon_tags_25 },
    { 0x7065d9ad, SUC_lexicon_tags_1 },
    { 0xd825d9af, SUC_lexicon_tags_2 },
    { 0x72c5d9ae, SUC_lexicon_tags_44 },
    { 0xc165d9b1, SUC_lexicon_tags_0 },
    { 0xd145d9ae, SUC_lexicon_tags_17 },
    { 0x7465d9ac, SUC_lexicon_tags_15 },
    { 0xf8c5d9b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5405d9b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f85d9ba, SUC_lexicon_tags_37 },
    { 0xef65d9ba, SUC_lexicon_tags_11 },
    { 0xfd05d9bb, SUC_lexicon_tags_23 },
    { 0xa125d9bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8805d9c0, SUC_lexicon_tags_1 },
    { 0x7745d9c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9725d9c5, SUC_lexicon_tags_31 },
    { 0x2305d9c6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd85d9ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe405d9cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd645d9ce, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f45d9d2, SUC_lexicon_tags_14 },
    { 0xc985d9d3, SUC_lexicon_tags_2 },
    { 0xc6c5d9d3, SUC_lexicon_tags_2 },
    { 0xcd25d9d2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c5d9d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d25d9dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0a5d9de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6725d9e1, SUC_lexicon_tags_1 },
    { 0x9ac5d9e2, SUC_lexicon_tags_17 },
    { 0x90e5d9e3, SUC_lexicon_tags_38 },
    { 0xc7c5d9e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf8e5d9e6, SUC_lexicon_tags_49 },
    { 0x1785d9e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c5d9eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1565d9ef, SUC_lexicon_tags_2 },
    { 0xf7e5d9f0, SUC_lexicon_tags_11 },
    { 0x6865d9ef, SUC_lexicon_tags_37 },
    { 0x105d9f2, SUC_lexicon_tags_17 },
    { 0xf365d9f2, SUC_lexicon_tags_33 },
    { 0x7945d9f4, SUC_lexicon_tags_7 },
    { 0x105d9f5, SUC_lexicon_tags_25 },
    { 0xff05d9f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8085d9f9, SUC_lexicon_tags_17 },
    { 0x26e5d9fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7905da04, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e5da07, SUC_lexicon_tags_26 },
    { 0x4105da08, SUC_lexicon_tags_7 },
    { 0x9ae5da08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4c5da0b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe5da11, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe85da18, SUC_lexicon_tags_12 },
    { 0x94e5da18, SUC_lexicon_tags_63 },
    { 0xf0e5da1a, SUC_lexicon_tags_114 },
    { 0x5fe5da1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc45da1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c5da21, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac5da24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa05da27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865da2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8405da2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c5da35, SUC_lexicon_tags_25 },
    { 0xa05da36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb185da39, SUC_lexicon_tags_34 },
    { 0x40c5da39, SUC_lexicon_tags_17 },
    { 0x6ea5da3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9a65da3d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x3205da40, SUC_lexicon_tags_15 },
    { 0x4fa5da40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f85da46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd865da49, SUC_lexicon_tags_60 },
    { 0xe425da4a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5665da4f, SUC_lexicon_tags_92 },
    { 0xa6a5da4f, SUC_lexicon_tags_15 },
    { 0xb585da4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca65da58, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2505da5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da5da5d, SUC_lexicon_tags_10 },
    { 0x3605da5e, SUC_lexicon_tags_11 },
    { 0x6fa5da5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6745da61, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x45da64, SUC_lexicon_tags_7 },
    { 0x8665da65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e5da68, SUC_lexicon_tags_39 },
    { 0xf9a5da69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5685da6b, SUC_lexicon_tags_12 },
    { 0xfc45da6c, SUC_lexicon_tags_27 },
    { 0x7b85da6d, SUC_lexicon_tags_9 },
    { 0xc105da6e, SUC_lexicon_tags_19 },
    { 0xb9c5da6d, SUC_lexicon_tags_7 },
    { 0x6445da70, SUC_lexicon_tags_25 },
    { 0x43c5da71, SUC_lexicon_tags_12 },
    { 0x9fa5da72, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf5a5da74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e45da79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb425da7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5605da7f, SUC_lexicon_tags_14 },
    { 0xa1e5da80, SUC_lexicon_tags_26 },
    { 0x9c65da80, SUC_lexicon_tags_32 },
    { 0xc325da82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c5da85, SUC_lexicon_tags_7 },
    { 0xeb45da85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaac5da88, SUC_lexicon_tags_9 },
    { 0x2ce5da89, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4565da8c, SUC_lexicon_tags_32 },
    { 0x7f65da8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a5da91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a5da93, SUC_lexicon_tags_2 },
    { 0xcfe5da94, SUC_lexicon_tags_25 },
    { 0x9a5da93, SUC_lexicon_tags_17 },
    { 0xa8e5da93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3365da98, SUC_lexicon_tags_14 },
    { 0xf525da99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa0c5da9b, SUC_lexicon_tags_49 },
    { 0xd9c5da9c, SUC_lexicon_tags_37 },
    { 0xdc65da9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea05daab, SUC_lexicon_tags_45 },
    { 0x7425daac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ec5daae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ee5dab0, SUC_lexicon_tags_12 },
    { 0x99c5dab0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa985dab3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29c5dab5, SUC_lexicon_tags_47 },
    { 0x6a85dab6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e65dab9, SUC_lexicon_tags_2 },
    { 0x9965daba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ca5dabc, SUC_lexicon_tags_23 },
    { 0x1a5dabd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a5dac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba5dac6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9665dac9, SUC_lexicon_tags_1 },
    { 0x1d65dac9, SUC_lexicon_tags_10 },
    { 0x3225dacb, SUC_lexicon_tags_11 },
    { 0x75c5dacc, SUC_lexicon_tags_45 },
    { 0x4de5dacd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbf85dacf, SUC_lexicon_tags_27 },
    { 0xb425dad0, SUC_lexicon_tags_30 },
    { 0xb0a5dad1, SUC_lexicon_tags_1 },
    { 0x4de5dad1, SUC_lexicon_tags_60 },
    { 0xe0c5dad1, SUC_lexicon_tags_27 },
    { 0xaca5dad4, SUC_lexicon_tags_11 },
    { 0x765dad4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a45dad7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4125dad9, SUC_lexicon_tags_60 },
    { 0xa905dada, SUC_lexicon_tags_45 },
    { 0x6ce5dadb, SUC_lexicon_tags_25 },
    { 0x2fe5dadc, SUC_lexicon_tags_25 },
    { 0x985dadb, SUC_lexicon_tags_60 },
    { 0x4f45dadb, SUC_lexicon_tags_14 },
    { 0x23c5dadf, SUC_lexicon_tags_11 },
    { 0xa305dadc, SUC_lexicon_tags_37 },
    { 0x8545dadb, SUC_lexicon_tags_33 },
    { 0x31c5dada, SUC_lexicon_tags_3 },
    { 0x7c45dae3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a5daeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1605daee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9265daf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e5daf5, SUC_lexicon_tags_23 },
    { 0xc2c5daf5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc5dafc, SUC_lexicon_tags_11 },
    { 0xa345dafc, SUC_lexicon_tags_61 },
    { 0xbcc5dafc, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5db05, SUC_lexicon_tags_49 },
    { 0xea45db05, SUC_lexicon_tags_17 },
    { 0xce85db07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c5db0a, SUC_lexicon_tags_8 },
    { 0x3805db0b, SUC_lexicon_tags_17 },
    { 0xa425db0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba25db10, SUC_lexicon_tags_26 },
    { 0x4fc5db11, SUC_lexicon_tags_15 },
    { 0x4b25db12, SUC_lexicon_tags_465 },
    { 0, NULL },
    { 0xdb45db14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27c5db16, SUC_lexicon_tags_25 },
    { 0x73a5db17, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e5db1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19c5db1f, SUC_lexicon_tags_1188 },
    { 0x5545db20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9885db22, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaf65db24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7865db26, SUC_lexicon_tags_1 },
    { 0x2445db27, SUC_lexicon_tags_2 },
    { 0x2745db28, SUC_lexicon_tags_27 },
    { 0xdf45db29, SUC_lexicon_tags_11 },
    { 0x2d45db2a, SUC_lexicon_tags_3 },
    { 0x41c5db2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd485db2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa545db31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1405db37, SUC_lexicon_tags_15 },
    { 0x9d85db37, SUC_lexicon_tags_12 },
    { 0x9905db39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f45db3c, SUC_lexicon_tags_12 },
    { 0xe545db3c, SUC_lexicon_tags_7 },
    { 0xc225db3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8065db45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6765db49, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x39e5db4b, SUC_lexicon_tags_19 },
    { 0xcfe5db4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc05db4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c5db53, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2565db5a, SUC_lexicon_tags_15 },
    { 0x8245db5b, SUC_lexicon_tags_0 },
    { 0xf7a5db5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb685db5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc45db61, SUC_lexicon_tags_13 },
    { 0xa9c5db61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf345db65, SUC_lexicon_tags_11 },
    { 0xc405db66, SUC_lexicon_tags_14 },
    { 0x825db65, SUC_lexicon_tags_9 },
    { 0x8c25db68, SUC_lexicon_tags_26 },
    { 0x9545db69, SUC_lexicon_tags_172 },
    { 0x1d25db65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8e25db6c, SUC_lexicon_tags_79 },
    { 0x6545db6c, SUC_lexicon_tags_30 },
    { 0x725db6c, SUC_lexicon_tags_14 },
    { 0x9e85db6f, SUC_lexicon_tags_17 },
    { 0x4705db70, SUC_lexicon_tags_1 },
    { 0xafc5db6f, SUC_lexicon_tags_49 },
    { 0x2045db71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c5db77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd525db7a, SUC_lexicon_tags_26 },
    { 0x4b45db7a, SUC_lexicon_tags_27 },
    { 0xf945db7c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc5db7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe05db82, SUC_lexicon_tags_8 },
    { 0x8d45db83, SUC_lexicon_tags_49 },
    { 0x5a85db83, SUC_lexicon_tags_38 },
    { 0xcf05db83, SUC_lexicon_tags_16 },
    { 0xd265db86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3385db89, SUC_lexicon_tags_15 },
    { 0x72c5db89, SUC_lexicon_tags_43 },
    { 0xa665db89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9345db8e, SUC_lexicon_tags_37 },
    { 0x3c25db8e, SUC_lexicon_tags_15 },
    { 0xc345db8e, SUC_lexicon_tags_1 },
    { 0x8085db91, SUC_lexicon_tags_25 },
    { 0xffa5db92, SUC_lexicon_tags_17 },
    { 0xdf65db91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b25db96, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf7c5db98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78c5db9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e5db9d, SUC_lexicon_tags_8 },
    { 0xe445db9d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc5dba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e65dba5, SUC_lexicon_tags_2 },
    { 0x3d45dba5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c5dbaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb65dbae, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd925dbb2, SUC_lexicon_tags_29 },
    { 0x4e85dbb3, SUC_lexicon_tags_478 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f65dbb6, SUC_lexicon_tags_12 },
    { 0x22a5dbb7, SUC_lexicon_tags_2 },
    { 0xf765dbb8, SUC_lexicon_tags_47 },
    { 0xb0c5dbb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce5dbbd, SUC_lexicon_tags_9 },
    { 0x75a5dbbe, SUC_lexicon_tags_25 },
    { 0x3d85dbbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb185dbc5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8845dbc9, SUC_lexicon_tags_0 },
    { 0x6b65dbca, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xbe25dbcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c5dbd2, SUC_lexicon_tags_25 },
    { 0xd685dbd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd865dbd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5705dbd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda85dbd9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e5dbdc, SUC_lexicon_tags_60 },
    { 0x6605dbdd, SUC_lexicon_tags_38 },
    { 0x1b05dbde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e85dbe0, SUC_lexicon_tags_16 },
    { 0x3d25dbe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe85dbe5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc5dbe8, SUC_lexicon_tags_11 },
    { 0xfdc5dbe8, SUC_lexicon_tags_22 },
    { 0x56a5dbea, SUC_lexicon_tags_19 },
    { 0x76c5dbeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3025dbf0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2745dbf2, SUC_lexicon_tags_37 },
    { 0x5c85dbf3, SUC_lexicon_tags_2 },
    { 0xa985dbf3, SUC_lexicon_tags_11 },
    { 0x4ec5dbf2, SUC_lexicon_tags_65 },
    { 0x2ce5dbf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8c5dbf8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9025dbfa, SUC_lexicon_tags_37 },
    { 0x7165dbfb, SUC_lexicon_tags_11 },
    { 0x1765dbfa, SUC_lexicon_tags_6 },
    { 0xbac5dbfc, SUC_lexicon_tags_48 },
    { 0x8805dbfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7005dc05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c65dc0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d65dc0e, SUC_lexicon_tags_14 },
    { 0xf765dc0e, SUC_lexicon_tags_63 },
    { 0x80c5dc10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e5dc13, SUC_lexicon_tags_27 },
    { 0xa6c5dc13, SUC_lexicon_tags_7 },
    { 0x5525dc15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d05dc17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5a5dc19, SUC_lexicon_tags_7 },
    { 0x7ce5dc1a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505dc1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3725dc20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e5dc23, SUC_lexicon_tags_25 },
    { 0x6f45dc24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ba5dc26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf285dc29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc525dc2b, SUC_lexicon_tags_9 },
    { 0x5525dc2b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xace5dc2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e5dc3b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba25dc41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc5dc53, SUC_lexicon_tags_37 },
    { 0x7c85dc53, SUC_lexicon_tags_14 },
    { 0xbd65dc53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc5dc5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa905dc62, SUC_lexicon_tags_48 },
    { 0x2025dc63, SUC_lexicon_tags_25 },
    { 0x905dc63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e5dc68, SUC_lexicon_tags_7 },
    { 0xe065dc69, SUC_lexicon_tags_12 },
    { 0x4205dc68, SUC_lexicon_tags_0 },
    { 0x6b85dc6b, SUC_lexicon_tags_12 },
    { 0x585dc6b, SUC_lexicon_tags_10 },
    { 0x29e5dc6c, SUC_lexicon_tags_48 },
    { 0x1c25dc6e, SUC_lexicon_tags_14 },
    { 0xede5dc6c, SUC_lexicon_tags_17 },
    { 0x7605dc70, SUC_lexicon_tags_60 },
    { 0x4525dc6d, SUC_lexicon_tags_126 },
    { 0x785dc72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfe5dc74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9965dc77, SUC_lexicon_tags_12 },
    { 0x20e5dc77, SUC_lexicon_tags_47 },
    { 0xc4e5dc79, SUC_lexicon_tags_12 },
    { 0xb125dc79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba5dc7d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f65dc80, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe805dc83, SUC_lexicon_tags_2 },
    { 0xea05dc84, SUC_lexicon_tags_57 },
    { 0x45e5dc84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c5dc88, SUC_lexicon_tags_7 },
    { 0x9ae5dc89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa245dc8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad85dc90, SUC_lexicon_tags_37 },
    { 0x24a5dc90, SUC_lexicon_tags_19 },
    { 0xd5a5dc92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3cc5dc94, SUC_lexicon_tags_43 },
    { 0x9525dc95, SUC_lexicon_tags_11 },
    { 0x1cc5dc95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5dc9c, SUC_lexicon_tags_37 },
    { 0x75e5dc9c, SUC_lexicon_tags_37 },
    { 0xcfc5dc9d, SUC_lexicon_tags_37 },
    { 0xf265dc9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf05dca3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a5dca8, SUC_lexicon_tags_25 },
    { 0x72c5dca9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac5dcad, SUC_lexicon_tags_7 },
    { 0xfb85dcad, SUC_lexicon_tags_17 },
    { 0xd385dcaf, SUC_lexicon_tags_29 },
    { 0x44c5dcaf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc285dcb4, SUC_lexicon_tags_12 },
    { 0xd665dcb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa605dcb8, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa5dcbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd225dcc7, SUC_lexicon_tags_38 },
    { 0x8ae5dcc8, SUC_lexicon_tags_17 },
    { 0x6785dcc7, SUC_lexicon_tags_114 },
    { 0xe885dcca, SUC_lexicon_tags_25 },
    { 0xe965dccb, SUC_lexicon_tags_6 },
    { 0x9225dccc, SUC_lexicon_tags_35 },
    { 0xa705dccd, SUC_lexicon_tags_639 },
    { 0x1f85dccd, SUC_lexicon_tags_27 },
    { 0x25a5dcce, SUC_lexicon_tags_9 },
    { 0x3005dcd0, SUC_lexicon_tags_37 },
    { 0x86c5dccd, SUC_lexicon_tags_38 },
    { 0xefc5dcd2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f85dcd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1065dcd7, SUC_lexicon_tags_11 },
    { 0xf185dcd8, SUC_lexicon_tags_26 },
    { 0xabc5dcd8, SUC_lexicon_tags_63 },
    { 0xe1c5dcd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc885dcde, SUC_lexicon_tags_15 },
    { 0xfa25dcdf, SUC_lexicon_tags_15 },
    { 0x3185dce0, SUC_lexicon_tags_84 },
    { 0x1105dce0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc5dce4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a5dce7, SUC_lexicon_tags_10 },
    { 0xd185dce7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd565dcea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9ca5dcec, SUC_lexicon_tags_17 },
    { 0x8fa5dcec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x84a5dcef, SUC_lexicon_tags_31 },
    { 0x9645dcef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e5dcf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc65dcf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a5dcfe, SUC_lexicon_tags_37 },
    { 0xea45dcff, SUC_lexicon_tags_38 },
    { 0x12c5dd00, SUC_lexicon_tags_15 },
    { 0xde25dd01, SUC_lexicon_tags_37 },
    { 0xde45dd02, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2a85dd04, SUC_lexicon_tags_87 },
    { 0x2d25dd04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e45dd09, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8825dd0c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe5dd10, SUC_lexicon_tags_40 },
    { 0x1e25dd11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c85dd13, SUC_lexicon_tags_28 },
    { 0x5ce5dd14, SUC_lexicon_tags_11 },
    { 0xf465dd15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb305dd17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a05dd1e, SUC_lexicon_tags_49 },
    { 0xa705dd1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d05dd25, SUC_lexicon_tags_27 },
    { 0x9065dd26, SUC_lexicon_tags_12 },
    { 0x1a25dd27, SUC_lexicon_tags_11 },
    { 0x4a5dd28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05dd2f, SUC_lexicon_tags_25 },
    { 0xe6c5dd2f, SUC_lexicon_tags_11 },
    { 0xaf25dd31, SUC_lexicon_tags_46 },
    { 0x3045dd32, SUC_lexicon_tags_1 },
    { 0x685dd33, SUC_lexicon_tags_0 },
    { 0xa1a5dd34, SUC_lexicon_tags_0 },
    { 0x4e05dd35, SUC_lexicon_tags_60 },
    { 0xad05dd34, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e5dd39, SUC_lexicon_tags_70 },
    { 0x28c5dd39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf65dd3d, SUC_lexicon_tags_14 },
    { 0xe525dd3e, SUC_lexicon_tags_25 },
    { 0x4465dd3f, SUC_lexicon_tags_15 },
    { 0xc505dd3e, SUC_lexicon_tags_11 },
    { 0xfa65dd41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe5dd48, SUC_lexicon_tags_7 },
    { 0x8c45dd49, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe5dd4c, SUC_lexicon_tags_4 },
    { 0x30a5dd4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7065dd4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf05dd51, SUC_lexicon_tags_57 },
    { 0x7345dd51, SUC_lexicon_tags_15 },
    { 0xde05dd53, SUC_lexicon_tags_6 },
    { 0xdbc5dd52, SUC_lexicon_tags_26 },
    { 0xe3c5dd55, SUC_lexicon_tags_60 },
    { 0x8525dd56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc865dd5b, SUC_lexicon_tags_2 },
    { 0xc405dd5b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf365dd61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c5dd68, SUC_lexicon_tags_38 },
    { 0x3725dd69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb05dd6b, SUC_lexicon_tags_12 },
    { 0x54c5dd6c, SUC_lexicon_tags_27 },
    { 0xcb85dd6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1225dd6f, SUC_lexicon_tags_2 },
    { 0x8025dd70, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7105dd72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb625dd75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e5dd77, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd745dd7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c85dd86, SUC_lexicon_tags_1 },
    { 0x23c5dd87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace5dd8b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a5dd8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9005dd92, SUC_lexicon_tags_11 },
    { 0xa445dd92, SUC_lexicon_tags_8 },
    { 0x3505dd94, SUC_lexicon_tags_21 },
    { 0xa2c5dd95, SUC_lexicon_tags_43 },
    { 0x7285dd95, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda5dd99, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb925dd9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1025dd9f, SUC_lexicon_tags_38 },
    { 0xeac5dda0, SUC_lexicon_tags_29 },
    { 0x8205dda0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2485dda4, SUC_lexicon_tags_79 },
    { 0xe105dda5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa645dda7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c5ddb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f05ddb5, SUC_lexicon_tags_12 },
    { 0x2ae5ddb6, SUC_lexicon_tags_49 },
    { 0xe05ddb7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2645ddb9, SUC_lexicon_tags_39 },
    { 0x56e5ddba, SUC_lexicon_tags_11 },
    { 0xeac5ddb9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d65ddc2, SUC_lexicon_tags_18 },
    { 0x8c25ddc3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa045ddc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb85ddca, SUC_lexicon_tags_29 },
    { 0x1025ddcb, SUC_lexicon_tags_37 },
    { 0x9705ddcb, SUC_lexicon_tags_7 },
    { 0xf705ddcc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7c85ddcf, SUC_lexicon_tags_26 },
    { 0xe8a5ddd0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ddd3, SUC_lexicon_tags_10 },
    { 0x41a5ddd3, SUC_lexicon_tags_28 },
    { 0x1065ddd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea85ddd9, SUC_lexicon_tags_37 },
    { 0x48a5ddda, SUC_lexicon_tags_1 },
    { 0x5f65ddd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b25ddde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e05dde0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de5dde3, SUC_lexicon_tags_65 },
    { 0x3fa5dde3, SUC_lexicon_tags_28 },
    { 0xdf25dde5, SUC_lexicon_tags_25 },
    { 0x5665dde6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365dded, SUC_lexicon_tags_95 },
    { 0x7945ddee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe5ddf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a85ddf5, SUC_lexicon_tags_2 },
    { 0xebe5ddf6, SUC_lexicon_tags_38 },
    { 0x7fe5ddf7, SUC_lexicon_tags_44 },
    { 0x3a05ddf5, SUC_lexicon_tags_19 },
    { 0xe305ddf9, SUC_lexicon_tags_49 },
    { 0x2b25ddfa, SUC_lexicon_tags_25 },
    { 0x3605ddf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e05ddfd, SUC_lexicon_tags_11 },
    { 0x97e5ddfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e5de06, SUC_lexicon_tags_25 },
    { 0x9325de07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x705de09, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a45de0b, SUC_lexicon_tags_7 },
    { 0x4a25de0c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a05de0f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9285de13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9685de18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445de1f, SUC_lexicon_tags_53 },
    { 0x4ca5de20, SUC_lexicon_tags_16 },
    { 0x1ca5de21, SUC_lexicon_tags_1 },
    { 0x37e5de21, SUC_lexicon_tags_38 },
    { 0x8ac5de21, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe385de25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7245de27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6705de29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbba5de2b, SUC_lexicon_tags_47 },
    { 0xa065de2b, SUC_lexicon_tags_43 },
    { 0x63e5de2d, SUC_lexicon_tags_32 },
    { 0xeee5de2e, SUC_lexicon_tags_21 },
    { 0x4385de2f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5de3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe125de3f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe85de47, SUC_lexicon_tags_27 },
    { 0xd285de48, SUC_lexicon_tags_38 },
    { 0x8e85de49, SUC_lexicon_tags_8 },
    { 0xbd05de4a, SUC_lexicon_tags_68 },
    { 0xfb65de48, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc805de4e, SUC_lexicon_tags_510 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7985de52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e5de58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4705de5a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc5de5d, SUC_lexicon_tags_3 },
    { 0xe9e5de5e, SUC_lexicon_tags_6 },
    { 0x76c5de5f, SUC_lexicon_tags_21 },
    { 0x1185de60, SUC_lexicon_tags_55 },
    { 0x68e5de5d, SUC_lexicon_tags_1 },
    { 0x1545de60, SUC_lexicon_tags_16 },
    { 0x16c5de63, SUC_lexicon_tags_12 },
    { 0x6b05de64, SUC_lexicon_tags_23 },
    { 0x2905de60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3805de67, SUC_lexicon_tags_27 },
    { 0xbd05de68, SUC_lexicon_tags_9 },
    { 0x98c5de68, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e5de6f, SUC_lexicon_tags_6 },
    { 0x1325de6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d45de72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe325de77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb05de81, SUC_lexicon_tags_62 },
    { 0x34c5de82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c5de85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcaa5de87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee05de8b, SUC_lexicon_tags_37 },
    { 0xc6a5de8c, SUC_lexicon_tags_1 },
    { 0xd5c5de8d, SUC_lexicon_tags_11 },
    { 0x78c5de8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7925de91, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd965de93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7925de97, SUC_lexicon_tags_25 },
    { 0x7b05de98, SUC_lexicon_tags_29 },
    { 0xbce5de99, SUC_lexicon_tags_47 },
    { 0x6b05de98, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9125de9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c5dea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e65dea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c5deb1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf565deb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf325deb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9325debb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e5debe, SUC_lexicon_tags_7 },
    { 0xcb65debf, SUC_lexicon_tags_10 },
    { 0xeba5dec0, SUC_lexicon_tags_45 },
    { 0x7665debe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf705dec6, SUC_lexicon_tags_37 },
    { 0x4bc5dec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad85decb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e5dece, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45ded3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc865ded5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8685ded7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbae5ded9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd245dedc, SUC_lexicon_tags_68 },
    { 0x13a5dedd, SUC_lexicon_tags_31 },
    { 0x2645dede, SUC_lexicon_tags_17 },
    { 0xeb25dedd, SUC_lexicon_tags_37 },
    { 0xd6a5dedd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xaec5dee2, SUC_lexicon_tags_37 },
    { 0x9925dee2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2e25dee5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae5dee7, SUC_lexicon_tags_33 },
    { 0xa585dee8, SUC_lexicon_tags_14 },
    { 0x6da5dee9, SUC_lexicon_tags_21 },
    { 0x2565deea, SUC_lexicon_tags_14 },
    { 0x8ca5deea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa525deee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c5def0, SUC_lexicon_tags_37 },
    { 0x2685def1, SUC_lexicon_tags_37 },
    { 0x4005def2, SUC_lexicon_tags_66 },
    { 0x6c25def0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9265def5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c5def8, SUC_lexicon_tags_54 },
    { 0xb825def9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc5defd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa5df04, SUC_lexicon_tags_17 },
    { 0x5645df04, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0x2f65df07, SUC_lexicon_tags_61 },
    { 0x1e05df08, SUC_lexicon_tags_2 },
    { 0xb4a5df09, SUC_lexicon_tags_27 },
    { 0xa6a5df07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2c85df0c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd805df0f, SUC_lexicon_tags_61 },
    { 0x28e5df10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc65df13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8505df1a, SUC_lexicon_tags_12 },
    { 0xcaa5df1b, SUC_lexicon_tags_2 },
    { 0x4d85df1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd925df22, SUC_lexicon_tags_27 },
    { 0xda65df23, SUC_lexicon_tags_25 },
    { 0xb585df22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9605df29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e25df2d, SUC_lexicon_tags_10 },
    { 0x7505df2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c5df37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4045df3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe165df40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x825df42, SUC_lexicon_tags_14 },
    { 0xfd85df43, SUC_lexicon_tags_66 },
    { 0x6e65df43, SUC_lexicon_tags_33 },
    { 0xc005df45, SUC_lexicon_tags_15 },
    { 0x7d45df45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2365df49, SUC_lexicon_tags_12 },
    { 0x9f25df4a, SUC_lexicon_tags_14 },
    { 0xafc5df49, SUC_lexicon_tags_57 },
    { 0x3665df4c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4165df4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7325df57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b85df5a, SUC_lexicon_tags_2 },
    { 0x6a25df5b, SUC_lexicon_tags_1 },
    { 0xc5df5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b85df5f, SUC_lexicon_tags_11 },
    { 0x7d25df5f, SUC_lexicon_tags_36 },
    { 0x8945df60, SUC_lexicon_tags_33 },
    { 0xd785df62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc925df65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc385df72, SUC_lexicon_tags_92 },
    { 0xdc45df73, SUC_lexicon_tags_61 },
    { 0xcb45df73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e45df78, SUC_lexicon_tags_29 },
    { 0x9405df79, SUC_lexicon_tags_160 },
    { 0xdb65df7a, SUC_lexicon_tags_7 },
    { 0xdda5df7b, SUC_lexicon_tags_8 },
    { 0x7305df7c, SUC_lexicon_tags_14 },
    { 0x50a5df7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a5df81, SUC_lexicon_tags_1 },
    { 0xc2a5df82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7d65df84, SUC_lexicon_tags_20 },
    { 0xb045df84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a5df8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a45df94, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda5df99, SUC_lexicon_tags_0 },
    { 0x2065df9a, SUC_lexicon_tags_29 },
    { 0xe0c5df9a, SUC_lexicon_tags_15 },
    { 0xf65df9c, SUC_lexicon_tags_47 },
    { 0x59c5df9d, SUC_lexicon_tags_31 },
    { 0xf365df9e, SUC_lexicon_tags_14 },
    { 0x9f25df9f, SUC_lexicon_tags_37 },
    { 0x3165df9b, SUC_lexicon_tags_31 },
    { 0x7645dfa1, SUC_lexicon_tags_35 },
    { 0x3e05dfa1, SUC_lexicon_tags_17 },
    { 0xa1e5dfa3, SUC_lexicon_tags_31 },
    { 0x525df9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5185dfa8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9dc5dfaa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d05dfad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ac5dfaf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c5dfb3, SUC_lexicon_tags_82 },
    { 0xe3c5dfb4, SUC_lexicon_tags_11 },
    { 0x1ac5dfb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1305dfba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde05dfbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5dfc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5865dfc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e5dfca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8505dfd3, SUC_lexicon_tags_45 },
    { 0xa5c5dfd4, SUC_lexicon_tags_197 },
    { 0x8ec5dfd5, SUC_lexicon_tags_21 },
    { 0xbe85dfd3, SUC_lexicon_tags_2 },
    { 0x7fc5dfd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5965dfd9, SUC_lexicon_tags_15 },
    { 0xc945dfda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6165dfdc, SUC_lexicon_tags_47 },
    { 0x32e5dfdd, SUC_lexicon_tags_15 },
    { 0x82c5dfdc, SUC_lexicon_tags_49 },
    { 0x88a5dfde, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c5dfe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2805dfe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62e5dfea, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe5dfee, SUC_lexicon_tags_37 },
    { 0xfe85dfef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49a5dff1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa45dff3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc445dff6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa545dffc, SUC_lexicon_tags_11 },
    { 0x4985dffd, SUC_lexicon_tags_15 },
    { 0xc6e5dffe, SUC_lexicon_tags_2 },
    { 0xd3a5dfff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a5e004, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9525e006, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8dc5e008, SUC_lexicon_tags_2 },
    { 0x5a25e009, SUC_lexicon_tags_1 },
    { 0x95e5e009, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a25e00d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e5e010, SUC_lexicon_tags_13 },
    { 0xd085e011, SUC_lexicon_tags_7 },
    { 0x2be5e012, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c85e016, SUC_lexicon_tags_45 },
    { 0xe585e016, SUC_lexicon_tags_27 },
    { 0xc445e018, SUC_lexicon_tags_33 },
    { 0x4305e018, SUC_lexicon_tags_47 },
    { 0xf625e01a, SUC_lexicon_tags_47 },
    { 0x2465e01a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5765e01e, SUC_lexicon_tags_2 },
    { 0x1505e01f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25e027, SUC_lexicon_tags_2 },
    { 0x29a5e028, SUC_lexicon_tags_47 },
    { 0x5045e028, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1dc5e02b, SUC_lexicon_tags_79 },
    { 0x8625e02c, SUC_lexicon_tags_11 },
    { 0xb165e02d, SUC_lexicon_tags_2 },
    { 0xf925e02c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4465e033, SUC_lexicon_tags_11 },
    { 0x80a5e034, SUC_lexicon_tags_14 },
    { 0xe1e5e034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4665e038, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x5925e03a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xac45e03d, SUC_lexicon_tags_11 },
    { 0xc6c5e03e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3885e041, SUC_lexicon_tags_17 },
    { 0xa505e041, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a65e045, SUC_lexicon_tags_66 },
    { 0xc4c5e045, SUC_lexicon_tags_60 },
    { 0xf305e045, SUC_lexicon_tags_11 },
    { 0xc2a5e048, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a5e04b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7765e04d, SUC_lexicon_tags_1 },
    { 0x9ae5e04e, SUC_lexicon_tags_15 },
    { 0x73c5e04f, SUC_lexicon_tags_84 },
    { 0xc05e04f, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce5e057, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc065e059, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8c45e05b, SUC_lexicon_tags_25 },
    { 0x5aa5e05b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4065e05f, SUC_lexicon_tags_38 },
    { 0xb9a5e060, SUC_lexicon_tags_14 },
    { 0x5925e060, SUC_lexicon_tags_47 },
    { 0x61c5e062, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505e066, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f25e06a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b85e06e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe985e073, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5065e075, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf145e07a, SUC_lexicon_tags_15 },
    { 0x7ca5e07b, SUC_lexicon_tags_46 },
    { 0x1e65e07c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x145e07f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x305e081, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7a5e083, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xce45e085, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8425e08e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1745e093, SUC_lexicon_tags_12 },
    { 0xbe5e094, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xa1a5e096, SUC_lexicon_tags_27 },
    { 0xab65e097, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba5e09a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2825e09d, SUC_lexicon_tags_47 },
    { 0x12e5e09d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc745e0a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc5e0a7, SUC_lexicon_tags_4 },
    { 0x8ec5e0a8, SUC_lexicon_tags_17 },
    { 0x85e0a9, SUC_lexicon_tags_17 },
    { 0x2465e0aa, SUC_lexicon_tags_30 },
    { 0x7b25e0ab, SUC_lexicon_tags_17 },
    { 0x5705e0ac, SUC_lexicon_tags_2 },
    { 0x2b25e0aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe925e0b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e5e0b5, SUC_lexicon_tags_27 },
    { 0xc0a5e0b5, SUC_lexicon_tags_37 },
    { 0x3d85e0b7, SUC_lexicon_tags_12 },
    { 0x2225e0b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee5e0bb, SUC_lexicon_tags_2 },
    { 0xc485e0bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xab45e0be, SUC_lexicon_tags_15 },
    { 0x8705e0be, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da5e0c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xab65e0ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b65e0ce, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c5e0d1, SUC_lexicon_tags_11 },
    { 0x4a5e0d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa465e0d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8165e0da, SUC_lexicon_tags_23 },
    { 0x72e5e0db, SUC_lexicon_tags_48 },
    { 0x6fa5e0db, SUC_lexicon_tags_45 },
    { 0x6c85e0dc, SUC_lexicon_tags_10 },
    { 0xb4a5e0de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d05e0e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7785e0e4, SUC_lexicon_tags_2 },
    { 0xe605e0e5, SUC_lexicon_tags_2 },
    { 0x36c5e0e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6705e0ed, SUC_lexicon_tags_7 },
    { 0x7365e0ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe865e0f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a5e0f8, SUC_lexicon_tags_2 },
    { 0xb005e0f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a5e0fc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c5e0ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece5e107, SUC_lexicon_tags_8 },
    { 0x265e107, SUC_lexicon_tags_9 },
    { 0x5085e109, SUC_lexicon_tags_23 },
    { 0x3e45e10a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe465e10c, SUC_lexicon_tags_33 },
    { 0x325e10c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba5e110, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca5e113, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9885e116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e5e119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28c5e11b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a65e122, SUC_lexicon_tags_60 },
    { 0x8265e122, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f45e12d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf185e12f, SUC_lexicon_tags_3 },
    { 0x3085e12f, SUC_lexicon_tags_7 },
    { 0x63e5e131, SUC_lexicon_tags_7 },
    { 0x245e131, SUC_lexicon_tags_17 },
    { 0x5cc5e133, SUC_lexicon_tags_0 },
    { 0xc685e132, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e5e137, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e5e13a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf345e142, SUC_lexicon_tags_664 },
    { 0x5ca5e143, SUC_lexicon_tags_2 },
    { 0x8f25e144, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fa5e146, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e5e14e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6645e150, SUC_lexicon_tags_91 },
    { 0x9185e151, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9745e155, SUC_lexicon_tags_10 },
    { 0x5465e155, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf845e15b, SUC_lexicon_tags_63 },
    { 0x1aa5e15c, SUC_lexicon_tags_15 },
    { 0x7745e15d, SUC_lexicon_tags_29 },
    { 0x6165e15e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e05e161, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5765e165, SUC_lexicon_tags_37 },
    { 0xa725e165, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf45e169, SUC_lexicon_tags_14 },
    { 0x4245e16a, SUC_lexicon_tags_25 },
    { 0xb0a5e16b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce05e171, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3265e175, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2625e17a, SUC_lexicon_tags_0 },
    { 0xc1e5e17b, SUC_lexicon_tags_25 },
    { 0x8605e17b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c5e182, SUC_lexicon_tags_1 },
    { 0xcc5e182, SUC_lexicon_tags_17 },
    { 0x80e5e184, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc525e188, SUC_lexicon_tags_1 },
    { 0x26a5e189, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd605e18c, SUC_lexicon_tags_1 },
    { 0x63c5e18d, SUC_lexicon_tags_1 },
    { 0xc725e18e, SUC_lexicon_tags_37 },
    { 0x3385e18e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85e5e191, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb605e196, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e5e19b, SUC_lexicon_tags_7 },
    { 0x2345e19c, SUC_lexicon_tags_0 },
    { 0xbd65e19d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99e5e19f, SUC_lexicon_tags_39 },
    { 0x76c5e1a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f65e1a3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac25e1a7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3625e1ac, SUC_lexicon_tags_19 },
    { 0x1645e1ad, SUC_lexicon_tags_37 },
    { 0xa025e1ae, SUC_lexicon_tags_38 },
    { 0xb785e1ac, SUC_lexicon_tags_37 },
    { 0x21c5e1b0, SUC_lexicon_tags_185 },
    { 0xe165e1b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45e1b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf745e1c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c5e1ca, SUC_lexicon_tags_11 },
    { 0x6425e1cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc5e1ce, SUC_lexicon_tags_191 },
    { 0x7065e1ce, SUC_lexicon_tags_49 },
    { 0x6165e1d0, SUC_lexicon_tags_12 },
    { 0xaac5e1d1, SUC_lexicon_tags_11 },
    { 0xd45e1d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e5e1d9, SUC_lexicon_tags_16 },
    { 0x5925e1da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1465e1e3, SUC_lexicon_tags_84 },
    { 0x58e5e1e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9225e1e6, SUC_lexicon_tags_27 },
    { 0xa905e1e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc545e1ed, SUC_lexicon_tags_29 },
    { 0xcca5e1ee, SUC_lexicon_tags_12 },
    { 0xde85e1ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd805e1f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe5e1f4, SUC_lexicon_tags_46 },
    { 0xe1a5e1f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e85e1f8, SUC_lexicon_tags_15 },
    { 0x41a5e1f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c5e1ff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac5e202, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc65e206, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d05e20b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc245e20d, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x3985e20f, SUC_lexicon_tags_55 },
    { 0xc6c5e210, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf125e212, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd05e218, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc65e21e, SUC_lexicon_tags_37 },
    { 0xa5c5e21e, SUC_lexicon_tags_11 },
    { 0x3805e21e, SUC_lexicon_tags_17 },
    { 0x6845e221, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de5e225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x425e227, SUC_lexicon_tags_17 },
    { 0x92a5e227, SUC_lexicon_tags_27 },
    { 0x94c5e227, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85e231, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa25e23a, SUC_lexicon_tags_19 },
    { 0xf645e23b, SUC_lexicon_tags_47 },
    { 0xf6e5e23c, SUC_lexicon_tags_17 },
    { 0x9a5e23d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8b25e23f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39e5e241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2dc5e243, SUC_lexicon_tags_38 },
    { 0xcba5e244, SUC_lexicon_tags_25 },
    { 0x8305e245, SUC_lexicon_tags_6 },
    { 0xe45e245, SUC_lexicon_tags_11 },
    { 0x5e25e245, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5985e24b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd645e250, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfdc5e252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5205e255, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63e5e257, SUC_lexicon_tags_11 },
    { 0xc665e258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ae5e25a, SUC_lexicon_tags_43 },
    { 0xe4c5e25a, SUC_lexicon_tags_27 },
    { 0xbec5e25c, SUC_lexicon_tags_0 },
    { 0x90a5e25d, SUC_lexicon_tags_301 },
    { 0x9065e25d, SUC_lexicon_tags_31 },
    { 0xa3c5e25f, SUC_lexicon_tags_14 },
    { 0xe7e5e260, SUC_lexicon_tags_17 },
    { 0xb645e261, SUC_lexicon_tags_11 },
    { 0x5005e25c, SUC_lexicon_tags_12 },
    { 0x50a5e262, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc5e266, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8a5e268, SUC_lexicon_tags_2 },
    { 0x57c5e268, SUC_lexicon_tags_12 },
    { 0x3265e26a, SUC_lexicon_tags_25 },
    { 0x6325e26b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1265e26f, SUC_lexicon_tags_62 },
    { 0xf4c5e270, SUC_lexicon_tags_25 },
    { 0xf065e271, SUC_lexicon_tags_37 },
    { 0x3e65e271, SUC_lexicon_tags_82 },
    { 0x48e5e273, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde5e276, SUC_lexicon_tags_13 },
    { 0x7045e277, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a5e27f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x66e5e281, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1965e289, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1165e28e, SUC_lexicon_tags_11 },
    { 0x3a85e28e, SUC_lexicon_tags_14 },
    { 0x9f05e290, SUC_lexicon_tags_33 },
    { 0x40a5e28f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c5e297, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9be5e299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1925e29b, SUC_lexicon_tags_7 },
    { 0xb805e29c, SUC_lexicon_tags_91 },
    { 0xc965e29d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a05e2a0, SUC_lexicon_tags_19 },
    { 0xa3a5e2a1, SUC_lexicon_tags_47 },
    { 0x5a85e2a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9265e2a9, SUC_lexicon_tags_17 },
    { 0xfde5e2aa, SUC_lexicon_tags_0 },
    { 0xccc5e2ab, SUC_lexicon_tags_47 },
    { 0x3cc5e2aa, SUC_lexicon_tags_43 },
    { 0xfdc5e2ad, SUC_lexicon_tags_32 },
    { 0x3a5e2ae, SUC_lexicon_tags_46 },
    { 0x11c5e2ac, SUC_lexicon_tags_37 },
    { 0x9365e2ab, SUC_lexicon_tags_343 },
    { 0xb8a5e2ae, SUC_lexicon_tags_6 },
    { 0x8fe5e2ad, SUC_lexicon_tags_25 },
    { 0x3a45e2a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce5e2b6, SUC_lexicon_tags_12 },
    { 0xe0a5e2b6, SUC_lexicon_tags_31 },
    { 0x5585e2b8, SUC_lexicon_tags_11 },
    { 0xad45e2b9, SUC_lexicon_tags_12 },
    { 0x8b65e2b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa505e2be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4025e2c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5465e2cd, SUC_lexicon_tags_16 },
    { 0x58a5e2ce, SUC_lexicon_tags_60 },
    { 0x1c05e2cf, SUC_lexicon_tags_8 },
    { 0x5465e2d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a5e2d4, SUC_lexicon_tags_12 },
    { 0x89c5e2d5, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x45c5e2d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ea5e2d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b25e2e2, SUC_lexicon_tags_17 },
    { 0x80a5e2e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17e5e2e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc945e2e7, SUC_lexicon_tags_12 },
    { 0x1a25e2e8, SUC_lexicon_tags_34 },
    { 0x23a5e2e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ae5e2eb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x30a5e2ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa885e2ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf825e2f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc5e2fa, SUC_lexicon_tags_7 },
    { 0x9a65e2fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeae5e2fd, SUC_lexicon_tags_37 },
    { 0x6fc5e2fd, SUC_lexicon_tags_27 },
    { 0x2365e2ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf425e302, SUC_lexicon_tags_33 },
    { 0x92a5e303, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c5e306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca65e310, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93c5e312, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4845e319, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4e5e31b, SUC_lexicon_tags_11 },
    { 0x5de5e31b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f65e31e, SUC_lexicon_tags_15 },
    { 0x9de5e31f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb45e321, SUC_lexicon_tags_31 },
    { 0x8305e322, SUC_lexicon_tags_37 },
    { 0x9c45e322, SUC_lexicon_tags_25 },
    { 0xb4c5e324, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe5e329, SUC_lexicon_tags_1 },
    { 0xd6a5e32a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc325e32c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a05e331, SUC_lexicon_tags_551 },
    { 0x70e5e331, SUC_lexicon_tags_34 },
    { 0xc4a5e332, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd445e335, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c5e338, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2465e33b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c65e33f, SUC_lexicon_tags_15 },
    { 0xfd65e340, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6b25e342, SUC_lexicon_tags_15 },
    { 0x9b05e343, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaee5e345, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xca45e347, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a5e34a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec5e34f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45e351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1625e354, SUC_lexicon_tags_12 },
    { 0xedc5e354, SUC_lexicon_tags_39 },
    { 0x8ea5e356, SUC_lexicon_tags_6 },
    { 0x68c5e356, SUC_lexicon_tags_102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c5e35b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f45e35e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf25e360, SUC_lexicon_tags_37 },
    { 0x4825e361, SUC_lexicon_tags_37 },
    { 0x1985e362, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1625e366, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c25e36b, SUC_lexicon_tags_13 },
    { 0xfc45e36c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd145e36f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf25e373, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e5e378, SUC_lexicon_tags_39 },
    { 0x15e5e379, SUC_lexicon_tags_47 },
    { 0x7aa5e379, SUC_lexicon_tags_10 },
    { 0xd265e37b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f65e37f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa2c5e381, SUC_lexicon_tags_7 },
    { 0x125e382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd685e386, SUC_lexicon_tags_32 },
    { 0x8aa5e386, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4b05e389, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c5e38e, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c05e395, SUC_lexicon_tags_14 },
    { 0x8ba5e395, SUC_lexicon_tags_7 },
    { 0x5e45e397, SUC_lexicon_tags_11 },
    { 0xc805e398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x185e39b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a25e39e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4c5e3a0, SUC_lexicon_tags_37 },
    { 0x3e5e3a0, SUC_lexicon_tags_95 },
    { 0xa785e3a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c5e3a9, SUC_lexicon_tags_17 },
    { 0x8365e3a9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485e3af, SUC_lexicon_tags_17 },
    { 0x79c5e3af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf045e3b4, SUC_lexicon_tags_1 },
    { 0x1205e3b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a5e3bd, SUC_lexicon_tags_11 },
    { 0xf045e3bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a5e3c2, SUC_lexicon_tags_15 },
    { 0x95e5e3c3, SUC_lexicon_tags_12 },
    { 0x4805e3c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b45e3c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8345e3d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7105e3d5, SUC_lexicon_tags_23 },
    { 0x9725e3d5, SUC_lexicon_tags_14 },
    { 0x1bc5e3d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e5e3df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd265e3e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c5e3e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8045e3e8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e5e3ec, SUC_lexicon_tags_26 },
    { 0xed45e3ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a5e3f0, SUC_lexicon_tags_11 },
    { 0xea05e3f1, SUC_lexicon_tags_12 },
    { 0x65e5e3f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6585e3f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa25e3f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a25e401, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e5e404, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e5e408, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc5e411, SUC_lexicon_tags_37 },
    { 0x8f45e412, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c5e41a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x325e41c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x125e41e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed65e422, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb585e426, SUC_lexicon_tags_145 },
    { 0x66e5e427, SUC_lexicon_tags_1 },
    { 0x9125e426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c25e42e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa5e432, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8465e435, SUC_lexicon_tags_3 },
    { 0xb165e435, SUC_lexicon_tags_27 },
    { 0xa345e436, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf885e43c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9e5e43e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4525e444, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa585e446, SUC_lexicon_tags_15 },
    { 0x1ea5e446, SUC_lexicon_tags_17 },
    { 0x2e65e447, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e5e44d, SUC_lexicon_tags_26 },
    { 0x9045e44e, SUC_lexicon_tags_23 },
    { 0x6485e44f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8085e451, SUC_lexicon_tags_43 },
    { 0xa2c5e451, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1505e454, SUC_lexicon_tags_35 },
    { 0x85e454, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae5e45f, SUC_lexicon_tags_47 },
    { 0x4865e45f, SUC_lexicon_tags_25 },
    { 0x8745e461, SUC_lexicon_tags_27 },
    { 0xa205e462, SUC_lexicon_tags_2 },
    { 0x7fc5e463, SUC_lexicon_tags_12 },
    { 0xb205e460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8605e466, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1525e46f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8205e471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe5e474, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e5e477, SUC_lexicon_tags_68 },
    { 0xd5a5e478, SUC_lexicon_tags_37 },
    { 0x2265e477, SUC_lexicon_tags_79 },
    { 0x5685e47a, SUC_lexicon_tags_1 },
    { 0x5185e47b, SUC_lexicon_tags_19 },
    { 0xbe5e47c, SUC_lexicon_tags_7 },
    { 0xe145e47d, SUC_lexicon_tags_63 },
    { 0x385e47e, SUC_lexicon_tags_39 },
    { 0x10a5e47f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa5e483, SUC_lexicon_tags_25 },
    { 0x3365e484, SUC_lexicon_tags_2 },
    { 0x8545e483, SUC_lexicon_tags_28 },
    { 0x7d25e486, SUC_lexicon_tags_7 },
    { 0x9a85e483, SUC_lexicon_tags_22 },
    { 0xf885e485, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d05e48d, SUC_lexicon_tags_37 },
    { 0x2cc5e48e, SUC_lexicon_tags_33 },
    { 0x3c85e48f, SUC_lexicon_tags_2 },
    { 0xb805e490, SUC_lexicon_tags_12 },
    { 0x5f25e491, SUC_lexicon_tags_63 },
    { 0xc1a5e48f, SUC_lexicon_tags_12 },
    { 0xe005e493, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7545e495, SUC_lexicon_tags_45 },
    { 0x7925e496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x305e498, SUC_lexicon_tags_15 },
    { 0xd0c5e499, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9785e49b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeee5e49d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2325e49f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3045e4a1, SUC_lexicon_tags_30 },
    { 0xd425e4a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe05e4a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd45e4a8, SUC_lexicon_tags_6 },
    { 0xbb25e4a9, SUC_lexicon_tags_37 },
    { 0x2905e4a8, SUC_lexicon_tags_37 },
    { 0x1a05e4aa, SUC_lexicon_tags_19 },
    { 0xe3a5e4ac, SUC_lexicon_tags_48 },
    { 0xb6e5e4a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2905e4b2, SUC_lexicon_tags_49 },
    { 0x8685e4b3, SUC_lexicon_tags_100 },
    { 0x4685e4b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c5e4b7, SUC_lexicon_tags_66 },
    { 0xb525e4b8, SUC_lexicon_tags_47 },
    { 0xf45e4b9, SUC_lexicon_tags_77 },
    { 0xfc25e4b7, SUC_lexicon_tags_49 },
    { 0xb765e4b7, SUC_lexicon_tags_47 },
    { 0x8625e4bb, SUC_lexicon_tags_108 },
    { 0xcec5e4bc, SUC_lexicon_tags_12 },
    { 0x4465e4bc, SUC_lexicon_tags_12 },
    { 0x3be5e4bd, SUC_lexicon_tags_0 },
    { 0xb1c5e4c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc325e4c5, SUC_lexicon_tags_37 },
    { 0x4e85e4c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd045e4c8, SUC_lexicon_tags_12 },
    { 0xaaa5e4c9, SUC_lexicon_tags_3 },
    { 0x4405e4c9, SUC_lexicon_tags_6 },
    { 0x2ce5e4cb, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1505e4d0, SUC_lexicon_tags_10 },
    { 0x1245e4d0, SUC_lexicon_tags_17 },
    { 0x9d65e4d2, SUC_lexicon_tags_7 },
    { 0xad45e4d2, SUC_lexicon_tags_2 },
    { 0x1d05e4d2, SUC_lexicon_tags_2 },
    { 0xde5e4d2, SUC_lexicon_tags_2 },
    { 0x5c65e4d6, SUC_lexicon_tags_47 },
    { 0x9325e4d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d65e4d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b65e4e1, SUC_lexicon_tags_37 },
    { 0x5525e4e2, SUC_lexicon_tags_17 },
    { 0x8125e4e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea5e4e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec5e4eb, SUC_lexicon_tags_8 },
    { 0xc865e4ec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6a25e4ee, SUC_lexicon_tags_15 },
    { 0xc285e4ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4725e4f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf7c5e4f8, SUC_lexicon_tags_46 },
    { 0xad25e4f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a5e4fc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e5e4ff, SUC_lexicon_tags_11 },
    { 0x1205e500, SUC_lexicon_tags_63 },
    { 0xa3c5e501, SUC_lexicon_tags_15 },
    { 0x8925e4ff, SUC_lexicon_tags_63 },
    { 0xb0a5e4ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe905e508, SUC_lexicon_tags_37 },
    { 0x65e509, SUC_lexicon_tags_17 },
    { 0x8465e50a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe165e50d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e5e512, SUC_lexicon_tags_82 },
    { 0xe565e513, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6745e515, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe845e518, SUC_lexicon_tags_7 },
    { 0x8ba5e518, SUC_lexicon_tags_0 },
    { 0x5b25e51a, SUC_lexicon_tags_7 },
    { 0xcd85e51b, SUC_lexicon_tags_49 },
    { 0xb85e51b, SUC_lexicon_tags_37 },
    { 0xd625e51b, SUC_lexicon_tags_6 },
    { 0xbee5e518, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e5e521, SUC_lexicon_tags_11 },
    { 0x76c5e522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8665e525, SUC_lexicon_tags_60 },
    { 0x62a5e526, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc5e52c, SUC_lexicon_tags_14 },
    { 0x2f45e52d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xec5e52f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2c5e531, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x325e533, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce25e535, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da5e539, SUC_lexicon_tags_11 },
    { 0x65a5e539, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6005e53c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf425e53f, SUC_lexicon_tags_14 },
    { 0xf7e5e540, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1185e543, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2105e54b, SUC_lexicon_tags_28 },
    { 0xd4a5e54c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbea5e54e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd425e551, SUC_lexicon_tags_9 },
    { 0x5fc5e552, SUC_lexicon_tags_2 },
    { 0x3305e552, SUC_lexicon_tags_37 },
    { 0x1345e554, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf505e558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda5e55e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d65e560, SUC_lexicon_tags_54 },
    { 0xc4c5e561, SUC_lexicon_tags_12 },
    { 0xacc5e561, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d65e569, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54e5e56b, SUC_lexicon_tags_7 },
    { 0xbae5e56c, SUC_lexicon_tags_32 },
    { 0xc145e56c, SUC_lexicon_tags_1 },
    { 0x24a5e56e, SUC_lexicon_tags_12 },
    { 0x4725e56f, SUC_lexicon_tags_1 },
    { 0x29a5e56e, SUC_lexicon_tags_27 },
    { 0xdca5e56b, SUC_lexicon_tags_8 },
    { 0x145e572, SUC_lexicon_tags_45 },
    { 0x1a05e571, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2d65e575, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e5e579, SUC_lexicon_tags_69 },
    { 0xdc5e57a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa945e57d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4d25e57f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e5e582, SUC_lexicon_tags_37 },
    { 0xf05e582, SUC_lexicon_tags_18 },
    { 0x285e584, SUC_lexicon_tags_12 },
    { 0x51c5e583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc845e588, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7365e58b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4325e58e, SUC_lexicon_tags_1 },
    { 0xb45e58f, SUC_lexicon_tags_19 },
    { 0xc845e590, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8905e592, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa5e597, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5125e599, SUC_lexicon_tags_1 },
    { 0x1025e59a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf805e5a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x765e5a6, SUC_lexicon_tags_148 },
    { 0xb045e5a7, SUC_lexicon_tags_25 },
    { 0x5e85e5a8, SUC_lexicon_tags_2 },
    { 0x4865e5a8, SUC_lexicon_tags_43 },
    { 0xb025e5a9, SUC_lexicon_tags_33 },
    { 0xa325e5a9, SUC_lexicon_tags_27 },
    { 0x5105e5ac, SUC_lexicon_tags_46 },
    { 0x7005e5ad, SUC_lexicon_tags_0 },
    { 0x1a05e5ac, SUC_lexicon_tags_17 },
    { 0x3c25e5ab, SUC_lexicon_tags_11 },
    { 0x3745e5b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4945e5b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8de5e5b7, SUC_lexicon_tags_2 },
    { 0xac65e5b8, SUC_lexicon_tags_11 },
    { 0xeea5e5b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5a5e5bb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb505e5c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6ac5e5c2, SUC_lexicon_tags_43 },
    { 0xf985e5c3, SUC_lexicon_tags_17 },
    { 0x5d45e5c4, SUC_lexicon_tags_37 },
    { 0x36a5e5c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85e5c8, SUC_lexicon_tags_1 },
    { 0x5145e5c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x94a5e5cb, SUC_lexicon_tags_63 },
    { 0xdae5e5cb, SUC_lexicon_tags_27 },
    { 0x6785e5cd, SUC_lexicon_tags_47 },
    { 0x8545e5cd, SUC_lexicon_tags_17 },
    { 0x6a45e5cd, SUC_lexicon_tags_26 },
    { 0x1a85e5cc, SUC_lexicon_tags_60 },
    { 0x3965e5d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa245e5d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae45e5d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe605e5d9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e5e5e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd805e5e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4045e5ed, SUC_lexicon_tags_12 },
    { 0x7a85e5ed, SUC_lexicon_tags_170 },
    { 0xc985e5ef, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee5e5f6, SUC_lexicon_tags_15 },
    { 0x4a25e5f7, SUC_lexicon_tags_14 },
    { 0x3e05e5f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1285e5ff, SUC_lexicon_tags_37 },
    { 0xa285e5ff, SUC_lexicon_tags_1 },
    { 0x14c5e601, SUC_lexicon_tags_15 },
    { 0x1e45e602, SUC_lexicon_tags_38 },
    { 0xcf25e5ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9685e606, SUC_lexicon_tags_11 },
    { 0x8e65e607, SUC_lexicon_tags_26 },
    { 0xde85e606, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa5e60f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5f45e611, SUC_lexicon_tags_28 },
    { 0x88a5e612, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9045e615, SUC_lexicon_tags_63 },
    { 0x2525e616, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca25e61b, SUC_lexicon_tags_12 },
    { 0x4c05e61b, SUC_lexicon_tags_37 },
    { 0x885e61b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4445e61f, SUC_lexicon_tags_11 },
    { 0x9865e620, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9045e62d, SUC_lexicon_tags_0 },
    { 0xc245e62e, SUC_lexicon_tags_37 },
    { 0x6a25e62f, SUC_lexicon_tags_21 },
    { 0xe585e630, SUC_lexicon_tags_15 },
    { 0xaf05e62e, SUC_lexicon_tags_12 },
    { 0xd7c5e632, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc5e635, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a5e638, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5865e641, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda25e644, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e5e647, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e5e64d, SUC_lexicon_tags_29 },
    { 0xaf25e64e, SUC_lexicon_tags_11 },
    { 0xe45e64d, SUC_lexicon_tags_7 },
    { 0xb3a5e650, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a5e655, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbe5e657, SUC_lexicon_tags_27 },
    { 0xda05e658, SUC_lexicon_tags_26 },
    { 0xb485e659, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a5e65e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5a5e660, SUC_lexicon_tags_37 },
    { 0x2b65e661, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c05e66a, SUC_lexicon_tags_45 },
    { 0xa985e66b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed65e66d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a5e671, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa5e676, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e5e679, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe25e67c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e5e67f, SUC_lexicon_tags_10 },
    { 0x7d45e67f, SUC_lexicon_tags_14 },
    { 0x5465e680, SUC_lexicon_tags_37 },
    { 0xe165e682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7445e684, SUC_lexicon_tags_61 },
    { 0x68e5e684, SUC_lexicon_tags_2 },
    { 0x345e684, SUC_lexicon_tags_10 },
    { 0xa225e687, SUC_lexicon_tags_38 },
    { 0xbfa5e688, SUC_lexicon_tags_23 },
    { 0x91c5e688, SUC_lexicon_tags_33 },
    { 0x9565e686, SUC_lexicon_tags_37 },
    { 0x7fa5e684, SUC_lexicon_tags_17 },
    { 0xcfe5e68c, SUC_lexicon_tags_33 },
    { 0xed25e68c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xece5e690, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e693, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb825e696, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa725e698, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc725e69d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05e6a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fe5e6a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6225e6a8, SUC_lexicon_tags_43 },
    { 0x4305e6a8, SUC_lexicon_tags_33 },
    { 0x9405e6a9, SUC_lexicon_tags_15 },
    { 0x1905e6ab, SUC_lexicon_tags_56 },
    { 0xa325e6a9, SUC_lexicon_tags_7 },
    { 0x8f25e6ac, SUC_lexicon_tags_25 },
    { 0x9345e6a8, SUC_lexicon_tags_61 },
    { 0x4465e6af, SUC_lexicon_tags_82 },
    { 0x3f25e6af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b25e6b2, SUC_lexicon_tags_25 },
    { 0x2105e6b2, SUC_lexicon_tags_42 },
    { 0x30a5e6b4, SUC_lexicon_tags_17 },
    { 0xefe5e6b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc205e6b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba5e6ba, SUC_lexicon_tags_2 },
    { 0xdfc5e6bb, SUC_lexicon_tags_1 },
    { 0x8b65e6bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23c5e6be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed85e6c1, SUC_lexicon_tags_25 },
    { 0xc5e6c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6125e6c5, SUC_lexicon_tags_0 },
    { 0x39c5e6c6, SUC_lexicon_tags_37 },
    { 0x2cc5e6c7, SUC_lexicon_tags_37 },
    { 0x7665e6c7, SUC_lexicon_tags_108 },
    { 0xc345e6c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d25e6cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe25e6cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a5e6d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca5e6da, SUC_lexicon_tags_7 },
    { 0xc545e6db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4705e6dd, SUC_lexicon_tags_13 },
    { 0xc185e6de, SUC_lexicon_tags_14 },
    { 0x3405e6df, SUC_lexicon_tags_2 },
    { 0xba85e6e0, SUC_lexicon_tags_6 },
    { 0xc425e6e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f25e6eb, SUC_lexicon_tags_0 },
    { 0xc3e5e6ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3385e6f1, SUC_lexicon_tags_7 },
    { 0xb3e5e6f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf45e6f5, SUC_lexicon_tags_19 },
    { 0x8ec5e6f6, SUC_lexicon_tags_30 },
    { 0xbd65e6f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe865e6fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea5e702, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c5e705, SUC_lexicon_tags_25 },
    { 0xe485e705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a5e709, SUC_lexicon_tags_37 },
    { 0xe145e70a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3145e70e, SUC_lexicon_tags_59 },
    { 0xfda5e70f, SUC_lexicon_tags_25 },
    { 0x1e85e710, SUC_lexicon_tags_13 },
    { 0x16c5e711, SUC_lexicon_tags_11 },
    { 0x3565e712, SUC_lexicon_tags_84 },
    { 0xdd45e710, SUC_lexicon_tags_25 },
    { 0x2745e714, SUC_lexicon_tags_66 },
    { 0xd8e5e715, SUC_lexicon_tags_68 },
    { 0xac5e715, SUC_lexicon_tags_12 },
    { 0xb965e717, SUC_lexicon_tags_23 },
    { 0x6665e716, SUC_lexicon_tags_87 },
    { 0x81a5e70f, SUC_lexicon_tags_12 },
    { 0xc9a5e711, SUC_lexicon_tags_13 },
    { 0x38c5e71b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d45e71d, SUC_lexicon_tags_0 },
    { 0x2985e71e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdea5e720, SUC_lexicon_tags_25 },
    { 0x8005e721, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2005e728, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfca5e72a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e65e72f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2005e734, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a5e737, SUC_lexicon_tags_17 },
    { 0x2325e737, SUC_lexicon_tags_15 },
    { 0x31c5e738, SUC_lexicon_tags_37 },
    { 0x4905e738, SUC_lexicon_tags_42 },
    { 0x1c65e739, SUC_lexicon_tags_7 },
    { 0xfee5e73a, SUC_lexicon_tags_7 },
    { 0xb2e5e73d, SUC_lexicon_tags_11 },
    { 0xf845e73e, SUC_lexicon_tags_25 },
    { 0x9e45e73e, SUC_lexicon_tags_1 },
    { 0x9fc5e73c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c45e742, SUC_lexicon_tags_2 },
    { 0xe3c5e743, SUC_lexicon_tags_8 },
    { 0xcea5e744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2265e747, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf325e74d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b65e74f, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e45e752, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e85e757, SUC_lexicon_tags_11 },
    { 0x8c65e758, SUC_lexicon_tags_1 },
    { 0x7c5e759, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6a5e75b, SUC_lexicon_tags_37 },
    { 0x33a5e75c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x11c5e75e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c45e769, SUC_lexicon_tags_2 },
    { 0x6625e76a, SUC_lexicon_tags_25 },
    { 0x3bc5e76a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba5e76e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c5e770, SUC_lexicon_tags_70 },
    { 0xe2c5e770, SUC_lexicon_tags_60 },
    { 0xf9e5e770, SUC_lexicon_tags_17 },
    { 0xe5e5e770, SUC_lexicon_tags_48 },
    { 0x54a5e770, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e85e777, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5885e77b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac5e77f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e85e782, SUC_lexicon_tags_25 },
    { 0x8325e783, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9945e78c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x26a5e78e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69a5e790, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40e5e792, SUC_lexicon_tags_23 },
    { 0x8f85e792, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc45e797, SUC_lexicon_tags_37 },
    { 0xb885e798, SUC_lexicon_tags_55 },
    { 0x82e5e799, SUC_lexicon_tags_17 },
    { 0x1445e79a, SUC_lexicon_tags_33 },
    { 0x6745e79b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc25e7a6, SUC_lexicon_tags_49 },
    { 0xe245e7a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd345e7ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6885e7ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9865e7b1, SUC_lexicon_tags_416 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3205e7ba, SUC_lexicon_tags_1 },
    { 0x3365e7ba, SUC_lexicon_tags_65 },
    { 0xcfe5e7bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe885e7bf, SUC_lexicon_tags_11 },
    { 0xf665e7bf, SUC_lexicon_tags_45 },
    { 0x1425e7c0, SUC_lexicon_tags_66 },
    { 0xa4e5e7c2, SUC_lexicon_tags_65 },
    { 0x7065e7c3, SUC_lexicon_tags_23 },
    { 0x7a05e7c2, SUC_lexicon_tags_10 },
    { 0x8a85e7c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c5e7ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf65e7cc, SUC_lexicon_tags_27 },
    { 0x8725e7cd, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x1a85e7cf, SUC_lexicon_tags_47 },
    { 0xbe05e7d0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1025e7d3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x37c5e7d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c5e7dd, SUC_lexicon_tags_25 },
    { 0xebe5e7dd, SUC_lexicon_tags_17 },
    { 0x5d45e7df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac5e7e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6365e7e7, SUC_lexicon_tags_16 },
    { 0xc805e7e8, SUC_lexicon_tags_27 },
    { 0x44e5e7e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe265e7ed, SUC_lexicon_tags_1 },
    { 0x9525e7ee, SUC_lexicon_tags_2 },
    { 0xc325e7ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7545e7f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d45e7f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e5e7f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf905e7fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc585e7fe, SUC_lexicon_tags_27 },
    { 0x7e45e7ff, SUC_lexicon_tags_32 },
    { 0x425e800, SUC_lexicon_tags_1 },
    { 0xaae5e800, SUC_lexicon_tags_6 },
    { 0x1be5e802, SUC_lexicon_tags_77 },
    { 0xf0a5e803, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1da5e805, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e5e80b, SUC_lexicon_tags_23 },
    { 0xbb25e80b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c5e812, SUC_lexicon_tags_17 },
    { 0xdd65e813, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc945e815, SUC_lexicon_tags_47 },
    { 0x7865e816, SUC_lexicon_tags_18 },
    { 0x3ba5e817, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe5e81c, SUC_lexicon_tags_15 },
    { 0x2a65e81d, SUC_lexicon_tags_37 },
    { 0xd125e81d, SUC_lexicon_tags_37 },
    { 0xdf45e81f, SUC_lexicon_tags_61 },
    { 0xbb65e820, SUC_lexicon_tags_46 },
    { 0x9585e821, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b05e82d, SUC_lexicon_tags_15 },
    { 0x2125e82e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7705e832, SUC_lexicon_tags_30 },
    { 0x7265e833, SUC_lexicon_tags_44 },
    { 0x4a05e833, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece5e83b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3625e83d, SUC_lexicon_tags_29 },
    { 0x4405e83e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e5e846, SUC_lexicon_tags_8 },
    { 0xed05e847, SUC_lexicon_tags_87 },
    { 0x4c45e848, SUC_lexicon_tags_46 },
    { 0x1445e848, SUC_lexicon_tags_32 },
    { 0xb2a5e846, SUC_lexicon_tags_26 },
    { 0xb4e5e84b, SUC_lexicon_tags_69 },
    { 0x9f25e84a, SUC_lexicon_tags_37 },
    { 0xefc5e84d, SUC_lexicon_tags_54 },
    { 0x4f65e84c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc665e854, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab45e859, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4885e85b, SUC_lexicon_tags_17 },
    { 0x7b45e85b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c5e85f, SUC_lexicon_tags_19 },
    { 0xd265e860, SUC_lexicon_tags_47 },
    { 0x3085e861, SUC_lexicon_tags_0 },
    { 0xa345e862, SUC_lexicon_tags_17 },
    { 0x7b85e863, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea5e866, SUC_lexicon_tags_47 },
    { 0xe885e867, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a5e86c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde25e86e, SUC_lexicon_tags_25 },
    { 0xd605e86e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d65e873, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de5e876, SUC_lexicon_tags_19 },
    { 0x6365e876, SUC_lexicon_tags_64 },
    { 0x2925e876, SUC_lexicon_tags_37 },
    { 0xa7a5e879, SUC_lexicon_tags_15 },
    { 0x33a5e87a, SUC_lexicon_tags_9 },
    { 0x1fe5e879, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1145e87d, SUC_lexicon_tags_38 },
    { 0xbcc5e87e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9745e880, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e45e883, SUC_lexicon_tags_39 },
    { 0xf505e883, SUC_lexicon_tags_62 },
    { 0xc025e884, SUC_lexicon_tags_31 },
    { 0x1a05e886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ce5e888, SUC_lexicon_tags_17 },
    { 0xf125e888, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e5e88d, SUC_lexicon_tags_10 },
    { 0x565e88e, SUC_lexicon_tags_87 },
    { 0xfec5e88e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6185e892, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb525e894, SUC_lexicon_tags_37 },
    { 0xa4a5e894, SUC_lexicon_tags_82 },
    { 0xc8c5e896, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe25e899, SUC_lexicon_tags_60 },
    { 0x9045e899, SUC_lexicon_tags_68 },
    { 0x7705e89b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a5e8a0, SUC_lexicon_tags_25 },
    { 0xb605e8a0, SUC_lexicon_tags_9 },
    { 0x6a65e8a2, SUC_lexicon_tags_77 },
    { 0xaa5e8a3, SUC_lexicon_tags_15 },
    { 0x96c5e8a4, SUC_lexicon_tags_47 },
    { 0xbfc5e8a3, SUC_lexicon_tags_12 },
    { 0x7fc5e8a4, SUC_lexicon_tags_49 },
    { 0x9005e8a7, SUC_lexicon_tags_84 },
    { 0xfc5e8a4, SUC_lexicon_tags_7 },
    { 0xe5c5e8a0, SUC_lexicon_tags_7 },
    { 0x1905e8aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c65e8ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x99e5e8b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed45e8b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6385e8bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8805e8be, SUC_lexicon_tags_0 },
    { 0x9405e8be, SUC_lexicon_tags_12 },
    { 0xf305e8bf, SUC_lexicon_tags_1 },
    { 0x73a5e8c1, SUC_lexicon_tags_15 },
    { 0xf25e8bf, SUC_lexicon_tags_542 },
    { 0x1d85e8bf, SUC_lexicon_tags_11 },
    { 0xc485e8c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5705e8c7, SUC_lexicon_tags_31 },
    { 0x4365e8c8, SUC_lexicon_tags_45 },
    { 0xf5a5e8c9, SUC_lexicon_tags_39 },
    { 0x3e45e8c8, SUC_lexicon_tags_68 },
    { 0x8565e8cb, SUC_lexicon_tags_17 },
    { 0x93c5e8c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd585e8d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa65e8d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c5e8e0, SUC_lexicon_tags_15 },
    { 0x8405e8e0, SUC_lexicon_tags_77 },
    { 0x28a5e8e2, SUC_lexicon_tags_15 },
    { 0x425e8e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e5e8e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a45e8e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe5c5e8ea, SUC_lexicon_tags_11 },
    { 0x9ac5e8ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe05e8ef, SUC_lexicon_tags_60 },
    { 0xc2c5e8f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9545e8f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c5e8f9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa5e8fc, SUC_lexicon_tags_17 },
    { 0x3bc5e8fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2025e8ff, SUC_lexicon_tags_155 },
    { 0, NULL },
    { 0x8f65e901, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa105e909, SUC_lexicon_tags_17 },
    { 0xcf65e90a, SUC_lexicon_tags_11 },
    { 0xf8e5e90b, SUC_lexicon_tags_11 },
    { 0xd1a5e90c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef85e910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c5e914, SUC_lexicon_tags_37 },
    { 0x9ca5e914, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6985e917, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5e91a, SUC_lexicon_tags_0 },
    { 0x2965e91b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5e5e91d, SUC_lexicon_tags_1 },
    { 0xb1a5e91d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3a85e920, SUC_lexicon_tags_7 },
    { 0x3bc5e921, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdb05e923, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4285e925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6425e92b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf925e92e, SUC_lexicon_tags_39 },
    { 0x9ce5e92e, SUC_lexicon_tags_7 },
    { 0xbe65e930, SUC_lexicon_tags_60 },
    { 0xe65e92f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4085e933, SUC_lexicon_tags_12 },
    { 0xb25e934, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc705e93b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee5e93f, SUC_lexicon_tags_47 },
    { 0x83c5e93f, SUC_lexicon_tags_48 },
    { 0xa625e93f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdfe5e943, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b25e94c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2265e94f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9065e951, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9485e956, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e5e959, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e5e95e, SUC_lexicon_tags_43 },
    { 0x205e95f, SUC_lexicon_tags_11 },
    { 0x9e25e960, SUC_lexicon_tags_25 },
    { 0x24c5e961, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec5e968, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e5e96d, SUC_lexicon_tags_2 },
    { 0xb705e96e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92c5e970, SUC_lexicon_tags_8 },
    { 0xb685e970, SUC_lexicon_tags_65 },
    { 0xca85e972, SUC_lexicon_tags_25 },
    { 0x2ce5e973, SUC_lexicon_tags_52 },
    { 0xbbe5e970, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d45e97b, SUC_lexicon_tags_5 },
    { 0x3ca5e97c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2685e97f, SUC_lexicon_tags_0 },
    { 0x99a5e97f, SUC_lexicon_tags_25 },
    { 0xe645e97f, SUC_lexicon_tags_12 },
    { 0x6405e982, SUC_lexicon_tags_5 },
    { 0x3005e980, SUC_lexicon_tags_2 },
    { 0xa165e980, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d05e988, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c5e98c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf45e98e, SUC_lexicon_tags_33 },
    { 0x95a5e98f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f45e993, SUC_lexicon_tags_17 },
    { 0x9265e994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf05e996, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8965e999, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da5e9a0, SUC_lexicon_tags_7 },
    { 0xdce5e9a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d05e9a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a5e9ac, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x3225e9af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6845e9b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2a5e9b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45e9b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8845e9ba, SUC_lexicon_tags_14 },
    { 0x1a05e9bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4b25e9bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5cc5e9bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9985e9c1, SUC_lexicon_tags_15 },
    { 0xaf85e9c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a5e9c5, SUC_lexicon_tags_61 },
    { 0x1545e9c6, SUC_lexicon_tags_11 },
    { 0x55a5e9c7, SUC_lexicon_tags_1 },
    { 0xcc5e9c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e5e9cc, SUC_lexicon_tags_37 },
    { 0x92e5e9cd, SUC_lexicon_tags_39 },
    { 0x5165e9cd, SUC_lexicon_tags_14 },
    { 0xe7c5e9cf, SUC_lexicon_tags_17 },
    { 0x7465e9cf, SUC_lexicon_tags_11 },
    { 0x1f65e9d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e5e9d5, SUC_lexicon_tags_12 },
    { 0xa265e9d5, SUC_lexicon_tags_11 },
    { 0xabc5e9d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc25e9da, SUC_lexicon_tags_34 },
    { 0xd285e9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf45e9e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a45e9f2, SUC_lexicon_tags_26 },
    { 0x8a45e9f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc605e9f9, SUC_lexicon_tags_32 },
    { 0x45e9f9, SUC_lexicon_tags_19 },
    { 0x1ac5e9f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c5ea08, SUC_lexicon_tags_1 },
    { 0x4d85ea09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3605ea0c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9785ea0f, SUC_lexicon_tags_19 },
    { 0x46c5ea10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcd65ea12, SUC_lexicon_tags_15 },
    { 0xd1c5ea13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e5ea16, SUC_lexicon_tags_39 },
    { 0x2ac5ea17, SUC_lexicon_tags_11 },
    { 0x4c65ea18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6dc5ea1a, SUC_lexicon_tags_3 },
    { 0xa685ea1a, SUC_lexicon_tags_14 },
    { 0xa905ea1c, SUC_lexicon_tags_0 },
    { 0x94c5ea1c, SUC_lexicon_tags_7 },
    { 0x6605ea1e, SUC_lexicon_tags_17 },
    { 0x14c5ea1d, SUC_lexicon_tags_30 },
    { 0x5a65ea1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1945ea22, SUC_lexicon_tags_25 },
    { 0x245ea23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb025ea25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f45ea2b, SUC_lexicon_tags_25 },
    { 0x1a65ea2c, SUC_lexicon_tags_17 },
    { 0x4a5ea2d, SUC_lexicon_tags_38 },
    { 0x7245ea2e, SUC_lexicon_tags_12 },
    { 0x70c5ea2e, SUC_lexicon_tags_25 },
    { 0x8565ea2d, SUC_lexicon_tags_23 },
    { 0x6f05ea31, SUC_lexicon_tags_37 },
    { 0xa685ea32, SUC_lexicon_tags_1 },
    { 0x5c5ea33, SUC_lexicon_tags_1 },
    { 0xcf25ea33, SUC_lexicon_tags_0 },
    { 0xdc45ea35, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8045ea38, SUC_lexicon_tags_25 },
    { 0xb625ea39, SUC_lexicon_tags_60 },
    { 0x3b25ea3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba5ea3d, SUC_lexicon_tags_1 },
    { 0x3665ea3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e85ea46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b25ea48, SUC_lexicon_tags_1 },
    { 0x6225ea49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e5ea4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5785ea51, SUC_lexicon_tags_25 },
    { 0xa145ea52, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9945ea55, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc805ea58, SUC_lexicon_tags_27 },
    { 0xcc05ea58, SUC_lexicon_tags_17 },
    { 0xc685ea5a, SUC_lexicon_tags_87 },
    { 0x7b65ea5b, SUC_lexicon_tags_0 },
    { 0xb2e5ea5b, SUC_lexicon_tags_31 },
    { 0x4485ea5a, SUC_lexicon_tags_29 },
    { 0x8725ea5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef25ea65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac05ea6a, SUC_lexicon_tags_37 },
    { 0x2f65ea6a, SUC_lexicon_tags_65 },
    { 0xdc45ea6b, SUC_lexicon_tags_7 },
    { 0x6c05ea6d, SUC_lexicon_tags_28 },
    { 0x5925ea6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e65ea70, SUC_lexicon_tags_2 },
    { 0x5e45ea71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf385ea73, SUC_lexicon_tags_17 },
    { 0x5f85ea74, SUC_lexicon_tags_24 },
    { 0x2ca5ea74, SUC_lexicon_tags_12 },
    { 0xc945ea73, SUC_lexicon_tags_38 },
    { 0x1025ea75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5185ea7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd765ea7c, SUC_lexicon_tags_37 },
    { 0xbce5ea7d, SUC_lexicon_tags_29 },
    { 0x64a5ea7e, SUC_lexicon_tags_19 },
    { 0x94e5ea7f, SUC_lexicon_tags_63 },
    { 0x51e5ea7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a5ea85, SUC_lexicon_tags_16 },
    { 0xa8a5ea86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4525ea89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x24a5ea8c, SUC_lexicon_tags_25 },
    { 0x8c45ea8c, SUC_lexicon_tags_15 },
    { 0x96c5ea8e, SUC_lexicon_tags_4 },
    { 0x5d65ea8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa445ea93, SUC_lexicon_tags_11 },
    { 0xd125ea93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f25ea96, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c65ea99, SUC_lexicon_tags_43 },
    { 0x11e5ea99, SUC_lexicon_tags_12 },
    { 0x825ea99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc65ea9f, SUC_lexicon_tags_25 },
    { 0x2d05eaa0, SUC_lexicon_tags_60 },
    { 0x4e65eaa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5785eaa4, SUC_lexicon_tags_25 },
    { 0x405eaa4, SUC_lexicon_tags_42 },
    { 0x3085eaa5, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x7ac5eaa8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6105eaaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9665eaad, SUC_lexicon_tags_15 },
    { 0x9885eaad, SUC_lexicon_tags_5 },
    { 0x8345eaaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf125eab1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8345eab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59e5eab6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1725eab9, SUC_lexicon_tags_49 },
    { 0x4005eaba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa5eabd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce5eac2, SUC_lexicon_tags_174 },
    { 0xb1a5eac3, SUC_lexicon_tags_17 },
    { 0xe605eac4, SUC_lexicon_tags_57 },
    { 0x5ce5eac5, SUC_lexicon_tags_49 },
    { 0x385eac6, SUC_lexicon_tags_47 },
    { 0x7f25eac2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaec5eac9, SUC_lexicon_tags_56 },
    { 0xed85eaca, SUC_lexicon_tags_45 },
    { 0x8c25eac9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3125eacf, SUC_lexicon_tags_2 },
    { 0xca65ead0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7ce5ead2, SUC_lexicon_tags_27 },
    { 0x24e5ead3, SUC_lexicon_tags_66 },
    { 0x9585ead3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe465ead6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8585ead9, SUC_lexicon_tags_25 },
    { 0x3ec5eada, SUC_lexicon_tags_4 },
    { 0xe445ead9, SUC_lexicon_tags_15 },
    { 0x3685eada, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3605eade, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc5eae0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e5eae6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e5eae9, SUC_lexicon_tags_65 },
    { 0x205eae9, SUC_lexicon_tags_11 },
    { 0xd25eaeb, SUC_lexicon_tags_57 },
    { 0xa2a5eae9, SUC_lexicon_tags_37 },
    { 0xb145eaed, SUC_lexicon_tags_7 },
    { 0x5e65eaee, SUC_lexicon_tags_2 },
    { 0xc745eaef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41c5eaf1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba65eaf6, SUC_lexicon_tags_103 },
    { 0x37a5eaf6, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6445eafc, SUC_lexicon_tags_25 },
    { 0xcfa5eafd, SUC_lexicon_tags_61 },
    { 0x3f85eafe, SUC_lexicon_tags_11 },
    { 0xbe5eafd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f25eb03, SUC_lexicon_tags_43 },
    { 0xa8a5eb04, SUC_lexicon_tags_45 },
    { 0xd2e5eb04, SUC_lexicon_tags_1 },
    { 0x31a5eb03, SUC_lexicon_tags_85 },
    { 0x2ce5eb07, SUC_lexicon_tags_15 },
    { 0x7885eb08, SUC_lexicon_tags_15 },
    { 0xcc05eb09, SUC_lexicon_tags_1 },
    { 0xc4c5eb0a, SUC_lexicon_tags_46 },
    { 0xe0e5eb0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c45eb0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd445eb10, SUC_lexicon_tags_13 },
    { 0x6c65eb11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f25eb19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c45eb1b, SUC_lexicon_tags_87 },
    { 0xdcc5eb1b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf4c5eb1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xee05eb21, SUC_lexicon_tags_21 },
    { 0xe0e5eb22, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c65eb25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45e5eb27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e5eb2c, SUC_lexicon_tags_25 },
    { 0xd865eb2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a5eb31, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7d65eb33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a5eb37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe125eb39, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c85eb3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcfc5eb3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8a5eb40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9125eb43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf805eb45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4285eb48, SUC_lexicon_tags_7 },
    { 0x2985eb49, SUC_lexicon_tags_644 },
    { 0, NULL },
    { 0, NULL },
    { 0xf665eb4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9025eb4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae5eb53, SUC_lexicon_tags_47 },
    { 0x7fc5eb54, SUC_lexicon_tags_43 },
    { 0xcb25eb54, SUC_lexicon_tags_63 },
    { 0x1085eb53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa5eb5a, SUC_lexicon_tags_17 },
    { 0x5e65eb5b, SUC_lexicon_tags_19 },
    { 0x1205eb5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfae5eb5e, SUC_lexicon_tags_15 },
    { 0x3f05eb5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace5eb64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3705eb67, SUC_lexicon_tags_17 },
    { 0x1be5eb68, SUC_lexicon_tags_9 },
    { 0x3865eb69, SUC_lexicon_tags_7 },
    { 0xa545eb6a, SUC_lexicon_tags_15 },
    { 0x1ba5eb6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6665eb6d, SUC_lexicon_tags_37 },
    { 0x3d65eb6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e45eb73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaa45eb75, SUC_lexicon_tags_49 },
    { 0xa7a5eb75, SUC_lexicon_tags_282 },
    { 0x25eb75, SUC_lexicon_tags_11 },
    { 0x6245eb77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4585eb7c, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb105eb81, SUC_lexicon_tags_8 },
    { 0x21a5eb82, SUC_lexicon_tags_82 },
    { 0xba45eb82, SUC_lexicon_tags_11 },
    { 0x28c5eb84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe25eb88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2945eb8c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9a5eb8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b25eb91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc185eb93, SUC_lexicon_tags_15 },
    { 0xbe85eb94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6585eb97, SUC_lexicon_tags_32 },
    { 0x1b65eb98, SUC_lexicon_tags_40 },
    { 0x21c5eb99, SUC_lexicon_tags_26 },
    { 0x5465eb99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba5eb9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3365eb9f, SUC_lexicon_tags_29 },
    { 0xc585eb9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f65eba5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc3c5eba7, SUC_lexicon_tags_15 },
    { 0x4165eba8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e5ebab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5c5ebad, SUC_lexicon_tags_1 },
    { 0x7da5ebae, SUC_lexicon_tags_11 },
    { 0x2465ebaf, SUC_lexicon_tags_9 },
    { 0x8345ebae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb605ebb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb85ebb9, SUC_lexicon_tags_11 },
    { 0x9445ebba, SUC_lexicon_tags_25 },
    { 0xa065ebbb, SUC_lexicon_tags_17 },
    { 0xd325ebbc, SUC_lexicon_tags_43 },
    { 0x16c5ebba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd205ebc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c5ebc3, SUC_lexicon_tags_27 },
    { 0x9785ebc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed05ebd5, SUC_lexicon_tags_12 },
    { 0x5645ebd5, SUC_lexicon_tags_12 },
    { 0xb085ebd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdae5ebd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b25ebdf, SUC_lexicon_tags_103 },
    { 0xa1c5ebe0, SUC_lexicon_tags_6 },
    { 0xd605ebe1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e5ebe6, SUC_lexicon_tags_45 },
    { 0x5505ebe6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x485ebe9, SUC_lexicon_tags_45 },
    { 0xe985ebea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12c5ebec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2285ebee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8305ebf1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b05ebfc, SUC_lexicon_tags_17 },
    { 0xbc05ebfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68e5ebff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7545ec01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a5ec05, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x93e5ec07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1945ec0a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x27e5ec0c, SUC_lexicon_tags_17 },
    { 0x205ec0d, SUC_lexicon_tags_37 },
    { 0xaba5ec0e, SUC_lexicon_tags_33 },
    { 0xd365ec0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94e5ec11, SUC_lexicon_tags_19 },
    { 0x41e5ec12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f45ec17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce85ec27, SUC_lexicon_tags_60 },
    { 0xa305ec27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1025ec2b, SUC_lexicon_tags_7 },
    { 0x7e5ec2c, SUC_lexicon_tags_7 },
    { 0xdb05ec2b, SUC_lexicon_tags_4 },
    { 0x7405ec2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67a5ec30, SUC_lexicon_tags_6 },
    { 0x9a05ec30, SUC_lexicon_tags_10 },
    { 0xfd65ec32, SUC_lexicon_tags_33 },
    { 0xe685ec33, SUC_lexicon_tags_37 },
    { 0x1b85ec30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba45ec39, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xfbe5ec3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe605ec46, SUC_lexicon_tags_25 },
    { 0xe365ec47, SUC_lexicon_tags_49 },
    { 0x2c05ec47, SUC_lexicon_tags_7 },
    { 0xc9e5ec49, SUC_lexicon_tags_45 },
    { 0xed85ec47, SUC_lexicon_tags_15 },
    { 0xa9c5ec4a, SUC_lexicon_tags_13 },
    { 0xf645ec46, SUC_lexicon_tags_9 },
    { 0x81e5ec4d, SUC_lexicon_tags_45 },
    { 0x32a5ec4e, SUC_lexicon_tags_1 },
    { 0x65c5ec4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c25ec54, SUC_lexicon_tags_12 },
    { 0xbc25ec55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37a5ec57, SUC_lexicon_tags_2 },
    { 0x9865ec58, SUC_lexicon_tags_40 },
    { 0x3ce5ec59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7645ec5b, SUC_lexicon_tags_37 },
    { 0xdd65ec5b, SUC_lexicon_tags_65 },
    { 0x205ec5c, SUC_lexicon_tags_1 },
    { 0x44c5ec5c, SUC_lexicon_tags_29 },
    { 0x7045ec5f, SUC_lexicon_tags_23 },
    { 0x92a5ec5f, SUC_lexicon_tags_48 },
    { 0x8c85ec61, SUC_lexicon_tags_28 },
    { 0x2a45ec5e, SUC_lexicon_tags_19 },
    { 0x45c5ec5e, SUC_lexicon_tags_15 },
    { 0xc0a5ec64, SUC_lexicon_tags_10 },
    { 0xe5e5ec5b, SUC_lexicon_tags_79 },
    { 0xeb25ec5f, SUC_lexicon_tags_1 },
    { 0xb045ec5f, SUC_lexicon_tags_25 },
    { 0x86a5ec68, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x6605ec6a, SUC_lexicon_tags_12 },
    { 0xd865ec6a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4845ec6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd045ec71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a5ec74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65c5ec76, SUC_lexicon_tags_92 },
    { 0x6485ec76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ec7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a5ec7d, SUC_lexicon_tags_15 },
    { 0x3045ec7d, SUC_lexicon_tags_45 },
    { 0xf65ec7e, SUC_lexicon_tags_45 },
    { 0x3625ec7f, SUC_lexicon_tags_37 },
    { 0xfde5ec81, SUC_lexicon_tags_37 },
    { 0xdb05ec82, SUC_lexicon_tags_87 },
    { 0x4e5ec81, SUC_lexicon_tags_38 },
    { 0x9845ec83, SUC_lexicon_tags_11 },
    { 0x6b05ec81, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd505ec87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe5ec8a, SUC_lexicon_tags_28 },
    { 0x8a85ec8b, SUC_lexicon_tags_37 },
    { 0x91a5ec8a, SUC_lexicon_tags_27 },
    { 0xa1c5ec8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f05ec91, SUC_lexicon_tags_27 },
    { 0xbce5ec92, SUC_lexicon_tags_2 },
    { 0xf4c5ec91, SUC_lexicon_tags_27 },
    { 0xa5a5ec91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90c5ec96, SUC_lexicon_tags_1 },
    { 0xeec5ec97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3525eca0, SUC_lexicon_tags_79 },
    { 0xcdc5eca1, SUC_lexicon_tags_873 },
    { 0xd865eca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1da5eca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ba5eca6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f45ecad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb25ecb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbe5ecb3, SUC_lexicon_tags_37 },
    { 0x11e5ecb4, SUC_lexicon_tags_10 },
    { 0xe785ecb4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a5ecb8, SUC_lexicon_tags_7 },
    { 0x9a85ecb9, SUC_lexicon_tags_47 },
    { 0xbf45ecb8, SUC_lexicon_tags_46 },
    { 0xad05ecbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9445ecbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf245ecc2, SUC_lexicon_tags_38 },
    { 0xc85ecc2, SUC_lexicon_tags_11 },
    { 0xd665ecc4, SUC_lexicon_tags_15 },
    { 0x985ecc2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8605ecca, SUC_lexicon_tags_37 },
    { 0xe6c5ecca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5225eccf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a5ecd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b05ecd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad45ecd8, SUC_lexicon_tags_27 },
    { 0x3c5ecd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c25ece1, SUC_lexicon_tags_23 },
    { 0x57e5ece1, SUC_lexicon_tags_28 },
    { 0x3d05ece1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb185ece6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4725ece9, SUC_lexicon_tags_29 },
    { 0xc605ecea, SUC_lexicon_tags_15 },
    { 0xeba5eceb, SUC_lexicon_tags_23 },
    { 0x7d65ece9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fe5ecee, SUC_lexicon_tags_7 },
    { 0xe205ecee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa05ecf3, SUC_lexicon_tags_2 },
    { 0xbce5ecf4, SUC_lexicon_tags_640 },
    { 0xc685ecf4, SUC_lexicon_tags_37 },
    { 0x56c5ecf5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xce25ecf9, SUC_lexicon_tags_30 },
    { 0xefc5ecfa, SUC_lexicon_tags_25 },
    { 0xa65ecfb, SUC_lexicon_tags_2 },
    { 0xf625ecfb, SUC_lexicon_tags_38 },
    { 0xf265ecfd, SUC_lexicon_tags_25 },
    { 0x7ae5ecfe, SUC_lexicon_tags_7 },
    { 0xa985ecfe, SUC_lexicon_tags_217 },
    { 0x27c5ecfe, SUC_lexicon_tags_12 },
    { 0x1da5ed01, SUC_lexicon_tags_80 },
    { 0xe905ed01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c5ed06, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd845ed09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed65ed0c, SUC_lexicon_tags_1 },
    { 0x4d85ed0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa225ed11, SUC_lexicon_tags_31 },
    { 0xf2a5ed12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4705ed15, SUC_lexicon_tags_60 },
    { 0x75e5ed16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7885ed18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdee5ed1a, SUC_lexicon_tags_17 },
    { 0xda65ed1b, SUC_lexicon_tags_46 },
    { 0x9fa5ed1a, SUC_lexicon_tags_2 },
    { 0xd1c5ed1d, SUC_lexicon_tags_11 },
    { 0x4da5ed1b, SUC_lexicon_tags_15 },
    { 0x6465ed1f, SUC_lexicon_tags_27 },
    { 0x7d65ed1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c5ed27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa445ed2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x48c5ed2c, SUC_lexicon_tags_11 },
    { 0x9c45ed2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa185ed32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd25ed34, SUC_lexicon_tags_25 },
    { 0x53c5ed34, SUC_lexicon_tags_0 },
    { 0x9ac5ed36, SUC_lexicon_tags_17 },
    { 0x31e5ed37, SUC_lexicon_tags_47 },
    { 0xa3a5ed35, SUC_lexicon_tags_45 },
    { 0x1f25ed34, SUC_lexicon_tags_37 },
    { 0x9fc5ed3a, SUC_lexicon_tags_1 },
    { 0xa965ed3b, SUC_lexicon_tags_2 },
    { 0x1fc5ed39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ec5ed3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2c45ed40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf925ed42, SUC_lexicon_tags_96 },
    { 0x365ed43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf65ed46, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c45ed4a, SUC_lexicon_tags_2 },
    { 0xdcc5ed4a, SUC_lexicon_tags_15 },
    { 0xeec5ed4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e5ed4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc45ed54, SUC_lexicon_tags_47 },
    { 0xdfa5ed55, SUC_lexicon_tags_87 },
    { 0x4e25ed56, SUC_lexicon_tags_15 },
    { 0x4385ed57, SUC_lexicon_tags_37 },
    { 0x2865ed58, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e5ed5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb85ed5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb425ed60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5705ed63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c5ed6b, SUC_lexicon_tags_26 },
    { 0xbb25ed6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2345ed6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa05ed7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd805ed7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3465ed7f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd645ed85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc5ed88, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3285ed8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc05ed91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd825ed93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdac5ed95, SUC_lexicon_tags_43 },
    { 0xf05ed95, SUC_lexicon_tags_21 },
    { 0x9125ed97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5705ed9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c65ed9d, SUC_lexicon_tags_25 },
    { 0xb825ed9e, SUC_lexicon_tags_0 },
    { 0x38a5ed9e, SUC_lexicon_tags_4 },
    { 0x9425eda0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x92e5eda2, SUC_lexicon_tags_25 },
    { 0x37e5eda2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba25eda7, SUC_lexicon_tags_0 },
    { 0x8d45eda7, SUC_lexicon_tags_10 },
    { 0xd745eda7, SUC_lexicon_tags_349 },
    { 0x3965eda9, SUC_lexicon_tags_68 },
    { 0xc025edab, SUC_lexicon_tags_31 },
    { 0x4925edab, SUC_lexicon_tags_201 },
    { 0xfe25edad, SUC_lexicon_tags_43 },
    { 0x58c5eda8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec45edb2, SUC_lexicon_tags_8 },
    { 0x4c45edb2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xe25edb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd65edb7, SUC_lexicon_tags_60 },
    { 0xf605edb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc05edbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb85edc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5e5edc2, SUC_lexicon_tags_1 },
    { 0x52a5edc3, SUC_lexicon_tags_25 },
    { 0x3fc5edc4, SUC_lexicon_tags_1 },
    { 0xd7e5edc5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b05edcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e5edd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c5edd3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2ca5edd5, SUC_lexicon_tags_10 },
    { 0x425edd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a5edd9, SUC_lexicon_tags_69 },
    { 0xd7c5edda, SUC_lexicon_tags_14 },
    { 0xed65edda, SUC_lexicon_tags_2 },
    { 0x6a45eddc, SUC_lexicon_tags_61 },
    { 0xe085eddd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec5ede1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4185ede7, SUC_lexicon_tags_63 },
    { 0xdca5ede8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2345edeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1025edee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51e5edf0, SUC_lexicon_tags_30 },
    { 0xf305edf1, SUC_lexicon_tags_26 },
    { 0x3be5edf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c5edf8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa225edfa, SUC_lexicon_tags_9 },
    { 0x7fe5edfb, SUC_lexicon_tags_17 },
    { 0x1305edfc, SUC_lexicon_tags_27 },
    { 0x1bc5edfd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe5ee02, SUC_lexicon_tags_72 },
    { 0x4845ee02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3665ee08, SUC_lexicon_tags_26 },
    { 0x3e85ee09, SUC_lexicon_tags_11 },
    { 0x98a5ee09, SUC_lexicon_tags_2 },
    { 0x7945ee0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa25ee0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7165ee11, SUC_lexicon_tags_29 },
    { 0xe545ee11, SUC_lexicon_tags_25 },
    { 0x1085ee11, SUC_lexicon_tags_15 },
    { 0xb545ee14, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c5ee17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8585ee1a, SUC_lexicon_tags_25 },
    { 0xc385ee1b, SUC_lexicon_tags_4 },
    { 0x4785ee1c, SUC_lexicon_tags_46 },
    { 0xfe45ee1b, SUC_lexicon_tags_37 },
    { 0x9c65ee1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c05ee23, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a65ee2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x225ee2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc805ee30, SUC_lexicon_tags_46 },
    { 0xa7a5ee31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a5ee35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1e85ee37, SUC_lexicon_tags_56 },
    { 0xd2e5ee38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65ee3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a45ee43, SUC_lexicon_tags_26 },
    { 0xb345ee43, SUC_lexicon_tags_9 },
    { 0xca45ee43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4725ee47, SUC_lexicon_tags_32 },
    { 0x30e5ee48, SUC_lexicon_tags_15 },
    { 0x7ea5ee49, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf765ee51, SUC_lexicon_tags_65 },
    { 0x8be5ee52, SUC_lexicon_tags_10 },
    { 0xfcc5ee53, SUC_lexicon_tags_12 },
    { 0xe8c5ee54, SUC_lexicon_tags_151 },
    { 0xc8a5ee53, SUC_lexicon_tags_15 },
    { 0x9725ee54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d05ee58, SUC_lexicon_tags_1 },
    { 0x5925ee59, SUC_lexicon_tags_37 },
    { 0xb885ee5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce5ee60, SUC_lexicon_tags_2 },
    { 0x6f05ee61, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6205ee64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc5ee6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a5ee76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c5ee7a, SUC_lexicon_tags_8 },
    { 0xccc5ee7b, SUC_lexicon_tags_18 },
    { 0x8385ee7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c5ee7f, SUC_lexicon_tags_28 },
    { 0x9e05ee80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a5ee83, SUC_lexicon_tags_1 },
    { 0x30a5ee84, SUC_lexicon_tags_60 },
    { 0x1765ee83, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef85ee89, SUC_lexicon_tags_28 },
    { 0xbb25ee8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c65ee8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe65ee92, SUC_lexicon_tags_48 },
    { 0xe1a5ee93, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1905ee95, SUC_lexicon_tags_87 },
    { 0x9565ee96, SUC_lexicon_tags_23 },
    { 0x59a5ee95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc225ee9e, SUC_lexicon_tags_0 },
    { 0x1ba5ee9f, SUC_lexicon_tags_0 },
    { 0x6ec5eea0, SUC_lexicon_tags_15 },
    { 0xf9e5eea1, SUC_lexicon_tags_12 },
    { 0x8045eea2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b25eea4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5945eea6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a5eea9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd425eeab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ba5eead, SUC_lexicon_tags_10 },
    { 0x7245eeae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf305eeb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e5eeb4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c5eebb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd525eec0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca5eec3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b45eec5, SUC_lexicon_tags_26 },
    { 0xc5c5eec5, SUC_lexicon_tags_15 },
    { 0xda45eec7, SUC_lexicon_tags_9 },
    { 0x7ce5eec8, SUC_lexicon_tags_38 },
    { 0x1645eec8, SUC_lexicon_tags_63 },
    { 0xf325eeca, SUC_lexicon_tags_60 },
    { 0xca05eecb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6b25eecd, SUC_lexicon_tags_3 },
    { 0x1445eece, SUC_lexicon_tags_15 },
    { 0x5645eecf, SUC_lexicon_tags_79 },
    { 0x70a5eecd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e5eed4, SUC_lexicon_tags_27 },
    { 0xd0a5eed4, SUC_lexicon_tags_15 },
    { 0x7525eed5, SUC_lexicon_tags_15 },
    { 0xd9a5eed5, SUC_lexicon_tags_2 },
    { 0xdc85eed8, SUC_lexicon_tags_41 },
    { 0xf365eed9, SUC_lexicon_tags_2 },
    { 0xfa5eed9, SUC_lexicon_tags_11 },
    { 0xdba5eedb, SUC_lexicon_tags_7 },
    { 0xdd25eedb, SUC_lexicon_tags_65 },
    { 0x29e5eed4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c5eee1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb2a5eee3, SUC_lexicon_tags_77 },
    { 0x39c5eee4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de5eee8, SUC_lexicon_tags_25 },
    { 0x8745eee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe065eeec, SUC_lexicon_tags_27 },
    { 0x94a5eeed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab45eeef, SUC_lexicon_tags_47 },
    { 0xaae5eeef, SUC_lexicon_tags_10 },
    { 0xe565eef1, SUC_lexicon_tags_104 },
    { 0xf5a5eef2, SUC_lexicon_tags_12 },
    { 0xabe5eeef, SUC_lexicon_tags_23 },
    { 0xa165eef4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc345eef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc5eefc, SUC_lexicon_tags_37 },
    { 0x8265eefc, SUC_lexicon_tags_7 },
    { 0xf6a5eefe, SUC_lexicon_tags_40 },
    { 0xca85eeff, SUC_lexicon_tags_12 },
    { 0xb165ef00, SUC_lexicon_tags_23 },
    { 0x6b05ef00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c5ef03, SUC_lexicon_tags_37 },
    { 0x7965ef03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8e5ef06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4c5ef08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6425ef0a, SUC_lexicon_tags_33 },
    { 0xe6a5ef0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3445ef10, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c5ef16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb805ef19, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9665ef1b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225ef20, SUC_lexicon_tags_25 },
    { 0xf185ef20, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e05ef27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b05ef29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c85ef2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e5ef2f, SUC_lexicon_tags_27 },
    { 0xdd05ef2f, SUC_lexicon_tags_10 },
    { 0xeae5ef31, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xad45ef34, SUC_lexicon_tags_35 },
    { 0x4405ef35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc485ef37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71c5ef39, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5da5ef3b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2c5ef3d, SUC_lexicon_tags_1 },
    { 0x6985ef3e, SUC_lexicon_tags_46 },
    { 0xb845ef3f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b85ef43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c45ef47, SUC_lexicon_tags_11 },
    { 0xf745ef48, SUC_lexicon_tags_44 },
    { 0xc8e5ef48, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e5ef4e, SUC_lexicon_tags_37 },
    { 0x28c5ef4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d85ef51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a65ef54, SUC_lexicon_tags_65 },
    { 0x1745ef55, SUC_lexicon_tags_60 },
    { 0xe725ef54, SUC_lexicon_tags_6 },
    { 0x7de5ef57, SUC_lexicon_tags_12 },
    { 0x53a5ef54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde45ef5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2665ef60, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8265ef63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc5ef6b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc105ef6e, SUC_lexicon_tags_26 },
    { 0x9ce5ef6e, SUC_lexicon_tags_11 },
    { 0x6085ef6e, SUC_lexicon_tags_30 },
    { 0x785ef71, SUC_lexicon_tags_25 },
    { 0xf545ef72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee5ef75, SUC_lexicon_tags_7 },
    { 0x76a5ef76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf145ef7d, SUC_lexicon_tags_45 },
    { 0x8405ef7d, SUC_lexicon_tags_151 },
    { 0x9125ef7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc165ef85, SUC_lexicon_tags_23 },
    { 0x90c5ef86, SUC_lexicon_tags_15 },
    { 0xf45ef86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81c5ef89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4985ef8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9705ef91, SUC_lexicon_tags_37 },
    { 0x6c5ef92, SUC_lexicon_tags_12 },
    { 0x2f25ef92, SUC_lexicon_tags_14 },
    { 0x6a45ef92, SUC_lexicon_tags_47 },
    { 0x2305ef95, SUC_lexicon_tags_37 },
    { 0x3c45ef93, SUC_lexicon_tags_38 },
    { 0x7c05ef93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405ef9f, SUC_lexicon_tags_2 },
    { 0x46c5efa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf705efa2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd265efa4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf065efa6, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf85efac, SUC_lexicon_tags_19 },
    { 0x8fe5efac, SUC_lexicon_tags_8 },
    { 0x7a05efac, SUC_lexicon_tags_2 },
    { 0x7385efaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa625efb2, SUC_lexicon_tags_1 },
    { 0xf8e5efb3, SUC_lexicon_tags_84 },
    { 0x5c65efb2, SUC_lexicon_tags_82 },
    { 0x4b45efb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb05efb8, SUC_lexicon_tags_17 },
    { 0xd845efb9, SUC_lexicon_tags_15 },
    { 0xc45efb8, SUC_lexicon_tags_37 },
    { 0x6565efb9, SUC_lexicon_tags_12 },
    { 0x3ec5efbc, SUC_lexicon_tags_15 },
    { 0x6ca5efbc, SUC_lexicon_tags_15 },
    { 0x1045efbc, SUC_lexicon_tags_663 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa5efc4, SUC_lexicon_tags_26 },
    { 0x2085efc4, SUC_lexicon_tags_47 },
    { 0x37a5efc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3405efcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7165efd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a5efd4, SUC_lexicon_tags_38 },
    { 0x9145efd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a5efd8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc885efe4, SUC_lexicon_tags_63 },
    { 0x3ec5efe4, SUC_lexicon_tags_124 },
    { 0xd05efe4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1265efeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd05efed, SUC_lexicon_tags_12 },
    { 0x26c5efee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30c5eff0, SUC_lexicon_tags_47 },
    { 0x7185eff1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde5eff4, SUC_lexicon_tags_25 },
    { 0x50a5eff4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b45eff8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4865effa, SUC_lexicon_tags_7 },
    { 0xe385effa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c5f000, SUC_lexicon_tags_12 },
    { 0x71e5f000, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf945f004, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be5f00b, SUC_lexicon_tags_97 },
    { 0x6bc5f00b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae5f00f, SUC_lexicon_tags_1065 },
    { 0x41e5f010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1405f014, SUC_lexicon_tags_17 },
    { 0xe405f015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe625f017, SUC_lexicon_tags_1 },
    { 0x2625f017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b05f01a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x30c5f01c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a5f01f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c5f023, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425f027, SUC_lexicon_tags_8 },
    { 0xd765f027, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf245f02a, SUC_lexicon_tags_12 },
    { 0xb5e5f02b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce5f034, SUC_lexicon_tags_7 },
    { 0xbc05f034, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c5f03e, SUC_lexicon_tags_19 },
    { 0x94e5f03f, SUC_lexicon_tags_11 },
    { 0x8fc5f040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3145f044, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a5f048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1325f04b, SUC_lexicon_tags_0 },
    { 0x5a45f04c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ea5f04e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a5f051, SUC_lexicon_tags_26 },
    { 0x7fa5f052, SUC_lexicon_tags_46 },
    { 0x7b45f053, SUC_lexicon_tags_17 },
    { 0x205f054, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1105f05d, SUC_lexicon_tags_2 },
    { 0xe805f05d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0c5f060, SUC_lexicon_tags_11 },
    { 0x8c65f060, SUC_lexicon_tags_9 },
    { 0x1405f062, SUC_lexicon_tags_16 },
    { 0xcda5f063, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b85f068, SUC_lexicon_tags_12 },
    { 0xc385f068, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3045f06d, SUC_lexicon_tags_15 },
    { 0x62e5f06e, SUC_lexicon_tags_19 },
    { 0xee05f06f, SUC_lexicon_tags_2 },
    { 0x1985f070, SUC_lexicon_tags_2 },
    { 0x2825f071, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb485f075, SUC_lexicon_tags_38 },
    { 0x43c5f076, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce5f079, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a65f07e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe545f081, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e5f084, SUC_lexicon_tags_1 },
    { 0x1045f084, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4be5f087, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a5f08e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x73a5f090, SUC_lexicon_tags_12 },
    { 0xf1e5f090, SUC_lexicon_tags_11 },
    { 0x4f05f092, SUC_lexicon_tags_27 },
    { 0x3c85f093, SUC_lexicon_tags_2 },
    { 0xf1a5f094, SUC_lexicon_tags_27 },
    { 0xd425f095, SUC_lexicon_tags_26 },
    { 0x18e5f094, SUC_lexicon_tags_19 },
    { 0xae5f097, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa005f09c, SUC_lexicon_tags_15 },
    { 0x1285f09d, SUC_lexicon_tags_27 },
    { 0x1865f09c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb45f0a5, SUC_lexicon_tags_17 },
    { 0x5425f0a6, SUC_lexicon_tags_34 },
    { 0xa9c5f0a7, SUC_lexicon_tags_65 },
    { 0xb2a5f0a8, SUC_lexicon_tags_17 },
    { 0xc6c5f0a7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225f0ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa705f0b1, SUC_lexicon_tags_49 },
    { 0x4ae5f0b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd45f0b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe5f0bd, SUC_lexicon_tags_37 },
    { 0x3705f0be, SUC_lexicon_tags_61 },
    { 0xb1a5f0bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c5f0c2, SUC_lexicon_tags_25 },
    { 0x79a5f0c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf905f0c5, SUC_lexicon_tags_62 },
    { 0xcc25f0c5, SUC_lexicon_tags_2 },
    { 0x7645f0c5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca5f0cc, SUC_lexicon_tags_17 },
    { 0xd125f0cd, SUC_lexicon_tags_7 },
    { 0x65a5f0ce, SUC_lexicon_tags_27 },
    { 0x9925f0cf, SUC_lexicon_tags_17 },
    { 0xcba5f0cd, SUC_lexicon_tags_7 },
    { 0x48e5f0cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f25f0d3, SUC_lexicon_tags_19 },
    { 0x7f65f0d4, SUC_lexicon_tags_2 },
    { 0x31c5f0d5, SUC_lexicon_tags_17 },
    { 0x7a45f0d6, SUC_lexicon_tags_27 },
    { 0xffe5f0d7, SUC_lexicon_tags_32 },
    { 0x6ec5f0d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x905f0da, SUC_lexicon_tags_37 },
    { 0x39a5f0db, SUC_lexicon_tags_15 },
    { 0x4085f0dc, SUC_lexicon_tags_0 },
    { 0xd085f0db, SUC_lexicon_tags_27 },
    { 0x54c5f0de, SUC_lexicon_tags_15 },
    { 0x4c5f0df, SUC_lexicon_tags_25 },
    { 0x7c65f0e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe285f0e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45f0e8, SUC_lexicon_tags_39 },
    { 0xb425f0e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e5f0ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe745f0f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc725f0f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b85f0f6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5f0f9, SUC_lexicon_tags_47 },
    { 0x5745f0fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc5f0fd, SUC_lexicon_tags_8 },
    { 0x6965f0fe, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xcd05f100, SUC_lexicon_tags_37 },
    { 0xe305f101, SUC_lexicon_tags_25 },
    { 0xf585f100, SUC_lexicon_tags_12 },
    { 0xfa05f102, SUC_lexicon_tags_25 },
    { 0x9005f104, SUC_lexicon_tags_37 },
    { 0x44c5f104, SUC_lexicon_tags_12 },
    { 0x9d25f106, SUC_lexicon_tags_6 },
    { 0x42c5f100, SUC_lexicon_tags_1 },
    { 0xc105f108, SUC_lexicon_tags_23 },
    { 0xe365f109, SUC_lexicon_tags_64 },
    { 0xa025f108, SUC_lexicon_tags_97 },
    { 0x1045f10b, SUC_lexicon_tags_26 },
    { 0xb5e5f102, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac65f113, SUC_lexicon_tags_3 },
    { 0x64c5f114, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2945f116, SUC_lexicon_tags_12 },
    { 0xf0e5f117, SUC_lexicon_tags_64 },
    { 0x9385f117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c5f11c, SUC_lexicon_tags_15 },
    { 0x90c5f11d, SUC_lexicon_tags_27 },
    { 0x6345f11c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c5f121, SUC_lexicon_tags_37 },
    { 0xa445f122, SUC_lexicon_tags_15 },
    { 0xb725f123, SUC_lexicon_tags_7 },
    { 0xb4e5f124, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa5f127, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb705f12a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9345f12c, SUC_lexicon_tags_63 },
    { 0xd2c5f12d, SUC_lexicon_tags_26 },
    { 0x5285f12d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd925f130, SUC_lexicon_tags_2 },
    { 0x98e5f130, SUC_lexicon_tags_64 },
    { 0x4ba5f132, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa945f136, SUC_lexicon_tags_2 },
    { 0xb985f136, SUC_lexicon_tags_21 },
    { 0x6a05f137, SUC_lexicon_tags_19 },
    { 0xd25f139, SUC_lexicon_tags_60 },
    { 0xc165f137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc465f13c, SUC_lexicon_tags_12 },
    { 0xf9a5f13d, SUC_lexicon_tags_0 },
    { 0xd245f13c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a5f143, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765f14a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4065f14c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee5f14f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5805f151, SUC_lexicon_tags_4 },
    { 0x9405f152, SUC_lexicon_tags_14 },
    { 0xcfc5f153, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc1c5f155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x785f157, SUC_lexicon_tags_60 },
    { 0x7325f157, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c5f15b, SUC_lexicon_tags_0 },
    { 0xf285f15c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c5f15f, SUC_lexicon_tags_26 },
    { 0x4a85f15f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0a5f162, SUC_lexicon_tags_32 },
    { 0x7405f162, SUC_lexicon_tags_48 },
    { 0x8005f162, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e65f167, SUC_lexicon_tags_87 },
    { 0x6525f168, SUC_lexicon_tags_108 },
    { 0xbea5f167, SUC_lexicon_tags_12 },
    { 0x47a5f16a, SUC_lexicon_tags_11 },
    { 0xe3a5f168, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x82c5f16d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c5f170, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa845f174, SUC_lexicon_tags_1 },
    { 0x6a5f174, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x82a5f177, SUC_lexicon_tags_17 },
    { 0xaa85f178, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6845f17a, SUC_lexicon_tags_61 },
    { 0x6865f17b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4305f17f, SUC_lexicon_tags_117 },
    { 0xa2c5f180, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb765f182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba5f188, SUC_lexicon_tags_9 },
    { 0x345f189, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6505f18e, SUC_lexicon_tags_27 },
    { 0xa1e5f18f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe945f193, SUC_lexicon_tags_13 },
    { 0xa6e5f194, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcec5f196, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc5f199, SUC_lexicon_tags_25 },
    { 0xd245f19a, SUC_lexicon_tags_70 },
    { 0xa2c5f199, SUC_lexicon_tags_25 },
    { 0x10c5f19c, SUC_lexicon_tags_47 },
    { 0x27c5f19d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4e5f19f, SUC_lexicon_tags_47 },
    { 0xd685f1a0, SUC_lexicon_tags_45 },
    { 0x4b65f1a0, SUC_lexicon_tags_19 },
    { 0xe685f1a0, SUC_lexicon_tags_2 },
    { 0xbc85f1a3, SUC_lexicon_tags_23 },
    { 0x8425f1a1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd305f1a7, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x4585f1aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e5f1ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7485f1b6, SUC_lexicon_tags_140 },
    { 0x9365f1b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c5f1c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea5f1c8, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xd005f1ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1665f1cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4dc5f1ce, SUC_lexicon_tags_14 },
    { 0x9fa5f1cf, SUC_lexicon_tags_15 },
    { 0xf245f1d0, SUC_lexicon_tags_16 },
    { 0xdd05f1d1, SUC_lexicon_tags_12 },
    { 0xfe25f1d0, SUC_lexicon_tags_31 },
    { 0x1245f1d1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc665f1d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4385f1d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5f1de, SUC_lexicon_tags_61 },
    { 0x7c85f1df, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa5f1e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4c5f1e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa565f1e9, SUC_lexicon_tags_11 },
    { 0xa885f1ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcea5f1ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6c5f1ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d05f1f0, SUC_lexicon_tags_11 },
    { 0xcec5f1f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcc85f1f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c05f1f8, SUC_lexicon_tags_11 },
    { 0x8c85f1f8, SUC_lexicon_tags_23 },
    { 0xe6e5f1fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa5f1fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa45f202, SUC_lexicon_tags_10 },
    { 0xef85f203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e85f206, SUC_lexicon_tags_3 },
    { 0x9ce5f206, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c45f20a, SUC_lexicon_tags_7 },
    { 0x4765f20b, SUC_lexicon_tags_37 },
    { 0x8085f20a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe285f211, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc945f213, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3925f215, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc945f217, SUC_lexicon_tags_57 },
    { 0x4345f218, SUC_lexicon_tags_38 },
    { 0x6005f219, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x75a5f21b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc5f221, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c5f227, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c45f22a, SUC_lexicon_tags_136 },
    { 0xbe45f22b, SUC_lexicon_tags_28 },
    { 0x91c5f22c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd125f236, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f45f23a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb65f23d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc765f241, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a25f245, SUC_lexicon_tags_6 },
    { 0x76a5f246, SUC_lexicon_tags_17 },
    { 0x96c5f246, SUC_lexicon_tags_7 },
    { 0x4545f248, SUC_lexicon_tags_7 },
    { 0xb465f249, SUC_lexicon_tags_7 },
    { 0xd305f248, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc5e5f24c, SUC_lexicon_tags_12 },
    { 0x52a5f24d, SUC_lexicon_tags_25 },
    { 0x69a5f24e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd665f253, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x52a5f255, SUC_lexicon_tags_2 },
    { 0xdf85f255, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba5f25a, SUC_lexicon_tags_45 },
    { 0x9065f25b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb805f263, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9425f267, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46a5f269, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3c5f26b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c5f26e, SUC_lexicon_tags_14 },
    { 0x2465f26f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd665f274, SUC_lexicon_tags_11 },
    { 0x9d25f275, SUC_lexicon_tags_33 },
    { 0x1c65f276, SUC_lexicon_tags_43 },
    { 0xeba5f277, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9105f279, SUC_lexicon_tags_0 },
    { 0x5c05f27a, SUC_lexicon_tags_37 },
    { 0xea45f279, SUC_lexicon_tags_25 },
    { 0x10c5f27c, SUC_lexicon_tags_27 },
    { 0xb585f27c, SUC_lexicon_tags_37 },
    { 0x39c5f27e, SUC_lexicon_tags_15 },
    { 0xfe05f27f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6565f287, SUC_lexicon_tags_37 },
    { 0x145f288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1bc5f28a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc5f28c, SUC_lexicon_tags_39 },
    { 0x6225f28d, SUC_lexicon_tags_29 },
    { 0x445f28d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e5f291, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe545f294, SUC_lexicon_tags_21 },
    { 0x4da5f295, SUC_lexicon_tags_194 },
    { 0x6545f296, SUC_lexicon_tags_11 },
    { 0x9e25f296, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb845f29e, SUC_lexicon_tags_15 },
    { 0xb525f29f, SUC_lexicon_tags_12 },
    { 0xf865f2a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8545f2a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1885f2a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e5f2aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb365f2ac, SUC_lexicon_tags_14 },
    { 0x87e5f2ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2c5f2af, SUC_lexicon_tags_37 },
    { 0x6f25f2b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb905f2b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc65f2bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x63c5f2bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf885f2c3, SUC_lexicon_tags_8 },
    { 0xf545f2c4, SUC_lexicon_tags_8 },
    { 0xddc5f2c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c5f2c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c5f2cc, SUC_lexicon_tags_0 },
    { 0x1745f2cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe5f2d0, SUC_lexicon_tags_39 },
    { 0x2225f2d1, SUC_lexicon_tags_17 },
    { 0xeae5f2d2, SUC_lexicon_tags_2 },
    { 0x2485f2d3, SUC_lexicon_tags_19 },
    { 0xf6c5f2d4, SUC_lexicon_tags_43 },
    { 0xa645f2d5, SUC_lexicon_tags_1 },
    { 0xb325f2d6, SUC_lexicon_tags_65 },
    { 0x2e25f2d7, SUC_lexicon_tags_0 },
    { 0x5e65f2d3, SUC_lexicon_tags_1 },
    { 0x305f2d9, SUC_lexicon_tags_1 },
    { 0xbd25f2d7, SUC_lexicon_tags_26 },
    { 0xaf65f2db, SUC_lexicon_tags_45 },
    { 0xe5c5f2dc, SUC_lexicon_tags_11 },
    { 0x77c5f2d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa5a5f2df, SUC_lexicon_tags_15 },
    { 0x5b25f2e0, SUC_lexicon_tags_0 },
    { 0x55e5f2e1, SUC_lexicon_tags_26 },
    { 0x1465f2e1, SUC_lexicon_tags_15 },
    { 0x66e5f2e0, SUC_lexicon_tags_17 },
    { 0x68a5f2e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c85f2e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee5f2ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a5f2f2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa5f2f6, SUC_lexicon_tags_4 },
    { 0x9e85f2f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a45f2fa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda5f2fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd805f2ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c5f302, SUC_lexicon_tags_11 },
    { 0x2e25f303, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91a5f305, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x12a5f307, SUC_lexicon_tags_10 },
    { 0x7705f308, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x17a5f30a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8985f30e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x205f310, SUC_lexicon_tags_3 },
    { 0xbea5f311, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf285f313, SUC_lexicon_tags_9 },
    { 0xf4e5f314, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa5f317, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e5f31b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca5f32a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe45f331, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7125f335, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e5f33b, SUC_lexicon_tags_25 },
    { 0x2965f33b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e5f341, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c5f344, SUC_lexicon_tags_7 },
    { 0x91e5f345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8265f347, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c5f34b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d05f34f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf125f352, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd2e5f354, SUC_lexicon_tags_33 },
    { 0x3b05f355, SUC_lexicon_tags_26 },
    { 0xcbc5f356, SUC_lexicon_tags_11 },
    { 0x31c5f354, SUC_lexicon_tags_12 },
    { 0x5c85f355, SUC_lexicon_tags_6 },
    { 0x4fc5f359, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a5f35c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b45f361, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f45f366, SUC_lexicon_tags_34 },
    { 0x8d85f367, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c45f369, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e45f36c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf25f371, SUC_lexicon_tags_43 },
    { 0xcf45f372, SUC_lexicon_tags_7 },
    { 0x8285f372, SUC_lexicon_tags_17 },
    { 0x8125f371, SUC_lexicon_tags_15 },
    { 0xc5c5f375, SUC_lexicon_tags_1 },
    { 0x4445f375, SUC_lexicon_tags_98 },
    { 0xb4e5f377, SUC_lexicon_tags_45 },
    { 0x2c85f373, SUC_lexicon_tags_15 },
    { 0x8145f379, SUC_lexicon_tags_23 },
    { 0x3de5f37a, SUC_lexicon_tags_12 },
    { 0x2a65f37a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e5f381, SUC_lexicon_tags_31 },
    { 0xf525f382, SUC_lexicon_tags_37 },
    { 0xb725f381, SUC_lexicon_tags_15 },
    { 0x41c5f381, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a45f387, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c5f396, SUC_lexicon_tags_28 },
    { 0x8d25f396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3365f399, SUC_lexicon_tags_39 },
    { 0xb505f39a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd185f3a0, SUC_lexicon_tags_25 },
    { 0x1305f3a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7065f3a6, SUC_lexicon_tags_25 },
    { 0xfd85f3a6, SUC_lexicon_tags_0 },
    { 0x1425f3a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9485f3ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b85f3b0, SUC_lexicon_tags_87 },
    { 0xcc05f3b1, SUC_lexicon_tags_47 },
    { 0x2fa5f3b2, SUC_lexicon_tags_61 },
    { 0x20a5f3b3, SUC_lexicon_tags_38 },
    { 0xea65f3b4, SUC_lexicon_tags_12 },
    { 0xa3c5f3b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4665f3b8, SUC_lexicon_tags_16 },
    { 0x1e85f3b9, SUC_lexicon_tags_23 },
    { 0xa005f3b8, SUC_lexicon_tags_21 },
    { 0x2d65f3bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b45f3c1, SUC_lexicon_tags_10 },
    { 0x47e5f3c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d85f3c7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb065f3c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8725f3cb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5f3d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb225f3d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9725f3df, SUC_lexicon_tags_25 },
    { 0x6465f3df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd05f3ef, SUC_lexicon_tags_17 },
    { 0x2205f3f0, SUC_lexicon_tags_12 },
    { 0x1005f3ef, SUC_lexicon_tags_11 },
    { 0xf9c5f3f2, SUC_lexicon_tags_15 },
    { 0x1fa5f3f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b05f3f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6e5f3f7, SUC_lexicon_tags_82 },
    { 0x6325f3f7, SUC_lexicon_tags_26 },
    { 0xe1a5f3f7, SUC_lexicon_tags_33 },
    { 0x91c5f3f9, SUC_lexicon_tags_34 },
    { 0xcd65f3f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5125f3ff, SUC_lexicon_tags_25 },
    { 0xbb45f3ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe585f402, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf285f406, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a5f40a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e5f40d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e5f410, SUC_lexicon_tags_15 },
    { 0x29a5f410, SUC_lexicon_tags_1 },
    { 0x4185f412, SUC_lexicon_tags_19 },
    { 0x31a5f413, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee5f41a, SUC_lexicon_tags_25 },
    { 0xaba5f41b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e5f41d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22c5f41f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c5f423, SUC_lexicon_tags_15 },
    { 0x8305f424, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f25f428, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3685f42a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87e5f42c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa885f433, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6845f439, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a5f43c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2fc5f43e, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x1985f440, SUC_lexicon_tags_43 },
    { 0x3b05f440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe405f447, SUC_lexicon_tags_782 },
    { 0xf365f448, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e45f44c, SUC_lexicon_tags_37 },
    { 0xa645f44d, SUC_lexicon_tags_37 },
    { 0x9b05f44e, SUC_lexicon_tags_62 },
    { 0x2745f44f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f45f455, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a85f458, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe45f45a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4ca5f45c, SUC_lexicon_tags_60 },
    { 0xb685f45c, SUC_lexicon_tags_1 },
    { 0x1305f45e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd605f460, SUC_lexicon_tags_23 },
    { 0x3ea5f460, SUC_lexicon_tags_11 },
    { 0x3745f460, SUC_lexicon_tags_10 },
    { 0x89c5f463, SUC_lexicon_tags_61 },
    { 0x9c65f464, SUC_lexicon_tags_0 },
    { 0x7ec5f463, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca5f46b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7845f473, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x8585f475, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5f47b, SUC_lexicon_tags_60 },
    { 0xd125f47b, SUC_lexicon_tags_32 },
    { 0xdf45f47c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e85f480, SUC_lexicon_tags_15 },
    { 0x13a5f480, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33c5f483, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605f487, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e5f48c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4185f48f, SUC_lexicon_tags_7 },
    { 0xc825f490, SUC_lexicon_tags_7 },
    { 0xea05f491, SUC_lexicon_tags_12 },
    { 0x9f65f492, SUC_lexicon_tags_32 },
    { 0x1305f493, SUC_lexicon_tags_11 },
    { 0xe565f48f, SUC_lexicon_tags_37 },
    { 0xb7a5f495, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8005f49a, SUC_lexicon_tags_60 },
    { 0x81a5f49b, SUC_lexicon_tags_37 },
    { 0xf1e5f49c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2185f49e, SUC_lexicon_tags_17 },
    { 0x9c45f49f, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c5f4a2, SUC_lexicon_tags_1 },
    { 0x1a65f4a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb545f4a7, SUC_lexicon_tags_11 },
    { 0xfee5f4a8, SUC_lexicon_tags_37 },
    { 0x7145f4a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd785f4ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a5f4ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf145f4b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc945f4b8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5045f4bd, SUC_lexicon_tags_1 },
    { 0x665f4be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3885f4c7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc5f4ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a5f4d0, SUC_lexicon_tags_17 },
    { 0xe065f4d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac5f4d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72c5f4d6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4845f4d8, SUC_lexicon_tags_7 },
    { 0x5905f4d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5445f4e0, SUC_lexicon_tags_60 },
    { 0x5a45f4e1, SUC_lexicon_tags_17 },
    { 0x85a5f4e0, SUC_lexicon_tags_82 },
    { 0xa0e5f4e3, SUC_lexicon_tags_87 },
    { 0x7c85f4e4, SUC_lexicon_tags_5 },
    { 0xfe45f4e5, SUC_lexicon_tags_12 },
    { 0xf985f4e4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625f4ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa905f4ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6e5f4ee, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd565f4f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde5f4fd, SUC_lexicon_tags_31 },
    { 0x2005f4fe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd905f502, SUC_lexicon_tags_31 },
    { 0x54c5f502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfe5f505, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x38a5f507, SUC_lexicon_tags_7 },
    { 0x3d25f507, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca5f50b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8085f510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f65f514, SUC_lexicon_tags_26 },
    { 0x97a5f515, SUC_lexicon_tags_25 },
    { 0x2905f515, SUC_lexicon_tags_45 },
    { 0xb0c5f516, SUC_lexicon_tags_57 },
    { 0x8e25f516, SUC_lexicon_tags_77 },
    { 0x95e5f519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe185f51e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4665f524, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19a5f526, SUC_lexicon_tags_77 },
    { 0xff45f527, SUC_lexicon_tags_40 },
    { 0xdc65f528, SUC_lexicon_tags_2 },
    { 0x4805f526, SUC_lexicon_tags_7 },
    { 0x28e5f52a, SUC_lexicon_tags_11 },
    { 0xb885f52a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d65f530, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7125f534, SUC_lexicon_tags_234 },
    { 0x1385f535, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e65f539, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf785f53c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe365f545, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3185f547, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56c5f549, SUC_lexicon_tags_25 },
    { 0x9445f549, SUC_lexicon_tags_48 },
    { 0x965f54b, SUC_lexicon_tags_11 },
    { 0x88c5f54c, SUC_lexicon_tags_66 },
    { 0x9065f54b, SUC_lexicon_tags_47 },
    { 0xd625f54e, SUC_lexicon_tags_12 },
    { 0xf225f54e, SUC_lexicon_tags_1 },
    { 0xd945f550, SUC_lexicon_tags_16 },
    { 0x3185f54e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8485f553, SUC_lexicon_tags_1 },
    { 0x69a5f553, SUC_lexicon_tags_11 },
    { 0x2b25f555, SUC_lexicon_tags_83 },
    { 0xff85f556, SUC_lexicon_tags_141 },
    { 0xca25f555, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb725f559, SUC_lexicon_tags_49 },
    { 0x8ae5f55a, SUC_lexicon_tags_12 },
    { 0x21c5f559, SUC_lexicon_tags_60 },
    { 0xfd25f55c, SUC_lexicon_tags_264 },
    { 0xd0e5f55a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5f561, SUC_lexicon_tags_7 },
    { 0xc285f562, SUC_lexicon_tags_43 },
    { 0xf165f562, SUC_lexicon_tags_45 },
    { 0xc465f564, SUC_lexicon_tags_0 },
    { 0x45c5f565, SUC_lexicon_tags_30 },
    { 0xa565f566, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e5f56a, SUC_lexicon_tags_33 },
    { 0x8c65f56b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5725f575, SUC_lexicon_tags_27 },
    { 0xe325f576, SUC_lexicon_tags_14 },
    { 0xe365f577, SUC_lexicon_tags_7 },
    { 0xb485f578, SUC_lexicon_tags_0 },
    { 0x1a45f577, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2e5f57b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8945f57d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeac5f57f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05f582, SUC_lexicon_tags_93 },
    { 0xbd85f582, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ec5f585, SUC_lexicon_tags_15 },
    { 0xb205f586, SUC_lexicon_tags_33 },
    { 0xa665f587, SUC_lexicon_tags_2 },
    { 0x2b05f588, SUC_lexicon_tags_56 },
    { 0x1b85f589, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee5f58c, SUC_lexicon_tags_23 },
    { 0x1785f58d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5325f58f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b25f591, SUC_lexicon_tags_138 },
    { 0xd485f592, SUC_lexicon_tags_0 },
    { 0xb345f591, SUC_lexicon_tags_25 },
    { 0xd8a5f593, SUC_lexicon_tags_25 },
    { 0x4da5f595, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05f598, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe125f59b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e5f59f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a5f5a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3345f5a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e5f5aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf785f5ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe325f5b0, SUC_lexicon_tags_7 },
    { 0x22c5f5b1, SUC_lexicon_tags_17 },
    { 0x3545f5b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b85f5b5, SUC_lexicon_tags_33 },
    { 0xb505f5b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b85f5b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1545f5ba, SUC_lexicon_tags_15 },
    { 0x20c5f5bb, SUC_lexicon_tags_1 },
    { 0x18c5f5bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe965f5be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ac5f5c0, SUC_lexicon_tags_19 },
    { 0x3f65f5c1, SUC_lexicon_tags_12 },
    { 0xe965f5c2, SUC_lexicon_tags_9 },
    { 0x31e5f5c2, SUC_lexicon_tags_31 },
    { 0x28a5f5c4, SUC_lexicon_tags_10 },
    { 0xe285f5c5, SUC_lexicon_tags_15 },
    { 0x8745f5c5, SUC_lexicon_tags_11 },
    { 0xac85f5c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef65f5cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8885f5cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a45f5d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9125f5d5, SUC_lexicon_tags_26 },
    { 0xe325f5d6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e5f5d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9e5f5db, SUC_lexicon_tags_37 },
    { 0x8365f5db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbbe5f5de, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0xa945f5e1, SUC_lexicon_tags_45 },
    { 0x1e05f5e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb485f5e4, SUC_lexicon_tags_77 },
    { 0x9065f5e4, SUC_lexicon_tags_7 },
    { 0x85c5f5e6, SUC_lexicon_tags_2 },
    { 0x7885f5e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde5f5e9, SUC_lexicon_tags_37 },
    { 0xfe5f5e9, SUC_lexicon_tags_15 },
    { 0xd345f5eb, SUC_lexicon_tags_25 },
    { 0xae85f5ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce5f5f1, SUC_lexicon_tags_8 },
    { 0x8185f5f2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb085f5f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58e5f5f7, SUC_lexicon_tags_9 },
    { 0xaf05f5f8, SUC_lexicon_tags_15 },
    { 0xc545f5f8, SUC_lexicon_tags_11 },
    { 0xe5a5f5f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad05f5fe, SUC_lexicon_tags_4 },
    { 0xb025f5fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16c5f601, SUC_lexicon_tags_30 },
    { 0x2365f602, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de5f60a, SUC_lexicon_tags_17 },
    { 0xa7c5f60a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef25f60e, SUC_lexicon_tags_55 },
    { 0xd365f60f, SUC_lexicon_tags_49 },
    { 0x1e5f610, SUC_lexicon_tags_14 },
    { 0xd6c5f611, SUC_lexicon_tags_7 },
    { 0xc2e5f612, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14e5f614, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445f61a, SUC_lexicon_tags_1 },
    { 0xc345f61b, SUC_lexicon_tags_14 },
    { 0xfd25f61b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea45f622, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a25f629, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2705f62b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1945f62f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9885f631, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7765f635, SUC_lexicon_tags_11 },
    { 0xb9c5f636, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8165f63b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c65f63d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b05f645, SUC_lexicon_tags_49 },
    { 0x9c65f646, SUC_lexicon_tags_92 },
    { 0xe4c5f647, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2485f649, SUC_lexicon_tags_19 },
    { 0x1c65f64a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba85f64d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a85f651, SUC_lexicon_tags_37 },
    { 0x1845f652, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a5f65c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c85f662, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c5f665, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3265f667, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5165f66b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2545f66d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29c5f66f, SUC_lexicon_tags_12 },
    { 0xdae5f670, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa905f675, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0xa145f677, SUC_lexicon_tags_17 },
    { 0x88c5f678, SUC_lexicon_tags_38 },
    { 0x2405f678, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e5f67f, SUC_lexicon_tags_37 },
    { 0x45f67f, SUC_lexicon_tags_0 },
    { 0x5f67f, SUC_lexicon_tags_11 },
    { 0x7705f682, SUC_lexicon_tags_2 },
    { 0xb965f681, SUC_lexicon_tags_2 },
    { 0x5a65f683, SUC_lexicon_tags_39 },
    { 0xfe05f685, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd585f689, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa825f68d, SUC_lexicon_tags_25 },
    { 0xf205f68e, SUC_lexicon_tags_12 },
    { 0x7325f68f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc5f694, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c5f697, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a5f6a3, SUC_lexicon_tags_66 },
    { 0xc465f6a4, SUC_lexicon_tags_25 },
    { 0x9d85f6a4, SUC_lexicon_tags_26 },
    { 0x5d85f6a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285f6aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc5f6ae, SUC_lexicon_tags_26 },
    { 0xc005f6ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5f6b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd85f6b4, SUC_lexicon_tags_17 },
    { 0xce45f6b5, SUC_lexicon_tags_23 },
    { 0xf0e5f6b6, SUC_lexicon_tags_11 },
    { 0xc45f6b6, SUC_lexicon_tags_17 },
    { 0xb025f6b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed85f6bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc605f6be, SUC_lexicon_tags_8 },
    { 0xcdc5f6bf, SUC_lexicon_tags_19 },
    { 0x8dc5f6c0, SUC_lexicon_tags_6 },
    { 0x3a45f6bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3c5f6c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf85f6c6, SUC_lexicon_tags_49 },
    { 0xd625f6c7, SUC_lexicon_tags_60 },
    { 0x3145f6c7, SUC_lexicon_tags_17 },
    { 0x6105f6c9, SUC_lexicon_tags_17 },
    { 0xede5f6c9, SUC_lexicon_tags_15 },
    { 0x9a65f6cb, SUC_lexicon_tags_25 },
    { 0x2925f6ca, SUC_lexicon_tags_34 },
    { 0x69c5f6c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45e5f6cf, SUC_lexicon_tags_35 },
    { 0x6085f6d0, SUC_lexicon_tags_29 },
    { 0xfde5f6cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2de5f6d3, SUC_lexicon_tags_37 },
    { 0xc205f6d4, SUC_lexicon_tags_11 },
    { 0xea45f6d4, SUC_lexicon_tags_17 },
    { 0x3845f6d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x57c5f6d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5805f6da, SUC_lexicon_tags_11 },
    { 0x3fc5f6da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9305f6de, SUC_lexicon_tags_15 },
    { 0x1c05f6de, SUC_lexicon_tags_15 },
    { 0x21e5f6df, SUC_lexicon_tags_60 },
    { 0xac45f6df, SUC_lexicon_tags_2 },
    { 0x25f6e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5f6e7, SUC_lexicon_tags_25 },
    { 0x1985f6e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1705f6ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef05f6ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f85f6f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec05f6f5, SUC_lexicon_tags_7 },
    { 0xd485f6f5, SUC_lexicon_tags_23 },
    { 0x7f25f6f5, SUC_lexicon_tags_23 },
    { 0x9785f6f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a65f6ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf405f703, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfe85f705, SUC_lexicon_tags_43 },
    { 0xea85f706, SUC_lexicon_tags_15 },
    { 0x68e5f707, SUC_lexicon_tags_48 },
    { 0x5145f708, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7365f70b, SUC_lexicon_tags_8 },
    { 0x11c5f70c, SUC_lexicon_tags_16 },
    { 0x3dc5f70d, SUC_lexicon_tags_2 },
    { 0xce45f70e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe5f712, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1165f714, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a5f718, SUC_lexicon_tags_15 },
    { 0x805f718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab25f71c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a45f71f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c5f722, SUC_lexicon_tags_1 },
    { 0x3285f723, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6905f725, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c05f729, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a5f72c, SUC_lexicon_tags_63 },
    { 0x1e65f72d, SUC_lexicon_tags_680 },
    { 0, NULL },
    { 0x1785f72f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e5f734, SUC_lexicon_tags_46 },
    { 0x7085f735, SUC_lexicon_tags_45 },
    { 0x145f736, SUC_lexicon_tags_5 },
    { 0x5ee5f737, SUC_lexicon_tags_175 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405f73c, SUC_lexicon_tags_12 },
    { 0x1ce5f73d, SUC_lexicon_tags_837 },
    { 0xece5f73d, SUC_lexicon_tags_65 },
    { 0x52e5f73d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de5f742, SUC_lexicon_tags_37 },
    { 0x7925f743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe905f745, SUC_lexicon_tags_17 },
    { 0x8bc5f746, SUC_lexicon_tags_29 },
    { 0xc345f746, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x58a5f749, SUC_lexicon_tags_25 },
    { 0x18c5f74a, SUC_lexicon_tags_2 },
    { 0xd685f74b, SUC_lexicon_tags_49 },
    { 0xafa5f74a, SUC_lexicon_tags_25 },
    { 0x465f74b, SUC_lexicon_tags_45 },
    { 0xb805f74e, SUC_lexicon_tags_44 },
    { 0x1445f74b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c5f752, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c65f755, SUC_lexicon_tags_19 },
    { 0x4145f756, SUC_lexicon_tags_17 },
    { 0xcaa5f755, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb145f75a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e05f763, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e05f76c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1845f770, SUC_lexicon_tags_30 },
    { 0x265f771, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f775, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x16a5f777, SUC_lexicon_tags_12 },
    { 0x2c5f777, SUC_lexicon_tags_35 },
    { 0x80a5f779, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc5f77c, SUC_lexicon_tags_27 },
    { 0x1ce5f77c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45f782, SUC_lexicon_tags_37 },
    { 0x6425f782, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a5f788, SUC_lexicon_tags_2 },
    { 0x545f789, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a5f78f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6465f791, SUC_lexicon_tags_37 },
    { 0x48c5f791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6825f795, SUC_lexicon_tags_2 },
    { 0x6765f796, SUC_lexicon_tags_23 },
    { 0xdc5f797, SUC_lexicon_tags_11 },
    { 0x3e25f798, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf205f79a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94e5f79c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c5f79f, SUC_lexicon_tags_2 },
    { 0xe425f7a0, SUC_lexicon_tags_49 },
    { 0x365f7a1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x55a5f7a3, SUC_lexicon_tags_29 },
    { 0xc125f7a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fc5f7a6, SUC_lexicon_tags_43 },
    { 0x22c5f7a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2205f7ae, SUC_lexicon_tags_0 },
    { 0xfe45f7af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05f7b4, SUC_lexicon_tags_151 },
    { 0xf245f7b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7625f7bf, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff45f7c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x34c5f7c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf985f7cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a45f7d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d65f7d4, SUC_lexicon_tags_55 },
    { 0xe485f7d5, SUC_lexicon_tags_17 },
    { 0x4b45f7d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4905f7d8, SUC_lexicon_tags_17 },
    { 0xbf05f7d8, SUC_lexicon_tags_47 },
    { 0x4885f7d9, SUC_lexicon_tags_37 },
    { 0xb645f7db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea85f7df, SUC_lexicon_tags_11 },
    { 0x9085f7e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf105f7e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2605f7e5, SUC_lexicon_tags_1 },
    { 0x6aa5f7e6, SUC_lexicon_tags_2 },
    { 0x1525f7e5, SUC_lexicon_tags_6 },
    { 0x545f7e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc5f7ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc345f7f5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2a5f7f7, SUC_lexicon_tags_37 },
    { 0x1185f7f7, SUC_lexicon_tags_14 },
    { 0xa1c5f7f7, SUC_lexicon_tags_4 },
    { 0x8085f7fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa5f7fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe605f7fe, SUC_lexicon_tags_63 },
    { 0xe805f7ff, SUC_lexicon_tags_15 },
    { 0x9da5f800, SUC_lexicon_tags_1 },
    { 0xf925f800, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2885f805, SUC_lexicon_tags_11 },
    { 0xf285f806, SUC_lexicon_tags_37 },
    { 0xbd65f807, SUC_lexicon_tags_18 },
    { 0x5585f807, SUC_lexicon_tags_14 },
    { 0x7405f809, SUC_lexicon_tags_1 },
    { 0xb825f80a, SUC_lexicon_tags_2 },
    { 0xfc65f80b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda5f811, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a5f819, SUC_lexicon_tags_12 },
    { 0xffa5f81a, SUC_lexicon_tags_49 },
    { 0x85e5f81a, SUC_lexicon_tags_47 },
    { 0x5425f81c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb25f821, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1165f823, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe585f827, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee05f82c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf505f82e, SUC_lexicon_tags_12 },
    { 0x8c65f82f, SUC_lexicon_tags_0 },
    { 0x95a5f82e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae85f838, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5f83e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe85f842, SUC_lexicon_tags_1 },
    { 0x59e5f843, SUC_lexicon_tags_37 },
    { 0xfd05f843, SUC_lexicon_tags_16 },
    { 0xdfc5f845, SUC_lexicon_tags_15 },
    { 0xe5f844, SUC_lexicon_tags_17 },
    { 0x4165f844, SUC_lexicon_tags_25 },
    { 0xf025f842, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc5f84c, SUC_lexicon_tags_25 },
    { 0xe605f84c, SUC_lexicon_tags_27 },
    { 0x2745f84e, SUC_lexicon_tags_11 },
    { 0xff85f84f, SUC_lexicon_tags_37 },
    { 0xb485f84e, SUC_lexicon_tags_61 },
    { 0xa825f84e, SUC_lexicon_tags_7 },
    { 0x1825f84c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x69a5f854, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb485f859, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc5f85c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac25f85e, SUC_lexicon_tags_37 },
    { 0xabe5f85f, SUC_lexicon_tags_17 },
    { 0xee05f860, SUC_lexicon_tags_11 },
    { 0x2885f861, SUC_lexicon_tags_1 },
    { 0xae25f862, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd925f865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc565f86a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7405f86c, SUC_lexicon_tags_3 },
    { 0xd4e5f86d, SUC_lexicon_tags_14 },
    { 0xcd45f86e, SUC_lexicon_tags_1 },
    { 0xbca5f86f, SUC_lexicon_tags_30 },
    { 0x21e5f86e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba5f873, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6ce5f875, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac5f87b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde45f87d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe665f87f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x305f881, SUC_lexicon_tags_3 },
    { 0x2ec5f882, SUC_lexicon_tags_11 },
    { 0x6dc5f883, SUC_lexicon_tags_7 },
    { 0xc1c5f883, SUC_lexicon_tags_12 },
    { 0x58e5f885, SUC_lexicon_tags_11 },
    { 0x14c5f886, SUC_lexicon_tags_37 },
    { 0x3145f885, SUC_lexicon_tags_62 },
    { 0x22e5f886, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1565f88a, SUC_lexicon_tags_37 },
    { 0xe405f88a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c5f892, SUC_lexicon_tags_12 },
    { 0x89c5f893, SUC_lexicon_tags_15 },
    { 0xdfe5f894, SUC_lexicon_tags_11 },
    { 0x68a5f895, SUC_lexicon_tags_2 },
    { 0x3645f895, SUC_lexicon_tags_25 },
    { 0x5005f894, SUC_lexicon_tags_1273 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8665f89c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe5f8a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e65f8a4, SUC_lexicon_tags_28 },
    { 0xf3a5f8a4, SUC_lexicon_tags_37 },
    { 0xbb85f8a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7265f8a8, SUC_lexicon_tags_11 },
    { 0xc765f8a9, SUC_lexicon_tags_23 },
    { 0x3e25f8a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce45f8ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1845f8bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a5f8bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665f8c4, SUC_lexicon_tags_15 },
    { 0xe605f8c5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba5f8cb, SUC_lexicon_tags_23 },
    { 0x2a5f8cc, SUC_lexicon_tags_1 },
    { 0x2605f8cd, SUC_lexicon_tags_74 },
    { 0x6885f8cb, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd85f8d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef85f8e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e5f8e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6445f8e8, SUC_lexicon_tags_11 },
    { 0x9a65f8e9, SUC_lexicon_tags_9 },
    { 0x36e5f8ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e5f8ee, SUC_lexicon_tags_70 },
    { 0x77a5f8ef, SUC_lexicon_tags_16 },
    { 0x4585f8f0, SUC_lexicon_tags_37 },
    { 0x7ec5f8f0, SUC_lexicon_tags_55 },
    { 0x5885f8f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a5f8f5, SUC_lexicon_tags_37 },
    { 0x7025f8f6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc5f8fa, SUC_lexicon_tags_0 },
    { 0x61c5f8fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa05f900, SUC_lexicon_tags_0 },
    { 0x4145f900, SUC_lexicon_tags_37 },
    { 0x2f25f902, SUC_lexicon_tags_8 },
    { 0x6c85f903, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe165f905, SUC_lexicon_tags_38 },
    { 0x89c5f906, SUC_lexicon_tags_31 },
    { 0x9ac5f905, SUC_lexicon_tags_23 },
    { 0xf625f908, SUC_lexicon_tags_38 },
    { 0x9885f909, SUC_lexicon_tags_23 },
    { 0x3a85f906, SUC_lexicon_tags_7 },
    { 0xc05f90b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3a85f90d, SUC_lexicon_tags_2 },
    { 0x85c5f90d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a5f911, SUC_lexicon_tags_31 },
    { 0xac5f911, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e5f915, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e5f919, SUC_lexicon_tags_19 },
    { 0xaa45f91a, SUC_lexicon_tags_25 },
    { 0xe025f919, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca85f91d, SUC_lexicon_tags_0 },
    { 0x78e5f91e, SUC_lexicon_tags_19 },
    { 0xda5f91f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd145f925, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3085f928, SUC_lexicon_tags_11 },
    { 0x7745f928, SUC_lexicon_tags_7 },
    { 0x8605f928, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde5f92d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6145f930, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a05f936, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a65f938, SUC_lexicon_tags_17 },
    { 0x3265f938, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2425f93b, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c25f941, SUC_lexicon_tags_7 },
    { 0xfe65f941, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c5f948, SUC_lexicon_tags_25 },
    { 0x1fe5f949, SUC_lexicon_tags_11 },
    { 0xaa65f949, SUC_lexicon_tags_1 },
    { 0x18a5f94a, SUC_lexicon_tags_2 },
    { 0x3725f94c, SUC_lexicon_tags_9 },
    { 0x7525f94d, SUC_lexicon_tags_25 },
    { 0xa1a5f94e, SUC_lexicon_tags_47 },
    { 0x9f25f94c, SUC_lexicon_tags_27 },
    { 0xe9a5f94e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9ba5f952, SUC_lexicon_tags_33 },
    { 0x9dc5f952, SUC_lexicon_tags_63 },
    { 0xa365f954, SUC_lexicon_tags_15 },
    { 0x59e5f952, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb85f957, SUC_lexicon_tags_28 },
    { 0xee05f958, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7005f95b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ce5f95d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d05f963, SUC_lexicon_tags_11 },
    { 0xa45f964, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2545f967, SUC_lexicon_tags_146 },
    { 0x6fa5f968, SUC_lexicon_tags_62 },
    { 0x7725f969, SUC_lexicon_tags_45 },
    { 0xd145f968, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda5f971, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa685f973, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6085f975, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5f97c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5245f97f, SUC_lexicon_tags_34 },
    { 0xf865f97f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x89e5f982, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf65f984, SUC_lexicon_tags_9 },
    { 0xa105f985, SUC_lexicon_tags_34 },
    { 0xd0c5f986, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b25f988, SUC_lexicon_tags_15 },
    { 0xc985f989, SUC_lexicon_tags_25 },
    { 0x82a5f988, SUC_lexicon_tags_7 },
    { 0xb365f988, SUC_lexicon_tags_17 },
    { 0x8925f98c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3245f991, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c5f997, SUC_lexicon_tags_2 },
    { 0xe345f998, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3705f99a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a5f9a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9045f9a4, SUC_lexicon_tags_32 },
    { 0x1a65f9a5, SUC_lexicon_tags_1 },
    { 0xb7a5f9a4, SUC_lexicon_tags_26 },
    { 0x40c5f9a7, SUC_lexicon_tags_47 },
    { 0x56a5f9a6, SUC_lexicon_tags_65 },
    { 0x8685f9a9, SUC_lexicon_tags_29 },
    { 0x1345f9aa, SUC_lexicon_tags_11 },
    { 0xf9e5f9a7, SUC_lexicon_tags_33 },
    { 0xa4e5f9ac, SUC_lexicon_tags_47 },
    { 0x7845f9ac, SUC_lexicon_tags_12 },
    { 0x5a85f9a6, SUC_lexicon_tags_43 },
    { 0xea5f9ab, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce5f9b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc525f9bc, SUC_lexicon_tags_25 },
    { 0x30a5f9bd, SUC_lexicon_tags_19 },
    { 0xd185f9be, SUC_lexicon_tags_47 },
    { 0x1ee5f9bf, SUC_lexicon_tags_12 },
    { 0x3e85f9c0, SUC_lexicon_tags_0 },
    { 0xce85f9c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c5f9c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b85f9cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc305f9ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c5f9d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2c5f9d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a65f9d8, SUC_lexicon_tags_25 },
    { 0x2c65f9d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc985f9dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe365f9e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8625f9e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f9ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xec25f9ec, SUC_lexicon_tags_29 },
    { 0x2345f9ec, SUC_lexicon_tags_25 },
    { 0x7605f9ec, SUC_lexicon_tags_8 },
    { 0x3c65f9ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe85f9f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xee65f9f7, SUC_lexicon_tags_38 },
    { 0xce65f9f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf6a5f9fa, SUC_lexicon_tags_15 },
    { 0x3625f9fb, SUC_lexicon_tags_26 },
    { 0xc3e5f9fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3985f9fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1365fa00, SUC_lexicon_tags_37 },
    { 0xfe85fa01, SUC_lexicon_tags_47 },
    { 0x5be5fa02, SUC_lexicon_tags_26 },
    { 0xc105fa02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5745fa05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc65fa0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x225fa0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x285fa10, SUC_lexicon_tags_49 },
    { 0x4565fa11, SUC_lexicon_tags_37 },
    { 0x8d25fa10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e5fa15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5fa1a, SUC_lexicon_tags_1 },
    { 0xa905fa1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7305fa1d, SUC_lexicon_tags_19 },
    { 0xfe85fa1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a5fa2a, SUC_lexicon_tags_9 },
    { 0x25c5fa2b, SUC_lexicon_tags_37 },
    { 0xc305fa2b, SUC_lexicon_tags_87 },
    { 0x4e45fa2d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a5fa31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a5fa38, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf605fa3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc665fa3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6585fa3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe485fa40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf445fa42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a05fa45, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1105fa47, SUC_lexicon_tags_14 },
    { 0x93a5fa48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb05fa4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe45fa4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5fa54, SUC_lexicon_tags_25 },
    { 0x2285fa55, SUC_lexicon_tags_9 },
    { 0x55a5fa56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c5fa5a, SUC_lexicon_tags_12 },
    { 0x2285fa5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7285fa5d, SUC_lexicon_tags_25 },
    { 0x8565fa5d, SUC_lexicon_tags_23 },
    { 0x2285fa5f, SUC_lexicon_tags_21 },
    { 0xb45fa60, SUC_lexicon_tags_26 },
    { 0x5b85fa61, SUC_lexicon_tags_15 },
    { 0x64a5fa62, SUC_lexicon_tags_49 },
    { 0xce65fa62, SUC_lexicon_tags_25 },
    { 0x91e5fa64, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa825fa67, SUC_lexicon_tags_12 },
    { 0xca45fa67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1785fa6f, SUC_lexicon_tags_11 },
    { 0xf045fa70, SUC_lexicon_tags_1 },
    { 0x2665fa70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc05fa73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4185fa75, SUC_lexicon_tags_1 },
    { 0xbe5fa76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9985fa79, SUC_lexicon_tags_14 },
    { 0x2725fa79, SUC_lexicon_tags_22 },
    { 0xa785fa7b, SUC_lexicon_tags_15 },
    { 0x8605fa79, SUC_lexicon_tags_35 },
    { 0x25a5fa7b, SUC_lexicon_tags_48 },
    { 0xd985fa7e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f65fa83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdfa5fa85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e25fa87, SUC_lexicon_tags_29 },
    { 0x6f85fa87, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x33a5fa8a, SUC_lexicon_tags_13 },
    { 0xaa45fa8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c05fa8d, SUC_lexicon_tags_1 },
    { 0x545fa8d, SUC_lexicon_tags_7 },
    { 0xb345fa8f, SUC_lexicon_tags_48 },
    { 0x1d85fa90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf05fa92, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f25fa95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b85fa9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e5fa9e, SUC_lexicon_tags_37 },
    { 0x4305fa9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb845faa1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8785faa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8585faa9, SUC_lexicon_tags_11 },
    { 0x57e5faa9, SUC_lexicon_tags_1 },
    { 0xfc65faab, SUC_lexicon_tags_12 },
    { 0x66e5faac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xca45faaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a85fab2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6005fab4, SUC_lexicon_tags_11 },
    { 0x4065fab5, SUC_lexicon_tags_63 },
    { 0xb65fab6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6385faca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c5fad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd005fad5, SUC_lexicon_tags_15 },
    { 0xfb85fad5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x35a5fad8, SUC_lexicon_tags_10 },
    { 0x5725fad8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2905fadf, SUC_lexicon_tags_12 },
    { 0x7525fae0, SUC_lexicon_tags_37 },
    { 0xa3a5fae1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a5fae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa845faea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc845faec, SUC_lexicon_tags_37 },
    { 0x2ba5faed, SUC_lexicon_tags_37 },
    { 0xc6e5faee, SUC_lexicon_tags_87 },
    { 0x8c25faef, SUC_lexicon_tags_87 },
    { 0xc525faed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45faf4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca5faf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c45fafe, SUC_lexicon_tags_79 },
    { 0xfd25faff, SUC_lexicon_tags_2 },
    { 0x3665fb00, SUC_lexicon_tags_19 },
    { 0x60a5fb01, SUC_lexicon_tags_8 },
    { 0x1985fb00, SUC_lexicon_tags_17 },
    { 0xb5a5fb02, SUC_lexicon_tags_23 },
    { 0x15a5fb04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c5fb08, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2285fb0c, SUC_lexicon_tags_37 },
    { 0xda5fb0d, SUC_lexicon_tags_49 },
    { 0x8005fb0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8205fb13, SUC_lexicon_tags_10 },
    { 0x2e65fb13, SUC_lexicon_tags_11 },
    { 0xcc45fb15, SUC_lexicon_tags_33 },
    { 0x93e5fb15, SUC_lexicon_tags_10 },
    { 0xd985fb15, SUC_lexicon_tags_33 },
    { 0xc45fb18, SUC_lexicon_tags_23 },
    { 0x3245fb19, SUC_lexicon_tags_17 },
    { 0x4145fb19, SUC_lexicon_tags_25 },
    { 0x49c5fb1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a5fb1e, SUC_lexicon_tags_25 },
    { 0xb065fb1e, SUC_lexicon_tags_15 },
    { 0x5aa5fb20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1645fb23, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9805fb25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc725fb29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd125fb30, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea5fb35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe5fb38, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd445fb3a, SUC_lexicon_tags_15 },
    { 0x4b45fb3b, SUC_lexicon_tags_11 },
    { 0xc425fb3c, SUC_lexicon_tags_25 },
    { 0xae25fb3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa805fb40, SUC_lexicon_tags_28 },
    { 0x85e5fb40, SUC_lexicon_tags_37 },
    { 0x6965fb42, SUC_lexicon_tags_2 },
    { 0x8745fb43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4fe5fb45, SUC_lexicon_tags_37 },
    { 0x5c65fb45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e5fb49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a5fb4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8005fb50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca5fb54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e45fb58, SUC_lexicon_tags_33 },
    { 0x4785fb58, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xff25fb5c, SUC_lexicon_tags_6 },
    { 0x5565fb5d, SUC_lexicon_tags_37 },
    { 0xa585fb5c, SUC_lexicon_tags_25 },
    { 0xe905fb5f, SUC_lexicon_tags_43 },
    { 0x5a25fb5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf85fb64, SUC_lexicon_tags_14 },
    { 0xffa5fb65, SUC_lexicon_tags_138 },
    { 0x72a5fb66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2885fb68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205fb6f, SUC_lexicon_tags_25 },
    { 0x5805fb70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce25fb75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac5fb78, SUC_lexicon_tags_38 },
    { 0xed45fb79, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd885fb7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa45fb7e, SUC_lexicon_tags_2 },
    { 0xe9a5fb7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d65fb82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a5fb86, SUC_lexicon_tags_25 },
    { 0x64c5fb87, SUC_lexicon_tags_9 },
    { 0xdee5fb87, SUC_lexicon_tags_31 },
    { 0x5f85fb89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1585fb8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e25fb91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a5fb95, SUC_lexicon_tags_15 },
    { 0x18e5fb96, SUC_lexicon_tags_27 },
    { 0xb4e5fb96, SUC_lexicon_tags_25 },
    { 0xcc65fb98, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd205fb9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e85fb9f, SUC_lexicon_tags_60 },
    { 0x6965fb9f, SUC_lexicon_tags_37 },
    { 0xca05fba1, SUC_lexicon_tags_28 },
    { 0x9d65fba2, SUC_lexicon_tags_47 },
    { 0xd565fba2, SUC_lexicon_tags_60 },
    { 0x1145fb9f, SUC_lexicon_tags_1 },
    { 0x69a5fba4, SUC_lexicon_tags_0 },
    { 0x5245fba6, SUC_lexicon_tags_37 },
    { 0x2a05fba5, SUC_lexicon_tags_37 },
    { 0xd4e5fba8, SUC_lexicon_tags_7 },
    { 0x1185fba9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3665fbad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c25fbaf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc5fbb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c5fbb6, SUC_lexicon_tags_34 },
    { 0xb765fbb7, SUC_lexicon_tags_15 },
    { 0xdf05fbb8, SUC_lexicon_tags_32 },
    { 0x6fe5fbb9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de5fbbc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5fbc0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e5fbc9, SUC_lexicon_tags_12 },
    { 0xf305fbca, SUC_lexicon_tags_11 },
    { 0x8de5fbcb, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf45fbd1, SUC_lexicon_tags_46 },
    { 0xa765fbd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6a5fbd4, SUC_lexicon_tags_11 },
    { 0x65e5fbd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89e5fbd7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6345fbde, SUC_lexicon_tags_16 },
    { 0x1065fbdf, SUC_lexicon_tags_64 },
    { 0x9b25fbdf, SUC_lexicon_tags_38 },
    { 0x8825fbe1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b65fbe7, SUC_lexicon_tags_47 },
    { 0x74c5fbe8, SUC_lexicon_tags_33 },
    { 0x9165fbe9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd525fbeb, SUC_lexicon_tags_7 },
    { 0x4c65fbec, SUC_lexicon_tags_17 },
    { 0xd025fbed, SUC_lexicon_tags_17 },
    { 0xf1a5fbee, SUC_lexicon_tags_17 },
    { 0xd85fbef, SUC_lexicon_tags_1 },
    { 0xc325fbeb, SUC_lexicon_tags_30 },
    { 0x745fbf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e5fbf5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xa965fbf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e05fbf9, SUC_lexicon_tags_6 },
    { 0x8de5fbf9, SUC_lexicon_tags_87 },
    { 0x2fe5fbf9, SUC_lexicon_tags_47 },
    { 0xf1c5fbfc, SUC_lexicon_tags_60 },
    { 0x6f85fbfd, SUC_lexicon_tags_10 },
    { 0x5985fbfb, SUC_lexicon_tags_25 },
    { 0x58a5fbff, SUC_lexicon_tags_47 },
    { 0xab65fbff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a5fc06, SUC_lexicon_tags_1 },
    { 0xb585fc06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac5fc10, SUC_lexicon_tags_6 },
    { 0xb4c5fc11, SUC_lexicon_tags_19 },
    { 0x4505fc12, SUC_lexicon_tags_2 },
    { 0x2f05fc11, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1c65fc15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3585fc18, SUC_lexicon_tags_36 },
    { 0xf105fc19, SUC_lexicon_tags_14 },
    { 0x23a5fc1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca45fc24, SUC_lexicon_tags_15 },
    { 0x64a5fc25, SUC_lexicon_tags_15 },
    { 0xdae5fc25, SUC_lexicon_tags_7 },
    { 0x3f45fc24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7385fc29, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0x4645fc2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc5fc33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3cc5fc35, SUC_lexicon_tags_26 },
    { 0x7585fc36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c5fc3f, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xd0e5fc41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e5fc49, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3845fc4c, SUC_lexicon_tags_14 },
    { 0xf725fc4c, SUC_lexicon_tags_17 },
    { 0x6fe5fc4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4c5fc50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b45fc53, SUC_lexicon_tags_2 },
    { 0xd645fc54, SUC_lexicon_tags_10 },
    { 0x47c5fc55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb05fc5c, SUC_lexicon_tags_0 },
    { 0xa785fc5d, SUC_lexicon_tags_37 },
    { 0x9ea5fc5e, SUC_lexicon_tags_37 },
    { 0x9f65fc5f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8e5fc61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa485fc68, SUC_lexicon_tags_16 },
    { 0x53e5fc69, SUC_lexicon_tags_45 },
    { 0xb3e5fc69, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3085fc6c, SUC_lexicon_tags_1 },
    { 0x9665fc6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe885fc71, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d85fc74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a5fc78, SUC_lexicon_tags_18 },
    { 0xefc5fc79, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5345fc7b, SUC_lexicon_tags_16 },
    { 0xa745fc7c, SUC_lexicon_tags_37 },
    { 0x2905fc7c, SUC_lexicon_tags_24 },
    { 0x12c5fc7e, SUC_lexicon_tags_45 },
    { 0xa1a5fc7b, SUC_lexicon_tags_11 },
    { 0x84e5fc7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fc5fc82, SUC_lexicon_tags_6 },
    { 0x7f45fc83, SUC_lexicon_tags_16 },
    { 0x61a5fc83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c65fc87, SUC_lexicon_tags_11 },
    { 0xf065fc87, SUC_lexicon_tags_27 },
    { 0xda05fc89, SUC_lexicon_tags_65 },
    { 0xfc85fc89, SUC_lexicon_tags_15 },
    { 0xfbc5fc8a, SUC_lexicon_tags_47 },
    { 0x19c5fc8b, SUC_lexicon_tags_17 },
    { 0x8fa5fc8c, SUC_lexicon_tags_61 },
    { 0x5145fc8e, SUC_lexicon_tags_45 },
    { 0x5be5fc8d, SUC_lexicon_tags_37 },
    { 0x705fc8c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf065fc92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de5fc98, SUC_lexicon_tags_2 },
    { 0x62c5fc99, SUC_lexicon_tags_37 },
    { 0xe3a5fc99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82e5fc9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa885fc9e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb845fca3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2285fca6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9865fca8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2805fcab, SUC_lexicon_tags_2 },
    { 0x79c5fcab, SUC_lexicon_tags_17 },
    { 0xa685fcad, SUC_lexicon_tags_55 },
    { 0x9005fcae, SUC_lexicon_tags_27 },
    { 0xee25fcaf, SUC_lexicon_tags_12 },
    { 0x5ae5fcb0, SUC_lexicon_tags_65 },
    { 0x33a5fcb1, SUC_lexicon_tags_43 },
    { 0x9565fcad, SUC_lexicon_tags_1 },
    { 0x3065fcb3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x19a5fcb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a5fcbb, SUC_lexicon_tags_27 },
    { 0x8cc5fcbc, SUC_lexicon_tags_69 },
    { 0x8365fcbd, SUC_lexicon_tags_10 },
    { 0xf465fcbb, SUC_lexicon_tags_25 },
    { 0xc765fcbd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3325fcc1, SUC_lexicon_tags_43 },
    { 0x32a5fcc2, SUC_lexicon_tags_2 },
    { 0xdc45fcc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8985fcc5, SUC_lexicon_tags_1 },
    { 0xe5e5fcc5, SUC_lexicon_tags_19 },
    { 0xa545fcc6, SUC_lexicon_tags_6 },
    { 0xff85fcc7, SUC_lexicon_tags_14 },
    { 0xae25fcc5, SUC_lexicon_tags_15 },
    { 0x2ca5fcc7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e5fccd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf2c5fccf, SUC_lexicon_tags_31 },
    { 0xdbc5fcd0, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba5fcd5, SUC_lexicon_tags_21 },
    { 0xf425fcd6, SUC_lexicon_tags_13 },
    { 0xb1c5fcd5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5525fcdd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7325fcdf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c5fce2, SUC_lexicon_tags_68 },
    { 0x1e85fce2, SUC_lexicon_tags_2 },
    { 0x9385fce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a5fce7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9005fce9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb245fcec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5fcf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc485fcf3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xd845fcf6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b45fcfd, SUC_lexicon_tags_19 },
    { 0x17c5fcfd, SUC_lexicon_tags_47 },
    { 0xa0e5fcff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e5fd08, SUC_lexicon_tags_15 },
    { 0xc65fd09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8985fd0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2e5fd0d, SUC_lexicon_tags_15 },
    { 0x1c45fd0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd85fd13, SUC_lexicon_tags_13 },
    { 0x9285fd14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba65fd16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdae5fd18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c85fd1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb725fd22, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd05fd24, SUC_lexicon_tags_27 },
    { 0x1ec5fd24, SUC_lexicon_tags_69 },
    { 0x8e25fd25, SUC_lexicon_tags_1 },
    { 0xbb05fd27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd845fd2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa25fd2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7365fd32, SUC_lexicon_tags_43 },
    { 0x465fd33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9a5fd35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25fd38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b85fd42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e5fd46, SUC_lexicon_tags_1 },
    { 0x2a05fd46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc565fd4c, SUC_lexicon_tags_9 },
    { 0xe0e5fd4d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c5fd50, SUC_lexicon_tags_11 },
    { 0x9e45fd51, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e5fd58, SUC_lexicon_tags_43 },
    { 0x3f65fd59, SUC_lexicon_tags_43 },
    { 0xc5e5fd5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce5fd68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6145fd6a, SUC_lexicon_tags_293 },
    { 0x6c05fd6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe825fd70, SUC_lexicon_tags_16 },
    { 0x3e5fd70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xada5fd74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee5fd7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1765fd82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4425fd84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf545fd89, SUC_lexicon_tags_37 },
    { 0x1dc5fd89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d05fd8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e05fd8e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc1a5fd90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aa5fd92, SUC_lexicon_tags_17 },
    { 0x4765fd92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x27a5fd95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4a5fd97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd005fd99, SUC_lexicon_tags_12 },
    { 0x7f05fd9a, SUC_lexicon_tags_47 },
    { 0x9645fd9a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1745fd9f, SUC_lexicon_tags_15 },
    { 0x8ba5fda0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e5fda8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x105fdab, SUC_lexicon_tags_32 },
    { 0x6605fdac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c5fdb0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2985fdb2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea5fdb8, SUC_lexicon_tags_32 },
    { 0x9f45fdb9, SUC_lexicon_tags_37 },
    { 0x6ba5fdb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe085fdc1, SUC_lexicon_tags_39 },
    { 0x6e5fdc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b25fdc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76c5fdc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45c5fdc9, SUC_lexicon_tags_14 },
    { 0xb725fdc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd05fdcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc625fdd2, SUC_lexicon_tags_26 },
    { 0x7fc5fdd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb725fdd5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e25fdd7, SUC_lexicon_tags_3 },
    { 0x9785fdd8, SUC_lexicon_tags_55 },
    { 0xf4e5fdd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa45fddf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2845fde3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13c5fde5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff25fde7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8e25fde9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c5fdec, SUC_lexicon_tags_17 },
    { 0x2ce5fdec, SUC_lexicon_tags_49 },
    { 0x3065fdec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe405fdf1, SUC_lexicon_tags_7 },
    { 0x3545fdf2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e5fdf5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a5fdfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4c5fdfd, SUC_lexicon_tags_11 },
    { 0x85a5fdfe, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b25fe05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc245fe0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xae5fe0f, SUC_lexicon_tags_25 },
    { 0xec25fe10, SUC_lexicon_tags_82 },
    { 0x11a5fe11, SUC_lexicon_tags_87 },
    { 0x3f45fe10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb585fe14, SUC_lexicon_tags_19 },
    { 0xb405fe15, SUC_lexicon_tags_2 },
    { 0x6b65fe14, SUC_lexicon_tags_47 },
    { 0xd2a5fe16, SUC_lexicon_tags_29 },
    { 0x51a5fe18, SUC_lexicon_tags_16 },
    { 0x8305fe18, SUC_lexicon_tags_0 },
    { 0x1e85fe19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae5fe1f, SUC_lexicon_tags_60 },
    { 0xf0c5fe1f, SUC_lexicon_tags_34 },
    { 0x145fe20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71c5fe23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25fe27, SUC_lexicon_tags_26 },
    { 0x4445fe27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe505fe2a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e5fe31, SUC_lexicon_tags_6 },
    { 0x92e5fe31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac65fe35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3e5fe37, SUC_lexicon_tags_9 },
    { 0x4985fe38, SUC_lexicon_tags_16 },
    { 0x67e5fe39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf05fe3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1565fe40, SUC_lexicon_tags_1 },
    { 0xd665fe41, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x53c5fe43, SUC_lexicon_tags_11 },
    { 0x4765fe43, SUC_lexicon_tags_14 },
    { 0x74e5fe45, SUC_lexicon_tags_37 },
    { 0xf85fe43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe245fe53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c5fe57, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x3005fe59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff85fe5c, SUC_lexicon_tags_37 },
    { 0xd1a5fe5d, SUC_lexicon_tags_12 },
    { 0x4625fe5e, SUC_lexicon_tags_17 },
    { 0x33a5fe5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c5fe68, SUC_lexicon_tags_35 },
    { 0xeec5fe69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xffe5fe6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2685fe70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf845fe72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42a5fe74, SUC_lexicon_tags_17 },
    { 0xfe25fe75, SUC_lexicon_tags_23 },
    { 0xdae5fe74, SUC_lexicon_tags_38 },
    { 0xc1a5fe74, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae5fe86, SUC_lexicon_tags_11 },
    { 0x4bc5fe87, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a5fe8a, SUC_lexicon_tags_17 },
    { 0x1fc5fe8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc5fe8e, SUC_lexicon_tags_25 },
    { 0xdac5fe8f, SUC_lexicon_tags_31 },
    { 0x2ea5fe90, SUC_lexicon_tags_62 },
    { 0x5545fe91, SUC_lexicon_tags_55 },
    { 0xaac5fe90, SUC_lexicon_tags_27 },
    { 0x72c5fe8e, SUC_lexicon_tags_11 },
    { 0xadc5fe92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a45fe98, SUC_lexicon_tags_2 },
    { 0x6de5fe99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a5fea0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f85fea6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3005feab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d25feae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c5feb2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f85feb6, SUC_lexicon_tags_17 },
    { 0x5dc5feb7, SUC_lexicon_tags_10 },
    { 0xc6c5feb8, SUC_lexicon_tags_0 },
    { 0x9a05feb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf525febb, SUC_lexicon_tags_15 },
    { 0x3d05febc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e45fec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5fec8, SUC_lexicon_tags_13 },
    { 0xee25fec9, SUC_lexicon_tags_15 },
    { 0x6c85feca, SUC_lexicon_tags_14 },
    { 0x2b85fecb, SUC_lexicon_tags_0 },
    { 0x5325feca, SUC_lexicon_tags_9 },
    { 0x4325fecd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6805fecf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e25fed3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9045fed5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe5fedf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd145fee2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c5fee8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1c5feea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8865feef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4065fef1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2ce5fef3, SUC_lexicon_tags_5 },
    { 0xa6a5fef3, SUC_lexicon_tags_11 },
    { 0x7225fef5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f05fef7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb45fef9, SUC_lexicon_tags_37 },
    { 0x5a45fefa, SUC_lexicon_tags_7 },
    { 0x75e5fefb, SUC_lexicon_tags_37 },
    { 0x2ba5fefb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa5ff00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd565ff02, SUC_lexicon_tags_11 },
    { 0x1e25ff03, SUC_lexicon_tags_5 },
    { 0xbea5ff04, SUC_lexicon_tags_25 },
    { 0xd685ff05, SUC_lexicon_tags_14 },
    { 0x4a65ff06, SUC_lexicon_tags_2 },
    { 0x2585ff03, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4445ff0d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe5ff10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c05ff15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe465ff19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c5ff23, SUC_lexicon_tags_55 },
    { 0x9aa5ff24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd05ff2a, SUC_lexicon_tags_29 },
    { 0x6be5ff2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f65ff2d, SUC_lexicon_tags_2 },
    { 0x4d05ff2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e5ff32, SUC_lexicon_tags_47 },
    { 0x9605ff33, SUC_lexicon_tags_45 },
    { 0x8ce5ff34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7525ff36, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa5ff39, SUC_lexicon_tags_26 },
    { 0x2705ff3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8505ff3c, SUC_lexicon_tags_48 },
    { 0x7d45ff3c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcb05ff3f, SUC_lexicon_tags_54 },
    { 0x2405ff3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb645ff42, SUC_lexicon_tags_1 },
    { 0xdf65ff43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9045ff4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8a65ff4c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a5ff50, SUC_lexicon_tags_37 },
    { 0xae25ff51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8e5ff53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb545ff57, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f65ff5b, SUC_lexicon_tags_25 },
    { 0x1505ff5b, SUC_lexicon_tags_45 },
    { 0xf25ff5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3165ff62, SUC_lexicon_tags_11 },
    { 0x2e65ff63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b65ff6c, SUC_lexicon_tags_66 },
    { 0x8965ff6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8525ff72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcba5ff74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a85ff7b, SUC_lexicon_tags_15 },
    { 0xab45ff7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x825ff7f, SUC_lexicon_tags_65 },
    { 0xcfc5ff80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf7c5ff82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a5ff85, SUC_lexicon_tags_15 },
    { 0xff25ff86, SUC_lexicon_tags_37 },
    { 0x6f05ff87, SUC_lexicon_tags_15 },
    { 0x4c65ff85, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3bc5ff8a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa085ff8e, SUC_lexicon_tags_10 },
    { 0xae5ff8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb705ff92, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e5ff98, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa5ff9c, SUC_lexicon_tags_57 },
    { 0xf005ff9d, SUC_lexicon_tags_38 },
    { 0x5625ff9e, SUC_lexicon_tags_17 },
    { 0x5905ff9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f25ffa2, SUC_lexicon_tags_15 },
    { 0xaac5ffa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8725ffa6, SUC_lexicon_tags_70 },
    { 0x5625ffa7, SUC_lexicon_tags_0 },
    { 0x67a5ffa7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96c5ffaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8325ffad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6285ffaf, SUC_lexicon_tags_33 },
    { 0x6b65ffb0, SUC_lexicon_tags_19 },
    { 0xd025ffb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be5ffb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b45ffb9, SUC_lexicon_tags_63 },
    { 0x5a65ffba, SUC_lexicon_tags_33 },
    { 0x97e5ffba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2725ffc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5ffc5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae5ffcb, SUC_lexicon_tags_19 },
    { 0x4ae5ffcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3465ffd0, SUC_lexicon_tags_7 },
    { 0x1945ffd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a5ffd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba5ffd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a5ffdd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea5ffe0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1c5ffe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea5ffe6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b45ffee, SUC_lexicon_tags_55 },
    { 0x8365ffef, SUC_lexicon_tags_22 },
    { 0xa285ffee, SUC_lexicon_tags_11 },
    { 0x39c5fff1, SUC_lexicon_tags_63 },
    { 0x3625fff2, SUC_lexicon_tags_17 },
    { 0xdae5fff3, SUC_lexicon_tags_11 },
    { 0x24c5fff3, SUC_lexicon_tags_1 },
    { 0x85a5fff4, SUC_lexicon_tags_34 },
    { 0xefe5fff3, SUC_lexicon_tags_37 },
    { 0x1985ffef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa5fffb, SUC_lexicon_tags_47 },
    { 0xd445fffc, SUC_lexicon_tags_39 },
    { 0xc245fffd, SUC_lexicon_tags_1 },
    { 0x4e05fffe, SUC_lexicon_tags_60 },
    { 0xaae5ffff, SUC_lexicon_tags_72 },
    { 0x74a5fffd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82660002, SUC_lexicon_tags_65 },
    { 0x72e60003, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa60008, SUC_lexicon_tags_37 },
    { 0x52e60008, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb606000b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd606000e, SUC_lexicon_tags_14 },
    { 0xbac6000f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c60012, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f260018, SUC_lexicon_tags_8 },
    { 0x19e60018, SUC_lexicon_tags_46 },
    { 0x16260019, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad26001e, SUC_lexicon_tags_7 },
    { 0xdba6001e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee660023, SUC_lexicon_tags_33 },
    { 0x48060024, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x76a60026, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c86002b, SUC_lexicon_tags_37 },
    { 0x6002c, SUC_lexicon_tags_63 },
    { 0x5206002b, SUC_lexicon_tags_11 },
    { 0x3066002b, SUC_lexicon_tags_49 },
    { 0xc6c6002f, SUC_lexicon_tags_27 },
    { 0x31e60030, SUC_lexicon_tags_17 },
    { 0x26e60030, SUC_lexicon_tags_2 },
    { 0x4506002b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b660036, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c6003a, SUC_lexicon_tags_17 },
    { 0x80c6003b, SUC_lexicon_tags_37 },
    { 0x8e46003b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2260040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf660049, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf06004b, SUC_lexicon_tags_44 },
    { 0xf446004b, SUC_lexicon_tags_60 },
    { 0x8e06004d, SUC_lexicon_tags_56 },
    { 0x1a26004e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a60052, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca60055, SUC_lexicon_tags_18 },
    { 0xda060056, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e60059, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44060061, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa0460063, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea60067, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c26006f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x84060072, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac60076, SUC_lexicon_tags_8 },
    { 0x79260077, SUC_lexicon_tags_5 },
    { 0xc3260078, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be6007b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24060082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f460084, SUC_lexicon_tags_12 },
    { 0x49a60084, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb346008b, SUC_lexicon_tags_0 },
    { 0xa1a6008c, SUC_lexicon_tags_17 },
    { 0x2f86008d, SUC_lexicon_tags_37 },
    { 0xb9a6008b, SUC_lexicon_tags_17 },
    { 0xa0a6008d, SUC_lexicon_tags_23 },
    { 0x98860090, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e60093, SUC_lexicon_tags_43 },
    { 0x6f060094, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f060099, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4c6009b, SUC_lexicon_tags_14 },
    { 0x6906009b, SUC_lexicon_tags_2 },
    { 0xd106009d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x212600a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76c600a2, SUC_lexicon_tags_42 },
    { 0x87a600a3, SUC_lexicon_tags_65 },
    { 0xc22600a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x870600a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xabe600a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368600ae, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xb8a600b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb9a600b2, SUC_lexicon_tags_19 },
    { 0xb3e600b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x92a600b5, SUC_lexicon_tags_39 },
    { 0x660600b6, SUC_lexicon_tags_25 },
    { 0x1e0600b7, SUC_lexicon_tags_17 },
    { 0xa20600b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c600bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48e600be, SUC_lexicon_tags_15 },
    { 0x5b4600bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512600c8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8600cc, SUC_lexicon_tags_61 },
    { 0x522600cd, SUC_lexicon_tags_3 },
    { 0x35c600ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36600d1, SUC_lexicon_tags_7 },
    { 0xf60600d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48600d7, SUC_lexicon_tags_9 },
    { 0x390600d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0600dc, SUC_lexicon_tags_37 },
    { 0xfb0600dd, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262600e3, SUC_lexicon_tags_38 },
    { 0xf60600e3, SUC_lexicon_tags_12 },
    { 0xa38600e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb80600e7, SUC_lexicon_tags_37 },
    { 0x694600e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e2600ea, SUC_lexicon_tags_37 },
    { 0xb40600ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6600ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce600f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20600f6, SUC_lexicon_tags_19 },
    { 0x11e600f6, SUC_lexicon_tags_38 },
    { 0x1f8600f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x696600fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x332600fe, SUC_lexicon_tags_37 },
    { 0xca6600ff, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8260102, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf060106, SUC_lexicon_tags_29 },
    { 0x44e60107, SUC_lexicon_tags_27 },
    { 0xa3860108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a6010d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf286010f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba060111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e6011f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e60123, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff260126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c60129, SUC_lexicon_tags_0 },
    { 0x81460129, SUC_lexicon_tags_2 },
    { 0x7d06012a, SUC_lexicon_tags_38 },
    { 0x7866012a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5ca6012e, SUC_lexicon_tags_47 },
    { 0xf4e6012f, SUC_lexicon_tags_28 },
    { 0xdc460130, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdac60132, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x99e60134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c6013a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b66013f, SUC_lexicon_tags_19 },
    { 0x23860140, SUC_lexicon_tags_37 },
    { 0x72c60141, SUC_lexicon_tags_43 },
    { 0xaf86013f, SUC_lexicon_tags_38 },
    { 0xa60143, SUC_lexicon_tags_23 },
    { 0x78460140, SUC_lexicon_tags_117 },
    { 0xe3660141, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1026014b, SUC_lexicon_tags_71 },
    { 0xfa26014c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e60154, SUC_lexicon_tags_17 },
    { 0xfe260155, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5060157, SUC_lexicon_tags_15 },
    { 0x9a060158, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedc6015a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86015d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd860160, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c60164, SUC_lexicon_tags_26 },
    { 0x6a060165, SUC_lexicon_tags_12 },
    { 0x81c60166, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe4460168, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e6016b, SUC_lexicon_tags_17 },
    { 0xf2a6016b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a60170, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe460172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf860174, SUC_lexicon_tags_14 },
    { 0xf1c60175, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdce60177, SUC_lexicon_tags_104 },
    { 0xf2c60178, SUC_lexicon_tags_9 },
    { 0xa4e60178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74460181, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x65460184, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf2e60186, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb06018c, SUC_lexicon_tags_33 },
    { 0xa1c6018c, SUC_lexicon_tags_2 },
    { 0xa0c6018d, SUC_lexicon_tags_47 },
    { 0x3286018d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a60193, SUC_lexicon_tags_25 },
    { 0xa60194, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a60197, SUC_lexicon_tags_27 },
    { 0x37c60198, SUC_lexicon_tags_38 },
    { 0x64260199, SUC_lexicon_tags_45 },
    { 0x4f06019a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9246019d, SUC_lexicon_tags_79 },
    { 0x4226019e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a601a1, SUC_lexicon_tags_6 },
    { 0x4e2601a1, SUC_lexicon_tags_17 },
    { 0x102601a1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x662601a5, SUC_lexicon_tags_11 },
    { 0x8d2601a6, SUC_lexicon_tags_12 },
    { 0x972601a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764601ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa3c601ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8601b4, SUC_lexicon_tags_47 },
    { 0x714601b5, SUC_lexicon_tags_21 },
    { 0xc40601b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54601b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0601bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcd6601bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222601c8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x838601ca, SUC_lexicon_tags_0 },
    { 0x604601ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x468601cd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2a6601cf, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a601d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbe601d6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6bc601d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd88601da, SUC_lexicon_tags_15 },
    { 0xb8c601da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x870601dd, SUC_lexicon_tags_37 },
    { 0x316601dd, SUC_lexicon_tags_49 },
    { 0xad4601dd, SUC_lexicon_tags_38 },
    { 0x8fa601e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92601e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458601e7, SUC_lexicon_tags_49 },
    { 0xb7c601e8, SUC_lexicon_tags_11 },
    { 0xf34601e8, SUC_lexicon_tags_37 },
    { 0x6ea601ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdda601ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaf2601ee, SUC_lexicon_tags_9 },
    { 0xcc0601ef, SUC_lexicon_tags_0 },
    { 0x94e601ef, SUC_lexicon_tags_42 },
    { 0x4ee601f1, SUC_lexicon_tags_32 },
    { 0x20c601f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae8601f4, SUC_lexicon_tags_6 },
    { 0xc90601f5, SUC_lexicon_tags_1 },
    { 0x11e601f5, SUC_lexicon_tags_12 },
    { 0xfb2601f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00601fa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe64601fc, SUC_lexicon_tags_234 },
    { 0xe3a601fd, SUC_lexicon_tags_21 },
    { 0x128601fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a60201, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9860204, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57060208, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8606020b, SUC_lexicon_tags_7 },
    { 0x3ec6020b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91460215, SUC_lexicon_tags_7 },
    { 0x8b260215, SUC_lexicon_tags_37 },
    { 0x52260216, SUC_lexicon_tags_15 },
    { 0x58c60218, SUC_lexicon_tags_61 },
    { 0x74060218, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2da6021b, SUC_lexicon_tags_15 },
    { 0x9a26021c, SUC_lexicon_tags_7 },
    { 0x11a6021d, SUC_lexicon_tags_43 },
    { 0x34e6021e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e60221, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d660224, SUC_lexicon_tags_26 },
    { 0x28060225, SUC_lexicon_tags_37 },
    { 0x54860226, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5660229, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e460232, SUC_lexicon_tags_15 },
    { 0xffc60233, SUC_lexicon_tags_63 },
    { 0x38460234, SUC_lexicon_tags_47 },
    { 0x59c60234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa6023d, SUC_lexicon_tags_17 },
    { 0xb3a6023e, SUC_lexicon_tags_7 },
    { 0x28e6023f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e60245, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd5260247, SUC_lexicon_tags_14 },
    { 0x6fc60248, SUC_lexicon_tags_25 },
    { 0xc9460247, SUC_lexicon_tags_7 },
    { 0x5ea6024a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0460250, SUC_lexicon_tags_96 },
    { 0xd3260251, SUC_lexicon_tags_4 },
    { 0x61660251, SUC_lexicon_tags_7 },
    { 0xc7260253, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31660255, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e60258, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6025b, SUC_lexicon_tags_16 },
    { 0xdde6025c, SUC_lexicon_tags_12 },
    { 0x68c6025d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42260260, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd660263, SUC_lexicon_tags_33 },
    { 0x52860263, SUC_lexicon_tags_69 },
    { 0x91a60265, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7786026a, SUC_lexicon_tags_15 },
    { 0x6aa6026a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b06026f, SUC_lexicon_tags_17 },
    { 0xcf060270, SUC_lexicon_tags_25 },
    { 0xdd26026f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6860277, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc06027d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c060280, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f460284, SUC_lexicon_tags_2 },
    { 0x49c60285, SUC_lexicon_tags_2 },
    { 0xa5660285, SUC_lexicon_tags_65 },
    { 0x81660286, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b26028d, SUC_lexicon_tags_23 },
    { 0xad66028d, SUC_lexicon_tags_25 },
    { 0xb446028f, SUC_lexicon_tags_113 },
    { 0xf46028e, SUC_lexicon_tags_8 },
    { 0x99c60291, SUC_lexicon_tags_11 },
    { 0xbfc60292, SUC_lexicon_tags_37 },
    { 0xbb060293, SUC_lexicon_tags_11 },
    { 0xf3460292, SUC_lexicon_tags_60 },
    { 0x9c860294, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x32860297, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca6029c, SUC_lexicon_tags_6 },
    { 0x3ea6029d, SUC_lexicon_tags_45 },
    { 0xf766029e, SUC_lexicon_tags_11 },
    { 0x18c6029c, SUC_lexicon_tags_17 },
    { 0xb48602a0, SUC_lexicon_tags_25 },
    { 0x480602a1, SUC_lexicon_tags_7 },
    { 0x7e0602a2, SUC_lexicon_tags_11 },
    { 0xf7e602a3, SUC_lexicon_tags_82 },
    { 0xf566029d, SUC_lexicon_tags_17 },
    { 0xa5a602a2, SUC_lexicon_tags_17 },
    { 0x724602a6, SUC_lexicon_tags_45 },
    { 0x414602a7, SUC_lexicon_tags_63 },
    { 0x24602a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa602ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d4602ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x664602b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e602b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0602b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2602bb, SUC_lexicon_tags_1 },
    { 0x7f2602bc, SUC_lexicon_tags_17 },
    { 0xb7a602bd, SUC_lexicon_tags_13 },
    { 0x682602be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9e602c0, SUC_lexicon_tags_11 },
    { 0xf88602c1, SUC_lexicon_tags_2 },
    { 0x63e602c2, SUC_lexicon_tags_17 },
    { 0x65a602c2, SUC_lexicon_tags_1 },
    { 0xb10602c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x964602c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e602c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc4602cb, SUC_lexicon_tags_11 },
    { 0xcf6602cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0602d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28602d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796602da, SUC_lexicon_tags_14 },
    { 0x2fc602da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2602de, SUC_lexicon_tags_63 },
    { 0x3b6602df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x802602e1, SUC_lexicon_tags_27 },
    { 0x7d8602e2, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0x758602e5, SUC_lexicon_tags_25 },
    { 0xfc2602e6, SUC_lexicon_tags_56 },
    { 0xf36602e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5da602e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d2602eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06602ee, SUC_lexicon_tags_46 },
    { 0x63a602ef, SUC_lexicon_tags_21 },
    { 0xc62602ee, SUC_lexicon_tags_25 },
    { 0x870602f1, SUC_lexicon_tags_46 },
    { 0x332602f2, SUC_lexicon_tags_25 },
    { 0xd3c602f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ca602f5, SUC_lexicon_tags_25 },
    { 0xd4e602f6, SUC_lexicon_tags_33 },
    { 0x168602f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164602fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e602ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a60304, SUC_lexicon_tags_25 },
    { 0xaf660304, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1660308, SUC_lexicon_tags_29 },
    { 0xd5e60309, SUC_lexicon_tags_17 },
    { 0xcf46030a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a6030e, SUC_lexicon_tags_26 },
    { 0x3da6030e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7660312, SUC_lexicon_tags_2 },
    { 0x24460313, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc60317, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62260319, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6346031b, SUC_lexicon_tags_37 },
    { 0x9606031b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6366031f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c860322, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf860327, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x31c60329, SUC_lexicon_tags_53 },
    { 0xab66032a, SUC_lexicon_tags_17 },
    { 0xcd06032b, SUC_lexicon_tags_0 },
    { 0x8a66032b, SUC_lexicon_tags_0 },
    { 0xeba60329, SUC_lexicon_tags_21 },
    { 0x8206032e, SUC_lexicon_tags_38 },
    { 0x7726032f, SUC_lexicon_tags_26 },
    { 0x10060330, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31460335, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb266033a, SUC_lexicon_tags_17 },
    { 0xc086033b, SUC_lexicon_tags_11 },
    { 0xada6033a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe826033e, SUC_lexicon_tags_49 },
    { 0xd306033f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc06034f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24e60351, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda060359, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89a6035b, SUC_lexicon_tags_12 },
    { 0xa6a6035c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90060364, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc860366, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5fe60368, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa106036a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb26036d, SUC_lexicon_tags_2 },
    { 0xb426036d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0a60370, SUC_lexicon_tags_47 },
    { 0x9da60371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22a60373, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a60376, SUC_lexicon_tags_45 },
    { 0xd3260376, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf4a60379, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd346037c, SUC_lexicon_tags_19 },
    { 0xc346037d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c60384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5826038c, SUC_lexicon_tags_25 },
    { 0xc346038d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2086038f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2260392, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x68260395, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xee860398, SUC_lexicon_tags_37 },
    { 0x15460398, SUC_lexicon_tags_77 },
    { 0xd7260399, SUC_lexicon_tags_6 },
    { 0xffa60398, SUC_lexicon_tags_25 },
    { 0xa8e6039c, SUC_lexicon_tags_15 },
    { 0xb2860398, SUC_lexicon_tags_2 },
    { 0xe826039e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b2603a0, SUC_lexicon_tags_33 },
    { 0xe5c603a1, SUC_lexicon_tags_2 },
    { 0x87e603a2, SUC_lexicon_tags_17 },
    { 0xf74603a2, SUC_lexicon_tags_26 },
    { 0xb30603a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x732603a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4603ab, SUC_lexicon_tags_15 },
    { 0x538603ac, SUC_lexicon_tags_17 },
    { 0x1e2603ac, SUC_lexicon_tags_60 },
    { 0xe6603ae, SUC_lexicon_tags_15 },
    { 0xb2a603af, SUC_lexicon_tags_6 },
    { 0xa3c603b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a603b5, SUC_lexicon_tags_121 },
    { 0x1f8603b6, SUC_lexicon_tags_15 },
    { 0xba0603b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba603ba, SUC_lexicon_tags_7 },
    { 0xcaa603bb, SUC_lexicon_tags_60 },
    { 0x8c4603bc, SUC_lexicon_tags_0 },
    { 0xf62603bd, SUC_lexicon_tags_11 },
    { 0x4ec603be, SUC_lexicon_tags_6 },
    { 0x464603bb, SUC_lexicon_tags_0 },
    { 0x16a603be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4603ce, SUC_lexicon_tags_37 },
    { 0x39a603cf, SUC_lexicon_tags_30 },
    { 0xbae603cf, SUC_lexicon_tags_23 },
    { 0xa4e603d1, SUC_lexicon_tags_65 },
    { 0xda2603d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6603d5, SUC_lexicon_tags_27 },
    { 0x36a603d6, SUC_lexicon_tags_9 },
    { 0x87e603d7, SUC_lexicon_tags_18 },
    { 0x76c603d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x728603db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4603e0, SUC_lexicon_tags_37 },
    { 0x642603e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a603e8, SUC_lexicon_tags_15 },
    { 0x302603e9, SUC_lexicon_tags_23 },
    { 0xb2603ea, SUC_lexicon_tags_27 },
    { 0x66e603ea, SUC_lexicon_tags_4 },
    { 0xb1e603ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x854603ef, SUC_lexicon_tags_61 },
    { 0xc7a603f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e603f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x328603f5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300603fc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a60401, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca60408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac6040c, SUC_lexicon_tags_8 },
    { 0xa366040d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce60410, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1060418, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c66041d, SUC_lexicon_tags_25 },
    { 0x52e6041d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52860421, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3460423, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85060428, SUC_lexicon_tags_12 },
    { 0x9d660429, SUC_lexicon_tags_15 },
    { 0x3886042a, SUC_lexicon_tags_1 },
    { 0xf7c6042b, SUC_lexicon_tags_26 },
    { 0x77c6042c, SUC_lexicon_tags_2 },
    { 0x75c6042d, SUC_lexicon_tags_62 },
    { 0x73e60428, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c60433, SUC_lexicon_tags_19 },
    { 0xc6e60434, SUC_lexicon_tags_0 },
    { 0x50c60433, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ac60437, SUC_lexicon_tags_60 },
    { 0x10a60438, SUC_lexicon_tags_7 },
    { 0xaa60439, SUC_lexicon_tags_15 },
    { 0x6bc6043a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62060440, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x19260442, SUC_lexicon_tags_16 },
    { 0xf8060442, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe2660445, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6044a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dc6044c, SUC_lexicon_tags_37 },
    { 0x6446044d, SUC_lexicon_tags_49 },
    { 0xba26044e, SUC_lexicon_tags_17 },
    { 0xc986044d, SUC_lexicon_tags_31 },
    { 0xaf660450, SUC_lexicon_tags_12 },
    { 0x42660451, SUC_lexicon_tags_61 },
    { 0x43a60451, SUC_lexicon_tags_17 },
    { 0xda46044c, SUC_lexicon_tags_60 },
    { 0x2b060454, SUC_lexicon_tags_60 },
    { 0xa6c60455, SUC_lexicon_tags_1 },
    { 0x2a060454, SUC_lexicon_tags_87 },
    { 0x53e6044d, SUC_lexicon_tags_45 },
    { 0x9b060454, SUC_lexicon_tags_151 },
    { 0x2c660458, SUC_lexicon_tags_2 },
    { 0xe66045a, SUC_lexicon_tags_14 },
    { 0xa886045b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71060460, SUC_lexicon_tags_26 },
    { 0xfe660461, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e60465, SUC_lexicon_tags_49 },
    { 0xff860465, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3ac60468, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66046b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb486046d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac60471, SUC_lexicon_tags_7 },
    { 0x5b260472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20a60474, SUC_lexicon_tags_91 },
    { 0xd6860474, SUC_lexicon_tags_7 },
    { 0xa5e60474, SUC_lexicon_tags_25 },
    { 0x19660475, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a6047a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1146047c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78260484, SUC_lexicon_tags_63 },
    { 0xd1460484, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd586048d, SUC_lexicon_tags_7 },
    { 0xce66048e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b060491, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad460496, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3646049c, SUC_lexicon_tags_12 },
    { 0x56e6049d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e604a0, SUC_lexicon_tags_201 },
    { 0xb70604a0, SUC_lexicon_tags_15 },
    { 0x402604a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x664604a5, SUC_lexicon_tags_14 },
    { 0x298604a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0604ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x184604af, SUC_lexicon_tags_47 },
    { 0xea4604af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48604b3, SUC_lexicon_tags_11 },
    { 0xf78604b4, SUC_lexicon_tags_1 },
    { 0xb74604b3, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8604bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7e604bd, SUC_lexicon_tags_31 },
    { 0xd72604be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e604c4, SUC_lexicon_tags_23 },
    { 0xde8604c4, SUC_lexicon_tags_27 },
    { 0xc68604c6, SUC_lexicon_tags_37 },
    { 0xa80604c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8604d0, SUC_lexicon_tags_70 },
    { 0x334604d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda8604d3, SUC_lexicon_tags_7 },
    { 0x9a2604d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0604d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc90604da, SUC_lexicon_tags_25 },
    { 0x4e6604db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x38604dd, SUC_lexicon_tags_1 },
    { 0x4e604de, SUC_lexicon_tags_15 },
    { 0x1d8604df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e604e5, SUC_lexicon_tags_0 },
    { 0x6ac604e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6604ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85e604ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510604f0, SUC_lexicon_tags_108 },
    { 0xeba604f0, SUC_lexicon_tags_2 },
    { 0x5b8604f2, SUC_lexicon_tags_12 },
    { 0x3c6604f3, SUC_lexicon_tags_8 },
    { 0x5b2604f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756604fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1260500, SUC_lexicon_tags_74 },
    { 0x68860501, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4460505, SUC_lexicon_tags_25 },
    { 0x1ae60506, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a6050a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe726050e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xae60511, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea060515, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe060519, SUC_lexicon_tags_15 },
    { 0x72860519, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab86051e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46a60520, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4460522, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47060524, SUC_lexicon_tags_15 },
    { 0x70060525, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x75e60527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27c60529, SUC_lexicon_tags_25 },
    { 0x7306052a, SUC_lexicon_tags_31 },
    { 0xa5e6052a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50260533, SUC_lexicon_tags_15 },
    { 0x6ba60534, SUC_lexicon_tags_68 },
    { 0x6d460535, SUC_lexicon_tags_37 },
    { 0x8460535, SUC_lexicon_tags_4 },
    { 0xa7e60534, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8346053c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a60540, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d860544, SUC_lexicon_tags_0 },
    { 0xfec60544, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1060548, SUC_lexicon_tags_42 },
    { 0xb6860548, SUC_lexicon_tags_15 },
    { 0x660548, SUC_lexicon_tags_4 },
    { 0x2e06054a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde6054e, SUC_lexicon_tags_19 },
    { 0xeaa6054e, SUC_lexicon_tags_7 },
    { 0x9fa6054e, SUC_lexicon_tags_37 },
    { 0x5c26054f, SUC_lexicon_tags_12 },
    { 0xf8460552, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1860554, SUC_lexicon_tags_35 },
    { 0x1660555, SUC_lexicon_tags_85 },
    { 0x40660556, SUC_lexicon_tags_40 },
    { 0x9a60557, SUC_lexicon_tags_1 },
    { 0x43460557, SUC_lexicon_tags_11 },
    { 0x7a660557, SUC_lexicon_tags_11 },
    { 0xa9060556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca6055e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1860562, SUC_lexicon_tags_45 },
    { 0x4be60563, SUC_lexicon_tags_38 },
    { 0xed060564, SUC_lexicon_tags_1 },
    { 0xc4a60565, SUC_lexicon_tags_34 },
    { 0x22060566, SUC_lexicon_tags_1 },
    { 0x46460567, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca46056b, SUC_lexicon_tags_39 },
    { 0xe786056c, SUC_lexicon_tags_10 },
    { 0xc626056b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c60572, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83c60574, SUC_lexicon_tags_48 },
    { 0x8dc60574, SUC_lexicon_tags_1 },
    { 0xfb460576, SUC_lexicon_tags_26 },
    { 0xe5c60577, SUC_lexicon_tags_19 },
    { 0x7f460575, SUC_lexicon_tags_1 },
    { 0x14260576, SUC_lexicon_tags_17 },
    { 0x1a260575, SUC_lexicon_tags_23 },
    { 0xfce6057b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee6057e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25060581, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe860584, SUC_lexicon_tags_17 },
    { 0xd2a60585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c60589, SUC_lexicon_tags_26 },
    { 0x37a60589, SUC_lexicon_tags_7 },
    { 0x9686058b, SUC_lexicon_tags_23 },
    { 0x83a60589, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b660597, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec86059a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb506059c, SUC_lexicon_tags_138 },
    { 0x9ca6059d, SUC_lexicon_tags_47 },
    { 0xfc66059e, SUC_lexicon_tags_6 },
    { 0x4c26059d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7c605a1, SUC_lexicon_tags_65 },
    { 0x1a6605a2, SUC_lexicon_tags_27 },
    { 0x886605a3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa605a9, SUC_lexicon_tags_17 },
    { 0x120605a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd60605ac, SUC_lexicon_tags_14 },
    { 0x21e605ac, SUC_lexicon_tags_33 },
    { 0xf20605ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72605b6, SUC_lexicon_tags_17 },
    { 0x122605b7, SUC_lexicon_tags_45 },
    { 0x59e605b8, SUC_lexicon_tags_3 },
    { 0x818605b9, SUC_lexicon_tags_15 },
    { 0xb74605b8, SUC_lexicon_tags_0 },
    { 0x668605ba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a605be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6605c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7e605c3, SUC_lexicon_tags_12 },
    { 0xe28605c4, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c605c7, SUC_lexicon_tags_38 },
    { 0x898605c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x164605ca, SUC_lexicon_tags_12 },
    { 0xbaa605cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47c605cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4605d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x548605d3, SUC_lexicon_tags_38 },
    { 0xc02605d4, SUC_lexicon_tags_7 },
    { 0x49a605d5, SUC_lexicon_tags_17 },
    { 0x32c605d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946605e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe605e9, SUC_lexicon_tags_75 },
    { 0x83c605e9, SUC_lexicon_tags_17 },
    { 0xbdc605eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa605ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688605f5, SUC_lexicon_tags_2 },
    { 0x864605f6, SUC_lexicon_tags_11 },
    { 0xb7c605f7, SUC_lexicon_tags_68 },
    { 0x63c605f7, SUC_lexicon_tags_17 },
    { 0x570605f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80660601, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cc60603, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc60606, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9ca60608, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x146060b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb406060f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc60612, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a60616, SUC_lexicon_tags_38 },
    { 0xef660617, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5060619, SUC_lexicon_tags_56 },
    { 0xda860619, SUC_lexicon_tags_37 },
    { 0x65e6061b, SUC_lexicon_tags_65 },
    { 0x5260619, SUC_lexicon_tags_32 },
    { 0x9606061c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa166061f, SUC_lexicon_tags_47 },
    { 0x33460620, SUC_lexicon_tags_1 },
    { 0xb4860621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd660623, SUC_lexicon_tags_12 },
    { 0x6c860624, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37c60626, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61060631, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x57660633, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56260635, SUC_lexicon_tags_25 },
    { 0x6660636, SUC_lexicon_tags_45 },
    { 0x34e60636, SUC_lexicon_tags_11 },
    { 0x45a60636, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc926063d, SUC_lexicon_tags_37 },
    { 0x2066063e, SUC_lexicon_tags_10 },
    { 0xf4c6063f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e60644, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc60648, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a6064b, SUC_lexicon_tags_27 },
    { 0xa606064c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a6064f, SUC_lexicon_tags_11 },
    { 0x9426064f, SUC_lexicon_tags_65 },
    { 0xaa060651, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18460656, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac6065d, SUC_lexicon_tags_17 },
    { 0x8da6065e, SUC_lexicon_tags_8 },
    { 0xd506065f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe60665, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a260669, SUC_lexicon_tags_61 },
    { 0x8da60669, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec86066c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a6066f, SUC_lexicon_tags_37 },
    { 0x4c06066f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68260672, SUC_lexicon_tags_47 },
    { 0xe8260673, SUC_lexicon_tags_30 },
    { 0xeec60674, SUC_lexicon_tags_7 },
    { 0xcee60675, SUC_lexicon_tags_11 },
    { 0xdc460675, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8460679, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7166067c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1dc6067e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0060680, SUC_lexicon_tags_1 },
    { 0xdbe60680, SUC_lexicon_tags_2 },
    { 0x2e60681, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3c60684, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa5260686, SUC_lexicon_tags_15 },
    { 0xf9060687, SUC_lexicon_tags_17 },
    { 0x90860686, SUC_lexicon_tags_10 },
    { 0xab460689, SUC_lexicon_tags_22 },
    { 0xfc26068a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46068d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8060694, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b660697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294606a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58606a8, SUC_lexicon_tags_15 },
    { 0xc8a606a9, SUC_lexicon_tags_14 },
    { 0x412606a9, SUC_lexicon_tags_17 },
    { 0x482606a9, SUC_lexicon_tags_32 },
    { 0x6ce606ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd84606ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4e606b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0606b4, SUC_lexicon_tags_19 },
    { 0x384606b5, SUC_lexicon_tags_17 },
    { 0xa68606b5, SUC_lexicon_tags_15 },
    { 0x18e606b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e606bc, SUC_lexicon_tags_17 },
    { 0xdde606bd, SUC_lexicon_tags_9 },
    { 0x536606bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6606c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec606c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x144606c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722606cc, SUC_lexicon_tags_2 },
    { 0xbc606cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e606d0, SUC_lexicon_tags_0 },
    { 0x32c606d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536606d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a606db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06606e3, SUC_lexicon_tags_2 },
    { 0xb02606e4, SUC_lexicon_tags_7 },
    { 0x3c606e4, SUC_lexicon_tags_25 },
    { 0x7d0606e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c606ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0606f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1ae606f3, SUC_lexicon_tags_951 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc606fd, SUC_lexicon_tags_28 },
    { 0x88e606fd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c60701, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc660708, SUC_lexicon_tags_17 },
    { 0xb3460708, SUC_lexicon_tags_0 },
    { 0x43460708, SUC_lexicon_tags_37 },
    { 0x9706070b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e6070e, SUC_lexicon_tags_12 },
    { 0x3be6070e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7be60711, SUC_lexicon_tags_1 },
    { 0xdb060712, SUC_lexicon_tags_27 },
    { 0xed860712, SUC_lexicon_tags_1 },
    { 0xae60714, SUC_lexicon_tags_63 },
    { 0x28c60714, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x63260718, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed46071e, SUC_lexicon_tags_1 },
    { 0xb826071f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7260726, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda6072e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87660735, SUC_lexicon_tags_1388 },
    { 0xf9260735, SUC_lexicon_tags_65 },
    { 0x45e60737, SUC_lexicon_tags_8 },
    { 0x60260738, SUC_lexicon_tags_37 },
    { 0x85e60739, SUC_lexicon_tags_60 },
    { 0x8ea60737, SUC_lexicon_tags_5 },
    { 0x9a60738, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa660741, SUC_lexicon_tags_15 },
    { 0x58c60742, SUC_lexicon_tags_21 },
    { 0xa7260743, SUC_lexicon_tags_47 },
    { 0x1f460744, SUC_lexicon_tags_49 },
    { 0x2aa60745, SUC_lexicon_tags_47 },
    { 0x30260745, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa6074a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17260753, SUC_lexicon_tags_15 },
    { 0x6a260753, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74860757, SUC_lexicon_tags_15 },
    { 0x75860758, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e66075b, SUC_lexicon_tags_11 },
    { 0xda06075b, SUC_lexicon_tags_4 },
    { 0xe6075b, SUC_lexicon_tags_0 },
    { 0x1a46075b, SUC_lexicon_tags_17 },
    { 0x51e6075f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9660761, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60765, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x79460767, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde6076a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8460777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a6077a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6886077c, SUC_lexicon_tags_2 },
    { 0x6646077d, SUC_lexicon_tags_12 },
    { 0x6206077c, SUC_lexicon_tags_33 },
    { 0xd3e6077f, SUC_lexicon_tags_38 },
    { 0x9dc60780, SUC_lexicon_tags_7 },
    { 0x6dc6077f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e60786, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb260789, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc346078b, SUC_lexicon_tags_38 },
    { 0x48e6078c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c60790, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x99060793, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf6460795, SUC_lexicon_tags_17 },
    { 0x71460795, SUC_lexicon_tags_39 },
    { 0x90260797, SUC_lexicon_tags_27 },
    { 0x4660798, SUC_lexicon_tags_46 },
    { 0x19460799, SUC_lexicon_tags_30 },
    { 0x3c260797, SUC_lexicon_tags_15 },
    { 0x9506079b, SUC_lexicon_tags_37 },
    { 0x3246079c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e6079f, SUC_lexicon_tags_48 },
    { 0x5e26079f, SUC_lexicon_tags_17 },
    { 0x13a607a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x914607a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4607a6, SUC_lexicon_tags_16 },
    { 0x746607a7, SUC_lexicon_tags_14 },
    { 0x6e2607a8, SUC_lexicon_tags_17 },
    { 0x5d6607a8, SUC_lexicon_tags_7 },
    { 0x386607a9, SUC_lexicon_tags_91 },
    { 0x33c607ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb90607ad, SUC_lexicon_tags_11 },
    { 0xd6607ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec607b2, SUC_lexicon_tags_7 },
    { 0xc46607b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfda607b5, SUC_lexicon_tags_8 },
    { 0xf4607b6, SUC_lexicon_tags_17 },
    { 0xd02607b7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a607be, SUC_lexicon_tags_17 },
    { 0xfc6607bf, SUC_lexicon_tags_12 },
    { 0xea607c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x490607c2, SUC_lexicon_tags_16 },
    { 0x9f4607c3, SUC_lexicon_tags_63 },
    { 0x802607c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40e607c6, SUC_lexicon_tags_17 },
    { 0x2da607c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0607cb, SUC_lexicon_tags_2 },
    { 0x230607cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a607d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3a607d6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a607db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e607de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446607e2, SUC_lexicon_tags_301 },
    { 0x882607e2, SUC_lexicon_tags_6 },
    { 0x752607e4, SUC_lexicon_tags_55 },
    { 0x9f8607e4, SUC_lexicon_tags_12 },
    { 0x7d8607e2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfde607e8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc88607ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3c607ec, SUC_lexicon_tags_63 },
    { 0x832607ed, SUC_lexicon_tags_23 },
    { 0x4d0607ee, SUC_lexicon_tags_28 },
    { 0xf62607ee, SUC_lexicon_tags_37 },
    { 0x600607ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6607f3, SUC_lexicon_tags_23 },
    { 0xf34607f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4607f6, SUC_lexicon_tags_60 },
    { 0x3ce607f7, SUC_lexicon_tags_16 },
    { 0xf84607f8, SUC_lexicon_tags_7 },
    { 0xac607f9, SUC_lexicon_tags_2 },
    { 0x3e2607fa, SUC_lexicon_tags_15 },
    { 0x36c607f6, SUC_lexicon_tags_23 },
    { 0xb68607fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x932607fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc60802, SUC_lexicon_tags_12 },
    { 0xe8860803, SUC_lexicon_tags_84 },
    { 0x57460802, SUC_lexicon_tags_22 },
    { 0x98060804, SUC_lexicon_tags_17 },
    { 0x8e860805, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x32c60808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34e6080a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc26080c, SUC_lexicon_tags_68 },
    { 0xee26080d, SUC_lexicon_tags_39 },
    { 0x4666080d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c60811, SUC_lexicon_tags_11 },
    { 0x20260812, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4860814, SUC_lexicon_tags_17 },
    { 0xa1860814, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6081a, SUC_lexicon_tags_31 },
    { 0xb186081b, SUC_lexicon_tags_79 },
    { 0x6d06081b, SUC_lexicon_tags_37 },
    { 0x7226081c, SUC_lexicon_tags_23 },
    { 0x7006081c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe60824, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a6082c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6966082f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66660831, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6d860833, SUC_lexicon_tags_1 },
    { 0xde460834, SUC_lexicon_tags_12 },
    { 0x44660834, SUC_lexicon_tags_29 },
    { 0x68860836, SUC_lexicon_tags_49 },
    { 0x8d060837, SUC_lexicon_tags_17 },
    { 0x8fc60835, SUC_lexicon_tags_25 },
    { 0x20860834, SUC_lexicon_tags_19 },
    { 0x6486083a, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x2e6083c, SUC_lexicon_tags_63 },
    { 0xb3e6083d, SUC_lexicon_tags_4 },
    { 0x2ee6083e, SUC_lexicon_tags_18 },
    { 0xd206083d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9546084a, SUC_lexicon_tags_18 },
    { 0x5ec6084b, SUC_lexicon_tags_37 },
    { 0x7606084b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be60850, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec660854, SUC_lexicon_tags_7 },
    { 0xe1860855, SUC_lexicon_tags_29 },
    { 0xda460854, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e660859, SUC_lexicon_tags_1 },
    { 0xc706085a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a60861, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6260863, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68a60865, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81860869, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8526086d, SUC_lexicon_tags_40 },
    { 0x5226086d, SUC_lexicon_tags_1 },
    { 0xa3c6086f, SUC_lexicon_tags_10 },
    { 0x17e60870, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc60873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dc60875, SUC_lexicon_tags_85 },
    { 0xa5660876, SUC_lexicon_tags_17 },
    { 0x77060876, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a6087a, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xca46087c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46260880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xffc60882, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd1460884, SUC_lexicon_tags_12 },
    { 0x5d060884, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b060887, SUC_lexicon_tags_11 },
    { 0x7e660888, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5186088a, SUC_lexicon_tags_37 },
    { 0x526088b, SUC_lexicon_tags_35 },
    { 0xf686088c, SUC_lexicon_tags_35 },
    { 0x2b66088d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c60891, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4260894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2260897, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa8e60899, SUC_lexicon_tags_25 },
    { 0xb2c6089a, SUC_lexicon_tags_21 },
    { 0x5ee6089b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa566089d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248608a3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de608a7, SUC_lexicon_tags_9 },
    { 0x734608a8, SUC_lexicon_tags_14 },
    { 0x1a0608a8, SUC_lexicon_tags_17 },
    { 0xaf2608aa, SUC_lexicon_tags_2 },
    { 0x746608aa, SUC_lexicon_tags_61 },
    { 0x718608aa, SUC_lexicon_tags_43 },
    { 0x850608ad, SUC_lexicon_tags_12 },
    { 0xe20608ae, SUC_lexicon_tags_37 },
    { 0xc9a608af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a608b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4608c0, SUC_lexicon_tags_12 },
    { 0xc86608c1, SUC_lexicon_tags_25 },
    { 0xbac608c2, SUC_lexicon_tags_1 },
    { 0x23c608c3, SUC_lexicon_tags_38 },
    { 0xeec608c2, SUC_lexicon_tags_98 },
    { 0xcd0608c0, SUC_lexicon_tags_35 },
    { 0xc0e608c6, SUC_lexicon_tags_39 },
    { 0x10e608c2, SUC_lexicon_tags_15 },
    { 0xaea608c8, SUC_lexicon_tags_60 },
    { 0x7b4608c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa608d3, SUC_lexicon_tags_35 },
    { 0x100608d4, SUC_lexicon_tags_2 },
    { 0xd0a608d5, SUC_lexicon_tags_14 },
    { 0x87e608d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fe608d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32608dc, SUC_lexicon_tags_8 },
    { 0x7ee608dd, SUC_lexicon_tags_7 },
    { 0x25e608dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30608e6, SUC_lexicon_tags_92 },
    { 0xcae608e6, SUC_lexicon_tags_45 },
    { 0x46e608e8, SUC_lexicon_tags_48 },
    { 0xb7a608e9, SUC_lexicon_tags_87 },
    { 0x8ae608e8, SUC_lexicon_tags_25 },
    { 0x822608ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c608ef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86608f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac608f5, SUC_lexicon_tags_17 },
    { 0xca6608f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c608f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec608fe, SUC_lexicon_tags_11 },
    { 0x4ea608ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0660905, SUC_lexicon_tags_12 },
    { 0xf060905, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b660908, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1466090e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4860910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f060913, SUC_lexicon_tags_7 },
    { 0x74260913, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f660919, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba86091b, SUC_lexicon_tags_37 },
    { 0xcda6091c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5660920, SUC_lexicon_tags_7 },
    { 0xc3460921, SUC_lexicon_tags_16 },
    { 0x3fc60922, SUC_lexicon_tags_4 },
    { 0x62460923, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0x9d460925, SUC_lexicon_tags_7 },
    { 0xa3c60926, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d860928, SUC_lexicon_tags_33 },
    { 0x6f860928, SUC_lexicon_tags_27 },
    { 0x7c460928, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c60935, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe460937, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c6093b, SUC_lexicon_tags_25 },
    { 0xf4e6093c, SUC_lexicon_tags_12 },
    { 0xdae6093b, SUC_lexicon_tags_11 },
    { 0xd966093d, SUC_lexicon_tags_33 },
    { 0xa1a6093f, SUC_lexicon_tags_11 },
    { 0x1ca6093e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc60942, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a60945, SUC_lexicon_tags_25 },
    { 0x87060946, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa860948, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3126094d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x31460950, SUC_lexicon_tags_22 },
    { 0x7e460950, SUC_lexicon_tags_29 },
    { 0x43c60952, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f660955, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71460957, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9ae60959, SUC_lexicon_tags_61 },
    { 0x51060959, SUC_lexicon_tags_33 },
    { 0xb6460959, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e6095e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a60964, SUC_lexicon_tags_61 },
    { 0xe060965, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa260968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75e6096a, SUC_lexicon_tags_33 },
    { 0xfe66096b, SUC_lexicon_tags_15 },
    { 0xb526096b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1086096e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd460974, SUC_lexicon_tags_6 },
    { 0x7a460975, SUC_lexicon_tags_33 },
    { 0xb6060976, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9aa60978, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf326097c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde86097e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a60983, SUC_lexicon_tags_19 },
    { 0x91c60984, SUC_lexicon_tags_43 },
    { 0xac860983, SUC_lexicon_tags_11 },
    { 0xf4260986, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a60989, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd506098b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf260991, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xca460993, SUC_lexicon_tags_37 },
    { 0x9fa60994, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a6099c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28a6099e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe609a1, SUC_lexicon_tags_29 },
    { 0x7e609a2, SUC_lexicon_tags_30 },
    { 0x8fa609a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a609a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6609ab, SUC_lexicon_tags_19 },
    { 0x4a609ac, SUC_lexicon_tags_61 },
    { 0x9ee609ad, SUC_lexicon_tags_1 },
    { 0x8b0609ac, SUC_lexicon_tags_25 },
    { 0x344609af, SUC_lexicon_tags_37 },
    { 0x7a8609b0, SUC_lexicon_tags_11 },
    { 0x504609b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c609b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8609ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x324609bd, SUC_lexicon_tags_25 },
    { 0x2b4609be, SUC_lexicon_tags_11 },
    { 0x960609be, SUC_lexicon_tags_32 },
    { 0xb16609be, SUC_lexicon_tags_27 },
    { 0x900609bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e609c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0609c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4609cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x766609ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64609d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38609d4, SUC_lexicon_tags_11 },
    { 0x186609d5, SUC_lexicon_tags_27 },
    { 0x3ce609d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2609dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc52609de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde609e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2609e5, SUC_lexicon_tags_2 },
    { 0x1ea609e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90609ea, SUC_lexicon_tags_0 },
    { 0x442609eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c609ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b2609f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x194609f3, SUC_lexicon_tags_37 },
    { 0xc18609f4, SUC_lexicon_tags_25 },
    { 0xd9a609f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba8609f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8609fa, SUC_lexicon_tags_45 },
    { 0xffc609fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc609fe, SUC_lexicon_tags_39 },
    { 0x634609fe, SUC_lexicon_tags_70 },
    { 0x2f660a00, SUC_lexicon_tags_55 },
    { 0xa1660a00, SUC_lexicon_tags_37 },
    { 0x19c60a02, SUC_lexicon_tags_38 },
    { 0x5e060a03, SUC_lexicon_tags_61 },
    { 0x72060a01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4260a09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25660a0b, SUC_lexicon_tags_4 },
    { 0x1e460a0c, SUC_lexicon_tags_2 },
    { 0x4460a0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d860a10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a460a17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c60a1b, SUC_lexicon_tags_68 },
    { 0xa1e60a1c, SUC_lexicon_tags_12 },
    { 0x12a60a1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38860a21, SUC_lexicon_tags_17 },
    { 0x4d260a22, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee60a27, SUC_lexicon_tags_7 },
    { 0x76460a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42060a2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c260a2f, SUC_lexicon_tags_37 },
    { 0x9ac60a30, SUC_lexicon_tags_7 },
    { 0xf2c60a31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b660a34, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c460a37, SUC_lexicon_tags_12 },
    { 0xe5860a38, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73260a44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44860a46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2060a4d, SUC_lexicon_tags_16 },
    { 0x2c660a4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4460a51, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1660a55, SUC_lexicon_tags_60 },
    { 0x53660a56, SUC_lexicon_tags_11 },
    { 0x2b660a57, SUC_lexicon_tags_37 },
    { 0x72860a55, SUC_lexicon_tags_15 },
    { 0xb5e60a59, SUC_lexicon_tags_63 },
    { 0x6460a5a, SUC_lexicon_tags_15 },
    { 0x7fa60a55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5060a5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f060a5f, SUC_lexicon_tags_33 },
    { 0xa8e60a60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9fa60a62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a860a65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2660a67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e60a6c, SUC_lexicon_tags_26 },
    { 0xf1260a6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6260a78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba260a7f, SUC_lexicon_tags_15 },
    { 0xafe60a80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8260a82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdec60a85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x75860a87, SUC_lexicon_tags_49 },
    { 0xa8a60a87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x34260a8b, SUC_lexicon_tags_25 },
    { 0x46c60a8b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda460a94, SUC_lexicon_tags_11 },
    { 0xa9860a94, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xff060a97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e860a9b, SUC_lexicon_tags_37 },
    { 0x3060a9c, SUC_lexicon_tags_32 },
    { 0x4da60a9d, SUC_lexicon_tags_1 },
    { 0xc1e60a9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e60aa3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ce60aa5, SUC_lexicon_tags_65 },
    { 0x62c60aa5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a60aa9, SUC_lexicon_tags_37 },
    { 0x7ac60aaa, SUC_lexicon_tags_1 },
    { 0xbfe60aab, SUC_lexicon_tags_60 },
    { 0x660aab, SUC_lexicon_tags_19 },
    { 0x1da60aaa, SUC_lexicon_tags_104 },
    { 0xec060aaa, SUC_lexicon_tags_37 },
    { 0xc8e60aac, SUC_lexicon_tags_27 },
    { 0x38660aae, SUC_lexicon_tags_32 },
    { 0x45c60ab1, SUC_lexicon_tags_26 },
    { 0xef460aa9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddc60ab4, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x5d460ab6, SUC_lexicon_tags_15 },
    { 0x51060ab6, SUC_lexicon_tags_1 },
    { 0x7ce60ab8, SUC_lexicon_tags_25 },
    { 0x28260ab9, SUC_lexicon_tags_37 },
    { 0xf0460ab8, SUC_lexicon_tags_2 },
    { 0xf7060abb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f660abe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82c60ac0, SUC_lexicon_tags_14 },
    { 0x93a60ac1, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a60ac5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdda60ac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c060aca, SUC_lexicon_tags_11 },
    { 0xdec60aca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de60acf, SUC_lexicon_tags_37 },
    { 0xbc660ad0, SUC_lexicon_tags_1 },
    { 0x74260ad1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x23260ad3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe60ad9, SUC_lexicon_tags_7 },
    { 0x8e060ad9, SUC_lexicon_tags_26 },
    { 0x47260adb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf660adf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd660ae2, SUC_lexicon_tags_7 },
    { 0x1260ae3, SUC_lexicon_tags_31 },
    { 0x12660ae4, SUC_lexicon_tags_16 },
    { 0x6c660ae2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75460ae9, SUC_lexicon_tags_12 },
    { 0x95460ae9, SUC_lexicon_tags_7 },
    { 0x2f060aeb, SUC_lexicon_tags_60 },
    { 0xd3460aea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11660af3, SUC_lexicon_tags_60 },
    { 0x83860af4, SUC_lexicon_tags_33 },
    { 0xe3260af5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e60afa, SUC_lexicon_tags_1 },
    { 0x5a860afb, SUC_lexicon_tags_29 },
    { 0x37660afb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e60aff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc60b02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc60b05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8a60b07, SUC_lexicon_tags_14 },
    { 0x36860b07, SUC_lexicon_tags_28 },
    { 0x20460b09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae60b0d, SUC_lexicon_tags_15 },
    { 0x69860b0e, SUC_lexicon_tags_5 },
    { 0x4c660b0f, SUC_lexicon_tags_55 },
    { 0x6f660b0d, SUC_lexicon_tags_1 },
    { 0x43460b11, SUC_lexicon_tags_37 },
    { 0x71c60b10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97660b16, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94860b1a, SUC_lexicon_tags_60 },
    { 0x99060b1b, SUC_lexicon_tags_38 },
    { 0x9d460b1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba060b1e, SUC_lexicon_tags_29 },
    { 0x7a60b1f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xe460b21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68460b23, SUC_lexicon_tags_2 },
    { 0x7e60b24, SUC_lexicon_tags_0 },
    { 0xcc060b25, SUC_lexicon_tags_770 },
    { 0, NULL },
    { 0xb2460b27, SUC_lexicon_tags_23 },
    { 0xd7460b27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a60b2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xade60b2e, SUC_lexicon_tags_0 },
    { 0xeb860b2f, SUC_lexicon_tags_12 },
    { 0x52860b30, SUC_lexicon_tags_37 },
    { 0xf9c60b31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89860b36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd860b3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9c860b3f, SUC_lexicon_tags_23 },
    { 0xd5260b40, SUC_lexicon_tags_37 },
    { 0xa7860b40, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35660b4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41060b51, SUC_lexicon_tags_23 },
    { 0x1fa60b52, SUC_lexicon_tags_45 },
    { 0x42860b51, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc60b55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a460b57, SUC_lexicon_tags_46 },
    { 0x34460b57, SUC_lexicon_tags_17 },
    { 0xdee60b57, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ce60b5b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa2e60b5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd860b60, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a60b65, SUC_lexicon_tags_26 },
    { 0xaa860b65, SUC_lexicon_tags_11 },
    { 0x6d460b65, SUC_lexicon_tags_37 },
    { 0x99c60b68, SUC_lexicon_tags_25 },
    { 0x20260b69, SUC_lexicon_tags_27 },
    { 0x79860b6a, SUC_lexicon_tags_3 },
    { 0xa3a60b6a, SUC_lexicon_tags_30 },
    { 0x95e60b6a, SUC_lexicon_tags_37 },
    { 0x9dc60b66, SUC_lexicon_tags_2 },
    { 0x4d260b67, SUC_lexicon_tags_47 },
    { 0xe5e60b6f, SUC_lexicon_tags_37 },
    { 0xad860b6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x50260b73, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a60b7c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e60b83, SUC_lexicon_tags_16 },
    { 0x32c60b83, SUC_lexicon_tags_12 },
    { 0x60e60b83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90260b90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7260b95, SUC_lexicon_tags_25 },
    { 0xd9860b95, SUC_lexicon_tags_45 },
    { 0xa8a60b96, SUC_lexicon_tags_30 },
    { 0x77860b97, SUC_lexicon_tags_17 },
    { 0x71e60b98, SUC_lexicon_tags_634 },
    { 0xf1060b9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d660b9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba60baa, SUC_lexicon_tags_8 },
    { 0xdbc60bab, SUC_lexicon_tags_44 },
    { 0x16260baa, SUC_lexicon_tags_10 },
    { 0xc6a60bad, SUC_lexicon_tags_12 },
    { 0x92e60baa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6060bb1, SUC_lexicon_tags_7 },
    { 0xd9e60bb2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28260bb7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c60bbf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e60bc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63460bc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1460bce, SUC_lexicon_tags_1 },
    { 0xcd460bcf, SUC_lexicon_tags_66 },
    { 0xe5060bd0, SUC_lexicon_tags_27 },
    { 0x17860bd1, SUC_lexicon_tags_11 },
    { 0x7e060bcf, SUC_lexicon_tags_38 },
    { 0x17c60bcf, SUC_lexicon_tags_49 },
    { 0x86e60bd4, SUC_lexicon_tags_17 },
    { 0xdde60bce, SUC_lexicon_tags_6 },
    { 0x9d660bcf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3660bdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8ae60bde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e460be4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xba660be6, SUC_lexicon_tags_13 },
    { 0xd1260be7, SUC_lexicon_tags_38 },
    { 0x12660be8, SUC_lexicon_tags_0 },
    { 0x42260be9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1660bec, SUC_lexicon_tags_14 },
    { 0x1b860bed, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57860bf3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5860bf6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4f060bf8, SUC_lexicon_tags_27 },
    { 0x8860bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47860bfe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a60c02, SUC_lexicon_tags_23 },
    { 0xe4e60c02, SUC_lexicon_tags_47 },
    { 0xa5860c04, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf660c08, SUC_lexicon_tags_17 },
    { 0x26660c08, SUC_lexicon_tags_84 },
    { 0xb1c60c0a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2660c13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99a60c15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be60c1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76260c20, SUC_lexicon_tags_1 },
    { 0x9fa60c21, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52660c25, SUC_lexicon_tags_27 },
    { 0x7ee60c26, SUC_lexicon_tags_15 },
    { 0x4be60c27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e60c2b, SUC_lexicon_tags_37 },
    { 0x9a060c2c, SUC_lexicon_tags_26 },
    { 0x85460c2d, SUC_lexicon_tags_23 },
    { 0x71260c2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3660c35, SUC_lexicon_tags_2 },
    { 0x4b660c36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e260c38, SUC_lexicon_tags_15 },
    { 0xdc860c39, SUC_lexicon_tags_68 },
    { 0x5a260c39, SUC_lexicon_tags_19 },
    { 0x16e60c39, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xde660c3d, SUC_lexicon_tags_34 },
    { 0x6c460c3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8c060c40, SUC_lexicon_tags_54 },
    { 0xbb860c41, SUC_lexicon_tags_7 },
    { 0x9e460c42, SUC_lexicon_tags_25 },
    { 0xd3e60c42, SUC_lexicon_tags_1 },
    { 0xb0660c44, SUC_lexicon_tags_12 },
    { 0xe60c44, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x5ce60c47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c460c4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be60c55, SUC_lexicon_tags_19 },
    { 0xea460c56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63660c65, SUC_lexicon_tags_7 },
    { 0xd1e60c65, SUC_lexicon_tags_72 },
    { 0x1d460c67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c60c6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a60c6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4c60c70, SUC_lexicon_tags_1 },
    { 0xa4460c71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81260c78, SUC_lexicon_tags_27 },
    { 0xcea60c78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda060c7d, SUC_lexicon_tags_25 },
    { 0x85460c7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb5660c80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34460c84, SUC_lexicon_tags_15 },
    { 0xd7c60c85, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1460c88, SUC_lexicon_tags_7 },
    { 0x7e060c89, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89660c8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1c60c8f, SUC_lexicon_tags_25 },
    { 0x88260c90, SUC_lexicon_tags_105 },
    { 0xc8860c90, SUC_lexicon_tags_47 },
    { 0x94060c92, SUC_lexicon_tags_10 },
    { 0xd5a60c91, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba60c99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a260c9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b860c9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9060ca1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x94860ca4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33860caa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f860cac, SUC_lexicon_tags_33 },
    { 0xea460cad, SUC_lexicon_tags_16 },
    { 0x52460cae, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61060cb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9860cb9, SUC_lexicon_tags_61 },
    { 0x28a60cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92860cbd, SUC_lexicon_tags_10 },
    { 0xe1860cbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc60cc2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x86a60cc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20060cc6, SUC_lexicon_tags_14 },
    { 0xc1660cc7, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8460cdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a460cde, SUC_lexicon_tags_19 },
    { 0x16460cdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ae60ce1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c860ce6, SUC_lexicon_tags_12 },
    { 0x86a60ce7, SUC_lexicon_tags_1407 },
    { 0x70860ce7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x22060ceb, SUC_lexicon_tags_37 },
    { 0xb8260cec, SUC_lexicon_tags_7 },
    { 0xc9060ceb, SUC_lexicon_tags_15 },
    { 0xc8260cee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9260cf1, SUC_lexicon_tags_32 },
    { 0x86c60cf2, SUC_lexicon_tags_2 },
    { 0x39e60cf3, SUC_lexicon_tags_29 },
    { 0xb5060cf4, SUC_lexicon_tags_14 },
    { 0xcf660cf5, SUC_lexicon_tags_37 },
    { 0x92c60cf6, SUC_lexicon_tags_11 },
    { 0x71e60cf6, SUC_lexicon_tags_46 },
    { 0x37260cf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64a60cfa, SUC_lexicon_tags_3 },
    { 0xfee60cfb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc60cfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa60d00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a860d04, SUC_lexicon_tags_7 },
    { 0x5c260d04, SUC_lexicon_tags_17 },
    { 0xeba60d04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9460d0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6260d0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60260d11, SUC_lexicon_tags_63 },
    { 0x14660d12, SUC_lexicon_tags_11 },
    { 0x25a60d13, SUC_lexicon_tags_2 },
    { 0xf4660d14, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c60d18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xff260d1a, SUC_lexicon_tags_27 },
    { 0x8dc60d1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e60d1e, SUC_lexicon_tags_4 },
    { 0x20060d1e, SUC_lexicon_tags_15 },
    { 0x97060d20, SUC_lexicon_tags_68 },
    { 0x6a60d1e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x26260d23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb0460d25, SUC_lexicon_tags_49 },
    { 0xae660d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3260d29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee60d2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c60d2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50e60d31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36260d33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb060d36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8260d38, SUC_lexicon_tags_7 },
    { 0xf4e60d38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2260d3b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3660d41, SUC_lexicon_tags_6 },
    { 0x3a060d42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3660d46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a60d4a, SUC_lexicon_tags_12 },
    { 0x71c60d4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba60d50, SUC_lexicon_tags_35 },
    { 0x8d060d50, SUC_lexicon_tags_37 },
    { 0xd6660d52, SUC_lexicon_tags_37 },
    { 0xf2660d53, SUC_lexicon_tags_15 },
    { 0x75060d54, SUC_lexicon_tags_3 },
    { 0x42c60d54, SUC_lexicon_tags_2 },
    { 0x66660d55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73060d58, SUC_lexicon_tags_23 },
    { 0x34060d59, SUC_lexicon_tags_63 },
    { 0x80a60d5a, SUC_lexicon_tags_38 },
    { 0xf0260d5b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5fa60d5d, SUC_lexicon_tags_30 },
    { 0xc6860d5e, SUC_lexicon_tags_25 },
    { 0x3f460d5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e060d62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36660d6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13a60d6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba60d6f, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0xbc260d71, SUC_lexicon_tags_17 },
    { 0x77460d72, SUC_lexicon_tags_39 },
    { 0x5b660d73, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea60d79, SUC_lexicon_tags_68 },
    { 0x7a660d79, SUC_lexicon_tags_38 },
    { 0xb5a60d7a, SUC_lexicon_tags_0 },
    { 0x21660d79, SUC_lexicon_tags_11 },
    { 0xfb460d7d, SUC_lexicon_tags_37 },
    { 0x77e60d7e, SUC_lexicon_tags_49 },
    { 0xa9060d7f, SUC_lexicon_tags_47 },
    { 0x7260d80, SUC_lexicon_tags_30 },
    { 0x92460d7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x95660d84, SUC_lexicon_tags_108 },
    { 0x8fe60d85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50660d8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95460d8c, SUC_lexicon_tags_17 },
    { 0x4b860d8d, SUC_lexicon_tags_1092 },
    { 0xc9c60d8e, SUC_lexicon_tags_220 },
    { 0x1fa60d8e, SUC_lexicon_tags_1340 },
    { 0x6460d90, SUC_lexicon_tags_37 },
    { 0x9ac60d8e, SUC_lexicon_tags_1 },
    { 0xc0e60d92, SUC_lexicon_tags_25 },
    { 0x85c60d8d, SUC_lexicon_tags_47 },
    { 0xe6a60d8f, SUC_lexicon_tags_65 },
    { 0x31460d8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c60d97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec60d9b, SUC_lexicon_tags_12 },
    { 0x5b660d9b, SUC_lexicon_tags_14 },
    { 0x49460d9d, SUC_lexicon_tags_37 },
    { 0x36e60d9c, SUC_lexicon_tags_11 },
    { 0xd8060d9f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x57e60da1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c60da7, SUC_lexicon_tags_45 },
    { 0xab860da8, SUC_lexicon_tags_26 },
    { 0x70a60da8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9460dab, SUC_lexicon_tags_68 },
    { 0x74060dab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad860dae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21660db3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b260dba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea860dbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a60dc0, SUC_lexicon_tags_32 },
    { 0xcdc60dc1, SUC_lexicon_tags_12 },
    { 0xa9860dc1, SUC_lexicon_tags_7 },
    { 0x49460dc3, SUC_lexicon_tags_37 },
    { 0xba660dc3, SUC_lexicon_tags_37 },
    { 0x38860dc5, SUC_lexicon_tags_17 },
    { 0x79060dc4, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x2d260dc8, SUC_lexicon_tags_1 },
    { 0xff460dc9, SUC_lexicon_tags_30 },
    { 0x89260dca, SUC_lexicon_tags_12 },
    { 0x1ba60dca, SUC_lexicon_tags_43 },
    { 0x8a860dcc, SUC_lexicon_tags_83 },
    { 0xb2a60dca, SUC_lexicon_tags_38 },
    { 0x24a60dc9, SUC_lexicon_tags_0 },
    { 0x48460dcc, SUC_lexicon_tags_31 },
    { 0xf4860dcf, SUC_lexicon_tags_2 },
    { 0x48c60dd1, SUC_lexicon_tags_37 },
    { 0x79c60dca, SUC_lexicon_tags_0 },
    { 0x29c60dd3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfe060dd5, SUC_lexicon_tags_43 },
    { 0xd4860dd6, SUC_lexicon_tags_33 },
    { 0x90660dd6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c60de2, SUC_lexicon_tags_0 },
    { 0x52c60de3, SUC_lexicon_tags_25 },
    { 0xb2e60de3, SUC_lexicon_tags_37 },
    { 0xf5860de2, SUC_lexicon_tags_37 },
    { 0x38e60de6, SUC_lexicon_tags_7 },
    { 0x98060de4, SUC_lexicon_tags_47 },
    { 0x17860de2, SUC_lexicon_tags_63 },
    { 0x2ac60de9, SUC_lexicon_tags_15 },
    { 0xde860dea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9460def, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80e60df1, SUC_lexicon_tags_8 },
    { 0xa4a60df1, SUC_lexicon_tags_23 },
    { 0xfa060df3, SUC_lexicon_tags_17 },
    { 0x48c60df4, SUC_lexicon_tags_19 },
    { 0x12860df4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae060df7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e60dfc, SUC_lexicon_tags_65 },
    { 0xd660dfc, SUC_lexicon_tags_7 },
    { 0x8ac60dfe, SUC_lexicon_tags_11 },
    { 0x60660dff, SUC_lexicon_tags_43 },
    { 0x77660e00, SUC_lexicon_tags_31 },
    { 0x7a260e01, SUC_lexicon_tags_26 },
    { 0x7f660e02, SUC_lexicon_tags_11 },
    { 0x7c860e00, SUC_lexicon_tags_2 },
    { 0x96660e03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad660e06, SUC_lexicon_tags_9 },
    { 0x6f460e07, SUC_lexicon_tags_25 },
    { 0xd8a60e06, SUC_lexicon_tags_70 },
    { 0x7c660e08, SUC_lexicon_tags_48 },
    { 0xfa660e0a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x76660e0c, SUC_lexicon_tags_48 },
    { 0xe9a60e0d, SUC_lexicon_tags_28 },
    { 0x40660e0e, SUC_lexicon_tags_126 },
    { 0x50460e0e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b060e16, SUC_lexicon_tags_1 },
    { 0xd3660e16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9260e1a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x52060e1c, SUC_lexicon_tags_10 },
    { 0x9d060e1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e60e22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a60e27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9860e29, SUC_lexicon_tags_47 },
    { 0xa7460e2a, SUC_lexicon_tags_28 },
    { 0x54460e2b, SUC_lexicon_tags_12 },
    { 0x44e60e2a, SUC_lexicon_tags_11 },
    { 0x97660e2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50060e35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86660e39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc060e3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3260e3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13260e41, SUC_lexicon_tags_43 },
    { 0x73660e42, SUC_lexicon_tags_17 },
    { 0x74060e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35860e47, SUC_lexicon_tags_27 },
    { 0x5660e48, SUC_lexicon_tags_106 },
    { 0x1dc60e47, SUC_lexicon_tags_66 },
    { 0x7b860e49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64c60e4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be60e4f, SUC_lexicon_tags_37 },
    { 0x61460e50, SUC_lexicon_tags_44 },
    { 0xf3460e51, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e60e54, SUC_lexicon_tags_12 },
    { 0xaec60e55, SUC_lexicon_tags_25 },
    { 0xc9a60e55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a60e5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8260e5f, SUC_lexicon_tags_47 },
    { 0xe2460e5f, SUC_lexicon_tags_15 },
    { 0x6b860e61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ba60e63, SUC_lexicon_tags_25 },
    { 0x95660e64, SUC_lexicon_tags_34 },
    { 0xb7e60e65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8860e6c, SUC_lexicon_tags_15 },
    { 0x47c60e6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56060e73, SUC_lexicon_tags_15 },
    { 0x260e74, SUC_lexicon_tags_4 },
    { 0x6ea60e75, SUC_lexicon_tags_55 },
    { 0x52460e76, SUC_lexicon_tags_26 },
    { 0x9f660e77, SUC_lexicon_tags_37 },
    { 0x10e60e78, SUC_lexicon_tags_11 },
    { 0x42660e77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e60e7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d460e85, SUC_lexicon_tags_37 },
    { 0x12860e85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be60e8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47260e91, SUC_lexicon_tags_65 },
    { 0xc4460e91, SUC_lexicon_tags_38 },
    { 0x67c60e93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e060e96, SUC_lexicon_tags_1 },
    { 0x6fc60e97, SUC_lexicon_tags_8 },
    { 0xc2a60e97, SUC_lexicon_tags_25 },
    { 0x7dc60e97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b060e9d, SUC_lexicon_tags_2 },
    { 0x33e60e9e, SUC_lexicon_tags_12 },
    { 0x27860e9f, SUC_lexicon_tags_65 },
    { 0x42c60ea0, SUC_lexicon_tags_11 },
    { 0xa4660e9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e60ea4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a60ea9, SUC_lexicon_tags_61 },
    { 0xf1660eaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4060eac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb860eaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca60eb2, SUC_lexicon_tags_15 },
    { 0x20860eb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34260eb7, SUC_lexicon_tags_27 },
    { 0x81260eb7, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x21060eba, SUC_lexicon_tags_27 },
    { 0x70c60ebb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a60ebf, SUC_lexicon_tags_27 },
    { 0x35260ec0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd3e60ec2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d260ec5, SUC_lexicon_tags_27 },
    { 0xe7660ec6, SUC_lexicon_tags_26 },
    { 0x85c60ec6, SUC_lexicon_tags_15 },
    { 0x19860ec8, SUC_lexicon_tags_56 },
    { 0x22460ec6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc660ed3, SUC_lexicon_tags_60 },
    { 0xf4c60ed3, SUC_lexicon_tags_27 },
    { 0xe2460ed4, SUC_lexicon_tags_7 },
    { 0x58e60ed4, SUC_lexicon_tags_9 },
    { 0xec660ed7, SUC_lexicon_tags_13 },
    { 0x62660ed8, SUC_lexicon_tags_124 },
    { 0x44e60ed9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e60edd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8260ee5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11660ee9, SUC_lexicon_tags_37 },
    { 0xc460eea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19260eee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c460ef0, SUC_lexicon_tags_32 },
    { 0xbd460ef0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x2e860ef3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x96e60ef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea260efa, SUC_lexicon_tags_11 },
    { 0x4da60efb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd460f0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6060f0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a60f13, SUC_lexicon_tags_66 },
    { 0xc0a60f13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66460f1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf860f1e, SUC_lexicon_tags_17 },
    { 0x32e60f1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd060f24, SUC_lexicon_tags_37 },
    { 0xc0c60f25, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f860f29, SUC_lexicon_tags_55 },
    { 0x61860f2a, SUC_lexicon_tags_26 },
    { 0xce660f2a, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c60f33, SUC_lexicon_tags_22 },
    { 0x49e60f34, SUC_lexicon_tags_14 },
    { 0x8ca60f34, SUC_lexicon_tags_56 },
    { 0xf660f35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xad060f39, SUC_lexicon_tags_11 },
    { 0xd8e60f3a, SUC_lexicon_tags_65 },
    { 0xb3660f3a, SUC_lexicon_tags_11 },
    { 0x6da60f3a, SUC_lexicon_tags_11 },
    { 0x15a60f3d, SUC_lexicon_tags_12 },
    { 0x78c60f3b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd660f41, SUC_lexicon_tags_7 },
    { 0x9d260f42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89e60f44, SUC_lexicon_tags_14 },
    { 0xda460f45, SUC_lexicon_tags_19 },
    { 0xbd460f45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39260f4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca860f53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbc460f55, SUC_lexicon_tags_38 },
    { 0xc1a60f56, SUC_lexicon_tags_12 },
    { 0x28a60f56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e260f59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2860f5d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad660f62, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed060f68, SUC_lexicon_tags_6 },
    { 0xbb660f69, SUC_lexicon_tags_12 },
    { 0x54060f6a, SUC_lexicon_tags_7 },
    { 0x46860f6b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xfc660f6d, SUC_lexicon_tags_7 },
    { 0x29860f6d, SUC_lexicon_tags_2 },
    { 0x21660f6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a60f73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a60f78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5060f7a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92460f7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc460f7f, SUC_lexicon_tags_12 },
    { 0x57260f7f, SUC_lexicon_tags_7 },
    { 0x2de60f7f, SUC_lexicon_tags_25 },
    { 0x61e60f81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12860f84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e60f87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b260f8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x36460f8d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19060f99, SUC_lexicon_tags_0 },
    { 0xb0060f9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3860f9c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xea660f9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de60fa2, SUC_lexicon_tags_13 },
    { 0x61660fa3, SUC_lexicon_tags_6 },
    { 0x8be60fa4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8de60fa6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd7460fa8, SUC_lexicon_tags_27 },
    { 0x60c60fa9, SUC_lexicon_tags_17 },
    { 0x6de60faa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x89460fac, SUC_lexicon_tags_15 },
    { 0xd7860fac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x22460fb0, SUC_lexicon_tags_31 },
    { 0x3060fb1, SUC_lexicon_tags_25 },
    { 0x9f260fb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8060fb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a060fb7, SUC_lexicon_tags_34 },
    { 0x46a60fb8, SUC_lexicon_tags_31 },
    { 0xfc460fb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5f460fbb, SUC_lexicon_tags_31 },
    { 0x79c60fbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b460fc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6a60fc2, SUC_lexicon_tags_8 },
    { 0x42860fc3, SUC_lexicon_tags_63 },
    { 0x3d060fc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40660fc7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96860fcf, SUC_lexicon_tags_47 },
    { 0xa4c60fcf, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1260fd4, SUC_lexicon_tags_1 },
    { 0x4060fd5, SUC_lexicon_tags_343 },
    { 0x52c60fd5, SUC_lexicon_tags_19 },
    { 0xfae60fd6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c60fda, SUC_lexicon_tags_7 },
    { 0x8e260fdb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x89860fdd, SUC_lexicon_tags_29 },
    { 0x3c60fde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c460fe6, SUC_lexicon_tags_9 },
    { 0x5a660fe7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2260fef, SUC_lexicon_tags_1 },
    { 0xedc60ff0, SUC_lexicon_tags_9 },
    { 0x64460fef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb260ff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc61000, SUC_lexicon_tags_7 },
    { 0x61e61001, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e6100b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a6100e, SUC_lexicon_tags_68 },
    { 0x9d46100f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44461015, SUC_lexicon_tags_607 },
    { 0, NULL },
    { 0x7dc61017, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9c61019, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc06101c, SUC_lexicon_tags_7 },
    { 0xaf66101d, SUC_lexicon_tags_44 },
    { 0x9046101e, SUC_lexicon_tags_37 },
    { 0x2be6101f, SUC_lexicon_tags_17 },
    { 0x15a61020, SUC_lexicon_tags_12 },
    { 0x4d66101c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c461023, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0061027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7666102b, SUC_lexicon_tags_178 },
    { 0xcd26102c, SUC_lexicon_tags_27 },
    { 0xf166102d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c61035, SUC_lexicon_tags_37 },
    { 0xbbe61035, SUC_lexicon_tags_34 },
    { 0xffc61037, SUC_lexicon_tags_17 },
    { 0x90861037, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa986103f, SUC_lexicon_tags_60 },
    { 0x73a6103f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce61044, SUC_lexicon_tags_25 },
    { 0x8ce61045, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba61049, SUC_lexicon_tags_10 },
    { 0x3a06104a, SUC_lexicon_tags_11 },
    { 0xc8e6104b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf06104e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8061050, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x91261053, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b861057, SUC_lexicon_tags_11 },
    { 0xf261057, SUC_lexicon_tags_17 },
    { 0x2d861059, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb646105b, SUC_lexicon_tags_32 },
    { 0x1a06105c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61061, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc61064, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e61067, SUC_lexicon_tags_27 },
    { 0x6f061067, SUC_lexicon_tags_1 },
    { 0x4ba61068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8c6106b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86a6106d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9da6106f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe61072, SUC_lexicon_tags_17 },
    { 0x6c661072, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42661075, SUC_lexicon_tags_37 },
    { 0xe261075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a6107d, SUC_lexicon_tags_11 },
    { 0x9a46107d, SUC_lexicon_tags_9 },
    { 0xf1e6107f, SUC_lexicon_tags_60 },
    { 0x2f26107f, SUC_lexicon_tags_30 },
    { 0x2e661081, SUC_lexicon_tags_47 },
    { 0x2bc61082, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf106108b, SUC_lexicon_tags_11 },
    { 0xf126108b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79261090, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xdc61092, SUC_lexicon_tags_11 },
    { 0x3ce61093, SUC_lexicon_tags_37 },
    { 0x9fe61094, SUC_lexicon_tags_15 },
    { 0x6e861094, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26109b, SUC_lexicon_tags_17 },
    { 0x4606109c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ba6109e, SUC_lexicon_tags_10 },
    { 0xdaa6109e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba8610a1, SUC_lexicon_tags_0 },
    { 0x3c8610a1, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xac4610a4, SUC_lexicon_tags_7 },
    { 0x278610a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0610aa, SUC_lexicon_tags_58 },
    { 0x738610ab, SUC_lexicon_tags_30 },
    { 0x41a610aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa8c610ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8610b5, SUC_lexicon_tags_6 },
    { 0x37a610b6, SUC_lexicon_tags_19 },
    { 0xe9a610b7, SUC_lexicon_tags_11 },
    { 0x590610b6, SUC_lexicon_tags_25 },
    { 0x7d4610b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e610be, SUC_lexicon_tags_17 },
    { 0x2be610bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb38610c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52610cb, SUC_lexicon_tags_37 },
    { 0x6b2610cb, SUC_lexicon_tags_2 },
    { 0xe4c610cb, SUC_lexicon_tags_15 },
    { 0x9d0610cb, SUC_lexicon_tags_25 },
    { 0xbba610cf, SUC_lexicon_tags_24 },
    { 0xfbe610d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2610d4, SUC_lexicon_tags_17 },
    { 0xaf2610d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x68e610d7, SUC_lexicon_tags_25 },
    { 0x47a610d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4610db, SUC_lexicon_tags_17 },
    { 0x78a610dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950610e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe3c610e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504610e8, SUC_lexicon_tags_30 },
    { 0x6ea610e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64610ed, SUC_lexicon_tags_39 },
    { 0x356610ee, SUC_lexicon_tags_2 },
    { 0x75c610ef, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0610f2, SUC_lexicon_tags_37 },
    { 0xe8e610f3, SUC_lexicon_tags_43 },
    { 0x430610f2, SUC_lexicon_tags_16 },
    { 0x808610f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xca6610f7, SUC_lexicon_tags_12 },
    { 0x5ae610f7, SUC_lexicon_tags_70 },
    { 0x74610f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x84c610fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f661102, SUC_lexicon_tags_44 },
    { 0x77861102, SUC_lexicon_tags_7 },
    { 0x9e61104, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce261108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c26110a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac6110d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c61111, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd4061113, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19661115, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa4061117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a86111a, SUC_lexicon_tags_1 },
    { 0xeec6111b, SUC_lexicon_tags_17 },
    { 0xd0c6111c, SUC_lexicon_tags_17 },
    { 0x9ce6111d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8861122, SUC_lexicon_tags_37 },
    { 0x4e861123, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1e61125, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb66112e, SUC_lexicon_tags_37 },
    { 0xa506112f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55861134, SUC_lexicon_tags_18 },
    { 0xac061135, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf2661137, SUC_lexicon_tags_43 },
    { 0x30c61137, SUC_lexicon_tags_60 },
    { 0xc3c61139, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e6113c, SUC_lexicon_tags_9 },
    { 0xcaa6113d, SUC_lexicon_tags_7 },
    { 0xc126113e, SUC_lexicon_tags_6 },
    { 0xaaa6113e, SUC_lexicon_tags_12 },
    { 0x26a6113d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de61143, SUC_lexicon_tags_12 },
    { 0x7a461144, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d261146, SUC_lexicon_tags_17 },
    { 0x1c261146, SUC_lexicon_tags_47 },
    { 0xea661148, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a6114b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3406114f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf261153, SUC_lexicon_tags_37 },
    { 0x8fe61154, SUC_lexicon_tags_10 },
    { 0x29061153, SUC_lexicon_tags_12 },
    { 0xe6261156, SUC_lexicon_tags_9 },
    { 0x60061157, SUC_lexicon_tags_37 },
    { 0x7ae61158, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c6115b, SUC_lexicon_tags_15 },
    { 0xb466115b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e6115f, SUC_lexicon_tags_37 },
    { 0x3fe6115f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc261163, SUC_lexicon_tags_11 },
    { 0xfc61164, SUC_lexicon_tags_31 },
    { 0x91c61164, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb661167, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x40061169, SUC_lexicon_tags_25 },
    { 0xea46116a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4986116f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50661172, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x55261174, SUC_lexicon_tags_7 },
    { 0xab061175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77261179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe6117c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf206117e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93e61180, SUC_lexicon_tags_1 },
    { 0x96661181, SUC_lexicon_tags_34 },
    { 0xdfe61182, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x47061185, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4861188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf86118f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf661192, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc61197, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be6119c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8766119e, SUC_lexicon_tags_17 },
    { 0x5366119e, SUC_lexicon_tags_37 },
    { 0x1f46119f, SUC_lexicon_tags_1 },
    { 0xa90611a1, SUC_lexicon_tags_37 },
    { 0x4a66119f, SUC_lexicon_tags_38 },
    { 0xa606119e, SUC_lexicon_tags_11 },
    { 0x6fa611a4, SUC_lexicon_tags_23 },
    { 0xfd0611a0, SUC_lexicon_tags_65 },
    { 0x936611a1, SUC_lexicon_tags_25 },
    { 0xad6611a2, SUC_lexicon_tags_66 },
    { 0x7dc611a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5a611aa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb70611ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1b8611ae, SUC_lexicon_tags_12 },
    { 0x346611ae, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788611b4, SUC_lexicon_tags_1 },
    { 0x55c611b4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8611ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22c611bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6611bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74611c3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e611c6, SUC_lexicon_tags_282 },
    { 0x60c611c7, SUC_lexicon_tags_2 },
    { 0xaf2611c8, SUC_lexicon_tags_127 },
    { 0x3d0611c6, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34611cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c611cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ee611d1, SUC_lexicon_tags_27 },
    { 0x51c611d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84611d5, SUC_lexicon_tags_12 },
    { 0x8a611d5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20611dd, SUC_lexicon_tags_47 },
    { 0xed8611dd, SUC_lexicon_tags_12 },
    { 0x322611df, SUC_lexicon_tags_47 },
    { 0xe0c611dd, SUC_lexicon_tags_16 },
    { 0x2c2611df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72611e3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x464611e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674611ea, SUC_lexicon_tags_17 },
    { 0x920611eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac611ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4611f1, SUC_lexicon_tags_32 },
    { 0xc22611f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78611f6, SUC_lexicon_tags_15 },
    { 0xa2e611f7, SUC_lexicon_tags_46 },
    { 0x584611f8, SUC_lexicon_tags_17 },
    { 0xc52611f9, SUC_lexicon_tags_45 },
    { 0x670611f8, SUC_lexicon_tags_27 },
    { 0x286611f8, SUC_lexicon_tags_15 },
    { 0xc5c611f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x446611ff, SUC_lexicon_tags_75 },
    { 0x98c61200, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45861205, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2461208, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6e46120a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c66120e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa861212, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa61216, SUC_lexicon_tags_0 },
    { 0x96461217, SUC_lexicon_tags_25 },
    { 0x28061218, SUC_lexicon_tags_37 },
    { 0xd3261219, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de6121d, SUC_lexicon_tags_2 },
    { 0x7d06121e, SUC_lexicon_tags_19 },
    { 0xe386121f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89861223, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16061225, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51861229, SUC_lexicon_tags_27 },
    { 0x9d66122a, SUC_lexicon_tags_37 },
    { 0xfb46122b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x60c6122d, SUC_lexicon_tags_30 },
    { 0xd466122e, SUC_lexicon_tags_12 },
    { 0xb2a6122f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a61233, SUC_lexicon_tags_17 },
    { 0xe6e61234, SUC_lexicon_tags_1 },
    { 0xca261235, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb326123a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd986123c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf5e6123e, SUC_lexicon_tags_4 },
    { 0xec66123f, SUC_lexicon_tags_0 },
    { 0x4f06123f, SUC_lexicon_tags_2 },
    { 0x6a261241, SUC_lexicon_tags_15 },
    { 0x7b061241, SUC_lexicon_tags_15 },
    { 0xa9c61243, SUC_lexicon_tags_43 },
    { 0x95261243, SUC_lexicon_tags_82 },
    { 0x20e61243, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3046124c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e6124e, SUC_lexicon_tags_2 },
    { 0x4386124f, SUC_lexicon_tags_0 },
    { 0xa826124e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb661253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57261258, SUC_lexicon_tags_25 },
    { 0xa3a61259, SUC_lexicon_tags_18 },
    { 0xf2461259, SUC_lexicon_tags_25 },
    { 0x75661258, SUC_lexicon_tags_25 },
    { 0xade6125c, SUC_lexicon_tags_19 },
    { 0x4266125c, SUC_lexicon_tags_17 },
    { 0xa9a6125e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94261262, SUC_lexicon_tags_19 },
    { 0x37c61263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0861265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4061267, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2c861269, SUC_lexicon_tags_6 },
    { 0x77e6126a, SUC_lexicon_tags_37 },
    { 0xc86126a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf446126e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca61271, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xcda61273, SUC_lexicon_tags_7 },
    { 0x1fa61273, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9926127f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4ee61281, SUC_lexicon_tags_37 },
    { 0xdea61281, SUC_lexicon_tags_7 },
    { 0x31061281, SUC_lexicon_tags_8 },
    { 0x20061284, SUC_lexicon_tags_7 },
    { 0x64c61284, SUC_lexicon_tags_11 },
    { 0xa0a61286, SUC_lexicon_tags_56 },
    { 0xf3861287, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13661292, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0061294, SUC_lexicon_tags_11 },
    { 0x58261295, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9a461297, SUC_lexicon_tags_9 },
    { 0xaaa61297, SUC_lexicon_tags_1 },
    { 0xc0661299, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d06129f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a612a2, SUC_lexicon_tags_204 },
    { 0x4d4612a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe612a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318612aa, SUC_lexicon_tags_46 },
    { 0x4ca612ab, SUC_lexicon_tags_33 },
    { 0xbaa612aa, SUC_lexicon_tags_11 },
    { 0x996612ad, SUC_lexicon_tags_23 },
    { 0x820612ae, SUC_lexicon_tags_15 },
    { 0x8be612ab, SUC_lexicon_tags_42 },
    { 0x136612ab, SUC_lexicon_tags_61 },
    { 0x628612b1, SUC_lexicon_tags_15 },
    { 0x83c612b1, SUC_lexicon_tags_17 },
    { 0xa9e612b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2612b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x89a612b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932612bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16612c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46612c4, SUC_lexicon_tags_9 },
    { 0xd62612c5, SUC_lexicon_tags_0 },
    { 0xd34612c6, SUC_lexicon_tags_112 },
    { 0xc8e612c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde612cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xabe612ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78e612d0, SUC_lexicon_tags_11 },
    { 0xb00612d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc612d4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a612d7, SUC_lexicon_tags_1 },
    { 0xa18612d8, SUC_lexicon_tags_5 },
    { 0x464612d9, SUC_lexicon_tags_19 },
    { 0xc90612d9, SUC_lexicon_tags_31 },
    { 0x74612db, SUC_lexicon_tags_49 },
    { 0x422612dc, SUC_lexicon_tags_12 },
    { 0xef2612dd, SUC_lexicon_tags_25 },
    { 0x33e612d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe612e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x89a612e3, SUC_lexicon_tags_25 },
    { 0x9f2612e4, SUC_lexicon_tags_14 },
    { 0x986612e3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8612e8, SUC_lexicon_tags_19 },
    { 0x87e612e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e612ed, SUC_lexicon_tags_160 },
    { 0x5de612ee, SUC_lexicon_tags_28 },
    { 0xa16612ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x28c612f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2612f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee612f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc92612fb, SUC_lexicon_tags_37 },
    { 0x74612fc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e61304, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e061308, SUC_lexicon_tags_46 },
    { 0x8a061308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde86130d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2066130f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2e61311, SUC_lexicon_tags_35 },
    { 0x9e861312, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c61315, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77661318, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c6131d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd806131f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d061322, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c61326, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35061328, SUC_lexicon_tags_2 },
    { 0x30661329, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3766132b, SUC_lexicon_tags_12 },
    { 0x7266132c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6132f, SUC_lexicon_tags_25 },
    { 0x11e61330, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e61335, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5e661337, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e86133a, SUC_lexicon_tags_7 },
    { 0xec86133a, SUC_lexicon_tags_47 },
    { 0x59e6133c, SUC_lexicon_tags_17 },
    { 0xc726133d, SUC_lexicon_tags_23 },
    { 0xeea6133c, SUC_lexicon_tags_15 },
    { 0xf46133e, SUC_lexicon_tags_25 },
    { 0x27061340, SUC_lexicon_tags_37 },
    { 0xf6661340, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a61346, SUC_lexicon_tags_37 },
    { 0xb7061347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c661349, SUC_lexicon_tags_2 },
    { 0x3926134a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c6134f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e261352, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb646135b, SUC_lexicon_tags_33 },
    { 0xab46135c, SUC_lexicon_tags_1 },
    { 0x33e6135d, SUC_lexicon_tags_47 },
    { 0x4a46135d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe61361, SUC_lexicon_tags_37 },
    { 0xad461361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4661364, SUC_lexicon_tags_2 },
    { 0x44461365, SUC_lexicon_tags_25 },
    { 0xcd461366, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc06136a, SUC_lexicon_tags_12 },
    { 0x9d26136b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e61370, SUC_lexicon_tags_0 },
    { 0xdd461370, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31061375, SUC_lexicon_tags_61 },
    { 0xc5c61376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb686137b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc61381, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x59e61383, SUC_lexicon_tags_15 },
    { 0xcf661383, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3486138c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1e6138e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x83661391, SUC_lexicon_tags_29 },
    { 0x9da61391, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde61396, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a61399, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc6139c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e6139e, SUC_lexicon_tags_12 },
    { 0xb226139f, SUC_lexicon_tags_13 },
    { 0x100613a0, SUC_lexicon_tags_64 },
    { 0x688613a0, SUC_lexicon_tags_25 },
    { 0x582613a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330613ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a613af, SUC_lexicon_tags_5 },
    { 0xb0613b0, SUC_lexicon_tags_17 },
    { 0x380613b1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a613b4, SUC_lexicon_tags_17 },
    { 0xd02613b4, SUC_lexicon_tags_11 },
    { 0x89a613b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9a613b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7c2613ba, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x544613bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4de613be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e613c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0613c4, SUC_lexicon_tags_0 },
    { 0x862613c5, SUC_lexicon_tags_38 },
    { 0xcee613c5, SUC_lexicon_tags_13 },
    { 0xabc613c5, SUC_lexicon_tags_25 },
    { 0x62613c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102613cc, SUC_lexicon_tags_39 },
    { 0x518613cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4ac613cf, SUC_lexicon_tags_21 },
    { 0xc50613d0, SUC_lexicon_tags_63 },
    { 0x80c613d1, SUC_lexicon_tags_47 },
    { 0xba8613d2, SUC_lexicon_tags_1 },
    { 0x904613d0, SUC_lexicon_tags_47 },
    { 0x504613d4, SUC_lexicon_tags_46 },
    { 0x47e613d1, SUC_lexicon_tags_11 },
    { 0x9d2613d4, SUC_lexicon_tags_9 },
    { 0x7a0613d7, SUC_lexicon_tags_11 },
    { 0xb2a613d7, SUC_lexicon_tags_15 },
    { 0x242613d7, SUC_lexicon_tags_7 },
    { 0x2bc613d9, SUC_lexicon_tags_77 },
    { 0x902613db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8613df, SUC_lexicon_tags_7 },
    { 0x570613e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcf6613e2, SUC_lexicon_tags_37 },
    { 0x4c0613e3, SUC_lexicon_tags_15 },
    { 0xffa613e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x274613e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe613ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcba613ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbea613f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde613f7, SUC_lexicon_tags_1 },
    { 0x9de613f8, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6a0613fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4613fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f461408, SUC_lexicon_tags_47 },
    { 0xa2061409, SUC_lexicon_tags_37 },
    { 0xfe06140a, SUC_lexicon_tags_12 },
    { 0xcb66140b, SUC_lexicon_tags_37 },
    { 0x2dc61409, SUC_lexicon_tags_0 },
    { 0x8886140d, SUC_lexicon_tags_12 },
    { 0xfe86140e, SUC_lexicon_tags_33 },
    { 0x5026140f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e61412, SUC_lexicon_tags_8 },
    { 0xb2461413, SUC_lexicon_tags_66 },
    { 0x5b461414, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7006141b, SUC_lexicon_tags_43 },
    { 0xd066141b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62c6141e, SUC_lexicon_tags_12 },
    { 0x6006141f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da61422, SUC_lexicon_tags_2 },
    { 0x49e61423, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b661425, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xadc61427, SUC_lexicon_tags_11 },
    { 0xf4261428, SUC_lexicon_tags_60 },
    { 0x3f061428, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1206142c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6061431, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca61434, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce61439, SUC_lexicon_tags_8 },
    { 0xa526143a, SUC_lexicon_tags_27 },
    { 0x4486143b, SUC_lexicon_tags_15 },
    { 0x25c6143a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb461448, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4386144c, SUC_lexicon_tags_46 },
    { 0x84c6144d, SUC_lexicon_tags_7 },
    { 0xf2e6144d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a61452, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55461458, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1666145a, SUC_lexicon_tags_11 },
    { 0x2966145a, SUC_lexicon_tags_43 },
    { 0x5a6145c, SUC_lexicon_tags_37 },
    { 0xd486145d, SUC_lexicon_tags_2 },
    { 0x7966145d, SUC_lexicon_tags_0 },
    { 0x6e26145d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d061461, SUC_lexicon_tags_60 },
    { 0x84261461, SUC_lexicon_tags_27 },
    { 0x90e61463, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50c61465, SUC_lexicon_tags_43 },
    { 0x9c061466, SUC_lexicon_tags_15 },
    { 0xe0461466, SUC_lexicon_tags_17 },
    { 0xa9e61465, SUC_lexicon_tags_37 },
    { 0x9ce61469, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb686146d, SUC_lexicon_tags_17 },
    { 0xcb26146e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xede61470, SUC_lexicon_tags_12 },
    { 0xd3661470, SUC_lexicon_tags_68 },
    { 0x81e61472, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa61475, SUC_lexicon_tags_7 },
    { 0x3ec61476, SUC_lexicon_tags_2 },
    { 0xb2e61476, SUC_lexicon_tags_28 },
    { 0xe7261478, SUC_lexicon_tags_14 },
    { 0x12c61479, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d461481, SUC_lexicon_tags_1 },
    { 0xcf261482, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e61487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa6148c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e61491, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0x94861494, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5466149b, SUC_lexicon_tags_38 },
    { 0xd646149c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f66149f, SUC_lexicon_tags_27 },
    { 0x45c614a0, SUC_lexicon_tags_17 },
    { 0x500614a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4614a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8614a9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x77e614ab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0614ae, SUC_lexicon_tags_7 },
    { 0x24614ae, SUC_lexicon_tags_0 },
    { 0x25e614b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e614b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4614b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8614bb, SUC_lexicon_tags_21 },
    { 0x5b8614bc, SUC_lexicon_tags_47 },
    { 0x1dc614bd, SUC_lexicon_tags_0 },
    { 0x4ee614bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a614c1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c614ca, SUC_lexicon_tags_17 },
    { 0x9f2614ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x246614ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x434614d0, SUC_lexicon_tags_14 },
    { 0xc4c614d1, SUC_lexicon_tags_91 },
    { 0xe1c614d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6614d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384614df, SUC_lexicon_tags_49 },
    { 0x234614df, SUC_lexicon_tags_163 },
    { 0x98a614e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x630614e3, SUC_lexicon_tags_8 },
    { 0x732614e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x378614e6, SUC_lexicon_tags_38 },
    { 0x6e2614e6, SUC_lexicon_tags_32 },
    { 0x5c4614e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e614eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x390614ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6614f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x576614f2, SUC_lexicon_tags_0 },
    { 0xfbc614f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6614fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x77e614fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e61501, SUC_lexicon_tags_14 },
    { 0x31a61502, SUC_lexicon_tags_32 },
    { 0x31061502, SUC_lexicon_tags_25 },
    { 0xc1c61502, SUC_lexicon_tags_37 },
    { 0x63a61503, SUC_lexicon_tags_47 },
    { 0x30c61502, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea61509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63c6150b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9006150f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70661515, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a61518, SUC_lexicon_tags_15 },
    { 0x98a61519, SUC_lexicon_tags_60 },
    { 0xa826151a, SUC_lexicon_tags_1 },
    { 0xba26151b, SUC_lexicon_tags_37 },
    { 0x6466151c, SUC_lexicon_tags_14 },
    { 0x5d46151c, SUC_lexicon_tags_9 },
    { 0x8826151e, SUC_lexicon_tags_6 },
    { 0x36c6151f, SUC_lexicon_tags_37 },
    { 0x23e61520, SUC_lexicon_tags_23 },
    { 0x9c861521, SUC_lexicon_tags_3 },
    { 0x4546151f, SUC_lexicon_tags_129 },
    { 0x15061523, SUC_lexicon_tags_37 },
    { 0xb8261524, SUC_lexicon_tags_15 },
    { 0x77c61520, SUC_lexicon_tags_25 },
    { 0x59261523, SUC_lexicon_tags_30 },
    { 0x58261527, SUC_lexicon_tags_15 },
    { 0xe5a61527, SUC_lexicon_tags_91 },
    { 0x52c61522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e6152e, SUC_lexicon_tags_5 },
    { 0xf8a6152f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e61536, SUC_lexicon_tags_15 },
    { 0x9a261536, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c6153a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee6153f, SUC_lexicon_tags_33 },
    { 0xa2a61540, SUC_lexicon_tags_2 },
    { 0xace61540, SUC_lexicon_tags_27 },
    { 0x99c61542, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54461544, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x66261546, SUC_lexicon_tags_9 },
    { 0xd8261547, SUC_lexicon_tags_49 },
    { 0x41861547, SUC_lexicon_tags_1 },
    { 0x25461549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e61552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a61558, SUC_lexicon_tags_25 },
    { 0x55a61558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d26155c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef661565, SUC_lexicon_tags_38 },
    { 0xe2e61566, SUC_lexicon_tags_79 },
    { 0x9e61565, SUC_lexicon_tags_27 },
    { 0x69261566, SUC_lexicon_tags_82 },
    { 0x5e061565, SUC_lexicon_tags_27 },
    { 0xe4e6156a, SUC_lexicon_tags_17 },
    { 0x8be6156b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a261571, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59661576, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c6157a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1086157d, SUC_lexicon_tags_32 },
    { 0xae66157e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e61581, SUC_lexicon_tags_63 },
    { 0x75661581, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a66158b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a06158d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25261598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad66159a, SUC_lexicon_tags_17 },
    { 0x7906159b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e86159e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x142615a1, SUC_lexicon_tags_30 },
    { 0x37a615a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x560615a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e615a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9ac615ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87a615ad, SUC_lexicon_tags_11 },
    { 0x9b2615ae, SUC_lexicon_tags_4 },
    { 0x964615ae, SUC_lexicon_tags_70 },
    { 0xc30615ad, SUC_lexicon_tags_2 },
    { 0xbb6615af, SUC_lexicon_tags_12 },
    { 0x4e4615ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa38615b4, SUC_lexicon_tags_49 },
    { 0x884615b5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac615b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe615bd, SUC_lexicon_tags_1 },
    { 0x838615be, SUC_lexicon_tags_1 },
    { 0xf06615bd, SUC_lexicon_tags_11 },
    { 0x992615c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1ce615c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x480615c4, SUC_lexicon_tags_17 },
    { 0xb34615c4, SUC_lexicon_tags_160 },
    { 0xcec615c6, SUC_lexicon_tags_3 },
    { 0x604615c6, SUC_lexicon_tags_7 },
    { 0x40a615c6, SUC_lexicon_tags_25 },
    { 0xb26615c9, SUC_lexicon_tags_2 },
    { 0xa8615c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4615cc, SUC_lexicon_tags_10 },
    { 0xd20615cc, SUC_lexicon_tags_0 },
    { 0xdac615cd, SUC_lexicon_tags_17 },
    { 0x6f4615ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2615d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558615d9, SUC_lexicon_tags_37 },
    { 0x2bc615d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974615de, SUC_lexicon_tags_17 },
    { 0xea2615df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210615e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45a615ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x154615ec, SUC_lexicon_tags_21 },
    { 0xd28615ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x88a615ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6615f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902615f7, SUC_lexicon_tags_45 },
    { 0xd9c615f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb8615fa, SUC_lexicon_tags_65 },
    { 0xa6c615fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77261602, SUC_lexicon_tags_56 },
    { 0x39861603, SUC_lexicon_tags_14 },
    { 0x68c61604, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2d261606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e6160f, SUC_lexicon_tags_6 },
    { 0x52261610, SUC_lexicon_tags_25 },
    { 0xa6261610, SUC_lexicon_tags_11 },
    { 0xa3e61610, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d26161b, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xb9e6161d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3746161f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4261621, SUC_lexicon_tags_30 },
    { 0x84261621, SUC_lexicon_tags_15 },
    { 0x55e61621, SUC_lexicon_tags_10 },
    { 0xe4861622, SUC_lexicon_tags_12 },
    { 0xfc061623, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43061630, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60861634, SUC_lexicon_tags_37 },
    { 0x4261635, SUC_lexicon_tags_15 },
    { 0x87861635, SUC_lexicon_tags_0 },
    { 0xeb461637, SUC_lexicon_tags_15 },
    { 0x2061637, SUC_lexicon_tags_11 },
    { 0x11a61639, SUC_lexicon_tags_82 },
    { 0x6c861638, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2926163f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb461644, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20861646, SUC_lexicon_tags_11 },
    { 0xed261647, SUC_lexicon_tags_17 },
    { 0x33461648, SUC_lexicon_tags_37 },
    { 0xa9061646, SUC_lexicon_tags_33 },
    { 0xfda6164a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce61650, SUC_lexicon_tags_46 },
    { 0x33061651, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92a61653, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15861662, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a6166c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a661677, SUC_lexicon_tags_1 },
    { 0x4a861677, SUC_lexicon_tags_61 },
    { 0x33e61679, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9306167d, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0x9be6167f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe2e61681, SUC_lexicon_tags_46 },
    { 0xa3261682, SUC_lexicon_tags_11 },
    { 0xed061683, SUC_lexicon_tags_49 },
    { 0xcbc61684, SUC_lexicon_tags_14 },
    { 0xecc61684, SUC_lexicon_tags_37 },
    { 0x8461682, SUC_lexicon_tags_25 },
    { 0x2b261681, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c6168b, SUC_lexicon_tags_49 },
    { 0x5c66168c, SUC_lexicon_tags_7 },
    { 0x7226168b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a46168f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0e61691, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47661696, SUC_lexicon_tags_12 },
    { 0x26861697, SUC_lexicon_tags_29 },
    { 0x44c61698, SUC_lexicon_tags_29 },
    { 0x8d861699, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdd06169b, SUC_lexicon_tags_26 },
    { 0x97a6169c, SUC_lexicon_tags_117 },
    { 0x46a6169d, SUC_lexicon_tags_49 },
    { 0x6026169b, SUC_lexicon_tags_4 },
    { 0x3f06169e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e616a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x792616a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744616ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4616ae, SUC_lexicon_tags_36 },
    { 0x11e616af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x558616b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a616b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92616bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x514616bd, SUC_lexicon_tags_31 },
    { 0xadc616be, SUC_lexicon_tags_9 },
    { 0xfc0616bf, SUC_lexicon_tags_68 },
    { 0x596616c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2616c4, SUC_lexicon_tags_15 },
    { 0x688616c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x466616c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd5a616ca, SUC_lexicon_tags_4 },
    { 0xd5e616ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e616cf, SUC_lexicon_tags_26 },
    { 0x552616d0, SUC_lexicon_tags_15 },
    { 0x4b8616cf, SUC_lexicon_tags_47 },
    { 0x788616d0, SUC_lexicon_tags_0 },
    { 0x9aa616d3, SUC_lexicon_tags_85 },
    { 0x9a0616d4, SUC_lexicon_tags_11 },
    { 0x4d0616d3, SUC_lexicon_tags_25 },
    { 0xf1c616d6, SUC_lexicon_tags_25 },
    { 0x104616cf, SUC_lexicon_tags_87 },
    { 0x4b8616d8, SUC_lexicon_tags_11 },
    { 0x532616d0, SUC_lexicon_tags_65 },
    { 0x18c616da, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a616e0, SUC_lexicon_tags_6 },
    { 0xee0616e1, SUC_lexicon_tags_19 },
    { 0xbf2616e2, SUC_lexicon_tags_1 },
    { 0x76616e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc616e6, SUC_lexicon_tags_47 },
    { 0xa30616e7, SUC_lexicon_tags_56 },
    { 0x42616e8, SUC_lexicon_tags_87 },
    { 0xa2a616e7, SUC_lexicon_tags_11 },
    { 0xefa616e8, SUC_lexicon_tags_17 },
    { 0x12616ea, SUC_lexicon_tags_47 },
    { 0xcc6616ec, SUC_lexicon_tags_32 },
    { 0xc98616ea, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x1c2616ef, SUC_lexicon_tags_15 },
    { 0xea4616f0, SUC_lexicon_tags_15 },
    { 0xf9a616ef, SUC_lexicon_tags_17 },
    { 0xbdc616f2, SUC_lexicon_tags_106 },
    { 0x76616f3, SUC_lexicon_tags_27 },
    { 0xd46616f3, SUC_lexicon_tags_14 },
    { 0xf94616f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x398616f7, SUC_lexicon_tags_16 },
    { 0x17c616f8, SUC_lexicon_tags_27 },
    { 0x798616f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c616fc, SUC_lexicon_tags_5 },
    { 0xe02616fd, SUC_lexicon_tags_25 },
    { 0xa42616fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x63861701, SUC_lexicon_tags_9 },
    { 0x13461702, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da61709, SUC_lexicon_tags_11 },
    { 0x7086170a, SUC_lexicon_tags_12 },
    { 0x6826170a, SUC_lexicon_tags_2 },
    { 0x1d26170a, SUC_lexicon_tags_68 },
    { 0x3bc6170b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a61710, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6c61712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32a61714, SUC_lexicon_tags_27 },
    { 0x85061715, SUC_lexicon_tags_47 },
    { 0xf661714, SUC_lexicon_tags_15 },
    { 0x31261717, SUC_lexicon_tags_48 },
    { 0xae61715, SUC_lexicon_tags_17 },
    { 0x1be61718, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8466171d, SUC_lexicon_tags_45 },
    { 0xebc6171d, SUC_lexicon_tags_15 },
    { 0x9826171f, SUC_lexicon_tags_14 },
    { 0xf6e6171f, SUC_lexicon_tags_26 },
    { 0x13861721, SUC_lexicon_tags_17 },
    { 0xf2e61722, SUC_lexicon_tags_19 },
    { 0x6c661720, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c6172c, SUC_lexicon_tags_27 },
    { 0xe166172c, SUC_lexicon_tags_33 },
    { 0x6366172c, SUC_lexicon_tags_17 },
    { 0x1806172f, SUC_lexicon_tags_12 },
    { 0xe886172d, SUC_lexicon_tags_45 },
    { 0x6b061731, SUC_lexicon_tags_17 },
    { 0xcc661732, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x11061735, SUC_lexicon_tags_65 },
    { 0x9c861736, SUC_lexicon_tags_37 },
    { 0x44861737, SUC_lexicon_tags_28 },
    { 0xa0861738, SUC_lexicon_tags_25 },
    { 0x1ec61737, SUC_lexicon_tags_208 },
    { 0xa426173a, SUC_lexicon_tags_23 },
    { 0xc066173a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x11e6173d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f461743, SUC_lexicon_tags_17 },
    { 0x2ac61744, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a61749, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e61750, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x96e61752, SUC_lexicon_tags_12 },
    { 0x67661753, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea061756, SUC_lexicon_tags_19 },
    { 0x7fc61756, SUC_lexicon_tags_38 },
    { 0xf2661756, SUC_lexicon_tags_33 },
    { 0xa8a61759, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5926175d, SUC_lexicon_tags_26 },
    { 0x65e6175d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f661762, SUC_lexicon_tags_23 },
    { 0x2a661763, SUC_lexicon_tags_25 },
    { 0xd9e61763, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83c61766, SUC_lexicon_tags_14 },
    { 0xf9861767, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12861769, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e26176d, SUC_lexicon_tags_14 },
    { 0x8866176e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9bc61770, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6f461772, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3861775, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a6177a, SUC_lexicon_tags_17 },
    { 0xc706177b, SUC_lexicon_tags_1 },
    { 0xfdc6177b, SUC_lexicon_tags_47 },
    { 0xa86177d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9261780, SUC_lexicon_tags_25 },
    { 0x5be61781, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x85061783, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6a461785, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb266178a, SUC_lexicon_tags_4 },
    { 0xe2e6178a, SUC_lexicon_tags_37 },
    { 0x2c86178c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c6178f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66661796, SUC_lexicon_tags_31 },
    { 0xef461797, SUC_lexicon_tags_0 },
    { 0xdb861798, SUC_lexicon_tags_15 },
    { 0x25c61798, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326179e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1c617a0, SUC_lexicon_tags_26 },
    { 0xde8617a0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58617a4, SUC_lexicon_tags_49 },
    { 0x938617a5, SUC_lexicon_tags_9 },
    { 0xb32617a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188617aa, SUC_lexicon_tags_25 },
    { 0x356617ab, SUC_lexicon_tags_23 },
    { 0x60a617ac, SUC_lexicon_tags_65 },
    { 0xc1a617ad, SUC_lexicon_tags_47 },
    { 0xbfa617ad, SUC_lexicon_tags_35 },
    { 0x130617ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa30617b1, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602617b6, SUC_lexicon_tags_33 },
    { 0x4ae617b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c617bb, SUC_lexicon_tags_25 },
    { 0xf78617bc, SUC_lexicon_tags_87 },
    { 0xd4a617bc, SUC_lexicon_tags_2 },
    { 0x1d2617be, SUC_lexicon_tags_49 },
    { 0x178617bf, SUC_lexicon_tags_49 },
    { 0x44617bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85e617c2, SUC_lexicon_tags_9 },
    { 0xd5e617c3, SUC_lexicon_tags_2 },
    { 0x73c617c2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856617c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x404617cc, SUC_lexicon_tags_7 },
    { 0x828617cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8617d3, SUC_lexicon_tags_39 },
    { 0x556617d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9c617d6, SUC_lexicon_tags_6 },
    { 0xe86617d6, SUC_lexicon_tags_12 },
    { 0xae4617d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb32617da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x592617dd, SUC_lexicon_tags_1374 },
    { 0x900617de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bc617e0, SUC_lexicon_tags_12 },
    { 0x5cc617e1, SUC_lexicon_tags_25 },
    { 0x4e617e1, SUC_lexicon_tags_1 },
    { 0x802617e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a617e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf8a617e8, SUC_lexicon_tags_15 },
    { 0xfde617e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a617ec, SUC_lexicon_tags_11 },
    { 0xae4617ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2617f0, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x5ea617f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x432617f5, SUC_lexicon_tags_47 },
    { 0xb9c617f6, SUC_lexicon_tags_35 },
    { 0xec2617f7, SUC_lexicon_tags_118 },
    { 0x56c617f7, SUC_lexicon_tags_0 },
    { 0xc44617f5, SUC_lexicon_tags_37 },
    { 0xa9a617fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a61800, SUC_lexicon_tags_12 },
    { 0x69861800, SUC_lexicon_tags_49 },
    { 0xe9861802, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x99a61804, SUC_lexicon_tags_39 },
    { 0x98461804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5266180a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc861811, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f661818, SUC_lexicon_tags_2 },
    { 0xc9e61819, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f86181d, SUC_lexicon_tags_12 },
    { 0xd8c6181e, SUC_lexicon_tags_60 },
    { 0x4486181f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5661823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe61826, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa61829, SUC_lexicon_tags_63 },
    { 0x9ce61829, SUC_lexicon_tags_25 },
    { 0x9b86182b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7261838, SUC_lexicon_tags_11 },
    { 0xe7661839, SUC_lexicon_tags_17 },
    { 0x40e61839, SUC_lexicon_tags_15 },
    { 0x5a46183a, SUC_lexicon_tags_28 },
    { 0x5126183c, SUC_lexicon_tags_7 },
    { 0xbc46183d, SUC_lexicon_tags_8 },
    { 0x7346183e, SUC_lexicon_tags_17 },
    { 0x6006183d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97461845, SUC_lexicon_tags_38 },
    { 0x8ae61845, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xefa61848, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6186184d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea661851, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb261853, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x40261855, SUC_lexicon_tags_45 },
    { 0xe461855, SUC_lexicon_tags_1 },
    { 0x1f661857, SUC_lexicon_tags_2 },
    { 0x24a61857, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb546185f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb661861, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2261864, SUC_lexicon_tags_45 },
    { 0xbfa61864, SUC_lexicon_tags_37 },
    { 0x24861866, SUC_lexicon_tags_11 },
    { 0x3c061864, SUC_lexicon_tags_54 },
    { 0x29a61868, SUC_lexicon_tags_17 },
    { 0xad661864, SUC_lexicon_tags_65 },
    { 0x60861866, SUC_lexicon_tags_0 },
    { 0x1326186b, SUC_lexicon_tags_31 },
    { 0xaae61868, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xa186186e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x7ae61870, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14a61872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a61879, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee6187c, SUC_lexicon_tags_47 },
    { 0x86c6187c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb346187f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x30861881, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e61886, SUC_lexicon_tags_23 },
    { 0xee061886, SUC_lexicon_tags_30 },
    { 0x37661888, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba6188c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34461890, SUC_lexicon_tags_28 },
    { 0x4de61891, SUC_lexicon_tags_63 },
    { 0x28461890, SUC_lexicon_tags_17 },
    { 0x51261891, SUC_lexicon_tags_0 },
    { 0xc5461892, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3861896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7661898, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6146189c, SUC_lexicon_tags_27 },
    { 0x4886189c, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x8926189f, SUC_lexicon_tags_68 },
    { 0x9fe6189f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a618a3, SUC_lexicon_tags_25 },
    { 0x8d4618a4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbb6618a6, SUC_lexicon_tags_9 },
    { 0xa9e618a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a2618a9, SUC_lexicon_tags_25 },
    { 0x13e618a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8618b1, SUC_lexicon_tags_25 },
    { 0x36a618b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92618b9, SUC_lexicon_tags_7 },
    { 0x698618ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x572618bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5a618be, SUC_lexicon_tags_60 },
    { 0x40e618be, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76618c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x98618c6, SUC_lexicon_tags_14 },
    { 0x5b2618c7, SUC_lexicon_tags_79 },
    { 0xfc6618c7, SUC_lexicon_tags_28 },
    { 0xa6618c9, SUC_lexicon_tags_27 },
    { 0x32a618c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x492618cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0618cf, SUC_lexicon_tags_60 },
    { 0xa2a618d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6a0618d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a618d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc618dc, SUC_lexicon_tags_15 },
    { 0x9b0618dd, SUC_lexicon_tags_25 },
    { 0x78c618dc, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40618e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x224618e3, SUC_lexicon_tags_731 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46618e6, SUC_lexicon_tags_1 },
    { 0xc72618e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e618ea, SUC_lexicon_tags_38 },
    { 0x188618ea, SUC_lexicon_tags_1 },
    { 0xe0a618ec, SUC_lexicon_tags_12 },
    { 0x23e618ed, SUC_lexicon_tags_12 },
    { 0x230618eb, SUC_lexicon_tags_26 },
    { 0xb2a618ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6618f7, SUC_lexicon_tags_21 },
    { 0xc0e618f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90618fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de61901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e461904, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfe61906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f86190a, SUC_lexicon_tags_32 },
    { 0xcba6190b, SUC_lexicon_tags_7 },
    { 0x7846190c, SUC_lexicon_tags_63 },
    { 0x1d66190b, SUC_lexicon_tags_12 },
    { 0x28c6190e, SUC_lexicon_tags_37 },
    { 0xca66190f, SUC_lexicon_tags_47 },
    { 0x94661910, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x57c61912, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c261917, SUC_lexicon_tags_19 },
    { 0xd1061918, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67e6191a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e61921, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa8a61923, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb0461925, SUC_lexicon_tags_14 },
    { 0x2fa61926, SUC_lexicon_tags_25 },
    { 0xdec61926, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f26192a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc6192e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86661932, SUC_lexicon_tags_66 },
    { 0xe2061933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92861935, SUC_lexicon_tags_15 },
    { 0x58861936, SUC_lexicon_tags_47 },
    { 0xf2261937, SUC_lexicon_tags_43 },
    { 0x81a61936, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a6193f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae61942, SUC_lexicon_tags_25 },
    { 0xe9861942, SUC_lexicon_tags_11 },
    { 0xec461942, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa661947, SUC_lexicon_tags_14 },
    { 0x1dc61948, SUC_lexicon_tags_17 },
    { 0x2a261949, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10c6194b, SUC_lexicon_tags_57 },
    { 0xaae6194c, SUC_lexicon_tags_7 },
    { 0x65e6194b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc61957, SUC_lexicon_tags_11 },
    { 0x73661958, SUC_lexicon_tags_12 },
    { 0x1ac61959, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x746195c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e61960, SUC_lexicon_tags_33 },
    { 0x14261960, SUC_lexicon_tags_55 },
    { 0xa0a61962, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7661965, SUC_lexicon_tags_63 },
    { 0xbee61966, SUC_lexicon_tags_11 },
    { 0xa0a61967, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec6196a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc6196d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc61973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28261976, SUC_lexicon_tags_45 },
    { 0x36061977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b66197d, SUC_lexicon_tags_11 },
    { 0x8b46197d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ce61980, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc61983, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba61987, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c26198b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2206198e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99a61990, SUC_lexicon_tags_43 },
    { 0x13061991, SUC_lexicon_tags_12 },
    { 0x9ca61992, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x80661995, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b86199b, SUC_lexicon_tags_23 },
    { 0x5966199c, SUC_lexicon_tags_63 },
    { 0xfe06199b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8619a1, SUC_lexicon_tags_17 },
    { 0xb1a619a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x914619a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32619a8, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x164619ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd80619ad, SUC_lexicon_tags_27 },
    { 0x564619ad, SUC_lexicon_tags_7 },
    { 0xbbc619af, SUC_lexicon_tags_60 },
    { 0x9ec619af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a619b3, SUC_lexicon_tags_60 },
    { 0xc8e619b4, SUC_lexicon_tags_15 },
    { 0x7aa619b5, SUC_lexicon_tags_27 },
    { 0x8ce619b5, SUC_lexicon_tags_61 },
    { 0x9b8619b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbc619b9, SUC_lexicon_tags_19 },
    { 0x736619b9, SUC_lexicon_tags_37 },
    { 0xf9a619bb, SUC_lexicon_tags_7 },
    { 0x57a619b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfaa619be, SUC_lexicon_tags_8 },
    { 0x204619bf, SUC_lexicon_tags_43 },
    { 0xf70619bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4619c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa619c7, SUC_lexicon_tags_7 },
    { 0x95a619c8, SUC_lexicon_tags_12 },
    { 0x7d2619c7, SUC_lexicon_tags_43 },
    { 0x15a619ca, SUC_lexicon_tags_51 },
    { 0x18619cb, SUC_lexicon_tags_37 },
    { 0xa46619cb, SUC_lexicon_tags_43 },
    { 0x5ce619c9, SUC_lexicon_tags_148 },
    { 0x6fe619ce, SUC_lexicon_tags_63 },
    { 0x454619cf, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x2be619d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x370619d4, SUC_lexicon_tags_94 },
    { 0x6d6619d4, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c619d8, SUC_lexicon_tags_47 },
    { 0x8b4619d9, SUC_lexicon_tags_60 },
    { 0x704619d9, SUC_lexicon_tags_38 },
    { 0x8e8619db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xac4619dd, SUC_lexicon_tags_12 },
    { 0xebe619de, SUC_lexicon_tags_17 },
    { 0xd18619dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2619eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a619ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61e619f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8619f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906619fb, SUC_lexicon_tags_38 },
    { 0xa04619fc, SUC_lexicon_tags_21 },
    { 0x52a619fd, SUC_lexicon_tags_40 },
    { 0x8e619fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c61a05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6be61a07, SUC_lexicon_tags_48 },
    { 0x24861a08, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2661a0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa861a10, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a61a1d, SUC_lexicon_tags_7 },
    { 0x76e61a1e, SUC_lexicon_tags_24 },
    { 0x6b061a1f, SUC_lexicon_tags_27 },
    { 0x1b661a20, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e61a25, SUC_lexicon_tags_7 },
    { 0xeb661a26, SUC_lexicon_tags_25 },
    { 0xa8c61a27, SUC_lexicon_tags_16 },
    { 0x5ba61a28, SUC_lexicon_tags_25 },
    { 0xbc261a29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf661a2c, SUC_lexicon_tags_8 },
    { 0x43461a2c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec661a33, SUC_lexicon_tags_1 },
    { 0xb1261a33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1a61a36, SUC_lexicon_tags_63 },
    { 0x3d661a36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25e61a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37c61a3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee61a3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd6861a41, SUC_lexicon_tags_25 },
    { 0xcf661a41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x21661a45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a261a4f, SUC_lexicon_tags_15 },
    { 0xb6061a4f, SUC_lexicon_tags_29 },
    { 0x6c61a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46461a53, SUC_lexicon_tags_4 },
    { 0x79261a53, SUC_lexicon_tags_37 },
    { 0xb9261a55, SUC_lexicon_tags_25 },
    { 0x48861a56, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xfca61a58, SUC_lexicon_tags_63 },
    { 0x84c61a58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c461a5b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13261a63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x38461a66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x43e61a68, SUC_lexicon_tags_15 },
    { 0x84661a69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56061a6c, SUC_lexicon_tags_8 },
    { 0x7061a6d, SUC_lexicon_tags_27 },
    { 0xc9461a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e261a73, SUC_lexicon_tags_1 },
    { 0x22c61a74, SUC_lexicon_tags_87 },
    { 0xa2261a75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4061a78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23861a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4861a7e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91461a82, SUC_lexicon_tags_68 },
    { 0xb1261a82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbca61a85, SUC_lexicon_tags_9 },
    { 0x2ba61a85, SUC_lexicon_tags_11 },
    { 0x26461a87, SUC_lexicon_tags_11 },
    { 0x6be61a88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31061a93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb661a97, SUC_lexicon_tags_60 },
    { 0xec861a98, SUC_lexicon_tags_14 },
    { 0x21c61a99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a61a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0461aa5, SUC_lexicon_tags_55 },
    { 0xfce61aa6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x51061aa8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae61aae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a61ab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1061ab8, SUC_lexicon_tags_38 },
    { 0x33861ab9, SUC_lexicon_tags_56 },
    { 0x49061aba, SUC_lexicon_tags_17 },
    { 0x7dc61aba, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f261abe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a61ac2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5e261ac4, SUC_lexicon_tags_37 },
    { 0xe5461ac4, SUC_lexicon_tags_87 },
    { 0x58061ac6, SUC_lexicon_tags_26 },
    { 0x5e861ac5, SUC_lexicon_tags_33 },
    { 0xe3c61ac8, SUC_lexicon_tags_12 },
    { 0x52261ac9, SUC_lexicon_tags_60 },
    { 0x2cc61ac8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92461ace, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51c61ad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad261ad4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7061adc, SUC_lexicon_tags_37 },
    { 0xa8a61adc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x47261ae0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe61ae3, SUC_lexicon_tags_47 },
    { 0xf1461ae4, SUC_lexicon_tags_40 },
    { 0x19861ae5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c61ae9, SUC_lexicon_tags_37 },
    { 0x2fc61aea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c61aed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a61af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e061af2, SUC_lexicon_tags_55 },
    { 0xdd261af2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e661af7, SUC_lexicon_tags_23 },
    { 0x6d061af8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4a61afa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a61aff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe61b04, SUC_lexicon_tags_6 },
    { 0xe6061b05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8c61b07, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xae461b09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12461b10, SUC_lexicon_tags_15 },
    { 0x8861b11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f261b1a, SUC_lexicon_tags_2 },
    { 0xf4861b1b, SUC_lexicon_tags_37 },
    { 0xce261b1c, SUC_lexicon_tags_1 },
    { 0xb0661b1d, SUC_lexicon_tags_0 },
    { 0x5ca61b1e, SUC_lexicon_tags_15 },
    { 0xa5a61b1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65261b23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec61b28, SUC_lexicon_tags_15 },
    { 0x71261b29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2861b2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a61b30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91461b35, SUC_lexicon_tags_25 },
    { 0xe9861b36, SUC_lexicon_tags_47 },
    { 0x2a861b37, SUC_lexicon_tags_43 },
    { 0xce661b38, SUC_lexicon_tags_34 },
    { 0x54c61b36, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de61b40, SUC_lexicon_tags_35 },
    { 0x15a61b40, SUC_lexicon_tags_1 },
    { 0x67661b40, SUC_lexicon_tags_17 },
    { 0x64a61b42, SUC_lexicon_tags_11 },
    { 0x99061b44, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18861b48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb461b4d, SUC_lexicon_tags_1 },
    { 0x78a61b4e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e61b52, SUC_lexicon_tags_25 },
    { 0x14461b52, SUC_lexicon_tags_11 },
    { 0xca261b52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67261b5a, SUC_lexicon_tags_26 },
    { 0xdda61b5b, SUC_lexicon_tags_23 },
    { 0x2b461b5a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c61b60, SUC_lexicon_tags_31 },
    { 0xc3261b60, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4461b65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc061b67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a61b6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce61b6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74861b70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8461b72, SUC_lexicon_tags_7 },
    { 0x4fa61b72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c61b76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63061b7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd4061b7c, SUC_lexicon_tags_25 },
    { 0x48a61b7d, SUC_lexicon_tags_545 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18861b82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac61b8a, SUC_lexicon_tags_15 },
    { 0x35661b8b, SUC_lexicon_tags_11 },
    { 0xb1061b8b, SUC_lexicon_tags_49 },
    { 0x2aa61b8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d261b90, SUC_lexicon_tags_49 },
    { 0xcc861b91, SUC_lexicon_tags_43 },
    { 0xfe661b91, SUC_lexicon_tags_1 },
    { 0x8fe61b93, SUC_lexicon_tags_8 },
    { 0x8a261b93, SUC_lexicon_tags_7 },
    { 0xd6261b95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c661b98, SUC_lexicon_tags_8 },
    { 0x65e61b99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35061b9e, SUC_lexicon_tags_47 },
    { 0x4f061b9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e61ba5, SUC_lexicon_tags_25 },
    { 0x18061ba5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7861ba9, SUC_lexicon_tags_25 },
    { 0x60861baa, SUC_lexicon_tags_11 },
    { 0x56061bab, SUC_lexicon_tags_28 },
    { 0x4a461ba9, SUC_lexicon_tags_37 },
    { 0xfde61bac, SUC_lexicon_tags_1222 },
    { 0, NULL },
    { 0x4e861baf, SUC_lexicon_tags_11 },
    { 0x90061bb0, SUC_lexicon_tags_4 },
    { 0xa0661bb1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x85061bb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a61bb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca61bb9, SUC_lexicon_tags_2 },
    { 0x73a61bb9, SUC_lexicon_tags_25 },
    { 0x55661bbb, SUC_lexicon_tags_105 },
    { 0xa4861bbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e61bc0, SUC_lexicon_tags_6 },
    { 0x8d061bc0, SUC_lexicon_tags_19 },
    { 0xfa461bc2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x38861bc4, SUC_lexicon_tags_26 },
    { 0xa861bc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4461bc8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a461bce, SUC_lexicon_tags_37 },
    { 0x8a61bce, SUC_lexicon_tags_0 },
    { 0x63c61bd0, SUC_lexicon_tags_36 },
    { 0x63061bd1, SUC_lexicon_tags_15 },
    { 0x93661bd2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b661bd7, SUC_lexicon_tags_12 },
    { 0x88461bd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3861bda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13861bdf, SUC_lexicon_tags_29 },
    { 0xfa061be0, SUC_lexicon_tags_33 },
    { 0xdf061be1, SUC_lexicon_tags_30 },
    { 0xbbe61be0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x38861be5, SUC_lexicon_tags_11 },
    { 0x7261be6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e61be9, SUC_lexicon_tags_37 },
    { 0x4e461bea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8261bf2, SUC_lexicon_tags_63 },
    { 0x89c61bf3, SUC_lexicon_tags_4 },
    { 0x1c261bf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae61bfa, SUC_lexicon_tags_14 },
    { 0xdc861bfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82e61bfd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x31a61bff, SUC_lexicon_tags_0 },
    { 0x1d461bff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0661c03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22861c08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x67c61c0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b861c0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a61c13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa61c16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x61861c19, SUC_lexicon_tags_8 },
    { 0xe7461c1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22661c1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1861c24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c61c29, SUC_lexicon_tags_26 },
    { 0xf1e61c29, SUC_lexicon_tags_17 },
    { 0xce861c2a, SUC_lexicon_tags_12 },
    { 0x98061c2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85661c2e, SUC_lexicon_tags_30 },
    { 0x4f261c2f, SUC_lexicon_tags_1033 },
    { 0, NULL },
    { 0x91461c31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da61c3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d261c3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4661c42, SUC_lexicon_tags_7 },
    { 0x9fe61c43, SUC_lexicon_tags_37 },
    { 0xe5e61c44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa261c4e, SUC_lexicon_tags_17 },
    { 0x2d861c4f, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x64e61c51, SUC_lexicon_tags_37 },
    { 0x82c61c52, SUC_lexicon_tags_47 },
    { 0x98c61c51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a261c55, SUC_lexicon_tags_49 },
    { 0xed061c55, SUC_lexicon_tags_2 },
    { 0x2b261c57, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x85661c59, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe3861c5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a61c5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36861c62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea61c68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a061c6d, SUC_lexicon_tags_39 },
    { 0x9bc61c6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19461c71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f661c76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa61c79, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf8461c7b, SUC_lexicon_tags_25 },
    { 0x19e61c7c, SUC_lexicon_tags_15 },
    { 0x9e061c7c, SUC_lexicon_tags_1 },
    { 0x6a261c7b, SUC_lexicon_tags_17 },
    { 0xef461c7f, SUC_lexicon_tags_6 },
    { 0x71461c80, SUC_lexicon_tags_15 },
    { 0x75861c81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8061c84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e461c8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12461c8f, SUC_lexicon_tags_87 },
    { 0x19c61c8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a61c96, SUC_lexicon_tags_82 },
    { 0x71e61c97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a61c9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50261c9d, SUC_lexicon_tags_37 },
    { 0xfb461c9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e461ca2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x55061ca4, SUC_lexicon_tags_14 },
    { 0x6e261ca5, SUC_lexicon_tags_25 },
    { 0x4a061ca6, SUC_lexicon_tags_11 },
    { 0xe8e61ca7, SUC_lexicon_tags_45 },
    { 0x17e61ca8, SUC_lexicon_tags_4 },
    { 0x64861ca9, SUC_lexicon_tags_43 },
    { 0x6ba61ca5, SUC_lexicon_tags_22 },
    { 0x7d461cab, SUC_lexicon_tags_378 },
    { 0x42861cac, SUC_lexicon_tags_1 },
    { 0x4c661cac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x40a61caf, SUC_lexicon_tags_15 },
    { 0x69e61caf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c61cb4, SUC_lexicon_tags_21 },
    { 0xb7461cb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d061cb8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c61cbb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f861cbe, SUC_lexicon_tags_48 },
    { 0xd8261cbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74461cc1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa9661cc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61461ccb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe61cd7, SUC_lexicon_tags_33 },
    { 0x82061cd7, SUC_lexicon_tags_37 },
    { 0xeca61cd7, SUC_lexicon_tags_14 },
    { 0x37e61cda, SUC_lexicon_tags_14 },
    { 0x63e61cda, SUC_lexicon_tags_26 },
    { 0xaaa61cdc, SUC_lexicon_tags_25 },
    { 0xad461cdd, SUC_lexicon_tags_25 },
    { 0x44461cdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb261ce2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa461ce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdac61ce9, SUC_lexicon_tags_15 },
    { 0x7be61cea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac61cef, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1661cf3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x46461cf6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x64061cf9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e61cff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2c61d01, SUC_lexicon_tags_1 },
    { 0x2ec61d02, SUC_lexicon_tags_4 },
    { 0x6cc61d03, SUC_lexicon_tags_34 },
    { 0xc7661d03, SUC_lexicon_tags_146 },
    { 0x96c61d02, SUC_lexicon_tags_82 },
    { 0xf061d03, SUC_lexicon_tags_7 },
    { 0x6a061d03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e461d0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda061d0f, SUC_lexicon_tags_8 },
    { 0xa6661d0f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf7261d12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b861d16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6661d1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7861d1f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce61d22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x98461d24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50261d27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11661d2a, SUC_lexicon_tags_46 },
    { 0x34a61d2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ca61d2d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13261d34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71061d38, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x34e61d3a, SUC_lexicon_tags_10 },
    { 0xca061d3b, SUC_lexicon_tags_16 },
    { 0xcca61d3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e61d3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4061d41, SUC_lexicon_tags_37 },
    { 0xea461d41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ec61d44, SUC_lexicon_tags_25 },
    { 0xbb861d45, SUC_lexicon_tags_77 },
    { 0x26261d46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e61d53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e261d55, SUC_lexicon_tags_1 },
    { 0xadc61d55, SUC_lexicon_tags_1 },
    { 0x13061d55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5861d59, SUC_lexicon_tags_49 },
    { 0xe5261d5a, SUC_lexicon_tags_70 },
    { 0x70461d5b, SUC_lexicon_tags_27 },
    { 0x8dc61d5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xea061d5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be61d64, SUC_lexicon_tags_33 },
    { 0xcc661d64, SUC_lexicon_tags_2 },
    { 0x89a61d66, SUC_lexicon_tags_2 },
    { 0x2ec61d67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75261d6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14861d73, SUC_lexicon_tags_12 },
    { 0x46e61d74, SUC_lexicon_tags_27 },
    { 0x96061d75, SUC_lexicon_tags_1 },
    { 0x15a61d75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x50061d79, SUC_lexicon_tags_2 },
    { 0xa8e61d7a, SUC_lexicon_tags_7 },
    { 0x38261d7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72861d7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2c61d80, SUC_lexicon_tags_12 },
    { 0x37e61d81, SUC_lexicon_tags_45 },
    { 0xa8261d82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e61d85, SUC_lexicon_tags_43 },
    { 0x1b661d86, SUC_lexicon_tags_10 },
    { 0x51e61d86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0861d8d, SUC_lexicon_tags_23 },
    { 0xb861d8e, SUC_lexicon_tags_9 },
    { 0xa5061d8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c261d92, SUC_lexicon_tags_66 },
    { 0xf2061d93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ea61d95, SUC_lexicon_tags_61 },
    { 0x58c61d96, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9261d9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xda861d9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a861da0, SUC_lexicon_tags_43 },
    { 0xff261da1, SUC_lexicon_tags_37 },
    { 0x2ee61da2, SUC_lexicon_tags_15 },
    { 0xf8a61da3, SUC_lexicon_tags_25 },
    { 0xdd061da3, SUC_lexicon_tags_4 },
    { 0x21261da4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e61da8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x10061daa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3461daf, SUC_lexicon_tags_11 },
    { 0x7ec61daf, SUC_lexicon_tags_15 },
    { 0x32461db0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1261db6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7661dbc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d861dbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc061dc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e61dc6, SUC_lexicon_tags_37 },
    { 0x77261dc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe61dcf, SUC_lexicon_tags_39 },
    { 0xe2461dcf, SUC_lexicon_tags_32 },
    { 0xd4861dd1, SUC_lexicon_tags_14 },
    { 0xe261dd2, SUC_lexicon_tags_1 },
    { 0xf9c61dd3, SUC_lexicon_tags_29 },
    { 0xfb461dd1, SUC_lexicon_tags_84 },
    { 0x41461dd2, SUC_lexicon_tags_0 },
    { 0xd0e61dd2, SUC_lexicon_tags_27 },
    { 0x68261dd1, SUC_lexicon_tags_47 },
    { 0x60861dd8, SUC_lexicon_tags_38 },
    { 0x85c61dd1, SUC_lexicon_tags_11 },
    { 0xa4261dda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88061ddd, SUC_lexicon_tags_13 },
    { 0xeb261ddd, SUC_lexicon_tags_0 },
    { 0x15661ddd, SUC_lexicon_tags_27 },
    { 0x50061dde, SUC_lexicon_tags_45 },
    { 0x98e61de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec861de4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e261de6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc61dea, SUC_lexicon_tags_31 },
    { 0x60c61dea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da61df0, SUC_lexicon_tags_1 },
    { 0x47261df0, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe61df4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c661df6, SUC_lexicon_tags_1 },
    { 0x5b861df7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14661dfe, SUC_lexicon_tags_48 },
    { 0x4a261dff, SUC_lexicon_tags_54 },
    { 0xede61dff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4461e02, SUC_lexicon_tags_11 },
    { 0xcc461e03, SUC_lexicon_tags_2 },
    { 0x4e661e04, SUC_lexicon_tags_11 },
    { 0x6e61e04, SUC_lexicon_tags_37 },
    { 0xe261e06, SUC_lexicon_tags_12 },
    { 0x75a61e06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9461e09, SUC_lexicon_tags_32 },
    { 0xcb661e0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfa261e0c, SUC_lexicon_tags_10 },
    { 0x7ec61e0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8261e10, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc861e16, SUC_lexicon_tags_17 },
    { 0xb661e17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a61e1c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce61e22, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xfc661e24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67661e28, SUC_lexicon_tags_60 },
    { 0xcca61e28, SUC_lexicon_tags_38 },
    { 0xe9c61e2a, SUC_lexicon_tags_4 },
    { 0x2e661e2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37461e30, SUC_lexicon_tags_28 },
    { 0x66261e30, SUC_lexicon_tags_87 },
    { 0x36261e30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48061e36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d261e39, SUC_lexicon_tags_49 },
    { 0xc461e3a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f061e46, SUC_lexicon_tags_2 },
    { 0x92061e47, SUC_lexicon_tags_37 },
    { 0xaaa61e47, SUC_lexicon_tags_55 },
    { 0x9aa61e49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e61e4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c61e51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc2661e53, SUC_lexicon_tags_37 },
    { 0xcec61e54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62461e58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe261e5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12261e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe861e5e, SUC_lexicon_tags_27 },
    { 0x1f861e5e, SUC_lexicon_tags_72 },
    { 0xc0e61e60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x45661e62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c61e66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d461e6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf661e71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a061e73, SUC_lexicon_tags_11 },
    { 0xfe661e73, SUC_lexicon_tags_17 },
    { 0xb0661e75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc61e79, SUC_lexicon_tags_2 },
    { 0xe9c61e7a, SUC_lexicon_tags_37 },
    { 0x8bc61e7b, SUC_lexicon_tags_39 },
    { 0x7e461e7c, SUC_lexicon_tags_53 },
    { 0xa7261e7d, SUC_lexicon_tags_17 },
    { 0xca461e7d, SUC_lexicon_tags_49 },
    { 0x7ac61e7d, SUC_lexicon_tags_17 },
    { 0x43261e7f, SUC_lexicon_tags_4 },
    { 0x19061e81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2861e84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a61e87, SUC_lexicon_tags_6 },
    { 0x13c61e87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d861e90, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a61e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80861e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec661ea0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a61ea4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49e61ea6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f061ead, SUC_lexicon_tags_8 },
    { 0x6ce61ead, SUC_lexicon_tags_57 },
    { 0xc261eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa61eb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b861eb5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c61eb8, SUC_lexicon_tags_14 },
    { 0xb3c61eb9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a61ebf, SUC_lexicon_tags_37 },
    { 0x42661ebf, SUC_lexicon_tags_1 },
    { 0xb2a61ec1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6861ec3, SUC_lexicon_tags_35 },
    { 0x70e61ec4, SUC_lexicon_tags_17 },
    { 0x5b861ec5, SUC_lexicon_tags_42 },
    { 0xf461ec6, SUC_lexicon_tags_15 },
    { 0x7b061ec6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a61eca, SUC_lexicon_tags_25 },
    { 0xf5661ecb, SUC_lexicon_tags_1 },
    { 0x70a61ecc, SUC_lexicon_tags_43 },
    { 0x88a61ecb, SUC_lexicon_tags_45 },
    { 0x4ae61ece, SUC_lexicon_tags_9 },
    { 0x35a61ecf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22461ed1, SUC_lexicon_tags_8 },
    { 0xcfa61ed2, SUC_lexicon_tags_19 },
    { 0xf061ed2, SUC_lexicon_tags_7 },
    { 0xdae61ed4, SUC_lexicon_tags_17 },
    { 0xdee61ed5, SUC_lexicon_tags_37 },
    { 0xa7a61ed2, SUC_lexicon_tags_40 },
    { 0x43661ed7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c61edc, SUC_lexicon_tags_23 },
    { 0x2c861edd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b061ee4, SUC_lexicon_tags_67 },
    { 0xf0e61ee5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8b261ee7, SUC_lexicon_tags_25 },
    { 0x65a61ee7, SUC_lexicon_tags_1 },
    { 0xe1861ee9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a61eed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda861ef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e61ef9, SUC_lexicon_tags_25 },
    { 0xa5e61ef9, SUC_lexicon_tags_2 },
    { 0x86261efb, SUC_lexicon_tags_2 },
    { 0x23661ef9, SUC_lexicon_tags_0 },
    { 0xa0461efc, SUC_lexicon_tags_19 },
    { 0x50461efe, SUC_lexicon_tags_6 },
    { 0x30861eff, SUC_lexicon_tags_37 },
    { 0xf6661eff, SUC_lexicon_tags_10 },
    { 0xfd461f01, SUC_lexicon_tags_7 },
    { 0x14e61f01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44861f06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f861f08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x91861f0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc61f0f, SUC_lexicon_tags_61 },
    { 0x92061f0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c61f14, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x19661f17, SUC_lexicon_tags_25 },
    { 0xedc61f18, SUC_lexicon_tags_12 },
    { 0x2dc61f19, SUC_lexicon_tags_39 },
    { 0x8ae61f19, SUC_lexicon_tags_11 },
    { 0x95261f1b, SUC_lexicon_tags_27 },
    { 0x88861f1a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x88461f1e, SUC_lexicon_tags_60 },
    { 0x94a61f1f, SUC_lexicon_tags_60 },
    { 0x7ce61f20, SUC_lexicon_tags_55 },
    { 0xecc61f1f, SUC_lexicon_tags_37 },
    { 0x20c61f22, SUC_lexicon_tags_41 },
    { 0x38461f23, SUC_lexicon_tags_64 },
    { 0xf9e61f24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83661f26, SUC_lexicon_tags_12 },
    { 0x2e861f27, SUC_lexicon_tags_48 },
    { 0xc9e61f27, SUC_lexicon_tags_27 },
    { 0x66461f29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4d261f2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea061f2e, SUC_lexicon_tags_63 },
    { 0xe1c61f2f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6661f33, SUC_lexicon_tags_7 },
    { 0x91061f34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c61f38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a61f42, SUC_lexicon_tags_9 },
    { 0xc5661f43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5861f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0061f48, SUC_lexicon_tags_2 },
    { 0x25061f48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c61f4c, SUC_lexicon_tags_114 },
    { 0xa3661f4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a61f51, SUC_lexicon_tags_60 },
    { 0x8e61f52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc061f55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf661f57, SUC_lexicon_tags_116 },
    { 0x40061f57, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a61f5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81861f60, SUC_lexicon_tags_2 },
    { 0x5de61f61, SUC_lexicon_tags_17 },
    { 0x83a61f62, SUC_lexicon_tags_38 },
    { 0x20061f63, SUC_lexicon_tags_12 },
    { 0x72061f61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82861f68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb261f6e, SUC_lexicon_tags_0 },
    { 0xfb861f6e, SUC_lexicon_tags_12 },
    { 0xa7061f70, SUC_lexicon_tags_19 },
    { 0xb1e61f71, SUC_lexicon_tags_1 },
    { 0x17661f71, SUC_lexicon_tags_1 },
    { 0x2ea61f73, SUC_lexicon_tags_60 },
    { 0x4c061f71, SUC_lexicon_tags_37 },
    { 0xd9661f71, SUC_lexicon_tags_23 },
    { 0x17c61f74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75061f78, SUC_lexicon_tags_31 },
    { 0xc6061f79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1861f7b, SUC_lexicon_tags_27 },
    { 0x66e61f7c, SUC_lexicon_tags_0 },
    { 0x4b261f7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca61f80, SUC_lexicon_tags_37 },
    { 0xe5e61f81, SUC_lexicon_tags_7 },
    { 0x77261f82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x96e61f84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c61f88, SUC_lexicon_tags_37 },
    { 0x53c61f88, SUC_lexicon_tags_7 },
    { 0x96e61f88, SUC_lexicon_tags_7 },
    { 0x48a61f8a, SUC_lexicon_tags_15 },
    { 0xf7861f88, SUC_lexicon_tags_23 },
    { 0xd4e61f89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f061f8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2261f91, SUC_lexicon_tags_27 },
    { 0x85461f92, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49261f9b, SUC_lexicon_tags_1 },
    { 0x33061f9c, SUC_lexicon_tags_25 },
    { 0xe7a61f9d, SUC_lexicon_tags_5 },
    { 0x30e61f9c, SUC_lexicon_tags_27 },
    { 0xb0e61f9f, SUC_lexicon_tags_1 },
    { 0xa7a61f9b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24661fa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f461fa7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82461fad, SUC_lexicon_tags_3 },
    { 0x4d861fae, SUC_lexicon_tags_17 },
    { 0xde661fae, SUC_lexicon_tags_184 },
    { 0x8aa61fb0, SUC_lexicon_tags_17 },
    { 0xa3c61fad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a61fb4, SUC_lexicon_tags_25 },
    { 0xab461fb4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1e61fb7, SUC_lexicon_tags_49 },
    { 0x46e61fb8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f461fbc, SUC_lexicon_tags_37 },
    { 0x64c61fbd, SUC_lexicon_tags_19 },
    { 0x7da61fbe, SUC_lexicon_tags_25 },
    { 0xad861fbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e261fc2, SUC_lexicon_tags_1 },
    { 0x73e61fc3, SUC_lexicon_tags_34 },
    { 0x84a61fc3, SUC_lexicon_tags_47 },
    { 0xfd461fc5, SUC_lexicon_tags_63 },
    { 0x35e61fc6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51661fcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c61fda, SUC_lexicon_tags_87 },
    { 0x6b861fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42061fdf, SUC_lexicon_tags_19 },
    { 0xfba61fe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72461fe3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe3061fe5, SUC_lexicon_tags_129 },
    { 0x26861fe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3061feb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a61fef, SUC_lexicon_tags_37 },
    { 0x4a861fef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x93a61ff2, SUC_lexicon_tags_16 },
    { 0x49e61ff2, SUC_lexicon_tags_27 },
    { 0x99c61ff4, SUC_lexicon_tags_10 },
    { 0x5d261ff5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x70061ff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5661ffb, SUC_lexicon_tags_48 },
    { 0x56261ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89e61ffe, SUC_lexicon_tags_27 },
    { 0xeba61fff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a62004, SUC_lexicon_tags_12 },
    { 0x77862004, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa5262007, SUC_lexicon_tags_25 },
    { 0xafa62008, SUC_lexicon_tags_26 },
    { 0x97662008, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32662013, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe6262015, SUC_lexicon_tags_46 },
    { 0xb7e62016, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e6201b, SUC_lexicon_tags_19 },
    { 0xa426201c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b66201f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc62022, SUC_lexicon_tags_17 },
    { 0xec862023, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16662025, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x18062028, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa206202d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf246202f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc262034, SUC_lexicon_tags_25 },
    { 0xd3862035, SUC_lexicon_tags_25 },
    { 0xce862035, SUC_lexicon_tags_6 },
    { 0x1d262037, SUC_lexicon_tags_2 },
    { 0xb0262038, SUC_lexicon_tags_47 },
    { 0xf2062039, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb266203f, SUC_lexicon_tags_2 },
    { 0x84e62040, SUC_lexicon_tags_37 },
    { 0xa8c62041, SUC_lexicon_tags_30 },
    { 0x38462042, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8262046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec6204a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba6204f, SUC_lexicon_tags_39 },
    { 0x9be62050, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x80662052, SUC_lexicon_tags_39 },
    { 0x48462052, SUC_lexicon_tags_25 },
    { 0xd9262054, SUC_lexicon_tags_60 },
    { 0x92662054, SUC_lexicon_tags_77 },
    { 0xe9c62055, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c6205c, SUC_lexicon_tags_12 },
    { 0xa706205d, SUC_lexicon_tags_2 },
    { 0x5506205d, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x73c62060, SUC_lexicon_tags_27 },
    { 0x99062060, SUC_lexicon_tags_11 },
    { 0x8ac62061, SUC_lexicon_tags_49 },
    { 0x2662063, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f062067, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6206a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a6206f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd5c62071, SUC_lexicon_tags_7 },
    { 0x13a62071, SUC_lexicon_tags_25 },
    { 0xc2262071, SUC_lexicon_tags_35 },
    { 0xf4662071, SUC_lexicon_tags_17 },
    { 0x90662071, SUC_lexicon_tags_31 },
    { 0xcb862076, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8946207a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e86207d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd062080, SUC_lexicon_tags_15 },
    { 0xcc62080, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be6208b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfea6208d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb006208f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94862092, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c62094, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaea62096, SUC_lexicon_tags_1 },
    { 0x41462097, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x78662099, SUC_lexicon_tags_33 },
    { 0xb5a6209a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec6209d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe526209f, SUC_lexicon_tags_7 },
    { 0xbd86209f, SUC_lexicon_tags_11 },
    { 0xd6c6209f, SUC_lexicon_tags_43 },
    { 0x2cc6209f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678620a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfac620a9, SUC_lexicon_tags_14 },
    { 0x784620a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88620ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9e620af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf06620b1, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x850620b3, SUC_lexicon_tags_11 },
    { 0xbfc620b3, SUC_lexicon_tags_25 },
    { 0x882620b5, SUC_lexicon_tags_37 },
    { 0x630620b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a620ba, SUC_lexicon_tags_25 },
    { 0x402620bb, SUC_lexicon_tags_1 },
    { 0x5fa620bc, SUC_lexicon_tags_36 },
    { 0x97c620bd, SUC_lexicon_tags_37 },
    { 0xfdc620bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x15c620c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x296620c3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb16620c5, SUC_lexicon_tags_2 },
    { 0x80620c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6e6620c8, SUC_lexicon_tags_37 },
    { 0xbfe620c8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442620cd, SUC_lexicon_tags_17 },
    { 0x81c620ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6620d1, SUC_lexicon_tags_15 },
    { 0xe3c620d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc2620d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6620d9, SUC_lexicon_tags_7 },
    { 0x666620da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34620e0, SUC_lexicon_tags_7 },
    { 0xa38620e1, SUC_lexicon_tags_28 },
    { 0xfc4620e2, SUC_lexicon_tags_15 },
    { 0x858620e0, SUC_lexicon_tags_57 },
    { 0x17a620e4, SUC_lexicon_tags_12 },
    { 0xcda620e1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2620eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd78620ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64620f2, SUC_lexicon_tags_243 },
    { 0x216620f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288620fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94e620fc, SUC_lexicon_tags_21 },
    { 0x54a620fd, SUC_lexicon_tags_25 },
    { 0x728620fd, SUC_lexicon_tags_31 },
    { 0xf00620fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe262102, SUC_lexicon_tags_25 },
    { 0xeb662103, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d062105, SUC_lexicon_tags_11 },
    { 0x43662106, SUC_lexicon_tags_7 },
    { 0x6de62106, SUC_lexicon_tags_27 },
    { 0xc3c62108, SUC_lexicon_tags_17 },
    { 0xa8a62109, SUC_lexicon_tags_18 },
    { 0x6d662108, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcf46210c, SUC_lexicon_tags_109 },
    { 0x5206210d, SUC_lexicon_tags_17 },
    { 0xf186210e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ba62110, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e62113, SUC_lexicon_tags_14 },
    { 0x26862113, SUC_lexicon_tags_34 },
    { 0x34262115, SUC_lexicon_tags_47 },
    { 0xaec62116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47062118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e6211a, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0x3326211c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c46211f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c62124, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7a062126, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4486212d, SUC_lexicon_tags_243 },
    { 0xee46212e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49462133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41e62135, SUC_lexicon_tags_23 },
    { 0x83a62136, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f66213c, SUC_lexicon_tags_11 },
    { 0x2ec6213d, SUC_lexicon_tags_6 },
    { 0x90e6213d, SUC_lexicon_tags_13 },
    { 0x7a86213c, SUC_lexicon_tags_11 },
    { 0x6b46213e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e62143, SUC_lexicon_tags_19 },
    { 0x48662144, SUC_lexicon_tags_26 },
    { 0xefe62145, SUC_lexicon_tags_15 },
    { 0xb9462146, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce62149, SUC_lexicon_tags_8 },
    { 0xc6262149, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5262159, SUC_lexicon_tags_11 },
    { 0xe6c62159, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae62160, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x18062162, SUC_lexicon_tags_46 },
    { 0x51862163, SUC_lexicon_tags_86 },
    { 0x51062163, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf862167, SUC_lexicon_tags_60 },
    { 0x2ea62168, SUC_lexicon_tags_11 },
    { 0xd9662168, SUC_lexicon_tags_19 },
    { 0x5546216a, SUC_lexicon_tags_19 },
    { 0xd4e6216b, SUC_lexicon_tags_62 },
    { 0x9f66216c, SUC_lexicon_tags_257 },
    { 0xe7862168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd062171, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x60262173, SUC_lexicon_tags_1 },
    { 0x19a62174, SUC_lexicon_tags_17 },
    { 0xbb662175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96262178, SUC_lexicon_tags_13 },
    { 0xdaa62179, SUC_lexicon_tags_12 },
    { 0x4de6217a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfbe6217c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd006217f, SUC_lexicon_tags_66 },
    { 0x2e662180, SUC_lexicon_tags_37 },
    { 0xe746217f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35662186, SUC_lexicon_tags_17 },
    { 0x28462187, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa5062189, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26062196, SUC_lexicon_tags_37 },
    { 0xa9c62197, SUC_lexicon_tags_17 },
    { 0x8f462198, SUC_lexicon_tags_17 },
    { 0xc7462198, SUC_lexicon_tags_17 },
    { 0xd062198, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe66219c, SUC_lexicon_tags_1 },
    { 0xfa46219c, SUC_lexicon_tags_32 },
    { 0xb7e6219d, SUC_lexicon_tags_61 },
    { 0x77c6219f, SUC_lexicon_tags_1 },
    { 0x95a6219e, SUC_lexicon_tags_2 },
    { 0xe7e6219d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00621a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c621a7, SUC_lexicon_tags_27 },
    { 0x42c621a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x330621ab, SUC_lexicon_tags_46 },
    { 0x5e6621ab, SUC_lexicon_tags_44 },
    { 0x33e621ad, SUC_lexicon_tags_15 },
    { 0x992621ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a621b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670621b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c621bd, SUC_lexicon_tags_45 },
    { 0xf1e621be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c621c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x838621c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9b8621c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2621ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2621ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce621d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334621e2, SUC_lexicon_tags_17 },
    { 0x68c621e3, SUC_lexicon_tags_17 },
    { 0x46621e4, SUC_lexicon_tags_12 },
    { 0xa7e621e5, SUC_lexicon_tags_49 },
    { 0xf38621e6, SUC_lexicon_tags_17 },
    { 0xf36621e2, SUC_lexicon_tags_32 },
    { 0xbe4621e4, SUC_lexicon_tags_23 },
    { 0x2fa621e9, SUC_lexicon_tags_19 },
    { 0xe36621ea, SUC_lexicon_tags_7 },
    { 0x31a621eb, SUC_lexicon_tags_38 },
    { 0xa56621ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de621ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524621f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86c621f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46621f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960621fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ac62200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1e62202, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35462207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa206220c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2462216, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd062219, SUC_lexicon_tags_4 },
    { 0xfee62219, SUC_lexicon_tags_77 },
    { 0xdb46221b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3062223, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89262226, SUC_lexicon_tags_37 },
    { 0xb6262227, SUC_lexicon_tags_2 },
    { 0xcb062228, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c62230, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x68462233, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba662239, SUC_lexicon_tags_25 },
    { 0x5d86223a, SUC_lexicon_tags_14 },
    { 0x5a06223b, SUC_lexicon_tags_15 },
    { 0x84c62239, SUC_lexicon_tags_37 },
    { 0x786223d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc62241, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c62249, SUC_lexicon_tags_12 },
    { 0xce66224a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31a6224c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9262252, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a62257, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d462260, SUC_lexicon_tags_2 },
    { 0xf8862261, SUC_lexicon_tags_7 },
    { 0x73462262, SUC_lexicon_tags_25 },
    { 0xdda62263, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd262265, SUC_lexicon_tags_15 },
    { 0xcf662266, SUC_lexicon_tags_11 },
    { 0x45662267, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x71262269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3046226c, SUC_lexicon_tags_1 },
    { 0xe126226d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e62272, SUC_lexicon_tags_14 },
    { 0x662273, SUC_lexicon_tags_37 },
    { 0xd1a62274, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e262277, SUC_lexicon_tags_2 },
    { 0x46262277, SUC_lexicon_tags_2 },
    { 0x46262278, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a6227c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b86227f, SUC_lexicon_tags_14 },
    { 0x7926227f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a62283, SUC_lexicon_tags_14 },
    { 0x71262284, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7462287, SUC_lexicon_tags_10 },
    { 0x9bc62287, SUC_lexicon_tags_2 },
    { 0xd5862289, SUC_lexicon_tags_11 },
    { 0x8f862289, SUC_lexicon_tags_17 },
    { 0xeec6228b, SUC_lexicon_tags_11 },
    { 0x41a6228c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc62294, SUC_lexicon_tags_12 },
    { 0x5062295, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x46862297, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc806229a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2046229c, SUC_lexicon_tags_7 },
    { 0x94a6229d, SUC_lexicon_tags_31 },
    { 0x7dc6229d, SUC_lexicon_tags_2 },
    { 0x2526229e, SUC_lexicon_tags_14 },
    { 0x6b86229f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798622a9, SUC_lexicon_tags_17 },
    { 0x1ea622aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ac622ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324622b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e622b3, SUC_lexicon_tags_1 },
    { 0x782622b4, SUC_lexicon_tags_1 },
    { 0x47e622b5, SUC_lexicon_tags_17 },
    { 0x78622b6, SUC_lexicon_tags_2 },
    { 0xd60622b7, SUC_lexicon_tags_11 },
    { 0x4f4622b6, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8622bc, SUC_lexicon_tags_37 },
    { 0x858622bd, SUC_lexicon_tags_60 },
    { 0xa46622be, SUC_lexicon_tags_0 },
    { 0xb38622bf, SUC_lexicon_tags_33 },
    { 0x2d4622c0, SUC_lexicon_tags_38 },
    { 0x87c622c1, SUC_lexicon_tags_387 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc622c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbfc622c6, SUC_lexicon_tags_12 },
    { 0xaa2622c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x94622c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa622cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bc622cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18622d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa16622d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c622d9, SUC_lexicon_tags_39 },
    { 0xede622d9, SUC_lexicon_tags_25 },
    { 0xfbe622d9, SUC_lexicon_tags_29 },
    { 0x898622dc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2622e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2622e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8622eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x244622ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22622f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc622f6, SUC_lexicon_tags_43 },
    { 0x472622f7, SUC_lexicon_tags_12 },
    { 0x3a8622f6, SUC_lexicon_tags_2 },
    { 0x7e0622f9, SUC_lexicon_tags_19 },
    { 0x4e0622fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda622fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d862306, SUC_lexicon_tags_2 },
    { 0xd6262307, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13662309, SUC_lexicon_tags_1 },
    { 0xe6a62309, SUC_lexicon_tags_12 },
    { 0x2586230b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc62311, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ca62313, SUC_lexicon_tags_29 },
    { 0x6bc62313, SUC_lexicon_tags_45 },
    { 0xba262315, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a62319, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e6231e, SUC_lexicon_tags_56 },
    { 0x3546231f, SUC_lexicon_tags_17 },
    { 0x22e62320, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec62325, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xabc62327, SUC_lexicon_tags_230 },
    { 0x20062328, SUC_lexicon_tags_82 },
    { 0x66a62329, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1686232c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d66232f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43462333, SUC_lexicon_tags_6 },
    { 0xf5462333, SUC_lexicon_tags_25 },
    { 0xb3062334, SUC_lexicon_tags_37 },
    { 0x5f462335, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49862338, SUC_lexicon_tags_15 },
    { 0x90e62339, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5fc6233b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf006233d, SUC_lexicon_tags_184 },
    { 0x3ea6233d, SUC_lexicon_tags_17 },
    { 0xab26233f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69462343, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5c62345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea62348, SUC_lexicon_tags_2 },
    { 0x10862349, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4446234b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf106234e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e62354, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf862356, SUC_lexicon_tags_17 },
    { 0x98062357, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b86235a, SUC_lexicon_tags_2 },
    { 0x586235b, SUC_lexicon_tags_15 },
    { 0x4786235a, SUC_lexicon_tags_15 },
    { 0x826235d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc62360, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a062365, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b662367, SUC_lexicon_tags_22 },
    { 0x6d262368, SUC_lexicon_tags_39 },
    { 0x9c062369, SUC_lexicon_tags_4 },
    { 0xe1c6236a, SUC_lexicon_tags_6 },
    { 0xcfc6236b, SUC_lexicon_tags_0 },
    { 0x3866236c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f462371, SUC_lexicon_tags_17 },
    { 0xade62372, SUC_lexicon_tags_2 },
    { 0x26a62373, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x22462376, SUC_lexicon_tags_12 },
    { 0x37c62376, SUC_lexicon_tags_19 },
    { 0x31262378, SUC_lexicon_tags_49 },
    { 0x37262379, SUC_lexicon_tags_37 },
    { 0x5ea6237a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5d46237c, SUC_lexicon_tags_2 },
    { 0x3166237d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e62383, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c62386, SUC_lexicon_tags_13 },
    { 0xf2e62386, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc2a62389, SUC_lexicon_tags_30 },
    { 0xc466238a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff662391, SUC_lexicon_tags_3 },
    { 0x5bc62392, SUC_lexicon_tags_34 },
    { 0xf7e62393, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c6239a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5e06239c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678623a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18623a7, SUC_lexicon_tags_1 },
    { 0xa00623a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26623aa, SUC_lexicon_tags_37 },
    { 0xeae623aa, SUC_lexicon_tags_41 },
    { 0xc4a623ab, SUC_lexicon_tags_31 },
    { 0x760623ad, SUC_lexicon_tags_49 },
    { 0xa30623ae, SUC_lexicon_tags_29 },
    { 0xefa623ab, SUC_lexicon_tags_17 },
    { 0xec0623b0, SUC_lexicon_tags_11 },
    { 0x5f2623b0, SUC_lexicon_tags_25 },
    { 0x1dc623b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252623ba, SUC_lexicon_tags_7 },
    { 0x98a623ba, SUC_lexicon_tags_12 },
    { 0x342623bc, SUC_lexicon_tags_16 },
    { 0x3f8623bd, SUC_lexicon_tags_27 },
    { 0xa56623be, SUC_lexicon_tags_2 },
    { 0x580623bd, SUC_lexicon_tags_15 },
    { 0x4d2623be, SUC_lexicon_tags_39 },
    { 0xb46623c1, SUC_lexicon_tags_2 },
    { 0x34a623c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf66623c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0623c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x310623ca, SUC_lexicon_tags_22 },
    { 0x830623cb, SUC_lexicon_tags_37 },
    { 0x1de623ca, SUC_lexicon_tags_17 },
    { 0xee6623cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc623d2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe623d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384623da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe64623dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe623e1, SUC_lexicon_tags_9 },
    { 0xfc4623e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e623e5, SUC_lexicon_tags_17 },
    { 0x1ca623e5, SUC_lexicon_tags_14 },
    { 0xdd2623e6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd18623e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11c623eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9dc623ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x132623ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a623f5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x152623f7, SUC_lexicon_tags_46 },
    { 0x814623f8, SUC_lexicon_tags_7 },
    { 0x3a623f8, SUC_lexicon_tags_16 },
    { 0xd00623f9, SUC_lexicon_tags_47 },
    { 0x4c4623f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b0623fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec62404, SUC_lexicon_tags_25 },
    { 0x2c862405, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb566240a, SUC_lexicon_tags_14 },
    { 0x5b06240b, SUC_lexicon_tags_2 },
    { 0x7c66240c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7a86240e, SUC_lexicon_tags_60 },
    { 0x1d06240f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40e62411, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xba62413, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x98e62415, SUC_lexicon_tags_25 },
    { 0xf7e62416, SUC_lexicon_tags_37 },
    { 0x12a62417, SUC_lexicon_tags_0 },
    { 0x84a62418, SUC_lexicon_tags_17 },
    { 0xc5262417, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6046241c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27062424, SUC_lexicon_tags_38 },
    { 0xd462425, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a6242b, SUC_lexicon_tags_7 },
    { 0x9d86242c, SUC_lexicon_tags_6 },
    { 0x2806242d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec62430, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b262437, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1346243c, SUC_lexicon_tags_151 },
    { 0x6286243d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e62441, SUC_lexicon_tags_17 },
    { 0x56662441, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e62445, SUC_lexicon_tags_63 },
    { 0x36e62446, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34862449, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9306244b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde6244d, SUC_lexicon_tags_38 },
    { 0x4ac6244d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xaac62450, SUC_lexicon_tags_17 },
    { 0x9e862450, SUC_lexicon_tags_60 },
    { 0x45062451, SUC_lexicon_tags_27 },
    { 0xc8862453, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbde62455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdce62457, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea6245b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea6245f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4aa62461, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3e62463, SUC_lexicon_tags_11 },
    { 0x96a62463, SUC_lexicon_tags_61 },
    { 0x86062464, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfc662467, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ea62469, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f46246d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1262472, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3662475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c062479, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2262480, SUC_lexicon_tags_14 },
    { 0xc7662481, SUC_lexicon_tags_27 },
    { 0x2ce62482, SUC_lexicon_tags_30 },
    { 0xa0662480, SUC_lexicon_tags_37 },
    { 0x39e62484, SUC_lexicon_tags_70 },
    { 0x2f862485, SUC_lexicon_tags_91 },
    { 0xbbc62482, SUC_lexicon_tags_12 },
    { 0x21e62483, SUC_lexicon_tags_70 },
    { 0x68862488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe366248a, SUC_lexicon_tags_57 },
    { 0x6106248b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a62492, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c62495, SUC_lexicon_tags_12 },
    { 0xa2262496, SUC_lexicon_tags_37 },
    { 0x78e62495, SUC_lexicon_tags_1 },
    { 0x65462496, SUC_lexicon_tags_13 },
    { 0x10262499, SUC_lexicon_tags_107 },
    { 0x9ee62499, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5166249d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x826249f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a624a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a624a5, SUC_lexicon_tags_8 },
    { 0xffc624a5, SUC_lexicon_tags_21 },
    { 0x49a624a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52e624a9, SUC_lexicon_tags_19 },
    { 0xc3c624aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2624ac, SUC_lexicon_tags_27 },
    { 0x5f8624ac, SUC_lexicon_tags_1 },
    { 0x16624ae, SUC_lexicon_tags_1 },
    { 0x6fe624ac, SUC_lexicon_tags_16 },
    { 0xdc624ae, SUC_lexicon_tags_56 },
    { 0xcfc624af, SUC_lexicon_tags_9 },
    { 0x75a624b2, SUC_lexicon_tags_37 },
    { 0x3a0624ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04624b7, SUC_lexicon_tags_37 },
    { 0x184624b8, SUC_lexicon_tags_26 },
    { 0x910624b9, SUC_lexicon_tags_8 },
    { 0x822624ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x180624bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x356624be, SUC_lexicon_tags_11 },
    { 0x150624bf, SUC_lexicon_tags_9 },
    { 0x3a4624c0, SUC_lexicon_tags_158 },
    { 0x70c624bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a624c6, SUC_lexicon_tags_49 },
    { 0xec8624c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2624ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x946624cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c624d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a624d4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c624d8, SUC_lexicon_tags_11 },
    { 0x288624d8, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0x98624dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c624e6, SUC_lexicon_tags_9 },
    { 0x40624e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a624f0, SUC_lexicon_tags_1 },
    { 0xd4e624f0, SUC_lexicon_tags_49 },
    { 0x350624f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x892624f4, SUC_lexicon_tags_15 },
    { 0xf34624f5, SUC_lexicon_tags_2 },
    { 0x52a624f5, SUC_lexicon_tags_14 },
    { 0xd2a624f6, SUC_lexicon_tags_19 },
    { 0xf86624f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6624fc, SUC_lexicon_tags_26 },
    { 0xad8624fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b062500, SUC_lexicon_tags_14 },
    { 0xb862501, SUC_lexicon_tags_92 },
    { 0x58262502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee862506, SUC_lexicon_tags_151 },
    { 0xbe462506, SUC_lexicon_tags_15 },
    { 0x5462508, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f86250d, SUC_lexicon_tags_1 },
    { 0xe4a6250e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c62511, SUC_lexicon_tags_49 },
    { 0x4b062511, SUC_lexicon_tags_2 },
    { 0x75262513, SUC_lexicon_tags_17 },
    { 0xdd662514, SUC_lexicon_tags_17 },
    { 0xa2262512, SUC_lexicon_tags_29 },
    { 0xcc62516, SUC_lexicon_tags_11 },
    { 0x43662515, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63a62519, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad66251b, SUC_lexicon_tags_37 },
    { 0x46c6251b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a62520, SUC_lexicon_tags_22 },
    { 0xcbc62521, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea662525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a462528, SUC_lexicon_tags_92 },
    { 0xa7262528, SUC_lexicon_tags_25 },
    { 0xfc46252a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xade6252c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e46252e, SUC_lexicon_tags_11 },
    { 0xe166252e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc62539, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc386253d, SUC_lexicon_tags_65 },
    { 0x19e6253e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa862543, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5462545, SUC_lexicon_tags_11 },
    { 0xc2c62546, SUC_lexicon_tags_14 },
    { 0xf262545, SUC_lexicon_tags_37 },
    { 0xf1462548, SUC_lexicon_tags_23 },
    { 0xaf062546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92c6254b, SUC_lexicon_tags_33 },
    { 0x2f26254c, SUC_lexicon_tags_11 },
    { 0x4166254c, SUC_lexicon_tags_37 },
    { 0xa466254d, SUC_lexicon_tags_37 },
    { 0x3c26254f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e262553, SUC_lexicon_tags_12 },
    { 0x32e62554, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27462558, SUC_lexicon_tags_40 },
    { 0xb6c62558, SUC_lexicon_tags_25 },
    { 0xcdc62559, SUC_lexicon_tags_38 },
    { 0x6ee6255b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f26255d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1706255f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf262565, SUC_lexicon_tags_38 },
    { 0x33062566, SUC_lexicon_tags_11 },
    { 0xeb062566, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a462569, SUC_lexicon_tags_15 },
    { 0x12662569, SUC_lexicon_tags_32 },
    { 0xac86256b, SUC_lexicon_tags_2 },
    { 0xb006256c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf506256e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x31462571, SUC_lexicon_tags_12 },
    { 0x63862572, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2862575, SUC_lexicon_tags_30 },
    { 0x8b462576, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e62579, SUC_lexicon_tags_25 },
    { 0xe066257a, SUC_lexicon_tags_8 },
    { 0x3c662579, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac062581, SUC_lexicon_tags_7 },
    { 0x6d662581, SUC_lexicon_tags_18 },
    { 0x19e62583, SUC_lexicon_tags_37 },
    { 0x6be62582, SUC_lexicon_tags_26 },
    { 0xf3e62585, SUC_lexicon_tags_1143 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9326258a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf626258c, SUC_lexicon_tags_15 },
    { 0x4086258c, SUC_lexicon_tags_13 },
    { 0xf006258e, SUC_lexicon_tags_55 },
    { 0xaa26258e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c62592, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2862596, SUC_lexicon_tags_1 },
    { 0x9ec62596, SUC_lexicon_tags_26 },
    { 0x73c62596, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x64a6259a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1986259d, SUC_lexicon_tags_30 },
    { 0x5f06259d, SUC_lexicon_tags_27 },
    { 0x70c6259d, SUC_lexicon_tags_15 },
    { 0xa8e6259d, SUC_lexicon_tags_45 },
    { 0xfe2625a1, SUC_lexicon_tags_48 },
    { 0xbd8625a1, SUC_lexicon_tags_7 },
    { 0x7b6625a1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14625a8, SUC_lexicon_tags_38 },
    { 0x184625a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822625b0, SUC_lexicon_tags_17 },
    { 0x6ba625b1, SUC_lexicon_tags_70 },
    { 0xae6625b1, SUC_lexicon_tags_19 },
    { 0x2b4625b1, SUC_lexicon_tags_70 },
    { 0x30e625b4, SUC_lexicon_tags_2 },
    { 0x11c625b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8625bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa32625bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2625c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332625cc, SUC_lexicon_tags_38 },
    { 0xbdc625cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a625d0, SUC_lexicon_tags_624 },
    { 0x4a4625d1, SUC_lexicon_tags_446 },
    { 0xd62625d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48625d7, SUC_lexicon_tags_49 },
    { 0x44a625d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb20625da, SUC_lexicon_tags_17 },
    { 0x50625db, SUC_lexicon_tags_49 },
    { 0x522625dc, SUC_lexicon_tags_11 },
    { 0xeda625db, SUC_lexicon_tags_12 },
    { 0xaf6625de, SUC_lexicon_tags_25 },
    { 0x74625de, SUC_lexicon_tags_42 },
    { 0x680625db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb0625e2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4625e5, SUC_lexicon_tags_1 },
    { 0xbba625e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a625f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xec6625f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef6625f5, SUC_lexicon_tags_17 },
    { 0x224625f6, SUC_lexicon_tags_64 },
    { 0x95e625f7, SUC_lexicon_tags_65 },
    { 0x498625f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1462600, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e462602, SUC_lexicon_tags_31 },
    { 0x97c62603, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x72262606, SUC_lexicon_tags_23 },
    { 0x54262606, SUC_lexicon_tags_6 },
    { 0x62862608, SUC_lexicon_tags_37 },
    { 0x62c62609, SUC_lexicon_tags_60 },
    { 0x3c62609, SUC_lexicon_tags_12 },
    { 0xaac62609, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x59c6260d, SUC_lexicon_tags_15 },
    { 0xb866260d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7462613, SUC_lexicon_tags_95 },
    { 0x94262614, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f862619, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe526261b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4e6261d, SUC_lexicon_tags_25 },
    { 0x9006261e, SUC_lexicon_tags_25 },
    { 0x4906261e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x10462621, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa6262b, SUC_lexicon_tags_7 },
    { 0xa446262b, SUC_lexicon_tags_34 },
    { 0xe086262d, SUC_lexicon_tags_56 },
    { 0x8286262e, SUC_lexicon_tags_48 },
    { 0x4fc6262b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16062631, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc462637, SUC_lexicon_tags_23 },
    { 0x95862638, SUC_lexicon_tags_43 },
    { 0xe462638, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84a6263b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa106263d, SUC_lexicon_tags_7 },
    { 0x5cc6263e, SUC_lexicon_tags_15 },
    { 0x9086263f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x48a62641, SUC_lexicon_tags_17 },
    { 0xbf262642, SUC_lexicon_tags_6 },
    { 0x34662642, SUC_lexicon_tags_55 },
    { 0x7cc62643, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e62647, SUC_lexicon_tags_61 },
    { 0xb462648, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd386264d, SUC_lexicon_tags_63 },
    { 0xb8a6264e, SUC_lexicon_tags_15 },
    { 0xcbe6264f, SUC_lexicon_tags_19 },
    { 0xa166264f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe1062652, SUC_lexicon_tags_47 },
    { 0x9662653, SUC_lexicon_tags_26 },
    { 0x70c62654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6466265a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd26265d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x86862660, SUC_lexicon_tags_43 },
    { 0xcc262661, SUC_lexicon_tags_7 },
    { 0x6ee62662, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e62666, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a62669, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c6266d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbca6266f, SUC_lexicon_tags_19 },
    { 0x55a62670, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1262673, SUC_lexicon_tags_34 },
    { 0x98462673, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x29462677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4906267a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e6267d, SUC_lexicon_tags_2 },
    { 0xb766267d, SUC_lexicon_tags_11 },
    { 0x9f26267f, SUC_lexicon_tags_2 },
    { 0xe4a62680, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4662685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7062688, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf786268a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d66268c, SUC_lexicon_tags_33 },
    { 0xe526268d, SUC_lexicon_tags_17 },
    { 0x72a6268e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8662693, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0x75a62695, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0462698, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc6269e, SUC_lexicon_tags_7 },
    { 0xc66269f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2626a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb16626a4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x752626a6, SUC_lexicon_tags_43 },
    { 0x2626a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c626ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa3c626ad, SUC_lexicon_tags_1 },
    { 0x1a8626ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770626b2, SUC_lexicon_tags_27 },
    { 0xf50626b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22626b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd2626b8, SUC_lexicon_tags_26 },
    { 0x198626b8, SUC_lexicon_tags_65 },
    { 0x1de626b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60626c0, SUC_lexicon_tags_35 },
    { 0x6a0626c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6626c6, SUC_lexicon_tags_63 },
    { 0xeb8626c7, SUC_lexicon_tags_47 },
    { 0xcec626c8, SUC_lexicon_tags_9 },
    { 0xb4c626c9, SUC_lexicon_tags_15 },
    { 0x3d8626ca, SUC_lexicon_tags_25 },
    { 0x240626ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752626d0, SUC_lexicon_tags_15 },
    { 0xaba626d0, SUC_lexicon_tags_25 },
    { 0x9ae626d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16626d5, SUC_lexicon_tags_11 },
    { 0x6be626d6, SUC_lexicon_tags_45 },
    { 0x8fc626d7, SUC_lexicon_tags_389 },
    { 0x504626d8, SUC_lexicon_tags_15 },
    { 0x8a4626d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c6626db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0626e1, SUC_lexicon_tags_6 },
    { 0xc36626e1, SUC_lexicon_tags_2 },
    { 0x756626e3, SUC_lexicon_tags_12 },
    { 0x1bc626e4, SUC_lexicon_tags_44 },
    { 0x4f0626e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4626e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac626eb, SUC_lexicon_tags_1 },
    { 0xbf4626ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c626f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6a626f2, SUC_lexicon_tags_1 },
    { 0xfe8626f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7d2626f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x974626f8, SUC_lexicon_tags_26 },
    { 0x30c626f9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xca0626fb, SUC_lexicon_tags_43 },
    { 0x8ca626fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c62704, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c662709, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e6270d, SUC_lexicon_tags_37 },
    { 0xa3a6270e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c62712, SUC_lexicon_tags_8 },
    { 0x56c62713, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8b262715, SUC_lexicon_tags_47 },
    { 0x75862715, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe586271b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd66271f, SUC_lexicon_tags_11 },
    { 0x80662720, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d262724, SUC_lexicon_tags_11 },
    { 0xea862724, SUC_lexicon_tags_37 },
    { 0xf3e62724, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4226272f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5062734, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4862738, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd826273a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a6273d, SUC_lexicon_tags_15 },
    { 0xd26273d, SUC_lexicon_tags_18 },
    { 0xf386273f, SUC_lexicon_tags_37 },
    { 0x9d06273e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e62743, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd2262745, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd46274c, SUC_lexicon_tags_16 },
    { 0xb446274d, SUC_lexicon_tags_60 },
    { 0xb66274e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf9062750, SUC_lexicon_tags_42 },
    { 0xf6062751, SUC_lexicon_tags_17 },
    { 0x5fe62752, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206275b, SUC_lexicon_tags_40 },
    { 0xdbe6275b, SUC_lexicon_tags_16 },
    { 0xc746275d, SUC_lexicon_tags_29 },
    { 0xb446275d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae062760, SUC_lexicon_tags_7 },
    { 0xa7462760, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8862769, SUC_lexicon_tags_49 },
    { 0x99c6276a, SUC_lexicon_tags_7 },
    { 0x31e6276b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd826276f, SUC_lexicon_tags_1 },
    { 0x23462770, SUC_lexicon_tags_9 },
    { 0x5fe62770, SUC_lexicon_tags_17 },
    { 0xabe62772, SUC_lexicon_tags_15 },
    { 0x9dc62773, SUC_lexicon_tags_30 },
    { 0xefa62774, SUC_lexicon_tags_37 },
    { 0x2262775, SUC_lexicon_tags_31 },
    { 0x79c62776, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5866277b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86277e, SUC_lexicon_tags_17 },
    { 0x5dc6277f, SUC_lexicon_tags_37 },
    { 0x5a06277e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54662785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e62788, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2262790, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0xc1462792, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe62796, SUC_lexicon_tags_25 },
    { 0x43662796, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd906279b, SUC_lexicon_tags_25 },
    { 0xb2e6279c, SUC_lexicon_tags_37 },
    { 0xada6279d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2627a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaec627a3, SUC_lexicon_tags_25 },
    { 0x3c4627a4, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0x65c627a6, SUC_lexicon_tags_0 },
    { 0x30c627a7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0627aa, SUC_lexicon_tags_15 },
    { 0x3e8627ab, SUC_lexicon_tags_20 },
    { 0x76c627ac, SUC_lexicon_tags_12 },
    { 0xeda627ad, SUC_lexicon_tags_6 },
    { 0xa32627aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc627b1, SUC_lexicon_tags_12 },
    { 0x89a627b1, SUC_lexicon_tags_10 },
    { 0x424627b2, SUC_lexicon_tags_27 },
    { 0x48a627b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc627b9, SUC_lexicon_tags_1 },
    { 0x4ea627ba, SUC_lexicon_tags_7 },
    { 0x9b8627b9, SUC_lexicon_tags_37 },
    { 0xdce627bc, SUC_lexicon_tags_25 },
    { 0x9fe627bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0627c6, SUC_lexicon_tags_29 },
    { 0x352627c7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2627ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14627ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132627d2, SUC_lexicon_tags_14 },
    { 0xffe627d3, SUC_lexicon_tags_65 },
    { 0x716627d4, SUC_lexicon_tags_35 },
    { 0x784627d3, SUC_lexicon_tags_41 },
    { 0xf96627d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42627e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a6627e4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2627eb, SUC_lexicon_tags_17 },
    { 0x962627ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd2627ee, SUC_lexicon_tags_31 },
    { 0xaac627ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb98627f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40627f4, SUC_lexicon_tags_17 },
    { 0xce8627f5, SUC_lexicon_tags_23 },
    { 0x5ae627f6, SUC_lexicon_tags_1 },
    { 0x2c6627f7, SUC_lexicon_tags_14 },
    { 0x17a627f8, SUC_lexicon_tags_2 },
    { 0x7b0627f9, SUC_lexicon_tags_27 },
    { 0x478627f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e2627fc, SUC_lexicon_tags_28 },
    { 0x18a627fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d462807, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b46280b, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x3c06280d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e62811, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1462814, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5fc62816, SUC_lexicon_tags_66 },
    { 0x55462817, SUC_lexicon_tags_27 },
    { 0x47062816, SUC_lexicon_tags_80 },
    { 0x8fa62819, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4566281d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a62821, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c62824, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d862829, SUC_lexicon_tags_33 },
    { 0x13462829, SUC_lexicon_tags_14 },
    { 0x5646282b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76c6282d, SUC_lexicon_tags_48 },
    { 0x96e6282e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb462830, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdce62832, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8062835, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xede62837, SUC_lexicon_tags_15 },
    { 0x86662838, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c6283b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e6283e, SUC_lexicon_tags_201 },
    { 0x6826283e, SUC_lexicon_tags_17 },
    { 0xc4a6283f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8de62842, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0262846, SUC_lexicon_tags_31 },
    { 0x90062846, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae6284b, SUC_lexicon_tags_11 },
    { 0x80e6284b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1262853, SUC_lexicon_tags_54 },
    { 0x81662854, SUC_lexicon_tags_28 },
    { 0xa4462853, SUC_lexicon_tags_46 },
    { 0xb0462856, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1c62858, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be6285c, SUC_lexicon_tags_1127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4062862, SUC_lexicon_tags_17 },
    { 0x5d662863, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac6286a, SUC_lexicon_tags_29 },
    { 0x4926286b, SUC_lexicon_tags_55 },
    { 0xac6286a, SUC_lexicon_tags_37 },
    { 0xd0c6286d, SUC_lexicon_tags_47 },
    { 0xbd86286d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8ea62870, SUC_lexicon_tags_17 },
    { 0xf3262871, SUC_lexicon_tags_213 },
    { 0x9cc62872, SUC_lexicon_tags_27 },
    { 0x9262872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd086287b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb506287e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3fc62880, SUC_lexicon_tags_7 },
    { 0x74062881, SUC_lexicon_tags_49 },
    { 0x4a262882, SUC_lexicon_tags_0 },
    { 0x34e62883, SUC_lexicon_tags_37 },
    { 0xec062884, SUC_lexicon_tags_23 },
    { 0x9f262882, SUC_lexicon_tags_25 },
    { 0x9f462886, SUC_lexicon_tags_37 },
    { 0xb7a62887, SUC_lexicon_tags_7 },
    { 0x4be62888, SUC_lexicon_tags_37 },
    { 0xe3662880, SUC_lexicon_tags_25 },
    { 0xb0862882, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa62894, SUC_lexicon_tags_12 },
    { 0x42e62895, SUC_lexicon_tags_17 },
    { 0xd5e62896, SUC_lexicon_tags_37 },
    { 0xff462897, SUC_lexicon_tags_27 },
    { 0x88a62894, SUC_lexicon_tags_7 },
    { 0x75e62899, SUC_lexicon_tags_8 },
    { 0xd8a62897, SUC_lexicon_tags_30 },
    { 0x8226289a, SUC_lexicon_tags_25 },
    { 0xd2e62894, SUC_lexicon_tags_27 },
    { 0xf326289d, SUC_lexicon_tags_11 },
    { 0xab06289d, SUC_lexicon_tags_15 },
    { 0x6e66289b, SUC_lexicon_tags_17 },
    { 0x458628a0, SUC_lexicon_tags_15 },
    { 0x6e6628a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8628a5, SUC_lexicon_tags_34 },
    { 0x3a8628a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4e628a8, SUC_lexicon_tags_45 },
    { 0xf6a628a8, SUC_lexicon_tags_138 },
    { 0x904628aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5de628ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x25c628ae, SUC_lexicon_tags_9 },
    { 0x592628ae, SUC_lexicon_tags_15 },
    { 0x2b0628b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x544628b3, SUC_lexicon_tags_11 },
    { 0xc1e628b4, SUC_lexicon_tags_25 },
    { 0xa58628b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b4628b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe628bb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x604628bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80628c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6628c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6628cd, SUC_lexicon_tags_7 },
    { 0x620628ce, SUC_lexicon_tags_61 },
    { 0x7ea628cf, SUC_lexicon_tags_11 },
    { 0x2dc628d0, SUC_lexicon_tags_4 },
    { 0xb56628d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a628d4, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x382628d6, SUC_lexicon_tags_37 },
    { 0xc90628d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e628dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84628e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x104628e3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6628e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf20628e8, SUC_lexicon_tags_7 },
    { 0xc76628e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b4628eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10628ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2628f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac628f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88628fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x228628fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dc62900, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65862908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a46290c, SUC_lexicon_tags_2 },
    { 0xbb86290d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2662915, SUC_lexicon_tags_37 },
    { 0x97862916, SUC_lexicon_tags_2 },
    { 0xcf462916, SUC_lexicon_tags_14 },
    { 0xba262918, SUC_lexicon_tags_12 },
    { 0x9262916, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8ee6291b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a6291e, SUC_lexicon_tags_10 },
    { 0x5da6291f, SUC_lexicon_tags_11 },
    { 0x63a62920, SUC_lexicon_tags_94 },
    { 0xad462921, SUC_lexicon_tags_25 },
    { 0xd3e62921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a662926, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2926292b, SUC_lexicon_tags_1 },
    { 0xaaa6292c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43462932, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23c62934, SUC_lexicon_tags_37 },
    { 0xab062934, SUC_lexicon_tags_39 },
    { 0xf8862936, SUC_lexicon_tags_65 },
    { 0x1ec62936, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd826293a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9666293c, SUC_lexicon_tags_30 },
    { 0x50e6293d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe62940, SUC_lexicon_tags_87 },
    { 0x2e62941, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f262945, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4462949, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6a6294b, SUC_lexicon_tags_37 },
    { 0x7426294b, SUC_lexicon_tags_37 },
    { 0xac46294b, SUC_lexicon_tags_16 },
    { 0xf4e6294e, SUC_lexicon_tags_16 },
    { 0x26294f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c62952, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc62957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb626295e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcc62960, SUC_lexicon_tags_84 },
    { 0x66462961, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff662963, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdda62965, SUC_lexicon_tags_43 },
    { 0xe0862966, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e6296a, SUC_lexicon_tags_65 },
    { 0x3b66296a, SUC_lexicon_tags_5 },
    { 0x21a6296c, SUC_lexicon_tags_25 },
    { 0x4a6296d, SUC_lexicon_tags_2 },
    { 0x28a6296e, SUC_lexicon_tags_9 },
    { 0xcee6296f, SUC_lexicon_tags_12 },
    { 0x3606296d, SUC_lexicon_tags_48 },
    { 0x2a262971, SUC_lexicon_tags_19 },
    { 0xd4662972, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0862975, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff862977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb466297b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2e6297d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x95e6297f, SUC_lexicon_tags_13 },
    { 0xe7662980, SUC_lexicon_tags_46 },
    { 0x2386297f, SUC_lexicon_tags_1 },
    { 0x7f662982, SUC_lexicon_tags_4 },
    { 0x4c62983, SUC_lexicon_tags_37 },
    { 0xb262984, SUC_lexicon_tags_14 },
    { 0xdbc62983, SUC_lexicon_tags_112 },
    { 0x18062986, SUC_lexicon_tags_15 },
    { 0xb3862987, SUC_lexicon_tags_4 },
    { 0x37662987, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8566298e, SUC_lexicon_tags_38 },
    { 0xe706298f, SUC_lexicon_tags_39 },
    { 0x73e6298e, SUC_lexicon_tags_118 },
    { 0xe3262991, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba62994, SUC_lexicon_tags_7 },
    { 0x22e62995, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906299c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8629a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8629a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900629b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6629b9, SUC_lexicon_tags_7 },
    { 0xba0629ba, SUC_lexicon_tags_40 },
    { 0x624629ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x588629bd, SUC_lexicon_tags_224 },
    { 0x94629bd, SUC_lexicon_tags_37 },
    { 0x4ba629bf, SUC_lexicon_tags_25 },
    { 0xdb2629c0, SUC_lexicon_tags_359 },
    { 0x704629c1, SUC_lexicon_tags_25 },
    { 0xb2a629c1, SUC_lexicon_tags_38 },
    { 0x9a2629c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x640629c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd14629c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182629cb, SUC_lexicon_tags_37 },
    { 0xd8e629cc, SUC_lexicon_tags_1 },
    { 0xbbe629cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da629d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a629dd, SUC_lexicon_tags_7 },
    { 0x4fa629dd, SUC_lexicon_tags_27 },
    { 0x24629df, SUC_lexicon_tags_45 },
    { 0xb54629e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88629e4, SUC_lexicon_tags_25 },
    { 0x456629e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a0629e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x688629ea, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32629f2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6629f8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a629ff, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec62a02, SUC_lexicon_tags_60 },
    { 0xff462a03, SUC_lexicon_tags_26 },
    { 0xd5062a04, SUC_lexicon_tags_69 },
    { 0xe5a62a02, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x89862a08, SUC_lexicon_tags_17 },
    { 0xa3062a09, SUC_lexicon_tags_39 },
    { 0xba662a0a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc62a0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55062a0f, SUC_lexicon_tags_23 },
    { 0x23462a10, SUC_lexicon_tags_43 },
    { 0x9862a11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f462a14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c62a18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x20662a1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd862a20, SUC_lexicon_tags_25 },
    { 0x3fc62a20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5fe62a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2462a29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e462a31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a62a39, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62462a3d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba62a40, SUC_lexicon_tags_10 },
    { 0xc5262a41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa0862a43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8062a48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11c62a4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcf862a4c, SUC_lexicon_tags_66 },
    { 0xc4062a4d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69662a52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa62a60, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4062a63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5862a66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6862a68, SUC_lexicon_tags_33 },
    { 0x2ba62a69, SUC_lexicon_tags_13 },
    { 0xe5e62a69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa62a6d, SUC_lexicon_tags_0 },
    { 0x25e62a6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2462a71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e662a74, SUC_lexicon_tags_49 },
    { 0xca662a75, SUC_lexicon_tags_229 },
    { 0xf5c62a76, SUC_lexicon_tags_12 },
    { 0xaa662a76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e62a7b, SUC_lexicon_tags_17 },
    { 0x24662a7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3262a81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62a86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88062a8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6862a8e, SUC_lexicon_tags_25 },
    { 0x53262a8f, SUC_lexicon_tags_2 },
    { 0x36262a90, SUC_lexicon_tags_11 },
    { 0x94662a90, SUC_lexicon_tags_17 },
    { 0x2d262a92, SUC_lexicon_tags_1 },
    { 0x18462a8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa62a97, SUC_lexicon_tags_39 },
    { 0xb5262a98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4062aa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe62aa7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba662aab, SUC_lexicon_tags_0 },
    { 0xa1a62aab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c62ab0, SUC_lexicon_tags_65 },
    { 0x50a62ab1, SUC_lexicon_tags_45 },
    { 0x88262ab0, SUC_lexicon_tags_47 },
    { 0xa4462ab3, SUC_lexicon_tags_2 },
    { 0x90862ab3, SUC_lexicon_tags_45 },
    { 0x8a862ab4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a62ab8, SUC_lexicon_tags_61 },
    { 0x34462ab8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf062abb, SUC_lexicon_tags_29 },
    { 0x2cc62abc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c662ac0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3a62ac2, SUC_lexicon_tags_1 },
    { 0xf6c62ac3, SUC_lexicon_tags_33 },
    { 0xe4262ac4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e262acc, SUC_lexicon_tags_60 },
    { 0xb7262acc, SUC_lexicon_tags_37 },
    { 0x62062acd, SUC_lexicon_tags_17 },
    { 0x57a62acd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe0462ad1, SUC_lexicon_tags_37 },
    { 0xe6e62ad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8862ad4, SUC_lexicon_tags_11 },
    { 0x17e62ad5, SUC_lexicon_tags_27 },
    { 0x55a62ad4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69262ada, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9262ae2, SUC_lexicon_tags_25 },
    { 0x42c62ae2, SUC_lexicon_tags_17 },
    { 0x41662ae4, SUC_lexicon_tags_25 },
    { 0xf3062ae4, SUC_lexicon_tags_45 },
    { 0x8ca62ae4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a62aed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb262aef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f662af6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77e62af8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e62afd, SUC_lexicon_tags_720 },
    { 0, NULL },
    { 0xf1e62aff, SUC_lexicon_tags_11 },
    { 0x4c62b00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe462b02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c62b0a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x70862b0d, SUC_lexicon_tags_7 },
    { 0xba662b0d, SUC_lexicon_tags_43 },
    { 0x2e462b0e, SUC_lexicon_tags_11 },
    { 0xfde62b0d, SUC_lexicon_tags_29 },
    { 0x2fc62b11, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89662b1a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xda062b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60862b1f, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x6462b22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4062b24, SUC_lexicon_tags_84 },
    { 0x72c62b25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe262b29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6462b2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6a62b2d, SUC_lexicon_tags_1 },
    { 0x47c62b2e, SUC_lexicon_tags_37 },
    { 0xb2262b2d, SUC_lexicon_tags_11 },
    { 0x25262b30, SUC_lexicon_tags_17 },
    { 0x6b862b30, SUC_lexicon_tags_13 },
    { 0xfea62b32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8662b34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2e62b36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51062b3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d262b3f, SUC_lexicon_tags_23 },
    { 0x29062b40, SUC_lexicon_tags_31 },
    { 0x9de62b3f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd862b44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0462b47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7862b49, SUC_lexicon_tags_2 },
    { 0x19662b49, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a62b52, SUC_lexicon_tags_37 },
    { 0xac262b53, SUC_lexicon_tags_87 },
    { 0x97862b54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x30062b57, SUC_lexicon_tags_55 },
    { 0x6a662b58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x93e62b5a, SUC_lexicon_tags_0 },
    { 0x59462b5a, SUC_lexicon_tags_32 },
    { 0xe1e62b5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xce262b5f, SUC_lexicon_tags_11 },
    { 0xb8662b60, SUC_lexicon_tags_2 },
    { 0x4f262b61, SUC_lexicon_tags_15 },
    { 0x64a62b62, SUC_lexicon_tags_87 },
    { 0x2a262b62, SUC_lexicon_tags_10 },
    { 0x5c062b62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0662b66, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b462b6a, SUC_lexicon_tags_2 },
    { 0xd1062b6b, SUC_lexicon_tags_14 },
    { 0x50662b6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91462b6e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa9c62b70, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd662b74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88262b7f, SUC_lexicon_tags_37 },
    { 0xfc262b80, SUC_lexicon_tags_33 },
    { 0x49862b7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c62b86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa662b8c, SUC_lexicon_tags_37 },
    { 0x70e62b8d, SUC_lexicon_tags_2 },
    { 0xa9c62b8d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c262b9d, SUC_lexicon_tags_1 },
    { 0x27862b9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb9a62ba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71462ba4, SUC_lexicon_tags_15 },
    { 0x9da62ba4, SUC_lexicon_tags_7 },
    { 0x1d062ba5, SUC_lexicon_tags_60 },
    { 0x7c862ba7, SUC_lexicon_tags_28 },
    { 0x81462ba4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0062bb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x36662bb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e662bb8, SUC_lexicon_tags_26 },
    { 0x9c62bb8, SUC_lexicon_tags_12 },
    { 0x76e62bba, SUC_lexicon_tags_25 },
    { 0x80a62bbb, SUC_lexicon_tags_30 },
    { 0x6b062bbb, SUC_lexicon_tags_11 },
    { 0x4a662bbc, SUC_lexicon_tags_81 },
    { 0xb2862bbe, SUC_lexicon_tags_201 },
    { 0x29062bbc, SUC_lexicon_tags_0 },
    { 0x97462bc0, SUC_lexicon_tags_12 },
    { 0xa1062bc1, SUC_lexicon_tags_75 },
    { 0xeb262bc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e62bc7, SUC_lexicon_tags_13 },
    { 0xe1a62bc8, SUC_lexicon_tags_8 },
    { 0x90a62bc9, SUC_lexicon_tags_60 },
    { 0xef662bc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57462bcc, SUC_lexicon_tags_49 },
    { 0xfae62bcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a62bd2, SUC_lexicon_tags_15 },
    { 0xe2c62bd2, SUC_lexicon_tags_11 },
    { 0x62c62bd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b462bd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa62bda, SUC_lexicon_tags_33 },
    { 0x5de62bdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98462be4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3062be7, SUC_lexicon_tags_2 },
    { 0xe7462be7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa62beb, SUC_lexicon_tags_23 },
    { 0x34062bec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c62bf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68c62bf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66862bff, SUC_lexicon_tags_7 },
    { 0xcbe62bff, SUC_lexicon_tags_37 },
    { 0x8fc62bff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1262c04, SUC_lexicon_tags_6 },
    { 0xfbc62c05, SUC_lexicon_tags_11 },
    { 0x1a862c05, SUC_lexicon_tags_6 },
    { 0x3e262c06, SUC_lexicon_tags_25 },
    { 0x44662c08, SUC_lexicon_tags_37 },
    { 0x13662c08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1262c0b, SUC_lexicon_tags_0 },
    { 0xeaa62c0c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e662c0f, SUC_lexicon_tags_2 },
    { 0x8fa62c10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f862c12, SUC_lexicon_tags_66 },
    { 0x56e62c12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c62c15, SUC_lexicon_tags_7 },
    { 0x43c62c16, SUC_lexicon_tags_23 },
    { 0xebe62c15, SUC_lexicon_tags_35 },
    { 0x1ae62c18, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x89462c1b, SUC_lexicon_tags_11 },
    { 0x77062c1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d062c1e, SUC_lexicon_tags_35 },
    { 0xd1c62c1e, SUC_lexicon_tags_23 },
    { 0x18062c1e, SUC_lexicon_tags_17 },
    { 0xcc862c21, SUC_lexicon_tags_55 },
    { 0x75662c22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ca62c24, SUC_lexicon_tags_1 },
    { 0x44c62c25, SUC_lexicon_tags_6 },
    { 0xc5662c26, SUC_lexicon_tags_22 },
    { 0x20e62c27, SUC_lexicon_tags_1 },
    { 0x38c62c25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e62c2b, SUC_lexicon_tags_17 },
    { 0x64c62c2b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d862c2f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc62c32, SUC_lexicon_tags_1 },
    { 0x50862c33, SUC_lexicon_tags_11 },
    { 0xb1062c33, SUC_lexicon_tags_15 },
    { 0x7be62c34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e62c38, SUC_lexicon_tags_2 },
    { 0x6f662c39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a62c3e, SUC_lexicon_tags_21 },
    { 0x3c862c3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84462c45, SUC_lexicon_tags_15 },
    { 0x2ca62c45, SUC_lexicon_tags_38 },
    { 0xcd662c47, SUC_lexicon_tags_7 },
    { 0xd3462c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73662c4d, SUC_lexicon_tags_7 },
    { 0xbec62c4e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa62c51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca62c55, SUC_lexicon_tags_25 },
    { 0x89062c55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1062c58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb062c5e, SUC_lexicon_tags_849 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c62c61, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc8862c63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e62c66, SUC_lexicon_tags_15 },
    { 0x48e62c66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49e62c69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea62c6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a62c70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac62c78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53462c7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3f862c7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a62c80, SUC_lexicon_tags_17 },
    { 0x40462c80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a262c83, SUC_lexicon_tags_25 },
    { 0x4ce62c84, SUC_lexicon_tags_12 },
    { 0x12a62c83, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x10a62c87, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x97062c8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6fc62c8c, SUC_lexicon_tags_12 },
    { 0x8b462c8d, SUC_lexicon_tags_63 },
    { 0x7c662c8e, SUC_lexicon_tags_65 },
    { 0xf1262c8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x74262c91, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3662c95, SUC_lexicon_tags_26 },
    { 0xf0a62c96, SUC_lexicon_tags_23 },
    { 0xb2a62c96, SUC_lexicon_tags_25 },
    { 0xec262c97, SUC_lexicon_tags_15 },
    { 0xd862c99, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c62c9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa462ca1, SUC_lexicon_tags_18 },
    { 0x57e62ca1, SUC_lexicon_tags_13 },
    { 0xe2662ca3, SUC_lexicon_tags_39 },
    { 0x42e62ca1, SUC_lexicon_tags_9 },
    { 0x59c62ca5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a62caa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7c62cac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaea62cae, SUC_lexicon_tags_25 },
    { 0x44c62caf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e62cb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a862cbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e062cbf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c62cc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc62cc7, SUC_lexicon_tags_14 },
    { 0xe8662cc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc3e62cca, SUC_lexicon_tags_77 },
    { 0xeec62ccb, SUC_lexicon_tags_25 },
    { 0x82662ccc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2a62cce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c62cd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3062cd4, SUC_lexicon_tags_37 },
    { 0x8b262cd5, SUC_lexicon_tags_9 },
    { 0x99262cd5, SUC_lexicon_tags_11 },
    { 0x8b462cd7, SUC_lexicon_tags_1 },
    { 0x99e62cd7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45662cde, SUC_lexicon_tags_10 },
    { 0x6ea62cde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee62ce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a262ce6, SUC_lexicon_tags_29 },
    { 0x37062ce7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e62cee, SUC_lexicon_tags_7 },
    { 0x262cef, SUC_lexicon_tags_8 },
    { 0x73862cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c62cf4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45262cf8, SUC_lexicon_tags_15 },
    { 0x1d262cf8, SUC_lexicon_tags_33 },
    { 0xeca62cf8, SUC_lexicon_tags_31 },
    { 0xcea62cf9, SUC_lexicon_tags_23 },
    { 0xc9062cfc, SUC_lexicon_tags_2 },
    { 0x7f462cfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c862cff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x94462d01, SUC_lexicon_tags_33 },
    { 0x7c862d02, SUC_lexicon_tags_2 },
    { 0xf8062d01, SUC_lexicon_tags_12 },
    { 0x50c62d02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f862d0c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf5a62d0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe62d11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c62d15, SUC_lexicon_tags_19 },
    { 0x9bc62d15, SUC_lexicon_tags_38 },
    { 0x55862d15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc62d1a, SUC_lexicon_tags_65 },
    { 0xa5a62d1a, SUC_lexicon_tags_69 },
    { 0xe1062d1c, SUC_lexicon_tags_43 },
    { 0x8b462d1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e62d23, SUC_lexicon_tags_11 },
    { 0xdbc62d24, SUC_lexicon_tags_1 },
    { 0x15662d25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e62d2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa62d32, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4862d34, SUC_lexicon_tags_25 },
    { 0x9c462d35, SUC_lexicon_tags_32 },
    { 0x50c62d34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab262d3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3262d3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeaa62d40, SUC_lexicon_tags_45 },
    { 0xe3c62d41, SUC_lexicon_tags_56 },
    { 0x51262d42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x12062d44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3862d47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc462d50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a62d53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb462d57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3e62d59, SUC_lexicon_tags_2 },
    { 0x2da62d59, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa462d5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4462d61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee62d6a, SUC_lexicon_tags_25 },
    { 0xe4662d6a, SUC_lexicon_tags_11 },
    { 0x2f062d6c, SUC_lexicon_tags_47 },
    { 0x2a462d6c, SUC_lexicon_tags_25 },
    { 0x3e62d6e, SUC_lexicon_tags_27 },
    { 0xee062d6e, SUC_lexicon_tags_1 },
    { 0x43e62d6a, SUC_lexicon_tags_27 },
    { 0x27a62d71, SUC_lexicon_tags_12 },
    { 0x6c62d71, SUC_lexicon_tags_37 },
    { 0x3662d72, SUC_lexicon_tags_34 },
    { 0x56a62d74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5a62d76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e662d7c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x79e62d7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81462d86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46662d8a, SUC_lexicon_tags_37 },
    { 0xf2e62d8b, SUC_lexicon_tags_49 },
    { 0x6ec62d8c, SUC_lexicon_tags_47 },
    { 0x94462d8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a862d90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c062d97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7262d99, SUC_lexicon_tags_26 },
    { 0x96862d9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b062da3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc462da6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd262da8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbb462daa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14262dac, SUC_lexicon_tags_31 },
    { 0x7e62dad, SUC_lexicon_tags_104 },
    { 0x95c62dae, SUC_lexicon_tags_23 },
    { 0xa0262daf, SUC_lexicon_tags_6 },
    { 0xbc862db0, SUC_lexicon_tags_37 },
    { 0xc7462db1, SUC_lexicon_tags_1 },
    { 0x76662db1, SUC_lexicon_tags_55 },
    { 0x31262db3, SUC_lexicon_tags_10 },
    { 0x6e62dae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0462db9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c462dbe, SUC_lexicon_tags_17 },
    { 0xc4062dbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee62dc5, SUC_lexicon_tags_37 },
    { 0x2d062dc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd462dc9, SUC_lexicon_tags_16 },
    { 0x2c662dc9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6e62dcc, SUC_lexicon_tags_11 },
    { 0xa2c62dcc, SUC_lexicon_tags_45 },
    { 0x2e062dcd, SUC_lexicon_tags_45 },
    { 0x51662dcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c862dd3, SUC_lexicon_tags_15 },
    { 0xb8a62dd4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc9662dd6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcaa62dd8, SUC_lexicon_tags_25 },
    { 0x3c62dd8, SUC_lexicon_tags_151 },
    { 0x3ee62dda, SUC_lexicon_tags_87 },
    { 0xa9a62ddb, SUC_lexicon_tags_19 },
    { 0x48a62ddb, SUC_lexicon_tags_7 },
    { 0x59a62dda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a462de3, SUC_lexicon_tags_1 },
    { 0x20a62de4, SUC_lexicon_tags_47 },
    { 0x1a062de4, SUC_lexicon_tags_37 },
    { 0xa8262de6, SUC_lexicon_tags_37 },
    { 0xfde62de6, SUC_lexicon_tags_6 },
    { 0x21262de4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa9462dea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x32862ded, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e62df0, SUC_lexicon_tags_15 },
    { 0xbc62df1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9662df3, SUC_lexicon_tags_37 },
    { 0xd0862df4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca62df7, SUC_lexicon_tags_55 },
    { 0x87062df7, SUC_lexicon_tags_25 },
    { 0x1a662df8, SUC_lexicon_tags_38 },
    { 0x95a62dfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8262dfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a262e04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e262e07, SUC_lexicon_tags_12 },
    { 0x3862e08, SUC_lexicon_tags_11 },
    { 0x93262e09, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2cc62e0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4262e10, SUC_lexicon_tags_32 },
    { 0xdba62e10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12462e13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9662e19, SUC_lexicon_tags_31 },
    { 0x61a62e19, SUC_lexicon_tags_49 },
    { 0x62462e19, SUC_lexicon_tags_15 },
    { 0x88262e1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c262e1e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a62e25, SUC_lexicon_tags_18 },
    { 0xd5262e26, SUC_lexicon_tags_0 },
    { 0xa4e62e27, SUC_lexicon_tags_14 },
    { 0x7ca62e27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81662e2e, SUC_lexicon_tags_17 },
    { 0xe62e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbde62e31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x15a62e33, SUC_lexicon_tags_37 },
    { 0x50662e33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a62e37, SUC_lexicon_tags_4 },
    { 0x51262e38, SUC_lexicon_tags_6 },
    { 0x6f662e38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b662e3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a62e42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84462e46, SUC_lexicon_tags_38 },
    { 0xedc62e46, SUC_lexicon_tags_48 },
    { 0xfc662e48, SUC_lexicon_tags_2 },
    { 0xfa262e49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ce62e4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a62e4e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xd9c62e50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3462e55, SUC_lexicon_tags_62 },
    { 0x1ae62e55, SUC_lexicon_tags_65 },
    { 0xd5a62e57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbe62e59, SUC_lexicon_tags_45 },
    { 0x30e62e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e62e5e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x36462e60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8862e62, SUC_lexicon_tags_27 },
    { 0xc862e63, SUC_lexicon_tags_17 },
    { 0xa5262e62, SUC_lexicon_tags_39 },
    { 0xac462e65, SUC_lexicon_tags_32 },
    { 0x93e62e63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef262e6a, SUC_lexicon_tags_23 },
    { 0x92662e6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa62e6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5262e71, SUC_lexicon_tags_27 },
    { 0xe3a62e72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f462e7e, SUC_lexicon_tags_11 },
    { 0x60662e7e, SUC_lexicon_tags_14 },
    { 0x22462e80, SUC_lexicon_tags_12 },
    { 0x23062e81, SUC_lexicon_tags_49 },
    { 0x9d862e82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c62e85, SUC_lexicon_tags_8 },
    { 0x2c662e86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf862e8a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f862e99, SUC_lexicon_tags_4 },
    { 0x3e262e9a, SUC_lexicon_tags_27 },
    { 0x55262e9a, SUC_lexicon_tags_16 },
    { 0x72462e9a, SUC_lexicon_tags_47 },
    { 0x4be62e9c, SUC_lexicon_tags_32 },
    { 0x39862e9a, SUC_lexicon_tags_37 },
    { 0x74462e9f, SUC_lexicon_tags_38 },
    { 0x4dc62ea0, SUC_lexicon_tags_23 },
    { 0x31462ea0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb4c62ea3, SUC_lexicon_tags_1 },
    { 0x70862ea4, SUC_lexicon_tags_2 },
    { 0xefa62ea3, SUC_lexicon_tags_43 },
    { 0xb7262ea5, SUC_lexicon_tags_49 },
    { 0x26262ea6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8262ead, SUC_lexicon_tags_61 },
    { 0x76462eae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18c62eb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bc62eb2, SUC_lexicon_tags_31 },
    { 0x58262eb2, SUC_lexicon_tags_17 },
    { 0x3aa62eb4, SUC_lexicon_tags_11 },
    { 0x49c62eb5, SUC_lexicon_tags_80 },
    { 0x46e62eb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f062ebb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7662ec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4cc62ec2, SUC_lexicon_tags_25 },
    { 0xa6062ec2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a62ec6, SUC_lexicon_tags_33 },
    { 0xf7e62ec7, SUC_lexicon_tags_37 },
    { 0x23c62ec6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc62ecb, SUC_lexicon_tags_4 },
    { 0x99462ecb, SUC_lexicon_tags_1 },
    { 0xd5e62ecb, SUC_lexicon_tags_17 },
    { 0x8b262ece, SUC_lexicon_tags_1 },
    { 0x1fe62ecf, SUC_lexicon_tags_23 },
    { 0x15e62ed0, SUC_lexicon_tags_11 },
    { 0x88062ed1, SUC_lexicon_tags_23 },
    { 0xa9062ecf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9d862ed4, SUC_lexicon_tags_16 },
    { 0x40a62ed4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde62ed8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd862eda, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x50e62edc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e62ee1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b462ee4, SUC_lexicon_tags_148 },
    { 0x41862ee4, SUC_lexicon_tags_17 },
    { 0x54a62ee6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9062eea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1262ef3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c62ef7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8dc62ef9, SUC_lexicon_tags_7 },
    { 0x68462efa, SUC_lexicon_tags_33 },
    { 0x4e262efb, SUC_lexicon_tags_37 },
    { 0x80862efc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4862f04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83462f08, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa62f0c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4862f0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ea62f11, SUC_lexicon_tags_170 },
    { 0x2ce62f11, SUC_lexicon_tags_140 },
    { 0x6a62f12, SUC_lexicon_tags_47 },
    { 0x18862f12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a62f17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e62f1a, SUC_lexicon_tags_2 },
    { 0xf2c62f1a, SUC_lexicon_tags_27 },
    { 0xbb262f1c, SUC_lexicon_tags_17 },
    { 0x76662f1d, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa062f23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe62f26, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab062f2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e662f35, SUC_lexicon_tags_21 },
    { 0xafa62f35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf262f38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7062f3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb462f3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1862f41, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5062f44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0662f48, SUC_lexicon_tags_9 },
    { 0x84462f49, SUC_lexicon_tags_1 },
    { 0x18062f4a, SUC_lexicon_tags_2 },
    { 0x8f062f4b, SUC_lexicon_tags_17 },
    { 0x72262f4c, SUC_lexicon_tags_18 },
    { 0x1be62f4d, SUC_lexicon_tags_2 },
    { 0x7cc62f4d, SUC_lexicon_tags_10 },
    { 0x42262f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea62f53, SUC_lexicon_tags_63 },
    { 0x37262f53, SUC_lexicon_tags_11 },
    { 0x69a62f53, SUC_lexicon_tags_25 },
    { 0xafc62f56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb262f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e62f5e, SUC_lexicon_tags_32 },
    { 0x9d262f5f, SUC_lexicon_tags_1 },
    { 0x6c862f5f, SUC_lexicon_tags_15 },
    { 0x61262f61, SUC_lexicon_tags_0 },
    { 0xa1c62f62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c62f65, SUC_lexicon_tags_17 },
    { 0xa3262f66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc62f6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4262f6e, SUC_lexicon_tags_27 },
    { 0xd8a62f6e, SUC_lexicon_tags_40 },
    { 0xd9c62f70, SUC_lexicon_tags_2 },
    { 0x79862f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd462f74, SUC_lexicon_tags_17 },
    { 0xfea62f75, SUC_lexicon_tags_25 },
    { 0x9bc62f76, SUC_lexicon_tags_25 },
    { 0x91062f75, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3262f7b, SUC_lexicon_tags_37 },
    { 0xa4a62f7c, SUC_lexicon_tags_42 },
    { 0x38462f7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18662f80, SUC_lexicon_tags_27 },
    { 0x4262f80, SUC_lexicon_tags_37 },
    { 0xe0e62f81, SUC_lexicon_tags_15 },
    { 0xe3262f83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e262f86, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e62f8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc662f92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22462f96, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x41462f98, SUC_lexicon_tags_84 },
    { 0x23c62f98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe7e62f9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9462f9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc62fa0, SUC_lexicon_tags_38 },
    { 0x91062fa1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4662fa8, SUC_lexicon_tags_8 },
    { 0x2462fa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce62fb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cc62fb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a62fb5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfd862fb7, SUC_lexicon_tags_31 },
    { 0xcdc62fb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe062fbc, SUC_lexicon_tags_47 },
    { 0x9ae62fbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec62fc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e62fc5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12062fc7, SUC_lexicon_tags_37 },
    { 0xb8062fc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x94462fca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6262fcc, SUC_lexicon_tags_17 },
    { 0xf2862fcd, SUC_lexicon_tags_38 },
    { 0xeac62fcd, SUC_lexicon_tags_63 },
    { 0x17462fcf, SUC_lexicon_tags_7 },
    { 0xd9462fcf, SUC_lexicon_tags_31 },
    { 0xfca62fcd, SUC_lexicon_tags_23 },
    { 0x9fc62fd1, SUC_lexicon_tags_11 },
    { 0x14a62fd3, SUC_lexicon_tags_2 },
    { 0xe2c62fd4, SUC_lexicon_tags_12 },
    { 0xf7262fd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee62fd9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x73c62fdb, SUC_lexicon_tags_37 },
    { 0xc1862fdc, SUC_lexicon_tags_38 },
    { 0x25662fdc, SUC_lexicon_tags_9 },
    { 0x82262fde, SUC_lexicon_tags_2 },
    { 0x2d862fdf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65862fed, SUC_lexicon_tags_47 },
    { 0xd6662fee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e62ff3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa62ffa, SUC_lexicon_tags_12 },
    { 0xe3062ffb, SUC_lexicon_tags_0 },
    { 0xafa62ffc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea62fff, SUC_lexicon_tags_79 },
    { 0xe7a62fff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcd463002, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8c63004, SUC_lexicon_tags_45 },
    { 0xeaa63005, SUC_lexicon_tags_37 },
    { 0xbc263006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e6300c, SUC_lexicon_tags_11 },
    { 0xe766300c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc063012, SUC_lexicon_tags_47 },
    { 0xed263012, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc63018, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34863023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac63028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf806302a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeec6302c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa63034, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1266303a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec26303c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaca6303e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9063040, SUC_lexicon_tags_43 },
    { 0x9fc63041, SUC_lexicon_tags_18 },
    { 0x19c63041, SUC_lexicon_tags_25 },
    { 0x54a63043, SUC_lexicon_tags_216 },
    { 0xf3663044, SUC_lexicon_tags_43 },
    { 0x4f863040, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b463048, SUC_lexicon_tags_46 },
    { 0x9d263049, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67a6304b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7b66304d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba63050, SUC_lexicon_tags_37 },
    { 0x49e63051, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x34863054, SUC_lexicon_tags_31 },
    { 0x47a63055, SUC_lexicon_tags_40 },
    { 0x42263055, SUC_lexicon_tags_15 },
    { 0x16463057, SUC_lexicon_tags_12 },
    { 0x74e63058, SUC_lexicon_tags_39 },
    { 0x41c63059, SUC_lexicon_tags_43 },
    { 0x21463055, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c66305d, SUC_lexicon_tags_11 },
    { 0x4426305d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17463061, SUC_lexicon_tags_25 },
    { 0xfae63062, SUC_lexicon_tags_0 },
    { 0x48a63063, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e63065, SUC_lexicon_tags_6 },
    { 0xf3a63066, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa226306b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ba6306d, SUC_lexicon_tags_1 },
    { 0x8066306e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe463074, SUC_lexicon_tags_11 },
    { 0x56663075, SUC_lexicon_tags_37 },
    { 0x46463076, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c6307a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec6307d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57a6307f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94663086, SUC_lexicon_tags_6 },
    { 0x88a63086, SUC_lexicon_tags_27 },
    { 0xe5463088, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe886308b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7063090, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbea63092, SUC_lexicon_tags_37 },
    { 0xaa263092, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb463096, SUC_lexicon_tags_17 },
    { 0xede63097, SUC_lexicon_tags_25 },
    { 0xac663098, SUC_lexicon_tags_37 },
    { 0x77863099, SUC_lexicon_tags_23 },
    { 0xc7663099, SUC_lexicon_tags_19 },
    { 0x83a6309a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4346309d, SUC_lexicon_tags_25 },
    { 0x5986309e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e630a2, SUC_lexicon_tags_17 },
    { 0x1ba630a2, SUC_lexicon_tags_12 },
    { 0xb36630a4, SUC_lexicon_tags_7 },
    { 0xba2630a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0630ab, SUC_lexicon_tags_47 },
    { 0x314630ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x514630ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x476630b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc70630b3, SUC_lexicon_tags_1 },
    { 0x352630b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c630b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x964630ba, SUC_lexicon_tags_30 },
    { 0x6d8630bb, SUC_lexicon_tags_37 },
    { 0xc16630bb, SUC_lexicon_tags_11 },
    { 0xb62630bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356630c2, SUC_lexicon_tags_65 },
    { 0x486630c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe32630c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae630c8, SUC_lexicon_tags_53 },
    { 0xea6630c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbf0630cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54a630cd, SUC_lexicon_tags_12 },
    { 0x5c4630cd, SUC_lexicon_tags_30 },
    { 0xce4630cf, SUC_lexicon_tags_57 },
    { 0xf06630ce, SUC_lexicon_tags_1 },
    { 0x3f0630cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcb0630d3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2630dd, SUC_lexicon_tags_43 },
    { 0xd76630de, SUC_lexicon_tags_27 },
    { 0x6dc630df, SUC_lexicon_tags_11 },
    { 0x11a630e0, SUC_lexicon_tags_4 },
    { 0x6ca630df, SUC_lexicon_tags_2 },
    { 0x6da630e2, SUC_lexicon_tags_2 },
    { 0x38630e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4630e6, SUC_lexicon_tags_62 },
    { 0x726630e7, SUC_lexicon_tags_45 },
    { 0x592630e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x76a630ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd88630ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138630f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac630f9, SUC_lexicon_tags_15 },
    { 0x932630f9, SUC_lexicon_tags_25 },
    { 0x5c630fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x262630fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38863101, SUC_lexicon_tags_74 },
    { 0x56063102, SUC_lexicon_tags_37 },
    { 0x5f063103, SUC_lexicon_tags_2 },
    { 0x2c663104, SUC_lexicon_tags_21 },
    { 0x13e63104, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c6310a, SUC_lexicon_tags_77 },
    { 0x7c86310b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2866310f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4663111, SUC_lexicon_tags_57 },
    { 0x7c463112, SUC_lexicon_tags_9 },
    { 0xe8863113, SUC_lexicon_tags_28 },
    { 0x54463114, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6a63116, SUC_lexicon_tags_2 },
    { 0x2c463117, SUC_lexicon_tags_37 },
    { 0x4ee63118, SUC_lexicon_tags_0 },
    { 0x87a63117, SUC_lexicon_tags_23 },
    { 0x7e06311a, SUC_lexicon_tags_25 },
    { 0xb2a6311a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc63122, SUC_lexicon_tags_15 },
    { 0xbd863123, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb0063125, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92463127, SUC_lexicon_tags_27 },
    { 0xf0a63128, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe666312d, SUC_lexicon_tags_23 },
    { 0xbd06312d, SUC_lexicon_tags_17 },
    { 0x366312d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf4063131, SUC_lexicon_tags_47 },
    { 0x9ac63131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1463134, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a63137, SUC_lexicon_tags_14 },
    { 0x2d263137, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c6313b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c6313f, SUC_lexicon_tags_45 },
    { 0x2f863140, SUC_lexicon_tags_11 },
    { 0x4263141, SUC_lexicon_tags_21 },
    { 0xa9e6313f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f663146, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c63149, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9366314e, SUC_lexicon_tags_25 },
    { 0x1d86314f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x77a63151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa463154, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef463158, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c63160, SUC_lexicon_tags_2 },
    { 0x8fa63161, SUC_lexicon_tags_25 },
    { 0x5ee63161, SUC_lexicon_tags_11 },
    { 0x74063160, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47663165, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7063168, SUC_lexicon_tags_11 },
    { 0x89063169, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9426316c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac63171, SUC_lexicon_tags_57 },
    { 0x85463172, SUC_lexicon_tags_37 },
    { 0x16c63173, SUC_lexicon_tags_16 },
    { 0xca663174, SUC_lexicon_tags_11 },
    { 0xfa663173, SUC_lexicon_tags_19 },
    { 0xe5663176, SUC_lexicon_tags_63 },
    { 0xb1e63177, SUC_lexicon_tags_12 },
    { 0x8a263174, SUC_lexicon_tags_17 },
    { 0x12263178, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x84a6317b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5286317d, SUC_lexicon_tags_77 },
    { 0x66e6317e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3063180, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x86e63182, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea63185, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc786318a, SUC_lexicon_tags_25 },
    { 0x5046318a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1826318f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1663194, SUC_lexicon_tags_19 },
    { 0x70463195, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a63198, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb106319e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96631a2, SUC_lexicon_tags_29 },
    { 0xaf0631a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c631a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x826631a8, SUC_lexicon_tags_38 },
    { 0x3e6631a9, SUC_lexicon_tags_16 },
    { 0xdc6631aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06631ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe50631af, SUC_lexicon_tags_88 },
    { 0x240631b0, SUC_lexicon_tags_12 },
    { 0x262631b1, SUC_lexicon_tags_19 },
    { 0x642631b1, SUC_lexicon_tags_12 },
    { 0xdb4631b1, SUC_lexicon_tags_40 },
    { 0x6aa631b3, SUC_lexicon_tags_47 },
    { 0xe7a631b5, SUC_lexicon_tags_6 },
    { 0xc2c631b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a631b9, SUC_lexicon_tags_1 },
    { 0xc84631ba, SUC_lexicon_tags_37 },
    { 0x3ec631bb, SUC_lexicon_tags_25 },
    { 0x96c631bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa631c1, SUC_lexicon_tags_14 },
    { 0x79c631c2, SUC_lexicon_tags_17 },
    { 0x26c631c2, SUC_lexicon_tags_77 },
    { 0xc0a631c4, SUC_lexicon_tags_8 },
    { 0x6ca631c3, SUC_lexicon_tags_34 },
    { 0xbe8631c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9c631c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x208631cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08631ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614631d2, SUC_lexicon_tags_141 },
    { 0x324631d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c631d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c631de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2631e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22e631e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a631ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e631ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c631f1, SUC_lexicon_tags_34 },
    { 0x528631f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2631f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d863202, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fa63204, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36863207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2e63209, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe426320e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0063215, SUC_lexicon_tags_37 },
    { 0x77863216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc566321c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d263223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff263226, SUC_lexicon_tags_6 },
    { 0x2c63227, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf06322a, SUC_lexicon_tags_46 },
    { 0xe46322b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xf546322e, SUC_lexicon_tags_17 },
    { 0x83c6322e, SUC_lexicon_tags_0 },
    { 0xa1663230, SUC_lexicon_tags_9 },
    { 0x4ea63231, SUC_lexicon_tags_0 },
    { 0x1d063232, SUC_lexicon_tags_8 },
    { 0xf3263233, SUC_lexicon_tags_2 },
    { 0xc4463234, SUC_lexicon_tags_12 },
    { 0x79c63231, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a63245, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e6324a, SUC_lexicon_tags_60 },
    { 0x8a6324b, SUC_lexicon_tags_71 },
    { 0x2086324a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x626324f, SUC_lexicon_tags_87 },
    { 0xe63250, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaf063252, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf3463254, SUC_lexicon_tags_23 },
    { 0xf2863255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea63259, SUC_lexicon_tags_55 },
    { 0x17a63259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5786325e, SUC_lexicon_tags_37 },
    { 0xae6325e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6063267, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2c63269, SUC_lexicon_tags_17 },
    { 0x8146326a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1986326d, SUC_lexicon_tags_45 },
    { 0x3a46326e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff86327a, SUC_lexicon_tags_38 },
    { 0xd0c6327b, SUC_lexicon_tags_7 },
    { 0x3006327c, SUC_lexicon_tags_33 },
    { 0x7f46327a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89063280, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94463282, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb263285, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6663287, SUC_lexicon_tags_63 },
    { 0x92a63287, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea6328b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e26328d, SUC_lexicon_tags_25 },
    { 0xfac6328d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c463292, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff863295, SUC_lexicon_tags_15 },
    { 0xbb263296, SUC_lexicon_tags_9 },
    { 0x96463297, SUC_lexicon_tags_47 },
    { 0x60c63298, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d86329b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a6329e, SUC_lexicon_tags_11 },
    { 0xf2c6329e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25a632a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x622632a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8632a8, SUC_lexicon_tags_45 },
    { 0xa28632a9, SUC_lexicon_tags_82 },
    { 0x2a2632a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x200632ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc96632ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0632b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x194632b4, SUC_lexicon_tags_0 },
    { 0xc18632b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686632ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e632be, SUC_lexicon_tags_0 },
    { 0xa18632bf, SUC_lexicon_tags_33 },
    { 0x8e632c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xebc632c2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca632c6, SUC_lexicon_tags_33 },
    { 0x5ba632c7, SUC_lexicon_tags_17 },
    { 0x88e632c7, SUC_lexicon_tags_15 },
    { 0x882632c7, SUC_lexicon_tags_70 },
    { 0xba8632c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410632ce, SUC_lexicon_tags_38 },
    { 0x438632cf, SUC_lexicon_tags_39 },
    { 0xe6c632d0, SUC_lexicon_tags_66 },
    { 0x9e4632cf, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x490632d4, SUC_lexicon_tags_6 },
    { 0x4f0632d5, SUC_lexicon_tags_0 },
    { 0x9b6632d5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8632d9, SUC_lexicon_tags_12 },
    { 0xfdc632da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e632dd, SUC_lexicon_tags_25 },
    { 0x226632de, SUC_lexicon_tags_25 },
    { 0x18632dd, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0x2fe632e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca632e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x176632eb, SUC_lexicon_tags_15 },
    { 0x2a4632ec, SUC_lexicon_tags_17 },
    { 0xcae632ed, SUC_lexicon_tags_12 },
    { 0xb6632ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade632f2, SUC_lexicon_tags_23 },
    { 0x26c632f3, SUC_lexicon_tags_4 },
    { 0x26632f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e632f8, SUC_lexicon_tags_7 },
    { 0xab2632f9, SUC_lexicon_tags_38 },
    { 0xc6632f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd30632fc, SUC_lexicon_tags_29 },
    { 0x658632fd, SUC_lexicon_tags_45 },
    { 0x1d8632fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x36063301, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa463308, SUC_lexicon_tags_19 },
    { 0x80063309, SUC_lexicon_tags_11 },
    { 0xbfa6330a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb663316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb863318, SUC_lexicon_tags_15 },
    { 0x18063319, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9006331d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce63321, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c63324, SUC_lexicon_tags_14 },
    { 0xffc63325, SUC_lexicon_tags_25 },
    { 0x7f663325, SUC_lexicon_tags_17 },
    { 0x34263327, SUC_lexicon_tags_26 },
    { 0xe3863326, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xc986332b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8026332d, SUC_lexicon_tags_2 },
    { 0x4a6332d, SUC_lexicon_tags_17 },
    { 0xa1c6332f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x80663332, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57063339, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7586333b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x9aa6333d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1263340, SUC_lexicon_tags_9 },
    { 0xe0463340, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98663343, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1063348, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a86334c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c63350, SUC_lexicon_tags_11 },
    { 0xe3c63350, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a63354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1463356, SUC_lexicon_tags_68 },
    { 0x79663357, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa6335a, SUC_lexicon_tags_17 },
    { 0x41a6335a, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b26335f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb263363, SUC_lexicon_tags_62 },
    { 0x4c863364, SUC_lexicon_tags_11 },
    { 0x7bc63365, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9a63367, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d063369, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f06336b, SUC_lexicon_tags_13 },
    { 0xb266336b, SUC_lexicon_tags_19 },
    { 0xbb26336d, SUC_lexicon_tags_0 },
    { 0x79c6336c, SUC_lexicon_tags_28 },
    { 0xd46336f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9263371, SUC_lexicon_tags_11 },
    { 0x29e63372, SUC_lexicon_tags_25 },
    { 0x92c63371, SUC_lexicon_tags_60 },
    { 0x1ee63373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93663379, SUC_lexicon_tags_10 },
    { 0x2ce6337a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fc6337c, SUC_lexicon_tags_16 },
    { 0xb306337d, SUC_lexicon_tags_65 },
    { 0xcce6337c, SUC_lexicon_tags_38 },
    { 0xe7a6337d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1663382, SUC_lexicon_tags_8 },
    { 0xc0863383, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a63386, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5263389, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3586338d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeaa6338f, SUC_lexicon_tags_12 },
    { 0x4126338f, SUC_lexicon_tags_11 },
    { 0xa586338f, SUC_lexicon_tags_88 },
    { 0x4f263391, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3ee63394, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4746339b, SUC_lexicon_tags_23 },
    { 0x9fc6339b, SUC_lexicon_tags_18 },
    { 0x2da6339c, SUC_lexicon_tags_60 },
    { 0x89e6339d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb72633a0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134633a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x74633aa, SUC_lexicon_tags_25 },
    { 0x4bc633ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0633ae, SUC_lexicon_tags_7 },
    { 0xb00633af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa633b3, SUC_lexicon_tags_12 },
    { 0x6c633b4, SUC_lexicon_tags_31 },
    { 0x7ee633b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d8633b7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x172633b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6633bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x532633bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c633c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a633c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc633c9, SUC_lexicon_tags_15 },
    { 0xe633ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xac6633cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x168633cf, SUC_lexicon_tags_2 },
    { 0x6f0633d0, SUC_lexicon_tags_14 },
    { 0x70633d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c633d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba633d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8633de, SUC_lexicon_tags_0 },
    { 0x266633de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440633e4, SUC_lexicon_tags_68 },
    { 0x7ba633e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6633ec, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0633f1, SUC_lexicon_tags_21 },
    { 0x558633f2, SUC_lexicon_tags_37 },
    { 0xaa0633f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0633f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ce633f8, SUC_lexicon_tags_65 },
    { 0xc46633f9, SUC_lexicon_tags_19 },
    { 0xed2633fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8633fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48863400, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47463404, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec63408, SUC_lexicon_tags_15 },
    { 0x45663409, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xda26340b, SUC_lexicon_tags_27 },
    { 0xd866340c, SUC_lexicon_tags_9 },
    { 0xa8e6340c, SUC_lexicon_tags_25 },
    { 0x6646340b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79a63410, SUC_lexicon_tags_14 },
    { 0xf7263410, SUC_lexicon_tags_39 },
    { 0x22263412, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c63416, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa63419, SUC_lexicon_tags_47 },
    { 0x4ac6341a, SUC_lexicon_tags_1 },
    { 0x17a63419, SUC_lexicon_tags_23 },
    { 0xf7463419, SUC_lexicon_tags_54 },
    { 0x4186341d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8063428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c66342e, SUC_lexicon_tags_49 },
    { 0x2386342f, SUC_lexicon_tags_15 },
    { 0x6f66342f, SUC_lexicon_tags_49 },
    { 0x21063431, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c63435, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x97863438, SUC_lexicon_tags_25 },
    { 0x1b263439, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e863441, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1663443, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba63447, SUC_lexicon_tags_55 },
    { 0x54c63448, SUC_lexicon_tags_2 },
    { 0xa1263447, SUC_lexicon_tags_23 },
    { 0x2c63448, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c6344e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xa5863450, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde063453, SUC_lexicon_tags_25 },
    { 0x4f263454, SUC_lexicon_tags_47 },
    { 0xf8063453, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c06345a, SUC_lexicon_tags_12 },
    { 0xb526345a, SUC_lexicon_tags_25 },
    { 0x7866345c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97663461, SUC_lexicon_tags_9 },
    { 0xdb463462, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a63466, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfea63468, SUC_lexicon_tags_46 },
    { 0xe2063468, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd06346b, SUC_lexicon_tags_87 },
    { 0xfa46346c, SUC_lexicon_tags_11 },
    { 0x6346346c, SUC_lexicon_tags_10 },
    { 0x20a6346e, SUC_lexicon_tags_2 },
    { 0x1d46346f, SUC_lexicon_tags_25 },
    { 0x86c6346f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa3263472, SUC_lexicon_tags_2 },
    { 0xa5c63472, SUC_lexicon_tags_11 },
    { 0xd3463472, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba63479, SUC_lexicon_tags_37 },
    { 0xb2a63479, SUC_lexicon_tags_0 },
    { 0xd846347b, SUC_lexicon_tags_17 },
    { 0xd286347c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54463482, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x58463484, SUC_lexicon_tags_2 },
    { 0x33e63485, SUC_lexicon_tags_12 },
    { 0xb5263486, SUC_lexicon_tags_17 },
    { 0xfb263487, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe046348a, SUC_lexicon_tags_27 },
    { 0x8946348b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee63491, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0863495, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8ba63497, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d66349a, SUC_lexicon_tags_7 },
    { 0xba6349b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x766349d, SUC_lexicon_tags_33 },
    { 0xb986349e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x216634a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2634a4, SUC_lexicon_tags_2 },
    { 0x1d0634a5, SUC_lexicon_tags_60 },
    { 0x648634a4, SUC_lexicon_tags_16 },
    { 0xcb0634a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x530634aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa68634ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc80634ae, SUC_lexicon_tags_0 },
    { 0x124634af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a634b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6634b5, SUC_lexicon_tags_46 },
    { 0x29e634b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8634bb, SUC_lexicon_tags_16 },
    { 0x87c634bb, SUC_lexicon_tags_17 },
    { 0x212634bb, SUC_lexicon_tags_7 },
    { 0x3c634bc, SUC_lexicon_tags_23 },
    { 0xb2a634bf, SUC_lexicon_tags_7 },
    { 0x88634c0, SUC_lexicon_tags_1 },
    { 0xb84634c1, SUC_lexicon_tags_13 },
    { 0xda2634c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c634c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4634cb, SUC_lexicon_tags_37 },
    { 0xd6a634cb, SUC_lexicon_tags_12 },
    { 0xea4634cd, SUC_lexicon_tags_92 },
    { 0x22e634ce, SUC_lexicon_tags_55 },
    { 0x56634cb, SUC_lexicon_tags_70 },
    { 0x924634d0, SUC_lexicon_tags_17 },
    { 0x5d4634d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x59c634d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9a4634d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8c0634d7, SUC_lexicon_tags_37 },
    { 0x702634d8, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xc74634da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a634e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce634e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc72634e5, SUC_lexicon_tags_2 },
    { 0x2dc634e6, SUC_lexicon_tags_43 },
    { 0xd4634e7, SUC_lexicon_tags_9 },
    { 0x7d6634e8, SUC_lexicon_tags_2 },
    { 0xfd6634e5, SUC_lexicon_tags_25 },
    { 0x560634ea, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0xa14634ec, SUC_lexicon_tags_0 },
    { 0xd74634ec, SUC_lexicon_tags_45 },
    { 0xa0a634ee, SUC_lexicon_tags_11 },
    { 0x1d6634ef, SUC_lexicon_tags_7 },
    { 0x7c6634ef, SUC_lexicon_tags_28 },
    { 0xfaa634f1, SUC_lexicon_tags_106 },
    { 0xf22634ec, SUC_lexicon_tags_17 },
    { 0x674634ec, SUC_lexicon_tags_14 },
    { 0xac4634f4, SUC_lexicon_tags_17 },
    { 0x7bc634f5, SUC_lexicon_tags_28 },
    { 0x210634f3, SUC_lexicon_tags_11 },
    { 0x9e2634f2, SUC_lexicon_tags_29 },
    { 0x3f8634f2, SUC_lexicon_tags_39 },
    { 0xf48634f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf46634fb, SUC_lexicon_tags_33 },
    { 0x61c634fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45063500, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2663504, SUC_lexicon_tags_37 },
    { 0xde863505, SUC_lexicon_tags_33 },
    { 0xabe63506, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f863508, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x54e6350a, SUC_lexicon_tags_37 },
    { 0xb426350b, SUC_lexicon_tags_38 },
    { 0xc2c6350b, SUC_lexicon_tags_12 },
    { 0xf6c6350d, SUC_lexicon_tags_46 },
    { 0x32a6350d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d463510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23263515, SUC_lexicon_tags_31 },
    { 0x6ea63516, SUC_lexicon_tags_37 },
    { 0x26c63517, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e06351c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf466351e, SUC_lexicon_tags_11 },
    { 0x3866351e, SUC_lexicon_tags_11 },
    { 0x23e6351f, SUC_lexicon_tags_7 },
    { 0xdec63521, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee463523, SUC_lexicon_tags_9 },
    { 0x40c63524, SUC_lexicon_tags_14 },
    { 0x57663525, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4863527, SUC_lexicon_tags_87 },
    { 0x84c63528, SUC_lexicon_tags_2 },
    { 0x45663529, SUC_lexicon_tags_12 },
    { 0x4c263529, SUC_lexicon_tags_28 },
    { 0x3ea6352b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c6352e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x28263531, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68e63533, SUC_lexicon_tags_38 },
    { 0x3d063534, SUC_lexicon_tags_27 },
    { 0xe3663533, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x33863537, SUC_lexicon_tags_7 },
    { 0xc4463537, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a26353b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4b26353d, SUC_lexicon_tags_63 },
    { 0x3286353e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e263540, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac063542, SUC_lexicon_tags_87 },
    { 0x55863543, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac863545, SUC_lexicon_tags_172 },
    { 0x20863546, SUC_lexicon_tags_30 },
    { 0x87263547, SUC_lexicon_tags_3 },
    { 0x1e663547, SUC_lexicon_tags_2 },
    { 0xc0463549, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf86354b, SUC_lexicon_tags_15 },
    { 0x46354c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53463552, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9463559, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6355d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7706355f, SUC_lexicon_tags_19 },
    { 0xa4c63560, SUC_lexicon_tags_40 },
    { 0xfac63560, SUC_lexicon_tags_25 },
    { 0xf8263562, SUC_lexicon_tags_25 },
    { 0xb5c6355f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e63567, SUC_lexicon_tags_37 },
    { 0x53463568, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa426356c, SUC_lexicon_tags_6 },
    { 0xdc26356d, SUC_lexicon_tags_11 },
    { 0x9d46356e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4663572, SUC_lexicon_tags_17 },
    { 0x13663573, SUC_lexicon_tags_37 },
    { 0xb6063574, SUC_lexicon_tags_5 },
    { 0x3ca63575, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52063578, SUC_lexicon_tags_14 },
    { 0x51a63578, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbb26357b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe63584, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a63588, SUC_lexicon_tags_2 },
    { 0x63263589, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3886358b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe463590, SUC_lexicon_tags_45 },
    { 0xfbc63591, SUC_lexicon_tags_30 },
    { 0xac263591, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c63598, SUC_lexicon_tags_25 },
    { 0x5de63599, SUC_lexicon_tags_12 },
    { 0xde6359a, SUC_lexicon_tags_43 },
    { 0x35c6359a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce6359f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682635ab, SUC_lexicon_tags_13 },
    { 0xbc6635ac, SUC_lexicon_tags_17 },
    { 0xa2c635ad, SUC_lexicon_tags_37 },
    { 0xce6635ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xca0635b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82635b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb92635b5, SUC_lexicon_tags_19 },
    { 0xa50635b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x984635b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae635bc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x890635bf, SUC_lexicon_tags_38 },
    { 0xc2a635bf, SUC_lexicon_tags_43 },
    { 0xa16635c1, SUC_lexicon_tags_7 },
    { 0xad2635c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466635c8, SUC_lexicon_tags_46 },
    { 0x7b0635c8, SUC_lexicon_tags_0 },
    { 0x13a635c8, SUC_lexicon_tags_17 },
    { 0xdc0635c8, SUC_lexicon_tags_37 },
    { 0xea635cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776635d0, SUC_lexicon_tags_63 },
    { 0x40a635d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95a635d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98635d7, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6635db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2ac635dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb12635df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x774635e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x724635e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0635e8, SUC_lexicon_tags_11 },
    { 0xf8a635e9, SUC_lexicon_tags_14 },
    { 0x1c635ea, SUC_lexicon_tags_68 },
    { 0x2ca635eb, SUC_lexicon_tags_55 },
    { 0x67e635ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212635f3, SUC_lexicon_tags_47 },
    { 0x958635f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x656635f6, SUC_lexicon_tags_69 },
    { 0x856635f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa635ff, SUC_lexicon_tags_63 },
    { 0x964635ff, SUC_lexicon_tags_0 },
    { 0xf1463600, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e263605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec63609, SUC_lexicon_tags_1 },
    { 0xbde6360a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59463610, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e63616, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3486361e, SUC_lexicon_tags_28 },
    { 0x8aa6361f, SUC_lexicon_tags_14 },
    { 0xc4a6361f, SUC_lexicon_tags_48 },
    { 0x33e63621, SUC_lexicon_tags_37 },
    { 0xe086361f, SUC_lexicon_tags_2 },
    { 0x62e63623, SUC_lexicon_tags_12 },
    { 0x31c6361e, SUC_lexicon_tags_69 },
    { 0x6cc63620, SUC_lexicon_tags_30 },
    { 0xe8263626, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x95063629, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd926362c, SUC_lexicon_tags_17 },
    { 0xea6362d, SUC_lexicon_tags_1 },
    { 0x2b46362d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9863632, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba463634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91e63636, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2263638, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e6363b, SUC_lexicon_tags_45 },
    { 0x7546363c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb06363e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e63644, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe63651, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8263656, SUC_lexicon_tags_7 },
    { 0xade63656, SUC_lexicon_tags_1 },
    { 0x72463657, SUC_lexicon_tags_2 },
    { 0x65663659, SUC_lexicon_tags_12 },
    { 0xa3c63658, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13c6365c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d66365e, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e63661, SUC_lexicon_tags_1 },
    { 0xc4e63662, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6366366d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f063673, SUC_lexicon_tags_2 },
    { 0x4d663673, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca63677, SUC_lexicon_tags_7 },
    { 0x24263678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3926367b, SUC_lexicon_tags_7 },
    { 0x9be6367c, SUC_lexicon_tags_37 },
    { 0xaf66367c, SUC_lexicon_tags_9 },
    { 0x4f26367e, SUC_lexicon_tags_47 },
    { 0xfc86367c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c63683, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa263687, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9263689, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7006368c, SUC_lexicon_tags_38 },
    { 0xca66368d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b263691, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc63696, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c26369b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1e66369d, SUC_lexicon_tags_3 },
    { 0x6e86369e, SUC_lexicon_tags_37 },
    { 0x6886369f, SUC_lexicon_tags_11 },
    { 0x102636a0, SUC_lexicon_tags_7 },
    { 0x9ae636a0, SUC_lexicon_tags_33 },
    { 0x6006369f, SUC_lexicon_tags_38 },
    { 0xe26636a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114636a7, SUC_lexicon_tags_31 },
    { 0x8c2636a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6636ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2636b1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc92636b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae636b6, SUC_lexicon_tags_66 },
    { 0x2d4636b6, SUC_lexicon_tags_65 },
    { 0xca4636b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee636bb, SUC_lexicon_tags_25 },
    { 0xfbe636bb, SUC_lexicon_tags_14 },
    { 0x6de636bd, SUC_lexicon_tags_37 },
    { 0x678636bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa42636c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbfa636c2, SUC_lexicon_tags_45 },
    { 0x4ca636c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x93c636c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282636d6, SUC_lexicon_tags_23 },
    { 0x54c636d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8ec636d9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc636dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xffc636de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ae636e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa636e7, SUC_lexicon_tags_17 },
    { 0xc58636e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e636ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x322636ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x686636f2, SUC_lexicon_tags_2 },
    { 0xe08636f3, SUC_lexicon_tags_15 },
    { 0xb0c636f2, SUC_lexicon_tags_9 },
    { 0x430636f5, SUC_lexicon_tags_14 },
    { 0x418636f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x760636f8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6636ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c63703, SUC_lexicon_tags_27 },
    { 0xabc63704, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4863706, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4846370a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd326370e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb463712, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabe63714, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9863717, SUC_lexicon_tags_63 },
    { 0xbf663718, SUC_lexicon_tags_65 },
    { 0xfc663719, SUC_lexicon_tags_151 },
    { 0x5166371a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14a6371c, SUC_lexicon_tags_56 },
    { 0xe486371d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea63723, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x21463725, SUC_lexicon_tags_34 },
    { 0xeec63725, SUC_lexicon_tags_15 },
    { 0xee063727, SUC_lexicon_tags_14 },
    { 0x5463728, SUC_lexicon_tags_37 },
    { 0x93e63725, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaae6372b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d46372d, SUC_lexicon_tags_92 },
    { 0x2426372e, SUC_lexicon_tags_17 },
    { 0xe866372e, SUC_lexicon_tags_9 },
    { 0xb666372f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29863734, SUC_lexicon_tags_34 },
    { 0xf5063735, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d263738, SUC_lexicon_tags_7 },
    { 0x1a263738, SUC_lexicon_tags_88 },
    { 0x13063738, SUC_lexicon_tags_7 },
    { 0xfac6373b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e66373e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a63741, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb8c63743, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac863748, SUC_lexicon_tags_25 },
    { 0xc0663749, SUC_lexicon_tags_15 },
    { 0x55c6374a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a6374f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc63753, SUC_lexicon_tags_0 },
    { 0xb8c63754, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be6375c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c63761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2063763, SUC_lexicon_tags_82 },
    { 0x43e63764, SUC_lexicon_tags_7 },
    { 0xd7863764, SUC_lexicon_tags_19 },
    { 0x68663764, SUC_lexicon_tags_27 },
    { 0x47063765, SUC_lexicon_tags_37 },
    { 0xa6663768, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa6376c, SUC_lexicon_tags_1 },
    { 0x4126376c, SUC_lexicon_tags_18 },
    { 0x1746376e, SUC_lexicon_tags_45 },
    { 0xfa6376f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x46463771, SUC_lexicon_tags_7 },
    { 0xe9863772, SUC_lexicon_tags_23 },
    { 0x90263773, SUC_lexicon_tags_65 },
    { 0xf3463773, SUC_lexicon_tags_78 },
    { 0xf4863774, SUC_lexicon_tags_30 },
    { 0x78463776, SUC_lexicon_tags_21 },
    { 0xe7263777, SUC_lexicon_tags_17 },
    { 0xbaa63773, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a6377c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd166377f, SUC_lexicon_tags_11 },
    { 0xb606377f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae63783, SUC_lexicon_tags_14 },
    { 0x5d263784, SUC_lexicon_tags_17 },
    { 0xf1063785, SUC_lexicon_tags_343 },
    { 0xe5a63786, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8da63788, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc6378d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd6a6378f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x82c63791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd863793, SUC_lexicon_tags_35 },
    { 0xf9c63794, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c637a0, SUC_lexicon_tags_25 },
    { 0xe4c637a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e637a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2637a7, SUC_lexicon_tags_15 },
    { 0xca2637a8, SUC_lexicon_tags_1 },
    { 0x622637a9, SUC_lexicon_tags_65 },
    { 0x34a637aa, SUC_lexicon_tags_45 },
    { 0xfca637ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528637b0, SUC_lexicon_tags_2 },
    { 0x4c2637b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a637b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4637b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850637bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a637be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a637c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fe637c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x124637c6, SUC_lexicon_tags_14 },
    { 0x4a2637c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368637cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8637d0, SUC_lexicon_tags_17 },
    { 0xff6637d0, SUC_lexicon_tags_8 },
    { 0xeec637d2, SUC_lexicon_tags_37 },
    { 0x5a8637d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a637d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96637db, SUC_lexicon_tags_162 },
    { 0x652637dc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2637e1, SUC_lexicon_tags_12 },
    { 0x55a637e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a637e5, SUC_lexicon_tags_56 },
    { 0x258637e6, SUC_lexicon_tags_46 },
    { 0xe1c637e7, SUC_lexicon_tags_37 },
    { 0x22c637e8, SUC_lexicon_tags_47 },
    { 0x74c637e8, SUC_lexicon_tags_47 },
    { 0xeda637ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc2637ec, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0x120637ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2be637f1, SUC_lexicon_tags_49 },
    { 0x13c637f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec0637f4, SUC_lexicon_tags_15 },
    { 0xc0637f5, SUC_lexicon_tags_113 },
    { 0x60a637f4, SUC_lexicon_tags_1 },
    { 0x5c4637f7, SUC_lexicon_tags_7 },
    { 0xf7c637f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x668637fa, SUC_lexicon_tags_14 },
    { 0xf64637fb, SUC_lexicon_tags_15 },
    { 0xdf0637fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a63800, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8463803, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x23063805, SUC_lexicon_tags_23 },
    { 0xf4663805, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x76463808, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c6380b, SUC_lexicon_tags_55 },
    { 0x1826380b, SUC_lexicon_tags_61 },
    { 0x5226380d, SUC_lexicon_tags_34 },
    { 0xece6380d, SUC_lexicon_tags_87 },
    { 0x36c6380f, SUC_lexicon_tags_17 },
    { 0x8186380d, SUC_lexicon_tags_66 },
    { 0x6d06380f, SUC_lexicon_tags_16 },
    { 0x49263812, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7463814, SUC_lexicon_tags_17 },
    { 0xf0063815, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf863818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a66381c, SUC_lexicon_tags_63 },
    { 0xc886381c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1063822, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f863825, SUC_lexicon_tags_17 },
    { 0xca663826, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7663828, SUC_lexicon_tags_7 },
    { 0x9d463829, SUC_lexicon_tags_31 },
    { 0x4b263828, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf526382f, SUC_lexicon_tags_25 },
    { 0x37e63830, SUC_lexicon_tags_7 },
    { 0x90863831, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbee63833, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd463839, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb26383b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8566383d, SUC_lexicon_tags_37 },
    { 0xcde6383e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a63849, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c6384d, SUC_lexicon_tags_1 },
    { 0x9c46384e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf7663850, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d663853, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5263857, SUC_lexicon_tags_47 },
    { 0x4c463857, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e6385c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be63866, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e6386a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4186386f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94863873, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c86387c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad663882, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb4463884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84263886, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5663889, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf86388b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba63891, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce263899, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9186389d, SUC_lexicon_tags_1 },
    { 0x2566389e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6638a8, SUC_lexicon_tags_45 },
    { 0x694638a9, SUC_lexicon_tags_37 },
    { 0xe638aa, SUC_lexicon_tags_14 },
    { 0x1f0638a9, SUC_lexicon_tags_40 },
    { 0x664638aa, SUC_lexicon_tags_47 },
    { 0x854638aa, SUC_lexicon_tags_37 },
    { 0xc52638ae, SUC_lexicon_tags_37 },
    { 0x33e638ae, SUC_lexicon_tags_11 },
    { 0xae6638b0, SUC_lexicon_tags_45 },
    { 0x2b0638b0, SUC_lexicon_tags_8 },
    { 0x842638ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa96638b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6638ba, SUC_lexicon_tags_25 },
    { 0xfd4638bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be638bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75a638c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x470638c3, SUC_lexicon_tags_25 },
    { 0x574638c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644638c8, SUC_lexicon_tags_15 },
    { 0x558638c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb74638cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x448638ce, SUC_lexicon_tags_37 },
    { 0xee6638cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718638d6, SUC_lexicon_tags_7 },
    { 0xf1c638d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8638dc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa638e1, SUC_lexicon_tags_37 },
    { 0xd20638e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754638ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192638f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2638f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00638fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x852638fd, SUC_lexicon_tags_12 },
    { 0x512638fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5063900, SUC_lexicon_tags_17 },
    { 0xede63901, SUC_lexicon_tags_6 },
    { 0xa7663901, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda63906, SUC_lexicon_tags_26 },
    { 0xdcc63907, SUC_lexicon_tags_37 },
    { 0x40e63908, SUC_lexicon_tags_11 },
    { 0x4e863906, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xff46390b, SUC_lexicon_tags_45 },
    { 0x3046390c, SUC_lexicon_tags_12 },
    { 0xd7c6390d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72663913, SUC_lexicon_tags_25 },
    { 0xf2863914, SUC_lexicon_tags_14 },
    { 0xcd663914, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x53263917, SUC_lexicon_tags_6 },
    { 0x4c863918, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9726391b, SUC_lexicon_tags_49 },
    { 0xf326391c, SUC_lexicon_tags_15 },
    { 0xef26391d, SUC_lexicon_tags_66 },
    { 0xc446391d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a063924, SUC_lexicon_tags_27 },
    { 0x7e863925, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae6392f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92e63931, SUC_lexicon_tags_15 },
    { 0x49463931, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdde63934, SUC_lexicon_tags_38 },
    { 0xe7c63935, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c63937, SUC_lexicon_tags_25 },
    { 0x63463937, SUC_lexicon_tags_13 },
    { 0x58863939, SUC_lexicon_tags_37 },
    { 0x8c663937, SUC_lexicon_tags_4 },
    { 0x2966393b, SUC_lexicon_tags_285 },
    { 0xdc06393c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9166393f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49463946, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc906394f, SUC_lexicon_tags_17 },
    { 0x6e463950, SUC_lexicon_tags_9 },
    { 0xde263951, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94a63953, SUC_lexicon_tags_46 },
    { 0x1f663954, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a6395b, SUC_lexicon_tags_14 },
    { 0x27c6395c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65463961, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f663966, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a6396d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf063970, SUC_lexicon_tags_23 },
    { 0xd0c63970, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d063974, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8786397e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fa63980, SUC_lexicon_tags_27 },
    { 0xcfc63980, SUC_lexicon_tags_48 },
    { 0x88863982, SUC_lexicon_tags_7 },
    { 0x7b263982, SUC_lexicon_tags_49 },
    { 0x68063983, SUC_lexicon_tags_33 },
    { 0xac463985, SUC_lexicon_tags_47 },
    { 0xfb663986, SUC_lexicon_tags_61 },
    { 0x29463982, SUC_lexicon_tags_95 },
    { 0x8d063987, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186398f, SUC_lexicon_tags_30 },
    { 0xd9863990, SUC_lexicon_tags_12 },
    { 0x96863991, SUC_lexicon_tags_230 },
    { 0x51463991, SUC_lexicon_tags_37 },
    { 0xee463993, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80063995, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa06399a, SUC_lexicon_tags_16 },
    { 0x3c46399b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e6399f, SUC_lexicon_tags_37 },
    { 0xce639a0, SUC_lexicon_tags_7 },
    { 0xe5e639a1, SUC_lexicon_tags_15 },
    { 0xe54639a2, SUC_lexicon_tags_25 },
    { 0x69c639a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4639a6, SUC_lexicon_tags_17 },
    { 0xda639a7, SUC_lexicon_tags_11 },
    { 0xb44639a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac639ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818639b0, SUC_lexicon_tags_193 },
    { 0xd94639b0, SUC_lexicon_tags_1 },
    { 0xb7e639b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf48639b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84639b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a639bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc0639bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e639c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x91a639c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6639c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46639cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x190639cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd2639d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d0639d3, SUC_lexicon_tags_6 },
    { 0xbac639d4, SUC_lexicon_tags_7 },
    { 0x336639d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c639db, SUC_lexicon_tags_14 },
    { 0x952639db, SUC_lexicon_tags_33 },
    { 0xee639dc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x972639e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e639e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca639e8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcae639ea, SUC_lexicon_tags_14 },
    { 0x6c4639eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6639ee, SUC_lexicon_tags_32 },
    { 0x3da639ee, SUC_lexicon_tags_29 },
    { 0xd32639f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc4639f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc20639f4, SUC_lexicon_tags_17 },
    { 0x434639f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeda639f7, SUC_lexicon_tags_1 },
    { 0x956639f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf74639fa, SUC_lexicon_tags_2 },
    { 0xf30639fa, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a639ff, SUC_lexicon_tags_25 },
    { 0x7ae63a00, SUC_lexicon_tags_1 },
    { 0x1ee63a00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68863a03, SUC_lexicon_tags_23 },
    { 0x7aa63a03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f463a08, SUC_lexicon_tags_65 },
    { 0xbec63a08, SUC_lexicon_tags_55 },
    { 0x13063a0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a63a10, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x92063a12, SUC_lexicon_tags_45 },
    { 0xba63a12, SUC_lexicon_tags_27 },
    { 0xbb663a14, SUC_lexicon_tags_13 },
    { 0xbac63a14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20263a19, SUC_lexicon_tags_37 },
    { 0xa2663a19, SUC_lexicon_tags_15 },
    { 0x35a63a1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2a663a1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0863a1f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2063a2a, SUC_lexicon_tags_1 },
    { 0x14c63a2b, SUC_lexicon_tags_30 },
    { 0x3263a2a, SUC_lexicon_tags_1 },
    { 0xa3863a2c, SUC_lexicon_tags_4 },
    { 0xcec63a2d, SUC_lexicon_tags_63 },
    { 0xb5463a2f, SUC_lexicon_tags_17 },
    { 0xeb463a30, SUC_lexicon_tags_2 },
    { 0x3e63a31, SUC_lexicon_tags_14 },
    { 0xe3463a2c, SUC_lexicon_tags_25 },
    { 0x21063a2c, SUC_lexicon_tags_46 },
    { 0xd6c63a2c, SUC_lexicon_tags_17 },
    { 0x2be63a35, SUC_lexicon_tags_16 },
    { 0xfd863a35, SUC_lexicon_tags_11 },
    { 0xe1463a31, SUC_lexicon_tags_11 },
    { 0x93c63a38, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea63a41, SUC_lexicon_tags_25 },
    { 0x8ce63a42, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a63a4a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a63a51, SUC_lexicon_tags_12 },
    { 0x29a63a52, SUC_lexicon_tags_1 },
    { 0x82263a53, SUC_lexicon_tags_7 },
    { 0x66263a54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd063a59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e63a5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a63a67, SUC_lexicon_tags_7 },
    { 0xe3463a68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd263a6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec63a6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0263a73, SUC_lexicon_tags_26 },
    { 0xf2e63a74, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe63a7c, SUC_lexicon_tags_113 },
    { 0x1ae63a7c, SUC_lexicon_tags_12 },
    { 0x74c63a7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c63a83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50463a85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44a63a87, SUC_lexicon_tags_10 },
    { 0x4f663a88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x87063a8a, SUC_lexicon_tags_1156 },
    { 0xd1c63a8b, SUC_lexicon_tags_45 },
    { 0xb3c63a8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e63a95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a63a99, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x82263a9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee063aa1, SUC_lexicon_tags_3 },
    { 0x4fa63aa2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1063aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b863ab3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c863aba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12463abd, SUC_lexicon_tags_8 },
    { 0xb9c63abe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15263ac0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c863ac5, SUC_lexicon_tags_25 },
    { 0xace63ac6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c63aca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1e63acc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc9a63ace, SUC_lexicon_tags_15 },
    { 0xbda63acf, SUC_lexicon_tags_124 },
    { 0xa3e63ad0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3263ad4, SUC_lexicon_tags_15 },
    { 0x2f663ad5, SUC_lexicon_tags_8 },
    { 0x84063ad6, SUC_lexicon_tags_14 },
    { 0x37063ad5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdae63ad9, SUC_lexicon_tags_11 },
    { 0xf5e63ada, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a63ae0, SUC_lexicon_tags_39 },
    { 0x25263ae1, SUC_lexicon_tags_49 },
    { 0x6a63ae1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43e63ae4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee63aee, SUC_lexicon_tags_2 },
    { 0x6c063aef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34063af7, SUC_lexicon_tags_26 },
    { 0x8463af8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0263afc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3c63afe, SUC_lexicon_tags_37 },
    { 0x26663aff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x67e63b01, SUC_lexicon_tags_40 },
    { 0xa0263b02, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x1c063b04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8663b09, SUC_lexicon_tags_37 },
    { 0xfa863b09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42463b14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef463b16, SUC_lexicon_tags_48 },
    { 0x76263b17, SUC_lexicon_tags_17 },
    { 0x65c63b18, SUC_lexicon_tags_25 },
    { 0xa9e63b18, SUC_lexicon_tags_7 },
    { 0x71463b1a, SUC_lexicon_tags_17 },
    { 0x83463b16, SUC_lexicon_tags_29 },
    { 0xe3463b1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19463b21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80463b23, SUC_lexicon_tags_53 },
    { 0x15c63b23, SUC_lexicon_tags_15 },
    { 0x9ba63b25, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x60c63b27, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23663b32, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663b36, SUC_lexicon_tags_19 },
    { 0x48a63b37, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad663b3c, SUC_lexicon_tags_25 },
    { 0xe1e63b3d, SUC_lexicon_tags_46 },
    { 0x1ae63b3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa463b40, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x61063b42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf7e63b44, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd263b48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a63b4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c463b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57663b54, SUC_lexicon_tags_38 },
    { 0x93463b55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a63b58, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e263b5b, SUC_lexicon_tags_23 },
    { 0x43c63b5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x41063b5e, SUC_lexicon_tags_15 },
    { 0x6a63b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fa63b61, SUC_lexicon_tags_10 },
    { 0x35e63b62, SUC_lexicon_tags_0 },
    { 0xbb463b63, SUC_lexicon_tags_17 },
    { 0xb8a63b64, SUC_lexicon_tags_0 },
    { 0x6fc63b63, SUC_lexicon_tags_17 },
    { 0xfc063b61, SUC_lexicon_tags_25 },
    { 0x4a463b67, SUC_lexicon_tags_1 },
    { 0xab263b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa63b6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc63b6e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27863b74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9d663b76, SUC_lexicon_tags_49 },
    { 0x24463b77, SUC_lexicon_tags_25 },
    { 0xbe663b78, SUC_lexicon_tags_7 },
    { 0x27e63b78, SUC_lexicon_tags_12 },
    { 0x40e63b7a, SUC_lexicon_tags_15 },
    { 0xbce63b7b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e63b7e, SUC_lexicon_tags_1 },
    { 0x67863b7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83a63b81, SUC_lexicon_tags_23 },
    { 0x7dc63b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb863b85, SUC_lexicon_tags_15 },
    { 0x463b85, SUC_lexicon_tags_14 },
    { 0x26e63b86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x88263b89, SUC_lexicon_tags_1 },
    { 0xa9a63b8a, SUC_lexicon_tags_46 },
    { 0x84263b8a, SUC_lexicon_tags_12 },
    { 0xc7863b89, SUC_lexicon_tags_38 },
    { 0x8863b8d, SUC_lexicon_tags_11 },
    { 0xedc63b8d, SUC_lexicon_tags_25 },
    { 0xe2a63b8e, SUC_lexicon_tags_45 },
    { 0xa0463b8e, SUC_lexicon_tags_68 },
    { 0x18a63b91, SUC_lexicon_tags_1 },
    { 0x8c263b91, SUC_lexicon_tags_1 },
    { 0xe0863b92, SUC_lexicon_tags_11 },
    { 0xa6c63b94, SUC_lexicon_tags_29 },
    { 0xb7263b94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93063b99, SUC_lexicon_tags_11 },
    { 0x60463b99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e63b9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c63ba0, SUC_lexicon_tags_11 },
    { 0xf2663ba0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x71063ba4, SUC_lexicon_tags_1 },
    { 0x1a663ba5, SUC_lexicon_tags_26 },
    { 0xee463ba6, SUC_lexicon_tags_15 },
    { 0x11e63ba7, SUC_lexicon_tags_37 },
    { 0xf4a63ba6, SUC_lexicon_tags_25 },
    { 0xb7663ba6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda63bad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea63bb0, SUC_lexicon_tags_1286 },
    { 0, NULL },
    { 0xc9063bb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a263bb4, SUC_lexicon_tags_9 },
    { 0x5c463bb5, SUC_lexicon_tags_28 },
    { 0xad663bb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff863bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8463bbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be63bc0, SUC_lexicon_tags_87 },
    { 0x45263bc0, SUC_lexicon_tags_1 },
    { 0xbb063bc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a63bc6, SUC_lexicon_tags_29 },
    { 0x38463bc7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83263bc9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e263bcb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x53463bce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0663bd4, SUC_lexicon_tags_55 },
    { 0x1aa63bd5, SUC_lexicon_tags_30 },
    { 0x90463bd5, SUC_lexicon_tags_37 },
    { 0x5b063bd7, SUC_lexicon_tags_14 },
    { 0x8b063bd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe4463bda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa263bdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7263be4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe2263be6, SUC_lexicon_tags_80 },
    { 0xfd863be7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb463be9, SUC_lexicon_tags_17 },
    { 0x6aa63bea, SUC_lexicon_tags_1 },
    { 0x2fc63beb, SUC_lexicon_tags_60 },
    { 0xfd063bec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde463bef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e663bf1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24c63bf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe63bf6, SUC_lexicon_tags_25 },
    { 0xf9463bf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64063bfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21e63bfd, SUC_lexicon_tags_7 },
    { 0x7ea63bfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca63c02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f463c05, SUC_lexicon_tags_85 },
    { 0x19063c06, SUC_lexicon_tags_70 },
    { 0x76863c07, SUC_lexicon_tags_87 },
    { 0xe9863c08, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa263c0b, SUC_lexicon_tags_37 },
    { 0xb0c63c0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0063c0e, SUC_lexicon_tags_25 },
    { 0x8f463c0f, SUC_lexicon_tags_65 },
    { 0x98263c0f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb1463c12, SUC_lexicon_tags_11 },
    { 0x2b863c12, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x32663c15, SUC_lexicon_tags_33 },
    { 0xaf263c15, SUC_lexicon_tags_33 },
    { 0x16063c15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be63c1c, SUC_lexicon_tags_62 },
    { 0x2b863c1d, SUC_lexicon_tags_2 },
    { 0x46063c1e, SUC_lexicon_tags_9 },
    { 0x80863c1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x97c63c21, SUC_lexicon_tags_47 },
    { 0x7ce63c22, SUC_lexicon_tags_114 },
    { 0xbb263c22, SUC_lexicon_tags_45 },
    { 0xfd663c23, SUC_lexicon_tags_15 },
    { 0x5463c25, SUC_lexicon_tags_17 },
    { 0xe9063c26, SUC_lexicon_tags_15 },
    { 0x31c63c25, SUC_lexicon_tags_0 },
    { 0x6ce63c27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a463c2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba63c34, SUC_lexicon_tags_0 },
    { 0x76a63c35, SUC_lexicon_tags_664 },
    { 0xd2663c36, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5a663c38, SUC_lexicon_tags_14 },
    { 0x52063c38, SUC_lexicon_tags_26 },
    { 0xbcc63c39, SUC_lexicon_tags_15 },
    { 0xad863c38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2263c3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfb063c3f, SUC_lexicon_tags_27 },
    { 0xa863c3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6863c46, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1c63c48, SUC_lexicon_tags_10 },
    { 0x48663c49, SUC_lexicon_tags_56 },
    { 0x73c63c49, SUC_lexicon_tags_17 },
    { 0xedc63c4a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x55063c4d, SUC_lexicon_tags_0 },
    { 0x9e463c4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x71463c50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b863c52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c63c55, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x92663c58, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x55263c5a, SUC_lexicon_tags_114 },
    { 0x8063c5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a63c61, SUC_lexicon_tags_14 },
    { 0xdc663c62, SUC_lexicon_tags_6 },
    { 0x26063c63, SUC_lexicon_tags_17 },
    { 0xb2463c64, SUC_lexicon_tags_6 },
    { 0xd7863c65, SUC_lexicon_tags_25 },
    { 0x35663c62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80e63c68, SUC_lexicon_tags_38 },
    { 0x10263c69, SUC_lexicon_tags_89 },
    { 0xab663c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16263c6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85063c6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55863c70, SUC_lexicon_tags_25 },
    { 0x44663c70, SUC_lexicon_tags_15 },
    { 0xaf663c72, SUC_lexicon_tags_23 },
    { 0x2d263c73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a63c7a, SUC_lexicon_tags_19 },
    { 0x28263c7a, SUC_lexicon_tags_38 },
    { 0x4263c7c, SUC_lexicon_tags_136 },
    { 0x85263c7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75663c7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa63c83, SUC_lexicon_tags_30 },
    { 0x89263c84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec63c88, SUC_lexicon_tags_12 },
    { 0xef463c89, SUC_lexicon_tags_60 },
    { 0xf663c89, SUC_lexicon_tags_59 },
    { 0xeba63c88, SUC_lexicon_tags_87 },
    { 0x87463c8c, SUC_lexicon_tags_49 },
    { 0xa3e63c8c, SUC_lexicon_tags_28 },
    { 0x88263c8a, SUC_lexicon_tags_12 },
    { 0x1f663c8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6063c91, SUC_lexicon_tags_60 },
    { 0x1863c92, SUC_lexicon_tags_23 },
    { 0x9e63c92, SUC_lexicon_tags_56 },
    { 0xa7e63c93, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xb263c97, SUC_lexicon_tags_2 },
    { 0x37e63c97, SUC_lexicon_tags_11 },
    { 0x33c63c98, SUC_lexicon_tags_12 },
    { 0xc3a63c97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66263c9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd663c9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c63ca2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34463ca5, SUC_lexicon_tags_25 },
    { 0x48863ca5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed463cac, SUC_lexicon_tags_7 },
    { 0x2ec63cac, SUC_lexicon_tags_37 },
    { 0xec863cae, SUC_lexicon_tags_17 },
    { 0x1da63caf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a63cb2, SUC_lexicon_tags_14 },
    { 0x14863cb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e63cb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7063cbe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf663cc6, SUC_lexicon_tags_17 },
    { 0xfb263cc7, SUC_lexicon_tags_65 },
    { 0x75c63cc7, SUC_lexicon_tags_12 },
    { 0xb5463cc9, SUC_lexicon_tags_23 },
    { 0x97863cc9, SUC_lexicon_tags_12 },
    { 0xc8a63ccb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a63cd1, SUC_lexicon_tags_6 },
    { 0x1dc63cd1, SUC_lexicon_tags_8 },
    { 0xbc663cd3, SUC_lexicon_tags_85 },
    { 0x3a63cd4, SUC_lexicon_tags_47 },
    { 0x4663cd5, SUC_lexicon_tags_25 },
    { 0x12863cd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46463cd8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd1863cda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c63ce0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34863ce4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ee63ce6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f863ce9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81063ceb, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43063cf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d263cf2, SUC_lexicon_tags_23 },
    { 0xc9263cf2, SUC_lexicon_tags_28 },
    { 0x48863cf3, SUC_lexicon_tags_29 },
    { 0xfaa63cf5, SUC_lexicon_tags_38 },
    { 0xf1c63cf6, SUC_lexicon_tags_48 },
    { 0xa5663cf6, SUC_lexicon_tags_10 },
    { 0x6663cf7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd863cfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95263cfd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8b463cff, SUC_lexicon_tags_25 },
    { 0x9663d00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a063d03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e63d07, SUC_lexicon_tags_17 },
    { 0x4bc63d08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42c63d0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x32663d0d, SUC_lexicon_tags_11 },
    { 0x13463d0e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x59463d11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4863d13, SUC_lexicon_tags_104 },
    { 0x8c263d14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2063d16, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8c663d18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a63d1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89063d20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x19a63d22, SUC_lexicon_tags_21 },
    { 0xd1a63d23, SUC_lexicon_tags_8 },
    { 0x77863d24, SUC_lexicon_tags_38 },
    { 0x7a463d25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e63d29, SUC_lexicon_tags_2 },
    { 0xdd063d2a, SUC_lexicon_tags_12 },
    { 0xa9663d2a, SUC_lexicon_tags_24 },
    { 0xb5863d2c, SUC_lexicon_tags_47 },
    { 0x16c63d2c, SUC_lexicon_tags_23 },
    { 0x7e063d2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e63d30, SUC_lexicon_tags_17 },
    { 0xe0a63d31, SUC_lexicon_tags_38 },
    { 0xbca63d31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea063d34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f463d37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfaa63d39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e63d3c, SUC_lexicon_tags_63 },
    { 0x8c063d3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31863d42, SUC_lexicon_tags_12 },
    { 0x10463d43, SUC_lexicon_tags_45 },
    { 0x14263d43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae63d4a, SUC_lexicon_tags_17 },
    { 0xde63d4a, SUC_lexicon_tags_42 },
    { 0x8f663d4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa63d4f, SUC_lexicon_tags_37 },
    { 0x18863d4f, SUC_lexicon_tags_37 },
    { 0x2b063d51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb263d58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83263d5c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x54063d5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x16a63d61, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4663d63, SUC_lexicon_tags_25 },
    { 0xe2663d64, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc0663d66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa863d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cc63d6a, SUC_lexicon_tags_37 },
    { 0x33663d6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95263d6f, SUC_lexicon_tags_15 },
    { 0xf2263d70, SUC_lexicon_tags_26 },
    { 0x6c663d71, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d063d74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x93063d77, SUC_lexicon_tags_39 },
    { 0x18663d77, SUC_lexicon_tags_63 },
    { 0xc063d79, SUC_lexicon_tags_6 },
    { 0x8f863d7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa63d7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4863d82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b663d85, SUC_lexicon_tags_0 },
    { 0xa9c63d85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10063d8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x75463d8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97463d95, SUC_lexicon_tags_377 },
    { 0x6063d96, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd863d99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2463da1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea63da4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93063da7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1863dab, SUC_lexicon_tags_56 },
    { 0xdd663dab, SUC_lexicon_tags_6 },
    { 0x47463dad, SUC_lexicon_tags_26 },
    { 0x36a63dad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43a63db0, SUC_lexicon_tags_48 },
    { 0x1c863db1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38663db4, SUC_lexicon_tags_63 },
    { 0x72a63db5, SUC_lexicon_tags_11 },
    { 0x9c63db6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f663dba, SUC_lexicon_tags_38 },
    { 0xdd063dba, SUC_lexicon_tags_16 },
    { 0x97663dbc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c63dbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c63dc2, SUC_lexicon_tags_12 },
    { 0x1f263dc3, SUC_lexicon_tags_19 },
    { 0x10863dc4, SUC_lexicon_tags_27 },
    { 0x1c63dc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf863dc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2063dcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a63dcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6e63dd1, SUC_lexicon_tags_87 },
    { 0x43063dd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc63dd7, SUC_lexicon_tags_12 },
    { 0x93263dd8, SUC_lexicon_tags_9 },
    { 0x4bc63dd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22c63ddb, SUC_lexicon_tags_7 },
    { 0x8a863ddb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663de0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a263deb, SUC_lexicon_tags_38 },
    { 0x8be63dec, SUC_lexicon_tags_49 },
    { 0x1b263ded, SUC_lexicon_tags_7 },
    { 0x50663deb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb463df4, SUC_lexicon_tags_8 },
    { 0xdfa63df4, SUC_lexicon_tags_65 },
    { 0x32463df4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d263e04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46863e06, SUC_lexicon_tags_37 },
    { 0xa5463e07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1263e0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce863e0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5463e12, SUC_lexicon_tags_11 },
    { 0xd8a63e12, SUC_lexicon_tags_60 },
    { 0xcea63e14, SUC_lexicon_tags_37 },
    { 0x9b663e14, SUC_lexicon_tags_63 },
    { 0x8d863e16, SUC_lexicon_tags_0 },
    { 0x10663e17, SUC_lexicon_tags_45 },
    { 0xe7e63e17, SUC_lexicon_tags_37 },
    { 0xb9263e18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38463e1d, SUC_lexicon_tags_45 },
    { 0x20663e1e, SUC_lexicon_tags_26 },
    { 0xf5863e1f, SUC_lexicon_tags_66 },
    { 0x98a63e1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x14263e22, SUC_lexicon_tags_7 },
    { 0x41c63e22, SUC_lexicon_tags_151 },
    { 0x25863e22, SUC_lexicon_tags_47 },
    { 0xbde63e25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80863e2a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7063e2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x64063e2e, SUC_lexicon_tags_11 },
    { 0xaac63e2f, SUC_lexicon_tags_65 },
    { 0xda663e30, SUC_lexicon_tags_12 },
    { 0x8e263e30, SUC_lexicon_tags_22 },
    { 0xa5663e31, SUC_lexicon_tags_7 },
    { 0xf8663e33, SUC_lexicon_tags_25 },
    { 0x8da63e34, SUC_lexicon_tags_11 },
    { 0xeba63e33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba63e3a, SUC_lexicon_tags_93 },
    { 0x99063e3b, SUC_lexicon_tags_19 },
    { 0x83063e3c, SUC_lexicon_tags_12 },
    { 0x87663e3d, SUC_lexicon_tags_37 },
    { 0x9ac63e3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86863e43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b863e49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50663e4f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c63e56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cc63e58, SUC_lexicon_tags_0 },
    { 0x9e263e58, SUC_lexicon_tags_45 },
    { 0x47463e59, SUC_lexicon_tags_48 },
    { 0x60063e5b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f463e5e, SUC_lexicon_tags_37 },
    { 0xb8e63e5f, SUC_lexicon_tags_49 },
    { 0x2f463e5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc063e62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef263e64, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x56463e66, SUC_lexicon_tags_7 },
    { 0xc7a63e67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40863e6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3063e70, SUC_lexicon_tags_14 },
    { 0x1d063e71, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8063e79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c63e7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a63e82, SUC_lexicon_tags_63 },
    { 0x8f063e83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89263e86, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e63e8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39463e8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3e63e91, SUC_lexicon_tags_31 },
    { 0x4ec63e91, SUC_lexicon_tags_27 },
    { 0x51863e92, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9863ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8a63eaa, SUC_lexicon_tags_1 },
    { 0xa2a63eab, SUC_lexicon_tags_7 },
    { 0xa6463eaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x96463eaf, SUC_lexicon_tags_7 },
    { 0x5e463eb0, SUC_lexicon_tags_11 },
    { 0xfcc63eb1, SUC_lexicon_tags_0 },
    { 0x8a663eb0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2263eb4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c63eb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98863ec0, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x7ca63ec2, SUC_lexicon_tags_37 },
    { 0xd7863ec3, SUC_lexicon_tags_25 },
    { 0x15e63ec3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0063ecd, SUC_lexicon_tags_6 },
    { 0x9aa63ece, SUC_lexicon_tags_37 },
    { 0xa3063ecd, SUC_lexicon_tags_27 },
    { 0xee063ece, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee63ed7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e63edb, SUC_lexicon_tags_11 },
    { 0x46863edc, SUC_lexicon_tags_25 },
    { 0x3b663edd, SUC_lexicon_tags_11 },
    { 0x9f463edc, SUC_lexicon_tags_23 },
    { 0xd463ede, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0263ee2, SUC_lexicon_tags_25 },
    { 0xb9863ee2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d463eea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef263ef0, SUC_lexicon_tags_1 },
    { 0x12a63ef1, SUC_lexicon_tags_14 },
    { 0xdc863ef1, SUC_lexicon_tags_2 },
    { 0x14c63ef1, SUC_lexicon_tags_65 },
    { 0xc7063ef4, SUC_lexicon_tags_11 },
    { 0x7d263ef5, SUC_lexicon_tags_23 },
    { 0xed063ef5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xee663ef9, SUC_lexicon_tags_61 },
    { 0x96063efa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7a63efc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5c263efe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79663f00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e63f04, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x78863f06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8863f08, SUC_lexicon_tags_0 },
    { 0x42663f08, SUC_lexicon_tags_17 },
    { 0xf1063f0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a63f0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde863f10, SUC_lexicon_tags_47 },
    { 0xa6e63f11, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53863f24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66863f28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc63f2d, SUC_lexicon_tags_28 },
    { 0xe263f2e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a663f31, SUC_lexicon_tags_31 },
    { 0xee663f32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26463f3c, SUC_lexicon_tags_56 },
    { 0xa0a63f3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57463f40, SUC_lexicon_tags_216 },
    { 0x16a63f41, SUC_lexicon_tags_37 },
    { 0xcfa63f42, SUC_lexicon_tags_25 },
    { 0x40663f42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5063f46, SUC_lexicon_tags_104 },
    { 0xebe63f46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c63f4c, SUC_lexicon_tags_10 },
    { 0x4ec63f4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8463f4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fa63f51, SUC_lexicon_tags_45 },
    { 0x97263f52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16463f57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c63f5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96a63f5c, SUC_lexicon_tags_11 },
    { 0xf3a63f5d, SUC_lexicon_tags_10 },
    { 0xe1463f5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2863f68, SUC_lexicon_tags_1 },
    { 0x4a263f69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fe63f6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4863f6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ea63f6f, SUC_lexicon_tags_13 },
    { 0xb5c63f70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f663f73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15663f7b, SUC_lexicon_tags_25 },
    { 0x89863f7b, SUC_lexicon_tags_25 },
    { 0xf9a63f7b, SUC_lexicon_tags_29 },
    { 0xd1663f7e, SUC_lexicon_tags_37 },
    { 0xb8a63f7b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63f85, SUC_lexicon_tags_49 },
    { 0x53e63f86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb863f89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd0a63f8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8063f8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c63f96, SUC_lexicon_tags_1 },
    { 0x6c863f97, SUC_lexicon_tags_4 },
    { 0xc1463f98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81863f9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e63f9e, SUC_lexicon_tags_29 },
    { 0xafe63f9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85463fa5, SUC_lexicon_tags_15 },
    { 0x66063fa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c63fbb, SUC_lexicon_tags_47 },
    { 0xf6c63fbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde63fc2, SUC_lexicon_tags_6 },
    { 0xece63fc2, SUC_lexicon_tags_32 },
    { 0x2e663fc3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4063fc7, SUC_lexicon_tags_14 },
    { 0xea63fc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6a63fca, SUC_lexicon_tags_2 },
    { 0x5d663fcb, SUC_lexicon_tags_65 },
    { 0x91c63fca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36063fd3, SUC_lexicon_tags_27 },
    { 0x2f063fd4, SUC_lexicon_tags_12 },
    { 0xb1263fd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0263fdc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff663fe6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16263fed, SUC_lexicon_tags_1 },
    { 0xc1663fee, SUC_lexicon_tags_17 },
    { 0x92263fee, SUC_lexicon_tags_49 },
    { 0x4ea63fef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c63ff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8263ff9, SUC_lexicon_tags_27 },
    { 0x63e63ff9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d463ffc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca64003, SUC_lexicon_tags_47 },
    { 0x98664004, SUC_lexicon_tags_2 },
    { 0x61a64003, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee464008, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2be6400a, SUC_lexicon_tags_8 },
    { 0xf0a6400b, SUC_lexicon_tags_27 },
    { 0x5ee6400b, SUC_lexicon_tags_23 },
    { 0x5706400d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27064010, SUC_lexicon_tags_5 },
    { 0xf6664011, SUC_lexicon_tags_17 },
    { 0xd1864012, SUC_lexicon_tags_17 },
    { 0xc7064012, SUC_lexicon_tags_15 },
    { 0x8ba64013, SUC_lexicon_tags_7 },
    { 0x6a464015, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd0e64017, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31264019, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e64021, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7664024, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfb664026, SUC_lexicon_tags_8 },
    { 0x99864027, SUC_lexicon_tags_0 },
    { 0x99464027, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ca6402a, SUC_lexicon_tags_27 },
    { 0x5146402b, SUC_lexicon_tags_15 },
    { 0x7106402b, SUC_lexicon_tags_64 },
    { 0xc926402d, SUC_lexicon_tags_0 },
    { 0x5706402a, SUC_lexicon_tags_37 },
    { 0xaac6402e, SUC_lexicon_tags_25 },
    { 0x1ce64030, SUC_lexicon_tags_9 },
    { 0xa7464031, SUC_lexicon_tags_25 },
    { 0xe7864030, SUC_lexicon_tags_85 },
    { 0x9726402c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1864035, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd864037, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf546403a, SUC_lexicon_tags_15 },
    { 0x7866403a, SUC_lexicon_tags_17 },
    { 0x5b26403c, SUC_lexicon_tags_14 },
    { 0xdc46403d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3c6403f, SUC_lexicon_tags_15 },
    { 0x9e6403f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c064042, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da64045, SUC_lexicon_tags_12 },
    { 0xa864046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fe64048, SUC_lexicon_tags_27 },
    { 0xdc664049, SUC_lexicon_tags_25 },
    { 0x3a64049, SUC_lexicon_tags_12 },
    { 0x3346404a, SUC_lexicon_tags_31 },
    { 0xd566404a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e6404f, SUC_lexicon_tags_12 },
    { 0x58864050, SUC_lexicon_tags_47 },
    { 0xc0864051, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9264054, SUC_lexicon_tags_37 },
    { 0x5d864054, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc064058, SUC_lexicon_tags_33 },
    { 0xff064058, SUC_lexicon_tags_8 },
    { 0xdc06405a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5286405c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc64065, SUC_lexicon_tags_9 },
    { 0x13064066, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a6406b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfac6406d, SUC_lexicon_tags_7 },
    { 0x21e6406e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x264070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47864079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92a6407b, SUC_lexicon_tags_2 },
    { 0xbde6407c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50064082, SUC_lexicon_tags_37 },
    { 0x6fa64082, SUC_lexicon_tags_11 },
    { 0x65c64083, SUC_lexicon_tags_31 },
    { 0xace64082, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x95264088, SUC_lexicon_tags_11 },
    { 0x51c64088, SUC_lexicon_tags_28 },
    { 0x43064088, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a46408c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1464091, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x61264094, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba64098, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fc6409a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad26409d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x796640a0, SUC_lexicon_tags_14 },
    { 0xd98640a0, SUC_lexicon_tags_6 },
    { 0x294640a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2640a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x382640a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x95e640ab, SUC_lexicon_tags_23 },
    { 0x99e640ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8640b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea640b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x914640b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70640bc, SUC_lexicon_tags_29 },
    { 0x58640bd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0640c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8640c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2640c9, SUC_lexicon_tags_28 },
    { 0xdfe640ca, SUC_lexicon_tags_2 },
    { 0x1e640cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c640ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa640d1, SUC_lexicon_tags_9 },
    { 0x182640d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36640d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e640d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c640dc, SUC_lexicon_tags_2 },
    { 0x2c4640dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc640e8, SUC_lexicon_tags_27 },
    { 0x402640e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8e640eb, SUC_lexicon_tags_7 },
    { 0x6e640eb, SUC_lexicon_tags_25 },
    { 0x88e640ec, SUC_lexicon_tags_15 },
    { 0x6c2640ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc640f5, SUC_lexicon_tags_27 },
    { 0x896640f6, SUC_lexicon_tags_11 },
    { 0x5d0640f7, SUC_lexicon_tags_1 },
    { 0x142640f6, SUC_lexicon_tags_1 },
    { 0xb4c640f7, SUC_lexicon_tags_17 },
    { 0x810640f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c0640fc, SUC_lexicon_tags_2 },
    { 0xf7e640fd, SUC_lexicon_tags_17 },
    { 0x616640fc, SUC_lexicon_tags_49 },
    { 0x84a640fc, SUC_lexicon_tags_28 },
    { 0xf8a64100, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd464103, SUC_lexicon_tags_68 },
    { 0x59864104, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae64108, SUC_lexicon_tags_31 },
    { 0x38864109, SUC_lexicon_tags_0 },
    { 0x3d264108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206410e, SUC_lexicon_tags_61 },
    { 0x2b06410f, SUC_lexicon_tags_68 },
    { 0x80c64110, SUC_lexicon_tags_15 },
    { 0xdd864111, SUC_lexicon_tags_19 },
    { 0x11e64112, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a6411e, SUC_lexicon_tags_1 },
    { 0x80e6411e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9664123, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0864126, SUC_lexicon_tags_17 },
    { 0x6ce64126, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6746412c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa64132, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f264137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1366413b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6686413e, SUC_lexicon_tags_38 },
    { 0xf946413f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63664143, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf8264145, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd264147, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a86414a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a6414d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2064150, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6da64152, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48864157, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a6415b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb266415f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53c64161, SUC_lexicon_tags_7 },
    { 0x5a664162, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c64167, SUC_lexicon_tags_160 },
    { 0xd9264168, SUC_lexicon_tags_30 },
    { 0xe9e64168, SUC_lexicon_tags_15 },
    { 0xa226416a, SUC_lexicon_tags_53 },
    { 0xe906416b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7766416e, SUC_lexicon_tags_43 },
    { 0x9e86416f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x87e64171, SUC_lexicon_tags_4 },
    { 0x6b264172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf064178, SUC_lexicon_tags_7 },
    { 0xc9a64178, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e6417d, SUC_lexicon_tags_6 },
    { 0xcda6417d, SUC_lexicon_tags_7 },
    { 0xc406417f, SUC_lexicon_tags_14 },
    { 0x39064180, SUC_lexicon_tags_14 },
    { 0x9e464181, SUC_lexicon_tags_7 },
    { 0xdf06417d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e64188, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde6418e, SUC_lexicon_tags_60 },
    { 0x3366418f, SUC_lexicon_tags_15 },
    { 0xdfa64190, SUC_lexicon_tags_63 },
    { 0x91464191, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc64194, SUC_lexicon_tags_47 },
    { 0x6dc64195, SUC_lexicon_tags_12 },
    { 0x66864196, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a864198, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a06419c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2aa6419e, SUC_lexicon_tags_33 },
    { 0xce06419f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512641a4, SUC_lexicon_tags_11 },
    { 0xf0a641a4, SUC_lexicon_tags_15 },
    { 0x98e641a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54641a9, SUC_lexicon_tags_29 },
    { 0xa66641aa, SUC_lexicon_tags_7 },
    { 0x58e641ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02641b2, SUC_lexicon_tags_8 },
    { 0x552641b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366641b7, SUC_lexicon_tags_2 },
    { 0xb84641b7, SUC_lexicon_tags_4 },
    { 0xae0641b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32641bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6641c3, SUC_lexicon_tags_3 },
    { 0x2a0641c4, SUC_lexicon_tags_0 },
    { 0x6f8641c3, SUC_lexicon_tags_11 },
    { 0x38a641c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf28641c8, SUC_lexicon_tags_7 },
    { 0xb6e641c9, SUC_lexicon_tags_4 },
    { 0x554641ca, SUC_lexicon_tags_11 },
    { 0xf92641c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c641ce, SUC_lexicon_tags_37 },
    { 0xab8641ce, SUC_lexicon_tags_49 },
    { 0x12c641cf, SUC_lexicon_tags_48 },
    { 0x518641d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae641d4, SUC_lexicon_tags_275 },
    { 0x748641d4, SUC_lexicon_tags_5 },
    { 0x38a641d5, SUC_lexicon_tags_7 },
    { 0x694641d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8641da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a641dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380641e4, SUC_lexicon_tags_31 },
    { 0xe42641e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x576641e7, SUC_lexicon_tags_25 },
    { 0x722641e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xede641ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88641f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6641f6, SUC_lexicon_tags_10 },
    { 0x1e0641f7, SUC_lexicon_tags_47 },
    { 0x4fc641f8, SUC_lexicon_tags_59 },
    { 0x10e641f9, SUC_lexicon_tags_21 },
    { 0xe68641f6, SUC_lexicon_tags_15 },
    { 0x41e641f8, SUC_lexicon_tags_15 },
    { 0xb98641fa, SUC_lexicon_tags_7 },
    { 0xb2e641f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x334641ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ba64201, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d664203, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3c064205, SUC_lexicon_tags_17 },
    { 0xf4464206, SUC_lexicon_tags_1 },
    { 0xaa064207, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x7de64209, SUC_lexicon_tags_15 },
    { 0x29864209, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda64212, SUC_lexicon_tags_1 },
    { 0x94864213, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29464217, SUC_lexicon_tags_15 },
    { 0x90264218, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xba66421a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de6421d, SUC_lexicon_tags_25 },
    { 0xbae6421d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x83464220, SUC_lexicon_tags_7 },
    { 0x8064221, SUC_lexicon_tags_7 },
    { 0x95464220, SUC_lexicon_tags_11 },
    { 0x79c64220, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c064226, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xde664228, SUC_lexicon_tags_17 },
    { 0x3d664229, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe546422c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc264232, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6686423b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5e6423d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6864243, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2264245, SUC_lexicon_tags_275 },
    { 0x53a64246, SUC_lexicon_tags_35 },
    { 0x6cc64245, SUC_lexicon_tags_2 },
    { 0x71c64248, SUC_lexicon_tags_9 },
    { 0x8dc64247, SUC_lexicon_tags_25 },
    { 0xb766424a, SUC_lexicon_tags_40 },
    { 0x5766424a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xade6424d, SUC_lexicon_tags_9 },
    { 0xa626424d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ea64250, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e64255, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8566425c, SUC_lexicon_tags_19 },
    { 0xd666425c, SUC_lexicon_tags_27 },
    { 0x6926425e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8c464260, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce64265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd664269, SUC_lexicon_tags_107 },
    { 0x77a6426a, SUC_lexicon_tags_28 },
    { 0xa626426b, SUC_lexicon_tags_10 },
    { 0xcc66426c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7726426e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b064276, SUC_lexicon_tags_15 },
    { 0x83064277, SUC_lexicon_tags_0 },
    { 0xdc64278, SUC_lexicon_tags_7 },
    { 0xd0464279, SUC_lexicon_tags_17 },
    { 0xbb46427a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc346427c, SUC_lexicon_tags_40 },
    { 0x82a6427c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe64286, SUC_lexicon_tags_14 },
    { 0x1c64286, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb8864289, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26428d, SUC_lexicon_tags_11 },
    { 0x3b26428e, SUC_lexicon_tags_12 },
    { 0x7e6428f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c664291, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc64295, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f06429a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe386429c, SUC_lexicon_tags_37 },
    { 0x1f66429c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0642a0, SUC_lexicon_tags_12 },
    { 0x2a2642a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1e642a3, SUC_lexicon_tags_7 },
    { 0x32e642a4, SUC_lexicon_tags_160 },
    { 0xd20642a5, SUC_lexicon_tags_26 },
    { 0x69c642a6, SUC_lexicon_tags_17 },
    { 0x9ae642a3, SUC_lexicon_tags_29 },
    { 0x20642a8, SUC_lexicon_tags_12 },
    { 0xd1c642a9, SUC_lexicon_tags_7 },
    { 0xbd2642a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x79a642ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb60642ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea642b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e642b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x294642b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc2e642ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4642c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde642c3, SUC_lexicon_tags_2 },
    { 0x92c642c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582642c8, SUC_lexicon_tags_39 },
    { 0x70a642c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0642d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec642d4, SUC_lexicon_tags_37 },
    { 0xc7a642d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c642db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfa0642dd, SUC_lexicon_tags_8 },
    { 0xa52642de, SUC_lexicon_tags_3 },
    { 0x6ae642df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4642e5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x62a642e7, SUC_lexicon_tags_25 },
    { 0x442642e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e642eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c642f0, SUC_lexicon_tags_11 },
    { 0xe0642f1, SUC_lexicon_tags_17 },
    { 0x75e642f2, SUC_lexicon_tags_11 },
    { 0xdc8642f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x462642f6, SUC_lexicon_tags_7 },
    { 0x4e8642f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a642fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c642ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae64302, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c864307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d26430b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4264312, SUC_lexicon_tags_29 },
    { 0x99464312, SUC_lexicon_tags_33 },
    { 0x84864313, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x70864316, SUC_lexicon_tags_47 },
    { 0xd6064316, SUC_lexicon_tags_65 },
    { 0xe5e64318, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe6431d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfa6431f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x864321, SUC_lexicon_tags_17 },
    { 0xeb664322, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x2d464324, SUC_lexicon_tags_0 },
    { 0x86864325, SUC_lexicon_tags_48 },
    { 0x34664326, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5566432a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e64333, SUC_lexicon_tags_25 },
    { 0x35664334, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9a064336, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5646433a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcc46433c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc6433f, SUC_lexicon_tags_37 },
    { 0x58464340, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee64345, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1786434a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbe26434c, SUC_lexicon_tags_27 },
    { 0x2bc6434d, SUC_lexicon_tags_47 },
    { 0xa3c6434c, SUC_lexicon_tags_37 },
    { 0x8006434f, SUC_lexicon_tags_85 },
    { 0x9e26434f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb264354, SUC_lexicon_tags_47 },
    { 0x4b264355, SUC_lexicon_tags_31 },
    { 0x45464355, SUC_lexicon_tags_2 },
    { 0x85664357, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6f864359, SUC_lexicon_tags_47 },
    { 0x7866435a, SUC_lexicon_tags_33 },
    { 0xe2a6435b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2606435d, SUC_lexicon_tags_25 },
    { 0x4746435d, SUC_lexicon_tags_17 },
    { 0x92c6435f, SUC_lexicon_tags_2 },
    { 0x67864360, SUC_lexicon_tags_1 },
    { 0x33864361, SUC_lexicon_tags_247 },
    { 0xb2264362, SUC_lexicon_tags_66 },
    { 0x72c64363, SUC_lexicon_tags_48 },
    { 0x98064362, SUC_lexicon_tags_47 },
    { 0xdbe64364, SUC_lexicon_tags_26 },
    { 0x9b464366, SUC_lexicon_tags_32 },
    { 0x3a664367, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0e64369, SUC_lexicon_tags_21 },
    { 0xd486436a, SUC_lexicon_tags_17 },
    { 0x5846436a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d06436e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49864370, SUC_lexicon_tags_11 },
    { 0x2de64371, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa4864373, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a64377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf666437b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a6437e, SUC_lexicon_tags_1 },
    { 0x2e26437e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a64383, SUC_lexicon_tags_11 },
    { 0x79064383, SUC_lexicon_tags_30 },
    { 0x9fa64385, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a264388, SUC_lexicon_tags_215 },
    { 0x3e264389, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72c6438b, SUC_lexicon_tags_23 },
    { 0xee66438c, SUC_lexicon_tags_11 },
    { 0x2126438b, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c64394, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46864399, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc406439b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c6439e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6643a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14643a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732643ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53a643ae, SUC_lexicon_tags_17 },
    { 0x66c643ae, SUC_lexicon_tags_13 },
    { 0xca8643b0, SUC_lexicon_tags_26 },
    { 0x422643b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c643b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4c643b8, SUC_lexicon_tags_1 },
    { 0xee0643b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8643bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c643c2, SUC_lexicon_tags_2 },
    { 0xce6643c3, SUC_lexicon_tags_45 },
    { 0x21c643c4, SUC_lexicon_tags_14 },
    { 0xeec643c4, SUC_lexicon_tags_60 },
    { 0xd00643c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf28643c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b4643ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0643d2, SUC_lexicon_tags_15 },
    { 0x3f0643d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8643d6, SUC_lexicon_tags_38 },
    { 0xe5a643d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818643dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba643df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a643e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6643e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xada643ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xede643ef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a643f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a643fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42643fe, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a64403, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94c64405, SUC_lexicon_tags_28 },
    { 0x63e64406, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae6440a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae6440e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc664413, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7064420, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64664424, SUC_lexicon_tags_11 },
    { 0xba864425, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cc64427, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a6442a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246442e, SUC_lexicon_tags_68 },
    { 0x2846442f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbec64431, SUC_lexicon_tags_30 },
    { 0x75464431, SUC_lexicon_tags_12 },
    { 0x7dc64432, SUC_lexicon_tags_47 },
    { 0xf1e64434, SUC_lexicon_tags_12 },
    { 0x6de64434, SUC_lexicon_tags_52 },
    { 0x3ea64436, SUC_lexicon_tags_19 },
    { 0x1c664434, SUC_lexicon_tags_6 },
    { 0xf1a64438, SUC_lexicon_tags_2 },
    { 0x7ce64436, SUC_lexicon_tags_11 },
    { 0xc1e64432, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa026443d, SUC_lexicon_tags_11 },
    { 0x68c6443e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e64440, SUC_lexicon_tags_33 },
    { 0x95064441, SUC_lexicon_tags_63 },
    { 0xb5864442, SUC_lexicon_tags_1 },
    { 0x9d664443, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x63664445, SUC_lexicon_tags_37 },
    { 0x37a64445, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x56e64448, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4666444c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa766444f, SUC_lexicon_tags_17 },
    { 0xab26444f, SUC_lexicon_tags_17 },
    { 0x47c64450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44464456, SUC_lexicon_tags_65 },
    { 0x5b664456, SUC_lexicon_tags_10 },
    { 0xd1c64457, SUC_lexicon_tags_8 },
    { 0x76864459, SUC_lexicon_tags_11 },
    { 0x7486445a, SUC_lexicon_tags_23 },
    { 0x7766445b, SUC_lexicon_tags_0 },
    { 0xf0864459, SUC_lexicon_tags_61 },
    { 0xecc6445a, SUC_lexicon_tags_16 },
    { 0x1726445e, SUC_lexicon_tags_17 },
    { 0xfb86445a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3f864461, SUC_lexicon_tags_92 },
    { 0x4d064461, SUC_lexicon_tags_17 },
    { 0xfe064463, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2664465, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba64468, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5286446a, SUC_lexicon_tags_7 },
    { 0xeee6446a, SUC_lexicon_tags_25 },
    { 0x59a6446b, SUC_lexicon_tags_11 },
    { 0x35e6446d, SUC_lexicon_tags_60 },
    { 0x8926446e, SUC_lexicon_tags_14 },
    { 0x2526446e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac864476, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc64479, SUC_lexicon_tags_17 },
    { 0x9486447a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb666447d, SUC_lexicon_tags_25 },
    { 0x71c6447d, SUC_lexicon_tags_12 },
    { 0x3526447f, SUC_lexicon_tags_15 },
    { 0x5e264480, SUC_lexicon_tags_15 },
    { 0x9aa64480, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a664484, SUC_lexicon_tags_12 },
    { 0x37864485, SUC_lexicon_tags_859 },
    { 0x6d064486, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x48264489, SUC_lexicon_tags_25 },
    { 0x5d46448a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3666448e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56464498, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd066449a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6e6449c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x36e6449e, SUC_lexicon_tags_2 },
    { 0x1c46449f, SUC_lexicon_tags_11 },
    { 0xc3e6449f, SUC_lexicon_tags_31 },
    { 0x370644a1, SUC_lexicon_tags_66 },
    { 0x9ba644a1, SUC_lexicon_tags_1 },
    { 0xc62644a3, SUC_lexicon_tags_46 },
    { 0x9ee644a0, SUC_lexicon_tags_11 },
    { 0xb06644a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20644ae, SUC_lexicon_tags_2 },
    { 0x6b0644ae, SUC_lexicon_tags_49 },
    { 0xd62644af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a644b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa644b8, SUC_lexicon_tags_7 },
    { 0x1b8644b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x606644bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c8644bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e644c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536644cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2644d3, SUC_lexicon_tags_25 },
    { 0xe44644d4, SUC_lexicon_tags_6 },
    { 0x89c644d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac644d8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc644db, SUC_lexicon_tags_47 },
    { 0x40c644dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce644ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0644f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x530644f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a644f7, SUC_lexicon_tags_39 },
    { 0x818644f8, SUC_lexicon_tags_23 },
    { 0x15e644f9, SUC_lexicon_tags_31 },
    { 0x2d2644fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8644fd, SUC_lexicon_tags_49 },
    { 0xaa4644fe, SUC_lexicon_tags_25 },
    { 0xb88644fd, SUC_lexicon_tags_0 },
    { 0x5e664500, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e64505, SUC_lexicon_tags_1 },
    { 0xf0464506, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5864508, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f66450a, SUC_lexicon_tags_63 },
    { 0x8b66450b, SUC_lexicon_tags_3 },
    { 0xe6e6450c, SUC_lexicon_tags_7 },
    { 0x3d06450d, SUC_lexicon_tags_14 },
    { 0x3986450c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xef264510, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe64513, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x80864516, SUC_lexicon_tags_47 },
    { 0xb6c64517, SUC_lexicon_tags_15 },
    { 0xae464518, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb746451b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e464520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca64524, SUC_lexicon_tags_82 },
    { 0xd3664525, SUC_lexicon_tags_17 },
    { 0x55a64525, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e64529, SUC_lexicon_tags_44 },
    { 0x17c6452a, SUC_lexicon_tags_12 },
    { 0xaaa6452b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac6452f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8264536, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x89264538, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1786453d, SUC_lexicon_tags_8 },
    { 0x40e6453d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a64541, SUC_lexicon_tags_25 },
    { 0xa4264542, SUC_lexicon_tags_27 },
    { 0x7064543, SUC_lexicon_tags_12 },
    { 0xa5264542, SUC_lexicon_tags_9 },
    { 0xb2864545, SUC_lexicon_tags_1 },
    { 0x7d664545, SUC_lexicon_tags_1 },
    { 0x15e64542, SUC_lexicon_tags_38 },
    { 0xfce64548, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f664555, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c64558, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd426455b, SUC_lexicon_tags_11 },
    { 0x1346455c, SUC_lexicon_tags_6 },
    { 0x2086455d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf864560, SUC_lexicon_tags_47 },
    { 0x78264561, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6864563, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a64566, SUC_lexicon_tags_12 },
    { 0x82a64567, SUC_lexicon_tags_37 },
    { 0xf8264566, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa9e6456a, SUC_lexicon_tags_7 },
    { 0x9f86456b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e64573, SUC_lexicon_tags_47 },
    { 0x8664574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64578, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa926457c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69264580, SUC_lexicon_tags_47 },
    { 0x10e64580, SUC_lexicon_tags_7 },
    { 0x2dc64582, SUC_lexicon_tags_87 },
    { 0x8fc64581, SUC_lexicon_tags_33 },
    { 0xa8264583, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e64587, SUC_lexicon_tags_37 },
    { 0xdb864588, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xab46458b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x44e6458d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a64590, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe066459c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32645a1, SUC_lexicon_tags_37 },
    { 0xfc8645a2, SUC_lexicon_tags_37 },
    { 0xcac645a3, SUC_lexicon_tags_14 },
    { 0xe00645a4, SUC_lexicon_tags_11 },
    { 0x18c645a5, SUC_lexicon_tags_224 },
    { 0xa26645a4, SUC_lexicon_tags_11 },
    { 0xf32645a7, SUC_lexicon_tags_2 },
    { 0xfca645a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96645ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xece645ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50645b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x44645b3, SUC_lexicon_tags_37 },
    { 0x84c645b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102645bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08645c2, SUC_lexicon_tags_79 },
    { 0x402645c2, SUC_lexicon_tags_23 },
    { 0x64c645c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e645ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2645d0, SUC_lexicon_tags_138 },
    { 0xa6a645d1, SUC_lexicon_tags_1 },
    { 0x3a8645d0, SUC_lexicon_tags_64 },
    { 0x4d0645d3, SUC_lexicon_tags_25 },
    { 0x7c2645d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90c645d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758645de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf90645e0, SUC_lexicon_tags_69 },
    { 0x7ee645e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8645ec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a645ef, SUC_lexicon_tags_45 },
    { 0xe0645f0, SUC_lexicon_tags_48 },
    { 0xe22645f0, SUC_lexicon_tags_46 },
    { 0x234645ef, SUC_lexicon_tags_12 },
    { 0x79a645f3, SUC_lexicon_tags_14 },
    { 0xe5a645f2, SUC_lexicon_tags_25 },
    { 0x2f2645f5, SUC_lexicon_tags_37 },
    { 0x35e645f5, SUC_lexicon_tags_25 },
    { 0x630645f7, SUC_lexicon_tags_37 },
    { 0xf6e645f7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x78c645fa, SUC_lexicon_tags_37 },
    { 0xbba645fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa82645fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5064601, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc664606, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc64609, SUC_lexicon_tags_1 },
    { 0x4b26460a, SUC_lexicon_tags_55 },
    { 0x92664609, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a06460d, SUC_lexicon_tags_11 },
    { 0xf9e6460e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5064612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc64615, SUC_lexicon_tags_17 },
    { 0x39064616, SUC_lexicon_tags_1068 },
    { 0x74e64617, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50264619, SUC_lexicon_tags_37 },
    { 0x20c6461a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc6461e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e064623, SUC_lexicon_tags_37 },
    { 0x72264624, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e464627, SUC_lexicon_tags_0 },
    { 0x42a64628, SUC_lexicon_tags_26 },
    { 0xd3c64627, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe626462e, SUC_lexicon_tags_47 },
    { 0xf8c6462e, SUC_lexicon_tags_44 },
    { 0x1d064630, SUC_lexicon_tags_60 },
    { 0x43c64630, SUC_lexicon_tags_37 },
    { 0x2ca6462e, SUC_lexicon_tags_38 },
    { 0x39a64630, SUC_lexicon_tags_11 },
    { 0x3dc64634, SUC_lexicon_tags_7 },
    { 0xb0264635, SUC_lexicon_tags_39 },
    { 0x7ba64636, SUC_lexicon_tags_12 },
    { 0x50264637, SUC_lexicon_tags_47 },
    { 0xb0464637, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5546463a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6ce6463c, SUC_lexicon_tags_11 },
    { 0x8006463d, SUC_lexicon_tags_33 },
    { 0xb5c6463d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe64644, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57664648, SUC_lexicon_tags_2 },
    { 0x9ea64649, SUC_lexicon_tags_60 },
    { 0x5de6464a, SUC_lexicon_tags_10 },
    { 0x5186464b, SUC_lexicon_tags_4 },
    { 0x60c6464c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe26464e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7664651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c6465b, SUC_lexicon_tags_31 },
    { 0x7526465c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e64663, SUC_lexicon_tags_6 },
    { 0xa8464664, SUC_lexicon_tags_32 },
    { 0x78664664, SUC_lexicon_tags_7 },
    { 0xcf264663, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a6466a, SUC_lexicon_tags_1 },
    { 0xbce6466b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b864673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f064676, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e6467b, SUC_lexicon_tags_68 },
    { 0xdda6467c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7506467f, SUC_lexicon_tags_8 },
    { 0x8546467f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb26468f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa64692, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e264695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27c64697, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf5864699, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6646a3, SUC_lexicon_tags_12 },
    { 0xa48646a3, SUC_lexicon_tags_1 },
    { 0x350646a5, SUC_lexicon_tags_12 },
    { 0x858646a6, SUC_lexicon_tags_53 },
    { 0x6c2646a6, SUC_lexicon_tags_37 },
    { 0x62646a7, SUC_lexicon_tags_56 },
    { 0xb74646a9, SUC_lexicon_tags_1 },
    { 0x534646aa, SUC_lexicon_tags_7 },
    { 0xb7c646aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21a646ad, SUC_lexicon_tags_17 },
    { 0xe52646ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7c6646b0, SUC_lexicon_tags_15 },
    { 0xb1a646b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4646b5, SUC_lexicon_tags_2 },
    { 0xe44646b5, SUC_lexicon_tags_25 },
    { 0xbb0646b7, SUC_lexicon_tags_25 },
    { 0x9b2646b7, SUC_lexicon_tags_13 },
    { 0x106646b9, SUC_lexicon_tags_14 },
    { 0xcee646b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9be646bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83c646be, SUC_lexicon_tags_11 },
    { 0xa78646bf, SUC_lexicon_tags_16 },
    { 0x304646c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc1c646c2, SUC_lexicon_tags_56 },
    { 0x582646c2, SUC_lexicon_tags_0 },
    { 0xd6c646c4, SUC_lexicon_tags_23 },
    { 0x55e646c3, SUC_lexicon_tags_25 },
    { 0x254646c3, SUC_lexicon_tags_37 },
    { 0xdcc646c7, SUC_lexicon_tags_17 },
    { 0xffa646c8, SUC_lexicon_tags_17 },
    { 0xbd0646c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36646cc, SUC_lexicon_tags_5 },
    { 0x254646cc, SUC_lexicon_tags_15 },
    { 0x8c0646ce, SUC_lexicon_tags_36 },
    { 0x2ae646cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xafe646d1, SUC_lexicon_tags_12 },
    { 0xe60646d2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc646d5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c646d9, SUC_lexicon_tags_38 },
    { 0x302646da, SUC_lexicon_tags_0 },
    { 0xc7e646d9, SUC_lexicon_tags_45 },
    { 0xc68646dc, SUC_lexicon_tags_46 },
    { 0xb42646db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4646e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4646e5, SUC_lexicon_tags_37 },
    { 0x88a646e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1b8646e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be646ec, SUC_lexicon_tags_39 },
    { 0x3a8646ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x25c646ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc46646f1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a646fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a64702, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf664707, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa126470b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xece6470d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x686470f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc106471a, SUC_lexicon_tags_8 },
    { 0xf3e6471a, SUC_lexicon_tags_4 },
    { 0x7246471b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1526471f, SUC_lexicon_tags_25 },
    { 0xa2e64720, SUC_lexicon_tags_14 },
    { 0x2ea64721, SUC_lexicon_tags_165 },
    { 0x19864722, SUC_lexicon_tags_46 },
    { 0x9f864723, SUC_lexicon_tags_31 },
    { 0x5f064724, SUC_lexicon_tags_419 },
    { 0x4da64722, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc64729, SUC_lexicon_tags_1 },
    { 0xdd66472a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd286472c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6264730, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc264733, SUC_lexicon_tags_7 },
    { 0xee464734, SUC_lexicon_tags_8 },
    { 0x2e264735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e464737, SUC_lexicon_tags_17 },
    { 0x9464737, SUC_lexicon_tags_47 },
    { 0x5be64739, SUC_lexicon_tags_27 },
    { 0xbd86473a, SUC_lexicon_tags_12 },
    { 0x7906473b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde66473f, SUC_lexicon_tags_15 },
    { 0xe6464740, SUC_lexicon_tags_45 },
    { 0x4864740, SUC_lexicon_tags_25 },
    { 0x23e64741, SUC_lexicon_tags_82 },
    { 0x54a64743, SUC_lexicon_tags_45 },
    { 0x63e64744, SUC_lexicon_tags_27 },
    { 0x92064745, SUC_lexicon_tags_0 },
    { 0x54a64746, SUC_lexicon_tags_47 },
    { 0x4cc64747, SUC_lexicon_tags_1 },
    { 0xefe64744, SUC_lexicon_tags_32 },
    { 0x4b664749, SUC_lexicon_tags_37 },
    { 0xec264747, SUC_lexicon_tags_47 },
    { 0xec264749, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b664754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52864757, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfa64759, SUC_lexicon_tags_37 },
    { 0x4ae64759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa346475e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3464765, SUC_lexicon_tags_29 },
    { 0x6f664765, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e64769, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88c6476b, SUC_lexicon_tags_30 },
    { 0xed86476c, SUC_lexicon_tags_34 },
    { 0x88e6476d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3a66476f, SUC_lexicon_tags_11 },
    { 0xb2e64770, SUC_lexicon_tags_23 },
    { 0x2e86476f, SUC_lexicon_tags_25 },
    { 0xd6464772, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6566477b, SUC_lexicon_tags_37 },
    { 0x20e6477c, SUC_lexicon_tags_37 },
    { 0x4fe6477d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53064780, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbf864782, SUC_lexicon_tags_39 },
    { 0x3c464783, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86478a, SUC_lexicon_tags_38 },
    { 0xe5a6478b, SUC_lexicon_tags_11 },
    { 0xaa46478c, SUC_lexicon_tags_27 },
    { 0x7f86478b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41864796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd264799, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d46479d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4647a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6647a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ac647a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22647a8, SUC_lexicon_tags_25 },
    { 0x770647a9, SUC_lexicon_tags_12 },
    { 0x828647a8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc647b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70647b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3f4647b9, SUC_lexicon_tags_30 },
    { 0x67a647ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114647c3, SUC_lexicon_tags_33 },
    { 0xc90647c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3e4647c6, SUC_lexicon_tags_15 },
    { 0xeb2647c6, SUC_lexicon_tags_15 },
    { 0x402647c8, SUC_lexicon_tags_2 },
    { 0xaba647c9, SUC_lexicon_tags_26 },
    { 0x71c647c9, SUC_lexicon_tags_19 },
    { 0x5c2647cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80647ce, SUC_lexicon_tags_1 },
    { 0xa30647ce, SUC_lexicon_tags_37 },
    { 0x572647cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58647d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5e8647d6, SUC_lexicon_tags_48 },
    { 0xaee647d7, SUC_lexicon_tags_14 },
    { 0xe58647d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4647dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa70647de, SUC_lexicon_tags_28 },
    { 0x262647df, SUC_lexicon_tags_15 },
    { 0x7cc647e0, SUC_lexicon_tags_50 },
    { 0xd4e647de, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x77e647e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x408647e6, SUC_lexicon_tags_0 },
    { 0x2c8647e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746647ee, SUC_lexicon_tags_2 },
    { 0xa80647ef, SUC_lexicon_tags_19 },
    { 0x978647f0, SUC_lexicon_tags_25 },
    { 0xd0a647f0, SUC_lexicon_tags_17 },
    { 0xea0647ee, SUC_lexicon_tags_63 },
    { 0xde4647f3, SUC_lexicon_tags_1 },
    { 0x3c0647f4, SUC_lexicon_tags_25 },
    { 0x9b6647f5, SUC_lexicon_tags_11 },
    { 0xef0647f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaca647f8, SUC_lexicon_tags_7 },
    { 0xa8647f8, SUC_lexicon_tags_541 },
    { 0x46e647f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c2647fc, SUC_lexicon_tags_7 },
    { 0x724647fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97a647ff, SUC_lexicon_tags_1 },
    { 0x1c464800, SUC_lexicon_tags_26 },
    { 0xf4864800, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e64808, SUC_lexicon_tags_0 },
    { 0xdc864808, SUC_lexicon_tags_14 },
    { 0x346480a, SUC_lexicon_tags_27 },
    { 0xfd464809, SUC_lexicon_tags_25 },
    { 0xebc6480b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe326480e, SUC_lexicon_tags_43 },
    { 0xf366480f, SUC_lexicon_tags_45 },
    { 0xaa66480e, SUC_lexicon_tags_11 },
    { 0xee264811, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe864813, SUC_lexicon_tags_11 },
    { 0xd0064814, SUC_lexicon_tags_21 },
    { 0xb2664815, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5264817, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeb464819, SUC_lexicon_tags_11 },
    { 0x8966481a, SUC_lexicon_tags_12 },
    { 0xa2e6481a, SUC_lexicon_tags_19 },
    { 0x5386481c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6da6481e, SUC_lexicon_tags_43 },
    { 0x8b06481f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c64823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae6482a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5666482d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a64831, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc64834, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8864836, SUC_lexicon_tags_37 },
    { 0x51e64837, SUC_lexicon_tags_17 },
    { 0xabc64838, SUC_lexicon_tags_29 },
    { 0x26664838, SUC_lexicon_tags_28 },
    { 0x81c64839, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb106483c, SUC_lexicon_tags_11 },
    { 0x5846483d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77a6483f, SUC_lexicon_tags_19 },
    { 0xe1864840, SUC_lexicon_tags_11 },
    { 0x10a64841, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc2e64843, SUC_lexicon_tags_39 },
    { 0x7864843, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb6c64846, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc206484d, SUC_lexicon_tags_25 },
    { 0xa526484e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e664853, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b664856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3046485b, SUC_lexicon_tags_60 },
    { 0xd826485c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b464868, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f26486a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd586486e, SUC_lexicon_tags_11 },
    { 0xfaa6486f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a64872, SUC_lexicon_tags_0 },
    { 0x44864873, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67464875, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad464877, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42864879, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c6487f, SUC_lexicon_tags_49 },
    { 0x12a64880, SUC_lexicon_tags_43 },
    { 0x9fa64881, SUC_lexicon_tags_38 },
    { 0xe6c64881, SUC_lexicon_tags_63 },
    { 0x6b664883, SUC_lexicon_tags_2 },
    { 0x82664883, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e6488a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe26488c, SUC_lexicon_tags_26 },
    { 0x6de6488d, SUC_lexicon_tags_37 },
    { 0x19a6488d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaec64890, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67464892, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb064894, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6864897, SUC_lexicon_tags_10 },
    { 0x76e64898, SUC_lexicon_tags_14 },
    { 0x2de64899, SUC_lexicon_tags_17 },
    { 0x7526489a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b06489e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c648a3, SUC_lexicon_tags_7 },
    { 0x4be648a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600648ac, SUC_lexicon_tags_4 },
    { 0xf3e648ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x258648af, SUC_lexicon_tags_25 },
    { 0x646648b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa648b3, SUC_lexicon_tags_23 },
    { 0xf12648b4, SUC_lexicon_tags_118 },
    { 0x1fe648b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4648ba, SUC_lexicon_tags_66 },
    { 0x416648bb, SUC_lexicon_tags_28 },
    { 0x706648bc, SUC_lexicon_tags_26 },
    { 0x1d8648bd, SUC_lexicon_tags_12 },
    { 0x174648bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x760648c1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x90c648c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4648cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe3a648cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88648d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac648d8, SUC_lexicon_tags_60 },
    { 0xca8648d8, SUC_lexicon_tags_1 },
    { 0x6a0648da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x372648dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58648e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70648e7, SUC_lexicon_tags_7 },
    { 0xc0a648e8, SUC_lexicon_tags_25 },
    { 0x734648e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28648ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xade648f1, SUC_lexicon_tags_60 },
    { 0x78648f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704648fa, SUC_lexicon_tags_43 },
    { 0x758648fa, SUC_lexicon_tags_37 },
    { 0xc84648fc, SUC_lexicon_tags_8 },
    { 0x216648fd, SUC_lexicon_tags_10 },
    { 0x458648fe, SUC_lexicon_tags_74 },
    { 0xc46648fe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4264902, SUC_lexicon_tags_16 },
    { 0xf3064903, SUC_lexicon_tags_25 },
    { 0xda64904, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e264906, SUC_lexicon_tags_40 },
    { 0xe7e64907, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4646490b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4c26490d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4064910, SUC_lexicon_tags_25 },
    { 0xdd864911, SUC_lexicon_tags_37 },
    { 0xcd264911, SUC_lexicon_tags_47 },
    { 0xdb264912, SUC_lexicon_tags_12 },
    { 0xe4864914, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a6491b, SUC_lexicon_tags_11 },
    { 0xf46491c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12c6491e, SUC_lexicon_tags_14 },
    { 0x4946491f, SUC_lexicon_tags_19 },
    { 0x5226491f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a264924, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8264929, SUC_lexicon_tags_63 },
    { 0x60664929, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafa6492c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f664936, SUC_lexicon_tags_21 },
    { 0x6e864937, SUC_lexicon_tags_47 },
    { 0xa4064938, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3986493e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264943, SUC_lexicon_tags_37 },
    { 0xa464943, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3464946, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6864948, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8886494a, SUC_lexicon_tags_19 },
    { 0xcf66494b, SUC_lexicon_tags_25 },
    { 0xae26494c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd664953, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x6b464955, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2166495a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a64962, SUC_lexicon_tags_23 },
    { 0x43864963, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x49a64965, SUC_lexicon_tags_2 },
    { 0x4c264966, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c6496d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb664970, SUC_lexicon_tags_25 },
    { 0x24e64971, SUC_lexicon_tags_57 },
    { 0x8b464971, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb5464974, SUC_lexicon_tags_37 },
    { 0x3fa64975, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e6497a, SUC_lexicon_tags_91 },
    { 0xfa06497a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f464980, SUC_lexicon_tags_7 },
    { 0xcbe64980, SUC_lexicon_tags_1 },
    { 0x74064982, SUC_lexicon_tags_17 },
    { 0x6d864980, SUC_lexicon_tags_937 },
    { 0x7ee64980, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8664987, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c6498a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa6498e, SUC_lexicon_tags_15 },
    { 0xd146498f, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da64997, SUC_lexicon_tags_37 },
    { 0x27264998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa986499c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78649a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8649a9, SUC_lexicon_tags_9 },
    { 0xaf4649aa, SUC_lexicon_tags_1183 },
    { 0x642649ab, SUC_lexicon_tags_47 },
    { 0xdb6649ac, SUC_lexicon_tags_12 },
    { 0xb24649ab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd5c649af, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf94649b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xda649b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x146649b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6649b9, SUC_lexicon_tags_15 },
    { 0x7aa649ba, SUC_lexicon_tags_45 },
    { 0xe1a649ba, SUC_lexicon_tags_17 },
    { 0xdb0649bc, SUC_lexicon_tags_43 },
    { 0xf0e649bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6649c6, SUC_lexicon_tags_43 },
    { 0x5d4649c7, SUC_lexicon_tags_67 },
    { 0x6bc649c8, SUC_lexicon_tags_12 },
    { 0x6e649c9, SUC_lexicon_tags_68 },
    { 0x7e2649c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e649ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba6649d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674649d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0649da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398649df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x384649e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48a649e3, SUC_lexicon_tags_11 },
    { 0x3f6649e4, SUC_lexicon_tags_11 },
    { 0x6ca649e4, SUC_lexicon_tags_72 },
    { 0x79e649e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56649e9, SUC_lexicon_tags_16 },
    { 0xbc8649ea, SUC_lexicon_tags_47 },
    { 0x4b4649eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216649ef, SUC_lexicon_tags_42 },
    { 0x42649ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10a649f2, SUC_lexicon_tags_37 },
    { 0xa4649f3, SUC_lexicon_tags_28 },
    { 0xc24649f2, SUC_lexicon_tags_12 },
    { 0xc76649f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0649f9, SUC_lexicon_tags_10 },
    { 0x538649fa, SUC_lexicon_tags_11 },
    { 0xefc649fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e649fe, SUC_lexicon_tags_54 },
    { 0x7e649ff, SUC_lexicon_tags_31 },
    { 0x25864a00, SUC_lexicon_tags_1 },
    { 0x4e4649fe, SUC_lexicon_tags_37 },
    { 0x3ea64a00, SUC_lexicon_tags_49 },
    { 0xd2664a03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e64a09, SUC_lexicon_tags_30 },
    { 0xe5e64a0a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c64a0d, SUC_lexicon_tags_12 },
    { 0xc3664a0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea664a11, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a64a14, SUC_lexicon_tags_39 },
    { 0x20464a15, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2064a1f, SUC_lexicon_tags_4 },
    { 0xd8664a20, SUC_lexicon_tags_65 },
    { 0xc9e64a1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb9064a23, SUC_lexicon_tags_25 },
    { 0x79264a24, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x75a64a26, SUC_lexicon_tags_11 },
    { 0x75264a27, SUC_lexicon_tags_2 },
    { 0x33c64a28, SUC_lexicon_tags_11 },
    { 0x11464a29, SUC_lexicon_tags_30 },
    { 0xb8e64a2a, SUC_lexicon_tags_25 },
    { 0xa0064a28, SUC_lexicon_tags_11 },
    { 0xcf464a2c, SUC_lexicon_tags_22 },
    { 0x3a464a2d, SUC_lexicon_tags_2 },
    { 0xa6864a29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed464a31, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9a464a33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x50a64a35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcae64a37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb864a3b, SUC_lexicon_tags_37 },
    { 0xfec64a3c, SUC_lexicon_tags_45 },
    { 0x59664a3c, SUC_lexicon_tags_63 },
    { 0xc1464a3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13264a40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c64a43, SUC_lexicon_tags_7 },
    { 0x54c64a44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc464a46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47664a52, SUC_lexicon_tags_12 },
    { 0x47664a53, SUC_lexicon_tags_17 },
    { 0x81c64a54, SUC_lexicon_tags_27 },
    { 0x8b864a55, SUC_lexicon_tags_8 },
    { 0x15c64a55, SUC_lexicon_tags_14 },
    { 0xf2464a57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e64a5c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ae64a5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x58864a60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81664a63, SUC_lexicon_tags_23 },
    { 0x2f464a64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc264a66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad264a68, SUC_lexicon_tags_65 },
    { 0x76864a69, SUC_lexicon_tags_7 },
    { 0x70864a68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4864a70, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8e64a72, SUC_lexicon_tags_17 },
    { 0x2f864a72, SUC_lexicon_tags_65 },
    { 0x93a64a72, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e664a78, SUC_lexicon_tags_55 },
    { 0x97464a79, SUC_lexicon_tags_7 },
    { 0x85864a79, SUC_lexicon_tags_7 },
    { 0x4ac64a7b, SUC_lexicon_tags_39 },
    { 0x25864a78, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca64a7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e64a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36864a85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e64a89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c64a8f, SUC_lexicon_tags_15 },
    { 0x93a64a90, SUC_lexicon_tags_31 },
    { 0x40864a90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd064a94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3264a96, SUC_lexicon_tags_2 },
    { 0xb7c64a97, SUC_lexicon_tags_14 },
    { 0x81a64a97, SUC_lexicon_tags_1 },
    { 0x27864a98, SUC_lexicon_tags_15 },
    { 0xcd464a98, SUC_lexicon_tags_25 },
    { 0x79264a97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4064a9d, SUC_lexicon_tags_51 },
    { 0x8a064a9e, SUC_lexicon_tags_14 },
    { 0xda064a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91264aa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1864aa3, SUC_lexicon_tags_63 },
    { 0x60664aa3, SUC_lexicon_tags_2 },
    { 0x40064aa4, SUC_lexicon_tags_1 },
    { 0x22664aa6, SUC_lexicon_tags_10 },
    { 0xef864aa7, SUC_lexicon_tags_46 },
    { 0x2b064aa3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a64aab, SUC_lexicon_tags_2 },
    { 0x84e64aac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c64ab2, SUC_lexicon_tags_7 },
    { 0xfee64ab3, SUC_lexicon_tags_27 },
    { 0x57e64ab4, SUC_lexicon_tags_11 },
    { 0xee864ab3, SUC_lexicon_tags_15 },
    { 0x89864ab5, SUC_lexicon_tags_25 },
    { 0xa6664ab7, SUC_lexicon_tags_27 },
    { 0x41064ab7, SUC_lexicon_tags_33 },
    { 0x51864ab9, SUC_lexicon_tags_19 },
    { 0xb7864aba, SUC_lexicon_tags_2 },
    { 0x7c264abb, SUC_lexicon_tags_1 },
    { 0x56c64aba, SUC_lexicon_tags_37 },
    { 0x6d464ab5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a64ac0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd6464ac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47464ac4, SUC_lexicon_tags_70 },
    { 0x4a864ac5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19464ac8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc64ad0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e264ad6, SUC_lexicon_tags_38 },
    { 0x36064ad6, SUC_lexicon_tags_12 },
    { 0x5a664ad7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be64ae1, SUC_lexicon_tags_38 },
    { 0x41864ae2, SUC_lexicon_tags_17 },
    { 0x6c464ae3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa7864ae5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92464af3, SUC_lexicon_tags_104 },
    { 0x17064af3, SUC_lexicon_tags_49 },
    { 0xf5464af4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xff464af8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a64afb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd464b03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5264b07, SUC_lexicon_tags_26 },
    { 0xd3664b07, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x10a64b0a, SUC_lexicon_tags_15 },
    { 0x58c64b0a, SUC_lexicon_tags_15 },
    { 0x5dc64b0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a64b0f, SUC_lexicon_tags_12 },
    { 0x62864b10, SUC_lexicon_tags_23 },
    { 0xab264b11, SUC_lexicon_tags_25 },
    { 0x8b864b12, SUC_lexicon_tags_81 },
    { 0xcc864b13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a064b16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c64b1b, SUC_lexicon_tags_11 },
    { 0x61864b1c, SUC_lexicon_tags_33 },
    { 0xa3c64b1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c664b25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa64b2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0c64b2c, SUC_lexicon_tags_11 },
    { 0xb3464b2d, SUC_lexicon_tags_1 },
    { 0x6be64b2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xba264b31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb264b35, SUC_lexicon_tags_27 },
    { 0xc3664b35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ea64b38, SUC_lexicon_tags_49 },
    { 0xb6c64b39, SUC_lexicon_tags_25 },
    { 0x7264b3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a864b3e, SUC_lexicon_tags_46 },
    { 0xda264b3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6664b43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ee64b45, SUC_lexicon_tags_23 },
    { 0xf4a64b46, SUC_lexicon_tags_37 },
    { 0x34064b46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb8e64b49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a64b4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a664b50, SUC_lexicon_tags_10 },
    { 0xff864b51, SUC_lexicon_tags_37 },
    { 0x2ac64b51, SUC_lexicon_tags_37 },
    { 0x41a64b50, SUC_lexicon_tags_45 },
    { 0xbe464b53, SUC_lexicon_tags_33 },
    { 0x6be64b52, SUC_lexicon_tags_28 },
    { 0x39464b56, SUC_lexicon_tags_37 },
    { 0x5064b57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22464b59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a264b5b, SUC_lexicon_tags_18 },
    { 0x9c64b5c, SUC_lexicon_tags_8 },
    { 0xdf864b5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdac64b5f, SUC_lexicon_tags_25 },
    { 0x25464b60, SUC_lexicon_tags_56 },
    { 0xd464b60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c64b64, SUC_lexicon_tags_11 },
    { 0xa7a64b65, SUC_lexicon_tags_37 },
    { 0x60464b66, SUC_lexicon_tags_7 },
    { 0xd9e64b67, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8cc64b69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e64b6c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a64b74, SUC_lexicon_tags_15 },
    { 0xa9464b75, SUC_lexicon_tags_8 },
    { 0x64a64b75, SUC_lexicon_tags_28 },
    { 0x5ba64b74, SUC_lexicon_tags_2 },
    { 0x7f664b78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e64b7b, SUC_lexicon_tags_12 },
    { 0x27664b7c, SUC_lexicon_tags_12 },
    { 0x4f864b7d, SUC_lexicon_tags_14 },
    { 0xf5e64b7d, SUC_lexicon_tags_27 },
    { 0xdec64b7e, SUC_lexicon_tags_33 },
    { 0x14064b7c, SUC_lexicon_tags_30 },
    { 0x30e64b80, SUC_lexicon_tags_11 },
    { 0x56464b82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51a64b84, SUC_lexicon_tags_35 },
    { 0x3e464b84, SUC_lexicon_tags_65 },
    { 0x75a64b86, SUC_lexicon_tags_60 },
    { 0x42c64b87, SUC_lexicon_tags_62 },
    { 0xbf264b88, SUC_lexicon_tags_66 },
    { 0x23264b85, SUC_lexicon_tags_32 },
    { 0xb7864b8a, SUC_lexicon_tags_56 },
    { 0x19464b8b, SUC_lexicon_tags_25 },
    { 0x29064b86, SUC_lexicon_tags_8 },
    { 0x53464b8d, SUC_lexicon_tags_19 },
    { 0xb8464b8e, SUC_lexicon_tags_1 },
    { 0x48464b8e, SUC_lexicon_tags_33 },
    { 0xa0464b90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb264b97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a64b9a, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x44664b9d, SUC_lexicon_tags_0 },
    { 0x19864b9d, SUC_lexicon_tags_56 },
    { 0xbf864b9f, SUC_lexicon_tags_7 },
    { 0x41e64b9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93864ba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45e64ba5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23864ba7, SUC_lexicon_tags_17 },
    { 0x50664ba8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c64bb1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc64bb7, SUC_lexicon_tags_13 },
    { 0xc6664bb8, SUC_lexicon_tags_28 },
    { 0xca264bb7, SUC_lexicon_tags_47 },
    { 0xfac64bb7, SUC_lexicon_tags_63 },
    { 0x13464bbb, SUC_lexicon_tags_12 },
    { 0xcd864bbc, SUC_lexicon_tags_11 },
    { 0xac864bbd, SUC_lexicon_tags_28 },
    { 0x92e64bbe, SUC_lexicon_tags_49 },
    { 0x55264bbf, SUC_lexicon_tags_17 },
    { 0x7d864bc0, SUC_lexicon_tags_82 },
    { 0xb2464bbf, SUC_lexicon_tags_25 },
    { 0xc1064bc2, SUC_lexicon_tags_37 },
    { 0x9cc64bb8, SUC_lexicon_tags_118 },
    { 0x98c64bc0, SUC_lexicon_tags_37 },
    { 0x13c64bc2, SUC_lexicon_tags_38 },
    { 0xd1064bc6, SUC_lexicon_tags_37 },
    { 0x83664bc6, SUC_lexicon_tags_1 },
    { 0xef064bb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcdc64bca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83464bcc, SUC_lexicon_tags_7 },
    { 0xc1864bcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb664bd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d864bd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2464bd8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x41264bdb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x34264bdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee64be0, SUC_lexicon_tags_38 },
    { 0xd7664be1, SUC_lexicon_tags_33 },
    { 0xd2a64be2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4264be4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc64beb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe64bee, SUC_lexicon_tags_11 },
    { 0xd8064bee, SUC_lexicon_tags_2 },
    { 0x26e64bf0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x56264bf2, SUC_lexicon_tags_3 },
    { 0x63864bf3, SUC_lexicon_tags_43 },
    { 0x7c664bf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63e64bf6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98a64bf8, SUC_lexicon_tags_15 },
    { 0xc8e64bf9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d064bfd, SUC_lexicon_tags_17 },
    { 0x6f464bfe, SUC_lexicon_tags_37 },
    { 0xafc64bff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23864c01, SUC_lexicon_tags_2 },
    { 0x7f264c01, SUC_lexicon_tags_8 },
    { 0xaee64c01, SUC_lexicon_tags_27 },
    { 0xda464c02, SUC_lexicon_tags_2 },
    { 0xd3664c02, SUC_lexicon_tags_28 },
    { 0xe8e64c06, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a64c09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8664c0d, SUC_lexicon_tags_11 },
    { 0xd4664c0e, SUC_lexicon_tags_27 },
    { 0x3064c0f, SUC_lexicon_tags_27 },
    { 0x3ac64c0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc64c13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x51064c16, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee64c19, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x4ce64c1b, SUC_lexicon_tags_60 },
    { 0x9ba64c1b, SUC_lexicon_tags_25 },
    { 0x7de64c1b, SUC_lexicon_tags_28 },
    { 0x24864c1e, SUC_lexicon_tags_12 },
    { 0x5ba64c1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7e64c21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44a64c23, SUC_lexicon_tags_19 },
    { 0x32a64c24, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b864c28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92664c2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x78464c2d, SUC_lexicon_tags_2 },
    { 0x46c64c2d, SUC_lexicon_tags_25 },
    { 0x7b664c2f, SUC_lexicon_tags_7 },
    { 0x21864c2e, SUC_lexicon_tags_37 },
    { 0x12c64c2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9e864c33, SUC_lexicon_tags_46 },
    { 0xa5c64c34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1464c36, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c64c39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbc64c3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ac64c3d, SUC_lexicon_tags_91 },
    { 0x25664c3e, SUC_lexicon_tags_7 },
    { 0x4c64c3f, SUC_lexicon_tags_11 },
    { 0xb2c64c3f, SUC_lexicon_tags_56 },
    { 0xd7864c41, SUC_lexicon_tags_25 },
    { 0x3ac64c42, SUC_lexicon_tags_8 },
    { 0x63264c3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e64c4a, SUC_lexicon_tags_37 },
    { 0x43a64c4b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d664c51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8464c53, SUC_lexicon_tags_15 },
    { 0xbe664c54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c64c58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2f264c5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a664c5e, SUC_lexicon_tags_38 },
    { 0x7f664c5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66a64c61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b264c63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc264c65, SUC_lexicon_tags_68 },
    { 0xd0864c66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c64c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb864c6c, SUC_lexicon_tags_39 },
    { 0x56864c6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe2864c6f, SUC_lexicon_tags_1 },
    { 0x32264c6f, SUC_lexicon_tags_1 },
    { 0xee64c70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6264c79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56e64c7b, SUC_lexicon_tags_66 },
    { 0x60064c7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf664c7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe64c83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f664c88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa64c8c, SUC_lexicon_tags_7 },
    { 0x9ae64c8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9de64c8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e64c94, SUC_lexicon_tags_29 },
    { 0xcfc64c95, SUC_lexicon_tags_61 },
    { 0xafc64c95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea64c99, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x32c64c9b, SUC_lexicon_tags_17 },
    { 0x25264c9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90064ca2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d864ca5, SUC_lexicon_tags_39 },
    { 0xa5464ca5, SUC_lexicon_tags_13 },
    { 0x25464ca7, SUC_lexicon_tags_4 },
    { 0xa0264ca6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb864cb2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27864cbd, SUC_lexicon_tags_25 },
    { 0x66664cbe, SUC_lexicon_tags_26 },
    { 0x90464cbf, SUC_lexicon_tags_11 },
    { 0x8dc64cbf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8864cc8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e464ccb, SUC_lexicon_tags_19 },
    { 0x8a264ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e64cd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa864cd5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3264cda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed064cdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68864cde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde264ce0, SUC_lexicon_tags_133 },
    { 0x22864ce1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x73864ce3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x21864ce6, SUC_lexicon_tags_17 },
    { 0xd0c64ce7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48064ce9, SUC_lexicon_tags_15 },
    { 0x78664cea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x63664ced, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1864cf2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a264cf5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c264cf8, SUC_lexicon_tags_9 },
    { 0xb6a64cf9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a64cfd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xee864d00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaca64d02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78264d04, SUC_lexicon_tags_12 },
    { 0x94464d05, SUC_lexicon_tags_10 },
    { 0xd5a64d04, SUC_lexicon_tags_34 },
    { 0xeac64d04, SUC_lexicon_tags_29 },
    { 0xfe264d08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a64d0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c64d10, SUC_lexicon_tags_12 },
    { 0x5c464d11, SUC_lexicon_tags_23 },
    { 0xf7664d11, SUC_lexicon_tags_37 },
    { 0xa4864d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19664d15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8664d1f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x79664d21, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45464d25, SUC_lexicon_tags_26 },
    { 0xdba64d25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec64d2a, SUC_lexicon_tags_15 },
    { 0xd5864d2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e64d33, SUC_lexicon_tags_43 },
    { 0x60464d34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39a64d36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18264d3b, SUC_lexicon_tags_25 },
    { 0xdaa64d3c, SUC_lexicon_tags_17 },
    { 0x6a664d3d, SUC_lexicon_tags_9 },
    { 0x8c064d3c, SUC_lexicon_tags_25 },
    { 0x47664d3f, SUC_lexicon_tags_26 },
    { 0xc4464d3f, SUC_lexicon_tags_11 },
    { 0xfc664d41, SUC_lexicon_tags_1 },
    { 0xce264d42, SUC_lexicon_tags_7 },
    { 0x90064d41, SUC_lexicon_tags_60 },
    { 0x40c64d44, SUC_lexicon_tags_39 },
    { 0x54064d44, SUC_lexicon_tags_39 },
    { 0x9ce64d45, SUC_lexicon_tags_2 },
    { 0x22c64d47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x64864d49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8664d54, SUC_lexicon_tags_11 },
    { 0x59a64d55, SUC_lexicon_tags_1 },
    { 0xc8664d55, SUC_lexicon_tags_25 },
    { 0xe7664d55, SUC_lexicon_tags_15 },
    { 0x54e64d55, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e64d5c, SUC_lexicon_tags_1 },
    { 0x95464d5d, SUC_lexicon_tags_29 },
    { 0xe3a64d5e, SUC_lexicon_tags_87 },
    { 0x3f664d5d, SUC_lexicon_tags_11 },
    { 0x6a064d5e, SUC_lexicon_tags_17 },
    { 0x9ac64d61, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x68264d63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ea64d65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3664d6a, SUC_lexicon_tags_737 },
    { 0x45664d6a, SUC_lexicon_tags_15 },
    { 0x89664d6c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca64d6f, SUC_lexicon_tags_63 },
    { 0x4b864d70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96c64d72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xafa64d74, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe9464d76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c64d79, SUC_lexicon_tags_12 },
    { 0x51264d79, SUC_lexicon_tags_30 },
    { 0x60064d7b, SUC_lexicon_tags_25 },
    { 0x23864d7c, SUC_lexicon_tags_49 },
    { 0xdaa64d7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7a864d7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe7c64d81, SUC_lexicon_tags_45 },
    { 0x8f464d82, SUC_lexicon_tags_16 },
    { 0xa5864d83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28264d86, SUC_lexicon_tags_17 },
    { 0xa4c64d87, SUC_lexicon_tags_30 },
    { 0xa1864d88, SUC_lexicon_tags_3 },
    { 0xd3464d89, SUC_lexicon_tags_20 },
    { 0x5d264d8a, SUC_lexicon_tags_12 },
    { 0x26264d89, SUC_lexicon_tags_17 },
    { 0x63864d8c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a464d8f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a464d94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c64d98, SUC_lexicon_tags_101 },
    { 0xc64d99, SUC_lexicon_tags_47 },
    { 0xe2064d98, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf664d9d, SUC_lexicon_tags_11 },
    { 0x6d464d9d, SUC_lexicon_tags_1 },
    { 0xd5464d9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5864da3, SUC_lexicon_tags_29 },
    { 0xea864da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda064da9, SUC_lexicon_tags_33 },
    { 0x24e64da9, SUC_lexicon_tags_12 },
    { 0x36c64dab, SUC_lexicon_tags_2 },
    { 0x72064daa, SUC_lexicon_tags_138 },
    { 0x2c064dad, SUC_lexicon_tags_49 },
    { 0x37464dae, SUC_lexicon_tags_12 },
    { 0xba864dae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e64db3, SUC_lexicon_tags_1 },
    { 0xaaa64db4, SUC_lexicon_tags_28 },
    { 0x32464db4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e64db9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a64dbc, SUC_lexicon_tags_48 },
    { 0xbfa64dbd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e64dc1, SUC_lexicon_tags_25 },
    { 0x4064dc2, SUC_lexicon_tags_1 },
    { 0x76664dc3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x99464dc6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6bc64dc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97464dcd, SUC_lexicon_tags_12 },
    { 0xbf064dce, SUC_lexicon_tags_47 },
    { 0x84064dcf, SUC_lexicon_tags_87 },
    { 0x25064dd0, SUC_lexicon_tags_63 },
    { 0xe3264dce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a64dd3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfa64dd5, SUC_lexicon_tags_146 },
    { 0x16e64dd6, SUC_lexicon_tags_37 },
    { 0xad064dd5, SUC_lexicon_tags_11 },
    { 0xba264dd6, SUC_lexicon_tags_15 },
    { 0x1c464dd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e64ddf, SUC_lexicon_tags_48 },
    { 0xd7a64de0, SUC_lexicon_tags_17 },
    { 0xcee64ddf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e64de5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61664deb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69664df7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a64dfa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c64dfe, SUC_lexicon_tags_25 },
    { 0x3d064dfe, SUC_lexicon_tags_55 },
    { 0x16c64e00, SUC_lexicon_tags_63 },
    { 0xd0c64e00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94664e03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e64e06, SUC_lexicon_tags_39 },
    { 0x36664e07, SUC_lexicon_tags_27 },
    { 0x55464e07, SUC_lexicon_tags_1 },
    { 0x8e464e08, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcf664e0b, SUC_lexicon_tags_17 },
    { 0x4e664e0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b464e0e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd0864e10, SUC_lexicon_tags_19 },
    { 0x9e064e11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc64e15, SUC_lexicon_tags_47 },
    { 0x64e64e16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc064e18, SUC_lexicon_tags_15 },
    { 0x7be64e19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ce64e1b, SUC_lexicon_tags_19 },
    { 0x8bc64e1b, SUC_lexicon_tags_48 },
    { 0xece64e1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8264e22, SUC_lexicon_tags_65 },
    { 0x45264e23, SUC_lexicon_tags_1 },
    { 0xf4e64e24, SUC_lexicon_tags_49 },
    { 0x25664e25, SUC_lexicon_tags_43 },
    { 0xf5064e26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6264e29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x93064e2b, SUC_lexicon_tags_1 },
    { 0x7ba64e2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46e64e2e, SUC_lexicon_tags_11 },
    { 0xcfc64e2f, SUC_lexicon_tags_11 },
    { 0x1ec64e2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70064e33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd7664e35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7264e37, SUC_lexicon_tags_17 },
    { 0x20264e37, SUC_lexicon_tags_17 },
    { 0x46864e37, SUC_lexicon_tags_47 },
    { 0x664e3a, SUC_lexicon_tags_45 },
    { 0xf0664e3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8e64e3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ba64e3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97a64e41, SUC_lexicon_tags_2 },
    { 0x8e664e42, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe64e47, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x15464e4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfac64e4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8c64e4e, SUC_lexicon_tags_16 },
    { 0x57664e4f, SUC_lexicon_tags_47 },
    { 0xe1a64e50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x35a64e52, SUC_lexicon_tags_43 },
    { 0x2464e53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x92464e56, SUC_lexicon_tags_1 },
    { 0x68a64e56, SUC_lexicon_tags_28 },
    { 0xd2c64e58, SUC_lexicon_tags_31 },
    { 0xe4e64e59, SUC_lexicon_tags_49 },
    { 0x85a64e58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a64e61, SUC_lexicon_tags_49 },
    { 0x53864e62, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca64e6c, SUC_lexicon_tags_0 },
    { 0xafe64e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28664e6f, SUC_lexicon_tags_0 },
    { 0xdfc64e70, SUC_lexicon_tags_7 },
    { 0xbbc64e71, SUC_lexicon_tags_17 },
    { 0x62e64e71, SUC_lexicon_tags_27 },
    { 0x91a64e73, SUC_lexicon_tags_27 },
    { 0x85864e73, SUC_lexicon_tags_12 },
    { 0xa8264e71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x29a64e77, SUC_lexicon_tags_17 },
    { 0x6e664e78, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e64e7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c064e80, SUC_lexicon_tags_12 },
    { 0xb9c64e81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8264e84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc864e88, SUC_lexicon_tags_63 },
    { 0xbac64e88, SUC_lexicon_tags_37 },
    { 0xc6464e88, SUC_lexicon_tags_1 },
    { 0x89064e8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21064e90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff864e92, SUC_lexicon_tags_0 },
    { 0x93864e92, SUC_lexicon_tags_17 },
    { 0x9d264e92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9864e97, SUC_lexicon_tags_25 },
    { 0x8fe64e98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4fc64e9a, SUC_lexicon_tags_35 },
    { 0xd6c64e9b, SUC_lexicon_tags_15 },
    { 0x5c264e9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed464ea0, SUC_lexicon_tags_37 },
    { 0xdf464ea1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a464ea4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49064ea9, SUC_lexicon_tags_19 },
    { 0xcf464eaa, SUC_lexicon_tags_22 },
    { 0xab264eaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27864eb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f464eb6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1464eb9, SUC_lexicon_tags_56 },
    { 0x77664eba, SUC_lexicon_tags_108 },
    { 0x5ba64ebb, SUC_lexicon_tags_8 },
    { 0xadc64ebb, SUC_lexicon_tags_8 },
    { 0x7fe64ebc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c464ec2, SUC_lexicon_tags_17 },
    { 0x29064ec3, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x70064ec5, SUC_lexicon_tags_43 },
    { 0x81e64ec6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x14064ec9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x31664ecc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e64ecf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc64ed8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6464edc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4664ee0, SUC_lexicon_tags_29 },
    { 0xbdc64ee0, SUC_lexicon_tags_2 },
    { 0xd5464ee1, SUC_lexicon_tags_13 },
    { 0xdb464ee3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x51064ee6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf664ee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43864eeb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x69664eee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x68264ef1, SUC_lexicon_tags_19 },
    { 0xc2664ef2, SUC_lexicon_tags_29 },
    { 0x3a264ef3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c64ef7, SUC_lexicon_tags_17 },
    { 0x2c464ef7, SUC_lexicon_tags_47 },
    { 0x6c864ef8, SUC_lexicon_tags_69 },
    { 0xc4264efa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x80864efd, SUC_lexicon_tags_15 },
    { 0x69864efe, SUC_lexicon_tags_9 },
    { 0xd5664eff, SUC_lexicon_tags_56 },
    { 0x4ce64f00, SUC_lexicon_tags_25 },
    { 0x5b664efd, SUC_lexicon_tags_16 },
    { 0xad264f02, SUC_lexicon_tags_7 },
    { 0x95264efd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x75a64f05, SUC_lexicon_tags_15 },
    { 0x17064f06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc64f08, SUC_lexicon_tags_15 },
    { 0xe5c64f08, SUC_lexicon_tags_11 },
    { 0x6b264f0a, SUC_lexicon_tags_17 },
    { 0x30264f0a, SUC_lexicon_tags_60 },
    { 0x5a664f0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a64f0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65064f11, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d264f14, SUC_lexicon_tags_11 },
    { 0x64f15, SUC_lexicon_tags_9 },
    { 0x9aa64f15, SUC_lexicon_tags_0 },
    { 0x97064f17, SUC_lexicon_tags_61 },
    { 0x19664f18, SUC_lexicon_tags_49 },
    { 0x33a64f17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ba64f1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f664f1e, SUC_lexicon_tags_29 },
    { 0x78264f1e, SUC_lexicon_tags_9 },
    { 0x72664f20, SUC_lexicon_tags_0 },
    { 0xc7c64f21, SUC_lexicon_tags_27 },
    { 0xa6e64f1f, SUC_lexicon_tags_2 },
    { 0x61e64f1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b664f25, SUC_lexicon_tags_32 },
    { 0x65a64f25, SUC_lexicon_tags_37 },
    { 0x20064f27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87064f2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41064f2e, SUC_lexicon_tags_11 },
    { 0xe064f2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3864f32, SUC_lexicon_tags_11 },
    { 0xc9064f32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba64f36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77464f3b, SUC_lexicon_tags_63 },
    { 0xde064f3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14064f40, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe64f44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72464f46, SUC_lexicon_tags_68 },
    { 0xb2a64f47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa064f4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38064f51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e664f53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f064f5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8e64f5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9664f5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d264f60, SUC_lexicon_tags_7 },
    { 0x50264f61, SUC_lexicon_tags_27 },
    { 0xe5264f60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc64f65, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c264f69, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a64f6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8664f71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4264f73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c664f75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9264f7a, SUC_lexicon_tags_33 },
    { 0xb2e64f7a, SUC_lexicon_tags_49 },
    { 0x8d264f7c, SUC_lexicon_tags_15 },
    { 0x3e464f7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d464f82, SUC_lexicon_tags_15 },
    { 0x6ea64f83, SUC_lexicon_tags_2 },
    { 0xbf264f84, SUC_lexicon_tags_37 },
    { 0x82c64f84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c64f88, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4864f8b, SUC_lexicon_tags_15 },
    { 0x3ca64f8b, SUC_lexicon_tags_2 },
    { 0xe064f8d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7864f92, SUC_lexicon_tags_49 },
    { 0xd4a64f92, SUC_lexicon_tags_12 },
    { 0x31e64f94, SUC_lexicon_tags_1 },
    { 0xc4064f93, SUC_lexicon_tags_15 },
    { 0x40a64f96, SUC_lexicon_tags_79 },
    { 0x95664f93, SUC_lexicon_tags_37 },
    { 0xefe64f98, SUC_lexicon_tags_418 },
    { 0xe664f97, SUC_lexicon_tags_17 },
    { 0xa0064f92, SUC_lexicon_tags_13 },
    { 0x6d864f94, SUC_lexicon_tags_55 },
    { 0xd6664f93, SUC_lexicon_tags_38 },
    { 0x79264f9d, SUC_lexicon_tags_2 },
    { 0xec64f97, SUC_lexicon_tags_28 },
    { 0x10464f9f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9464fa4, SUC_lexicon_tags_7 },
    { 0x24664fa4, SUC_lexicon_tags_17 },
    { 0xc464fa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4de64fa8, SUC_lexicon_tags_38 },
    { 0xcf464fa8, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1664fb1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6c64fb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9f064fb5, SUC_lexicon_tags_26 },
    { 0x51864fb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c064fb8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0664fbb, SUC_lexicon_tags_33 },
    { 0x87664fbc, SUC_lexicon_tags_27 },
    { 0xf4864fbc, SUC_lexicon_tags_85 },
    { 0xe4664fbb, SUC_lexicon_tags_6 },
    { 0xd2a64fbc, SUC_lexicon_tags_19 },
    { 0x2a64fc0, SUC_lexicon_tags_15 },
    { 0xb7264fbe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x74a64fc3, SUC_lexicon_tags_11 },
    { 0x43064fc4, SUC_lexicon_tags_37 },
    { 0x5d464fc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c64fc8, SUC_lexicon_tags_5 },
    { 0x4aa64fc8, SUC_lexicon_tags_60 },
    { 0xff264fc8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e64fce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23264fd2, SUC_lexicon_tags_30 },
    { 0xc4864fd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e64fd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13064fdb, SUC_lexicon_tags_25 },
    { 0x25064fdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7264fdf, SUC_lexicon_tags_15 },
    { 0xc5e64fe0, SUC_lexicon_tags_16 },
    { 0x47664fdf, SUC_lexicon_tags_18 },
    { 0x34e64fe0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3264fe4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c64fe7, SUC_lexicon_tags_7 },
    { 0x1fe64fe7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf864fec, SUC_lexicon_tags_38 },
    { 0xcd264fed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80664ff3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ac64ff5, SUC_lexicon_tags_26 },
    { 0xe9864ff6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ee64ff8, SUC_lexicon_tags_47 },
    { 0x15664ff9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a464ffd, SUC_lexicon_tags_47 },
    { 0x47e64ffe, SUC_lexicon_tags_15 },
    { 0xd9664ffd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7465004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a66500a, SUC_lexicon_tags_31 },
    { 0xab26500b, SUC_lexicon_tags_5 },
    { 0x3ce6500c, SUC_lexicon_tags_1 },
    { 0xef06500d, SUC_lexicon_tags_41 },
    { 0xb646500e, SUC_lexicon_tags_30 },
    { 0xf06500f, SUC_lexicon_tags_37 },
    { 0x5086500e, SUC_lexicon_tags_37 },
    { 0x5246500f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad265013, SUC_lexicon_tags_10 },
    { 0xf1a65014, SUC_lexicon_tags_8 },
    { 0x31065015, SUC_lexicon_tags_11 },
    { 0xed465014, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c6501f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf265026, SUC_lexicon_tags_16 },
    { 0x19265027, SUC_lexicon_tags_11 },
    { 0x25c65028, SUC_lexicon_tags_26 },
    { 0x82a65029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc666502d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe265032, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a865034, SUC_lexicon_tags_11 },
    { 0x7de65035, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6265037, SUC_lexicon_tags_55 },
    { 0xeac65038, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c6503d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd86503f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda65042, SUC_lexicon_tags_28 },
    { 0x2b065043, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac65048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca66504a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8866504d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcde6504f, SUC_lexicon_tags_25 },
    { 0x61265050, SUC_lexicon_tags_33 },
    { 0x4a065050, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5665056, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106505b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd06505d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3065060, SUC_lexicon_tags_1 },
    { 0x81665060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x65065063, SUC_lexicon_tags_1 },
    { 0xeb265064, SUC_lexicon_tags_85 },
    { 0xe1265064, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb8c65067, SUC_lexicon_tags_15 },
    { 0xcc065068, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd26506a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0265071, SUC_lexicon_tags_23 },
    { 0x9ea65072, SUC_lexicon_tags_19 },
    { 0x30265073, SUC_lexicon_tags_46 },
    { 0x1d065071, SUC_lexicon_tags_12 },
    { 0x57c65075, SUC_lexicon_tags_17 },
    { 0xc3c65076, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbb465078, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c6507c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68465082, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe7265084, SUC_lexicon_tags_25 },
    { 0x46e65085, SUC_lexicon_tags_17 },
    { 0x71c65086, SUC_lexicon_tags_0 },
    { 0xd7a65087, SUC_lexicon_tags_47 },
    { 0xb8065084, SUC_lexicon_tags_435 },
    { 0x7065089, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa606508c, SUC_lexicon_tags_13 },
    { 0xd086508d, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x7946508f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f265092, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f065095, SUC_lexicon_tags_37 },
    { 0xbf265096, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc006509b, SUC_lexicon_tags_15 },
    { 0xd8e6509c, SUC_lexicon_tags_17 },
    { 0x41e6509d, SUC_lexicon_tags_33 },
    { 0x8e26509c, SUC_lexicon_tags_3 },
    { 0x8c66509c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78650a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4650a6, SUC_lexicon_tags_44 },
    { 0x794650a7, SUC_lexicon_tags_34 },
    { 0x8c6650a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a650ab, SUC_lexicon_tags_19 },
    { 0xa64650ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x672650af, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6650b2, SUC_lexicon_tags_25 },
    { 0x8c4650b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb4650b5, SUC_lexicon_tags_83 },
    { 0x4a8650b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec650bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a650bf, SUC_lexicon_tags_38 },
    { 0x3bc650c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb90650c2, SUC_lexicon_tags_7 },
    { 0xeec650c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8650c7, SUC_lexicon_tags_1 },
    { 0x7a6650c8, SUC_lexicon_tags_9 },
    { 0xfae650c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e650cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c650d0, SUC_lexicon_tags_6 },
    { 0x262650d1, SUC_lexicon_tags_49 },
    { 0xba0650d2, SUC_lexicon_tags_8 },
    { 0x62a650d0, SUC_lexicon_tags_63 },
    { 0xb1c650d2, SUC_lexicon_tags_7 },
    { 0x3e0650d5, SUC_lexicon_tags_27 },
    { 0x71a650d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e650da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c650de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a650e4, SUC_lexicon_tags_8 },
    { 0x3de650e5, SUC_lexicon_tags_2 },
    { 0x76650e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a650ec, SUC_lexicon_tags_1 },
    { 0xc3e650ed, SUC_lexicon_tags_30 },
    { 0x102650ec, SUC_lexicon_tags_37 },
    { 0xb9e650ed, SUC_lexicon_tags_14 },
    { 0x7e8650ec, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x52a650f2, SUC_lexicon_tags_11 },
    { 0x184650f3, SUC_lexicon_tags_25 },
    { 0x25e650f3, SUC_lexicon_tags_45 },
    { 0x6e0650f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x50c650f7, SUC_lexicon_tags_30 },
    { 0x446650f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920650fc, SUC_lexicon_tags_10 },
    { 0x8d2650fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e65102, SUC_lexicon_tags_29 },
    { 0x84665103, SUC_lexicon_tags_37 },
    { 0x82065104, SUC_lexicon_tags_12 },
    { 0x40865105, SUC_lexicon_tags_33 },
    { 0x2b665104, SUC_lexicon_tags_13 },
    { 0x72665102, SUC_lexicon_tags_0 },
    { 0xf0665108, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9286510d, SUC_lexicon_tags_31 },
    { 0xdbe6510e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e65111, SUC_lexicon_tags_1 },
    { 0xc4465112, SUC_lexicon_tags_2 },
    { 0xfd065113, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe65118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8265124, SUC_lexicon_tags_37 },
    { 0xcfa65125, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e6512a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3e6512c, SUC_lexicon_tags_21 },
    { 0x4686512d, SUC_lexicon_tags_12 },
    { 0x49e6512e, SUC_lexicon_tags_63 },
    { 0xfca6512c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a465133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c665135, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba6513b, SUC_lexicon_tags_11 },
    { 0xdc26513c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4265141, SUC_lexicon_tags_45 },
    { 0x48665142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a665147, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1c465149, SUC_lexicon_tags_0 },
    { 0xc986514a, SUC_lexicon_tags_19 },
    { 0x6d66514b, SUC_lexicon_tags_49 },
    { 0x88465149, SUC_lexicon_tags_2 },
    { 0xdfc6514d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7266514f, SUC_lexicon_tags_7 },
    { 0xe006514f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3665154, SUC_lexicon_tags_32 },
    { 0x22465154, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8065157, SUC_lexicon_tags_17 },
    { 0xaa865158, SUC_lexicon_tags_33 },
    { 0xc0265159, SUC_lexicon_tags_25 },
    { 0x4a26515a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b26515e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x37e65160, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa865167, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e6516b, SUC_lexicon_tags_14 },
    { 0xf866516c, SUC_lexicon_tags_43 },
    { 0xad46516b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bc6516f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d265171, SUC_lexicon_tags_63 },
    { 0xd4865172, SUC_lexicon_tags_63 },
    { 0x2e465173, SUC_lexicon_tags_49 },
    { 0x23e65173, SUC_lexicon_tags_84 },
    { 0x30e65174, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x72e65177, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9066517a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a6517d, SUC_lexicon_tags_49 },
    { 0x72a6517e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xee665180, SUC_lexicon_tags_17 },
    { 0x26e65181, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc65186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf46518a, SUC_lexicon_tags_49 },
    { 0x2c66518a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18065190, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c65193, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1e665195, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e65198, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1f86519a, SUC_lexicon_tags_8 },
    { 0x2ac6519b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6651a2, SUC_lexicon_tags_17 },
    { 0x476651a2, SUC_lexicon_tags_17 },
    { 0x68c651a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82651a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x166651ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc651b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a651b7, SUC_lexicon_tags_15 },
    { 0x41a651b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e651bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2651be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0651c3, SUC_lexicon_tags_44 },
    { 0x4a8651c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0651c7, SUC_lexicon_tags_17 },
    { 0x74a651c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x966651ca, SUC_lexicon_tags_32 },
    { 0xde4651ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474651d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4651d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x158651d7, SUC_lexicon_tags_28 },
    { 0xbbc651d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e651db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x824651dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed2651df, SUC_lexicon_tags_45 },
    { 0xbee651e0, SUC_lexicon_tags_47 },
    { 0xec4651e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe10651e3, SUC_lexicon_tags_9 },
    { 0x73a651e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d8651e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4651e9, SUC_lexicon_tags_9 },
    { 0x506651ea, SUC_lexicon_tags_10 },
    { 0xc651ea, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8e0651ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a651f0, SUC_lexicon_tags_15 },
    { 0x330651f0, SUC_lexicon_tags_29 },
    { 0x63e651f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe98651f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc651f7, SUC_lexicon_tags_23 },
    { 0xc6a651f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x728651fb, SUC_lexicon_tags_38 },
    { 0xb70651fc, SUC_lexicon_tags_37 },
    { 0x36c651fd, SUC_lexicon_tags_43 },
    { 0xfe6651fe, SUC_lexicon_tags_911 },
    { 0xe40651ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2865201, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92065204, SUC_lexicon_tags_49 },
    { 0xa2665205, SUC_lexicon_tags_43 },
    { 0x6f265205, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da65209, SUC_lexicon_tags_65 },
    { 0xbda6520a, SUC_lexicon_tags_65 },
    { 0x6706520b, SUC_lexicon_tags_44 },
    { 0x8f66520c, SUC_lexicon_tags_82 },
    { 0x3c86520b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c65210, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81065214, SUC_lexicon_tags_7 },
    { 0xf6665214, SUC_lexicon_tags_17 },
    { 0xc6665216, SUC_lexicon_tags_12 },
    { 0x75c65215, SUC_lexicon_tags_37 },
    { 0x4465217, SUC_lexicon_tags_48 },
    { 0xe6c65219, SUC_lexicon_tags_18 },
    { 0x36a65218, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b86521c, SUC_lexicon_tags_45 },
    { 0x6ce6521d, SUC_lexicon_tags_0 },
    { 0x10e6521e, SUC_lexicon_tags_68 },
    { 0x1186521f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78265221, SUC_lexicon_tags_27 },
    { 0x16865221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa65224, SUC_lexicon_tags_30 },
    { 0x53c65225, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec265227, SUC_lexicon_tags_43 },
    { 0xd7065228, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4006522a, SUC_lexicon_tags_21 },
    { 0x7ec6522b, SUC_lexicon_tags_753 },
    { 0x9986522a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea65233, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3665237, SUC_lexicon_tags_12 },
    { 0x1a065237, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc566523a, SUC_lexicon_tags_27 },
    { 0xeb46523b, SUC_lexicon_tags_11 },
    { 0x6b26523c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e46523e, SUC_lexicon_tags_4 },
    { 0x24a6523f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66265245, SUC_lexicon_tags_17 },
    { 0x92e65246, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec65249, SUC_lexicon_tags_37 },
    { 0xde46524a, SUC_lexicon_tags_12 },
    { 0xce6524b, SUC_lexicon_tags_1 },
    { 0x97c6524c, SUC_lexicon_tags_15 },
    { 0x36e6524d, SUC_lexicon_tags_77 },
    { 0x6426524e, SUC_lexicon_tags_51 },
    { 0x3ce6524e, SUC_lexicon_tags_25 },
    { 0x3a46524b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c65256, SUC_lexicon_tags_8 },
    { 0xa7e65257, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde6525a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12a6525c, SUC_lexicon_tags_29 },
    { 0xb466525d, SUC_lexicon_tags_37 },
    { 0xe246525c, SUC_lexicon_tags_1 },
    { 0xd2a6525f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e665264, SUC_lexicon_tags_57 },
    { 0x17865265, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3c665267, SUC_lexicon_tags_17 },
    { 0xec265267, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9046526c, SUC_lexicon_tags_15 },
    { 0x3766526c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0465271, SUC_lexicon_tags_87 },
    { 0x8bc65272, SUC_lexicon_tags_17 },
    { 0x27265273, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d865275, SUC_lexicon_tags_77 },
    { 0x13065276, SUC_lexicon_tags_2 },
    { 0xc3a65277, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8786527d, SUC_lexicon_tags_6 },
    { 0x29c6527d, SUC_lexicon_tags_98 },
    { 0xcca6527f, SUC_lexicon_tags_15 },
    { 0x6d26527f, SUC_lexicon_tags_47 },
    { 0xc246527d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae465289, SUC_lexicon_tags_31 },
    { 0x4de6528a, SUC_lexicon_tags_30 },
    { 0x4046528a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9786528d, SUC_lexicon_tags_37 },
    { 0xede6528e, SUC_lexicon_tags_38 },
    { 0x326528e, SUC_lexicon_tags_47 },
    { 0x3586528f, SUC_lexicon_tags_2 },
    { 0xb7065290, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4265295, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa65298, SUC_lexicon_tags_400 },
    { 0x68c65298, SUC_lexicon_tags_12 },
    { 0x9ce65298, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa206529d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf06529f, SUC_lexicon_tags_17 },
    { 0x1fe652a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a652a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x630652a9, SUC_lexicon_tags_46 },
    { 0x6ea652aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48652ae, SUC_lexicon_tags_14 },
    { 0x97e652ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a652b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8652b8, SUC_lexicon_tags_301 },
    { 0x21c652b9, SUC_lexicon_tags_12 },
    { 0x192652ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0652be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38652c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c652ca, SUC_lexicon_tags_1 },
    { 0x24c652cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea2652cd, SUC_lexicon_tags_27 },
    { 0xacc652ce, SUC_lexicon_tags_37 },
    { 0xef0652cf, SUC_lexicon_tags_38 },
    { 0x5fe652cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a652d5, SUC_lexicon_tags_12 },
    { 0xefa652d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x774652d8, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94652db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdda652dd, SUC_lexicon_tags_29 },
    { 0x4652de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6652e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c652e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4652eb, SUC_lexicon_tags_1 },
    { 0x7b6652ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e652f1, SUC_lexicon_tags_19 },
    { 0x96c652f1, SUC_lexicon_tags_63 },
    { 0x47a652f3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44652fa, SUC_lexicon_tags_23 },
    { 0x810652fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x466652fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da65301, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63465303, SUC_lexicon_tags_37 },
    { 0x74065304, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c65308, SUC_lexicon_tags_68 },
    { 0xe865309, SUC_lexicon_tags_37 },
    { 0x5306530a, SUC_lexicon_tags_39 },
    { 0x4666530b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59265314, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50e65316, SUC_lexicon_tags_45 },
    { 0x2265317, SUC_lexicon_tags_17 },
    { 0x8b465318, SUC_lexicon_tags_12 },
    { 0x99265319, SUC_lexicon_tags_8 },
    { 0x2506531a, SUC_lexicon_tags_19 },
    { 0x2da6531b, SUC_lexicon_tags_43 },
    { 0x98c6531c, SUC_lexicon_tags_15 },
    { 0xf926531d, SUC_lexicon_tags_12 },
    { 0xc826531d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4265320, SUC_lexicon_tags_25 },
    { 0xa2a65321, SUC_lexicon_tags_15 },
    { 0xbc65321, SUC_lexicon_tags_61 },
    { 0x57265323, SUC_lexicon_tags_25 },
    { 0x20465324, SUC_lexicon_tags_60 },
    { 0xe6465325, SUC_lexicon_tags_37 },
    { 0xe3665326, SUC_lexicon_tags_2 },
    { 0x29e65325, SUC_lexicon_tags_7 },
    { 0xd5a65328, SUC_lexicon_tags_11 },
    { 0x47065327, SUC_lexicon_tags_6 },
    { 0x9fc6532a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b66532d, SUC_lexicon_tags_30 },
    { 0x2146532d, SUC_lexicon_tags_38 },
    { 0x7086532f, SUC_lexicon_tags_17 },
    { 0x13a65330, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1065333, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd66533a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfe6533c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6a6533e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a265341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c65347, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc6534a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c6534d, SUC_lexicon_tags_37 },
    { 0xf0e6534d, SUC_lexicon_tags_1 },
    { 0xb046534f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12265354, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8fa65356, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28265363, SUC_lexicon_tags_230 },
    { 0xd8a65364, SUC_lexicon_tags_0 },
    { 0xc7665363, SUC_lexicon_tags_19 },
    { 0xb8665365, SUC_lexicon_tags_17 },
    { 0x6a265366, SUC_lexicon_tags_14 },
    { 0x15265368, SUC_lexicon_tags_27 },
    { 0xca465368, SUC_lexicon_tags_201 },
    { 0xa8a6536a, SUC_lexicon_tags_16 },
    { 0xc1e6536a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a06536d, SUC_lexicon_tags_2 },
    { 0xfae6536e, SUC_lexicon_tags_8 },
    { 0x5526536e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9b865371, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x85265373, SUC_lexicon_tags_2 },
    { 0x83065374, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4766537c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96e6537e, SUC_lexicon_tags_2 },
    { 0xcc66537f, SUC_lexicon_tags_33 },
    { 0x9a65380, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b265386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5865389, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xee86538b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3126538f, SUC_lexicon_tags_4 },
    { 0x3bc65390, SUC_lexicon_tags_7 },
    { 0xdf265390, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce065396, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c65399, SUC_lexicon_tags_174 },
    { 0x63465399, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22653a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54653a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92653a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x244653ab, SUC_lexicon_tags_9 },
    { 0x95e653ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de653b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x712653b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa653b6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72653ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x782653bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x662653bf, SUC_lexicon_tags_60 },
    { 0xde6653bf, SUC_lexicon_tags_28 },
    { 0x2e0653c1, SUC_lexicon_tags_1 },
    { 0xd22653c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84653c8, SUC_lexicon_tags_49 },
    { 0x9e2653c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b6653cb, SUC_lexicon_tags_15 },
    { 0xbca653cc, SUC_lexicon_tags_7 },
    { 0xfda653cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c653d3, SUC_lexicon_tags_33 },
    { 0x214653d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf2653d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6653e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x720653e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0653e7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4653ea, SUC_lexicon_tags_21 },
    { 0xe60653ea, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2653f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x500653f2, SUC_lexicon_tags_2 },
    { 0xf04653f3, SUC_lexicon_tags_38 },
    { 0x51a653f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570653f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72865400, SUC_lexicon_tags_6 },
    { 0x1e265401, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc665405, SUC_lexicon_tags_23 },
    { 0xe4a65406, SUC_lexicon_tags_11 },
    { 0xf1465406, SUC_lexicon_tags_21 },
    { 0xfa65408, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca06540a, SUC_lexicon_tags_2 },
    { 0xac46540b, SUC_lexicon_tags_0 },
    { 0x5b06540c, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea865410, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a065412, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf8065414, SUC_lexicon_tags_15 },
    { 0x64a65415, SUC_lexicon_tags_7 },
    { 0x7c865414, SUC_lexicon_tags_30 },
    { 0x81a65416, SUC_lexicon_tags_25 },
    { 0xda65415, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9b86541a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc346541d, SUC_lexicon_tags_45 },
    { 0x4866541e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51865422, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa65427, SUC_lexicon_tags_4 },
    { 0x97065428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4166542b, SUC_lexicon_tags_11 },
    { 0xa1e6542c, SUC_lexicon_tags_60 },
    { 0xc926542c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xf886542f, SUC_lexicon_tags_32 },
    { 0xc226542f, SUC_lexicon_tags_1 },
    { 0x8e6542f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be65434, SUC_lexicon_tags_26 },
    { 0x3c865435, SUC_lexicon_tags_27 },
    { 0xf6665436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0265439, SUC_lexicon_tags_1 },
    { 0xf4465439, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e65440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c65445, SUC_lexicon_tags_19 },
    { 0xe3665446, SUC_lexicon_tags_11 },
    { 0x6d265445, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xb6e65449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e6544f, SUC_lexicon_tags_63 },
    { 0x62a6544f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce665456, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d26545a, SUC_lexicon_tags_26 },
    { 0xa046545b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdba6545d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37465461, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c65468, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9626546a, SUC_lexicon_tags_15 },
    { 0xc86546b, SUC_lexicon_tags_19 },
    { 0xfae6546c, SUC_lexicon_tags_1 },
    { 0xb6c6546a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc26546f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2665471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86465473, SUC_lexicon_tags_43 },
    { 0x84465473, SUC_lexicon_tags_17 },
    { 0xae865473, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf8665477, SUC_lexicon_tags_7 },
    { 0x2d265478, SUC_lexicon_tags_28 },
    { 0x1c265477, SUC_lexicon_tags_0 },
    { 0xd186547a, SUC_lexicon_tags_38 },
    { 0xc026547b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f26547f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe65482, SUC_lexicon_tags_48 },
    { 0xfc465482, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c65486, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x49265488, SUC_lexicon_tags_17 },
    { 0x72a65489, SUC_lexicon_tags_15 },
    { 0x4f465488, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa6548e, SUC_lexicon_tags_11 },
    { 0x6306548e, SUC_lexicon_tags_11 },
    { 0x6a865490, SUC_lexicon_tags_19 },
    { 0x39c6548f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x16265493, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c65496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1e65498, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7806549a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d26549e, SUC_lexicon_tags_7 },
    { 0x4866549f, SUC_lexicon_tags_26 },
    { 0xf8c6549f, SUC_lexicon_tags_12 },
    { 0x7a2654a1, SUC_lexicon_tags_10 },
    { 0x236654a1, SUC_lexicon_tags_79 },
    { 0x9ba654a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e654a6, SUC_lexicon_tags_37 },
    { 0x610654a7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x436654a9, SUC_lexicon_tags_39 },
    { 0x412654a9, SUC_lexicon_tags_168 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152654af, SUC_lexicon_tags_17 },
    { 0xec654b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4654b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fa654b8, SUC_lexicon_tags_37 },
    { 0xab6654b9, SUC_lexicon_tags_9 },
    { 0x248654ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0654bf, SUC_lexicon_tags_31 },
    { 0xcd2654bf, SUC_lexicon_tags_32 },
    { 0x5ac654c1, SUC_lexicon_tags_47 },
    { 0x732654c2, SUC_lexicon_tags_29 },
    { 0xf56654bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x960654c6, SUC_lexicon_tags_37 },
    { 0x848654c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa20654c9, SUC_lexicon_tags_25 },
    { 0x316654ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c654ce, SUC_lexicon_tags_14 },
    { 0xee8654cf, SUC_lexicon_tags_37 },
    { 0xe7e654ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x116654d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0654db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5d8654dd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06654e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c654e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930654ed, SUC_lexicon_tags_37 },
    { 0xeae654ed, SUC_lexicon_tags_1 },
    { 0x1a6654ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ca654f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbf6654f3, SUC_lexicon_tags_26 },
    { 0xc26654f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120654f8, SUC_lexicon_tags_12 },
    { 0x150654f8, SUC_lexicon_tags_37 },
    { 0x4f6654fa, SUC_lexicon_tags_6 },
    { 0x40e654f8, SUC_lexicon_tags_6 },
    { 0xa60654fc, SUC_lexicon_tags_60 },
    { 0xa60654fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae65502, SUC_lexicon_tags_63 },
    { 0xc1c65503, SUC_lexicon_tags_68 },
    { 0x72465502, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a65508, SUC_lexicon_tags_29 },
    { 0xdba65509, SUC_lexicon_tags_6 },
    { 0x4d86550a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf66550d, SUC_lexicon_tags_25 },
    { 0x2ca6550e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9665511, SUC_lexicon_tags_21 },
    { 0x67c65511, SUC_lexicon_tags_25 },
    { 0x24465513, SUC_lexicon_tags_2 },
    { 0x18c65514, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7c465516, SUC_lexicon_tags_2 },
    { 0x85a65517, SUC_lexicon_tags_10 },
    { 0x3c65516, SUC_lexicon_tags_27 },
    { 0xa0065518, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca65522, SUC_lexicon_tags_25 },
    { 0x9f665523, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2606552a, SUC_lexicon_tags_60 },
    { 0x2d26552b, SUC_lexicon_tags_69 },
    { 0xe166552c, SUC_lexicon_tags_37 },
    { 0x4586552c, SUC_lexicon_tags_37 },
    { 0xcb86552c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e465530, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2065534, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb7a65536, SUC_lexicon_tags_47 },
    { 0x41e65537, SUC_lexicon_tags_30 },
    { 0x2fc65538, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2846553a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3526553f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36865544, SUC_lexicon_tags_9 },
    { 0xcde65545, SUC_lexicon_tags_25 },
    { 0xb7c65546, SUC_lexicon_tags_11 },
    { 0x86c65546, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f66554b, SUC_lexicon_tags_7 },
    { 0x5406554c, SUC_lexicon_tags_1 },
    { 0x1f66554c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a265556, SUC_lexicon_tags_27 },
    { 0x39a65557, SUC_lexicon_tags_209 },
    { 0, NULL },
    { 0, NULL },
    { 0x6566555a, SUC_lexicon_tags_31 },
    { 0xf746555a, SUC_lexicon_tags_37 },
    { 0x79e6555b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e665561, SUC_lexicon_tags_39 },
    { 0x87665562, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2a65564, SUC_lexicon_tags_17 },
    { 0x1865565, SUC_lexicon_tags_47 },
    { 0x16665566, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e865568, SUC_lexicon_tags_27 },
    { 0xba65569, SUC_lexicon_tags_12 },
    { 0x50a65568, SUC_lexicon_tags_43 },
    { 0x906556b, SUC_lexicon_tags_11 },
    { 0xa4e6556c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e46556f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe65573, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdee65575, SUC_lexicon_tags_17 },
    { 0x9ee65576, SUC_lexicon_tags_15 },
    { 0x38665576, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50c65579, SUC_lexicon_tags_25 },
    { 0x93c65579, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4a06557c, SUC_lexicon_tags_33 },
    { 0xc7e6557c, SUC_lexicon_tags_17 },
    { 0xaea6557d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d665582, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1865588, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a6558b, SUC_lexicon_tags_30 },
    { 0x8ec6558c, SUC_lexicon_tags_0 },
    { 0x1f26558d, SUC_lexicon_tags_45 },
    { 0xeea6558e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb2465590, SUC_lexicon_tags_1 },
    { 0xeec65591, SUC_lexicon_tags_25 },
    { 0x4fe65590, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x33c65594, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a65598, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c6559a, SUC_lexicon_tags_48 },
    { 0xda26559a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6fa6559d, SUC_lexicon_tags_23 },
    { 0xd746559e, SUC_lexicon_tags_47 },
    { 0x46e6559e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2655a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x796655a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x606655aa, SUC_lexicon_tags_17 },
    { 0x356655ab, SUC_lexicon_tags_4 },
    { 0xbd0655ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62655ae, SUC_lexicon_tags_15 },
    { 0xc6c655ae, SUC_lexicon_tags_17 },
    { 0x4f2655af, SUC_lexicon_tags_21 },
    { 0xc9c655b1, SUC_lexicon_tags_12 },
    { 0x34c655b2, SUC_lexicon_tags_12 },
    { 0xb60655b3, SUC_lexicon_tags_10 },
    { 0x3e2655b4, SUC_lexicon_tags_9 },
    { 0x136655b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8de655b7, SUC_lexicon_tags_28 },
    { 0xc7a655b8, SUC_lexicon_tags_12 },
    { 0x6ac655b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936655c3, SUC_lexicon_tags_8 },
    { 0x5dc655c4, SUC_lexicon_tags_17 },
    { 0x3bc655c3, SUC_lexicon_tags_68 },
    { 0x906655c6, SUC_lexicon_tags_26 },
    { 0xb68655c7, SUC_lexicon_tags_37 },
    { 0x8ae655c7, SUC_lexicon_tags_14 },
    { 0x256655c6, SUC_lexicon_tags_15 },
    { 0xcf6655c7, SUC_lexicon_tags_25 },
    { 0x5c4655c5, SUC_lexicon_tags_11 },
    { 0x972655cc, SUC_lexicon_tags_43 },
    { 0xa12655cc, SUC_lexicon_tags_11 },
    { 0xa80655cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3e2655d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdda655d2, SUC_lexicon_tags_31 },
    { 0x442655d3, SUC_lexicon_tags_27 },
    { 0x306655d3, SUC_lexicon_tags_11 },
    { 0xa3c655d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e655db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a655de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2655e2, SUC_lexicon_tags_2 },
    { 0xab8655e3, SUC_lexicon_tags_28 },
    { 0x528655e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81e655e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdee655e8, SUC_lexicon_tags_11 },
    { 0x204655e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66655ed, SUC_lexicon_tags_34 },
    { 0x40a655ee, SUC_lexicon_tags_23 },
    { 0xe06655ed, SUC_lexicon_tags_0 },
    { 0x2d2655ee, SUC_lexicon_tags_13 },
    { 0x772655f1, SUC_lexicon_tags_25 },
    { 0xe20655f2, SUC_lexicon_tags_12 },
    { 0x5b8655f2, SUC_lexicon_tags_12 },
    { 0xec4655f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c655f6, SUC_lexicon_tags_38 },
    { 0x13c655f7, SUC_lexicon_tags_2 },
    { 0x98655f7, SUC_lexicon_tags_11 },
    { 0x156655f9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4655fc, SUC_lexicon_tags_13 },
    { 0xda8655fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a65600, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e465604, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x20065607, SUC_lexicon_tags_17 },
    { 0xab265608, SUC_lexicon_tags_69 },
    { 0xb7a65609, SUC_lexicon_tags_15 },
    { 0x9b26560a, SUC_lexicon_tags_22 },
    { 0xa2a6560b, SUC_lexicon_tags_6 },
    { 0x4be6560c, SUC_lexicon_tags_2 },
    { 0x32a6560d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7806560f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec65612, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83265618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6006561f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17265621, SUC_lexicon_tags_7 },
    { 0xefc65622, SUC_lexicon_tags_27 },
    { 0xf4c65622, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8865625, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb465629, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34265632, SUC_lexicon_tags_104 },
    { 0x9ce65633, SUC_lexicon_tags_15 },
    { 0xe5865634, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a6563a, SUC_lexicon_tags_47 },
    { 0xe6a6563b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5826563f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x58a65641, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae065645, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7465647, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2865649, SUC_lexicon_tags_2 },
    { 0x96a6564a, SUC_lexicon_tags_87 },
    { 0x1b06564a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf426564e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac65652, SUC_lexicon_tags_74 },
    { 0x5de65653, SUC_lexicon_tags_14 },
    { 0x84665654, SUC_lexicon_tags_8 },
    { 0x51e65655, SUC_lexicon_tags_63 },
    { 0x29865656, SUC_lexicon_tags_63 },
    { 0x5f465654, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ea65659, SUC_lexicon_tags_37 },
    { 0x3f66565a, SUC_lexicon_tags_11 },
    { 0xfda6565b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca65665, SUC_lexicon_tags_984 },
    { 0x12065666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c465668, SUC_lexicon_tags_9 },
    { 0x6665668, SUC_lexicon_tags_15 },
    { 0x8e6566a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad665671, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbce65673, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x54665675, SUC_lexicon_tags_2 },
    { 0x83a65676, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a65681, SUC_lexicon_tags_60 },
    { 0xe0e65682, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f665684, SUC_lexicon_tags_4 },
    { 0x13065684, SUC_lexicon_tags_11 },
    { 0xda465686, SUC_lexicon_tags_27 },
    { 0x4fa65687, SUC_lexicon_tags_17 },
    { 0x5a265688, SUC_lexicon_tags_28 },
    { 0x16e65689, SUC_lexicon_tags_49 },
    { 0xb2465689, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b46568c, SUC_lexicon_tags_19 },
    { 0xbba6568c, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4865690, SUC_lexicon_tags_29 },
    { 0x39a65691, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1265695, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b865698, SUC_lexicon_tags_49 },
    { 0x50a65699, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3266569d, SUC_lexicon_tags_9 },
    { 0xcb26569e, SUC_lexicon_tags_1 },
    { 0xdd06569e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x452656a1, SUC_lexicon_tags_61 },
    { 0x2a656a2, SUC_lexicon_tags_26 },
    { 0xf68656a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8656a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0656ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a8656b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6656b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e656b9, SUC_lexicon_tags_23 },
    { 0x21a656b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad0656bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84656bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca656c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x352656c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738656d1, SUC_lexicon_tags_24 },
    { 0xe38656d2, SUC_lexicon_tags_2 },
    { 0x9a2656d3, SUC_lexicon_tags_61 },
    { 0xc656d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x552656d6, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a656e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbde656e8, SUC_lexicon_tags_84 },
    { 0x13a656e9, SUC_lexicon_tags_15 },
    { 0x15e656ea, SUC_lexicon_tags_1 },
    { 0x6b2656eb, SUC_lexicon_tags_14 },
    { 0xe00656ec, SUC_lexicon_tags_15 },
    { 0xa14656ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8656f2, SUC_lexicon_tags_1 },
    { 0x69e656f3, SUC_lexicon_tags_7 },
    { 0x152656f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x308656f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06656fa, SUC_lexicon_tags_37 },
    { 0x732656fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4656fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a465701, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c65709, SUC_lexicon_tags_1 },
    { 0x5c65709, SUC_lexicon_tags_37 },
    { 0xd8465709, SUC_lexicon_tags_12 },
    { 0x9f46570c, SUC_lexicon_tags_38 },
    { 0x4466570d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c65712, SUC_lexicon_tags_25 },
    { 0x5f465713, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca65717, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd86571a, SUC_lexicon_tags_126 },
    { 0xea6571b, SUC_lexicon_tags_33 },
    { 0x1066571c, SUC_lexicon_tags_15 },
    { 0x31e6571c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x76465720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e6572a, SUC_lexicon_tags_21 },
    { 0x53c6572b, SUC_lexicon_tags_113 },
    { 0x5086572c, SUC_lexicon_tags_21 },
    { 0x5986572d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca65731, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1ce65733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a465735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee65739, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9806573b, SUC_lexicon_tags_6 },
    { 0xd146573c, SUC_lexicon_tags_19 },
    { 0x9c66573b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51065740, SUC_lexicon_tags_25 },
    { 0xec65741, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59665749, SUC_lexicon_tags_9 },
    { 0x4826574a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c86574f, SUC_lexicon_tags_82 },
    { 0x97e6574f, SUC_lexicon_tags_25 },
    { 0x54665751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf865756, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2465759, SUC_lexicon_tags_46 },
    { 0x83865759, SUC_lexicon_tags_60 },
    { 0x9b46575b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a6575e, SUC_lexicon_tags_10 },
    { 0x5106575e, SUC_lexicon_tags_25 },
    { 0x4a265760, SUC_lexicon_tags_11 },
    { 0x3f86575f, SUC_lexicon_tags_23 },
    { 0xdb665760, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xca065765, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa686576b, SUC_lexicon_tags_19 },
    { 0x77c6576c, SUC_lexicon_tags_38 },
    { 0xfa26576d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a65770, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a265773, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x46465775, SUC_lexicon_tags_11 },
    { 0xbca65776, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0865779, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8306577b, SUC_lexicon_tags_16 },
    { 0x2a06577b, SUC_lexicon_tags_61 },
    { 0x1166577d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa65780, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe5c65782, SUC_lexicon_tags_38 },
    { 0x85c65783, SUC_lexicon_tags_37 },
    { 0xcc65783, SUC_lexicon_tags_1 },
    { 0xc3865782, SUC_lexicon_tags_7 },
    { 0xe065786, SUC_lexicon_tags_25 },
    { 0xac065786, SUC_lexicon_tags_11 },
    { 0xfac65786, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b46578a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf66578d, SUC_lexicon_tags_23 },
    { 0x7d46578e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c65792, SUC_lexicon_tags_6 },
    { 0x71c65793, SUC_lexicon_tags_87 },
    { 0xb8065794, SUC_lexicon_tags_16 },
    { 0xcee65792, SUC_lexicon_tags_17 },
    { 0xca865796, SUC_lexicon_tags_37 },
    { 0xa1265792, SUC_lexicon_tags_2 },
    { 0x23c65798, SUC_lexicon_tags_30 },
    { 0xf3465798, SUC_lexicon_tags_11 },
    { 0xaa65793, SUC_lexicon_tags_12 },
    { 0x2966579b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf186579d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe657a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c657a6, SUC_lexicon_tags_2 },
    { 0x60a657a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x704657a9, SUC_lexicon_tags_15 },
    { 0x494657aa, SUC_lexicon_tags_23 },
    { 0x496657ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0657af, SUC_lexicon_tags_27 },
    { 0x224657b0, SUC_lexicon_tags_33 },
    { 0xbf6657b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6657b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf12657ba, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xcc2657bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2657c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfb6657c2, SUC_lexicon_tags_28 },
    { 0x28c657c2, SUC_lexicon_tags_23 },
    { 0xbf6657c4, SUC_lexicon_tags_37 },
    { 0x680657c4, SUC_lexicon_tags_37 },
    { 0xf26657c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ac657c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e657ce, SUC_lexicon_tags_49 },
    { 0x2e4657cf, SUC_lexicon_tags_10 },
    { 0xa8e657cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee657d5, SUC_lexicon_tags_17 },
    { 0x348657d5, SUC_lexicon_tags_25 },
    { 0xe6657d7, SUC_lexicon_tags_26 },
    { 0xf0657d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b8657da, SUC_lexicon_tags_15 },
    { 0x498657da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x478657dd, SUC_lexicon_tags_8 },
    { 0x9f8657dd, SUC_lexicon_tags_15 },
    { 0x58e657de, SUC_lexicon_tags_79 },
    { 0xcee657e0, SUC_lexicon_tags_17 },
    { 0xc7e657e0, SUC_lexicon_tags_15 },
    { 0xf62657e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c657e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b8657e9, SUC_lexicon_tags_16 },
    { 0x84e657e9, SUC_lexicon_tags_10 },
    { 0x672657eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52657f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45665808, SUC_lexicon_tags_0 },
    { 0xbb865809, SUC_lexicon_tags_45 },
    { 0x7d265809, SUC_lexicon_tags_21 },
    { 0x8d66580b, SUC_lexicon_tags_37 },
    { 0x16c6580c, SUC_lexicon_tags_26 },
    { 0x2e86580c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x62a6580f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea65812, SUC_lexicon_tags_11 },
    { 0xc2865813, SUC_lexicon_tags_63 },
    { 0x82a65814, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35465818, SUC_lexicon_tags_27 },
    { 0xa9865819, SUC_lexicon_tags_25 },
    { 0xc286581a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7946581d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1465824, SUC_lexicon_tags_2 },
    { 0x4b265825, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb526582a, SUC_lexicon_tags_35 },
    { 0x4646582b, SUC_lexicon_tags_1 },
    { 0xd46582a, SUC_lexicon_tags_12 },
    { 0x8986582d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5065830, SUC_lexicon_tags_6 },
    { 0x4de65830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5865833, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37265836, SUC_lexicon_tags_11 },
    { 0x76e65837, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc946583d, SUC_lexicon_tags_47 },
    { 0x53c6583e, SUC_lexicon_tags_16 },
    { 0x6786583f, SUC_lexicon_tags_21 },
    { 0xc0265840, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de65845, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc65848, SUC_lexicon_tags_11 },
    { 0x46e65848, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3546584b, SUC_lexicon_tags_32 },
    { 0xe466584c, SUC_lexicon_tags_0 },
    { 0xba66584d, SUC_lexicon_tags_60 },
    { 0x2586584d, SUC_lexicon_tags_19 },
    { 0xfc46584f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c65853, SUC_lexicon_tags_61 },
    { 0xab665854, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e65859, SUC_lexicon_tags_65 },
    { 0x6ba6585a, SUC_lexicon_tags_37 },
    { 0xf66585b, SUC_lexicon_tags_31 },
    { 0xe106585a, SUC_lexicon_tags_17 },
    { 0x7786585d, SUC_lexicon_tags_14 },
    { 0xbc46585e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10665863, SUC_lexicon_tags_43 },
    { 0x9fa65864, SUC_lexicon_tags_1 },
    { 0xfc065865, SUC_lexicon_tags_10 },
    { 0x51065866, SUC_lexicon_tags_14 },
    { 0x54a65867, SUC_lexicon_tags_37 },
    { 0x7a65864, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9146586c, SUC_lexicon_tags_2 },
    { 0xca26586d, SUC_lexicon_tags_27 },
    { 0x54e6586e, SUC_lexicon_tags_84 },
    { 0x1ec6586f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3865871, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c465873, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd8e65875, SUC_lexicon_tags_38 },
    { 0x10c65876, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de65879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd006587f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a65885, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44465887, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c86588d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c465891, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x15065893, SUC_lexicon_tags_65 },
    { 0x71265894, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0665897, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e465899, SUC_lexicon_tags_21 },
    { 0x2fa6589a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9ce6589c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf26589e, SUC_lexicon_tags_31 },
    { 0xe686589e, SUC_lexicon_tags_11 },
    { 0xd64658a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a2658a2, SUC_lexicon_tags_15 },
    { 0x74e658a3, SUC_lexicon_tags_65 },
    { 0x5f6658a3, SUC_lexicon_tags_47 },
    { 0x84658a3, SUC_lexicon_tags_27 },
    { 0xfc8658a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1c658a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e658af, SUC_lexicon_tags_14 },
    { 0x1c2658b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe658b3, SUC_lexicon_tags_15 },
    { 0xbf8658b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa04658b6, SUC_lexicon_tags_30 },
    { 0x2ae658b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544658bb, SUC_lexicon_tags_32 },
    { 0xb18658bc, SUC_lexicon_tags_11 },
    { 0xc46658bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf28658bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2658c2, SUC_lexicon_tags_16 },
    { 0xc16658c3, SUC_lexicon_tags_11 },
    { 0x664658c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a8658c6, SUC_lexicon_tags_0 },
    { 0x802658c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6658cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2658d0, SUC_lexicon_tags_1 },
    { 0x6e4658d1, SUC_lexicon_tags_37 },
    { 0x41c658d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c658db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66658de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x306658e0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x98e658e2, SUC_lexicon_tags_60 },
    { 0x124658e3, SUC_lexicon_tags_140 },
    { 0xdaa658e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x772658e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe58658e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa658f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d2658f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558658f9, SUC_lexicon_tags_43 },
    { 0x8c2658fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x828658fc, SUC_lexicon_tags_25 },
    { 0xa62658fc, SUC_lexicon_tags_12 },
    { 0x3d0658fc, SUC_lexicon_tags_12 },
    { 0x83a658ff, SUC_lexicon_tags_14 },
    { 0xf0e658ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be65903, SUC_lexicon_tags_7 },
    { 0x8d065904, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x73465907, SUC_lexicon_tags_0 },
    { 0x3ac65908, SUC_lexicon_tags_168 },
    { 0x68865908, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2826590b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33465910, SUC_lexicon_tags_1 },
    { 0xc5a65911, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64265918, SUC_lexicon_tags_5 },
    { 0xcd065918, SUC_lexicon_tags_65 },
    { 0xf1265918, SUC_lexicon_tags_31 },
    { 0xb486591b, SUC_lexicon_tags_82 },
    { 0x7da65919, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcc46591e, SUC_lexicon_tags_12 },
    { 0xe786591f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd865928, SUC_lexicon_tags_82 },
    { 0x22065928, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806592d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe65934, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74465936, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4966593a, SUC_lexicon_tags_647 },
    { 0x1686593b, SUC_lexicon_tags_28 },
    { 0x6246593c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdee6593e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50065944, SUC_lexicon_tags_29 },
    { 0xd2a65944, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be65948, SUC_lexicon_tags_65 },
    { 0xf6a65948, SUC_lexicon_tags_48 },
    { 0x12665949, SUC_lexicon_tags_9 },
    { 0x666594a, SUC_lexicon_tags_1 },
    { 0x14a6594b, SUC_lexicon_tags_14 },
    { 0x7426594d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa665950, SUC_lexicon_tags_47 },
    { 0xca465951, SUC_lexicon_tags_55 },
    { 0x265952, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9265954, SUC_lexicon_tags_15 },
    { 0x8fc65955, SUC_lexicon_tags_47 },
    { 0xde665956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8465958, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae65960, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a465964, SUC_lexicon_tags_1 },
    { 0xdd865964, SUC_lexicon_tags_47 },
    { 0x54465965, SUC_lexicon_tags_23 },
    { 0x6e65967, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec6596a, SUC_lexicon_tags_13 },
    { 0x5bc6596b, SUC_lexicon_tags_21 },
    { 0x5d26596c, SUC_lexicon_tags_49 },
    { 0xd646596a, SUC_lexicon_tags_8 },
    { 0xfb66596e, SUC_lexicon_tags_33 },
    { 0x2486596f, SUC_lexicon_tags_174 },
    { 0x7e46596e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62c65972, SUC_lexicon_tags_0 },
    { 0x68665973, SUC_lexicon_tags_17 },
    { 0x8ac65974, SUC_lexicon_tags_2 },
    { 0x1fa65975, SUC_lexicon_tags_440 },
    { 0xbe865975, SUC_lexicon_tags_11 },
    { 0x58065976, SUC_lexicon_tags_25 },
    { 0x1c065976, SUC_lexicon_tags_27 },
    { 0xe5c65979, SUC_lexicon_tags_30 },
    { 0x2406597a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a6597f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c65983, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2465986, SUC_lexicon_tags_14 },
    { 0xd5665986, SUC_lexicon_tags_38 },
    { 0xf5865988, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1406598f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8665991, SUC_lexicon_tags_25 },
    { 0xa7465992, SUC_lexicon_tags_25 },
    { 0xe2465993, SUC_lexicon_tags_15 },
    { 0xe7e65994, SUC_lexicon_tags_65 },
    { 0x35e65995, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20865997, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a46599c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47c6599e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0659a7, SUC_lexicon_tags_79 },
    { 0x738659a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8c659aa, SUC_lexicon_tags_27 },
    { 0x12e659ab, SUC_lexicon_tags_6 },
    { 0xf3e659ab, SUC_lexicon_tags_0 },
    { 0xef2659ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53c659af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x564659b2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140659b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d8659b8, SUC_lexicon_tags_37 },
    { 0xcf6659b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c659c1, SUC_lexicon_tags_26 },
    { 0x518659c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9cc659c4, SUC_lexicon_tags_2 },
    { 0x1b8659c5, SUC_lexicon_tags_32 },
    { 0x126659c6, SUC_lexicon_tags_37 },
    { 0xe0659c7, SUC_lexicon_tags_87 },
    { 0xe50659c5, SUC_lexicon_tags_27 },
    { 0x7de659c9, SUC_lexicon_tags_26 },
    { 0x126659ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c659cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe06659cf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea659d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a659da, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9f2659dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84659e1, SUC_lexicon_tags_11 },
    { 0xf8e659e2, SUC_lexicon_tags_12 },
    { 0x286659e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be659e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x490659e8, SUC_lexicon_tags_38 },
    { 0xa88659e9, SUC_lexicon_tags_60 },
    { 0x224659e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4659ed, SUC_lexicon_tags_14 },
    { 0x5e8659ee, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be659f1, SUC_lexicon_tags_65 },
    { 0xa52659f2, SUC_lexicon_tags_15 },
    { 0x706659f3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x44a659f5, SUC_lexicon_tags_99 },
    { 0xf94659f6, SUC_lexicon_tags_11 },
    { 0xcbe659f6, SUC_lexicon_tags_45 },
    { 0x2c8659f8, SUC_lexicon_tags_2 },
    { 0x8c2659f9, SUC_lexicon_tags_15 },
    { 0x51a659fa, SUC_lexicon_tags_8 },
    { 0x5be659fa, SUC_lexicon_tags_13 },
    { 0x96e659fc, SUC_lexicon_tags_25 },
    { 0x9aa659fc, SUC_lexicon_tags_2 },
    { 0xed4659fd, SUC_lexicon_tags_12 },
    { 0xed0659fc, SUC_lexicon_tags_64 },
    { 0x98a659ff, SUC_lexicon_tags_45 },
    { 0x8ec65a01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c65a04, SUC_lexicon_tags_15 },
    { 0xf4265a05, SUC_lexicon_tags_23 },
    { 0x15265a06, SUC_lexicon_tags_43 },
    { 0x4ec65a07, SUC_lexicon_tags_68 },
    { 0xa7065a06, SUC_lexicon_tags_12 },
    { 0xb5065a09, SUC_lexicon_tags_6 },
    { 0x80465a06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xca865a0c, SUC_lexicon_tags_25 },
    { 0xcf865a0c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a65a12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b265a17, SUC_lexicon_tags_15 },
    { 0x11865a17, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b665a1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d465a21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34e65a23, SUC_lexicon_tags_1 },
    { 0x1e065a23, SUC_lexicon_tags_57 },
    { 0xb8c65a25, SUC_lexicon_tags_40 },
    { 0x49065a23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8865a28, SUC_lexicon_tags_28 },
    { 0x58865a29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78265a31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99865a39, SUC_lexicon_tags_25 },
    { 0x70065a3a, SUC_lexicon_tags_26 },
    { 0xc0665a3a, SUC_lexicon_tags_124 },
    { 0xfdc65a3c, SUC_lexicon_tags_2 },
    { 0x9a065a3d, SUC_lexicon_tags_11 },
    { 0x23665a3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce65a44, SUC_lexicon_tags_65 },
    { 0x23665a45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31665a48, SUC_lexicon_tags_27 },
    { 0xdfc65a49, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1865a4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55265a50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e65a57, SUC_lexicon_tags_0 },
    { 0x52c65a58, SUC_lexicon_tags_11 },
    { 0xeec65a58, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec65a63, SUC_lexicon_tags_28 },
    { 0xb3465a64, SUC_lexicon_tags_14 },
    { 0x37265a65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf7065a67, SUC_lexicon_tags_37 },
    { 0x9d865a67, SUC_lexicon_tags_1 },
    { 0xfa065a69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe865a6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fe65a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcce65a70, SUC_lexicon_tags_14 },
    { 0xa7465a70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a65a74, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3865a77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x60665a7a, SUC_lexicon_tags_33 },
    { 0x18665a7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2c65a7d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86265a85, SUC_lexicon_tags_65 },
    { 0x70065a86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da65a90, SUC_lexicon_tags_47 },
    { 0x1bc65a91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc65a95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac065a97, SUC_lexicon_tags_23 },
    { 0x55a65a98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86c65a9a, SUC_lexicon_tags_33 },
    { 0x25c65a9b, SUC_lexicon_tags_47 },
    { 0x32a65a9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2465a9e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a865aa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80265aa8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e865aaf, SUC_lexicon_tags_14 },
    { 0xede65aaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fa65ab2, SUC_lexicon_tags_12 },
    { 0x26865ab2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49865abb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd4a65abd, SUC_lexicon_tags_37 },
    { 0x5f665abe, SUC_lexicon_tags_1 },
    { 0x7e665abf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93e65ac1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc65ac8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2065acb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26865ad2, SUC_lexicon_tags_17 },
    { 0xc5465ad3, SUC_lexicon_tags_32 },
    { 0x77e65ad3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da65adc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2de65ade, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d265ae1, SUC_lexicon_tags_12 },
    { 0x88c65ae2, SUC_lexicon_tags_12 },
    { 0xd8465ae1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd865ae8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b665aeb, SUC_lexicon_tags_23 },
    { 0xf5065aeb, SUC_lexicon_tags_14 },
    { 0x99865aeb, SUC_lexicon_tags_23 },
    { 0xba865aee, SUC_lexicon_tags_1 },
    { 0xa4865aef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa665af4, SUC_lexicon_tags_45 },
    { 0x29865af5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8c65af7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c65afa, SUC_lexicon_tags_31 },
    { 0x49265afb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa65aff, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8a465b01, SUC_lexicon_tags_14 },
    { 0x97465b01, SUC_lexicon_tags_8 },
    { 0x5d265b03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69865b06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba65b09, SUC_lexicon_tags_2 },
    { 0x9c665b0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc65b10, SUC_lexicon_tags_23 },
    { 0xcd265b11, SUC_lexicon_tags_10 },
    { 0x1d665b11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a65b15, SUC_lexicon_tags_9 },
    { 0x3b465b15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5465b18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa65b1d, SUC_lexicon_tags_43 },
    { 0x64065b1e, SUC_lexicon_tags_37 },
    { 0x82265b1f, SUC_lexicon_tags_2 },
    { 0x92265b1f, SUC_lexicon_tags_0 },
    { 0xfee65b1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a65b25, SUC_lexicon_tags_63 },
    { 0xf1865b25, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f865b2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44265b2c, SUC_lexicon_tags_15 },
    { 0x80e65b2d, SUC_lexicon_tags_17 },
    { 0xf7465b2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f665b30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d665b32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a65b35, SUC_lexicon_tags_11 },
    { 0xa0265b36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78065b3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99065b43, SUC_lexicon_tags_15 },
    { 0xc5e65b43, SUC_lexicon_tags_17 },
    { 0x1cc65b43, SUC_lexicon_tags_66 },
    { 0x6f065b46, SUC_lexicon_tags_26 },
    { 0x4aa65b47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3065b4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55c65b4c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc665b50, SUC_lexicon_tags_14 },
    { 0x53065b51, SUC_lexicon_tags_16 },
    { 0x97e65b52, SUC_lexicon_tags_7 },
    { 0xe7e65b53, SUC_lexicon_tags_37 },
    { 0x7ee65b53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfbc65b56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x38865b59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e65b5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae65b62, SUC_lexicon_tags_16 },
    { 0xf5865b62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f665b65, SUC_lexicon_tags_36 },
    { 0x13a65b66, SUC_lexicon_tags_15 },
    { 0x92665b67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46c65b69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55265b6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf0c65b6d, SUC_lexicon_tags_29 },
    { 0xcd665b6e, SUC_lexicon_tags_11 },
    { 0x6ec65b6e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3065b72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3e065b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc65b81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd9065b83, SUC_lexicon_tags_1 },
    { 0x36065b84, SUC_lexicon_tags_37 },
    { 0x4bc65b85, SUC_lexicon_tags_25 },
    { 0x2aa65b84, SUC_lexicon_tags_2 },
    { 0x88665b84, SUC_lexicon_tags_28 },
    { 0xa3c65b83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15865b92, SUC_lexicon_tags_17 },
    { 0xaa865b93, SUC_lexicon_tags_43 },
    { 0x5c465b92, SUC_lexicon_tags_27 },
    { 0xcae65b94, SUC_lexicon_tags_61 },
    { 0x1a265b96, SUC_lexicon_tags_0 },
    { 0x76a65b96, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0065b9a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c65b9f, SUC_lexicon_tags_80 },
    { 0x52865ba0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb865ba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc65ba9, SUC_lexicon_tags_49 },
    { 0xa1665baa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa465bad, SUC_lexicon_tags_63 },
    { 0xe2a65bad, SUC_lexicon_tags_25 },
    { 0x8c65baf, SUC_lexicon_tags_87 },
    { 0x51e65bb0, SUC_lexicon_tags_9 },
    { 0x99a65bb0, SUC_lexicon_tags_17 },
    { 0x73065baf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6e65bb4, SUC_lexicon_tags_12 },
    { 0xa6465bb5, SUC_lexicon_tags_9 },
    { 0x48e65bb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba065bbb, SUC_lexicon_tags_37 },
    { 0x2bc65bbb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x98065bbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a65bc2, SUC_lexicon_tags_37 },
    { 0x9d265bc3, SUC_lexicon_tags_19 },
    { 0xf5865bc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8265bc8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe265bcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11665bcf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf065bd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9465bd4, SUC_lexicon_tags_87 },
    { 0x7f865bd5, SUC_lexicon_tags_27 },
    { 0x2a65bd6, SUC_lexicon_tags_63 },
    { 0x23c65bd6, SUC_lexicon_tags_7 },
    { 0xfa265bd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a65bdb, SUC_lexicon_tags_1 },
    { 0xe4665bdb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5065be2, SUC_lexicon_tags_37 },
    { 0x7b665be3, SUC_lexicon_tags_15 },
    { 0xa5465be3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e465be6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d065be8, SUC_lexicon_tags_17 },
    { 0x19c65be9, SUC_lexicon_tags_134 },
    { 0x27065bea, SUC_lexicon_tags_49 },
    { 0xaae65be9, SUC_lexicon_tags_37 },
    { 0xc2665bec, SUC_lexicon_tags_47 },
    { 0xa0065bed, SUC_lexicon_tags_208 },
    { 0xdb265bed, SUC_lexicon_tags_43 },
    { 0x5f265bec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53a65bf1, SUC_lexicon_tags_28 },
    { 0xfe465bf2, SUC_lexicon_tags_17 },
    { 0xf6a65bf2, SUC_lexicon_tags_45 },
    { 0x72a65bf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44a65bf6, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0x65065bf8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x22265bfa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34265bfe, SUC_lexicon_tags_6 },
    { 0x37465bfe, SUC_lexicon_tags_49 },
    { 0x8f065c00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8865c02, SUC_lexicon_tags_71 },
    { 0x6de65c03, SUC_lexicon_tags_14 },
    { 0xca265c04, SUC_lexicon_tags_10 },
    { 0xd1465c04, SUC_lexicon_tags_84 },
    { 0x9ec65c03, SUC_lexicon_tags_37 },
    { 0x2cc65c07, SUC_lexicon_tags_38 },
    { 0x7d865c05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed865c0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe65c0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7865c0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c665c12, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc5465c14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9265c16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb665c19, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c65c1d, SUC_lexicon_tags_45 },
    { 0x5f065c1e, SUC_lexicon_tags_43 },
    { 0xb7265c1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14065c25, SUC_lexicon_tags_55 },
    { 0x25465c25, SUC_lexicon_tags_11 },
    { 0x31865c25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39065c29, SUC_lexicon_tags_21 },
    { 0x59865c29, SUC_lexicon_tags_27 },
    { 0x4b665c29, SUC_lexicon_tags_17 },
    { 0x16865c2c, SUC_lexicon_tags_21 },
    { 0xc1665c29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19465c31, SUC_lexicon_tags_17 },
    { 0x80865c32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0e65c34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e65c3a, SUC_lexicon_tags_11 },
    { 0x55065c3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8465c3f, SUC_lexicon_tags_25 },
    { 0x4e65c40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1265c42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c65c47, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9465c4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34865c4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56865c54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46e65c56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c465c5b, SUC_lexicon_tags_11 },
    { 0x6265c5c, SUC_lexicon_tags_15 },
    { 0xe7a65c5c, SUC_lexicon_tags_0 },
    { 0xab865c5e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6865c63, SUC_lexicon_tags_26 },
    { 0x78465c64, SUC_lexicon_tags_63 },
    { 0x45465c65, SUC_lexicon_tags_6 },
    { 0xa2865c65, SUC_lexicon_tags_37 },
    { 0x3c665c64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31865c6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9865c71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63465c76, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x24665c78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40265c7a, SUC_lexicon_tags_33 },
    { 0x50065c7b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4a065c7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c265c84, SUC_lexicon_tags_61 },
    { 0x99c65c85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39265c8a, SUC_lexicon_tags_17 },
    { 0xbde65c8b, SUC_lexicon_tags_63 },
    { 0x2da65c8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73465c90, SUC_lexicon_tags_140 },
    { 0x91665c91, SUC_lexicon_tags_5 },
    { 0x72665c92, SUC_lexicon_tags_15 },
    { 0xc2465c93, SUC_lexicon_tags_11 },
    { 0x9e265c93, SUC_lexicon_tags_37 },
    { 0x72e65c92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9065c9a, SUC_lexicon_tags_12 },
    { 0xa7065c9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75465ca0, SUC_lexicon_tags_51 },
    { 0xf0265ca0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0465ca3, SUC_lexicon_tags_16 },
    { 0xcee65ca3, SUC_lexicon_tags_14 },
    { 0xe3465ca4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca65cad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e65cb1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4465cb5, SUC_lexicon_tags_2 },
    { 0xd9265cb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1665cb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca265cbc, SUC_lexicon_tags_129 },
    { 0x28865cbd, SUC_lexicon_tags_25 },
    { 0x76865cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b265cc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x21865cc2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc65ccc, SUC_lexicon_tags_21 },
    { 0xb4465ccd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x84e65ccf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xee465cd1, SUC_lexicon_tags_11 },
    { 0xbb465cd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93465cd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf065cd7, SUC_lexicon_tags_25 },
    { 0x48a65cd8, SUC_lexicon_tags_37 },
    { 0x60c65cd9, SUC_lexicon_tags_66 },
    { 0xae65cd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c65ce0, SUC_lexicon_tags_38 },
    { 0x64e65ce0, SUC_lexicon_tags_37 },
    { 0x68265ce2, SUC_lexicon_tags_16 },
    { 0xf7065ce2, SUC_lexicon_tags_38 },
    { 0xbd665ce3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc65ce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6265cea, SUC_lexicon_tags_0 },
    { 0xeec65ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10665cef, SUC_lexicon_tags_2 },
    { 0xf8c65cef, SUC_lexicon_tags_134 },
    { 0x9c665cef, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc65cf4, SUC_lexicon_tags_26 },
    { 0xc2065cf5, SUC_lexicon_tags_10 },
    { 0x3be65cf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4465cf9, SUC_lexicon_tags_38 },
    { 0x52865cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb465cff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a465d01, SUC_lexicon_tags_116 },
    { 0xf0665d02, SUC_lexicon_tags_11 },
    { 0x76a65d03, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x20c65d05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8465d0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x21065d0c, SUC_lexicon_tags_33 },
    { 0x23665d0d, SUC_lexicon_tags_55 },
    { 0x72c65d0c, SUC_lexicon_tags_21 },
    { 0xe6665d0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae65d12, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c865d15, SUC_lexicon_tags_3 },
    { 0x60c65d16, SUC_lexicon_tags_9 },
    { 0x81a65d17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c865d19, SUC_lexicon_tags_6 },
    { 0x9ae65d1a, SUC_lexicon_tags_11 },
    { 0xd3065d1b, SUC_lexicon_tags_25 },
    { 0x58065d1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a65d22, SUC_lexicon_tags_7 },
    { 0xed465d23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x51665d26, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae65d29, SUC_lexicon_tags_5 },
    { 0x89265d2a, SUC_lexicon_tags_19 },
    { 0x4f865d2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30665d2d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22865d32, SUC_lexicon_tags_34 },
    { 0x9a665d33, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8cc65d35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a65d3c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x38065d3e, SUC_lexicon_tags_37 },
    { 0x89265d3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a65d43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac65d4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90665d51, SUC_lexicon_tags_11 },
    { 0xaa65d52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21065d58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7265d5b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95665d5f, SUC_lexicon_tags_7 },
    { 0x37e65d60, SUC_lexicon_tags_25 },
    { 0xfee65d60, SUC_lexicon_tags_65 },
    { 0xc6265d5f, SUC_lexicon_tags_0 },
    { 0x1f465d63, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xac265d66, SUC_lexicon_tags_1 },
    { 0xa9065d66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93465d6b, SUC_lexicon_tags_27 },
    { 0xc265d6b, SUC_lexicon_tags_12 },
    { 0xa9c65d6d, SUC_lexicon_tags_11 },
    { 0x9d665d6d, SUC_lexicon_tags_11 },
    { 0xaae65d6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4865d78, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4065d86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f065d88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0a65d8a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4665d8e, SUC_lexicon_tags_0 },
    { 0x56865d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb865d91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa665d93, SUC_lexicon_tags_43 },
    { 0x8c65d94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60465d98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f265d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94e65d9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59465d9f, SUC_lexicon_tags_31 },
    { 0x8e665da0, SUC_lexicon_tags_25 },
    { 0xd065d9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36665da3, SUC_lexicon_tags_15 },
    { 0xc8265da4, SUC_lexicon_tags_12 },
    { 0x75e65da5, SUC_lexicon_tags_38 },
    { 0x8ba65da6, SUC_lexicon_tags_23 },
    { 0x34c65da6, SUC_lexicon_tags_25 },
    { 0xe5065da8, SUC_lexicon_tags_17 },
    { 0x6a865da9, SUC_lexicon_tags_310 },
    { 0xacc65da9, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a65dad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a465db0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69065db4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3665db6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82c65db8, SUC_lexicon_tags_7 },
    { 0x98e65db9, SUC_lexicon_tags_87 },
    { 0x54465dba, SUC_lexicon_tags_26 },
    { 0x2ee65db9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91865dbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x55065dc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc665dc3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4265dc7, SUC_lexicon_tags_15 },
    { 0x66a65dc7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x59065dcb, SUC_lexicon_tags_25 },
    { 0x9a265dcc, SUC_lexicon_tags_15 },
    { 0x8ba65dcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e65dd0, SUC_lexicon_tags_48 },
    { 0x59e65dd0, SUC_lexicon_tags_7 },
    { 0x8fc65dd0, SUC_lexicon_tags_63 },
    { 0xa5c65dd3, SUC_lexicon_tags_14 },
    { 0xcd865dd4, SUC_lexicon_tags_17 },
    { 0x66c65dd4, SUC_lexicon_tags_32 },
    { 0x4ba65dd4, SUC_lexicon_tags_12 },
    { 0x8465dd1, SUC_lexicon_tags_17 },
    { 0x80a65dd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3465dda, SUC_lexicon_tags_45 },
    { 0x3ae65dda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2465ddf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41e65de1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcde65de3, SUC_lexicon_tags_7 },
    { 0x1ae65de4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3065de7, SUC_lexicon_tags_12 },
    { 0xba865de8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x15465deb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83465df0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15665df4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c065df6, SUC_lexicon_tags_7 },
    { 0x3dc65df7, SUC_lexicon_tags_17 },
    { 0x9c865df6, SUC_lexicon_tags_87 },
    { 0x42865df6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc65dff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd865e01, SUC_lexicon_tags_37 },
    { 0xe8665e02, SUC_lexicon_tags_31 },
    { 0x84865e03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a865e08, SUC_lexicon_tags_30 },
    { 0x92a65e08, SUC_lexicon_tags_54 },
    { 0x42265e0a, SUC_lexicon_tags_17 },
    { 0xb8265e0a, SUC_lexicon_tags_12 },
    { 0xa7e65e0c, SUC_lexicon_tags_68 },
    { 0x3ba65e0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7865e10, SUC_lexicon_tags_27 },
    { 0x6e065e11, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5265e14, SUC_lexicon_tags_17 },
    { 0x12a65e15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4865e1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60665e20, SUC_lexicon_tags_19 },
    { 0x6de65e21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa8865e23, SUC_lexicon_tags_0 },
    { 0x4e665e24, SUC_lexicon_tags_9 },
    { 0x13c65e25, SUC_lexicon_tags_29 },
    { 0x51465e23, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa5865e28, SUC_lexicon_tags_6 },
    { 0xcc465e29, SUC_lexicon_tags_7 },
    { 0x1fa65e29, SUC_lexicon_tags_49 },
    { 0x9d065e2b, SUC_lexicon_tags_1 },
    { 0xab865e2b, SUC_lexicon_tags_25 },
    { 0x1f265e29, SUC_lexicon_tags_48 },
    { 0xf1e65e2c, SUC_lexicon_tags_37 },
    { 0x28465e2f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x12065e31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace65e36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23465e3c, SUC_lexicon_tags_7 },
    { 0xaca65e3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58065e3f, SUC_lexicon_tags_43 },
    { 0xcc465e40, SUC_lexicon_tags_45 },
    { 0x40465e3f, SUC_lexicon_tags_26 },
    { 0xaf865e42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35265e46, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf0265e48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b265e4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9865e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda65e52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7465e55, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9865e5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3de65e5c, SUC_lexicon_tags_57 },
    { 0x50c65e5d, SUC_lexicon_tags_37 },
    { 0x46e65e5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a65e65, SUC_lexicon_tags_12 },
    { 0x6fa65e66, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x62c65e68, SUC_lexicon_tags_37 },
    { 0xb1865e69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdac65e6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56a65e6d, SUC_lexicon_tags_56 },
    { 0xdf465e6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca65e71, SUC_lexicon_tags_27 },
    { 0xc4865e71, SUC_lexicon_tags_79 },
    { 0x76465e71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x78465e75, SUC_lexicon_tags_32 },
    { 0xfe865e76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdaa65e78, SUC_lexicon_tags_33 },
    { 0x72e65e78, SUC_lexicon_tags_16 },
    { 0x68e65e78, SUC_lexicon_tags_11 },
    { 0xdd665e7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c65e7f, SUC_lexicon_tags_14 },
    { 0x6a65e80, SUC_lexicon_tags_38 },
    { 0x5a065e7f, SUC_lexicon_tags_30 },
    { 0xf7865e81, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x46665e85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5865e88, SUC_lexicon_tags_15 },
    { 0xd0665e89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93865e8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3065e91, SUC_lexicon_tags_17 },
    { 0xba865e91, SUC_lexicon_tags_28 },
    { 0x16c65e92, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94265e9d, SUC_lexicon_tags_17 },
    { 0xe265e9e, SUC_lexicon_tags_25 },
    { 0xc1465e9f, SUC_lexicon_tags_38 },
    { 0xc8665e9f, SUC_lexicon_tags_28 },
    { 0x27c65ea1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e465ea3, SUC_lexicon_tags_62 },
    { 0x4dc65ea3, SUC_lexicon_tags_33 },
    { 0x20865ea5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba65ea8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48065eab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d465ead, SUC_lexicon_tags_14 },
    { 0x4e665eae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c65eb1, SUC_lexicon_tags_7 },
    { 0xf8865eb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce65ebc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec65ec0, SUC_lexicon_tags_30 },
    { 0xbd865ec0, SUC_lexicon_tags_2 },
    { 0x7aa65ec2, SUC_lexicon_tags_25 },
    { 0x41465ec1, SUC_lexicon_tags_2 },
    { 0x21665ec4, SUC_lexicon_tags_49 },
    { 0xe4265ec5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8665ec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27865eca, SUC_lexicon_tags_7 },
    { 0xbca65eca, SUC_lexicon_tags_118 },
    { 0x36265ecc, SUC_lexicon_tags_15 },
    { 0x49265ecc, SUC_lexicon_tags_19 },
    { 0x4d465ece, SUC_lexicon_tags_63 },
    { 0x1fa65ecb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x84865ed2, SUC_lexicon_tags_3 },
    { 0x1a465ed3, SUC_lexicon_tags_37 },
    { 0x1fe65ed3, SUC_lexicon_tags_47 },
    { 0xb4665ed3, SUC_lexicon_tags_29 },
    { 0x49665ed6, SUC_lexicon_tags_69 },
    { 0x40265ed4, SUC_lexicon_tags_15 },
    { 0xe0265ed2, SUC_lexicon_tags_11 },
    { 0xe4265ed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71865edf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c65ee5, SUC_lexicon_tags_11 },
    { 0xe4c65ee6, SUC_lexicon_tags_63 },
    { 0x3bc65ee6, SUC_lexicon_tags_1 },
    { 0x2ca65ee8, SUC_lexicon_tags_2 },
    { 0x91c65ee9, SUC_lexicon_tags_23 },
    { 0x12c65eea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c865ef1, SUC_lexicon_tags_63 },
    { 0xde265ef2, SUC_lexicon_tags_12 },
    { 0x94665ef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b465ef6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x48065ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f865efc, SUC_lexicon_tags_19 },
    { 0xb8665efc, SUC_lexicon_tags_47 },
    { 0xb2465efe, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54665f02, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x12c65f04, SUC_lexicon_tags_31 },
    { 0x35065f05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4865f08, SUC_lexicon_tags_47 },
    { 0x28865f08, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a865f0d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xea465f0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e65f13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e265f1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a65f1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x94265f20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5065f22, SUC_lexicon_tags_14 },
    { 0xc7c65f23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a665f26, SUC_lexicon_tags_11 },
    { 0x7da65f27, SUC_lexicon_tags_4 },
    { 0x5e65f27, SUC_lexicon_tags_15 },
    { 0x67665f28, SUC_lexicon_tags_14 },
    { 0xb1465f2a, SUC_lexicon_tags_6 },
    { 0x55865f28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47065f30, SUC_lexicon_tags_7 },
    { 0x6b865f30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8865f39, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x65265f3b, SUC_lexicon_tags_17 },
    { 0x26a65f3b, SUC_lexicon_tags_48 },
    { 0x1fe65f3d, SUC_lexicon_tags_2 },
    { 0x7e665f3c, SUC_lexicon_tags_60 },
    { 0x11465f3b, SUC_lexicon_tags_7 },
    { 0xce065f40, SUC_lexicon_tags_8 },
    { 0xe1065f41, SUC_lexicon_tags_55 },
    { 0x6ea65f3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a65f4a, SUC_lexicon_tags_1 },
    { 0xf9865f4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d065f52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea265f56, SUC_lexicon_tags_17 },
    { 0xa5e65f57, SUC_lexicon_tags_0 },
    { 0xdb465f56, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x89665f5b, SUC_lexicon_tags_17 },
    { 0xd0e65f5b, SUC_lexicon_tags_49 },
    { 0xcfa65f5d, SUC_lexicon_tags_15 },
    { 0xd3e65f5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f465f60, SUC_lexicon_tags_8 },
    { 0x8c065f61, SUC_lexicon_tags_2 },
    { 0xef465f60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cc65f64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2265f67, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa65f6d, SUC_lexicon_tags_1 },
    { 0x25265f6d, SUC_lexicon_tags_1 },
    { 0xf6e65f6f, SUC_lexicon_tags_7 },
    { 0x7fc65f70, SUC_lexicon_tags_6 },
    { 0xb9a65f6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc65f74, SUC_lexicon_tags_6 },
    { 0x1ca65f75, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a65f7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9065f7e, SUC_lexicon_tags_17 },
    { 0xdc865f7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0065f82, SUC_lexicon_tags_45 },
    { 0xfb265f82, SUC_lexicon_tags_27 },
    { 0x34e65f83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0e65f86, SUC_lexicon_tags_17 },
    { 0x22465f87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74865f8f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf5e65f91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81865f97, SUC_lexicon_tags_19 },
    { 0x34665f98, SUC_lexicon_tags_0 },
    { 0xa265f98, SUC_lexicon_tags_63 },
    { 0xb0c65f97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea65fa1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c65fa9, SUC_lexicon_tags_17 },
    { 0x68665faa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a65fac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa65fb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a865fb5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb265fbd, SUC_lexicon_tags_23 },
    { 0x9cc65fbe, SUC_lexicon_tags_29 },
    { 0xba065fbf, SUC_lexicon_tags_28 },
    { 0x9cc65fbe, SUC_lexicon_tags_7 },
    { 0xc5a65fc1, SUC_lexicon_tags_26 },
    { 0x1b065fc0, SUC_lexicon_tags_5 },
    { 0x4d665fc1, SUC_lexicon_tags_11 },
    { 0xcde65fc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa865fc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a65fc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce865fcb, SUC_lexicon_tags_11 },
    { 0x4dc65fcb, SUC_lexicon_tags_2 },
    { 0xb1c65fcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6665fcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9de65fd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8465fd4, SUC_lexicon_tags_13 },
    { 0x99665fd5, SUC_lexicon_tags_60 },
    { 0x9be65fd6, SUC_lexicon_tags_37 },
    { 0x87665fd7, SUC_lexicon_tags_40 },
    { 0x83465fd8, SUC_lexicon_tags_10 },
    { 0x10265fd9, SUC_lexicon_tags_7 },
    { 0xa265fda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc65fdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e65fe2, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x3dc65fe4, SUC_lexicon_tags_63 },
    { 0x4f065fe5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0465fe7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c65fea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b265fee, SUC_lexicon_tags_25 },
    { 0xb065fef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c665ff1, SUC_lexicon_tags_37 },
    { 0xdc265ff2, SUC_lexicon_tags_9 },
    { 0xc0465ff2, SUC_lexicon_tags_6 },
    { 0xee65ff3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade65ff8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e65ffd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88865fff, SUC_lexicon_tags_37 },
    { 0x92066000, SUC_lexicon_tags_17 },
    { 0xd6666000, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a66004, SUC_lexicon_tags_4 },
    { 0xf3866005, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b266007, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886600e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4666016, SUC_lexicon_tags_38 },
    { 0xeb866016, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xdde66019, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d06601c, SUC_lexicon_tags_11 },
    { 0xae86601d, SUC_lexicon_tags_9 },
    { 0x9466601e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe666023, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a66026, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc5c66028, SUC_lexicon_tags_0 },
    { 0x4ee66029, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13266031, SUC_lexicon_tags_30 },
    { 0x94466031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8066034, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50666036, SUC_lexicon_tags_38 },
    { 0x60666036, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x61e66039, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd6e6603b, SUC_lexicon_tags_2 },
    { 0x14c6603c, SUC_lexicon_tags_48 },
    { 0xed06603d, SUC_lexicon_tags_7 },
    { 0x8986603b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c66043, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac66046, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a6604c, SUC_lexicon_tags_63 },
    { 0x2446604d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e66050, SUC_lexicon_tags_45 },
    { 0xc6066051, SUC_lexicon_tags_28 },
    { 0x38a66052, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc266056, SUC_lexicon_tags_17 },
    { 0x9c866057, SUC_lexicon_tags_6 },
    { 0x25466056, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b066061, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1866063, SUC_lexicon_tags_0 },
    { 0xdf866064, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1a066066, SUC_lexicon_tags_47 },
    { 0x71466067, SUC_lexicon_tags_37 },
    { 0xa0e66068, SUC_lexicon_tags_2 },
    { 0x58666067, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4466073, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47c66075, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe206607a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd06607d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51266082, SUC_lexicon_tags_40 },
    { 0xfb066083, SUC_lexicon_tags_2 },
    { 0x66666083, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea6608a, SUC_lexicon_tags_37 },
    { 0x9f46608b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xfa06608d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7506608f, SUC_lexicon_tags_25 },
    { 0x3f666090, SUC_lexicon_tags_7 },
    { 0xc5c66091, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e866094, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdc66096, SUC_lexicon_tags_7 },
    { 0x80266097, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae6609a, SUC_lexicon_tags_23 },
    { 0xe9e6609b, SUC_lexicon_tags_25 },
    { 0xd266609a, SUC_lexicon_tags_6 },
    { 0x50c6609c, SUC_lexicon_tags_63 },
    { 0x6226609e, SUC_lexicon_tags_38 },
    { 0x88e6609c, SUC_lexicon_tags_17 },
    { 0x162660a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4660a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6660a8, SUC_lexicon_tags_1 },
    { 0xb40660a9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804660af, SUC_lexicon_tags_34 },
    { 0x3f6660b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a660b3, SUC_lexicon_tags_7 },
    { 0x73e660b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x420660b6, SUC_lexicon_tags_15 },
    { 0xf60660b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3a2660b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcce660bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e660be, SUC_lexicon_tags_7 },
    { 0x790660be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2660c2, SUC_lexicon_tags_60 },
    { 0x8f2660c3, SUC_lexicon_tags_12 },
    { 0x172660c4, SUC_lexicon_tags_0 },
    { 0x4e660c3, SUC_lexicon_tags_84 },
    { 0xfc660c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe5a660c8, SUC_lexicon_tags_9 },
    { 0xcd6660c9, SUC_lexicon_tags_12 },
    { 0x680660ca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848660ce, SUC_lexicon_tags_47 },
    { 0xfe8660ce, SUC_lexicon_tags_77 },
    { 0x4e6660d0, SUC_lexicon_tags_12 },
    { 0x938660d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972660d5, SUC_lexicon_tags_32 },
    { 0x1cc660d6, SUC_lexicon_tags_11 },
    { 0xf84660d7, SUC_lexicon_tags_35 },
    { 0xd6c660d7, SUC_lexicon_tags_15 },
    { 0xcea660d9, SUC_lexicon_tags_4 },
    { 0x44e660da, SUC_lexicon_tags_2 },
    { 0xd0c660da, SUC_lexicon_tags_25 },
    { 0x34a660dc, SUC_lexicon_tags_63 },
    { 0xe46660d9, SUC_lexicon_tags_7 },
    { 0x796660de, SUC_lexicon_tags_39 },
    { 0x604660df, SUC_lexicon_tags_38 },
    { 0xb96660e0, SUC_lexicon_tags_15 },
    { 0x6c0660df, SUC_lexicon_tags_23 },
    { 0x666660d7, SUC_lexicon_tags_436 },
    { 0x710660df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532660ef, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcdc660f1, SUC_lexicon_tags_49 },
    { 0x756660f1, SUC_lexicon_tags_21 },
    { 0x2e8660f1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa660f6, SUC_lexicon_tags_11 },
    { 0x54c660f7, SUC_lexicon_tags_32 },
    { 0x7f6660f8, SUC_lexicon_tags_26 },
    { 0x74a660f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86660fd, SUC_lexicon_tags_9 },
    { 0x412660fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9186610a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x70a6610c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d266110, SUC_lexicon_tags_25 },
    { 0xaf866111, SUC_lexicon_tags_26 },
    { 0x6c666112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81466114, SUC_lexicon_tags_39 },
    { 0x8a466115, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e66119, SUC_lexicon_tags_55 },
    { 0x59c6611a, SUC_lexicon_tags_6 },
    { 0xc66119, SUC_lexicon_tags_77 },
    { 0xabe6611c, SUC_lexicon_tags_66 },
    { 0x8b86611d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa6a6611f, SUC_lexicon_tags_7 },
    { 0xd5866120, SUC_lexicon_tags_9 },
    { 0xb8066121, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b066123, SUC_lexicon_tags_1 },
    { 0x50666124, SUC_lexicon_tags_31 },
    { 0xb3466124, SUC_lexicon_tags_17 },
    { 0x7e066125, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8266128, SUC_lexicon_tags_37 },
    { 0x3a066128, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1706612d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x66e6612f, SUC_lexicon_tags_48 },
    { 0xa9c6612f, SUC_lexicon_tags_66 },
    { 0x7d866131, SUC_lexicon_tags_39 },
    { 0xf7c66131, SUC_lexicon_tags_31 },
    { 0xb6266132, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8666137, SUC_lexicon_tags_9 },
    { 0x10066137, SUC_lexicon_tags_11 },
    { 0xb0466139, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xad66613b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x73a6613d, SUC_lexicon_tags_66 },
    { 0xf4a6613d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e66141, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a66146, SUC_lexicon_tags_6 },
    { 0x72e66146, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd46614c, SUC_lexicon_tags_12 },
    { 0xa7c6614d, SUC_lexicon_tags_1 },
    { 0xf5e6614e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a266152, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7666155, SUC_lexicon_tags_19 },
    { 0x3c066156, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x67466159, SUC_lexicon_tags_51 },
    { 0xb3e6615a, SUC_lexicon_tags_25 },
    { 0xd6c6615b, SUC_lexicon_tags_19 },
    { 0x9706615a, SUC_lexicon_tags_98 },
    { 0xbca6615d, SUC_lexicon_tags_17 },
    { 0xf786615e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x38466160, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x72466163, SUC_lexicon_tags_29 },
    { 0xab066163, SUC_lexicon_tags_37 },
    { 0xe0466165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a66167, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf666169, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9106616c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb266177, SUC_lexicon_tags_11 },
    { 0x49e66177, SUC_lexicon_tags_23 },
    { 0x7d266179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d26617c, SUC_lexicon_tags_16 },
    { 0x9546617d, SUC_lexicon_tags_0 },
    { 0xf226617e, SUC_lexicon_tags_40 },
    { 0x2566617f, SUC_lexicon_tags_65 },
    { 0xefe6617f, SUC_lexicon_tags_11 },
    { 0x25866180, SUC_lexicon_tags_0 },
    { 0x8a26617c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x37a66184, SUC_lexicon_tags_15 },
    { 0xd8466185, SUC_lexicon_tags_39 },
    { 0xb3c66185, SUC_lexicon_tags_19 },
    { 0xc2866184, SUC_lexicon_tags_25 },
    { 0x32866188, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f266193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f866195, SUC_lexicon_tags_47 },
    { 0x3d466196, SUC_lexicon_tags_17 },
    { 0x29066196, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3d666199, SUC_lexicon_tags_2 },
    { 0x47666199, SUC_lexicon_tags_12 },
    { 0xb3e6619a, SUC_lexicon_tags_12 },
    { 0xe8c6619c, SUC_lexicon_tags_2 },
    { 0x9306619c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac661a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x284661a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a661a7, SUC_lexicon_tags_1 },
    { 0x9de661a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62661ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2661b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336661b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942661bc, SUC_lexicon_tags_14 },
    { 0xc32661bc, SUC_lexicon_tags_17 },
    { 0x19a661be, SUC_lexicon_tags_11 },
    { 0x452661bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe50661c1, SUC_lexicon_tags_11 },
    { 0xb84661c2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac661c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc661cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa661ce, SUC_lexicon_tags_11 },
    { 0x770661cf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2661d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a661d8, SUC_lexicon_tags_11 },
    { 0xdcc661d9, SUC_lexicon_tags_31 },
    { 0xec661da, SUC_lexicon_tags_38 },
    { 0x86c661db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9aa661dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c6661df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8661e2, SUC_lexicon_tags_29 },
    { 0x6fc661e3, SUC_lexicon_tags_1 },
    { 0x7b0661e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a661e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x886661ea, SUC_lexicon_tags_11 },
    { 0x1c2661eb, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xcf4661ed, SUC_lexicon_tags_1 },
    { 0xebc661ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704661f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2661f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2066201, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5666205, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9986620a, SUC_lexicon_tags_14 },
    { 0x56c6620b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be6620f, SUC_lexicon_tags_49 },
    { 0x59e66210, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6866215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe66218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb446621a, SUC_lexicon_tags_7 },
    { 0x5d86621b, SUC_lexicon_tags_45 },
    { 0x8126621c, SUC_lexicon_tags_25 },
    { 0xa506621d, SUC_lexicon_tags_44 },
    { 0xace6621e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc0466220, SUC_lexicon_tags_37 },
    { 0xe4066221, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed666223, SUC_lexicon_tags_45 },
    { 0x6b066224, SUC_lexicon_tags_25 },
    { 0xfa66225, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58466229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c86622b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6826622d, SUC_lexicon_tags_8 },
    { 0xf3c6622e, SUC_lexicon_tags_2 },
    { 0x2ee6622e, SUC_lexicon_tags_0 },
    { 0xac866230, SUC_lexicon_tags_34 },
    { 0x1b86622f, SUC_lexicon_tags_7 },
    { 0x60c66230, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xca666234, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae66237, SUC_lexicon_tags_7 },
    { 0x78266237, SUC_lexicon_tags_25 },
    { 0x2e266239, SUC_lexicon_tags_2 },
    { 0x48266239, SUC_lexicon_tags_26 },
    { 0x59066237, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a66623d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e66240, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ee66242, SUC_lexicon_tags_7 },
    { 0x29666243, SUC_lexicon_tags_45 },
    { 0xf3e66244, SUC_lexicon_tags_37 },
    { 0x62e66244, SUC_lexicon_tags_53 },
    { 0x74466245, SUC_lexicon_tags_15 },
    { 0x5a266245, SUC_lexicon_tags_1 },
    { 0xd066248, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50a6624a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3386624e, SUC_lexicon_tags_806 },
    { 0x26a6624f, SUC_lexicon_tags_7 },
    { 0xfac66250, SUC_lexicon_tags_2 },
    { 0x5c066250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0a66253, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea266258, SUC_lexicon_tags_32 },
    { 0x55266258, SUC_lexicon_tags_55 },
    { 0xb686625a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a66263, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c06626b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e6626f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaca66271, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba66276, SUC_lexicon_tags_0 },
    { 0xb3666277, SUC_lexicon_tags_7 },
    { 0x28866278, SUC_lexicon_tags_37 },
    { 0x16666279, SUC_lexicon_tags_65 },
    { 0xcc26627a, SUC_lexicon_tags_12 },
    { 0x27a6627b, SUC_lexicon_tags_25 },
    { 0x4fe6627c, SUC_lexicon_tags_27 },
    { 0x6bc6627a, SUC_lexicon_tags_12 },
    { 0xfde6627e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8466281, SUC_lexicon_tags_15 },
    { 0x4c666282, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x67c66284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61c66286, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86666293, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e6629a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3e6629c, SUC_lexicon_tags_37 },
    { 0xfe06629d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e662a0, SUC_lexicon_tags_12 },
    { 0xcaa662a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a662a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e662a7, SUC_lexicon_tags_2 },
    { 0x228662a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd56662aa, SUC_lexicon_tags_23 },
    { 0x64662ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92662ad, SUC_lexicon_tags_65 },
    { 0x3f2662ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10a662b0, SUC_lexicon_tags_16 },
    { 0x11a662b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x422662b3, SUC_lexicon_tags_8 },
    { 0x138662b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46662c3, SUC_lexicon_tags_38 },
    { 0x7d8662c4, SUC_lexicon_tags_37 },
    { 0xd80662c3, SUC_lexicon_tags_49 },
    { 0x3ac662c6, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xada662c9, SUC_lexicon_tags_46 },
    { 0x5aa662ca, SUC_lexicon_tags_11 },
    { 0x9ae662cb, SUC_lexicon_tags_846 },
    { 0, NULL },
    { 0xd42662cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6662d6, SUC_lexicon_tags_29 },
    { 0x722662d7, SUC_lexicon_tags_17 },
    { 0xd18662d8, SUC_lexicon_tags_15 },
    { 0x49e662d9, SUC_lexicon_tags_11 },
    { 0x63a662d9, SUC_lexicon_tags_17 },
    { 0x870662d9, SUC_lexicon_tags_2 },
    { 0x576662d6, SUC_lexicon_tags_17 },
    { 0x818662dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae662e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c662ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8662f2, SUC_lexicon_tags_0 },
    { 0xd2662f3, SUC_lexicon_tags_30 },
    { 0xae8662f4, SUC_lexicon_tags_49 },
    { 0x862662f4, SUC_lexicon_tags_25 },
    { 0x7d4662f5, SUC_lexicon_tags_48 },
    { 0xcc6662f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x982662f9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2662fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x852662ff, SUC_lexicon_tags_7 },
    { 0x51c66300, SUC_lexicon_tags_37 },
    { 0x7a2662ff, SUC_lexicon_tags_32 },
    { 0x3f2662ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf1a66304, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1946630b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6666313, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24666315, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae66318, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d26631a, SUC_lexicon_tags_33 },
    { 0x6a06631a, SUC_lexicon_tags_47 },
    { 0x5246631c, SUC_lexicon_tags_48 },
    { 0x39c6631c, SUC_lexicon_tags_45 },
    { 0x2906631c, SUC_lexicon_tags_12 },
    { 0x81e6631f, SUC_lexicon_tags_30 },
    { 0xa606631c, SUC_lexicon_tags_8 },
    { 0x1b866321, SUC_lexicon_tags_3 },
    { 0x686631f, SUC_lexicon_tags_31 },
    { 0xb8e66323, SUC_lexicon_tags_28 },
    { 0xbca6631e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29e66326, SUC_lexicon_tags_4 },
    { 0x78666327, SUC_lexicon_tags_15 },
    { 0x1aa66327, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7906632a, SUC_lexicon_tags_15 },
    { 0xe86632a, SUC_lexicon_tags_17 },
    { 0x946632a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa466330, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa66334, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2866633c, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba66343, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa66346, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e06634b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7066634e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x45866350, SUC_lexicon_tags_138 },
    { 0xe6466350, SUC_lexicon_tags_33 },
    { 0x92a66352, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb8066354, SUC_lexicon_tags_30 },
    { 0x8a666355, SUC_lexicon_tags_3 },
    { 0x6d066356, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x18066358, SUC_lexicon_tags_16 },
    { 0x87a66359, SUC_lexicon_tags_32 },
    { 0xae86635a, SUC_lexicon_tags_0 },
    { 0xd4e6635b, SUC_lexicon_tags_43 },
    { 0x8d46635c, SUC_lexicon_tags_30 },
    { 0x42a6635c, SUC_lexicon_tags_18 },
    { 0x146635d, SUC_lexicon_tags_68 },
    { 0x5a46635f, SUC_lexicon_tags_15 },
    { 0xef86635f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6266363, SUC_lexicon_tags_11 },
    { 0x7d666363, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18666366, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e46636c, SUC_lexicon_tags_107 },
    { 0xc106636d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c66376, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x96a66378, SUC_lexicon_tags_37 },
    { 0xf3c66379, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a6637c, SUC_lexicon_tags_38 },
    { 0xc146637d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c66388, SUC_lexicon_tags_27 },
    { 0x28466389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e6638e, SUC_lexicon_tags_17 },
    { 0x4846638f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e66395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7026639b, SUC_lexicon_tags_11 },
    { 0xb546639b, SUC_lexicon_tags_77 },
    { 0xae06639d, SUC_lexicon_tags_35 },
    { 0x4246639d, SUC_lexicon_tags_17 },
    { 0x87a6639f, SUC_lexicon_tags_19 },
    { 0x374663a0, SUC_lexicon_tags_15 },
    { 0x40a663a1, SUC_lexicon_tags_57 },
    { 0xc0e663a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbe8663a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2663a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c663ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89a663af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e663b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c663b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22663b9, SUC_lexicon_tags_37 },
    { 0x4de663ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8663c1, SUC_lexicon_tags_44 },
    { 0xbc6663c1, SUC_lexicon_tags_37 },
    { 0x872663c3, SUC_lexicon_tags_43 },
    { 0xd5c663c3, SUC_lexicon_tags_37 },
    { 0xcd4663c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x95e663c7, SUC_lexicon_tags_27 },
    { 0x3d8663c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x810663cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbf4663cd, SUC_lexicon_tags_45 },
    { 0xf88663cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810663d3, SUC_lexicon_tags_15 },
    { 0x1c2663d4, SUC_lexicon_tags_37 },
    { 0x4ca663d5, SUC_lexicon_tags_1502 },
    { 0x71e663d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74663de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e663e2, SUC_lexicon_tags_14 },
    { 0xc82663e2, SUC_lexicon_tags_12 },
    { 0xe5a663e4, SUC_lexicon_tags_15 },
    { 0x496663e5, SUC_lexicon_tags_3 },
    { 0x7da663e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0663ea, SUC_lexicon_tags_39 },
    { 0xe36663ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4663ef, SUC_lexicon_tags_8 },
    { 0x5e663f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a663f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc0663f5, SUC_lexicon_tags_37 },
    { 0x48a663f6, SUC_lexicon_tags_7 },
    { 0x8f6663f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a663fc, SUC_lexicon_tags_9 },
    { 0x146663fd, SUC_lexicon_tags_17 },
    { 0x660663fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5e66400, SUC_lexicon_tags_19 },
    { 0xd4a66401, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4666408, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x75e6640a, SUC_lexicon_tags_1 },
    { 0x6566640b, SUC_lexicon_tags_19 },
    { 0xd686640c, SUC_lexicon_tags_38 },
    { 0xae66640b, SUC_lexicon_tags_15 },
    { 0xe426640c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4c66410, SUC_lexicon_tags_68 },
    { 0xb5666411, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7bc66413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbbc66415, SUC_lexicon_tags_33 },
    { 0x93866416, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbd666418, SUC_lexicon_tags_11 },
    { 0x89666418, SUC_lexicon_tags_37 },
    { 0xb9c6641a, SUC_lexicon_tags_87 },
    { 0x83c6641b, SUC_lexicon_tags_28 },
    { 0x4966641c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e66420, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90266424, SUC_lexicon_tags_17 },
    { 0xb3e66425, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc66430, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb606643c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8006643f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91266442, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa6644a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc6644d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c466452, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2f866454, SUC_lexicon_tags_30 },
    { 0x14666455, SUC_lexicon_tags_30 },
    { 0x95266456, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c66462, SUC_lexicon_tags_114 },
    { 0x7066462, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3466465, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e6646e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c66474, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98466476, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5066479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3926647f, SUC_lexicon_tags_23 },
    { 0x53a66480, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1e66482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca666485, SUC_lexicon_tags_31 },
    { 0xcf266485, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc386648a, SUC_lexicon_tags_29 },
    { 0xe0e6648a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e26648e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16466494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e6649c, SUC_lexicon_tags_0 },
    { 0x4726649d, SUC_lexicon_tags_11 },
    { 0xd306649e, SUC_lexicon_tags_27 },
    { 0x51c6649e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116664a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3a664ab, SUC_lexicon_tags_3 },
    { 0x874664ac, SUC_lexicon_tags_17 },
    { 0xdbc664ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9a664af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6664b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c664b9, SUC_lexicon_tags_60 },
    { 0x984664ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2664bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26664c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e664c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c664c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80664ca, SUC_lexicon_tags_87 },
    { 0x54e664ca, SUC_lexicon_tags_30 },
    { 0x630664cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8664d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a664d3, SUC_lexicon_tags_65 },
    { 0xe14664d3, SUC_lexicon_tags_3 },
    { 0x7d6664d3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5cc664d7, SUC_lexicon_tags_8 },
    { 0xd42664d8, SUC_lexicon_tags_2 },
    { 0xc20664d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6de664db, SUC_lexicon_tags_12 },
    { 0x890664db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc664e3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xda664e5, SUC_lexicon_tags_49 },
    { 0x266664e6, SUC_lexicon_tags_39 },
    { 0x714664e6, SUC_lexicon_tags_0 },
    { 0x218664e8, SUC_lexicon_tags_2 },
    { 0x5c2664e9, SUC_lexicon_tags_30 },
    { 0xebe664e7, SUC_lexicon_tags_37 },
    { 0xa58664eb, SUC_lexicon_tags_2 },
    { 0x35e664ec, SUC_lexicon_tags_19 },
    { 0x7a6664ec, SUC_lexicon_tags_64 },
    { 0x5c6664ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba664f1, SUC_lexicon_tags_17 },
    { 0xcb0664f1, SUC_lexicon_tags_31 },
    { 0x78664f2, SUC_lexicon_tags_25 },
    { 0xc6e664f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e664f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38e664fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x484664fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec66500, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xade66503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b666508, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6926650d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e66510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c066513, SUC_lexicon_tags_11 },
    { 0x49266514, SUC_lexicon_tags_6 },
    { 0xda066515, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd7466517, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdbc66519, SUC_lexicon_tags_70 },
    { 0x1e06651a, SUC_lexicon_tags_27 },
    { 0x53c66519, SUC_lexicon_tags_11 },
    { 0x76e6651b, SUC_lexicon_tags_44 },
    { 0x1e66651d, SUC_lexicon_tags_11 },
    { 0xaf86651b, SUC_lexicon_tags_23 },
    { 0x6d06651b, SUC_lexicon_tags_17 },
    { 0xfd66651c, SUC_lexicon_tags_2 },
    { 0x7b06651e, SUC_lexicon_tags_2 },
    { 0x65066522, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a66525, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a6652b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b66652e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x43866530, SUC_lexicon_tags_23 },
    { 0x8d666530, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81c66533, SUC_lexicon_tags_9 },
    { 0x5c466533, SUC_lexicon_tags_48 },
    { 0x4ba66535, SUC_lexicon_tags_46 },
    { 0x86466536, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee66539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b66653c, SUC_lexicon_tags_198 },
    { 0x12c6653d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa986653f, SUC_lexicon_tags_17 },
    { 0xf086653f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a66546, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac66549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c86654d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3066553, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5266559, SUC_lexicon_tags_11 },
    { 0x62266559, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8066655e, SUC_lexicon_tags_7 },
    { 0x366655f, SUC_lexicon_tags_15 },
    { 0xe666560, SUC_lexicon_tags_11 },
    { 0xc826655f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf266565, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa1a66567, SUC_lexicon_tags_25 },
    { 0xc6066568, SUC_lexicon_tags_47 },
    { 0x50066569, SUC_lexicon_tags_15 },
    { 0x4046656a, SUC_lexicon_tags_8 },
    { 0x2606656b, SUC_lexicon_tags_15 },
    { 0x6b06656b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4246656e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada66572, SUC_lexicon_tags_7 },
    { 0x8fa66573, SUC_lexicon_tags_49 },
    { 0x8fe66574, SUC_lexicon_tags_63 },
    { 0xa7c66575, SUC_lexicon_tags_17 },
    { 0x60c66576, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c866579, SUC_lexicon_tags_54 },
    { 0x6d46657a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1326657c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfb86657e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2466584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9466586, SUC_lexicon_tags_19 },
    { 0x23e66587, SUC_lexicon_tags_11 },
    { 0xe2866588, SUC_lexicon_tags_12 },
    { 0xb2266589, SUC_lexicon_tags_49 },
    { 0x3666658a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2046658d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da66592, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0c66594, SUC_lexicon_tags_63 },
    { 0xa66595, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x70066597, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e665a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be665a4, SUC_lexicon_tags_37 },
    { 0x9ee665a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa665a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd58665ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686665b0, SUC_lexicon_tags_12 },
    { 0x3fe665b1, SUC_lexicon_tags_79 },
    { 0xce0665b0, SUC_lexicon_tags_25 },
    { 0x718665b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38665bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb4a665bd, SUC_lexicon_tags_66 },
    { 0x956665bd, SUC_lexicon_tags_32 },
    { 0x25e665be, SUC_lexicon_tags_15 },
    { 0x91c665c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48665c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x828665c9, SUC_lexicon_tags_39 },
    { 0x674665c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e665ce, SUC_lexicon_tags_27 },
    { 0x664665cf, SUC_lexicon_tags_31 },
    { 0x74a665cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2fc665d2, SUC_lexicon_tags_14 },
    { 0x1da665d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa665d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e665d9, SUC_lexicon_tags_19 },
    { 0xbf2665da, SUC_lexicon_tags_11 },
    { 0x19a665db, SUC_lexicon_tags_15 },
    { 0xdfa665dc, SUC_lexicon_tags_17 },
    { 0x6c8665dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746665e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x380665e3, SUC_lexicon_tags_27 },
    { 0x256665e3, SUC_lexicon_tags_1 },
    { 0xd7c665e3, SUC_lexicon_tags_26 },
    { 0x52c665e6, SUC_lexicon_tags_12 },
    { 0x202665e7, SUC_lexicon_tags_2 },
    { 0x8e0665e7, SUC_lexicon_tags_27 },
    { 0x58e665e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c665ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x742665ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x424665f0, SUC_lexicon_tags_11 },
    { 0x39a665f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xee0665f3, SUC_lexicon_tags_33 },
    { 0xf78665f3, SUC_lexicon_tags_1139 },
    { 0, NULL },
    { 0x4da665f6, SUC_lexicon_tags_12 },
    { 0xf4a665f7, SUC_lexicon_tags_3 },
    { 0xf5e665f8, SUC_lexicon_tags_37 },
    { 0x55e665f9, SUC_lexicon_tags_12 },
    { 0x1d6665f9, SUC_lexicon_tags_12 },
    { 0x306665fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe68665fd, SUC_lexicon_tags_23 },
    { 0x70a665fd, SUC_lexicon_tags_27 },
    { 0xa4a665fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a66609, SUC_lexicon_tags_17 },
    { 0x40466609, SUC_lexicon_tags_23 },
    { 0x2786660a, SUC_lexicon_tags_19 },
    { 0xacc66609, SUC_lexicon_tags_37 },
    { 0x4386660d, SUC_lexicon_tags_2 },
    { 0x4b66660e, SUC_lexicon_tags_78 },
    { 0xb806660d, SUC_lexicon_tags_47 },
    { 0x7706660e, SUC_lexicon_tags_45 },
    { 0xf2e66611, SUC_lexicon_tags_22 },
    { 0x17866612, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a866614, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a66617, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a26661a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe86661c, SUC_lexicon_tags_25 },
    { 0xac46661c, SUC_lexicon_tags_37 },
    { 0x9386661e, SUC_lexicon_tags_43 },
    { 0x1486661f, SUC_lexicon_tags_7 },
    { 0xdd86661f, SUC_lexicon_tags_46 },
    { 0x2be66621, SUC_lexicon_tags_60 },
    { 0x1aa66621, SUC_lexicon_tags_37 },
    { 0x1cc6661e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25266626, SUC_lexicon_tags_43 },
    { 0xf2a66627, SUC_lexicon_tags_11 },
    { 0x49466628, SUC_lexicon_tags_47 },
    { 0xa8866627, SUC_lexicon_tags_9 },
    { 0xcbc66629, SUC_lexicon_tags_1 },
    { 0x5826662b, SUC_lexicon_tags_11 },
    { 0xf6c6662c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46662f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc66632, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e666636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93666638, SUC_lexicon_tags_40 },
    { 0xa9066638, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a66642, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae466647, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe6866649, SUC_lexicon_tags_25 },
    { 0xf8e6664a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c6664d, SUC_lexicon_tags_15 },
    { 0x7966664d, SUC_lexicon_tags_9 },
    { 0xaf66664f, SUC_lexicon_tags_38 },
    { 0x4a6664f, SUC_lexicon_tags_17 },
    { 0xc5266651, SUC_lexicon_tags_15 },
    { 0xc6a66652, SUC_lexicon_tags_27 },
    { 0xaa266651, SUC_lexicon_tags_11 },
    { 0xbb466653, SUC_lexicon_tags_25 },
    { 0x58e66655, SUC_lexicon_tags_27 },
    { 0x38666653, SUC_lexicon_tags_17 },
    { 0x80666657, SUC_lexicon_tags_17 },
    { 0x2b866658, SUC_lexicon_tags_9 },
    { 0x6b466659, SUC_lexicon_tags_8 },
    { 0xd9c6665a, SUC_lexicon_tags_30 },
    { 0x3f06665b, SUC_lexicon_tags_37 },
    { 0xa7e66654, SUC_lexicon_tags_35 },
    { 0x8a86664d, SUC_lexicon_tags_6 },
    { 0x2bc6665e, SUC_lexicon_tags_31 },
    { 0x1b66665e, SUC_lexicon_tags_0 },
    { 0xd306665e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x93266662, SUC_lexicon_tags_15 },
    { 0xaa066663, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb286666a, SUC_lexicon_tags_14 },
    { 0xf606666a, SUC_lexicon_tags_1 },
    { 0xa986666c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4e6666e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3066670, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54666674, SUC_lexicon_tags_82 },
    { 0xa3a66675, SUC_lexicon_tags_45 },
    { 0xbea66674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17266679, SUC_lexicon_tags_11 },
    { 0x7b466679, SUC_lexicon_tags_15 },
    { 0x5a66667b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ea6667d, SUC_lexicon_tags_12 },
    { 0xce06667e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26668f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda66696, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc6669b, SUC_lexicon_tags_37 },
    { 0x38e6669c, SUC_lexicon_tags_16 },
    { 0xeac6669b, SUC_lexicon_tags_38 },
    { 0xd426669d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854666a5, SUC_lexicon_tags_15 },
    { 0xf12666a5, SUC_lexicon_tags_29 },
    { 0xcd8666a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa666aa, SUC_lexicon_tags_12 },
    { 0x6e8666aa, SUC_lexicon_tags_17 },
    { 0xf10666ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe02666ae, SUC_lexicon_tags_17 },
    { 0x21c666ae, SUC_lexicon_tags_38 },
    { 0x4ce666ae, SUC_lexicon_tags_12 },
    { 0xe56666b1, SUC_lexicon_tags_17 },
    { 0x3f6666b1, SUC_lexicon_tags_37 },
    { 0x8de666b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c666b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0666bd, SUC_lexicon_tags_45 },
    { 0x86a666be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6666c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x180666c6, SUC_lexicon_tags_10 },
    { 0x30c666c7, SUC_lexicon_tags_27 },
    { 0xf8a666c7, SUC_lexicon_tags_75 },
    { 0x5f2666c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8666cc, SUC_lexicon_tags_11 },
    { 0x24e666cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26666d1, SUC_lexicon_tags_17 },
    { 0x1a6666d1, SUC_lexicon_tags_11 },
    { 0x784666d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8666db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xce6666dd, SUC_lexicon_tags_0 },
    { 0x2f4666de, SUC_lexicon_tags_6 },
    { 0xd86666de, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x230666e1, SUC_lexicon_tags_14 },
    { 0x250666e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x448666e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x366666e6, SUC_lexicon_tags_15 },
    { 0x960666e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x638666e9, SUC_lexicon_tags_25 },
    { 0x4a6666e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34666ed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de666f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x69a666f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8666f9, SUC_lexicon_tags_2 },
    { 0x6e666fa, SUC_lexicon_tags_68 },
    { 0xc3e666fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92666fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a66703, SUC_lexicon_tags_23 },
    { 0xc5866704, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5a66706, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa6670c, SUC_lexicon_tags_20 },
    { 0xc646670c, SUC_lexicon_tags_1 },
    { 0xfa86670e, SUC_lexicon_tags_23 },
    { 0xf9a6670d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38e66711, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa1266713, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f26671b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x4f86671d, SUC_lexicon_tags_116 },
    { 0xb686671e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c066722, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e466728, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea6672e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc666734, SUC_lexicon_tags_47 },
    { 0x92066735, SUC_lexicon_tags_53 },
    { 0x666735, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd866673c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c66742, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc266749, SUC_lexicon_tags_11 },
    { 0x9a26674a, SUC_lexicon_tags_37 },
    { 0x3da6674b, SUC_lexicon_tags_15 },
    { 0x9fc66749, SUC_lexicon_tags_37 },
    { 0x3b26674d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c26674f, SUC_lexicon_tags_81 },
    { 0xda666750, SUC_lexicon_tags_107 },
    { 0xe0666751, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x52066753, SUC_lexicon_tags_27 },
    { 0x94466753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64266758, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e6675c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9f86675e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee66761, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a66766, SUC_lexicon_tags_17 },
    { 0xd4866767, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4ee66769, SUC_lexicon_tags_15 },
    { 0x40466769, SUC_lexicon_tags_32 },
    { 0x8f66676b, SUC_lexicon_tags_11 },
    { 0xe466676c, SUC_lexicon_tags_79 },
    { 0xeae6676b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe086676f, SUC_lexicon_tags_37 },
    { 0x51c66770, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf86677b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa426677f, SUC_lexicon_tags_17 },
    { 0x4dc6677f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e66785, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee66787, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb046678a, SUC_lexicon_tags_25 },
    { 0xda86678b, SUC_lexicon_tags_2 },
    { 0x786678c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46678f, SUC_lexicon_tags_15 },
    { 0x2286678f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c66799, SUC_lexicon_tags_7 },
    { 0xb666679a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde06679c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc426679e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0667a2, SUC_lexicon_tags_11 },
    { 0xcac667a3, SUC_lexicon_tags_2 },
    { 0x822667a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x554667a6, SUC_lexicon_tags_23 },
    { 0xa08667a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac4667a9, SUC_lexicon_tags_25 },
    { 0x1e8667aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe667ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386667b1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0667bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2667bf, SUC_lexicon_tags_15 },
    { 0x28e667c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6e667c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c667ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a667d1, SUC_lexicon_tags_2 },
    { 0xa16667d1, SUC_lexicon_tags_11 },
    { 0xbac667d3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8aa667d5, SUC_lexicon_tags_11 },
    { 0xef8667d6, SUC_lexicon_tags_17 },
    { 0xcaa667d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x524667da, SUC_lexicon_tags_27 },
    { 0x30a667db, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394667e3, SUC_lexicon_tags_66 },
    { 0xaec667e4, SUC_lexicon_tags_33 },
    { 0x280667e4, SUC_lexicon_tags_25 },
    { 0x8d6667e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa667ec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00667f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2667f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488667ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4f666801, SUC_lexicon_tags_37 },
    { 0xdde66801, SUC_lexicon_tags_7 },
    { 0xbf866803, SUC_lexicon_tags_17 },
    { 0x94866804, SUC_lexicon_tags_15 },
    { 0x97666805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3a66807, SUC_lexicon_tags_66 },
    { 0x1066807, SUC_lexicon_tags_108 },
    { 0x67c66808, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a86680c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6766680e, SUC_lexicon_tags_17 },
    { 0x5d26680e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c66814, SUC_lexicon_tags_8 },
    { 0x47466814, SUC_lexicon_tags_49 },
    { 0x85466814, SUC_lexicon_tags_170 },
    { 0xf9866815, SUC_lexicon_tags_37 },
    { 0x80e66818, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa326681c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x27a6681e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6666820, SUC_lexicon_tags_10 },
    { 0x97466820, SUC_lexicon_tags_15 },
    { 0xd2866822, SUC_lexicon_tags_25 },
    { 0xaa666823, SUC_lexicon_tags_7 },
    { 0x5da66824, SUC_lexicon_tags_25 },
    { 0x20266824, SUC_lexicon_tags_60 },
    { 0x26c66826, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a66682a, SUC_lexicon_tags_15 },
    { 0x6d26682a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x59c6682d, SUC_lexicon_tags_25 },
    { 0x2786682e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88866832, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1bc66834, SUC_lexicon_tags_14 },
    { 0x7a466834, SUC_lexicon_tags_14 },
    { 0x20c66834, SUC_lexicon_tags_37 },
    { 0x82666837, SUC_lexicon_tags_37 },
    { 0xa8466835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c86683a, SUC_lexicon_tags_17 },
    { 0xbf86683b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33066842, SUC_lexicon_tags_12 },
    { 0xc2e66842, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c66846, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc66849, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5646684c, SUC_lexicon_tags_27 },
    { 0xcf86684d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a66850, SUC_lexicon_tags_12 },
    { 0x8cc66851, SUC_lexicon_tags_25 },
    { 0xdfc66852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e666856, SUC_lexicon_tags_2 },
    { 0x5a666857, SUC_lexicon_tags_17 },
    { 0x78466856, SUC_lexicon_tags_47 },
    { 0x6c266859, SUC_lexicon_tags_25 },
    { 0xd866685a, SUC_lexicon_tags_15 },
    { 0xcb666856, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14266861, SUC_lexicon_tags_30 },
    { 0x4a66862, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae66686c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4046686f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce66872, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30866878, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc6687b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b26687d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e66881, SUC_lexicon_tags_2 },
    { 0xc1c66882, SUC_lexicon_tags_13 },
    { 0x9ca66882, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c66688e, SUC_lexicon_tags_25 },
    { 0x45e6688f, SUC_lexicon_tags_15 },
    { 0xa6688e, SUC_lexicon_tags_30 },
    { 0x2a266891, SUC_lexicon_tags_61 },
    { 0x6e06688f, SUC_lexicon_tags_9 },
    { 0xd0466890, SUC_lexicon_tags_14 },
    { 0xbd266894, SUC_lexicon_tags_65 },
    { 0x64866895, SUC_lexicon_tags_49 },
    { 0xd9466896, SUC_lexicon_tags_47 },
    { 0x9c466895, SUC_lexicon_tags_14 },
    { 0x25c66893, SUC_lexicon_tags_11 },
    { 0x1a466899, SUC_lexicon_tags_11 },
    { 0xcd66689a, SUC_lexicon_tags_48 },
    { 0xa9c6689b, SUC_lexicon_tags_1 },
    { 0xc766689c, SUC_lexicon_tags_22 },
    { 0x4ac6689d, SUC_lexicon_tags_17 },
    { 0x8646689a, SUC_lexicon_tags_17 },
    { 0x77c6689e, SUC_lexicon_tags_17 },
    { 0x5e46689f, SUC_lexicon_tags_1 },
    { 0x8da668a1, SUC_lexicon_tags_7 },
    { 0xfe466895, SUC_lexicon_tags_27 },
    { 0xb0a66894, SUC_lexicon_tags_27 },
    { 0x7ae668a4, SUC_lexicon_tags_25 },
    { 0x366668a5, SUC_lexicon_tags_32 },
    { 0x6dc668a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa58668a8, SUC_lexicon_tags_25 },
    { 0x9ac668a9, SUC_lexicon_tags_37 },
    { 0xff0668a9, SUC_lexicon_tags_25 },
    { 0x6aa668aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8668ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00668b3, SUC_lexicon_tags_14 },
    { 0x4ac668b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60668b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc48668b9, SUC_lexicon_tags_2 },
    { 0x3d0668ba, SUC_lexicon_tags_17 },
    { 0x68a668ba, SUC_lexicon_tags_43 },
    { 0xb42668bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd74668be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb3c668c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566668c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c668c8, SUC_lexicon_tags_28 },
    { 0x85e668c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8668cd, SUC_lexicon_tags_23 },
    { 0x858668ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b6668d0, SUC_lexicon_tags_38 },
    { 0xc90668d0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xd92668d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0668d8, SUC_lexicon_tags_35 },
    { 0x8a2668d8, SUC_lexicon_tags_41 },
    { 0xdfa668da, SUC_lexicon_tags_37 },
    { 0x9f6668db, SUC_lexicon_tags_27 },
    { 0x78a668dc, SUC_lexicon_tags_37 },
    { 0x2ae668dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac668e5, SUC_lexicon_tags_38 },
    { 0x858668e5, SUC_lexicon_tags_11 },
    { 0xa3c668e5, SUC_lexicon_tags_37 },
    { 0xf50668e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba668f3, SUC_lexicon_tags_45 },
    { 0x8e0668f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e668fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4668fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20466902, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xed266904, SUC_lexicon_tags_0 },
    { 0x8a266905, SUC_lexicon_tags_29 },
    { 0xb3e66906, SUC_lexicon_tags_37 },
    { 0x9466906, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e06690a, SUC_lexicon_tags_33 },
    { 0xaf46690b, SUC_lexicon_tags_2 },
    { 0xd246690c, SUC_lexicon_tags_21 },
    { 0x4786690d, SUC_lexicon_tags_37 },
    { 0x5ec6690b, SUC_lexicon_tags_11 },
    { 0xaa66690f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44e66911, SUC_lexicon_tags_37 },
    { 0x33466912, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfee66914, SUC_lexicon_tags_2 },
    { 0x77066915, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a6691a, SUC_lexicon_tags_15 },
    { 0xf246691b, SUC_lexicon_tags_16 },
    { 0x646691c, SUC_lexicon_tags_5 },
    { 0x7f66691c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb146691f, SUC_lexicon_tags_14 },
    { 0x2be66920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dc66922, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e266925, SUC_lexicon_tags_146 },
    { 0x5a866926, SUC_lexicon_tags_12 },
    { 0x94a66927, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd26692c, SUC_lexicon_tags_37 },
    { 0x9e06692d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xda26692f, SUC_lexicon_tags_23 },
    { 0x31c66930, SUC_lexicon_tags_16 },
    { 0x1ea66930, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e666933, SUC_lexicon_tags_1 },
    { 0x1de66934, SUC_lexicon_tags_31 },
    { 0xdc466935, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x70666938, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc6693e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc66942, SUC_lexicon_tags_116 },
    { 0x20266943, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb466945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb666947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e6694a, SUC_lexicon_tags_14 },
    { 0xfae6694b, SUC_lexicon_tags_9 },
    { 0x81a6694a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed266951, SUC_lexicon_tags_38 },
    { 0x20066952, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e66956, SUC_lexicon_tags_64 },
    { 0x82066957, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf226695b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf686695d, SUC_lexicon_tags_38 },
    { 0xe166695e, SUC_lexicon_tags_7 },
    { 0xd406695d, SUC_lexicon_tags_63 },
    { 0xfd66695e, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x88066963, SUC_lexicon_tags_15 },
    { 0x55666964, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a66968, SUC_lexicon_tags_40 },
    { 0xe8066969, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6906696e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5ea66970, SUC_lexicon_tags_15 },
    { 0x58066970, SUC_lexicon_tags_47 },
    { 0x73466970, SUC_lexicon_tags_25 },
    { 0xfbc66973, SUC_lexicon_tags_17 },
    { 0xbdc66974, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfa66976, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e6697a, SUC_lexicon_tags_12 },
    { 0xae66697b, SUC_lexicon_tags_581 },
    { 0x1026697c, SUC_lexicon_tags_17 },
    { 0xc306697d, SUC_lexicon_tags_37 },
    { 0x6ba6697d, SUC_lexicon_tags_45 },
    { 0xe0e6697f, SUC_lexicon_tags_19 },
    { 0x4ba66980, SUC_lexicon_tags_19 },
    { 0x37866981, SUC_lexicon_tags_62 },
    { 0xa7266982, SUC_lexicon_tags_12 },
    { 0xea066983, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x54a66985, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7266989, SUC_lexicon_tags_47 },
    { 0x33e6698a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b46698c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5a86698e, SUC_lexicon_tags_16 },
    { 0xc666698f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce6699b, SUC_lexicon_tags_49 },
    { 0xdd86699c, SUC_lexicon_tags_6 },
    { 0x2b06699b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0669a4, SUC_lexicon_tags_60 },
    { 0x510669a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a669aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0a669ac, SUC_lexicon_tags_25 },
    { 0xf30669ac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8669b9, SUC_lexicon_tags_48 },
    { 0x64c669b9, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x972669bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72669c0, SUC_lexicon_tags_15 },
    { 0xfca669c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf2669c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6669c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa22669c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0669d8, SUC_lexicon_tags_1483 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e669db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4669de, SUC_lexicon_tags_2 },
    { 0x8c8669df, SUC_lexicon_tags_17 },
    { 0xdd0669e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x168669e2, SUC_lexicon_tags_11 },
    { 0x2be669e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a4669e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a669e8, SUC_lexicon_tags_56 },
    { 0xf06669e9, SUC_lexicon_tags_25 },
    { 0xcd4669e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x136669ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c669f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40669f7, SUC_lexicon_tags_84 },
    { 0xed4669f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf4669fa, SUC_lexicon_tags_7 },
    { 0xbce669fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca669fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c66a02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc9066a04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e66a0b, SUC_lexicon_tags_99 },
    { 0xc0666a0b, SUC_lexicon_tags_14 },
    { 0x18a66a0d, SUC_lexicon_tags_7 },
    { 0xe6466a0c, SUC_lexicon_tags_0 },
    { 0x64866a0f, SUC_lexicon_tags_63 },
    { 0x84a66a0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1e66a12, SUC_lexicon_tags_45 },
    { 0x51e66a12, SUC_lexicon_tags_17 },
    { 0xa9266a12, SUC_lexicon_tags_23 },
    { 0x82866a15, SUC_lexicon_tags_3 },
    { 0xd6466a12, SUC_lexicon_tags_37 },
    { 0xebc66a17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8866a19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0466a1d, SUC_lexicon_tags_9 },
    { 0x1b866a1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c66a23, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c66a27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf466a2a, SUC_lexicon_tags_6 },
    { 0x59466a2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a66a2e, SUC_lexicon_tags_48 },
    { 0xa7c66a2f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a66a32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f466a36, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc66a42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc66a45, SUC_lexicon_tags_307 },
    { 0xac066a46, SUC_lexicon_tags_3 },
    { 0x85466a47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f266a4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee66a50, SUC_lexicon_tags_25 },
    { 0xda666a51, SUC_lexicon_tags_1 },
    { 0x66866a51, SUC_lexicon_tags_7 },
    { 0xddc66a50, SUC_lexicon_tags_14 },
    { 0x87c66a51, SUC_lexicon_tags_38 },
    { 0x75266a55, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99466a59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x46066a5c, SUC_lexicon_tags_38 },
    { 0xbf466a5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b666a60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18266a64, SUC_lexicon_tags_6 },
    { 0xc5a66a64, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc66a69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f666a6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa666a71, SUC_lexicon_tags_29 },
    { 0x9b866a72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7cc66a74, SUC_lexicon_tags_1 },
    { 0x4466a75, SUC_lexicon_tags_7 },
    { 0xd4466a76, SUC_lexicon_tags_25 },
    { 0xe6266a75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5266a7b, SUC_lexicon_tags_55 },
    { 0x1fa66a7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83466a7e, SUC_lexicon_tags_17 },
    { 0x23e66a7e, SUC_lexicon_tags_12 },
    { 0xa5266a80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c66a84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5bc66a86, SUC_lexicon_tags_15 },
    { 0x72e66a87, SUC_lexicon_tags_66 },
    { 0xdda66a88, SUC_lexicon_tags_84 },
    { 0x30066a86, SUC_lexicon_tags_3 },
    { 0x1ae66a86, SUC_lexicon_tags_21 },
    { 0x7fc66a8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f866a99, SUC_lexicon_tags_77 },
    { 0xf3466a9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12066aa6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c666aad, SUC_lexicon_tags_19 },
    { 0x51866aad, SUC_lexicon_tags_17 },
    { 0xbc066aaf, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c66ab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be66ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e666ab9, SUC_lexicon_tags_25 },
    { 0xbb266aba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2866abe, SUC_lexicon_tags_25 },
    { 0x76866abf, SUC_lexicon_tags_33 },
    { 0x5de66abf, SUC_lexicon_tags_11 },
    { 0x9666ac0, SUC_lexicon_tags_6 },
    { 0x92a66ac2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20266ac5, SUC_lexicon_tags_14 },
    { 0x8e066ac5, SUC_lexicon_tags_17 },
    { 0xb4466ac7, SUC_lexicon_tags_7 },
    { 0x74266ac7, SUC_lexicon_tags_57 },
    { 0xa8c66ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e66ace, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8ac66ad0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa3466ad2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc666ad4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x78666ad6, SUC_lexicon_tags_30 },
    { 0x10e66ad7, SUC_lexicon_tags_16 },
    { 0xc0466ad8, SUC_lexicon_tags_28 },
    { 0xdca66ad9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90666adb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b066add, SUC_lexicon_tags_23 },
    { 0x4e266add, SUC_lexicon_tags_8 },
    { 0x54666ade, SUC_lexicon_tags_38 },
    { 0xbc466adf, SUC_lexicon_tags_37 },
    { 0x93e66ae1, SUC_lexicon_tags_19 },
    { 0xbd666ae2, SUC_lexicon_tags_17 },
    { 0x4f266ae2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a66ae6, SUC_lexicon_tags_15 },
    { 0x6c266ae6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7aa66ae9, SUC_lexicon_tags_11 },
    { 0x5ba66aea, SUC_lexicon_tags_25 },
    { 0x7c66aeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf266aee, SUC_lexicon_tags_17 },
    { 0x76266aef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a266af1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ec66af3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4666af5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53266afb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x69c66afd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62a66aff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe466b04, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe66b07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad866b0a, SUC_lexicon_tags_10 },
    { 0x73066b0b, SUC_lexicon_tags_66 },
    { 0xbc466b0c, SUC_lexicon_tags_12 },
    { 0x56666b0d, SUC_lexicon_tags_1 },
    { 0xdaa66b0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87666b13, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c866b16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31c66b18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe66b1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe866b1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbe66b1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfa66b21, SUC_lexicon_tags_11 },
    { 0x8a466b22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19466b29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7666b2e, SUC_lexicon_tags_77 },
    { 0xda266b2e, SUC_lexicon_tags_0 },
    { 0xef666b30, SUC_lexicon_tags_37 },
    { 0xb4866b30, SUC_lexicon_tags_2 },
    { 0xb3666b30, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x16466b35, SUC_lexicon_tags_14 },
    { 0x48466b35, SUC_lexicon_tags_38 },
    { 0xa6066b37, SUC_lexicon_tags_19 },
    { 0xb6a66b38, SUC_lexicon_tags_3 },
    { 0xaba66b38, SUC_lexicon_tags_39 },
    { 0x2ec66b39, SUC_lexicon_tags_45 },
    { 0x57666b3b, SUC_lexicon_tags_12 },
    { 0xf9866b37, SUC_lexicon_tags_7 },
    { 0x4066b37, SUC_lexicon_tags_56 },
    { 0x52466b3e, SUC_lexicon_tags_31 },
    { 0x41266b3f, SUC_lexicon_tags_9 },
    { 0x16266b40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x67266b42, SUC_lexicon_tags_103 },
    { 0x2de66b43, SUC_lexicon_tags_19 },
    { 0xf1c66b44, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83666b4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42866b4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x17466b52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6066b58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b666b5a, SUC_lexicon_tags_7 },
    { 0xd8666b5b, SUC_lexicon_tags_37 },
    { 0x7c866b5a, SUC_lexicon_tags_15 },
    { 0x8f066b5b, SUC_lexicon_tags_27 },
    { 0x19066b5a, SUC_lexicon_tags_37 },
    { 0x6f466b5f, SUC_lexicon_tags_1 },
    { 0x6fc66b5f, SUC_lexicon_tags_12 },
    { 0xab266b61, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x87666b63, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69666b68, SUC_lexicon_tags_1 },
    { 0x13066b69, SUC_lexicon_tags_12 },
    { 0x5f666b6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be66b6f, SUC_lexicon_tags_25 },
    { 0xc2666b70, SUC_lexicon_tags_39 },
    { 0x71c66b71, SUC_lexicon_tags_30 },
    { 0xce666b71, SUC_lexicon_tags_38 },
    { 0xa2866b73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb666b76, SUC_lexicon_tags_33 },
    { 0x7f466b77, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x30a66b79, SUC_lexicon_tags_11 },
    { 0xf4a66b79, SUC_lexicon_tags_9 },
    { 0x4b066b7b, SUC_lexicon_tags_1 },
    { 0xef666b7b, SUC_lexicon_tags_45 },
    { 0x52266b7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf266b7f, SUC_lexicon_tags_17 },
    { 0xcba66b80, SUC_lexicon_tags_7 },
    { 0x6466b80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c66b85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xece66b88, SUC_lexicon_tags_5 },
    { 0x9da66b89, SUC_lexicon_tags_68 },
    { 0x8ea66b8a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xea866b8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e066b90, SUC_lexicon_tags_1 },
    { 0x78c66b91, SUC_lexicon_tags_11 },
    { 0x9ce66b92, SUC_lexicon_tags_33 },
    { 0x3a066b91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50066b96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e666b98, SUC_lexicon_tags_33 },
    { 0xa9a66b98, SUC_lexicon_tags_79 },
    { 0x9b866b9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf066b9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x39066b9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4c66ba0, SUC_lexicon_tags_17 },
    { 0xaba66ba1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc666ba5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5066ba8, SUC_lexicon_tags_21 },
    { 0x36e66ba9, SUC_lexicon_tags_17 },
    { 0xba66ba8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c666bb4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce66bba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe866bbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc66bc4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf3466bc6, SUC_lexicon_tags_11 },
    { 0xd2666bc6, SUC_lexicon_tags_4 },
    { 0x9fe66bc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5266bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c66bd3, SUC_lexicon_tags_17 },
    { 0xaf866bd4, SUC_lexicon_tags_33 },
    { 0x57e66bd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea66bd9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e66bdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61666bde, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66666be2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb466bef, SUC_lexicon_tags_37 },
    { 0x8dc66bf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce66bf3, SUC_lexicon_tags_6 },
    { 0x3e466bf3, SUC_lexicon_tags_46 },
    { 0x3dc66bf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e66bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e466c06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae66c0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466c10, SUC_lexicon_tags_47 },
    { 0x7e666c11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7fa66c13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34a66c15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x15266c17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9866c1f, SUC_lexicon_tags_7 },
    { 0x21a66c20, SUC_lexicon_tags_63 },
    { 0x3c066c21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58066c25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab466c28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae66c2c, SUC_lexicon_tags_25 },
    { 0xf9866c2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x25466c2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd666c32, SUC_lexicon_tags_17 },
    { 0x2a666c33, SUC_lexicon_tags_37 },
    { 0x39866c34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4466c36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6866c38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4666c3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17666c3f, SUC_lexicon_tags_43 },
    { 0x5d466c40, SUC_lexicon_tags_25 },
    { 0x15666c40, SUC_lexicon_tags_27 },
    { 0x22e66c41, SUC_lexicon_tags_25 },
    { 0x1c66c40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f866c45, SUC_lexicon_tags_6 },
    { 0xca666c46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e66c4b, SUC_lexicon_tags_1 },
    { 0x2ec66c4c, SUC_lexicon_tags_11 },
    { 0xa4e66c4c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc066c56, SUC_lexicon_tags_37 },
    { 0x3f066c57, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda66c5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97066c5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ea66c5e, SUC_lexicon_tags_26 },
    { 0xfd866c5f, SUC_lexicon_tags_17 },
    { 0x11466c5f, SUC_lexicon_tags_12 },
    { 0xb7066c61, SUC_lexicon_tags_0 },
    { 0x94e66c61, SUC_lexicon_tags_17 },
    { 0x9a066c63, SUC_lexicon_tags_190 },
    { 0xa8266c64, SUC_lexicon_tags_11 },
    { 0x5c66c65, SUC_lexicon_tags_7 },
    { 0x7b266c66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada66c6d, SUC_lexicon_tags_60 },
    { 0x89866c6e, SUC_lexicon_tags_60 },
    { 0x9d066c6f, SUC_lexicon_tags_17 },
    { 0xea266c70, SUC_lexicon_tags_38 },
    { 0x49666c6f, SUC_lexicon_tags_65 },
    { 0x3fa66c72, SUC_lexicon_tags_7 },
    { 0xcbc66c6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a66c7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c66c83, SUC_lexicon_tags_12 },
    { 0x8da66c84, SUC_lexicon_tags_25 },
    { 0x64866c85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea66c89, SUC_lexicon_tags_2 },
    { 0xf7066c89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65466c8e, SUC_lexicon_tags_32 },
    { 0x6266c8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d866c93, SUC_lexicon_tags_23 },
    { 0x8a666c94, SUC_lexicon_tags_92 },
    { 0x96466c93, SUC_lexicon_tags_43 },
    { 0x33c66c96, SUC_lexicon_tags_12 },
    { 0x71e66c93, SUC_lexicon_tags_12 },
    { 0x9d866c93, SUC_lexicon_tags_60 },
    { 0x12c66c95, SUC_lexicon_tags_15 },
    { 0x40866c97, SUC_lexicon_tags_19 },
    { 0x5266c9b, SUC_lexicon_tags_17 },
    { 0xcae66c9c, SUC_lexicon_tags_60 },
    { 0x9ac66c9d, SUC_lexicon_tags_7 },
    { 0x5bc66c9e, SUC_lexicon_tags_151 },
    { 0x99466c9f, SUC_lexicon_tags_43 },
    { 0xeb866c9f, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0x96266ca2, SUC_lexicon_tags_15 },
    { 0xd3e66ca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73066ca9, SUC_lexicon_tags_0 },
    { 0x5ea66caa, SUC_lexicon_tags_25 },
    { 0x18466cab, SUC_lexicon_tags_62 },
    { 0x5cc66caa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a66caf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c66cb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f666cb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e66cbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42266cbf, SUC_lexicon_tags_37 },
    { 0x3f866cc0, SUC_lexicon_tags_30 },
    { 0xfc466cc1, SUC_lexicon_tags_37 },
    { 0xb7666cc2, SUC_lexicon_tags_77 },
    { 0xc0266cc0, SUC_lexicon_tags_48 },
    { 0xc1c66cc2, SUC_lexicon_tags_40 },
    { 0x64e66cc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a266cca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a66cd2, SUC_lexicon_tags_84 },
    { 0xebc66cd2, SUC_lexicon_tags_19 },
    { 0x36666cd4, SUC_lexicon_tags_61 },
    { 0x6a666cd4, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0x25866cd7, SUC_lexicon_tags_17 },
    { 0xe3a66cd8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a66cdd, SUC_lexicon_tags_37 },
    { 0xe6266cdd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83266ce3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc66ce6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb266cee, SUC_lexicon_tags_47 },
    { 0x5266cef, SUC_lexicon_tags_15 },
    { 0x3e866cf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60666cf2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x72066cf5, SUC_lexicon_tags_14 },
    { 0x9466cf6, SUC_lexicon_tags_9 },
    { 0xcc066cf7, SUC_lexicon_tags_11 },
    { 0xe9066cf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d266cfd, SUC_lexicon_tags_8 },
    { 0x4d066cfd, SUC_lexicon_tags_11 },
    { 0x43e66cff, SUC_lexicon_tags_18 },
    { 0x55a66d00, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5066d03, SUC_lexicon_tags_29 },
    { 0x31a66d04, SUC_lexicon_tags_43 },
    { 0x73066d03, SUC_lexicon_tags_82 },
    { 0xefa66d03, SUC_lexicon_tags_47 },
    { 0x15e66d07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85666d0b, SUC_lexicon_tags_87 },
    { 0xe7c66d0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e66d11, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a466d15, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c66d1b, SUC_lexicon_tags_0 },
    { 0x7cc66d1c, SUC_lexicon_tags_17 },
    { 0xeb466d1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe266d22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21266d26, SUC_lexicon_tags_30 },
    { 0x94a66d26, SUC_lexicon_tags_47 },
    { 0x43866d28, SUC_lexicon_tags_61 },
    { 0xba466d29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae866d2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99866d2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11066d31, SUC_lexicon_tags_17 },
    { 0xfa066d31, SUC_lexicon_tags_2 },
    { 0x69c66d33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x32266d35, SUC_lexicon_tags_12 },
    { 0xda066d36, SUC_lexicon_tags_8 },
    { 0x27066d37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23066d3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b666d3d, SUC_lexicon_tags_160 },
    { 0x68066d3d, SUC_lexicon_tags_16 },
    { 0xa9866d3d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca66d42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac66d4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c66d4d, SUC_lexicon_tags_11 },
    { 0xbca66d4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe66d50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13866d54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee466d5e, SUC_lexicon_tags_24 },
    { 0x17e66d5f, SUC_lexicon_tags_1 },
    { 0x39666d60, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x72666d63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c066d65, SUC_lexicon_tags_17 },
    { 0xc4466d66, SUC_lexicon_tags_0 },
    { 0x9c466d65, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0x14066d6a, SUC_lexicon_tags_20 },
    { 0x34266d6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20466d71, SUC_lexicon_tags_19 },
    { 0x2f266d71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x28466d75, SUC_lexicon_tags_69 },
    { 0xdcc66d76, SUC_lexicon_tags_12 },
    { 0x44866d76, SUC_lexicon_tags_14 },
    { 0x5b466d78, SUC_lexicon_tags_3 },
    { 0xcd266d78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30466d7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5466d7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e266d82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c66d85, SUC_lexicon_tags_1 },
    { 0x5e666d86, SUC_lexicon_tags_37 },
    { 0xc9c66d85, SUC_lexicon_tags_15 },
    { 0x4d666d88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc66d8b, SUC_lexicon_tags_57 },
    { 0x27466d8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c266d8f, SUC_lexicon_tags_14 },
    { 0xe7866d8f, SUC_lexicon_tags_1 },
    { 0x37e66d8f, SUC_lexicon_tags_35 },
    { 0x3466d92, SUC_lexicon_tags_8 },
    { 0x21466d93, SUC_lexicon_tags_37 },
    { 0xbc266d92, SUC_lexicon_tags_12 },
    { 0x9a866d95, SUC_lexicon_tags_11 },
    { 0xd9c66d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ea66d98, SUC_lexicon_tags_17 },
    { 0xf5066d99, SUC_lexicon_tags_63 },
    { 0xb3066d9a, SUC_lexicon_tags_11 },
    { 0xa2066d98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b066d9e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xac666da0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98866da6, SUC_lexicon_tags_60 },
    { 0x4c66da7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc66dab, SUC_lexicon_tags_8 },
    { 0xc1066dac, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x73c66dae, SUC_lexicon_tags_19 },
    { 0x63466daf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9066db1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c66db5, SUC_lexicon_tags_43 },
    { 0xb7e66db6, SUC_lexicon_tags_11 },
    { 0x8ee66db6, SUC_lexicon_tags_12 },
    { 0x32466db8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9c66dba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26466dc1, SUC_lexicon_tags_37 },
    { 0xb7c66dc1, SUC_lexicon_tags_4 },
    { 0x84066dc1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb266dc5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32266dcb, SUC_lexicon_tags_17 },
    { 0xb7666dcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d866dd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca66ddc, SUC_lexicon_tags_6 },
    { 0x63066ddd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64266de3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x43066de6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6666deb, SUC_lexicon_tags_37 },
    { 0x8cc66dec, SUC_lexicon_tags_17 },
    { 0xec466ded, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56c66def, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80266df8, SUC_lexicon_tags_30 },
    { 0xc8466df9, SUC_lexicon_tags_10 },
    { 0xbbc66dfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40066dfc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c66dff, SUC_lexicon_tags_1 },
    { 0xec066e00, SUC_lexicon_tags_103 },
    { 0xc9a66e00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c66e04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x98866e06, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcb066e08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d266e0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x83a66e0e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3266e10, SUC_lexicon_tags_137 },
    { 0xaea66e11, SUC_lexicon_tags_29 },
    { 0xe1a66e12, SUC_lexicon_tags_17 },
    { 0x88466e11, SUC_lexicon_tags_26 },
    { 0x64e66e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5866e16, SUC_lexicon_tags_47 },
    { 0x5e066e17, SUC_lexicon_tags_25 },
    { 0x47e66e17, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44066e1d, SUC_lexicon_tags_2 },
    { 0xa6266e1e, SUC_lexicon_tags_11 },
    { 0x2c266e1f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d266e21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0066e25, SUC_lexicon_tags_38 },
    { 0x10c66e26, SUC_lexicon_tags_17 },
    { 0x30266e26, SUC_lexicon_tags_15 },
    { 0xf666e28, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x90666e2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e466e34, SUC_lexicon_tags_26 },
    { 0xc1c66e35, SUC_lexicon_tags_31 },
    { 0x6da66e35, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0xebe66e38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5f266e3a, SUC_lexicon_tags_17 },
    { 0x9f666e3b, SUC_lexicon_tags_33 },
    { 0x9fa66e3c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5666e43, SUC_lexicon_tags_48 },
    { 0x36066e44, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0266e48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe66e4b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c066e4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e866e50, SUC_lexicon_tags_8 },
    { 0x35266e51, SUC_lexicon_tags_37 },
    { 0xe7666e51, SUC_lexicon_tags_38 },
    { 0xabc66e53, SUC_lexicon_tags_12 },
    { 0xa4c66e54, SUC_lexicon_tags_30 },
    { 0x9cc66e53, SUC_lexicon_tags_28 },
    { 0x77c66e56, SUC_lexicon_tags_39 },
    { 0x69c66e57, SUC_lexicon_tags_6 },
    { 0x68666e57, SUC_lexicon_tags_39 },
    { 0x45c66e58, SUC_lexicon_tags_25 },
    { 0xc8e66e5a, SUC_lexicon_tags_60 },
    { 0x5b666e5b, SUC_lexicon_tags_26 },
    { 0xf0c66e59, SUC_lexicon_tags_9 },
    { 0xc8c66e5d, SUC_lexicon_tags_33 },
    { 0xe6a66e5e, SUC_lexicon_tags_65 },
    { 0x4ac66e54, SUC_lexicon_tags_2 },
    { 0x5b466e50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc4066e62, SUC_lexicon_tags_16 },
    { 0xa6a66e63, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba66e69, SUC_lexicon_tags_37 },
    { 0x4ca66e69, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9266e6e, SUC_lexicon_tags_23 },
    { 0x75a66e6f, SUC_lexicon_tags_37 },
    { 0x4b666e70, SUC_lexicon_tags_17 },
    { 0xf0866e71, SUC_lexicon_tags_15 },
    { 0x8f466e71, SUC_lexicon_tags_33 },
    { 0xcd466e73, SUC_lexicon_tags_16 },
    { 0xfba66e74, SUC_lexicon_tags_11 },
    { 0x19e66e75, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a66e78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a66e7b, SUC_lexicon_tags_12 },
    { 0xb4066e7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5066e82, SUC_lexicon_tags_33 },
    { 0x52c66e83, SUC_lexicon_tags_6 },
    { 0x2b066e84, SUC_lexicon_tags_23 },
    { 0x5e866e85, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x32466e87, SUC_lexicon_tags_4 },
    { 0xc66e88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c66e8c, SUC_lexicon_tags_23 },
    { 0x9e666e8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc66e90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c066e96, SUC_lexicon_tags_25 },
    { 0x11466e96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x41c66e99, SUC_lexicon_tags_38 },
    { 0x96c66e9a, SUC_lexicon_tags_23 },
    { 0x66c66e99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c66e9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xee466ea1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x66466ea3, SUC_lexicon_tags_43 },
    { 0x47466ea4, SUC_lexicon_tags_23 },
    { 0xc0866ea5, SUC_lexicon_tags_2 },
    { 0xa8e66ea6, SUC_lexicon_tags_11 },
    { 0xb4e66ea5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56e66ea9, SUC_lexicon_tags_1 },
    { 0x95466ea9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2466eac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d466eaf, SUC_lexicon_tags_151 },
    { 0x2a866eb0, SUC_lexicon_tags_23 },
    { 0x1d666eaf, SUC_lexicon_tags_14 },
    { 0x33866eb1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeb666eb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3666eb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79e66ebb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e466ec2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b066ec5, SUC_lexicon_tags_27 },
    { 0xae466ec6, SUC_lexicon_tags_38 },
    { 0x7c266ec7, SUC_lexicon_tags_47 },
    { 0x60666ec5, SUC_lexicon_tags_45 },
    { 0x8e066ec5, SUC_lexicon_tags_17 },
    { 0x24866ec8, SUC_lexicon_tags_11 },
    { 0x81266ec6, SUC_lexicon_tags_2 },
    { 0x8f666ecc, SUC_lexicon_tags_11 },
    { 0xa266ecd, SUC_lexicon_tags_1 },
    { 0x1f266ece, SUC_lexicon_tags_19 },
    { 0x74266ecf, SUC_lexicon_tags_37 },
    { 0xb5e66ec7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a866ed2, SUC_lexicon_tags_0 },
    { 0xfea66ed2, SUC_lexicon_tags_9 },
    { 0x18666ed4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3066ede, SUC_lexicon_tags_2 },
    { 0x8d466edf, SUC_lexicon_tags_17 },
    { 0x45e66edf, SUC_lexicon_tags_151 },
    { 0x65266ee0, SUC_lexicon_tags_14 },
    { 0x2e66ee2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5466ee4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbba66ee6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e266ee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0266eec, SUC_lexicon_tags_27 },
    { 0xb4866eed, SUC_lexicon_tags_2 },
    { 0xa4a66eee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2666ef0, SUC_lexicon_tags_6 },
    { 0x6ee66ef1, SUC_lexicon_tags_113 },
    { 0x80a66ef2, SUC_lexicon_tags_923 },
    { 0x5be66ef3, SUC_lexicon_tags_756 },
    { 0xfca66ef0, SUC_lexicon_tags_34 },
    { 0x39c66ef4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b066ef8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea66efa, SUC_lexicon_tags_49 },
    { 0x9b866efa, SUC_lexicon_tags_14 },
    { 0x2666efa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x90866eff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c66f06, SUC_lexicon_tags_14 },
    { 0x8a466f07, SUC_lexicon_tags_33 },
    { 0xe4c66f08, SUC_lexicon_tags_46 },
    { 0xd1866f08, SUC_lexicon_tags_39 },
    { 0x27a66f09, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x77666f0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9ba66f0e, SUC_lexicon_tags_47 },
    { 0x14c66f0e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x27266f12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad466f1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e666f1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4066f21, SUC_lexicon_tags_49 },
    { 0x9c466f22, SUC_lexicon_tags_29 },
    { 0xaa466f21, SUC_lexicon_tags_38 },
    { 0xe8c66f24, SUC_lexicon_tags_37 },
    { 0x53666f23, SUC_lexicon_tags_0 },
    { 0x1f466f26, SUC_lexicon_tags_27 },
    { 0x66c66f27, SUC_lexicon_tags_17 },
    { 0xba266f28, SUC_lexicon_tags_77 },
    { 0x49c66f27, SUC_lexicon_tags_4 },
    { 0x2b266f27, SUC_lexicon_tags_2 },
    { 0xd9066f2b, SUC_lexicon_tags_49 },
    { 0xec66f22, SUC_lexicon_tags_10 },
    { 0xc5666f25, SUC_lexicon_tags_14 },
    { 0x60266f2e, SUC_lexicon_tags_3 },
    { 0xb6066f2f, SUC_lexicon_tags_25 },
    { 0xcea66f2e, SUC_lexicon_tags_1 },
    { 0xf3866f31, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5aa66f33, SUC_lexicon_tags_4 },
    { 0x85a66f33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f466f39, SUC_lexicon_tags_1216 },
    { 0x37266f3a, SUC_lexicon_tags_2 },
    { 0xe8866f3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43466f3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8666f43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e66f49, SUC_lexicon_tags_38 },
    { 0x80466f49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3c66f4c, SUC_lexicon_tags_27 },
    { 0x67e66f4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde866f50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45466f56, SUC_lexicon_tags_12 },
    { 0xc6c66f56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a66f5b, SUC_lexicon_tags_11 },
    { 0x5d066f5c, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b866f61, SUC_lexicon_tags_25 },
    { 0xca066f62, SUC_lexicon_tags_11 },
    { 0xb0266f61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9866f66, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6866f6d, SUC_lexicon_tags_29 },
    { 0x4c66f6e, SUC_lexicon_tags_45 },
    { 0xc0466f6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65066f74, SUC_lexicon_tags_16 },
    { 0xa4666f75, SUC_lexicon_tags_63 },
    { 0x12066f74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc66f79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x96666f7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a66f80, SUC_lexicon_tags_12 },
    { 0xada66f81, SUC_lexicon_tags_25 },
    { 0x9d066f81, SUC_lexicon_tags_28 },
    { 0x91666f83, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc66f88, SUC_lexicon_tags_10 },
    { 0x1f666f89, SUC_lexicon_tags_9 },
    { 0xea866f88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84866f8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3466f92, SUC_lexicon_tags_29 },
    { 0xc9e66f93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e666f9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c66fa4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b666fa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e066fa9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47266faf, SUC_lexicon_tags_27 },
    { 0x2e066fb0, SUC_lexicon_tags_2 },
    { 0x94866fb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84066fb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8266fb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a66fb9, SUC_lexicon_tags_37 },
    { 0x65266fba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51266fbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23666fbf, SUC_lexicon_tags_12 },
    { 0xfa066fbf, SUC_lexicon_tags_48 },
    { 0x58866fbf, SUC_lexicon_tags_878 },
    { 0, NULL },
    { 0xb4e66fc3, SUC_lexicon_tags_17 },
    { 0xe3066fc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xde266fc6, SUC_lexicon_tags_25 },
    { 0xdca66fc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6066fc9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c66fcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1066fd2, SUC_lexicon_tags_29 },
    { 0x54e66fd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca66fd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff866fdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83666fdf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe66fe3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4466fe7, SUC_lexicon_tags_3 },
    { 0xcec66fe8, SUC_lexicon_tags_11 },
    { 0x8a066fe9, SUC_lexicon_tags_43 },
    { 0x2b866fea, SUC_lexicon_tags_2 },
    { 0xda666feb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc666fef, SUC_lexicon_tags_47 },
    { 0x5d266ff0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15466ff2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc66ff5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55a66ff7, SUC_lexicon_tags_12 },
    { 0x29666ff7, SUC_lexicon_tags_1 },
    { 0x9be66ff9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da66ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1866ffe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2067001, SUC_lexicon_tags_7 },
    { 0x8267002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5867007, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd7c67009, SUC_lexicon_tags_40 },
    { 0x94a6700a, SUC_lexicon_tags_51 },
    { 0x66e6700a, SUC_lexicon_tags_0 },
    { 0xdd86700c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac86700f, SUC_lexicon_tags_37 },
    { 0x37267010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba67013, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99867015, SUC_lexicon_tags_33 },
    { 0xa5e67016, SUC_lexicon_tags_9 },
    { 0x89667017, SUC_lexicon_tags_48 },
    { 0x78667017, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea6701d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c26701f, SUC_lexicon_tags_17 },
    { 0xba667020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3867023, SUC_lexicon_tags_7 },
    { 0x14467024, SUC_lexicon_tags_17 },
    { 0x85867025, SUC_lexicon_tags_17 },
    { 0x8f667026, SUC_lexicon_tags_26 },
    { 0x6b667025, SUC_lexicon_tags_60 },
    { 0xaa467027, SUC_lexicon_tags_8 },
    { 0xfc067029, SUC_lexicon_tags_30 },
    { 0xfd267024, SUC_lexicon_tags_351 },
    { 0xe6c67024, SUC_lexicon_tags_1 },
    { 0xa9467029, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec67032, SUC_lexicon_tags_31 },
    { 0x9f467033, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57667035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a67039, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10e6703b, SUC_lexicon_tags_60 },
    { 0x3806703b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8ce6703e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83267042, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a06704d, SUC_lexicon_tags_12 },
    { 0xf146704d, SUC_lexicon_tags_28 },
    { 0x18c6704d, SUC_lexicon_tags_57 },
    { 0x4906704f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3867054, SUC_lexicon_tags_7 },
    { 0xec667055, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa67058, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e67066, SUC_lexicon_tags_12 },
    { 0x83c67067, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc67069, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf146706c, SUC_lexicon_tags_31 },
    { 0xd206706c, SUC_lexicon_tags_11 },
    { 0x49c6706e, SUC_lexicon_tags_10 },
    { 0x4346706e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c67074, SUC_lexicon_tags_12 },
    { 0x82e67075, SUC_lexicon_tags_0 },
    { 0xcaa67076, SUC_lexicon_tags_17 },
    { 0xa1667077, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec6707b, SUC_lexicon_tags_60 },
    { 0x5ac6707c, SUC_lexicon_tags_15 },
    { 0x3da6707d, SUC_lexicon_tags_1 },
    { 0x8f46707d, SUC_lexicon_tags_21 },
    { 0x1c06707b, SUC_lexicon_tags_14 },
    { 0xd2a67080, SUC_lexicon_tags_25 },
    { 0xe026707d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2a667083, SUC_lexicon_tags_55 },
    { 0x3e267083, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba67088, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3626708c, SUC_lexicon_tags_12 },
    { 0xbbe6708d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb467092, SUC_lexicon_tags_13 },
    { 0x20267093, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45667096, SUC_lexicon_tags_1 },
    { 0x3ca67097, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x75667099, SUC_lexicon_tags_63 },
    { 0x6e66709a, SUC_lexicon_tags_11 },
    { 0x1ae6709b, SUC_lexicon_tags_38 },
    { 0xbae6709b, SUC_lexicon_tags_19 },
    { 0x1566709d, SUC_lexicon_tags_16 },
    { 0x346709e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x602670a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd54670a2, SUC_lexicon_tags_68 },
    { 0xe8c670a3, SUC_lexicon_tags_25 },
    { 0xe0c670a4, SUC_lexicon_tags_33 },
    { 0x622670a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8670ad, SUC_lexicon_tags_47 },
    { 0x4cc670ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54670b1, SUC_lexicon_tags_3 },
    { 0x6c0670b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2670b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336670ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fc670bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc670bf, SUC_lexicon_tags_47 },
    { 0xff8670bf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x720670c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98670c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb14670ca, SUC_lexicon_tags_47 },
    { 0xe5a670cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40670d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4670d5, SUC_lexicon_tags_6 },
    { 0x620670d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6670e4, SUC_lexicon_tags_23 },
    { 0x184670e5, SUC_lexicon_tags_25 },
    { 0xe96670e6, SUC_lexicon_tags_34 },
    { 0x98670e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716670ee, SUC_lexicon_tags_56 },
    { 0x920670ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc670fa, SUC_lexicon_tags_7 },
    { 0xaa6670fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0670fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21a67100, SUC_lexicon_tags_31 },
    { 0x74e67101, SUC_lexicon_tags_27 },
    { 0xc8067102, SUC_lexicon_tags_14 },
    { 0x63e67103, SUC_lexicon_tags_11 },
    { 0x6a267104, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe67109, SUC_lexicon_tags_7 },
    { 0xbf06710a, SUC_lexicon_tags_0 },
    { 0xe466710b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d667110, SUC_lexicon_tags_2 },
    { 0x4b867110, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x61e67113, SUC_lexicon_tags_7 },
    { 0x23e67113, SUC_lexicon_tags_63 },
    { 0xe8067115, SUC_lexicon_tags_10 },
    { 0x5f867115, SUC_lexicon_tags_32 },
    { 0xde467117, SUC_lexicon_tags_1 },
    { 0x4c867118, SUC_lexicon_tags_6 },
    { 0x61c67114, SUC_lexicon_tags_1 },
    { 0xfc46711a, SUC_lexicon_tags_63 },
    { 0x2f267113, SUC_lexicon_tags_45 },
    { 0x7b267113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed26711f, SUC_lexicon_tags_7 },
    { 0x97a6711f, SUC_lexicon_tags_15 },
    { 0x34667120, SUC_lexicon_tags_8 },
    { 0xd2667122, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d267125, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc67129, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45867132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d86713a, SUC_lexicon_tags_23 },
    { 0xd9a6713b, SUC_lexicon_tags_201 },
    { 0x31a6713c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1526713e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58267143, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x97c67145, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28c67147, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e6714a, SUC_lexicon_tags_26 },
    { 0xf346714b, SUC_lexicon_tags_12 },
    { 0xfb66714c, SUC_lexicon_tags_2 },
    { 0x4de6714c, SUC_lexicon_tags_37 },
    { 0xf5e6714a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49267153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da67157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa267159, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d66715c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a06715f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c67169, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf686716b, SUC_lexicon_tags_61 },
    { 0xc386716b, SUC_lexicon_tags_45 },
    { 0xb586716d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c67170, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e867175, SUC_lexicon_tags_1 },
    { 0xa7867176, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27867178, SUC_lexicon_tags_27 },
    { 0xca267179, SUC_lexicon_tags_15 },
    { 0x3c467178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c6717e, SUC_lexicon_tags_15 },
    { 0x3626717f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4067182, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1067188, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9267191, SUC_lexicon_tags_0 },
    { 0xf6a67191, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7067195, SUC_lexicon_tags_1 },
    { 0x3b467195, SUC_lexicon_tags_37 },
    { 0x18267195, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5906719e, SUC_lexicon_tags_39 },
    { 0x2926719f, SUC_lexicon_tags_15 },
    { 0x40e671a0, SUC_lexicon_tags_17 },
    { 0xaca671a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666671a9, SUC_lexicon_tags_47 },
    { 0xc10671aa, SUC_lexicon_tags_55 },
    { 0xc32671a9, SUC_lexicon_tags_49 },
    { 0xba2671a9, SUC_lexicon_tags_9 },
    { 0x54a671a9, SUC_lexicon_tags_17 },
    { 0xe20671aa, SUC_lexicon_tags_60 },
    { 0xe42671af, SUC_lexicon_tags_7 },
    { 0xd8671af, SUC_lexicon_tags_9 },
    { 0x870671ad, SUC_lexicon_tags_37 },
    { 0xe94671aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0671b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a671bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec671be, SUC_lexicon_tags_17 },
    { 0xcde671bf, SUC_lexicon_tags_29 },
    { 0xa42671bf, SUC_lexicon_tags_35 },
    { 0xef8671c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe8c671c3, SUC_lexicon_tags_11 },
    { 0x3cc671c4, SUC_lexicon_tags_17 },
    { 0x158671c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x656671c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd8671c9, SUC_lexicon_tags_63 },
    { 0xd08671ca, SUC_lexicon_tags_1 },
    { 0xf18671cb, SUC_lexicon_tags_45 },
    { 0x480671cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x238671ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69a671d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96671d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e671d6, SUC_lexicon_tags_7 },
    { 0x41c671d7, SUC_lexicon_tags_1 },
    { 0x174671d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e671dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14c671de, SUC_lexicon_tags_51 },
    { 0x93c671df, SUC_lexicon_tags_37 },
    { 0x8a4671e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b2671e2, SUC_lexicon_tags_15 },
    { 0xd7c671e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x606671e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e671ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x510671ec, SUC_lexicon_tags_12 },
    { 0x84c671ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6671f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916671f9, SUC_lexicon_tags_15 },
    { 0xca671f9, SUC_lexicon_tags_14 },
    { 0xf14671fa, SUC_lexicon_tags_25 },
    { 0x7d2671fc, SUC_lexicon_tags_2 },
    { 0xd7c671fd, SUC_lexicon_tags_11 },
    { 0x7f6671fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6da67200, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1667202, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff067206, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a26720d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6467211, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13067216, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43067218, SUC_lexicon_tags_25 },
    { 0x1a867219, SUC_lexicon_tags_2 },
    { 0x92c67219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3686721f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae67222, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a067227, SUC_lexicon_tags_4 },
    { 0x87e67228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f26722e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba267230, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34267235, SUC_lexicon_tags_60 },
    { 0x36c67236, SUC_lexicon_tags_33 },
    { 0x85867235, SUC_lexicon_tags_1 },
    { 0xbea67238, SUC_lexicon_tags_45 },
    { 0x9e467236, SUC_lexicon_tags_14 },
    { 0xe2867236, SUC_lexicon_tags_46 },
    { 0xb5e6723b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce667241, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5267244, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c67249, SUC_lexicon_tags_6 },
    { 0x64c6724a, SUC_lexicon_tags_19 },
    { 0x986724b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9e6724d, SUC_lexicon_tags_49 },
    { 0x9a86724e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a67251, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0467253, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7186725b, SUC_lexicon_tags_23 },
    { 0x7306725b, SUC_lexicon_tags_5 },
    { 0x5c06725d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e67261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a67263, SUC_lexicon_tags_33 },
    { 0x3da67264, SUC_lexicon_tags_14 },
    { 0x3ce67263, SUC_lexicon_tags_37 },
    { 0x2f867266, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec6726a, SUC_lexicon_tags_14 },
    { 0xffc6726b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8526726e, SUC_lexicon_tags_47 },
    { 0x38c6726e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x79667271, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e67275, SUC_lexicon_tags_68 },
    { 0xc7a67275, SUC_lexicon_tags_5 },
    { 0xb5e67277, SUC_lexicon_tags_25 },
    { 0x8bc67277, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7526727f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1067282, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4467285, SUC_lexicon_tags_11 },
    { 0xafc67286, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa67289, SUC_lexicon_tags_1 },
    { 0xb5e6728a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a6728e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a67292, SUC_lexicon_tags_33 },
    { 0x6d667293, SUC_lexicon_tags_100 },
    { 0xe0467293, SUC_lexicon_tags_29 },
    { 0x55c67295, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8946729a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4786729e, SUC_lexicon_tags_29 },
    { 0xd346729e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c672a3, SUC_lexicon_tags_7 },
    { 0xbce672a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81e672a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc672ab, SUC_lexicon_tags_583 },
    { 0x250672ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9d2672ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x380672b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad4672b2, SUC_lexicon_tags_17 },
    { 0x9a4672b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x204672b6, SUC_lexicon_tags_47 },
    { 0xda0672b6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa92672b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac672bd, SUC_lexicon_tags_67 },
    { 0x8de672be, SUC_lexicon_tags_15 },
    { 0x71a672bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x316672c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2672c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864672c8, SUC_lexicon_tags_11 },
    { 0xd64672c9, SUC_lexicon_tags_4 },
    { 0x142672ca, SUC_lexicon_tags_27 },
    { 0xc8a672cb, SUC_lexicon_tags_17 },
    { 0x2d6672c9, SUC_lexicon_tags_1 },
    { 0x99e672cb, SUC_lexicon_tags_9 },
    { 0x53c672ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2672d0, SUC_lexicon_tags_57 },
    { 0x388672d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x460672d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed2672d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec672d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e672db, SUC_lexicon_tags_7 },
    { 0x3fe672dc, SUC_lexicon_tags_11 },
    { 0x97c672db, SUC_lexicon_tags_1 },
    { 0x1a0672de, SUC_lexicon_tags_46 },
    { 0xdb0672dc, SUC_lexicon_tags_15 },
    { 0x88c672dc, SUC_lexicon_tags_0 },
    { 0xb56672e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc672e5, SUC_lexicon_tags_16 },
    { 0xfda672e6, SUC_lexicon_tags_7 },
    { 0x6f8672e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x308672e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06672ec, SUC_lexicon_tags_148 },
    { 0xb04672ed, SUC_lexicon_tags_12 },
    { 0xde4672ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc52672f0, SUC_lexicon_tags_1 },
    { 0x2ae672f1, SUC_lexicon_tags_65 },
    { 0xa4a672f1, SUC_lexicon_tags_49 },
    { 0x2ba672f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba672f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3be672fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8672fd, SUC_lexicon_tags_8 },
    { 0x7dc672fe, SUC_lexicon_tags_15 },
    { 0x600672fd, SUC_lexicon_tags_30 },
    { 0xd04672fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e467302, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b06730e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e667312, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xec67314, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e67318, SUC_lexicon_tags_60 },
    { 0x4267319, SUC_lexicon_tags_12 },
    { 0xcea6731a, SUC_lexicon_tags_1 },
    { 0x8a46731b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95c6731d, SUC_lexicon_tags_15 },
    { 0x8ee6731d, SUC_lexicon_tags_43 },
    { 0x10c6731f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe267322, SUC_lexicon_tags_37 },
    { 0x67867323, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4a67325, SUC_lexicon_tags_25 },
    { 0xb1067326, SUC_lexicon_tags_34 },
    { 0x60a67327, SUC_lexicon_tags_11 },
    { 0x2d867328, SUC_lexicon_tags_151 },
    { 0x2e667329, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8506732c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c6732f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2867333, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a667336, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e6733a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb666733c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa6733f, SUC_lexicon_tags_2 },
    { 0xf067340, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a467347, SUC_lexicon_tags_2 },
    { 0x4067348, SUC_lexicon_tags_17 },
    { 0x62467348, SUC_lexicon_tags_12 },
    { 0x20a6734a, SUC_lexicon_tags_15 },
    { 0xf5c6734a, SUC_lexicon_tags_485 },
    { 0xa566734a, SUC_lexicon_tags_37 },
    { 0x2d86734a, SUC_lexicon_tags_49 },
    { 0xd66734e, SUC_lexicon_tags_13 },
    { 0xad26734f, SUC_lexicon_tags_1 },
    { 0x4067349, SUC_lexicon_tags_48 },
    { 0xf3467351, SUC_lexicon_tags_39 },
    { 0xf7867351, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7867354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe6735c, SUC_lexicon_tags_12 },
    { 0xca46735d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1486735f, SUC_lexicon_tags_1 },
    { 0x3fa67360, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf8e67362, SUC_lexicon_tags_37 },
    { 0xfbc67363, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x20067365, SUC_lexicon_tags_0 },
    { 0xa9867365, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb6667368, SUC_lexicon_tags_55 },
    { 0xc67368, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf06736e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc67374, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x27667377, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9d267379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a6737b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd246737e, SUC_lexicon_tags_84 },
    { 0x7206737f, SUC_lexicon_tags_12 },
    { 0x1fa6737f, SUC_lexicon_tags_14 },
    { 0x4b467381, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30667384, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5a67386, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8667388, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c6738c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf667391, SUC_lexicon_tags_7 },
    { 0x6a267392, SUC_lexicon_tags_43 },
    { 0xf8067392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76c67395, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36a67397, SUC_lexicon_tags_25 },
    { 0xa0e67398, SUC_lexicon_tags_45 },
    { 0xbf067398, SUC_lexicon_tags_14 },
    { 0xc606739a, SUC_lexicon_tags_141 },
    { 0x1d06739a, SUC_lexicon_tags_25 },
    { 0x2bc67399, SUC_lexicon_tags_14 },
    { 0x2866739b, SUC_lexicon_tags_49 },
    { 0x88a6739a, SUC_lexicon_tags_8 },
    { 0x45a6739a, SUC_lexicon_tags_37 },
    { 0xd5067397, SUC_lexicon_tags_26 },
    { 0x3e2673a1, SUC_lexicon_tags_1 },
    { 0xa1c67398, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c8673a4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0673a8, SUC_lexicon_tags_12 },
    { 0x67a673a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8673ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2673b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x992673b4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec673b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02673bc, SUC_lexicon_tags_49 },
    { 0xc66673bc, SUC_lexicon_tags_30 },
    { 0xab2673be, SUC_lexicon_tags_37 },
    { 0x198673bf, SUC_lexicon_tags_15 },
    { 0x40c673bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6c673c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd4673c4, SUC_lexicon_tags_15 },
    { 0xc3c673c5, SUC_lexicon_tags_25 },
    { 0x54673c4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c673d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4673d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a673dd, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6673e0, SUC_lexicon_tags_12 },
    { 0xe4673e0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980673e7, SUC_lexicon_tags_43 },
    { 0x334673e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0e673ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe673ef, SUC_lexicon_tags_0 },
    { 0x9c8673f0, SUC_lexicon_tags_7 },
    { 0x56c673f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6673f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4c673f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14673fb, SUC_lexicon_tags_900 },
    { 0, NULL },
    { 0xde0673fd, SUC_lexicon_tags_2 },
    { 0xfb0673fe, SUC_lexicon_tags_29 },
    { 0xb52673ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4267409, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd146740b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc67412, SUC_lexicon_tags_60 },
    { 0x81c67412, SUC_lexicon_tags_27 },
    { 0x7e267414, SUC_lexicon_tags_43 },
    { 0xdfe67415, SUC_lexicon_tags_27 },
    { 0xa1467416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c67419, SUC_lexicon_tags_25 },
    { 0x1fc6741a, SUC_lexicon_tags_32 },
    { 0x6726741b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3266742a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6526742c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e26742e, SUC_lexicon_tags_60 },
    { 0x57c6742f, SUC_lexicon_tags_11 },
    { 0x55867430, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0267435, SUC_lexicon_tags_15 },
    { 0x20067435, SUC_lexicon_tags_14 },
    { 0x31067437, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xddc67439, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2426743b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8406743f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e67445, SUC_lexicon_tags_45 },
    { 0xaa67445, SUC_lexicon_tags_62 },
    { 0x83a67447, SUC_lexicon_tags_9 },
    { 0xc0667447, SUC_lexicon_tags_43 },
    { 0xd2c67449, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa446744b, SUC_lexicon_tags_31 },
    { 0xd1a6744c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a66744f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf067455, SUC_lexicon_tags_25 },
    { 0xbd467456, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74c67458, SUC_lexicon_tags_15 },
    { 0x63067459, SUC_lexicon_tags_91 },
    { 0xd9467458, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a46745c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d26745e, SUC_lexicon_tags_27 },
    { 0x67a6745f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a67462, SUC_lexicon_tags_15 },
    { 0x57467463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50867466, SUC_lexicon_tags_29 },
    { 0xb6867466, SUC_lexicon_tags_17 },
    { 0x10c67467, SUC_lexicon_tags_27 },
    { 0xa1667469, SUC_lexicon_tags_63 },
    { 0xb586746a, SUC_lexicon_tags_23 },
    { 0x96e6746b, SUC_lexicon_tags_29 },
    { 0x94667466, SUC_lexicon_tags_68 },
    { 0xe6e6746d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c67471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39e67473, SUC_lexicon_tags_68 },
    { 0x52c67473, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99067478, SUC_lexicon_tags_31 },
    { 0x7f467478, SUC_lexicon_tags_29 },
    { 0xae86747a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf8a6747c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc306747e, SUC_lexicon_tags_11 },
    { 0x5f86747f, SUC_lexicon_tags_11 },
    { 0xb0e67480, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff467487, SUC_lexicon_tags_37 },
    { 0x26667488, SUC_lexicon_tags_41 },
    { 0x7ae67488, SUC_lexicon_tags_14 },
    { 0x2f86748a, SUC_lexicon_tags_21 },
    { 0x1e467487, SUC_lexicon_tags_17 },
    { 0x5166748c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29667492, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9267497, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6c67499, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a6749c, SUC_lexicon_tags_1 },
    { 0xcb46749c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc674a0, SUC_lexicon_tags_900 },
    { 0xbba674a1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc674a8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4674ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2674b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe16674b2, SUC_lexicon_tags_1 },
    { 0xd98674b3, SUC_lexicon_tags_11 },
    { 0xf68674b3, SUC_lexicon_tags_8 },
    { 0xf6674b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96674b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898674c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60a674c5, SUC_lexicon_tags_47 },
    { 0xef0674c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242674ca, SUC_lexicon_tags_97 },
    { 0x5fe674cb, SUC_lexicon_tags_1 },
    { 0xe5e674cc, SUC_lexicon_tags_7 },
    { 0x23a674cd, SUC_lexicon_tags_33 },
    { 0x7e2674cc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5a8674d0, SUC_lexicon_tags_15 },
    { 0x5e8674d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa4674d3, SUC_lexicon_tags_27 },
    { 0x38e674d3, SUC_lexicon_tags_29 },
    { 0x468674d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16e674d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8674e0, SUC_lexicon_tags_0 },
    { 0xbda674e0, SUC_lexicon_tags_31 },
    { 0x57a674e1, SUC_lexicon_tags_15 },
    { 0x4ea674e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48674e6, SUC_lexicon_tags_57 },
    { 0xcb2674e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x926674e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1aa674eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x626674ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808674f1, SUC_lexicon_tags_1 },
    { 0x4e8674f1, SUC_lexicon_tags_63 },
    { 0xe26674f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc94674f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x574674f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x918674f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x63a674fb, SUC_lexicon_tags_31 },
    { 0xe86674fb, SUC_lexicon_tags_11 },
    { 0x638674fd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a67504, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6750a, SUC_lexicon_tags_42 },
    { 0x7d06750b, SUC_lexicon_tags_17 },
    { 0xbcc6750c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2e6750e, SUC_lexicon_tags_2 },
    { 0xb26750e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47867516, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc646751b, SUC_lexicon_tags_6 },
    { 0x4906751b, SUC_lexicon_tags_27 },
    { 0x2a26751b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca667520, SUC_lexicon_tags_1 },
    { 0xb8467521, SUC_lexicon_tags_63 },
    { 0xeb467520, SUC_lexicon_tags_60 },
    { 0xa7c67523, SUC_lexicon_tags_27 },
    { 0x82667522, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a67529, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb26752b, SUC_lexicon_tags_63 },
    { 0x16e6752b, SUC_lexicon_tags_38 },
    { 0xb226752b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa667530, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3667532, SUC_lexicon_tags_38 },
    { 0x56a67533, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a067536, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe3867538, SUC_lexicon_tags_25 },
    { 0xbfa67539, SUC_lexicon_tags_12 },
    { 0xfac6753a, SUC_lexicon_tags_29 },
    { 0x14867539, SUC_lexicon_tags_26 },
    { 0x1cc6753c, SUC_lexicon_tags_48 },
    { 0x3fe6753d, SUC_lexicon_tags_9 },
    { 0xc1267538, SUC_lexicon_tags_37 },
    { 0xc5e6753f, SUC_lexicon_tags_60 },
    { 0x10867540, SUC_lexicon_tags_12 },
    { 0xe0e67541, SUC_lexicon_tags_19 },
    { 0x3ea67541, SUC_lexicon_tags_12 },
    { 0x50e67543, SUC_lexicon_tags_11 },
    { 0xcec67544, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x17867547, SUC_lexicon_tags_165 },
    { 0x62a67547, SUC_lexicon_tags_37 },
    { 0x82467549, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5a6754b, SUC_lexicon_tags_11 },
    { 0x8066754b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c67553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc467557, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c6755b, SUC_lexicon_tags_26 },
    { 0xbe06755c, SUC_lexicon_tags_37 },
    { 0xb6a6755c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c6756c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa267570, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70e67572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd667576, SUC_lexicon_tags_42 },
    { 0x2f267577, SUC_lexicon_tags_4 },
    { 0x9ba67578, SUC_lexicon_tags_42 },
    { 0xbcc67579, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x95a6757b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6126757d, SUC_lexicon_tags_49 },
    { 0xde06757d, SUC_lexicon_tags_11 },
    { 0x45a6757d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a867581, SUC_lexicon_tags_11 },
    { 0x89067582, SUC_lexicon_tags_27 },
    { 0x2d467583, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e67588, SUC_lexicon_tags_10 },
    { 0xaac67589, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c46758b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a6758f, SUC_lexicon_tags_17 },
    { 0x80a67590, SUC_lexicon_tags_4 },
    { 0xc5267591, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x73267593, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0x70e67595, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x92667598, SUC_lexicon_tags_16 },
    { 0x25e67598, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e86759b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c6759f, SUC_lexicon_tags_15 },
    { 0xa546759f, SUC_lexicon_tags_25 },
    { 0x742675a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec675a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6675a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c675ac, SUC_lexicon_tags_13 },
    { 0x816675ad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4675b7, SUC_lexicon_tags_57 },
    { 0x23c675b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136675bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e4675be, SUC_lexicon_tags_46 },
    { 0x8cc675bf, SUC_lexicon_tags_49 },
    { 0x5b4675c0, SUC_lexicon_tags_8 },
    { 0x14c675c1, SUC_lexicon_tags_17 },
    { 0x4be675be, SUC_lexicon_tags_37 },
    { 0xde0675c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16c675c5, SUC_lexicon_tags_27 },
    { 0x478675c6, SUC_lexicon_tags_47 },
    { 0x822675c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e675cc, SUC_lexicon_tags_55 },
    { 0x302675cd, SUC_lexicon_tags_25 },
    { 0x694675ce, SUC_lexicon_tags_14 },
    { 0xb5e675cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c675d5, SUC_lexicon_tags_12 },
    { 0xf88675d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4e675d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e675db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6675e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74675e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26e675e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e675ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96675ef, SUC_lexicon_tags_7 },
    { 0x398675ef, SUC_lexicon_tags_32 },
    { 0xea6675f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6675f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x910675fc, SUC_lexicon_tags_12 },
    { 0xc8a675fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c67600, SUC_lexicon_tags_9 },
    { 0xfb867601, SUC_lexicon_tags_25 },
    { 0x5f267602, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36067607, SUC_lexicon_tags_208 },
    { 0x76e67607, SUC_lexicon_tags_45 },
    { 0xfcc67609, SUC_lexicon_tags_1 },
    { 0xa3a6760a, SUC_lexicon_tags_49 },
    { 0x8166760b, SUC_lexicon_tags_1 },
    { 0xbae6760c, SUC_lexicon_tags_17 },
    { 0xb46760b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa67611, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b867614, SUC_lexicon_tags_17 },
    { 0xe4a67615, SUC_lexicon_tags_12 },
    { 0xa5267616, SUC_lexicon_tags_7 },
    { 0x7b867617, SUC_lexicon_tags_28 },
    { 0xb6e67617, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a06761f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e67624, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd667627, SUC_lexicon_tags_37 },
    { 0x99067628, SUC_lexicon_tags_15 },
    { 0x70a67629, SUC_lexicon_tags_27 },
    { 0x98e6762a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e06762f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ca67631, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9267634, SUC_lexicon_tags_25 },
    { 0x4cc67635, SUC_lexicon_tags_25 },
    { 0x95267634, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5067639, SUC_lexicon_tags_17 },
    { 0x2f46763a, SUC_lexicon_tags_14 },
    { 0xb926763a, SUC_lexicon_tags_32 },
    { 0xe4c6763c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bc6763e, SUC_lexicon_tags_17 },
    { 0xd766763f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6c67641, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6667645, SUC_lexicon_tags_82 },
    { 0x56067646, SUC_lexicon_tags_2 },
    { 0x52467646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdc67649, SUC_lexicon_tags_62 },
    { 0xe3267649, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de67654, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f267656, SUC_lexicon_tags_108 },
    { 0x15067657, SUC_lexicon_tags_49 },
    { 0x38c67658, SUC_lexicon_tags_14 },
    { 0x1d467658, SUC_lexicon_tags_17 },
    { 0x22867658, SUC_lexicon_tags_56 },
    { 0x5de67659, SUC_lexicon_tags_19 },
    { 0xee467656, SUC_lexicon_tags_47 },
    { 0xa206765d, SUC_lexicon_tags_10 },
    { 0x5e66765e, SUC_lexicon_tags_37 },
    { 0x73667659, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8867662, SUC_lexicon_tags_47 },
    { 0xff867663, SUC_lexicon_tags_37 },
    { 0xf0667664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b067666, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a6766a, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x2c26766c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37267673, SUC_lexicon_tags_11 },
    { 0x41667674, SUC_lexicon_tags_15 },
    { 0xcaa67675, SUC_lexicon_tags_30 },
    { 0x27267676, SUC_lexicon_tags_23 },
    { 0x12667677, SUC_lexicon_tags_37 },
    { 0x6dc67674, SUC_lexicon_tags_1 },
    { 0x44667679, SUC_lexicon_tags_28 },
    { 0x92267675, SUC_lexicon_tags_2 },
    { 0xb6667678, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de67680, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd0e67682, SUC_lexicon_tags_4 },
    { 0xcfe67682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a67686, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2d267688, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6286768e, SUC_lexicon_tags_12 },
    { 0xfcc6768e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77e67691, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x91467693, SUC_lexicon_tags_15 },
    { 0xcde67694, SUC_lexicon_tags_201 },
    { 0xeb667695, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x52867697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc867699, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c6769c, SUC_lexicon_tags_11 },
    { 0x25c6769d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0676a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce676a6, SUC_lexicon_tags_25 },
    { 0x120676a7, SUC_lexicon_tags_45 },
    { 0x44e676a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82676b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa676b8, SUC_lexicon_tags_14 },
    { 0xbc8676b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x444676bb, SUC_lexicon_tags_17 },
    { 0x84c676bc, SUC_lexicon_tags_42 },
    { 0xc676bd, SUC_lexicon_tags_37 },
    { 0xf4e676be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x930676c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e676c4, SUC_lexicon_tags_55 },
    { 0x1fc676c5, SUC_lexicon_tags_84 },
    { 0xec8676c5, SUC_lexicon_tags_19 },
    { 0xca2676c7, SUC_lexicon_tags_26 },
    { 0x3e8676c7, SUC_lexicon_tags_15 },
    { 0x39e676c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a676cc, SUC_lexicon_tags_14 },
    { 0x24a676cc, SUC_lexicon_tags_37 },
    { 0xf9a676ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ec676d0, SUC_lexicon_tags_2 },
    { 0x558676d0, SUC_lexicon_tags_17 },
    { 0xa14676d1, SUC_lexicon_tags_19 },
    { 0xbaa676d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4676d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a676e0, SUC_lexicon_tags_7 },
    { 0x1d2676e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe52676e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x332676e6, SUC_lexicon_tags_15 },
    { 0xc8676e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258676ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104676f2, SUC_lexicon_tags_2 },
    { 0x7b4676f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e676f6, SUC_lexicon_tags_60 },
    { 0x522676f7, SUC_lexicon_tags_140 },
    { 0x220676f8, SUC_lexicon_tags_11 },
    { 0x1ac676f9, SUC_lexicon_tags_7 },
    { 0xf90676fa, SUC_lexicon_tags_30 },
    { 0x564676fa, SUC_lexicon_tags_19 },
    { 0xcfa676fb, SUC_lexicon_tags_38 },
    { 0xf58676fd, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc67700, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x97c67702, SUC_lexicon_tags_28 },
    { 0xce867703, SUC_lexicon_tags_37 },
    { 0x43467702, SUC_lexicon_tags_38 },
    { 0x8a067703, SUC_lexicon_tags_31 },
    { 0x36467703, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33667708, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e46770c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f267712, SUC_lexicon_tags_31 },
    { 0xca867712, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa67719, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26771c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31667722, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c46772a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5226772d, SUC_lexicon_tags_23 },
    { 0xaea6772e, SUC_lexicon_tags_32 },
    { 0x4706772f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xebc67731, SUC_lexicon_tags_6 },
    { 0x4b267732, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97867738, SUC_lexicon_tags_2 },
    { 0xd2267739, SUC_lexicon_tags_68 },
    { 0x566773a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc206773c, SUC_lexicon_tags_37 },
    { 0xa526773d, SUC_lexicon_tags_4 },
    { 0x8bc6773e, SUC_lexicon_tags_11 },
    { 0xd286773d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19467743, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8667746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a467749, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd46774c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a267752, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7466775a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf786775e, SUC_lexicon_tags_37 },
    { 0x36c6775e, SUC_lexicon_tags_25 },
    { 0x60267760, SUC_lexicon_tags_87 },
    { 0xb1067760, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fa67763, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b267766, SUC_lexicon_tags_25 },
    { 0xb5067766, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x22667769, SUC_lexicon_tags_57 },
    { 0x2806776a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2467771, SUC_lexicon_tags_30 },
    { 0x70467772, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c67777, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc346777a, SUC_lexicon_tags_1 },
    { 0x2446777b, SUC_lexicon_tags_4 },
    { 0x7826777c, SUC_lexicon_tags_17 },
    { 0xcfe6777d, SUC_lexicon_tags_65 },
    { 0x59e6777e, SUC_lexicon_tags_25 },
    { 0x3b66777b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c667782, SUC_lexicon_tags_10 },
    { 0xe1667783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdbe67785, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf267787, SUC_lexicon_tags_17 },
    { 0x7ac67788, SUC_lexicon_tags_9 },
    { 0x39267789, SUC_lexicon_tags_25 },
    { 0x30c67789, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3806778d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47867790, SUC_lexicon_tags_15 },
    { 0x72267791, SUC_lexicon_tags_27 },
    { 0x1667792, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a67796, SUC_lexicon_tags_2 },
    { 0xb6867796, SUC_lexicon_tags_7 },
    { 0xb1867798, SUC_lexicon_tags_23 },
    { 0xbac67798, SUC_lexicon_tags_1 },
    { 0x18867798, SUC_lexicon_tags_37 },
    { 0x52e6779b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838677a0, SUC_lexicon_tags_17 },
    { 0x51e677a0, SUC_lexicon_tags_70 },
    { 0xa1c677a2, SUC_lexicon_tags_37 },
    { 0xca2677a1, SUC_lexicon_tags_17 },
    { 0xf18677a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7a8677a6, SUC_lexicon_tags_3 },
    { 0xf2a677a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x322677a9, SUC_lexicon_tags_11 },
    { 0xc98677a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdce677ac, SUC_lexicon_tags_38 },
    { 0xf7a677ac, SUC_lexicon_tags_87 },
    { 0x948677ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6677b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x63c677b7, SUC_lexicon_tags_11 },
    { 0x84677b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa66677ba, SUC_lexicon_tags_17 },
    { 0x9f6677ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e677c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x660677c2, SUC_lexicon_tags_37 },
    { 0x338677c2, SUC_lexicon_tags_7 },
    { 0x352677c4, SUC_lexicon_tags_14 },
    { 0x90c677c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506677c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e677ce, SUC_lexicon_tags_25 },
    { 0xc92677cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x294677d1, SUC_lexicon_tags_7 },
    { 0x49e677d1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58677db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x806677dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x538677df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4677e3, SUC_lexicon_tags_32 },
    { 0xd1c677e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4677e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6677ed, SUC_lexicon_tags_59 },
    { 0xbfe677ed, SUC_lexicon_tags_37 },
    { 0x40c677ed, SUC_lexicon_tags_7 },
    { 0x590677f0, SUC_lexicon_tags_11 },
    { 0x2b6677f1, SUC_lexicon_tags_79 },
    { 0xbc4677f0, SUC_lexicon_tags_32 },
    { 0x7fc677f3, SUC_lexicon_tags_11 },
    { 0xc5e677f0, SUC_lexicon_tags_11 },
    { 0x4c2677f3, SUC_lexicon_tags_13 },
    { 0x272677ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37867803, SUC_lexicon_tags_60 },
    { 0x8c267804, SUC_lexicon_tags_37 },
    { 0x92e67804, SUC_lexicon_tags_7 },
    { 0xe4267805, SUC_lexicon_tags_17 },
    { 0xce667807, SUC_lexicon_tags_65 },
    { 0x3a667804, SUC_lexicon_tags_27 },
    { 0x73e67807, SUC_lexicon_tags_38 },
    { 0x3b06780a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x26780c, SUC_lexicon_tags_33 },
    { 0x41e6780d, SUC_lexicon_tags_25 },
    { 0x88e6780c, SUC_lexicon_tags_15 },
    { 0xc4e6780f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab467817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90467819, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46781c, SUC_lexicon_tags_7 },
    { 0x2106781d, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0x9066781f, SUC_lexicon_tags_63 },
    { 0x7de67820, SUC_lexicon_tags_17 },
    { 0x76867821, SUC_lexicon_tags_37 },
    { 0x8be67820, SUC_lexicon_tags_0 },
    { 0x21a67822, SUC_lexicon_tags_6 },
    { 0x80667824, SUC_lexicon_tags_32 },
    { 0x8a67825, SUC_lexicon_tags_2 },
    { 0xdce67826, SUC_lexicon_tags_6 },
    { 0xa5c67827, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e6782b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3046782e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60467833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17467835, SUC_lexicon_tags_12 },
    { 0xc8467836, SUC_lexicon_tags_48 },
    { 0xb667837, SUC_lexicon_tags_23 },
    { 0x1a667838, SUC_lexicon_tags_25 },
    { 0xf267839, SUC_lexicon_tags_15 },
    { 0xbdc67837, SUC_lexicon_tags_11 },
    { 0xdc86783b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e6783e, SUC_lexicon_tags_15 },
    { 0x4306783f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a867847, SUC_lexicon_tags_17 },
    { 0x7b067847, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3806784a, SUC_lexicon_tags_17 },
    { 0xae46784b, SUC_lexicon_tags_2 },
    { 0x926784c, SUC_lexicon_tags_27 },
    { 0xd826784d, SUC_lexicon_tags_0 },
    { 0x3306784b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee267855, SUC_lexicon_tags_11 },
    { 0x42e67856, SUC_lexicon_tags_37 },
    { 0x92a67855, SUC_lexicon_tags_37 },
    { 0x88e67858, SUC_lexicon_tags_2 },
    { 0xdaa67858, SUC_lexicon_tags_0 },
    { 0xfe26785a, SUC_lexicon_tags_11 },
    { 0x2dc67859, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4166785e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe67861, SUC_lexicon_tags_17 },
    { 0xd2067862, SUC_lexicon_tags_37 },
    { 0xd2c67861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40467866, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb446786e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a667877, SUC_lexicon_tags_32 },
    { 0xad267877, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x82a6787a, SUC_lexicon_tags_17 },
    { 0xe66787b, SUC_lexicon_tags_1 },
    { 0xa866787c, SUC_lexicon_tags_48 },
    { 0x74c6787d, SUC_lexicon_tags_4 },
    { 0xeb46787b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x53667880, SUC_lexicon_tags_6 },
    { 0xee467881, SUC_lexicon_tags_25 },
    { 0x58267880, SUC_lexicon_tags_7 },
    { 0xe1e67883, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa0267885, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda467889, SUC_lexicon_tags_49 },
    { 0xb646788a, SUC_lexicon_tags_16 },
    { 0x79a6788a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c67894, SUC_lexicon_tags_11 },
    { 0xc7e67894, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c67899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a26789c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3366789e, SUC_lexicon_tags_30 },
    { 0x526789f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2678a5, SUC_lexicon_tags_33 },
    { 0x3a2678a6, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966678aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6678ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2678b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44678b5, SUC_lexicon_tags_17 },
    { 0x5f0678b6, SUC_lexicon_tags_1141 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e678b9, SUC_lexicon_tags_3 },
    { 0x168678ba, SUC_lexicon_tags_7 },
    { 0x17c678bb, SUC_lexicon_tags_2 },
    { 0xac6678bb, SUC_lexicon_tags_17 },
    { 0x778678bd, SUC_lexicon_tags_25 },
    { 0x898678bd, SUC_lexicon_tags_49 },
    { 0x496678bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x232678c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a678c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290678cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43e678cd, SUC_lexicon_tags_84 },
    { 0xb2678ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c678d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa678d4, SUC_lexicon_tags_1 },
    { 0xa9c678d5, SUC_lexicon_tags_17 },
    { 0xd14678d4, SUC_lexicon_tags_37 },
    { 0x266678d7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc40678d9, SUC_lexicon_tags_38 },
    { 0xd24678da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2678e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa678e5, SUC_lexicon_tags_25 },
    { 0x6d4678e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550678eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522678f1, SUC_lexicon_tags_7 },
    { 0xd2c678f2, SUC_lexicon_tags_19 },
    { 0x6be678f2, SUC_lexicon_tags_30 },
    { 0x30e678f1, SUC_lexicon_tags_23 },
    { 0x190678f5, SUC_lexicon_tags_17 },
    { 0x6f2678f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x694678f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x500678fb, SUC_lexicon_tags_0 },
    { 0x7c678fb, SUC_lexicon_tags_0 },
    { 0xcc0678fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefc678ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10467901, SUC_lexicon_tags_33 },
    { 0x61667902, SUC_lexicon_tags_37 },
    { 0xf9267903, SUC_lexicon_tags_60 },
    { 0x1e067903, SUC_lexicon_tags_37 },
    { 0x4ec67902, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2986790a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53667910, SUC_lexicon_tags_11 },
    { 0x2c467911, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71067913, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29667917, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2d067919, SUC_lexicon_tags_55 },
    { 0x11c6791a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1226791c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9667926, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa67929, SUC_lexicon_tags_21 },
    { 0x4486792a, SUC_lexicon_tags_12 },
    { 0xc946792b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b867932, SUC_lexicon_tags_30 },
    { 0x44c67933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9467935, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x26467937, SUC_lexicon_tags_2 },
    { 0x36c67938, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb46793a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe726793f, SUC_lexicon_tags_25 },
    { 0x9f46793f, SUC_lexicon_tags_37 },
    { 0x5ea67941, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d267946, SUC_lexicon_tags_19 },
    { 0x13667947, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de6794a, SUC_lexicon_tags_12 },
    { 0x7d86794a, SUC_lexicon_tags_17 },
    { 0x7806794b, SUC_lexicon_tags_14 },
    { 0x3c66794a, SUC_lexicon_tags_14 },
    { 0xcf66794e, SUC_lexicon_tags_25 },
    { 0x9886794a, SUC_lexicon_tags_83 },
    { 0xa306794d, SUC_lexicon_tags_1 },
    { 0xce067951, SUC_lexicon_tags_11 },
    { 0x47467952, SUC_lexicon_tags_30 },
    { 0xaba6794f, SUC_lexicon_tags_11 },
    { 0x5cc6794a, SUC_lexicon_tags_91 },
    { 0x7766794c, SUC_lexicon_tags_10 },
    { 0xb6a67956, SUC_lexicon_tags_7 },
    { 0x2ac67953, SUC_lexicon_tags_26 },
    { 0x7b867953, SUC_lexicon_tags_17 },
    { 0xc486794a, SUC_lexicon_tags_9 },
    { 0x3986794e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9366795c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a67961, SUC_lexicon_tags_17 },
    { 0xc6867962, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xef267965, SUC_lexicon_tags_17 },
    { 0xfba67966, SUC_lexicon_tags_27 },
    { 0x6467967, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5166796c, SUC_lexicon_tags_19 },
    { 0x7746796d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3667970, SUC_lexicon_tags_63 },
    { 0x6f467971, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a67978, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2346797c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a26797e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24067980, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7aa67982, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x92a67984, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2c67986, SUC_lexicon_tags_16 },
    { 0xe6867987, SUC_lexicon_tags_11 },
    { 0x28267988, SUC_lexicon_tags_1 },
    { 0x14c67987, SUC_lexicon_tags_46 },
    { 0x50867987, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x89a6798c, SUC_lexicon_tags_57 },
    { 0x1a86798d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f667991, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc5267993, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2526799e, SUC_lexicon_tags_124 },
    { 0xf2c6799f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c679ab, SUC_lexicon_tags_10 },
    { 0x148679ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a679b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a679b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24679ba, SUC_lexicon_tags_60 },
    { 0x8de679bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0679cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e679cf, SUC_lexicon_tags_38 },
    { 0xace679d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da679d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36679de, SUC_lexicon_tags_15 },
    { 0xc48679df, SUC_lexicon_tags_37 },
    { 0x2ca679e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0679e4, SUC_lexicon_tags_25 },
    { 0xfee679e5, SUC_lexicon_tags_25 },
    { 0x1d4679e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556679ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0679f6, SUC_lexicon_tags_82 },
    { 0xf1c679f7, SUC_lexicon_tags_7 },
    { 0x596679f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x88a679fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a679fe, SUC_lexicon_tags_2 },
    { 0xd8c679ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b067a03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2dc67a05, SUC_lexicon_tags_25 },
    { 0x5bc67a06, SUC_lexicon_tags_60 },
    { 0x5067a07, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54c67a09, SUC_lexicon_tags_11 },
    { 0xa4e67a09, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8867a0d, SUC_lexicon_tags_82 },
    { 0x6da67a0e, SUC_lexicon_tags_4 },
    { 0x3d467a0f, SUC_lexicon_tags_170 },
    { 0x3c267a0f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9867a13, SUC_lexicon_tags_620 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da67a19, SUC_lexicon_tags_2 },
    { 0x32067a1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7aa67a1c, SUC_lexicon_tags_56 },
    { 0xe7867a1c, SUC_lexicon_tags_46 },
    { 0x2c867a1e, SUC_lexicon_tags_45 },
    { 0xe667a1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87067a2f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x29e67a31, SUC_lexicon_tags_7 },
    { 0x48c67a31, SUC_lexicon_tags_13 },
    { 0xdb667a33, SUC_lexicon_tags_47 },
    { 0xcc467a33, SUC_lexicon_tags_47 },
    { 0x8be67a35, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a67a3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba667a3e, SUC_lexicon_tags_45 },
    { 0x74467a3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51067a47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a67a4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7667a50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be67a55, SUC_lexicon_tags_10 },
    { 0x4f867a56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaac67a58, SUC_lexicon_tags_47 },
    { 0x9ac67a59, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc2067a5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70667a69, SUC_lexicon_tags_15 },
    { 0x53267a6a, SUC_lexicon_tags_28 },
    { 0x41e67a69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e267a6e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdf667a70, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7467a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0867a78, SUC_lexicon_tags_7 },
    { 0xc5267a79, SUC_lexicon_tags_47 },
    { 0xcd267a78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47467a7f, SUC_lexicon_tags_37 },
    { 0xafa67a80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7067a84, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x24a67a86, SUC_lexicon_tags_12 },
    { 0xeb467a86, SUC_lexicon_tags_12 },
    { 0x81667a87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea67a92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c67a95, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11067a99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c67a9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c67aa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6a67aa4, SUC_lexicon_tags_60 },
    { 0x57c67aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25867aaa, SUC_lexicon_tags_15 },
    { 0xd0867aab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c467aad, SUC_lexicon_tags_12 },
    { 0x6de67aad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3067ab2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ca67ab4, SUC_lexicon_tags_11 },
    { 0x22267ab5, SUC_lexicon_tags_1 },
    { 0xba067ab4, SUC_lexicon_tags_3 },
    { 0x4267ab7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26067ac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48667ac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec67ad8, SUC_lexicon_tags_2 },
    { 0xe8067ad9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c467add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc67ae3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a67ae7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e67aeb, SUC_lexicon_tags_25 },
    { 0xce067aec, SUC_lexicon_tags_19 },
    { 0x60467aec, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a467af2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x50a67af4, SUC_lexicon_tags_37 },
    { 0x8e667af5, SUC_lexicon_tags_25 },
    { 0xebe67af6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46067b00, SUC_lexicon_tags_25 },
    { 0xb1a67b01, SUC_lexicon_tags_15 },
    { 0x17c67b01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92067b04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d867b09, SUC_lexicon_tags_46 },
    { 0x68067b0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fe67b0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c467b0e, SUC_lexicon_tags_13 },
    { 0x84467b0f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xe3667b11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x12667b13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x89067b15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e67b1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43467b21, SUC_lexicon_tags_25 },
    { 0x2be67b21, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da67b26, SUC_lexicon_tags_1 },
    { 0x44e67b26, SUC_lexicon_tags_12 },
    { 0x6e667b28, SUC_lexicon_tags_7 },
    { 0x52467b28, SUC_lexicon_tags_31 },
    { 0x54867b28, SUC_lexicon_tags_55 },
    { 0x77867b2b, SUC_lexicon_tags_15 },
    { 0x4f267b2c, SUC_lexicon_tags_25 },
    { 0x37e67b2d, SUC_lexicon_tags_47 },
    { 0xaf067b2d, SUC_lexicon_tags_61 },
    { 0x5fe67b2c, SUC_lexicon_tags_65 },
    { 0x7067b30, SUC_lexicon_tags_15 },
    { 0x14e67b31, SUC_lexicon_tags_17 },
    { 0xf2c67b2f, SUC_lexicon_tags_32 },
    { 0xbdc67b33, SUC_lexicon_tags_25 },
    { 0x99467b32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a067b37, SUC_lexicon_tags_2 },
    { 0x6fa67b38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff267b3e, SUC_lexicon_tags_7 },
    { 0x37667b3f, SUC_lexicon_tags_27 },
    { 0x25a67b40, SUC_lexicon_tags_15 },
    { 0x3f867b41, SUC_lexicon_tags_27 },
    { 0x1b067b42, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5667b47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc67b4a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x71067b4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a67b52, SUC_lexicon_tags_38 },
    { 0x84c67b52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a67b58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70a67b5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdb867b5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e67b5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4667b62, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xece67b65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44267b67, SUC_lexicon_tags_13 },
    { 0x34e67b68, SUC_lexicon_tags_7 },
    { 0x9a467b67, SUC_lexicon_tags_5 },
    { 0xb2067b6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcaa67b6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7867b6e, SUC_lexicon_tags_11 },
    { 0xc1467b6e, SUC_lexicon_tags_38 },
    { 0x56667b6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2fa67b72, SUC_lexicon_tags_7 },
    { 0xfd867b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda067b77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f667b79, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e67b7d, SUC_lexicon_tags_60 },
    { 0xe9267b7e, SUC_lexicon_tags_45 },
    { 0xb1867b7f, SUC_lexicon_tags_37 },
    { 0x69867b80, SUC_lexicon_tags_47 },
    { 0xd5667b81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e067b88, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x59267b8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b867b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4267b93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f467b97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31e67b99, SUC_lexicon_tags_19 },
    { 0x89467b99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89667ba2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e67baa, SUC_lexicon_tags_37 },
    { 0x5d867baa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47267bad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a67bb0, SUC_lexicon_tags_6 },
    { 0xfaa67bb1, SUC_lexicon_tags_2 },
    { 0xa8467bb2, SUC_lexicon_tags_4 },
    { 0x81a67bb2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a67bb8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x70067bbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0467bbe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd667bc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x63667bc4, SUC_lexicon_tags_15 },
    { 0x9067bc5, SUC_lexicon_tags_25 },
    { 0x77067bc4, SUC_lexicon_tags_11 },
    { 0x5b267bc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d867bca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89467bcf, SUC_lexicon_tags_63 },
    { 0x88c67bd0, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c67bd6, SUC_lexicon_tags_37 },
    { 0xdde67bd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70467bda, SUC_lexicon_tags_37 },
    { 0xa9e67bdb, SUC_lexicon_tags_43 },
    { 0xadc67bdc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25867be0, SUC_lexicon_tags_27 },
    { 0x91067be1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec67be5, SUC_lexicon_tags_46 },
    { 0x36467be6, SUC_lexicon_tags_2 },
    { 0x51067be6, SUC_lexicon_tags_27 },
    { 0x66467be8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb267bed, SUC_lexicon_tags_57 },
    { 0x97067bee, SUC_lexicon_tags_25 },
    { 0xbaa67bef, SUC_lexicon_tags_10 },
    { 0x70467bf0, SUC_lexicon_tags_7 },
    { 0xf4667bf1, SUC_lexicon_tags_29 },
    { 0x3cc67bf2, SUC_lexicon_tags_61 },
    { 0x56c67bf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17067bf8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85067bfd, SUC_lexicon_tags_37 },
    { 0xb1c67bfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e867c00, SUC_lexicon_tags_20 },
    { 0xb9c67c00, SUC_lexicon_tags_40 },
    { 0xe8667c02, SUC_lexicon_tags_27 },
    { 0xd5c67c03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec67c08, SUC_lexicon_tags_267 },
    { 0x68c67c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc67c0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c67c10, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x64867c12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c67c16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a67c1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d667c22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39467c28, SUC_lexicon_tags_29 },
    { 0x80267c29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8867c31, SUC_lexicon_tags_7 },
    { 0x5f667c32, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac67c36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc67c3e, SUC_lexicon_tags_85 },
    { 0xafe67c3e, SUC_lexicon_tags_8 },
    { 0x66c67c40, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3de67c42, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e267c45, SUC_lexicon_tags_37 },
    { 0xede67c45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x45867c49, SUC_lexicon_tags_1 },
    { 0x42e67c49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe9467c4c, SUC_lexicon_tags_61 },
    { 0xe0067c4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd867c5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45867c5e, SUC_lexicon_tags_27 },
    { 0x8a067c5e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdd467c61, SUC_lexicon_tags_26 },
    { 0xcdc67c62, SUC_lexicon_tags_27 },
    { 0x2c067c61, SUC_lexicon_tags_37 },
    { 0x78a67c63, SUC_lexicon_tags_0 },
    { 0x77667c65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce67c68, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x59267c6a, SUC_lexicon_tags_62 },
    { 0x28067c6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a67c72, SUC_lexicon_tags_6 },
    { 0x42c67c73, SUC_lexicon_tags_47 },
    { 0xc8067c74, SUC_lexicon_tags_10 },
    { 0x21667c74, SUC_lexicon_tags_12 },
    { 0x7cc67c74, SUC_lexicon_tags_25 },
    { 0xdc267c77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd867c7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7667c7d, SUC_lexicon_tags_37 },
    { 0x91e67c7e, SUC_lexicon_tags_47 },
    { 0xba467c7f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c67c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8467c86, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a67c89, SUC_lexicon_tags_0 },
    { 0x1f267c8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe3867c8c, SUC_lexicon_tags_25 },
    { 0xa3067c8d, SUC_lexicon_tags_341 },
    { 0xa0667c8c, SUC_lexicon_tags_37 },
    { 0x89267c8f, SUC_lexicon_tags_9 },
    { 0x3f467c8e, SUC_lexicon_tags_14 },
    { 0x43867c8d, SUC_lexicon_tags_35 },
    { 0x7d067c91, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x90867c94, SUC_lexicon_tags_65 },
    { 0x6f067c94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd667c98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf467c9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39067ca0, SUC_lexicon_tags_2 },
    { 0xe8667ca0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a67ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c67cac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32667cae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8067cb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5867cb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e467cba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc67cbe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c67cc2, SUC_lexicon_tags_25 },
    { 0x1f467cc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8667cc9, SUC_lexicon_tags_69 },
    { 0x7d267cca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73667cd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4a67cd3, SUC_lexicon_tags_37 },
    { 0xb2267cd4, SUC_lexicon_tags_38 },
    { 0x25067cd4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e67cd8, SUC_lexicon_tags_12 },
    { 0xe5067cd9, SUC_lexicon_tags_61 },
    { 0xb7067cd9, SUC_lexicon_tags_35 },
    { 0x72667cdb, SUC_lexicon_tags_11 },
    { 0xcf267cdc, SUC_lexicon_tags_45 },
    { 0xf1267cd8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e67ce7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1e67ce9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7dc67ceb, SUC_lexicon_tags_27 },
    { 0x62667cec, SUC_lexicon_tags_17 },
    { 0x1ec67cec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a67cf0, SUC_lexicon_tags_27 },
    { 0x95667cf1, SUC_lexicon_tags_17 },
    { 0xa7e67cf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88467cf6, SUC_lexicon_tags_0 },
    { 0x9a667cf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6867cfb, SUC_lexicon_tags_91 },
    { 0x5da67cfb, SUC_lexicon_tags_10 },
    { 0x9ca67cfd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e67d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50067d02, SUC_lexicon_tags_68 },
    { 0x8da67d03, SUC_lexicon_tags_60 },
    { 0x7c867d02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8467d08, SUC_lexicon_tags_12 },
    { 0xd5e67d08, SUC_lexicon_tags_38 },
    { 0xec667d0a, SUC_lexicon_tags_92 },
    { 0xdd867d0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5c067d0d, SUC_lexicon_tags_9 },
    { 0x12867d0e, SUC_lexicon_tags_17 },
    { 0xaca67d0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa67d12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x68667d14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2267d16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e67d1c, SUC_lexicon_tags_1 },
    { 0x82267d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b467d23, SUC_lexicon_tags_6 },
    { 0x87667d24, SUC_lexicon_tags_2 },
    { 0xa5667d25, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de67d29, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4667d2c, SUC_lexicon_tags_44 },
    { 0x17667d2d, SUC_lexicon_tags_17 },
    { 0x3ba67d2c, SUC_lexicon_tags_25 },
    { 0x6a067d2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6be67d31, SUC_lexicon_tags_23 },
    { 0x9a67d32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c67d36, SUC_lexicon_tags_15 },
    { 0xc4667d37, SUC_lexicon_tags_28 },
    { 0xa1667d38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a67d3b, SUC_lexicon_tags_198 },
    { 0x4ae67d3b, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48467d42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28467d48, SUC_lexicon_tags_37 },
    { 0xfee67d49, SUC_lexicon_tags_7 },
    { 0xb8c67d4a, SUC_lexicon_tags_103 },
    { 0x1a67d4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa1267d4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce67d50, SUC_lexicon_tags_15 },
    { 0xa8067d51, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac467d58, SUC_lexicon_tags_15 },
    { 0x4ba67d59, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x8267d5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24a67d5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20c67d5f, SUC_lexicon_tags_28 },
    { 0xbb267d60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e67d63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc5067d65, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4b467d67, SUC_lexicon_tags_38 },
    { 0x7ec67d68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a67d6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8667d71, SUC_lexicon_tags_37 },
    { 0xb467d71, SUC_lexicon_tags_7 },
    { 0x75c67d73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be67d76, SUC_lexicon_tags_66 },
    { 0x89067d77, SUC_lexicon_tags_38 },
    { 0xb7467d78, SUC_lexicon_tags_11 },
    { 0xefa67d76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdea67d7b, SUC_lexicon_tags_17 },
    { 0x6be67d7c, SUC_lexicon_tags_24 },
    { 0xc2667d7d, SUC_lexicon_tags_27 },
    { 0x7e467d7d, SUC_lexicon_tags_0 },
    { 0x7e267d7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21467d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e67d89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c67d8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4667d93, SUC_lexicon_tags_32 },
    { 0x19667d93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66867d9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a67da2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc6267da4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4aa67da6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de67daa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c267dad, SUC_lexicon_tags_11 },
    { 0x8dc67dae, SUC_lexicon_tags_11 },
    { 0x62867daf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba67db5, SUC_lexicon_tags_12 },
    { 0xa9667db6, SUC_lexicon_tags_14 },
    { 0xd9867db7, SUC_lexicon_tags_11 },
    { 0x52c67db8, SUC_lexicon_tags_27 },
    { 0xf3267db9, SUC_lexicon_tags_17 },
    { 0xfea67db7, SUC_lexicon_tags_25 },
    { 0x9d867dbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da67dbf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12667dc9, SUC_lexicon_tags_30 },
    { 0x14c67dca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9667dcc, SUC_lexicon_tags_7 },
    { 0x8ca67dcd, SUC_lexicon_tags_32 },
    { 0x17267dce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3067dd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e467dd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe67dd7, SUC_lexicon_tags_7 },
    { 0x61467dd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed067ddd, SUC_lexicon_tags_68 },
    { 0x76667ddd, SUC_lexicon_tags_7 },
    { 0xf0e67ddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb867de1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10467de5, SUC_lexicon_tags_12 },
    { 0xd3467de5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e67dea, SUC_lexicon_tags_25 },
    { 0x3267deb, SUC_lexicon_tags_12 },
    { 0x91067deb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be67def, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x45867df2, SUC_lexicon_tags_27 },
    { 0xf3867df2, SUC_lexicon_tags_17 },
    { 0x7d467df3, SUC_lexicon_tags_32 },
    { 0xdcc67df2, SUC_lexicon_tags_28 },
    { 0x82067df6, SUC_lexicon_tags_21 },
    { 0xae667df2, SUC_lexicon_tags_28 },
    { 0x10067df6, SUC_lexicon_tags_16 },
    { 0xf8867df4, SUC_lexicon_tags_6 },
    { 0x5e867df5, SUC_lexicon_tags_60 },
    { 0x51667dfb, SUC_lexicon_tags_1 },
    { 0xda067dfc, SUC_lexicon_tags_9 },
    { 0x32a67dfd, SUC_lexicon_tags_7 },
    { 0xeb867dfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a67e01, SUC_lexicon_tags_47 },
    { 0xe9067e02, SUC_lexicon_tags_19 },
    { 0x7aa67e03, SUC_lexicon_tags_19 },
    { 0xc9467e04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e67e09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a267e0c, SUC_lexicon_tags_39 },
    { 0xcc467e0d, SUC_lexicon_tags_32 },
    { 0xa9467e0e, SUC_lexicon_tags_39 },
    { 0xe4867e0f, SUC_lexicon_tags_45 },
    { 0xc2267e0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de67e13, SUC_lexicon_tags_27 },
    { 0xe1667e14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e67e19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3867e1d, SUC_lexicon_tags_11 },
    { 0x28867e1e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b867e20, SUC_lexicon_tags_46 },
    { 0x43c67e21, SUC_lexicon_tags_15 },
    { 0xb2067e22, SUC_lexicon_tags_19 },
    { 0xb7a67e20, SUC_lexicon_tags_7 },
    { 0xdb867e24, SUC_lexicon_tags_10 },
    { 0x2e467e24, SUC_lexicon_tags_35 },
    { 0x69067e26, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d467e29, SUC_lexicon_tags_60 },
    { 0xb4667e2a, SUC_lexicon_tags_47 },
    { 0xc8867e2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f867e33, SUC_lexicon_tags_17 },
    { 0x1a067e34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d467e36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ea67e38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ec67e3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a67e3e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x66667e40, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e467e43, SUC_lexicon_tags_7 },
    { 0xef267e44, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xce867e46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60a67e48, SUC_lexicon_tags_48 },
    { 0xe9267e49, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x53667e4b, SUC_lexicon_tags_27 },
    { 0x14067e4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c67e51, SUC_lexicon_tags_65 },
    { 0xa9467e52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6a67e54, SUC_lexicon_tags_15 },
    { 0x3d667e55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e67e5a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8267e5d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a67e60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60067e62, SUC_lexicon_tags_32 },
    { 0x7ce67e63, SUC_lexicon_tags_11 },
    { 0x46067e63, SUC_lexicon_tags_1 },
    { 0xe6467e65, SUC_lexicon_tags_7 },
    { 0x2d267e62, SUC_lexicon_tags_11 },
    { 0x73067e67, SUC_lexicon_tags_70 },
    { 0x29467e67, SUC_lexicon_tags_47 },
    { 0xbde67e62, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdf067e6b, SUC_lexicon_tags_38 },
    { 0xf0067e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e667e6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab667e70, SUC_lexicon_tags_0 },
    { 0xc6467e71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c867e76, SUC_lexicon_tags_0 },
    { 0xb0a67e76, SUC_lexicon_tags_10 },
    { 0x33c67e78, SUC_lexicon_tags_11 },
    { 0x35c67e76, SUC_lexicon_tags_39 },
    { 0x85e67e78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c67e7d, SUC_lexicon_tags_45 },
    { 0x12267e7e, SUC_lexicon_tags_14 },
    { 0xd9667e7d, SUC_lexicon_tags_8 },
    { 0xf9867e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c67e86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc67e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a67e91, SUC_lexicon_tags_11 },
    { 0x3267e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e467e94, SUC_lexicon_tags_12 },
    { 0x70a67e95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2da67e97, SUC_lexicon_tags_37 },
    { 0xfaa67e97, SUC_lexicon_tags_17 },
    { 0x58067e98, SUC_lexicon_tags_2 },
    { 0xf2467e99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcaa67e9c, SUC_lexicon_tags_15 },
    { 0xeba67e9d, SUC_lexicon_tags_28 },
    { 0xa1267e9e, SUC_lexicon_tags_15 },
    { 0xfb667e9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88467ea2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e67eaf, SUC_lexicon_tags_2 },
    { 0x7067eaf, SUC_lexicon_tags_11 },
    { 0x39067eb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde867eba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca067ec0, SUC_lexicon_tags_7 },
    { 0x8a067ec1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e67ec4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x62e67ec6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1667ec9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b867ecb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64667ed5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e467ed8, SUC_lexicon_tags_60 },
    { 0xabe67ed9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6267edc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f867ee2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec867ee5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16c67ee7, SUC_lexicon_tags_19 },
    { 0xc0267ee8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61667eeb, SUC_lexicon_tags_9 },
    { 0x4b067eec, SUC_lexicon_tags_49 },
    { 0xef467eeb, SUC_lexicon_tags_12 },
    { 0x75667eee, SUC_lexicon_tags_40 },
    { 0x16e67eef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ec67ef1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d067ef3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda67ef8, SUC_lexicon_tags_12 },
    { 0xc5867ef9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb667efe, SUC_lexicon_tags_77 },
    { 0x99267efe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d667f01, SUC_lexicon_tags_1 },
    { 0xeb467f02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6267f05, SUC_lexicon_tags_49 },
    { 0x99267f05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa0a67f08, SUC_lexicon_tags_14 },
    { 0xed067f09, SUC_lexicon_tags_1 },
    { 0xa2467f09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14267f13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x54a67f15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda067f1f, SUC_lexicon_tags_77 },
    { 0x9c267f1f, SUC_lexicon_tags_15 },
    { 0xfc67f21, SUC_lexicon_tags_25 },
    { 0x5ac67f22, SUC_lexicon_tags_4 },
    { 0x6f467f23, SUC_lexicon_tags_37 },
    { 0x63267f21, SUC_lexicon_tags_11 },
    { 0x31267f23, SUC_lexicon_tags_13 },
    { 0x7e467f20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b467f2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12267f33, SUC_lexicon_tags_49 },
    { 0xfc467f34, SUC_lexicon_tags_17 },
    { 0xd0e67f35, SUC_lexicon_tags_27 },
    { 0x4ec67f36, SUC_lexicon_tags_15 },
    { 0x6267f37, SUC_lexicon_tags_12 },
    { 0x60267f38, SUC_lexicon_tags_1 },
    { 0x1d867f38, SUC_lexicon_tags_25 },
    { 0x2d067f3a, SUC_lexicon_tags_63 },
    { 0xc6867f3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe7867f3d, SUC_lexicon_tags_9 },
    { 0x13c67f3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d067f42, SUC_lexicon_tags_14 },
    { 0xace67f43, SUC_lexicon_tags_47 },
    { 0xdd667f43, SUC_lexicon_tags_38 },
    { 0x8f267f44, SUC_lexicon_tags_32 },
    { 0xbf267f43, SUC_lexicon_tags_27 },
    { 0xf7267f42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1467f4a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a67f50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e67f53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9067f57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1867f59, SUC_lexicon_tags_43 },
    { 0x69067f5a, SUC_lexicon_tags_21 },
    { 0x5f067f5b, SUC_lexicon_tags_37 },
    { 0xeb067f5a, SUC_lexicon_tags_1 },
    { 0x9aa67f5d, SUC_lexicon_tags_2 },
    { 0x24267f5e, SUC_lexicon_tags_45 },
    { 0xe5267f5e, SUC_lexicon_tags_23 },
    { 0x86267f5a, SUC_lexicon_tags_17 },
    { 0xdd467f61, SUC_lexicon_tags_11 },
    { 0x60067f5b, SUC_lexicon_tags_2 },
    { 0x67467f63, SUC_lexicon_tags_48 },
    { 0x14067f64, SUC_lexicon_tags_82 },
    { 0x26c67f5b, SUC_lexicon_tags_17 },
    { 0xa4867f5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81667f69, SUC_lexicon_tags_79 },
    { 0xe8867f6a, SUC_lexicon_tags_23 },
    { 0x15667f6b, SUC_lexicon_tags_18 },
    { 0x2e067f6c, SUC_lexicon_tags_2 },
    { 0x1b067f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c067f73, SUC_lexicon_tags_6 },
    { 0xa6c67f74, SUC_lexicon_tags_16 },
    { 0xc7267f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d667f77, SUC_lexicon_tags_12 },
    { 0xec267f78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e67f7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e667f7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b067f80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec67f83, SUC_lexicon_tags_14 },
    { 0x2ac67f84, SUC_lexicon_tags_25 },
    { 0x92467f85, SUC_lexicon_tags_26 },
    { 0x9e467f83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea67f89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaae67f8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b467f8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9e67f91, SUC_lexicon_tags_37 },
    { 0x52c67f92, SUC_lexicon_tags_8 },
    { 0xce67f93, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xc067f96, SUC_lexicon_tags_18 },
    { 0xb1867f97, SUC_lexicon_tags_6 },
    { 0x7b067f96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c67fa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17c67fa3, SUC_lexicon_tags_11 },
    { 0xfa467fa4, SUC_lexicon_tags_4 },
    { 0x44867fa5, SUC_lexicon_tags_42 },
    { 0x3f267fa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3267faa, SUC_lexicon_tags_15 },
    { 0xa6267fab, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4467fb0, SUC_lexicon_tags_33 },
    { 0x6c67fb0, SUC_lexicon_tags_7 },
    { 0xde667fb0, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc67fb8, SUC_lexicon_tags_15 },
    { 0x9e67fb8, SUC_lexicon_tags_43 },
    { 0x23a67fba, SUC_lexicon_tags_63 },
    { 0xac67fb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf667fc1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a67fc6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a267fc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8667fcd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac67fd2, SUC_lexicon_tags_25 },
    { 0xcd267fd3, SUC_lexicon_tags_47 },
    { 0xbf667fd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68c67fd6, SUC_lexicon_tags_43 },
    { 0xca467fd6, SUC_lexicon_tags_92 },
    { 0x59867fd8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8667fde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82467fe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3067fe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e67fe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c267feb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc67ff6, SUC_lexicon_tags_39 },
    { 0xee667ff7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56667ffc, SUC_lexicon_tags_25 },
    { 0x2a667ffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc268002, SUC_lexicon_tags_37 },
    { 0xe9668003, SUC_lexicon_tags_7 },
    { 0xbba68003, SUC_lexicon_tags_1 },
    { 0xdfe68002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6268008, SUC_lexicon_tags_49 },
    { 0x71268009, SUC_lexicon_tags_25 },
    { 0x8d26800a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd9e6800c, SUC_lexicon_tags_47 },
    { 0xa766800c, SUC_lexicon_tags_12 },
    { 0x4726800e, SUC_lexicon_tags_37 },
    { 0x2d66800c, SUC_lexicon_tags_12 },
    { 0x17e68010, SUC_lexicon_tags_47 },
    { 0x2be68011, SUC_lexicon_tags_15 },
    { 0x4c06800e, SUC_lexicon_tags_1 },
    { 0x59c68013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6868015, SUC_lexicon_tags_27 },
    { 0x22a68016, SUC_lexicon_tags_1 },
    { 0x10a68016, SUC_lexicon_tags_12 },
    { 0x2a68018, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e6801e, SUC_lexicon_tags_35 },
    { 0x4a86801f, SUC_lexicon_tags_60 },
    { 0x9266801f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c868025, SUC_lexicon_tags_15 },
    { 0x59668026, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66802a, SUC_lexicon_tags_25 },
    { 0x4ee6802b, SUC_lexicon_tags_0 },
    { 0x42a6802c, SUC_lexicon_tags_65 },
    { 0xdc46802b, SUC_lexicon_tags_62 },
    { 0x4fc6802b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa7468030, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4068034, SUC_lexicon_tags_6 },
    { 0xdf268035, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45868037, SUC_lexicon_tags_49 },
    { 0x8fc68038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5006803a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13e6803c, SUC_lexicon_tags_38 },
    { 0xce66803c, SUC_lexicon_tags_17 },
    { 0xc666803c, SUC_lexicon_tags_4 },
    { 0xd506803f, SUC_lexicon_tags_17 },
    { 0xbe86803f, SUC_lexicon_tags_38 },
    { 0x15e6803f, SUC_lexicon_tags_28 },
    { 0x2ce68042, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc6804c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee6804f, SUC_lexicon_tags_27 },
    { 0x69a68050, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba68053, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c668056, SUC_lexicon_tags_17 },
    { 0xd9668056, SUC_lexicon_tags_77 },
    { 0x8c068058, SUC_lexicon_tags_37 },
    { 0x75868057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8006805b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3346805d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaa46805f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xac668062, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa668066, SUC_lexicon_tags_37 },
    { 0x10068067, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d26806b, SUC_lexicon_tags_0 },
    { 0x3766806b, SUC_lexicon_tags_28 },
    { 0xf526806b, SUC_lexicon_tags_0 },
    { 0x15a6806e, SUC_lexicon_tags_0 },
    { 0x7906806e, SUC_lexicon_tags_30 },
    { 0x8b06806e, SUC_lexicon_tags_2 },
    { 0x3b268070, SUC_lexicon_tags_77 },
    { 0x38068072, SUC_lexicon_tags_1 },
    { 0x67c68073, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe468077, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a26807c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e6807f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac68084, SUC_lexicon_tags_27 },
    { 0xf4068085, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe68087, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4886808f, SUC_lexicon_tags_106 },
    { 0xf06808f, SUC_lexicon_tags_14 },
    { 0xf5e68091, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6068096, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e6809b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xa426809e, SUC_lexicon_tags_25 },
    { 0xc026809e, SUC_lexicon_tags_3 },
    { 0x53a680a0, SUC_lexicon_tags_30 },
    { 0x80680a0, SUC_lexicon_tags_63 },
    { 0xf0a680a2, SUC_lexicon_tags_17 },
    { 0xb4a680a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8680a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x20e680a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8aa680aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b2680ac, SUC_lexicon_tags_47 },
    { 0x93c680ad, SUC_lexicon_tags_46 },
    { 0xeb8680ad, SUC_lexicon_tags_27 },
    { 0xfc680af, SUC_lexicon_tags_12 },
    { 0x74680b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96680b9, SUC_lexicon_tags_18 },
    { 0x2e0680b9, SUC_lexicon_tags_96 },
    { 0x2c2680b9, SUC_lexicon_tags_38 },
    { 0xf4a680ba, SUC_lexicon_tags_38 },
    { 0xcd2680bd, SUC_lexicon_tags_2 },
    { 0xc6c680bd, SUC_lexicon_tags_22 },
    { 0xb34680bf, SUC_lexicon_tags_18 },
    { 0x38680c0, SUC_lexicon_tags_11 },
    { 0xd96680c0, SUC_lexicon_tags_25 },
    { 0x610680bf, SUC_lexicon_tags_14 },
    { 0x160680c3, SUC_lexicon_tags_25 },
    { 0x61a680c3, SUC_lexicon_tags_38 },
    { 0xc68680c5, SUC_lexicon_tags_17 },
    { 0xdd6680c6, SUC_lexicon_tags_37 },
    { 0x3c680c6, SUC_lexicon_tags_30 },
    { 0x4a4680c7, SUC_lexicon_tags_25 },
    { 0x772680c9, SUC_lexicon_tags_45 },
    { 0x444680ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9f6680cc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9d2680ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf0c680d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8680d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb44680d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6680db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360680e0, SUC_lexicon_tags_15 },
    { 0x6a0680e0, SUC_lexicon_tags_28 },
    { 0x4e6680e1, SUC_lexicon_tags_37 },
    { 0x44a680e3, SUC_lexicon_tags_38 },
    { 0xde8680e3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc680e8, SUC_lexicon_tags_12 },
    { 0x348680e8, SUC_lexicon_tags_47 },
    { 0xf92680e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe66680ec, SUC_lexicon_tags_14 },
    { 0x726680ed, SUC_lexicon_tags_16 },
    { 0x808680ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98680f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4c0680f4, SUC_lexicon_tags_65 },
    { 0x1f2680f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb6a680f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3e2680f9, SUC_lexicon_tags_11 },
    { 0x800680fa, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x156680fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c68100, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2f468102, SUC_lexicon_tags_9 },
    { 0x10068103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc68106, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d06810e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0668112, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde468117, SUC_lexicon_tags_11 },
    { 0xe7a68118, SUC_lexicon_tags_32 },
    { 0xee468119, SUC_lexicon_tags_11 },
    { 0xa5c68117, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e6811d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4268120, SUC_lexicon_tags_39 },
    { 0xa3c68121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67068124, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xe0468126, SUC_lexicon_tags_31 },
    { 0xbc068127, SUC_lexicon_tags_11 },
    { 0xa3c68127, SUC_lexicon_tags_47 },
    { 0x60468129, SUC_lexicon_tags_62 },
    { 0xd6c68129, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c68132, SUC_lexicon_tags_37 },
    { 0x61a68132, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6186813a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce6813e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x91868140, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28668146, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa6814a, SUC_lexicon_tags_15 },
    { 0x98e6814b, SUC_lexicon_tags_2 },
    { 0x37c6814c, SUC_lexicon_tags_39 },
    { 0x3ae6814c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6646814f, SUC_lexicon_tags_37 },
    { 0x29668150, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a68153, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e068155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a6815a, SUC_lexicon_tags_7 },
    { 0x3e6815b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd426815d, SUC_lexicon_tags_37 },
    { 0x5706815e, SUC_lexicon_tags_25 },
    { 0xf0c6815f, SUC_lexicon_tags_11 },
    { 0x53c68160, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e068167, SUC_lexicon_tags_38 },
    { 0x12068167, SUC_lexicon_tags_26 },
    { 0x95e68167, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5226816c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ac6816e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13868172, SUC_lexicon_tags_30 },
    { 0x47e68173, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2668177, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6817b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb246817f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea268182, SUC_lexicon_tags_66 },
    { 0xbd268182, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6468186, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4e68188, SUC_lexicon_tags_17 },
    { 0xe9a68189, SUC_lexicon_tags_45 },
    { 0xe0e68189, SUC_lexicon_tags_27 },
    { 0x53c6818b, SUC_lexicon_tags_62 },
    { 0xf786818c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c66818e, SUC_lexicon_tags_25 },
    { 0xbd66818e, SUC_lexicon_tags_17 },
    { 0xb9a68190, SUC_lexicon_tags_17 },
    { 0x6c68190, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60468194, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1b668196, SUC_lexicon_tags_29 },
    { 0x89668197, SUC_lexicon_tags_11 },
    { 0xff668198, SUC_lexicon_tags_14 },
    { 0x57868199, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f06819c, SUC_lexicon_tags_6 },
    { 0x4086819c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e681a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x504681a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaca681a5, SUC_lexicon_tags_87 },
    { 0xaf8681a6, SUC_lexicon_tags_68 },
    { 0x21e681a7, SUC_lexicon_tags_37 },
    { 0xf32681a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68681aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe681ad, SUC_lexicon_tags_12 },
    { 0x35a681ad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96681ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe12681bc, SUC_lexicon_tags_254 },
    { 0xf46681bd, SUC_lexicon_tags_14 },
    { 0xd62681be, SUC_lexicon_tags_25 },
    { 0xe50681be, SUC_lexicon_tags_2 },
    { 0x4c8681be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x212681c3, SUC_lexicon_tags_23 },
    { 0xa9c681c4, SUC_lexicon_tags_12 },
    { 0xcf8681c3, SUC_lexicon_tags_50 },
    { 0xd98681c5, SUC_lexicon_tags_19 },
    { 0x39c681c7, SUC_lexicon_tags_17 },
    { 0xeac681c8, SUC_lexicon_tags_17 },
    { 0xf0a681c9, SUC_lexicon_tags_45 },
    { 0x558681c9, SUC_lexicon_tags_0 },
    { 0xef0681c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x270681ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30681d5, SUC_lexicon_tags_1 },
    { 0xd62681d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76681d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa36681db, SUC_lexicon_tags_1 },
    { 0x5da681dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4681df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02681e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea2681e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe681ec, SUC_lexicon_tags_12 },
    { 0x6c681ec, SUC_lexicon_tags_15 },
    { 0x220681ee, SUC_lexicon_tags_7 },
    { 0x272681ee, SUC_lexicon_tags_37 },
    { 0xbba681f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x978681f2, SUC_lexicon_tags_17 },
    { 0xc24681f3, SUC_lexicon_tags_37 },
    { 0xd9c681f4, SUC_lexicon_tags_0 },
    { 0xdfa681f5, SUC_lexicon_tags_15 },
    { 0x23a681f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ae681f8, SUC_lexicon_tags_2 },
    { 0xc7a681f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0681fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71868202, SUC_lexicon_tags_7 },
    { 0x3a68202, SUC_lexicon_tags_10 },
    { 0xca668202, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6820f, SUC_lexicon_tags_44 },
    { 0xf668210, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb4068212, SUC_lexicon_tags_0 },
    { 0x9268212, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b268217, SUC_lexicon_tags_37 },
    { 0x49868217, SUC_lexicon_tags_1 },
    { 0x27468218, SUC_lexicon_tags_17 },
    { 0x8b86821a, SUC_lexicon_tags_9 },
    { 0x1b468217, SUC_lexicon_tags_33 },
    { 0xed86821c, SUC_lexicon_tags_37 },
    { 0x68e68219, SUC_lexicon_tags_15 },
    { 0x2ee6821e, SUC_lexicon_tags_171 },
    { 0xaf46821f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf068223, SUC_lexicon_tags_4 },
    { 0x51868224, SUC_lexicon_tags_11 },
    { 0xe4868225, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x36a68227, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8226822b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8266822e, SUC_lexicon_tags_30 },
    { 0xb0c6822f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c68233, SUC_lexicon_tags_37 },
    { 0x58268234, SUC_lexicon_tags_17 },
    { 0xbda68234, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f26823a, SUC_lexicon_tags_22 },
    { 0x2d26823b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf9c6823d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf9c6823f, SUC_lexicon_tags_17 },
    { 0xf6868240, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26268244, SUC_lexicon_tags_21 },
    { 0x58668245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9626824b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc6824f, SUC_lexicon_tags_14 },
    { 0x2c468250, SUC_lexicon_tags_15 },
    { 0xe8668251, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9468254, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab668257, SUC_lexicon_tags_1 },
    { 0xe4868257, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e86825d, SUC_lexicon_tags_7 },
    { 0xd806825e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a268265, SUC_lexicon_tags_37 },
    { 0x2e068266, SUC_lexicon_tags_42 },
    { 0x52e68265, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a86826c, SUC_lexicon_tags_11 },
    { 0xc266826d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32268271, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b068273, SUC_lexicon_tags_31 },
    { 0xed268274, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa146827a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd986827c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde68280, SUC_lexicon_tags_7 },
    { 0x13e68281, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c68285, SUC_lexicon_tags_5 },
    { 0xd0668286, SUC_lexicon_tags_27 },
    { 0xb9668287, SUC_lexicon_tags_57 },
    { 0x7a68288, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e68294, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1566829a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c46829c, SUC_lexicon_tags_46 },
    { 0x99c6829d, SUC_lexicon_tags_42 },
    { 0x5a26829e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d0682a0, SUC_lexicon_tags_27 },
    { 0xbdc682a1, SUC_lexicon_tags_10 },
    { 0x2d0682a2, SUC_lexicon_tags_1 },
    { 0xd38682a1, SUC_lexicon_tags_7 },
    { 0xbb8682a4, SUC_lexicon_tags_1 },
    { 0x664682a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae682a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e682b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de682b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x708682b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf78682ba, SUC_lexicon_tags_47 },
    { 0x8e682bb, SUC_lexicon_tags_17 },
    { 0x9b8682ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x240682be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626682c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20682c4, SUC_lexicon_tags_15 },
    { 0x19c682c5, SUC_lexicon_tags_1 },
    { 0x936682c4, SUC_lexicon_tags_47 },
    { 0xf8e682c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8682ca, SUC_lexicon_tags_25 },
    { 0x338682cb, SUC_lexicon_tags_11 },
    { 0xf6c682cb, SUC_lexicon_tags_38 },
    { 0x724682ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42c682cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1c682d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x400682d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74682d7, SUC_lexicon_tags_19 },
    { 0x2a4682d8, SUC_lexicon_tags_53 },
    { 0xee6682d9, SUC_lexicon_tags_84 },
    { 0xe68682da, SUC_lexicon_tags_3 },
    { 0x7b6682db, SUC_lexicon_tags_7 },
    { 0x3aa682db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8682e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x168682e2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x100682e4, SUC_lexicon_tags_7 },
    { 0xec6682e5, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x2d0682e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x52682ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66682ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb3a682ee, SUC_lexicon_tags_17 },
    { 0x1bc682ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64682f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe00682f5, SUC_lexicon_tags_74 },
    { 0x5d0682f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe14682f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x744682fb, SUC_lexicon_tags_2 },
    { 0xaa0682fc, SUC_lexicon_tags_2 },
    { 0x7a4682fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a068300, SUC_lexicon_tags_32 },
    { 0x7468300, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1668304, SUC_lexicon_tags_70 },
    { 0x69e68305, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa68309, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa68311, SUC_lexicon_tags_17 },
    { 0x8a068312, SUC_lexicon_tags_17 },
    { 0xa0868311, SUC_lexicon_tags_26 },
    { 0x9be68314, SUC_lexicon_tags_2 },
    { 0xf9a68315, SUC_lexicon_tags_27 },
    { 0xfd268316, SUC_lexicon_tags_37 },
    { 0x65a68317, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6986831c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5106831f, SUC_lexicon_tags_19 },
    { 0xe8c68320, SUC_lexicon_tags_25 },
    { 0x69868320, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea6832a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c26832c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c668331, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x30068334, SUC_lexicon_tags_1 },
    { 0xe5668335, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d06833a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41068340, SUC_lexicon_tags_46 },
    { 0x2dc68340, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10068348, SUC_lexicon_tags_85 },
    { 0xd7868348, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5606834b, SUC_lexicon_tags_25 },
    { 0x4e26834c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9c66834e, SUC_lexicon_tags_1 },
    { 0xc386834e, SUC_lexicon_tags_1 },
    { 0x81068350, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97668353, SUC_lexicon_tags_12 },
    { 0x38268354, SUC_lexicon_tags_44 },
    { 0x87268354, SUC_lexicon_tags_37 },
    { 0x7668356, SUC_lexicon_tags_2 },
    { 0x50068354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10268359, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c6835c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a6835f, SUC_lexicon_tags_14 },
    { 0x2a668360, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7868364, SUC_lexicon_tags_33 },
    { 0x5ba68364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8068367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a06836a, SUC_lexicon_tags_38 },
    { 0x8546836b, SUC_lexicon_tags_7 },
    { 0x9ee6836a, SUC_lexicon_tags_11 },
    { 0xb426836d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85068371, SUC_lexicon_tags_68 },
    { 0x8f468371, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c68375, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c468378, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c6837f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc868389, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda6838e, SUC_lexicon_tags_60 },
    { 0x30a6838f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc68395, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6839a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c6839d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8683a1, SUC_lexicon_tags_15 },
    { 0xd14683a1, SUC_lexicon_tags_14 },
    { 0xc6c683a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a683a8, SUC_lexicon_tags_53 },
    { 0x84e683a9, SUC_lexicon_tags_14 },
    { 0x1dc683a9, SUC_lexicon_tags_7 },
    { 0x95c683a9, SUC_lexicon_tags_37 },
    { 0x792683a8, SUC_lexicon_tags_12 },
    { 0x6de683ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e683b7, SUC_lexicon_tags_94 },
    { 0xc30683b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc683be, SUC_lexicon_tags_0 },
    { 0x9aa683be, SUC_lexicon_tags_1 },
    { 0x2ba683c0, SUC_lexicon_tags_25 },
    { 0xccc683c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x412683c3, SUC_lexicon_tags_37 },
    { 0xf0683c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40683cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dc683cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x976683d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6683d4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd0e683d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa2683d8, SUC_lexicon_tags_25 },
    { 0xd5e683d8, SUC_lexicon_tags_63 },
    { 0xd0e683d9, SUC_lexicon_tags_37 },
    { 0x80683db, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x57e683dd, SUC_lexicon_tags_56 },
    { 0xfa2683de, SUC_lexicon_tags_49 },
    { 0x464683df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64683e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e683ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81a683ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326683f0, SUC_lexicon_tags_104 },
    { 0xd98683f1, SUC_lexicon_tags_0 },
    { 0xe4c683f1, SUC_lexicon_tags_19 },
    { 0x8a2683f2, SUC_lexicon_tags_25 },
    { 0x1f6683f4, SUC_lexicon_tags_11 },
    { 0x2ae683f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8683f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c683ff, SUC_lexicon_tags_37 },
    { 0xcc068400, SUC_lexicon_tags_2 },
    { 0xe3a68400, SUC_lexicon_tags_25 },
    { 0x868402, SUC_lexicon_tags_11 },
    { 0xe4868403, SUC_lexicon_tags_25 },
    { 0x87468404, SUC_lexicon_tags_38 },
    { 0xc1068405, SUC_lexicon_tags_14 },
    { 0x34068406, SUC_lexicon_tags_46 },
    { 0xc6068403, SUC_lexicon_tags_34 },
    { 0x5da68406, SUC_lexicon_tags_11 },
    { 0xf1a68404, SUC_lexicon_tags_136 },
    { 0x9268401, SUC_lexicon_tags_27 },
    { 0x3b26840b, SUC_lexicon_tags_26 },
    { 0xacc6840b, SUC_lexicon_tags_25 },
    { 0x55a6840d, SUC_lexicon_tags_14 },
    { 0x4826840e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34068410, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7668412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e268416, SUC_lexicon_tags_19 },
    { 0x9268417, SUC_lexicon_tags_151 },
    { 0xfa68418, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85c6841a, SUC_lexicon_tags_13 },
    { 0xe106841b, SUC_lexicon_tags_12 },
    { 0x21a6841c, SUC_lexicon_tags_1 },
    { 0x886841c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48068421, SUC_lexicon_tags_46 },
    { 0x84668422, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4468427, SUC_lexicon_tags_87 },
    { 0xd9468428, SUC_lexicon_tags_38 },
    { 0xcc668429, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce68432, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e68436, SUC_lexicon_tags_49 },
    { 0x46e68437, SUC_lexicon_tags_2 },
    { 0x70868438, SUC_lexicon_tags_65 },
    { 0xc6268437, SUC_lexicon_tags_37 },
    { 0xe4e6843a, SUC_lexicon_tags_13 },
    { 0x10c6843b, SUC_lexicon_tags_60 },
    { 0x5306843b, SUC_lexicon_tags_47 },
    { 0x23068439, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc586843f, SUC_lexicon_tags_31 },
    { 0x43868440, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7268444, SUC_lexicon_tags_38 },
    { 0xd4068445, SUC_lexicon_tags_23 },
    { 0x6aa68445, SUC_lexicon_tags_38 },
    { 0xee868447, SUC_lexicon_tags_11 },
    { 0xaaa68448, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e06844a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d86844e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a68455, SUC_lexicon_tags_230 },
    { 0xe9068456, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa166845a, SUC_lexicon_tags_37 },
    { 0xe8a6845b, SUC_lexicon_tags_38 },
    { 0xa386845c, SUC_lexicon_tags_13 },
    { 0x8266845c, SUC_lexicon_tags_63 },
    { 0x2206845e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x37e68460, SUC_lexicon_tags_23 },
    { 0xe8e68461, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea68465, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e68469, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3066846b, SUC_lexicon_tags_86 },
    { 0x9226846c, SUC_lexicon_tags_60 },
    { 0xb606846d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48668475, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d268480, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c68485, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d868489, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x55a6848b, SUC_lexicon_tags_30 },
    { 0xeda6848c, SUC_lexicon_tags_10 },
    { 0xf106848d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a068490, SUC_lexicon_tags_11 },
    { 0x2e468491, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d268495, SUC_lexicon_tags_1 },
    { 0xb6068495, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a6849c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e684a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8684ab, SUC_lexicon_tags_38 },
    { 0x502684ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b4684ae, SUC_lexicon_tags_63 },
    { 0x306684af, SUC_lexicon_tags_23 },
    { 0x9e4684af, SUC_lexicon_tags_11 },
    { 0x18c684b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x380684b3, SUC_lexicon_tags_63 },
    { 0x63a684b4, SUC_lexicon_tags_47 },
    { 0x686684b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fc684b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc684bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a684c3, SUC_lexicon_tags_17 },
    { 0x696684c3, SUC_lexicon_tags_23 },
    { 0x7d2684c5, SUC_lexicon_tags_25 },
    { 0x932684c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66684c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e684cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a684cf, SUC_lexicon_tags_10 },
    { 0x22684cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2a684d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7e684d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6c684d6, SUC_lexicon_tags_45 },
    { 0xc62684d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e0684d9, SUC_lexicon_tags_33 },
    { 0x3e2684da, SUC_lexicon_tags_2 },
    { 0xecc684d9, SUC_lexicon_tags_15 },
    { 0xee0684dc, SUC_lexicon_tags_1 },
    { 0x2d2684dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe684e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc16684e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2684e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dc684e8, SUC_lexicon_tags_60 },
    { 0x2f4684e9, SUC_lexicon_tags_66 },
    { 0xca2684e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2684ed, SUC_lexicon_tags_2 },
    { 0x934684ed, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x266684f0, SUC_lexicon_tags_23 },
    { 0x5dc684f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5dc684f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e684fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea68500, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a68506, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x10e68508, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc266850a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2926850e, SUC_lexicon_tags_14 },
    { 0x74c6850f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39468517, SUC_lexicon_tags_26 },
    { 0x5b268518, SUC_lexicon_tags_43 },
    { 0x2a668519, SUC_lexicon_tags_2 },
    { 0xf2c68517, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x5be6851c, SUC_lexicon_tags_23 },
    { 0x526851c, SUC_lexicon_tags_21 },
    { 0x5a86851e, SUC_lexicon_tags_47 },
    { 0x9566851e, SUC_lexicon_tags_1 },
    { 0x99468520, SUC_lexicon_tags_13 },
    { 0xd9868520, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32468527, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc6852d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c68533, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c068537, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xd2468539, SUC_lexicon_tags_63 },
    { 0xd1468539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe386853e, SUC_lexicon_tags_6 },
    { 0xd366853f, SUC_lexicon_tags_30 },
    { 0x6668540, SUC_lexicon_tags_1 },
    { 0x5a068541, SUC_lexicon_tags_17 },
    { 0x60268542, SUC_lexicon_tags_56 },
    { 0xd7868543, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xea68545, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c68550, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5668553, SUC_lexicon_tags_9 },
    { 0xed868553, SUC_lexicon_tags_19 },
    { 0x58868555, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3846855d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4c66855f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9468564, SUC_lexicon_tags_12 },
    { 0x62068565, SUC_lexicon_tags_12 },
    { 0x69a68564, SUC_lexicon_tags_46 },
    { 0x30668567, SUC_lexicon_tags_17 },
    { 0xdc468568, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c6856c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdc66856e, SUC_lexicon_tags_23 },
    { 0x9926856f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a68574, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8286857a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4868581, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x14e68583, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20268588, SUC_lexicon_tags_2 },
    { 0x9f268589, SUC_lexicon_tags_63 },
    { 0x2366858a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a86858e, SUC_lexicon_tags_30 },
    { 0xd186858f, SUC_lexicon_tags_47 },
    { 0x29a6858e, SUC_lexicon_tags_25 },
    { 0x35a68591, SUC_lexicon_tags_17 },
    { 0x9c68591, SUC_lexicon_tags_17 },
    { 0xffc68593, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc268597, SUC_lexicon_tags_1 },
    { 0x95668597, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8166859d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12a6859f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418685a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae685a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8685ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd98685ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd42685af, SUC_lexicon_tags_25 },
    { 0x658685af, SUC_lexicon_tags_45 },
    { 0x1fe685af, SUC_lexicon_tags_4 },
    { 0xc04685b2, SUC_lexicon_tags_57 },
    { 0x93e685b3, SUC_lexicon_tags_12 },
    { 0x706685b1, SUC_lexicon_tags_64 },
    { 0x3e6685b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b0685b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8685be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d4685c0, SUC_lexicon_tags_37 },
    { 0xaec685c1, SUC_lexicon_tags_63 },
    { 0xfd2685c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfac685c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd2685c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x268685c8, SUC_lexicon_tags_17 },
    { 0xfd6685c8, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce685cc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a685d0, SUC_lexicon_tags_12 },
    { 0x7cc685d1, SUC_lexicon_tags_39 },
    { 0x30c685d2, SUC_lexicon_tags_37 },
    { 0xf58685d0, SUC_lexicon_tags_25 },
    { 0xb6685d4, SUC_lexicon_tags_25 },
    { 0xf36685d4, SUC_lexicon_tags_11 },
    { 0xc74685d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344685e3, SUC_lexicon_tags_142 },
    { 0, NULL },
    { 0x2d0685e5, SUC_lexicon_tags_4 },
    { 0x1bc685e5, SUC_lexicon_tags_109 },
    { 0x1bc685e6, SUC_lexicon_tags_12 },
    { 0xdf4685e8, SUC_lexicon_tags_65 },
    { 0xe24685e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58685ed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c685f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08685fc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x604685fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba68604, SUC_lexicon_tags_39 },
    { 0xcae68604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c68608, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf706860b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac6860e, SUC_lexicon_tags_8 },
    { 0x6906860f, SUC_lexicon_tags_25 },
    { 0xcba68610, SUC_lexicon_tags_13 },
    { 0xdaa68610, SUC_lexicon_tags_12 },
    { 0xcd268612, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4668615, SUC_lexicon_tags_7 },
    { 0xe2c68616, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c68620, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d268622, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x76a68624, SUC_lexicon_tags_26 },
    { 0xf5868625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c68629, SUC_lexicon_tags_16 },
    { 0xfe26862a, SUC_lexicon_tags_17 },
    { 0xbb26862b, SUC_lexicon_tags_60 },
    { 0xd66862c, SUC_lexicon_tags_12 },
    { 0xf86862c, SUC_lexicon_tags_45 },
    { 0x9986862b, SUC_lexicon_tags_56 },
    { 0x4446862a, SUC_lexicon_tags_12 },
    { 0x5b668630, SUC_lexicon_tags_17 },
    { 0x70c68631, SUC_lexicon_tags_17 },
    { 0xf9068629, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc68639, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8706863e, SUC_lexicon_tags_25 },
    { 0x49a6863e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb068645, SUC_lexicon_tags_82 },
    { 0xde68646, SUC_lexicon_tags_25 },
    { 0xc7868646, SUC_lexicon_tags_23 },
    { 0xf2668648, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd6c6864a, SUC_lexicon_tags_15 },
    { 0x12e6864a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18c6864d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae68650, SUC_lexicon_tags_146 },
    { 0x22c68651, SUC_lexicon_tags_26 },
    { 0x8da68651, SUC_lexicon_tags_7 },
    { 0x56c68653, SUC_lexicon_tags_0 },
    { 0x64668654, SUC_lexicon_tags_54 },
    { 0x43468654, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce468658, SUC_lexicon_tags_33 },
    { 0x76c68659, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xaf86865b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xfb86865d, SUC_lexicon_tags_109 },
    { 0x8e86865d, SUC_lexicon_tags_12 },
    { 0x1486865f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb068661, SUC_lexicon_tags_29 },
    { 0x2ac68662, SUC_lexicon_tags_7 },
    { 0x2d268663, SUC_lexicon_tags_23 },
    { 0x97868664, SUC_lexicon_tags_15 },
    { 0xdca68664, SUC_lexicon_tags_17 },
    { 0x20268666, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xefa68668, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e68670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f868674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a68677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc46867e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa68682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2de68684, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4868687, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c66868f, SUC_lexicon_tags_22 },
    { 0x72c6868f, SUC_lexicon_tags_8 },
    { 0xea068691, SUC_lexicon_tags_15 },
    { 0xb7c68692, SUC_lexicon_tags_43 },
    { 0x50a68693, SUC_lexicon_tags_91 },
    { 0xef468693, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5268696, SUC_lexicon_tags_49 },
    { 0x3ee68697, SUC_lexicon_tags_2 },
    { 0x4f868698, SUC_lexicon_tags_55 },
    { 0xfec68696, SUC_lexicon_tags_27 },
    { 0x50a6869a, SUC_lexicon_tags_2 },
    { 0xf426869b, SUC_lexicon_tags_60 },
    { 0xe066869c, SUC_lexicon_tags_4 },
    { 0x2646869b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4686a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c686a3, SUC_lexicon_tags_38 },
    { 0x978686a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50686a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0686ad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678686b4, SUC_lexicon_tags_26 },
    { 0x3e0686b5, SUC_lexicon_tags_2 },
    { 0xdae686b4, SUC_lexicon_tags_37 },
    { 0x9c2686b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98686bd, SUC_lexicon_tags_27 },
    { 0xbe686be, SUC_lexicon_tags_26 },
    { 0x38686bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca686c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe88686c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x116686c6, SUC_lexicon_tags_10 },
    { 0x422686c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3c2686c9, SUC_lexicon_tags_2 },
    { 0x6de686ca, SUC_lexicon_tags_63 },
    { 0x59e686cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92686cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e686d5, SUC_lexicon_tags_49 },
    { 0xeca686d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2686da, SUC_lexicon_tags_17 },
    { 0xae8686db, SUC_lexicon_tags_1 },
    { 0xdf4686dc, SUC_lexicon_tags_7 },
    { 0xd36686dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x980686e0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xae6686e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c686e9, SUC_lexicon_tags_37 },
    { 0xe30686ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe686ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80686f4, SUC_lexicon_tags_0 },
    { 0x47c686f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2686fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a686ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fa68701, SUC_lexicon_tags_1 },
    { 0x81068701, SUC_lexicon_tags_15 },
    { 0xa2868703, SUC_lexicon_tags_32 },
    { 0xd6c68702, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x25a68706, SUC_lexicon_tags_62 },
    { 0xecc68706, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0e68709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7166870b, SUC_lexicon_tags_40 },
    { 0x1f46870b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c068712, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10868714, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe0268716, SUC_lexicon_tags_13 },
    { 0x5ca68717, SUC_lexicon_tags_25 },
    { 0xefe68717, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a6871e, SUC_lexicon_tags_33 },
    { 0xb2e6871e, SUC_lexicon_tags_60 },
    { 0xf0868720, SUC_lexicon_tags_25 },
    { 0x3c068721, SUC_lexicon_tags_12 },
    { 0xf766871f, SUC_lexicon_tags_11 },
    { 0x43068721, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c068725, SUC_lexicon_tags_13 },
    { 0xf2668726, SUC_lexicon_tags_68 },
    { 0x63668727, SUC_lexicon_tags_2 },
    { 0x87068725, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc6872b, SUC_lexicon_tags_7 },
    { 0x3da6872c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc366872e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a68733, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3268735, SUC_lexicon_tags_28 },
    { 0x6f868735, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a6873c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e26873f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a68744, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f268746, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb8868748, SUC_lexicon_tags_37 },
    { 0x7a68748, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc86874b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf766874e, SUC_lexicon_tags_12 },
    { 0x9246874f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b468755, SUC_lexicon_tags_19 },
    { 0xede68755, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x81668758, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf06875a, SUC_lexicon_tags_12 },
    { 0x5fe6875b, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e6875f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c68762, SUC_lexicon_tags_2 },
    { 0x90068763, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31868765, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28268767, SUC_lexicon_tags_12 },
    { 0xc5468767, SUC_lexicon_tags_60 },
    { 0x26c68769, SUC_lexicon_tags_26 },
    { 0x10668768, SUC_lexicon_tags_25 },
    { 0x2ac6876b, SUC_lexicon_tags_37 },
    { 0xea86876c, SUC_lexicon_tags_23 },
    { 0x46c6876a, SUC_lexicon_tags_29 },
    { 0x67a6876e, SUC_lexicon_tags_25 },
    { 0x7f068767, SUC_lexicon_tags_2 },
    { 0xbe868770, SUC_lexicon_tags_12 },
    { 0x48e68768, SUC_lexicon_tags_15 },
    { 0x4a66876c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc6877d, SUC_lexicon_tags_23 },
    { 0x5c26877d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28468780, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85868786, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a6878c, SUC_lexicon_tags_1 },
    { 0xd106878d, SUC_lexicon_tags_1 },
    { 0xc9a6878d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a68796, SUC_lexicon_tags_31 },
    { 0x54c68797, SUC_lexicon_tags_25 },
    { 0xeda68796, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be687a0, SUC_lexicon_tags_33 },
    { 0x3a2687a0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196687a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2687ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x830687ae, SUC_lexicon_tags_8 },
    { 0x466687af, SUC_lexicon_tags_11 },
    { 0x8fc687ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x804687b3, SUC_lexicon_tags_0 },
    { 0x6d0687b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe90687b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37c687b8, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x486687bb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8687bf, SUC_lexicon_tags_65 },
    { 0x7be687c0, SUC_lexicon_tags_47 },
    { 0x8b0687c1, SUC_lexicon_tags_17 },
    { 0xb9e687c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc12687c4, SUC_lexicon_tags_11 },
    { 0x5b0687c5, SUC_lexicon_tags_19 },
    { 0xb3a687c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6687cc, SUC_lexicon_tags_0 },
    { 0x446687cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88687cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564687d3, SUC_lexicon_tags_15 },
    { 0x226687d4, SUC_lexicon_tags_0 },
    { 0xde6687d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128687da, SUC_lexicon_tags_25 },
    { 0xf4c687da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c687df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26c687e1, SUC_lexicon_tags_0 },
    { 0x4e6687e2, SUC_lexicon_tags_17 },
    { 0xab4687e3, SUC_lexicon_tags_25 },
    { 0x530687e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f2687e6, SUC_lexicon_tags_1 },
    { 0xbf0687e7, SUC_lexicon_tags_2 },
    { 0xe00687e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522687ee, SUC_lexicon_tags_47 },
    { 0x88687ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ce687f1, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8687f5, SUC_lexicon_tags_60 },
    { 0x2e4687f6, SUC_lexicon_tags_17 },
    { 0x68a687f7, SUC_lexicon_tags_12 },
    { 0xb0687f8, SUC_lexicon_tags_60 },
    { 0x1fa687f9, SUC_lexicon_tags_47 },
    { 0xc24687fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a068801, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf346880d, SUC_lexicon_tags_1 },
    { 0xbda6880e, SUC_lexicon_tags_82 },
    { 0x4086880f, SUC_lexicon_tags_0 },
    { 0xa066880f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6868815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e46881a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e06881d, SUC_lexicon_tags_11 },
    { 0xe9c6881e, SUC_lexicon_tags_25 },
    { 0x4d66881f, SUC_lexicon_tags_14 },
    { 0xf9468820, SUC_lexicon_tags_30 },
    { 0x9668821, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6668824, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x82868826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa6882d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de68832, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28668834, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a668838, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8e6883a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2fe6883c, SUC_lexicon_tags_27 },
    { 0x1d06883d, SUC_lexicon_tags_1 },
    { 0x1a66883d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d468847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1e68849, SUC_lexicon_tags_48 },
    { 0x5106884a, SUC_lexicon_tags_17 },
    { 0x9dc6884b, SUC_lexicon_tags_12 },
    { 0xffc6884c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf166884f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb068851, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87268856, SUC_lexicon_tags_27 },
    { 0x90268857, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c6885c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d06885e, SUC_lexicon_tags_17 },
    { 0xae26885f, SUC_lexicon_tags_262 },
    { 0x80668860, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a068863, SUC_lexicon_tags_26 },
    { 0x2068864, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x1ca68866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb468868, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe366886a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e46886f, SUC_lexicon_tags_1 },
    { 0x28468870, SUC_lexicon_tags_55 },
    { 0x3f868871, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x28668873, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e68876, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c6887c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc226887f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc68884, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87668889, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa826888b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e68892, SUC_lexicon_tags_2 },
    { 0xd668893, SUC_lexicon_tags_37 },
    { 0x21a68894, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3c68896, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc266889c, SUC_lexicon_tags_45 },
    { 0x62a6889d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8688a0, SUC_lexicon_tags_84 },
    { 0xc48688a1, SUC_lexicon_tags_17 },
    { 0xd40688a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x614688a4, SUC_lexicon_tags_37 },
    { 0xa4a688a4, SUC_lexicon_tags_68 },
    { 0xcbc688a6, SUC_lexicon_tags_33 },
    { 0xc6e688a7, SUC_lexicon_tags_2 },
    { 0x326688a4, SUC_lexicon_tags_17 },
    { 0x730688a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc688ac, SUC_lexicon_tags_17 },
    { 0xa688ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a688b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a688b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca688bb, SUC_lexicon_tags_10 },
    { 0xd0c688bc, SUC_lexicon_tags_17 },
    { 0xce8688bc, SUC_lexicon_tags_158 },
    { 0x2e4688be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2e688c0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca688c5, SUC_lexicon_tags_33 },
    { 0x526688c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba688c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x51e688cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c6688cd, SUC_lexicon_tags_0 },
    { 0x7d6688ce, SUC_lexicon_tags_126 },
    { 0xb1c688cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x66688d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x842688d3, SUC_lexicon_tags_48 },
    { 0x3f8688d4, SUC_lexicon_tags_184 },
    { 0x3c8688d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc86688d7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0688da, SUC_lexicon_tags_10 },
    { 0xc88688db, SUC_lexicon_tags_33 },
    { 0xe74688dc, SUC_lexicon_tags_60 },
    { 0x538688dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ea688df, SUC_lexicon_tags_43 },
    { 0xed6688df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02688e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc88688e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a688e8, SUC_lexicon_tags_15 },
    { 0x3d8688e8, SUC_lexicon_tags_42 },
    { 0x64e688e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560688ef, SUC_lexicon_tags_17 },
    { 0x4d2688ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce688f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf72688f7, SUC_lexicon_tags_25 },
    { 0x3e4688f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd74688fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x894688fc, SUC_lexicon_tags_37 },
    { 0x1b8688fd, SUC_lexicon_tags_14 },
    { 0x6d0688fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a68900, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ba68902, SUC_lexicon_tags_11 },
    { 0xacc68903, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc6890a, SUC_lexicon_tags_11 },
    { 0x7d86890a, SUC_lexicon_tags_1 },
    { 0x8a06890c, SUC_lexicon_tags_27 },
    { 0x25c6890d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c68916, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78668919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1406891d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6746891f, SUC_lexicon_tags_17 },
    { 0x5d06891f, SUC_lexicon_tags_2 },
    { 0xf1a6891f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf668923, SUC_lexicon_tags_27 },
    { 0xa7668923, SUC_lexicon_tags_11 },
    { 0xcd268925, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xeb068927, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3f268929, SUC_lexicon_tags_7 },
    { 0x5706892a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef26892d, SUC_lexicon_tags_17 },
    { 0x6306892e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x22468931, SUC_lexicon_tags_15 },
    { 0xf8868932, SUC_lexicon_tags_27 },
    { 0x15c68931, SUC_lexicon_tags_26 },
    { 0xbd068934, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe468937, SUC_lexicon_tags_31 },
    { 0x9de68938, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e6893e, SUC_lexicon_tags_45 },
    { 0xac6893f, SUC_lexicon_tags_65 },
    { 0xe1c6893f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc68944, SUC_lexicon_tags_11 },
    { 0x72268944, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x22468948, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc6894b, SUC_lexicon_tags_19 },
    { 0x56e6894c, SUC_lexicon_tags_65 },
    { 0x1446894d, SUC_lexicon_tags_29 },
    { 0x9626894d, SUC_lexicon_tags_25 },
    { 0x5306894d, SUC_lexicon_tags_39 },
    { 0x80468950, SUC_lexicon_tags_47 },
    { 0xf0a68951, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x51e68953, SUC_lexicon_tags_7 },
    { 0xf4268953, SUC_lexicon_tags_19 },
    { 0xf3468955, SUC_lexicon_tags_1 },
    { 0xf8668956, SUC_lexicon_tags_28 },
    { 0x1fa68957, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106895e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49068960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73668964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a468967, SUC_lexicon_tags_17 },
    { 0xec668967, SUC_lexicon_tags_15 },
    { 0xa4068969, SUC_lexicon_tags_14 },
    { 0x74e68969, SUC_lexicon_tags_23 },
    { 0xbde6896a, SUC_lexicon_tags_17 },
    { 0x64268969, SUC_lexicon_tags_4 },
    { 0x766896d, SUC_lexicon_tags_44 },
    { 0xade6896b, SUC_lexicon_tags_12 },
    { 0xe4c6896f, SUC_lexicon_tags_6 },
    { 0x466896f, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a68973, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70a68975, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55868977, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6897c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58868982, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5268986, SUC_lexicon_tags_25 },
    { 0x29268987, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e268989, SUC_lexicon_tags_17 },
    { 0x9ca68989, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d86898c, SUC_lexicon_tags_12 },
    { 0x3e86898d, SUC_lexicon_tags_17 },
    { 0x65c6898c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6468992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54c68994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75c68996, SUC_lexicon_tags_17 },
    { 0x4ea68997, SUC_lexicon_tags_43 },
    { 0x57e68996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c06899a, SUC_lexicon_tags_34 },
    { 0x81a6899b, SUC_lexicon_tags_4 },
    { 0xc066899c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e689a2, SUC_lexicon_tags_31 },
    { 0x736689a2, SUC_lexicon_tags_25 },
    { 0x1ba689a3, SUC_lexicon_tags_49 },
    { 0x23a689a5, SUC_lexicon_tags_27 },
    { 0x7a689a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a689a9, SUC_lexicon_tags_37 },
    { 0xec689a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc08689ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6689b0, SUC_lexicon_tags_26 },
    { 0x898689b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62689b3, SUC_lexicon_tags_583 },
    { 0x752689b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06689b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa0e689b9, SUC_lexicon_tags_48 },
    { 0xeee689ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbac689bc, SUC_lexicon_tags_21 },
    { 0xcf0689bd, SUC_lexicon_tags_7 },
    { 0x91e689bd, SUC_lexicon_tags_85 },
    { 0x710689bd, SUC_lexicon_tags_12 },
    { 0xc62689c0, SUC_lexicon_tags_180 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e689c5, SUC_lexicon_tags_66 },
    { 0xa6e689c6, SUC_lexicon_tags_0 },
    { 0xd78689c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e689cb, SUC_lexicon_tags_40 },
    { 0x856689cb, SUC_lexicon_tags_37 },
    { 0xb96689cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8bc689cf, SUC_lexicon_tags_42 },
    { 0xc82689cf, SUC_lexicon_tags_37 },
    { 0x294689d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a689d6, SUC_lexicon_tags_14 },
    { 0xed4689d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30689dc, SUC_lexicon_tags_7 },
    { 0xc30689dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c689e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa12689e2, SUC_lexicon_tags_46 },
    { 0xfd2689e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8689e7, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x698689e9, SUC_lexicon_tags_12 },
    { 0x168689ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a0689ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e8689ee, SUC_lexicon_tags_49 },
    { 0xd66689ee, SUC_lexicon_tags_7 },
    { 0x1c689ee, SUC_lexicon_tags_9 },
    { 0x486689f1, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a689f6, SUC_lexicon_tags_47 },
    { 0x58e689f7, SUC_lexicon_tags_37 },
    { 0xccc689f6, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x440689fa, SUC_lexicon_tags_39 },
    { 0x938689fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e868a00, SUC_lexicon_tags_7 },
    { 0xd7a68a00, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x4ce68a03, SUC_lexicon_tags_38 },
    { 0x23868a04, SUC_lexicon_tags_12 },
    { 0x5fa68a05, SUC_lexicon_tags_17 },
    { 0x85468a05, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5668a0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e68a0f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92268a11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee68a14, SUC_lexicon_tags_45 },
    { 0xd4468a15, SUC_lexicon_tags_37 },
    { 0x29e68a16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeba68a18, SUC_lexicon_tags_48 },
    { 0xf7068a18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a68a1c, SUC_lexicon_tags_17 },
    { 0x97c68a1d, SUC_lexicon_tags_14 },
    { 0xa3468a1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d068a20, SUC_lexicon_tags_1 },
    { 0x4a68a21, SUC_lexicon_tags_217 },
    { 0x64c68a20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0868a2c, SUC_lexicon_tags_38 },
    { 0x14e68a2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2e68a2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc868a31, SUC_lexicon_tags_15 },
    { 0xb3668a32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11068a34, SUC_lexicon_tags_30 },
    { 0xcb468a34, SUC_lexicon_tags_7 },
    { 0xe668a36, SUC_lexicon_tags_12 },
    { 0x9468a37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a68a3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3068a3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46668a3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0068a42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x39868a44, SUC_lexicon_tags_16 },
    { 0xfa068a45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ee68a47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2868a49, SUC_lexicon_tags_11 },
    { 0x29068a4a, SUC_lexicon_tags_2 },
    { 0x24468a4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0a68a4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1068a4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd2a68a51, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c68a55, SUC_lexicon_tags_15 },
    { 0x95e68a55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea68a59, SUC_lexicon_tags_12 },
    { 0xf3868a5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8268a5c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc68a5f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce68a62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb6668a64, SUC_lexicon_tags_37 },
    { 0xfb068a65, SUC_lexicon_tags_49 },
    { 0x6dc68a66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7668a6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9fe68a6e, SUC_lexicon_tags_47 },
    { 0xef668a6f, SUC_lexicon_tags_275 },
    { 0x4d268a6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c268a72, SUC_lexicon_tags_12 },
    { 0xbc868a72, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x99868a76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a668a79, SUC_lexicon_tags_6 },
    { 0x53268a79, SUC_lexicon_tags_49 },
    { 0xd6668a7b, SUC_lexicon_tags_298 },
    { 0x2aa68a7c, SUC_lexicon_tags_26 },
    { 0xc1468a7d, SUC_lexicon_tags_47 },
    { 0x72268a7b, SUC_lexicon_tags_66 },
    { 0xbc668a7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d268a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e68a86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6868a8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec68a8f, SUC_lexicon_tags_25 },
    { 0x3a668a90, SUC_lexicon_tags_7 },
    { 0xb3468a90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be68a94, SUC_lexicon_tags_15 },
    { 0xe8868a95, SUC_lexicon_tags_37 },
    { 0x34268a96, SUC_lexicon_tags_37 },
    { 0xac268a95, SUC_lexicon_tags_7 },
    { 0x9a068a98, SUC_lexicon_tags_19 },
    { 0xc1868a99, SUC_lexicon_tags_23 },
    { 0x4c268a99, SUC_lexicon_tags_7 },
    { 0xbe68a96, SUC_lexicon_tags_2 },
    { 0xba468a9c, SUC_lexicon_tags_16 },
    { 0x84868a9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b468aa0, SUC_lexicon_tags_4 },
    { 0x7c468aa0, SUC_lexicon_tags_23 },
    { 0x7da68aa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c68aa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a068aa8, SUC_lexicon_tags_60 },
    { 0xdec68aa9, SUC_lexicon_tags_49 },
    { 0xb068aaa, SUC_lexicon_tags_63 },
    { 0xd5868aa8, SUC_lexicon_tags_1 },
    { 0xf1468aa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdca68aae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c68ab8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9268abb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c68abe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da68ac1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e68ac5, SUC_lexicon_tags_15 },
    { 0x6b468ac6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c68aca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc4668acc, SUC_lexicon_tags_80 },
    { 0x4e68acd, SUC_lexicon_tags_17 },
    { 0xece68ace, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a68ad7, SUC_lexicon_tags_14 },
    { 0xcc268ad7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57c68ada, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c68ae4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56268ae6, SUC_lexicon_tags_12 },
    { 0x5b068ae6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c68aea, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de68aed, SUC_lexicon_tags_38 },
    { 0xdf068aee, SUC_lexicon_tags_26 },
    { 0x48e68aef, SUC_lexicon_tags_33 },
    { 0x54a68af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0668af2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9468af4, SUC_lexicon_tags_11 },
    { 0x49e68af5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8468af7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a068afa, SUC_lexicon_tags_11 },
    { 0xede68afb, SUC_lexicon_tags_14 },
    { 0x34468afc, SUC_lexicon_tags_92 },
    { 0xcf068afc, SUC_lexicon_tags_3 },
    { 0x30068afc, SUC_lexicon_tags_66 },
    { 0x72668aff, SUC_lexicon_tags_7 },
    { 0xfe468aff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6468b03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59468b06, SUC_lexicon_tags_15 },
    { 0x3ee68b07, SUC_lexicon_tags_8 },
    { 0xcb668b08, SUC_lexicon_tags_7 },
    { 0xc0668b09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65068b0d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c68b16, SUC_lexicon_tags_12 },
    { 0xdec68b16, SUC_lexicon_tags_37 },
    { 0xc0c68b18, SUC_lexicon_tags_53 },
    { 0x53868b19, SUC_lexicon_tags_25 },
    { 0x5ca68b1a, SUC_lexicon_tags_87 },
    { 0x7b668b1a, SUC_lexicon_tags_45 },
    { 0xac68b1c, SUC_lexicon_tags_34 },
    { 0x4e68b17, SUC_lexicon_tags_1 },
    { 0xa2668b1d, SUC_lexicon_tags_37 },
    { 0xc3a68b1f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a68b22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x41e68b24, SUC_lexicon_tags_61 },
    { 0x75268b25, SUC_lexicon_tags_1 },
    { 0xb3e68b24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72a68b28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf2468b2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd468b2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cc68b2e, SUC_lexicon_tags_15 },
    { 0x42268b2e, SUC_lexicon_tags_2 },
    { 0x53868b2e, SUC_lexicon_tags_37 },
    { 0xbc068b31, SUC_lexicon_tags_14 },
    { 0x9aa68b32, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xff668b34, SUC_lexicon_tags_19 },
    { 0xa2068b35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3868b38, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c68b3c, SUC_lexicon_tags_25 },
    { 0xc7068b3c, SUC_lexicon_tags_2 },
    { 0x66c68b3e, SUC_lexicon_tags_23 },
    { 0x97c68b3f, SUC_lexicon_tags_37 },
    { 0xa6e68b40, SUC_lexicon_tags_31 },
    { 0x7c868b3e, SUC_lexicon_tags_38 },
    { 0x75668b42, SUC_lexicon_tags_37 },
    { 0x57668b42, SUC_lexicon_tags_198 },
    { 0xf3868b3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c68b4b, SUC_lexicon_tags_60 },
    { 0xc5e68b4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71068b4e, SUC_lexicon_tags_15 },
    { 0xd0a68b4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae68b56, SUC_lexicon_tags_7 },
    { 0x9a668b57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56e68b59, SUC_lexicon_tags_25 },
    { 0x93268b59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe6268b5c, SUC_lexicon_tags_26 },
    { 0x4c268b5d, SUC_lexicon_tags_85 },
    { 0x40268b5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa68b64, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de68b67, SUC_lexicon_tags_25 },
    { 0x83868b67, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a68b6b, SUC_lexicon_tags_37 },
    { 0xef868b6b, SUC_lexicon_tags_63 },
    { 0xc5e68b6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e68b72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da68b77, SUC_lexicon_tags_25 },
    { 0x9ee68b78, SUC_lexicon_tags_12 },
    { 0xd4a68b79, SUC_lexicon_tags_47 },
    { 0xf2668b79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1268b7c, SUC_lexicon_tags_45 },
    { 0xc8268b7c, SUC_lexicon_tags_15 },
    { 0x78e68b7d, SUC_lexicon_tags_19 },
    { 0x78668b7e, SUC_lexicon_tags_63 },
    { 0x16068b80, SUC_lexicon_tags_47 },
    { 0x72068b81, SUC_lexicon_tags_96 },
    { 0xc1c68b7c, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0x91268b84, SUC_lexicon_tags_61 },
    { 0xd1468b85, SUC_lexicon_tags_12 },
    { 0x84468b84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50468b88, SUC_lexicon_tags_37 },
    { 0x11268b88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43068b8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5ae68b90, SUC_lexicon_tags_77 },
    { 0x61668b91, SUC_lexicon_tags_34 },
    { 0x9d468b90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0668b9b, SUC_lexicon_tags_37 },
    { 0x87868b9b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e068ba2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1fe68ba4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e68ba8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f268bab, SUC_lexicon_tags_27 },
    { 0xfc868bac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72268baf, SUC_lexicon_tags_15 },
    { 0x7b468baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3868bb2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b068bb5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcc68bb7, SUC_lexicon_tags_59 },
    { 0x70468bb8, SUC_lexicon_tags_30 },
    { 0xbfc68bb9, SUC_lexicon_tags_15 },
    { 0xa0468bb9, SUC_lexicon_tags_7 },
    { 0xdb468bb8, SUC_lexicon_tags_25 },
    { 0xae668bbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0668bbe, SUC_lexicon_tags_2 },
    { 0x268bbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc68bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae68bc8, SUC_lexicon_tags_38 },
    { 0xe5268bc9, SUC_lexicon_tags_27 },
    { 0xd8a68bc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe68bcf, SUC_lexicon_tags_11 },
    { 0x5f268bd0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c68bdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e68be0, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xd468be2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ec68be4, SUC_lexicon_tags_15 },
    { 0x5a868be5, SUC_lexicon_tags_14 },
    { 0x62868be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da68beb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32468bf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a68bf3, SUC_lexicon_tags_7 },
    { 0xa6e68bf3, SUC_lexicon_tags_1 },
    { 0x42a68bf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87668bf7, SUC_lexicon_tags_31 },
    { 0x9ac68bf8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c268bff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c68c01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x92e68c03, SUC_lexicon_tags_11 },
    { 0x2ec68c03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa668c09, SUC_lexicon_tags_37 },
    { 0x6c868c0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20668c0d, SUC_lexicon_tags_37 },
    { 0xcc268c0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9468c11, SUC_lexicon_tags_43 },
    { 0x51c68c12, SUC_lexicon_tags_37 },
    { 0xaac68c13, SUC_lexicon_tags_39 },
    { 0xe5268c14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11868c18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a68c1e, SUC_lexicon_tags_170 },
    { 0x51c68c1f, SUC_lexicon_tags_26 },
    { 0x90068c1e, SUC_lexicon_tags_28 },
    { 0xca868c21, SUC_lexicon_tags_61 },
    { 0x2d868c22, SUC_lexicon_tags_37 },
    { 0x5868c22, SUC_lexicon_tags_183 },
    { 0xd0e68c24, SUC_lexicon_tags_1 },
    { 0x25e68c25, SUC_lexicon_tags_9 },
    { 0x5068c26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d668c28, SUC_lexicon_tags_501 },
    { 0x1ca68c29, SUC_lexicon_tags_15 },
    { 0xfac68c2a, SUC_lexicon_tags_15 },
    { 0xa1c68c2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72468c32, SUC_lexicon_tags_15 },
    { 0xa3e68c32, SUC_lexicon_tags_0 },
    { 0x85c68c34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x56468c37, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf4868c39, SUC_lexicon_tags_14 },
    { 0xb6c68c3a, SUC_lexicon_tags_672 },
    { 0xfe468c39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40468c3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a268c41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0268c44, SUC_lexicon_tags_25 },
    { 0x11468c44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae68c4d, SUC_lexicon_tags_8 },
    { 0xd9868c4e, SUC_lexicon_tags_37 },
    { 0xb3468c4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3c68c51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6e68c53, SUC_lexicon_tags_15 },
    { 0xf3868c54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd9c68c56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a68c59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0668c5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86068c63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7068c67, SUC_lexicon_tags_11 },
    { 0x5d268c67, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x2d268c6a, SUC_lexicon_tags_62 },
    { 0x3ba68c6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b068c6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a868c75, SUC_lexicon_tags_13 },
    { 0x78068c76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb868c79, SUC_lexicon_tags_13 },
    { 0xa5e68c79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f468c7c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeda68c7e, SUC_lexicon_tags_19 },
    { 0x84c68c7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd068c81, SUC_lexicon_tags_43 },
    { 0xdc068c81, SUC_lexicon_tags_19 },
    { 0x68668c83, SUC_lexicon_tags_37 },
    { 0x8c668c82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ba68c86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f268c8b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf068c8e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd3468c90, SUC_lexicon_tags_14 },
    { 0x55068c91, SUC_lexicon_tags_21 },
    { 0x76068c92, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdc068c94, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc468c96, SUC_lexicon_tags_1 },
    { 0x67868c97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c268c9e, SUC_lexicon_tags_31 },
    { 0x48c68c9f, SUC_lexicon_tags_7 },
    { 0xeca68c9f, SUC_lexicon_tags_23 },
    { 0x88068c9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae68ca4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x58e68ca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99268ca8, SUC_lexicon_tags_66 },
    { 0x4de68ca9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e68cac, SUC_lexicon_tags_25 },
    { 0xc8c68cad, SUC_lexicon_tags_25 },
    { 0xdf868cad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0468cb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2868cb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84868cb6, SUC_lexicon_tags_2 },
    { 0x9fc68cb7, SUC_lexicon_tags_7 },
    { 0x2ae68cb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba68cbd, SUC_lexicon_tags_37 },
    { 0x9068cbe, SUC_lexicon_tags_25 },
    { 0x74268cbf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f268cc6, SUC_lexicon_tags_16 },
    { 0x98868cc7, SUC_lexicon_tags_12 },
    { 0x27868cc6, SUC_lexicon_tags_25 },
    { 0xf0868cc9, SUC_lexicon_tags_7 },
    { 0x1c068cca, SUC_lexicon_tags_1 },
    { 0x97a68ccb, SUC_lexicon_tags_21 },
    { 0x54068cc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17668cd3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c68cd9, SUC_lexicon_tags_79 },
    { 0x58068cd9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c68cdf, SUC_lexicon_tags_3 },
    { 0x7b468ce0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7268ce4, SUC_lexicon_tags_23 },
    { 0x5b668ce5, SUC_lexicon_tags_15 },
    { 0xbfe68ce6, SUC_lexicon_tags_13 },
    { 0xe5068ce7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a668ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb068ced, SUC_lexicon_tags_46 },
    { 0xa1468cee, SUC_lexicon_tags_25 },
    { 0x88c68cee, SUC_lexicon_tags_7 },
    { 0x75268cee, SUC_lexicon_tags_25 },
    { 0xd8c68cf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12c68cf3, SUC_lexicon_tags_7 },
    { 0xd9a68cf3, SUC_lexicon_tags_12 },
    { 0x89868cf5, SUC_lexicon_tags_37 },
    { 0x62868cf6, SUC_lexicon_tags_1 },
    { 0x90e68cf7, SUC_lexicon_tags_11 },
    { 0x1f668cf7, SUC_lexicon_tags_68 },
    { 0xe2e68cf5, SUC_lexicon_tags_31 },
    { 0x868cfa, SUC_lexicon_tags_1 },
    { 0xf5e68cfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95268d01, SUC_lexicon_tags_65 },
    { 0x8a68d01, SUC_lexicon_tags_25 },
    { 0x53268d03, SUC_lexicon_tags_37 },
    { 0xf2268d03, SUC_lexicon_tags_30 },
    { 0xcbe68d05, SUC_lexicon_tags_37 },
    { 0xa7068d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc68d0d, SUC_lexicon_tags_3 },
    { 0x27268d0e, SUC_lexicon_tags_15 },
    { 0xb0e68d0f, SUC_lexicon_tags_30 },
    { 0xac468d0d, SUC_lexicon_tags_19 },
    { 0xc5468d11, SUC_lexicon_tags_17 },
    { 0x32668d12, SUC_lexicon_tags_19 },
    { 0xc0468d11, SUC_lexicon_tags_10 },
    { 0x44268d12, SUC_lexicon_tags_1 },
    { 0x97e68d10, SUC_lexicon_tags_17 },
    { 0xba468d11, SUC_lexicon_tags_17 },
    { 0x30268d17, SUC_lexicon_tags_7 },
    { 0x1d668d18, SUC_lexicon_tags_26 },
    { 0xfea68d18, SUC_lexicon_tags_60 },
    { 0x2f668d1a, SUC_lexicon_tags_30 },
    { 0x93668d0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x14868d1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49068d1f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc68d24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a68d27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9068d29, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x53468d2b, SUC_lexicon_tags_70 },
    { 0x6a268d2c, SUC_lexicon_tags_49 },
    { 0x68068d2d, SUC_lexicon_tags_37 },
    { 0xf4868d2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36a68d30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1268d37, SUC_lexicon_tags_70 },
    { 0xa0c68d38, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e68d3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89668d40, SUC_lexicon_tags_5 },
    { 0x7a268d41, SUC_lexicon_tags_17 },
    { 0xf5c68d42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d268d47, SUC_lexicon_tags_11 },
    { 0x88e68d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65068d4b, SUC_lexicon_tags_12 },
    { 0xce868d4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41a68d4e, SUC_lexicon_tags_49 },
    { 0x4de68d4e, SUC_lexicon_tags_26 },
    { 0x29868d4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a68d55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x32268d58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68d5b, SUC_lexicon_tags_27 },
    { 0x19868d5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f468d61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57468d69, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a468d6d, SUC_lexicon_tags_6 },
    { 0xd3068d6d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa8868d70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x41268d72, SUC_lexicon_tags_6 },
    { 0x3b668d73, SUC_lexicon_tags_12 },
    { 0xfb668d74, SUC_lexicon_tags_47 },
    { 0xa7868d75, SUC_lexicon_tags_0 },
    { 0x82c68d76, SUC_lexicon_tags_7 },
    { 0x9e868d77, SUC_lexicon_tags_65 },
    { 0x53a68d78, SUC_lexicon_tags_30 },
    { 0x6f468d76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x14468d7b, SUC_lexicon_tags_31 },
    { 0x34e68d7c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2cc68d7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e068d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18468d86, SUC_lexicon_tags_15 },
    { 0xfd668d87, SUC_lexicon_tags_6 },
    { 0xeb868d88, SUC_lexicon_tags_11 },
    { 0xd0068d87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d868d8b, SUC_lexicon_tags_11 },
    { 0x5bc68d8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd468d8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc268d92, SUC_lexicon_tags_8 },
    { 0x76268d93, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c068d99, SUC_lexicon_tags_38 },
    { 0xa4868d99, SUC_lexicon_tags_6 },
    { 0x4468d9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeac68d9d, SUC_lexicon_tags_2 },
    { 0xd7a68d9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67068da0, SUC_lexicon_tags_36 },
    { 0xf4e68da1, SUC_lexicon_tags_14 },
    { 0xd8c68da2, SUC_lexicon_tags_15 },
    { 0x8c468da1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad268da7, SUC_lexicon_tags_15 },
    { 0x28268da7, SUC_lexicon_tags_37 },
    { 0x7c068da9, SUC_lexicon_tags_27 },
    { 0xaec68da8, SUC_lexicon_tags_8 },
    { 0x84e68daa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c668db0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7268db3, SUC_lexicon_tags_19 },
    { 0x3c468db3, SUC_lexicon_tags_9 },
    { 0xf2268db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82468db9, SUC_lexicon_tags_26 },
    { 0x48c68dba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35068dbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30a68dc0, SUC_lexicon_tags_11 },
    { 0x5b468dc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18c68dc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10268dc8, SUC_lexicon_tags_11 },
    { 0x18e68dc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c068dcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3068dce, SUC_lexicon_tags_27 },
    { 0x6b468dcf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e68dd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81468dd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81c68dd6, SUC_lexicon_tags_2 },
    { 0xbe268dd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a868ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0868ddd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f268de0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2868de3, SUC_lexicon_tags_64 },
    { 0x80a68de4, SUC_lexicon_tags_1 },
    { 0xc3a68de5, SUC_lexicon_tags_136 },
    { 0x44268de6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa68dea, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xd1068dec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff668df3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13468df5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8a268df7, SUC_lexicon_tags_17 },
    { 0x97868df8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a68dfe, SUC_lexicon_tags_1 },
    { 0xb5068dff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c68e04, SUC_lexicon_tags_38 },
    { 0x50668e04, SUC_lexicon_tags_11 },
    { 0x9468e06, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcda68e08, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4268e0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5668e16, SUC_lexicon_tags_38 },
    { 0x10268e16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6e868e19, SUC_lexicon_tags_25 },
    { 0xd9e68e1a, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xa668e1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29068e20, SUC_lexicon_tags_25 },
    { 0x6ae68e21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0e68e23, SUC_lexicon_tags_10 },
    { 0xda468e23, SUC_lexicon_tags_18 },
    { 0x3bc68e25, SUC_lexicon_tags_38 },
    { 0x77468e25, SUC_lexicon_tags_29 },
    { 0xa668e26, SUC_lexicon_tags_42 },
    { 0xc0a68e26, SUC_lexicon_tags_1 },
    { 0x89c68e29, SUC_lexicon_tags_31 },
    { 0xfbe68e29, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0268e30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xde668e32, SUC_lexicon_tags_12 },
    { 0x65c68e33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ec68e35, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a68e38, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdae68e3a, SUC_lexicon_tags_1 },
    { 0x82268e3b, SUC_lexicon_tags_15 },
    { 0x8268e3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3068e3f, SUC_lexicon_tags_37 },
    { 0x16a68e3f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a068e44, SUC_lexicon_tags_9 },
    { 0x81e68e45, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x23268e47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x49068e4a, SUC_lexicon_tags_10 },
    { 0x6ba68e4b, SUC_lexicon_tags_49 },
    { 0xd9068e4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xad268e4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed668e51, SUC_lexicon_tags_11 },
    { 0x1a68e52, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa0a68e54, SUC_lexicon_tags_11 },
    { 0xc2668e54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4268e57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb868e68, SUC_lexicon_tags_18 },
    { 0x51468e69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62e68e6b, SUC_lexicon_tags_47 },
    { 0x31068e6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9668e70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b068e72, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad668e76, SUC_lexicon_tags_15 },
    { 0xc7268e77, SUC_lexicon_tags_40 },
    { 0xd7868e78, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1468e7c, SUC_lexicon_tags_25 },
    { 0x5ca68e7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa1868e7f, SUC_lexicon_tags_14 },
    { 0x95a68e80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77668e86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13268e8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc868e8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94268e91, SUC_lexicon_tags_25 },
    { 0x2a68e92, SUC_lexicon_tags_47 },
    { 0x6ee68e93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80068e95, SUC_lexicon_tags_6 },
    { 0x27c68e96, SUC_lexicon_tags_25 },
    { 0xa4268e97, SUC_lexicon_tags_79 },
    { 0x82c68e98, SUC_lexicon_tags_9 },
    { 0x90068e96, SUC_lexicon_tags_45 },
    { 0x1be68e9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa68e9f, SUC_lexicon_tags_25 },
    { 0x38a68ea0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdba68ea2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbe268ea4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x18668ea7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25268eaa, SUC_lexicon_tags_39 },
    { 0xc0068eaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c68eae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a268eb2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c868eb7, SUC_lexicon_tags_11 },
    { 0x25a68eb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26868ebc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcde68ebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xefe68ec0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x33868ec3, SUC_lexicon_tags_70 },
    { 0x75868ec4, SUC_lexicon_tags_15 },
    { 0xc2c68ec5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7b468ec7, SUC_lexicon_tags_27 },
    { 0xfa268ec7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6868ed0, SUC_lexicon_tags_31 },
    { 0x2f268ed0, SUC_lexicon_tags_16 },
    { 0xfb668ed2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x61268ed4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50c68ed6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5868eda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fc68edc, SUC_lexicon_tags_63 },
    { 0xc3468edd, SUC_lexicon_tags_2 },
    { 0x54668ede, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x13268ee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d668ee3, SUC_lexicon_tags_39 },
    { 0x84468ee3, SUC_lexicon_tags_1 },
    { 0x17a68ee3, SUC_lexicon_tags_68 },
    { 0xdf468ee6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c68eef, SUC_lexicon_tags_82 },
    { 0xbf868eef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96668ef7, SUC_lexicon_tags_74 },
    { 0x3e668ef7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba68f01, SUC_lexicon_tags_37 },
    { 0xda468f02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8068f06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94e68f08, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55468f0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1468f0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x55468f10, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc68f13, SUC_lexicon_tags_37 },
    { 0x47668f14, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3268f16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e068f19, SUC_lexicon_tags_68 },
    { 0xc1e68f1a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da68f20, SUC_lexicon_tags_11 },
    { 0x87468f21, SUC_lexicon_tags_37 },
    { 0xf6868f22, SUC_lexicon_tags_11 },
    { 0x95868f23, SUC_lexicon_tags_15 },
    { 0xa0e68f22, SUC_lexicon_tags_8 },
    { 0x27868f21, SUC_lexicon_tags_60 },
    { 0x61468f21, SUC_lexicon_tags_6 },
    { 0x33668f24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64e68f29, SUC_lexicon_tags_12 },
    { 0xcee68f2a, SUC_lexicon_tags_37 },
    { 0x8be68f2b, SUC_lexicon_tags_849 },
    { 0, NULL },
    { 0xd5068f2d, SUC_lexicon_tags_11 },
    { 0xab668f2d, SUC_lexicon_tags_38 },
    { 0xf0668f2e, SUC_lexicon_tags_37 },
    { 0x4b468f30, SUC_lexicon_tags_7 },
    { 0x35468f31, SUC_lexicon_tags_25 },
    { 0xa2a68f2f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e68f38, SUC_lexicon_tags_25 },
    { 0x73a68f39, SUC_lexicon_tags_63 },
    { 0xef468f3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24268f41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x57668f44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfc68f46, SUC_lexicon_tags_12 },
    { 0x71a68f47, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xffa68f49, SUC_lexicon_tags_79 },
    { 0x3d668f4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98c68f4c, SUC_lexicon_tags_11 },
    { 0x91668f4c, SUC_lexicon_tags_12 },
    { 0xbb268f4e, SUC_lexicon_tags_38 },
    { 0xb9868f4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21068f5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x16468f5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36668f64, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d468f70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8be68f72, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c268f77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf068f7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76268f7f, SUC_lexicon_tags_88 },
    { 0x6ca68f80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5a68f82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16668f84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57468f86, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c68f89, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e68f91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50068f93, SUC_lexicon_tags_37 },
    { 0xa6268f94, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11268f9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fa6, SUC_lexicon_tags_37 },
    { 0x40a68fa7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a68faa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a068faf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0868fb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc68fbd, SUC_lexicon_tags_15 },
    { 0x70068fbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e68fc1, SUC_lexicon_tags_23 },
    { 0x34c68fc1, SUC_lexicon_tags_60 },
    { 0x43268fc3, SUC_lexicon_tags_87 },
    { 0xd8668fc4, SUC_lexicon_tags_4 },
    { 0x3f268fc5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57268fc9, SUC_lexicon_tags_14 },
    { 0x3fc68fc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe868fce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad468fd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a68fd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a68fd9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9468fdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c68fe3, SUC_lexicon_tags_11 },
    { 0x2268fe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b868ff1, SUC_lexicon_tags_15 },
    { 0xee468ff2, SUC_lexicon_tags_68 },
    { 0x40468ff2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b068ff5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec68ff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x80268ffc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfa468ffe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7269000, SUC_lexicon_tags_77 },
    { 0xde869001, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35469005, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfea69007, SUC_lexicon_tags_69 },
    { 0x73069008, SUC_lexicon_tags_37 },
    { 0x3b269009, SUC_lexicon_tags_12 },
    { 0xabc6900a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb326900c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xda6900e, SUC_lexicon_tags_2 },
    { 0x4ee6900f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72269012, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9269016, SUC_lexicon_tags_167 },
    { 0xf2c69016, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c6901b, SUC_lexicon_tags_7 },
    { 0x53c6901c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a69021, SUC_lexicon_tags_24 },
    { 0xe0869022, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb069025, SUC_lexicon_tags_34 },
    { 0x68669026, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10269029, SUC_lexicon_tags_26 },
    { 0x26c69029, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e6902e, SUC_lexicon_tags_17 },
    { 0x366902e, SUC_lexicon_tags_69 },
    { 0xfc669030, SUC_lexicon_tags_55 },
    { 0x8926902e, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb869038, SUC_lexicon_tags_14 },
    { 0x14869038, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d86904a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa346904d, SUC_lexicon_tags_0 },
    { 0x3446904e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x27069050, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e69058, SUC_lexicon_tags_37 },
    { 0x8ae69059, SUC_lexicon_tags_17 },
    { 0x4ea69059, SUC_lexicon_tags_65 },
    { 0x87069058, SUC_lexicon_tags_21 },
    { 0x2486905c, SUC_lexicon_tags_37 },
    { 0xc966905d, SUC_lexicon_tags_23 },
    { 0x64e6905e, SUC_lexicon_tags_2 },
    { 0x1726905e, SUC_lexicon_tags_63 },
    { 0x9606905a, SUC_lexicon_tags_38 },
    { 0xe4669058, SUC_lexicon_tags_32 },
    { 0xcf869062, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x11269064, SUC_lexicon_tags_37 },
    { 0xed669064, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae69068, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8f06906a, SUC_lexicon_tags_17 },
    { 0xa446906b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a6906e, SUC_lexicon_tags_74 },
    { 0xd846906f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad469072, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47069074, SUC_lexicon_tags_25 },
    { 0x82869074, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e69078, SUC_lexicon_tags_82 },
    { 0x4ee69079, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa566907c, SUC_lexicon_tags_1 },
    { 0x5de6907d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa646907f, SUC_lexicon_tags_60 },
    { 0x48c69080, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x6c669082, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f669085, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1069087, SUC_lexicon_tags_39 },
    { 0x6bc69088, SUC_lexicon_tags_8 },
    { 0xf8a69089, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2546908b, SUC_lexicon_tags_34 },
    { 0xce06908c, SUC_lexicon_tags_25 },
    { 0xc906908d, SUC_lexicon_tags_39 },
    { 0xad66908b, SUC_lexicon_tags_21 },
    { 0xb326908d, SUC_lexicon_tags_32 },
    { 0x8426908b, SUC_lexicon_tags_69 },
    { 0xc3469091, SUC_lexicon_tags_6 },
    { 0xac069090, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5869095, SUC_lexicon_tags_47 },
    { 0x67069095, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e6909a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf2a6909c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966690a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8690a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa690ae, SUC_lexicon_tags_12 },
    { 0x31c690af, SUC_lexicon_tags_31 },
    { 0xb7e690b0, SUC_lexicon_tags_25 },
    { 0x570690b1, SUC_lexicon_tags_27 },
    { 0x86c690ae, SUC_lexicon_tags_5 },
    { 0x504690b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8690b6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96690ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502690be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538690c2, SUC_lexicon_tags_46 },
    { 0x306690c3, SUC_lexicon_tags_11 },
    { 0xaa690c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e690c7, SUC_lexicon_tags_1 },
    { 0x5fa690c8, SUC_lexicon_tags_1 },
    { 0xea0690c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfee690cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e690d3, SUC_lexicon_tags_11 },
    { 0xdda690d3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x176690d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0690dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe50690df, SUC_lexicon_tags_6 },
    { 0x482690df, SUC_lexicon_tags_37 },
    { 0xe16690e1, SUC_lexicon_tags_6 },
    { 0x29c690e2, SUC_lexicon_tags_37 },
    { 0x358690e0, SUC_lexicon_tags_15 },
    { 0x24c690e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c690e7, SUC_lexicon_tags_1 },
    { 0xb0a690e8, SUC_lexicon_tags_11 },
    { 0xdea690e9, SUC_lexicon_tags_15 },
    { 0xdb0690e7, SUC_lexicon_tags_47 },
    { 0xee690eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c690ee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb30690f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x374690f3, SUC_lexicon_tags_6 },
    { 0x208690f3, SUC_lexicon_tags_16 },
    { 0xcbe690f5, SUC_lexicon_tags_37 },
    { 0x404690f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69101, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58669105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c6910d, SUC_lexicon_tags_38 },
    { 0xae06910e, SUC_lexicon_tags_47 },
    { 0xc906910e, SUC_lexicon_tags_37 },
    { 0xa246910f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80269115, SUC_lexicon_tags_12 },
    { 0x8ec69116, SUC_lexicon_tags_33 },
    { 0xba669117, SUC_lexicon_tags_23 },
    { 0xa4e69118, SUC_lexicon_tags_15 },
    { 0xf7a69119, SUC_lexicon_tags_25 },
    { 0x58469118, SUC_lexicon_tags_37 },
    { 0xe2069116, SUC_lexicon_tags_68 },
    { 0xd066911c, SUC_lexicon_tags_32 },
    { 0x8c46911c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89469123, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86c69125, SUC_lexicon_tags_4 },
    { 0xfbc69125, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c6912c, SUC_lexicon_tags_0 },
    { 0x9766912c, SUC_lexicon_tags_38 },
    { 0xa506912e, SUC_lexicon_tags_7 },
    { 0x8026912f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1269131, SUC_lexicon_tags_68 },
    { 0xfdc69132, SUC_lexicon_tags_32 },
    { 0x5669132, SUC_lexicon_tags_0 },
    { 0xf069131, SUC_lexicon_tags_42 },
    { 0x7ae69131, SUC_lexicon_tags_103 },
    { 0x59a69134, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f669138, SUC_lexicon_tags_23 },
    { 0x9e669139, SUC_lexicon_tags_19 },
    { 0xe3c69139, SUC_lexicon_tags_37 },
    { 0xca6913b, SUC_lexicon_tags_15 },
    { 0x7d86913c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc6913f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c69143, SUC_lexicon_tags_14 },
    { 0x61469144, SUC_lexicon_tags_25 },
    { 0x1cc69145, SUC_lexicon_tags_1 },
    { 0x2d269145, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12e69148, SUC_lexicon_tags_17 },
    { 0x1e669149, SUC_lexicon_tags_45 },
    { 0x7f869149, SUC_lexicon_tags_30 },
    { 0xd106914b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7686914e, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x42669151, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36269157, SUC_lexicon_tags_1 },
    { 0xd1a69158, SUC_lexicon_tags_63 },
    { 0x90869157, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xbb46915b, SUC_lexicon_tags_55 },
    { 0x67e6915b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x646915f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47869161, SUC_lexicon_tags_46 },
    { 0xb4269162, SUC_lexicon_tags_208 },
    { 0x25069162, SUC_lexicon_tags_11 },
    { 0x42469161, SUC_lexicon_tags_12 },
    { 0xadc69165, SUC_lexicon_tags_11 },
    { 0xb5069166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda669168, SUC_lexicon_tags_37 },
    { 0xae069168, SUC_lexicon_tags_15 },
    { 0x12c69169, SUC_lexicon_tags_43 },
    { 0xd746916a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84a6916d, SUC_lexicon_tags_11 },
    { 0xcb46916e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35069173, SUC_lexicon_tags_11 },
    { 0xa8069174, SUC_lexicon_tags_23 },
    { 0x869174, SUC_lexicon_tags_11 },
    { 0x83669176, SUC_lexicon_tags_23 },
    { 0x5ea69175, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcbe69179, SUC_lexicon_tags_27 },
    { 0xcfe69179, SUC_lexicon_tags_1 },
    { 0x63869179, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d66917e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6a69180, SUC_lexicon_tags_19 },
    { 0x21669181, SUC_lexicon_tags_43 },
    { 0x7c069182, SUC_lexicon_tags_29 },
    { 0xcbe69181, SUC_lexicon_tags_31 },
    { 0x49c69184, SUC_lexicon_tags_37 },
    { 0x13a69185, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b66918a, SUC_lexicon_tags_11 },
    { 0x6c86918b, SUC_lexicon_tags_63 },
    { 0xe086918a, SUC_lexicon_tags_1 },
    { 0xff66918b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa69193, SUC_lexicon_tags_25 },
    { 0x1f669194, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43269198, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5866919f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa48691a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe42691a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a691aa, SUC_lexicon_tags_17 },
    { 0xbb4691ab, SUC_lexicon_tags_25 },
    { 0xb2a691ac, SUC_lexicon_tags_47 },
    { 0xba8691ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22691b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x686691b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac691b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2691c2, SUC_lexicon_tags_0 },
    { 0xe82691c2, SUC_lexicon_tags_9 },
    { 0x502691c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e0691c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a691cc, SUC_lexicon_tags_37 },
    { 0xcc6691cd, SUC_lexicon_tags_82 },
    { 0xf6e691ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x394691d1, SUC_lexicon_tags_37 },
    { 0xd5e691d2, SUC_lexicon_tags_37 },
    { 0xccc691d3, SUC_lexicon_tags_28 },
    { 0x1a6691d3, SUC_lexicon_tags_17 },
    { 0x11e691d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xec6691d7, SUC_lexicon_tags_612 },
    { 0x6d6691d7, SUC_lexicon_tags_8 },
    { 0x990691d9, SUC_lexicon_tags_24 },
    { 0xfbc691da, SUC_lexicon_tags_23 },
    { 0x210691d9, SUC_lexicon_tags_1 },
    { 0xb38691dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1ce691de, SUC_lexicon_tags_2 },
    { 0xe5a691de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24e691e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6691e4, SUC_lexicon_tags_0 },
    { 0xea0691e5, SUC_lexicon_tags_8 },
    { 0x16e691e6, SUC_lexicon_tags_17 },
    { 0xcb2691e6, SUC_lexicon_tags_2 },
    { 0x691e6, SUC_lexicon_tags_29 },
    { 0x6c8691e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbc691eb, SUC_lexicon_tags_21 },
    { 0x691ec, SUC_lexicon_tags_29 },
    { 0x844691eb, SUC_lexicon_tags_19 },
    { 0x1c6691eb, SUC_lexicon_tags_39 },
    { 0x30691ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86691f3, SUC_lexicon_tags_12 },
    { 0x6d6691f3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4691fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da69200, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb8069202, SUC_lexicon_tags_68 },
    { 0x19c69203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a869208, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0x5c66920a, SUC_lexicon_tags_68 },
    { 0xd8e6920b, SUC_lexicon_tags_16 },
    { 0xb766920b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6206920e, SUC_lexicon_tags_7 },
    { 0x7046920f, SUC_lexicon_tags_2 },
    { 0xf9269210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37869212, SUC_lexicon_tags_48 },
    { 0x7fc69212, SUC_lexicon_tags_7 },
    { 0x23c69214, SUC_lexicon_tags_15 },
    { 0x57e69215, SUC_lexicon_tags_7 },
    { 0xa9e69216, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36469218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c6921f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51269223, SUC_lexicon_tags_10 },
    { 0xdc069224, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b469226, SUC_lexicon_tags_59 },
    { 0xe8469227, SUC_lexicon_tags_25 },
    { 0x75c69227, SUC_lexicon_tags_29 },
    { 0xb7869229, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e6922e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5469232, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2669235, SUC_lexicon_tags_185 },
    { 0x3e269236, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbaa69238, SUC_lexicon_tags_15 },
    { 0x49669239, SUC_lexicon_tags_37 },
    { 0xc966923a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe886923c, SUC_lexicon_tags_39 },
    { 0xcac6923d, SUC_lexicon_tags_46 },
    { 0x326923e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3069244, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee86924c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a69251, SUC_lexicon_tags_133 },
    { 0xfee69252, SUC_lexicon_tags_17 },
    { 0xd0e69252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6a69255, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb469258, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c6925b, SUC_lexicon_tags_25 },
    { 0xd726925b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2646925f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c069261, SUC_lexicon_tags_16 },
    { 0xefc69262, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e69265, SUC_lexicon_tags_27 },
    { 0x9d269266, SUC_lexicon_tags_60 },
    { 0x6ee69267, SUC_lexicon_tags_11 },
    { 0x19869267, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b66926a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98e6926c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a69273, SUC_lexicon_tags_43 },
    { 0xb2669274, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x61069277, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce06927b, SUC_lexicon_tags_30 },
    { 0x24c6927c, SUC_lexicon_tags_12 },
    { 0x65e6927d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89669282, SUC_lexicon_tags_15 },
    { 0xaca69283, SUC_lexicon_tags_15 },
    { 0xd2a69283, SUC_lexicon_tags_43 },
    { 0x24869285, SUC_lexicon_tags_13 },
    { 0xa1e69286, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e6928e, SUC_lexicon_tags_43 },
    { 0x6346928f, SUC_lexicon_tags_37 },
    { 0xf0a69290, SUC_lexicon_tags_37 },
    { 0xaa269290, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb469294, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb946929c, SUC_lexicon_tags_31 },
    { 0x6c66929c, SUC_lexicon_tags_37 },
    { 0xe846929d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc692a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c0692a4, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92692a8, SUC_lexicon_tags_12 },
    { 0xeb6692a8, SUC_lexicon_tags_43 },
    { 0xebe692aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e692b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6692b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90692b8, SUC_lexicon_tags_23 },
    { 0x6e2692b8, SUC_lexicon_tags_47 },
    { 0xec6692b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66692bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe86692bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x264692c2, SUC_lexicon_tags_0 },
    { 0x82c692c2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452692c9, SUC_lexicon_tags_60 },
    { 0x700692c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ca692cc, SUC_lexicon_tags_11 },
    { 0xd88692cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca692d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62692e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a692e4, SUC_lexicon_tags_55 },
    { 0x438692e4, SUC_lexicon_tags_1 },
    { 0x6692e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc692e9, SUC_lexicon_tags_17 },
    { 0xbbc692ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a692ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a692f9, SUC_lexicon_tags_47 },
    { 0x90e692fa, SUC_lexicon_tags_45 },
    { 0xcec692f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x88a692fd, SUC_lexicon_tags_11 },
    { 0xe02692fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1669301, SUC_lexicon_tags_12 },
    { 0xf7269301, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da69305, SUC_lexicon_tags_1 },
    { 0xb4a69306, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb366930c, SUC_lexicon_tags_26 },
    { 0xe066930c, SUC_lexicon_tags_0 },
    { 0xf706930d, SUC_lexicon_tags_25 },
    { 0xfb86930f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c69316, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd2e69318, SUC_lexicon_tags_11 },
    { 0xb8869318, SUC_lexicon_tags_11 },
    { 0x4586931a, SUC_lexicon_tags_17 },
    { 0xee26931a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e69321, SUC_lexicon_tags_23 },
    { 0x76669322, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24669326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9669328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf746932b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c6932f, SUC_lexicon_tags_7 },
    { 0x76a69330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff669336, SUC_lexicon_tags_16 },
    { 0x11069337, SUC_lexicon_tags_15 },
    { 0xb4469337, SUC_lexicon_tags_43 },
    { 0xe6069339, SUC_lexicon_tags_16 },
    { 0x63269338, SUC_lexicon_tags_2 },
    { 0x4006933b, SUC_lexicon_tags_63 },
    { 0xe0c69336, SUC_lexicon_tags_37 },
    { 0xff26933d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc366933f, SUC_lexicon_tags_65 },
    { 0x5c269340, SUC_lexicon_tags_63 },
    { 0x6c869340, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x97e69343, SUC_lexicon_tags_29 },
    { 0x64669343, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11c69346, SUC_lexicon_tags_19 },
    { 0xfe669347, SUC_lexicon_tags_0 },
    { 0x18a69348, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c6934c, SUC_lexicon_tags_11 },
    { 0x82e6934c, SUC_lexicon_tags_13 },
    { 0x7026934d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f669351, SUC_lexicon_tags_27 },
    { 0xa7c69352, SUC_lexicon_tags_19 },
    { 0x23469352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd669359, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd386935b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386935f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96869364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1706936f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d469378, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5026937a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x22c6937c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30069380, SUC_lexicon_tags_38 },
    { 0x9cc69380, SUC_lexicon_tags_38 },
    { 0xa269382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57c69384, SUC_lexicon_tags_1 },
    { 0x72e69384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76669388, SUC_lexicon_tags_7 },
    { 0xc9669389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83e6938b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc6938e, SUC_lexicon_tags_4 },
    { 0xd806938f, SUC_lexicon_tags_68 },
    { 0x13069390, SUC_lexicon_tags_37 },
    { 0xc566938e, SUC_lexicon_tags_63 },
    { 0xcf469390, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57469394, SUC_lexicon_tags_65 },
    { 0xa5069395, SUC_lexicon_tags_7 },
    { 0x7a69396, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb669398, SUC_lexicon_tags_0 },
    { 0x40469399, SUC_lexicon_tags_47 },
    { 0x72869399, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80e6939c, SUC_lexicon_tags_37 },
    { 0x67c6939c, SUC_lexicon_tags_29 },
    { 0xd686939d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2693a6, SUC_lexicon_tags_59 },
    { 0xae6693a7, SUC_lexicon_tags_13 },
    { 0xaa4693a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2693ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x806693ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2693b4, SUC_lexicon_tags_2 },
    { 0x3f2693b5, SUC_lexicon_tags_17 },
    { 0xe72693b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02693ba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c693be, SUC_lexicon_tags_25 },
    { 0xd76693be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec693c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8693ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4693ce, SUC_lexicon_tags_7 },
    { 0x39a693cf, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x51e693d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e6693d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2693d6, SUC_lexicon_tags_11 },
    { 0xfd6693d7, SUC_lexicon_tags_83 },
    { 0x762693d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc693df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x770693e2, SUC_lexicon_tags_37 },
    { 0xd1e693e3, SUC_lexicon_tags_7 },
    { 0x6e4693e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf82693e6, SUC_lexicon_tags_14 },
    { 0x91e693e7, SUC_lexicon_tags_11 },
    { 0xfc8693e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a693ed, SUC_lexicon_tags_33 },
    { 0xb26693ee, SUC_lexicon_tags_14 },
    { 0x33a693ed, SUC_lexicon_tags_10 },
    { 0x402693f0, SUC_lexicon_tags_82 },
    { 0x162693ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26693f6, SUC_lexicon_tags_25 },
    { 0xfde693f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e693fd, SUC_lexicon_tags_47 },
    { 0x4ca693fd, SUC_lexicon_tags_25 },
    { 0x3f4693ff, SUC_lexicon_tags_7 },
    { 0x888693fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd286940c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e69413, SUC_lexicon_tags_4 },
    { 0x18e69413, SUC_lexicon_tags_1 },
    { 0xb9269415, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a69418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea26941a, SUC_lexicon_tags_107 },
    { 0xc546941b, SUC_lexicon_tags_25 },
    { 0x69c6941b, SUC_lexicon_tags_9 },
    { 0x5ee6941d, SUC_lexicon_tags_37 },
    { 0x1c06941e, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8469423, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee6942d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee46942f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50e69431, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e69435, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1de69437, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e6943e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb5c69440, SUC_lexicon_tags_25 },
    { 0xefc69441, SUC_lexicon_tags_14 },
    { 0x3d469442, SUC_lexicon_tags_2 },
    { 0xe9c69441, SUC_lexicon_tags_9 },
    { 0xa0269444, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5069446, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e69449, SUC_lexicon_tags_19 },
    { 0x8166944a, SUC_lexicon_tags_7 },
    { 0xca66944b, SUC_lexicon_tags_37 },
    { 0x8426944a, SUC_lexicon_tags_17 },
    { 0x8dc6944d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac469452, SUC_lexicon_tags_14 },
    { 0x3469453, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa4c69455, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe2069457, SUC_lexicon_tags_25 },
    { 0xcaa69458, SUC_lexicon_tags_92 },
    { 0x10669459, SUC_lexicon_tags_39 },
    { 0x72e6945a, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0xea06945d, SUC_lexicon_tags_15 },
    { 0xc366945e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb269466, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ea69468, SUC_lexicon_tags_7 },
    { 0x1ce69468, SUC_lexicon_tags_45 },
    { 0xfe669469, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a46946e, SUC_lexicon_tags_37 },
    { 0x8306946e, SUC_lexicon_tags_29 },
    { 0x40c6946f, SUC_lexicon_tags_6 },
    { 0x6be6946f, SUC_lexicon_tags_47 },
    { 0x30869472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6e69474, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x30069476, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9aa69478, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4e6947a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc366947f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x6b469481, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x60c69483, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7469486, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a6948b, SUC_lexicon_tags_1 },
    { 0xfdc6948b, SUC_lexicon_tags_43 },
    { 0x7306948d, SUC_lexicon_tags_6 },
    { 0x2e06948d, SUC_lexicon_tags_37 },
    { 0x2186948f, SUC_lexicon_tags_15 },
    { 0x1d46948e, SUC_lexicon_tags_33 },
    { 0x4ba69491, SUC_lexicon_tags_27 },
    { 0x7a06948e, SUC_lexicon_tags_25 },
    { 0xdd269492, SUC_lexicon_tags_25 },
    { 0xf8869494, SUC_lexicon_tags_37 },
    { 0x3e86948c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc6949b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde6949e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9c8694a0, SUC_lexicon_tags_77 },
    { 0xb5c694a1, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee694a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44694aa, SUC_lexicon_tags_17 },
    { 0xf9a694aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbd6694ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a4694af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4694b2, SUC_lexicon_tags_33 },
    { 0x572694b2, SUC_lexicon_tags_25 },
    { 0x44c694b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0694b8, SUC_lexicon_tags_37 },
    { 0xa70694b8, SUC_lexicon_tags_15 },
    { 0x214694ba, SUC_lexicon_tags_15 },
    { 0xbee694b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e694bd, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x71c694bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28694c2, SUC_lexicon_tags_23 },
    { 0xf98694c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50694c7, SUC_lexicon_tags_17 },
    { 0x41c694c7, SUC_lexicon_tags_29 },
    { 0x2c0694c9, SUC_lexicon_tags_29 },
    { 0x692694c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2694cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4694d2, SUC_lexicon_tags_46 },
    { 0x798694d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xca0694d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda6694d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2694da, SUC_lexicon_tags_1028 },
    { 0x270694da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c694e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0694e4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8694e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2694ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee694f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x290694f3, SUC_lexicon_tags_25 },
    { 0xd16694f4, SUC_lexicon_tags_15 },
    { 0x21c694f4, SUC_lexicon_tags_15 },
    { 0x12c694f6, SUC_lexicon_tags_27 },
    { 0xc50694f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60694fa, SUC_lexicon_tags_12 },
    { 0x2a694fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd469502, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a69506, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3469509, SUC_lexicon_tags_25 },
    { 0xd146950a, SUC_lexicon_tags_32 },
    { 0x36a69509, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe86950f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd669512, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd869517, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x10669519, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2466951b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a69521, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75469523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b269525, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869529, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf46952b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6446952d, SUC_lexicon_tags_30 },
    { 0x4fa6952e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a69532, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5069539, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6206953c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20e6953e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa69541, SUC_lexicon_tags_2 },
    { 0x9c69541, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0269546, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c6954a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4146954d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xea69550, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d069557, SUC_lexicon_tags_23 },
    { 0x87669558, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33469562, SUC_lexicon_tags_4 },
    { 0x97869562, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25869569, SUC_lexicon_tags_66 },
    { 0x80a69569, SUC_lexicon_tags_14 },
    { 0xb366956b, SUC_lexicon_tags_27 },
    { 0x5706956b, SUC_lexicon_tags_53 },
    { 0xf366956a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x826956f, SUC_lexicon_tags_38 },
    { 0xc86956f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9269572, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7526957c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e6957f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ce69581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11469586, SUC_lexicon_tags_1 },
    { 0x76e69586, SUC_lexicon_tags_15 },
    { 0xeba69588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7966958d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28869593, SUC_lexicon_tags_15 },
    { 0x3ce69593, SUC_lexicon_tags_23 },
    { 0x4ae69593, SUC_lexicon_tags_28 },
    { 0x65269596, SUC_lexicon_tags_60 },
    { 0x21269596, SUC_lexicon_tags_27 },
    { 0xd6669598, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8566959a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a6959d, SUC_lexicon_tags_12 },
    { 0x6086959e, SUC_lexicon_tags_11 },
    { 0x2fa6959f, SUC_lexicon_tags_12 },
    { 0xbf66959e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa695a3, SUC_lexicon_tags_2 },
    { 0x17a695a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36695a6, SUC_lexicon_tags_25 },
    { 0x7dc695a7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xea695a9, SUC_lexicon_tags_62 },
    { 0x62a695a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a695ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68695b2, SUC_lexicon_tags_34 },
    { 0x7de695b3, SUC_lexicon_tags_11 },
    { 0x7d4695b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8695c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8695c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea695d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560695d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6695da, SUC_lexicon_tags_25 },
    { 0x75e695da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ee695dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x580695df, SUC_lexicon_tags_1 },
    { 0xb8e695df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b4695e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0695e5, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c695eb, SUC_lexicon_tags_37 },
    { 0xaf8695ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec695f0, SUC_lexicon_tags_7 },
    { 0x79a695f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x148695f3, SUC_lexicon_tags_11 },
    { 0x99c695f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb70695f6, SUC_lexicon_tags_49 },
    { 0x5fc695f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4dc695f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3069603, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1269607, SUC_lexicon_tags_2 },
    { 0xf0269607, SUC_lexicon_tags_107 },
    { 0x6c669609, SUC_lexicon_tags_37 },
    { 0x4ba69609, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc486960e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc6069610, SUC_lexicon_tags_65 },
    { 0x2ca69611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd269613, SUC_lexicon_tags_27 },
    { 0xc1469613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1a69616, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd46961b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdc6961d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7469621, SUC_lexicon_tags_15 },
    { 0xca469621, SUC_lexicon_tags_23 },
    { 0x27e69623, SUC_lexicon_tags_25 },
    { 0x8c069621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cc69626, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6469629, SUC_lexicon_tags_19 },
    { 0x8966962a, SUC_lexicon_tags_23 },
    { 0x50c69629, SUC_lexicon_tags_33 },
    { 0xc6e69629, SUC_lexicon_tags_13 },
    { 0x2a06962c, SUC_lexicon_tags_19 },
    { 0x63e6962e, SUC_lexicon_tags_0 },
    { 0xac6962f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae69636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4126963a, SUC_lexicon_tags_138 },
    { 0xafc6963b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48669640, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9a69642, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x17669645, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x39669647, SUC_lexicon_tags_37 },
    { 0xe7869647, SUC_lexicon_tags_49 },
    { 0x61669649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7626964e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e69652, SUC_lexicon_tags_17 },
    { 0xcdc69653, SUC_lexicon_tags_1 },
    { 0xd5669653, SUC_lexicon_tags_21 },
    { 0x7a669655, SUC_lexicon_tags_11 },
    { 0xed069653, SUC_lexicon_tags_14 },
    { 0x4ea69657, SUC_lexicon_tags_39 },
    { 0xd9869658, SUC_lexicon_tags_7 },
    { 0x4ce69659, SUC_lexicon_tags_60 },
    { 0x47c6965a, SUC_lexicon_tags_63 },
    { 0x526965b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6965e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe069660, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x69469662, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d069664, SUC_lexicon_tags_29 },
    { 0xe3e69664, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6426966a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd666966e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae69674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52269677, SUC_lexicon_tags_11 },
    { 0xe0869678, SUC_lexicon_tags_8 },
    { 0xef469679, SUC_lexicon_tags_19 },
    { 0x21c69679, SUC_lexicon_tags_1 },
    { 0xddc69679, SUC_lexicon_tags_12 },
    { 0xbf66967a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3206967f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84669684, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x75269687, SUC_lexicon_tags_47 },
    { 0x55a69688, SUC_lexicon_tags_38 },
    { 0xd7669689, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286968f, SUC_lexicon_tags_7 },
    { 0xcd669690, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a69697, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7746969a, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0x18a6969c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c696a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be696a5, SUC_lexicon_tags_37 },
    { 0x394696a6, SUC_lexicon_tags_47 },
    { 0xcd2696a7, SUC_lexicon_tags_12 },
    { 0xb36696a7, SUC_lexicon_tags_17 },
    { 0x3de696a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x642696ab, SUC_lexicon_tags_47 },
    { 0x4a0696ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c696af, SUC_lexicon_tags_9 },
    { 0xb18696af, SUC_lexicon_tags_11 },
    { 0x870696b1, SUC_lexicon_tags_10 },
    { 0x696b2, SUC_lexicon_tags_17 },
    { 0xe0696b3, SUC_lexicon_tags_15 },
    { 0x718696b4, SUC_lexicon_tags_7 },
    { 0x922696b4, SUC_lexicon_tags_23 },
    { 0x9de696b6, SUC_lexicon_tags_77 },
    { 0x8fa696b2, SUC_lexicon_tags_37 },
    { 0xf8a696b8, SUC_lexicon_tags_15 },
    { 0x8f6696b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40696bd, SUC_lexicon_tags_47 },
    { 0x3d6696be, SUC_lexicon_tags_8 },
    { 0x39c696bf, SUC_lexicon_tags_41 },
    { 0x4d8696bf, SUC_lexicon_tags_15 },
    { 0x8f6696bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ec696c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10696c7, SUC_lexicon_tags_49 },
    { 0x3e6696c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd98696ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc696cf, SUC_lexicon_tags_2 },
    { 0xda4696cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a696d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd4696d5, SUC_lexicon_tags_31 },
    { 0x498696d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea696db, SUC_lexicon_tags_37 },
    { 0xd44696dc, SUC_lexicon_tags_11 },
    { 0x65c696dd, SUC_lexicon_tags_79 },
    { 0xe9c696dd, SUC_lexicon_tags_1 },
    { 0x548696de, SUC_lexicon_tags_25 },
    { 0x3d4696e0, SUC_lexicon_tags_4 },
    { 0x52696e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6696e4, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x602696e6, SUC_lexicon_tags_45 },
    { 0x6a696e7, SUC_lexicon_tags_25 },
    { 0x198696e6, SUC_lexicon_tags_30 },
    { 0xf48696e6, SUC_lexicon_tags_14 },
    { 0x998696e6, SUC_lexicon_tags_37 },
    { 0xe4696eb, SUC_lexicon_tags_30 },
    { 0x98a696eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfe696ee, SUC_lexicon_tags_9 },
    { 0xfb2696ef, SUC_lexicon_tags_0 },
    { 0xcc2696f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f6696f2, SUC_lexicon_tags_11 },
    { 0xaa8696f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be696f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c696fc, SUC_lexicon_tags_33 },
    { 0x92a696fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52e696ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8269706, SUC_lexicon_tags_17 },
    { 0x34669707, SUC_lexicon_tags_49 },
    { 0xdc669708, SUC_lexicon_tags_1 },
    { 0x95269707, SUC_lexicon_tags_11 },
    { 0x18c6970a, SUC_lexicon_tags_43 },
    { 0x34669709, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2806970f, SUC_lexicon_tags_17 },
    { 0xf4c69710, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a69713, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9e69715, SUC_lexicon_tags_37 },
    { 0xd4269716, SUC_lexicon_tags_2 },
    { 0xf5a69717, SUC_lexicon_tags_28 },
    { 0x6c469717, SUC_lexicon_tags_28 },
    { 0x69c69719, SUC_lexicon_tags_37 },
    { 0xf1c69718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9806971c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa126971f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74c69721, SUC_lexicon_tags_29 },
    { 0x95869722, SUC_lexicon_tags_60 },
    { 0x99869721, SUC_lexicon_tags_15 },
    { 0x33469724, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7269726, SUC_lexicon_tags_31 },
    { 0xf5e69727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2686972a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9186972d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde69730, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbba69732, SUC_lexicon_tags_37 },
    { 0x86a69733, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xda269735, SUC_lexicon_tags_34 },
    { 0xcae69736, SUC_lexicon_tags_31 },
    { 0xedc69736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4269739, SUC_lexicon_tags_47 },
    { 0x8069739, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1246973c, SUC_lexicon_tags_13 },
    { 0xcba6973c, SUC_lexicon_tags_23 },
    { 0x7fe6973e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a69743, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba869748, SUC_lexicon_tags_25 },
    { 0xa7e69749, SUC_lexicon_tags_25 },
    { 0xdae6974a, SUC_lexicon_tags_118 },
    { 0x2e46974b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e6974d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac69750, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30669755, SUC_lexicon_tags_47 },
    { 0xb3a69756, SUC_lexicon_tags_55 },
    { 0x66c69757, SUC_lexicon_tags_8 },
    { 0x91669758, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x99a6975a, SUC_lexicon_tags_2 },
    { 0xedc6975b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5546975e, SUC_lexicon_tags_10 },
    { 0xbfc6975f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1869766, SUC_lexicon_tags_12 },
    { 0x21e69767, SUC_lexicon_tags_11 },
    { 0x5cc69768, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e46976c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee6976f, SUC_lexicon_tags_33 },
    { 0xc0c69770, SUC_lexicon_tags_11 },
    { 0xe1e69771, SUC_lexicon_tags_25 },
    { 0x4b869772, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d669774, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91269779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e6977c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71a6977e, SUC_lexicon_tags_68 },
    { 0xc066977f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x26669781, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6269784, SUC_lexicon_tags_12 },
    { 0x3ba69784, SUC_lexicon_tags_81 },
    { 0x5d069786, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3669789, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6986978b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea269793, SUC_lexicon_tags_47 },
    { 0x55469794, SUC_lexicon_tags_37 },
    { 0xf069795, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6469798, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6979b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c697a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf26697a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0697a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea697ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca697af, SUC_lexicon_tags_37 },
    { 0x64e697b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a697b5, SUC_lexicon_tags_43 },
    { 0xa8c697b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c697c0, SUC_lexicon_tags_2 },
    { 0xd4a697c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e697c4, SUC_lexicon_tags_1 },
    { 0x88697c5, SUC_lexicon_tags_11 },
    { 0xff6697c6, SUC_lexicon_tags_9 },
    { 0x894697c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc697cf, SUC_lexicon_tags_1 },
    { 0x344697d0, SUC_lexicon_tags_23 },
    { 0x7c8697d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5a0697d3, SUC_lexicon_tags_12 },
    { 0xba6697d3, SUC_lexicon_tags_33 },
    { 0x210697d3, SUC_lexicon_tags_28 },
    { 0x47a697d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfb2697d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20697de, SUC_lexicon_tags_14 },
    { 0xfa8697df, SUC_lexicon_tags_31 },
    { 0x58a697e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa68697e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66697e6, SUC_lexicon_tags_19 },
    { 0x198697e6, SUC_lexicon_tags_15 },
    { 0x44c697e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586697ec, SUC_lexicon_tags_37 },
    { 0x588697ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb74697ef, SUC_lexicon_tags_38 },
    { 0x368697ef, SUC_lexicon_tags_47 },
    { 0x1b8697ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6697f5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a69804, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a69809, SUC_lexicon_tags_48 },
    { 0xf846980a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a6980d, SUC_lexicon_tags_37 },
    { 0xa186980d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c69816, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12669818, SUC_lexicon_tags_17 },
    { 0x53269818, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4866981b, SUC_lexicon_tags_2 },
    { 0x6a66981b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9da6981e, SUC_lexicon_tags_17 },
    { 0xe446981f, SUC_lexicon_tags_47 },
    { 0x4b069820, SUC_lexicon_tags_30 },
    { 0xe0a69821, SUC_lexicon_tags_25 },
    { 0xa1a69821, SUC_lexicon_tags_28 },
    { 0x64e69823, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x59069825, SUC_lexicon_tags_1 },
    { 0xdc69825, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7a69828, SUC_lexicon_tags_140 },
    { 0x48069828, SUC_lexicon_tags_39 },
    { 0x1e6982a, SUC_lexicon_tags_17 },
    { 0xc1e6982b, SUC_lexicon_tags_11 },
    { 0x9ca6982c, SUC_lexicon_tags_12 },
    { 0xce269828, SUC_lexicon_tags_2 },
    { 0xd866982e, SUC_lexicon_tags_23 },
    { 0xb3e69828, SUC_lexicon_tags_68 },
    { 0x2966982c, SUC_lexicon_tags_31 },
    { 0x18e69831, SUC_lexicon_tags_18 },
    { 0x93669832, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78469835, SUC_lexicon_tags_26 },
    { 0x3de69835, SUC_lexicon_tags_17 },
    { 0x3e269836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4146983b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e6983f, SUC_lexicon_tags_57 },
    { 0xbc069840, SUC_lexicon_tags_31 },
    { 0xbe069840, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xea69843, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e69848, SUC_lexicon_tags_12 },
    { 0x66c69849, SUC_lexicon_tags_10 },
    { 0x4f06984a, SUC_lexicon_tags_25 },
    { 0x21e69848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3a6984d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c69850, SUC_lexicon_tags_3 },
    { 0x18269850, SUC_lexicon_tags_63 },
    { 0x5d669852, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea6985a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe946985e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e269861, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x30269863, SUC_lexicon_tags_27 },
    { 0xb6c69864, SUC_lexicon_tags_17 },
    { 0xc8e69865, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e6986c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa6986f, SUC_lexicon_tags_25 },
    { 0xf2c6986f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x53669872, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f669875, SUC_lexicon_tags_11 },
    { 0xa2c69875, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec6987a, SUC_lexicon_tags_1 },
    { 0x8f46987a, SUC_lexicon_tags_7 },
    { 0x4126987c, SUC_lexicon_tags_15 },
    { 0x3c86987d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52469882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a469885, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc626988a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f06988e, SUC_lexicon_tags_262 },
    { 0x5406988f, SUC_lexicon_tags_43 },
    { 0xe646988f, SUC_lexicon_tags_30 },
    { 0x84a69891, SUC_lexicon_tags_15 },
    { 0x45a6988e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa7469894, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8586989d, SUC_lexicon_tags_25 },
    { 0xfee6989e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c698a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700698a7, SUC_lexicon_tags_7 },
    { 0xc20698a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee698ab, SUC_lexicon_tags_23 },
    { 0x76a698ac, SUC_lexicon_tags_49 },
    { 0x180698ad, SUC_lexicon_tags_19 },
    { 0xaea698ae, SUC_lexicon_tags_25 },
    { 0x5da698ae, SUC_lexicon_tags_6 },
    { 0x902698b0, SUC_lexicon_tags_37 },
    { 0xec2698ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8698b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a698b9, SUC_lexicon_tags_22 },
    { 0xbba698b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e698be, SUC_lexicon_tags_63 },
    { 0x936698bf, SUC_lexicon_tags_38 },
    { 0x190698c0, SUC_lexicon_tags_3 },
    { 0x4b0698c1, SUC_lexicon_tags_55 },
    { 0x934698c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaba698c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54698cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc698d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae698d7, SUC_lexicon_tags_15 },
    { 0x5ac698d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf58698da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd06698dc, SUC_lexicon_tags_25 },
    { 0x6b0698dc, SUC_lexicon_tags_66 },
    { 0x6d4698de, SUC_lexicon_tags_21 },
    { 0x7c2698df, SUC_lexicon_tags_2 },
    { 0x4ec698df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c698e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e698e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa698ec, SUC_lexicon_tags_12 },
    { 0x604698ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0698f3, SUC_lexicon_tags_25 },
    { 0xa3e698f4, SUC_lexicon_tags_11 },
    { 0x64a698f5, SUC_lexicon_tags_14 },
    { 0xc68698f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec698fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e698fe, SUC_lexicon_tags_6 },
    { 0xa02698ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65269901, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17e69903, SUC_lexicon_tags_46 },
    { 0xa8c69904, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6669908, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d469913, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x18a69915, SUC_lexicon_tags_43 },
    { 0x90669916, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf26991a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4166991c, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xcb06991e, SUC_lexicon_tags_25 },
    { 0xb926991e, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a469925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73469928, SUC_lexicon_tags_25 },
    { 0xff869928, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b86992c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e69933, SUC_lexicon_tags_46 },
    { 0x84069934, SUC_lexicon_tags_45 },
    { 0xcfa69935, SUC_lexicon_tags_11 },
    { 0x7b069935, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa7c69938, SUC_lexicon_tags_11 },
    { 0xf9069938, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e46993b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4266993d, SUC_lexicon_tags_0 },
    { 0xaac6993e, SUC_lexicon_tags_82 },
    { 0x3b86993f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8669943, SUC_lexicon_tags_38 },
    { 0x13269944, SUC_lexicon_tags_25 },
    { 0xdf669944, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c66994e, SUC_lexicon_tags_11 },
    { 0xa086994e, SUC_lexicon_tags_11 },
    { 0xfea69950, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e69954, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa269956, SUC_lexicon_tags_63 },
    { 0x63c69957, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa06995c, SUC_lexicon_tags_59 },
    { 0x3226995d, SUC_lexicon_tags_25 },
    { 0xb86995d, SUC_lexicon_tags_11 },
    { 0xa986995f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b069969, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x85c6996b, SUC_lexicon_tags_17 },
    { 0x4a66996c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54869971, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c69975, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90e69977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e6997a, SUC_lexicon_tags_38 },
    { 0x6826997a, SUC_lexicon_tags_33 },
    { 0x41c6997a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a6997f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49869983, SUC_lexicon_tags_0 },
    { 0x3ae69984, SUC_lexicon_tags_11 },
    { 0xa9269985, SUC_lexicon_tags_32 },
    { 0xf1a69985, SUC_lexicon_tags_79 },
    { 0xb1469983, SUC_lexicon_tags_15 },
    { 0x30669988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8a6998a, SUC_lexicon_tags_26 },
    { 0xb2e6998b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a26998e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64869990, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50e69992, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd469996, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a669999, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ae6999b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda06999e, SUC_lexicon_tags_38 },
    { 0xde6999f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef2699a1, SUC_lexicon_tags_65 },
    { 0x9a2699a2, SUC_lexicon_tags_38 },
    { 0x16e699a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x988699a6, SUC_lexicon_tags_37 },
    { 0xe48699a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda699aa, SUC_lexicon_tags_12 },
    { 0x7ce699aa, SUC_lexicon_tags_28 },
    { 0x4d8699ac, SUC_lexicon_tags_2 },
    { 0xc82699ad, SUC_lexicon_tags_1 },
    { 0xb50699ae, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46699b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc2699b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x562699b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916699ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x362699bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4699c0, SUC_lexicon_tags_11 },
    { 0x660699c1, SUC_lexicon_tags_27 },
    { 0xffa699c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4c699c4, SUC_lexicon_tags_12 },
    { 0x9b6699c5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8699cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924699d5, SUC_lexicon_tags_0 },
    { 0x830699d6, SUC_lexicon_tags_15 },
    { 0x5b6699d7, SUC_lexicon_tags_65 },
    { 0xfe8699d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496699dd, SUC_lexicon_tags_14 },
    { 0x1a8699de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0699e9, SUC_lexicon_tags_14 },
    { 0x33e699ea, SUC_lexicon_tags_32 },
    { 0x6b0699ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaae699ed, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba699f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa699f8, SUC_lexicon_tags_1 },
    { 0x1cc699f8, SUC_lexicon_tags_7 },
    { 0x416699fa, SUC_lexicon_tags_25 },
    { 0x900699fa, SUC_lexicon_tags_37 },
    { 0x7a4699fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc69a01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de69a04, SUC_lexicon_tags_31 },
    { 0x95069a05, SUC_lexicon_tags_14 },
    { 0x8b669a05, SUC_lexicon_tags_23 },
    { 0xa5069a07, SUC_lexicon_tags_8 },
    { 0xdc469a07, SUC_lexicon_tags_17 },
    { 0x47069a06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9469a0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e69a11, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85869a15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc869a17, SUC_lexicon_tags_2 },
    { 0xfb269a18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fe69a1a, SUC_lexicon_tags_25 },
    { 0xed069a1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc069a1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a69a23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97869a27, SUC_lexicon_tags_275 },
    { 0xa9069a28, SUC_lexicon_tags_2 },
    { 0xc4869a29, SUC_lexicon_tags_34 },
    { 0xda69a27, SUC_lexicon_tags_12 },
    { 0x99c69a27, SUC_lexicon_tags_49 },
    { 0xe269a2c, SUC_lexicon_tags_17 },
    { 0xb3c69a2d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x60e69a2f, SUC_lexicon_tags_12 },
    { 0x97269a30, SUC_lexicon_tags_30 },
    { 0x75c69a31, SUC_lexicon_tags_2 },
    { 0x3d669a30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50069a34, SUC_lexicon_tags_15 },
    { 0xdd469a35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7869a37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b069a3c, SUC_lexicon_tags_6 },
    { 0x3f069a3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3a69a3f, SUC_lexicon_tags_17 },
    { 0x50e69a40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f869a44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a69a47, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae69a4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x25669a4d, SUC_lexicon_tags_1 },
    { 0x6e669a4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd0469a50, SUC_lexicon_tags_37 },
    { 0xbfa69a51, SUC_lexicon_tags_2 },
    { 0xd7269a52, SUC_lexicon_tags_47 },
    { 0xbb669a53, SUC_lexicon_tags_9 },
    { 0xc1669a54, SUC_lexicon_tags_31 },
    { 0x62269a55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cc69a57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30669a66, SUC_lexicon_tags_31 },
    { 0x50069a67, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92469a6b, SUC_lexicon_tags_7 },
    { 0x93269a6c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xda269a6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba69a72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b869a75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe3469a77, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7669a7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6669a7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9469a82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca669a88, SUC_lexicon_tags_8 },
    { 0x4d469a88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3ac69a8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf7e69a8d, SUC_lexicon_tags_48 },
    { 0x88669a8e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc5669a90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdbe69a92, SUC_lexicon_tags_250 },
    { 0x3ec69a93, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc69a96, SUC_lexicon_tags_0 },
    { 0x3e469a97, SUC_lexicon_tags_37 },
    { 0x7a69a98, SUC_lexicon_tags_23 },
    { 0x69a98, SUC_lexicon_tags_32 },
    { 0xba869a9a, SUC_lexicon_tags_9 },
    { 0xc9a69a9b, SUC_lexicon_tags_65 },
    { 0x87869a9b, SUC_lexicon_tags_47 },
    { 0xe5669a9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7469aa1, SUC_lexicon_tags_32 },
    { 0x83a69aa1, SUC_lexicon_tags_11 },
    { 0xb7069aa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49a69aa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10669aa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a69aad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8269ab5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62669ab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8869abe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd8869ac0, SUC_lexicon_tags_47 },
    { 0x95e69ac1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60a69ac3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52c69ac5, SUC_lexicon_tags_56 },
    { 0xb4a69ac6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a69ac9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb069acd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc0269acf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7069adb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a69ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3669ae3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4469ae6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4269ae9, SUC_lexicon_tags_11 },
    { 0x52e69aea, SUC_lexicon_tags_7 },
    { 0x1f269aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1269af0, SUC_lexicon_tags_9 },
    { 0xfe69af1, SUC_lexicon_tags_37 },
    { 0xee69af2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8669af6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x88269af9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce669afb, SUC_lexicon_tags_37 },
    { 0xdc69afc, SUC_lexicon_tags_14 },
    { 0xbd669afc, SUC_lexicon_tags_14 },
    { 0xbcc69afb, SUC_lexicon_tags_12 },
    { 0xbb269aff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de69b03, SUC_lexicon_tags_45 },
    { 0xd2669b03, SUC_lexicon_tags_17 },
    { 0x99469b05, SUC_lexicon_tags_37 },
    { 0x57e69b05, SUC_lexicon_tags_25 },
    { 0x74669b07, SUC_lexicon_tags_21 },
    { 0x25069b03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73269b0a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e69b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d669b10, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe7c69b12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3269b15, SUC_lexicon_tags_37 },
    { 0x77069b16, SUC_lexicon_tags_31 },
    { 0x7a469b16, SUC_lexicon_tags_7 },
    { 0xc1869b17, SUC_lexicon_tags_32 },
    { 0xe3269b19, SUC_lexicon_tags_28 },
    { 0x8ca69b19, SUC_lexicon_tags_33 },
    { 0x1ca69b18, SUC_lexicon_tags_11 },
    { 0x4c669b1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3e069b1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3c69b20, SUC_lexicon_tags_10 },
    { 0xeac69b20, SUC_lexicon_tags_37 },
    { 0x33869b22, SUC_lexicon_tags_11 },
    { 0x69c69b21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e269b2c, SUC_lexicon_tags_1 },
    { 0xd3869b2d, SUC_lexicon_tags_15 },
    { 0x99869b2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a69b32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a69b36, SUC_lexicon_tags_48 },
    { 0xe4c69b37, SUC_lexicon_tags_37 },
    { 0xe3869b38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5469b3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63e69b3d, SUC_lexicon_tags_34 },
    { 0x35869b3e, SUC_lexicon_tags_37 },
    { 0x10a69b3f, SUC_lexicon_tags_47 },
    { 0x5469b3d, SUC_lexicon_tags_26 },
    { 0x1c269b3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f469b44, SUC_lexicon_tags_34 },
    { 0xc8669b44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c69b49, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51869b4d, SUC_lexicon_tags_47 },
    { 0x16469b4d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2069b54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52269b56, SUC_lexicon_tags_39 },
    { 0xe3c69b57, SUC_lexicon_tags_45 },
    { 0x7669b57, SUC_lexicon_tags_14 },
    { 0x2d069b58, SUC_lexicon_tags_37 },
    { 0x3b869b57, SUC_lexicon_tags_11 },
    { 0x44e69b5b, SUC_lexicon_tags_19 },
    { 0x86469b5c, SUC_lexicon_tags_37 },
    { 0xfe069b5b, SUC_lexicon_tags_48 },
    { 0x3f269b5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95469b64, SUC_lexicon_tags_14 },
    { 0x1cc69b65, SUC_lexicon_tags_11 },
    { 0xad669b65, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x35469b68, SUC_lexicon_tags_38 },
    { 0x9269b69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40669b70, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e869b73, SUC_lexicon_tags_7 },
    { 0x9f269b73, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc69b7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad469b84, SUC_lexicon_tags_17 },
    { 0xfb669b85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c69b8c, SUC_lexicon_tags_152 },
    { 0xa9069b8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a69b91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e69b96, SUC_lexicon_tags_60 },
    { 0xeb069b97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7469b99, SUC_lexicon_tags_37 },
    { 0x62c69b9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc69b9d, SUC_lexicon_tags_29 },
    { 0x12869b9e, SUC_lexicon_tags_28 },
    { 0xe4669b9f, SUC_lexicon_tags_18 },
    { 0xcac69b9e, SUC_lexicon_tags_11 },
    { 0x34a69b9d, SUC_lexicon_tags_17 },
    { 0xb6e69ba2, SUC_lexicon_tags_32 },
    { 0x34069ba0, SUC_lexicon_tags_7 },
    { 0xdd069ba4, SUC_lexicon_tags_38 },
    { 0xc6869b9d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c69bab, SUC_lexicon_tags_6 },
    { 0xda469bac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x23669baf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1669bb3, SUC_lexicon_tags_15 },
    { 0xa2a69bb4, SUC_lexicon_tags_79 },
    { 0x98269bb4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9e069bb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6c69bb9, SUC_lexicon_tags_47 },
    { 0x8a469bb9, SUC_lexicon_tags_17 },
    { 0xaa069bba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe6869bbd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba69bc1, SUC_lexicon_tags_12 },
    { 0x41069bc2, SUC_lexicon_tags_37 },
    { 0x4be69bc2, SUC_lexicon_tags_15 },
    { 0xe0269bc2, SUC_lexicon_tags_1 },
    { 0x6be69bc4, SUC_lexicon_tags_61 },
    { 0xe4e69bc6, SUC_lexicon_tags_49 },
    { 0xb6669bc6, SUC_lexicon_tags_11 },
    { 0x32869bc8, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x12e69bca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a69bd2, SUC_lexicon_tags_2 },
    { 0xeb869bd3, SUC_lexicon_tags_45 },
    { 0xb3069bd4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b469bdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e69be5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc6869be7, SUC_lexicon_tags_11 },
    { 0xd1c69be7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce69bec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2be69bee, SUC_lexicon_tags_1 },
    { 0x32069bef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7469bf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2869bfd, SUC_lexicon_tags_17 },
    { 0x1de69bfd, SUC_lexicon_tags_2 },
    { 0xbe269bff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7469c09, SUC_lexicon_tags_11 },
    { 0xf3869c0a, SUC_lexicon_tags_151 },
    { 0x14469c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3669c11, SUC_lexicon_tags_17 },
    { 0x5a669c12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71269c14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa269c17, SUC_lexicon_tags_32 },
    { 0x7c669c18, SUC_lexicon_tags_34 },
    { 0x20269c19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc69c1c, SUC_lexicon_tags_23 },
    { 0xdf269c1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4469c21, SUC_lexicon_tags_28 },
    { 0xc0a69c22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e69c26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b469c2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed669c2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe9869c2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80869c31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba269c33, SUC_lexicon_tags_37 },
    { 0xa4469c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95869c36, SUC_lexicon_tags_7 },
    { 0x65269c37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e69c3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5669c3f, SUC_lexicon_tags_37 },
    { 0x3f869c40, SUC_lexicon_tags_23 },
    { 0x93069c41, SUC_lexicon_tags_48 },
    { 0x54069c41, SUC_lexicon_tags_17 },
    { 0x2e469c43, SUC_lexicon_tags_37 },
    { 0xa0a69c43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2669c4a, SUC_lexicon_tags_25 },
    { 0x33269c4b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x87269c4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62469c52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e69c56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1869c58, SUC_lexicon_tags_31 },
    { 0x83469c58, SUC_lexicon_tags_60 },
    { 0x4ee69c5a, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a69c5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e69c61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e69c64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a869c6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad269c6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e69c73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4c69c75, SUC_lexicon_tags_1 },
    { 0xbb269c75, SUC_lexicon_tags_47 },
    { 0x50469c77, SUC_lexicon_tags_28 },
    { 0x1fe69c75, SUC_lexicon_tags_34 },
    { 0x33869c76, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x75469c7b, SUC_lexicon_tags_45 },
    { 0x21a69c7b, SUC_lexicon_tags_7 },
    { 0xfa69c7b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a69c82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a69c86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a069c8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa269c8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c869c92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a069c94, SUC_lexicon_tags_12 },
    { 0x56669c95, SUC_lexicon_tags_33 },
    { 0x46e69c95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23269c9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae869ca1, SUC_lexicon_tags_37 },
    { 0x87e69ca2, SUC_lexicon_tags_26 },
    { 0xec069ca3, SUC_lexicon_tags_60 },
    { 0x8a669ca4, SUC_lexicon_tags_8 },
    { 0x97a69ca5, SUC_lexicon_tags_55 },
    { 0x95869ca3, SUC_lexicon_tags_10 },
    { 0x83469ca1, SUC_lexicon_tags_37 },
    { 0x3c469ca8, SUC_lexicon_tags_624 },
    { 0x7f069ca7, SUC_lexicon_tags_17 },
    { 0x37269caa, SUC_lexicon_tags_47 },
    { 0x41469cab, SUC_lexicon_tags_10 },
    { 0x3c269cab, SUC_lexicon_tags_11 },
    { 0x84869cab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x10469cb0, SUC_lexicon_tags_151 },
    { 0xc9c69cb1, SUC_lexicon_tags_60 },
    { 0xc0869cb2, SUC_lexicon_tags_11 },
    { 0xf7e69cb3, SUC_lexicon_tags_29 },
    { 0x14a69cb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8069cb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be69cbe, SUC_lexicon_tags_43 },
    { 0xd7469cbe, SUC_lexicon_tags_8 },
    { 0x45269cc0, SUC_lexicon_tags_38 },
    { 0x7f669cbe, SUC_lexicon_tags_13 },
    { 0x63a69cc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9a69cc4, SUC_lexicon_tags_1 },
    { 0xfc469cc5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfe069cc7, SUC_lexicon_tags_11 },
    { 0x10a69cc8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee869cce, SUC_lexicon_tags_4 },
    { 0xc2669ccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2869cd3, SUC_lexicon_tags_7 },
    { 0x30c69cd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e69cd7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1269cdf, SUC_lexicon_tags_60 },
    { 0xdcc69cdf, SUC_lexicon_tags_7 },
    { 0x4669cdf, SUC_lexicon_tags_45 },
    { 0xa7069ce2, SUC_lexicon_tags_11 },
    { 0x9ea69ce2, SUC_lexicon_tags_11 },
    { 0x39869ce1, SUC_lexicon_tags_12 },
    { 0x28069ce5, SUC_lexicon_tags_23 },
    { 0x8ba69ce5, SUC_lexicon_tags_37 },
    { 0x65a69ce0, SUC_lexicon_tags_63 },
    { 0xc9269ce8, SUC_lexicon_tags_37 },
    { 0x11669ce8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a469cfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xace69d00, SUC_lexicon_tags_34 },
    { 0x14869d01, SUC_lexicon_tags_45 },
    { 0x6e869d02, SUC_lexicon_tags_47 },
    { 0xd6a69d01, SUC_lexicon_tags_12 },
    { 0x48869d01, SUC_lexicon_tags_2 },
    { 0xac669d04, SUC_lexicon_tags_84 },
    { 0xbc69d06, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4269d09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98669d0b, SUC_lexicon_tags_39 },
    { 0x4c869d0b, SUC_lexicon_tags_17 },
    { 0x7cc69d0b, SUC_lexicon_tags_14 },
    { 0x15869d0e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8e869d10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec69d13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7469d15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce69d18, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaf469d1a, SUC_lexicon_tags_2 },
    { 0x6c69d1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x39269d1e, SUC_lexicon_tags_13 },
    { 0x95269d1e, SUC_lexicon_tags_15 },
    { 0x4b669d1e, SUC_lexicon_tags_30 },
    { 0x3e669d1f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x85c69d23, SUC_lexicon_tags_46 },
    { 0x6d669d23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe3069d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb269d29, SUC_lexicon_tags_12 },
    { 0x4ee69d29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e69d2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c69d31, SUC_lexicon_tags_9 },
    { 0xde069d31, SUC_lexicon_tags_1 },
    { 0xd3c69d31, SUC_lexicon_tags_17 },
    { 0x5c69d34, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xace69d36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74469d38, SUC_lexicon_tags_17 },
    { 0xbaa69d38, SUC_lexicon_tags_84 },
    { 0x84869d3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe869d3c, SUC_lexicon_tags_26 },
    { 0x12669d3d, SUC_lexicon_tags_43 },
    { 0x9869d3e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71069d43, SUC_lexicon_tags_17 },
    { 0x5cc69d44, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa69d49, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c69d55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c69d59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec69d5c, SUC_lexicon_tags_7 },
    { 0xcfc69d5d, SUC_lexicon_tags_1 },
    { 0xc4269d5c, SUC_lexicon_tags_4 },
    { 0xe3069d5d, SUC_lexicon_tags_29 },
    { 0xddc69d5e, SUC_lexicon_tags_33 },
    { 0x2ae69d61, SUC_lexicon_tags_17 },
    { 0x53469d62, SUC_lexicon_tags_30 },
    { 0x4e269d63, SUC_lexicon_tags_19 },
    { 0xbf469d5d, SUC_lexicon_tags_31 },
    { 0x7d469d65, SUC_lexicon_tags_22 },
    { 0x9e469d66, SUC_lexicon_tags_38 },
    { 0x95c69d67, SUC_lexicon_tags_23 },
    { 0x29269d5f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6869d6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8be69d6d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x86269d6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be69d74, SUC_lexicon_tags_11 },
    { 0x39269d75, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa69d78, SUC_lexicon_tags_14 },
    { 0xe6869d79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a69d7c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec69d82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x83269d84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c69d8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc69d92, SUC_lexicon_tags_47 },
    { 0x72469d92, SUC_lexicon_tags_23 },
    { 0xa8a69d94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86669d9a, SUC_lexicon_tags_17 },
    { 0x64669d9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaea69d9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6a69d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62869da3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x82869da6, SUC_lexicon_tags_19 },
    { 0x6e869da6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x90a69da9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1869dad, SUC_lexicon_tags_19 },
    { 0xac069dad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4869db1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6869db3, SUC_lexicon_tags_6 },
    { 0x2de69db3, SUC_lexicon_tags_11 },
    { 0x83269db5, SUC_lexicon_tags_27 },
    { 0x85669db4, SUC_lexicon_tags_12 },
    { 0xb6e69db7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f269dbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7469dc3, SUC_lexicon_tags_25 },
    { 0x20669dc3, SUC_lexicon_tags_60 },
    { 0x9a469dc5, SUC_lexicon_tags_37 },
    { 0x7b669dc6, SUC_lexicon_tags_37 },
    { 0x8469dc5, SUC_lexicon_tags_11 },
    { 0xf0e69dc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79869dcc, SUC_lexicon_tags_43 },
    { 0x9e069dcc, SUC_lexicon_tags_25 },
    { 0xc0c69dce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a69dd4, SUC_lexicon_tags_1119 },
    { 0x90669dd5, SUC_lexicon_tags_11 },
    { 0xbc869dd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a69dd9, SUC_lexicon_tags_28 },
    { 0xa669dda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a469de1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e69de7, SUC_lexicon_tags_23 },
    { 0x5c269de8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xda869dea, SUC_lexicon_tags_30 },
    { 0xee669deb, SUC_lexicon_tags_37 },
    { 0xf4e69dec, SUC_lexicon_tags_49 },
    { 0xf6669dec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98869df0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xef069df2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28869df7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8669dfb, SUC_lexicon_tags_60 },
    { 0xb3269dfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19269dff, SUC_lexicon_tags_38 },
    { 0xffc69e00, SUC_lexicon_tags_64 },
    { 0x35a69dff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17469e03, SUC_lexicon_tags_5 },
    { 0xfd669e04, SUC_lexicon_tags_11 },
    { 0xb5869e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52669e07, SUC_lexicon_tags_17 },
    { 0x9ba69e08, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb269e0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef069e12, SUC_lexicon_tags_17 },
    { 0x4e469e13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e69e19, SUC_lexicon_tags_16 },
    { 0xf7869e1a, SUC_lexicon_tags_43 },
    { 0xb9669e1a, SUC_lexicon_tags_61 },
    { 0xdde69e1b, SUC_lexicon_tags_9 },
    { 0x5bc69e1c, SUC_lexicon_tags_46 },
    { 0x67469e1e, SUC_lexicon_tags_28 },
    { 0xd6069e1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba69e22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3b069e24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d269e26, SUC_lexicon_tags_103 },
    { 0x3c069e27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7fa69e29, SUC_lexicon_tags_2 },
    { 0x71869e2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92669e2e, SUC_lexicon_tags_10 },
    { 0x8fe69e2e, SUC_lexicon_tags_1 },
    { 0x37869e2f, SUC_lexicon_tags_7 },
    { 0x82e69e30, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbae69e33, SUC_lexicon_tags_23 },
    { 0xb3669e34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f469e38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4269e40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6269e46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1069e4b, SUC_lexicon_tags_47 },
    { 0x5fc69e4b, SUC_lexicon_tags_21 },
    { 0xa1269e4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe1c69e4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe469e52, SUC_lexicon_tags_123 },
    { 0xdb669e53, SUC_lexicon_tags_32 },
    { 0xcb869e54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89269e56, SUC_lexicon_tags_19 },
    { 0xcd669e57, SUC_lexicon_tags_17 },
    { 0xa5469e58, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9869e5b, SUC_lexicon_tags_12 },
    { 0x26269e5c, SUC_lexicon_tags_37 },
    { 0xc8e69e5d, SUC_lexicon_tags_13 },
    { 0x6e669e5e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x17a69e60, SUC_lexicon_tags_16 },
    { 0x17269e61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x78669e63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98469e65, SUC_lexicon_tags_23 },
    { 0x3c669e66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a69e69, SUC_lexicon_tags_35 },
    { 0x2ee69e6a, SUC_lexicon_tags_26 },
    { 0x33869e6a, SUC_lexicon_tags_27 },
    { 0x6ce69e6b, SUC_lexicon_tags_63 },
    { 0xc8469e6a, SUC_lexicon_tags_11 },
    { 0x23c69e6c, SUC_lexicon_tags_11 },
    { 0xc9e69e6f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1869e72, SUC_lexicon_tags_64 },
    { 0x22669e73, SUC_lexicon_tags_29 },
    { 0xb1469e74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72669e78, SUC_lexicon_tags_39 },
    { 0x46069e79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc1669e7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12269e7e, SUC_lexicon_tags_11 },
    { 0xe5869e7f, SUC_lexicon_tags_10 },
    { 0xca69e80, SUC_lexicon_tags_3 },
    { 0x5f869e81, SUC_lexicon_tags_2 },
    { 0xb4069e82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x60269e85, SUC_lexicon_tags_1 },
    { 0xdaa69e86, SUC_lexicon_tags_32 },
    { 0x8a869e87, SUC_lexicon_tags_17 },
    { 0x17c69e88, SUC_lexicon_tags_1 },
    { 0x2ba69e88, SUC_lexicon_tags_14 },
    { 0x1fc69e8a, SUC_lexicon_tags_198 },
    { 0xc469e8b, SUC_lexicon_tags_1 },
    { 0xb6669e8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc669e90, SUC_lexicon_tags_21 },
    { 0x43069e91, SUC_lexicon_tags_37 },
    { 0x8a269e90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4869e95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a69e98, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9469e9b, SUC_lexicon_tags_0 },
    { 0xb9469e9c, SUC_lexicon_tags_25 },
    { 0xbcc69e9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c69ea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76e69ea4, SUC_lexicon_tags_10 },
    { 0x65869ea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab069ea9, SUC_lexicon_tags_7 },
    { 0xc4469eaa, SUC_lexicon_tags_9 },
    { 0x7fa69eaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1069eb2, SUC_lexicon_tags_37 },
    { 0xb7269eb3, SUC_lexicon_tags_45 },
    { 0x90c69eb2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19a69eb6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ec69eb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe5469eba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0669ebe, SUC_lexicon_tags_45 },
    { 0x4a69ebf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e069ec3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4c669ec5, SUC_lexicon_tags_38 },
    { 0x4a069ec5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3aa69ec8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a469eca, SUC_lexicon_tags_4 },
    { 0xbe269ecb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3469ecd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc7a69ecf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85469ed1, SUC_lexicon_tags_2 },
    { 0x14a69ed2, SUC_lexicon_tags_37 },
    { 0x71269ed2, SUC_lexicon_tags_26 },
    { 0x6a469ed4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de69edb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e69ede, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xf6469ee0, SUC_lexicon_tags_60 },
    { 0xcd069ee1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc669ee5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb7c69ee7, SUC_lexicon_tags_43 },
    { 0x7f869ee7, SUC_lexicon_tags_45 },
    { 0x5dc69ee9, SUC_lexicon_tags_14 },
    { 0x98469ee9, SUC_lexicon_tags_37 },
    { 0xf8669eea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb5869eed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39669ef1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52869efa, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a69eff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22269f02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11069f04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a69f0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c69f10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25069f14, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4cc69f16, SUC_lexicon_tags_2 },
    { 0xac069f17, SUC_lexicon_tags_1 },
    { 0x63869f18, SUC_lexicon_tags_1 },
    { 0x83869f18, SUC_lexicon_tags_2 },
    { 0x7ea69f18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a69f1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x89269f22, SUC_lexicon_tags_15 },
    { 0x7a69f23, SUC_lexicon_tags_12 },
    { 0x86a69f23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c469f26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99669f29, SUC_lexicon_tags_7 },
    { 0x98869f2a, SUC_lexicon_tags_33 },
    { 0xd6e69f2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd7669f2d, SUC_lexicon_tags_2 },
    { 0x17269f2e, SUC_lexicon_tags_37 },
    { 0xd9a69f2f, SUC_lexicon_tags_46 },
    { 0x86069f2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269f36, SUC_lexicon_tags_45 },
    { 0x49669f36, SUC_lexicon_tags_15 },
    { 0x45e69f38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0069f3b, SUC_lexicon_tags_11 },
    { 0x2aa69f3c, SUC_lexicon_tags_11 },
    { 0x8dc69f3d, SUC_lexicon_tags_17 },
    { 0xb6c69f3d, SUC_lexicon_tags_37 },
    { 0xe3869f3f, SUC_lexicon_tags_2 },
    { 0x25e69f3f, SUC_lexicon_tags_1 },
    { 0x19269f41, SUC_lexicon_tags_12 },
    { 0x71869f42, SUC_lexicon_tags_11 },
    { 0x1869f42, SUC_lexicon_tags_0 },
    { 0xcae69f43, SUC_lexicon_tags_82 },
    { 0x19c69f42, SUC_lexicon_tags_60 },
    { 0xf7669f46, SUC_lexicon_tags_47 },
    { 0x6be69f47, SUC_lexicon_tags_26 },
    { 0x48469f48, SUC_lexicon_tags_7 },
    { 0x55c69f48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7669f4f, SUC_lexicon_tags_47 },
    { 0xb1e69f4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12c69f52, SUC_lexicon_tags_25 },
    { 0xc7a69f52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c69f5b, SUC_lexicon_tags_6 },
    { 0x22669f5b, SUC_lexicon_tags_2 },
    { 0x8fe69f5d, SUC_lexicon_tags_27 },
    { 0x47069f5e, SUC_lexicon_tags_56 },
    { 0x1ac69f5e, SUC_lexicon_tags_17 },
    { 0x7f469f60, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x21469f63, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e69f66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c69f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70669f72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a69f75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5e69f77, SUC_lexicon_tags_27 },
    { 0x49669f77, SUC_lexicon_tags_25 },
    { 0xa2269f79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0469f7e, SUC_lexicon_tags_29 },
    { 0xf9a69f7f, SUC_lexicon_tags_15 },
    { 0xc8669f80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4869f83, SUC_lexicon_tags_705 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d269f86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff669f88, SUC_lexicon_tags_3 },
    { 0x80069f89, SUC_lexicon_tags_25 },
    { 0xde869f88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x23269f8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff469f92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a69f95, SUC_lexicon_tags_37 },
    { 0xdd269f95, SUC_lexicon_tags_15 },
    { 0xfa869f97, SUC_lexicon_tags_56 },
    { 0x4c469f98, SUC_lexicon_tags_2 },
    { 0x98069f99, SUC_lexicon_tags_48 },
    { 0xfcc69f9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68669f9e, SUC_lexicon_tags_15 },
    { 0xc8e69f9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26069fa1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47069fa3, SUC_lexicon_tags_1 },
    { 0x99269fa4, SUC_lexicon_tags_27 },
    { 0x5ec69fa5, SUC_lexicon_tags_21 },
    { 0xad269fa3, SUC_lexicon_tags_11 },
    { 0xc6469fa7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34069fae, SUC_lexicon_tags_31 },
    { 0x71869fae, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e69fb8, SUC_lexicon_tags_39 },
    { 0xc1669fb8, SUC_lexicon_tags_92 },
    { 0x1ea69fba, SUC_lexicon_tags_37 },
    { 0xa5469fbb, SUC_lexicon_tags_1 },
    { 0x32e69fba, SUC_lexicon_tags_0 },
    { 0xa6669fba, SUC_lexicon_tags_53 },
    { 0x70269fbe, SUC_lexicon_tags_1 },
    { 0xdb669fbb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b269fc5, SUC_lexicon_tags_74 },
    { 0x69869fc5, SUC_lexicon_tags_43 },
    { 0x5a469fc7, SUC_lexicon_tags_27 },
    { 0x3c269fc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c69fcb, SUC_lexicon_tags_29 },
    { 0xa4869fcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87669fd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae69fd4, SUC_lexicon_tags_37 },
    { 0xa269fd5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x42069fd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19669fda, SUC_lexicon_tags_12 },
    { 0x4f069fdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc669fdf, SUC_lexicon_tags_12 },
    { 0x5a669fe0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8e69fe2, SUC_lexicon_tags_11 },
    { 0x5669fe2, SUC_lexicon_tags_39 },
    { 0x4d269fe4, SUC_lexicon_tags_15 },
    { 0x98669fe5, SUC_lexicon_tags_38 },
    { 0x7269fe6, SUC_lexicon_tags_37 },
    { 0x8cc69fe7, SUC_lexicon_tags_17 },
    { 0xe69fe8, SUC_lexicon_tags_11 },
    { 0xba869fe9, SUC_lexicon_tags_45 },
    { 0xaf069fea, SUC_lexicon_tags_25 },
    { 0xf1c69feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0869ff3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5669ffc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc69fff, SUC_lexicon_tags_0 },
    { 0x8966a000, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6a003, SUC_lexicon_tags_7 },
    { 0xb386a003, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2026a008, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc946a010, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa746a012, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b46a018, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb726a01c, SUC_lexicon_tags_49 },
    { 0x1566a01d, SUC_lexicon_tags_39 },
    { 0xa4e6a01e, SUC_lexicon_tags_3 },
    { 0x8566a01c, SUC_lexicon_tags_27 },
    { 0x3486a020, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a6a027, SUC_lexicon_tags_33 },
    { 0x3006a027, SUC_lexicon_tags_8 },
    { 0x2fe6a029, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc06a02d, SUC_lexicon_tags_15 },
    { 0x13e6a02e, SUC_lexicon_tags_2 },
    { 0x37a6a02e, SUC_lexicon_tags_2 },
    { 0xf6c6a030, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa026a034, SUC_lexicon_tags_7 },
    { 0xeb46a035, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da6a03a, SUC_lexicon_tags_31 },
    { 0xcf66a03b, SUC_lexicon_tags_34 },
    { 0xa466a03c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc6a041, SUC_lexicon_tags_17 },
    { 0xc5c6a042, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d06a048, SUC_lexicon_tags_15 },
    { 0xd826a048, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe486a04c, SUC_lexicon_tags_0 },
    { 0x9be6a04d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf26a051, SUC_lexicon_tags_62 },
    { 0x32c6a052, SUC_lexicon_tags_27 },
    { 0x4e6a053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d46a059, SUC_lexicon_tags_31 },
    { 0xc366a05a, SUC_lexicon_tags_2 },
    { 0xb326a05a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9726a05e, SUC_lexicon_tags_19 },
    { 0x1ee6a05f, SUC_lexicon_tags_45 },
    { 0x5726a05f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2546a062, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e6a068, SUC_lexicon_tags_18 },
    { 0x1e66a069, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce06a06e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6786a070, SUC_lexicon_tags_9 },
    { 0x23c6a071, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc966a077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7726a07f, SUC_lexicon_tags_19 },
    { 0xde06a07f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2526a083, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6506a085, SUC_lexicon_tags_49 },
    { 0xbf86a086, SUC_lexicon_tags_108 },
    { 0xcf06a087, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa426a090, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8706a092, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1506a095, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1326a098, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9186a09e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa326a0a0, SUC_lexicon_tags_0 },
    { 0x83c6a0a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a6a0a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c86a0ab, SUC_lexicon_tags_19 },
    { 0xa8a6a0ac, SUC_lexicon_tags_32 },
    { 0xf946a0ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa926a0b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe066a0b3, SUC_lexicon_tags_27 },
    { 0x6ac6a0b4, SUC_lexicon_tags_25 },
    { 0x8706a0b3, SUC_lexicon_tags_12 },
    { 0xfde6a0b4, SUC_lexicon_tags_37 },
    { 0xd286a0b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e86a0bc, SUC_lexicon_tags_11 },
    { 0x7006a0bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99c6a0bf, SUC_lexicon_tags_70 },
    { 0x5626a0bf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1de6a0c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb986a0c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1466a0cc, SUC_lexicon_tags_7 },
    { 0xa466a0cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6a0d0, SUC_lexicon_tags_16 },
    { 0x4e06a0d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1406a0d3, SUC_lexicon_tags_1 },
    { 0x8e06a0d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c6a0db, SUC_lexicon_tags_37 },
    { 0x4806a0db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe726a0e0, SUC_lexicon_tags_26 },
    { 0xf366a0e0, SUC_lexicon_tags_25 },
    { 0x42c6a0e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd346a0e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ec6a0e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc6a0e9, SUC_lexicon_tags_12 },
    { 0xb926a0ea, SUC_lexicon_tags_1 },
    { 0x4b46a0eb, SUC_lexicon_tags_39 },
    { 0xc6a6a0eb, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a46a0fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb66a105, SUC_lexicon_tags_7 },
    { 0xe4a6a106, SUC_lexicon_tags_68 },
    { 0x4a26a105, SUC_lexicon_tags_14 },
    { 0x1fc6a108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3126a10c, SUC_lexicon_tags_14 },
    { 0x8446a10d, SUC_lexicon_tags_60 },
    { 0x4c46a10e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf0a6a110, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x70e6a112, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e06a114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe606a116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x586a119, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1f66a11b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1606a122, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2586a125, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1226a128, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc66a12d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa6a130, SUC_lexicon_tags_11 },
    { 0x9a66a131, SUC_lexicon_tags_29 },
    { 0x2c06a132, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2126a135, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1dc6a137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c26a139, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8c86a13b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c66a13f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5606a141, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3566a143, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd206a148, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f26a14b, SUC_lexicon_tags_37 },
    { 0x1406a14c, SUC_lexicon_tags_11 },
    { 0x1fa6a14d, SUC_lexicon_tags_15 },
    { 0x6e66a14e, SUC_lexicon_tags_14 },
    { 0x9546a14f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69e6a151, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a6a157, SUC_lexicon_tags_15 },
    { 0xf866a157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea06a15d, SUC_lexicon_tags_39 },
    { 0x9146a15e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf786a160, SUC_lexicon_tags_15 },
    { 0x17c6a161, SUC_lexicon_tags_11 },
    { 0xde06a162, SUC_lexicon_tags_8 },
    { 0x8106a160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa426a165, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2546a168, SUC_lexicon_tags_7 },
    { 0x66e6a169, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b46a172, SUC_lexicon_tags_3 },
    { 0xa7e6a173, SUC_lexicon_tags_0 },
    { 0xc966a173, SUC_lexicon_tags_113 },
    { 0x566a175, SUC_lexicon_tags_39 },
    { 0x79e6a173, SUC_lexicon_tags_38 },
    { 0xc226a172, SUC_lexicon_tags_141 },
    { 0x3d66a174, SUC_lexicon_tags_17 },
    { 0x7626a179, SUC_lexicon_tags_9 },
    { 0x2766a176, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde6a181, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7166a183, SUC_lexicon_tags_2 },
    { 0xa5c6a184, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06a188, SUC_lexicon_tags_45 },
    { 0x9a46a189, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3386a18f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c6a193, SUC_lexicon_tags_77 },
    { 0xc246a193, SUC_lexicon_tags_59 },
    { 0xc146a193, SUC_lexicon_tags_11 },
    { 0x8646a195, SUC_lexicon_tags_60 },
    { 0x5826a195, SUC_lexicon_tags_27 },
    { 0xd7c6a198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e6a19e, SUC_lexicon_tags_26 },
    { 0x7a26a19f, SUC_lexicon_tags_17 },
    { 0x7746a19f, SUC_lexicon_tags_1 },
    { 0xdf46a1a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e66a1a4, SUC_lexicon_tags_15 },
    { 0xee66a1a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa366a1a9, SUC_lexicon_tags_49 },
    { 0xe486a1aa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa806a1ac, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6266a1b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18e6a1b2, SUC_lexicon_tags_11 },
    { 0x4c6a1b3, SUC_lexicon_tags_39 },
    { 0x87e6a1b4, SUC_lexicon_tags_45 },
    { 0xd406a1b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e6a1ba, SUC_lexicon_tags_34 },
    { 0xa6c6a1bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386a1c6, SUC_lexicon_tags_21 },
    { 0xf166a1c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x306a1c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9506a1cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6086a1ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc886a1da, SUC_lexicon_tags_46 },
    { 0xadc6a1db, SUC_lexicon_tags_17 },
    { 0xe66a1dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa226a1df, SUC_lexicon_tags_12 },
    { 0x5ea6a1e0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e6a1e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1106a1e7, SUC_lexicon_tags_12 },
    { 0xa686a1e8, SUC_lexicon_tags_25 },
    { 0x2e46a1e9, SUC_lexicon_tags_17 },
    { 0xbe26a1e9, SUC_lexicon_tags_65 },
    { 0x9fa6a1ea, SUC_lexicon_tags_47 },
    { 0x78e6a1ec, SUC_lexicon_tags_1 },
    { 0x3a26a1ec, SUC_lexicon_tags_12 },
    { 0xd486a1e9, SUC_lexicon_tags_61 },
    { 0x25a6a1ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c6a1f2, SUC_lexicon_tags_15 },
    { 0x6766a1f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9aa6a1f5, SUC_lexicon_tags_29 },
    { 0xeae6a1f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92c6a1f8, SUC_lexicon_tags_27 },
    { 0x8466a1f9, SUC_lexicon_tags_17 },
    { 0x9766a1fa, SUC_lexicon_tags_11 },
    { 0x92e6a1fb, SUC_lexicon_tags_19 },
    { 0x546a1f8, SUC_lexicon_tags_6 },
    { 0xaba6a1fd, SUC_lexicon_tags_23 },
    { 0xa66a1fe, SUC_lexicon_tags_7 },
    { 0x4ba6a1f8, SUC_lexicon_tags_17 },
    { 0xe7c6a1fa, SUC_lexicon_tags_15 },
    { 0x33e6a201, SUC_lexicon_tags_23 },
    { 0x1b66a201, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4e6a204, SUC_lexicon_tags_25 },
    { 0x8726a205, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcd46a207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64c6a209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c86a20b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xed06a20d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf626a214, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2886a216, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc6a21c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c86a220, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbba6a222, SUC_lexicon_tags_49 },
    { 0xf06a223, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb546a226, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61c6a228, SUC_lexicon_tags_80 },
    { 0x5346a229, SUC_lexicon_tags_15 },
    { 0x7b46a22a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b86a22d, SUC_lexicon_tags_19 },
    { 0xdb66a22e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7506a230, SUC_lexicon_tags_12 },
    { 0xff86a231, SUC_lexicon_tags_138 },
    { 0x4de6a232, SUC_lexicon_tags_9 },
    { 0xd966a232, SUC_lexicon_tags_0 },
    { 0xb346a231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd626a236, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6046a23a, SUC_lexicon_tags_63 },
    { 0x46c6a23a, SUC_lexicon_tags_37 },
    { 0xea06a23c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xab86a23e, SUC_lexicon_tags_56 },
    { 0x4de6a23f, SUC_lexicon_tags_47 },
    { 0x70a6a240, SUC_lexicon_tags_15 },
    { 0xf846a240, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb086a244, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf86a246, SUC_lexicon_tags_15 },
    { 0xbfc6a247, SUC_lexicon_tags_114 },
    { 0x1b86a247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f66a24a, SUC_lexicon_tags_28 },
    { 0x97c6a24a, SUC_lexicon_tags_38 },
    { 0xf386a24c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1946a24f, SUC_lexicon_tags_17 },
    { 0x6ca6a250, SUC_lexicon_tags_49 },
    { 0x8286a251, SUC_lexicon_tags_15 },
    { 0x9766a252, SUC_lexicon_tags_37 },
    { 0x4646a253, SUC_lexicon_tags_1 },
    { 0x5566a252, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd86a259, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb486a25b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e6a25f, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a6a265, SUC_lexicon_tags_19 },
    { 0x3606a265, SUC_lexicon_tags_61 },
    { 0x326a267, SUC_lexicon_tags_25 },
    { 0x1fe6a268, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e26a26e, SUC_lexicon_tags_9 },
    { 0x1686a26e, SUC_lexicon_tags_34 },
    { 0x1686a270, SUC_lexicon_tags_27 },
    { 0xb286a270, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa46a274, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a6a277, SUC_lexicon_tags_0 },
    { 0xf806a278, SUC_lexicon_tags_25 },
    { 0x53c6a279, SUC_lexicon_tags_35 },
    { 0xdb26a278, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa226a27e, SUC_lexicon_tags_38 },
    { 0xe886a27f, SUC_lexicon_tags_2 },
    { 0x39a6a280, SUC_lexicon_tags_1 },
    { 0xc606a280, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9586a285, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c26a28a, SUC_lexicon_tags_0 },
    { 0x3186a28a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c06a299, SUC_lexicon_tags_25 },
    { 0x3746a29a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea6a29c, SUC_lexicon_tags_29 },
    { 0xb6e6a29d, SUC_lexicon_tags_37 },
    { 0x9f06a29e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c6a2a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9ac6a2a3, SUC_lexicon_tags_27 },
    { 0xe446a2a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x25c6a2a6, SUC_lexicon_tags_26 },
    { 0x806a2a7, SUC_lexicon_tags_19 },
    { 0xa626a2a8, SUC_lexicon_tags_92 },
    { 0x3e66a2a7, SUC_lexicon_tags_29 },
    { 0xd286a2aa, SUC_lexicon_tags_29 },
    { 0xc086a2ab, SUC_lexicon_tags_61 },
    { 0x5b86a2ac, SUC_lexicon_tags_47 },
    { 0xb9c6a2ab, SUC_lexicon_tags_45 },
    { 0xb986a2a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea86a2b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc6a2b8, SUC_lexicon_tags_10 },
    { 0x71e6a2b9, SUC_lexicon_tags_29 },
    { 0xaae6a2ba, SUC_lexicon_tags_4 },
    { 0xd666a2b9, SUC_lexicon_tags_35 },
    { 0x5d46a2bc, SUC_lexicon_tags_23 },
    { 0x2126a2bd, SUC_lexicon_tags_17 },
    { 0x1426a2b9, SUC_lexicon_tags_25 },
    { 0xa1e6a2bf, SUC_lexicon_tags_25 },
    { 0xe06a2c0, SUC_lexicon_tags_19 },
    { 0xfb26a2bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc106a2c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1926a2c8, SUC_lexicon_tags_106 },
    { 0xeac6a2c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe746a2cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21a6a2cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe686a2d1, SUC_lexicon_tags_15 },
    { 0x5c46a2d2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4566a2d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e06a2dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b46a2df, SUC_lexicon_tags_47 },
    { 0x4606a2e0, SUC_lexicon_tags_31 },
    { 0x9fe6a2e1, SUC_lexicon_tags_39 },
    { 0x7266a2e1, SUC_lexicon_tags_37 },
    { 0xe666a2e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3066a2e5, SUC_lexicon_tags_27 },
    { 0x3886a2e5, SUC_lexicon_tags_2 },
    { 0xefe6a2e5, SUC_lexicon_tags_91 },
    { 0x9106a2e6, SUC_lexicon_tags_19 },
    { 0xcaa6a2e8, SUC_lexicon_tags_25 },
    { 0x6b66a2e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5da6a2ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd26a2ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe6a2f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e6a2f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c46a2f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d26a305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63a6a307, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec6a30b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdea6a30d, SUC_lexicon_tags_25 },
    { 0xf706a30e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa826a311, SUC_lexicon_tags_1 },
    { 0xa8a6a312, SUC_lexicon_tags_7 },
    { 0xe6a6a312, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c6a316, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9ce6a318, SUC_lexicon_tags_37 },
    { 0xff66a318, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff26a31f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746a324, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa366a327, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28c6a329, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a6a32c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6126a32e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2006a332, SUC_lexicon_tags_30 },
    { 0x8a06a333, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa966a337, SUC_lexicon_tags_47 },
    { 0xac86a338, SUC_lexicon_tags_1 },
    { 0x8ea6a337, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e6a33d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d26a342, SUC_lexicon_tags_47 },
    { 0x2ba6a343, SUC_lexicon_tags_85 },
    { 0x6686a342, SUC_lexicon_tags_43 },
    { 0xd266a345, SUC_lexicon_tags_15 },
    { 0xcd06a346, SUC_lexicon_tags_8 },
    { 0x9f06a347, SUC_lexicon_tags_104 },
    { 0xa886a348, SUC_lexicon_tags_33 },
    { 0xb566a347, SUC_lexicon_tags_11 },
    { 0xe2e6a349, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb306a34d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a86a353, SUC_lexicon_tags_27 },
    { 0xc486a354, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce6a358, SUC_lexicon_tags_15 },
    { 0xb6a6a359, SUC_lexicon_tags_23 },
    { 0xd166a358, SUC_lexicon_tags_11 },
    { 0xc26a35b, SUC_lexicon_tags_31 },
    { 0xfb86a35c, SUC_lexicon_tags_31 },
    { 0x8726a35b, SUC_lexicon_tags_15 },
    { 0x1426a35c, SUC_lexicon_tags_25 },
    { 0x4ba6a35b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1166a366, SUC_lexicon_tags_63 },
    { 0xbf46a367, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb66a36c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a6a370, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1626a373, SUC_lexicon_tags_23 },
    { 0x9906a373, SUC_lexicon_tags_77 },
    { 0xe8e6a375, SUC_lexicon_tags_11 },
    { 0x8566a376, SUC_lexicon_tags_33 },
    { 0xc846a377, SUC_lexicon_tags_29 },
    { 0xaee6a378, SUC_lexicon_tags_46 },
    { 0x64e6a379, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c86a37f, SUC_lexicon_tags_1 },
    { 0x9fa6a380, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe246a38d, SUC_lexicon_tags_23 },
    { 0x7566a38e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6a396, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd466a39b, SUC_lexicon_tags_63 },
    { 0x6326a39b, SUC_lexicon_tags_15 },
    { 0x23a6a39d, SUC_lexicon_tags_38 },
    { 0x96e6a39e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb126a3a0, SUC_lexicon_tags_334 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a6a3a4, SUC_lexicon_tags_43 },
    { 0xd386a3a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa6a3a8, SUC_lexicon_tags_12 },
    { 0x3c66a3a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2726a3ae, SUC_lexicon_tags_12 },
    { 0x3ac6a3ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5606a3b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85e6a3b3, SUC_lexicon_tags_11 },
    { 0xad46a3b4, SUC_lexicon_tags_25 },
    { 0x8706a3b5, SUC_lexicon_tags_38 },
    { 0x346a3b4, SUC_lexicon_tags_37 },
    { 0x1c6a3b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc206a3b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1ac6a3bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a26a3be, SUC_lexicon_tags_15 },
    { 0xa1c6a3bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa306a3c1, SUC_lexicon_tags_25 },
    { 0xdc6a3c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e6a3cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb686a3d3, SUC_lexicon_tags_31 },
    { 0xb006a3d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x52a6a3d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x966a3d9, SUC_lexicon_tags_1 },
    { 0xa526a3da, SUC_lexicon_tags_10 },
    { 0x4cc6a3da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c6a3de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3646a3e1, SUC_lexicon_tags_37 },
    { 0x2246a3e1, SUC_lexicon_tags_37 },
    { 0x8c6a3e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c6a3e6, SUC_lexicon_tags_29 },
    { 0x5da6a3e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xad46a3e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1626a3ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf446a3f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75c6a3f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc6a3f6, SUC_lexicon_tags_7 },
    { 0x5746a3f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc846a3f9, SUC_lexicon_tags_16 },
    { 0x30c6a3f9, SUC_lexicon_tags_12 },
    { 0x6a26a3fb, SUC_lexicon_tags_8 },
    { 0x15a6a3fc, SUC_lexicon_tags_37 },
    { 0x3506a3fc, SUC_lexicon_tags_49 },
    { 0x89c6a3fe, SUC_lexicon_tags_43 },
    { 0x8f06a3ff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4626a401, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c46a405, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6a408, SUC_lexicon_tags_96 },
    { 0xa806a409, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a6a40d, SUC_lexicon_tags_11 },
    { 0xf3a6a40d, SUC_lexicon_tags_27 },
    { 0x2826a40f, SUC_lexicon_tags_33 },
    { 0x466a410, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3366a412, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba6a415, SUC_lexicon_tags_2 },
    { 0x7226a415, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56c6a418, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e6a41b, SUC_lexicon_tags_8 },
    { 0x4f26a41c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5986a41e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf06a420, SUC_lexicon_tags_17 },
    { 0xac86a421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c06a424, SUC_lexicon_tags_33 },
    { 0xac06a425, SUC_lexicon_tags_15 },
    { 0x5906a425, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586a430, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5266a43a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a6a43f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e46a443, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c6a447, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x666a44a, SUC_lexicon_tags_33 },
    { 0xac26a44a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa226a44f, SUC_lexicon_tags_9 },
    { 0x7e26a450, SUC_lexicon_tags_4 },
    { 0x6906a450, SUC_lexicon_tags_1 },
    { 0x84a6a452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe46a454, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa6a458, SUC_lexicon_tags_2 },
    { 0x4f46a458, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e6a45d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5566a45f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e6a462, SUC_lexicon_tags_8 },
    { 0xc526a463, SUC_lexicon_tags_8 },
    { 0x3f26a463, SUC_lexicon_tags_37 },
    { 0x2f26a465, SUC_lexicon_tags_15 },
    { 0x12e6a465, SUC_lexicon_tags_47 },
    { 0x48a6a463, SUC_lexicon_tags_7 },
    { 0x64a6a468, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa46a46a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78c6a46c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xebe6a46e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd66a470, SUC_lexicon_tags_25 },
    { 0x66c6a471, SUC_lexicon_tags_15 },
    { 0x5a6a471, SUC_lexicon_tags_68 },
    { 0x5206a473, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5e6a475, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21c6a477, SUC_lexicon_tags_12 },
    { 0x5f46a477, SUC_lexicon_tags_17 },
    { 0xb0a6a477, SUC_lexicon_tags_37 },
    { 0xe526a47a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46a47d, SUC_lexicon_tags_30 },
    { 0x9706a47e, SUC_lexicon_tags_60 },
    { 0xe546a47f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd326a482, SUC_lexicon_tags_21 },
    { 0x6ea6a483, SUC_lexicon_tags_31 },
    { 0xfde6a483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda6a489, SUC_lexicon_tags_25 },
    { 0x2026a48a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2286a48c, SUC_lexicon_tags_15 },
    { 0xd326a48d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9e6a48f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc06a496, SUC_lexicon_tags_17 },
    { 0x15c6a497, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd06a49f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d26a4a3, SUC_lexicon_tags_49 },
    { 0x32c6a4a3, SUC_lexicon_tags_25 },
    { 0x5a86a4a5, SUC_lexicon_tags_87 },
    { 0xc466a4a5, SUC_lexicon_tags_12 },
    { 0x5986a4a5, SUC_lexicon_tags_49 },
    { 0xcb86a4a8, SUC_lexicon_tags_16 },
    { 0xec6a4a5, SUC_lexicon_tags_11 },
    { 0xc746a4a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc6a4b0, SUC_lexicon_tags_11 },
    { 0x3186a4b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8406a4b3, SUC_lexicon_tags_9 },
    { 0xe186a4b4, SUC_lexicon_tags_34 },
    { 0xb46a4b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5786a4b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb06a4b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46a4bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc546a4c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e6a4c5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8aa6a4c7, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x62c6a4c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6266a4cb, SUC_lexicon_tags_7 },
    { 0xa0e6a4cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf66a4d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7966a4d7, SUC_lexicon_tags_79 },
    { 0x8fe6a4d7, SUC_lexicon_tags_31 },
    { 0x6e66a4d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa6a4dd, SUC_lexicon_tags_48 },
    { 0xc586a4de, SUC_lexicon_tags_243 },
    { 0xa146a4df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e46a4e1, SUC_lexicon_tags_7 },
    { 0xf1a6a4e1, SUC_lexicon_tags_11 },
    { 0x8d06a4e3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x126a4e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac6a4e9, SUC_lexicon_tags_7 },
    { 0x6086a4e9, SUC_lexicon_tags_11 },
    { 0xb386a4eb, SUC_lexicon_tags_10 },
    { 0xbb86a4eb, SUC_lexicon_tags_17 },
    { 0x4626a4ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2366a4f1, SUC_lexicon_tags_15 },
    { 0xb6a6a4f2, SUC_lexicon_tags_1231 },
    { 0xec66a4f2, SUC_lexicon_tags_14 },
    { 0x2d06a4f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9366a4f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8a6a4fb, SUC_lexicon_tags_8 },
    { 0xe8c6a4fc, SUC_lexicon_tags_15 },
    { 0x73c6a4fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd946a500, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff86a502, SUC_lexicon_tags_17 },
    { 0xc766a503, SUC_lexicon_tags_17 },
    { 0xf8a6a504, SUC_lexicon_tags_14 },
    { 0x62a6a502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1506a508, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa266a50a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb826a50c, SUC_lexicon_tags_25 },
    { 0x83e6a50c, SUC_lexicon_tags_11 },
    { 0xc66a50c, SUC_lexicon_tags_6 },
    { 0xb326a50f, SUC_lexicon_tags_25 },
    { 0x2586a50f, SUC_lexicon_tags_23 },
    { 0x16c6a510, SUC_lexicon_tags_7 },
    { 0xe786a50f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a6a518, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b66a51d, SUC_lexicon_tags_17 },
    { 0x45a6a51e, SUC_lexicon_tags_27 },
    { 0x2966a51f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d06a521, SUC_lexicon_tags_15 },
    { 0x14a6a522, SUC_lexicon_tags_26 },
    { 0xb9c6a523, SUC_lexicon_tags_12 },
    { 0x82e6a524, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xffa6a526, SUC_lexicon_tags_37 },
    { 0xab06a527, SUC_lexicon_tags_14 },
    { 0x1f06a528, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfba6a52a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a6a52d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8346a52f, SUC_lexicon_tags_39 },
    { 0x8f46a530, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a66a535, SUC_lexicon_tags_25 },
    { 0xf526a535, SUC_lexicon_tags_47 },
    { 0x72e6a537, SUC_lexicon_tags_23 },
    { 0x9166a538, SUC_lexicon_tags_27 },
    { 0xd26a538, SUC_lexicon_tags_182 },
    { 0xe346a539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c6a53d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd506a53f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226a545, SUC_lexicon_tags_27 },
    { 0x5e6a546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6a54b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf246a54d, SUC_lexicon_tags_48 },
    { 0x4de6a54e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6186a550, SUC_lexicon_tags_11 },
    { 0x46a6a551, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a6a554, SUC_lexicon_tags_25 },
    { 0xa566a554, SUC_lexicon_tags_49 },
    { 0xc266a556, SUC_lexicon_tags_1 },
    { 0xcf26a557, SUC_lexicon_tags_17 },
    { 0x9c46a557, SUC_lexicon_tags_8 },
    { 0xfbe6a557, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c6a55f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c6a562, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6a569, SUC_lexicon_tags_39 },
    { 0xf5c6a56a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb666a56f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e26a573, SUC_lexicon_tags_45 },
    { 0xd6c6a573, SUC_lexicon_tags_141 },
    { 0xee06a575, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7c86a577, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7486a57c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf126a582, SUC_lexicon_tags_56 },
    { 0xbd46a582, SUC_lexicon_tags_61 },
    { 0x5686a583, SUC_lexicon_tags_30 },
    { 0x2dc6a585, SUC_lexicon_tags_2 },
    { 0x1386a585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe466a588, SUC_lexicon_tags_14 },
    { 0x7806a588, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc846a58d, SUC_lexicon_tags_15 },
    { 0xcae6a58e, SUC_lexicon_tags_2 },
    { 0x4166a58f, SUC_lexicon_tags_17 },
    { 0x5246a590, SUC_lexicon_tags_2 },
    { 0xfac6a58e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe686a595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf026a597, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1406a599, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd206a59c, SUC_lexicon_tags_25 },
    { 0xb5a6a59c, SUC_lexicon_tags_17 },
    { 0x92a6a59c, SUC_lexicon_tags_14 },
    { 0xffa6a59f, SUC_lexicon_tags_9 },
    { 0xfdc6a5a0, SUC_lexicon_tags_11 },
    { 0x7e6a5a1, SUC_lexicon_tags_37 },
    { 0xe4c6a5a0, SUC_lexicon_tags_27 },
    { 0xcee6a5a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc6a5a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b06a5ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5366a5b0, SUC_lexicon_tags_25 },
    { 0xae86a5b1, SUC_lexicon_tags_6 },
    { 0x6086a5b2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a6a5b5, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c6a5b8, SUC_lexicon_tags_2 },
    { 0xd7e6a5b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca06a5bf, SUC_lexicon_tags_12 },
    { 0xef46a5bf, SUC_lexicon_tags_11 },
    { 0x3fa6a5c0, SUC_lexicon_tags_25 },
    { 0xc386a5c1, SUC_lexicon_tags_23 },
    { 0x3046a5c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e6a5c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c6a5cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72c6a5ce, SUC_lexicon_tags_25 },
    { 0x75c6a5cf, SUC_lexicon_tags_37 },
    { 0x1be6a5d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c6a5d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3806a5d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a5d9, SUC_lexicon_tags_63 },
    { 0xdc6a5d9, SUC_lexicon_tags_1 },
    { 0xc946a5db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946a5df, SUC_lexicon_tags_37 },
    { 0x3aa6a5e0, SUC_lexicon_tags_2 },
    { 0xcba6a5e1, SUC_lexicon_tags_8 },
    { 0xd726a5e1, SUC_lexicon_tags_116 },
    { 0xf86a5e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf26a5e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb46a5e8, SUC_lexicon_tags_17 },
    { 0xe686a5e8, SUC_lexicon_tags_3 },
    { 0x9526a5ea, SUC_lexicon_tags_15 },
    { 0x8146a5eb, SUC_lexicon_tags_27 },
    { 0x6406a5ec, SUC_lexicon_tags_27 },
    { 0xd166a5ed, SUC_lexicon_tags_25 },
    { 0x46e6a5ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb886a5f0, SUC_lexicon_tags_55 },
    { 0x7fe6a5f0, SUC_lexicon_tags_15 },
    { 0xa666a5f2, SUC_lexicon_tags_1 },
    { 0xb466a5f3, SUC_lexicon_tags_15 },
    { 0x69e6a5f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26a5f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c86a5fe, SUC_lexicon_tags_28 },
    { 0x5ac6a5ff, SUC_lexicon_tags_37 },
    { 0xa9a6a5ff, SUC_lexicon_tags_9 },
    { 0x97c6a600, SUC_lexicon_tags_17 },
    { 0xd186a602, SUC_lexicon_tags_26 },
    { 0xbfe6a602, SUC_lexicon_tags_45 },
    { 0xeea6a604, SUC_lexicon_tags_11 },
    { 0x4486a605, SUC_lexicon_tags_37 },
    { 0x25a6a606, SUC_lexicon_tags_0 },
    { 0x5346a605, SUC_lexicon_tags_0 },
    { 0x1386a600, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3466a60a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca46a60c, SUC_lexicon_tags_27 },
    { 0x88c6a60d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6a611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e26a613, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e6a619, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa246a61f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0a6a621, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x5fc6a623, SUC_lexicon_tags_47 },
    { 0x41c6a624, SUC_lexicon_tags_2 },
    { 0x1966a624, SUC_lexicon_tags_15 },
    { 0xee26a623, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206a62a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b26a62c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a6a633, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb146a63a, SUC_lexicon_tags_17 },
    { 0x95a6a63b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30a6a63d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1a6a63f, SUC_lexicon_tags_33 },
    { 0xbec6a640, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c6a648, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da6a64e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc46a650, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc26a653, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d66a659, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd366a65b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xed86a65d, SUC_lexicon_tags_47 },
    { 0xba06a65d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a6a661, SUC_lexicon_tags_38 },
    { 0xbc26a662, SUC_lexicon_tags_37 },
    { 0x3d06a663, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea6a669, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xdf46a66b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e66a672, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5886a675, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6486a679, SUC_lexicon_tags_3 },
    { 0x7e6a679, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb586a67c, SUC_lexicon_tags_37 },
    { 0xd46a67c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2a6a67f, SUC_lexicon_tags_26 },
    { 0xe366a680, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5446a68c, SUC_lexicon_tags_42 },
    { 0xef46a68d, SUC_lexicon_tags_259 },
    { 0xc7e6a68e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4586a690, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de6a694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7786a699, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdbc6a69b, SUC_lexicon_tags_11 },
    { 0x9586a69c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c6a6aa, SUC_lexicon_tags_1 },
    { 0xef66a6ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b26a6af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4926a6bc, SUC_lexicon_tags_29 },
    { 0x1626a6bd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e06a6c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6a6ca, SUC_lexicon_tags_29 },
    { 0x3986a6ca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de6a6d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4726a6d2, SUC_lexicon_tags_37 },
    { 0xac86a6d2, SUC_lexicon_tags_1 },
    { 0xe5c6a6d2, SUC_lexicon_tags_17 },
    { 0xfae6a6d5, SUC_lexicon_tags_60 },
    { 0xb206a6d4, SUC_lexicon_tags_43 },
    { 0x1706a6d4, SUC_lexicon_tags_37 },
    { 0x53a6a6d8, SUC_lexicon_tags_4 },
    { 0xb846a6d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe6a6e3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7406a6e5, SUC_lexicon_tags_85 },
    { 0x3c66a6e6, SUC_lexicon_tags_27 },
    { 0xdaa6a6e5, SUC_lexicon_tags_11 },
    { 0x2aa6a6e8, SUC_lexicon_tags_5 },
    { 0xb986a6e8, SUC_lexicon_tags_27 },
    { 0x5646a6ea, SUC_lexicon_tags_4 },
    { 0x5aa6a6eb, SUC_lexicon_tags_37 },
    { 0x7b66a6eb, SUC_lexicon_tags_49 },
    { 0x4906a6eb, SUC_lexicon_tags_79 },
    { 0x66a6a6ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb5e6a6f0, SUC_lexicon_tags_12 },
    { 0x8ae6a6f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8566a6f4, SUC_lexicon_tags_25 },
    { 0xf866a6f5, SUC_lexicon_tags_63 },
    { 0x68a6a6f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a6a6f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc26a6fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d46a6ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf206a704, SUC_lexicon_tags_37 },
    { 0xd066a705, SUC_lexicon_tags_17 },
    { 0xcaa6a706, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc126a70c, SUC_lexicon_tags_2 },
    { 0xa2c6a70d, SUC_lexicon_tags_60 },
    { 0xc566a70d, SUC_lexicon_tags_47 },
    { 0x30a6a70f, SUC_lexicon_tags_29 },
    { 0xc826a70f, SUC_lexicon_tags_2 },
    { 0x50e6a70f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1a86a713, SUC_lexicon_tags_45 },
    { 0x3e66a714, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9726a717, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c26a71a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c6a71d, SUC_lexicon_tags_11 },
    { 0xdf66a71e, SUC_lexicon_tags_98 },
    { 0xdfa6a71e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe266a722, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e6a729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6a737, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c6a73b, SUC_lexicon_tags_49 },
    { 0xeb26a73c, SUC_lexicon_tags_37 },
    { 0x5826a73d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e66a740, SUC_lexicon_tags_17 },
    { 0xa9a6a741, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe526a748, SUC_lexicon_tags_60 },
    { 0x4b66a748, SUC_lexicon_tags_12 },
    { 0xb5c6a74a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe566a750, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e86a755, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d06a757, SUC_lexicon_tags_1 },
    { 0xba86a758, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa6a762, SUC_lexicon_tags_16 },
    { 0x2f26a762, SUC_lexicon_tags_8 },
    { 0xd06a762, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06a767, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4fc6a769, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x96e6a76b, SUC_lexicon_tags_37 },
    { 0xadc6a76b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x56e6a76e, SUC_lexicon_tags_25 },
    { 0x226a76e, SUC_lexicon_tags_234 },
    { 0xbc06a770, SUC_lexicon_tags_49 },
    { 0xe1c6a771, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6106a778, SUC_lexicon_tags_13 },
    { 0x6a66a778, SUC_lexicon_tags_65 },
    { 0x3f46a77a, SUC_lexicon_tags_30 },
    { 0x40e6a778, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd706a77d, SUC_lexicon_tags_19 },
    { 0x4886a77e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366a783, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b86a787, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c86a789, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd4e6a78b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e66a78e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf846a792, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8146a796, SUC_lexicon_tags_17 },
    { 0xa546a797, SUC_lexicon_tags_10 },
    { 0xcf26a798, SUC_lexicon_tags_15 },
    { 0x9446a799, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a6a79f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc6a7aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91e6a7ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe6a7b0, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa206a7b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9386a7ba, SUC_lexicon_tags_55 },
    { 0x49a6a7bb, SUC_lexicon_tags_23 },
    { 0xa026a7bb, SUC_lexicon_tags_25 },
    { 0xa6c6a7bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe826a7bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe486a7c1, SUC_lexicon_tags_2 },
    { 0xa066a7c2, SUC_lexicon_tags_12 },
    { 0x37a6a7c3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe6a7c7, SUC_lexicon_tags_2 },
    { 0xa06a7c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb086a7cd, SUC_lexicon_tags_17 },
    { 0xc0a6a7ce, SUC_lexicon_tags_12 },
    { 0x46a7ce, SUC_lexicon_tags_10 },
    { 0xe1e6a7d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8486a7d4, SUC_lexicon_tags_37 },
    { 0xad46a7d5, SUC_lexicon_tags_1 },
    { 0x12e6a7d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e6a7d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6a7df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8646a7e4, SUC_lexicon_tags_8 },
    { 0xbdc6a7e5, SUC_lexicon_tags_43 },
    { 0x95e6a7e6, SUC_lexicon_tags_15 },
    { 0xfac6a7e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d66a7ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5286a7ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe6a7f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa466a7f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5bc6a7f7, SUC_lexicon_tags_23 },
    { 0xfea6a7f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec6a7fd, SUC_lexicon_tags_11 },
    { 0x43c6a7fe, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xd286a800, SUC_lexicon_tags_15 },
    { 0x4506a801, SUC_lexicon_tags_57 },
    { 0xab46a802, SUC_lexicon_tags_12 },
    { 0xb646a803, SUC_lexicon_tags_15 },
    { 0xbe6a804, SUC_lexicon_tags_23 },
    { 0xff06a804, SUC_lexicon_tags_2 },
    { 0xd3c6a805, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd186a808, SUC_lexicon_tags_39 },
    { 0xea66a809, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1406a80d, SUC_lexicon_tags_49 },
    { 0xe986a80e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9886a811, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7da6a813, SUC_lexicon_tags_30 },
    { 0x606a813, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca6a818, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae6a81b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2f06a81d, SUC_lexicon_tags_37 },
    { 0x59e6a81d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1a66a820, SUC_lexicon_tags_28 },
    { 0x1706a821, SUC_lexicon_tags_47 },
    { 0x6f46a822, SUC_lexicon_tags_38 },
    { 0x5e46a822, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c66a825, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4086a829, SUC_lexicon_tags_55 },
    { 0x98e6a829, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a6a82e, SUC_lexicon_tags_182 },
    { 0xda6a82f, SUC_lexicon_tags_87 },
    { 0x9c6a830, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ec6a832, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e6a836, SUC_lexicon_tags_17 },
    { 0x4546a837, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a83d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48c6a83f, SUC_lexicon_tags_38 },
    { 0x3ea6a840, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9746a842, SUC_lexicon_tags_47 },
    { 0x5926a843, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6646a845, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7726a84a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe006a84e, SUC_lexicon_tags_30 },
    { 0xe086a84e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0e6a851, SUC_lexicon_tags_39 },
    { 0xb4c6a852, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e46a856, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x3426a858, SUC_lexicon_tags_11 },
    { 0x2026a858, SUC_lexicon_tags_19 },
    { 0x8266a85a, SUC_lexicon_tags_25 },
    { 0x53e6a859, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c26a85e, SUC_lexicon_tags_26 },
    { 0x3886a85f, SUC_lexicon_tags_2 },
    { 0xd086a860, SUC_lexicon_tags_35 },
    { 0x69a6a861, SUC_lexicon_tags_27 },
    { 0xeae6a85e, SUC_lexicon_tags_17 },
    { 0x74e6a863, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c6a867, SUC_lexicon_tags_25 },
    { 0x2246a868, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9106a86d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a26a870, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e6a874, SUC_lexicon_tags_38 },
    { 0x4826a874, SUC_lexicon_tags_60 },
    { 0x5546a876, SUC_lexicon_tags_17 },
    { 0x4786a874, SUC_lexicon_tags_919 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd06a87a, SUC_lexicon_tags_43 },
    { 0x8e6a87a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e6a87e, SUC_lexicon_tags_1 },
    { 0x21e6a87f, SUC_lexicon_tags_25 },
    { 0x7d46a880, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a884, SUC_lexicon_tags_11 },
    { 0xf1c6a885, SUC_lexicon_tags_37 },
    { 0xb786a886, SUC_lexicon_tags_8 },
    { 0x2dc6a884, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1dc6a889, SUC_lexicon_tags_43 },
    { 0x5886a889, SUC_lexicon_tags_17 },
    { 0xd886a88b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3a6a88d, SUC_lexicon_tags_63 },
    { 0xd946a88d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe086a898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c6a89c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae6a8aa, SUC_lexicon_tags_26 },
    { 0x1a26a8aa, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca6a8af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ce6a8b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe186a8b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7c6a8b6, SUC_lexicon_tags_2 },
    { 0xbd86a8b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7826a8ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae86a8bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c6a8c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6a8cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1e6a8cd, SUC_lexicon_tags_25 },
    { 0xc986a8cd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7b86a8d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76c6a8d2, SUC_lexicon_tags_56 },
    { 0x6a26a8d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5586a8d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc6a8e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2206a8e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf046a8eb, SUC_lexicon_tags_47 },
    { 0x4586a8ec, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2046a8f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa846a903, SUC_lexicon_tags_25 },
    { 0xd6a6a904, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd626a907, SUC_lexicon_tags_87 },
    { 0xa206a908, SUC_lexicon_tags_5 },
    { 0xdfe6a909, SUC_lexicon_tags_43 },
    { 0xdce6a907, SUC_lexicon_tags_37 },
    { 0x8de6a90b, SUC_lexicon_tags_84 },
    { 0xa266a90c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9e6a90e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e86a911, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166a919, SUC_lexicon_tags_23 },
    { 0xb06a919, SUC_lexicon_tags_38 },
    { 0x3226a91b, SUC_lexicon_tags_79 },
    { 0xeb26a91c, SUC_lexicon_tags_38 },
    { 0x7226a91d, SUC_lexicon_tags_11 },
    { 0x1146a91a, SUC_lexicon_tags_65 },
    { 0x7c46a91f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce6a923, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3006a925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b66a92a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x47a6a92c, SUC_lexicon_tags_10 },
    { 0x7346a92d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e46a92f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa86a932, SUC_lexicon_tags_23 },
    { 0xf6a6a933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3226a935, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e06a939, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc46a93b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe6a93e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x986a940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a86a944, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a6a948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1566a94c, SUC_lexicon_tags_11 },
    { 0x1c86a94d, SUC_lexicon_tags_15 },
    { 0x4586a94d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e86a952, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd66a954, SUC_lexicon_tags_48 },
    { 0x2666a955, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4426a957, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf326a95a, SUC_lexicon_tags_8 },
    { 0x7c86a95b, SUC_lexicon_tags_0 },
    { 0x2466a95c, SUC_lexicon_tags_17 },
    { 0x1e06a95a, SUC_lexicon_tags_26 },
    { 0x1666a95e, SUC_lexicon_tags_66 },
    { 0xc626a95e, SUC_lexicon_tags_8 },
    { 0x4c66a95e, SUC_lexicon_tags_48 },
    { 0x9326a961, SUC_lexicon_tags_15 },
    { 0x5486a962, SUC_lexicon_tags_25 },
    { 0x87a6a95e, SUC_lexicon_tags_27 },
    { 0xc4c6a964, SUC_lexicon_tags_46 },
    { 0xc6e6a961, SUC_lexicon_tags_11 },
    { 0x6c66a963, SUC_lexicon_tags_29 },
    { 0x5186a95f, SUC_lexicon_tags_17 },
    { 0x6106a968, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c6a96f, SUC_lexicon_tags_33 },
    { 0x4446a96f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a6a973, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e6a977, SUC_lexicon_tags_37 },
    { 0x7626a978, SUC_lexicon_tags_10 },
    { 0x326a978, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4046a981, SUC_lexicon_tags_43 },
    { 0x1a26a982, SUC_lexicon_tags_9 },
    { 0x8d86a982, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c46a985, SUC_lexicon_tags_37 },
    { 0x4cc6a985, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde86a98f, SUC_lexicon_tags_23 },
    { 0xaa46a98f, SUC_lexicon_tags_16 },
    { 0x83c6a990, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6a994, SUC_lexicon_tags_29 },
    { 0x7d46a994, SUC_lexicon_tags_37 },
    { 0xb786a996, SUC_lexicon_tags_26 },
    { 0x3366a995, SUC_lexicon_tags_61 },
    { 0x69c6a998, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7986a99a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba66a99d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a46a99f, SUC_lexicon_tags_11 },
    { 0xb566a9a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3966a9a2, SUC_lexicon_tags_15 },
    { 0xe2c6a9a3, SUC_lexicon_tags_12 },
    { 0x1906a9a2, SUC_lexicon_tags_37 },
    { 0xb766a9a4, SUC_lexicon_tags_25 },
    { 0x8746a9a6, SUC_lexicon_tags_33 },
    { 0x9526a9a6, SUC_lexicon_tags_27 },
    { 0xcda6a9a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x326a9aa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6fe6a9ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7426a9af, SUC_lexicon_tags_33 },
    { 0xf5a6a9b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce6a9b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c6a9b6, SUC_lexicon_tags_9 },
    { 0xa006a9b7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6366a9ba, SUC_lexicon_tags_49 },
    { 0x8306a9bb, SUC_lexicon_tags_66 },
    { 0x6ce6a9bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcea6a9bf, SUC_lexicon_tags_8 },
    { 0x2406a9bf, SUC_lexicon_tags_37 },
    { 0xf7c6a9c1, SUC_lexicon_tags_0 },
    { 0x3bc6a9c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1226a9c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xade6a9c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb746a9cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4366a9d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d26a9d4, SUC_lexicon_tags_25 },
    { 0x6306a9d5, SUC_lexicon_tags_37 },
    { 0xeea6a9d4, SUC_lexicon_tags_33 },
    { 0x3686a9d7, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0xae66a9d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9586a9dc, SUC_lexicon_tags_26 },
    { 0xc5e6a9dd, SUC_lexicon_tags_49 },
    { 0xd846a9dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1646a9e1, SUC_lexicon_tags_15 },
    { 0x63a6a9e1, SUC_lexicon_tags_14 },
    { 0xe5c6a9e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b86a9e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a6a9e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9146a9ec, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa5e6a9ee, SUC_lexicon_tags_6 },
    { 0xfd46a9ef, SUC_lexicon_tags_25 },
    { 0xd9c6a9ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6506a9f4, SUC_lexicon_tags_1 },
    { 0x77c6a9f5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e06a9f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4926aa00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a6aa04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa906aa08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd086aa0c, SUC_lexicon_tags_25 },
    { 0xece6aa0c, SUC_lexicon_tags_53 },
    { 0x6c86aa0d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa946aa16, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b46aa19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9146aa1d, SUC_lexicon_tags_178 },
    { 0xd4e6aa1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc526aa22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae46aa25, SUC_lexicon_tags_2 },
    { 0x2b86aa26, SUC_lexicon_tags_29 },
    { 0x9506aa27, SUC_lexicon_tags_37 },
    { 0x806aa25, SUC_lexicon_tags_47 },
    { 0x366aa28, SUC_lexicon_tags_17 },
    { 0x4426aa2a, SUC_lexicon_tags_11 },
    { 0xf5a6aa2b, SUC_lexicon_tags_45 },
    { 0xf866aa2b, SUC_lexicon_tags_14 },
    { 0x8c86aa2c, SUC_lexicon_tags_15 },
    { 0x4d06aa2d, SUC_lexicon_tags_1 },
    { 0x7c66aa2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c66aa3e, SUC_lexicon_tags_30 },
    { 0x7ba6aa3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6aa42, SUC_lexicon_tags_31 },
    { 0x8e46aa43, SUC_lexicon_tags_2 },
    { 0xdac6aa43, SUC_lexicon_tags_8 },
    { 0xe6a6aa43, SUC_lexicon_tags_1 },
    { 0x7306aa46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e6aa49, SUC_lexicon_tags_15 },
    { 0x3c06aa4a, SUC_lexicon_tags_3 },
    { 0xe606aa4a, SUC_lexicon_tags_27 },
    { 0x4106aa4c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe6aa5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca6aa63, SUC_lexicon_tags_29 },
    { 0x7f06aa63, SUC_lexicon_tags_31 },
    { 0x3526aa65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3886aa67, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6aa6a, SUC_lexicon_tags_15 },
    { 0xbf66aa6b, SUC_lexicon_tags_1 },
    { 0x12a6aa6c, SUC_lexicon_tags_15 },
    { 0xd506aa6c, SUC_lexicon_tags_41 },
    { 0x70a6aa6c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2526aa72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb766aa74, SUC_lexicon_tags_6 },
    { 0x1ca6aa75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33e6aa77, SUC_lexicon_tags_66 },
    { 0xc26aa78, SUC_lexicon_tags_25 },
    { 0xc426aa79, SUC_lexicon_tags_12 },
    { 0x6986aa7a, SUC_lexicon_tags_33 },
    { 0x47e6aa77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee6aa7e, SUC_lexicon_tags_17 },
    { 0xad66aa7e, SUC_lexicon_tags_0 },
    { 0xdbc6aa80, SUC_lexicon_tags_25 },
    { 0x2a26aa80, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7226aa83, SUC_lexicon_tags_10 },
    { 0xf086aa84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2706aa86, SUC_lexicon_tags_2 },
    { 0x5886aa86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe206aa89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeaa6aa8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6206aa92, SUC_lexicon_tags_65 },
    { 0x8e26aa92, SUC_lexicon_tags_25 },
    { 0x67a6aa94, SUC_lexicon_tags_14 },
    { 0xcd46aa93, SUC_lexicon_tags_133 },
    { 0x4e66aa93, SUC_lexicon_tags_27 },
    { 0x62a6aa97, SUC_lexicon_tags_63 },
    { 0xe226aa97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f26aa9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8826aa9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e06aa9f, SUC_lexicon_tags_25 },
    { 0x3246aa9f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3886aaa3, SUC_lexicon_tags_37 },
    { 0x9426aaa4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd06aaa7, SUC_lexicon_tags_11 },
    { 0xdb26aaa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7646aaab, SUC_lexicon_tags_4 },
    { 0x9d06aaac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfec6aaae, SUC_lexicon_tags_7 },
    { 0x6fc6aaae, SUC_lexicon_tags_0 },
    { 0x3e66aaaf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfc6aab2, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8846aab6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a6aab9, SUC_lexicon_tags_9 },
    { 0x71c6aaba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4666aabc, SUC_lexicon_tags_15 },
    { 0x83a6aabd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a66aac3, SUC_lexicon_tags_0 },
    { 0x9046aac4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6ec6aac6, SUC_lexicon_tags_11 },
    { 0x34c6aac7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce6aacd, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed46aad2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e6aad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3986aad7, SUC_lexicon_tags_28 },
    { 0xada6aad8, SUC_lexicon_tags_30 },
    { 0x1146aad7, SUC_lexicon_tags_9 },
    { 0x2dc6aad8, SUC_lexicon_tags_17 },
    { 0x5f26aadb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e86aadf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb706aae6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7d06aae8, SUC_lexicon_tags_26 },
    { 0x96c6aae9, SUC_lexicon_tags_1 },
    { 0xb3a6aaea, SUC_lexicon_tags_33 },
    { 0x2f46aaeb, SUC_lexicon_tags_2 },
    { 0x3c66aaec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6aaf3, SUC_lexicon_tags_7 },
    { 0xf46aaf4, SUC_lexicon_tags_65 },
    { 0xc006aaf5, SUC_lexicon_tags_77 },
    { 0xa386aaf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa406aaf8, SUC_lexicon_tags_15 },
    { 0x54c6aaf8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b06aafc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe526aaff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8006ab01, SUC_lexicon_tags_1 },
    { 0x27e6ab02, SUC_lexicon_tags_27 },
    { 0x4e26ab03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf626ab05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc6ab08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce6ab0e, SUC_lexicon_tags_27 },
    { 0xeb06ab0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fa6ab11, SUC_lexicon_tags_32 },
    { 0x1026ab12, SUC_lexicon_tags_2 },
    { 0x73c6ab12, SUC_lexicon_tags_25 },
    { 0xd186ab14, SUC_lexicon_tags_0 },
    { 0x7486ab15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c6ab18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcc06ab1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a6ab1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf26ab20, SUC_lexicon_tags_38 },
    { 0xda66ab20, SUC_lexicon_tags_11 },
    { 0xa886ab22, SUC_lexicon_tags_15 },
    { 0x3846ab20, SUC_lexicon_tags_35 },
    { 0x506ab24, SUC_lexicon_tags_47 },
    { 0x32c6ab25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee6ab29, SUC_lexicon_tags_37 },
    { 0x8666ab29, SUC_lexicon_tags_7 },
    { 0xf5a6ab2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea6ab30, SUC_lexicon_tags_23 },
    { 0x6ae6ab31, SUC_lexicon_tags_27 },
    { 0x9b46ab32, SUC_lexicon_tags_11 },
    { 0x4666ab33, SUC_lexicon_tags_6 },
    { 0xcfa6ab34, SUC_lexicon_tags_8 },
    { 0x1946ab33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x76a6ab37, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c6ab3f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaea6ab41, SUC_lexicon_tags_37 },
    { 0x33c6ab42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9086ab46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1206ab49, SUC_lexicon_tags_40 },
    { 0x8086ab49, SUC_lexicon_tags_46 },
    { 0x37e6ab4b, SUC_lexicon_tags_47 },
    { 0xf166ab4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5366ab51, SUC_lexicon_tags_17 },
    { 0xc3c6ab52, SUC_lexicon_tags_7 },
    { 0x3c46ab53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe66ab55, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab5a, SUC_lexicon_tags_13 },
    { 0xd826ab5a, SUC_lexicon_tags_1 },
    { 0x73a6ab5c, SUC_lexicon_tags_38 },
    { 0x446ab5d, SUC_lexicon_tags_2 },
    { 0xa766ab5c, SUC_lexicon_tags_9 },
    { 0x4f26ab5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186ab63, SUC_lexicon_tags_12 },
    { 0x6106ab63, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a46ab70, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2f26ab72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc746ab75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a6ab7b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c6ab7f, SUC_lexicon_tags_1 },
    { 0x81a6ab80, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e6ab83, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e46ab86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7766ab89, SUC_lexicon_tags_6 },
    { 0x53c6ab8a, SUC_lexicon_tags_9 },
    { 0x94c6ab8a, SUC_lexicon_tags_15 },
    { 0x43a6ab89, SUC_lexicon_tags_64 },
    { 0xb3c6ab8c, SUC_lexicon_tags_11 },
    { 0xac86ab8d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed66ab95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd286ab9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9126ab9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6aba3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca6abaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8886abb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6abb6, SUC_lexicon_tags_11 },
    { 0x3fc6abb7, SUC_lexicon_tags_12 },
    { 0x3a86abb8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba6abbc, SUC_lexicon_tags_6 },
    { 0x34e6abbc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1386abbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc6abc4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e66abc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e6abd4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xde66abd6, SUC_lexicon_tags_37 },
    { 0x21c6abd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c6abda, SUC_lexicon_tags_45 },
    { 0x2446abda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c26abdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcb86abe1, SUC_lexicon_tags_33 },
    { 0xf286abe2, SUC_lexicon_tags_17 },
    { 0x5c06abe3, SUC_lexicon_tags_37 },
    { 0xce66abe4, SUC_lexicon_tags_22 },
    { 0x84a6abe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2366abe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d66abe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca6abec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4346abf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae6abf5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba46abf7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd786abf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfce6abfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d86ac01, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e6ac08, SUC_lexicon_tags_11 },
    { 0x3d06ac08, SUC_lexicon_tags_14 },
    { 0x666ac0a, SUC_lexicon_tags_19 },
    { 0x3a86ac0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c26ac0e, SUC_lexicon_tags_25 },
    { 0xbae6ac0f, SUC_lexicon_tags_45 },
    { 0x4526ac0e, SUC_lexicon_tags_120 },
    { 0x886ac11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf766ac13, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x69a6ac15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c6ac1c, SUC_lexicon_tags_25 },
    { 0xf5c6ac1c, SUC_lexicon_tags_28 },
    { 0x4be6ac1e, SUC_lexicon_tags_14 },
    { 0xc326ac1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5706ac23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec6ac26, SUC_lexicon_tags_12 },
    { 0xbb06ac27, SUC_lexicon_tags_38 },
    { 0x5286ac28, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec6ac2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd806ac2e, SUC_lexicon_tags_26 },
    { 0x7aa6ac2f, SUC_lexicon_tags_19 },
    { 0x8286ac30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a86ac32, SUC_lexicon_tags_45 },
    { 0x6d26ac32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa626ac35, SUC_lexicon_tags_2 },
    { 0xb3a6ac36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf566ac39, SUC_lexicon_tags_25 },
    { 0x8f86ac3a, SUC_lexicon_tags_28 },
    { 0xcac6ac3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe526ac40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8986ac46, SUC_lexicon_tags_37 },
    { 0xf826ac47, SUC_lexicon_tags_37 },
    { 0x3d86ac47, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7686ac51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e6ac58, SUC_lexicon_tags_37 },
    { 0xd166ac58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66a6ac5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7086ac5d, SUC_lexicon_tags_77 },
    { 0xfd46ac5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaba6ac60, SUC_lexicon_tags_12 },
    { 0xa986ac61, SUC_lexicon_tags_49 },
    { 0x3d26ac61, SUC_lexicon_tags_11 },
    { 0xc926ac62, SUC_lexicon_tags_9 },
    { 0xa8e6ac64, SUC_lexicon_tags_27 },
    { 0xdf26ac61, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd66ac6b, SUC_lexicon_tags_19 },
    { 0xaa06ac6b, SUC_lexicon_tags_11 },
    { 0xf86ac6b, SUC_lexicon_tags_25 },
    { 0x69a6ac6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6146ac70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18e6ac72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd846ac77, SUC_lexicon_tags_19 },
    { 0xdde6ac77, SUC_lexicon_tags_37 },
    { 0x5366ac79, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2906ac7b, SUC_lexicon_tags_2 },
    { 0xb8a6ac7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde6ac81, SUC_lexicon_tags_29 },
    { 0x2b66ac82, SUC_lexicon_tags_29 },
    { 0x48c6ac82, SUC_lexicon_tags_33 },
    { 0x4a86ac84, SUC_lexicon_tags_60 },
    { 0x7a26ac83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a6ac88, SUC_lexicon_tags_6 },
    { 0x1da6ac89, SUC_lexicon_tags_25 },
    { 0x81a6ac8a, SUC_lexicon_tags_25 },
    { 0x58c6ac8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa566ac8e, SUC_lexicon_tags_68 },
    { 0x3fc6ac8f, SUC_lexicon_tags_38 },
    { 0x7626ac90, SUC_lexicon_tags_35 },
    { 0xe786ac8f, SUC_lexicon_tags_17 },
    { 0x4926ac91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9546ac9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f86aca5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1a6aca7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a6acaa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1266acac, SUC_lexicon_tags_17 },
    { 0x3906acad, SUC_lexicon_tags_1 },
    { 0xa846acad, SUC_lexicon_tags_5 },
    { 0x2c66acaf, SUC_lexicon_tags_6 },
    { 0x97c6acb0, SUC_lexicon_tags_25 },
    { 0xcf66acb1, SUC_lexicon_tags_25 },
    { 0x6486acb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526acb7, SUC_lexicon_tags_17 },
    { 0x9446acb8, SUC_lexicon_tags_26 },
    { 0x54e6acb8, SUC_lexicon_tags_38 },
    { 0xb786acba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c66acbd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba06acbf, SUC_lexicon_tags_2 },
    { 0xda86acc0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b46acc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78a6acc4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5926accd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e6acd1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xec06acd4, SUC_lexicon_tags_6 },
    { 0x4ba6acd5, SUC_lexicon_tags_25 },
    { 0x78c6acd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6686ace1, SUC_lexicon_tags_29 },
    { 0x52e6ace2, SUC_lexicon_tags_65 },
    { 0x3ea6ace3, SUC_lexicon_tags_2 },
    { 0x8f86ace4, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a6ace7, SUC_lexicon_tags_21 },
    { 0xda46ace7, SUC_lexicon_tags_7 },
    { 0x3886ace9, SUC_lexicon_tags_38 },
    { 0x1646ace7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2226acee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc6acf3, SUC_lexicon_tags_1 },
    { 0x7f06acf4, SUC_lexicon_tags_17 },
    { 0x8646acf5, SUC_lexicon_tags_2 },
    { 0x93e6acf4, SUC_lexicon_tags_17 },
    { 0xab06acf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b86acfd, SUC_lexicon_tags_25 },
    { 0x8ea6acfe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b06ad01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa06ad08, SUC_lexicon_tags_7 },
    { 0x2606ad09, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d86ad0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb66ad14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc346ad18, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad86ad1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf66ad20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3586ad23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26ad27, SUC_lexicon_tags_25 },
    { 0x97c6ad27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe666ad2d, SUC_lexicon_tags_160 },
    { 0xfbc6ad2e, SUC_lexicon_tags_37 },
    { 0xef26ad2e, SUC_lexicon_tags_0 },
    { 0x7726ad30, SUC_lexicon_tags_11 },
    { 0x9e06ad31, SUC_lexicon_tags_27 },
    { 0xbbc6ad32, SUC_lexicon_tags_49 },
    { 0xf206ad2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xebe6ad35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1c46ad37, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa746ad3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5706ad3d, SUC_lexicon_tags_79 },
    { 0x5006ad3e, SUC_lexicon_tags_12 },
    { 0x4e46ad3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1e6ad41, SUC_lexicon_tags_43 },
    { 0x14c6ad42, SUC_lexicon_tags_28 },
    { 0x5546ad42, SUC_lexicon_tags_60 },
    { 0x8106ad42, SUC_lexicon_tags_26 },
    { 0x9266ad44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2406ad4a, SUC_lexicon_tags_25 },
    { 0xe46ad4b, SUC_lexicon_tags_37 },
    { 0x3406ad4c, SUC_lexicon_tags_79 },
    { 0xc986ad4a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x27a6ad4f, SUC_lexicon_tags_11 },
    { 0x2906ad50, SUC_lexicon_tags_38 },
    { 0xd9e6ad4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf766ad53, SUC_lexicon_tags_17 },
    { 0x3446ad53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3046ad56, SUC_lexicon_tags_11 },
    { 0xbb06ad57, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa386ad5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb5c6ad5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8206ad60, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf546ad62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c26ad6c, SUC_lexicon_tags_14 },
    { 0x6286ad6d, SUC_lexicon_tags_17 },
    { 0x1906ad6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa6ad71, SUC_lexicon_tags_12 },
    { 0xefe6ad71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe826ad7a, SUC_lexicon_tags_57 },
    { 0x7206ad7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46ad7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de6ad84, SUC_lexicon_tags_1 },
    { 0xdf46ad85, SUC_lexicon_tags_9 },
    { 0x8b46ad85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29e6ad88, SUC_lexicon_tags_25 },
    { 0xa826ad89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8166ad8d, SUC_lexicon_tags_12 },
    { 0x4486ad8d, SUC_lexicon_tags_32 },
    { 0x2046ad8f, SUC_lexicon_tags_37 },
    { 0xbb46ad90, SUC_lexicon_tags_16 },
    { 0x2d46ad91, SUC_lexicon_tags_17 },
    { 0xa8e6ad92, SUC_lexicon_tags_11 },
    { 0xcba6ad93, SUC_lexicon_tags_12 },
    { 0xc3a6ad91, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2866ad98, SUC_lexicon_tags_14 },
    { 0x6ee6ad98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6dc6ad9b, SUC_lexicon_tags_17 },
    { 0x5ca6ad9c, SUC_lexicon_tags_25 },
    { 0x5c66ad9d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c6ada5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc006ada8, SUC_lexicon_tags_7 },
    { 0xc366ada9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xeb26adab, SUC_lexicon_tags_47 },
    { 0x8f06adab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c6adaf, SUC_lexicon_tags_58 },
    { 0xa726adb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae86adb3, SUC_lexicon_tags_114 },
    { 0x6fc6adb4, SUC_lexicon_tags_27 },
    { 0xba6adb5, SUC_lexicon_tags_15 },
    { 0x4e66adb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda6adb9, SUC_lexicon_tags_17 },
    { 0x54c6adba, SUC_lexicon_tags_14 },
    { 0x706adbb, SUC_lexicon_tags_37 },
    { 0xec06adba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe566adc5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x90e6adc7, SUC_lexicon_tags_47 },
    { 0x4306adc8, SUC_lexicon_tags_33 },
    { 0xc6a6adc9, SUC_lexicon_tags_38 },
    { 0x5ba6adca, SUC_lexicon_tags_45 },
    { 0x386adca, SUC_lexicon_tags_1 },
    { 0xf166adcc, SUC_lexicon_tags_12 },
    { 0xcdc6adcd, SUC_lexicon_tags_32 },
    { 0xfec6adce, SUC_lexicon_tags_17 },
    { 0xa46adc9, SUC_lexicon_tags_33 },
    { 0x1406adc7, SUC_lexicon_tags_1 },
    { 0xd666adcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3646add8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e6ade0, SUC_lexicon_tags_38 },
    { 0x8126ade1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6346ade7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1a46ade9, SUC_lexicon_tags_43 },
    { 0xf826ade9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e6adee, SUC_lexicon_tags_68 },
    { 0x91e6adef, SUC_lexicon_tags_62 },
    { 0x4d06adef, SUC_lexicon_tags_61 },
    { 0xc766adf1, SUC_lexicon_tags_7 },
    { 0x2f26adf2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa126adf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0e6adf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486adfe, SUC_lexicon_tags_11 },
    { 0xcae6adfe, SUC_lexicon_tags_26 },
    { 0xc86adff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be6ae05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7026ae0c, SUC_lexicon_tags_21 },
    { 0xecc6ae0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6a6ae0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8426ae11, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e26ae19, SUC_lexicon_tags_23 },
    { 0x3366ae1a, SUC_lexicon_tags_1 },
    { 0x1c6ae19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcb86ae1d, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b26ae20, SUC_lexicon_tags_14 },
    { 0x6fa6ae20, SUC_lexicon_tags_25 },
    { 0x19c6ae20, SUC_lexicon_tags_6 },
    { 0xd546ae23, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x38c6ae25, SUC_lexicon_tags_37 },
    { 0xb106ae26, SUC_lexicon_tags_45 },
    { 0x3406ae27, SUC_lexicon_tags_47 },
    { 0xc1e6ae27, SUC_lexicon_tags_28 },
    { 0x82c6ae28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2346ae2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e6ae30, SUC_lexicon_tags_11 },
    { 0x78c6ae30, SUC_lexicon_tags_47 },
    { 0xa146ae31, SUC_lexicon_tags_0 },
    { 0x80e6ae33, SUC_lexicon_tags_37 },
    { 0x25e6ae34, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd406ae36, SUC_lexicon_tags_7 },
    { 0xa286ae37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc346ae39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed06ae3d, SUC_lexicon_tags_31 },
    { 0xb986ae3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e6ae41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c6ae44, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c6ae49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a6ae4c, SUC_lexicon_tags_47 },
    { 0xcb06ae4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac06ae54, SUC_lexicon_tags_2 },
    { 0xe866ae55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1346ae57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b46ae59, SUC_lexicon_tags_14 },
    { 0x6ae6ae5a, SUC_lexicon_tags_11 },
    { 0x33a6ae5b, SUC_lexicon_tags_43 },
    { 0x2e86ae5b, SUC_lexicon_tags_29 },
    { 0x2ae6ae5a, SUC_lexicon_tags_25 },
    { 0xbc86ae5c, SUC_lexicon_tags_6 },
    { 0xeaa6ae5f, SUC_lexicon_tags_47 },
    { 0x50a6ae5d, SUC_lexicon_tags_25 },
    { 0xf106ae5c, SUC_lexicon_tags_7 },
    { 0xa386ae62, SUC_lexicon_tags_39 },
    { 0x986ae63, SUC_lexicon_tags_9 },
    { 0xdc46ae62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86ae67, SUC_lexicon_tags_21 },
    { 0x1f86ae68, SUC_lexicon_tags_27 },
    { 0x1746ae69, SUC_lexicon_tags_103 },
    { 0xf7c6ae6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed46ae6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe686ae71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc046ae78, SUC_lexicon_tags_17 },
    { 0xfd06ae79, SUC_lexicon_tags_2 },
    { 0x9886ae79, SUC_lexicon_tags_3 },
    { 0x6466ae7b, SUC_lexicon_tags_15 },
    { 0xc506ae7c, SUC_lexicon_tags_2 },
    { 0x2006ae7d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9266ae80, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc6ae83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcee6ae85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb206ae87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda6ae8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd706ae8c, SUC_lexicon_tags_43 },
    { 0x3506ae8d, SUC_lexicon_tags_27 },
    { 0x6c86ae8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc6ae91, SUC_lexicon_tags_7 },
    { 0xaf46ae92, SUC_lexicon_tags_26 },
    { 0x2b46ae91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e6ae97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3446ae9a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6246ae9d, SUC_lexicon_tags_6 },
    { 0x8c66ae9e, SUC_lexicon_tags_15 },
    { 0xe1c6ae9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc6aea3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7a6aea5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a6aea8, SUC_lexicon_tags_60 },
    { 0xa26aea8, SUC_lexicon_tags_25 },
    { 0x6946aeaa, SUC_lexicon_tags_0 },
    { 0xea6aea8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7686aead, SUC_lexicon_tags_15 },
    { 0x3206aeae, SUC_lexicon_tags_17 },
    { 0x1a86aeaf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc6aeb4, SUC_lexicon_tags_6 },
    { 0xd426aeb4, SUC_lexicon_tags_77 },
    { 0xf226aeb4, SUC_lexicon_tags_11 },
    { 0x1dc6aeb7, SUC_lexicon_tags_15 },
    { 0xbf46aeb6, SUC_lexicon_tags_47 },
    { 0xc806aeb9, SUC_lexicon_tags_63 },
    { 0x5f06aeb6, SUC_lexicon_tags_7 },
    { 0x9f26aebb, SUC_lexicon_tags_65 },
    { 0x8226aebb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef26aec4, SUC_lexicon_tags_38 },
    { 0xf506aec5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x186aec8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66aecc, SUC_lexicon_tags_12 },
    { 0x62e6aecc, SUC_lexicon_tags_28 },
    { 0x6866aecc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c66aed9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a66aedf, SUC_lexicon_tags_23 },
    { 0x79c6aee0, SUC_lexicon_tags_91 },
    { 0xa306aee0, SUC_lexicon_tags_23 },
    { 0x4d06aee0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e46aee6, SUC_lexicon_tags_1 },
    { 0x4286aee7, SUC_lexicon_tags_45 },
    { 0x5026aee8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6106aeed, SUC_lexicon_tags_10 },
    { 0xdb46aeed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6aef2, SUC_lexicon_tags_2 },
    { 0xf446aef3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2b86aef5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee6aefb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1c6aefd, SUC_lexicon_tags_0 },
    { 0x73c6aefe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb86af01, SUC_lexicon_tags_15 },
    { 0x41a6af02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3266af05, SUC_lexicon_tags_21 },
    { 0xa7c6af05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba6af0b, SUC_lexicon_tags_43 },
    { 0xcde6af0c, SUC_lexicon_tags_12 },
    { 0x4166af0c, SUC_lexicon_tags_30 },
    { 0x3b46af0e, SUC_lexicon_tags_25 },
    { 0x3086af0f, SUC_lexicon_tags_47 },
    { 0x6966af0f, SUC_lexicon_tags_7 },
    { 0xb406af0e, SUC_lexicon_tags_37 },
    { 0x8466af12, SUC_lexicon_tags_6 },
    { 0xc786af12, SUC_lexicon_tags_47 },
    { 0xa346af14, SUC_lexicon_tags_23 },
    { 0x8126af15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9506af18, SUC_lexicon_tags_2 },
    { 0xf446af18, SUC_lexicon_tags_10 },
    { 0x1426af1a, SUC_lexicon_tags_27 },
    { 0x4dc6af18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a6af20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb46af25, SUC_lexicon_tags_25 },
    { 0x24c6af25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4506af2a, SUC_lexicon_tags_15 },
    { 0x1226af2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f86af31, SUC_lexicon_tags_11 },
    { 0x4086af32, SUC_lexicon_tags_43 },
    { 0xf386af32, SUC_lexicon_tags_25 },
    { 0x66e6af32, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726af39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5906af3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e6af43, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a66af46, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4326af4d, SUC_lexicon_tags_25 },
    { 0xdd26af4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28e6af50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f66af52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb286af54, SUC_lexicon_tags_12 },
    { 0x1386af55, SUC_lexicon_tags_17 },
    { 0x50e6af55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd646af59, SUC_lexicon_tags_7 },
    { 0x55a6af5a, SUC_lexicon_tags_17 },
    { 0xb1a6af5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x746af5d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee46af62, SUC_lexicon_tags_3 },
    { 0x2726af63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5866af66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc6af75, SUC_lexicon_tags_60 },
    { 0x2c06af75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a6af79, SUC_lexicon_tags_66 },
    { 0x6226af79, SUC_lexicon_tags_1 },
    { 0x3146af79, SUC_lexicon_tags_45 },
    { 0x5626af7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6986af81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6966af83, SUC_lexicon_tags_6 },
    { 0x7a6af83, SUC_lexicon_tags_95 },
    { 0x85e6af85, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x8da6af87, SUC_lexicon_tags_1 },
    { 0xd286af87, SUC_lexicon_tags_7 },
    { 0xe026af89, SUC_lexicon_tags_17 },
    { 0x5186af89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8286af8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a6af91, SUC_lexicon_tags_37 },
    { 0xfc46af92, SUC_lexicon_tags_17 },
    { 0x40a6af91, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e46af96, SUC_lexicon_tags_33 },
    { 0x7c26af97, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6c06af99, SUC_lexicon_tags_7 },
    { 0x6906af9a, SUC_lexicon_tags_17 },
    { 0x7dc6af9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b66afa5, SUC_lexicon_tags_25 },
    { 0xe586afa6, SUC_lexicon_tags_8 },
    { 0x4606afa6, SUC_lexicon_tags_25 },
    { 0x6306afa8, SUC_lexicon_tags_37 },
    { 0x9906afa8, SUC_lexicon_tags_11 },
    { 0xd266afa7, SUC_lexicon_tags_47 },
    { 0x24e6afab, SUC_lexicon_tags_0 },
    { 0xfcc6afab, SUC_lexicon_tags_11 },
    { 0xdf06afa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ae6afaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe6afb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1506afb7, SUC_lexicon_tags_11 },
    { 0x5a46afb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f46afc0, SUC_lexicon_tags_2 },
    { 0x7286afc0, SUC_lexicon_tags_37 },
    { 0x7b46afc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66afcf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfd46afd1, SUC_lexicon_tags_11 },
    { 0xc1c6afd1, SUC_lexicon_tags_15 },
    { 0xc266afd3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a6afd9, SUC_lexicon_tags_17 },
    { 0xc006afd9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfba6afdc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa2c6afde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7a6afe0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a6afe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2326afe6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e6afe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da6afed, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca6aff4, SUC_lexicon_tags_7 },
    { 0x8146aff5, SUC_lexicon_tags_37 },
    { 0x6ae6aff6, SUC_lexicon_tags_11 },
    { 0x91c6aff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfec6aff9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbc6affb, SUC_lexicon_tags_506 },
    { 0x2e46affc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa546affe, SUC_lexicon_tags_45 },
    { 0x43e6affe, SUC_lexicon_tags_6 },
    { 0x4f86b000, SUC_lexicon_tags_25 },
    { 0x49a6affe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14e6b003, SUC_lexicon_tags_31 },
    { 0xeb46b003, SUC_lexicon_tags_11 },
    { 0x3646b003, SUC_lexicon_tags_28 },
    { 0xc026b006, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb266b00e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2506b010, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xe106b012, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e6b01a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a6b01f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c6b023, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26b026, SUC_lexicon_tags_25 },
    { 0xcc46b027, SUC_lexicon_tags_14 },
    { 0x8246b027, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9866b02b, SUC_lexicon_tags_33 },
    { 0xe5c6b02c, SUC_lexicon_tags_44 },
    { 0x8986b02d, SUC_lexicon_tags_37 },
    { 0x5a66b02d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e6b032, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e6b035, SUC_lexicon_tags_17 },
    { 0x8fa6b036, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6106b03a, SUC_lexicon_tags_2 },
    { 0x4566b03b, SUC_lexicon_tags_12 },
    { 0xf326b03c, SUC_lexicon_tags_25 },
    { 0xd46b03b, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6646b046, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe6b04f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e6b054, SUC_lexicon_tags_49 },
    { 0xf4c6b055, SUC_lexicon_tags_68 },
    { 0x1426b054, SUC_lexicon_tags_15 },
    { 0x2fa6b057, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e86b05e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf186b060, SUC_lexicon_tags_23 },
    { 0x5586b061, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd26b063, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7706b065, SUC_lexicon_tags_11 },
    { 0xd786b066, SUC_lexicon_tags_15 },
    { 0xe5c6b066, SUC_lexicon_tags_15 },
    { 0xf866b068, SUC_lexicon_tags_33 },
    { 0xf566b069, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5106b06b, SUC_lexicon_tags_1 },
    { 0x3146b06b, SUC_lexicon_tags_1630 },
    { 0, NULL },
    { 0xe5e6b06e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6b077, SUC_lexicon_tags_32 },
    { 0x3586b078, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4b86b07a, SUC_lexicon_tags_9 },
    { 0x9a66b07b, SUC_lexicon_tags_37 },
    { 0xf8a6b07c, SUC_lexicon_tags_47 },
    { 0x1166b07d, SUC_lexicon_tags_60 },
    { 0x2366b07c, SUC_lexicon_tags_33 },
    { 0xf26b07f, SUC_lexicon_tags_11 },
    { 0x43e6b07f, SUC_lexicon_tags_554 },
    { 0xa2e6b07f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c6b084, SUC_lexicon_tags_12 },
    { 0x2fc6b085, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea6b08f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb26b093, SUC_lexicon_tags_87 },
    { 0xbd26b094, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee26b098, SUC_lexicon_tags_8 },
    { 0xb026b099, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a86b09b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526b09f, SUC_lexicon_tags_49 },
    { 0xa686b09f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1286b0a3, SUC_lexicon_tags_37 },
    { 0x9266b0a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe726b0a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d06b0ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e66b0b2, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c6b0b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe826b0b7, SUC_lexicon_tags_47 },
    { 0x15e6b0b8, SUC_lexicon_tags_15 },
    { 0xc086b0b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26b0bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a46b0c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb166b0c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce6b0c6, SUC_lexicon_tags_37 },
    { 0xf7c6b0c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c6b0cb, SUC_lexicon_tags_8 },
    { 0xdb26b0cc, SUC_lexicon_tags_10 },
    { 0xb7a6b0cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa666b0cf, SUC_lexicon_tags_12 },
    { 0x51c6b0d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3386b0d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x666b0d6, SUC_lexicon_tags_9 },
    { 0x39c6b0d7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1066b0db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe206b0e1, SUC_lexicon_tags_7 },
    { 0x4ee6b0e1, SUC_lexicon_tags_12 },
    { 0x7a66b0e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb566b0e7, SUC_lexicon_tags_7 },
    { 0xf5c6b0e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6926b0f1, SUC_lexicon_tags_37 },
    { 0x9fe6b0f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1906b0fa, SUC_lexicon_tags_46 },
    { 0xed66b0fb, SUC_lexicon_tags_24 },
    { 0x7fa6b0fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8946b0fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de6b103, SUC_lexicon_tags_6 },
    { 0x4646b104, SUC_lexicon_tags_37 },
    { 0x2206b105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8546b107, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a26b10c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e6b10f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c46b111, SUC_lexicon_tags_2 },
    { 0x6346b112, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc6b115, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e6b120, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca6b128, SUC_lexicon_tags_17 },
    { 0xdcc6b129, SUC_lexicon_tags_7 },
    { 0x7966b128, SUC_lexicon_tags_11 },
    { 0x4dc6b12b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c6b130, SUC_lexicon_tags_23 },
    { 0xeca6b131, SUC_lexicon_tags_15 },
    { 0xebe6b130, SUC_lexicon_tags_16 },
    { 0x43e6b130, SUC_lexicon_tags_17 },
    { 0x25a6b134, SUC_lexicon_tags_37 },
    { 0xd286b132, SUC_lexicon_tags_15 },
    { 0x5a66b136, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a26b13a, SUC_lexicon_tags_25 },
    { 0x1286b13b, SUC_lexicon_tags_47 },
    { 0xdc06b13a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3926b13f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd246b141, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce26b147, SUC_lexicon_tags_3 },
    { 0x8e6b148, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ea6b14a, SUC_lexicon_tags_7 },
    { 0x39e6b14b, SUC_lexicon_tags_12 },
    { 0x2346b14c, SUC_lexicon_tags_12 },
    { 0xf906b14d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e6b152, SUC_lexicon_tags_49 },
    { 0x3cc6b153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cc6b155, SUC_lexicon_tags_12 },
    { 0x6dc6b155, SUC_lexicon_tags_15 },
    { 0xc1e6b157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb286b15a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7126b15d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c6b162, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed66b164, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb26b169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7906b16e, SUC_lexicon_tags_10 },
    { 0x7206b16f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4806b172, SUC_lexicon_tags_23 },
    { 0x6b06b172, SUC_lexicon_tags_43 },
    { 0xacc6b172, SUC_lexicon_tags_14 },
    { 0xb46b175, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c66b179, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8626b17c, SUC_lexicon_tags_25 },
    { 0xedc6b17d, SUC_lexicon_tags_19 },
    { 0xc446b17c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad46b180, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b66b182, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6926b185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc266b188, SUC_lexicon_tags_10 },
    { 0x84c6b189, SUC_lexicon_tags_39 },
    { 0x8026b188, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa066b190, SUC_lexicon_tags_14 },
    { 0x1a46b191, SUC_lexicon_tags_37 },
    { 0x1e06b192, SUC_lexicon_tags_37 },
    { 0xea86b193, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e6b19e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x486b1a0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a6b1aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2c6b1ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a06b1af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c6b1b4, SUC_lexicon_tags_32 },
    { 0x5666b1b4, SUC_lexicon_tags_85 },
    { 0x4846b1b6, SUC_lexicon_tags_19 },
    { 0x4e6b1b5, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xb486b1b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce6b1bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ea6b1bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbde6b1c1, SUC_lexicon_tags_53 },
    { 0xa06b1c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc6b1c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee6b1ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2e6b1cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf726b1ce, SUC_lexicon_tags_25 },
    { 0x64a6b1cf, SUC_lexicon_tags_49 },
    { 0xb4e6b1d0, SUC_lexicon_tags_16 },
    { 0x65e6b1d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a6b1d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee6b1dc, SUC_lexicon_tags_33 },
    { 0x4e46b1dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e6b1e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf626b1e4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4246b1e7, SUC_lexicon_tags_11 },
    { 0x2126b1e8, SUC_lexicon_tags_17 },
    { 0xc3e6b1e8, SUC_lexicon_tags_15 },
    { 0x3926b1ea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9b06b1ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c26b1ef, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb66b1f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806b1fa, SUC_lexicon_tags_47 },
    { 0xa286b1fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e6b1fe, SUC_lexicon_tags_48 },
    { 0x9a46b1ff, SUC_lexicon_tags_11 },
    { 0xe846b1ff, SUC_lexicon_tags_5 },
    { 0xbe86b201, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6926b20f, SUC_lexicon_tags_63 },
    { 0xc126b20f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7926b212, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f46b216, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6326b21b, SUC_lexicon_tags_17 },
    { 0xd46b21c, SUC_lexicon_tags_11 },
    { 0x1546b21b, SUC_lexicon_tags_2 },
    { 0x5366b21e, SUC_lexicon_tags_5 },
    { 0x7686b21f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2806b22e, SUC_lexicon_tags_2 },
    { 0x7386b22e, SUC_lexicon_tags_17 },
    { 0x6aa6b22e, SUC_lexicon_tags_32 },
    { 0xe026b231, SUC_lexicon_tags_11 },
    { 0x13e6b22f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef86b234, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc526b23c, SUC_lexicon_tags_9 },
    { 0x2526b23c, SUC_lexicon_tags_127 },
    { 0x63e6b23d, SUC_lexicon_tags_30 },
    { 0x2e06b23f, SUC_lexicon_tags_2 },
    { 0x966b23e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3326b249, SUC_lexicon_tags_11 },
    { 0x3346b24a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd286b24d, SUC_lexicon_tags_12 },
    { 0xde46b24d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c6b251, SUC_lexicon_tags_50 },
    { 0xdac6b252, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x16a6b254, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6666b25a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c6b261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d66b267, SUC_lexicon_tags_45 },
    { 0xfb46b268, SUC_lexicon_tags_17 },
    { 0x9cc6b269, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a6b26c, SUC_lexicon_tags_23 },
    { 0x5786b26d, SUC_lexicon_tags_37 },
    { 0x9806b26e, SUC_lexicon_tags_2 },
    { 0x5ec6b26f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7c6b271, SUC_lexicon_tags_0 },
    { 0x9a06b272, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c46b279, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c6b27d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdc6b27f, SUC_lexicon_tags_25 },
    { 0xe66b27f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xace6b282, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9a6b284, SUC_lexicon_tags_25 },
    { 0xdbe6b285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf26b287, SUC_lexicon_tags_28 },
    { 0x7d46b288, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc126b28c, SUC_lexicon_tags_179 },
    { 0x7e6b28d, SUC_lexicon_tags_17 },
    { 0x33c6b28e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64c6b290, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd26b293, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a6b299, SUC_lexicon_tags_47 },
    { 0x2f26b299, SUC_lexicon_tags_8 },
    { 0x79c6b299, SUC_lexicon_tags_12 },
    { 0x3d06b29c, SUC_lexicon_tags_37 },
    { 0x4846b29d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c6b2a3, SUC_lexicon_tags_12 },
    { 0x2e46b2a4, SUC_lexicon_tags_15 },
    { 0xe26b2a5, SUC_lexicon_tags_28 },
    { 0x2a66b2a6, SUC_lexicon_tags_25 },
    { 0xf6a6b2a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd146b2ab, SUC_lexicon_tags_25 },
    { 0x54a6b2ac, SUC_lexicon_tags_7 },
    { 0x6a6b2ab, SUC_lexicon_tags_15 },
    { 0xcec6b2ae, SUC_lexicon_tags_12 },
    { 0xbcc6b2af, SUC_lexicon_tags_12 },
    { 0xa906b2ae, SUC_lexicon_tags_1 },
    { 0x12e6b2ac, SUC_lexicon_tags_6 },
    { 0x9746b2b2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c6b2b6, SUC_lexicon_tags_63 },
    { 0x1586b2b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd286b2ba, SUC_lexicon_tags_17 },
    { 0xc526b2ba, SUC_lexicon_tags_0 },
    { 0x14e6b2ba, SUC_lexicon_tags_25 },
    { 0x3466b2bd, SUC_lexicon_tags_27 },
    { 0x64a6b2be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb826b2c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6306b2c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb746b2c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2066b2c8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x5066b2cb, SUC_lexicon_tags_13 },
    { 0x8f06b2cc, SUC_lexicon_tags_11 },
    { 0xbf06b2cd, SUC_lexicon_tags_63 },
    { 0xf3c6b2ce, SUC_lexicon_tags_15 },
    { 0x5486b2cb, SUC_lexicon_tags_43 },
    { 0x89e6b2d0, SUC_lexicon_tags_12 },
    { 0x4566b2d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8586b2d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf406b2da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe8c6b2dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf846b2de, SUC_lexicon_tags_79 },
    { 0x1e46b2df, SUC_lexicon_tags_1 },
    { 0x8546b2e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea6b2e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e46b2e8, SUC_lexicon_tags_49 },
    { 0x17e6b2e9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xde66b2eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x386b2ed, SUC_lexicon_tags_6 },
    { 0x3046b2ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c6b2f3, SUC_lexicon_tags_917 },
    { 0, NULL },
    { 0x8566b2f5, SUC_lexicon_tags_45 },
    { 0x7e26b2f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ee6b2f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5666b2fa, SUC_lexicon_tags_1 },
    { 0xa286b2fb, SUC_lexicon_tags_15 },
    { 0x85e6b2fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6686b2fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8606b300, SUC_lexicon_tags_2 },
    { 0xe706b300, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90e6b303, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd606b30b, SUC_lexicon_tags_11 },
    { 0xe0c6b30c, SUC_lexicon_tags_16 },
    { 0x18c6b30d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc6b310, SUC_lexicon_tags_27 },
    { 0xdfc6b311, SUC_lexicon_tags_57 },
    { 0x8086b312, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7346b318, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe26b31f, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x88a6b321, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a6b324, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d66b328, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa46b32a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3066b32f, SUC_lexicon_tags_49 },
    { 0xa386b330, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2146b334, SUC_lexicon_tags_17 },
    { 0x4ba6b334, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa6b33d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a6b344, SUC_lexicon_tags_12 },
    { 0x5126b345, SUC_lexicon_tags_17 },
    { 0xcd86b346, SUC_lexicon_tags_23 },
    { 0x6966b344, SUC_lexicon_tags_11 },
    { 0x2bc6b348, SUC_lexicon_tags_1 },
    { 0x2666b349, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2826b34c, SUC_lexicon_tags_1 },
    { 0x3b46b34d, SUC_lexicon_tags_15 },
    { 0xb346b34e, SUC_lexicon_tags_1 },
    { 0xea06b34f, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0x5486b351, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a6b359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf46b35b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a6b35e, SUC_lexicon_tags_25 },
    { 0xab46b35f, SUC_lexicon_tags_55 },
    { 0x3c06b360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc566b362, SUC_lexicon_tags_14 },
    { 0x5fe6b362, SUC_lexicon_tags_0 },
    { 0xc4a6b364, SUC_lexicon_tags_17 },
    { 0x1ac6b364, SUC_lexicon_tags_2 },
    { 0x766b366, SUC_lexicon_tags_34 },
    { 0x7c46b367, SUC_lexicon_tags_37 },
    { 0x5866b368, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b06b36a, SUC_lexicon_tags_30 },
    { 0x7e6b36b, SUC_lexicon_tags_12 },
    { 0xc1a6b36a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14e6b36e, SUC_lexicon_tags_25 },
    { 0xb2a6b36f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6986b377, SUC_lexicon_tags_15 },
    { 0x8766b378, SUC_lexicon_tags_2 },
    { 0x8b66b378, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf086b37d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6ee6b37f, SUC_lexicon_tags_1 },
    { 0xe286b37f, SUC_lexicon_tags_3 },
    { 0xfc66b37f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x21e6b383, SUC_lexicon_tags_25 },
    { 0x6f66b383, SUC_lexicon_tags_8 },
    { 0xc586b385, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6886b387, SUC_lexicon_tags_151 },
    { 0x4d66b388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46b38b, SUC_lexicon_tags_43 },
    { 0x8246b38b, SUC_lexicon_tags_9 },
    { 0x40c6b38b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcc46b38f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43c6b391, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc446b393, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5046b395, SUC_lexicon_tags_2 },
    { 0xa4a6b396, SUC_lexicon_tags_27 },
    { 0x4c66b397, SUC_lexicon_tags_1 },
    { 0xbce6b395, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe666b39d, SUC_lexicon_tags_17 },
    { 0x39e6b39e, SUC_lexicon_tags_32 },
    { 0x67c6b39e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf86b3a1, SUC_lexicon_tags_15 },
    { 0x6e86b3a1, SUC_lexicon_tags_2 },
    { 0xd126b3a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d06b3a5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3506b3b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7766b3b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1746b3b6, SUC_lexicon_tags_37 },
    { 0x33c6b3b6, SUC_lexicon_tags_1 },
    { 0x8f46b3b7, SUC_lexicon_tags_27 },
    { 0x54a6b3b9, SUC_lexicon_tags_7 },
    { 0x3fa6b3ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1486b3c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8426b3c4, SUC_lexicon_tags_47 },
    { 0x2c26b3c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d06b3ca, SUC_lexicon_tags_65 },
    { 0xb0e6b3cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8a26b3cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81e6b3cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe866b3d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc726b3d4, SUC_lexicon_tags_37 },
    { 0xf506b3d5, SUC_lexicon_tags_9 },
    { 0x86b3d6, SUC_lexicon_tags_21 },
    { 0x506b3d6, SUC_lexicon_tags_17 },
    { 0x4666b3d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1086b3de, SUC_lexicon_tags_17 },
    { 0xb646b3df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb06b3e1, SUC_lexicon_tags_492 },
    { 0, NULL },
    { 0xcb66b3e3, SUC_lexicon_tags_15 },
    { 0x98c6b3e4, SUC_lexicon_tags_33 },
    { 0xebe6b3e5, SUC_lexicon_tags_38 },
    { 0xdd66b3e6, SUC_lexicon_tags_12 },
    { 0x5286b3e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8446b3ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdc6b3ef, SUC_lexicon_tags_25 },
    { 0x7746b3f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe286b3f3, SUC_lexicon_tags_19 },
    { 0x6c6b3f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6426b3f6, SUC_lexicon_tags_63 },
    { 0xfa66b3f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f06b3f9, SUC_lexicon_tags_11 },
    { 0x2f66b3f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4726b3fc, SUC_lexicon_tags_37 },
    { 0x3d86b3fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e6b400, SUC_lexicon_tags_4 },
    { 0xac6b401, SUC_lexicon_tags_11 },
    { 0xef46b401, SUC_lexicon_tags_9 },
    { 0x2626b403, SUC_lexicon_tags_19 },
    { 0xd6e6b403, SUC_lexicon_tags_17 },
    { 0x5986b400, SUC_lexicon_tags_15 },
    { 0x7086b406, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a6b40a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3326b40f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c6b413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2566b418, SUC_lexicon_tags_29 },
    { 0x6546b419, SUC_lexicon_tags_23 },
    { 0x9766b418, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae6b41d, SUC_lexicon_tags_15 },
    { 0xaf46b41e, SUC_lexicon_tags_87 },
    { 0x82a6b41f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xda06b421, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa6b424, SUC_lexicon_tags_17 },
    { 0x4066b424, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd346b427, SUC_lexicon_tags_37 },
    { 0xe906b428, SUC_lexicon_tags_68 },
    { 0x8b66b429, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xdd26b42b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe66b42e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c6b431, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746b436, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e46b43d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4e86b43f, SUC_lexicon_tags_8 },
    { 0x5446b440, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x42c6b442, SUC_lexicon_tags_61 },
    { 0x726b443, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7886b445, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6886b44b, SUC_lexicon_tags_25 },
    { 0x3346b44c, SUC_lexicon_tags_12 },
    { 0x55c6b44d, SUC_lexicon_tags_14 },
    { 0xc366b44d, SUC_lexicon_tags_23 },
    { 0xa286b44e, SUC_lexicon_tags_15 },
    { 0xc986b44f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c46b452, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xef46b454, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x866b456, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb466b45a, SUC_lexicon_tags_15 },
    { 0x7e26b45a, SUC_lexicon_tags_38 },
    { 0x8c06b45c, SUC_lexicon_tags_11 },
    { 0x586b45d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9fa6b45f, SUC_lexicon_tags_0 },
    { 0x5626b45f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e6b464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa246b466, SUC_lexicon_tags_17 },
    { 0x5de6b467, SUC_lexicon_tags_43 },
    { 0xd4a6b466, SUC_lexicon_tags_11 },
    { 0x2226b469, SUC_lexicon_tags_10 },
    { 0xa526b46a, SUC_lexicon_tags_761 },
    { 0x1c86b46b, SUC_lexicon_tags_33 },
    { 0x7706b469, SUC_lexicon_tags_27 },
    { 0xcc06b46d, SUC_lexicon_tags_33 },
    { 0x1c6b46d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x51c6b470, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12e6b472, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xe166b475, SUC_lexicon_tags_64 },
    { 0xbfe6b476, SUC_lexicon_tags_10 },
    { 0x1e66b476, SUC_lexicon_tags_9 },
    { 0x2ae6b478, SUC_lexicon_tags_29 },
    { 0x4c6b479, SUC_lexicon_tags_25 },
    { 0x9626b479, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba06b481, SUC_lexicon_tags_14 },
    { 0xb446b482, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda6b485, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x53a6b487, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9986b489, SUC_lexicon_tags_17 },
    { 0xe466b48a, SUC_lexicon_tags_25 },
    { 0xe206b48b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4826b48e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85a6b490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a6b496, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f26b499, SUC_lexicon_tags_30 },
    { 0x94e6b49a, SUC_lexicon_tags_27 },
    { 0x17e6b49b, SUC_lexicon_tags_17 },
    { 0xd4a6b49c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a6b4a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e6b4a5, SUC_lexicon_tags_21 },
    { 0xe2c6b4a6, SUC_lexicon_tags_14 },
    { 0x6ee6b4a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f06b4aa, SUC_lexicon_tags_49 },
    { 0xbf06b4ab, SUC_lexicon_tags_9 },
    { 0xba6b4ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3a6b4ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67a6b4b0, SUC_lexicon_tags_124 },
    { 0xce26b4b1, SUC_lexicon_tags_47 },
    { 0xabe6b4b1, SUC_lexicon_tags_37 },
    { 0xba86b4b3, SUC_lexicon_tags_7 },
    { 0xb6a6b4b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3666b4b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7066b4b9, SUC_lexicon_tags_23 },
    { 0x66a6b4b9, SUC_lexicon_tags_15 },
    { 0x7246b4bb, SUC_lexicon_tags_79 },
    { 0xb5e6b4b9, SUC_lexicon_tags_2 },
    { 0x6f26b4bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b86b4c0, SUC_lexicon_tags_14 },
    { 0xb0c6b4c1, SUC_lexicon_tags_31 },
    { 0xdac6b4c0, SUC_lexicon_tags_30 },
    { 0xa846b4c3, SUC_lexicon_tags_2 },
    { 0x44e6b4c4, SUC_lexicon_tags_2 },
    { 0xab06b4c4, SUC_lexicon_tags_8 },
    { 0x2526b4c3, SUC_lexicon_tags_49 },
    { 0x3e26b4c5, SUC_lexicon_tags_15 },
    { 0x566b4c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8166b4ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb086b4d0, SUC_lexicon_tags_2 },
    { 0xb546b4d1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xed06b4d4, SUC_lexicon_tags_8 },
    { 0x6686b4d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xca66b4d7, SUC_lexicon_tags_25 },
    { 0x22c6b4d8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xdd26b4da, SUC_lexicon_tags_26 },
    { 0xc366b4db, SUC_lexicon_tags_11 },
    { 0xbe6b4dc, SUC_lexicon_tags_12 },
    { 0xa5e6b4dd, SUC_lexicon_tags_19 },
    { 0x81e6b4dc, SUC_lexicon_tags_14 },
    { 0xda46b4de, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfe66b4e1, SUC_lexicon_tags_30 },
    { 0xd966b4e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec6b4e7, SUC_lexicon_tags_104 },
    { 0x3b46b4e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb86b4ec, SUC_lexicon_tags_60 },
    { 0x99e6b4ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba6b4f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c66b4fd, SUC_lexicon_tags_23 },
    { 0xe746b4fe, SUC_lexicon_tags_14 },
    { 0x7646b4fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9aa6b501, SUC_lexicon_tags_19 },
    { 0x7de6b502, SUC_lexicon_tags_17 },
    { 0xe206b503, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0a6b505, SUC_lexicon_tags_11 },
    { 0x9dc6b505, SUC_lexicon_tags_2 },
    { 0x4026b507, SUC_lexicon_tags_46 },
    { 0x5b06b508, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1026b50a, SUC_lexicon_tags_30 },
    { 0x3a86b50b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca6b511, SUC_lexicon_tags_31 },
    { 0x5246b512, SUC_lexicon_tags_63 },
    { 0x9106b512, SUC_lexicon_tags_39 },
    { 0xaf66b512, SUC_lexicon_tags_33 },
    { 0xc826b515, SUC_lexicon_tags_49 },
    { 0xe2c6b516, SUC_lexicon_tags_2 },
    { 0xf66b517, SUC_lexicon_tags_15 },
    { 0x8a86b518, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x266b51a, SUC_lexicon_tags_62 },
    { 0xeec6b51b, SUC_lexicon_tags_17 },
    { 0x87a6b51b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6b52a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a66b52e, SUC_lexicon_tags_15 },
    { 0xcee6b52f, SUC_lexicon_tags_19 },
    { 0xcc86b52f, SUC_lexicon_tags_37 },
    { 0x3c06b52f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a6b535, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e06b537, SUC_lexicon_tags_45 },
    { 0xd346b538, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe866b53f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9066b541, SUC_lexicon_tags_11 },
    { 0xb2a6b541, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f06b544, SUC_lexicon_tags_1 },
    { 0x58a6b545, SUC_lexicon_tags_84 },
    { 0x73e6b545, SUC_lexicon_tags_37 },
    { 0x8e06b544, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa6b54a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe06b54e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfe46b550, SUC_lexicon_tags_8 },
    { 0xd806b551, SUC_lexicon_tags_17 },
    { 0x99c6b552, SUC_lexicon_tags_49 },
    { 0xbe26b553, SUC_lexicon_tags_17 },
    { 0x8846b553, SUC_lexicon_tags_23 },
    { 0x1ae6b551, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe206b55a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe6c6b55c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbc26b55e, SUC_lexicon_tags_33 },
    { 0xd46b55f, SUC_lexicon_tags_37 },
    { 0x3ac6b560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3826b565, SUC_lexicon_tags_37 },
    { 0x6446b566, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6106b568, SUC_lexicon_tags_37 },
    { 0xcd26b569, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e6b56c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b572, SUC_lexicon_tags_26 },
    { 0x5046b572, SUC_lexicon_tags_2 },
    { 0x2026b574, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd146b57a, SUC_lexicon_tags_2 },
    { 0xc306b57a, SUC_lexicon_tags_15 },
    { 0xf266b57a, SUC_lexicon_tags_60 },
    { 0xa3c6b57d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd206b582, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c6b585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68a6b587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8746b58a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be6b58d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c6b590, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x666b593, SUC_lexicon_tags_28 },
    { 0xd126b593, SUC_lexicon_tags_61 },
    { 0xb246b594, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6266b597, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac6b59b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c46b59e, SUC_lexicon_tags_63 },
    { 0xdae6b59f, SUC_lexicon_tags_12 },
    { 0x3026b59f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e06b5a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c66b5a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4546b5ab, SUC_lexicon_tags_25 },
    { 0x8446b5ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf366b5b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a6b5b5, SUC_lexicon_tags_29 },
    { 0x3766b5b5, SUC_lexicon_tags_30 },
    { 0xea86b5b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a06b5b9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd226b5bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8226b5be, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x286b5c0, SUC_lexicon_tags_17 },
    { 0xddc6b5c1, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xd886b5c3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf686b5c5, SUC_lexicon_tags_17 },
    { 0xae06b5c6, SUC_lexicon_tags_160 },
    { 0xd826b5c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc466b5cd, SUC_lexicon_tags_15 },
    { 0x6206b5ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf166b5d0, SUC_lexicon_tags_4 },
    { 0xe026b5d0, SUC_lexicon_tags_1 },
    { 0x41e6b5d2, SUC_lexicon_tags_17 },
    { 0xf946b5d3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee6b5dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7546b5e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b66b5e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9006b5e6, SUC_lexicon_tags_29 },
    { 0x50a6b5e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4366b5e9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e6b5ec, SUC_lexicon_tags_27 },
    { 0x6e66b5ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8e6b5ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c6b5f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2806b5f5, SUC_lexicon_tags_25 },
    { 0xe206b5f6, SUC_lexicon_tags_17 },
    { 0x26c6b5f5, SUC_lexicon_tags_9 },
    { 0x7ba6b5f5, SUC_lexicon_tags_2 },
    { 0x8ec6b5f9, SUC_lexicon_tags_27 },
    { 0x9ba6b5fa, SUC_lexicon_tags_30 },
    { 0x1706b5fb, SUC_lexicon_tags_57 },
    { 0xa146b5fb, SUC_lexicon_tags_174 },
    { 0xda46b5fa, SUC_lexicon_tags_46 },
    { 0xcb06b5fe, SUC_lexicon_tags_7 },
    { 0xf266b5ff, SUC_lexicon_tags_45 },
    { 0x9746b5f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77a6b602, SUC_lexicon_tags_63 },
    { 0x2626b603, SUC_lexicon_tags_11 },
    { 0xa0e6b604, SUC_lexicon_tags_8 },
    { 0xce86b603, SUC_lexicon_tags_11 },
    { 0x2306b603, SUC_lexicon_tags_28 },
    { 0xb746b607, SUC_lexicon_tags_25 },
    { 0x4fc6b608, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbc66b60a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc786b60d, SUC_lexicon_tags_63 },
    { 0xafe6b60e, SUC_lexicon_tags_37 },
    { 0x1146b60e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3346b611, SUC_lexicon_tags_21 },
    { 0xcce6b612, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3426b614, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf446b61a, SUC_lexicon_tags_11 },
    { 0x2c86b61a, SUC_lexicon_tags_17 },
    { 0xc9e6b61a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e6b61f, SUC_lexicon_tags_65 },
    { 0xe7e6b620, SUC_lexicon_tags_47 },
    { 0x1286b621, SUC_lexicon_tags_25 },
    { 0x4a86b620, SUC_lexicon_tags_11 },
    { 0xdbe6b620, SUC_lexicon_tags_7 },
    { 0x7626b623, SUC_lexicon_tags_11 },
    { 0x4c06b625, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9966b628, SUC_lexicon_tags_0 },
    { 0xb2a6b628, SUC_lexicon_tags_7 },
    { 0xd906b62a, SUC_lexicon_tags_44 },
    { 0x9a66b62b, SUC_lexicon_tags_49 },
    { 0xb46b62a, SUC_lexicon_tags_2 },
    { 0xe646b62b, SUC_lexicon_tags_2 },
    { 0x62e6b62e, SUC_lexicon_tags_9 },
    { 0xef46b62f, SUC_lexicon_tags_1 },
    { 0x5726b630, SUC_lexicon_tags_63 },
    { 0x6c06b631, SUC_lexicon_tags_11 },
    { 0x5ec6b632, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe006b635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeee6b637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6b63c, SUC_lexicon_tags_0 },
    { 0xdaa6b63d, SUC_lexicon_tags_8 },
    { 0x7786b63d, SUC_lexicon_tags_7 },
    { 0xa326b63d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd8a6b641, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba26b645, SUC_lexicon_tags_37 },
    { 0x3bc6b646, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a46b64b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f46b64e, SUC_lexicon_tags_12 },
    { 0x75a6b64e, SUC_lexicon_tags_1 },
    { 0x7866b64f, SUC_lexicon_tags_33 },
    { 0xf66b64f, SUC_lexicon_tags_11 },
    { 0xc406b652, SUC_lexicon_tags_19 },
    { 0xa666b652, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd866b657, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc6b65b, SUC_lexicon_tags_12 },
    { 0x6806b65c, SUC_lexicon_tags_37 },
    { 0x81c6b65c, SUC_lexicon_tags_6 },
    { 0xe4e6b65b, SUC_lexicon_tags_234 },
    { 0xd5e6b65e, SUC_lexicon_tags_35 },
    { 0x4686b660, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde6b664, SUC_lexicon_tags_37 },
    { 0xc3c6b665, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f66b669, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e6b66c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcc6b66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa386b673, SUC_lexicon_tags_9 },
    { 0x4866b673, SUC_lexicon_tags_23 },
    { 0xbee6b675, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2a6b677, SUC_lexicon_tags_47 },
    { 0x7486b677, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f86b67c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb806b681, SUC_lexicon_tags_63 },
    { 0xdc46b682, SUC_lexicon_tags_14 },
    { 0x22e6b681, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c86b68e, SUC_lexicon_tags_23 },
    { 0xcc46b68f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c86b691, SUC_lexicon_tags_7 },
    { 0x7b86b691, SUC_lexicon_tags_12 },
    { 0x5086b693, SUC_lexicon_tags_48 },
    { 0x59e6b692, SUC_lexicon_tags_49 },
    { 0x6f26b692, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5526b69b, SUC_lexicon_tags_25 },
    { 0x566b69c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc726b69e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7126b6a1, SUC_lexicon_tags_25 },
    { 0x8126b6a1, SUC_lexicon_tags_35 },
    { 0x2846b6a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb46b6a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8186b6aa, SUC_lexicon_tags_38 },
    { 0xbfa6b6aa, SUC_lexicon_tags_2 },
    { 0x3186b6ac, SUC_lexicon_tags_29 },
    { 0x5cc6b6ac, SUC_lexicon_tags_15 },
    { 0xa766b6ac, SUC_lexicon_tags_4 },
    { 0x88e6b6af, SUC_lexicon_tags_60 },
    { 0x2006b6af, SUC_lexicon_tags_17 },
    { 0x25c6b6b0, SUC_lexicon_tags_16 },
    { 0x30c6b6b2, SUC_lexicon_tags_48 },
    { 0x6626b6b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2186b6b8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x6446b6bb, SUC_lexicon_tags_7 },
    { 0xdec6b6bc, SUC_lexicon_tags_15 },
    { 0x66a6b6bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc6b6c3, SUC_lexicon_tags_13 },
    { 0x7426b6c4, SUC_lexicon_tags_28 },
    { 0x1c6b6c4, SUC_lexicon_tags_15 },
    { 0x3de6b6c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6226b6c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b46b6cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c6b6d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e6b6d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9246b6df, SUC_lexicon_tags_395 },
    { 0, NULL },
    { 0xc306b6e1, SUC_lexicon_tags_3 },
    { 0xdb66b6e2, SUC_lexicon_tags_4 },
    { 0x90a6b6e3, SUC_lexicon_tags_25 },
    { 0x4526b6e4, SUC_lexicon_tags_21 },
    { 0xa9a6b6e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbfe6b6e7, SUC_lexicon_tags_87 },
    { 0x2fc6b6e8, SUC_lexicon_tags_11 },
    { 0xbe46b6e9, SUC_lexicon_tags_9 },
    { 0x88a6b6ea, SUC_lexicon_tags_27 },
    { 0x266b6eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c26b6f1, SUC_lexicon_tags_17 },
    { 0x29c6b6f2, SUC_lexicon_tags_23 },
    { 0x7ee6b6f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc6b6f6, SUC_lexicon_tags_23 },
    { 0xd386b6f7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a6b6fc, SUC_lexicon_tags_7 },
    { 0xfa06b6fd, SUC_lexicon_tags_15 },
    { 0xdfe6b6fe, SUC_lexicon_tags_12 },
    { 0xfc6b6fd, SUC_lexicon_tags_79 },
    { 0xdf66b700, SUC_lexicon_tags_25 },
    { 0x59a6b701, SUC_lexicon_tags_32 },
    { 0x9b66b6fc, SUC_lexicon_tags_17 },
    { 0xcb46b701, SUC_lexicon_tags_11 },
    { 0x2d26b704, SUC_lexicon_tags_40 },
    { 0xa206b705, SUC_lexicon_tags_47 },
    { 0xc006b706, SUC_lexicon_tags_2 },
    { 0xbc6b703, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd66b709, SUC_lexicon_tags_29 },
    { 0x84c6b709, SUC_lexicon_tags_17 },
    { 0x1306b709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4486b713, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5106b716, SUC_lexicon_tags_2 },
    { 0x2186b716, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa46b71a, SUC_lexicon_tags_43 },
    { 0xdd66b71b, SUC_lexicon_tags_12 },
    { 0x87e6b71c, SUC_lexicon_tags_37 },
    { 0xb946b71b, SUC_lexicon_tags_0 },
    { 0xf66b71d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4986b725, SUC_lexicon_tags_12 },
    { 0xcd26b725, SUC_lexicon_tags_38 },
    { 0xd246b727, SUC_lexicon_tags_2 },
    { 0xd206b728, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2966b72e, SUC_lexicon_tags_68 },
    { 0x3586b72f, SUC_lexicon_tags_11 },
    { 0xbbc6b72f, SUC_lexicon_tags_37 },
    { 0x84a6b731, SUC_lexicon_tags_17 },
    { 0xeb06b731, SUC_lexicon_tags_12 },
    { 0x3b66b732, SUC_lexicon_tags_23 },
    { 0x3966b733, SUC_lexicon_tags_4 },
    { 0x74a6b735, SUC_lexicon_tags_10 },
    { 0x3066b736, SUC_lexicon_tags_11 },
    { 0xabc6b737, SUC_lexicon_tags_4 },
    { 0x3f86b737, SUC_lexicon_tags_1 },
    { 0x6a66b736, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6866b740, SUC_lexicon_tags_43 },
    { 0x5826b740, SUC_lexicon_tags_37 },
    { 0xd26b742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc6b749, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c6b750, SUC_lexicon_tags_29 },
    { 0x26e6b751, SUC_lexicon_tags_25 },
    { 0x6fa6b752, SUC_lexicon_tags_7 },
    { 0x4c26b753, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7686b755, SUC_lexicon_tags_14 },
    { 0x1be6b756, SUC_lexicon_tags_10 },
    { 0x9e46b755, SUC_lexicon_tags_33 },
    { 0xb666b757, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2306b75d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b06b760, SUC_lexicon_tags_65 },
    { 0x6cc6b760, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb846b763, SUC_lexicon_tags_1 },
    { 0xdb46b764, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbe6b766, SUC_lexicon_tags_17 },
    { 0x3726b767, SUC_lexicon_tags_68 },
    { 0x36a6b767, SUC_lexicon_tags_1 },
    { 0xf0a6b769, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee6b76c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9526b772, SUC_lexicon_tags_2 },
    { 0x4cc6b773, SUC_lexicon_tags_60 },
    { 0x306b774, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9ea6b776, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab46b778, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4786b77b, SUC_lexicon_tags_2 },
    { 0x1c26b77b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8486b77e, SUC_lexicon_tags_23 },
    { 0x7de6b77f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea6b784, SUC_lexicon_tags_71 },
    { 0xc586b785, SUC_lexicon_tags_25 },
    { 0x82e6b786, SUC_lexicon_tags_32 },
    { 0x12c6b785, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7866b78c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc26b790, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5606b792, SUC_lexicon_tags_17 },
    { 0x6d26b792, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b46b799, SUC_lexicon_tags_19 },
    { 0x5726b79a, SUC_lexicon_tags_12 },
    { 0x206b79a, SUC_lexicon_tags_1 },
    { 0x89c6b79b, SUC_lexicon_tags_65 },
    { 0x33e6b79d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa826b7a0, SUC_lexicon_tags_57 },
    { 0xa3e6b7a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a6b7aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd26b7ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c6b7b4, SUC_lexicon_tags_33 },
    { 0xdb86b7b5, SUC_lexicon_tags_11 },
    { 0x4c66b7b4, SUC_lexicon_tags_14 },
    { 0xc26b7b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb06b7bd, SUC_lexicon_tags_16 },
    { 0xd5e6b7bd, SUC_lexicon_tags_17 },
    { 0x62e6b7bf, SUC_lexicon_tags_60 },
    { 0xf686b7bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3086b7c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c26b7c7, SUC_lexicon_tags_11 },
    { 0x41a6b7c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3446b7ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe26b7cd, SUC_lexicon_tags_14 },
    { 0xec86b7ce, SUC_lexicon_tags_37 },
    { 0x3826b7ce, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf606b7d1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9186b7d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9166b7d6, SUC_lexicon_tags_0 },
    { 0xd1e6b7d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc6b7db, SUC_lexicon_tags_27 },
    { 0x4626b7dc, SUC_lexicon_tags_27 },
    { 0x7186b7dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb066b7e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e6b7e5, SUC_lexicon_tags_9 },
    { 0x8046b7e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf486b7eb, SUC_lexicon_tags_45 },
    { 0xc46b7eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6046b7ef, SUC_lexicon_tags_98 },
    { 0x8226b7f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc326b7f3, SUC_lexicon_tags_49 },
    { 0x3546b7f3, SUC_lexicon_tags_7 },
    { 0xcfa6b7f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a6b7fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d46b7ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe686b801, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e6b804, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xee86b807, SUC_lexicon_tags_29 },
    { 0xb5e6b808, SUC_lexicon_tags_23 },
    { 0x8206b807, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646b812, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b26b815, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9246b817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa786b819, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e6b81e, SUC_lexicon_tags_12 },
    { 0x8ec6b81f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf746b822, SUC_lexicon_tags_27 },
    { 0xaa46b823, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a06b826, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4946b82a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89c6b82c, SUC_lexicon_tags_0 },
    { 0x34c6b82d, SUC_lexicon_tags_45 },
    { 0xb6a6b82d, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6b832, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee6b835, SUC_lexicon_tags_45 },
    { 0x2866b836, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc306b839, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c6b83d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2906b83f, SUC_lexicon_tags_16 },
    { 0x4fe6b83f, SUC_lexicon_tags_231 },
    { 0xb366b840, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1586b846, SUC_lexicon_tags_37 },
    { 0x5c86b847, SUC_lexicon_tags_1 },
    { 0x59c6b846, SUC_lexicon_tags_4 },
    { 0x78c6b849, SUC_lexicon_tags_2 },
    { 0x97c6b84a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4926b84e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd846b851, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0x4306b854, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd606b856, SUC_lexicon_tags_12 },
    { 0x5386b857, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4c6b859, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7ba6b85b, SUC_lexicon_tags_25 },
    { 0x3a26b85c, SUC_lexicon_tags_1 },
    { 0x9a86b85c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e26b861, SUC_lexicon_tags_43 },
    { 0x6ee6b862, SUC_lexicon_tags_47 },
    { 0x5826b863, SUC_lexicon_tags_4 },
    { 0xaba6b864, SUC_lexicon_tags_7 },
    { 0xa0e6b864, SUC_lexicon_tags_79 },
    { 0xe86b866, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e6b86b, SUC_lexicon_tags_71 },
    { 0xee6b86c, SUC_lexicon_tags_47 },
    { 0xb526b86d, SUC_lexicon_tags_68 },
    { 0xf266b86e, SUC_lexicon_tags_25 },
    { 0x3046b86b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e6b872, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c86b875, SUC_lexicon_tags_49 },
    { 0xc046b876, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f26b87b, SUC_lexicon_tags_60 },
    { 0xbfc6b87c, SUC_lexicon_tags_37 },
    { 0xa626b87d, SUC_lexicon_tags_10 },
    { 0x41e6b87d, SUC_lexicon_tags_23 },
    { 0xf286b87f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7726b881, SUC_lexicon_tags_0 },
    { 0x6e66b882, SUC_lexicon_tags_65 },
    { 0x98a6b881, SUC_lexicon_tags_156 },
    { 0xfda6b884, SUC_lexicon_tags_9 },
    { 0x6166b885, SUC_lexicon_tags_23 },
    { 0x5b86b883, SUC_lexicon_tags_12 },
    { 0x1d46b886, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a6b88a, SUC_lexicon_tags_33 },
    { 0xef66b88a, SUC_lexicon_tags_60 },
    { 0xaa06b88c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586b891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0e6b893, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaa6b895, SUC_lexicon_tags_25 },
    { 0xdbc6b896, SUC_lexicon_tags_1 },
    { 0xa9e6b897, SUC_lexicon_tags_7 },
    { 0x2966b897, SUC_lexicon_tags_43 },
    { 0x7726b899, SUC_lexicon_tags_37 },
    { 0x7306b896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c6b89e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1626b8a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9146b8a3, SUC_lexicon_tags_27 },
    { 0x2626b8a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2166b8a8, SUC_lexicon_tags_22 },
    { 0xa966b8a9, SUC_lexicon_tags_74 },
    { 0x86a6b8aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe06b8ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8366b8af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e6b8b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da6b8bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd246b8bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c6b8c0, SUC_lexicon_tags_6 },
    { 0xf66b8c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7046b8c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x88c6b8ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe426b8ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4306b8d1, SUC_lexicon_tags_18 },
    { 0xdb46b8d1, SUC_lexicon_tags_65 },
    { 0xbc86b8d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a6b8d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a6b8da, SUC_lexicon_tags_9 },
    { 0x8f06b8da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6b8e4, SUC_lexicon_tags_23 },
    { 0xeb86b8e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2226b8f8, SUC_lexicon_tags_47 },
    { 0x8e46b8f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e6b903, SUC_lexicon_tags_17 },
    { 0xfca6b903, SUC_lexicon_tags_31 },
    { 0xd9c6b905, SUC_lexicon_tags_27 },
    { 0x12e6b903, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf146b909, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc6b90c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26b913, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d06b918, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2646b924, SUC_lexicon_tags_2 },
    { 0x2d46b925, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5166b929, SUC_lexicon_tags_11 },
    { 0xdd86b92a, SUC_lexicon_tags_11 },
    { 0x4d66b92b, SUC_lexicon_tags_19 },
    { 0x84a6b92c, SUC_lexicon_tags_2 },
    { 0x906b92c, SUC_lexicon_tags_27 },
    { 0xd486b92a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd66b931, SUC_lexicon_tags_45 },
    { 0x8586b931, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e6b935, SUC_lexicon_tags_14 },
    { 0x8c6b936, SUC_lexicon_tags_48 },
    { 0x7326b937, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xca86b93a, SUC_lexicon_tags_32 },
    { 0xfa46b93b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9e66b93d, SUC_lexicon_tags_2 },
    { 0x3926b93d, SUC_lexicon_tags_19 },
    { 0x59c6b93f, SUC_lexicon_tags_7 },
    { 0xc086b940, SUC_lexicon_tags_23 },
    { 0xc546b93e, SUC_lexicon_tags_45 },
    { 0x4f26b942, SUC_lexicon_tags_23 },
    { 0x6106b941, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c6b948, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e6b94f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2566b952, SUC_lexicon_tags_23 },
    { 0xe306b952, SUC_lexicon_tags_37 },
    { 0x8386b953, SUC_lexicon_tags_45 },
    { 0xa6c6b952, SUC_lexicon_tags_9 },
    { 0x3566b956, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a6b959, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4e06b95b, SUC_lexicon_tags_1 },
    { 0x3126b95b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26c6b95e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa866b960, SUC_lexicon_tags_1 },
    { 0xe226b961, SUC_lexicon_tags_37 },
    { 0x6f86b962, SUC_lexicon_tags_20 },
    { 0x9606b963, SUC_lexicon_tags_12 },
    { 0x3586b964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe246b967, SUC_lexicon_tags_1 },
    { 0xeb66b968, SUC_lexicon_tags_47 },
    { 0x3c6b969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd426b96c, SUC_lexicon_tags_15 },
    { 0x5886b96c, SUC_lexicon_tags_45 },
    { 0xe2e6b96d, SUC_lexicon_tags_13 },
    { 0x7746b96d, SUC_lexicon_tags_8 },
    { 0xcec6b96f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc186b974, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x266b977, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd486b97a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa266b97c, SUC_lexicon_tags_17 },
    { 0xf366b97d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c6b981, SUC_lexicon_tags_17 },
    { 0xeac6b981, SUC_lexicon_tags_12 },
    { 0x20e6b981, SUC_lexicon_tags_17 },
    { 0xab66b984, SUC_lexicon_tags_37 },
    { 0x33a6b984, SUC_lexicon_tags_6 },
    { 0x9646b985, SUC_lexicon_tags_11 },
    { 0x2c26b987, SUC_lexicon_tags_77 },
    { 0x7ae6b988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc566b98f, SUC_lexicon_tags_38 },
    { 0xaba6b990, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9506b992, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a6b995, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf526b997, SUC_lexicon_tags_136 },
    { 0xa546b998, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5086b99a, SUC_lexicon_tags_17 },
    { 0x46e6b99b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4106b9a4, SUC_lexicon_tags_10 },
    { 0x9e86b9a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8146b9af, SUC_lexicon_tags_7 },
    { 0xd466b9b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5786b9b8, SUC_lexicon_tags_28 },
    { 0x1d26b9b9, SUC_lexicon_tags_61 },
    { 0x7606b9b9, SUC_lexicon_tags_5 },
    { 0x7e86b9ba, SUC_lexicon_tags_37 },
    { 0x7066b9bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9886b9c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5166b9c6, SUC_lexicon_tags_15 },
    { 0xc946b9c7, SUC_lexicon_tags_79 },
    { 0x4f06b9c7, SUC_lexicon_tags_15 },
    { 0x44a6b9c9, SUC_lexicon_tags_23 },
    { 0x9166b9c8, SUC_lexicon_tags_27 },
    { 0x2686b9cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c86b9cd, SUC_lexicon_tags_14 },
    { 0xe726b9cd, SUC_lexicon_tags_6 },
    { 0x68c6b9cf, SUC_lexicon_tags_1 },
    { 0x4aa6b9cf, SUC_lexicon_tags_17 },
    { 0xf886b9d0, SUC_lexicon_tags_11 },
    { 0x106b9d1, SUC_lexicon_tags_419 },
    { 0x61e6b9cd, SUC_lexicon_tags_15 },
    { 0x19a6b9d4, SUC_lexicon_tags_25 },
    { 0x7766b9d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86b9da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xace6b9dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe86b9e1, SUC_lexicon_tags_38 },
    { 0x66b9e2, SUC_lexicon_tags_23 },
    { 0x4466b9e3, SUC_lexicon_tags_10 },
    { 0x83c6b9e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde06b9e7, SUC_lexicon_tags_39 },
    { 0x2486b9e8, SUC_lexicon_tags_179 },
    { 0x99a6b9e9, SUC_lexicon_tags_47 },
    { 0x53a6b9ea, SUC_lexicon_tags_37 },
    { 0xea86b9ea, SUC_lexicon_tags_7 },
    { 0xcd46b9eb, SUC_lexicon_tags_7 },
    { 0x2c6b9ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd446b9f4, SUC_lexicon_tags_43 },
    { 0xade6b9f4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x746b9f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c6b9fc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb026ba03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c6ba0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2066ba0e, SUC_lexicon_tags_27 },
    { 0xfc86ba0f, SUC_lexicon_tags_14 },
    { 0xa106ba0f, SUC_lexicon_tags_37 },
    { 0xe7c6ba11, SUC_lexicon_tags_3 },
    { 0xa0e6ba0e, SUC_lexicon_tags_17 },
    { 0x1d26ba13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe6ba21, SUC_lexicon_tags_63 },
    { 0x3126ba22, SUC_lexicon_tags_4 },
    { 0xb806ba22, SUC_lexicon_tags_30 },
    { 0xd886ba24, SUC_lexicon_tags_151 },
    { 0x686ba23, SUC_lexicon_tags_25 },
    { 0x6da6ba26, SUC_lexicon_tags_14 },
    { 0x44c6ba23, SUC_lexicon_tags_60 },
    { 0x2206ba28, SUC_lexicon_tags_48 },
    { 0xb126ba28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1846ba2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac6ba2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5986ba30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8086ba35, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd326ba38, SUC_lexicon_tags_15 },
    { 0xce06ba39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5086ba3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe66ba43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd926ba45, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c6ba49, SUC_lexicon_tags_15 },
    { 0xd2a6ba4a, SUC_lexicon_tags_14 },
    { 0x5de6ba4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31e6ba4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb886ba52, SUC_lexicon_tags_23 },
    { 0x3d26ba52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46ba57, SUC_lexicon_tags_12 },
    { 0x2e26ba57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ae6ba5a, SUC_lexicon_tags_17 },
    { 0x3ea6ba5b, SUC_lexicon_tags_37 },
    { 0xbce6ba5b, SUC_lexicon_tags_1 },
    { 0x8666ba5d, SUC_lexicon_tags_11 },
    { 0xaa26ba5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e6ba61, SUC_lexicon_tags_7 },
    { 0x4d46ba61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b86ba69, SUC_lexicon_tags_10 },
    { 0x1626ba6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec26ba6e, SUC_lexicon_tags_10 },
    { 0xa346ba6f, SUC_lexicon_tags_96 },
    { 0x5306ba6f, SUC_lexicon_tags_27 },
    { 0x4586ba6f, SUC_lexicon_tags_9 },
    { 0x23e6ba71, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe5c6ba74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a26ba76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7166ba7c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe706ba7e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa6e6ba80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3306ba82, SUC_lexicon_tags_25 },
    { 0x4266ba83, SUC_lexicon_tags_19 },
    { 0xc26ba82, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9126ba86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c6ba8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1746ba8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3706ba8e, SUC_lexicon_tags_27 },
    { 0xc4c6ba8e, SUC_lexicon_tags_5 },
    { 0x8666ba8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d66ba92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc986ba9b, SUC_lexicon_tags_705 },
    { 0x5086ba9c, SUC_lexicon_tags_3 },
    { 0x8ba6ba9d, SUC_lexicon_tags_55 },
    { 0x2066ba9e, SUC_lexicon_tags_7 },
    { 0x60e6ba9c, SUC_lexicon_tags_47 },
    { 0xd706baa0, SUC_lexicon_tags_61 },
    { 0x1be6ba9f, SUC_lexicon_tags_61 },
    { 0xf386ba9e, SUC_lexicon_tags_25 },
    { 0xfb46ba9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4786baa7, SUC_lexicon_tags_16 },
    { 0xc966baa8, SUC_lexicon_tags_60 },
    { 0x6266baa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x686baac, SUC_lexicon_tags_27 },
    { 0x2506baac, SUC_lexicon_tags_27 },
    { 0x2886baad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4966bab6, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0x9526bab9, SUC_lexicon_tags_9 },
    { 0xc226baba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5686babc, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e6bac0, SUC_lexicon_tags_44 },
    { 0x39c6bac1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d06bac3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd26bac6, SUC_lexicon_tags_84 },
    { 0xa966bac6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a6baca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x97e6bacc, SUC_lexicon_tags_12 },
    { 0xc646bacc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e6bad0, SUC_lexicon_tags_103 },
    { 0xb9e6bad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71e6bad3, SUC_lexicon_tags_1 },
    { 0x84c6bad4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a86bad6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e6badd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9006badf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf726bae4, SUC_lexicon_tags_15 },
    { 0x17e6bae4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f66bae7, SUC_lexicon_tags_8 },
    { 0x2a6bae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac06baed, SUC_lexicon_tags_37 },
    { 0x98e6baed, SUC_lexicon_tags_26 },
    { 0xa526baef, SUC_lexicon_tags_27 },
    { 0xc226baf0, SUC_lexicon_tags_15 },
    { 0x90a6baf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8a06baf3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x526baf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1606baf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7466bafb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2746baff, SUC_lexicon_tags_419 },
    { 0x31a6bb00, SUC_lexicon_tags_28 },
    { 0xa66bb01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e6bb05, SUC_lexicon_tags_2 },
    { 0x8f66bb06, SUC_lexicon_tags_66 },
    { 0x1f66bb06, SUC_lexicon_tags_11 },
    { 0x2d06bb08, SUC_lexicon_tags_49 },
    { 0xaca6bb09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe6bb0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece6bb11, SUC_lexicon_tags_65 },
    { 0x89e6bb11, SUC_lexicon_tags_28 },
    { 0x6b26bb13, SUC_lexicon_tags_14 },
    { 0xc1e6bb14, SUC_lexicon_tags_33 },
    { 0xd426bb13, SUC_lexicon_tags_17 },
    { 0xf2e6bb11, SUC_lexicon_tags_37 },
    { 0xe2a6bb15, SUC_lexicon_tags_14 },
    { 0x8346bb12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8966bb1c, SUC_lexicon_tags_33 },
    { 0x3666bb1d, SUC_lexicon_tags_11 },
    { 0x3ea6bb1e, SUC_lexicon_tags_23 },
    { 0x5ba6bb1f, SUC_lexicon_tags_7 },
    { 0xbe26bb1f, SUC_lexicon_tags_17 },
    { 0xba6bb1c, SUC_lexicon_tags_27 },
    { 0xfa6bb1c, SUC_lexicon_tags_7 },
    { 0x5926bb22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c6bb2a, SUC_lexicon_tags_5 },
    { 0x8626bb2a, SUC_lexicon_tags_13 },
    { 0xfb26bb2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a66bb30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3306bb34, SUC_lexicon_tags_79 },
    { 0x69c6bb35, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6446bb3a, SUC_lexicon_tags_13 },
    { 0xfde6bb3b, SUC_lexicon_tags_2 },
    { 0x2b46bb3a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5f86bb3e, SUC_lexicon_tags_37 },
    { 0xd886bb3f, SUC_lexicon_tags_1 },
    { 0x9606bb40, SUC_lexicon_tags_7 },
    { 0x9de6bb40, SUC_lexicon_tags_84 },
    { 0x3f26bb3f, SUC_lexicon_tags_28 },
    { 0xa046bb43, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce6bb46, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x9426bb48, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1846bb4d, SUC_lexicon_tags_33 },
    { 0x1926bb4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa6bb55, SUC_lexicon_tags_241 },
    { 0, NULL },
    { 0, NULL },
    { 0x3306bb58, SUC_lexicon_tags_68 },
    { 0x26bb58, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a6bb5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c6bb63, SUC_lexicon_tags_55 },
    { 0x7586bb64, SUC_lexicon_tags_15 },
    { 0xabe6bb65, SUC_lexicon_tags_17 },
    { 0xf7c6bb66, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4526bb6a, SUC_lexicon_tags_47 },
    { 0x8b06bb6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f86bb6d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c66bb70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e6bb75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c6bb79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3646bb7b, SUC_lexicon_tags_65 },
    { 0x6666bb7c, SUC_lexicon_tags_17 },
    { 0xe426bb7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c6bb81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a26bb86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2006bb89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9446bb8d, SUC_lexicon_tags_47 },
    { 0xbc26bb8e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8186bb92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc6bb96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e6bb99, SUC_lexicon_tags_33 },
    { 0x3ee6bb9a, SUC_lexicon_tags_0 },
    { 0xfbe6bb9b, SUC_lexicon_tags_37 },
    { 0x6b86bb9c, SUC_lexicon_tags_2 },
    { 0x2386bb9b, SUC_lexicon_tags_29 },
    { 0xa626bb9e, SUC_lexicon_tags_1 },
    { 0x2ca6bb9f, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xd66bba1, SUC_lexicon_tags_45 },
    { 0xa406bba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6bba6, SUC_lexicon_tags_17 },
    { 0x5c06bba6, SUC_lexicon_tags_0 },
    { 0x2b86bba7, SUC_lexicon_tags_23 },
    { 0xc4c6bba8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7306bbab, SUC_lexicon_tags_38 },
    { 0x1bc6bbab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe446bbb0, SUC_lexicon_tags_14 },
    { 0xe7e6bbb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc66bbb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e66bbb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9106bbc0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb166bbc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7586bbc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c6bbca, SUC_lexicon_tags_87 },
    { 0xf9c6bbca, SUC_lexicon_tags_17 },
    { 0x99c6bbcb, SUC_lexicon_tags_37 },
    { 0xe066bbcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8826bbd0, SUC_lexicon_tags_19 },
    { 0xd86bbd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f66bbd5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf946bbdb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8926bbe0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x22a6bbe2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6bbe6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c6bbea, SUC_lexicon_tags_37 },
    { 0xc8a6bbeb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf26bbee, SUC_lexicon_tags_9 },
    { 0xd5c6bbef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x146bbf1, SUC_lexicon_tags_17 },
    { 0x3546bbf2, SUC_lexicon_tags_15 },
    { 0xed46bbf2, SUC_lexicon_tags_82 },
    { 0xaf86bbf1, SUC_lexicon_tags_0 },
    { 0x8166bbf5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6046bbf7, SUC_lexicon_tags_7 },
    { 0xa6c6bbf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc6bc01, SUC_lexicon_tags_2 },
    { 0x1786bc02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf766bc05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc026bc07, SUC_lexicon_tags_6 },
    { 0xbcc6bc08, SUC_lexicon_tags_2 },
    { 0xa3e6bc07, SUC_lexicon_tags_26 },
    { 0x2826bc09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6bc12, SUC_lexicon_tags_2 },
    { 0x8ac6bc12, SUC_lexicon_tags_69 },
    { 0x7466bc14, SUC_lexicon_tags_37 },
    { 0xc606bc13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a6bc1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5186bc1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdec6bc21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1e6bc23, SUC_lexicon_tags_60 },
    { 0x8c06bc23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4006bc26, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xcb86bc28, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xbbe6bc2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3926bc2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e46bc2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5946bc30, SUC_lexicon_tags_11 },
    { 0xfd06bc31, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x43e6bc33, SUC_lexicon_tags_14 },
    { 0xc5c6bc34, SUC_lexicon_tags_23 },
    { 0xf406bc34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a6bc38, SUC_lexicon_tags_24 },
    { 0x7c06bc38, SUC_lexicon_tags_17 },
    { 0xd5a6bc3a, SUC_lexicon_tags_63 },
    { 0xea26bc39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf126bc3d, SUC_lexicon_tags_13 },
    { 0x6d26bc3d, SUC_lexicon_tags_7 },
    { 0x20c6bc3f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c6bc44, SUC_lexicon_tags_12 },
    { 0x5fa6bc44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12e6bc47, SUC_lexicon_tags_37 },
    { 0x75a6bc47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d26bc4c, SUC_lexicon_tags_12 },
    { 0x9686bc4d, SUC_lexicon_tags_11 },
    { 0x7c86bc4e, SUC_lexicon_tags_7 },
    { 0xf086bc4f, SUC_lexicon_tags_60 },
    { 0xf2e6bc50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3266bc58, SUC_lexicon_tags_15 },
    { 0x9e26bc59, SUC_lexicon_tags_5 },
    { 0x5406bc59, SUC_lexicon_tags_37 },
    { 0x9066bc5b, SUC_lexicon_tags_11 },
    { 0xa1c6bc5c, SUC_lexicon_tags_28 },
    { 0x22c6bc5d, SUC_lexicon_tags_1 },
    { 0xf626bc5e, SUC_lexicon_tags_19 },
    { 0x8606bc5f, SUC_lexicon_tags_37 },
    { 0xbf66bc5f, SUC_lexicon_tags_7 },
    { 0x386bc61, SUC_lexicon_tags_1 },
    { 0x9d06bc5a, SUC_lexicon_tags_7 },
    { 0xb3c6bc63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a6bc68, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf646bc6c, SUC_lexicon_tags_21 },
    { 0x72e6bc6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc166bc6f, SUC_lexicon_tags_29 },
    { 0x3906bc6f, SUC_lexicon_tags_35 },
    { 0xe646bc71, SUC_lexicon_tags_25 },
    { 0x20e6bc72, SUC_lexicon_tags_12 },
    { 0x1a46bc73, SUC_lexicon_tags_45 },
    { 0xc46bc74, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd506bc7f, SUC_lexicon_tags_23 },
    { 0x5ca6bc80, SUC_lexicon_tags_37 },
    { 0x9626bc81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9926bc84, SUC_lexicon_tags_1 },
    { 0x2da6bc85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06bc89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd446bc8e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd266bc90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfe06bc92, SUC_lexicon_tags_25 },
    { 0x6946bc92, SUC_lexicon_tags_1 },
    { 0xc746bc94, SUC_lexicon_tags_37 },
    { 0xa466bc95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa286bc99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c6bc9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd486bca1, SUC_lexicon_tags_37 },
    { 0x3006bca1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5946bca7, SUC_lexicon_tags_14 },
    { 0x6f86bca8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d86bcaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4706bcad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e46bcb0, SUC_lexicon_tags_160 },
    { 0xc806bcb0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc6bcb4, SUC_lexicon_tags_15 },
    { 0x1506bcb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e86bcb9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b86bcbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e6bcc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7506bcc5, SUC_lexicon_tags_25 },
    { 0x8746bcc6, SUC_lexicon_tags_64 },
    { 0x1506bcc7, SUC_lexicon_tags_27 },
    { 0x826bcc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c6bccb, SUC_lexicon_tags_37 },
    { 0x1f06bccb, SUC_lexicon_tags_11 },
    { 0x7ac6bccb, SUC_lexicon_tags_0 },
    { 0xf2e6bcce, SUC_lexicon_tags_0 },
    { 0x3ca6bccf, SUC_lexicon_tags_7 },
    { 0x4e26bcd0, SUC_lexicon_tags_12 },
    { 0xa086bccb, SUC_lexicon_tags_17 },
    { 0x2546bccf, SUC_lexicon_tags_60 },
    { 0xf1a6bcd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e6bcd7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec6bcda, SUC_lexicon_tags_2 },
    { 0xb166bcdb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a6bce1, SUC_lexicon_tags_10 },
    { 0x81e6bce2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1ee6bce4, SUC_lexicon_tags_10 },
    { 0x80c6bce5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5546bcea, SUC_lexicon_tags_2 },
    { 0x586bcea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd266bcf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe446bcf4, SUC_lexicon_tags_3 },
    { 0x4f66bcf4, SUC_lexicon_tags_151 },
    { 0xc7a6bcf4, SUC_lexicon_tags_63 },
    { 0xe426bcf7, SUC_lexicon_tags_37 },
    { 0xab86bcf8, SUC_lexicon_tags_15 },
    { 0x6506bcf5, SUC_lexicon_tags_37 },
    { 0xdf66bcfa, SUC_lexicon_tags_26 },
    { 0xb0e6bcf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee6bcff, SUC_lexicon_tags_37 },
    { 0x6826bcff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6bd03, SUC_lexicon_tags_14 },
    { 0x75a6bd04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa6bd07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd666bd0a, SUC_lexicon_tags_0 },
    { 0x7866bd0b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa266bd12, SUC_lexicon_tags_1 },
    { 0xc5c6bd13, SUC_lexicon_tags_37 },
    { 0xd6c6bd14, SUC_lexicon_tags_37 },
    { 0x56a6bd15, SUC_lexicon_tags_19 },
    { 0x4e86bd16, SUC_lexicon_tags_23 },
    { 0x64e6bd13, SUC_lexicon_tags_37 },
    { 0x8566bd18, SUC_lexicon_tags_7 },
    { 0x68a6bd17, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xd686bd1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2de6bd1e, SUC_lexicon_tags_138 },
    { 0x7e46bd1f, SUC_lexicon_tags_43 },
    { 0xefa6bd20, SUC_lexicon_tags_17 },
    { 0xd806bd1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa6bd23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c26bd25, SUC_lexicon_tags_45 },
    { 0xa706bd25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e6bd2d, SUC_lexicon_tags_84 },
    { 0x4726bd2d, SUC_lexicon_tags_38 },
    { 0xde46bd2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5586bd31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca6bd38, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06bd3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xde46bd3e, SUC_lexicon_tags_25 },
    { 0xc86bd3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5746bd43, SUC_lexicon_tags_8 },
    { 0x3486bd43, SUC_lexicon_tags_31 },
    { 0xb566bd45, SUC_lexicon_tags_12 },
    { 0xece6bd46, SUC_lexicon_tags_0 },
    { 0x6bd46, SUC_lexicon_tags_39 },
    { 0xe286bd43, SUC_lexicon_tags_15 },
    { 0x6966bd44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7346bd4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac6bd4e, SUC_lexicon_tags_46 },
    { 0xc786bd4f, SUC_lexicon_tags_2 },
    { 0x96a6bd4f, SUC_lexicon_tags_65 },
    { 0x1ac6bd51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3286bd53, SUC_lexicon_tags_8 },
    { 0xb366bd54, SUC_lexicon_tags_40 },
    { 0xc5c6bd54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca6bd58, SUC_lexicon_tags_25 },
    { 0xfb26bd59, SUC_lexicon_tags_31 },
    { 0x37a6bd59, SUC_lexicon_tags_68 },
    { 0xf3c6bd5b, SUC_lexicon_tags_17 },
    { 0x4526bd5c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd606bd63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de6bd66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b26bd69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7de6bd6b, SUC_lexicon_tags_25 },
    { 0x8e86bd6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xad06bd6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5966bd74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3aa6bd76, SUC_lexicon_tags_65 },
    { 0xab26bd76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c6bd7c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d06bd7f, SUC_lexicon_tags_1 },
    { 0x6ac6bd7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ce6bd82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5906bd86, SUC_lexicon_tags_8 },
    { 0xf86bd86, SUC_lexicon_tags_30 },
    { 0x1ce6bd88, SUC_lexicon_tags_26 },
    { 0xd606bd89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6186bd8e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9866bd93, SUC_lexicon_tags_37 },
    { 0x5426bd94, SUC_lexicon_tags_15 },
    { 0x8b26bd94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca6bd98, SUC_lexicon_tags_63 },
    { 0xae06bd99, SUC_lexicon_tags_0 },
    { 0xd0a6bd9a, SUC_lexicon_tags_48 },
    { 0x17a6bd9b, SUC_lexicon_tags_47 },
    { 0xfec6bd9a, SUC_lexicon_tags_8 },
    { 0xe506bd9b, SUC_lexicon_tags_38 },
    { 0xaec6bd9e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeec6bda0, SUC_lexicon_tags_0 },
    { 0xbfa6bda0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e6bda8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd366bdab, SUC_lexicon_tags_63 },
    { 0xd26bdac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6526bdae, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd1c6bdb0, SUC_lexicon_tags_14 },
    { 0x5066bdb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcce6bdb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcf86bdb5, SUC_lexicon_tags_31 },
    { 0x8c6bdb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15c6bdb8, SUC_lexicon_tags_12 },
    { 0x5986bdb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b06bdbb, SUC_lexicon_tags_15 },
    { 0x4966bdbc, SUC_lexicon_tags_28 },
    { 0x64a6bdbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea6bdc1, SUC_lexicon_tags_7 },
    { 0x5ca6bdc1, SUC_lexicon_tags_9 },
    { 0x266bdc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4726bdc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8646bdce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6086bdd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x346bdd3, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c6bdd7, SUC_lexicon_tags_21 },
    { 0xdd86bdd8, SUC_lexicon_tags_45 },
    { 0x1a46bdd9, SUC_lexicon_tags_32 },
    { 0x8a06bdda, SUC_lexicon_tags_11 },
    { 0xb7c6bddb, SUC_lexicon_tags_23 },
    { 0xaec6bddb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef46bde2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac6bde4, SUC_lexicon_tags_31 },
    { 0x3f66bde5, SUC_lexicon_tags_43 },
    { 0xac26bde6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb86bdea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4786bdf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de6bdf3, SUC_lexicon_tags_27 },
    { 0x9746bdf4, SUC_lexicon_tags_48 },
    { 0xfbc6bdf5, SUC_lexicon_tags_13 },
    { 0xb386bdf5, SUC_lexicon_tags_17 },
    { 0xba46bdf7, SUC_lexicon_tags_60 },
    { 0x506bdf8, SUC_lexicon_tags_49 },
    { 0xcb66bdf8, SUC_lexicon_tags_33 },
    { 0x6e26bdfa, SUC_lexicon_tags_25 },
    { 0x9a86bdfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x366bdfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa366be02, SUC_lexicon_tags_25 },
    { 0xe066be03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f26be05, SUC_lexicon_tags_62 },
    { 0x7206be05, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe46be0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c6be12, SUC_lexicon_tags_37 },
    { 0x8ea6be12, SUC_lexicon_tags_15 },
    { 0xc7c6be14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f26be17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe86be1c, SUC_lexicon_tags_33 },
    { 0x5566be1d, SUC_lexicon_tags_37 },
    { 0x5206be1e, SUC_lexicon_tags_0 },
    { 0x8186be1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5fe6be21, SUC_lexicon_tags_11 },
    { 0xf3c6be22, SUC_lexicon_tags_8 },
    { 0x7ba6be23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb546be26, SUC_lexicon_tags_17 },
    { 0xa9e6be26, SUC_lexicon_tags_26 },
    { 0x52e6be27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5006be2c, SUC_lexicon_tags_9 },
    { 0x9c66be2c, SUC_lexicon_tags_402 },
    { 0x20c6be2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1046be31, SUC_lexicon_tags_45 },
    { 0x2026be32, SUC_lexicon_tags_37 },
    { 0x9146be32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc6be36, SUC_lexicon_tags_7 },
    { 0xc4c6be37, SUC_lexicon_tags_10 },
    { 0x2786be36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32e6be3a, SUC_lexicon_tags_1 },
    { 0xc0c6be3b, SUC_lexicon_tags_26 },
    { 0x11c6be3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de6be44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce6be48, SUC_lexicon_tags_151 },
    { 0x10a6be48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa66be50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1826be56, SUC_lexicon_tags_11 },
    { 0x45c6be57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf846be5c, SUC_lexicon_tags_13 },
    { 0x7086be5d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x21c6be5f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe706be62, SUC_lexicon_tags_60 },
    { 0x2146be63, SUC_lexicon_tags_25 },
    { 0x45e6be63, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x21c6be66, SUC_lexicon_tags_25 },
    { 0x88e6be66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f66be6b, SUC_lexicon_tags_25 },
    { 0xe686be6c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd06be6e, SUC_lexicon_tags_17 },
    { 0x4be6be6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa746be76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1326be7f, SUC_lexicon_tags_25 },
    { 0x7366be7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3586be86, SUC_lexicon_tags_30 },
    { 0x9526be86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf46be8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7e26be8d, SUC_lexicon_tags_14 },
    { 0xda86be8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42a6be90, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdec6be92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e6be94, SUC_lexicon_tags_25 },
    { 0xdc86be94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a6be99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a66bea0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd406bea4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4106bea9, SUC_lexicon_tags_17 },
    { 0xafa6beaa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4826beaf, SUC_lexicon_tags_37 },
    { 0xcc86beaf, SUC_lexicon_tags_37 },
    { 0xa926beb0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc746beb4, SUC_lexicon_tags_33 },
    { 0xb346beb5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c6beb8, SUC_lexicon_tags_12 },
    { 0xc6a6beb9, SUC_lexicon_tags_0 },
    { 0xd3c6beb9, SUC_lexicon_tags_10 },
    { 0xd3c6beba, SUC_lexicon_tags_2 },
    { 0xe386bebb, SUC_lexicon_tags_37 },
    { 0xcf26beba, SUC_lexicon_tags_14 },
    { 0x3a46bebe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda6bec9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26c6becb, SUC_lexicon_tags_710 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de6bed3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c6bed6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2d66bed8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b06bedb, SUC_lexicon_tags_10 },
    { 0xf306bedb, SUC_lexicon_tags_84 },
    { 0xa686bedd, SUC_lexicon_tags_37 },
    { 0x9d86bede, SUC_lexicon_tags_0 },
    { 0x5526bedf, SUC_lexicon_tags_37 },
    { 0x2566bedf, SUC_lexicon_tags_6 },
    { 0x5dc6bee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8266bee4, SUC_lexicon_tags_12 },
    { 0x4fa6bee4, SUC_lexicon_tags_47 },
    { 0xdbe6bee6, SUC_lexicon_tags_11 },
    { 0x8826bee7, SUC_lexicon_tags_37 },
    { 0x79c6bee8, SUC_lexicon_tags_44 },
    { 0x60c6bee9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6bef1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9946bef7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc66befa, SUC_lexicon_tags_27 },
    { 0xfbe6befb, SUC_lexicon_tags_38 },
    { 0x6506befc, SUC_lexicon_tags_0 },
    { 0x9086befb, SUC_lexicon_tags_17 },
    { 0x346befe, SUC_lexicon_tags_82 },
    { 0xa5c6befb, SUC_lexicon_tags_28 },
    { 0xf9c6bf00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2c6bf02, SUC_lexicon_tags_37 },
    { 0xf5a6bf03, SUC_lexicon_tags_107 },
    { 0x2c66bf04, SUC_lexicon_tags_15 },
    { 0x3726bf05, SUC_lexicon_tags_45 },
    { 0x6706bf03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f06bf0a, SUC_lexicon_tags_70 },
    { 0x3ba6bf0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e6bf12, SUC_lexicon_tags_25 },
    { 0xf846bf12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a6bf16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6bf21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a6bf25, SUC_lexicon_tags_47 },
    { 0x3446bf25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce6bf2a, SUC_lexicon_tags_27 },
    { 0x1d26bf2b, SUC_lexicon_tags_11 },
    { 0xbc6bf2c, SUC_lexicon_tags_11 },
    { 0x80e6bf2c, SUC_lexicon_tags_43 },
    { 0x1406bf2e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8546bf31, SUC_lexicon_tags_17 },
    { 0xf586bf32, SUC_lexicon_tags_2 },
    { 0x9d46bf31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf46bf35, SUC_lexicon_tags_2 },
    { 0xc026bf36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a66bf3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaa86bf3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9146bf41, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2806bf45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe786bf47, SUC_lexicon_tags_12 },
    { 0x7e66bf47, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc66bf4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb66bf4c, SUC_lexicon_tags_45 },
    { 0xea46bf4c, SUC_lexicon_tags_4 },
    { 0xeec6bf4d, SUC_lexicon_tags_14 },
    { 0x78e6bf4c, SUC_lexicon_tags_25 },
    { 0x2a6bf50, SUC_lexicon_tags_141 },
    { 0x59e6bf4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8a6bf53, SUC_lexicon_tags_15 },
    { 0x4c26bf53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x866bf56, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc506bf60, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa146bf66, SUC_lexicon_tags_11 },
    { 0xcfe6bf67, SUC_lexicon_tags_9 },
    { 0xaa66bf68, SUC_lexicon_tags_25 },
    { 0xb986bf68, SUC_lexicon_tags_56 },
    { 0x7886bf6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5e6bf6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc566bf6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f66bf73, SUC_lexicon_tags_37 },
    { 0x8646bf74, SUC_lexicon_tags_23 },
    { 0xe746bf75, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c6bf79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd146bf7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef26bf7e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc6bf82, SUC_lexicon_tags_2 },
    { 0xdea6bf83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d26bf85, SUC_lexicon_tags_17 },
    { 0xa3a6bf85, SUC_lexicon_tags_6 },
    { 0xeda6bf87, SUC_lexicon_tags_25 },
    { 0x9ac6bf87, SUC_lexicon_tags_60 },
    { 0xb966bf89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf066bf8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f86bf8e, SUC_lexicon_tags_30 },
    { 0x6006bf8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1186bf92, SUC_lexicon_tags_1 },
    { 0xe986bf93, SUC_lexicon_tags_27 },
    { 0x7746bf94, SUC_lexicon_tags_2 },
    { 0xe026bf94, SUC_lexicon_tags_63 },
    { 0x8b06bf96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c6bf9c, SUC_lexicon_tags_0 },
    { 0x7bc6bf9d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef66bf9f, SUC_lexicon_tags_12 },
    { 0x2d26bfa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e6bfa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d06bfa8, SUC_lexicon_tags_38 },
    { 0xadc6bfa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba6bfb5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9326bfb7, SUC_lexicon_tags_60 },
    { 0x5926bfb7, SUC_lexicon_tags_1 },
    { 0xaf66bfb9, SUC_lexicon_tags_15 },
    { 0x1026bfba, SUC_lexicon_tags_2 },
    { 0xa3a6bfb7, SUC_lexicon_tags_7 },
    { 0x3b66bfb9, SUC_lexicon_tags_11 },
    { 0xf386bfbd, SUC_lexicon_tags_15 },
    { 0x6806bfbe, SUC_lexicon_tags_37 },
    { 0x3926bfb7, SUC_lexicon_tags_25 },
    { 0xf906bfba, SUC_lexicon_tags_31 },
    { 0x4726bfb9, SUC_lexicon_tags_19 },
    { 0x7de6bfc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c6bfc7, SUC_lexicon_tags_11 },
    { 0x4d66bfc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6bfca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5646bfcd, SUC_lexicon_tags_43 },
    { 0xe806bfcd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe26bfd0, SUC_lexicon_tags_1 },
    { 0x4e46bfd1, SUC_lexicon_tags_15 },
    { 0xcc66bfd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f26bfd8, SUC_lexicon_tags_1327 },
    { 0x3e06bfd9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b26bfdc, SUC_lexicon_tags_25 },
    { 0xd686bfdd, SUC_lexicon_tags_33 },
    { 0xb066bfde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc106bfe1, SUC_lexicon_tags_29 },
    { 0x3fc6bfe2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ec6bfe4, SUC_lexicon_tags_17 },
    { 0xaec6bfe5, SUC_lexicon_tags_46 },
    { 0x1d66bfe5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf066bfe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a46bfea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f46bfef, SUC_lexicon_tags_6 },
    { 0x6e46bff0, SUC_lexicon_tags_1 },
    { 0x9886bff1, SUC_lexicon_tags_63 },
    { 0xab06bff1, SUC_lexicon_tags_15 },
    { 0x8d46bff3, SUC_lexicon_tags_26 },
    { 0x9646bff2, SUC_lexicon_tags_7 },
    { 0x8c06bff5, SUC_lexicon_tags_23 },
    { 0xb166bff6, SUC_lexicon_tags_49 },
    { 0xd8c6bff7, SUC_lexicon_tags_1 },
    { 0xc306bff7, SUC_lexicon_tags_17 },
    { 0xae66bff9, SUC_lexicon_tags_15 },
    { 0x7b06bff8, SUC_lexicon_tags_23 },
    { 0x8006bffb, SUC_lexicon_tags_47 },
    { 0xca46bffb, SUC_lexicon_tags_62 },
    { 0xcb66bffd, SUC_lexicon_tags_37 },
    { 0xcae6bffe, SUC_lexicon_tags_15 },
    { 0xf386bfff, SUC_lexicon_tags_0 },
    { 0xf986c000, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d46c002, SUC_lexicon_tags_7 },
    { 0xd1e6c003, SUC_lexicon_tags_12 },
    { 0xcb06c003, SUC_lexicon_tags_15 },
    { 0xe126c002, SUC_lexicon_tags_14 },
    { 0x2f46c006, SUC_lexicon_tags_7 },
    { 0x4506c007, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec6c00a, SUC_lexicon_tags_15 },
    { 0xd546c00b, SUC_lexicon_tags_24 },
    { 0x486c00c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x52e6c00e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf86c015, SUC_lexicon_tags_47 },
    { 0xce86c016, SUC_lexicon_tags_84 },
    { 0xd06c016, SUC_lexicon_tags_49 },
    { 0xa2a6c018, SUC_lexicon_tags_56 },
    { 0xfb46c019, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5046c01f, SUC_lexicon_tags_10 },
    { 0xc6c6c01f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf206c023, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9fe6c025, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc466c029, SUC_lexicon_tags_37 },
    { 0x2a26c029, SUC_lexicon_tags_6 },
    { 0x9aa6c02b, SUC_lexicon_tags_9 },
    { 0xb1a6c02a, SUC_lexicon_tags_27 },
    { 0xaae6c02d, SUC_lexicon_tags_7 },
    { 0x69e6c02e, SUC_lexicon_tags_0 },
    { 0xcc6c02e, SUC_lexicon_tags_2 },
    { 0x2986c030, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67e6c032, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae6c035, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9786c038, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a6c03c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1666c040, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe826c042, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac86c044, SUC_lexicon_tags_10 },
    { 0x18a6c044, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c6c048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46c055, SUC_lexicon_tags_44 },
    { 0x9a86c055, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c66c05b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6f86c05d, SUC_lexicon_tags_15 },
    { 0x5206c05e, SUC_lexicon_tags_12 },
    { 0x1a86c05f, SUC_lexicon_tags_12 },
    { 0xa5c6c060, SUC_lexicon_tags_31 },
    { 0x1606c061, SUC_lexicon_tags_2 },
    { 0xd186c062, SUC_lexicon_tags_27 },
    { 0xa7c6c063, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106c068, SUC_lexicon_tags_6 },
    { 0x2206c069, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4606c06c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd626c070, SUC_lexicon_tags_37 },
    { 0x2846c071, SUC_lexicon_tags_4 },
    { 0xccc6c072, SUC_lexicon_tags_11 },
    { 0x6c86c073, SUC_lexicon_tags_23 },
    { 0xa1c6c072, SUC_lexicon_tags_37 },
    { 0x23a6c075, SUC_lexicon_tags_165 },
    { 0xe0a6c076, SUC_lexicon_tags_12 },
    { 0xbe6c077, SUC_lexicon_tags_10 },
    { 0xdc26c078, SUC_lexicon_tags_39 },
    { 0xbee6c079, SUC_lexicon_tags_17 },
    { 0xd986c070, SUC_lexicon_tags_37 },
    { 0x5c26c074, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6c083, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a6c086, SUC_lexicon_tags_22 },
    { 0xb886c087, SUC_lexicon_tags_8 },
    { 0xad46c088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c46c08c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65a6c08e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa466c091, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b46c093, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e6c09f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f66c0a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6c6c0a4, SUC_lexicon_tags_15 },
    { 0x79c6c0a5, SUC_lexicon_tags_65 },
    { 0x8ce6c0a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a66c0b5, SUC_lexicon_tags_37 },
    { 0xd226c0b6, SUC_lexicon_tags_11 },
    { 0x37e6c0b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc486c0bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4886c0c1, SUC_lexicon_tags_236 },
    { 0x4186c0c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9086c0c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f66c0cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3346c0cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8386c0d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b66c0d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a66c0d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1386c0e0, SUC_lexicon_tags_27 },
    { 0x2246c0e1, SUC_lexicon_tags_25 },
    { 0xb166c0e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9126c0ec, SUC_lexicon_tags_98 },
    { 0xfec6c0ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7346c0f1, SUC_lexicon_tags_21 },
    { 0x67e6c0f2, SUC_lexicon_tags_11 },
    { 0x4fc6c0f3, SUC_lexicon_tags_15 },
    { 0x7f06c0f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa46c0f9, SUC_lexicon_tags_82 },
    { 0xf0c6c0fa, SUC_lexicon_tags_11 },
    { 0x49a6c0fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3206c0fd, SUC_lexicon_tags_17 },
    { 0xde66c0fe, SUC_lexicon_tags_30 },
    { 0xd266c0ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1406c103, SUC_lexicon_tags_9 },
    { 0x5706c104, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaa66c106, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb006c109, SUC_lexicon_tags_17 },
    { 0xd7c6c109, SUC_lexicon_tags_8 },
    { 0x2586c10b, SUC_lexicon_tags_37 },
    { 0x9a86c10a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a6c111, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6c114, SUC_lexicon_tags_68 },
    { 0xee46c114, SUC_lexicon_tags_31 },
    { 0x5446c114, SUC_lexicon_tags_12 },
    { 0xd7c6c115, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c6c11d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e6c123, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb206c126, SUC_lexicon_tags_38 },
    { 0x9be6c127, SUC_lexicon_tags_63 },
    { 0x8c86c128, SUC_lexicon_tags_20 },
    { 0x98e6c128, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfa6c12b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e06c131, SUC_lexicon_tags_47 },
    { 0x1ea6c132, SUC_lexicon_tags_17 },
    { 0x1d66c133, SUC_lexicon_tags_13 },
    { 0xaae6c131, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd686c136, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5246c13b, SUC_lexicon_tags_19 },
    { 0xa966c13b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e6c140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c06c142, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3a86c144, SUC_lexicon_tags_2 },
    { 0x6a26c145, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26c149, SUC_lexicon_tags_17 },
    { 0x1cc6c14a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9c66c14c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa186c152, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcd06c154, SUC_lexicon_tags_2 },
    { 0xf426c155, SUC_lexicon_tags_16 },
    { 0x59c6c156, SUC_lexicon_tags_14 },
    { 0xf06c157, SUC_lexicon_tags_29 },
    { 0x2506c158, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7326c15a, SUC_lexicon_tags_28 },
    { 0x61a6c15a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb46c161, SUC_lexicon_tags_14 },
    { 0x5566c162, SUC_lexicon_tags_23 },
    { 0x6ba6c163, SUC_lexicon_tags_33 },
    { 0x8746c161, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2146c169, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed66c16d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6c171, SUC_lexicon_tags_7 },
    { 0x1d26c171, SUC_lexicon_tags_15 },
    { 0x3146c172, SUC_lexicon_tags_46 },
    { 0x5ce6c174, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf386c179, SUC_lexicon_tags_41 },
    { 0xc266c17a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbb66c17c, SUC_lexicon_tags_12 },
    { 0x1a06c17d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd166c185, SUC_lexicon_tags_0 },
    { 0xf686c185, SUC_lexicon_tags_106 },
    { 0xaac6c187, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c66c194, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e6c19f, SUC_lexicon_tags_7 },
    { 0x4ba6c1a0, SUC_lexicon_tags_37 },
    { 0x1126c1a0, SUC_lexicon_tags_33 },
    { 0xb586c1a2, SUC_lexicon_tags_55 },
    { 0x4e66c1a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd566c1a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e6c1a9, SUC_lexicon_tags_7 },
    { 0xa526c1a9, SUC_lexicon_tags_7 },
    { 0xbe06c1ab, SUC_lexicon_tags_33 },
    { 0x81e6c1aa, SUC_lexicon_tags_4 },
    { 0xcee6c1aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ba6c1af, SUC_lexicon_tags_37 },
    { 0xd226c1af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd606c1b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1306c1b5, SUC_lexicon_tags_14 },
    { 0x4c86c1b6, SUC_lexicon_tags_17 },
    { 0x99a6c1b7, SUC_lexicon_tags_2 },
    { 0xcf66c1b7, SUC_lexicon_tags_25 },
    { 0xb4e6c1b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac06c1bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1766c1c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe186c1c7, SUC_lexicon_tags_47 },
    { 0xf786c1c7, SUC_lexicon_tags_13 },
    { 0x286c1c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3c6c1cb, SUC_lexicon_tags_25 },
    { 0x3206c1cc, SUC_lexicon_tags_17 },
    { 0x58e6c1cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e66c1d2, SUC_lexicon_tags_2 },
    { 0xc046c1d3, SUC_lexicon_tags_42 },
    { 0x4806c1d4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e6c1d8, SUC_lexicon_tags_11 },
    { 0x5606c1d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x76c6c1db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4646c1de, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe046c1e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x646c1e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba6c1e7, SUC_lexicon_tags_12 },
    { 0xfd66c1e7, SUC_lexicon_tags_17 },
    { 0x4e86c1e9, SUC_lexicon_tags_13 },
    { 0xa1a6c1ea, SUC_lexicon_tags_5 },
    { 0x36c6c1e9, SUC_lexicon_tags_27 },
    { 0xafc6c1eb, SUC_lexicon_tags_37 },
    { 0x3886c1ec, SUC_lexicon_tags_37 },
    { 0xdf66c1ee, SUC_lexicon_tags_7 },
    { 0xf7a6c1e9, SUC_lexicon_tags_15 },
    { 0x58a6c1f0, SUC_lexicon_tags_136 },
    { 0x2026c1eb, SUC_lexicon_tags_17 },
    { 0x15c6c1f2, SUC_lexicon_tags_38 },
    { 0xe806c1ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3886c1f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa786c1fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16a6c1fe, SUC_lexicon_tags_45 },
    { 0x3846c1fe, SUC_lexicon_tags_37 },
    { 0x3ce6c1fe, SUC_lexicon_tags_11 },
    { 0x77a6c201, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e6c206, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f26c208, SUC_lexicon_tags_17 },
    { 0xb8c6c209, SUC_lexicon_tags_468 },
    { 0x8026c209, SUC_lexicon_tags_64 },
    { 0xa946c20b, SUC_lexicon_tags_7 },
    { 0xb786c209, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc46c20e, SUC_lexicon_tags_30 },
    { 0x6e6c20e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7946c214, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9466c219, SUC_lexicon_tags_56 },
    { 0x3086c21a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6c21d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee6c221, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa066c225, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6886c227, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f46c22d, SUC_lexicon_tags_87 },
    { 0xfca6c22e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e46c235, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xefc6c237, SUC_lexicon_tags_14 },
    { 0x4f46c238, SUC_lexicon_tags_25 },
    { 0x36e6c238, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd306c23c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e86c243, SUC_lexicon_tags_61 },
    { 0xf26c244, SUC_lexicon_tags_87 },
    { 0xc406c244, SUC_lexicon_tags_33 },
    { 0xf246c246, SUC_lexicon_tags_15 },
    { 0x1b46c247, SUC_lexicon_tags_27 },
    { 0x67e6c248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166c24d, SUC_lexicon_tags_84 },
    { 0xf446c24d, SUC_lexicon_tags_7 },
    { 0x3f46c24e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a06c252, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a6c256, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3c46c258, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc46c25b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4086c25d, SUC_lexicon_tags_1 },
    { 0x2666c25e, SUC_lexicon_tags_19 },
    { 0x7026c25f, SUC_lexicon_tags_27 },
    { 0x2826c260, SUC_lexicon_tags_1 },
    { 0x3406c261, SUC_lexicon_tags_26 },
    { 0xd726c25e, SUC_lexicon_tags_60 },
    { 0x6be6c263, SUC_lexicon_tags_12 },
    { 0x9ec6c264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe086c266, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be6c269, SUC_lexicon_tags_53 },
    { 0xabe6c26a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea6c26f, SUC_lexicon_tags_1 },
    { 0xec06c270, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf786c275, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c6c279, SUC_lexicon_tags_11 },
    { 0x42a6c279, SUC_lexicon_tags_15 },
    { 0x2946c27a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6206c27f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e26c283, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3426c285, SUC_lexicon_tags_16 },
    { 0x6146c286, SUC_lexicon_tags_11 },
    { 0xa626c285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5c6c289, SUC_lexicon_tags_10 },
    { 0x7bc6c289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e46c28c, SUC_lexicon_tags_56 },
    { 0x86a6c28d, SUC_lexicon_tags_3 },
    { 0xcf86c28e, SUC_lexicon_tags_7 },
    { 0x8b66c28c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd766c295, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2926c299, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe966c2a2, SUC_lexicon_tags_60 },
    { 0xfb66c2a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf106c2a5, SUC_lexicon_tags_11 },
    { 0xf666c2a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x666c2a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c6c2ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a6c2b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e46c2b5, SUC_lexicon_tags_28 },
    { 0x2286c2b5, SUC_lexicon_tags_17 },
    { 0x16a6c2b7, SUC_lexicon_tags_37 },
    { 0xb0e6c2b7, SUC_lexicon_tags_17 },
    { 0xbb86c2b9, SUC_lexicon_tags_17 },
    { 0xe6a6c2ba, SUC_lexicon_tags_57 },
    { 0x2ea6c2b9, SUC_lexicon_tags_12 },
    { 0x8666c2bb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6906c2c1, SUC_lexicon_tags_11 },
    { 0x2b86c2c2, SUC_lexicon_tags_16 },
    { 0x7da6c2c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c6c2c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8566c2ca, SUC_lexicon_tags_3 },
    { 0x1906c2cb, SUC_lexicon_tags_33 },
    { 0xfd06c2cc, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc46c2cf, SUC_lexicon_tags_11 },
    { 0x6cc6c2d0, SUC_lexicon_tags_15 },
    { 0xdf66c2d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd086c2d3, SUC_lexicon_tags_8 },
    { 0xc946c2d4, SUC_lexicon_tags_7 },
    { 0x4206c2d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d66c2d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1886c2db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa646c2df, SUC_lexicon_tags_1 },
    { 0xe106c2df, SUC_lexicon_tags_65 },
    { 0xb006c2e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa526c2e3, SUC_lexicon_tags_11 },
    { 0x59c6c2e4, SUC_lexicon_tags_7 },
    { 0x1726c2e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3306c2e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf186c2eb, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0xc166c2ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x93a6c2ef, SUC_lexicon_tags_16 },
    { 0x4906c2ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5966c2f6, SUC_lexicon_tags_7 },
    { 0x3806c2f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd66c2fa, SUC_lexicon_tags_1 },
    { 0x3bc6c2fb, SUC_lexicon_tags_33 },
    { 0xb3c6c2fb, SUC_lexicon_tags_0 },
    { 0xfe6c2fc, SUC_lexicon_tags_37 },
    { 0xf5a6c2fe, SUC_lexicon_tags_11 },
    { 0xec26c2ff, SUC_lexicon_tags_9 },
    { 0x1cc6c300, SUC_lexicon_tags_60 },
    { 0x1966c2fa, SUC_lexicon_tags_21 },
    { 0xeb26c2fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19a6c304, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c06c307, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b46c309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf06c30e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e6c312, SUC_lexicon_tags_15 },
    { 0xea66c312, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6c316, SUC_lexicon_tags_14 },
    { 0x1566c316, SUC_lexicon_tags_0 },
    { 0x27a6c316, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf066c31a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a46c31d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbf66c31f, SUC_lexicon_tags_61 },
    { 0xd866c31f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa606c322, SUC_lexicon_tags_2 },
    { 0x4f66c322, SUC_lexicon_tags_25 },
    { 0xab06c322, SUC_lexicon_tags_46 },
    { 0xa3c6c325, SUC_lexicon_tags_151 },
    { 0x4a66c323, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1706c329, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec6c32d, SUC_lexicon_tags_60 },
    { 0x2a86c32d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd446c331, SUC_lexicon_tags_35 },
    { 0x50c6c332, SUC_lexicon_tags_4 },
    { 0xd2e6c331, SUC_lexicon_tags_12 },
    { 0x7d06c333, SUC_lexicon_tags_38 },
    { 0x6f06c331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc6c337, SUC_lexicon_tags_26 },
    { 0x8b86c337, SUC_lexicon_tags_7 },
    { 0xfca6c339, SUC_lexicon_tags_2 },
    { 0x37a6c337, SUC_lexicon_tags_11 },
    { 0xb546c33b, SUC_lexicon_tags_26 },
    { 0x7e6c338, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa566c342, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3226c349, SUC_lexicon_tags_17 },
    { 0xe946c349, SUC_lexicon_tags_38 },
    { 0x45c6c34a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626c34f, SUC_lexicon_tags_39 },
    { 0xd986c350, SUC_lexicon_tags_30 },
    { 0x53c6c34f, SUC_lexicon_tags_12 },
    { 0x1166c351, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e6c356, SUC_lexicon_tags_17 },
    { 0x93c6c357, SUC_lexicon_tags_21 },
    { 0x7a66c358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc6c35c, SUC_lexicon_tags_17 },
    { 0xc386c35c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e66c364, SUC_lexicon_tags_27 },
    { 0xcc06c365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4e6c367, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c6c36c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c46c370, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c6c375, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca6c378, SUC_lexicon_tags_6 },
    { 0x79a6c378, SUC_lexicon_tags_47 },
    { 0xcf46c378, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc6c37e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x51c6c380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e6c382, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2846c385, SUC_lexicon_tags_63 },
    { 0xc0e6c386, SUC_lexicon_tags_25 },
    { 0xaa46c386, SUC_lexicon_tags_2 },
    { 0x2106c387, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7c26c38a, SUC_lexicon_tags_37 },
    { 0xcda6c38a, SUC_lexicon_tags_27 },
    { 0xb866c38c, SUC_lexicon_tags_47 },
    { 0x34e6c38d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2e6c38f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e6c392, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa26c394, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e6c39b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9a6c39d, SUC_lexicon_tags_13 },
    { 0xa1e6c39e, SUC_lexicon_tags_2 },
    { 0xfd46c39f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8806c3a2, SUC_lexicon_tags_8 },
    { 0xb366c3a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca6c3a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf86c3ad, SUC_lexicon_tags_68 },
    { 0xcdc6c3ad, SUC_lexicon_tags_0 },
    { 0x14c6c3ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6246c3b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7446c3b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc6c3b9, SUC_lexicon_tags_33 },
    { 0x2206c3ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca66c3bc, SUC_lexicon_tags_1 },
    { 0xea66c3bd, SUC_lexicon_tags_14 },
    { 0xcee6c3be, SUC_lexicon_tags_25 },
    { 0x71a6c3bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c6c3c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba66c3c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a6c3c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf66c3c9, SUC_lexicon_tags_37 },
    { 0xb426c3c9, SUC_lexicon_tags_47 },
    { 0x6866c3ca, SUC_lexicon_tags_37 },
    { 0xedc6c3cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc326c3db, SUC_lexicon_tags_15 },
    { 0x4546c3dc, SUC_lexicon_tags_25 },
    { 0xcee6c3dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f86c3e1, SUC_lexicon_tags_17 },
    { 0x7486c3e2, SUC_lexicon_tags_45 },
    { 0xf186c3e3, SUC_lexicon_tags_21 },
    { 0xa106c3e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4526c3e6, SUC_lexicon_tags_11 },
    { 0x5d26c3e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc86c3e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c6c3ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6626c3f0, SUC_lexicon_tags_2 },
    { 0x63e6c3f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c6c3f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca6c3f7, SUC_lexicon_tags_25 },
    { 0x5826c3f8, SUC_lexicon_tags_37 },
    { 0xbb86c3f7, SUC_lexicon_tags_15 },
    { 0x8d46c3fa, SUC_lexicon_tags_23 },
    { 0xa146c3fa, SUC_lexicon_tags_11 },
    { 0x5026c3fc, SUC_lexicon_tags_47 },
    { 0xef66c3fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d66c401, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x4486c403, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec6c407, SUC_lexicon_tags_11 },
    { 0x2b86c408, SUC_lexicon_tags_27 },
    { 0xfc26c409, SUC_lexicon_tags_45 },
    { 0x2346c408, SUC_lexicon_tags_16 },
    { 0x3fc6c40a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b26c40e, SUC_lexicon_tags_25 },
    { 0x3a86c40f, SUC_lexicon_tags_13 },
    { 0x1206c410, SUC_lexicon_tags_0 },
    { 0xe606c411, SUC_lexicon_tags_38 },
    { 0xb26c410, SUC_lexicon_tags_47 },
    { 0x9386c413, SUC_lexicon_tags_2 },
    { 0x5e06c40f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a6c417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca86c41d, SUC_lexicon_tags_62 },
    { 0x89e6c41e, SUC_lexicon_tags_0 },
    { 0xe846c41f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95a6c421, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf26c424, SUC_lexicon_tags_28 },
    { 0x606c424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d26c42a, SUC_lexicon_tags_17 },
    { 0x4646c42b, SUC_lexicon_tags_57 },
    { 0xc786c42b, SUC_lexicon_tags_8 },
    { 0x4706c42d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea86c42f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68a6c431, SUC_lexicon_tags_33 },
    { 0xc506c431, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dc6c434, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3026c436, SUC_lexicon_tags_7 },
    { 0xdcc6c437, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe966c439, SUC_lexicon_tags_1 },
    { 0x9e6c439, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf546c43c, SUC_lexicon_tags_17 },
    { 0xf9c6c43d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf26c440, SUC_lexicon_tags_38 },
    { 0x4306c440, SUC_lexicon_tags_87 },
    { 0x99c6c442, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c6c44d, SUC_lexicon_tags_14 },
    { 0xbb26c44e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c86c450, SUC_lexicon_tags_26 },
    { 0xde46c451, SUC_lexicon_tags_2 },
    { 0x4766c452, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe6c455, SUC_lexicon_tags_2 },
    { 0xf246c455, SUC_lexicon_tags_33 },
    { 0x8c66c457, SUC_lexicon_tags_63 },
    { 0xe706c456, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e6c463, SUC_lexicon_tags_91 },
    { 0x6206c463, SUC_lexicon_tags_25 },
    { 0xe146c465, SUC_lexicon_tags_29 },
    { 0x146c466, SUC_lexicon_tags_37 },
    { 0xbe6c467, SUC_lexicon_tags_55 },
    { 0xa386c467, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc346c472, SUC_lexicon_tags_2 },
    { 0x53a6c473, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d46c476, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a86c478, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a6c47d, SUC_lexicon_tags_48 },
    { 0xa846c47e, SUC_lexicon_tags_33 },
    { 0xb946c47e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xee26c481, SUC_lexicon_tags_62 },
    { 0x4fa6c481, SUC_lexicon_tags_27 },
    { 0xd066c483, SUC_lexicon_tags_27 },
    { 0xf886c484, SUC_lexicon_tags_12 },
    { 0xdf66c485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2346c487, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9e66c489, SUC_lexicon_tags_618 },
    { 0x84e6c48a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a6c48d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e46c492, SUC_lexicon_tags_188 },
    { 0x1766c492, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x7026c495, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c6c49b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c26c49d, SUC_lexicon_tags_30 },
    { 0x81e6c49e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc426c4a0, SUC_lexicon_tags_33 },
    { 0x83a6c4a0, SUC_lexicon_tags_11 },
    { 0x3a26c4a2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8c86c4a4, SUC_lexicon_tags_37 },
    { 0xa4e6c4a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8466c4a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae6c4ac, SUC_lexicon_tags_15 },
    { 0x7426c4ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e6c4b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c6c4b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a6c4c0, SUC_lexicon_tags_10 },
    { 0x7346c4c0, SUC_lexicon_tags_15 },
    { 0x7846c4c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4666c4c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea6c4c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c6c4d1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc6c4d6, SUC_lexicon_tags_38 },
    { 0xdae6c4d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec66c4e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a46c4e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ce6c4eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3e06c4ed, SUC_lexicon_tags_17 },
    { 0x5d86c4ee, SUC_lexicon_tags_66 },
    { 0xec6c4ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a6c4f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37e6c4f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc26c4fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3506c500, SUC_lexicon_tags_9 },
    { 0xfb66c500, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f46c505, SUC_lexicon_tags_63 },
    { 0xeac6c505, SUC_lexicon_tags_14 },
    { 0xd106c507, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde06c50b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c46c50e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb746c510, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca6c513, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfc6c515, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe826c517, SUC_lexicon_tags_37 },
    { 0xfba6c518, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb966c523, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b46c528, SUC_lexicon_tags_6 },
    { 0xd1a6c529, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f66c52e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c6c533, SUC_lexicon_tags_11 },
    { 0xd2a6c533, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8166c536, SUC_lexicon_tags_14 },
    { 0x1e46c537, SUC_lexicon_tags_1 },
    { 0x6386c538, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d06c540, SUC_lexicon_tags_16 },
    { 0x9506c541, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa286c543, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a6c546, SUC_lexicon_tags_9 },
    { 0x2c6c547, SUC_lexicon_tags_2 },
    { 0xcda6c548, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb766c54c, SUC_lexicon_tags_2 },
    { 0xe0c6c54d, SUC_lexicon_tags_16 },
    { 0x7626c54d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd026c551, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3886c555, SUC_lexicon_tags_17 },
    { 0xc686c556, SUC_lexicon_tags_85 },
    { 0xba66c557, SUC_lexicon_tags_79 },
    { 0xc46c558, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a6c55d, SUC_lexicon_tags_25 },
    { 0xb186c55d, SUC_lexicon_tags_49 },
    { 0x7fc6c55f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c26c562, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c86c566, SUC_lexicon_tags_45 },
    { 0x3ac6c566, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e6c56a, SUC_lexicon_tags_39 },
    { 0x6526c56b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec06c56d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb826c570, SUC_lexicon_tags_656 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc06c573, SUC_lexicon_tags_14 },
    { 0x75e6c574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4726c57c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1786c57e, SUC_lexicon_tags_17 },
    { 0x346c57f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd446c584, SUC_lexicon_tags_6 },
    { 0x7746c585, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e6c58b, SUC_lexicon_tags_38 },
    { 0x2806c58c, SUC_lexicon_tags_29 },
    { 0x6766c58d, SUC_lexicon_tags_33 },
    { 0xba06c58b, SUC_lexicon_tags_68 },
    { 0x2ca6c58f, SUC_lexicon_tags_0 },
    { 0xa766c590, SUC_lexicon_tags_10 },
    { 0x74a6c58c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb26c599, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e6c5a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc6c5a3, SUC_lexicon_tags_38 },
    { 0x7dc6c5a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8946c5a7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5146c5aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c6c5ae, SUC_lexicon_tags_22 },
    { 0x5986c5ae, SUC_lexicon_tags_39 },
    { 0xc226c5b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce26c5b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf526c5b9, SUC_lexicon_tags_14 },
    { 0x4ca6c5ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe226c5bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c6c5c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f26c5c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb106c5c6, SUC_lexicon_tags_7 },
    { 0xe9c6c5c7, SUC_lexicon_tags_32 },
    { 0x7e86c5c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6926c5ca, SUC_lexicon_tags_35 },
    { 0xd4e6c5cb, SUC_lexicon_tags_28 },
    { 0x72a6c5cc, SUC_lexicon_tags_15 },
    { 0x9c26c5ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc446c5cf, SUC_lexicon_tags_55 },
    { 0xb286c5cf, SUC_lexicon_tags_47 },
    { 0xcd66c5d1, SUC_lexicon_tags_65 },
    { 0xcfe6c5d2, SUC_lexicon_tags_65 },
    { 0xf066c5d2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x30e6c5d5, SUC_lexicon_tags_42 },
    { 0x74e6c5d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8e26c5d8, SUC_lexicon_tags_47 },
    { 0xfd66c5d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e66c5dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c6c5e0, SUC_lexicon_tags_25 },
    { 0xb766c5e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4706c5e5, SUC_lexicon_tags_47 },
    { 0x20c6c5e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x48a6c5e8, SUC_lexicon_tags_1 },
    { 0xaf26c5e8, SUC_lexicon_tags_2 },
    { 0x5266c5ea, SUC_lexicon_tags_47 },
    { 0xf06c5ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x646c5ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a6c5f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e6c5f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e86c5fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa706c5fe, SUC_lexicon_tags_25 },
    { 0xed26c5ff, SUC_lexicon_tags_37 },
    { 0x72a6c600, SUC_lexicon_tags_2 },
    { 0x83e6c601, SUC_lexicon_tags_17 },
    { 0xa846c5ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e6c608, SUC_lexicon_tags_38 },
    { 0x18c6c609, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2026c60d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9446c60f, SUC_lexicon_tags_65 },
    { 0x8606c610, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e46c613, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d26c61a, SUC_lexicon_tags_12 },
    { 0x95c6c61a, SUC_lexicon_tags_0 },
    { 0x4e86c61c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f26c61f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5326c622, SUC_lexicon_tags_15 },
    { 0x7dc6c623, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xb0a6c625, SUC_lexicon_tags_10 },
    { 0x5c26c626, SUC_lexicon_tags_32 },
    { 0x81a6c627, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce6c62a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb286c62c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8406c62e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c06c635, SUC_lexicon_tags_12 },
    { 0x7846c636, SUC_lexicon_tags_37 },
    { 0xd846c637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc046c63b, SUC_lexicon_tags_25 },
    { 0xf066c63c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2666c640, SUC_lexicon_tags_17 },
    { 0xcf46c641, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xde86c643, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb86c648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8306c64a, SUC_lexicon_tags_45 },
    { 0x4406c64b, SUC_lexicon_tags_7 },
    { 0x23a6c64b, SUC_lexicon_tags_15 },
    { 0xf386c64c, SUC_lexicon_tags_9 },
    { 0xb1c6c64e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3566c655, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9106c65a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4766c65c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c6c662, SUC_lexicon_tags_29 },
    { 0xba6c663, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x3ee6c665, SUC_lexicon_tags_11 },
    { 0xa2c6c665, SUC_lexicon_tags_27 },
    { 0x1bc6c667, SUC_lexicon_tags_7 },
    { 0x1046c668, SUC_lexicon_tags_45 },
    { 0xdba6c668, SUC_lexicon_tags_37 },
    { 0x8846c669, SUC_lexicon_tags_38 },
    { 0x8846c669, SUC_lexicon_tags_31 },
    { 0x2b66c66a, SUC_lexicon_tags_39 },
    { 0xd866c669, SUC_lexicon_tags_17 },
    { 0xe8a6c66c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x38a6c670, SUC_lexicon_tags_23 },
    { 0xe846c671, SUC_lexicon_tags_66 },
    { 0xdd26c672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17e6c674, SUC_lexicon_tags_2 },
    { 0x8006c675, SUC_lexicon_tags_66 },
    { 0xae26c676, SUC_lexicon_tags_17 },
    { 0x48c6c677, SUC_lexicon_tags_12 },
    { 0x9406c675, SUC_lexicon_tags_11 },
    { 0xeec6c679, SUC_lexicon_tags_15 },
    { 0xb126c679, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f86c67d, SUC_lexicon_tags_10 },
    { 0xa526c67e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2366c680, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1e6c682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ca6c684, SUC_lexicon_tags_12 },
    { 0x2606c685, SUC_lexicon_tags_37 },
    { 0x2d66c684, SUC_lexicon_tags_37 },
    { 0x52c6c687, SUC_lexicon_tags_1 },
    { 0x446c688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cc6c68a, SUC_lexicon_tags_87 },
    { 0xbc06c68b, SUC_lexicon_tags_49 },
    { 0xaa66c68c, SUC_lexicon_tags_32 },
    { 0x3886c68d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae6c695, SUC_lexicon_tags_32 },
    { 0x50a6c695, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9706c698, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc026c69c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446c6a4, SUC_lexicon_tags_45 },
    { 0xd866c6a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5746c6a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c6c6ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c6c6af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc6c6b3, SUC_lexicon_tags_39 },
    { 0x4086c6b4, SUC_lexicon_tags_7 },
    { 0x2886c6b5, SUC_lexicon_tags_11 },
    { 0xb366c6b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e6c6c1, SUC_lexicon_tags_7 },
    { 0x6f86c6c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9506c6c5, SUC_lexicon_tags_74 },
    { 0x14a6c6c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7c86c6c8, SUC_lexicon_tags_8 },
    { 0xc1e6c6c9, SUC_lexicon_tags_22 },
    { 0xba46c6c9, SUC_lexicon_tags_8 },
    { 0x4bc6c6c9, SUC_lexicon_tags_64 },
    { 0x61c6c6cc, SUC_lexicon_tags_25 },
    { 0x4426c6c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc6c6d1, SUC_lexicon_tags_27 },
    { 0x1526c6d1, SUC_lexicon_tags_1 },
    { 0x7906c6d3, SUC_lexicon_tags_12 },
    { 0x4526c6d4, SUC_lexicon_tags_25 },
    { 0x5dc6c6d3, SUC_lexicon_tags_47 },
    { 0x9fc6c6d5, SUC_lexicon_tags_45 },
    { 0x2ba6c6d7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a6c6db, SUC_lexicon_tags_10 },
    { 0x7a46c6dc, SUC_lexicon_tags_26 },
    { 0x8a46c6dc, SUC_lexicon_tags_24 },
    { 0xd486c6de, SUC_lexicon_tags_12 },
    { 0x3246c6de, SUC_lexicon_tags_31 },
    { 0xfb86c6e0, SUC_lexicon_tags_23 },
    { 0xdbe6c6db, SUC_lexicon_tags_37 },
    { 0x8c26c6e2, SUC_lexicon_tags_17 },
    { 0xe6e6c6e2, SUC_lexicon_tags_25 },
    { 0xae26c6de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7326c6e6, SUC_lexicon_tags_0 },
    { 0x8506c6e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb686c6ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc6c6f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb126c6f9, SUC_lexicon_tags_39 },
    { 0xaea6c6f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd466c6ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c06c705, SUC_lexicon_tags_1 },
    { 0xe3a6c705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe6c70d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a6c711, SUC_lexicon_tags_85 },
    { 0xb66c711, SUC_lexicon_tags_37 },
    { 0x5a66c713, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5546c716, SUC_lexicon_tags_31 },
    { 0xb146c716, SUC_lexicon_tags_7 },
    { 0xc746c716, SUC_lexicon_tags_10 },
    { 0x9d86c717, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5a6c71b, SUC_lexicon_tags_7 },
    { 0x326c71c, SUC_lexicon_tags_57 },
    { 0x5a86c71c, SUC_lexicon_tags_7 },
    { 0xe646c71e, SUC_lexicon_tags_27 },
    { 0xdb26c71e, SUC_lexicon_tags_37 },
    { 0x8466c720, SUC_lexicon_tags_37 },
    { 0x63e6c721, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa286c723, SUC_lexicon_tags_38 },
    { 0x1e6c723, SUC_lexicon_tags_27 },
    { 0x1446c724, SUC_lexicon_tags_28 },
    { 0x4386c726, SUC_lexicon_tags_16 },
    { 0x5366c727, SUC_lexicon_tags_25 },
    { 0xc286c728, SUC_lexicon_tags_17 },
    { 0x6c86c726, SUC_lexicon_tags_37 },
    { 0x99c6c724, SUC_lexicon_tags_64 },
    { 0xbca6c72b, SUC_lexicon_tags_8 },
    { 0xe8c6c72c, SUC_lexicon_tags_7 },
    { 0x3626c72d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa106c72f, SUC_lexicon_tags_50 },
    { 0xdec6c730, SUC_lexicon_tags_11 },
    { 0x7a46c731, SUC_lexicon_tags_65 },
    { 0x3886c731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb906c734, SUC_lexicon_tags_60 },
    { 0x8546c735, SUC_lexicon_tags_14 },
    { 0x99c6c734, SUC_lexicon_tags_9 },
    { 0xf526c735, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x706c739, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa6c73c, SUC_lexicon_tags_12 },
    { 0x1886c73d, SUC_lexicon_tags_37 },
    { 0xe786c73d, SUC_lexicon_tags_13 },
    { 0xb26c73d, SUC_lexicon_tags_17 },
    { 0x23e6c73d, SUC_lexicon_tags_12 },
    { 0x8746c741, SUC_lexicon_tags_29 },
    { 0xff66c742, SUC_lexicon_tags_24 },
    { 0xad86c743, SUC_lexicon_tags_37 },
    { 0x1b06c744, SUC_lexicon_tags_33 },
    { 0x7386c745, SUC_lexicon_tags_69 },
    { 0x5d66c745, SUC_lexicon_tags_37 },
    { 0x4526c73c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf486c74f, SUC_lexicon_tags_17 },
    { 0x92a6c750, SUC_lexicon_tags_28 },
    { 0x9646c751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9866c753, SUC_lexicon_tags_1 },
    { 0x6ec6c754, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a06c756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e6c75d, SUC_lexicon_tags_10 },
    { 0xc526c75d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddc6c760, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde86c766, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146c76a, SUC_lexicon_tags_1 },
    { 0x1206c76b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd026c76d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f46c76f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb86c775, SUC_lexicon_tags_29 },
    { 0xe646c776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c6c77a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae6c77f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9c6c781, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e6c786, SUC_lexicon_tags_45 },
    { 0x5766c787, SUC_lexicon_tags_46 },
    { 0xbe06c788, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd346c78e, SUC_lexicon_tags_19 },
    { 0xd666c78e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5486c795, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8426c79a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e6c7a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf426c7a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea06c7ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec6c7b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc166c7b3, SUC_lexicon_tags_69 },
    { 0xd786c7b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab86c7b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe6c7b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6da6c7bb, SUC_lexicon_tags_47 },
    { 0xd0a6c7bc, SUC_lexicon_tags_14 },
    { 0x2446c7bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x386c7c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf46c7c2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c6c7cb, SUC_lexicon_tags_17 },
    { 0xa5c6c7cb, SUC_lexicon_tags_3 },
    { 0x5566c7cc, SUC_lexicon_tags_23 },
    { 0x91a6c7cb, SUC_lexicon_tags_9 },
    { 0xbc06c7ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c6c7d4, SUC_lexicon_tags_35 },
    { 0x9006c7d4, SUC_lexicon_tags_60 },
    { 0x2be6c7d5, SUC_lexicon_tags_66 },
    { 0x8126c7d7, SUC_lexicon_tags_17 },
    { 0xc4a6c7d7, SUC_lexicon_tags_9 },
    { 0x7a66c7d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1806c7db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa6c7de, SUC_lexicon_tags_0 },
    { 0x2e26c7df, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1186c7e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6346c7ea, SUC_lexicon_tags_57 },
    { 0x1626c7ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8966c7f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c6c7f5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8066c7f9, SUC_lexicon_tags_21 },
    { 0x1886c7fa, SUC_lexicon_tags_7 },
    { 0x506c7f9, SUC_lexicon_tags_28 },
    { 0x30e6c7fc, SUC_lexicon_tags_37 },
    { 0x3a26c7fa, SUC_lexicon_tags_19 },
    { 0x2d66c7fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a6c801, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaca6c803, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xca66c805, SUC_lexicon_tags_15 },
    { 0xc526c805, SUC_lexicon_tags_47 },
    { 0xcde6c806, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6c80c, SUC_lexicon_tags_28 },
    { 0x93c6c80d, SUC_lexicon_tags_25 },
    { 0x1546c80e, SUC_lexicon_tags_12 },
    { 0x6d26c80d, SUC_lexicon_tags_38 },
    { 0xe2a6c80f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8446c815, SUC_lexicon_tags_15 },
    { 0x7b66c816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd306c819, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4786c821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8806c824, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8386c830, SUC_lexicon_tags_37 },
    { 0xe5c6c831, SUC_lexicon_tags_30 },
    { 0xc966c832, SUC_lexicon_tags_37 },
    { 0x6ae6c830, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88a6c835, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e6c838, SUC_lexicon_tags_23 },
    { 0xb666c838, SUC_lexicon_tags_49 },
    { 0x9986c839, SUC_lexicon_tags_2 },
    { 0x41a6c83a, SUC_lexicon_tags_21 },
    { 0x3786c83c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f06c845, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xcc06c847, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcee6c849, SUC_lexicon_tags_48 },
    { 0x7a26c849, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5126c84e, SUC_lexicon_tags_8 },
    { 0xb286c84e, SUC_lexicon_tags_43 },
    { 0x6006c84f, SUC_lexicon_tags_63 },
    { 0x1746c84e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc06c858, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xca86c85b, SUC_lexicon_tags_106 },
    { 0xed66c85c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2306c860, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50c6c862, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4746c866, SUC_lexicon_tags_16 },
    { 0xf7a6c867, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6c86b, SUC_lexicon_tags_1 },
    { 0x7a6c86c, SUC_lexicon_tags_30 },
    { 0xff86c86c, SUC_lexicon_tags_6 },
    { 0x706c86e, SUC_lexicon_tags_61 },
    { 0x93a6c86f, SUC_lexicon_tags_12 },
    { 0xf4c6c86d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc206c872, SUC_lexicon_tags_7 },
    { 0x8606c873, SUC_lexicon_tags_33 },
    { 0xbc6c874, SUC_lexicon_tags_0 },
    { 0x9e66c875, SUC_lexicon_tags_17 },
    { 0x69c6c876, SUC_lexicon_tags_62 },
    { 0x2566c875, SUC_lexicon_tags_17 },
    { 0xe3c6c876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd06c87e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6906c881, SUC_lexicon_tags_31 },
    { 0xa1e6c882, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xee06c884, SUC_lexicon_tags_25 },
    { 0xf5c6c885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4a6c887, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc526c889, SUC_lexicon_tags_47 },
    { 0xa286c88a, SUC_lexicon_tags_62 },
    { 0x1486c88a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e06c890, SUC_lexicon_tags_19 },
    { 0x7d06c890, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c6c896, SUC_lexicon_tags_1 },
    { 0x4f46c897, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a6c89b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9fc6c89d, SUC_lexicon_tags_60 },
    { 0xc966c89d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c6c8a1, SUC_lexicon_tags_1 },
    { 0xf046c8a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc06c8a7, SUC_lexicon_tags_17 },
    { 0xc826c8a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde66c8ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb506c8ad, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x81e6c8af, SUC_lexicon_tags_8 },
    { 0xe606c8b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb86c8c0, SUC_lexicon_tags_133 },
    { 0x9bc6c8c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a6c8c4, SUC_lexicon_tags_17 },
    { 0x526c8c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb66c8c9, SUC_lexicon_tags_45 },
    { 0xe446c8ca, SUC_lexicon_tags_35 },
    { 0xed86c8cb, SUC_lexicon_tags_12 },
    { 0xd7a6c8cc, SUC_lexicon_tags_12 },
    { 0xe006c8cd, SUC_lexicon_tags_17 },
    { 0x9206c8ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc6c8d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7506c8d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1446c8d6, SUC_lexicon_tags_26 },
    { 0x93a6c8d7, SUC_lexicon_tags_45 },
    { 0x3826c8d8, SUC_lexicon_tags_49 },
    { 0x8e06c8d9, SUC_lexicon_tags_22 },
    { 0x8186c8da, SUC_lexicon_tags_33 },
    { 0xd6a6c8db, SUC_lexicon_tags_37 },
    { 0x90e6c8dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e26c8df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c6c8e4, SUC_lexicon_tags_45 },
    { 0x9526c8e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5126c8ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf566c8ef, SUC_lexicon_tags_12 },
    { 0xb5c6c8f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab26c8f7, SUC_lexicon_tags_17 },
    { 0xeb86c8f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc906c8fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e6c8ff, SUC_lexicon_tags_29 },
    { 0x2146c900, SUC_lexicon_tags_29 },
    { 0x74e6c900, SUC_lexicon_tags_37 },
    { 0x25e6c902, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d66c907, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc26c90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e6c911, SUC_lexicon_tags_58 },
    { 0x4f86c912, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c26c914, SUC_lexicon_tags_47 },
    { 0x55a6c914, SUC_lexicon_tags_29 },
    { 0x13e6c916, SUC_lexicon_tags_25 },
    { 0x12e6c915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91e6c919, SUC_lexicon_tags_1 },
    { 0x6a86c919, SUC_lexicon_tags_11 },
    { 0x5426c91b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3086c91d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5806c920, SUC_lexicon_tags_2 },
    { 0xdae6c920, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e06c924, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68e6c926, SUC_lexicon_tags_37 },
    { 0x1ec6c926, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a6c92f, SUC_lexicon_tags_17 },
    { 0x5b46c92f, SUC_lexicon_tags_30 },
    { 0x1be6c931, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde6c937, SUC_lexicon_tags_7 },
    { 0x4766c937, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1426c93c, SUC_lexicon_tags_60 },
    { 0x9c6c93c, SUC_lexicon_tags_25 },
    { 0x7566c93e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3266c941, SUC_lexicon_tags_11 },
    { 0x9a26c941, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e66c944, SUC_lexicon_tags_45 },
    { 0x9e66c945, SUC_lexicon_tags_15 },
    { 0x5706c944, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5186c949, SUC_lexicon_tags_25 },
    { 0x6766c94a, SUC_lexicon_tags_25 },
    { 0x8be6c94b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8746c94d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb86c951, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb286c956, SUC_lexicon_tags_12 },
    { 0x6746c957, SUC_lexicon_tags_37 },
    { 0x98c6c958, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b26c95b, SUC_lexicon_tags_11 },
    { 0xc9a6c95b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6026c965, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba6c967, SUC_lexicon_tags_61 },
    { 0xb606c968, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a6c96c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc6c970, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9426c975, SUC_lexicon_tags_12 },
    { 0x62a6c975, SUC_lexicon_tags_63 },
    { 0x3506c976, SUC_lexicon_tags_63 },
    { 0xf2a6c975, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a6c97c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346c980, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x98c6c982, SUC_lexicon_tags_39 },
    { 0xaac6c982, SUC_lexicon_tags_19 },
    { 0x6ae6c984, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe046c988, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xee66c98b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe6c98e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c66c997, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706c99b, SUC_lexicon_tags_17 },
    { 0x8386c99c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b66c9a1, SUC_lexicon_tags_14 },
    { 0x9846c9a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4846c9a6, SUC_lexicon_tags_31 },
    { 0x3966c9a7, SUC_lexicon_tags_15 },
    { 0xd766c9a7, SUC_lexicon_tags_11 },
    { 0x4d06c9a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646c9b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c46c9b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c86c9b8, SUC_lexicon_tags_10 },
    { 0x6c66c9b9, SUC_lexicon_tags_15 },
    { 0x8fe6c9b9, SUC_lexicon_tags_37 },
    { 0xad06c9bb, SUC_lexicon_tags_15 },
    { 0xe386c9ba, SUC_lexicon_tags_45 },
    { 0x64e6c9bb, SUC_lexicon_tags_85 },
    { 0x7426c9b9, SUC_lexicon_tags_37 },
    { 0x726c9b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3426c9c4, SUC_lexicon_tags_25 },
    { 0x3fc6c9c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae06c9c7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x36e6c9c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e6c9cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x646c9d0, SUC_lexicon_tags_66 },
    { 0x1086c9d1, SUC_lexicon_tags_8 },
    { 0xe106c9d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6c9d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5626c9de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c6c9e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe046c9ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ee6c9ec, SUC_lexicon_tags_11 },
    { 0x2406c9ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf126c9f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac6c9f7, SUC_lexicon_tags_47 },
    { 0xce66c9f8, SUC_lexicon_tags_17 },
    { 0xf906c9f9, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe6c9fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c6ca01, SUC_lexicon_tags_65 },
    { 0xb166ca02, SUC_lexicon_tags_57 },
    { 0xae46ca03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4446ca06, SUC_lexicon_tags_23 },
    { 0x426ca07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d66ca09, SUC_lexicon_tags_77 },
    { 0x4106ca0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2346ca10, SUC_lexicon_tags_33 },
    { 0xdd66ca11, SUC_lexicon_tags_1 },
    { 0xde06ca10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e6ca15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9546ca1f, SUC_lexicon_tags_60 },
    { 0xc646ca1f, SUC_lexicon_tags_15 },
    { 0x58e6ca21, SUC_lexicon_tags_17 },
    { 0x60c6ca22, SUC_lexicon_tags_2 },
    { 0x9b26ca20, SUC_lexicon_tags_11 },
    { 0xdea6ca21, SUC_lexicon_tags_37 },
    { 0xdc86ca25, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4286ca2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfde6ca2f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c06ca37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e46ca3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedc6ca3d, SUC_lexicon_tags_2 },
    { 0xe1e6ca3e, SUC_lexicon_tags_45 },
    { 0x8f86ca3f, SUC_lexicon_tags_34 },
    { 0xa906ca40, SUC_lexicon_tags_11 },
    { 0x3da6ca3d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e46ca44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee6ca47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa006ca4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a26ca51, SUC_lexicon_tags_17 },
    { 0x5b06ca52, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54a6ca54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb86ca56, SUC_lexicon_tags_9 },
    { 0x1e66ca56, SUC_lexicon_tags_1 },
    { 0x8f26ca58, SUC_lexicon_tags_14 },
    { 0xee66ca59, SUC_lexicon_tags_33 },
    { 0xe9e6ca58, SUC_lexicon_tags_12 },
    { 0x2e06ca5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e26ca60, SUC_lexicon_tags_25 },
    { 0xa646ca60, SUC_lexicon_tags_25 },
    { 0x1166ca60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d86ca65, SUC_lexicon_tags_2 },
    { 0x9806ca66, SUC_lexicon_tags_12 },
    { 0xb746ca67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7d06ca69, SUC_lexicon_tags_29 },
    { 0xd486ca6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2526ca6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06ca72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb506ca74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b46ca79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6ca7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd406ca80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e6ca84, SUC_lexicon_tags_32 },
    { 0x55e6ca84, SUC_lexicon_tags_47 },
    { 0xd686ca86, SUC_lexicon_tags_43 },
    { 0x1566ca85, SUC_lexicon_tags_26 },
    { 0x8206ca88, SUC_lexicon_tags_14 },
    { 0x7086ca89, SUC_lexicon_tags_0 },
    { 0x37e6ca8a, SUC_lexicon_tags_17 },
    { 0xfe6ca8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa186ca90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa06ca92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf226ca98, SUC_lexicon_tags_12 },
    { 0x1a46ca99, SUC_lexicon_tags_30 },
    { 0x1d66ca9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb66ca9d, SUC_lexicon_tags_63 },
    { 0x4526ca9e, SUC_lexicon_tags_14 },
    { 0x5146ca9e, SUC_lexicon_tags_17 },
    { 0x6fc6caa0, SUC_lexicon_tags_12 },
    { 0x4046ca9f, SUC_lexicon_tags_6 },
    { 0xdca6caa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e86caa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c86caa9, SUC_lexicon_tags_116 },
    { 0xc8c6caaa, SUC_lexicon_tags_9 },
    { 0x66a6caaa, SUC_lexicon_tags_5 },
    { 0xdc06caa9, SUC_lexicon_tags_25 },
    { 0x9066caad, SUC_lexicon_tags_1 },
    { 0x3206caaa, SUC_lexicon_tags_31 },
    { 0x51c6caae, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8406cab5, SUC_lexicon_tags_10 },
    { 0x7ee6cab6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b26cabb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1186cabd, SUC_lexicon_tags_27 },
    { 0xfcc6cabe, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0xdd46cac0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3806cac2, SUC_lexicon_tags_17 },
    { 0x1946cac2, SUC_lexicon_tags_15 },
    { 0x77c6cac4, SUC_lexicon_tags_12 },
    { 0x3c86cac5, SUC_lexicon_tags_1 },
    { 0xc946cac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4646cacc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3306cace, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3546cad1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe966cad5, SUC_lexicon_tags_10 },
    { 0xf506cad5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87e6cad8, SUC_lexicon_tags_37 },
    { 0x4c46cad9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26cae1, SUC_lexicon_tags_15 },
    { 0x2ea6cae2, SUC_lexicon_tags_4 },
    { 0x7b06cae3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x326cae5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe6cae9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c6caf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf946caf3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4cc6caf5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe6cafa, SUC_lexicon_tags_1 },
    { 0xc66cafb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e6cb06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5066cb08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f06cb0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d66cb10, SUC_lexicon_tags_15 },
    { 0x3726cb10, SUC_lexicon_tags_40 },
    { 0x8a26cb12, SUC_lexicon_tags_0 },
    { 0xde6cb13, SUC_lexicon_tags_23 },
    { 0xc506cb10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6cb1b, SUC_lexicon_tags_11 },
    { 0x2c6cb1c, SUC_lexicon_tags_7 },
    { 0x2b06cb1c, SUC_lexicon_tags_2 },
    { 0x1ba6cb1e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d26cb21, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf106cb23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46cb26, SUC_lexicon_tags_55 },
    { 0x9f06cb27, SUC_lexicon_tags_11 },
    { 0x6d06cb26, SUC_lexicon_tags_29 },
    { 0x6386cb26, SUC_lexicon_tags_37 },
    { 0x4126cb2a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb046cb36, SUC_lexicon_tags_2 },
    { 0xd586cb36, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf66cb3b, SUC_lexicon_tags_2 },
    { 0xb666cb3c, SUC_lexicon_tags_8 },
    { 0x8d26cb3d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea6cb41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fa6cb43, SUC_lexicon_tags_14 },
    { 0x8ee6cb44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea6cb48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeea6cb4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5166cb4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x506cb51, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcbc6cb53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1586cb57, SUC_lexicon_tags_6 },
    { 0x4126cb58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd546cb5a, SUC_lexicon_tags_65 },
    { 0x6a86cb5b, SUC_lexicon_tags_26 },
    { 0x7e66cb5b, SUC_lexicon_tags_45 },
    { 0xc246cb5d, SUC_lexicon_tags_17 },
    { 0xcf46cb5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfc66cb60, SUC_lexicon_tags_10 },
    { 0xc206cb61, SUC_lexicon_tags_12 },
    { 0x26e6cb62, SUC_lexicon_tags_37 },
    { 0x8fa6cb63, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1606cb68, SUC_lexicon_tags_39 },
    { 0xd006cb68, SUC_lexicon_tags_15 },
    { 0x7126cb6a, SUC_lexicon_tags_91 },
    { 0x2c66cb6b, SUC_lexicon_tags_27 },
    { 0x3646cb69, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3726cb6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x606cb71, SUC_lexicon_tags_34 },
    { 0x4a06cb72, SUC_lexicon_tags_23 },
    { 0x1046cb72, SUC_lexicon_tags_23 },
    { 0xc26cb74, SUC_lexicon_tags_15 },
    { 0xdb86cb72, SUC_lexicon_tags_30 },
    { 0x4a26cb76, SUC_lexicon_tags_37 },
    { 0x1c86cb73, SUC_lexicon_tags_35 },
    { 0x7046cb71, SUC_lexicon_tags_14 },
    { 0x7ee6cb77, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2be6cb7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1966cb7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x87a6cb7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x786cb82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xde06cb84, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e6cb87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x38c6cb89, SUC_lexicon_tags_12 },
    { 0x9fa6cb89, SUC_lexicon_tags_38 },
    { 0x2d66cb8a, SUC_lexicon_tags_49 },
    { 0xe246cb8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc926cb8e, SUC_lexicon_tags_39 },
    { 0x5ce6cb8f, SUC_lexicon_tags_15 },
    { 0xa546cb90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f66cb93, SUC_lexicon_tags_27 },
    { 0x6b26cb94, SUC_lexicon_tags_37 },
    { 0x11c6cb95, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc6cb9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a6cba1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2406cba6, SUC_lexicon_tags_1 },
    { 0x9726cba7, SUC_lexicon_tags_38 },
    { 0x89c6cba8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7c6cbaa, SUC_lexicon_tags_23 },
    { 0x2826cbab, SUC_lexicon_tags_19 },
    { 0x9666cbac, SUC_lexicon_tags_6 },
    { 0x9366cbad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d86cbb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x366cbb5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3de6cbb7, SUC_lexicon_tags_2 },
    { 0xec66cbb8, SUC_lexicon_tags_37 },
    { 0xa106cbb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc166cbc2, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x87e6cbc4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6466cbc7, SUC_lexicon_tags_17 },
    { 0xb3a6cbc8, SUC_lexicon_tags_63 },
    { 0x81a6cbc9, SUC_lexicon_tags_7 },
    { 0x6006cbc9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf286cbcf, SUC_lexicon_tags_11 },
    { 0x5806cbd0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4de6cbd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a6cbd9, SUC_lexicon_tags_12 },
    { 0xea66cbda, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x926cbdc, SUC_lexicon_tags_96 },
    { 0x4c06cbdd, SUC_lexicon_tags_104 },
    { 0xecc6cbde, SUC_lexicon_tags_27 },
    { 0x9e86cbdd, SUC_lexicon_tags_2 },
    { 0xdc6cbe0, SUC_lexicon_tags_65 },
    { 0xe966cbe0, SUC_lexicon_tags_11 },
    { 0x11e6cbde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5606cbe4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd66cbe7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1746cbea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3a6cbec, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa026cbf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c6cbf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa846cbfb, SUC_lexicon_tags_45 },
    { 0xc406cbfc, SUC_lexicon_tags_32 },
    { 0x3f26cbfd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd766cc00, SUC_lexicon_tags_1 },
    { 0x93e6cc00, SUC_lexicon_tags_40 },
    { 0x4c66cc02, SUC_lexicon_tags_23 },
    { 0x7626cc01, SUC_lexicon_tags_31 },
    { 0x66a6cc04, SUC_lexicon_tags_2 },
    { 0x7bc6cc04, SUC_lexicon_tags_2 },
    { 0x6946cc05, SUC_lexicon_tags_31 },
    { 0xeb46cc07, SUC_lexicon_tags_40 },
    { 0x3f46cc06, SUC_lexicon_tags_87 },
    { 0x8766cc04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9766cc0b, SUC_lexicon_tags_68 },
    { 0xf546cc0c, SUC_lexicon_tags_63 },
    { 0xf966cc0d, SUC_lexicon_tags_1 },
    { 0xb386cc0e, SUC_lexicon_tags_8 },
    { 0x386cc0f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6246cc11, SUC_lexicon_tags_2 },
    { 0xa606cc11, SUC_lexicon_tags_10 },
    { 0xcbe6cc13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e66cc18, SUC_lexicon_tags_23 },
    { 0xf86cc19, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1986cc1b, SUC_lexicon_tags_61 },
    { 0xc1c6cc1b, SUC_lexicon_tags_33 },
    { 0xb5e6cc1d, SUC_lexicon_tags_4 },
    { 0x14a6cc1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3546cc25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3206cc28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3986cc2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb86cc2f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x506cc31, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a6cc35, SUC_lexicon_tags_10 },
    { 0x8dc6cc36, SUC_lexicon_tags_17 },
    { 0x1526cc35, SUC_lexicon_tags_34 },
    { 0x27c6cc38, SUC_lexicon_tags_7 },
    { 0x79c6cc39, SUC_lexicon_tags_29 },
    { 0x8e46cc3a, SUC_lexicon_tags_12 },
    { 0x2566cc3b, SUC_lexicon_tags_38 },
    { 0x3406cc3a, SUC_lexicon_tags_17 },
    { 0x1e06cc3c, SUC_lexicon_tags_1 },
    { 0xce6cc3e, SUC_lexicon_tags_15 },
    { 0x4f66cc3d, SUC_lexicon_tags_11 },
    { 0xcdc6cc3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd366cc42, SUC_lexicon_tags_30 },
    { 0x95c6cc43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd906cc4e, SUC_lexicon_tags_60 },
    { 0xf606cc4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6cc55, SUC_lexicon_tags_113 },
    { 0xa146cc56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf86cc5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c66cc5e, SUC_lexicon_tags_12 },
    { 0xbfc6cc5f, SUC_lexicon_tags_49 },
    { 0xb46cc5f, SUC_lexicon_tags_10 },
    { 0x6766cc5e, SUC_lexicon_tags_7 },
    { 0x5226cc60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfba6cc64, SUC_lexicon_tags_23 },
    { 0xf906cc65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc6cc6a, SUC_lexicon_tags_1 },
    { 0x52a6cc6b, SUC_lexicon_tags_275 },
    { 0x1306cc6c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7c6cc6e, SUC_lexicon_tags_84 },
    { 0x3f66cc6f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f86cc72, SUC_lexicon_tags_96 },
    { 0x9106cc73, SUC_lexicon_tags_66 },
    { 0x6c06cc74, SUC_lexicon_tags_25 },
    { 0x11a6cc75, SUC_lexicon_tags_25 },
    { 0xf246cc76, SUC_lexicon_tags_37 },
    { 0xffc6cc74, SUC_lexicon_tags_11 },
    { 0x4626cc78, SUC_lexicon_tags_28 },
    { 0xad46cc72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f46cc7b, SUC_lexicon_tags_31 },
    { 0x9426cc7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb66cc82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f46cc86, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x94c6cc88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ba6cc8a, SUC_lexicon_tags_47 },
    { 0x8e66cc8a, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0x4566cc8e, SUC_lexicon_tags_25 },
    { 0x6566cc8e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a6cc94, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d46cc98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe6cc9d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd26cca0, SUC_lexicon_tags_33 },
    { 0x7806cca0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce66cca3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d86cca5, SUC_lexicon_tags_1 },
    { 0x34a6cca6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfc46cca8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb06ccab, SUC_lexicon_tags_11 },
    { 0x9406ccac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1466ccaf, SUC_lexicon_tags_33 },
    { 0x7406ccaf, SUC_lexicon_tags_7 },
    { 0x5f46ccb0, SUC_lexicon_tags_27 },
    { 0x9b26ccb0, SUC_lexicon_tags_49 },
    { 0xb346ccb2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d06ccb5, SUC_lexicon_tags_25 },
    { 0xae66ccb6, SUC_lexicon_tags_25 },
    { 0xf806ccb7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1006ccbd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb006ccc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b86ccc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4946ccce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe166ccd2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4c6ccd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6ccdf, SUC_lexicon_tags_27 },
    { 0xccc6cce0, SUC_lexicon_tags_43 },
    { 0xe0a6cce1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf246cce3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x66cce5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e6cce8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8186cced, SUC_lexicon_tags_25 },
    { 0x6b26cced, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2a06ccf0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6386ccf2, SUC_lexicon_tags_27 },
    { 0xfba6ccf3, SUC_lexicon_tags_11 },
    { 0x8466ccf4, SUC_lexicon_tags_15 },
    { 0x4226ccf5, SUC_lexicon_tags_27 },
    { 0x6006ccf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc6ccfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6ccfd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d66ccff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae26cd05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc6cd08, SUC_lexicon_tags_61 },
    { 0x8ba6cd09, SUC_lexicon_tags_26 },
    { 0xa526cd08, SUC_lexicon_tags_37 },
    { 0x8d86cd0b, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0x1046cd0d, SUC_lexicon_tags_1 },
    { 0x64a6cd0d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc6cd14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6cd1a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a6cd1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ac6cd1f, SUC_lexicon_tags_10 },
    { 0xe706cd20, SUC_lexicon_tags_29 },
    { 0xf586cd21, SUC_lexicon_tags_1 },
    { 0xa9e6cd22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e46cd25, SUC_lexicon_tags_12 },
    { 0xd386cd26, SUC_lexicon_tags_8 },
    { 0xaf66cd26, SUC_lexicon_tags_21 },
    { 0xa926cd28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b86cd2a, SUC_lexicon_tags_37 },
    { 0xc346cd2b, SUC_lexicon_tags_37 },
    { 0x5486cd2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb606cd2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d26cd30, SUC_lexicon_tags_3 },
    { 0xce66cd31, SUC_lexicon_tags_30 },
    { 0x9246cd32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f66cd36, SUC_lexicon_tags_1 },
    { 0xd426cd36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1506cd39, SUC_lexicon_tags_27 },
    { 0xe5c6cd39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5986cd3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3066cd46, SUC_lexicon_tags_23 },
    { 0xa6e6cd47, SUC_lexicon_tags_15 },
    { 0xbea6cd48, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf226cd4c, SUC_lexicon_tags_15 },
    { 0xe686cd4c, SUC_lexicon_tags_25 },
    { 0x77c6cd4d, SUC_lexicon_tags_38 },
    { 0xfd26cd4f, SUC_lexicon_tags_2 },
    { 0x2206cd4f, SUC_lexicon_tags_1 },
    { 0xb1a6cd4f, SUC_lexicon_tags_32 },
    { 0xc906cd52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3246cd56, SUC_lexicon_tags_43 },
    { 0x28e6cd57, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c6cd5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c6cd5d, SUC_lexicon_tags_7 },
    { 0xc6c6cd5e, SUC_lexicon_tags_47 },
    { 0x53e6cd5f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b46cd63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2726cd66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x946cd68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb06cd6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe6cd72, SUC_lexicon_tags_28 },
    { 0xd646cd72, SUC_lexicon_tags_11 },
    { 0x35a6cd72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a6cd7f, SUC_lexicon_tags_37 },
    { 0x80a6cd80, SUC_lexicon_tags_15 },
    { 0xc7c6cd81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44c6cd83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81a6cd85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e6cd8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa386cd90, SUC_lexicon_tags_61 },
    { 0xb586cd91, SUC_lexicon_tags_15 },
    { 0x6206cd92, SUC_lexicon_tags_11 },
    { 0x8106cd91, SUC_lexicon_tags_15 },
    { 0xd7c6cd93, SUC_lexicon_tags_0 },
    { 0x4106cd95, SUC_lexicon_tags_15 },
    { 0xafc6cd96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd186cd9a, SUC_lexicon_tags_188 },
    { 0x5166cd9a, SUC_lexicon_tags_1 },
    { 0x2846cd9b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc6cd9f, SUC_lexicon_tags_33 },
    { 0x2226cda0, SUC_lexicon_tags_9 },
    { 0x6fe6cda0, SUC_lexicon_tags_11 },
    { 0xc686cda2, SUC_lexicon_tags_30 },
    { 0x1b66cda1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e06cda7, SUC_lexicon_tags_4 },
    { 0xa206cda7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2206cdac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7726cdae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf06cdb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7986cdb2, SUC_lexicon_tags_47 },
    { 0xe0e6cdb3, SUC_lexicon_tags_15 },
    { 0x8186cdb4, SUC_lexicon_tags_31 },
    { 0x8d86cdb5, SUC_lexicon_tags_23 },
    { 0xc426cdb2, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0xf606cdb8, SUC_lexicon_tags_15 },
    { 0x9e06cdb9, SUC_lexicon_tags_63 },
    { 0x6d86cdba, SUC_lexicon_tags_39 },
    { 0xcb06cdbb, SUC_lexicon_tags_17 },
    { 0xe4c6cdb8, SUC_lexicon_tags_17 },
    { 0xb246cdb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa6cdc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde6cdcd, SUC_lexicon_tags_13 },
    { 0xf926cdce, SUC_lexicon_tags_65 },
    { 0xe766cdcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa6cdd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5686cddc, SUC_lexicon_tags_4 },
    { 0xf46cddd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c86cde0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e66cde9, SUC_lexicon_tags_1 },
    { 0x3486cdea, SUC_lexicon_tags_56 },
    { 0x3ca6cdeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa726cded, SUC_lexicon_tags_962 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f46cdf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5206cdf5, SUC_lexicon_tags_101 },
    { 0x4c46cdf6, SUC_lexicon_tags_25 },
    { 0xfd46cdf7, SUC_lexicon_tags_63 },
    { 0x4a26cdf7, SUC_lexicon_tags_70 },
    { 0x8226cdf6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6106cdfd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9a6cdff, SUC_lexicon_tags_0 },
    { 0xec86cdff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6126ce04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a66ce0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1426ce0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef66ce0e, SUC_lexicon_tags_2 },
    { 0x83a6ce0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41e6ce11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae6ce14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6ce19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a06ce1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xee66ce1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf986ce23, SUC_lexicon_tags_6 },
    { 0x5ca6ce24, SUC_lexicon_tags_7 },
    { 0xcf86ce24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686ce2a, SUC_lexicon_tags_32 },
    { 0x31a6ce2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x706ce2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7906ce30, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc226ce37, SUC_lexicon_tags_25 },
    { 0x5146ce38, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3046ce3c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4426ce3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1526ce42, SUC_lexicon_tags_17 },
    { 0x9726ce43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd46ce45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5546ce49, SUC_lexicon_tags_1 },
    { 0x5106ce49, SUC_lexicon_tags_70 },
    { 0xe486ce4b, SUC_lexicon_tags_32 },
    { 0xec06ce49, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2786ce51, SUC_lexicon_tags_4 },
    { 0x3666ce51, SUC_lexicon_tags_15 },
    { 0xa526ce51, SUC_lexicon_tags_7 },
    { 0x92c6ce54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2566ce56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8826ce59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd06ce5e, SUC_lexicon_tags_37 },
    { 0x6486ce5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e26ce66, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6f66ce68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc6ce6b, SUC_lexicon_tags_17 },
    { 0xce06ce6c, SUC_lexicon_tags_47 },
    { 0xc6ce6c, SUC_lexicon_tags_87 },
    { 0x4ca6ce6e, SUC_lexicon_tags_29 },
    { 0xfea6ce6f, SUC_lexicon_tags_11 },
    { 0xfce6ce70, SUC_lexicon_tags_37 },
    { 0x58e6ce6c, SUC_lexicon_tags_15 },
    { 0x8f06ce72, SUC_lexicon_tags_25 },
    { 0x7b06ce72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e6ce7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc686ce7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f86ce80, SUC_lexicon_tags_37 },
    { 0x4aa6ce80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda26ce89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f66ce8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a6ce91, SUC_lexicon_tags_2 },
    { 0x4626ce92, SUC_lexicon_tags_37 },
    { 0x8326ce93, SUC_lexicon_tags_47 },
    { 0xd26ce94, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf606ce99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3866cea1, SUC_lexicon_tags_31 },
    { 0x4286cea2, SUC_lexicon_tags_45 },
    { 0xc0a6cea2, SUC_lexicon_tags_136 },
    { 0x73a6cea4, SUC_lexicon_tags_15 },
    { 0x3786cea5, SUC_lexicon_tags_61 },
    { 0x7e66cea6, SUC_lexicon_tags_56 },
    { 0x8e6cea1, SUC_lexicon_tags_1 },
    { 0xb9e6cea8, SUC_lexicon_tags_25 },
    { 0xd866cea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe6ceae, SUC_lexicon_tags_8 },
    { 0x6e86ceae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa6ceb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c6cebe, SUC_lexicon_tags_25 },
    { 0x16c6cebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd726cec1, SUC_lexicon_tags_37 },
    { 0x2666cec2, SUC_lexicon_tags_25 },
    { 0xa0c6cec3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb06cec7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc26ceca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c6cecf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a06ced3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6ced6, SUC_lexicon_tags_40 },
    { 0x4286ced6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc286ceda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc86cedf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e6cee5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe8c6cee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2886cee9, SUC_lexicon_tags_1 },
    { 0x25c6ceea, SUC_lexicon_tags_11 },
    { 0x52a6ceeb, SUC_lexicon_tags_1 },
    { 0x9606ceec, SUC_lexicon_tags_15 },
    { 0x9066ceed, SUC_lexicon_tags_55 },
    { 0xbe6ceeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc686cef0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4826cef5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb66cefb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce6ceff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7286cf05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8666cf09, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae6cf0f, SUC_lexicon_tags_45 },
    { 0xca66cf10, SUC_lexicon_tags_17 },
    { 0x3c46cf11, SUC_lexicon_tags_129 },
    { 0xb126cf12, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e26cf18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7846cf1b, SUC_lexicon_tags_7 },
    { 0x2586cf1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a6cf1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ea6cf21, SUC_lexicon_tags_25 },
    { 0x5146cf22, SUC_lexicon_tags_1 },
    { 0x7546cf23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a26cf25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x706cf28, SUC_lexicon_tags_69 },
    { 0x1a46cf29, SUC_lexicon_tags_43 },
    { 0x2446cf29, SUC_lexicon_tags_12 },
    { 0x6f66cf2b, SUC_lexicon_tags_45 },
    { 0x73a6cf2c, SUC_lexicon_tags_11 },
    { 0x2846cf2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e6cf30, SUC_lexicon_tags_25 },
    { 0x8cc6cf30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f66cf35, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa126cf38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e6cf3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7726cf3f, SUC_lexicon_tags_65 },
    { 0xc706cf40, SUC_lexicon_tags_129 },
    { 0x6fe6cf40, SUC_lexicon_tags_11 },
    { 0xa006cf42, SUC_lexicon_tags_25 },
    { 0x2106cf43, SUC_lexicon_tags_37 },
    { 0x5126cf44, SUC_lexicon_tags_37 },
    { 0xdf86cf44, SUC_lexicon_tags_27 },
    { 0xbc66cf43, SUC_lexicon_tags_15 },
    { 0x686cf3f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9586cf4e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x446cf51, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5886cf55, SUC_lexicon_tags_12 },
    { 0x9946cf56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b06cf5d, SUC_lexicon_tags_2 },
    { 0x23e6cf5e, SUC_lexicon_tags_2 },
    { 0x1a26cf5f, SUC_lexicon_tags_33 },
    { 0x7d46cf60, SUC_lexicon_tags_45 },
    { 0xf46cf60, SUC_lexicon_tags_6 },
    { 0x1b66cf5f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc486cf67, SUC_lexicon_tags_12 },
    { 0x4e06cf68, SUC_lexicon_tags_7 },
    { 0xf566cf67, SUC_lexicon_tags_17 },
    { 0x106cf68, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe326cf6c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a6cf6f, SUC_lexicon_tags_6 },
    { 0x84a6cf70, SUC_lexicon_tags_1 },
    { 0x4c06cf71, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5526cf74, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd006cf76, SUC_lexicon_tags_21 },
    { 0x3a26cf77, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa526cf7e, SUC_lexicon_tags_37 },
    { 0xf4c6cf7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc566cf81, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8746cf85, SUC_lexicon_tags_4 },
    { 0x1566cf86, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbc66cf88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f26cf8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6cf94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27e6cf96, SUC_lexicon_tags_12 },
    { 0x84a6cf96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8386cf9c, SUC_lexicon_tags_58 },
    { 0x6626cf9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66cfa3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8106cfa5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7246cfaa, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0x9f86cfac, SUC_lexicon_tags_33 },
    { 0x4566cfad, SUC_lexicon_tags_47 },
    { 0x7ac6cfad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8386cfb1, SUC_lexicon_tags_15 },
    { 0x88a6cfb2, SUC_lexicon_tags_64 },
    { 0xa266cfb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5506cfb6, SUC_lexicon_tags_4 },
    { 0x84a6cfb6, SUC_lexicon_tags_17 },
    { 0xc266cfb8, SUC_lexicon_tags_2 },
    { 0xcf06cfb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa6cfbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986cfc3, SUC_lexicon_tags_7 },
    { 0x67e6cfc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c86cfc7, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xfda6cfc9, SUC_lexicon_tags_1 },
    { 0x43a6cfca, SUC_lexicon_tags_113 },
    { 0x4a86cfc9, SUC_lexicon_tags_60 },
    { 0x6e66cfcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeae6cfce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6926cfd2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26cfd8, SUC_lexicon_tags_6 },
    { 0x87c6cfd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d86cfdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6946cfdf, SUC_lexicon_tags_25 },
    { 0xeae6cfe0, SUC_lexicon_tags_11 },
    { 0x3ae6cfe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a6cfe5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3606cfe8, SUC_lexicon_tags_47 },
    { 0x6086cfe8, SUC_lexicon_tags_66 },
    { 0xfac6cfe9, SUC_lexicon_tags_25 },
    { 0x7246cfea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1e66cfed, SUC_lexicon_tags_35 },
    { 0xb66cfee, SUC_lexicon_tags_2 },
    { 0x3a26cfef, SUC_lexicon_tags_37 },
    { 0xfe46cff0, SUC_lexicon_tags_15 },
    { 0x5246cfee, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4e66cff3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf086cff6, SUC_lexicon_tags_45 },
    { 0x9626cff6, SUC_lexicon_tags_17 },
    { 0x71a6cff8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc6cffc, SUC_lexicon_tags_46 },
    { 0x8206cffd, SUC_lexicon_tags_1 },
    { 0xe366cffe, SUC_lexicon_tags_0 },
    { 0xc766cffc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2686d004, SUC_lexicon_tags_49 },
    { 0xb06d004, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3c6d007, SUC_lexicon_tags_63 },
    { 0x7566d008, SUC_lexicon_tags_11 },
    { 0xfd46d009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d86d00e, SUC_lexicon_tags_68 },
    { 0xe1a6d00f, SUC_lexicon_tags_25 },
    { 0x5946d00f, SUC_lexicon_tags_7 },
    { 0x2926d011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe446d013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3386d016, SUC_lexicon_tags_38 },
    { 0x3646d016, SUC_lexicon_tags_7 },
    { 0xbbe6d018, SUC_lexicon_tags_15 },
    { 0x93c6d017, SUC_lexicon_tags_37 },
    { 0x6cc6d01a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e6d01d, SUC_lexicon_tags_174 },
    { 0xfa86d01e, SUC_lexicon_tags_2 },
    { 0x3186d01d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc6d024, SUC_lexicon_tags_17 },
    { 0xbc86d025, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc346d02e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7066d031, SUC_lexicon_tags_12 },
    { 0x4e26d031, SUC_lexicon_tags_9 },
    { 0x13e6d033, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9906d035, SUC_lexicon_tags_7 },
    { 0xa706d036, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d26d03c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd66d03e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac6d044, SUC_lexicon_tags_10 },
    { 0x9c86d044, SUC_lexicon_tags_37 },
    { 0xe206d046, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xcc26d048, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5386d04a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da6d04e, SUC_lexicon_tags_77 },
    { 0x3ca6d04e, SUC_lexicon_tags_18 },
    { 0xc746d050, SUC_lexicon_tags_12 },
    { 0x6c86d051, SUC_lexicon_tags_26 },
    { 0x5f06d050, SUC_lexicon_tags_32 },
    { 0x6e6d052, SUC_lexicon_tags_11 },
    { 0xc7a6d054, SUC_lexicon_tags_41 },
    { 0x52a6d054, SUC_lexicon_tags_17 },
    { 0x68a6d056, SUC_lexicon_tags_11 },
    { 0x3086d057, SUC_lexicon_tags_38 },
    { 0xe46d058, SUC_lexicon_tags_15 },
    { 0x30a6d058, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4266d061, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c6d064, SUC_lexicon_tags_17 },
    { 0x1ea6d065, SUC_lexicon_tags_7 },
    { 0x6286d066, SUC_lexicon_tags_37 },
    { 0x4e66d067, SUC_lexicon_tags_1 },
    { 0x62c6d068, SUC_lexicon_tags_17 },
    { 0x5c46d069, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c6d06e, SUC_lexicon_tags_2 },
    { 0x53e6d06f, SUC_lexicon_tags_30 },
    { 0xee06d070, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6706d072, SUC_lexicon_tags_17 },
    { 0xfdc6d072, SUC_lexicon_tags_68 },
    { 0x6bc6d073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da6d077, SUC_lexicon_tags_33 },
    { 0x13a6d078, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5966d07b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5066d080, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x386d082, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d26d084, SUC_lexicon_tags_15 },
    { 0xae66d085, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a66d08a, SUC_lexicon_tags_43 },
    { 0xea66d08b, SUC_lexicon_tags_37 },
    { 0x7c46d08b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6446d08f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac86d097, SUC_lexicon_tags_17 },
    { 0x7cc6d098, SUC_lexicon_tags_28 },
    { 0x166d099, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xed46d09c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5186d0a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf46d0af, SUC_lexicon_tags_30 },
    { 0xdac6d0b0, SUC_lexicon_tags_79 },
    { 0x68e6d0b1, SUC_lexicon_tags_34 },
    { 0x43a6d0b1, SUC_lexicon_tags_47 },
    { 0xde6d0b3, SUC_lexicon_tags_26 },
    { 0x9306d0b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe306d0b6, SUC_lexicon_tags_15 },
    { 0x2c46d0b6, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xee46d0b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e06d0bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b06d0c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc6d0c9, SUC_lexicon_tags_11 },
    { 0x4fc6d0ca, SUC_lexicon_tags_27 },
    { 0xe786d0c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3866d0d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c6d0d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6506d0dc, SUC_lexicon_tags_11 },
    { 0xc506d0dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a6d0e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7786d0e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5126d0f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc666d0f3, SUC_lexicon_tags_11 },
    { 0x42c6d0f3, SUC_lexicon_tags_39 },
    { 0x3e26d0f5, SUC_lexicon_tags_1 },
    { 0x6e86d0f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7526d0f8, SUC_lexicon_tags_12 },
    { 0xaf06d0f9, SUC_lexicon_tags_25 },
    { 0xa3a6d0f8, SUC_lexicon_tags_33 },
    { 0x37c6d0fb, SUC_lexicon_tags_19 },
    { 0x2a26d0fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf46d100, SUC_lexicon_tags_25 },
    { 0xec66d101, SUC_lexicon_tags_11 },
    { 0xff26d102, SUC_lexicon_tags_37 },
    { 0xd866d100, SUC_lexicon_tags_14 },
    { 0xb686d104, SUC_lexicon_tags_14 },
    { 0xae6d103, SUC_lexicon_tags_1 },
    { 0x8ec6d101, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf06d108, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae6d10e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8686d113, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a6d119, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb026d123, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe146d125, SUC_lexicon_tags_37 },
    { 0x1e46d126, SUC_lexicon_tags_12 },
    { 0xb726d127, SUC_lexicon_tags_15 },
    { 0x8446d128, SUC_lexicon_tags_820 },
    { 0, NULL },
    { 0, NULL },
    { 0x3406d12b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f66d12f, SUC_lexicon_tags_25 },
    { 0x3406d12f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbca6d132, SUC_lexicon_tags_202 },
    { 0x1e46d133, SUC_lexicon_tags_37 },
    { 0x26e6d134, SUC_lexicon_tags_11 },
    { 0x25c6d133, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde06d139, SUC_lexicon_tags_25 },
    { 0x1286d13a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb226d13d, SUC_lexicon_tags_92 },
    { 0x3a06d13d, SUC_lexicon_tags_15 },
    { 0x2c6d13d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa426d141, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c46d144, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a86d14c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6d157, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba6d15a, SUC_lexicon_tags_27 },
    { 0x7a6d15b, SUC_lexicon_tags_9 },
    { 0xdf66d15c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb7e6d15e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc6d161, SUC_lexicon_tags_45 },
    { 0x1be6d162, SUC_lexicon_tags_38 },
    { 0x2b86d163, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c6d166, SUC_lexicon_tags_1 },
    { 0x9d86d167, SUC_lexicon_tags_1 },
    { 0x8be6d168, SUC_lexicon_tags_25 },
    { 0x4226d169, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x1ac6d16b, SUC_lexicon_tags_17 },
    { 0xb3a6d16b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf206d16f, SUC_lexicon_tags_11 },
    { 0x99a6d170, SUC_lexicon_tags_7 },
    { 0x4926d171, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c6d178, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9e6d17a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa026d180, SUC_lexicon_tags_27 },
    { 0xcc26d181, SUC_lexicon_tags_15 },
    { 0x3c66d180, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8566d184, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a6d189, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2106d18b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e6d190, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9da6d192, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6fa6d194, SUC_lexicon_tags_37 },
    { 0x3146d194, SUC_lexicon_tags_37 },
    { 0xf546d196, SUC_lexicon_tags_14 },
    { 0xa146d197, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b26d19e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd486d1a2, SUC_lexicon_tags_60 },
    { 0x406d1a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4766d1a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa946d1a9, SUC_lexicon_tags_8 },
    { 0x7046d1aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c6d1af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c6d1b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26d1b8, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xe526d1bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e6d1c5, SUC_lexicon_tags_35 },
    { 0x2e26d1c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c46d1cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5726d1cd, SUC_lexicon_tags_19 },
    { 0xfa86d1ce, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a26d1d1, SUC_lexicon_tags_17 },
    { 0xd946d1d2, SUC_lexicon_tags_11 },
    { 0x1866d1d1, SUC_lexicon_tags_25 },
    { 0x56a6d1d2, SUC_lexicon_tags_747 },
    { 0x25e6d1d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a6d1da, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0xb266d1dc, SUC_lexicon_tags_37 },
    { 0xb26d1dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c6d1e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f06d1ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcc86d1f1, SUC_lexicon_tags_107 },
    { 0xb46d1f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2906d1f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d46d1fc, SUC_lexicon_tags_6 },
    { 0xef46d1fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x57a6d1ff, SUC_lexicon_tags_26 },
    { 0xf6e6d200, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfc6d202, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a26d205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3186d20b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec6d20e, SUC_lexicon_tags_47 },
    { 0xcc06d20e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9906d211, SUC_lexicon_tags_37 },
    { 0xb2e6d211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d215, SUC_lexicon_tags_27 },
    { 0xe1a6d216, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xba46d219, SUC_lexicon_tags_17 },
    { 0xc446d219, SUC_lexicon_tags_32 },
    { 0xa346d21b, SUC_lexicon_tags_15 },
    { 0x1a66d21c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7926d226, SUC_lexicon_tags_47 },
    { 0x56a6d227, SUC_lexicon_tags_17 },
    { 0x8786d227, SUC_lexicon_tags_21 },
    { 0x9606d227, SUC_lexicon_tags_37 },
    { 0xe666d22a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe446d22f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x366d232, SUC_lexicon_tags_11 },
    { 0x8066d233, SUC_lexicon_tags_23 },
    { 0xb6a6d234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1366d238, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de6d23c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdf46d23e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e6d243, SUC_lexicon_tags_1 },
    { 0x9426d243, SUC_lexicon_tags_7 },
    { 0x5ac6d245, SUC_lexicon_tags_37 },
    { 0x9de6d243, SUC_lexicon_tags_26 },
    { 0x5bc6d243, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc86d24a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5246d24e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe446d254, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166d258, SUC_lexicon_tags_60 },
    { 0x3f46d258, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a6d25d, SUC_lexicon_tags_4 },
    { 0x3906d25e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xed66d260, SUC_lexicon_tags_12 },
    { 0xf426d261, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa946d268, SUC_lexicon_tags_16 },
    { 0xed46d268, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3ac6d26b, SUC_lexicon_tags_60 },
    { 0x7846d26c, SUC_lexicon_tags_7 },
    { 0xe086d26d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46d273, SUC_lexicon_tags_37 },
    { 0x8286d273, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb06d278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e66d27b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5646d27d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe246d27f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa266d281, SUC_lexicon_tags_16 },
    { 0x6846d282, SUC_lexicon_tags_7 },
    { 0xeba6d283, SUC_lexicon_tags_17 },
    { 0x5166d282, SUC_lexicon_tags_1 },
    { 0xb266d285, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1e86d287, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d66d28a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c26d292, SUC_lexicon_tags_29 },
    { 0x6726d293, SUC_lexicon_tags_28 },
    { 0x25a6d292, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9786d298, SUC_lexicon_tags_1 },
    { 0x86e6d298, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e66d29c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf106d2a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf186d2a2, SUC_lexicon_tags_14 },
    { 0x8886d2a3, SUC_lexicon_tags_17 },
    { 0xa766d2a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e6d2aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4486d2ac, SUC_lexicon_tags_2 },
    { 0xc226d2ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee6d2b5, SUC_lexicon_tags_15 },
    { 0xaf86d2b6, SUC_lexicon_tags_8 },
    { 0xb6e6d2b5, SUC_lexicon_tags_17 },
    { 0xab06d2b8, SUC_lexicon_tags_7 },
    { 0x6cc6d2b7, SUC_lexicon_tags_12 },
    { 0x4be6d2ba, SUC_lexicon_tags_11 },
    { 0x7226d2bb, SUC_lexicon_tags_26 },
    { 0x45e6d2bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb586d2c0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7946d2c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4666d2c8, SUC_lexicon_tags_47 },
    { 0xec66d2c9, SUC_lexicon_tags_6 },
    { 0x3ea6d2c9, SUC_lexicon_tags_37 },
    { 0x7fc6d2cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d26d2cd, SUC_lexicon_tags_0 },
    { 0xf7c6d2cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x806d2d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x566d2d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc626d2d7, SUC_lexicon_tags_25 },
    { 0x14e6d2d8, SUC_lexicon_tags_28 },
    { 0xec66d2d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c6d2dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8446d2df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c6d2e3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd806d2eb, SUC_lexicon_tags_7 },
    { 0xd9a6d2eb, SUC_lexicon_tags_12 },
    { 0xda86d2eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb746d2f0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c6d2f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca6d2fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8246d2fc, SUC_lexicon_tags_25 },
    { 0x6c06d2fd, SUC_lexicon_tags_14 },
    { 0xaf46d2fe, SUC_lexicon_tags_16 },
    { 0xebe6d2fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c66d303, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8226d306, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe466d30a, SUC_lexicon_tags_49 },
    { 0xa486d30b, SUC_lexicon_tags_30 },
    { 0x2c06d30c, SUC_lexicon_tags_14 },
    { 0x7886d30d, SUC_lexicon_tags_25 },
    { 0x8646d30d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3226d311, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a6d314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a6d319, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ce6d31b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30e6d31d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4566d31f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb26d327, SUC_lexicon_tags_28 },
    { 0x5886d328, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a46d32d, SUC_lexicon_tags_4 },
    { 0x7346d32e, SUC_lexicon_tags_65 },
    { 0xdfa6d32f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a86d332, SUC_lexicon_tags_50 },
    { 0xb246d333, SUC_lexicon_tags_26 },
    { 0x8046d334, SUC_lexicon_tags_25 },
    { 0xaa06d335, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3586d33a, SUC_lexicon_tags_12 },
    { 0xf786d33b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6326d33d, SUC_lexicon_tags_45 },
    { 0x41e6d33d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xe2e6d340, SUC_lexicon_tags_23 },
    { 0x96c6d341, SUC_lexicon_tags_7 },
    { 0xcc6d341, SUC_lexicon_tags_25 },
    { 0x2a06d343, SUC_lexicon_tags_17 },
    { 0x8e06d343, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcdc6d346, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x45e6d348, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd66d34d, SUC_lexicon_tags_49 },
    { 0xd946d34d, SUC_lexicon_tags_7 },
    { 0xdc46d34e, SUC_lexicon_tags_15 },
    { 0x4826d34d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7466d352, SUC_lexicon_tags_45 },
    { 0xf506d353, SUC_lexicon_tags_87 },
    { 0xc966d352, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb266d357, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc446d360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda66d365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b46d367, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7426d369, SUC_lexicon_tags_15 },
    { 0xdce6d36a, SUC_lexicon_tags_15 },
    { 0xea06d36b, SUC_lexicon_tags_117 },
    { 0xde26d36a, SUC_lexicon_tags_56 },
    { 0xb9e6d36d, SUC_lexicon_tags_29 },
    { 0x8606d369, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72c6d370, SUC_lexicon_tags_11 },
    { 0xeb46d371, SUC_lexicon_tags_1 },
    { 0xcc66d372, SUC_lexicon_tags_17 },
    { 0xc26d373, SUC_lexicon_tags_17 },
    { 0x7b86d373, SUC_lexicon_tags_16 },
    { 0xfc6d371, SUC_lexicon_tags_13 },
    { 0x33a6d376, SUC_lexicon_tags_27 },
    { 0x366d377, SUC_lexicon_tags_6 },
    { 0x1066d378, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc6d37f, SUC_lexicon_tags_5 },
    { 0xf746d380, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe26d384, SUC_lexicon_tags_23 },
    { 0xc0e6d385, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c6d38a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde6d38d, SUC_lexicon_tags_26 },
    { 0x5d86d38d, SUC_lexicon_tags_84 },
    { 0xa5c6d38f, SUC_lexicon_tags_4 },
    { 0xfac6d38e, SUC_lexicon_tags_107 },
    { 0xd346d391, SUC_lexicon_tags_1 },
    { 0x7586d392, SUC_lexicon_tags_7 },
    { 0x3146d392, SUC_lexicon_tags_2 },
    { 0xef06d394, SUC_lexicon_tags_2 },
    { 0xbd46d395, SUC_lexicon_tags_74 },
    { 0x3e66d396, SUC_lexicon_tags_23 },
    { 0xfe26d396, SUC_lexicon_tags_7 },
    { 0xf66d398, SUC_lexicon_tags_27 },
    { 0xfc86d394, SUC_lexicon_tags_7 },
    { 0x8246d393, SUC_lexicon_tags_25 },
    { 0x1726d39b, SUC_lexicon_tags_11 },
    { 0x8c46d399, SUC_lexicon_tags_6 },
    { 0x8ae6d39c, SUC_lexicon_tags_60 },
    { 0x6926d394, SUC_lexicon_tags_61 },
    { 0xade6d39f, SUC_lexicon_tags_49 },
    { 0x3d86d3a0, SUC_lexicon_tags_19 },
    { 0xbda6d393, SUC_lexicon_tags_15 },
    { 0x4706d393, SUC_lexicon_tags_37 },
    { 0x80c6d394, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7546d3a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa086d3aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc6d3ad, SUC_lexicon_tags_27 },
    { 0x3b46d3ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea6d3b1, SUC_lexicon_tags_15 },
    { 0xd206d3b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xca26d3b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e66d3b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe086d3c0, SUC_lexicon_tags_28 },
    { 0x3a06d3c1, SUC_lexicon_tags_8 },
    { 0x83e6d3c2, SUC_lexicon_tags_2 },
    { 0x1fe6d3c2, SUC_lexicon_tags_12 },
    { 0xd726d3c1, SUC_lexicon_tags_15 },
    { 0x7346d3c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf86d3c7, SUC_lexicon_tags_25 },
    { 0x1e6d3c8, SUC_lexicon_tags_47 },
    { 0x1b26d3c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2a66d3cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae66d3cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec46d3d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x646d3d4, SUC_lexicon_tags_25 },
    { 0x9906d3d5, SUC_lexicon_tags_15 },
    { 0xc46d3d6, SUC_lexicon_tags_38 },
    { 0xaa06d3d7, SUC_lexicon_tags_49 },
    { 0xc486d3d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2306d3dd, SUC_lexicon_tags_30 },
    { 0x5a6d3de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd086d3e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd86d3e4, SUC_lexicon_tags_39 },
    { 0x2766d3e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e26d3e8, SUC_lexicon_tags_34 },
    { 0x7426d3e9, SUC_lexicon_tags_9 },
    { 0x1926d3e9, SUC_lexicon_tags_37 },
    { 0xa026d3e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e6d3f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b06d3f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a06d3f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc6d3f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa326d3fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506d3ff, SUC_lexicon_tags_34 },
    { 0xc406d3ff, SUC_lexicon_tags_17 },
    { 0xae66d401, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa266d404, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc266d406, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29c6d408, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2c6d40a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e6d40c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106d411, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6d418, SUC_lexicon_tags_46 },
    { 0x3846d419, SUC_lexicon_tags_46 },
    { 0x1246d418, SUC_lexicon_tags_26 },
    { 0xdd26d41b, SUC_lexicon_tags_7 },
    { 0xe446d419, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae6d420, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5386d422, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b26d425, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae6d428, SUC_lexicon_tags_26 },
    { 0x4cc6d429, SUC_lexicon_tags_37 },
    { 0xbc26d42a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e6d42f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e6d43b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x53c6d43d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf526d440, SUC_lexicon_tags_45 },
    { 0xe026d440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b26d443, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec6d447, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4086d449, SUC_lexicon_tags_15 },
    { 0x74a6d44a, SUC_lexicon_tags_45 },
    { 0xb206d44a, SUC_lexicon_tags_11 },
    { 0x8826d44c, SUC_lexicon_tags_19 },
    { 0xd366d44d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa666d44f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6d453, SUC_lexicon_tags_2 },
    { 0xb3a6d454, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa906d459, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c6d45d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c6d464, SUC_lexicon_tags_1 },
    { 0x7706d465, SUC_lexicon_tags_7 },
    { 0x5c86d465, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7da6d468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe226d46e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c66d473, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a6d477, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae6d47b, SUC_lexicon_tags_25 },
    { 0xb4e6d47c, SUC_lexicon_tags_29 },
    { 0x7586d47d, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a86d481, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc6d487, SUC_lexicon_tags_26 },
    { 0x1006d488, SUC_lexicon_tags_2 },
    { 0xfd06d489, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa6d48c, SUC_lexicon_tags_11 },
    { 0xab66d48d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c6d48f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a06d493, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6d49b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6d49e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8026d4a0, SUC_lexicon_tags_31 },
    { 0xb626d4a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd186d4a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7446d4a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be6d4ab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4866d4ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d26d4b3, SUC_lexicon_tags_7 },
    { 0x7846d4b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a6d4b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x87a6d4ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b86d4c2, SUC_lexicon_tags_27 },
    { 0xb526d4c3, SUC_lexicon_tags_17 },
    { 0x5f46d4c4, SUC_lexicon_tags_25 },
    { 0xe2a6d4c5, SUC_lexicon_tags_1 },
    { 0xfb66d4c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd06d4ca, SUC_lexicon_tags_37 },
    { 0xbba6d4cb, SUC_lexicon_tags_10 },
    { 0xb8c6d4cc, SUC_lexicon_tags_15 },
    { 0x406d4ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0c6d4cf, SUC_lexicon_tags_63 },
    { 0xb466d4d0, SUC_lexicon_tags_26 },
    { 0x8ac6d4d1, SUC_lexicon_tags_2 },
    { 0x5ee6d4d2, SUC_lexicon_tags_37 },
    { 0xd866d4d0, SUC_lexicon_tags_3 },
    { 0x3cc6d4d2, SUC_lexicon_tags_23 },
    { 0xb586d4d5, SUC_lexicon_tags_23 },
    { 0x4546d4d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd886d4dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe366d4df, SUC_lexicon_tags_15 },
    { 0x90a6d4e0, SUC_lexicon_tags_37 },
    { 0x7646d4e1, SUC_lexicon_tags_28 },
    { 0xf9a6d4e2, SUC_lexicon_tags_13 },
    { 0xba06d4e3, SUC_lexicon_tags_60 },
    { 0xd926d4e2, SUC_lexicon_tags_12 },
    { 0xe406d4e3, SUC_lexicon_tags_16 },
    { 0xa746d4e5, SUC_lexicon_tags_17 },
    { 0xd686d4e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa486d4e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d06d4ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c6d4ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x72a6d4f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcea6d4f3, SUC_lexicon_tags_2 },
    { 0xc766d4f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe206d4f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ec6d4f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8546d4fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda06d500, SUC_lexicon_tags_296 },
    { 0xc66d501, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4366d504, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a06d509, SUC_lexicon_tags_11 },
    { 0x7446d50a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda6d512, SUC_lexicon_tags_28 },
    { 0xf806d513, SUC_lexicon_tags_2 },
    { 0x3466d513, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4446d518, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8006d51a, SUC_lexicon_tags_12 },
    { 0x9e66d51b, SUC_lexicon_tags_27 },
    { 0x4766d51a, SUC_lexicon_tags_11 },
    { 0x3986d51c, SUC_lexicon_tags_25 },
    { 0x2f66d51b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3866d520, SUC_lexicon_tags_8 },
    { 0xe386d520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a66d525, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8e6d527, SUC_lexicon_tags_15 },
    { 0x4c6d528, SUC_lexicon_tags_16 },
    { 0x3b66d529, SUC_lexicon_tags_23 },
    { 0x96c6d52a, SUC_lexicon_tags_908 },
    { 0xfa46d52b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3826d52d, SUC_lexicon_tags_7 },
    { 0x4726d52d, SUC_lexicon_tags_46 },
    { 0x7c06d52d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a6d532, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a26d537, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f86d53f, SUC_lexicon_tags_10 },
    { 0xb026d540, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d543, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c6d546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2c6d548, SUC_lexicon_tags_49 },
    { 0x4626d548, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x826d54b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a6d552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6d556, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac26d562, SUC_lexicon_tags_38 },
    { 0xd1e6d563, SUC_lexicon_tags_47 },
    { 0x9d66d564, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa6d56a, SUC_lexicon_tags_45 },
    { 0x226d56a, SUC_lexicon_tags_11 },
    { 0x6d56c, SUC_lexicon_tags_15 },
    { 0x6826d56a, SUC_lexicon_tags_61 },
    { 0xe506d56e, SUC_lexicon_tags_15 },
    { 0x5fc6d56e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4b06d571, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd166d576, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x4906d579, SUC_lexicon_tags_4 },
    { 0x5266d57a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e6d57e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e6d581, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9686d586, SUC_lexicon_tags_62 },
    { 0xdd66d586, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c6d58f, SUC_lexicon_tags_3 },
    { 0x7e06d590, SUC_lexicon_tags_63 },
    { 0x4466d591, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a6d594, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da6d59b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x186d59e, SUC_lexicon_tags_23 },
    { 0xe006d59e, SUC_lexicon_tags_37 },
    { 0x91c6d59f, SUC_lexicon_tags_7 },
    { 0x26c6d5a1, SUC_lexicon_tags_70 },
    { 0xe626d5a2, SUC_lexicon_tags_34 },
    { 0xfcc6d59e, SUC_lexicon_tags_1 },
    { 0x4446d5a4, SUC_lexicon_tags_63 },
    { 0x7046d5a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e46d5b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06d5b3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa26d5b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90e6d5ba, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb66d5c1, SUC_lexicon_tags_17 },
    { 0x8666d5c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x65a6d5c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe466d5c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa266d5c9, SUC_lexicon_tags_19 },
    { 0x85c6d5c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c6d5cf, SUC_lexicon_tags_9 },
    { 0x8fa6d5cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe266d5d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf166d5d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf766d5dc, SUC_lexicon_tags_2 },
    { 0x9086d5dd, SUC_lexicon_tags_7 },
    { 0x6d26d5dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4726d5e1, SUC_lexicon_tags_15 },
    { 0xd66d5e1, SUC_lexicon_tags_15 },
    { 0x186d5e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8626d5e5, SUC_lexicon_tags_68 },
    { 0xb626d5e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae6d5ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e6d5ef, SUC_lexicon_tags_15 },
    { 0x2aa6d5f0, SUC_lexicon_tags_15 },
    { 0xc9a6d5ef, SUC_lexicon_tags_12 },
    { 0xef06d5f1, SUC_lexicon_tags_14 },
    { 0xd786d5f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4686d5f5, SUC_lexicon_tags_12 },
    { 0x99a6d5f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8ce6d5f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc6d5fd, SUC_lexicon_tags_6 },
    { 0x30a6d5fe, SUC_lexicon_tags_17 },
    { 0x6746d5ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9106d604, SUC_lexicon_tags_37 },
    { 0xb4a6d605, SUC_lexicon_tags_37 },
    { 0x3f66d604, SUC_lexicon_tags_2 },
    { 0xdec6d605, SUC_lexicon_tags_57 },
    { 0x38a6d605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa266d60b, SUC_lexicon_tags_15 },
    { 0x9026d60c, SUC_lexicon_tags_11 },
    { 0xba06d60d, SUC_lexicon_tags_45 },
    { 0xf646d60e, SUC_lexicon_tags_92 },
    { 0x60a6d60b, SUC_lexicon_tags_7 },
    { 0xb8a6d610, SUC_lexicon_tags_43 },
    { 0x4e46d60c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c6d614, SUC_lexicon_tags_35 },
    { 0xbe06d614, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e6d618, SUC_lexicon_tags_25 },
    { 0x16c6d619, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5086d61b, SUC_lexicon_tags_17 },
    { 0x5266d61c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea6d61f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b86d622, SUC_lexicon_tags_25 },
    { 0x6da6d623, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8a86d625, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a6d629, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52a6d62b, SUC_lexicon_tags_17 },
    { 0x92a6d62b, SUC_lexicon_tags_37 },
    { 0x5066d62b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc566d633, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd306d635, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa326d63a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4486d63e, SUC_lexicon_tags_1 },
    { 0x9706d63f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b46d644, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8486d646, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb0e6d648, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4426d64b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x87a6d64d, SUC_lexicon_tags_17 },
    { 0xe9a6d64d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc6d652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a6d655, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd646d65b, SUC_lexicon_tags_1 },
    { 0xd66d65c, SUC_lexicon_tags_1 },
    { 0x47e6d65c, SUC_lexicon_tags_29 },
    { 0x9066d65b, SUC_lexicon_tags_2 },
    { 0xc486d65e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1a06d661, SUC_lexicon_tags_15 },
    { 0x5946d661, SUC_lexicon_tags_23 },
    { 0xb8a6d663, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb1e6d665, SUC_lexicon_tags_50 },
    { 0x6fe6d666, SUC_lexicon_tags_1464 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e26d669, SUC_lexicon_tags_96 },
    { 0x6466d66a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6d673, SUC_lexicon_tags_27 },
    { 0xac86d674, SUC_lexicon_tags_84 },
    { 0x1aa6d675, SUC_lexicon_tags_17 },
    { 0x5106d675, SUC_lexicon_tags_7 },
    { 0x4686d673, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce86d67a, SUC_lexicon_tags_3 },
    { 0x2c6d67b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1a6d67d, SUC_lexicon_tags_49 },
    { 0xb746d67e, SUC_lexicon_tags_35 },
    { 0xa546d67d, SUC_lexicon_tags_43 },
    { 0xaa26d680, SUC_lexicon_tags_7 },
    { 0x1806d681, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe646d685, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2046d68e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26d692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xace6d694, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd226d697, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b46d699, SUC_lexicon_tags_25 },
    { 0x8f46d69a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xee66d69c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e86d69e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf646d6a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb86d6ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5fa6d6ad, SUC_lexicon_tags_1 },
    { 0x57a6d6ae, SUC_lexicon_tags_63 },
    { 0xd4a6d6af, SUC_lexicon_tags_11 },
    { 0xe206d6ae, SUC_lexicon_tags_27 },
    { 0x126d6b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc066d6b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe566d6b5, SUC_lexicon_tags_2 },
    { 0xc806d6b5, SUC_lexicon_tags_11 },
    { 0xcd46d6b7, SUC_lexicon_tags_27 },
    { 0x1dc6d6b8, SUC_lexicon_tags_27 },
    { 0x35c6d6b5, SUC_lexicon_tags_83 },
    { 0xeba6d6ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c66d6c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5866d6c4, SUC_lexicon_tags_1 },
    { 0x1446d6c5, SUC_lexicon_tags_17 },
    { 0x78a6d6c6, SUC_lexicon_tags_26 },
    { 0x9886d6c7, SUC_lexicon_tags_11 },
    { 0xda66d6c7, SUC_lexicon_tags_47 },
    { 0xb826d6c7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a6d6cc, SUC_lexicon_tags_26 },
    { 0xed46d6cd, SUC_lexicon_tags_43 },
    { 0xb566d6cc, SUC_lexicon_tags_11 },
    { 0xf6e6d6ce, SUC_lexicon_tags_33 },
    { 0x6126d6cf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3aa6d6d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddc6d6d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8846d6d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa906d6db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef86d6e1, SUC_lexicon_tags_10 },
    { 0x6f46d6e2, SUC_lexicon_tags_12 },
    { 0x8066d6e3, SUC_lexicon_tags_15 },
    { 0x3706d6e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfaa6d6e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc6d6f2, SUC_lexicon_tags_15 },
    { 0x76c6d6f3, SUC_lexicon_tags_5 },
    { 0x8006d6f3, SUC_lexicon_tags_7 },
    { 0x726d6f4, SUC_lexicon_tags_15 },
    { 0x11e6d6f6, SUC_lexicon_tags_47 },
    { 0x886d6f4, SUC_lexicon_tags_23 },
    { 0x6406d6f8, SUC_lexicon_tags_28 },
    { 0xf3a6d6f9, SUC_lexicon_tags_37 },
    { 0x5da6d6f7, SUC_lexicon_tags_1 },
    { 0x96c6d6f8, SUC_lexicon_tags_12 },
    { 0x3b46d6f2, SUC_lexicon_tags_43 },
    { 0x446d6fd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7806d6ff, SUC_lexicon_tags_12 },
    { 0xfac6d700, SUC_lexicon_tags_0 },
    { 0x9286d700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10a6d703, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffa6d705, SUC_lexicon_tags_11 },
    { 0xca66d705, SUC_lexicon_tags_7 },
    { 0xfd06d707, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e6d70a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e66d70c, SUC_lexicon_tags_17 },
    { 0xe826d70d, SUC_lexicon_tags_12 },
    { 0xcc66d70e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94e6d710, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf66d715, SUC_lexicon_tags_31 },
    { 0xde46d716, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e06d71a, SUC_lexicon_tags_7 },
    { 0x35c6d71a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc6d71e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe626d725, SUC_lexicon_tags_17 },
    { 0x78e6d726, SUC_lexicon_tags_82 },
    { 0x6186d727, SUC_lexicon_tags_37 },
    { 0x97a6d725, SUC_lexicon_tags_29 },
    { 0x8386d729, SUC_lexicon_tags_47 },
    { 0x8246d72a, SUC_lexicon_tags_1 },
    { 0x49e6d729, SUC_lexicon_tags_152 },
    { 0x4526d72c, SUC_lexicon_tags_47 },
    { 0xa086d72c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae6d735, SUC_lexicon_tags_15 },
    { 0x3ce6d735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e6d739, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e6d73b, SUC_lexicon_tags_19 },
    { 0x55e6d73b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d46d740, SUC_lexicon_tags_39 },
    { 0xb3a6d741, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6c06d743, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdca6d745, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c46d748, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea06d74a, SUC_lexicon_tags_27 },
    { 0x5c6d74b, SUC_lexicon_tags_17 },
    { 0x13a6d74c, SUC_lexicon_tags_38 },
    { 0xe506d74a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf746d758, SUC_lexicon_tags_32 },
    { 0x79a6d759, SUC_lexicon_tags_11 },
    { 0xd606d75a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x48e6d75c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf626d75f, SUC_lexicon_tags_47 },
    { 0x4fc6d760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6a6d762, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf046d765, SUC_lexicon_tags_28 },
    { 0xaba6d766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e6d76c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f86d76e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a6d770, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9466d772, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc266d775, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4546d777, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c86d779, SUC_lexicon_tags_2 },
    { 0xaaa6d77a, SUC_lexicon_tags_26 },
    { 0x37c6d77b, SUC_lexicon_tags_1 },
    { 0xb266d77b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da6d77f, SUC_lexicon_tags_14 },
    { 0x8ac6d780, SUC_lexicon_tags_30 },
    { 0x28c6d780, SUC_lexicon_tags_25 },
    { 0xcc06d782, SUC_lexicon_tags_126 },
    { 0x9606d782, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1786d785, SUC_lexicon_tags_25 },
    { 0xaf06d785, SUC_lexicon_tags_32 },
    { 0x6e06d786, SUC_lexicon_tags_25 },
    { 0xc326d788, SUC_lexicon_tags_32 },
    { 0x7686d787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4046d78b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8246d795, SUC_lexicon_tags_37 },
    { 0x4266d796, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1406d79a, SUC_lexicon_tags_27 },
    { 0xce26d79a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0e6d79d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46d7a0, SUC_lexicon_tags_40 },
    { 0x1e6d7a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a6d7a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4186d7a6, SUC_lexicon_tags_88 },
    { 0xfc66d7a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa6d7af, SUC_lexicon_tags_0 },
    { 0xc2a6d7b0, SUC_lexicon_tags_11 },
    { 0xda66d7b1, SUC_lexicon_tags_12 },
    { 0x41a6d7b2, SUC_lexicon_tags_25 },
    { 0x95a6d7b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e6d7b6, SUC_lexicon_tags_65 },
    { 0x3ac6d7b7, SUC_lexicon_tags_0 },
    { 0x7be6d7b8, SUC_lexicon_tags_11 },
    { 0x2606d7b7, SUC_lexicon_tags_17 },
    { 0x606d7ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7126d7c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e6d7cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe6d7d0, SUC_lexicon_tags_60 },
    { 0xde26d7d1, SUC_lexicon_tags_11 },
    { 0x7a06d7d2, SUC_lexicon_tags_0 },
    { 0xbc26d7d3, SUC_lexicon_tags_12 },
    { 0xfd66d7d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc6d7d8, SUC_lexicon_tags_13 },
    { 0xfe86d7d8, SUC_lexicon_tags_524 },
    { 0x4c66d7da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfd46d7dc, SUC_lexicon_tags_26 },
    { 0x4086d7dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa106d7e0, SUC_lexicon_tags_6 },
    { 0xcea6d7e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca6d7e4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9366d7e8, SUC_lexicon_tags_1 },
    { 0x9346d7e9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e6d7f0, SUC_lexicon_tags_151 },
    { 0x7de6d7f0, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc6d7f5, SUC_lexicon_tags_19 },
    { 0xf1e6d7f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f06d7f9, SUC_lexicon_tags_106 },
    { 0xb9c6d7fa, SUC_lexicon_tags_0 },
    { 0x1b86d7fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe446d7fe, SUC_lexicon_tags_45 },
    { 0xa5e6d7ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe6d807, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e46d809, SUC_lexicon_tags_11 },
    { 0x89c6d809, SUC_lexicon_tags_29 },
    { 0x5f06d809, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd886d80f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2826d811, SUC_lexicon_tags_38 },
    { 0x18c6d811, SUC_lexicon_tags_29 },
    { 0x9c66d812, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa6d817, SUC_lexicon_tags_7 },
    { 0x88a6d818, SUC_lexicon_tags_19 },
    { 0x7be6d818, SUC_lexicon_tags_47 },
    { 0x25e6d81a, SUC_lexicon_tags_12 },
    { 0x1066d81b, SUC_lexicon_tags_11 },
    { 0xc526d81b, SUC_lexicon_tags_1 },
    { 0xe9a6d81b, SUC_lexicon_tags_15 },
    { 0xe966d81e, SUC_lexicon_tags_25 },
    { 0xf2a6d81f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06d823, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4766d825, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0xb946d827, SUC_lexicon_tags_68 },
    { 0x9706d828, SUC_lexicon_tags_11 },
    { 0x33e6d829, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c6d82c, SUC_lexicon_tags_17 },
    { 0x77a6d82d, SUC_lexicon_tags_920 },
    { 0x1bc6d82e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa646d831, SUC_lexicon_tags_37 },
    { 0xdc46d832, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe226d834, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5846d836, SUC_lexicon_tags_47 },
    { 0x7a26d837, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc6d83a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca6d83f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfc6d841, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2aa6d843, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da6d847, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x16a6d849, SUC_lexicon_tags_17 },
    { 0x886d849, SUC_lexicon_tags_16 },
    { 0x7506d84b, SUC_lexicon_tags_38 },
    { 0x8706d84b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee6d851, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ee6d853, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc426d856, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6c86d858, SUC_lexicon_tags_21 },
    { 0x92c6d859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c6d85c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f06d85e, SUC_lexicon_tags_60 },
    { 0x4686d85f, SUC_lexicon_tags_7 },
    { 0xf826d860, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xee26d862, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d26d865, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c46d86b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4886d870, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x41a6d872, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d26d875, SUC_lexicon_tags_23 },
    { 0x7e6d876, SUC_lexicon_tags_87 },
    { 0x6286d876, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb066d87c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e6d880, SUC_lexicon_tags_25 },
    { 0xd066d881, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8566d883, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9c6d885, SUC_lexicon_tags_49 },
    { 0xcac6d885, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a6d888, SUC_lexicon_tags_11 },
    { 0x3726d888, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc246d88e, SUC_lexicon_tags_1 },
    { 0x95a6d88f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1786d893, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8206d895, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7566d89a, SUC_lexicon_tags_12 },
    { 0xa366d89a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88c6d89d, SUC_lexicon_tags_16 },
    { 0x3ea6d89d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd026d8a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd506d8a3, SUC_lexicon_tags_1 },
    { 0x2186d8a4, SUC_lexicon_tags_15 },
    { 0xd8a6d8a5, SUC_lexicon_tags_40 },
    { 0x51a6d8a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca66d8b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xda26d8b3, SUC_lexicon_tags_15 },
    { 0x6d66d8b3, SUC_lexicon_tags_39 },
    { 0xa0c6d8b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce66d8b7, SUC_lexicon_tags_0 },
    { 0x7086d8b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7406d8c0, SUC_lexicon_tags_66 },
    { 0x2726d8c1, SUC_lexicon_tags_9 },
    { 0xbde6d8c1, SUC_lexicon_tags_4 },
    { 0x2ce6d8c1, SUC_lexicon_tags_23 },
    { 0x80e6d8c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ac6d8c6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d46d8d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a6d8d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9a6d8d8, SUC_lexicon_tags_12 },
    { 0x75e6d8d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66d8e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b26d8e5, SUC_lexicon_tags_1 },
    { 0x2686d8e6, SUC_lexicon_tags_29 },
    { 0x8a66d8e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7726d8e9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf8a6d8eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee6d8ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d86d8f3, SUC_lexicon_tags_49 },
    { 0xea6d8f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ae6d8f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5386d8f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e6d900, SUC_lexicon_tags_25 },
    { 0x5866d901, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc506d905, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x866d907, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9666d90a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1866d90f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90c6d911, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xed86d914, SUC_lexicon_tags_0 },
    { 0x8846d915, SUC_lexicon_tags_27 },
    { 0xf786d916, SUC_lexicon_tags_21 },
    { 0xab66d914, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c6d91b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c6d91f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9726d923, SUC_lexicon_tags_1 },
    { 0x4f86d924, SUC_lexicon_tags_37 },
    { 0x1cc6d925, SUC_lexicon_tags_31 },
    { 0xb6c6d926, SUC_lexicon_tags_2 },
    { 0x9d26d927, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x186d929, SUC_lexicon_tags_3 },
    { 0xe386d92a, SUC_lexicon_tags_14 },
    { 0xf7c6d92b, SUC_lexicon_tags_14 },
    { 0xae46d92a, SUC_lexicon_tags_29 },
    { 0x1146d92b, SUC_lexicon_tags_47 },
    { 0x1c66d92a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf86d932, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4f06d934, SUC_lexicon_tags_25 },
    { 0x81a6d934, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6d93c, SUC_lexicon_tags_17 },
    { 0xe5a6d93d, SUC_lexicon_tags_12 },
    { 0xdca6d93d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce6d942, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc6d94a, SUC_lexicon_tags_23 },
    { 0x4f46d94b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd486d94d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa06d950, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62a6d952, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe686d955, SUC_lexicon_tags_17 },
    { 0x96e6d956, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4706d958, SUC_lexicon_tags_26 },
    { 0xf226d959, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2006d95c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae6d962, SUC_lexicon_tags_37 },
    { 0xe566d962, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc6d966, SUC_lexicon_tags_6 },
    { 0x7666d967, SUC_lexicon_tags_37 },
    { 0xd766d968, SUC_lexicon_tags_1 },
    { 0x8e66d969, SUC_lexicon_tags_2 },
    { 0xf46d969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e6d96d, SUC_lexicon_tags_26 },
    { 0x3cc6d96e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad86d971, SUC_lexicon_tags_49 },
    { 0x6ca6d972, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c6d976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae6d979, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea6d97d, SUC_lexicon_tags_17 },
    { 0xfc6d97e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c6d983, SUC_lexicon_tags_39 },
    { 0x87a6d983, SUC_lexicon_tags_4 },
    { 0x8f26d985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a6d988, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e6d98c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae6d98f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf626d991, SUC_lexicon_tags_43 },
    { 0xe4c6d992, SUC_lexicon_tags_63 },
    { 0x13c6d993, SUC_lexicon_tags_17 },
    { 0x1486d991, SUC_lexicon_tags_789 },
    { 0, NULL },
    { 0x426d996, SUC_lexicon_tags_49 },
    { 0x606d997, SUC_lexicon_tags_1 },
    { 0xcae6d998, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5926d99a, SUC_lexicon_tags_2 },
    { 0x766d99a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4166d99e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x14c6d9a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee46d9a4, SUC_lexicon_tags_17 },
    { 0xd7a6d9a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7626d9aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd926d9ad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x77a6d9af, SUC_lexicon_tags_37 },
    { 0xc026d9af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8406d9b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf166d9b6, SUC_lexicon_tags_11 },
    { 0x6866d9b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b26d9b9, SUC_lexicon_tags_16 },
    { 0xb606d9b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a6d9bf, SUC_lexicon_tags_63 },
    { 0xfdc6d9bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a6d9c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c6d9cb, SUC_lexicon_tags_1 },
    { 0xf046d9cc, SUC_lexicon_tags_70 },
    { 0x3706d9cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c6d9cf, SUC_lexicon_tags_37 },
    { 0x7ba6d9cf, SUC_lexicon_tags_47 },
    { 0x4526d9cf, SUC_lexicon_tags_45 },
    { 0x8b86d9cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe386d9d7, SUC_lexicon_tags_68 },
    { 0x8ac6d9d8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5046d9df, SUC_lexicon_tags_23 },
    { 0x8a26d9e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3566d9e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb646d9ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a6d9f2, SUC_lexicon_tags_12 },
    { 0xbc26d9f3, SUC_lexicon_tags_6 },
    { 0x9c66d9f3, SUC_lexicon_tags_15 },
    { 0xe46d9f5, SUC_lexicon_tags_28 },
    { 0x4c26d9f5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6d9fa, SUC_lexicon_tags_59 },
    { 0x2ae6d9fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb46d9fd, SUC_lexicon_tags_15 },
    { 0x13c6d9fd, SUC_lexicon_tags_27 },
    { 0x7126d9ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6986da02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd506da06, SUC_lexicon_tags_29 },
    { 0x3dc6da07, SUC_lexicon_tags_43 },
    { 0x8266da08, SUC_lexicon_tags_15 },
    { 0x5ba6da09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf46da0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed06da0d, SUC_lexicon_tags_25 },
    { 0x6da0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c6da11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c6da14, SUC_lexicon_tags_31 },
    { 0xe006da15, SUC_lexicon_tags_37 },
    { 0xbd86da16, SUC_lexicon_tags_60 },
    { 0x85a6da16, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc6da1c, SUC_lexicon_tags_11 },
    { 0x4c6da1d, SUC_lexicon_tags_68 },
    { 0x2246da1e, SUC_lexicon_tags_7 },
    { 0xa206da1f, SUC_lexicon_tags_1 },
    { 0x2626da20, SUC_lexicon_tags_49 },
    { 0x53a6da1f, SUC_lexicon_tags_25 },
    { 0x1fe6da22, SUC_lexicon_tags_45 },
    { 0xcf06da21, SUC_lexicon_tags_37 },
    { 0xa246da24, SUC_lexicon_tags_37 },
    { 0x99c6da22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2486da2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4706da34, SUC_lexicon_tags_65 },
    { 0x8846da35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a6da39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9ae6da3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8486da3d, SUC_lexicon_tags_23 },
    { 0x7e46da3e, SUC_lexicon_tags_60 },
    { 0x5206da3d, SUC_lexicon_tags_23 },
    { 0x61a6da40, SUC_lexicon_tags_2 },
    { 0xba86da3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8da6da43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3746da47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5846da49, SUC_lexicon_tags_201 },
    { 0x9a86da4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe6da52, SUC_lexicon_tags_49 },
    { 0xa446da52, SUC_lexicon_tags_46 },
    { 0x4526da54, SUC_lexicon_tags_25 },
    { 0x9a06da54, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2d66da57, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5206da59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6886da5c, SUC_lexicon_tags_17 },
    { 0xdd46da5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de6da64, SUC_lexicon_tags_1 },
    { 0x6386da65, SUC_lexicon_tags_48 },
    { 0x4586da66, SUC_lexicon_tags_2 },
    { 0xaf06da66, SUC_lexicon_tags_59 },
    { 0x6e06da67, SUC_lexicon_tags_2 },
    { 0x25a6da66, SUC_lexicon_tags_30 },
    { 0xb266da67, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc2c6da6c, SUC_lexicon_tags_0 },
    { 0x1026da6d, SUC_lexicon_tags_64 },
    { 0xaca6da6d, SUC_lexicon_tags_18 },
    { 0x73a6da6f, SUC_lexicon_tags_11 },
    { 0xe0e6da6d, SUC_lexicon_tags_19 },
    { 0xfa46da6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfdc6da73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5406da78, SUC_lexicon_tags_70 },
    { 0x73e6da79, SUC_lexicon_tags_25 },
    { 0x5186da7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa06da7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae06da80, SUC_lexicon_tags_16 },
    { 0x7d66da81, SUC_lexicon_tags_4 },
    { 0x6f26da82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4526da8f, SUC_lexicon_tags_19 },
    { 0xcc86da90, SUC_lexicon_tags_25 },
    { 0xf986da91, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c6da94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdc6da96, SUC_lexicon_tags_47 },
    { 0xe746da96, SUC_lexicon_tags_25 },
    { 0xa486da98, SUC_lexicon_tags_11 },
    { 0xf46da99, SUC_lexicon_tags_11 },
    { 0xd06da96, SUC_lexicon_tags_79 },
    { 0xd926da9b, SUC_lexicon_tags_12 },
    { 0xeec6da9a, SUC_lexicon_tags_17 },
    { 0x2066da98, SUC_lexicon_tags_17 },
    { 0x7c86da9e, SUC_lexicon_tags_47 },
    { 0xa806da9c, SUC_lexicon_tags_7 },
    { 0xaba6da9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e6daa3, SUC_lexicon_tags_2 },
    { 0x6846daa4, SUC_lexicon_tags_37 },
    { 0xfd86daa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06daa9, SUC_lexicon_tags_27 },
    { 0x7b66daaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd366daad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf326dabb, SUC_lexicon_tags_26 },
    { 0xbe86dabc, SUC_lexicon_tags_47 },
    { 0x18e6dabc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c66dac1, SUC_lexicon_tags_37 },
    { 0x6ec6dac2, SUC_lexicon_tags_7 },
    { 0x4ac6dac3, SUC_lexicon_tags_37 },
    { 0xe746dac3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd866dac6, SUC_lexicon_tags_11 },
    { 0xf4e6dac7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d26daca, SUC_lexicon_tags_0 },
    { 0xfb46dacb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a46dacd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3206dacf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e6dad8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf5a6dada, SUC_lexicon_tags_86 },
    { 0x1886dadb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf906dae1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b86dae4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa146dae6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c66daea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8946daed, SUC_lexicon_tags_25 },
    { 0x4c66daee, SUC_lexicon_tags_21 },
    { 0x6946daed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a86daf2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c46daf4, SUC_lexicon_tags_11 },
    { 0x9fe6daf5, SUC_lexicon_tags_19 },
    { 0x5e26daf6, SUC_lexicon_tags_9 },
    { 0x8da6daf5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2c26daf9, SUC_lexicon_tags_12 },
    { 0x9c66dafa, SUC_lexicon_tags_17 },
    { 0xd6c6dafb, SUC_lexicon_tags_67 },
    { 0x9766dafb, SUC_lexicon_tags_27 },
    { 0x78c6dafd, SUC_lexicon_tags_17 },
    { 0xd646daf9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a6db04, SUC_lexicon_tags_47 },
    { 0xf9c6db04, SUC_lexicon_tags_77 },
    { 0x23a6db04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de6db0c, SUC_lexicon_tags_17 },
    { 0x8706db0d, SUC_lexicon_tags_48 },
    { 0x7846db0c, SUC_lexicon_tags_77 },
    { 0x6e06db0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5546db12, SUC_lexicon_tags_27 },
    { 0xac86db12, SUC_lexicon_tags_1 },
    { 0xcc06db13, SUC_lexicon_tags_7 },
    { 0xca6db12, SUC_lexicon_tags_57 },
    { 0xdb46db16, SUC_lexicon_tags_47 },
    { 0xa7c6db15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c6db1a, SUC_lexicon_tags_15 },
    { 0x3b66db1a, SUC_lexicon_tags_37 },
    { 0xb2e6db1c, SUC_lexicon_tags_17 },
    { 0xd066db1d, SUC_lexicon_tags_12 },
    { 0x9da6db1e, SUC_lexicon_tags_47 },
    { 0xb246db1f, SUC_lexicon_tags_8 },
    { 0xe5c6db1c, SUC_lexicon_tags_33 },
    { 0xf986db21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa86db23, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa6db28, SUC_lexicon_tags_43 },
    { 0x7126db29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc426db2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de6db31, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf126db34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f86db36, SUC_lexicon_tags_8 },
    { 0xd686db37, SUC_lexicon_tags_12 },
    { 0x6266db36, SUC_lexicon_tags_23 },
    { 0xb926db39, SUC_lexicon_tags_146 },
    { 0x4b66db3a, SUC_lexicon_tags_2 },
    { 0xdf66db38, SUC_lexicon_tags_72 },
    { 0xee26db39, SUC_lexicon_tags_60 },
    { 0x7026db3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x29c6db3f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1cc6db41, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc346db44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b66db47, SUC_lexicon_tags_9 },
    { 0x27a6db48, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca6db4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2046db52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a6db58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7326db5c, SUC_lexicon_tags_0 },
    { 0xdd86db5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446db62, SUC_lexicon_tags_25 },
    { 0x2ea6db63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea6db66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9166db6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfe46db6c, SUC_lexicon_tags_17 },
    { 0x5806db6d, SUC_lexicon_tags_11 },
    { 0x25e6db6e, SUC_lexicon_tags_47 },
    { 0x306db6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe186db73, SUC_lexicon_tags_7 },
    { 0xbb46db74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbe6db76, SUC_lexicon_tags_12 },
    { 0x23c6db77, SUC_lexicon_tags_27 },
    { 0x7fa6db78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ec6db7a, SUC_lexicon_tags_17 },
    { 0x1066db7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e6db80, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc6db83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2606db85, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca6db88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc6db93, SUC_lexicon_tags_1 },
    { 0x9486db94, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca6db97, SUC_lexicon_tags_25 },
    { 0x57e6db97, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce6db9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd006dba1, SUC_lexicon_tags_0 },
    { 0xda26dba2, SUC_lexicon_tags_43 },
    { 0x34a6dba3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6be6dba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd86dbab, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb906dbb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8146dbb5, SUC_lexicon_tags_27 },
    { 0xfa26dbb6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c86dbb9, SUC_lexicon_tags_2 },
    { 0x5a66dbba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba06dbbc, SUC_lexicon_tags_23 },
    { 0xb5c6dbbd, SUC_lexicon_tags_17 },
    { 0x2546dbbe, SUC_lexicon_tags_11 },
    { 0xdda6dbbe, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd366dbc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe726dbc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ae6dbcb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea6dbd3, SUC_lexicon_tags_1 },
    { 0xcea6dbd4, SUC_lexicon_tags_26 },
    { 0x566dbd3, SUC_lexicon_tags_4 },
    { 0xba86dbd6, SUC_lexicon_tags_0 },
    { 0x50c6dbd7, SUC_lexicon_tags_43 },
    { 0x866dbd5, SUC_lexicon_tags_31 },
    { 0x69a6dbd8, SUC_lexicon_tags_23 },
    { 0xc946dbd7, SUC_lexicon_tags_25 },
    { 0x8ce6dbd7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c6dbde, SUC_lexicon_tags_17 },
    { 0x6386dbdf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1fe6dbe1, SUC_lexicon_tags_55 },
    { 0x3106dbe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69e6dbe4, SUC_lexicon_tags_17 },
    { 0xd5c6dbe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcae6dbe7, SUC_lexicon_tags_7 },
    { 0x13c6dbe7, SUC_lexicon_tags_53 },
    { 0x8ca6dbe9, SUC_lexicon_tags_39 },
    { 0xea06dbea, SUC_lexicon_tags_12 },
    { 0x7746dbea, SUC_lexicon_tags_33 },
    { 0xabc6dbea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4c6dbee, SUC_lexicon_tags_37 },
    { 0x3366dbee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc6dbf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c66dbf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4366dbf8, SUC_lexicon_tags_30 },
    { 0x4946dbf9, SUC_lexicon_tags_158 },
    { 0x8326dbf9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c6dc05, SUC_lexicon_tags_29 },
    { 0x446dc05, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe46dc08, SUC_lexicon_tags_47 },
    { 0x50a6dc09, SUC_lexicon_tags_38 },
    { 0xf3c6dc08, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e6dc0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5386dc10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed26dc18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x946dc1a, SUC_lexicon_tags_11 },
    { 0x19e6dc1a, SUC_lexicon_tags_15 },
    { 0x9e06dc1a, SUC_lexicon_tags_30 },
    { 0xb406dc1d, SUC_lexicon_tags_28 },
    { 0x8ce6dc1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x846dc20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3366dc2c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6686dc30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf406dc33, SUC_lexicon_tags_12 },
    { 0x2446dc34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba6dc38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a26dc3b, SUC_lexicon_tags_28 },
    { 0xbc46dc3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc826dc3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66dc41, SUC_lexicon_tags_7 },
    { 0x10a6dc41, SUC_lexicon_tags_37 },
    { 0x99e6dc43, SUC_lexicon_tags_14 },
    { 0x11e6dc43, SUC_lexicon_tags_37 },
    { 0xf946dc43, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9546dc4e, SUC_lexicon_tags_1 },
    { 0xa006dc4f, SUC_lexicon_tags_43 },
    { 0x8046dc4e, SUC_lexicon_tags_29 },
    { 0x1926dc51, SUC_lexicon_tags_12 },
    { 0x17a6dc50, SUC_lexicon_tags_9 },
    { 0x2aa6dc4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe306dc55, SUC_lexicon_tags_9 },
    { 0x1006dc56, SUC_lexicon_tags_17 },
    { 0xc106dc57, SUC_lexicon_tags_15 },
    { 0x6426dc58, SUC_lexicon_tags_0 },
    { 0xc626dc56, SUC_lexicon_tags_30 },
    { 0x6f46dc5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2166dc60, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xad06dc63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4066dc65, SUC_lexicon_tags_9 },
    { 0x2e6dc66, SUC_lexicon_tags_108 },
    { 0x9c66dc66, SUC_lexicon_tags_19 },
    { 0x3766dc68, SUC_lexicon_tags_9 },
    { 0x8846dc69, SUC_lexicon_tags_7 },
    { 0x63c6dc69, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe6dc74, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb86dc78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc6dc7d, SUC_lexicon_tags_66 },
    { 0x5d86dc7e, SUC_lexicon_tags_12 },
    { 0x99e6dc7e, SUC_lexicon_tags_1 },
    { 0xda06dc80, SUC_lexicon_tags_43 },
    { 0xe9a6dc80, SUC_lexicon_tags_11 },
    { 0x5366dc82, SUC_lexicon_tags_2 },
    { 0x3ea6dc83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38a6dc85, SUC_lexicon_tags_38 },
    { 0xa46dc86, SUC_lexicon_tags_17 },
    { 0xe4e6dc86, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1786dc8b, SUC_lexicon_tags_68 },
    { 0x3f26dc8c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c6dc90, SUC_lexicon_tags_17 },
    { 0xfa6dc91, SUC_lexicon_tags_63 },
    { 0xe006dc92, SUC_lexicon_tags_15 },
    { 0x67c6dc92, SUC_lexicon_tags_11 },
    { 0x4126dc93, SUC_lexicon_tags_15 },
    { 0x6386dc94, SUC_lexicon_tags_9 },
    { 0xdf66dc96, SUC_lexicon_tags_9 },
    { 0x7b46dc92, SUC_lexicon_tags_17 },
    { 0x70e6dc98, SUC_lexicon_tags_37 },
    { 0x8866dc96, SUC_lexicon_tags_26 },
    { 0xe0c6dc9a, SUC_lexicon_tags_29 },
    { 0xf6c6dc9b, SUC_lexicon_tags_1 },
    { 0x5086dc9c, SUC_lexicon_tags_2 },
    { 0x1906dc9c, SUC_lexicon_tags_25 },
    { 0xa726dc9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c6dca1, SUC_lexicon_tags_49 },
    { 0xfb86dca2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5806dca7, SUC_lexicon_tags_29 },
    { 0x5ee6dca7, SUC_lexicon_tags_27 },
    { 0xa866dca9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c06dcac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa246dcb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a26dcb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd386dcbc, SUC_lexicon_tags_17 },
    { 0xe666dcbd, SUC_lexicon_tags_1 },
    { 0xd6a6dcbe, SUC_lexicon_tags_4 },
    { 0xaf86dcbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1086dcc3, SUC_lexicon_tags_25 },
    { 0xf86dcc4, SUC_lexicon_tags_26 },
    { 0xcb86dcc3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd106dcca, SUC_lexicon_tags_19 },
    { 0x5246dcca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5026dcce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc6dcd3, SUC_lexicon_tags_0 },
    { 0x5426dcd4, SUC_lexicon_tags_45 },
    { 0xa786dcd5, SUC_lexicon_tags_27 },
    { 0xd66dcd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf326dce0, SUC_lexicon_tags_2 },
    { 0x5ca6dce0, SUC_lexicon_tags_37 },
    { 0x8486dce2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7506dce5, SUC_lexicon_tags_63 },
    { 0x89e6dce6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa746dce8, SUC_lexicon_tags_17 },
    { 0x49e6dce9, SUC_lexicon_tags_33 },
    { 0x5b86dce9, SUC_lexicon_tags_7 },
    { 0xffe6dce8, SUC_lexicon_tags_43 },
    { 0x4666dce8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb66dcf0, SUC_lexicon_tags_23 },
    { 0xbc86dcf1, SUC_lexicon_tags_27 },
    { 0xc326dcf2, SUC_lexicon_tags_25 },
    { 0x9f46dcf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e6dcfb, SUC_lexicon_tags_2 },
    { 0x6ee6dcfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc286dcfe, SUC_lexicon_tags_11 },
    { 0x1586dcff, SUC_lexicon_tags_26 },
    { 0xa6dd00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac6dd03, SUC_lexicon_tags_1 },
    { 0xbee6dd04, SUC_lexicon_tags_2 },
    { 0x6e86dd05, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80a6dd07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6626dd0e, SUC_lexicon_tags_77 },
    { 0xd446dd0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a6dd12, SUC_lexicon_tags_4 },
    { 0x75e6dd12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b46dd16, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6086dd1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a06dd20, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc866dd22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace6dd27, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7226dd2a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdd86dd2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb426dd31, SUC_lexicon_tags_15 },
    { 0x5686dd32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e06dd3a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1ee6dd3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c6dd3e, SUC_lexicon_tags_47 },
    { 0xb26dd3e, SUC_lexicon_tags_6 },
    { 0x3546dd40, SUC_lexicon_tags_22 },
    { 0xf5e6dd41, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x62c6dd43, SUC_lexicon_tags_23 },
    { 0xdc26dd44, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9f26dd46, SUC_lexicon_tags_17 },
    { 0x6a26dd47, SUC_lexicon_tags_17 },
    { 0x9ae6dd48, SUC_lexicon_tags_64 },
    { 0xc326dd49, SUC_lexicon_tags_11 },
    { 0x2ca6dd49, SUC_lexicon_tags_46 },
    { 0x2f86dd4b, SUC_lexicon_tags_37 },
    { 0x7d46dd49, SUC_lexicon_tags_2 },
    { 0xfca6dd48, SUC_lexicon_tags_1 },
    { 0xcd26dd4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8686dd56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4926dd5d, SUC_lexicon_tags_15 },
    { 0x9886dd5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6766dd60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6126dd62, SUC_lexicon_tags_33 },
    { 0x84c6dd63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a6dd67, SUC_lexicon_tags_66 },
    { 0x11c6dd68, SUC_lexicon_tags_15 },
    { 0x4906dd69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4246dd6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x686dd71, SUC_lexicon_tags_11 },
    { 0x4d46dd72, SUC_lexicon_tags_49 },
    { 0x3a66dd73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86dd76, SUC_lexicon_tags_46 },
    { 0xfce6dd77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6446dd79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9406dd7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7486dd7f, SUC_lexicon_tags_14 },
    { 0x67a6dd80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c6dd85, SUC_lexicon_tags_37 },
    { 0xf246dd86, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x58e6dd88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd446dd8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac6dd91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6046dd94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186dd98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23e6dd9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x366dd9d, SUC_lexicon_tags_38 },
    { 0x28e6dd9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa6dda1, SUC_lexicon_tags_1 },
    { 0xbce6dda2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9626dda5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8566dda7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1366ddab, SUC_lexicon_tags_17 },
    { 0x46a6ddac, SUC_lexicon_tags_7 },
    { 0xb386ddac, SUC_lexicon_tags_2 },
    { 0x9a46ddae, SUC_lexicon_tags_7 },
    { 0x326ddac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb46ddb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5046ddb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5246ddb9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb466ddbe, SUC_lexicon_tags_11 },
    { 0xe6a6ddbf, SUC_lexicon_tags_19 },
    { 0xcba6ddbf, SUC_lexicon_tags_30 },
    { 0x4ea6ddbe, SUC_lexicon_tags_11 },
    { 0x13a6ddc2, SUC_lexicon_tags_12 },
    { 0xf906ddc0, SUC_lexicon_tags_8 },
    { 0x6066ddc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x50a6ddc6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x31e6ddc8, SUC_lexicon_tags_16 },
    { 0x7f66ddc8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x2506ddcb, SUC_lexicon_tags_47 },
    { 0xea06ddcc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xde06ddce, SUC_lexicon_tags_26 },
    { 0xd906ddcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc6ddd4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb9c6ddd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5126ddd8, SUC_lexicon_tags_2 },
    { 0x68e6ddd9, SUC_lexicon_tags_17 },
    { 0xba06ddd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c6dddf, SUC_lexicon_tags_37 },
    { 0x6586dddf, SUC_lexicon_tags_14 },
    { 0xc3e6dde1, SUC_lexicon_tags_29 },
    { 0xa1c6dde2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf306dde8, SUC_lexicon_tags_37 },
    { 0x1106dde9, SUC_lexicon_tags_7 },
    { 0xaf06ddea, SUC_lexicon_tags_19 },
    { 0xe2e6ddeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf826ddef, SUC_lexicon_tags_38 },
    { 0x746ddf0, SUC_lexicon_tags_45 },
    { 0x3f66ddf0, SUC_lexicon_tags_11 },
    { 0x3046ddf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbda6ddf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1366ddf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc346ddf9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa86ddfe, SUC_lexicon_tags_1 },
    { 0x1d46ddfe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2706de02, SUC_lexicon_tags_27 },
    { 0xb086de03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7dc6de05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c26de0a, SUC_lexicon_tags_282 },
    { 0xdac6de0a, SUC_lexicon_tags_79 },
    { 0x5706de0c, SUC_lexicon_tags_2 },
    { 0x6c26de0c, SUC_lexicon_tags_11 },
    { 0xa9e6de0e, SUC_lexicon_tags_26 },
    { 0x646de0e, SUC_lexicon_tags_17 },
    { 0x4ee6de10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x94e6de12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de6de16, SUC_lexicon_tags_11 },
    { 0xa086de17, SUC_lexicon_tags_87 },
    { 0xc1a6de18, SUC_lexicon_tags_12 },
    { 0x6566de19, SUC_lexicon_tags_1 },
    { 0xf806de18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be6de2b, SUC_lexicon_tags_33 },
    { 0x9be6de2c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca6de33, SUC_lexicon_tags_2 },
    { 0x8506de33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69a6de36, SUC_lexicon_tags_1 },
    { 0x8746de37, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd0e6de39, SUC_lexicon_tags_25 },
    { 0xa0a6de3a, SUC_lexicon_tags_17 },
    { 0x2d86de3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb06de41, SUC_lexicon_tags_14 },
    { 0x8626de42, SUC_lexicon_tags_25 },
    { 0x9b46de42, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc606de47, SUC_lexicon_tags_22 },
    { 0x3e06de47, SUC_lexicon_tags_0 },
    { 0x1e46de49, SUC_lexicon_tags_37 },
    { 0xb7e6de4a, SUC_lexicon_tags_11 },
    { 0x44e6de47, SUC_lexicon_tags_12 },
    { 0x12a6de4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc506de4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c6de52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5be6de54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd46de58, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9326de5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd566de64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad86de6c, SUC_lexicon_tags_17 },
    { 0x9946de6d, SUC_lexicon_tags_2 },
    { 0xf406de6e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd246de73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa706de79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8546de7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86de80, SUC_lexicon_tags_63 },
    { 0xca86de80, SUC_lexicon_tags_37 },
    { 0xfd06de82, SUC_lexicon_tags_4 },
    { 0x3366de82, SUC_lexicon_tags_15 },
    { 0x9546de84, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a6de8a, SUC_lexicon_tags_25 },
    { 0xfe6de8b, SUC_lexicon_tags_7 },
    { 0x98a6de8a, SUC_lexicon_tags_33 },
    { 0xe8c6de8b, SUC_lexicon_tags_11 },
    { 0x5a86de8e, SUC_lexicon_tags_17 },
    { 0x73e6de8f, SUC_lexicon_tags_8 },
    { 0x84a6de8a, SUC_lexicon_tags_68 },
    { 0xdee6de8e, SUC_lexicon_tags_17 },
    { 0xabc6de8d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xdd26de94, SUC_lexicon_tags_30 },
    { 0x7f46de95, SUC_lexicon_tags_26 },
    { 0x40a6de96, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4426de9b, SUC_lexicon_tags_25 },
    { 0x5726de9c, SUC_lexicon_tags_63 },
    { 0xd386de9d, SUC_lexicon_tags_27 },
    { 0x4a06de9d, SUC_lexicon_tags_7 },
    { 0x8126de9d, SUC_lexicon_tags_15 },
    { 0x2da6dea0, SUC_lexicon_tags_0 },
    { 0xf786dea0, SUC_lexicon_tags_68 },
    { 0x7606dea2, SUC_lexicon_tags_45 },
    { 0xc846dea3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5106dea7, SUC_lexicon_tags_37 },
    { 0x4b86dea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e6deb1, SUC_lexicon_tags_37 },
    { 0x4c46deb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e6deb5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c6deb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da6dec0, SUC_lexicon_tags_3 },
    { 0xf686dec0, SUC_lexicon_tags_27 },
    { 0x4b06dec2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b26dec7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb06dec9, SUC_lexicon_tags_60 },
    { 0xd586deca, SUC_lexicon_tags_11 },
    { 0x7266deca, SUC_lexicon_tags_11 },
    { 0x1ac6decc, SUC_lexicon_tags_15 },
    { 0xd546decc, SUC_lexicon_tags_82 },
    { 0x2186dece, SUC_lexicon_tags_17 },
    { 0xda06decf, SUC_lexicon_tags_31 },
    { 0x4206decf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf286ded5, SUC_lexicon_tags_15 },
    { 0x5f06ded6, SUC_lexicon_tags_39 },
    { 0x71c6ded7, SUC_lexicon_tags_25 },
    { 0x986ded6, SUC_lexicon_tags_17 },
    { 0xa186ded5, SUC_lexicon_tags_1 },
    { 0x386ded5, SUC_lexicon_tags_15 },
    { 0x79e6dedb, SUC_lexicon_tags_43 },
    { 0xcaa6dedc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e6dee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d06dee6, SUC_lexicon_tags_7 },
    { 0x3526dee7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9a6dee9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a46deeb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6deee, SUC_lexicon_tags_77 },
    { 0x5146deef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x986def1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f06def3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e6def5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa466def8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca6defd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd06df01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4a6df03, SUC_lexicon_tags_19 },
    { 0x3706df03, SUC_lexicon_tags_17 },
    { 0x6826df05, SUC_lexicon_tags_17 },
    { 0x2b46df04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc6df0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf66df10, SUC_lexicon_tags_25 },
    { 0xaa06df11, SUC_lexicon_tags_33 },
    { 0x27c6df12, SUC_lexicon_tags_11 },
    { 0x5ba6df13, SUC_lexicon_tags_7 },
    { 0x7686df12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d86df19, SUC_lexicon_tags_47 },
    { 0x6286df1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a6df1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c6df23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xad06df26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa6df28, SUC_lexicon_tags_3 },
    { 0x3486df28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7326df2b, SUC_lexicon_tags_12 },
    { 0x79e6df2c, SUC_lexicon_tags_60 },
    { 0x94a6df2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c6df30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c6df47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4746df4b, SUC_lexicon_tags_6 },
    { 0xd966df4c, SUC_lexicon_tags_15 },
    { 0x74a6df4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a26df50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8e6df52, SUC_lexicon_tags_3 },
    { 0xd4c6df53, SUC_lexicon_tags_8 },
    { 0x7ea6df54, SUC_lexicon_tags_63 },
    { 0x3c86df55, SUC_lexicon_tags_19 },
    { 0x65a6df56, SUC_lexicon_tags_60 },
    { 0x506df57, SUC_lexicon_tags_63 },
    { 0x96c6df57, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e6df5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ae6df5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde86df5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe6df62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd86df65, SUC_lexicon_tags_0 },
    { 0x3586df66, SUC_lexicon_tags_25 },
    { 0xcc46df66, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x12c6df69, SUC_lexicon_tags_15 },
    { 0x79a6df69, SUC_lexicon_tags_1 },
    { 0x146df6b, SUC_lexicon_tags_25 },
    { 0x2026df69, SUC_lexicon_tags_68 },
    { 0xe8c6df6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c46df70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba6df73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ae6df75, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc6df7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc346df7f, SUC_lexicon_tags_15 },
    { 0x78c6df80, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x6a06df82, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e46df86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc446df8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e46df91, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9706df93, SUC_lexicon_tags_7 },
    { 0xc7e6df94, SUC_lexicon_tags_15 },
    { 0x3926df95, SUC_lexicon_tags_63 },
    { 0xfd06df95, SUC_lexicon_tags_25 },
    { 0x7946df95, SUC_lexicon_tags_61 },
    { 0x9b86df96, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4186df9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3026df9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3106dfa0, SUC_lexicon_tags_10 },
    { 0x9146dfa0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa826dfa3, SUC_lexicon_tags_47 },
    { 0xb6e6dfa4, SUC_lexicon_tags_27 },
    { 0xa3a6dfa5, SUC_lexicon_tags_37 },
    { 0xbc66dfa5, SUC_lexicon_tags_7 },
    { 0x2846dfa7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc246dfab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2e86dfad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e6dfb0, SUC_lexicon_tags_15 },
    { 0x25c6dfb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa626dfb4, SUC_lexicon_tags_49 },
    { 0x15c6dfb4, SUC_lexicon_tags_64 },
    { 0x8206dfb6, SUC_lexicon_tags_8 },
    { 0x2a6dfb7, SUC_lexicon_tags_49 },
    { 0x5546dfb7, SUC_lexicon_tags_7 },
    { 0xe2a6dfb9, SUC_lexicon_tags_10 },
    { 0x19a6dfba, SUC_lexicon_tags_9 },
    { 0xc46dfbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca6dfc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd246dfca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8de6dfcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7546dfd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2a6dfd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b46dfd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e26dfde, SUC_lexicon_tags_8 },
    { 0xf786dfde, SUC_lexicon_tags_7 },
    { 0x4366dfdf, SUC_lexicon_tags_101 },
    { 0xf886dfe1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32a6dfe3, SUC_lexicon_tags_23 },
    { 0x6ac6dfe4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x84a6dfe6, SUC_lexicon_tags_38 },
    { 0xbfa6dfe7, SUC_lexicon_tags_37 },
    { 0x40a6dfe7, SUC_lexicon_tags_17 },
    { 0x4506dfe9, SUC_lexicon_tags_21 },
    { 0x5a6dfea, SUC_lexicon_tags_7 },
    { 0x1a6dfe9, SUC_lexicon_tags_26 },
    { 0x1de6dfec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9006dfee, SUC_lexicon_tags_25 },
    { 0xfe86dfef, SUC_lexicon_tags_27 },
    { 0xd5a6dff0, SUC_lexicon_tags_49 },
    { 0x4b06dff1, SUC_lexicon_tags_15 },
    { 0x5986dff2, SUC_lexicon_tags_6 },
    { 0xf2c6dff1, SUC_lexicon_tags_14 },
    { 0xae06dfee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8e66dff6, SUC_lexicon_tags_60 },
    { 0x5726dff6, SUC_lexicon_tags_15 },
    { 0xaba6dff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c66dffa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1006dffe, SUC_lexicon_tags_4 },
    { 0xab86dfff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e6e005, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e00a, SUC_lexicon_tags_17 },
    { 0x7be6e00b, SUC_lexicon_tags_1 },
    { 0x15c6e00b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1ce6e00e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c6e013, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9366e017, SUC_lexicon_tags_21 },
    { 0xc766e017, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1a6e01a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea26e020, SUC_lexicon_tags_15 },
    { 0x90a6e021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb786e028, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf546e02a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f86e02f, SUC_lexicon_tags_56 },
    { 0x90c6e030, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9806e035, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f06e039, SUC_lexicon_tags_10 },
    { 0xff86e039, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2c6e03c, SUC_lexicon_tags_26 },
    { 0xa186e03c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc46e045, SUC_lexicon_tags_9 },
    { 0xfe26e045, SUC_lexicon_tags_11 },
    { 0x23a6e046, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c6e04c, SUC_lexicon_tags_12 },
    { 0x866e04d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75c6e04f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d66e051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x846e053, SUC_lexicon_tags_11 },
    { 0x41e6e053, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xd546e057, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6126e059, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e6e05e, SUC_lexicon_tags_60 },
    { 0x7b06e05e, SUC_lexicon_tags_2 },
    { 0xe6e6e05e, SUC_lexicon_tags_49 },
    { 0x9426e061, SUC_lexicon_tags_37 },
    { 0x29e6e05e, SUC_lexicon_tags_12 },
    { 0x3186e063, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa6e066, SUC_lexicon_tags_27 },
    { 0xf146e067, SUC_lexicon_tags_19 },
    { 0xb266e067, SUC_lexicon_tags_2 },
    { 0x4686e066, SUC_lexicon_tags_8 },
    { 0x2946e06a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4686e06d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb366e06f, SUC_lexicon_tags_15 },
    { 0x82e6e070, SUC_lexicon_tags_29 },
    { 0x8086e071, SUC_lexicon_tags_59 },
    { 0xc3e6e071, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe7c6e074, SUC_lexicon_tags_14 },
    { 0x7366e074, SUC_lexicon_tags_83 },
    { 0x16c6e076, SUC_lexicon_tags_11 },
    { 0x6186e077, SUC_lexicon_tags_27 },
    { 0x3666e075, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6246e07d, SUC_lexicon_tags_25 },
    { 0xace6e07e, SUC_lexicon_tags_12 },
    { 0xb066e07f, SUC_lexicon_tags_25 },
    { 0xe686e07d, SUC_lexicon_tags_82 },
    { 0x6086e081, SUC_lexicon_tags_25 },
    { 0xf606e081, SUC_lexicon_tags_23 },
    { 0x1e06e083, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa06e085, SUC_lexicon_tags_11 },
    { 0xb26e086, SUC_lexicon_tags_30 },
    { 0x4a06e086, SUC_lexicon_tags_62 },
    { 0x5ce6e088, SUC_lexicon_tags_25 },
    { 0xd366e087, SUC_lexicon_tags_45 },
    { 0xe746e08a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d86e08e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe6e095, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e6e099, SUC_lexicon_tags_30 },
    { 0x3e26e099, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde86e0a0, SUC_lexicon_tags_54 },
    { 0x9486e0a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdce6e0a3, SUC_lexicon_tags_11 },
    { 0x9946e0a4, SUC_lexicon_tags_37 },
    { 0x44c6e0a4, SUC_lexicon_tags_138 },
    { 0x8ca6e0a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba6e0a9, SUC_lexicon_tags_34 },
    { 0x2e86e0aa, SUC_lexicon_tags_37 },
    { 0x5866e0aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6e0af, SUC_lexicon_tags_35 },
    { 0xa6a6e0b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf46e0ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae6e0be, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xdb66e0c0, SUC_lexicon_tags_15 },
    { 0x2386e0c1, SUC_lexicon_tags_7 },
    { 0xfaa6e0c2, SUC_lexicon_tags_68 },
    { 0xdd66e0c3, SUC_lexicon_tags_2 },
    { 0xf406e0c4, SUC_lexicon_tags_15 },
    { 0x4e26e0c5, SUC_lexicon_tags_56 },
    { 0xdb86e0c6, SUC_lexicon_tags_26 },
    { 0x56a6e0c7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6d66e0c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a26e0ce, SUC_lexicon_tags_30 },
    { 0x2a06e0ce, SUC_lexicon_tags_19 },
    { 0x3086e0d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa26e0d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa606e0d8, SUC_lexicon_tags_84 },
    { 0x3c6e0d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4606e0df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac06e0e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xddc6e0e3, SUC_lexicon_tags_47 },
    { 0xa6e0e3, SUC_lexicon_tags_46 },
    { 0x5d06e0e4, SUC_lexicon_tags_15 },
    { 0x4f66e0e6, SUC_lexicon_tags_0 },
    { 0x9826e0e4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x30a6e0e9, SUC_lexicon_tags_30 },
    { 0x4ee6e0ea, SUC_lexicon_tags_0 },
    { 0x5886e0e9, SUC_lexicon_tags_11 },
    { 0x9126e0e9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a6e0ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6426e0f1, SUC_lexicon_tags_48 },
    { 0x9fe6e0f1, SUC_lexicon_tags_1 },
    { 0xca46e0f2, SUC_lexicon_tags_10 },
    { 0x7386e0f4, SUC_lexicon_tags_1 },
    { 0xff26e0f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f66e0fa, SUC_lexicon_tags_17 },
    { 0x2326e0fb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd806e0fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5846e100, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48e6e102, SUC_lexicon_tags_7 },
    { 0x1c6e103, SUC_lexicon_tags_28 },
    { 0xb166e104, SUC_lexicon_tags_4 },
    { 0x31c6e103, SUC_lexicon_tags_11 },
    { 0xbf86e106, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e6e10a, SUC_lexicon_tags_38 },
    { 0xeee6e10a, SUC_lexicon_tags_10 },
    { 0x65c6e10b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c6e10f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb926e115, SUC_lexicon_tags_15 },
    { 0x6186e115, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd26e11a, SUC_lexicon_tags_37 },
    { 0x87e6e11b, SUC_lexicon_tags_45 },
    { 0x9f26e11b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a6e11f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x946e121, SUC_lexicon_tags_37 },
    { 0x17a6e121, SUC_lexicon_tags_47 },
    { 0xeaa6e121, SUC_lexicon_tags_30 },
    { 0x5ea6e124, SUC_lexicon_tags_37 },
    { 0x6646e124, SUC_lexicon_tags_30 },
    { 0xfc46e125, SUC_lexicon_tags_11 },
    { 0xc026e127, SUC_lexicon_tags_30 },
    { 0x4286e127, SUC_lexicon_tags_81 },
    { 0xeb46e129, SUC_lexicon_tags_27 },
    { 0xa066e12a, SUC_lexicon_tags_45 },
    { 0x2ce6e12b, SUC_lexicon_tags_6 },
    { 0x27e6e12a, SUC_lexicon_tags_23 },
    { 0x45c6e12d, SUC_lexicon_tags_19 },
    { 0x7866e12d, SUC_lexicon_tags_63 },
    { 0xe086e12f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4446e132, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8886e136, SUC_lexicon_tags_14 },
    { 0xb046e137, SUC_lexicon_tags_31 },
    { 0x2e46e136, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6e13b, SUC_lexicon_tags_1 },
    { 0xf306e13c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa386e143, SUC_lexicon_tags_34 },
    { 0xac86e143, SUC_lexicon_tags_6 },
    { 0xb026e145, SUC_lexicon_tags_1 },
    { 0x5806e143, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9e6e148, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd006e14b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb66e14e, SUC_lexicon_tags_8 },
    { 0x6ca6e14e, SUC_lexicon_tags_1 },
    { 0x4c6e14e, SUC_lexicon_tags_47 },
    { 0x23e6e151, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b66e154, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe6e15b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c86e15e, SUC_lexicon_tags_2 },
    { 0x9966e15f, SUC_lexicon_tags_27 },
    { 0xc026e15e, SUC_lexicon_tags_8 },
    { 0xbf66e161, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e6e165, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc6e16d, SUC_lexicon_tags_25 },
    { 0xa5a6e16e, SUC_lexicon_tags_62 },
    { 0x32e6e16f, SUC_lexicon_tags_25 },
    { 0x9486e170, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7446e172, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4526e175, SUC_lexicon_tags_17 },
    { 0x9366e175, SUC_lexicon_tags_11 },
    { 0x27c6e176, SUC_lexicon_tags_2 },
    { 0x2a86e177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c6e17b, SUC_lexicon_tags_29 },
    { 0xeac6e17c, SUC_lexicon_tags_31 },
    { 0x5466e17d, SUC_lexicon_tags_27 },
    { 0xe806e17e, SUC_lexicon_tags_31 },
    { 0xf766e17b, SUC_lexicon_tags_26 },
    { 0x3426e180, SUC_lexicon_tags_19 },
    { 0x63a6e181, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc06e190, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce06e194, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6846e199, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e6e19c, SUC_lexicon_tags_63 },
    { 0x4466e19d, SUC_lexicon_tags_37 },
    { 0xbc26e19e, SUC_lexicon_tags_12 },
    { 0x85a6e19e, SUC_lexicon_tags_11 },
    { 0xa626e19c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94a6e1a2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5ec6e1a4, SUC_lexicon_tags_47 },
    { 0x72a6e1a4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x5246e1a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1606e1aa, SUC_lexicon_tags_33 },
    { 0x1546e1ab, SUC_lexicon_tags_57 },
    { 0x2fe6e1ac, SUC_lexicon_tags_11 },
    { 0x9b86e1ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e6e1b0, SUC_lexicon_tags_37 },
    { 0x95c6e1b1, SUC_lexicon_tags_37 },
    { 0x7666e1b1, SUC_lexicon_tags_7 },
    { 0xb886e1b2, SUC_lexicon_tags_25 },
    { 0x9fc6e1b0, SUC_lexicon_tags_12 },
    { 0xa7e6e1b5, SUC_lexicon_tags_7 },
    { 0x7626e1b4, SUC_lexicon_tags_25 },
    { 0x18c6e1b7, SUC_lexicon_tags_17 },
    { 0xe486e1b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcbc6e1ba, SUC_lexicon_tags_25 },
    { 0x8726e1bb, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f86e1c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6e1c4, SUC_lexicon_tags_37 },
    { 0x1f46e1c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefc6e1c7, SUC_lexicon_tags_37 },
    { 0x7a6e1c8, SUC_lexicon_tags_49 },
    { 0x2026e1c8, SUC_lexicon_tags_14 },
    { 0xa166e1ca, SUC_lexicon_tags_6 },
    { 0x52e6e1cb, SUC_lexicon_tags_30 },
    { 0x4066e1cc, SUC_lexicon_tags_1 },
    { 0x3986e1ca, SUC_lexicon_tags_7 },
    { 0xc046e1cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c6e1d2, SUC_lexicon_tags_42 },
    { 0x5766e1d2, SUC_lexicon_tags_27 },
    { 0x5466e1d4, SUC_lexicon_tags_47 },
    { 0xdf86e1d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa06e1d9, SUC_lexicon_tags_26 },
    { 0x3ea6e1da, SUC_lexicon_tags_7 },
    { 0xea46e1d9, SUC_lexicon_tags_2 },
    { 0xa2c6e1dc, SUC_lexicon_tags_11 },
    { 0x4e6e1dd, SUC_lexicon_tags_17 },
    { 0xd786e1dd, SUC_lexicon_tags_63 },
    { 0x7e6e1df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfbe6e1e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf86e1e6, SUC_lexicon_tags_15 },
    { 0xe286e1e7, SUC_lexicon_tags_17 },
    { 0x8326e1e8, SUC_lexicon_tags_14 },
    { 0xce6e1e7, SUC_lexicon_tags_14 },
    { 0x9c86e1e6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2c46e1ec, SUC_lexicon_tags_230 },
    { 0x9b66e1ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x946e1f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd086e1f6, SUC_lexicon_tags_170 },
    { 0xe2c6e1f7, SUC_lexicon_tags_11 },
    { 0x43a6e1f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6e1fb, SUC_lexicon_tags_11 },
    { 0xb086e1fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5026e1fe, SUC_lexicon_tags_27 },
    { 0x40a6e1ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe2c6e201, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4166e209, SUC_lexicon_tags_11 },
    { 0x95c6e20a, SUC_lexicon_tags_12 },
    { 0x8a06e20b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7486e211, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba6e215, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xaac6e217, SUC_lexicon_tags_17 },
    { 0x6106e218, SUC_lexicon_tags_7 },
    { 0xf86e219, SUC_lexicon_tags_2 },
    { 0xe866e21a, SUC_lexicon_tags_37 },
    { 0x58c6e21b, SUC_lexicon_tags_15 },
    { 0x8d26e21c, SUC_lexicon_tags_22 },
    { 0x3226e21c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1046e21f, SUC_lexicon_tags_0 },
    { 0x1d66e21f, SUC_lexicon_tags_8 },
    { 0xe8c6e221, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe6e224, SUC_lexicon_tags_60 },
    { 0x5586e225, SUC_lexicon_tags_37 },
    { 0x2ce6e226, SUC_lexicon_tags_0 },
    { 0x5406e225, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb8c6e229, SUC_lexicon_tags_7 },
    { 0x9e26e229, SUC_lexicon_tags_29 },
    { 0x5ac6e22b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb366e22d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d46e231, SUC_lexicon_tags_49 },
    { 0xd046e232, SUC_lexicon_tags_17 },
    { 0x95a6e233, SUC_lexicon_tags_60 },
    { 0x3466e234, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5e86e236, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20c6e238, SUC_lexicon_tags_38 },
    { 0xa886e238, SUC_lexicon_tags_60 },
    { 0xb726e23a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc46e23e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446e243, SUC_lexicon_tags_11 },
    { 0x46e6e244, SUC_lexicon_tags_45 },
    { 0x98e6e243, SUC_lexicon_tags_0 },
    { 0xc666e245, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9906e24c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c6e24e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c06e258, SUC_lexicon_tags_25 },
    { 0xb846e258, SUC_lexicon_tags_17 },
    { 0xcd86e258, SUC_lexicon_tags_16 },
    { 0x9866e25b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a86e263, SUC_lexicon_tags_37 },
    { 0x7c26e264, SUC_lexicon_tags_7 },
    { 0x91e6e265, SUC_lexicon_tags_38 },
    { 0x4be6e266, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6786e270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf326e273, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc986e277, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6e27a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d26e27c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee6e27f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd7e6e281, SUC_lexicon_tags_33 },
    { 0xfb66e282, SUC_lexicon_tags_3 },
    { 0x7946e282, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2566e287, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe406e289, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3106e292, SUC_lexicon_tags_262 },
    { 0x5746e293, SUC_lexicon_tags_12 },
    { 0x8606e293, SUC_lexicon_tags_107 },
    { 0x8926e295, SUC_lexicon_tags_37 },
    { 0xaa6e296, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe6e29c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d06e29e, SUC_lexicon_tags_0 },
    { 0x2bc6e29f, SUC_lexicon_tags_19 },
    { 0xe2c6e2a0, SUC_lexicon_tags_60 },
    { 0xfc06e29f, SUC_lexicon_tags_28 },
    { 0x4606e2a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf446e2a5, SUC_lexicon_tags_2 },
    { 0x5786e2a6, SUC_lexicon_tags_25 },
    { 0xa3a6e2a6, SUC_lexicon_tags_19 },
    { 0xcf26e2a8, SUC_lexicon_tags_63 },
    { 0xb9a6e2a5, SUC_lexicon_tags_2 },
    { 0x26e6e2aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3666e2af, SUC_lexicon_tags_26 },
    { 0x6c66e2b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf386e2b4, SUC_lexicon_tags_15 },
    { 0x49a6e2b5, SUC_lexicon_tags_15 },
    { 0xb7c6e2b6, SUC_lexicon_tags_29 },
    { 0x3ee6e2b7, SUC_lexicon_tags_2 },
    { 0x5866e2b8, SUC_lexicon_tags_17 },
    { 0x7826e2b9, SUC_lexicon_tags_28 },
    { 0xed66e2ba, SUC_lexicon_tags_27 },
    { 0xdfc6e2bb, SUC_lexicon_tags_48 },
    { 0xe4e6e2bc, SUC_lexicon_tags_12 },
    { 0xd886e2bd, SUC_lexicon_tags_39 },
    { 0xb6c6e2bc, SUC_lexicon_tags_1 },
    { 0x2466e2bb, SUC_lexicon_tags_29 },
    { 0x4e6e2be, SUC_lexicon_tags_4 },
    { 0x8cc6e2b6, SUC_lexicon_tags_1 },
    { 0x4866e2bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc626e2c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbb26e2c7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5246e2c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2606e2cb, SUC_lexicon_tags_12 },
    { 0x2d06e2cc, SUC_lexicon_tags_7 },
    { 0x4b06e2cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa226e2d3, SUC_lexicon_tags_75 },
    { 0xfc46e2d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b06e2d8, SUC_lexicon_tags_25 },
    { 0x65c6e2d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a26e2db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1306e2df, SUC_lexicon_tags_37 },
    { 0xdbc6e2e0, SUC_lexicon_tags_5 },
    { 0xeac6e2df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57e6e2e3, SUC_lexicon_tags_3 },
    { 0x8b26e2e4, SUC_lexicon_tags_43 },
    { 0x2ac6e2e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1106e2e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e26e2eb, SUC_lexicon_tags_37 },
    { 0x2626e2ec, SUC_lexicon_tags_37 },
    { 0xd3c6e2eb, SUC_lexicon_tags_274 },
    { 0xef46e2ec, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e6e2f2, SUC_lexicon_tags_4 },
    { 0xfc6e2f3, SUC_lexicon_tags_25 },
    { 0x9fe6e2f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e6e2f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe86e2fb, SUC_lexicon_tags_8 },
    { 0x4946e2fc, SUC_lexicon_tags_38 },
    { 0x19c6e2fd, SUC_lexicon_tags_4 },
    { 0x2746e2fc, SUC_lexicon_tags_6 },
    { 0x29c6e2fb, SUC_lexicon_tags_12 },
    { 0xc4e6e2fe, SUC_lexicon_tags_1 },
    { 0xb026e301, SUC_lexicon_tags_8 },
    { 0x6306e301, SUC_lexicon_tags_37 },
    { 0x20a6e2fd, SUC_lexicon_tags_60 },
    { 0xa666e304, SUC_lexicon_tags_37 },
    { 0x1506e304, SUC_lexicon_tags_27 },
    { 0x8806e2fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746e30e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc6e311, SUC_lexicon_tags_14 },
    { 0xee66e311, SUC_lexicon_tags_12 },
    { 0x89c6e312, SUC_lexicon_tags_48 },
    { 0x7b46e314, SUC_lexicon_tags_25 },
    { 0x2b66e315, SUC_lexicon_tags_25 },
    { 0xd746e315, SUC_lexicon_tags_17 },
    { 0xd226e317, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2806e319, SUC_lexicon_tags_37 },
    { 0xdd66e319, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc2a6e31c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca6e323, SUC_lexicon_tags_6 },
    { 0x526e323, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe6e328, SUC_lexicon_tags_6 },
    { 0x1646e329, SUC_lexicon_tags_84 },
    { 0x2da6e32a, SUC_lexicon_tags_269 },
    { 0xa866e32b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf946e330, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf06e332, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e6e335, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2806e337, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc846e343, SUC_lexicon_tags_9 },
    { 0x6f26e343, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5e6e346, SUC_lexicon_tags_31 },
    { 0x76c6e346, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7546e34c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1166e34f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d86e352, SUC_lexicon_tags_25 },
    { 0xc286e353, SUC_lexicon_tags_25 },
    { 0x67c6e352, SUC_lexicon_tags_85 },
    { 0x6286e354, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc866e35c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39a6e35e, SUC_lexicon_tags_37 },
    { 0xb506e35f, SUC_lexicon_tags_46 },
    { 0x6d46e360, SUC_lexicon_tags_11 },
    { 0x8a6e361, SUC_lexicon_tags_14 },
    { 0x9e66e362, SUC_lexicon_tags_69 },
    { 0xb0e6e363, SUC_lexicon_tags_27 },
    { 0x92e6e364, SUC_lexicon_tags_15 },
    { 0x5a66e361, SUC_lexicon_tags_17 },
    { 0x9606e364, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe406e369, SUC_lexicon_tags_7 },
    { 0xe726e36a, SUC_lexicon_tags_7 },
    { 0xca06e36b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2566e36f, SUC_lexicon_tags_11 },
    { 0x1b86e36f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1306e372, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c06e375, SUC_lexicon_tags_36 },
    { 0x4926e376, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d26e379, SUC_lexicon_tags_17 },
    { 0xe246e37a, SUC_lexicon_tags_17 },
    { 0x5226e37a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b06e386, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf366e389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec6e390, SUC_lexicon_tags_12 },
    { 0x5fa6e391, SUC_lexicon_tags_12 },
    { 0x4506e392, SUC_lexicon_tags_9 },
    { 0x4d06e393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e6e397, SUC_lexicon_tags_15 },
    { 0x7ee6e398, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa6e39d, SUC_lexicon_tags_26 },
    { 0x41e6e39e, SUC_lexicon_tags_36 },
    { 0x64c6e39e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e6e3ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac86e3af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9846e3b1, SUC_lexicon_tags_60 },
    { 0xb066e3b2, SUC_lexicon_tags_11 },
    { 0x1ee6e3b2, SUC_lexicon_tags_0 },
    { 0x71e6e3b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa6e3b7, SUC_lexicon_tags_37 },
    { 0x4a06e3b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b66e3bc, SUC_lexicon_tags_285 },
    { 0, NULL },
    { 0x1546e3be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6406e3c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c06e3c2, SUC_lexicon_tags_1 },
    { 0x47e6e3c3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e6e3c6, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xbc66e3c8, SUC_lexicon_tags_29 },
    { 0xfec6e3c9, SUC_lexicon_tags_25 },
    { 0xca86e3c9, SUC_lexicon_tags_31 },
    { 0x8986e3cb, SUC_lexicon_tags_84 },
    { 0x7d06e3cc, SUC_lexicon_tags_37 },
    { 0xb266e3c8, SUC_lexicon_tags_17 },
    { 0x4dc6e3c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab86e3d1, SUC_lexicon_tags_25 },
    { 0x4fc6e3d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36a6e3d4, SUC_lexicon_tags_2 },
    { 0xffa6e3d5, SUC_lexicon_tags_136 },
    { 0x2d06e3d4, SUC_lexicon_tags_8 },
    { 0x29e6e3d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca6e3db, SUC_lexicon_tags_17 },
    { 0xa366e3dc, SUC_lexicon_tags_15 },
    { 0xa906e3dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf06e3df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc86e3e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86e3e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6e3ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c26e3f0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x40c6e3f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb686e3f4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4646e3f7, SUC_lexicon_tags_27 },
    { 0x51a6e3f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3926e3fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd126e400, SUC_lexicon_tags_32 },
    { 0x19c6e401, SUC_lexicon_tags_13 },
    { 0x58a6e402, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e06e406, SUC_lexicon_tags_38 },
    { 0x406e407, SUC_lexicon_tags_11 },
    { 0x6626e406, SUC_lexicon_tags_16 },
    { 0x85e6e409, SUC_lexicon_tags_49 },
    { 0x386e40a, SUC_lexicon_tags_11 },
    { 0xdb46e40b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2646e410, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7786e415, SUC_lexicon_tags_84 },
    { 0x9706e416, SUC_lexicon_tags_118 },
    { 0x69e6e415, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5b26e419, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa66e41b, SUC_lexicon_tags_23 },
    { 0x8066e41b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc6e422, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa666e425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd06e427, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306e42b, SUC_lexicon_tags_60 },
    { 0x6e42c, SUC_lexicon_tags_7 },
    { 0x4fc6e42c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf306e42f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c6e432, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf146e437, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbe6e439, SUC_lexicon_tags_15 },
    { 0x4c26e43a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cc6e43c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41a6e43e, SUC_lexicon_tags_7 },
    { 0x9946e43f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3346e442, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5026e44a, SUC_lexicon_tags_38 },
    { 0x3826e44b, SUC_lexicon_tags_22 },
    { 0x9766e44c, SUC_lexicon_tags_12 },
    { 0xb446e44a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a6e450, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1066e452, SUC_lexicon_tags_11 },
    { 0xa546e452, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9a6e455, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4386e457, SUC_lexicon_tags_23 },
    { 0xcb66e458, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e6e45e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa186e462, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec6e464, SUC_lexicon_tags_9 },
    { 0xbf66e465, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32c6e467, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe366e46c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d26e471, SUC_lexicon_tags_11 },
    { 0x70c6e472, SUC_lexicon_tags_19 },
    { 0xf306e473, SUC_lexicon_tags_48 },
    { 0x686e474, SUC_lexicon_tags_0 },
    { 0x9f86e475, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe046e477, SUC_lexicon_tags_12 },
    { 0x52c6e478, SUC_lexicon_tags_47 },
    { 0xca6e477, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb266e47d, SUC_lexicon_tags_14 },
    { 0x6546e47e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a6e481, SUC_lexicon_tags_21 },
    { 0xfca6e482, SUC_lexicon_tags_39 },
    { 0x4e86e483, SUC_lexicon_tags_6 },
    { 0x71c6e484, SUC_lexicon_tags_1 },
    { 0x8846e485, SUC_lexicon_tags_38 },
    { 0xbf26e485, SUC_lexicon_tags_27 },
    { 0x7526e487, SUC_lexicon_tags_16 },
    { 0xd726e488, SUC_lexicon_tags_7 },
    { 0x2c26e484, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1c6e48b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa6e493, SUC_lexicon_tags_15 },
    { 0x1ca6e494, SUC_lexicon_tags_15 },
    { 0x8b46e494, SUC_lexicon_tags_0 },
    { 0x4bc6e495, SUC_lexicon_tags_25 },
    { 0xe746e497, SUC_lexicon_tags_25 },
    { 0x8e26e497, SUC_lexicon_tags_26 },
    { 0x1ba6e499, SUC_lexicon_tags_33 },
    { 0x7bc6e499, SUC_lexicon_tags_37 },
    { 0x4306e497, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2046e49e, SUC_lexicon_tags_60 },
    { 0x22a6e49e, SUC_lexicon_tags_11 },
    { 0x9a26e49f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2766e4a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2386e4a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6066e4ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d86e4af, SUC_lexicon_tags_63 },
    { 0xcae6e4b0, SUC_lexicon_tags_12 },
    { 0xbac6e4b1, SUC_lexicon_tags_84 },
    { 0x8046e4b1, SUC_lexicon_tags_68 },
    { 0x2dc6e4b3, SUC_lexicon_tags_43 },
    { 0x6606e4b2, SUC_lexicon_tags_7 },
    { 0x6126e4b5, SUC_lexicon_tags_33 },
    { 0x2726e4b5, SUC_lexicon_tags_723 },
    { 0xf1c6e4b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c6e4bb, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6486e4c0, SUC_lexicon_tags_17 },
    { 0x946e4c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40e6e4c3, SUC_lexicon_tags_11 },
    { 0x4366e4c4, SUC_lexicon_tags_60 },
    { 0xee66e4c5, SUC_lexicon_tags_49 },
    { 0xbd46e4c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c06e4ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea6e4d0, SUC_lexicon_tags_28 },
    { 0x57e6e4d1, SUC_lexicon_tags_37 },
    { 0xef46e4d2, SUC_lexicon_tags_2 },
    { 0x56c6e4d3, SUC_lexicon_tags_2 },
    { 0xc9a6e4d4, SUC_lexicon_tags_2 },
    { 0x5426e4d2, SUC_lexicon_tags_17 },
    { 0xc06e4d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd866e4da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe426e4dd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfac6e4df, SUC_lexicon_tags_27 },
    { 0x51c6e4e0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6286e4e4, SUC_lexicon_tags_14 },
    { 0xa346e4e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc726e4e9, SUC_lexicon_tags_15 },
    { 0x7886e4ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x63a6e4ec, SUC_lexicon_tags_2 },
    { 0x846e4ed, SUC_lexicon_tags_11 },
    { 0xa226e4ed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe826e4f0, SUC_lexicon_tags_47 },
    { 0xbc86e4f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb626e4f3, SUC_lexicon_tags_12 },
    { 0x8f46e4f3, SUC_lexicon_tags_49 },
    { 0x6e26e4f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed26e4fc, SUC_lexicon_tags_47 },
    { 0x2f66e4fc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06e500, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f46e503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb406e505, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a6e50b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf746e518, SUC_lexicon_tags_15 },
    { 0x8e6e519, SUC_lexicon_tags_714 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1986e51d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4706e522, SUC_lexicon_tags_16 },
    { 0x1126e522, SUC_lexicon_tags_45 },
    { 0xe3e6e524, SUC_lexicon_tags_2 },
    { 0xb1c6e522, SUC_lexicon_tags_1 },
    { 0xc346e526, SUC_lexicon_tags_9 },
    { 0x85c6e527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab26e529, SUC_lexicon_tags_7 },
    { 0x5e46e529, SUC_lexicon_tags_37 },
    { 0xf1a6e529, SUC_lexicon_tags_75 },
    { 0xd046e52c, SUC_lexicon_tags_60 },
    { 0xeb26e52c, SUC_lexicon_tags_23 },
    { 0xeac6e52c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6e531, SUC_lexicon_tags_33 },
    { 0x1046e532, SUC_lexicon_tags_47 },
    { 0xd386e533, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf826e535, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e6e538, SUC_lexicon_tags_23 },
    { 0xa4e6e539, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb486e53b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92e6e53d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc526e53f, SUC_lexicon_tags_14 },
    { 0xd466e53f, SUC_lexicon_tags_27 },
    { 0x5ce6e53f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c26e544, SUC_lexicon_tags_33 },
    { 0xca86e545, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf106e54a, SUC_lexicon_tags_37 },
    { 0x56e6e54b, SUC_lexicon_tags_11 },
    { 0xec66e54c, SUC_lexicon_tags_45 },
    { 0x3406e54c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3466e550, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1526e559, SUC_lexicon_tags_25 },
    { 0x5b06e55a, SUC_lexicon_tags_12 },
    { 0x3466e55b, SUC_lexicon_tags_8 },
    { 0x8966e559, SUC_lexicon_tags_1 },
    { 0xffc6e55d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee86e564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6646e567, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1166e56a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8266e56e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc9c6e570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb86e578, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8bc6e57a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xff46e57d, SUC_lexicon_tags_1 },
    { 0xfb86e57e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc6e581, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5926e584, SUC_lexicon_tags_25 },
    { 0xe606e584, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a86e588, SUC_lexicon_tags_59 },
    { 0x9426e588, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xed46e58c, SUC_lexicon_tags_31 },
    { 0x4c06e58d, SUC_lexicon_tags_12 },
    { 0x2ce6e58e, SUC_lexicon_tags_7 },
    { 0xf106e58e, SUC_lexicon_tags_37 },
    { 0x38a6e58e, SUC_lexicon_tags_21 },
    { 0xfe46e58d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb506e593, SUC_lexicon_tags_1 },
    { 0x6386e594, SUC_lexicon_tags_1 },
    { 0xd3a6e595, SUC_lexicon_tags_11 },
    { 0x7ba6e596, SUC_lexicon_tags_25 },
    { 0xad66e596, SUC_lexicon_tags_15 },
    { 0x86c6e598, SUC_lexicon_tags_17 },
    { 0x9ca6e599, SUC_lexicon_tags_2 },
    { 0x9b06e59a, SUC_lexicon_tags_17 },
    { 0x7b86e59b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdde6e59d, SUC_lexicon_tags_82 },
    { 0x87a6e59e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7786e5a0, SUC_lexicon_tags_7 },
    { 0x57c6e5a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a26e5a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf66e5a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa906e5ab, SUC_lexicon_tags_12 },
    { 0xfb66e5ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a6e5b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7106e5b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2146e5b6, SUC_lexicon_tags_8 },
    { 0x2c86e5b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a66e5ba, SUC_lexicon_tags_11 },
    { 0x6846e5bb, SUC_lexicon_tags_2 },
    { 0x2a46e5bb, SUC_lexicon_tags_23 },
    { 0x92e6e5bc, SUC_lexicon_tags_816 },
    { 0, NULL },
    { 0x77a6e5bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e6e5c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcec6e5c6, SUC_lexicon_tags_15 },
    { 0xecc6e5c7, SUC_lexicon_tags_12 },
    { 0xec86e5c8, SUC_lexicon_tags_10 },
    { 0xdaa6e5c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7c6e5cb, SUC_lexicon_tags_55 },
    { 0xcd66e5cc, SUC_lexicon_tags_12 },
    { 0x1446e5cd, SUC_lexicon_tags_37 },
    { 0x8ec6e5ce, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x3686e5d1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdae6e5d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c6e5dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d26e5e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb246e5e5, SUC_lexicon_tags_17 },
    { 0xd8e6e5e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78e6e5e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe46e5ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8746e5f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5106e5f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d26e5f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef26e5ff, SUC_lexicon_tags_32 },
    { 0x3286e600, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e6e605, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2546e60a, SUC_lexicon_tags_6 },
    { 0x35a6e60a, SUC_lexicon_tags_31 },
    { 0x4186e60c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa246e612, SUC_lexicon_tags_47 },
    { 0xf646e612, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9166e616, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c06e61b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7926e621, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc6e628, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6e62b, SUC_lexicon_tags_46 },
    { 0xa6e6e62c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe6e630, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b26e635, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e6e638, SUC_lexicon_tags_15 },
    { 0x4086e639, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2646e642, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c6e645, SUC_lexicon_tags_1 },
    { 0x3966e645, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2166e648, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd166e64a, SUC_lexicon_tags_13 },
    { 0x7866e64a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29c6e64d, SUC_lexicon_tags_25 },
    { 0x8aa6e64e, SUC_lexicon_tags_45 },
    { 0x6906e64e, SUC_lexicon_tags_92 },
    { 0x3d86e64f, SUC_lexicon_tags_37 },
    { 0xc866e64e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x37e6e653, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a6e657, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8cc6e659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28a6e65b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaf66e65d, SUC_lexicon_tags_2 },
    { 0xe506e65e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce6e663, SUC_lexicon_tags_10 },
    { 0xa486e664, SUC_lexicon_tags_47 },
    { 0xf6e6e664, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e6e66a, SUC_lexicon_tags_977 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc6e66f, SUC_lexicon_tags_0 },
    { 0xb766e670, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c6e674, SUC_lexicon_tags_66 },
    { 0x7ce6e675, SUC_lexicon_tags_15 },
    { 0x7b66e674, SUC_lexicon_tags_15 },
    { 0xc006e676, SUC_lexicon_tags_2 },
    { 0xd006e678, SUC_lexicon_tags_17 },
    { 0x4d46e677, SUC_lexicon_tags_7 },
    { 0x1e06e67a, SUC_lexicon_tags_17 },
    { 0xf506e678, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c6e681, SUC_lexicon_tags_12 },
    { 0x4cc6e681, SUC_lexicon_tags_39 },
    { 0xd0e6e681, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe6e686, SUC_lexicon_tags_2 },
    { 0x20a6e687, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7b66e689, SUC_lexicon_tags_25 },
    { 0x2ba6e689, SUC_lexicon_tags_56 },
    { 0x3006e68a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x37e6e68d, SUC_lexicon_tags_31 },
    { 0xc3e6e68e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5aa6e690, SUC_lexicon_tags_17 },
    { 0x7926e691, SUC_lexicon_tags_11 },
    { 0x2e66e691, SUC_lexicon_tags_60 },
    { 0x9dc6e691, SUC_lexicon_tags_43 },
    { 0x3226e694, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xc366e697, SUC_lexicon_tags_0 },
    { 0x9ae6e697, SUC_lexicon_tags_39 },
    { 0x7ce6e699, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc846e69e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa346e6a2, SUC_lexicon_tags_57 },
    { 0x51a6e6a2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xcd26e6a5, SUC_lexicon_tags_45 },
    { 0x1de6e6a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8926e6aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9a6e6ac, SUC_lexicon_tags_17 },
    { 0x57c6e6ac, SUC_lexicon_tags_23 },
    { 0x7466e6ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2866e6b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c86e6b2, SUC_lexicon_tags_25 },
    { 0xb5c6e6b2, SUC_lexicon_tags_63 },
    { 0x8d66e6b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc7e6e6b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6746e6b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ac6e6ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b06e6c0, SUC_lexicon_tags_1 },
    { 0x8966e6c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xff06e6c3, SUC_lexicon_tags_26 },
    { 0x4626e6c4, SUC_lexicon_tags_17 },
    { 0xef46e6c4, SUC_lexicon_tags_23 },
    { 0x4206e6c5, SUC_lexicon_tags_23 },
    { 0xbfa6e6c7, SUC_lexicon_tags_37 },
    { 0xb8c6e6c8, SUC_lexicon_tags_25 },
    { 0xcc66e6c9, SUC_lexicon_tags_63 },
    { 0xd126e6c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf846e6ce, SUC_lexicon_tags_1 },
    { 0x8e6e6ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26e6d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3766e6d7, SUC_lexicon_tags_7 },
    { 0xba06e6d7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcc86e6da, SUC_lexicon_tags_12 },
    { 0x3ce6e6db, SUC_lexicon_tags_15 },
    { 0x94c6e6dc, SUC_lexicon_tags_37 },
    { 0x7906e6dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba06e6df, SUC_lexicon_tags_32 },
    { 0x87e6e6e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de6e6e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcd86e6e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdb46e6eb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e06e6f1, SUC_lexicon_tags_47 },
    { 0x7026e6f2, SUC_lexicon_tags_39 },
    { 0xcc86e6f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d26e6f8, SUC_lexicon_tags_25 },
    { 0x4de6e6f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba6e6fc, SUC_lexicon_tags_16 },
    { 0xfba6e6fd, SUC_lexicon_tags_15 },
    { 0xa26e6fe, SUC_lexicon_tags_15 },
    { 0x4f46e6ff, SUC_lexicon_tags_92 },
    { 0x5dc6e6ff, SUC_lexicon_tags_31 },
    { 0x8dc6e701, SUC_lexicon_tags_23 },
    { 0x37c6e6ff, SUC_lexicon_tags_17 },
    { 0x5666e702, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa6e708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fa6e70a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2586e70c, SUC_lexicon_tags_11 },
    { 0x99e6e70d, SUC_lexicon_tags_28 },
    { 0x8ba6e70e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3886e712, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6e715, SUC_lexicon_tags_60 },
    { 0xef06e716, SUC_lexicon_tags_30 },
    { 0xd726e716, SUC_lexicon_tags_46 },
    { 0x6066e717, SUC_lexicon_tags_45 },
    { 0x7f26e718, SUC_lexicon_tags_2 },
    { 0x1a46e71a, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xf126e71c, SUC_lexicon_tags_118 },
    { 0xc506e71c, SUC_lexicon_tags_32 },
    { 0x6886e71e, SUC_lexicon_tags_37 },
    { 0x1206e71c, SUC_lexicon_tags_31 },
    { 0x79a6e720, SUC_lexicon_tags_45 },
    { 0x9446e721, SUC_lexicon_tags_17 },
    { 0x2466e722, SUC_lexicon_tags_30 },
    { 0xdbe6e723, SUC_lexicon_tags_17 },
    { 0x98c6e721, SUC_lexicon_tags_37 },
    { 0xc626e721, SUC_lexicon_tags_32 },
    { 0x8786e721, SUC_lexicon_tags_21 },
    { 0xd9a6e71d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2286e72c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa626e72f, SUC_lexicon_tags_30 },
    { 0xfb86e730, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac6e734, SUC_lexicon_tags_12 },
    { 0x5406e735, SUC_lexicon_tags_19 },
    { 0x6ee6e736, SUC_lexicon_tags_43 },
    { 0x3f66e737, SUC_lexicon_tags_7 },
    { 0x95e6e738, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb386e742, SUC_lexicon_tags_12 },
    { 0x6f06e743, SUC_lexicon_tags_46 },
    { 0xdc26e743, SUC_lexicon_tags_1 },
    { 0xd306e745, SUC_lexicon_tags_34 },
    { 0x3646e745, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9a26e748, SUC_lexicon_tags_4 },
    { 0xae6e749, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xec06e74b, SUC_lexicon_tags_37 },
    { 0x8da6e74c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf226e74e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd46e758, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da6e75d, SUC_lexicon_tags_47 },
    { 0x2b46e75e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2146e762, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c46e769, SUC_lexicon_tags_37 },
    { 0x3fa6e76a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c86e76f, SUC_lexicon_tags_12 },
    { 0x3d06e770, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e6e776, SUC_lexicon_tags_27 },
    { 0x6906e777, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3fc6e779, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d46e77c, SUC_lexicon_tags_33 },
    { 0xa6c6e77d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e66e780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a86e783, SUC_lexicon_tags_106 },
    { 0x8da6e783, SUC_lexicon_tags_12 },
    { 0xac6e784, SUC_lexicon_tags_0 },
    { 0x6d06e786, SUC_lexicon_tags_7 },
    { 0x5c86e785, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa726e78a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4986e78d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2766e792, SUC_lexicon_tags_13 },
    { 0xbc46e793, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee6e796, SUC_lexicon_tags_14 },
    { 0x4486e797, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c6e79a, SUC_lexicon_tags_47 },
    { 0xcc86e79b, SUC_lexicon_tags_9 },
    { 0xeb86e79c, SUC_lexicon_tags_23 },
    { 0xdfa6e79d, SUC_lexicon_tags_43 },
    { 0xca66e79d, SUC_lexicon_tags_65 },
    { 0x2d06e79f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6106e7a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa86e7a4, SUC_lexicon_tags_17 },
    { 0xe66e7a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c6e7aa, SUC_lexicon_tags_28 },
    { 0x5fc6e7ab, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba6e7b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x106e7b4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a6e7b7, SUC_lexicon_tags_37 },
    { 0xf006e7b7, SUC_lexicon_tags_1 },
    { 0xce06e7b9, SUC_lexicon_tags_2 },
    { 0x9786e7b9, SUC_lexicon_tags_0 },
    { 0xeb66e7bb, SUC_lexicon_tags_39 },
    { 0x8ec6e7bc, SUC_lexicon_tags_60 },
    { 0x4146e7b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b86e7c0, SUC_lexicon_tags_8 },
    { 0x6a46e7c1, SUC_lexicon_tags_15 },
    { 0x6e26e7c0, SUC_lexicon_tags_47 },
    { 0xa5c6e7c3, SUC_lexicon_tags_3 },
    { 0x36e6e7c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c6e7cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de6e7d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5626e7d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a6e7de, SUC_lexicon_tags_37 },
    { 0xa526e7df, SUC_lexicon_tags_7 },
    { 0x8986e7df, SUC_lexicon_tags_11 },
    { 0x3546e7e1, SUC_lexicon_tags_39 },
    { 0xbaa6e7e2, SUC_lexicon_tags_8 },
    { 0x7d66e7e3, SUC_lexicon_tags_29 },
    { 0x2c66e7e4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8786e7e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6e7e9, SUC_lexicon_tags_49 },
    { 0x79a6e7ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a86e7ed, SUC_lexicon_tags_43 },
    { 0x806e7ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f26e7f4, SUC_lexicon_tags_37 },
    { 0x4b86e7f5, SUC_lexicon_tags_15 },
    { 0x1bc6e7f6, SUC_lexicon_tags_2 },
    { 0xc686e7f7, SUC_lexicon_tags_37 },
    { 0x13c6e7f8, SUC_lexicon_tags_25 },
    { 0x66c6e7f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a6e805, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a6e80b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x166e80d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e6e810, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfea6e812, SUC_lexicon_tags_4 },
    { 0x3506e813, SUC_lexicon_tags_37 },
    { 0x486e812, SUC_lexicon_tags_16 },
    { 0xd446e812, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e66e818, SUC_lexicon_tags_11 },
    { 0x9d66e819, SUC_lexicon_tags_38 },
    { 0xf446e819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2046e81c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2186e81f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e6e825, SUC_lexicon_tags_2 },
    { 0xc686e825, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe06e828, SUC_lexicon_tags_6 },
    { 0x6266e828, SUC_lexicon_tags_8 },
    { 0x3da6e828, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc6e830, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6366e832, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc6e837, SUC_lexicon_tags_62 },
    { 0xcf86e837, SUC_lexicon_tags_1346 },
    { 0, NULL },
    { 0x9a06e83a, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e6e83e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6e842, SUC_lexicon_tags_1 },
    { 0x8246e843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e66e846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca6e849, SUC_lexicon_tags_9 },
    { 0x20c6e84a, SUC_lexicon_tags_25 },
    { 0x6026e84a, SUC_lexicon_tags_32 },
    { 0x5306e84a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe6e850, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c6e854, SUC_lexicon_tags_17 },
    { 0x9606e855, SUC_lexicon_tags_7 },
    { 0xb006e856, SUC_lexicon_tags_49 },
    { 0xa626e857, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1406e85b, SUC_lexicon_tags_21 },
    { 0x5fe6e85c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xac26e85f, SUC_lexicon_tags_11 },
    { 0x9406e860, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a6e864, SUC_lexicon_tags_33 },
    { 0xc646e865, SUC_lexicon_tags_1 },
    { 0x17c6e866, SUC_lexicon_tags_2 },
    { 0xa706e866, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc826e86a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8a6e86c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd186e86f, SUC_lexicon_tags_11 },
    { 0xc386e870, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa6e877, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c6e87a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d26e87d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a6e880, SUC_lexicon_tags_5 },
    { 0x76c6e880, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c6e884, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e6e887, SUC_lexicon_tags_48 },
    { 0xadc6e888, SUC_lexicon_tags_62 },
    { 0x6906e889, SUC_lexicon_tags_32 },
    { 0x7406e88a, SUC_lexicon_tags_64 },
    { 0x1206e888, SUC_lexicon_tags_7 },
    { 0x3906e888, SUC_lexicon_tags_50 },
    { 0x6226e88a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e890, SUC_lexicon_tags_15 },
    { 0x6866e891, SUC_lexicon_tags_17 },
    { 0xd186e892, SUC_lexicon_tags_27 },
    { 0xc0a6e890, SUC_lexicon_tags_26 },
    { 0x57a6e893, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c6e898, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a6e89c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d46e8a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce6e8a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa906e8a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee6e8ae, SUC_lexicon_tags_15 },
    { 0x64e6e8af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a6e8b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8006e8b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7226e8bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f46e8bf, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc406e8c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7426e8c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfb66e8ca, SUC_lexicon_tags_31 },
    { 0x3266e8cb, SUC_lexicon_tags_14 },
    { 0x77e6e8cc, SUC_lexicon_tags_63 },
    { 0xa726e8cb, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6e8d1, SUC_lexicon_tags_4 },
    { 0xd2e6e8d2, SUC_lexicon_tags_25 },
    { 0x6ee6e8d1, SUC_lexicon_tags_45 },
    { 0x8f06e8d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e6e8d8, SUC_lexicon_tags_201 },
    { 0xefc6e8d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1446e8dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dc6e8df, SUC_lexicon_tags_0 },
    { 0xfec6e8e0, SUC_lexicon_tags_45 },
    { 0x9b66e8df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6386e8e7, SUC_lexicon_tags_26 },
    { 0xa346e8e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7606e8eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc6e8f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b86e8fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9766e8fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4966e905, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7406e90e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1346e914, SUC_lexicon_tags_336 },
    { 0x3046e914, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1b66e917, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7486e91a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c86e921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2226e923, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3306e928, SUC_lexicon_tags_9 },
    { 0x2c06e929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a06e931, SUC_lexicon_tags_27 },
    { 0x2586e932, SUC_lexicon_tags_7 },
    { 0x3d06e932, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6e938, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a6e93c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e6e949, SUC_lexicon_tags_37 },
    { 0x32c6e94a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5966e94e, SUC_lexicon_tags_25 },
    { 0x7be6e94f, SUC_lexicon_tags_4 },
    { 0x7a26e950, SUC_lexicon_tags_21 },
    { 0xb866e950, SUC_lexicon_tags_53 },
    { 0x1206e94f, SUC_lexicon_tags_124 },
    { 0x93e6e953, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c6e956, SUC_lexicon_tags_17 },
    { 0x6206e957, SUC_lexicon_tags_14 },
    { 0xe486e958, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7086e95f, SUC_lexicon_tags_17 },
    { 0x7946e960, SUC_lexicon_tags_1479 },
    { 0x84e6e961, SUC_lexicon_tags_28 },
    { 0x8bc6e961, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c86e968, SUC_lexicon_tags_57 },
    { 0xdae6e969, SUC_lexicon_tags_12 },
    { 0x66e96a, SUC_lexicon_tags_26 },
    { 0x6206e96b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c6e96e, SUC_lexicon_tags_1 },
    { 0x8486e96f, SUC_lexicon_tags_77 },
    { 0x71a6e970, SUC_lexicon_tags_37 },
    { 0x6826e970, SUC_lexicon_tags_33 },
    { 0xb286e971, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4426e976, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x16a6e978, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc646e97a, SUC_lexicon_tags_33 },
    { 0x8b66e97b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa26e97d, SUC_lexicon_tags_29 },
    { 0x3ac6e97d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff26e981, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b46e984, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1406e986, SUC_lexicon_tags_63 },
    { 0xd6c6e986, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a6e98a, SUC_lexicon_tags_25 },
    { 0xb3e6e98b, SUC_lexicon_tags_10 },
    { 0x21a6e98c, SUC_lexicon_tags_19 },
    { 0x9de6e98d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7606e994, SUC_lexicon_tags_2 },
    { 0x8206e994, SUC_lexicon_tags_0 },
    { 0x4226e996, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a6e999, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a86e99c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a86e99e, SUC_lexicon_tags_117 },
    { 0x40c6e99e, SUC_lexicon_tags_11 },
    { 0x7846e99f, SUC_lexicon_tags_12 },
    { 0x6566e9a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3e6e9a3, SUC_lexicon_tags_45 },
    { 0x5d06e9a4, SUC_lexicon_tags_39 },
    { 0x9e66e9a5, SUC_lexicon_tags_15 },
    { 0x4126e9a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc486e9a8, SUC_lexicon_tags_9 },
    { 0x75c6e9a9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb4c6e9ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c6e9ae, SUC_lexicon_tags_25 },
    { 0x8e6e9af, SUC_lexicon_tags_17 },
    { 0xe106e9b0, SUC_lexicon_tags_87 },
    { 0x67e6e9b1, SUC_lexicon_tags_49 },
    { 0xcf06e9b1, SUC_lexicon_tags_26 },
    { 0x37a6e9b2, SUC_lexicon_tags_8 },
    { 0x1e86e9b4, SUC_lexicon_tags_19 },
    { 0xe966e9b5, SUC_lexicon_tags_66 },
    { 0x6c46e9b6, SUC_lexicon_tags_15 },
    { 0xaec6e9b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7206e9bf, SUC_lexicon_tags_32 },
    { 0x3b46e9c0, SUC_lexicon_tags_33 },
    { 0x946e9bf, SUC_lexicon_tags_37 },
    { 0x3aa6e9c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da6e9c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb706e9cb, SUC_lexicon_tags_37 },
    { 0x6aa6e9cb, SUC_lexicon_tags_2 },
    { 0x8f46e9cd, SUC_lexicon_tags_39 },
    { 0x5446e9ce, SUC_lexicon_tags_2 },
    { 0xf9a6e9cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9126e9d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x626e9d7, SUC_lexicon_tags_68 },
    { 0x95a6e9d8, SUC_lexicon_tags_2 },
    { 0xe466e9d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e6e9dc, SUC_lexicon_tags_10 },
    { 0x90e6e9dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe926e9e1, SUC_lexicon_tags_2 },
    { 0x1da6e9e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe006e9e7, SUC_lexicon_tags_46 },
    { 0xe046e9e7, SUC_lexicon_tags_2 },
    { 0xe7a6e9e9, SUC_lexicon_tags_63 },
    { 0xad66e9e8, SUC_lexicon_tags_31 },
    { 0xd526e9ea, SUC_lexicon_tags_29 },
    { 0xbac6e9ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b06e9ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc106e9f1, SUC_lexicon_tags_1 },
    { 0xd706e9f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe006e9f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5186e9ff, SUC_lexicon_tags_17 },
    { 0x8f26e9ff, SUC_lexicon_tags_2 },
    { 0xaea6ea01, SUC_lexicon_tags_39 },
    { 0x3206ea02, SUC_lexicon_tags_39 },
    { 0x2bc6ea03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4706ea09, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca6ea0c, SUC_lexicon_tags_48 },
    { 0x2f06ea0d, SUC_lexicon_tags_29 },
    { 0xd4c6ea0c, SUC_lexicon_tags_68 },
    { 0x24e6ea0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd06ea11, SUC_lexicon_tags_11 },
    { 0x3486ea11, SUC_lexicon_tags_65 },
    { 0xa5a6ea11, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa5c6ea15, SUC_lexicon_tags_23 },
    { 0x56a6ea16, SUC_lexicon_tags_0 },
    { 0x9086ea15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d86ea1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a26ea1d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba6ea23, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb846ea2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1966ea33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26ea36, SUC_lexicon_tags_10 },
    { 0xae66ea36, SUC_lexicon_tags_8 },
    { 0x2846ea38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb786ea3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4726ea3f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x37c6ea41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee26ea43, SUC_lexicon_tags_56 },
    { 0xfba6ea44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x43a6ea46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e6ea49, SUC_lexicon_tags_25 },
    { 0xa3c6ea49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7086ea4e, SUC_lexicon_tags_3 },
    { 0x49a6ea4f, SUC_lexicon_tags_17 },
    { 0x3666ea50, SUC_lexicon_tags_1 },
    { 0x70a6ea50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c6ea57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c6ea5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda46ea63, SUC_lexicon_tags_17 },
    { 0xdac6ea64, SUC_lexicon_tags_9 },
    { 0x9b66ea65, SUC_lexicon_tags_37 },
    { 0x96e6ea66, SUC_lexicon_tags_70 },
    { 0x4de6ea66, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a6ea6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7066ea6e, SUC_lexicon_tags_28 },
    { 0xc686ea6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x226ea72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8406ea75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb3a6ea77, SUC_lexicon_tags_14 },
    { 0x50a6ea77, SUC_lexicon_tags_37 },
    { 0xe086ea78, SUC_lexicon_tags_37 },
    { 0xe486ea7a, SUC_lexicon_tags_37 },
    { 0x4566ea7b, SUC_lexicon_tags_29 },
    { 0x5706ea7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41c6ea7e, SUC_lexicon_tags_68 },
    { 0x106ea7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6ea8a, SUC_lexicon_tags_25 },
    { 0xcac6ea8b, SUC_lexicon_tags_29 },
    { 0xcc26ea8a, SUC_lexicon_tags_49 },
    { 0xa806ea8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf06ea8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa6ea93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a6ea95, SUC_lexicon_tags_17 },
    { 0x8786ea95, SUC_lexicon_tags_43 },
    { 0x4c06ea97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3706ea9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81c6ea9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f66eaa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e6eaa7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd766eaac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6ec6eaae, SUC_lexicon_tags_15 },
    { 0x846eaae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b06eab1, SUC_lexicon_tags_18 },
    { 0xee66eab2, SUC_lexicon_tags_26 },
    { 0x3ca6eab2, SUC_lexicon_tags_23 },
    { 0xd246eab3, SUC_lexicon_tags_12 },
    { 0x5c86eab5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5326eab7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a6eaba, SUC_lexicon_tags_25 },
    { 0xd386eabb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf26eabf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb66eac2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d66eaca, SUC_lexicon_tags_45 },
    { 0x4006eacb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x91e6eacd, SUC_lexicon_tags_25 },
    { 0xb5a6eacd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e46ead1, SUC_lexicon_tags_34 },
    { 0x9646ead2, SUC_lexicon_tags_49 },
    { 0xac26ead1, SUC_lexicon_tags_31 },
    { 0x386ead4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c6ead7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a6eada, SUC_lexicon_tags_26 },
    { 0xd126eadb, SUC_lexicon_tags_12 },
    { 0x8326eadc, SUC_lexicon_tags_15 },
    { 0xbd66eadd, SUC_lexicon_tags_34 },
    { 0xc366eadd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6e6eae0, SUC_lexicon_tags_37 },
    { 0x2566eae0, SUC_lexicon_tags_17 },
    { 0x11a6eae0, SUC_lexicon_tags_1 },
    { 0x3386eae2, SUC_lexicon_tags_25 },
    { 0x3946eae4, SUC_lexicon_tags_19 },
    { 0xc586eae4, SUC_lexicon_tags_56 },
    { 0xf766eae6, SUC_lexicon_tags_11 },
    { 0x6c66eae4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc2a6eae9, SUC_lexicon_tags_85 },
    { 0x16c6eaea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e6eaed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a6eaf0, SUC_lexicon_tags_14 },
    { 0xe2e6eaf0, SUC_lexicon_tags_29 },
    { 0x3946eaf1, SUC_lexicon_tags_11 },
    { 0x1366eaf3, SUC_lexicon_tags_151 },
    { 0x5a6eaf4, SUC_lexicon_tags_98 },
    { 0xda06eaf4, SUC_lexicon_tags_15 },
    { 0x9846eaf6, SUC_lexicon_tags_26 },
    { 0x2646eaf6, SUC_lexicon_tags_15 },
    { 0xa26eaf8, SUC_lexicon_tags_37 },
    { 0x8106eaf2, SUC_lexicon_tags_7 },
    { 0xa0e6eafa, SUC_lexicon_tags_25 },
    { 0x42a6eaf6, SUC_lexicon_tags_43 },
    { 0xe826eafc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd986eb05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8c6eb07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7066eb0f, SUC_lexicon_tags_60 },
    { 0x3c26eb10, SUC_lexicon_tags_37 },
    { 0x3926eb0f, SUC_lexicon_tags_66 },
    { 0x83c6eb12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be6eb17, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6b66eb19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e66eb20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26eb23, SUC_lexicon_tags_2 },
    { 0x7b06eb24, SUC_lexicon_tags_45 },
    { 0xb766eb24, SUC_lexicon_tags_25 },
    { 0xdd06eb26, SUC_lexicon_tags_11 },
    { 0xe7c6eb27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc326eb29, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe246eb32, SUC_lexicon_tags_1 },
    { 0xbc06eb32, SUC_lexicon_tags_10 },
    { 0xf466eb34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f66eb37, SUC_lexicon_tags_43 },
    { 0x5826eb38, SUC_lexicon_tags_17 },
    { 0x14a6eb38, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3366eb3d, SUC_lexicon_tags_14 },
    { 0xa486eb3d, SUC_lexicon_tags_11 },
    { 0xfde6eb3f, SUC_lexicon_tags_55 },
    { 0x6426eb3f, SUC_lexicon_tags_17 },
    { 0x85e6eb41, SUC_lexicon_tags_25 },
    { 0x1106eb42, SUC_lexicon_tags_62 },
    { 0x36e6eb3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a6eb46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80a6eb48, SUC_lexicon_tags_17 },
    { 0x8b26eb49, SUC_lexicon_tags_60 },
    { 0x9946eb4a, SUC_lexicon_tags_14 },
    { 0x8f06eb4a, SUC_lexicon_tags_1 },
    { 0x50c6eb4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d06eb50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc6eb53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a26eb56, SUC_lexicon_tags_2 },
    { 0x8986eb57, SUC_lexicon_tags_17 },
    { 0x66e6eb58, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e6eb5f, SUC_lexicon_tags_25 },
    { 0xc586eb5f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e6eb64, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a6eb6a, SUC_lexicon_tags_61 },
    { 0xe4a6eb6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcde6eb6d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaca6eb6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26eb72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7206eb74, SUC_lexicon_tags_7 },
    { 0xd6a6eb74, SUC_lexicon_tags_21 },
    { 0x3766eb76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3466eb78, SUC_lexicon_tags_25 },
    { 0xa1a6eb79, SUC_lexicon_tags_27 },
    { 0x6d06eb7a, SUC_lexicon_tags_17 },
    { 0x9f06eb79, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7486eb86, SUC_lexicon_tags_45 },
    { 0xfc06eb86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b26eb8a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd6c6eb8c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb9e6eb8e, SUC_lexicon_tags_4 },
    { 0xcf06eb8f, SUC_lexicon_tags_24 },
    { 0xe906eb90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c6eb97, SUC_lexicon_tags_1 },
    { 0xfdc6eb98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe966eb9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4946eb9f, SUC_lexicon_tags_15 },
    { 0xd86eba0, SUC_lexicon_tags_12 },
    { 0x4a86eba0, SUC_lexicon_tags_14 },
    { 0xd586eba2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e86eba8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e6ebac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5786ebae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb406ebb2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x92c6ebb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7a6ebb6, SUC_lexicon_tags_37 },
    { 0x82e6ebb7, SUC_lexicon_tags_25 },
    { 0xfa6ebb7, SUC_lexicon_tags_19 },
    { 0xe6ebb8, SUC_lexicon_tags_47 },
    { 0x6106ebba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c6ebbd, SUC_lexicon_tags_25 },
    { 0xfba6ebbd, SUC_lexicon_tags_14 },
    { 0x5226ebbf, SUC_lexicon_tags_25 },
    { 0x5ec6ebc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb526ebc2, SUC_lexicon_tags_11 },
    { 0x78c6ebc2, SUC_lexicon_tags_151 },
    { 0x11e6ebc3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x30c6ebc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a6ebd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7146ebd2, SUC_lexicon_tags_0 },
    { 0x4386ebd3, SUC_lexicon_tags_6 },
    { 0x3686ebd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc6ebd9, SUC_lexicon_tags_31 },
    { 0x9fe6ebd9, SUC_lexicon_tags_1 },
    { 0x3706ebdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d26ebdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa6ebe9, SUC_lexicon_tags_29 },
    { 0x3e86ebea, SUC_lexicon_tags_12 },
    { 0xf3a6ebeb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646ebef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9706ebf5, SUC_lexicon_tags_37 },
    { 0x18e6ebf6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd166ebf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe606ebfc, SUC_lexicon_tags_28 },
    { 0xfda6ebfd, SUC_lexicon_tags_12 },
    { 0xfd66ebfe, SUC_lexicon_tags_1 },
    { 0x5406ebff, SUC_lexicon_tags_29 },
    { 0x6686ec00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a6ec07, SUC_lexicon_tags_13 },
    { 0xa346ec07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb26ec0a, SUC_lexicon_tags_11 },
    { 0x9f46ec0a, SUC_lexicon_tags_47 },
    { 0x5066ec0c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa6ec0f, SUC_lexicon_tags_25 },
    { 0x5826ec10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfda6ec12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33a6ec14, SUC_lexicon_tags_2 },
    { 0x746ec15, SUC_lexicon_tags_37 },
    { 0x806ec15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a6ec1c, SUC_lexicon_tags_15 },
    { 0x1b46ec1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64e6ec1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x666ec22, SUC_lexicon_tags_2 },
    { 0xb306ec23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aa6ec25, SUC_lexicon_tags_21 },
    { 0xada6ec26, SUC_lexicon_tags_1 },
    { 0x21a6ec27, SUC_lexicon_tags_37 },
    { 0xb346ec28, SUC_lexicon_tags_27 },
    { 0x7ce6ec28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e46ec2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff86ec30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd626ec38, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9786ec3b, SUC_lexicon_tags_191 },
    { 0xe986ec3c, SUC_lexicon_tags_23 },
    { 0xa266ec3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3a6ec3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c86ec41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa006ec45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b46ec49, SUC_lexicon_tags_32 },
    { 0x1ae6ec49, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c66ec4f, SUC_lexicon_tags_25 },
    { 0x54e6ec4f, SUC_lexicon_tags_25 },
    { 0xd526ec51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa906ec53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c6ec57, SUC_lexicon_tags_17 },
    { 0x8686ec58, SUC_lexicon_tags_17 },
    { 0xeae6ec58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae26ec5d, SUC_lexicon_tags_43 },
    { 0x47c6ec5d, SUC_lexicon_tags_17 },
    { 0x326ec5f, SUC_lexicon_tags_9 },
    { 0x3d86ec60, SUC_lexicon_tags_2 },
    { 0x1246ec60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e86ec66, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd06ec6b, SUC_lexicon_tags_100 },
    { 0xf466ec6c, SUC_lexicon_tags_11 },
    { 0xd466ec6d, SUC_lexicon_tags_11 },
    { 0x19c6ec6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e6ec74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8da6ec76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf606ec7c, SUC_lexicon_tags_28 },
    { 0x1b66ec7d, SUC_lexicon_tags_37 },
    { 0x3846ec7e, SUC_lexicon_tags_35 },
    { 0x9206ec7f, SUC_lexicon_tags_23 },
    { 0x79a6ec7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf86ec85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc26ec87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc6ec8a, SUC_lexicon_tags_19 },
    { 0x13e6ec8a, SUC_lexicon_tags_38 },
    { 0x7c6ec8c, SUC_lexicon_tags_31 },
    { 0xab66ec8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c6ec91, SUC_lexicon_tags_12 },
    { 0xe466ec91, SUC_lexicon_tags_77 },
    { 0x9a86ec93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e6ec97, SUC_lexicon_tags_37 },
    { 0x71a6ec98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d66ec9a, SUC_lexicon_tags_15 },
    { 0x4206ec9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4686ec9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae46eca3, SUC_lexicon_tags_1127 },
    { 0xa566eca4, SUC_lexicon_tags_17 },
    { 0x1a26eca5, SUC_lexicon_tags_8 },
    { 0xee86eca4, SUC_lexicon_tags_9 },
    { 0xbf26eca7, SUC_lexicon_tags_7 },
    { 0x94e6eca8, SUC_lexicon_tags_23 },
    { 0x4906eca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6486ecab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6ecaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d86ecbc, SUC_lexicon_tags_10 },
    { 0x3f06ecbc, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1666ecc8, SUC_lexicon_tags_31 },
    { 0x5326ecc9, SUC_lexicon_tags_11 },
    { 0x9526ecca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa6ecd3, SUC_lexicon_tags_15 },
    { 0xdb46ecd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3686ecda, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a6ece2, SUC_lexicon_tags_37 },
    { 0x5206ece3, SUC_lexicon_tags_11 },
    { 0xc6a6ece3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee6ece8, SUC_lexicon_tags_0 },
    { 0xf7a6ece8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc266eceb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c6ecef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d66ecf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5586ecf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c6ecf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9606ecfc, SUC_lexicon_tags_17 },
    { 0x6926ecfd, SUC_lexicon_tags_2 },
    { 0xfe26ecfd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5586ed02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb06ed05, SUC_lexicon_tags_11 },
    { 0x6646ed06, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5186ed0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdda6ed0c, SUC_lexicon_tags_3 },
    { 0xf3a6ed0d, SUC_lexicon_tags_7 },
    { 0x2286ed0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe326ed11, SUC_lexicon_tags_106 },
    { 0x6a86ed11, SUC_lexicon_tags_2 },
    { 0x46e6ed13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf486ed18, SUC_lexicon_tags_13 },
    { 0xbf06ed18, SUC_lexicon_tags_31 },
    { 0x3646ed1a, SUC_lexicon_tags_38 },
    { 0xe6a6ed1b, SUC_lexicon_tags_19 },
    { 0x1c26ed19, SUC_lexicon_tags_46 },
    { 0x4686ed1c, SUC_lexicon_tags_39 },
    { 0x8a6ed1e, SUC_lexicon_tags_43 },
    { 0x3186ed1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ae6ed21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa546ed25, SUC_lexicon_tags_40 },
    { 0xda46ed26, SUC_lexicon_tags_25 },
    { 0x3ec6ed25, SUC_lexicon_tags_42 },
    { 0xcfc6ed26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e66ed2a, SUC_lexicon_tags_37 },
    { 0x7c86ed2b, SUC_lexicon_tags_17 },
    { 0x3ca6ed2b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a6ed2f, SUC_lexicon_tags_37 },
    { 0x1ea6ed2f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe526ed33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe426ed38, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2fe6ed3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ba6ed3c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa86ed47, SUC_lexicon_tags_27 },
    { 0x8226ed48, SUC_lexicon_tags_33 },
    { 0x3f46ed49, SUC_lexicon_tags_17 },
    { 0x3f86ed4a, SUC_lexicon_tags_25 },
    { 0x51c6ed4a, SUC_lexicon_tags_7 },
    { 0x4ee6ed4c, SUC_lexicon_tags_2 },
    { 0xf2e6ed4d, SUC_lexicon_tags_25 },
    { 0x6146ed4d, SUC_lexicon_tags_21 },
    { 0xf2e6ed4e, SUC_lexicon_tags_37 },
    { 0x6806ed4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5726ed52, SUC_lexicon_tags_87 },
    { 0xe366ed52, SUC_lexicon_tags_127 },
    { 0x13a6ed52, SUC_lexicon_tags_2 },
    { 0x6a06ed54, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6be6ed57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94c6ed59, SUC_lexicon_tags_6 },
    { 0xe826ed59, SUC_lexicon_tags_11 },
    { 0x43a6ed5b, SUC_lexicon_tags_37 },
    { 0xd546ed5b, SUC_lexicon_tags_46 },
    { 0xbbc6ed5d, SUC_lexicon_tags_2 },
    { 0x9f26ed5d, SUC_lexicon_tags_27 },
    { 0xbea6ed5f, SUC_lexicon_tags_17 },
    { 0x1186ed5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb86ed64, SUC_lexicon_tags_6 },
    { 0x5e86ed65, SUC_lexicon_tags_11 },
    { 0x4b66ed66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf806ed6b, SUC_lexicon_tags_1 },
    { 0x1a86ed6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x906ed6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea66ed70, SUC_lexicon_tags_2 },
    { 0xe4a6ed71, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xe946ed74, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8946ed77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c6ed7a, SUC_lexicon_tags_2 },
    { 0xb3a6ed7b, SUC_lexicon_tags_24 },
    { 0x87e6ed7a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdda6ed7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc6ed81, SUC_lexicon_tags_49 },
    { 0xfa46ed81, SUC_lexicon_tags_39 },
    { 0xfd06ed81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c6ed87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d86ed89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x326ed8c, SUC_lexicon_tags_1 },
    { 0x4326ed8d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5546ed8f, SUC_lexicon_tags_151 },
    { 0x3566ed90, SUC_lexicon_tags_43 },
    { 0xd586ed91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a6ed94, SUC_lexicon_tags_27 },
    { 0x6986ed94, SUC_lexicon_tags_63 },
    { 0xbee6ed96, SUC_lexicon_tags_37 },
    { 0x2366ed97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc6ed9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de6eda0, SUC_lexicon_tags_39 },
    { 0x9886eda0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x386eda4, SUC_lexicon_tags_25 },
    { 0xe606eda5, SUC_lexicon_tags_91 },
    { 0x4846eda6, SUC_lexicon_tags_84 },
    { 0xff86eda7, SUC_lexicon_tags_37 },
    { 0xf6a6eda7, SUC_lexicon_tags_47 },
    { 0xe846eda7, SUC_lexicon_tags_32 },
    { 0x3c06edaa, SUC_lexicon_tags_23 },
    { 0x2186eda5, SUC_lexicon_tags_48 },
    { 0x5166eda5, SUC_lexicon_tags_37 },
    { 0x1586edad, SUC_lexicon_tags_7 },
    { 0xcce6edad, SUC_lexicon_tags_37 },
    { 0x9a26edae, SUC_lexicon_tags_49 },
    { 0xbb86edae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c46edb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3746edb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe6edba, SUC_lexicon_tags_9 },
    { 0x98e6edbb, SUC_lexicon_tags_1 },
    { 0xde6edba, SUC_lexicon_tags_31 },
    { 0x5246edbd, SUC_lexicon_tags_40 },
    { 0x2b86edbd, SUC_lexicon_tags_17 },
    { 0x2866edbe, SUC_lexicon_tags_27 },
    { 0x5b26edc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe126edc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa366edc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee6edcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee6edd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c6edd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6386eddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae46edde, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x66ede0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc6ede3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xca66ede6, SUC_lexicon_tags_25 },
    { 0xa9c6ede6, SUC_lexicon_tags_49 },
    { 0x3e26ede8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e66edf0, SUC_lexicon_tags_3 },
    { 0x85a6edf0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b86edf4, SUC_lexicon_tags_87 },
    { 0x51a6edf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2406edfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee6edff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc6ee02, SUC_lexicon_tags_82 },
    { 0xab26ee02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da6ee06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e6ee09, SUC_lexicon_tags_19 },
    { 0x9f06ee09, SUC_lexicon_tags_29 },
    { 0xb126ee0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1606ee0d, SUC_lexicon_tags_40 },
    { 0x20c6ee0e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b06ee11, SUC_lexicon_tags_0 },
    { 0x44c6ee11, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2146ee14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36a6ee16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc786ee18, SUC_lexicon_tags_12 },
    { 0x40e6ee18, SUC_lexicon_tags_25 },
    { 0xd86ee1a, SUC_lexicon_tags_12 },
    { 0x1b66ee1b, SUC_lexicon_tags_47 },
    { 0x1d26ee1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb026ee1e, SUC_lexicon_tags_11 },
    { 0x6806ee1f, SUC_lexicon_tags_11 },
    { 0x4826ee1f, SUC_lexicon_tags_63 },
    { 0xb426ee1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc406ee23, SUC_lexicon_tags_45 },
    { 0x3c66ee24, SUC_lexicon_tags_17 },
    { 0x5986ee25, SUC_lexicon_tags_6 },
    { 0x4c6ee26, SUC_lexicon_tags_9 },
    { 0x82a6ee25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86c6ee29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8146ee2b, SUC_lexicon_tags_0 },
    { 0x3266ee2b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f86ee2f, SUC_lexicon_tags_7 },
    { 0x84c6ee30, SUC_lexicon_tags_6 },
    { 0x7446ee31, SUC_lexicon_tags_87 },
    { 0x9e6ee30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2546ee34, SUC_lexicon_tags_0 },
    { 0xfaa6ee34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfc6ee37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1186ee3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc6ee3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe246ee41, SUC_lexicon_tags_39 },
    { 0xffc6ee42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b26ee44, SUC_lexicon_tags_12 },
    { 0xf926ee45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3546ee47, SUC_lexicon_tags_0 },
    { 0xaa6ee48, SUC_lexicon_tags_2 },
    { 0x1946ee48, SUC_lexicon_tags_2 },
    { 0x4a06ee4a, SUC_lexicon_tags_27 },
    { 0xa3c6ee4b, SUC_lexicon_tags_11 },
    { 0x7506ee4b, SUC_lexicon_tags_49 },
    { 0x3186ee4d, SUC_lexicon_tags_63 },
    { 0x1be6ee4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe6ee51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a06ee53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7646ee55, SUC_lexicon_tags_19 },
    { 0xcb66ee55, SUC_lexicon_tags_8 },
    { 0xa686ee56, SUC_lexicon_tags_12 },
    { 0xcde6ee55, SUC_lexicon_tags_45 },
    { 0x3446ee56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1226ee5c, SUC_lexicon_tags_3 },
    { 0xc3c6ee5d, SUC_lexicon_tags_12 },
    { 0x6666ee5e, SUC_lexicon_tags_6 },
    { 0x5ec6ee5f, SUC_lexicon_tags_46 },
    { 0xdaa6ee5f, SUC_lexicon_tags_12 },
    { 0x2d46ee61, SUC_lexicon_tags_37 },
    { 0xea06ee5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566ee6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca6ee72, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc6ee76, SUC_lexicon_tags_32 },
    { 0xc066ee77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x686ee79, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da6ee7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda6ee7f, SUC_lexicon_tags_6 },
    { 0x326ee80, SUC_lexicon_tags_2 },
    { 0x1046ee81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9206ee83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c06ee85, SUC_lexicon_tags_2 },
    { 0xa466ee86, SUC_lexicon_tags_12 },
    { 0x5246ee87, SUC_lexicon_tags_15 },
    { 0x5c66ee88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3926ee8d, SUC_lexicon_tags_15 },
    { 0x7d46ee8e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4a6ee90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x11a6ee92, SUC_lexicon_tags_11 },
    { 0xcb66ee93, SUC_lexicon_tags_37 },
    { 0x1ac6ee94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b26ee99, SUC_lexicon_tags_14 },
    { 0x746ee9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe626ee9c, SUC_lexicon_tags_37 },
    { 0x1906ee9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb566ee9f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766eea3, SUC_lexicon_tags_34 },
    { 0x74c6eea3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae6eea7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4bc6eea9, SUC_lexicon_tags_14 },
    { 0x24c6eeaa, SUC_lexicon_tags_12 },
    { 0x17a6eeab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7366eead, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd046eeb3, SUC_lexicon_tags_100 },
    { 0x5806eeb3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e46eeb7, SUC_lexicon_tags_66 },
    { 0x1466eeb7, SUC_lexicon_tags_1 },
    { 0x7246eeb9, SUC_lexicon_tags_37 },
    { 0x1a66eeb9, SUC_lexicon_tags_30 },
    { 0xe6c6eebb, SUC_lexicon_tags_11 },
    { 0xa4e6eeb9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa806eebf, SUC_lexicon_tags_2 },
    { 0x6f86eec0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5026eec4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e6eec8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfe6eeca, SUC_lexicon_tags_60 },
    { 0x4826eecb, SUC_lexicon_tags_17 },
    { 0xe6e6eeca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe466eece, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde66eed4, SUC_lexicon_tags_30 },
    { 0xb046eed5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35a6eed7, SUC_lexicon_tags_12 },
    { 0xbc66eed7, SUC_lexicon_tags_9 },
    { 0x9a86eed9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x946eedb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd786eede, SUC_lexicon_tags_15 },
    { 0xa8e6eedf, SUC_lexicon_tags_11 },
    { 0xf906eedf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa6eee3, SUC_lexicon_tags_17 },
    { 0x6fc6eee4, SUC_lexicon_tags_8 },
    { 0xf006eee3, SUC_lexicon_tags_15 },
    { 0x2966eee3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f66eeed, SUC_lexicon_tags_14 },
    { 0x67a6eeee, SUC_lexicon_tags_40 },
    { 0x2be6eeef, SUC_lexicon_tags_6 },
    { 0xf8c6eef0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc6eef5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfe86eef7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb806eef9, SUC_lexicon_tags_16 },
    { 0xe286eef9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc6eefe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5806ef00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c6ef03, SUC_lexicon_tags_49 },
    { 0x6b06ef04, SUC_lexicon_tags_7 },
    { 0x7ec6ef05, SUC_lexicon_tags_28 },
    { 0x1ca6ef06, SUC_lexicon_tags_37 },
    { 0xd166ef06, SUC_lexicon_tags_27 },
    { 0x7f86ef04, SUC_lexicon_tags_15 },
    { 0x86e6ef09, SUC_lexicon_tags_7 },
    { 0x2fa6ef08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa406ef0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6c46ef11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2fe6ef13, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xf0a6ef15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4926ef18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1946ef1b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9846ef1f, SUC_lexicon_tags_47 },
    { 0xe1e6ef20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7c6ef22, SUC_lexicon_tags_25 },
    { 0x5fe6ef23, SUC_lexicon_tags_7 },
    { 0xdca6ef23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb946ef2e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb8a6ef30, SUC_lexicon_tags_12 },
    { 0x9826ef31, SUC_lexicon_tags_12 },
    { 0xf426ef32, SUC_lexicon_tags_37 },
    { 0x15e6ef30, SUC_lexicon_tags_2 },
    { 0xe66ef32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x566ef36, SUC_lexicon_tags_14 },
    { 0x18c6ef37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a6ef3a, SUC_lexicon_tags_25 },
    { 0xad46ef3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e66ef3d, SUC_lexicon_tags_12 },
    { 0xdbe6ef3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a6ef41, SUC_lexicon_tags_63 },
    { 0xfa06ef42, SUC_lexicon_tags_25 },
    { 0x5426ef43, SUC_lexicon_tags_7 },
    { 0x7666ef44, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f06ef47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd306ef4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d46ef4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8566ef4f, SUC_lexicon_tags_25 },
    { 0x5946ef50, SUC_lexicon_tags_15 },
    { 0xd266ef51, SUC_lexicon_tags_22 },
    { 0x4a26ef52, SUC_lexicon_tags_12 },
    { 0xa146ef51, SUC_lexicon_tags_29 },
    { 0xdca6ef54, SUC_lexicon_tags_12 },
    { 0x5b46ef51, SUC_lexicon_tags_61 },
    { 0xec26ef55, SUC_lexicon_tags_38 },
    { 0x82a6ef57, SUC_lexicon_tags_63 },
    { 0x60a6ef52, SUC_lexicon_tags_11 },
    { 0x35e6ef50, SUC_lexicon_tags_43 },
    { 0xe9e6ef58, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeb86ef5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xef26ef5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf686ef63, SUC_lexicon_tags_1 },
    { 0xc186ef64, SUC_lexicon_tags_34 },
    { 0xaa66ef65, SUC_lexicon_tags_14 },
    { 0x7706ef66, SUC_lexicon_tags_23 },
    { 0xf746ef65, SUC_lexicon_tags_27 },
    { 0x6d06ef68, SUC_lexicon_tags_33 },
    { 0xa026ef69, SUC_lexicon_tags_17 },
    { 0x9b66ef67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a6ef70, SUC_lexicon_tags_1 },
    { 0x9666ef71, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa6ef75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7246ef77, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6066ef7a, SUC_lexicon_tags_12 },
    { 0xdee6ef7b, SUC_lexicon_tags_1 },
    { 0x5806ef7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7686ef81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c6ef85, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaa66ef87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7006ef89, SUC_lexicon_tags_43 },
    { 0xfec6ef8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4826ef8c, SUC_lexicon_tags_3 },
    { 0xad46ef8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf9a6ef8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a6ef99, SUC_lexicon_tags_15 },
    { 0xcbe6ef9a, SUC_lexicon_tags_7 },
    { 0xfe66ef9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba6ef9f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6466efa4, SUC_lexicon_tags_7 },
    { 0x49c6efa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c6efad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca86efb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b26efb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5006efb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf26efb9, SUC_lexicon_tags_33 },
    { 0x2886efb9, SUC_lexicon_tags_37 },
    { 0xe1a6efbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1046efbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7866efc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d26efc4, SUC_lexicon_tags_49 },
    { 0x2a66efc5, SUC_lexicon_tags_11 },
    { 0x7346efc6, SUC_lexicon_tags_12 },
    { 0x5246efc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a6efcc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e6efd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xefe6efd6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9046efda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4be6efdc, SUC_lexicon_tags_47 },
    { 0x8146efdd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd166efe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c6efe3, SUC_lexicon_tags_39 },
    { 0x7ca6efe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa586efea, SUC_lexicon_tags_10 },
    { 0x1846efea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec6eff6, SUC_lexicon_tags_15 },
    { 0x9986eff6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a46effb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9d66effd, SUC_lexicon_tags_30 },
    { 0x8ac6effe, SUC_lexicon_tags_37 },
    { 0x1b66efff, SUC_lexicon_tags_16 },
    { 0x6c46efff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06f00c, SUC_lexicon_tags_7 },
    { 0x21e6f00d, SUC_lexicon_tags_17 },
    { 0x1c46f00e, SUC_lexicon_tags_37 },
    { 0x7086f00e, SUC_lexicon_tags_55 },
    { 0xc2a6f00e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1fa6f012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeda6f014, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3626f019, SUC_lexicon_tags_17 },
    { 0xf926f019, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa866f01c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe6f022, SUC_lexicon_tags_19 },
    { 0x8bc6f023, SUC_lexicon_tags_37 },
    { 0x6ec6f023, SUC_lexicon_tags_29 },
    { 0x4606f023, SUC_lexicon_tags_15 },
    { 0x8826f024, SUC_lexicon_tags_11 },
    { 0xbaa6f027, SUC_lexicon_tags_37 },
    { 0xdc86f028, SUC_lexicon_tags_7 },
    { 0x2706f027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2806f02b, SUC_lexicon_tags_4 },
    { 0x5b66f02b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x64c6f02e, SUC_lexicon_tags_0 },
    { 0xf306f02f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa086f031, SUC_lexicon_tags_7 },
    { 0x8106f032, SUC_lexicon_tags_21 },
    { 0xb946f033, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3006f036, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58a6f038, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7166f03b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc086f03e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6f045, SUC_lexicon_tags_9 },
    { 0xa406f045, SUC_lexicon_tags_17 },
    { 0x1c26f045, SUC_lexicon_tags_16 },
    { 0x1b46f046, SUC_lexicon_tags_37 },
    { 0x3046f046, SUC_lexicon_tags_25 },
    { 0x9f66f04a, SUC_lexicon_tags_25 },
    { 0xcc26f04a, SUC_lexicon_tags_0 },
    { 0x6566f04c, SUC_lexicon_tags_45 },
    { 0x5666f04d, SUC_lexicon_tags_0 },
    { 0x3706f04c, SUC_lexicon_tags_47 },
    { 0x32e6f04c, SUC_lexicon_tags_11 },
    { 0x23a6f050, SUC_lexicon_tags_25 },
    { 0xb3a6f050, SUC_lexicon_tags_7 },
    { 0xd526f050, SUC_lexicon_tags_21 },
    { 0xf666f053, SUC_lexicon_tags_31 },
    { 0xfd06f054, SUC_lexicon_tags_0 },
    { 0x7286f055, SUC_lexicon_tags_38 },
    { 0xf726f056, SUC_lexicon_tags_61 },
    { 0xe686f053, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa546f059, SUC_lexicon_tags_60 },
    { 0x4e66f059, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4866f060, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186f064, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e6f067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd426f06a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c6f06d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8486f071, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d26f073, SUC_lexicon_tags_16 },
    { 0x53c6f074, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06f078, SUC_lexicon_tags_25 },
    { 0xa066f079, SUC_lexicon_tags_31 },
    { 0x72e6f07a, SUC_lexicon_tags_65 },
    { 0xf86f07b, SUC_lexicon_tags_60 },
    { 0x2386f079, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6f07f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc5c6f081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12e6f083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x106f086, SUC_lexicon_tags_4 },
    { 0xb886f087, SUC_lexicon_tags_17 },
    { 0x2fa6f088, SUC_lexicon_tags_1 },
    { 0xcb46f089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe166f090, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ac6f092, SUC_lexicon_tags_9 },
    { 0x2fe6f092, SUC_lexicon_tags_46 },
    { 0x90e6f094, SUC_lexicon_tags_27 },
    { 0x52a6f094, SUC_lexicon_tags_7 },
    { 0x19e6f094, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8606f09a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4a86f09c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3506f09f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d46f0a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa706f0ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e6f0b2, SUC_lexicon_tags_2 },
    { 0x92c6f0b3, SUC_lexicon_tags_47 },
    { 0x8966f0b4, SUC_lexicon_tags_11 },
    { 0xdec6f0b5, SUC_lexicon_tags_61 },
    { 0x41a6f0b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf66f0b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5646f0c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c6f0c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae06f0cd, SUC_lexicon_tags_37 },
    { 0xe6f0ce, SUC_lexicon_tags_113 },
    { 0x9aa6f0ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e6f0d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a6f0db, SUC_lexicon_tags_14 },
    { 0xf186f0db, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a6f0df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c6f0e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf046f0eb, SUC_lexicon_tags_0 },
    { 0x8f06f0ec, SUC_lexicon_tags_26 },
    { 0xc306f0ed, SUC_lexicon_tags_15 },
    { 0x34c6f0ed, SUC_lexicon_tags_11 },
    { 0x7726f0ef, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd826f0f1, SUC_lexicon_tags_30 },
    { 0x3bc6f0f2, SUC_lexicon_tags_37 },
    { 0x6806f0f3, SUC_lexicon_tags_12 },
    { 0x38c6f0f3, SUC_lexicon_tags_69 },
    { 0x4f06f0f5, SUC_lexicon_tags_11 },
    { 0x7b46f0f4, SUC_lexicon_tags_25 },
    { 0xac06f0f7, SUC_lexicon_tags_7 },
    { 0x866f0f6, SUC_lexicon_tags_56 },
    { 0xee66f0f9, SUC_lexicon_tags_2 },
    { 0x3f26f0fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed26f0fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf46f0ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa326f10e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x39c6f110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabe6f112, SUC_lexicon_tags_28 },
    { 0xbee6f113, SUC_lexicon_tags_8 },
    { 0x386f114, SUC_lexicon_tags_27 },
    { 0x186f114, SUC_lexicon_tags_77 },
    { 0x13a6f115, SUC_lexicon_tags_31 },
    { 0xefe6f113, SUC_lexicon_tags_37 },
    { 0xae06f118, SUC_lexicon_tags_12 },
    { 0x73c6f119, SUC_lexicon_tags_63 },
    { 0x4f26f11a, SUC_lexicon_tags_11 },
    { 0x89c6f11b, SUC_lexicon_tags_2 },
    { 0xfe66f11c, SUC_lexicon_tags_10 },
    { 0x1f66f11d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe486f120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe6f124, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc886f128, SUC_lexicon_tags_17 },
    { 0x8a6f128, SUC_lexicon_tags_15 },
    { 0x8366f129, SUC_lexicon_tags_48 },
    { 0xce26f12b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea6f12e, SUC_lexicon_tags_2 },
    { 0x2106f12f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd06f131, SUC_lexicon_tags_37 },
    { 0x59a6f131, SUC_lexicon_tags_11 },
    { 0x4726f133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39c6f135, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1346f137, SUC_lexicon_tags_87 },
    { 0xb486f137, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c6f13b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae46f13f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ee6f141, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d26f150, SUC_lexicon_tags_17 },
    { 0x5ba6f151, SUC_lexicon_tags_62 },
    { 0x5ba6f152, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d86f154, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x25e6f156, SUC_lexicon_tags_12 },
    { 0x3146f156, SUC_lexicon_tags_12 },
    { 0xacc6f156, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a46f15e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa406f161, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a06f167, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3666f171, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe486f174, SUC_lexicon_tags_7 },
    { 0xc506f175, SUC_lexicon_tags_37 },
    { 0x62a6f176, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e6f179, SUC_lexicon_tags_25 },
    { 0xc846f17a, SUC_lexicon_tags_27 },
    { 0x9426f179, SUC_lexicon_tags_25 },
    { 0xe346f17c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede6f180, SUC_lexicon_tags_9 },
    { 0x7566f181, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd66f186, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e6f191, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb006f196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ca6f198, SUC_lexicon_tags_96 },
    { 0x7a06f198, SUC_lexicon_tags_45 },
    { 0x3c86f19a, SUC_lexicon_tags_37 },
    { 0x2ac6f19b, SUC_lexicon_tags_48 },
    { 0xd0e6f19c, SUC_lexicon_tags_47 },
    { 0xaec6f199, SUC_lexicon_tags_2 },
    { 0xf6c6f19e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf786f1a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13e6f1a2, SUC_lexicon_tags_11 },
    { 0x9846f1a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e6f1a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa126f1a8, SUC_lexicon_tags_31 },
    { 0x65c6f1a9, SUC_lexicon_tags_43 },
    { 0xb8c6f1a9, SUC_lexicon_tags_47 },
    { 0x6506f1aa, SUC_lexicon_tags_17 },
    { 0xc866f1ac, SUC_lexicon_tags_77 },
    { 0xbb66f1ad, SUC_lexicon_tags_25 },
    { 0x9a46f1ae, SUC_lexicon_tags_17 },
    { 0xf046f1a8, SUC_lexicon_tags_16 },
    { 0xca06f1b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6906f1b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c6f1b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc6f1bc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a6f1c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf26f1c7, SUC_lexicon_tags_96 },
    { 0x56e6f1c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e6f1cf, SUC_lexicon_tags_29 },
    { 0x7c66f1d0, SUC_lexicon_tags_19 },
    { 0xc7c6f1d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11c6f1d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3e6f1d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5126f1d9, SUC_lexicon_tags_21 },
    { 0x5186f1da, SUC_lexicon_tags_45 },
    { 0x2e6f1db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc086f1dd, SUC_lexicon_tags_7 },
    { 0x7146f1de, SUC_lexicon_tags_1 },
    { 0x81c6f1df, SUC_lexicon_tags_49 },
    { 0x2dc6f1df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5486f1e5, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb426f1e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76c6f1eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f46f1ed, SUC_lexicon_tags_45 },
    { 0x9986f1ee, SUC_lexicon_tags_1 },
    { 0xa0a6f1ef, SUC_lexicon_tags_7 },
    { 0x15e6f1f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1e6f1f2, SUC_lexicon_tags_60 },
    { 0xd9a6f1f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a86f1f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c6f1f9, SUC_lexicon_tags_15 },
    { 0x7846f1fa, SUC_lexicon_tags_0 },
    { 0xdb66f1fb, SUC_lexicon_tags_12 },
    { 0x9626f1fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf926f200, SUC_lexicon_tags_7 },
    { 0xa266f201, SUC_lexicon_tags_0 },
    { 0xd386f202, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9226f204, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b06f207, SUC_lexicon_tags_47 },
    { 0x4ba6f208, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xa926f20a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c6f20f, SUC_lexicon_tags_60 },
    { 0x2a6f210, SUC_lexicon_tags_12 },
    { 0x4c46f210, SUC_lexicon_tags_32 },
    { 0xdf06f212, SUC_lexicon_tags_6 },
    { 0x14c6f213, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a6f21a, SUC_lexicon_tags_2 },
    { 0x2fa6f21b, SUC_lexicon_tags_1 },
    { 0x7686f21b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3406f222, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3006f224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x166f227, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc86f22a, SUC_lexicon_tags_45 },
    { 0xd5e6f22a, SUC_lexicon_tags_37 },
    { 0x3686f22c, SUC_lexicon_tags_10 },
    { 0x6f86f22d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe906f230, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a6f23b, SUC_lexicon_tags_87 },
    { 0xcd86f23c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb726f23e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1926f241, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2826f244, SUC_lexicon_tags_30 },
    { 0xdf66f245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a6f249, SUC_lexicon_tags_12 },
    { 0x4e66f24a, SUC_lexicon_tags_23 },
    { 0x5fc6f24a, SUC_lexicon_tags_0 },
    { 0x2b26f24c, SUC_lexicon_tags_45 },
    { 0x4f66f24d, SUC_lexicon_tags_32 },
    { 0x4f66f24e, SUC_lexicon_tags_25 },
    { 0x4ca6f24e, SUC_lexicon_tags_49 },
    { 0x626f250, SUC_lexicon_tags_17 },
    { 0x3826f24f, SUC_lexicon_tags_45 },
    { 0xc466f252, SUC_lexicon_tags_0 },
    { 0xc746f252, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a6f256, SUC_lexicon_tags_2 },
    { 0x36a6f257, SUC_lexicon_tags_38 },
    { 0x1a46f257, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746f25c, SUC_lexicon_tags_14 },
    { 0xa3c6f25d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6186f260, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7606f263, SUC_lexicon_tags_49 },
    { 0xeac6f264, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6046f26b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9546f26d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2c6f26f, SUC_lexicon_tags_17 },
    { 0x2686f26f, SUC_lexicon_tags_5 },
    { 0x706f271, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c6f274, SUC_lexicon_tags_25 },
    { 0x1446f275, SUC_lexicon_tags_318 },
    { 0xaa66f276, SUC_lexicon_tags_61 },
    { 0xdf86f277, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3de6f279, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4066f27c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe86f27e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d06f289, SUC_lexicon_tags_2 },
    { 0xfb66f289, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f86f28c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7006f294, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e6f297, SUC_lexicon_tags_28 },
    { 0x2f46f297, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb06f29d, SUC_lexicon_tags_19 },
    { 0xbe06f29e, SUC_lexicon_tags_25 },
    { 0x4186f29f, SUC_lexicon_tags_14 },
    { 0x8bc6f29e, SUC_lexicon_tags_15 },
    { 0xa426f29d, SUC_lexicon_tags_0 },
    { 0x4186f2a2, SUC_lexicon_tags_16 },
    { 0x97c6f2a1, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6f2a8, SUC_lexicon_tags_38 },
    { 0xe86f2a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc066f2ac, SUC_lexicon_tags_56 },
    { 0xf306f2ad, SUC_lexicon_tags_26 },
    { 0xb386f2ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc6f2b3, SUC_lexicon_tags_37 },
    { 0xc926f2b3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x85c6f2b6, SUC_lexicon_tags_349 },
    { 0x84c6f2b7, SUC_lexicon_tags_42 },
    { 0xd926f2b8, SUC_lexicon_tags_14 },
    { 0xbfe6f2b9, SUC_lexicon_tags_49 },
    { 0x27c6f2b8, SUC_lexicon_tags_2 },
    { 0xb366f2b6, SUC_lexicon_tags_1 },
    { 0x46c6f2bc, SUC_lexicon_tags_37 },
    { 0x2506f2bd, SUC_lexicon_tags_10 },
    { 0x3a86f2be, SUC_lexicon_tags_74 },
    { 0x5e6f2bf, SUC_lexicon_tags_63 },
    { 0x54e6f2c0, SUC_lexicon_tags_37 },
    { 0x2766f2be, SUC_lexicon_tags_47 },
    { 0x4d26f2b7, SUC_lexicon_tags_25 },
    { 0x5046f2c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc66f2c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6f2ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x966f2cc, SUC_lexicon_tags_63 },
    { 0x1e66f2cd, SUC_lexicon_tags_25 },
    { 0x406f2ce, SUC_lexicon_tags_15 },
    { 0x846f2cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a6f2d4, SUC_lexicon_tags_1 },
    { 0x70e6f2d5, SUC_lexicon_tags_7 },
    { 0xd386f2d4, SUC_lexicon_tags_39 },
    { 0x3726f2d7, SUC_lexicon_tags_48 },
    { 0xee06f2d8, SUC_lexicon_tags_1 },
    { 0x8a66f2d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5726f2dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbce6f2de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e6f2e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa126f2ec, SUC_lexicon_tags_17 },
    { 0x8426f2ed, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xcf86f2ef, SUC_lexicon_tags_2 },
    { 0x69a6f2f0, SUC_lexicon_tags_38 },
    { 0x62a6f2f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e6f2f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a46f2f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46f2f9, SUC_lexicon_tags_11 },
    { 0x7c46f2fa, SUC_lexicon_tags_29 },
    { 0x27c6f2fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf66f2fd, SUC_lexicon_tags_17 },
    { 0x7486f2fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f26f300, SUC_lexicon_tags_15 },
    { 0x2326f301, SUC_lexicon_tags_863 },
    { 0, NULL },
    { 0x88a6f303, SUC_lexicon_tags_2 },
    { 0x3706f304, SUC_lexicon_tags_11 },
    { 0x26c6f305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a6f308, SUC_lexicon_tags_11 },
    { 0x3ce6f309, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc666f30e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xac86f310, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a6f314, SUC_lexicon_tags_25 },
    { 0x55e6f315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6266f317, SUC_lexicon_tags_33 },
    { 0x4be6f317, SUC_lexicon_tags_82 },
    { 0x9aa6f318, SUC_lexicon_tags_29 },
    { 0x1106f319, SUC_lexicon_tags_81 },
    { 0xe166f31b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb66f31e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9286f321, SUC_lexicon_tags_31 },
    { 0xd6c6f322, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9986f328, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d66f32a, SUC_lexicon_tags_47 },
    { 0x5646f32b, SUC_lexicon_tags_11 },
    { 0xc726f32c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da6f32f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a26f333, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e86f335, SUC_lexicon_tags_140 },
    { 0x9026f336, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06f33f, SUC_lexicon_tags_17 },
    { 0x11a6f340, SUC_lexicon_tags_17 },
    { 0xc006f33f, SUC_lexicon_tags_68 },
    { 0xbb06f33f, SUC_lexicon_tags_6 },
    { 0xadc6f343, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1066f345, SUC_lexicon_tags_45 },
    { 0x126f345, SUC_lexicon_tags_2 },
    { 0xa626f347, SUC_lexicon_tags_106 },
    { 0x4a26f348, SUC_lexicon_tags_14 },
    { 0x8d06f347, SUC_lexicon_tags_280 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f34d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb366f357, SUC_lexicon_tags_21 },
    { 0xda66f358, SUC_lexicon_tags_45 },
    { 0xb726f358, SUC_lexicon_tags_68 },
    { 0x5f06f35a, SUC_lexicon_tags_1 },
    { 0xab66f35b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95a6f35d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x846f360, SUC_lexicon_tags_17 },
    { 0x8246f361, SUC_lexicon_tags_17 },
    { 0x5506f362, SUC_lexicon_tags_79 },
    { 0x9fe6f363, SUC_lexicon_tags_55 },
    { 0x8ba6f363, SUC_lexicon_tags_15 },
    { 0x8ee6f360, SUC_lexicon_tags_7 },
    { 0x8346f365, SUC_lexicon_tags_2 },
    { 0xa506f364, SUC_lexicon_tags_15 },
    { 0xd686f360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a6f36f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3426f374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70a6f376, SUC_lexicon_tags_32 },
    { 0xb526f376, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd406f379, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5ce6f37b, SUC_lexicon_tags_14 },
    { 0xe66f37c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa106f37e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6f382, SUC_lexicon_tags_27 },
    { 0xe5c6f382, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d26f385, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e6f388, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78a6f38a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe666f38c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3746f392, SUC_lexicon_tags_12 },
    { 0x4c46f393, SUC_lexicon_tags_6 },
    { 0x9e66f393, SUC_lexicon_tags_33 },
    { 0x31c6f395, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x51a6f397, SUC_lexicon_tags_11 },
    { 0x90a6f397, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a6f39c, SUC_lexicon_tags_12 },
    { 0x83a6f39c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd286f3a0, SUC_lexicon_tags_39 },
    { 0x9e66f3a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac6f3a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e6f3ab, SUC_lexicon_tags_43 },
    { 0x85c6f3ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e6f3b5, SUC_lexicon_tags_47 },
    { 0xfde6f3b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe46f3c3, SUC_lexicon_tags_33 },
    { 0x7906f3c3, SUC_lexicon_tags_4 },
    { 0x1626f3c5, SUC_lexicon_tags_11 },
    { 0x1946f3c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac86f3ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff26f3d0, SUC_lexicon_tags_47 },
    { 0x2286f3d1, SUC_lexicon_tags_37 },
    { 0x7aa6f3d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf46f3d7, SUC_lexicon_tags_47 },
    { 0xce66f3d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b46f3da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2466f3df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc6f3e5, SUC_lexicon_tags_37 },
    { 0x30a6f3e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdbc6f3e8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2266f3eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a6f3ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd186f3f4, SUC_lexicon_tags_25 },
    { 0xe3a6f3f5, SUC_lexicon_tags_3 },
    { 0xb8e6f3f6, SUC_lexicon_tags_13 },
    { 0x51a6f3f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4746f3fe, SUC_lexicon_tags_11 },
    { 0xafc6f3fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc466f401, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xc3e6f403, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe786f407, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e86f40a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9ec6f40c, SUC_lexicon_tags_82 },
    { 0xa6a6f40d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc6f412, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246f416, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8066f41d, SUC_lexicon_tags_38 },
    { 0x6de6f41d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb146f420, SUC_lexicon_tags_25 },
    { 0xf2a6f420, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7886f423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6886f428, SUC_lexicon_tags_70 },
    { 0x83e6f428, SUC_lexicon_tags_14 },
    { 0x9de6f42a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e6f42d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf46f42f, SUC_lexicon_tags_17 },
    { 0x73a6f430, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf746f432, SUC_lexicon_tags_46 },
    { 0x2146f432, SUC_lexicon_tags_47 },
    { 0x11e6f434, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x87a6f436, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc6f438, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3926f43c, SUC_lexicon_tags_1 },
    { 0x4dc6f43d, SUC_lexicon_tags_15 },
    { 0xbd86f43d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9966f443, SUC_lexicon_tags_17 },
    { 0x56a6f444, SUC_lexicon_tags_37 },
    { 0xd626f444, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c66f44c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5806f452, SUC_lexicon_tags_12 },
    { 0x4306f453, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2726f455, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc66f457, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb506f45e, SUC_lexicon_tags_9 },
    { 0x6be6f45f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb46f462, SUC_lexicon_tags_25 },
    { 0x6c6f463, SUC_lexicon_tags_14 },
    { 0xf66f464, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc526f466, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1686f469, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7026f46b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc5a6f46d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e6f472, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe166f474, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a6f478, SUC_lexicon_tags_16 },
    { 0xd3e6f478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a6f47c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe06f481, SUC_lexicon_tags_47 },
    { 0xa546f481, SUC_lexicon_tags_14 },
    { 0x7206f481, SUC_lexicon_tags_12 },
    { 0xfdc6f484, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7aa6f486, SUC_lexicon_tags_2 },
    { 0xf746f487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e6f48b, SUC_lexicon_tags_25 },
    { 0x7266f48b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b06f48f, SUC_lexicon_tags_70 },
    { 0xd146f490, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b66f496, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a06f499, SUC_lexicon_tags_54 },
    { 0x21c6f49a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d06f4a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5006f4a4, SUC_lexicon_tags_1 },
    { 0x10c6f4a5, SUC_lexicon_tags_37 },
    { 0xeb86f4a6, SUC_lexicon_tags_14 },
    { 0x9086f4a7, SUC_lexicon_tags_2 },
    { 0x1ec6f4a8, SUC_lexicon_tags_4 },
    { 0x9c46f4a8, SUC_lexicon_tags_37 },
    { 0x7ea6f4a8, SUC_lexicon_tags_38 },
    { 0xc0c6f4a9, SUC_lexicon_tags_13 },
    { 0xac46f4ac, SUC_lexicon_tags_11 },
    { 0x74e6f4aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e6f4b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x406f4b8, SUC_lexicon_tags_25 },
    { 0xe3a6f4b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e6f4bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7406f4c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85e6f4c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7266f4ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d86f4d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5046f4d5, SUC_lexicon_tags_30 },
    { 0x3526f4d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7906f4d8, SUC_lexicon_tags_23 },
    { 0x6026f4d9, SUC_lexicon_tags_48 },
    { 0x5546f4da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7ea6f4dc, SUC_lexicon_tags_7 },
    { 0x7946f4dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5106f4e2, SUC_lexicon_tags_17 },
    { 0xe3a6f4e3, SUC_lexicon_tags_47 },
    { 0x89e6f4e3, SUC_lexicon_tags_17 },
    { 0x9c86f4e5, SUC_lexicon_tags_38 },
    { 0x2ce6f4e3, SUC_lexicon_tags_10 },
    { 0x15a6f4e7, SUC_lexicon_tags_38 },
    { 0x1c6f4e4, SUC_lexicon_tags_14 },
    { 0x17c6f4e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xba46f4eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd7a6f4ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe806f4ef, SUC_lexicon_tags_25 },
    { 0x2b86f4f0, SUC_lexicon_tags_29 },
    { 0x7a06f4ef, SUC_lexicon_tags_15 },
    { 0xf466f4f2, SUC_lexicon_tags_1 },
    { 0x34a6f4f3, SUC_lexicon_tags_33 },
    { 0x1886f4f3, SUC_lexicon_tags_11 },
    { 0x2a66f4f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d26f4fa, SUC_lexicon_tags_47 },
    { 0xf426f4fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x63e6f4fd, SUC_lexicon_tags_8 },
    { 0x4fe6f4fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8446f503, SUC_lexicon_tags_61 },
    { 0x9fa6f503, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4686f508, SUC_lexicon_tags_63 },
    { 0x1a46f509, SUC_lexicon_tags_17 },
    { 0xe426f509, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e6f50d, SUC_lexicon_tags_23 },
    { 0x3466f50d, SUC_lexicon_tags_63 },
    { 0x2ca6f50f, SUC_lexicon_tags_37 },
    { 0xcb46f50e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb06f513, SUC_lexicon_tags_17 },
    { 0x406f514, SUC_lexicon_tags_2 },
    { 0xd8c6f515, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe26f519, SUC_lexicon_tags_40 },
    { 0x8886f51a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa106f51d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8366f520, SUC_lexicon_tags_12 },
    { 0xdbe6f521, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da6f524, SUC_lexicon_tags_7 },
    { 0xa5c6f524, SUC_lexicon_tags_37 },
    { 0xeca6f525, SUC_lexicon_tags_2 },
    { 0xfa26f525, SUC_lexicon_tags_15 },
    { 0x9926f528, SUC_lexicon_tags_2 },
    { 0x2ec6f529, SUC_lexicon_tags_17 },
    { 0x8ee6f529, SUC_lexicon_tags_12 },
    { 0xa306f52b, SUC_lexicon_tags_25 },
    { 0xb8a6f52c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9506f531, SUC_lexicon_tags_23 },
    { 0x1b66f532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e26f534, SUC_lexicon_tags_25 },
    { 0xe666f535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1686f53b, SUC_lexicon_tags_2 },
    { 0xf8a6f53b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9686f53e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc6f541, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4686f548, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a6f54d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c6f553, SUC_lexicon_tags_17 },
    { 0x4f66f553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c6f55b, SUC_lexicon_tags_28 },
    { 0xed66f55c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4046f560, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe566f562, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6386f564, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa6f568, SUC_lexicon_tags_15 },
    { 0x7e6f569, SUC_lexicon_tags_17 },
    { 0x50c6f569, SUC_lexicon_tags_60 },
    { 0xad06f569, SUC_lexicon_tags_10 },
    { 0xd0e6f56c, SUC_lexicon_tags_11 },
    { 0xb4e6f56c, SUC_lexicon_tags_17 },
    { 0x5006f569, SUC_lexicon_tags_47 },
    { 0x626f56e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1466f572, SUC_lexicon_tags_126 },
    { 0x8406f573, SUC_lexicon_tags_17 },
    { 0x3386f572, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e6f578, SUC_lexicon_tags_13 },
    { 0x1e06f578, SUC_lexicon_tags_33 },
    { 0x1126f57a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf926f57f, SUC_lexicon_tags_32 },
    { 0xb806f580, SUC_lexicon_tags_12 },
    { 0x1846f581, SUC_lexicon_tags_43 },
    { 0x2d06f582, SUC_lexicon_tags_12 },
    { 0xed86f583, SUC_lexicon_tags_11 },
    { 0xa206f584, SUC_lexicon_tags_70 },
    { 0x98a6f582, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcfe6f587, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5766f590, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf866f595, SUC_lexicon_tags_15 },
    { 0xe0c6f596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc6f59d, SUC_lexicon_tags_43 },
    { 0x2ca6f59d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66f5a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d26f5a6, SUC_lexicon_tags_29 },
    { 0x9b86f5a7, SUC_lexicon_tags_17 },
    { 0x9406f5a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2586f5ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ce6f5ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa166f5b0, SUC_lexicon_tags_46 },
    { 0x7f26f5b1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5146f5b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xad06f5b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9286f5c0, SUC_lexicon_tags_37 },
    { 0xda06f5c0, SUC_lexicon_tags_45 },
    { 0x6ec6f5c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e86f5c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe306f5c9, SUC_lexicon_tags_7 },
    { 0xa646f5ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7346f5cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e6f5d0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xae86f5d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe286f5d6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc6f5db, SUC_lexicon_tags_27 },
    { 0x446f5dc, SUC_lexicon_tags_28 },
    { 0x95a6f5dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x95c6f5df, SUC_lexicon_tags_7 },
    { 0xaa66f5e0, SUC_lexicon_tags_21 },
    { 0xf1a6f5e1, SUC_lexicon_tags_37 },
    { 0x6bc6f5df, SUC_lexicon_tags_17 },
    { 0xa426f5e3, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x646f5e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f26f5e9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba6f5f0, SUC_lexicon_tags_23 },
    { 0x9fa6f5f1, SUC_lexicon_tags_8 },
    { 0x4086f5f2, SUC_lexicon_tags_29 },
    { 0x7e86f5f3, SUC_lexicon_tags_11 },
    { 0xa706f5f2, SUC_lexicon_tags_237 },
    { 0xe746f5f5, SUC_lexicon_tags_25 },
    { 0x2be6f5f6, SUC_lexicon_tags_60 },
    { 0x41c6f5f1, SUC_lexicon_tags_26 },
    { 0x6c6f5f8, SUC_lexicon_tags_37 },
    { 0x42c6f5f9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d66f5fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42a6f5fe, SUC_lexicon_tags_11 },
    { 0xb286f5fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6086f602, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8026f605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba66f607, SUC_lexicon_tags_11 },
    { 0xec66f608, SUC_lexicon_tags_37 },
    { 0xaca6f608, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad86f60e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e6f619, SUC_lexicon_tags_63 },
    { 0x5da6f619, SUC_lexicon_tags_15 },
    { 0xe566f619, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c86f61f, SUC_lexicon_tags_7 },
    { 0x1f86f61f, SUC_lexicon_tags_60 },
    { 0x4c6f621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a6f625, SUC_lexicon_tags_2 },
    { 0x346f625, SUC_lexicon_tags_12 },
    { 0x8846f627, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb606f62e, SUC_lexicon_tags_7 },
    { 0x3ce6f62f, SUC_lexicon_tags_77 },
    { 0x98a6f62f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd606f632, SUC_lexicon_tags_23 },
    { 0x8686f633, SUC_lexicon_tags_23 },
    { 0xc986f632, SUC_lexicon_tags_15 },
    { 0x39c6f635, SUC_lexicon_tags_1 },
    { 0x72c6f636, SUC_lexicon_tags_66 },
    { 0xf126f636, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b06f63a, SUC_lexicon_tags_32 },
    { 0xa5e6f63b, SUC_lexicon_tags_11 },
    { 0x5306f63b, SUC_lexicon_tags_7 },
    { 0xb426f63c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c6f641, SUC_lexicon_tags_7 },
    { 0xd366f642, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6f645, SUC_lexicon_tags_15 },
    { 0x6906f645, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe266f64a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b66f651, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c6f654, SUC_lexicon_tags_2 },
    { 0x5ae6f654, SUC_lexicon_tags_15 },
    { 0x3706f656, SUC_lexicon_tags_68 },
    { 0x5346f656, SUC_lexicon_tags_26 },
    { 0xcee6f657, SUC_lexicon_tags_47 },
    { 0x5be6f655, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xa386f65b, SUC_lexicon_tags_15 },
    { 0xdee6f65c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5546f660, SUC_lexicon_tags_7 },
    { 0xa1c6f660, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6f665, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xca86f668, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7a6f66a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c6f66d, SUC_lexicon_tags_9 },
    { 0xe486f66d, SUC_lexicon_tags_55 },
    { 0x5026f66f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda26f672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9826f674, SUC_lexicon_tags_11 },
    { 0x8f66f675, SUC_lexicon_tags_14 },
    { 0x16c6f676, SUC_lexicon_tags_1 },
    { 0x4746f677, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e6f681, SUC_lexicon_tags_29 },
    { 0x96c6f682, SUC_lexicon_tags_25 },
    { 0xf966f683, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe346f685, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a6f688, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8226f692, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd326f696, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e46f69a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e6f69e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaae6f6a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e6f6a4, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de6f6a7, SUC_lexicon_tags_38 },
    { 0x7586f6a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e6f6ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7246f6ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xff46f6af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1126f6b1, SUC_lexicon_tags_12 },
    { 0xc706f6b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a6f6b6, SUC_lexicon_tags_0 },
    { 0xbc06f6b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e66f6b9, SUC_lexicon_tags_7 },
    { 0xb846f6ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a6f6bd, SUC_lexicon_tags_29 },
    { 0x2206f6bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x986f6c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x12e6f6c3, SUC_lexicon_tags_17 },
    { 0x13c6f6c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a6f6d1, SUC_lexicon_tags_1 },
    { 0x93e6f6d1, SUC_lexicon_tags_28 },
    { 0x7406f6d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xada6f6d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x686f6d8, SUC_lexicon_tags_28 },
    { 0xa6a6f6d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c46f6de, SUC_lexicon_tags_11 },
    { 0x47e6f6de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4246f6e2, SUC_lexicon_tags_23 },
    { 0xa526f6e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47e6f6e5, SUC_lexicon_tags_11 },
    { 0x8ec6f6e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8206f6e8, SUC_lexicon_tags_8 },
    { 0xb7a6f6e9, SUC_lexicon_tags_11 },
    { 0x75e6f6ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a86f6ed, SUC_lexicon_tags_2 },
    { 0x92e6f6ee, SUC_lexicon_tags_11 },
    { 0xf3c6f6ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87c6f6f1, SUC_lexicon_tags_11 },
    { 0x7466f6f1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd66f6f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd06f6f9, SUC_lexicon_tags_25 },
    { 0x88a6f6fa, SUC_lexicon_tags_1 },
    { 0x9266f6f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc6f700, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1486f704, SUC_lexicon_tags_40 },
    { 0xfdc6f705, SUC_lexicon_tags_2 },
    { 0x766f705, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d46f708, SUC_lexicon_tags_2 },
    { 0x1b46f709, SUC_lexicon_tags_12 },
    { 0xcda6f708, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26f710, SUC_lexicon_tags_4 },
    { 0xdac6f711, SUC_lexicon_tags_11 },
    { 0xad26f712, SUC_lexicon_tags_37 },
    { 0x5a06f713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c6f716, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfde6f718, SUC_lexicon_tags_38 },
    { 0x2206f719, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2066f71b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a26f71d, SUC_lexicon_tags_7 },
    { 0xec66f71d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe146f720, SUC_lexicon_tags_12 },
    { 0x6606f720, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce6f724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7cc6f726, SUC_lexicon_tags_23 },
    { 0x1446f727, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e86f731, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3566f734, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc306f736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0a6f738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4246f73a, SUC_lexicon_tags_69 },
    { 0x3be6f73b, SUC_lexicon_tags_46 },
    { 0xf8c6f73c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7f66f73e, SUC_lexicon_tags_38 },
    { 0xac6f73f, SUC_lexicon_tags_17 },
    { 0x47c6f73f, SUC_lexicon_tags_2 },
    { 0x3126f741, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80e6f743, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8e6f745, SUC_lexicon_tags_63 },
    { 0xc986f745, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde6f74a, SUC_lexicon_tags_38 },
    { 0xc586f74a, SUC_lexicon_tags_25 },
    { 0xabc6f74b, SUC_lexicon_tags_2 },
    { 0x8ec6f74c, SUC_lexicon_tags_17 },
    { 0xc06f74a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e6f752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8506f754, SUC_lexicon_tags_25 },
    { 0x3406f754, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33c6f757, SUC_lexicon_tags_26 },
    { 0xfca6f757, SUC_lexicon_tags_23 },
    { 0x49c6f759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a6f760, SUC_lexicon_tags_11 },
    { 0x8ec6f761, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe586f766, SUC_lexicon_tags_68 },
    { 0x94c6f767, SUC_lexicon_tags_17 },
    { 0xa7a6f768, SUC_lexicon_tags_43 },
    { 0x3326f766, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebe6f76b, SUC_lexicon_tags_15 },
    { 0xe6e6f76b, SUC_lexicon_tags_25 },
    { 0xa946f76d, SUC_lexicon_tags_0 },
    { 0xfd06f76c, SUC_lexicon_tags_1384 },
    { 0xe1a6f76c, SUC_lexicon_tags_15 },
    { 0x20e6f76b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a86f77a, SUC_lexicon_tags_15 },
    { 0x34c6f77b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c6f782, SUC_lexicon_tags_2 },
    { 0x11a6f783, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb0e6f785, SUC_lexicon_tags_17 },
    { 0x1e46f786, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86f78a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b46f78d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7106f78f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1266f792, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa226f79a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc06f79c, SUC_lexicon_tags_12 },
    { 0x72c6f79d, SUC_lexicon_tags_26 },
    { 0x7aa6f79c, SUC_lexicon_tags_25 },
    { 0xf226f79f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca6f7a4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c46f7a8, SUC_lexicon_tags_43 },
    { 0x1966f7a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4e6f7ab, SUC_lexicon_tags_91 },
    { 0xe9c6f7ac, SUC_lexicon_tags_17 },
    { 0x4e6f7ad, SUC_lexicon_tags_37 },
    { 0x5406f7ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa026f7b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9a06f7b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd446f7b4, SUC_lexicon_tags_55 },
    { 0xcc26f7b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae6f7b8, SUC_lexicon_tags_15 },
    { 0x66c6f7b9, SUC_lexicon_tags_30 },
    { 0x6346f7ba, SUC_lexicon_tags_11 },
    { 0x926f7bb, SUC_lexicon_tags_151 },
    { 0xff26f7bc, SUC_lexicon_tags_11 },
    { 0x2ca6f7bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a6f7c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4506f7c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e66f7c9, SUC_lexicon_tags_1 },
    { 0x3c26f7ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf46f7ce, SUC_lexicon_tags_6 },
    { 0x31c6f7cf, SUC_lexicon_tags_21 },
    { 0x1f66f7ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a6f7d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7906f7d7, SUC_lexicon_tags_21 },
    { 0x7d66f7d8, SUC_lexicon_tags_63 },
    { 0x5446f7d9, SUC_lexicon_tags_7 },
    { 0x3ba6f7da, SUC_lexicon_tags_12 },
    { 0x61a6f7d9, SUC_lexicon_tags_25 },
    { 0x4406f7d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb426f7de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf7e6f7e0, SUC_lexicon_tags_63 },
    { 0xaec6f7e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66f7e4, SUC_lexicon_tags_28 },
    { 0xd486f7e5, SUC_lexicon_tags_25 },
    { 0xd46f7e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x43e6f7e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e6f7f0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e6f7f5, SUC_lexicon_tags_1 },
    { 0xcb66f7f6, SUC_lexicon_tags_48 },
    { 0x7786f7f6, SUC_lexicon_tags_2 },
    { 0xc4e6f7f8, SUC_lexicon_tags_37 },
    { 0xbfc6f7f6, SUC_lexicon_tags_23 },
    { 0x62a6f7fa, SUC_lexicon_tags_16 },
    { 0x8f86f7fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc486f7ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3446f802, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c06f807, SUC_lexicon_tags_60 },
    { 0x70e6f808, SUC_lexicon_tags_37 },
    { 0x78c6f809, SUC_lexicon_tags_37 },
    { 0xeee6f809, SUC_lexicon_tags_103 },
    { 0x2166f80b, SUC_lexicon_tags_37 },
    { 0x6366f808, SUC_lexicon_tags_45 },
    { 0xe0a6f80d, SUC_lexicon_tags_37 },
    { 0x7c26f80c, SUC_lexicon_tags_11 },
    { 0x5a6f80d, SUC_lexicon_tags_424 },
    { 0xb7e6f80d, SUC_lexicon_tags_0 },
    { 0x3826f808, SUC_lexicon_tags_8 },
    { 0x6e86f812, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e6f815, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ae6f817, SUC_lexicon_tags_38 },
    { 0xf4e6f817, SUC_lexicon_tags_2 },
    { 0xb9a6f819, SUC_lexicon_tags_27 },
    { 0x4bc6f81a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb06f81e, SUC_lexicon_tags_48 },
    { 0x2366f81f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b46f825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc6f828, SUC_lexicon_tags_9 },
    { 0xae86f829, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9066f82d, SUC_lexicon_tags_36 },
    { 0x3e66f82d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc966f833, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3546f83c, SUC_lexicon_tags_10 },
    { 0x27e6f83d, SUC_lexicon_tags_12 },
    { 0x6b66f83d, SUC_lexicon_tags_0 },
    { 0xcd26f83f, SUC_lexicon_tags_4 },
    { 0x3a26f840, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ce6f842, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5246f845, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xede6f847, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc846f84a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8226f84e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1826f850, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c6f853, SUC_lexicon_tags_12 },
    { 0xa646f853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6f858, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40a6f85a, SUC_lexicon_tags_2 },
    { 0x1ea6f85b, SUC_lexicon_tags_65 },
    { 0x7306f85b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6c06f85e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf726f860, SUC_lexicon_tags_23 },
    { 0xc986f861, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc086f863, SUC_lexicon_tags_36 },
    { 0x5e26f864, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd266f86d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9246f870, SUC_lexicon_tags_7 },
    { 0xce06f870, SUC_lexicon_tags_36 },
    { 0xd2a6f870, SUC_lexicon_tags_15 },
    { 0x72a6f873, SUC_lexicon_tags_17 },
    { 0xe306f871, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa406f878, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7586f87a, SUC_lexicon_tags_5 },
    { 0xe46f87b, SUC_lexicon_tags_7 },
    { 0xd4a6f87b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae6f883, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c6f888, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d86f88d, SUC_lexicon_tags_15 },
    { 0x3786f88e, SUC_lexicon_tags_7 },
    { 0xf866f88d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e06f891, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7726f895, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e6f89a, SUC_lexicon_tags_23 },
    { 0x3d06f89b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c86f89e, SUC_lexicon_tags_1 },
    { 0x5086f89f, SUC_lexicon_tags_17 },
    { 0xbe6f8a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94e6f8a2, SUC_lexicon_tags_11 },
    { 0x7c26f8a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3286f8a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e26f8a9, SUC_lexicon_tags_172 },
    { 0x69c6f8aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e6f8b1, SUC_lexicon_tags_8 },
    { 0x3306f8b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b66f8b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46f8bb, SUC_lexicon_tags_33 },
    { 0x9e06f8bc, SUC_lexicon_tags_9 },
    { 0xe846f8bd, SUC_lexicon_tags_1 },
    { 0x82a6f8be, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc106f8c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca6f8ca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7886f8cd, SUC_lexicon_tags_35 },
    { 0x2d46f8cd, SUC_lexicon_tags_14 },
    { 0x16e6f8cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e6f8d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x63e6f8d4, SUC_lexicon_tags_28 },
    { 0x1da6f8d4, SUC_lexicon_tags_2 },
    { 0x1da6f8d4, SUC_lexicon_tags_11 },
    { 0x1d86f8d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x30a6f8d9, SUC_lexicon_tags_47 },
    { 0x54e6f8d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6da6f8dc, SUC_lexicon_tags_25 },
    { 0xb846f8dd, SUC_lexicon_tags_6 },
    { 0x38e6f8de, SUC_lexicon_tags_37 },
    { 0x4ec6f8df, SUC_lexicon_tags_8 },
    { 0xf46f8dc, SUC_lexicon_tags_423 },
    { 0xc506f8e1, SUC_lexicon_tags_43 },
    { 0x2fa6f8e2, SUC_lexicon_tags_1 },
    { 0xff26f8e3, SUC_lexicon_tags_21 },
    { 0x3466f8e4, SUC_lexicon_tags_12 },
    { 0x5d26f8e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8866f8e7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc9a6f8e9, SUC_lexicon_tags_56 },
    { 0x3c66f8e9, SUC_lexicon_tags_17 },
    { 0x1b06f8e9, SUC_lexicon_tags_16 },
    { 0x1306f8ec, SUC_lexicon_tags_63 },
    { 0xa66f8eb, SUC_lexicon_tags_65 },
    { 0x84c6f8ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc586f8f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a6f8f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ac6f8f7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9726f8f9, SUC_lexicon_tags_40 },
    { 0xdec6f8fa, SUC_lexicon_tags_22 },
    { 0x1886f8fb, SUC_lexicon_tags_0 },
    { 0xc026f8fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3a6f8fe, SUC_lexicon_tags_37 },
    { 0x5dc6f8ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da6f902, SUC_lexicon_tags_33 },
    { 0xd826f903, SUC_lexicon_tags_1 },
    { 0x2606f904, SUC_lexicon_tags_15 },
    { 0x2866f905, SUC_lexicon_tags_37 },
    { 0xe726f905, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb106f90d, SUC_lexicon_tags_1 },
    { 0x6c86f90e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ea6f910, SUC_lexicon_tags_57 },
    { 0x346f910, SUC_lexicon_tags_37 },
    { 0xa926f912, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc86f918, SUC_lexicon_tags_12 },
    { 0x3406f919, SUC_lexicon_tags_34 },
    { 0xbac6f918, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa566f91c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d66f922, SUC_lexicon_tags_1 },
    { 0x9106f922, SUC_lexicon_tags_37 },
    { 0xd0a6f924, SUC_lexicon_tags_29 },
    { 0xb206f924, SUC_lexicon_tags_29 },
    { 0xbf26f926, SUC_lexicon_tags_29 },
    { 0x3626f927, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x426f929, SUC_lexicon_tags_49 },
    { 0xa7c6f92a, SUC_lexicon_tags_1 },
    { 0x25c6f929, SUC_lexicon_tags_1 },
    { 0x40e6f92b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x73a6f92e, SUC_lexicon_tags_11 },
    { 0xe46f92e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfdc6f931, SUC_lexicon_tags_19 },
    { 0xcbe6f931, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e26f937, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd806f939, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e6f941, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8766f945, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5146f94a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6dc6f94c, SUC_lexicon_tags_17 },
    { 0x8606f94c, SUC_lexicon_tags_0 },
    { 0x2266f94e, SUC_lexicon_tags_42 },
    { 0x6786f94d, SUC_lexicon_tags_21 },
    { 0x5766f94c, SUC_lexicon_tags_10 },
    { 0xbc46f950, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33a6f953, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2546f959, SUC_lexicon_tags_23 },
    { 0x406f959, SUC_lexicon_tags_2 },
    { 0x6b66f95b, SUC_lexicon_tags_35 },
    { 0xa546f95c, SUC_lexicon_tags_220 },
    { 0xb866f95a, SUC_lexicon_tags_11 },
    { 0x8886f95e, SUC_lexicon_tags_60 },
    { 0xf746f95f, SUC_lexicon_tags_2 },
    { 0xef46f95f, SUC_lexicon_tags_38 },
    { 0x91e6f960, SUC_lexicon_tags_2 },
    { 0x626f960, SUC_lexicon_tags_46 },
    { 0x6206f963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78e6f965, SUC_lexicon_tags_37 },
    { 0x9b46f966, SUC_lexicon_tags_99 },
    { 0xa9c6f965, SUC_lexicon_tags_2 },
    { 0x21e6f965, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb426f96a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e6f96d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd66f96f, SUC_lexicon_tags_43 },
    { 0xc606f96f, SUC_lexicon_tags_47 },
    { 0xa026f971, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8026f977, SUC_lexicon_tags_37 },
    { 0x6286f978, SUC_lexicon_tags_1 },
    { 0xdb86f979, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6206f97b, SUC_lexicon_tags_26 },
    { 0xc1a6f97b, SUC_lexicon_tags_14 },
    { 0x8646f97d, SUC_lexicon_tags_60 },
    { 0x1166f97c, SUC_lexicon_tags_83 },
    { 0x71c6f97b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a46f98a, SUC_lexicon_tags_104 },
    { 0xa386f98b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e6f98e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf06f994, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba6f997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06f99a, SUC_lexicon_tags_27 },
    { 0x1a46f99b, SUC_lexicon_tags_1 },
    { 0x9f66f99c, SUC_lexicon_tags_63 },
    { 0x1306f99d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba66f9a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee26f9a4, SUC_lexicon_tags_11 },
    { 0x4ee6f9a4, SUC_lexicon_tags_151 },
    { 0xb1e6f9a6, SUC_lexicon_tags_26 },
    { 0x26a6f9a7, SUC_lexicon_tags_66 },
    { 0xee86f9a8, SUC_lexicon_tags_7 },
    { 0x16c6f9a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf746f9ac, SUC_lexicon_tags_20 },
    { 0xaa26f9ad, SUC_lexicon_tags_7 },
    { 0x5f06f9ae, SUC_lexicon_tags_35 },
    { 0x5586f9ac, SUC_lexicon_tags_17 },
    { 0x71e6f9af, SUC_lexicon_tags_33 },
    { 0xdb06f9b0, SUC_lexicon_tags_7 },
    { 0x4b46f9b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2906f9b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7826f9b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe46f9bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc066f9c1, SUC_lexicon_tags_37 },
    { 0x7166f9c2, SUC_lexicon_tags_126 },
    { 0xd866f9c2, SUC_lexicon_tags_15 },
    { 0xc786f9c3, SUC_lexicon_tags_30 },
    { 0x4146f9c1, SUC_lexicon_tags_17 },
    { 0x9186f9c6, SUC_lexicon_tags_7 },
    { 0xee6f9c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x29e6f9c9, SUC_lexicon_tags_1 },
    { 0x2506f9c9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe666f9cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1266f9d3, SUC_lexicon_tags_34 },
    { 0x50c6f9d4, SUC_lexicon_tags_2 },
    { 0x91e6f9d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726f9da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61a6f9dc, SUC_lexicon_tags_32 },
    { 0x27e6f9dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2166f9e1, SUC_lexicon_tags_28 },
    { 0x9986f9e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe946f9e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e6f9e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3726f9ea, SUC_lexicon_tags_60 },
    { 0xd646f9ea, SUC_lexicon_tags_31 },
    { 0xed06f9ec, SUC_lexicon_tags_53 },
    { 0x3ee6f9ec, SUC_lexicon_tags_11 },
    { 0xb626f9eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7426f9f1, SUC_lexicon_tags_49 },
    { 0x5c86f9f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e86f9f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73e6f9f8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1066f9fa, SUC_lexicon_tags_30 },
    { 0x89c6f9fa, SUC_lexicon_tags_11 },
    { 0x98e6f9fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd46f9fe, SUC_lexicon_tags_25 },
    { 0x76c6f9ff, SUC_lexicon_tags_7 },
    { 0xc566f9ff, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac6fa04, SUC_lexicon_tags_1 },
    { 0x26c6fa04, SUC_lexicon_tags_4 },
    { 0x2986fa05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a6fa09, SUC_lexicon_tags_27 },
    { 0x1666fa09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5546fa10, SUC_lexicon_tags_26 },
    { 0x20e6fa11, SUC_lexicon_tags_23 },
    { 0x7646fa12, SUC_lexicon_tags_33 },
    { 0xbb26fa12, SUC_lexicon_tags_11 },
    { 0xc586fa14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae6fa17, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7746fa1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3906fa26, SUC_lexicon_tags_27 },
    { 0x2ca6fa26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea6fa2d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf286fa2f, SUC_lexicon_tags_15 },
    { 0xf1c6fa2f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6446fa33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9026fa38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c46fa3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6fa3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4346fa41, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec26fa52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1146fa56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa6fa5d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c6fa60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ba6fa62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff86fa6c, SUC_lexicon_tags_38 },
    { 0xf526fa6c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc6fa72, SUC_lexicon_tags_27 },
    { 0x8cc6fa72, SUC_lexicon_tags_49 },
    { 0xa426fa72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1886fa76, SUC_lexicon_tags_15 },
    { 0x63a6fa76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b86fa7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c6fa80, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a6fa84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c6fa8a, SUC_lexicon_tags_12 },
    { 0xf786fa8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0a6fa8d, SUC_lexicon_tags_33 },
    { 0x2e6fa8d, SUC_lexicon_tags_28 },
    { 0x5d46fa8f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1206fa92, SUC_lexicon_tags_67 },
    { 0x6746fa93, SUC_lexicon_tags_17 },
    { 0x5aa6fa94, SUC_lexicon_tags_77 },
    { 0xd446fa95, SUC_lexicon_tags_1 },
    { 0x9f26fa92, SUC_lexicon_tags_17 },
    { 0x3bc6fa95, SUC_lexicon_tags_2 },
    { 0x8226fa98, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f46fa9e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c26faaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86a6faac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e6faaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5426fab1, SUC_lexicon_tags_28 },
    { 0xb846fab2, SUC_lexicon_tags_30 },
    { 0xbbc6fab3, SUC_lexicon_tags_11 },
    { 0xc7e6fab4, SUC_lexicon_tags_19 },
    { 0x9d86fab5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10e6fab7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fabc, SUC_lexicon_tags_10 },
    { 0x2786fabc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2a66fabf, SUC_lexicon_tags_46 },
    { 0x8fe6fabf, SUC_lexicon_tags_33 },
    { 0x8f46fac1, SUC_lexicon_tags_7 },
    { 0x3de6fac2, SUC_lexicon_tags_49 },
    { 0x7b86fac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1746fac6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe6facc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc86face, SUC_lexicon_tags_25 },
    { 0xe606face, SUC_lexicon_tags_32 },
    { 0xdf26fad0, SUC_lexicon_tags_31 },
    { 0x6f66fad1, SUC_lexicon_tags_7 },
    { 0x8c46face, SUC_lexicon_tags_63 },
    { 0xb326fad0, SUC_lexicon_tags_1 },
    { 0x1d86fad4, SUC_lexicon_tags_2 },
    { 0xbcc6fad5, SUC_lexicon_tags_15 },
    { 0x2c6fad5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c6fad9, SUC_lexicon_tags_17 },
    { 0xb406fada, SUC_lexicon_tags_29 },
    { 0x96a6fad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9586fadd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c46fadf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4866fae6, SUC_lexicon_tags_12 },
    { 0xe846fae7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a06fae9, SUC_lexicon_tags_26 },
    { 0x31c6fae9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb206faed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42a6faef, SUC_lexicon_tags_1 },
    { 0xa406faf0, SUC_lexicon_tags_25 },
    { 0xc886faf0, SUC_lexicon_tags_8 },
    { 0x3ee6faf0, SUC_lexicon_tags_11 },
    { 0x9226faf3, SUC_lexicon_tags_28 },
    { 0xb186faf0, SUC_lexicon_tags_23 },
    { 0xf606faf5, SUC_lexicon_tags_7 },
    { 0xc4a6faf2, SUC_lexicon_tags_2 },
    { 0xae06faf7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd866faf9, SUC_lexicon_tags_14 },
    { 0xc466faf9, SUC_lexicon_tags_17 },
    { 0x7c86fafa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a86fafd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4526fb00, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5166fb05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca66fb0a, SUC_lexicon_tags_60 },
    { 0x8ce6fb0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a6fb0f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2466fb12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c6fb15, SUC_lexicon_tags_25 },
    { 0x2b06fb16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2546fb18, SUC_lexicon_tags_47 },
    { 0x7406fb19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3706fb1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9866fb21, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xc306fb24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee6fb27, SUC_lexicon_tags_2 },
    { 0xefe6fb28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb946fb2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c6fb2d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c6fb37, SUC_lexicon_tags_207 },
    { 0x9d66fb38, SUC_lexicon_tags_44 },
    { 0xf506fb39, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xdd26fb3b, SUC_lexicon_tags_8 },
    { 0x6786fb3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c6fb43, SUC_lexicon_tags_46 },
    { 0xe466fb44, SUC_lexicon_tags_48 },
    { 0x5606fb45, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9426fb47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb86fb4b, SUC_lexicon_tags_26 },
    { 0xe766fb4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5446fb51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f86fb54, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c6fb5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a66fb68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6a6fb6a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf86fb6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf186fb72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc6fb75, SUC_lexicon_tags_21 },
    { 0xda86fb75, SUC_lexicon_tags_27 },
    { 0x7066fb77, SUC_lexicon_tags_11 },
    { 0xea06fb78, SUC_lexicon_tags_42 },
    { 0x7726fb79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7246fb7c, SUC_lexicon_tags_1 },
    { 0x7986fb7c, SUC_lexicon_tags_27 },
    { 0x34e6fb7e, SUC_lexicon_tags_0 },
    { 0x7fc6fb7f, SUC_lexicon_tags_29 },
    { 0xdca6fb7e, SUC_lexicon_tags_43 },
    { 0x6be6fb81, SUC_lexicon_tags_24 },
    { 0x16a6fb82, SUC_lexicon_tags_27 },
    { 0x3d26fb83, SUC_lexicon_tags_31 },
    { 0xfd66fb81, SUC_lexicon_tags_43 },
    { 0xae66fb82, SUC_lexicon_tags_12 },
    { 0xe626fb83, SUC_lexicon_tags_15 },
    { 0x22e6fb86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd686fb8a, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0x826fb8c, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x5fe6fb8e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b66fb91, SUC_lexicon_tags_27 },
    { 0x93c6fb92, SUC_lexicon_tags_21 },
    { 0xd446fb93, SUC_lexicon_tags_2 },
    { 0xb346fb92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a46fb9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a66fba2, SUC_lexicon_tags_0 },
    { 0x92a6fba2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d66fba7, SUC_lexicon_tags_25 },
    { 0x7a66fba7, SUC_lexicon_tags_34 },
    { 0x7046fba7, SUC_lexicon_tags_63 },
    { 0xc7a6fbaa, SUC_lexicon_tags_268 },
    { 0x2446fbab, SUC_lexicon_tags_11 },
    { 0x4c26fbaa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5186fbae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4346fbb0, SUC_lexicon_tags_23 },
    { 0x2aa6fbb1, SUC_lexicon_tags_17 },
    { 0xd0e6fbb0, SUC_lexicon_tags_17 },
    { 0xca06fbb0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c6fbb7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe586fbb9, SUC_lexicon_tags_45 },
    { 0xb346fbba, SUC_lexicon_tags_47 },
    { 0xf9a6fbba, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a06fbbe, SUC_lexicon_tags_45 },
    { 0x6c86fbbf, SUC_lexicon_tags_37 },
    { 0x2e66fbbe, SUC_lexicon_tags_12 },
    { 0xe7c6fbbf, SUC_lexicon_tags_37 },
    { 0x446fbc1, SUC_lexicon_tags_14 },
    { 0x606fbc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de6fbc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe086fbc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea86fbcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a46fbd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdba6fbd7, SUC_lexicon_tags_47 },
    { 0xd5e6fbd8, SUC_lexicon_tags_65 },
    { 0x9b26fbd7, SUC_lexicon_tags_7 },
    { 0xdc86fbda, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd566fbdc, SUC_lexicon_tags_7 },
    { 0x9b46fbdd, SUC_lexicon_tags_174 },
    { 0xa886fbdd, SUC_lexicon_tags_2 },
    { 0x51a6fbdf, SUC_lexicon_tags_60 },
    { 0x2a26fbe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac06fbe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb26fbee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x95e6fbf0, SUC_lexicon_tags_25 },
    { 0xe5c6fbf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8726fbf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e46fbfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8666fbfd, SUC_lexicon_tags_37 },
    { 0x1306fbfe, SUC_lexicon_tags_2 },
    { 0x8706fbfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd226fc06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46fc09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a6fc0d, SUC_lexicon_tags_27 },
    { 0xbda6fc0e, SUC_lexicon_tags_80 },
    { 0x506fc0e, SUC_lexicon_tags_49 },
    { 0xf406fc10, SUC_lexicon_tags_11 },
    { 0x55c6fc0f, SUC_lexicon_tags_17 },
    { 0x3086fc12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb126fc17, SUC_lexicon_tags_45 },
    { 0xff86fc18, SUC_lexicon_tags_47 },
    { 0x9086fc19, SUC_lexicon_tags_88 },
    { 0xda6fc1a, SUC_lexicon_tags_19 },
    { 0xc566fc19, SUC_lexicon_tags_17 },
    { 0x20e6fc1c, SUC_lexicon_tags_14 },
    { 0xd026fc1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a6fc20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a6fc23, SUC_lexicon_tags_15 },
    { 0x9f66fc23, SUC_lexicon_tags_1 },
    { 0x1326fc24, SUC_lexicon_tags_7 },
    { 0x4706fc26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a6fc2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca86fc2f, SUC_lexicon_tags_37 },
    { 0xd3e6fc30, SUC_lexicon_tags_2 },
    { 0xf486fc31, SUC_lexicon_tags_38 },
    { 0x9486fc32, SUC_lexicon_tags_38 },
    { 0xc506fc33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b06fc35, SUC_lexicon_tags_0 },
    { 0x46c6fc36, SUC_lexicon_tags_49 },
    { 0xf246fc36, SUC_lexicon_tags_34 },
    { 0xaa86fc35, SUC_lexicon_tags_45 },
    { 0xc986fc37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd86fc3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e6fc3f, SUC_lexicon_tags_91 },
    { 0x6e86fc40, SUC_lexicon_tags_87 },
    { 0xaae6fc40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf026fc44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8726fc4d, SUC_lexicon_tags_15 },
    { 0xa646fc4e, SUC_lexicon_tags_47 },
    { 0xcc86fc4f, SUC_lexicon_tags_2 },
    { 0xd86fc4f, SUC_lexicon_tags_18 },
    { 0xfee6fc4f, SUC_lexicon_tags_95 },
    { 0x7506fc4e, SUC_lexicon_tags_1 },
    { 0x1ce6fc51, SUC_lexicon_tags_10 },
    { 0xe7a6fc52, SUC_lexicon_tags_15 },
    { 0xe5a6fc4d, SUC_lexicon_tags_8 },
    { 0x7086fc50, SUC_lexicon_tags_27 },
    { 0x1f46fc51, SUC_lexicon_tags_17 },
    { 0x6766fc58, SUC_lexicon_tags_60 },
    { 0x24c6fc51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee86fc5c, SUC_lexicon_tags_7 },
    { 0x7aa6fc5d, SUC_lexicon_tags_1 },
    { 0x2cc6fc5e, SUC_lexicon_tags_62 },
    { 0x41c6fc5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6666fc61, SUC_lexicon_tags_37 },
    { 0x4ca6fc62, SUC_lexicon_tags_27 },
    { 0xa926fc61, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xb1a6fc65, SUC_lexicon_tags_19 },
    { 0xbb26fc65, SUC_lexicon_tags_17 },
    { 0xc866fc67, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5726fc6e, SUC_lexicon_tags_39 },
    { 0xf7e6fc6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a6fc73, SUC_lexicon_tags_320 },
    { 0xc86fc73, SUC_lexicon_tags_6 },
    { 0xf826fc75, SUC_lexicon_tags_33 },
    { 0x2906fc76, SUC_lexicon_tags_12 },
    { 0x2e86fc77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb26fc79, SUC_lexicon_tags_60 },
    { 0xfd66fc79, SUC_lexicon_tags_33 },
    { 0x8ec6fc7b, SUC_lexicon_tags_43 },
    { 0xee6fc7c, SUC_lexicon_tags_19 },
    { 0xa886fc7d, SUC_lexicon_tags_23 },
    { 0xed46fc7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c6fc87, SUC_lexicon_tags_30 },
    { 0x1b26fc88, SUC_lexicon_tags_28 },
    { 0xe4c6fc89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42c6fc8b, SUC_lexicon_tags_234 },
    { 0x9006fc8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be6fc8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6fc95, SUC_lexicon_tags_4 },
    { 0x2206fc96, SUC_lexicon_tags_117 },
    { 0xe506fc96, SUC_lexicon_tags_87 },
    { 0x1506fc97, SUC_lexicon_tags_25 },
    { 0xdaa6fc99, SUC_lexicon_tags_17 },
    { 0x1486fc98, SUC_lexicon_tags_23 },
    { 0xa326fc9b, SUC_lexicon_tags_8 },
    { 0xc346fc9a, SUC_lexicon_tags_25 },
    { 0xf186fc9d, SUC_lexicon_tags_33 },
    { 0xed46fc9a, SUC_lexicon_tags_25 },
    { 0x6fc9c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x78c6fca1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe566fca4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb46fca8, SUC_lexicon_tags_14 },
    { 0x42a6fca8, SUC_lexicon_tags_48 },
    { 0x4666fca9, SUC_lexicon_tags_14 },
    { 0x94e6fcab, SUC_lexicon_tags_11 },
    { 0x3d26fcac, SUC_lexicon_tags_33 },
    { 0x9de6fcac, SUC_lexicon_tags_23 },
    { 0x6e66fcac, SUC_lexicon_tags_47 },
    { 0x20e6fcaf, SUC_lexicon_tags_104 },
    { 0x5686fcb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bc6fcb2, SUC_lexicon_tags_15 },
    { 0x2a26fcb3, SUC_lexicon_tags_14 },
    { 0xd386fcb2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xcd66fcb6, SUC_lexicon_tags_47 },
    { 0x4c06fcb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c6fcbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4606fcbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f06fcc1, SUC_lexicon_tags_17 },
    { 0x45a6fcc2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe46fcc6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa6fccd, SUC_lexicon_tags_2 },
    { 0x5b86fcce, SUC_lexicon_tags_37 },
    { 0x6e46fcce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2586fcd2, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0x9846fcd4, SUC_lexicon_tags_29 },
    { 0xba6fcd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06fcd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e06fcdc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8646fcde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5c6fce0, SUC_lexicon_tags_7 },
    { 0x1566fce1, SUC_lexicon_tags_29 },
    { 0xe3a6fce1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb726fce5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3906fce7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf306fcf2, SUC_lexicon_tags_7 },
    { 0x13e6fcf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x66e6fcf5, SUC_lexicon_tags_1 },
    { 0xe8e6fcf6, SUC_lexicon_tags_37 },
    { 0x7946fcf7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8066fcfa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x71a6fcfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf726fcfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba06fd00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc466fd08, SUC_lexicon_tags_47 },
    { 0x8c6fd09, SUC_lexicon_tags_83 },
    { 0xf186fd0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5126fd0e, SUC_lexicon_tags_37 },
    { 0xb7c6fd0f, SUC_lexicon_tags_2 },
    { 0x60c6fd10, SUC_lexicon_tags_17 },
    { 0xe1a6fd10, SUC_lexicon_tags_28 },
    { 0xc726fd12, SUC_lexicon_tags_47 },
    { 0xaf66fd13, SUC_lexicon_tags_11 },
    { 0x7086fd14, SUC_lexicon_tags_37 },
    { 0x9a06fd15, SUC_lexicon_tags_13 },
    { 0x6366fd15, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8446fd19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b66fd1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c86fd21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7926fd25, SUC_lexicon_tags_61 },
    { 0x6e86fd25, SUC_lexicon_tags_1 },
    { 0x9aa6fd27, SUC_lexicon_tags_30 },
    { 0x6e86fd28, SUC_lexicon_tags_15 },
    { 0x8926fd29, SUC_lexicon_tags_43 },
    { 0xa406fd2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e6fd2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe6fd32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40a6fd34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc886fd39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1866fd3c, SUC_lexicon_tags_9 },
    { 0x7f86fd3d, SUC_lexicon_tags_29 },
    { 0xdf66fd3e, SUC_lexicon_tags_30 },
    { 0xd766fd3f, SUC_lexicon_tags_14 },
    { 0x1286fd40, SUC_lexicon_tags_37 },
    { 0xc6a6fd3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa666fd48, SUC_lexicon_tags_15 },
    { 0x8346fd48, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2886fd4b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb3a6fd4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8286fd53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdec6fd56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc6fd5a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c6fd5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c6fd61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef46fd64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2806fd6a, SUC_lexicon_tags_15 },
    { 0xad86fd6b, SUC_lexicon_tags_36 },
    { 0x546fd6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc366fd75, SUC_lexicon_tags_37 },
    { 0x5c26fd75, SUC_lexicon_tags_23 },
    { 0x6766fd76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb726fd7a, SUC_lexicon_tags_7 },
    { 0x2486fd7b, SUC_lexicon_tags_1 },
    { 0x23a6fd7c, SUC_lexicon_tags_11 },
    { 0x91a6fd7c, SUC_lexicon_tags_0 },
    { 0xcec6fd7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1626fd81, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1c86fd83, SUC_lexicon_tags_63 },
    { 0x6aa6fd84, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5766fd86, SUC_lexicon_tags_6 },
    { 0xa926fd87, SUC_lexicon_tags_0 },
    { 0xdf66fd87, SUC_lexicon_tags_11 },
    { 0x6c26fd87, SUC_lexicon_tags_1 },
    { 0x7426fd8a, SUC_lexicon_tags_27 },
    { 0x34e6fd8b, SUC_lexicon_tags_33 },
    { 0xc5a6fd8c, SUC_lexicon_tags_32 },
    { 0x4246fd8c, SUC_lexicon_tags_27 },
    { 0x7fc6fd8e, SUC_lexicon_tags_66 },
    { 0x166fd8c, SUC_lexicon_tags_2 },
    { 0xb1c6fd8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef26fd9d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce6fda0, SUC_lexicon_tags_4 },
    { 0xb1e6fda0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6fda5, SUC_lexicon_tags_33 },
    { 0xb2a6fda6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbc66fda8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3bc6fdaa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea6fdae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4c6fdb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3946fdb2, SUC_lexicon_tags_12 },
    { 0x9166fdb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9846fdb6, SUC_lexicon_tags_4 },
    { 0xecc6fdb6, SUC_lexicon_tags_25 },
    { 0x88a6fdb6, SUC_lexicon_tags_47 },
    { 0xb5e6fdb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf766fdbb, SUC_lexicon_tags_15 },
    { 0xfb06fdbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fa6fdbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa346fdc0, SUC_lexicon_tags_48 },
    { 0xed06fdc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c6fdc4, SUC_lexicon_tags_38 },
    { 0xa5c6fdc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1766fdc9, SUC_lexicon_tags_49 },
    { 0x28e6fdc9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66fdcd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a6fdd2, SUC_lexicon_tags_57 },
    { 0x1166fdd3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4186fdd9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06fddc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a06fddf, SUC_lexicon_tags_26 },
    { 0xd86fddf, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa6fde4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c6fde7, SUC_lexicon_tags_8 },
    { 0x1b06fde7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac86fdeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e6fdee, SUC_lexicon_tags_30 },
    { 0xac6fdee, SUC_lexicon_tags_62 },
    { 0xb106fdee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a6fdfa, SUC_lexicon_tags_64 },
    { 0x6966fdfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c6fdff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd406fe03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3826fe05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf566fe07, SUC_lexicon_tags_77 },
    { 0x1b66fe08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e6fe0d, SUC_lexicon_tags_14 },
    { 0x7446fe0d, SUC_lexicon_tags_32 },
    { 0x3c86fe0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66fe12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb66fe1d, SUC_lexicon_tags_87 },
    { 0x9d66fe1e, SUC_lexicon_tags_79 },
    { 0xb226fe1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc726fe21, SUC_lexicon_tags_17 },
    { 0x8166fe21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6006fe26, SUC_lexicon_tags_17 },
    { 0x8b26fe27, SUC_lexicon_tags_11 },
    { 0x8686fe28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e66fe2b, SUC_lexicon_tags_1 },
    { 0x3a6fe2b, SUC_lexicon_tags_46 },
    { 0x8de6fe2c, SUC_lexicon_tags_37 },
    { 0xd66fe2e, SUC_lexicon_tags_56 },
    { 0xfb06fe2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e6fe32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5fe6fe34, SUC_lexicon_tags_63 },
    { 0x2ea6fe34, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f26fe3b, SUC_lexicon_tags_7 },
    { 0x7c66fe3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd506fe40, SUC_lexicon_tags_14 },
    { 0xdf86fe41, SUC_lexicon_tags_7 },
    { 0x65a6fe40, SUC_lexicon_tags_17 },
    { 0x88e6fe43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6906fe51, SUC_lexicon_tags_27 },
    { 0xbde6fe51, SUC_lexicon_tags_37 },
    { 0xfba6fe53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a6fe58, SUC_lexicon_tags_17 },
    { 0x1726fe58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9326fe5f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc6fe63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c6fe6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x61a6fe6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1586fe6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa6fe71, SUC_lexicon_tags_37 },
    { 0x646fe71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e6fe77, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x58a6fe79, SUC_lexicon_tags_2 },
    { 0x6e86fe7a, SUC_lexicon_tags_15 },
    { 0xa406fe7b, SUC_lexicon_tags_38 },
    { 0x79e6fe7c, SUC_lexicon_tags_25 },
    { 0x4b06fe7d, SUC_lexicon_tags_11 },
    { 0x6986fe7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef26fe84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fa6fe86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9806fe88, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2406fe8b, SUC_lexicon_tags_7 },
    { 0x2ee6fe8b, SUC_lexicon_tags_23 },
    { 0x4286fe8c, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba06fe94, SUC_lexicon_tags_48 },
    { 0xb4c6fe94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdb46fe97, SUC_lexicon_tags_12 },
    { 0x3ca6fe97, SUC_lexicon_tags_35 },
    { 0x2a86fe99, SUC_lexicon_tags_11 },
    { 0x89e6fe99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e6fe9d, SUC_lexicon_tags_87 },
    { 0x2d46fe9e, SUC_lexicon_tags_47 },
    { 0xc506fe9f, SUC_lexicon_tags_69 },
    { 0xc886fe9e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x77a6fea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa6fea8, SUC_lexicon_tags_27 },
    { 0x17e6fea9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f26feac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4626feae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb86feb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd86feb3, SUC_lexicon_tags_1689 },
    { 0x7f26feb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ae6feb6, SUC_lexicon_tags_8 },
    { 0x1c66feb6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf86febb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e86febf, SUC_lexicon_tags_1 },
    { 0x11c6febf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a6fec3, SUC_lexicon_tags_17 },
    { 0xe086fec4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7246fed4, SUC_lexicon_tags_55 },
    { 0xa166fed5, SUC_lexicon_tags_25 },
    { 0x9e06fed5, SUC_lexicon_tags_25 },
    { 0x6726fed7, SUC_lexicon_tags_2 },
    { 0xe6e6fed5, SUC_lexicon_tags_7 },
    { 0xf606fed9, SUC_lexicon_tags_10 },
    { 0x306fed9, SUC_lexicon_tags_7 },
    { 0xc406fedb, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5866fee4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5186fee6, SUC_lexicon_tags_17 },
    { 0xbdc6fee7, SUC_lexicon_tags_1 },
    { 0x7866fee6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda46feeb, SUC_lexicon_tags_2 },
    { 0x5f06feeb, SUC_lexicon_tags_45 },
    { 0x5306feed, SUC_lexicon_tags_34 },
    { 0xcd26feed, SUC_lexicon_tags_28 },
    { 0x946feef, SUC_lexicon_tags_4 },
    { 0x3c6fef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7246fef2, SUC_lexicon_tags_60 },
    { 0x4286fef3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6fef6, SUC_lexicon_tags_43 },
    { 0x3a46fef7, SUC_lexicon_tags_23 },
    { 0xb286fef8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a6ff05, SUC_lexicon_tags_27 },
    { 0x2c46ff06, SUC_lexicon_tags_8 },
    { 0x5906ff05, SUC_lexicon_tags_48 },
    { 0x2426ff05, SUC_lexicon_tags_17 },
    { 0x1226ff09, SUC_lexicon_tags_18 },
    { 0x606ff0a, SUC_lexicon_tags_65 },
    { 0xaba6ff06, SUC_lexicon_tags_25 },
    { 0x75e6ff05, SUC_lexicon_tags_7 },
    { 0x64c6ff0a, SUC_lexicon_tags_485 },
    { 0x8286ff0b, SUC_lexicon_tags_47 },
    { 0x7026ff0f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a6ff12, SUC_lexicon_tags_8 },
    { 0x4646ff13, SUC_lexicon_tags_26 },
    { 0x4ec6ff13, SUC_lexicon_tags_1 },
    { 0x4106ff13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3906ff1e, SUC_lexicon_tags_1 },
    { 0xff46ff1e, SUC_lexicon_tags_31 },
    { 0x3766ff20, SUC_lexicon_tags_77 },
    { 0xe366ff21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c46ff25, SUC_lexicon_tags_60 },
    { 0x2be6ff26, SUC_lexicon_tags_39 },
    { 0x3c06ff27, SUC_lexicon_tags_4 },
    { 0x78a6ff27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16e6ff2a, SUC_lexicon_tags_37 },
    { 0x9e6ff2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecc6ff2d, SUC_lexicon_tags_19 },
    { 0x2d86ff2d, SUC_lexicon_tags_15 },
    { 0x6446ff2e, SUC_lexicon_tags_11 },
    { 0xe86ff30, SUC_lexicon_tags_1 },
    { 0xc866ff30, SUC_lexicon_tags_14 },
    { 0xec26ff31, SUC_lexicon_tags_21 },
    { 0x4026ff33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa106ff36, SUC_lexicon_tags_31 },
    { 0x3326ff37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a6ff3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82e6ff3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7966ff41, SUC_lexicon_tags_79 },
    { 0x6be6ff41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fc6ff44, SUC_lexicon_tags_38 },
    { 0xbea6ff45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba46ff47, SUC_lexicon_tags_8 },
    { 0xfac6ff48, SUC_lexicon_tags_15 },
    { 0x5346ff49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56c6ff4b, SUC_lexicon_tags_38 },
    { 0xce86ff4b, SUC_lexicon_tags_37 },
    { 0x75a6ff4d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5466ff53, SUC_lexicon_tags_77 },
    { 0xfde6ff54, SUC_lexicon_tags_62 },
    { 0xa006ff55, SUC_lexicon_tags_1 },
    { 0x6a26ff56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1646ff58, SUC_lexicon_tags_11 },
    { 0xf446ff59, SUC_lexicon_tags_17 },
    { 0x5226ff59, SUC_lexicon_tags_12 },
    { 0x2c66ff5b, SUC_lexicon_tags_11 },
    { 0xe006ff5b, SUC_lexicon_tags_1 },
    { 0xe946ff59, SUC_lexicon_tags_12 },
    { 0x9b86ff5e, SUC_lexicon_tags_87 },
    { 0xc7c6ff5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb766ff64, SUC_lexicon_tags_43 },
    { 0x4a06ff64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc686ff6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc966ff6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa6ff73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd046ff75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f46ff77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc286ff7e, SUC_lexicon_tags_13 },
    { 0x92e6ff7f, SUC_lexicon_tags_1 },
    { 0xc106ff7f, SUC_lexicon_tags_8 },
    { 0xb5c6ff7e, SUC_lexicon_tags_17 },
    { 0x9a6ff80, SUC_lexicon_tags_33 },
    { 0x1186ff83, SUC_lexicon_tags_15 },
    { 0x1246ff84, SUC_lexicon_tags_15 },
    { 0x4526ff85, SUC_lexicon_tags_12 },
    { 0x87a6ff85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e6ff8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6586ff95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc06ff98, SUC_lexicon_tags_87 },
    { 0xcd46ff99, SUC_lexicon_tags_39 },
    { 0xaa66ff9a, SUC_lexicon_tags_19 },
    { 0x5aa6ff98, SUC_lexicon_tags_6 },
    { 0x526ff9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25e6ff9e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc786ffa2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x13c6ffa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e6ffa8, SUC_lexicon_tags_33 },
    { 0xc3e6ffa9, SUC_lexicon_tags_45 },
    { 0xa846ffa8, SUC_lexicon_tags_60 },
    { 0x18c6ffa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf466ffad, SUC_lexicon_tags_15 },
    { 0xf166ffae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c6ffb4, SUC_lexicon_tags_8 },
    { 0xb3e6ffb5, SUC_lexicon_tags_2 },
    { 0x7bc6ffb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1886ffbb, SUC_lexicon_tags_15 },
    { 0xe806ffbb, SUC_lexicon_tags_15 },
    { 0xc3c6ffbd, SUC_lexicon_tags_1 },
    { 0x6f26ffbe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6686ffc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e6ffc3, SUC_lexicon_tags_11 },
    { 0x8266ffc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a6ffc6, SUC_lexicon_tags_63 },
    { 0xc2a6ffc7, SUC_lexicon_tags_30 },
    { 0x31e6ffc6, SUC_lexicon_tags_11 },
    { 0x98a6ffc8, SUC_lexicon_tags_11 },
    { 0xed86ffca, SUC_lexicon_tags_37 },
    { 0xfe66ffcb, SUC_lexicon_tags_48 },
    { 0x8d86ffcc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd166ffd0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966ffd5, SUC_lexicon_tags_37 },
    { 0x1506ffd5, SUC_lexicon_tags_7 },
    { 0x9be6ffd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x106ffd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c6ffdc, SUC_lexicon_tags_6 },
    { 0x80a6ffdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e6ffe6, SUC_lexicon_tags_1425 },
    { 0, NULL },
    { 0x8ac6ffe8, SUC_lexicon_tags_9 },
    { 0x2ae6ffe9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66ffee, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x68c6fff0, SUC_lexicon_tags_25 },
    { 0x9d66fff1, SUC_lexicon_tags_7 },
    { 0x1ce6fff2, SUC_lexicon_tags_37 },
    { 0x5b66fff2, SUC_lexicon_tags_3 },
    { 0xd26fff2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4566fff7, SUC_lexicon_tags_26 },
    { 0x1d66fff8, SUC_lexicon_tags_31 },
    { 0x69e6fff9, SUC_lexicon_tags_45 },
    { 0xa226fffa, SUC_lexicon_tags_25 },
    { 0x4466fff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8270000, SUC_lexicon_tags_38 },
    { 0x3b470001, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc70005, SUC_lexicon_tags_27 },
    { 0x9b870006, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf9a70008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f27000b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec70016, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6547001b, SUC_lexicon_tags_37 },
    { 0x1ca7001c, SUC_lexicon_tags_5 },
    { 0x3247001b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0870020, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x470022, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4470024, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x28870026, SUC_lexicon_tags_2 },
    { 0x93c70026, SUC_lexicon_tags_12 },
    { 0x87470028, SUC_lexicon_tags_4 },
    { 0x8d070029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f67002d, SUC_lexicon_tags_1 },
    { 0x9c27002e, SUC_lexicon_tags_32 },
    { 0x3a67002f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39870031, SUC_lexicon_tags_66 },
    { 0x20470032, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4270034, SUC_lexicon_tags_63 },
    { 0xc0070034, SUC_lexicon_tags_37 },
    { 0xaa870036, SUC_lexicon_tags_23 },
    { 0xef870036, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5567003a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4070042, SUC_lexicon_tags_62 },
    { 0xfc470043, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a70049, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a67004c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3987004f, SUC_lexicon_tags_37 },
    { 0x29e70050, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc70057, SUC_lexicon_tags_27 },
    { 0xa6070058, SUC_lexicon_tags_1 },
    { 0x1b870057, SUC_lexicon_tags_1 },
    { 0xe527005a, SUC_lexicon_tags_23 },
    { 0x61a70057, SUC_lexicon_tags_62 },
    { 0xe527005c, SUC_lexicon_tags_2 },
    { 0x98e7005c, SUC_lexicon_tags_21 },
    { 0xb5c7005a, SUC_lexicon_tags_17 },
    { 0x5907005f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91070064, SUC_lexicon_tags_19 },
    { 0x9c70065, SUC_lexicon_tags_0 },
    { 0xa1870065, SUC_lexicon_tags_27 },
    { 0x4a70067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5270069, SUC_lexicon_tags_12 },
    { 0x4a07006a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c70072, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad470078, SUC_lexicon_tags_31 },
    { 0xa0670078, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fe7007b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac70081, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x11e70083, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe070087, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb47008a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c7008f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d070091, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c70094, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc3e70096, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde7009a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4700a1, SUC_lexicon_tags_33 },
    { 0xe4700a2, SUC_lexicon_tags_25 },
    { 0xbbc700a3, SUC_lexicon_tags_16 },
    { 0x40c700a4, SUC_lexicon_tags_37 },
    { 0x6ec700a5, SUC_lexicon_tags_25 },
    { 0x982700a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36700b1, SUC_lexicon_tags_17 },
    { 0x5e2700b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6700b5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2700bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b4700bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e700c4, SUC_lexicon_tags_6 },
    { 0x24a700c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c700ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482700cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c700d2, SUC_lexicon_tags_37 },
    { 0x68e700d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8700db, SUC_lexicon_tags_45 },
    { 0x5f8700dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96700e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e700e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a700eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x336700ed, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x746700f0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5ae700f2, SUC_lexicon_tags_1 },
    { 0x7e6700f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0700f5, SUC_lexicon_tags_2 },
    { 0xf4700f5, SUC_lexicon_tags_7 },
    { 0x158700f7, SUC_lexicon_tags_17 },
    { 0x97a700f8, SUC_lexicon_tags_14 },
    { 0x2c4700f9, SUC_lexicon_tags_37 },
    { 0x22700fa, SUC_lexicon_tags_55 },
    { 0xc9c700fa, SUC_lexicon_tags_65 },
    { 0x4c700fa, SUC_lexicon_tags_47 },
    { 0x928700fd, SUC_lexicon_tags_7 },
    { 0x44700fe, SUC_lexicon_tags_61 },
    { 0xbaa700fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x60670101, SUC_lexicon_tags_23 },
    { 0x42670102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80e70104, SUC_lexicon_tags_63 },
    { 0x75270104, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac70108, SUC_lexicon_tags_46 },
    { 0x10870109, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa867010d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74e7010f, SUC_lexicon_tags_34 },
    { 0xb9870110, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93270112, SUC_lexicon_tags_207 },
    { 0x6f270113, SUC_lexicon_tags_37 },
    { 0x80e70114, SUC_lexicon_tags_42 },
    { 0x1cc70115, SUC_lexicon_tags_49 },
    { 0x73c70116, SUC_lexicon_tags_17 },
    { 0xd8c70112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1547011d, SUC_lexicon_tags_17 },
    { 0xa747011e, SUC_lexicon_tags_17 },
    { 0x9be7011e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d670123, SUC_lexicon_tags_37 },
    { 0x54670124, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a70127, SUC_lexicon_tags_68 },
    { 0x93c70128, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e70139, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5027013f, SUC_lexicon_tags_778 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c70142, SUC_lexicon_tags_37 },
    { 0x1f670142, SUC_lexicon_tags_37 },
    { 0x69a70144, SUC_lexicon_tags_7 },
    { 0x61a70145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47470147, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6207014a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5de7014c, SUC_lexicon_tags_1 },
    { 0x34c7014c, SUC_lexicon_tags_31 },
    { 0xd6e7014e, SUC_lexicon_tags_10 },
    { 0xac07014f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4a70151, SUC_lexicon_tags_43 },
    { 0x70070151, SUC_lexicon_tags_47 },
    { 0xf4470153, SUC_lexicon_tags_35 },
    { 0x8c470154, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a70158, SUC_lexicon_tags_23 },
    { 0x91870159, SUC_lexicon_tags_37 },
    { 0xf2870158, SUC_lexicon_tags_16 },
    { 0xcf07015a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e67015d, SUC_lexicon_tags_17 },
    { 0x34e7015d, SUC_lexicon_tags_17 },
    { 0xcd87015f, SUC_lexicon_tags_11 },
    { 0xff27015e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97c70162, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7da70164, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c70167, SUC_lexicon_tags_9 },
    { 0x1c470168, SUC_lexicon_tags_26 },
    { 0x42c70169, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f470170, SUC_lexicon_tags_2 },
    { 0x37c70171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37070173, SUC_lexicon_tags_45 },
    { 0x9fe70174, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x38670177, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47a70179, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe9a7017b, SUC_lexicon_tags_7 },
    { 0xbfa7017c, SUC_lexicon_tags_6 },
    { 0xad07017d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e70180, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9e70182, SUC_lexicon_tags_22 },
    { 0x8fa70183, SUC_lexicon_tags_15 },
    { 0x14470183, SUC_lexicon_tags_0 },
    { 0x95e70184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e70188, SUC_lexicon_tags_27 },
    { 0xde670189, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9470192, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1aa70194, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e470196, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2c70198, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4701a5, SUC_lexicon_tags_9 },
    { 0x6f2701a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0701ab, SUC_lexicon_tags_13 },
    { 0xd94701ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ca701ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c701b7, SUC_lexicon_tags_2 },
    { 0x49c701b7, SUC_lexicon_tags_12 },
    { 0x5f4701b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x782701bc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x294701be, SUC_lexicon_tags_140 },
    { 0x4cc701be, SUC_lexicon_tags_25 },
    { 0x1d0701bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c701c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc701c8, SUC_lexicon_tags_136 },
    { 0xc3a701c9, SUC_lexicon_tags_17 },
    { 0xc44701c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48701cd, SUC_lexicon_tags_11 },
    { 0x31c701cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c701d1, SUC_lexicon_tags_26 },
    { 0x58701d2, SUC_lexicon_tags_25 },
    { 0xca6701d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x816701d5, SUC_lexicon_tags_12 },
    { 0x794701d6, SUC_lexicon_tags_12 },
    { 0x51e701d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x92e701d9, SUC_lexicon_tags_3 },
    { 0xd16701da, SUC_lexicon_tags_14 },
    { 0xa66701da, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e701de, SUC_lexicon_tags_1 },
    { 0x6d2701df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x808701e2, SUC_lexicon_tags_27 },
    { 0xf5a701e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e701e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc701eb, SUC_lexicon_tags_33 },
    { 0x648701ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c701f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24e701f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99e701f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac701f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480701ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c70203, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0xf2a70205, SUC_lexicon_tags_22 },
    { 0xbf870206, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c70208, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe987020c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe767020f, SUC_lexicon_tags_29 },
    { 0xbaa70210, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca70214, SUC_lexicon_tags_9 },
    { 0xc1c70215, SUC_lexicon_tags_44 },
    { 0x9c870216, SUC_lexicon_tags_33 },
    { 0x7270217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1927021c, SUC_lexicon_tags_47 },
    { 0xc9e7021d, SUC_lexicon_tags_14 },
    { 0x6e67021e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17470220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80270222, SUC_lexicon_tags_26 },
    { 0x9ec70222, SUC_lexicon_tags_38 },
    { 0xcf270224, SUC_lexicon_tags_10 },
    { 0x82270223, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e70228, SUC_lexicon_tags_48 },
    { 0x96870229, SUC_lexicon_tags_27 },
    { 0x2c070229, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc47022d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e87022f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x41870231, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc70234, SUC_lexicon_tags_17 },
    { 0xd070235, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54870239, SUC_lexicon_tags_7 },
    { 0x47470239, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe547023c, SUC_lexicon_tags_7 },
    { 0xb647023d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e70241, SUC_lexicon_tags_29 },
    { 0x8e670242, SUC_lexicon_tags_37 },
    { 0xcd270242, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x20870245, SUC_lexicon_tags_33 },
    { 0x8c470246, SUC_lexicon_tags_11 },
    { 0x3470247, SUC_lexicon_tags_49 },
    { 0xdbc70248, SUC_lexicon_tags_27 },
    { 0x5ac70249, SUC_lexicon_tags_33 },
    { 0x51c7024a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac7024e, SUC_lexicon_tags_15 },
    { 0xda27024f, SUC_lexicon_tags_15 },
    { 0xa367024e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf3670252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96870257, SUC_lexicon_tags_19 },
    { 0x10870257, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e7025d, SUC_lexicon_tags_31 },
    { 0x5e27025d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefa70260, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a70263, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee70266, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c70269, SUC_lexicon_tags_38 },
    { 0xe287026a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xcb47026c, SUC_lexicon_tags_12 },
    { 0x5087026d, SUC_lexicon_tags_25 },
    { 0x6467026c, SUC_lexicon_tags_25 },
    { 0x5d27026f, SUC_lexicon_tags_38 },
    { 0x52e70270, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa870275, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b470278, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9847027d, SUC_lexicon_tags_31 },
    { 0xecc7027e, SUC_lexicon_tags_27 },
    { 0xaa27027e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a70281, SUC_lexicon_tags_45 },
    { 0x8fa70282, SUC_lexicon_tags_17 },
    { 0xbf470283, SUC_lexicon_tags_45 },
    { 0x78a70284, SUC_lexicon_tags_2 },
    { 0x7f270285, SUC_lexicon_tags_2 },
    { 0xe4e70285, SUC_lexicon_tags_29 },
    { 0xaba70287, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da7028a, SUC_lexicon_tags_31 },
    { 0x4687028a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb07028f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1870293, SUC_lexicon_tags_27 },
    { 0x47870293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeea70296, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e702a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c702ac, SUC_lexicon_tags_7 },
    { 0xbd2702ad, SUC_lexicon_tags_57 },
    { 0xf80702ae, SUC_lexicon_tags_12 },
    { 0x134702af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc702b2, SUC_lexicon_tags_28 },
    { 0xcf2702b2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e702b7, SUC_lexicon_tags_12 },
    { 0xab0702b8, SUC_lexicon_tags_1 },
    { 0xb3c702b9, SUC_lexicon_tags_11 },
    { 0x476702ba, SUC_lexicon_tags_16 },
    { 0xa7a702bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x784702bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2702c1, SUC_lexicon_tags_2 },
    { 0x742702c2, SUC_lexicon_tags_2 },
    { 0x2c6702c3, SUC_lexicon_tags_56 },
    { 0x950702c4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfd4702c6, SUC_lexicon_tags_34 },
    { 0xea8702c7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac702ca, SUC_lexicon_tags_12 },
    { 0x898702cb, SUC_lexicon_tags_39 },
    { 0x4ae702cc, SUC_lexicon_tags_37 },
    { 0x2e8702cd, SUC_lexicon_tags_31 },
    { 0x63a702ce, SUC_lexicon_tags_12 },
    { 0x566702ce, SUC_lexicon_tags_53 },
    { 0x70e702d0, SUC_lexicon_tags_2 },
    { 0xada702d0, SUC_lexicon_tags_33 },
    { 0xab2702d2, SUC_lexicon_tags_2 },
    { 0x1a6702d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da702d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24a702d8, SUC_lexicon_tags_17 },
    { 0x80c702d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a702dd, SUC_lexicon_tags_7 },
    { 0xe72702de, SUC_lexicon_tags_56 },
    { 0xe1e702df, SUC_lexicon_tags_17 },
    { 0x6a8702df, SUC_lexicon_tags_17 },
    { 0x35c702e1, SUC_lexicon_tags_29 },
    { 0xe62702e2, SUC_lexicon_tags_198 },
    { 0x5ea702e0, SUC_lexicon_tags_12 },
    { 0xad6702de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbba702e6, SUC_lexicon_tags_22 },
    { 0xdee702e6, SUC_lexicon_tags_47 },
    { 0x47e702e8, SUC_lexicon_tags_31 },
    { 0x398702e7, SUC_lexicon_tags_29 },
    { 0x46c702e6, SUC_lexicon_tags_1 },
    { 0xb9c702ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a8702ed, SUC_lexicon_tags_43 },
    { 0x8ec702ee, SUC_lexicon_tags_17 },
    { 0x8de702ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904702f4, SUC_lexicon_tags_43 },
    { 0x828702f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846702fc, SUC_lexicon_tags_9 },
    { 0xa02702fd, SUC_lexicon_tags_68 },
    { 0x2b6702fe, SUC_lexicon_tags_26 },
    { 0x6bc702ff, SUC_lexicon_tags_27 },
    { 0xbf270300, SUC_lexicon_tags_29 },
    { 0x90070301, SUC_lexicon_tags_25 },
    { 0x85070302, SUC_lexicon_tags_25 },
    { 0x9a270300, SUC_lexicon_tags_45 },
    { 0x70070303, SUC_lexicon_tags_19 },
    { 0x6f070301, SUC_lexicon_tags_37 },
    { 0xb1a702fd, SUC_lexicon_tags_275 },
    { 0x69a70307, SUC_lexicon_tags_19 },
    { 0xd0070308, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda7030c, SUC_lexicon_tags_47 },
    { 0x2827030d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8127030f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a70316, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba7031b, SUC_lexicon_tags_18 },
    { 0x22c7031c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc987031e, SUC_lexicon_tags_1 },
    { 0x8de7031f, SUC_lexicon_tags_15 },
    { 0x5e470320, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3070322, SUC_lexicon_tags_19 },
    { 0xa6470322, SUC_lexicon_tags_2 },
    { 0xb870323, SUC_lexicon_tags_28 },
    { 0x32270325, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x26270327, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba27032a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfd67032c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c7032f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x67a70331, SUC_lexicon_tags_16 },
    { 0x98270332, SUC_lexicon_tags_7 },
    { 0xcdc70333, SUC_lexicon_tags_1 },
    { 0xb2a70334, SUC_lexicon_tags_26 },
    { 0x3e070335, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e70338, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdfe7033a, SUC_lexicon_tags_45 },
    { 0xcba7033b, SUC_lexicon_tags_37 },
    { 0x9f67033c, SUC_lexicon_tags_7 },
    { 0xdac7033a, SUC_lexicon_tags_27 },
    { 0x4267033e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa470340, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a70349, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb667034b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c67034d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d070358, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada7035d, SUC_lexicon_tags_33 },
    { 0x4a27035e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c70361, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4da70363, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9007036b, SUC_lexicon_tags_66 },
    { 0x6c47036c, SUC_lexicon_tags_33 },
    { 0x28c7036c, SUC_lexicon_tags_5 },
    { 0x49e7036e, SUC_lexicon_tags_2 },
    { 0xa187036c, SUC_lexicon_tags_29 },
    { 0x6d470370, SUC_lexicon_tags_84 },
    { 0x8270371, SUC_lexicon_tags_17 },
    { 0xb0a70372, SUC_lexicon_tags_85 },
    { 0x26070373, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2670376, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x84c70378, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e87037d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba70384, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa4c70386, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a270389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5747038c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2e87038e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9470391, SUC_lexicon_tags_10 },
    { 0xe5070391, SUC_lexicon_tags_11 },
    { 0xd6070391, SUC_lexicon_tags_318 },
    { 0x83070392, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x68e70396, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29870398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4807039b, SUC_lexicon_tags_1 },
    { 0xdae7039c, SUC_lexicon_tags_45 },
    { 0x9287039d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1027039f, SUC_lexicon_tags_14 },
    { 0x608703a0, SUC_lexicon_tags_37 },
    { 0x916703a1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c703a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350703aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a703b2, SUC_lexicon_tags_15 },
    { 0x29c703b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce703b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804703bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd04703bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e703c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c703cd, SUC_lexicon_tags_7 },
    { 0xeb8703ce, SUC_lexicon_tags_27 },
    { 0x76e703cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b2703d1, SUC_lexicon_tags_10 },
    { 0x154703d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6703d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e703d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc703dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214703e3, SUC_lexicon_tags_37 },
    { 0xd86703e3, SUC_lexicon_tags_11 },
    { 0xf10703e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56703e8, SUC_lexicon_tags_0 },
    { 0x7a6703e9, SUC_lexicon_tags_17 },
    { 0xc26703ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x980703ed, SUC_lexicon_tags_1 },
    { 0xa9c703ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4703f3, SUC_lexicon_tags_31 },
    { 0x3c703f4, SUC_lexicon_tags_31 },
    { 0x46c703f5, SUC_lexicon_tags_1 },
    { 0xc00703f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6703f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54703fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68270401, SUC_lexicon_tags_25 },
    { 0x5a270401, SUC_lexicon_tags_2 },
    { 0x8fa70402, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7670408, SUC_lexicon_tags_11 },
    { 0x1d270409, SUC_lexicon_tags_11 },
    { 0xf3c70409, SUC_lexicon_tags_49 },
    { 0x3c7040b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c70410, SUC_lexicon_tags_30 },
    { 0x5a70411, SUC_lexicon_tags_2 },
    { 0x61470412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca70416, SUC_lexicon_tags_25 },
    { 0xdee70416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe467041b, SUC_lexicon_tags_31 },
    { 0x7fc7041c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78e7041e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a70422, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49870424, SUC_lexicon_tags_2 },
    { 0x96c70425, SUC_lexicon_tags_37 },
    { 0x7d670426, SUC_lexicon_tags_11 },
    { 0x12e70427, SUC_lexicon_tags_60 },
    { 0xee270428, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc87042a, SUC_lexicon_tags_52 },
    { 0xf7e7042b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5327042d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e70432, SUC_lexicon_tags_16 },
    { 0xe070433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64670435, SUC_lexicon_tags_2 },
    { 0x27470436, SUC_lexicon_tags_59 },
    { 0xe5c70437, SUC_lexicon_tags_0 },
    { 0x12c70438, SUC_lexicon_tags_13 },
    { 0x94070435, SUC_lexicon_tags_33 },
    { 0xfb87043a, SUC_lexicon_tags_2 },
    { 0xe9c7043b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe047043d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a70440, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13270442, SUC_lexicon_tags_25 },
    { 0xc5270443, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7ca70445, SUC_lexicon_tags_5 },
    { 0x48c70446, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8670449, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6887044c, SUC_lexicon_tags_25 },
    { 0xaee7044d, SUC_lexicon_tags_23 },
    { 0xd07044e, SUC_lexicon_tags_37 },
    { 0x8487044f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc70452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b070456, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63070458, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf747045f, SUC_lexicon_tags_29 },
    { 0xa4870460, SUC_lexicon_tags_1 },
    { 0xb5c7045f, SUC_lexicon_tags_13 },
    { 0x86270460, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b670468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e70470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a70475, SUC_lexicon_tags_160 },
    { 0xa9a70476, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5ca70478, SUC_lexicon_tags_43 },
    { 0x33270479, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6567047b, SUC_lexicon_tags_19 },
    { 0xbcc7047c, SUC_lexicon_tags_17 },
    { 0xd8c7047d, SUC_lexicon_tags_37 },
    { 0xc187047b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0470481, SUC_lexicon_tags_11 },
    { 0x18a70482, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8470485, SUC_lexicon_tags_31 },
    { 0x61a70485, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x48070488, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa927048a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6367048c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a270491, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea7049b, SUC_lexicon_tags_46 },
    { 0xf487049c, SUC_lexicon_tags_33 },
    { 0x5527049b, SUC_lexicon_tags_68 },
    { 0x4467049e, SUC_lexicon_tags_27 },
    { 0xe9e7049f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e704a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2704ab, SUC_lexicon_tags_48 },
    { 0xac2704ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66704b0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14704b3, SUC_lexicon_tags_25 },
    { 0x790704b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c704b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4704bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6704c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa704c6, SUC_lexicon_tags_24 },
    { 0x6e8704c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c704ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a704cd, SUC_lexicon_tags_31 },
    { 0xa40704ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3e704d0, SUC_lexicon_tags_311 },
    { 0x29e704d1, SUC_lexicon_tags_8 },
    { 0x952704d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6704d5, SUC_lexicon_tags_17 },
    { 0x79a704d6, SUC_lexicon_tags_57 },
    { 0x36c704d7, SUC_lexicon_tags_7 },
    { 0x12a704d6, SUC_lexicon_tags_17 },
    { 0x858704d9, SUC_lexicon_tags_28 },
    { 0x4b0704da, SUC_lexicon_tags_7 },
    { 0x5b4704d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96704df, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c704e3, SUC_lexicon_tags_63 },
    { 0xfc2704e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d2704e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa08704e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22704f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x926704f2, SUC_lexicon_tags_63 },
    { 0x596704f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e704f9, SUC_lexicon_tags_27 },
    { 0x750704fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8da704fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e70507, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd4270509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56670514, SUC_lexicon_tags_11 },
    { 0x7ea70514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb527051a, SUC_lexicon_tags_11 },
    { 0xcd67051a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e7051e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c70523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe270525, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee7052c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc70531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1007053c, SUC_lexicon_tags_19 },
    { 0xcae7053d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8be7053f, SUC_lexicon_tags_30 },
    { 0x84c70540, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefe70542, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62070544, SUC_lexicon_tags_29 },
    { 0xc7270545, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1dc70547, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe6270549, SUC_lexicon_tags_25 },
    { 0xbce7054a, SUC_lexicon_tags_47 },
    { 0x32a7054b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca7054e, SUC_lexicon_tags_14 },
    { 0x7007054f, SUC_lexicon_tags_6 },
    { 0xf9270550, SUC_lexicon_tags_7 },
    { 0x70e70551, SUC_lexicon_tags_15 },
    { 0xe9a70550, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec7055e, SUC_lexicon_tags_26 },
    { 0x6747055f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7270563, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa2270565, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a7056c, SUC_lexicon_tags_47 },
    { 0x2e67056c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a70571, SUC_lexicon_tags_1 },
    { 0xc5270572, SUC_lexicon_tags_47 },
    { 0x9ae70573, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa7057a, SUC_lexicon_tags_29 },
    { 0xe987057a, SUC_lexicon_tags_25 },
    { 0x3567057c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c470584, SUC_lexicon_tags_50 },
    { 0x80670584, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59670588, SUC_lexicon_tags_25 },
    { 0xa3c70589, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xff87058c, SUC_lexicon_tags_2 },
    { 0x45a7058d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2870595, SUC_lexicon_tags_25 },
    { 0x1c470596, SUC_lexicon_tags_29 },
    { 0xe5a70595, SUC_lexicon_tags_15 },
    { 0x47a70595, SUC_lexicon_tags_0 },
    { 0x54470599, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de7059e, SUC_lexicon_tags_1 },
    { 0x1f47059f, SUC_lexicon_tags_29 },
    { 0x83e7059f, SUC_lexicon_tags_1255 },
    { 0x85c705a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x95a705a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa4705a5, SUC_lexicon_tags_23 },
    { 0x386705a6, SUC_lexicon_tags_1 },
    { 0x4c705a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe705ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6705af, SUC_lexicon_tags_1 },
    { 0xc6a705af, SUC_lexicon_tags_45 },
    { 0xbf2705b1, SUC_lexicon_tags_1 },
    { 0x1e0705b1, SUC_lexicon_tags_39 },
    { 0xed0705af, SUC_lexicon_tags_27 },
    { 0xfd4705b2, SUC_lexicon_tags_45 },
    { 0xf8c705b5, SUC_lexicon_tags_21 },
    { 0xf08705b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6705c3, SUC_lexicon_tags_27 },
    { 0xf4e705c3, SUC_lexicon_tags_31 },
    { 0xf04705c5, SUC_lexicon_tags_2 },
    { 0x6f2705c6, SUC_lexicon_tags_12 },
    { 0x318705c4, SUC_lexicon_tags_63 },
    { 0x44c705c8, SUC_lexicon_tags_44 },
    { 0xe96705c9, SUC_lexicon_tags_45 },
    { 0x34c705c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc705d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6705dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6705df, SUC_lexicon_tags_68 },
    { 0xda0705df, SUC_lexicon_tags_14 },
    { 0x33c705df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2705e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c705ea, SUC_lexicon_tags_25 },
    { 0xea8705eb, SUC_lexicon_tags_9 },
    { 0xfa0705eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x702705ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60705f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba705f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a705fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x284705ff, SUC_lexicon_tags_17 },
    { 0x70670600, SUC_lexicon_tags_2 },
    { 0x21a70601, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd470603, SUC_lexicon_tags_31 },
    { 0x49870604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22c70606, SUC_lexicon_tags_23 },
    { 0x94a70606, SUC_lexicon_tags_11 },
    { 0x7ca70608, SUC_lexicon_tags_26 },
    { 0x3aa70606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f070615, SUC_lexicon_tags_45 },
    { 0xbec70616, SUC_lexicon_tags_82 },
    { 0x59e70615, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xece7061a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94070620, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7dc70622, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e87062a, SUC_lexicon_tags_27 },
    { 0x58e7062b, SUC_lexicon_tags_39 },
    { 0x2ce7062c, SUC_lexicon_tags_15 },
    { 0xdb87062d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96070634, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e70638, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e47063a, SUC_lexicon_tags_87 },
    { 0x4b47063a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeea7063d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e70640, SUC_lexicon_tags_23 },
    { 0x19e70640, SUC_lexicon_tags_32 },
    { 0xc9e70640, SUC_lexicon_tags_1 },
    { 0xd2e70643, SUC_lexicon_tags_6 },
    { 0x33870644, SUC_lexicon_tags_25 },
    { 0x48070645, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b870648, SUC_lexicon_tags_7 },
    { 0xeae70649, SUC_lexicon_tags_17 },
    { 0xe327064a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc7064f, SUC_lexicon_tags_6 },
    { 0xf407064f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd670653, SUC_lexicon_tags_9 },
    { 0x12270654, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc670658, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be7065c, SUC_lexicon_tags_1337 },
    { 0, NULL },
    { 0xd927065e, SUC_lexicon_tags_17 },
    { 0xf367065e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46470663, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d470666, SUC_lexicon_tags_2 },
    { 0x77870667, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a7066a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc707066d, SUC_lexicon_tags_37 },
    { 0x8aa7066e, SUC_lexicon_tags_0 },
    { 0xa867066e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2870674, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc967067a, SUC_lexicon_tags_15 },
    { 0x5fe7067b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb870681, SUC_lexicon_tags_23 },
    { 0x98270682, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a70685, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e270687, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c7068f, SUC_lexicon_tags_26 },
    { 0x60c70690, SUC_lexicon_tags_2 },
    { 0xab87068f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a70694, SUC_lexicon_tags_17 },
    { 0x12270695, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa7069c, SUC_lexicon_tags_17 },
    { 0x4a27069d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88706a3, SUC_lexicon_tags_19 },
    { 0x31c706a4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6706a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8706ad, SUC_lexicon_tags_23 },
    { 0x1b8706ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156706b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2706bd, SUC_lexicon_tags_46 },
    { 0x256706be, SUC_lexicon_tags_1 },
    { 0xc54706bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0706c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76706c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322706ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x844706cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x188706cf, SUC_lexicon_tags_68 },
    { 0x802706cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a706d3, SUC_lexicon_tags_17 },
    { 0x14a706d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c706da, SUC_lexicon_tags_37 },
    { 0x96e706da, SUC_lexicon_tags_134 },
    { 0xb64706da, SUC_lexicon_tags_28 },
    { 0xa54706db, SUC_lexicon_tags_1 },
    { 0xd6706de, SUC_lexicon_tags_2 },
    { 0x738706de, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x342706e2, SUC_lexicon_tags_47 },
    { 0x1fa706e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a706e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca706f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4706f9, SUC_lexicon_tags_56 },
    { 0xe22706fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x698706fc, SUC_lexicon_tags_31 },
    { 0x348706fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3070702, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77270707, SUC_lexicon_tags_0 },
    { 0x80c70708, SUC_lexicon_tags_27 },
    { 0x30470707, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5947070b, SUC_lexicon_tags_0 },
    { 0xb8a7070c, SUC_lexicon_tags_1 },
    { 0x4287070d, SUC_lexicon_tags_37 },
    { 0x2607070e, SUC_lexicon_tags_61 },
    { 0xec7070f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92a70711, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd5270713, SUC_lexicon_tags_30 },
    { 0x69670713, SUC_lexicon_tags_104 },
    { 0x65a70713, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93070719, SUC_lexicon_tags_15 },
    { 0xe887071a, SUC_lexicon_tags_3 },
    { 0x3b87071b, SUC_lexicon_tags_14 },
    { 0x21070719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5207071f, SUC_lexicon_tags_87 },
    { 0x5b070720, SUC_lexicon_tags_37 },
    { 0xc2270721, SUC_lexicon_tags_11 },
    { 0xdd070721, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d27072e, SUC_lexicon_tags_27 },
    { 0xed47072f, SUC_lexicon_tags_27 },
    { 0x73c70730, SUC_lexicon_tags_23 },
    { 0x9c670730, SUC_lexicon_tags_53 },
    { 0x91070732, SUC_lexicon_tags_45 },
    { 0xa270732, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e70736, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbf270738, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd367073a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ee7073c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce7073f, SUC_lexicon_tags_201 },
    { 0x1e470740, SUC_lexicon_tags_1 },
    { 0xe3070741, SUC_lexicon_tags_32 },
    { 0x88670742, SUC_lexicon_tags_8 },
    { 0x9be70741, SUC_lexicon_tags_17 },
    { 0xa4c70744, SUC_lexicon_tags_1 },
    { 0x10c70745, SUC_lexicon_tags_2 },
    { 0x73e70746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c70749, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a7074c, SUC_lexicon_tags_37 },
    { 0xf447074d, SUC_lexicon_tags_37 },
    { 0x81a7074c, SUC_lexicon_tags_43 },
    { 0x8ae7074f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71470752, SUC_lexicon_tags_37 },
    { 0xf2070752, SUC_lexicon_tags_14 },
    { 0xbf270754, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc70759, SUC_lexicon_tags_9 },
    { 0xc6e7075a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35670763, SUC_lexicon_tags_48 },
    { 0x93270764, SUC_lexicon_tags_11 },
    { 0x50670764, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7ae70767, SUC_lexicon_tags_25 },
    { 0x86870767, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x507076a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c7076d, SUC_lexicon_tags_3 },
    { 0x1767076e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8c70770, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a70777, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c7077a, SUC_lexicon_tags_12 },
    { 0xabc7077b, SUC_lexicon_tags_14 },
    { 0x1787077b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35070782, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40870786, SUC_lexicon_tags_27 },
    { 0xa5670786, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xea07078a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d67078f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a870794, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2270799, SUC_lexicon_tags_7 },
    { 0x24c7079a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e47079c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7487079e, SUC_lexicon_tags_35 },
    { 0xdf27079f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556707a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa42707a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6707b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a707b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2707bc, SUC_lexicon_tags_17 },
    { 0x59e707bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x142707c0, SUC_lexicon_tags_14 },
    { 0x90a707c1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc88707c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52707c7, SUC_lexicon_tags_11 },
    { 0x978707c8, SUC_lexicon_tags_0 },
    { 0xa0707c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa707cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7a6707ce, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0707d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x510707d5, SUC_lexicon_tags_17 },
    { 0xd14707d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4707da, SUC_lexicon_tags_39 },
    { 0xd38707db, SUC_lexicon_tags_3 },
    { 0xc2c707db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd3c707de, SUC_lexicon_tags_15 },
    { 0x91c707df, SUC_lexicon_tags_146 },
    { 0x490707df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e707e3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4fa707e5, SUC_lexicon_tags_7 },
    { 0xe9c707e6, SUC_lexicon_tags_17 },
    { 0x630707e6, SUC_lexicon_tags_11 },
    { 0xa34707e8, SUC_lexicon_tags_92 },
    { 0xa66707e9, SUC_lexicon_tags_25 },
    { 0x7fa707ea, SUC_lexicon_tags_39 },
    { 0xd18707ea, SUC_lexicon_tags_17 },
    { 0x67e707e9, SUC_lexicon_tags_46 },
    { 0xf60707ec, SUC_lexicon_tags_20 },
    { 0x2a4707ee, SUC_lexicon_tags_10 },
    { 0x64c707ee, SUC_lexicon_tags_31 },
    { 0x318707ef, SUC_lexicon_tags_151 },
    { 0xda4707f1, SUC_lexicon_tags_27 },
    { 0xfa2707e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18707f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6707f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac707fe, SUC_lexicon_tags_2 },
    { 0x490707ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee70801, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e070803, SUC_lexicon_tags_5 },
    { 0xfc70804, SUC_lexicon_tags_31 },
    { 0x22870804, SUC_lexicon_tags_63 },
    { 0xff670805, SUC_lexicon_tags_22 },
    { 0x84c70804, SUC_lexicon_tags_7 },
    { 0x4dc70807, SUC_lexicon_tags_68 },
    { 0xde670804, SUC_lexicon_tags_7 },
    { 0x9947080a, SUC_lexicon_tags_8 },
    { 0xc4c7080b, SUC_lexicon_tags_2 },
    { 0xc1e7080c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59870813, SUC_lexicon_tags_32 },
    { 0x58470814, SUC_lexicon_tags_17 },
    { 0xcc670815, SUC_lexicon_tags_11 },
    { 0xf0a70813, SUC_lexicon_tags_2 },
    { 0xae870817, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf27081a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23470822, SUC_lexicon_tags_37 },
    { 0x86470823, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae7082a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb347082e, SUC_lexicon_tags_30 },
    { 0xf6c7082e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a470831, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2470834, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd670836, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63c70838, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x55a7083a, SUC_lexicon_tags_43 },
    { 0xf007083b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a87083d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a470840, SUC_lexicon_tags_17 },
    { 0x2e470841, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b870843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3707084b, SUC_lexicon_tags_46 },
    { 0x87c7084c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc70852, SUC_lexicon_tags_14 },
    { 0xec670853, SUC_lexicon_tags_14 },
    { 0x73070854, SUC_lexicon_tags_49 },
    { 0x1c70855, SUC_lexicon_tags_1 },
    { 0xcd470856, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a870858, SUC_lexicon_tags_0 },
    { 0xe6a70859, SUC_lexicon_tags_11 },
    { 0xfb670858, SUC_lexicon_tags_45 },
    { 0xad070859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31870864, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17470868, SUC_lexicon_tags_37 },
    { 0xf3070869, SUC_lexicon_tags_15 },
    { 0x19670868, SUC_lexicon_tags_15 },
    { 0xc070868, SUC_lexicon_tags_23 },
    { 0x6bc7086c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcc67086e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xec270870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a70873, SUC_lexicon_tags_25 },
    { 0x1ba70874, SUC_lexicon_tags_9 },
    { 0x8dc70875, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4407087e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe670880, SUC_lexicon_tags_37 },
    { 0x7d270881, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1070885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e70888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127088c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3da7088e, SUC_lexicon_tags_0 },
    { 0xef67088f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43070891, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32c70893, SUC_lexicon_tags_262 },
    { 0xd9e70894, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5870896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67270899, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf87089b, SUC_lexicon_tags_19 },
    { 0x19c7089c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78708a4, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xcec708a6, SUC_lexicon_tags_49 },
    { 0x8aa708a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeea708a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdec708ab, SUC_lexicon_tags_17 },
    { 0x11c708ac, SUC_lexicon_tags_37 },
    { 0x352708ad, SUC_lexicon_tags_6 },
    { 0xfe4708ae, SUC_lexicon_tags_6 },
    { 0x7d0708af, SUC_lexicon_tags_8 },
    { 0xe8a708af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e708b4, SUC_lexicon_tags_12 },
    { 0xf66708b5, SUC_lexicon_tags_26 },
    { 0x72c708b5, SUC_lexicon_tags_45 },
    { 0x778708b7, SUC_lexicon_tags_0 },
    { 0x172708b7, SUC_lexicon_tags_65 },
    { 0xdb8708b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0708c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce708ce, SUC_lexicon_tags_26 },
    { 0x31e708cf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5a6708d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44708de, SUC_lexicon_tags_55 },
    { 0x18e708df, SUC_lexicon_tags_11 },
    { 0x1e708de, SUC_lexicon_tags_37 },
    { 0xc02708e0, SUC_lexicon_tags_13 },
    { 0x8da708e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe76708e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702708e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e708ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x978708ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0708f1, SUC_lexicon_tags_63 },
    { 0x778708f1, SUC_lexicon_tags_15 },
    { 0x99e708f2, SUC_lexicon_tags_7 },
    { 0x832708f3, SUC_lexicon_tags_11 },
    { 0xa22708f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a708f8, SUC_lexicon_tags_1 },
    { 0x76c708f8, SUC_lexicon_tags_15 },
    { 0xe42708fa, SUC_lexicon_tags_17 },
    { 0x9c0708fa, SUC_lexicon_tags_66 },
    { 0x2c708fc, SUC_lexicon_tags_0 },
    { 0xa4e708fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x810708ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c270902, SUC_lexicon_tags_15 },
    { 0x9ea70903, SUC_lexicon_tags_25 },
    { 0x44e70904, SUC_lexicon_tags_17 },
    { 0xf3c70903, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c7090a, SUC_lexicon_tags_19 },
    { 0x27a7090b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2767090d, SUC_lexicon_tags_12 },
    { 0x22a7090e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce70911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21070915, SUC_lexicon_tags_12 },
    { 0x95870916, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4e870918, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa047091b, SUC_lexicon_tags_65 },
    { 0x80c7091c, SUC_lexicon_tags_23 },
    { 0xdd07091d, SUC_lexicon_tags_8 },
    { 0x22a7091c, SUC_lexicon_tags_2 },
    { 0x14c7091c, SUC_lexicon_tags_10 },
    { 0x2a87091e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x44270922, SUC_lexicon_tags_1 },
    { 0xa0070923, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef270937, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda7093a, SUC_lexicon_tags_35 },
    { 0x7247093a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c870947, SUC_lexicon_tags_11 },
    { 0xae70947, SUC_lexicon_tags_11 },
    { 0x1e70949, SUC_lexicon_tags_37 },
    { 0xe9470949, SUC_lexicon_tags_7 },
    { 0x9047094a, SUC_lexicon_tags_38 },
    { 0x4c270948, SUC_lexicon_tags_0 },
    { 0x81e70949, SUC_lexicon_tags_106 },
    { 0xbb670947, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba070950, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa4470952, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3870958, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae7095c, SUC_lexicon_tags_26 },
    { 0x5067095d, SUC_lexicon_tags_33 },
    { 0x5cc7095e, SUC_lexicon_tags_37 },
    { 0xe927095f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x66270961, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88670963, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea7096a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e7096d, SUC_lexicon_tags_46 },
    { 0x9307096e, SUC_lexicon_tags_8 },
    { 0xaca7096e, SUC_lexicon_tags_235 },
    { 0xcf07096d, SUC_lexicon_tags_17 },
    { 0xa7470971, SUC_lexicon_tags_2 },
    { 0xbb67096d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a7097a, SUC_lexicon_tags_39 },
    { 0x8e67097b, SUC_lexicon_tags_25 },
    { 0x2ba7097c, SUC_lexicon_tags_14 },
    { 0x3407097d, SUC_lexicon_tags_56 },
    { 0x9927097b, SUC_lexicon_tags_27 },
    { 0x8b27097f, SUC_lexicon_tags_47 },
    { 0x18c7097e, SUC_lexicon_tags_2 },
    { 0x1fa70981, SUC_lexicon_tags_19 },
    { 0x4a070980, SUC_lexicon_tags_496 },
    { 0x47e70983, SUC_lexicon_tags_25 },
    { 0x50670984, SUC_lexicon_tags_11 },
    { 0xe3470983, SUC_lexicon_tags_25 },
    { 0xa647097f, SUC_lexicon_tags_27 },
    { 0x49a70982, SUC_lexicon_tags_34 },
    { 0x46e70987, SUC_lexicon_tags_46 },
    { 0x5b270980, SUC_lexicon_tags_12 },
    { 0xec7098a, SUC_lexicon_tags_26 },
    { 0x6de70986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd67098d, SUC_lexicon_tags_63 },
    { 0xf5c7098e, SUC_lexicon_tags_45 },
    { 0x5c27098f, SUC_lexicon_tags_7 },
    { 0xade7098d, SUC_lexicon_tags_6 },
    { 0x19270991, SUC_lexicon_tags_2 },
    { 0x4c670992, SUC_lexicon_tags_7 },
    { 0x7270993, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55270995, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b27099b, SUC_lexicon_tags_37 },
    { 0x3ac7099c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfe87099e, SUC_lexicon_tags_2 },
    { 0x6387099e, SUC_lexicon_tags_47 },
    { 0xd527099e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8709a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb66709a5, SUC_lexicon_tags_7 },
    { 0xa709a6, SUC_lexicon_tags_34 },
    { 0xde0709a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc709ab, SUC_lexicon_tags_63 },
    { 0x1f0709ac, SUC_lexicon_tags_7 },
    { 0x79a709ad, SUC_lexicon_tags_2 },
    { 0xf34709ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58709b2, SUC_lexicon_tags_28 },
    { 0x41a709b3, SUC_lexicon_tags_33 },
    { 0x5a709b4, SUC_lexicon_tags_8 },
    { 0xfc6709b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x384709b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522709bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cc709be, SUC_lexicon_tags_1 },
    { 0x6fe709be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf84709c1, SUC_lexicon_tags_37 },
    { 0xaaa709c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3a709c4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e709c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a709cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70709d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a4709d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf04709d7, SUC_lexicon_tags_12 },
    { 0xaf4709d8, SUC_lexicon_tags_11 },
    { 0xef4709d7, SUC_lexicon_tags_16 },
    { 0x5dc709d8, SUC_lexicon_tags_43 },
    { 0x172709d9, SUC_lexicon_tags_21 },
    { 0x746709db, SUC_lexicon_tags_235 },
    { 0x12709dd, SUC_lexicon_tags_7 },
    { 0x388709dd, SUC_lexicon_tags_0 },
    { 0xfe6709dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2dc709e1, SUC_lexicon_tags_1 },
    { 0x792709e2, SUC_lexicon_tags_9 },
    { 0x3f2709e3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4709e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63a709eb, SUC_lexicon_tags_17 },
    { 0xb24709ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x132709ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e709f3, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56709f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0709fe, SUC_lexicon_tags_25 },
    { 0x748709ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f270a01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b070a04, SUC_lexicon_tags_13 },
    { 0x24670a04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37470a08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf3070a0a, SUC_lexicon_tags_8 },
    { 0xea270a0b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c270a14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d870a17, SUC_lexicon_tags_25 },
    { 0x68070a18, SUC_lexicon_tags_7 },
    { 0x65870a17, SUC_lexicon_tags_43 },
    { 0xc4870a18, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9270a20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57070a25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49a70a27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33a70a29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c070a2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76270a32, SUC_lexicon_tags_33 },
    { 0x1ba70a33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c70a37, SUC_lexicon_tags_21 },
    { 0x30e70a38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3070a3d, SUC_lexicon_tags_17 },
    { 0x1bc70a3e, SUC_lexicon_tags_46 },
    { 0x20470a3d, SUC_lexicon_tags_34 },
    { 0xf1c70a40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e70a43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4270a46, SUC_lexicon_tags_7 },
    { 0xc8870a47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa9870a49, SUC_lexicon_tags_485 },
    { 0x20a70a4a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a270a4d, SUC_lexicon_tags_26 },
    { 0xde70a4e, SUC_lexicon_tags_723 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c70a51, SUC_lexicon_tags_12 },
    { 0x5e670a52, SUC_lexicon_tags_56 },
    { 0x40e70a51, SUC_lexicon_tags_45 },
    { 0x4c470a51, SUC_lexicon_tags_7 },
    { 0x70870a52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x63670a58, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf670a5d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x6ba70a5f, SUC_lexicon_tags_17 },
    { 0x99c70a60, SUC_lexicon_tags_28 },
    { 0xd0070a61, SUC_lexicon_tags_27 },
    { 0x4e670a62, SUC_lexicon_tags_33 },
    { 0x6fa70a63, SUC_lexicon_tags_47 },
    { 0xfde70a62, SUC_lexicon_tags_65 },
    { 0xb2a70a63, SUC_lexicon_tags_65 },
    { 0x18470a66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9070a6c, SUC_lexicon_tags_17 },
    { 0x2c870a6d, SUC_lexicon_tags_91 },
    { 0x23c70a6e, SUC_lexicon_tags_11 },
    { 0x98070a6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8070a73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c270a7c, SUC_lexicon_tags_46 },
    { 0x92e70a7d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce70a84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c70a89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64670a8d, SUC_lexicon_tags_49 },
    { 0x3a070a8e, SUC_lexicon_tags_40 },
    { 0xd9070a8f, SUC_lexicon_tags_11 },
    { 0xd6270a8f, SUC_lexicon_tags_17 },
    { 0x24a70a90, SUC_lexicon_tags_15 },
    { 0x77c70a92, SUC_lexicon_tags_146 },
    { 0x1f470a92, SUC_lexicon_tags_37 },
    { 0xe7a70a94, SUC_lexicon_tags_1 },
    { 0x75a70a95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdbc70a97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce270a99, SUC_lexicon_tags_15 },
    { 0x32c70a99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c70a9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e70aa1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7470aa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9870aa8, SUC_lexicon_tags_23 },
    { 0x52070aa9, SUC_lexicon_tags_47 },
    { 0x9aa70aa9, SUC_lexicon_tags_39 },
    { 0xbb070aab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c70aaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55670ab3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7270ab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c470aba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86070abf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7070ac1, SUC_lexicon_tags_11 },
    { 0x57a70ac2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f870ac9, SUC_lexicon_tags_2 },
    { 0x4be70aca, SUC_lexicon_tags_87 },
    { 0x51c70acb, SUC_lexicon_tags_37 },
    { 0x3c670aca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b870acf, SUC_lexicon_tags_19 },
    { 0x48c70acf, SUC_lexicon_tags_12 },
    { 0x9ae70ad1, SUC_lexicon_tags_65 },
    { 0x38a70ad2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de70ad5, SUC_lexicon_tags_79 },
    { 0x1fa70ad5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a70ad9, SUC_lexicon_tags_15 },
    { 0xd6a70ad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba670adc, SUC_lexicon_tags_27 },
    { 0xaec70add, SUC_lexicon_tags_26 },
    { 0xe3a70ade, SUC_lexicon_tags_11 },
    { 0xc3870add, SUC_lexicon_tags_47 },
    { 0x92e70adf, SUC_lexicon_tags_17 },
    { 0x31270ae1, SUC_lexicon_tags_0 },
    { 0x37870adf, SUC_lexicon_tags_37 },
    { 0x87270ae3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48270ae8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d270aeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee70af1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ee70af3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18070af9, SUC_lexicon_tags_49 },
    { 0x8c670afa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80270afd, SUC_lexicon_tags_25 },
    { 0xcca70afe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xace70b00, SUC_lexicon_tags_19 },
    { 0x21870b00, SUC_lexicon_tags_29 },
    { 0x42670b02, SUC_lexicon_tags_15 },
    { 0x5c70b03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d870b09, SUC_lexicon_tags_7 },
    { 0xd8470b0a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4b470b0c, SUC_lexicon_tags_43 },
    { 0x85470b0d, SUC_lexicon_tags_7 },
    { 0xbf070b0e, SUC_lexicon_tags_6 },
    { 0x46e70b0f, SUC_lexicon_tags_25 },
    { 0x34870b0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba70b13, SUC_lexicon_tags_11 },
    { 0xb0870b14, SUC_lexicon_tags_19 },
    { 0x3c270b13, SUC_lexicon_tags_11 },
    { 0x55870b16, SUC_lexicon_tags_55 },
    { 0x5dc70b17, SUC_lexicon_tags_27 },
    { 0x30e70b16, SUC_lexicon_tags_15 },
    { 0x7b470b17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcf670b1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e870b1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6870b21, SUC_lexicon_tags_13 },
    { 0xb9270b22, SUC_lexicon_tags_28 },
    { 0xff870b22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x19470b25, SUC_lexicon_tags_23 },
    { 0xcc70b26, SUC_lexicon_tags_33 },
    { 0x80670b27, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a70b2b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10270b30, SUC_lexicon_tags_23 },
    { 0x6d670b31, SUC_lexicon_tags_7 },
    { 0xfa070b32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac70b39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17470b3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2e70b3d, SUC_lexicon_tags_126 },
    { 0x5870b3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a70b42, SUC_lexicon_tags_37 },
    { 0x7c270b42, SUC_lexicon_tags_45 },
    { 0x43c70b43, SUC_lexicon_tags_35 },
    { 0x7a070b43, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e70b49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce70b4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e070b51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a70b55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a070b58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67670b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c70b66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3470b69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcac70b6b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e470b6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95670b74, SUC_lexicon_tags_17 },
    { 0x7cc70b75, SUC_lexicon_tags_12 },
    { 0x52670b75, SUC_lexicon_tags_32 },
    { 0xa0070b77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17670b7f, SUC_lexicon_tags_11 },
    { 0x3aa70b7f, SUC_lexicon_tags_46 },
    { 0x41070b81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x55070b84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9670b89, SUC_lexicon_tags_79 },
    { 0x5da70b8a, SUC_lexicon_tags_11 },
    { 0x67070b89, SUC_lexicon_tags_11 },
    { 0x31870b8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e70b8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xade70b92, SUC_lexicon_tags_7 },
    { 0xb5e70b93, SUC_lexicon_tags_27 },
    { 0xeea70b92, SUC_lexicon_tags_14 },
    { 0xb5270b95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f270b98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x54870b9b, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0x15c70b9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46070ba2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8670ba5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x65c70ba7, SUC_lexicon_tags_19 },
    { 0xc1670ba8, SUC_lexicon_tags_61 },
    { 0xc070ba8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b870bab, SUC_lexicon_tags_47 },
    { 0x87e70bac, SUC_lexicon_tags_12 },
    { 0x34270bab, SUC_lexicon_tags_38 },
    { 0x2a270bae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f270bb0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c70bbb, SUC_lexicon_tags_10 },
    { 0x15470bbc, SUC_lexicon_tags_66 },
    { 0xdc70bbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x22270bbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf270bc9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b870bcd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d670bd4, SUC_lexicon_tags_37 },
    { 0x60c70bd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8670bd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61e70bda, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c70bdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9270bdf, SUC_lexicon_tags_43 },
    { 0x8e470be0, SUC_lexicon_tags_0 },
    { 0xcbe70be1, SUC_lexicon_tags_1437 },
    { 0x7be70be1, SUC_lexicon_tags_17 },
    { 0x66070be3, SUC_lexicon_tags_17 },
    { 0x55070be3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4dc70be6, SUC_lexicon_tags_57 },
    { 0x69470be7, SUC_lexicon_tags_26 },
    { 0x5c270be8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c70bec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44270bee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6c070bf0, SUC_lexicon_tags_49 },
    { 0x5ea70bf1, SUC_lexicon_tags_10 },
    { 0x6a070bf2, SUC_lexicon_tags_27 },
    { 0xdb670bf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9870bf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x49e70bf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2070bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2070c01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60c70c03, SUC_lexicon_tags_23 },
    { 0x4a270c04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51a70c06, SUC_lexicon_tags_43 },
    { 0x81e70c07, SUC_lexicon_tags_27 },
    { 0x6a670c08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x56270c0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4070c0d, SUC_lexicon_tags_21 },
    { 0x27270c0e, SUC_lexicon_tags_25 },
    { 0x18070c0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55470c18, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f670c1c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd5270c1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e70c29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7a70c2b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd070c30, SUC_lexicon_tags_2 },
    { 0x50470c31, SUC_lexicon_tags_25 },
    { 0xb0270c31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe70c36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79270c3f, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e70c42, SUC_lexicon_tags_11 },
    { 0x85a70c43, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c670c46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65870c48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e70c4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a70c4f, SUC_lexicon_tags_118 },
    { 0x64870c50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a70c54, SUC_lexicon_tags_9 },
    { 0x69270c55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99070c58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b870c5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec670c61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6270c63, SUC_lexicon_tags_25 },
    { 0x38070c63, SUC_lexicon_tags_17 },
    { 0xb3c70c65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba70c68, SUC_lexicon_tags_43 },
    { 0xafa70c69, SUC_lexicon_tags_1 },
    { 0x9d670c68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1870c6c, SUC_lexicon_tags_68 },
    { 0xe9e70c6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33270c6f, SUC_lexicon_tags_25 },
    { 0xbce70c70, SUC_lexicon_tags_2 },
    { 0xffe70c71, SUC_lexicon_tags_25 },
    { 0xdf670c72, SUC_lexicon_tags_31 },
    { 0x22870c71, SUC_lexicon_tags_100 },
    { 0xb4070c70, SUC_lexicon_tags_35 },
    { 0x88670c75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0270c7a, SUC_lexicon_tags_15 },
    { 0xca070c7b, SUC_lexicon_tags_11 },
    { 0xde670c7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac70c7f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70270c83, SUC_lexicon_tags_37 },
    { 0x6e70c83, SUC_lexicon_tags_7 },
    { 0x26e70c85, SUC_lexicon_tags_0 },
    { 0x94c70c86, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47e70c88, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb5070c8a, SUC_lexicon_tags_62 },
    { 0xc4c70c8b, SUC_lexicon_tags_47 },
    { 0x1ae70c8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c70c8f, SUC_lexicon_tags_60 },
    { 0x75070c90, SUC_lexicon_tags_7 },
    { 0x54270c91, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b870c94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88070c97, SUC_lexicon_tags_68 },
    { 0xf4070c97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29070c9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55670c9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e70ca1, SUC_lexicon_tags_49 },
    { 0x5f870ca2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4070ca6, SUC_lexicon_tags_343 },
    { 0xe8070ca7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a70cb0, SUC_lexicon_tags_35 },
    { 0x72270cb0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a70cb4, SUC_lexicon_tags_230 },
    { 0xb670cb5, SUC_lexicon_tags_7 },
    { 0x54e70cb6, SUC_lexicon_tags_25 },
    { 0x99470cb7, SUC_lexicon_tags_21 },
    { 0x96c70cb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fc70cba, SUC_lexicon_tags_11 },
    { 0xa3c70cba, SUC_lexicon_tags_17 },
    { 0x2be70cba, SUC_lexicon_tags_35 },
    { 0x71270cbc, SUC_lexicon_tags_25 },
    { 0x94c70cbe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c70cca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbf270ccc, SUC_lexicon_tags_17 },
    { 0x40a70ccd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3bc70ccf, SUC_lexicon_tags_32 },
    { 0x2f670ccf, SUC_lexicon_tags_15 },
    { 0x2e870cd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a70cd6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x33670cd9, SUC_lexicon_tags_17 },
    { 0x57870cda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x36670cdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70ce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2070ce4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8a70ce6, SUC_lexicon_tags_23 },
    { 0xe5670ce7, SUC_lexicon_tags_11 },
    { 0x89e70ce8, SUC_lexicon_tags_1 },
    { 0xe8670ce7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xafc70ceb, SUC_lexicon_tags_15 },
    { 0x67470ceb, SUC_lexicon_tags_17 },
    { 0x6b670ced, SUC_lexicon_tags_31 },
    { 0x61470cee, SUC_lexicon_tags_37 },
    { 0xe4a70cef, SUC_lexicon_tags_9 },
    { 0x18a70cf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff870cf6, SUC_lexicon_tags_55 },
    { 0xaae70cf7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cc70cf9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce70cfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f070d00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb670d03, SUC_lexicon_tags_12 },
    { 0x20e70d04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26470d06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c70d0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x48c70d0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb270d15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c870d19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x54270d1b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e70d23, SUC_lexicon_tags_39 },
    { 0x36670d24, SUC_lexicon_tags_26 },
    { 0x1c870d24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e70d2b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x96470d2e, SUC_lexicon_tags_33 },
    { 0x88270d2e, SUC_lexicon_tags_23 },
    { 0xf0870d30, SUC_lexicon_tags_17 },
    { 0x8d670d30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b670d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd3e70d38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc70d3e, SUC_lexicon_tags_5 },
    { 0xc7e70d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b870d41, SUC_lexicon_tags_9 },
    { 0x7c070d42, SUC_lexicon_tags_19 },
    { 0x4e270d42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22470d47, SUC_lexicon_tags_11 },
    { 0xc0270d47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c70d4d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa1670d4f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a70d58, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd8070d5a, SUC_lexicon_tags_25 },
    { 0xb3070d5b, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xec670d5d, SUC_lexicon_tags_15 },
    { 0xb1470d5e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba70d63, SUC_lexicon_tags_43 },
    { 0xd4c70d64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50470d66, SUC_lexicon_tags_2 },
    { 0x3b670d67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11e70d69, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3b670d6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a70d72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa70d77, SUC_lexicon_tags_15 },
    { 0xa4270d77, SUC_lexicon_tags_12 },
    { 0x34470d79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870d7f, SUC_lexicon_tags_25 },
    { 0xca270d7f, SUC_lexicon_tags_60 },
    { 0xd4670d7f, SUC_lexicon_tags_35 },
    { 0x31470d82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac70d86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8070d89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6c70d8b, SUC_lexicon_tags_69 },
    { 0x14c70d8c, SUC_lexicon_tags_17 },
    { 0x98470d8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c70d98, SUC_lexicon_tags_34 },
    { 0x18670d98, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8870d9d, SUC_lexicon_tags_6 },
    { 0xc5270d9e, SUC_lexicon_tags_19 },
    { 0x2f870d9d, SUC_lexicon_tags_17 },
    { 0xf5470d9d, SUC_lexicon_tags_65 },
    { 0x8e870d9e, SUC_lexicon_tags_7 },
    { 0xe2470da2, SUC_lexicon_tags_26 },
    { 0x53a70d9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x80e70da5, SUC_lexicon_tags_19 },
    { 0x2e270da5, SUC_lexicon_tags_79 },
    { 0x11e70da5, SUC_lexicon_tags_17 },
    { 0x83270da8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12270dab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e70daf, SUC_lexicon_tags_11 },
    { 0xde670db0, SUC_lexicon_tags_45 },
    { 0x67a70db1, SUC_lexicon_tags_7 },
    { 0xb4070db2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c70db6, SUC_lexicon_tags_25 },
    { 0x85070db6, SUC_lexicon_tags_15 },
    { 0x47870db8, SUC_lexicon_tags_19 },
    { 0x84470db7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28470dc0, SUC_lexicon_tags_29 },
    { 0x25e70dc1, SUC_lexicon_tags_12 },
    { 0xcec70dc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b470dc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd270dc7, SUC_lexicon_tags_27 },
    { 0x47870dc8, SUC_lexicon_tags_1 },
    { 0x46e70dc8, SUC_lexicon_tags_23 },
    { 0x19e70dca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac70dce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7670dd1, SUC_lexicon_tags_12 },
    { 0xd7470dd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e70dda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x84470ddd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84270ddf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc870de2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10a70de4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ba70de6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0670dea, SUC_lexicon_tags_38 },
    { 0x42670deb, SUC_lexicon_tags_26 },
    { 0x3c670deb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x71670dee, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a70df1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e70dfc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f270e00, SUC_lexicon_tags_2 },
    { 0xb3870e01, SUC_lexicon_tags_40 },
    { 0xd5070e01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ba70e04, SUC_lexicon_tags_1 },
    { 0xe1670e05, SUC_lexicon_tags_1 },
    { 0xc6e70e06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c70e09, SUC_lexicon_tags_2 },
    { 0xcf270e0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1870e0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64070e11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb6270e13, SUC_lexicon_tags_37 },
    { 0xffa70e14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac70e1a, SUC_lexicon_tags_2 },
    { 0xede70e1b, SUC_lexicon_tags_38 },
    { 0x9ea70e1b, SUC_lexicon_tags_19 },
    { 0x54c70e1d, SUC_lexicon_tags_29 },
    { 0x9bc70e1b, SUC_lexicon_tags_0 },
    { 0xe8470e1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a870e2b, SUC_lexicon_tags_29 },
    { 0xca70e2c, SUC_lexicon_tags_599 },
    { 0, NULL },
    { 0x23670e2e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd5870e30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a70e33, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd270e37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x96670e39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4870e3f, SUC_lexicon_tags_29 },
    { 0x62070e40, SUC_lexicon_tags_11 },
    { 0x2bc70e40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75870e43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9c70e45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4870e48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa70e4b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x82a70e4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d870e52, SUC_lexicon_tags_49 },
    { 0xdf070e53, SUC_lexicon_tags_35 },
    { 0x8d870e54, SUC_lexicon_tags_11 },
    { 0x30070e55, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e70e61, SUC_lexicon_tags_30 },
    { 0x7aa70e62, SUC_lexicon_tags_60 },
    { 0x87270e61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4be70e65, SUC_lexicon_tags_1 },
    { 0xb5a70e66, SUC_lexicon_tags_15 },
    { 0x2d270e67, SUC_lexicon_tags_30 },
    { 0xeb670e67, SUC_lexicon_tags_1 },
    { 0x9da70e67, SUC_lexicon_tags_30 },
    { 0x50470e67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e70e6d, SUC_lexicon_tags_65 },
    { 0xd0a70e6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x92c70e70, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x21670e73, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c70e7a, SUC_lexicon_tags_11 },
    { 0xe4a70e7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2270e7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47670e8c, SUC_lexicon_tags_2 },
    { 0xa6670e8d, SUC_lexicon_tags_0 },
    { 0x3cc70e8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17470e92, SUC_lexicon_tags_17 },
    { 0x88c70e92, SUC_lexicon_tags_7 },
    { 0x30270e94, SUC_lexicon_tags_25 },
    { 0x3ea70e93, SUC_lexicon_tags_13 },
    { 0x56870e96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63670e98, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87270e9e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c70ea5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc7c70ea7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e70eaa, SUC_lexicon_tags_26 },
    { 0x3ba70eaa, SUC_lexicon_tags_25 },
    { 0x1fa70eac, SUC_lexicon_tags_11 },
    { 0xf1670ead, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be70eb1, SUC_lexicon_tags_0 },
    { 0x44c70eb1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xea870eb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69470eb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9a470eb8, SUC_lexicon_tags_47 },
    { 0x37a70eb8, SUC_lexicon_tags_6 },
    { 0x65470eba, SUC_lexicon_tags_2 },
    { 0x28270ebb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x54670ebd, SUC_lexicon_tags_7 },
    { 0xee070ebd, SUC_lexicon_tags_17 },
    { 0x64270ebf, SUC_lexicon_tags_73 },
    { 0x60270ec0, SUC_lexicon_tags_63 },
    { 0x89670ebd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x12670ec3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12670ec7, SUC_lexicon_tags_25 },
    { 0x4fa70ec8, SUC_lexicon_tags_29 },
    { 0xdc70ec8, SUC_lexicon_tags_15 },
    { 0x8e870ec9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec70ed3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88470edb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16470edf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3070ee2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3270ee9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa9870eeb, SUC_lexicon_tags_8 },
    { 0xbaa70eec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c70ef0, SUC_lexicon_tags_25 },
    { 0xf4670ef1, SUC_lexicon_tags_48 },
    { 0x5e670ef2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe670ef4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31470ef9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xffc70efb, SUC_lexicon_tags_4 },
    { 0xa2870efc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4470f05, SUC_lexicon_tags_60 },
    { 0x54670f05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79670f0a, SUC_lexicon_tags_7 },
    { 0x9bc70f0a, SUC_lexicon_tags_12 },
    { 0x78e70f0b, SUC_lexicon_tags_1 },
    { 0xd1070f0d, SUC_lexicon_tags_10 },
    { 0x74870f0a, SUC_lexicon_tags_5 },
    { 0x49c70f0f, SUC_lexicon_tags_118 },
    { 0x80a70f0d, SUC_lexicon_tags_11 },
    { 0xce270f11, SUC_lexicon_tags_7 },
    { 0x7c270f11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e70f15, SUC_lexicon_tags_7 },
    { 0x35470f15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a70f19, SUC_lexicon_tags_77 },
    { 0xd3470f19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x96470f1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc70f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e70f23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30470f26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf270f2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c70f2e, SUC_lexicon_tags_4 },
    { 0xa7a70f2e, SUC_lexicon_tags_25 },
    { 0x66c70f30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49a70f32, SUC_lexicon_tags_0 },
    { 0xdf070f33, SUC_lexicon_tags_11 },
    { 0x3a270f34, SUC_lexicon_tags_0 },
    { 0x1a70f34, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1670f38, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb870f3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2470f40, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc70f45, SUC_lexicon_tags_8 },
    { 0xe5c70f45, SUC_lexicon_tags_25 },
    { 0x4be70f47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41c70f49, SUC_lexicon_tags_35 },
    { 0xa2470f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9270f4d, SUC_lexicon_tags_7 },
    { 0x4dc70f4d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x17470f50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46870f58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9870f5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c70f61, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2670f67, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14470f6b, SUC_lexicon_tags_4 },
    { 0x53670f6b, SUC_lexicon_tags_4 },
    { 0x7f470f6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe2470f6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a70f75, SUC_lexicon_tags_7 },
    { 0x84070f76, SUC_lexicon_tags_37 },
    { 0x98270f77, SUC_lexicon_tags_7 },
    { 0x3ba70f78, SUC_lexicon_tags_19 },
    { 0xf4e70f79, SUC_lexicon_tags_25 },
    { 0x88670f79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97870f81, SUC_lexicon_tags_14 },
    { 0xa2c70f82, SUC_lexicon_tags_37 },
    { 0x41470f82, SUC_lexicon_tags_17 },
    { 0xd7a70f84, SUC_lexicon_tags_49 },
    { 0xea870f85, SUC_lexicon_tags_66 },
    { 0xa4670f86, SUC_lexicon_tags_7 },
    { 0x86670f85, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba70f8e, SUC_lexicon_tags_25 },
    { 0x6b470f8e, SUC_lexicon_tags_31 },
    { 0x16070f8f, SUC_lexicon_tags_26 },
    { 0x6c70f91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42870f95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d270f9c, SUC_lexicon_tags_37 },
    { 0x32470f9d, SUC_lexicon_tags_60 },
    { 0x56c70f9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb870fa2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e70fa9, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a270fac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32a70fae, SUC_lexicon_tags_1 },
    { 0x61a70faf, SUC_lexicon_tags_47 },
    { 0x4cc70faf, SUC_lexicon_tags_1 },
    { 0xa9870fb1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b070fb4, SUC_lexicon_tags_45 },
    { 0x2b670fb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0070fb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5070fba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xae470fbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce470fbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83670fc3, SUC_lexicon_tags_37 },
    { 0x13470fc4, SUC_lexicon_tags_27 },
    { 0xa5470fc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63670fc9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2470fcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79c70fcf, SUC_lexicon_tags_17 },
    { 0x13e70fd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a870fd4, SUC_lexicon_tags_25 },
    { 0xa7a70fd5, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a70fda, SUC_lexicon_tags_17 },
    { 0x46c70fdb, SUC_lexicon_tags_1 },
    { 0x5e270fda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c70fe1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2470fe3, SUC_lexicon_tags_77 },
    { 0x51670fe4, SUC_lexicon_tags_37 },
    { 0xb3a70fe5, SUC_lexicon_tags_17 },
    { 0x2c070fe5, SUC_lexicon_tags_45 },
    { 0x55670fe7, SUC_lexicon_tags_25 },
    { 0xdda70fe4, SUC_lexicon_tags_0 },
    { 0x56670fe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96670feb, SUC_lexicon_tags_7 },
    { 0x85870fec, SUC_lexicon_tags_47 },
    { 0x4c870fec, SUC_lexicon_tags_11 },
    { 0x8ea70fee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc70ff7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc70ffa, SUC_lexicon_tags_66 },
    { 0x4a670ffb, SUC_lexicon_tags_1 },
    { 0x10a70ffc, SUC_lexicon_tags_1 },
    { 0xebe70ffd, SUC_lexicon_tags_61 },
    { 0x5b870ffe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4271000, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1367100a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6847100c, SUC_lexicon_tags_23 },
    { 0x83e7100d, SUC_lexicon_tags_37 },
    { 0x4987100e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70471014, SUC_lexicon_tags_37 },
    { 0xa3c71015, SUC_lexicon_tags_24 },
    { 0x6b871016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13671019, SUC_lexicon_tags_8 },
    { 0x1227101a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb367101d, SUC_lexicon_tags_60 },
    { 0xdb27101d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79a71020, SUC_lexicon_tags_6 },
    { 0x11671021, SUC_lexicon_tags_45 },
    { 0xc0c71022, SUC_lexicon_tags_0 },
    { 0x37e71023, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeba71025, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fe71027, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa287102a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe847102f, SUC_lexicon_tags_11 },
    { 0x5471030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca71033, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf327103c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa147103f, SUC_lexicon_tags_17 },
    { 0xf5271040, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x38e71042, SUC_lexicon_tags_1 },
    { 0xebc71042, SUC_lexicon_tags_37 },
    { 0xa0a71044, SUC_lexicon_tags_7 },
    { 0xf3a71044, SUC_lexicon_tags_0 },
    { 0xa3271046, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5327104a, SUC_lexicon_tags_28 },
    { 0x7987104b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x37a7104d, SUC_lexicon_tags_6 },
    { 0x1f27104d, SUC_lexicon_tags_25 },
    { 0xb047104f, SUC_lexicon_tags_25 },
    { 0xcf87104f, SUC_lexicon_tags_15 },
    { 0xc7a7104f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc71055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3671058, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc7105d, SUC_lexicon_tags_12 },
    { 0xcbc7105e, SUC_lexicon_tags_39 },
    { 0xf867105f, SUC_lexicon_tags_14 },
    { 0x95e71060, SUC_lexicon_tags_25 },
    { 0x9b67105f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca71064, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f471067, SUC_lexicon_tags_15 },
    { 0x7ae71067, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa207106a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a7106f, SUC_lexicon_tags_11 },
    { 0x28c71070, SUC_lexicon_tags_17 },
    { 0xb4071070, SUC_lexicon_tags_63 },
    { 0x89871072, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a71075, SUC_lexicon_tags_1 },
    { 0x8471076, SUC_lexicon_tags_13 },
    { 0x44871077, SUC_lexicon_tags_27 },
    { 0xdee71076, SUC_lexicon_tags_7 },
    { 0xe7e71077, SUC_lexicon_tags_53 },
    { 0xb67107a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3807107e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc1c71080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefe71082, SUC_lexicon_tags_9 },
    { 0xb1e71083, SUC_lexicon_tags_201 },
    { 0x25e71084, SUC_lexicon_tags_7 },
    { 0x5da71085, SUC_lexicon_tags_25 },
    { 0x87671086, SUC_lexicon_tags_9 },
    { 0x29871087, SUC_lexicon_tags_63 },
    { 0x81271087, SUC_lexicon_tags_63 },
    { 0x3d471086, SUC_lexicon_tags_17 },
    { 0x8ec7108a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe947108e, SUC_lexicon_tags_25 },
    { 0x5267108e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38071093, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8d671095, SUC_lexicon_tags_23 },
    { 0xf5a71096, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d471098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a7109c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120710a5, SUC_lexicon_tags_1 },
    { 0xd3c710a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e710ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758710b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf2710b6, SUC_lexicon_tags_8 },
    { 0x1a710b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758710bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x54c710bf, SUC_lexicon_tags_25 },
    { 0xebe710bf, SUC_lexicon_tags_27 },
    { 0x54c710bf, SUC_lexicon_tags_45 },
    { 0x6c710c2, SUC_lexicon_tags_1 },
    { 0x106710c0, SUC_lexicon_tags_79 },
    { 0xbf6710c4, SUC_lexicon_tags_15 },
    { 0x61a710c4, SUC_lexicon_tags_49 },
    { 0x36c710bf, SUC_lexicon_tags_17 },
    { 0xcfc710c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee710ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde710d3, SUC_lexicon_tags_1 },
    { 0x47c710d4, SUC_lexicon_tags_17 },
    { 0xbce710d3, SUC_lexicon_tags_7 },
    { 0x4fa710d6, SUC_lexicon_tags_29 },
    { 0x1f2710d6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8710e1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0710e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x66e710e7, SUC_lexicon_tags_25 },
    { 0xa1a710e8, SUC_lexicon_tags_87 },
    { 0x29e710e8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a710ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe00710ee, SUC_lexicon_tags_19 },
    { 0xfea710ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6710f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484710f7, SUC_lexicon_tags_33 },
    { 0x2f8710f8, SUC_lexicon_tags_65 },
    { 0xc72710f9, SUC_lexicon_tags_45 },
    { 0xdf8710f9, SUC_lexicon_tags_39 },
    { 0x85c710fb, SUC_lexicon_tags_11 },
    { 0x942710fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9d8710fe, SUC_lexicon_tags_47 },
    { 0x490710ff, SUC_lexicon_tags_12 },
    { 0xb4271100, SUC_lexicon_tags_28 },
    { 0xd871100, SUC_lexicon_tags_15 },
    { 0x95271102, SUC_lexicon_tags_25 },
    { 0xdce71103, SUC_lexicon_tags_31 },
    { 0x3c471104, SUC_lexicon_tags_15 },
    { 0x76671104, SUC_lexicon_tags_27 },
    { 0xac471102, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x33471108, SUC_lexicon_tags_2 },
    { 0x61c71108, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcb67110b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd147110d, SUC_lexicon_tags_17 },
    { 0x4267110e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46671111, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3e71113, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24e71115, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e27111b, SUC_lexicon_tags_28 },
    { 0xfe47111b, SUC_lexicon_tags_2 },
    { 0x42a7111d, SUC_lexicon_tags_23 },
    { 0x7107111e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb471121, SUC_lexicon_tags_15 },
    { 0xac671122, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88c71124, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6271127, SUC_lexicon_tags_1 },
    { 0xcfe71127, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92a7112a, SUC_lexicon_tags_54 },
    { 0x5d27112b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c7112e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb071130, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e71135, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fc71137, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x62e71139, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe087113e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5471141, SUC_lexicon_tags_7 },
    { 0xd1671141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be71147, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x69071149, SUC_lexicon_tags_37 },
    { 0x6b47114a, SUC_lexicon_tags_19 },
    { 0xb027114b, SUC_lexicon_tags_2 },
    { 0x9da7114b, SUC_lexicon_tags_11 },
    { 0x6dc7114d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7567114f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea71153, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c71156, SUC_lexicon_tags_21 },
    { 0xe2471157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c7115a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a71165, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e7116a, SUC_lexicon_tags_49 },
    { 0xa2c7116a, SUC_lexicon_tags_23 },
    { 0xeaa7116c, SUC_lexicon_tags_2 },
    { 0xc607116a, SUC_lexicon_tags_25 },
    { 0xf487116e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb071175, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c71179, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1ca7117b, SUC_lexicon_tags_14 },
    { 0xde67117c, SUC_lexicon_tags_87 },
    { 0x9fe7117c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3671180, SUC_lexicon_tags_0 },
    { 0x35a71180, SUC_lexicon_tags_25 },
    { 0x5ba71181, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c71188, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4271190, SUC_lexicon_tags_45 },
    { 0xd4471191, SUC_lexicon_tags_7 },
    { 0xb9271192, SUC_lexicon_tags_15 },
    { 0x79c71190, SUC_lexicon_tags_10 },
    { 0xb4071194, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x20a71196, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38271199, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7119c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e711a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34711a5, SUC_lexicon_tags_11 },
    { 0xcd8711a5, SUC_lexicon_tags_48 },
    { 0xfe2711a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10711af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304711b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa4c711b5, SUC_lexicon_tags_15 },
    { 0xbcc711b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc34711b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8711bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62e711bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04711c1, SUC_lexicon_tags_17 },
    { 0x124711c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e6711c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4711c8, SUC_lexicon_tags_33 },
    { 0x162711c9, SUC_lexicon_tags_66 },
    { 0x82c711ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a711d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e711d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84e711d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42711dd, SUC_lexicon_tags_8 },
    { 0x7e4711dd, SUC_lexicon_tags_49 },
    { 0xc10711dd, SUC_lexicon_tags_27 },
    { 0xb42711de, SUC_lexicon_tags_1 },
    { 0xb78711e1, SUC_lexicon_tags_37 },
    { 0xd80711e2, SUC_lexicon_tags_4 },
    { 0xbc711e2, SUC_lexicon_tags_38 },
    { 0x338711e4, SUC_lexicon_tags_19 },
    { 0xd6711e1, SUC_lexicon_tags_624 },
    { 0x87a711e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74711e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26711ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6711f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8711fb, SUC_lexicon_tags_33 },
    { 0xe3c711fb, SUC_lexicon_tags_45 },
    { 0xf1e711fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x792711ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e271202, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e871206, SUC_lexicon_tags_138 },
    { 0x6ba71207, SUC_lexicon_tags_15 },
    { 0x43671206, SUC_lexicon_tags_17 },
    { 0x43c71207, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5247120d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63071214, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26271218, SUC_lexicon_tags_2 },
    { 0x5c071219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c7121c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4987121e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1071222, SUC_lexicon_tags_11 },
    { 0x1de71223, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9071226, SUC_lexicon_tags_8 },
    { 0x60271227, SUC_lexicon_tags_9 },
    { 0xd0e71227, SUC_lexicon_tags_17 },
    { 0x27c71228, SUC_lexicon_tags_9 },
    { 0x9727122a, SUC_lexicon_tags_53 },
    { 0x6ea7122b, SUC_lexicon_tags_46 },
    { 0xf5871229, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d471230, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x33071233, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5471235, SUC_lexicon_tags_11 },
    { 0x62671236, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d87123d, SUC_lexicon_tags_6 },
    { 0xc5e7123e, SUC_lexicon_tags_7 },
    { 0x5267123e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e71245, SUC_lexicon_tags_84 },
    { 0xb3871246, SUC_lexicon_tags_15 },
    { 0xe9071247, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5967124b, SUC_lexicon_tags_15 },
    { 0x94c7124c, SUC_lexicon_tags_198 },
    { 0x7a27124c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4067124f, SUC_lexicon_tags_17 },
    { 0x75271250, SUC_lexicon_tags_16 },
    { 0x8bc71250, SUC_lexicon_tags_4 },
    { 0xd6471251, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff671256, SUC_lexicon_tags_17 },
    { 0xf3a71256, SUC_lexicon_tags_14 },
    { 0x7de71257, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2167125e, SUC_lexicon_tags_47 },
    { 0xbd07125e, SUC_lexicon_tags_8 },
    { 0xcc271260, SUC_lexicon_tags_31 },
    { 0xa7471261, SUC_lexicon_tags_37 },
    { 0xafa71262, SUC_lexicon_tags_21 },
    { 0xdbe71260, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9fe71265, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba71269, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a07126c, SUC_lexicon_tags_53 },
    { 0xac87126c, SUC_lexicon_tags_49 },
    { 0x6167126c, SUC_lexicon_tags_43 },
    { 0xb327126c, SUC_lexicon_tags_11 },
    { 0x9c071270, SUC_lexicon_tags_1092 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95071274, SUC_lexicon_tags_39 },
    { 0x47e71274, SUC_lexicon_tags_37 },
    { 0xcb271274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12071279, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3da7127b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14c7127d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88671280, SUC_lexicon_tags_57 },
    { 0xe6071280, SUC_lexicon_tags_26 },
    { 0xa8071282, SUC_lexicon_tags_25 },
    { 0x1bc71283, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd9a71285, SUC_lexicon_tags_26 },
    { 0x37c71285, SUC_lexicon_tags_25 },
    { 0xac871287, SUC_lexicon_tags_2 },
    { 0x2c471288, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e7128b, SUC_lexicon_tags_9 },
    { 0x1667128c, SUC_lexicon_tags_25 },
    { 0xb7a7128b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef871293, SUC_lexicon_tags_37 },
    { 0xad471293, SUC_lexicon_tags_13 },
    { 0xfac71295, SUC_lexicon_tags_45 },
    { 0xbc071296, SUC_lexicon_tags_27 },
    { 0x41271297, SUC_lexicon_tags_29 },
    { 0x6a71294, SUC_lexicon_tags_12 },
    { 0x30871299, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa712ab, SUC_lexicon_tags_17 },
    { 0xf2e712ab, SUC_lexicon_tags_17 },
    { 0x392712ab, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x29e712af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606712b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd58712b5, SUC_lexicon_tags_47 },
    { 0x618712b5, SUC_lexicon_tags_49 },
    { 0x45a712b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec712bc, SUC_lexicon_tags_49 },
    { 0xb32712bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a712c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32712cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28c712cd, SUC_lexicon_tags_9 },
    { 0x304712ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be712d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeec712d5, SUC_lexicon_tags_29 },
    { 0x6d4712d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4712da, SUC_lexicon_tags_226 },
    { 0x3ec712db, SUC_lexicon_tags_60 },
    { 0x4d2712db, SUC_lexicon_tags_2 },
    { 0x4da712dd, SUC_lexicon_tags_11 },
    { 0xc90712dd, SUC_lexicon_tags_5 },
    { 0x302712df, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb7c712e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12712e4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32712e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32712ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8712f3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa712f7, SUC_lexicon_tags_43 },
    { 0xda8712f8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e712ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff471307, SUC_lexicon_tags_61 },
    { 0x7b271308, SUC_lexicon_tags_2 },
    { 0x24271308, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c47130c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e67130f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37871311, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9271314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a871316, SUC_lexicon_tags_91 },
    { 0xfc071316, SUC_lexicon_tags_25 },
    { 0x2e871317, SUC_lexicon_tags_38 },
    { 0x47c71319, SUC_lexicon_tags_63 },
    { 0x6be7131a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe71325, SUC_lexicon_tags_17 },
    { 0xfa71326, SUC_lexicon_tags_12 },
    { 0x32e71325, SUC_lexicon_tags_45 },
    { 0x2b071328, SUC_lexicon_tags_39 },
    { 0xb3271329, SUC_lexicon_tags_15 },
    { 0x53471326, SUC_lexicon_tags_40 },
    { 0xffa7132b, SUC_lexicon_tags_37 },
    { 0x5b07132c, SUC_lexicon_tags_37 },
    { 0xb0271327, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4fa7132f, SUC_lexicon_tags_12 },
    { 0xa7871330, SUC_lexicon_tags_29 },
    { 0xd0a7132f, SUC_lexicon_tags_47 },
    { 0xba7132f, SUC_lexicon_tags_87 },
    { 0xe3871333, SUC_lexicon_tags_27 },
    { 0x4f271334, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e71338, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da7133e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce71342, SUC_lexicon_tags_49 },
    { 0xd7a71342, SUC_lexicon_tags_66 },
    { 0x55671343, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb6871346, SUC_lexicon_tags_7 },
    { 0x1de71346, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8671349, SUC_lexicon_tags_15 },
    { 0x2c07134a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda87134d, SUC_lexicon_tags_34 },
    { 0x80c7134e, SUC_lexicon_tags_33 },
    { 0x2267134f, SUC_lexicon_tags_17 },
    { 0x70a7134d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2471357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd871359, SUC_lexicon_tags_151 },
    { 0xc3071359, SUC_lexicon_tags_15 },
    { 0x9067135b, SUC_lexicon_tags_63 },
    { 0x1a67135b, SUC_lexicon_tags_53 },
    { 0x2c67135c, SUC_lexicon_tags_23 },
    { 0xd027135c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e471360, SUC_lexicon_tags_28 },
    { 0x16271361, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa3871363, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e71367, SUC_lexicon_tags_32 },
    { 0x2b271368, SUC_lexicon_tags_15 },
    { 0x78071368, SUC_lexicon_tags_63 },
    { 0xdcc7136a, SUC_lexicon_tags_17 },
    { 0x13a71367, SUC_lexicon_tags_12 },
    { 0x3927136b, SUC_lexicon_tags_38 },
    { 0xb07136d, SUC_lexicon_tags_146 },
    { 0xd887136e, SUC_lexicon_tags_22 },
    { 0x2467136f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32271372, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37c71374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad671377, SUC_lexicon_tags_27 },
    { 0x3e871378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75e7137a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe087137c, SUC_lexicon_tags_11 },
    { 0x4647137d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b471382, SUC_lexicon_tags_47 },
    { 0xaac71383, SUC_lexicon_tags_27 },
    { 0x8671382, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8387138a, SUC_lexicon_tags_0 },
    { 0x807138b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6647138e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a071392, SUC_lexicon_tags_15 },
    { 0x5071393, SUC_lexicon_tags_11 },
    { 0xa9c71394, SUC_lexicon_tags_17 },
    { 0xa1e71394, SUC_lexicon_tags_37 },
    { 0xa6a71396, SUC_lexicon_tags_37 },
    { 0x98471393, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc327139b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7427139d, SUC_lexicon_tags_11 },
    { 0xdfa7139e, SUC_lexicon_tags_46 },
    { 0xf907139e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e0713a1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6713a5, SUC_lexicon_tags_1 },
    { 0x808713a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c713a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa68713aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x484713ac, SUC_lexicon_tags_25 },
    { 0xc26713ac, SUC_lexicon_tags_19 },
    { 0x14713ae, SUC_lexicon_tags_22 },
    { 0x17a713ac, SUC_lexicon_tags_16 },
    { 0x13a713b0, SUC_lexicon_tags_25 },
    { 0xf2c713af, SUC_lexicon_tags_29 },
    { 0xcd2713b2, SUC_lexicon_tags_28 },
    { 0x748713ae, SUC_lexicon_tags_29 },
    { 0x186713ae, SUC_lexicon_tags_15 },
    { 0x1a713b5, SUC_lexicon_tags_15 },
    { 0x4713ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e713bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68713c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950713c8, SUC_lexicon_tags_65 },
    { 0x426713c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe36713cb, SUC_lexicon_tags_25 },
    { 0x2713cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a713cf, SUC_lexicon_tags_19 },
    { 0xa3a713d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902713d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x946713d7, SUC_lexicon_tags_14 },
    { 0xcd4713d7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x792713da, SUC_lexicon_tags_7 },
    { 0xa7c713da, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820713df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x614713e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4713e5, SUC_lexicon_tags_37 },
    { 0x44c713e6, SUC_lexicon_tags_14 },
    { 0xb94713e7, SUC_lexicon_tags_37 },
    { 0x274713e7, SUC_lexicon_tags_25 },
    { 0xac2713e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e713ed, SUC_lexicon_tags_23 },
    { 0x492713ee, SUC_lexicon_tags_17 },
    { 0xb12713ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x306713f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e713f6, SUC_lexicon_tags_25 },
    { 0x95e713f7, SUC_lexicon_tags_17 },
    { 0xd54713f7, SUC_lexicon_tags_8 },
    { 0x96a713f9, SUC_lexicon_tags_7 },
    { 0x902713fa, SUC_lexicon_tags_25 },
    { 0xefa713f8, SUC_lexicon_tags_12 },
    { 0x406713f6, SUC_lexicon_tags_1 },
    { 0x8ba713fd, SUC_lexicon_tags_35 },
    { 0xc46713fe, SUC_lexicon_tags_4 },
    { 0x6e2713fd, SUC_lexicon_tags_40 },
    { 0xb74713fd, SUC_lexicon_tags_4 },
    { 0xc0271400, SUC_lexicon_tags_15 },
    { 0x188713f9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xac71404, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89471408, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7140f, SUC_lexicon_tags_17 },
    { 0x35671410, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c471414, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa727141b, SUC_lexicon_tags_17 },
    { 0x7b07141c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba87141e, SUC_lexicon_tags_473 },
    { 0x8e7141f, SUC_lexicon_tags_37 },
    { 0xbd071420, SUC_lexicon_tags_8 },
    { 0x2707141f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x18671424, SUC_lexicon_tags_25 },
    { 0xa5871425, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe67142b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7142f, SUC_lexicon_tags_17 },
    { 0x11c71430, SUC_lexicon_tags_18 },
    { 0x4f071431, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1d271433, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x871435, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66071439, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa567143c, SUC_lexicon_tags_7 },
    { 0xcac7143d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b671443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e671446, SUC_lexicon_tags_37 },
    { 0x87a71446, SUC_lexicon_tags_60 },
    { 0xda71448, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0xe0e7144a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb087144c, SUC_lexicon_tags_8 },
    { 0x8e47144d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a71453, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2f071455, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87871457, SUC_lexicon_tags_38 },
    { 0xd5871458, SUC_lexicon_tags_129 },
    { 0x68871458, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d67145d, SUC_lexicon_tags_46 },
    { 0x83a7145e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xeea71460, SUC_lexicon_tags_48 },
    { 0x17c71461, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd671465, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e71468, SUC_lexicon_tags_38 },
    { 0xbaa71468, SUC_lexicon_tags_8 },
    { 0xcde7146a, SUC_lexicon_tags_31 },
    { 0x3327146b, SUC_lexicon_tags_11 },
    { 0x4347146c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd671471, SUC_lexicon_tags_56 },
    { 0x99671472, SUC_lexicon_tags_12 },
    { 0xc8c71471, SUC_lexicon_tags_32 },
    { 0x50a71472, SUC_lexicon_tags_38 },
    { 0xd8271474, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c871479, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f87147d, SUC_lexicon_tags_17 },
    { 0xe87147d, SUC_lexicon_tags_60 },
    { 0xffc7147f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3071482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa7148b, SUC_lexicon_tags_25 },
    { 0xda7148c, SUC_lexicon_tags_1 },
    { 0xef47148c, SUC_lexicon_tags_8 },
    { 0x8d07148c, SUC_lexicon_tags_1 },
    { 0xee27148e, SUC_lexicon_tags_11 },
    { 0x8c671490, SUC_lexicon_tags_7 },
    { 0x71471491, SUC_lexicon_tags_1 },
    { 0x10a7148e, SUC_lexicon_tags_12 },
    { 0xfaa7148d, SUC_lexicon_tags_12 },
    { 0xc5071494, SUC_lexicon_tags_2 },
    { 0xf2e71495, SUC_lexicon_tags_7 },
    { 0x5ce71490, SUC_lexicon_tags_26 },
    { 0x1dc71497, SUC_lexicon_tags_30 },
    { 0xf9271497, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcec7149a, SUC_lexicon_tags_25 },
    { 0xeea7149a, SUC_lexicon_tags_31 },
    { 0xc907149a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e07149f, SUC_lexicon_tags_25 },
    { 0x86e714a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x12714a3, SUC_lexicon_tags_37 },
    { 0xdd8714a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8714aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x900714ac, SUC_lexicon_tags_25 },
    { 0xac2714ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2714b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a714b5, SUC_lexicon_tags_15 },
    { 0x474714b6, SUC_lexicon_tags_11 },
    { 0x412714b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58714b9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c714bc, SUC_lexicon_tags_2 },
    { 0x5c2714bd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xa02714bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae714c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x992714c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc2a714c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8714ca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e714d1, SUC_lexicon_tags_19 },
    { 0x374714d2, SUC_lexicon_tags_12 },
    { 0x9c714d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362714d9, SUC_lexicon_tags_6 },
    { 0x9b4714da, SUC_lexicon_tags_38 },
    { 0x1b0714d9, SUC_lexicon_tags_11 },
    { 0x706714dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x202714de, SUC_lexicon_tags_37 },
    { 0xc1c714df, SUC_lexicon_tags_39 },
    { 0xc1c714de, SUC_lexicon_tags_17 },
    { 0x6ac714e1, SUC_lexicon_tags_12 },
    { 0x24714e2, SUC_lexicon_tags_63 },
    { 0xe86714e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc714e8, SUC_lexicon_tags_7 },
    { 0x9e2714e9, SUC_lexicon_tags_21 },
    { 0xd48714e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a714f3, SUC_lexicon_tags_37 },
    { 0x7e6714f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c714f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c714ff, SUC_lexicon_tags_302 },
    { 0xf0e714ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdee71502, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d671504, SUC_lexicon_tags_37 },
    { 0x16271505, SUC_lexicon_tags_27 },
    { 0xfd271505, SUC_lexicon_tags_17 },
    { 0x1a071507, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b27150d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc71510, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd471513, SUC_lexicon_tags_113 },
    { 0xe5c71514, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f471519, SUC_lexicon_tags_37 },
    { 0x9047151a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7827151e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27071525, SUC_lexicon_tags_46 },
    { 0x67471526, SUC_lexicon_tags_45 },
    { 0x9a271526, SUC_lexicon_tags_12 },
    { 0xfb671528, SUC_lexicon_tags_46 },
    { 0x84e71528, SUC_lexicon_tags_62 },
    { 0xfa71527, SUC_lexicon_tags_14 },
    { 0xa8c71528, SUC_lexicon_tags_1 },
    { 0xf507152c, SUC_lexicon_tags_15 },
    { 0x3d67152d, SUC_lexicon_tags_63 },
    { 0xb9e7152e, SUC_lexicon_tags_4 },
    { 0xfea7152d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6471536, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x80c71538, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a7153b, SUC_lexicon_tags_60 },
    { 0xad87153c, SUC_lexicon_tags_1 },
    { 0x24c7153d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca71540, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa4c71542, SUC_lexicon_tags_15 },
    { 0xbee71543, SUC_lexicon_tags_14 },
    { 0x25271544, SUC_lexicon_tags_47 },
    { 0xedc71543, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe7154c, SUC_lexicon_tags_23 },
    { 0x3e47154d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3871552, SUC_lexicon_tags_11 },
    { 0x47471553, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4a71555, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d471558, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8627155c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c87155f, SUC_lexicon_tags_116 },
    { 0xe1471560, SUC_lexicon_tags_15 },
    { 0x8f071561, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0471563, SUC_lexicon_tags_27 },
    { 0xa3671564, SUC_lexicon_tags_11 },
    { 0x54871565, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8071567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d071569, SUC_lexicon_tags_6 },
    { 0xc8e7156a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a7156d, SUC_lexicon_tags_11 },
    { 0x35c7156d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecc71570, SUC_lexicon_tags_11 },
    { 0x16a71571, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83871575, SUC_lexicon_tags_45 },
    { 0xdda71576, SUC_lexicon_tags_26 },
    { 0xe1e71577, SUC_lexicon_tags_30 },
    { 0xa4271578, SUC_lexicon_tags_7 },
    { 0xbe671579, SUC_lexicon_tags_46 },
    { 0xefa71579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2c7157c, SUC_lexicon_tags_16 },
    { 0x4f07157d, SUC_lexicon_tags_11 },
    { 0xdee7157e, SUC_lexicon_tags_21 },
    { 0x34c7157d, SUC_lexicon_tags_37 },
    { 0x17471580, SUC_lexicon_tags_2 },
    { 0xc9271581, SUC_lexicon_tags_15 },
    { 0xbe471582, SUC_lexicon_tags_46 },
    { 0xf7a71583, SUC_lexicon_tags_11 },
    { 0xfe471584, SUC_lexicon_tags_60 },
    { 0xc5e71585, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89471588, SUC_lexicon_tags_0 },
    { 0xfc071589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb087158b, SUC_lexicon_tags_11 },
    { 0xa307158c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1f47158e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6271593, SUC_lexicon_tags_0 },
    { 0x23871593, SUC_lexicon_tags_11 },
    { 0xcfe71595, SUC_lexicon_tags_17 },
    { 0x83271595, SUC_lexicon_tags_2 },
    { 0xcc071597, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x25071599, SUC_lexicon_tags_11 },
    { 0xae87159a, SUC_lexicon_tags_11 },
    { 0x8ac7159b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5047159f, SUC_lexicon_tags_17 },
    { 0xce6715a0, SUC_lexicon_tags_17 },
    { 0x3ce715a1, SUC_lexicon_tags_7 },
    { 0xbd4715a2, SUC_lexicon_tags_33 },
    { 0xef2715a3, SUC_lexicon_tags_23 },
    { 0x4a0715a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2715ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a715af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6a715b1, SUC_lexicon_tags_46 },
    { 0x176715b1, SUC_lexicon_tags_15 },
    { 0x3e0715b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e2715b5, SUC_lexicon_tags_63 },
    { 0x92a715b5, SUC_lexicon_tags_37 },
    { 0xd40715b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20715bd, SUC_lexicon_tags_17 },
    { 0x24a715be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf00715c0, SUC_lexicon_tags_15 },
    { 0x15e715c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108715c7, SUC_lexicon_tags_17 },
    { 0xd50715c8, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x86715ca, SUC_lexicon_tags_12 },
    { 0x10c715cb, SUC_lexicon_tags_87 },
    { 0xd04715cc, SUC_lexicon_tags_7 },
    { 0xcc715ca, SUC_lexicon_tags_17 },
    { 0x64c715ca, SUC_lexicon_tags_12 },
    { 0xd0c715cf, SUC_lexicon_tags_28 },
    { 0xf7e715d0, SUC_lexicon_tags_25 },
    { 0x20c715d1, SUC_lexicon_tags_29 },
    { 0xee0715d1, SUC_lexicon_tags_23 },
    { 0x5f2715d2, SUC_lexicon_tags_7 },
    { 0xd20715d3, SUC_lexicon_tags_17 },
    { 0x7b0715d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40715d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374715de, SUC_lexicon_tags_15 },
    { 0xc62715df, SUC_lexicon_tags_15 },
    { 0x1d8715de, SUC_lexicon_tags_34 },
    { 0x94e715df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec715e7, SUC_lexicon_tags_27 },
    { 0x87c715e8, SUC_lexicon_tags_23 },
    { 0x16715e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe2715eb, SUC_lexicon_tags_35 },
    { 0x61e715ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47a715ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x112715f1, SUC_lexicon_tags_15 },
    { 0x642715f1, SUC_lexicon_tags_38 },
    { 0xbbc715f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f2715f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee715fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a715ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25271603, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x10871605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9271608, SUC_lexicon_tags_46 },
    { 0x72471609, SUC_lexicon_tags_61 },
    { 0xc2c71608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee87160d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4327160f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13071614, SUC_lexicon_tags_37 },
    { 0x46e71614, SUC_lexicon_tags_6 },
    { 0x26c71616, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe027161a, SUC_lexicon_tags_23 },
    { 0x8487161b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b271624, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a71628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6a7162a, SUC_lexicon_tags_154 },
    { 0xf67162b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf087162d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e071631, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x65271633, SUC_lexicon_tags_64 },
    { 0x39a71634, SUC_lexicon_tags_37 },
    { 0x7a671635, SUC_lexicon_tags_14 },
    { 0xc1271634, SUC_lexicon_tags_15 },
    { 0x2871634, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc627163b, SUC_lexicon_tags_0 },
    { 0xdc67163c, SUC_lexicon_tags_10 },
    { 0x50a7163d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c71640, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77271642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a7164c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a71651, SUC_lexicon_tags_10 },
    { 0x3d471651, SUC_lexicon_tags_17 },
    { 0x51871653, SUC_lexicon_tags_11 },
    { 0x6de71651, SUC_lexicon_tags_43 },
    { 0x66a71655, SUC_lexicon_tags_65 },
    { 0x10e71656, SUC_lexicon_tags_11 },
    { 0x91c71657, SUC_lexicon_tags_3 },
    { 0x65871658, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c7165c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a87165f, SUC_lexicon_tags_11 },
    { 0x37471660, SUC_lexicon_tags_12 },
    { 0xdc47165f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb471664, SUC_lexicon_tags_2 },
    { 0xdb471665, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa71668, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2907166b, SUC_lexicon_tags_37 },
    { 0xbda7166c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1687166f, SUC_lexicon_tags_1 },
    { 0xf4a71670, SUC_lexicon_tags_11 },
    { 0xd9671670, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb071674, SUC_lexicon_tags_60 },
    { 0xf3871674, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e71678, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8467167a, SUC_lexicon_tags_61 },
    { 0x1b67167b, SUC_lexicon_tags_60 },
    { 0xc747167c, SUC_lexicon_tags_2 },
    { 0x4847167d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x68071680, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38671686, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3567168a, SUC_lexicon_tags_10 },
    { 0xff07168a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9267168e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63271693, SUC_lexicon_tags_31 },
    { 0x8be71694, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x20671696, SUC_lexicon_tags_26 },
    { 0x12071696, SUC_lexicon_tags_25 },
    { 0xbe271697, SUC_lexicon_tags_26 },
    { 0xffe71698, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1de7169b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6716a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2716a6, SUC_lexicon_tags_25 },
    { 0x4e4716a6, SUC_lexicon_tags_15 },
    { 0x8fa716a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462716af, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be716b3, SUC_lexicon_tags_7 },
    { 0xf90716b4, SUC_lexicon_tags_12 },
    { 0x390716b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xee8716b7, SUC_lexicon_tags_38 },
    { 0xc7a716b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b6716ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152716c1, SUC_lexicon_tags_37 },
    { 0x94716c2, SUC_lexicon_tags_27 },
    { 0x11e716c2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf8e716c5, SUC_lexicon_tags_8 },
    { 0xc2c716c6, SUC_lexicon_tags_65 },
    { 0x2c4716c6, SUC_lexicon_tags_9 },
    { 0xbf4716c8, SUC_lexicon_tags_11 },
    { 0x90a716c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4716d0, SUC_lexicon_tags_11 },
    { 0x9d6716d0, SUC_lexicon_tags_45 },
    { 0x8fa716d2, SUC_lexicon_tags_23 },
    { 0xdee716d3, SUC_lexicon_tags_1 },
    { 0x8a716d4, SUC_lexicon_tags_2 },
    { 0x5bc716d5, SUC_lexicon_tags_47 },
    { 0x5f0716d6, SUC_lexicon_tags_26 },
    { 0x666716d7, SUC_lexicon_tags_15 },
    { 0x6f8716d8, SUC_lexicon_tags_25 },
    { 0x6cc716d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x808716dc, SUC_lexicon_tags_72 },
    { 0xd1e716dd, SUC_lexicon_tags_37 },
    { 0xeae716dd, SUC_lexicon_tags_8 },
    { 0x354716df, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4716e3, SUC_lexicon_tags_38 },
    { 0x1e4716e4, SUC_lexicon_tags_11 },
    { 0x7fa716e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x834716e8, SUC_lexicon_tags_8 },
    { 0x440716e9, SUC_lexicon_tags_49 },
    { 0x814716e8, SUC_lexicon_tags_4 },
    { 0xcfc716eb, SUC_lexicon_tags_0 },
    { 0xb32716ec, SUC_lexicon_tags_19 },
    { 0x59a716ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4be716ef, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a716f3, SUC_lexicon_tags_49 },
    { 0x412716f4, SUC_lexicon_tags_33 },
    { 0x84a716f5, SUC_lexicon_tags_43 },
    { 0xf9a716f5, SUC_lexicon_tags_11 },
    { 0x4ae716f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45471701, SUC_lexicon_tags_33 },
    { 0x39271701, SUC_lexicon_tags_7 },
    { 0x2c871702, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xff871706, SUC_lexicon_tags_49 },
    { 0xcaa71706, SUC_lexicon_tags_2 },
    { 0x55e71708, SUC_lexicon_tags_761 },
    { 0xf9271709, SUC_lexicon_tags_33 },
    { 0xafa71707, SUC_lexicon_tags_15 },
    { 0x81e71709, SUC_lexicon_tags_7 },
    { 0xeba71709, SUC_lexicon_tags_37 },
    { 0x2b87170d, SUC_lexicon_tags_7 },
    { 0x97e7170e, SUC_lexicon_tags_4 },
    { 0x69e7170f, SUC_lexicon_tags_36 },
    { 0x7e27170c, SUC_lexicon_tags_2 },
    { 0xf1a71711, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a07171a, SUC_lexicon_tags_34 },
    { 0x6867171a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa071721, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe8c71723, SUC_lexicon_tags_1 },
    { 0xc8a71724, SUC_lexicon_tags_25 },
    { 0x57871725, SUC_lexicon_tags_37 },
    { 0xfa871726, SUC_lexicon_tags_7 },
    { 0xde871727, SUC_lexicon_tags_37 },
    { 0x7fc71728, SUC_lexicon_tags_16 },
    { 0x88e71729, SUC_lexicon_tags_21 },
    { 0xc9a71729, SUC_lexicon_tags_15 },
    { 0xadc7172b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7607172e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c71735, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9d871737, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba7173a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x327173d, SUC_lexicon_tags_25 },
    { 0xe307173d, SUC_lexicon_tags_12 },
    { 0xdfc7173f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a71744, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1671746, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4271749, SUC_lexicon_tags_33 },
    { 0xf4e7174a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ee7174c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7471756, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfe471758, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e7175c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83671762, SUC_lexicon_tags_12 },
    { 0xf9c71763, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae7176b, SUC_lexicon_tags_12 },
    { 0x2a7176c, SUC_lexicon_tags_1 },
    { 0xfae7176d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4527176f, SUC_lexicon_tags_17 },
    { 0xdfe7176f, SUC_lexicon_tags_592 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f071775, SUC_lexicon_tags_37 },
    { 0xa0c71776, SUC_lexicon_tags_26 },
    { 0xd0c71776, SUC_lexicon_tags_8 },
    { 0xf271778, SUC_lexicon_tags_60 },
    { 0x13a71778, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7177d, SUC_lexicon_tags_37 },
    { 0x6087177e, SUC_lexicon_tags_10 },
    { 0xf747177e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a71785, SUC_lexicon_tags_7 },
    { 0xf7471786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb067178a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a7178c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6471793, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe7179a, SUC_lexicon_tags_33 },
    { 0x3c67179a, SUC_lexicon_tags_15 },
    { 0xe87179c, SUC_lexicon_tags_5 },
    { 0x72e7179a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c717a0, SUC_lexicon_tags_26 },
    { 0x246717a1, SUC_lexicon_tags_17 },
    { 0xba8717a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9c717a4, SUC_lexicon_tags_5 },
    { 0xabc717a4, SUC_lexicon_tags_7 },
    { 0xb96717a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x534717a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea717ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a717b4, SUC_lexicon_tags_16 },
    { 0xfc8717b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4e4717b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x610717ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a717bd, SUC_lexicon_tags_6 },
    { 0x132717be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04717c9, SUC_lexicon_tags_82 },
    { 0xef6717ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee717d0, SUC_lexicon_tags_23 },
    { 0xf98717d1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8717d4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466717d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x588717da, SUC_lexicon_tags_29 },
    { 0xe7a717da, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe70717dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a717e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928717e7, SUC_lexicon_tags_17 },
    { 0xc1c717e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade717ed, SUC_lexicon_tags_11 },
    { 0x522717ee, SUC_lexicon_tags_17 },
    { 0xe2e717ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124717f6, SUC_lexicon_tags_41 },
    { 0x11c717f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfac717f9, SUC_lexicon_tags_8 },
    { 0xf80717f9, SUC_lexicon_tags_61 },
    { 0x4717f9, SUC_lexicon_tags_45 },
    { 0xc8e717fc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b271803, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71806, SUC_lexicon_tags_25 },
    { 0x12e71806, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f87180d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba71812, SUC_lexicon_tags_65 },
    { 0xb5271813, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e71819, SUC_lexicon_tags_19 },
    { 0x4027181a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3987181c, SUC_lexicon_tags_7 },
    { 0x4aa7181d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e71820, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0x20871823, SUC_lexicon_tags_82 },
    { 0x21071823, SUC_lexicon_tags_60 },
    { 0x50671825, SUC_lexicon_tags_12 },
    { 0xc2271823, SUC_lexicon_tags_63 },
    { 0x67a71823, SUC_lexicon_tags_37 },
    { 0xc71823, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c07182e, SUC_lexicon_tags_33 },
    { 0x7707182f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c071832, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xe4a71834, SUC_lexicon_tags_146 },
    { 0xcd271835, SUC_lexicon_tags_56 },
    { 0x8d271836, SUC_lexicon_tags_0 },
    { 0x25071837, SUC_lexicon_tags_29 },
    { 0x76c71838, SUC_lexicon_tags_1 },
    { 0x2a871839, SUC_lexicon_tags_1 },
    { 0xc5a71839, SUC_lexicon_tags_68 },
    { 0x70a71835, SUC_lexicon_tags_34 },
    { 0xede7183c, SUC_lexicon_tags_9 },
    { 0xd167183b, SUC_lexicon_tags_38 },
    { 0xbc27183b, SUC_lexicon_tags_37 },
    { 0x67c7183f, SUC_lexicon_tags_84 },
    { 0x4227183a, SUC_lexicon_tags_43 },
    { 0xa9071841, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f671846, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc71849, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7184d, SUC_lexicon_tags_43 },
    { 0x9a7184d, SUC_lexicon_tags_25 },
    { 0x2367184e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0271853, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4f271855, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba71858, SUC_lexicon_tags_1 },
    { 0x8f271858, SUC_lexicon_tags_49 },
    { 0x3e27185a, SUC_lexicon_tags_151 },
    { 0x1c27185b, SUC_lexicon_tags_827 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65071863, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e671868, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f47186c, SUC_lexicon_tags_55 },
    { 0x3e27186d, SUC_lexicon_tags_47 },
    { 0x3ea7186e, SUC_lexicon_tags_1 },
    { 0xca27186f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6071878, SUC_lexicon_tags_280 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a7187b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7b47187d, SUC_lexicon_tags_9 },
    { 0x4a87187e, SUC_lexicon_tags_63 },
    { 0xb827187f, SUC_lexicon_tags_1 },
    { 0xa807187e, SUC_lexicon_tags_8 },
    { 0x57471881, SUC_lexicon_tags_37 },
    { 0x13471882, SUC_lexicon_tags_11 },
    { 0x28a71883, SUC_lexicon_tags_11 },
    { 0xd7a71884, SUC_lexicon_tags_37 },
    { 0x3871885, SUC_lexicon_tags_26 },
    { 0x4ec71886, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2107188c, SUC_lexicon_tags_5 },
    { 0x9ae7188c, SUC_lexicon_tags_2 },
    { 0x2d07188e, SUC_lexicon_tags_6 },
    { 0xca7188d, SUC_lexicon_tags_29 },
    { 0xe0671890, SUC_lexicon_tags_23 },
    { 0x12471891, SUC_lexicon_tags_4 },
    { 0x5a271892, SUC_lexicon_tags_37 },
    { 0xa927188e, SUC_lexicon_tags_45 },
    { 0x6b471892, SUC_lexicon_tags_45 },
    { 0x47071895, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3e471897, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89471899, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da7189d, SUC_lexicon_tags_11 },
    { 0x7147189e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a718a2, SUC_lexicon_tags_9 },
    { 0x6dc718a3, SUC_lexicon_tags_43 },
    { 0x794718a4, SUC_lexicon_tags_48 },
    { 0x462718a3, SUC_lexicon_tags_15 },
    { 0x910718a6, SUC_lexicon_tags_14 },
    { 0x75c718a5, SUC_lexicon_tags_87 },
    { 0xa2a718a8, SUC_lexicon_tags_22 },
    { 0xbd2718a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63a718ab, SUC_lexicon_tags_30 },
    { 0xb02718ac, SUC_lexicon_tags_21 },
    { 0xd8c718ad, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164718b1, SUC_lexicon_tags_37 },
    { 0x4e0718b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654718b6, SUC_lexicon_tags_19 },
    { 0xa2c718b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4718ba, SUC_lexicon_tags_17 },
    { 0x158718ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca718be, SUC_lexicon_tags_25 },
    { 0x8e718bf, SUC_lexicon_tags_1 },
    { 0xbb4718c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x268718c3, SUC_lexicon_tags_60 },
    { 0x534718c4, SUC_lexicon_tags_26 },
    { 0x2b8718c5, SUC_lexicon_tags_17 },
    { 0xa8a718c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x67c718c8, SUC_lexicon_tags_17 },
    { 0x2c718c9, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x4ba718cb, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c718ce, SUC_lexicon_tags_37 },
    { 0x866718cf, SUC_lexicon_tags_19 },
    { 0xc0c718d0, SUC_lexicon_tags_15 },
    { 0x896718d1, SUC_lexicon_tags_17 },
    { 0xab8718cf, SUC_lexicon_tags_63 },
    { 0xb4a718d3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6718db, SUC_lexicon_tags_17 },
    { 0x640718db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfc6718de, SUC_lexicon_tags_12 },
    { 0x14a718df, SUC_lexicon_tags_77 },
    { 0xf16718e0, SUC_lexicon_tags_9 },
    { 0xbc0718de, SUC_lexicon_tags_11 },
    { 0x930718de, SUC_lexicon_tags_27 },
    { 0xd78718e3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac718e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6718ec, SUC_lexicon_tags_1 },
    { 0x702718ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9d0718ef, SUC_lexicon_tags_30 },
    { 0x9b0718f0, SUC_lexicon_tags_17 },
    { 0xdf0718f0, SUC_lexicon_tags_21 },
    { 0x9fa718ef, SUC_lexicon_tags_15 },
    { 0x758718f0, SUC_lexicon_tags_37 },
    { 0x58e718f4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a718f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6718fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x23c718ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96871905, SUC_lexicon_tags_49 },
    { 0x1fe71906, SUC_lexicon_tags_37 },
    { 0x28471906, SUC_lexicon_tags_7 },
    { 0xe5e71906, SUC_lexicon_tags_19 },
    { 0x39671909, SUC_lexicon_tags_11 },
    { 0x33e7190a, SUC_lexicon_tags_29 },
    { 0x7a71906, SUC_lexicon_tags_82 },
    { 0xe187190c, SUC_lexicon_tags_11 },
    { 0x5a87190d, SUC_lexicon_tags_104 },
    { 0xd9a7190d, SUC_lexicon_tags_127 },
    { 0xf27190f, SUC_lexicon_tags_25 },
    { 0x3de7190f, SUC_lexicon_tags_141 },
    { 0x88c71910, SUC_lexicon_tags_38 },
    { 0x84071912, SUC_lexicon_tags_19 },
    { 0x1c471912, SUC_lexicon_tags_37 },
    { 0x42671914, SUC_lexicon_tags_39 },
    { 0x33871913, SUC_lexicon_tags_15 },
    { 0x84271913, SUC_lexicon_tags_12 },
    { 0x49071913, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d67191b, SUC_lexicon_tags_25 },
    { 0xf347191c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0c7191e, SUC_lexicon_tags_6 },
    { 0x9a87191f, SUC_lexicon_tags_37 },
    { 0x43871920, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b471927, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95071929, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9ce7192b, SUC_lexicon_tags_30 },
    { 0xd587192c, SUC_lexicon_tags_48 },
    { 0xde27192d, SUC_lexicon_tags_12 },
    { 0x887192d, SUC_lexicon_tags_12 },
    { 0x33c7192e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc871932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3671936, SUC_lexicon_tags_63 },
    { 0x3e071936, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf67193a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc7193f, SUC_lexicon_tags_14 },
    { 0x71271940, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c871943, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc271946, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x97871949, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa2c7194b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa127194e, SUC_lexicon_tags_30 },
    { 0x8d07194f, SUC_lexicon_tags_15 },
    { 0x7ba71950, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20a71952, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf707195a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa827195c, SUC_lexicon_tags_25 },
    { 0xd3a7195d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de71963, SUC_lexicon_tags_2 },
    { 0x24071964, SUC_lexicon_tags_11 },
    { 0xab271964, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab271969, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc71970, SUC_lexicon_tags_19 },
    { 0x5b871971, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3d671973, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x87271975, SUC_lexicon_tags_37 },
    { 0x88c71976, SUC_lexicon_tags_118 },
    { 0x74471977, SUC_lexicon_tags_2 },
    { 0x4b671977, SUC_lexicon_tags_170 },
    { 0xb0871979, SUC_lexicon_tags_17 },
    { 0x8c7197a, SUC_lexicon_tags_7 },
    { 0xdc07197b, SUC_lexicon_tags_1 },
    { 0x7407197c, SUC_lexicon_tags_37 },
    { 0x68c7197c, SUC_lexicon_tags_47 },
    { 0x1fe7197e, SUC_lexicon_tags_25 },
    { 0x3f87197f, SUC_lexicon_tags_7 },
    { 0xd6a7197e, SUC_lexicon_tags_11 },
    { 0x23671981, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x83471984, SUC_lexicon_tags_25 },
    { 0x1de71985, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a071990, SUC_lexicon_tags_25 },
    { 0x66871991, SUC_lexicon_tags_33 },
    { 0xba871992, SUC_lexicon_tags_15 },
    { 0x3de71991, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e071995, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xf7e71997, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9471999, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e47199c, SUC_lexicon_tags_37 },
    { 0x7107199c, SUC_lexicon_tags_24 },
    { 0x68e7199e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e719a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x252719a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x768719a8, SUC_lexicon_tags_8 },
    { 0xabc719a8, SUC_lexicon_tags_216 },
    { 0xf12719aa, SUC_lexicon_tags_1 },
    { 0x118719aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218719b1, SUC_lexicon_tags_45 },
    { 0xf7a719b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7fc719b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76719b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x492719b9, SUC_lexicon_tags_25 },
    { 0x354719ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x518719bc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1cc719be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ba719c0, SUC_lexicon_tags_26 },
    { 0x290719c0, SUC_lexicon_tags_15 },
    { 0x332719c2, SUC_lexicon_tags_1 },
    { 0x61a719c3, SUC_lexicon_tags_25 },
    { 0x24c719c3, SUC_lexicon_tags_12 },
    { 0xaa8719c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x650719c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324719cc, SUC_lexicon_tags_37 },
    { 0x1c4719cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc719d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2719d6, SUC_lexicon_tags_92 },
    { 0xab4719d7, SUC_lexicon_tags_15 },
    { 0xdca719d7, SUC_lexicon_tags_11 },
    { 0x8a2719d9, SUC_lexicon_tags_19 },
    { 0xd50719d9, SUC_lexicon_tags_44 },
    { 0xdc2719db, SUC_lexicon_tags_47 },
    { 0xdf4719dc, SUC_lexicon_tags_19 },
    { 0xdb0719da, SUC_lexicon_tags_38 },
    { 0x65a719de, SUC_lexicon_tags_12 },
    { 0x48e719dd, SUC_lexicon_tags_31 },
    { 0xb5a719e0, SUC_lexicon_tags_37 },
    { 0x148719e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98719e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74719eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0719ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce2719f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a719f6, SUC_lexicon_tags_12 },
    { 0xd06719f7, SUC_lexicon_tags_77 },
    { 0x7f4719f7, SUC_lexicon_tags_3 },
    { 0x2fa719f9, SUC_lexicon_tags_30 },
    { 0x4c2719f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fa719fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c719ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1bc71a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74c71a03, SUC_lexicon_tags_37 },
    { 0xa9671a04, SUC_lexicon_tags_1 },
    { 0x10e71a03, SUC_lexicon_tags_38 },
    { 0x70071a06, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f471a0a, SUC_lexicon_tags_15 },
    { 0x36871a0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe871a0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a871a10, SUC_lexicon_tags_12 },
    { 0x9e271a11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5271a1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12071a1f, SUC_lexicon_tags_15 },
    { 0xba071a20, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6671a23, SUC_lexicon_tags_38 },
    { 0xbec71a24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8e71a26, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe1271a28, SUC_lexicon_tags_1 },
    { 0x2271a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c871a2c, SUC_lexicon_tags_33 },
    { 0x6d271a2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2671a2f, SUC_lexicon_tags_12 },
    { 0xe8871a30, SUC_lexicon_tags_2 },
    { 0xd0a71a30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb871a36, SUC_lexicon_tags_49 },
    { 0xf7c71a37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa71a3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x39871a3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8c71a3f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc9e71a41, SUC_lexicon_tags_32 },
    { 0xe5271a41, SUC_lexicon_tags_15 },
    { 0xbe471a43, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64671a47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40471a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d871a4d, SUC_lexicon_tags_44 },
    { 0x90871a4e, SUC_lexicon_tags_6 },
    { 0x9a271a4e, SUC_lexicon_tags_11 },
    { 0xb9271a4e, SUC_lexicon_tags_37 },
    { 0x5cc71a51, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd271a55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c671a58, SUC_lexicon_tags_141 },
    { 0xe4a71a58, SUC_lexicon_tags_62 },
    { 0x78c71a5a, SUC_lexicon_tags_12 },
    { 0x8ec71a59, SUC_lexicon_tags_2 },
    { 0x4cc71a58, SUC_lexicon_tags_7 },
    { 0x6d471a5d, SUC_lexicon_tags_0 },
    { 0x51471a5d, SUC_lexicon_tags_28 },
    { 0x6271a5a, SUC_lexicon_tags_7 },
    { 0x8aa71a5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5271a63, SUC_lexicon_tags_12 },
    { 0x1b671a64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3471a69, SUC_lexicon_tags_17 },
    { 0xca071a6a, SUC_lexicon_tags_11 },
    { 0xe471a6b, SUC_lexicon_tags_2 },
    { 0x23671a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49471a6f, SUC_lexicon_tags_2 },
    { 0xb0e71a70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9871a73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9071a75, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x84871a77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78071a7e, SUC_lexicon_tags_28 },
    { 0x82671a7e, SUC_lexicon_tags_37 },
    { 0x55a71a80, SUC_lexicon_tags_7 },
    { 0x2071a81, SUC_lexicon_tags_19 },
    { 0x53471a82, SUC_lexicon_tags_15 },
    { 0x10871a83, SUC_lexicon_tags_7 },
    { 0x72e71a84, SUC_lexicon_tags_37 },
    { 0x4a271a85, SUC_lexicon_tags_63 },
    { 0x7da71a83, SUC_lexicon_tags_33 },
    { 0x29c71a87, SUC_lexicon_tags_60 },
    { 0x81471a88, SUC_lexicon_tags_45 },
    { 0x16071a88, SUC_lexicon_tags_18 },
    { 0x64871a7e, SUC_lexicon_tags_42 },
    { 0x91e71a88, SUC_lexicon_tags_37 },
    { 0xa7a71a8c, SUC_lexicon_tags_9 },
    { 0x28e71a8c, SUC_lexicon_tags_25 },
    { 0x1d471a8e, SUC_lexicon_tags_2 },
    { 0x40471a8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d071a91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf4671a93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60e71a95, SUC_lexicon_tags_11 },
    { 0x5c071a96, SUC_lexicon_tags_0 },
    { 0xba471a97, SUC_lexicon_tags_37 },
    { 0xa6871a96, SUC_lexicon_tags_37 },
    { 0x77c71a99, SUC_lexicon_tags_45 },
    { 0xf3e71a97, SUC_lexicon_tags_16 },
    { 0xd7671a9b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x15671a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72471aa0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2471aa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a71aaa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb871aaf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb471ab4, SUC_lexicon_tags_49 },
    { 0x5dc71ab4, SUC_lexicon_tags_12 },
    { 0x9f271ab5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x15271ab8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d471abb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7c71abd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa71ac2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69a71ac4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae71ac7, SUC_lexicon_tags_4 },
    { 0x63a71ac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a671aca, SUC_lexicon_tags_160 },
    { 0x1ac71aca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71871acd, SUC_lexicon_tags_38 },
    { 0xe8871acd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e71ad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e71ad6, SUC_lexicon_tags_84 },
    { 0x12e71ad6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45871ada, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22e71adc, SUC_lexicon_tags_4 },
    { 0x7b671add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce871ae1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1871ae3, SUC_lexicon_tags_12 },
    { 0x36c71ae4, SUC_lexicon_tags_51 },
    { 0x17c71ae5, SUC_lexicon_tags_37 },
    { 0x9cc71ae6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf671ae9, SUC_lexicon_tags_37 },
    { 0x5bc71aea, SUC_lexicon_tags_2 },
    { 0xfe271aea, SUC_lexicon_tags_7 },
    { 0x43071aec, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91471af9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe71afc, SUC_lexicon_tags_21 },
    { 0xb1671afc, SUC_lexicon_tags_1 },
    { 0x3ca71afe, SUC_lexicon_tags_16 },
    { 0x88471afe, SUC_lexicon_tags_27 },
    { 0x7b871afe, SUC_lexicon_tags_1 },
    { 0x2e271b01, SUC_lexicon_tags_8 },
    { 0x9ba71b01, SUC_lexicon_tags_63 },
    { 0xc6c71b03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3871b05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8671b08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9871b0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe7e71b10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b071b15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a71b18, SUC_lexicon_tags_40 },
    { 0xa4c71b18, SUC_lexicon_tags_9 },
    { 0xcdc71b1a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97871b20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4071b2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2071b2c, SUC_lexicon_tags_29 },
    { 0xbea71b2d, SUC_lexicon_tags_0 },
    { 0xa5671b2e, SUC_lexicon_tags_14 },
    { 0x60a71b2e, SUC_lexicon_tags_63 },
    { 0x1e671b2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98071b34, SUC_lexicon_tags_31 },
    { 0x30a71b35, SUC_lexicon_tags_17 },
    { 0x51e71b36, SUC_lexicon_tags_37 },
    { 0x7ec71b37, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c71b3c, SUC_lexicon_tags_17 },
    { 0x8fa71b3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20871b3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4271b45, SUC_lexicon_tags_15 },
    { 0xbe471b46, SUC_lexicon_tags_2 },
    { 0x9e471b47, SUC_lexicon_tags_49 },
    { 0x39871b48, SUC_lexicon_tags_33 },
    { 0xd2071b47, SUC_lexicon_tags_47 },
    { 0xb8c71b4a, SUC_lexicon_tags_43 },
    { 0xb7471b4a, SUC_lexicon_tags_15 },
    { 0xce871b4a, SUC_lexicon_tags_14 },
    { 0x7ea71b4d, SUC_lexicon_tags_43 },
    { 0x3c471b4c, SUC_lexicon_tags_37 },
    { 0x36471b4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de71b53, SUC_lexicon_tags_33 },
    { 0xb9271b54, SUC_lexicon_tags_63 },
    { 0xf9c71b54, SUC_lexicon_tags_7 },
    { 0x3f071b56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2071b5c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea71b5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea71b63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec71b6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c71b6e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1671b71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f471b73, SUC_lexicon_tags_39 },
    { 0x6b671b73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf871b77, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5a271b79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3071b85, SUC_lexicon_tags_11 },
    { 0x39071b85, SUC_lexicon_tags_7 },
    { 0xf3071b87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x82871b89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65471b8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7271b90, SUC_lexicon_tags_30 },
    { 0x25c71b90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2e71b93, SUC_lexicon_tags_49 },
    { 0xa3271b93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a671b99, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd071b9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d871b9e, SUC_lexicon_tags_29 },
    { 0xa3e71b9f, SUC_lexicon_tags_17 },
    { 0xac271ba0, SUC_lexicon_tags_11 },
    { 0x49871ba0, SUC_lexicon_tags_11 },
    { 0xb1c71ba1, SUC_lexicon_tags_48 },
    { 0xdd271ba0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x73071ba5, SUC_lexicon_tags_37 },
    { 0x14671ba6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b671ba8, SUC_lexicon_tags_40 },
    { 0xaba71ba9, SUC_lexicon_tags_7 },
    { 0x93671ba8, SUC_lexicon_tags_9 },
    { 0x77271baa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2871bae, SUC_lexicon_tags_47 },
    { 0x1871baf, SUC_lexicon_tags_25 },
    { 0x14071bb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f271bb4, SUC_lexicon_tags_27 },
    { 0x34471bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a71bba, SUC_lexicon_tags_29 },
    { 0x4da71bbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd871bbd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a71bc1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5cc71bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55871bc9, SUC_lexicon_tags_7 },
    { 0x3f671bca, SUC_lexicon_tags_31 },
    { 0x2a271bcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52671bce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87c71bd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2271bd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a71bd9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63071bde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6871be0, SUC_lexicon_tags_26 },
    { 0xb7471be1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21271be5, SUC_lexicon_tags_25 },
    { 0x73071be6, SUC_lexicon_tags_56 },
    { 0xa8471be7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57671bef, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7271bf2, SUC_lexicon_tags_25 },
    { 0xdec71bf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f871bf5, SUC_lexicon_tags_64 },
    { 0x97c71bf6, SUC_lexicon_tags_23 },
    { 0xa1a71bf7, SUC_lexicon_tags_11 },
    { 0x69a71bf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1a71bfa, SUC_lexicon_tags_12 },
    { 0xea671bfa, SUC_lexicon_tags_23 },
    { 0x6d671bfc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb071c00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3271c05, SUC_lexicon_tags_17 },
    { 0x34471c05, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x50271c08, SUC_lexicon_tags_17 },
    { 0xd2471c09, SUC_lexicon_tags_47 },
    { 0x7871c09, SUC_lexicon_tags_15 },
    { 0xe271c0a, SUC_lexicon_tags_11 },
    { 0x6071c0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73671c10, SUC_lexicon_tags_18 },
    { 0x99271c11, SUC_lexicon_tags_46 },
    { 0xd4471c11, SUC_lexicon_tags_11 },
    { 0xec71c12, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d871c1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c71c22, SUC_lexicon_tags_79 },
    { 0x4a671c23, SUC_lexicon_tags_26 },
    { 0xb2e71c24, SUC_lexicon_tags_7 },
    { 0x1c871c25, SUC_lexicon_tags_60 },
    { 0x29e71c26, SUC_lexicon_tags_11 },
    { 0x6ba71c26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe071c29, SUC_lexicon_tags_14 },
    { 0x2ca71c2a, SUC_lexicon_tags_16 },
    { 0xcd671c2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78a71c2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7071c31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a71c35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7471c3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfc71c3d, SUC_lexicon_tags_37 },
    { 0xe2c71c3e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55671c42, SUC_lexicon_tags_262 },
    { 0x3f071c43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9871c46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeda71c48, SUC_lexicon_tags_37 },
    { 0xd0471c49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x42c71c4b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e871c4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4871c53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd071c57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32471c59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5071c5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa071c61, SUC_lexicon_tags_45 },
    { 0x96e71c62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c71c65, SUC_lexicon_tags_47 },
    { 0xe9271c66, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4071c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe871c6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c71c72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a71c76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95471c7a, SUC_lexicon_tags_30 },
    { 0x72a71c7b, SUC_lexicon_tags_14 },
    { 0xade71c7c, SUC_lexicon_tags_47 },
    { 0x74c71c7c, SUC_lexicon_tags_47 },
    { 0x3da71c7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae671c86, SUC_lexicon_tags_26 },
    { 0x7aa71c87, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb4471c89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa071c90, SUC_lexicon_tags_14 },
    { 0x43871c91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6071c98, SUC_lexicon_tags_21 },
    { 0xfd871c99, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1ec71c9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c71ca0, SUC_lexicon_tags_39 },
    { 0x77671ca1, SUC_lexicon_tags_45 },
    { 0x1ac71ca2, SUC_lexicon_tags_37 },
    { 0xd0671ca2, SUC_lexicon_tags_45 },
    { 0x4aa71ca0, SUC_lexicon_tags_4 },
    { 0x45471ca5, SUC_lexicon_tags_7 },
    { 0x7f871ca2, SUC_lexicon_tags_47 },
    { 0x50271ca0, SUC_lexicon_tags_91 },
    { 0x56a71ca6, SUC_lexicon_tags_15 },
    { 0xcf271ca6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x62471cab, SUC_lexicon_tags_17 },
    { 0x28a71cab, SUC_lexicon_tags_3 },
    { 0x2aa71cad, SUC_lexicon_tags_37 },
    { 0x5a71cae, SUC_lexicon_tags_63 },
    { 0x53071cad, SUC_lexicon_tags_33 },
    { 0xc3671cb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2871cb5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xee071cb7, SUC_lexicon_tags_7 },
    { 0xf8c71cb8, SUC_lexicon_tags_7 },
    { 0x22271cb8, SUC_lexicon_tags_25 },
    { 0x44a71cb7, SUC_lexicon_tags_12 },
    { 0x48e71cba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57471cbd, SUC_lexicon_tags_19 },
    { 0x2c471cbd, SUC_lexicon_tags_17 },
    { 0xbc871cbf, SUC_lexicon_tags_7 },
    { 0x9cc71cc0, SUC_lexicon_tags_11 },
    { 0xc1c71cc0, SUC_lexicon_tags_40 },
    { 0x72071cc2, SUC_lexicon_tags_1 },
    { 0xcce71cc3, SUC_lexicon_tags_65 },
    { 0x78271cc3, SUC_lexicon_tags_17 },
    { 0x88071cc0, SUC_lexicon_tags_49 },
    { 0xdcc71cc6, SUC_lexicon_tags_0 },
    { 0xe4a71cc6, SUC_lexicon_tags_17 },
    { 0x59271cc6, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b471ccf, SUC_lexicon_tags_15 },
    { 0xeb671ccf, SUC_lexicon_tags_13 },
    { 0x8ae71cd0, SUC_lexicon_tags_25 },
    { 0x13871cd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42471cd4, SUC_lexicon_tags_33 },
    { 0x1a671cd4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x14a71cd7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f871cda, SUC_lexicon_tags_23 },
    { 0x49a71cdb, SUC_lexicon_tags_15 },
    { 0xb3271cdb, SUC_lexicon_tags_38 },
    { 0x5e071cdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86a71cdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74271ce1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab671ceb, SUC_lexicon_tags_46 },
    { 0xe8071cec, SUC_lexicon_tags_23 },
    { 0x1ba71cec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e71cf3, SUC_lexicon_tags_12 },
    { 0x2271cf3, SUC_lexicon_tags_14 },
    { 0x45271cf5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe671cf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49871cfc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d871d01, SUC_lexicon_tags_112 },
    { 0x4071d02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c71d07, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e71d0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c71d11, SUC_lexicon_tags_60 },
    { 0x44c71d11, SUC_lexicon_tags_6 },
    { 0x89671d13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x67871d16, SUC_lexicon_tags_12 },
    { 0x99e71d17, SUC_lexicon_tags_39 },
    { 0x10271d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6871d1d, SUC_lexicon_tags_25 },
    { 0x36071d1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3471d21, SUC_lexicon_tags_15 },
    { 0x25271d22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce71d26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74671d2a, SUC_lexicon_tags_47 },
    { 0xfc271d2b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x73071d2d, SUC_lexicon_tags_14 },
    { 0x2871d2d, SUC_lexicon_tags_33 },
    { 0xdea71d2f, SUC_lexicon_tags_43 },
    { 0xa471d30, SUC_lexicon_tags_14 },
    { 0x6e471d31, SUC_lexicon_tags_12 },
    { 0xbbe71d30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce71d36, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23071d3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d871d41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb671d47, SUC_lexicon_tags_0 },
    { 0xce871d48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce71d4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da71d53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ee71d55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a871d59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec71d5f, SUC_lexicon_tags_11 },
    { 0x20071d60, SUC_lexicon_tags_37 },
    { 0x17e71d61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97271d63, SUC_lexicon_tags_1 },
    { 0xd5671d63, SUC_lexicon_tags_2 },
    { 0xa9a71d65, SUC_lexicon_tags_6 },
    { 0x49a71d66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd271d6b, SUC_lexicon_tags_47 },
    { 0xf0e71d6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe1871d6e, SUC_lexicon_tags_28 },
    { 0x9a271d6f, SUC_lexicon_tags_38 },
    { 0xd2e71d6f, SUC_lexicon_tags_65 },
    { 0x5d671d70, SUC_lexicon_tags_15 },
    { 0x7dc71d6f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a71d79, SUC_lexicon_tags_37 },
    { 0xe7471d7a, SUC_lexicon_tags_19 },
    { 0x2e071d7b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba71d7e, SUC_lexicon_tags_61 },
    { 0x5fa71d7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4271d81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe471d83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc71d87, SUC_lexicon_tags_7 },
    { 0xd2871d88, SUC_lexicon_tags_19 },
    { 0xb4a71d89, SUC_lexicon_tags_37 },
    { 0xebc71d87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47271d8c, SUC_lexicon_tags_91 },
    { 0xff871d8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6de71d8f, SUC_lexicon_tags_17 },
    { 0x46271d8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd071d93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x40471d96, SUC_lexicon_tags_60 },
    { 0x26271d96, SUC_lexicon_tags_8 },
    { 0xc4671d98, SUC_lexicon_tags_30 },
    { 0x9c71d96, SUC_lexicon_tags_30 },
    { 0x1fe71d9a, SUC_lexicon_tags_63 },
    { 0xef671d9b, SUC_lexicon_tags_37 },
    { 0x20471d9a, SUC_lexicon_tags_26 },
    { 0x71271d9d, SUC_lexicon_tags_2 },
    { 0x90471d9d, SUC_lexicon_tags_47 },
    { 0x8ae71d9d, SUC_lexicon_tags_55 },
    { 0xcba71d9b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a071da2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a71da6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e71db1, SUC_lexicon_tags_15 },
    { 0xd1c71db2, SUC_lexicon_tags_1 },
    { 0x5c071db2, SUC_lexicon_tags_14 },
    { 0x3c471db4, SUC_lexicon_tags_17 },
    { 0x5de71db5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a671dba, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2ba71dbc, SUC_lexicon_tags_27 },
    { 0x5fa71dbd, SUC_lexicon_tags_7 },
    { 0x5ba71dbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda271dc0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3471dc2, SUC_lexicon_tags_37 },
    { 0xf4271dc3, SUC_lexicon_tags_47 },
    { 0x60e71dc4, SUC_lexicon_tags_4 },
    { 0xeee71dc4, SUC_lexicon_tags_6 },
    { 0x49071dc3, SUC_lexicon_tags_1 },
    { 0x19471dc7, SUC_lexicon_tags_8 },
    { 0xe6c71dc8, SUC_lexicon_tags_33 },
    { 0x8d071dc8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x88071dcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc71dce, SUC_lexicon_tags_2 },
    { 0x48071dcf, SUC_lexicon_tags_37 },
    { 0xdd671dd0, SUC_lexicon_tags_37 },
    { 0x22071dce, SUC_lexicon_tags_15 },
    { 0x4ca71dd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa71dd6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25271dda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57471ddc, SUC_lexicon_tags_35 },
    { 0x22871ddd, SUC_lexicon_tags_12 },
    { 0x6ee71dde, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6ec71de0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb471de3, SUC_lexicon_tags_91 },
    { 0x2b471de4, SUC_lexicon_tags_33 },
    { 0x14471de3, SUC_lexicon_tags_37 },
    { 0x60e71de4, SUC_lexicon_tags_1 },
    { 0xb8071de7, SUC_lexicon_tags_33 },
    { 0x5da71de8, SUC_lexicon_tags_48 },
    { 0xab271de6, SUC_lexicon_tags_11 },
    { 0xea271de3, SUC_lexicon_tags_25 },
    { 0xf8e71deb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba71def, SUC_lexicon_tags_28 },
    { 0x70871df0, SUC_lexicon_tags_43 },
    { 0xc1871df1, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xb8671df3, SUC_lexicon_tags_27 },
    { 0x28471df4, SUC_lexicon_tags_10 },
    { 0x6b671df3, SUC_lexicon_tags_17 },
    { 0x6ae71df6, SUC_lexicon_tags_11 },
    { 0x1c71df7, SUC_lexicon_tags_66 },
    { 0xd3271df8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b671dfb, SUC_lexicon_tags_74 },
    { 0xde071dfc, SUC_lexicon_tags_46 },
    { 0xc0e71dfd, SUC_lexicon_tags_39 },
    { 0xbd471dfb, SUC_lexicon_tags_49 },
    { 0xcbc71dff, SUC_lexicon_tags_80 },
    { 0x6fa71dff, SUC_lexicon_tags_60 },
    { 0xbce71e01, SUC_lexicon_tags_11 },
    { 0x86671e02, SUC_lexicon_tags_39 },
    { 0xb6e71e02, SUC_lexicon_tags_12 },
    { 0x4be71e04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21271e09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88271e0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f071e1a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6271e1e, SUC_lexicon_tags_27 },
    { 0xac271e1f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x71271e21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3a71e23, SUC_lexicon_tags_37 },
    { 0xe2271e24, SUC_lexicon_tags_27 },
    { 0x6d071e23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc71e29, SUC_lexicon_tags_7 },
    { 0x7e271e29, SUC_lexicon_tags_15 },
    { 0xb4c71e2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb671e30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec71e33, SUC_lexicon_tags_19 },
    { 0x26071e34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60c71e36, SUC_lexicon_tags_44 },
    { 0x84871e36, SUC_lexicon_tags_17 },
    { 0x1d271e37, SUC_lexicon_tags_2 },
    { 0xf7471e39, SUC_lexicon_tags_31 },
    { 0x50471e3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb271e3f, SUC_lexicon_tags_1 },
    { 0x4de71e3f, SUC_lexicon_tags_179 },
    { 0xa5071e41, SUC_lexicon_tags_48 },
    { 0x1a271e41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c71e47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bc71e49, SUC_lexicon_tags_2 },
    { 0x13c71e49, SUC_lexicon_tags_47 },
    { 0x36c71e4a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce271e4f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x37e71e51, SUC_lexicon_tags_25 },
    { 0xbe271e52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c71e5a, SUC_lexicon_tags_25 },
    { 0xea671e5b, SUC_lexicon_tags_23 },
    { 0x56a71e5b, SUC_lexicon_tags_262 },
    { 0xb5671e5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2671e62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec71e65, SUC_lexicon_tags_49 },
    { 0xf3c71e65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49271e68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf471e6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5471e6e, SUC_lexicon_tags_14 },
    { 0xb7271e6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c71e74, SUC_lexicon_tags_31 },
    { 0x75871e74, SUC_lexicon_tags_7 },
    { 0xf3471e76, SUC_lexicon_tags_108 },
    { 0x94c71e76, SUC_lexicon_tags_65 },
    { 0xb6c71e76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78671e7b, SUC_lexicon_tags_15 },
    { 0xd4671e7b, SUC_lexicon_tags_12 },
    { 0xd0071e7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59c71e7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e471e8c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a71e92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82a71e94, SUC_lexicon_tags_2 },
    { 0x39871e95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3871e98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe071e9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfee71ea0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b271ea8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9671eae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa271eb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c71ebb, SUC_lexicon_tags_47 },
    { 0x1c71ebc, SUC_lexicon_tags_7 },
    { 0x7b071ebd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98871ec3, SUC_lexicon_tags_25 },
    { 0xeec71ec3, SUC_lexicon_tags_12 },
    { 0x7b071ec4, SUC_lexicon_tags_14 },
    { 0x11a71ec6, SUC_lexicon_tags_33 },
    { 0xd7871ec7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd4871ec9, SUC_lexicon_tags_17 },
    { 0x40e71ec9, SUC_lexicon_tags_10 },
    { 0x99a71eca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf9271ecd, SUC_lexicon_tags_2 },
    { 0xc0071ece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xade71ed0, SUC_lexicon_tags_61 },
    { 0x1e271ed1, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a71ed5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3b071ed7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83271edd, SUC_lexicon_tags_27 },
    { 0xa0271ede, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86471ee6, SUC_lexicon_tags_37 },
    { 0x65871ee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d871ee9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ac71eeb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26071eed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4a71eef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb071ef2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9071ef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c71efa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd0e71efc, SUC_lexicon_tags_2 },
    { 0x61471efd, SUC_lexicon_tags_40 },
    { 0x47471efd, SUC_lexicon_tags_38 },
    { 0x23e71efe, SUC_lexicon_tags_32 },
    { 0xb8e71efe, SUC_lexicon_tags_15 },
    { 0xd4271efe, SUC_lexicon_tags_37 },
    { 0x52271f02, SUC_lexicon_tags_17 },
    { 0x7c471efd, SUC_lexicon_tags_37 },
    { 0x42e71eff, SUC_lexicon_tags_17 },
    { 0x2d271f02, SUC_lexicon_tags_54 },
    { 0xd1071f05, SUC_lexicon_tags_5 },
    { 0x39271f04, SUC_lexicon_tags_19 },
    { 0xf7271f08, SUC_lexicon_tags_31 },
    { 0x70a71f09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e71f0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef071f0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15471f13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55071f16, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x50871f18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70271f1b, SUC_lexicon_tags_6 },
    { 0x7a471f1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30071f1e, SUC_lexicon_tags_0 },
    { 0x68a71f1f, SUC_lexicon_tags_12 },
    { 0x77e71f20, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc71f2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc071f2d, SUC_lexicon_tags_27 },
    { 0x4d271f2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecc71f30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a71f36, SUC_lexicon_tags_15 },
    { 0x1a671f36, SUC_lexicon_tags_2 },
    { 0x2271f38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57671f3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab471f43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf071f46, SUC_lexicon_tags_0 },
    { 0xbe271f47, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f071f4c, SUC_lexicon_tags_27 },
    { 0x4c871f4d, SUC_lexicon_tags_15 },
    { 0x38871f4d, SUC_lexicon_tags_38 },
    { 0x27671f4f, SUC_lexicon_tags_11 },
    { 0x7d071f50, SUC_lexicon_tags_0 },
    { 0x71a71f51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41a71f53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd0e71f55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1471f58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17871f5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78271f64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4471f67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbac71f69, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x68271f6c, SUC_lexicon_tags_25 },
    { 0xb9271f6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee71f72, SUC_lexicon_tags_2 },
    { 0x27e71f72, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f071f76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8671f79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63271f7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e071f7e, SUC_lexicon_tags_25 },
    { 0x37671f7f, SUC_lexicon_tags_2 },
    { 0x57e71f7f, SUC_lexicon_tags_25 },
    { 0xe7c71f81, SUC_lexicon_tags_10 },
    { 0x18671f82, SUC_lexicon_tags_31 },
    { 0x6f471f83, SUC_lexicon_tags_69 },
    { 0x71e71f84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a71f88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87271f8d, SUC_lexicon_tags_15 },
    { 0xc4e71f8d, SUC_lexicon_tags_80 },
    { 0xff871f8f, SUC_lexicon_tags_0 },
    { 0xfd071f8d, SUC_lexicon_tags_37 },
    { 0xb9671f91, SUC_lexicon_tags_10 },
    { 0x20871f92, SUC_lexicon_tags_26 },
    { 0x3ae71f93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba71f96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa6271f98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9c871f9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd471f9d, SUC_lexicon_tags_37 },
    { 0x73271f9e, SUC_lexicon_tags_15 },
    { 0xdda71f9d, SUC_lexicon_tags_0 },
    { 0x3f271fa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c71fa4, SUC_lexicon_tags_11 },
    { 0xa8271fa5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5271fa7, SUC_lexicon_tags_23 },
    { 0x22e71fa7, SUC_lexicon_tags_11 },
    { 0x64e71fa7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2871fac, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99071fb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64071fb4, SUC_lexicon_tags_29 },
    { 0x4e871fb5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6871fb9, SUC_lexicon_tags_63 },
    { 0x73471fba, SUC_lexicon_tags_12 },
    { 0xb9871fbb, SUC_lexicon_tags_38 },
    { 0x9de71fb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7a71fbe, SUC_lexicon_tags_39 },
    { 0x13471fbf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e671fc7, SUC_lexicon_tags_1 },
    { 0x18271fc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4671fd2, SUC_lexicon_tags_43 },
    { 0x36471fd3, SUC_lexicon_tags_11 },
    { 0x8ea71fd2, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65271fd8, SUC_lexicon_tags_11 },
    { 0x13871fd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdea71fdb, SUC_lexicon_tags_7 },
    { 0x5671fdc, SUC_lexicon_tags_27 },
    { 0xa2271fdd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3f071fdf, SUC_lexicon_tags_60 },
    { 0x7a671fe0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84671fe2, SUC_lexicon_tags_15 },
    { 0x65271fe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6471feb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18471ff3, SUC_lexicon_tags_7 },
    { 0xebe71ff4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5871ff8, SUC_lexicon_tags_9 },
    { 0xdce71ff9, SUC_lexicon_tags_0 },
    { 0xc871ffa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e72000, SUC_lexicon_tags_1 },
    { 0xb8072001, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9272003, SUC_lexicon_tags_33 },
    { 0xea872004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9272007, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea72009, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e7200f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9872012, SUC_lexicon_tags_30 },
    { 0xb3e72013, SUC_lexicon_tags_47 },
    { 0x35c72014, SUC_lexicon_tags_37 },
    { 0xc1472012, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6dc72017, SUC_lexicon_tags_1 },
    { 0x88472018, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5687201b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca7201e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca672023, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4272028, SUC_lexicon_tags_359 },
    { 0x2bc72029, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8487202b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f27202d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf327202f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb672031, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15472036, SUC_lexicon_tags_12 },
    { 0xb5a72037, SUC_lexicon_tags_37 },
    { 0x5f672037, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda7203b, SUC_lexicon_tags_12 },
    { 0x1167203b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43272040, SUC_lexicon_tags_14 },
    { 0x54272041, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3c72043, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a72047, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf647204d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9872051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25c72053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56672055, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4272057, SUC_lexicon_tags_17 },
    { 0x8c072058, SUC_lexicon_tags_12 },
    { 0x72472058, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e7205d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c72061, SUC_lexicon_tags_46 },
    { 0x88e72062, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e72065, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a7206c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc907206e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc272071, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba72075, SUC_lexicon_tags_31 },
    { 0x13672076, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b072078, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf967207b, SUC_lexicon_tags_60 },
    { 0x6dc7207c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1807207f, SUC_lexicon_tags_25 },
    { 0xc8072080, SUC_lexicon_tags_12 },
    { 0xa1672080, SUC_lexicon_tags_11 },
    { 0xf7a72082, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x41272084, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0872087, SUC_lexicon_tags_25 },
    { 0xaee72088, SUC_lexicon_tags_9 },
    { 0xb0e72087, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe7208e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8272091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e72098, SUC_lexicon_tags_38 },
    { 0x3b672099, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7047209d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c720a0, SUC_lexicon_tags_27 },
    { 0x1ee720a1, SUC_lexicon_tags_79 },
    { 0x10720a2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98720a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64720ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x854720ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x482720af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6720b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76720b5, SUC_lexicon_tags_11 },
    { 0x59c720b6, SUC_lexicon_tags_15 },
    { 0x99e720b7, SUC_lexicon_tags_77 },
    { 0x34a720b8, SUC_lexicon_tags_22 },
    { 0x734720b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf66720bb, SUC_lexicon_tags_28 },
    { 0x32a720bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32720c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68720c9, SUC_lexicon_tags_2 },
    { 0x9c0720ca, SUC_lexicon_tags_0 },
    { 0xa26720ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a720cf, SUC_lexicon_tags_38 },
    { 0xcd8720d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x446720d2, SUC_lexicon_tags_49 },
    { 0x7ea720d3, SUC_lexicon_tags_39 },
    { 0xb9e720d3, SUC_lexicon_tags_32 },
    { 0x48c720d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98720d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94720db, SUC_lexicon_tags_2 },
    { 0x62720dc, SUC_lexicon_tags_25 },
    { 0x8a720dd, SUC_lexicon_tags_11 },
    { 0x844720dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430720eb, SUC_lexicon_tags_12 },
    { 0x7e6720ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78720f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86a720f4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2720f8, SUC_lexicon_tags_38 },
    { 0xec2720f8, SUC_lexicon_tags_3 },
    { 0x73e720f8, SUC_lexicon_tags_15 },
    { 0xabe720fb, SUC_lexicon_tags_7 },
    { 0x9ea720fc, SUC_lexicon_tags_11 },
    { 0x316720fd, SUC_lexicon_tags_17 },
    { 0xeea720fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43c72100, SUC_lexicon_tags_1 },
    { 0xef272101, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c072104, SUC_lexicon_tags_7 },
    { 0x6ba72105, SUC_lexicon_tags_63 },
    { 0xf8a72105, SUC_lexicon_tags_2 },
    { 0x1ba72107, SUC_lexicon_tags_2 },
    { 0xb472104, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde67210c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c72117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20872119, SUC_lexicon_tags_17 },
    { 0x9007211a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e272122, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa6272124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c672127, SUC_lexicon_tags_25 },
    { 0x86672128, SUC_lexicon_tags_38 },
    { 0x71072129, SUC_lexicon_tags_25 },
    { 0x5a872127, SUC_lexicon_tags_7 },
    { 0xc047212b, SUC_lexicon_tags_30 },
    { 0x7727212c, SUC_lexicon_tags_19 },
    { 0x2307212d, SUC_lexicon_tags_2 },
    { 0x44a7212a, SUC_lexicon_tags_37 },
    { 0x5be7212b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a72134, SUC_lexicon_tags_12 },
    { 0x27872134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c72138, SUC_lexicon_tags_7 },
    { 0x1d272138, SUC_lexicon_tags_15 },
    { 0x3227213a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc72145, SUC_lexicon_tags_11 },
    { 0x40e72146, SUC_lexicon_tags_23 },
    { 0x27e72146, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f872149, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5487214b, SUC_lexicon_tags_0 },
    { 0xcfc7214c, SUC_lexicon_tags_11 },
    { 0x84e7214b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc72151, SUC_lexicon_tags_160 },
    { 0x40872152, SUC_lexicon_tags_25 },
    { 0x87072152, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x44c72155, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b67215a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65872160, SUC_lexicon_tags_15 },
    { 0x51472160, SUC_lexicon_tags_16 },
    { 0xe1672162, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16072164, SUC_lexicon_tags_26 },
    { 0x80272165, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c272167, SUC_lexicon_tags_25 },
    { 0x5d472167, SUC_lexicon_tags_37 },
    { 0xac672167, SUC_lexicon_tags_17 },
    { 0x74872168, SUC_lexicon_tags_29 },
    { 0xa3e7216b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6472170, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe5872172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71672175, SUC_lexicon_tags_2 },
    { 0xc872176, SUC_lexicon_tags_17 },
    { 0x76072177, SUC_lexicon_tags_4 },
    { 0x3b072178, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xeec7217a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x59e7217c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3807218a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7247218d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e72190, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ae72192, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc72195, SUC_lexicon_tags_37 },
    { 0x3ce72196, SUC_lexicon_tags_37 },
    { 0x91672197, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7867219e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a721a3, SUC_lexicon_tags_20 },
    { 0xa1e721a3, SUC_lexicon_tags_1 },
    { 0xe5c721a3, SUC_lexicon_tags_1 },
    { 0x218721a4, SUC_lexicon_tags_39 },
    { 0x4ca721a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc08721a9, SUC_lexicon_tags_16 },
    { 0x78e721a9, SUC_lexicon_tags_17 },
    { 0x398721ab, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20721ae, SUC_lexicon_tags_14 },
    { 0x7c721af, SUC_lexicon_tags_19 },
    { 0xcba721b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x716721b2, SUC_lexicon_tags_65 },
    { 0xee4721b3, SUC_lexicon_tags_56 },
    { 0x6ca721b2, SUC_lexicon_tags_37 },
    { 0xd38721b5, SUC_lexicon_tags_38 },
    { 0x4c721b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x362721b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364721c4, SUC_lexicon_tags_11 },
    { 0x8c4721c4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa721cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952721d0, SUC_lexicon_tags_17 },
    { 0x5ca721d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2721d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa56721d7, SUC_lexicon_tags_124 },
    { 0x8e0721d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfca721da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6721dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476721e4, SUC_lexicon_tags_21 },
    { 0x894721e5, SUC_lexicon_tags_32 },
    { 0x242721e4, SUC_lexicon_tags_21 },
    { 0xbd4721e7, SUC_lexicon_tags_12 },
    { 0xaee721e4, SUC_lexicon_tags_10 },
    { 0xe6721e4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c721ed, SUC_lexicon_tags_17 },
    { 0x71c721ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4721f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ec721f3, SUC_lexicon_tags_34 },
    { 0xcfc721f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a721f8, SUC_lexicon_tags_17 },
    { 0xc68721f9, SUC_lexicon_tags_37 },
    { 0x1e4721fa, SUC_lexicon_tags_7 },
    { 0xd9a721f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x18e721fd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea72200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24272203, SUC_lexicon_tags_15 },
    { 0xf6272204, SUC_lexicon_tags_21 },
    { 0x30672205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc472207, SUC_lexicon_tags_2 },
    { 0xa4472208, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e7220b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7220f, SUC_lexicon_tags_70 },
    { 0xcc87220f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48e72212, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36272216, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb072219, SUC_lexicon_tags_11 },
    { 0xc0e7221a, SUC_lexicon_tags_15 },
    { 0x2ee7221b, SUC_lexicon_tags_60 },
    { 0x58c72219, SUC_lexicon_tags_7 },
    { 0xa407221b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c72223, SUC_lexicon_tags_38 },
    { 0x37e72224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cc72226, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a72229, SUC_lexicon_tags_17 },
    { 0xc467222a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7222c, SUC_lexicon_tags_12 },
    { 0xd5e7222d, SUC_lexicon_tags_3 },
    { 0x8287222e, SUC_lexicon_tags_65 },
    { 0xcfe7222c, SUC_lexicon_tags_17 },
    { 0xd07222d, SUC_lexicon_tags_27 },
    { 0x50e7222d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a72237, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba07223b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7527223d, SUC_lexicon_tags_37 },
    { 0x5547223d, SUC_lexicon_tags_15 },
    { 0xade7223f, SUC_lexicon_tags_7 },
    { 0x4ba72240, SUC_lexicon_tags_15 },
    { 0x60072240, SUC_lexicon_tags_39 },
    { 0x9ea72240, SUC_lexicon_tags_7 },
    { 0x2a67223e, SUC_lexicon_tags_9 },
    { 0x17872244, SUC_lexicon_tags_84 },
    { 0x2867223e, SUC_lexicon_tags_40 },
    { 0x1a872246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb147224b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e72251, SUC_lexicon_tags_0 },
    { 0xc272252, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26872258, SUC_lexicon_tags_2 },
    { 0x6472259, SUC_lexicon_tags_25 },
    { 0xa3072258, SUC_lexicon_tags_25 },
    { 0xfa47225b, SUC_lexicon_tags_46 },
    { 0xfb27225c, SUC_lexicon_tags_27 },
    { 0x1ac7225d, SUC_lexicon_tags_19 },
    { 0xb347225e, SUC_lexicon_tags_0 },
    { 0xae67225d, SUC_lexicon_tags_2 },
    { 0xa5272260, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a672262, SUC_lexicon_tags_8 },
    { 0xd9872262, SUC_lexicon_tags_7 },
    { 0x4c072263, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe272267, SUC_lexicon_tags_28 },
    { 0xfb272268, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9007226b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c872276, SUC_lexicon_tags_46 },
    { 0x23e72277, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73872279, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c272282, SUC_lexicon_tags_68 },
    { 0xa2c72283, SUC_lexicon_tags_12 },
    { 0x6a272284, SUC_lexicon_tags_23 },
    { 0xa7872285, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0272288, SUC_lexicon_tags_8 },
    { 0x94672288, SUC_lexicon_tags_7 },
    { 0xeb272289, SUC_lexicon_tags_63 },
    { 0x6727228a, SUC_lexicon_tags_47 },
    { 0x8e07228b, SUC_lexicon_tags_55 },
    { 0x7547228d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x38872290, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21072295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81072298, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd1e7229a, SUC_lexicon_tags_23 },
    { 0xfac7229b, SUC_lexicon_tags_8 },
    { 0x97c7229b, SUC_lexicon_tags_28 },
    { 0x28a7229a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e27229f, SUC_lexicon_tags_24 },
    { 0xe0722a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4722a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x660722a7, SUC_lexicon_tags_29 },
    { 0x688722a8, SUC_lexicon_tags_15 },
    { 0x3de722a9, SUC_lexicon_tags_25 },
    { 0xe0e722a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0722ae, SUC_lexicon_tags_15 },
    { 0xc2a722af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8722b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a722b6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae722ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e722bd, SUC_lexicon_tags_33 },
    { 0x8d0722be, SUC_lexicon_tags_11 },
    { 0x14e722bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf42722c1, SUC_lexicon_tags_39 },
    { 0x8a8722c2, SUC_lexicon_tags_49 },
    { 0xfd0722c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ae722c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a722cb, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8722ce, SUC_lexicon_tags_17 },
    { 0x222722cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6722d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a722db, SUC_lexicon_tags_37 },
    { 0x33a722db, SUC_lexicon_tags_17 },
    { 0x6b0722dd, SUC_lexicon_tags_27 },
    { 0xcd2722de, SUC_lexicon_tags_6 },
    { 0x4ce722de, SUC_lexicon_tags_2 },
    { 0x850722dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e722e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x252722e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5c722e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46c722eb, SUC_lexicon_tags_1 },
    { 0x348722ec, SUC_lexicon_tags_29 },
    { 0xd36722ed, SUC_lexicon_tags_1 },
    { 0x48722ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce722f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5c722f5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08722fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3272301, SUC_lexicon_tags_11 },
    { 0x80072301, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92472305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3072307, SUC_lexicon_tags_12 },
    { 0x11e72308, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447230c, SUC_lexicon_tags_47 },
    { 0x4fc7230d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2472314, SUC_lexicon_tags_160 },
    { 0x2d072315, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa327231c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf607231f, SUC_lexicon_tags_28 },
    { 0x5f072320, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5672324, SUC_lexicon_tags_25 },
    { 0xa8072325, SUC_lexicon_tags_17 },
    { 0x8fa72326, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2472328, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb327232e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18672330, SUC_lexicon_tags_25 },
    { 0x81872330, SUC_lexicon_tags_3 },
    { 0x47072330, SUC_lexicon_tags_45 },
    { 0xb6872330, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77272335, SUC_lexicon_tags_2 },
    { 0x31a72336, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4472339, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b67233b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb27233e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6472343, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x30a72345, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe872349, SUC_lexicon_tags_737 },
    { 0x94872349, SUC_lexicon_tags_31 },
    { 0xfd27234a, SUC_lexicon_tags_14 },
    { 0xc027234c, SUC_lexicon_tags_107 },
    { 0x5f27234d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x447234f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x95a72351, SUC_lexicon_tags_54 },
    { 0xbc272352, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1207235b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeec7235d, SUC_lexicon_tags_45 },
    { 0x1aa7235e, SUC_lexicon_tags_18 },
    { 0x2ec7235e, SUC_lexicon_tags_10 },
    { 0x86a72360, SUC_lexicon_tags_7 },
    { 0xffa72360, SUC_lexicon_tags_19 },
    { 0xe3c72362, SUC_lexicon_tags_33 },
    { 0xc6e72363, SUC_lexicon_tags_42 },
    { 0xb7e72364, SUC_lexicon_tags_17 },
    { 0xe5872361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7236a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf507236d, SUC_lexicon_tags_16 },
    { 0x12a7236d, SUC_lexicon_tags_7 },
    { 0x2dc7236d, SUC_lexicon_tags_17 },
    { 0xb7c72370, SUC_lexicon_tags_162 },
    { 0x88272371, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e72376, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x23472379, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6872381, SUC_lexicon_tags_15 },
    { 0xb2c72381, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xda072385, SUC_lexicon_tags_37 },
    { 0x1fe72386, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c72392, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c072399, SUC_lexicon_tags_19 },
    { 0xa2c7239a, SUC_lexicon_tags_39 },
    { 0x3dc7239a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0723a4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe4a723a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x512723a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b2723ab, SUC_lexicon_tags_65 },
    { 0x7be723ac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94723b1, SUC_lexicon_tags_15 },
    { 0xd0723b2, SUC_lexicon_tags_2 },
    { 0xe80723b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90723b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836723be, SUC_lexicon_tags_461 },
    { 0x962723bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9fe723c1, SUC_lexicon_tags_33 },
    { 0xc0e723c1, SUC_lexicon_tags_6 },
    { 0x202723c3, SUC_lexicon_tags_31 },
    { 0x3ce723c4, SUC_lexicon_tags_12 },
    { 0x3c6723c5, SUC_lexicon_tags_25 },
    { 0x342723c5, SUC_lexicon_tags_45 },
    { 0x6c0723c5, SUC_lexicon_tags_11 },
    { 0xf40723c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75e723ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcd2723cc, SUC_lexicon_tags_1 },
    { 0x654723cc, SUC_lexicon_tags_15 },
    { 0x816723cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5b8723d0, SUC_lexicon_tags_27 },
    { 0x45a723d1, SUC_lexicon_tags_39 },
    { 0xf50723d0, SUC_lexicon_tags_12 },
    { 0x6e2723d3, SUC_lexicon_tags_37 },
    { 0xf62723d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e723d7, SUC_lexicon_tags_28 },
    { 0x964723d7, SUC_lexicon_tags_37 },
    { 0x4aa723d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8723df, SUC_lexicon_tags_55 },
    { 0xbd8723df, SUC_lexicon_tags_19 },
    { 0x6f4723df, SUC_lexicon_tags_15 },
    { 0x3c723e2, SUC_lexicon_tags_63 },
    { 0x60a723e2, SUC_lexicon_tags_151 },
    { 0x640723e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6723ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4ec723ee, SUC_lexicon_tags_1 },
    { 0xaba723ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba723f3, SUC_lexicon_tags_4 },
    { 0x2b4723f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26723f7, SUC_lexicon_tags_17 },
    { 0xc56723f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x698723fa, SUC_lexicon_tags_7 },
    { 0xbe2723fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8723ff, SUC_lexicon_tags_7 },
    { 0xb8072400, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf1c72402, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e72407, SUC_lexicon_tags_17 },
    { 0xfa672408, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c7240b, SUC_lexicon_tags_7 },
    { 0x1747240c, SUC_lexicon_tags_31 },
    { 0xfa47240d, SUC_lexicon_tags_14 },
    { 0xf007240e, SUC_lexicon_tags_37 },
    { 0xb6a7240f, SUC_lexicon_tags_2 },
    { 0x4407240d, SUC_lexicon_tags_9 },
    { 0x1ee72411, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e72415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2272417, SUC_lexicon_tags_17 },
    { 0x37c72418, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c7241b, SUC_lexicon_tags_43 },
    { 0x2fe7241c, SUC_lexicon_tags_68 },
    { 0x3ca7241d, SUC_lexicon_tags_1 },
    { 0x1127241e, SUC_lexicon_tags_21 },
    { 0x5b67241f, SUC_lexicon_tags_15 },
    { 0x8507241e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7072423, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d272427, SUC_lexicon_tags_11 },
    { 0xdac72428, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c72431, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a72435, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9472439, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967243d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa72442, SUC_lexicon_tags_17 },
    { 0xd0872442, SUC_lexicon_tags_6 },
    { 0xff272444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e72447, SUC_lexicon_tags_11 },
    { 0x94672448, SUC_lexicon_tags_0 },
    { 0x40872449, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce7244c, SUC_lexicon_tags_7 },
    { 0x1787244d, SUC_lexicon_tags_12 },
    { 0x2387244e, SUC_lexicon_tags_63 },
    { 0xce87244f, SUC_lexicon_tags_37 },
    { 0x9ac7244f, SUC_lexicon_tags_47 },
    { 0x4ac72451, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d272454, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21872458, SUC_lexicon_tags_35 },
    { 0x54272459, SUC_lexicon_tags_7 },
    { 0x77272458, SUC_lexicon_tags_7 },
    { 0x5f27245a, SUC_lexicon_tags_12 },
    { 0x66e72458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac72462, SUC_lexicon_tags_11 },
    { 0x5d072463, SUC_lexicon_tags_4 },
    { 0x96272464, SUC_lexicon_tags_7 },
    { 0xf1472465, SUC_lexicon_tags_4 },
    { 0xf0e72464, SUC_lexicon_tags_37 },
    { 0xaf272464, SUC_lexicon_tags_25 },
    { 0x7aa72464, SUC_lexicon_tags_87 },
    { 0x45472469, SUC_lexicon_tags_37 },
    { 0x6127246a, SUC_lexicon_tags_45 },
    { 0x83672469, SUC_lexicon_tags_91 },
    { 0x1227246a, SUC_lexicon_tags_63 },
    { 0x82c7246b, SUC_lexicon_tags_37 },
    { 0x23a7246d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa72470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23072479, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3872481, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x46272483, SUC_lexicon_tags_13 },
    { 0xe1c72484, SUC_lexicon_tags_27 },
    { 0x8d472484, SUC_lexicon_tags_27 },
    { 0x20672483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4872488, SUC_lexicon_tags_11 },
    { 0xeca72489, SUC_lexicon_tags_26 },
    { 0x5ae7248a, SUC_lexicon_tags_63 },
    { 0xc927248b, SUC_lexicon_tags_63 },
    { 0x767248c, SUC_lexicon_tags_37 },
    { 0x4a7248b, SUC_lexicon_tags_217 },
    { 0x9b67248b, SUC_lexicon_tags_32 },
    { 0xa807248e, SUC_lexicon_tags_37 },
    { 0x1fe72490, SUC_lexicon_tags_31 },
    { 0x5b872491, SUC_lexicon_tags_65 },
    { 0x5ea72492, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a72495, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9987249f, SUC_lexicon_tags_6 },
    { 0x658724a0, SUC_lexicon_tags_2 },
    { 0xd30724a0, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xb96724a3, SUC_lexicon_tags_11 },
    { 0x602724a4, SUC_lexicon_tags_25 },
    { 0xdfe724a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188724a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e724ae, SUC_lexicon_tags_37 },
    { 0xdfc724af, SUC_lexicon_tags_55 },
    { 0xed8724af, SUC_lexicon_tags_56 },
    { 0x306724b1, SUC_lexicon_tags_31 },
    { 0xfd6724b2, SUC_lexicon_tags_87 },
    { 0xfe8724b3, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xb34724b5, SUC_lexicon_tags_34 },
    { 0x624724b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61c724b8, SUC_lexicon_tags_12 },
    { 0x30e724b9, SUC_lexicon_tags_74 },
    { 0x6d6724b9, SUC_lexicon_tags_2 },
    { 0x9d2724b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc724bf, SUC_lexicon_tags_19 },
    { 0xfde724c0, SUC_lexicon_tags_45 },
    { 0x4c8724c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928724c6, SUC_lexicon_tags_21 },
    { 0x332724c6, SUC_lexicon_tags_37 },
    { 0x79a724c7, SUC_lexicon_tags_7 },
    { 0x282724c7, SUC_lexicon_tags_11 },
    { 0x52e724c7, SUC_lexicon_tags_15 },
    { 0x8dc724ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e724cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0724d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e6724d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c724d9, SUC_lexicon_tags_15 },
    { 0x8bc724da, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c724de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8724e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc84724e3, SUC_lexicon_tags_15 },
    { 0x8e6724e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14724e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36e724e9, SUC_lexicon_tags_1 },
    { 0x5cc724ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec724ee, SUC_lexicon_tags_2 },
    { 0x1ae724ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86724f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68724f7, SUC_lexicon_tags_15 },
    { 0xe2a724f7, SUC_lexicon_tags_12 },
    { 0x9dc724f9, SUC_lexicon_tags_7 },
    { 0x2d2724f8, SUC_lexicon_tags_6 },
    { 0xf24724f9, SUC_lexicon_tags_55 },
    { 0x25e724fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6072501, SUC_lexicon_tags_21 },
    { 0xd3272501, SUC_lexicon_tags_38 },
    { 0x3f872503, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d472507, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf707250b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb827250f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcac72511, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d672515, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd087251b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb472520, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8872527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec7252f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c872531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a7253a, SUC_lexicon_tags_14 },
    { 0xcbc7253b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa7253e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x61672541, SUC_lexicon_tags_45 },
    { 0x11072542, SUC_lexicon_tags_25 },
    { 0xfcc72543, SUC_lexicon_tags_44 },
    { 0xeba72544, SUC_lexicon_tags_11 },
    { 0x15672545, SUC_lexicon_tags_15 },
    { 0x80e72545, SUC_lexicon_tags_10 },
    { 0x2de72545, SUC_lexicon_tags_27 },
    { 0x95a72548, SUC_lexicon_tags_29 },
    { 0x4c672544, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9927254b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a7254e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa72554, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e72558, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47255c, SUC_lexicon_tags_25 },
    { 0x4207255d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a72560, SUC_lexicon_tags_14 },
    { 0xb9072560, SUC_lexicon_tags_65 },
    { 0x2e272561, SUC_lexicon_tags_0 },
    { 0xdd072561, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x95472566, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x35a72568, SUC_lexicon_tags_62 },
    { 0x5ee72568, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e67256c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a72571, SUC_lexicon_tags_29 },
    { 0xd9672572, SUC_lexicon_tags_33 },
    { 0x74672572, SUC_lexicon_tags_47 },
    { 0x69e72574, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x37472576, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfbe72578, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a7257b, SUC_lexicon_tags_37 },
    { 0x8c87257c, SUC_lexicon_tags_11 },
    { 0x8847257d, SUC_lexicon_tags_19 },
    { 0xfcc7257b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7072580, SUC_lexicon_tags_27 },
    { 0x78a72580, SUC_lexicon_tags_0 },
    { 0x38a72580, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c72588, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b27258a, SUC_lexicon_tags_60 },
    { 0x1787258a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa227258f, SUC_lexicon_tags_49 },
    { 0xe0272590, SUC_lexicon_tags_15 },
    { 0xb7472590, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x49072593, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4472597, SUC_lexicon_tags_60 },
    { 0x73272598, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd727259c, SUC_lexicon_tags_15 },
    { 0x1b87259d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5d07259f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a725a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x548725aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994725af, SUC_lexicon_tags_19 },
    { 0x2a6725b0, SUC_lexicon_tags_34 },
    { 0x26a725b1, SUC_lexicon_tags_15 },
    { 0xe5e725b2, SUC_lexicon_tags_12 },
    { 0x5aa725b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272725bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc725be, SUC_lexicon_tags_15 },
    { 0x126725be, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7e6725c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ce725c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x984725c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c725c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x244725cc, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xa8e725ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb84725d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32725d3, SUC_lexicon_tags_13 },
    { 0xc9e725d3, SUC_lexicon_tags_12 },
    { 0xa0725d5, SUC_lexicon_tags_1 },
    { 0x9d2725d3, SUC_lexicon_tags_1 },
    { 0xf04725d7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde725db, SUC_lexicon_tags_23 },
    { 0xe60725dc, SUC_lexicon_tags_49 },
    { 0xfb2725dd, SUC_lexicon_tags_12 },
    { 0xf0e725de, SUC_lexicon_tags_27 },
    { 0xe36725df, SUC_lexicon_tags_15 },
    { 0xc34725e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6a725e2, SUC_lexicon_tags_68 },
    { 0x774725e3, SUC_lexicon_tags_19 },
    { 0xc78725e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6725ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48725ed, SUC_lexicon_tags_15 },
    { 0x544725ee, SUC_lexicon_tags_29 },
    { 0x926725ee, SUC_lexicon_tags_17 },
    { 0xb92725ee, SUC_lexicon_tags_15 },
    { 0xc44725f1, SUC_lexicon_tags_51 },
    { 0xd4c725f1, SUC_lexicon_tags_7 },
    { 0xd14725ef, SUC_lexicon_tags_17 },
    { 0xe9a725f1, SUC_lexicon_tags_11 },
    { 0xde2725f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8b4725f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c725fa, SUC_lexicon_tags_1 },
    { 0xdd4725fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116725ff, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x54872602, SUC_lexicon_tags_12 },
    { 0x38e72602, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e72606, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29272609, SUC_lexicon_tags_17 },
    { 0x20a7260a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac72610, SUC_lexicon_tags_45 },
    { 0x88e72611, SUC_lexicon_tags_72 },
    { 0x60e72612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63472615, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9c672617, SUC_lexicon_tags_37 },
    { 0x54872618, SUC_lexicon_tags_25 },
    { 0xd272619, SUC_lexicon_tags_15 },
    { 0xa3a72619, SUC_lexicon_tags_34 },
    { 0x65a7261b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6672620, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda72623, SUC_lexicon_tags_25 },
    { 0xcf672624, SUC_lexicon_tags_30 },
    { 0x5c272625, SUC_lexicon_tags_14 },
    { 0x89a72626, SUC_lexicon_tags_8 },
    { 0xee872625, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33872629, SUC_lexicon_tags_11 },
    { 0x47262a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfc7262c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe7262f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae72634, SUC_lexicon_tags_25 },
    { 0x41872635, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84072637, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb227263a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x467263c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2ee7263e, SUC_lexicon_tags_38 },
    { 0x5167263e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a72643, SUC_lexicon_tags_33 },
    { 0x15272643, SUC_lexicon_tags_31 },
    { 0x3a072645, SUC_lexicon_tags_17 },
    { 0x9cc72645, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa7264b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd667264e, SUC_lexicon_tags_8 },
    { 0xd687264e, SUC_lexicon_tags_26 },
    { 0x64c72650, SUC_lexicon_tags_5 },
    { 0x81872650, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x85472654, SUC_lexicon_tags_17 },
    { 0x15672655, SUC_lexicon_tags_33 },
    { 0x67872655, SUC_lexicon_tags_23 },
    { 0x88672657, SUC_lexicon_tags_37 },
    { 0x3c672658, SUC_lexicon_tags_25 },
    { 0x55872657, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xef87265b, SUC_lexicon_tags_47 },
    { 0x3447265b, SUC_lexicon_tags_11 },
    { 0xa6e7265d, SUC_lexicon_tags_8 },
    { 0x6987265e, SUC_lexicon_tags_37 },
    { 0xc047265b, SUC_lexicon_tags_17 },
    { 0x8b272660, SUC_lexicon_tags_17 },
    { 0x3e67265c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x95672664, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c7266a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1272671, SUC_lexicon_tags_23 },
    { 0x1f872672, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac72675, SUC_lexicon_tags_10 },
    { 0x99472676, SUC_lexicon_tags_25 },
    { 0x39472675, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c7267b, SUC_lexicon_tags_8 },
    { 0xfde7267b, SUC_lexicon_tags_37 },
    { 0x57c7267c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef672683, SUC_lexicon_tags_15 },
    { 0xe7272684, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92672688, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6907268a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e72695, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c7269b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7587269e, SUC_lexicon_tags_25 },
    { 0xcc07269f, SUC_lexicon_tags_25 },
    { 0xdf8726a0, SUC_lexicon_tags_27 },
    { 0x7a6726a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa58726a3, SUC_lexicon_tags_26 },
    { 0x3ee726a3, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82726a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90726ab, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938726b2, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2726b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384726ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e0726bc, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x994726be, SUC_lexicon_tags_2 },
    { 0x92e726bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0726c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2726c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf84726c8, SUC_lexicon_tags_11 },
    { 0xcfa726c9, SUC_lexicon_tags_17 },
    { 0x564726c8, SUC_lexicon_tags_198 },
    { 0xf6c726cb, SUC_lexicon_tags_63 },
    { 0x790726cc, SUC_lexicon_tags_25 },
    { 0x82726cd, SUC_lexicon_tags_12 },
    { 0x8ec726cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2726d3, SUC_lexicon_tags_68 },
    { 0xada726d4, SUC_lexicon_tags_9 },
    { 0xb9a726d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54726df, SUC_lexicon_tags_2 },
    { 0x284726e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4726ed, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72726f0, SUC_lexicon_tags_6 },
    { 0x3b8726f1, SUC_lexicon_tags_48 },
    { 0xb78726f2, SUC_lexicon_tags_27 },
    { 0x95c726f2, SUC_lexicon_tags_1174 },
    { 0x524726f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c726f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876726fb, SUC_lexicon_tags_0 },
    { 0xb42726fc, SUC_lexicon_tags_15 },
    { 0x344726fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe34726ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb872701, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e72704, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26472708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe587270f, SUC_lexicon_tags_47 },
    { 0x6c67270f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a72713, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb8672715, SUC_lexicon_tags_37 },
    { 0x33072716, SUC_lexicon_tags_19 },
    { 0x4c872715, SUC_lexicon_tags_37 },
    { 0xe8072718, SUC_lexicon_tags_11 },
    { 0x9dc72715, SUC_lexicon_tags_0 },
    { 0xa5072719, SUC_lexicon_tags_55 },
    { 0xef87271b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf987271d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x507271f, SUC_lexicon_tags_19 },
    { 0xc4e72720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f272722, SUC_lexicon_tags_17 },
    { 0x68072722, SUC_lexicon_tags_27 },
    { 0x96072724, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c72727, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe2a72729, SUC_lexicon_tags_28 },
    { 0x83c7272a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda7272e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2672734, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7273d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9472744, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0272746, SUC_lexicon_tags_25 },
    { 0x48c72746, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1a72749, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e7274c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd487274e, SUC_lexicon_tags_3 },
    { 0xa767274e, SUC_lexicon_tags_25 },
    { 0x8127274f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa472753, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x81472756, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x61072758, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd347275c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e7275f, SUC_lexicon_tags_17 },
    { 0x6e72760, SUC_lexicon_tags_26 },
    { 0xfc072760, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a72765, SUC_lexicon_tags_25 },
    { 0x8b672766, SUC_lexicon_tags_8 },
    { 0x9cc72766, SUC_lexicon_tags_23 },
    { 0x7c472767, SUC_lexicon_tags_23 },
    { 0xc7672767, SUC_lexicon_tags_17 },
    { 0x13072766, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2872778, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe207277a, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0xb087277c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2107277f, SUC_lexicon_tags_38 },
    { 0xd1c72780, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3472783, SUC_lexicon_tags_30 },
    { 0xc272784, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c72787, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a27278a, SUC_lexicon_tags_37 },
    { 0x1a7278b, SUC_lexicon_tags_9 },
    { 0x1ae7278c, SUC_lexicon_tags_37 },
    { 0xa547278b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9ba7278f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce72796, SUC_lexicon_tags_65 },
    { 0x5a272797, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd147279c, SUC_lexicon_tags_23 },
    { 0x75a7279c, SUC_lexicon_tags_17 },
    { 0xc907279e, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e727a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf4a727a3, SUC_lexicon_tags_63 },
    { 0xfde727a4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x96e727a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2727ab, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xce8727ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e727b2, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c727b9, SUC_lexicon_tags_25 },
    { 0x154727ba, SUC_lexicon_tags_10 },
    { 0x12e727bb, SUC_lexicon_tags_31 },
    { 0x700727bc, SUC_lexicon_tags_17 },
    { 0x1e0727bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x284727bf, SUC_lexicon_tags_17 },
    { 0x5ac727bf, SUC_lexicon_tags_4 },
    { 0xe0e727bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x944727c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8727cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e727cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2727d2, SUC_lexicon_tags_48 },
    { 0xb4727d3, SUC_lexicon_tags_37 },
    { 0x76e727d4, SUC_lexicon_tags_15 },
    { 0x2bc727d5, SUC_lexicon_tags_11 },
    { 0xc0727d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348727db, SUC_lexicon_tags_35 },
    { 0x4e727dc, SUC_lexicon_tags_26 },
    { 0x8a6727dd, SUC_lexicon_tags_25 },
    { 0x998727dc, SUC_lexicon_tags_8 },
    { 0xc08727df, SUC_lexicon_tags_2 },
    { 0xaa727dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2727e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a727e7, SUC_lexicon_tags_124 },
    { 0x8a8727e8, SUC_lexicon_tags_1 },
    { 0xaea727e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x320727eb, SUC_lexicon_tags_17 },
    { 0xa74727ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3c727ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x122727f1, SUC_lexicon_tags_47 },
    { 0x6ba727f2, SUC_lexicon_tags_26 },
    { 0x6b0727f3, SUC_lexicon_tags_66 },
    { 0xe3a727f3, SUC_lexicon_tags_63 },
    { 0xebc727f3, SUC_lexicon_tags_77 },
    { 0x16c727f1, SUC_lexicon_tags_2 },
    { 0x260727f2, SUC_lexicon_tags_12 },
    { 0x27c727f6, SUC_lexicon_tags_26 },
    { 0x5f4727f3, SUC_lexicon_tags_33 },
    { 0xfc0727fa, SUC_lexicon_tags_6 },
    { 0xe3e727fb, SUC_lexicon_tags_15 },
    { 0x866727f3, SUC_lexicon_tags_17 },
    { 0xe74727fd, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xfa727ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff872803, SUC_lexicon_tags_47 },
    { 0xf9472803, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb672809, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f672814, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x75672817, SUC_lexicon_tags_19 },
    { 0x59272817, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3507281c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be72823, SUC_lexicon_tags_2 },
    { 0x9e872824, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a472827, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb067282c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde27282e, SUC_lexicon_tags_2 },
    { 0x4407282e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43072834, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12672837, SUC_lexicon_tags_30 },
    { 0x8f472838, SUC_lexicon_tags_7 },
    { 0xe7872838, SUC_lexicon_tags_0 },
    { 0x5d072838, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa47283c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d272840, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a72844, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c72849, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8887284e, SUC_lexicon_tags_25 },
    { 0x2707284f, SUC_lexicon_tags_55 },
    { 0x45c7284f, SUC_lexicon_tags_15 },
    { 0x2f87284f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14272853, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de72858, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e7285d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f272860, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf907286c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3072876, SUC_lexicon_tags_11 },
    { 0xf9272877, SUC_lexicon_tags_15 },
    { 0x3e472877, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc7287b, SUC_lexicon_tags_2 },
    { 0x7d27287c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac7287f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb472884, SUC_lexicon_tags_79 },
    { 0xc6c72884, SUC_lexicon_tags_16 },
    { 0x20872886, SUC_lexicon_tags_17 },
    { 0xd9e72887, SUC_lexicon_tags_1 },
    { 0x4fa72884, SUC_lexicon_tags_60 },
    { 0xe2e72889, SUC_lexicon_tags_30 },
    { 0x24e7288a, SUC_lexicon_tags_28 },
    { 0xd5272885, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c72890, SUC_lexicon_tags_23 },
    { 0xc872891, SUC_lexicon_tags_11 },
    { 0xc5872892, SUC_lexicon_tags_15 },
    { 0x7f672893, SUC_lexicon_tags_23 },
    { 0xfde72894, SUC_lexicon_tags_17 },
    { 0x9d672892, SUC_lexicon_tags_15 },
    { 0x6e872896, SUC_lexicon_tags_0 },
    { 0xb8c72893, SUC_lexicon_tags_3 },
    { 0x72872898, SUC_lexicon_tags_49 },
    { 0xa0872899, SUC_lexicon_tags_8 },
    { 0x5b072895, SUC_lexicon_tags_92 },
    { 0x8d67289b, SUC_lexicon_tags_33 },
    { 0xa347289b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c7289f, SUC_lexicon_tags_47 },
    { 0x6f6728a0, SUC_lexicon_tags_15 },
    { 0x426728a1, SUC_lexicon_tags_1 },
    { 0x1e728a2, SUC_lexicon_tags_32 },
    { 0x1a2728a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19e728a5, SUC_lexicon_tags_38 },
    { 0xdf8728a6, SUC_lexicon_tags_23 },
    { 0x4ba728a7, SUC_lexicon_tags_66 },
    { 0x82c728a8, SUC_lexicon_tags_19 },
    { 0xd42728a7, SUC_lexicon_tags_9 },
    { 0xa60728aa, SUC_lexicon_tags_38 },
    { 0xd8e728ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fc728ad, SUC_lexicon_tags_1 },
    { 0x404728ae, SUC_lexicon_tags_14 },
    { 0xa20728af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6728b2, SUC_lexicon_tags_37 },
    { 0x742728b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436728b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736728bf, SUC_lexicon_tags_17 },
    { 0x4f6728c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4728cd, SUC_lexicon_tags_26 },
    { 0x860728cd, SUC_lexicon_tags_1 },
    { 0x55a728cd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa728d2, SUC_lexicon_tags_11 },
    { 0x73c728d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346728db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8728de, SUC_lexicon_tags_15 },
    { 0x340728de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60728e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x222728e6, SUC_lexicon_tags_17 },
    { 0x256728e7, SUC_lexicon_tags_11 },
    { 0x3aa728e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcca728ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x818728ed, SUC_lexicon_tags_11 },
    { 0xa78728ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x972728f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e728f4, SUC_lexicon_tags_47 },
    { 0xe8e728f4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x426728f7, SUC_lexicon_tags_17 },
    { 0xb60728f8, SUC_lexicon_tags_12 },
    { 0x2d6728f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x96a728fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x426728fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe72900, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c72904, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb267290b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e7290f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc72913, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96472916, SUC_lexicon_tags_25 },
    { 0xf5c72916, SUC_lexicon_tags_10 },
    { 0x88472918, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x4947291b, SUC_lexicon_tags_17 },
    { 0x16c7291c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e72920, SUC_lexicon_tags_21 },
    { 0xb3c72921, SUC_lexicon_tags_15 },
    { 0x1c272922, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c72927, SUC_lexicon_tags_2 },
    { 0xd8c72928, SUC_lexicon_tags_15 },
    { 0x85072928, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd327292c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e72930, SUC_lexicon_tags_37 },
    { 0x61e72931, SUC_lexicon_tags_7 },
    { 0xb2872931, SUC_lexicon_tags_14 },
    { 0x30c72930, SUC_lexicon_tags_1 },
    { 0x17872934, SUC_lexicon_tags_1 },
    { 0x27472934, SUC_lexicon_tags_37 },
    { 0xf4672936, SUC_lexicon_tags_68 },
    { 0xf6872936, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ca72939, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e7293c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbba7293e, SUC_lexicon_tags_15 },
    { 0xc5e7293f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x74672942, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xed472945, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b47294a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9947294c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec72951, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b672955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c7295c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d27295f, SUC_lexicon_tags_1 },
    { 0x9fe72960, SUC_lexicon_tags_28 },
    { 0x5347295f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da72964, SUC_lexicon_tags_26 },
    { 0xc9672965, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e7296e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c72972, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa72979, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xae27297c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6b07297e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8272980, SUC_lexicon_tags_25 },
    { 0xcd472981, SUC_lexicon_tags_27 },
    { 0x98072982, SUC_lexicon_tags_12 },
    { 0x8ba72980, SUC_lexicon_tags_25 },
    { 0x8b072981, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc507298b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c72996, SUC_lexicon_tags_1 },
    { 0x3b872997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc127299a, SUC_lexicon_tags_66 },
    { 0xdf47299a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87c7299d, SUC_lexicon_tags_37 },
    { 0x107299e, SUC_lexicon_tags_25 },
    { 0x4f87299f, SUC_lexicon_tags_2 },
    { 0x220729a0, SUC_lexicon_tags_0 },
    { 0x2587299e, SUC_lexicon_tags_28 },
    { 0x7227299e, SUC_lexicon_tags_12 },
    { 0xa72729a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2729a6, SUC_lexicon_tags_37 },
    { 0xd78729a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x240729aa, SUC_lexicon_tags_17 },
    { 0xef8729aa, SUC_lexicon_tags_31 },
    { 0x266729ac, SUC_lexicon_tags_11 },
    { 0xb7e729ad, SUC_lexicon_tags_2 },
    { 0x570729ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4729b3, SUC_lexicon_tags_25 },
    { 0x752729b4, SUC_lexicon_tags_35 },
    { 0x940729b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6c2729b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c729ba, SUC_lexicon_tags_17 },
    { 0x630729bb, SUC_lexicon_tags_26 },
    { 0x182729bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x702729be, SUC_lexicon_tags_59 },
    { 0x436729bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384729c6, SUC_lexicon_tags_25 },
    { 0x942729c6, SUC_lexicon_tags_354 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e729d0, SUC_lexicon_tags_25 },
    { 0x490729d1, SUC_lexicon_tags_69 },
    { 0x38a729d2, SUC_lexicon_tags_525 },
    { 0x36729d3, SUC_lexicon_tags_17 },
    { 0x6ec729d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfa8729d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a729da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0a729dc, SUC_lexicon_tags_1105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52729e0, SUC_lexicon_tags_23 },
    { 0x754729e1, SUC_lexicon_tags_37 },
    { 0x34729e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c729e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b0729ea, SUC_lexicon_tags_25 },
    { 0x5e2729eb, SUC_lexicon_tags_17 },
    { 0x448729ec, SUC_lexicon_tags_37 },
    { 0x9f4729ec, SUC_lexicon_tags_15 },
    { 0xcf8729ee, SUC_lexicon_tags_2 },
    { 0x1c2729ed, SUC_lexicon_tags_37 },
    { 0x3f6729eb, SUC_lexicon_tags_26 },
    { 0x5a729f1, SUC_lexicon_tags_11 },
    { 0x20c729f1, SUC_lexicon_tags_2 },
    { 0xa6a729f3, SUC_lexicon_tags_0 },
    { 0x93a729ea, SUC_lexicon_tags_37 },
    { 0x46c729f5, SUC_lexicon_tags_45 },
    { 0x58c729f5, SUC_lexicon_tags_12 },
    { 0xea2729f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa729fc, SUC_lexicon_tags_11 },
    { 0xc26729fd, SUC_lexicon_tags_37 },
    { 0xc16729fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe72a02, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe3c72a04, SUC_lexicon_tags_11 },
    { 0x472a05, SUC_lexicon_tags_82 },
    { 0x59c72a04, SUC_lexicon_tags_61 },
    { 0x5e272a06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3672a09, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a72a0f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e72a13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74472a18, SUC_lexicon_tags_22 },
    { 0xd5872a18, SUC_lexicon_tags_25 },
    { 0x94072a1a, SUC_lexicon_tags_7 },
    { 0xf4672a1b, SUC_lexicon_tags_25 },
    { 0x90672a1b, SUC_lexicon_tags_17 },
    { 0xbbe72a1d, SUC_lexicon_tags_25 },
    { 0x39a72a18, SUC_lexicon_tags_23 },
    { 0xd2272a1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9a272a21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82072a25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1e72a27, SUC_lexicon_tags_11 },
    { 0xc2072a28, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8672a2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4e72a2f, SUC_lexicon_tags_55 },
    { 0x2b272a2f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c72a33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a872a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34272a39, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14472a42, SUC_lexicon_tags_8 },
    { 0x9e72a43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e72a47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb6c72a49, SUC_lexicon_tags_25 },
    { 0x73472a4a, SUC_lexicon_tags_15 },
    { 0x8e672a49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff672a4f, SUC_lexicon_tags_7 },
    { 0x2de72a50, SUC_lexicon_tags_2 },
    { 0x80672a51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea72a54, SUC_lexicon_tags_0 },
    { 0x51872a55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc072a59, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c72a5c, SUC_lexicon_tags_15 },
    { 0xff272a5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc7072a5f, SUC_lexicon_tags_11 },
    { 0x8fa72a60, SUC_lexicon_tags_12 },
    { 0x1e72a61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6aa72a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a72a6a, SUC_lexicon_tags_2 },
    { 0x69e72a6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c72a72, SUC_lexicon_tags_10 },
    { 0x26072a73, SUC_lexicon_tags_14 },
    { 0xb5e72a73, SUC_lexicon_tags_38 },
    { 0x5c272a72, SUC_lexicon_tags_0 },
    { 0x4d272a76, SUC_lexicon_tags_47 },
    { 0x59872a77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0472a7d, SUC_lexicon_tags_17 },
    { 0x72c72a7e, SUC_lexicon_tags_16 },
    { 0xf5e72a7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5672a81, SUC_lexicon_tags_27 },
    { 0xdf672a81, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb5c72a84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83272a87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3472a89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e72a90, SUC_lexicon_tags_1 },
    { 0xc4672a91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f272a9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e72aa2, SUC_lexicon_tags_38 },
    { 0xf0e72aa3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a72aa8, SUC_lexicon_tags_8 },
    { 0x68c72aa9, SUC_lexicon_tags_61 },
    { 0x58c72aa8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe072aae, SUC_lexicon_tags_17 },
    { 0xac872aaf, SUC_lexicon_tags_2 },
    { 0x1ce72aae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4072ab3, SUC_lexicon_tags_273 },
    { 0x20e72ab4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f872aba, SUC_lexicon_tags_11 },
    { 0x33272abb, SUC_lexicon_tags_2 },
    { 0xf4a72abc, SUC_lexicon_tags_31 },
    { 0xa3072abc, SUC_lexicon_tags_1 },
    { 0x26272abc, SUC_lexicon_tags_35 },
    { 0xeda72abf, SUC_lexicon_tags_33 },
    { 0xac672abf, SUC_lexicon_tags_49 },
    { 0x9a472ac1, SUC_lexicon_tags_16 },
    { 0xe4672ac1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a72ace, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42872ad2, SUC_lexicon_tags_28 },
    { 0xb2a72ad2, SUC_lexicon_tags_14 },
    { 0x5b872ad3, SUC_lexicon_tags_27 },
    { 0xc8872ad5, SUC_lexicon_tags_32 },
    { 0xa2872ad5, SUC_lexicon_tags_29 },
    { 0x15872ad7, SUC_lexicon_tags_9 },
    { 0xa7872ad8, SUC_lexicon_tags_1 },
    { 0x63472ad6, SUC_lexicon_tags_118 },
    { 0xb672ada, SUC_lexicon_tags_12 },
    { 0xc4e72adb, SUC_lexicon_tags_7 },
    { 0x30072ad4, SUC_lexicon_tags_38 },
    { 0x98672add, SUC_lexicon_tags_7 },
    { 0x63672adc, SUC_lexicon_tags_15 },
    { 0xa9272adf, SUC_lexicon_tags_14 },
    { 0x3a672ae0, SUC_lexicon_tags_17 },
    { 0x3ce72adf, SUC_lexicon_tags_15 },
    { 0x3aa72ae2, SUC_lexicon_tags_17 },
    { 0x79472ae3, SUC_lexicon_tags_14 },
    { 0x16e72ae4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad072ae8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36872af4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e72af7, SUC_lexicon_tags_11 },
    { 0xa5e72af7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e72afb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcc272afd, SUC_lexicon_tags_15 },
    { 0x25c72afd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a72b07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e72b0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40672b0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18272b0f, SUC_lexicon_tags_17 },
    { 0x8e872b10, SUC_lexicon_tags_11 },
    { 0x34472b11, SUC_lexicon_tags_4 },
    { 0x5d072b10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d672b15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84072b1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c72b1d, SUC_lexicon_tags_19 },
    { 0xd3e72b1e, SUC_lexicon_tags_25 },
    { 0x16072b1e, SUC_lexicon_tags_37 },
    { 0x43672b20, SUC_lexicon_tags_1 },
    { 0x2d672b21, SUC_lexicon_tags_4 },
    { 0x37a72b20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93c72b24, SUC_lexicon_tags_7 },
    { 0xd8272b24, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da72b29, SUC_lexicon_tags_71 },
    { 0xea472b2a, SUC_lexicon_tags_198 },
    { 0xea672b29, SUC_lexicon_tags_82 },
    { 0x10872b2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b072b2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa072b31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae672b34, SUC_lexicon_tags_34 },
    { 0xbda72b35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdea72b37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x82872b39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7672b3b, SUC_lexicon_tags_37 },
    { 0x10872b3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8272b40, SUC_lexicon_tags_25 },
    { 0xb9072b41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c472b46, SUC_lexicon_tags_103 },
    { 0xd1472b47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9d272b49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1c72b4b, SUC_lexicon_tags_12 },
    { 0xcc472b4c, SUC_lexicon_tags_8 },
    { 0x47e72b4d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xab472b50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xda072b52, SUC_lexicon_tags_12 },
    { 0x10672b52, SUC_lexicon_tags_23 },
    { 0xd4672b54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd472b56, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0xace72b58, SUC_lexicon_tags_17 },
    { 0x2c672b59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x65272b5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1672b5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5b472b5f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f272b62, SUC_lexicon_tags_4 },
    { 0x2d072b63, SUC_lexicon_tags_7 },
    { 0x13472b62, SUC_lexicon_tags_43 },
    { 0x7d672b64, SUC_lexicon_tags_66 },
    { 0x4ec72b62, SUC_lexicon_tags_14 },
    { 0x29a72b67, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc6872b69, SUC_lexicon_tags_28 },
    { 0x4ce72b69, SUC_lexicon_tags_19 },
    { 0xfc272b6b, SUC_lexicon_tags_19 },
    { 0x1b072b6c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x92272b6e, SUC_lexicon_tags_11 },
    { 0xe0c72b6e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae72b74, SUC_lexicon_tags_11 },
    { 0x45672b74, SUC_lexicon_tags_17 },
    { 0x86272b76, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x1a472b78, SUC_lexicon_tags_2 },
    { 0xb6272b79, SUC_lexicon_tags_37 },
    { 0xc3a72b79, SUC_lexicon_tags_45 },
    { 0x1c272b7b, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc72b80, SUC_lexicon_tags_256 },
    { 0x61472b80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81672b84, SUC_lexicon_tags_49 },
    { 0xdee72b85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66872b89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2da72b8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79272b8d, SUC_lexicon_tags_336 },
    { 0xb1272b8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be72b93, SUC_lexicon_tags_4 },
    { 0xc1272b94, SUC_lexicon_tags_60 },
    { 0x61e72b93, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31672ba4, SUC_lexicon_tags_0 },
    { 0x6a272ba5, SUC_lexicon_tags_25 },
    { 0x44e72ba5, SUC_lexicon_tags_7 },
    { 0x74c72ba4, SUC_lexicon_tags_15 },
    { 0x9d472ba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf072baa, SUC_lexicon_tags_21 },
    { 0x40072bab, SUC_lexicon_tags_37 },
    { 0x3d272baa, SUC_lexicon_tags_17 },
    { 0x30672bad, SUC_lexicon_tags_10 },
    { 0x7f472baa, SUC_lexicon_tags_341 },
    { 0x8ca72bac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f872bb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c72bb4, SUC_lexicon_tags_35 },
    { 0x14272bb5, SUC_lexicon_tags_15 },
    { 0x4f472bb5, SUC_lexicon_tags_30 },
    { 0x55c72bb6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7272bbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d872bbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a672bc1, SUC_lexicon_tags_8 },
    { 0xa1a72bc2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0472bc5, SUC_lexicon_tags_17 },
    { 0x78c72bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c72bcf, SUC_lexicon_tags_33 },
    { 0xe7072bd0, SUC_lexicon_tags_37 },
    { 0x4da72bd1, SUC_lexicon_tags_32 },
    { 0x6c72bd2, SUC_lexicon_tags_7 },
    { 0xc0872bd2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd1e72bd5, SUC_lexicon_tags_48 },
    { 0x1d672bd6, SUC_lexicon_tags_68 },
    { 0x3a072bd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ea72bd9, SUC_lexicon_tags_68 },
    { 0x81872bda, SUC_lexicon_tags_25 },
    { 0x9b072bdb, SUC_lexicon_tags_25 },
    { 0xcb272bdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a72be3, SUC_lexicon_tags_37 },
    { 0x2c272be4, SUC_lexicon_tags_32 },
    { 0x28c72be3, SUC_lexicon_tags_25 },
    { 0xcbc72be6, SUC_lexicon_tags_37 },
    { 0x52072be7, SUC_lexicon_tags_11 },
    { 0xd4c72be8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a72beb, SUC_lexicon_tags_17 },
    { 0x3dc72bec, SUC_lexicon_tags_12 },
    { 0x9872bed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e72bf1, SUC_lexicon_tags_60 },
    { 0xd4672bf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36672bf7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e272bfa, SUC_lexicon_tags_14 },
    { 0x74872bfa, SUC_lexicon_tags_95 },
    { 0x2672bfc, SUC_lexicon_tags_10 },
    { 0x63472bfc, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x6b472bff, SUC_lexicon_tags_2 },
    { 0x96e72c00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x82a72c02, SUC_lexicon_tags_19 },
    { 0xf6272c03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85072c0d, SUC_lexicon_tags_9 },
    { 0x1a672c0e, SUC_lexicon_tags_25 },
    { 0xc0872c0e, SUC_lexicon_tags_7 },
    { 0xbfc72c0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee472c12, SUC_lexicon_tags_12 },
    { 0xfea72c13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc872c17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e872c1a, SUC_lexicon_tags_17 },
    { 0x93072c1b, SUC_lexicon_tags_30 },
    { 0xe5e72c1a, SUC_lexicon_tags_11 },
    { 0x3c072c1d, SUC_lexicon_tags_43 },
    { 0x68872c1e, SUC_lexicon_tags_12 },
    { 0x36272c1d, SUC_lexicon_tags_45 },
    { 0x46272c1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72272c28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15472c2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cc72c2c, SUC_lexicon_tags_9 },
    { 0x26072c2d, SUC_lexicon_tags_49 },
    { 0x1e872c2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x43072c30, SUC_lexicon_tags_15 },
    { 0x2e72c31, SUC_lexicon_tags_45 },
    { 0xe2672c32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61072c38, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x92872c3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1472c3e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbdc72c40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c72c49, SUC_lexicon_tags_1 },
    { 0x7fc72c4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60272c4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc72c51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c72c54, SUC_lexicon_tags_7 },
    { 0xa2c72c55, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d672c5e, SUC_lexicon_tags_27 },
    { 0xcd272c5f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x68872c61, SUC_lexicon_tags_15 },
    { 0xbb672c61, SUC_lexicon_tags_27 },
    { 0x4e672c62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2272c67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9de72c69, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc4a72c6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a72c77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x79c72c79, SUC_lexicon_tags_27 },
    { 0x4c472c7a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xacc72c7c, SUC_lexicon_tags_27 },
    { 0x3de72c7d, SUC_lexicon_tags_65 },
    { 0x672c7c, SUC_lexicon_tags_48 },
    { 0xa1e72c7e, SUC_lexicon_tags_87 },
    { 0x7872c80, SUC_lexicon_tags_60 },
    { 0x9b472c7f, SUC_lexicon_tags_15 },
    { 0xeec72c81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc72c86, SUC_lexicon_tags_2 },
    { 0xd5472c87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc7472c89, SUC_lexicon_tags_15 },
    { 0xb5872c8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe672c8d, SUC_lexicon_tags_37 },
    { 0x5c472c8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca72c94, SUC_lexicon_tags_23 },
    { 0x7ce72c95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa272c99, SUC_lexicon_tags_25 },
    { 0x8d872c9a, SUC_lexicon_tags_33 },
    { 0x52872c9b, SUC_lexicon_tags_18 },
    { 0x48c72c9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c72ca2, SUC_lexicon_tags_19 },
    { 0x5c072ca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6472ca6, SUC_lexicon_tags_11 },
    { 0x40472ca6, SUC_lexicon_tags_7 },
    { 0x37872ca8, SUC_lexicon_tags_11 },
    { 0x77072ca8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72cac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16a72cae, SUC_lexicon_tags_14 },
    { 0xf5472cae, SUC_lexicon_tags_11 },
    { 0x87872cae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec72cb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe272cbb, SUC_lexicon_tags_11 },
    { 0xa5672cbb, SUC_lexicon_tags_2 },
    { 0x7472cbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99672cbf, SUC_lexicon_tags_25 },
    { 0x75c72cc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce72cc5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3072cc9, SUC_lexicon_tags_0 },
    { 0xe8a72cca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xde872ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1472ccf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e272cd2, SUC_lexicon_tags_17 },
    { 0xa7a72cd3, SUC_lexicon_tags_15 },
    { 0x63672cd3, SUC_lexicon_tags_7 },
    { 0x1072cd3, SUC_lexicon_tags_31 },
    { 0x2872cd4, SUC_lexicon_tags_28 },
    { 0x98472cd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2872cde, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbae72ce0, SUC_lexicon_tags_85 },
    { 0xce072ce1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e72ce9, SUC_lexicon_tags_45 },
    { 0xc7a72ce9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f072cee, SUC_lexicon_tags_60 },
    { 0xbd072cef, SUC_lexicon_tags_47 },
    { 0x4ca72cef, SUC_lexicon_tags_37 },
    { 0x6ac72cf0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87872cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4872cf9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a72cfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4672d03, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94072d0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x28272d0d, SUC_lexicon_tags_29 },
    { 0x75872d0e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41072d15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e72d1a, SUC_lexicon_tags_37 },
    { 0x51a72d1b, SUC_lexicon_tags_30 },
    { 0xc3c72d1c, SUC_lexicon_tags_1 },
    { 0x69e72d1d, SUC_lexicon_tags_38 },
    { 0x47672d1d, SUC_lexicon_tags_37 },
    { 0xca672d1a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3672d24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6d872d26, SUC_lexicon_tags_31 },
    { 0x22e72d27, SUC_lexicon_tags_7 },
    { 0x39a72d28, SUC_lexicon_tags_2 },
    { 0x29672d26, SUC_lexicon_tags_68 },
    { 0x29c72d2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76072d2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1872d31, SUC_lexicon_tags_16 },
    { 0xf3672d32, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9072d36, SUC_lexicon_tags_17 },
    { 0xbfa72d37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd472d39, SUC_lexicon_tags_30 },
    { 0x70c72d3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1872d3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd072d40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34272d44, SUC_lexicon_tags_6 },
    { 0xa7272d45, SUC_lexicon_tags_23 },
    { 0x32e72d46, SUC_lexicon_tags_1 },
    { 0xf7c72d45, SUC_lexicon_tags_37 },
    { 0xe9672d48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83872d4a, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xea272d4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8872d52, SUC_lexicon_tags_23 },
    { 0xbc872d52, SUC_lexicon_tags_39 },
    { 0x67872d52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9072d5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5472d5c, SUC_lexicon_tags_15 },
    { 0xa1e72d5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x47672d5f, SUC_lexicon_tags_17 },
    { 0x91c72d5f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdba72d62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f472d64, SUC_lexicon_tags_25 },
    { 0x5aa72d64, SUC_lexicon_tags_37 },
    { 0x9d872d66, SUC_lexicon_tags_37 },
    { 0x7cc72d65, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26c72d69, SUC_lexicon_tags_17 },
    { 0x72872d69, SUC_lexicon_tags_39 },
    { 0x6ee72d6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d672d6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27272d74, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5272d76, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a72d7a, SUC_lexicon_tags_25 },
    { 0x8c072d7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9672d7e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee72d81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90272d83, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca072d87, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6672d8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6872d95, SUC_lexicon_tags_63 },
    { 0xe3e72d95, SUC_lexicon_tags_17 },
    { 0xbbe72d96, SUC_lexicon_tags_1 },
    { 0xf4672d98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa72d9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c72da2, SUC_lexicon_tags_47 },
    { 0x97672da2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce072da7, SUC_lexicon_tags_14 },
    { 0xcee72da8, SUC_lexicon_tags_62 },
    { 0xd7872da8, SUC_lexicon_tags_27 },
    { 0x2d072da9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5872dad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a72db2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6272db5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1472db9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa5a72dbb, SUC_lexicon_tags_6 },
    { 0xfba72dbc, SUC_lexicon_tags_6 },
    { 0xaa72dbc, SUC_lexicon_tags_25 },
    { 0xba272dbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c72dc1, SUC_lexicon_tags_7 },
    { 0x6e872dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a472dc6, SUC_lexicon_tags_15 },
    { 0x8c272dc7, SUC_lexicon_tags_1 },
    { 0x50472dc6, SUC_lexicon_tags_25 },
    { 0x3872dc6, SUC_lexicon_tags_19 },
    { 0x34e72dca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf4a72dcc, SUC_lexicon_tags_6 },
    { 0x5a272dcd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x91872dcf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x18e72dd1, SUC_lexicon_tags_21 },
    { 0x65672dd2, SUC_lexicon_tags_17 },
    { 0x72272dd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8472dd7, SUC_lexicon_tags_19 },
    { 0xe5a72dd7, SUC_lexicon_tags_17 },
    { 0x73c72dd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda072ddc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a272dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a672de0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e72de3, SUC_lexicon_tags_37 },
    { 0x1fa72de3, SUC_lexicon_tags_14 },
    { 0xcee72de4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb072de7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9472deb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80872df2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66672df4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e472df9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf4e72dfb, SUC_lexicon_tags_14 },
    { 0xf9472dfc, SUC_lexicon_tags_11 },
    { 0x1272dfd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe72dff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60672e01, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1672e03, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa72e0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1272e0f, SUC_lexicon_tags_17 },
    { 0x77e72e10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3b672e12, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f272e16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba472e18, SUC_lexicon_tags_29 },
    { 0x64272e19, SUC_lexicon_tags_25 },
    { 0x84c72e1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8672e1f, SUC_lexicon_tags_25 },
    { 0x4cc72e20, SUC_lexicon_tags_9 },
    { 0x61c72e21, SUC_lexicon_tags_53 },
    { 0x83e72e1f, SUC_lexicon_tags_19 },
    { 0x1b672e23, SUC_lexicon_tags_15 },
    { 0xdf672e24, SUC_lexicon_tags_47 },
    { 0x86e72e23, SUC_lexicon_tags_32 },
    { 0x92072e26, SUC_lexicon_tags_87 },
    { 0xddc72e26, SUC_lexicon_tags_17 },
    { 0x5b472e28, SUC_lexicon_tags_25 },
    { 0x9f872e28, SUC_lexicon_tags_1 },
    { 0x93672e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c72e2d, SUC_lexicon_tags_37 },
    { 0xebe72e2e, SUC_lexicon_tags_11 },
    { 0xb1272e2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8072e31, SUC_lexicon_tags_14 },
    { 0xc0072e32, SUC_lexicon_tags_2 },
    { 0xfea72e33, SUC_lexicon_tags_77 },
    { 0x47672e34, SUC_lexicon_tags_17 },
    { 0x68872e35, SUC_lexicon_tags_68 },
    { 0x86072e36, SUC_lexicon_tags_17 },
    { 0xf2872e37, SUC_lexicon_tags_31 },
    { 0x7d272e38, SUC_lexicon_tags_47 },
    { 0x57e72e34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa072e3b, SUC_lexicon_tags_63 },
    { 0x6a472e3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33272e3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e72e41, SUC_lexicon_tags_7 },
    { 0xd3472e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d072e45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29672e4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec72e51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c72e56, SUC_lexicon_tags_8 },
    { 0xe0c72e56, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3672e5c, SUC_lexicon_tags_14 },
    { 0x1bc72e5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c72e60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdf072e62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50672e64, SUC_lexicon_tags_35 },
    { 0xce472e65, SUC_lexicon_tags_7 },
    { 0xd272e64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e72e6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44c72e6e, SUC_lexicon_tags_7 },
    { 0x39672e6f, SUC_lexicon_tags_17 },
    { 0xb7472e6f, SUC_lexicon_tags_10 },
    { 0xada72e71, SUC_lexicon_tags_7 },
    { 0x67e72e6f, SUC_lexicon_tags_47 },
    { 0x66e72e73, SUC_lexicon_tags_59 },
    { 0xac872e73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b472e78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18472e7a, SUC_lexicon_tags_17 },
    { 0x8c072e7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a72e7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9072e82, SUC_lexicon_tags_37 },
    { 0xc2072e82, SUC_lexicon_tags_15 },
    { 0xee072e84, SUC_lexicon_tags_39 },
    { 0x14e72e83, SUC_lexicon_tags_7 },
    { 0x41272e83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8072e89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49a72e8b, SUC_lexicon_tags_23 },
    { 0x76672e8c, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a72e8f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa72e94, SUC_lexicon_tags_31 },
    { 0x7d672e95, SUC_lexicon_tags_191 },
    { 0xef472e96, SUC_lexicon_tags_47 },
    { 0xf3a72e97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a72ea0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a72ea3, SUC_lexicon_tags_49 },
    { 0xbda72ea3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12072ea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33872eaa, SUC_lexicon_tags_1 },
    { 0x5fe72eab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x93472ead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c472eaf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e272eb6, SUC_lexicon_tags_15 },
    { 0x32072eb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96872eb9, SUC_lexicon_tags_21 },
    { 0xcb472eb9, SUC_lexicon_tags_7 },
    { 0xa3a72eba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3272ebe, SUC_lexicon_tags_1 },
    { 0x26a72ebe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce72ec5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbc72ec7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe272ecb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9672ecf, SUC_lexicon_tags_12 },
    { 0x50672ed0, SUC_lexicon_tags_11 },
    { 0x30672ed1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58a72ed3, SUC_lexicon_tags_25 },
    { 0xc2a72ed4, SUC_lexicon_tags_17 },
    { 0xbb672ed3, SUC_lexicon_tags_2 },
    { 0xb5472ed6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe472edd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43472ee4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c72ee7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e872eea, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48272eef, SUC_lexicon_tags_10 },
    { 0x26072ef0, SUC_lexicon_tags_15 },
    { 0x49a72ef0, SUC_lexicon_tags_23 },
    { 0x8d872ef0, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272ef8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7672eff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c72f05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe072f0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7d072f0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4272f0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe472f11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59272f13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc72f16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c72f1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba72f23, SUC_lexicon_tags_14 },
    { 0xabe72f23, SUC_lexicon_tags_0 },
    { 0xc8072f25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be72f29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5072f2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a72f30, SUC_lexicon_tags_23 },
    { 0x76272f30, SUC_lexicon_tags_2 },
    { 0xb7272f31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ea72f34, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9e672f36, SUC_lexicon_tags_10 },
    { 0xa6c72f37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e72f3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x68672f3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce72f42, SUC_lexicon_tags_30 },
    { 0x2a872f42, SUC_lexicon_tags_15 },
    { 0x57872f44, SUC_lexicon_tags_11 },
    { 0x6f672f45, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x50872f48, SUC_lexicon_tags_17 },
    { 0x6a472f49, SUC_lexicon_tags_19 },
    { 0x96a72f49, SUC_lexicon_tags_7 },
    { 0x9da72f49, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xeea72f4d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ee72f4f, SUC_lexicon_tags_9 },
    { 0x87672f4f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe872f55, SUC_lexicon_tags_49 },
    { 0xb6e72f56, SUC_lexicon_tags_31 },
    { 0x82a72f57, SUC_lexicon_tags_12 },
    { 0xbae72f55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81472f5c, SUC_lexicon_tags_15 },
    { 0xa72f5d, SUC_lexicon_tags_37 },
    { 0xcd872f5c, SUC_lexicon_tags_28 },
    { 0x75872f5f, SUC_lexicon_tags_27 },
    { 0x35872f60, SUC_lexicon_tags_47 },
    { 0x9a272f61, SUC_lexicon_tags_2 },
    { 0xece72f5e, SUC_lexicon_tags_2 },
    { 0xc9672f63, SUC_lexicon_tags_48 },
    { 0xc0072f5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf0472f66, SUC_lexicon_tags_31 },
    { 0xbf072f66, SUC_lexicon_tags_6 },
    { 0x3a272f68, SUC_lexicon_tags_60 },
    { 0x53a72f68, SUC_lexicon_tags_17 },
    { 0xdb472f6a, SUC_lexicon_tags_4 },
    { 0xdb272f66, SUC_lexicon_tags_19 },
    { 0x2ec72f69, SUC_lexicon_tags_0 },
    { 0x19672f6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbee72f6f, SUC_lexicon_tags_17 },
    { 0xb0e72f70, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x15072f72, SUC_lexicon_tags_43 },
    { 0xdac72f73, SUC_lexicon_tags_12 },
    { 0xb5272f73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e072f77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80672f79, SUC_lexicon_tags_1 },
    { 0xacc72f7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5072f88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d272f8a, SUC_lexicon_tags_11 },
    { 0x88a72f8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4e72f8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e72f91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1a472f93, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75072f9a, SUC_lexicon_tags_63 },
    { 0x3dc72f9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69672fa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88672fa4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x58472fa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a72fac, SUC_lexicon_tags_47 },
    { 0x52472fad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48872fb1, SUC_lexicon_tags_6 },
    { 0xbb472fb2, SUC_lexicon_tags_42 },
    { 0x61072fb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf872fb6, SUC_lexicon_tags_37 },
    { 0x12472fb7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x49672fb9, SUC_lexicon_tags_16 },
    { 0x30c72fba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23872fbd, SUC_lexicon_tags_11 },
    { 0xde872fbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99272fc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x43472fc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb272fce, SUC_lexicon_tags_23 },
    { 0xaf472fcf, SUC_lexicon_tags_17 },
    { 0x68472fcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c72fd5, SUC_lexicon_tags_460 },
    { 0xdae72fd6, SUC_lexicon_tags_23 },
    { 0x77272fd6, SUC_lexicon_tags_17 },
    { 0xd2a72fd8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a72fe2, SUC_lexicon_tags_12 },
    { 0x17c72fe2, SUC_lexicon_tags_64 },
    { 0x1e872fe4, SUC_lexicon_tags_15 },
    { 0xdca72fe5, SUC_lexicon_tags_25 },
    { 0x60672fe6, SUC_lexicon_tags_17 },
    { 0x10872fe5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83472feb, SUC_lexicon_tags_2 },
    { 0x5f472feb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e72ff0, SUC_lexicon_tags_65 },
    { 0x29e72ff1, SUC_lexicon_tags_37 },
    { 0x38072ff2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc2e72ff4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e872ff9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2672ffb, SUC_lexicon_tags_0 },
    { 0xea272ffc, SUC_lexicon_tags_45 },
    { 0xdf672ffd, SUC_lexicon_tags_17 },
    { 0xf6272ffe, SUC_lexicon_tags_7 },
    { 0x77a72fff, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33473003, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc273005, SUC_lexicon_tags_25 },
    { 0x4a673006, SUC_lexicon_tags_11 },
    { 0xfbe73005, SUC_lexicon_tags_49 },
    { 0xb6a73006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e7300f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e73012, SUC_lexicon_tags_27 },
    { 0xc2c73012, SUC_lexicon_tags_60 },
    { 0xe7873013, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e73019, SUC_lexicon_tags_32 },
    { 0x7d47301a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c873022, SUC_lexicon_tags_15 },
    { 0x34073022, SUC_lexicon_tags_25 },
    { 0x3e873024, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5473027, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c67302b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9427302d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee73031, SUC_lexicon_tags_7 },
    { 0xb873032, SUC_lexicon_tags_17 },
    { 0x49a73032, SUC_lexicon_tags_37 },
    { 0x39e73031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c73036, SUC_lexicon_tags_26 },
    { 0x68673036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e67303a, SUC_lexicon_tags_32 },
    { 0xc1e7303b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b67303f, SUC_lexicon_tags_9 },
    { 0xd8c73040, SUC_lexicon_tags_37 },
    { 0x66673041, SUC_lexicon_tags_14 },
    { 0xd073042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32c73044, SUC_lexicon_tags_12 },
    { 0x9b673045, SUC_lexicon_tags_12 },
    { 0xa5c73046, SUC_lexicon_tags_17 },
    { 0x97073047, SUC_lexicon_tags_25 },
    { 0x58873048, SUC_lexicon_tags_45 },
    { 0xcba73047, SUC_lexicon_tags_15 },
    { 0xd5e7304a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a7304e, SUC_lexicon_tags_27 },
    { 0x3cc7304e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47c73051, SUC_lexicon_tags_23 },
    { 0xa7073051, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2873054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e73059, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be7305d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c73060, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e673062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec73067, SUC_lexicon_tags_37 },
    { 0xebe73068, SUC_lexicon_tags_25 },
    { 0x95c73069, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9073072, SUC_lexicon_tags_27 },
    { 0xcd873072, SUC_lexicon_tags_2 },
    { 0xea873074, SUC_lexicon_tags_17 },
    { 0x51473074, SUC_lexicon_tags_25 },
    { 0xbf073076, SUC_lexicon_tags_26 },
    { 0xace73072, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707307f, SUC_lexicon_tags_7 },
    { 0x5f073080, SUC_lexicon_tags_33 },
    { 0x57e73081, SUC_lexicon_tags_7 },
    { 0x44873082, SUC_lexicon_tags_27 },
    { 0x8c873082, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b473085, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de7308a, SUC_lexicon_tags_15 },
    { 0x85a7308b, SUC_lexicon_tags_74 },
    { 0xe47308c, SUC_lexicon_tags_2 },
    { 0xba47308b, SUC_lexicon_tags_87 },
    { 0x49a7308d, SUC_lexicon_tags_26 },
    { 0x2fa7308b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5473092, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc873095, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9873099, SUC_lexicon_tags_45 },
    { 0x347309a, SUC_lexicon_tags_1 },
    { 0x7ae7309a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37e7309e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40730a9, SUC_lexicon_tags_25 },
    { 0x230730a9, SUC_lexicon_tags_37 },
    { 0x718730ab, SUC_lexicon_tags_12 },
    { 0x2e2730ac, SUC_lexicon_tags_37 },
    { 0xb96730ad, SUC_lexicon_tags_11 },
    { 0x30e730a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8c730b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8730bc, SUC_lexicon_tags_33 },
    { 0x77c730bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7a730bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa6730c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a730c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x776730c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa7e730ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26730cd, SUC_lexicon_tags_25 },
    { 0x1a2730cd, SUC_lexicon_tags_2 },
    { 0x58c730cf, SUC_lexicon_tags_11 },
    { 0xd0a730cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e730d7, SUC_lexicon_tags_23 },
    { 0x2ec730d7, SUC_lexicon_tags_25 },
    { 0x560730d7, SUC_lexicon_tags_84 },
    { 0xc76730da, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442730df, SUC_lexicon_tags_57 },
    { 0xb4e730e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94730e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x692730e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a730ec, SUC_lexicon_tags_12 },
    { 0x14e730ed, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xbcc730ef, SUC_lexicon_tags_25 },
    { 0x5b8730f0, SUC_lexicon_tags_63 },
    { 0x478730f1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0730f5, SUC_lexicon_tags_30 },
    { 0x8d2730f5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x296730f8, SUC_lexicon_tags_4 },
    { 0x4ae730f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fe730fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50073102, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93073104, SUC_lexicon_tags_37 },
    { 0x97073104, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e073109, SUC_lexicon_tags_82 },
    { 0xee07310a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5447310d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x2a7310f, SUC_lexicon_tags_14 },
    { 0x12873110, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15473115, SUC_lexicon_tags_46 },
    { 0x89e73116, SUC_lexicon_tags_39 },
    { 0x47473115, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27311c, SUC_lexicon_tags_14 },
    { 0xc467311d, SUC_lexicon_tags_55 },
    { 0x7067311c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8873121, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23073128, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6027312a, SUC_lexicon_tags_9 },
    { 0x2d27312b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef87312f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46473131, SUC_lexicon_tags_47 },
    { 0x3ea73131, SUC_lexicon_tags_11 },
    { 0x1073131, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca7313a, SUC_lexicon_tags_62 },
    { 0x40a7313b, SUC_lexicon_tags_45 },
    { 0xb1a7313c, SUC_lexicon_tags_10 },
    { 0x6747313d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50273141, SUC_lexicon_tags_39 },
    { 0xcdc73141, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e873144, SUC_lexicon_tags_114 },
    { 0xdaa73145, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8e73147, SUC_lexicon_tags_11 },
    { 0xcec73148, SUC_lexicon_tags_7 },
    { 0xc73149, SUC_lexicon_tags_47 },
    { 0xa3e7314a, SUC_lexicon_tags_63 },
    { 0xa873147, SUC_lexicon_tags_47 },
    { 0x4827314c, SUC_lexicon_tags_34 },
    { 0xc927314c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6673152, SUC_lexicon_tags_9 },
    { 0xa6e73153, SUC_lexicon_tags_17 },
    { 0x81473153, SUC_lexicon_tags_27 },
    { 0x80073155, SUC_lexicon_tags_37 },
    { 0x83e73154, SUC_lexicon_tags_12 },
    { 0xc6a73157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce873159, SUC_lexicon_tags_7 },
    { 0xb427315a, SUC_lexicon_tags_12 },
    { 0x1ec7315a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c7315e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae73168, SUC_lexicon_tags_7 },
    { 0x76a73169, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a73170, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c73174, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c73179, SUC_lexicon_tags_9 },
    { 0x7c7317a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a7317e, SUC_lexicon_tags_4 },
    { 0x29a7317e, SUC_lexicon_tags_6 },
    { 0x2dc7317e, SUC_lexicon_tags_17 },
    { 0x40073181, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3673184, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba73187, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e27318b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xede7318d, SUC_lexicon_tags_13 },
    { 0x6ee7318e, SUC_lexicon_tags_0 },
    { 0x8b67318e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50073191, SUC_lexicon_tags_10 },
    { 0x4f873192, SUC_lexicon_tags_11 },
    { 0x3e73193, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x46473195, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a73199, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3887319c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e7319f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45c731a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8731a7, SUC_lexicon_tags_19 },
    { 0x6f0731a8, SUC_lexicon_tags_25 },
    { 0x6fa731a7, SUC_lexicon_tags_12 },
    { 0x4f4731aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84731ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfba731af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc731b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x718731b4, SUC_lexicon_tags_7 },
    { 0xce6731b4, SUC_lexicon_tags_25 },
    { 0xf42731b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40731b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0731bd, SUC_lexicon_tags_49 },
    { 0xb54731be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714731c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x80e731c5, SUC_lexicon_tags_44 },
    { 0x626731c6, SUC_lexicon_tags_47 },
    { 0x7d8731c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x584731c9, SUC_lexicon_tags_33 },
    { 0x1b8731ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe731ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a731d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc28731d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e731d8, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2731dc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xab8731de, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0731e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8731eb, SUC_lexicon_tags_10 },
    { 0xcbe731ec, SUC_lexicon_tags_37 },
    { 0x8e4731ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb32731ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x840731f1, SUC_lexicon_tags_1 },
    { 0xf3e731f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54731fb, SUC_lexicon_tags_0 },
    { 0x2d4731fc, SUC_lexicon_tags_7 },
    { 0xda6731fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xefe731ff, SUC_lexicon_tags_28 },
    { 0xf40731ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53073203, SUC_lexicon_tags_61 },
    { 0xffc73204, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d273207, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30873215, SUC_lexicon_tags_17 },
    { 0xd8673215, SUC_lexicon_tags_22 },
    { 0xb7273216, SUC_lexicon_tags_25 },
    { 0x82073218, SUC_lexicon_tags_26 },
    { 0x93473218, SUC_lexicon_tags_6 },
    { 0x287321a, SUC_lexicon_tags_38 },
    { 0x5087321a, SUC_lexicon_tags_27 },
    { 0x3a473216, SUC_lexicon_tags_45 },
    { 0x1cc7321d, SUC_lexicon_tags_184 },
    { 0x6d47321e, SUC_lexicon_tags_25 },
    { 0xf987321f, SUC_lexicon_tags_11 },
    { 0xdcc7321f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92673227, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27322a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff873230, SUC_lexicon_tags_170 },
    { 0x1473231, SUC_lexicon_tags_77 },
    { 0x2da73230, SUC_lexicon_tags_12 },
    { 0x53673233, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xde473236, SUC_lexicon_tags_2 },
    { 0x78c73237, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89673239, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc73241, SUC_lexicon_tags_12 },
    { 0x60273241, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65873245, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e673248, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27324b, SUC_lexicon_tags_45 },
    { 0x3327324c, SUC_lexicon_tags_25 },
    { 0x5fa7324c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4273250, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e73256, SUC_lexicon_tags_12 },
    { 0xfca73257, SUC_lexicon_tags_2 },
    { 0x2ba73257, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa7325d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe9e7325f, SUC_lexicon_tags_27 },
    { 0x81673260, SUC_lexicon_tags_12 },
    { 0xf2273260, SUC_lexicon_tags_17 },
    { 0xb4073262, SUC_lexicon_tags_65 },
    { 0xe2c73263, SUC_lexicon_tags_1 },
    { 0xda73263, SUC_lexicon_tags_33 },
    { 0x59c73265, SUC_lexicon_tags_7 },
    { 0x4b073263, SUC_lexicon_tags_27 },
    { 0x77c7325f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8987326c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c73272, SUC_lexicon_tags_27 },
    { 0xd7673273, SUC_lexicon_tags_25 },
    { 0x70673274, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8673277, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c7327d, SUC_lexicon_tags_4 },
    { 0xd307327e, SUC_lexicon_tags_14 },
    { 0x1727327e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5473283, SUC_lexicon_tags_14 },
    { 0xdf273284, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x89873286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce73289, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f27328b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x60a7328d, SUC_lexicon_tags_37 },
    { 0xa427328d, SUC_lexicon_tags_11 },
    { 0xfa27328d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1873292, SUC_lexicon_tags_39 },
    { 0xcb873292, SUC_lexicon_tags_1293 },
    { 0xe1473294, SUC_lexicon_tags_2 },
    { 0x13873295, SUC_lexicon_tags_9 },
    { 0xaae73292, SUC_lexicon_tags_0 },
    { 0x31c73297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa7329a, SUC_lexicon_tags_21 },
    { 0x53c7329b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4732a3, SUC_lexicon_tags_74 },
    { 0xf04732a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac732a7, SUC_lexicon_tags_26 },
    { 0x324732a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e0732aa, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x432732ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2732b0, SUC_lexicon_tags_25 },
    { 0xb74732b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc732bb, SUC_lexicon_tags_12 },
    { 0xebc732bc, SUC_lexicon_tags_152 },
    { 0xba6732bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x392732bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4f2732c1, SUC_lexicon_tags_23 },
    { 0x9c2732c1, SUC_lexicon_tags_9 },
    { 0xe78732c3, SUC_lexicon_tags_892 },
    { 0x590732c4, SUC_lexicon_tags_15 },
    { 0x864732c4, SUC_lexicon_tags_25 },
    { 0x9a6732c6, SUC_lexicon_tags_14 },
    { 0xc94732c6, SUC_lexicon_tags_48 },
    { 0xfc0732c8, SUC_lexicon_tags_37 },
    { 0xe22732c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa88732cb, SUC_lexicon_tags_11 },
    { 0x92c732cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd8a732ce, SUC_lexicon_tags_7 },
    { 0x33c732ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774732d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4732dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfc732de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbdc732e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x128732e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x482732e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0732e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x806732ec, SUC_lexicon_tags_7 },
    { 0x6d6732ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38e732ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4a732f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x852732f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe732fd, SUC_lexicon_tags_33 },
    { 0xf60732fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee73301, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x93673303, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc873305, SUC_lexicon_tags_45 },
    { 0x98a73306, SUC_lexicon_tags_7 },
    { 0x9e473307, SUC_lexicon_tags_34 },
    { 0xab073308, SUC_lexicon_tags_43 },
    { 0x7e073309, SUC_lexicon_tags_17 },
    { 0x6027330a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6ea7330c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a73311, SUC_lexicon_tags_12 },
    { 0x71e73311, SUC_lexicon_tags_51 },
    { 0x23273313, SUC_lexicon_tags_7 },
    { 0x45c73312, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb273316, SUC_lexicon_tags_64 },
    { 0x3dc73317, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xea073319, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x49c7331b, SUC_lexicon_tags_66 },
    { 0x3387331c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7273321, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c73325, SUC_lexicon_tags_2 },
    { 0xa0273325, SUC_lexicon_tags_7 },
    { 0xf9c73327, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e7332b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59473333, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ee73335, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54073339, SUC_lexicon_tags_145 },
    { 0xe247333a, SUC_lexicon_tags_17 },
    { 0x4e07333a, SUC_lexicon_tags_43 },
    { 0x3f27333a, SUC_lexicon_tags_11 },
    { 0x8a67333b, SUC_lexicon_tags_47 },
    { 0xb87333e, SUC_lexicon_tags_0 },
    { 0xefe7333f, SUC_lexicon_tags_12 },
    { 0xf387333e, SUC_lexicon_tags_29 },
    { 0x24473341, SUC_lexicon_tags_17 },
    { 0x3f47333f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f473344, SUC_lexicon_tags_66 },
    { 0x15073344, SUC_lexicon_tags_17 },
    { 0x76c73346, SUC_lexicon_tags_65 },
    { 0x46e73347, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a73350, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc73358, SUC_lexicon_tags_37 },
    { 0x68873359, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7927335f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf273361, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31073369, SUC_lexicon_tags_60 },
    { 0x7ac7336a, SUC_lexicon_tags_46 },
    { 0x7cc7336b, SUC_lexicon_tags_15 },
    { 0x86a7336c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4247336f, SUC_lexicon_tags_15 },
    { 0xaea73370, SUC_lexicon_tags_91 },
    { 0xda273371, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6273377, SUC_lexicon_tags_23 },
    { 0x99073378, SUC_lexicon_tags_11 },
    { 0x84e73377, SUC_lexicon_tags_11 },
    { 0x2ac73378, SUC_lexicon_tags_12 },
    { 0x7bc7337b, SUC_lexicon_tags_2 },
    { 0x9667337b, SUC_lexicon_tags_2 },
    { 0x4207337d, SUC_lexicon_tags_4 },
    { 0x6467337b, SUC_lexicon_tags_65 },
    { 0x2707337f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65473381, SUC_lexicon_tags_7 },
    { 0xf4e73382, SUC_lexicon_tags_506 },
    { 0, NULL },
    { 0x14c73384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e7338c, SUC_lexicon_tags_14 },
    { 0x4e27338d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe9c7338f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e673396, SUC_lexicon_tags_14 },
    { 0xdfe73397, SUC_lexicon_tags_63 },
    { 0xa3e73396, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf0a7339a, SUC_lexicon_tags_0 },
    { 0x8447339b, SUC_lexicon_tags_8 },
    { 0xf507339c, SUC_lexicon_tags_10 },
    { 0xbde7339c, SUC_lexicon_tags_12 },
    { 0x4cc7339e, SUC_lexicon_tags_17 },
    { 0xa847339f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4733a8, SUC_lexicon_tags_33 },
    { 0x464733a9, SUC_lexicon_tags_95 },
    { 0xe44733a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188733b0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x968733b2, SUC_lexicon_tags_49 },
    { 0xac2733b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a733b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396733bb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e733bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba733c5, SUC_lexicon_tags_21 },
    { 0xa10733c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x262733c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e733cc, SUC_lexicon_tags_1 },
    { 0x330733cc, SUC_lexicon_tags_1 },
    { 0xacc733ce, SUC_lexicon_tags_47 },
    { 0xec8733ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x420733d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94733d7, SUC_lexicon_tags_34 },
    { 0x91a733d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c733e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a733e5, SUC_lexicon_tags_65 },
    { 0x312733e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x668733e9, SUC_lexicon_tags_17 },
    { 0x5da733ea, SUC_lexicon_tags_17 },
    { 0xe6733eb, SUC_lexicon_tags_55 },
    { 0x87a733eb, SUC_lexicon_tags_2 },
    { 0x40a733ec, SUC_lexicon_tags_63 },
    { 0x794733ee, SUC_lexicon_tags_17 },
    { 0xee0733ef, SUC_lexicon_tags_7 },
    { 0xcda733f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x772733f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68c733f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98e733f7, SUC_lexicon_tags_7 },
    { 0x4b0733f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c733fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32073404, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce73408, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1207340b, SUC_lexicon_tags_31 },
    { 0x8e47340c, SUC_lexicon_tags_25 },
    { 0xce07340c, SUC_lexicon_tags_23 },
    { 0x8847340e, SUC_lexicon_tags_17 },
    { 0x83e7340f, SUC_lexicon_tags_12 },
    { 0x8d27340e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae73416, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c73418, SUC_lexicon_tags_37 },
    { 0xf0c73418, SUC_lexicon_tags_27 },
    { 0x8487341a, SUC_lexicon_tags_47 },
    { 0xe567341b, SUC_lexicon_tags_6 },
    { 0xde07341c, SUC_lexicon_tags_7 },
    { 0x2887341d, SUC_lexicon_tags_46 },
    { 0x6447341c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8a73420, SUC_lexicon_tags_37 },
    { 0x62c73420, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a67342c, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87342f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee73434, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e073438, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb627343b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a73442, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f67344e, SUC_lexicon_tags_19 },
    { 0x4dc7344f, SUC_lexicon_tags_136 },
    { 0x72c73450, SUC_lexicon_tags_6 },
    { 0x3de7344f, SUC_lexicon_tags_2 },
    { 0xee873452, SUC_lexicon_tags_33 },
    { 0xc9a73452, SUC_lexicon_tags_18 },
    { 0xdc673454, SUC_lexicon_tags_2 },
    { 0x2d273455, SUC_lexicon_tags_13 },
    { 0x2aa73456, SUC_lexicon_tags_27 },
    { 0xbec73457, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcfc73459, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce7345f, SUC_lexicon_tags_25 },
    { 0xe3e73460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67073466, SUC_lexicon_tags_25 },
    { 0x33073467, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e7346b, SUC_lexicon_tags_12 },
    { 0xaba7346b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58073471, SUC_lexicon_tags_48 },
    { 0x75c73471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82673474, SUC_lexicon_tags_37 },
    { 0x5f473474, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe73479, SUC_lexicon_tags_1 },
    { 0xa6a73479, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd27347c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e073482, SUC_lexicon_tags_41 },
    { 0x71073483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73873490, SUC_lexicon_tags_4 },
    { 0x74a73491, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc73495, SUC_lexicon_tags_15 },
    { 0x8d673495, SUC_lexicon_tags_0 },
    { 0xb3473496, SUC_lexicon_tags_37 },
    { 0x72073495, SUC_lexicon_tags_68 },
    { 0xf9673499, SUC_lexicon_tags_12 },
    { 0x5d673495, SUC_lexicon_tags_17 },
    { 0x66673497, SUC_lexicon_tags_7 },
    { 0x63c73496, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70734a1, SUC_lexicon_tags_9 },
    { 0x8e0734a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e734a7, SUC_lexicon_tags_60 },
    { 0xed4734a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2a8734aa, SUC_lexicon_tags_48 },
    { 0x22a734aa, SUC_lexicon_tags_17 },
    { 0x730734aa, SUC_lexicon_tags_2 },
    { 0xa36734ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e734b0, SUC_lexicon_tags_21 },
    { 0xd48734b1, SUC_lexicon_tags_29 },
    { 0x34e734b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc734b6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88734bb, SUC_lexicon_tags_1 },
    { 0xa9c734bc, SUC_lexicon_tags_47 },
    { 0xe08734bd, SUC_lexicon_tags_37 },
    { 0x8ec734be, SUC_lexicon_tags_43 },
    { 0x36e734bd, SUC_lexicon_tags_25 },
    { 0xfa734c0, SUC_lexicon_tags_1 },
    { 0x72e734c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2734c7, SUC_lexicon_tags_50 },
    { 0x8ea734c8, SUC_lexicon_tags_47 },
    { 0x82c734c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x422734cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124734cf, SUC_lexicon_tags_7 },
    { 0xb4734d0, SUC_lexicon_tags_27 },
    { 0x1f0734d0, SUC_lexicon_tags_49 },
    { 0x516734cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a734d9, SUC_lexicon_tags_28 },
    { 0x218734da, SUC_lexicon_tags_15 },
    { 0xdd0734db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x236734de, SUC_lexicon_tags_11 },
    { 0xa50734df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed6734e1, SUC_lexicon_tags_6 },
    { 0xda6734e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4734e5, SUC_lexicon_tags_74 },
    { 0xd52734e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e734ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4734ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x540734ef, SUC_lexicon_tags_27 },
    { 0x718734f0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x968734f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x938734f5, SUC_lexicon_tags_13 },
    { 0xb66734f5, SUC_lexicon_tags_25 },
    { 0xe2e734f5, SUC_lexicon_tags_1 },
    { 0xab4734f8, SUC_lexicon_tags_11 },
    { 0xcb4734f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x148734fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbae734fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea2734ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c873509, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x52a7350b, SUC_lexicon_tags_1 },
    { 0x2607350c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xad87350e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5da73510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f873513, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c273516, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c73519, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6473521, SUC_lexicon_tags_29 },
    { 0x78273522, SUC_lexicon_tags_17 },
    { 0x51473523, SUC_lexicon_tags_37 },
    { 0xbc273522, SUC_lexicon_tags_42 },
    { 0x9d073521, SUC_lexicon_tags_2 },
    { 0xf0a73526, SUC_lexicon_tags_11 },
    { 0x29e73524, SUC_lexicon_tags_2 },
    { 0x48273526, SUC_lexicon_tags_28 },
    { 0xc0873529, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8327352b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb073532, SUC_lexicon_tags_17 },
    { 0x62673532, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e73536, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3167353a, SUC_lexicon_tags_12 },
    { 0xa647353b, SUC_lexicon_tags_11 },
    { 0x8007353a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59273542, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20a73544, SUC_lexicon_tags_37 },
    { 0x27e73544, SUC_lexicon_tags_33 },
    { 0xf1a73545, SUC_lexicon_tags_81 },
    { 0xae073546, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x54073549, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5c7354b, SUC_lexicon_tags_12 },
    { 0xc3e7354c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd907354f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe873556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce7355e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x37473560, SUC_lexicon_tags_19 },
    { 0x6cc73561, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3673566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2473568, SUC_lexicon_tags_47 },
    { 0x62273569, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd947356b, SUC_lexicon_tags_39 },
    { 0x7667356b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7327356e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb073571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a73574, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e73577, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2047357c, SUC_lexicon_tags_26 },
    { 0x1cc7357c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e73580, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8bc73582, SUC_lexicon_tags_17 },
    { 0x6ae73582, SUC_lexicon_tags_10 },
    { 0x89273584, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca273587, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x37c73589, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7358c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa307358f, SUC_lexicon_tags_12 },
    { 0x19a73590, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37873597, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32473599, SUC_lexicon_tags_63 },
    { 0x69a7359a, SUC_lexicon_tags_28 },
    { 0xce87359b, SUC_lexicon_tags_2 },
    { 0xf9c7359c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f67359e, SUC_lexicon_tags_12 },
    { 0x3a47359f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6735a3, SUC_lexicon_tags_7 },
    { 0x5c8735a4, SUC_lexicon_tags_26 },
    { 0x9a735a5, SUC_lexicon_tags_23 },
    { 0x824735a3, SUC_lexicon_tags_16 },
    { 0xf72735a7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a735aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc48735ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a735b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e735b5, SUC_lexicon_tags_75 },
    { 0x916735b6, SUC_lexicon_tags_39 },
    { 0x75c735b7, SUC_lexicon_tags_179 },
    { 0x4ae735b5, SUC_lexicon_tags_37 },
    { 0x40e735b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0735c1, SUC_lexicon_tags_19 },
    { 0x522735c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6fa735c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318735ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x758735d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8735d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4735d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x756735d9, SUC_lexicon_tags_37 },
    { 0x412735da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba735dd, SUC_lexicon_tags_7 },
    { 0x328735de, SUC_lexicon_tags_17 },
    { 0x908735df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c735e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02735ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8a735ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc4735ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8735f1, SUC_lexicon_tags_11 },
    { 0xa58735f2, SUC_lexicon_tags_25 },
    { 0xc5e735f3, SUC_lexicon_tags_1221 },
    { 0x3e0735f4, SUC_lexicon_tags_37 },
    { 0x2cc735f2, SUC_lexicon_tags_25 },
    { 0x958735f6, SUC_lexicon_tags_17 },
    { 0xe82735f4, SUC_lexicon_tags_43 },
    { 0xdc0735f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x996735fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf42735fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e473601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcec73603, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52873605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb67360b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a73611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22e73613, SUC_lexicon_tags_47 },
    { 0x71673614, SUC_lexicon_tags_1 },
    { 0x5073615, SUC_lexicon_tags_32 },
    { 0x54c73615, SUC_lexicon_tags_0 },
    { 0x8fc73617, SUC_lexicon_tags_27 },
    { 0x3f073618, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e7361b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace73621, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x92673624, SUC_lexicon_tags_596 },
    { 0x87a73624, SUC_lexicon_tags_63 },
    { 0xbac73625, SUC_lexicon_tags_17 },
    { 0x32c73624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd127362f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d273632, SUC_lexicon_tags_29 },
    { 0x6c073633, SUC_lexicon_tags_46 },
    { 0x14873633, SUC_lexicon_tags_46 },
    { 0x52673633, SUC_lexicon_tags_11 },
    { 0x38e73635, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b27363a, SUC_lexicon_tags_25 },
    { 0x6da7363a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8647363f, SUC_lexicon_tags_60 },
    { 0x6be73640, SUC_lexicon_tags_16 },
    { 0xbb273641, SUC_lexicon_tags_75 },
    { 0x35c73641, SUC_lexicon_tags_25 },
    { 0x79273642, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xece73645, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c73649, SUC_lexicon_tags_4 },
    { 0x8b07364a, SUC_lexicon_tags_47 },
    { 0xe3e7364b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6873655, SUC_lexicon_tags_37 },
    { 0x4c73656, SUC_lexicon_tags_11 },
    { 0x71273657, SUC_lexicon_tags_17 },
    { 0xc4e73658, SUC_lexicon_tags_19 },
    { 0xa6a73658, SUC_lexicon_tags_39 },
    { 0xc447365a, SUC_lexicon_tags_11 },
    { 0xb127365a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcca7365d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a073661, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73673663, SUC_lexicon_tags_38 },
    { 0x8273664, SUC_lexicon_tags_14 },
    { 0x85073664, SUC_lexicon_tags_38 },
    { 0x73c73663, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12473669, SUC_lexicon_tags_2 },
    { 0xab673669, SUC_lexicon_tags_7 },
    { 0xe07366b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9e07366d, SUC_lexicon_tags_31 },
    { 0x447366e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a73671, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a73676, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ac73678, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c7367c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfe7367e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf673682, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5273686, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc7368d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e473690, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6c73692, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x53a73694, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd873698, SUC_lexicon_tags_8 },
    { 0xc9473699, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde736a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b8736a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x820736a5, SUC_lexicon_tags_33 },
    { 0xb6736a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54c736a8, SUC_lexicon_tags_29 },
    { 0x542736a9, SUC_lexicon_tags_27 },
    { 0x67a736a9, SUC_lexicon_tags_43 },
    { 0x976736ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60736af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618736b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x518736b7, SUC_lexicon_tags_26 },
    { 0x252736b7, SUC_lexicon_tags_17 },
    { 0xbce736b9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6736bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4736c0, SUC_lexicon_tags_6 },
    { 0x1ac736c1, SUC_lexicon_tags_30 },
    { 0xc9e736c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4736c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x696736cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0736cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x256736d2, SUC_lexicon_tags_15 },
    { 0xc4736d2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472736da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfb6736dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2736e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69a736e2, SUC_lexicon_tags_7 },
    { 0x366736e2, SUC_lexicon_tags_15 },
    { 0xee8736e4, SUC_lexicon_tags_9 },
    { 0x34a736e5, SUC_lexicon_tags_15 },
    { 0x790736e3, SUC_lexicon_tags_63 },
    { 0x10a736e4, SUC_lexicon_tags_40 },
    { 0x3d6736e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x174736ea, SUC_lexicon_tags_26 },
    { 0xe4c736ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x45c736ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394736f4, SUC_lexicon_tags_4 },
    { 0xc96736f5, SUC_lexicon_tags_12 },
    { 0x3ce736f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14736fa, SUC_lexicon_tags_1 },
    { 0x33e736fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4736ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba73703, SUC_lexicon_tags_63 },
    { 0x92a73704, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa273707, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c7370a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe7370d, SUC_lexicon_tags_11 },
    { 0x12e7370e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4673711, SUC_lexicon_tags_2 },
    { 0x64a73712, SUC_lexicon_tags_27 },
    { 0x4c273713, SUC_lexicon_tags_11 },
    { 0x9c873711, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a73718, SUC_lexicon_tags_65 },
    { 0xa4c73718, SUC_lexicon_tags_160 },
    { 0xd0e7371a, SUC_lexicon_tags_47 },
    { 0x3ea7371b, SUC_lexicon_tags_17 },
    { 0xbb67371c, SUC_lexicon_tags_21 },
    { 0x5027371c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa73721, SUC_lexicon_tags_17 },
    { 0xc4e73722, SUC_lexicon_tags_116 },
    { 0x3cc73722, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93273729, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee67372e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c73734, SUC_lexicon_tags_7 },
    { 0xf9873735, SUC_lexicon_tags_14 },
    { 0x52a73736, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c73739, SUC_lexicon_tags_60 },
    { 0xd027373a, SUC_lexicon_tags_25 },
    { 0xaa07373b, SUC_lexicon_tags_37 },
    { 0xe4a7373c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c47373e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd473744, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea073747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b27374b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6147374f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a73751, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca73755, SUC_lexicon_tags_13 },
    { 0x91a73756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a07375d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xee47375f, SUC_lexicon_tags_35 },
    { 0x86673760, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64c73762, SUC_lexicon_tags_2 },
    { 0x1ca73762, SUC_lexicon_tags_7 },
    { 0xad273763, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0xde873766, SUC_lexicon_tags_7 },
    { 0xe5e73767, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa207376a, SUC_lexicon_tags_37 },
    { 0xbe47376a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7376e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67a73770, SUC_lexicon_tags_46 },
    { 0x9b473771, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a73774, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8073776, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb907377b, SUC_lexicon_tags_32 },
    { 0xebc7377b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1507377f, SUC_lexicon_tags_12 },
    { 0x22473780, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc1273782, SUC_lexicon_tags_27 },
    { 0x29673783, SUC_lexicon_tags_9 },
    { 0xc5e73783, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc73787, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24073789, SUC_lexicon_tags_45 },
    { 0x92c7378a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbee7378c, SUC_lexicon_tags_63 },
    { 0x6387378d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30873790, SUC_lexicon_tags_8 },
    { 0xf3073791, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c873794, SUC_lexicon_tags_15 },
    { 0xf3a73795, SUC_lexicon_tags_37 },
    { 0xb873796, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa7379b, SUC_lexicon_tags_21 },
    { 0xe847379c, SUC_lexicon_tags_7 },
    { 0x43a7379d, SUC_lexicon_tags_9 },
    { 0x767379e, SUC_lexicon_tags_69 },
    { 0xc7379f, SUC_lexicon_tags_23 },
    { 0x8d47379f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e737a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaa6737a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6737aa, SUC_lexicon_tags_1 },
    { 0x682737ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x716737ad, SUC_lexicon_tags_14 },
    { 0xc14737ae, SUC_lexicon_tags_49 },
    { 0xeda737af, SUC_lexicon_tags_47 },
    { 0xed0737b0, SUC_lexicon_tags_31 },
    { 0x8ea737b1, SUC_lexicon_tags_34 },
    { 0x630737b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e737b5, SUC_lexicon_tags_12 },
    { 0x6e2737b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30737b8, SUC_lexicon_tags_13 },
    { 0x5c737b8, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe737bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82737c0, SUC_lexicon_tags_12 },
    { 0x2a8737c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0737c8, SUC_lexicon_tags_17 },
    { 0x40a737c9, SUC_lexicon_tags_87 },
    { 0xb16737c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e737cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c737d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e737d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da737dd, SUC_lexicon_tags_2 },
    { 0x238737dd, SUC_lexicon_tags_17 },
    { 0x1dc737df, SUC_lexicon_tags_37 },
    { 0xb4e737df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0737e7, SUC_lexicon_tags_45 },
    { 0xe2a737e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x224737eb, SUC_lexicon_tags_17 },
    { 0x550737eb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe737f1, SUC_lexicon_tags_37 },
    { 0xd5e737f2, SUC_lexicon_tags_12 },
    { 0x76c737f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc18737f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc6737f7, SUC_lexicon_tags_11 },
    { 0x7fa737f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbe6737fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72e737fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b473808, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe307380a, SUC_lexicon_tags_1 },
    { 0xebe7380b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e7380e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1ec73810, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84873815, SUC_lexicon_tags_27 },
    { 0x67c73815, SUC_lexicon_tags_37 },
    { 0x27273817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8473821, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e473823, SUC_lexicon_tags_17 },
    { 0x49a73824, SUC_lexicon_tags_1 },
    { 0xdda73823, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4873828, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f07382d, SUC_lexicon_tags_104 },
    { 0xa0c7382d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7073830, SUC_lexicon_tags_9 },
    { 0x58673831, SUC_lexicon_tags_25 },
    { 0x92273832, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdc73834, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f273836, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52a73838, SUC_lexicon_tags_9 },
    { 0xeea73838, SUC_lexicon_tags_1 },
    { 0x1967383a, SUC_lexicon_tags_14 },
    { 0x4ee7383b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e073840, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8a673842, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c73845, SUC_lexicon_tags_2 },
    { 0x40e73846, SUC_lexicon_tags_37 },
    { 0x61073847, SUC_lexicon_tags_32 },
    { 0x24073846, SUC_lexicon_tags_25 },
    { 0xaac73849, SUC_lexicon_tags_21 },
    { 0xaa273849, SUC_lexicon_tags_38 },
    { 0x4507384a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e73851, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa473854, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac873856, SUC_lexicon_tags_12 },
    { 0x11e73857, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1867385b, SUC_lexicon_tags_31 },
    { 0xb347385c, SUC_lexicon_tags_7 },
    { 0x9827385c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72873863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1007386a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda7386c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6273871, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52873877, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x37873879, SUC_lexicon_tags_45 },
    { 0x5147387a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x2047387c, SUC_lexicon_tags_37 },
    { 0xdd27387d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6e27387f, SUC_lexicon_tags_31 },
    { 0x6cc73880, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1b673882, SUC_lexicon_tags_17 },
    { 0x83073883, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa73887, SUC_lexicon_tags_600 },
    { 0x1a673887, SUC_lexicon_tags_17 },
    { 0x45a73889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb067388b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e73895, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcfe73897, SUC_lexicon_tags_25 },
    { 0xe7473898, SUC_lexicon_tags_11 },
    { 0x72a73899, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92e7389b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa738a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a738a5, SUC_lexicon_tags_47 },
    { 0x54e738a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30738ab, SUC_lexicon_tags_1 },
    { 0x19c738ac, SUC_lexicon_tags_43 },
    { 0x320738ac, SUC_lexicon_tags_27 },
    { 0xc88738ad, SUC_lexicon_tags_2 },
    { 0xf26738ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x238738b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4738b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a738bd, SUC_lexicon_tags_25 },
    { 0xc74738be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x432738c1, SUC_lexicon_tags_11 },
    { 0x912738c1, SUC_lexicon_tags_5 },
    { 0x5d8738c3, SUC_lexicon_tags_37 },
    { 0x752738c4, SUC_lexicon_tags_37 },
    { 0x468738c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26738c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x792738ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x420738cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfc738ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca738d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x91a738d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498738dd, SUC_lexicon_tags_23 },
    { 0xeca738dd, SUC_lexicon_tags_11 },
    { 0xf10738df, SUC_lexicon_tags_0 },
    { 0x11e738df, SUC_lexicon_tags_2 },
    { 0xe40738e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe2738e3, SUC_lexicon_tags_11 },
    { 0xaf0738e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x676738e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56738ea, SUC_lexicon_tags_57 },
    { 0x622738ea, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x346738ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e738f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18738fa, SUC_lexicon_tags_15 },
    { 0xad8738fb, SUC_lexicon_tags_16 },
    { 0x3da738fb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe90738fe, SUC_lexicon_tags_31 },
    { 0xf9e738ff, SUC_lexicon_tags_23 },
    { 0xdc2738fe, SUC_lexicon_tags_8 },
    { 0xa2473900, SUC_lexicon_tags_66 },
    { 0xb4273902, SUC_lexicon_tags_12 },
    { 0xa8c73903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e673907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9027390a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc227390c, SUC_lexicon_tags_17 },
    { 0x8e87390d, SUC_lexicon_tags_0 },
    { 0xa167390e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99473916, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4807391b, SUC_lexicon_tags_17 },
    { 0x82c7391c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d47391f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaf673921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5873923, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc73926, SUC_lexicon_tags_31 },
    { 0x49873926, SUC_lexicon_tags_0 },
    { 0xff073926, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb027392a, SUC_lexicon_tags_1 },
    { 0x6e07392b, SUC_lexicon_tags_25 },
    { 0xd2e7392c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec7392f, SUC_lexicon_tags_2 },
    { 0xf7e7392f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa273934, SUC_lexicon_tags_37 },
    { 0xc6073935, SUC_lexicon_tags_498 },
    { 0x25a73936, SUC_lexicon_tags_63 },
    { 0x52873937, SUC_lexicon_tags_68 },
    { 0xbfa73937, SUC_lexicon_tags_29 },
    { 0x66c73936, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5873941, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c67394b, SUC_lexicon_tags_820 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76473957, SUC_lexicon_tags_68 },
    { 0xd7a73958, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa7395d, SUC_lexicon_tags_39 },
    { 0x8e07395d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf673960, SUC_lexicon_tags_30 },
    { 0xa7473961, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a73969, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab47396d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d273970, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c473973, SUC_lexicon_tags_27 },
    { 0x15473974, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x81073976, SUC_lexicon_tags_0 },
    { 0xa1273976, SUC_lexicon_tags_243 },
    { 0x16073976, SUC_lexicon_tags_27 },
    { 0xde073978, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc67397c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaac7397e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x97073980, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27273985, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e73989, SUC_lexicon_tags_11 },
    { 0xe0c7398a, SUC_lexicon_tags_160 },
    { 0xdce7398b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3427398d, SUC_lexicon_tags_15 },
    { 0xf9a7398e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88273992, SUC_lexicon_tags_37 },
    { 0xc1473992, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da73999, SUC_lexicon_tags_25 },
    { 0x2747399a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb267399d, SUC_lexicon_tags_63 },
    { 0x7467399e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10c739a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360739a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x994739a9, SUC_lexicon_tags_19 },
    { 0xe0739a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80739af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64739b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a2739b5, SUC_lexicon_tags_37 },
    { 0xd74739b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2739bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2739bf, SUC_lexicon_tags_37 },
    { 0xc90739bf, SUC_lexicon_tags_47 },
    { 0x6e4739c1, SUC_lexicon_tags_95 },
    { 0xeca739c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2739c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccc739cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76739cf, SUC_lexicon_tags_1 },
    { 0x582739cf, SUC_lexicon_tags_38 },
    { 0x84e739cf, SUC_lexicon_tags_25 },
    { 0x3c739d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70739d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a739de, SUC_lexicon_tags_47 },
    { 0x910739de, SUC_lexicon_tags_8 },
    { 0x500739e0, SUC_lexicon_tags_17 },
    { 0xd40739e1, SUC_lexicon_tags_23 },
    { 0xb18739e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2739eb, SUC_lexicon_tags_7 },
    { 0x1da739eb, SUC_lexicon_tags_25 },
    { 0x27a739ed, SUC_lexicon_tags_27 },
    { 0xe7c739ec, SUC_lexicon_tags_46 },
    { 0x4f0739ef, SUC_lexicon_tags_62 },
    { 0xeba739ef, SUC_lexicon_tags_1 },
    { 0xde2739f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2739f4, SUC_lexicon_tags_23 },
    { 0x3c739f5, SUC_lexicon_tags_7 },
    { 0x636739f6, SUC_lexicon_tags_14 },
    { 0x610739f5, SUC_lexicon_tags_11 },
    { 0x5f4739f6, SUC_lexicon_tags_47 },
    { 0x374739f9, SUC_lexicon_tags_43 },
    { 0xd46739fa, SUC_lexicon_tags_14 },
    { 0x73c739f5, SUC_lexicon_tags_626 },
    { 0x6a2739fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14739ff, SUC_lexicon_tags_27 },
    { 0x8a739ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24873a04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x89273a07, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a73a0b, SUC_lexicon_tags_62 },
    { 0xdb473a0b, SUC_lexicon_tags_172 },
    { 0x65473a0d, SUC_lexicon_tags_82 },
    { 0xd0a73a0e, SUC_lexicon_tags_47 },
    { 0xd3873a0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c73a15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8aa73a17, SUC_lexicon_tags_12 },
    { 0x57a73a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c473a1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6673a21, SUC_lexicon_tags_12 },
    { 0x2b873a22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20073a28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e73a2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeca73a2e, SUC_lexicon_tags_1 },
    { 0xda473a2f, SUC_lexicon_tags_7 },
    { 0xc3473a30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbda73a32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6a73a34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6873a37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c073a3b, SUC_lexicon_tags_25 },
    { 0xcaa73a3c, SUC_lexicon_tags_63 },
    { 0x4c473a3d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5473a42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e73a46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e73a49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea73a4c, SUC_lexicon_tags_37 },
    { 0xfe73a4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd473a4f, SUC_lexicon_tags_17 },
    { 0x54a73a4f, SUC_lexicon_tags_39 },
    { 0x66e73a51, SUC_lexicon_tags_63 },
    { 0x19e73a50, SUC_lexicon_tags_19 },
    { 0x2fa73a53, SUC_lexicon_tags_11 },
    { 0xd4273a54, SUC_lexicon_tags_37 },
    { 0x6873a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca873a5a, SUC_lexicon_tags_45 },
    { 0xae273a5a, SUC_lexicon_tags_11 },
    { 0xda873a5c, SUC_lexicon_tags_0 },
    { 0x6f673a5a, SUC_lexicon_tags_14 },
    { 0x46a73a5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4473a60, SUC_lexicon_tags_19 },
    { 0x22273a61, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x36273a64, SUC_lexicon_tags_93 },
    { 0xd2273a64, SUC_lexicon_tags_49 },
    { 0x6d873a66, SUC_lexicon_tags_38 },
    { 0xe7873a67, SUC_lexicon_tags_49 },
    { 0x79673a68, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90073a6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa873a77, SUC_lexicon_tags_14 },
    { 0x6c73a78, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0873a7b, SUC_lexicon_tags_63 },
    { 0xaf273a7c, SUC_lexicon_tags_7 },
    { 0x51673a7d, SUC_lexicon_tags_55 },
    { 0xdf073a7e, SUC_lexicon_tags_25 },
    { 0x9d873a7f, SUC_lexicon_tags_26 },
    { 0xb3e73a7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb673a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22273a85, SUC_lexicon_tags_15 },
    { 0x32273a86, SUC_lexicon_tags_15 },
    { 0x5d673a87, SUC_lexicon_tags_33 },
    { 0x9ae73a87, SUC_lexicon_tags_54 },
    { 0x83273a89, SUC_lexicon_tags_19 },
    { 0xe1a73a8a, SUC_lexicon_tags_88 },
    { 0xf0a73a8b, SUC_lexicon_tags_49 },
    { 0xd8073a8c, SUC_lexicon_tags_37 },
    { 0x55473a8d, SUC_lexicon_tags_25 },
    { 0xe4873a8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20473a90, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa73a93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91273a9a, SUC_lexicon_tags_37 },
    { 0x28673a9a, SUC_lexicon_tags_17 },
    { 0xf2c73a9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6e73a9e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf673aa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee73aa8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7073ab1, SUC_lexicon_tags_87 },
    { 0x99673ab2, SUC_lexicon_tags_27 },
    { 0xdda73ab2, SUC_lexicon_tags_45 },
    { 0x98a73ab4, SUC_lexicon_tags_7 },
    { 0xa5873ab2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbc873ab7, SUC_lexicon_tags_25 },
    { 0x58473ab7, SUC_lexicon_tags_49 },
    { 0xbac73ab9, SUC_lexicon_tags_1 },
    { 0xa8a73ab9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5273abc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e73abf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8873ac1, SUC_lexicon_tags_37 },
    { 0x77873ac1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa73ac9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x12473acb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59e73acd, SUC_lexicon_tags_127 },
    { 0x72e73ace, SUC_lexicon_tags_15 },
    { 0xade73acd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac73ad4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4c73ad6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5273ad8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a73ade, SUC_lexicon_tags_37 },
    { 0x8f073adf, SUC_lexicon_tags_15 },
    { 0x5473ae0, SUC_lexicon_tags_23 },
    { 0x6c473ae0, SUC_lexicon_tags_17 },
    { 0x9c673ae1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec73ae5, SUC_lexicon_tags_0 },
    { 0x2a673ae6, SUC_lexicon_tags_25 },
    { 0x5e273ae7, SUC_lexicon_tags_37 },
    { 0x94a73ae8, SUC_lexicon_tags_1 },
    { 0xfe473ae9, SUC_lexicon_tags_17 },
    { 0xab873ae8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x90e73aec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0073aef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea073af5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x58873af7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6473af9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a273afb, SUC_lexicon_tags_38 },
    { 0xd873afc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f273afe, SUC_lexicon_tags_27 },
    { 0xfc873aff, SUC_lexicon_tags_63 },
    { 0x35e73b00, SUC_lexicon_tags_43 },
    { 0x17e73b01, SUC_lexicon_tags_55 },
    { 0x88c73b02, SUC_lexicon_tags_7 },
    { 0xb8073b01, SUC_lexicon_tags_57 },
    { 0xa2873b01, SUC_lexicon_tags_37 },
    { 0xa5073b05, SUC_lexicon_tags_11 },
    { 0x8f673b05, SUC_lexicon_tags_1 },
    { 0x9a873b07, SUC_lexicon_tags_14 },
    { 0x30473b05, SUC_lexicon_tags_37 },
    { 0x9f873b04, SUC_lexicon_tags_13 },
    { 0x30473b0a, SUC_lexicon_tags_11 },
    { 0xa1473b02, SUC_lexicon_tags_10 },
    { 0xebc73b00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa273b0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6473b10, SUC_lexicon_tags_60 },
    { 0x5b473b11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5673b15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0273b19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5073b1f, SUC_lexicon_tags_11 },
    { 0xa4a73b20, SUC_lexicon_tags_55 },
    { 0x69473b1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a873b28, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a73b2c, SUC_lexicon_tags_27 },
    { 0x15073b2d, SUC_lexicon_tags_45 },
    { 0x56673b2e, SUC_lexicon_tags_38 },
    { 0xc2073b2d, SUC_lexicon_tags_14 },
    { 0x2ee73b30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a73b35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f073b3a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0473b41, SUC_lexicon_tags_12 },
    { 0x76073b42, SUC_lexicon_tags_8 },
    { 0x54e73b42, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a73b46, SUC_lexicon_tags_31 },
    { 0xcea73b47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e473b4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee73b54, SUC_lexicon_tags_21 },
    { 0xc3c73b55, SUC_lexicon_tags_26 },
    { 0x85073b54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e73b5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81473b5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c73b63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4473b65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a73b69, SUC_lexicon_tags_25 },
    { 0x97c73b6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e73b6d, SUC_lexicon_tags_10 },
    { 0xf5c73b6e, SUC_lexicon_tags_5 },
    { 0xbc073b6f, SUC_lexicon_tags_15 },
    { 0xc5073b70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a73b76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c73b79, SUC_lexicon_tags_12 },
    { 0x57e73b7a, SUC_lexicon_tags_25 },
    { 0xce673b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde73b7e, SUC_lexicon_tags_15 },
    { 0x6be73b7f, SUC_lexicon_tags_17 },
    { 0x7ae73b80, SUC_lexicon_tags_25 },
    { 0xe4673b80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44473b85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5873b88, SUC_lexicon_tags_2 },
    { 0xbac73b89, SUC_lexicon_tags_63 },
    { 0x30473b8a, SUC_lexicon_tags_15 },
    { 0xb9e73b8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f673b8d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x39273b90, SUC_lexicon_tags_11 },
    { 0xc3a73b91, SUC_lexicon_tags_11 },
    { 0xd1e73b90, SUC_lexicon_tags_66 },
    { 0x67673b93, SUC_lexicon_tags_25 },
    { 0x61a73b94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce73b97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfaa73b99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b273b9f, SUC_lexicon_tags_2 },
    { 0x72673b9f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc73ba3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaf673ba5, SUC_lexicon_tags_63 },
    { 0x5a473ba6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7fe73ba8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c73bab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x45673bad, SUC_lexicon_tags_15 },
    { 0x89673bae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7073bb3, SUC_lexicon_tags_44 },
    { 0x88273bb3, SUC_lexicon_tags_43 },
    { 0xfb073bb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b473bba, SUC_lexicon_tags_12 },
    { 0xe2473bba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5073bc2, SUC_lexicon_tags_39 },
    { 0x95873bc2, SUC_lexicon_tags_37 },
    { 0x6bc73bc4, SUC_lexicon_tags_1 },
    { 0xa4073bc5, SUC_lexicon_tags_37 },
    { 0x8c073bc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65273bc8, SUC_lexicon_tags_0 },
    { 0x3a73bc8, SUC_lexicon_tags_37 },
    { 0x61873bca, SUC_lexicon_tags_26 },
    { 0x4e073bcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c73bd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1073bd7, SUC_lexicon_tags_11 },
    { 0x5aa73bd8, SUC_lexicon_tags_12 },
    { 0x20673bd9, SUC_lexicon_tags_4 },
    { 0xe3e73bd7, SUC_lexicon_tags_58 },
    { 0xc0073bd7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc73be1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91273be6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8873be9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba73bec, SUC_lexicon_tags_25 },
    { 0x70473bed, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85873bf2, SUC_lexicon_tags_68 },
    { 0xda073bf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68673bf8, SUC_lexicon_tags_15 },
    { 0xa3473bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c73c02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f673c04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c73c07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59273c0c, SUC_lexicon_tags_25 },
    { 0x3e273c0d, SUC_lexicon_tags_6 },
    { 0x29e73c0c, SUC_lexicon_tags_17 },
    { 0xfe473c0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ba73c11, SUC_lexicon_tags_27 },
    { 0x26673c11, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e073c15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a73c18, SUC_lexicon_tags_37 },
    { 0x8f473c19, SUC_lexicon_tags_37 },
    { 0x24e73c19, SUC_lexicon_tags_26 },
    { 0x75873c1a, SUC_lexicon_tags_35 },
    { 0x98c73c18, SUC_lexicon_tags_133 },
    { 0x97c73c1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff673c2b, SUC_lexicon_tags_15 },
    { 0x3fa73c2c, SUC_lexicon_tags_40 },
    { 0xcbc73c2b, SUC_lexicon_tags_1 },
    { 0x15c73c2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a73c31, SUC_lexicon_tags_14 },
    { 0xb9673c32, SUC_lexicon_tags_2 },
    { 0xd4273c33, SUC_lexicon_tags_21 },
    { 0x22e73c34, SUC_lexicon_tags_7 },
    { 0xd1a73c35, SUC_lexicon_tags_0 },
    { 0x75e73c36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb873c40, SUC_lexicon_tags_0 },
    { 0x67073c41, SUC_lexicon_tags_14 },
    { 0x7c873c42, SUC_lexicon_tags_32 },
    { 0xa8073c43, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e73c46, SUC_lexicon_tags_2 },
    { 0x873c46, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7a873c49, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb073c4c, SUC_lexicon_tags_61 },
    { 0x5b073c4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89273c4f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9673c56, SUC_lexicon_tags_14 },
    { 0x3ca73c57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3473c5c, SUC_lexicon_tags_7 },
    { 0x34873c5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5473c61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37873c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc73c66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24473c6a, SUC_lexicon_tags_1 },
    { 0x65873c6b, SUC_lexicon_tags_47 },
    { 0x8b273c6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff473c73, SUC_lexicon_tags_11 },
    { 0x8c073c73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5673c77, SUC_lexicon_tags_37 },
    { 0x1a073c78, SUC_lexicon_tags_38 },
    { 0xdba73c79, SUC_lexicon_tags_15 },
    { 0x97673c79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcc73c7c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6d073c7e, SUC_lexicon_tags_33 },
    { 0x91a73c7f, SUC_lexicon_tags_2 },
    { 0x44673c80, SUC_lexicon_tags_11 },
    { 0xc2673c81, SUC_lexicon_tags_150 },
    { 0xe2073c81, SUC_lexicon_tags_25 },
    { 0x9f473c83, SUC_lexicon_tags_6 },
    { 0x20c73c83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c73c89, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaac73c8b, SUC_lexicon_tags_47 },
    { 0xc2e73c8b, SUC_lexicon_tags_15 },
    { 0x9b273c8d, SUC_lexicon_tags_16 },
    { 0xeb073c8e, SUC_lexicon_tags_37 },
    { 0x5a873c8f, SUC_lexicon_tags_36 },
    { 0xff473c90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca73c93, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce73c97, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4073c9d, SUC_lexicon_tags_60 },
    { 0xa2273c9e, SUC_lexicon_tags_31 },
    { 0x82073c9f, SUC_lexicon_tags_1 },
    { 0x21673ca0, SUC_lexicon_tags_12 },
    { 0x8e473ca1, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec473ca8, SUC_lexicon_tags_23 },
    { 0xc5473ca9, SUC_lexicon_tags_37 },
    { 0x80c73caa, SUC_lexicon_tags_15 },
    { 0xc8c73ca8, SUC_lexicon_tags_12 },
    { 0xf9273caa, SUC_lexicon_tags_6 },
    { 0x54873cad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4273cb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53873cb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60473cb9, SUC_lexicon_tags_4 },
    { 0x6b273cb9, SUC_lexicon_tags_9 },
    { 0xdc873cbb, SUC_lexicon_tags_9 },
    { 0xaa473cbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x80673cbe, SUC_lexicon_tags_46 },
    { 0x94873cbe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c73cc2, SUC_lexicon_tags_9 },
    { 0x33a73cc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1673cca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe73ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c473ccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3873cd1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73473cd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c73cdb, SUC_lexicon_tags_177 },
    { 0xcb673cdc, SUC_lexicon_tags_25 },
    { 0xcc273cdc, SUC_lexicon_tags_23 },
    { 0x93a73cde, SUC_lexicon_tags_26 },
    { 0x8ae73cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273ce5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2e73ce7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbba73ce9, SUC_lexicon_tags_13 },
    { 0x4fe73cea, SUC_lexicon_tags_1 },
    { 0xeaa73ceb, SUC_lexicon_tags_2 },
    { 0xd4473ceb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c73cf0, SUC_lexicon_tags_7 },
    { 0xc073cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc873cf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8e73cf5, SUC_lexicon_tags_12 },
    { 0x64673cf6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x18e73cf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ee73cfa, SUC_lexicon_tags_8 },
    { 0x47273cfb, SUC_lexicon_tags_37 },
    { 0x7e73cfb, SUC_lexicon_tags_15 },
    { 0xf5673cfc, SUC_lexicon_tags_43 },
    { 0x8d473cfe, SUC_lexicon_tags_108 },
    { 0x83873cff, SUC_lexicon_tags_16 },
    { 0x44a73cff, SUC_lexicon_tags_82 },
    { 0xa2073cfa, SUC_lexicon_tags_29 },
    { 0xbda73cfe, SUC_lexicon_tags_17 },
    { 0xc9673d03, SUC_lexicon_tags_8 },
    { 0x40273d04, SUC_lexicon_tags_644 },
    { 0x6c673d05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a73d0a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xaa273d0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba73d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7473d18, SUC_lexicon_tags_8 },
    { 0x37e73d19, SUC_lexicon_tags_19 },
    { 0x7fa73d19, SUC_lexicon_tags_25 },
    { 0x3ac73d1b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a73d1e, SUC_lexicon_tags_81 },
    { 0x42073d1e, SUC_lexicon_tags_35 },
    { 0xfee73d1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a73d26, SUC_lexicon_tags_12 },
    { 0x20873d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8673d29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d473d2b, SUC_lexicon_tags_14 },
    { 0xdba73d2c, SUC_lexicon_tags_60 },
    { 0x5ba73d2d, SUC_lexicon_tags_298 },
    { 0x76a73d2b, SUC_lexicon_tags_4 },
    { 0x55c73d2f, SUC_lexicon_tags_17 },
    { 0xc8e73d2c, SUC_lexicon_tags_33 },
    { 0xee073d2d, SUC_lexicon_tags_11 },
    { 0x65073d2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ee73d34, SUC_lexicon_tags_49 },
    { 0x96673d34, SUC_lexicon_tags_16 },
    { 0x7bc73d34, SUC_lexicon_tags_23 },
    { 0xcb073d37, SUC_lexicon_tags_6 },
    { 0x35273d38, SUC_lexicon_tags_31 },
    { 0xcee73d39, SUC_lexicon_tags_63 },
    { 0xdc873d39, SUC_lexicon_tags_11 },
    { 0xf7273d3a, SUC_lexicon_tags_25 },
    { 0x42273d3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a73d40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53273d42, SUC_lexicon_tags_66 },
    { 0x5c073d43, SUC_lexicon_tags_1 },
    { 0x9e873d44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9873d4c, SUC_lexicon_tags_15 },
    { 0x60473d4d, SUC_lexicon_tags_60 },
    { 0xfc73d4e, SUC_lexicon_tags_38 },
    { 0x24673d4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc273d51, SUC_lexicon_tags_17 },
    { 0x2ba73d52, SUC_lexicon_tags_4 },
    { 0x26873d52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17673d55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c73d58, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb873d5d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c73d63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92873d70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e73d74, SUC_lexicon_tags_26 },
    { 0xc8473d75, SUC_lexicon_tags_37 },
    { 0x78a73d75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43673d79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde73d7d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e273d80, SUC_lexicon_tags_37 },
    { 0x93473d80, SUC_lexicon_tags_15 },
    { 0x33673d82, SUC_lexicon_tags_41 },
    { 0x5b873d83, SUC_lexicon_tags_37 },
    { 0xa2e73d82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe73d89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd873d8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca273d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a873d93, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda73d9a, SUC_lexicon_tags_47 },
    { 0x7e873d9b, SUC_lexicon_tags_47 },
    { 0xdfc73d9b, SUC_lexicon_tags_1 },
    { 0xdcc73d9b, SUC_lexicon_tags_45 },
    { 0x84e73d9b, SUC_lexicon_tags_12 },
    { 0xc4673d9f, SUC_lexicon_tags_25 },
    { 0x7e073da0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc673da3, SUC_lexicon_tags_12 },
    { 0x1e73da3, SUC_lexicon_tags_1 },
    { 0x67a73da3, SUC_lexicon_tags_106 },
    { 0xe4e73da6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a73dac, SUC_lexicon_tags_61 },
    { 0x1d273dad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x22073daf, SUC_lexicon_tags_25 },
    { 0xeb073db0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c73db3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1473db5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b873db8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30e73dba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x39473dbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd673dc3, SUC_lexicon_tags_15 },
    { 0x42c73dc3, SUC_lexicon_tags_11 },
    { 0x60473dc5, SUC_lexicon_tags_1 },
    { 0x81273dc3, SUC_lexicon_tags_37 },
    { 0xfc473dc3, SUC_lexicon_tags_54 },
    { 0xc4073dc8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xea73dca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f073dd2, SUC_lexicon_tags_63 },
    { 0xa6a73dd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21473dd7, SUC_lexicon_tags_10 },
    { 0xa3673dd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88473dda, SUC_lexicon_tags_2 },
    { 0xed673ddb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda73dde, SUC_lexicon_tags_60 },
    { 0x51873ddf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c73de4, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9873deb, SUC_lexicon_tags_11 },
    { 0xc7c73deb, SUC_lexicon_tags_2 },
    { 0x6c073ded, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc73df2, SUC_lexicon_tags_1 },
    { 0x3e473df2, SUC_lexicon_tags_28 },
    { 0x20073df4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c73dfe, SUC_lexicon_tags_25 },
    { 0x10473dfe, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe73e08, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e73e0e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6673e10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d073e12, SUC_lexicon_tags_2 },
    { 0xec473e13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a73e16, SUC_lexicon_tags_37 },
    { 0x7e273e16, SUC_lexicon_tags_79 },
    { 0x59c73e18, SUC_lexicon_tags_25 },
    { 0xf0e73e17, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c73e1c, SUC_lexicon_tags_15 },
    { 0x87a73e1d, SUC_lexicon_tags_26 },
    { 0x56273e1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24473e21, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7873e25, SUC_lexicon_tags_23 },
    { 0x53e73e26, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2b873e28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10873e2a, SUC_lexicon_tags_11 },
    { 0x31473e2b, SUC_lexicon_tags_7 },
    { 0x2c473e2b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda73e30, SUC_lexicon_tags_414 },
    { 0x47073e31, SUC_lexicon_tags_46 },
    { 0xbc073e32, SUC_lexicon_tags_11 },
    { 0xeee73e33, SUC_lexicon_tags_70 },
    { 0x79673e30, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe9873e36, SUC_lexicon_tags_23 },
    { 0x72073e36, SUC_lexicon_tags_0 },
    { 0x71a73e38, SUC_lexicon_tags_7 },
    { 0x17e73e39, SUC_lexicon_tags_27 },
    { 0xd3e73e3a, SUC_lexicon_tags_15 },
    { 0x83473e38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf673e3e, SUC_lexicon_tags_60 },
    { 0xa7e73e3f, SUC_lexicon_tags_129 },
    { 0x84a73e40, SUC_lexicon_tags_37 },
    { 0x87873e40, SUC_lexicon_tags_15 },
    { 0x70473e41, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4f273e44, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e73e47, SUC_lexicon_tags_23 },
    { 0x9f273e48, SUC_lexicon_tags_16 },
    { 0xe1473e49, SUC_lexicon_tags_37 },
    { 0xbca73e4a, SUC_lexicon_tags_9 },
    { 0x75673e4a, SUC_lexicon_tags_25 },
    { 0x83273e4c, SUC_lexicon_tags_45 },
    { 0xcdc73e4a, SUC_lexicon_tags_26 },
    { 0x7d473e4d, SUC_lexicon_tags_84 },
    { 0xfb273e49, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28273e58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe73e5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc0873e5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de73e60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a73e64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcc73e66, SUC_lexicon_tags_33 },
    { 0x84c73e67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c73e6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a473e6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x68673e6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb9473e70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2473e72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x24a73e74, SUC_lexicon_tags_17 },
    { 0xb0c73e75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62873e7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63273e81, SUC_lexicon_tags_11 },
    { 0xafe73e81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ce73e84, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e073e89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e73e8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67473e91, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac73e97, SUC_lexicon_tags_37 },
    { 0x83873e98, SUC_lexicon_tags_26 },
    { 0xb9873e99, SUC_lexicon_tags_61 },
    { 0x54e73e98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e273e9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcf673e9f, SUC_lexicon_tags_25 },
    { 0xf8673ea0, SUC_lexicon_tags_45 },
    { 0xca673ea1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac73ea8, SUC_lexicon_tags_25 },
    { 0x7f473ea8, SUC_lexicon_tags_23 },
    { 0x8f473eaa, SUC_lexicon_tags_37 },
    { 0x81c73eaa, SUC_lexicon_tags_23 },
    { 0xb5673eaa, SUC_lexicon_tags_11 },
    { 0x94473eab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ce73eaf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe3073eb1, SUC_lexicon_tags_60 },
    { 0x70273eb1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c73eb7, SUC_lexicon_tags_44 },
    { 0x8a873eb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8e73eba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x14673ebc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ee73ebe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc73ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86073ec4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c73ec7, SUC_lexicon_tags_37 },
    { 0xe0c73ec8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xea73eca, SUC_lexicon_tags_17 },
    { 0xe3a73eca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf873ece, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x68673ed1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xac873ed4, SUC_lexicon_tags_11 },
    { 0xe4a73ed5, SUC_lexicon_tags_17 },
    { 0x87473ed4, SUC_lexicon_tags_25 },
    { 0x5bc73ed6, SUC_lexicon_tags_46 },
    { 0x93273ed8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e873edb, SUC_lexicon_tags_11 },
    { 0xa2273edc, SUC_lexicon_tags_68 },
    { 0x31673edc, SUC_lexicon_tags_15 },
    { 0x14273ede, SUC_lexicon_tags_46 },
    { 0x24e73edf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe873ee7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x50a73ee9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ba73eeb, SUC_lexicon_tags_25 },
    { 0xa8e73eec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a73ef7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80673efb, SUC_lexicon_tags_2 },
    { 0xcdc73efb, SUC_lexicon_tags_47 },
    { 0xa8a73efc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcc673eff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78273f01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9273f03, SUC_lexicon_tags_37 },
    { 0x9473f04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc73f07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a73f0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32673f13, SUC_lexicon_tags_28 },
    { 0x24c73f13, SUC_lexicon_tags_68 },
    { 0x4da73f15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3f873f17, SUC_lexicon_tags_66 },
    { 0xec73f18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c73f1b, SUC_lexicon_tags_33 },
    { 0xe8c73f1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc873f24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfca73f26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e73f28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa73f2e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e73f32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c73f36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x88873f38, SUC_lexicon_tags_42 },
    { 0x51473f39, SUC_lexicon_tags_27 },
    { 0xd3673f3a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59673f3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf8273f40, SUC_lexicon_tags_7 },
    { 0xed873f41, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d473f44, SUC_lexicon_tags_3 },
    { 0x31e73f44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c073f47, SUC_lexicon_tags_17 },
    { 0x99273f48, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7c873f4a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33473f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99673f51, SUC_lexicon_tags_1 },
    { 0x87e73f52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xce73f55, SUC_lexicon_tags_17 },
    { 0x53c73f56, SUC_lexicon_tags_23 },
    { 0x46a73f57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1673f5f, SUC_lexicon_tags_12 },
    { 0xd5673f60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e73f63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32273f68, SUC_lexicon_tags_23 },
    { 0x8c273f69, SUC_lexicon_tags_2 },
    { 0x64273f6a, SUC_lexicon_tags_12 },
    { 0xfe073f6b, SUC_lexicon_tags_56 },
    { 0x2a873f68, SUC_lexicon_tags_39 },
    { 0xca873f6d, SUC_lexicon_tags_11 },
    { 0xcc873f68, SUC_lexicon_tags_7 },
    { 0x42e73f6a, SUC_lexicon_tags_1 },
    { 0x59e73f70, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb1c73f72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe673f75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91273f7a, SUC_lexicon_tags_29 },
    { 0x63473f7b, SUC_lexicon_tags_37 },
    { 0xc4073f7b, SUC_lexicon_tags_49 },
    { 0xc2e73f7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x81873f80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a073f83, SUC_lexicon_tags_10 },
    { 0x78473f83, SUC_lexicon_tags_1 },
    { 0xae073f85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81a73f87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d273f8b, SUC_lexicon_tags_26 },
    { 0x85473f8c, SUC_lexicon_tags_11 },
    { 0xde073f8d, SUC_lexicon_tags_25 },
    { 0x14c73f8c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x70473f90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc273f92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e73f95, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c73f98, SUC_lexicon_tags_8 },
    { 0xd4673f98, SUC_lexicon_tags_1 },
    { 0xf4473f9a, SUC_lexicon_tags_23 },
    { 0x93273f9a, SUC_lexicon_tags_14 },
    { 0x84c73f9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79873f9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae73fa2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c73fa5, SUC_lexicon_tags_8 },
    { 0xb5073fa6, SUC_lexicon_tags_8 },
    { 0xd4c73fa6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20473fa9, SUC_lexicon_tags_28 },
    { 0xd5673faa, SUC_lexicon_tags_30 },
    { 0x93c73faa, SUC_lexicon_tags_136 },
    { 0xad673fac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17473fae, SUC_lexicon_tags_11 },
    { 0xc6073faf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb673fb1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcfe73fb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe073fb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5273fbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca73fbf, SUC_lexicon_tags_25 },
    { 0x29a73fbf, SUC_lexicon_tags_37 },
    { 0x6c873fc1, SUC_lexicon_tags_19 },
    { 0xf9c73fc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e73fc5, SUC_lexicon_tags_61 },
    { 0x6a473fc5, SUC_lexicon_tags_37 },
    { 0xa2c73fc7, SUC_lexicon_tags_11 },
    { 0x41073fc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea73fcb, SUC_lexicon_tags_16 },
    { 0x84673fcb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82073fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51c73fd8, SUC_lexicon_tags_63 },
    { 0x7bc73fd8, SUC_lexicon_tags_2 },
    { 0xdfc73fda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c73fe0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62473fe5, SUC_lexicon_tags_15 },
    { 0x66a73fe6, SUC_lexicon_tags_17 },
    { 0x9ac73fe5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf873fe9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71673feb, SUC_lexicon_tags_7 },
    { 0x57e73fec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a73ff0, SUC_lexicon_tags_37 },
    { 0xb3273ff1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c73ff5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b673ff7, SUC_lexicon_tags_37 },
    { 0xd3873ff8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe473ffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26873ffd, SUC_lexicon_tags_9 },
    { 0xfcc73ffe, SUC_lexicon_tags_37 },
    { 0x43873fff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2274002, SUC_lexicon_tags_33 },
    { 0xeb474003, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b874008, SUC_lexicon_tags_1 },
    { 0x26a74009, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e27400b, SUC_lexicon_tags_25 },
    { 0xf527400b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec7400f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be74014, SUC_lexicon_tags_12 },
    { 0x96e74015, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x82874018, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5287401c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7d07401e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x67274021, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe347402a, SUC_lexicon_tags_61 },
    { 0xd727402a, SUC_lexicon_tags_138 },
    { 0x51c7402c, SUC_lexicon_tags_17 },
    { 0xd1c7402d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a47402f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb474032, SUC_lexicon_tags_27 },
    { 0x10074032, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6da74035, SUC_lexicon_tags_30 },
    { 0xda874036, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2274039, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9f47403b, SUC_lexicon_tags_6 },
    { 0x8a07403b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd47403e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1674046, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6de74048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67404e, SUC_lexicon_tags_30 },
    { 0x97a7404f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21874057, SUC_lexicon_tags_87 },
    { 0xaa674057, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac7405d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb674065, SUC_lexicon_tags_15 },
    { 0xd3074066, SUC_lexicon_tags_26 },
    { 0xf5474067, SUC_lexicon_tags_21 },
    { 0x27474066, SUC_lexicon_tags_2 },
    { 0x12074066, SUC_lexicon_tags_15 },
    { 0x23a7406a, SUC_lexicon_tags_30 },
    { 0xbd674067, SUC_lexicon_tags_10 },
    { 0xd647406c, SUC_lexicon_tags_38 },
    { 0xe0874068, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1447406f, SUC_lexicon_tags_27 },
    { 0xb7474070, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xabe74072, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x36274075, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd4474077, SUC_lexicon_tags_25 },
    { 0xbd874078, SUC_lexicon_tags_12 },
    { 0xd9074077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb807407b, SUC_lexicon_tags_37 },
    { 0x6e07407c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1e7407e, SUC_lexicon_tags_7 },
    { 0x6bc7407e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94874081, SUC_lexicon_tags_2 },
    { 0x54c74081, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee74086, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4c074088, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4687408a, SUC_lexicon_tags_17 },
    { 0x43c7408b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7a07408d, SUC_lexicon_tags_25 },
    { 0x8447408e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7e74090, SUC_lexicon_tags_1 },
    { 0x37c74090, SUC_lexicon_tags_12 },
    { 0xa1474091, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb074095, SUC_lexicon_tags_25 },
    { 0x68674096, SUC_lexicon_tags_37 },
    { 0x91474095, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d07409c, SUC_lexicon_tags_25 },
    { 0x1847409d, SUC_lexicon_tags_37 },
    { 0xc527409d, SUC_lexicon_tags_55 },
    { 0x2cc7409f, SUC_lexicon_tags_19 },
    { 0x28e740a0, SUC_lexicon_tags_25 },
    { 0x574740a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b6740a3, SUC_lexicon_tags_39 },
    { 0xb88740a4, SUC_lexicon_tags_37 },
    { 0x956740a5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4740a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744740b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x400740b5, SUC_lexicon_tags_55 },
    { 0x95c740b6, SUC_lexicon_tags_37 },
    { 0x928740b7, SUC_lexicon_tags_8 },
    { 0xa54740b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98740bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe5c740bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e740c0, SUC_lexicon_tags_1 },
    { 0xddc740c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xac4740c3, SUC_lexicon_tags_80 },
    { 0x46740c4, SUC_lexicon_tags_65 },
    { 0x3fa740c5, SUC_lexicon_tags_62 },
    { 0xb3c740c4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e740ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14740d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fc740d2, SUC_lexicon_tags_29 },
    { 0xdb8740d2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da740d7, SUC_lexicon_tags_8 },
    { 0xe0a740d7, SUC_lexicon_tags_12 },
    { 0xa02740d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a740dc, SUC_lexicon_tags_15 },
    { 0x61e740dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292740e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396740ea, SUC_lexicon_tags_172 },
    { 0xd64740ea, SUC_lexicon_tags_30 },
    { 0xc76740ec, SUC_lexicon_tags_17 },
    { 0x854740ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x828740ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaf8740f1, SUC_lexicon_tags_25 },
    { 0xa64740f1, SUC_lexicon_tags_17 },
    { 0x172740f3, SUC_lexicon_tags_0 },
    { 0x708740f4, SUC_lexicon_tags_11 },
    { 0x4d2740f3, SUC_lexicon_tags_38 },
    { 0x74740f6, SUC_lexicon_tags_11 },
    { 0x738740f7, SUC_lexicon_tags_43 },
    { 0xa6a740f3, SUC_lexicon_tags_10 },
    { 0xd40740f9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952740fd, SUC_lexicon_tags_26 },
    { 0xc8e740fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54674100, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe074102, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a74105, SUC_lexicon_tags_84 },
    { 0x2074106, SUC_lexicon_tags_1 },
    { 0x7fe74107, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6567410e, SUC_lexicon_tags_17 },
    { 0xa567410e, SUC_lexicon_tags_11 },
    { 0xe274110, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe74114, SUC_lexicon_tags_69 },
    { 0x3a674115, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6847411c, SUC_lexicon_tags_12 },
    { 0x5067411d, SUC_lexicon_tags_34 },
    { 0xb9c7411e, SUC_lexicon_tags_15 },
    { 0x1087411f, SUC_lexicon_tags_26 },
    { 0xec7411f, SUC_lexicon_tags_37 },
    { 0xc8a74121, SUC_lexicon_tags_44 },
    { 0x10e7411f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c074126, SUC_lexicon_tags_25 },
    { 0xe5274126, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4427412a, SUC_lexicon_tags_47 },
    { 0xf907412b, SUC_lexicon_tags_22 },
    { 0x4e67412c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2047412e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac74136, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc74139, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e07413c, SUC_lexicon_tags_13 },
    { 0xd967413d, SUC_lexicon_tags_12 },
    { 0x2707413e, SUC_lexicon_tags_37 },
    { 0x62a7413e, SUC_lexicon_tags_37 },
    { 0x6ec7413e, SUC_lexicon_tags_11 },
    { 0x674141, SUC_lexicon_tags_1 },
    { 0xc8274141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfec74144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2227414a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c7414f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9674158, SUC_lexicon_tags_63 },
    { 0x3a74159, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa167415b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e74161, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47874167, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9674169, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7416d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4674170, SUC_lexicon_tags_11 },
    { 0x99874170, SUC_lexicon_tags_38 },
    { 0xdf074171, SUC_lexicon_tags_41 },
    { 0xa7e74170, SUC_lexicon_tags_7 },
    { 0x9c874172, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd874176, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8474178, SUC_lexicon_tags_17 },
    { 0x68e74178, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e47417e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc674182, SUC_lexicon_tags_57 },
    { 0x40674183, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41674186, SUC_lexicon_tags_28 },
    { 0x94c74187, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27418a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c7418d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaae7418f, SUC_lexicon_tags_45 },
    { 0x4e874190, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7419a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc487419d, SUC_lexicon_tags_33 },
    { 0xcf27419d, SUC_lexicon_tags_9 },
    { 0xfca7419e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x45e741a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae741a6, SUC_lexicon_tags_1 },
    { 0x46e741a7, SUC_lexicon_tags_11 },
    { 0xab8741a8, SUC_lexicon_tags_24 },
    { 0xdbc741a8, SUC_lexicon_tags_1 },
    { 0xc32741aa, SUC_lexicon_tags_31 },
    { 0x6ce741ab, SUC_lexicon_tags_25 },
    { 0xd40741a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa34741ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98741b6, SUC_lexicon_tags_2 },
    { 0x7ee741b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc741bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e741c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2741c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2741d0, SUC_lexicon_tags_11 },
    { 0xde0741d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e741d5, SUC_lexicon_tags_7 },
    { 0x18741d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18741d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bc741db, SUC_lexicon_tags_12 },
    { 0xa12741db, SUC_lexicon_tags_17 },
    { 0x22e741db, SUC_lexicon_tags_25 },
    { 0x474741dd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6f2741e0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a741e3, SUC_lexicon_tags_198 },
    { 0xf74741e3, SUC_lexicon_tags_15 },
    { 0xc28741e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0741e8, SUC_lexicon_tags_49 },
    { 0xd98741e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0741ee, SUC_lexicon_tags_25 },
    { 0x31a741ef, SUC_lexicon_tags_38 },
    { 0xc64741f0, SUC_lexicon_tags_31 },
    { 0x88a741f1, SUC_lexicon_tags_170 },
    { 0x4d2741f2, SUC_lexicon_tags_17 },
    { 0xdc8741f3, SUC_lexicon_tags_47 },
    { 0xeb6741ee, SUC_lexicon_tags_7 },
    { 0x99e741f5, SUC_lexicon_tags_45 },
    { 0xbac741f6, SUC_lexicon_tags_26 },
    { 0x152741f1, SUC_lexicon_tags_0 },
    { 0x1cc741f8, SUC_lexicon_tags_29 },
    { 0x344741f4, SUC_lexicon_tags_7 },
    { 0x482741fa, SUC_lexicon_tags_29 },
    { 0x6ae741fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5074203, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce74206, SUC_lexicon_tags_26 },
    { 0xbb074206, SUC_lexicon_tags_12 },
    { 0x7a274208, SUC_lexicon_tags_17 },
    { 0x84874209, SUC_lexicon_tags_12 },
    { 0x8e474207, SUC_lexicon_tags_1 },
    { 0x3407420b, SUC_lexicon_tags_12 },
    { 0x32e7420c, SUC_lexicon_tags_11 },
    { 0x40a7420c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a74212, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf27421c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3e7421e, SUC_lexicon_tags_43 },
    { 0xbd47421e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61074224, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c74227, SUC_lexicon_tags_17 },
    { 0x24074227, SUC_lexicon_tags_53 },
    { 0x99c74228, SUC_lexicon_tags_29 },
    { 0x4307422a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c7422e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x72874231, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2474234, SUC_lexicon_tags_2 },
    { 0x86c74235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1274237, SUC_lexicon_tags_15 },
    { 0x43a74238, SUC_lexicon_tags_63 },
    { 0xb9e74239, SUC_lexicon_tags_37 },
    { 0x68474239, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70874242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8674244, SUC_lexicon_tags_0 },
    { 0xbb874245, SUC_lexicon_tags_16 },
    { 0x4b874245, SUC_lexicon_tags_55 },
    { 0xf6874247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e7424e, SUC_lexicon_tags_8 },
    { 0x6dc7424e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2274252, SUC_lexicon_tags_1 },
    { 0x79274253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58874255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce27425c, SUC_lexicon_tags_56 },
    { 0x1ea7425d, SUC_lexicon_tags_11 },
    { 0xe107425e, SUC_lexicon_tags_9 },
    { 0xd8a7425d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd874264, SUC_lexicon_tags_77 },
    { 0xde074265, SUC_lexicon_tags_1 },
    { 0x8e674266, SUC_lexicon_tags_2 },
    { 0x12674267, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7627426c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbde7426e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c74272, SUC_lexicon_tags_0 },
    { 0xb7474272, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d674276, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe3874278, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee7427f, SUC_lexicon_tags_201 },
    { 0xcd874280, SUC_lexicon_tags_25 },
    { 0xbde74281, SUC_lexicon_tags_60 },
    { 0xaf074280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1aa74284, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x74e74286, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a7428c, SUC_lexicon_tags_10 },
    { 0xdbc7428d, SUC_lexicon_tags_48 },
    { 0xd6e7428e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11674290, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69874293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42474297, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd627429c, SUC_lexicon_tags_2 },
    { 0x7a7429d, SUC_lexicon_tags_44 },
    { 0xfdc7429d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6742a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2742a4, SUC_lexicon_tags_16 },
    { 0x3fe742a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36742a8, SUC_lexicon_tags_47 },
    { 0x742742a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18742ad, SUC_lexicon_tags_49 },
    { 0x8e2742ae, SUC_lexicon_tags_7 },
    { 0x58e742ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc0742b1, SUC_lexicon_tags_31 },
    { 0xa20742b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654742b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4742ba, SUC_lexicon_tags_77 },
    { 0x448742ba, SUC_lexicon_tags_14 },
    { 0x204742bc, SUC_lexicon_tags_11 },
    { 0x536742bc, SUC_lexicon_tags_11 },
    { 0x270742be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50742c6, SUC_lexicon_tags_16 },
    { 0xbfc742c7, SUC_lexicon_tags_2 },
    { 0xcc2742c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde742cb, SUC_lexicon_tags_11 },
    { 0x5a742cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbca742ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80742d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1be742d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee742d7, SUC_lexicon_tags_33 },
    { 0xa02742d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x820742da, SUC_lexicon_tags_87 },
    { 0x4ae742da, SUC_lexicon_tags_7 },
    { 0x8aa742dc, SUC_lexicon_tags_30 },
    { 0xec8742dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf0742df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f0742e1, SUC_lexicon_tags_25 },
    { 0xe52742e1, SUC_lexicon_tags_25 },
    { 0x328742e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6742eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa56742ed, SUC_lexicon_tags_26 },
    { 0xae2742ee, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6742f8, SUC_lexicon_tags_31 },
    { 0x6e6742f9, SUC_lexicon_tags_15 },
    { 0x134742f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf22742fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3074303, SUC_lexicon_tags_107 },
    { 0x54674303, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c74308, SUC_lexicon_tags_37 },
    { 0x65e74309, SUC_lexicon_tags_201 },
    { 0xd647430a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1147430d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c74313, SUC_lexicon_tags_7 },
    { 0xb0274314, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc74317, SUC_lexicon_tags_15 },
    { 0x93a74318, SUC_lexicon_tags_49 },
    { 0xc3274319, SUC_lexicon_tags_82 },
    { 0x4a07431a, SUC_lexicon_tags_15 },
    { 0xbfa74319, SUC_lexicon_tags_32 },
    { 0x1547431c, SUC_lexicon_tags_25 },
    { 0x5e7431d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be74320, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91274323, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d274328, SUC_lexicon_tags_11 },
    { 0xd6a74328, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e7432d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31674331, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2274334, SUC_lexicon_tags_45 },
    { 0x7aa74335, SUC_lexicon_tags_1 },
    { 0xc3874335, SUC_lexicon_tags_25 },
    { 0xe1474334, SUC_lexicon_tags_7 },
    { 0x65e74338, SUC_lexicon_tags_37 },
    { 0xe4474339, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9607433c, SUC_lexicon_tags_11 },
    { 0x4ca7433d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12674344, SUC_lexicon_tags_11 },
    { 0xb6874345, SUC_lexicon_tags_29 },
    { 0xc9c74346, SUC_lexicon_tags_25 },
    { 0x32674347, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a7434b, SUC_lexicon_tags_39 },
    { 0x5047434c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1fa7434e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c7435a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b27435d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1274368, SUC_lexicon_tags_0 },
    { 0x61074369, SUC_lexicon_tags_25 },
    { 0x3027436a, SUC_lexicon_tags_17 },
    { 0x92074369, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c87436d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60674371, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc74378, SUC_lexicon_tags_27 },
    { 0xf2e74379, SUC_lexicon_tags_0 },
    { 0xac474379, SUC_lexicon_tags_26 },
    { 0x3ba7437b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x31c7437d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff47437f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3874382, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32274384, SUC_lexicon_tags_38 },
    { 0x17474384, SUC_lexicon_tags_63 },
    { 0xca274385, SUC_lexicon_tags_37 },
    { 0x3e274387, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x72274389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5247438b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda874391, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1f274393, SUC_lexicon_tags_17 },
    { 0xddc74393, SUC_lexicon_tags_1 },
    { 0x84274395, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da74399, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdee7439b, SUC_lexicon_tags_7 },
    { 0x8127439c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe743a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x360743a2, SUC_lexicon_tags_14 },
    { 0x5a2743a3, SUC_lexicon_tags_17 },
    { 0xf06743a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758743a9, SUC_lexicon_tags_26 },
    { 0xc32743a9, SUC_lexicon_tags_17 },
    { 0xf58743a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x194743ae, SUC_lexicon_tags_0 },
    { 0xdf8743ae, SUC_lexicon_tags_15 },
    { 0xa56743b0, SUC_lexicon_tags_65 },
    { 0x460743ae, SUC_lexicon_tags_26 },
    { 0x154743b2, SUC_lexicon_tags_47 },
    { 0xb08743b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x812743b6, SUC_lexicon_tags_18 },
    { 0x302743b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e743ba, SUC_lexicon_tags_26 },
    { 0x788743bb, SUC_lexicon_tags_172 },
    { 0x44e743bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98743c4, SUC_lexicon_tags_63 },
    { 0x880743c4, SUC_lexicon_tags_11 },
    { 0x5ce743c6, SUC_lexicon_tags_25 },
    { 0x1f0743c7, SUC_lexicon_tags_30 },
    { 0xcea743c8, SUC_lexicon_tags_8 },
    { 0xd1a743c9, SUC_lexicon_tags_10 },
    { 0xc82743c6, SUC_lexicon_tags_63 },
    { 0xd38743cb, SUC_lexicon_tags_56 },
    { 0x724743cc, SUC_lexicon_tags_26 },
    { 0x29c743ca, SUC_lexicon_tags_23 },
    { 0x716743ce, SUC_lexicon_tags_37 },
    { 0xc86743cf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72743d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a0743da, SUC_lexicon_tags_27 },
    { 0x558743db, SUC_lexicon_tags_26 },
    { 0x6a6743dc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0743e0, SUC_lexicon_tags_31 },
    { 0x2ce743e1, SUC_lexicon_tags_68 },
    { 0x55c743e1, SUC_lexicon_tags_32 },
    { 0xd02743e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918743e9, SUC_lexicon_tags_7 },
    { 0x9f6743ea, SUC_lexicon_tags_14 },
    { 0x48c743ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798743f2, SUC_lexicon_tags_47 },
    { 0xfea743f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40743fc, SUC_lexicon_tags_22 },
    { 0xa08743fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb474401, SUC_lexicon_tags_25 },
    { 0xd8e74402, SUC_lexicon_tags_30 },
    { 0x23c74402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xebc74405, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b474408, SUC_lexicon_tags_1 },
    { 0xe2874409, SUC_lexicon_tags_15 },
    { 0x26874409, SUC_lexicon_tags_11 },
    { 0x2587440b, SUC_lexicon_tags_12 },
    { 0x9e67440c, SUC_lexicon_tags_17 },
    { 0xfd87440d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac7440f, SUC_lexicon_tags_12 },
    { 0x5874410, SUC_lexicon_tags_182 },
    { 0x1e274411, SUC_lexicon_tags_43 },
    { 0xda74411, SUC_lexicon_tags_108 },
    { 0x42c74412, SUC_lexicon_tags_27 },
    { 0x4a674414, SUC_lexicon_tags_12 },
    { 0x84074414, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e74418, SUC_lexicon_tags_37 },
    { 0x67474419, SUC_lexicon_tags_25 },
    { 0x9ca7441a, SUC_lexicon_tags_7 },
    { 0x4de74419, SUC_lexicon_tags_5 },
    { 0xd127441c, SUC_lexicon_tags_28 },
    { 0xd427441c, SUC_lexicon_tags_61 },
    { 0x9187441e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a74422, SUC_lexicon_tags_0 },
    { 0x41c74422, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc6a74425, SUC_lexicon_tags_8 },
    { 0xb8874426, SUC_lexicon_tags_49 },
    { 0x1d874427, SUC_lexicon_tags_2 },
    { 0x22474428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc247442d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54674430, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a74433, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3307443b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2aa7443d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4474440, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e74447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef074449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78c7444b, SUC_lexicon_tags_37 },
    { 0xf087444c, SUC_lexicon_tags_2 },
    { 0x30a7444d, SUC_lexicon_tags_37 },
    { 0x8527444e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6674451, SUC_lexicon_tags_17 },
    { 0xb0a74452, SUC_lexicon_tags_4 },
    { 0x28e74453, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a74457, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3547445b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb87445f, SUC_lexicon_tags_15 },
    { 0x9c7445f, SUC_lexicon_tags_1 },
    { 0xba474461, SUC_lexicon_tags_46 },
    { 0x89074460, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce74466, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xec474469, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4307446c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f474472, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x34474474, SUC_lexicon_tags_26 },
    { 0xc5074475, SUC_lexicon_tags_49 },
    { 0x90e74474, SUC_lexicon_tags_63 },
    { 0x3ca74476, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e07447b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe727447e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77274483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a74488, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf527448b, SUC_lexicon_tags_17 },
    { 0xfaa7448c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca7448f, SUC_lexicon_tags_45 },
    { 0xbd474490, SUC_lexicon_tags_25 },
    { 0x96874491, SUC_lexicon_tags_25 },
    { 0x1aa74492, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x82874495, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5c074497, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3767449a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa787449c, SUC_lexicon_tags_60 },
    { 0x89e7449c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4744a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc744a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a744aa, SUC_lexicon_tags_23 },
    { 0x1f2744aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78e744ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06744b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf06744b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a8744b5, SUC_lexicon_tags_24 },
    { 0x744744b5, SUC_lexicon_tags_1 },
    { 0xcc0744b7, SUC_lexicon_tags_7 },
    { 0x552744b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4744bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b4744be, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8744c6, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xcf2744c8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xec2744ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc02744cc, SUC_lexicon_tags_136 },
    { 0x5b0744cc, SUC_lexicon_tags_33 },
    { 0x5a0744ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8744d3, SUC_lexicon_tags_34 },
    { 0x456744d3, SUC_lexicon_tags_107 },
    { 0x960744d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef0744d7, SUC_lexicon_tags_8 },
    { 0xcee744d7, SUC_lexicon_tags_17 },
    { 0x418744d9, SUC_lexicon_tags_17 },
    { 0x202744da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562744e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbde744e3, SUC_lexicon_tags_12 },
    { 0x60c744e4, SUC_lexicon_tags_49 },
    { 0xec0744e4, SUC_lexicon_tags_33 },
    { 0x14e744e6, SUC_lexicon_tags_45 },
    { 0xb4e744e7, SUC_lexicon_tags_29 },
    { 0xeb0744e8, SUC_lexicon_tags_33 },
    { 0xd3c744e8, SUC_lexicon_tags_2 },
    { 0x770744ea, SUC_lexicon_tags_7 },
    { 0xab6744eb, SUC_lexicon_tags_17 },
    { 0xea8744ec, SUC_lexicon_tags_11 },
    { 0xe18744ed, SUC_lexicon_tags_11 },
    { 0xff6744eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6744f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28744f6, SUC_lexicon_tags_14 },
    { 0x26e744f6, SUC_lexicon_tags_35 },
    { 0x4de744f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42744fc, SUC_lexicon_tags_25 },
    { 0x24e744fc, SUC_lexicon_tags_63 },
    { 0xf5a744fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3074500, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a74505, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ac74507, SUC_lexicon_tags_26 },
    { 0x37c74508, SUC_lexicon_tags_15 },
    { 0xae074509, SUC_lexicon_tags_11 },
    { 0xee7450a, SUC_lexicon_tags_17 },
    { 0x7147450b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7d07450d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd147450f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9274513, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf874516, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb4474518, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x91a7451a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xae07451c, SUC_lexicon_tags_37 },
    { 0xb3c7451d, SUC_lexicon_tags_14 },
    { 0xc227451e, SUC_lexicon_tags_25 },
    { 0xc067451c, SUC_lexicon_tags_55 },
    { 0x4e74520, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x27a74522, SUC_lexicon_tags_55 },
    { 0x76c74523, SUC_lexicon_tags_29 },
    { 0x99874524, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6874527, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b67452b, SUC_lexicon_tags_37 },
    { 0xe087452b, SUC_lexicon_tags_25 },
    { 0xbfe7452b, SUC_lexicon_tags_7 },
    { 0x3aa7452e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38a74530, SUC_lexicon_tags_16 },
    { 0x2f474530, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a7453c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1674540, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9c74542, SUC_lexicon_tags_25 },
    { 0x8a074542, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7967454c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da7454f, SUC_lexicon_tags_7 },
    { 0xc5874550, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd8274552, SUC_lexicon_tags_31 },
    { 0xcb074552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c07455b, SUC_lexicon_tags_11 },
    { 0x907455c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba274561, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e74564, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5aa74566, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe74568, SUC_lexicon_tags_9 },
    { 0x1c074569, SUC_lexicon_tags_60 },
    { 0x60a7456a, SUC_lexicon_tags_7 },
    { 0xf067456b, SUC_lexicon_tags_11 },
    { 0x3f27456a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd487456e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7474573, SUC_lexicon_tags_17 },
    { 0x7f874573, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36074577, SUC_lexicon_tags_45 },
    { 0x58274577, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a7457d, SUC_lexicon_tags_17 },
    { 0xa567457e, SUC_lexicon_tags_10 },
    { 0xcca7457e, SUC_lexicon_tags_27 },
    { 0xdea7457e, SUC_lexicon_tags_23 },
    { 0x15474581, SUC_lexicon_tags_31 },
    { 0xd2a74582, SUC_lexicon_tags_49 },
    { 0x12074581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae874587, SUC_lexicon_tags_11 },
    { 0xfc474588, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9667458a, SUC_lexicon_tags_1 },
    { 0xe1c7458b, SUC_lexicon_tags_33 },
    { 0x85a7458b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda274591, SUC_lexicon_tags_14 },
    { 0x36274591, SUC_lexicon_tags_17 },
    { 0x5be74592, SUC_lexicon_tags_23 },
    { 0x4fa74594, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d074596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b27459c, SUC_lexicon_tags_85 },
    { 0x34e7459d, SUC_lexicon_tags_70 },
    { 0x62c7459e, SUC_lexicon_tags_7 },
    { 0xb827459f, SUC_lexicon_tags_17 },
    { 0x77c745a0, SUC_lexicon_tags_45 },
    { 0x1d6745a0, SUC_lexicon_tags_16 },
    { 0xcd2745a2, SUC_lexicon_tags_1 },
    { 0x678745a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e745a8, SUC_lexicon_tags_2 },
    { 0x1ce745a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe745ac, SUC_lexicon_tags_10 },
    { 0xb6c745ad, SUC_lexicon_tags_37 },
    { 0x114745ae, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06745b3, SUC_lexicon_tags_6 },
    { 0x592745b4, SUC_lexicon_tags_0 },
    { 0x572745b5, SUC_lexicon_tags_14 },
    { 0x850745b4, SUC_lexicon_tags_1 },
    { 0xe4e745b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2745ba, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xf60745bc, SUC_lexicon_tags_2 },
    { 0xf00745bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30745c4, SUC_lexicon_tags_98 },
    { 0x956745c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5da745c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4745ca, SUC_lexicon_tags_47 },
    { 0x2b8745ca, SUC_lexicon_tags_0 },
    { 0xeb0745cc, SUC_lexicon_tags_49 },
    { 0xb82745cc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x58c745cf, SUC_lexicon_tags_2 },
    { 0x782745cf, SUC_lexicon_tags_84 },
    { 0x7c4745d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0745d5, SUC_lexicon_tags_26 },
    { 0x340745d5, SUC_lexicon_tags_2 },
    { 0xcbc745d5, SUC_lexicon_tags_0 },
    { 0x9e2745d8, SUC_lexicon_tags_11 },
    { 0x532745d9, SUC_lexicon_tags_4 },
    { 0x128745da, SUC_lexicon_tags_11 },
    { 0x446745d8, SUC_lexicon_tags_11 },
    { 0x920745dc, SUC_lexicon_tags_17 },
    { 0x6c0745dd, SUC_lexicon_tags_15 },
    { 0xc30745d8, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac745e1, SUC_lexicon_tags_14 },
    { 0xfe0745e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x210745e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9be745e7, SUC_lexicon_tags_2 },
    { 0xa6c745e8, SUC_lexicon_tags_57 },
    { 0x8d8745e9, SUC_lexicon_tags_170 },
    { 0xef0745ea, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa4c745ec, SUC_lexicon_tags_25 },
    { 0x8b8745ed, SUC_lexicon_tags_23 },
    { 0x1b4745ee, SUC_lexicon_tags_8 },
    { 0x50a745ef, SUC_lexicon_tags_29 },
    { 0x4cc745f0, SUC_lexicon_tags_43 },
    { 0x464745ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e745f8, SUC_lexicon_tags_27 },
    { 0x420745f9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6745fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcba745fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ea74600, SUC_lexicon_tags_60 },
    { 0x20474600, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f474609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b27460c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e7460e, SUC_lexicon_tags_1 },
    { 0xa447460e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54074613, SUC_lexicon_tags_23 },
    { 0x18874614, SUC_lexicon_tags_11 },
    { 0xac274614, SUC_lexicon_tags_7 },
    { 0xafe74615, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x32074619, SUC_lexicon_tags_49 },
    { 0xc107461a, SUC_lexicon_tags_17 },
    { 0xb207461b, SUC_lexicon_tags_37 },
    { 0x2967461a, SUC_lexicon_tags_29 },
    { 0x95e7461c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc74623, SUC_lexicon_tags_26 },
    { 0xb0e74624, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a74628, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd467462a, SUC_lexicon_tags_25 },
    { 0xc1e7462b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xae47462d, SUC_lexicon_tags_6 },
    { 0x7507462e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d47463a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3874644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e074648, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6187464a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5074651, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37274658, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d27465b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72474660, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c74666, SUC_lexicon_tags_10 },
    { 0x45c74667, SUC_lexicon_tags_14 },
    { 0x8b074666, SUC_lexicon_tags_136 },
    { 0x8e474669, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6707466c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7727466f, SUC_lexicon_tags_47 },
    { 0x9ee74670, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c874672, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e74677, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f27467a, SUC_lexicon_tags_32 },
    { 0x3027467a, SUC_lexicon_tags_63 },
    { 0x7a7467a, SUC_lexicon_tags_1 },
    { 0x9be7467d, SUC_lexicon_tags_11 },
    { 0x2e87467e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47074683, SUC_lexicon_tags_4 },
    { 0xd6674684, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8274688, SUC_lexicon_tags_7 },
    { 0xdee74688, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7887468c, SUC_lexicon_tags_32 },
    { 0xb3e7468c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21074690, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3674694, SUC_lexicon_tags_23 },
    { 0x58e74694, SUC_lexicon_tags_0 },
    { 0xdea74695, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xdd674698, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeba7469a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990746a2, SUC_lexicon_tags_26 },
    { 0x478746a2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0746a7, SUC_lexicon_tags_31 },
    { 0xd54746a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a746ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x214746ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5c746b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224746b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xee6746b6, SUC_lexicon_tags_63 },
    { 0xdc4746b7, SUC_lexicon_tags_1 },
    { 0x2da746b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e746bb, SUC_lexicon_tags_43 },
    { 0x67e746bb, SUC_lexicon_tags_21 },
    { 0xd06746bc, SUC_lexicon_tags_46 },
    { 0x5e6746bc, SUC_lexicon_tags_1 },
    { 0x322746bc, SUC_lexicon_tags_25 },
    { 0xae8746c0, SUC_lexicon_tags_16 },
    { 0x6cc746c1, SUC_lexicon_tags_15 },
    { 0xec6746bb, SUC_lexicon_tags_23 },
    { 0x846746c3, SUC_lexicon_tags_15 },
    { 0xc56746c4, SUC_lexicon_tags_13 },
    { 0xccc746c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e746c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2746cb, SUC_lexicon_tags_15 },
    { 0x154746cc, SUC_lexicon_tags_29 },
    { 0x50746cd, SUC_lexicon_tags_0 },
    { 0x54746cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516746d7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba746da, SUC_lexicon_tags_7 },
    { 0xbee746da, SUC_lexicon_tags_123 },
    { 0xc04746db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06746e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4746e3, SUC_lexicon_tags_2 },
    { 0x45c746e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x596746e6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7c4746e8, SUC_lexicon_tags_15 },
    { 0x4fe746e9, SUC_lexicon_tags_24 },
    { 0x58e746ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a746ed, SUC_lexicon_tags_13 },
    { 0xb1e746ee, SUC_lexicon_tags_44 },
    { 0xd60746ef, SUC_lexicon_tags_2 },
    { 0x9ba746ee, SUC_lexicon_tags_136 },
    { 0xbfa746f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ee746f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6746f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02746f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cc746fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x186746fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9674705, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6c74707, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c07470b, SUC_lexicon_tags_13 },
    { 0xbc07470c, SUC_lexicon_tags_113 },
    { 0x8a7470d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe07470f, SUC_lexicon_tags_37 },
    { 0xfba74710, SUC_lexicon_tags_25 },
    { 0x47a74710, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfc74713, SUC_lexicon_tags_4 },
    { 0xd2274713, SUC_lexicon_tags_3 },
    { 0x5ca74715, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5674717, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ec74719, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4367471c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd07471e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4874726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55674728, SUC_lexicon_tags_15 },
    { 0x32074729, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb87472c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf967472f, SUC_lexicon_tags_10 },
    { 0x80e74730, SUC_lexicon_tags_15 },
    { 0x91474731, SUC_lexicon_tags_23 },
    { 0x89e74731, SUC_lexicon_tags_9 },
    { 0x7f474733, SUC_lexicon_tags_10 },
    { 0xabe74734, SUC_lexicon_tags_22 },
    { 0xbfc74733, SUC_lexicon_tags_3 },
    { 0xa9c74736, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5447473b, SUC_lexicon_tags_32 },
    { 0x3d67473c, SUC_lexicon_tags_46 },
    { 0x9fe7473d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25674745, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2474747, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0a74749, SUC_lexicon_tags_7 },
    { 0xdbe74749, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda07474f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b074752, SUC_lexicon_tags_30 },
    { 0xffc74753, SUC_lexicon_tags_29 },
    { 0xeac74753, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e74758, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5947475a, SUC_lexicon_tags_33 },
    { 0x87c7475b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa74763, SUC_lexicon_tags_0 },
    { 0x4c74764, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf27476b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0274774, SUC_lexicon_tags_11 },
    { 0xa5674774, SUC_lexicon_tags_12 },
    { 0x36c74776, SUC_lexicon_tags_29 },
    { 0x75874777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66674779, SUC_lexicon_tags_37 },
    { 0x4ac7477a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd87477d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e874782, SUC_lexicon_tags_15 },
    { 0xba274783, SUC_lexicon_tags_26 },
    { 0x3874784, SUC_lexicon_tags_7 },
    { 0xc874785, SUC_lexicon_tags_33 },
    { 0xb8874786, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7487478b, SUC_lexicon_tags_9 },
    { 0x1527478c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xede7478e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6074792, SUC_lexicon_tags_7 },
    { 0xfb074793, SUC_lexicon_tags_25 },
    { 0xc2e74794, SUC_lexicon_tags_29 },
    { 0xfb674795, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac27479a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8747479c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6127479e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc50747a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcec747a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb2c747a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2747ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ac747af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa8747b1, SUC_lexicon_tags_11 },
    { 0xf7e747b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938747b6, SUC_lexicon_tags_536 },
    { 0x5aa747b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee747ba, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x720747bd, SUC_lexicon_tags_11 },
    { 0x702747be, SUC_lexicon_tags_634 },
    { 0x9bc747be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88c747c1, SUC_lexicon_tags_15 },
    { 0xc64747c1, SUC_lexicon_tags_14 },
    { 0x362747c3, SUC_lexicon_tags_55 },
    { 0x2fa747c3, SUC_lexicon_tags_17 },
    { 0x578747c5, SUC_lexicon_tags_37 },
    { 0x5d6747c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644747cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe747cf, SUC_lexicon_tags_26 },
    { 0x17a747d0, SUC_lexicon_tags_17 },
    { 0x1f2747d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf2e747d3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a747d9, SUC_lexicon_tags_0 },
    { 0xa40747da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43a747dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86747e0, SUC_lexicon_tags_30 },
    { 0xb2e747e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c4747e3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6dc747e5, SUC_lexicon_tags_11 },
    { 0x80747e6, SUC_lexicon_tags_63 },
    { 0x594747e7, SUC_lexicon_tags_17 },
    { 0x9bc747e6, SUC_lexicon_tags_11 },
    { 0x4e6747e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30747f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940747f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a747f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c747fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd50747fd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e74804, SUC_lexicon_tags_12 },
    { 0x44e74805, SUC_lexicon_tags_2 },
    { 0x60074806, SUC_lexicon_tags_46 },
    { 0xdea74807, SUC_lexicon_tags_7 },
    { 0xed874807, SUC_lexicon_tags_25 },
    { 0xf6a74809, SUC_lexicon_tags_33 },
    { 0xc8674805, SUC_lexicon_tags_17 },
    { 0xcdc74808, SUC_lexicon_tags_45 },
    { 0xa9a7480a, SUC_lexicon_tags_26 },
    { 0xfb274806, SUC_lexicon_tags_2 },
    { 0xda674808, SUC_lexicon_tags_17 },
    { 0xf947480c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae74814, SUC_lexicon_tags_30 },
    { 0x39074815, SUC_lexicon_tags_43 },
    { 0x1a074816, SUC_lexicon_tags_37 },
    { 0xc6074816, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd227481a, SUC_lexicon_tags_14 },
    { 0x6c47481a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd327481f, SUC_lexicon_tags_1 },
    { 0x17074820, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54074826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce7482b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa87482d, SUC_lexicon_tags_47 },
    { 0x1767482e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26474837, SUC_lexicon_tags_31 },
    { 0xe274838, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a47483b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ec7483d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x87c7483f, SUC_lexicon_tags_25 },
    { 0xa1a7483f, SUC_lexicon_tags_21 },
    { 0xb747483f, SUC_lexicon_tags_17 },
    { 0x29074840, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97074846, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7cc74848, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f67484c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9d27484e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17874853, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xa3874855, SUC_lexicon_tags_17 },
    { 0x20874855, SUC_lexicon_tags_26 },
    { 0xbb074855, SUC_lexicon_tags_11 },
    { 0x25474856, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c27485d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x40e7485f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c74863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c74867, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8987486a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70c7486c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a07486e, SUC_lexicon_tags_11 },
    { 0x1247486e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d874872, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdc474874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a7487a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da74881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3874885, SUC_lexicon_tags_25 },
    { 0x21a74886, SUC_lexicon_tags_49 },
    { 0x5ec74885, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5c274889, SUC_lexicon_tags_47 },
    { 0x5ce7488a, SUC_lexicon_tags_28 },
    { 0x90c7488b, SUC_lexicon_tags_25 },
    { 0xbe87488b, SUC_lexicon_tags_2 },
    { 0x2667488d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x57674890, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c74896, SUC_lexicon_tags_4 },
    { 0x39674897, SUC_lexicon_tags_29 },
    { 0x33874898, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38c7489a, SUC_lexicon_tags_33 },
    { 0x7867489b, SUC_lexicon_tags_14 },
    { 0xeea7489c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6748a7, SUC_lexicon_tags_11 },
    { 0xdec748a7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x8fc748aa, SUC_lexicon_tags_17 },
    { 0x9bc748ab, SUC_lexicon_tags_8 },
    { 0x950748ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734748b1, SUC_lexicon_tags_101 },
    { 0x17c748b2, SUC_lexicon_tags_9 },
    { 0x2a6748b3, SUC_lexicon_tags_30 },
    { 0x7ea748b4, SUC_lexicon_tags_69 },
    { 0xcb0748b5, SUC_lexicon_tags_11 },
    { 0xe4a748b4, SUC_lexicon_tags_63 },
    { 0xf0748b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a748bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66748c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50c748c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x110748c5, SUC_lexicon_tags_28 },
    { 0xb88748c5, SUC_lexicon_tags_37 },
    { 0x4be748c7, SUC_lexicon_tags_25 },
    { 0x244748c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33e748ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c748d2, SUC_lexicon_tags_53 },
    { 0xe28748d3, SUC_lexicon_tags_33 },
    { 0x39a748d4, SUC_lexicon_tags_2 },
    { 0x598748d5, SUC_lexicon_tags_10 },
    { 0xf78748d2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6bc748d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x196748da, SUC_lexicon_tags_39 },
    { 0x7a6748db, SUC_lexicon_tags_17 },
    { 0x908748dc, SUC_lexicon_tags_81 },
    { 0xfda748db, SUC_lexicon_tags_12 },
    { 0x4ca748de, SUC_lexicon_tags_12 },
    { 0xdaa748de, SUC_lexicon_tags_19 },
    { 0xc20748e0, SUC_lexicon_tags_47 },
    { 0x280748e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2f2748e3, SUC_lexicon_tags_12 },
    { 0x93c748e4, SUC_lexicon_tags_62 },
    { 0x1c748e3, SUC_lexicon_tags_2 },
    { 0x2748e3, SUC_lexicon_tags_38 },
    { 0x5a2748e7, SUC_lexicon_tags_31 },
    { 0x778748e8, SUC_lexicon_tags_4 },
    { 0x25e748e9, SUC_lexicon_tags_17 },
    { 0x480748e9, SUC_lexicon_tags_45 },
    { 0x43c748eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x840748ee, SUC_lexicon_tags_185 },
    { 0x6d4748ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ec748f1, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x76e748f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e6748f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c748f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81074900, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a74903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca74906, SUC_lexicon_tags_11 },
    { 0x30a74906, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f87490a, SUC_lexicon_tags_6 },
    { 0x6ca7490b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c74910, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x38674912, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d674915, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f074918, SUC_lexicon_tags_12 },
    { 0x4ea74919, SUC_lexicon_tags_7 },
    { 0x7c87491a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a7491d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf874921, SUC_lexicon_tags_17 },
    { 0x9e474921, SUC_lexicon_tags_1 },
    { 0x6fa74923, SUC_lexicon_tags_29 },
    { 0x91274923, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf874926, SUC_lexicon_tags_17 },
    { 0x5b674927, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82e74929, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd67492c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c7492f, SUC_lexicon_tags_27 },
    { 0xa4074930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e74933, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33c74935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a7493d, SUC_lexicon_tags_15 },
    { 0xc7c7493e, SUC_lexicon_tags_63 },
    { 0xe5c7493f, SUC_lexicon_tags_30 },
    { 0x37e74940, SUC_lexicon_tags_60 },
    { 0x3ca74941, SUC_lexicon_tags_23 },
    { 0xc74942, SUC_lexicon_tags_12 },
    { 0x2d874941, SUC_lexicon_tags_21 },
    { 0x8a07493d, SUC_lexicon_tags_1 },
    { 0xbca74945, SUC_lexicon_tags_14 },
    { 0x63474943, SUC_lexicon_tags_27 },
    { 0xaac7493f, SUC_lexicon_tags_32 },
    { 0x49a74948, SUC_lexicon_tags_63 },
    { 0x1ca74940, SUC_lexicon_tags_15 },
    { 0x8107494a, SUC_lexicon_tags_43 },
    { 0xb587494b, SUC_lexicon_tags_28 },
    { 0x7707494c, SUC_lexicon_tags_234 },
    { 0x7de7494d, SUC_lexicon_tags_7 },
    { 0x4607493f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40474950, SUC_lexicon_tags_9 },
    { 0xb874950, SUC_lexicon_tags_207 },
    { 0xc6074952, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5274955, SUC_lexicon_tags_17 },
    { 0xbe474956, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc74959, SUC_lexicon_tags_215 },
    { 0xe2c7495a, SUC_lexicon_tags_37 },
    { 0x8a7495a, SUC_lexicon_tags_9 },
    { 0xd767495c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea7495e, SUC_lexicon_tags_39 },
    { 0xace7495f, SUC_lexicon_tags_49 },
    { 0xfa474960, SUC_lexicon_tags_7 },
    { 0xf787495e, SUC_lexicon_tags_36 },
    { 0x89674962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c7496a, SUC_lexicon_tags_11 },
    { 0x7627496b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a7496f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17674972, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcd874974, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea74977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5267497a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c7497e, SUC_lexicon_tags_2 },
    { 0x64a7497e, SUC_lexicon_tags_13 },
    { 0x6967497f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a74983, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe5074985, SUC_lexicon_tags_2 },
    { 0x84474986, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe0a74988, SUC_lexicon_tags_19 },
    { 0xf8074989, SUC_lexicon_tags_11 },
    { 0xe7c74989, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb67498d, SUC_lexicon_tags_11 },
    { 0xb827498e, SUC_lexicon_tags_63 },
    { 0xd607498e, SUC_lexicon_tags_1 },
    { 0x2887498d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9c74992, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa7499e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2749a1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xce6749a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x468749a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea749ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724749b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1e749b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7c749b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x876749b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe6749bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2749be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2749c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e749c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5a749c8, SUC_lexicon_tags_1 },
    { 0x67c749c9, SUC_lexicon_tags_38 },
    { 0x3c8749ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2749ce, SUC_lexicon_tags_7 },
    { 0xde749cf, SUC_lexicon_tags_14 },
    { 0x38e749cf, SUC_lexicon_tags_17 },
    { 0x892749d0, SUC_lexicon_tags_14 },
    { 0x842749d2, SUC_lexicon_tags_1 },
    { 0xe54749ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xea2749d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6749d8, SUC_lexicon_tags_13 },
    { 0x7e6749d9, SUC_lexicon_tags_9 },
    { 0x926749d9, SUC_lexicon_tags_17 },
    { 0x1b0749d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38749e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8a749e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a749e8, SUC_lexicon_tags_2 },
    { 0x4be749e8, SUC_lexicon_tags_32 },
    { 0x736749ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2749ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc749f7, SUC_lexicon_tags_1 },
    { 0xee6749f8, SUC_lexicon_tags_12 },
    { 0x5b2749f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4749fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec74a00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59874a02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a74a09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c74a0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe674a11, SUC_lexicon_tags_38 },
    { 0xeb874a12, SUC_lexicon_tags_15 },
    { 0xccc74a13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfca74a15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c74a1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c674a1d, SUC_lexicon_tags_7 },
    { 0x5fe74a1d, SUC_lexicon_tags_30 },
    { 0x18874a1d, SUC_lexicon_tags_6 },
    { 0x57874a20, SUC_lexicon_tags_47 },
    { 0x33874a20, SUC_lexicon_tags_10 },
    { 0x4e874a20, SUC_lexicon_tags_2 },
    { 0x21674a20, SUC_lexicon_tags_23 },
    { 0xe6a74a24, SUC_lexicon_tags_1 },
    { 0xc4274a24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x83474a27, SUC_lexicon_tags_63 },
    { 0xb5874a27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e474a2c, SUC_lexicon_tags_47 },
    { 0x36474a2d, SUC_lexicon_tags_23 },
    { 0x1474a2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4674a30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x14074a32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7074a35, SUC_lexicon_tags_49 },
    { 0xb4a74a36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e74a39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x44874a3c, SUC_lexicon_tags_14 },
    { 0xae74a3c, SUC_lexicon_tags_17 },
    { 0xe7674a3e, SUC_lexicon_tags_7 },
    { 0xe7a74a3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xce474a42, SUC_lexicon_tags_15 },
    { 0x17a74a43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9074a45, SUC_lexicon_tags_12 },
    { 0xb7274a46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0674a4a, SUC_lexicon_tags_11 },
    { 0x12274a4a, SUC_lexicon_tags_13 },
    { 0xa7c74a4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4aa74a4e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e74a53, SUC_lexicon_tags_11 },
    { 0x73e74a53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe74a59, SUC_lexicon_tags_17 },
    { 0xe074a5a, SUC_lexicon_tags_11 },
    { 0x8a674a5b, SUC_lexicon_tags_114 },
    { 0xf074a59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17874a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b674a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54074a6e, SUC_lexicon_tags_2 },
    { 0xd6674a6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc074a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf674a75, SUC_lexicon_tags_39 },
    { 0xdd274a75, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f074a7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2a74a7e, SUC_lexicon_tags_15 },
    { 0x4da74a7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e74a83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44474a87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71674a92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d274a94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8474a97, SUC_lexicon_tags_3 },
    { 0xcda74a98, SUC_lexicon_tags_17 },
    { 0xfc874a99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce874a9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46874aa5, SUC_lexicon_tags_14 },
    { 0xf4e74aa6, SUC_lexicon_tags_17 },
    { 0xfd274aa7, SUC_lexicon_tags_26 },
    { 0xe3474aa8, SUC_lexicon_tags_37 },
    { 0xbb074aa6, SUC_lexicon_tags_25 },
    { 0x27474aa8, SUC_lexicon_tags_12 },
    { 0x92474aab, SUC_lexicon_tags_49 },
    { 0x75274aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9c74aae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a474ab0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf874ab2, SUC_lexicon_tags_2 },
    { 0x30074ab2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e74ab6, SUC_lexicon_tags_47 },
    { 0x7ea74ab6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79674ab9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f874abf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca74ac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae74acb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5c74acd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60674acf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfd474ad1, SUC_lexicon_tags_25 },
    { 0x82674ad2, SUC_lexicon_tags_37 },
    { 0xee274ad1, SUC_lexicon_tags_1 },
    { 0xf3e74ad4, SUC_lexicon_tags_15 },
    { 0x47074ad4, SUC_lexicon_tags_17 },
    { 0x58474ad6, SUC_lexicon_tags_15 },
    { 0xeec74ad5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c674ad9, SUC_lexicon_tags_2 },
    { 0x5a074ad9, SUC_lexicon_tags_21 },
    { 0x9ea74ad9, SUC_lexicon_tags_10 },
    { 0xcb074adc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d474ae0, SUC_lexicon_tags_6 },
    { 0xf6274ae1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33874ae3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1474ae8, SUC_lexicon_tags_14 },
    { 0xc5c74ae8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb5c74aeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47074aef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x24a74af1, SUC_lexicon_tags_1 },
    { 0x39874af2, SUC_lexicon_tags_25 },
    { 0x16074af2, SUC_lexicon_tags_37 },
    { 0xba274af4, SUC_lexicon_tags_47 },
    { 0x95874af5, SUC_lexicon_tags_60 },
    { 0xf2674af6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3674afa, SUC_lexicon_tags_17 },
    { 0xb0c74afa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85674afe, SUC_lexicon_tags_27 },
    { 0xc0c74aff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57074b06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e874b0c, SUC_lexicon_tags_57 },
    { 0x5c474b0d, SUC_lexicon_tags_9 },
    { 0x31074b0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b474b11, SUC_lexicon_tags_56 },
    { 0xaea74b12, SUC_lexicon_tags_7 },
    { 0x2c74b11, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc74b19, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e074b1c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84674b25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ac74b27, SUC_lexicon_tags_27 },
    { 0xb3874b27, SUC_lexicon_tags_26 },
    { 0xc1a74b29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81e74b2b, SUC_lexicon_tags_63 },
    { 0xb7c74b2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9874b31, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c474b35, SUC_lexicon_tags_29 },
    { 0x96a74b36, SUC_lexicon_tags_37 },
    { 0xf7c74b37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9d674b39, SUC_lexicon_tags_2 },
    { 0x44674b39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc874b3e, SUC_lexicon_tags_21 },
    { 0x37a74b3f, SUC_lexicon_tags_0 },
    { 0xcf874b40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8074b44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e074b46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf274b4a, SUC_lexicon_tags_27 },
    { 0x7b474b4b, SUC_lexicon_tags_15 },
    { 0x90a74b4c, SUC_lexicon_tags_1 },
    { 0x59274b4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed274b4f, SUC_lexicon_tags_17 },
    { 0x28a74b4f, SUC_lexicon_tags_56 },
    { 0xec674b4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde74b5a, SUC_lexicon_tags_12 },
    { 0xd0674b5b, SUC_lexicon_tags_37 },
    { 0x71074b5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x23474b5f, SUC_lexicon_tags_60 },
    { 0x62674b60, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea74b63, SUC_lexicon_tags_14 },
    { 0x28874b63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52474b67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c74b6d, SUC_lexicon_tags_25 },
    { 0x9f074b6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4ce74b70, SUC_lexicon_tags_0 },
    { 0xe0c74b71, SUC_lexicon_tags_7 },
    { 0x46c74b72, SUC_lexicon_tags_11 },
    { 0x27074b70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa74b77, SUC_lexicon_tags_37 },
    { 0x13874b77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd5674b7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd674b7f, SUC_lexicon_tags_11 },
    { 0x6ec74b80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd674b82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b874b85, SUC_lexicon_tags_37 },
    { 0xdf074b86, SUC_lexicon_tags_7 },
    { 0xf8c74b86, SUC_lexicon_tags_29 },
    { 0x92a74b88, SUC_lexicon_tags_16 },
    { 0x4cc74b89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34274b8d, SUC_lexicon_tags_27 },
    { 0xcb674b8e, SUC_lexicon_tags_28 },
    { 0xa4674b8f, SUC_lexicon_tags_15 },
    { 0xf6a74b90, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75674b94, SUC_lexicon_tags_17 },
    { 0xb074b95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x95674b97, SUC_lexicon_tags_80 },
    { 0xa3274b97, SUC_lexicon_tags_30 },
    { 0x7bc74b99, SUC_lexicon_tags_6 },
    { 0x77674b9a, SUC_lexicon_tags_31 },
    { 0xb4874b9b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99074ba3, SUC_lexicon_tags_15 },
    { 0x5dc74ba3, SUC_lexicon_tags_48 },
    { 0xab874ba5, SUC_lexicon_tags_46 },
    { 0x3674ba6, SUC_lexicon_tags_37 },
    { 0x88e74ba7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc2274ba9, SUC_lexicon_tags_12 },
    { 0x3274baa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x34e74bac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a874bae, SUC_lexicon_tags_9 },
    { 0x82674bae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88e74bb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x73e74bb3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7274bb5, SUC_lexicon_tags_70 },
    { 0x10a74bb6, SUC_lexicon_tags_46 },
    { 0xf0a74bb5, SUC_lexicon_tags_2 },
    { 0x3b674bb8, SUC_lexicon_tags_25 },
    { 0xef674bb9, SUC_lexicon_tags_11 },
    { 0x80674bba, SUC_lexicon_tags_17 },
    { 0x6c074bbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86074bbd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49874bc3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x80e74bc5, SUC_lexicon_tags_31 },
    { 0x47474bc6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1074bcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc74bce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74474bd7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6ac74bd9, SUC_lexicon_tags_1 },
    { 0x8be74bd9, SUC_lexicon_tags_14 },
    { 0x3fa74bdb, SUC_lexicon_tags_23 },
    { 0x28a74bdc, SUC_lexicon_tags_21 },
    { 0xb4c74bdb, SUC_lexicon_tags_17 },
    { 0xf8474bde, SUC_lexicon_tags_14 },
    { 0xcec74bdf, SUC_lexicon_tags_4 },
    { 0x89674be0, SUC_lexicon_tags_14 },
    { 0x7b274be0, SUC_lexicon_tags_37 },
    { 0x1e074be1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xad874be5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a74be8, SUC_lexicon_tags_114 },
    { 0xb9e74be9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f074bef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8274bf6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e74bfa, SUC_lexicon_tags_10 },
    { 0x7c274bfb, SUC_lexicon_tags_68 },
    { 0x55874bfc, SUC_lexicon_tags_17 },
    { 0xd0c74bfd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba74c01, SUC_lexicon_tags_47 },
    { 0x40074c02, SUC_lexicon_tags_2 },
    { 0x4a074c01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae474c07, SUC_lexicon_tags_27 },
    { 0x51474c07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5474c0e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce274c12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74474c16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c74c19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61474c1d, SUC_lexicon_tags_15 },
    { 0xecc74c1e, SUC_lexicon_tags_46 },
    { 0xac474c1e, SUC_lexicon_tags_63 },
    { 0xa8874c20, SUC_lexicon_tags_25 },
    { 0x4ae74c1f, SUC_lexicon_tags_19 },
    { 0xbd274c21, SUC_lexicon_tags_1 },
    { 0x15e74c23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37274c27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x62c74c29, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x81674c2c, SUC_lexicon_tags_37 },
    { 0x93674c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f674c31, SUC_lexicon_tags_74 },
    { 0xe0674c32, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf0e74c34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a74c37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa74c3a, SUC_lexicon_tags_26 },
    { 0xc5c74c3b, SUC_lexicon_tags_45 },
    { 0x2f074c3b, SUC_lexicon_tags_43 },
    { 0xb4c74c3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc74c40, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46474c4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x92074c4d, SUC_lexicon_tags_17 },
    { 0x4ec74c4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0074c51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f074c54, SUC_lexicon_tags_15 },
    { 0xc8a74c54, SUC_lexicon_tags_0 },
    { 0xe7674c55, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcb474c58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e874c60, SUC_lexicon_tags_11 },
    { 0xe7274c61, SUC_lexicon_tags_15 },
    { 0x53e74c62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6074c65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc74c69, SUC_lexicon_tags_37 },
    { 0x7474c69, SUC_lexicon_tags_25 },
    { 0x65e74c6a, SUC_lexicon_tags_23 },
    { 0x6de74c6c, SUC_lexicon_tags_7 },
    { 0xc3c74c6d, SUC_lexicon_tags_63 },
    { 0x74c74c6d, SUC_lexicon_tags_23 },
    { 0xbfe74c6f, SUC_lexicon_tags_47 },
    { 0x15874c6e, SUC_lexicon_tags_17 },
    { 0x85074c6d, SUC_lexicon_tags_30 },
    { 0xe1e74c71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c74c7c, SUC_lexicon_tags_7 },
    { 0xe8274c7d, SUC_lexicon_tags_1 },
    { 0x92874c7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1874c81, SUC_lexicon_tags_13 },
    { 0x1ae74c81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b074c84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x59874c86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e474c88, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2474c8b, SUC_lexicon_tags_25 },
    { 0x2ea74c8c, SUC_lexicon_tags_2 },
    { 0x6ae74c8d, SUC_lexicon_tags_11 },
    { 0xee74c8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51674c93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3874c9a, SUC_lexicon_tags_35 },
    { 0x99274c9b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x9c674c9d, SUC_lexicon_tags_37 },
    { 0x4e274c9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a474ca3, SUC_lexicon_tags_141 },
    { 0x30474ca4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a74cae, SUC_lexicon_tags_10 },
    { 0xf1674caf, SUC_lexicon_tags_9 },
    { 0x50274caf, SUC_lexicon_tags_17 },
    { 0x31e74cb1, SUC_lexicon_tags_65 },
    { 0x7c274caf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8a74cb4, SUC_lexicon_tags_0 },
    { 0x37c74cb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4de74cb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95474cbb, SUC_lexicon_tags_0 },
    { 0xd0874cbc, SUC_lexicon_tags_16 },
    { 0x90474cbd, SUC_lexicon_tags_29 },
    { 0x6c874cbb, SUC_lexicon_tags_25 },
    { 0xda674cbf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4474cc3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4874cc7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5074cca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35674cd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ee74cd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1874cd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f274cd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66674cdc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a74ce1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc074ce4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c74ce7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c74cee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe74cf2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6c274cf4, SUC_lexicon_tags_15 },
    { 0x42474cf5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89c74cf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28274cfb, SUC_lexicon_tags_37 },
    { 0xdd274cfc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23674d01, SUC_lexicon_tags_0 },
    { 0x7d474d02, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x34c74d04, SUC_lexicon_tags_23 },
    { 0xcd874d05, SUC_lexicon_tags_37 },
    { 0x4c674d06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14674d10, SUC_lexicon_tags_17 },
    { 0x95c74d11, SUC_lexicon_tags_12 },
    { 0xaa874d11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34c74d14, SUC_lexicon_tags_47 },
    { 0xca474d14, SUC_lexicon_tags_8 },
    { 0x7f274d16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee74d1b, SUC_lexicon_tags_7 },
    { 0xbd474d1c, SUC_lexicon_tags_25 },
    { 0x78474d1b, SUC_lexicon_tags_0 },
    { 0xa3674d1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a074d21, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd7c74d23, SUC_lexicon_tags_45 },
    { 0xeac74d23, SUC_lexicon_tags_2 },
    { 0x7e874d25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbe074d27, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa74d2d, SUC_lexicon_tags_38 },
    { 0xbb474d2e, SUC_lexicon_tags_19 },
    { 0x9b874d2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81474d32, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad074d37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x27e74d39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29a74d3b, SUC_lexicon_tags_46 },
    { 0x44274d3c, SUC_lexicon_tags_12 },
    { 0x3fc74d3d, SUC_lexicon_tags_43 },
    { 0xc0474d3e, SUC_lexicon_tags_65 },
    { 0xfc74d3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44674d47, SUC_lexicon_tags_60 },
    { 0x12e74d48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaba74d4a, SUC_lexicon_tags_25 },
    { 0x87674d4a, SUC_lexicon_tags_33 },
    { 0x67274d4c, SUC_lexicon_tags_15 },
    { 0xa5074d4a, SUC_lexicon_tags_11 },
    { 0xb6074d4c, SUC_lexicon_tags_8 },
    { 0x42674d4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d274d59, SUC_lexicon_tags_27 },
    { 0x5874d59, SUC_lexicon_tags_106 },
    { 0x15074d5b, SUC_lexicon_tags_10 },
    { 0x57474d5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2e74d5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a74d64, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f474d68, SUC_lexicon_tags_37 },
    { 0xd9074d69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6a74d6b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3674d74, SUC_lexicon_tags_60 },
    { 0x41274d75, SUC_lexicon_tags_1 },
    { 0xcee74d74, SUC_lexicon_tags_17 },
    { 0xc6674d77, SUC_lexicon_tags_12 },
    { 0xe3c74d78, SUC_lexicon_tags_21 },
    { 0xc5e74d79, SUC_lexicon_tags_17 },
    { 0x20474d7a, SUC_lexicon_tags_49 },
    { 0xb1274d7a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd9474d7d, SUC_lexicon_tags_2 },
    { 0x4b274d7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb4c74d80, SUC_lexicon_tags_1 },
    { 0xdf474d81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd474d84, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa6874d86, SUC_lexicon_tags_27 },
    { 0x53874d87, SUC_lexicon_tags_5 },
    { 0x88274d88, SUC_lexicon_tags_7 },
    { 0x6f074d89, SUC_lexicon_tags_25 },
    { 0x8d674d8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf874d8f, SUC_lexicon_tags_26 },
    { 0x8c674d90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f674d92, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xca474d95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf1074d97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e74d9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb674da1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c74da6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96674daa, SUC_lexicon_tags_15 },
    { 0xc8474dab, SUC_lexicon_tags_18 },
    { 0x58674daa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90e74dae, SUC_lexicon_tags_19 },
    { 0xf9c74daf, SUC_lexicon_tags_65 },
    { 0x88274daf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a674db5, SUC_lexicon_tags_26 },
    { 0x1a74db6, SUC_lexicon_tags_56 },
    { 0x42c74db5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4ce74db9, SUC_lexicon_tags_21 },
    { 0xd0e74dba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c474dbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e74dbf, SUC_lexicon_tags_25 },
    { 0x68274dc0, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0xaaa74dc2, SUC_lexicon_tags_17 },
    { 0xe6074dc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6674dc5, SUC_lexicon_tags_45 },
    { 0xac274dc6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xae74dc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc74dcb, SUC_lexicon_tags_11 },
    { 0x14874dcc, SUC_lexicon_tags_29 },
    { 0x52274dcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2be74dcf, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x3d674dd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c74dd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x29c74dd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa74dd9, SUC_lexicon_tags_17 },
    { 0x15e74dda, SUC_lexicon_tags_14 },
    { 0xe9474ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b874de1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x12874de3, SUC_lexicon_tags_24 },
    { 0xcfe74de4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26e74de6, SUC_lexicon_tags_60 },
    { 0x77074de7, SUC_lexicon_tags_87 },
    { 0x25e74de8, SUC_lexicon_tags_46 },
    { 0xe9874de9, SUC_lexicon_tags_47 },
    { 0x3f274de9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7de74dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c074def, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc874df3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27874df5, SUC_lexicon_tags_68 },
    { 0xd5274df6, SUC_lexicon_tags_37 },
    { 0x5274df7, SUC_lexicon_tags_38 },
    { 0xd4674df8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7074dfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1874e00, SUC_lexicon_tags_12 },
    { 0x5dc74e01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8674e04, SUC_lexicon_tags_28 },
    { 0x13074e05, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa6074e07, SUC_lexicon_tags_25 },
    { 0xed674e08, SUC_lexicon_tags_113 },
    { 0xefe74e07, SUC_lexicon_tags_59 },
    { 0xc9e74e0a, SUC_lexicon_tags_15 },
    { 0xbc074e07, SUC_lexicon_tags_45 },
    { 0x32a74e09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c274e10, SUC_lexicon_tags_43 },
    { 0x59274e11, SUC_lexicon_tags_11 },
    { 0x46674e12, SUC_lexicon_tags_11 },
    { 0xaf474e13, SUC_lexicon_tags_30 },
    { 0x56c74e10, SUC_lexicon_tags_25 },
    { 0xf4e74e10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c74e18, SUC_lexicon_tags_37 },
    { 0x62474e19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40074e1c, SUC_lexicon_tags_17 },
    { 0x41c74e1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x49074e1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66074e21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a74e24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc74e29, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c474e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53674e32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73674e37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc74e3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca74e3e, SUC_lexicon_tags_49 },
    { 0x3a274e3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac074e43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0074e45, SUC_lexicon_tags_45 },
    { 0x77274e46, SUC_lexicon_tags_15 },
    { 0x51674e47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c74e4c, SUC_lexicon_tags_8 },
    { 0xd3274e4d, SUC_lexicon_tags_7 },
    { 0x18674e4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8874e51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46274e54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b074e56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd074e58, SUC_lexicon_tags_7 },
    { 0x42a74e58, SUC_lexicon_tags_61 },
    { 0x22a74e58, SUC_lexicon_tags_12 },
    { 0x4cc74e5b, SUC_lexicon_tags_25 },
    { 0x9b874e5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a674e61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb6e74e63, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84074e6c, SUC_lexicon_tags_37 },
    { 0x86274e6d, SUC_lexicon_tags_55 },
    { 0x40c74e6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16674e74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x67074e76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d474e79, SUC_lexicon_tags_2 },
    { 0x32c74e7a, SUC_lexicon_tags_5 },
    { 0xed074e7b, SUC_lexicon_tags_25 },
    { 0x49674e79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a674e80, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x12274e83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77674e88, SUC_lexicon_tags_1 },
    { 0x54274e89, SUC_lexicon_tags_19 },
    { 0xd8074e8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb674e8d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed674e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabe74e93, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9674e95, SUC_lexicon_tags_27 },
    { 0xcba74e96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c674e98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c074e9b, SUC_lexicon_tags_1 },
    { 0x28e74e9c, SUC_lexicon_tags_0 },
    { 0xaa674e9d, SUC_lexicon_tags_7 },
    { 0x80874e9d, SUC_lexicon_tags_45 },
    { 0xd6e74e9f, SUC_lexicon_tags_20 },
    { 0x4e674e9e, SUC_lexicon_tags_2 },
    { 0xff074ea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a74ea4, SUC_lexicon_tags_14 },
    { 0xd7274ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6874ea7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd3e74ea9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d074eab, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe1274ead, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76274eb0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f074eb4, SUC_lexicon_tags_148 },
    { 0x2f674eb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f474eb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b474eba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd474ec1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc74ec6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92474ec8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21074ed2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce74ed5, SUC_lexicon_tags_60 },
    { 0xa674ed6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5274ed9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e74edb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a74ede, SUC_lexicon_tags_56 },
    { 0x6ac74ede, SUC_lexicon_tags_57 },
    { 0x89074ee0, SUC_lexicon_tags_27 },
    { 0x97474ee0, SUC_lexicon_tags_59 },
    { 0x22e74ee2, SUC_lexicon_tags_11 },
    { 0x70c74ee1, SUC_lexicon_tags_29 },
    { 0x67e74ee4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x98674ee7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a74eed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd674eef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb4874ef1, SUC_lexicon_tags_37 },
    { 0xb9a74ef1, SUC_lexicon_tags_68 },
    { 0x9ee74ef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62474ef7, SUC_lexicon_tags_37 },
    { 0x74c74ef8, SUC_lexicon_tags_43 },
    { 0xfc74ef9, SUC_lexicon_tags_84 },
    { 0xe1074ef8, SUC_lexicon_tags_15 },
    { 0xff474efb, SUC_lexicon_tags_2 },
    { 0xa2874ef8, SUC_lexicon_tags_3 },
    { 0xbda74efc, SUC_lexicon_tags_17 },
    { 0x7c274efb, SUC_lexicon_tags_46 },
    { 0x2be74eff, SUC_lexicon_tags_28 },
    { 0x24a74f00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f874f07, SUC_lexicon_tags_15 },
    { 0x16c74f08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d474f0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe874f0d, SUC_lexicon_tags_15 },
    { 0x39474f0d, SUC_lexicon_tags_2 },
    { 0x36474f0f, SUC_lexicon_tags_25 },
    { 0xbac74f10, SUC_lexicon_tags_12 },
    { 0x55a74f11, SUC_lexicon_tags_9 },
    { 0x66e74f12, SUC_lexicon_tags_107 },
    { 0x68674f11, SUC_lexicon_tags_37 },
    { 0xa1074f14, SUC_lexicon_tags_60 },
    { 0x59474f15, SUC_lexicon_tags_17 },
    { 0x9a674f16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f474f19, SUC_lexicon_tags_246 },
    { 0x86674f1a, SUC_lexicon_tags_45 },
    { 0x7b074f1b, SUC_lexicon_tags_37 },
    { 0xb4e74f1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5674f1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da74f22, SUC_lexicon_tags_63 },
    { 0x8cc74f23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e474f29, SUC_lexicon_tags_30 },
    { 0xfd674f2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x67874f2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26874f32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a74f35, SUC_lexicon_tags_10 },
    { 0x53a74f35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3274f3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e474f3f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4d874f41, SUC_lexicon_tags_11 },
    { 0x10474f41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7de74f44, SUC_lexicon_tags_37 },
    { 0x59e74f45, SUC_lexicon_tags_21 },
    { 0x51a74f44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb274f49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe274f54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6274f56, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8fc74f58, SUC_lexicon_tags_31 },
    { 0xbd274f59, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe9c74f5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd274f67, SUC_lexicon_tags_4 },
    { 0x19074f68, SUC_lexicon_tags_43 },
    { 0x15a74f69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85874f6d, SUC_lexicon_tags_49 },
    { 0x56c74f6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c74f72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8874f76, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92274f7f, SUC_lexicon_tags_60 },
    { 0xca474f80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27874f8c, SUC_lexicon_tags_25 },
    { 0x13a74f8d, SUC_lexicon_tags_33 },
    { 0x32274f8e, SUC_lexicon_tags_14 },
    { 0x42674f8f, SUC_lexicon_tags_7 },
    { 0x76a74f90, SUC_lexicon_tags_2 },
    { 0x36274f90, SUC_lexicon_tags_32 },
    { 0xbde74f90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1a74f94, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc74f97, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4ae74f99, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf5a74f9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c74fa1, SUC_lexicon_tags_1 },
    { 0xea74fa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf674fa5, SUC_lexicon_tags_2 },
    { 0x6c674fa6, SUC_lexicon_tags_12 },
    { 0x64674fa7, SUC_lexicon_tags_140 },
    { 0xaea74fa8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x98874faa, SUC_lexicon_tags_7 },
    { 0x7ee74faa, SUC_lexicon_tags_12 },
    { 0xa2e74fac, SUC_lexicon_tags_31 },
    { 0x2b674faa, SUC_lexicon_tags_1 },
    { 0x2a674fae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c074fb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca74fbb, SUC_lexicon_tags_25 },
    { 0x3dc74fbb, SUC_lexicon_tags_7 },
    { 0x8a274fbd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6874fc1, SUC_lexicon_tags_31 },
    { 0xed474fc2, SUC_lexicon_tags_64 },
    { 0x37474fc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a274fc5, SUC_lexicon_tags_57 },
    { 0x1b074fc6, SUC_lexicon_tags_14 },
    { 0x1fe74fc7, SUC_lexicon_tags_2 },
    { 0x94674fc8, SUC_lexicon_tags_151 },
    { 0x74674fc6, SUC_lexicon_tags_43 },
    { 0xd5e74fca, SUC_lexicon_tags_59 },
    { 0x16c74fcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42674fcd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb074fd3, SUC_lexicon_tags_27 },
    { 0x46674fd4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4f474fd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc74fdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9a74fdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c74fe2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f674fe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b474fe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a74fec, SUC_lexicon_tags_0 },
    { 0xf5074fed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb474ff1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ee74ff3, SUC_lexicon_tags_38 },
    { 0x4c674ff3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e74ff9, SUC_lexicon_tags_7 },
    { 0x62e74ffa, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94674ffe, SUC_lexicon_tags_80 },
    { 0x7ac74fff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfe475001, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4475008, SUC_lexicon_tags_17 },
    { 0xb2075009, SUC_lexicon_tags_12 },
    { 0xdda7500a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a7500d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c75013, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c75019, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6587501f, SUC_lexicon_tags_70 },
    { 0xf4875020, SUC_lexicon_tags_66 },
    { 0x60275021, SUC_lexicon_tags_43 },
    { 0x2a675022, SUC_lexicon_tags_17 },
    { 0x69075021, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22275026, SUC_lexicon_tags_3 },
    { 0x22e75027, SUC_lexicon_tags_1 },
    { 0x85275028, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd4e7502a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a7502e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48075035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4475038, SUC_lexicon_tags_33 },
    { 0x4e75039, SUC_lexicon_tags_26 },
    { 0xdee7503a, SUC_lexicon_tags_35 },
    { 0xc1e75039, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1447503f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84075043, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56475047, SUC_lexicon_tags_14 },
    { 0x6b475047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe367504a, SUC_lexicon_tags_63 },
    { 0x39e7504b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf27504d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60c7504f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdba75051, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c75058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2547505c, SUC_lexicon_tags_49 },
    { 0x81c7505c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51675066, SUC_lexicon_tags_43 },
    { 0xee075067, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee7506b, SUC_lexicon_tags_92 },
    { 0x9747506c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6675070, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7475074, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90475077, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2467507d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e75080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40475088, SUC_lexicon_tags_17 },
    { 0xe7075088, SUC_lexicon_tags_45 },
    { 0x3767508a, SUC_lexicon_tags_62 },
    { 0x17e7508b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e75094, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b27509b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2750a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa78750aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0750b2, SUC_lexicon_tags_31 },
    { 0x73c750b2, SUC_lexicon_tags_83 },
    { 0x2de750b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a4750b6, SUC_lexicon_tags_48 },
    { 0x9a0750b7, SUC_lexicon_tags_11 },
    { 0x886750b8, SUC_lexicon_tags_15 },
    { 0x4e0750b9, SUC_lexicon_tags_12 },
    { 0x94750b9, SUC_lexicon_tags_11 },
    { 0xeee750ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9c750bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x872750c0, SUC_lexicon_tags_45 },
    { 0xaee750c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba750c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x868750c7, SUC_lexicon_tags_15 },
    { 0xa76750c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x816750ca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe750ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188750d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a750d9, SUC_lexicon_tags_17 },
    { 0xe8c750da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e750de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe70750e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3c750e2, SUC_lexicon_tags_2 },
    { 0xc00750e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d8750e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f4750e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea750ea, SUC_lexicon_tags_47 },
    { 0x1f8750eb, SUC_lexicon_tags_60 },
    { 0x7f0750eb, SUC_lexicon_tags_25 },
    { 0x63c750ec, SUC_lexicon_tags_7 },
    { 0x54e750ee, SUC_lexicon_tags_23 },
    { 0x6e2750ef, SUC_lexicon_tags_37 },
    { 0xa76750ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x918750f2, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde750f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2750f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x654750fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c750fe, SUC_lexicon_tags_1 },
    { 0xb9a750fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6075105, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fe75107, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6275110, SUC_lexicon_tags_7 },
    { 0x1da75111, SUC_lexicon_tags_21 },
    { 0xa5a75112, SUC_lexicon_tags_28 },
    { 0x81e75112, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac75116, SUC_lexicon_tags_45 },
    { 0xe0a75117, SUC_lexicon_tags_15 },
    { 0x99475118, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1127511a, SUC_lexicon_tags_48 },
    { 0xc9e7511b, SUC_lexicon_tags_30 },
    { 0x5887511c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07511f, SUC_lexicon_tags_0 },
    { 0xfaa7511f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0275128, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa07512a, SUC_lexicon_tags_37 },
    { 0xdd87512b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1807512e, SUC_lexicon_tags_1 },
    { 0x8f87512e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xc475131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50075136, SUC_lexicon_tags_8 },
    { 0x5f075137, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf867513d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe75140, SUC_lexicon_tags_43 },
    { 0xddc75141, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x96675143, SUC_lexicon_tags_17 },
    { 0x8c75143, SUC_lexicon_tags_25 },
    { 0x82a75145, SUC_lexicon_tags_14 },
    { 0x1675146, SUC_lexicon_tags_23 },
    { 0xc0075145, SUC_lexicon_tags_37 },
    { 0xd1475147, SUC_lexicon_tags_17 },
    { 0x13675149, SUC_lexicon_tags_39 },
    { 0x98a7514a, SUC_lexicon_tags_43 },
    { 0xb3a7514b, SUC_lexicon_tags_17 },
    { 0x8be75145, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1c67514e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a75151, SUC_lexicon_tags_4 },
    { 0xdb075152, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a75157, SUC_lexicon_tags_17 },
    { 0xd1075158, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c7515f, SUC_lexicon_tags_55 },
    { 0x6fc75160, SUC_lexicon_tags_29 },
    { 0xac75161, SUC_lexicon_tags_69 },
    { 0xa4075161, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a75168, SUC_lexicon_tags_11 },
    { 0x97275169, SUC_lexicon_tags_48 },
    { 0x40a7516a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce27516e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba75171, SUC_lexicon_tags_55 },
    { 0xfcc75172, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc9675174, SUC_lexicon_tags_0 },
    { 0xcd275175, SUC_lexicon_tags_17 },
    { 0xcac75176, SUC_lexicon_tags_1 },
    { 0xf5275176, SUC_lexicon_tags_17 },
    { 0xa675178, SUC_lexicon_tags_1 },
    { 0xe4c75175, SUC_lexicon_tags_57 },
    { 0xa507517a, SUC_lexicon_tags_25 },
    { 0xaee7517b, SUC_lexicon_tags_25 },
    { 0xf627517b, SUC_lexicon_tags_43 },
    { 0x17275178, SUC_lexicon_tags_1 },
    { 0x3e67517d, SUC_lexicon_tags_12 },
    { 0xc3e7517f, SUC_lexicon_tags_25 },
    { 0xe2675175, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d475183, SUC_lexicon_tags_38 },
    { 0xf1875184, SUC_lexicon_tags_16 },
    { 0xdca75184, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4c675187, SUC_lexicon_tags_82 },
    { 0xde875188, SUC_lexicon_tags_37 },
    { 0xdee75188, SUC_lexicon_tags_63 },
    { 0xbfc7518a, SUC_lexicon_tags_15 },
    { 0x7ce7518a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba75190, SUC_lexicon_tags_37 },
    { 0xbea75191, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c7519c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6fc7519e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e751a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2751a5, SUC_lexicon_tags_63 },
    { 0x4e6751a6, SUC_lexicon_tags_7 },
    { 0x430751a6, SUC_lexicon_tags_17 },
    { 0x9de751a7, SUC_lexicon_tags_60 },
    { 0xd00751a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8f0751ab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156751b1, SUC_lexicon_tags_49 },
    { 0xad0751b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac4751b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e751b7, SUC_lexicon_tags_31 },
    { 0x342751b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x814751ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca751be, SUC_lexicon_tags_12 },
    { 0x452751bf, SUC_lexicon_tags_2 },
    { 0x912751bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb2751c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66751ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ae751cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5c751ce, SUC_lexicon_tags_45 },
    { 0xe06751cf, SUC_lexicon_tags_44 },
    { 0x1e0751ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0751d3, SUC_lexicon_tags_11 },
    { 0x9c4751d4, SUC_lexicon_tags_7 },
    { 0x36a751d5, SUC_lexicon_tags_33 },
    { 0x114751d4, SUC_lexicon_tags_37 },
    { 0x7f8751d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888751e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x546751e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfea751e5, SUC_lexicon_tags_17 },
    { 0x680751e5, SUC_lexicon_tags_61 },
    { 0x5ca751e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640751ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e0751ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e751f3, SUC_lexicon_tags_8 },
    { 0xfc751f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa44751f6, SUC_lexicon_tags_0 },
    { 0x40c751f7, SUC_lexicon_tags_44 },
    { 0xc2a751f8, SUC_lexicon_tags_62 },
    { 0x2d2751f9, SUC_lexicon_tags_68 },
    { 0x4ba751f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0751fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca75203, SUC_lexicon_tags_191 },
    { 0xb1e75204, SUC_lexicon_tags_48 },
    { 0x1e675204, SUC_lexicon_tags_1 },
    { 0xee075206, SUC_lexicon_tags_46 },
    { 0xb4a75207, SUC_lexicon_tags_39 },
    { 0x79875208, SUC_lexicon_tags_11 },
    { 0xa3075207, SUC_lexicon_tags_7 },
    { 0xb0675206, SUC_lexicon_tags_6 },
    { 0x37a7520b, SUC_lexicon_tags_12 },
    { 0x2de7520c, SUC_lexicon_tags_27 },
    { 0x58075208, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa75212, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc75216, SUC_lexicon_tags_15 },
    { 0xe1075217, SUC_lexicon_tags_47 },
    { 0x4a075217, SUC_lexicon_tags_25 },
    { 0xdca75219, SUC_lexicon_tags_8 },
    { 0x90a7521a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe727521d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45875220, SUC_lexicon_tags_2 },
    { 0x64075221, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdac75223, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb875229, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7347522c, SUC_lexicon_tags_10 },
    { 0xc9e7522c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c75231, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda75236, SUC_lexicon_tags_17 },
    { 0x96275237, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b27523a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4875241, SUC_lexicon_tags_38 },
    { 0x85275241, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c75245, SUC_lexicon_tags_38 },
    { 0x7075246, SUC_lexicon_tags_25 },
    { 0x80475247, SUC_lexicon_tags_69 },
    { 0x61675247, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ca7524a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7907524c, SUC_lexicon_tags_1 },
    { 0xad47524d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3275250, SUC_lexicon_tags_37 },
    { 0x98e75251, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc75255, SUC_lexicon_tags_15 },
    { 0xedc75256, SUC_lexicon_tags_37 },
    { 0xdac75256, SUC_lexicon_tags_26 },
    { 0xc8475258, SUC_lexicon_tags_33 },
    { 0x2d275258, SUC_lexicon_tags_19 },
    { 0xe407525a, SUC_lexicon_tags_27 },
    { 0x43475258, SUC_lexicon_tags_35 },
    { 0xee7525c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de75264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c475266, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x58875269, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fc7526b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22275273, SUC_lexicon_tags_11 },
    { 0x7c875274, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4475279, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba7527c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2767527f, SUC_lexicon_tags_77 },
    { 0x22275280, SUC_lexicon_tags_21 },
    { 0xa5075281, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e75285, SUC_lexicon_tags_11 },
    { 0x5f075286, SUC_lexicon_tags_29 },
    { 0xea675287, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee07528c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf675292, SUC_lexicon_tags_2 },
    { 0x6fc75293, SUC_lexicon_tags_11 },
    { 0x36675294, SUC_lexicon_tags_37 },
    { 0x53275295, SUC_lexicon_tags_33 },
    { 0xb6275295, SUC_lexicon_tags_9 },
    { 0x99675297, SUC_lexicon_tags_7 },
    { 0x93a75298, SUC_lexicon_tags_65 },
    { 0x55875295, SUC_lexicon_tags_17 },
    { 0x4b07529a, SUC_lexicon_tags_37 },
    { 0xa0275293, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a87529f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x958752a2, SUC_lexicon_tags_37 },
    { 0xe18752a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79c752a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x904752a7, SUC_lexicon_tags_15 },
    { 0x210752a8, SUC_lexicon_tags_15 },
    { 0x504752a9, SUC_lexicon_tags_25 },
    { 0x118752a9, SUC_lexicon_tags_23 },
    { 0xdae752a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52752b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16752b4, SUC_lexicon_tags_27 },
    { 0x69a752b5, SUC_lexicon_tags_63 },
    { 0xd8752b6, SUC_lexicon_tags_68 },
    { 0x9b8752b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0752ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x706752bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28752c2, SUC_lexicon_tags_12 },
    { 0x906752c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e752c7, SUC_lexicon_tags_25 },
    { 0xc44752c8, SUC_lexicon_tags_1 },
    { 0x750752c9, SUC_lexicon_tags_27 },
    { 0x29a752c9, SUC_lexicon_tags_2 },
    { 0x410752cb, SUC_lexicon_tags_2 },
    { 0x3fc752cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x750752cf, SUC_lexicon_tags_49 },
    { 0xe54752d0, SUC_lexicon_tags_12 },
    { 0x2ce752d0, SUC_lexicon_tags_7 },
    { 0xc80752d1, SUC_lexicon_tags_43 },
    { 0x6f8752d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6752d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcd8752db, SUC_lexicon_tags_95 },
    { 0x3b4752dc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86752df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52752e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x896752e6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xcce752e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86752ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc752f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9e4752f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8752f9, SUC_lexicon_tags_79 },
    { 0x3e6752fa, SUC_lexicon_tags_17 },
    { 0x1c0752f9, SUC_lexicon_tags_39 },
    { 0x1de752fc, SUC_lexicon_tags_25 },
    { 0x3d0752fd, SUC_lexicon_tags_17 },
    { 0x4e752fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x62075300, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89075307, SUC_lexicon_tags_28 },
    { 0xd6a75308, SUC_lexicon_tags_47 },
    { 0x51075309, SUC_lexicon_tags_19 },
    { 0xc707530a, SUC_lexicon_tags_12 },
    { 0x7087530b, SUC_lexicon_tags_47 },
    { 0xaac75308, SUC_lexicon_tags_8 },
    { 0xdf07530d, SUC_lexicon_tags_70 },
    { 0x27c7530e, SUC_lexicon_tags_60 },
    { 0x2be75308, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x53275311, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a75316, SUC_lexicon_tags_17 },
    { 0x49275317, SUC_lexicon_tags_17 },
    { 0x6e475318, SUC_lexicon_tags_17 },
    { 0x90875319, SUC_lexicon_tags_37 },
    { 0x7bc75319, SUC_lexicon_tags_17 },
    { 0x92875316, SUC_lexicon_tags_37 },
    { 0x9f475317, SUC_lexicon_tags_17 },
    { 0xe767531d, SUC_lexicon_tags_2 },
    { 0xbbe7531d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a75323, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc75325, SUC_lexicon_tags_2 },
    { 0x26475326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c7532d, SUC_lexicon_tags_77 },
    { 0x8f07532e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb7675330, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33275336, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb475339, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xae87533b, SUC_lexicon_tags_48 },
    { 0xb807533c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b075340, SUC_lexicon_tags_11 },
    { 0xe5875340, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6875347, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e7534f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1275351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee875355, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x23c75357, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a7535b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe475360, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9a75362, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2047536c, SUC_lexicon_tags_17 },
    { 0x1487536d, SUC_lexicon_tags_17 },
    { 0x9027536c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84a75370, SUC_lexicon_tags_17 },
    { 0xbc675371, SUC_lexicon_tags_11 },
    { 0x2ee75372, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7075377, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x41875379, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa267537d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe7a7537f, SUC_lexicon_tags_66 },
    { 0x2ee75380, SUC_lexicon_tags_28 },
    { 0x63c75380, SUC_lexicon_tags_21 },
    { 0x43a75382, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2d475384, SUC_lexicon_tags_4 },
    { 0x9ea75385, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e75388, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf027538c, SUC_lexicon_tags_39 },
    { 0x9387538d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa75391, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef475396, SUC_lexicon_tags_2 },
    { 0x63675397, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e875399, SUC_lexicon_tags_15 },
    { 0x4ce7539a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b27539d, SUC_lexicon_tags_43 },
    { 0xb227539d, SUC_lexicon_tags_16 },
    { 0x6be7539e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b6753a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe50753a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6753a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea753ab, SUC_lexicon_tags_1587 },
    { 0, NULL },
    { 0xe10753ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08753b0, SUC_lexicon_tags_29 },
    { 0x7c6753b1, SUC_lexicon_tags_106 },
    { 0x753b1, SUC_lexicon_tags_11 },
    { 0xf6e753b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e753b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58e753b8, SUC_lexicon_tags_10 },
    { 0x7a2753b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516753bf, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2753c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf56753c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344753cd, SUC_lexicon_tags_4 },
    { 0x876753ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8753d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12753e1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3fe753e3, SUC_lexicon_tags_151 },
    { 0xeca753e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850753e9, SUC_lexicon_tags_2 },
    { 0x3d2753ea, SUC_lexicon_tags_29 },
    { 0xd66753ea, SUC_lexicon_tags_11 },
    { 0x8ce753ec, SUC_lexicon_tags_7 },
    { 0x6a2753ea, SUC_lexicon_tags_25 },
    { 0x6fe753ee, SUC_lexicon_tags_63 },
    { 0xc6753ee, SUC_lexicon_tags_17 },
    { 0x910753ec, SUC_lexicon_tags_15 },
    { 0xc30753f1, SUC_lexicon_tags_0 },
    { 0x3fa753ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd56753f4, SUC_lexicon_tags_31 },
    { 0x690753f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92753fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde6753ff, SUC_lexicon_tags_2 },
    { 0xffc75400, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e75404, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b67540e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0275415, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84475419, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da75424, SUC_lexicon_tags_4 },
    { 0x8b875425, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x64c75427, SUC_lexicon_tags_60 },
    { 0x84a75427, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc7542c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9275430, SUC_lexicon_tags_21 },
    { 0xe3475430, SUC_lexicon_tags_129 },
    { 0xed675430, SUC_lexicon_tags_6 },
    { 0x1c75430, SUC_lexicon_tags_160 },
    { 0xa4475434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2075436, SUC_lexicon_tags_11 },
    { 0x7d075437, SUC_lexicon_tags_47 },
    { 0x5a875437, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c7543d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf875444, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2c75446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e75449, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6127544e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60475452, SUC_lexicon_tags_14 },
    { 0xb2c75453, SUC_lexicon_tags_61 },
    { 0xaca75452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc75457, SUC_lexicon_tags_15 },
    { 0x43275458, SUC_lexicon_tags_37 },
    { 0x68c75458, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d47545b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6c87545d, SUC_lexicon_tags_23 },
    { 0xd727545e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79475468, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x427546a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6327546c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1875470, SUC_lexicon_tags_1 },
    { 0x5f875471, SUC_lexicon_tags_47 },
    { 0x50c75470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d075476, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f87547c, SUC_lexicon_tags_49 },
    { 0xa467547c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a75487, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19275489, SUC_lexicon_tags_5 },
    { 0x9227548a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe27548c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69275491, SUC_lexicon_tags_30 },
    { 0x36c75491, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a75495, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de7549a, SUC_lexicon_tags_32 },
    { 0x64e7549b, SUC_lexicon_tags_33 },
    { 0x8c7549c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x23a7549e, SUC_lexicon_tags_14 },
    { 0x62c7549f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32754a6, SUC_lexicon_tags_49 },
    { 0x790754a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x948754aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x692754ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466754b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2754ba, SUC_lexicon_tags_87 },
    { 0x244754bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868754bf, SUC_lexicon_tags_8 },
    { 0xbfc754c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be754c9, SUC_lexicon_tags_80 },
    { 0x11c754ca, SUC_lexicon_tags_158 },
    { 0x9aa754ca, SUC_lexicon_tags_9 },
    { 0x7d4754cc, SUC_lexicon_tags_87 },
    { 0xfc0754cd, SUC_lexicon_tags_11 },
    { 0xf64754ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec754d1, SUC_lexicon_tags_68 },
    { 0xcbc754d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x850754d5, SUC_lexicon_tags_17 },
    { 0xc8e754d6, SUC_lexicon_tags_17 },
    { 0xefe754d6, SUC_lexicon_tags_18 },
    { 0x832754d6, SUC_lexicon_tags_37 },
    { 0xdc8754d9, SUC_lexicon_tags_19 },
    { 0x99c754da, SUC_lexicon_tags_47 },
    { 0xcd2754d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x326754dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886754e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ca754e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25a754e5, SUC_lexicon_tags_70 },
    { 0xcfc754e6, SUC_lexicon_tags_17 },
    { 0x64a754e7, SUC_lexicon_tags_27 },
    { 0x938754e8, SUC_lexicon_tags_17 },
    { 0x1a0754e5, SUC_lexicon_tags_7 },
    { 0x564754ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da754ee, SUC_lexicon_tags_11 },
    { 0x9f0754ee, SUC_lexicon_tags_15 },
    { 0xffa754ee, SUC_lexicon_tags_12 },
    { 0x134754f0, SUC_lexicon_tags_31 },
    { 0x21e754f2, SUC_lexicon_tags_7 },
    { 0x6d4754f3, SUC_lexicon_tags_57 },
    { 0x482754f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec6754f6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca754f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa60754fc, SUC_lexicon_tags_17 },
    { 0xda8754fd, SUC_lexicon_tags_17 },
    { 0x794754fe, SUC_lexicon_tags_38 },
    { 0xa42754ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de75503, SUC_lexicon_tags_12 },
    { 0xef075504, SUC_lexicon_tags_1 },
    { 0xf3075503, SUC_lexicon_tags_47 },
    { 0x91e75506, SUC_lexicon_tags_37 },
    { 0x10c75504, SUC_lexicon_tags_63 },
    { 0xdac75508, SUC_lexicon_tags_23 },
    { 0x44c75508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f47550b, SUC_lexicon_tags_19 },
    { 0xb407550c, SUC_lexicon_tags_15 },
    { 0x9927550b, SUC_lexicon_tags_25 },
    { 0xd927550b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63075515, SUC_lexicon_tags_17 },
    { 0xb7c75516, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c7551a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3787551f, SUC_lexicon_tags_2 },
    { 0x58c7551f, SUC_lexicon_tags_581 },
    { 0x7b275521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2a75523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc275525, SUC_lexicon_tags_25 },
    { 0x5f475526, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf67552b, SUC_lexicon_tags_47 },
    { 0x73c7552c, SUC_lexicon_tags_14 },
    { 0x4267552d, SUC_lexicon_tags_0 },
    { 0x6d27552d, SUC_lexicon_tags_45 },
    { 0x45c7552f, SUC_lexicon_tags_15 },
    { 0x26e7552c, SUC_lexicon_tags_25 },
    { 0x70075531, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x80275534, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c75537, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8607553b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x54c7553d, SUC_lexicon_tags_25 },
    { 0x8ea7553e, SUC_lexicon_tags_11 },
    { 0x47c7553f, SUC_lexicon_tags_27 },
    { 0x71a7553f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc5475542, SUC_lexicon_tags_15 },
    { 0x65a75542, SUC_lexicon_tags_15 },
    { 0xce275543, SUC_lexicon_tags_17 },
    { 0x54e75545, SUC_lexicon_tags_11 },
    { 0x61c75545, SUC_lexicon_tags_15 },
    { 0x21475546, SUC_lexicon_tags_17 },
    { 0xa7c75545, SUC_lexicon_tags_27 },
    { 0xff875542, SUC_lexicon_tags_12 },
    { 0x17e75547, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce75550, SUC_lexicon_tags_15 },
    { 0x92075551, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c75556, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a275559, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8547555b, SUC_lexicon_tags_6 },
    { 0x6387555b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd647555e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc75563, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e75568, SUC_lexicon_tags_61 },
    { 0xab075568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c7556c, SUC_lexicon_tags_36 },
    { 0xa7e7556d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4875575, SUC_lexicon_tags_1 },
    { 0xdcc75575, SUC_lexicon_tags_7 },
    { 0x79e75577, SUC_lexicon_tags_11 },
    { 0x81875578, SUC_lexicon_tags_32 },
    { 0xd2e75579, SUC_lexicon_tags_11 },
    { 0x3ea75577, SUC_lexicon_tags_25 },
    { 0xd8c75576, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x31c7557d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c75581, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x81675584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a675586, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1e75588, SUC_lexicon_tags_23 },
    { 0x62475589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec47558b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc7558e, SUC_lexicon_tags_14 },
    { 0x87a7558f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27275594, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbe275596, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b675598, SUC_lexicon_tags_33 },
    { 0x67275599, SUC_lexicon_tags_11 },
    { 0xc675598, SUC_lexicon_tags_8 },
    { 0x1947559b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd1a7559d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a755a3, SUC_lexicon_tags_17 },
    { 0x444755a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8755a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68755ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x954755af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x756755b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c755b4, SUC_lexicon_tags_30 },
    { 0xfd6755b5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbc6755b7, SUC_lexicon_tags_19 },
    { 0x3de755b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbfa755ba, SUC_lexicon_tags_2 },
    { 0xb04755ba, SUC_lexicon_tags_14 },
    { 0x2b0755ba, SUC_lexicon_tags_15 },
    { 0xab8755bd, SUC_lexicon_tags_1 },
    { 0xa1c755be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x422755c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a755c3, SUC_lexicon_tags_91 },
    { 0xdfe755c3, SUC_lexicon_tags_11 },
    { 0xb7a755c3, SUC_lexicon_tags_37 },
    { 0xebc755c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36755cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc755ce, SUC_lexicon_tags_33 },
    { 0x810755ce, SUC_lexicon_tags_37 },
    { 0xa34755d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2755d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84755da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a755e2, SUC_lexicon_tags_2 },
    { 0xd52755e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6755eb, SUC_lexicon_tags_17 },
    { 0x13a755ec, SUC_lexicon_tags_46 },
    { 0x2b8755ed, SUC_lexicon_tags_61 },
    { 0xbfa755ee, SUC_lexicon_tags_25 },
    { 0x8c4755ee, SUC_lexicon_tags_1 },
    { 0x9d8755ee, SUC_lexicon_tags_2 },
    { 0x752755ec, SUC_lexicon_tags_16 },
    { 0x726755f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e755f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc9c755f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x34755fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a755fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4875600, SUC_lexicon_tags_11 },
    { 0x9b075601, SUC_lexicon_tags_16 },
    { 0xde275600, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c75605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf367560b, SUC_lexicon_tags_53 },
    { 0xe2e7560c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf875611, SUC_lexicon_tags_11 },
    { 0x42475612, SUC_lexicon_tags_11 },
    { 0xa0475612, SUC_lexicon_tags_14 },
    { 0x7e475614, SUC_lexicon_tags_75 },
    { 0xbe075613, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e75618, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a7561b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2847561e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x19075620, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba07562a, SUC_lexicon_tags_66 },
    { 0x9ea7562a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c7562f, SUC_lexicon_tags_15 },
    { 0x78a75630, SUC_lexicon_tags_8 },
    { 0x11275630, SUC_lexicon_tags_37 },
    { 0x6da75632, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad475639, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07563f, SUC_lexicon_tags_60 },
    { 0x5b475640, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x54675642, SUC_lexicon_tags_12 },
    { 0x7e75643, SUC_lexicon_tags_8 },
    { 0x94475644, SUC_lexicon_tags_11 },
    { 0x3fe75645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a7564b, SUC_lexicon_tags_29 },
    { 0x2367564b, SUC_lexicon_tags_63 },
    { 0xfdc7564c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d475650, SUC_lexicon_tags_21 },
    { 0xa1c75651, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a75654, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a75657, SUC_lexicon_tags_4 },
    { 0x2fe75658, SUC_lexicon_tags_14 },
    { 0x3e475657, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b27565b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3a7565d, SUC_lexicon_tags_30 },
    { 0xa167565e, SUC_lexicon_tags_63 },
    { 0x2367565f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ae75661, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40075665, SUC_lexicon_tags_79 },
    { 0xfd075666, SUC_lexicon_tags_31 },
    { 0x1c475667, SUC_lexicon_tags_33 },
    { 0x6c675668, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7307566b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4be7566d, SUC_lexicon_tags_37 },
    { 0xcba7566e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdb075670, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75675, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3675677, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca7567a, SUC_lexicon_tags_30 },
    { 0x7cc7567b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a75681, SUC_lexicon_tags_39 },
    { 0xd6075682, SUC_lexicon_tags_47 },
    { 0x4fe75683, SUC_lexicon_tags_63 },
    { 0xe5875682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce7568a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc7568d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda75690, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e875695, SUC_lexicon_tags_77 },
    { 0x89875696, SUC_lexicon_tags_15 },
    { 0xd6475696, SUC_lexicon_tags_1 },
    { 0x1d875695, SUC_lexicon_tags_11 },
    { 0xdca75699, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a7569e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766756a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ec756a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0756ab, SUC_lexicon_tags_65 },
    { 0x596756ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e756b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b6756b2, SUC_lexicon_tags_25 },
    { 0x680756b3, SUC_lexicon_tags_7 },
    { 0x7b6756b4, SUC_lexicon_tags_30 },
    { 0x898756b2, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x4ba756b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2756ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5e756bc, SUC_lexicon_tags_7 },
    { 0x1d2756bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4ce756bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64756c2, SUC_lexicon_tags_27 },
    { 0xfa0756c3, SUC_lexicon_tags_11 },
    { 0x61e756c2, SUC_lexicon_tags_65 },
    { 0x1f4756c3, SUC_lexicon_tags_37 },
    { 0x2a2756c6, SUC_lexicon_tags_38 },
    { 0x792756c6, SUC_lexicon_tags_19 },
    { 0xe38756c8, SUC_lexicon_tags_23 },
    { 0x7d0756c9, SUC_lexicon_tags_48 },
    { 0x8756ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3a756cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c4756ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x95a756d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92756d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4756db, SUC_lexicon_tags_7 },
    { 0xe28756dc, SUC_lexicon_tags_27 },
    { 0x760756dd, SUC_lexicon_tags_12 },
    { 0xaf6756de, SUC_lexicon_tags_0 },
    { 0x134756de, SUC_lexicon_tags_39 },
    { 0x788756de, SUC_lexicon_tags_0 },
    { 0x998756de, SUC_lexicon_tags_7 },
    { 0xf10756e2, SUC_lexicon_tags_19 },
    { 0x7bc756e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8756f8, SUC_lexicon_tags_84 },
    { 0x5fe756f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc675700, SUC_lexicon_tags_45 },
    { 0x10e75700, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a75705, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce75708, SUC_lexicon_tags_25 },
    { 0x5ec75708, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a7570c, SUC_lexicon_tags_37 },
    { 0xf9e7570d, SUC_lexicon_tags_12 },
    { 0x35e7570e, SUC_lexicon_tags_0 },
    { 0xbc07570f, SUC_lexicon_tags_1 },
    { 0x7ac7570c, SUC_lexicon_tags_11 },
    { 0x2f875711, SUC_lexicon_tags_15 },
    { 0x27a75712, SUC_lexicon_tags_47 },
    { 0x46875713, SUC_lexicon_tags_47 },
    { 0x4127570d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9b675716, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf547571a, SUC_lexicon_tags_27 },
    { 0xa987571a, SUC_lexicon_tags_11 },
    { 0x6e07571c, SUC_lexicon_tags_25 },
    { 0xdc47571a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83675721, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a7572c, SUC_lexicon_tags_25 },
    { 0xd727572d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a75731, SUC_lexicon_tags_23 },
    { 0xe9675732, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6275735, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x66875737, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1247573c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5075740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71275745, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc75748, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4607574a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8675751, SUC_lexicon_tags_55 },
    { 0xeb275752, SUC_lexicon_tags_13 },
    { 0x4fa75753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8675756, SUC_lexicon_tags_13 },
    { 0xe3475757, SUC_lexicon_tags_49 },
    { 0x96675758, SUC_lexicon_tags_0 },
    { 0xd8075758, SUC_lexicon_tags_38 },
    { 0x29e75759, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b47575f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86475762, SUC_lexicon_tags_23 },
    { 0x62475763, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b675767, SUC_lexicon_tags_19 },
    { 0xd1c75768, SUC_lexicon_tags_17 },
    { 0x94a75768, SUC_lexicon_tags_15 },
    { 0xfc7576a, SUC_lexicon_tags_28 },
    { 0x2d47576b, SUC_lexicon_tags_25 },
    { 0x8c7576a, SUC_lexicon_tags_25 },
    { 0x2dc75769, SUC_lexicon_tags_19 },
    { 0x667576d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc75779, SUC_lexicon_tags_21 },
    { 0x5487577a, SUC_lexicon_tags_63 },
    { 0xce7577b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e75780, SUC_lexicon_tags_8 },
    { 0xa2675781, SUC_lexicon_tags_47 },
    { 0x25075782, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef275784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e75787, SUC_lexicon_tags_46 },
    { 0x2fa75788, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x5727578a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae75792, SUC_lexicon_tags_17 },
    { 0xdbc75793, SUC_lexicon_tags_7 },
    { 0x7b675794, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5875796, SUC_lexicon_tags_56 },
    { 0x48e75797, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf3075799, SUC_lexicon_tags_39 },
    { 0xb787579a, SUC_lexicon_tags_10 },
    { 0x4827579a, SUC_lexicon_tags_4 },
    { 0x2087579a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5a7579e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x972757a1, SUC_lexicon_tags_63 },
    { 0x5ec757a2, SUC_lexicon_tags_26 },
    { 0x19c757a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe16757a5, SUC_lexicon_tags_12 },
    { 0xed2757a6, SUC_lexicon_tags_46 },
    { 0x27a757a7, SUC_lexicon_tags_63 },
    { 0xe9e757a8, SUC_lexicon_tags_35 },
    { 0x758757a9, SUC_lexicon_tags_15 },
    { 0xddc757a9, SUC_lexicon_tags_29 },
    { 0x644757aa, SUC_lexicon_tags_13 },
    { 0x9ba757a8, SUC_lexicon_tags_28 },
    { 0x332757aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc757b0, SUC_lexicon_tags_60 },
    { 0x294757b0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa90757b3, SUC_lexicon_tags_11 },
    { 0x6d2757b4, SUC_lexicon_tags_47 },
    { 0xb4e757b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc4757b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04757ba, SUC_lexicon_tags_14 },
    { 0xb0c757bb, SUC_lexicon_tags_25 },
    { 0x75c757bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x444757bf, SUC_lexicon_tags_33 },
    { 0x612757c0, SUC_lexicon_tags_12 },
    { 0x5b0757c1, SUC_lexicon_tags_1 },
    { 0x422757c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c757c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6757cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xae0757ce, SUC_lexicon_tags_47 },
    { 0x2a6757cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a757d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x37a757d4, SUC_lexicon_tags_49 },
    { 0xea2757d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeda757d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be757dd, SUC_lexicon_tags_22 },
    { 0xe5c757de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00757e1, SUC_lexicon_tags_46 },
    { 0x100757e2, SUC_lexicon_tags_37 },
    { 0x9d8757e3, SUC_lexicon_tags_84 },
    { 0x29a757e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a757ef, SUC_lexicon_tags_47 },
    { 0xc04757ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3e0757f2, SUC_lexicon_tags_37 },
    { 0xef2757f2, SUC_lexicon_tags_49 },
    { 0x58757f4, SUC_lexicon_tags_29 },
    { 0xb8757f4, SUC_lexicon_tags_60 },
    { 0x48c757f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee4757f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a75804, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2fa75806, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4675808, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc207580a, SUC_lexicon_tags_12 },
    { 0x3ca7580a, SUC_lexicon_tags_10 },
    { 0xe627580b, SUC_lexicon_tags_11 },
    { 0x4387580c, SUC_lexicon_tags_49 },
    { 0xbee7580e, SUC_lexicon_tags_4 },
    { 0xa227580b, SUC_lexicon_tags_7 },
    { 0x8f675810, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25075813, SUC_lexicon_tags_12 },
    { 0xe7c75814, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa567581a, SUC_lexicon_tags_26 },
    { 0x5b27581b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x587581d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa75821, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xdb475823, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc87582e, SUC_lexicon_tags_47 },
    { 0x5687582f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3a75831, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e75836, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7583c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c75840, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa75845, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd075847, SUC_lexicon_tags_37 },
    { 0x6e75848, SUC_lexicon_tags_2 },
    { 0x1aa75849, SUC_lexicon_tags_35 },
    { 0x63e7584a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3587584e, SUC_lexicon_tags_30 },
    { 0xb1a7584e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f675851, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e275854, SUC_lexicon_tags_2 },
    { 0x28875855, SUC_lexicon_tags_63 },
    { 0xf5875856, SUC_lexicon_tags_27 },
    { 0xeda75856, SUC_lexicon_tags_49 },
    { 0x49675857, SUC_lexicon_tags_45 },
    { 0xd7a75857, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d67585b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf1c7585d, SUC_lexicon_tags_31 },
    { 0xe227585e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77875860, SUC_lexicon_tags_47 },
    { 0x2675861, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd475866, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x26a75868, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e7586c, SUC_lexicon_tags_17 },
    { 0x3187586d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a75870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57675872, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38075874, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5475879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca27587b, SUC_lexicon_tags_43 },
    { 0xb527587b, SUC_lexicon_tags_15 },
    { 0x2ac7587c, SUC_lexicon_tags_7 },
    { 0xec27587e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3e75880, SUC_lexicon_tags_27 },
    { 0xff075880, SUC_lexicon_tags_15 },
    { 0x32275882, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x25875885, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fe75887, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32875889, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2547588c, SUC_lexicon_tags_0 },
    { 0x8b07588d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc75891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c75893, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc7589a, SUC_lexicon_tags_55 },
    { 0xd7c7589b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3667589d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0758a9, SUC_lexicon_tags_1 },
    { 0xe42758a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cc758ac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a758b0, SUC_lexicon_tags_82 },
    { 0x20a758b0, SUC_lexicon_tags_1 },
    { 0xbc0758b2, SUC_lexicon_tags_39 },
    { 0x672758b3, SUC_lexicon_tags_31 },
    { 0xbda758b4, SUC_lexicon_tags_37 },
    { 0x49a758b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6758b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa82758bb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a758be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c758c1, SUC_lexicon_tags_12 },
    { 0xc8e758c2, SUC_lexicon_tags_28 },
    { 0xdc758c3, SUC_lexicon_tags_15 },
    { 0xe52758c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xece758c7, SUC_lexicon_tags_49 },
    { 0x4e6758c8, SUC_lexicon_tags_40 },
    { 0xf4e758c7, SUC_lexicon_tags_60 },
    { 0x340758c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8758d2, SUC_lexicon_tags_14 },
    { 0x610758d2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x274758d5, SUC_lexicon_tags_37 },
    { 0xb9c758d6, SUC_lexicon_tags_7 },
    { 0x574758d5, SUC_lexicon_tags_14 },
    { 0xb0a758d7, SUC_lexicon_tags_6 },
    { 0x1f8758d9, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a758dd, SUC_lexicon_tags_12 },
    { 0x456758de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca0758e0, SUC_lexicon_tags_17 },
    { 0xe54758e0, SUC_lexicon_tags_7 },
    { 0xeb2758e0, SUC_lexicon_tags_1 },
    { 0x552758e0, SUC_lexicon_tags_25 },
    { 0xb8a758e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82758e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca758ee, SUC_lexicon_tags_12 },
    { 0xe04758ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2758f2, SUC_lexicon_tags_1 },
    { 0x3d4758f3, SUC_lexicon_tags_25 },
    { 0x84c758f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698758fa, SUC_lexicon_tags_12 },
    { 0xa30758fb, SUC_lexicon_tags_19 },
    { 0xffc758fc, SUC_lexicon_tags_63 },
    { 0x12e758fb, SUC_lexicon_tags_37 },
    { 0xbae758fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4475901, SUC_lexicon_tags_2 },
    { 0xb9075901, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7e75904, SUC_lexicon_tags_7 },
    { 0x6f075905, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x33275908, SUC_lexicon_tags_6 },
    { 0x9875909, SUC_lexicon_tags_8 },
    { 0x4ce75908, SUC_lexicon_tags_15 },
    { 0xeb67590a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf07590f, SUC_lexicon_tags_91 },
    { 0x21875910, SUC_lexicon_tags_2 },
    { 0x14275910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a75914, SUC_lexicon_tags_7 },
    { 0x85675914, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ac75917, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e7591b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a7591f, SUC_lexicon_tags_11 },
    { 0xea875920, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbde75922, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97475924, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee275928, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x79c7592a, SUC_lexicon_tags_15 },
    { 0x4887592b, SUC_lexicon_tags_15 },
    { 0x1047592c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd675930, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e75933, SUC_lexicon_tags_48 },
    { 0x70a75934, SUC_lexicon_tags_1 },
    { 0x9b275935, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4075938, SUC_lexicon_tags_31 },
    { 0x1fe75939, SUC_lexicon_tags_8 },
    { 0x36875939, SUC_lexicon_tags_37 },
    { 0xe2a7593b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a67593d, SUC_lexicon_tags_0 },
    { 0x7767593e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7c75940, SUC_lexicon_tags_6 },
    { 0xc4675941, SUC_lexicon_tags_82 },
    { 0xdca75942, SUC_lexicon_tags_11 },
    { 0xa6e75941, SUC_lexicon_tags_26 },
    { 0xa9a75941, SUC_lexicon_tags_1 },
    { 0x79475943, SUC_lexicon_tags_11 },
    { 0xf3c75942, SUC_lexicon_tags_35 },
    { 0xabe75944, SUC_lexicon_tags_21 },
    { 0xb9675947, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca87594a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb87594d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e475950, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e75958, SUC_lexicon_tags_11 },
    { 0x14a75958, SUC_lexicon_tags_1 },
    { 0xba7595a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5007595c, SUC_lexicon_tags_49 },
    { 0x3767595d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a75960, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcec75962, SUC_lexicon_tags_19 },
    { 0xef675963, SUC_lexicon_tags_33 },
    { 0x63475963, SUC_lexicon_tags_15 },
    { 0xf4e75963, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c675969, SUC_lexicon_tags_37 },
    { 0x70c7596a, SUC_lexicon_tags_15 },
    { 0xe207596b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9675971, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9c75973, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62475978, SUC_lexicon_tags_37 },
    { 0x19875979, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xab07597c, SUC_lexicon_tags_1 },
    { 0x2a7597c, SUC_lexicon_tags_59 },
    { 0x5a47597d, SUC_lexicon_tags_23 },
    { 0x2087597e, SUC_lexicon_tags_39 },
    { 0xd67597c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e75985, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd587598a, SUC_lexicon_tags_37 },
    { 0x7fe7598b, SUC_lexicon_tags_37 },
    { 0xd6a7598b, SUC_lexicon_tags_23 },
    { 0xc827598d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93075993, SUC_lexicon_tags_34 },
    { 0xce675994, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13475996, SUC_lexicon_tags_25 },
    { 0xfdc75996, SUC_lexicon_tags_12 },
    { 0x9e475998, SUC_lexicon_tags_12 },
    { 0x2ac75996, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc5c7599b, SUC_lexicon_tags_1 },
    { 0xbdc7599c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe8e7599e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4759a1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x798759a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xca759a6, SUC_lexicon_tags_77 },
    { 0x27e759a7, SUC_lexicon_tags_33 },
    { 0x3e8759a8, SUC_lexicon_tags_11 },
    { 0x1b0759a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250759ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8759b4, SUC_lexicon_tags_25 },
    { 0x59e759b5, SUC_lexicon_tags_183 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74759b8, SUC_lexicon_tags_12 },
    { 0xef0759b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142759be, SUC_lexicon_tags_1 },
    { 0xb5c759be, SUC_lexicon_tags_17 },
    { 0xa2c759be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbe759c2, SUC_lexicon_tags_15 },
    { 0x418759c2, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c759cb, SUC_lexicon_tags_15 },
    { 0x6f8759cc, SUC_lexicon_tags_30 },
    { 0x2a4759cb, SUC_lexicon_tags_33 },
    { 0x4fc759ce, SUC_lexicon_tags_29 },
    { 0xb7a759cf, SUC_lexicon_tags_49 },
    { 0x344759cf, SUC_lexicon_tags_66 },
    { 0x3ae759cf, SUC_lexicon_tags_18 },
    { 0xd78759cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79c759d4, SUC_lexicon_tags_27 },
    { 0xa0759d5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xeb0759d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8759db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326759df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940759e4, SUC_lexicon_tags_80 },
    { 0x61e759e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4759e8, SUC_lexicon_tags_17 },
    { 0x7a8759e9, SUC_lexicon_tags_11 },
    { 0x5ca759ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9b6759ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17e759ee, SUC_lexicon_tags_25 },
    { 0x1a4759ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x92759f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa32759f4, SUC_lexicon_tags_9 },
    { 0x132759f5, SUC_lexicon_tags_28 },
    { 0xec0759f5, SUC_lexicon_tags_11 },
    { 0x674759f6, SUC_lexicon_tags_68 },
    { 0xe92759f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe88759fa, SUC_lexicon_tags_23 },
    { 0xc3e759fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2759fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49875a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42275a07, SUC_lexicon_tags_17 },
    { 0xc3e75a08, SUC_lexicon_tags_2 },
    { 0xd675a08, SUC_lexicon_tags_37 },
    { 0x9a675a0a, SUC_lexicon_tags_2 },
    { 0x17475a0a, SUC_lexicon_tags_9 },
    { 0x36075a0b, SUC_lexicon_tags_7 },
    { 0x9ea75a08, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc75a12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51275a17, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6275a1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0275a1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e275a20, SUC_lexicon_tags_11 },
    { 0x55475a21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84a75a23, SUC_lexicon_tags_33 },
    { 0xccc75a23, SUC_lexicon_tags_7 },
    { 0x46c75a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a75a29, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x59875a2c, SUC_lexicon_tags_25 },
    { 0xcde75a2d, SUC_lexicon_tags_23 },
    { 0x8c075a2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6475a33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39075a3a, SUC_lexicon_tags_10 },
    { 0x30675a3b, SUC_lexicon_tags_10 },
    { 0x56675a3c, SUC_lexicon_tags_66 },
    { 0xb7875a3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb675a48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18075a4f, SUC_lexicon_tags_42 },
    { 0xe4275a50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a75a53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a75a56, SUC_lexicon_tags_37 },
    { 0x15675a57, SUC_lexicon_tags_25 },
    { 0x77475a56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca275a5c, SUC_lexicon_tags_31 },
    { 0xc9e75a5d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc75a60, SUC_lexicon_tags_19 },
    { 0x42875a61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b075a65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x48675a68, SUC_lexicon_tags_87 },
    { 0xd3e75a69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec75a6c, SUC_lexicon_tags_37 },
    { 0xa6475a6d, SUC_lexicon_tags_15 },
    { 0xcfa75a6e, SUC_lexicon_tags_26 },
    { 0x8b875a6c, SUC_lexicon_tags_25 },
    { 0xf8475a70, SUC_lexicon_tags_11 },
    { 0xe8475a71, SUC_lexicon_tags_11 },
    { 0x3e275a71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d075a76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd075a7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9a75a7d, SUC_lexicon_tags_32 },
    { 0x1da75a7e, SUC_lexicon_tags_23 },
    { 0x1f875a7d, SUC_lexicon_tags_2 },
    { 0xbaa75a80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c675a86, SUC_lexicon_tags_29 },
    { 0x2cc75a87, SUC_lexicon_tags_45 },
    { 0x26e75a88, SUC_lexicon_tags_12 },
    { 0x51c75a89, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1cc75a8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed075a8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c75a90, SUC_lexicon_tags_43 },
    { 0x77e75a91, SUC_lexicon_tags_68 },
    { 0x9a675a91, SUC_lexicon_tags_15 },
    { 0x60c75a93, SUC_lexicon_tags_39 },
    { 0x52875a91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x36e75a96, SUC_lexicon_tags_47 },
    { 0xaee75a97, SUC_lexicon_tags_17 },
    { 0x89c75a98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xabe75a9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1275aa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24475aa7, SUC_lexicon_tags_7 },
    { 0x49e75aa8, SUC_lexicon_tags_61 },
    { 0x56075aa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb675aab, SUC_lexicon_tags_15 },
    { 0xbde75aac, SUC_lexicon_tags_13 },
    { 0xc3475aac, SUC_lexicon_tags_12 },
    { 0x23c75aab, SUC_lexicon_tags_29 },
    { 0x29475aaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4875ab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30675ab4, SUC_lexicon_tags_37 },
    { 0x8ee75ab5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae875abc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f075abe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59675ac1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x16275ac4, SUC_lexicon_tags_12 },
    { 0x71875ac5, SUC_lexicon_tags_15 },
    { 0xdea75ac6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e75acc, SUC_lexicon_tags_17 },
    { 0xe2c75acd, SUC_lexicon_tags_39 },
    { 0x4675acd, SUC_lexicon_tags_47 },
    { 0x7de75acf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84275ad1, SUC_lexicon_tags_19 },
    { 0x77475ad2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f875ad4, SUC_lexicon_tags_7 },
    { 0xac275ad5, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x4b875ad7, SUC_lexicon_tags_27 },
    { 0x5b475ad8, SUC_lexicon_tags_33 },
    { 0x96875ad8, SUC_lexicon_tags_63 },
    { 0xb8075ad9, SUC_lexicon_tags_113 },
    { 0xa8675adb, SUC_lexicon_tags_17 },
    { 0xb8c75ad7, SUC_lexicon_tags_25 },
    { 0x2da75adb, SUC_lexicon_tags_2 },
    { 0xfa075ade, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a75ae4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e75ae7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a675ae9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34875aee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc875af7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59675b00, SUC_lexicon_tags_7 },
    { 0x1875b01, SUC_lexicon_tags_38 },
    { 0x68e75b02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc075b05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x39e75b07, SUC_lexicon_tags_45 },
    { 0x33275b08, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd275b14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c75b17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6de75b19, SUC_lexicon_tags_65 },
    { 0x3e675b19, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xff875b1c, SUC_lexicon_tags_45 },
    { 0xe7275b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf875b27, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c275b2d, SUC_lexicon_tags_29 },
    { 0x14c75b2e, SUC_lexicon_tags_11 },
    { 0x6ea75b2f, SUC_lexicon_tags_23 },
    { 0x90875b2f, SUC_lexicon_tags_45 },
    { 0xe2275b2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8075b35, SUC_lexicon_tags_19 },
    { 0xc6675b35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22275b3a, SUC_lexicon_tags_49 },
    { 0x44c75b3a, SUC_lexicon_tags_33 },
    { 0xcac75b3a, SUC_lexicon_tags_19 },
    { 0x2c075b3c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xad075b40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa275b44, SUC_lexicon_tags_85 },
    { 0xa8a75b44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0875b52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6e75b54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9c475b56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23a75b58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c475b5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd275b63, SUC_lexicon_tags_14 },
    { 0x8c875b64, SUC_lexicon_tags_25 },
    { 0x6b475b65, SUC_lexicon_tags_37 },
    { 0x1d275b66, SUC_lexicon_tags_7 },
    { 0x26675b67, SUC_lexicon_tags_69 },
    { 0xdd075b66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c75b6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9675b70, SUC_lexicon_tags_37 },
    { 0x2b075b70, SUC_lexicon_tags_3 },
    { 0x62075b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80475b78, SUC_lexicon_tags_17 },
    { 0x6ec75b79, SUC_lexicon_tags_12 },
    { 0x37e75b7a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de75b7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b275b81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c875b85, SUC_lexicon_tags_6 },
    { 0xb5c75b85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa75b89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec75b8d, SUC_lexicon_tags_56 },
    { 0xfc675b8e, SUC_lexicon_tags_37 },
    { 0xa9675b8f, SUC_lexicon_tags_60 },
    { 0xc0a75b90, SUC_lexicon_tags_27 },
    { 0x9a075b8e, SUC_lexicon_tags_25 },
    { 0xa5e75b92, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa75b9d, SUC_lexicon_tags_12 },
    { 0x51875b9e, SUC_lexicon_tags_12 },
    { 0xac075b9e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x70475ba2, SUC_lexicon_tags_7 },
    { 0x33c75ba3, SUC_lexicon_tags_2 },
    { 0xabe75ba4, SUC_lexicon_tags_26 },
    { 0xd1475ba5, SUC_lexicon_tags_29 },
    { 0x25475ba6, SUC_lexicon_tags_14 },
    { 0x80e75ba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26e75ba9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5475bab, SUC_lexicon_tags_1 },
    { 0xc5675bab, SUC_lexicon_tags_0 },
    { 0xd5675bac, SUC_lexicon_tags_14 },
    { 0xda675bad, SUC_lexicon_tags_27 },
    { 0xb6075baf, SUC_lexicon_tags_27 },
    { 0x6b875bae, SUC_lexicon_tags_29 },
    { 0x56075bad, SUC_lexicon_tags_7 },
    { 0x5d875bb2, SUC_lexicon_tags_9 },
    { 0x30e75bb3, SUC_lexicon_tags_9 },
    { 0x3b675baf, SUC_lexicon_tags_25 },
    { 0x7be75bb5, SUC_lexicon_tags_4 },
    { 0xb8275bb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11475bb8, SUC_lexicon_tags_17 },
    { 0xab475bb8, SUC_lexicon_tags_37 },
    { 0x40675bb8, SUC_lexicon_tags_48 },
    { 0x14c75bb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc75bbe, SUC_lexicon_tags_37 },
    { 0x1dc75bbf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c075bc1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f475bc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c475bcf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb5e75bd1, SUC_lexicon_tags_15 },
    { 0x9ea75bd2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e75bd6, SUC_lexicon_tags_38 },
    { 0x24e75bd7, SUC_lexicon_tags_17 },
    { 0x4e675bd8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89275bdf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe9475be1, SUC_lexicon_tags_82 },
    { 0x59875be1, SUC_lexicon_tags_23 },
    { 0x34a75be2, SUC_lexicon_tags_12 },
    { 0x8c75be1, SUC_lexicon_tags_15 },
    { 0x7e75be1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c75be9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c75bed, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4275bf1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a875bf5, SUC_lexicon_tags_13 },
    { 0x87e75bf5, SUC_lexicon_tags_17 },
    { 0xcd675bf7, SUC_lexicon_tags_28 },
    { 0x7f875bf8, SUC_lexicon_tags_49 },
    { 0x5a475bf9, SUC_lexicon_tags_38 },
    { 0xa6a75bf9, SUC_lexicon_tags_14 },
    { 0xa5e75bf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4875bfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de75c01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e75c09, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be75c0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98875c11, SUC_lexicon_tags_1 },
    { 0x75875c12, SUC_lexicon_tags_670 },
    { 0, NULL },
    { 0, NULL },
    { 0x24075c15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a075c1a, SUC_lexicon_tags_254 },
    { 0x47c75c1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3675c28, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xee875c2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3aa75c2d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3a75c2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec875c34, SUC_lexicon_tags_7 },
    { 0xf6075c35, SUC_lexicon_tags_37 },
    { 0xfca75c36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee75c3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6075c3e, SUC_lexicon_tags_80 },
    { 0x29875c3f, SUC_lexicon_tags_47 },
    { 0xa2c75c3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e75c43, SUC_lexicon_tags_37 },
    { 0x24c75c44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42475c46, SUC_lexicon_tags_23 },
    { 0x62c75c47, SUC_lexicon_tags_9 },
    { 0x7a675c47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc75c4c, SUC_lexicon_tags_46 },
    { 0x1ec75c4d, SUC_lexicon_tags_17 },
    { 0x3fc75c4c, SUC_lexicon_tags_32 },
    { 0x55a75c4c, SUC_lexicon_tags_11 },
    { 0x7a675c4c, SUC_lexicon_tags_11 },
    { 0x4f875c4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf6075c53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4475c55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc75c58, SUC_lexicon_tags_0 },
    { 0xe8875c59, SUC_lexicon_tags_17 },
    { 0xd9675c5a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc75c5e, SUC_lexicon_tags_2 },
    { 0xcc275c5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e75c62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e75c65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e75c6a, SUC_lexicon_tags_19 },
    { 0x3a475c6b, SUC_lexicon_tags_7 },
    { 0xd7075c6c, SUC_lexicon_tags_23 },
    { 0x73a75c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3875c71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4275c74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe8a75c76, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31275c7a, SUC_lexicon_tags_16 },
    { 0x94875c7b, SUC_lexicon_tags_25 },
    { 0x86675c7b, SUC_lexicon_tags_0 },
    { 0x72675c7d, SUC_lexicon_tags_14 },
    { 0xca75c7e, SUC_lexicon_tags_11 },
    { 0x9f875c7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8475c83, SUC_lexicon_tags_12 },
    { 0x50a75c84, SUC_lexicon_tags_33 },
    { 0x4075c85, SUC_lexicon_tags_28 },
    { 0x6fc75c84, SUC_lexicon_tags_2 },
    { 0x10475c84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e75c8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3e75c8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93475c91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58075c93, SUC_lexicon_tags_4 },
    { 0x68c75c94, SUC_lexicon_tags_12 },
    { 0x6e075c93, SUC_lexicon_tags_27 },
    { 0xd075c96, SUC_lexicon_tags_1 },
    { 0x23c75c96, SUC_lexicon_tags_69 },
    { 0x7875c98, SUC_lexicon_tags_1 },
    { 0x1f475c99, SUC_lexicon_tags_12 },
    { 0x3b275c96, SUC_lexicon_tags_31 },
    { 0x3f275c93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60e75c9d, SUC_lexicon_tags_23 },
    { 0xfaa75c9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5275ca0, SUC_lexicon_tags_25 },
    { 0x64875ca0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe475ca5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed675caa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c075cad, SUC_lexicon_tags_25 },
    { 0x4b675cae, SUC_lexicon_tags_37 },
    { 0x5fa75cae, SUC_lexicon_tags_46 },
    { 0xbd075cb0, SUC_lexicon_tags_79 },
    { 0x5c075cb1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e75cb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a75cbb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e75cc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76875cc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b075cca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b875ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c75cd0, SUC_lexicon_tags_25 },
    { 0x4c675cd1, SUC_lexicon_tags_11 },
    { 0xc6875cd2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5be75cd4, SUC_lexicon_tags_28 },
    { 0x48c75cd5, SUC_lexicon_tags_17 },
    { 0x7a475cd6, SUC_lexicon_tags_91 },
    { 0x68475cd7, SUC_lexicon_tags_55 },
    { 0xf4e75cd8, SUC_lexicon_tags_63 },
    { 0x87275cd9, SUC_lexicon_tags_25 },
    { 0x81a75cda, SUC_lexicon_tags_15 },
    { 0xeb275cd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59a75cdd, SUC_lexicon_tags_1 },
    { 0x11a75cdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94675ce2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63275ce4, SUC_lexicon_tags_11 },
    { 0x81c75ce5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0a75ce7, SUC_lexicon_tags_25 },
    { 0x45275ce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc275cea, SUC_lexicon_tags_17 },
    { 0x44675cea, SUC_lexicon_tags_2 },
    { 0xb6e75cec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5675cf1, SUC_lexicon_tags_45 },
    { 0xa1c75cf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18475cfc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc75d01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be75d04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56875d06, SUC_lexicon_tags_47 },
    { 0x1ae75d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78675d09, SUC_lexicon_tags_25 },
    { 0xb6475d0a, SUC_lexicon_tags_10 },
    { 0x95075d0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4075d0d, SUC_lexicon_tags_14 },
    { 0x2ba75d0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c75d14, SUC_lexicon_tags_11 },
    { 0xac75d15, SUC_lexicon_tags_7 },
    { 0xbc275d15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47075d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee75d1b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x92c75d1d, SUC_lexicon_tags_31 },
    { 0xaa075d1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36675d20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6275d22, SUC_lexicon_tags_11 },
    { 0x97e75d23, SUC_lexicon_tags_17 },
    { 0xe6275d24, SUC_lexicon_tags_25 },
    { 0xcac75d23, SUC_lexicon_tags_60 },
    { 0x2c475d26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29675d29, SUC_lexicon_tags_37 },
    { 0xea075d2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa9075d2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x50875d2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86475d33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c275d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19675d3d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c75d40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9875d44, SUC_lexicon_tags_1 },
    { 0xc8c75d45, SUC_lexicon_tags_30 },
    { 0x62e75d44, SUC_lexicon_tags_14 },
    { 0x8c675d47, SUC_lexicon_tags_12 },
    { 0x16c75d48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfce75d4b, SUC_lexicon_tags_82 },
    { 0x2f275d4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4875d51, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32675d56, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2ae75d58, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe75d5b, SUC_lexicon_tags_25 },
    { 0xe1075d5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9e75d5e, SUC_lexicon_tags_60 },
    { 0xe3275d5f, SUC_lexicon_tags_37 },
    { 0x6a075d60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf075d64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98875d68, SUC_lexicon_tags_11 },
    { 0x71a75d69, SUC_lexicon_tags_37 },
    { 0x68a75d6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x39475d6d, SUC_lexicon_tags_10 },
    { 0x82c75d6e, SUC_lexicon_tags_17 },
    { 0x99075d6f, SUC_lexicon_tags_12 },
    { 0x9e875d6f, SUC_lexicon_tags_57 },
    { 0xdf875d6e, SUC_lexicon_tags_37 },
    { 0xcca75d71, SUC_lexicon_tags_2 },
    { 0x2f275d72, SUC_lexicon_tags_33 },
    { 0x36675d6d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a75d77, SUC_lexicon_tags_17 },
    { 0xbea75d78, SUC_lexicon_tags_28 },
    { 0x7a275d77, SUC_lexicon_tags_47 },
    { 0x57075d7a, SUC_lexicon_tags_91 },
    { 0x5a075d7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbd075d7d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a75d81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10275d86, SUC_lexicon_tags_65 },
    { 0xf7a75d86, SUC_lexicon_tags_11 },
    { 0x62a75d87, SUC_lexicon_tags_65 },
    { 0xdb475d86, SUC_lexicon_tags_15 },
    { 0xe5075d8a, SUC_lexicon_tags_15 },
    { 0xf6a75d8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69075d98, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc75d9b, SUC_lexicon_tags_10 },
    { 0x1f075d9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a75da0, SUC_lexicon_tags_84 },
    { 0x66e75da0, SUC_lexicon_tags_15 },
    { 0xb875da0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3275da4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7875da7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa75dab, SUC_lexicon_tags_12 },
    { 0xaee75dac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaac75dae, SUC_lexicon_tags_65 },
    { 0x96c75daf, SUC_lexicon_tags_49 },
    { 0x72675db0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4075db3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed875dba, SUC_lexicon_tags_3 },
    { 0xc9075dbb, SUC_lexicon_tags_37 },
    { 0x59e75dbc, SUC_lexicon_tags_6 },
    { 0xf1e75dbb, SUC_lexicon_tags_62 },
    { 0x85c75dbc, SUC_lexicon_tags_28 },
    { 0x17675dbc, SUC_lexicon_tags_25 },
    { 0xf3075dbd, SUC_lexicon_tags_15 },
    { 0xff275dbf, SUC_lexicon_tags_47 },
    { 0x475dbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc75dc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x51875dc8, SUC_lexicon_tags_1 },
    { 0xf2c75dc8, SUC_lexicon_tags_19 },
    { 0x4a075dc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a75dcd, SUC_lexicon_tags_11 },
    { 0xa2275dce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91675dd2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xba875dd4, SUC_lexicon_tags_38 },
    { 0x81075dd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18475dd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b675dd9, SUC_lexicon_tags_65 },
    { 0x39e75dda, SUC_lexicon_tags_7 },
    { 0x63e75ddb, SUC_lexicon_tags_37 },
    { 0xe7475ddb, SUC_lexicon_tags_7 },
    { 0x30475ddd, SUC_lexicon_tags_32 },
    { 0x60a75ddb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e275de0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28675de7, SUC_lexicon_tags_47 },
    { 0xe9475de8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81075dea, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x29675dec, SUC_lexicon_tags_65 },
    { 0x41075ded, SUC_lexicon_tags_6 },
    { 0x54275dee, SUC_lexicon_tags_37 },
    { 0x26c75dee, SUC_lexicon_tags_17 },
    { 0xd8875def, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e675df4, SUC_lexicon_tags_70 },
    { 0x82275df5, SUC_lexicon_tags_46 },
    { 0x1bc75df5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x70475df8, SUC_lexicon_tags_2 },
    { 0xb2475df9, SUC_lexicon_tags_16 },
    { 0xcda75dfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe875dfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0a75dff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12675e03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7275e06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c75e09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39475e0d, SUC_lexicon_tags_37 },
    { 0x73475e0e, SUC_lexicon_tags_15 },
    { 0x18c75e0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9675e15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3fe75e17, SUC_lexicon_tags_12 },
    { 0xf4075e17, SUC_lexicon_tags_17 },
    { 0x43475e19, SUC_lexicon_tags_11 },
    { 0x2ba75e1a, SUC_lexicon_tags_31 },
    { 0xb6075e1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76875e24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc75e28, SUC_lexicon_tags_25 },
    { 0xa9875e29, SUC_lexicon_tags_8 },
    { 0xade75e2a, SUC_lexicon_tags_17 },
    { 0xfce75e28, SUC_lexicon_tags_31 },
    { 0xe4e75e2c, SUC_lexicon_tags_9 },
    { 0xfa275e2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e75e31, SUC_lexicon_tags_23 },
    { 0xe0c75e32, SUC_lexicon_tags_48 },
    { 0x9c275e33, SUC_lexicon_tags_10 },
    { 0xdf875e34, SUC_lexicon_tags_15 },
    { 0x9de75e34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57675e39, SUC_lexicon_tags_17 },
    { 0x4f475e39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f875e3c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c75e46, SUC_lexicon_tags_7 },
    { 0x1a475e47, SUC_lexicon_tags_613 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e75e4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a75e4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec75e52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40275e54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c75e57, SUC_lexicon_tags_11 },
    { 0xf0c75e57, SUC_lexicon_tags_15 },
    { 0x67875e58, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3a75e5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5075e5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1d075e60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a75e64, SUC_lexicon_tags_95 },
    { 0xdc475e65, SUC_lexicon_tags_33 },
    { 0xc0875e64, SUC_lexicon_tags_28 },
    { 0xb0075e67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e75e69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a75e70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e75e74, SUC_lexicon_tags_30 },
    { 0xafa75e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf675e77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda075e7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6875e7e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5b275e80, SUC_lexicon_tags_31 },
    { 0xa1075e80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e75e86, SUC_lexicon_tags_60 },
    { 0xa3275e87, SUC_lexicon_tags_43 },
    { 0x12675e88, SUC_lexicon_tags_49 },
    { 0xeb075e88, SUC_lexicon_tags_25 },
    { 0x10a75e88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e275e8c, SUC_lexicon_tags_70 },
    { 0xb9e75e8d, SUC_lexicon_tags_15 },
    { 0x3c275e8e, SUC_lexicon_tags_1 },
    { 0x7075e8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda675e91, SUC_lexicon_tags_37 },
    { 0x49675e92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda75e96, SUC_lexicon_tags_17 },
    { 0xf5275e97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a75e9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd875e9d, SUC_lexicon_tags_12 },
    { 0x2ea75e9e, SUC_lexicon_tags_15 },
    { 0xb3a75e9f, SUC_lexicon_tags_19 },
    { 0x4a875e9d, SUC_lexicon_tags_11 },
    { 0xc9075e9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f475ea5, SUC_lexicon_tags_57 },
    { 0xc4e75ea5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e75eaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xabc75eb1, SUC_lexicon_tags_25 },
    { 0x21a75eb1, SUC_lexicon_tags_11 },
    { 0x19475eb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f675eb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e75eba, SUC_lexicon_tags_47 },
    { 0x59675eba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x36875ebe, SUC_lexicon_tags_37 },
    { 0x3b675ebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd475ec1, SUC_lexicon_tags_32 },
    { 0x8a75ec1, SUC_lexicon_tags_19 },
    { 0xc2e75ec1, SUC_lexicon_tags_15 },
    { 0x37075ec4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68675ec8, SUC_lexicon_tags_106 },
    { 0x62075ec9, SUC_lexicon_tags_17 },
    { 0x71675ec9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c75ecd, SUC_lexicon_tags_29 },
    { 0x6f475ece, SUC_lexicon_tags_12 },
    { 0x1d075ecd, SUC_lexicon_tags_38 },
    { 0x4c75ed0, SUC_lexicon_tags_25 },
    { 0x9c875ed0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f275ed4, SUC_lexicon_tags_1 },
    { 0xd0c75ed4, SUC_lexicon_tags_7 },
    { 0x5fa75ed6, SUC_lexicon_tags_1 },
    { 0x55875ed7, SUC_lexicon_tags_45 },
    { 0x77e75ed8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a75edb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e75ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73475ee3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf9e75ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa75ee8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9e75eea, SUC_lexicon_tags_30 },
    { 0x58075eeb, SUC_lexicon_tags_1 },
    { 0xb5275eec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcc475eee, SUC_lexicon_tags_2 },
    { 0x70075eef, SUC_lexicon_tags_1 },
    { 0x94075eee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe3075ef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55675ef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77a75ef7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c875efb, SUC_lexicon_tags_6 },
    { 0xda875efc, SUC_lexicon_tags_25 },
    { 0x5ba75efd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe75f02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd075f0f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5275f12, SUC_lexicon_tags_1 },
    { 0x9ae75f13, SUC_lexicon_tags_25 },
    { 0xa2075f13, SUC_lexicon_tags_61 },
    { 0xaf275f15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2475f19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcce75f1b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e275f1e, SUC_lexicon_tags_12 },
    { 0x80c75f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7c75f21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1675f27, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e475f2a, SUC_lexicon_tags_14 },
    { 0x9bc75f2a, SUC_lexicon_tags_17 },
    { 0x1d675f2c, SUC_lexicon_tags_60 },
    { 0xac75f2d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d875f31, SUC_lexicon_tags_25 },
    { 0x9f075f32, SUC_lexicon_tags_17 },
    { 0x31475f33, SUC_lexicon_tags_12 },
    { 0xe9c75f31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3675f37, SUC_lexicon_tags_60 },
    { 0x98275f38, SUC_lexicon_tags_953 },
    { 0x43a75f39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa475f3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5875f3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e75f42, SUC_lexicon_tags_6 },
    { 0xae475f43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f875f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60c75f48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe075f4e, SUC_lexicon_tags_25 },
    { 0xb3275f4f, SUC_lexicon_tags_7 },
    { 0xbd275f4e, SUC_lexicon_tags_45 },
    { 0xb9a75f51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a475f54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca75f57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c875f5a, SUC_lexicon_tags_15 },
    { 0xd1075f5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac75f61, SUC_lexicon_tags_2 },
    { 0x88875f62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb075f6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3075f6c, SUC_lexicon_tags_17 },
    { 0x51075f6c, SUC_lexicon_tags_17 },
    { 0xbbc75f6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e75f71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f075f73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b275f7a, SUC_lexicon_tags_9 },
    { 0xe8275f7a, SUC_lexicon_tags_25 },
    { 0xe0475f7b, SUC_lexicon_tags_1 },
    { 0xe2075f7d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c75f82, SUC_lexicon_tags_0 },
    { 0xe4875f82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x40275f85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccc75f87, SUC_lexicon_tags_27 },
    { 0x7275f88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xafa75f8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7675f90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e75f93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50475f95, SUC_lexicon_tags_98 },
    { 0xfb475f96, SUC_lexicon_tags_45 },
    { 0x9d675f97, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f475f9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc6475f9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58475f9f, SUC_lexicon_tags_27 },
    { 0x9b875f9f, SUC_lexicon_tags_11 },
    { 0x8d875fa1, SUC_lexicon_tags_17 },
    { 0xc4a75fa0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb475fa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3075faa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa675fad, SUC_lexicon_tags_27 },
    { 0x7e675fae, SUC_lexicon_tags_25 },
    { 0x6a475fae, SUC_lexicon_tags_179 },
    { 0x72e75fb0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a75fb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6475fb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17475fb8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa3075fba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5675fbf, SUC_lexicon_tags_25 },
    { 0x26475fbf, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e75fc4, SUC_lexicon_tags_11 },
    { 0xee075fc5, SUC_lexicon_tags_33 },
    { 0x89e75fc5, SUC_lexicon_tags_11 },
    { 0x9fe75fc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc275fc9, SUC_lexicon_tags_7 },
    { 0xc6075fca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b275fcc, SUC_lexicon_tags_47 },
    { 0x62475fcd, SUC_lexicon_tags_1 },
    { 0xc2675fcc, SUC_lexicon_tags_16 },
    { 0x57875fcf, SUC_lexicon_tags_63 },
    { 0x57275fcd, SUC_lexicon_tags_7 },
    { 0x2fc75fcf, SUC_lexicon_tags_15 },
    { 0xb0875fd0, SUC_lexicon_tags_28 },
    { 0x8a275fd3, SUC_lexicon_tags_26 },
    { 0x9c875fd4, SUC_lexicon_tags_27 },
    { 0x51a75fd4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da75fdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17a75fdf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x95a75fe1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2075fe3, SUC_lexicon_tags_17 },
    { 0x39a75fe4, SUC_lexicon_tags_30 },
    { 0x42675fe3, SUC_lexicon_tags_12 },
    { 0x4c075fe6, SUC_lexicon_tags_2 },
    { 0xba875fe4, SUC_lexicon_tags_21 },
    { 0xef875fe3, SUC_lexicon_tags_20 },
    { 0xc0a75fe5, SUC_lexicon_tags_15 },
    { 0x55275fea, SUC_lexicon_tags_12 },
    { 0x9d875fe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x20475fee, SUC_lexicon_tags_46 },
    { 0xe9a75fef, SUC_lexicon_tags_77 },
    { 0x86275ff0, SUC_lexicon_tags_45 },
    { 0x5d275ff1, SUC_lexicon_tags_38 },
    { 0xe4875ff1, SUC_lexicon_tags_12 },
    { 0xb3275ff1, SUC_lexicon_tags_15 },
    { 0xe8a75ff2, SUC_lexicon_tags_12 },
    { 0xefc75ff1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb275ff8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e75fff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d276002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e76006, SUC_lexicon_tags_7 },
    { 0x1c676007, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7876009, SUC_lexicon_tags_15 },
    { 0xb947600a, SUC_lexicon_tags_17 },
    { 0xc1e7600b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa47600d, SUC_lexicon_tags_39 },
    { 0x92e7600d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a76011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89a76013, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a76017, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90476019, SUC_lexicon_tags_761 },
    { 0x2287601a, SUC_lexicon_tags_7 },
    { 0x4f67601b, SUC_lexicon_tags_7 },
    { 0xc947601c, SUC_lexicon_tags_63 },
    { 0xc5c7601c, SUC_lexicon_tags_62 },
    { 0xdb47601e, SUC_lexicon_tags_15 },
    { 0xe4a7601a, SUC_lexicon_tags_11 },
    { 0x40276020, SUC_lexicon_tags_19 },
    { 0x54076021, SUC_lexicon_tags_37 },
    { 0xb1876021, SUC_lexicon_tags_12 },
    { 0x30476023, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa76028, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7c7602a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b67602f, SUC_lexicon_tags_25 },
    { 0x90a76030, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a76036, SUC_lexicon_tags_39 },
    { 0x84076036, SUC_lexicon_tags_27 },
    { 0x41076038, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3127603e, SUC_lexicon_tags_14 },
    { 0x1547603e, SUC_lexicon_tags_12 },
    { 0x3067603f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda476042, SUC_lexicon_tags_17 },
    { 0x8e876043, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcb276045, SUC_lexicon_tags_55 },
    { 0xc4076045, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x52a76048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1687604a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e7604d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93876051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e76055, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9476058, SUC_lexicon_tags_16 },
    { 0x71876058, SUC_lexicon_tags_37 },
    { 0x27605a, SUC_lexicon_tags_1 },
    { 0x2a76059, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8407605d, SUC_lexicon_tags_11 },
    { 0x327605d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8076062, SUC_lexicon_tags_25 },
    { 0xe8876062, SUC_lexicon_tags_47 },
    { 0x4a476064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97e76066, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71476068, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca87606c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2476077, SUC_lexicon_tags_0 },
    { 0x32676078, SUC_lexicon_tags_12 },
    { 0x39676079, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e7607c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2967607f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec076082, SUC_lexicon_tags_2 },
    { 0x53276083, SUC_lexicon_tags_60 },
    { 0xfe276084, SUC_lexicon_tags_11 },
    { 0x2b676083, SUC_lexicon_tags_47 },
    { 0xda676086, SUC_lexicon_tags_12 },
    { 0xcba76086, SUC_lexicon_tags_9 },
    { 0x1a676085, SUC_lexicon_tags_47 },
    { 0x32876089, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9067608e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e76091, SUC_lexicon_tags_1 },
    { 0xc6e76092, SUC_lexicon_tags_39 },
    { 0x79c76091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b47609c, SUC_lexicon_tags_46 },
    { 0x9da7609d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c760a4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x820760a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e760ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32760af, SUC_lexicon_tags_17 },
    { 0xb54760b0, SUC_lexicon_tags_37 },
    { 0x9e2760b1, SUC_lexicon_tags_15 },
    { 0xa2760b2, SUC_lexicon_tags_17 },
    { 0x488760b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x618760b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae760ba, SUC_lexicon_tags_15 },
    { 0xdc2760ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a760be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c760c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x882760c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xab2760c5, SUC_lexicon_tags_2 },
    { 0x2f8760c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6760cd, SUC_lexicon_tags_15 },
    { 0xbce760ce, SUC_lexicon_tags_60 },
    { 0xbf4760cf, SUC_lexicon_tags_1 },
    { 0xb18760d0, SUC_lexicon_tags_37 },
    { 0x3f0760cf, SUC_lexicon_tags_65 },
    { 0xe64760d2, SUC_lexicon_tags_43 },
    { 0xc26760cf, SUC_lexicon_tags_37 },
    { 0x298760d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122760de, SUC_lexicon_tags_37 },
    { 0x68a760de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x114760e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x62a760e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14760e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c760ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168760f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a760f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7aa760f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b4760f8, SUC_lexicon_tags_4 },
    { 0x69a760f8, SUC_lexicon_tags_14 },
    { 0xbe0760f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32760fd, SUC_lexicon_tags_0 },
    { 0xd24760fd, SUC_lexicon_tags_25 },
    { 0x6ba760ff, SUC_lexicon_tags_7 },
    { 0x31076100, SUC_lexicon_tags_11 },
    { 0xd5076100, SUC_lexicon_tags_17 },
    { 0x85676102, SUC_lexicon_tags_15 },
    { 0xe5876103, SUC_lexicon_tags_7 },
    { 0xbe676103, SUC_lexicon_tags_6 },
    { 0xe12760fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64076108, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef27610c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c076110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f676113, SUC_lexicon_tags_62 },
    { 0xcfe76114, SUC_lexicon_tags_17 },
    { 0xba676114, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc76120, SUC_lexicon_tags_56 },
    { 0x10076121, SUC_lexicon_tags_54 },
    { 0x2c276121, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc887612b, SUC_lexicon_tags_37 },
    { 0xe4e7612c, SUC_lexicon_tags_55 },
    { 0xad07612c, SUC_lexicon_tags_1 },
    { 0x42c7612e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c076130, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab876133, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea76137, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x75076139, SUC_lexicon_tags_10 },
    { 0x7427613a, SUC_lexicon_tags_38 },
    { 0xaf87613b, SUC_lexicon_tags_1 },
    { 0x5347613c, SUC_lexicon_tags_3 },
    { 0x8ee7613c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe676142, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8676145, SUC_lexicon_tags_31 },
    { 0x87076146, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72876148, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc27614c, SUC_lexicon_tags_19 },
    { 0xfac7614c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb047614f, SUC_lexicon_tags_17 },
    { 0x77276150, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec76152, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d276154, SUC_lexicon_tags_14 },
    { 0x5b476155, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc76158, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd547615a, SUC_lexicon_tags_27 },
    { 0x47c7615b, SUC_lexicon_tags_26 },
    { 0x34c7615c, SUC_lexicon_tags_0 },
    { 0x76c7615d, SUC_lexicon_tags_7 },
    { 0xefc7615d, SUC_lexicon_tags_49 },
    { 0xa8c7615f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c76163, SUC_lexicon_tags_23 },
    { 0xcb276164, SUC_lexicon_tags_49 },
    { 0xd3076164, SUC_lexicon_tags_17 },
    { 0x4d876164, SUC_lexicon_tags_60 },
    { 0x4d076163, SUC_lexicon_tags_38 },
    { 0xfae76164, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c76172, SUC_lexicon_tags_11 },
    { 0xa0676173, SUC_lexicon_tags_49 },
    { 0x29676174, SUC_lexicon_tags_48 },
    { 0xc3c76172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe7617e, SUC_lexicon_tags_17 },
    { 0x18a7617f, SUC_lexicon_tags_28 },
    { 0x7a076180, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb276189, SUC_lexicon_tags_11 },
    { 0x35876189, SUC_lexicon_tags_2 },
    { 0x6cc7618b, SUC_lexicon_tags_62 },
    { 0x63c7618c, SUC_lexicon_tags_7 },
    { 0xf647618d, SUC_lexicon_tags_1 },
    { 0x3f07618b, SUC_lexicon_tags_39 },
    { 0x9527618c, SUC_lexicon_tags_28 },
    { 0x6847618b, SUC_lexicon_tags_40 },
    { 0xde87618e, SUC_lexicon_tags_70 },
    { 0xfa76190, SUC_lexicon_tags_39 },
    { 0x9cc76193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc76199, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e7619e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30761a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4761a9, SUC_lexicon_tags_25 },
    { 0x106761aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x494761ad, SUC_lexicon_tags_15 },
    { 0x77e761ae, SUC_lexicon_tags_2 },
    { 0xd48761ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10761b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x324761b5, SUC_lexicon_tags_38 },
    { 0xc9e761b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd2761b8, SUC_lexicon_tags_31 },
    { 0xe30761b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x808761bb, SUC_lexicon_tags_138 },
    { 0x40c761bc, SUC_lexicon_tags_17 },
    { 0xd9a761bd, SUC_lexicon_tags_34 },
    { 0xf42761be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a761c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460761c7, SUC_lexicon_tags_60 },
    { 0x54a761c8, SUC_lexicon_tags_7 },
    { 0xcc4761c9, SUC_lexicon_tags_37 },
    { 0x38e761c9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e761cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4761d2, SUC_lexicon_tags_1 },
    { 0x5ba761d3, SUC_lexicon_tags_15 },
    { 0xa5e761d3, SUC_lexicon_tags_17 },
    { 0xa96761d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa8761d7, SUC_lexicon_tags_37 },
    { 0x210761d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580761dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bc761de, SUC_lexicon_tags_25 },
    { 0x9f2761df, SUC_lexicon_tags_29 },
    { 0xc46761e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0761e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x644761e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x22761e7, SUC_lexicon_tags_37 },
    { 0x4fc761e8, SUC_lexicon_tags_7 },
    { 0xc6c761e9, SUC_lexicon_tags_43 },
    { 0xd82761ea, SUC_lexicon_tags_19 },
    { 0xbc8761ea, SUC_lexicon_tags_66 },
    { 0xa36761ec, SUC_lexicon_tags_55 },
    { 0xfec761e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4761f2, SUC_lexicon_tags_1 },
    { 0xab8761f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f8761f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a761fe, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9676202, SUC_lexicon_tags_63 },
    { 0x4476203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31e76205, SUC_lexicon_tags_45 },
    { 0xa4c76206, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xec876209, SUC_lexicon_tags_15 },
    { 0x3887620a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c7620e, SUC_lexicon_tags_12 },
    { 0x3f47620f, SUC_lexicon_tags_17 },
    { 0x71276210, SUC_lexicon_tags_12 },
    { 0x2d67620f, SUC_lexicon_tags_2 },
    { 0x94a7620e, SUC_lexicon_tags_6 },
    { 0x91e76213, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd476216, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6c76218, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc7621b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1c47621d, SUC_lexicon_tags_17 },
    { 0x1747621d, SUC_lexicon_tags_0 },
    { 0x1fe7621f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46876228, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c7622c, SUC_lexicon_tags_12 },
    { 0xfc7622d, SUC_lexicon_tags_43 },
    { 0xd7e7622e, SUC_lexicon_tags_7 },
    { 0xb627622c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7076233, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x93c76235, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x12476237, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a7623d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a76241, SUC_lexicon_tags_63 },
    { 0xbd076242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21076246, SUC_lexicon_tags_33 },
    { 0xdb876247, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x83876249, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab07624d, SUC_lexicon_tags_19 },
    { 0x6e67624e, SUC_lexicon_tags_11 },
    { 0xae47624d, SUC_lexicon_tags_12 },
    { 0xe27624d, SUC_lexicon_tags_12 },
    { 0x5c67624f, SUC_lexicon_tags_55 },
    { 0x6e47624f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1876258, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x607625a, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0xc5a7625c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e67625e, SUC_lexicon_tags_31 },
    { 0x8467625e, SUC_lexicon_tags_1 },
    { 0x2447625e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ae76262, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8a476264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c76267, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xede76269, SUC_lexicon_tags_14 },
    { 0x3fe7626a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5967626c, SUC_lexicon_tags_37 },
    { 0xef07626d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e76270, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2276274, SUC_lexicon_tags_55 },
    { 0x4b876274, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x98676277, SUC_lexicon_tags_17 },
    { 0x38076277, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e7627c, SUC_lexicon_tags_17 },
    { 0x8167627d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed07627f, SUC_lexicon_tags_17 },
    { 0xb7276280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb476282, SUC_lexicon_tags_10 },
    { 0x64a76283, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8676286, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a7628a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf267628c, SUC_lexicon_tags_33 },
    { 0x72c7628d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e27628f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2476292, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9076294, SUC_lexicon_tags_38 },
    { 0x86a76294, SUC_lexicon_tags_11 },
    { 0x69876295, SUC_lexicon_tags_37 },
    { 0x2c676297, SUC_lexicon_tags_7 },
    { 0x4f276296, SUC_lexicon_tags_8 },
    { 0xe4c76294, SUC_lexicon_tags_40 },
    { 0x9fe76296, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c87629f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a762a2, SUC_lexicon_tags_7 },
    { 0x42a762a3, SUC_lexicon_tags_7 },
    { 0x3ac762a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8a6762a6, SUC_lexicon_tags_216 },
    { 0x47e762a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x156762a9, SUC_lexicon_tags_2 },
    { 0xe62762a9, SUC_lexicon_tags_1386 },
    { 0, NULL },
    { 0xbe8762ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x648762ae, SUC_lexicon_tags_48 },
    { 0xdee762ae, SUC_lexicon_tags_12 },
    { 0xf92762b0, SUC_lexicon_tags_27 },
    { 0x5ea762b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe4762b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc98762b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x302762b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82762bc, SUC_lexicon_tags_29 },
    { 0x794762bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988762c1, SUC_lexicon_tags_17 },
    { 0xb28762c2, SUC_lexicon_tags_47 },
    { 0xc6c762c3, SUC_lexicon_tags_61 },
    { 0x7de762c4, SUC_lexicon_tags_38 },
    { 0x570762c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee762c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a762cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52762cf, SUC_lexicon_tags_48 },
    { 0x7ea762cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x94e762d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608762da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2762dd, SUC_lexicon_tags_54 },
    { 0xb1c762de, SUC_lexicon_tags_12 },
    { 0x3c6762df, SUC_lexicon_tags_56 },
    { 0x35e762df, SUC_lexicon_tags_11 },
    { 0x3da762df, SUC_lexicon_tags_6 },
    { 0x158762e2, SUC_lexicon_tags_12 },
    { 0x7f4762e3, SUC_lexicon_tags_1 },
    { 0x66c762e2, SUC_lexicon_tags_25 },
    { 0x9c762e5, SUC_lexicon_tags_60 },
    { 0xdc6762e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a762e9, SUC_lexicon_tags_29 },
    { 0xb6a762ea, SUC_lexicon_tags_3 },
    { 0x284762eb, SUC_lexicon_tags_23 },
    { 0xcb4762ea, SUC_lexicon_tags_17 },
    { 0xc68762ec, SUC_lexicon_tags_46 },
    { 0x34c762eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2762f1, SUC_lexicon_tags_3 },
    { 0x7a2762f1, SUC_lexicon_tags_37 },
    { 0xd5e762f2, SUC_lexicon_tags_7 },
    { 0x970762f1, SUC_lexicon_tags_61 },
    { 0xcac762f5, SUC_lexicon_tags_45 },
    { 0x6c762f4, SUC_lexicon_tags_15 },
    { 0xaf2762f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8762fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91676301, SUC_lexicon_tags_12 },
    { 0xd6e76302, SUC_lexicon_tags_35 },
    { 0xd6476303, SUC_lexicon_tags_11 },
    { 0x95676303, SUC_lexicon_tags_15 },
    { 0xeae76305, SUC_lexicon_tags_19 },
    { 0xa9a76305, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5327630a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3667630e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x35476310, SUC_lexicon_tags_17 },
    { 0x93476311, SUC_lexicon_tags_26 },
    { 0xbb276312, SUC_lexicon_tags_47 },
    { 0x8fc76310, SUC_lexicon_tags_27 },
    { 0x37c76311, SUC_lexicon_tags_35 },
    { 0x42276315, SUC_lexicon_tags_63 },
    { 0x3076316, SUC_lexicon_tags_25 },
    { 0x30a76317, SUC_lexicon_tags_1 },
    { 0xeba76317, SUC_lexicon_tags_29 },
    { 0x2476319, SUC_lexicon_tags_12 },
    { 0x81676315, SUC_lexicon_tags_27 },
    { 0xff276316, SUC_lexicon_tags_25 },
    { 0x1827631c, SUC_lexicon_tags_30 },
    { 0x54a76312, SUC_lexicon_tags_118 },
    { 0x83e76318, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58276324, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa76327, SUC_lexicon_tags_63 },
    { 0x22a76327, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb427632a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c7632e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e76331, SUC_lexicon_tags_23 },
    { 0x40276332, SUC_lexicon_tags_55 },
    { 0xd0a76333, SUC_lexicon_tags_1 },
    { 0xd8276332, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f47633c, SUC_lexicon_tags_11 },
    { 0x54c7633d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2307633f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbe876341, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e76346, SUC_lexicon_tags_0 },
    { 0xbde76347, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7507634c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b67634e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0276357, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbc876359, SUC_lexicon_tags_15 },
    { 0x44876359, SUC_lexicon_tags_49 },
    { 0x9687635b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca76360, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea76364, SUC_lexicon_tags_1 },
    { 0x41676365, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e76369, SUC_lexicon_tags_87 },
    { 0xfbe7636a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687636f, SUC_lexicon_tags_37 },
    { 0x24e7636f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfac76372, SUC_lexicon_tags_32 },
    { 0x46276373, SUC_lexicon_tags_33 },
    { 0xe6476373, SUC_lexicon_tags_15 },
    { 0xe2076375, SUC_lexicon_tags_37 },
    { 0x8f276375, SUC_lexicon_tags_11 },
    { 0x2a476377, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10076380, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53476385, SUC_lexicon_tags_14 },
    { 0xf5076385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6c76388, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1187638d, SUC_lexicon_tags_38 },
    { 0x32a7638e, SUC_lexicon_tags_1 },
    { 0x4567638e, SUC_lexicon_tags_1 },
    { 0x47e7638e, SUC_lexicon_tags_23 },
    { 0xe007638e, SUC_lexicon_tags_0 },
    { 0x62a76390, SUC_lexicon_tags_9 },
    { 0xd427638f, SUC_lexicon_tags_95 },
    { 0x7ae76390, SUC_lexicon_tags_2 },
    { 0x8007638f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6276398, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x23a7639a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398763a0, SUC_lexicon_tags_12 },
    { 0xd18763a1, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44763a4, SUC_lexicon_tags_37 },
    { 0x772763a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686763a9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x776763ab, SUC_lexicon_tags_17 },
    { 0x674763ac, SUC_lexicon_tags_1 },
    { 0x668763ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682763b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc5a763b4, SUC_lexicon_tags_15 },
    { 0xca2763b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x598763b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34763b9, SUC_lexicon_tags_25 },
    { 0x1da763ba, SUC_lexicon_tags_37 },
    { 0x208763ba, SUC_lexicon_tags_1 },
    { 0x282763bc, SUC_lexicon_tags_17 },
    { 0x72e763bc, SUC_lexicon_tags_25 },
    { 0x22a763bc, SUC_lexicon_tags_16 },
    { 0xb2e763bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98763c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17c763c4, SUC_lexicon_tags_9 },
    { 0x9f8763c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15a763c7, SUC_lexicon_tags_15 },
    { 0xb4c763c7, SUC_lexicon_tags_11 },
    { 0xb9c763c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8763ce, SUC_lexicon_tags_25 },
    { 0x906763cf, SUC_lexicon_tags_15 },
    { 0xa3c763d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbcc763d2, SUC_lexicon_tags_2 },
    { 0xe08763d3, SUC_lexicon_tags_8 },
    { 0xd30763d4, SUC_lexicon_tags_17 },
    { 0xeb0763d5, SUC_lexicon_tags_17 },
    { 0xf7a763d6, SUC_lexicon_tags_10 },
    { 0xb60763d3, SUC_lexicon_tags_15 },
    { 0xa46763d8, SUC_lexicon_tags_37 },
    { 0xe14763d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce763e3, SUC_lexicon_tags_38 },
    { 0x6da763e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e763e9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2763f0, SUC_lexicon_tags_15 },
    { 0xca2763f0, SUC_lexicon_tags_28 },
    { 0x302763f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41c763f4, SUC_lexicon_tags_2 },
    { 0x5de763f4, SUC_lexicon_tags_15 },
    { 0xbec763f4, SUC_lexicon_tags_11 },
    { 0xb02763f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16763fc, SUC_lexicon_tags_14 },
    { 0xa7e763fc, SUC_lexicon_tags_1 },
    { 0x82e763fe, SUC_lexicon_tags_23 },
    { 0x630763fc, SUC_lexicon_tags_66 },
    { 0x88763fc, SUC_lexicon_tags_11 },
    { 0x164763fe, SUC_lexicon_tags_17 },
    { 0x5e676402, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa76408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x347640b, SUC_lexicon_tags_2 },
    { 0x2587640b, SUC_lexicon_tags_25 },
    { 0x1787640d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29676412, SUC_lexicon_tags_25 },
    { 0x8c276413, SUC_lexicon_tags_12 },
    { 0xf2c76414, SUC_lexicon_tags_1 },
    { 0x9c876412, SUC_lexicon_tags_37 },
    { 0x6ca76416, SUC_lexicon_tags_12 },
    { 0x47476412, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4647641a, SUC_lexicon_tags_37 },
    { 0xf8e7641b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b47641f, SUC_lexicon_tags_11 },
    { 0xd6876420, SUC_lexicon_tags_136 },
    { 0xe7476420, SUC_lexicon_tags_39 },
    { 0xe4e76422, SUC_lexicon_tags_60 },
    { 0xa3876422, SUC_lexicon_tags_17 },
    { 0x48876421, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e7642c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75276433, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee476437, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7643c, SUC_lexicon_tags_25 },
    { 0x27a7643d, SUC_lexicon_tags_25 },
    { 0xf687643e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e76441, SUC_lexicon_tags_30 },
    { 0x38276441, SUC_lexicon_tags_17 },
    { 0xbfe76443, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb276447, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x507644a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa47644d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5907644f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7476451, SUC_lexicon_tags_17 },
    { 0x76876451, SUC_lexicon_tags_34 },
    { 0xef276452, SUC_lexicon_tags_48 },
    { 0x80e76452, SUC_lexicon_tags_14 },
    { 0xba076452, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xafe76457, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da7645a, SUC_lexicon_tags_27 },
    { 0xb687645a, SUC_lexicon_tags_15 },
    { 0xe927645a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1476461, SUC_lexicon_tags_12 },
    { 0x7e676462, SUC_lexicon_tags_1 },
    { 0x8cc76463, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea76473, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de76477, SUC_lexicon_tags_141 },
    { 0x1076477, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x65c7647a, SUC_lexicon_tags_37 },
    { 0xee7647b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe447647e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c76482, SUC_lexicon_tags_101 },
    { 0xfd276483, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc76486, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a476489, SUC_lexicon_tags_29 },
    { 0x13e76489, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f87648e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39c76490, SUC_lexicon_tags_13 },
    { 0xf1c76491, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6476493, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e276496, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda47649b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988764a0, SUC_lexicon_tags_25 },
    { 0x5f2764a1, SUC_lexicon_tags_33 },
    { 0x87e764a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6764ac, SUC_lexicon_tags_12 },
    { 0x1a6764ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae764b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a764b4, SUC_lexicon_tags_38 },
    { 0x9d8764b5, SUC_lexicon_tags_35 },
    { 0xb3e764b6, SUC_lexicon_tags_2 },
    { 0x1f4764b7, SUC_lexicon_tags_37 },
    { 0x70764b8, SUC_lexicon_tags_25 },
    { 0x818764b5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e764bf, SUC_lexicon_tags_66 },
    { 0xa9c764c0, SUC_lexicon_tags_23 },
    { 0xde764bf, SUC_lexicon_tags_17 },
    { 0x34e764c2, SUC_lexicon_tags_7 },
    { 0xe18764c3, SUC_lexicon_tags_11 },
    { 0x486764c2, SUC_lexicon_tags_56 },
    { 0x3d4764c2, SUC_lexicon_tags_16 },
    { 0x514764c6, SUC_lexicon_tags_15 },
    { 0x3ea764c3, SUC_lexicon_tags_2 },
    { 0x476764c6, SUC_lexicon_tags_21 },
    { 0xc64764c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116764ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf4764d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc764d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a764d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ca764d9, SUC_lexicon_tags_9 },
    { 0x380764da, SUC_lexicon_tags_27 },
    { 0xaee764d9, SUC_lexicon_tags_30 },
    { 0xcfa764d9, SUC_lexicon_tags_16 },
    { 0xade764da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2764e0, SUC_lexicon_tags_17 },
    { 0xfc2764e1, SUC_lexicon_tags_79 },
    { 0x3d4764e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8764e5, SUC_lexicon_tags_12 },
    { 0x142764e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xce0764e8, SUC_lexicon_tags_25 },
    { 0x918764e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x692764ec, SUC_lexicon_tags_60 },
    { 0x180764ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ee764ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98764f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6764fb, SUC_lexicon_tags_151 },
    { 0xc9c764fc, SUC_lexicon_tags_25 },
    { 0xa8764fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdac764ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe8476501, SUC_lexicon_tags_17 },
    { 0xa7276502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11476505, SUC_lexicon_tags_14 },
    { 0x63a76506, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c76509, SUC_lexicon_tags_6 },
    { 0x5f876509, SUC_lexicon_tags_37 },
    { 0x5567650b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa87650d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80a7650f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e76512, SUC_lexicon_tags_66 },
    { 0x46a76512, SUC_lexicon_tags_25 },
    { 0x96c76514, SUC_lexicon_tags_47 },
    { 0xb0876515, SUC_lexicon_tags_25 },
    { 0xf1c76515, SUC_lexicon_tags_1 },
    { 0xb6076515, SUC_lexicon_tags_12 },
    { 0xdbc76518, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf047651d, SUC_lexicon_tags_79 },
    { 0x7c87651e, SUC_lexicon_tags_44 },
    { 0x4807651d, SUC_lexicon_tags_79 },
    { 0x30e7651e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e076526, SUC_lexicon_tags_27 },
    { 0x2476527, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x73876529, SUC_lexicon_tags_207 },
    { 0x4e7652a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa267652f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe5276531, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff276534, SUC_lexicon_tags_14 },
    { 0x3ba76534, SUC_lexicon_tags_2 },
    { 0xbc676535, SUC_lexicon_tags_65 },
    { 0x83476534, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x57676539, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca7653c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c7653f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11076543, SUC_lexicon_tags_32 },
    { 0xdc276544, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a076548, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb07654e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d476551, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c76553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6676558, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb07655f, SUC_lexicon_tags_15 },
    { 0xee676560, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede76565, SUC_lexicon_tags_47 },
    { 0x78a76565, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x4276568, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46e7656a, SUC_lexicon_tags_66 },
    { 0xf847656a, SUC_lexicon_tags_59 },
    { 0xab07656c, SUC_lexicon_tags_17 },
    { 0x4527656d, SUC_lexicon_tags_47 },
    { 0xc747656c, SUC_lexicon_tags_25 },
    { 0xbd07656f, SUC_lexicon_tags_17 },
    { 0x167656c, SUC_lexicon_tags_47 },
    { 0x9c876570, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe2476573, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e476575, SUC_lexicon_tags_0 },
    { 0x2c76575, SUC_lexicon_tags_1 },
    { 0x71e76577, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee7657a, SUC_lexicon_tags_38 },
    { 0x6727657b, SUC_lexicon_tags_6 },
    { 0x5f87657c, SUC_lexicon_tags_7 },
    { 0xb6c7657d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6267657f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84476583, SUC_lexicon_tags_60 },
    { 0x71e76584, SUC_lexicon_tags_68 },
    { 0x63c76585, SUC_lexicon_tags_2 },
    { 0xf5876586, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a7658a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f87658e, SUC_lexicon_tags_31 },
    { 0xb5e7658f, SUC_lexicon_tags_12 },
    { 0xe8276590, SUC_lexicon_tags_31 },
    { 0x53276590, SUC_lexicon_tags_6 },
    { 0x92a76592, SUC_lexicon_tags_37 },
    { 0x9e47658f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb476596, SUC_lexicon_tags_1 },
    { 0xabc76596, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee7659b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8765a2, SUC_lexicon_tags_2 },
    { 0xd5c765a3, SUC_lexicon_tags_31 },
    { 0xa28765a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb06765a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744765aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6765ae, SUC_lexicon_tags_12 },
    { 0xbf2765af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2765b2, SUC_lexicon_tags_49 },
    { 0x4ac765b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa5c765b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8cc765b7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a765bd, SUC_lexicon_tags_26 },
    { 0xc38765be, SUC_lexicon_tags_14 },
    { 0xdc8765bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7a765c1, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52765c5, SUC_lexicon_tags_77 },
    { 0x266765c6, SUC_lexicon_tags_0 },
    { 0x7e4765c7, SUC_lexicon_tags_17 },
    { 0x4a8765c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7c8765ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcc765cc, SUC_lexicon_tags_37 },
    { 0x25e765cd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x57e765cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a2765d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a765d7, SUC_lexicon_tags_2 },
    { 0x6e765d8, SUC_lexicon_tags_32 },
    { 0x270765d9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe2c765db, SUC_lexicon_tags_15 },
    { 0xa0e765dc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x360765de, SUC_lexicon_tags_12 },
    { 0x53c765de, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x328765e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6765e5, SUC_lexicon_tags_11 },
    { 0x86765e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86765e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a765ec, SUC_lexicon_tags_3 },
    { 0x24e765ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2765f4, SUC_lexicon_tags_15 },
    { 0x248765f5, SUC_lexicon_tags_32 },
    { 0x7fe765f6, SUC_lexicon_tags_43 },
    { 0xe44765f7, SUC_lexicon_tags_11 },
    { 0xeb0765f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8765fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a76602, SUC_lexicon_tags_43 },
    { 0x5f076602, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x19a76605, SUC_lexicon_tags_91 },
    { 0xdb876606, SUC_lexicon_tags_17 },
    { 0x1276606, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde476609, SUC_lexicon_tags_1 },
    { 0xe187660a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e7660d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d876614, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60876616, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa907661e, SUC_lexicon_tags_30 },
    { 0x23e7661e, SUC_lexicon_tags_38 },
    { 0x27a76620, SUC_lexicon_tags_26 },
    { 0xe6a7661f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x11276623, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e76628, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83a7662a, SUC_lexicon_tags_79 },
    { 0xc67662a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49676631, SUC_lexicon_tags_37 },
    { 0xce76631, SUC_lexicon_tags_49 },
    { 0x36c76631, SUC_lexicon_tags_8 },
    { 0x11276633, SUC_lexicon_tags_1617 },
    { 0xe9e76635, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c7663a, SUC_lexicon_tags_37 },
    { 0x2587663a, SUC_lexicon_tags_7 },
    { 0xdc27663a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7747663f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd476641, SUC_lexicon_tags_63 },
    { 0xbae76641, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe4476644, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3587664a, SUC_lexicon_tags_32 },
    { 0x3687664a, SUC_lexicon_tags_12 },
    { 0xf647664a, SUC_lexicon_tags_2 },
    { 0xa007664a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3ee7664f, SUC_lexicon_tags_61 },
    { 0xfce76650, SUC_lexicon_tags_14 },
    { 0xc0a7664f, SUC_lexicon_tags_0 },
    { 0xe9876652, SUC_lexicon_tags_11 },
    { 0x92c76652, SUC_lexicon_tags_33 },
    { 0x54e76654, SUC_lexicon_tags_0 },
    { 0xc0c76652, SUC_lexicon_tags_11 },
    { 0x84876653, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d276658, SUC_lexicon_tags_12 },
    { 0xaa076659, SUC_lexicon_tags_12 },
    { 0xac07665a, SUC_lexicon_tags_46 },
    { 0xdf87665b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf267665f, SUC_lexicon_tags_1 },
    { 0xf6a76660, SUC_lexicon_tags_25 },
    { 0x36276661, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6076664, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58476668, SUC_lexicon_tags_19 },
    { 0x2d676669, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc527666c, SUC_lexicon_tags_33 },
    { 0x3e87666d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c76670, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x95076673, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xbfc76675, SUC_lexicon_tags_19 },
    { 0xa3676676, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68276679, SUC_lexicon_tags_109 },
    { 0xcf87667a, SUC_lexicon_tags_12 },
    { 0xd5e7667a, SUC_lexicon_tags_13 },
    { 0x8dc7667b, SUC_lexicon_tags_37 },
    { 0xbc87667c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e07667f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a7668d, SUC_lexicon_tags_19 },
    { 0x3e87668e, SUC_lexicon_tags_29 },
    { 0xa907668f, SUC_lexicon_tags_35 },
    { 0x9027668d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6ce76692, SUC_lexicon_tags_26 },
    { 0x59a76693, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c476696, SUC_lexicon_tags_7 },
    { 0xfc476697, SUC_lexicon_tags_37 },
    { 0xca276698, SUC_lexicon_tags_7 },
    { 0x81876699, SUC_lexicon_tags_27 },
    { 0x49c7669a, SUC_lexicon_tags_14 },
    { 0xd427669a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4ec7669d, SUC_lexicon_tags_7 },
    { 0xc767669e, SUC_lexicon_tags_19 },
    { 0xfc67669e, SUC_lexicon_tags_37 },
    { 0x7e7669f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba766a3, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204766a7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x13e766a9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda766ac, SUC_lexicon_tags_59 },
    { 0x71e766ad, SUC_lexicon_tags_6 },
    { 0xff6766ae, SUC_lexicon_tags_27 },
    { 0x95c766af, SUC_lexicon_tags_37 },
    { 0xbc8766b0, SUC_lexicon_tags_12 },
    { 0xec0766af, SUC_lexicon_tags_23 },
    { 0xd9c766b2, SUC_lexicon_tags_11 },
    { 0xbd4766b1, SUC_lexicon_tags_7 },
    { 0x136766af, SUC_lexicon_tags_25 },
    { 0x402766b5, SUC_lexicon_tags_6 },
    { 0x51c766b6, SUC_lexicon_tags_2 },
    { 0xae0766b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x212766b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeec766bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84766c0, SUC_lexicon_tags_1 },
    { 0x26a766c1, SUC_lexicon_tags_15 },
    { 0x37c766c2, SUC_lexicon_tags_19 },
    { 0x190766c3, SUC_lexicon_tags_45 },
    { 0x9bc766c4, SUC_lexicon_tags_38 },
    { 0x160766c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x720766c7, SUC_lexicon_tags_2 },
    { 0x680766c8, SUC_lexicon_tags_7 },
    { 0xd30766c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c766cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2766d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d6766d2, SUC_lexicon_tags_30 },
    { 0xde8766d3, SUC_lexicon_tags_41 },
    { 0x584766d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x44a766d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8766d8, SUC_lexicon_tags_2 },
    { 0xb24766d9, SUC_lexicon_tags_37 },
    { 0x978766d9, SUC_lexicon_tags_46 },
    { 0xd90766da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x886766dd, SUC_lexicon_tags_60 },
    { 0x274766de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x118766e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x374766e3, SUC_lexicon_tags_49 },
    { 0x8d0766e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36766ed, SUC_lexicon_tags_15 },
    { 0x9b0766ed, SUC_lexicon_tags_17 },
    { 0xb72766ef, SUC_lexicon_tags_11 },
    { 0x93a766f0, SUC_lexicon_tags_23 },
    { 0x9a2766ed, SUC_lexicon_tags_11 },
    { 0xed4766f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98766f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a766fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6766fd, SUC_lexicon_tags_3 },
    { 0xc5e766fd, SUC_lexicon_tags_12 },
    { 0xdba766fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15076703, SUC_lexicon_tags_11 },
    { 0xbae76703, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c876707, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47670a, SUC_lexicon_tags_31 },
    { 0xdd07670b, SUC_lexicon_tags_29 },
    { 0x16c7670c, SUC_lexicon_tags_0 },
    { 0x9767670c, SUC_lexicon_tags_26 },
    { 0xe127670e, SUC_lexicon_tags_14 },
    { 0x11e7670e, SUC_lexicon_tags_15 },
    { 0x81c7670a, SUC_lexicon_tags_28 },
    { 0xd8e76711, SUC_lexicon_tags_49 },
    { 0x3c076712, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde47671b, SUC_lexicon_tags_11 },
    { 0x7487671b, SUC_lexicon_tags_14 },
    { 0x8967671d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae76723, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x97076725, SUC_lexicon_tags_12 },
    { 0x82e76726, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x83276728, SUC_lexicon_tags_12 },
    { 0x49076728, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc267672c, SUC_lexicon_tags_17 },
    { 0x2f87672d, SUC_lexicon_tags_30 },
    { 0x4ec7672e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b276733, SUC_lexicon_tags_12 },
    { 0xbe476734, SUC_lexicon_tags_19 },
    { 0xef076735, SUC_lexicon_tags_60 },
    { 0xbe676735, SUC_lexicon_tags_2 },
    { 0x14276737, SUC_lexicon_tags_15 },
    { 0xa2876738, SUC_lexicon_tags_38 },
    { 0xba76738, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9407673d, SUC_lexicon_tags_14 },
    { 0x50a7673e, SUC_lexicon_tags_33 },
    { 0x16a7673f, SUC_lexicon_tags_47 },
    { 0xd6e76740, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc276747, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c7674d, SUC_lexicon_tags_33 },
    { 0x20e7674e, SUC_lexicon_tags_12 },
    { 0xb727674f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a76758, SUC_lexicon_tags_56 },
    { 0x41c76759, SUC_lexicon_tags_30 },
    { 0xa187675a, SUC_lexicon_tags_0 },
    { 0x9387675a, SUC_lexicon_tags_2 },
    { 0xdda7675a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0876760, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c76764, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc276767, SUC_lexicon_tags_47 },
    { 0xe7676768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6007676b, SUC_lexicon_tags_14 },
    { 0xc547676c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a47676f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x77476771, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cc76773, SUC_lexicon_tags_17 },
    { 0x35676773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6087677b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fc7677d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4076780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1076784, SUC_lexicon_tags_14 },
    { 0x51c76785, SUC_lexicon_tags_45 },
    { 0xd4c76784, SUC_lexicon_tags_25 },
    { 0x19e76787, SUC_lexicon_tags_37 },
    { 0x9c676788, SUC_lexicon_tags_25 },
    { 0x1b076789, SUC_lexicon_tags_25 },
    { 0xe867678a, SUC_lexicon_tags_25 },
    { 0x7507678b, SUC_lexicon_tags_58 },
    { 0x46c7678a, SUC_lexicon_tags_45 },
    { 0xa3076789, SUC_lexicon_tags_45 },
    { 0x8a7678e, SUC_lexicon_tags_13 },
    { 0xfce7678f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec76796, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc47679a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf687679c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24e7679e, SUC_lexicon_tags_49 },
    { 0xb7c7679f, SUC_lexicon_tags_42 },
    { 0x452767a0, SUC_lexicon_tags_17 },
    { 0xf9e767a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22767a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe06767a8, SUC_lexicon_tags_15 },
    { 0xf4a767a9, SUC_lexicon_tags_25 },
    { 0xa30767aa, SUC_lexicon_tags_69 },
    { 0xd88767a9, SUC_lexicon_tags_6 },
    { 0x930767ac, SUC_lexicon_tags_28 },
    { 0xc0c767ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8767b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4ee767b2, SUC_lexicon_tags_12 },
    { 0xed4767b2, SUC_lexicon_tags_27 },
    { 0x254767b4, SUC_lexicon_tags_1 },
    { 0x98a767b2, SUC_lexicon_tags_14 },
    { 0xcfa767b4, SUC_lexicon_tags_17 },
    { 0xe76767b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964767be, SUC_lexicon_tags_49 },
    { 0x1bc767bf, SUC_lexicon_tags_0 },
    { 0x14767bf, SUC_lexicon_tags_47 },
    { 0xe3c767be, SUC_lexicon_tags_23 },
    { 0x416767be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xade767c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x640767c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92767ca, SUC_lexicon_tags_10 },
    { 0x174767cb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2767cf, SUC_lexicon_tags_3 },
    { 0x9d4767cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64c767d2, SUC_lexicon_tags_38 },
    { 0x68e767d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca767dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54767eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2767ee, SUC_lexicon_tags_19 },
    { 0x35c767ef, SUC_lexicon_tags_84 },
    { 0x94e767f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60e767f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc767f6, SUC_lexicon_tags_53 },
    { 0x76767f7, SUC_lexicon_tags_27 },
    { 0xbc767f8, SUC_lexicon_tags_15 },
    { 0xc06767f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c767fc, SUC_lexicon_tags_23 },
    { 0xf6e767fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a76800, SUC_lexicon_tags_47 },
    { 0x10276801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a676803, SUC_lexicon_tags_2 },
    { 0x33a76804, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0876807, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a876816, SUC_lexicon_tags_7 },
    { 0xeb076817, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf676819, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e76823, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfe876825, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c7682b, SUC_lexicon_tags_17 },
    { 0xc107682c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47276831, SUC_lexicon_tags_25 },
    { 0x9e76831, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50476836, SUC_lexicon_tags_33 },
    { 0x81676837, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a7683a, SUC_lexicon_tags_21 },
    { 0xf727683b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x60e7683d, SUC_lexicon_tags_25 },
    { 0xad87683d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8476841, SUC_lexicon_tags_1 },
    { 0xb3c76842, SUC_lexicon_tags_49 },
    { 0xe1276843, SUC_lexicon_tags_25 },
    { 0x15076843, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x14476847, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec87684b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e67684f, SUC_lexicon_tags_2 },
    { 0x1676850, SUC_lexicon_tags_17 },
    { 0xd4476851, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9c76853, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c276856, SUC_lexicon_tags_13 },
    { 0xb7476857, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7987685a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7c7685c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59c7685e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8076861, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xe876863, SUC_lexicon_tags_7 },
    { 0x53c76864, SUC_lexicon_tags_11 },
    { 0x73876864, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfa676867, SUC_lexicon_tags_7 },
    { 0x49676867, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a7686c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a87686f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c276871, SUC_lexicon_tags_1 },
    { 0xc7076871, SUC_lexicon_tags_7 },
    { 0xb9076873, SUC_lexicon_tags_23 },
    { 0x93876874, SUC_lexicon_tags_61 },
    { 0xbfc76875, SUC_lexicon_tags_44 },
    { 0xd8e76876, SUC_lexicon_tags_10 },
    { 0xbe476876, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xea476879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47687c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c76880, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e76885, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xb676888, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a76890, SUC_lexicon_tags_11 },
    { 0x3da76890, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6476893, SUC_lexicon_tags_10 },
    { 0xcb476894, SUC_lexicon_tags_16 },
    { 0x5ba76895, SUC_lexicon_tags_0 },
    { 0x85476895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be7689b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc7689e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242768a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6768a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8ae768a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06768ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8768ae, SUC_lexicon_tags_4 },
    { 0x934768af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x338768b2, SUC_lexicon_tags_25 },
    { 0x640768b3, SUC_lexicon_tags_25 },
    { 0x8a8768b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e768ba, SUC_lexicon_tags_43 },
    { 0x5be768bb, SUC_lexicon_tags_7 },
    { 0xba0768bc, SUC_lexicon_tags_7 },
    { 0x51e768bd, SUC_lexicon_tags_60 },
    { 0x67e768be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c768c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc24768c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x990768c7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x68768c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58768cb, SUC_lexicon_tags_27 },
    { 0x1da768cc, SUC_lexicon_tags_37 },
    { 0x228768cd, SUC_lexicon_tags_10 },
    { 0x9ce768ce, SUC_lexicon_tags_34 },
    { 0x4ce768cf, SUC_lexicon_tags_6 },
    { 0x8c8768d0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xeca768d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4768d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e768da, SUC_lexicon_tags_11 },
    { 0x602768da, SUC_lexicon_tags_15 },
    { 0x8a4768db, SUC_lexicon_tags_25 },
    { 0x676768dd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8768e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x50c768e6, SUC_lexicon_tags_6 },
    { 0xecc768e7, SUC_lexicon_tags_4 },
    { 0x8da768e7, SUC_lexicon_tags_3 },
    { 0xaa0768e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912768ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66768f6, SUC_lexicon_tags_1 },
    { 0x16768f6, SUC_lexicon_tags_57 },
    { 0x7b8768f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x274768fb, SUC_lexicon_tags_0 },
    { 0xa00768fc, SUC_lexicon_tags_14 },
    { 0xd28768fb, SUC_lexicon_tags_18 },
    { 0xb6a768fb, SUC_lexicon_tags_1 },
    { 0x98e768ff, SUC_lexicon_tags_4 },
    { 0x424768ff, SUC_lexicon_tags_38 },
    { 0xc1476901, SUC_lexicon_tags_15 },
    { 0xb1876901, SUC_lexicon_tags_19 },
    { 0xedc76903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7876908, SUC_lexicon_tags_7 },
    { 0x1ec76908, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd807690e, SUC_lexicon_tags_17 },
    { 0xcde7690e, SUC_lexicon_tags_16 },
    { 0x6087690f, SUC_lexicon_tags_8 },
    { 0x58676911, SUC_lexicon_tags_29 },
    { 0x1f876911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1276919, SUC_lexicon_tags_12 },
    { 0xf47691a, SUC_lexicon_tags_15 },
    { 0x1d67691b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e76921, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd676923, SUC_lexicon_tags_8 },
    { 0xe0a76924, SUC_lexicon_tags_45 },
    { 0xc5e76925, SUC_lexicon_tags_12 },
    { 0x3ac76926, SUC_lexicon_tags_9 },
    { 0x2ca76927, SUC_lexicon_tags_45 },
    { 0x2a476928, SUC_lexicon_tags_26 },
    { 0x65e76928, SUC_lexicon_tags_47 },
    { 0x9ae76924, SUC_lexicon_tags_38 },
    { 0xd8476924, SUC_lexicon_tags_14 },
    { 0xc5e76923, SUC_lexicon_tags_12 },
    { 0x2de7692d, SUC_lexicon_tags_29 },
    { 0xd07692a, SUC_lexicon_tags_91 },
    { 0x5b676925, SUC_lexicon_tags_11 },
    { 0x95c7692d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28676932, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7676939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e27693d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29276944, SUC_lexicon_tags_11 },
    { 0xa2a76944, SUC_lexicon_tags_29 },
    { 0xa8876946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94c76948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae47694a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa507694e, SUC_lexicon_tags_26 },
    { 0x27c7694f, SUC_lexicon_tags_9 },
    { 0xdf876950, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xabe76952, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc676955, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf276958, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a27695e, SUC_lexicon_tags_15 },
    { 0x9187695f, SUC_lexicon_tags_12 },
    { 0x36a76960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c676963, SUC_lexicon_tags_35 },
    { 0xb1676964, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc76967, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a7696a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c7696f, SUC_lexicon_tags_8 },
    { 0x1676970, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba76974, SUC_lexicon_tags_11 },
    { 0xde276975, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75076977, SUC_lexicon_tags_1 },
    { 0x9fe76977, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c7697d, SUC_lexicon_tags_23 },
    { 0x6ce7697e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea76981, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea76988, SUC_lexicon_tags_61 },
    { 0x23a76989, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c7698c, SUC_lexicon_tags_25 },
    { 0xbae7698d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57e7698f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6476991, SUC_lexicon_tags_9 },
    { 0x1ae76991, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac76999, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82a7699b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7447699e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6769a1, SUC_lexicon_tags_819 },
    { 0x4a6769a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x268769a5, SUC_lexicon_tags_7 },
    { 0xd38769a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f4769a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x670769ab, SUC_lexicon_tags_25 },
    { 0xa38769ac, SUC_lexicon_tags_11 },
    { 0xe06769ab, SUC_lexicon_tags_55 },
    { 0x6f8769ab, SUC_lexicon_tags_1 },
    { 0x472769af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x694769b2, SUC_lexicon_tags_12 },
    { 0x486769b2, SUC_lexicon_tags_29 },
    { 0x776769b4, SUC_lexicon_tags_2 },
    { 0xbd6769b5, SUC_lexicon_tags_12 },
    { 0x426769b6, SUC_lexicon_tags_46 },
    { 0xb1e769b6, SUC_lexicon_tags_38 },
    { 0xe96769b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x90a769ba, SUC_lexicon_tags_38 },
    { 0xab2769bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c6769bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb8769bf, SUC_lexicon_tags_32 },
    { 0x4c4769bf, SUC_lexicon_tags_43 },
    { 0xdaa769c0, SUC_lexicon_tags_11 },
    { 0xfbc769bf, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xb84769c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32769cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe2769ce, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06769d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c769e0, SUC_lexicon_tags_7 },
    { 0x1d2769e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fe769e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc769ec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820769f5, SUC_lexicon_tags_0 },
    { 0xe5c769f6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e769fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6769fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c676a05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a076a07, SUC_lexicon_tags_7 },
    { 0x22876a07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xafc76a0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x20476a0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd476a0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25276a11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd876a13, SUC_lexicon_tags_31 },
    { 0x57a76a14, SUC_lexicon_tags_17 },
    { 0x20c76a15, SUC_lexicon_tags_7 },
    { 0x83476a16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2076a19, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3076a1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12276a1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec76a28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e476a2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27676a31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed876a33, SUC_lexicon_tags_87 },
    { 0xba76a33, SUC_lexicon_tags_25 },
    { 0xc276a34, SUC_lexicon_tags_60 },
    { 0x1cc76a36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a076a38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8fe76a3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11a76a3c, SUC_lexicon_tags_23 },
    { 0x1ee76a3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdc76a3f, SUC_lexicon_tags_87 },
    { 0xd5876a40, SUC_lexicon_tags_11 },
    { 0xeb476a40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9676a49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa476a4b, SUC_lexicon_tags_17 },
    { 0x66676a4b, SUC_lexicon_tags_11 },
    { 0xae476a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7076a53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c76a58, SUC_lexicon_tags_2 },
    { 0xc6876a59, SUC_lexicon_tags_21 },
    { 0x22c76a5a, SUC_lexicon_tags_38 },
    { 0x3ac76a5b, SUC_lexicon_tags_23 },
    { 0x24876a5c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc276a60, SUC_lexicon_tags_37 },
    { 0x27876a61, SUC_lexicon_tags_19 },
    { 0x59a76a61, SUC_lexicon_tags_1 },
    { 0xe3676a63, SUC_lexicon_tags_49 },
    { 0xf6476a63, SUC_lexicon_tags_25 },
    { 0xc9c76a65, SUC_lexicon_tags_55 },
    { 0x1c276a65, SUC_lexicon_tags_0 },
    { 0xecc76a63, SUC_lexicon_tags_0 },
    { 0x18676a66, SUC_lexicon_tags_17 },
    { 0x87876a65, SUC_lexicon_tags_25 },
    { 0xf6676a6a, SUC_lexicon_tags_37 },
    { 0xd076a61, SUC_lexicon_tags_56 },
    { 0x39276a6b, SUC_lexicon_tags_4 },
    { 0xd3076a6d, SUC_lexicon_tags_26 },
    { 0x59076a6e, SUC_lexicon_tags_47 },
    { 0xf3676a68, SUC_lexicon_tags_188 },
    { 0x2e276a62, SUC_lexicon_tags_11 },
    { 0xd5076a67, SUC_lexicon_tags_43 },
    { 0x64a76a62, SUC_lexicon_tags_19 },
    { 0xddc76a73, SUC_lexicon_tags_62 },
    { 0xb2276a6f, SUC_lexicon_tags_2 },
    { 0xd6676a75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec076a78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a76a7c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b476a85, SUC_lexicon_tags_2 },
    { 0x8ea76a85, SUC_lexicon_tags_17 },
    { 0xde876a87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35476a8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69876a98, SUC_lexicon_tags_7 },
    { 0x1c276a99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x10c76a9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c76a9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce676aa1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a76aa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b676aac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43a76aae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc876ab1, SUC_lexicon_tags_7 },
    { 0xdf476ab2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e476abc, SUC_lexicon_tags_118 },
    { 0x5e676abd, SUC_lexicon_tags_1 },
    { 0x8076abd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e76ac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0276aca, SUC_lexicon_tags_31 },
    { 0x75a76aca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b076ad2, SUC_lexicon_tags_37 },
    { 0x45476ad3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc276ad6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be76ada, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8676adc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1676adf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2ae76ae1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xac76ae3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xaee76ae5, SUC_lexicon_tags_15 },
    { 0x8b276ae6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x94476ae8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x20876aea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10276aef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17076af3, SUC_lexicon_tags_0 },
    { 0xb5a76af4, SUC_lexicon_tags_17 },
    { 0x42076af5, SUC_lexicon_tags_37 },
    { 0x3e76af6, SUC_lexicon_tags_12 },
    { 0x64a76af3, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0x8aa76af9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3076b00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b876b04, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a76b0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec76b11, SUC_lexicon_tags_4 },
    { 0x52276b12, SUC_lexicon_tags_37 },
    { 0x2b876b13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58676b17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc76b1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77676b1e, SUC_lexicon_tags_27 },
    { 0xbcc76b1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60076b21, SUC_lexicon_tags_11 },
    { 0x1476b21, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c76b25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76b2a, SUC_lexicon_tags_23 },
    { 0x1a676b2b, SUC_lexicon_tags_37 },
    { 0xcf276b2c, SUC_lexicon_tags_23 },
    { 0x92076b2a, SUC_lexicon_tags_12 },
    { 0xd9e76b2e, SUC_lexicon_tags_33 },
    { 0xcac76b2f, SUC_lexicon_tags_27 },
    { 0xee76b2f, SUC_lexicon_tags_87 },
    { 0xb9e76b31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95276b33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19676b36, SUC_lexicon_tags_25 },
    { 0xb8476b37, SUC_lexicon_tags_2 },
    { 0x65476b37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa76b3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43c76b3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b876b41, SUC_lexicon_tags_47 },
    { 0xc5276b42, SUC_lexicon_tags_77 },
    { 0xf0c76b43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c76b45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb276b47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7476b4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xacc76b4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8a76b4e, SUC_lexicon_tags_6 },
    { 0x6fa76b4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x80076b52, SUC_lexicon_tags_17 },
    { 0xc1a76b52, SUC_lexicon_tags_4 },
    { 0x16876b54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a76b58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da76b5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d476b5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84a76b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f676b63, SUC_lexicon_tags_47 },
    { 0x7b076b64, SUC_lexicon_tags_17 },
    { 0x91e76b65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e76b6a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcc876b6c, SUC_lexicon_tags_17 },
    { 0xc9a76b6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf676b71, SUC_lexicon_tags_21 },
    { 0xa9a76b72, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea76b76, SUC_lexicon_tags_39 },
    { 0x10e76b77, SUC_lexicon_tags_38 },
    { 0xc0076b78, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf8876b7a, SUC_lexicon_tags_6 },
    { 0x13876b7b, SUC_lexicon_tags_2 },
    { 0xe6e76b7c, SUC_lexicon_tags_7 },
    { 0x56c76b7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3476b83, SUC_lexicon_tags_64 },
    { 0xc876b84, SUC_lexicon_tags_21 },
    { 0x71476b83, SUC_lexicon_tags_25 },
    { 0xc4c76b85, SUC_lexicon_tags_14 },
    { 0x2f876b83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x83676b89, SUC_lexicon_tags_30 },
    { 0xd9476b89, SUC_lexicon_tags_33 },
    { 0x54a76b8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94276b8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd2676b91, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12676b95, SUC_lexicon_tags_47 },
    { 0x47476b96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21476b9e, SUC_lexicon_tags_98 },
    { 0xc0676b9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a76ba2, SUC_lexicon_tags_47 },
    { 0xf4676ba3, SUC_lexicon_tags_17 },
    { 0x7d876ba4, SUC_lexicon_tags_2 },
    { 0xaf676ba5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47076ba7, SUC_lexicon_tags_17 },
    { 0x8b076ba7, SUC_lexicon_tags_25 },
    { 0xe6676ba9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81876bab, SUC_lexicon_tags_26 },
    { 0xae076bac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c76baf, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a876bb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e76bb6, SUC_lexicon_tags_12 },
    { 0xaa276bb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49476bbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73c76bbf, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x36876bc2, SUC_lexicon_tags_61 },
    { 0x7d476bc2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e76bc7, SUC_lexicon_tags_17 },
    { 0x94076bc8, SUC_lexicon_tags_60 },
    { 0xcae76bc8, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8676bcf, SUC_lexicon_tags_9 },
    { 0x5b476bcf, SUC_lexicon_tags_25 },
    { 0x6fe76bd1, SUC_lexicon_tags_710 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da76bd5, SUC_lexicon_tags_10 },
    { 0xaee76bd6, SUC_lexicon_tags_2 },
    { 0xeac76bd7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87476bdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce76be1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x76676be4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec76be7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83476bea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf076bec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10076bf1, SUC_lexicon_tags_15 },
    { 0xbcc76bf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc76bf8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa76bfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9276bff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8876c0a, SUC_lexicon_tags_12 },
    { 0x8b076c0a, SUC_lexicon_tags_29 },
    { 0x46076c0c, SUC_lexicon_tags_7 },
    { 0x49276c0d, SUC_lexicon_tags_16 },
    { 0x71c76c0d, SUC_lexicon_tags_37 },
    { 0x99a76c0b, SUC_lexicon_tags_47 },
    { 0xb5676c0d, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x10676c12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f276c14, SUC_lexicon_tags_7 },
    { 0x51276c15, SUC_lexicon_tags_92 },
    { 0x8fc76c16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3c276c18, SUC_lexicon_tags_27 },
    { 0x53476c18, SUC_lexicon_tags_30 },
    { 0x57076c19, SUC_lexicon_tags_55 },
    { 0xbb276c1b, SUC_lexicon_tags_39 },
    { 0x26476c1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa276c1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc76c21, SUC_lexicon_tags_33 },
    { 0x9276c22, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3076c28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x74276c2a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x67076c2c, SUC_lexicon_tags_14 },
    { 0xde76c2c, SUC_lexicon_tags_26 },
    { 0x4f076c2e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e76c31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b276c35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c876c39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d676c40, SUC_lexicon_tags_10 },
    { 0xc3276c40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a76c4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77476c4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49e76c50, SUC_lexicon_tags_12 },
    { 0x40276c51, SUC_lexicon_tags_77 },
    { 0x3da76c52, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0x29276c54, SUC_lexicon_tags_7 },
    { 0xe7076c55, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e76c58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a76c5b, SUC_lexicon_tags_49 },
    { 0x48c76c5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33676c5f, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x61676c62, SUC_lexicon_tags_32 },
    { 0xbda76c63, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a76c66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4476c6f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b076c72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc76c7b, SUC_lexicon_tags_63 },
    { 0x1e476c7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a76c80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a476c82, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x68076c84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16876c86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5de76c88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc8276c8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23676c8e, SUC_lexicon_tags_150 },
    { 0x24c76c8f, SUC_lexicon_tags_19 },
    { 0xf3e76c8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63c76c92, SUC_lexicon_tags_61 },
    { 0xf9e76c93, SUC_lexicon_tags_25 },
    { 0x8a76c94, SUC_lexicon_tags_25 },
    { 0x92e76c95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc76c99, SUC_lexicon_tags_60 },
    { 0xfca76c9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2e76c9c, SUC_lexicon_tags_27 },
    { 0x1aa76c9d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc676ca1, SUC_lexicon_tags_62 },
    { 0xf0676ca2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9476cac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e276caf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e76cb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf076cbc, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c76cc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c76cca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4876cce, SUC_lexicon_tags_30 },
    { 0x4a676ccf, SUC_lexicon_tags_11 },
    { 0xf9276cd0, SUC_lexicon_tags_37 },
    { 0x70e76cd0, SUC_lexicon_tags_15 },
    { 0xba076cd2, SUC_lexicon_tags_21 },
    { 0xcd476cd3, SUC_lexicon_tags_901 },
    { 0x8fe76cd3, SUC_lexicon_tags_8 },
    { 0x71276cd1, SUC_lexicon_tags_33 },
    { 0xf1676cd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47076cd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c076cda, SUC_lexicon_tags_6 },
    { 0x68276cdb, SUC_lexicon_tags_44 },
    { 0x1b876cdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a76cdf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa876ce2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee076cea, SUC_lexicon_tags_25 },
    { 0x39c76ceb, SUC_lexicon_tags_27 },
    { 0x8d276ceb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b276cf8, SUC_lexicon_tags_79 },
    { 0xad076cf8, SUC_lexicon_tags_60 },
    { 0x57476cfa, SUC_lexicon_tags_25 },
    { 0x8a76cfa, SUC_lexicon_tags_37 },
    { 0x57476cfc, SUC_lexicon_tags_17 },
    { 0x95476cfd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x56c76cff, SUC_lexicon_tags_11 },
    { 0xa0c76d00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc76d04, SUC_lexicon_tags_25 },
    { 0xe1076d05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11476d08, SUC_lexicon_tags_37 },
    { 0x79a76d08, SUC_lexicon_tags_2 },
    { 0x3a876d0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82876d0e, SUC_lexicon_tags_48 },
    { 0xe476d0f, SUC_lexicon_tags_8 },
    { 0xc2276d10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45676d16, SUC_lexicon_tags_15 },
    { 0x9fc76d17, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c76d1a, SUC_lexicon_tags_2 },
    { 0x3c876d1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9276d1e, SUC_lexicon_tags_11 },
    { 0x31476d1e, SUC_lexicon_tags_17 },
    { 0x81076d20, SUC_lexicon_tags_7 },
    { 0xe0876d21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79a76d23, SUC_lexicon_tags_35 },
    { 0xfa076d23, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xbe876d26, SUC_lexicon_tags_82 },
    { 0xa6076d27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc76d2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46676d30, SUC_lexicon_tags_33 },
    { 0x1de76d31, SUC_lexicon_tags_7 },
    { 0x8da76d32, SUC_lexicon_tags_14 },
    { 0xb3c76d33, SUC_lexicon_tags_65 },
    { 0x2ce76d34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f276d3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0876d3f, SUC_lexicon_tags_37 },
    { 0x7da76d40, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x51c76d42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e76d45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a76d4b, SUC_lexicon_tags_33 },
    { 0xe2876d4c, SUC_lexicon_tags_60 },
    { 0xf0a76d4c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x77876d4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75876d52, SUC_lexicon_tags_15 },
    { 0xbc76d53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e676d58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31c76d5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27476d5f, SUC_lexicon_tags_38 },
    { 0xd4e76d60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1476d63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a876d68, SUC_lexicon_tags_17 },
    { 0xece76d69, SUC_lexicon_tags_17 },
    { 0x7f076d6a, SUC_lexicon_tags_33 },
    { 0xf0c76d68, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac76d6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82076d71, SUC_lexicon_tags_47 },
    { 0x3f876d72, SUC_lexicon_tags_12 },
    { 0x56076d73, SUC_lexicon_tags_17 },
    { 0x37676d74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c76d78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9276d7e, SUC_lexicon_tags_34 },
    { 0x7ae76d7e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56076d87, SUC_lexicon_tags_17 },
    { 0x2b876d87, SUC_lexicon_tags_37 },
    { 0x6ac76d88, SUC_lexicon_tags_30 },
    { 0xc7e76d89, SUC_lexicon_tags_56 },
    { 0x57a76d88, SUC_lexicon_tags_45 },
    { 0xf8c76d87, SUC_lexicon_tags_10 },
    { 0x61676d8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41676d91, SUC_lexicon_tags_25 },
    { 0x88476d91, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x54076d94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87e76d96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d876d99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9a76d9b, SUC_lexicon_tags_34 },
    { 0x11876d9c, SUC_lexicon_tags_37 },
    { 0x7d676d9b, SUC_lexicon_tags_28 },
    { 0xde876d9e, SUC_lexicon_tags_26 },
    { 0xef476d9f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79276da3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a76daa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7276dad, SUC_lexicon_tags_44 },
    { 0xf8876dad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e276db2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a076db8, SUC_lexicon_tags_38 },
    { 0x31276db9, SUC_lexicon_tags_15 },
    { 0x2c076dba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1a76dbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c876dc7, SUC_lexicon_tags_49 },
    { 0x75476dc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2476dcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6076dd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e76dd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85876ddd, SUC_lexicon_tags_38 },
    { 0x3da76ddd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e476de0, SUC_lexicon_tags_38 },
    { 0x4f076de1, SUC_lexicon_tags_47 },
    { 0x7276de0, SUC_lexicon_tags_14 },
    { 0xd3a76de1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52276de5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2e76de7, SUC_lexicon_tags_25 },
    { 0xe276de8, SUC_lexicon_tags_12 },
    { 0x7be76de9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2676dec, SUC_lexicon_tags_9 },
    { 0x4aa76ded, SUC_lexicon_tags_731 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c76df4, SUC_lexicon_tags_2 },
    { 0x4c876df5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8276df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55476dfe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea76e03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0076e06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a876e08, SUC_lexicon_tags_7 },
    { 0xba76e08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec276e0b, SUC_lexicon_tags_67 },
    { 0x57276e0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc876e12, SUC_lexicon_tags_63 },
    { 0xb2876e12, SUC_lexicon_tags_6 },
    { 0xf3276e13, SUC_lexicon_tags_31 },
    { 0xd2276e15, SUC_lexicon_tags_25 },
    { 0x8a76e12, SUC_lexicon_tags_12 },
    { 0x53676e16, SUC_lexicon_tags_47 },
    { 0x56e76e18, SUC_lexicon_tags_410 },
    { 0x7b476e14, SUC_lexicon_tags_32 },
    { 0xbc276e1a, SUC_lexicon_tags_84 },
    { 0x4cc76e1a, SUC_lexicon_tags_2 },
    { 0x6a876e1c, SUC_lexicon_tags_8 },
    { 0x2ba76e1d, SUC_lexicon_tags_12 },
    { 0xe3076e1e, SUC_lexicon_tags_19 },
    { 0x42276e1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ce76e21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98476e23, SUC_lexicon_tags_28 },
    { 0xd1676e24, SUC_lexicon_tags_31 },
    { 0x69876e25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91276e2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d276e32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf476e35, SUC_lexicon_tags_19 },
    { 0x62876e36, SUC_lexicon_tags_0 },
    { 0x36c76e36, SUC_lexicon_tags_11 },
    { 0xb2276e38, SUC_lexicon_tags_45 },
    { 0x91476e38, SUC_lexicon_tags_27 },
    { 0xdea76e3a, SUC_lexicon_tags_25 },
    { 0xf1476e3a, SUC_lexicon_tags_1 },
    { 0x18c76e38, SUC_lexicon_tags_15 },
    { 0xd9c76e3d, SUC_lexicon_tags_14 },
    { 0x2ea76e3a, SUC_lexicon_tags_46 },
    { 0x3b876e3c, SUC_lexicon_tags_25 },
    { 0xd6c76e40, SUC_lexicon_tags_17 },
    { 0x1ae76e38, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c76e45, SUC_lexicon_tags_9 },
    { 0x4476e46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec276e4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62676e4c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac76e50, SUC_lexicon_tags_28 },
    { 0x22676e51, SUC_lexicon_tags_15 },
    { 0xce676e52, SUC_lexicon_tags_10 },
    { 0xaae76e53, SUC_lexicon_tags_7 },
    { 0x9b276e52, SUC_lexicon_tags_37 },
    { 0x6e476e55, SUC_lexicon_tags_17 },
    { 0x40c76e56, SUC_lexicon_tags_7 },
    { 0xe5a76e57, SUC_lexicon_tags_15 },
    { 0xa6476e58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb476e5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd8076e5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96676e61, SUC_lexicon_tags_13 },
    { 0x6f076e62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4076e65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10876e69, SUC_lexicon_tags_7 },
    { 0x5e676e69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b276e6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc76e70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40676e73, SUC_lexicon_tags_25 },
    { 0x8cc76e73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x75276e77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x34076e79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6676e7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7276e80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0c76e82, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a76e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6676e8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51876e93, SUC_lexicon_tags_15 },
    { 0x68276e93, SUC_lexicon_tags_11 },
    { 0xc2e76e95, SUC_lexicon_tags_63 },
    { 0x68076e95, SUC_lexicon_tags_19 },
    { 0x85476e95, SUC_lexicon_tags_87 },
    { 0xac876e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20676e9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8076e9f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaf676ea1, SUC_lexicon_tags_45 },
    { 0xc0476ea2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e76ea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca476ead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e76eb5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb8e76eb7, SUC_lexicon_tags_9 },
    { 0xb0c76eb7, SUC_lexicon_tags_26 },
    { 0x55a76eb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe876ebc, SUC_lexicon_tags_23 },
    { 0x8c276ebd, SUC_lexicon_tags_15 },
    { 0x5ca76ebc, SUC_lexicon_tags_12 },
    { 0xa4c76ebf, SUC_lexicon_tags_0 },
    { 0xc2476ebf, SUC_lexicon_tags_56 },
    { 0x62676ebc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7a76ec3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4da76ec5, SUC_lexicon_tags_12 },
    { 0x6f476ec6, SUC_lexicon_tags_15 },
    { 0xf9c76ec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5a76ec9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c76ece, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55876ed0, SUC_lexicon_tags_55 },
    { 0xb0876ed1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e76ed6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90876edc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e076edf, SUC_lexicon_tags_15 },
    { 0x17076edf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba676ee2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5076ee8, SUC_lexicon_tags_11 },
    { 0xe2c76ee9, SUC_lexicon_tags_17 },
    { 0xf4e76eea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ba76eec, SUC_lexicon_tags_10 },
    { 0x2876eed, SUC_lexicon_tags_55 },
    { 0x93e76eee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9276ef4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d276ef8, SUC_lexicon_tags_1 },
    { 0x72e76ef9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83076eff, SUC_lexicon_tags_11 },
    { 0xa5c76f00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca876f04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xef476f06, SUC_lexicon_tags_15 },
    { 0xb6e76f07, SUC_lexicon_tags_45 },
    { 0xcca76f08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe276f0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7676f13, SUC_lexicon_tags_57 },
    { 0x2b076f14, SUC_lexicon_tags_188 },
    { 0xa1c76f15, SUC_lexicon_tags_68 },
    { 0xbc476f14, SUC_lexicon_tags_10 },
    { 0x8b076f17, SUC_lexicon_tags_12 },
    { 0x74a76f18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33e76f1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4276f1d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcfe76f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d076f25, SUC_lexicon_tags_49 },
    { 0xe0876f25, SUC_lexicon_tags_30 },
    { 0x1d276f26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95476f2d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1f076f2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53e76f31, SUC_lexicon_tags_6 },
    { 0x87076f32, SUC_lexicon_tags_14 },
    { 0xd3e76f32, SUC_lexicon_tags_37 },
    { 0x9bc76f34, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b476f37, SUC_lexicon_tags_7 },
    { 0x676f37, SUC_lexicon_tags_47 },
    { 0xa6876f39, SUC_lexicon_tags_38 },
    { 0x4ce76f38, SUC_lexicon_tags_49 },
    { 0xad876f3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6076f3e, SUC_lexicon_tags_31 },
    { 0x39476f3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbca76f41, SUC_lexicon_tags_17 },
    { 0x8a76f41, SUC_lexicon_tags_9 },
    { 0xe276f43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea076f45, SUC_lexicon_tags_12 },
    { 0x37c76f46, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x99076f49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a676f4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5476f4f, SUC_lexicon_tags_39 },
    { 0x35e76f50, SUC_lexicon_tags_12 },
    { 0xa0876f4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4676f53, SUC_lexicon_tags_16 },
    { 0xf8676f54, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaee76f56, SUC_lexicon_tags_23 },
    { 0xeb476f56, SUC_lexicon_tags_10 },
    { 0x9a276f58, SUC_lexicon_tags_8 },
    { 0x3dc76f56, SUC_lexicon_tags_37 },
    { 0x4ca76f57, SUC_lexicon_tags_37 },
    { 0x5be76f5b, SUC_lexicon_tags_12 },
    { 0x35276f57, SUC_lexicon_tags_43 },
    { 0xb876f5d, SUC_lexicon_tags_46 },
    { 0x86476f5e, SUC_lexicon_tags_7 },
    { 0xb8476f5f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x16876f61, SUC_lexicon_tags_25 },
    { 0xe5876f62, SUC_lexicon_tags_47 },
    { 0x98676f63, SUC_lexicon_tags_2 },
    { 0xf1276f63, SUC_lexicon_tags_47 },
    { 0xd9076f63, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61476f69, SUC_lexicon_tags_19 },
    { 0x4c76f6a, SUC_lexicon_tags_31 },
    { 0x76f6b, SUC_lexicon_tags_2 },
    { 0x9e276f6c, SUC_lexicon_tags_21 },
    { 0x2c76f6d, SUC_lexicon_tags_1 },
    { 0xcfa76f6e, SUC_lexicon_tags_7 },
    { 0x31076f6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e76f72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a76f75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43076f77, SUC_lexicon_tags_17 },
    { 0xe4076f78, SUC_lexicon_tags_23 },
    { 0x20876f77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80a76f7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47276f80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b676f87, SUC_lexicon_tags_63 },
    { 0x4e276f88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d276f8a, SUC_lexicon_tags_9 },
    { 0x89e76f8b, SUC_lexicon_tags_15 },
    { 0xd0876f8c, SUC_lexicon_tags_1 },
    { 0x62476f8c, SUC_lexicon_tags_17 },
    { 0x5876f8c, SUC_lexicon_tags_37 },
    { 0xe2476f8e, SUC_lexicon_tags_55 },
    { 0x6ec76f8f, SUC_lexicon_tags_120 },
    { 0x23076f91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3076f9c, SUC_lexicon_tags_31 },
    { 0xc7a76f9d, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc276fa3, SUC_lexicon_tags_12 },
    { 0x3c676fa3, SUC_lexicon_tags_14 },
    { 0xb4e76fa5, SUC_lexicon_tags_7 },
    { 0x76876fa6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a76fab, SUC_lexicon_tags_33 },
    { 0x2ca76fac, SUC_lexicon_tags_37 },
    { 0x1dc76fab, SUC_lexicon_tags_7 },
    { 0x9ca76fae, SUC_lexicon_tags_14 },
    { 0x32076faf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a76fb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc76fb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xff076fba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1476fbd, SUC_lexicon_tags_15 },
    { 0xe0476fbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44276fc0, SUC_lexicon_tags_60 },
    { 0x40876fc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15076fc6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e076fc9, SUC_lexicon_tags_30 },
    { 0x2be76fca, SUC_lexicon_tags_25 },
    { 0x3b876fca, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c76fce, SUC_lexicon_tags_49 },
    { 0xb7a76fce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e676fd3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc076fd7, SUC_lexicon_tags_65 },
    { 0x44076fd8, SUC_lexicon_tags_11 },
    { 0x86476fd9, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x82876fdb, SUC_lexicon_tags_16 },
    { 0x25076fdb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9b676fde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21076fe0, SUC_lexicon_tags_11 },
    { 0xf3476fe0, SUC_lexicon_tags_63 },
    { 0xd5476fe1, SUC_lexicon_tags_26 },
    { 0x73276fe1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15876fe7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd476feb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0c76fed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x79476fef, SUC_lexicon_tags_46 },
    { 0xf8676ff0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a676ff2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20476ff4, SUC_lexicon_tags_3 },
    { 0x4de76ff5, SUC_lexicon_tags_25 },
    { 0xafa76ff6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc77001, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc7700b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb467700d, SUC_lexicon_tags_30 },
    { 0xf507700e, SUC_lexicon_tags_15 },
    { 0x6847700f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca77014, SUC_lexicon_tags_17 },
    { 0x31077015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef877017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c7701a, SUC_lexicon_tags_33 },
    { 0xcca7701b, SUC_lexicon_tags_5 },
    { 0xf647701c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8077021, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99277024, SUC_lexicon_tags_44 },
    { 0xd0277025, SUC_lexicon_tags_25 },
    { 0x86877026, SUC_lexicon_tags_45 },
    { 0xcc877025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8227702c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac77039, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf007703d, SUC_lexicon_tags_25 },
    { 0xba7703e, SUC_lexicon_tags_17 },
    { 0xcd67703f, SUC_lexicon_tags_29 },
    { 0xf047703f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a77046, SUC_lexicon_tags_1 },
    { 0x9a277047, SUC_lexicon_tags_14 },
    { 0x6d277048, SUC_lexicon_tags_19 },
    { 0xb2277048, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8287704b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a47704e, SUC_lexicon_tags_65 },
    { 0xf8a7704f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da77052, SUC_lexicon_tags_28 },
    { 0xb3e77053, SUC_lexicon_tags_7 },
    { 0x2a477054, SUC_lexicon_tags_12 },
    { 0x8de77055, SUC_lexicon_tags_54 },
    { 0x48a77056, SUC_lexicon_tags_25 },
    { 0x82e77056, SUC_lexicon_tags_43 },
    { 0xa1677058, SUC_lexicon_tags_37 },
    { 0xe9077059, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e7705e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de77061, SUC_lexicon_tags_65 },
    { 0xdf077062, SUC_lexicon_tags_37 },
    { 0xe1477062, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c77066, SUC_lexicon_tags_7 },
    { 0x12a77066, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6027706f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c77072, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7877075, SUC_lexicon_tags_25 },
    { 0x16c77076, SUC_lexicon_tags_25 },
    { 0xac477076, SUC_lexicon_tags_114 },
    { 0x16a77078, SUC_lexicon_tags_38 },
    { 0xc0677077, SUC_lexicon_tags_48 },
    { 0x4277078, SUC_lexicon_tags_17 },
    { 0xba7707b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3f27707d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c077081, SUC_lexicon_tags_25 },
    { 0x82677082, SUC_lexicon_tags_15 },
    { 0x10477082, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9c77085, SUC_lexicon_tags_2 },
    { 0xd1477086, SUC_lexicon_tags_16 },
    { 0x48077086, SUC_lexicon_tags_37 },
    { 0x2aa77085, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2627708b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a67708d, SUC_lexicon_tags_81 },
    { 0xaa7708e, SUC_lexicon_tags_17 },
    { 0xda47708e, SUC_lexicon_tags_0 },
    { 0x5b07708d, SUC_lexicon_tags_12 },
    { 0xe0c77091, SUC_lexicon_tags_8 },
    { 0x56277090, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64677095, SUC_lexicon_tags_7 },
    { 0x6477095, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46c77098, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc87709b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1607709d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57a7709f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4770a4, SUC_lexicon_tags_37 },
    { 0x96a770a5, SUC_lexicon_tags_11 },
    { 0x8a6770a6, SUC_lexicon_tags_1 },
    { 0x8ae770a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc770ab, SUC_lexicon_tags_17 },
    { 0x56a770ac, SUC_lexicon_tags_10 },
    { 0x8a6770ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8770b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a770b4, SUC_lexicon_tags_7 },
    { 0x45c770b5, SUC_lexicon_tags_17 },
    { 0x9b4770b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34770be, SUC_lexicon_tags_11 },
    { 0x210770bf, SUC_lexicon_tags_27 },
    { 0xf42770be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2770c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca770cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6770da, SUC_lexicon_tags_16 },
    { 0xc80770db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8770dd, SUC_lexicon_tags_46 },
    { 0xdd0770de, SUC_lexicon_tags_14 },
    { 0x7ce770df, SUC_lexicon_tags_48 },
    { 0xf24770df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x440770e3, SUC_lexicon_tags_17 },
    { 0xd92770e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68770f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36770f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60770f6, SUC_lexicon_tags_6 },
    { 0xa86770f6, SUC_lexicon_tags_65 },
    { 0xc46770f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x964770fa, SUC_lexicon_tags_17 },
    { 0x2a0770fb, SUC_lexicon_tags_2 },
    { 0xf1c770fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ba770fe, SUC_lexicon_tags_7 },
    { 0xe0c770ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x34e77101, SUC_lexicon_tags_25 },
    { 0x71477101, SUC_lexicon_tags_44 },
    { 0xd5877103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9677106, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc3677108, SUC_lexicon_tags_11 },
    { 0xf2e77109, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd87710b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a77111, SUC_lexicon_tags_2 },
    { 0x67e77111, SUC_lexicon_tags_26 },
    { 0xcc877113, SUC_lexicon_tags_60 },
    { 0x92077114, SUC_lexicon_tags_60 },
    { 0xd877115, SUC_lexicon_tags_42 },
    { 0x4e877115, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6527711c, SUC_lexicon_tags_60 },
    { 0xa127711d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc87711f, SUC_lexicon_tags_38 },
    { 0x15477120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66877122, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7477124, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c77127, SUC_lexicon_tags_25 },
    { 0xac77128, SUC_lexicon_tags_1 },
    { 0xd0677127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee7712c, SUC_lexicon_tags_54 },
    { 0x7687712c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8be7712f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f877136, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac7713e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c77145, SUC_lexicon_tags_11 },
    { 0x31277145, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c47714b, SUC_lexicon_tags_179 },
    { 0x93a7714b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe787714e, SUC_lexicon_tags_4 },
    { 0xa1a7714f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab077152, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x75e77154, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e67715f, SUC_lexicon_tags_3 },
    { 0x5477160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f077166, SUC_lexicon_tags_133 },
    { 0x4477166, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c7716a, SUC_lexicon_tags_19 },
    { 0x6a67716b, SUC_lexicon_tags_38 },
    { 0x7b67716c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c7716f, SUC_lexicon_tags_37 },
    { 0x1e47716f, SUC_lexicon_tags_23 },
    { 0xd1277171, SUC_lexicon_tags_87 },
    { 0x90277172, SUC_lexicon_tags_37 },
    { 0xdac7716f, SUC_lexicon_tags_17 },
    { 0x36877174, SUC_lexicon_tags_25 },
    { 0x37677175, SUC_lexicon_tags_4 },
    { 0x607716f, SUC_lexicon_tags_217 },
    { 0x59477171, SUC_lexicon_tags_27 },
    { 0x10e77174, SUC_lexicon_tags_37 },
    { 0x97277176, SUC_lexicon_tags_12 },
    { 0x5a7717a, SUC_lexicon_tags_12 },
    { 0xbf67717b, SUC_lexicon_tags_16 },
    { 0x9b67717c, SUC_lexicon_tags_17 },
    { 0xd887717c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c77180, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0277187, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7718b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5187718e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77277192, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a77198, SUC_lexicon_tags_12 },
    { 0xb1477199, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb7e7719b, SUC_lexicon_tags_12 },
    { 0xe7a7719c, SUC_lexicon_tags_4 },
    { 0x7367719d, SUC_lexicon_tags_2 },
    { 0x68c7719b, SUC_lexicon_tags_8 },
    { 0x87c7719f, SUC_lexicon_tags_37 },
    { 0x64e7719c, SUC_lexicon_tags_25 },
    { 0x300771a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8771ae, SUC_lexicon_tags_27 },
    { 0x6c771ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618771b3, SUC_lexicon_tags_37 },
    { 0x2a6771b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60771b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6771c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc771c7, SUC_lexicon_tags_1 },
    { 0x710771c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb94771ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf72771cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x276771cf, SUC_lexicon_tags_8 },
    { 0x9a6771d0, SUC_lexicon_tags_2 },
    { 0x444771cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4771d9, SUC_lexicon_tags_12 },
    { 0x90a771d9, SUC_lexicon_tags_49 },
    { 0xe18771db, SUC_lexicon_tags_37 },
    { 0xab8771dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2771e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e771e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6771e7, SUC_lexicon_tags_3 },
    { 0x200771e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb2a771ea, SUC_lexicon_tags_19 },
    { 0x134771ea, SUC_lexicon_tags_53 },
    { 0x960771ec, SUC_lexicon_tags_17 },
    { 0xa38771ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x136771f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460771f6, SUC_lexicon_tags_12 },
    { 0x466771f7, SUC_lexicon_tags_1 },
    { 0xba8771f6, SUC_lexicon_tags_15 },
    { 0x458771f7, SUC_lexicon_tags_56 },
    { 0xf4e771fa, SUC_lexicon_tags_1 },
    { 0xb10771fb, SUC_lexicon_tags_37 },
    { 0x5da771fa, SUC_lexicon_tags_30 },
    { 0x466771fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15c771ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb677201, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26077204, SUC_lexicon_tags_29 },
    { 0xfb677204, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a7720a, SUC_lexicon_tags_3 },
    { 0xfce7720b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e77219, SUC_lexicon_tags_68 },
    { 0xad277219, SUC_lexicon_tags_37 },
    { 0x7887721b, SUC_lexicon_tags_11 },
    { 0x53477219, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec77220, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc77227, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd747722a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec7722e, SUC_lexicon_tags_12 },
    { 0x13e7722f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c77232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8477234, SUC_lexicon_tags_12 },
    { 0x57677235, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e7723d, SUC_lexicon_tags_70 },
    { 0xb987723d, SUC_lexicon_tags_11 },
    { 0xbf27723f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf677241, SUC_lexicon_tags_19 },
    { 0xb5277241, SUC_lexicon_tags_23 },
    { 0x38877243, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e77247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70877249, SUC_lexicon_tags_92 },
    { 0x9fa7724a, SUC_lexicon_tags_30 },
    { 0x2a27724b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e7724d, SUC_lexicon_tags_28 },
    { 0x2227724d, SUC_lexicon_tags_3 },
    { 0xad07724e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d877253, SUC_lexicon_tags_25 },
    { 0x37477254, SUC_lexicon_tags_11 },
    { 0xa7677254, SUC_lexicon_tags_77 },
    { 0xf8477255, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0677259, SUC_lexicon_tags_1 },
    { 0x5227725a, SUC_lexicon_tags_47 },
    { 0x73277259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd07725d, SUC_lexicon_tags_23 },
    { 0x2ee7725e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7877260, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1277262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7477264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5277267, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79477269, SUC_lexicon_tags_56 },
    { 0xc5a7726a, SUC_lexicon_tags_84 },
    { 0x17c7726a, SUC_lexicon_tags_38 },
    { 0x8947726c, SUC_lexicon_tags_29 },
    { 0x4327726c, SUC_lexicon_tags_10 },
    { 0xeea7726e, SUC_lexicon_tags_2 },
    { 0x1277269, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a77273, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4cc77275, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d277277, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7327727b, SUC_lexicon_tags_7 },
    { 0xe407727c, SUC_lexicon_tags_7 },
    { 0x6a47727d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a77280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf677283, SUC_lexicon_tags_40 },
    { 0x37277284, SUC_lexicon_tags_25 },
    { 0xbd877285, SUC_lexicon_tags_23 },
    { 0xd6e77285, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe7728c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51277290, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x14877292, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c77299, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7729d, SUC_lexicon_tags_1 },
    { 0xe987729d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba772a2, SUC_lexicon_tags_27 },
    { 0x468772a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xffa772a5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdfa772a7, SUC_lexicon_tags_28 },
    { 0xac6772a8, SUC_lexicon_tags_0 },
    { 0x648772a9, SUC_lexicon_tags_45 },
    { 0x2b0772a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38772b0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x648772b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222772b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c772bd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x826772bf, SUC_lexicon_tags_19 },
    { 0x1c0772c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd16772c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xba0772c4, SUC_lexicon_tags_12 },
    { 0x60e772c4, SUC_lexicon_tags_47 },
    { 0x21e772c6, SUC_lexicon_tags_47 },
    { 0xcee772c4, SUC_lexicon_tags_11 },
    { 0x8c6772c8, SUC_lexicon_tags_21 },
    { 0x57c772c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x190772cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x122772cf, SUC_lexicon_tags_15 },
    { 0x5f4772cf, SUC_lexicon_tags_37 },
    { 0x4dc772d1, SUC_lexicon_tags_2 },
    { 0x2e2772d2, SUC_lexicon_tags_29 },
    { 0x6da772d3, SUC_lexicon_tags_60 },
    { 0xdac772d4, SUC_lexicon_tags_46 },
    { 0x894772d5, SUC_lexicon_tags_14 },
    { 0x7ba772d3, SUC_lexicon_tags_25 },
    { 0x6c4772d7, SUC_lexicon_tags_37 },
    { 0x912772d3, SUC_lexicon_tags_37 },
    { 0x5a0772d2, SUC_lexicon_tags_3 },
    { 0xeb4772d6, SUC_lexicon_tags_127 },
    { 0xa98772db, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x82772dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e772e1, SUC_lexicon_tags_12 },
    { 0xd02772e1, SUC_lexicon_tags_38 },
    { 0x480772e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20772e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x72772ea, SUC_lexicon_tags_3 },
    { 0xdb4772eb, SUC_lexicon_tags_10 },
    { 0xbc772ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6772f1, SUC_lexicon_tags_6 },
    { 0x39c772f2, SUC_lexicon_tags_32 },
    { 0x41c772f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x832772f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592772fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae77300, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f077304, SUC_lexicon_tags_25 },
    { 0xd4477305, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7477308, SUC_lexicon_tags_7 },
    { 0x59c77309, SUC_lexicon_tags_38 },
    { 0xa6e77308, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687730f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51277315, SUC_lexicon_tags_68 },
    { 0xdfe77315, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc207731a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8587731c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e77322, SUC_lexicon_tags_37 },
    { 0x1277323, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7e877325, SUC_lexicon_tags_2 },
    { 0xd8677326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc77328, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2187732c, SUC_lexicon_tags_27 },
    { 0xfd47732c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9477332, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7277335, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a7733c, SUC_lexicon_tags_13 },
    { 0xe87733d, SUC_lexicon_tags_37 },
    { 0x207733d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27877340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83c77342, SUC_lexicon_tags_7 },
    { 0x40a77342, SUC_lexicon_tags_17 },
    { 0xc0c77342, SUC_lexicon_tags_23 },
    { 0x99677342, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5a77347, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x17c77349, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf407734b, SUC_lexicon_tags_12 },
    { 0x7ce7734c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf27734e, SUC_lexicon_tags_44 },
    { 0xcf47734f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xac677351, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8077354, SUC_lexicon_tags_34 },
    { 0xdc477354, SUC_lexicon_tags_4 },
    { 0x44077356, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29e77358, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea7735b, SUC_lexicon_tags_12 },
    { 0x8ee7735c, SUC_lexicon_tags_15 },
    { 0xbf67735b, SUC_lexicon_tags_11 },
    { 0x85c7735e, SUC_lexicon_tags_31 },
    { 0xbe87735f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde277363, SUC_lexicon_tags_0 },
    { 0x8e477364, SUC_lexicon_tags_12 },
    { 0xf2277363, SUC_lexicon_tags_1 },
    { 0x27a77366, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x96a77368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee7736e, SUC_lexicon_tags_61 },
    { 0xcc7736f, SUC_lexicon_tags_27 },
    { 0xe2277370, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe077378, SUC_lexicon_tags_47 },
    { 0xfaa77378, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e27737f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4477384, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0x55477386, SUC_lexicon_tags_15 },
    { 0x63077387, SUC_lexicon_tags_15 },
    { 0xcd677388, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a7738e, SUC_lexicon_tags_37 },
    { 0xd487738f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xee77392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e677395, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66277399, SUC_lexicon_tags_27 },
    { 0x3507739a, SUC_lexicon_tags_25 },
    { 0xdba7739a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeea7739d, SUC_lexicon_tags_42 },
    { 0x8a47739d, SUC_lexicon_tags_37 },
    { 0x8ea7739f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbc773a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1e773a3, SUC_lexicon_tags_28 },
    { 0x706773a4, SUC_lexicon_tags_60 },
    { 0xec4773a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6a773a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x27e773aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4773ad, SUC_lexicon_tags_7 },
    { 0xad4773ae, SUC_lexicon_tags_13 },
    { 0x976773af, SUC_lexicon_tags_68 },
    { 0xb9a773b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c0773b2, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xe18773b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xccc773b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x676773b8, SUC_lexicon_tags_1 },
    { 0x35c773b9, SUC_lexicon_tags_19 },
    { 0xc1a773b9, SUC_lexicon_tags_35 },
    { 0xd8773b9, SUC_lexicon_tags_12 },
    { 0x70a773bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa26773be, SUC_lexicon_tags_55 },
    { 0xd6e773be, SUC_lexicon_tags_8 },
    { 0xdfa773be, SUC_lexicon_tags_12 },
    { 0xf78773c0, SUC_lexicon_tags_48 },
    { 0xaa2773c2, SUC_lexicon_tags_21 },
    { 0xd5a773c3, SUC_lexicon_tags_17 },
    { 0x1aa773c3, SUC_lexicon_tags_26 },
    { 0xc3e773c1, SUC_lexicon_tags_35 },
    { 0x7c773c6, SUC_lexicon_tags_11 },
    { 0x93a773c7, SUC_lexicon_tags_31 },
    { 0x118773c7, SUC_lexicon_tags_41 },
    { 0xe2e773c9, SUC_lexicon_tags_17 },
    { 0xdb8773c9, SUC_lexicon_tags_25 },
    { 0x90a773ca, SUC_lexicon_tags_16 },
    { 0xc28773c8, SUC_lexicon_tags_1 },
    { 0xd4a773cd, SUC_lexicon_tags_12 },
    { 0xc0773ce, SUC_lexicon_tags_25 },
    { 0xd54773cd, SUC_lexicon_tags_11 },
    { 0xb10773cd, SUC_lexicon_tags_7 },
    { 0x118773d1, SUC_lexicon_tags_17 },
    { 0x62773d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584773d7, SUC_lexicon_tags_17 },
    { 0x2c8773d7, SUC_lexicon_tags_15 },
    { 0xbf0773d7, SUC_lexicon_tags_3 },
    { 0xe1a773da, SUC_lexicon_tags_2 },
    { 0xe00773db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66773df, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2773ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60773ed, SUC_lexicon_tags_29 },
    { 0x656773ee, SUC_lexicon_tags_57 },
    { 0x906773ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a773f2, SUC_lexicon_tags_49 },
    { 0xc8c773f3, SUC_lexicon_tags_13 },
    { 0xd58773f2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2773fb, SUC_lexicon_tags_25 },
    { 0xb86773fb, SUC_lexicon_tags_337 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82277402, SUC_lexicon_tags_9 },
    { 0x9e677403, SUC_lexicon_tags_25 },
    { 0xf8a77404, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c277407, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5887740d, SUC_lexicon_tags_37 },
    { 0xfd87740d, SUC_lexicon_tags_17 },
    { 0xc427740f, SUC_lexicon_tags_52 },
    { 0x11677410, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce77418, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe87741c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47677425, SUC_lexicon_tags_37 },
    { 0xca477426, SUC_lexicon_tags_8 },
    { 0xc3a77427, SUC_lexicon_tags_84 },
    { 0x65c77428, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa607742b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b07742f, SUC_lexicon_tags_60 },
    { 0x9877430, SUC_lexicon_tags_25 },
    { 0xb3e7742f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd477434, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1d077436, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e77439, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8277443, SUC_lexicon_tags_27 },
    { 0x8a277443, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa077448, SUC_lexicon_tags_7 },
    { 0xfa877449, SUC_lexicon_tags_1 },
    { 0x2ce77448, SUC_lexicon_tags_17 },
    { 0x69e77448, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ba7744d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e077454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a077456, SUC_lexicon_tags_29 },
    { 0xdaa77456, SUC_lexicon_tags_37 },
    { 0xfac77457, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a7745b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c77462, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc77469, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea67746b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40477472, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc77474, SUC_lexicon_tags_39 },
    { 0x36877475, SUC_lexicon_tags_11 },
    { 0x1fe77476, SUC_lexicon_tags_43 },
    { 0xc8c77476, SUC_lexicon_tags_13 },
    { 0xdd077478, SUC_lexicon_tags_37 },
    { 0xc2a77476, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea7747d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1e07747f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb877482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a77489, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92a7748b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a77491, SUC_lexicon_tags_1 },
    { 0x75277492, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c77497, SUC_lexicon_tags_7 },
    { 0xe4477498, SUC_lexicon_tags_10 },
    { 0xa7e77497, SUC_lexicon_tags_38 },
    { 0xc567749a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5fc7749c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6774a0, SUC_lexicon_tags_37 },
    { 0x138774a1, SUC_lexicon_tags_17 },
    { 0x418774a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c774a9, SUC_lexicon_tags_2 },
    { 0x718774aa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x520774ad, SUC_lexicon_tags_26 },
    { 0x23a774ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ac774b0, SUC_lexicon_tags_26 },
    { 0xb90774b1, SUC_lexicon_tags_3 },
    { 0x4d4774b2, SUC_lexicon_tags_37 },
    { 0x362774b2, SUC_lexicon_tags_82 },
    { 0xa08774b4, SUC_lexicon_tags_7 },
    { 0x5ee774b5, SUC_lexicon_tags_60 },
    { 0x308774b4, SUC_lexicon_tags_30 },
    { 0x1e4774b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16774ba, SUC_lexicon_tags_386 },
    { 0, NULL },
    { 0, NULL },
    { 0xa28774bd, SUC_lexicon_tags_0 },
    { 0x846774bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e774c1, SUC_lexicon_tags_2 },
    { 0xbaa774c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c4774c4, SUC_lexicon_tags_5 },
    { 0x2f4774c5, SUC_lexicon_tags_60 },
    { 0xbb4774c6, SUC_lexicon_tags_24 },
    { 0x208774c6, SUC_lexicon_tags_9 },
    { 0xffc774c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e774ce, SUC_lexicon_tags_12 },
    { 0xec0774cf, SUC_lexicon_tags_8 },
    { 0x300774d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c774d4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64774da, SUC_lexicon_tags_33 },
    { 0x886774db, SUC_lexicon_tags_17 },
    { 0x896774db, SUC_lexicon_tags_2 },
    { 0xdd8774dd, SUC_lexicon_tags_37 },
    { 0xc2c774de, SUC_lexicon_tags_15 },
    { 0xf4774dd, SUC_lexicon_tags_1 },
    { 0x56774e0, SUC_lexicon_tags_46 },
    { 0xefa774e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x512774e4, SUC_lexicon_tags_47 },
    { 0x312774e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a774ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x472774ef, SUC_lexicon_tags_64 },
    { 0xda4774f0, SUC_lexicon_tags_11 },
    { 0x80774f0, SUC_lexicon_tags_37 },
    { 0x84774f2, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0x434774f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340774fa, SUC_lexicon_tags_12 },
    { 0xfee774fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3477503, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3277508, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f47750f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d077513, SUC_lexicon_tags_15 },
    { 0x46c77513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62477517, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91477522, SUC_lexicon_tags_11 },
    { 0xe0877522, SUC_lexicon_tags_11 },
    { 0xcae77524, SUC_lexicon_tags_12 },
    { 0xcda77525, SUC_lexicon_tags_26 },
    { 0xd9a77523, SUC_lexicon_tags_37 },
    { 0xdc77527, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3247752c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x267752e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c77531, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6277534, SUC_lexicon_tags_45 },
    { 0x9f677535, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15c77537, SUC_lexicon_tags_37 },
    { 0x9d277538, SUC_lexicon_tags_43 },
    { 0x78e77537, SUC_lexicon_tags_83 },
    { 0x4aa77537, SUC_lexicon_tags_9 },
    { 0xc7c7753b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae67753e, SUC_lexicon_tags_11 },
    { 0x6f87753e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a77543, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9877545, SUC_lexicon_tags_27 },
    { 0x2e77545, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf547754c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8677555, SUC_lexicon_tags_7 },
    { 0xbcc77556, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8677560, SUC_lexicon_tags_46 },
    { 0x20477560, SUC_lexicon_tags_21 },
    { 0x6ae77562, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x28677564, SUC_lexicon_tags_40 },
    { 0x8ea77565, SUC_lexicon_tags_23 },
    { 0x70277566, SUC_lexicon_tags_47 },
    { 0xf3c77566, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b47756b, SUC_lexicon_tags_45 },
    { 0x74a7756c, SUC_lexicon_tags_84 },
    { 0xf767756d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16477571, SUC_lexicon_tags_11 },
    { 0x86477572, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf1877574, SUC_lexicon_tags_33 },
    { 0xe277575, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x13077577, SUC_lexicon_tags_77 },
    { 0x43c77577, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c7757c, SUC_lexicon_tags_53 },
    { 0x3e27757d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2477582, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d477585, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a7758a, SUC_lexicon_tags_61 },
    { 0xb527758b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fc7758d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cc7758f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8677592, SUC_lexicon_tags_2 },
    { 0x46677593, SUC_lexicon_tags_25 },
    { 0x15877593, SUC_lexicon_tags_30 },
    { 0x38277595, SUC_lexicon_tags_45 },
    { 0xd7477592, SUC_lexicon_tags_17 },
    { 0x15277593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d477599, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1367759e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de775a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbfe775a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x294775a6, SUC_lexicon_tags_169 },
    { 0, NULL },
    { 0xe5e775a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea775ab, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x340775ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x980775af, SUC_lexicon_tags_25 },
    { 0xf34775b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6c775b2, SUC_lexicon_tags_45 },
    { 0x2d2775b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x800775b5, SUC_lexicon_tags_11 },
    { 0x7b0775b6, SUC_lexicon_tags_32 },
    { 0xa96775b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc775be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54775c9, SUC_lexicon_tags_7 },
    { 0xa6a775c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x756775cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0775d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a775d6, SUC_lexicon_tags_11 },
    { 0x8f2775d6, SUC_lexicon_tags_197 },
    { 0xe2e775d8, SUC_lexicon_tags_8 },
    { 0x632775d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeee775db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952775e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e775e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38a775e5, SUC_lexicon_tags_25 },
    { 0x46e775e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0775eb, SUC_lexicon_tags_33 },
    { 0x2b8775ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a775fe, SUC_lexicon_tags_17 },
    { 0x6775fe, SUC_lexicon_tags_38 },
    { 0xe0877600, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18677602, SUC_lexicon_tags_2 },
    { 0x62077603, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c7760c, SUC_lexicon_tags_15 },
    { 0x4627760d, SUC_lexicon_tags_56 },
    { 0xc767760e, SUC_lexicon_tags_11 },
    { 0x2787760f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84277617, SUC_lexicon_tags_1 },
    { 0x60a77618, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc887761d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a77620, SUC_lexicon_tags_35 },
    { 0x55e77620, SUC_lexicon_tags_65 },
    { 0x1d277622, SUC_lexicon_tags_14 },
    { 0xabe77620, SUC_lexicon_tags_2 },
    { 0x64277622, SUC_lexicon_tags_14 },
    { 0x96077623, SUC_lexicon_tags_21 },
    { 0x7a77626, SUC_lexicon_tags_45 },
    { 0x2a477625, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb07762a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec7762d, SUC_lexicon_tags_26 },
    { 0xcda7762e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa77634, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x22477637, SUC_lexicon_tags_8 },
    { 0x81c77638, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7207763b, SUC_lexicon_tags_17 },
    { 0x2927763c, SUC_lexicon_tags_25 },
    { 0x12c7763c, SUC_lexicon_tags_17 },
    { 0x4d87763e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b677641, SUC_lexicon_tags_19 },
    { 0x9c677641, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab677646, SUC_lexicon_tags_7 },
    { 0xae077646, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2077649, SUC_lexicon_tags_38 },
    { 0x63877649, SUC_lexicon_tags_25 },
    { 0xe927764b, SUC_lexicon_tags_12 },
    { 0xa2a7764c, SUC_lexicon_tags_49 },
    { 0x52e7764d, SUC_lexicon_tags_2 },
    { 0x6847764e, SUC_lexicon_tags_38 },
    { 0xe647764f, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x1e077651, SUC_lexicon_tags_37 },
    { 0x39277652, SUC_lexicon_tags_37 },
    { 0x1277653, SUC_lexicon_tags_33 },
    { 0x92277651, SUC_lexicon_tags_66 },
    { 0xe0477655, SUC_lexicon_tags_27 },
    { 0x62c77655, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4287765d, SUC_lexicon_tags_26 },
    { 0xffe7765d, SUC_lexicon_tags_43 },
    { 0x1c47765f, SUC_lexicon_tags_2 },
    { 0x3c07765e, SUC_lexicon_tags_11 },
    { 0xfe077660, SUC_lexicon_tags_37 },
    { 0xb0c77660, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27677664, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9477667, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9707766a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99c7766c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4477670, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9877679, SUC_lexicon_tags_7 },
    { 0xaec77679, SUC_lexicon_tags_80 },
    { 0xe767767a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe207767d, SUC_lexicon_tags_6 },
    { 0x8327767e, SUC_lexicon_tags_9 },
    { 0x12c7767e, SUC_lexicon_tags_19 },
    { 0xb407767f, SUC_lexicon_tags_25 },
    { 0x11877680, SUC_lexicon_tags_510 },
    { 0xd6877682, SUC_lexicon_tags_2 },
    { 0x9647767f, SUC_lexicon_tags_47 },
    { 0x80a7767f, SUC_lexicon_tags_27 },
    { 0xc7677683, SUC_lexicon_tags_16 },
    { 0x61477686, SUC_lexicon_tags_14 },
    { 0xb8277686, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18677689, SUC_lexicon_tags_4 },
    { 0x23c7768a, SUC_lexicon_tags_63 },
    { 0x7827768b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a07768d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c77692, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3877696, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6187769b, SUC_lexicon_tags_45 },
    { 0x5587769b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0776a1, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a776a7, SUC_lexicon_tags_6 },
    { 0x972776a8, SUC_lexicon_tags_7 },
    { 0x838776a9, SUC_lexicon_tags_37 },
    { 0x7d0776a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a776ad, SUC_lexicon_tags_33 },
    { 0x71a776ae, SUC_lexicon_tags_30 },
    { 0xee4776af, SUC_lexicon_tags_63 },
    { 0xbcc776b0, SUC_lexicon_tags_39 },
    { 0x7ba776b1, SUC_lexicon_tags_43 },
    { 0x332776b1, SUC_lexicon_tags_2 },
    { 0x6d8776b1, SUC_lexicon_tags_65 },
    { 0x354776b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6a776b6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460776bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9ca776be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8776c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x222776c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8776c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562776cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa2c776cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa88776d1, SUC_lexicon_tags_12 },
    { 0x7de776d1, SUC_lexicon_tags_62 },
    { 0x726776d2, SUC_lexicon_tags_26 },
    { 0x3d0776d4, SUC_lexicon_tags_37 },
    { 0x6e0776d4, SUC_lexicon_tags_57 },
    { 0x5a776d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdc776d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a2776da, SUC_lexicon_tags_10 },
    { 0x414776da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x750776dd, SUC_lexicon_tags_2 },
    { 0xfb4776de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78776e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a776ec, SUC_lexicon_tags_42 },
    { 0xbac776ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2776f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ac776f3, SUC_lexicon_tags_47 },
    { 0x4fc776f3, SUC_lexicon_tags_23 },
    { 0x6776f5, SUC_lexicon_tags_2 },
    { 0xe8e776f4, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e77701, SUC_lexicon_tags_16 },
    { 0xf4077702, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e877707, SUC_lexicon_tags_19 },
    { 0x5f477708, SUC_lexicon_tags_1 },
    { 0xfdc77707, SUC_lexicon_tags_12 },
    { 0x7d67770a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac77711, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe677716, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a7771c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b67771f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c77724, SUC_lexicon_tags_49 },
    { 0xb7277725, SUC_lexicon_tags_9 },
    { 0xfe77726, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a77729, SUC_lexicon_tags_39 },
    { 0xc87772a, SUC_lexicon_tags_37 },
    { 0x44a7772a, SUC_lexicon_tags_8 },
    { 0xe887772a, SUC_lexicon_tags_19 },
    { 0xe9c77729, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a77730, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c77732, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c677734, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1877739, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b47773c, SUC_lexicon_tags_11 },
    { 0x7d27773d, SUC_lexicon_tags_15 },
    { 0xc707773e, SUC_lexicon_tags_9 },
    { 0x5307773f, SUC_lexicon_tags_17 },
    { 0xa4677740, SUC_lexicon_tags_66 },
    { 0x39a7773f, SUC_lexicon_tags_15 },
    { 0x7677741, SUC_lexicon_tags_14 },
    { 0x78677742, SUC_lexicon_tags_47 },
    { 0xd2477741, SUC_lexicon_tags_17 },
    { 0x3f877744, SUC_lexicon_tags_17 },
    { 0x95277746, SUC_lexicon_tags_77 },
    { 0x94477747, SUC_lexicon_tags_23 },
    { 0x98277747, SUC_lexicon_tags_8 },
    { 0xac77741, SUC_lexicon_tags_40 },
    { 0x6de7774a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc7774e, SUC_lexicon_tags_64 },
    { 0x9ec7774e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e77752, SUC_lexicon_tags_47 },
    { 0xf2677752, SUC_lexicon_tags_29 },
    { 0x40c77753, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0c77756, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4077758, SUC_lexicon_tags_37 },
    { 0xf6e77759, SUC_lexicon_tags_63 },
    { 0x6c47775a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6677760, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaca77762, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc677766, SUC_lexicon_tags_38 },
    { 0xc477766, SUC_lexicon_tags_41 },
    { 0x20877768, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc7776c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5e7776e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4a77770, SUC_lexicon_tags_25 },
    { 0x62077770, SUC_lexicon_tags_37 },
    { 0x47277772, SUC_lexicon_tags_77 },
    { 0xf9877773, SUC_lexicon_tags_1 },
    { 0x82077774, SUC_lexicon_tags_37 },
    { 0x1da77774, SUC_lexicon_tags_17 },
    { 0xdb077774, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d877779, SUC_lexicon_tags_27 },
    { 0x3e87777a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9c7777c, SUC_lexicon_tags_49 },
    { 0x9227777d, SUC_lexicon_tags_23 },
    { 0xec07777d, SUC_lexicon_tags_29 },
    { 0x3887777f, SUC_lexicon_tags_57 },
    { 0xd467777e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46877786, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfe677788, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7847778e, SUC_lexicon_tags_15 },
    { 0xf2a7778f, SUC_lexicon_tags_19 },
    { 0xc3277790, SUC_lexicon_tags_3 },
    { 0xf5677791, SUC_lexicon_tags_15 },
    { 0x60c7778f, SUC_lexicon_tags_8 },
    { 0x47677793, SUC_lexicon_tags_4 },
    { 0x14277794, SUC_lexicon_tags_37 },
    { 0x1f277795, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa4a77797, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8be77799, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe807779b, SUC_lexicon_tags_1 },
    { 0x9c07779b, SUC_lexicon_tags_144 },
    { 0xcd07779d, SUC_lexicon_tags_45 },
    { 0xda27779e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a777a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38777ac, SUC_lexicon_tags_47 },
    { 0x696777ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760777b4, SUC_lexicon_tags_31 },
    { 0xde4777b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8c777b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd76777b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a777bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24777c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x798777c4, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480777cb, SUC_lexicon_tags_6 },
    { 0x6a4777cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0777cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2777d3, SUC_lexicon_tags_11 },
    { 0xdd6777d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50777d8, SUC_lexicon_tags_14 },
    { 0x49a777d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc8777db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe777dd, SUC_lexicon_tags_4 },
    { 0xe62777dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936777e6, SUC_lexicon_tags_2 },
    { 0x91a777e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66777ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3e777ee, SUC_lexicon_tags_45 },
    { 0xe0c777ee, SUC_lexicon_tags_7 },
    { 0xa08777f0, SUC_lexicon_tags_15 },
    { 0x360777f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74a777f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6777fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56677802, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c77806, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1677809, SUC_lexicon_tags_17 },
    { 0xe1e7780a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x467780c, SUC_lexicon_tags_2 },
    { 0xd727780d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a77811, SUC_lexicon_tags_23 },
    { 0x4de77812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30277814, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8077817, SUC_lexicon_tags_37 },
    { 0x31077818, SUC_lexicon_tags_252 },
    { 0x66c77818, SUC_lexicon_tags_17 },
    { 0x39a7781a, SUC_lexicon_tags_55 },
    { 0x2967781a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83c7781d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb127781f, SUC_lexicon_tags_4 },
    { 0xc067781f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a77824, SUC_lexicon_tags_31 },
    { 0x5ec77825, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe277827, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb277829, SUC_lexicon_tags_46 },
    { 0xcba77829, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb87782f, SUC_lexicon_tags_7 },
    { 0x14277830, SUC_lexicon_tags_15 },
    { 0xfa077831, SUC_lexicon_tags_45 },
    { 0x88077832, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5077838, SUC_lexicon_tags_37 },
    { 0x7e877839, SUC_lexicon_tags_60 },
    { 0xabe7783a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2307783e, SUC_lexicon_tags_19 },
    { 0x1267783f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea677849, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de7784c, SUC_lexicon_tags_26 },
    { 0xcfc7784c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2c7784f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a77858, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61077861, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a77864, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4077866, SUC_lexicon_tags_14 },
    { 0x31277867, SUC_lexicon_tags_10 },
    { 0xa6877867, SUC_lexicon_tags_10 },
    { 0xa0277869, SUC_lexicon_tags_11 },
    { 0xdb677867, SUC_lexicon_tags_9 },
    { 0xaf87786b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c47786e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40277871, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b477875, SUC_lexicon_tags_79 },
    { 0x1fa77875, SUC_lexicon_tags_14 },
    { 0xf8077877, SUC_lexicon_tags_17 },
    { 0x8a677876, SUC_lexicon_tags_17 },
    { 0x29877879, SUC_lexicon_tags_12 },
    { 0x5a67787a, SUC_lexicon_tags_19 },
    { 0x83677879, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x547787e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e77881, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c77884, SUC_lexicon_tags_33 },
    { 0xa6a77884, SUC_lexicon_tags_37 },
    { 0x99677886, SUC_lexicon_tags_15 },
    { 0x39477886, SUC_lexicon_tags_43 },
    { 0xf8677888, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x487788a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a7788e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37677890, SUC_lexicon_tags_19 },
    { 0x1d477891, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc77894, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd447789f, SUC_lexicon_tags_37 },
    { 0xfe7789f, SUC_lexicon_tags_1 },
    { 0xa84778a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x678778a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46778a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd30778a7, SUC_lexicon_tags_107 },
    { 0xf84778a8, SUC_lexicon_tags_37 },
    { 0xff0778a9, SUC_lexicon_tags_19 },
    { 0xf2c778aa, SUC_lexicon_tags_30 },
    { 0x97e778aa, SUC_lexicon_tags_47 },
    { 0x1d4778ac, SUC_lexicon_tags_37 },
    { 0xaa2778ad, SUC_lexicon_tags_15 },
    { 0xa96778ab, SUC_lexicon_tags_33 },
    { 0xbac778af, SUC_lexicon_tags_14 },
    { 0xfaa778a8, SUC_lexicon_tags_14 },
    { 0x318778ab, SUC_lexicon_tags_49 },
    { 0xa28778ab, SUC_lexicon_tags_37 },
    { 0x292778af, SUC_lexicon_tags_16 },
    { 0xc82778b4, SUC_lexicon_tags_16 },
    { 0x920778b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4778b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80778bd, SUC_lexicon_tags_11 },
    { 0x65c778be, SUC_lexicon_tags_19 },
    { 0xde6778be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x18778c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30778c7, SUC_lexicon_tags_12 },
    { 0x722778c8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c778cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4778d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbda778d2, SUC_lexicon_tags_19 },
    { 0xbba778d3, SUC_lexicon_tags_17 },
    { 0x21a778d4, SUC_lexicon_tags_49 },
    { 0x3a778d5, SUC_lexicon_tags_11 },
    { 0x240778d6, SUC_lexicon_tags_17 },
    { 0x526778d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10778da, SUC_lexicon_tags_11 },
    { 0x6ae778db, SUC_lexicon_tags_14 },
    { 0x84e778db, SUC_lexicon_tags_31 },
    { 0xc2e778dd, SUC_lexicon_tags_9 },
    { 0xc8e778db, SUC_lexicon_tags_15 },
    { 0xbd2778df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x524778e1, SUC_lexicon_tags_16 },
    { 0xade778e2, SUC_lexicon_tags_22 },
    { 0xa0a778e3, SUC_lexicon_tags_3 },
    { 0xe6778e3, SUC_lexicon_tags_7 },
    { 0xeac778e2, SUC_lexicon_tags_15 },
    { 0xb5c778e5, SUC_lexicon_tags_0 },
    { 0x442778e2, SUC_lexicon_tags_28 },
    { 0xb50778e8, SUC_lexicon_tags_1 },
    { 0xa7c778e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478778f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5cc778f4, SUC_lexicon_tags_33 },
    { 0x8fa778f5, SUC_lexicon_tags_37 },
    { 0x95a778f6, SUC_lexicon_tags_33 },
    { 0xe66778f6, SUC_lexicon_tags_72 },
    { 0x382778f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cc778fa, SUC_lexicon_tags_28 },
    { 0xbbc778fb, SUC_lexicon_tags_28 },
    { 0xb9a778fa, SUC_lexicon_tags_14 },
    { 0x2f2778fd, SUC_lexicon_tags_1 },
    { 0x256778fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c77901, SUC_lexicon_tags_8 },
    { 0x1a477901, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43677906, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcf277908, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a7790d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xef07790f, SUC_lexicon_tags_63 },
    { 0xf0077910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11477916, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae7791b, SUC_lexicon_tags_61 },
    { 0x4b27791b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a07791e, SUC_lexicon_tags_64 },
    { 0x34c7791e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c77928, SUC_lexicon_tags_11 },
    { 0x9877928, SUC_lexicon_tags_63 },
    { 0xc5a7792a, SUC_lexicon_tags_43 },
    { 0xe87792b, SUC_lexicon_tags_11 },
    { 0x3e07792c, SUC_lexicon_tags_46 },
    { 0xcd47792d, SUC_lexicon_tags_8 },
    { 0xfa47792e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf477930, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4677932, SUC_lexicon_tags_4 },
    { 0x55477932, SUC_lexicon_tags_11 },
    { 0xe8077933, SUC_lexicon_tags_79 },
    { 0xcfe77933, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x88077938, SUC_lexicon_tags_8 },
    { 0x46077938, SUC_lexicon_tags_15 },
    { 0x51a7793a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4867793d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0477942, SUC_lexicon_tags_1 },
    { 0xe9c77943, SUC_lexicon_tags_45 },
    { 0xfd677944, SUC_lexicon_tags_15 },
    { 0x9ca77945, SUC_lexicon_tags_2 },
    { 0xb9877945, SUC_lexicon_tags_7 },
    { 0x80c77946, SUC_lexicon_tags_70 },
    { 0x4ca77946, SUC_lexicon_tags_19 },
    { 0x48c77949, SUC_lexicon_tags_28 },
    { 0x98677949, SUC_lexicon_tags_12 },
    { 0x4c27794b, SUC_lexicon_tags_8 },
    { 0x2947794c, SUC_lexicon_tags_21 },
    { 0x68e7794a, SUC_lexicon_tags_11 },
    { 0x8f47794b, SUC_lexicon_tags_17 },
    { 0xb007794f, SUC_lexicon_tags_37 },
    { 0xd5a77950, SUC_lexicon_tags_37 },
    { 0x877951, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e77955, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6277958, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd547795d, SUC_lexicon_tags_3 },
    { 0x1247795e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff677963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce477965, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea77968, SUC_lexicon_tags_174 },
    { 0x55077968, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf567796d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a77970, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8477972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76477977, SUC_lexicon_tags_11 },
    { 0xbbe77977, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc7797c, SUC_lexicon_tags_92 },
    { 0x2067797d, SUC_lexicon_tags_23 },
    { 0x467797e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29277983, SUC_lexicon_tags_1 },
    { 0x58877984, SUC_lexicon_tags_25 },
    { 0xdc477984, SUC_lexicon_tags_37 },
    { 0x4ae77986, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa877988, SUC_lexicon_tags_238 },
    { 0x58e77988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8067798c, SUC_lexicon_tags_0 },
    { 0xe327798d, SUC_lexicon_tags_19 },
    { 0xd8a7798e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e477990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59677993, SUC_lexicon_tags_15 },
    { 0x8ee77994, SUC_lexicon_tags_37 },
    { 0x51277995, SUC_lexicon_tags_10 },
    { 0x31877994, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x51677998, SUC_lexicon_tags_15 },
    { 0xbac77999, SUC_lexicon_tags_60 },
    { 0x8c07799a, SUC_lexicon_tags_15 },
    { 0x9f07799a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a7799f, SUC_lexicon_tags_38 },
    { 0x692779a0, SUC_lexicon_tags_11 },
    { 0x47c779a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae0779a3, SUC_lexicon_tags_16 },
    { 0x4b6779a3, SUC_lexicon_tags_87 },
    { 0x6e6779a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8779a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe72779ab, SUC_lexicon_tags_6 },
    { 0x5a8779ac, SUC_lexicon_tags_66 },
    { 0x5b6779ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbd4779af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x158779b1, SUC_lexicon_tags_12 },
    { 0xf5c779b2, SUC_lexicon_tags_8 },
    { 0xef4779b2, SUC_lexicon_tags_0 },
    { 0xcda779b4, SUC_lexicon_tags_61 },
    { 0x4ae779b3, SUC_lexicon_tags_37 },
    { 0x7b4779b3, SUC_lexicon_tags_1 },
    { 0x28c779b2, SUC_lexicon_tags_17 },
    { 0x536779b3, SUC_lexicon_tags_17 },
    { 0xc6c779b9, SUC_lexicon_tags_55 },
    { 0x322779b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e779c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e779c5, SUC_lexicon_tags_28 },
    { 0x63c779c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf2779c8, SUC_lexicon_tags_11 },
    { 0xc08779c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x278779cb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4da779cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2779d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0779e0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xea8779e2, SUC_lexicon_tags_8 },
    { 0xaae779e3, SUC_lexicon_tags_32 },
    { 0xc92779e3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944779e9, SUC_lexicon_tags_44 },
    { 0x652779ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a4779ec, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8779f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280779f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x794779fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb877a03, SUC_lexicon_tags_17 },
    { 0xa4677a04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d277a07, SUC_lexicon_tags_4 },
    { 0x58e77a08, SUC_lexicon_tags_30 },
    { 0xd1077a08, SUC_lexicon_tags_19 },
    { 0x38477a0a, SUC_lexicon_tags_17 },
    { 0x78277a07, SUC_lexicon_tags_19 },
    { 0xbd677a0c, SUC_lexicon_tags_60 },
    { 0xb7e77a0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a77a12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60477a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78677a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85877a1b, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x2277a1d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e77a21, SUC_lexicon_tags_55 },
    { 0xd1077a21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac677a28, SUC_lexicon_tags_37 },
    { 0x31a77a29, SUC_lexicon_tags_1 },
    { 0xbac77a2a, SUC_lexicon_tags_43 },
    { 0x5e77a2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8677a2d, SUC_lexicon_tags_17 },
    { 0x6be77a2e, SUC_lexicon_tags_37 },
    { 0xb2277a2f, SUC_lexicon_tags_11 },
    { 0x8677a30, SUC_lexicon_tags_15 },
    { 0xb0077a30, SUC_lexicon_tags_87 },
    { 0x81277a32, SUC_lexicon_tags_41 },
    { 0x67a77a33, SUC_lexicon_tags_1 },
    { 0x5c77a31, SUC_lexicon_tags_11 },
    { 0x13077a32, SUC_lexicon_tags_7 },
    { 0x95a77a34, SUC_lexicon_tags_47 },
    { 0x76a77a37, SUC_lexicon_tags_13 },
    { 0x7f677a38, SUC_lexicon_tags_15 },
    { 0x6d277a37, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1277a3c, SUC_lexicon_tags_37 },
    { 0x46c77a3d, SUC_lexicon_tags_4 },
    { 0x90877a3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa77a47, SUC_lexicon_tags_37 },
    { 0x2c877a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c77a4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e277a50, SUC_lexicon_tags_23 },
    { 0xa2e77a51, SUC_lexicon_tags_43 },
    { 0x90477a52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31877a55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9de77a57, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2477a62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc477a64, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67677a69, SUC_lexicon_tags_0 },
    { 0xe5877a6a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b477a6d, SUC_lexicon_tags_60 },
    { 0x4077a6e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x14477a70, SUC_lexicon_tags_7 },
    { 0x63877a71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa77a77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93677a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf477a88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57477a8a, SUC_lexicon_tags_15 },
    { 0x84677a8a, SUC_lexicon_tags_30 },
    { 0x58277a8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a77a8f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8f477a91, SUC_lexicon_tags_38 },
    { 0x67c77a91, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95677a9c, SUC_lexicon_tags_15 },
    { 0x3c077a9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa77aa0, SUC_lexicon_tags_33 },
    { 0x22477aa0, SUC_lexicon_tags_8 },
    { 0x62277aa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9277aa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8477aac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xedc77aae, SUC_lexicon_tags_0 },
    { 0x4a277aae, SUC_lexicon_tags_108 },
    { 0x42c77ab0, SUC_lexicon_tags_17 },
    { 0x1d277ab1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8c077ab3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48077ab5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49477aba, SUC_lexicon_tags_8 },
    { 0xdc77aba, SUC_lexicon_tags_2 },
    { 0xb1a77abc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x17677abf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6477ac3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de77ac6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e077ac9, SUC_lexicon_tags_47 },
    { 0x79077ac9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab477acc, SUC_lexicon_tags_37 },
    { 0xb0677acc, SUC_lexicon_tags_25 },
    { 0x81a77ace, SUC_lexicon_tags_17 },
    { 0x8e877acf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba77ad1, SUC_lexicon_tags_11 },
    { 0x1ac77ad2, SUC_lexicon_tags_120 },
    { 0xb6877ad2, SUC_lexicon_tags_17 },
    { 0xe1277ad4, SUC_lexicon_tags_37 },
    { 0x73677ad5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8677ad7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaba77ad9, SUC_lexicon_tags_23 },
    { 0xbd677ad9, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x2bc77adc, SUC_lexicon_tags_50 },
    { 0xd1e77add, SUC_lexicon_tags_7 },
    { 0xe0477ade, SUC_lexicon_tags_31 },
    { 0xd9c77adf, SUC_lexicon_tags_17 },
    { 0x9f477add, SUC_lexicon_tags_45 },
    { 0x98e77ae1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a77ae4, SUC_lexicon_tags_32 },
    { 0x89277ae5, SUC_lexicon_tags_11 },
    { 0xa4277ae5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f677aea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa077aec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c277aef, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x22a77af1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59677af4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e277af6, SUC_lexicon_tags_11 },
    { 0xf3477af7, SUC_lexicon_tags_1 },
    { 0xd6e77af8, SUC_lexicon_tags_14 },
    { 0x59c77af9, SUC_lexicon_tags_16 },
    { 0xe9677afa, SUC_lexicon_tags_12 },
    { 0x7f477af9, SUC_lexicon_tags_38 },
    { 0x31877afc, SUC_lexicon_tags_32 },
    { 0xafa77afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa677b02, SUC_lexicon_tags_23 },
    { 0xeb477b03, SUC_lexicon_tags_37 },
    { 0x5c77b03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd877b06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc77b0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4077b0f, SUC_lexicon_tags_49 },
    { 0x72677b10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8677b13, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x39477b15, SUC_lexicon_tags_11 },
    { 0x75677b16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a77b19, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeee77b1b, SUC_lexicon_tags_37 },
    { 0x69e77b1c, SUC_lexicon_tags_12 },
    { 0x42077b1d, SUC_lexicon_tags_33 },
    { 0xe4477b1e, SUC_lexicon_tags_37 },
    { 0x6ac77b1e, SUC_lexicon_tags_7 },
    { 0xf9877b1f, SUC_lexicon_tags_19 },
    { 0xed877b1e, SUC_lexicon_tags_26 },
    { 0x12e77b21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce77b27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x22877b29, SUC_lexicon_tags_12 },
    { 0x87477b2a, SUC_lexicon_tags_30 },
    { 0xcf477b2a, SUC_lexicon_tags_37 },
    { 0x8c677b2a, SUC_lexicon_tags_30 },
    { 0x20077b2d, SUC_lexicon_tags_37 },
    { 0x5de77b2a, SUC_lexicon_tags_25 },
    { 0x8f477b2c, SUC_lexicon_tags_23 },
    { 0xe077b2b, SUC_lexicon_tags_80 },
    { 0x74277b2b, SUC_lexicon_tags_11 },
    { 0xb0a77b31, SUC_lexicon_tags_30 },
    { 0x92a77b29, SUC_lexicon_tags_21 },
    { 0x6d477b2e, SUC_lexicon_tags_25 },
    { 0x25077b35, SUC_lexicon_tags_788 },
    { 0xd5677b36, SUC_lexicon_tags_19 },
    { 0xa3477b31, SUC_lexicon_tags_23 },
    { 0x4ec77b2c, SUC_lexicon_tags_107 },
    { 0x96677b35, SUC_lexicon_tags_15 },
    { 0xb3c77b2e, SUC_lexicon_tags_30 },
    { 0xa4077b38, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e677b45, SUC_lexicon_tags_23 },
    { 0x97077b45, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xe7277b48, SUC_lexicon_tags_66 },
    { 0xb3277b49, SUC_lexicon_tags_7 },
    { 0xe4c77b4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c77b4d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6e077b4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xebc77b51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d277b54, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4477b57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7677b5b, SUC_lexicon_tags_15 },
    { 0x8a477b5c, SUC_lexicon_tags_17 },
    { 0x7b877b5b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c077b64, SUC_lexicon_tags_65 },
    { 0x8ea77b65, SUC_lexicon_tags_4 },
    { 0x60e77b66, SUC_lexicon_tags_11 },
    { 0x67277b67, SUC_lexicon_tags_48 },
    { 0x7dc77b68, SUC_lexicon_tags_17 },
    { 0x94e77b66, SUC_lexicon_tags_25 },
    { 0xc6a77b64, SUC_lexicon_tags_15 },
    { 0x9f677b6b, SUC_lexicon_tags_17 },
    { 0x3e677b6b, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xf5a77b6e, SUC_lexicon_tags_1 },
    { 0xa3477b6f, SUC_lexicon_tags_43 },
    { 0x66477b70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27877b72, SUC_lexicon_tags_27 },
    { 0xf7077b72, SUC_lexicon_tags_45 },
    { 0x4a077b74, SUC_lexicon_tags_11 },
    { 0xb1477b75, SUC_lexicon_tags_47 },
    { 0xbc877b76, SUC_lexicon_tags_187 },
    { 0x3dc77b76, SUC_lexicon_tags_100 },
    { 0x4ec77b75, SUC_lexicon_tags_46 },
    { 0x20c77b78, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x77677b7b, SUC_lexicon_tags_38 },
    { 0x63077b7c, SUC_lexicon_tags_23 },
    { 0x47c77b7b, SUC_lexicon_tags_37 },
    { 0x8ba77b7e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81077b85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e077b87, SUC_lexicon_tags_2 },
    { 0x6f477b88, SUC_lexicon_tags_15 },
    { 0xe6877b89, SUC_lexicon_tags_1 },
    { 0xd7877b88, SUC_lexicon_tags_57 },
    { 0x64877b8b, SUC_lexicon_tags_7 },
    { 0xe077b89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c77b8e, SUC_lexicon_tags_11 },
    { 0x8b077b8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee77b93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab077b97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c077b99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5ba77b9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7dc77b9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67477ba0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x15877ba2, SUC_lexicon_tags_25 },
    { 0x90e77ba3, SUC_lexicon_tags_17 },
    { 0xde077ba4, SUC_lexicon_tags_47 },
    { 0xc6277ba5, SUC_lexicon_tags_12 },
    { 0xed677ba3, SUC_lexicon_tags_0 },
    { 0xe1877ba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca677ba9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea77bac, SUC_lexicon_tags_11 },
    { 0x5d477bad, SUC_lexicon_tags_1 },
    { 0xe677bae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe477bb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f477bb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15477bbc, SUC_lexicon_tags_2 },
    { 0x40277bbd, SUC_lexicon_tags_15 },
    { 0xae877bbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6677bc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9a77bc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6877bc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c77bc8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc77bcb, SUC_lexicon_tags_27 },
    { 0xee077bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94e77bce, SUC_lexicon_tags_11 },
    { 0x5c877bcf, SUC_lexicon_tags_13 },
    { 0xb077bcf, SUC_lexicon_tags_11 },
    { 0xdc677bd1, SUC_lexicon_tags_2 },
    { 0x2b077bd0, SUC_lexicon_tags_37 },
    { 0x66a77bd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c77bd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a077bd9, SUC_lexicon_tags_11 },
    { 0x3d877bda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b077bdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0677be5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xea477be7, SUC_lexicon_tags_33 },
    { 0x19277be7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd877bee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd077bf4, SUC_lexicon_tags_43 },
    { 0xb3a77bf5, SUC_lexicon_tags_11 },
    { 0x9ea77bf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a477bf9, SUC_lexicon_tags_25 },
    { 0xfb277bfa, SUC_lexicon_tags_2 },
    { 0x1d077bfb, SUC_lexicon_tags_12 },
    { 0x19277bf9, SUC_lexicon_tags_16 },
    { 0xa6077bfd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x54077c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4277c02, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe8a77c04, SUC_lexicon_tags_1 },
    { 0x23c77c04, SUC_lexicon_tags_5 },
    { 0xd5c77c06, SUC_lexicon_tags_37 },
    { 0x73c77c07, SUC_lexicon_tags_33 },
    { 0x30877c08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91277c0d, SUC_lexicon_tags_31 },
    { 0x64e77c0d, SUC_lexicon_tags_2 },
    { 0xf9e77c0f, SUC_lexicon_tags_25 },
    { 0x72077c0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90277c12, SUC_lexicon_tags_56 },
    { 0xbde77c13, SUC_lexicon_tags_17 },
    { 0x93077c14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc77c17, SUC_lexicon_tags_1 },
    { 0x5ec77c18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x62877c1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7677c22, SUC_lexicon_tags_25 },
    { 0x8b877c23, SUC_lexicon_tags_37 },
    { 0x83877c24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e677c28, SUC_lexicon_tags_49 },
    { 0x6ca77c28, SUC_lexicon_tags_11 },
    { 0x76a77c28, SUC_lexicon_tags_25 },
    { 0x2ce77c2b, SUC_lexicon_tags_1 },
    { 0xd6477c28, SUC_lexicon_tags_7 },
    { 0xc6277c28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51077c31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e877c36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaa77c38, SUC_lexicon_tags_17 },
    { 0xbe277c39, SUC_lexicon_tags_2 },
    { 0x9f677c3a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c77c40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6ba77c42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6477c46, SUC_lexicon_tags_37 },
    { 0x6677c47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e877c4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57077c4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c77c50, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c77c54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e77c57, SUC_lexicon_tags_7 },
    { 0x2f277c58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e77c5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7a77c61, SUC_lexicon_tags_45 },
    { 0xb4a77c62, SUC_lexicon_tags_11 },
    { 0xde077c62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f477c65, SUC_lexicon_tags_79 },
    { 0x4077c66, SUC_lexicon_tags_65 },
    { 0xa077c67, SUC_lexicon_tags_2 },
    { 0xee077c68, SUC_lexicon_tags_25 },
    { 0x13c77c69, SUC_lexicon_tags_26 },
    { 0x72477c6a, SUC_lexicon_tags_26 },
    { 0x4e877c69, SUC_lexicon_tags_19 },
    { 0x21677c6c, SUC_lexicon_tags_11 },
    { 0xd8077c6c, SUC_lexicon_tags_17 },
    { 0xd5a77c6d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4877c71, SUC_lexicon_tags_21 },
    { 0xa4277c72, SUC_lexicon_tags_35 },
    { 0x7c277c73, SUC_lexicon_tags_21 },
    { 0x6fa77c71, SUC_lexicon_tags_17 },
    { 0x28c77c74, SUC_lexicon_tags_33 },
    { 0xaa877c76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7e77c78, SUC_lexicon_tags_9 },
    { 0x3da77c79, SUC_lexicon_tags_53 },
    { 0xa9877c78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17477c7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae77c81, SUC_lexicon_tags_1 },
    { 0x4277c82, SUC_lexicon_tags_25 },
    { 0x26277c83, SUC_lexicon_tags_12 },
    { 0x5a877c84, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd477c87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44477c8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a77c8f, SUC_lexicon_tags_17 },
    { 0x25877c90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e77c97, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa0e77c99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a677c9c, SUC_lexicon_tags_7 },
    { 0xde77c9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcae77c9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67077ca4, SUC_lexicon_tags_11 },
    { 0x44077ca5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d477ca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e677cab, SUC_lexicon_tags_44 },
    { 0xe3877cac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce477cb0, SUC_lexicon_tags_1 },
    { 0x60877cb0, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3877cb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x85277cba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb277cbd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x82c77cbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4477cc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1ec77cc4, SUC_lexicon_tags_1 },
    { 0x81677cc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12077cc8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x20077cca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77077cd6, SUC_lexicon_tags_25 },
    { 0x34c77cd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93277cd9, SUC_lexicon_tags_37 },
    { 0xab877cda, SUC_lexicon_tags_48 },
    { 0x9f477cdb, SUC_lexicon_tags_47 },
    { 0x91277cdc, SUC_lexicon_tags_9 },
    { 0xbba77cd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9877ce4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x78077ce6, SUC_lexicon_tags_1 },
    { 0x35877ce7, SUC_lexicon_tags_15 },
    { 0x80477ce8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a677cec, SUC_lexicon_tags_11 },
    { 0xd5e77ced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68077cf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b677cf8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8ce77cfa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2877d00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4277d04, SUC_lexicon_tags_8 },
    { 0xcb877d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0877d09, SUC_lexicon_tags_2 },
    { 0x3b277d0a, SUC_lexicon_tags_25 },
    { 0xe8c77d0a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfaa77d0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6a77d0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23e77d11, SUC_lexicon_tags_25 },
    { 0xc0277d11, SUC_lexicon_tags_30 },
    { 0xf5877d13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe77d1a, SUC_lexicon_tags_37 },
    { 0xfda77d1b, SUC_lexicon_tags_28 },
    { 0x57e77d1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c77d1f, SUC_lexicon_tags_21 },
    { 0xe7677d20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6277d23, SUC_lexicon_tags_7 },
    { 0xa5c77d24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b677d26, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x80077d29, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x15677d2c, SUC_lexicon_tags_82 },
    { 0x62a77d2d, SUC_lexicon_tags_25 },
    { 0x90277d2c, SUC_lexicon_tags_1 },
    { 0x35c77d2f, SUC_lexicon_tags_6 },
    { 0xb7c77d30, SUC_lexicon_tags_56 },
    { 0x2c877d2f, SUC_lexicon_tags_38 },
    { 0xf2077d32, SUC_lexicon_tags_11 },
    { 0xd6c77d33, SUC_lexicon_tags_63 },
    { 0x63a77d34, SUC_lexicon_tags_60 },
    { 0xcea77d35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91677d3e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55077d4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53077d4c, SUC_lexicon_tags_32 },
    { 0x84077d4c, SUC_lexicon_tags_11 },
    { 0x75877d4e, SUC_lexicon_tags_25 },
    { 0x61677d4f, SUC_lexicon_tags_49 },
    { 0x4c077d50, SUC_lexicon_tags_9 },
    { 0xdfe77d4e, SUC_lexicon_tags_54 },
    { 0xf2e77d50, SUC_lexicon_tags_26 },
    { 0xb7a77d53, SUC_lexicon_tags_60 },
    { 0x13a77d54, SUC_lexicon_tags_37 },
    { 0x86677d4c, SUC_lexicon_tags_6 },
    { 0x12277d51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba77d5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a77d60, SUC_lexicon_tags_17 },
    { 0xab277d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66077d68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd277d6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6a77d6c, SUC_lexicon_tags_82 },
    { 0x99877d6c, SUC_lexicon_tags_17 },
    { 0x98877d6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6477d70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec77d75, SUC_lexicon_tags_37 },
    { 0x86877d76, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea477d7c, SUC_lexicon_tags_60 },
    { 0x5b477d7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd677d81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc0877d83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc77d89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c77d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e77d96, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1877d9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2077d9f, SUC_lexicon_tags_37 },
    { 0xac277d9f, SUC_lexicon_tags_11 },
    { 0xb3c77d9f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5477da4, SUC_lexicon_tags_62 },
    { 0xa6677da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e877da9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33277dad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6477db7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x39677dba, SUC_lexicon_tags_19 },
    { 0x3e077dbb, SUC_lexicon_tags_63 },
    { 0xe0877dbb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6677dbf, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xc4e77dc1, SUC_lexicon_tags_6 },
    { 0xae677dc1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa677dc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3477dc8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xec677dca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c277dcf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72877dd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec77dd7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87477ddb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa9a77ddd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c77de0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf4c77de2, SUC_lexicon_tags_63 },
    { 0x4f677de2, SUC_lexicon_tags_47 },
    { 0xaf077de4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac77de7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x86677dea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2077def, SUC_lexicon_tags_2 },
    { 0xf6677def, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72677df4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34677df9, SUC_lexicon_tags_11 },
    { 0x14877dfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d477dfd, SUC_lexicon_tags_184 },
    { 0xee77dfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a77e04, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a77e07, SUC_lexicon_tags_11 },
    { 0xb8a77e08, SUC_lexicon_tags_7 },
    { 0x3de77e09, SUC_lexicon_tags_43 },
    { 0x8f877e0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e77e0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a77e11, SUC_lexicon_tags_65 },
    { 0xfba77e12, SUC_lexicon_tags_31 },
    { 0x9cc77e12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c77e17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea77e1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a77e21, SUC_lexicon_tags_30 },
    { 0x4ea77e22, SUC_lexicon_tags_1 },
    { 0x72c77e23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6077e25, SUC_lexicon_tags_39 },
    { 0xeec77e26, SUC_lexicon_tags_12 },
    { 0xcd677e27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e077e29, SUC_lexicon_tags_8 },
    { 0x7da77e2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce77e2e, SUC_lexicon_tags_37 },
    { 0xc8277e2e, SUC_lexicon_tags_15 },
    { 0x89e77e2e, SUC_lexicon_tags_1 },
    { 0xcf877e2f, SUC_lexicon_tags_11 },
    { 0x57c77e32, SUC_lexicon_tags_27 },
    { 0x41477e33, SUC_lexicon_tags_12 },
    { 0xcb677e34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e277e38, SUC_lexicon_tags_11 },
    { 0x5be77e38, SUC_lexicon_tags_35 },
    { 0x2de77e3a, SUC_lexicon_tags_25 },
    { 0xa77e3a, SUC_lexicon_tags_17 },
    { 0xfac77e3c, SUC_lexicon_tags_23 },
    { 0xeb477e3d, SUC_lexicon_tags_15 },
    { 0xe0277e3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7477e42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c77e45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x96677e47, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9077e4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62a77e4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d877e4f, SUC_lexicon_tags_64 },
    { 0x9a77e50, SUC_lexicon_tags_37 },
    { 0xd5477e50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25677e53, SUC_lexicon_tags_25 },
    { 0xeba77e54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e277e57, SUC_lexicon_tags_35 },
    { 0x22e77e58, SUC_lexicon_tags_17 },
    { 0xfa477e59, SUC_lexicon_tags_0 },
    { 0x37877e5a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x42277e5d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f277e63, SUC_lexicon_tags_1 },
    { 0x1b077e63, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xae477e66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29877e68, SUC_lexicon_tags_7 },
    { 0x96c77e69, SUC_lexicon_tags_1 },
    { 0x94877e6a, SUC_lexicon_tags_17 },
    { 0xaec77e68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c77e71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89677e74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d677e77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c77e7a, SUC_lexicon_tags_30 },
    { 0xc0077e7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec77e80, SUC_lexicon_tags_11 },
    { 0x5f277e80, SUC_lexicon_tags_60 },
    { 0x29a77e82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a677e84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8877e88, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d877e8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x56677e8f, SUC_lexicon_tags_82 },
    { 0x82c77e8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x13077e93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e77e97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7277e9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4677ea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e877ea5, SUC_lexicon_tags_2 },
    { 0x87877ea6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bc77ea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f277eab, SUC_lexicon_tags_123 },
    { 0xc3277eac, SUC_lexicon_tags_38 },
    { 0xc1477ead, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x12077eaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bc77eb1, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2677eb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc77ebb, SUC_lexicon_tags_2 },
    { 0x71677ebb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde277ebf, SUC_lexicon_tags_56 },
    { 0x56477ec0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a077ec2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e77ec5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa277ec8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4277eca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c77ecf, SUC_lexicon_tags_6 },
    { 0x34c77ed0, SUC_lexicon_tags_2 },
    { 0x6f477ed0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7877edd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfee77edf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x98277ee1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x76677ee3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2877ee8, SUC_lexicon_tags_1 },
    { 0x2c77ee8, SUC_lexicon_tags_17 },
    { 0xf6477eea, SUC_lexicon_tags_2 },
    { 0x57877ee8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa7677eed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5dc77eef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x47077ef1, SUC_lexicon_tags_39 },
    { 0x18077ef2, SUC_lexicon_tags_37 },
    { 0xad677ef3, SUC_lexicon_tags_33 },
    { 0xbae77ef4, SUC_lexicon_tags_16 },
    { 0xc4877ef5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ba77ef7, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3677efb, SUC_lexicon_tags_49 },
    { 0xc7a77efc, SUC_lexicon_tags_12 },
    { 0x33677efc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac77f00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e77f03, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e77f0a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce77f14, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x27477f17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1677f1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3077f20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a77f23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5677f27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80e77f29, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7277f2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98077f30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc77f33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44477f3b, SUC_lexicon_tags_15 },
    { 0x2d677f3c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6c77f3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b277f41, SUC_lexicon_tags_27 },
    { 0xf3877f42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85877f47, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf077f4e, SUC_lexicon_tags_7 },
    { 0xf0077f4f, SUC_lexicon_tags_85 },
    { 0xcda77f4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7fc77f52, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe1e77f54, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x18877f56, SUC_lexicon_tags_83 },
    { 0xa9077f57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada77f5b, SUC_lexicon_tags_2 },
    { 0x31677f5b, SUC_lexicon_tags_19 },
    { 0xaa077f5d, SUC_lexicon_tags_37 },
    { 0xa3877f5c, SUC_lexicon_tags_37 },
    { 0xea077f5d, SUC_lexicon_tags_14 },
    { 0xe6c77f60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c77f6a, SUC_lexicon_tags_55 },
    { 0x2f477f6b, SUC_lexicon_tags_341 },
    { 0x7da77f6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x66477f6f, SUC_lexicon_tags_8 },
    { 0x78877f6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa77f75, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79477f80, SUC_lexicon_tags_31 },
    { 0x2ee77f80, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc77f87, SUC_lexicon_tags_11 },
    { 0x63877f88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a77f8a, SUC_lexicon_tags_12 },
    { 0x16c77f8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31077f8f, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49677f93, SUC_lexicon_tags_22 },
    { 0xc1a77f93, SUC_lexicon_tags_49 },
    { 0xc7277f95, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x81e77f97, SUC_lexicon_tags_11 },
    { 0x51477f98, SUC_lexicon_tags_17 },
    { 0xc2877f97, SUC_lexicon_tags_37 },
    { 0x20677f98, SUC_lexicon_tags_32 },
    { 0x95a77f9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e677f9e, SUC_lexicon_tags_11 },
    { 0xf1e77f9f, SUC_lexicon_tags_37 },
    { 0x6e277fa0, SUC_lexicon_tags_98 },
    { 0x88c77fa1, SUC_lexicon_tags_1 },
    { 0x35877fa2, SUC_lexicon_tags_4 },
    { 0xdcc77fa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73277fa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6c77fab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3477faf, SUC_lexicon_tags_29 },
    { 0x5aa77fb0, SUC_lexicon_tags_32 },
    { 0x11877fb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacc77fb3, SUC_lexicon_tags_14 },
    { 0xea277fb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c77fb7, SUC_lexicon_tags_57 },
    { 0xb9277fb8, SUC_lexicon_tags_19 },
    { 0x3f477fb9, SUC_lexicon_tags_124 },
    { 0x64c77fba, SUC_lexicon_tags_17 },
    { 0xbd277fba, SUC_lexicon_tags_11 },
    { 0x6c477fbc, SUC_lexicon_tags_7 },
    { 0x47e77fbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c77fc5, SUC_lexicon_tags_60 },
    { 0xb8a77fc6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa477fc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95a77fcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a677fce, SUC_lexicon_tags_47 },
    { 0x9a77fcf, SUC_lexicon_tags_21 },
    { 0xd6277fd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e77fd3, SUC_lexicon_tags_27 },
    { 0xac077fd4, SUC_lexicon_tags_17 },
    { 0xa1e77fd3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac77fd8, SUC_lexicon_tags_14 },
    { 0x1c477fd9, SUC_lexicon_tags_38 },
    { 0x29277fda, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe77fe0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x36477fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6a77fe5, SUC_lexicon_tags_33 },
    { 0x25477fe6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25877fe8, SUC_lexicon_tags_234 },
    { 0x22277fe9, SUC_lexicon_tags_27 },
    { 0xb2477fe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65877fee, SUC_lexicon_tags_111 },
    { 0xa9877fef, SUC_lexicon_tags_11 },
    { 0x6a877ff0, SUC_lexicon_tags_48 },
    { 0x13a77ff1, SUC_lexicon_tags_25 },
    { 0xeae77ff1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e77ff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c77ff8, SUC_lexicon_tags_25 },
    { 0xc6a77ff8, SUC_lexicon_tags_25 },
    { 0x64a77ffa, SUC_lexicon_tags_419 },
    { 0xf277ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c78000, SUC_lexicon_tags_30 },
    { 0x1e78001, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x97278003, SUC_lexicon_tags_17 },
    { 0x4a278004, SUC_lexicon_tags_45 },
    { 0x74078005, SUC_lexicon_tags_11 },
    { 0x8ec78005, SUC_lexicon_tags_27 },
    { 0xdf478007, SUC_lexicon_tags_43 },
    { 0x98c78008, SUC_lexicon_tags_37 },
    { 0x5de78009, SUC_lexicon_tags_25 },
    { 0x4ba7800a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4e7800c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83278010, SUC_lexicon_tags_35 },
    { 0x42278011, SUC_lexicon_tags_2 },
    { 0x6d878011, SUC_lexicon_tags_1 },
    { 0x38a78011, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47a78015, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e7801b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0678023, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb278025, SUC_lexicon_tags_25 },
    { 0xbc78026, SUC_lexicon_tags_12 },
    { 0x13a78027, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887802d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9047802f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a78033, SUC_lexicon_tags_15 },
    { 0xa5a78033, SUC_lexicon_tags_30 },
    { 0x2ba78035, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d278037, SUC_lexicon_tags_1 },
    { 0x8478038, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2f67803a, SUC_lexicon_tags_31 },
    { 0xfe07803b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x527803d, SUC_lexicon_tags_37 },
    { 0x4de7803d, SUC_lexicon_tags_7 },
    { 0x5c67803f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a78042, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf078046, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea278049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc7804e, SUC_lexicon_tags_15 },
    { 0x2687804f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0a78051, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11878059, SUC_lexicon_tags_47 },
    { 0x6c7805a, SUC_lexicon_tags_25 },
    { 0x5087805b, SUC_lexicon_tags_60 },
    { 0x507805a, SUC_lexicon_tags_15 },
    { 0x6007805d, SUC_lexicon_tags_17 },
    { 0x80e7805e, SUC_lexicon_tags_65 },
    { 0x2987805f, SUC_lexicon_tags_35 },
    { 0x24a7805e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a078062, SUC_lexicon_tags_43 },
    { 0x39c78062, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc78066, SUC_lexicon_tags_19 },
    { 0x6d478067, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8827806a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5678072, SUC_lexicon_tags_12 },
    { 0x76478073, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf678076, SUC_lexicon_tags_34 },
    { 0x96a78076, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1267807d, SUC_lexicon_tags_12 },
    { 0x6847807e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bc78080, SUC_lexicon_tags_2 },
    { 0xe0878081, SUC_lexicon_tags_15 },
    { 0xfa678081, SUC_lexicon_tags_29 },
    { 0xcfa78081, SUC_lexicon_tags_14 },
    { 0x88e78083, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9ca78086, SUC_lexicon_tags_14 },
    { 0x14878087, SUC_lexicon_tags_47 },
    { 0xf2478088, SUC_lexicon_tags_25 },
    { 0x88078087, SUC_lexicon_tags_47 },
    { 0x4ca7808a, SUC_lexicon_tags_15 },
    { 0x13e78089, SUC_lexicon_tags_28 },
    { 0x60678089, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0678090, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f078094, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d078096, SUC_lexicon_tags_25 },
    { 0x9d078096, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a27809d, SUC_lexicon_tags_0 },
    { 0x9447809d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144780a3, SUC_lexicon_tags_0 },
    { 0x6aa780a4, SUC_lexicon_tags_84 },
    { 0x54c780a5, SUC_lexicon_tags_26 },
    { 0xacc780a6, SUC_lexicon_tags_17 },
    { 0x11e780a7, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4780aa, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e780ad, SUC_lexicon_tags_15 },
    { 0x32780ae, SUC_lexicon_tags_37 },
    { 0x14e780af, SUC_lexicon_tags_25 },
    { 0x834780b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a780b5, SUC_lexicon_tags_25 },
    { 0x69a780b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84780b9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444780bd, SUC_lexicon_tags_2 },
    { 0x2a4780be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a780c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f6780c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3ce780c9, SUC_lexicon_tags_17 },
    { 0x49e780c9, SUC_lexicon_tags_0 },
    { 0xb88780ca, SUC_lexicon_tags_37 },
    { 0x636780cc, SUC_lexicon_tags_25 },
    { 0x36780cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae780d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26e780d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be780d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3c780da, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb90780dc, SUC_lexicon_tags_31 },
    { 0x850780dd, SUC_lexicon_tags_65 },
    { 0xa8c780dc, SUC_lexicon_tags_8 },
    { 0x35e780df, SUC_lexicon_tags_2 },
    { 0x1fe780e0, SUC_lexicon_tags_63 },
    { 0xf74780e1, SUC_lexicon_tags_49 },
    { 0x2fa780df, SUC_lexicon_tags_17 },
    { 0x934780e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72780e6, SUC_lexicon_tags_43 },
    { 0xa82780e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0780eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6780f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84780f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda780f7, SUC_lexicon_tags_56 },
    { 0xdb0780f8, SUC_lexicon_tags_7 },
    { 0xc58780f9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608780fd, SUC_lexicon_tags_13 },
    { 0xdac780fd, SUC_lexicon_tags_15 },
    { 0x94a780ff, SUC_lexicon_tags_11 },
    { 0xf56780fe, SUC_lexicon_tags_15 },
    { 0x60678101, SUC_lexicon_tags_17 },
    { 0xf078102, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1678106, SUC_lexicon_tags_38 },
    { 0x2fe78107, SUC_lexicon_tags_30 },
    { 0xd5c78107, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e7810c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a078111, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9dc78113, SUC_lexicon_tags_8 },
    { 0x63e78114, SUC_lexicon_tags_17 },
    { 0xf4c78114, SUC_lexicon_tags_9 },
    { 0xc1c78116, SUC_lexicon_tags_37 },
    { 0x51e78116, SUC_lexicon_tags_28 },
    { 0xbaa78116, SUC_lexicon_tags_1 },
    { 0xbe78116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba7811b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3e7811d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be78120, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc78126, SUC_lexicon_tags_30 },
    { 0x25478127, SUC_lexicon_tags_23 },
    { 0xa0078127, SUC_lexicon_tags_49 },
    { 0x58a78129, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdc47812b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a78132, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16c78134, SUC_lexicon_tags_49 },
    { 0x34c78135, SUC_lexicon_tags_25 },
    { 0xe1a78135, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7387813a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x23e7813c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34c7813e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6be78140, SUC_lexicon_tags_12 },
    { 0x72c78140, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe1078143, SUC_lexicon_tags_8 },
    { 0x91a78143, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba678149, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff07814e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb8a78150, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48678154, SUC_lexicon_tags_30 },
    { 0x75278154, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed47815c, SUC_lexicon_tags_45 },
    { 0x1ee7815c, SUC_lexicon_tags_25 },
    { 0xc9c7815e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe78161, SUC_lexicon_tags_38 },
    { 0x9a478162, SUC_lexicon_tags_183 },
    { 0xece78162, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24078165, SUC_lexicon_tags_39 },
    { 0xa1c78166, SUC_lexicon_tags_11 },
    { 0x96078167, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf3e78169, SUC_lexicon_tags_19 },
    { 0xf1e7816a, SUC_lexicon_tags_17 },
    { 0x4007816b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17478171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b878175, SUC_lexicon_tags_11 },
    { 0x4ca78176, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e7817c, SUC_lexicon_tags_15 },
    { 0xe747817d, SUC_lexicon_tags_12 },
    { 0x8287817c, SUC_lexicon_tags_114 },
    { 0xafc7817c, SUC_lexicon_tags_54 },
    { 0x94278180, SUC_lexicon_tags_25 },
    { 0x6ae7817c, SUC_lexicon_tags_27 },
    { 0xc9878182, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6278186, SUC_lexicon_tags_2 },
    { 0xb6e78187, SUC_lexicon_tags_19 },
    { 0xb1678186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f27818a, SUC_lexicon_tags_14 },
    { 0x1c47818b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9147818d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9478191, SUC_lexicon_tags_6 },
    { 0xc2c78191, SUC_lexicon_tags_531 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d678195, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f278197, SUC_lexicon_tags_22 },
    { 0xedc78198, SUC_lexicon_tags_25 },
    { 0xd8c78199, SUC_lexicon_tags_23 },
    { 0x48c78198, SUC_lexicon_tags_12 },
    { 0x9f478199, SUC_lexicon_tags_47 },
    { 0x6bc7819c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd227819e, SUC_lexicon_tags_37 },
    { 0x97e7819f, SUC_lexicon_tags_17 },
    { 0xcce781a0, SUC_lexicon_tags_18 },
    { 0x860781a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4781aa, SUC_lexicon_tags_25 },
    { 0x608781ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e781af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e781b2, SUC_lexicon_tags_37 },
    { 0x172781b2, SUC_lexicon_tags_21 },
    { 0xb10781b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78781b7, SUC_lexicon_tags_66 },
    { 0xcb4781b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa00781ba, SUC_lexicon_tags_96 },
    { 0x28781bb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc10781be, SUC_lexicon_tags_31 },
    { 0x9b8781bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76781c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa781c7, SUC_lexicon_tags_27 },
    { 0xeb0781c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c781cb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x528781ce, SUC_lexicon_tags_2 },
    { 0x154781cf, SUC_lexicon_tags_10 },
    { 0x7ee781d0, SUC_lexicon_tags_39 },
    { 0x510781ce, SUC_lexicon_tags_7 },
    { 0xc8781d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842781d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46781d9, SUC_lexicon_tags_14 },
    { 0x732781da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6781dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6781df, SUC_lexicon_tags_60 },
    { 0x28c781df, SUC_lexicon_tags_0 },
    { 0x66a781e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c781ec, SUC_lexicon_tags_25 },
    { 0xf0e781ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488781f4, SUC_lexicon_tags_1 },
    { 0xd1e781f5, SUC_lexicon_tags_17 },
    { 0xb54781f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa781fb, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb878200, SUC_lexicon_tags_17 },
    { 0xa5e78201, SUC_lexicon_tags_11 },
    { 0x98a78201, SUC_lexicon_tags_13 },
    { 0xf878203, SUC_lexicon_tags_37 },
    { 0xa4e78202, SUC_lexicon_tags_63 },
    { 0xde878205, SUC_lexicon_tags_37 },
    { 0xf0678206, SUC_lexicon_tags_10 },
    { 0x9ba78207, SUC_lexicon_tags_1 },
    { 0x6ba78208, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x11e7820a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4678210, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x35278213, SUC_lexicon_tags_27 },
    { 0x6c278214, SUC_lexicon_tags_0 },
    { 0xc3078215, SUC_lexicon_tags_43 },
    { 0x7ba78215, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc27821a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc47821c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa7821f, SUC_lexicon_tags_17 },
    { 0xa1078220, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a78223, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8b478225, SUC_lexicon_tags_11 },
    { 0x4c78225, SUC_lexicon_tags_17 },
    { 0x8b678227, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e7822b, SUC_lexicon_tags_49 },
    { 0xc207822b, SUC_lexicon_tags_43 },
    { 0xf5e7822d, SUC_lexicon_tags_118 },
    { 0x77a7822e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7678233, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa78236, SUC_lexicon_tags_17 },
    { 0x99678237, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xee078239, SUC_lexicon_tags_47 },
    { 0x3be7823a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x227823d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd278243, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc2c78245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20078249, SUC_lexicon_tags_17 },
    { 0x6767824a, SUC_lexicon_tags_27 },
    { 0x2ac7824b, SUC_lexicon_tags_19 },
    { 0xf5e78249, SUC_lexicon_tags_37 },
    { 0xb5a7824d, SUC_lexicon_tags_60 },
    { 0xb4a7824c, SUC_lexicon_tags_0 },
    { 0x20a78249, SUC_lexicon_tags_49 },
    { 0xf3078250, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68678254, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xef878257, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97678260, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c78263, SUC_lexicon_tags_2 },
    { 0x4e278264, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b67826e, SUC_lexicon_tags_47 },
    { 0x9987826e, SUC_lexicon_tags_29 },
    { 0xa4c78270, SUC_lexicon_tags_25 },
    { 0xa07826f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1678277, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e78280, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc278283, SUC_lexicon_tags_28 },
    { 0x3b078284, SUC_lexicon_tags_12 },
    { 0x93a78285, SUC_lexicon_tags_37 },
    { 0x7bc78284, SUC_lexicon_tags_15 },
    { 0xdd478287, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3627828a, SUC_lexicon_tags_66 },
    { 0x5dc7828b, SUC_lexicon_tags_37 },
    { 0xe2a7828b, SUC_lexicon_tags_10 },
    { 0x2487828d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a78290, SUC_lexicon_tags_43 },
    { 0x2be78291, SUC_lexicon_tags_108 },
    { 0xf9c78291, SUC_lexicon_tags_8 },
    { 0x47678291, SUC_lexicon_tags_25 },
    { 0x66a78291, SUC_lexicon_tags_28 },
    { 0x22478295, SUC_lexicon_tags_12 },
    { 0xac478295, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91278299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1547829b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96a7829d, SUC_lexicon_tags_25 },
    { 0x91c7829e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386782a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c782a8, SUC_lexicon_tags_25 },
    { 0x88a782a9, SUC_lexicon_tags_11 },
    { 0x2c782a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a782ad, SUC_lexicon_tags_11 },
    { 0x31a782ad, SUC_lexicon_tags_60 },
    { 0x186782af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa782b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x208782b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee782b8, SUC_lexicon_tags_63 },
    { 0xcca782b9, SUC_lexicon_tags_31 },
    { 0x18c782b8, SUC_lexicon_tags_17 },
    { 0xd5c782bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2782c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870782c4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4782c9, SUC_lexicon_tags_55 },
    { 0xc9a782ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0782cd, SUC_lexicon_tags_6 },
    { 0x818782ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36782d4, SUC_lexicon_tags_12 },
    { 0xa08782d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe4e782d7, SUC_lexicon_tags_1 },
    { 0xc7a782d8, SUC_lexicon_tags_12 },
    { 0xda6782d8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x436782dc, SUC_lexicon_tags_14 },
    { 0x6da782dc, SUC_lexicon_tags_47 },
    { 0x6a782de, SUC_lexicon_tags_18 },
    { 0x642782df, SUC_lexicon_tags_6 },
    { 0xb0a782df, SUC_lexicon_tags_39 },
    { 0x9f8782e1, SUC_lexicon_tags_2 },
    { 0x8ee782e0, SUC_lexicon_tags_4 },
    { 0x1e782e3, SUC_lexicon_tags_42 },
    { 0x212782e4, SUC_lexicon_tags_2 },
    { 0xe70782e5, SUC_lexicon_tags_25 },
    { 0xbde782e6, SUC_lexicon_tags_21 },
    { 0xc12782e7, SUC_lexicon_tags_8 },
    { 0x7e782e8, SUC_lexicon_tags_7 },
    { 0x292782e7, SUC_lexicon_tags_12 },
    { 0xbf2782ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20782ef, SUC_lexicon_tags_16 },
    { 0xfb8782f0, SUC_lexicon_tags_11 },
    { 0xd58782f1, SUC_lexicon_tags_65 },
    { 0x932782f2, SUC_lexicon_tags_11 },
    { 0x8e4782f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x29e782f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6782fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b878300, SUC_lexicon_tags_28 },
    { 0x22e78301, SUC_lexicon_tags_2 },
    { 0x1fc78302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23278306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa878308, SUC_lexicon_tags_19 },
    { 0x88c78309, SUC_lexicon_tags_87 },
    { 0x927830a, SUC_lexicon_tags_11 },
    { 0x6a7830a, SUC_lexicon_tags_2 },
    { 0x2327830b, SUC_lexicon_tags_17 },
    { 0x2547830d, SUC_lexicon_tags_28 },
    { 0x5927830a, SUC_lexicon_tags_33 },
    { 0x98c7830f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80078313, SUC_lexicon_tags_45 },
    { 0x1ac78314, SUC_lexicon_tags_60 },
    { 0x26078314, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e7831e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f67832d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c78333, SUC_lexicon_tags_19 },
    { 0xa1878333, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe78338, SUC_lexicon_tags_25 },
    { 0xe1278339, SUC_lexicon_tags_35 },
    { 0x7207833a, SUC_lexicon_tags_37 },
    { 0x6727833b, SUC_lexicon_tags_7 },
    { 0x9e27833c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fe7833e, SUC_lexicon_tags_1 },
    { 0x9c47833e, SUC_lexicon_tags_37 },
    { 0xcf478340, SUC_lexicon_tags_28 },
    { 0xbca78341, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d078343, SUC_lexicon_tags_37 },
    { 0x13e78344, SUC_lexicon_tags_12 },
    { 0xb0278345, SUC_lexicon_tags_16 },
    { 0x8a78345, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb847834f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x11878351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73c78353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe78357, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e7835c, SUC_lexicon_tags_17 },
    { 0x2847835c, SUC_lexicon_tags_63 },
    { 0xd867835e, SUC_lexicon_tags_33 },
    { 0x1e67835f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x75478361, SUC_lexicon_tags_60 },
    { 0x4b078361, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80278364, SUC_lexicon_tags_12 },
    { 0x27e78364, SUC_lexicon_tags_43 },
    { 0x19678366, SUC_lexicon_tags_0 },
    { 0x9fc78367, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd47836d, SUC_lexicon_tags_37 },
    { 0x7836e, SUC_lexicon_tags_11 },
    { 0x6f87836f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be78376, SUC_lexicon_tags_14 },
    { 0xb8a78376, SUC_lexicon_tags_27 },
    { 0xd0878378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c47837a, SUC_lexicon_tags_61 },
    { 0x127837a, SUC_lexicon_tags_37 },
    { 0xa647837c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1647837f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e78382, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c78388, SUC_lexicon_tags_17 },
    { 0xd0878389, SUC_lexicon_tags_37 },
    { 0xdda7838a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c7838d, SUC_lexicon_tags_11 },
    { 0x5d27838d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdea78390, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66078392, SUC_lexicon_tags_49 },
    { 0x8d678392, SUC_lexicon_tags_31 },
    { 0xaf078394, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc678396, SUC_lexicon_tags_27 },
    { 0x63478397, SUC_lexicon_tags_37 },
    { 0xffe78398, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8047839b, SUC_lexicon_tags_17 },
    { 0x65c7839c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2307839e, SUC_lexicon_tags_17 },
    { 0x5a47839f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54783a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576783a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x444783a9, SUC_lexicon_tags_583 },
    { 0x648783a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x158783ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea783b0, SUC_lexicon_tags_25 },
    { 0xe66783b1, SUC_lexicon_tags_11 },
    { 0xe1e783b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30783b5, SUC_lexicon_tags_38 },
    { 0x7f6783b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x590783b9, SUC_lexicon_tags_25 },
    { 0xb80783ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ac783bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a783bf, SUC_lexicon_tags_3 },
    { 0x52c783c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd76783c2, SUC_lexicon_tags_25 },
    { 0x11a783c2, SUC_lexicon_tags_1 },
    { 0x6fc783c4, SUC_lexicon_tags_11 },
    { 0x9e8783c3, SUC_lexicon_tags_15 },
    { 0xbb4783c6, SUC_lexicon_tags_15 },
    { 0xa60783c7, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0, NULL },
    { 0x994783ca, SUC_lexicon_tags_7 },
    { 0xe18783cb, SUC_lexicon_tags_1 },
    { 0x364783cb, SUC_lexicon_tags_20 },
    { 0x7be783cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa783cf, SUC_lexicon_tags_32 },
    { 0x55c783cf, SUC_lexicon_tags_68 },
    { 0xbcc783d1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4f8783d3, SUC_lexicon_tags_11 },
    { 0x4c0783d3, SUC_lexicon_tags_44 },
    { 0x4ae783d4, SUC_lexicon_tags_15 },
    { 0xae8783d6, SUC_lexicon_tags_49 },
    { 0x58783d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4783da, SUC_lexicon_tags_15 },
    { 0xd6783da, SUC_lexicon_tags_12 },
    { 0x184783db, SUC_lexicon_tags_35 },
    { 0x84c783dd, SUC_lexicon_tags_45 },
    { 0x78783de, SUC_lexicon_tags_47 },
    { 0x786783df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x222783e2, SUC_lexicon_tags_25 },
    { 0xc30783e3, SUC_lexicon_tags_17 },
    { 0x7c6783e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc72783e6, SUC_lexicon_tags_62 },
    { 0x832783e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x67c783e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa98783eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30783f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc783f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21a783f6, SUC_lexicon_tags_43 },
    { 0x634783f7, SUC_lexicon_tags_37 },
    { 0x7c2783f8, SUC_lexicon_tags_23 },
    { 0x330783f7, SUC_lexicon_tags_55 },
    { 0x5be783f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4783fd, SUC_lexicon_tags_37 },
    { 0xc74783fd, SUC_lexicon_tags_14 },
    { 0xc26783ff, SUC_lexicon_tags_29 },
    { 0xfb478400, SUC_lexicon_tags_20 },
    { 0x4e783ff, SUC_lexicon_tags_79 },
    { 0xcf478402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d878405, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc78408, SUC_lexicon_tags_15 },
    { 0xf8e78409, SUC_lexicon_tags_12 },
    { 0x50478409, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3347840c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a78411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd678418, SUC_lexicon_tags_45 },
    { 0x5d478419, SUC_lexicon_tags_15 },
    { 0xe1a7841a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4967841e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e78424, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7078428, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e78439, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74a7843b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8467843e, SUC_lexicon_tags_25 },
    { 0x507843f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9278443, SUC_lexicon_tags_52 },
    { 0x83478444, SUC_lexicon_tags_17 },
    { 0xad078445, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0478448, SUC_lexicon_tags_62 },
    { 0x6d278449, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b67844c, SUC_lexicon_tags_49 },
    { 0x7867844d, SUC_lexicon_tags_30 },
    { 0x2a7844e, SUC_lexicon_tags_25 },
    { 0xe8a7844c, SUC_lexicon_tags_31 },
    { 0x99e7844e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x93878452, SUC_lexicon_tags_25 },
    { 0xd8478452, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x91e78455, SUC_lexicon_tags_38 },
    { 0xb9078456, SUC_lexicon_tags_37 },
    { 0x1b878457, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfae78459, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7845f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da7846b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x64c7846d, SUC_lexicon_tags_59 },
    { 0x6227846e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b678471, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b278473, SUC_lexicon_tags_17 },
    { 0x14878474, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1478477, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca7847c, SUC_lexicon_tags_2 },
    { 0xc847847c, SUC_lexicon_tags_15 },
    { 0xa0a7847e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63078480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6678483, SUC_lexicon_tags_17 },
    { 0x49278484, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae78488, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d67848a, SUC_lexicon_tags_12 },
    { 0x2a07848b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8607848e, SUC_lexicon_tags_15 },
    { 0x6427848f, SUC_lexicon_tags_38 },
    { 0x6627848e, SUC_lexicon_tags_33 },
    { 0xb4a78491, SUC_lexicon_tags_8 },
    { 0x6847848e, SUC_lexicon_tags_25 },
    { 0x75e78493, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40678495, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e7849e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2784a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84784aa, SUC_lexicon_tags_66 },
    { 0x5c6784ab, SUC_lexicon_tags_19 },
    { 0xefc784ac, SUC_lexicon_tags_11 },
    { 0x3cc784ad, SUC_lexicon_tags_21 },
    { 0x50784ad, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a784b5, SUC_lexicon_tags_45 },
    { 0x5ba784b5, SUC_lexicon_tags_63 },
    { 0xc62784b5, SUC_lexicon_tags_603 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc784ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a784c1, SUC_lexicon_tags_1 },
    { 0x584784c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96784c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226784c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec784d1, SUC_lexicon_tags_11 },
    { 0x62a784d1, SUC_lexicon_tags_9 },
    { 0x77a784d3, SUC_lexicon_tags_55 },
    { 0xe44784d3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c784da, SUC_lexicon_tags_14 },
    { 0x46c784da, SUC_lexicon_tags_43 },
    { 0xd50784dc, SUC_lexicon_tags_55 },
    { 0xff8784dc, SUC_lexicon_tags_15 },
    { 0x76784de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44784e1, SUC_lexicon_tags_34 },
    { 0x7e4784e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be784e6, SUC_lexicon_tags_87 },
    { 0xfac784e6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c784ea, SUC_lexicon_tags_38 },
    { 0xe0e784ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0784f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca4784fb, SUC_lexicon_tags_33 },
    { 0xc64784fc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba478502, SUC_lexicon_tags_220 },
    { 0x6a478502, SUC_lexicon_tags_34 },
    { 0xb6678504, SUC_lexicon_tags_17 },
    { 0xb2a78504, SUC_lexicon_tags_60 },
    { 0xe0a78506, SUC_lexicon_tags_25 },
    { 0xce278506, SUC_lexicon_tags_40 },
    { 0xf9e78508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc47850a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfca7850c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7850f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23278514, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2278517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3c78519, SUC_lexicon_tags_25 },
    { 0x7127851a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c78520, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20a78522, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29478526, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07852a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6f27852c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7852e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3478532, SUC_lexicon_tags_17 },
    { 0x7f478532, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x82278536, SUC_lexicon_tags_97 },
    { 0x57878537, SUC_lexicon_tags_38 },
    { 0x8ec78538, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e67853a, SUC_lexicon_tags_17 },
    { 0x1e87853b, SUC_lexicon_tags_31 },
    { 0x7807853c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd987853f, SUC_lexicon_tags_15 },
    { 0xf4a78540, SUC_lexicon_tags_11 },
    { 0x8678541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15078543, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3287854b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe107854d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22678551, SUC_lexicon_tags_2 },
    { 0x4b678552, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d278558, SUC_lexicon_tags_49 },
    { 0x17678559, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8287855b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a27855d, SUC_lexicon_tags_17 },
    { 0x1747855e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x48678561, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0878564, SUC_lexicon_tags_17 },
    { 0x4e078564, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba78568, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a678572, SUC_lexicon_tags_23 },
    { 0x64c78573, SUC_lexicon_tags_25 },
    { 0x71478573, SUC_lexicon_tags_17 },
    { 0xc1878572, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1078579, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66e7857b, SUC_lexicon_tags_43 },
    { 0x2627857c, SUC_lexicon_tags_1 },
    { 0xff87857d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe527857f, SUC_lexicon_tags_2 },
    { 0xb1078580, SUC_lexicon_tags_15 },
    { 0xf6878581, SUC_lexicon_tags_21 },
    { 0x3fa78582, SUC_lexicon_tags_15 },
    { 0x25878583, SUC_lexicon_tags_37 },
    { 0xbb078584, SUC_lexicon_tags_7 },
    { 0xfda78585, SUC_lexicon_tags_15 },
    { 0x25478583, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c7858b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea7858e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9078599, SUC_lexicon_tags_61 },
    { 0x627859a, SUC_lexicon_tags_14 },
    { 0xf207859a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8807859d, SUC_lexicon_tags_19 },
    { 0x38e7859d, SUC_lexicon_tags_17 },
    { 0x6b67859d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa785a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a785a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97c785ab, SUC_lexicon_tags_15 },
    { 0x58e785ac, SUC_lexicon_tags_68 },
    { 0x89a785ad, SUC_lexicon_tags_25 },
    { 0x708785ad, SUC_lexicon_tags_12 },
    { 0x604785ad, SUC_lexicon_tags_33 },
    { 0x116785b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x992785b2, SUC_lexicon_tags_31 },
    { 0xfc4785b3, SUC_lexicon_tags_17 },
    { 0x816785b3, SUC_lexicon_tags_31 },
    { 0x24e785b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34785b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c2785bb, SUC_lexicon_tags_37 },
    { 0x500785bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48e785be, SUC_lexicon_tags_11 },
    { 0x5e6785bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988785c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x248785c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb56785c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd96785cb, SUC_lexicon_tags_77 },
    { 0x9be785cc, SUC_lexicon_tags_26 },
    { 0x46e785cd, SUC_lexicon_tags_6 },
    { 0xa5a785ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4785d1, SUC_lexicon_tags_19 },
    { 0x170785d2, SUC_lexicon_tags_12 },
    { 0xeee785d3, SUC_lexicon_tags_14 },
    { 0x570785d4, SUC_lexicon_tags_25 },
    { 0xe38785d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a785df, SUC_lexicon_tags_9 },
    { 0x296785df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c785e6, SUC_lexicon_tags_30 },
    { 0xc86785e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcce785e9, SUC_lexicon_tags_27 },
    { 0x74c785ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17a785ec, SUC_lexicon_tags_30 },
    { 0xaa0785ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82e785ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x75c785f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd0785f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4785f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d6785f8, SUC_lexicon_tags_11 },
    { 0x1aa785f8, SUC_lexicon_tags_11 },
    { 0xb64785fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e785fe, SUC_lexicon_tags_11 },
    { 0x914785ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e78607, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc7860b, SUC_lexicon_tags_15 },
    { 0x9987860c, SUC_lexicon_tags_9 },
    { 0x7d27860c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a478613, SUC_lexicon_tags_25 },
    { 0xd2078613, SUC_lexicon_tags_45 },
    { 0x1f878615, SUC_lexicon_tags_47 },
    { 0x59a78613, SUC_lexicon_tags_13 },
    { 0xffe78617, SUC_lexicon_tags_6 },
    { 0xbea78614, SUC_lexicon_tags_17 },
    { 0x478616, SUC_lexicon_tags_1 },
    { 0x8007861a, SUC_lexicon_tags_9 },
    { 0xaf878615, SUC_lexicon_tags_34 },
    { 0xa8a7861c, SUC_lexicon_tags_28 },
    { 0x3d078617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d878622, SUC_lexicon_tags_65 },
    { 0x50278622, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb278626, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0478629, SUC_lexicon_tags_47 },
    { 0x6de78629, SUC_lexicon_tags_2 },
    { 0x7867862b, SUC_lexicon_tags_2 },
    { 0xfb078629, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e7862f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x68278631, SUC_lexicon_tags_17 },
    { 0xfe278631, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97678638, SUC_lexicon_tags_23 },
    { 0xd1478638, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4827863d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc678646, SUC_lexicon_tags_14 },
    { 0xa4878646, SUC_lexicon_tags_8 },
    { 0x27878647, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca7864b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1387864f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15078654, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49678659, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd8c7865b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1147865d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb07865f, SUC_lexicon_tags_11 },
    { 0x89c78660, SUC_lexicon_tags_37 },
    { 0x53678661, SUC_lexicon_tags_16 },
    { 0x93478662, SUC_lexicon_tags_2 },
    { 0xa3078662, SUC_lexicon_tags_19 },
    { 0x60278664, SUC_lexicon_tags_23 },
    { 0xfc078665, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20678668, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6447866a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a67866e, SUC_lexicon_tags_14 },
    { 0x7ce7866f, SUC_lexicon_tags_37 },
    { 0x1d078670, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c278674, SUC_lexicon_tags_17 },
    { 0x20e78675, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x16878678, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xffe7867b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9a27867d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda67867f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a78684, SUC_lexicon_tags_17 },
    { 0x5c878685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47868e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a78693, SUC_lexicon_tags_8 },
    { 0x94278694, SUC_lexicon_tags_43 },
    { 0xfa478695, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x58678698, SUC_lexicon_tags_37 },
    { 0x2c278699, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e7869c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22786a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50786a6, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xf36786a8, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64786b0, SUC_lexicon_tags_2 },
    { 0xb9c786b0, SUC_lexicon_tags_7 },
    { 0x224786b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec786b5, SUC_lexicon_tags_69 },
    { 0xf4c786b6, SUC_lexicon_tags_37 },
    { 0xaf0786b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f4786b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb86786bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a786c1, SUC_lexicon_tags_21 },
    { 0xe9a786c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x584786c5, SUC_lexicon_tags_7 },
    { 0x484786c6, SUC_lexicon_tags_47 },
    { 0x72786c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa84786c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de786cc, SUC_lexicon_tags_46 },
    { 0x5bc786cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4786d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7fa786d3, SUC_lexicon_tags_27 },
    { 0xe6c786d3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96786d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62786dc, SUC_lexicon_tags_43 },
    { 0xbdc786dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0786e0, SUC_lexicon_tags_11 },
    { 0xa14786e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574786e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x108786e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c786f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86786f6, SUC_lexicon_tags_4 },
    { 0x924786f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980786fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f278702, SUC_lexicon_tags_17 },
    { 0xa3478703, SUC_lexicon_tags_37 },
    { 0xce278702, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe5278706, SUC_lexicon_tags_37 },
    { 0x20678707, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e7870a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4027870d, SUC_lexicon_tags_15 },
    { 0x7827870e, SUC_lexicon_tags_0 },
    { 0x23c7870f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe6c78711, SUC_lexicon_tags_13 },
    { 0x61e78712, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5878716, SUC_lexicon_tags_1 },
    { 0x81278717, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa327871f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa78727, SUC_lexicon_tags_17 },
    { 0xca78728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac7872c, SUC_lexicon_tags_13 },
    { 0xb407872d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa27872f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a78737, SUC_lexicon_tags_38 },
    { 0xd5078738, SUC_lexicon_tags_12 },
    { 0x9c878739, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3747873b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47078741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba78745, SUC_lexicon_tags_60 },
    { 0xd8278745, SUC_lexicon_tags_107 },
    { 0x76878747, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33c78749, SUC_lexicon_tags_17 },
    { 0x4947874a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c7874d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc78751, SUC_lexicon_tags_25 },
    { 0x6678752, SUC_lexicon_tags_1 },
    { 0xd7c78753, SUC_lexicon_tags_37 },
    { 0x1678754, SUC_lexicon_tags_11 },
    { 0x8b878754, SUC_lexicon_tags_14 },
    { 0x29878756, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e78759, SUC_lexicon_tags_1 },
    { 0x8d078759, SUC_lexicon_tags_9 },
    { 0xa247875a, SUC_lexicon_tags_37 },
    { 0xd47875b, SUC_lexicon_tags_23 },
    { 0x9fe7875d, SUC_lexicon_tags_4 },
    { 0x5747875d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9a878760, SUC_lexicon_tags_65 },
    { 0xf3478760, SUC_lexicon_tags_40 },
    { 0xc1678760, SUC_lexicon_tags_60 },
    { 0xa2e78763, SUC_lexicon_tags_2 },
    { 0xabc78764, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88e78766, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xecc78768, SUC_lexicon_tags_24 },
    { 0x48c78769, SUC_lexicon_tags_11 },
    { 0x6d27876a, SUC_lexicon_tags_39 },
    { 0xf9c7876a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c78772, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc78775, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc27877d, SUC_lexicon_tags_22 },
    { 0xe007877e, SUC_lexicon_tags_43 },
    { 0x5447877f, SUC_lexicon_tags_17 },
    { 0xf2478780, SUC_lexicon_tags_11 },
    { 0x48078781, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be78785, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x75a78787, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd1478789, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1aa7878b, SUC_lexicon_tags_37 },
    { 0xcf27878c, SUC_lexicon_tags_49 },
    { 0x78c7878b, SUC_lexicon_tags_10 },
    { 0xdd07878e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa678791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca78795, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47678798, SUC_lexicon_tags_34 },
    { 0x1a878799, SUC_lexicon_tags_11 },
    { 0xc0c78799, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb287879d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8787a1, SUC_lexicon_tags_29 },
    { 0x224787a2, SUC_lexicon_tags_17 },
    { 0xb7a787a3, SUC_lexicon_tags_0 },
    { 0x6e0787a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x520787a7, SUC_lexicon_tags_37 },
    { 0x1e787a8, SUC_lexicon_tags_27 },
    { 0xb40787a9, SUC_lexicon_tags_25 },
    { 0x1e8787aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce787b1, SUC_lexicon_tags_7 },
    { 0x5aa787b1, SUC_lexicon_tags_33 },
    { 0xd5e787b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770787bc, SUC_lexicon_tags_2 },
    { 0xf82787bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0787c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0787c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x800787c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50787cd, SUC_lexicon_tags_49 },
    { 0x66a787ce, SUC_lexicon_tags_23 },
    { 0x24c787ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a787d4, SUC_lexicon_tags_29 },
    { 0xf0787d5, SUC_lexicon_tags_15 },
    { 0x8c8787d6, SUC_lexicon_tags_57 },
    { 0x288787d6, SUC_lexicon_tags_31 },
    { 0xb0a787d4, SUC_lexicon_tags_37 },
    { 0xda4787d5, SUC_lexicon_tags_66 },
    { 0x2b8787d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe787dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x392787df, SUC_lexicon_tags_11 },
    { 0xd3e787e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00787e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf70787e7, SUC_lexicon_tags_26 },
    { 0x2da787e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72787ed, SUC_lexicon_tags_37 },
    { 0x21a787ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcc787f0, SUC_lexicon_tags_12 },
    { 0x986787f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e787f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c787fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8478802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7880a, SUC_lexicon_tags_37 },
    { 0xeb67880b, SUC_lexicon_tags_0 },
    { 0x12a7880c, SUC_lexicon_tags_15 },
    { 0xb267880d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2278810, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c678815, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe078817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89078819, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27881c, SUC_lexicon_tags_15 },
    { 0x4647881c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x80078820, SUC_lexicon_tags_33 },
    { 0xd8e78821, SUC_lexicon_tags_7 },
    { 0xdfc78822, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d478829, SUC_lexicon_tags_37 },
    { 0x47078829, SUC_lexicon_tags_8 },
    { 0xd947882b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc587882d, SUC_lexicon_tags_7 },
    { 0xb267882d, SUC_lexicon_tags_30 },
    { 0x1ae7882f, SUC_lexicon_tags_11 },
    { 0xbb07882e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf878833, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3678836, SUC_lexicon_tags_63 },
    { 0x3ba78836, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd7278839, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a47883b, SUC_lexicon_tags_92 },
    { 0x43c7883b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac478843, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c78847, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8678850, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xf478852, SUC_lexicon_tags_172 },
    { 0x8a478853, SUC_lexicon_tags_19 },
    { 0x5e878852, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2878858, SUC_lexicon_tags_46 },
    { 0xe3678859, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xca47885c, SUC_lexicon_tags_49 },
    { 0x19c7885c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xef47885f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b678863, SUC_lexicon_tags_46 },
    { 0xdc678864, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5878867, SUC_lexicon_tags_14 },
    { 0xeaa78867, SUC_lexicon_tags_19 },
    { 0x5b078868, SUC_lexicon_tags_1 },
    { 0x3a67886a, SUC_lexicon_tags_7 },
    { 0xef67886b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c7886f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf087887a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1a7887c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4a27887e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe478880, SUC_lexicon_tags_25 },
    { 0xc1278881, SUC_lexicon_tags_77 },
    { 0x45878881, SUC_lexicon_tags_48 },
    { 0xd2e78883, SUC_lexicon_tags_12 },
    { 0x47078881, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c78887, SUC_lexicon_tags_25 },
    { 0x25478888, SUC_lexicon_tags_11 },
    { 0x23e78889, SUC_lexicon_tags_27 },
    { 0xef07888a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7047888e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f078891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c78899, SUC_lexicon_tags_27 },
    { 0xce07889a, SUC_lexicon_tags_1 },
    { 0x2887889b, SUC_lexicon_tags_11 },
    { 0x6207889a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee788a1, SUC_lexicon_tags_27 },
    { 0x8b0788a2, SUC_lexicon_tags_12 },
    { 0x5d2788a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c788a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a788a9, SUC_lexicon_tags_15 },
    { 0x162788a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6788ae, SUC_lexicon_tags_33 },
    { 0x720788ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8788b1, SUC_lexicon_tags_8 },
    { 0xc2c788b2, SUC_lexicon_tags_17 },
    { 0x878788b3, SUC_lexicon_tags_43 },
    { 0xc8788b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x70e788b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x998788b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2788bb, SUC_lexicon_tags_15 },
    { 0xee0788bb, SUC_lexicon_tags_28 },
    { 0xb2a788bd, SUC_lexicon_tags_30 },
    { 0x628788bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae788c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566788c6, SUC_lexicon_tags_21 },
    { 0x7ee788c7, SUC_lexicon_tags_43 },
    { 0x858788c7, SUC_lexicon_tags_23 },
    { 0x742788c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88788cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfe788ce, SUC_lexicon_tags_13 },
    { 0xb10788ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a788d3, SUC_lexicon_tags_38 },
    { 0x252788d3, SUC_lexicon_tags_7 },
    { 0x5d6788d5, SUC_lexicon_tags_15 },
    { 0xe2c788d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba788d8, SUC_lexicon_tags_26 },
    { 0x2cc788d9, SUC_lexicon_tags_37 },
    { 0x99a788da, SUC_lexicon_tags_82 },
    { 0x1fc788db, SUC_lexicon_tags_7 },
    { 0x894788da, SUC_lexicon_tags_15 },
    { 0x3c788d9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2788e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c788e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1fe788e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4788e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e788ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a788ef, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x98788f1, SUC_lexicon_tags_19 },
    { 0xdc8788f2, SUC_lexicon_tags_46 },
    { 0x75e788f3, SUC_lexicon_tags_66 },
    { 0x434788f3, SUC_lexicon_tags_37 },
    { 0x16a788f5, SUC_lexicon_tags_7 },
    { 0x66c788f3, SUC_lexicon_tags_47 },
    { 0xdba788f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e788fa, SUC_lexicon_tags_56 },
    { 0xff8788fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c78907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ec78909, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9c7890b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb387890d, SUC_lexicon_tags_1 },
    { 0x6707890d, SUC_lexicon_tags_4 },
    { 0x5c07890e, SUC_lexicon_tags_3 },
    { 0xb9878910, SUC_lexicon_tags_32 },
    { 0x97278910, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd278913, SUC_lexicon_tags_69 },
    { 0x21878914, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a278919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d27891b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64878924, SUC_lexicon_tags_1 },
    { 0x14478925, SUC_lexicon_tags_47 },
    { 0x31478926, SUC_lexicon_tags_37 },
    { 0x53678926, SUC_lexicon_tags_15 },
    { 0x6c078928, SUC_lexicon_tags_25 },
    { 0xb9478929, SUC_lexicon_tags_137 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a78932, SUC_lexicon_tags_2 },
    { 0x36e78932, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9607893b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa878941, SUC_lexicon_tags_10 },
    { 0x33678942, SUC_lexicon_tags_12 },
    { 0xfa078943, SUC_lexicon_tags_19 },
    { 0x3ca78944, SUC_lexicon_tags_1 },
    { 0x2a478945, SUC_lexicon_tags_33 },
    { 0x26078946, SUC_lexicon_tags_17 },
    { 0x64678944, SUC_lexicon_tags_17 },
    { 0x31678945, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd127894d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7a7894f, SUC_lexicon_tags_17 },
    { 0x2e78950, SUC_lexicon_tags_11 },
    { 0xf078951, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcc78953, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f278956, SUC_lexicon_tags_47 },
    { 0x78c78957, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a678959, SUC_lexicon_tags_28 },
    { 0xf3e7895a, SUC_lexicon_tags_7 },
    { 0xdba7895a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x96a7895d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4427895f, SUC_lexicon_tags_17 },
    { 0x86e78960, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10e78962, SUC_lexicon_tags_25 },
    { 0x6d478962, SUC_lexicon_tags_1 },
    { 0x29e78962, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9a78966, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb678969, SUC_lexicon_tags_11 },
    { 0x1f27896a, SUC_lexicon_tags_138 },
    { 0xc1a7896b, SUC_lexicon_tags_29 },
    { 0xd3e7896c, SUC_lexicon_tags_25 },
    { 0x8867896b, SUC_lexicon_tags_7 },
    { 0xfb47896d, SUC_lexicon_tags_31 },
    { 0xbd67896d, SUC_lexicon_tags_43 },
    { 0xa5c7896e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc9878972, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1427897c, SUC_lexicon_tags_27 },
    { 0x1387897d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55a7897f, SUC_lexicon_tags_37 },
    { 0x9d278980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe7898f, SUC_lexicon_tags_12 },
    { 0x1167898f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdee78992, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c878994, SUC_lexicon_tags_0 },
    { 0x24078995, SUC_lexicon_tags_6 },
    { 0x98478995, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7078999, SUC_lexicon_tags_15 },
    { 0xda478999, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x7c7899c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796789a0, SUC_lexicon_tags_11 },
    { 0xd54789a1, SUC_lexicon_tags_17 },
    { 0x3de789a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1c789a4, SUC_lexicon_tags_45 },
    { 0x11c789a5, SUC_lexicon_tags_27 },
    { 0xa8c789a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834789aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x306789ad, SUC_lexicon_tags_47 },
    { 0xdd0789ae, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4789b2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2f2789b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x712789b7, SUC_lexicon_tags_15 },
    { 0xc8a789b8, SUC_lexicon_tags_65 },
    { 0xae6789b7, SUC_lexicon_tags_27 },
    { 0xb0c789ba, SUC_lexicon_tags_1 },
    { 0x180789b9, SUC_lexicon_tags_15 },
    { 0x384789bc, SUC_lexicon_tags_12 },
    { 0x464789b8, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x314789c0, SUC_lexicon_tags_17 },
    { 0xa74789c1, SUC_lexicon_tags_17 },
    { 0xb2a789c2, SUC_lexicon_tags_91 },
    { 0xa18789c0, SUC_lexicon_tags_12 },
    { 0xb50789c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a789ce, SUC_lexicon_tags_2 },
    { 0x55c789cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4789d2, SUC_lexicon_tags_11 },
    { 0xc74789d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec789d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b2789d8, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x416789db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158789e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a789e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778789e8, SUC_lexicon_tags_7 },
    { 0x7e6789e9, SUC_lexicon_tags_1 },
    { 0x3a0789e9, SUC_lexicon_tags_32 },
    { 0x758789eb, SUC_lexicon_tags_37 },
    { 0x1b0789e9, SUC_lexicon_tags_25 },
    { 0x26789ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0789f0, SUC_lexicon_tags_43 },
    { 0x420789f1, SUC_lexicon_tags_63 },
    { 0x54e789f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x834789f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be789f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x424789fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf38789fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8278a00, SUC_lexicon_tags_33 },
    { 0x91478a01, SUC_lexicon_tags_29 },
    { 0x90e78a02, SUC_lexicon_tags_49 },
    { 0x79e78a01, SUC_lexicon_tags_43 },
    { 0x32478a01, SUC_lexicon_tags_7 },
    { 0x14c78a00, SUC_lexicon_tags_26 },
    { 0xa9878a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35078a0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d478a11, SUC_lexicon_tags_17 },
    { 0x3c678a11, SUC_lexicon_tags_25 },
    { 0x22878a13, SUC_lexicon_tags_38 },
    { 0x5b078a12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65678a17, SUC_lexicon_tags_11 },
    { 0x82078a17, SUC_lexicon_tags_15 },
    { 0x19278a17, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47878a1d, SUC_lexicon_tags_1 },
    { 0x6ee78a1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x10278a21, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae78a24, SUC_lexicon_tags_2 },
    { 0xabc78a25, SUC_lexicon_tags_27 },
    { 0x85678a26, SUC_lexicon_tags_1 },
    { 0xc8078a27, SUC_lexicon_tags_25 },
    { 0x3878a26, SUC_lexicon_tags_10 },
    { 0x9478a29, SUC_lexicon_tags_25 },
    { 0xbf678a2a, SUC_lexicon_tags_7 },
    { 0xb9878a2b, SUC_lexicon_tags_26 },
    { 0xce478a2c, SUC_lexicon_tags_30 },
    { 0x53a78a2d, SUC_lexicon_tags_49 },
    { 0xb8a78a2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7078a30, SUC_lexicon_tags_25 },
    { 0x16278a30, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca78a48, SUC_lexicon_tags_29 },
    { 0x3878a48, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e78a4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99878a51, SUC_lexicon_tags_11 },
    { 0x4a78a52, SUC_lexicon_tags_47 },
    { 0xda678a51, SUC_lexicon_tags_44 },
    { 0x9e278a54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb878a58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6478a5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be78a61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d478a63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3678a68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe278a6c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x95478a6f, SUC_lexicon_tags_7 },
    { 0xb7a78a70, SUC_lexicon_tags_37 },
    { 0x41e78a71, SUC_lexicon_tags_49 },
    { 0xf1678a71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec78a74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae78a79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf278a7e, SUC_lexicon_tags_87 },
    { 0x50278a7f, SUC_lexicon_tags_9 },
    { 0x9278a80, SUC_lexicon_tags_37 },
    { 0x42478a7f, SUC_lexicon_tags_11 },
    { 0xe2078a82, SUC_lexicon_tags_23 },
    { 0x1ba78a83, SUC_lexicon_tags_55 },
    { 0xc9478a84, SUC_lexicon_tags_27 },
    { 0x67678a83, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a78a88, SUC_lexicon_tags_29 },
    { 0x8ea78a88, SUC_lexicon_tags_49 },
    { 0x95678a88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54278a8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0c78a8e, SUC_lexicon_tags_7 },
    { 0x6cc78a8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4478a93, SUC_lexicon_tags_45 },
    { 0xa3878a93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4478a9a, SUC_lexicon_tags_63 },
    { 0x91e78a9b, SUC_lexicon_tags_7 },
    { 0xc4478a9c, SUC_lexicon_tags_1 },
    { 0x50878a9d, SUC_lexicon_tags_7 },
    { 0xdc278a9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e478aa0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x35078aa2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95678aa7, SUC_lexicon_tags_7 },
    { 0xc7a78aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a78aad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0a78aaf, SUC_lexicon_tags_29 },
    { 0xbc278ab0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5478ab2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31478ab5, SUC_lexicon_tags_14 },
    { 0x27078ab6, SUC_lexicon_tags_15 },
    { 0x8ca78ab7, SUC_lexicon_tags_33 },
    { 0x29e78ab8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d678abc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x92878abe, SUC_lexicon_tags_47 },
    { 0x91278abf, SUC_lexicon_tags_15 },
    { 0x72678ac0, SUC_lexicon_tags_15 },
    { 0x2aa78ac1, SUC_lexicon_tags_15 },
    { 0x21278abe, SUC_lexicon_tags_37 },
    { 0x9878ac3, SUC_lexicon_tags_27 },
    { 0x4e478abf, SUC_lexicon_tags_17 },
    { 0x31a78ac0, SUC_lexicon_tags_25 },
    { 0xdce78abf, SUC_lexicon_tags_30 },
    { 0x5ec78ac3, SUC_lexicon_tags_62 },
    { 0xc1c78ac8, SUC_lexicon_tags_27 },
    { 0x2c478ac9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc278ace, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x97478ad1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2478ad6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2678ad9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f278add, SUC_lexicon_tags_1 },
    { 0x1ea78add, SUC_lexicon_tags_65 },
    { 0x7d078adf, SUC_lexicon_tags_69 },
    { 0x7e878add, SUC_lexicon_tags_15 },
    { 0x9278ae1, SUC_lexicon_tags_33 },
    { 0xa9878ae2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8678ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd478aea, SUC_lexicon_tags_25 },
    { 0x88278aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63a78aed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0078af3, SUC_lexicon_tags_37 },
    { 0x1e878af4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72678af6, SUC_lexicon_tags_37 },
    { 0x15078af7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa078afe, SUC_lexicon_tags_17 },
    { 0x52878afe, SUC_lexicon_tags_1 },
    { 0x1f278b00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x75278b03, SUC_lexicon_tags_15 },
    { 0x8b678b04, SUC_lexicon_tags_7 },
    { 0xef078b05, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c78b0a, SUC_lexicon_tags_25 },
    { 0x8678b0b, SUC_lexicon_tags_31 },
    { 0xafa78b0c, SUC_lexicon_tags_12 },
    { 0x96e78b0b, SUC_lexicon_tags_33 },
    { 0xa278b0c, SUC_lexicon_tags_25 },
    { 0xfda78b0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e78b14, SUC_lexicon_tags_25 },
    { 0x53e78b15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22078b17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb478b1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0878b25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe678b2a, SUC_lexicon_tags_6 },
    { 0xb4478b2b, SUC_lexicon_tags_25 },
    { 0x19a78b2a, SUC_lexicon_tags_17 },
    { 0xf4678b2d, SUC_lexicon_tags_23 },
    { 0x85278b2b, SUC_lexicon_tags_49 },
    { 0xf7078b2d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdbe78b31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x24078b34, SUC_lexicon_tags_17 },
    { 0x2c278b35, SUC_lexicon_tags_17 },
    { 0xc1e78b35, SUC_lexicon_tags_11 },
    { 0x6ca78b37, SUC_lexicon_tags_12 },
    { 0x98a78b36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76278b40, SUC_lexicon_tags_1 },
    { 0x8d878b40, SUC_lexicon_tags_11 },
    { 0x33078b40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c78b49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81878b4b, SUC_lexicon_tags_47 },
    { 0x51878b4c, SUC_lexicon_tags_19 },
    { 0xaa878b4d, SUC_lexicon_tags_38 },
    { 0x64478b4e, SUC_lexicon_tags_8 },
    { 0xee278b4f, SUC_lexicon_tags_13 },
    { 0x9f878b4d, SUC_lexicon_tags_11 },
    { 0xc3678b51, SUC_lexicon_tags_1 },
    { 0x55e78b4d, SUC_lexicon_tags_9 },
    { 0xa0678b53, SUC_lexicon_tags_12 },
    { 0xaca78b51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c78b57, SUC_lexicon_tags_37 },
    { 0x2a278b57, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x32478b5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c678b5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8c678b5e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe78b69, SUC_lexicon_tags_165 },
    { 0xa1e78b6a, SUC_lexicon_tags_11 },
    { 0xd7078b6b, SUC_lexicon_tags_43 },
    { 0xfc878b6c, SUC_lexicon_tags_45 },
    { 0xe2878b6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98878b70, SUC_lexicon_tags_14 },
    { 0x15478b71, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x17478b73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ca78b75, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xbee78b77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98078b7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee78b83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdba78b85, SUC_lexicon_tags_14 },
    { 0xec078b85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x12878b89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da78b8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2a78b8e, SUC_lexicon_tags_4 },
    { 0x17c78b8f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e78b93, SUC_lexicon_tags_12 },
    { 0xe78b93, SUC_lexicon_tags_31 },
    { 0x12a78b95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e78b9a, SUC_lexicon_tags_11 },
    { 0x7fe78b9b, SUC_lexicon_tags_25 },
    { 0x7a878b9a, SUC_lexicon_tags_25 },
    { 0x1b078b9a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde78ba3, SUC_lexicon_tags_14 },
    { 0x4b878ba3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68278ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5a78baa, SUC_lexicon_tags_47 },
    { 0xb3078bab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca478bad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91878baf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c78bb2, SUC_lexicon_tags_37 },
    { 0x61a78bb3, SUC_lexicon_tags_31 },
    { 0x5e878bb2, SUC_lexicon_tags_37 },
    { 0xece78bb2, SUC_lexicon_tags_34 },
    { 0x4ba78bb3, SUC_lexicon_tags_30 },
    { 0x14c78bb3, SUC_lexicon_tags_151 },
    { 0x7cc78bb5, SUC_lexicon_tags_2 },
    { 0x18a78bb8, SUC_lexicon_tags_31 },
    { 0x78478bb5, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0x5de78bbc, SUC_lexicon_tags_25 },
    { 0x64c78bbc, SUC_lexicon_tags_2 },
    { 0x16c78bbe, SUC_lexicon_tags_33 },
    { 0xaf478bbf, SUC_lexicon_tags_1 },
    { 0x6478bc0, SUC_lexicon_tags_25 },
    { 0x3f278bc1, SUC_lexicon_tags_15 },
    { 0xf0a78bc2, SUC_lexicon_tags_92 },
    { 0x82278bc3, SUC_lexicon_tags_15 },
    { 0xed478bbe, SUC_lexicon_tags_11 },
    { 0x34478bc5, SUC_lexicon_tags_14 },
    { 0x19478bc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdda78bc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57c78bca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a78bce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a478bd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb678bd5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e78bd9, SUC_lexicon_tags_17 },
    { 0x41e78bda, SUC_lexicon_tags_160 },
    { 0xb1678bdb, SUC_lexicon_tags_25 },
    { 0xb5c78bda, SUC_lexicon_tags_7 },
    { 0x64a78bdd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1278be3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5878be5, SUC_lexicon_tags_7 },
    { 0xd7c78be6, SUC_lexicon_tags_15 },
    { 0xd0e78be7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e278be9, SUC_lexicon_tags_68 },
    { 0xdc278be9, SUC_lexicon_tags_17 },
    { 0x2da78beb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9878bee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9878bf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e78bf4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1478bf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25078bfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6878bff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6678c09, SUC_lexicon_tags_11 },
    { 0x17278c0a, SUC_lexicon_tags_17 },
    { 0xb1478c0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd278c12, SUC_lexicon_tags_17 },
    { 0x33a78c12, SUC_lexicon_tags_44 },
    { 0x1f678c12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e478c16, SUC_lexicon_tags_37 },
    { 0x98678c16, SUC_lexicon_tags_14 },
    { 0x59278c18, SUC_lexicon_tags_0 },
    { 0x80678c16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4478c1b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa5678c1d, SUC_lexicon_tags_2 },
    { 0xd4e78c1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa678c20, SUC_lexicon_tags_15 },
    { 0xcfa78c20, SUC_lexicon_tags_0 },
    { 0x85678c20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1078c26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84878c28, SUC_lexicon_tags_47 },
    { 0xbc78c29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa878c2b, SUC_lexicon_tags_63 },
    { 0x13e78c2b, SUC_lexicon_tags_31 },
    { 0x22478c2d, SUC_lexicon_tags_39 },
    { 0xaa678c2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1da78c30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91278c32, SUC_lexicon_tags_32 },
    { 0x50478c33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c078c35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7278c39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe1e78c3b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a78c3e, SUC_lexicon_tags_7 },
    { 0x2b878c3f, SUC_lexicon_tags_30 },
    { 0xf3878c3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x39678c42, SUC_lexicon_tags_38 },
    { 0x1ac78c42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49478c46, SUC_lexicon_tags_16 },
    { 0xc6c78c47, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd8078c49, SUC_lexicon_tags_1 },
    { 0x7c878c4a, SUC_lexicon_tags_37 },
    { 0xf4678c49, SUC_lexicon_tags_11 },
    { 0x47a78c4c, SUC_lexicon_tags_7 },
    { 0x59478c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a78c51, SUC_lexicon_tags_45 },
    { 0x94c78c52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6278c5a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9278c60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a78c63, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc878c69, SUC_lexicon_tags_45 },
    { 0x3678c69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf278c6d, SUC_lexicon_tags_87 },
    { 0xbca78c6d, SUC_lexicon_tags_53 },
    { 0xe3a78c6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55278c72, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa78c75, SUC_lexicon_tags_17 },
    { 0x40c78c75, SUC_lexicon_tags_17 },
    { 0x3ae78c77, SUC_lexicon_tags_11 },
    { 0xf6078c78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfa78c7a, SUC_lexicon_tags_27 },
    { 0x23278c7a, SUC_lexicon_tags_11 },
    { 0xf6c78c7c, SUC_lexicon_tags_12 },
    { 0x99c78c7d, SUC_lexicon_tags_48 },
    { 0x61478c7e, SUC_lexicon_tags_61 },
    { 0x2d678c7f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa78c87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1878c8b, SUC_lexicon_tags_54 },
    { 0xb0a78c8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc78c97, SUC_lexicon_tags_2 },
    { 0x90e78c98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1bc78c9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd878c9c, SUC_lexicon_tags_8 },
    { 0xea678c9c, SUC_lexicon_tags_17 },
    { 0xd3a78c9c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa78ca1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x39278ca4, SUC_lexicon_tags_6 },
    { 0xfc278ca5, SUC_lexicon_tags_23 },
    { 0xf7a78ca4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6678ca8, SUC_lexicon_tags_17 },
    { 0xb4478ca9, SUC_lexicon_tags_351 },
    { 0xbc078caa, SUC_lexicon_tags_25 },
    { 0x90678ca8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab878cad, SUC_lexicon_tags_23 },
    { 0x71478cae, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xafe78cb0, SUC_lexicon_tags_11 },
    { 0xa0a78cb1, SUC_lexicon_tags_68 },
    { 0xc6c78cb2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x81278cb4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcec78cb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc78cba, SUC_lexicon_tags_5 },
    { 0x71678cbb, SUC_lexicon_tags_12 },
    { 0xca078cba, SUC_lexicon_tags_12 },
    { 0x9c678cba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc3a78cbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58278cc1, SUC_lexicon_tags_12 },
    { 0x6a278cc2, SUC_lexicon_tags_12 },
    { 0xf7878cc3, SUC_lexicon_tags_37 },
    { 0xbc78cc4, SUC_lexicon_tags_26 },
    { 0xea78cc3, SUC_lexicon_tags_2 },
    { 0x86478cc2, SUC_lexicon_tags_8 },
    { 0x7a878cc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec78ccb, SUC_lexicon_tags_4 },
    { 0x2de78ccb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x29078cce, SUC_lexicon_tags_15 },
    { 0xd6a78ccf, SUC_lexicon_tags_60 },
    { 0x61478ccf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb078cd5, SUC_lexicon_tags_17 },
    { 0x8a878cd6, SUC_lexicon_tags_1 },
    { 0x50e78cd5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xefc78cd9, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c78cde, SUC_lexicon_tags_14 },
    { 0xe6278cdf, SUC_lexicon_tags_0 },
    { 0x4b478cdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba078ce4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21078ce6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd3c78ce8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c478cec, SUC_lexicon_tags_8 },
    { 0x61678ced, SUC_lexicon_tags_37 },
    { 0xcb678cee, SUC_lexicon_tags_11 },
    { 0xfc278cef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x36678cf2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a78cf6, SUC_lexicon_tags_32 },
    { 0x82c78cf6, SUC_lexicon_tags_17 },
    { 0x9a878cf8, SUC_lexicon_tags_109 },
    { 0x1a678cf9, SUC_lexicon_tags_23 },
    { 0x65a78cfa, SUC_lexicon_tags_57 },
    { 0x33478cfb, SUC_lexicon_tags_12 },
    { 0xc2678cfc, SUC_lexicon_tags_7 },
    { 0x13278cf9, SUC_lexicon_tags_37 },
    { 0xcfc78cfe, SUC_lexicon_tags_17 },
    { 0x8ce78cfe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4078d02, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8678d05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7b078d07, SUC_lexicon_tags_149 },
    { 0x1a278d08, SUC_lexicon_tags_23 },
    { 0x9b878d09, SUC_lexicon_tags_7 },
    { 0x92878d09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4878d0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x55478d0f, SUC_lexicon_tags_19 },
    { 0x3e078d10, SUC_lexicon_tags_7 },
    { 0x9ba78d11, SUC_lexicon_tags_70 },
    { 0x81c78d10, SUC_lexicon_tags_15 },
    { 0x60e78d13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a478d20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c78d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4678d31, SUC_lexicon_tags_12 },
    { 0x19478d32, SUC_lexicon_tags_10 },
    { 0x4b878d33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd878d38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefc78d3a, SUC_lexicon_tags_27 },
    { 0xa4478d3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e278d41, SUC_lexicon_tags_17 },
    { 0x37278d42, SUC_lexicon_tags_60 },
    { 0x37278d43, SUC_lexicon_tags_19 },
    { 0x81c78d44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2078d47, SUC_lexicon_tags_14 },
    { 0x9278d48, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x77c78d4a, SUC_lexicon_tags_2 },
    { 0x80878d4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c478d4d, SUC_lexicon_tags_46 },
    { 0xd1278d4d, SUC_lexicon_tags_14 },
    { 0xb0c78d4f, SUC_lexicon_tags_2 },
    { 0xff478d4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57278d52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e078d55, SUC_lexicon_tags_6 },
    { 0x78c78d56, SUC_lexicon_tags_37 },
    { 0x47a78d57, SUC_lexicon_tags_25 },
    { 0x8c478d57, SUC_lexicon_tags_104 },
    { 0xea078d58, SUC_lexicon_tags_33 },
    { 0xd9c78d58, SUC_lexicon_tags_37 },
    { 0x24a78d5a, SUC_lexicon_tags_47 },
    { 0x24e78d5c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8878d63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd878d65, SUC_lexicon_tags_376 },
    { 0xba78d65, SUC_lexicon_tags_62 },
    { 0x6aa78d67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84278d6a, SUC_lexicon_tags_40 },
    { 0x1278d6b, SUC_lexicon_tags_1 },
    { 0x2e878d6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6ba78d6e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e78d71, SUC_lexicon_tags_17 },
    { 0xe6a78d71, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92078d77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5278d79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc678d7e, SUC_lexicon_tags_31 },
    { 0x35078d7f, SUC_lexicon_tags_13 },
    { 0x80878d7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe78d88, SUC_lexicon_tags_15 },
    { 0xbd478d88, SUC_lexicon_tags_17 },
    { 0x7ca78d8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28678d8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3878d94, SUC_lexicon_tags_37 },
    { 0x68e78d94, SUC_lexicon_tags_11 },
    { 0x3ca78d94, SUC_lexicon_tags_10 },
    { 0xb8e78d94, SUC_lexicon_tags_27 },
    { 0x68678d94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78d9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a878da0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d478da7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d278da9, SUC_lexicon_tags_60 },
    { 0xe0678daa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7dc78dac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4678dae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede78db5, SUC_lexicon_tags_37 },
    { 0xd878db5, SUC_lexicon_tags_14 },
    { 0xee678db7, SUC_lexicon_tags_37 },
    { 0xfac78db8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c478dba, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e78dc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e78dc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7878dcd, SUC_lexicon_tags_37 },
    { 0xd5278dcd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1078dd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9878dd4, SUC_lexicon_tags_11 },
    { 0xe4478dd5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x27278dd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2878dd9, SUC_lexicon_tags_17 },
    { 0x5a878dda, SUC_lexicon_tags_2 },
    { 0x92c78ddb, SUC_lexicon_tags_2 },
    { 0x37078ddb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9b878dde, SUC_lexicon_tags_14 },
    { 0xa4478dde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e78de6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3078de9, SUC_lexicon_tags_27 },
    { 0xb5078de9, SUC_lexicon_tags_37 },
    { 0xe3678deb, SUC_lexicon_tags_14 },
    { 0xcc678deb, SUC_lexicon_tags_15 },
    { 0xad278dec, SUC_lexicon_tags_2 },
    { 0xf9478ded, SUC_lexicon_tags_11 },
    { 0x6ae78def, SUC_lexicon_tags_27 },
    { 0x17e78dea, SUC_lexicon_tags_8 },
    { 0x28c78def, SUC_lexicon_tags_65 },
    { 0xa478df2, SUC_lexicon_tags_37 },
    { 0x9a78df2, SUC_lexicon_tags_12 },
    { 0x2de78df4, SUC_lexicon_tags_9 },
    { 0x15278df2, SUC_lexicon_tags_25 },
    { 0xeee78df4, SUC_lexicon_tags_45 },
    { 0xd4878df7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95278df9, SUC_lexicon_tags_1 },
    { 0x62e78dfa, SUC_lexicon_tags_25 },
    { 0xf7078dfb, SUC_lexicon_tags_0 },
    { 0x29478dfc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e78dff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c78e03, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c278e08, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x77878e0a, SUC_lexicon_tags_8 },
    { 0x89078e0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3678e0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8be78e10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be78e13, SUC_lexicon_tags_49 },
    { 0xe4678e13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba278e1c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xad278e1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5078e24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d478e29, SUC_lexicon_tags_17 },
    { 0xc078e2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xba478e2d, SUC_lexicon_tags_869 },
    { 0, NULL },
    { 0xdc478e2f, SUC_lexicon_tags_37 },
    { 0x30878e2f, SUC_lexicon_tags_45 },
    { 0xe9678e31, SUC_lexicon_tags_32 },
    { 0x61478e2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e78e3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5e678e3c, SUC_lexicon_tags_8 },
    { 0x8ce78e3c, SUC_lexicon_tags_11 },
    { 0xd1c78e3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c78e45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44878e4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31478e51, SUC_lexicon_tags_17 },
    { 0x16c78e52, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72678e56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x77c78e58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3878e5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b878e63, SUC_lexicon_tags_7 },
    { 0x10278e64, SUC_lexicon_tags_25 },
    { 0xf8878e65, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfd478e67, SUC_lexicon_tags_15 },
    { 0x8bc78e68, SUC_lexicon_tags_62 },
    { 0xd8078e67, SUC_lexicon_tags_37 },
    { 0xea278e6a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc278e6d, SUC_lexicon_tags_15 },
    { 0x75678e6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6478e70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7078e73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x38878e75, SUC_lexicon_tags_14 },
    { 0x3fa78e75, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93078e7d, SUC_lexicon_tags_25 },
    { 0x9a478e7e, SUC_lexicon_tags_3 },
    { 0xfea78e7f, SUC_lexicon_tags_27 },
    { 0x9d678e80, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa078e82, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a78e8a, SUC_lexicon_tags_15 },
    { 0x45478e8b, SUC_lexicon_tags_2 },
    { 0x4478e8b, SUC_lexicon_tags_37 },
    { 0xb2e78e8d, SUC_lexicon_tags_11 },
    { 0x6f878e8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e78e91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e678e96, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c78e9b, SUC_lexicon_tags_91 },
    { 0xfc78e9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b278ea1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0478ea3, SUC_lexicon_tags_25 },
    { 0xa3678ea3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xba278ea6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4678ea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaea78eaa, SUC_lexicon_tags_77 },
    { 0xd3478eaa, SUC_lexicon_tags_15 },
    { 0xa878eac, SUC_lexicon_tags_0 },
    { 0xdba78eab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae078eaf, SUC_lexicon_tags_35 },
    { 0x8e478eb0, SUC_lexicon_tags_45 },
    { 0xfde78eb1, SUC_lexicon_tags_60 },
    { 0x57c78eb2, SUC_lexicon_tags_178 },
    { 0xd5078eb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8c78eb5, SUC_lexicon_tags_28 },
    { 0xad278eb6, SUC_lexicon_tags_85 },
    { 0xafa78eb6, SUC_lexicon_tags_47 },
    { 0x91c78eb7, SUC_lexicon_tags_15 },
    { 0x6ac78eb9, SUC_lexicon_tags_43 },
    { 0xc1278eb8, SUC_lexicon_tags_27 },
    { 0x22a78ebb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39c78ebd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57478ec3, SUC_lexicon_tags_7 },
    { 0xb0c78ec4, SUC_lexicon_tags_18 },
    { 0x7be78ec5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb078ec7, SUC_lexicon_tags_43 },
    { 0xea678ec8, SUC_lexicon_tags_11 },
    { 0xeb278ec8, SUC_lexicon_tags_140 },
    { 0x32478ec7, SUC_lexicon_tags_6 },
    { 0x56878ecb, SUC_lexicon_tags_29 },
    { 0xc2c78ecb, SUC_lexicon_tags_17 },
    { 0xa7e78ecc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9878ed3, SUC_lexicon_tags_47 },
    { 0xaf878ed3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6678ed9, SUC_lexicon_tags_27 },
    { 0xa1278ed9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x58078edd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xef278edf, SUC_lexicon_tags_43 },
    { 0x98c78edf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb478ee2, SUC_lexicon_tags_7 },
    { 0xcf278ee2, SUC_lexicon_tags_2 },
    { 0x8478ee4, SUC_lexicon_tags_15 },
    { 0x8e078ee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae878ee7, SUC_lexicon_tags_23 },
    { 0x3a278ee7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b478eeb, SUC_lexicon_tags_25 },
    { 0x1c278eeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55c78eee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x13878ef0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x32478ef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6da78ef4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53678ef7, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0078efa, SUC_lexicon_tags_11 },
    { 0xa7078efa, SUC_lexicon_tags_32 },
    { 0x57478efc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e78eff, SUC_lexicon_tags_0 },
    { 0x45878f00, SUC_lexicon_tags_23 },
    { 0x51078f00, SUC_lexicon_tags_7 },
    { 0x2a478eff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78c78f04, SUC_lexicon_tags_17 },
    { 0x8d678f05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a78f0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1278f14, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a78f18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a78f1c, SUC_lexicon_tags_12 },
    { 0xfde78f1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3678f21, SUC_lexicon_tags_11 },
    { 0xb078f21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f278f24, SUC_lexicon_tags_1 },
    { 0x31078f24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x55278f28, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28478f2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61e78f2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8c78f31, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c78f36, SUC_lexicon_tags_60 },
    { 0xde78f36, SUC_lexicon_tags_9 },
    { 0xfa278f38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20878f3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8078f41, SUC_lexicon_tags_15 },
    { 0x9e278f41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7878f44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba478f49, SUC_lexicon_tags_28 },
    { 0x40678f49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96878f4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc78f50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1878f57, SUC_lexicon_tags_12 },
    { 0x53678f58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1478f5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64478f60, SUC_lexicon_tags_12 },
    { 0xa078f60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e78f64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92478f68, SUC_lexicon_tags_1 },
    { 0x72278f68, SUC_lexicon_tags_29 },
    { 0xc7678f6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc1078f6c, SUC_lexicon_tags_68 },
    { 0x19a78f6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84678f6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x59878f72, SUC_lexicon_tags_45 },
    { 0x83078f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ac78f75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5e78f77, SUC_lexicon_tags_64 },
    { 0xd4a78f78, SUC_lexicon_tags_15 },
    { 0x47a78f79, SUC_lexicon_tags_8 },
    { 0xe3c78f7a, SUC_lexicon_tags_21 },
    { 0x93878f7b, SUC_lexicon_tags_37 },
    { 0xb4878f7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e78f7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e78f83, SUC_lexicon_tags_12 },
    { 0xade78f83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd7a78f86, SUC_lexicon_tags_16 },
    { 0x69078f86, SUC_lexicon_tags_4 },
    { 0x77a78f88, SUC_lexicon_tags_7 },
    { 0xa5878f88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x65478f8b, SUC_lexicon_tags_1 },
    { 0x17078f8c, SUC_lexicon_tags_17 },
    { 0xeb678f8d, SUC_lexicon_tags_17 },
    { 0xca478f8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea278f95, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d078f99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee78f9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0478f9e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a78fa3, SUC_lexicon_tags_31 },
    { 0x5d078fa4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x27a78fa6, SUC_lexicon_tags_124 },
    { 0xa9278fa6, SUC_lexicon_tags_15 },
    { 0x6b478fa8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb5678faa, SUC_lexicon_tags_17 },
    { 0x95078faa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x12078fad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e78fb0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4478fb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ae78fb6, SUC_lexicon_tags_26 },
    { 0x69878fb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x86478fba, SUC_lexicon_tags_9 },
    { 0x9ac78fbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77278fc0, SUC_lexicon_tags_1 },
    { 0x13478fc1, SUC_lexicon_tags_27 },
    { 0x44e78fc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6078fc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3478fd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14478fd4, SUC_lexicon_tags_2 },
    { 0xe4478fd4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5878fd8, SUC_lexicon_tags_14 },
    { 0x3aa78fd8, SUC_lexicon_tags_15 },
    { 0x35678fda, SUC_lexicon_tags_37 },
    { 0x2c78fda, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c678fe2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x89878fe4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5478fea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6a78fec, SUC_lexicon_tags_60 },
    { 0x57c78fed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54a78fef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74078ff4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x42878ff6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62078ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0079002, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d879006, SUC_lexicon_tags_25 },
    { 0xef479006, SUC_lexicon_tags_684 },
    { 0xa7279008, SUC_lexicon_tags_43 },
    { 0x53479008, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d47900b, SUC_lexicon_tags_37 },
    { 0x3aa7900c, SUC_lexicon_tags_11 },
    { 0x3d47900d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62479010, SUC_lexicon_tags_62 },
    { 0xd3e79011, SUC_lexicon_tags_17 },
    { 0x82c79012, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c79015, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e79019, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e79020, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3079023, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6627902b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xf147902e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c079030, SUC_lexicon_tags_1 },
    { 0xee79030, SUC_lexicon_tags_1 },
    { 0xc0679030, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3b879034, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6479038, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e7903c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa907903f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32479041, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c79048, SUC_lexicon_tags_45 },
    { 0x9f479048, SUC_lexicon_tags_25 },
    { 0x9ac7904a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x367904c, SUC_lexicon_tags_47 },
    { 0x2507904c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c79050, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec479057, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xac07905a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd487905c, SUC_lexicon_tags_27 },
    { 0x5087905c, SUC_lexicon_tags_45 },
    { 0x5b87905d, SUC_lexicon_tags_11 },
    { 0x17c7905f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9079061, SUC_lexicon_tags_140 },
    { 0xea879062, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x21279064, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6927906a, SUC_lexicon_tags_16 },
    { 0xae47906b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e7906e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c79071, SUC_lexicon_tags_68 },
    { 0x53c79071, SUC_lexicon_tags_5 },
    { 0x3f079073, SUC_lexicon_tags_127 },
    { 0x62479074, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c279076, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc827907a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc7907e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac79088, SUC_lexicon_tags_37 },
    { 0x6d479089, SUC_lexicon_tags_2 },
    { 0x3cc7908a, SUC_lexicon_tags_25 },
    { 0x3fa7908a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee7908e, SUC_lexicon_tags_25 },
    { 0x47c7908f, SUC_lexicon_tags_15 },
    { 0x3fc7908e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d679092, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc79096, SUC_lexicon_tags_39 },
    { 0xa0e79096, SUC_lexicon_tags_84 },
    { 0x27c79098, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc7909f, SUC_lexicon_tags_17 },
    { 0xa42790a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c790a5, SUC_lexicon_tags_15 },
    { 0x8e8790a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82790a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be790ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e2790b0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc790b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140790b9, SUC_lexicon_tags_17 },
    { 0x406790b9, SUC_lexicon_tags_37 },
    { 0x72790bb, SUC_lexicon_tags_11 },
    { 0xf8a790bb, SUC_lexicon_tags_37 },
    { 0x13c790b9, SUC_lexicon_tags_63 },
    { 0x146790be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2ec790c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a790c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4790c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x366790cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00790cf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe4790d1, SUC_lexicon_tags_15 },
    { 0x4d4790d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2790d5, SUC_lexicon_tags_37 },
    { 0x7c790d6, SUC_lexicon_tags_32 },
    { 0xb5a790d7, SUC_lexicon_tags_31 },
    { 0x1e0790d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x544790db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab4790dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20790e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20790ea, SUC_lexicon_tags_23 },
    { 0xc7e790eb, SUC_lexicon_tags_19 },
    { 0xf04790ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fc790ee, SUC_lexicon_tags_47 },
    { 0xfa4790ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb88790f1, SUC_lexicon_tags_23 },
    { 0x410790f1, SUC_lexicon_tags_31 },
    { 0x2da790f3, SUC_lexicon_tags_15 },
    { 0xe00790f4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20790f8, SUC_lexicon_tags_17 },
    { 0xb46790f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76790fd, SUC_lexicon_tags_65 },
    { 0x390790fe, SUC_lexicon_tags_11 },
    { 0x546790ff, SUC_lexicon_tags_55 },
    { 0xa1479100, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x46a79102, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa4879104, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4947910a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f47910c, SUC_lexicon_tags_26 },
    { 0xece7910d, SUC_lexicon_tags_37 },
    { 0x3667910c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d079112, SUC_lexicon_tags_33 },
    { 0xf7e79112, SUC_lexicon_tags_27 },
    { 0x45079114, SUC_lexicon_tags_38 },
    { 0x40279113, SUC_lexicon_tags_8 },
    { 0x3a079116, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6887911a, SUC_lexicon_tags_0 },
    { 0xcba7911b, SUC_lexicon_tags_68 },
    { 0x1f67911b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x70e7911e, SUC_lexicon_tags_47 },
    { 0xa747911f, SUC_lexicon_tags_12 },
    { 0x2ea79120, SUC_lexicon_tags_47 },
    { 0x94e79121, SUC_lexicon_tags_0 },
    { 0xf7879122, SUC_lexicon_tags_9 },
    { 0x7f07911f, SUC_lexicon_tags_47 },
    { 0xb9679124, SUC_lexicon_tags_33 },
    { 0x5a79125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b87912b, SUC_lexicon_tags_7 },
    { 0x6ee7912b, SUC_lexicon_tags_29 },
    { 0xe07912d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e79130, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5679133, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7879135, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x87c79137, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86879142, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c79147, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9a079149, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xda47914b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdbc7914d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c79159, SUC_lexicon_tags_26 },
    { 0xd0079159, SUC_lexicon_tags_25 },
    { 0x3267915a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b87915e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e79162, SUC_lexicon_tags_37 },
    { 0x17479163, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc479166, SUC_lexicon_tags_1 },
    { 0x76c79166, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd279169, SUC_lexicon_tags_37 },
    { 0x66279169, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92279170, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83879172, SUC_lexicon_tags_7 },
    { 0x59279172, SUC_lexicon_tags_0 },
    { 0xb0079173, SUC_lexicon_tags_37 },
    { 0x67679175, SUC_lexicon_tags_43 },
    { 0x9fa79173, SUC_lexicon_tags_29 },
    { 0xee879173, SUC_lexicon_tags_12 },
    { 0xdde79174, SUC_lexicon_tags_87 },
    { 0xf0279179, SUC_lexicon_tags_439 },
    { 0xe4e7917a, SUC_lexicon_tags_24 },
    { 0x1747917b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xb247917e, SUC_lexicon_tags_10 },
    { 0x5d27917e, SUC_lexicon_tags_2 },
    { 0x52079180, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34879183, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e79189, SUC_lexicon_tags_39 },
    { 0x2367918a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadc7918c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c7918f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x72e79191, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfea79193, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb5a79195, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e87919a, SUC_lexicon_tags_64 },
    { 0xb187919a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c27919e, SUC_lexicon_tags_15 },
    { 0x2047919f, SUC_lexicon_tags_8 },
    { 0x2b6791a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x47e791a2, SUC_lexicon_tags_2 },
    { 0x8e4791a3, SUC_lexicon_tags_11 },
    { 0xff8791a3, SUC_lexicon_tags_25 },
    { 0x94c791a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd70791a7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xab0791a9, SUC_lexicon_tags_1 },
    { 0x57a791aa, SUC_lexicon_tags_63 },
    { 0x11a791aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x350791ae, SUC_lexicon_tags_19 },
    { 0x90e791af, SUC_lexicon_tags_21 },
    { 0x4e8791af, SUC_lexicon_tags_1 },
    { 0xaa4791b1, SUC_lexicon_tags_14 },
    { 0xf2a791b2, SUC_lexicon_tags_1 },
    { 0xa1a791b2, SUC_lexicon_tags_63 },
    { 0xb64791b3, SUC_lexicon_tags_25 },
    { 0x2a4791af, SUC_lexicon_tags_37 },
    { 0xde8791af, SUC_lexicon_tags_25 },
    { 0x7dc791b7, SUC_lexicon_tags_12 },
    { 0x9c8791b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8791bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150791c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38791c5, SUC_lexicon_tags_47 },
    { 0x290791c5, SUC_lexicon_tags_7 },
    { 0xc0a791c7, SUC_lexicon_tags_15 },
    { 0x710791c5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x186791cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2791d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7de791d5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdf6791d7, SUC_lexicon_tags_11 },
    { 0x894791d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c791dc, SUC_lexicon_tags_37 },
    { 0xc6e791dd, SUC_lexicon_tags_33 },
    { 0x28a791de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc791e3, SUC_lexicon_tags_40 },
    { 0x66e791e3, SUC_lexicon_tags_37 },
    { 0x4f2791e5, SUC_lexicon_tags_37 },
    { 0xb96791e5, SUC_lexicon_tags_43 },
    { 0xd56791e7, SUC_lexicon_tags_49 },
    { 0x436791e8, SUC_lexicon_tags_37 },
    { 0x486791e4, SUC_lexicon_tags_23 },
    { 0x7f2791e8, SUC_lexicon_tags_2 },
    { 0xbe791e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e791f5, SUC_lexicon_tags_101 },
    { 0x9bc791f6, SUC_lexicon_tags_1 },
    { 0xe9e791f7, SUC_lexicon_tags_17 },
    { 0x3d4791f8, SUC_lexicon_tags_17 },
    { 0xb54791f9, SUC_lexicon_tags_26 },
    { 0x52791fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b479201, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c679203, SUC_lexicon_tags_33 },
    { 0x42679204, SUC_lexicon_tags_30 },
    { 0x7e879205, SUC_lexicon_tags_37 },
    { 0x14c79205, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf827920e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b879213, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31279218, SUC_lexicon_tags_63 },
    { 0xdbe79218, SUC_lexicon_tags_7 },
    { 0x95679219, SUC_lexicon_tags_6 },
    { 0xbd279219, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7921f, SUC_lexicon_tags_27 },
    { 0x93a7921f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a479226, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac87922a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e7922f, SUC_lexicon_tags_61 },
    { 0xaa7922f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a79235, SUC_lexicon_tags_7 },
    { 0x7a679236, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67923a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xda47923c, SUC_lexicon_tags_13 },
    { 0x5a87923d, SUC_lexicon_tags_37 },
    { 0x8707923e, SUC_lexicon_tags_25 },
    { 0x3507923f, SUC_lexicon_tags_48 },
    { 0xe1e79240, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867924b, SUC_lexicon_tags_25 },
    { 0xc567924c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30a7924e, SUC_lexicon_tags_2 },
    { 0xa747924f, SUC_lexicon_tags_37 },
    { 0x4d67924e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd479253, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a79256, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xed279258, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c7925d, SUC_lexicon_tags_12 },
    { 0xb627925e, SUC_lexicon_tags_29 },
    { 0xd447925d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e279261, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e79264, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b879266, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d67926e, SUC_lexicon_tags_17 },
    { 0x9bc7926e, SUC_lexicon_tags_35 },
    { 0x5c279270, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31279275, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9a79277, SUC_lexicon_tags_17 },
    { 0xade79278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e7927b, SUC_lexicon_tags_63 },
    { 0x52a7927b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b079287, SUC_lexicon_tags_1 },
    { 0xbc679287, SUC_lexicon_tags_14 },
    { 0x34a79289, SUC_lexicon_tags_17 },
    { 0x1f47928a, SUC_lexicon_tags_84 },
    { 0x79e7928b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x97c7928d, SUC_lexicon_tags_1 },
    { 0xc8c7928e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16279295, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a79299, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a07929f, SUC_lexicon_tags_15 },
    { 0x9aa7929f, SUC_lexicon_tags_174 },
    { 0x78c792a1, SUC_lexicon_tags_0 },
    { 0xac0792a2, SUC_lexicon_tags_2 },
    { 0xc4e792a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84792a7, SUC_lexicon_tags_25 },
    { 0xc5a792a8, SUC_lexicon_tags_6 },
    { 0x6a2792a9, SUC_lexicon_tags_17 },
    { 0xf5e792a9, SUC_lexicon_tags_49 },
    { 0xcda792ab, SUC_lexicon_tags_23 },
    { 0xfd8792ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e792b1, SUC_lexicon_tags_4 },
    { 0x272792b2, SUC_lexicon_tags_11 },
    { 0xe52792b3, SUC_lexicon_tags_91 },
    { 0x75a792b3, SUC_lexicon_tags_49 },
    { 0xe1a792b4, SUC_lexicon_tags_15 },
    { 0x63e792b6, SUC_lexicon_tags_30 },
    { 0xf38792b6, SUC_lexicon_tags_2 },
    { 0xb9e792b8, SUC_lexicon_tags_65 },
    { 0xe10792b9, SUC_lexicon_tags_25 },
    { 0xdce792ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4792be, SUC_lexicon_tags_28 },
    { 0x71c792be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86792c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54792c8, SUC_lexicon_tags_7 },
    { 0x216792c9, SUC_lexicon_tags_19 },
    { 0xc26792ca, SUC_lexicon_tags_11 },
    { 0x7c8792cb, SUC_lexicon_tags_38 },
    { 0x87c792cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2c792ce, SUC_lexicon_tags_2 },
    { 0xda0792cf, SUC_lexicon_tags_17 },
    { 0xeca792d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5d2792d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8792d9, SUC_lexicon_tags_23 },
    { 0x26c792da, SUC_lexicon_tags_38 },
    { 0xb8c792d9, SUC_lexicon_tags_37 },
    { 0x2e4792dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b8792de, SUC_lexicon_tags_25 },
    { 0xfc4792de, SUC_lexicon_tags_60 },
    { 0x376792e0, SUC_lexicon_tags_138 },
    { 0x508792e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c792e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6b0792e6, SUC_lexicon_tags_2 },
    { 0x9dc792e6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70792ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124792f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf4792f3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xede792f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x234792f7, SUC_lexicon_tags_15 },
    { 0xd68792f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb16792fa, SUC_lexicon_tags_10 },
    { 0x3ea792fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c79300, SUC_lexicon_tags_33 },
    { 0xbae79300, SUC_lexicon_tags_12 },
    { 0x2f879302, SUC_lexicon_tags_30 },
    { 0xaa79303, SUC_lexicon_tags_12 },
    { 0x9ce79304, SUC_lexicon_tags_37 },
    { 0xc4a79304, SUC_lexicon_tags_25 },
    { 0x41679302, SUC_lexicon_tags_30 },
    { 0x7c679305, SUC_lexicon_tags_43 },
    { 0xcc479301, SUC_lexicon_tags_37 },
    { 0x2ee79309, SUC_lexicon_tags_7 },
    { 0x83c79308, SUC_lexicon_tags_14 },
    { 0x7ea79302, SUC_lexicon_tags_2 },
    { 0xa8a7930c, SUC_lexicon_tags_2 },
    { 0x65a7930a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9479310, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e79320, SUC_lexicon_tags_2 },
    { 0x3ba79321, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdc79323, SUC_lexicon_tags_59 },
    { 0xf5479324, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c479327, SUC_lexicon_tags_19 },
    { 0x38c79328, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3cc7932a, SUC_lexicon_tags_19 },
    { 0xe067932a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e879332, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64479338, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5827933e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7279342, SUC_lexicon_tags_7 },
    { 0xb4479343, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e079345, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a7934a, SUC_lexicon_tags_11 },
    { 0x5ae7934b, SUC_lexicon_tags_7 },
    { 0x1b67934c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4667934e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x72879351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a79358, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49679361, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6147936b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d47936e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51479370, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d679374, SUC_lexicon_tags_0 },
    { 0x8aa79375, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x48679378, SUC_lexicon_tags_15 },
    { 0xfe479379, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e7937f, SUC_lexicon_tags_29 },
    { 0x1a079380, SUC_lexicon_tags_27 },
    { 0xac479381, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d679384, SUC_lexicon_tags_47 },
    { 0x1ea79384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92879387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5479389, SUC_lexicon_tags_5 },
    { 0x52879389, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5c7938c, SUC_lexicon_tags_9 },
    { 0xbae7938c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee879393, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae79399, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c87939b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf127939d, SUC_lexicon_tags_63 },
    { 0x6c27939e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a793a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa793a9, SUC_lexicon_tags_1 },
    { 0xc90793aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892793af, SUC_lexicon_tags_7 },
    { 0x7ae793b0, SUC_lexicon_tags_1 },
    { 0x478793b1, SUC_lexicon_tags_12 },
    { 0x3da793b1, SUC_lexicon_tags_33 },
    { 0xb90793af, SUC_lexicon_tags_2 },
    { 0x5c2793b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710793bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dc793c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfd6793c3, SUC_lexicon_tags_91 },
    { 0x17e793c4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x60793c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac793ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x564793d0, SUC_lexicon_tags_17 },
    { 0xab0793d1, SUC_lexicon_tags_33 },
    { 0x1d0793d0, SUC_lexicon_tags_47 },
    { 0x710793d3, SUC_lexicon_tags_26 },
    { 0x530793d4, SUC_lexicon_tags_0 },
    { 0xdb4793d0, SUC_lexicon_tags_79 },
    { 0x39e793d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x588793d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9b2793da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6793e0, SUC_lexicon_tags_49 },
    { 0x79a793e1, SUC_lexicon_tags_12 },
    { 0x6ae793e0, SUC_lexicon_tags_7 },
    { 0xfd8793e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac793ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0793ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea793fa, SUC_lexicon_tags_66 },
    { 0x3e8793fb, SUC_lexicon_tags_37 },
    { 0xa56793fa, SUC_lexicon_tags_7 },
    { 0xfea793fd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa3e793ff, SUC_lexicon_tags_15 },
    { 0x4fe79400, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc79409, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca47940b, SUC_lexicon_tags_37 },
    { 0xc487940c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ca7940e, SUC_lexicon_tags_82 },
    { 0xace7940f, SUC_lexicon_tags_25 },
    { 0xafc7940e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c879413, SUC_lexicon_tags_15 },
    { 0x18679414, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84679416, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4007941a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe447941d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf727941f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c079422, SUC_lexicon_tags_13 },
    { 0x6d479423, SUC_lexicon_tags_29 },
    { 0xf3279424, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66679428, SUC_lexicon_tags_16 },
    { 0x8fc79428, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2147942b, SUC_lexicon_tags_33 },
    { 0x34e7942c, SUC_lexicon_tags_12 },
    { 0x3cc7942d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce79432, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1279439, SUC_lexicon_tags_15 },
    { 0x6067943a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8b67943c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb87943e, SUC_lexicon_tags_8 },
    { 0x6ec7943f, SUC_lexicon_tags_37 },
    { 0x84679440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa479442, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f879445, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e79448, SUC_lexicon_tags_34 },
    { 0x66879448, SUC_lexicon_tags_12 },
    { 0x3dc7944a, SUC_lexicon_tags_1 },
    { 0xa307944b, SUC_lexicon_tags_82 },
    { 0xc107944b, SUC_lexicon_tags_23 },
    { 0x5907944d, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7679455, SUC_lexicon_tags_25 },
    { 0xd879455, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf687945a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4047945c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae79462, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2a279464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda79467, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7879469, SUC_lexicon_tags_17 },
    { 0x9ac79469, SUC_lexicon_tags_19 },
    { 0x2987946b, SUC_lexicon_tags_37 },
    { 0x4a07946c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7827946e, SUC_lexicon_tags_27 },
    { 0x93c7946f, SUC_lexicon_tags_21 },
    { 0xfe79470, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e879476, SUC_lexicon_tags_2 },
    { 0x4a279477, SUC_lexicon_tags_47 },
    { 0x8e079478, SUC_lexicon_tags_1 },
    { 0x80279479, SUC_lexicon_tags_37 },
    { 0xe0079477, SUC_lexicon_tags_12 },
    { 0xd3a7947b, SUC_lexicon_tags_1 },
    { 0x9de79476, SUC_lexicon_tags_116 },
    { 0x7927947c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4847947f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33a79481, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91879485, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb4279487, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77a79489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc479495, SUC_lexicon_tags_63 },
    { 0xaf079496, SUC_lexicon_tags_39 },
    { 0x17079496, SUC_lexicon_tags_30 },
    { 0xe3079497, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a7949b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27949e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x528794a0, SUC_lexicon_tags_1 },
    { 0xf8e794a1, SUC_lexicon_tags_19 },
    { 0xf96794a0, SUC_lexicon_tags_15 },
    { 0xb9a794a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x262794a5, SUC_lexicon_tags_65 },
    { 0xb96794a5, SUC_lexicon_tags_65 },
    { 0x75c794a6, SUC_lexicon_tags_30 },
    { 0x8a8794a6, SUC_lexicon_tags_47 },
    { 0x698794a9, SUC_lexicon_tags_12 },
    { 0x2ce794a9, SUC_lexicon_tags_17 },
    { 0x13e794a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8da794ad, SUC_lexicon_tags_2 },
    { 0xb9c794ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20794b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14794b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a794bf, SUC_lexicon_tags_48 },
    { 0xfd0794c0, SUC_lexicon_tags_11 },
    { 0xe80794c1, SUC_lexicon_tags_47 },
    { 0xeaa794c2, SUC_lexicon_tags_2 },
    { 0xafa794c2, SUC_lexicon_tags_45 },
    { 0x3e0794c3, SUC_lexicon_tags_45 },
    { 0xeca794c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0794c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64794cb, SUC_lexicon_tags_17 },
    { 0xf0a794cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x770794ce, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x126794d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e794d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ae794d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6794da, SUC_lexicon_tags_79 },
    { 0x31e794db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65a794dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a794df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca794e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8794e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e794e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa794eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6794ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976794f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2794f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa79502, SUC_lexicon_tags_19 },
    { 0x3a679502, SUC_lexicon_tags_0 },
    { 0x72c79504, SUC_lexicon_tags_11 },
    { 0xc5679502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92679507, SUC_lexicon_tags_17 },
    { 0x41e79507, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2607950b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679512, SUC_lexicon_tags_26 },
    { 0x35c79513, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaee79515, SUC_lexicon_tags_14 },
    { 0xbaa79515, SUC_lexicon_tags_22 },
    { 0xcb479517, SUC_lexicon_tags_2 },
    { 0x38279515, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f27951b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6107951d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8679522, SUC_lexicon_tags_43 },
    { 0xe2279522, SUC_lexicon_tags_48 },
    { 0x9dc79524, SUC_lexicon_tags_37 },
    { 0xd1079524, SUC_lexicon_tags_45 },
    { 0xd1479524, SUC_lexicon_tags_29 },
    { 0x59879527, SUC_lexicon_tags_25 },
    { 0xe1c79527, SUC_lexicon_tags_49 },
    { 0x77e79528, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb787952b, SUC_lexicon_tags_1 },
    { 0xf867952c, SUC_lexicon_tags_17 },
    { 0x3227952b, SUC_lexicon_tags_15 },
    { 0xb207952e, SUC_lexicon_tags_63 },
    { 0xec87952e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c79537, SUC_lexicon_tags_8 },
    { 0xea479538, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd387953b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4479545, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x54279548, SUC_lexicon_tags_37 },
    { 0x6d079549, SUC_lexicon_tags_12 },
    { 0x54479549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd647954c, SUC_lexicon_tags_12 },
    { 0xa787954c, SUC_lexicon_tags_7 },
    { 0xf187954e, SUC_lexicon_tags_39 },
    { 0x7b87954e, SUC_lexicon_tags_25 },
    { 0x5287954d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac79553, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e879559, SUC_lexicon_tags_25 },
    { 0x56879559, SUC_lexicon_tags_82 },
    { 0x7d67955b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x687955d, SUC_lexicon_tags_37 },
    { 0x75e7955e, SUC_lexicon_tags_25 },
    { 0x3347955f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc79562, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a79567, SUC_lexicon_tags_46 },
    { 0xd5a79567, SUC_lexicon_tags_21 },
    { 0x6c79568, SUC_lexicon_tags_45 },
    { 0xa3a7956a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87c7956c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d679571, SUC_lexicon_tags_30 },
    { 0x1b479571, SUC_lexicon_tags_43 },
    { 0xec879571, SUC_lexicon_tags_1 },
    { 0xe8079574, SUC_lexicon_tags_30 },
    { 0xa5679575, SUC_lexicon_tags_11 },
    { 0x3f879576, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98079578, SUC_lexicon_tags_66 },
    { 0x70a79579, SUC_lexicon_tags_25 },
    { 0x8607957a, SUC_lexicon_tags_38 },
    { 0xe8e7957b, SUC_lexicon_tags_6 },
    { 0xbb87957c, SUC_lexicon_tags_63 },
    { 0xb827957d, SUC_lexicon_tags_38 },
    { 0xa107957d, SUC_lexicon_tags_33 },
    { 0xb027957d, SUC_lexicon_tags_37 },
    { 0x8187957a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c679582, SUC_lexicon_tags_87 },
    { 0xe479583, SUC_lexicon_tags_2 },
    { 0x35e79584, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc47958d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaf27958f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b879592, SUC_lexicon_tags_60 },
    { 0xd2679593, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97679597, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b67959d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c795a2, SUC_lexicon_tags_12 },
    { 0xd26795a3, SUC_lexicon_tags_7 },
    { 0x250795a4, SUC_lexicon_tags_37 },
    { 0xe88795a2, SUC_lexicon_tags_63 },
    { 0x228795a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd1a795a8, SUC_lexicon_tags_30 },
    { 0xbb8795a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x564795ab, SUC_lexicon_tags_28 },
    { 0xd6a795ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8795ae, SUC_lexicon_tags_87 },
    { 0x1ba795af, SUC_lexicon_tags_12 },
    { 0xa58795b0, SUC_lexicon_tags_25 },
    { 0xc72795b1, SUC_lexicon_tags_57 },
    { 0xb34795b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdb4795b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83e795b6, SUC_lexicon_tags_82 },
    { 0x528795b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6a795b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206795be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e795c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a795c7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6795cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf4c795cd, SUC_lexicon_tags_3 },
    { 0x884795ce, SUC_lexicon_tags_37 },
    { 0x1d4795cd, SUC_lexicon_tags_31 },
    { 0x14c795cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10795d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122795d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2795dc, SUC_lexicon_tags_1 },
    { 0x714795dd, SUC_lexicon_tags_11 },
    { 0x7f2795dd, SUC_lexicon_tags_37 },
    { 0xc50795dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13e795e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e795e5, SUC_lexicon_tags_6 },
    { 0x2a2795e6, SUC_lexicon_tags_15 },
    { 0xedc795e5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a795ed, SUC_lexicon_tags_37 },
    { 0xe70795ee, SUC_lexicon_tags_849 },
    { 0, NULL },
    { 0xa40795f0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2e795f2, SUC_lexicon_tags_31 },
    { 0xcca795f2, SUC_lexicon_tags_11 },
    { 0x5f6795f4, SUC_lexicon_tags_35 },
    { 0x4fc795f5, SUC_lexicon_tags_11 },
    { 0x554795f4, SUC_lexicon_tags_11 },
    { 0x93c795f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98795fd, SUC_lexicon_tags_17 },
    { 0xd00795fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x63079601, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x24079603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16279605, SUC_lexicon_tags_25 },
    { 0x5fa79606, SUC_lexicon_tags_6 },
    { 0x85479606, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2587960a, SUC_lexicon_tags_30 },
    { 0xc67960a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6847960d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3a7960f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e79612, SUC_lexicon_tags_7 },
    { 0x10a79613, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c79616, SUC_lexicon_tags_7 },
    { 0x78879616, SUC_lexicon_tags_4 },
    { 0xbb879618, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1e7961a, SUC_lexicon_tags_38 },
    { 0x7d67961b, SUC_lexicon_tags_65 },
    { 0x1f07961c, SUC_lexicon_tags_47 },
    { 0x1007961d, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de79620, SUC_lexicon_tags_11 },
    { 0xb3679620, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7079624, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x56c79626, SUC_lexicon_tags_37 },
    { 0xbf079627, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58a79629, SUC_lexicon_tags_27 },
    { 0x32479629, SUC_lexicon_tags_47 },
    { 0xe6e7962b, SUC_lexicon_tags_23 },
    { 0x7847962c, SUC_lexicon_tags_37 },
    { 0xd3c79629, SUC_lexicon_tags_37 },
    { 0x9be7962e, SUC_lexicon_tags_63 },
    { 0xbc07962e, SUC_lexicon_tags_7 },
    { 0x30279630, SUC_lexicon_tags_19 },
    { 0xb5679630, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e79635, SUC_lexicon_tags_15 },
    { 0xeae79635, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7963c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe5e7963e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f879644, SUC_lexicon_tags_25 },
    { 0x86e79644, SUC_lexicon_tags_27 },
    { 0xc7479646, SUC_lexicon_tags_60 },
    { 0xf7479644, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e7964a, SUC_lexicon_tags_63 },
    { 0x7ca7964b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb479650, SUC_lexicon_tags_7 },
    { 0xec79651, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8879653, SUC_lexicon_tags_29 },
    { 0x44e79654, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xede79656, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9de79658, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc947965a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba79660, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c79664, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e79669, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19279676, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57679678, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa507967e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2b879680, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d079689, SUC_lexicon_tags_49 },
    { 0xeca7968a, SUC_lexicon_tags_37 },
    { 0x19479689, SUC_lexicon_tags_25 },
    { 0x1347968c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e7968f, SUC_lexicon_tags_2 },
    { 0xa207968f, SUC_lexicon_tags_15 },
    { 0x8e879691, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e79695, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5fe79697, SUC_lexicon_tags_23 },
    { 0xe9279698, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e7969d, SUC_lexicon_tags_30 },
    { 0xdc27969e, SUC_lexicon_tags_2 },
    { 0x47e7969f, SUC_lexicon_tags_63 },
    { 0x786796a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x52a796a2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd42796a4, SUC_lexicon_tags_0 },
    { 0xf10796a4, SUC_lexicon_tags_32 },
    { 0xe26796a6, SUC_lexicon_tags_29 },
    { 0x28e796a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8c796a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380796ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c796b2, SUC_lexicon_tags_1 },
    { 0xc2c796b3, SUC_lexicon_tags_27 },
    { 0xcec796b2, SUC_lexicon_tags_25 },
    { 0x174796b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54796be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270796c3, SUC_lexicon_tags_12 },
    { 0xcb8796c4, SUC_lexicon_tags_27 },
    { 0x3fe796c5, SUC_lexicon_tags_12 },
    { 0xb30796c6, SUC_lexicon_tags_1 },
    { 0xffc796c6, SUC_lexicon_tags_2 },
    { 0xd88796c7, SUC_lexicon_tags_8 },
    { 0xb8796c5, SUC_lexicon_tags_37 },
    { 0x87a796ca, SUC_lexicon_tags_63 },
    { 0xd20796cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c796d4, SUC_lexicon_tags_36 },
    { 0x3be796d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb14796d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97e796d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c796df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e796e4, SUC_lexicon_tags_19 },
    { 0x2cc796e4, SUC_lexicon_tags_37 },
    { 0x742796e6, SUC_lexicon_tags_17 },
    { 0xb1e796e7, SUC_lexicon_tags_46 },
    { 0x572796e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c796eb, SUC_lexicon_tags_63 },
    { 0x742796eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10796ef, SUC_lexicon_tags_62 },
    { 0xd6c796f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34796f7, SUC_lexicon_tags_11 },
    { 0x8e8796f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46796fd, SUC_lexicon_tags_2 },
    { 0xc6a796fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c79701, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6c879703, SUC_lexicon_tags_6 },
    { 0xb0e79704, SUC_lexicon_tags_26 },
    { 0xfb679705, SUC_lexicon_tags_40 },
    { 0x7ba79706, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde879710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c79716, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf679718, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa187971c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9867971e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0e79720, SUC_lexicon_tags_17 },
    { 0xa6279721, SUC_lexicon_tags_31 },
    { 0x92679722, SUC_lexicon_tags_136 },
    { 0x2a79721, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdce79725, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae7972c, SUC_lexicon_tags_30 },
    { 0x5227972d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767973a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf447973d, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8279740, SUC_lexicon_tags_7 },
    { 0xd0c79741, SUC_lexicon_tags_37 },
    { 0xa8079741, SUC_lexicon_tags_45 },
    { 0x12c79740, SUC_lexicon_tags_31 },
    { 0xe9e79742, SUC_lexicon_tags_43 },
    { 0x3d079745, SUC_lexicon_tags_159 },
    { 0xa7a79745, SUC_lexicon_tags_38 },
    { 0x78279747, SUC_lexicon_tags_14 },
    { 0xd4679747, SUC_lexicon_tags_63 },
    { 0x73e79747, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd47974d, SUC_lexicon_tags_31 },
    { 0x5747974d, SUC_lexicon_tags_39 },
    { 0xbd07974d, SUC_lexicon_tags_43 },
    { 0x3a679750, SUC_lexicon_tags_33 },
    { 0xf8279751, SUC_lexicon_tags_11 },
    { 0x2307974f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3079756, SUC_lexicon_tags_4 },
    { 0x2ae79756, SUC_lexicon_tags_28 },
    { 0x10279758, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef87975b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf987975d, SUC_lexicon_tags_0 },
    { 0xf067975d, SUC_lexicon_tags_7 },
    { 0x8a07975f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x87079762, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x36479765, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd479768, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7976c, SUC_lexicon_tags_2 },
    { 0x4587976d, SUC_lexicon_tags_17 },
    { 0xada7976e, SUC_lexicon_tags_47 },
    { 0xbfe7976d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7f479771, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c879773, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96079779, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x487977c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c79780, SUC_lexicon_tags_11 },
    { 0xb8e79781, SUC_lexicon_tags_37 },
    { 0x70879781, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0xa1e79784, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x33279786, SUC_lexicon_tags_23 },
    { 0xe7879786, SUC_lexicon_tags_14 },
    { 0xb5679788, SUC_lexicon_tags_16 },
    { 0x63479788, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa7978e, SUC_lexicon_tags_17 },
    { 0xe747978f, SUC_lexicon_tags_1 },
    { 0x9bc7978f, SUC_lexicon_tags_49 },
    { 0x3f279790, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9679797, SUC_lexicon_tags_2 },
    { 0xe8a79797, SUC_lexicon_tags_103 },
    { 0xfca79799, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce47979b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a7979e, SUC_lexicon_tags_1 },
    { 0x7d87979e, SUC_lexicon_tags_37 },
    { 0x9d87979e, SUC_lexicon_tags_2 },
    { 0xf64797a1, SUC_lexicon_tags_10 },
    { 0x9c797a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2797a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x424797a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0797aa, SUC_lexicon_tags_11 },
    { 0x152797ab, SUC_lexicon_tags_25 },
    { 0x41a797ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9c4797ae, SUC_lexicon_tags_11 },
    { 0x98e797af, SUC_lexicon_tags_29 },
    { 0x572797ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc797b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c797b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x664797b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0797be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x410797c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6797c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcde797c5, SUC_lexicon_tags_43 },
    { 0x6d8797c6, SUC_lexicon_tags_37 },
    { 0x318797c6, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xf82797c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e797cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f2797d1, SUC_lexicon_tags_85 },
    { 0xca8797d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c0797d4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec797d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406797dd, SUC_lexicon_tags_84 },
    { 0xc7e797de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378797e2, SUC_lexicon_tags_11 },
    { 0x886797e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x804797e5, SUC_lexicon_tags_3 },
    { 0xc7c797e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7fc797e8, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0797eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x302797ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64797f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598797f6, SUC_lexicon_tags_37 },
    { 0xd7e797f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x60c797f9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4d4797fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3479804, SUC_lexicon_tags_7 },
    { 0x5c279805, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1487980b, SUC_lexicon_tags_12 },
    { 0x46c7980b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8747980e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c79811, SUC_lexicon_tags_8 },
    { 0x81e79811, SUC_lexicon_tags_1 },
    { 0x54a79813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da79816, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa7981f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0c79821, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3879829, SUC_lexicon_tags_19 },
    { 0x8f47982a, SUC_lexicon_tags_14 },
    { 0xdba7982b, SUC_lexicon_tags_15 },
    { 0xd787982b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e79832, SUC_lexicon_tags_4 },
    { 0x63c79832, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8e479835, SUC_lexicon_tags_9 },
    { 0x2d279836, SUC_lexicon_tags_8 },
    { 0x2be79836, SUC_lexicon_tags_49 },
    { 0xb5679835, SUC_lexicon_tags_0 },
    { 0x4d279836, SUC_lexicon_tags_15 },
    { 0xa7a7983a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5727983e, SUC_lexicon_tags_37 },
    { 0xf567983f, SUC_lexicon_tags_7 },
    { 0x36c7983e, SUC_lexicon_tags_82 },
    { 0xb6879841, SUC_lexicon_tags_66 },
    { 0xf0e7983e, SUC_lexicon_tags_47 },
    { 0x72a79843, SUC_lexicon_tags_56 },
    { 0xd479844, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c79847, SUC_lexicon_tags_74 },
    { 0xaaa79848, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e7984e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc79852, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7079855, SUC_lexicon_tags_65 },
    { 0xc8079855, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3427985a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2767985d, SUC_lexicon_tags_37 },
    { 0x8767985e, SUC_lexicon_tags_1 },
    { 0x8727985f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x43079861, SUC_lexicon_tags_11 },
    { 0x45a79861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21e79864, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59e79866, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x38679869, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea79872, SUC_lexicon_tags_19 },
    { 0x8f679872, SUC_lexicon_tags_28 },
    { 0x4f879874, SUC_lexicon_tags_11 },
    { 0x67679875, SUC_lexicon_tags_2 },
    { 0x8ac79875, SUC_lexicon_tags_44 },
    { 0xf279877, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x82879879, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbf47987b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c7987f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe079881, SUC_lexicon_tags_11 },
    { 0xcce79882, SUC_lexicon_tags_23 },
    { 0xea79883, SUC_lexicon_tags_12 },
    { 0xbfc79882, SUC_lexicon_tags_348 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe079887, SUC_lexicon_tags_7 },
    { 0xa9079887, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x8207988b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6607988d, SUC_lexicon_tags_25 },
    { 0xccc7988d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc79891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e79894, SUC_lexicon_tags_37 },
    { 0x2079895, SUC_lexicon_tags_37 },
    { 0xd5279895, SUC_lexicon_tags_17 },
    { 0xe4679894, SUC_lexicon_tags_39 },
    { 0xcd479898, SUC_lexicon_tags_33 },
    { 0x68e79899, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa7989e, SUC_lexicon_tags_25 },
    { 0xadc7989f, SUC_lexicon_tags_38 },
    { 0x8ee798a0, SUC_lexicon_tags_12 },
    { 0x4a0798a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2798a4, SUC_lexicon_tags_17 },
    { 0xe92798a5, SUC_lexicon_tags_19 },
    { 0x510798a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4798ac, SUC_lexicon_tags_28 },
    { 0xb92798ad, SUC_lexicon_tags_37 },
    { 0xc18798ae, SUC_lexicon_tags_37 },
    { 0xa36798af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0798b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x304798b6, SUC_lexicon_tags_15 },
    { 0x45c798b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c798bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968798c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926798cc, SUC_lexicon_tags_37 },
    { 0x832798cd, SUC_lexicon_tags_198 },
    { 0x56a798cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x860798d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0798d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670798da, SUC_lexicon_tags_39 },
    { 0xeae798db, SUC_lexicon_tags_29 },
    { 0x3e6798dc, SUC_lexicon_tags_33 },
    { 0xb24798dd, SUC_lexicon_tags_14 },
    { 0x5d2798de, SUC_lexicon_tags_7 },
    { 0xa94798df, SUC_lexicon_tags_38 },
    { 0xbf6798dc, SUC_lexicon_tags_60 },
    { 0x238798e1, SUC_lexicon_tags_55 },
    { 0x8e798e2, SUC_lexicon_tags_2 },
    { 0xcc798e3, SUC_lexicon_tags_2 },
    { 0x8ba798e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9a798e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886798eb, SUC_lexicon_tags_47 },
    { 0x540798ec, SUC_lexicon_tags_33 },
    { 0x8b4798ed, SUC_lexicon_tags_77 },
    { 0x560798ee, SUC_lexicon_tags_17 },
    { 0x4f8798ee, SUC_lexicon_tags_43 },
    { 0xac2798ee, SUC_lexicon_tags_15 },
    { 0x79c798f1, SUC_lexicon_tags_3 },
    { 0x942798f2, SUC_lexicon_tags_37 },
    { 0xbd2798f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256798f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c798fb, SUC_lexicon_tags_0 },
    { 0xeb2798fc, SUC_lexicon_tags_29 },
    { 0x80a798fd, SUC_lexicon_tags_1 },
    { 0xa8c798fe, SUC_lexicon_tags_12 },
    { 0xb3e798fe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x19879901, SUC_lexicon_tags_12 },
    { 0x1ee79902, SUC_lexicon_tags_2 },
    { 0xc5279903, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd679905, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xad879907, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c7990a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1407990c, SUC_lexicon_tags_29 },
    { 0xb87990d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c879910, SUC_lexicon_tags_12 },
    { 0xee79911, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaec79913, SUC_lexicon_tags_25 },
    { 0x43e79913, SUC_lexicon_tags_12 },
    { 0x96279913, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b079922, SUC_lexicon_tags_38 },
    { 0xd2a79922, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce79928, SUC_lexicon_tags_39 },
    { 0xf9c79929, SUC_lexicon_tags_37 },
    { 0x7787992a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbee7992c, SUC_lexicon_tags_55 },
    { 0x7a07992c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1647992f, SUC_lexicon_tags_17 },
    { 0x6de7992f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30479934, SUC_lexicon_tags_25 },
    { 0x4d079934, SUC_lexicon_tags_17 },
    { 0x90c79936, SUC_lexicon_tags_2 },
    { 0x2bc79937, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39479939, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c7993c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8967993e, SUC_lexicon_tags_37 },
    { 0xd7a7993e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a79942, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2387994a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c279951, SUC_lexicon_tags_15 },
    { 0x1e679952, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9c79954, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e679957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7995b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa87995e, SUC_lexicon_tags_11 },
    { 0x667995f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4279966, SUC_lexicon_tags_14 },
    { 0xdfc79966, SUC_lexicon_tags_11 },
    { 0x91079966, SUC_lexicon_tags_37 },
    { 0x1aa79969, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f679974, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee79979, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3aa7997b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f67997f, SUC_lexicon_tags_33 },
    { 0x7067997f, SUC_lexicon_tags_45 },
    { 0x8f079980, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5679984, SUC_lexicon_tags_37 },
    { 0x1f079985, SUC_lexicon_tags_17 },
    { 0x7a279984, SUC_lexicon_tags_1 },
    { 0xce279987, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b47998f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x33a79991, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e79994, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7279996, SUC_lexicon_tags_13 },
    { 0xac79997, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967999d, SUC_lexicon_tags_61 },
    { 0x947999d, SUC_lexicon_tags_33 },
    { 0x5b07999f, SUC_lexicon_tags_2 },
    { 0xe947999d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200799a4, SUC_lexicon_tags_27 },
    { 0x596799a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x904799a7, SUC_lexicon_tags_56 },
    { 0xfe6799a8, SUC_lexicon_tags_2 },
    { 0x544799a8, SUC_lexicon_tags_11 },
    { 0x22e799a7, SUC_lexicon_tags_26 },
    { 0xd18799ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554799b0, SUC_lexicon_tags_68 },
    { 0x258799b0, SUC_lexicon_tags_14 },
    { 0x47e799b2, SUC_lexicon_tags_9 },
    { 0xd82799b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a799b6, SUC_lexicon_tags_1 },
    { 0x766799b6, SUC_lexicon_tags_12 },
    { 0x57c799b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x58c799bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdea799bd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa799c0, SUC_lexicon_tags_27 },
    { 0x72e799c0, SUC_lexicon_tags_16 },
    { 0xee0799c2, SUC_lexicon_tags_26 },
    { 0xfa799c3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8799c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c799cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x162799ce, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450799d7, SUC_lexicon_tags_33 },
    { 0x5a2799d8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6799dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06799e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x398799e4, SUC_lexicon_tags_10 },
    { 0x176799e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x144799e8, SUC_lexicon_tags_11 },
    { 0x308799e8, SUC_lexicon_tags_17 },
    { 0xcc2799e8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x258799ec, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390799f9, SUC_lexicon_tags_13 },
    { 0xf28799f9, SUC_lexicon_tags_0 },
    { 0x88799f9, SUC_lexicon_tags_77 },
    { 0x714799fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20799ff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xba879a01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd879a07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x73e79a09, SUC_lexicon_tags_55 },
    { 0xd3079a0a, SUC_lexicon_tags_5 },
    { 0x80479a0b, SUC_lexicon_tags_15 },
    { 0x4a679a0c, SUC_lexicon_tags_63 },
    { 0xd7c79a0b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xde79a0f, SUC_lexicon_tags_6 },
    { 0x1e679a10, SUC_lexicon_tags_11 },
    { 0x3c879a11, SUC_lexicon_tags_38 },
    { 0x4d279a12, SUC_lexicon_tags_12 },
    { 0x3e879a13, SUC_lexicon_tags_34 },
    { 0xd0079a12, SUC_lexicon_tags_33 },
    { 0xa2a79a14, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef279a1d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80879a21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57c79a23, SUC_lexicon_tags_47 },
    { 0xbfe79a24, SUC_lexicon_tags_14 },
    { 0xbde79a25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9079a28, SUC_lexicon_tags_37 },
    { 0xdf679a29, SUC_lexicon_tags_1 },
    { 0x1f279a2a, SUC_lexicon_tags_40 },
    { 0x64079a29, SUC_lexicon_tags_25 },
    { 0xf0c79a29, SUC_lexicon_tags_130 },
    { 0x46279a2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e79a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5679a36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4e79a38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97879a3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a279a3e, SUC_lexicon_tags_17 },
    { 0xe7679a3e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda279a43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4c79a45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b079a49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f879a4f, SUC_lexicon_tags_37 },
    { 0xc1079a50, SUC_lexicon_tags_57 },
    { 0xca679a50, SUC_lexicon_tags_14 },
    { 0x71679a52, SUC_lexicon_tags_15 },
    { 0x68679a53, SUC_lexicon_tags_12 },
    { 0xaf679a4f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a79a59, SUC_lexicon_tags_12 },
    { 0x5679a5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c79a66, SUC_lexicon_tags_47 },
    { 0xad079a67, SUC_lexicon_tags_33 },
    { 0xbfa79a68, SUC_lexicon_tags_37 },
    { 0xa0879a69, SUC_lexicon_tags_6 },
    { 0xef879a6a, SUC_lexicon_tags_9 },
    { 0x12279a6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73079a70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2679a74, SUC_lexicon_tags_38 },
    { 0x21079a74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4879a78, SUC_lexicon_tags_27 },
    { 0xae679a78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb479a7b, SUC_lexicon_tags_37 },
    { 0x72e79a7c, SUC_lexicon_tags_1 },
    { 0x83679a7c, SUC_lexicon_tags_56 },
    { 0xeb279a7e, SUC_lexicon_tags_25 },
    { 0x4c679a7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f479a83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ba79a85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x20079a88, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f479a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71a79a92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1479a97, SUC_lexicon_tags_25 },
    { 0x69479a98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a79aa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29079aa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b079aa5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52479aa7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90279aac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62279aae, SUC_lexicon_tags_47 },
    { 0x2679aaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27479ab6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c79abc, SUC_lexicon_tags_7 },
    { 0xe3a79abc, SUC_lexicon_tags_0 },
    { 0x22079abe, SUC_lexicon_tags_72 },
    { 0x30879abe, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0xebe79ac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4879ac4, SUC_lexicon_tags_8 },
    { 0xbaa79ac5, SUC_lexicon_tags_25 },
    { 0xb4679ac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6679acb, SUC_lexicon_tags_12 },
    { 0x6ec79acc, SUC_lexicon_tags_63 },
    { 0xeb079acd, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x26679acf, SUC_lexicon_tags_16 },
    { 0x4ea79acf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61479ad3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1479ad5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a79ad9, SUC_lexicon_tags_2 },
    { 0x2e879ada, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8c79adc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44479ae4, SUC_lexicon_tags_37 },
    { 0x32879ae4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9079aec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe79af2, SUC_lexicon_tags_28 },
    { 0x67879af2, SUC_lexicon_tags_47 },
    { 0x24079af3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42a79af6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4479af8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a79afb, SUC_lexicon_tags_14 },
    { 0xb5679afb, SUC_lexicon_tags_65 },
    { 0x66479afb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ca79aff, SUC_lexicon_tags_91 },
    { 0xad679b00, SUC_lexicon_tags_11 },
    { 0x2c079b01, SUC_lexicon_tags_11 },
    { 0xef279b02, SUC_lexicon_tags_10 },
    { 0x15879b02, SUC_lexicon_tags_15 },
    { 0x2b279b04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44479b0f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e679b15, SUC_lexicon_tags_14 },
    { 0xc4a79b16, SUC_lexicon_tags_68 },
    { 0x5a879b16, SUC_lexicon_tags_25 },
    { 0xeb679b17, SUC_lexicon_tags_17 },
    { 0x61479b17, SUC_lexicon_tags_37 },
    { 0x4da79b1a, SUC_lexicon_tags_6 },
    { 0xe5279b1b, SUC_lexicon_tags_19 },
    { 0xc8079b1c, SUC_lexicon_tags_47 },
    { 0xa7c79b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c79b1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee79b28, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29479b2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71879b2e, SUC_lexicon_tags_17 },
    { 0x13e79b2e, SUC_lexicon_tags_100 },
    { 0x98c79b30, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4679b33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xef479b35, SUC_lexicon_tags_14 },
    { 0x57679b35, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x18679b38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1279b3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7679b3e, SUC_lexicon_tags_12 },
    { 0x43a79b3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7479b42, SUC_lexicon_tags_37 },
    { 0x13079b43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da79b47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7a479b49, SUC_lexicon_tags_151 },
    { 0xcb879b4a, SUC_lexicon_tags_1 },
    { 0x4479b4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c79b56, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xffc79b58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a79b5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2079b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a679b62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59479b64, SUC_lexicon_tags_15 },
    { 0x85079b65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c679b68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c679b6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f279b70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37879b72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc79b78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae79b7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51279b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf479b80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e79b84, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc79b8b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e79b93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd079b97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c79b9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63a79ba1, SUC_lexicon_tags_17 },
    { 0x4d479ba2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36a79ba4, SUC_lexicon_tags_15 },
    { 0xab279ba5, SUC_lexicon_tags_27 },
    { 0x4a79ba5, SUC_lexicon_tags_39 },
    { 0xf6e79ba4, SUC_lexicon_tags_82 },
    { 0x4ee79ba8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae79baf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdc479bb1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da79bb6, SUC_lexicon_tags_84 },
    { 0xb479bb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7279bbf, SUC_lexicon_tags_10 },
    { 0x56e79bbf, SUC_lexicon_tags_29 },
    { 0xc0c79bc1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d879bc5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4879bca, SUC_lexicon_tags_33 },
    { 0xf479bcb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x37079bcd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1479bd2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9c79bd4, SUC_lexicon_tags_0 },
    { 0x44e79bd4, SUC_lexicon_tags_6 },
    { 0xd9c79bd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42079bda, SUC_lexicon_tags_84 },
    { 0x8c879bdb, SUC_lexicon_tags_29 },
    { 0xc4879bdc, SUC_lexicon_tags_2 },
    { 0x78079bdb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2aa79bdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee79be2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c79be4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x77879be7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x40479be9, SUC_lexicon_tags_1 },
    { 0x67279bea, SUC_lexicon_tags_25 },
    { 0xcd079beb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7279bed, SUC_lexicon_tags_127 },
    { 0x68e79bed, SUC_lexicon_tags_37 },
    { 0x57e79bef, SUC_lexicon_tags_17 },
    { 0xc8679bef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d279bf2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38079bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece79bfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51679c01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e879c04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa4479c06, SUC_lexicon_tags_8 },
    { 0x49a79c06, SUC_lexicon_tags_39 },
    { 0x2d079c06, SUC_lexicon_tags_13 },
    { 0x11079c06, SUC_lexicon_tags_9 },
    { 0x52e79c09, SUC_lexicon_tags_87 },
    { 0xf9279c0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61079c0d, SUC_lexicon_tags_37 },
    { 0x33479c0e, SUC_lexicon_tags_19 },
    { 0xc7e79c0f, SUC_lexicon_tags_2 },
    { 0x7b279c10, SUC_lexicon_tags_14 },
    { 0x30079c11, SUC_lexicon_tags_31 },
    { 0x49879c12, SUC_lexicon_tags_15 },
    { 0x7ae79c10, SUC_lexicon_tags_38 },
    { 0xcdc79c12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a79c18, SUC_lexicon_tags_17 },
    { 0x4b079c19, SUC_lexicon_tags_1 },
    { 0xc5e79c1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a79c1f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde479c2c, SUC_lexicon_tags_47 },
    { 0x77679c2d, SUC_lexicon_tags_14 },
    { 0x61879c2c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f879c31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba79c3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1679c3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c79c42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0679c44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e79c49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ce79c4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20679c4e, SUC_lexicon_tags_6 },
    { 0x58879c4f, SUC_lexicon_tags_17 },
    { 0x92a79c4f, SUC_lexicon_tags_17 },
    { 0xb2a79c51, SUC_lexicon_tags_7 },
    { 0x48679c51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdec79c54, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x65279c57, SUC_lexicon_tags_2 },
    { 0xd2279c58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f879c5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0c79c5f, SUC_lexicon_tags_1 },
    { 0xd2279c60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c279c63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65879c65, SUC_lexicon_tags_68 },
    { 0x25c79c66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb879c6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfc79c6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbf279c70, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9279c73, SUC_lexicon_tags_17 },
    { 0xf6a79c74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d879c76, SUC_lexicon_tags_15 },
    { 0x39c79c77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc679c79, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x68079c7b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x72479c7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0879c92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5279c97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc679c99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f679c9d, SUC_lexicon_tags_0 },
    { 0x33879c9e, SUC_lexicon_tags_12 },
    { 0x6aa79c9f, SUC_lexicon_tags_7 },
    { 0x1879ca0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83679ca4, SUC_lexicon_tags_12 },
    { 0x2da79ca5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33879ca8, SUC_lexicon_tags_19 },
    { 0x37e79ca9, SUC_lexicon_tags_25 },
    { 0x63079caa, SUC_lexicon_tags_19 },
    { 0xce279cab, SUC_lexicon_tags_82 },
    { 0xdda79cac, SUC_lexicon_tags_28 },
    { 0xc4c79cab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3679caf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3279cb2, SUC_lexicon_tags_61 },
    { 0x63679cb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74c79cb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23879cba, SUC_lexicon_tags_91 },
    { 0x6fe79cbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6de79cbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86e79cbf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e79cc5, SUC_lexicon_tags_47 },
    { 0xc2a79cc6, SUC_lexicon_tags_17 },
    { 0x43079cc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a79cca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45e79ccc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3aa79cce, SUC_lexicon_tags_32 },
    { 0x17c79cce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e79cd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa479cda, SUC_lexicon_tags_1 },
    { 0x29e79cdb, SUC_lexicon_tags_14 },
    { 0xa2079cdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x87679cdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1479ce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a79cea, SUC_lexicon_tags_17 },
    { 0x60a79ceb, SUC_lexicon_tags_17 },
    { 0xae79cea, SUC_lexicon_tags_1 },
    { 0x41879ced, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a079cf0, SUC_lexicon_tags_37 },
    { 0x50c79cf1, SUC_lexicon_tags_11 },
    { 0x36c79cf2, SUC_lexicon_tags_45 },
    { 0x9be79cf3, SUC_lexicon_tags_7 },
    { 0xb3679cf4, SUC_lexicon_tags_11 },
    { 0x7879cf5, SUC_lexicon_tags_12 },
    { 0xc2c79cf4, SUC_lexicon_tags_2 },
    { 0x1e79cf7, SUC_lexicon_tags_14 },
    { 0x55c79cf2, SUC_lexicon_tags_2 },
    { 0x9b879cf9, SUC_lexicon_tags_12 },
    { 0x35a79cf9, SUC_lexicon_tags_27 },
    { 0xbc479cf9, SUC_lexicon_tags_11 },
    { 0x3c079cf9, SUC_lexicon_tags_15 },
    { 0xd2e79cfd, SUC_lexicon_tags_23 },
    { 0xcd079cfd, SUC_lexicon_tags_17 },
    { 0xfd279cff, SUC_lexicon_tags_380 },
    { 0x61079d00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94479d05, SUC_lexicon_tags_27 },
    { 0xda279d06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x94079d08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc679d0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b279d0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6479d11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f079d13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3479d15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8679d17, SUC_lexicon_tags_32 },
    { 0x62279d18, SUC_lexicon_tags_55 },
    { 0x82479d18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19279d1b, SUC_lexicon_tags_7 },
    { 0x6a279d1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67279d20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9e79d22, SUC_lexicon_tags_48 },
    { 0xdcc79d22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21879d27, SUC_lexicon_tags_8 },
    { 0xd9079d27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa79d2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20679d2f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee79d33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54e79d35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb279d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25679d3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61279d3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf0279d41, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x33c79d43, SUC_lexicon_tags_17 },
    { 0x34079d44, SUC_lexicon_tags_37 },
    { 0x95279d45, SUC_lexicon_tags_45 },
    { 0xa1679d46, SUC_lexicon_tags_28 },
    { 0x51079d47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd0479d49, SUC_lexicon_tags_11 },
    { 0x2ec79d49, SUC_lexicon_tags_27 },
    { 0xe4a79d49, SUC_lexicon_tags_21 },
    { 0x85479d4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1279d52, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a79d58, SUC_lexicon_tags_26 },
    { 0x46a79d59, SUC_lexicon_tags_47 },
    { 0x71e79d5a, SUC_lexicon_tags_2 },
    { 0xf0e79d5a, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x1f879d5d, SUC_lexicon_tags_64 },
    { 0x42c79d5e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da79d61, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca79d64, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1e79d66, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb279d6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c79d6f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10879d73, SUC_lexicon_tags_7 },
    { 0x3ee79d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f879d77, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d479d7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3c279d7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c79d7f, SUC_lexicon_tags_2 },
    { 0x87079d7f, SUC_lexicon_tags_82 },
    { 0xf2e79d81, SUC_lexicon_tags_12 },
    { 0x2a879d82, SUC_lexicon_tags_12 },
    { 0x7a79d81, SUC_lexicon_tags_53 },
    { 0xfac79d84, SUC_lexicon_tags_37 },
    { 0x24a79d85, SUC_lexicon_tags_17 },
    { 0xbf479d85, SUC_lexicon_tags_14 },
    { 0x63c79d83, SUC_lexicon_tags_63 },
    { 0xa3c79d88, SUC_lexicon_tags_6 },
    { 0xd3079d89, SUC_lexicon_tags_12 },
    { 0x60679d8a, SUC_lexicon_tags_23 },
    { 0x68079d8b, SUC_lexicon_tags_47 },
    { 0x6fe79d88, SUC_lexicon_tags_2 },
    { 0xe679d8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68079d90, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xae79d92, SUC_lexicon_tags_37 },
    { 0x7da79d92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa4879d95, SUC_lexicon_tags_11 },
    { 0xbdc79d95, SUC_lexicon_tags_1 },
    { 0x23679d95, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d479d9e, SUC_lexicon_tags_3 },
    { 0x7da79d9e, SUC_lexicon_tags_17 },
    { 0x6ba79d9e, SUC_lexicon_tags_15 },
    { 0x35679da1, SUC_lexicon_tags_0 },
    { 0xdc79da2, SUC_lexicon_tags_37 },
    { 0x35679d9f, SUC_lexicon_tags_60 },
    { 0x54879da4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc79da8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb479dad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe79db2, SUC_lexicon_tags_30 },
    { 0xde79db2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44879db9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7a679dbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77879dbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3279dc3, SUC_lexicon_tags_63 },
    { 0x9ac79dc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a679dc7, SUC_lexicon_tags_1 },
    { 0xf2a79dc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec79dd3, SUC_lexicon_tags_33 },
    { 0x9dc79dd4, SUC_lexicon_tags_27 },
    { 0x67479dd5, SUC_lexicon_tags_28 },
    { 0x7d079dd5, SUC_lexicon_tags_11 },
    { 0xa5279dd7, SUC_lexicon_tags_46 },
    { 0xf9879dd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b479dda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde79ddf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc8a79de1, SUC_lexicon_tags_15 },
    { 0x1679de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97879de5, SUC_lexicon_tags_27 },
    { 0x33679de5, SUC_lexicon_tags_25 },
    { 0x2aa79de7, SUC_lexicon_tags_29 },
    { 0xbe79de6, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x41a79dea, SUC_lexicon_tags_2 },
    { 0xd3e79dea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3679ded, SUC_lexicon_tags_12 },
    { 0xbd279dee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9079df0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a279df4, SUC_lexicon_tags_17 },
    { 0x26c79df5, SUC_lexicon_tags_11 },
    { 0xe2479df6, SUC_lexicon_tags_1 },
    { 0xe3a79df7, SUC_lexicon_tags_63 },
    { 0x67c79df6, SUC_lexicon_tags_65 },
    { 0xeb679df9, SUC_lexicon_tags_569 },
    { 0x71c79df9, SUC_lexicon_tags_37 },
    { 0xb2a79dfb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0279dff, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xa1479e01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a79e05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d079e07, SUC_lexicon_tags_25 },
    { 0x4679e08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e79e0c, SUC_lexicon_tags_6 },
    { 0xab079e0d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6879e10, SUC_lexicon_tags_91 },
    { 0x11679e11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b479e17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7279e1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca79e1e, SUC_lexicon_tags_579 },
    { 0x66c79e1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99279e22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9679e24, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8e079e26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc79e2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x92879e2d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4879e35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73279e38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e79e3b, SUC_lexicon_tags_13 },
    { 0x81679e3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa079e3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd679e41, SUC_lexicon_tags_8 },
    { 0xf7c79e42, SUC_lexicon_tags_12 },
    { 0x59679e43, SUC_lexicon_tags_40 },
    { 0xbf279e41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5c79e46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5079e4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5279e4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c79e52, SUC_lexicon_tags_14 },
    { 0xff479e53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8679e5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x19879e5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3679e60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea279e63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13279e6c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9c79e6e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7ea79e70, SUC_lexicon_tags_31 },
    { 0xdbe79e70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e79e75, SUC_lexicon_tags_12 },
    { 0x9679e76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54279e78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46a79e7a, SUC_lexicon_tags_37 },
    { 0x6b879e7a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea79e7e, SUC_lexicon_tags_25 },
    { 0x6fc79e7f, SUC_lexicon_tags_19 },
    { 0x2c079e7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c79e84, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9479e8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8a79e8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92279e92, SUC_lexicon_tags_11 },
    { 0xe2479e93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39679e95, SUC_lexicon_tags_39 },
    { 0xe2679e96, SUC_lexicon_tags_7 },
    { 0xed679e97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0679e9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e879e9e, SUC_lexicon_tags_1 },
    { 0x84e79e9f, SUC_lexicon_tags_7 },
    { 0x76879ea0, SUC_lexicon_tags_14 },
    { 0x4b879ea1, SUC_lexicon_tags_7 },
    { 0xfe079ea2, SUC_lexicon_tags_2 },
    { 0x1b679ea3, SUC_lexicon_tags_8 },
    { 0x32a79ea3, SUC_lexicon_tags_41 },
    { 0x4de79ea3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43079eae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1079eb0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2fa79eb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2879eb6, SUC_lexicon_tags_27 },
    { 0xeb279eb7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a679eba, SUC_lexicon_tags_37 },
    { 0xd7e79ebb, SUC_lexicon_tags_0 },
    { 0x5a279ebc, SUC_lexicon_tags_12 },
    { 0x14a79ebb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f279ebf, SUC_lexicon_tags_0 },
    { 0x6fe79ec0, SUC_lexicon_tags_17 },
    { 0xcf079ebf, SUC_lexicon_tags_11 },
    { 0x5a479ec1, SUC_lexicon_tags_25 },
    { 0x12679ec3, SUC_lexicon_tags_38 },
    { 0xf4079ec0, SUC_lexicon_tags_61 },
    { 0x3ba79ec5, SUC_lexicon_tags_11 },
    { 0x50279ec5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a79ecb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2479ed0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18479ed6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e79ed9, SUC_lexicon_tags_87 },
    { 0x59c79eda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92879edc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a679ede, SUC_lexicon_tags_14 },
    { 0x17679edf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x77479ee2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4479ee4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8079ee6, SUC_lexicon_tags_30 },
    { 0x67279ee7, SUC_lexicon_tags_37 },
    { 0x54a79ee7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe279eea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1079eec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x82279eee, SUC_lexicon_tags_37 },
    { 0x7b079eee, SUC_lexicon_tags_1 },
    { 0x52279eee, SUC_lexicon_tags_15 },
    { 0xbbe79ef1, SUC_lexicon_tags_15 },
    { 0x92079ef0, SUC_lexicon_tags_37 },
    { 0x26479ef1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2679ef6, SUC_lexicon_tags_63 },
    { 0x7ca79ef6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae79efb, SUC_lexicon_tags_33 },
    { 0xa3e79efc, SUC_lexicon_tags_14 },
    { 0x62c79efd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53a79eff, SUC_lexicon_tags_63 },
    { 0xa4e79f00, SUC_lexicon_tags_31 },
    { 0x37a79eff, SUC_lexicon_tags_12 },
    { 0x64e79f02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8c79f04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50279f10, SUC_lexicon_tags_3 },
    { 0x76079f11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe1279f13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78679f18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c079f23, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95479f27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb279f2e, SUC_lexicon_tags_38 },
    { 0xf6c79f2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c79f36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc279f39, SUC_lexicon_tags_61 },
    { 0x4f079f3a, SUC_lexicon_tags_37 },
    { 0x3a79f3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c079f48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xefc79f4a, SUC_lexicon_tags_17 },
    { 0x42c79f4b, SUC_lexicon_tags_34 },
    { 0x3fa79f4c, SUC_lexicon_tags_1 },
    { 0x42479f4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10e79f4f, SUC_lexicon_tags_25 },
    { 0x92679f4f, SUC_lexicon_tags_25 },
    { 0xa3e79f51, SUC_lexicon_tags_39 },
    { 0xbac79f52, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb4079f54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45679f56, SUC_lexicon_tags_2 },
    { 0xad079f56, SUC_lexicon_tags_17 },
    { 0xf879f58, SUC_lexicon_tags_1 },
    { 0xf2a79f59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4079f66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2879f69, SUC_lexicon_tags_7 },
    { 0x3fa79f69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b079f6e, SUC_lexicon_tags_77 },
    { 0x76c79f6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c79f72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x42a79f74, SUC_lexicon_tags_19 },
    { 0xbba79f75, SUC_lexicon_tags_1 },
    { 0x50079f76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba79f7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69479f7f, SUC_lexicon_tags_12 },
    { 0x4c879f80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x40a79f82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42079f86, SUC_lexicon_tags_15 },
    { 0x87679f87, SUC_lexicon_tags_15 },
    { 0xd0879f88, SUC_lexicon_tags_11 },
    { 0x26a79f89, SUC_lexicon_tags_27 },
    { 0xf5079f8a, SUC_lexicon_tags_25 },
    { 0x3879f8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25679f8d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85479f9c, SUC_lexicon_tags_24 },
    { 0xe4479f9d, SUC_lexicon_tags_37 },
    { 0xf8e79f9e, SUC_lexicon_tags_11 },
    { 0x8ba79f9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71079fa5, SUC_lexicon_tags_60 },
    { 0x2cc79fa5, SUC_lexicon_tags_43 },
    { 0xc9079fa6, SUC_lexicon_tags_48 },
    { 0x3d879fa8, SUC_lexicon_tags_32 },
    { 0x38a79fa6, SUC_lexicon_tags_12 },
    { 0xcae79faa, SUC_lexicon_tags_13 },
    { 0xb4479fa6, SUC_lexicon_tags_39 },
    { 0x55079fac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac879fb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf079fb2, SUC_lexicon_tags_32 },
    { 0x5b279fb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28479fb5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2c879fb7, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x5e79fb9, SUC_lexicon_tags_7 },
    { 0x2ba79fba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9479fbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf879fbe, SUC_lexicon_tags_9 },
    { 0x9a879fbf, SUC_lexicon_tags_10 },
    { 0xe879fbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x23c79fc2, SUC_lexicon_tags_18 },
    { 0xae079fc3, SUC_lexicon_tags_17 },
    { 0xd1c79fc4, SUC_lexicon_tags_37 },
    { 0x39879fc2, SUC_lexicon_tags_37 },
    { 0x6ae79fc6, SUC_lexicon_tags_15 },
    { 0xad279fc3, SUC_lexicon_tags_2 },
    { 0x8bc79fc8, SUC_lexicon_tags_37 },
    { 0xb1079fc9, SUC_lexicon_tags_49 },
    { 0x57879fc9, SUC_lexicon_tags_27 },
    { 0xf4c79fcb, SUC_lexicon_tags_38 },
    { 0x98079fc4, SUC_lexicon_tags_37 },
    { 0x3b479fcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c79fd1, SUC_lexicon_tags_49 },
    { 0x19c79fd1, SUC_lexicon_tags_37 },
    { 0x89879fd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae79fdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf879fdf, SUC_lexicon_tags_87 },
    { 0x90079fe0, SUC_lexicon_tags_17 },
    { 0x16e79fdf, SUC_lexicon_tags_15 },
    { 0x8fe79fe1, SUC_lexicon_tags_26 },
    { 0x96679fe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c279fe5, SUC_lexicon_tags_15 },
    { 0x9c079fe6, SUC_lexicon_tags_3 },
    { 0xad679fe6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7679ff0, SUC_lexicon_tags_11 },
    { 0xe2479ff0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x49479ff4, SUC_lexicon_tags_0 },
    { 0xf0c79ff5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe879ff9, SUC_lexicon_tags_4 },
    { 0x6d279ff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81c79ffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee7a005, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf867a00c, SUC_lexicon_tags_17 },
    { 0xf467a00d, SUC_lexicon_tags_23 },
    { 0x26c7a00e, SUC_lexicon_tags_23 },
    { 0x5da7a00d, SUC_lexicon_tags_17 },
    { 0x6d67a00e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c7a014, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c07a01c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15e7a01e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5547a020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe107a027, SUC_lexicon_tags_14 },
    { 0x3527a028, SUC_lexicon_tags_37 },
    { 0x71e7a029, SUC_lexicon_tags_25 },
    { 0x53c7a029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1367a02d, SUC_lexicon_tags_60 },
    { 0x3e27a02e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f47a030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb127a037, SUC_lexicon_tags_43 },
    { 0xf8c7a038, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c7a03b, SUC_lexicon_tags_40 },
    { 0xbb07a03b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e7a04c, SUC_lexicon_tags_2 },
    { 0xa2a7a04c, SUC_lexicon_tags_30 },
    { 0xeea7a04d, SUC_lexicon_tags_38 },
    { 0xe287a04f, SUC_lexicon_tags_7 },
    { 0x9c07a04f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c7a054, SUC_lexicon_tags_15 },
    { 0x5167a055, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbe67a057, SUC_lexicon_tags_38 },
    { 0xe307a058, SUC_lexicon_tags_44 },
    { 0x45a7a058, SUC_lexicon_tags_37 },
    { 0x51c7a05a, SUC_lexicon_tags_6 },
    { 0xba7a05b, SUC_lexicon_tags_29 },
    { 0x9de7a05c, SUC_lexicon_tags_31 },
    { 0x6f87a05c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc3e7a05f, SUC_lexicon_tags_37 },
    { 0xb227a060, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae7a063, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8827a065, SUC_lexicon_tags_15 },
    { 0x17e7a065, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61e7a068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5447a06f, SUC_lexicon_tags_38 },
    { 0x8987a070, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1307a072, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe667a074, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd627a07b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1047a07d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa187a080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf87a084, SUC_lexicon_tags_15 },
    { 0x387a084, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5127a087, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6747a089, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c7a08d, SUC_lexicon_tags_14 },
    { 0x6867a08e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdbe7a090, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c7a093, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc707a096, SUC_lexicon_tags_37 },
    { 0xae7a096, SUC_lexicon_tags_2 },
    { 0x8447a096, SUC_lexicon_tags_476 },
    { 0xefa7a098, SUC_lexicon_tags_80 },
    { 0x7f27a09a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebe7a09c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e47a09e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4387a0a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8927a0a9, SUC_lexicon_tags_7 },
    { 0x39e7a0aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f87a0b1, SUC_lexicon_tags_26 },
    { 0xe467a0b2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x55a7a0b4, SUC_lexicon_tags_11 },
    { 0xe947a0b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e7a0b8, SUC_lexicon_tags_45 },
    { 0xeb67a0b9, SUC_lexicon_tags_32 },
    { 0xbbc7a0ba, SUC_lexicon_tags_32 },
    { 0x867a0ba, SUC_lexicon_tags_16 },
    { 0x6307a0bc, SUC_lexicon_tags_17 },
    { 0x82e7a0bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c67a0c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c07a0c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1387a0c6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e7a0c9, SUC_lexicon_tags_12 },
    { 0x6b07a0ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe67a0cc, SUC_lexicon_tags_17 },
    { 0x7f07a0cc, SUC_lexicon_tags_17 },
    { 0x81c7a0ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b67a0d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a7a0d9, SUC_lexicon_tags_19 },
    { 0x6f07a0d9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x9427a0dd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x59c7a0df, SUC_lexicon_tags_17 },
    { 0xa2e7a0df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea7a0ee, SUC_lexicon_tags_10 },
    { 0x1bc7a0ef, SUC_lexicon_tags_1 },
    { 0x6a27a0f0, SUC_lexicon_tags_25 },
    { 0xe447a0f1, SUC_lexicon_tags_12 },
    { 0xe827a0f2, SUC_lexicon_tags_19 },
    { 0xe907a0ee, SUC_lexicon_tags_31 },
    { 0xc247a0f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2967a0f7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b87a0fe, SUC_lexicon_tags_0 },
    { 0x1b07a0ff, SUC_lexicon_tags_38 },
    { 0x5f07a100, SUC_lexicon_tags_48 },
    { 0x4187a101, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeca7a103, SUC_lexicon_tags_1 },
    { 0x5b67a104, SUC_lexicon_tags_9 },
    { 0x4087a104, SUC_lexicon_tags_17 },
    { 0x2ce7a106, SUC_lexicon_tags_8 },
    { 0xb207a107, SUC_lexicon_tags_47 },
    { 0x59c7a107, SUC_lexicon_tags_7 },
    { 0x1067a104, SUC_lexicon_tags_0 },
    { 0x9dc7a10a, SUC_lexicon_tags_37 },
    { 0x99c7a10b, SUC_lexicon_tags_28 },
    { 0x1807a10c, SUC_lexicon_tags_12 },
    { 0x9e67a10c, SUC_lexicon_tags_37 },
    { 0x67c7a10e, SUC_lexicon_tags_15 },
    { 0x7dc7a10b, SUC_lexicon_tags_33 },
    { 0x4ea7a110, SUC_lexicon_tags_31 },
    { 0xff67a10d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe7a114, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7667a118, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd507a11b, SUC_lexicon_tags_11 },
    { 0x1d47a11c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd87a11e, SUC_lexicon_tags_37 },
    { 0x4567a11f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e7a122, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c7a12c, SUC_lexicon_tags_60 },
    { 0xecc7a12d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e7a131, SUC_lexicon_tags_37 },
    { 0x9fe7a132, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc927a138, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a7a13b, SUC_lexicon_tags_2 },
    { 0xf687a13b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca87a141, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e7a148, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd107a14b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2427a14f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7607a151, SUC_lexicon_tags_11 },
    { 0xaa87a152, SUC_lexicon_tags_15 },
    { 0xff47a152, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5ce7a155, SUC_lexicon_tags_12 },
    { 0x5507a156, SUC_lexicon_tags_43 },
    { 0x2cc7a157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2947a15b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c07a160, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a47a167, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaf67a169, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc7a178, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc867a182, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5307a185, SUC_lexicon_tags_37 },
    { 0x6de7a186, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f27a18a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca67a18d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6627a18f, SUC_lexicon_tags_29 },
    { 0xa7a7a18f, SUC_lexicon_tags_0 },
    { 0x5d87a191, SUC_lexicon_tags_33 },
    { 0xebc7a191, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9487a194, SUC_lexicon_tags_14 },
    { 0xcc67a195, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1107a198, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7c07a19a, SUC_lexicon_tags_42 },
    { 0xd967a19b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8547a1a3, SUC_lexicon_tags_84 },
    { 0x9e87a1a3, SUC_lexicon_tags_66 },
    { 0xbc47a1a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4847a1a7, SUC_lexicon_tags_14 },
    { 0x6b87a1a8, SUC_lexicon_tags_0 },
    { 0xa807a1a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9007a1ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbd07a1ae, SUC_lexicon_tags_48 },
    { 0x2407a1af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e7a1b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefa7a1b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe7a1bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2827a1be, SUC_lexicon_tags_28 },
    { 0x8687a1bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e7a1c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6287a1c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd507a1c9, SUC_lexicon_tags_15 },
    { 0xaae7a1ca, SUC_lexicon_tags_42 },
    { 0xcb07a1ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc567a1ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7387a1d5, SUC_lexicon_tags_60 },
    { 0xbfa7a1d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7967a1d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a7a1dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c7a1e5, SUC_lexicon_tags_61 },
    { 0xbc7a1e6, SUC_lexicon_tags_35 },
    { 0x3c67a1e7, SUC_lexicon_tags_14 },
    { 0x7487a1e6, SUC_lexicon_tags_2 },
    { 0x43a7a1e8, SUC_lexicon_tags_17 },
    { 0x1ca7a1ea, SUC_lexicon_tags_9 },
    { 0xf707a1ea, SUC_lexicon_tags_51 },
    { 0x92c7a1ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5747a1ee, SUC_lexicon_tags_252 },
    { 0x45e7a1ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4847a1f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe127a1f7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d27a1fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae7a202, SUC_lexicon_tags_6 },
    { 0x8fe7a202, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6ac7a205, SUC_lexicon_tags_11 },
    { 0x4a27a206, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c87a208, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7427a20c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fa7a20e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea7a211, SUC_lexicon_tags_13 },
    { 0x5607a211, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xde87a214, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4267a216, SUC_lexicon_tags_2 },
    { 0x3a67a217, SUC_lexicon_tags_104 },
    { 0xd007a218, SUC_lexicon_tags_15 },
    { 0xe887a219, SUC_lexicon_tags_17 },
    { 0x1de7a217, SUC_lexicon_tags_37 },
    { 0x4f87a21a, SUC_lexicon_tags_15 },
    { 0xf887a217, SUC_lexicon_tags_25 },
    { 0x7bc7a21c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a7a221, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2627a224, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9787a230, SUC_lexicon_tags_7 },
    { 0xf547a231, SUC_lexicon_tags_17 },
    { 0x8887a232, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c7a235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa567a238, SUC_lexicon_tags_0 },
    { 0xc607a238, SUC_lexicon_tags_80 },
    { 0x8fa7a23a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35e7a23c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7667a23e, SUC_lexicon_tags_49 },
    { 0xae67a23f, SUC_lexicon_tags_33 },
    { 0xfa47a23f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf247a244, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb467a246, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4527a249, SUC_lexicon_tags_3 },
    { 0x9067a24a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf987a24c, SUC_lexicon_tags_27 },
    { 0x8de7a24d, SUC_lexicon_tags_12 },
    { 0x6be7a24e, SUC_lexicon_tags_15 },
    { 0x6447a24f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e7a254, SUC_lexicon_tags_7 },
    { 0x6ea7a255, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b07a259, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5047a261, SUC_lexicon_tags_46 },
    { 0x4b47a261, SUC_lexicon_tags_17 },
    { 0x7587a263, SUC_lexicon_tags_1 },
    { 0xc967a261, SUC_lexicon_tags_36 },
    { 0x7947a265, SUC_lexicon_tags_45 },
    { 0xb87a266, SUC_lexicon_tags_47 },
    { 0x3e27a266, SUC_lexicon_tags_7 },
    { 0xf9a7a268, SUC_lexicon_tags_13 },
    { 0xd667a261, SUC_lexicon_tags_49 },
    { 0xdf27a268, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e7a273, SUC_lexicon_tags_2 },
    { 0x8de7a273, SUC_lexicon_tags_25 },
    { 0xb067a275, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9b07a277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1747a27d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca67a280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf107a283, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc947a287, SUC_lexicon_tags_37 },
    { 0x7e67a288, SUC_lexicon_tags_5 },
    { 0x1e67a288, SUC_lexicon_tags_69 },
    { 0x40e7a289, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e7a28f, SUC_lexicon_tags_13 },
    { 0xefa7a290, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8587a294, SUC_lexicon_tags_15 },
    { 0x60c7a294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc7a299, SUC_lexicon_tags_37 },
    { 0xf8c7a29a, SUC_lexicon_tags_9 },
    { 0x1647a29a, SUC_lexicon_tags_0 },
    { 0x3687a299, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a7a2a0, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x4fa7a2a2, SUC_lexicon_tags_31 },
    { 0x5b07a2a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc407a2a6, SUC_lexicon_tags_96 },
    { 0x5707a2a7, SUC_lexicon_tags_30 },
    { 0xf647a2a6, SUC_lexicon_tags_43 },
    { 0x2d27a2a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7a2b2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac7a2b6, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a7a2bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x127a2bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e27a2c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d67a2c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb827a2c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8247a2cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c67a2cf, SUC_lexicon_tags_29 },
    { 0x8b47a2cf, SUC_lexicon_tags_37 },
    { 0xda47a2cf, SUC_lexicon_tags_37 },
    { 0x6847a2cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a87a2d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d07a2dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b47a2e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d87a2e3, SUC_lexicon_tags_15 },
    { 0xcc07a2e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e7a2e6, SUC_lexicon_tags_11 },
    { 0x8bc7a2e7, SUC_lexicon_tags_11 },
    { 0x5b67a2e8, SUC_lexicon_tags_63 },
    { 0x8087a2e8, SUC_lexicon_tags_22 },
    { 0x6327a2e9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee7a2ee, SUC_lexicon_tags_11 },
    { 0x9b87a2ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4127a2f3, SUC_lexicon_tags_25 },
    { 0x4447a2f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c47a2f7, SUC_lexicon_tags_7 },
    { 0x9187a2f8, SUC_lexicon_tags_48 },
    { 0xd627a2f9, SUC_lexicon_tags_15 },
    { 0xd287a2f9, SUC_lexicon_tags_7 },
    { 0x3ec7a2fa, SUC_lexicon_tags_31 },
    { 0x127a2fc, SUC_lexicon_tags_7 },
    { 0x2c27a2f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45a7a2ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be7a305, SUC_lexicon_tags_2 },
    { 0x607a306, SUC_lexicon_tags_1208 },
    { 0xf287a307, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf907a30c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67a311, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe127a316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5807a320, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a7a323, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc07a328, SUC_lexicon_tags_11 },
    { 0x17e7a329, SUC_lexicon_tags_37 },
    { 0x2107a32a, SUC_lexicon_tags_12 },
    { 0x6787a32a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2987a32d, SUC_lexicon_tags_12 },
    { 0xfb67a32e, SUC_lexicon_tags_79 },
    { 0x7a47a32e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb87a331, SUC_lexicon_tags_17 },
    { 0x29e7a331, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd347a336, SUC_lexicon_tags_11 },
    { 0x2967a337, SUC_lexicon_tags_7 },
    { 0x54c7a338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe527a33b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd87a33f, SUC_lexicon_tags_37 },
    { 0xfc7a340, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a7a344, SUC_lexicon_tags_1 },
    { 0xff67a345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6247a348, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47a34c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x73c7a34e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a7a352, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc7a358, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a7a35d, SUC_lexicon_tags_46 },
    { 0x9607a35e, SUC_lexicon_tags_32 },
    { 0x3767a35f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc747a361, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc7a366, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c07a36d, SUC_lexicon_tags_2 },
    { 0x2ac7a36e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d07a370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc07a377, SUC_lexicon_tags_17 },
    { 0xe8e7a377, SUC_lexicon_tags_48 },
    { 0x63a7a379, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee7a37c, SUC_lexicon_tags_15 },
    { 0xc307a37d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5007a37f, SUC_lexicon_tags_47 },
    { 0xa087a380, SUC_lexicon_tags_17 },
    { 0x8ac7a37f, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e7a386, SUC_lexicon_tags_26 },
    { 0x4847a387, SUC_lexicon_tags_8 },
    { 0x5c67a388, SUC_lexicon_tags_21 },
    { 0x33c7a388, SUC_lexicon_tags_37 },
    { 0xe767a38a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1ac7a38c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd647a38e, SUC_lexicon_tags_82 },
    { 0x18e7a38f, SUC_lexicon_tags_63 },
    { 0xbea7a390, SUC_lexicon_tags_13 },
    { 0x27e7a38e, SUC_lexicon_tags_46 },
    { 0x6e67a392, SUC_lexicon_tags_7 },
    { 0x8e47a393, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb087a395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1a7a397, SUC_lexicon_tags_45 },
    { 0x66e7a398, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7a39c, SUC_lexicon_tags_26 },
    { 0x2067a39d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67a3a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc7a3a9, SUC_lexicon_tags_14 },
    { 0x6e47a3aa, SUC_lexicon_tags_39 },
    { 0x1f67a3ab, SUC_lexicon_tags_19 },
    { 0x30c7a3ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49a7a3ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7a3b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x46a7a3b7, SUC_lexicon_tags_1 },
    { 0xcda7a3b7, SUC_lexicon_tags_1 },
    { 0xf167a3b9, SUC_lexicon_tags_7 },
    { 0xed07a3ba, SUC_lexicon_tags_60 },
    { 0xf087a3ba, SUC_lexicon_tags_79 },
    { 0x9f87a3bc, SUC_lexicon_tags_17 },
    { 0x3847a3ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a7a3c0, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e27a3c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b27a3c6, SUC_lexicon_tags_37 },
    { 0x5da7a3c7, SUC_lexicon_tags_7 },
    { 0xa3a7a3c8, SUC_lexicon_tags_15 },
    { 0x2287a3c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ea7a3cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf447a3cd, SUC_lexicon_tags_2 },
    { 0x2567a3cd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa027a3d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd307a3d4, SUC_lexicon_tags_15 },
    { 0x5ce7a3d4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47a3dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c27a3e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x33c7a3e2, SUC_lexicon_tags_7 },
    { 0x88e7a3e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3747a3e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd847a3ea, SUC_lexicon_tags_16 },
    { 0xba07a3eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa7a3ee, SUC_lexicon_tags_61 },
    { 0xedc7a3ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b47a3f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff47a3f8, SUC_lexicon_tags_63 },
    { 0x80c7a3f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97c7a3fb, SUC_lexicon_tags_4 },
    { 0xe5a7a3fc, SUC_lexicon_tags_8 },
    { 0xca47a3fc, SUC_lexicon_tags_37 },
    { 0x52c7a3fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e7a404, SUC_lexicon_tags_15 },
    { 0xd587a405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70a7a407, SUC_lexicon_tags_39 },
    { 0xee47a407, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f47a411, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967a416, SUC_lexicon_tags_45 },
    { 0xc487a416, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1747a41b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e7a41d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xade7a41f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x75c7a421, SUC_lexicon_tags_37 },
    { 0xb787a422, SUC_lexicon_tags_11 },
    { 0xc8c7a423, SUC_lexicon_tags_49 },
    { 0x6347a421, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3367a426, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f67a42b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8d87a42d, SUC_lexicon_tags_2 },
    { 0x75c7a42e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc927a430, SUC_lexicon_tags_7 },
    { 0xbec7a430, SUC_lexicon_tags_14 },
    { 0x4367a432, SUC_lexicon_tags_12 },
    { 0x42e7a433, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe7a436, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd47a438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c67a43d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b87a444, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x547a447, SUC_lexicon_tags_11 },
    { 0x7ce7a447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c7a44b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa07a44e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc307a451, SUC_lexicon_tags_31 },
    { 0x1447a451, SUC_lexicon_tags_1 },
    { 0xf727a452, SUC_lexicon_tags_77 },
    { 0x99e7a452, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb67a456, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d47a45e, SUC_lexicon_tags_17 },
    { 0xf6c7a45f, SUC_lexicon_tags_2 },
    { 0x6b27a460, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ca7a462, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37e7a465, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea7a468, SUC_lexicon_tags_11 },
    { 0xe27a469, SUC_lexicon_tags_15 },
    { 0x76a7a46a, SUC_lexicon_tags_83 },
    { 0xb207a468, SUC_lexicon_tags_57 },
    { 0x4c07a46c, SUC_lexicon_tags_77 },
    { 0xc1a7a46d, SUC_lexicon_tags_66 },
    { 0xe6a7a469, SUC_lexicon_tags_25 },
    { 0x3d27a46f, SUC_lexicon_tags_29 },
    { 0xb0a7a46a, SUC_lexicon_tags_37 },
    { 0xe6c7a46a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x51e7a473, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a7a476, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa7a47f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xace7a481, SUC_lexicon_tags_37 },
    { 0x84a7a481, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1c7a484, SUC_lexicon_tags_11 },
    { 0xb867a485, SUC_lexicon_tags_43 },
    { 0xe247a485, SUC_lexicon_tags_84 },
    { 0xcf47a487, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce7a489, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6547a48e, SUC_lexicon_tags_19 },
    { 0xf247a48e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07a493, SUC_lexicon_tags_27 },
    { 0xb147a494, SUC_lexicon_tags_48 },
    { 0x3e47a495, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf987a497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9c7a499, SUC_lexicon_tags_8 },
    { 0x1b67a49a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9127a49e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d67a4a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1107a4a8, SUC_lexicon_tags_23 },
    { 0x3fc7a4a9, SUC_lexicon_tags_23 },
    { 0xf667a4aa, SUC_lexicon_tags_47 },
    { 0x79e7a4aa, SUC_lexicon_tags_86 },
    { 0x32c7a4ab, SUC_lexicon_tags_37 },
    { 0x8707a4aa, SUC_lexicon_tags_37 },
    { 0xf987a4ae, SUC_lexicon_tags_8 },
    { 0x1647a4af, SUC_lexicon_tags_25 },
    { 0xff47a4af, SUC_lexicon_tags_37 },
    { 0x6647a4b1, SUC_lexicon_tags_15 },
    { 0x7b67a4b2, SUC_lexicon_tags_63 },
    { 0x9987a4b1, SUC_lexicon_tags_11 },
    { 0xcf27a4b2, SUC_lexicon_tags_1 },
    { 0x9607a4b5, SUC_lexicon_tags_38 },
    { 0xa407a4b6, SUC_lexicon_tags_2 },
    { 0xf3e7a4b7, SUC_lexicon_tags_46 },
    { 0x7727a4b8, SUC_lexicon_tags_12 },
    { 0x6407a4b6, SUC_lexicon_tags_10 },
    { 0x4b27a4ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa7a4bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9aa7a4c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xede7a4c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e7a4c8, SUC_lexicon_tags_136 },
    { 0x3d07a4c8, SUC_lexicon_tags_29 },
    { 0x8da7a4ca, SUC_lexicon_tags_12 },
    { 0x2a7a4cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3727a4ce, SUC_lexicon_tags_15 },
    { 0xb587a4cf, SUC_lexicon_tags_8 },
    { 0xc047a4d0, SUC_lexicon_tags_2 },
    { 0x9a67a4d0, SUC_lexicon_tags_37 },
    { 0x83a7a4cf, SUC_lexicon_tags_66 },
    { 0x9687a4d3, SUC_lexicon_tags_39 },
    { 0xce07a4d4, SUC_lexicon_tags_2 },
    { 0xbdc7a4d3, SUC_lexicon_tags_43 },
    { 0x3e87a4d6, SUC_lexicon_tags_29 },
    { 0xb2a7a4d0, SUC_lexicon_tags_16 },
    { 0x28c7a4d8, SUC_lexicon_tags_14 },
    { 0x1a27a4d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf047a4db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a7a4e5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xb1c7a4e7, SUC_lexicon_tags_16 },
    { 0xaa7a4e8, SUC_lexicon_tags_38 },
    { 0xdd47a4e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e7a4ec, SUC_lexicon_tags_38 },
    { 0x9247a4ed, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a7a4f1, SUC_lexicon_tags_65 },
    { 0x5ce7a4f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9387a4f4, SUC_lexicon_tags_23 },
    { 0x71c7a4f5, SUC_lexicon_tags_11 },
    { 0x6d87a4f5, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xa1e7a4f8, SUC_lexicon_tags_31 },
    { 0x6467a4f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a7a4ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49a7a501, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6c7a503, SUC_lexicon_tags_60 },
    { 0x58a7a504, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ca7a506, SUC_lexicon_tags_2 },
    { 0x1a47a506, SUC_lexicon_tags_11 },
    { 0x5aa7a508, SUC_lexicon_tags_45 },
    { 0x7727a507, SUC_lexicon_tags_25 },
    { 0xd1c7a507, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d87a50f, SUC_lexicon_tags_13 },
    { 0xf267a510, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae7a514, SUC_lexicon_tags_11 },
    { 0xada7a515, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef67a518, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3947a51e, SUC_lexicon_tags_33 },
    { 0x2287a51f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94c7a521, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e87a523, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbbc7a525, SUC_lexicon_tags_25 },
    { 0x2947a526, SUC_lexicon_tags_37 },
    { 0xaa27a527, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16e7a529, SUC_lexicon_tags_37 },
    { 0x11a7a52a, SUC_lexicon_tags_45 },
    { 0x8a07a52b, SUC_lexicon_tags_32 },
    { 0x787a52c, SUC_lexicon_tags_61 },
    { 0x72c7a52d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a7a530, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4547a535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3827a537, SUC_lexicon_tags_98 },
    { 0x1f47a537, SUC_lexicon_tags_48 },
    { 0xd0e7a538, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa327a53b, SUC_lexicon_tags_25 },
    { 0x7b87a53c, SUC_lexicon_tags_17 },
    { 0x8507a53d, SUC_lexicon_tags_7 },
    { 0xbac7a53d, SUC_lexicon_tags_25 },
    { 0xeac7a53f, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc7a543, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6667a546, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd567a548, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38a7a54a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xec27a54d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e7a555, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf287a558, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe447a55a, SUC_lexicon_tags_5 },
    { 0x8a07a55b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c07a55e, SUC_lexicon_tags_30 },
    { 0xa687a55f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce7a562, SUC_lexicon_tags_25 },
    { 0xcfe7a563, SUC_lexicon_tags_31 },
    { 0x8567a564, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e7a568, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c7a56f, SUC_lexicon_tags_11 },
    { 0xefa7a570, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff27a576, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f47a578, SUC_lexicon_tags_1 },
    { 0x9527a579, SUC_lexicon_tags_2 },
    { 0x6847a579, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa287a57e, SUC_lexicon_tags_9 },
    { 0xbf87a57e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x207a581, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf907a584, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5687a589, SUC_lexicon_tags_23 },
    { 0x87a589, SUC_lexicon_tags_82 },
    { 0xf7e7a58b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb567a58d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2c87a58f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc827a591, SUC_lexicon_tags_37 },
    { 0xf427a592, SUC_lexicon_tags_15 },
    { 0x4e07a593, SUC_lexicon_tags_0 },
    { 0x6f07a593, SUC_lexicon_tags_25 },
    { 0x947a591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58e7a597, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba7a59a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xef67a59d, SUC_lexicon_tags_14 },
    { 0xdde7a59e, SUC_lexicon_tags_33 },
    { 0x9ce7a59f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd667a5a2, SUC_lexicon_tags_71 },
    { 0x18e7a5a3, SUC_lexicon_tags_0 },
    { 0xc367a5a2, SUC_lexicon_tags_35 },
    { 0x8407a5a5, SUC_lexicon_tags_0 },
    { 0xa8a7a5a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb67a5a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e7a5ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4987a5b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7a5b6, SUC_lexicon_tags_37 },
    { 0xda67a5b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9507a5b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a7a5bf, SUC_lexicon_tags_13 },
    { 0xf8e7a5c0, SUC_lexicon_tags_12 },
    { 0xf687a5bf, SUC_lexicon_tags_10 },
    { 0xac7a5bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2247a5c6, SUC_lexicon_tags_1 },
    { 0x44e7a5c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7727a5cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a7a5d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9987a5df, SUC_lexicon_tags_0 },
    { 0xa5a7a5e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c7a5e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c7a5e8, SUC_lexicon_tags_11 },
    { 0x6cc7a5e9, SUC_lexicon_tags_37 },
    { 0x54e7a5ea, SUC_lexicon_tags_27 },
    { 0xa527a5eb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xec67a5ed, SUC_lexicon_tags_25 },
    { 0xfca7a5ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e7a5f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5947a5f6, SUC_lexicon_tags_47 },
    { 0x9547a5f6, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xf47a5f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8467a601, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a7a604, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa867a607, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x387a609, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35c7a60b, SUC_lexicon_tags_15 },
    { 0xcb67a60b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a07a611, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe027a613, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xcd27a615, SUC_lexicon_tags_17 },
    { 0xa587a616, SUC_lexicon_tags_0 },
    { 0xb647a616, SUC_lexicon_tags_7 },
    { 0x17e7a615, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd07a61c, SUC_lexicon_tags_17 },
    { 0xf0e7a61d, SUC_lexicon_tags_53 },
    { 0x3367a61c, SUC_lexicon_tags_11 },
    { 0xaf27a61f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6c7a621, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce7a624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3987a627, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaee7a629, SUC_lexicon_tags_56 },
    { 0x4ca7a62a, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0xa067a62d, SUC_lexicon_tags_2 },
    { 0x46c7a62e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6a7a630, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc587a635, SUC_lexicon_tags_37 },
    { 0xbda7a636, SUC_lexicon_tags_2 },
    { 0xf327a636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff67a63d, SUC_lexicon_tags_37 },
    { 0x6447a63d, SUC_lexicon_tags_9 },
    { 0x49a7a63f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa67a642, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf67a649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7367a64f, SUC_lexicon_tags_19 },
    { 0xde67a650, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c87a65a, SUC_lexicon_tags_9 },
    { 0x57e7a65b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a7a65e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4007a662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767a669, SUC_lexicon_tags_2 },
    { 0x2c87a66a, SUC_lexicon_tags_63 },
    { 0x6ac7a66b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa07a671, SUC_lexicon_tags_23 },
    { 0x8647a672, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade7a676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa647a67d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5167a682, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a7a686, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5687a689, SUC_lexicon_tags_198 },
    { 0x2e7a68a, SUC_lexicon_tags_44 },
    { 0x85e7a68b, SUC_lexicon_tags_2 },
    { 0xcd67a68a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c7a68f, SUC_lexicon_tags_0 },
    { 0x6567a690, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa067a694, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba7a697, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6187a69b, SUC_lexicon_tags_37 },
    { 0x3067a69c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea7a69f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f07a6a2, SUC_lexicon_tags_39 },
    { 0x7c87a6a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd147a6a5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xca67a6a7, SUC_lexicon_tags_6 },
    { 0x1307a6a8, SUC_lexicon_tags_9 },
    { 0xa507a6a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247a6b1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4147a6b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x787a6b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c7a6ba, SUC_lexicon_tags_49 },
    { 0xbc67a6ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf87a6be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c87a6c2, SUC_lexicon_tags_0 },
    { 0xfdc7a6c3, SUC_lexicon_tags_38 },
    { 0xc2e7a6c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x63c7a6c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a7a6cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf747a6d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c47a6d2, SUC_lexicon_tags_14 },
    { 0x6087a6d3, SUC_lexicon_tags_47 },
    { 0x45a7a6d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad07a6dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e67a6de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a7a6e3, SUC_lexicon_tags_45 },
    { 0x4fa7a6e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c47a6e7, SUC_lexicon_tags_25 },
    { 0xaaa7a6e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e27a6ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c7a6f1, SUC_lexicon_tags_234 },
    { 0x7667a6f2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7a6f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8107a6f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac7a6fb, SUC_lexicon_tags_45 },
    { 0x3447a6fc, SUC_lexicon_tags_19 },
    { 0x98e7a6fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a7a701, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5767a706, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d47a709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe447a70b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c67a70e, SUC_lexicon_tags_11 },
    { 0xbce7a70f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2187a711, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6de7a713, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf947a715, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe207a718, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c87a71f, SUC_lexicon_tags_37 },
    { 0x6807a720, SUC_lexicon_tags_1 },
    { 0x2e87a720, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57a7a723, SUC_lexicon_tags_62 },
    { 0x7d87a724, SUC_lexicon_tags_15 },
    { 0xd667a724, SUC_lexicon_tags_9 },
    { 0xaec7a726, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc667a728, SUC_lexicon_tags_33 },
    { 0xb247a728, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa887a72b, SUC_lexicon_tags_31 },
    { 0xa647a72c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3547a72e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a7a731, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xb227a734, SUC_lexicon_tags_9 },
    { 0x1247a734, SUC_lexicon_tags_45 },
    { 0xd3c7a736, SUC_lexicon_tags_24 },
    { 0x65e7a734, SUC_lexicon_tags_9 },
    { 0x4327a738, SUC_lexicon_tags_37 },
    { 0x81c7a737, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a7a73d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe7a741, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c7a749, SUC_lexicon_tags_34 },
    { 0xb227a74a, SUC_lexicon_tags_30 },
    { 0x9007a74b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf267a756, SUC_lexicon_tags_17 },
    { 0x5687a756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa667a759, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a7a75d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xba27a760, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52a7a762, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x13c7a764, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c7a767, SUC_lexicon_tags_15 },
    { 0x8b47a768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd67a76a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c7a770, SUC_lexicon_tags_37 },
    { 0x7267a771, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x12a7a773, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe7a777, SUC_lexicon_tags_37 },
    { 0xada7a778, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea7a781, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa5a7a783, SUC_lexicon_tags_13 },
    { 0xb0e7a784, SUC_lexicon_tags_17 },
    { 0xe607a785, SUC_lexicon_tags_81 },
    { 0xb487a786, SUC_lexicon_tags_2 },
    { 0xb327a785, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a7a78c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c7a78f, SUC_lexicon_tags_30 },
    { 0xfca7a790, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2327a792, SUC_lexicon_tags_9 },
    { 0xd787a792, SUC_lexicon_tags_7 },
    { 0x9ca7a794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6c7a796, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b67a799, SUC_lexicon_tags_25 },
    { 0xce67a79a, SUC_lexicon_tags_32 },
    { 0xf147a79b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x3987a79d, SUC_lexicon_tags_19 },
    { 0x5347a79d, SUC_lexicon_tags_37 },
    { 0x2e87a79f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de7a7aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19a7a7ac, SUC_lexicon_tags_7 },
    { 0x48c7a7ac, SUC_lexicon_tags_11 },
    { 0xd807a7ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9327a7b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x607a7b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9327a7b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd587a7ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1507a7be, SUC_lexicon_tags_26 },
    { 0xad07a7bf, SUC_lexicon_tags_37 },
    { 0xab67a7bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9867a7c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a7a7c5, SUC_lexicon_tags_55 },
    { 0x9627a7c6, SUC_lexicon_tags_23 },
    { 0x66c7a7c7, SUC_lexicon_tags_65 },
    { 0x25c7a7c8, SUC_lexicon_tags_7 },
    { 0x4427a7c7, SUC_lexicon_tags_28 },
    { 0x5d07a7c8, SUC_lexicon_tags_37 },
    { 0x11c7a7cb, SUC_lexicon_tags_17 },
    { 0x1a07a7cb, SUC_lexicon_tags_0 },
    { 0x2fa7a7cc, SUC_lexicon_tags_25 },
    { 0xe1a7a7cd, SUC_lexicon_tags_1 },
    { 0xc767a7cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4987a7d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba47a7d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd27a7df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a7a7e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd727a7e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe267a7e7, SUC_lexicon_tags_12 },
    { 0xc5e7a7e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc7a7ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc7a7f3, SUC_lexicon_tags_1 },
    { 0x407a7f4, SUC_lexicon_tags_37 },
    { 0x74e7a7f5, SUC_lexicon_tags_33 },
    { 0x86e7a7f6, SUC_lexicon_tags_26 },
    { 0xef67a7f3, SUC_lexicon_tags_41 },
    { 0x73e7a7f4, SUC_lexicon_tags_23 },
    { 0x2ce7a7f3, SUC_lexicon_tags_37 },
    { 0x3607a7fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3527a7fe, SUC_lexicon_tags_11 },
    { 0x8d07a7ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda27a805, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c27a807, SUC_lexicon_tags_23 },
    { 0xff47a808, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda67a80a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b27a80c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e7a810, SUC_lexicon_tags_1 },
    { 0x1ba7a811, SUC_lexicon_tags_25 },
    { 0x7607a812, SUC_lexicon_tags_29 },
    { 0x6967a813, SUC_lexicon_tags_8 },
    { 0xa967a810, SUC_lexicon_tags_33 },
    { 0x1cc7a814, SUC_lexicon_tags_11 },
    { 0x67a7a814, SUC_lexicon_tags_33 },
    { 0x5127a816, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a7a81d, SUC_lexicon_tags_10 },
    { 0x8c07a81e, SUC_lexicon_tags_25 },
    { 0x1627a81f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e7a822, SUC_lexicon_tags_37 },
    { 0x38c7a823, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3767a826, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9087a82b, SUC_lexicon_tags_11 },
    { 0xcb07a82c, SUC_lexicon_tags_1 },
    { 0x8fc7a82d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c7a830, SUC_lexicon_tags_30 },
    { 0xb347a831, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd127a836, SUC_lexicon_tags_15 },
    { 0xfd27a837, SUC_lexicon_tags_31 },
    { 0x99a7a838, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a7a83c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeca7a83e, SUC_lexicon_tags_17 },
    { 0xb9a7a83f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4207a848, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x327a84a, SUC_lexicon_tags_9 },
    { 0xef67a84b, SUC_lexicon_tags_47 },
    { 0xb167a84c, SUC_lexicon_tags_19 },
    { 0x2847a84a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2307a84f, SUC_lexicon_tags_15 },
    { 0xcf07a84f, SUC_lexicon_tags_12 },
    { 0x2c87a851, SUC_lexicon_tags_17 },
    { 0x1b87a852, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3da7a854, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e07a856, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d67a85b, SUC_lexicon_tags_47 },
    { 0x1da7a85c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d07a860, SUC_lexicon_tags_15 },
    { 0x9ba7a861, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6987a867, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d67a86b, SUC_lexicon_tags_27 },
    { 0x8c67a86b, SUC_lexicon_tags_11 },
    { 0x347a86b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c7a871, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9327a879, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea7a87e, SUC_lexicon_tags_31 },
    { 0xaae7a87e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2187a884, SUC_lexicon_tags_11 },
    { 0x7027a885, SUC_lexicon_tags_25 },
    { 0x22a7a886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a7a88e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb387a893, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc8c7a895, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e7a898, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf87a89c, SUC_lexicon_tags_1 },
    { 0x89a7a89c, SUC_lexicon_tags_11 },
    { 0xd5e7a89e, SUC_lexicon_tags_7 },
    { 0x83a7a89f, SUC_lexicon_tags_0 },
    { 0x2487a89f, SUC_lexicon_tags_17 },
    { 0x1b47a8a1, SUC_lexicon_tags_47 },
    { 0xb507a8a0, SUC_lexicon_tags_31 },
    { 0x547a8a3, SUC_lexicon_tags_19 },
    { 0xf8a7a89c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a87a8a6, SUC_lexicon_tags_37 },
    { 0x6a07a8a7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x3d47a8a9, SUC_lexicon_tags_7 },
    { 0x6227a8aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d07a8ac, SUC_lexicon_tags_63 },
    { 0xb307a8ad, SUC_lexicon_tags_49 },
    { 0x6747a8ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3087a8b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb607a8b8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a7a8c1, SUC_lexicon_tags_25 },
    { 0xda47a8c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x17a7a8c4, SUC_lexicon_tags_39 },
    { 0x2687a8c5, SUC_lexicon_tags_10 },
    { 0x2c07a8c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6987a8c8, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff27a8d1, SUC_lexicon_tags_12 },
    { 0x7a27a8d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c07a8da, SUC_lexicon_tags_17 },
    { 0x2427a8db, SUC_lexicon_tags_17 },
    { 0x23a7a8dc, SUC_lexicon_tags_6 },
    { 0x4d47a8dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e47a8df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda07a8e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc2a7a8e3, SUC_lexicon_tags_7 },
    { 0xa3e7a8e3, SUC_lexicon_tags_572 },
    { 0xd8c7a8e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa947a8e7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xc2c7a8e9, SUC_lexicon_tags_7 },
    { 0x24e7a8ea, SUC_lexicon_tags_37 },
    { 0x9867a8eb, SUC_lexicon_tags_37 },
    { 0xd327a8ec, SUC_lexicon_tags_37 },
    { 0x2227a8ed, SUC_lexicon_tags_55 },
    { 0x2ae7a8ec, SUC_lexicon_tags_25 },
    { 0x12a7a8ef, SUC_lexicon_tags_43 },
    { 0x5127a8f0, SUC_lexicon_tags_1 },
    { 0x2dc7a8f0, SUC_lexicon_tags_26 },
    { 0xf9a7a8f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea7a8f7, SUC_lexicon_tags_25 },
    { 0x8747a8f8, SUC_lexicon_tags_7 },
    { 0x9b27a8f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde7a8ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb947a901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa047a905, SUC_lexicon_tags_45 },
    { 0xfbc7a906, SUC_lexicon_tags_7 },
    { 0xf5e7a906, SUC_lexicon_tags_12 },
    { 0x667a908, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3907a90d, SUC_lexicon_tags_47 },
    { 0x78e7a90d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa847a910, SUC_lexicon_tags_17 },
    { 0xf2e7a910, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe2e7a913, SUC_lexicon_tags_47 },
    { 0x53e7a914, SUC_lexicon_tags_12 },
    { 0xb7a7a914, SUC_lexicon_tags_12 },
    { 0x3d27a915, SUC_lexicon_tags_14 },
    { 0xac47a917, SUC_lexicon_tags_39 },
    { 0xc967a917, SUC_lexicon_tags_60 },
    { 0xdd27a919, SUC_lexicon_tags_55 },
    { 0x1827a917, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7507a91f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfe07a921, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2127a923, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7627a926, SUC_lexicon_tags_25 },
    { 0xa047a927, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7027a929, SUC_lexicon_tags_79 },
    { 0xccc7a92a, SUC_lexicon_tags_3 },
    { 0x2ac7a92b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae7a92e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75c7a930, SUC_lexicon_tags_19 },
    { 0xe347a931, SUC_lexicon_tags_84 },
    { 0x4007a931, SUC_lexicon_tags_17 },
    { 0xd627a931, SUC_lexicon_tags_15 },
    { 0x8247a934, SUC_lexicon_tags_68 },
    { 0xb347a934, SUC_lexicon_tags_750 },
    { 0xe1e7a932, SUC_lexicon_tags_17 },
    { 0xde87a937, SUC_lexicon_tags_47 },
    { 0xce47a935, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b07a941, SUC_lexicon_tags_1 },
    { 0x4b27a942, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf07a952, SUC_lexicon_tags_7 },
    { 0xe267a952, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc607a955, SUC_lexicon_tags_33 },
    { 0xb427a956, SUC_lexicon_tags_15 },
    { 0x9cc7a957, SUC_lexicon_tags_12 },
    { 0x4b27a955, SUC_lexicon_tags_2 },
    { 0xdc67a959, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5787a95d, SUC_lexicon_tags_31 },
    { 0x6de7a95e, SUC_lexicon_tags_21 },
    { 0xdce7a95f, SUC_lexicon_tags_21 },
    { 0x2547a95e, SUC_lexicon_tags_95 },
    { 0x76e7a961, SUC_lexicon_tags_70 },
    { 0x9727a961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f07a969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc747a971, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e7a976, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7547a97c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf367a983, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8967a985, SUC_lexicon_tags_11 },
    { 0x9167a986, SUC_lexicon_tags_7 },
    { 0x3cc7a987, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf807a98a, SUC_lexicon_tags_5 },
    { 0xa67a98a, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x2727a98d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cc7a98f, SUC_lexicon_tags_66 },
    { 0x827a990, SUC_lexicon_tags_32 },
    { 0xc47a991, SUC_lexicon_tags_45 },
    { 0xa8a7a990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8667a99b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x14e7a99d, SUC_lexicon_tags_25 },
    { 0x59e7a99e, SUC_lexicon_tags_7 },
    { 0xb607a99d, SUC_lexicon_tags_2 },
    { 0x86c7a9a0, SUC_lexicon_tags_6 },
    { 0x83c7a99f, SUC_lexicon_tags_11 },
    { 0xbfc7a9a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5947a9a4, SUC_lexicon_tags_19 },
    { 0x27a7a9a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd187a9aa, SUC_lexicon_tags_14 },
    { 0xcb07a9aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf387a9ad, SUC_lexicon_tags_0 },
    { 0xc647a9ae, SUC_lexicon_tags_31 },
    { 0x6327a9ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2487a9b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a7a9bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc7a9c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a7a9cb, SUC_lexicon_tags_5 },
    { 0xc187a9cc, SUC_lexicon_tags_14 },
    { 0x57e7a9cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0c7a9cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf07a9d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c7a9d8, SUC_lexicon_tags_37 },
    { 0xfa7a9d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a7a9dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee07a9de, SUC_lexicon_tags_12 },
    { 0x5967a9de, SUC_lexicon_tags_68 },
    { 0xfc67a9df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae27a9e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7a9e7, SUC_lexicon_tags_7 },
    { 0x4087a9e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5107a9ee, SUC_lexicon_tags_12 },
    { 0xbb27a9ef, SUC_lexicon_tags_26 },
    { 0x5ec7a9f0, SUC_lexicon_tags_39 },
    { 0x33c7a9f1, SUC_lexicon_tags_12 },
    { 0xcd07a9f2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf967a9f4, SUC_lexicon_tags_38 },
    { 0xe3c7a9f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba07a9f7, SUC_lexicon_tags_2 },
    { 0x7d67a9f8, SUC_lexicon_tags_60 },
    { 0xdd67a9f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x267a9fb, SUC_lexicon_tags_32 },
    { 0xbb87a9fc, SUC_lexicon_tags_7 },
    { 0xd127a9fd, SUC_lexicon_tags_46 },
    { 0x3d47a9fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec67aa02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e7aa06, SUC_lexicon_tags_74 },
    { 0xbc07aa07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c67aa0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae7aa16, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc587aa19, SUC_lexicon_tags_42 },
    { 0x367aa1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27aa20, SUC_lexicon_tags_14 },
    { 0xbde7aa21, SUC_lexicon_tags_82 },
    { 0xcf87aa20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1187aa26, SUC_lexicon_tags_25 },
    { 0x4d67aa27, SUC_lexicon_tags_87 },
    { 0x67a7aa27, SUC_lexicon_tags_37 },
    { 0xb127aa28, SUC_lexicon_tags_4 },
    { 0xc1c7aa2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd047aa2e, SUC_lexicon_tags_14 },
    { 0xce27aa2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb27aa31, SUC_lexicon_tags_43 },
    { 0x7f07aa31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb727aa35, SUC_lexicon_tags_15 },
    { 0x77e7aa36, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd567aa38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4287aa3c, SUC_lexicon_tags_1 },
    { 0xcb47aa3c, SUC_lexicon_tags_7 },
    { 0xd5e7aa3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb487aa46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44e7aa48, SUC_lexicon_tags_22 },
    { 0x1567aa49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c7aa4d, SUC_lexicon_tags_2 },
    { 0x5ec7aa4e, SUC_lexicon_tags_42 },
    { 0xec67aa4d, SUC_lexicon_tags_25 },
    { 0x3a67aa50, SUC_lexicon_tags_37 },
    { 0x37c7aa50, SUC_lexicon_tags_47 },
    { 0xfe07aa51, SUC_lexicon_tags_12 },
    { 0x4187aa53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb67aa58, SUC_lexicon_tags_46 },
    { 0x127aa59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7067aa61, SUC_lexicon_tags_23 },
    { 0x5887aa62, SUC_lexicon_tags_37 },
    { 0xd787aa63, SUC_lexicon_tags_37 },
    { 0xc3c7aa61, SUC_lexicon_tags_12 },
    { 0x5fc7aa65, SUC_lexicon_tags_27 },
    { 0x2e67aa62, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a7aa6e, SUC_lexicon_tags_27 },
    { 0xaf27aa6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7aa75, SUC_lexicon_tags_6 },
    { 0x4327aa76, SUC_lexicon_tags_7 },
    { 0x7247aa75, SUC_lexicon_tags_23 },
    { 0x7647aa77, SUC_lexicon_tags_5 },
    { 0x2f27aa79, SUC_lexicon_tags_25 },
    { 0xbb47aa79, SUC_lexicon_tags_65 },
    { 0x1047aa7b, SUC_lexicon_tags_82 },
    { 0xb967aa7c, SUC_lexicon_tags_27 },
    { 0x5227aa7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb87aa82, SUC_lexicon_tags_27 },
    { 0x4ba7aa83, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8bc7aa85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7067aa89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc7aa8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a7aa90, SUC_lexicon_tags_37 },
    { 0x1b87aa90, SUC_lexicon_tags_37 },
    { 0xed07aa92, SUC_lexicon_tags_37 },
    { 0x8e07aa93, SUC_lexicon_tags_12 },
    { 0xf27aa93, SUC_lexicon_tags_11 },
    { 0x627aa93, SUC_lexicon_tags_15 },
    { 0x2d47aa96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5927aa98, SUC_lexicon_tags_27 },
    { 0xf6c7aa98, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x77c7aa9b, SUC_lexicon_tags_12 },
    { 0xb787aa9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1247aa9e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa7aaa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7427aaa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c7aaac, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed87aab1, SUC_lexicon_tags_60 },
    { 0x7687aab2, SUC_lexicon_tags_31 },
    { 0x5267aab2, SUC_lexicon_tags_37 },
    { 0xd167aab3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9347aab7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a7aabb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7aabe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a47aac2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67aad2, SUC_lexicon_tags_37 },
    { 0x81c7aad3, SUC_lexicon_tags_128 },
    { 0x1127aad3, SUC_lexicon_tags_55 },
    { 0xf567aad5, SUC_lexicon_tags_7 },
    { 0x3307aad6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b87aadd, SUC_lexicon_tags_32 },
    { 0xa0e7aade, SUC_lexicon_tags_2 },
    { 0x1147aadf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff47aae6, SUC_lexicon_tags_25 },
    { 0x6387aae6, SUC_lexicon_tags_47 },
    { 0x4707aae6, SUC_lexicon_tags_11 },
    { 0x3b67aae9, SUC_lexicon_tags_38 },
    { 0xe2c7aaea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f47aaf0, SUC_lexicon_tags_1 },
    { 0x6347aaf1, SUC_lexicon_tags_16 },
    { 0x4ee7aaf1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb407aaf4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae7aafa, SUC_lexicon_tags_7 },
    { 0x3ee7aafb, SUC_lexicon_tags_25 },
    { 0x81e7aafb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa7aaff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5387ab02, SUC_lexicon_tags_38 },
    { 0x5807ab02, SUC_lexicon_tags_2 },
    { 0x48e7ab02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf87ab06, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a7ab0c, SUC_lexicon_tags_2 },
    { 0xa787ab0d, SUC_lexicon_tags_15 },
    { 0x8667ab0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90e7ab10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf547ab12, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9767ab1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc7ab1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xab07ab22, SUC_lexicon_tags_15 },
    { 0x1027ab23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c47ab26, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c7ab29, SUC_lexicon_tags_235 },
    { 0x8907ab2a, SUC_lexicon_tags_11 },
    { 0x9447ab2b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa587ab2f, SUC_lexicon_tags_25 },
    { 0x7887ab30, SUC_lexicon_tags_2 },
    { 0x9c87ab31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8527ab3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4847ab43, SUC_lexicon_tags_17 },
    { 0x4ea7ab44, SUC_lexicon_tags_7 },
    { 0x3767ab43, SUC_lexicon_tags_58 },
    { 0x9887ab46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf2c7ab48, SUC_lexicon_tags_1605 },
    { 0xb247ab49, SUC_lexicon_tags_19 },
    { 0xcd67ab4a, SUC_lexicon_tags_27 },
    { 0xf967ab4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a67ab4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe987ab4f, SUC_lexicon_tags_1 },
    { 0x5b07ab50, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e7ab56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5567ab59, SUC_lexicon_tags_49 },
    { 0xc727ab5a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a27ab5d, SUC_lexicon_tags_1 },
    { 0x3547ab5d, SUC_lexicon_tags_66 },
    { 0x2d07ab5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7ab6e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8847ab77, SUC_lexicon_tags_37 },
    { 0x2847ab77, SUC_lexicon_tags_19 },
    { 0xa6a7ab79, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb987ab7b, SUC_lexicon_tags_17 },
    { 0xa8a7ab7b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a7ab7f, SUC_lexicon_tags_33 },
    { 0x36a7ab80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe7ab82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a7ab85, SUC_lexicon_tags_19 },
    { 0x9707ab85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6707ab8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1bc7ab90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xafe7ab92, SUC_lexicon_tags_12 },
    { 0x4287ab93, SUC_lexicon_tags_15 },
    { 0x36e7ab94, SUC_lexicon_tags_37 },
    { 0x8387ab93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cc7ab97, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3827ab99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c7ab9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea7aba1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4587aba6, SUC_lexicon_tags_11 },
    { 0x8687aba7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa387aba9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c27abab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a27abae, SUC_lexicon_tags_33 },
    { 0x2f87abaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f07abbc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf007abbf, SUC_lexicon_tags_37 },
    { 0x8e47abbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc307abc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe927abc7, SUC_lexicon_tags_17 },
    { 0xed87abc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ce7abca, SUC_lexicon_tags_47 },
    { 0x9507abcb, SUC_lexicon_tags_35 },
    { 0x4fc7abcc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa247abcf, SUC_lexicon_tags_1 },
    { 0xac27abcf, SUC_lexicon_tags_1 },
    { 0xe407abcf, SUC_lexicon_tags_2 },
    { 0x9fc7abcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7c7abd4, SUC_lexicon_tags_29 },
    { 0xeca7abd5, SUC_lexicon_tags_267 },
    { 0xb327abd6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf367abda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6727abe3, SUC_lexicon_tags_2 },
    { 0x7ca7abe4, SUC_lexicon_tags_27 },
    { 0xf867abe5, SUC_lexicon_tags_25 },
    { 0x7b67abe5, SUC_lexicon_tags_38 },
    { 0x21c7abe6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd787abe9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4127abeb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5727abed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac7abf1, SUC_lexicon_tags_2 },
    { 0x4de7abf2, SUC_lexicon_tags_16 },
    { 0x82c7abf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a7abfe, SUC_lexicon_tags_12 },
    { 0xfc67abff, SUC_lexicon_tags_7 },
    { 0xfb27ac00, SUC_lexicon_tags_61 },
    { 0x24e7ac00, SUC_lexicon_tags_66 },
    { 0xcd67ac02, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd687ac04, SUC_lexicon_tags_0 },
    { 0x4407ac05, SUC_lexicon_tags_9 },
    { 0x4ba7ac05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c7ac0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407ac11, SUC_lexicon_tags_38 },
    { 0xa3c7ac11, SUC_lexicon_tags_77 },
    { 0xaac7ac13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e07ac16, SUC_lexicon_tags_25 },
    { 0x44e7ac17, SUC_lexicon_tags_60 },
    { 0xeca7ac18, SUC_lexicon_tags_14 },
    { 0xcde7ac19, SUC_lexicon_tags_14 },
    { 0x8a27ac17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2807ac1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdce7ac1f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9fc7ac21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fc7ac23, SUC_lexicon_tags_17 },
    { 0xfe7ac23, SUC_lexicon_tags_45 },
    { 0xbae7ac25, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf427ac27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a47ac29, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x3787ac2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6e7ac2d, SUC_lexicon_tags_62 },
    { 0xd847ac2e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e7ac31, SUC_lexicon_tags_11 },
    { 0xb287ac31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7067ac34, SUC_lexicon_tags_61 },
    { 0xf27ac35, SUC_lexicon_tags_91 },
    { 0x90c7ac36, SUC_lexicon_tags_12 },
    { 0x9907ac37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb147ac3c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a07ac45, SUC_lexicon_tags_21 },
    { 0x6687ac46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a7ac49, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba7ac4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b87ac51, SUC_lexicon_tags_17 },
    { 0x8e47ac52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1967ac54, SUC_lexicon_tags_38 },
    { 0x43c7ac55, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd27ac58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa387ac5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b87ac5f, SUC_lexicon_tags_81 },
    { 0x9647ac60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x43c7ac62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc27ac68, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce7ac6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b07ac71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3327ac74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9107ac77, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xce27ac7a, SUC_lexicon_tags_2 },
    { 0x9b67ac7b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f47ac81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6427ac84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb1a7ac86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b27ac88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c27ac8a, SUC_lexicon_tags_11 },
    { 0x51e7ac8a, SUC_lexicon_tags_11 },
    { 0x5c47ac8c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xe027ac8e, SUC_lexicon_tags_11 },
    { 0xce47ac8f, SUC_lexicon_tags_31 },
    { 0xb8c7ac90, SUC_lexicon_tags_1 },
    { 0xce7ac91, SUC_lexicon_tags_17 },
    { 0xd927ac92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd187ac95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6507ac97, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87aca2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e7aca5, SUC_lexicon_tags_7 },
    { 0xaa87aca6, SUC_lexicon_tags_26 },
    { 0x27a7aca6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a7acab, SUC_lexicon_tags_38 },
    { 0xa087acab, SUC_lexicon_tags_87 },
    { 0x5867acad, SUC_lexicon_tags_37 },
    { 0x6507acae, SUC_lexicon_tags_17 },
    { 0x7847acad, SUC_lexicon_tags_34 },
    { 0x10e7acac, SUC_lexicon_tags_10 },
    { 0xeaa7acac, SUC_lexicon_tags_33 },
    { 0xf887acad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e7acb8, SUC_lexicon_tags_17 },
    { 0x8a27acb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd2c7acbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd887acbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7947acc2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x78c7acc4, SUC_lexicon_tags_37 },
    { 0x76c7acc4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc6c7acc7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a47accb, SUC_lexicon_tags_15 },
    { 0x6c7accb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9987acce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf387acd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8627acd2, SUC_lexicon_tags_37 },
    { 0x9767acd3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x64e7acd5, SUC_lexicon_tags_43 },
    { 0x4aa7acd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b67acd9, SUC_lexicon_tags_229 },
    { 0xaba7acd9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x49c7acdc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa7ace5, SUC_lexicon_tags_194 },
    { 0xcee7ace5, SUC_lexicon_tags_19 },
    { 0x43a7ace5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c7acec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb847acf7, SUC_lexicon_tags_59 },
    { 0x94e7acf8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2227acfb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1007acff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67ad08, SUC_lexicon_tags_2 },
    { 0xcd87ad09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd967ad0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a7ad0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7587ad11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae7ad14, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x1847ad17, SUC_lexicon_tags_525 },
    { 0x3d27ad18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe167ad1a, SUC_lexicon_tags_40 },
    { 0x3e47ad1b, SUC_lexicon_tags_8 },
    { 0x5d27ad1a, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0xe6a7ad1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d47ad22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a7ad25, SUC_lexicon_tags_33 },
    { 0xf027ad26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7ad2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b07ad2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf1c7ad2f, SUC_lexicon_tags_7 },
    { 0x1407ad30, SUC_lexicon_tags_63 },
    { 0x5b07ad31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba7ad36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87ad39, SUC_lexicon_tags_28 },
    { 0xac67ad3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde7ad3f, SUC_lexicon_tags_17 },
    { 0xc0c7ad40, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf747ad42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7ad47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7ad4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c07ad4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4c07ad50, SUC_lexicon_tags_46 },
    { 0x9867ad51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647ad55, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcb67ad57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c7ad5a, SUC_lexicon_tags_25 },
    { 0x9d27ad5b, SUC_lexicon_tags_33 },
    { 0xbca7ad5c, SUC_lexicon_tags_17 },
    { 0x3f47ad5b, SUC_lexicon_tags_11 },
    { 0xdc07ad5e, SUC_lexicon_tags_32 },
    { 0xa087ad5e, SUC_lexicon_tags_47 },
    { 0x6e07ad5a, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e7ad63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4767ad65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c27ad69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x427ad6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6347ad70, SUC_lexicon_tags_45 },
    { 0xfc87ad70, SUC_lexicon_tags_35 },
    { 0x7447ad72, SUC_lexicon_tags_12 },
    { 0x2387ad72, SUC_lexicon_tags_37 },
    { 0xc0a7ad74, SUC_lexicon_tags_1088 },
    { 0x74a7ad73, SUC_lexicon_tags_15 },
    { 0x6227ad76, SUC_lexicon_tags_47 },
    { 0x21a7ad76, SUC_lexicon_tags_19 },
    { 0x8c67ad78, SUC_lexicon_tags_65 },
    { 0xf8c7ad79, SUC_lexicon_tags_37 },
    { 0x8c7ad75, SUC_lexicon_tags_15 },
    { 0x6807ad7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5587ad80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae7ad84, SUC_lexicon_tags_77 },
    { 0x25a7ad84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf727ad89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8a7ad8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1107ad8d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e27ad90, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7fc7ad92, SUC_lexicon_tags_46 },
    { 0xdba7ad93, SUC_lexicon_tags_14 },
    { 0xc887ad94, SUC_lexicon_tags_45 },
    { 0x6f47ad94, SUC_lexicon_tags_19 },
    { 0x26a7ad96, SUC_lexicon_tags_60 },
    { 0x3aa7ad97, SUC_lexicon_tags_37 },
    { 0x7b07ad98, SUC_lexicon_tags_16 },
    { 0x2827ad97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7ad9e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x55a7ada0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1527ada3, SUC_lexicon_tags_57 },
    { 0xfcc7ada3, SUC_lexicon_tags_0 },
    { 0x9507ada4, SUC_lexicon_tags_11 },
    { 0x46c7ada6, SUC_lexicon_tags_18 },
    { 0x69c7ada4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a7adae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d87adb0, SUC_lexicon_tags_17 },
    { 0x7747adb1, SUC_lexicon_tags_15 },
    { 0xde67adb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e7adba, SUC_lexicon_tags_29 },
    { 0x5287adbb, SUC_lexicon_tags_40 },
    { 0xc847adbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb087adbf, SUC_lexicon_tags_11 },
    { 0x8b07adc0, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xc747adc2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x367adc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdf87adc6, SUC_lexicon_tags_11 },
    { 0x31c7adc7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf007adca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae7adcf, SUC_lexicon_tags_53 },
    { 0x25a7add0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc247add2, SUC_lexicon_tags_2 },
    { 0x1487add3, SUC_lexicon_tags_30 },
    { 0xd327add4, SUC_lexicon_tags_12 },
    { 0xcb87add3, SUC_lexicon_tags_10 },
    { 0x8267add4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa5a7add8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c7ade3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee7ade7, SUC_lexicon_tags_17 },
    { 0x8287ade8, SUC_lexicon_tags_25 },
    { 0xdf27ade9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1987adef, SUC_lexicon_tags_11 },
    { 0x9487adf0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa7adf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7adfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8147adfc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c7ae01, SUC_lexicon_tags_7 },
    { 0x7027ae01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa927ae0a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1527ae0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e7ae0f, SUC_lexicon_tags_15 },
    { 0xc6a7ae10, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd307ae13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe7ae1a, SUC_lexicon_tags_49 },
    { 0x88e7ae1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd567ae1d, SUC_lexicon_tags_47 },
    { 0xf747ae1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf387ae20, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a7ae23, SUC_lexicon_tags_45 },
    { 0x8d47ae23, SUC_lexicon_tags_858 },
    { 0xa427ae25, SUC_lexicon_tags_25 },
    { 0x97c7ae26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab67ae28, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3307ae2b, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0xac27ae2d, SUC_lexicon_tags_37 },
    { 0x7b47ae2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea7ae32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae87ae37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7007ae39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a7ae3e, SUC_lexicon_tags_38 },
    { 0xfaa7ae3f, SUC_lexicon_tags_25 },
    { 0x4727ae3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d47ae44, SUC_lexicon_tags_9 },
    { 0x63c7ae45, SUC_lexicon_tags_47 },
    { 0x2447ae46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be7ae49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac7ae4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5087ae51, SUC_lexicon_tags_11 },
    { 0xa387ae52, SUC_lexicon_tags_37 },
    { 0x98c7ae51, SUC_lexicon_tags_8 },
    { 0x3c47ae54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72e7ae56, SUC_lexicon_tags_14 },
    { 0x48c7ae57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe887ae5a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf967ae5c, SUC_lexicon_tags_39 },
    { 0x3b67ae5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb027ae61, SUC_lexicon_tags_17 },
    { 0x40e7ae62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c7ae68, SUC_lexicon_tags_17 },
    { 0xa047ae69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf47ae6d, SUC_lexicon_tags_12 },
    { 0xece7ae6e, SUC_lexicon_tags_28 },
    { 0xcc87ae6f, SUC_lexicon_tags_63 },
    { 0x1b07ae6e, SUC_lexicon_tags_15 },
    { 0x9807ae71, SUC_lexicon_tags_12 },
    { 0xcdc7ae72, SUC_lexicon_tags_15 },
    { 0x6867ae73, SUC_lexicon_tags_37 },
    { 0xb7c7ae72, SUC_lexicon_tags_29 },
    { 0xfee7ae75, SUC_lexicon_tags_518 },
    { 0x547ae75, SUC_lexicon_tags_38 },
    { 0xf5c7ae73, SUC_lexicon_tags_7 },
    { 0x7c67ae78, SUC_lexicon_tags_82 },
    { 0x9e27ae6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a7ae7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3347ae82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ae88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5387ae8a, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0x30a7ae8c, SUC_lexicon_tags_6 },
    { 0x9f87ae8c, SUC_lexicon_tags_17 },
    { 0xb2c7ae8e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5567ae96, SUC_lexicon_tags_14 },
    { 0x1947ae97, SUC_lexicon_tags_39 },
    { 0xd4c7ae97, SUC_lexicon_tags_23 },
    { 0x4ba7ae99, SUC_lexicon_tags_25 },
    { 0x2127ae9a, SUC_lexicon_tags_12 },
    { 0xa27ae9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5227ae9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc7aea2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed07aea4, SUC_lexicon_tags_47 },
    { 0x4a67aea5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc7aeaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84e7aeb1, SUC_lexicon_tags_47 },
    { 0x5967aeb2, SUC_lexicon_tags_23 },
    { 0xf887aeb3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc7aeb8, SUC_lexicon_tags_68 },
    { 0xaae7aeb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a67aebc, SUC_lexicon_tags_13 },
    { 0x40c7aebd, SUC_lexicon_tags_15 },
    { 0xaea7aebe, SUC_lexicon_tags_31 },
    { 0x89e7aebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1247aec2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1187aec4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae7aec7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c07aece, SUC_lexicon_tags_30 },
    { 0x2447aecf, SUC_lexicon_tags_47 },
    { 0x8547aed0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc7aed3, SUC_lexicon_tags_25 },
    { 0x5ec7aed4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca7aeda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0e7aedc, SUC_lexicon_tags_1 },
    { 0xb487aedd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa067aee0, SUC_lexicon_tags_375 },
    { 0xa7c7aee1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c07aee3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c7aeec, SUC_lexicon_tags_6 },
    { 0xbb07aeed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc7aef0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc387aef2, SUC_lexicon_tags_17 },
    { 0xe487aef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a47aef5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe247aefb, SUC_lexicon_tags_11 },
    { 0xa87aefc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6207aefe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea7af01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x85c7af03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf707af09, SUC_lexicon_tags_1 },
    { 0xf067af09, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e07af0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c7af10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c7af15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfba7af17, SUC_lexicon_tags_2 },
    { 0xf707af17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c7af1b, SUC_lexicon_tags_38 },
    { 0xcf87af1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae27af23, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f87af28, SUC_lexicon_tags_37 },
    { 0x6687af28, SUC_lexicon_tags_11 },
    { 0xc067af2a, SUC_lexicon_tags_13 },
    { 0x3ee7af2a, SUC_lexicon_tags_17 },
    { 0xee47af2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d47af2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1307af31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe067af34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc7af39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3447af3d, SUC_lexicon_tags_75 },
    { 0x7787af3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa547af40, SUC_lexicon_tags_57 },
    { 0x6367af41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e7af45, SUC_lexicon_tags_2 },
    { 0x7847af46, SUC_lexicon_tags_25 },
    { 0x1c27af45, SUC_lexicon_tags_1 },
    { 0xb267af45, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2d07af4a, SUC_lexicon_tags_25 },
    { 0x5107af4a, SUC_lexicon_tags_39 },
    { 0x367af4c, SUC_lexicon_tags_25 },
    { 0x3647af4d, SUC_lexicon_tags_38 },
    { 0xebe7af4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d47af51, SUC_lexicon_tags_26 },
    { 0x3d27af52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a7af55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e7af59, SUC_lexicon_tags_23 },
    { 0xb667af5a, SUC_lexicon_tags_2 },
    { 0x8bc7af59, SUC_lexicon_tags_2 },
    { 0xa627af5c, SUC_lexicon_tags_11 },
    { 0x3e47af5c, SUC_lexicon_tags_39 },
    { 0xc087af5e, SUC_lexicon_tags_35 },
    { 0x1027af5e, SUC_lexicon_tags_31 },
    { 0x3567af5f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xeca7af62, SUC_lexicon_tags_140 },
    { 0x8647af62, SUC_lexicon_tags_15 },
    { 0xa1e7af62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e7af67, SUC_lexicon_tags_25 },
    { 0xbd47af68, SUC_lexicon_tags_19 },
    { 0x8047af69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca7af6f, SUC_lexicon_tags_26 },
    { 0xd4a7af70, SUC_lexicon_tags_8 },
    { 0x6487af71, SUC_lexicon_tags_37 },
    { 0x5287af71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ba7af74, SUC_lexicon_tags_1 },
    { 0x3727af75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e67af7a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xba87af7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc847af7e, SUC_lexicon_tags_34 },
    { 0x9a7af7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7527af86, SUC_lexicon_tags_11 },
    { 0x7cc7af87, SUC_lexicon_tags_6 },
    { 0x267af86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a7af8c, SUC_lexicon_tags_12 },
    { 0x9947af8d, SUC_lexicon_tags_1137 },
    { 0xd6e7af8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c7af92, SUC_lexicon_tags_25 },
    { 0x1ec7af93, SUC_lexicon_tags_2 },
    { 0x41e7af94, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec7af9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43e7af9c, SUC_lexicon_tags_16 },
    { 0x2b27af9d, SUC_lexicon_tags_8 },
    { 0xade7af9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70e7afa0, SUC_lexicon_tags_37 },
    { 0xa0e7afa1, SUC_lexicon_tags_28 },
    { 0xa867afa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c7afaf, SUC_lexicon_tags_68 },
    { 0xeb87afb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8ec7afb2, SUC_lexicon_tags_11 },
    { 0x26c7afb3, SUC_lexicon_tags_13 },
    { 0x40e7afb4, SUC_lexicon_tags_49 },
    { 0xc7c7afb5, SUC_lexicon_tags_60 },
    { 0x13a7afb6, SUC_lexicon_tags_37 },
    { 0xe427afb7, SUC_lexicon_tags_38 },
    { 0x8ce7afb6, SUC_lexicon_tags_7 },
    { 0x5407afb6, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc7afbc, SUC_lexicon_tags_31 },
    { 0x94e7afbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e27afc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a7afc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee07afce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc47afd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa7afd4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x61c7afd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd47afdd, SUC_lexicon_tags_46 },
    { 0x3787afde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x887afe1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3927afe3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd647afe6, SUC_lexicon_tags_0 },
    { 0x4967afe7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xee47afe9, SUC_lexicon_tags_17 },
    { 0x94e7afe9, SUC_lexicon_tags_21 },
    { 0x2147afea, SUC_lexicon_tags_7 },
    { 0x8f47afea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c7afef, SUC_lexicon_tags_23 },
    { 0xe4a7aff0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa7affa, SUC_lexicon_tags_4 },
    { 0xdfa7affb, SUC_lexicon_tags_25 },
    { 0x7747affc, SUC_lexicon_tags_17 },
    { 0xc387affd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f07b002, SUC_lexicon_tags_26 },
    { 0x3787b002, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb547b006, SUC_lexicon_tags_55 },
    { 0xb47b006, SUC_lexicon_tags_12 },
    { 0x4627b006, SUC_lexicon_tags_1 },
    { 0x8ba7b008, SUC_lexicon_tags_141 },
    { 0x6247b006, SUC_lexicon_tags_39 },
    { 0x7207b007, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5867b011, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbe7b013, SUC_lexicon_tags_21 },
    { 0x74c7b013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac67b016, SUC_lexicon_tags_13 },
    { 0xb847b016, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa927b019, SUC_lexicon_tags_11 },
    { 0xd747b01a, SUC_lexicon_tags_65 },
    { 0x847b019, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c7b01e, SUC_lexicon_tags_15 },
    { 0x2687b01f, SUC_lexicon_tags_37 },
    { 0x9847b01f, SUC_lexicon_tags_15 },
    { 0xf787b021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c7b028, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c7b02e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b27b035, SUC_lexicon_tags_47 },
    { 0x1a27b036, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe687b038, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba7b040, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9a27b042, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da7b048, SUC_lexicon_tags_60 },
    { 0x9d07b049, SUC_lexicon_tags_31 },
    { 0xbd87b049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b87b04e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb007b053, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce7b056, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8767b05a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1887b05c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc967b05e, SUC_lexicon_tags_37 },
    { 0x1bc7b05e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6227b063, SUC_lexicon_tags_45 },
    { 0xd0a7b064, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a7b06d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4587b072, SUC_lexicon_tags_7 },
    { 0x2e87b073, SUC_lexicon_tags_27 },
    { 0x6c87b074, SUC_lexicon_tags_11 },
    { 0x10c7b072, SUC_lexicon_tags_2 },
    { 0x3967b076, SUC_lexicon_tags_11 },
    { 0xa0c7b077, SUC_lexicon_tags_7 },
    { 0x9307b078, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e07b080, SUC_lexicon_tags_49 },
    { 0x4827b081, SUC_lexicon_tags_7 },
    { 0x2dc7b082, SUC_lexicon_tags_37 },
    { 0x5767b081, SUC_lexicon_tags_7 },
    { 0x9dc7b080, SUC_lexicon_tags_37 },
    { 0xb447b081, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe827b08a, SUC_lexicon_tags_17 },
    { 0x3027b08b, SUC_lexicon_tags_12 },
    { 0xc1e7b08b, SUC_lexicon_tags_2 },
    { 0x3567b08a, SUC_lexicon_tags_57 },
    { 0x34c7b08e, SUC_lexicon_tags_14 },
    { 0x6cc7b08f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad67b093, SUC_lexicon_tags_45 },
    { 0x3d47b093, SUC_lexicon_tags_11 },
    { 0x887b093, SUC_lexicon_tags_17 },
    { 0xd3e7b096, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x8507b099, SUC_lexicon_tags_21 },
    { 0x6b27b09a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc7b0a0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a07b0a7, SUC_lexicon_tags_17 },
    { 0x6b27b0a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa367b0ab, SUC_lexicon_tags_25 },
    { 0x9447b0ac, SUC_lexicon_tags_23 },
    { 0xe67b0ad, SUC_lexicon_tags_1 },
    { 0x4f47b0ad, SUC_lexicon_tags_7 },
    { 0x3a47b0af, SUC_lexicon_tags_9 },
    { 0x3327b0b0, SUC_lexicon_tags_115 },
    { 0x7f07b0b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc007b0b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x527b0b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf447b0b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7e7b0bb, SUC_lexicon_tags_65 },
    { 0x6087b0bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4547b0be, SUC_lexicon_tags_37 },
    { 0x36c7b0bf, SUC_lexicon_tags_7 },
    { 0x8d87b0be, SUC_lexicon_tags_60 },
    { 0x4b27b0c1, SUC_lexicon_tags_1 },
    { 0x11c7b0c1, SUC_lexicon_tags_1 },
    { 0x4367b0c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e67b0c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3007b0c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c7b0cb, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0x5307b0cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a87b0cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea7b0d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40e7b0d5, SUC_lexicon_tags_23 },
    { 0x4747b0d6, SUC_lexicon_tags_25 },
    { 0x5ae7b0d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d87b0db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc07b0de, SUC_lexicon_tags_25 },
    { 0x48a7b0de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe7b0e4, SUC_lexicon_tags_43 },
    { 0x9687b0e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e7b0e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf727b0ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1307b0ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e7b0f1, SUC_lexicon_tags_21 },
    { 0x4727b0f2, SUC_lexicon_tags_17 },
    { 0x8867b0f3, SUC_lexicon_tags_29 },
    { 0xed47b0f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a7b0f9, SUC_lexicon_tags_17 },
    { 0xf867b0fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e7b100, SUC_lexicon_tags_34 },
    { 0x38c7b101, SUC_lexicon_tags_37 },
    { 0x4847b102, SUC_lexicon_tags_1 },
    { 0x5f27b103, SUC_lexicon_tags_9 },
    { 0xd8c7b104, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c7b107, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6487b10b, SUC_lexicon_tags_11 },
    { 0x6427b10b, SUC_lexicon_tags_11 },
    { 0x8ce7b10c, SUC_lexicon_tags_13 },
    { 0x84c7b10e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b07b112, SUC_lexicon_tags_6 },
    { 0x2e27b113, SUC_lexicon_tags_2 },
    { 0xb327b114, SUC_lexicon_tags_17 },
    { 0xa547b114, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe087b119, SUC_lexicon_tags_17 },
    { 0x2287b119, SUC_lexicon_tags_17 },
    { 0x9f87b119, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a7b11e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4487b120, SUC_lexicon_tags_37 },
    { 0x97a7b121, SUC_lexicon_tags_1 },
    { 0x8787b121, SUC_lexicon_tags_2 },
    { 0x5147b123, SUC_lexicon_tags_12 },
    { 0x5447b124, SUC_lexicon_tags_12 },
    { 0x1ca7b125, SUC_lexicon_tags_63 },
    { 0x3a47b126, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac47b128, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8327b12a, SUC_lexicon_tags_17 },
    { 0xc247b12a, SUC_lexicon_tags_1 },
    { 0xd3a7b12c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec47b12f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xe027b132, SUC_lexicon_tags_47 },
    { 0x4727b133, SUC_lexicon_tags_0 },
    { 0x96e7b134, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc7b137, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c7b13b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a7b141, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb67b143, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac7b146, SUC_lexicon_tags_11 },
    { 0x2087b147, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1667b149, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc807b153, SUC_lexicon_tags_17 },
    { 0x38a7b153, SUC_lexicon_tags_17 },
    { 0xb5a7b155, SUC_lexicon_tags_15 },
    { 0x16c7b154, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef47b15c, SUC_lexicon_tags_2 },
    { 0xfe27b15d, SUC_lexicon_tags_37 },
    { 0xe8c7b15e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4747b161, SUC_lexicon_tags_9 },
    { 0xb007b161, SUC_lexicon_tags_25 },
    { 0x6b07b163, SUC_lexicon_tags_8 },
    { 0x8b87b164, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4407b167, SUC_lexicon_tags_45 },
    { 0x25a7b168, SUC_lexicon_tags_65 },
    { 0xc5a7b169, SUC_lexicon_tags_45 },
    { 0x1d67b16a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4e7b16c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f67b171, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde7b175, SUC_lexicon_tags_23 },
    { 0xf847b176, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf027b179, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2327b17d, SUC_lexicon_tags_11 },
    { 0x9e67b17e, SUC_lexicon_tags_2 },
    { 0xf227b17e, SUC_lexicon_tags_31 },
    { 0x1167b17d, SUC_lexicon_tags_47 },
    { 0x2d07b17d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e7b184, SUC_lexicon_tags_1 },
    { 0x9f47b185, SUC_lexicon_tags_387 },
    { 0x7547b186, SUC_lexicon_tags_46 },
    { 0x9d87b185, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a7b18c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5387b18e, SUC_lexicon_tags_1 },
    { 0x70c7b18f, SUC_lexicon_tags_25 },
    { 0x9987b190, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab07b192, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a7b19d, SUC_lexicon_tags_6 },
    { 0xb47b19d, SUC_lexicon_tags_15 },
    { 0xd3c7b19f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8387b1a7, SUC_lexicon_tags_15 },
    { 0xe307b1a8, SUC_lexicon_tags_63 },
    { 0x2727b1a9, SUC_lexicon_tags_45 },
    { 0xade7b1a9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c27b1ae, SUC_lexicon_tags_45 },
    { 0x9b87b1af, SUC_lexicon_tags_31 },
    { 0xe427b1ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1c7b1b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9387b1b5, SUC_lexicon_tags_17 },
    { 0xa127b1b6, SUC_lexicon_tags_32 },
    { 0x4927b1b7, SUC_lexicon_tags_15 },
    { 0x83c7b1b8, SUC_lexicon_tags_66 },
    { 0x9ee7b1b8, SUC_lexicon_tags_33 },
    { 0x5867b1b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1be7b1bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6067b1bf, SUC_lexicon_tags_33 },
    { 0xef27b1c0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb3c7b1c2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d67b1cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2de7b1cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee7b1d0, SUC_lexicon_tags_26 },
    { 0x32e7b1d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7a7b1d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe67b1d5, SUC_lexicon_tags_56 },
    { 0x9c27b1d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2da7b1d8, SUC_lexicon_tags_1 },
    { 0x1607b1d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7367b1dd, SUC_lexicon_tags_14 },
    { 0x3427b1de, SUC_lexicon_tags_30 },
    { 0xc107b1df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3907b1e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e7b1e9, SUC_lexicon_tags_136 },
    { 0x6c47b1ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x11e7b1ec, SUC_lexicon_tags_17 },
    { 0x6d67b1ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc47b1f1, SUC_lexicon_tags_47 },
    { 0x1ee7b1f2, SUC_lexicon_tags_47 },
    { 0x49a7b1f2, SUC_lexicon_tags_37 },
    { 0x1b47b1f4, SUC_lexicon_tags_11 },
    { 0xd27b1f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6287b1f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5667b1f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7b1fd, SUC_lexicon_tags_80 },
    { 0x7647b1fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf87b200, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a7b203, SUC_lexicon_tags_685 },
    { 0, NULL },
    { 0xfc67b205, SUC_lexicon_tags_38 },
    { 0x1847b206, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc7b20c, SUC_lexicon_tags_37 },
    { 0x3f87b20c, SUC_lexicon_tags_48 },
    { 0x5e7b20c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ca7b210, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a87b212, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0x627b215, SUC_lexicon_tags_47 },
    { 0xc547b216, SUC_lexicon_tags_49 },
    { 0xe827b215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97a7b219, SUC_lexicon_tags_15 },
    { 0xb6e7b21a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e7b21f, SUC_lexicon_tags_15 },
    { 0xf4c7b21f, SUC_lexicon_tags_25 },
    { 0x3907b221, SUC_lexicon_tags_16 },
    { 0x4947b220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e07b225, SUC_lexicon_tags_7 },
    { 0xeba7b226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d87b229, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbc7b22b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x76e7b22d, SUC_lexicon_tags_7 },
    { 0x12c7b22e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9947b238, SUC_lexicon_tags_37 },
    { 0xd067b239, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9987b23c, SUC_lexicon_tags_48 },
    { 0x8927b23d, SUC_lexicon_tags_7 },
    { 0xf087b23e, SUC_lexicon_tags_0 },
    { 0xb667b23f, SUC_lexicon_tags_7 },
    { 0x6c47b23e, SUC_lexicon_tags_0 },
    { 0x5327b23f, SUC_lexicon_tags_21 },
    { 0x2967b240, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xecc7b244, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a7b24a, SUC_lexicon_tags_39 },
    { 0xaa67b24b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa4e7b24d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9347b24f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b67b252, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa747b256, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xada7b259, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2767b25b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe587b25d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d27b260, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea87b263, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da7b266, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe47b26c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd067b274, SUC_lexicon_tags_15 },
    { 0x39c7b275, SUC_lexicon_tags_46 },
    { 0x7b07b276, SUC_lexicon_tags_49 },
    { 0x88c7b277, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e7b27a, SUC_lexicon_tags_39 },
    { 0x52e7b27b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b07b286, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec7b28a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a7b28e, SUC_lexicon_tags_37 },
    { 0x1de7b28e, SUC_lexicon_tags_23 },
    { 0xe107b28e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c7b293, SUC_lexicon_tags_0 },
    { 0xda7b294, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc87b296, SUC_lexicon_tags_3 },
    { 0xb527b296, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xd2c7b299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20e7b29b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4e7b29d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b27b2a0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c27b2a3, SUC_lexicon_tags_1 },
    { 0xf87b2a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c7b2a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ea7b2a9, SUC_lexicon_tags_63 },
    { 0x70c7b2a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb007b2af, SUC_lexicon_tags_17 },
    { 0xb8a7b2b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167b2b7, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d67b2be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac07b2c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c7b2c5, SUC_lexicon_tags_19 },
    { 0xd0e7b2c6, SUC_lexicon_tags_37 },
    { 0xf387b2c7, SUC_lexicon_tags_33 },
    { 0x4a27b2c8, SUC_lexicon_tags_25 },
    { 0xa947b2c8, SUC_lexicon_tags_11 },
    { 0x2507b2c9, SUC_lexicon_tags_23 },
    { 0x4de7b2cb, SUC_lexicon_tags_12 },
    { 0x17a7b2cc, SUC_lexicon_tags_2 },
    { 0xee87b2cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9807b2d2, SUC_lexicon_tags_30 },
    { 0x2487b2d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86c7b2d5, SUC_lexicon_tags_9 },
    { 0xc387b2d5, SUC_lexicon_tags_25 },
    { 0x38e7b2d6, SUC_lexicon_tags_37 },
    { 0x61a7b2d8, SUC_lexicon_tags_17 },
    { 0x8fc7b2d6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde7b2e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b67b2e3, SUC_lexicon_tags_14 },
    { 0xc0c7b2e4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e7b2e7, SUC_lexicon_tags_55 },
    { 0xd07b2e8, SUC_lexicon_tags_17 },
    { 0x84c7b2e7, SUC_lexicon_tags_25 },
    { 0x5807b2ea, SUC_lexicon_tags_19 },
    { 0xebe7b2eb, SUC_lexicon_tags_63 },
    { 0x4067b2eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a87b2f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4c67b2f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x867b2f6, SUC_lexicon_tags_435 },
    { 0xe807b2f7, SUC_lexicon_tags_33 },
    { 0x99a7b2f8, SUC_lexicon_tags_65 },
    { 0x7b2f9, SUC_lexicon_tags_7 },
    { 0x18a7b2fa, SUC_lexicon_tags_49 },
    { 0x2ae7b2fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a47b301, SUC_lexicon_tags_23 },
    { 0x3d67b302, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c07b305, SUC_lexicon_tags_9 },
    { 0xbe47b306, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd107b30d, SUC_lexicon_tags_45 },
    { 0x8587b30d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc7b312, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e7b314, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a7b318, SUC_lexicon_tags_12 },
    { 0xc047b318, SUC_lexicon_tags_1 },
    { 0x15a7b31a, SUC_lexicon_tags_12 },
    { 0xa2e7b31b, SUC_lexicon_tags_12 },
    { 0x4467b31c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e67b31e, SUC_lexicon_tags_30 },
    { 0xaa27b31f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf27b321, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb3e7b323, SUC_lexicon_tags_7 },
    { 0xf8e7b324, SUC_lexicon_tags_117 },
    { 0x87c7b324, SUC_lexicon_tags_17 },
    { 0xbc67b326, SUC_lexicon_tags_8 },
    { 0x3347b327, SUC_lexicon_tags_14 },
    { 0x93c7b326, SUC_lexicon_tags_70 },
    { 0xcd27b329, SUC_lexicon_tags_29 },
    { 0x2447b32a, SUC_lexicon_tags_25 },
    { 0xe367b325, SUC_lexicon_tags_27 },
    { 0x8887b32c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc947b331, SUC_lexicon_tags_7 },
    { 0x9287b332, SUC_lexicon_tags_7 },
    { 0x7f07b331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a7b33b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a7b33e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf27b340, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec7b34d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a7b352, SUC_lexicon_tags_31 },
    { 0xf7a7b353, SUC_lexicon_tags_11 },
    { 0x9b87b352, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7b357, SUC_lexicon_tags_17 },
    { 0x24c7b357, SUC_lexicon_tags_19 },
    { 0x36e7b359, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2267b35c, SUC_lexicon_tags_60 },
    { 0x16a7b35d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a7b362, SUC_lexicon_tags_23 },
    { 0x5747b362, SUC_lexicon_tags_8 },
    { 0x54e7b363, SUC_lexicon_tags_79 },
    { 0xd507b363, SUC_lexicon_tags_25 },
    { 0x86e7b366, SUC_lexicon_tags_27 },
    { 0x54e7b366, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8727b36c, SUC_lexicon_tags_35 },
    { 0x32e7b36d, SUC_lexicon_tags_43 },
    { 0xe207b36d, SUC_lexicon_tags_60 },
    { 0xb447b36e, SUC_lexicon_tags_98 },
    { 0xb107b36e, SUC_lexicon_tags_12 },
    { 0xefa7b371, SUC_lexicon_tags_15 },
    { 0xd747b36c, SUC_lexicon_tags_27 },
    { 0x2e87b36e, SUC_lexicon_tags_103 },
    { 0x7a87b371, SUC_lexicon_tags_1 },
    { 0x42c7b375, SUC_lexicon_tags_47 },
    { 0xa8e7b375, SUC_lexicon_tags_12 },
    { 0xf6a7b36e, SUC_lexicon_tags_213 },
    { 0x5927b378, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9747b37b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8c47b37d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7227b38c, SUC_lexicon_tags_49 },
    { 0xe8a7b38d, SUC_lexicon_tags_55 },
    { 0x1ee7b38e, SUC_lexicon_tags_42 },
    { 0x3087b38f, SUC_lexicon_tags_11 },
    { 0xfac7b38f, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x99c7b392, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a67b394, SUC_lexicon_tags_45 },
    { 0x8787b395, SUC_lexicon_tags_2 },
    { 0x6327b395, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be7b399, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd807b39d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9087b3a2, SUC_lexicon_tags_63 },
    { 0x5487b3a2, SUC_lexicon_tags_26 },
    { 0x5c07b3a4, SUC_lexicon_tags_21 },
    { 0x4147b3a4, SUC_lexicon_tags_7 },
    { 0xcc7b3a6, SUC_lexicon_tags_11 },
    { 0xbce7b3a3, SUC_lexicon_tags_28 },
    { 0xf287b3a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb227b3aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f87b3b1, SUC_lexicon_tags_1 },
    { 0x56c7b3b1, SUC_lexicon_tags_17 },
    { 0x6027b3b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1747b3b9, SUC_lexicon_tags_2 },
    { 0xace7b3ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb827b3be, SUC_lexicon_tags_25 },
    { 0xdaa7b3bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4047b3c2, SUC_lexicon_tags_15 },
    { 0x5aa7b3c2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e07b3c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe47b3c8, SUC_lexicon_tags_11 },
    { 0x2aa7b3c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca7b3cf, SUC_lexicon_tags_11 },
    { 0x36e7b3cf, SUC_lexicon_tags_7 },
    { 0x9fa7b3d1, SUC_lexicon_tags_22 },
    { 0xeb67b3d2, SUC_lexicon_tags_60 },
    { 0xe067b3d3, SUC_lexicon_tags_47 },
    { 0x4087b3cf, SUC_lexicon_tags_4 },
    { 0xef67b3d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27b3dc, SUC_lexicon_tags_0 },
    { 0x3427b3dc, SUC_lexicon_tags_1 },
    { 0xb227b3dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b07b3e0, SUC_lexicon_tags_39 },
    { 0x8307b3e1, SUC_lexicon_tags_13 },
    { 0x3ce7b3e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567b3e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe327b3eb, SUC_lexicon_tags_37 },
    { 0x9a47b3ec, SUC_lexicon_tags_49 },
    { 0x3a47b3ed, SUC_lexicon_tags_28 },
    { 0x1567b3ec, SUC_lexicon_tags_10 },
    { 0x4b67b3ec, SUC_lexicon_tags_1 },
    { 0x5907b3f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18c7b3f2, SUC_lexicon_tags_55 },
    { 0xfe87b3f3, SUC_lexicon_tags_12 },
    { 0x9487b3f2, SUC_lexicon_tags_33 },
    { 0x3267b3f5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c7b3f9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e7b403, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2467b405, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf047b408, SUC_lexicon_tags_15 },
    { 0xab07b409, SUC_lexicon_tags_25 },
    { 0x7887b409, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92c7b40c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc667b414, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a67b417, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5567b419, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x8927b41b, SUC_lexicon_tags_25 },
    { 0x7ca7b41b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf387b424, SUC_lexicon_tags_37 },
    { 0x5a27b425, SUC_lexicon_tags_11 },
    { 0x5ae7b426, SUC_lexicon_tags_1 },
    { 0x8827b427, SUC_lexicon_tags_45 },
    { 0x8c27b424, SUC_lexicon_tags_12 },
    { 0x42a7b425, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7b42d, SUC_lexicon_tags_7 },
    { 0x5287b42e, SUC_lexicon_tags_25 },
    { 0xeec7b42e, SUC_lexicon_tags_1 },
    { 0x44e7b430, SUC_lexicon_tags_65 },
    { 0x1027b430, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9227b434, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc87b436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d87b438, SUC_lexicon_tags_11 },
    { 0xc1c7b439, SUC_lexicon_tags_29 },
    { 0x75c7b43a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4727b43e, SUC_lexicon_tags_66 },
    { 0x7f67b43f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4547b442, SUC_lexicon_tags_43 },
    { 0xa827b442, SUC_lexicon_tags_23 },
    { 0xe747b444, SUC_lexicon_tags_17 },
    { 0xa5c7b445, SUC_lexicon_tags_25 },
    { 0xcfc7b444, SUC_lexicon_tags_1 },
    { 0xeea7b447, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6787b449, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca7b44c, SUC_lexicon_tags_47 },
    { 0x26a7b44d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaa07b44f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x37c7b451, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa9e7b453, SUC_lexicon_tags_29 },
    { 0xd447b454, SUC_lexicon_tags_43 },
    { 0x46a7b454, SUC_lexicon_tags_63 },
    { 0x9187b456, SUC_lexicon_tags_15 },
    { 0x9687b455, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a7b45a, SUC_lexicon_tags_25 },
    { 0xc487b45a, SUC_lexicon_tags_15 },
    { 0x42a7b45c, SUC_lexicon_tags_53 },
    { 0x4667b45d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e7b460, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf947b464, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1607b46a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c07b46d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c27b477, SUC_lexicon_tags_37 },
    { 0xfc87b478, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8b07b47a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5147b47d, SUC_lexicon_tags_10 },
    { 0x407b47d, SUC_lexicon_tags_30 },
    { 0x2f47b47e, SUC_lexicon_tags_45 },
    { 0x1827b480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa527b482, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2707b484, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0xd207b486, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb27b48e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7667b491, SUC_lexicon_tags_63 },
    { 0xfd47b491, SUC_lexicon_tags_25 },
    { 0x20e7b491, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc767b499, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a7b49d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6bc7b49f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd347b4a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f47b4a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3c07b4a8, SUC_lexicon_tags_3 },
    { 0x8867b4a8, SUC_lexicon_tags_11 },
    { 0x1fa7b4aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6147b4b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x347b4b4, SUC_lexicon_tags_4 },
    { 0x3cc7b4b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4267b4bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c47b4c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdd67b4c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9267b4c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7127b4d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a07b4d3, SUC_lexicon_tags_19 },
    { 0x9427b4d3, SUC_lexicon_tags_39 },
    { 0xb7e7b4d5, SUC_lexicon_tags_25 },
    { 0x9aa7b4d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b47b4d8, SUC_lexicon_tags_25 },
    { 0xd1c7b4d9, SUC_lexicon_tags_45 },
    { 0xd9c7b4d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e7b4dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1b07b4df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8027b4e2, SUC_lexicon_tags_23 },
    { 0xfc27b4e3, SUC_lexicon_tags_6 },
    { 0xe907b4e4, SUC_lexicon_tags_1 },
    { 0x7ea7b4e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e07b4e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa147b4eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7de7b4ed, SUC_lexicon_tags_33 },
    { 0xc5e7b4ee, SUC_lexicon_tags_43 },
    { 0x6627b4ee, SUC_lexicon_tags_12 },
    { 0xf4e7b4f0, SUC_lexicon_tags_37 },
    { 0xe947b4f0, SUC_lexicon_tags_12 },
    { 0x8267b4ef, SUC_lexicon_tags_27 },
    { 0x24a7b4f3, SUC_lexicon_tags_34 },
    { 0xbf67b4f2, SUC_lexicon_tags_33 },
    { 0xf8c7b4f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc7b4fb, SUC_lexicon_tags_53 },
    { 0xbc67b4fc, SUC_lexicon_tags_37 },
    { 0xba07b4fc, SUC_lexicon_tags_12 },
    { 0xd4a7b4fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e7b501, SUC_lexicon_tags_17 },
    { 0x7007b502, SUC_lexicon_tags_4 },
    { 0x1d87b502, SUC_lexicon_tags_35 },
    { 0xb527b504, SUC_lexicon_tags_7 },
    { 0xb747b504, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b67b509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc107b50b, SUC_lexicon_tags_37 },
    { 0xa47b50c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b67b50f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4147b512, SUC_lexicon_tags_12 },
    { 0xdac7b512, SUC_lexicon_tags_33 },
    { 0x5767b514, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6767b516, SUC_lexicon_tags_140 },
    { 0xb447b517, SUC_lexicon_tags_29 },
    { 0xb747b516, SUC_lexicon_tags_2 },
    { 0xb6e7b519, SUC_lexicon_tags_27 },
    { 0xed47b51a, SUC_lexicon_tags_17 },
    { 0xd287b519, SUC_lexicon_tags_13 },
    { 0xc2c7b51c, SUC_lexicon_tags_37 },
    { 0x9167b51d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea7b520, SUC_lexicon_tags_45 },
    { 0x3ee7b521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e7b524, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8267b526, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ca7b528, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe67b52b, SUC_lexicon_tags_14 },
    { 0x1507b52b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc167b530, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e7b534, SUC_lexicon_tags_7 },
    { 0xce47b534, SUC_lexicon_tags_45 },
    { 0x4047b535, SUC_lexicon_tags_26 },
    { 0x33e7b537, SUC_lexicon_tags_37 },
    { 0x4be7b536, SUC_lexicon_tags_3 },
    { 0x2427b539, SUC_lexicon_tags_1 },
    { 0xc747b539, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9ea7b53c, SUC_lexicon_tags_1 },
    { 0xfea7b53c, SUC_lexicon_tags_43 },
    { 0xf1c7b53e, SUC_lexicon_tags_77 },
    { 0xb9c7b53e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7847b542, SUC_lexicon_tags_2 },
    { 0x1c87b542, SUC_lexicon_tags_11 },
    { 0xe6e7b544, SUC_lexicon_tags_47 },
    { 0xc6c7b545, SUC_lexicon_tags_12 },
    { 0x82e7b543, SUC_lexicon_tags_2 },
    { 0xec87b547, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e7b54f, SUC_lexicon_tags_37 },
    { 0xcec7b54f, SUC_lexicon_tags_46 },
    { 0xeb87b551, SUC_lexicon_tags_19 },
    { 0x6bc7b551, SUC_lexicon_tags_3 },
    { 0xd127b553, SUC_lexicon_tags_15 },
    { 0x8587b54f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e47b559, SUC_lexicon_tags_15 },
    { 0x4b87b55a, SUC_lexicon_tags_7 },
    { 0x4167b55b, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x7b87b55d, SUC_lexicon_tags_33 },
    { 0x5e47b55e, SUC_lexicon_tags_7 },
    { 0x2fe7b55f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de7b562, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x147b565, SUC_lexicon_tags_15 },
    { 0x1867b566, SUC_lexicon_tags_43 },
    { 0xe67b567, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4807b56d, SUC_lexicon_tags_12 },
    { 0x527b56e, SUC_lexicon_tags_64 },
    { 0xcac7b56d, SUC_lexicon_tags_1 },
    { 0x6827b56e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5087b572, SUC_lexicon_tags_15 },
    { 0xf5c7b573, SUC_lexicon_tags_31 },
    { 0xa767b574, SUC_lexicon_tags_15 },
    { 0x1067b575, SUC_lexicon_tags_1188 },
    { 0xbc67b576, SUC_lexicon_tags_15 },
    { 0x8b27b577, SUC_lexicon_tags_15 },
    { 0xdd27b572, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc7b57c, SUC_lexicon_tags_47 },
    { 0x7467b57d, SUC_lexicon_tags_15 },
    { 0x7fa7b57e, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xd707b580, SUC_lexicon_tags_27 },
    { 0x9587b581, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e7b588, SUC_lexicon_tags_7 },
    { 0x9047b589, SUC_lexicon_tags_15 },
    { 0xdc07b588, SUC_lexicon_tags_13 },
    { 0x3587b58b, SUC_lexicon_tags_15 },
    { 0xc847b588, SUC_lexicon_tags_26 },
    { 0xd667b58d, SUC_lexicon_tags_48 },
    { 0x4a07b58e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f27b590, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be7b593, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2c7b595, SUC_lexicon_tags_47 },
    { 0xdb67b595, SUC_lexicon_tags_11 },
    { 0x2f87b597, SUC_lexicon_tags_16 },
    { 0x1fa7b597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa7b59b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a87b59e, SUC_lexicon_tags_23 },
    { 0xdc47b59f, SUC_lexicon_tags_28 },
    { 0x5f27b5a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8747b5a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b67b5a8, SUC_lexicon_tags_49 },
    { 0x7c27b5a8, SUC_lexicon_tags_38 },
    { 0xf9a7b5aa, SUC_lexicon_tags_1 },
    { 0x5607b5aa, SUC_lexicon_tags_1041 },
    { 0x3a27b5aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07b5b0, SUC_lexicon_tags_25 },
    { 0xf567b5b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e7b5b6, SUC_lexicon_tags_11 },
    { 0x8b07b5b7, SUC_lexicon_tags_37 },
    { 0x3a07b5b8, SUC_lexicon_tags_299 },
    { 0x19c7b5b8, SUC_lexicon_tags_2 },
    { 0xb847b5ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427b5c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe87b5c6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1e07b5c8, SUC_lexicon_tags_1 },
    { 0x5ae7b5c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca07b5ce, SUC_lexicon_tags_2 },
    { 0x7da7b5cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8a07b5d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de7b5d4, SUC_lexicon_tags_11 },
    { 0x2c7b5d5, SUC_lexicon_tags_19 },
    { 0x7ae7b5d5, SUC_lexicon_tags_17 },
    { 0xc6a7b5d7, SUC_lexicon_tags_27 },
    { 0x7ba7b5d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1927b5db, SUC_lexicon_tags_6 },
    { 0x1647b5db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf4a7b5de, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa727b5e3, SUC_lexicon_tags_39 },
    { 0x6567b5e4, SUC_lexicon_tags_37 },
    { 0x4d67b5e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9187b5e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xec7b5e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e7b5ed, SUC_lexicon_tags_2 },
    { 0x2c27b5ed, SUC_lexicon_tags_14 },
    { 0xc567b5ef, SUC_lexicon_tags_15 },
    { 0xad07b5f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6807b5f4, SUC_lexicon_tags_12 },
    { 0xca27b5f5, SUC_lexicon_tags_25 },
    { 0x6e67b5f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa7b5f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfd27b5fb, SUC_lexicon_tags_25 },
    { 0xc907b5fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e7b602, SUC_lexicon_tags_25 },
    { 0xb8c7b603, SUC_lexicon_tags_46 },
    { 0xaac7b602, SUC_lexicon_tags_2 },
    { 0xa347b603, SUC_lexicon_tags_25 },
    { 0x25c7b606, SUC_lexicon_tags_116 },
    { 0x3147b606, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca7b60a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae7b60e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5227b612, SUC_lexicon_tags_37 },
    { 0x9dc7b613, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2e7b615, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8da7b617, SUC_lexicon_tags_264 },
    { 0xe907b618, SUC_lexicon_tags_27 },
    { 0x1867b617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf407b61b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4107b61e, SUC_lexicon_tags_25 },
    { 0xf3e7b61f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd67b624, SUC_lexicon_tags_75 },
    { 0x32e7b625, SUC_lexicon_tags_49 },
    { 0x247b626, SUC_lexicon_tags_33 },
    { 0xda67b627, SUC_lexicon_tags_17 },
    { 0xb887b627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe27b62a, SUC_lexicon_tags_12 },
    { 0xd3e7b62b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4027b62d, SUC_lexicon_tags_11 },
    { 0x83c7b62e, SUC_lexicon_tags_30 },
    { 0xd107b62e, SUC_lexicon_tags_2 },
    { 0x2427b630, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd87b637, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac7b63b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd807b63e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd07b642, SUC_lexicon_tags_14 },
    { 0xe387b643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3327b645, SUC_lexicon_tags_1 },
    { 0x92e7b646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb067b64f, SUC_lexicon_tags_11 },
    { 0x9f47b64f, SUC_lexicon_tags_38 },
    { 0x3807b651, SUC_lexicon_tags_63 },
    { 0xbfa7b650, SUC_lexicon_tags_29 },
    { 0xc4c7b653, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87b664, SUC_lexicon_tags_14 },
    { 0xdc27b665, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb107b668, SUC_lexicon_tags_112 },
    { 0x8067b668, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe067b66b, SUC_lexicon_tags_87 },
    { 0x1167b66c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x99e7b66e, SUC_lexicon_tags_1 },
    { 0xe2e7b66f, SUC_lexicon_tags_15 },
    { 0x1be7b66f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6e7b672, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c7b675, SUC_lexicon_tags_25 },
    { 0x2c7b675, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x887b679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7667b67b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a7b67e, SUC_lexicon_tags_25 },
    { 0x6327b67f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d27b681, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e7b686, SUC_lexicon_tags_17 },
    { 0x5007b687, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaca7b689, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe847b68d, SUC_lexicon_tags_23 },
    { 0xa547b68e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e7b691, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x56a7b693, SUC_lexicon_tags_65 },
    { 0xc927b694, SUC_lexicon_tags_2 },
    { 0xf1e7b693, SUC_lexicon_tags_7 },
    { 0xa6a7b694, SUC_lexicon_tags_63 },
    { 0xad67b696, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50c7b699, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7767b69c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9027b6a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf607b6a5, SUC_lexicon_tags_11 },
    { 0xbe7b6a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae7b6aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xae67b6ad, SUC_lexicon_tags_37 },
    { 0x8fe7b6ad, SUC_lexicon_tags_7 },
    { 0x6807b6af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e7b6b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa087b6bc, SUC_lexicon_tags_47 },
    { 0xe7c7b6bd, SUC_lexicon_tags_26 },
    { 0xb427b6be, SUC_lexicon_tags_87 },
    { 0xd827b6be, SUC_lexicon_tags_57 },
    { 0x9687b6c0, SUC_lexicon_tags_14 },
    { 0x6867b6c0, SUC_lexicon_tags_126 },
    { 0x7827b6c1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8ee7b6c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd687b6ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd1c7b6cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a7b6d0, SUC_lexicon_tags_77 },
    { 0xa427b6d1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1547b6d5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xff07b6d8, SUC_lexicon_tags_16 },
    { 0x58c7b6d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca7b6dc, SUC_lexicon_tags_17 },
    { 0xdd27b6dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d67b6e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb187b6e9, SUC_lexicon_tags_15 },
    { 0xab27b6ea, SUC_lexicon_tags_15 },
    { 0xb07b6eb, SUC_lexicon_tags_1 },
    { 0x86e7b6ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa67b6ee, SUC_lexicon_tags_133 },
    { 0xaba7b6ee, SUC_lexicon_tags_38 },
    { 0x2727b6f0, SUC_lexicon_tags_15 },
    { 0xe727b6f0, SUC_lexicon_tags_46 },
    { 0x3047b6f1, SUC_lexicon_tags_27 },
    { 0xa3e7b6f1, SUC_lexicon_tags_25 },
    { 0xd067b6f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11e7b6f6, SUC_lexicon_tags_6 },
    { 0x2ae7b6f7, SUC_lexicon_tags_25 },
    { 0x3aa7b6f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de7b6ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9787b701, SUC_lexicon_tags_0 },
    { 0x3127b701, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9207b704, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb647b709, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa47b70b, SUC_lexicon_tags_19 },
    { 0x73e7b70c, SUC_lexicon_tags_11 },
    { 0x1647b70c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fa7b70f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc7b714, SUC_lexicon_tags_45 },
    { 0x5cc7b715, SUC_lexicon_tags_138 },
    { 0x91e7b716, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a67b719, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa647b71b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d67b71f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a7b722, SUC_lexicon_tags_27 },
    { 0x1de7b723, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc087b725, SUC_lexicon_tags_1 },
    { 0x5fc7b726, SUC_lexicon_tags_68 },
    { 0x5a87b726, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfce7b729, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd867b72d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe207b72f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee7b732, SUC_lexicon_tags_17 },
    { 0xc1c7b733, SUC_lexicon_tags_2 },
    { 0x7467b734, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc507b737, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f87b740, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e7b748, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade7b74c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47b752, SUC_lexicon_tags_7 },
    { 0xafc7b753, SUC_lexicon_tags_25 },
    { 0xdb07b754, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3627b756, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4127b75a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c67b75c, SUC_lexicon_tags_23 },
    { 0xb207b75c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8287b763, SUC_lexicon_tags_7 },
    { 0xfb47b764, SUC_lexicon_tags_2 },
    { 0xf227b765, SUC_lexicon_tags_1 },
    { 0xaa67b763, SUC_lexicon_tags_2 },
    { 0xaac7b767, SUC_lexicon_tags_31 },
    { 0xcba7b767, SUC_lexicon_tags_12 },
    { 0xb6a7b769, SUC_lexicon_tags_33 },
    { 0xa27b767, SUC_lexicon_tags_37 },
    { 0x9ca7b76b, SUC_lexicon_tags_4 },
    { 0x4747b76c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd807b773, SUC_lexicon_tags_7 },
    { 0x28e7b774, SUC_lexicon_tags_15 },
    { 0x4667b774, SUC_lexicon_tags_35 },
    { 0x7747b774, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2567b781, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e7b786, SUC_lexicon_tags_25 },
    { 0xb947b786, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a67b78b, SUC_lexicon_tags_26 },
    { 0xd8a7b78c, SUC_lexicon_tags_2 },
    { 0xb9a7b78d, SUC_lexicon_tags_37 },
    { 0xe1c7b78b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe7b792, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xaf07b794, SUC_lexicon_tags_60 },
    { 0x3d87b795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f87b797, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2547b799, SUC_lexicon_tags_7 },
    { 0xc387b79a, SUC_lexicon_tags_12 },
    { 0x3ec7b79b, SUC_lexicon_tags_27 },
    { 0xe267b79a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xa8a7b79e, SUC_lexicon_tags_1 },
    { 0xec67b79f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5827b7a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39e7b7a3, SUC_lexicon_tags_45 },
    { 0xe607b7a4, SUC_lexicon_tags_12 },
    { 0xdb27b7a4, SUC_lexicon_tags_49 },
    { 0x57a7b7a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1147b7a8, SUC_lexicon_tags_30 },
    { 0xef67b7a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a47b7ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf687b7af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f87b7b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1e7b7b5, SUC_lexicon_tags_19 },
    { 0xcd67b7b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcee7b7b8, SUC_lexicon_tags_17 },
    { 0x8c87b7b9, SUC_lexicon_tags_144 },
    { 0xd787b7ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5a27b7bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a7b7bf, SUC_lexicon_tags_2 },
    { 0x16c7b7c0, SUC_lexicon_tags_17 },
    { 0x2f07b7bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa7b7c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x327b7c9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4307b7ce, SUC_lexicon_tags_27 },
    { 0x21c7b7cf, SUC_lexicon_tags_60 },
    { 0xae07b7d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a7b7d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xada7b7d9, SUC_lexicon_tags_17 },
    { 0x2e7b7da, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a27b7de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42a7b7e0, SUC_lexicon_tags_92 },
    { 0xdf87b7e1, SUC_lexicon_tags_25 },
    { 0x2fe7b7e2, SUC_lexicon_tags_7 },
    { 0x7d67b7e3, SUC_lexicon_tags_12 },
    { 0x7fc7b7e3, SUC_lexicon_tags_7 },
    { 0x8127b7e5, SUC_lexicon_tags_13 },
    { 0x1e27b7e5, SUC_lexicon_tags_55 },
    { 0x9e7b7e2, SUC_lexicon_tags_105 },
    { 0x5fa7b7e2, SUC_lexicon_tags_1 },
    { 0x4007b7e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a7b7ee, SUC_lexicon_tags_25 },
    { 0xa587b7ee, SUC_lexicon_tags_33 },
    { 0xb547b7f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd967b7f5, SUC_lexicon_tags_17 },
    { 0xf087b7f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f27b7fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f07b802, SUC_lexicon_tags_25 },
    { 0x6627b803, SUC_lexicon_tags_7 },
    { 0x4007b803, SUC_lexicon_tags_11 },
    { 0xb2a7b802, SUC_lexicon_tags_19 },
    { 0x82e7b806, SUC_lexicon_tags_1 },
    { 0xa687b806, SUC_lexicon_tags_51 },
    { 0x5fe7b808, SUC_lexicon_tags_38 },
    { 0xd487b809, SUC_lexicon_tags_47 },
    { 0x9e67b809, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c7b80d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e7b811, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a7b814, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd47b819, SUC_lexicon_tags_29 },
    { 0x3287b819, SUC_lexicon_tags_1 },
    { 0xfce7b81b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e7b81e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a27b821, SUC_lexicon_tags_2 },
    { 0x2fa7b821, SUC_lexicon_tags_2 },
    { 0xf3e7b823, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd867b825, SUC_lexicon_tags_45 },
    { 0x37e7b826, SUC_lexicon_tags_12 },
    { 0xbaa7b825, SUC_lexicon_tags_11 },
    { 0xdfa7b828, SUC_lexicon_tags_17 },
    { 0x5b67b827, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe727b82b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3047b833, SUC_lexicon_tags_37 },
    { 0x2bc7b834, SUC_lexicon_tags_16 },
    { 0xa887b835, SUC_lexicon_tags_12 },
    { 0x15e7b833, SUC_lexicon_tags_84 },
    { 0x7a67b837, SUC_lexicon_tags_43 },
    { 0xeb27b835, SUC_lexicon_tags_1 },
    { 0xf1a7b839, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe747b83b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c7b83e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2fe7b840, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9087b843, SUC_lexicon_tags_66 },
    { 0x2e07b844, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6247b84b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e7b853, SUC_lexicon_tags_29 },
    { 0x2c67b854, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7b85a, SUC_lexicon_tags_31 },
    { 0xe3e7b85a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee7b861, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf5c7b863, SUC_lexicon_tags_47 },
    { 0x6aa7b864, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc7b86a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7327b86e, SUC_lexicon_tags_12 },
    { 0x14e7b86f, SUC_lexicon_tags_7 },
    { 0x1987b870, SUC_lexicon_tags_45 },
    { 0xa2a7b871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9727b873, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x84e7b875, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7927b878, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc67b87a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a27b87d, SUC_lexicon_tags_49 },
    { 0x6547b87d, SUC_lexicon_tags_3 },
    { 0x60e7b87d, SUC_lexicon_tags_11 },
    { 0xad87b880, SUC_lexicon_tags_37 },
    { 0x5d67b881, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd667b887, SUC_lexicon_tags_10 },
    { 0x9327b888, SUC_lexicon_tags_23 },
    { 0xff07b889, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a07b898, SUC_lexicon_tags_32 },
    { 0xdfa7b899, SUC_lexicon_tags_31 },
    { 0x1dc7b89a, SUC_lexicon_tags_21 },
    { 0x11c7b89b, SUC_lexicon_tags_1 },
    { 0x8427b89c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9627b8a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9607b8a5, SUC_lexicon_tags_19 },
    { 0xa907b8a5, SUC_lexicon_tags_30 },
    { 0xdde7b8a7, SUC_lexicon_tags_17 },
    { 0x2747b8a8, SUC_lexicon_tags_0 },
    { 0xaee7b8a5, SUC_lexicon_tags_2 },
    { 0xd467b8aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc87b8ac, SUC_lexicon_tags_27 },
    { 0x9167b8ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d87b8b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3947b8b2, SUC_lexicon_tags_4 },
    { 0xa7c7b8b3, SUC_lexicon_tags_138 },
    { 0xb607b8b4, SUC_lexicon_tags_23 },
    { 0xe0e7b8b2, SUC_lexicon_tags_12 },
    { 0x2f27b8b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27e7b8b8, SUC_lexicon_tags_17 },
    { 0xac7b8b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b27b8bb, SUC_lexicon_tags_1 },
    { 0xff47b8bc, SUC_lexicon_tags_15 },
    { 0xcd27b8bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5647b8c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c7b8c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c7b8ca, SUC_lexicon_tags_32 },
    { 0x5d87b8cb, SUC_lexicon_tags_7 },
    { 0xbdc7b8ca, SUC_lexicon_tags_49 },
    { 0x6807b8ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e7b8d1, SUC_lexicon_tags_15 },
    { 0x2947b8d2, SUC_lexicon_tags_88 },
    { 0x6a47b8d3, SUC_lexicon_tags_1 },
    { 0x9bc7b8d2, SUC_lexicon_tags_7 },
    { 0xf7e7b8d5, SUC_lexicon_tags_63 },
    { 0x9127b8d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x69a7b8d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dc7b8da, SUC_lexicon_tags_14 },
    { 0xa887b8da, SUC_lexicon_tags_46 },
    { 0x2aa7b8dc, SUC_lexicon_tags_39 },
    { 0xcee7b8dd, SUC_lexicon_tags_37 },
    { 0x5ce7b8db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1947b8e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc7b8e5, SUC_lexicon_tags_7 },
    { 0xf2e7b8e6, SUC_lexicon_tags_37 },
    { 0x20c7b8e7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c7b8ea, SUC_lexicon_tags_27 },
    { 0xf7e7b8eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c7b8ee, SUC_lexicon_tags_7 },
    { 0x3c7b8ef, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1887b8f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe2a7b8f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4287b8f7, SUC_lexicon_tags_21 },
    { 0xf127b8f7, SUC_lexicon_tags_7 },
    { 0xcea7b8f7, SUC_lexicon_tags_23 },
    { 0x7267b8f8, SUC_lexicon_tags_43 },
    { 0xdbe7b8fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a07b8ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc667b901, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a87b907, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1347b909, SUC_lexicon_tags_37 },
    { 0xa5c7b909, SUC_lexicon_tags_17 },
    { 0x97e7b90b, SUC_lexicon_tags_2 },
    { 0xfdc7b90b, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x1e47b90e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e7b916, SUC_lexicon_tags_17 },
    { 0xb8e7b917, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8507b91c, SUC_lexicon_tags_8 },
    { 0x7c27b91c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae7b922, SUC_lexicon_tags_17 },
    { 0x1427b922, SUC_lexicon_tags_26 },
    { 0xe3c7b924, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1187b926, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a7b929, SUC_lexicon_tags_11 },
    { 0x5347b92a, SUC_lexicon_tags_2 },
    { 0x6bc7b929, SUC_lexicon_tags_11 },
    { 0x3027b92c, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0xb627b92e, SUC_lexicon_tags_7 },
    { 0xf2a7b92e, SUC_lexicon_tags_7 },
    { 0x90c7b930, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc387b933, SUC_lexicon_tags_37 },
    { 0x1887b934, SUC_lexicon_tags_17 },
    { 0xa327b935, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2287b93a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb827b941, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6167b944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf367b947, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc127b94a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e7b94e, SUC_lexicon_tags_33 },
    { 0x2147b94f, SUC_lexicon_tags_10 },
    { 0x7da7b950, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae7b953, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x52e7b955, SUC_lexicon_tags_7 },
    { 0x5d27b956, SUC_lexicon_tags_62 },
    { 0x8f67b955, SUC_lexicon_tags_15 },
    { 0xf8a7b958, SUC_lexicon_tags_65 },
    { 0x9c47b958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5da7b95b, SUC_lexicon_tags_31 },
    { 0x6da7b95b, SUC_lexicon_tags_25 },
    { 0x33c7b95b, SUC_lexicon_tags_87 },
    { 0x7c7b95e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd4a7b960, SUC_lexicon_tags_0 },
    { 0xcfa7b961, SUC_lexicon_tags_38 },
    { 0xcd07b962, SUC_lexicon_tags_17 },
    { 0xcbe7b960, SUC_lexicon_tags_11 },
    { 0x8ea7b964, SUC_lexicon_tags_30 },
    { 0x4c27b964, SUC_lexicon_tags_1 },
    { 0xeee7b966, SUC_lexicon_tags_2 },
    { 0x17e7b964, SUC_lexicon_tags_15 },
    { 0x4047b968, SUC_lexicon_tags_46 },
    { 0x3ce7b966, SUC_lexicon_tags_25 },
    { 0x847b96a, SUC_lexicon_tags_33 },
    { 0xb0c7b96a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa247b96d, SUC_lexicon_tags_38 },
    { 0xf3a7b96d, SUC_lexicon_tags_15 },
    { 0xfc67b96d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d87b972, SUC_lexicon_tags_43 },
    { 0x79a7b973, SUC_lexicon_tags_43 },
    { 0xd427b973, SUC_lexicon_tags_25 },
    { 0xe547b973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a7b97b, SUC_lexicon_tags_63 },
    { 0x93a7b97b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc27b981, SUC_lexicon_tags_84 },
    { 0xb527b982, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e27b986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf987b988, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe7b98b, SUC_lexicon_tags_21 },
    { 0x4c27b98c, SUC_lexicon_tags_16 },
    { 0x3087b98d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf387b98f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9027b992, SUC_lexicon_tags_32 },
    { 0x5107b993, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4347b997, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c7b9a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc7b9a3, SUC_lexicon_tags_25 },
    { 0xa7c7b9a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86a7b9a6, SUC_lexicon_tags_46 },
    { 0x1c47b9a7, SUC_lexicon_tags_39 },
    { 0x7c27b9a7, SUC_lexicon_tags_38 },
    { 0x7d27b9a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa9e7b9ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb2e7b9ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f47b9b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc567b9b7, SUC_lexicon_tags_37 },
    { 0xf347b9b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd347b9bf, SUC_lexicon_tags_57 },
    { 0x7447b9c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5447b9c2, SUC_lexicon_tags_49 },
    { 0xa4e7b9c3, SUC_lexicon_tags_7 },
    { 0xc4a7b9c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8007b9c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec7b9c9, SUC_lexicon_tags_12 },
    { 0xabc7b9c9, SUC_lexicon_tags_684 },
    { 0, NULL },
    { 0x23e7b9cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e7b9d2, SUC_lexicon_tags_28 },
    { 0xd147b9d3, SUC_lexicon_tags_45 },
    { 0x3e7b9d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa7b9d7, SUC_lexicon_tags_87 },
    { 0x65a7b9d7, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea7b9db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd627b9e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x18a7b9e3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2447b9e8, SUC_lexicon_tags_0 },
    { 0xd4e7b9e9, SUC_lexicon_tags_17 },
    { 0x4aa7b9ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xde87b9ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc27b9f0, SUC_lexicon_tags_8 },
    { 0x3567b9f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57a7b9f3, SUC_lexicon_tags_37 },
    { 0x1d27b9f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb67b9f9, SUC_lexicon_tags_7 },
    { 0x8487b9fa, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c7b9fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a07b9ff, SUC_lexicon_tags_1 },
    { 0x4607ba00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe267ba04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd87ba07, SUC_lexicon_tags_45 },
    { 0xba87ba07, SUC_lexicon_tags_53 },
    { 0x4647ba09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a07ba11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa7ba14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba7ba18, SUC_lexicon_tags_7 },
    { 0xea47ba19, SUC_lexicon_tags_1 },
    { 0x3d07ba1a, SUC_lexicon_tags_17 },
    { 0x20a7ba1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e47ba1d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2187ba1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa7ba22, SUC_lexicon_tags_0 },
    { 0xc27ba22, SUC_lexicon_tags_216 },
    { 0xbb47ba23, SUC_lexicon_tags_2 },
    { 0xf627ba22, SUC_lexicon_tags_26 },
    { 0x3f87ba26, SUC_lexicon_tags_79 },
    { 0xaa07ba27, SUC_lexicon_tags_7 },
    { 0xe7a7ba22, SUC_lexicon_tags_37 },
    { 0xbd07ba29, SUC_lexicon_tags_60 },
    { 0x4087ba22, SUC_lexicon_tags_1 },
    { 0xe6a7ba2b, SUC_lexicon_tags_68 },
    { 0x54a7ba2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da7ba30, SUC_lexicon_tags_7 },
    { 0xd6a7ba31, SUC_lexicon_tags_63 },
    { 0xd667ba32, SUC_lexicon_tags_25 },
    { 0x7fe7ba33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5067ba35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7ba39, SUC_lexicon_tags_11 },
    { 0xe647ba3a, SUC_lexicon_tags_2 },
    { 0xe687ba3a, SUC_lexicon_tags_12 },
    { 0x327ba3c, SUC_lexicon_tags_11 },
    { 0x5d27ba3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x98c7ba3f, SUC_lexicon_tags_23 },
    { 0x6827ba40, SUC_lexicon_tags_47 },
    { 0xb327ba40, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6147ba43, SUC_lexicon_tags_61 },
    { 0xd67ba44, SUC_lexicon_tags_37 },
    { 0xa9c7ba45, SUC_lexicon_tags_23 },
    { 0x30c7ba45, SUC_lexicon_tags_27 },
    { 0x4587ba45, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2087ba49, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a27ba4e, SUC_lexicon_tags_25 },
    { 0xd07ba4f, SUC_lexicon_tags_25 },
    { 0x6427ba50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2847ba56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c67ba59, SUC_lexicon_tags_37 },
    { 0xc767ba5a, SUC_lexicon_tags_1 },
    { 0xe547ba5b, SUC_lexicon_tags_19 },
    { 0x3527ba5b, SUC_lexicon_tags_11 },
    { 0x1447ba5d, SUC_lexicon_tags_1 },
    { 0x367ba5e, SUC_lexicon_tags_19 },
    { 0x1e87ba5d, SUC_lexicon_tags_2 },
    { 0x4027ba5f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8267ba67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70a7ba69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc67ba6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd927ba70, SUC_lexicon_tags_6 },
    { 0x2f07ba71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1ae7ba73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7447ba75, SUC_lexicon_tags_30 },
    { 0xc067ba76, SUC_lexicon_tags_0 },
    { 0xc207ba76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa067ba79, SUC_lexicon_tags_6 },
    { 0xb8c7ba7a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8227ba80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa7ba84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcee7ba86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce7ba89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d27ba8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda67ba90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1547ba93, SUC_lexicon_tags_21 },
    { 0x1a47ba94, SUC_lexicon_tags_37 },
    { 0xb9e7ba94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe327ba99, SUC_lexicon_tags_65 },
    { 0x667ba9a, SUC_lexicon_tags_11 },
    { 0xeca7ba9a, SUC_lexicon_tags_25 },
    { 0x43e7ba9a, SUC_lexicon_tags_33 },
    { 0xf2c7ba9c, SUC_lexicon_tags_48 },
    { 0x8567ba9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a27baa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e7baac, SUC_lexicon_tags_27 },
    { 0xfac7baad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1447baaf, SUC_lexicon_tags_1 },
    { 0x1747bab0, SUC_lexicon_tags_12 },
    { 0xece7baaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb847bab4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe667baba, SUC_lexicon_tags_15 },
    { 0x2fc7baba, SUC_lexicon_tags_15 },
    { 0x4ea7babc, SUC_lexicon_tags_64 },
    { 0xb4e7babd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa07bac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b07bac9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf827bacc, SUC_lexicon_tags_17 },
    { 0x4b87bacd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d27bad0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe887bad5, SUC_lexicon_tags_82 },
    { 0xe9a7bad6, SUC_lexicon_tags_12 },
    { 0xb527bad7, SUC_lexicon_tags_29 },
    { 0xa487bad8, SUC_lexicon_tags_38 },
    { 0x2027bad7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247badf, SUC_lexicon_tags_11 },
    { 0x22e7bae0, SUC_lexicon_tags_23 },
    { 0xad27bae1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2567bae4, SUC_lexicon_tags_11 },
    { 0x3ea7bae5, SUC_lexicon_tags_2 },
    { 0x2427bae4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d07bae8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ea7baea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b47baee, SUC_lexicon_tags_15 },
    { 0xc487baef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da7baf4, SUC_lexicon_tags_43 },
    { 0xf667baf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5627baf8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9587bafa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8607bafe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf67bb02, SUC_lexicon_tags_2 },
    { 0x36a7bb03, SUC_lexicon_tags_17 },
    { 0x79a7bb04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7647bb06, SUC_lexicon_tags_12 },
    { 0xb607bb07, SUC_lexicon_tags_37 },
    { 0x2b87bb07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67bb0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a7bb0f, SUC_lexicon_tags_12 },
    { 0x7467bb0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc67bb12, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9707bb17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d07bb1e, SUC_lexicon_tags_19 },
    { 0x8307bb1f, SUC_lexicon_tags_27 },
    { 0xbe27bb20, SUC_lexicon_tags_14 },
    { 0x5127bb1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad87bb26, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4547bb30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa807bb32, SUC_lexicon_tags_278 },
    { 0, NULL },
    { 0, NULL },
    { 0x4467bb35, SUC_lexicon_tags_11 },
    { 0x5427bb36, SUC_lexicon_tags_10 },
    { 0x43a7bb37, SUC_lexicon_tags_49 },
    { 0x3587bb37, SUC_lexicon_tags_25 },
    { 0x6ac7bb39, SUC_lexicon_tags_23 },
    { 0xe6c7bb39, SUC_lexicon_tags_12 },
    { 0x51e7bb3b, SUC_lexicon_tags_33 },
    { 0xee07bb39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f27bb3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb567bb43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4727bb45, SUC_lexicon_tags_47 },
    { 0x47a7bb45, SUC_lexicon_tags_10 },
    { 0xcb47bb47, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b27bb4e, SUC_lexicon_tags_7 },
    { 0x4f07bb4f, SUC_lexicon_tags_7 },
    { 0x427bb50, SUC_lexicon_tags_6 },
    { 0x61a7bb51, SUC_lexicon_tags_12 },
    { 0xb787bb52, SUC_lexicon_tags_2 },
    { 0x4927bb53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c7bb56, SUC_lexicon_tags_37 },
    { 0x35e7bb57, SUC_lexicon_tags_37 },
    { 0xb227bb58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfee7bb5a, SUC_lexicon_tags_67 },
    { 0xac47bb5b, SUC_lexicon_tags_21 },
    { 0x90a7bb5a, SUC_lexicon_tags_47 },
    { 0xad87bb5d, SUC_lexicon_tags_118 },
    { 0xcd07bb5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3407bb60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe127bb63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe247bb66, SUC_lexicon_tags_14 },
    { 0x4467bb66, SUC_lexicon_tags_25 },
    { 0x2e67bb68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a7bb6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7327bb72, SUC_lexicon_tags_38 },
    { 0xaf67bb72, SUC_lexicon_tags_32 },
    { 0x8207bb73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7567bb77, SUC_lexicon_tags_47 },
    { 0xf3e7bb78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1087bb7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea67bb7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b27bb7e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x1427bb81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee87bb85, SUC_lexicon_tags_37 },
    { 0xdec7bb85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bb8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1147bb8c, SUC_lexicon_tags_10 },
    { 0x5e27bb8c, SUC_lexicon_tags_26 },
    { 0x2ce7bb8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c7bb92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9267bb94, SUC_lexicon_tags_10 },
    { 0xa547bb94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9aa7bb97, SUC_lexicon_tags_37 },
    { 0x44c7bb97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c27bb9b, SUC_lexicon_tags_39 },
    { 0x2a67bb9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4047bba2, SUC_lexicon_tags_134 },
    { 0x2f67bba3, SUC_lexicon_tags_60 },
    { 0x5127bba4, SUC_lexicon_tags_1 },
    { 0x5087bba4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9327bbaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3507bbac, SUC_lexicon_tags_30 },
    { 0x86c7bbad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3207bbaf, SUC_lexicon_tags_11 },
    { 0x1c87bbb0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1887bbb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467bbb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfce7bbbb, SUC_lexicon_tags_92 },
    { 0x4187bbbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7627bbbe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4e27bbc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67bbc4, SUC_lexicon_tags_48 },
    { 0x8cc7bbc4, SUC_lexicon_tags_17 },
    { 0x7407bbc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7e67bbc8, SUC_lexicon_tags_31 },
    { 0x1d67bbc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba7bbcd, SUC_lexicon_tags_63 },
    { 0x7c87bbce, SUC_lexicon_tags_12 },
    { 0xcd07bbcf, SUC_lexicon_tags_63 },
    { 0xb007bbcf, SUC_lexicon_tags_11 },
    { 0xab87bbd0, SUC_lexicon_tags_33 },
    { 0xc467bbd2, SUC_lexicon_tags_25 },
    { 0xb2e7bbce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1927bbd9, SUC_lexicon_tags_14 },
    { 0x9727bbd9, SUC_lexicon_tags_7 },
    { 0x94e7bbdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4767bbdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6467bbe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8a7bbe5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa247bbe7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf27bbe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb747bbec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd8a7bbee, SUC_lexicon_tags_29 },
    { 0xd0a7bbef, SUC_lexicon_tags_28 },
    { 0xfa27bbef, SUC_lexicon_tags_7 },
    { 0xf8e7bbee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2427bbf5, SUC_lexicon_tags_12 },
    { 0x62e7bbf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc7bbfc, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c07bc00, SUC_lexicon_tags_25 },
    { 0x63a7bc01, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c7bc04, SUC_lexicon_tags_12 },
    { 0xb6a7bc05, SUC_lexicon_tags_47 },
    { 0xfbe7bc05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4627bc09, SUC_lexicon_tags_17 },
    { 0x2247bc0a, SUC_lexicon_tags_56 },
    { 0xd6e7bc0a, SUC_lexicon_tags_24 },
    { 0x4dc7bc0c, SUC_lexicon_tags_0 },
    { 0xb307bc0b, SUC_lexicon_tags_46 },
    { 0xf387bc0e, SUC_lexicon_tags_1 },
    { 0xce47bc0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6327bc17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa507bc1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb07bc2c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a7bc30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a7bc35, SUC_lexicon_tags_37 },
    { 0x17a7bc36, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8fc7bc38, SUC_lexicon_tags_46 },
    { 0xf727bc38, SUC_lexicon_tags_25 },
    { 0x8447bc3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcf07bc3c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x327bc3f, SUC_lexicon_tags_9 },
    { 0x9127bc40, SUC_lexicon_tags_7 },
    { 0x1d67bc40, SUC_lexicon_tags_12 },
    { 0x2cc7bc40, SUC_lexicon_tags_25 },
    { 0x52c7bc43, SUC_lexicon_tags_7 },
    { 0x8b67bc44, SUC_lexicon_tags_1 },
    { 0xa167bc3f, SUC_lexicon_tags_2 },
    { 0x1b47bc41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe7bc49, SUC_lexicon_tags_11 },
    { 0x5ca7bc4a, SUC_lexicon_tags_16 },
    { 0x5367bc4b, SUC_lexicon_tags_15 },
    { 0x2187bc4c, SUC_lexicon_tags_17 },
    { 0xb327bc4d, SUC_lexicon_tags_31 },
    { 0x8de7bc4e, SUC_lexicon_tags_15 },
    { 0x6f87bc4f, SUC_lexicon_tags_7 },
    { 0xcea7bc50, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c7bc53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7987bc55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9307bc57, SUC_lexicon_tags_1 },
    { 0xf707bc57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b67bc5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1187bc5e, SUC_lexicon_tags_19 },
    { 0x2a87bc5f, SUC_lexicon_tags_1 },
    { 0x9da7bc5f, SUC_lexicon_tags_13 },
    { 0x587bc5f, SUC_lexicon_tags_23 },
    { 0x6327bc62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a7bc66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c87bc69, SUC_lexicon_tags_1 },
    { 0x4c67bc69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c7bc6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc767bc6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2847bc76, SUC_lexicon_tags_63 },
    { 0xe427bc76, SUC_lexicon_tags_23 },
    { 0x3927bc78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8007bc7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b47bc81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c27bc84, SUC_lexicon_tags_60 },
    { 0x4b07bc84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b27bc89, SUC_lexicon_tags_19 },
    { 0xd0c7bc8a, SUC_lexicon_tags_1 },
    { 0x6167bc8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc507bc8d, SUC_lexicon_tags_12 },
    { 0xd9e7bc8e, SUC_lexicon_tags_17 },
    { 0x6e07bc8f, SUC_lexicon_tags_2 },
    { 0x3027bc8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5247bc93, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa427bc97, SUC_lexicon_tags_24 },
    { 0xf267bc98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1027bc9a, SUC_lexicon_tags_47 },
    { 0x387bc9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f67bc9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4667bc9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3847bca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc87bca3, SUC_lexicon_tags_9 },
    { 0x56a7bca4, SUC_lexicon_tags_0 },
    { 0x17c7bca4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a7bcaa, SUC_lexicon_tags_37 },
    { 0xd0c7bcab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb027bcb2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7c27bcb4, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0x4c27bcb6, SUC_lexicon_tags_12 },
    { 0x487bcb6, SUC_lexicon_tags_25 },
    { 0xdca7bcb7, SUC_lexicon_tags_26 },
    { 0x1207bcb6, SUC_lexicon_tags_60 },
    { 0x1347bcba, SUC_lexicon_tags_85 },
    { 0xa827bcbb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2447bcbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde67bcc3, SUC_lexicon_tags_37 },
    { 0x1c07bcc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b07bcc8, SUC_lexicon_tags_37 },
    { 0x7ac7bcc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7bccc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35c7bcce, SUC_lexicon_tags_12 },
    { 0xfc87bccf, SUC_lexicon_tags_38 },
    { 0xf387bccf, SUC_lexicon_tags_35 },
    { 0x79e7bcd1, SUC_lexicon_tags_47 },
    { 0xcd67bcd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36c7bcd4, SUC_lexicon_tags_2 },
    { 0xa967bcd4, SUC_lexicon_tags_34 },
    { 0xa467bcd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd67bcda, SUC_lexicon_tags_17 },
    { 0xb2c7bcda, SUC_lexicon_tags_22 },
    { 0xd727bcdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c7bce2, SUC_lexicon_tags_40 },
    { 0x9ce7bce3, SUC_lexicon_tags_82 },
    { 0x5e87bce3, SUC_lexicon_tags_131 },
    { 0x2327bce5, SUC_lexicon_tags_8 },
    { 0x6c7bce6, SUC_lexicon_tags_60 },
    { 0x13a7bce7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fe7bce9, SUC_lexicon_tags_0 },
    { 0xa087bce9, SUC_lexicon_tags_19 },
    { 0xea07bceb, SUC_lexicon_tags_15 },
    { 0xcb47bcec, SUC_lexicon_tags_475 },
    { 0xac07bcea, SUC_lexicon_tags_23 },
    { 0x49c7bcee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9187bcf1, SUC_lexicon_tags_15 },
    { 0xe247bcf2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6587bcf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc67bcf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a7bcfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5a7bcfd, SUC_lexicon_tags_68 },
    { 0xd1a7bcfe, SUC_lexicon_tags_19 },
    { 0x8207bcfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a7bd03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9187bd0a, SUC_lexicon_tags_15 },
    { 0x6247bd0b, SUC_lexicon_tags_75 },
    { 0xf387bd0b, SUC_lexicon_tags_11 },
    { 0xd447bd0d, SUC_lexicon_tags_17 },
    { 0x61c7bd0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd267bd16, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c7bd19, SUC_lexicon_tags_37 },
    { 0xeb67bd19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb47bd1d, SUC_lexicon_tags_4 },
    { 0x1067bd1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb267bd20, SUC_lexicon_tags_28 },
    { 0xdd67bd21, SUC_lexicon_tags_37 },
    { 0x5827bd22, SUC_lexicon_tags_1 },
    { 0x2647bd21, SUC_lexicon_tags_17 },
    { 0x76c7bd24, SUC_lexicon_tags_0 },
    { 0x1e47bd25, SUC_lexicon_tags_109 },
    { 0xf667bd23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x41e7bd28, SUC_lexicon_tags_25 },
    { 0x3ae7bd29, SUC_lexicon_tags_11 },
    { 0x4407bd2a, SUC_lexicon_tags_11 },
    { 0x7e67bd2a, SUC_lexicon_tags_6 },
    { 0xf4e7bd2a, SUC_lexicon_tags_2 },
    { 0x2de7bd2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c7bd32, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3e07bd34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8687bd36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda7bd39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6447bd3d, SUC_lexicon_tags_26 },
    { 0xeb07bd3e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7927bd49, SUC_lexicon_tags_37 },
    { 0x51e7bd49, SUC_lexicon_tags_11 },
    { 0x9be7bd4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c7bd4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7bd51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6327bd54, SUC_lexicon_tags_11 },
    { 0xd8a7bd55, SUC_lexicon_tags_87 },
    { 0x7c27bd55, SUC_lexicon_tags_40 },
    { 0xd67bd57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe347bd5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc4a7bd5c, SUC_lexicon_tags_29 },
    { 0xb507bd5d, SUC_lexicon_tags_7 },
    { 0xf5e7bd5e, SUC_lexicon_tags_31 },
    { 0x7da7bd5c, SUC_lexicon_tags_37 },
    { 0x5867bd5e, SUC_lexicon_tags_19 },
    { 0xb267bd5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee27bd68, SUC_lexicon_tags_6 },
    { 0x6a47bd69, SUC_lexicon_tags_21 },
    { 0x3687bd6a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe0a7bd6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ae7bd6e, SUC_lexicon_tags_25 },
    { 0x2d87bd6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd827bd76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb87bd78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee07bd7d, SUC_lexicon_tags_53 },
    { 0xe67bd7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa07bd80, SUC_lexicon_tags_6 },
    { 0x4a87bd81, SUC_lexicon_tags_17 },
    { 0xd2a7bd82, SUC_lexicon_tags_11 },
    { 0xef47bd83, SUC_lexicon_tags_9 },
    { 0x9307bd80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d87bd88, SUC_lexicon_tags_27 },
    { 0xde47bd89, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x1ba7bd8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1567bd91, SUC_lexicon_tags_2 },
    { 0xf207bd92, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27bd99, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6dc7bd9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf747bd9f, SUC_lexicon_tags_2 },
    { 0x4cc7bd9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a7bdaf, SUC_lexicon_tags_25 },
    { 0xb927bdb0, SUC_lexicon_tags_5 },
    { 0x3107bdb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4007bdb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1367bdb6, SUC_lexicon_tags_84 },
    { 0x77c7bdb6, SUC_lexicon_tags_17 },
    { 0x3a87bdb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce87bdba, SUC_lexicon_tags_28 },
    { 0xcf67bdba, SUC_lexicon_tags_68 },
    { 0xa647bdbc, SUC_lexicon_tags_4 },
    { 0x4027bdbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4787bdc0, SUC_lexicon_tags_25 },
    { 0xc67bdc0, SUC_lexicon_tags_1 },
    { 0xc187bdc2, SUC_lexicon_tags_61 },
    { 0x4267bdc3, SUC_lexicon_tags_14 },
    { 0x9607bdc2, SUC_lexicon_tags_92 },
    { 0xe627bdc1, SUC_lexicon_tags_47 },
    { 0x1d67bdc6, SUC_lexicon_tags_28 },
    { 0xa47bdc7, SUC_lexicon_tags_46 },
    { 0x9ce7bdc7, SUC_lexicon_tags_45 },
    { 0x18a7bdc9, SUC_lexicon_tags_1 },
    { 0xf5e7bdc3, SUC_lexicon_tags_12 },
    { 0x40c7bdc5, SUC_lexicon_tags_55 },
    { 0x46c7bdcc, SUC_lexicon_tags_37 },
    { 0x1bc7bdcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea47bdd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa707bdde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6687bde1, SUC_lexicon_tags_1 },
    { 0x5667bde2, SUC_lexicon_tags_35 },
    { 0x59e7bde3, SUC_lexicon_tags_30 },
    { 0xc147bde3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4167bde8, SUC_lexicon_tags_37 },
    { 0xf4a7bde9, SUC_lexicon_tags_61 },
    { 0xcf67bde8, SUC_lexicon_tags_38 },
    { 0x7f67bde8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c07bdef, SUC_lexicon_tags_7 },
    { 0x9947bdf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf547bdf3, SUC_lexicon_tags_46 },
    { 0xd7c7bdf4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed47be06, SUC_lexicon_tags_47 },
    { 0x97e7be07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f67be09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da7be0c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e67be11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c07be15, SUC_lexicon_tags_33 },
    { 0x2d47be16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287be1a, SUC_lexicon_tags_7 },
    { 0x21c7be1a, SUC_lexicon_tags_11 },
    { 0x1f47be1c, SUC_lexicon_tags_15 },
    { 0x2a67be1d, SUC_lexicon_tags_15 },
    { 0xf87be1e, SUC_lexicon_tags_15 },
    { 0x2f87be1f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7247be23, SUC_lexicon_tags_13 },
    { 0xca47be23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b67be28, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4387be30, SUC_lexicon_tags_25 },
    { 0xc867be31, SUC_lexicon_tags_37 },
    { 0x3ac7be32, SUC_lexicon_tags_15 },
    { 0x4067be31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc7be3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe07be42, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5007be45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a07be47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a7be4d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c7be51, SUC_lexicon_tags_25 },
    { 0xf007be52, SUC_lexicon_tags_6 },
    { 0x83a7be52, SUC_lexicon_tags_27 },
    { 0x52c7be54, SUC_lexicon_tags_17 },
    { 0xd547be51, SUC_lexicon_tags_34 },
    { 0x3d87be53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f47be5b, SUC_lexicon_tags_33 },
    { 0xe7a7be5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5187be5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb87be61, SUC_lexicon_tags_49 },
    { 0x6fc7be62, SUC_lexicon_tags_30 },
    { 0xe8a7be62, SUC_lexicon_tags_37 },
    { 0x46c7be63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6be7be66, SUC_lexicon_tags_37 },
    { 0xbee7be67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8307be6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xab07be6e, SUC_lexicon_tags_25 },
    { 0x74e7be6f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x54a7be71, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a7be74, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x82e7be76, SUC_lexicon_tags_8 },
    { 0xb647be77, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe667be7a, SUC_lexicon_tags_23 },
    { 0xd247be7b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc627be80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf287be82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd67be84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8307be87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a7be8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea47be8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe7be92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4827be94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c7be99, SUC_lexicon_tags_37 },
    { 0x9747be9a, SUC_lexicon_tags_11 },
    { 0x747be9b, SUC_lexicon_tags_42 },
    { 0xde67be9b, SUC_lexicon_tags_7 },
    { 0x1ba7be9d, SUC_lexicon_tags_14 },
    { 0x4a7be9c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc907bea4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e7beab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9067bead, SUC_lexicon_tags_0 },
    { 0xe747beae, SUC_lexicon_tags_25 },
    { 0x41c7beaf, SUC_lexicon_tags_77 },
    { 0x9847beae, SUC_lexicon_tags_17 },
    { 0xd487beb1, SUC_lexicon_tags_30 },
    { 0xd447beb2, SUC_lexicon_tags_12 },
    { 0xa547beb3, SUC_lexicon_tags_15 },
    { 0x2be7beb4, SUC_lexicon_tags_25 },
    { 0x4987beb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a7beba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8047bec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3be7bec3, SUC_lexicon_tags_6 },
    { 0xc287bec4, SUC_lexicon_tags_7 },
    { 0x9207bec4, SUC_lexicon_tags_2 },
    { 0xd847bec6, SUC_lexicon_tags_15 },
    { 0x1287bec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89a7bec9, SUC_lexicon_tags_38 },
    { 0x1f87bec9, SUC_lexicon_tags_37 },
    { 0xfdc7beca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a7bed3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc7bed8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc7bee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb47bee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de7bee9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef87beeb, SUC_lexicon_tags_9 },
    { 0x6d07beec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9147bef0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87bef3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1207bef5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a67bef7, SUC_lexicon_tags_17 },
    { 0x9da7bef7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a7befb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e7beff, SUC_lexicon_tags_7 },
    { 0xf4c7beff, SUC_lexicon_tags_61 },
    { 0xbe27beff, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d27bf04, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d47bf07, SUC_lexicon_tags_25 },
    { 0x2ae7bf07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc267bf0b, SUC_lexicon_tags_66 },
    { 0xa4a7bf0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a7bf0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2427bf11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5687bf15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa7bf19, SUC_lexicon_tags_15 },
    { 0xfb07bf19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac7bf1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e7bf21, SUC_lexicon_tags_37 },
    { 0xe927bf22, SUC_lexicon_tags_1 },
    { 0xbd87bf23, SUC_lexicon_tags_39 },
    { 0xf067bf24, SUC_lexicon_tags_7 },
    { 0x5fe7bf23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b67bf27, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67bf2b, SUC_lexicon_tags_38 },
    { 0x6d07bf2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e67bf2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53c7bf30, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687bf34, SUC_lexicon_tags_14 },
    { 0x29a7bf34, SUC_lexicon_tags_2 },
    { 0x1787bf35, SUC_lexicon_tags_49 },
    { 0xf307bf37, SUC_lexicon_tags_7 },
    { 0x1947bf35, SUC_lexicon_tags_15 },
    { 0xee67bf39, SUC_lexicon_tags_60 },
    { 0xea67bf39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e7bf3d, SUC_lexicon_tags_15 },
    { 0x2727bf3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa447bf42, SUC_lexicon_tags_17 },
    { 0x1d87bf43, SUC_lexicon_tags_21 },
    { 0x7127bf43, SUC_lexicon_tags_55 },
    { 0x9e7bf45, SUC_lexicon_tags_9 },
    { 0xfbc7bf46, SUC_lexicon_tags_40 },
    { 0xf9e7bf46, SUC_lexicon_tags_15 },
    { 0x1ea7bf46, SUC_lexicon_tags_2 },
    { 0x7c87bf44, SUC_lexicon_tags_28 },
    { 0x1a27bf4a, SUC_lexicon_tags_14 },
    { 0x2947bf4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6207bf4d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb867bf4f, SUC_lexicon_tags_98 },
    { 0xcbe7bf4f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac87bf57, SUC_lexicon_tags_15 },
    { 0x7727bf58, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xc707bf5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b87bf5c, SUC_lexicon_tags_27 },
    { 0xd5c7bf5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc047bf63, SUC_lexicon_tags_11 },
    { 0xf147bf63, SUC_lexicon_tags_37 },
    { 0x8d07bf65, SUC_lexicon_tags_29 },
    { 0xbc47bf66, SUC_lexicon_tags_38 },
    { 0x6e47bf67, SUC_lexicon_tags_13 },
    { 0xb2c7bf67, SUC_lexicon_tags_37 },
    { 0xc327bf69, SUC_lexicon_tags_7 },
    { 0xa5c7bf67, SUC_lexicon_tags_15 },
    { 0x7c87bf6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc7bf72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7387bf78, SUC_lexicon_tags_11 },
    { 0xa447bf79, SUC_lexicon_tags_11 },
    { 0xab07bf7a, SUC_lexicon_tags_11 },
    { 0xa427bf7b, SUC_lexicon_tags_103 },
    { 0x4a7bf7c, SUC_lexicon_tags_11 },
    { 0xe27bf7a, SUC_lexicon_tags_39 },
    { 0x4bc7bf7e, SUC_lexicon_tags_33 },
    { 0x48a7bf7f, SUC_lexicon_tags_31 },
    { 0xce87bf79, SUC_lexicon_tags_30 },
    { 0xf847bf80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee7bf89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be7bf8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc487bf90, SUC_lexicon_tags_31 },
    { 0xe587bf91, SUC_lexicon_tags_23 },
    { 0xbbe7bf91, SUC_lexicon_tags_13 },
    { 0x6367bf93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa627bf97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe387bf9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5427bf9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca7bfa0, SUC_lexicon_tags_1 },
    { 0x8367bfa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4a7bfa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2387bfa8, SUC_lexicon_tags_15 },
    { 0x4487bfa9, SUC_lexicon_tags_46 },
    { 0xf5e7bfa9, SUC_lexicon_tags_15 },
    { 0x2827bfaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea7bfb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107bfb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c7bfba, SUC_lexicon_tags_1 },
    { 0x18a7bfbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6447bfbd, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6787bfc3, SUC_lexicon_tags_28 },
    { 0x2a47bfc4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c7bfce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3167bfd0, SUC_lexicon_tags_12 },
    { 0xa267bfd1, SUC_lexicon_tags_43 },
    { 0x45c7bfd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ca7bfd4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e7bfda, SUC_lexicon_tags_7 },
    { 0x38c7bfdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf487bfdd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1887bfdf, SUC_lexicon_tags_83 },
    { 0xac47bfdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7667bfe3, SUC_lexicon_tags_14 },
    { 0x45e7bfe4, SUC_lexicon_tags_11 },
    { 0x4fe7bfe5, SUC_lexicon_tags_31 },
    { 0x1cc7bfe4, SUC_lexicon_tags_11 },
    { 0xdb87bfe6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c87bfec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c7bff0, SUC_lexicon_tags_77 },
    { 0x5207bff1, SUC_lexicon_tags_27 },
    { 0xb4c7bff2, SUC_lexicon_tags_11 },
    { 0x4047bff0, SUC_lexicon_tags_43 },
    { 0x9a87bff2, SUC_lexicon_tags_2 },
    { 0x9307bff4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5d27bff7, SUC_lexicon_tags_11 },
    { 0xc467bff8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a7bffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d67bffe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3347c001, SUC_lexicon_tags_27 },
    { 0x8067c002, SUC_lexicon_tags_25 },
    { 0x76c7c003, SUC_lexicon_tags_38 },
    { 0x60a7c004, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc7c008, SUC_lexicon_tags_4 },
    { 0x827c009, SUC_lexicon_tags_37 },
    { 0x6467c009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2a7c00c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x84e7c00e, SUC_lexicon_tags_15 },
    { 0x8147c00f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c7c013, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3c7c015, SUC_lexicon_tags_37 },
    { 0xd7e7c015, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x27a7c018, SUC_lexicon_tags_2 },
    { 0xef47c019, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3307c01c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xac47c01e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6fa7c020, SUC_lexicon_tags_45 },
    { 0x8ae7c021, SUC_lexicon_tags_12 },
    { 0xd0c7c021, SUC_lexicon_tags_73 },
    { 0xa187c022, SUC_lexicon_tags_48 },
    { 0x1547c024, SUC_lexicon_tags_17 },
    { 0x52a7c024, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2f27c027, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9447c02c, SUC_lexicon_tags_63 },
    { 0xc2e7c02d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3067c02f, SUC_lexicon_tags_39 },
    { 0x70e7c030, SUC_lexicon_tags_7 },
    { 0x2047c031, SUC_lexicon_tags_15 },
    { 0x6767c032, SUC_lexicon_tags_2 },
    { 0xef47c031, SUC_lexicon_tags_17 },
    { 0xc367c034, SUC_lexicon_tags_25 },
    { 0xf207c035, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3327c037, SUC_lexicon_tags_30 },
    { 0xfd87c038, SUC_lexicon_tags_25 },
    { 0xea67c039, SUC_lexicon_tags_12 },
    { 0x4767c03a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3587c03d, SUC_lexicon_tags_8 },
    { 0xac67c03d, SUC_lexicon_tags_45 },
    { 0x7b07c03d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5487c042, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa067c044, SUC_lexicon_tags_45 },
    { 0x44c7c044, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e7c049, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe707c04d, SUC_lexicon_tags_2 },
    { 0x5227c04d, SUC_lexicon_tags_37 },
    { 0x2e07c04e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc07c054, SUC_lexicon_tags_8 },
    { 0xe227c054, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x867c057, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e7c05c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc807c05e, SUC_lexicon_tags_33 },
    { 0x8d47c05f, SUC_lexicon_tags_17 },
    { 0xe067c060, SUC_lexicon_tags_37 },
    { 0x6027c060, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1e7c063, SUC_lexicon_tags_11 },
    { 0x5947c064, SUC_lexicon_tags_47 },
    { 0x58e7c064, SUC_lexicon_tags_12 },
    { 0x41a7c064, SUC_lexicon_tags_7 },
    { 0xa107c065, SUC_lexicon_tags_60 },
    { 0xa907c068, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a67c06b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27c073, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c67c076, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x61e7c078, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x147c07a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd687c07c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4ec7c07e, SUC_lexicon_tags_33 },
    { 0x3587c07f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf407c082, SUC_lexicon_tags_17 },
    { 0x5be7c083, SUC_lexicon_tags_37 },
    { 0xb247c083, SUC_lexicon_tags_17 },
    { 0xfc67c085, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc227c08b, SUC_lexicon_tags_23 },
    { 0x64c7c08c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9187c090, SUC_lexicon_tags_14 },
    { 0x6ec7c091, SUC_lexicon_tags_47 },
    { 0x91a7c092, SUC_lexicon_tags_1 },
    { 0x3747c093, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x12e7c095, SUC_lexicon_tags_14 },
    { 0x2147c095, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e7c099, SUC_lexicon_tags_17 },
    { 0x4e87c09a, SUC_lexicon_tags_12 },
    { 0x9627c09b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf67c09e, SUC_lexicon_tags_37 },
    { 0x7b67c09f, SUC_lexicon_tags_30 },
    { 0x6dc7c0a0, SUC_lexicon_tags_17 },
    { 0x9fc7c0a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa307c0a6, SUC_lexicon_tags_0 },
    { 0xcc27c0a7, SUC_lexicon_tags_17 },
    { 0xaf67c0a8, SUC_lexicon_tags_11 },
    { 0xcd87c0a8, SUC_lexicon_tags_31 },
    { 0x69e7c0aa, SUC_lexicon_tags_112 },
    { 0x5967c0aa, SUC_lexicon_tags_873 },
    { 0x8487c0ab, SUC_lexicon_tags_9 },
    { 0x32c7c0ad, SUC_lexicon_tags_47 },
    { 0x3407c0ae, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e7c0b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d27c0b4, SUC_lexicon_tags_17 },
    { 0x207c0b5, SUC_lexicon_tags_8 },
    { 0x3c47c0b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a7c0bb, SUC_lexicon_tags_33 },
    { 0x5ee7c0bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c7c0bf, SUC_lexicon_tags_1 },
    { 0xd147c0c0, SUC_lexicon_tags_4 },
    { 0xcea7c0c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac7c0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4447c0c8, SUC_lexicon_tags_37 },
    { 0x65e7c0c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a7c0cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e7c0d2, SUC_lexicon_tags_26 },
    { 0xbda7c0d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c7c0da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee7c0df, SUC_lexicon_tags_25 },
    { 0x67a7c0e0, SUC_lexicon_tags_11 },
    { 0xa5a7c0df, SUC_lexicon_tags_18 },
    { 0x5c47c0e2, SUC_lexicon_tags_2 },
    { 0x3b87c0e2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb5c7c0e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9387c0e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b47c0ec, SUC_lexicon_tags_17 },
    { 0x6f27c0ed, SUC_lexicon_tags_17 },
    { 0x9467c0ed, SUC_lexicon_tags_14 },
    { 0x5747c0ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae7c0f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96c7c0f6, SUC_lexicon_tags_37 },
    { 0xd3e7c0f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb327c0f9, SUC_lexicon_tags_69 },
    { 0x2cc7c0f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e7c100, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e7c103, SUC_lexicon_tags_25 },
    { 0x8407c103, SUC_lexicon_tags_63 },
    { 0x7fa7c105, SUC_lexicon_tags_11 },
    { 0xd7e7c106, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95e7c108, SUC_lexicon_tags_11 },
    { 0x3a7c109, SUC_lexicon_tags_17 },
    { 0xd427c108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x287c10c, SUC_lexicon_tags_37 },
    { 0xc207c10d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3367c10f, SUC_lexicon_tags_32 },
    { 0x27c110, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74e7c112, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19c7c114, SUC_lexicon_tags_61 },
    { 0xf2c7c115, SUC_lexicon_tags_25 },
    { 0x78a7c115, SUC_lexicon_tags_8 },
    { 0xde7c114, SUC_lexicon_tags_4 },
    { 0xf8e7c117, SUC_lexicon_tags_57 },
    { 0xaa7c119, SUC_lexicon_tags_47 },
    { 0xb767c119, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc47c11d, SUC_lexicon_tags_39 },
    { 0x1ce7c11e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e07c121, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xda7c123, SUC_lexicon_tags_9 },
    { 0x2247c123, SUC_lexicon_tags_49 },
    { 0xf3c7c125, SUC_lexicon_tags_85 },
    { 0x6107c126, SUC_lexicon_tags_2 },
    { 0x5767c123, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf27c12a, SUC_lexicon_tags_11 },
    { 0x8c27c12b, SUC_lexicon_tags_29 },
    { 0xe4c7c12b, SUC_lexicon_tags_1 },
    { 0x44c7c12c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec87c132, SUC_lexicon_tags_21 },
    { 0x4907c132, SUC_lexicon_tags_25 },
    { 0x16e7c132, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2547c136, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4447c13d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7407c140, SUC_lexicon_tags_13 },
    { 0x8727c140, SUC_lexicon_tags_2 },
    { 0xe67c140, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e7c145, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c7c149, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e67c14d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e7c151, SUC_lexicon_tags_63 },
    { 0x1ea7c151, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2467c155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c7c158, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8507c15e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xda67c160, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75e7c162, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e7c166, SUC_lexicon_tags_37 },
    { 0xdaa7c166, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f67c16a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa1e7c16c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9507c16f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca7c172, SUC_lexicon_tags_4 },
    { 0x64a7c172, SUC_lexicon_tags_23 },
    { 0x527c172, SUC_lexicon_tags_23 },
    { 0x8e7c172, SUC_lexicon_tags_12 },
    { 0x9287c176, SUC_lexicon_tags_84 },
    { 0x3627c177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b67c179, SUC_lexicon_tags_39 },
    { 0xeb67c179, SUC_lexicon_tags_17 },
    { 0xd907c17b, SUC_lexicon_tags_1 },
    { 0xe8c7c179, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1b07c17e, SUC_lexicon_tags_11 },
    { 0xfde7c17f, SUC_lexicon_tags_16 },
    { 0xd3e7c180, SUC_lexicon_tags_65 },
    { 0x19a7c180, SUC_lexicon_tags_15 },
    { 0xc407c182, SUC_lexicon_tags_26 },
    { 0x4787c183, SUC_lexicon_tags_23 },
    { 0xfe67c184, SUC_lexicon_tags_7 },
    { 0x4b67c185, SUC_lexicon_tags_7 },
    { 0x5ae7c182, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7c18d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc347c18f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd127c192, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a7c196, SUC_lexicon_tags_38 },
    { 0x7087c197, SUC_lexicon_tags_47 },
    { 0x6567c198, SUC_lexicon_tags_10 },
    { 0xb687c197, SUC_lexicon_tags_1 },
    { 0x62e7c19a, SUC_lexicon_tags_113 },
    { 0x3a67c19b, SUC_lexicon_tags_14 },
    { 0xf07c19c, SUC_lexicon_tags_29 },
    { 0x5d67c19d, SUC_lexicon_tags_27 },
    { 0xcea7c19e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd747c1a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce7c1a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e7c1a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1727c1ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167c1b2, SUC_lexicon_tags_12 },
    { 0x52a7c1b3, SUC_lexicon_tags_184 },
    { 0x1487c1b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e7c1ba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c7c1bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3127c1c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba07c1c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7c1c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd727c1cf, SUC_lexicon_tags_25 },
    { 0x9167c1d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x187c1d3, SUC_lexicon_tags_12 },
    { 0x9f47c1d3, SUC_lexicon_tags_25 },
    { 0xcd47c1d5, SUC_lexicon_tags_9 },
    { 0xd47c1d6, SUC_lexicon_tags_37 },
    { 0x8a7c1d6, SUC_lexicon_tags_7 },
    { 0xa627c1d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe967c1dc, SUC_lexicon_tags_17 },
    { 0xbca7c1dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e47c1e2, SUC_lexicon_tags_25 },
    { 0x4547c1e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e7c1e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c7c1ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xae47c1ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7667c1ef, SUC_lexicon_tags_2 },
    { 0x3b27c1ef, SUC_lexicon_tags_7 },
    { 0xd5a7c1f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d07c1f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf387c1f5, SUC_lexicon_tags_60 },
    { 0x89c7c1f5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd5c7c1f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a7c1fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6667c203, SUC_lexicon_tags_15 },
    { 0xf747c204, SUC_lexicon_tags_29 },
    { 0xe107c205, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff87c211, SUC_lexicon_tags_33 },
    { 0x1827c212, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe947c219, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb467c21b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c7c21e, SUC_lexicon_tags_37 },
    { 0x4007c21e, SUC_lexicon_tags_2 },
    { 0x3387c220, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe687c222, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0xc127c224, SUC_lexicon_tags_8 },
    { 0x84e7c225, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3da7c227, SUC_lexicon_tags_37 },
    { 0x9d87c227, SUC_lexicon_tags_30 },
    { 0x3d87c227, SUC_lexicon_tags_47 },
    { 0x61e7c227, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd87c22c, SUC_lexicon_tags_12 },
    { 0x1667c22d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xda67c22f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e67c234, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb47c238, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc047c23c, SUC_lexicon_tags_25 },
    { 0xef47c23d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ee7c23f, SUC_lexicon_tags_2 },
    { 0x43a7c240, SUC_lexicon_tags_1 },
    { 0xe887c240, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc867c244, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4047c247, SUC_lexicon_tags_1 },
    { 0xca87c247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea27c24b, SUC_lexicon_tags_28 },
    { 0x8907c24c, SUC_lexicon_tags_60 },
    { 0x3387c24d, SUC_lexicon_tags_65 },
    { 0x9ea7c24e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20a7c250, SUC_lexicon_tags_14 },
    { 0xc607c251, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde07c254, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90c7c256, SUC_lexicon_tags_79 },
    { 0x98a7c256, SUC_lexicon_tags_15 },
    { 0x8707c258, SUC_lexicon_tags_26 },
    { 0x60e7c259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e7c25e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b27c264, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c7c26c, SUC_lexicon_tags_25 },
    { 0x2607c26c, SUC_lexicon_tags_37 },
    { 0x1987c26e, SUC_lexicon_tags_37 },
    { 0x8a87c26f, SUC_lexicon_tags_77 },
    { 0x24e7c26d, SUC_lexicon_tags_19 },
    { 0x37e7c271, SUC_lexicon_tags_92 },
    { 0x7807c272, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e7c275, SUC_lexicon_tags_28 },
    { 0x3f27c276, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca7c27c, SUC_lexicon_tags_37 },
    { 0x8727c27d, SUC_lexicon_tags_27 },
    { 0xb307c27c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x48c7c280, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf27c282, SUC_lexicon_tags_2 },
    { 0xb167c283, SUC_lexicon_tags_47 },
    { 0x8407c284, SUC_lexicon_tags_15 },
    { 0x1a27c285, SUC_lexicon_tags_12 },
    { 0x23c7c285, SUC_lexicon_tags_29 },
    { 0xbee7c282, SUC_lexicon_tags_39 },
    { 0xcf47c288, SUC_lexicon_tags_37 },
    { 0x6587c288, SUC_lexicon_tags_39 },
    { 0xce7c282, SUC_lexicon_tags_318 },
    { 0x4a07c28b, SUC_lexicon_tags_8 },
    { 0x72e7c28c, SUC_lexicon_tags_27 },
    { 0x87c7c28a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a7c291, SUC_lexicon_tags_37 },
    { 0xd847c292, SUC_lexicon_tags_38 },
    { 0x51a7c293, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfc47c295, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4227c297, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae47c29b, SUC_lexicon_tags_14 },
    { 0x60a7c29c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a7c2a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc7c2a4, SUC_lexicon_tags_1 },
    { 0xb8e7c2a4, SUC_lexicon_tags_17 },
    { 0xce87c2a6, SUC_lexicon_tags_7 },
    { 0x10a7c2a7, SUC_lexicon_tags_17 },
    { 0x30e7c2a8, SUC_lexicon_tags_7 },
    { 0xf167c2a6, SUC_lexicon_tags_27 },
    { 0xd8c7c2a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25e7c2ac, SUC_lexicon_tags_27 },
    { 0x9c7c2ad, SUC_lexicon_tags_37 },
    { 0x2707c2ad, SUC_lexicon_tags_47 },
    { 0xaf87c2ae, SUC_lexicon_tags_14 },
    { 0xd187c2b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc7c7c2b2, SUC_lexicon_tags_26 },
    { 0xf8a7c2b3, SUC_lexicon_tags_37 },
    { 0xe7a7c2b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca7c2b8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6de7c2ba, SUC_lexicon_tags_12 },
    { 0xb8c7c2ba, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xf047c2bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab87c2bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe7c2c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd067c2cb, SUC_lexicon_tags_10 },
    { 0xa9e7c2cc, SUC_lexicon_tags_17 },
    { 0x90a7c2cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287c2d1, SUC_lexicon_tags_9 },
    { 0x1fc7c2d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf27c2d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e67c2d7, SUC_lexicon_tags_7 },
    { 0x16e7c2d8, SUC_lexicon_tags_21 },
    { 0xb087c2d9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1207c2dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a47c2de, SUC_lexicon_tags_15 },
    { 0xa8c7c2de, SUC_lexicon_tags_103 },
    { 0x87c7c2e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa7c2e3, SUC_lexicon_tags_26 },
    { 0x9027c2e3, SUC_lexicon_tags_38 },
    { 0xbfe7c2e4, SUC_lexicon_tags_33 },
    { 0xcb07c2e6, SUC_lexicon_tags_26 },
    { 0x8e87c2e7, SUC_lexicon_tags_37 },
    { 0xd3c7c2e8, SUC_lexicon_tags_30 },
    { 0x1de7c2e9, SUC_lexicon_tags_66 },
    { 0xf827c2e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec7c2ee, SUC_lexicon_tags_25 },
    { 0xa587c2ee, SUC_lexicon_tags_37 },
    { 0xbc07c2f0, SUC_lexicon_tags_68 },
    { 0x9767c2f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a7c2f9, SUC_lexicon_tags_4 },
    { 0x21c7c2fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd847c2fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd067c30a, SUC_lexicon_tags_17 },
    { 0x5787c30b, SUC_lexicon_tags_91 },
    { 0xf207c30c, SUC_lexicon_tags_16 },
    { 0xc5e7c30d, SUC_lexicon_tags_25 },
    { 0x9647c30c, SUC_lexicon_tags_18 },
    { 0xace7c30f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd947c312, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c7c317, SUC_lexicon_tags_14 },
    { 0xc67c318, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67c31c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc7c321, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe307c323, SUC_lexicon_tags_12 },
    { 0xc6e7c323, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe467c326, SUC_lexicon_tags_69 },
    { 0x8dc7c327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee7c32a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe27c32c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07c32f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6027c333, SUC_lexicon_tags_11 },
    { 0x9c47c334, SUC_lexicon_tags_14 },
    { 0xbd87c333, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a7c33b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9107c340, SUC_lexicon_tags_19 },
    { 0xc487c341, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1147c345, SUC_lexicon_tags_88 },
    { 0x5a47c346, SUC_lexicon_tags_30 },
    { 0x1107c346, SUC_lexicon_tags_17 },
    { 0xc367c346, SUC_lexicon_tags_6 },
    { 0x2947c349, SUC_lexicon_tags_37 },
    { 0x4f07c34a, SUC_lexicon_tags_25 },
    { 0xd2a7c34a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52c7c34d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe27c34f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x93a7c351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb067c354, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e07c358, SUC_lexicon_tags_12 },
    { 0x4027c358, SUC_lexicon_tags_0 },
    { 0xf647c35a, SUC_lexicon_tags_65 },
    { 0x7867c35b, SUC_lexicon_tags_8 },
    { 0x6247c35b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7a87c35e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87c361, SUC_lexicon_tags_37 },
    { 0x3287c361, SUC_lexicon_tags_9 },
    { 0x187c363, SUC_lexicon_tags_12 },
    { 0xd5a7c363, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6807c367, SUC_lexicon_tags_2 },
    { 0x8ae7c368, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb467c36b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e7c376, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a7c37a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e7c37d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb087c380, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e7c389, SUC_lexicon_tags_14 },
    { 0xe7c7c389, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdce7c38c, SUC_lexicon_tags_45 },
    { 0x5607c38d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe667c38f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a7c392, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa7c39e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3107c3a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe467c3a4, SUC_lexicon_tags_17 },
    { 0xb767c3a4, SUC_lexicon_tags_6 },
    { 0x3267c3a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe07c3a8, SUC_lexicon_tags_26 },
    { 0x71e7c3a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x71a7c3ab, SUC_lexicon_tags_2 },
    { 0xa087c3ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba7c3b3, SUC_lexicon_tags_1 },
    { 0xc547c3b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4727c3b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd467c3ba, SUC_lexicon_tags_32 },
    { 0x3ec7c3bb, SUC_lexicon_tags_11 },
    { 0x30a7c3bb, SUC_lexicon_tags_38 },
    { 0xec67c3bd, SUC_lexicon_tags_10 },
    { 0xd9a7c3be, SUC_lexicon_tags_34 },
    { 0xbd07c3bf, SUC_lexicon_tags_37 },
    { 0x6ac7c3be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec07c3c4, SUC_lexicon_tags_19 },
    { 0x9627c3c5, SUC_lexicon_tags_47 },
    { 0x5f47c3c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc7c3c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x607c3cb, SUC_lexicon_tags_25 },
    { 0x9687c3cc, SUC_lexicon_tags_1 },
    { 0x4707c3cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc1c7c3cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e7c3d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7727c3da, SUC_lexicon_tags_355 },
    { 0x6007c3db, SUC_lexicon_tags_37 },
    { 0xb647c3da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc567c3df, SUC_lexicon_tags_25 },
    { 0x8fa7c3df, SUC_lexicon_tags_0 },
    { 0x7f67c3e0, SUC_lexicon_tags_1645 },
    { 0x2187c3e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a7c3e5, SUC_lexicon_tags_27 },
    { 0x6a7c3e6, SUC_lexicon_tags_7 },
    { 0x7f87c3e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb667c3ea, SUC_lexicon_tags_87 },
    { 0x63a7c3ea, SUC_lexicon_tags_111 },
    { 0x74e7c3ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427c3f1, SUC_lexicon_tags_77 },
    { 0x6547c3f1, SUC_lexicon_tags_17 },
    { 0x2f87c3f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4927c3f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b87c3f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c67c3fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc327c404, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac7c408, SUC_lexicon_tags_87 },
    { 0xab07c409, SUC_lexicon_tags_12 },
    { 0x927c40a, SUC_lexicon_tags_43 },
    { 0x5587c40b, SUC_lexicon_tags_25 },
    { 0x23a7c409, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07c411, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x73e7c413, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3107c417, SUC_lexicon_tags_34 },
    { 0x9cc7c418, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a7c41b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3587c41d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1127c422, SUC_lexicon_tags_39 },
    { 0x9707c422, SUC_lexicon_tags_37 },
    { 0xa787c424, SUC_lexicon_tags_12 },
    { 0x5a47c424, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2627c42a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb187c42c, SUC_lexicon_tags_38 },
    { 0xa427c42d, SUC_lexicon_tags_8 },
    { 0x8e67c42e, SUC_lexicon_tags_38 },
    { 0xd07c42e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdf27c431, SUC_lexicon_tags_7 },
    { 0x8f27c431, SUC_lexicon_tags_49 },
    { 0x2b07c433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9827c435, SUC_lexicon_tags_198 },
    { 0x8cc7c436, SUC_lexicon_tags_27 },
    { 0xde7c437, SUC_lexicon_tags_36 },
    { 0x3c87c438, SUC_lexicon_tags_32 },
    { 0x5507c436, SUC_lexicon_tags_25 },
    { 0x7ba7c43a, SUC_lexicon_tags_37 },
    { 0xfc87c43b, SUC_lexicon_tags_43 },
    { 0xbec7c43c, SUC_lexicon_tags_6 },
    { 0x8607c438, SUC_lexicon_tags_15 },
    { 0x5c27c435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe7c441, SUC_lexicon_tags_35 },
    { 0xdfa7c442, SUC_lexicon_tags_17 },
    { 0x1e07c442, SUC_lexicon_tags_38 },
    { 0x7c27c444, SUC_lexicon_tags_26 },
    { 0xb107c443, SUC_lexicon_tags_25 },
    { 0x3467c446, SUC_lexicon_tags_17 },
    { 0x1047c444, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2067c449, SUC_lexicon_tags_29 },
    { 0xc747c44a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7c44f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1a7c451, SUC_lexicon_tags_11 },
    { 0x6627c452, SUC_lexicon_tags_23 },
    { 0x8527c452, SUC_lexicon_tags_2 },
    { 0xd967c454, SUC_lexicon_tags_63 },
    { 0x6d07c455, SUC_lexicon_tags_25 },
    { 0x2d27c454, SUC_lexicon_tags_8 },
    { 0x9ec7c457, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc7c45b, SUC_lexicon_tags_39 },
    { 0x82c7c45c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3687c464, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5247c468, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe7c46b, SUC_lexicon_tags_1 },
    { 0x8f47c46c, SUC_lexicon_tags_17 },
    { 0x7f67c46b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4107c46f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8667c476, SUC_lexicon_tags_47 },
    { 0x1887c477, SUC_lexicon_tags_10 },
    { 0x427c478, SUC_lexicon_tags_11 },
    { 0x3427c479, SUC_lexicon_tags_15 },
    { 0xc27c47a, SUC_lexicon_tags_1 },
    { 0x427c478, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4867c47d, SUC_lexicon_tags_87 },
    { 0xd407c47e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4647c484, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c67c487, SUC_lexicon_tags_82 },
    { 0xd687c487, SUC_lexicon_tags_25 },
    { 0x30e7c489, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147c48d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4507c48f, SUC_lexicon_tags_15 },
    { 0x987c490, SUC_lexicon_tags_46 },
    { 0x9347c491, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7b27c493, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9c7c495, SUC_lexicon_tags_17 },
    { 0x6167c495, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b47c49a, SUC_lexicon_tags_1 },
    { 0x9247c49b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2b87c49d, SUC_lexicon_tags_15 },
    { 0x4b27c49e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb627c4a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2607c4a7, SUC_lexicon_tags_46 },
    { 0xc4a7c4a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de7c4ab, SUC_lexicon_tags_29 },
    { 0xe367c4ac, SUC_lexicon_tags_48 },
    { 0xf647c4ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8a87c4af, SUC_lexicon_tags_30 },
    { 0x6447c4b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de7c4b4, SUC_lexicon_tags_4 },
    { 0x78e7c4b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a7c4b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb807c4c7, SUC_lexicon_tags_25 },
    { 0xf647c4c8, SUC_lexicon_tags_1 },
    { 0x4ee7c4c9, SUC_lexicon_tags_11 },
    { 0x4867c4ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa427c4cf, SUC_lexicon_tags_23 },
    { 0xb707c4cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c7c4d8, SUC_lexicon_tags_15 },
    { 0xb367c4d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x947c4dc, SUC_lexicon_tags_21 },
    { 0x7e87c4dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf947c4df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd07c4e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd67c4e5, SUC_lexicon_tags_32 },
    { 0xeb47c4e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c7c4ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3927c4ec, SUC_lexicon_tags_46 },
    { 0x3067c4ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe207c4f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b27c4f7, SUC_lexicon_tags_12 },
    { 0x4087c4f7, SUC_lexicon_tags_10 },
    { 0x1967c4f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcee7c4fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd407c4fe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b47c501, SUC_lexicon_tags_14 },
    { 0xee47c502, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e7c50b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd47c50e, SUC_lexicon_tags_44 },
    { 0x4627c50f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9907c511, SUC_lexicon_tags_11 },
    { 0x6247c511, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xab67c515, SUC_lexicon_tags_11 },
    { 0x6347c516, SUC_lexicon_tags_4 },
    { 0x5907c515, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc7c51f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d07c52d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb707c533, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e7c536, SUC_lexicon_tags_33 },
    { 0x7ca7c537, SUC_lexicon_tags_27 },
    { 0x1367c537, SUC_lexicon_tags_25 },
    { 0x6487c537, SUC_lexicon_tags_47 },
    { 0x4c47c53a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6007c53e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7c542, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc27c545, SUC_lexicon_tags_38 },
    { 0xfb47c546, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a7c54e, SUC_lexicon_tags_11 },
    { 0xb447c54e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a7c553, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2447c55b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70a7c55d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a7c561, SUC_lexicon_tags_39 },
    { 0x1067c562, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa107c567, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fc7c569, SUC_lexicon_tags_9 },
    { 0x5ec7c56a, SUC_lexicon_tags_37 },
    { 0x85c7c56b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb487c56d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a07c571, SUC_lexicon_tags_63 },
    { 0xe8a7c572, SUC_lexicon_tags_17 },
    { 0x3507c573, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2787c575, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2287c577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe7c57b, SUC_lexicon_tags_27 },
    { 0xab87c57c, SUC_lexicon_tags_25 },
    { 0x1927c57b, SUC_lexicon_tags_7 },
    { 0xe7e7c57e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca7c581, SUC_lexicon_tags_2 },
    { 0x7827c581, SUC_lexicon_tags_7 },
    { 0x2e67c581, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40a7c585, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb247c587, SUC_lexicon_tags_31 },
    { 0x3467c588, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6507c58a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc527c58d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7c590, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe067c592, SUC_lexicon_tags_0 },
    { 0x3067c593, SUC_lexicon_tags_27 },
    { 0xec87c593, SUC_lexicon_tags_37 },
    { 0xe267c593, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a87c59d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e7c5a1, SUC_lexicon_tags_5 },
    { 0x4dc7c5a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae7c5a7, SUC_lexicon_tags_7 },
    { 0x4867c5a7, SUC_lexicon_tags_19 },
    { 0x7807c5a9, SUC_lexicon_tags_14 },
    { 0x40c7c5aa, SUC_lexicon_tags_12 },
    { 0xa47c5ab, SUC_lexicon_tags_60 },
    { 0xc67c5aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc547c5af, SUC_lexicon_tags_7 },
    { 0xd187c5b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93e7c5b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca67c5b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e7c5b8, SUC_lexicon_tags_25 },
    { 0x2127c5b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a67c5bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7347c5bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcd87c5c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9847c5c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb887c5c5, SUC_lexicon_tags_37 },
    { 0xa2c7c5c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc7c5cb, SUC_lexicon_tags_77 },
    { 0x9707c5cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e7c5d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2747c5d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ec7c5d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4567c5d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ae7c5da, SUC_lexicon_tags_56 },
    { 0x5d47c5da, SUC_lexicon_tags_33 },
    { 0x4c67c5dc, SUC_lexicon_tags_15 },
    { 0xa927c5dd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x907c5e0, SUC_lexicon_tags_40 },
    { 0xf907c5e0, SUC_lexicon_tags_15 },
    { 0x47c5e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae87c5e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6bc7c5e6, SUC_lexicon_tags_49 },
    { 0xbc87c5e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc7c5eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec7c5ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e7c5f2, SUC_lexicon_tags_19 },
    { 0x9e87c5f3, SUC_lexicon_tags_17 },
    { 0xcf67c5f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc7c5fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3727c5fc, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbda7c600, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5387c602, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867c606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2547c60a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa147c60e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb047c614, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67c61d, SUC_lexicon_tags_25 },
    { 0xe5a7c61e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50a7c620, SUC_lexicon_tags_27 },
    { 0xb827c620, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea7c624, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb607c626, SUC_lexicon_tags_43 },
    { 0xf9c7c626, SUC_lexicon_tags_87 },
    { 0x53e7c626, SUC_lexicon_tags_8 },
    { 0x9787c627, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e27c62c, SUC_lexicon_tags_38 },
    { 0x22e7c62c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda47c632, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e7c636, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e7c639, SUC_lexicon_tags_1 },
    { 0x4f47c63a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdee7c63c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1a7c63e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9967c641, SUC_lexicon_tags_1 },
    { 0xd227c642, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x127c645, SUC_lexicon_tags_17 },
    { 0x92e7c646, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3947c648, SUC_lexicon_tags_2 },
    { 0x2a87c649, SUC_lexicon_tags_1 },
    { 0xea87c64a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe207c64d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e7c653, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae7c656, SUC_lexicon_tags_1 },
    { 0x9767c657, SUC_lexicon_tags_17 },
    { 0x9e67c656, SUC_lexicon_tags_38 },
    { 0x5e47c659, SUC_lexicon_tags_14 },
    { 0xcd47c65a, SUC_lexicon_tags_33 },
    { 0x947c65a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60e7c65d, SUC_lexicon_tags_19 },
    { 0xbfc7c65d, SUC_lexicon_tags_12 },
    { 0x4c87c65d, SUC_lexicon_tags_11 },
    { 0x8f67c660, SUC_lexicon_tags_11 },
    { 0xe747c660, SUC_lexicon_tags_6 },
    { 0x4407c662, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa127c669, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9de7c66b, SUC_lexicon_tags_16 },
    { 0x77a7c66b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a7c66f, SUC_lexicon_tags_48 },
    { 0x6467c670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe207c677, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa87c679, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c67c67b, SUC_lexicon_tags_113 },
    { 0x8b67c67c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc7c683, SUC_lexicon_tags_16 },
    { 0xb267c683, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe1e7c686, SUC_lexicon_tags_63 },
    { 0x2607c686, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e7c68f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf547c694, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa127c696, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd07c698, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c7c69c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe7c6a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc87c6ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdce7c6af, SUC_lexicon_tags_24 },
    { 0x867c6af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467c6b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7287c6b8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xae27c6ba, SUC_lexicon_tags_17 },
    { 0xba27c6bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6de7c6bd, SUC_lexicon_tags_32 },
    { 0xba7c6be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc67c6c0, SUC_lexicon_tags_9 },
    { 0xca27c6c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc987c6c8, SUC_lexicon_tags_63 },
    { 0x34c7c6c9, SUC_lexicon_tags_0 },
    { 0x1887c6ca, SUC_lexicon_tags_15 },
    { 0xd747c6cb, SUC_lexicon_tags_29 },
    { 0x46a7c6ca, SUC_lexicon_tags_4 },
    { 0x7e27c6cd, SUC_lexicon_tags_6 },
    { 0x3a7c6cd, SUC_lexicon_tags_0 },
    { 0x3f27c6c9, SUC_lexicon_tags_1 },
    { 0x8747c6d0, SUC_lexicon_tags_124 },
    { 0x47a7c6d1, SUC_lexicon_tags_25 },
    { 0xeca7c6d2, SUC_lexicon_tags_37 },
    { 0xc7e7c6cd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x4367c6d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe07c6dc, SUC_lexicon_tags_15 },
    { 0x3347c6dd, SUC_lexicon_tags_37 },
    { 0xcd67c6dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a7c6e4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5487c6e6, SUC_lexicon_tags_25 },
    { 0x3f47c6e7, SUC_lexicon_tags_61 },
    { 0x3ca7c6e8, SUC_lexicon_tags_31 },
    { 0xa2c7c6e8, SUC_lexicon_tags_23 },
    { 0x287c6e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf607c6f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd547c6f2, SUC_lexicon_tags_1 },
    { 0x187c6f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd487c6f5, SUC_lexicon_tags_60 },
    { 0x60a7c6f6, SUC_lexicon_tags_68 },
    { 0x7887c6f7, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xc887c6fa, SUC_lexicon_tags_11 },
    { 0x6f67c6fb, SUC_lexicon_tags_37 },
    { 0xf627c6fc, SUC_lexicon_tags_11 },
    { 0x44c7c6fd, SUC_lexicon_tags_7 },
    { 0xa8c7c6fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b07c702, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x86e7c704, SUC_lexicon_tags_7 },
    { 0xd007c705, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c7c709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e7c70c, SUC_lexicon_tags_7 },
    { 0x6a07c70d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f87c711, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd707c718, SUC_lexicon_tags_61 },
    { 0xce27c719, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a67c71c, SUC_lexicon_tags_56 },
    { 0xd4e7c71d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2827c720, SUC_lexicon_tags_25 },
    { 0xcf47c721, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c47c724, SUC_lexicon_tags_13 },
    { 0x1d87c724, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x93e7c727, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f47c72f, SUC_lexicon_tags_15 },
    { 0x30e7c72f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf187c732, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be7c73c, SUC_lexicon_tags_47 },
    { 0xff87c73d, SUC_lexicon_tags_47 },
    { 0x45e7c73e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x307c740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda87c742, SUC_lexicon_tags_43 },
    { 0x6307c743, SUC_lexicon_tags_3 },
    { 0xc347c744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a7c74b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9da7c74d, SUC_lexicon_tags_25 },
    { 0xf727c74e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a7c755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87c75d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6207c75f, SUC_lexicon_tags_37 },
    { 0x8027c760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0e7c762, SUC_lexicon_tags_15 },
    { 0xc7e7c763, SUC_lexicon_tags_13 },
    { 0x83e7c764, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc7c766, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc387c76a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a7c76f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd47c771, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f87c775, SUC_lexicon_tags_551 },
    { 0, NULL },
    { 0x4b67c777, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2747c77a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b07c782, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3907c784, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a7c788, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86e7c78a, SUC_lexicon_tags_26 },
    { 0x8807c78a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dc7c78d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e7c792, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7c795, SUC_lexicon_tags_17 },
    { 0xeb27c796, SUC_lexicon_tags_61 },
    { 0x7867c796, SUC_lexicon_tags_19 },
    { 0xe447c798, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ec7c79a, SUC_lexicon_tags_218 },
    { 0xcda7c79a, SUC_lexicon_tags_7 },
    { 0x8b87c79c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a7c79f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcec7c7a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa07c7a5, SUC_lexicon_tags_17 },
    { 0x4127c7a6, SUC_lexicon_tags_37 },
    { 0x6c7c7a7, SUC_lexicon_tags_19 },
    { 0xb067c7a8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa7e7c7aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd107c7ae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd607c7b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8467c7b4, SUC_lexicon_tags_2 },
    { 0x327c7b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfc7c7b7, SUC_lexicon_tags_0 },
    { 0xaca7c7b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e47c7ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf107c7bf, SUC_lexicon_tags_33 },
    { 0x6f47c7c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb67c7c3, SUC_lexicon_tags_17 },
    { 0x3c07c7c4, SUC_lexicon_tags_29 },
    { 0xacc7c7c3, SUC_lexicon_tags_12 },
    { 0xaae7c7c6, SUC_lexicon_tags_48 },
    { 0x1147c7c4, SUC_lexicon_tags_7 },
    { 0x4ac7c7c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8987c7d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aa7c7d5, SUC_lexicon_tags_15 },
    { 0x3907c7d6, SUC_lexicon_tags_29 },
    { 0x367c7d6, SUC_lexicon_tags_41 },
    { 0xb8e7c7d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa3a7c7da, SUC_lexicon_tags_19 },
    { 0x5b87c7db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd927c7e0, SUC_lexicon_tags_25 },
    { 0xa167c7e0, SUC_lexicon_tags_15 },
    { 0xa107c7e0, SUC_lexicon_tags_19 },
    { 0xa767c7e3, SUC_lexicon_tags_43 },
    { 0xaba7c7e4, SUC_lexicon_tags_48 },
    { 0xca7c7e4, SUC_lexicon_tags_1 },
    { 0x7b47c7e6, SUC_lexicon_tags_11 },
    { 0xbf27c7e6, SUC_lexicon_tags_12 },
    { 0x6427c7e8, SUC_lexicon_tags_7 },
    { 0xca87c7e9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c7c7ec, SUC_lexicon_tags_17 },
    { 0x4b27c7ed, SUC_lexicon_tags_40 },
    { 0xf627c7ed, SUC_lexicon_tags_15 },
    { 0x6107c7ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb567c7f3, SUC_lexicon_tags_9 },
    { 0x1b67c7f4, SUC_lexicon_tags_17 },
    { 0x4067c7f5, SUC_lexicon_tags_2 },
    { 0xfd27c7f6, SUC_lexicon_tags_37 },
    { 0x327c7f4, SUC_lexicon_tags_29 },
    { 0x8447c7f8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d67c7fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa487c803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc527c807, SUC_lexicon_tags_12 },
    { 0x9347c808, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a7c80f, SUC_lexicon_tags_9 },
    { 0xe447c810, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b27c812, SUC_lexicon_tags_2 },
    { 0x6ac7c813, SUC_lexicon_tags_6 },
    { 0xca47c814, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a7c818, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9727c81a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1427c821, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7027c825, SUC_lexicon_tags_7 },
    { 0x5987c826, SUC_lexicon_tags_38 },
    { 0x4d07c827, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3067c829, SUC_lexicon_tags_2 },
    { 0xd0c7c82a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc807c82f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcea7c831, SUC_lexicon_tags_48 },
    { 0x33c7c832, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c7c836, SUC_lexicon_tags_37 },
    { 0xb1a7c836, SUC_lexicon_tags_12 },
    { 0xe7c838, SUC_lexicon_tags_1 },
    { 0xad47c839, SUC_lexicon_tags_43 },
    { 0x8c27c83a, SUC_lexicon_tags_18 },
    { 0x3347c83b, SUC_lexicon_tags_12 },
    { 0x8fe7c83a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3407c840, SUC_lexicon_tags_151 },
    { 0x7707c840, SUC_lexicon_tags_3 },
    { 0x22e7c842, SUC_lexicon_tags_11 },
    { 0x767c842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5267c847, SUC_lexicon_tags_19 },
    { 0x28a7c848, SUC_lexicon_tags_1 },
    { 0xde27c849, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae7c850, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e7c854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe7c859, SUC_lexicon_tags_2 },
    { 0x1f27c859, SUC_lexicon_tags_1 },
    { 0xbb67c859, SUC_lexicon_tags_11 },
    { 0x42a7c85c, SUC_lexicon_tags_23 },
    { 0x8c87c85d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2807c85f, SUC_lexicon_tags_17 },
    { 0x7e27c85f, SUC_lexicon_tags_32 },
    { 0xd887c860, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a7c868, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae7c86b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x17e7c86d, SUC_lexicon_tags_17 },
    { 0xc007c86e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd07c871, SUC_lexicon_tags_17 },
    { 0x55c7c872, SUC_lexicon_tags_31 },
    { 0x5427c873, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd47c876, SUC_lexicon_tags_39 },
    { 0x28a7c877, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe527c879, SUC_lexicon_tags_42 },
    { 0x7107c87a, SUC_lexicon_tags_7 },
    { 0x1c67c87b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf947c87d, SUC_lexicon_tags_17 },
    { 0x9927c87d, SUC_lexicon_tags_60 },
    { 0x7467c87e, SUC_lexicon_tags_37 },
    { 0x3ca7c87e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce7c882, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7c885, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c7c889, SUC_lexicon_tags_1025 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c88d, SUC_lexicon_tags_25 },
    { 0x3c27c88d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd67c890, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2807c892, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd87c894, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf847c896, SUC_lexicon_tags_23 },
    { 0x6c47c897, SUC_lexicon_tags_16 },
    { 0x587c898, SUC_lexicon_tags_9 },
    { 0xf927c899, SUC_lexicon_tags_7 },
    { 0xd727c89a, SUC_lexicon_tags_79 },
    { 0x1aa7c896, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31a7c89d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe7c8a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3047c8a5, SUC_lexicon_tags_17 },
    { 0x6d27c8a6, SUC_lexicon_tags_1 },
    { 0x4c47c8a7, SUC_lexicon_tags_27 },
    { 0x5907c8a7, SUC_lexicon_tags_47 },
    { 0x97c7c8a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be7c8ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a27c8af, SUC_lexicon_tags_37 },
    { 0x607c8b0, SUC_lexicon_tags_14 },
    { 0xde7c8b1, SUC_lexicon_tags_1 },
    { 0x3ba7c8b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3f67c8b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46c7c8b6, SUC_lexicon_tags_11 },
    { 0x9ee7c8b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bc7c8b9, SUC_lexicon_tags_30 },
    { 0x70e7c8ba, SUC_lexicon_tags_3 },
    { 0x2287c8bb, SUC_lexicon_tags_26 },
    { 0x467c8bb, SUC_lexicon_tags_11 },
    { 0x7f87c8bd, SUC_lexicon_tags_19 },
    { 0x7827c8be, SUC_lexicon_tags_2 },
    { 0xfb27c8b9, SUC_lexicon_tags_30 },
    { 0x39e7c8c0, SUC_lexicon_tags_27 },
    { 0x96a7c8c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1787c8c3, SUC_lexicon_tags_30 },
    { 0x53a7c8c4, SUC_lexicon_tags_87 },
    { 0x98e7c8c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4367c8c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc367c8cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8847c8d1, SUC_lexicon_tags_37 },
    { 0x6887c8d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe967c8d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0a7c8d7, SUC_lexicon_tags_7 },
    { 0x5fc7c8d8, SUC_lexicon_tags_25 },
    { 0x73c7c8d8, SUC_lexicon_tags_146 },
    { 0xabc7c8da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b07c8df, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6767c8e1, SUC_lexicon_tags_25 },
    { 0xeb47c8e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbce7c8e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea07c8e7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xfce7c8e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x46a7c8eb, SUC_lexicon_tags_27 },
    { 0x7227c8eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c47c8ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a67c8f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb27c8ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2007c901, SUC_lexicon_tags_65 },
    { 0xba47c902, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf107c904, SUC_lexicon_tags_39 },
    { 0xf387c904, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47c908, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce7c915, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf767c91d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8b27c91f, SUC_lexicon_tags_37 },
    { 0xa8e7c91f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe827c924, SUC_lexicon_tags_0 },
    { 0x1e87c925, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf87c928, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc807c932, SUC_lexicon_tags_15 },
    { 0x27e7c932, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4127c937, SUC_lexicon_tags_12 },
    { 0xdc87c938, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a7c93c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba7c940, SUC_lexicon_tags_126 },
    { 0x8ec7c941, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f67c945, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e7c948, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb87c94b, SUC_lexicon_tags_38 },
    { 0x2d07c94c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa7c94f, SUC_lexicon_tags_15 },
    { 0x1be7c950, SUC_lexicon_tags_17 },
    { 0xed07c94f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfa7c953, SUC_lexicon_tags_11 },
    { 0xb147c953, SUC_lexicon_tags_21 },
    { 0x6707c955, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4b27c957, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46a7c959, SUC_lexicon_tags_17 },
    { 0xc467c95a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9c7c95c, SUC_lexicon_tags_11 },
    { 0x53a7c95d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9167c95f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3a7c961, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1607c965, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xab67c967, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f47c96a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b47c96d, SUC_lexicon_tags_65 },
    { 0xc967c96e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb27c970, SUC_lexicon_tags_2 },
    { 0x6d87c971, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb447c973, SUC_lexicon_tags_42 },
    { 0x5b27c974, SUC_lexicon_tags_1 },
    { 0x7467c975, SUC_lexicon_tags_17 },
    { 0xda07c976, SUC_lexicon_tags_12 },
    { 0x6aa7c976, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9927c97b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe907c980, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2527c985, SUC_lexicon_tags_4 },
    { 0x98c7c986, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7447c988, SUC_lexicon_tags_15 },
    { 0xe727c989, SUC_lexicon_tags_63 },
    { 0x5507c98a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe2c7c98c, SUC_lexicon_tags_11 },
    { 0x41e7c98d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7fa7c98f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe307c991, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac7c994, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f27c99a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc427c9a2, SUC_lexicon_tags_30 },
    { 0xb47c9a3, SUC_lexicon_tags_27 },
    { 0x5ba7c9a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd47c9a8, SUC_lexicon_tags_7 },
    { 0x9a87c9a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e07c9ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1387c9ad, SUC_lexicon_tags_4 },
    { 0x9c47c9ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7c9b0, SUC_lexicon_tags_37 },
    { 0x9407c9b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfbc7c9b3, SUC_lexicon_tags_1103 },
    { 0xacc7c9b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc47c9b9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd847c9bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8167c9c0, SUC_lexicon_tags_12 },
    { 0xc847c9c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a7c9c4, SUC_lexicon_tags_13 },
    { 0xce67c9c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd307c9c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa87c9cc, SUC_lexicon_tags_11 },
    { 0x7387c9cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5727c9d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8527c9d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1147c9d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa387c9d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c7c9de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c7c9e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb07c9e7, SUC_lexicon_tags_1 },
    { 0xa947c9e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8067c9ea, SUC_lexicon_tags_166 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3247c9ef, SUC_lexicon_tags_32 },
    { 0x9387c9f0, SUC_lexicon_tags_2 },
    { 0x14a7c9ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c7c9f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2227c9f7, SUC_lexicon_tags_17 },
    { 0x9c87c9f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7e7c9fa, SUC_lexicon_tags_79 },
    { 0x28e7c9fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6867c9fd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e7ca01, SUC_lexicon_tags_372 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c7ca04, SUC_lexicon_tags_14 },
    { 0x8b07ca04, SUC_lexicon_tags_17 },
    { 0xa507ca06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a7ca09, SUC_lexicon_tags_60 },
    { 0x30e7ca09, SUC_lexicon_tags_7 },
    { 0x3a27ca0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f87ca14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ea7ca16, SUC_lexicon_tags_330 },
    { 0x12c7ca17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa507ca19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28c7ca1b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3607ca1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4167ca21, SUC_lexicon_tags_11 },
    { 0xd527ca21, SUC_lexicon_tags_17 },
    { 0xc907ca23, SUC_lexicon_tags_7 },
    { 0xcf47ca22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2f47ca26, SUC_lexicon_tags_15 },
    { 0xbb67ca27, SUC_lexicon_tags_60 },
    { 0x9c7ca28, SUC_lexicon_tags_7 },
    { 0x20c7ca28, SUC_lexicon_tags_65 },
    { 0xfd67ca28, SUC_lexicon_tags_7 },
    { 0x2047ca2b, SUC_lexicon_tags_17 },
    { 0xa587ca26, SUC_lexicon_tags_37 },
    { 0xbe87ca2b, SUC_lexicon_tags_37 },
    { 0xedc7ca2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9307ca30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd027ca32, SUC_lexicon_tags_11 },
    { 0xfc27ca33, SUC_lexicon_tags_37 },
    { 0x2d07ca32, SUC_lexicon_tags_60 },
    { 0xa767ca35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c7ca38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8a7ca3a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc7ca3e, SUC_lexicon_tags_34 },
    { 0x8aa7ca3e, SUC_lexicon_tags_7 },
    { 0x5c67ca40, SUC_lexicon_tags_15 },
    { 0xd2a7ca41, SUC_lexicon_tags_0 },
    { 0xec47ca42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda27ca48, SUC_lexicon_tags_31 },
    { 0xb887ca49, SUC_lexicon_tags_45 },
    { 0xfd27ca48, SUC_lexicon_tags_1 },
    { 0xb207ca4b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca7ca4e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4667ca52, SUC_lexicon_tags_26 },
    { 0xf4e7ca53, SUC_lexicon_tags_27 },
    { 0x3da7ca54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b27ca58, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb447ca5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9667ca5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1727ca63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd87ca68, SUC_lexicon_tags_25 },
    { 0xe527ca68, SUC_lexicon_tags_146 },
    { 0xf147ca6a, SUC_lexicon_tags_37 },
    { 0x4667ca6b, SUC_lexicon_tags_37 },
    { 0x1307ca6b, SUC_lexicon_tags_14 },
    { 0x4887ca6d, SUC_lexicon_tags_12 },
    { 0xca47ca6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc67ca70, SUC_lexicon_tags_7 },
    { 0x1427ca71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd67ca73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ca7ca75, SUC_lexicon_tags_2 },
    { 0xd587ca76, SUC_lexicon_tags_55 },
    { 0xdb47ca77, SUC_lexicon_tags_25 },
    { 0x9da7ca78, SUC_lexicon_tags_7 },
    { 0xaa07ca77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee27ca7b, SUC_lexicon_tags_16 },
    { 0xb5c7ca7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fc7ca7e, SUC_lexicon_tags_31 },
    { 0x7ec7ca7f, SUC_lexicon_tags_12 },
    { 0x6207ca7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7667ca86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7587ca88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7587ca8b, SUC_lexicon_tags_8 },
    { 0xa1c7ca8c, SUC_lexicon_tags_68 },
    { 0x8727ca8c, SUC_lexicon_tags_47 },
    { 0x6767ca8e, SUC_lexicon_tags_2 },
    { 0x6387ca8f, SUC_lexicon_tags_0 },
    { 0xc807ca8b, SUC_lexicon_tags_1610 },
    { 0xa67ca91, SUC_lexicon_tags_25 },
    { 0xe5a7ca8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b67ca95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d87ca9b, SUC_lexicon_tags_17 },
    { 0x8f07ca9c, SUC_lexicon_tags_68 },
    { 0xca67ca9b, SUC_lexicon_tags_21 },
    { 0x40c7ca9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf87caa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb787caa7, SUC_lexicon_tags_29 },
    { 0xac87caa7, SUC_lexicon_tags_23 },
    { 0xc1e7caa9, SUC_lexicon_tags_25 },
    { 0x3bc7caa9, SUC_lexicon_tags_47 },
    { 0x2b27caa9, SUC_lexicon_tags_37 },
    { 0xd587caac, SUC_lexicon_tags_25 },
    { 0x3e87caac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a7caaf, SUC_lexicon_tags_15 },
    { 0x37a7caaf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x37c7cab2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe267cab4, SUC_lexicon_tags_15 },
    { 0x387cab5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fa7cab7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b87caba, SUC_lexicon_tags_11 },
    { 0x9907caba, SUC_lexicon_tags_37 },
    { 0xc527cabc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f27cabe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc7cac2, SUC_lexicon_tags_25 },
    { 0x43c7cac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab67cac6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83e7cac8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e7cacb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2767cad0, SUC_lexicon_tags_2 },
    { 0x6667cad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba67cadb, SUC_lexicon_tags_42 },
    { 0xa107cadb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb47cade, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa587cae1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5267cae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9de7cae7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7caec, SUC_lexicon_tags_15 },
    { 0x9227caec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd387caf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4987cafa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e7cafe, SUC_lexicon_tags_12 },
    { 0x1807caff, SUC_lexicon_tags_19 },
    { 0x8507cb00, SUC_lexicon_tags_28 },
    { 0x727cb01, SUC_lexicon_tags_33 },
    { 0x5267cb02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac27cb04, SUC_lexicon_tags_28 },
    { 0x3da7cb04, SUC_lexicon_tags_235 },
    { 0xaf67cb06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4447cb0b, SUC_lexicon_tags_57 },
    { 0x1167cb0c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2627cb0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba7cb14, SUC_lexicon_tags_37 },
    { 0x41a7cb15, SUC_lexicon_tags_2 },
    { 0x2a47cb16, SUC_lexicon_tags_28 },
    { 0xdd47cb17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7c7cb19, SUC_lexicon_tags_11 },
    { 0xd507cb1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4767cb1c, SUC_lexicon_tags_37 },
    { 0x2fc7cb1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb707cb1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc507cb21, SUC_lexicon_tags_1 },
    { 0x3787cb21, SUC_lexicon_tags_12 },
    { 0x2ea7cb23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a7cb26, SUC_lexicon_tags_1 },
    { 0x6747cb26, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5fe7cb29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcf27cb2b, SUC_lexicon_tags_12 },
    { 0x3f67cb2c, SUC_lexicon_tags_37 },
    { 0x7be7cb2d, SUC_lexicon_tags_12 },
    { 0x8c07cb2e, SUC_lexicon_tags_15 },
    { 0xf8e7cb2e, SUC_lexicon_tags_14 },
    { 0xbf07cb2c, SUC_lexicon_tags_27 },
    { 0x9787cb31, SUC_lexicon_tags_15 },
    { 0x46e7cb2d, SUC_lexicon_tags_60 },
    { 0x8cc7cb2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd07cb37, SUC_lexicon_tags_25 },
    { 0x1387cb37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf987cb3a, SUC_lexicon_tags_7 },
    { 0x8ae7cb3b, SUC_lexicon_tags_15 },
    { 0xb627cb3b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b07cb40, SUC_lexicon_tags_27 },
    { 0xea07cb41, SUC_lexicon_tags_65 },
    { 0x7bc7cb41, SUC_lexicon_tags_6 },
    { 0xdbe7cb42, SUC_lexicon_tags_0 },
    { 0xee87cb44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xace7cb46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e67cb49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde07cb53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f87cb56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d27cb5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2787cb61, SUC_lexicon_tags_38 },
    { 0x47cb61, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d07cb66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf187cb68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c07cb6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1887cb6d, SUC_lexicon_tags_46 },
    { 0x6387cb6e, SUC_lexicon_tags_16 },
    { 0xfc47cb6e, SUC_lexicon_tags_11 },
    { 0xfe47cb70, SUC_lexicon_tags_57 },
    { 0x607cb70, SUC_lexicon_tags_0 },
    { 0x89a7cb71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5687cb74, SUC_lexicon_tags_17 },
    { 0x2f27cb74, SUC_lexicon_tags_1 },
    { 0xcfa7cb76, SUC_lexicon_tags_11 },
    { 0xb3e7cb75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4287cb7a, SUC_lexicon_tags_11 },
    { 0xb227cb7a, SUC_lexicon_tags_0 },
    { 0x76e7cb7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a7cb8c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea7cb90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc847cb92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe847cb94, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2947cb98, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xd767cb9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb3c7cb9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb507cba1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x31a7cba3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d27cba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89a7cba8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa7cbad, SUC_lexicon_tags_100 },
    { 0x5c07cbad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec7cbb4, SUC_lexicon_tags_1 },
    { 0x67cbb5, SUC_lexicon_tags_6 },
    { 0x78c7cbb5, SUC_lexicon_tags_33 },
    { 0xaf27cbb7, SUC_lexicon_tags_70 },
    { 0xcd47cbb6, SUC_lexicon_tags_25 },
    { 0x4cc7cbb7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f67cbbd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb767cbbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe67cbc3, SUC_lexicon_tags_29 },
    { 0x2427cbc4, SUC_lexicon_tags_19 },
    { 0xd2a7cbc4, SUC_lexicon_tags_15 },
    { 0x5167cbc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb27cbc9, SUC_lexicon_tags_13 },
    { 0xe947cbca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a7cbcf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7cbd2, SUC_lexicon_tags_12 },
    { 0x1aa7cbd3, SUC_lexicon_tags_0 },
    { 0x2687cbd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb847cbd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ee7cbda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc7cbde, SUC_lexicon_tags_27 },
    { 0xc0c7cbdf, SUC_lexicon_tags_33 },
    { 0x7247cbde, SUC_lexicon_tags_1 },
    { 0x70e7cbe1, SUC_lexicon_tags_0 },
    { 0xee87cbe2, SUC_lexicon_tags_33 },
    { 0x94c7cbe3, SUC_lexicon_tags_11 },
    { 0x24a7cbe3, SUC_lexicon_tags_29 },
    { 0xe367cbe4, SUC_lexicon_tags_29 },
    { 0xbe07cbe5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e7cbe9, SUC_lexicon_tags_64 },
    { 0xa167cbea, SUC_lexicon_tags_19 },
    { 0x9ee7cbea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9c27cbed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb307cbf1, SUC_lexicon_tags_38 },
    { 0x3a07cbf2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7cbf6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807cbfa, SUC_lexicon_tags_9 },
    { 0xcd07cbfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x39c7cbfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca7cc04, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xff47cc07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace7cc0c, SUC_lexicon_tags_15 },
    { 0xc7c7cc0d, SUC_lexicon_tags_11 },
    { 0x4107cc0e, SUC_lexicon_tags_24 },
    { 0xf8c7cc0f, SUC_lexicon_tags_12 },
    { 0x2507cc10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8a7cc12, SUC_lexicon_tags_4 },
    { 0x1b67cc13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9987cc15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f07cc19, SUC_lexicon_tags_33 },
    { 0x2247cc19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfea7cc1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a7cc22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b87cc2e, SUC_lexicon_tags_17 },
    { 0x75e7cc2e, SUC_lexicon_tags_77 },
    { 0x827cc2e, SUC_lexicon_tags_45 },
    { 0xdc27cc30, SUC_lexicon_tags_25 },
    { 0xaae7cc32, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5687cc35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b07cc37, SUC_lexicon_tags_8 },
    { 0xe947cc38, SUC_lexicon_tags_47 },
    { 0x3e27cc38, SUC_lexicon_tags_2 },
    { 0xbd67cc3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc7cc3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2887cc40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb567cc44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2507cc47, SUC_lexicon_tags_11 },
    { 0x46c7cc47, SUC_lexicon_tags_11 },
    { 0xed67cc49, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6f27cc4b, SUC_lexicon_tags_0 },
    { 0x2e87cc4c, SUC_lexicon_tags_82 },
    { 0xcb67cc4d, SUC_lexicon_tags_215 },
    { 0x7a7cc4d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7cc52, SUC_lexicon_tags_17 },
    { 0xbfa7cc53, SUC_lexicon_tags_5 },
    { 0x80a7cc53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f87cc56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6307cc58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e87cc5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a7cc5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a7cc62, SUC_lexicon_tags_63 },
    { 0x44c7cc63, SUC_lexicon_tags_63 },
    { 0x4567cc64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c7cc68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe007cc6d, SUC_lexicon_tags_43 },
    { 0x39a7cc6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2607cc70, SUC_lexicon_tags_61 },
    { 0x90e7cc70, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x987cc73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa7cc76, SUC_lexicon_tags_28 },
    { 0x667cc77, SUC_lexicon_tags_198 },
    { 0xd727cc76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e47cc7c, SUC_lexicon_tags_49 },
    { 0xfa87cc7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e47cc82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac67cc86, SUC_lexicon_tags_12 },
    { 0xba87cc86, SUC_lexicon_tags_226 },
    { 0x3227cc87, SUC_lexicon_tags_23 },
    { 0xefa7cc89, SUC_lexicon_tags_49 },
    { 0x51a7cc8a, SUC_lexicon_tags_45 },
    { 0x9347cc86, SUC_lexicon_tags_11 },
    { 0xb4a7cc86, SUC_lexicon_tags_23 },
    { 0x6727cc88, SUC_lexicon_tags_17 },
    { 0x37c7cc8e, SUC_lexicon_tags_25 },
    { 0x6207cc8f, SUC_lexicon_tags_37 },
    { 0xde67cc86, SUC_lexicon_tags_1 },
    { 0x8247cc91, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa7cc95, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf647cc9b, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xba47cc9e, SUC_lexicon_tags_19 },
    { 0xa127cc9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e7cca2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4487cca4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf847ccb0, SUC_lexicon_tags_11 },
    { 0xdd67ccb1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc7ccb7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c47ccba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc767ccbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc7ccc3, SUC_lexicon_tags_11 },
    { 0xc5a7ccc3, SUC_lexicon_tags_15 },
    { 0xf3c7ccc5, SUC_lexicon_tags_11 },
    { 0xae47ccc6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4567ccca, SUC_lexicon_tags_37 },
    { 0x187ccca, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf07cccf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb187ccd2, SUC_lexicon_tags_15 },
    { 0xd727ccd3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2dc7ccd5, SUC_lexicon_tags_12 },
    { 0x8667ccd5, SUC_lexicon_tags_12 },
    { 0x2aa7ccd7, SUC_lexicon_tags_12 },
    { 0x7887ccd6, SUC_lexicon_tags_49 },
    { 0x37e7ccd8, SUC_lexicon_tags_16 },
    { 0x2847ccd8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7d27ccdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d27cce0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd467cce7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c7ccef, SUC_lexicon_tags_70 },
    { 0x11e7ccf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c7ccf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3927ccf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb387ccf7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae7ccfa, SUC_lexicon_tags_15 },
    { 0xcca7ccfa, SUC_lexicon_tags_106 },
    { 0x7d07ccfb, SUC_lexicon_tags_11 },
    { 0x6447ccfd, SUC_lexicon_tags_17 },
    { 0x8b67ccfe, SUC_lexicon_tags_43 },
    { 0x24a7ccff, SUC_lexicon_tags_6 },
    { 0x15c7ccfc, SUC_lexicon_tags_0 },
    { 0x87cd01, SUC_lexicon_tags_17 },
    { 0x4427cd01, SUC_lexicon_tags_34 },
    { 0xd507cd00, SUC_lexicon_tags_35 },
    { 0xaa27ccfb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa027cd07, SUC_lexicon_tags_1 },
    { 0xf4a7cd08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc707cd0d, SUC_lexicon_tags_63 },
    { 0x3807cd0d, SUC_lexicon_tags_7 },
    { 0x4087cd0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2bc7cd11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b47cd13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e7cd17, SUC_lexicon_tags_3 },
    { 0xa0a7cd18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66a7cd1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x527cd1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a7cd20, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac7cd27, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8267cd2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1a7cd2e, SUC_lexicon_tags_16 },
    { 0x48e7cd2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea07cd32, SUC_lexicon_tags_37 },
    { 0x59c7cd32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd547cd35, SUC_lexicon_tags_14 },
    { 0xb1a7cd36, SUC_lexicon_tags_33 },
    { 0x1567cd35, SUC_lexicon_tags_15 },
    { 0x3d47cd38, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7cd3b, SUC_lexicon_tags_30 },
    { 0xae87cd3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c7cd3e, SUC_lexicon_tags_2 },
    { 0xdfe7cd3e, SUC_lexicon_tags_47 },
    { 0xfa67cd40, SUC_lexicon_tags_15 },
    { 0x62e7cd3e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c7cd49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21e7cd4b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x9087cd4e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c7cd53, SUC_lexicon_tags_63 },
    { 0x1387cd53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2867cd58, SUC_lexicon_tags_1 },
    { 0x1ac7cd59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fa7cd5b, SUC_lexicon_tags_16 },
    { 0xba27cd5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c87cd5f, SUC_lexicon_tags_17 },
    { 0x5507cd5f, SUC_lexicon_tags_37 },
    { 0x32a7cd5f, SUC_lexicon_tags_60 },
    { 0x78a7cd62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb07cd67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc107cd6e, SUC_lexicon_tags_11 },
    { 0x1ae7cd6f, SUC_lexicon_tags_15 },
    { 0x5007cd70, SUC_lexicon_tags_45 },
    { 0x5f07cd71, SUC_lexicon_tags_31 },
    { 0x6707cd72, SUC_lexicon_tags_15 },
    { 0x87cd6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x62c7cd75, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa287cd77, SUC_lexicon_tags_14 },
    { 0xe427cd78, SUC_lexicon_tags_151 },
    { 0x9ca7cd77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4067cd7d, SUC_lexicon_tags_37 },
    { 0xe0c7cd7e, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c7cd81, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x447cd83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68a7cd85, SUC_lexicon_tags_23 },
    { 0x11e7cd86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4027cd8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7467cd91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad07cd93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a7cd99, SUC_lexicon_tags_17 },
    { 0x787cd99, SUC_lexicon_tags_25 },
    { 0x3167cd9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdfc7cd9d, SUC_lexicon_tags_11 },
    { 0x5ce7cd9e, SUC_lexicon_tags_2 },
    { 0xe0a7cd9d, SUC_lexicon_tags_33 },
    { 0x57a7cd9f, SUC_lexicon_tags_7 },
    { 0x1287cda1, SUC_lexicon_tags_25 },
    { 0xe07cd9d, SUC_lexicon_tags_14 },
    { 0x85c7cda3, SUC_lexicon_tags_15 },
    { 0xaf27cda4, SUC_lexicon_tags_40 },
    { 0x8267cda5, SUC_lexicon_tags_33 },
    { 0xc067cda6, SUC_lexicon_tags_17 },
    { 0xe487cd9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcce7cda9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b07cdb5, SUC_lexicon_tags_11 },
    { 0x7507cdb5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xad07cdb8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbec7cdba, SUC_lexicon_tags_12 },
    { 0xcb87cdbb, SUC_lexicon_tags_19 },
    { 0x23c7cdbc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x527cdbf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7727cdc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7cdc8, SUC_lexicon_tags_37 },
    { 0x23c7cdc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb207cdcc, SUC_lexicon_tags_10 },
    { 0xa227cdcc, SUC_lexicon_tags_27 },
    { 0x6a87cdce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e87cdd0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b27cdd5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97c7cdd7, SUC_lexicon_tags_43 },
    { 0x61c7cdd8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb167cddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd27cddd, SUC_lexicon_tags_33 },
    { 0xede7cddd, SUC_lexicon_tags_16 },
    { 0x1927cddd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1927cde1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1287cde4, SUC_lexicon_tags_17 },
    { 0x10e7cde5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9167cde7, SUC_lexicon_tags_6 },
    { 0x6987cde8, SUC_lexicon_tags_7 },
    { 0x5f87cde9, SUC_lexicon_tags_61 },
    { 0x7c67cde8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9787cded, SUC_lexicon_tags_25 },
    { 0x3327cded, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c7cdf4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f47cdf7, SUC_lexicon_tags_17 },
    { 0xf0a7cdf7, SUC_lexicon_tags_12 },
    { 0x107cdf7, SUC_lexicon_tags_11 },
    { 0xf107cdfa, SUC_lexicon_tags_34 },
    { 0x5247cdfb, SUC_lexicon_tags_12 },
    { 0x86e7cdfc, SUC_lexicon_tags_7 },
    { 0x9d67cdf9, SUC_lexicon_tags_25 },
    { 0xe827cdfe, SUC_lexicon_tags_6 },
    { 0xef07cdff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27ce03, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a7ce0b, SUC_lexicon_tags_33 },
    { 0xe247ce0b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe187ce15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce7ce18, SUC_lexicon_tags_25 },
    { 0x9c47ce19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3487ce1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc207ce1d, SUC_lexicon_tags_44 },
    { 0xd947ce1e, SUC_lexicon_tags_25 },
    { 0xb487ce1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1867ce22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed87ce26, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4ea7ce28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7ce2b, SUC_lexicon_tags_82 },
    { 0x4627ce2c, SUC_lexicon_tags_15 },
    { 0x6f47ce2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd87ce2f, SUC_lexicon_tags_26 },
    { 0x9ae7ce30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe47ce32, SUC_lexicon_tags_59 },
    { 0x6c87ce33, SUC_lexicon_tags_68 },
    { 0x87c7ce34, SUC_lexicon_tags_19 },
    { 0xce07ce33, SUC_lexicon_tags_12 },
    { 0x4767ce35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe27ce38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c7ce3b, SUC_lexicon_tags_47 },
    { 0xbee7ce3b, SUC_lexicon_tags_16 },
    { 0x79e7ce3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e7ce3f, SUC_lexicon_tags_87 },
    { 0xd9c7ce40, SUC_lexicon_tags_2 },
    { 0x46c7ce3f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4907ce4b, SUC_lexicon_tags_7 },
    { 0x8d67ce4c, SUC_lexicon_tags_26 },
    { 0x33c7ce4d, SUC_lexicon_tags_1 },
    { 0xa7a7ce4d, SUC_lexicon_tags_15 },
    { 0x687ce4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1067ce54, SUC_lexicon_tags_17 },
    { 0x5c7ce55, SUC_lexicon_tags_11 },
    { 0x9fe7ce55, SUC_lexicon_tags_27 },
    { 0xab47ce54, SUC_lexicon_tags_4 },
    { 0xd707ce58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc007ce5b, SUC_lexicon_tags_0 },
    { 0x4e67ce5c, SUC_lexicon_tags_4 },
    { 0xd667ce5b, SUC_lexicon_tags_2 },
    { 0x24a7ce5e, SUC_lexicon_tags_45 },
    { 0xfac7ce5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf107ce64, SUC_lexicon_tags_19 },
    { 0xa027ce65, SUC_lexicon_tags_2 },
    { 0x4be7ce66, SUC_lexicon_tags_16 },
    { 0x86c7ce65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb887ce69, SUC_lexicon_tags_6 },
    { 0x8dc7ce6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2887ce72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb07ce75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf507ce77, SUC_lexicon_tags_77 },
    { 0xd167ce78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb727ce7c, SUC_lexicon_tags_2 },
    { 0x8827ce7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a7ce80, SUC_lexicon_tags_1 },
    { 0x2a47ce80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5467ce85, SUC_lexicon_tags_12 },
    { 0x6c27ce85, SUC_lexicon_tags_15 },
    { 0x4667ce87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa7ce8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6607ce8d, SUC_lexicon_tags_40 },
    { 0xb3a7ce8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2787ce90, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x33a7ce92, SUC_lexicon_tags_21 },
    { 0xf87ce93, SUC_lexicon_tags_1 },
    { 0xc1e7ce94, SUC_lexicon_tags_17 },
    { 0xf2c7ce95, SUC_lexicon_tags_25 },
    { 0x8be7ce96, SUC_lexicon_tags_38 },
    { 0xe5c7ce96, SUC_lexicon_tags_1 },
    { 0xc4a7ce94, SUC_lexicon_tags_16 },
    { 0xb527ce99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6807ce9c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4747ce9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c7cea5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8687cea7, SUC_lexicon_tags_27 },
    { 0x4f67cea7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f07ceaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc167ceae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7e7ceb0, SUC_lexicon_tags_12 },
    { 0xb4e7ceb1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe27ceb8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c7cec4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb787cec6, SUC_lexicon_tags_37 },
    { 0xbb27cec7, SUC_lexicon_tags_82 },
    { 0x6867cec6, SUC_lexicon_tags_30 },
    { 0xcc07cec9, SUC_lexicon_tags_9 },
    { 0x25e7ceca, SUC_lexicon_tags_6 },
    { 0xf587cec7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf147cecd, SUC_lexicon_tags_11 },
    { 0xcc7cece, SUC_lexicon_tags_25 },
    { 0x387cecf, SUC_lexicon_tags_9 },
    { 0x35a7cece, SUC_lexicon_tags_37 },
    { 0xafc7ced1, SUC_lexicon_tags_7 },
    { 0x1d87ced2, SUC_lexicon_tags_7 },
    { 0xd07ced0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e47ced5, SUC_lexicon_tags_37 },
    { 0x7ca7ced6, SUC_lexicon_tags_19 },
    { 0x1887ced6, SUC_lexicon_tags_11 },
    { 0x78a7ced5, SUC_lexicon_tags_37 },
    { 0x58c7ced5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e27cedc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4067cee0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6e87cee2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd667cee4, SUC_lexicon_tags_3 },
    { 0x11c7cee5, SUC_lexicon_tags_43 },
    { 0xce87cee4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7707ceed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac7cef1, SUC_lexicon_tags_15 },
    { 0x1847cef2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2887cef5, SUC_lexicon_tags_20 },
    { 0x9827cef6, SUC_lexicon_tags_12 },
    { 0x2507cef7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9847cef9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x40c7cefb, SUC_lexicon_tags_17 },
    { 0x2d27cefc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c47ceff, SUC_lexicon_tags_10 },
    { 0xc0a7cf00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd867cf02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd087cf04, SUC_lexicon_tags_32 },
    { 0x18e7cf04, SUC_lexicon_tags_39 },
    { 0xcb27cf06, SUC_lexicon_tags_1 },
    { 0xc467cf07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa3a7cf09, SUC_lexicon_tags_11 },
    { 0x80e7cf0a, SUC_lexicon_tags_79 },
    { 0x55a7cf0b, SUC_lexicon_tags_60 },
    { 0xf27cf0c, SUC_lexicon_tags_1 },
    { 0x9c27cf0d, SUC_lexicon_tags_33 },
    { 0xf1e7cf0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b67cf11, SUC_lexicon_tags_4 },
    { 0xcf47cf12, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x18c7cf14, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc7cf18, SUC_lexicon_tags_54 },
    { 0x5607cf18, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x327cf1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc7cf20, SUC_lexicon_tags_25 },
    { 0xeaa7cf21, SUC_lexicon_tags_21 },
    { 0x2b87cf21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb067cf24, SUC_lexicon_tags_17 },
    { 0x2ba7cf25, SUC_lexicon_tags_0 },
    { 0x2a47cf25, SUC_lexicon_tags_25 },
    { 0x4247cf25, SUC_lexicon_tags_1 },
    { 0x5ca7cf25, SUC_lexicon_tags_13 },
    { 0x3807cf29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7cf2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8647cf2e, SUC_lexicon_tags_9 },
    { 0xf427cf2f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc7cf32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a7cf36, SUC_lexicon_tags_11 },
    { 0x3127cf37, SUC_lexicon_tags_2 },
    { 0x3127cf36, SUC_lexicon_tags_30 },
    { 0xe527cf39, SUC_lexicon_tags_1 },
    { 0x2b27cf3a, SUC_lexicon_tags_31 },
    { 0x7a27cf36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d67cf3d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7c7cf3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6487cf41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd107cf4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c67cf4e, SUC_lexicon_tags_37 },
    { 0x7b87cf4f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd0e7cf51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a7cf54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa67cf57, SUC_lexicon_tags_25 },
    { 0x7be7cf58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3867cf5b, SUC_lexicon_tags_63 },
    { 0x107cf5c, SUC_lexicon_tags_17 },
    { 0xee07cf5b, SUC_lexicon_tags_17 },
    { 0x5fc7cf5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c87cf60, SUC_lexicon_tags_15 },
    { 0x9aa7cf61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e27cf65, SUC_lexicon_tags_46 },
    { 0xc707cf65, SUC_lexicon_tags_17 },
    { 0x5cc7cf67, SUC_lexicon_tags_48 },
    { 0xf3c7cf68, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1267cf6a, SUC_lexicon_tags_19 },
    { 0x3227cf6a, SUC_lexicon_tags_37 },
    { 0xbca7cf6c, SUC_lexicon_tags_46 },
    { 0x4067cf6c, SUC_lexicon_tags_551 },
    { 0xe07cf6a, SUC_lexicon_tags_32 },
    { 0x4b27cf6b, SUC_lexicon_tags_25 },
    { 0x1447cf6c, SUC_lexicon_tags_68 },
    { 0x8687cf71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa167cf74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbbc7cf76, SUC_lexicon_tags_45 },
    { 0x9f27cf77, SUC_lexicon_tags_31 },
    { 0x3a47cf78, SUC_lexicon_tags_14 },
    { 0x99e7cf76, SUC_lexicon_tags_17 },
    { 0x1e27cf7a, SUC_lexicon_tags_39 },
    { 0xa707cf7b, SUC_lexicon_tags_4 },
    { 0x7127cf7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e47cf7e, SUC_lexicon_tags_33 },
    { 0xa927cf7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8347cf89, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcb47cf8b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a7cf93, SUC_lexicon_tags_21 },
    { 0x6b87cf94, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a7cf98, SUC_lexicon_tags_15 },
    { 0xc507cf98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae7cf9d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc007cfa2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xace7cfa4, SUC_lexicon_tags_2 },
    { 0x8367cfa4, SUC_lexicon_tags_138 },
    { 0xa87cfa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55a7cfa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa367cfab, SUC_lexicon_tags_37 },
    { 0xa407cfac, SUC_lexicon_tags_37 },
    { 0xa6a7cfad, SUC_lexicon_tags_37 },
    { 0x89c7cfad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa7cfb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x287cfb6, SUC_lexicon_tags_33 },
    { 0xe987cfb6, SUC_lexicon_tags_5 },
    { 0x7947cfb8, SUC_lexicon_tags_35 },
    { 0x72c7cfb9, SUC_lexicon_tags_37 },
    { 0xb3e7cfba, SUC_lexicon_tags_0 },
    { 0x7b07cfb7, SUC_lexicon_tags_0 },
    { 0x6287cfbb, SUC_lexicon_tags_12 },
    { 0x2ce7cfb7, SUC_lexicon_tags_65 },
    { 0x56e7cfbe, SUC_lexicon_tags_27 },
    { 0x1947cfbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9927cfc1, SUC_lexicon_tags_15 },
    { 0xf727cfc1, SUC_lexicon_tags_77 },
    { 0x3ae7cfc3, SUC_lexicon_tags_14 },
    { 0x8fc7cfc3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x33c7cfc6, SUC_lexicon_tags_16 },
    { 0x6747cfc7, SUC_lexicon_tags_26 },
    { 0xc927cfc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a07cfcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c7cfce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7cfd0, SUC_lexicon_tags_37 },
    { 0x6de7cfd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa7cfd4, SUC_lexicon_tags_17 },
    { 0x48e7cfd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9007cfd8, SUC_lexicon_tags_33 },
    { 0x7547cfd8, SUC_lexicon_tags_12 },
    { 0x1f87cfd8, SUC_lexicon_tags_82 },
    { 0xbe87cfdb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe7a7cfdd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9667cfdf, SUC_lexicon_tags_25 },
    { 0xcc27cfdf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc2c7cfe2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc107cfe4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8067cfe6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ba7cfe8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e7cfee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc7cff2, SUC_lexicon_tags_1 },
    { 0xb2a7cff3, SUC_lexicon_tags_43 },
    { 0x4fc7cff4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4267cff9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8667cffd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3727d001, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd027d004, SUC_lexicon_tags_29 },
    { 0x5487d005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70c7d007, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2e47d009, SUC_lexicon_tags_17 },
    { 0xc27d00a, SUC_lexicon_tags_534 },
    { 0, NULL },
    { 0, NULL },
    { 0x7887d00d, SUC_lexicon_tags_55 },
    { 0x6987d00e, SUC_lexicon_tags_9 },
    { 0xfe27d00f, SUC_lexicon_tags_62 },
    { 0x9447d010, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a7d013, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7d016, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca7d020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c7d024, SUC_lexicon_tags_37 },
    { 0xe167d024, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb67d02a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc7d02d, SUC_lexicon_tags_84 },
    { 0x84e7d02e, SUC_lexicon_tags_7 },
    { 0x8c07d02d, SUC_lexicon_tags_1 },
    { 0xed47d030, SUC_lexicon_tags_11 },
    { 0xdee7d02f, SUC_lexicon_tags_15 },
    { 0xd627d032, SUC_lexicon_tags_2 },
    { 0xdec7d033, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda7d03b, SUC_lexicon_tags_38 },
    { 0xac27d03b, SUC_lexicon_tags_14 },
    { 0xb347d03b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe7d040, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfec7d042, SUC_lexicon_tags_26 },
    { 0x7f47d043, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3807d045, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33a7d047, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c7d04a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8f47d04c, SUC_lexicon_tags_44 },
    { 0x26a7d04d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd527d04f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7547d054, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e87d059, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc4e7d05b, SUC_lexicon_tags_11 },
    { 0xbc27d05b, SUC_lexicon_tags_33 },
    { 0xa327d05d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x49a7d05f, SUC_lexicon_tags_68 },
    { 0x2487d060, SUC_lexicon_tags_65 },
    { 0x9567d061, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88e7d063, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe107d068, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b07d06c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c7d073, SUC_lexicon_tags_34 },
    { 0xcac7d073, SUC_lexicon_tags_7 },
    { 0xf447d075, SUC_lexicon_tags_65 },
    { 0x49a7d075, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5407d078, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3847d07e, SUC_lexicon_tags_60 },
    { 0xfaa7d07f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2e7d081, SUC_lexicon_tags_19 },
    { 0x83e7d082, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfd07d084, SUC_lexicon_tags_2 },
    { 0xdf47d085, SUC_lexicon_tags_19 },
    { 0xc407d085, SUC_lexicon_tags_1 },
    { 0x5b87d085, SUC_lexicon_tags_12 },
    { 0xad67d085, SUC_lexicon_tags_38 },
    { 0xa967d086, SUC_lexicon_tags_43 },
    { 0x1567d085, SUC_lexicon_tags_1 },
    { 0x1587d08a, SUC_lexicon_tags_57 },
    { 0x3047d08c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd27d08e, SUC_lexicon_tags_65 },
    { 0xc207d08f, SUC_lexicon_tags_12 },
    { 0xab87d090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed67d092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c27d09a, SUC_lexicon_tags_12 },
    { 0xf587d09a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca7d09d, SUC_lexicon_tags_11 },
    { 0xbcc7d09e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e87d0a4, SUC_lexicon_tags_49 },
    { 0x7227d0a4, SUC_lexicon_tags_11 },
    { 0x1047d0a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5007d0a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xffc7d0ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe7d0b3, SUC_lexicon_tags_8 },
    { 0x6dc7d0b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbee7d0b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3847d0bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5927d0c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e7d0c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c7d0cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2407d0d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb327d0d6, SUC_lexicon_tags_12 },
    { 0x53c7d0d7, SUC_lexicon_tags_37 },
    { 0x1a87d0d7, SUC_lexicon_tags_37 },
    { 0x2c27d0d8, SUC_lexicon_tags_15 },
    { 0x42e7d0da, SUC_lexicon_tags_13 },
    { 0x29e7d0d6, SUC_lexicon_tags_19 },
    { 0xff47d0dc, SUC_lexicon_tags_17 },
    { 0x4407d0dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb587d0e2, SUC_lexicon_tags_5 },
    { 0xb447d0e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a7d0e8, SUC_lexicon_tags_47 },
    { 0xae07d0e9, SUC_lexicon_tags_43 },
    { 0xe2c7d0e9, SUC_lexicon_tags_0 },
    { 0x727d0eb, SUC_lexicon_tags_5 },
    { 0x7427d0e8, SUC_lexicon_tags_61 },
    { 0x9b07d0ed, SUC_lexicon_tags_17 },
    { 0x84c7d0ee, SUC_lexicon_tags_15 },
    { 0x64a7d0ef, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda07d0f4, SUC_lexicon_tags_11 },
    { 0x6887d0f4, SUC_lexicon_tags_30 },
    { 0x11c7d0f5, SUC_lexicon_tags_48 },
    { 0xaa7d0f5, SUC_lexicon_tags_11 },
    { 0x16a7d0f5, SUC_lexicon_tags_12 },
    { 0x9a7d0f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e7d0fd, SUC_lexicon_tags_14 },
    { 0x4987d0fe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c7d104, SUC_lexicon_tags_23 },
    { 0x3d67d104, SUC_lexicon_tags_25 },
    { 0x2327d106, SUC_lexicon_tags_1 },
    { 0xd827d107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab27d109, SUC_lexicon_tags_54 },
    { 0x3407d10a, SUC_lexicon_tags_7 },
    { 0x1e47d10a, SUC_lexicon_tags_37 },
    { 0xaf67d10c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a67d10f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a67d115, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe947d118, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5467d124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd067d12a, SUC_lexicon_tags_12 },
    { 0x7ae7d12a, SUC_lexicon_tags_3 },
    { 0x48e7d12c, SUC_lexicon_tags_1 },
    { 0x7947d12d, SUC_lexicon_tags_49 },
    { 0xe127d12e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3807d135, SUC_lexicon_tags_2 },
    { 0xf847d136, SUC_lexicon_tags_10 },
    { 0xed07d135, SUC_lexicon_tags_47 },
    { 0x7d138, SUC_lexicon_tags_35 },
    { 0x7867d138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e7d13c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc247d13e, SUC_lexicon_tags_49 },
    { 0x2947d13e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe547d141, SUC_lexicon_tags_27 },
    { 0x3ba7d142, SUC_lexicon_tags_27 },
    { 0x70e7d143, SUC_lexicon_tags_39 },
    { 0xb947d144, SUC_lexicon_tags_2 },
    { 0xe267d145, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3087d149, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefa7d14b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6167d14d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7d150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb327d152, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bc7d154, SUC_lexicon_tags_27 },
    { 0xe987d155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44e7d157, SUC_lexicon_tags_40 },
    { 0xcc67d157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99e7d15a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1427d15c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e7d166, SUC_lexicon_tags_3 },
    { 0xa047d166, SUC_lexicon_tags_33 },
    { 0xb8c7d167, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4487d16b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5c7d16d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6867d16f, SUC_lexicon_tags_30 },
    { 0x3487d16f, SUC_lexicon_tags_14 },
    { 0xa707d171, SUC_lexicon_tags_15 },
    { 0x8a27d171, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167d17d, SUC_lexicon_tags_47 },
    { 0xad27d17e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4107d180, SUC_lexicon_tags_17 },
    { 0xcfa7d181, SUC_lexicon_tags_10 },
    { 0xc527d182, SUC_lexicon_tags_37 },
    { 0x9307d182, SUC_lexicon_tags_38 },
    { 0x2067d181, SUC_lexicon_tags_14 },
    { 0x3227d183, SUC_lexicon_tags_68 },
    { 0xda87d186, SUC_lexicon_tags_6 },
    { 0x3fa7d186, SUC_lexicon_tags_31 },
    { 0xbbe7d188, SUC_lexicon_tags_15 },
    { 0x3b27d189, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2707d18e, SUC_lexicon_tags_113 },
    { 0xbaa7d18f, SUC_lexicon_tags_43 },
    { 0x7187d18f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c7d195, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac7d198, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb5e7d19a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc7d19d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b47d1a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e7d1a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb827d1a5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa027d1a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf787d1aa, SUC_lexicon_tags_65 },
    { 0x2127d1aa, SUC_lexicon_tags_68 },
    { 0x4ac7d1ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c7d1b0, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a7d1ba, SUC_lexicon_tags_15 },
    { 0x7547d1bb, SUC_lexicon_tags_12 },
    { 0xbc07d1ba, SUC_lexicon_tags_29 },
    { 0x7147d1bc, SUC_lexicon_tags_17 },
    { 0xe2a7d1be, SUC_lexicon_tags_3 },
    { 0xb9a7d1bf, SUC_lexicon_tags_23 },
    { 0xa567d1bf, SUC_lexicon_tags_19 },
    { 0x9e67d1c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7067d1c3, SUC_lexicon_tags_2 },
    { 0x2087d1c4, SUC_lexicon_tags_87 },
    { 0xb507d1c3, SUC_lexicon_tags_19 },
    { 0xb827d1c6, SUC_lexicon_tags_2 },
    { 0x78e7d1c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e7d1ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3267d1ce, SUC_lexicon_tags_11 },
    { 0x17e7d1cf, SUC_lexicon_tags_49 },
    { 0x7687d1d0, SUC_lexicon_tags_1 },
    { 0x7fa7d1d0, SUC_lexicon_tags_56 },
    { 0xb027d1d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1447d1d4, SUC_lexicon_tags_25 },
    { 0x10a7d1d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb487d1db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xec07d1de, SUC_lexicon_tags_15 },
    { 0x9be7d1de, SUC_lexicon_tags_19 },
    { 0x7ec7d1de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507d1ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9527d1ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8a67d1f0, SUC_lexicon_tags_7 },
    { 0xb987d1f1, SUC_lexicon_tags_198 },
    { 0x5787d1f2, SUC_lexicon_tags_12 },
    { 0xfb67d1f0, SUC_lexicon_tags_23 },
    { 0xf6a7d1f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a87d1fe, SUC_lexicon_tags_32 },
    { 0x20a7d1ff, SUC_lexicon_tags_17 },
    { 0x4467d1ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfb87d202, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a7d206, SUC_lexicon_tags_19 },
    { 0x70c7d207, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3947d209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd07d20e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be7d213, SUC_lexicon_tags_13 },
    { 0xd967d213, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4387d21d, SUC_lexicon_tags_11 },
    { 0xd4a7d21d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xea67d221, SUC_lexicon_tags_39 },
    { 0xcb47d222, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6a67d224, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd27d226, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9e07d228, SUC_lexicon_tags_37 },
    { 0x8887d229, SUC_lexicon_tags_38 },
    { 0xe5c7d228, SUC_lexicon_tags_43 },
    { 0xc8e7d22b, SUC_lexicon_tags_25 },
    { 0xca27d22b, SUC_lexicon_tags_12 },
    { 0xc747d22d, SUC_lexicon_tags_17 },
    { 0x6707d22c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf587d231, SUC_lexicon_tags_37 },
    { 0xbb87d231, SUC_lexicon_tags_45 },
    { 0x6727d233, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6787d23e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7d248, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8247d24a, SUC_lexicon_tags_11 },
    { 0x1be7d24b, SUC_lexicon_tags_14 },
    { 0x2fc7d24c, SUC_lexicon_tags_0 },
    { 0x3627d24d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9547d250, SUC_lexicon_tags_1 },
    { 0x6327d250, SUC_lexicon_tags_7 },
    { 0x67d252, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa867d259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf07d25c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5107d260, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5647d263, SUC_lexicon_tags_49 },
    { 0x3127d264, SUC_lexicon_tags_46 },
    { 0xe7d265, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1127d26f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90a7d271, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a47d27c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe207d27e, SUC_lexicon_tags_39 },
    { 0xc607d27e, SUC_lexicon_tags_15 },
    { 0xcce7d280, SUC_lexicon_tags_38 },
    { 0x89a7d27f, SUC_lexicon_tags_19 },
    { 0x2e27d282, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6887d284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ee7d286, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a7d28b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a67d28e, SUC_lexicon_tags_43 },
    { 0x4567d28f, SUC_lexicon_tags_12 },
    { 0x2407d290, SUC_lexicon_tags_12 },
    { 0xfe7d28f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be7d29a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa07d29d, SUC_lexicon_tags_7 },
    { 0xb5c7d29e, SUC_lexicon_tags_7 },
    { 0x69e7d29e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec7d2a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67c7d2a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a7d2ab, SUC_lexicon_tags_60 },
    { 0xf547d2ac, SUC_lexicon_tags_6 },
    { 0xcf47d2ad, SUC_lexicon_tags_26 },
    { 0x5707d2ae, SUC_lexicon_tags_37 },
    { 0xdfc7d2af, SUC_lexicon_tags_37 },
    { 0x4787d2af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2887d2b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb947d2b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c7d2b9, SUC_lexicon_tags_30 },
    { 0x3647d2ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb387d2bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8767d2c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa047d2c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc7d2ca, SUC_lexicon_tags_61 },
    { 0xad87d2ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e87d2ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a47d2d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc967d2d5, SUC_lexicon_tags_26 },
    { 0x1e27d2d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67d2da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8747d2de, SUC_lexicon_tags_12 },
    { 0xe2a7d2de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1507d2e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c27d2e7, SUC_lexicon_tags_38 },
    { 0xb107d2e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf27d2eb, SUC_lexicon_tags_37 },
    { 0xf127d2ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a47d2ef, SUC_lexicon_tags_16 },
    { 0x27e7d2f0, SUC_lexicon_tags_25 },
    { 0x6e27d2f1, SUC_lexicon_tags_1 },
    { 0x65e7d2f1, SUC_lexicon_tags_38 },
    { 0x8be7d2f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6da7d2f5, SUC_lexicon_tags_45 },
    { 0xb047d2f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8047d300, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe827d307, SUC_lexicon_tags_11 },
    { 0xc067d308, SUC_lexicon_tags_23 },
    { 0xf687d309, SUC_lexicon_tags_1 },
    { 0x4587d30a, SUC_lexicon_tags_537 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc607d312, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xab47d314, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e7d31c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6ea7d31e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3107d320, SUC_lexicon_tags_9 },
    { 0xa627d321, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf67d323, SUC_lexicon_tags_11 },
    { 0xe5a7d324, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f87d327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d47d329, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7d32d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf027d32f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ce7d331, SUC_lexicon_tags_16 },
    { 0xe487d331, SUC_lexicon_tags_43 },
    { 0x79a7d333, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9467d33a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe827d33d, SUC_lexicon_tags_38 },
    { 0x6047d33e, SUC_lexicon_tags_11 },
    { 0x5107d33d, SUC_lexicon_tags_34 },
    { 0x81c7d340, SUC_lexicon_tags_49 },
    { 0x9587d340, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1767d345, SUC_lexicon_tags_29 },
    { 0x1667d346, SUC_lexicon_tags_8 },
    { 0x6727d345, SUC_lexicon_tags_60 },
    { 0x7107d346, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d27d350, SUC_lexicon_tags_1 },
    { 0x6ba7d351, SUC_lexicon_tags_1 },
    { 0x64c7d352, SUC_lexicon_tags_37 },
    { 0x8287d350, SUC_lexicon_tags_25 },
    { 0x5e47d350, SUC_lexicon_tags_17 },
    { 0x98e7d355, SUC_lexicon_tags_33 },
    { 0x6647d356, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x8827d358, SUC_lexicon_tags_25 },
    { 0xbde7d359, SUC_lexicon_tags_33 },
    { 0xf747d35a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aa7d35c, SUC_lexicon_tags_8 },
    { 0xfba7d35c, SUC_lexicon_tags_11 },
    { 0xe4a7d35e, SUC_lexicon_tags_17 },
    { 0x22a7d35e, SUC_lexicon_tags_9 },
    { 0x76c7d360, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3247d363, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd007d368, SUC_lexicon_tags_37 },
    { 0x28c7d369, SUC_lexicon_tags_2 },
    { 0xf767d36a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b27d36d, SUC_lexicon_tags_45 },
    { 0x9ee7d36e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1227d370, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0x3387d372, SUC_lexicon_tags_37 },
    { 0xa67d372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72e7d375, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a27d37b, SUC_lexicon_tags_12 },
    { 0x9fa7d37b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8587d383, SUC_lexicon_tags_1 },
    { 0xc3a7d384, SUC_lexicon_tags_65 },
    { 0x67e7d383, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1847d388, SUC_lexicon_tags_4 },
    { 0x6ac7d389, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6027d38b, SUC_lexicon_tags_107 },
    { 0xa6a7d38b, SUC_lexicon_tags_12 },
    { 0xb167d38d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6767d38f, SUC_lexicon_tags_29 },
    { 0xcf87d390, SUC_lexicon_tags_48 },
    { 0xc727d38f, SUC_lexicon_tags_27 },
    { 0xd67d391, SUC_lexicon_tags_47 },
    { 0xc6c7d393, SUC_lexicon_tags_2 },
    { 0x7207d394, SUC_lexicon_tags_22 },
    { 0xdde7d390, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c67d397, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b87d399, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe407d39b, SUC_lexicon_tags_26 },
    { 0x4f27d39b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec7d3a0, SUC_lexicon_tags_30 },
    { 0xf9c7d3a1, SUC_lexicon_tags_17 },
    { 0x8307d3a2, SUC_lexicon_tags_17 },
    { 0xd5e7d3a0, SUC_lexicon_tags_40 },
    { 0x9e67d3a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf607d3ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66c7d3ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4247d3b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8427d3b3, SUC_lexicon_tags_12 },
    { 0x15a7d3b4, SUC_lexicon_tags_6 },
    { 0xf847d3b3, SUC_lexicon_tags_38 },
    { 0xa1c7d3b6, SUC_lexicon_tags_17 },
    { 0x8d67d3b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7d3be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3007d3c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5527d3c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d47d3c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7727d3d3, SUC_lexicon_tags_2 },
    { 0xf4e7d3d4, SUC_lexicon_tags_2 },
    { 0xfd67d3d5, SUC_lexicon_tags_7 },
    { 0x9447d3d3, SUC_lexicon_tags_14 },
    { 0x5867d3d6, SUC_lexicon_tags_24 },
    { 0x90a7d3d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c87d3db, SUC_lexicon_tags_7 },
    { 0x5267d3dc, SUC_lexicon_tags_2 },
    { 0x7fe7d3dc, SUC_lexicon_tags_7 },
    { 0xb607d3dc, SUC_lexicon_tags_45 },
    { 0x1427d3df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf07d3e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xde67d3e3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc387d3ea, SUC_lexicon_tags_0 },
    { 0x4647d3eb, SUC_lexicon_tags_1 },
    { 0xbfc7d3ec, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd547d3ee, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a7d3f1, SUC_lexicon_tags_57 },
    { 0x58c7d3f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1387d3f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13a7d3fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf227d3fe, SUC_lexicon_tags_14 },
    { 0xf2a7d3ff, SUC_lexicon_tags_17 },
    { 0x6cc7d3ff, SUC_lexicon_tags_25 },
    { 0xd6e7d400, SUC_lexicon_tags_32 },
    { 0x8107d402, SUC_lexicon_tags_9 },
    { 0x8aa7d402, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c27d405, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1967d408, SUC_lexicon_tags_7 },
    { 0xe747d409, SUC_lexicon_tags_47 },
    { 0x2a47d40a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c87d40e, SUC_lexicon_tags_22 },
    { 0xdfe7d40f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e7d416, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e7d41c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa7d420, SUC_lexicon_tags_12 },
    { 0x6e47d421, SUC_lexicon_tags_7 },
    { 0x3d47d421, SUC_lexicon_tags_11 },
    { 0xabc7d421, SUC_lexicon_tags_19 },
    { 0x5e67d420, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x8707d426, SUC_lexicon_tags_27 },
    { 0xacc7d426, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbac7d429, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a7d42e, SUC_lexicon_tags_37 },
    { 0x7a67d42f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3b87d431, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe7e7d433, SUC_lexicon_tags_65 },
    { 0x1867d434, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7d43a, SUC_lexicon_tags_12 },
    { 0x2e07d43b, SUC_lexicon_tags_4 },
    { 0xd047d43b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc7d43f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca7d447, SUC_lexicon_tags_35 },
    { 0x9647d448, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8787d44a, SUC_lexicon_tags_25 },
    { 0xfdc7d44b, SUC_lexicon_tags_28 },
    { 0x51a7d44c, SUC_lexicon_tags_1 },
    { 0xa047d44d, SUC_lexicon_tags_42 },
    { 0x65e7d44e, SUC_lexicon_tags_4 },
    { 0xfe47d44f, SUC_lexicon_tags_2 },
    { 0x6de7d450, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac7d455, SUC_lexicon_tags_9 },
    { 0xf447d456, SUC_lexicon_tags_39 },
    { 0xa0e7d456, SUC_lexicon_tags_17 },
    { 0xb467d458, SUC_lexicon_tags_279 },
    { 0x7307d459, SUC_lexicon_tags_28 },
    { 0xb087d457, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7527d45c, SUC_lexicon_tags_46 },
    { 0x6767d45d, SUC_lexicon_tags_1 },
    { 0x7c07d45d, SUC_lexicon_tags_644 },
    { 0, NULL },
    { 0xb3a7d460, SUC_lexicon_tags_25 },
    { 0x81e7d460, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7207d464, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3927d47c, SUC_lexicon_tags_0 },
    { 0xab87d47c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1647d480, SUC_lexicon_tags_4 },
    { 0x8da7d481, SUC_lexicon_tags_37 },
    { 0x98e7d482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9827d486, SUC_lexicon_tags_27 },
    { 0x1da7d487, SUC_lexicon_tags_12 },
    { 0xcd87d486, SUC_lexicon_tags_37 },
    { 0xace7d489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e27d48c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f27d494, SUC_lexicon_tags_15 },
    { 0xd767d494, SUC_lexicon_tags_11 },
    { 0x2ca7d494, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8a7d498, SUC_lexicon_tags_49 },
    { 0xea7d499, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a7d49d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a27d49f, SUC_lexicon_tags_28 },
    { 0x3e47d49f, SUC_lexicon_tags_15 },
    { 0xcdc7d4a0, SUC_lexicon_tags_37 },
    { 0x5a07d4a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa7d4ac, SUC_lexicon_tags_37 },
    { 0xa6a7d4ac, SUC_lexicon_tags_17 },
    { 0x84a7d4ae, SUC_lexicon_tags_60 },
    { 0x5d67d4af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa7d4b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc7d4b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf227d4be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f27d4c1, SUC_lexicon_tags_47 },
    { 0x94e7d4c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b27d4c5, SUC_lexicon_tags_27 },
    { 0xad67d4c6, SUC_lexicon_tags_15 },
    { 0x8a87d4c7, SUC_lexicon_tags_25 },
    { 0x6fe7d4c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7d4cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a7d4d0, SUC_lexicon_tags_7 },
    { 0x67d4d0, SUC_lexicon_tags_0 },
    { 0xdba7d4d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6607d4d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8687d4d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fc7d4db, SUC_lexicon_tags_0 },
    { 0xc9c7d4dc, SUC_lexicon_tags_31 },
    { 0x8847d4db, SUC_lexicon_tags_1 },
    { 0x1a47d4de, SUC_lexicon_tags_48 },
    { 0x55a7d4df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa687d4e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcda7d4e3, SUC_lexicon_tags_15 },
    { 0x39c7d4e4, SUC_lexicon_tags_17 },
    { 0x4247d4e4, SUC_lexicon_tags_8 },
    { 0x64e7d4e6, SUC_lexicon_tags_37 },
    { 0xee27d4e7, SUC_lexicon_tags_7 },
    { 0x3f07d4e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2c7d4ea, SUC_lexicon_tags_7 },
    { 0x6307d4ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8267d4f1, SUC_lexicon_tags_37 },
    { 0xf727d4f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa327d4f6, SUC_lexicon_tags_37 },
    { 0xeee7d4f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa187d4f9, SUC_lexicon_tags_12 },
    { 0x29c7d4fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4547d4fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb567d4ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47d502, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda7d505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb67d50a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe7d50c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f67d511, SUC_lexicon_tags_1 },
    { 0xaf07d512, SUC_lexicon_tags_15 },
    { 0x4d67d513, SUC_lexicon_tags_47 },
    { 0x75e7d514, SUC_lexicon_tags_23 },
    { 0x22e7d515, SUC_lexicon_tags_4 },
    { 0xf567d516, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf707d51a, SUC_lexicon_tags_13 },
    { 0x1bc7d51b, SUC_lexicon_tags_25 },
    { 0xa1a7d51b, SUC_lexicon_tags_46 },
    { 0xb1e7d51c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e67d521, SUC_lexicon_tags_49 },
    { 0xe567d522, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52e7d524, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee7d527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4527d52a, SUC_lexicon_tags_25 },
    { 0xa307d52b, SUC_lexicon_tags_9 },
    { 0x77a7d52c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa0a7d52e, SUC_lexicon_tags_33 },
    { 0x5ce7d52e, SUC_lexicon_tags_29 },
    { 0x4d87d530, SUC_lexicon_tags_17 },
    { 0x81a7d530, SUC_lexicon_tags_63 },
    { 0x3a47d52e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f67d535, SUC_lexicon_tags_49 },
    { 0xa507d535, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a7d53b, SUC_lexicon_tags_37 },
    { 0xd507d53b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c7d548, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd667d54c, SUC_lexicon_tags_10 },
    { 0x1b87d54d, SUC_lexicon_tags_17 },
    { 0xc3e7d54e, SUC_lexicon_tags_17 },
    { 0xaa27d54d, SUC_lexicon_tags_12 },
    { 0xab07d54e, SUC_lexicon_tags_46 },
    { 0xdd27d54d, SUC_lexicon_tags_25 },
    { 0x1967d552, SUC_lexicon_tags_37 },
    { 0x4da7d553, SUC_lexicon_tags_6 },
    { 0x787d54d, SUC_lexicon_tags_38 },
    { 0x71c7d553, SUC_lexicon_tags_9 },
    { 0x8847d553, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f27d559, SUC_lexicon_tags_79 },
    { 0x9187d559, SUC_lexicon_tags_27 },
    { 0xe807d55b, SUC_lexicon_tags_8 },
    { 0xa287d55c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc247d55e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd427d560, SUC_lexicon_tags_84 },
    { 0xccc7d560, SUC_lexicon_tags_11 },
    { 0x3e07d561, SUC_lexicon_tags_43 },
    { 0x2347d563, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb507d567, SUC_lexicon_tags_7 },
    { 0x3ec7d568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab47d56c, SUC_lexicon_tags_1 },
    { 0x6e27d56d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf507d56f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e7d575, SUC_lexicon_tags_45 },
    { 0x17c7d576, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3927d579, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6387d57d, SUC_lexicon_tags_2 },
    { 0x7507d57e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c27d582, SUC_lexicon_tags_9 },
    { 0x5dc7d582, SUC_lexicon_tags_25 },
    { 0x8247d582, SUC_lexicon_tags_10 },
    { 0xd147d585, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a7d588, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe167d58e, SUC_lexicon_tags_8 },
    { 0x9d87d58f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc7d592, SUC_lexicon_tags_77 },
    { 0x5e27d592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27d595, SUC_lexicon_tags_45 },
    { 0xf2e7d596, SUC_lexicon_tags_55 },
    { 0x79c7d595, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2587d59a, SUC_lexicon_tags_63 },
    { 0x6e27d59b, SUC_lexicon_tags_53 },
    { 0x2367d59c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1267d5a3, SUC_lexicon_tags_47 },
    { 0x1f07d5a3, SUC_lexicon_tags_37 },
    { 0x1967d5a3, SUC_lexicon_tags_9 },
    { 0x3707d5a6, SUC_lexicon_tags_12 },
    { 0x9807d5a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a7d5ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa7d5b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49a7d5b3, SUC_lexicon_tags_49 },
    { 0xe327d5b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb507d5b6, SUC_lexicon_tags_7 },
    { 0x5567d5b7, SUC_lexicon_tags_27 },
    { 0x7c87d5b7, SUC_lexicon_tags_13 },
    { 0xe2e7d5b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e7d5c0, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xcf87d5c2, SUC_lexicon_tags_11 },
    { 0x1e7d5c3, SUC_lexicon_tags_561 },
    { 0xd8c7d5c3, SUC_lexicon_tags_16 },
    { 0xc2c7d5c5, SUC_lexicon_tags_7 },
    { 0x46a7d5c6, SUC_lexicon_tags_1 },
    { 0xd507d5c7, SUC_lexicon_tags_37 },
    { 0x5867d5c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea47d5cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc267d5d3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e7d5d8, SUC_lexicon_tags_48 },
    { 0x86a7d5d9, SUC_lexicon_tags_27 },
    { 0x9b07d5d9, SUC_lexicon_tags_45 },
    { 0x3547d5da, SUC_lexicon_tags_2 },
    { 0x7c07d5dc, SUC_lexicon_tags_626 },
    { 0x6ca7d5d9, SUC_lexicon_tags_66 },
    { 0x5a7d5de, SUC_lexicon_tags_28 },
    { 0x5687d5df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc7d5e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb07d5e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaa47d5e7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1ec7d5e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x26c7d5eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1887d5ed, SUC_lexicon_tags_15 },
    { 0x5927d5ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc967d5f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3027d5f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a27d5fa, SUC_lexicon_tags_27 },
    { 0xb307d5fb, SUC_lexicon_tags_61 },
    { 0xebc7d5fc, SUC_lexicon_tags_1 },
    { 0x7247d5fc, SUC_lexicon_tags_37 },
    { 0xe567d5fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7de7d600, SUC_lexicon_tags_17 },
    { 0xf1e7d601, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c7d606, SUC_lexicon_tags_6 },
    { 0x4747d607, SUC_lexicon_tags_15 },
    { 0xe387d608, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb87d60e, SUC_lexicon_tags_37 },
    { 0xcd07d60e, SUC_lexicon_tags_11 },
    { 0x3747d610, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea7d615, SUC_lexicon_tags_32 },
    { 0x6fa7d616, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc7d61b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcb67d61d, SUC_lexicon_tags_17 },
    { 0xd867d61e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4807d621, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a87d623, SUC_lexicon_tags_16 },
    { 0x5ae7d624, SUC_lexicon_tags_45 },
    { 0xfee7d623, SUC_lexicon_tags_17 },
    { 0x7387d626, SUC_lexicon_tags_2 },
    { 0x4e67d627, SUC_lexicon_tags_19 },
    { 0xbc27d628, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa07d62c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87d631, SUC_lexicon_tags_34 },
    { 0x967d632, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc7d635, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7d63a, SUC_lexicon_tags_26 },
    { 0x3b07d63b, SUC_lexicon_tags_837 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e47d641, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5747d649, SUC_lexicon_tags_0 },
    { 0x5807d649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd727d651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e7d655, SUC_lexicon_tags_38 },
    { 0x4487d656, SUC_lexicon_tags_17 },
    { 0x93e7d656, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4067d659, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6207d65d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8487d65f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd307d661, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4427d664, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe267d666, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf127d668, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c7d66a, SUC_lexicon_tags_37 },
    { 0x5067d66a, SUC_lexicon_tags_17 },
    { 0x6e47d66c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x7967d66f, SUC_lexicon_tags_37 },
    { 0x7f67d670, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce27d676, SUC_lexicon_tags_7 },
    { 0xb787d677, SUC_lexicon_tags_57 },
    { 0x5d67d676, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87d67b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15a7d67d, SUC_lexicon_tags_12 },
    { 0x8367d67d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a67d680, SUC_lexicon_tags_31 },
    { 0xee7d681, SUC_lexicon_tags_45 },
    { 0xb447d682, SUC_lexicon_tags_45 },
    { 0xb5a7d683, SUC_lexicon_tags_38 },
    { 0x7147d684, SUC_lexicon_tags_7 },
    { 0x4327d685, SUC_lexicon_tags_15 },
    { 0xec27d683, SUC_lexicon_tags_1 },
    { 0x3647d687, SUC_lexicon_tags_27 },
    { 0xb027d684, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5e67d68a, SUC_lexicon_tags_44 },
    { 0x6987d68a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc167d68d, SUC_lexicon_tags_821 },
    { 0xfec7d68e, SUC_lexicon_tags_15 },
    { 0xb1c7d68f, SUC_lexicon_tags_0 },
    { 0x78c7d68e, SUC_lexicon_tags_19 },
    { 0xa487d68e, SUC_lexicon_tags_177 },
    { 0x38c7d68d, SUC_lexicon_tags_67 },
    { 0xdee7d693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7927d696, SUC_lexicon_tags_351 },
    { 0xc227d697, SUC_lexicon_tags_1 },
    { 0x1dc7d698, SUC_lexicon_tags_8 },
    { 0xdf47d699, SUC_lexicon_tags_37 },
    { 0xe727d699, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34e7d69c, SUC_lexicon_tags_43 },
    { 0x5087d69d, SUC_lexicon_tags_25 },
    { 0xf6e7d69e, SUC_lexicon_tags_17 },
    { 0xef87d69c, SUC_lexicon_tags_15 },
    { 0x43c7d6a0, SUC_lexicon_tags_7 },
    { 0xfcc7d6a0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xaf87d6a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7787d6a5, SUC_lexicon_tags_7 },
    { 0x67a7d6a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7d6ac, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0x3007d6ae, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e07d6b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a7d6bb, SUC_lexicon_tags_15 },
    { 0xefa7d6bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2267d6be, SUC_lexicon_tags_12 },
    { 0xada7d6be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac27d6c3, SUC_lexicon_tags_12 },
    { 0x9de7d6c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6787d6c6, SUC_lexicon_tags_25 },
    { 0x98a7d6c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd147d6cb, SUC_lexicon_tags_0 },
    { 0x1dc7d6cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6607d6cf, SUC_lexicon_tags_26 },
    { 0x8067d6d0, SUC_lexicon_tags_17 },
    { 0x6d27d6d1, SUC_lexicon_tags_27 },
    { 0x99a7d6d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5907d6d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e7d6d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80a7d6d9, SUC_lexicon_tags_60 },
    { 0x2327d6d9, SUC_lexicon_tags_650 },
    { 0x6a67d6db, SUC_lexicon_tags_70 },
    { 0xfe07d6d9, SUC_lexicon_tags_12 },
    { 0xd47d6da, SUC_lexicon_tags_27 },
    { 0x5ac7d6da, SUC_lexicon_tags_1 },
    { 0xdd87d6d9, SUC_lexicon_tags_11 },
    { 0xe1a7d6e0, SUC_lexicon_tags_11 },
    { 0xed07d6dd, SUC_lexicon_tags_9 },
    { 0x6967d6e2, SUC_lexicon_tags_1 },
    { 0x7d87d6e3, SUC_lexicon_tags_87 },
    { 0x2367d6e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2907d6e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f27d6ea, SUC_lexicon_tags_7 },
    { 0x79c7d6eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c7d6f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c7d6f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7807d6f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b27d6fe, SUC_lexicon_tags_37 },
    { 0x45a7d6ff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbf47d701, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8967d710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9507d712, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd7a7d714, SUC_lexicon_tags_17 },
    { 0xd47d715, SUC_lexicon_tags_38 },
    { 0x9b67d716, SUC_lexicon_tags_12 },
    { 0xdcc7d717, SUC_lexicon_tags_6 },
    { 0xbce7d715, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5ca7d71a, SUC_lexicon_tags_114 },
    { 0x8d07d71b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb67d71d, SUC_lexicon_tags_87 },
    { 0xe27d71e, SUC_lexicon_tags_33 },
    { 0x9e67d71f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3b87d721, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe767d724, SUC_lexicon_tags_26 },
    { 0xb9a7d725, SUC_lexicon_tags_17 },
    { 0x9447d725, SUC_lexicon_tags_33 },
    { 0xf907d727, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffa7d729, SUC_lexicon_tags_56 },
    { 0x6567d729, SUC_lexicon_tags_10 },
    { 0x8be7d72b, SUC_lexicon_tags_37 },
    { 0x50c7d72c, SUC_lexicon_tags_85 },
    { 0xaba7d72d, SUC_lexicon_tags_30 },
    { 0xd027d72c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6227d732, SUC_lexicon_tags_15 },
    { 0xb7c7d733, SUC_lexicon_tags_16 },
    { 0xed87d733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7647d738, SUC_lexicon_tags_30 },
    { 0x17a7d739, SUC_lexicon_tags_28 },
    { 0x9c07d73a, SUC_lexicon_tags_82 },
    { 0xc0e7d73a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde7d740, SUC_lexicon_tags_49 },
    { 0x54c7d741, SUC_lexicon_tags_43 },
    { 0xc787d741, SUC_lexicon_tags_25 },
    { 0x1727d743, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7d74a, SUC_lexicon_tags_14 },
    { 0xd1e7d74a, SUC_lexicon_tags_8 },
    { 0x5ea7d74b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3287d74f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5e7d751, SUC_lexicon_tags_9 },
    { 0x2f27d752, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7487d755, SUC_lexicon_tags_38 },
    { 0x8567d756, SUC_lexicon_tags_17 },
    { 0x1027d757, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e7d75b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbfa7d75d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa7d760, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e7d763, SUC_lexicon_tags_39 },
    { 0x1de7d764, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2267d769, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1187d76d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43e7d76f, SUC_lexicon_tags_14 },
    { 0xbc7d76f, SUC_lexicon_tags_37 },
    { 0x3e27d771, SUC_lexicon_tags_68 },
    { 0x8a7d76f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae7d778, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6587d77d, SUC_lexicon_tags_11 },
    { 0x13c7d77e, SUC_lexicon_tags_17 },
    { 0xe2a7d77d, SUC_lexicon_tags_11 },
    { 0x1127d780, SUC_lexicon_tags_0 },
    { 0x2787d77d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb267d785, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7d788, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c7d78b, SUC_lexicon_tags_14 },
    { 0x80c7d78c, SUC_lexicon_tags_25 },
    { 0x67d78c, SUC_lexicon_tags_10 },
    { 0x8067d78e, SUC_lexicon_tags_46 },
    { 0x6767d78e, SUC_lexicon_tags_8 },
    { 0x267d78c, SUC_lexicon_tags_3 },
    { 0xcbc7d791, SUC_lexicon_tags_17 },
    { 0x5467d78e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaea7d794, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfc87d796, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe7d79a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c7d7a1, SUC_lexicon_tags_11 },
    { 0x6c47d7a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa2a7d7a4, SUC_lexicon_tags_33 },
    { 0x4007d7a4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd27d7ad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7407d7b0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9067d7b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf467d7b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d7b9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa007d7be, SUC_lexicon_tags_29 },
    { 0x3627d7bf, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2b87d7c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe287d7c6, SUC_lexicon_tags_5 },
    { 0x3647d7c7, SUC_lexicon_tags_160 },
    { 0x3467d7c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6447d7cc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c7d7d5, SUC_lexicon_tags_664 },
    { 0x6d07d7d5, SUC_lexicon_tags_31 },
    { 0x327d7d6, SUC_lexicon_tags_25 },
    { 0xac47d7d8, SUC_lexicon_tags_1 },
    { 0x2de7d7d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee7d7e3, SUC_lexicon_tags_17 },
    { 0xd3e7d7e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc607d7e8, SUC_lexicon_tags_1 },
    { 0x21a7d7e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6067d7eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9327d7f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac7d7f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da7d7f7, SUC_lexicon_tags_49 },
    { 0xe0c7d7f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa727d7fa, SUC_lexicon_tags_23 },
    { 0xb807d7fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xed87d7fd, SUC_lexicon_tags_15 },
    { 0xaa27d7fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba7d803, SUC_lexicon_tags_27 },
    { 0xe787d804, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb67d808, SUC_lexicon_tags_1 },
    { 0x6087d809, SUC_lexicon_tags_2 },
    { 0xfde7d80a, SUC_lexicon_tags_19 },
    { 0xf047d809, SUC_lexicon_tags_35 },
    { 0x1e27d809, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa7d811, SUC_lexicon_tags_65 },
    { 0xe887d812, SUC_lexicon_tags_11 },
    { 0x107d812, SUC_lexicon_tags_53 },
    { 0xa447d814, SUC_lexicon_tags_23 },
    { 0x1867d815, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea67d81b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede7d820, SUC_lexicon_tags_7 },
    { 0x4ea7d821, SUC_lexicon_tags_15 },
    { 0xb427d822, SUC_lexicon_tags_2 },
    { 0x7aa7d822, SUC_lexicon_tags_49 },
    { 0xbb67d824, SUC_lexicon_tags_8 },
    { 0xec27d823, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e07d82b, SUC_lexicon_tags_29 },
    { 0x88c7d82c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8927d832, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a7d83d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50c7d83f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec7d842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a7d845, SUC_lexicon_tags_10 },
    { 0x2047d845, SUC_lexicon_tags_17 },
    { 0x28c7d846, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc7d84b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6287d850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5187d854, SUC_lexicon_tags_1 },
    { 0xc27d855, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec67d85b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3487d860, SUC_lexicon_tags_43 },
    { 0xede7d861, SUC_lexicon_tags_11 },
    { 0x2927d861, SUC_lexicon_tags_37 },
    { 0x367d863, SUC_lexicon_tags_25 },
    { 0xace7d864, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a7d868, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7d86f, SUC_lexicon_tags_43 },
    { 0xefe7d86f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c7d876, SUC_lexicon_tags_33 },
    { 0x5087d877, SUC_lexicon_tags_7 },
    { 0x2a87d877, SUC_lexicon_tags_11 },
    { 0x3487d879, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c7d87f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf67d885, SUC_lexicon_tags_16 },
    { 0x3047d886, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e7d889, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbce7d88b, SUC_lexicon_tags_31 },
    { 0xa3e7d88c, SUC_lexicon_tags_1 },
    { 0xfbe7d88d, SUC_lexicon_tags_77 },
    { 0x2de7d88e, SUC_lexicon_tags_83 },
    { 0xa567d88f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8447d893, SUC_lexicon_tags_33 },
    { 0x2107d893, SUC_lexicon_tags_17 },
    { 0x66e7d895, SUC_lexicon_tags_47 },
    { 0x7ba7d896, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4c47d898, SUC_lexicon_tags_14 },
    { 0x43e7d899, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee7d89c, SUC_lexicon_tags_2 },
    { 0x3b27d89c, SUC_lexicon_tags_17 },
    { 0xed47d89e, SUC_lexicon_tags_38 },
    { 0x5b87d89e, SUC_lexicon_tags_38 },
    { 0x7687d89e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747d8a9, SUC_lexicon_tags_15 },
    { 0x2fa7d8a9, SUC_lexicon_tags_21 },
    { 0x72c7d8ab, SUC_lexicon_tags_84 },
    { 0x31a7d8a9, SUC_lexicon_tags_77 },
    { 0x33a7d8ad, SUC_lexicon_tags_158 },
    { 0xb6e7d8ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4087d8b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2207d8b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b07d8b4, SUC_lexicon_tags_2 },
    { 0xcfe7d8b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7d8b9, SUC_lexicon_tags_43 },
    { 0xfc87d8b9, SUC_lexicon_tags_43 },
    { 0x6b27d8bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1c7d8bd, SUC_lexicon_tags_14 },
    { 0x60e7d8bd, SUC_lexicon_tags_15 },
    { 0xd087d8be, SUC_lexicon_tags_13 },
    { 0xeb67d8bf, SUC_lexicon_tags_160 },
    { 0x23a7d8c1, SUC_lexicon_tags_1205 },
    { 0xca87d8c1, SUC_lexicon_tags_0 },
    { 0x7c87d8c3, SUC_lexicon_tags_7 },
    { 0x4d67d8c3, SUC_lexicon_tags_11 },
    { 0xbd67d8c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf367d8c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c7d8ce, SUC_lexicon_tags_14 },
    { 0x3547d8cf, SUC_lexicon_tags_65 },
    { 0xc6e7d8d0, SUC_lexicon_tags_47 },
    { 0x8d67d8d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f47d8d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3607d8dd, SUC_lexicon_tags_25 },
    { 0x5fc7d8dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7d8e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3c27d8e9, SUC_lexicon_tags_174 },
    { 0xe747d8e9, SUC_lexicon_tags_2 },
    { 0x9f47d8eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a7d8ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7d8f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1147d8fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7d901, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7d909, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9167d90b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc987d910, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa067d914, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e7d91f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e27d92b, SUC_lexicon_tags_7 },
    { 0xfda7d92c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4467d933, SUC_lexicon_tags_45 },
    { 0x5da7d933, SUC_lexicon_tags_27 },
    { 0x7887d935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe187d937, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1a7d939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfa7d93b, SUC_lexicon_tags_15 },
    { 0x4667d93c, SUC_lexicon_tags_1 },
    { 0x8547d93d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f07d93f, SUC_lexicon_tags_37 },
    { 0x927d93f, SUC_lexicon_tags_7 },
    { 0x58c7d93f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfc27d943, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6007d948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5c7d94a, SUC_lexicon_tags_7 },
    { 0x8ea7d94a, SUC_lexicon_tags_37 },
    { 0xd47d94b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xee47d94e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f87d951, SUC_lexicon_tags_0 },
    { 0x7207d951, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf507d957, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce47d95a, SUC_lexicon_tags_28 },
    { 0x9367d95b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba7d960, SUC_lexicon_tags_28 },
    { 0xb7a7d961, SUC_lexicon_tags_11 },
    { 0x92e7d962, SUC_lexicon_tags_1 },
    { 0x9a67d963, SUC_lexicon_tags_23 },
    { 0x83c7d962, SUC_lexicon_tags_30 },
    { 0xc927d965, SUC_lexicon_tags_82 },
    { 0x74c7d965, SUC_lexicon_tags_47 },
    { 0xb387d966, SUC_lexicon_tags_63 },
    { 0xac27d968, SUC_lexicon_tags_25 },
    { 0x5267d969, SUC_lexicon_tags_37 },
    { 0x1f47d96a, SUC_lexicon_tags_38 },
    { 0x6d67d96b, SUC_lexicon_tags_25 },
    { 0xf947d962, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9c7d96e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x947d971, SUC_lexicon_tags_25 },
    { 0xc487d972, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d67d976, SUC_lexicon_tags_11 },
    { 0x3087d977, SUC_lexicon_tags_12 },
    { 0x6a07d977, SUC_lexicon_tags_1 },
    { 0x727d978, SUC_lexicon_tags_17 },
    { 0x5827d97a, SUC_lexicon_tags_46 },
    { 0x9687d97b, SUC_lexicon_tags_25 },
    { 0x2d27d97b, SUC_lexicon_tags_47 },
    { 0xb0a7d978, SUC_lexicon_tags_17 },
    { 0x8ba7d97e, SUC_lexicon_tags_61 },
    { 0x20a7d97e, SUC_lexicon_tags_25 },
    { 0x27d980, SUC_lexicon_tags_12 },
    { 0x4687d978, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xde67d984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b67d987, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e7d98a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba7d98d, SUC_lexicon_tags_14 },
    { 0xb4e7d98d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b47d992, SUC_lexicon_tags_25 },
    { 0x7407d993, SUC_lexicon_tags_2 },
    { 0xfee7d994, SUC_lexicon_tags_45 },
    { 0xd187d995, SUC_lexicon_tags_37 },
    { 0xa947d995, SUC_lexicon_tags_56 },
    { 0xb447d997, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b07d999, SUC_lexicon_tags_23 },
    { 0xbf87d99a, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e87d99e, SUC_lexicon_tags_11 },
    { 0xb0a7d99f, SUC_lexicon_tags_37 },
    { 0xf927d9a0, SUC_lexicon_tags_16 },
    { 0x2087d9a0, SUC_lexicon_tags_19 },
    { 0x7e67d9a2, SUC_lexicon_tags_1 },
    { 0x5c87d9a3, SUC_lexicon_tags_31 },
    { 0xdc67d9a4, SUC_lexicon_tags_46 },
    { 0x69a7d9a5, SUC_lexicon_tags_165 },
    { 0x9f47d99f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa7d9b0, SUC_lexicon_tags_47 },
    { 0x5de7d9b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2bc7d9b3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa047d9b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e7d9be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf9a7d9c0, SUC_lexicon_tags_26 },
    { 0x85c7d9c0, SUC_lexicon_tags_138 },
    { 0x8ea7d9c2, SUC_lexicon_tags_37 },
    { 0x7027d9c3, SUC_lexicon_tags_25 },
    { 0x7ac7d9c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9427d9c8, SUC_lexicon_tags_12 },
    { 0x5a7d9c9, SUC_lexicon_tags_37 },
    { 0x6567d9ca, SUC_lexicon_tags_7 },
    { 0x5767d9ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6867d9cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde7d9d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x327d9d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c07d9d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c7d9dd, SUC_lexicon_tags_17 },
    { 0xe2a7d9de, SUC_lexicon_tags_14 },
    { 0x9507d9df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea7d9e6, SUC_lexicon_tags_19 },
    { 0x3307d9e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa7d9ec, SUC_lexicon_tags_7 },
    { 0x7847d9ed, SUC_lexicon_tags_38 },
    { 0x45e7d9ec, SUC_lexicon_tags_42 },
    { 0xc4c7d9ee, SUC_lexicon_tags_7 },
    { 0xcfa7d9f0, SUC_lexicon_tags_15 },
    { 0x8667d9f1, SUC_lexicon_tags_45 },
    { 0xbb27d9f2, SUC_lexicon_tags_14 },
    { 0xca67d9f3, SUC_lexicon_tags_108 },
    { 0x3f87d9f2, SUC_lexicon_tags_26 },
    { 0xc2e7d9f5, SUC_lexicon_tags_46 },
    { 0x547d9f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9507d9fd, SUC_lexicon_tags_17 },
    { 0xafc7d9fd, SUC_lexicon_tags_8 },
    { 0xbb47d9fe, SUC_lexicon_tags_65 },
    { 0xf407da00, SUC_lexicon_tags_27 },
    { 0x3007d9fd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567da07, SUC_lexicon_tags_12 },
    { 0xb527da08, SUC_lexicon_tags_87 },
    { 0xdea7da09, SUC_lexicon_tags_2 },
    { 0xeac7da0a, SUC_lexicon_tags_11 },
    { 0x2067da08, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c7da10, SUC_lexicon_tags_37 },
    { 0xcf47da11, SUC_lexicon_tags_12 },
    { 0x3de7da10, SUC_lexicon_tags_84 },
    { 0x2807da13, SUC_lexicon_tags_7 },
    { 0xdd87da14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8867da16, SUC_lexicon_tags_31 },
    { 0xdf27da16, SUC_lexicon_tags_16 },
    { 0xb67da18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac47da1e, SUC_lexicon_tags_55 },
    { 0x4e47da1e, SUC_lexicon_tags_4 },
    { 0xd2e7da1f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9467da27, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e7da2b, SUC_lexicon_tags_6 },
    { 0x22a7da2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e7da2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9807da31, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9907da37, SUC_lexicon_tags_17 },
    { 0xdb67da38, SUC_lexicon_tags_17 },
    { 0x3ae7da39, SUC_lexicon_tags_4 },
    { 0xa947da3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de7da42, SUC_lexicon_tags_43 },
    { 0x2547da43, SUC_lexicon_tags_17 },
    { 0xa5e7da44, SUC_lexicon_tags_0 },
    { 0xaa87da45, SUC_lexicon_tags_12 },
    { 0x8bc7da45, SUC_lexicon_tags_17 },
    { 0x7c07da47, SUC_lexicon_tags_21 },
    { 0x27e7da48, SUC_lexicon_tags_12 },
    { 0xce47da47, SUC_lexicon_tags_4 },
    { 0xa007da4a, SUC_lexicon_tags_28 },
    { 0x4287da4b, SUC_lexicon_tags_17 },
    { 0x3c07da4a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2f27da4e, SUC_lexicon_tags_2 },
    { 0x28a7da4f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3c7da51, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1627da53, SUC_lexicon_tags_37 },
    { 0x3e07da54, SUC_lexicon_tags_45 },
    { 0xc007da55, SUC_lexicon_tags_27 },
    { 0xc1a7da55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e7da5a, SUC_lexicon_tags_8 },
    { 0xcc27da5b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc7da5e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac7da61, SUC_lexicon_tags_1 },
    { 0x9667da62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7827da64, SUC_lexicon_tags_38 },
    { 0xdba7da64, SUC_lexicon_tags_25 },
    { 0x68c7da65, SUC_lexicon_tags_37 },
    { 0x6647da66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa707da6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a67da6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1587da6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8747da70, SUC_lexicon_tags_21 },
    { 0xc3a7da70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5027da78, SUC_lexicon_tags_16 },
    { 0xadc7da79, SUC_lexicon_tags_4 },
    { 0xbd67da7a, SUC_lexicon_tags_2 },
    { 0x147da7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e87da7d, SUC_lexicon_tags_49 },
    { 0x29c7da7d, SUC_lexicon_tags_1 },
    { 0x3427da7e, SUC_lexicon_tags_23 },
    { 0x5727da80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7027da86, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c7da8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56a7da8d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xca07da8f, SUC_lexicon_tags_15 },
    { 0xcbc7da90, SUC_lexicon_tags_56 },
    { 0x3ae7da90, SUC_lexicon_tags_12 },
    { 0xce67da92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2427da94, SUC_lexicon_tags_16 },
    { 0xe487da95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c7da9a, SUC_lexicon_tags_30 },
    { 0xa667da9b, SUC_lexicon_tags_6 },
    { 0xc9a7da9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68c7da9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27daa2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5827daa8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c7daad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d67dab3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a7daba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1547dabc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc507dabe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b47dac1, SUC_lexicon_tags_16 },
    { 0x52a7dac2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e7dac5, SUC_lexicon_tags_65 },
    { 0x3847dac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb87dac8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3707daca, SUC_lexicon_tags_1 },
    { 0xece7dacb, SUC_lexicon_tags_17 },
    { 0x2d87dacb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf127dad0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa7dad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x947dadb, SUC_lexicon_tags_17 },
    { 0x987dadc, SUC_lexicon_tags_25 },
    { 0x9f27dadb, SUC_lexicon_tags_43 },
    { 0xa6e7dade, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7727dae0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a7dae5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a47dae9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa487daeb, SUC_lexicon_tags_35 },
    { 0x1a07daeb, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x7407daee, SUC_lexicon_tags_5 },
    { 0x727daee, SUC_lexicon_tags_56 },
    { 0x6fe7daf0, SUC_lexicon_tags_17 },
    { 0xdc07daf0, SUC_lexicon_tags_49 },
    { 0x1627daf2, SUC_lexicon_tags_82 },
    { 0x7327daf2, SUC_lexicon_tags_23 },
    { 0x8b07daf1, SUC_lexicon_tags_17 },
    { 0xba27daef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad07daf7, SUC_lexicon_tags_15 },
    { 0xc907daf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd67db00, SUC_lexicon_tags_10 },
    { 0x1da7db01, SUC_lexicon_tags_63 },
    { 0x967db02, SUC_lexicon_tags_103 },
    { 0xcd87db02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f47db05, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c7db08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a7db12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c7db16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda7db19, SUC_lexicon_tags_32 },
    { 0xb347db1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb787db1c, SUC_lexicon_tags_12 },
    { 0xdf27db1d, SUC_lexicon_tags_10 },
    { 0x8947db1d, SUC_lexicon_tags_43 },
    { 0x5327db1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ea7db21, SUC_lexicon_tags_62 },
    { 0x3967db21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98a7db24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd227db2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c7db2d, SUC_lexicon_tags_14 },
    { 0x5a87db2e, SUC_lexicon_tags_15 },
    { 0x4ec7db2e, SUC_lexicon_tags_33 },
    { 0xdf67db30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x94c7db32, SUC_lexicon_tags_44 },
    { 0x8087db33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa087db35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e7db38, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb907db3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34e7db3e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb6e7db40, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xac67db42, SUC_lexicon_tags_7 },
    { 0x1a47db43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7027db4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeda7db4c, SUC_lexicon_tags_8 },
    { 0x4f47db4d, SUC_lexicon_tags_43 },
    { 0x4f47db4d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7db50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc47db54, SUC_lexicon_tags_124 },
    { 0xe007db54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa8a7db57, SUC_lexicon_tags_26 },
    { 0xdf67db57, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c7db5d, SUC_lexicon_tags_34 },
    { 0x9f47db5e, SUC_lexicon_tags_9 },
    { 0x1007db5f, SUC_lexicon_tags_38 },
    { 0x3bc7db5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6547db62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x45e7db64, SUC_lexicon_tags_65 },
    { 0x7e87db64, SUC_lexicon_tags_13 },
    { 0xee7db65, SUC_lexicon_tags_33 },
    { 0x1f07db64, SUC_lexicon_tags_7 },
    { 0x7b87db64, SUC_lexicon_tags_11 },
    { 0x9347db69, SUC_lexicon_tags_17 },
    { 0x8e27db64, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d87db6c, SUC_lexicon_tags_21 },
    { 0x2207db6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc947db71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c67db74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb167db77, SUC_lexicon_tags_20 },
    { 0x6867db77, SUC_lexicon_tags_7 },
    { 0x81c7db78, SUC_lexicon_tags_1 },
    { 0xd787db7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6c7db7c, SUC_lexicon_tags_2 },
    { 0x4f67db7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1547db80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc7db83, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xd647db86, SUC_lexicon_tags_11 },
    { 0xaf67db87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa47db8b, SUC_lexicon_tags_37 },
    { 0xb167db8c, SUC_lexicon_tags_65 },
    { 0x76c7db8d, SUC_lexicon_tags_31 },
    { 0x4407db8c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb6c7db90, SUC_lexicon_tags_37 },
    { 0x9b27db91, SUC_lexicon_tags_29 },
    { 0xce7db92, SUC_lexicon_tags_38 },
    { 0x1367db93, SUC_lexicon_tags_26 },
    { 0x7fa7db92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8547db96, SUC_lexicon_tags_12 },
    { 0xc3c7db97, SUC_lexicon_tags_26 },
    { 0x7f47db97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a7db9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80e7db9d, SUC_lexicon_tags_60 },
    { 0x99e7db9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe447dba1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5c67dba3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf67dba6, SUC_lexicon_tags_1 },
    { 0x51c7dba7, SUC_lexicon_tags_17 },
    { 0x8907dba8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe887dbaa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a7dbae, SUC_lexicon_tags_1 },
    { 0x1cc7dbaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa7dbb3, SUC_lexicon_tags_23 },
    { 0x96c7dbb4, SUC_lexicon_tags_19 },
    { 0x7cc7dbb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2467dbbe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2787dbc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e7dbc4, SUC_lexicon_tags_34 },
    { 0x44a7dbc5, SUC_lexicon_tags_151 },
    { 0x44a7dbc6, SUC_lexicon_tags_11 },
    { 0xd227dbc7, SUC_lexicon_tags_1 },
    { 0x2147dbc8, SUC_lexicon_tags_32 },
    { 0xb387dbc9, SUC_lexicon_tags_26 },
    { 0x92e7dbca, SUC_lexicon_tags_30 },
    { 0x4f87dbc5, SUC_lexicon_tags_17 },
    { 0x3967dbcc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a27dbd1, SUC_lexicon_tags_27 },
    { 0x56c7dbd2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb927dbd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c7dbdd, SUC_lexicon_tags_32 },
    { 0x10e7dbde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2987dbe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb227dbe3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6c7dbe5, SUC_lexicon_tags_12 },
    { 0xa4c7dbe6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7dbea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca7dbee, SUC_lexicon_tags_25 },
    { 0xa927dbef, SUC_lexicon_tags_15 },
    { 0xf2e7dbee, SUC_lexicon_tags_1 },
    { 0x78e7dbf1, SUC_lexicon_tags_11 },
    { 0x2807dbf2, SUC_lexicon_tags_1 },
    { 0x9927dbf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a7dbf9, SUC_lexicon_tags_11 },
    { 0xdee7dbfa, SUC_lexicon_tags_16 },
    { 0x7887dbfb, SUC_lexicon_tags_60 },
    { 0xca87dbfa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c7dc00, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e7dc03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa047dc05, SUC_lexicon_tags_27 },
    { 0xe007dc06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1147dc08, SUC_lexicon_tags_23 },
    { 0xb387dc09, SUC_lexicon_tags_11 },
    { 0x6947dc0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4027dc10, SUC_lexicon_tags_0 },
    { 0x7ce7dc10, SUC_lexicon_tags_14 },
    { 0x18a7dc10, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67dc16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4047dc1d, SUC_lexicon_tags_31 },
    { 0x3207dc1e, SUC_lexicon_tags_82 },
    { 0x4f47dc1f, SUC_lexicon_tags_12 },
    { 0x1207dc20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9607dc25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7067dc29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d07dc2f, SUC_lexicon_tags_7 },
    { 0x3067dc30, SUC_lexicon_tags_14 },
    { 0xaf67dc30, SUC_lexicon_tags_47 },
    { 0x58e7dc32, SUC_lexicon_tags_15 },
    { 0x5667dc2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4167dc35, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247dc3a, SUC_lexicon_tags_29 },
    { 0xd807dc3b, SUC_lexicon_tags_38 },
    { 0x6347dc3b, SUC_lexicon_tags_55 },
    { 0x5d87dc3b, SUC_lexicon_tags_87 },
    { 0x1ee7dc3e, SUC_lexicon_tags_19 },
    { 0x97a7dc3f, SUC_lexicon_tags_35 },
    { 0x94e7dc3c, SUC_lexicon_tags_37 },
    { 0x43a7dc40, SUC_lexicon_tags_11 },
    { 0x84c7dc3e, SUC_lexicon_tags_21 },
    { 0xe8a7dc3e, SUC_lexicon_tags_63 },
    { 0x74e7dc44, SUC_lexicon_tags_11 },
    { 0xd227dc43, SUC_lexicon_tags_28 },
    { 0xba07dc46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc07dc48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c7dc4b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7bc7dc4d, SUC_lexicon_tags_640 },
    { 0, NULL },
    { 0xef67dc4f, SUC_lexicon_tags_17 },
    { 0xfde7dc50, SUC_lexicon_tags_37 },
    { 0xa07dc51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3587dc56, SUC_lexicon_tags_15 },
    { 0x1607dc57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ee7dc59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c07dc5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2887dc61, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a7dc68, SUC_lexicon_tags_63 },
    { 0x1d47dc69, SUC_lexicon_tags_6 },
    { 0x247dc69, SUC_lexicon_tags_11 },
    { 0x8f87dc6b, SUC_lexicon_tags_2 },
    { 0xbc67dc6a, SUC_lexicon_tags_28 },
    { 0xe107dc6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5927dc6f, SUC_lexicon_tags_10 },
    { 0x65c7dc70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a7dc72, SUC_lexicon_tags_11 },
    { 0x9047dc73, SUC_lexicon_tags_31 },
    { 0xb387dc73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e7dc76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8127dc7d, SUC_lexicon_tags_4 },
    { 0xc087dc7e, SUC_lexicon_tags_1 },
    { 0xd007dc7e, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e7dc83, SUC_lexicon_tags_37 },
    { 0xb1a7dc84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a7dc87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3707dc89, SUC_lexicon_tags_11 },
    { 0x207dc8a, SUC_lexicon_tags_87 },
    { 0x5407dc8b, SUC_lexicon_tags_0 },
    { 0x8967dc8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8687dc8e, SUC_lexicon_tags_27 },
    { 0xcb87dc8f, SUC_lexicon_tags_59 },
    { 0x7467dc90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2c7dc92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c7dc96, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5767dc98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e07dc9b, SUC_lexicon_tags_11 },
    { 0x8ec7dc9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2607dca0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf067dca5, SUC_lexicon_tags_29 },
    { 0x78c7dca5, SUC_lexicon_tags_7 },
    { 0x1647dca7, SUC_lexicon_tags_37 },
    { 0xb767dca8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da7dcab, SUC_lexicon_tags_1 },
    { 0xaec7dcac, SUC_lexicon_tags_82 },
    { 0xc247dcad, SUC_lexicon_tags_43 },
    { 0xbf47dcad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb87dcb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a7dcb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f27dcb8, SUC_lexicon_tags_39 },
    { 0x3787dcb8, SUC_lexicon_tags_11 },
    { 0x49e7dcba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb127dcbe, SUC_lexicon_tags_11 },
    { 0x3de7dcbf, SUC_lexicon_tags_39 },
    { 0x40a7dcc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2287dcc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0a7dcc7, SUC_lexicon_tags_82 },
    { 0x7ba7dcc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd67dccd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5727dccf, SUC_lexicon_tags_37 },
    { 0xf207dccf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9027dcd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b47dce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e27dce6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4727dce8, SUC_lexicon_tags_25 },
    { 0xa5e7dce9, SUC_lexicon_tags_84 },
    { 0x59e7dcea, SUC_lexicon_tags_26 },
    { 0x5747dceb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4787dced, SUC_lexicon_tags_82 },
    { 0xa627dcee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc7dcf0, SUC_lexicon_tags_17 },
    { 0x6047dcf0, SUC_lexicon_tags_26 },
    { 0x407dcf1, SUC_lexicon_tags_37 },
    { 0x8c67dcf3, SUC_lexicon_tags_39 },
    { 0x46e7dcf3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b07dcfb, SUC_lexicon_tags_25 },
    { 0xe267dcfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc447dcfe, SUC_lexicon_tags_14 },
    { 0x2127dcfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c7dd02, SUC_lexicon_tags_42 },
    { 0x5827dd02, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce7dd06, SUC_lexicon_tags_2 },
    { 0x9987dd06, SUC_lexicon_tags_47 },
    { 0xd547dd08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xede7dd0b, SUC_lexicon_tags_79 },
    { 0x8e47dd0c, SUC_lexicon_tags_33 },
    { 0x20c7dd0c, SUC_lexicon_tags_37 },
    { 0x1127dd0b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e7dd14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f47dd1b, SUC_lexicon_tags_17 },
    { 0xb87dd1c, SUC_lexicon_tags_45 },
    { 0x1607dd1c, SUC_lexicon_tags_7 },
    { 0xf7c7dd1b, SUC_lexicon_tags_2 },
    { 0x147dd1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c7dd21, SUC_lexicon_tags_1 },
    { 0x1127dd22, SUC_lexicon_tags_27 },
    { 0xd887dd23, SUC_lexicon_tags_11 },
    { 0x44a7dd23, SUC_lexicon_tags_11 },
    { 0xfc7dd21, SUC_lexicon_tags_19 },
    { 0x3a27dd26, SUC_lexicon_tags_4 },
    { 0x5b27dd23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x51a7dd29, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9887dd2b, SUC_lexicon_tags_7 },
    { 0xe747dd2c, SUC_lexicon_tags_27 },
    { 0x4e07dd2d, SUC_lexicon_tags_25 },
    { 0x23a7dd2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb547dd31, SUC_lexicon_tags_11 },
    { 0x74a7dd31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x427dd34, SUC_lexicon_tags_7 },
    { 0x9187dd35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2447dd38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5c7dd3a, SUC_lexicon_tags_11 },
    { 0x10a7dd3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46a7dd3d, SUC_lexicon_tags_12 },
    { 0xd1e7dd3d, SUC_lexicon_tags_37 },
    { 0x2e47dd3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1507dd49, SUC_lexicon_tags_1 },
    { 0x1da7dd49, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d47dd51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4327dd53, SUC_lexicon_tags_72 },
    { 0x7807dd54, SUC_lexicon_tags_74 },
    { 0x9bc7dd55, SUC_lexicon_tags_11 },
    { 0xc1a7dd55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba67dd58, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa7dd5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce7dd60, SUC_lexicon_tags_12 },
    { 0xeb47dd61, SUC_lexicon_tags_11 },
    { 0xbea7dd62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a67dd64, SUC_lexicon_tags_25 },
    { 0xb467dd65, SUC_lexicon_tags_8 },
    { 0x8427dd66, SUC_lexicon_tags_7 },
    { 0x21e7dd66, SUC_lexicon_tags_9 },
    { 0x607dd66, SUC_lexicon_tags_33 },
    { 0x2387dd69, SUC_lexicon_tags_31 },
    { 0x5b07dd6a, SUC_lexicon_tags_46 },
    { 0x1ce7dd6b, SUC_lexicon_tags_1 },
    { 0xd6e7dd64, SUC_lexicon_tags_25 },
    { 0xbf87dd67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2607dd70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82c7dd72, SUC_lexicon_tags_25 },
    { 0xe6a7dd73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e7dd7a, SUC_lexicon_tags_38 },
    { 0xcbe7dd7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c7dd7e, SUC_lexicon_tags_11 },
    { 0xcfa7dd7e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6587dd82, SUC_lexicon_tags_39 },
    { 0x72e7dd82, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa367dd86, SUC_lexicon_tags_28 },
    { 0x4587dd87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb667dd8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c7dd8d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f07dd96, SUC_lexicon_tags_37 },
    { 0xd587dd96, SUC_lexicon_tags_0 },
    { 0xf967dd98, SUC_lexicon_tags_10 },
    { 0xfd27dd98, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc287dd9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8167dda0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e7dda4, SUC_lexicon_tags_11 },
    { 0x3d47dda5, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde07dda9, SUC_lexicon_tags_43 },
    { 0x85e7ddaa, SUC_lexicon_tags_43 },
    { 0x1187ddab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7767ddae, SUC_lexicon_tags_25 },
    { 0x8ee7ddaf, SUC_lexicon_tags_31 },
    { 0xb647ddae, SUC_lexicon_tags_3 },
    { 0x80a7ddb0, SUC_lexicon_tags_8 },
    { 0xf507ddb2, SUC_lexicon_tags_25 },
    { 0xfac7ddb3, SUC_lexicon_tags_43 },
    { 0xfba7ddb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c7ddb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c7ddbc, SUC_lexicon_tags_27 },
    { 0x1cc7ddbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a87ddbf, SUC_lexicon_tags_11 },
    { 0x8ca7ddbf, SUC_lexicon_tags_1 },
    { 0xea07ddc0, SUC_lexicon_tags_19 },
    { 0xee47ddc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53e7ddc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7887ddc7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f27ddca, SUC_lexicon_tags_11 },
    { 0xcba7ddcb, SUC_lexicon_tags_7 },
    { 0x7e87ddcb, SUC_lexicon_tags_61 },
    { 0xf567ddcd, SUC_lexicon_tags_25 },
    { 0x67c7ddce, SUC_lexicon_tags_47 },
    { 0x6687ddcc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc7ddd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe707ddd8, SUC_lexicon_tags_35 },
    { 0xf287ddd9, SUC_lexicon_tags_25 },
    { 0xed87ddda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc187dddc, SUC_lexicon_tags_15 },
    { 0x3947dddd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe447dde0, SUC_lexicon_tags_7 },
    { 0xb267dde1, SUC_lexicon_tags_25 },
    { 0x5467dde0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7127ddea, SUC_lexicon_tags_29 },
    { 0x12c7ddea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7ddf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc967ddf7, SUC_lexicon_tags_11 },
    { 0x2c07ddf8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1007ddfa, SUC_lexicon_tags_4 },
    { 0x1507ddfa, SUC_lexicon_tags_25 },
    { 0xa147ddfc, SUC_lexicon_tags_151 },
    { 0x207ddfd, SUC_lexicon_tags_28 },
    { 0x7aa7ddfd, SUC_lexicon_tags_37 },
    { 0xf87ddfd, SUC_lexicon_tags_2 },
    { 0x6267de00, SUC_lexicon_tags_14 },
    { 0xd107ddfb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc767de04, SUC_lexicon_tags_7 },
    { 0x7227de05, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4f07de07, SUC_lexicon_tags_38 },
    { 0x4a07de08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9a7de0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e7de0e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe607de12, SUC_lexicon_tags_20 },
    { 0x8be7de13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba27de1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ce7de1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd747de1e, SUC_lexicon_tags_30 },
    { 0x6787de1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb887de22, SUC_lexicon_tags_655 },
    { 0x1be7de22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87de28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf127de2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6427de32, SUC_lexicon_tags_47 },
    { 0x5707de32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a7de36, SUC_lexicon_tags_12 },
    { 0xfe87de37, SUC_lexicon_tags_1 },
    { 0x27c7de37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1647de3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd047de3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf47de43, SUC_lexicon_tags_60 },
    { 0xea67de43, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb0e7de46, SUC_lexicon_tags_37 },
    { 0xed27de47, SUC_lexicon_tags_47 },
    { 0x4227de48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c7de4d, SUC_lexicon_tags_35 },
    { 0x3327de4e, SUC_lexicon_tags_44 },
    { 0x8447de4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15e7de51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae7de55, SUC_lexicon_tags_19 },
    { 0xe627de56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea7de59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa827de60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da7de67, SUC_lexicon_tags_19 },
    { 0xf907de68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8187de6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef87de6c, SUC_lexicon_tags_11 },
    { 0x927de6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c7de82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb787de84, SUC_lexicon_tags_11 },
    { 0x17a7de85, SUC_lexicon_tags_63 },
    { 0x1a47de86, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3ee7de88, SUC_lexicon_tags_4 },
    { 0x7067de89, SUC_lexicon_tags_29 },
    { 0xb6e7de8a, SUC_lexicon_tags_1 },
    { 0x34c7de8b, SUC_lexicon_tags_13 },
    { 0xf4a7de8c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xfcc7de8e, SUC_lexicon_tags_45 },
    { 0x87de8e, SUC_lexicon_tags_56 },
    { 0xda07de90, SUC_lexicon_tags_48 },
    { 0x8527de90, SUC_lexicon_tags_37 },
    { 0xdac7de90, SUC_lexicon_tags_26 },
    { 0x21a7de92, SUC_lexicon_tags_26 },
    { 0x7ae7de94, SUC_lexicon_tags_45 },
    { 0xe2e7de92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd47de9a, SUC_lexicon_tags_81 },
    { 0xbee7de9b, SUC_lexicon_tags_17 },
    { 0xe7c7de9c, SUC_lexicon_tags_35 },
    { 0xeae7de9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb187de9f, SUC_lexicon_tags_37 },
    { 0xb827dea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a7dea2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2927dea4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdbe7dea6, SUC_lexicon_tags_2 },
    { 0x29a7dea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a7deab, SUC_lexicon_tags_500 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe7deb2, SUC_lexicon_tags_1 },
    { 0x1fa7deb3, SUC_lexicon_tags_54 },
    { 0xf5a7deb2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7deb9, SUC_lexicon_tags_37 },
    { 0x91c7deba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c07debf, SUC_lexicon_tags_28 },
    { 0x8767dec0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3387dec2, SUC_lexicon_tags_11 },
    { 0xe167dec2, SUC_lexicon_tags_70 },
    { 0x1ba7dec2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4147dec8, SUC_lexicon_tags_33 },
    { 0x7a07dec9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9bc7decb, SUC_lexicon_tags_38 },
    { 0x28e7decc, SUC_lexicon_tags_2 },
    { 0x2327decd, SUC_lexicon_tags_19 },
    { 0xeac7decc, SUC_lexicon_tags_3 },
    { 0xba7decf, SUC_lexicon_tags_19 },
    { 0x7027decd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4da7ded2, SUC_lexicon_tags_14 },
    { 0x2e67ded2, SUC_lexicon_tags_25 },
    { 0x3c27ded4, SUC_lexicon_tags_17 },
    { 0x2d87ded5, SUC_lexicon_tags_26 },
    { 0x1cc7ded4, SUC_lexicon_tags_25 },
    { 0x44a7ded7, SUC_lexicon_tags_32 },
    { 0xde67ded8, SUC_lexicon_tags_33 },
    { 0x7587ded7, SUC_lexicon_tags_507 },
    { 0x4a87ded7, SUC_lexicon_tags_46 },
    { 0xa867ded8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd887dee1, SUC_lexicon_tags_0 },
    { 0x9167dee2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdaa7dee4, SUC_lexicon_tags_82 },
    { 0xdb67dee5, SUC_lexicon_tags_2 },
    { 0xec7dee5, SUC_lexicon_tags_2 },
    { 0x1187dee7, SUC_lexicon_tags_7 },
    { 0x8a47dee7, SUC_lexicon_tags_37 },
    { 0xc9e7dee9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ea7deeb, SUC_lexicon_tags_48 },
    { 0x2767deec, SUC_lexicon_tags_30 },
    { 0x4f07deed, SUC_lexicon_tags_25 },
    { 0xbc87deee, SUC_lexicon_tags_39 },
    { 0x62c7deed, SUC_lexicon_tags_15 },
    { 0x5b07deec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9227def3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f07def8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8cc7defa, SUC_lexicon_tags_68 },
    { 0x7defb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb007defd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107df07, SUC_lexicon_tags_47 },
    { 0xa3a7df08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf27df0a, SUC_lexicon_tags_1 },
    { 0x5d47df0a, SUC_lexicon_tags_40 },
    { 0xc087df0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdc87df0e, SUC_lexicon_tags_15 },
    { 0xbb87df0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7707df13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf567df1a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xb047df1d, SUC_lexicon_tags_18 },
    { 0xbc7df1e, SUC_lexicon_tags_6 },
    { 0xca7df1e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe307df21, SUC_lexicon_tags_30 },
    { 0x3847df22, SUC_lexicon_tags_23 },
    { 0x3327df23, SUC_lexicon_tags_37 },
    { 0x3227df23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be7df28, SUC_lexicon_tags_60 },
    { 0x9f87df29, SUC_lexicon_tags_15 },
    { 0x59c7df2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347df3e, SUC_lexicon_tags_11 },
    { 0xa687df3f, SUC_lexicon_tags_15 },
    { 0x3f07df40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c7df49, SUC_lexicon_tags_26 },
    { 0x9347df4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf027df4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6867df54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60c7df56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9887df5d, SUC_lexicon_tags_12 },
    { 0x9407df5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae7df63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9727df65, SUC_lexicon_tags_1 },
    { 0xab87df66, SUC_lexicon_tags_2 },
    { 0x43c7df67, SUC_lexicon_tags_0 },
    { 0x1087df66, SUC_lexicon_tags_25 },
    { 0x27c7df69, SUC_lexicon_tags_6 },
    { 0xb347df6a, SUC_lexicon_tags_63 },
    { 0x7fe7df6b, SUC_lexicon_tags_60 },
    { 0x4d87df69, SUC_lexicon_tags_7 },
    { 0x1987df6c, SUC_lexicon_tags_25 },
    { 0x7207df6b, SUC_lexicon_tags_11 },
    { 0x1407df6f, SUC_lexicon_tags_1 },
    { 0x9827df68, SUC_lexicon_tags_22 },
    { 0x4a47df71, SUC_lexicon_tags_33 },
    { 0x1d47df69, SUC_lexicon_tags_37 },
    { 0x27a7df6b, SUC_lexicon_tags_35 },
    { 0x5ca7df66, SUC_lexicon_tags_15 },
    { 0x7a27df75, SUC_lexicon_tags_11 },
    { 0xb547df75, SUC_lexicon_tags_7 },
    { 0xce27df77, SUC_lexicon_tags_17 },
    { 0x9b07df77, SUC_lexicon_tags_10 },
    { 0xf6a7df79, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf987df80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf5c7df82, SUC_lexicon_tags_179 },
    { 0x4e67df82, SUC_lexicon_tags_45 },
    { 0x7b27df84, SUC_lexicon_tags_37 },
    { 0x4ec7df84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe487df87, SUC_lexicon_tags_39 },
    { 0x347df88, SUC_lexicon_tags_65 },
    { 0xba87df89, SUC_lexicon_tags_7 },
    { 0xd9a7df8a, SUC_lexicon_tags_65 },
    { 0xa6e7df8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc607df8f, SUC_lexicon_tags_2 },
    { 0x3a7df8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd687df92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22c7df94, SUC_lexicon_tags_23 },
    { 0x69c7df94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc387df98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c87df9a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca7df9e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa587dfa1, SUC_lexicon_tags_17 },
    { 0x1307dfa2, SUC_lexicon_tags_15 },
    { 0xb6c7dfa1, SUC_lexicon_tags_43 },
    { 0x2d67dfa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61a7dfa6, SUC_lexicon_tags_38 },
    { 0xc67dfa6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2907dfab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe7dfae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a7dfb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa387dfb5, SUC_lexicon_tags_30 },
    { 0x8f87dfb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c7dfbe, SUC_lexicon_tags_9 },
    { 0xe127dfbf, SUC_lexicon_tags_14 },
    { 0xfc67dfbf, SUC_lexicon_tags_5 },
    { 0x3ec7dfc1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbec7dfc3, SUC_lexicon_tags_1 },
    { 0xd447dfc4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba7dfc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e07dfd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67dfd6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4927dfd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a47dfde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc707dfe0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2047dfe3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c87dfe7, SUC_lexicon_tags_11 },
    { 0x7b47dfe7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x81a7dfea, SUC_lexicon_tags_25 },
    { 0x80e7dfeb, SUC_lexicon_tags_6 },
    { 0xd807dfec, SUC_lexicon_tags_11 },
    { 0x99a7dfeb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xde27dfef, SUC_lexicon_tags_25 },
    { 0x4487dff0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e7dff6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5227dffe, SUC_lexicon_tags_7 },
    { 0xc247dfff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ae7e001, SUC_lexicon_tags_0 },
    { 0x5c47e002, SUC_lexicon_tags_15 },
    { 0x96e7e003, SUC_lexicon_tags_68 },
    { 0x9a27e001, SUC_lexicon_tags_14 },
    { 0x4c07e005, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e27e008, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c7e015, SUC_lexicon_tags_9 },
    { 0xaa67e016, SUC_lexicon_tags_38 },
    { 0x2bc7e017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9327e01b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa507e01d, SUC_lexicon_tags_250 },
    { 0xb287e01d, SUC_lexicon_tags_25 },
    { 0x3fa7e01f, SUC_lexicon_tags_7 },
    { 0xf047e01f, SUC_lexicon_tags_11 },
    { 0xdf27e021, SUC_lexicon_tags_27 },
    { 0xec7e01e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c87e024, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f47e029, SUC_lexicon_tags_25 },
    { 0x1b87e029, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc387e02d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f07e030, SUC_lexicon_tags_52 },
    { 0x5f87e031, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c7e034, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xfe07e036, SUC_lexicon_tags_23 },
    { 0xbee7e037, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c47e03c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa967e041, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c7e044, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc7e04a, SUC_lexicon_tags_7 },
    { 0x50c7e04a, SUC_lexicon_tags_87 },
    { 0xd9e7e04c, SUC_lexicon_tags_37 },
    { 0xff87e04d, SUC_lexicon_tags_30 },
    { 0x7f67e04e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc987e053, SUC_lexicon_tags_25 },
    { 0xad27e054, SUC_lexicon_tags_17 },
    { 0xb087e055, SUC_lexicon_tags_82 },
    { 0xf5c7e053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd587e05b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f47e05d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a7e065, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e7e06a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27e06e, SUC_lexicon_tags_38 },
    { 0xb707e06e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x147e071, SUC_lexicon_tags_33 },
    { 0x80a7e072, SUC_lexicon_tags_9 },
    { 0xa9e7e073, SUC_lexicon_tags_2 },
    { 0x8387e073, SUC_lexicon_tags_11 },
    { 0x7307e075, SUC_lexicon_tags_74 },
    { 0xdc7e076, SUC_lexicon_tags_17 },
    { 0xe807e077, SUC_lexicon_tags_43 },
    { 0xf207e078, SUC_lexicon_tags_25 },
    { 0x1587e079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ce7e07b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa7e07f, SUC_lexicon_tags_34 },
    { 0xa707e080, SUC_lexicon_tags_68 },
    { 0x8027e080, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18a7e083, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e7e086, SUC_lexicon_tags_27 },
    { 0xe747e086, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c7e08a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4847e08d, SUC_lexicon_tags_83 },
    { 0x56a7e08e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5287e091, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece7e09a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe067e09c, SUC_lexicon_tags_15 },
    { 0xd6e7e09d, SUC_lexicon_tags_15 },
    { 0xdae7e09e, SUC_lexicon_tags_10 },
    { 0xd887e09e, SUC_lexicon_tags_2 },
    { 0x6987e09e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf27e0a6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4d87e0a8, SUC_lexicon_tags_23 },
    { 0x1767e0a9, SUC_lexicon_tags_44 },
    { 0xeb27e0aa, SUC_lexicon_tags_9 },
    { 0xfaa7e0a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb67e0ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a27e0b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9527e0b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e67e0b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e7e0bf, SUC_lexicon_tags_1 },
    { 0xf247e0c0, SUC_lexicon_tags_7 },
    { 0xb2c7e0bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a7e0c8, SUC_lexicon_tags_10 },
    { 0x2a47e0c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa7e0cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1387e0cf, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b67e0d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x727e0d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbe7e0d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6767e0df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad27e0e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7387e0ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4087e0ec, SUC_lexicon_tags_49 },
    { 0x4387e0ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c47e0ef, SUC_lexicon_tags_37 },
    { 0x7707e0ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ca7e0f2, SUC_lexicon_tags_45 },
    { 0x6dc7e0f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb8a7e0f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x667e0f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x367e0fa, SUC_lexicon_tags_15 },
    { 0x8da7e0fb, SUC_lexicon_tags_11 },
    { 0xd807e0fc, SUC_lexicon_tags_14 },
    { 0x6ee7e0fa, SUC_lexicon_tags_11 },
    { 0xf67e0fe, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x2307e101, SUC_lexicon_tags_17 },
    { 0xb907e102, SUC_lexicon_tags_60 },
    { 0x1007e101, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a7e107, SUC_lexicon_tags_3 },
    { 0x47e7e107, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba27e10b, SUC_lexicon_tags_66 },
    { 0x1ae7e10c, SUC_lexicon_tags_15 },
    { 0xddc7e10d, SUC_lexicon_tags_47 },
    { 0xe187e10d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb87e119, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xd427e11c, SUC_lexicon_tags_87 },
    { 0x3867e11c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c87e129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd607e12c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf107e12e, SUC_lexicon_tags_12 },
    { 0xa647e12e, SUC_lexicon_tags_15 },
    { 0xdde7e130, SUC_lexicon_tags_25 },
    { 0x3dc7e131, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67e134, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e7e139, SUC_lexicon_tags_25 },
    { 0x2227e139, SUC_lexicon_tags_0 },
    { 0x3cc7e13b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f07e13e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e7e143, SUC_lexicon_tags_15 },
    { 0xae7e143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5207e147, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9387e149, SUC_lexicon_tags_23 },
    { 0xa347e14a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x1767e14d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1227e14f, SUC_lexicon_tags_47 },
    { 0x1c07e150, SUC_lexicon_tags_43 },
    { 0xfa27e151, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0x7547e154, SUC_lexicon_tags_19 },
    { 0x4c47e154, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd0c7e157, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc27e163, SUC_lexicon_tags_46 },
    { 0xd987e164, SUC_lexicon_tags_15 },
    { 0x8467e164, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46e7e167, SUC_lexicon_tags_40 },
    { 0x9567e167, SUC_lexicon_tags_27 },
    { 0x6647e168, SUC_lexicon_tags_0 },
    { 0x8a47e167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c07e16d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb027e16f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e7e172, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c27e175, SUC_lexicon_tags_2 },
    { 0xb5a7e176, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd87e179, SUC_lexicon_tags_1 },
    { 0xb847e179, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9567e17c, SUC_lexicon_tags_37 },
    { 0xfd47e17d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c07e184, SUC_lexicon_tags_61 },
    { 0x35c7e184, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa167e194, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce7e197, SUC_lexicon_tags_30 },
    { 0x3a87e197, SUC_lexicon_tags_2 },
    { 0xe0a7e199, SUC_lexicon_tags_7 },
    { 0xfa67e19a, SUC_lexicon_tags_14 },
    { 0x92c7e19a, SUC_lexicon_tags_11 },
    { 0xafa7e19b, SUC_lexicon_tags_7 },
    { 0xac67e199, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6a7e19f, SUC_lexicon_tags_19 },
    { 0xe087e1a0, SUC_lexicon_tags_40 },
    { 0xd347e1a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee7e1a9, SUC_lexicon_tags_30 },
    { 0x887e1aa, SUC_lexicon_tags_16 },
    { 0x2407e1aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a7e1af, SUC_lexicon_tags_15 },
    { 0x42a7e1b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b07e1b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83a7e1b6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5707e1b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x96e7e1bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a87e1bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb687e1bf, SUC_lexicon_tags_16 },
    { 0xe987e1bf, SUC_lexicon_tags_25 },
    { 0xdc47e1c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21c7e1c3, SUC_lexicon_tags_6 },
    { 0xca87e1c4, SUC_lexicon_tags_12 },
    { 0x3f27e1c5, SUC_lexicon_tags_15 },
    { 0x9e47e1c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2767e1c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1a7e1ca, SUC_lexicon_tags_25 },
    { 0xe607e1cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe567e1ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd67e1d1, SUC_lexicon_tags_60 },
    { 0x71c7e1d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6547e1d4, SUC_lexicon_tags_8 },
    { 0x3467e1d5, SUC_lexicon_tags_43 },
    { 0xbe07e1d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4e7e1d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e7e1dc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a47e1e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6447e1e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c7e1e9, SUC_lexicon_tags_12 },
    { 0x11c7e1e9, SUC_lexicon_tags_31 },
    { 0xe9a7e1eb, SUC_lexicon_tags_28 },
    { 0x5d07e1ec, SUC_lexicon_tags_33 },
    { 0x56e7e1ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7e1f5, SUC_lexicon_tags_25 },
    { 0xe507e1f6, SUC_lexicon_tags_9 },
    { 0x4047e1f7, SUC_lexicon_tags_43 },
    { 0x7b47e1f8, SUC_lexicon_tags_26 },
    { 0x3a67e1f9, SUC_lexicon_tags_77 },
    { 0xf287e1f8, SUC_lexicon_tags_40 },
    { 0x17e7e1f9, SUC_lexicon_tags_31 },
    { 0x5f67e1fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c7e1ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a7e202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7e20b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5427e20f, SUC_lexicon_tags_8 },
    { 0x4da7e210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe547e212, SUC_lexicon_tags_27 },
    { 0xe27e212, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbc7e215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5367e21a, SUC_lexicon_tags_2 },
    { 0x9587e21b, SUC_lexicon_tags_17 },
    { 0xaf07e21b, SUC_lexicon_tags_1 },
    { 0x567e21d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc847e220, SUC_lexicon_tags_33 },
    { 0xf3a7e221, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce67e223, SUC_lexicon_tags_43 },
    { 0xcdc7e224, SUC_lexicon_tags_18 },
    { 0xcd87e225, SUC_lexicon_tags_15 },
    { 0x8f67e226, SUC_lexicon_tags_63 },
    { 0xdbc7e223, SUC_lexicon_tags_47 },
    { 0x2207e228, SUC_lexicon_tags_37 },
    { 0x8a47e229, SUC_lexicon_tags_12 },
    { 0x2cc7e22a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4387e231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7047e233, SUC_lexicon_tags_15 },
    { 0xd7c7e234, SUC_lexicon_tags_23 },
    { 0x66a7e234, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667e239, SUC_lexicon_tags_33 },
    { 0xda47e23a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427e243, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa307e245, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe367e248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e67e24a, SUC_lexicon_tags_8 },
    { 0xe3a7e24b, SUC_lexicon_tags_25 },
    { 0x3c7e24c, SUC_lexicon_tags_11 },
    { 0xf067e24d, SUC_lexicon_tags_15 },
    { 0x92c7e24e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6827e251, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3327e257, SUC_lexicon_tags_6 },
    { 0x80e7e258, SUC_lexicon_tags_27 },
    { 0x5007e258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7507e25b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb47e25f, SUC_lexicon_tags_12 },
    { 0xaae7e260, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e27e266, SUC_lexicon_tags_37 },
    { 0xa8c7e267, SUC_lexicon_tags_29 },
    { 0x287e268, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b67e271, SUC_lexicon_tags_65 },
    { 0xc907e272, SUC_lexicon_tags_17 },
    { 0x9407e273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7e27c, SUC_lexicon_tags_15 },
    { 0x15e7e27d, SUC_lexicon_tags_49 },
    { 0x8927e27e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71c7e280, SUC_lexicon_tags_14 },
    { 0x95e7e281, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e7e284, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2107e286, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9127e288, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a7e28b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7987e28d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e7e290, SUC_lexicon_tags_12 },
    { 0xed47e291, SUC_lexicon_tags_13 },
    { 0xaee7e290, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c87e297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de7e29b, SUC_lexicon_tags_20 },
    { 0xefa7e29c, SUC_lexicon_tags_2 },
    { 0xcaa7e29b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de7e2a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a7e2a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6587e2a6, SUC_lexicon_tags_15 },
    { 0xe8a7e2a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x527e2a9, SUC_lexicon_tags_7 },
    { 0x4d47e2aa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2ca7e2ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc867e2af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a67e2b6, SUC_lexicon_tags_47 },
    { 0x37e7e2b7, SUC_lexicon_tags_46 },
    { 0x95c7e2b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1107e2bd, SUC_lexicon_tags_15 },
    { 0xc627e2be, SUC_lexicon_tags_17 },
    { 0x2727e2bd, SUC_lexicon_tags_17 },
    { 0x1a27e2c0, SUC_lexicon_tags_37 },
    { 0x9707e2bd, SUC_lexicon_tags_15 },
    { 0xb5e7e2c2, SUC_lexicon_tags_37 },
    { 0x3867e2c3, SUC_lexicon_tags_15 },
    { 0xf767e2c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e7e2c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb407e2cc, SUC_lexicon_tags_11 },
    { 0xa3c7e2cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca7e2d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e7e2d4, SUC_lexicon_tags_2 },
    { 0x36c7e2d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6587e2d7, SUC_lexicon_tags_12 },
    { 0x84c7e2d7, SUC_lexicon_tags_7 },
    { 0x8ea7e2d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcfc7e2db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x94c7e2dd, SUC_lexicon_tags_6 },
    { 0xb6e7e2de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7947e2e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfc27e2e2, SUC_lexicon_tags_15 },
    { 0x4b87e2e2, SUC_lexicon_tags_38 },
    { 0xbc67e2e4, SUC_lexicon_tags_26 },
    { 0x37e7e2e4, SUC_lexicon_tags_65 },
    { 0x5447e2e6, SUC_lexicon_tags_24 },
    { 0x7927e2e3, SUC_lexicon_tags_11 },
    { 0x67e2e2, SUC_lexicon_tags_47 },
    { 0x5c87e2e3, SUC_lexicon_tags_25 },
    { 0x3727e2e6, SUC_lexicon_tags_17 },
    { 0x26e7e2e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2307e2ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1207e2f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf127e2f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb467e2f9, SUC_lexicon_tags_27 },
    { 0x9567e2f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c7e300, SUC_lexicon_tags_1 },
    { 0xea87e300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d87e304, SUC_lexicon_tags_1 },
    { 0x48a7e305, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8127e308, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc7e30e, SUC_lexicon_tags_29 },
    { 0x8807e30f, SUC_lexicon_tags_38 },
    { 0x6a7e30e, SUC_lexicon_tags_15 },
    { 0x7287e311, SUC_lexicon_tags_17 },
    { 0x6747e30e, SUC_lexicon_tags_77 },
    { 0xfa47e310, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c7e31b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e7e31f, SUC_lexicon_tags_29 },
    { 0x147e31f, SUC_lexicon_tags_47 },
    { 0x61a7e31f, SUC_lexicon_tags_37 },
    { 0x5027e322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e7e327, SUC_lexicon_tags_37 },
    { 0xf447e327, SUC_lexicon_tags_7 },
    { 0xd2a7e329, SUC_lexicon_tags_63 },
    { 0x5647e32a, SUC_lexicon_tags_2 },
    { 0x5007e32b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a7e330, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45c7e332, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f87e337, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf07e339, SUC_lexicon_tags_11 },
    { 0x1ba7e33a, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a7e33e, SUC_lexicon_tags_11 },
    { 0xb7e7e33e, SUC_lexicon_tags_35 },
    { 0x13e7e340, SUC_lexicon_tags_23 },
    { 0x9707e340, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8747e343, SUC_lexicon_tags_82 },
    { 0x9ea7e343, SUC_lexicon_tags_37 },
    { 0x7127e343, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a47e347, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367e34e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9527e351, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee7e354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc607e359, SUC_lexicon_tags_17 },
    { 0xc167e35a, SUC_lexicon_tags_8 },
    { 0xd07e35a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b47e363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa467e36a, SUC_lexicon_tags_37 },
    { 0x44a7e36b, SUC_lexicon_tags_35 },
    { 0xfde7e36b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49a7e36e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1447e371, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c87e377, SUC_lexicon_tags_12 },
    { 0x99a7e378, SUC_lexicon_tags_44 },
    { 0xcc7e379, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47e37c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e27e37e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc7e382, SUC_lexicon_tags_84 },
    { 0xb3c7e382, SUC_lexicon_tags_2 },
    { 0x9e7e384, SUC_lexicon_tags_25 },
    { 0xa7e7e385, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a27e389, SUC_lexicon_tags_60 },
    { 0x4707e38a, SUC_lexicon_tags_15 },
    { 0x91a7e38b, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd07e393, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc507e397, SUC_lexicon_tags_69 },
    { 0x2767e398, SUC_lexicon_tags_7 },
    { 0x1cc7e397, SUC_lexicon_tags_17 },
    { 0x84e7e39a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8a7e39c, SUC_lexicon_tags_87 },
    { 0x7e7e39c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce27e3a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec7e3a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a7e3ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf387e3b1, SUC_lexicon_tags_31 },
    { 0x8087e3b1, SUC_lexicon_tags_49 },
    { 0xf787e3b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec7e3b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xec07e3b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaca7e3ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f47e3bd, SUC_lexicon_tags_30 },
    { 0x2127e3be, SUC_lexicon_tags_37 },
    { 0x98c7e3bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e7e3c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ea7e3c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x907e3cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c7e3cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a7e3d3, SUC_lexicon_tags_68 },
    { 0x5f27e3d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1947e3d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb507e3d9, SUC_lexicon_tags_38 },
    { 0xec27e3da, SUC_lexicon_tags_12 },
    { 0x9dc7e3db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e7e3de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2207e3e0, SUC_lexicon_tags_38 },
    { 0xb27e3e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c7e3e5, SUC_lexicon_tags_29 },
    { 0x5a07e3e6, SUC_lexicon_tags_48 },
    { 0xbf67e3e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1ea7e3e9, SUC_lexicon_tags_1 },
    { 0xa6a7e3ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3c7e3ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4b67e3ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc7e3f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce27e3f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a87e40d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7347e40f, SUC_lexicon_tags_14 },
    { 0xb167e410, SUC_lexicon_tags_14 },
    { 0xe847e40f, SUC_lexicon_tags_2 },
    { 0x4ea7e412, SUC_lexicon_tags_2 },
    { 0x5447e412, SUC_lexicon_tags_75 },
    { 0x8de7e412, SUC_lexicon_tags_37 },
    { 0x43c7e414, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f67e419, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b67e41d, SUC_lexicon_tags_0 },
    { 0xb2c7e41d, SUC_lexicon_tags_24 },
    { 0x4b67e41f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc507e421, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf07e426, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f07e42f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc7e433, SUC_lexicon_tags_60 },
    { 0xa787e434, SUC_lexicon_tags_62 },
    { 0xffc7e435, SUC_lexicon_tags_40 },
    { 0x79e7e436, SUC_lexicon_tags_12 },
    { 0x26a7e437, SUC_lexicon_tags_14 },
    { 0x8827e437, SUC_lexicon_tags_9 },
    { 0xf407e437, SUC_lexicon_tags_23 },
    { 0xbc27e43a, SUC_lexicon_tags_37 },
    { 0xf887e438, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf987e43e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7087e442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e7e447, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf47e449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa7e450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e7e459, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc7e45c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x25e7e45e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3987e460, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7e468, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5627e46b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x44a7e46d, SUC_lexicon_tags_63 },
    { 0x967e46d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb87e472, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe07e475, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c67e479, SUC_lexicon_tags_15 },
    { 0xa887e479, SUC_lexicon_tags_7 },
    { 0x7bc7e47b, SUC_lexicon_tags_14 },
    { 0xece7e47c, SUC_lexicon_tags_26 },
    { 0xdb67e479, SUC_lexicon_tags_14 },
    { 0x8547e47e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2447e480, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7847e486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89c7e488, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8b07e48a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30a7e48c, SUC_lexicon_tags_107 },
    { 0xff07e48c, SUC_lexicon_tags_37 },
    { 0x1627e48d, SUC_lexicon_tags_11 },
    { 0x4e47e48f, SUC_lexicon_tags_53 },
    { 0xc5c7e490, SUC_lexicon_tags_0 },
    { 0x63c7e490, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa627e496, SUC_lexicon_tags_12 },
    { 0x50a7e496, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1447e4a1, SUC_lexicon_tags_49 },
    { 0xa327e4a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc7e4a7, SUC_lexicon_tags_15 },
    { 0x2007e4a8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8a07e4aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac7e4ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4567e4b1, SUC_lexicon_tags_68 },
    { 0xb527e4b2, SUC_lexicon_tags_69 },
    { 0xa5a7e4b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d07e4b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf447e4ba, SUC_lexicon_tags_38 },
    { 0x3087e4bb, SUC_lexicon_tags_1 },
    { 0xd87e4bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7e4c1, SUC_lexicon_tags_7 },
    { 0x9767e4c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e27e4c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8647e4c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27e4ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5a47e4d0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x19e7e4d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e7e4d7, SUC_lexicon_tags_44 },
    { 0x4ba7e4d8, SUC_lexicon_tags_37 },
    { 0x487e4d7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb467e4dc, SUC_lexicon_tags_45 },
    { 0x7807e4dd, SUC_lexicon_tags_64 },
    { 0xbe27e4de, SUC_lexicon_tags_7 },
    { 0x9ae7e4df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c07e4e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6147e4e8, SUC_lexicon_tags_11 },
    { 0xbbc7e4e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c7e4f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xebc7e4f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a27e4f8, SUC_lexicon_tags_25 },
    { 0x2b07e4f9, SUC_lexicon_tags_37 },
    { 0x1347e4fa, SUC_lexicon_tags_28 },
    { 0x77a7e4fb, SUC_lexicon_tags_7 },
    { 0x2bc7e4f9, SUC_lexicon_tags_23 },
    { 0xaba7e4fa, SUC_lexicon_tags_12 },
    { 0xf667e4fe, SUC_lexicon_tags_19 },
    { 0xbfa7e4fa, SUC_lexicon_tags_16 },
    { 0x547e4fc, SUC_lexicon_tags_28 },
    { 0x3dc7e501, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c7e504, SUC_lexicon_tags_127 },
    { 0xce27e505, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff27e50b, SUC_lexicon_tags_23 },
    { 0xfd27e50b, SUC_lexicon_tags_19 },
    { 0xf4a7e50c, SUC_lexicon_tags_11 },
    { 0x92c7e50b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c27e511, SUC_lexicon_tags_85 },
    { 0x3cc7e512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa587e518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e7e51b, SUC_lexicon_tags_17 },
    { 0xaf47e51b, SUC_lexicon_tags_1 },
    { 0x9f67e51c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a7e522, SUC_lexicon_tags_11 },
    { 0x4607e522, SUC_lexicon_tags_15 },
    { 0x9047e524, SUC_lexicon_tags_1 },
    { 0x9c67e524, SUC_lexicon_tags_37 },
    { 0x8007e523, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2847e529, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1747e52b, SUC_lexicon_tags_87 },
    { 0xb567e52c, SUC_lexicon_tags_2 },
    { 0x3b07e52c, SUC_lexicon_tags_33 },
    { 0xe7a7e52e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a27e534, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8347e539, SUC_lexicon_tags_1 },
    { 0x8fa7e53a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa187e53f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x1a87e541, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a7e544, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee7e547, SUC_lexicon_tags_37 },
    { 0xe407e547, SUC_lexicon_tags_7 },
    { 0x71c7e549, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd047e54b, SUC_lexicon_tags_3 },
    { 0xe0c7e54c, SUC_lexicon_tags_17 },
    { 0x8a07e54d, SUC_lexicon_tags_14 },
    { 0x21e7e54e, SUC_lexicon_tags_2 },
    { 0xa007e54c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c7e554, SUC_lexicon_tags_57 },
    { 0x6f27e555, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x75e7e557, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e7e55c, SUC_lexicon_tags_343 },
    { 0xeae7e55d, SUC_lexicon_tags_52 },
    { 0xb727e55d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2867e561, SUC_lexicon_tags_2 },
    { 0x4dc7e562, SUC_lexicon_tags_37 },
    { 0x7be7e563, SUC_lexicon_tags_37 },
    { 0xe1c7e564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd127e566, SUC_lexicon_tags_14 },
    { 0xc427e567, SUC_lexicon_tags_17 },
    { 0x49a7e568, SUC_lexicon_tags_60 },
    { 0xb647e568, SUC_lexicon_tags_45 },
    { 0x1687e568, SUC_lexicon_tags_37 },
    { 0x5ce7e56a, SUC_lexicon_tags_49 },
    { 0x78c7e567, SUC_lexicon_tags_15 },
    { 0xbd87e56d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3727e572, SUC_lexicon_tags_6 },
    { 0x5de7e573, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7807e576, SUC_lexicon_tags_11 },
    { 0x7a87e576, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9e7e579, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e47e57e, SUC_lexicon_tags_82 },
    { 0xfb87e57e, SUC_lexicon_tags_5 },
    { 0x2247e580, SUC_lexicon_tags_12 },
    { 0x9a7e580, SUC_lexicon_tags_60 },
    { 0x30a7e582, SUC_lexicon_tags_1 },
    { 0xbe7e583, SUC_lexicon_tags_77 },
    { 0x85c7e584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2847e588, SUC_lexicon_tags_26 },
    { 0x3907e588, SUC_lexicon_tags_49 },
    { 0x2a27e588, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf87e58d, SUC_lexicon_tags_7 },
    { 0x1ea7e58e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd27e590, SUC_lexicon_tags_17 },
    { 0x8067e591, SUC_lexicon_tags_11 },
    { 0x4b07e592, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe7e595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda07e598, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7867e59c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf087e5a0, SUC_lexicon_tags_21 },
    { 0x70c7e5a0, SUC_lexicon_tags_25 },
    { 0xd567e5a0, SUC_lexicon_tags_7 },
    { 0x8dc7e5a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ca7e5a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf547e5a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d67e5ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e07e5ae, SUC_lexicon_tags_37 },
    { 0x5987e5af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7e5b4, SUC_lexicon_tags_39 },
    { 0xa4c7e5b4, SUC_lexicon_tags_47 },
    { 0x9f27e5b6, SUC_lexicon_tags_59 },
    { 0xb467e5b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb067e5b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd707e5bb, SUC_lexicon_tags_27 },
    { 0xd327e5bb, SUC_lexicon_tags_31 },
    { 0x6627e5bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4407e5c0, SUC_lexicon_tags_2 },
    { 0x8327e5c1, SUC_lexicon_tags_28 },
    { 0x1b07e5c2, SUC_lexicon_tags_27 },
    { 0x1967e5c2, SUC_lexicon_tags_12 },
    { 0x3dc7e5c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a67e5cc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4607e5ce, SUC_lexicon_tags_17 },
    { 0x907e5cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x41e7e5d1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xef67e5d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e7e5d6, SUC_lexicon_tags_23 },
    { 0x9927e5d6, SUC_lexicon_tags_16 },
    { 0xf4a7e5d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd287e5da, SUC_lexicon_tags_25 },
    { 0x1847e5da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee7e5e3, SUC_lexicon_tags_49 },
    { 0x5d27e5e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e7e5e7, SUC_lexicon_tags_25 },
    { 0x4e87e5e8, SUC_lexicon_tags_27 },
    { 0x3c27e5e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a7e5ed, SUC_lexicon_tags_66 },
    { 0x7f67e5ee, SUC_lexicon_tags_14 },
    { 0x1687e5ed, SUC_lexicon_tags_15 },
    { 0x65e7e5f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbc07e5f2, SUC_lexicon_tags_11 },
    { 0xc6c7e5f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1347e5f9, SUC_lexicon_tags_11 },
    { 0xe9e7e5fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c7e5fd, SUC_lexicon_tags_15 },
    { 0x9927e5fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5207e604, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a7e60d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6967e60f, SUC_lexicon_tags_1 },
    { 0x9867e610, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf467e612, SUC_lexicon_tags_4 },
    { 0x8ce7e613, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf467e616, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc1a7e618, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1f07e61a, SUC_lexicon_tags_55 },
    { 0x2327e61b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a7e61f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b67e62c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc7e631, SUC_lexicon_tags_37 },
    { 0x57e7e632, SUC_lexicon_tags_7 },
    { 0x15c7e633, SUC_lexicon_tags_63 },
    { 0x7b07e632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd367e63b, SUC_lexicon_tags_56 },
    { 0x49a7e63c, SUC_lexicon_tags_1 },
    { 0x69a7e63d, SUC_lexicon_tags_32 },
    { 0xd2a7e63c, SUC_lexicon_tags_49 },
    { 0xb9a7e63c, SUC_lexicon_tags_33 },
    { 0x5167e63d, SUC_lexicon_tags_37 },
    { 0x3aa7e641, SUC_lexicon_tags_9 },
    { 0x8667e641, SUC_lexicon_tags_15 },
    { 0xb507e63b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e7e648, SUC_lexicon_tags_60 },
    { 0x7147e648, SUC_lexicon_tags_1 },
    { 0x627e64a, SUC_lexicon_tags_17 },
    { 0x67e64b, SUC_lexicon_tags_23 },
    { 0x14a7e64c, SUC_lexicon_tags_44 },
    { 0xdcc7e64d, SUC_lexicon_tags_37 },
    { 0x12e7e64d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa7e651, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4787e653, SUC_lexicon_tags_138 },
    { 0xd07e653, SUC_lexicon_tags_23 },
    { 0x59e7e655, SUC_lexicon_tags_25 },
    { 0x8487e656, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6507e659, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9e7e65b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe247e65d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15e7e65f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c7e663, SUC_lexicon_tags_37 },
    { 0x307e664, SUC_lexicon_tags_49 },
    { 0xcec7e664, SUC_lexicon_tags_17 },
    { 0x7107e663, SUC_lexicon_tags_9 },
    { 0x55e7e667, SUC_lexicon_tags_38 },
    { 0xd3e7e668, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a7e66c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xace7e66e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf207e675, SUC_lexicon_tags_25 },
    { 0x8e87e675, SUC_lexicon_tags_47 },
    { 0xd407e677, SUC_lexicon_tags_10 },
    { 0x9107e676, SUC_lexicon_tags_15 },
    { 0x2fa7e679, SUC_lexicon_tags_28 },
    { 0xfc07e679, SUC_lexicon_tags_58 },
    { 0x7367e67a, SUC_lexicon_tags_5 },
    { 0xdcc7e67c, SUC_lexicon_tags_1 },
    { 0x67c7e67d, SUC_lexicon_tags_17 },
    { 0x8f27e67c, SUC_lexicon_tags_37 },
    { 0xfa7e67e, SUC_lexicon_tags_27 },
    { 0x9747e680, SUC_lexicon_tags_7 },
    { 0x1e7e67e, SUC_lexicon_tags_11 },
    { 0xbd47e67b, SUC_lexicon_tags_30 },
    { 0x2147e677, SUC_lexicon_tags_55 },
    { 0xc7c7e684, SUC_lexicon_tags_2 },
    { 0x2ba7e685, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc47e687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae7e68a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c7e68d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e7e693, SUC_lexicon_tags_37 },
    { 0x1b87e693, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1627e69c, SUC_lexicon_tags_14 },
    { 0x4ec7e69c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x83e7e69f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc887e6a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf567e6a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4687e6aa, SUC_lexicon_tags_7 },
    { 0x2b27e6aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6367e6ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a67e6b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa87e6b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7947e6be, SUC_lexicon_tags_46 },
    { 0xb087e6bf, SUC_lexicon_tags_71 },
    { 0x6467e6c0, SUC_lexicon_tags_11 },
    { 0x8ee7e6c0, SUC_lexicon_tags_7 },
    { 0xd4c7e6be, SUC_lexicon_tags_16 },
    { 0xc387e6c3, SUC_lexicon_tags_7 },
    { 0x99c7e6c4, SUC_lexicon_tags_103 },
    { 0x2c07e6be, SUC_lexicon_tags_38 },
    { 0x8427e6c6, SUC_lexicon_tags_23 },
    { 0xea07e6c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd67e6c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2007e6cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1967e6cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac7e6d1, SUC_lexicon_tags_2 },
    { 0xad27e6d1, SUC_lexicon_tags_17 },
    { 0xe487e6d3, SUC_lexicon_tags_7 },
    { 0x4227e6d3, SUC_lexicon_tags_23 },
    { 0xece7e6d4, SUC_lexicon_tags_23 },
    { 0xc9c7e6d3, SUC_lexicon_tags_7 },
    { 0xb307e6d7, SUC_lexicon_tags_38 },
    { 0x3987e6d7, SUC_lexicon_tags_27 },
    { 0x8dc7e6d7, SUC_lexicon_tags_14 },
    { 0x3207e6da, SUC_lexicon_tags_12 },
    { 0x5887e6d7, SUC_lexicon_tags_17 },
    { 0x8487e6dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa187e6de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5027e6e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da7e6e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdee7e6e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf27e6e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c7e6ed, SUC_lexicon_tags_17 },
    { 0x59c7e6ee, SUC_lexicon_tags_12 },
    { 0x9367e6ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1367e6f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c07e6f7, SUC_lexicon_tags_51 },
    { 0x57c7e6f7, SUC_lexicon_tags_20 },
    { 0xc07e6f9, SUC_lexicon_tags_37 },
    { 0xab07e6fa, SUC_lexicon_tags_14 },
    { 0xd547e6fa, SUC_lexicon_tags_2 },
    { 0x2fc7e6fc, SUC_lexicon_tags_2 },
    { 0xc6e7e6f8, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xb787e6ff, SUC_lexicon_tags_6 },
    { 0x9f67e700, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd507e702, SUC_lexicon_tags_33 },
    { 0xd607e703, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb327e706, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e7e709, SUC_lexicon_tags_37 },
    { 0xc4c7e70a, SUC_lexicon_tags_15 },
    { 0xe207e70a, SUC_lexicon_tags_15 },
    { 0x8d47e70c, SUC_lexicon_tags_10 },
    { 0xbfc7e70d, SUC_lexicon_tags_37 },
    { 0x89c7e70e, SUC_lexicon_tags_7 },
    { 0x7407e70c, SUC_lexicon_tags_14 },
    { 0xbe07e709, SUC_lexicon_tags_15 },
    { 0x187e70e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67e714, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a7e717, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71e7e719, SUC_lexicon_tags_28 },
    { 0x17c7e71a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c07e71d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a7e722, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd367e724, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd707e728, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a7e72b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a7e72f, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x6147e731, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8307e735, SUC_lexicon_tags_2 },
    { 0x2487e735, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ec7e738, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8887e73a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2b67e73c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd427e73e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e7e742, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9827e744, SUC_lexicon_tags_25 },
    { 0x5807e745, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac7e748, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c7e74b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c7e755, SUC_lexicon_tags_17 },
    { 0x35a7e756, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x51a7e758, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafa7e75a, SUC_lexicon_tags_37 },
    { 0x27e75b, SUC_lexicon_tags_25 },
    { 0x2947e75b, SUC_lexicon_tags_48 },
    { 0xd927e75d, SUC_lexicon_tags_84 },
    { 0x9987e75b, SUC_lexicon_tags_14 },
    { 0xe607e75f, SUC_lexicon_tags_63 },
    { 0x3ec7e760, SUC_lexicon_tags_56 },
    { 0x107e760, SUC_lexicon_tags_2 },
    { 0xa27e760, SUC_lexicon_tags_15 },
    { 0x8f07e75f, SUC_lexicon_tags_10 },
    { 0xc1a7e764, SUC_lexicon_tags_0 },
    { 0x7687e764, SUC_lexicon_tags_68 },
    { 0x16c7e766, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9147e768, SUC_lexicon_tags_12 },
    { 0x6f67e769, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe3c7e76b, SUC_lexicon_tags_15 },
    { 0x9147e76c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fe7e76e, SUC_lexicon_tags_11 },
    { 0x88c7e76e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6047e772, SUC_lexicon_tags_7 },
    { 0xcd47e772, SUC_lexicon_tags_33 },
    { 0x5587e772, SUC_lexicon_tags_47 },
    { 0x32e7e775, SUC_lexicon_tags_11 },
    { 0xe847e774, SUC_lexicon_tags_43 },
    { 0x84e7e777, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x8047e77a, SUC_lexicon_tags_39 },
    { 0xe907e77b, SUC_lexicon_tags_31 },
    { 0x7aa7e77c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc647e77f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd547e788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a87e78a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f07e790, SUC_lexicon_tags_17 },
    { 0xbd87e791, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4e7e793, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd487e799, SUC_lexicon_tags_1 },
    { 0x8947e799, SUC_lexicon_tags_19 },
    { 0xaee7e79a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb27e7a4, SUC_lexicon_tags_29 },
    { 0x3767e7a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c7e7a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9187e7ae, SUC_lexicon_tags_38 },
    { 0xc687e7af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb487e7b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a7e7b5, SUC_lexicon_tags_20 },
    { 0xcf67e7b6, SUC_lexicon_tags_28 },
    { 0xee67e7b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3887e7b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8627e7be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5327e7c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf447e7c3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f67e7c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3347e7c8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6b07e7ca, SUC_lexicon_tags_12 },
    { 0xdf07e7ca, SUC_lexicon_tags_45 },
    { 0x2ae7e7cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1647e7d3, SUC_lexicon_tags_25 },
    { 0x2967e7d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc7e7db, SUC_lexicon_tags_37 },
    { 0x93a7e7dc, SUC_lexicon_tags_37 },
    { 0xe2e7e7dc, SUC_lexicon_tags_11 },
    { 0x2947e7de, SUC_lexicon_tags_43 },
    { 0xf987e7de, SUC_lexicon_tags_38 },
    { 0xcc87e7db, SUC_lexicon_tags_38 },
    { 0x9c7e7e1, SUC_lexicon_tags_1 },
    { 0x7107e7e2, SUC_lexicon_tags_6 },
    { 0xfc27e7e2, SUC_lexicon_tags_60 },
    { 0xfd27e7e2, SUC_lexicon_tags_11 },
    { 0x7ac7e7e5, SUC_lexicon_tags_1 },
    { 0x1ca7e7e1, SUC_lexicon_tags_7 },
    { 0x3c07e7e7, SUC_lexicon_tags_47 },
    { 0x9947e7e7, SUC_lexicon_tags_846 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa7e7ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a7e7f7, SUC_lexicon_tags_17 },
    { 0x8787e7f7, SUC_lexicon_tags_37 },
    { 0x9147e7f9, SUC_lexicon_tags_1 },
    { 0x3667e7f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8687e7fe, SUC_lexicon_tags_31 },
    { 0x667e7ff, SUC_lexicon_tags_4 },
    { 0x3a87e7ff, SUC_lexicon_tags_12 },
    { 0x1b07e801, SUC_lexicon_tags_17 },
    { 0xdae7e802, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5247e804, SUC_lexicon_tags_12 },
    { 0x8de7e804, SUC_lexicon_tags_33 },
    { 0x7f67e805, SUC_lexicon_tags_154 },
    { 0xf627e807, SUC_lexicon_tags_9 },
    { 0xec67e808, SUC_lexicon_tags_25 },
    { 0x1147e808, SUC_lexicon_tags_23 },
    { 0x9d87e80a, SUC_lexicon_tags_2 },
    { 0xa187e80b, SUC_lexicon_tags_19 },
    { 0x4aa7e80b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c7e80f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca7e814, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x2687e817, SUC_lexicon_tags_32 },
    { 0x9847e818, SUC_lexicon_tags_43 },
    { 0xd487e819, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x32a7e81b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8487e823, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7e826, SUC_lexicon_tags_15 },
    { 0x74c7e827, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf5a7e829, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7167e82b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc7e82d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb347e82f, SUC_lexicon_tags_55 },
    { 0x12a7e82f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4627e832, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3a07e834, SUC_lexicon_tags_29 },
    { 0x2c67e835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7587e837, SUC_lexicon_tags_37 },
    { 0x947e837, SUC_lexicon_tags_55 },
    { 0xd747e839, SUC_lexicon_tags_29 },
    { 0x4fc7e839, SUC_lexicon_tags_65 },
    { 0x8927e83b, SUC_lexicon_tags_8 },
    { 0x6687e83c, SUC_lexicon_tags_37 },
    { 0x4987e83c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x227e83f, SUC_lexicon_tags_47 },
    { 0x87e840, SUC_lexicon_tags_7 },
    { 0x4de7e841, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9847e843, SUC_lexicon_tags_14 },
    { 0xeb47e843, SUC_lexicon_tags_37 },
    { 0x7f67e845, SUC_lexicon_tags_70 },
    { 0x1447e846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f87e84b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e7e84f, SUC_lexicon_tags_49 },
    { 0x13c7e850, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4747e854, SUC_lexicon_tags_13 },
    { 0x5387e855, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5327e858, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xafa7e85a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c7e85d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e7e862, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe827e865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc47e867, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5407e869, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1027e872, SUC_lexicon_tags_33 },
    { 0x747e873, SUC_lexicon_tags_11 },
    { 0x2a27e874, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb187e876, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7a7e878, SUC_lexicon_tags_31 },
    { 0xf127e879, SUC_lexicon_tags_25 },
    { 0x1d47e87a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc667e87c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7c7e87e, SUC_lexicon_tags_60 },
    { 0xe647e87f, SUC_lexicon_tags_15 },
    { 0x9467e87f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c67e883, SUC_lexicon_tags_15 },
    { 0x3367e884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf247e888, SUC_lexicon_tags_12 },
    { 0x6f87e889, SUC_lexicon_tags_22 },
    { 0x27e889, SUC_lexicon_tags_77 },
    { 0x30c7e888, SUC_lexicon_tags_2 },
    { 0xa3e7e88c, SUC_lexicon_tags_17 },
    { 0x85a7e88c, SUC_lexicon_tags_11 },
    { 0x25e7e88c, SUC_lexicon_tags_10 },
    { 0xe1a7e88c, SUC_lexicon_tags_38 },
    { 0x3507e88f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1847e893, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9087e898, SUC_lexicon_tags_17 },
    { 0x12c7e899, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e27e89b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde47e89e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3387e8a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc867e8a3, SUC_lexicon_tags_33 },
    { 0x7207e8a4, SUC_lexicon_tags_88 },
    { 0x2127e8a3, SUC_lexicon_tags_37 },
    { 0x8007e8a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2807e8ac, SUC_lexicon_tags_141 },
    { 0xf3e7e8ad, SUC_lexicon_tags_79 },
    { 0xde47e8ae, SUC_lexicon_tags_37 },
    { 0x6507e8af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5667e8b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c27e8b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7e8bb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe47e8c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e7e8c7, SUC_lexicon_tags_7 },
    { 0x30a7e8c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee67e8d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45a7e8d3, SUC_lexicon_tags_85 },
    { 0x1a47e8d3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f87e8d7, SUC_lexicon_tags_23 },
    { 0xc487e8d8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a7e8dd, SUC_lexicon_tags_12 },
    { 0xb0a7e8de, SUC_lexicon_tags_7 },
    { 0x8667e8dd, SUC_lexicon_tags_15 },
    { 0x2da7e8de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee7e8ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee7e8ed, SUC_lexicon_tags_21 },
    { 0x8c27e8ee, SUC_lexicon_tags_87 },
    { 0x8fe7e8ef, SUC_lexicon_tags_26 },
    { 0x8787e8f0, SUC_lexicon_tags_47 },
    { 0xebc7e8f1, SUC_lexicon_tags_77 },
    { 0xcfe7e8ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e27e8f6, SUC_lexicon_tags_7 },
    { 0x3707e8f6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba7e8fe, SUC_lexicon_tags_47 },
    { 0x4f67e8ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5a7e901, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a7e904, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca7e907, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4b07e909, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a7e90f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa7e912, SUC_lexicon_tags_37 },
    { 0x22e7e913, SUC_lexicon_tags_9 },
    { 0x9887e914, SUC_lexicon_tags_25 },
    { 0x1527e915, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7307e91a, SUC_lexicon_tags_43 },
    { 0x8847e91a, SUC_lexicon_tags_15 },
    { 0xfa67e91a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e47e91e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a67e920, SUC_lexicon_tags_18 },
    { 0x4027e921, SUC_lexicon_tags_26 },
    { 0xf6a7e920, SUC_lexicon_tags_1 },
    { 0x6de7e921, SUC_lexicon_tags_87 },
    { 0xa407e923, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9387e929, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c07e930, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5187e937, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce7e93c, SUC_lexicon_tags_24 },
    { 0x69a7e93c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7e941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a87e943, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5567e947, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1007e94a, SUC_lexicon_tags_27 },
    { 0xf607e94b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8307e94d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7e950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x947e952, SUC_lexicon_tags_35 },
    { 0x3c07e953, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd167e955, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27e7e957, SUC_lexicon_tags_25 },
    { 0x56c7e958, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27e95b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f67e95d, SUC_lexicon_tags_15 },
    { 0x6ca7e95e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e87e960, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ce7e962, SUC_lexicon_tags_85 },
    { 0xf287e963, SUC_lexicon_tags_27 },
    { 0xfb47e964, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67e969, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba7e96c, SUC_lexicon_tags_4 },
    { 0x9947e96c, SUC_lexicon_tags_32 },
    { 0xb2e7e96c, SUC_lexicon_tags_7 },
    { 0x8987e96e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3c7e971, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de7e974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e7e977, SUC_lexicon_tags_25 },
    { 0x5dc7e978, SUC_lexicon_tags_11 },
    { 0x4d47e979, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73e7e97b, SUC_lexicon_tags_12 },
    { 0x41c7e97c, SUC_lexicon_tags_17 },
    { 0x9607e97d, SUC_lexicon_tags_1 },
    { 0xe1a7e97d, SUC_lexicon_tags_65 },
    { 0x7c07e97f, SUC_lexicon_tags_17 },
    { 0xe987e97e, SUC_lexicon_tags_1 },
    { 0x6467e97f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e7e989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be7e98c, SUC_lexicon_tags_101 },
    { 0xbb87e98c, SUC_lexicon_tags_27 },
    { 0xd087e98e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e7e994, SUC_lexicon_tags_49 },
    { 0x5d27e994, SUC_lexicon_tags_87 },
    { 0x5d07e996, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e99a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa7e99d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa107e9a0, SUC_lexicon_tags_31 },
    { 0xbe67e9a1, SUC_lexicon_tags_129 },
    { 0x6f07e9a0, SUC_lexicon_tags_17 },
    { 0x947e9a3, SUC_lexicon_tags_66 },
    { 0xea07e9a4, SUC_lexicon_tags_11 },
    { 0x6307e9a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1767e9a7, SUC_lexicon_tags_46 },
    { 0x3cc7e9a7, SUC_lexicon_tags_17 },
    { 0x2487e9a8, SUC_lexicon_tags_12 },
    { 0x8607e9a7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b07e9af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd487e9be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa407e9c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7627e9c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c7e9c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c67e9ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7347e9cc, SUC_lexicon_tags_15 },
    { 0x3927e9cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a27e9d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb07e9d3, SUC_lexicon_tags_63 },
    { 0xeee7e9d4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7e9d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f27e9dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5387e9e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe207e9e7, SUC_lexicon_tags_16 },
    { 0x62c7e9e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xefe7e9ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7247e9ed, SUC_lexicon_tags_23 },
    { 0x33a7e9ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a7e9f1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x14e7e9f3, SUC_lexicon_tags_9 },
    { 0x2c67e9f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c27e9f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a07e9fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c7ea02, SUC_lexicon_tags_35 },
    { 0x8bc7ea03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac07ea05, SUC_lexicon_tags_15 },
    { 0x9d87ea05, SUC_lexicon_tags_17 },
    { 0x6aa7ea07, SUC_lexicon_tags_38 },
    { 0x9187ea08, SUC_lexicon_tags_9 },
    { 0x1667ea09, SUC_lexicon_tags_172 },
    { 0x4aa7ea09, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa847ea0d, SUC_lexicon_tags_15 },
    { 0xa0c7ea0e, SUC_lexicon_tags_45 },
    { 0x85a7ea0f, SUC_lexicon_tags_15 },
    { 0x7c67ea10, SUC_lexicon_tags_45 },
    { 0xa1a7ea11, SUC_lexicon_tags_77 },
    { 0xc67ea12, SUC_lexicon_tags_19 },
    { 0xc5c7ea11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7307ea17, SUC_lexicon_tags_46 },
    { 0x467ea17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9fe7ea1a, SUC_lexicon_tags_25 },
    { 0xbb27ea1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d07ea22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e7ea29, SUC_lexicon_tags_16 },
    { 0x64a7ea2a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa187ea30, SUC_lexicon_tags_12 },
    { 0x61a7ea30, SUC_lexicon_tags_1 },
    { 0x88e7ea30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8667ea39, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xad67ea3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c67ea3e, SUC_lexicon_tags_11 },
    { 0x80e7ea3e, SUC_lexicon_tags_7 },
    { 0xa0a7ea40, SUC_lexicon_tags_45 },
    { 0x63a7ea41, SUC_lexicon_tags_14 },
    { 0x5fc7ea42, SUC_lexicon_tags_27 },
    { 0x4187ea43, SUC_lexicon_tags_1 },
    { 0xf5c7ea40, SUC_lexicon_tags_2 },
    { 0xe247ea43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d27ea4c, SUC_lexicon_tags_25 },
    { 0x94c7ea4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa907ea4f, SUC_lexicon_tags_15 },
    { 0x1947ea50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1387ea52, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4827ea55, SUC_lexicon_tags_15 },
    { 0xcae7ea56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7887ea5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc67ea60, SUC_lexicon_tags_15 },
    { 0x52e7ea60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3647ea66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a7ea69, SUC_lexicon_tags_77 },
    { 0xa2e7ea6a, SUC_lexicon_tags_43 },
    { 0xbcc7ea6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb67ea6d, SUC_lexicon_tags_11 },
    { 0x647ea6d, SUC_lexicon_tags_37 },
    { 0x6967ea6f, SUC_lexicon_tags_60 },
    { 0x8a47ea70, SUC_lexicon_tags_11 },
    { 0x43c7ea71, SUC_lexicon_tags_15 },
    { 0xb4a7ea71, SUC_lexicon_tags_7 },
    { 0x80a7ea6f, SUC_lexicon_tags_49 },
    { 0xe1a7ea74, SUC_lexicon_tags_60 },
    { 0x50c7ea75, SUC_lexicon_tags_32 },
    { 0x4e7ea76, SUC_lexicon_tags_27 },
    { 0xa0a7ea77, SUC_lexicon_tags_49 },
    { 0xb627ea77, SUC_lexicon_tags_26 },
    { 0x29e7ea75, SUC_lexicon_tags_12 },
    { 0x8aa7ea78, SUC_lexicon_tags_2 },
    { 0xf107ea7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac27ea83, SUC_lexicon_tags_49 },
    { 0xb67ea83, SUC_lexicon_tags_15 },
    { 0x8a7ea85, SUC_lexicon_tags_594 },
    { 0xa8a7ea84, SUC_lexicon_tags_12 },
    { 0xff47ea87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb887ea8d, SUC_lexicon_tags_74 },
    { 0x8e07ea8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe7ea92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac47ea96, SUC_lexicon_tags_45 },
    { 0x9e07ea97, SUC_lexicon_tags_1 },
    { 0x29c7ea98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c7ea9e, SUC_lexicon_tags_56 },
    { 0x4467ea9f, SUC_lexicon_tags_15 },
    { 0x3de7ea9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a7eaa7, SUC_lexicon_tags_17 },
    { 0xf467eaa7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f47eaaa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf7e7eaac, SUC_lexicon_tags_38 },
    { 0x2467eaad, SUC_lexicon_tags_33 },
    { 0x4087eaac, SUC_lexicon_tags_0 },
    { 0x79a7eaaf, SUC_lexicon_tags_43 },
    { 0x6b87eab0, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0xe867eab3, SUC_lexicon_tags_15 },
    { 0x20c7eab4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a7eab8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b07eabd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a7eac1, SUC_lexicon_tags_19 },
    { 0xfc47eac2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9507eac9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaa47eacb, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xd527eacd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf147eacf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6227ead1, SUC_lexicon_tags_29 },
    { 0xff67ead2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec7ead8, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x93c7eada, SUC_lexicon_tags_25 },
    { 0x7f27eadb, SUC_lexicon_tags_15 },
    { 0x8f47eadb, SUC_lexicon_tags_47 },
    { 0x6ae7eada, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a7eae0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe487eae7, SUC_lexicon_tags_27 },
    { 0xb8c7eae8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e7eaf0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c7eaf3, SUC_lexicon_tags_49 },
    { 0xf0c7eaf4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee7eaf7, SUC_lexicon_tags_7 },
    { 0x9927eaf7, SUC_lexicon_tags_15 },
    { 0xc3e7eaf9, SUC_lexicon_tags_25 },
    { 0xc1e7eaf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e7eb00, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x3a47eb02, SUC_lexicon_tags_14 },
    { 0x7087eb02, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3987eb09, SUC_lexicon_tags_15 },
    { 0x6d27eb0a, SUC_lexicon_tags_15 },
    { 0xa1c7eb0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc767eb0e, SUC_lexicon_tags_9 },
    { 0xa8e7eb0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7807eb14, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xef07eb16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xca67eb18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07eb20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6467eb25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e47eb30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76e7eb32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c07eb3f, SUC_lexicon_tags_15 },
    { 0x8647eb40, SUC_lexicon_tags_1 },
    { 0x3647eb40, SUC_lexicon_tags_7 },
    { 0x7ee7eb3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c47eb44, SUC_lexicon_tags_37 },
    { 0x48c7eb44, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5ae7eb47, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c7eb4b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfa07eb4d, SUC_lexicon_tags_11 },
    { 0x56e7eb4e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe347eb50, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa487eb52, SUC_lexicon_tags_63 },
    { 0x2dc7eb52, SUC_lexicon_tags_17 },
    { 0x47e7eb54, SUC_lexicon_tags_17 },
    { 0xe4a7eb55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ac7eb57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c7eb5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa27eb61, SUC_lexicon_tags_59 },
    { 0xdfc7eb62, SUC_lexicon_tags_85 },
    { 0x7387eb63, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7207eb65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a27eb6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e27eb6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd47eb73, SUC_lexicon_tags_37 },
    { 0xc667eb74, SUC_lexicon_tags_9 },
    { 0x59e7eb74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4827eb77, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc4a7eb79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7507eb7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca67eb7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf847eb7f, SUC_lexicon_tags_8 },
    { 0xdfe7eb7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a67eb82, SUC_lexicon_tags_0 },
    { 0xee67eb83, SUC_lexicon_tags_13 },
    { 0xb527eb84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdc67eb86, SUC_lexicon_tags_37 },
    { 0x49c7eb87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4027eb8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7eb8e, SUC_lexicon_tags_47 },
    { 0x5e87eb8f, SUC_lexicon_tags_47 },
    { 0x5727eb8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ae7eb92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1527eb94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b27eb97, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x5427eb9a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3067eb9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb07eb9e, SUC_lexicon_tags_7 },
    { 0x17a7eb9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a7eba6, SUC_lexicon_tags_11 },
    { 0xe367eba7, SUC_lexicon_tags_19 },
    { 0x9727eba8, SUC_lexicon_tags_60 },
    { 0xc967eba8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x907ebab, SUC_lexicon_tags_31 },
    { 0x2f67ebab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf927ebae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3707ebb7, SUC_lexicon_tags_11 },
    { 0x9dc7ebb8, SUC_lexicon_tags_43 },
    { 0x9487ebb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5967ebbb, SUC_lexicon_tags_7 },
    { 0x2707ebbb, SUC_lexicon_tags_46 },
    { 0x1e87ebbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe047ebc0, SUC_lexicon_tags_17 },
    { 0x667ebc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73c7ebc3, SUC_lexicon_tags_0 },
    { 0xa9e7ebc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c47ebc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d47ebc9, SUC_lexicon_tags_19 },
    { 0xbb07ebca, SUC_lexicon_tags_8 },
    { 0x7be7ebcb, SUC_lexicon_tags_8 },
    { 0xf07ebca, SUC_lexicon_tags_4 },
    { 0xa2c7ebcc, SUC_lexicon_tags_17 },
    { 0x367ebcb, SUC_lexicon_tags_46 },
    { 0x9367ebcf, SUC_lexicon_tags_29 },
    { 0xfdc7ebcf, SUC_lexicon_tags_37 },
    { 0x9307ebcf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab47ebd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd307ebd9, SUC_lexicon_tags_63 },
    { 0x5b07ebd9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb087ebde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9407ebe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9087ebe6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec7ebe9, SUC_lexicon_tags_87 },
    { 0x5847ebea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27ebed, SUC_lexicon_tags_38 },
    { 0x15a7ebed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3687ebf1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7fc7ebf3, SUC_lexicon_tags_37 },
    { 0x767ebf4, SUC_lexicon_tags_14 },
    { 0x4667ebf5, SUC_lexicon_tags_44 },
    { 0xb167ebf3, SUC_lexicon_tags_54 },
    { 0x17e7ebf4, SUC_lexicon_tags_33 },
    { 0xc5c7ebf8, SUC_lexicon_tags_21 },
    { 0x9f87ebf8, SUC_lexicon_tags_106 },
    { 0x267ebf6, SUC_lexicon_tags_19 },
    { 0xc7c7ebf7, SUC_lexicon_tags_63 },
    { 0xd987ebf7, SUC_lexicon_tags_600 },
    { 0xb6a7ebfd, SUC_lexicon_tags_15 },
    { 0x1127ebfc, SUC_lexicon_tags_1 },
    { 0xcb07ebf3, SUC_lexicon_tags_47 },
    { 0x3ba7ec00, SUC_lexicon_tags_11 },
    { 0xf4e7ec01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x13a7ec03, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf527ec09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee7ec0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e7ec11, SUC_lexicon_tags_12 },
    { 0xbc7ec12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b67ec14, SUC_lexicon_tags_67 },
    { 0xb127ec14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8767ec18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e7ec1e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7667ec23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa827ec26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf587ec28, SUC_lexicon_tags_1 },
    { 0x16a7ec29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde07ec2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54e7ec2d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e7ec32, SUC_lexicon_tags_47 },
    { 0x31a7ec33, SUC_lexicon_tags_7 },
    { 0x34a7ec33, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc647ec37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc707ec3e, SUC_lexicon_tags_55 },
    { 0x6ea7ec3e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece7ec45, SUC_lexicon_tags_10 },
    { 0x6927ec45, SUC_lexicon_tags_12 },
    { 0x3987ec47, SUC_lexicon_tags_12 },
    { 0x96c7ec46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa447ec4f, SUC_lexicon_tags_12 },
    { 0xd367ec4f, SUC_lexicon_tags_0 },
    { 0x8ec7ec4f, SUC_lexicon_tags_25 },
    { 0x7b27ec50, SUC_lexicon_tags_12 },
    { 0xc427ec50, SUC_lexicon_tags_43 },
    { 0x687ec50, SUC_lexicon_tags_9 },
    { 0x5f07ec51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8567ec58, SUC_lexicon_tags_23 },
    { 0x1327ec59, SUC_lexicon_tags_26 },
    { 0x9b87ec5a, SUC_lexicon_tags_14 },
    { 0x8ba7ec5b, SUC_lexicon_tags_77 },
    { 0x5aa7ec5c, SUC_lexicon_tags_25 },
    { 0xf827ec5d, SUC_lexicon_tags_17 },
    { 0xda47ec5e, SUC_lexicon_tags_14 },
    { 0x6447ec5f, SUC_lexicon_tags_45 },
    { 0x5127ec60, SUC_lexicon_tags_7 },
    { 0x26c7ec61, SUC_lexicon_tags_19 },
    { 0xd787ec60, SUC_lexicon_tags_65 },
    { 0x59a7ec61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d47ec6b, SUC_lexicon_tags_23 },
    { 0xf0e7ec6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c7ec74, SUC_lexicon_tags_25 },
    { 0x1667ec74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe67ec7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67ec81, SUC_lexicon_tags_1 },
    { 0x31a7ec82, SUC_lexicon_tags_85 },
    { 0xa567ec83, SUC_lexicon_tags_15 },
    { 0xa387ec84, SUC_lexicon_tags_26 },
    { 0x11a7ec84, SUC_lexicon_tags_42 },
    { 0x7aa7ec82, SUC_lexicon_tags_31 },
    { 0x7ca7ec87, SUC_lexicon_tags_27 },
    { 0x7c67ec87, SUC_lexicon_tags_11 },
    { 0x1027ec89, SUC_lexicon_tags_12 },
    { 0x9b27ec8a, SUC_lexicon_tags_17 },
    { 0xcc67ec8b, SUC_lexicon_tags_7 },
    { 0x367ec8c, SUC_lexicon_tags_17 },
    { 0xf87ec8b, SUC_lexicon_tags_11 },
    { 0x75c7ec8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaa27ec90, SUC_lexicon_tags_16 },
    { 0x8587ec90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5967ec99, SUC_lexicon_tags_17 },
    { 0x3c7ec99, SUC_lexicon_tags_12 },
    { 0x66c7ec9b, SUC_lexicon_tags_5 },
    { 0x98e7ec9c, SUC_lexicon_tags_23 },
    { 0xaa07ec9d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac7eca1, SUC_lexicon_tags_17 },
    { 0x9aa7eca2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07eca5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc987eca7, SUC_lexicon_tags_30 },
    { 0xa827eca7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e7ecab, SUC_lexicon_tags_33 },
    { 0x1f07ecac, SUC_lexicon_tags_60 },
    { 0x7e87ecad, SUC_lexicon_tags_113 },
    { 0xce87ecac, SUC_lexicon_tags_56 },
    { 0xae47ecac, SUC_lexicon_tags_26 },
    { 0x2787ecb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd27ecb4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9fe7ecb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7ecbe, SUC_lexicon_tags_10 },
    { 0x4687ecbe, SUC_lexicon_tags_17 },
    { 0x2ee7ecc0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7447ecc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fa7ecc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c7eccf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce47ecd5, SUC_lexicon_tags_17 },
    { 0x22a7ecd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18e7ecd8, SUC_lexicon_tags_1 },
    { 0xdbc7ecd9, SUC_lexicon_tags_13 },
    { 0xe07ecd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3847ece0, SUC_lexicon_tags_3 },
    { 0x8607ece1, SUC_lexicon_tags_11 },
    { 0x7567ece0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe67ece5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca07ece8, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba7ecee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa7ecf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f67ecfd, SUC_lexicon_tags_74 },
    { 0x4567ecfd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8a7ed00, SUC_lexicon_tags_93 },
    { 0x387ed00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c67ed04, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb047ed07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e7ed0c, SUC_lexicon_tags_12 },
    { 0xc5c7ed0c, SUC_lexicon_tags_12 },
    { 0x1987ed0d, SUC_lexicon_tags_91 },
    { 0xe547ed0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9e7ed11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5a7ed13, SUC_lexicon_tags_44 },
    { 0x3267ed14, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe87ed1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3407ed22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb47ed24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3907ed26, SUC_lexicon_tags_15 },
    { 0xc5e7ed27, SUC_lexicon_tags_7 },
    { 0xcf87ed28, SUC_lexicon_tags_31 },
    { 0x8267ed29, SUC_lexicon_tags_30 },
    { 0xfc07ed2a, SUC_lexicon_tags_17 },
    { 0x4e47ed2a, SUC_lexicon_tags_126 },
    { 0x4607ed2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeac7ed2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed47ed36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb127ed38, SUC_lexicon_tags_26 },
    { 0xb747ed38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e7ed3d, SUC_lexicon_tags_23 },
    { 0xbd07ed3d, SUC_lexicon_tags_48 },
    { 0x4b07ed3e, SUC_lexicon_tags_14 },
    { 0x4867ed3e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87ed43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaa47ed45, SUC_lexicon_tags_1 },
    { 0xb367ed46, SUC_lexicon_tags_30 },
    { 0x3167ed45, SUC_lexicon_tags_238 },
    { 0x6c27ed48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75e7ed4a, SUC_lexicon_tags_84 },
    { 0x9e7ed4a, SUC_lexicon_tags_14 },
    { 0xf9c7ed4c, SUC_lexicon_tags_33 },
    { 0xe407ed4d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5087ed50, SUC_lexicon_tags_39 },
    { 0x1aa7ed50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe7ed56, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9687ed5e, SUC_lexicon_tags_23 },
    { 0x7227ed5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa007ed62, SUC_lexicon_tags_25 },
    { 0x70e7ed63, SUC_lexicon_tags_37 },
    { 0xe827ed64, SUC_lexicon_tags_2 },
    { 0xa927ed65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7927ed6e, SUC_lexicon_tags_6 },
    { 0xca47ed6f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a7ed72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb347ed78, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc407ed7c, SUC_lexicon_tags_12 },
    { 0x9287ed7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd07ed80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf107ed82, SUC_lexicon_tags_53 },
    { 0x3187ed83, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3947ed87, SUC_lexicon_tags_37 },
    { 0xd087ed88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa767ed8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a7ed8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b47ed91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec7ed94, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c7ed97, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c27ed9a, SUC_lexicon_tags_63 },
    { 0x6407ed9b, SUC_lexicon_tags_27 },
    { 0x6ba7ed9c, SUC_lexicon_tags_38 },
    { 0x3f47ed9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe647eda0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1327eda6, SUC_lexicon_tags_26 },
    { 0x3007eda7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3347edaa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf067edae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d67edb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd267edb3, SUC_lexicon_tags_15 },
    { 0x8487edb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1947edb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a67edba, SUC_lexicon_tags_49 },
    { 0x1a87edba, SUC_lexicon_tags_12 },
    { 0x7187edba, SUC_lexicon_tags_69 },
    { 0x63c7edbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c7edc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a87edc2, SUC_lexicon_tags_0 },
    { 0xe87edc3, SUC_lexicon_tags_7 },
    { 0x46e7edc2, SUC_lexicon_tags_12 },
    { 0x96a7edc5, SUC_lexicon_tags_49 },
    { 0x2607edc6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf687edc8, SUC_lexicon_tags_16 },
    { 0x1ca7edc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1827edcb, SUC_lexicon_tags_43 },
    { 0x78c7edcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc827edce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6847edd1, SUC_lexicon_tags_47 },
    { 0xa727edd1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5807edd7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e7ede0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5487ede7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2547edec, SUC_lexicon_tags_15 },
    { 0xb287edec, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6527edf1, SUC_lexicon_tags_37 },
    { 0x94c7edf2, SUC_lexicon_tags_7 },
    { 0x9cc7edf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c7edf6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd347edf8, SUC_lexicon_tags_1 },
    { 0x7b67edf8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade7edff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e7ee06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49c7ee08, SUC_lexicon_tags_15 },
    { 0x73a7ee09, SUC_lexicon_tags_29 },
    { 0x60e7ee0a, SUC_lexicon_tags_37 },
    { 0x4c67ee0a, SUC_lexicon_tags_1 },
    { 0x3fc7ee0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a7ee11, SUC_lexicon_tags_25 },
    { 0x5167ee12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54e7ee14, SUC_lexicon_tags_13 },
    { 0x7d27ee15, SUC_lexicon_tags_2 },
    { 0xbb27ee15, SUC_lexicon_tags_25 },
    { 0x8967ee17, SUC_lexicon_tags_25 },
    { 0x8827ee18, SUC_lexicon_tags_17 },
    { 0x2487ee16, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8067ee1c, SUC_lexicon_tags_37 },
    { 0xb207ee1d, SUC_lexicon_tags_23 },
    { 0xe7c7ee1d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a7ee21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7ee24, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xff47ee26, SUC_lexicon_tags_14 },
    { 0xe5a7ee27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe707ee29, SUC_lexicon_tags_70 },
    { 0x25c7ee29, SUC_lexicon_tags_1 },
    { 0x74c7ee2b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc127ee2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ae7ee2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6467ee32, SUC_lexicon_tags_60 },
    { 0x9047ee32, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec47ee39, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a7ee3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc7ee41, SUC_lexicon_tags_15 },
    { 0x7b27ee42, SUC_lexicon_tags_27 },
    { 0xe267ee42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627ee47, SUC_lexicon_tags_39 },
    { 0xa047ee48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc67ee4b, SUC_lexicon_tags_11 },
    { 0xa0c7ee4c, SUC_lexicon_tags_6 },
    { 0x7d67ee4c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe7ee52, SUC_lexicon_tags_10 },
    { 0xaa07ee52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a7ee58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a7ee5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5047ee61, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd27ee63, SUC_lexicon_tags_7 },
    { 0xd667ee64, SUC_lexicon_tags_0 },
    { 0x9027ee65, SUC_lexicon_tags_13 },
    { 0x3507ee65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c7ee6c, SUC_lexicon_tags_37 },
    { 0xb1e7ee6d, SUC_lexicon_tags_25 },
    { 0x9a7ee6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7167ee70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb27ee76, SUC_lexicon_tags_0 },
    { 0x8a87ee76, SUC_lexicon_tags_2 },
    { 0x2147ee78, SUC_lexicon_tags_12 },
    { 0x987ee78, SUC_lexicon_tags_1 },
    { 0x2be7ee7a, SUC_lexicon_tags_31 },
    { 0x4527ee7b, SUC_lexicon_tags_37 },
    { 0x2427ee78, SUC_lexicon_tags_6 },
    { 0x8187ee7d, SUC_lexicon_tags_32 },
    { 0xbb07ee7e, SUC_lexicon_tags_15 },
    { 0xc1a7ee7c, SUC_lexicon_tags_19 },
    { 0xa2c7ee7f, SUC_lexicon_tags_12 },
    { 0xfe47ee7d, SUC_lexicon_tags_29 },
    { 0x9527ee7b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb87ee86, SUC_lexicon_tags_6 },
    { 0x3c27ee87, SUC_lexicon_tags_11 },
    { 0xf887ee87, SUC_lexicon_tags_47 },
    { 0x1407ee89, SUC_lexicon_tags_14 },
    { 0xf3e7ee8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b07ee8c, SUC_lexicon_tags_7 },
    { 0x2ea7ee8c, SUC_lexicon_tags_25 },
    { 0xbfe7ee8e, SUC_lexicon_tags_11 },
    { 0x2027ee8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9867ee94, SUC_lexicon_tags_84 },
    { 0x8b27ee95, SUC_lexicon_tags_16 },
    { 0xe3a7ee96, SUC_lexicon_tags_37 },
    { 0x71e7ee97, SUC_lexicon_tags_37 },
    { 0x18e7ee94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc527ee9a, SUC_lexicon_tags_25 },
    { 0xae27ee9a, SUC_lexicon_tags_38 },
    { 0x6f87ee9b, SUC_lexicon_tags_47 },
    { 0xd407ee9d, SUC_lexicon_tags_16 },
    { 0x2167ee9d, SUC_lexicon_tags_92 },
    { 0xf6c7ee9c, SUC_lexicon_tags_38 },
    { 0xe407eea0, SUC_lexicon_tags_2 },
    { 0x10a7eea1, SUC_lexicon_tags_28 },
    { 0xd327eea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1747eea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad47eeaa, SUC_lexicon_tags_19 },
    { 0xb367eeab, SUC_lexicon_tags_25 },
    { 0xe267eeab, SUC_lexicon_tags_49 },
    { 0x2ae7eeac, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x27eeb0, SUC_lexicon_tags_17 },
    { 0x5da7eeb0, SUC_lexicon_tags_11 },
    { 0x68a7eeb2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae7eeb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96c7eeb7, SUC_lexicon_tags_39 },
    { 0xbc7eeb8, SUC_lexicon_tags_10 },
    { 0xdc27eeb9, SUC_lexicon_tags_60 },
    { 0x7847eeba, SUC_lexicon_tags_9 },
    { 0x2b47eeba, SUC_lexicon_tags_23 },
    { 0x8427eebc, SUC_lexicon_tags_48 },
    { 0xabc7eebd, SUC_lexicon_tags_7 },
    { 0x227eeb9, SUC_lexicon_tags_11 },
    { 0x7eebb, SUC_lexicon_tags_140 },
    { 0x1267eec0, SUC_lexicon_tags_47 },
    { 0xb387eec0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3287eec8, SUC_lexicon_tags_27 },
    { 0x8d07eec9, SUC_lexicon_tags_148 },
    { 0x2327eeca, SUC_lexicon_tags_1 },
    { 0xe787eecb, SUC_lexicon_tags_4 },
    { 0x1fc7eec9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc967eece, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c7eed4, SUC_lexicon_tags_12 },
    { 0x6ce7eed5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc3c7eed7, SUC_lexicon_tags_37 },
    { 0x76e7eed8, SUC_lexicon_tags_60 },
    { 0xece7eed7, SUC_lexicon_tags_34 },
    { 0xb607eed7, SUC_lexicon_tags_33 },
    { 0xfb67eedb, SUC_lexicon_tags_11 },
    { 0xa4a7eedb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca7eee1, SUC_lexicon_tags_31 },
    { 0x8607eee1, SUC_lexicon_tags_7 },
    { 0x6f27eee3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf87eee7, SUC_lexicon_tags_1 },
    { 0x7ce7eee8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf867eeea, SUC_lexicon_tags_9 },
    { 0x45e7eeea, SUC_lexicon_tags_14 },
    { 0xa727eeec, SUC_lexicon_tags_37 },
    { 0x5127eeed, SUC_lexicon_tags_28 },
    { 0x25a7eeec, SUC_lexicon_tags_26 },
    { 0x7167eeef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a7eef4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1467eef7, SUC_lexicon_tags_41 },
    { 0x3447eef7, SUC_lexicon_tags_11 },
    { 0x367eef9, SUC_lexicon_tags_37 },
    { 0xda87eefa, SUC_lexicon_tags_7 },
    { 0x7387eefa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4247eeff, SUC_lexicon_tags_37 },
    { 0x9687ef00, SUC_lexicon_tags_7 },
    { 0x76e7ef01, SUC_lexicon_tags_25 },
    { 0x25c7ef02, SUC_lexicon_tags_43 },
    { 0x3927ef03, SUC_lexicon_tags_32 },
    { 0xcba7ef01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc727ef06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8e7ef08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a7ef0d, SUC_lexicon_tags_17 },
    { 0x8427ef0e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xa767ef11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7e7ef13, SUC_lexicon_tags_17 },
    { 0x2c7ef14, SUC_lexicon_tags_11 },
    { 0x5227ef13, SUC_lexicon_tags_25 },
    { 0xa4e7ef13, SUC_lexicon_tags_14 },
    { 0x7687ef17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b67ef19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a7ef21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4487ef2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc87ef30, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e7ef34, SUC_lexicon_tags_48 },
    { 0xe87ef35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b87ef40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a07ef44, SUC_lexicon_tags_1 },
    { 0x74c7ef44, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x84a7ef47, SUC_lexicon_tags_19 },
    { 0x3ec7ef47, SUC_lexicon_tags_21 },
    { 0x2107ef49, SUC_lexicon_tags_17 },
    { 0xc067ef4a, SUC_lexicon_tags_37 },
    { 0x8ae7ef4a, SUC_lexicon_tags_15 },
    { 0x6e27ef47, SUC_lexicon_tags_30 },
    { 0x7dc7ef4d, SUC_lexicon_tags_1 },
    { 0x4a27ef48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee7ef53, SUC_lexicon_tags_4 },
    { 0xa7ef54, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc667ef56, SUC_lexicon_tags_11 },
    { 0x2667ef57, SUC_lexicon_tags_8 },
    { 0xf9e7ef56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e7ef60, SUC_lexicon_tags_48 },
    { 0xc527ef61, SUC_lexicon_tags_62 },
    { 0xffe7ef60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf687ef67, SUC_lexicon_tags_7 },
    { 0x1e27ef68, SUC_lexicon_tags_53 },
    { 0xa947ef69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b67ef6b, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467ef6f, SUC_lexicon_tags_92 },
    { 0xb167ef70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7ef7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7747ef85, SUC_lexicon_tags_55 },
    { 0xf6c7ef86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6927ef8a, SUC_lexicon_tags_2 },
    { 0xa367ef8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc227ef90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f87ef94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe947ef99, SUC_lexicon_tags_16 },
    { 0x5a7ef9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd87efa5, SUC_lexicon_tags_38 },
    { 0xa3e7efa6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd567efa9, SUC_lexicon_tags_11 },
    { 0xc7efaa, SUC_lexicon_tags_1 },
    { 0x1c7efa9, SUC_lexicon_tags_70 },
    { 0x3d87efac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7dc7efae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee7efb1, SUC_lexicon_tags_25 },
    { 0x9847efb2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae67efb9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7efc0, SUC_lexicon_tags_37 },
    { 0x7947efc1, SUC_lexicon_tags_198 },
    { 0x8fa7efc2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a7efc5, SUC_lexicon_tags_8 },
    { 0x4687efc5, SUC_lexicon_tags_7 },
    { 0x7067efc7, SUC_lexicon_tags_37 },
    { 0x1bc7efc7, SUC_lexicon_tags_45 },
    { 0xdd87efc6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4d07efcb, SUC_lexicon_tags_35 },
    { 0x7b87efcc, SUC_lexicon_tags_9 },
    { 0x77e7efcb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf07efd4, SUC_lexicon_tags_37 },
    { 0x97c7efd5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf547efd7, SUC_lexicon_tags_282 },
    { 0x6807efd8, SUC_lexicon_tags_45 },
    { 0xd6c7efd9, SUC_lexicon_tags_15 },
    { 0xa9a7efd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa767efdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e7efdf, SUC_lexicon_tags_68 },
    { 0x6867efe0, SUC_lexicon_tags_17 },
    { 0x4527efdf, SUC_lexicon_tags_1 },
    { 0x2727efe2, SUC_lexicon_tags_1 },
    { 0x4fc7efe2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4287efe9, SUC_lexicon_tags_0 },
    { 0xe787efe9, SUC_lexicon_tags_1 },
    { 0x4607efeb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3787efed, SUC_lexicon_tags_7 },
    { 0x3227efee, SUC_lexicon_tags_37 },
    { 0xf227efef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a07eff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd87eff5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b47effa, SUC_lexicon_tags_7 },
    { 0xee7effb, SUC_lexicon_tags_2 },
    { 0xb267effc, SUC_lexicon_tags_4 },
    { 0xb407effb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a47f002, SUC_lexicon_tags_17 },
    { 0xcea7f003, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47f006, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd027f009, SUC_lexicon_tags_11 },
    { 0x2707f00a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7f00f, SUC_lexicon_tags_2 },
    { 0xb1a7f010, SUC_lexicon_tags_30 },
    { 0x9e67f011, SUC_lexicon_tags_1 },
    { 0x5bc7f012, SUC_lexicon_tags_60 },
    { 0xcae7f00f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe1a7f015, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x967f018, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaae7f01a, SUC_lexicon_tags_520 },
    { 0, NULL },
    { 0xb887f01c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1367f020, SUC_lexicon_tags_25 },
    { 0x7b87f021, SUC_lexicon_tags_2 },
    { 0x4e27f021, SUC_lexicon_tags_64 },
    { 0x62e7f021, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad07f025, SUC_lexicon_tags_17 },
    { 0xdb27f026, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8e7f028, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8a7f02a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9147f02c, SUC_lexicon_tags_30 },
    { 0xd927f02d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc707f02f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8287f034, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c7f038, SUC_lexicon_tags_37 },
    { 0x5a67f039, SUC_lexicon_tags_87 },
    { 0x24e7f039, SUC_lexicon_tags_179 },
    { 0x1347f03a, SUC_lexicon_tags_7 },
    { 0x1247f03c, SUC_lexicon_tags_29 },
    { 0x1107f03d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22c7f03f, SUC_lexicon_tags_6 },
    { 0xaee7f040, SUC_lexicon_tags_19 },
    { 0x4127f041, SUC_lexicon_tags_2 },
    { 0xb767f042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ec7f044, SUC_lexicon_tags_25 },
    { 0x9a07f045, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ce7f047, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfe7f049, SUC_lexicon_tags_11 },
    { 0xcfe7f04a, SUC_lexicon_tags_11 },
    { 0x8747f04b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7987f04d, SUC_lexicon_tags_11 },
    { 0x1767f04d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c7f052, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e27f058, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f87f05c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e7f062, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4107f065, SUC_lexicon_tags_27 },
    { 0x5347f065, SUC_lexicon_tags_37 },
    { 0x347f067, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa907f06a, SUC_lexicon_tags_2 },
    { 0x3da7f06b, SUC_lexicon_tags_17 },
    { 0xbee7f06b, SUC_lexicon_tags_7 },
    { 0x2107f06d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7f070, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4327f074, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a07f079, SUC_lexicon_tags_11 },
    { 0xe067f079, SUC_lexicon_tags_31 },
    { 0xfda7f07b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6967f07d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe2c7f07f, SUC_lexicon_tags_15 },
    { 0x81e7f080, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa7f088, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe047f08a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd507f08c, SUC_lexicon_tags_66 },
    { 0x5e67f08d, SUC_lexicon_tags_68 },
    { 0x1ac7f08d, SUC_lexicon_tags_33 },
    { 0xd0e7f08c, SUC_lexicon_tags_31 },
    { 0xc787f08f, SUC_lexicon_tags_25 },
    { 0xccc7f091, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe567f096, SUC_lexicon_tags_17 },
    { 0x1fe7f096, SUC_lexicon_tags_12 },
    { 0x5b67f098, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf07f09a, SUC_lexicon_tags_3 },
    { 0x5227f09b, SUC_lexicon_tags_12 },
    { 0xda7f09a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6547f0a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8607f0a6, SUC_lexicon_tags_7 },
    { 0xf067f0a7, SUC_lexicon_tags_21 },
    { 0x6d47f0a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa267f0af, SUC_lexicon_tags_37 },
    { 0xec47f0b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c7f0b5, SUC_lexicon_tags_1 },
    { 0x6767f0b6, SUC_lexicon_tags_11 },
    { 0xfa67f0b6, SUC_lexicon_tags_56 },
    { 0xa2a7f0b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9267f0bc, SUC_lexicon_tags_12 },
    { 0xd8c7f0bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7847f0c0, SUC_lexicon_tags_31 },
    { 0xa707f0c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e87f0d1, SUC_lexicon_tags_15 },
    { 0x8687f0d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9607f0d4, SUC_lexicon_tags_4 },
    { 0xa147f0d5, SUC_lexicon_tags_17 },
    { 0xb6c7f0d6, SUC_lexicon_tags_34 },
    { 0x8a7f0d5, SUC_lexicon_tags_60 },
    { 0x38e7f0d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa707f0dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5e47f0de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a7f0e1, SUC_lexicon_tags_49 },
    { 0x2da7f0e2, SUC_lexicon_tags_43 },
    { 0xb4e7f0e1, SUC_lexicon_tags_48 },
    { 0xb1e7f0e1, SUC_lexicon_tags_14 },
    { 0x8ec7f0e4, SUC_lexicon_tags_14 },
    { 0x5fe7f0e6, SUC_lexicon_tags_16 },
    { 0x8e7f0e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7787f0e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f27f0ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c7f0f5, SUC_lexicon_tags_15 },
    { 0x7b07f0f5, SUC_lexicon_tags_2 },
    { 0x81e7f0f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d87f0fc, SUC_lexicon_tags_0 },
    { 0xb6a7f0fd, SUC_lexicon_tags_15 },
    { 0x3fa7f0fc, SUC_lexicon_tags_43 },
    { 0x4847f0ff, SUC_lexicon_tags_15 },
    { 0x2ba7f100, SUC_lexicon_tags_63 },
    { 0x8ec7f101, SUC_lexicon_tags_2 },
    { 0xae47f101, SUC_lexicon_tags_49 },
    { 0x21a7f103, SUC_lexicon_tags_27 },
    { 0x6487f104, SUC_lexicon_tags_12 },
    { 0x2b27f105, SUC_lexicon_tags_45 },
    { 0x527f106, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb827f10a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb727f116, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x18e7f118, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x71a7f11a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb067f120, SUC_lexicon_tags_31 },
    { 0x2787f121, SUC_lexicon_tags_26 },
    { 0x41c7f122, SUC_lexicon_tags_28 },
    { 0x1de7f123, SUC_lexicon_tags_8 },
    { 0x8027f120, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc307f126, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc7f12c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e07f12f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xed07f132, SUC_lexicon_tags_11 },
    { 0x3747f133, SUC_lexicon_tags_37 },
    { 0x8c7f133, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3867f136, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc847f139, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a87f142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c7f149, SUC_lexicon_tags_14 },
    { 0x6ca7f14a, SUC_lexicon_tags_47 },
    { 0xe887f14b, SUC_lexicon_tags_44 },
    { 0x6f27f14b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee7f151, SUC_lexicon_tags_1 },
    { 0x9c7f152, SUC_lexicon_tags_37 },
    { 0xc647f151, SUC_lexicon_tags_15 },
    { 0xb307f154, SUC_lexicon_tags_289 },
    { 0x7667f151, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa7f159, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba7f15e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a47f161, SUC_lexicon_tags_27 },
    { 0x1ac7f162, SUC_lexicon_tags_2 },
    { 0x6e47f161, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f47f168, SUC_lexicon_tags_27 },
    { 0xb0a7f169, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e67f16d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e7f170, SUC_lexicon_tags_4 },
    { 0x947f171, SUC_lexicon_tags_12 },
    { 0xc87f171, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x5627f175, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec7f178, SUC_lexicon_tags_34 },
    { 0xd467f179, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd967f17b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82e7f17d, SUC_lexicon_tags_14 },
    { 0xe1e7f17e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c7f183, SUC_lexicon_tags_7 },
    { 0xa6a7f184, SUC_lexicon_tags_1 },
    { 0x65e7f185, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3827f18a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e7f18e, SUC_lexicon_tags_2 },
    { 0x4c07f18f, SUC_lexicon_tags_37 },
    { 0x5867f18f, SUC_lexicon_tags_21 },
    { 0x6ec7f191, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda7f196, SUC_lexicon_tags_23 },
    { 0x7b07f197, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c7f19c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e47f1a3, SUC_lexicon_tags_60 },
    { 0xe927f1a4, SUC_lexicon_tags_34 },
    { 0x2367f1a5, SUC_lexicon_tags_68 },
    { 0x2a27f1a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a7f1a9, SUC_lexicon_tags_11 },
    { 0xf0e7f1a9, SUC_lexicon_tags_45 },
    { 0x90e7f1a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a7f1ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2947f1b0, SUC_lexicon_tags_6 },
    { 0x8c87f1b1, SUC_lexicon_tags_37 },
    { 0x9a7f1b0, SUC_lexicon_tags_2 },
    { 0x4927f1b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea67f1ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccc7f1bc, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe627f1c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca67f1c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a07f1ca, SUC_lexicon_tags_25 },
    { 0xf6e7f1cb, SUC_lexicon_tags_124 },
    { 0x8527f1cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab87f1d2, SUC_lexicon_tags_27 },
    { 0x6a07f1d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbbc7f1d5, SUC_lexicon_tags_37 },
    { 0x5bc7f1d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f67f1d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7387f1db, SUC_lexicon_tags_49 },
    { 0xea67f1db, SUC_lexicon_tags_7 },
    { 0x9c47f1dd, SUC_lexicon_tags_11 },
    { 0x7e67f1db, SUC_lexicon_tags_12 },
    { 0x1727f1df, SUC_lexicon_tags_57 },
    { 0x9c7f1df, SUC_lexicon_tags_1 },
    { 0x2c7f1df, SUC_lexicon_tags_1 },
    { 0xf927f1e1, SUC_lexicon_tags_212 },
    { 0x9c27f1e3, SUC_lexicon_tags_15 },
    { 0x5467f1e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5247f1e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb907f1e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d87f1eb, SUC_lexicon_tags_31 },
    { 0xfc07f1ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc7f1f3, SUC_lexicon_tags_7 },
    { 0xafa7f1f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf467f1f7, SUC_lexicon_tags_45 },
    { 0x7387f1f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d67f1fb, SUC_lexicon_tags_17 },
    { 0xffc7f1fb, SUC_lexicon_tags_17 },
    { 0x39c7f1fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c7f200, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ba7f202, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e87f204, SUC_lexicon_tags_154 },
    { 0x63e7f205, SUC_lexicon_tags_31 },
    { 0x3e27f205, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e7f209, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f67f20f, SUC_lexicon_tags_33 },
    { 0x1ce7f20f, SUC_lexicon_tags_37 },
    { 0xfee7f20f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0e7f213, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e7f217, SUC_lexicon_tags_1 },
    { 0x2187f217, SUC_lexicon_tags_10 },
    { 0x60a7f218, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xce47f21b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc7f21d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e7f21f, SUC_lexicon_tags_12 },
    { 0x1227f220, SUC_lexicon_tags_69 },
    { 0x4687f220, SUC_lexicon_tags_1 },
    { 0x2107f222, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5067f226, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8607f228, SUC_lexicon_tags_15 },
    { 0x4ce7f229, SUC_lexicon_tags_7 },
    { 0xda47f22a, SUC_lexicon_tags_1 },
    { 0x5e67f22b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb87f22d, SUC_lexicon_tags_17 },
    { 0xd607f22e, SUC_lexicon_tags_1 },
    { 0x2467f22f, SUC_lexicon_tags_37 },
    { 0xee47f230, SUC_lexicon_tags_26 },
    { 0x8607f231, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9f07f233, SUC_lexicon_tags_37 },
    { 0x38e7f234, SUC_lexicon_tags_55 },
    { 0x4987f234, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c7f23b, SUC_lexicon_tags_8 },
    { 0xe6a7f23c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f47f240, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe3e7f242, SUC_lexicon_tags_49 },
    { 0x9e07f243, SUC_lexicon_tags_37 },
    { 0xaa87f242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbe7f246, SUC_lexicon_tags_33 },
    { 0x4f87f247, SUC_lexicon_tags_15 },
    { 0x8987f248, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe567f253, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c7f262, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x947f265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf947f267, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf747f269, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b67f26d, SUC_lexicon_tags_1 },
    { 0x5647f26e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x807f270, SUC_lexicon_tags_12 },
    { 0x1587f271, SUC_lexicon_tags_65 },
    { 0x3c87f271, SUC_lexicon_tags_19 },
    { 0x9747f270, SUC_lexicon_tags_11 },
    { 0x4687f273, SUC_lexicon_tags_6 },
    { 0x3987f275, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1887f277, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xefa7f279, SUC_lexicon_tags_4 },
    { 0x5387f27a, SUC_lexicon_tags_66 },
    { 0xe07f27b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f47f27f, SUC_lexicon_tags_57 },
    { 0xccc7f27f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe387f284, SUC_lexicon_tags_12 },
    { 0x1367f285, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c7f28a, SUC_lexicon_tags_25 },
    { 0x38a7f28a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52a7f28d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf447f290, SUC_lexicon_tags_6 },
    { 0xb647f290, SUC_lexicon_tags_1 },
    { 0x3e07f290, SUC_lexicon_tags_57 },
    { 0x6487f290, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b47f29d, SUC_lexicon_tags_30 },
    { 0xed87f29d, SUC_lexicon_tags_37 },
    { 0x86c7f29f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba7f2a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb547f2a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65c7f2a9, SUC_lexicon_tags_8 },
    { 0x6fc7f2aa, SUC_lexicon_tags_12 },
    { 0x6967f2aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3327f2b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd67f2b5, SUC_lexicon_tags_4 },
    { 0xeb47f2b5, SUC_lexicon_tags_47 },
    { 0x1207f2b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c47f2ba, SUC_lexicon_tags_17 },
    { 0x2bc7f2bb, SUC_lexicon_tags_27 },
    { 0xe0a7f2ba, SUC_lexicon_tags_29 },
    { 0x3ae7f2bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f27f2c0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd687f2c2, SUC_lexicon_tags_2 },
    { 0xc047f2c3, SUC_lexicon_tags_62 },
    { 0xff47f2c4, SUC_lexicon_tags_47 },
    { 0xb167f2c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf407f2ca, SUC_lexicon_tags_8 },
    { 0xb87f2cb, SUC_lexicon_tags_17 },
    { 0x1867f2ca, SUC_lexicon_tags_1 },
    { 0x7d67f2cd, SUC_lexicon_tags_25 },
    { 0x4367f2ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c7f2d3, SUC_lexicon_tags_11 },
    { 0x1947f2d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2007f2d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8587f2d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe967f2da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb847f2dd, SUC_lexicon_tags_57 },
    { 0x607f2dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb547f2e0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa7f2e5, SUC_lexicon_tags_37 },
    { 0x9c27f2e5, SUC_lexicon_tags_15 },
    { 0xf47f2e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6de7f2e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4e7f2eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfcc7f2ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd27f2ef, SUC_lexicon_tags_14 },
    { 0xd927f2f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7c87f2f2, SUC_lexicon_tags_32 },
    { 0x29a7f2f3, SUC_lexicon_tags_27 },
    { 0xb127f2f4, SUC_lexicon_tags_105 },
    { 0x8287f2f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5927f2f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e7f2fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x61a7f300, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde27f302, SUC_lexicon_tags_30 },
    { 0x26a7f303, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe07f305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae67f30a, SUC_lexicon_tags_17 },
    { 0x5ce7f30b, SUC_lexicon_tags_1 },
    { 0x22e7f30c, SUC_lexicon_tags_1 },
    { 0x14a7f30d, SUC_lexicon_tags_23 },
    { 0x25a7f30d, SUC_lexicon_tags_38 },
    { 0x4d47f30c, SUC_lexicon_tags_7 },
    { 0x9be7f30a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xda67f312, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a7f315, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x147f317, SUC_lexicon_tags_41 },
    { 0x3c07f317, SUC_lexicon_tags_7 },
    { 0xaa47f317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5627f31d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a47f31f, SUC_lexicon_tags_13 },
    { 0x46e7f320, SUC_lexicon_tags_11 },
    { 0x6587f31f, SUC_lexicon_tags_49 },
    { 0xde27f320, SUC_lexicon_tags_1614 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae7f326, SUC_lexicon_tags_25 },
    { 0x6e7f327, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc247f32d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd347f32f, SUC_lexicon_tags_38 },
    { 0x8167f330, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1de7f332, SUC_lexicon_tags_15 },
    { 0xbf67f333, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc987f336, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc87f33a, SUC_lexicon_tags_25 },
    { 0x26e7f33b, SUC_lexicon_tags_11 },
    { 0xd467f33a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47f344, SUC_lexicon_tags_11 },
    { 0x6867f345, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7f348, SUC_lexicon_tags_7 },
    { 0xede7f349, SUC_lexicon_tags_37 },
    { 0x7407f34a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xab87f34d, SUC_lexicon_tags_12 },
    { 0x76a7f34e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe7f351, SUC_lexicon_tags_7 },
    { 0x5aa7f352, SUC_lexicon_tags_17 },
    { 0x3587f351, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a7f363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc87f365, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa887f36b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb747f36d, SUC_lexicon_tags_49 },
    { 0x1ca7f36e, SUC_lexicon_tags_33 },
    { 0xd527f36f, SUC_lexicon_tags_44 },
    { 0x3bc7f370, SUC_lexicon_tags_9 },
    { 0x86c7f370, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf127f373, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8127f377, SUC_lexicon_tags_2 },
    { 0xc3a7f377, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xe287f37a, SUC_lexicon_tags_23 },
    { 0xd7c7f37b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3867f382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ec7f384, SUC_lexicon_tags_60 },
    { 0x9f87f385, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0e7f387, SUC_lexicon_tags_23 },
    { 0x24a7f388, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c07f38d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2727f390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d47f395, SUC_lexicon_tags_7 },
    { 0x4c7f396, SUC_lexicon_tags_8 },
    { 0x6907f397, SUC_lexicon_tags_17 },
    { 0xf007f398, SUC_lexicon_tags_37 },
    { 0xdae7f396, SUC_lexicon_tags_37 },
    { 0x1a67f399, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5e27f39c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a7f39f, SUC_lexicon_tags_2 },
    { 0x3f07f39f, SUC_lexicon_tags_17 },
    { 0xa07f39f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b87f3a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa487f3ae, SUC_lexicon_tags_11 },
    { 0x64e7f3ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbba7f3b1, SUC_lexicon_tags_25 },
    { 0x84a7f3b2, SUC_lexicon_tags_9 },
    { 0x8187f3b1, SUC_lexicon_tags_11 },
    { 0x7de7f3b4, SUC_lexicon_tags_24 },
    { 0x5647f3b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c7f3b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7447f3bb, SUC_lexicon_tags_10 },
    { 0x4347f3bb, SUC_lexicon_tags_85 },
    { 0xde7f3bc, SUC_lexicon_tags_316 },
    { 0x21e7f3be, SUC_lexicon_tags_17 },
    { 0xb167f3bd, SUC_lexicon_tags_6 },
    { 0xce47f3c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a7f3c5, SUC_lexicon_tags_60 },
    { 0xd027f3c6, SUC_lexicon_tags_133 },
    { 0x67a7f3c7, SUC_lexicon_tags_12 },
    { 0xace7f3c8, SUC_lexicon_tags_17 },
    { 0xfbc7f3c8, SUC_lexicon_tags_11 },
    { 0x3707f3c9, SUC_lexicon_tags_60 },
    { 0xfd27f3cb, SUC_lexicon_tags_30 },
    { 0xf567f3cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee27f3d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67f3da, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb587f3e0, SUC_lexicon_tags_12 },
    { 0x327f3e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7907f3ea, SUC_lexicon_tags_14 },
    { 0xac67f3eb, SUC_lexicon_tags_100 },
    { 0x7f3eb, SUC_lexicon_tags_1 },
    { 0xa207f3ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc367f3f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47f3f3, SUC_lexicon_tags_62 },
    { 0xee67f3f3, SUC_lexicon_tags_11 },
    { 0x5c47f3f4, SUC_lexicon_tags_7 },
    { 0xe947f3f6, SUC_lexicon_tags_6 },
    { 0x56c7f3f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x387f3f9, SUC_lexicon_tags_47 },
    { 0xb2a7f3fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4967f3fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5be7f400, SUC_lexicon_tags_15 },
    { 0xb987f401, SUC_lexicon_tags_12 },
    { 0x3e67f402, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6747f40a, SUC_lexicon_tags_45 },
    { 0x6387f40b, SUC_lexicon_tags_1 },
    { 0xfd87f40b, SUC_lexicon_tags_17 },
    { 0x6de7f40d, SUC_lexicon_tags_15 },
    { 0xd6e7f40e, SUC_lexicon_tags_38 },
    { 0xa2e7f40e, SUC_lexicon_tags_1 },
    { 0x22e7f410, SUC_lexicon_tags_37 },
    { 0x8747f411, SUC_lexicon_tags_20 },
    { 0x7f67f40f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6987f414, SUC_lexicon_tags_7 },
    { 0xea07f414, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfee7f417, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487f41c, SUC_lexicon_tags_15 },
    { 0xdd27f41c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4f27f41f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa747f421, SUC_lexicon_tags_27 },
    { 0x7a47f422, SUC_lexicon_tags_23 },
    { 0xae67f422, SUC_lexicon_tags_17 },
    { 0xe7e7f423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef07f42a, SUC_lexicon_tags_2 },
    { 0x6667f42b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace7f42f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf727f437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1267f440, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a67f445, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca7f44c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc87f44f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd07f458, SUC_lexicon_tags_56 },
    { 0x1847f459, SUC_lexicon_tags_23 },
    { 0xc987f45a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c7f45d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1947f461, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xaa47f463, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1247f468, SUC_lexicon_tags_0 },
    { 0x7767f469, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9287f46b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8527f46f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb527f471, SUC_lexicon_tags_1 },
    { 0xc707f471, SUC_lexicon_tags_17 },
    { 0x75a7f473, SUC_lexicon_tags_11 },
    { 0x8e07f474, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6587f477, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5347f479, SUC_lexicon_tags_17 },
    { 0xcee7f47a, SUC_lexicon_tags_25 },
    { 0x3147f47b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a7f47e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac67f487, SUC_lexicon_tags_37 },
    { 0x4fc7f488, SUC_lexicon_tags_15 },
    { 0xe987f489, SUC_lexicon_tags_4 },
    { 0x2507f488, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c7f48d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb67f492, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2e7f494, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf67f49a, SUC_lexicon_tags_7 },
    { 0x1c7f49b, SUC_lexicon_tags_45 },
    { 0x6927f49a, SUC_lexicon_tags_37 },
    { 0x867f49b, SUC_lexicon_tags_84 },
    { 0xcfe7f49e, SUC_lexicon_tags_33 },
    { 0x9487f49f, SUC_lexicon_tags_38 },
    { 0x2047f49e, SUC_lexicon_tags_11 },
    { 0xb127f4a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47a7f4a3, SUC_lexicon_tags_0 },
    { 0x4407f4a4, SUC_lexicon_tags_43 },
    { 0x58e7f4a5, SUC_lexicon_tags_17 },
    { 0xcb07f4a5, SUC_lexicon_tags_7 },
    { 0x6247f4a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a7f4aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x18c7f4ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d47f4af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xab07f4b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b47f4b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa167f4ba, SUC_lexicon_tags_23 },
    { 0xd947f4bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3087f4bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae7f4c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e7f4c5, SUC_lexicon_tags_17 },
    { 0xac07f4c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad67f4ca, SUC_lexicon_tags_11 },
    { 0x7607f4cb, SUC_lexicon_tags_14 },
    { 0xde87f4cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d07f4d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd707f4d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe07f4de, SUC_lexicon_tags_23 },
    { 0x70c7f4df, SUC_lexicon_tags_23 },
    { 0x4fc7f4df, SUC_lexicon_tags_23 },
    { 0x5f67f4e1, SUC_lexicon_tags_37 },
    { 0xabc7f4de, SUC_lexicon_tags_6 },
    { 0x6a07f4e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5347f4e6, SUC_lexicon_tags_28 },
    { 0xa987f4e7, SUC_lexicon_tags_39 },
    { 0xe607f4e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c67f4ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7807f4ec, SUC_lexicon_tags_37 },
    { 0xf647f4ed, SUC_lexicon_tags_47 },
    { 0x9e67f4ed, SUC_lexicon_tags_17 },
    { 0x7ea7f4ed, SUC_lexicon_tags_12 },
    { 0x3f07f4ee, SUC_lexicon_tags_2 },
    { 0xb247f4ed, SUC_lexicon_tags_29 },
    { 0x1947f4f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd947f4f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf67f4fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc7f4fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567f502, SUC_lexicon_tags_49 },
    { 0x6ae7f502, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1647f505, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9027f50a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d27f50c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a7f511, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a7f514, SUC_lexicon_tags_39 },
    { 0xe8a7f515, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xbfe7f517, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa127f51b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf707f524, SUC_lexicon_tags_27 },
    { 0x5da7f525, SUC_lexicon_tags_63 },
    { 0x29c7f526, SUC_lexicon_tags_60 },
    { 0xbfa7f527, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea27f532, SUC_lexicon_tags_17 },
    { 0x4887f533, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5747f536, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc7f53b, SUC_lexicon_tags_87 },
    { 0xd887f53b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cc7f53e, SUC_lexicon_tags_65 },
    { 0x4de7f53f, SUC_lexicon_tags_0 },
    { 0x2927f540, SUC_lexicon_tags_25 },
    { 0x7b87f53f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6167f548, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee67f551, SUC_lexicon_tags_12 },
    { 0xf8c7f552, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa827f556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcac7f558, SUC_lexicon_tags_7 },
    { 0xb247f559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc687f55b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89e7f55d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xff07f55f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667f565, SUC_lexicon_tags_262 },
    { 0x86a7f566, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc67f573, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a7f576, SUC_lexicon_tags_56 },
    { 0x8b87f577, SUC_lexicon_tags_7 },
    { 0x7e67f578, SUC_lexicon_tags_49 },
    { 0x7e67f578, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x627f57c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf067f582, SUC_lexicon_tags_12 },
    { 0x167f582, SUC_lexicon_tags_17 },
    { 0x3967f584, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5687f586, SUC_lexicon_tags_11 },
    { 0x6327f586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787f58d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xede7f58f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce7f593, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e7f596, SUC_lexicon_tags_15 },
    { 0xdd67f597, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6727f59b, SUC_lexicon_tags_12 },
    { 0xd5c7f59b, SUC_lexicon_tags_39 },
    { 0xd607f59d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e7f5a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4407f5a5, SUC_lexicon_tags_46 },
    { 0xf227f5a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a7f5ae, SUC_lexicon_tags_47 },
    { 0xf67f5af, SUC_lexicon_tags_14 },
    { 0x5a27f5af, SUC_lexicon_tags_15 },
    { 0x4527f5af, SUC_lexicon_tags_17 },
    { 0xb7e7f5b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9687f5b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc7f5ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d27f5c1, SUC_lexicon_tags_14 },
    { 0xf107f5c2, SUC_lexicon_tags_7 },
    { 0x4f87f5c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a47f5c5, SUC_lexicon_tags_11 },
    { 0xe2c7f5c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc587f5c8, SUC_lexicon_tags_45 },
    { 0x45c7f5c9, SUC_lexicon_tags_65 },
    { 0x4c67f5c9, SUC_lexicon_tags_60 },
    { 0xc47f5ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x37e7f5cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a7f5d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf327f5d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb07f5d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4927f5d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf427f5db, SUC_lexicon_tags_7 },
    { 0x6567f5db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4f87f5de, SUC_lexicon_tags_17 },
    { 0xe2e7f5de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c7f5e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d87f5e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a7f5ea, SUC_lexicon_tags_21 },
    { 0x8747f5ea, SUC_lexicon_tags_1 },
    { 0xebe7f5ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd467f5ef, SUC_lexicon_tags_17 },
    { 0x1e87f5f0, SUC_lexicon_tags_5 },
    { 0x6ac7f5f1, SUC_lexicon_tags_2 },
    { 0x2fe7f5f2, SUC_lexicon_tags_29 },
    { 0x2407f5f0, SUC_lexicon_tags_17 },
    { 0xde07f5f4, SUC_lexicon_tags_108 },
    { 0x3fa7f5f5, SUC_lexicon_tags_25 },
    { 0xc27f5f0, SUC_lexicon_tags_11 },
    { 0xbae7f5f2, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be7f5fa, SUC_lexicon_tags_151 },
    { 0x8287f5fb, SUC_lexicon_tags_17 },
    { 0x867f5fc, SUC_lexicon_tags_14 },
    { 0xe8e7f5fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1207f604, SUC_lexicon_tags_8 },
    { 0x8567f604, SUC_lexicon_tags_33 },
    { 0x57e7f605, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e7f609, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc647f610, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd307f617, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x907f619, SUC_lexicon_tags_8 },
    { 0x18a7f61a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc7f61e, SUC_lexicon_tags_17 },
    { 0xe927f61e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7ba7f621, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7f625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaa7f627, SUC_lexicon_tags_12 },
    { 0x2aa7f628, SUC_lexicon_tags_2 },
    { 0x5c67f628, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33c7f62b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb207f62e, SUC_lexicon_tags_17 },
    { 0x1587f62f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc667f637, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed87f63b, SUC_lexicon_tags_27 },
    { 0x2687f63c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe7f645, SUC_lexicon_tags_17 },
    { 0xa227f646, SUC_lexicon_tags_48 },
    { 0xf367f647, SUC_lexicon_tags_66 },
    { 0x10c7f648, SUC_lexicon_tags_17 },
    { 0x3287f647, SUC_lexicon_tags_15 },
    { 0xc667f64a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x59a7f64d, SUC_lexicon_tags_7 },
    { 0x1467f64d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ec7f650, SUC_lexicon_tags_529 },
    { 0x59c7f651, SUC_lexicon_tags_0 },
    { 0x9d87f652, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe527f655, SUC_lexicon_tags_8 },
    { 0x7007f656, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3087f65a, SUC_lexicon_tags_8 },
    { 0xba07f65b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6dc7f65d, SUC_lexicon_tags_35 },
    { 0xef87f65e, SUC_lexicon_tags_15 },
    { 0x1687f65d, SUC_lexicon_tags_17 },
    { 0xe8a7f65e, SUC_lexicon_tags_15 },
    { 0xfca7f661, SUC_lexicon_tags_49 },
    { 0x4a07f65f, SUC_lexicon_tags_23 },
    { 0x3687f663, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf067f672, SUC_lexicon_tags_1 },
    { 0x47a7f672, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7f676, SUC_lexicon_tags_65 },
    { 0x5d27f677, SUC_lexicon_tags_25 },
    { 0xcdc7f678, SUC_lexicon_tags_55 },
    { 0xc7e7f677, SUC_lexicon_tags_47 },
    { 0xcd07f677, SUC_lexicon_tags_7 },
    { 0xce07f67b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b27f684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd87f686, SUC_lexicon_tags_47 },
    { 0xeaa7f687, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5167f68a, SUC_lexicon_tags_37 },
    { 0xa87f68b, SUC_lexicon_tags_15 },
    { 0xde27f68b, SUC_lexicon_tags_30 },
    { 0x1247f68d, SUC_lexicon_tags_6 },
    { 0xe7c7f68d, SUC_lexicon_tags_43 },
    { 0x98a7f68d, SUC_lexicon_tags_1 },
    { 0x6507f68f, SUC_lexicon_tags_25 },
    { 0x5927f691, SUC_lexicon_tags_55 },
    { 0xea87f692, SUC_lexicon_tags_65 },
    { 0x83e7f693, SUC_lexicon_tags_10 },
    { 0x8c87f693, SUC_lexicon_tags_25 },
    { 0x3047f695, SUC_lexicon_tags_7 },
    { 0x5207f691, SUC_lexicon_tags_11 },
    { 0xc407f694, SUC_lexicon_tags_49 },
    { 0x8a67f698, SUC_lexicon_tags_11 },
    { 0x53c7f699, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a67f69b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdd87f69d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc7f6a0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4287f6a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeea7f6a4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa927f6a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e7f6aa, SUC_lexicon_tags_29 },
    { 0xd0a7f6ab, SUC_lexicon_tags_35 },
    { 0x6c47f6ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6f67f6ae, SUC_lexicon_tags_1 },
    { 0x9ae7f6af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc7f6b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x48a7f6b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2927f6b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb87f6b8, SUC_lexicon_tags_9 },
    { 0xd0e7f6b8, SUC_lexicon_tags_46 },
    { 0xd9e7f6ba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1187f6c1, SUC_lexicon_tags_60 },
    { 0xaa67f6c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9767f6c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1647f6cf, SUC_lexicon_tags_19 },
    { 0x90c7f6cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6107f6d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x447f6d5, SUC_lexicon_tags_37 },
    { 0xe6c7f6d5, SUC_lexicon_tags_15 },
    { 0x4ca7f6d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb687f6d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x247f6db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d27f6e2, SUC_lexicon_tags_10 },
    { 0x86a7f6e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4e7f6e5, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x18c7f6e7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xaa87f6e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa467f6ee, SUC_lexicon_tags_2 },
    { 0xf567f6ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab47f6f2, SUC_lexicon_tags_45 },
    { 0x2647f6f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd227f6f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7667f6fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f27f6fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1367f6ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf807f703, SUC_lexicon_tags_15 },
    { 0x90a7f703, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc67f706, SUC_lexicon_tags_30 },
    { 0xf907f707, SUC_lexicon_tags_38 },
    { 0xeec7f707, SUC_lexicon_tags_7 },
    { 0xf747f708, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b67f70b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda7f70f, SUC_lexicon_tags_42 },
    { 0x5e47f70f, SUC_lexicon_tags_9 },
    { 0x54e7f711, SUC_lexicon_tags_2 },
    { 0x3807f712, SUC_lexicon_tags_11 },
    { 0x6ba7f710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf487f717, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f87f719, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2047f71b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c87f71d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb027f723, SUC_lexicon_tags_12 },
    { 0x31c7f723, SUC_lexicon_tags_37 },
    { 0xe607f725, SUC_lexicon_tags_11 },
    { 0x87c7f725, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a7f72b, SUC_lexicon_tags_11 },
    { 0x9e7f72c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7267f731, SUC_lexicon_tags_7 },
    { 0x427f731, SUC_lexicon_tags_17 },
    { 0x7f87f731, SUC_lexicon_tags_12 },
    { 0x1167f734, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2dc7f736, SUC_lexicon_tags_37 },
    { 0xc707f737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc7f73d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeac7f73f, SUC_lexicon_tags_12 },
    { 0x7087f740, SUC_lexicon_tags_19 },
    { 0x24e7f73f, SUC_lexicon_tags_23 },
    { 0xf187f742, SUC_lexicon_tags_31 },
    { 0x7427f742, SUC_lexicon_tags_45 },
    { 0x447f740, SUC_lexicon_tags_11 },
    { 0x2347f73f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xed47f747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e47f74b, SUC_lexicon_tags_37 },
    { 0x75c7f74c, SUC_lexicon_tags_29 },
    { 0x5e87f74d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x88c7f74f, SUC_lexicon_tags_33 },
    { 0x83a7f74f, SUC_lexicon_tags_46 },
    { 0x1ce7f750, SUC_lexicon_tags_87 },
    { 0xc787f750, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9907f755, SUC_lexicon_tags_6 },
    { 0x2087f756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2407f760, SUC_lexicon_tags_25 },
    { 0xf047f761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xede7f764, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc7f768, SUC_lexicon_tags_8 },
    { 0x167f769, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5267f770, SUC_lexicon_tags_54 },
    { 0xb8a7f770, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8a7f773, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2947f77c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c7f77f, SUC_lexicon_tags_48 },
    { 0x26e7f77f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf67f782, SUC_lexicon_tags_17 },
    { 0xc127f783, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xd7e7f785, SUC_lexicon_tags_37 },
    { 0x1b87f786, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe47f78a, SUC_lexicon_tags_6 },
    { 0x92c7f78b, SUC_lexicon_tags_14 },
    { 0x1b07f78b, SUC_lexicon_tags_37 },
    { 0x7da7f78c, SUC_lexicon_tags_19 },
    { 0xa1a7f78c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4587f793, SUC_lexicon_tags_15 },
    { 0xd347f794, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7807f798, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x947f79b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e7f79f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9a87f7a1, SUC_lexicon_tags_38 },
    { 0x1687f7a2, SUC_lexicon_tags_15 },
    { 0xdb27f7a2, SUC_lexicon_tags_66 },
    { 0xf27f7a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a7f7a7, SUC_lexicon_tags_57 },
    { 0x9007f7a8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1827f7af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e7f7b5, SUC_lexicon_tags_37 },
    { 0xb447f7b6, SUC_lexicon_tags_25 },
    { 0xa567f7b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a7f7bc, SUC_lexicon_tags_14 },
    { 0xc247f7bd, SUC_lexicon_tags_11 },
    { 0xde7f7bd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaa47f7c0, SUC_lexicon_tags_12 },
    { 0xb007f7c0, SUC_lexicon_tags_63 },
    { 0x28e7f7c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87f7c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a7f7cd, SUC_lexicon_tags_0 },
    { 0xd667f7cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3667f7d0, SUC_lexicon_tags_1 },
    { 0xa947f7d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e7f7d5, SUC_lexicon_tags_10 },
    { 0x2a07f7d6, SUC_lexicon_tags_25 },
    { 0x3227f7d7, SUC_lexicon_tags_37 },
    { 0x3b07f7d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e87f7dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d67f7df, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe927f7e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d27f7e9, SUC_lexicon_tags_0 },
    { 0xe467f7e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe847f7f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2967f7f6, SUC_lexicon_tags_14 },
    { 0x99e7f7f6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e7f7fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa7f7ff, SUC_lexicon_tags_37 },
    { 0x2d67f800, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x1987f802, SUC_lexicon_tags_25 },
    { 0xc687f803, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae7f807, SUC_lexicon_tags_112 },
    { 0xe707f808, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5847f80a, SUC_lexicon_tags_11 },
    { 0x3de7f80b, SUC_lexicon_tags_17 },
    { 0x7887f80c, SUC_lexicon_tags_11 },
    { 0x3b67f80a, SUC_lexicon_tags_17 },
    { 0x4187f80e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1207f815, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xaa07f817, SUC_lexicon_tags_45 },
    { 0xa287f817, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e67f81c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee7f820, SUC_lexicon_tags_11 },
    { 0x5aa7f821, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x36a7f823, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc7f828, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xb567f82a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x71e7f82c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7187f82f, SUC_lexicon_tags_11 },
    { 0xe367f830, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca47f834, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdd67f836, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e47f83c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea7f844, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9107f846, SUC_lexicon_tags_46 },
    { 0x7267f847, SUC_lexicon_tags_38 },
    { 0x4ea7f848, SUC_lexicon_tags_26 },
    { 0x17e7f846, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf587f84c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd167f84e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea27f851, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa1e7f853, SUC_lexicon_tags_25 },
    { 0x38c7f854, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcc7f856, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7507f858, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f47f85b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6467f85e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7707f861, SUC_lexicon_tags_6 },
    { 0x6467f862, SUC_lexicon_tags_2 },
    { 0xd167f863, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb47f866, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f67f869, SUC_lexicon_tags_15 },
    { 0xa507f86a, SUC_lexicon_tags_1 },
    { 0xff47f869, SUC_lexicon_tags_17 },
    { 0xfc67f86b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xafa7f86e, SUC_lexicon_tags_25 },
    { 0xc507f86e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c7f874, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc227f878, SUC_lexicon_tags_7 },
    { 0xfca7f878, SUC_lexicon_tags_14 },
    { 0x3cc7f87a, SUC_lexicon_tags_26 },
    { 0x93a7f87b, SUC_lexicon_tags_12 },
    { 0xd487f87c, SUC_lexicon_tags_1 },
    { 0x5707f87d, SUC_lexicon_tags_47 },
    { 0xab47f87d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa7f883, SUC_lexicon_tags_25 },
    { 0x19c7f884, SUC_lexicon_tags_12 },
    { 0x7cc7f885, SUC_lexicon_tags_25 },
    { 0xe767f885, SUC_lexicon_tags_46 },
    { 0xa3a7f887, SUC_lexicon_tags_124 },
    { 0xb307f884, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbdc7f88a, SUC_lexicon_tags_68 },
    { 0xc247f88b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a7f88e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea27f895, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4627f898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe87f89e, SUC_lexicon_tags_7 },
    { 0xc947f89e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8727f8a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbc7f8a6, SUC_lexicon_tags_46 },
    { 0x7187f8a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xafe7f8a9, SUC_lexicon_tags_25 },
    { 0xfe7f8aa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607f8ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae7f8ba, SUC_lexicon_tags_17 },
    { 0xbb27f8bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3247f8c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2c7f8c3, SUC_lexicon_tags_19 },
    { 0x73c7f8c4, SUC_lexicon_tags_10 },
    { 0x4c7f8c5, SUC_lexicon_tags_39 },
    { 0x1b27f8c6, SUC_lexicon_tags_8 },
    { 0x6327f8c7, SUC_lexicon_tags_17 },
    { 0xaea7f8c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e7f8cc, SUC_lexicon_tags_32 },
    { 0xf2c7f8cd, SUC_lexicon_tags_48 },
    { 0xc707f8cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe427f8d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea7f8da, SUC_lexicon_tags_17 },
    { 0x3b87f8db, SUC_lexicon_tags_27 },
    { 0x5d27f8dc, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e7f8e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7f8e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7f8e9, SUC_lexicon_tags_63 },
    { 0x68c7f8ea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6347f8ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2fc7f8ef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb527f8f8, SUC_lexicon_tags_973 },
    { 0, NULL },
    { 0x8887f8fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de7f8fd, SUC_lexicon_tags_26 },
    { 0xe0a7f8fe, SUC_lexicon_tags_31 },
    { 0x147f8fe, SUC_lexicon_tags_38 },
    { 0x22e7f900, SUC_lexicon_tags_37 },
    { 0xf8c7f8ff, SUC_lexicon_tags_9 },
    { 0xe067f902, SUC_lexicon_tags_9 },
    { 0x667f903, SUC_lexicon_tags_19 },
    { 0xa807f904, SUC_lexicon_tags_22 },
    { 0xce47f903, SUC_lexicon_tags_30 },
    { 0x8a47f903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa27f90a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c7f910, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f47f912, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2907f91d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7f920, SUC_lexicon_tags_39 },
    { 0x4467f921, SUC_lexicon_tags_95 },
    { 0x2427f922, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb27f928, SUC_lexicon_tags_15 },
    { 0x867f928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4527f92e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24c7f930, SUC_lexicon_tags_17 },
    { 0x8247f931, SUC_lexicon_tags_14 },
    { 0xb0e7f932, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7967f938, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a67f93e, SUC_lexicon_tags_31 },
    { 0xae7f93e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe287f943, SUC_lexicon_tags_25 },
    { 0x1cc7f944, SUC_lexicon_tags_6 },
    { 0xe8e7f945, SUC_lexicon_tags_87 },
    { 0x26c7f943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x827f948, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb227f94f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x667f951, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e27f954, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ca7f956, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a7f95a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5247f95d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6447f962, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a47f964, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2667f966, SUC_lexicon_tags_27 },
    { 0xf067f967, SUC_lexicon_tags_30 },
    { 0x9827f968, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x25e7f96a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a7f96e, SUC_lexicon_tags_2 },
    { 0x9047f96e, SUC_lexicon_tags_49 },
    { 0x85a7f970, SUC_lexicon_tags_47 },
    { 0x3b07f971, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb67f976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7827f979, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x3587f97b, SUC_lexicon_tags_6 },
    { 0xf7a7f97c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xe0a7f97e, SUC_lexicon_tags_46 },
    { 0xc6e7f97e, SUC_lexicon_tags_27 },
    { 0x8447f980, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c7f984, SUC_lexicon_tags_29 },
    { 0x2347f984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e27f988, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe407f98c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc7f98f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec07f995, SUC_lexicon_tags_15 },
    { 0xc367f996, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf947f99a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f27f99e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5127f9a1, SUC_lexicon_tags_17 },
    { 0x93a7f9a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf627f9a6, SUC_lexicon_tags_37 },
    { 0x52c7f9a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4067f9ad, SUC_lexicon_tags_27 },
    { 0xada7f9ae, SUC_lexicon_tags_25 },
    { 0xc747f9af, SUC_lexicon_tags_37 },
    { 0x8947f9b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4e7f9b2, SUC_lexicon_tags_17 },
    { 0xe307f9b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe7f9ba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec7f9bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae7f9c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e7f9c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c67f9c8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b47f9cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a7f9ce, SUC_lexicon_tags_37 },
    { 0xd7c7f9ce, SUC_lexicon_tags_60 },
    { 0x24c7f9d0, SUC_lexicon_tags_23 },
    { 0x5ba7f9d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d27f9d6, SUC_lexicon_tags_7 },
    { 0x1ea7f9d7, SUC_lexicon_tags_35 },
    { 0xbe87f9d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5127f9e2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3147f9ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce7f9f1, SUC_lexicon_tags_37 },
    { 0xa467f9f2, SUC_lexicon_tags_26 },
    { 0x2b27f9f3, SUC_lexicon_tags_37 },
    { 0xcf67f9f2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xaa07f9f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e47f9fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c87f9fe, SUC_lexicon_tags_622 },
    { 0x2b27f9fe, SUC_lexicon_tags_38 },
    { 0xe927fa00, SUC_lexicon_tags_2 },
    { 0xaae7f9ff, SUC_lexicon_tags_7 },
    { 0x1e87fa02, SUC_lexicon_tags_12 },
    { 0x8fa7fa03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547fa07, SUC_lexicon_tags_49 },
    { 0x4747fa07, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa0b, SUC_lexicon_tags_27 },
    { 0x4fe7fa0c, SUC_lexicon_tags_15 },
    { 0x2ee7fa0c, SUC_lexicon_tags_10 },
    { 0x7bc7fa0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6707fa10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3a7fa12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x93e7fa14, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd27fa19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc787fa1c, SUC_lexicon_tags_154 },
    { 0x56e7fa1c, SUC_lexicon_tags_27 },
    { 0x7dc7fa1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb027fa30, SUC_lexicon_tags_17 },
    { 0xf4a7fa31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf747fa33, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4147fa35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb487fa37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c7fa3a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7607fa3e, SUC_lexicon_tags_16 },
    { 0x3487fa3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec7fa42, SUC_lexicon_tags_336 },
    { 0x9d27fa43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc47fa46, SUC_lexicon_tags_1 },
    { 0x8807fa47, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e7fa4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d27fa59, SUC_lexicon_tags_101 },
    { 0x5467fa5a, SUC_lexicon_tags_1 },
    { 0x2c47fa5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67fa5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x267fa61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e87fa6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb67fa6c, SUC_lexicon_tags_49 },
    { 0x6e7fa6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1567fa6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede7fa73, SUC_lexicon_tags_43 },
    { 0x11a7fa74, SUC_lexicon_tags_104 },
    { 0xf5c7fa73, SUC_lexicon_tags_2 },
    { 0xd047fa76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2567fa79, SUC_lexicon_tags_25 },
    { 0x4787fa7a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6907fa7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa7fa81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc7fa84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3207fa86, SUC_lexicon_tags_1 },
    { 0xbd07fa87, SUC_lexicon_tags_30 },
    { 0xc247fa87, SUC_lexicon_tags_8 },
    { 0x9147fa86, SUC_lexicon_tags_58 },
    { 0x5107fa86, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e7fa8d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6187fa8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e27fa91, SUC_lexicon_tags_25 },
    { 0xaf47fa92, SUC_lexicon_tags_1 },
    { 0xb0e7fa93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf027fa98, SUC_lexicon_tags_7 },
    { 0x45e7fa99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa267fa9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b47fa9d, SUC_lexicon_tags_17 },
    { 0x9b07fa9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4327faa3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c7faa6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba07faae, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xbce7fab0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7fab3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e87fab7, SUC_lexicon_tags_43 },
    { 0x4147fab7, SUC_lexicon_tags_10 },
    { 0xf2e7fab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7987fabc, SUC_lexicon_tags_17 },
    { 0x8027fabd, SUC_lexicon_tags_30 },
    { 0xda7fabe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75e7fac0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f47fac5, SUC_lexicon_tags_32 },
    { 0x3b27fac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3187facc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb47face, SUC_lexicon_tags_25 },
    { 0x22e7facf, SUC_lexicon_tags_37 },
    { 0x80c7fad0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac87fad2, SUC_lexicon_tags_47 },
    { 0xa867fad3, SUC_lexicon_tags_8 },
    { 0xfa47fad4, SUC_lexicon_tags_30 },
    { 0x4a87fad5, SUC_lexicon_tags_45 },
    { 0x3067fad5, SUC_lexicon_tags_9 },
    { 0x2847fad7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa7fadb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa807fadd, SUC_lexicon_tags_37 },
    { 0x65c7fade, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f87fae1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf727fae8, SUC_lexicon_tags_12 },
    { 0x87fae9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x92e7faeb, SUC_lexicon_tags_47 },
    { 0x1ba7faec, SUC_lexicon_tags_37 },
    { 0xec07faed, SUC_lexicon_tags_9 },
    { 0x86c7faec, SUC_lexicon_tags_77 },
    { 0x5847faef, SUC_lexicon_tags_38 },
    { 0x58a7faf0, SUC_lexicon_tags_6 },
    { 0xcf87faec, SUC_lexicon_tags_6 },
    { 0x8227faf2, SUC_lexicon_tags_17 },
    { 0x7f07faf2, SUC_lexicon_tags_10 },
    { 0x6027faf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c07faf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fa7faf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e47fafb, SUC_lexicon_tags_15 },
    { 0x3327fafc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b47fb02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c7fb06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5787fb08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4a7fb0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8387fb10, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1887fb14, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7fb18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2047fb1b, SUC_lexicon_tags_37 },
    { 0x6847fb1b, SUC_lexicon_tags_154 },
    { 0xd427fb1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2187fb21, SUC_lexicon_tags_17 },
    { 0x99e7fb22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7287fb25, SUC_lexicon_tags_25 },
    { 0xac87fb26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c7fb2e, SUC_lexicon_tags_17 },
    { 0x4267fb2f, SUC_lexicon_tags_129 },
    { 0xd347fb30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a7fb35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa7fb3c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x47c7fb3e, SUC_lexicon_tags_15 },
    { 0x4d47fb3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de7fb46, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c87fb4b, SUC_lexicon_tags_15 },
    { 0xada7fb4c, SUC_lexicon_tags_25 },
    { 0x21a7fb4c, SUC_lexicon_tags_2 },
    { 0x6407fb4e, SUC_lexicon_tags_61 },
    { 0xbae7fb4f, SUC_lexicon_tags_27 },
    { 0x3787fb50, SUC_lexicon_tags_23 },
    { 0x5da7fb50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae67fb55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9767fb5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ca7fb5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b07fb60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa527fb63, SUC_lexicon_tags_31 },
    { 0xd5a7fb64, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a7fb69, SUC_lexicon_tags_15 },
    { 0x8c47fb6a, SUC_lexicon_tags_30 },
    { 0x8707fb69, SUC_lexicon_tags_12 },
    { 0x2647fb6c, SUC_lexicon_tags_38 },
    { 0xeb87fb6d, SUC_lexicon_tags_48 },
    { 0xb4c7fb6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7387fb78, SUC_lexicon_tags_7 },
    { 0x3947fb79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c47fb7b, SUC_lexicon_tags_15 },
    { 0x7867fb7c, SUC_lexicon_tags_2 },
    { 0x6ac7fb7d, SUC_lexicon_tags_45 },
    { 0xed07fb7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6407fb80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd087fb85, SUC_lexicon_tags_27 },
    { 0x5fe7fb86, SUC_lexicon_tags_85 },
    { 0x64a7fb87, SUC_lexicon_tags_25 },
    { 0xe9c7fb88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae27fb8d, SUC_lexicon_tags_7 },
    { 0xbb07fb8d, SUC_lexicon_tags_7 },
    { 0xafc7fb8e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2bc7fb91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5987fb94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc67fba0, SUC_lexicon_tags_26 },
    { 0x1107fba1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e7fba4, SUC_lexicon_tags_38 },
    { 0x57e7fba5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5467fbac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9a87fbae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa27fbb2, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa27fbbe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c7fbc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a87fbc5, SUC_lexicon_tags_7 },
    { 0x32a7fbc6, SUC_lexicon_tags_20 },
    { 0xf707fbc7, SUC_lexicon_tags_25 },
    { 0xe087fbc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9307fbce, SUC_lexicon_tags_57 },
    { 0x5da7fbce, SUC_lexicon_tags_55 },
    { 0xbba7fbd0, SUC_lexicon_tags_37 },
    { 0x1947fbcf, SUC_lexicon_tags_33 },
    { 0xec47fbd2, SUC_lexicon_tags_6 },
    { 0xf487fbd2, SUC_lexicon_tags_1 },
    { 0x2747fbd4, SUC_lexicon_tags_11 },
    { 0xec87fbd5, SUC_lexicon_tags_12 },
    { 0xba47fbd6, SUC_lexicon_tags_2 },
    { 0xaf47fbd4, SUC_lexicon_tags_28 },
    { 0x9f07fbd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc7fbdb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a07fbe5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ce7fbe7, SUC_lexicon_tags_25 },
    { 0xb87fbe8, SUC_lexicon_tags_7 },
    { 0x97c7fbe9, SUC_lexicon_tags_18 },
    { 0x6d27fbe9, SUC_lexicon_tags_17 },
    { 0x5ca7fbeb, SUC_lexicon_tags_7 },
    { 0x7fe7fbeb, SUC_lexicon_tags_65 },
    { 0x8167fbed, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe7fbf0, SUC_lexicon_tags_19 },
    { 0x1087fbf1, SUC_lexicon_tags_43 },
    { 0x6267fbf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf47fbf4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb187fbfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63e7fbfe, SUC_lexicon_tags_127 },
    { 0x1fe7fbfe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3987fc01, SUC_lexicon_tags_31 },
    { 0xa567fc01, SUC_lexicon_tags_63 },
    { 0xdf87fc03, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc967fc0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7fc0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a7fc15, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2127fc1b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe27fc1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c47fc1f, SUC_lexicon_tags_37 },
    { 0x5387fc1f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de7fc25, SUC_lexicon_tags_7 },
    { 0xdc47fc26, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3c27fc28, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7d67fc2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e7fc2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xba27fc30, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf7e7fc32, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe907fc35, SUC_lexicon_tags_12 },
    { 0xa687fc35, SUC_lexicon_tags_7 },
    { 0x7487fc35, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1427fc45, SUC_lexicon_tags_12 },
    { 0x3467fc46, SUC_lexicon_tags_0 },
    { 0x83e7fc46, SUC_lexicon_tags_2 },
    { 0xea67fc48, SUC_lexicon_tags_14 },
    { 0xd4a7fc49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5847fc4c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe167fc51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d67fc56, SUC_lexicon_tags_7 },
    { 0x7307fc56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1607fc5a, SUC_lexicon_tags_2 },
    { 0xac27fc5b, SUC_lexicon_tags_15 },
    { 0x9ea7fc5c, SUC_lexicon_tags_11 },
    { 0x86c7fc5d, SUC_lexicon_tags_38 },
    { 0xf487fc5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f07fc62, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a47fc65, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b87fc6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07fc70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a27fc72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa7fc75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdce7fc77, SUC_lexicon_tags_14 },
    { 0xd687fc78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e7fc7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf87fc7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd07fc82, SUC_lexicon_tags_14 },
    { 0xb327fc83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9887fc85, SUC_lexicon_tags_11 },
    { 0xc727fc86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf407fc89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8007fc91, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b87fc97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e47fc99, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xabe7fc9b, SUC_lexicon_tags_1 },
    { 0x3507fc9b, SUC_lexicon_tags_31 },
    { 0xf5a7fc9d, SUC_lexicon_tags_46 },
    { 0x3b47fc9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd27fca4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8687fca6, SUC_lexicon_tags_38 },
    { 0x7667fca7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6da7fca9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3247fcac, SUC_lexicon_tags_19 },
    { 0x6907fcac, SUC_lexicon_tags_66 },
    { 0x9aa7fcae, SUC_lexicon_tags_2 },
    { 0x3347fcae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4107fcb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3067fcb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de7fcbd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4967fcbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf987fcc3, SUC_lexicon_tags_23 },
    { 0xa2a7fcc4, SUC_lexicon_tags_30 },
    { 0xb2e7fcc5, SUC_lexicon_tags_2 },
    { 0xc227fcc4, SUC_lexicon_tags_49 },
    { 0xf9c7fcc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5987fcc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e47fcce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347fcd2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5287fcd4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeb47fcd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a87fcd8, SUC_lexicon_tags_37 },
    { 0xf887fcd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227fcde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d87fce4, SUC_lexicon_tags_11 },
    { 0x5a47fce5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9587fce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82c7fce9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c7fcf0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a7fcfb, SUC_lexicon_tags_33 },
    { 0x79a7fcfb, SUC_lexicon_tags_25 },
    { 0xf9a7fcfd, SUC_lexicon_tags_12 },
    { 0x9f27fcfe, SUC_lexicon_tags_15 },
    { 0x2747fcfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe187fd03, SUC_lexicon_tags_25 },
    { 0x7807fd04, SUC_lexicon_tags_1 },
    { 0x4087fd04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae7fd09, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdd47fd0b, SUC_lexicon_tags_0 },
    { 0xdb07fd0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e47fd11, SUC_lexicon_tags_25 },
    { 0x9667fd11, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xde47fd14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e7fd1a, SUC_lexicon_tags_34 },
    { 0x2ce7fd1b, SUC_lexicon_tags_11 },
    { 0xc7e7fd1c, SUC_lexicon_tags_11 },
    { 0xbea7fd1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47fd20, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc67fd23, SUC_lexicon_tags_62 },
    { 0x3b67fd23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c7fd2f, SUC_lexicon_tags_25 },
    { 0x2d67fd2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b47fd33, SUC_lexicon_tags_31 },
    { 0x3727fd33, SUC_lexicon_tags_8 },
    { 0x7187fd35, SUC_lexicon_tags_37 },
    { 0xca87fd36, SUC_lexicon_tags_17 },
    { 0x80c7fd37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c7fd3f, SUC_lexicon_tags_27 },
    { 0xa1c7fd40, SUC_lexicon_tags_26 },
    { 0xe247fd41, SUC_lexicon_tags_0 },
    { 0x54a7fd42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa7fd4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x827fd50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a47fd53, SUC_lexicon_tags_45 },
    { 0xf187fd54, SUC_lexicon_tags_14 },
    { 0x7307fd54, SUC_lexicon_tags_25 },
    { 0x2ce7fd56, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6747fd58, SUC_lexicon_tags_60 },
    { 0xa1c7fd59, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf247fd5d, SUC_lexicon_tags_30 },
    { 0x81a7fd5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4847fd61, SUC_lexicon_tags_11 },
    { 0xf9c7fd62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1227fd65, SUC_lexicon_tags_1 },
    { 0x4d47fd65, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc7fd6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf547fd6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a7fd70, SUC_lexicon_tags_17 },
    { 0x6e07fd71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc047fd75, SUC_lexicon_tags_65 },
    { 0x8e27fd76, SUC_lexicon_tags_3 },
    { 0x56a7fd75, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c07fd7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e7fd81, SUC_lexicon_tags_2 },
    { 0x76e7fd82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca87fd87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa847fd8d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd27fd90, SUC_lexicon_tags_25 },
    { 0xbc27fd90, SUC_lexicon_tags_20 },
    { 0x507fd92, SUC_lexicon_tags_13 },
    { 0x1747fd92, SUC_lexicon_tags_2 },
    { 0x5587fd94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a7fd9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4e7fd9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaee7fd9f, SUC_lexicon_tags_9 },
    { 0xbf87fda0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9327fda4, SUC_lexicon_tags_230 },
    { 0xbba7fda5, SUC_lexicon_tags_63 },
    { 0xd887fda6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7fdac, SUC_lexicon_tags_2 },
    { 0xaa27fdad, SUC_lexicon_tags_37 },
    { 0x9107fdad, SUC_lexicon_tags_29 },
    { 0xcce7fdaf, SUC_lexicon_tags_11 },
    { 0x5fe7fdb0, SUC_lexicon_tags_61 },
    { 0x4fc7fdb1, SUC_lexicon_tags_28 },
    { 0xecc7fdb2, SUC_lexicon_tags_11 },
    { 0x9267fdb3, SUC_lexicon_tags_37 },
    { 0x2347fdac, SUC_lexicon_tags_27 },
    { 0x8e27fdb5, SUC_lexicon_tags_1 },
    { 0xa167fdb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe7fdbc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa47fdc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef07fdc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc027fdc8, SUC_lexicon_tags_1 },
    { 0xd1a7fdc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7447fdcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b07fdd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc67fdd2, SUC_lexicon_tags_31 },
    { 0x6187fdd3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9907fdd5, SUC_lexicon_tags_7 },
    { 0xf1e7fdd5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa5c7fdd8, SUC_lexicon_tags_14 },
    { 0xbce7fdd9, SUC_lexicon_tags_27 },
    { 0xb4e7fdd8, SUC_lexicon_tags_14 },
    { 0x6ee7fddb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc067fddd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee67fddf, SUC_lexicon_tags_22 },
    { 0x7107fddf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e47fde5, SUC_lexicon_tags_2 },
    { 0x687fde6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfee7fde8, SUC_lexicon_tags_0 },
    { 0x8467fde9, SUC_lexicon_tags_17 },
    { 0xc127fde9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6887fdef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd747fdf7, SUC_lexicon_tags_162 },
    { 0x5147fdf8, SUC_lexicon_tags_37 },
    { 0xd787fdf8, SUC_lexicon_tags_9 },
    { 0x3ea7fdfa, SUC_lexicon_tags_26 },
    { 0x79a7fdfb, SUC_lexicon_tags_32 },
    { 0xcac7fdf8, SUC_lexicon_tags_6 },
    { 0xec07fdfa, SUC_lexicon_tags_10 },
    { 0xf8e7fdfc, SUC_lexicon_tags_47 },
    { 0xe5e7fdfd, SUC_lexicon_tags_49 },
    { 0x4587fe00, SUC_lexicon_tags_2 },
    { 0xcc87fe00, SUC_lexicon_tags_11 },
    { 0xdc67fe02, SUC_lexicon_tags_65 },
    { 0x467fe02, SUC_lexicon_tags_33 },
    { 0x49e7fe04, SUC_lexicon_tags_10 },
    { 0xd6a7fe05, SUC_lexicon_tags_2 },
    { 0x46a7fe02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x28a7fe08, SUC_lexicon_tags_17 },
    { 0x22e7fe09, SUC_lexicon_tags_63 },
    { 0x2447fe0a, SUC_lexicon_tags_16 },
    { 0x8d27fe0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6a7fe0d, SUC_lexicon_tags_43 },
    { 0x2847fe0e, SUC_lexicon_tags_12 },
    { 0x18a7fe0f, SUC_lexicon_tags_1 },
    { 0x42a7fe10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c7fe14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b67fe16, SUC_lexicon_tags_28 },
    { 0xb2c7fe17, SUC_lexicon_tags_15 },
    { 0xf987fe18, SUC_lexicon_tags_7 },
    { 0xa2c7fe16, SUC_lexicon_tags_16 },
    { 0x5327fe1a, SUC_lexicon_tags_37 },
    { 0x107fe1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a7fe1e, SUC_lexicon_tags_37 },
    { 0xd2c7fe1e, SUC_lexicon_tags_37 },
    { 0x5607fe1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac7fe24, SUC_lexicon_tags_9 },
    { 0x2e07fe25, SUC_lexicon_tags_63 },
    { 0x1227fe26, SUC_lexicon_tags_33 },
    { 0xda87fe26, SUC_lexicon_tags_47 },
    { 0xb707fe28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefc7fe2a, SUC_lexicon_tags_1 },
    { 0x9c7fe2a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xeea7fe2d, SUC_lexicon_tags_12 },
    { 0xf667fe2e, SUC_lexicon_tags_15 },
    { 0x34c7fe2f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2047fe31, SUC_lexicon_tags_7 },
    { 0x5e27fe32, SUC_lexicon_tags_11 },
    { 0x2507fe33, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1427fe37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a07fe43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc7fe47, SUC_lexicon_tags_48 },
    { 0xd8c7fe48, SUC_lexicon_tags_37 },
    { 0xdfc7fe49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6a7fe4b, SUC_lexicon_tags_23 },
    { 0x407fe4b, SUC_lexicon_tags_55 },
    { 0x9f87fe4b, SUC_lexicon_tags_37 },
    { 0x49c7fe4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c67fe54, SUC_lexicon_tags_10 },
    { 0xbec7fe55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x28e7fe57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de7fe5a, SUC_lexicon_tags_26 },
    { 0xae87fe5b, SUC_lexicon_tags_11 },
    { 0xe747fe5b, SUC_lexicon_tags_5 },
    { 0xf567fe5d, SUC_lexicon_tags_17 },
    { 0x4687fe5a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb7a7fe60, SUC_lexicon_tags_83 },
    { 0x4347fe61, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x12a7fe63, SUC_lexicon_tags_62 },
    { 0x85c7fe64, SUC_lexicon_tags_63 },
    { 0x5747fe64, SUC_lexicon_tags_17 },
    { 0xdd47fe66, SUC_lexicon_tags_17 },
    { 0xbde7fe67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4687fe69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d27fe6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae7fe71, SUC_lexicon_tags_26 },
    { 0xcd07fe72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4047fe75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd687fe79, SUC_lexicon_tags_23 },
    { 0x45e7fe7a, SUC_lexicon_tags_37 },
    { 0x19e7fe7b, SUC_lexicon_tags_4 },
    { 0x62a7fe79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a7fe7e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfe87fe80, SUC_lexicon_tags_27 },
    { 0x6f47fe80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6367fe84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8067fe87, SUC_lexicon_tags_7 },
    { 0x7c67fe88, SUC_lexicon_tags_561 },
    { 0x3c47fe88, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4527fe8d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x8aa7fe8f, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e87fe93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb067fe96, SUC_lexicon_tags_23 },
    { 0xa87fe97, SUC_lexicon_tags_15 },
    { 0xd4c7fe97, SUC_lexicon_tags_15 },
    { 0x6027fe99, SUC_lexicon_tags_2 },
    { 0x47a7fe99, SUC_lexicon_tags_68 },
    { 0x9ec7fe9a, SUC_lexicon_tags_7 },
    { 0xbae7fe99, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c7fe9f, SUC_lexicon_tags_264 },
    { 0x2007fe9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1347fea2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5767feae, SUC_lexicon_tags_9 },
    { 0xfb27feaf, SUC_lexicon_tags_0 },
    { 0xa2e7feb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb227feb4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xed27feb7, SUC_lexicon_tags_39 },
    { 0xe927feb7, SUC_lexicon_tags_25 },
    { 0xd287feb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0e7febb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x607febe, SUC_lexicon_tags_37 },
    { 0x61c7febe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x967fec1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5e07fec3, SUC_lexicon_tags_7 },
    { 0xd387fec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc67fec8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x287fecb, SUC_lexicon_tags_2 },
    { 0x4667fecc, SUC_lexicon_tags_7 },
    { 0x7c67fecd, SUC_lexicon_tags_7 },
    { 0xa4a7fecd, SUC_lexicon_tags_28 },
    { 0xe187fecb, SUC_lexicon_tags_18 },
    { 0x72c7fecf, SUC_lexicon_tags_37 },
    { 0xbdc7fed1, SUC_lexicon_tags_10 },
    { 0x8087fece, SUC_lexicon_tags_1 },
    { 0xcac7fecb, SUC_lexicon_tags_27 },
    { 0x8ba7fecc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8a87fed6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e7fed9, SUC_lexicon_tags_45 },
    { 0x84a7fed9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x807fedc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c47fee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd627fee8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2d87feea, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc167feee, SUC_lexicon_tags_46 },
    { 0x6027feee, SUC_lexicon_tags_37 },
    { 0xdf67feee, SUC_lexicon_tags_1 },
    { 0xdf27fef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc727fef5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2127fef8, SUC_lexicon_tags_49 },
    { 0x33e7fef9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x43c7fefb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa647fefd, SUC_lexicon_tags_37 },
    { 0x33e7fefe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6547ff01, SUC_lexicon_tags_28 },
    { 0x74e7ff01, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xce47ff04, SUC_lexicon_tags_29 },
    { 0x9267ff05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7207ff0a, SUC_lexicon_tags_45 },
    { 0x1b47ff0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a7ff0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c7ff11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c7ff15, SUC_lexicon_tags_14 },
    { 0xa4a7ff15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee7ff1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2407ff1f, SUC_lexicon_tags_60 },
    { 0xf547ff20, SUC_lexicon_tags_17 },
    { 0x4547ff21, SUC_lexicon_tags_2 },
    { 0x4427ff22, SUC_lexicon_tags_38 },
    { 0xf167ff23, SUC_lexicon_tags_0 },
    { 0xa8c7ff22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee87ff28, SUC_lexicon_tags_30 },
    { 0xde27ff29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3767ff2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7ff30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6767ff32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea7ff37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5f87ff39, SUC_lexicon_tags_25 },
    { 0xd907ff3a, SUC_lexicon_tags_0 },
    { 0xbc47ff3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c7ff42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b87ff49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1607ff4b, SUC_lexicon_tags_25 },
    { 0xed47ff4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcc87ff4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5267ff50, SUC_lexicon_tags_57 },
    { 0x2a87ff50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f67ff54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a07ff57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc207ff5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf827ff5f, SUC_lexicon_tags_15 },
    { 0xcfe7ff60, SUC_lexicon_tags_56 },
    { 0x56a7ff61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fa7ff63, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a7ff6a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a7ff6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba7ff70, SUC_lexicon_tags_26 },
    { 0x2a67ff71, SUC_lexicon_tags_7 },
    { 0x2867ff71, SUC_lexicon_tags_2 },
    { 0xbe07ff73, SUC_lexicon_tags_14 },
    { 0xfd47ff74, SUC_lexicon_tags_63 },
    { 0x8d07ff72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf727ff77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c67ff79, SUC_lexicon_tags_17 },
    { 0xf847ff7a, SUC_lexicon_tags_1 },
    { 0x1467ff7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5e7ff7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b87ff82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c7ff86, SUC_lexicon_tags_28 },
    { 0x1967ff87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf87ff8e, SUC_lexicon_tags_11 },
    { 0xcb67ff8f, SUC_lexicon_tags_80 },
    { 0x77a7ff90, SUC_lexicon_tags_6 },
    { 0x9fa7ff90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe427ff94, SUC_lexicon_tags_15 },
    { 0x8e67ff94, SUC_lexicon_tags_79 },
    { 0x82e7ff95, SUC_lexicon_tags_7 },
    { 0x41a7ff97, SUC_lexicon_tags_13 },
    { 0xa407ff98, SUC_lexicon_tags_31 },
    { 0xc807ff99, SUC_lexicon_tags_48 },
    { 0x66e7ff9a, SUC_lexicon_tags_1 },
    { 0xe167ff9b, SUC_lexicon_tags_25 },
    { 0xd747ff99, SUC_lexicon_tags_19 },
    { 0x4e47ff9d, SUC_lexicon_tags_26 },
    { 0x3367ff9e, SUC_lexicon_tags_45 },
    { 0x1a47ff9b, SUC_lexicon_tags_34 },
    { 0x8b07ffa0, SUC_lexicon_tags_87 },
    { 0x20e7ffa1, SUC_lexicon_tags_2 },
    { 0x25a7ffa2, SUC_lexicon_tags_7 },
    { 0xb567ffa1, SUC_lexicon_tags_235 },
    { 0xd1a7ff9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b47ffa7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x30e7ffa9, SUC_lexicon_tags_17 },
    { 0x9a67ffaa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc7ffaf, SUC_lexicon_tags_17 },
    { 0xc1e7ffb0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31a7ffb2, SUC_lexicon_tags_17 },
    { 0x6827ffb2, SUC_lexicon_tags_0 },
    { 0xc187ffb4, SUC_lexicon_tags_37 },
    { 0x39a7ffb4, SUC_lexicon_tags_2 },
    { 0xbb27ffb5, SUC_lexicon_tags_39 },
    { 0x3727ffb7, SUC_lexicon_tags_29 },
    { 0xf2c7ffb8, SUC_lexicon_tags_21 },
    { 0x20e7ffb9, SUC_lexicon_tags_19 },
    { 0xcdc7ffb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8507ffbc, SUC_lexicon_tags_27 },
    { 0x6687ffbc, SUC_lexicon_tags_10 },
    { 0x20a7ffbc, SUC_lexicon_tags_23 },
    { 0xeaa7ffbc, SUC_lexicon_tags_23 },
    { 0x79a7ffc0, SUC_lexicon_tags_46 },
    { 0xf6e7ffc0, SUC_lexicon_tags_47 },
    { 0x8767ffc2, SUC_lexicon_tags_46 },
    { 0xa207ffc3, SUC_lexicon_tags_27 },
    { 0x7967ffc4, SUC_lexicon_tags_1 },
    { 0x1a27ffc0, SUC_lexicon_tags_33 },
    { 0x7aa7ffbd, SUC_lexicon_tags_12 },
    { 0xd3a7ffc7, SUC_lexicon_tags_33 },
    { 0x3be7ffc8, SUC_lexicon_tags_43 },
    { 0xfce7ffc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa827ffcc, SUC_lexicon_tags_27 },
    { 0xa767ffcd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a47ffd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb687ffd4, SUC_lexicon_tags_2 },
    { 0x5227ffd4, SUC_lexicon_tags_25 },
    { 0xb7e7ffd6, SUC_lexicon_tags_23 },
    { 0x23a7ffd6, SUC_lexicon_tags_14 },
    { 0x3147ffd8, SUC_lexicon_tags_7 },
    { 0xaea7ffd9, SUC_lexicon_tags_15 },
    { 0x2787ffda, SUC_lexicon_tags_15 },
    { 0xaec7ffda, SUC_lexicon_tags_6 },
    { 0x5887ffd4, SUC_lexicon_tags_27 },
    { 0x56e7ffdd, SUC_lexicon_tags_6 },
    { 0xf5e7ffde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a27ffe6, SUC_lexicon_tags_7 },
    { 0x4387ffe7, SUC_lexicon_tags_11 },
    { 0x5b47ffe7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca7ffec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf87fff0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c7fff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55e7fff5, SUC_lexicon_tags_54 },
    { 0x9887fff6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b87fff9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc2c7fffb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf567fffd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93680004, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e480009, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa8000c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e480014, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf080018, SUC_lexicon_tags_2 },
    { 0x7b280019, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a48001b, SUC_lexicon_tags_7 },
    { 0xb48001b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaee8001e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x77280020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18e80022, SUC_lexicon_tags_11 },
    { 0x35280023, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c280025, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a80028, SUC_lexicon_tags_2 },
    { 0x71880028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd48002d, SUC_lexicon_tags_47 },
    { 0xb8c8002d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd8080030, SUC_lexicon_tags_27 },
    { 0x47080031, SUC_lexicon_tags_37 },
    { 0xd680031, SUC_lexicon_tags_26 },
    { 0xd7c80033, SUC_lexicon_tags_38 },
    { 0x2fe80031, SUC_lexicon_tags_37 },
    { 0x98680035, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89c80037, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16c80039, SUC_lexicon_tags_62 },
    { 0x9b080039, SUC_lexicon_tags_47 },
    { 0x43e8003b, SUC_lexicon_tags_15 },
    { 0xb008003b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9948003f, SUC_lexicon_tags_8 },
    { 0x88280040, SUC_lexicon_tags_3 },
    { 0xbc480041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4880043, SUC_lexicon_tags_55 },
    { 0xca680044, SUC_lexicon_tags_28 },
    { 0xff280045, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7880047, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1728004a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b68004c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da80053, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67280055, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51080057, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf508005a, SUC_lexicon_tags_79 },
    { 0xb408005b, SUC_lexicon_tags_12 },
    { 0xfbe8005c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8c8005e, SUC_lexicon_tags_15 },
    { 0x7d48005f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87480063, SUC_lexicon_tags_37 },
    { 0x3de80063, SUC_lexicon_tags_1 },
    { 0xdb480063, SUC_lexicon_tags_30 },
    { 0xaf280066, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7680070, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd080076, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f880078, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3108007a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde88007c, SUC_lexicon_tags_19 },
    { 0x8908007d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2080081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8080084, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4c80086, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b680088, SUC_lexicon_tags_70 },
    { 0x75880089, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c8008b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24680091, SUC_lexicon_tags_34 },
    { 0x40a80092, SUC_lexicon_tags_12 },
    { 0xb2e80093, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6280099, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x748009b, SUC_lexicon_tags_32 },
    { 0xaae8009c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0800a0, SUC_lexicon_tags_31 },
    { 0xe36800a1, SUC_lexicon_tags_35 },
    { 0xe9e800a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcc800a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e800a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a800a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2800b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8800b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28800ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c800be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece800c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x202800c7, SUC_lexicon_tags_29 },
    { 0x9be800c8, SUC_lexicon_tags_43 },
    { 0x2f0800c9, SUC_lexicon_tags_2 },
    { 0xec800c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x690800cc, SUC_lexicon_tags_47 },
    { 0x428800cc, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0xefc800cf, SUC_lexicon_tags_17 },
    { 0x8f0800d0, SUC_lexicon_tags_81 },
    { 0xc40800d1, SUC_lexicon_tags_52 },
    { 0x4c6800d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc800d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a800df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b4800e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe68800e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40c800e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x75c800e7, SUC_lexicon_tags_6 },
    { 0x2be800e7, SUC_lexicon_tags_19 },
    { 0xae2800e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e800ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852800f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x984800f4, SUC_lexicon_tags_56 },
    { 0xde2800f4, SUC_lexicon_tags_26 },
    { 0xd90800f5, SUC_lexicon_tags_12 },
    { 0xff4800f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x404800f9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x19e800fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9c800fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x76080100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe80103, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf8c80105, SUC_lexicon_tags_17 },
    { 0x1dc80105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c80109, SUC_lexicon_tags_11 },
    { 0xcfa8010a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a8010e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c80113, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x14680115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5e80117, SUC_lexicon_tags_38 },
    { 0xd8880118, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a8011d, SUC_lexicon_tags_11 },
    { 0x4e08011d, SUC_lexicon_tags_37 },
    { 0xc1a8011f, SUC_lexicon_tags_6 },
    { 0xe4680120, SUC_lexicon_tags_11 },
    { 0xbf480121, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa280123, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfe80125, SUC_lexicon_tags_11 },
    { 0x1f080125, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba80129, SUC_lexicon_tags_1 },
    { 0x3d48012a, SUC_lexicon_tags_12 },
    { 0x1c88012b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe8012f, SUC_lexicon_tags_27 },
    { 0x3680130, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91880136, SUC_lexicon_tags_11 },
    { 0xa4280137, SUC_lexicon_tags_11 },
    { 0x1e80138, SUC_lexicon_tags_47 },
    { 0xd8a80139, SUC_lexicon_tags_47 },
    { 0x8008013a, SUC_lexicon_tags_15 },
    { 0x5188013b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f88013d, SUC_lexicon_tags_25 },
    { 0xcba8013e, SUC_lexicon_tags_12 },
    { 0xe4a8013e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f480146, SUC_lexicon_tags_49 },
    { 0xa4080147, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70680149, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc448014c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca80151, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c80157, SUC_lexicon_tags_26 },
    { 0xd280158, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2548015b, SUC_lexicon_tags_12 },
    { 0x7768015c, SUC_lexicon_tags_7 },
    { 0xf968015c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c80161, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe4080163, SUC_lexicon_tags_18 },
    { 0xa6080163, SUC_lexicon_tags_19 },
    { 0x5a680165, SUC_lexicon_tags_15 },
    { 0x25080165, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c8016b, SUC_lexicon_tags_117 },
    { 0x1068016c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37a8016e, SUC_lexicon_tags_63 },
    { 0xa148016f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82480172, SUC_lexicon_tags_17 },
    { 0x8e680172, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x22680176, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc1080178, SUC_lexicon_tags_68 },
    { 0xbf480178, SUC_lexicon_tags_1 },
    { 0x9a8017a, SUC_lexicon_tags_14 },
    { 0xace8017b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5408017e, SUC_lexicon_tags_37 },
    { 0xb688017f, SUC_lexicon_tags_15 },
    { 0x19a8017f, SUC_lexicon_tags_15 },
    { 0x44e80181, SUC_lexicon_tags_43 },
    { 0xb0680182, SUC_lexicon_tags_19 },
    { 0x55c8017e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72280187, SUC_lexicon_tags_57 },
    { 0xc6c80187, SUC_lexicon_tags_65 },
    { 0xde80189, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe8a8018b, SUC_lexicon_tags_47 },
    { 0xc288018b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0680191, SUC_lexicon_tags_33 },
    { 0x8c80192, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48080194, SUC_lexicon_tags_15 },
    { 0xd4080194, SUC_lexicon_tags_37 },
    { 0xb8c80196, SUC_lexicon_tags_81 },
    { 0x1ee80196, SUC_lexicon_tags_6 },
    { 0x2b480198, SUC_lexicon_tags_25 },
    { 0x7cc80198, SUC_lexicon_tags_27 },
    { 0x44c8019a, SUC_lexicon_tags_12 },
    { 0xd2080199, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476801a1, SUC_lexicon_tags_14 },
    { 0x8fe801a1, SUC_lexicon_tags_63 },
    { 0xe18801a1, SUC_lexicon_tags_19 },
    { 0x5fc801a4, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126801ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x806801af, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5e8801b1, SUC_lexicon_tags_11 },
    { 0x3b6801b2, SUC_lexicon_tags_17 },
    { 0x92801b3, SUC_lexicon_tags_66 },
    { 0x742801b3, SUC_lexicon_tags_14 },
    { 0x6d0801b4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8801bb, SUC_lexicon_tags_11 },
    { 0x70e801bc, SUC_lexicon_tags_15 },
    { 0xa26801bd, SUC_lexicon_tags_87 },
    { 0x6a8801be, SUC_lexicon_tags_17 },
    { 0x1ea801bf, SUC_lexicon_tags_29 },
    { 0x2c0801bf, SUC_lexicon_tags_9 },
    { 0xe76801bc, SUC_lexicon_tags_39 },
    { 0x16e801c2, SUC_lexicon_tags_15 },
    { 0x108801c3, SUC_lexicon_tags_38 },
    { 0x71c801c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e8801c6, SUC_lexicon_tags_27 },
    { 0x2d4801c7, SUC_lexicon_tags_49 },
    { 0xcf6801c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396801d1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70801d6, SUC_lexicon_tags_12 },
    { 0x304801d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8801db, SUC_lexicon_tags_11 },
    { 0xa7e801dc, SUC_lexicon_tags_8 },
    { 0x742801dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c801e0, SUC_lexicon_tags_15 },
    { 0xc54801e1, SUC_lexicon_tags_37 },
    { 0x686801e1, SUC_lexicon_tags_46 },
    { 0xe7c801e1, SUC_lexicon_tags_25 },
    { 0x112801e0, SUC_lexicon_tags_33 },
    { 0xf1e801e5, SUC_lexicon_tags_43 },
    { 0x604801e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be801ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c801ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44801f4, SUC_lexicon_tags_85 },
    { 0x72e801f5, SUC_lexicon_tags_15 },
    { 0x346801f6, SUC_lexicon_tags_60 },
    { 0x788801f7, SUC_lexicon_tags_12 },
    { 0xc1a801f8, SUC_lexicon_tags_7 },
    { 0x146801f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x652801fb, SUC_lexicon_tags_14 },
    { 0x8be801fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4680202, SUC_lexicon_tags_12 },
    { 0xa1680203, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc80208, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac80213, SUC_lexicon_tags_16 },
    { 0x59880214, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x21080217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7680219, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xac48021c, SUC_lexicon_tags_0 },
    { 0x9848021c, SUC_lexicon_tags_7 },
    { 0xc828021d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf880222, SUC_lexicon_tags_27 },
    { 0x1b680223, SUC_lexicon_tags_15 },
    { 0xd280224, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc80228, SUC_lexicon_tags_38 },
    { 0xfa480229, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c8022b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe268022f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34480234, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7668023f, SUC_lexicon_tags_2 },
    { 0xce880240, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea80244, SUC_lexicon_tags_38 },
    { 0x21a80245, SUC_lexicon_tags_6 },
    { 0xde480246, SUC_lexicon_tags_7 },
    { 0x97480244, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d080249, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71880250, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19c80252, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f280259, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9ea8025b, SUC_lexicon_tags_17 },
    { 0x1288025c, SUC_lexicon_tags_7 },
    { 0x6c28025c, SUC_lexicon_tags_8 },
    { 0xd328025d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb0880260, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60a80262, SUC_lexicon_tags_17 },
    { 0x2ba80263, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8cc80265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c8026a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8026e, SUC_lexicon_tags_15 },
    { 0x7f08026e, SUC_lexicon_tags_1 },
    { 0x5e480270, SUC_lexicon_tags_15 },
    { 0x34a8026e, SUC_lexicon_tags_231 },
    { 0x22a80272, SUC_lexicon_tags_15 },
    { 0x84a80270, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd0a80275, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1680279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9208027c, SUC_lexicon_tags_4 },
    { 0x19a8027d, SUC_lexicon_tags_7 },
    { 0xbc68027e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3680281, SUC_lexicon_tags_17 },
    { 0xc6880281, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19680285, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9280288, SUC_lexicon_tags_17 },
    { 0x43c80288, SUC_lexicon_tags_15 },
    { 0x16a8028a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6da8028c, SUC_lexicon_tags_28 },
    { 0xa688028d, SUC_lexicon_tags_11 },
    { 0x8868028d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc5080290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c80294, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e80299, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e48029c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6802a2, SUC_lexicon_tags_18 },
    { 0xfe8802a3, SUC_lexicon_tags_17 },
    { 0x8cc802a4, SUC_lexicon_tags_23 },
    { 0x938802a5, SUC_lexicon_tags_48 },
    { 0x292802a6, SUC_lexicon_tags_39 },
    { 0x68a802a7, SUC_lexicon_tags_33 },
    { 0x8c802a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x270802aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x68802ad, SUC_lexicon_tags_37 },
    { 0x578802ae, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xf6802b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee802b4, SUC_lexicon_tags_31 },
    { 0x48e802b5, SUC_lexicon_tags_17 },
    { 0xf28802b4, SUC_lexicon_tags_37 },
    { 0x9e8802b7, SUC_lexicon_tags_13 },
    { 0x504802b8, SUC_lexicon_tags_14 },
    { 0x958802b9, SUC_lexicon_tags_37 },
    { 0xcf0802ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42802c0, SUC_lexicon_tags_19 },
    { 0x622802c1, SUC_lexicon_tags_9 },
    { 0x908802c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e802c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50802ca, SUC_lexicon_tags_1 },
    { 0xdf2802cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a802cf, SUC_lexicon_tags_47 },
    { 0x416802d0, SUC_lexicon_tags_19 },
    { 0x286802cf, SUC_lexicon_tags_17 },
    { 0x2d4802d2, SUC_lexicon_tags_103 },
    { 0x306802d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbac802d5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xabe802d7, SUC_lexicon_tags_11 },
    { 0x1ba802d8, SUC_lexicon_tags_20 },
    { 0xda8802d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27c802db, SUC_lexicon_tags_25 },
    { 0x3d6802dc, SUC_lexicon_tags_15 },
    { 0x310802dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc802e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x724802e5, SUC_lexicon_tags_19 },
    { 0xdac802e6, SUC_lexicon_tags_15 },
    { 0x254802e5, SUC_lexicon_tags_2 },
    { 0xb46802e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c802ef, SUC_lexicon_tags_15 },
    { 0xd1a802f0, SUC_lexicon_tags_30 },
    { 0x986802f1, SUC_lexicon_tags_15 },
    { 0x638802f1, SUC_lexicon_tags_33 },
    { 0x34a802f3, SUC_lexicon_tags_16 },
    { 0x46802f4, SUC_lexicon_tags_7 },
    { 0xc44802f4, SUC_lexicon_tags_287 },
    { 0, NULL },
    { 0x508802f7, SUC_lexicon_tags_15 },
    { 0x11c802f8, SUC_lexicon_tags_28 },
    { 0x69a802f9, SUC_lexicon_tags_6 },
    { 0xb24802f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd70802fc, SUC_lexicon_tags_27 },
    { 0xf74802fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa80302, SUC_lexicon_tags_21 },
    { 0x63e80303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a8030d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b880313, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13880315, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ca80317, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7e80319, SUC_lexicon_tags_19 },
    { 0xb2080319, SUC_lexicon_tags_47 },
    { 0x6468031b, SUC_lexicon_tags_29 },
    { 0xfb68031c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de8031f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x68080321, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc80325, SUC_lexicon_tags_2 },
    { 0xff280326, SUC_lexicon_tags_15 },
    { 0xfd080326, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa68032b, SUC_lexicon_tags_25 },
    { 0x89a8032b, SUC_lexicon_tags_1 },
    { 0xb948032b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e80331, SUC_lexicon_tags_25 },
    { 0x68480332, SUC_lexicon_tags_8 },
    { 0x73c80331, SUC_lexicon_tags_19 },
    { 0xfce80334, SUC_lexicon_tags_15 },
    { 0xa80335, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3e80337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88033c, SUC_lexicon_tags_19 },
    { 0x3c8033d, SUC_lexicon_tags_38 },
    { 0x5e8033d, SUC_lexicon_tags_25 },
    { 0xa728033f, SUC_lexicon_tags_45 },
    { 0xbbe8033d, SUC_lexicon_tags_15 },
    { 0x6ac80341, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17080343, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf8e80345, SUC_lexicon_tags_55 },
    { 0x26080346, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdf680348, SUC_lexicon_tags_12 },
    { 0xc4e80349, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32e8034b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb48034e, SUC_lexicon_tags_11 },
    { 0x4a28034f, SUC_lexicon_tags_1 },
    { 0x6aa8034e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47480352, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12280356, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x22080358, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xada8035a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfec8035c, SUC_lexicon_tags_7 },
    { 0x13c8035c, SUC_lexicon_tags_25 },
    { 0xcfc8035d, SUC_lexicon_tags_17 },
    { 0x9d28035f, SUC_lexicon_tags_2 },
    { 0xd7480360, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e080366, SUC_lexicon_tags_47 },
    { 0xd5480367, SUC_lexicon_tags_4 },
    { 0x47880368, SUC_lexicon_tags_11 },
    { 0x83080367, SUC_lexicon_tags_14 },
    { 0x82e8036a, SUC_lexicon_tags_17 },
    { 0xdf280369, SUC_lexicon_tags_48 },
    { 0x60a80369, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e28036f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75080371, SUC_lexicon_tags_25 },
    { 0x9de80372, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e80375, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e80379, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e8037d, SUC_lexicon_tags_27 },
    { 0x8a08037d, SUC_lexicon_tags_2 },
    { 0x35c8037f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x67680381, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9480386, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25880388, SUC_lexicon_tags_25 },
    { 0xa7680389, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd688038c, SUC_lexicon_tags_23 },
    { 0xad88038d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e88038f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c80395, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca80398, SUC_lexicon_tags_38 },
    { 0x1ea80399, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4248039b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4408039d, SUC_lexicon_tags_282 },
    { 0x3e68039e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8803a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdae803a5, SUC_lexicon_tags_253 },
    { 0x3b6803a6, SUC_lexicon_tags_23 },
    { 0xa16803a5, SUC_lexicon_tags_12 },
    { 0x226803a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78803ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f8803ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda8803af, SUC_lexicon_tags_7 },
    { 0x28803b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca803b6, SUC_lexicon_tags_1 },
    { 0x73c803b6, SUC_lexicon_tags_17 },
    { 0xffe803b8, SUC_lexicon_tags_47 },
    { 0x3f2803b9, SUC_lexicon_tags_23 },
    { 0x2a803b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x962803bc, SUC_lexicon_tags_37 },
    { 0x7ea803bd, SUC_lexicon_tags_37 },
    { 0xeb0803bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6803c5, SUC_lexicon_tags_15 },
    { 0x438803c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x428803c8, SUC_lexicon_tags_9 },
    { 0x350803c9, SUC_lexicon_tags_10 },
    { 0xbf8803c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4803cd, SUC_lexicon_tags_21 },
    { 0x6d8803ce, SUC_lexicon_tags_0 },
    { 0xc72803cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba803d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e803db, SUC_lexicon_tags_37 },
    { 0x2be803dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc803e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20803ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c803ed, SUC_lexicon_tags_14 },
    { 0x254803ee, SUC_lexicon_tags_53 },
    { 0xa34803ee, SUC_lexicon_tags_25 },
    { 0x542803ee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50803f3, SUC_lexicon_tags_1 },
    { 0x548803f4, SUC_lexicon_tags_15 },
    { 0xd60803f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5aa803f7, SUC_lexicon_tags_32 },
    { 0x9b4803f7, SUC_lexicon_tags_31 },
    { 0xd56803f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c803fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x946803ff, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e680408, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2a8040a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7488040c, SUC_lexicon_tags_29 },
    { 0x8fc8040d, SUC_lexicon_tags_55 },
    { 0x4728040e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd080411, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaee80413, SUC_lexicon_tags_17 },
    { 0xdde80414, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15880418, SUC_lexicon_tags_11 },
    { 0xf8280419, SUC_lexicon_tags_32 },
    { 0xf2080419, SUC_lexicon_tags_37 },
    { 0x4ca8041b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85880420, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35080424, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61e80426, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd168042b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d08042e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a80432, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9680435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31080438, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7428043f, SUC_lexicon_tags_37 },
    { 0x42e80440, SUC_lexicon_tags_18 },
    { 0x6ee80441, SUC_lexicon_tags_22 },
    { 0xfa280442, SUC_lexicon_tags_37 },
    { 0x80e80442, SUC_lexicon_tags_12 },
    { 0x13480443, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf88044b, SUC_lexicon_tags_15 },
    { 0x4fa8044b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480451, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c8045a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d680460, SUC_lexicon_tags_2 },
    { 0x8e280461, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e80466, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd48046a, SUC_lexicon_tags_19 },
    { 0xcf28046a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28046e, SUC_lexicon_tags_63 },
    { 0xc708046f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da80474, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x43c80476, SUC_lexicon_tags_1 },
    { 0x77280477, SUC_lexicon_tags_5 },
    { 0xa8880478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de8047c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b08047f, SUC_lexicon_tags_26 },
    { 0x82280480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4c80482, SUC_lexicon_tags_49 },
    { 0xd7480483, SUC_lexicon_tags_17 },
    { 0xca480484, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea28048c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de80490, SUC_lexicon_tags_43 },
    { 0x49280491, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5680493, SUC_lexicon_tags_17 },
    { 0xab280494, SUC_lexicon_tags_82 },
    { 0x50080495, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf680497, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7048049d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8804a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a804a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a804aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb52804ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc804af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a804b2, SUC_lexicon_tags_25 },
    { 0x3a804b3, SUC_lexicon_tags_85 },
    { 0xa5c804b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476804bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c804bf, SUC_lexicon_tags_55 },
    { 0x832804bf, SUC_lexicon_tags_27 },
    { 0x40804c0, SUC_lexicon_tags_30 },
    { 0xcae804c2, SUC_lexicon_tags_17 },
    { 0xfc0804c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240804cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76804d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a804d9, SUC_lexicon_tags_1 },
    { 0xbd6804d9, SUC_lexicon_tags_2 },
    { 0x76e804da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcca804dd, SUC_lexicon_tags_11 },
    { 0xac0804de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0804e1, SUC_lexicon_tags_2 },
    { 0x894804e2, SUC_lexicon_tags_25 },
    { 0x8ba804e3, SUC_lexicon_tags_37 },
    { 0xf18804e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbc804e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e804e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a4804eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56804f4, SUC_lexicon_tags_17 },
    { 0x8f4804f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88804fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64a804fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6804ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1be80501, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1a80503, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x2a80505, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb680514, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9680517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2528051a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1168051c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6280520, SUC_lexicon_tags_47 },
    { 0xc9080521, SUC_lexicon_tags_17 },
    { 0x4ec80522, SUC_lexicon_tags_1 },
    { 0xc6c80521, SUC_lexicon_tags_47 },
    { 0x47a80522, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9368052b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xee68052d, SUC_lexicon_tags_63 },
    { 0x3bc8052e, SUC_lexicon_tags_11 },
    { 0x98e8052d, SUC_lexicon_tags_17 },
    { 0x84480530, SUC_lexicon_tags_33 },
    { 0xab68052d, SUC_lexicon_tags_7 },
    { 0x1de80532, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcfe80534, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea8053d, SUC_lexicon_tags_30 },
    { 0xffc8053e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21e80540, SUC_lexicon_tags_29 },
    { 0xc2280541, SUC_lexicon_tags_11 },
    { 0x3cc80541, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce80545, SUC_lexicon_tags_0 },
    { 0x60480546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6680548, SUC_lexicon_tags_23 },
    { 0xd8880549, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb0c8054b, SUC_lexicon_tags_27 },
    { 0x4b88054c, SUC_lexicon_tags_1 },
    { 0x25a8054d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f080552, SUC_lexicon_tags_15 },
    { 0x37e80553, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b280556, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf688055a, SUC_lexicon_tags_12 },
    { 0x7f48055b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a8055e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a80563, SUC_lexicon_tags_49 },
    { 0xf7280564, SUC_lexicon_tags_65 },
    { 0x6fa80564, SUC_lexicon_tags_2 },
    { 0x49c80566, SUC_lexicon_tags_7 },
    { 0x51a80563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a8056a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f08056e, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36880573, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3148057a, SUC_lexicon_tags_56 },
    { 0xb08057b, SUC_lexicon_tags_11 },
    { 0x41e8057b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8108057e, SUC_lexicon_tags_7 },
    { 0x9f08057f, SUC_lexicon_tags_14 },
    { 0x7108057e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x83080582, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a80587, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e68058a, SUC_lexicon_tags_27 },
    { 0x8408058a, SUC_lexicon_tags_11 },
    { 0xdc88058a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be80590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7880592, SUC_lexicon_tags_17 },
    { 0x34880593, SUC_lexicon_tags_3 },
    { 0x4ce80592, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a80597, SUC_lexicon_tags_0 },
    { 0xb2280597, SUC_lexicon_tags_0 },
    { 0x1f080599, SUC_lexicon_tags_37 },
    { 0xf3880598, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb608059e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x500805a0, SUC_lexicon_tags_25 },
    { 0xf6c805a1, SUC_lexicon_tags_11 },
    { 0x406805a1, SUC_lexicon_tags_23 },
    { 0x58805a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352805a8, SUC_lexicon_tags_55 },
    { 0xb70805a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18805ab, SUC_lexicon_tags_17 },
    { 0xcf2805ac, SUC_lexicon_tags_25 },
    { 0x7b0805ad, SUC_lexicon_tags_37 },
    { 0x678805ac, SUC_lexicon_tags_30 },
    { 0xcda805af, SUC_lexicon_tags_29 },
    { 0x7f6805b0, SUC_lexicon_tags_65 },
    { 0x212805b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x822805b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152805b7, SUC_lexicon_tags_66 },
    { 0xccc805b8, SUC_lexicon_tags_0 },
    { 0xa5c805b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e6805bb, SUC_lexicon_tags_14 },
    { 0x3e2805bc, SUC_lexicon_tags_0 },
    { 0x7ca805bd, SUC_lexicon_tags_65 },
    { 0xef4805be, SUC_lexicon_tags_37 },
    { 0x6fa805bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa805ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c805ce, SUC_lexicon_tags_49 },
    { 0x2ba805cf, SUC_lexicon_tags_28 },
    { 0xb64805d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x694805d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4805d6, SUC_lexicon_tags_49 },
    { 0xe90805d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c805db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe16805dd, SUC_lexicon_tags_15 },
    { 0x650805dd, SUC_lexicon_tags_17 },
    { 0x6b8805df, SUC_lexicon_tags_18 },
    { 0x314805df, SUC_lexicon_tags_14 },
    { 0x8ca805e1, SUC_lexicon_tags_1 },
    { 0x880805e2, SUC_lexicon_tags_28 },
    { 0x7fc805e3, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64805e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8805ec, SUC_lexicon_tags_46 },
    { 0x40a805ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc805fd, SUC_lexicon_tags_16 },
    { 0xb48805fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11080600, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a280603, SUC_lexicon_tags_27 },
    { 0xada80604, SUC_lexicon_tags_9 },
    { 0xa0680604, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e80608, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1788060e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7880612, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4e80614, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e80618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa588061a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x368061c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8528061e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a80621, SUC_lexicon_tags_38 },
    { 0x50080622, SUC_lexicon_tags_77 },
    { 0x64a80623, SUC_lexicon_tags_11 },
    { 0x34080624, SUC_lexicon_tags_28 },
    { 0xfae80623, SUC_lexicon_tags_87 },
    { 0x7c280622, SUC_lexicon_tags_15 },
    { 0x5480627, SUC_lexicon_tags_17 },
    { 0x31080627, SUC_lexicon_tags_31 },
    { 0x18c80628, SUC_lexicon_tags_29 },
    { 0x1bc80625, SUC_lexicon_tags_7 },
    { 0x76280626, SUC_lexicon_tags_15 },
    { 0x7108062c, SUC_lexicon_tags_0 },
    { 0x3468062d, SUC_lexicon_tags_39 },
    { 0xd768062e, SUC_lexicon_tags_11 },
    { 0x6bc8062e, SUC_lexicon_tags_37 },
    { 0xf868062c, SUC_lexicon_tags_9 },
    { 0x6ec80630, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e80638, SUC_lexicon_tags_25 },
    { 0xf9e80639, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4508063d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb880641, SUC_lexicon_tags_37 },
    { 0x94480642, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4e880644, SUC_lexicon_tags_27 },
    { 0x44880645, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a80648, SUC_lexicon_tags_11 },
    { 0x43680649, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb968064c, SUC_lexicon_tags_17 },
    { 0x77c8064d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35680651, SUC_lexicon_tags_37 },
    { 0xaa280651, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a80656, SUC_lexicon_tags_6 },
    { 0x5b880657, SUC_lexicon_tags_45 },
    { 0x3dc80657, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc8065c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be80666, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e88066a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6c8066c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71680671, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad080678, SUC_lexicon_tags_10 },
    { 0x7e680678, SUC_lexicon_tags_2 },
    { 0x9748067a, SUC_lexicon_tags_87 },
    { 0x1f880679, SUC_lexicon_tags_21 },
    { 0xa748067b, SUC_lexicon_tags_31 },
    { 0xfc48067c, SUC_lexicon_tags_17 },
    { 0xcce8067e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce80681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e080685, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x40080688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc588068a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2128068c, SUC_lexicon_tags_9 },
    { 0xf048068c, SUC_lexicon_tags_33 },
    { 0xf528068e, SUC_lexicon_tags_77 },
    { 0xe888068f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96680691, SUC_lexicon_tags_30 },
    { 0x30a80691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da80695, SUC_lexicon_tags_6 },
    { 0x72280696, SUC_lexicon_tags_27 },
    { 0xc8c80696, SUC_lexicon_tags_48 },
    { 0xfa880698, SUC_lexicon_tags_6 },
    { 0xca480698, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d08069e, SUC_lexicon_tags_26 },
    { 0x94c8069e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdce806a1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x28a806a3, SUC_lexicon_tags_18 },
    { 0xb8e806a3, SUC_lexicon_tags_1 },
    { 0xa3c806a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x186806a8, SUC_lexicon_tags_23 },
    { 0xa1c806a9, SUC_lexicon_tags_25 },
    { 0x4806a9, SUC_lexicon_tags_14 },
    { 0x754806a9, SUC_lexicon_tags_7 },
    { 0xea8806a8, SUC_lexicon_tags_15 },
    { 0x654806aa, SUC_lexicon_tags_79 },
    { 0xda2806ae, SUC_lexicon_tags_11 },
    { 0x28e806af, SUC_lexicon_tags_7 },
    { 0x7c8806a9, SUC_lexicon_tags_12 },
    { 0x524806b1, SUC_lexicon_tags_43 },
    { 0x4be806b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea806b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca2806b6, SUC_lexicon_tags_0 },
    { 0x484806b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4806ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x92806bd, SUC_lexicon_tags_7 },
    { 0x8c8806be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c806c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd8806c3, SUC_lexicon_tags_17 },
    { 0xa40806c4, SUC_lexicon_tags_61 },
    { 0x348806c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce806c9, SUC_lexicon_tags_55 },
    { 0x952806ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2806cd, SUC_lexicon_tags_56 },
    { 0x920806ce, SUC_lexicon_tags_7 },
    { 0xb40806ce, SUC_lexicon_tags_1 },
    { 0xe04806ce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78806d3, SUC_lexicon_tags_27 },
    { 0x894806d4, SUC_lexicon_tags_14 },
    { 0xa26806d5, SUC_lexicon_tags_23 },
    { 0x840806d5, SUC_lexicon_tags_31 },
    { 0x56a806d7, SUC_lexicon_tags_25 },
    { 0x8c2806d8, SUC_lexicon_tags_63 },
    { 0x580806d9, SUC_lexicon_tags_77 },
    { 0xef6806da, SUC_lexicon_tags_35 },
    { 0xe68806da, SUC_lexicon_tags_47 },
    { 0x204806da, SUC_lexicon_tags_12 },
    { 0x3ee806dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02806e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e806e4, SUC_lexicon_tags_68 },
    { 0x6ba806e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a806e8, SUC_lexicon_tags_39 },
    { 0x8aa806e8, SUC_lexicon_tags_2 },
    { 0x6f2806ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f6806ec, SUC_lexicon_tags_7 },
    { 0x696806ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a806f0, SUC_lexicon_tags_15 },
    { 0x772806f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e806f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x46806f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea806fc, SUC_lexicon_tags_12 },
    { 0xc1c806fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12680701, SUC_lexicon_tags_44 },
    { 0x4d080701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde80706, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd428070c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9888070e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd480713, SUC_lexicon_tags_17 },
    { 0xbb280714, SUC_lexicon_tags_27 },
    { 0x68080715, SUC_lexicon_tags_48 },
    { 0xc3280716, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c80719, SUC_lexicon_tags_49 },
    { 0x4c28071a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b08071d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca80720, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9680724, SUC_lexicon_tags_17 },
    { 0x47680724, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1280728, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f48072d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe280731, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfc80733, SUC_lexicon_tags_7 },
    { 0x34880734, SUC_lexicon_tags_37 },
    { 0x2ee80735, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28073b, SUC_lexicon_tags_8 },
    { 0x3aa8073c, SUC_lexicon_tags_7 },
    { 0x6288073d, SUC_lexicon_tags_7 },
    { 0x37a8073e, SUC_lexicon_tags_25 },
    { 0x20c8073c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6da80741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc80745, SUC_lexicon_tags_37 },
    { 0xe9c80746, SUC_lexicon_tags_0 },
    { 0x5de80747, SUC_lexicon_tags_11 },
    { 0x9880748, SUC_lexicon_tags_6 },
    { 0xd2a80749, SUC_lexicon_tags_37 },
    { 0x21680745, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d28074c, SUC_lexicon_tags_9 },
    { 0xe768074d, SUC_lexicon_tags_29 },
    { 0x2ea8074e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37880755, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f48075c, SUC_lexicon_tags_0 },
    { 0x8d28075d, SUC_lexicon_tags_37 },
    { 0x5bc8075e, SUC_lexicon_tags_25 },
    { 0xce08075e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a80762, SUC_lexicon_tags_7 },
    { 0x61c80763, SUC_lexicon_tags_17 },
    { 0xc7a80763, SUC_lexicon_tags_13 },
    { 0x98a80765, SUC_lexicon_tags_60 },
    { 0x83a80766, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c8076a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e8076d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d080771, SUC_lexicon_tags_63 },
    { 0xb3080771, SUC_lexicon_tags_15 },
    { 0xa3680773, SUC_lexicon_tags_19 },
    { 0xfc280774, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x27480777, SUC_lexicon_tags_25 },
    { 0xf6080777, SUC_lexicon_tags_47 },
    { 0x3e680777, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc80780, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf480785, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47a80787, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbbe80789, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2aa8078b, SUC_lexicon_tags_19 },
    { 0x22e8078c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1708078f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc4880791, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e80794, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13080798, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e88079b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a8079f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb5a807a1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x9c0807a3, SUC_lexicon_tags_38 },
    { 0x58a807a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b4807a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4807a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd2807ab, SUC_lexicon_tags_11 },
    { 0xda807ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x11c807ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19a807b0, SUC_lexicon_tags_39 },
    { 0x6e4807b1, SUC_lexicon_tags_19 },
    { 0x3dc807b2, SUC_lexicon_tags_7 },
    { 0xc88807b2, SUC_lexicon_tags_8 },
    { 0xb6c807b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef8807b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x430807b8, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x70e807ba, SUC_lexicon_tags_0 },
    { 0x15e807bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da807c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8807c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8807cd, SUC_lexicon_tags_14 },
    { 0x462807cd, SUC_lexicon_tags_15 },
    { 0x6de807ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdc4807d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4807d5, SUC_lexicon_tags_1 },
    { 0x8cc807d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c807d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57a807db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e807df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e807e2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e807e5, SUC_lexicon_tags_43 },
    { 0xe3e807e5, SUC_lexicon_tags_17 },
    { 0x86807e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984807ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e807f2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc807f6, SUC_lexicon_tags_7 },
    { 0xaf4807f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852807ff, SUC_lexicon_tags_26 },
    { 0x3a6807ff, SUC_lexicon_tags_33 },
    { 0xe8680801, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f080804, SUC_lexicon_tags_15 },
    { 0xb0e80805, SUC_lexicon_tags_389 },
    { 0xbda80804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e80809, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e8080d, SUC_lexicon_tags_37 },
    { 0xe4c8080e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce80818, SUC_lexicon_tags_7 },
    { 0xd2280819, SUC_lexicon_tags_87 },
    { 0xb788081a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa808081e, SUC_lexicon_tags_37 },
    { 0x9468081f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c280826, SUC_lexicon_tags_7 },
    { 0xb4680827, SUC_lexicon_tags_448 },
    { 0, NULL },
    { 0xf480829, SUC_lexicon_tags_11 },
    { 0xf1680829, SUC_lexicon_tags_7 },
    { 0xe6e8082b, SUC_lexicon_tags_2 },
    { 0x6908082c, SUC_lexicon_tags_4 },
    { 0x3228082d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5e8082f, SUC_lexicon_tags_12 },
    { 0x43680830, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3d880832, SUC_lexicon_tags_47 },
    { 0x3d280833, SUC_lexicon_tags_61 },
    { 0x70680833, SUC_lexicon_tags_21 },
    { 0xd2e80835, SUC_lexicon_tags_17 },
    { 0xe5480835, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3848083b, SUC_lexicon_tags_27 },
    { 0xdb68083c, SUC_lexicon_tags_19 },
    { 0x68a8083d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4748083f, SUC_lexicon_tags_31 },
    { 0xc388083f, SUC_lexicon_tags_40 },
    { 0xa3880840, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a80844, SUC_lexicon_tags_29 },
    { 0x13680845, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41c80847, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1448084e, SUC_lexicon_tags_0 },
    { 0x5c8084f, SUC_lexicon_tags_25 },
    { 0xbf680850, SUC_lexicon_tags_38 },
    { 0x45280851, SUC_lexicon_tags_7 },
    { 0x4e480852, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc880854, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7880856, SUC_lexicon_tags_26 },
    { 0x83a80856, SUC_lexicon_tags_25 },
    { 0x88a80858, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a68085e, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xeaa80860, SUC_lexicon_tags_12 },
    { 0xd5e80861, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe280864, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x24880866, SUC_lexicon_tags_63 },
    { 0xf1880867, SUC_lexicon_tags_23 },
    { 0x6c880866, SUC_lexicon_tags_25 },
    { 0x79080868, SUC_lexicon_tags_14 },
    { 0xafc80869, SUC_lexicon_tags_19 },
    { 0x85a8086b, SUC_lexicon_tags_37 },
    { 0x37280868, SUC_lexicon_tags_3 },
    { 0x2788086d, SUC_lexicon_tags_37 },
    { 0xfa80868, SUC_lexicon_tags_17 },
    { 0x12880869, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6280879, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac08087e, SUC_lexicon_tags_0 },
    { 0x9808087e, SUC_lexicon_tags_1 },
    { 0x9b880880, SUC_lexicon_tags_2 },
    { 0x1308087f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4e480883, SUC_lexicon_tags_27 },
    { 0x83680884, SUC_lexicon_tags_14 },
    { 0x90280883, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5be80887, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x76480889, SUC_lexicon_tags_17 },
    { 0x82480889, SUC_lexicon_tags_40 },
    { 0xe2080889, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a8088e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc80894, SUC_lexicon_tags_61 },
    { 0xd3880895, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x48080897, SUC_lexicon_tags_1 },
    { 0xf2e80897, SUC_lexicon_tags_70 },
    { 0xb5c80899, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x60c8089b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda8089e, SUC_lexicon_tags_263 },
    { 0x79a8089f, SUC_lexicon_tags_15 },
    { 0xaee8089f, SUC_lexicon_tags_57 },
    { 0x94808a1, SUC_lexicon_tags_11 },
    { 0x416808a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1ca808a4, SUC_lexicon_tags_16 },
    { 0x85e808a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8808a8, SUC_lexicon_tags_26 },
    { 0x92c808a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43a808ab, SUC_lexicon_tags_31 },
    { 0x9f0808ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1ae808ae, SUC_lexicon_tags_31 },
    { 0x62c808ae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd88808b1, SUC_lexicon_tags_63 },
    { 0x6f2808b2, SUC_lexicon_tags_17 },
    { 0xb96808b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7fe808b5, SUC_lexicon_tags_23 },
    { 0x79e808b5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xae6808b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x15a808ba, SUC_lexicon_tags_17 },
    { 0x660808bb, SUC_lexicon_tags_14 },
    { 0xc64808bc, SUC_lexicon_tags_23 },
    { 0x5b0808bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x650808c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4808c6, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc808c9, SUC_lexicon_tags_59 },
    { 0x428808ca, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x406808cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbca808ce, SUC_lexicon_tags_37 },
    { 0x7f2808cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b4808d1, SUC_lexicon_tags_9 },
    { 0x664808d2, SUC_lexicon_tags_37 },
    { 0x8fa808d3, SUC_lexicon_tags_32 },
    { 0xcca808d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30808da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be808dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c808e0, SUC_lexicon_tags_106 },
    { 0x216808e1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a808ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f8808ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c808ef, SUC_lexicon_tags_30 },
    { 0xa50808ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308808f6, SUC_lexicon_tags_37 },
    { 0x9c0808f6, SUC_lexicon_tags_37 },
    { 0x222808f8, SUC_lexicon_tags_17 },
    { 0xb6a808f9, SUC_lexicon_tags_74 },
    { 0x5c8808fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4680900, SUC_lexicon_tags_57 },
    { 0x1f680901, SUC_lexicon_tags_12 },
    { 0xe1280902, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7280905, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc80909, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73480911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc80919, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc668091c, SUC_lexicon_tags_31 },
    { 0x9088091d, SUC_lexicon_tags_63 },
    { 0x78e8091d, SUC_lexicon_tags_31 },
    { 0x8888091c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x93080921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59280924, SUC_lexicon_tags_37 },
    { 0xc3280925, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6048092e, SUC_lexicon_tags_25 },
    { 0x4d48092f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40080931, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x30680934, SUC_lexicon_tags_37 },
    { 0x22a80935, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4788093b, SUC_lexicon_tags_20 },
    { 0x5448093b, SUC_lexicon_tags_19 },
    { 0xd228093d, SUC_lexicon_tags_34 },
    { 0x6328093e, SUC_lexicon_tags_17 },
    { 0x89a8093f, SUC_lexicon_tags_37 },
    { 0x9328093f, SUC_lexicon_tags_47 },
    { 0x39480940, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee80946, SUC_lexicon_tags_2 },
    { 0x6f480946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9268094a, SUC_lexicon_tags_19 },
    { 0xde48094a, SUC_lexicon_tags_37 },
    { 0x72e8094b, SUC_lexicon_tags_11 },
    { 0xd848094c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e80951, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7c80953, SUC_lexicon_tags_66 },
    { 0x6c880954, SUC_lexicon_tags_7 },
    { 0x74080955, SUC_lexicon_tags_37 },
    { 0x2e280953, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0880959, SUC_lexicon_tags_6 },
    { 0x2d080959, SUC_lexicon_tags_87 },
    { 0x4708095b, SUC_lexicon_tags_15 },
    { 0x9648095c, SUC_lexicon_tags_0 },
    { 0x3848095d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8880960, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95880966, SUC_lexicon_tags_12 },
    { 0x1e280967, SUC_lexicon_tags_24 },
    { 0x1b280968, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9c68096a, SUC_lexicon_tags_17 },
    { 0xb9c8096b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfbe8096d, SUC_lexicon_tags_55 },
    { 0xc268096d, SUC_lexicon_tags_29 },
    { 0xb308096f, SUC_lexicon_tags_9 },
    { 0xd8280970, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40280979, SUC_lexicon_tags_4 },
    { 0xc8097a, SUC_lexicon_tags_63 },
    { 0xc1c8097b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9e48097d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79280989, SUC_lexicon_tags_23 },
    { 0x23e80989, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d08098d, SUC_lexicon_tags_7 },
    { 0x61e8098d, SUC_lexicon_tags_17 },
    { 0x1a8098d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5280993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc80996, SUC_lexicon_tags_292 },
    { 0, NULL },
    { 0x280998, SUC_lexicon_tags_60 },
    { 0x5f480999, SUC_lexicon_tags_11 },
    { 0xd9e8099a, SUC_lexicon_tags_7 },
    { 0xe1e8099a, SUC_lexicon_tags_26 },
    { 0x42c8099c, SUC_lexicon_tags_43 },
    { 0xd548099d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x62a8099f, SUC_lexicon_tags_38 },
    { 0xa40809a0, SUC_lexicon_tags_30 },
    { 0xeac809a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862809a7, SUC_lexicon_tags_279 },
    { 0x44e809a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e809ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e809ae, SUC_lexicon_tags_25 },
    { 0x492809af, SUC_lexicon_tags_49 },
    { 0x73e809af, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x620809b2, SUC_lexicon_tags_1 },
    { 0x818809b2, SUC_lexicon_tags_29 },
    { 0xd22809b3, SUC_lexicon_tags_11 },
    { 0xa26809b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c809b8, SUC_lexicon_tags_17 },
    { 0xc86809b9, SUC_lexicon_tags_56 },
    { 0x59e809ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6809be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590809c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1aa809c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4ba809c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34809cd, SUC_lexicon_tags_15 },
    { 0x4c2809cd, SUC_lexicon_tags_129 },
    { 0x548809cf, SUC_lexicon_tags_38 },
    { 0x9e6809ce, SUC_lexicon_tags_17 },
    { 0xbf0809d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be809d4, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xd3e809d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x660809d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6be809da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc809de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0c809e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36809e3, SUC_lexicon_tags_23 },
    { 0x476809e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce809e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e809eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90809ef, SUC_lexicon_tags_2 },
    { 0x3fe809f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa18809f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366809f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636809fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00809ff, SUC_lexicon_tags_1 },
    { 0x23c809ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20480a04, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a80a07, SUC_lexicon_tags_1 },
    { 0xbbe80a07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f480a0a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xd8a80a0c, SUC_lexicon_tags_1 },
    { 0x33080a0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9a80a0f, SUC_lexicon_tags_12 },
    { 0x87480a10, SUC_lexicon_tags_12 },
    { 0x77080a10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a80a14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c80a1a, SUC_lexicon_tags_37 },
    { 0xfd880a1b, SUC_lexicon_tags_7 },
    { 0xcd080a1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9480a22, SUC_lexicon_tags_54 },
    { 0x45a80a23, SUC_lexicon_tags_47 },
    { 0xf4480a24, SUC_lexicon_tags_11 },
    { 0xec80a23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18c80a27, SUC_lexicon_tags_25 },
    { 0x96480a28, SUC_lexicon_tags_23 },
    { 0x72e80a29, SUC_lexicon_tags_7 },
    { 0x28080a2a, SUC_lexicon_tags_2 },
    { 0x3c280a27, SUC_lexicon_tags_30 },
    { 0x30280a29, SUC_lexicon_tags_7 },
    { 0xec880a27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe280a31, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a80a34, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d080a39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a280a3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7080a3e, SUC_lexicon_tags_1 },
    { 0x2ea80a3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1be80a41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a80a45, SUC_lexicon_tags_27 },
    { 0x51080a45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb480a4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0880a4c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1480a4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e80a53, SUC_lexicon_tags_37 },
    { 0x58e80a54, SUC_lexicon_tags_12 },
    { 0x7aa80a54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x90480a57, SUC_lexicon_tags_30 },
    { 0xa4480a58, SUC_lexicon_tags_65 },
    { 0xdec80a59, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc80a61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68080a65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x85080a68, SUC_lexicon_tags_56 },
    { 0xe9680a69, SUC_lexicon_tags_38 },
    { 0xafe80a69, SUC_lexicon_tags_17 },
    { 0xaf880a68, SUC_lexicon_tags_47 },
    { 0x32a80a6a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3a880a6e, SUC_lexicon_tags_15 },
    { 0x86a80a6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4e80a71, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b280a76, SUC_lexicon_tags_15 },
    { 0x96880a77, SUC_lexicon_tags_19 },
    { 0x52e80a76, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa80a7b, SUC_lexicon_tags_37 },
    { 0xaa880a7b, SUC_lexicon_tags_14 },
    { 0x93a80a7b, SUC_lexicon_tags_34 },
    { 0x15080a7c, SUC_lexicon_tags_16 },
    { 0xdc480a7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f280a81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc480a83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d880a87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d280a91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x59880a93, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd7280a95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e80a9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99080aa1, SUC_lexicon_tags_7 },
    { 0x5ea80aa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f080aa9, SUC_lexicon_tags_46 },
    { 0xd2c80aaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c80aad, SUC_lexicon_tags_0 },
    { 0x40e80aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53680ab3, SUC_lexicon_tags_7 },
    { 0x9a80ab4, SUC_lexicon_tags_25 },
    { 0xfce80ab5, SUC_lexicon_tags_16 },
    { 0x6e680ab5, SUC_lexicon_tags_107 },
    { 0x9d480ab4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5c80ab9, SUC_lexicon_tags_15 },
    { 0x45080aba, SUC_lexicon_tags_80 },
    { 0xaf480abb, SUC_lexicon_tags_87 },
    { 0x92c80abb, SUC_lexicon_tags_23 },
    { 0xe0480aba, SUC_lexicon_tags_1 },
    { 0x13a80abe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3080ac2, SUC_lexicon_tags_30 },
    { 0x4c480ac3, SUC_lexicon_tags_8 },
    { 0xd7080ac4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e80ac7, SUC_lexicon_tags_27 },
    { 0xb4680ac8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2da80aca, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a80ace, SUC_lexicon_tags_29 },
    { 0xc1080acf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f680ad3, SUC_lexicon_tags_11 },
    { 0x1b480ad3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40280ad8, SUC_lexicon_tags_60 },
    { 0xb9e80ad9, SUC_lexicon_tags_12 },
    { 0x87080ada, SUC_lexicon_tags_37 },
    { 0xba880ada, SUC_lexicon_tags_14 },
    { 0xf4880adb, SUC_lexicon_tags_15 },
    { 0x1d280add, SUC_lexicon_tags_91 },
    { 0x64a80ada, SUC_lexicon_tags_17 },
    { 0xc9c80ada, SUC_lexicon_tags_7 },
    { 0x54280ae0, SUC_lexicon_tags_10 },
    { 0x2ac80ae1, SUC_lexicon_tags_33 },
    { 0x1b480adb, SUC_lexicon_tags_68 },
    { 0xdc680ad9, SUC_lexicon_tags_25 },
    { 0xf9280ae4, SUC_lexicon_tags_25 },
    { 0x9f280ae4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cc80ae7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80080aea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87880aee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde80af1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6680af3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c80af6, SUC_lexicon_tags_25 },
    { 0xcc680af6, SUC_lexicon_tags_107 },
    { 0x57280af8, SUC_lexicon_tags_35 },
    { 0xcae80af7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac480aff, SUC_lexicon_tags_43 },
    { 0x66480aff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1a80b02, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25880b06, SUC_lexicon_tags_37 },
    { 0x5b280b07, SUC_lexicon_tags_19 },
    { 0xb7480b06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa280b0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c80b11, SUC_lexicon_tags_17 },
    { 0xa1e80b12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c80b17, SUC_lexicon_tags_17 },
    { 0x6ee80b18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e080b1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74280b20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45280b22, SUC_lexicon_tags_7 },
    { 0xe8080b22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae80b26, SUC_lexicon_tags_37 },
    { 0x34480b27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34280b2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a80b31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2080b33, SUC_lexicon_tags_4 },
    { 0xa3a80b34, SUC_lexicon_tags_23 },
    { 0xf6080b33, SUC_lexicon_tags_11 },
    { 0xacc80b33, SUC_lexicon_tags_29 },
    { 0x8f280b33, SUC_lexicon_tags_47 },
    { 0x87e80b38, SUC_lexicon_tags_29 },
    { 0x1e280b39, SUC_lexicon_tags_34 },
    { 0x3a680b3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5e80b3c, SUC_lexicon_tags_43 },
    { 0xe1480b3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96480b3f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33280b47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e80b4a, SUC_lexicon_tags_12 },
    { 0x6d680b4b, SUC_lexicon_tags_3 },
    { 0xe280b4c, SUC_lexicon_tags_12 },
    { 0x15880b4d, SUC_lexicon_tags_7 },
    { 0x53080b4e, SUC_lexicon_tags_12 },
    { 0xe9280b4f, SUC_lexicon_tags_43 },
    { 0x19080b50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c80b5a, SUC_lexicon_tags_6 },
    { 0x4280b5b, SUC_lexicon_tags_25 },
    { 0xcbc80b5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x80880b5f, SUC_lexicon_tags_151 },
    { 0x1c880b60, SUC_lexicon_tags_1 },
    { 0x2ce80b61, SUC_lexicon_tags_8 },
    { 0x9a680b60, SUC_lexicon_tags_559 },
    { 0, NULL },
    { 0xa5080b64, SUC_lexicon_tags_17 },
    { 0x26680b65, SUC_lexicon_tags_35 },
    { 0xfba80b66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e80b6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfb280b6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea80b72, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e80b75, SUC_lexicon_tags_46 },
    { 0xfd080b75, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xdc480b78, SUC_lexicon_tags_15 },
    { 0xf6480b78, SUC_lexicon_tags_17 },
    { 0xbf080b78, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3480b7d, SUC_lexicon_tags_30 },
    { 0x1dc80b7e, SUC_lexicon_tags_20 },
    { 0x63880b7f, SUC_lexicon_tags_30 },
    { 0x95080b80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c80b85, SUC_lexicon_tags_136 },
    { 0xc9680b85, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe080b8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f680b8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2480b95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14a80b97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88880b99, SUC_lexicon_tags_12 },
    { 0xa0a80b9a, SUC_lexicon_tags_17 },
    { 0xb6c80b9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46e80b9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47680ba1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25480ba8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad080baa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f080bac, SUC_lexicon_tags_27 },
    { 0xb8080bac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4280bb1, SUC_lexicon_tags_27 },
    { 0x63e80bb2, SUC_lexicon_tags_7 },
    { 0xa9880bb3, SUC_lexicon_tags_25 },
    { 0xd9680bb3, SUC_lexicon_tags_32 },
    { 0x7a880bb5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x56480bb8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x63c80bba, SUC_lexicon_tags_37 },
    { 0x6b080bbb, SUC_lexicon_tags_14 },
    { 0x48c80bba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1f280bbe, SUC_lexicon_tags_2 },
    { 0xec280bbf, SUC_lexicon_tags_23 },
    { 0xf3680bc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37880bc3, SUC_lexicon_tags_37 },
    { 0xe4e80bc4, SUC_lexicon_tags_37 },
    { 0xf0c80bc5, SUC_lexicon_tags_25 },
    { 0x46280bc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a80bca, SUC_lexicon_tags_47 },
    { 0xd6280bca, SUC_lexicon_tags_56 },
    { 0xc4c80bcc, SUC_lexicon_tags_2 },
    { 0xbe680bcc, SUC_lexicon_tags_10 },
    { 0xefa80bcc, SUC_lexicon_tags_2 },
    { 0x3d280bcf, SUC_lexicon_tags_26 },
    { 0x9e80bd0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c80bd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86280bda, SUC_lexicon_tags_17 },
    { 0xfc80bdb, SUC_lexicon_tags_37 },
    { 0xcf880bdb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c80be0, SUC_lexicon_tags_31 },
    { 0xa3080be1, SUC_lexicon_tags_17 },
    { 0xbf280be2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba80be6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91880be9, SUC_lexicon_tags_17 },
    { 0x2ba80bea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a80bee, SUC_lexicon_tags_12 },
    { 0x46c80bee, SUC_lexicon_tags_14 },
    { 0xd9880bf0, SUC_lexicon_tags_30 },
    { 0x73c80bf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7880bf4, SUC_lexicon_tags_0 },
    { 0x35c80bf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c480bf7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58880bfc, SUC_lexicon_tags_4 },
    { 0x2f480bfd, SUC_lexicon_tags_12 },
    { 0xef680bfe, SUC_lexicon_tags_11 },
    { 0xb6680bff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24080c05, SUC_lexicon_tags_25 },
    { 0xbda80c06, SUC_lexicon_tags_37 },
    { 0x2dc80c05, SUC_lexicon_tags_1 },
    { 0x48e80c05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc80c0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3c80c0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x93880c0f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a80c12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x68a80c14, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73680c18, SUC_lexicon_tags_11 },
    { 0x92880c19, SUC_lexicon_tags_1 },
    { 0xa7280c1a, SUC_lexicon_tags_37 },
    { 0x5480c1b, SUC_lexicon_tags_48 },
    { 0x7fe80c1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11680c1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdee80c21, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7880c26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x73080c28, SUC_lexicon_tags_37 },
    { 0x56e80c29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x52480c2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdda80c2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20280c32, SUC_lexicon_tags_65 },
    { 0x8ea80c33, SUC_lexicon_tags_11 },
    { 0x76a80c34, SUC_lexicon_tags_15 },
    { 0xa1480c34, SUC_lexicon_tags_11 },
    { 0x9d680c35, SUC_lexicon_tags_11 },
    { 0xc9280c32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7480c3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb080c3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35080c42, SUC_lexicon_tags_25 },
    { 0xa0480c43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c80c48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8080c4f, SUC_lexicon_tags_0 },
    { 0x9ec80c50, SUC_lexicon_tags_7 },
    { 0xd9280c50, SUC_lexicon_tags_7 },
    { 0x1ea80c51, SUC_lexicon_tags_49 },
    { 0x7c480c53, SUC_lexicon_tags_11 },
    { 0x16280c54, SUC_lexicon_tags_60 },
    { 0xda080c54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46080c59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a80c61, SUC_lexicon_tags_14 },
    { 0x13280c62, SUC_lexicon_tags_82 },
    { 0xffa80c63, SUC_lexicon_tags_25 },
    { 0xb2080c61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xda880c67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78680c6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ca80c6f, SUC_lexicon_tags_38 },
    { 0xc8680c6f, SUC_lexicon_tags_4 },
    { 0x80280c6f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46480c78, SUC_lexicon_tags_25 },
    { 0xe1080c79, SUC_lexicon_tags_25 },
    { 0xd2a80c7a, SUC_lexicon_tags_31 },
    { 0x4880c7a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa80c7e, SUC_lexicon_tags_47 },
    { 0x58480c7f, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe80c85, SUC_lexicon_tags_17 },
    { 0x2e080c85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6880c88, SUC_lexicon_tags_68 },
    { 0x5bc80c88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e480c8d, SUC_lexicon_tags_27 },
    { 0x4fa80c8d, SUC_lexicon_tags_87 },
    { 0xbfe80c8f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a80c93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f280c97, SUC_lexicon_tags_13 },
    { 0x16880c97, SUC_lexicon_tags_25 },
    { 0x9ec80c98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3fe80c9b, SUC_lexicon_tags_2 },
    { 0x39880c9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da80c9f, SUC_lexicon_tags_37 },
    { 0xfb680ca0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8b280ca2, SUC_lexicon_tags_12 },
    { 0x50280ca3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c80cb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8680cb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81280cb7, SUC_lexicon_tags_23 },
    { 0x4ae80cb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea680cbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3480cbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a80cc4, SUC_lexicon_tags_38 },
    { 0x37680cc5, SUC_lexicon_tags_25 },
    { 0x21680cc5, SUC_lexicon_tags_25 },
    { 0x51480cc7, SUC_lexicon_tags_6 },
    { 0x4080cc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48280cd2, SUC_lexicon_tags_13 },
    { 0xd9880cd3, SUC_lexicon_tags_15 },
    { 0x3de80cd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82280cd8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x50e80cda, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd2880cdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8680cdf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c680ce2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c80ce7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x67280cea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26480cec, SUC_lexicon_tags_19 },
    { 0x95080ced, SUC_lexicon_tags_25 },
    { 0xe7480cee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48480cf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35e80cf2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cf5, SUC_lexicon_tags_17 },
    { 0x93680cf5, SUC_lexicon_tags_33 },
    { 0x86680cf6, SUC_lexicon_tags_7 },
    { 0x39c80cf8, SUC_lexicon_tags_10 },
    { 0x54e80cf5, SUC_lexicon_tags_0 },
    { 0x50a80cfa, SUC_lexicon_tags_11 },
    { 0xd8e80cfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xde80cfe, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c480d02, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x29080d04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14880d06, SUC_lexicon_tags_21 },
    { 0xd6480d07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b480d0a, SUC_lexicon_tags_64 },
    { 0x92080d0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72680d0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0080d15, SUC_lexicon_tags_2 },
    { 0x7e480d16, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e280d1b, SUC_lexicon_tags_15 },
    { 0x62080d1c, SUC_lexicon_tags_14 },
    { 0xcf680d1d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e80d20, SUC_lexicon_tags_8 },
    { 0x15280d21, SUC_lexicon_tags_25 },
    { 0xdde80d20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38680d2b, SUC_lexicon_tags_19 },
    { 0x9bc80d2c, SUC_lexicon_tags_27 },
    { 0x19480d2d, SUC_lexicon_tags_27 },
    { 0x7d880d2c, SUC_lexicon_tags_60 },
    { 0xf2080d2f, SUC_lexicon_tags_6 },
    { 0x8e280d30, SUC_lexicon_tags_25 },
    { 0xa3280d31, SUC_lexicon_tags_16 },
    { 0xfa280d2b, SUC_lexicon_tags_11 },
    { 0x14680d33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3280d35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6480d3a, SUC_lexicon_tags_48 },
    { 0x83080d3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0280d41, SUC_lexicon_tags_69 },
    { 0x3e880d42, SUC_lexicon_tags_15 },
    { 0x3e080d42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5080d47, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e480d4b, SUC_lexicon_tags_85 },
    { 0xc2a80d4b, SUC_lexicon_tags_15 },
    { 0x6f280d4d, SUC_lexicon_tags_47 },
    { 0xa7e80d4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0680d54, SUC_lexicon_tags_37 },
    { 0x3480d55, SUC_lexicon_tags_25 },
    { 0x6aa80d55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaa80d58, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6ac80d5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f480d5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5680d5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31480d61, SUC_lexicon_tags_12 },
    { 0x3e680d62, SUC_lexicon_tags_12 },
    { 0x65680d61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2680d65, SUC_lexicon_tags_25 },
    { 0x1c280d65, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76280d6b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a80d72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c80d75, SUC_lexicon_tags_49 },
    { 0x19a80d76, SUC_lexicon_tags_23 },
    { 0xa8880d76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99680d79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x33c80d7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f680d87, SUC_lexicon_tags_49 },
    { 0xc1a80d88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cc80d8a, SUC_lexicon_tags_27 },
    { 0xc0480d8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5080d8d, SUC_lexicon_tags_2 },
    { 0x99080d8e, SUC_lexicon_tags_12 },
    { 0xdf080d8e, SUC_lexicon_tags_49 },
    { 0xcfe80d8e, SUC_lexicon_tags_65 },
    { 0x6d080d90, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b080d94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49880d96, SUC_lexicon_tags_49 },
    { 0x32080d97, SUC_lexicon_tags_11 },
    { 0x8a80d98, SUC_lexicon_tags_37 },
    { 0x19680d96, SUC_lexicon_tags_14 },
    { 0xfe680d9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d480d9e, SUC_lexicon_tags_11 },
    { 0x52680d9e, SUC_lexicon_tags_47 },
    { 0x95c80da0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51c80da2, SUC_lexicon_tags_43 },
    { 0x3f680da3, SUC_lexicon_tags_17 },
    { 0xb0e80da3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c80da8, SUC_lexicon_tags_47 },
    { 0x76280da8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63680dab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc80db1, SUC_lexicon_tags_15 },
    { 0xe7c80db1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7280db8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc280dba, SUC_lexicon_tags_17 },
    { 0x30880dbb, SUC_lexicon_tags_30 },
    { 0xb7a80dbb, SUC_lexicon_tags_17 },
    { 0xcfe80dbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71280dc1, SUC_lexicon_tags_29 },
    { 0x1cc80dc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x81280dc5, SUC_lexicon_tags_68 },
    { 0xca680dc6, SUC_lexicon_tags_31 },
    { 0xc5a80dc7, SUC_lexicon_tags_11 },
    { 0x80e80dc6, SUC_lexicon_tags_37 },
    { 0xb6680dc5, SUC_lexicon_tags_14 },
    { 0x21480dc5, SUC_lexicon_tags_38 },
    { 0x16280dcb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f880dce, SUC_lexicon_tags_0 },
    { 0x9f280dce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd480dd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62a80dd3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8ec80dd5, SUC_lexicon_tags_2 },
    { 0x52280dd6, SUC_lexicon_tags_26 },
    { 0x1be80dd7, SUC_lexicon_tags_12 },
    { 0x60c80dd8, SUC_lexicon_tags_11 },
    { 0x94880dd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd080de1, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x7ca80de3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda80de6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x29480de8, SUC_lexicon_tags_25 },
    { 0x9b280de9, SUC_lexicon_tags_8 },
    { 0xb6680de9, SUC_lexicon_tags_45 },
    { 0x62a80deb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2d880ded, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c480def, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb4280df1, SUC_lexicon_tags_2 },
    { 0x14c80df1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca80df5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e80df7, SUC_lexicon_tags_37 },
    { 0x60080df8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71080dfa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa4880dfc, SUC_lexicon_tags_14 },
    { 0xe9280dfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7c80dff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a680e01, SUC_lexicon_tags_36 },
    { 0x9d280e02, SUC_lexicon_tags_92 },
    { 0x1bc80e02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe80e07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a80e0a, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e880e10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfcc80e12, SUC_lexicon_tags_63 },
    { 0xf9280e12, SUC_lexicon_tags_37 },
    { 0x60a80e14, SUC_lexicon_tags_8 },
    { 0xcd080e13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb080e1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62480e1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0880e20, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x95c80e22, SUC_lexicon_tags_19 },
    { 0x76280e22, SUC_lexicon_tags_43 },
    { 0xe280e24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15280e30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58480e32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc280e35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x97280e38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85680e3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30880e3f, SUC_lexicon_tags_34 },
    { 0x4fe80e40, SUC_lexicon_tags_141 },
    { 0x57080e41, SUC_lexicon_tags_25 },
    { 0x6c880e40, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e80e45, SUC_lexicon_tags_37 },
    { 0x54680e46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3c680e48, SUC_lexicon_tags_11 },
    { 0x84080e48, SUC_lexicon_tags_108 },
    { 0x9da80e4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58e80e4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b680e4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82e80e50, SUC_lexicon_tags_7 },
    { 0x98480e51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeac80e53, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b280e59, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e80e5e, SUC_lexicon_tags_15 },
    { 0xeba80e5e, SUC_lexicon_tags_7 },
    { 0x42880e5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc880e62, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac80e65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c80e68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87080e6b, SUC_lexicon_tags_9 },
    { 0xa2880e6c, SUC_lexicon_tags_37 },
    { 0x93880e6d, SUC_lexicon_tags_23 },
    { 0x6ce80e6b, SUC_lexicon_tags_2 },
    { 0xaa080e6e, SUC_lexicon_tags_11 },
    { 0x1ae80e70, SUC_lexicon_tags_15 },
    { 0xeac80e6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11080e7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e80e7e, SUC_lexicon_tags_33 },
    { 0xdb880e7f, SUC_lexicon_tags_47 },
    { 0x55a80e7e, SUC_lexicon_tags_12 },
    { 0xb5a80e81, SUC_lexicon_tags_2 },
    { 0xe4880e82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a80e85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1280e89, SUC_lexicon_tags_30 },
    { 0x40c80e8a, SUC_lexicon_tags_60 },
    { 0xfb280e8a, SUC_lexicon_tags_38 },
    { 0x48880e89, SUC_lexicon_tags_11 },
    { 0xae680e8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd080e91, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd080e95, SUC_lexicon_tags_37 },
    { 0x16280e96, SUC_lexicon_tags_15 },
    { 0xe7280e95, SUC_lexicon_tags_45 },
    { 0x3280e98, SUC_lexicon_tags_15 },
    { 0x70080e99, SUC_lexicon_tags_0 },
    { 0x6c480e99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8880e9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18680e9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeae80ea1, SUC_lexicon_tags_31 },
    { 0xdc80ea2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e80ea8, SUC_lexicon_tags_25 },
    { 0x23080ea9, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95480eb2, SUC_lexicon_tags_7 },
    { 0x5c680eb2, SUC_lexicon_tags_12 },
    { 0x3a280eb2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x77a80eb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x44e80eb8, SUC_lexicon_tags_27 },
    { 0x32e80eb9, SUC_lexicon_tags_2 },
    { 0x74c80eba, SUC_lexicon_tags_33 },
    { 0x48080ebb, SUC_lexicon_tags_11 },
    { 0x24280ebc, SUC_lexicon_tags_65 },
    { 0x32c80ebd, SUC_lexicon_tags_2 },
    { 0x2e880eb8, SUC_lexicon_tags_17 },
    { 0xfa80ebc, SUC_lexicon_tags_19 },
    { 0x36680ec0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a80ec4, SUC_lexicon_tags_25 },
    { 0xb4880ec5, SUC_lexicon_tags_43 },
    { 0x9aa80ec6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba80eca, SUC_lexicon_tags_87 },
    { 0x280ecb, SUC_lexicon_tags_8 },
    { 0xdf880ecb, SUC_lexicon_tags_15 },
    { 0xf9680ecb, SUC_lexicon_tags_25 },
    { 0x3ce80ece, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9080ed1, SUC_lexicon_tags_1 },
    { 0x34a80ed2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2c480ed4, SUC_lexicon_tags_46 },
    { 0xe0880ed5, SUC_lexicon_tags_48 },
    { 0xbb680ed6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12080edb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a80ede, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a80ee1, SUC_lexicon_tags_12 },
    { 0x27880ee1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5480ee7, SUC_lexicon_tags_14 },
    { 0xa4480ee8, SUC_lexicon_tags_12 },
    { 0x58680ee9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5280eed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbea80eef, SUC_lexicon_tags_61 },
    { 0x65680ef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee80ef8, SUC_lexicon_tags_27 },
    { 0xecc80ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee80efc, SUC_lexicon_tags_43 },
    { 0x2ea80efc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c80f00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51480f04, SUC_lexicon_tags_1 },
    { 0x67680f05, SUC_lexicon_tags_8 },
    { 0x4a680f04, SUC_lexicon_tags_45 },
    { 0xea080f04, SUC_lexicon_tags_45 },
    { 0x3f280f08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc80f12, SUC_lexicon_tags_17 },
    { 0xfc080f13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63680f17, SUC_lexicon_tags_44 },
    { 0x96080f17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca80f1b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3680f1d, SUC_lexicon_tags_1 },
    { 0x4a80f1e, SUC_lexicon_tags_15 },
    { 0xdb080f1f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a80f24, SUC_lexicon_tags_12 },
    { 0x85480f24, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x95480f28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8c880f2a, SUC_lexicon_tags_38 },
    { 0x7c680f2b, SUC_lexicon_tags_25 },
    { 0x14080f2c, SUC_lexicon_tags_70 },
    { 0x7c680f2d, SUC_lexicon_tags_37 },
    { 0xd9480f2b, SUC_lexicon_tags_66 },
    { 0x2e680f2d, SUC_lexicon_tags_25 },
    { 0xdc80f30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7a80f32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc80f37, SUC_lexicon_tags_48 },
    { 0xe6e80f38, SUC_lexicon_tags_7 },
    { 0x55480f39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb880f3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79680f3d, SUC_lexicon_tags_37 },
    { 0x75480f3e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87680f46, SUC_lexicon_tags_27 },
    { 0xe3280f46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6e80f49, SUC_lexicon_tags_15 },
    { 0x54880f4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d280f4d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba80f51, SUC_lexicon_tags_25 },
    { 0x90a80f52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1080f55, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33280f5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb080f62, SUC_lexicon_tags_1 },
    { 0x6a880f63, SUC_lexicon_tags_15 },
    { 0x68080f63, SUC_lexicon_tags_49 },
    { 0xd7680f65, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xbd880f67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9080f6d, SUC_lexicon_tags_15 },
    { 0x87c80f6e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc80f71, SUC_lexicon_tags_0 },
    { 0xe6480f71, SUC_lexicon_tags_2 },
    { 0xed680f73, SUC_lexicon_tags_37 },
    { 0xb6a80f74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f280f76, SUC_lexicon_tags_1 },
    { 0xaea80f77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d480f7b, SUC_lexicon_tags_45 },
    { 0x9c480f7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99480f7e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a80f82, SUC_lexicon_tags_7 },
    { 0xc3480f83, SUC_lexicon_tags_2 },
    { 0x9d880f84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c80f8c, SUC_lexicon_tags_15 },
    { 0x8b880f8d, SUC_lexicon_tags_45 },
    { 0xb6280f8d, SUC_lexicon_tags_0 },
    { 0x59a80f8f, SUC_lexicon_tags_25 },
    { 0x40c80f90, SUC_lexicon_tags_84 },
    { 0xc4680f90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9880f94, SUC_lexicon_tags_11 },
    { 0x6c480f95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xacc80f97, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeea80f99, SUC_lexicon_tags_53 },
    { 0x84480f9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x11480f9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf280fa0, SUC_lexicon_tags_25 },
    { 0xe3a80fa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5280fa3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a80faa, SUC_lexicon_tags_7 },
    { 0x4d480fab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb280fae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b480fb0, SUC_lexicon_tags_25 },
    { 0xd5a80fb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa280fb3, SUC_lexicon_tags_45 },
    { 0xeb280fb4, SUC_lexicon_tags_12 },
    { 0x80680fb4, SUC_lexicon_tags_17 },
    { 0xe9480fb4, SUC_lexicon_tags_25 },
    { 0x76c80fb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c80fbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe080fc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c880fc6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe80fcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6880fcf, SUC_lexicon_tags_37 },
    { 0x4ba80fd0, SUC_lexicon_tags_55 },
    { 0xcf680fd1, SUC_lexicon_tags_8 },
    { 0xdfc80fd0, SUC_lexicon_tags_14 },
    { 0x16a80fd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe80fdb, SUC_lexicon_tags_9 },
    { 0x9a280fdc, SUC_lexicon_tags_12 },
    { 0x98a80fdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e80fe1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a280fe4, SUC_lexicon_tags_33 },
    { 0xba80fe5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x57880fe8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb680feb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75680ff2, SUC_lexicon_tags_25 },
    { 0xd6c80ff3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1480ff9, SUC_lexicon_tags_38 },
    { 0xd480ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7480ffc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5480ffe, SUC_lexicon_tags_17 },
    { 0x49e80fff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad681005, SUC_lexicon_tags_43 },
    { 0x19281006, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe8100b, SUC_lexicon_tags_95 },
    { 0x9e88100c, SUC_lexicon_tags_7 },
    { 0x8b88100c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e81011, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be81017, SUC_lexicon_tags_32 },
    { 0xea81018, SUC_lexicon_tags_15 },
    { 0xa4c81017, SUC_lexicon_tags_1 },
    { 0x5dc81019, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa8101d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e81023, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9681027, SUC_lexicon_tags_60 },
    { 0xe4081028, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be8102e, SUC_lexicon_tags_26 },
    { 0x9108102e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44e81031, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a81034, SUC_lexicon_tags_7 },
    { 0x25881035, SUC_lexicon_tags_28 },
    { 0x8c881036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa481038, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1028103a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e481041, SUC_lexicon_tags_8 },
    { 0x96881041, SUC_lexicon_tags_15 },
    { 0x21681043, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a81049, SUC_lexicon_tags_23 },
    { 0x5428104a, SUC_lexicon_tags_0 },
    { 0xcbc8104b, SUC_lexicon_tags_26 },
    { 0x36e8104c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd8e8104e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc81053, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x65281055, SUC_lexicon_tags_6 },
    { 0x61681055, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba8105e, SUC_lexicon_tags_1 },
    { 0xb648105f, SUC_lexicon_tags_4 },
    { 0x3228105e, SUC_lexicon_tags_31 },
    { 0x1a881061, SUC_lexicon_tags_188 },
    { 0xe1c81061, SUC_lexicon_tags_33 },
    { 0xd5481062, SUC_lexicon_tags_43 },
    { 0x43081062, SUC_lexicon_tags_37 },
    { 0x9c281065, SUC_lexicon_tags_38 },
    { 0x2c81065, SUC_lexicon_tags_30 },
    { 0x7fe81067, SUC_lexicon_tags_7 },
    { 0x92a81068, SUC_lexicon_tags_26 },
    { 0xe0681069, SUC_lexicon_tags_49 },
    { 0xdf08106a, SUC_lexicon_tags_9 },
    { 0xb108106b, SUC_lexicon_tags_0 },
    { 0xab08106c, SUC_lexicon_tags_1 },
    { 0x8aa81063, SUC_lexicon_tags_72 },
    { 0x3da8106a, SUC_lexicon_tags_15 },
    { 0xc568106a, SUC_lexicon_tags_48 },
    { 0x7d081061, SUC_lexicon_tags_2 },
    { 0x75a81071, SUC_lexicon_tags_15 },
    { 0x96c81072, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5c81074, SUC_lexicon_tags_2 },
    { 0xd3681075, SUC_lexicon_tags_9 },
    { 0x1b081076, SUC_lexicon_tags_7 },
    { 0xb5c81077, SUC_lexicon_tags_1 },
    { 0x15481078, SUC_lexicon_tags_7 },
    { 0xd7481077, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91081083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff081086, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de81089, SUC_lexicon_tags_15 },
    { 0xa0c8108a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a8108c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13881093, SUC_lexicon_tags_47 },
    { 0x87081094, SUC_lexicon_tags_37 },
    { 0x32e81094, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xece81097, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c8109a, SUC_lexicon_tags_25 },
    { 0x17e8109b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea8109e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x210810a0, SUC_lexicon_tags_39 },
    { 0xd84810a1, SUC_lexicon_tags_44 },
    { 0xf94810a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfda810a4, SUC_lexicon_tags_0 },
    { 0xd7e810a4, SUC_lexicon_tags_37 },
    { 0xfa4810a6, SUC_lexicon_tags_29 },
    { 0x98e810a4, SUC_lexicon_tags_31 },
    { 0x62c810a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c810ad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc810b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a810b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130810b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x276810bb, SUC_lexicon_tags_26 },
    { 0x850810bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6de810be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3e6810c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x784810c3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c810c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x556810ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48810cd, SUC_lexicon_tags_15 },
    { 0x2ea810ce, SUC_lexicon_tags_1 },
    { 0x8fc810ce, SUC_lexicon_tags_0 },
    { 0xed2810d0, SUC_lexicon_tags_25 },
    { 0x932810d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a810d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x278810da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc6c810dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8810e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x604810e3, SUC_lexicon_tags_15 },
    { 0xb9e810e4, SUC_lexicon_tags_8 },
    { 0x132810e3, SUC_lexicon_tags_37 },
    { 0x4ae810e6, SUC_lexicon_tags_8 },
    { 0x910810e7, SUC_lexicon_tags_37 },
    { 0x45a810e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x27e810ea, SUC_lexicon_tags_15 },
    { 0x5c4810eb, SUC_lexicon_tags_0 },
    { 0xa1c810ec, SUC_lexicon_tags_20 },
    { 0xf80810ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2810f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66810f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d6810f7, SUC_lexicon_tags_31 },
    { 0xf0810f8, SUC_lexicon_tags_68 },
    { 0xbce810f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5e6810fb, SUC_lexicon_tags_47 },
    { 0x2e810fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe78810fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a81106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a08110e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f281111, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x41481114, SUC_lexicon_tags_17 },
    { 0xcee81115, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa728111a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7068111d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa81120, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8b681122, SUC_lexicon_tags_14 },
    { 0xe081123, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3681126, SUC_lexicon_tags_12 },
    { 0x7e481127, SUC_lexicon_tags_11 },
    { 0x5b681128, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b481133, SUC_lexicon_tags_37 },
    { 0xdb481133, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e81139, SUC_lexicon_tags_377 },
    { 0, NULL },
    { 0, NULL },
    { 0x4128113c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3808113f, SUC_lexicon_tags_25 },
    { 0x53481140, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd481142, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x69081145, SUC_lexicon_tags_37 },
    { 0x8ea81146, SUC_lexicon_tags_2 },
    { 0xa8c81146, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60c81149, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc28114c, SUC_lexicon_tags_11 },
    { 0x3da8114d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34281155, SUC_lexicon_tags_31 },
    { 0x77e81156, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b08115b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf0a8115d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd668115f, SUC_lexicon_tags_11 },
    { 0x6e881160, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26481162, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe81168, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc281171, SUC_lexicon_tags_15 },
    { 0x9d081172, SUC_lexicon_tags_49 },
    { 0xfc281171, SUC_lexicon_tags_1 },
    { 0x9f481174, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7081176, SUC_lexicon_tags_7 },
    { 0xc881176, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9681179, SUC_lexicon_tags_17 },
    { 0x88a8117a, SUC_lexicon_tags_6 },
    { 0x7208117a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea81181, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4281186, SUC_lexicon_tags_0 },
    { 0xe3c81187, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf568118a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec81190, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a81195, SUC_lexicon_tags_7 },
    { 0x46c81195, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e81199, SUC_lexicon_tags_27 },
    { 0xb908119a, SUC_lexicon_tags_43 },
    { 0x9048119b, SUC_lexicon_tags_37 },
    { 0x3908119c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16e8119e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x182811a0, SUC_lexicon_tags_25 },
    { 0xc70811a1, SUC_lexicon_tags_33 },
    { 0xa3c811a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50811a8, SUC_lexicon_tags_26 },
    { 0xe811a8, SUC_lexicon_tags_12 },
    { 0x36811a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe811b2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x530811b5, SUC_lexicon_tags_11 },
    { 0x6d4811b6, SUC_lexicon_tags_43 },
    { 0xf30811b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfe811b9, SUC_lexicon_tags_25 },
    { 0xe42811b9, SUC_lexicon_tags_38 },
    { 0x940811bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba811c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4811c6, SUC_lexicon_tags_17 },
    { 0x59c811c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf60811c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa811cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cc811ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x128811d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x740811d4, SUC_lexicon_tags_48 },
    { 0x328811d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786811db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf66811dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6a4811df, SUC_lexicon_tags_15 },
    { 0x990811df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe06811e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x992811e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72811e7, SUC_lexicon_tags_33 },
    { 0x14811e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14811eb, SUC_lexicon_tags_37 },
    { 0x3da811ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa18811ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9cc811f0, SUC_lexicon_tags_15 },
    { 0x71a811f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c811f4, SUC_lexicon_tags_2 },
    { 0x36a811f5, SUC_lexicon_tags_37 },
    { 0xa50811f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66811f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba81202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5681205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24081208, SUC_lexicon_tags_47 },
    { 0x78a81209, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5481210, SUC_lexicon_tags_37 },
    { 0xa2681211, SUC_lexicon_tags_15 },
    { 0x23481211, SUC_lexicon_tags_46 },
    { 0x20a81213, SUC_lexicon_tags_38 },
    { 0x4a481214, SUC_lexicon_tags_11 },
    { 0x12c81214, SUC_lexicon_tags_21 },
    { 0xdb681216, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3281219, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe08121d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be81224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa081226, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x34681228, SUC_lexicon_tags_29 },
    { 0x5b281229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ce8122b, SUC_lexicon_tags_37 },
    { 0x7968122c, SUC_lexicon_tags_33 },
    { 0x408122c, SUC_lexicon_tags_46 },
    { 0x92a8122c, SUC_lexicon_tags_11 },
    { 0x9848122f, SUC_lexicon_tags_43 },
    { 0xdf881230, SUC_lexicon_tags_505 },
    { 0xabc81230, SUC_lexicon_tags_25 },
    { 0xcca8122b, SUC_lexicon_tags_61 },
    { 0x97a8122b, SUC_lexicon_tags_17 },
    { 0x91a81234, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb88123e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b081241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c81245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfac81247, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9e81249, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e8124d, SUC_lexicon_tags_11 },
    { 0x3428124e, SUC_lexicon_tags_12 },
    { 0xda48124f, SUC_lexicon_tags_28 },
    { 0xa4281250, SUC_lexicon_tags_29 },
    { 0x9e281250, SUC_lexicon_tags_14 },
    { 0x6a281252, SUC_lexicon_tags_1 },
    { 0x9c481251, SUC_lexicon_tags_97 },
    { 0xf7081252, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be81257, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ce81259, SUC_lexicon_tags_11 },
    { 0x3aa81259, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf988125c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80081260, SUC_lexicon_tags_1 },
    { 0x76681261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81265, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8a81267, SUC_lexicon_tags_43 },
    { 0xebc81267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa148126a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa8126d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x22081270, SUC_lexicon_tags_2 },
    { 0xbd681271, SUC_lexicon_tags_15 },
    { 0x46481270, SUC_lexicon_tags_12 },
    { 0x1a081272, SUC_lexicon_tags_37 },
    { 0x9d681274, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9681278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc8127d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2b48127f, SUC_lexicon_tags_49 },
    { 0xeac81280, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa448128c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e8128f, SUC_lexicon_tags_15 },
    { 0x32e81290, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd881295, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91681299, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b88129c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5308129f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6812a6, SUC_lexicon_tags_92 },
    { 0x32e812a7, SUC_lexicon_tags_1 },
    { 0xb4e812a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a6812aa, SUC_lexicon_tags_82 },
    { 0x7b0812ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0812af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0812b5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0812b9, SUC_lexicon_tags_60 },
    { 0xc6c812ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a812bd, SUC_lexicon_tags_25 },
    { 0xd7c812be, SUC_lexicon_tags_60 },
    { 0xd04812be, SUC_lexicon_tags_11 },
    { 0x5f2812be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2812c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e812cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3bc812cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc80812cf, SUC_lexicon_tags_2 },
    { 0xc30812d0, SUC_lexicon_tags_28 },
    { 0x88812d0, SUC_lexicon_tags_47 },
    { 0x964812d2, SUC_lexicon_tags_33 },
    { 0x140812d3, SUC_lexicon_tags_37 },
    { 0x2a812d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36812d7, SUC_lexicon_tags_208 },
    { 0xea8812d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2812dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaee812df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04812e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6812e9, SUC_lexicon_tags_17 },
    { 0xf5e812ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28812ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc6812ef, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x348812f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2812f6, SUC_lexicon_tags_25 },
    { 0x2ee812f7, SUC_lexicon_tags_60 },
    { 0x7bc812f7, SUC_lexicon_tags_7 },
    { 0xd54812f9, SUC_lexicon_tags_37 },
    { 0x618812fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8812fc, SUC_lexicon_tags_49 },
    { 0xf4c812fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x922812ff, SUC_lexicon_tags_37 },
    { 0x5de81300, SUC_lexicon_tags_15 },
    { 0x50e81301, SUC_lexicon_tags_49 },
    { 0x54681301, SUC_lexicon_tags_17 },
    { 0x1a81303, SUC_lexicon_tags_43 },
    { 0x8481304, SUC_lexicon_tags_63 },
    { 0x6e081305, SUC_lexicon_tags_37 },
    { 0xea681305, SUC_lexicon_tags_27 },
    { 0xcf081305, SUC_lexicon_tags_212 },
    { 0x33c81308, SUC_lexicon_tags_79 },
    { 0x2cc81309, SUC_lexicon_tags_57 },
    { 0x21c8130a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d48130d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff681314, SUC_lexicon_tags_6 },
    { 0x34e81314, SUC_lexicon_tags_26 },
    { 0x7ca81314, SUC_lexicon_tags_29 },
    { 0x43681316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf88131b, SUC_lexicon_tags_0 },
    { 0x14e8131b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a28131f, SUC_lexicon_tags_25 },
    { 0x4ac8131f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e81324, SUC_lexicon_tags_16 },
    { 0xf4281324, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae8132c, SUC_lexicon_tags_85 },
    { 0xcce8132d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa2e8132f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3a81331, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66c81333, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x41281335, SUC_lexicon_tags_38 },
    { 0x39481336, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa1481338, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d88133c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10481341, SUC_lexicon_tags_25 },
    { 0x35881342, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1e81344, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc2281346, SUC_lexicon_tags_14 },
    { 0x5fe81347, SUC_lexicon_tags_55 },
    { 0x8e881347, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1768134a, SUC_lexicon_tags_2 },
    { 0x2528134b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9ee8134d, SUC_lexicon_tags_25 },
    { 0x1a88134e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a81352, SUC_lexicon_tags_16 },
    { 0x87c81353, SUC_lexicon_tags_7 },
    { 0xb9c81353, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7481357, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec8135f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a81362, SUC_lexicon_tags_30 },
    { 0x46681362, SUC_lexicon_tags_11 },
    { 0xe3881363, SUC_lexicon_tags_63 },
    { 0x64c81365, SUC_lexicon_tags_45 },
    { 0x4ac81362, SUC_lexicon_tags_31 },
    { 0x6fc81363, SUC_lexicon_tags_27 },
    { 0x39c81368, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdcc8136a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6468136f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66681372, SUC_lexicon_tags_11 },
    { 0xa3481373, SUC_lexicon_tags_12 },
    { 0x71681374, SUC_lexicon_tags_7 },
    { 0xea881374, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a08137b, SUC_lexicon_tags_37 },
    { 0xe588137c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde481380, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0881388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08138c, SUC_lexicon_tags_45 },
    { 0x6408138d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae28138f, SUC_lexicon_tags_1 },
    { 0x8f681390, SUC_lexicon_tags_1 },
    { 0xa5c81391, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d281393, SUC_lexicon_tags_22 },
    { 0x37c81394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda481398, SUC_lexicon_tags_37 },
    { 0xb881399, SUC_lexicon_tags_91 },
    { 0x66c8139a, SUC_lexicon_tags_26 },
    { 0x72e81398, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526813a0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a813a4, SUC_lexicon_tags_7 },
    { 0xdb2813a4, SUC_lexicon_tags_63 },
    { 0x3a813a4, SUC_lexicon_tags_27 },
    { 0x7f4813a7, SUC_lexicon_tags_0 },
    { 0xabe813a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x622813ab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70813ae, SUC_lexicon_tags_23 },
    { 0xfc2813af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x442813b1, SUC_lexicon_tags_43 },
    { 0x6b0813b2, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x866813b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd78813b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x576813b9, SUC_lexicon_tags_26 },
    { 0xefc813b9, SUC_lexicon_tags_43 },
    { 0xa14813ba, SUC_lexicon_tags_80 },
    { 0x28e813bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x446813bf, SUC_lexicon_tags_14 },
    { 0xb44813c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca813c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x698813c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42a813ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38813cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc813d2, SUC_lexicon_tags_17 },
    { 0xa9e813d2, SUC_lexicon_tags_17 },
    { 0xb90813d4, SUC_lexicon_tags_11 },
    { 0x5b2813d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9ca813d7, SUC_lexicon_tags_11 },
    { 0x4ca813d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x738813da, SUC_lexicon_tags_25 },
    { 0x5e0813db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa813e2, SUC_lexicon_tags_37 },
    { 0xe78813e2, SUC_lexicon_tags_32 },
    { 0xd32813e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0813e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e813ec, SUC_lexicon_tags_10 },
    { 0xbb2813ed, SUC_lexicon_tags_15 },
    { 0x426813ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6813f1, SUC_lexicon_tags_37 },
    { 0xa5c813f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c813f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb58813f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e813f9, SUC_lexicon_tags_11 },
    { 0x904813f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a813fe, SUC_lexicon_tags_11 },
    { 0x10813fe, SUC_lexicon_tags_61 },
    { 0x8aa81400, SUC_lexicon_tags_49 },
    { 0x8c881401, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c81406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe281409, SUC_lexicon_tags_25 },
    { 0x8588140a, SUC_lexicon_tags_9 },
    { 0x4348140b, SUC_lexicon_tags_2 },
    { 0x328140b, SUC_lexicon_tags_82 },
    { 0x97c8140c, SUC_lexicon_tags_47 },
    { 0x4528140e, SUC_lexicon_tags_15 },
    { 0x5fa8140f, SUC_lexicon_tags_15 },
    { 0x8948140c, SUC_lexicon_tags_39 },
    { 0x29481411, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e81414, SUC_lexicon_tags_35 },
    { 0xea881415, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xce681418, SUC_lexicon_tags_198 },
    { 0x1aa81419, SUC_lexicon_tags_61 },
    { 0x66881418, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7ae8141c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7281424, SUC_lexicon_tags_12 },
    { 0x31a81424, SUC_lexicon_tags_25 },
    { 0x6e881426, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90c81428, SUC_lexicon_tags_11 },
    { 0x16881428, SUC_lexicon_tags_37 },
    { 0xcc48142a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c81437, SUC_lexicon_tags_37 },
    { 0x51481438, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc408143d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59081440, SUC_lexicon_tags_12 },
    { 0x78281441, SUC_lexicon_tags_7 },
    { 0xd0a81441, SUC_lexicon_tags_33 },
    { 0x18881442, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe81446, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f08144a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7488144c, SUC_lexicon_tags_47 },
    { 0x1ac8144d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec81451, SUC_lexicon_tags_15 },
    { 0x88a81452, SUC_lexicon_tags_17 },
    { 0x28681451, SUC_lexicon_tags_87 },
    { 0xd6281454, SUC_lexicon_tags_8 },
    { 0x16481454, SUC_lexicon_tags_33 },
    { 0x8f481456, SUC_lexicon_tags_37 },
    { 0x11c81456, SUC_lexicon_tags_37 },
    { 0x76a81456, SUC_lexicon_tags_4 },
    { 0xac881454, SUC_lexicon_tags_59 },
    { 0xeaa8145a, SUC_lexicon_tags_37 },
    { 0x81a8145b, SUC_lexicon_tags_37 },
    { 0x75a81456, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe208145f, SUC_lexicon_tags_37 },
    { 0x90c81460, SUC_lexicon_tags_1 },
    { 0xa8881460, SUC_lexicon_tags_1 },
    { 0x73481460, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a8146a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c8146e, SUC_lexicon_tags_35 },
    { 0x1268146f, SUC_lexicon_tags_7 },
    { 0x6f681470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc81473, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x86c81475, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a81479, SUC_lexicon_tags_67 },
    { 0x9188147a, SUC_lexicon_tags_25 },
    { 0x7e28147b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c81480, SUC_lexicon_tags_25 },
    { 0x2c881481, SUC_lexicon_tags_14 },
    { 0x15481482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6881485, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde81488, SUC_lexicon_tags_37 },
    { 0xcb681489, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a28148c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f68148f, SUC_lexicon_tags_1 },
    { 0xb1a81490, SUC_lexicon_tags_7 },
    { 0xb3081491, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c81494, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98681497, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa5c81499, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a814a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ce814a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78814a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc814aa, SUC_lexicon_tags_14 },
    { 0x9f8814ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x426814ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80814b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50814b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4814bf, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x732814c1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36814c6, SUC_lexicon_tags_26 },
    { 0xb66814c7, SUC_lexicon_tags_11 },
    { 0xa72814c7, SUC_lexicon_tags_12 },
    { 0xede814c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30814cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a814cf, SUC_lexicon_tags_84 },
    { 0x366814cf, SUC_lexicon_tags_38 },
    { 0xfec814cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a814d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4814db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70814e3, SUC_lexicon_tags_37 },
    { 0xb1e814e3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x12e814e6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c814e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x360814ec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2fc814ee, SUC_lexicon_tags_27 },
    { 0xc18814ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0814f7, SUC_lexicon_tags_25 },
    { 0xd50814f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476814fe, SUC_lexicon_tags_11 },
    { 0xc7a814ff, SUC_lexicon_tags_6 },
    { 0xca6814ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70681504, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac88150b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad881510, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b081519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f08151f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c81526, SUC_lexicon_tags_0 },
    { 0x25c81527, SUC_lexicon_tags_49 },
    { 0xc1081528, SUC_lexicon_tags_37 },
    { 0x4ec81529, SUC_lexicon_tags_12 },
    { 0xa281529, SUC_lexicon_tags_19 },
    { 0xd688152b, SUC_lexicon_tags_101 },
    { 0x7dc8152c, SUC_lexicon_tags_7 },
    { 0xc9c8152d, SUC_lexicon_tags_7 },
    { 0x7468152b, SUC_lexicon_tags_6 },
    { 0x4268152f, SUC_lexicon_tags_17 },
    { 0xb528152e, SUC_lexicon_tags_30 },
    { 0x13a8152f, SUC_lexicon_tags_17 },
    { 0xd668152b, SUC_lexicon_tags_12 },
    { 0xe1c81532, SUC_lexicon_tags_15 },
    { 0x9fe81530, SUC_lexicon_tags_43 },
    { 0x85a81535, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90281537, SUC_lexicon_tags_7 },
    { 0x10e81538, SUC_lexicon_tags_19 },
    { 0x21681538, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45281543, SUC_lexicon_tags_17 },
    { 0xea481544, SUC_lexicon_tags_45 },
    { 0xbb281543, SUC_lexicon_tags_33 },
    { 0x9fa81546, SUC_lexicon_tags_37 },
    { 0x1881546, SUC_lexicon_tags_19 },
    { 0xa9881547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8968154a, SUC_lexicon_tags_37 },
    { 0xa6a8154b, SUC_lexicon_tags_1 },
    { 0x49c8154b, SUC_lexicon_tags_25 },
    { 0x2908154d, SUC_lexicon_tags_15 },
    { 0x5308154d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb281550, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e81555, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc6a81557, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f481559, SUC_lexicon_tags_12 },
    { 0xf1681559, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6528155f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe81567, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x51a81569, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1388156c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e8156f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a81578, SUC_lexicon_tags_12 },
    { 0xd0481579, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c81582, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xca681585, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb8681587, SUC_lexicon_tags_80 },
    { 0x90881588, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9708158a, SUC_lexicon_tags_1 },
    { 0xad68158b, SUC_lexicon_tags_11 },
    { 0x1268158c, SUC_lexicon_tags_60 },
    { 0x848158a, SUC_lexicon_tags_38 },
    { 0x2648158e, SUC_lexicon_tags_23 },
    { 0xf148158c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7881594, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xcfa81596, SUC_lexicon_tags_1 },
    { 0xd3281596, SUC_lexicon_tags_27 },
    { 0xd8681598, SUC_lexicon_tags_29 },
    { 0x81a81598, SUC_lexicon_tags_31 },
    { 0xb3a81598, SUC_lexicon_tags_2 },
    { 0x3f681596, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4648159d, SUC_lexicon_tags_38 },
    { 0x8ca8159e, SUC_lexicon_tags_14 },
    { 0x2b28159f, SUC_lexicon_tags_33 },
    { 0x5e8159e, SUC_lexicon_tags_1 },
    { 0x31c815a1, SUC_lexicon_tags_65 },
    { 0x522815a2, SUC_lexicon_tags_11 },
    { 0x926815a2, SUC_lexicon_tags_11 },
    { 0x3d0815a4, SUC_lexicon_tags_62 },
    { 0x8e0815a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0815a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4815ab, SUC_lexicon_tags_46 },
    { 0xeaa815ab, SUC_lexicon_tags_17 },
    { 0x87c815ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118815b1, SUC_lexicon_tags_47 },
    { 0xfae815b2, SUC_lexicon_tags_65 },
    { 0xe92815b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e815b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x972815bb, SUC_lexicon_tags_38 },
    { 0x16815bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x400815bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x170815c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec815c4, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x6a4815c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0815ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdec815cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58815cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a815d5, SUC_lexicon_tags_37 },
    { 0x8d0815d6, SUC_lexicon_tags_11 },
    { 0xdc6815d7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf02815d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c815e0, SUC_lexicon_tags_21 },
    { 0x4da815e1, SUC_lexicon_tags_11 },
    { 0xcdc815e2, SUC_lexicon_tags_27 },
    { 0x364815e3, SUC_lexicon_tags_26 },
    { 0x3fc815e4, SUC_lexicon_tags_7 },
    { 0xc40815e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4815e8, SUC_lexicon_tags_33 },
    { 0xa8a815e9, SUC_lexicon_tags_26 },
    { 0x22e815e8, SUC_lexicon_tags_45 },
    { 0x4a8815eb, SUC_lexicon_tags_11 },
    { 0x4e8815eb, SUC_lexicon_tags_19 },
    { 0x388815ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0815f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a815f4, SUC_lexicon_tags_38 },
    { 0xc9a815f5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a815fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6815fe, SUC_lexicon_tags_118 },
    { 0xeaa815ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa481601, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a81604, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe5881606, SUC_lexicon_tags_2 },
    { 0x4a281606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc948160b, SUC_lexicon_tags_11 },
    { 0x4c68160c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b28160f, SUC_lexicon_tags_26 },
    { 0x4748160f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x87481612, SUC_lexicon_tags_54 },
    { 0x99481613, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e681615, SUC_lexicon_tags_2 },
    { 0x69681615, SUC_lexicon_tags_56 },
    { 0x8da81616, SUC_lexicon_tags_14 },
    { 0x8e681615, SUC_lexicon_tags_8 },
    { 0x77a81619, SUC_lexicon_tags_63 },
    { 0xe208161a, SUC_lexicon_tags_37 },
    { 0x6888161b, SUC_lexicon_tags_1 },
    { 0x6728161a, SUC_lexicon_tags_16 },
    { 0xfdc8161d, SUC_lexicon_tags_0 },
    { 0x59681616, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xafc81620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b681625, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c081627, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a8162a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc28162f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e81632, SUC_lexicon_tags_25 },
    { 0x51081632, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c81636, SUC_lexicon_tags_17 },
    { 0x3e881637, SUC_lexicon_tags_15 },
    { 0xe2281638, SUC_lexicon_tags_47 },
    { 0xf4881636, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9668163b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b08163f, SUC_lexicon_tags_63 },
    { 0xf2c81640, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18281644, SUC_lexicon_tags_37 },
    { 0x93481645, SUC_lexicon_tags_14 },
    { 0x4aa81646, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd1481648, SUC_lexicon_tags_0 },
    { 0xa5481649, SUC_lexicon_tags_37 },
    { 0x4fc8164a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e8164d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8688164f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf881652, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9b281654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5481656, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd8c81658, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2548165a, SUC_lexicon_tags_2 },
    { 0xef08165b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c81660, SUC_lexicon_tags_28 },
    { 0x5de81661, SUC_lexicon_tags_17 },
    { 0xde681660, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6668166a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a8166d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c81670, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4081675, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb948167c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8aa8167e, SUC_lexicon_tags_3 },
    { 0x9ae8167e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81681681, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d881684, SUC_lexicon_tags_47 },
    { 0x92a81684, SUC_lexicon_tags_66 },
    { 0x41681686, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf281689, SUC_lexicon_tags_33 },
    { 0x5a68168a, SUC_lexicon_tags_15 },
    { 0xe9a8168a, SUC_lexicon_tags_49 },
    { 0xa008168c, SUC_lexicon_tags_11 },
    { 0xfb48168b, SUC_lexicon_tags_38 },
    { 0xd188168e, SUC_lexicon_tags_4 },
    { 0x55481689, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51681693, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee81696, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9528169c, SUC_lexicon_tags_27 },
    { 0x68169d, SUC_lexicon_tags_3 },
    { 0xd928169e, SUC_lexicon_tags_8 },
    { 0x53c8169f, SUC_lexicon_tags_11 },
    { 0xa848169f, SUC_lexicon_tags_65 },
    { 0x78c816a0, SUC_lexicon_tags_19 },
    { 0x9ea816a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1c816a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x426816a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a816ae, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee816b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4816b7, SUC_lexicon_tags_0 },
    { 0x7d4816b7, SUC_lexicon_tags_17 },
    { 0x656816b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6816bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148816c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc816c4, SUC_lexicon_tags_174 },
    { 0x6f4816c5, SUC_lexicon_tags_45 },
    { 0xd8816c6, SUC_lexicon_tags_2 },
    { 0x806816c5, SUC_lexicon_tags_23 },
    { 0x38c816c8, SUC_lexicon_tags_12 },
    { 0xae4816c6, SUC_lexicon_tags_11 },
    { 0x390816c9, SUC_lexicon_tags_37 },
    { 0xfde816cb, SUC_lexicon_tags_15 },
    { 0xa4816cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e816d7, SUC_lexicon_tags_15 },
    { 0x75e816d7, SUC_lexicon_tags_11 },
    { 0x754816d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd74816db, SUC_lexicon_tags_17 },
    { 0x3f6816dc, SUC_lexicon_tags_47 },
    { 0xeba816dd, SUC_lexicon_tags_1 },
    { 0x9a6816db, SUC_lexicon_tags_32 },
    { 0xf64816df, SUC_lexicon_tags_30 },
    { 0xb60816df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd44816e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a816e5, SUC_lexicon_tags_38 },
    { 0x5ca816e6, SUC_lexicon_tags_30 },
    { 0x50a816e7, SUC_lexicon_tags_54 },
    { 0x578816e8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e816ec, SUC_lexicon_tags_85 },
    { 0xad8816ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6816f1, SUC_lexicon_tags_26 },
    { 0x778816f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4816f5, SUC_lexicon_tags_39 },
    { 0x86816f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x622816f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e2816fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0816fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e81701, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a81706, SUC_lexicon_tags_11 },
    { 0x30881706, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc7481709, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc8170c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be81711, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x26281713, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33281719, SUC_lexicon_tags_47 },
    { 0x72a8171a, SUC_lexicon_tags_0 },
    { 0xd228171b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8881723, SUC_lexicon_tags_2 },
    { 0x12081724, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xdc081726, SUC_lexicon_tags_15 },
    { 0xeae81727, SUC_lexicon_tags_6 },
    { 0x55881728, SUC_lexicon_tags_0 },
    { 0xd5881726, SUC_lexicon_tags_11 },
    { 0x8be81726, SUC_lexicon_tags_33 },
    { 0x9868172b, SUC_lexicon_tags_25 },
    { 0xafe8172c, SUC_lexicon_tags_1 },
    { 0x3468172c, SUC_lexicon_tags_13 },
    { 0x9388172d, SUC_lexicon_tags_26 },
    { 0x86a8172f, SUC_lexicon_tags_31 },
    { 0xce281730, SUC_lexicon_tags_27 },
    { 0x20481731, SUC_lexicon_tags_27 },
    { 0x4ce81727, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd1081734, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c8173e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c81741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34c81743, SUC_lexicon_tags_25 },
    { 0x9081744, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12081746, SUC_lexicon_tags_28 },
    { 0xb7881747, SUC_lexicon_tags_5 },
    { 0xee681746, SUC_lexicon_tags_118 },
    { 0xc8a81749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d68174b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe448174e, SUC_lexicon_tags_48 },
    { 0x9648174f, SUC_lexicon_tags_23 },
    { 0xa3e81750, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f681755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8e81757, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6488175f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb081763, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bc81765, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6081768, SUC_lexicon_tags_69 },
    { 0x7fc81769, SUC_lexicon_tags_15 },
    { 0x9888176a, SUC_lexicon_tags_1 },
    { 0xc868176b, SUC_lexicon_tags_27 },
    { 0x1e68176b, SUC_lexicon_tags_15 },
    { 0x7708176b, SUC_lexicon_tags_15 },
    { 0x5d48176e, SUC_lexicon_tags_37 },
    { 0xc048176f, SUC_lexicon_tags_25 },
    { 0x5cc8176f, SUC_lexicon_tags_33 },
    { 0xb4a8176c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc5c81773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1081779, SUC_lexicon_tags_26 },
    { 0x628177a, SUC_lexicon_tags_7 },
    { 0x1188177b, SUC_lexicon_tags_17 },
    { 0x59c8177c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a481780, SUC_lexicon_tags_29 },
    { 0xd5081781, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a81784, SUC_lexicon_tags_45 },
    { 0x39081785, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4368178c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xae48178f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xd7081791, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4aa81793, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f881797, SUC_lexicon_tags_6 },
    { 0x1c081797, SUC_lexicon_tags_136 },
    { 0x66881799, SUC_lexicon_tags_14 },
    { 0x99081799, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb928179c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618817a1, SUC_lexicon_tags_1 },
    { 0x542817a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a817a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8817ac, SUC_lexicon_tags_48 },
    { 0x744817ad, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22817b0, SUC_lexicon_tags_11 },
    { 0x392817b1, SUC_lexicon_tags_12 },
    { 0x270817b2, SUC_lexicon_tags_1 },
    { 0x42a817b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x432817b5, SUC_lexicon_tags_37 },
    { 0x7f8817b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a817bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5d4817bd, SUC_lexicon_tags_12 },
    { 0x2fc817be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd6a817c0, SUC_lexicon_tags_43 },
    { 0xa7e817c1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be817c5, SUC_lexicon_tags_8 },
    { 0x82e817c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x190817c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x574817cb, SUC_lexicon_tags_17 },
    { 0xaac817cb, SUC_lexicon_tags_37 },
    { 0x2fe817cd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc3e817cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64e817d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a817d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d8817d6, SUC_lexicon_tags_63 },
    { 0xb8817d7, SUC_lexicon_tags_11 },
    { 0x946817d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8817de, SUC_lexicon_tags_341 },
    { 0x39e817df, SUC_lexicon_tags_66 },
    { 0xa12817df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae817e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4817e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de817eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x450817ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16817f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x950817f2, SUC_lexicon_tags_7 },
    { 0x4bc817f3, SUC_lexicon_tags_35 },
    { 0x742817f3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x402817f6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4f4817f8, SUC_lexicon_tags_3 },
    { 0x300817f8, SUC_lexicon_tags_25 },
    { 0x396817fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c81804, SUC_lexicon_tags_22 },
    { 0x69e81804, SUC_lexicon_tags_43 },
    { 0x73881804, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x30481809, SUC_lexicon_tags_17 },
    { 0x60a81809, SUC_lexicon_tags_11 },
    { 0x7708180b, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x8928180e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x96281811, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a81815, SUC_lexicon_tags_63 },
    { 0xc9a81816, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x93281818, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa348181c, SUC_lexicon_tags_81 },
    { 0x3668181d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x89481820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5281823, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17681829, SUC_lexicon_tags_49 },
    { 0xe988182a, SUC_lexicon_tags_11 },
    { 0xe248182a, SUC_lexicon_tags_6 },
    { 0x4e8182c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa8182f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea481833, SUC_lexicon_tags_738 },
    { 0xeb081834, SUC_lexicon_tags_25 },
    { 0xe0881833, SUC_lexicon_tags_25 },
    { 0x43681834, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7681839, SUC_lexicon_tags_25 },
    { 0x9648183a, SUC_lexicon_tags_60 },
    { 0x30681839, SUC_lexicon_tags_19 },
    { 0x3468183c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88081841, SUC_lexicon_tags_12 },
    { 0x43881842, SUC_lexicon_tags_10 },
    { 0x1a681843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f281845, SUC_lexicon_tags_55 },
    { 0xcf081846, SUC_lexicon_tags_19 },
    { 0x59281846, SUC_lexicon_tags_17 },
    { 0xa9281848, SUC_lexicon_tags_25 },
    { 0x3c081849, SUC_lexicon_tags_18 },
    { 0xe548184a, SUC_lexicon_tags_47 },
    { 0xd3681849, SUC_lexicon_tags_17 },
    { 0x16a8184c, SUC_lexicon_tags_27 },
    { 0x3da8184c, SUC_lexicon_tags_48 },
    { 0x7908184c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3748185b, SUC_lexicon_tags_15 },
    { 0x308185c, SUC_lexicon_tags_11 },
    { 0x8e68185c, SUC_lexicon_tags_25 },
    { 0x22c8185b, SUC_lexicon_tags_15 },
    { 0xc408185f, SUC_lexicon_tags_15 },
    { 0xc81860, SUC_lexicon_tags_17 },
    { 0xa5c81860, SUC_lexicon_tags_1 },
    { 0xd9e8185d, SUC_lexicon_tags_17 },
    { 0x62281863, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7481869, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c81870, SUC_lexicon_tags_15 },
    { 0xa5881871, SUC_lexicon_tags_46 },
    { 0x55681872, SUC_lexicon_tags_37 },
    { 0xf281873, SUC_lexicon_tags_14 },
    { 0xf4a81873, SUC_lexicon_tags_30 },
    { 0x52081875, SUC_lexicon_tags_8 },
    { 0xf2881876, SUC_lexicon_tags_34 },
    { 0x33681876, SUC_lexicon_tags_37 },
    { 0xd3e81878, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc8187e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64081884, SUC_lexicon_tags_37 },
    { 0x21c81885, SUC_lexicon_tags_37 },
    { 0x4c081885, SUC_lexicon_tags_158 },
    { 0x7ae81887, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd28188d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4b28188f, SUC_lexicon_tags_12 },
    { 0xa0a81890, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9281893, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c81896, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x73481898, SUC_lexicon_tags_12 },
    { 0xcbc81898, SUC_lexicon_tags_1 },
    { 0xa428189a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368189f, SUC_lexicon_tags_17 },
    { 0x7ba8189f, SUC_lexicon_tags_61 },
    { 0xc0818a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa80818a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2818a6, SUC_lexicon_tags_17 },
    { 0x286818a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690818ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4818b4, SUC_lexicon_tags_23 },
    { 0xda2818b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x890818b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36818ba, SUC_lexicon_tags_17 },
    { 0xb76818ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c818be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e818c4, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76818ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c818cd, SUC_lexicon_tags_1 },
    { 0xd62818cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c6818d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a818d4, SUC_lexicon_tags_15 },
    { 0xe48818d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2818df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x228818e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf82818e4, SUC_lexicon_tags_7 },
    { 0x174818e5, SUC_lexicon_tags_3 },
    { 0x9f0818e5, SUC_lexicon_tags_33 },
    { 0xbc8818e7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14818eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6818ef, SUC_lexicon_tags_12 },
    { 0x5a818f0, SUC_lexicon_tags_16 },
    { 0xdc8818f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37c818f3, SUC_lexicon_tags_23 },
    { 0x868818f4, SUC_lexicon_tags_25 },
    { 0xe68818f3, SUC_lexicon_tags_37 },
    { 0xc60818f5, SUC_lexicon_tags_11 },
    { 0x242818f7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe0e818f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4818fd, SUC_lexicon_tags_38 },
    { 0x1ae818fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec81904, SUC_lexicon_tags_48 },
    { 0x82481905, SUC_lexicon_tags_2 },
    { 0x6ce81906, SUC_lexicon_tags_11 },
    { 0x38281907, SUC_lexicon_tags_11 },
    { 0x33e81907, SUC_lexicon_tags_47 },
    { 0x52081909, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8788190b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d68190d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x57681910, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9081914, SUC_lexicon_tags_46 },
    { 0x88681914, SUC_lexicon_tags_11 },
    { 0xc2481916, SUC_lexicon_tags_12 },
    { 0x4f281916, SUC_lexicon_tags_37 },
    { 0xecc81918, SUC_lexicon_tags_25 },
    { 0x8ce81917, SUC_lexicon_tags_23 },
    { 0xeb88191a, SUC_lexicon_tags_38 },
    { 0x6328191b, SUC_lexicon_tags_7 },
    { 0x17881917, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c8191f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1be81921, SUC_lexicon_tags_14 },
    { 0x5681922, SUC_lexicon_tags_12 },
    { 0x54481922, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4681926, SUC_lexicon_tags_9 },
    { 0xc3e81927, SUC_lexicon_tags_35 },
    { 0x50881928, SUC_lexicon_tags_12 },
    { 0xe9c81927, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9168192e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a081931, SUC_lexicon_tags_48 },
    { 0xfba81932, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf881937, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc08193d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4881942, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a81945, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c81948, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc488194c, SUC_lexicon_tags_0 },
    { 0xd468194d, SUC_lexicon_tags_17 },
    { 0xd168194e, SUC_lexicon_tags_15 },
    { 0xbfa8194e, SUC_lexicon_tags_35 },
    { 0x29a8194c, SUC_lexicon_tags_31 },
    { 0xf108194d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c81956, SUC_lexicon_tags_2 },
    { 0xa5c81957, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b08195a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4b68195c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c68195e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13481960, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca81965, SUC_lexicon_tags_18 },
    { 0x44281966, SUC_lexicon_tags_37 },
    { 0x66881967, SUC_lexicon_tags_25 },
    { 0x36081967, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe68196e, SUC_lexicon_tags_40 },
    { 0x9468196f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8681971, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55081976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15681978, SUC_lexicon_tags_31 },
    { 0xb8881979, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08197d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa988197f, SUC_lexicon_tags_7 },
    { 0xc4c8197f, SUC_lexicon_tags_11 },
    { 0xc2a81981, SUC_lexicon_tags_33 },
    { 0xfc081982, SUC_lexicon_tags_2 },
    { 0x49081981, SUC_lexicon_tags_376 },
    { 0xe8481983, SUC_lexicon_tags_15 },
    { 0x80881984, SUC_lexicon_tags_27 },
    { 0x3e481980, SUC_lexicon_tags_28 },
    { 0x65c81987, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e8198a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7808198e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78081994, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a28199a, SUC_lexicon_tags_65 },
    { 0x1de8199a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a8199e, SUC_lexicon_tags_25 },
    { 0xa088199e, SUC_lexicon_tags_68 },
    { 0x9e08199f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a819a3, SUC_lexicon_tags_43 },
    { 0x1c4819a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2819a9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x142819ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c819b0, SUC_lexicon_tags_1 },
    { 0x292819b1, SUC_lexicon_tags_39 },
    { 0x794819b2, SUC_lexicon_tags_12 },
    { 0x8c6819b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6819b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x748819b9, SUC_lexicon_tags_17 },
    { 0xcac819ba, SUC_lexicon_tags_15 },
    { 0xa56819bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa819c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca819c5, SUC_lexicon_tags_132 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256819c9, SUC_lexicon_tags_61 },
    { 0x1b6819ca, SUC_lexicon_tags_33 },
    { 0x9fe819cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a819d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18819da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340819e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc4e819e2, SUC_lexicon_tags_0 },
    { 0x2d0819e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda819e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0819e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc70819eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7de819ed, SUC_lexicon_tags_17 },
    { 0x6e8819ee, SUC_lexicon_tags_25 },
    { 0xc2819ef, SUC_lexicon_tags_6 },
    { 0x9e6819ef, SUC_lexicon_tags_63 },
    { 0xa22819ee, SUC_lexicon_tags_21 },
    { 0x892819f2, SUC_lexicon_tags_7 },
    { 0x534819ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4819f6, SUC_lexicon_tags_17 },
    { 0xf76819f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06819fb, SUC_lexicon_tags_17 },
    { 0xd9e819fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34881a01, SUC_lexicon_tags_3 },
    { 0x48281a02, SUC_lexicon_tags_7 },
    { 0x62a81a02, SUC_lexicon_tags_1 },
    { 0x6dc81a04, SUC_lexicon_tags_1 },
    { 0xd2e81a01, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x31281a08, SUC_lexicon_tags_46 },
    { 0xede81a09, SUC_lexicon_tags_79 },
    { 0x86c81a0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c81a0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa81a14, SUC_lexicon_tags_25 },
    { 0xe7881a15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee81a18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36281a1d, SUC_lexicon_tags_17 },
    { 0x3da81a1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a81a24, SUC_lexicon_tags_12 },
    { 0xcfc81a25, SUC_lexicon_tags_0 },
    { 0xb1c81a24, SUC_lexicon_tags_68 },
    { 0xf281a27, SUC_lexicon_tags_49 },
    { 0x1de81a28, SUC_lexicon_tags_25 },
    { 0x52081a27, SUC_lexicon_tags_15 },
    { 0xbca81a2a, SUC_lexicon_tags_12 },
    { 0xec681a2b, SUC_lexicon_tags_11 },
    { 0xe4681a2c, SUC_lexicon_tags_37 },
    { 0xbba81a2d, SUC_lexicon_tags_17 },
    { 0x57e81a2a, SUC_lexicon_tags_23 },
    { 0x7b281a2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc6a81a31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e81a34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3c81a36, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da81a3a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xbf681a3c, SUC_lexicon_tags_9 },
    { 0x25c81a3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33881a40, SUC_lexicon_tags_23 },
    { 0xe7a81a41, SUC_lexicon_tags_15 },
    { 0x5f681a41, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c81a46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae81a4b, SUC_lexicon_tags_33 },
    { 0xd4c81a4c, SUC_lexicon_tags_60 },
    { 0xe7281a4c, SUC_lexicon_tags_8 },
    { 0x51481a4b, SUC_lexicon_tags_19 },
    { 0x36681a4f, SUC_lexicon_tags_14 },
    { 0x8481a50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa81a53, SUC_lexicon_tags_56 },
    { 0xe4a81a53, SUC_lexicon_tags_14 },
    { 0x8b881a55, SUC_lexicon_tags_47 },
    { 0x77a81a56, SUC_lexicon_tags_17 },
    { 0x4d881a55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e81a5a, SUC_lexicon_tags_2 },
    { 0xe7c81a5b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc6e81a5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47281a61, SUC_lexicon_tags_7 },
    { 0xb2681a62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ce81a64, SUC_lexicon_tags_56 },
    { 0x53481a65, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c81a6b, SUC_lexicon_tags_44 },
    { 0x51881a6c, SUC_lexicon_tags_17 },
    { 0x28281a6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x53e81a6f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x63081a72, SUC_lexicon_tags_34 },
    { 0xcba81a73, SUC_lexicon_tags_47 },
    { 0xf5081a74, SUC_lexicon_tags_17 },
    { 0x40081a75, SUC_lexicon_tags_23 },
    { 0x5f281a74, SUC_lexicon_tags_38 },
    { 0xd8481a77, SUC_lexicon_tags_28 },
    { 0x4e681a78, SUC_lexicon_tags_32 },
    { 0x1e081a79, SUC_lexicon_tags_25 },
    { 0x8de81a75, SUC_lexicon_tags_55 },
    { 0x6481a7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e81a7e, SUC_lexicon_tags_9 },
    { 0xf0281a7f, SUC_lexicon_tags_60 },
    { 0xa3e81a7f, SUC_lexicon_tags_68 },
    { 0x62c81a81, SUC_lexicon_tags_1 },
    { 0x2c881a7f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c81a87, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x68681a8a, SUC_lexicon_tags_23 },
    { 0x65081a8b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd681a8e, SUC_lexicon_tags_8 },
    { 0xcee81a8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb681a93, SUC_lexicon_tags_4 },
    { 0xab481a94, SUC_lexicon_tags_4 },
    { 0xe9681a95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53481a99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a81a9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22281aa9, SUC_lexicon_tags_17 },
    { 0xd1e81aaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e81aad, SUC_lexicon_tags_2 },
    { 0x3ba81aad, SUC_lexicon_tags_12 },
    { 0x2081aaf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca81ab3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19681ab7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c81abb, SUC_lexicon_tags_2 },
    { 0x9ca81abc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96081abf, SUC_lexicon_tags_15 },
    { 0x12881ac0, SUC_lexicon_tags_66 },
    { 0xf8a81ac1, SUC_lexicon_tags_37 },
    { 0x7c481ac2, SUC_lexicon_tags_14 },
    { 0x26281ac3, SUC_lexicon_tags_66 },
    { 0xe8e81ac1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf281ac6, SUC_lexicon_tags_11 },
    { 0xa7a81ac6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x46681ac9, SUC_lexicon_tags_65 },
    { 0x37e81aca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc4a81acc, SUC_lexicon_tags_19 },
    { 0xe281acd, SUC_lexicon_tags_27 },
    { 0x67081ace, SUC_lexicon_tags_27 },
    { 0x79e81ace, SUC_lexicon_tags_30 },
    { 0xb2e81ad0, SUC_lexicon_tags_23 },
    { 0xbf281ad1, SUC_lexicon_tags_25 },
    { 0x2a481ad2, SUC_lexicon_tags_48 },
    { 0x8a081ad2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0e81ad5, SUC_lexicon_tags_48 },
    { 0x71081ad6, SUC_lexicon_tags_23 },
    { 0xdc281ad7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a881ad9, SUC_lexicon_tags_33 },
    { 0x5e881ad9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e81add, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88281ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78681ae9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e81aec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3881aee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f081af1, SUC_lexicon_tags_29 },
    { 0xf7c81af1, SUC_lexicon_tags_29 },
    { 0xfbe81af3, SUC_lexicon_tags_25 },
    { 0x65c81af1, SUC_lexicon_tags_69 },
    { 0xe2281af5, SUC_lexicon_tags_15 },
    { 0xa1081af6, SUC_lexicon_tags_25 },
    { 0x5a881af7, SUC_lexicon_tags_23 },
    { 0xb4a81af7, SUC_lexicon_tags_31 },
    { 0x87281af9, SUC_lexicon_tags_17 },
    { 0x54481af9, SUC_lexicon_tags_27 },
    { 0xab481afb, SUC_lexicon_tags_2 },
    { 0xc7881afb, SUC_lexicon_tags_25 },
    { 0x74081af2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1681b01, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xabe81b03, SUC_lexicon_tags_25 },
    { 0x44a81b03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e81b0a, SUC_lexicon_tags_106 },
    { 0x4fe81b0a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2081b0d, SUC_lexicon_tags_84 },
    { 0x8ca81b0e, SUC_lexicon_tags_62 },
    { 0xb9a81b0e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20281b15, SUC_lexicon_tags_12 },
    { 0xb9681b16, SUC_lexicon_tags_1 },
    { 0x3d881b17, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7c681b19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5681b1c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa081b22, SUC_lexicon_tags_55 },
    { 0x2881b23, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca081b27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e681b2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5a81b2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59081b2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9081b31, SUC_lexicon_tags_11 },
    { 0xbc881b32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c81b37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98a81b39, SUC_lexicon_tags_719 },
    { 0x21481b39, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd7c81b3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc081b42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x60081b44, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc281b4a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x85a81b4c, SUC_lexicon_tags_11 },
    { 0x6c281b4d, SUC_lexicon_tags_12 },
    { 0xf6081b4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad881b52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2481b55, SUC_lexicon_tags_29 },
    { 0x6bc81b55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ec81b58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e81b62, SUC_lexicon_tags_11 },
    { 0x5d481b62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43681b66, SUC_lexicon_tags_4 },
    { 0xcca81b67, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb2481b69, SUC_lexicon_tags_1 },
    { 0x88681b69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1281b73, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xcba81b75, SUC_lexicon_tags_7 },
    { 0xd9281b76, SUC_lexicon_tags_53 },
    { 0x97e81b77, SUC_lexicon_tags_12 },
    { 0x78481b78, SUC_lexicon_tags_27 },
    { 0xb5e81b76, SUC_lexicon_tags_15 },
    { 0x84681b78, SUC_lexicon_tags_11 },
    { 0x88081b7b, SUC_lexicon_tags_33 },
    { 0x9481b7b, SUC_lexicon_tags_48 },
    { 0xe1081b7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e81b89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c081b8d, SUC_lexicon_tags_4 },
    { 0x61881b8d, SUC_lexicon_tags_17 },
    { 0x28481b8f, SUC_lexicon_tags_37 },
    { 0x33681b90, SUC_lexicon_tags_38 },
    { 0xc8081b90, SUC_lexicon_tags_49 },
    { 0x28881b90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe81b9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c81b9e, SUC_lexicon_tags_8 },
    { 0xd9881b9f, SUC_lexicon_tags_61 },
    { 0xbf281ba0, SUC_lexicon_tags_63 },
    { 0xd7881ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32281ba6, SUC_lexicon_tags_17 },
    { 0x7b681ba7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc281baa, SUC_lexicon_tags_17 },
    { 0xa2481bab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8881bad, SUC_lexicon_tags_12 },
    { 0x6ee81bae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c81bb2, SUC_lexicon_tags_14 },
    { 0xd881bb2, SUC_lexicon_tags_14 },
    { 0x53681bb4, SUC_lexicon_tags_17 },
    { 0x18881bb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e681bb9, SUC_lexicon_tags_11 },
    { 0xee081bba, SUC_lexicon_tags_60 },
    { 0xab281bb9, SUC_lexicon_tags_19 },
    { 0x46c81bbb, SUC_lexicon_tags_11 },
    { 0x52a81bbc, SUC_lexicon_tags_26 },
    { 0xdfa81bbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6881bc1, SUC_lexicon_tags_11 },
    { 0x99281bc1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf2481bc4, SUC_lexicon_tags_2 },
    { 0xef481bc5, SUC_lexicon_tags_43 },
    { 0x7fa81bc4, SUC_lexicon_tags_14 },
    { 0x88c81bc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x64681bc9, SUC_lexicon_tags_26 },
    { 0x22e81bc9, SUC_lexicon_tags_10 },
    { 0x48281bcb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd7081bcd, SUC_lexicon_tags_15 },
    { 0xb9a81bce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7c281bd0, SUC_lexicon_tags_19 },
    { 0x4b681bd1, SUC_lexicon_tags_11 },
    { 0xaa481bd2, SUC_lexicon_tags_32 },
    { 0x47881bd3, SUC_lexicon_tags_14 },
    { 0x9d481bd4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6881bd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74481bdb, SUC_lexicon_tags_25 },
    { 0xf2a81bdc, SUC_lexicon_tags_11 },
    { 0xd5081bdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e81be2, SUC_lexicon_tags_47 },
    { 0x6a881be3, SUC_lexicon_tags_1432 },
    { 0x5d881be2, SUC_lexicon_tags_31 },
    { 0xe1e81be2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8081be9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99681bf0, SUC_lexicon_tags_19 },
    { 0x3881bf0, SUC_lexicon_tags_37 },
    { 0xe2681bf2, SUC_lexicon_tags_15 },
    { 0xfb481bf0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66881bf8, SUC_lexicon_tags_12 },
    { 0xb8c81bf9, SUC_lexicon_tags_21 },
    { 0x89c81bf8, SUC_lexicon_tags_1 },
    { 0xa5081bfa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a881c00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65281c04, SUC_lexicon_tags_1 },
    { 0x93c81c05, SUC_lexicon_tags_2 },
    { 0x11081c05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67281c0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9681c0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8481c15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96481c19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c681c26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8481c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c81c2e, SUC_lexicon_tags_24 },
    { 0xeb281c2f, SUC_lexicon_tags_25 },
    { 0x15a81c2e, SUC_lexicon_tags_37 },
    { 0x5a281c31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e681c33, SUC_lexicon_tags_17 },
    { 0xa0a81c34, SUC_lexicon_tags_14 },
    { 0xbc81c34, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee81c3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c81c40, SUC_lexicon_tags_17 },
    { 0xd1a81c41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c481c46, SUC_lexicon_tags_13 },
    { 0xae881c47, SUC_lexicon_tags_15 },
    { 0xc6e81c46, SUC_lexicon_tags_15 },
    { 0x81681c49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40881c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e81c51, SUC_lexicon_tags_2 },
    { 0xd5481c52, SUC_lexicon_tags_0 },
    { 0x7cc81c53, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2681c59, SUC_lexicon_tags_15 },
    { 0x9c81c5a, SUC_lexicon_tags_45 },
    { 0x6cc81c59, SUC_lexicon_tags_45 },
    { 0xede81c5c, SUC_lexicon_tags_11 },
    { 0xb2281c5c, SUC_lexicon_tags_15 },
    { 0x10881c5e, SUC_lexicon_tags_9 },
    { 0xc9881c5a, SUC_lexicon_tags_35 },
    { 0x1e281c5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd881c63, SUC_lexicon_tags_19 },
    { 0x39081c64, SUC_lexicon_tags_1 },
    { 0x3681c65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3a81c67, SUC_lexicon_tags_7 },
    { 0xef481c68, SUC_lexicon_tags_13 },
    { 0x26c81c69, SUC_lexicon_tags_9 },
    { 0x90a81c6a, SUC_lexicon_tags_29 },
    { 0x39281c6a, SUC_lexicon_tags_15 },
    { 0xbfe81c6c, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x30081c6f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x66481c71, SUC_lexicon_tags_37 },
    { 0xc4681c71, SUC_lexicon_tags_37 },
    { 0x37881c73, SUC_lexicon_tags_32 },
    { 0x97881c74, SUC_lexicon_tags_7 },
    { 0x92681c71, SUC_lexicon_tags_15 },
    { 0x39281c71, SUC_lexicon_tags_2 },
    { 0xa1281c75, SUC_lexicon_tags_4 },
    { 0x39081c77, SUC_lexicon_tags_37 },
    { 0x70a81c76, SUC_lexicon_tags_38 },
    { 0xc6881c73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c81c7c, SUC_lexicon_tags_48 },
    { 0xef081c7d, SUC_lexicon_tags_1 },
    { 0x54281c7d, SUC_lexicon_tags_26 },
    { 0x55a81c7f, SUC_lexicon_tags_33 },
    { 0xfec81c80, SUC_lexicon_tags_6 },
    { 0xb7281c80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8881c85, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a81c88, SUC_lexicon_tags_7 },
    { 0x53481c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e881c8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3481c90, SUC_lexicon_tags_12 },
    { 0xb3281c91, SUC_lexicon_tags_27 },
    { 0xfd681c92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37281c94, SUC_lexicon_tags_15 },
    { 0xb1a81c95, SUC_lexicon_tags_26 },
    { 0xb1681c96, SUC_lexicon_tags_1 },
    { 0x66e81c96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c81c9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4881c9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa9281ca1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86881ca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e681cab, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2881caf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xada81cb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde881cb7, SUC_lexicon_tags_28 },
    { 0x61a81cb8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e81cbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a81cc3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c081cca, SUC_lexicon_tags_15 },
    { 0x45281ccb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80681ccd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba81cd0, SUC_lexicon_tags_17 },
    { 0x7aa81cd1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a81cd4, SUC_lexicon_tags_30 },
    { 0x23281cd4, SUC_lexicon_tags_25 },
    { 0x89681cd4, SUC_lexicon_tags_1720 },
    { 0, NULL },
    { 0x19481cd8, SUC_lexicon_tags_25 },
    { 0x5cc81cd9, SUC_lexicon_tags_14 },
    { 0x36481cd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a81ce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a81ce6, SUC_lexicon_tags_4 },
    { 0xb0081ce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac81ce9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b681ced, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc281cf2, SUC_lexicon_tags_19 },
    { 0xaec81cf3, SUC_lexicon_tags_11 },
    { 0xf6481cf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19c81cf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0e81cf8, SUC_lexicon_tags_25 },
    { 0x61e81cf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39481cfb, SUC_lexicon_tags_27 },
    { 0x1ea81cfc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d881cfe, SUC_lexicon_tags_17 },
    { 0x2e281cff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c81d06, SUC_lexicon_tags_29 },
    { 0x9b881d06, SUC_lexicon_tags_61 },
    { 0x22281d08, SUC_lexicon_tags_55 },
    { 0xe3e81d07, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xba681d0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6281d15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79881d17, SUC_lexicon_tags_25 },
    { 0x16081d17, SUC_lexicon_tags_85 },
    { 0x52281d19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e81d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e881d1f, SUC_lexicon_tags_7 },
    { 0x1b281d20, SUC_lexicon_tags_31 },
    { 0x9c681d21, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a81d27, SUC_lexicon_tags_37 },
    { 0x32c81d28, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e81d32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce81d37, SUC_lexicon_tags_14 },
    { 0xaf881d37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee281d3a, SUC_lexicon_tags_25 },
    { 0x56481d3b, SUC_lexicon_tags_47 },
    { 0x7ac81d3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae81d42, SUC_lexicon_tags_7 },
    { 0xfec81d42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf1681d45, SUC_lexicon_tags_14 },
    { 0xa081d46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14281d49, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14281d4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x99281d52, SUC_lexicon_tags_25 },
    { 0x35881d53, SUC_lexicon_tags_14 },
    { 0x74a81d54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba81d57, SUC_lexicon_tags_39 },
    { 0x9f281d57, SUC_lexicon_tags_30 },
    { 0xe481d59, SUC_lexicon_tags_61 },
    { 0xff681d5a, SUC_lexicon_tags_12 },
    { 0x41c81d59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5881d5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7881d5f, SUC_lexicon_tags_43 },
    { 0xee881d60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5881d6d, SUC_lexicon_tags_47 },
    { 0xcfc81d6e, SUC_lexicon_tags_98 },
    { 0xed481d6f, SUC_lexicon_tags_0 },
    { 0xeb281d70, SUC_lexicon_tags_25 },
    { 0x61481d6e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b681d74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c881d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86681d7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7ec81d7d, SUC_lexicon_tags_2 },
    { 0x70a81d7e, SUC_lexicon_tags_1 },
    { 0x4d481d7f, SUC_lexicon_tags_7 },
    { 0xc4481d80, SUC_lexicon_tags_21 },
    { 0xd6481d81, SUC_lexicon_tags_45 },
    { 0x26881d81, SUC_lexicon_tags_14 },
    { 0xa8e81d83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc681d86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc881d8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd2681d8d, SUC_lexicon_tags_34 },
    { 0x4ce81d8e, SUC_lexicon_tags_12 },
    { 0xa2281d8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d481d94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f681d97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7681d9a, SUC_lexicon_tags_0 },
    { 0x47881d9a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9081da2, SUC_lexicon_tags_12 },
    { 0x6481da3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf281da6, SUC_lexicon_tags_43 },
    { 0x99081da7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae81daf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde81db4, SUC_lexicon_tags_12 },
    { 0x1ce81db5, SUC_lexicon_tags_17 },
    { 0x87e81db5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa81db8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc481dbe, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c81dc5, SUC_lexicon_tags_37 },
    { 0x12081dc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6481dce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e081dd1, SUC_lexicon_tags_7 },
    { 0xd7281dd2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a81dd6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63881ddd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c81de1, SUC_lexicon_tags_37 },
    { 0x2a481de1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a081de5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a481df1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c481df6, SUC_lexicon_tags_11 },
    { 0xdc481df7, SUC_lexicon_tags_37 },
    { 0x3b281df8, SUC_lexicon_tags_38 },
    { 0x78081df9, SUC_lexicon_tags_38 },
    { 0x91c81dfa, SUC_lexicon_tags_46 },
    { 0x9de81dfb, SUC_lexicon_tags_0 },
    { 0x84481dfc, SUC_lexicon_tags_1 },
    { 0xcec81dfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33081e06, SUC_lexicon_tags_29 },
    { 0xb5281e06, SUC_lexicon_tags_14 },
    { 0x33e81e08, SUC_lexicon_tags_63 },
    { 0xc3481e09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff881e0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5281e10, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba81e14, SUC_lexicon_tags_17 },
    { 0x71281e15, SUC_lexicon_tags_30 },
    { 0x89081e16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71481e18, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64281e20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e081e28, SUC_lexicon_tags_15 },
    { 0x71681e28, SUC_lexicon_tags_25 },
    { 0x85081e2a, SUC_lexicon_tags_11 },
    { 0xfc681e2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a81e36, SUC_lexicon_tags_42 },
    { 0x1f481e36, SUC_lexicon_tags_70 },
    { 0x6c81e37, SUC_lexicon_tags_12 },
    { 0xe0881e36, SUC_lexicon_tags_37 },
    { 0x1fe81e38, SUC_lexicon_tags_49 },
    { 0x37681e38, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc81e3e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57081e43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15481e47, SUC_lexicon_tags_11 },
    { 0x7f481e48, SUC_lexicon_tags_2 },
    { 0x66481e49, SUC_lexicon_tags_40 },
    { 0x5e081e4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3281e4c, SUC_lexicon_tags_12 },
    { 0xb3c81e4d, SUC_lexicon_tags_27 },
    { 0xec881e4d, SUC_lexicon_tags_40 },
    { 0x4de81e4f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x26c81e51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8281e53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6681e5d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5281e61, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2081e64, SUC_lexicon_tags_29 },
    { 0x7aa81e65, SUC_lexicon_tags_17 },
    { 0xeec81e65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d681e69, SUC_lexicon_tags_35 },
    { 0x90e81e6a, SUC_lexicon_tags_4 },
    { 0xbe81e6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c481e70, SUC_lexicon_tags_12 },
    { 0x48a81e70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xba681e74, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c081e77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4681e79, SUC_lexicon_tags_55 },
    { 0x4fe81e7a, SUC_lexicon_tags_27 },
    { 0xb1c81e7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa681e7e, SUC_lexicon_tags_38 },
    { 0xe881e7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe281e85, SUC_lexicon_tags_63 },
    { 0xb0281e86, SUC_lexicon_tags_37 },
    { 0xb1281e85, SUC_lexicon_tags_14 },
    { 0xcd681e88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd681e8a, SUC_lexicon_tags_63 },
    { 0x52e81e8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee81e91, SUC_lexicon_tags_2 },
    { 0xfdc81e92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec81e96, SUC_lexicon_tags_82 },
    { 0xdbc81e96, SUC_lexicon_tags_1 },
    { 0xd4c81e98, SUC_lexicon_tags_37 },
    { 0x7c281e99, SUC_lexicon_tags_16 },
    { 0x8f881e9a, SUC_lexicon_tags_15 },
    { 0xb2c81e99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9bc81e9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9081ea1, SUC_lexicon_tags_25 },
    { 0x6da81ea2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b681ea8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c81eab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e81eae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c81eb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc881eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d481eb7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf5881eb9, SUC_lexicon_tags_2 },
    { 0x17881eba, SUC_lexicon_tags_10 },
    { 0x9ba81ebb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x37881ebd, SUC_lexicon_tags_37 },
    { 0x9b481ebd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7481ec0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x41681ec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae281ec4, SUC_lexicon_tags_45 },
    { 0xbe81ec5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a281ecb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb881ecd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea681ed0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a881ed6, SUC_lexicon_tags_46 },
    { 0x66481ed7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e81edc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc81ee3, SUC_lexicon_tags_23 },
    { 0xc3881ee4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84a81ee6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21081eee, SUC_lexicon_tags_11 },
    { 0x35e81eef, SUC_lexicon_tags_85 },
    { 0xa5481eef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4481ef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8c81ef5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x44281ef8, SUC_lexicon_tags_64 },
    { 0x4c081ef8, SUC_lexicon_tags_12 },
    { 0xc6881ef9, SUC_lexicon_tags_2 },
    { 0xfda81efb, SUC_lexicon_tags_17 },
    { 0xd6881ef8, SUC_lexicon_tags_48 },
    { 0x1b081efa, SUC_lexicon_tags_112 },
    { 0x18e81efe, SUC_lexicon_tags_34 },
    { 0x88a81eff, SUC_lexicon_tags_8 },
    { 0xad881f00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea81f03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e81f07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1681f0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a81f0e, SUC_lexicon_tags_1 },
    { 0x43a81f0e, SUC_lexicon_tags_33 },
    { 0x7ac81f0f, SUC_lexicon_tags_25 },
    { 0x9481f11, SUC_lexicon_tags_87 },
    { 0xcec81f12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e81f15, SUC_lexicon_tags_61 },
    { 0xc6281f16, SUC_lexicon_tags_27 },
    { 0x32481f17, SUC_lexicon_tags_11 },
    { 0x82681f18, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b881f1a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0881f1e, SUC_lexicon_tags_25 },
    { 0x4c881f1e, SUC_lexicon_tags_33 },
    { 0x59c81f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b081f22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7481f24, SUC_lexicon_tags_43 },
    { 0x21281f25, SUC_lexicon_tags_11 },
    { 0xcb081f26, SUC_lexicon_tags_47 },
    { 0x21481f27, SUC_lexicon_tags_29 },
    { 0xb1081f28, SUC_lexicon_tags_30 },
    { 0x21281f28, SUC_lexicon_tags_7 },
    { 0x29c81f2a, SUC_lexicon_tags_9 },
    { 0x2ca81f2b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6a481f2d, SUC_lexicon_tags_17 },
    { 0x21081f2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed081f30, SUC_lexicon_tags_1 },
    { 0x47681f30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d881f33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37881f35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea81f3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2681f43, SUC_lexicon_tags_37 },
    { 0xf2481f44, SUC_lexicon_tags_47 },
    { 0xe6a81f45, SUC_lexicon_tags_49 },
    { 0xe3681f45, SUC_lexicon_tags_11 },
    { 0xf4a81f43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43081f4a, SUC_lexicon_tags_15 },
    { 0xb681f4b, SUC_lexicon_tags_37 },
    { 0x6e681f4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f081f50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96281f54, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31881f5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x31681f5c, SUC_lexicon_tags_23 },
    { 0x95c81f5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a81f60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97281f67, SUC_lexicon_tags_63 },
    { 0x60a81f67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c81f6b, SUC_lexicon_tags_48 },
    { 0x2e081f6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c81f73, SUC_lexicon_tags_60 },
    { 0xe81f74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a81f79, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3d281f7b, SUC_lexicon_tags_29 },
    { 0x18281f7c, SUC_lexicon_tags_25 },
    { 0xa2681f7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c81f86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c81f89, SUC_lexicon_tags_11 },
    { 0x7a81f8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9081f8f, SUC_lexicon_tags_30 },
    { 0x5d481f90, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51881f94, SUC_lexicon_tags_37 },
    { 0xa8c81f95, SUC_lexicon_tags_25 },
    { 0x98a81f95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c281f98, SUC_lexicon_tags_55 },
    { 0xc9881f98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b681f9c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb3881f9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83881fa3, SUC_lexicon_tags_21 },
    { 0xe3281fa4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f681fa9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e81fad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64281faf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba81fb2, SUC_lexicon_tags_38 },
    { 0x69081fb2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62481fbd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e81fc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec81fc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec81fd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ce81fd2, SUC_lexicon_tags_37 },
    { 0x8ec81fd3, SUC_lexicon_tags_8 },
    { 0xdc681fd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe081fd7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa4081fd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e681fdb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc81fe1, SUC_lexicon_tags_39 },
    { 0x5ae81fe1, SUC_lexicon_tags_63 },
    { 0x21281fe2, SUC_lexicon_tags_17 },
    { 0x4a81fe4, SUC_lexicon_tags_31 },
    { 0x30881fe5, SUC_lexicon_tags_33 },
    { 0xe5c81fe5, SUC_lexicon_tags_25 },
    { 0xc5e81fe4, SUC_lexicon_tags_17 },
    { 0xb8681fe8, SUC_lexicon_tags_47 },
    { 0x9dc81fe9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8681feb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c81ff0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x881ff3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb081ff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3881ffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e82002, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34282005, SUC_lexicon_tags_613 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6882009, SUC_lexicon_tags_65 },
    { 0x228200a, SUC_lexicon_tags_16 },
    { 0x59e8200a, SUC_lexicon_tags_15 },
    { 0xe4e8200c, SUC_lexicon_tags_63 },
    { 0xcac82009, SUC_lexicon_tags_12 },
    { 0xddc8200d, SUC_lexicon_tags_29 },
    { 0x8968200f, SUC_lexicon_tags_17 },
    { 0xfa68200d, SUC_lexicon_tags_32 },
    { 0xa508200d, SUC_lexicon_tags_23 },
    { 0xc9082012, SUC_lexicon_tags_151 },
    { 0xf0482013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b282017, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xeec82019, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeea8201b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc708201e, SUC_lexicon_tags_7 },
    { 0xaca8201f, SUC_lexicon_tags_68 },
    { 0xb468201e, SUC_lexicon_tags_9 },
    { 0xa6882020, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c882028, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a82033, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xdc682035, SUC_lexicon_tags_12 },
    { 0x3fc82035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53282038, SUC_lexicon_tags_37 },
    { 0x63e82039, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd328203f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a82045, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x38c82047, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48204b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3ee8204d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa482051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b082053, SUC_lexicon_tags_7 },
    { 0x91882053, SUC_lexicon_tags_15 },
    { 0x19682055, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8282057, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a82060, SUC_lexicon_tags_38 },
    { 0xd282060, SUC_lexicon_tags_11 },
    { 0x4fa82062, SUC_lexicon_tags_49 },
    { 0x13e82063, SUC_lexicon_tags_12 },
    { 0x19482062, SUC_lexicon_tags_1 },
    { 0xb2282062, SUC_lexicon_tags_11 },
    { 0xf6882066, SUC_lexicon_tags_23 },
    { 0x95482066, SUC_lexicon_tags_2 },
    { 0xf5082068, SUC_lexicon_tags_3 },
    { 0x59c82068, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3348206b, SUC_lexicon_tags_2 },
    { 0x6e8206b, SUC_lexicon_tags_6 },
    { 0x5248206c, SUC_lexicon_tags_7 },
    { 0x7188206e, SUC_lexicon_tags_37 },
    { 0xa708206c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d882075, SUC_lexicon_tags_49 },
    { 0xc6e82076, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82882078, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d682080, SUC_lexicon_tags_25 },
    { 0x8e282081, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x21c82083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcea82085, SUC_lexicon_tags_116 },
    { 0x20a82086, SUC_lexicon_tags_37 },
    { 0xeb482086, SUC_lexicon_tags_47 },
    { 0x95a82085, SUC_lexicon_tags_38 },
    { 0x9be82089, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b68208c, SUC_lexicon_tags_19 },
    { 0x10c8208d, SUC_lexicon_tags_2 },
    { 0xd488208e, SUC_lexicon_tags_68 },
    { 0xe3c8208f, SUC_lexicon_tags_16 },
    { 0x1d88208d, SUC_lexicon_tags_2 },
    { 0xc3282091, SUC_lexicon_tags_55 },
    { 0x88882091, SUC_lexicon_tags_17 },
    { 0x19482093, SUC_lexicon_tags_28 },
    { 0x3d482091, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d882098, SUC_lexicon_tags_17 },
    { 0xac482099, SUC_lexicon_tags_17 },
    { 0x5968209a, SUC_lexicon_tags_37 },
    { 0x2fe8209b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef88209e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc4c820a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d2820a2, SUC_lexicon_tags_57 },
    { 0xbae820a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38820a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfa4820a8, SUC_lexicon_tags_11 },
    { 0x7b8820a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x92c820ab, SUC_lexicon_tags_12 },
    { 0x4b8820ab, SUC_lexicon_tags_33 },
    { 0xa0820ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x364820af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6820b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12820ba, SUC_lexicon_tags_8 },
    { 0xda4820ba, SUC_lexicon_tags_25 },
    { 0xf8e820bc, SUC_lexicon_tags_15 },
    { 0x594820bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5e820bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe60820c1, SUC_lexicon_tags_11 },
    { 0x306820c2, SUC_lexicon_tags_16 },
    { 0x57e820c3, SUC_lexicon_tags_30 },
    { 0xa5e820c3, SUC_lexicon_tags_23 },
    { 0xb70820c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d6820c7, SUC_lexicon_tags_69 },
    { 0x472820c8, SUC_lexicon_tags_2 },
    { 0x97c820c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc820cd, SUC_lexicon_tags_1 },
    { 0x988820ce, SUC_lexicon_tags_46 },
    { 0x602820ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00820d2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x282820d4, SUC_lexicon_tags_15 },
    { 0x162820d5, SUC_lexicon_tags_1 },
    { 0x28a820d4, SUC_lexicon_tags_25 },
    { 0x912820d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2820dc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x196820de, SUC_lexicon_tags_12 },
    { 0xca6820df, SUC_lexicon_tags_8 },
    { 0xda8820e0, SUC_lexicon_tags_17 },
    { 0x8de820e1, SUC_lexicon_tags_17 },
    { 0x1fe820e2, SUC_lexicon_tags_17 },
    { 0xa820e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c820e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e820ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa820f0, SUC_lexicon_tags_25 },
    { 0xa5c820f0, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xb00820f3, SUC_lexicon_tags_19 },
    { 0xdc4820f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636820f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2820fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17082100, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a82103, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5448210a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a82118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed88211a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7082120, SUC_lexicon_tags_95 },
    { 0x4c082121, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15282126, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc82129, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8e8212b, SUC_lexicon_tags_17 },
    { 0xa168212b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5048212f, SUC_lexicon_tags_31 },
    { 0x23282130, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a82132, SUC_lexicon_tags_37 },
    { 0xf4e82133, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb2c82135, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a48213b, SUC_lexicon_tags_2 },
    { 0x93c8213c, SUC_lexicon_tags_1 },
    { 0xbda8213d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75c8213f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89082145, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21282147, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc788214e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b482151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72682153, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9682155, SUC_lexicon_tags_1 },
    { 0xe882156, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea8215b, SUC_lexicon_tags_27 },
    { 0x97e8215c, SUC_lexicon_tags_17 },
    { 0xc028215b, SUC_lexicon_tags_7 },
    { 0x88215e, SUC_lexicon_tags_23 },
    { 0x7c88215c, SUC_lexicon_tags_11 },
    { 0x3228215f, SUC_lexicon_tags_37 },
    { 0xa6282161, SUC_lexicon_tags_47 },
    { 0xfe682161, SUC_lexicon_tags_46 },
    { 0x7ba8215c, SUC_lexicon_tags_12 },
    { 0x86c82160, SUC_lexicon_tags_9 },
    { 0x75682165, SUC_lexicon_tags_48 },
    { 0xe8c82166, SUC_lexicon_tags_31 },
    { 0x17082165, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4682169, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9482175, SUC_lexicon_tags_48 },
    { 0x6fa82176, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea08217f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb9682181, SUC_lexicon_tags_9 },
    { 0xe0482182, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e8218c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a48218f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c82195, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b48219a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd988219c, SUC_lexicon_tags_43 },
    { 0x1108219d, SUC_lexicon_tags_26 },
    { 0xcc8219e, SUC_lexicon_tags_11 },
    { 0x3ea8219f, SUC_lexicon_tags_17 },
    { 0x3528219f, SUC_lexicon_tags_11 },
    { 0x428219f, SUC_lexicon_tags_43 },
    { 0x3d6821a2, SUC_lexicon_tags_26 },
    { 0xb728219d, SUC_lexicon_tags_7 },
    { 0x172821a4, SUC_lexicon_tags_60 },
    { 0xa688219d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66821aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x282821ac, SUC_lexicon_tags_37 },
    { 0xf28821ad, SUC_lexicon_tags_61 },
    { 0x678821ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb78821b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c821b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4821b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a821bb, SUC_lexicon_tags_6 },
    { 0x87e821bc, SUC_lexicon_tags_12 },
    { 0xa12821bc, SUC_lexicon_tags_6 },
    { 0xf78821bc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42821c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da821c8, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e821ce, SUC_lexicon_tags_2 },
    { 0x3e6821cf, SUC_lexicon_tags_33 },
    { 0xff0821d0, SUC_lexicon_tags_136 },
    { 0x800821cf, SUC_lexicon_tags_18 },
    { 0x804821d2, SUC_lexicon_tags_1 },
    { 0xac6821d3, SUC_lexicon_tags_9 },
    { 0x8e2821d2, SUC_lexicon_tags_27 },
    { 0xc4e821ce, SUC_lexicon_tags_7 },
    { 0xd5c821d6, SUC_lexicon_tags_47 },
    { 0x2dc821d7, SUC_lexicon_tags_24 },
    { 0x882821d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e821e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc16821e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a821e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4a821e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6821eb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfc0821ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376821f2, SUC_lexicon_tags_1 },
    { 0xa2821f3, SUC_lexicon_tags_11 },
    { 0x1be821f4, SUC_lexicon_tags_2 },
    { 0xea6821f5, SUC_lexicon_tags_2 },
    { 0xc16821f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x27e821f8, SUC_lexicon_tags_14 },
    { 0x3b0821f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6821ff, SUC_lexicon_tags_17 },
    { 0xfee821ff, SUC_lexicon_tags_38 },
    { 0x7e082201, SUC_lexicon_tags_10 },
    { 0xcfa82201, SUC_lexicon_tags_64 },
    { 0x9ea82201, SUC_lexicon_tags_48 },
    { 0x4f082204, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a82209, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12482211, SUC_lexicon_tags_39 },
    { 0x8ca82211, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3882214, SUC_lexicon_tags_63 },
    { 0x2c682215, SUC_lexicon_tags_63 },
    { 0xe882216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6da82218, SUC_lexicon_tags_7 },
    { 0x81682219, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a82220, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaac82222, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5a682224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca8222e, SUC_lexicon_tags_11 },
    { 0xa4c8222f, SUC_lexicon_tags_37 },
    { 0xffa8222f, SUC_lexicon_tags_65 },
    { 0x12282231, SUC_lexicon_tags_17 },
    { 0xfc08222f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x81482235, SUC_lexicon_tags_38 },
    { 0xddc82236, SUC_lexicon_tags_11 },
    { 0x4b482237, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d482239, SUC_lexicon_tags_7 },
    { 0xf048223a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4768223c, SUC_lexicon_tags_9 },
    { 0x71c8223d, SUC_lexicon_tags_37 },
    { 0x2fe8223e, SUC_lexicon_tags_17 },
    { 0x8788223d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d282241, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x40e82243, SUC_lexicon_tags_65 },
    { 0x9fa82244, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4e82246, SUC_lexicon_tags_23 },
    { 0x4a282246, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b48224a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e82252, SUC_lexicon_tags_15 },
    { 0xb6e82252, SUC_lexicon_tags_50 },
    { 0xc6c82252, SUC_lexicon_tags_2 },
    { 0xd8c82253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef48225b, SUC_lexicon_tags_9 },
    { 0x78e8225c, SUC_lexicon_tags_7 },
    { 0x3908225c, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e82265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde682267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9bc82269, SUC_lexicon_tags_6 },
    { 0x4ec8226a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8088226d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb28226f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d482273, SUC_lexicon_tags_8 },
    { 0xe6e82274, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7fa82276, SUC_lexicon_tags_12 },
    { 0x5aa82277, SUC_lexicon_tags_37 },
    { 0xe5c82278, SUC_lexicon_tags_30 },
    { 0x15c82276, SUC_lexicon_tags_15 },
    { 0x9f682276, SUC_lexicon_tags_37 },
    { 0xde08227b, SUC_lexicon_tags_37 },
    { 0x4888227b, SUC_lexicon_tags_2 },
    { 0xce82276, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x71e8227f, SUC_lexicon_tags_66 },
    { 0xb3682280, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xdbe82282, SUC_lexicon_tags_26 },
    { 0xcd682283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86482285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca8228a, SUC_lexicon_tags_46 },
    { 0x5928228a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x948228d, SUC_lexicon_tags_37 },
    { 0x3c88228e, SUC_lexicon_tags_37 },
    { 0xf68228f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c82296, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692822a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2822a6, SUC_lexicon_tags_56 },
    { 0x878822a6, SUC_lexicon_tags_61 },
    { 0xe1a822a8, SUC_lexicon_tags_27 },
    { 0xe48822a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56822b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69a822b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc822bb, SUC_lexicon_tags_63 },
    { 0xe30822bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27e822be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4822c3, SUC_lexicon_tags_17 },
    { 0xcce822c3, SUC_lexicon_tags_53 },
    { 0x9a6822c5, SUC_lexicon_tags_12 },
    { 0x464822c3, SUC_lexicon_tags_31 },
    { 0x1e8822c3, SUC_lexicon_tags_25 },
    { 0x466822c6, SUC_lexicon_tags_362 },
    { 0xa88822c9, SUC_lexicon_tags_43 },
    { 0x66c822c9, SUC_lexicon_tags_7 },
    { 0x574822c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1e822cd, SUC_lexicon_tags_42 },
    { 0x5a2822ce, SUC_lexicon_tags_63 },
    { 0xcb0822ce, SUC_lexicon_tags_14 },
    { 0x4f8822d0, SUC_lexicon_tags_37 },
    { 0xa92822d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac822d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ae822d7, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x496822da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc822dd, SUC_lexicon_tags_38 },
    { 0x5c8822dd, SUC_lexicon_tags_17 },
    { 0xbe822dd, SUC_lexicon_tags_25 },
    { 0xcbe822de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56822e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224822ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39e822f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e822f8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2822fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcc822fe, SUC_lexicon_tags_11 },
    { 0x51c822ff, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4482305, SUC_lexicon_tags_39 },
    { 0x80482305, SUC_lexicon_tags_37 },
    { 0xf3e82306, SUC_lexicon_tags_1 },
    { 0xb7682308, SUC_lexicon_tags_31 },
    { 0xb0882309, SUC_lexicon_tags_7 },
    { 0xbc882307, SUC_lexicon_tags_32 },
    { 0xf688230b, SUC_lexicon_tags_16 },
    { 0x31e82305, SUC_lexicon_tags_38 },
    { 0x9d08230a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14282312, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe8082314, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee8231c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9482324, SUC_lexicon_tags_47 },
    { 0x15682324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a682327, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xbee82329, SUC_lexicon_tags_47 },
    { 0x6808232a, SUC_lexicon_tags_2 },
    { 0xb3a8232a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6128232e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c482334, SUC_lexicon_tags_14 },
    { 0x6c82334, SUC_lexicon_tags_25 },
    { 0x1b882335, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7482338, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa8233e, SUC_lexicon_tags_30 },
    { 0x4268233f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e282345, SUC_lexicon_tags_57 },
    { 0x15282345, SUC_lexicon_tags_12 },
    { 0x9a082346, SUC_lexicon_tags_27 },
    { 0x85e82346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a82357, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe8235a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa8235d, SUC_lexicon_tags_38 },
    { 0x5e08235e, SUC_lexicon_tags_25 },
    { 0x51a8235f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a82365, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e082368, SUC_lexicon_tags_2 },
    { 0x29e82368, SUC_lexicon_tags_17 },
    { 0xafc8236a, SUC_lexicon_tags_2 },
    { 0x9d48236a, SUC_lexicon_tags_1 },
    { 0x57c8236c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2468236e, SUC_lexicon_tags_38 },
    { 0xd6e8236f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa4482371, SUC_lexicon_tags_15 },
    { 0x21e82372, SUC_lexicon_tags_63 },
    { 0x20082373, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf968237b, SUC_lexicon_tags_69 },
    { 0x5528237c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb68237f, SUC_lexicon_tags_28 },
    { 0x9d48237f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36482386, SUC_lexicon_tags_15 },
    { 0xc2082387, SUC_lexicon_tags_15 },
    { 0x61082387, SUC_lexicon_tags_11 },
    { 0x14a82389, SUC_lexicon_tags_9 },
    { 0x52282387, SUC_lexicon_tags_25 },
    { 0x5d08238b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f08238e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a82396, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5882399, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5ea8239b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c8239f, SUC_lexicon_tags_20 },
    { 0xdcc823a0, SUC_lexicon_tags_17 },
    { 0xb60823a1, SUC_lexicon_tags_12 },
    { 0xb9c8239f, SUC_lexicon_tags_7 },
    { 0x2988239f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a823a5, SUC_lexicon_tags_44 },
    { 0xb32823a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24c823a8, SUC_lexicon_tags_104 },
    { 0x956823a9, SUC_lexicon_tags_11 },
    { 0x1ae823aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6ec823ac, SUC_lexicon_tags_37 },
    { 0x3a6823ad, SUC_lexicon_tags_45 },
    { 0xd08823ad, SUC_lexicon_tags_63 },
    { 0xbee823ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x690823b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98823b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e823bb, SUC_lexicon_tags_17 },
    { 0x52c823bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ac823be, SUC_lexicon_tags_95 },
    { 0xc70823bf, SUC_lexicon_tags_60 },
    { 0x90823bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a823c3, SUC_lexicon_tags_30 },
    { 0xbee823c4, SUC_lexicon_tags_30 },
    { 0xfbe823c3, SUC_lexicon_tags_25 },
    { 0x3e6823c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x53a823c8, SUC_lexicon_tags_11 },
    { 0x634823c8, SUC_lexicon_tags_17 },
    { 0xae0823ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x166823cd, SUC_lexicon_tags_45 },
    { 0xb10823cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6823d2, SUC_lexicon_tags_1307 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650823d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd94823d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80823dc, SUC_lexicon_tags_37 },
    { 0xf76823dd, SUC_lexicon_tags_37 },
    { 0x4b0823de, SUC_lexicon_tags_30 },
    { 0xb8823dc, SUC_lexicon_tags_14 },
    { 0x606823e0, SUC_lexicon_tags_15 },
    { 0x186823df, SUC_lexicon_tags_4 },
    { 0x7fa823e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaf8823e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabe823e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd48823e8, SUC_lexicon_tags_37 },
    { 0xfdc823e9, SUC_lexicon_tags_37 },
    { 0xe22823e9, SUC_lexicon_tags_26 },
    { 0x518823e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcb0823ed, SUC_lexicon_tags_47 },
    { 0xd66823ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434823f5, SUC_lexicon_tags_14 },
    { 0xbc8823f5, SUC_lexicon_tags_12 },
    { 0xd72823f6, SUC_lexicon_tags_25 },
    { 0xf2c823f5, SUC_lexicon_tags_15 },
    { 0x7a0823f9, SUC_lexicon_tags_38 },
    { 0x902823f6, SUC_lexicon_tags_31 },
    { 0x686823fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828823ff, SUC_lexicon_tags_1 },
    { 0xe2482400, SUC_lexicon_tags_11 },
    { 0x71c82401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea282403, SUC_lexicon_tags_63 },
    { 0xf3282404, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde82407, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3908240c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c82410, SUC_lexicon_tags_45 },
    { 0xaa682411, SUC_lexicon_tags_9 },
    { 0x19482412, SUC_lexicon_tags_43 },
    { 0xc082413, SUC_lexicon_tags_7 },
    { 0xc2682412, SUC_lexicon_tags_2 },
    { 0x7d682410, SUC_lexicon_tags_21 },
    { 0x2ba82412, SUC_lexicon_tags_40 },
    { 0x77e82417, SUC_lexicon_tags_62 },
    { 0x35e82411, SUC_lexicon_tags_62 },
    { 0x7c482416, SUC_lexicon_tags_14 },
    { 0xfd082418, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83e8241c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a48241f, SUC_lexicon_tags_14 },
    { 0x4088241f, SUC_lexicon_tags_15 },
    { 0xdb882421, SUC_lexicon_tags_25 },
    { 0x5b482422, SUC_lexicon_tags_39 },
    { 0x8a82421, SUC_lexicon_tags_12 },
    { 0x18e82424, SUC_lexicon_tags_1 },
    { 0x52a82422, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8708242a, SUC_lexicon_tags_31 },
    { 0x13a8242b, SUC_lexicon_tags_17 },
    { 0xbaa8242a, SUC_lexicon_tags_1 },
    { 0xe728242b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb482430, SUC_lexicon_tags_5 },
    { 0xc6c82431, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38a82433, SUC_lexicon_tags_79 },
    { 0x62482433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcaa82436, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f482439, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe368243e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffe82440, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e082443, SUC_lexicon_tags_25 },
    { 0x21c82444, SUC_lexicon_tags_2 },
    { 0xde482445, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd88244c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a682451, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c82457, SUC_lexicon_tags_15 },
    { 0x21882458, SUC_lexicon_tags_63 },
    { 0xa2c82459, SUC_lexicon_tags_11 },
    { 0x3388245a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c8245c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c8245f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5682461, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd3482463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8368246e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21682474, SUC_lexicon_tags_17 },
    { 0xb1a82475, SUC_lexicon_tags_17 },
    { 0x61a82474, SUC_lexicon_tags_60 },
    { 0x3de82477, SUC_lexicon_tags_27 },
    { 0x78882478, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc8247c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e8247f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd482484, SUC_lexicon_tags_37 },
    { 0x19e82485, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2c682487, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x6a482489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e8248d, SUC_lexicon_tags_30 },
    { 0x4b88248d, SUC_lexicon_tags_40 },
    { 0x82e8248f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xefa82491, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x21282494, SUC_lexicon_tags_28 },
    { 0x34682495, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac82499, SUC_lexicon_tags_7 },
    { 0xb4a82499, SUC_lexicon_tags_17 },
    { 0x28e8249b, SUC_lexicon_tags_33 },
    { 0x4d28249c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8824a6, SUC_lexicon_tags_26 },
    { 0x412824a7, SUC_lexicon_tags_8 },
    { 0xe1a824a8, SUC_lexicon_tags_17 },
    { 0xcd6824a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8824ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ae824ad, SUC_lexicon_tags_19 },
    { 0x4a0824ae, SUC_lexicon_tags_47 },
    { 0x8ac824af, SUC_lexicon_tags_15 },
    { 0xdc4824af, SUC_lexicon_tags_17 },
    { 0xa44824b1, SUC_lexicon_tags_11 },
    { 0x268824b2, SUC_lexicon_tags_14 },
    { 0x4ce824b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8c824b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e6824b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868824bb, SUC_lexicon_tags_11 },
    { 0xa2c824bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe824bf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c824c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13c824c9, SUC_lexicon_tags_25 },
    { 0x9d0824ca, SUC_lexicon_tags_11 },
    { 0x286824cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c824ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e824d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf78824d6, SUC_lexicon_tags_1 },
    { 0xbae824d7, SUC_lexicon_tags_2 },
    { 0x42a824d8, SUC_lexicon_tags_11 },
    { 0x32e824d6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e824dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaae824df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ec824e1, SUC_lexicon_tags_27 },
    { 0x592824e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee824e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x860824eb, SUC_lexicon_tags_82 },
    { 0xfa4824eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce824f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca6824f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100824fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb482502, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe082504, SUC_lexicon_tags_6 },
    { 0x6d482504, SUC_lexicon_tags_14 },
    { 0x97e82506, SUC_lexicon_tags_5 },
    { 0xd8a82507, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e082509, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0xb608250c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67e8250e, SUC_lexicon_tags_1 },
    { 0xdc08250f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x16282511, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4d282513, SUC_lexicon_tags_12 },
    { 0x5f082513, SUC_lexicon_tags_12 },
    { 0xb3082515, SUC_lexicon_tags_25 },
    { 0xe4e82514, SUC_lexicon_tags_37 },
    { 0xfce82517, SUC_lexicon_tags_17 },
    { 0x22c82518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28251b, SUC_lexicon_tags_28 },
    { 0x2548251c, SUC_lexicon_tags_87 },
    { 0xc7e8251d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7a8251f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c82527, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x49682529, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc968252b, SUC_lexicon_tags_37 },
    { 0x1728252b, SUC_lexicon_tags_33 },
    { 0xa348252d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec682530, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e82538, SUC_lexicon_tags_97 },
    { 0x1ce82539, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a8253f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f882544, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57882549, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e8254c, SUC_lexicon_tags_1 },
    { 0x7ca8254d, SUC_lexicon_tags_37 },
    { 0x56a8254c, SUC_lexicon_tags_5 },
    { 0x8a88254f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1682552, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc82555, SUC_lexicon_tags_47 },
    { 0xa1a82556, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc82561, SUC_lexicon_tags_10 },
    { 0xbe482561, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26482565, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd2882567, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7968256c, SUC_lexicon_tags_48 },
    { 0x6c48256d, SUC_lexicon_tags_12 },
    { 0xd448256d, SUC_lexicon_tags_6 },
    { 0xad08256e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0682572, SUC_lexicon_tags_25 },
    { 0x47282572, SUC_lexicon_tags_0 },
    { 0xd7c82574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x44682577, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc8257a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x6908257c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6568257e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0882582, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd8482584, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c482587, SUC_lexicon_tags_2 },
    { 0x5ea82587, SUC_lexicon_tags_23 },
    { 0x53c82589, SUC_lexicon_tags_9 },
    { 0x75082589, SUC_lexicon_tags_0 },
    { 0xd988258b, SUC_lexicon_tags_43 },
    { 0x5c8258c, SUC_lexicon_tags_23 },
    { 0x7268258a, SUC_lexicon_tags_37 },
    { 0x6708258e, SUC_lexicon_tags_25 },
    { 0xb4e8258e, SUC_lexicon_tags_25 },
    { 0x2aa82590, SUC_lexicon_tags_14 },
    { 0xd2882590, SUC_lexicon_tags_26 },
    { 0x90082592, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d682594, SUC_lexicon_tags_87 },
    { 0x7a082595, SUC_lexicon_tags_35 },
    { 0xa8c82596, SUC_lexicon_tags_13 },
    { 0x5a882597, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x82882599, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5848259d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6825a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x634825aa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9e6825ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d4825ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c825b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa6825b3, SUC_lexicon_tags_56 },
    { 0xc0a825b4, SUC_lexicon_tags_0 },
    { 0x6b2825b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9c825b7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6825ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb4825bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0a825be, SUC_lexicon_tags_77 },
    { 0xdfc825be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb8825c1, SUC_lexicon_tags_95 },
    { 0xe04825c1, SUC_lexicon_tags_65 },
    { 0xb7a825c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948825c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e825cc, SUC_lexicon_tags_28 },
    { 0x2dc825cd, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476825d6, SUC_lexicon_tags_31 },
    { 0x2e6825d7, SUC_lexicon_tags_43 },
    { 0x1fa825d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da825db, SUC_lexicon_tags_57 },
    { 0xbe8825dc, SUC_lexicon_tags_42 },
    { 0x5a825dd, SUC_lexicon_tags_15 },
    { 0x810825db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66825e1, SUC_lexicon_tags_9 },
    { 0x848825e1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x972825e4, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x2b4825e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892825ea, SUC_lexicon_tags_25 },
    { 0x984825ea, SUC_lexicon_tags_17 },
    { 0xa82825ea, SUC_lexicon_tags_12 },
    { 0xf84825ed, SUC_lexicon_tags_17 },
    { 0xe18825ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbec825f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8825f4, SUC_lexicon_tags_14 },
    { 0x182825f5, SUC_lexicon_tags_12 },
    { 0x6c6825f6, SUC_lexicon_tags_37 },
    { 0x758825f7, SUC_lexicon_tags_2 },
    { 0xffc825f8, SUC_lexicon_tags_11 },
    { 0xc42825f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb88825fb, SUC_lexicon_tags_12 },
    { 0x160825fc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa82602, SUC_lexicon_tags_7 },
    { 0xfdc82603, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe368260c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c82610, SUC_lexicon_tags_45 },
    { 0x88a82610, SUC_lexicon_tags_1 },
    { 0x6d282612, SUC_lexicon_tags_37 },
    { 0x75482613, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc82617, SUC_lexicon_tags_45 },
    { 0x97882617, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f08261b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5048261d, SUC_lexicon_tags_15 },
    { 0xee28261d, SUC_lexicon_tags_31 },
    { 0x9f88261f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14e82621, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x54482624, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0482629, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6728262b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae28262d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5228262f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70682632, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a82636, SUC_lexicon_tags_31 },
    { 0x81282637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a682639, SUC_lexicon_tags_34 },
    { 0x28482639, SUC_lexicon_tags_11 },
    { 0x2628263b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a8263e, SUC_lexicon_tags_30 },
    { 0x7448263f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56282646, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x34882649, SUC_lexicon_tags_15 },
    { 0xcb88264a, SUC_lexicon_tags_25 },
    { 0xca48264a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac8264f, SUC_lexicon_tags_11 },
    { 0x6228264f, SUC_lexicon_tags_31 },
    { 0x9482651, SUC_lexicon_tags_1 },
    { 0x11482651, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1a82654, SUC_lexicon_tags_41 },
    { 0x8fc82655, SUC_lexicon_tags_7 },
    { 0xdde82655, SUC_lexicon_tags_2 },
    { 0xf9882657, SUC_lexicon_tags_1 },
    { 0xac282655, SUC_lexicon_tags_25 },
    { 0xa5282655, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e8265d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c82666, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e82669, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca8266c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe388266f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9482671, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5aa82673, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e282677, SUC_lexicon_tags_60 },
    { 0x5b482678, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3788267b, SUC_lexicon_tags_2 },
    { 0x49e8267c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84282680, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56482682, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3de82684, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc882687, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be8268a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e68268c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e82691, SUC_lexicon_tags_21 },
    { 0xc8082692, SUC_lexicon_tags_9 },
    { 0xf2482693, SUC_lexicon_tags_8 },
    { 0x1ca82694, SUC_lexicon_tags_31 },
    { 0x53a82695, SUC_lexicon_tags_7 },
    { 0x2f082695, SUC_lexicon_tags_25 },
    { 0x28a82697, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a8269d, SUC_lexicon_tags_2 },
    { 0x34c8269e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x524826a1, SUC_lexicon_tags_7 },
    { 0x8b4826a2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6826a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x784826a8, SUC_lexicon_tags_7 },
    { 0x76e826a9, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x138826ab, SUC_lexicon_tags_434 },
    { 0, NULL },
    { 0xc92826ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x614826af, SUC_lexicon_tags_17 },
    { 0x930826b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x932826b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0826b5, SUC_lexicon_tags_19 },
    { 0x4ce826b5, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786826bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4826c1, SUC_lexicon_tags_14 },
    { 0x99c826c2, SUC_lexicon_tags_39 },
    { 0x6da826c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310826c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8e6826ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90826cf, SUC_lexicon_tags_23 },
    { 0x7de826d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabe826d2, SUC_lexicon_tags_38 },
    { 0x2ea826d2, SUC_lexicon_tags_37 },
    { 0x81a826d4, SUC_lexicon_tags_45 },
    { 0x64826d4, SUC_lexicon_tags_2 },
    { 0x5d8826d4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc826db, SUC_lexicon_tags_19 },
    { 0x722826dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16826e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2826e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6826e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x166826ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174826ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6826f4, SUC_lexicon_tags_31 },
    { 0x392826f5, SUC_lexicon_tags_17 },
    { 0x54c826f4, SUC_lexicon_tags_6 },
    { 0xc22826f6, SUC_lexicon_tags_56 },
    { 0x68e826f7, SUC_lexicon_tags_38 },
    { 0xa72826f9, SUC_lexicon_tags_1 },
    { 0xe66826fa, SUC_lexicon_tags_10 },
    { 0x8c8826f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a82702, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0482707, SUC_lexicon_tags_23 },
    { 0x6a882708, SUC_lexicon_tags_2 },
    { 0x83082708, SUC_lexicon_tags_37 },
    { 0xd8e82708, SUC_lexicon_tags_3 },
    { 0x84a82708, SUC_lexicon_tags_7 },
    { 0x30a8270c, SUC_lexicon_tags_7 },
    { 0xd288270c, SUC_lexicon_tags_11 },
    { 0xf6e82709, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e482718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4408271c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce48271e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc082722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e082724, SUC_lexicon_tags_7 },
    { 0x62882725, SUC_lexicon_tags_49 },
    { 0x3282726, SUC_lexicon_tags_15 },
    { 0x38e82725, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4048272c, SUC_lexicon_tags_488 },
    { 0xd148272d, SUC_lexicon_tags_19 },
    { 0x56c8272e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xca682730, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c82733, SUC_lexicon_tags_14 },
    { 0xcfa82734, SUC_lexicon_tags_38 },
    { 0xf8082734, SUC_lexicon_tags_27 },
    { 0xbbc82734, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc82740, SUC_lexicon_tags_32 },
    { 0x6fa82740, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc82744, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd7682746, SUC_lexicon_tags_37 },
    { 0x2c282747, SUC_lexicon_tags_30 },
    { 0x67e82747, SUC_lexicon_tags_35 },
    { 0x23482746, SUC_lexicon_tags_49 },
    { 0x9f08274a, SUC_lexicon_tags_11 },
    { 0x50c8274b, SUC_lexicon_tags_27 },
    { 0xd9082747, SUC_lexicon_tags_15 },
    { 0xb4482749, SUC_lexicon_tags_14 },
    { 0x6768274e, SUC_lexicon_tags_39 },
    { 0xb988274e, SUC_lexicon_tags_1 },
    { 0xa868274f, SUC_lexicon_tags_48 },
    { 0xe7882751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec882753, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6448275a, SUC_lexicon_tags_7 },
    { 0x49c8275b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d082760, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62082766, SUC_lexicon_tags_1 },
    { 0x56082767, SUC_lexicon_tags_25 },
    { 0x2c82768, SUC_lexicon_tags_38 },
    { 0xcf082769, SUC_lexicon_tags_38 },
    { 0x4388276a, SUC_lexicon_tags_27 },
    { 0x1508276b, SUC_lexicon_tags_85 },
    { 0xa768276b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf48276e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74082770, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8a082772, SUC_lexicon_tags_17 },
    { 0xd8e82772, SUC_lexicon_tags_28 },
    { 0xb9882774, SUC_lexicon_tags_2 },
    { 0x50682774, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf5082777, SUC_lexicon_tags_38 },
    { 0x1a882778, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6508277e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x80082780, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4682787, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf568278d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xffa8278f, SUC_lexicon_tags_65 },
    { 0xace82790, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76682795, SUC_lexicon_tags_11 },
    { 0xf6a82796, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74082798, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c827a1, SUC_lexicon_tags_2 },
    { 0x8e6827a2, SUC_lexicon_tags_62 },
    { 0x94a827a3, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338827a7, SUC_lexicon_tags_12 },
    { 0xcb2827a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a827ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc76827ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x764827af, SUC_lexicon_tags_17 },
    { 0xb4e827af, SUC_lexicon_tags_7 },
    { 0xc78827b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x320827b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20827b7, SUC_lexicon_tags_25 },
    { 0x744827b8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c827bd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xac6827bf, SUC_lexicon_tags_46 },
    { 0x19a827c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdae827c2, SUC_lexicon_tags_7 },
    { 0xb44827c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd94827c5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x216827c8, SUC_lexicon_tags_25 },
    { 0x70c827c9, SUC_lexicon_tags_17 },
    { 0xb2827c8, SUC_lexicon_tags_25 },
    { 0xc44827ca, SUC_lexicon_tags_5 },
    { 0xd7c827c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7a827ce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c827d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818827d6, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x430827d9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde827dd, SUC_lexicon_tags_33 },
    { 0xf8e827de, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e827e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9a6827e3, SUC_lexicon_tags_17 },
    { 0x33c827e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0827e9, SUC_lexicon_tags_2 },
    { 0x816827e9, SUC_lexicon_tags_48 },
    { 0xa66827eb, SUC_lexicon_tags_37 },
    { 0xc6a827ec, SUC_lexicon_tags_47 },
    { 0x87c827ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa4a827ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c827fb, SUC_lexicon_tags_63 },
    { 0xb46827fb, SUC_lexicon_tags_27 },
    { 0x1f6827fd, SUC_lexicon_tags_49 },
    { 0xab8827fe, SUC_lexicon_tags_37 },
    { 0x352827fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12282803, SUC_lexicon_tags_49 },
    { 0x3d082804, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de8280d, SUC_lexicon_tags_30 },
    { 0xfcc8280e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e082810, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b082813, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca82818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d68281a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb48281c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7608281f, SUC_lexicon_tags_46 },
    { 0x97682820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78282822, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e82826, SUC_lexicon_tags_7 },
    { 0x2a082827, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e8282f, SUC_lexicon_tags_65 },
    { 0xf968282f, SUC_lexicon_tags_38 },
    { 0x2ba82831, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd082836, SUC_lexicon_tags_7 },
    { 0x75c82836, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a48283a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d68283f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc682841, SUC_lexicon_tags_63 },
    { 0xeec82842, SUC_lexicon_tags_31 },
    { 0xe5a82843, SUC_lexicon_tags_27 },
    { 0x58c82844, SUC_lexicon_tags_4 },
    { 0x16e82844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a8284b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c8284e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea82857, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed88285d, SUC_lexicon_tags_16 },
    { 0xe928285e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c82861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd682863, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e82866, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36082868, SUC_lexicon_tags_33 },
    { 0x31082869, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a482876, SUC_lexicon_tags_39 },
    { 0xb2682876, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8a82879, SUC_lexicon_tags_8 },
    { 0x8e882879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c8287e, SUC_lexicon_tags_9 },
    { 0xb0e8287e, SUC_lexicon_tags_39 },
    { 0xfaa82880, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1482885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe528288b, SUC_lexicon_tags_68 },
    { 0xc528288c, SUC_lexicon_tags_9 },
    { 0xbee8288c, SUC_lexicon_tags_0 },
    { 0xe648288b, SUC_lexicon_tags_1 },
    { 0x4188288f, SUC_lexicon_tags_37 },
    { 0x1248288b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86682896, SUC_lexicon_tags_29 },
    { 0xc1082896, SUC_lexicon_tags_1 },
    { 0x45682898, SUC_lexicon_tags_9 },
    { 0xd0c82899, SUC_lexicon_tags_7 },
    { 0xea282896, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e828a1, SUC_lexicon_tags_29 },
    { 0xda6828a2, SUC_lexicon_tags_1 },
    { 0x210828a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa68828a5, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62828b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bc828b5, SUC_lexicon_tags_37 },
    { 0xcd8828b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14828b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a828bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa60828c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98a828c2, SUC_lexicon_tags_1 },
    { 0xa6828c2, SUC_lexicon_tags_27 },
    { 0x79e828c4, SUC_lexicon_tags_3 },
    { 0xf0c828c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc828c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118828cc, SUC_lexicon_tags_47 },
    { 0x522828cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a828d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c828d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2828d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x624828d8, SUC_lexicon_tags_7 },
    { 0xd72828d8, SUC_lexicon_tags_17 },
    { 0x92a828da, SUC_lexicon_tags_11 },
    { 0x814828db, SUC_lexicon_tags_29 },
    { 0x686828dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd58828de, SUC_lexicon_tags_27 },
    { 0x892828df, SUC_lexicon_tags_28 },
    { 0x132828e0, SUC_lexicon_tags_3 },
    { 0x876828e0, SUC_lexicon_tags_28 },
    { 0xfb2828e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x484828e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4828eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d4828ed, SUC_lexicon_tags_2 },
    { 0xac4828ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb1a828f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x252828f2, SUC_lexicon_tags_8 },
    { 0x72e828f3, SUC_lexicon_tags_2 },
    { 0xb3c828f2, SUC_lexicon_tags_27 },
    { 0x9fe828f5, SUC_lexicon_tags_25 },
    { 0xed8828f6, SUC_lexicon_tags_23 },
    { 0xcd0828f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x35a828f9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2828fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc682900, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8682903, SUC_lexicon_tags_8 },
    { 0xc4e82904, SUC_lexicon_tags_11 },
    { 0xc0282904, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6882907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a8290e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0282910, SUC_lexicon_tags_11 },
    { 0x14082911, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe282914, SUC_lexicon_tags_1 },
    { 0x5f282915, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f68291a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b48291c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xcd88291e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e82924, SUC_lexicon_tags_45 },
    { 0x25882925, SUC_lexicon_tags_47 },
    { 0x2ec82925, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf082929, SUC_lexicon_tags_60 },
    { 0x9ee8292a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x988292c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da82936, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde8293a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e8293f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3882945, SUC_lexicon_tags_37 },
    { 0x1be82945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa988294c, SUC_lexicon_tags_6 },
    { 0xd7c8294c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c82950, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae82953, SUC_lexicon_tags_17 },
    { 0xb8282954, SUC_lexicon_tags_9 },
    { 0xf3282955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3482959, SUC_lexicon_tags_103 },
    { 0x4ea8295a, SUC_lexicon_tags_25 },
    { 0xae08295b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a8295e, SUC_lexicon_tags_1 },
    { 0xf3c8295f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3482962, SUC_lexicon_tags_12 },
    { 0xe9882963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf282969, SUC_lexicon_tags_14 },
    { 0xbd682969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9748296c, SUC_lexicon_tags_15 },
    { 0x1d68296c, SUC_lexicon_tags_1 },
    { 0x8fc8296c, SUC_lexicon_tags_33 },
    { 0x4c88296e, SUC_lexicon_tags_11 },
    { 0x35682970, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc82973, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c82976, SUC_lexicon_tags_30 },
    { 0x33e82976, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf748297d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x4068297f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e82982, SUC_lexicon_tags_27 },
    { 0x48e82983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5082988, SUC_lexicon_tags_27 },
    { 0x9d282989, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x6788298b, SUC_lexicon_tags_11 },
    { 0xca08298c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75282991, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8682993, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64882996, SUC_lexicon_tags_14 },
    { 0x70882996, SUC_lexicon_tags_1 },
    { 0x7e682998, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x728299a, SUC_lexicon_tags_25 },
    { 0xda08299b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5028299e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e829a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x192829a6, SUC_lexicon_tags_25 },
    { 0x802829a6, SUC_lexicon_tags_33 },
    { 0xa40829a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe74829aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16829ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04829b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e829b5, SUC_lexicon_tags_2 },
    { 0xd78829b6, SUC_lexicon_tags_23 },
    { 0x860829b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c829ba, SUC_lexicon_tags_262 },
    { 0x69e829bb, SUC_lexicon_tags_108 },
    { 0x792829bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a829bf, SUC_lexicon_tags_27 },
    { 0xa20829bf, SUC_lexicon_tags_17 },
    { 0x6dc829c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc829c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a829cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2c6829cf, SUC_lexicon_tags_12 },
    { 0xbb8829d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc829d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca829d8, SUC_lexicon_tags_37 },
    { 0x550829d9, SUC_lexicon_tags_28 },
    { 0x1f2829da, SUC_lexicon_tags_12 },
    { 0xf86829da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x722829dd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e829e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba829e7, SUC_lexicon_tags_12 },
    { 0xfd8829e7, SUC_lexicon_tags_7 },
    { 0x38c829e9, SUC_lexicon_tags_16 },
    { 0xc98829e8, SUC_lexicon_tags_37 },
    { 0x278829eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a829ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6829f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e6829f2, SUC_lexicon_tags_15 },
    { 0xeb0829f3, SUC_lexicon_tags_14 },
    { 0xd8a829f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe829f6, SUC_lexicon_tags_43 },
    { 0xeba829f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64829fe, SUC_lexicon_tags_21 },
    { 0x210829ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a82a05, SUC_lexicon_tags_2 },
    { 0x73482a06, SUC_lexicon_tags_31 },
    { 0xfe682a06, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c82a0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e82a0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4882a14, SUC_lexicon_tags_25 },
    { 0x5fe82a15, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x99082a17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9e82a19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31482a1c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d482a1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ee82a21, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7682a24, SUC_lexicon_tags_47 },
    { 0x48e82a25, SUC_lexicon_tags_49 },
    { 0x89482a26, SUC_lexicon_tags_22 },
    { 0xc7482a27, SUC_lexicon_tags_17 },
    { 0xbf482a27, SUC_lexicon_tags_28 },
    { 0xfa882a29, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50082a2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47482a39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x70082a3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a82a3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe882a41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8882a44, SUC_lexicon_tags_1 },
    { 0xa9482a45, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x19082a47, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x55682a4a, SUC_lexicon_tags_19 },
    { 0x8882a4b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea82a56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd082a58, SUC_lexicon_tags_11 },
    { 0xbaa82a59, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1082a5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d482a5e, SUC_lexicon_tags_12 },
    { 0x8ee82a5f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbd082a61, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c82a65, SUC_lexicon_tags_11 },
    { 0x7e482a66, SUC_lexicon_tags_30 },
    { 0x6ee82a66, SUC_lexicon_tags_21 },
    { 0x58a82a68, SUC_lexicon_tags_17 },
    { 0xb5682a69, SUC_lexicon_tags_1 },
    { 0x4f482a6a, SUC_lexicon_tags_63 },
    { 0xdbe82a69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52482a71, SUC_lexicon_tags_0 },
    { 0x8dc82a71, SUC_lexicon_tags_37 },
    { 0x9f482a73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67882a75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44482a77, SUC_lexicon_tags_6 },
    { 0x1a482a77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18482a7b, SUC_lexicon_tags_27 },
    { 0x4b282a7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a682a80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae82a8b, SUC_lexicon_tags_27 },
    { 0xd9282a8c, SUC_lexicon_tags_27 },
    { 0xaf482a8d, SUC_lexicon_tags_1 },
    { 0xec882a8c, SUC_lexicon_tags_46 },
    { 0x4cc82a8f, SUC_lexicon_tags_2 },
    { 0xc9882a8b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc3682a92, SUC_lexicon_tags_47 },
    { 0x68e82a93, SUC_lexicon_tags_1 },
    { 0x3ea82a94, SUC_lexicon_tags_17 },
    { 0xd4e82a92, SUC_lexicon_tags_11 },
    { 0xf1a82a96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc82a99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86a82a9b, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e82aa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1882aa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1e82aa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21082aac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc82ab3, SUC_lexicon_tags_46 },
    { 0x9282ab4, SUC_lexicon_tags_2 },
    { 0x84082ab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33a82ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40282abd, SUC_lexicon_tags_226 },
    { 0x7482abe, SUC_lexicon_tags_37 },
    { 0xbba82abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6e82ac1, SUC_lexicon_tags_37 },
    { 0xc9c82ac1, SUC_lexicon_tags_37 },
    { 0xaa882ac3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea82acb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29882ad3, SUC_lexicon_tags_25 },
    { 0xc8a82ad4, SUC_lexicon_tags_198 },
    { 0x47082ad5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9882ad9, SUC_lexicon_tags_11 },
    { 0x78e82ad9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c82ade, SUC_lexicon_tags_7 },
    { 0x17c82adf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x10682ae2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e82ae5, SUC_lexicon_tags_37 },
    { 0x23682ae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e82aef, SUC_lexicon_tags_68 },
    { 0xc0c82af0, SUC_lexicon_tags_37 },
    { 0x26682af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64a82af3, SUC_lexicon_tags_36 },
    { 0xd7c82af3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95482af7, SUC_lexicon_tags_65 },
    { 0xf7082af8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c82afb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd682b00, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5be82b02, SUC_lexicon_tags_33 },
    { 0x60c82b03, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7482b07, SUC_lexicon_tags_47 },
    { 0x9b482b08, SUC_lexicon_tags_66 },
    { 0x18c82b09, SUC_lexicon_tags_117 },
    { 0x8e482b07, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9082b0e, SUC_lexicon_tags_17 },
    { 0x1b482b0f, SUC_lexicon_tags_2 },
    { 0x43c82b10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0682b12, SUC_lexicon_tags_14 },
    { 0xcbc82b13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x73a82b15, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x98e82b17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a82b1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11882b21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c82b27, SUC_lexicon_tags_25 },
    { 0xce482b28, SUC_lexicon_tags_34 },
    { 0xbe482b29, SUC_lexicon_tags_65 },
    { 0x63282b2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca82b2f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdfa82b31, SUC_lexicon_tags_3 },
    { 0xecc82b31, SUC_lexicon_tags_15 },
    { 0x7a882b33, SUC_lexicon_tags_61 },
    { 0xcea82b34, SUC_lexicon_tags_28 },
    { 0x50282b35, SUC_lexicon_tags_48 },
    { 0x3f082b36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x45882b38, SUC_lexicon_tags_25 },
    { 0x16882b39, SUC_lexicon_tags_41 },
    { 0x58482b3a, SUC_lexicon_tags_2 },
    { 0xf4c82b3a, SUC_lexicon_tags_25 },
    { 0x1bc82b3c, SUC_lexicon_tags_7 },
    { 0xbb682b3d, SUC_lexicon_tags_1 },
    { 0xd082b3d, SUC_lexicon_tags_30 },
    { 0xfc082b3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8082b42, SUC_lexicon_tags_29 },
    { 0xa482b42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec82b4d, SUC_lexicon_tags_14 },
    { 0xb482b4e, SUC_lexicon_tags_37 },
    { 0x2c082b4d, SUC_lexicon_tags_29 },
    { 0x6e682b4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e82b55, SUC_lexicon_tags_100 },
    { 0x94482b55, SUC_lexicon_tags_35 },
    { 0x8c82b57, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7ee82b59, SUC_lexicon_tags_1 },
    { 0x88e82b59, SUC_lexicon_tags_25 },
    { 0xd0282b59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3da82b5d, SUC_lexicon_tags_66 },
    { 0x99682b5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x94082b60, SUC_lexicon_tags_19 },
    { 0x59882b60, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde82b66, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaa682b68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a82b6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c82b6e, SUC_lexicon_tags_7 },
    { 0xaee82b6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3882b71, SUC_lexicon_tags_6 },
    { 0xc4482b72, SUC_lexicon_tags_118 },
    { 0xd1682b73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58882b79, SUC_lexicon_tags_17 },
    { 0x9be82b79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39882b7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a82b82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c82b86, SUC_lexicon_tags_27 },
    { 0x38682b87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4de82b89, SUC_lexicon_tags_69 },
    { 0x94282b8a, SUC_lexicon_tags_64 },
    { 0x5c482b8a, SUC_lexicon_tags_36 },
    { 0x40882b8a, SUC_lexicon_tags_60 },
    { 0xba682b8d, SUC_lexicon_tags_17 },
    { 0x7a682b8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6482b90, SUC_lexicon_tags_12 },
    { 0x19a82b91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x35682b93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a82b97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8282b9c, SUC_lexicon_tags_37 },
    { 0x46082b9c, SUC_lexicon_tags_127 },
    { 0x35c82b9e, SUC_lexicon_tags_82 },
    { 0x35082b9f, SUC_lexicon_tags_1 },
    { 0xb4482ba0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a082ba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9282ba8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8682bab, SUC_lexicon_tags_10 },
    { 0x45882bab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f082bae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd282bb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da82bbb, SUC_lexicon_tags_1 },
    { 0xa7682bbc, SUC_lexicon_tags_17 },
    { 0x1e082bbd, SUC_lexicon_tags_48 },
    { 0x55082bbe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb882bc1, SUC_lexicon_tags_60 },
    { 0xfba82bc2, SUC_lexicon_tags_1 },
    { 0x7f282bc1, SUC_lexicon_tags_25 },
    { 0xd1e82bc4, SUC_lexicon_tags_15 },
    { 0x22282bc5, SUC_lexicon_tags_2 },
    { 0xe8082bc1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa82bcd, SUC_lexicon_tags_49 },
    { 0x61282bce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a282bd1, SUC_lexicon_tags_4 },
    { 0x2c682bd2, SUC_lexicon_tags_1 },
    { 0x48882bd3, SUC_lexicon_tags_61 },
    { 0x19a82bd2, SUC_lexicon_tags_12 },
    { 0xa9282bd3, SUC_lexicon_tags_1 },
    { 0x7d082bd5, SUC_lexicon_tags_2 },
    { 0xdd882bd6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73082bd9, SUC_lexicon_tags_39 },
    { 0x34282bd9, SUC_lexicon_tags_19 },
    { 0xdd082bdb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x37682bdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac282bdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97e82be1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e882be6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44682bea, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd682bf2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35082bf4, SUC_lexicon_tags_25 },
    { 0x4b682bf5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77682bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe682bfa, SUC_lexicon_tags_15 },
    { 0x32a82bfb, SUC_lexicon_tags_46 },
    { 0x3c482bfc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea482c02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c82c07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58a82c09, SUC_lexicon_tags_12 },
    { 0x87082c0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80682c0d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e82c10, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c82c18, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xca882c1b, SUC_lexicon_tags_265 },
    { 0x6e82c1c, SUC_lexicon_tags_17 },
    { 0xae082c1c, SUC_lexicon_tags_11 },
    { 0xd3c82c1e, SUC_lexicon_tags_12 },
    { 0x44282c1f, SUC_lexicon_tags_0 },
    { 0x62282c1c, SUC_lexicon_tags_21 },
    { 0x48e82c21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0082c24, SUC_lexicon_tags_0 },
    { 0x47482c25, SUC_lexicon_tags_2 },
    { 0xf1082c24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3d682c28, SUC_lexicon_tags_30 },
    { 0xbe282c29, SUC_lexicon_tags_0 },
    { 0xa5282c2a, SUC_lexicon_tags_17 },
    { 0x7f082c28, SUC_lexicon_tags_14 },
    { 0xdf082c2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa682c31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8682c34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91682c38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6ec82c3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d082c3d, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x85882c3f, SUC_lexicon_tags_26 },
    { 0xa282c40, SUC_lexicon_tags_15 },
    { 0x5b682c41, SUC_lexicon_tags_33 },
    { 0xf7e82c42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45282c45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x70082c48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x32282c4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb682c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2882c4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c82c51, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xae482c53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43482c55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee082c59, SUC_lexicon_tags_26 },
    { 0xb0082c5a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x10e82c5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42282c60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c282c64, SUC_lexicon_tags_25 },
    { 0x86482c64, SUC_lexicon_tags_2 },
    { 0x3682c65, SUC_lexicon_tags_38 },
    { 0x8e482c67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda82c69, SUC_lexicon_tags_25 },
    { 0xdaa82c69, SUC_lexicon_tags_37 },
    { 0x15082c69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab482c6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2682c71, SUC_lexicon_tags_6 },
    { 0xe482c72, SUC_lexicon_tags_55 },
    { 0xa1682c73, SUC_lexicon_tags_24 },
    { 0x3b682c73, SUC_lexicon_tags_12 },
    { 0xf3c82c71, SUC_lexicon_tags_49 },
    { 0x2fc82c76, SUC_lexicon_tags_53 },
    { 0x8f882c76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6282c7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc82c7e, SUC_lexicon_tags_33 },
    { 0x5f682c7e, SUC_lexicon_tags_13 },
    { 0xb7e82c7f, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e882c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54a82c86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba82c8b, SUC_lexicon_tags_31 },
    { 0xdee82c8b, SUC_lexicon_tags_2 },
    { 0xbf282c8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82282c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9482c91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f082c94, SUC_lexicon_tags_9 },
    { 0xb4482c94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38882c97, SUC_lexicon_tags_26 },
    { 0xfd282c98, SUC_lexicon_tags_9 },
    { 0x2ca82c98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16082c9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5c682c9e, SUC_lexicon_tags_14 },
    { 0x3de82c9f, SUC_lexicon_tags_25 },
    { 0x79682c9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe4882ca2, SUC_lexicon_tags_59 },
    { 0x81682ca3, SUC_lexicon_tags_47 },
    { 0xdaa82ca4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x23c82ca6, SUC_lexicon_tags_31 },
    { 0x41882ca7, SUC_lexicon_tags_37 },
    { 0x7882ca7, SUC_lexicon_tags_31 },
    { 0x50e82ca8, SUC_lexicon_tags_25 },
    { 0x86682ca9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba82cad, SUC_lexicon_tags_11 },
    { 0x61682cae, SUC_lexicon_tags_56 },
    { 0x34c82cad, SUC_lexicon_tags_1 },
    { 0xdb082caf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13482cb5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15682cbc, SUC_lexicon_tags_45 },
    { 0xb4c82cbd, SUC_lexicon_tags_1 },
    { 0xf4a82cbd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x63882cc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c882cc6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x47082cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de82ccc, SUC_lexicon_tags_32 },
    { 0x66682ccd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x79282ccf, SUC_lexicon_tags_1 },
    { 0x6de82cd0, SUC_lexicon_tags_17 },
    { 0xa9e82cd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a82cd9, SUC_lexicon_tags_60 },
    { 0x40082cda, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc82cdf, SUC_lexicon_tags_37 },
    { 0xb482ce0, SUC_lexicon_tags_7 },
    { 0xa3e82ce1, SUC_lexicon_tags_33 },
    { 0x6ee82ce1, SUC_lexicon_tags_47 },
    { 0xe8482ce3, SUC_lexicon_tags_17 },
    { 0x77482ce4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x43882ce6, SUC_lexicon_tags_63 },
    { 0x61e82ce7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75082ce9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a82ced, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3882cf2, SUC_lexicon_tags_25 },
    { 0x38a82cf2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c82cf7, SUC_lexicon_tags_8 },
    { 0x2be82cf7, SUC_lexicon_tags_19 },
    { 0x19082cf9, SUC_lexicon_tags_106 },
    { 0xf0c82cf8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5882cfd, SUC_lexicon_tags_6 },
    { 0xa3a82cfe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3682d01, SUC_lexicon_tags_66 },
    { 0x91c82d02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d682d04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x44482d07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a882d0a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7282d0f, SUC_lexicon_tags_13 },
    { 0xdfc82d10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a82d17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfc282d19, SUC_lexicon_tags_11 },
    { 0x1b682d19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea882d1c, SUC_lexicon_tags_41 },
    { 0x47482d1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf282d21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1882d25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34082d2b, SUC_lexicon_tags_8 },
    { 0xb682d2b, SUC_lexicon_tags_37 },
    { 0x88082d2b, SUC_lexicon_tags_10 },
    { 0x24682d2c, SUC_lexicon_tags_2 },
    { 0xec282d2f, SUC_lexicon_tags_19 },
    { 0x25a82d30, SUC_lexicon_tags_7 },
    { 0xd0682d30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc82d38, SUC_lexicon_tags_0 },
    { 0x8c082d39, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xa1c82d3b, SUC_lexicon_tags_46 },
    { 0x7bc82d3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc6c82d3e, SUC_lexicon_tags_37 },
    { 0x4aa82d3f, SUC_lexicon_tags_61 },
    { 0xdbe82d40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fa82d42, SUC_lexicon_tags_17 },
    { 0xd7682d43, SUC_lexicon_tags_9 },
    { 0x68482d43, SUC_lexicon_tags_27 },
    { 0xe1a82d42, SUC_lexicon_tags_15 },
    { 0xb5882d46, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbfc82d48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99a82d4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba82d4d, SUC_lexicon_tags_1 },
    { 0x81a82d4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97e82d50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6c882d52, SUC_lexicon_tags_7 },
    { 0xc1082d53, SUC_lexicon_tags_65 },
    { 0xfc82d53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb482d56, SUC_lexicon_tags_38 },
    { 0x90882d56, SUC_lexicon_tags_25 },
    { 0xf9882d58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c82d5f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x1a82d61, SUC_lexicon_tags_15 },
    { 0xf5082d62, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca82d65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e82d68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a682d6a, SUC_lexicon_tags_66 },
    { 0xc4882d6b, SUC_lexicon_tags_11 },
    { 0xfba82d6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10882d6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa282d71, SUC_lexicon_tags_0 },
    { 0xfbc82d72, SUC_lexicon_tags_37 },
    { 0xc3682d71, SUC_lexicon_tags_53 },
    { 0x5fc82d74, SUC_lexicon_tags_39 },
    { 0x25082d72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x67e82d77, SUC_lexicon_tags_17 },
    { 0x51c82d77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4082d7a, SUC_lexicon_tags_29 },
    { 0xbb282d7b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x94482d7e, SUC_lexicon_tags_25 },
    { 0x51482d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e82d8a, SUC_lexicon_tags_25 },
    { 0x49a82d8a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6a282d8d, SUC_lexicon_tags_30 },
    { 0xdc082d8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec82d93, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c82d97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72682d99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7682d9b, SUC_lexicon_tags_65 },
    { 0xc2082d9c, SUC_lexicon_tags_30 },
    { 0xa8e82d9d, SUC_lexicon_tags_37 },
    { 0x87c82d9d, SUC_lexicon_tags_17 },
    { 0xabc82d9f, SUC_lexicon_tags_15 },
    { 0x83082da0, SUC_lexicon_tags_9 },
    { 0x60682da0, SUC_lexicon_tags_46 },
    { 0xae882da2, SUC_lexicon_tags_45 },
    { 0x29482d9e, SUC_lexicon_tags_37 },
    { 0xcc682da4, SUC_lexicon_tags_47 },
    { 0x4a682da1, SUC_lexicon_tags_7 },
    { 0x62882da5, SUC_lexicon_tags_42 },
    { 0xed082da2, SUC_lexicon_tags_17 },
    { 0x8b882d9e, SUC_lexicon_tags_1668 },
    { 0, NULL },
    { 0, NULL },
    { 0xce82dab, SUC_lexicon_tags_37 },
    { 0x27882dac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbe082dae, SUC_lexicon_tags_17 },
    { 0xb2282daf, SUC_lexicon_tags_30 },
    { 0xa9e82daf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f882db5, SUC_lexicon_tags_28 },
    { 0xd9a82db6, SUC_lexicon_tags_12 },
    { 0x6e882db7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a82dbb, SUC_lexicon_tags_25 },
    { 0xd4e82dbb, SUC_lexicon_tags_37 },
    { 0xe6e82dbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9482dbf, SUC_lexicon_tags_19 },
    { 0x4ec82dc0, SUC_lexicon_tags_13 },
    { 0x87482dc1, SUC_lexicon_tags_23 },
    { 0xe2c82dc2, SUC_lexicon_tags_30 },
    { 0x92082dc3, SUC_lexicon_tags_25 },
    { 0xff882dc4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6282dc7, SUC_lexicon_tags_33 },
    { 0xf482dc8, SUC_lexicon_tags_8 },
    { 0xcd682dc9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21682dcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a682dd0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3482dd7, SUC_lexicon_tags_45 },
    { 0xb7882dd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e82ddb, SUC_lexicon_tags_27 },
    { 0xdee82ddb, SUC_lexicon_tags_15 },
    { 0x7ae82ddc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb282de1, SUC_lexicon_tags_1 },
    { 0xb9a82de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97682de9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x62482deb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79482df0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc682df2, SUC_lexicon_tags_1 },
    { 0xc0c82df3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x99282df5, SUC_lexicon_tags_17 },
    { 0x59c82df6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb482df9, SUC_lexicon_tags_11 },
    { 0xc3a82dfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25282dfc, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x69282dff, SUC_lexicon_tags_12 },
    { 0xcb082e00, SUC_lexicon_tags_0 },
    { 0x8de82e00, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef682e09, SUC_lexicon_tags_25 },
    { 0x5ee82e0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11082e0d, SUC_lexicon_tags_61 },
    { 0xe6882e0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59282e10, SUC_lexicon_tags_66 },
    { 0xe082e11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc882e13, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x48c82e15, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb7282e17, SUC_lexicon_tags_2 },
    { 0x5fe82e17, SUC_lexicon_tags_9 },
    { 0xb2082e19, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xd6682e1b, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x2d482e1d, SUC_lexicon_tags_31 },
    { 0x43682e1d, SUC_lexicon_tags_7 },
    { 0xf2282e1d, SUC_lexicon_tags_24 },
    { 0x70882e20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ae82e22, SUC_lexicon_tags_0 },
    { 0x9b682e22, SUC_lexicon_tags_17 },
    { 0x7c482e24, SUC_lexicon_tags_11 },
    { 0x79282e25, SUC_lexicon_tags_17 },
    { 0x95682e23, SUC_lexicon_tags_9 },
    { 0x8e82e27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe4e82e29, SUC_lexicon_tags_0 },
    { 0x6c282e2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80482e2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2e82e2e, SUC_lexicon_tags_9 },
    { 0x14482e2f, SUC_lexicon_tags_2 },
    { 0x87e82e2f, SUC_lexicon_tags_43 },
    { 0x3ae82e30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62082e33, SUC_lexicon_tags_45 },
    { 0x91282e33, SUC_lexicon_tags_6 },
    { 0xdaa82e34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe682e37, SUC_lexicon_tags_37 },
    { 0x46882e38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e82e3b, SUC_lexicon_tags_12 },
    { 0x2f282e3c, SUC_lexicon_tags_5 },
    { 0x34e82e3d, SUC_lexicon_tags_29 },
    { 0xc3682e3b, SUC_lexicon_tags_46 },
    { 0x7e82e3e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x11682e42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e82e45, SUC_lexicon_tags_33 },
    { 0xfac82e45, SUC_lexicon_tags_12 },
    { 0x2082e47, SUC_lexicon_tags_79 },
    { 0x2a282e48, SUC_lexicon_tags_48 },
    { 0x64682e48, SUC_lexicon_tags_10 },
    { 0x42482e49, SUC_lexicon_tags_8 },
    { 0x39c82e4b, SUC_lexicon_tags_11 },
    { 0x21082e49, SUC_lexicon_tags_37 },
    { 0x46482e4d, SUC_lexicon_tags_5 },
    { 0x3e482e4e, SUC_lexicon_tags_33 },
    { 0xc2e82e4f, SUC_lexicon_tags_659 },
    { 0xe4e82e4f, SUC_lexicon_tags_33 },
    { 0xa7a82e51, SUC_lexicon_tags_30 },
    { 0xeb482e52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e82e5e, SUC_lexicon_tags_1 },
    { 0xd4882e5f, SUC_lexicon_tags_11 },
    { 0x8be82e60, SUC_lexicon_tags_20 },
    { 0xcc082e61, SUC_lexicon_tags_37 },
    { 0x28482e62, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x14a82e64, SUC_lexicon_tags_23 },
    { 0x6aa82e65, SUC_lexicon_tags_2 },
    { 0x18c82e66, SUC_lexicon_tags_37 },
    { 0x58e82e67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5b082e69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1082e6b, SUC_lexicon_tags_11 },
    { 0x53a82e6c, SUC_lexicon_tags_170 },
    { 0xeea82e6b, SUC_lexicon_tags_0 },
    { 0x6fe82e6e, SUC_lexicon_tags_1 },
    { 0xbb682e6f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e882e73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4082e76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x32082e78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac82e7b, SUC_lexicon_tags_2 },
    { 0x4dc82e7c, SUC_lexicon_tags_11 },
    { 0x88e82e7d, SUC_lexicon_tags_7 },
    { 0x5f482e7c, SUC_lexicon_tags_7 },
    { 0x77a82e7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae82e84, SUC_lexicon_tags_0 },
    { 0xfee82e85, SUC_lexicon_tags_11 },
    { 0x37e82e86, SUC_lexicon_tags_37 },
    { 0xb4e82e84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81482e8a, SUC_lexicon_tags_63 },
    { 0x7f682e8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc82e8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e82e93, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a82e97, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x38c82e99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12282e9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc82ea3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe4682ea5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb482ea8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9482eaa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb2282eac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1482eaf, SUC_lexicon_tags_12 },
    { 0xdd682eb0, SUC_lexicon_tags_30 },
    { 0x7282eb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10882eb4, SUC_lexicon_tags_7 },
    { 0xa082eb5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf3e82eb7, SUC_lexicon_tags_43 },
    { 0xf8882eb8, SUC_lexicon_tags_46 },
    { 0x93282eb9, SUC_lexicon_tags_11 },
    { 0xd5082eb8, SUC_lexicon_tags_15 },
    { 0xc6c82eb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a82ebe, SUC_lexicon_tags_7 },
    { 0x5f082ebf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80482ec3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67482ec7, SUC_lexicon_tags_2 },
    { 0x5da82ec8, SUC_lexicon_tags_11 },
    { 0x4ee82ec9, SUC_lexicon_tags_26 },
    { 0xfe082ec8, SUC_lexicon_tags_15 },
    { 0x13682ecb, SUC_lexicon_tags_45 },
    { 0xf5e82ecb, SUC_lexicon_tags_47 },
    { 0x73c82ecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73482ed1, SUC_lexicon_tags_54 },
    { 0x1fc82ed2, SUC_lexicon_tags_12 },
    { 0x52482ed2, SUC_lexicon_tags_45 },
    { 0xa2282ed4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a282ed8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c82edb, SUC_lexicon_tags_38 },
    { 0x74c82edc, SUC_lexicon_tags_1 },
    { 0xd8c82edc, SUC_lexicon_tags_0 },
    { 0xba682edd, SUC_lexicon_tags_7 },
    { 0x45082ede, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x15a82ee1, SUC_lexicon_tags_25 },
    { 0x97c82ee2, SUC_lexicon_tags_17 },
    { 0xd2082ee2, SUC_lexicon_tags_29 },
    { 0x98c82ee4, SUC_lexicon_tags_7 },
    { 0x62c82ee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45a82ee7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c82eea, SUC_lexicon_tags_6 },
    { 0xe9082eeb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc282ef0, SUC_lexicon_tags_25 },
    { 0xe9c82ef0, SUC_lexicon_tags_80 },
    { 0x7a82ef0, SUC_lexicon_tags_1 },
    { 0x94e82ef0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb6682ef5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e82ef8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a82efc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b082f07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87682f0b, SUC_lexicon_tags_25 },
    { 0x8d282f0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba82f15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x89282f18, SUC_lexicon_tags_15 },
    { 0x23a82f19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a82f1d, SUC_lexicon_tags_2 },
    { 0x65e82f1e, SUC_lexicon_tags_27 },
    { 0xeee82f1f, SUC_lexicon_tags_28 },
    { 0xfa082f20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38a82f22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b282f26, SUC_lexicon_tags_15 },
    { 0xb1482f27, SUC_lexicon_tags_7 },
    { 0x3be82f28, SUC_lexicon_tags_6 },
    { 0xd5882f29, SUC_lexicon_tags_68 },
    { 0x21a82f2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9082f31, SUC_lexicon_tags_10 },
    { 0x5282f32, SUC_lexicon_tags_15 },
    { 0xc8482f33, SUC_lexicon_tags_40 },
    { 0x28282f34, SUC_lexicon_tags_11 },
    { 0x8e882f35, SUC_lexicon_tags_56 },
    { 0xf882f36, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc82f3a, SUC_lexicon_tags_23 },
    { 0x8d882f3b, SUC_lexicon_tags_29 },
    { 0x8a482f3c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c82f40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69a82f42, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee82f4a, SUC_lexicon_tags_1 },
    { 0x57682f4a, SUC_lexicon_tags_36 },
    { 0xe2882f4c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a82f51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10282f55, SUC_lexicon_tags_25 },
    { 0xb7082f56, SUC_lexicon_tags_37 },
    { 0x11a82f56, SUC_lexicon_tags_12 },
    { 0xa2482f58, SUC_lexicon_tags_47 },
    { 0xca482f58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d682f5f, SUC_lexicon_tags_10 },
    { 0x86e82f5f, SUC_lexicon_tags_14 },
    { 0x86882f60, SUC_lexicon_tags_19 },
    { 0x86a82f5f, SUC_lexicon_tags_60 },
    { 0xf9682f63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4882f65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x90482f67, SUC_lexicon_tags_47 },
    { 0x80282f68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82682f6c, SUC_lexicon_tags_61 },
    { 0xd4a82f6d, SUC_lexicon_tags_12 },
    { 0x44082f6d, SUC_lexicon_tags_21 },
    { 0x9482f6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90882f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9482f76, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53082f7a, SUC_lexicon_tags_12 },
    { 0xed082f7a, SUC_lexicon_tags_7 },
    { 0xd8e82f7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f082f7e, SUC_lexicon_tags_37 },
    { 0x2bc82f7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35282f82, SUC_lexicon_tags_14 },
    { 0x7c882f82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc082f89, SUC_lexicon_tags_25 },
    { 0x26682f89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b882f8f, SUC_lexicon_tags_43 },
    { 0x3c682f8f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c82f93, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6482f97, SUC_lexicon_tags_37 },
    { 0x5d682f98, SUC_lexicon_tags_15 },
    { 0x30682f99, SUC_lexicon_tags_29 },
    { 0x73282f99, SUC_lexicon_tags_57 },
    { 0x76282f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f082f9f, SUC_lexicon_tags_49 },
    { 0xe0882f9f, SUC_lexicon_tags_92 },
    { 0xb682fa0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4482fa7, SUC_lexicon_tags_49 },
    { 0xa8282fa7, SUC_lexicon_tags_45 },
    { 0xeb482fa9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28282fab, SUC_lexicon_tags_37 },
    { 0xa2282fab, SUC_lexicon_tags_77 },
    { 0x8ca82fad, SUC_lexicon_tags_49 },
    { 0x23282fac, SUC_lexicon_tags_48 },
    { 0xbce82faf, SUC_lexicon_tags_47 },
    { 0xa9482fb0, SUC_lexicon_tags_8 },
    { 0x4ca82fb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94a82fb3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e82fb9, SUC_lexicon_tags_6 },
    { 0x23e82fb9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb082fc2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c82fc5, SUC_lexicon_tags_16 },
    { 0x42682fc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71282fcd, SUC_lexicon_tags_1 },
    { 0xb682fce, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e82fd3, SUC_lexicon_tags_15 },
    { 0x17e82fd3, SUC_lexicon_tags_7 },
    { 0x5f482fd4, SUC_lexicon_tags_33 },
    { 0x79682fd3, SUC_lexicon_tags_17 },
    { 0x72c82fd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a282fdb, SUC_lexicon_tags_141 },
    { 0x58682fdb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e682fe1, SUC_lexicon_tags_2 },
    { 0xb8c82fe1, SUC_lexicon_tags_3 },
    { 0x58482fe1, SUC_lexicon_tags_2 },
    { 0xd2282fe4, SUC_lexicon_tags_103 },
    { 0x80e82fe3, SUC_lexicon_tags_30 },
    { 0x4d882fe6, SUC_lexicon_tags_63 },
    { 0x14482fe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70482fe9, SUC_lexicon_tags_13 },
    { 0x4282fea, SUC_lexicon_tags_12 },
    { 0x46082feb, SUC_lexicon_tags_6 },
    { 0x35a82feb, SUC_lexicon_tags_37 },
    { 0xa0482fed, SUC_lexicon_tags_15 },
    { 0x48e82fee, SUC_lexicon_tags_37 },
    { 0xabe82fef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc82ff3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a82ff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13882ff8, SUC_lexicon_tags_19 },
    { 0xd7282ff9, SUC_lexicon_tags_15 },
    { 0x31082ff8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8483000, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82483002, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a83006, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a83009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a8300c, SUC_lexicon_tags_4 },
    { 0x29e8300d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x5a88300f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a83016, SUC_lexicon_tags_54 },
    { 0xa1083017, SUC_lexicon_tags_18 },
    { 0x5e083018, SUC_lexicon_tags_7 },
    { 0xf2283019, SUC_lexicon_tags_9 },
    { 0x4fc83019, SUC_lexicon_tags_32 },
    { 0xd828301b, SUC_lexicon_tags_15 },
    { 0x5ac8301b, SUC_lexicon_tags_48 },
    { 0xd788301d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a83021, SUC_lexicon_tags_19 },
    { 0x62083022, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7083025, SUC_lexicon_tags_707 },
    { 0, NULL },
    { 0xe6283027, SUC_lexicon_tags_66 },
    { 0x42483028, SUC_lexicon_tags_7 },
    { 0x25083029, SUC_lexicon_tags_47 },
    { 0xfc68302a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41883030, SUC_lexicon_tags_17 },
    { 0x4ea83030, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1883033, SUC_lexicon_tags_17 },
    { 0x13683033, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a283039, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1e8303b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7488303f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d483046, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x683048, SUC_lexicon_tags_25 },
    { 0x90083048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a8304d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62a8304f, SUC_lexicon_tags_25 },
    { 0x2f683050, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5b883052, SUC_lexicon_tags_19 },
    { 0xf5483053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9683055, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc8305a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x5808305c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e83063, SUC_lexicon_tags_2 },
    { 0xcae83063, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd848306b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9228306e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a83072, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd083077, SUC_lexicon_tags_45 },
    { 0x81483078, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2283082, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69483086, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc588308c, SUC_lexicon_tags_33 },
    { 0xb908308d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x36e8308f, SUC_lexicon_tags_6 },
    { 0x23c83090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94683092, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ce83094, SUC_lexicon_tags_1 },
    { 0x98883095, SUC_lexicon_tags_1 },
    { 0xf283096, SUC_lexicon_tags_37 },
    { 0x67283096, SUC_lexicon_tags_0 },
    { 0x43c83094, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbd28309a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa428309e, SUC_lexicon_tags_17 },
    { 0x6808309e, SUC_lexicon_tags_39 },
    { 0x472830a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a830a3, SUC_lexicon_tags_48 },
    { 0x954830a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a830aa, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4830ad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a830b2, SUC_lexicon_tags_2 },
    { 0xffe830b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71a830b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256830bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4b6830bd, SUC_lexicon_tags_0 },
    { 0x886830be, SUC_lexicon_tags_17 },
    { 0x60830bf, SUC_lexicon_tags_17 },
    { 0xde830be, SUC_lexicon_tags_224 },
    { 0xcd2830c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10830c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c830c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4830cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada830d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8830d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c0830d6, SUC_lexicon_tags_23 },
    { 0x6fe830d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732830db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c830de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdb6830e0, SUC_lexicon_tags_17 },
    { 0x2d0830e0, SUC_lexicon_tags_17 },
    { 0x1f2830e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e830e6, SUC_lexicon_tags_37 },
    { 0xf64830e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e830ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x132830f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72830f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6830f7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x226830f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e830fc, SUC_lexicon_tags_14 },
    { 0x10c830fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e83102, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c283108, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3248310d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6c8310f, SUC_lexicon_tags_17 },
    { 0x11483110, SUC_lexicon_tags_27 },
    { 0x7fe83111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe83114, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xb1283116, SUC_lexicon_tags_37 },
    { 0x14683117, SUC_lexicon_tags_15 },
    { 0x75e83117, SUC_lexicon_tags_7 },
    { 0x31083117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a8311d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae83122, SUC_lexicon_tags_25 },
    { 0xebe83123, SUC_lexicon_tags_16 },
    { 0x60683124, SUC_lexicon_tags_15 },
    { 0xd3a83124, SUC_lexicon_tags_60 },
    { 0x15283126, SUC_lexicon_tags_47 },
    { 0x3a483124, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2108312e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26683132, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a8313a, SUC_lexicon_tags_7 },
    { 0x3768313a, SUC_lexicon_tags_31 },
    { 0xb148313b, SUC_lexicon_tags_15 },
    { 0x868313a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8948313f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf083142, SUC_lexicon_tags_17 },
    { 0xb5083143, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x51483145, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f483148, SUC_lexicon_tags_45 },
    { 0x47c83148, SUC_lexicon_tags_7 },
    { 0x3c68314a, SUC_lexicon_tags_63 },
    { 0x90e8314b, SUC_lexicon_tags_37 },
    { 0x9708314c, SUC_lexicon_tags_1 },
    { 0xd528314d, SUC_lexicon_tags_15 },
    { 0x6f88314b, SUC_lexicon_tags_30 },
    { 0x4008314b, SUC_lexicon_tags_17 },
    { 0x7b28314a, SUC_lexicon_tags_13 },
    { 0x28314e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95883153, SUC_lexicon_tags_45 },
    { 0xc5c83153, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0883158, SUC_lexicon_tags_14 },
    { 0xda083159, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e8315d, SUC_lexicon_tags_25 },
    { 0x5868315e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94883162, SUC_lexicon_tags_61 },
    { 0x41e83162, SUC_lexicon_tags_12 },
    { 0x38e83164, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32a83166, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d883169, SUC_lexicon_tags_38 },
    { 0x6548316a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8316d, SUC_lexicon_tags_45 },
    { 0x24e8316d, SUC_lexicon_tags_11 },
    { 0xc6c8316d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a83173, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa48317e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1883183, SUC_lexicon_tags_14 },
    { 0xeea83184, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc8318a, SUC_lexicon_tags_45 },
    { 0x66c8318b, SUC_lexicon_tags_2 },
    { 0x5468318c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa28318e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c883193, SUC_lexicon_tags_21 },
    { 0x7e483194, SUC_lexicon_tags_37 },
    { 0x10083194, SUC_lexicon_tags_45 },
    { 0xede83193, SUC_lexicon_tags_1 },
    { 0x40083197, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7168319b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf748319d, SUC_lexicon_tags_27 },
    { 0x7288319d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0831a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x844831a9, SUC_lexicon_tags_33 },
    { 0x91a831a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2831b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c831b5, SUC_lexicon_tags_68 },
    { 0xe1a831b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c831c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc831c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e831ce, SUC_lexicon_tags_1 },
    { 0x52831ce, SUC_lexicon_tags_40 },
    { 0x138831cf, SUC_lexicon_tags_46 },
    { 0x1ba831cf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde831d4, SUC_lexicon_tags_11 },
    { 0x1d2831d5, SUC_lexicon_tags_15 },
    { 0xe8e831d6, SUC_lexicon_tags_47 },
    { 0xb16831d7, SUC_lexicon_tags_11 },
    { 0xbf2831d8, SUC_lexicon_tags_25 },
    { 0xfa2831d9, SUC_lexicon_tags_37 },
    { 0x794831d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x424831dd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x262831e0, SUC_lexicon_tags_3 },
    { 0x44831e1, SUC_lexicon_tags_49 },
    { 0x818831e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde831e5, SUC_lexicon_tags_33 },
    { 0xda6831e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab0831e8, SUC_lexicon_tags_37 },
    { 0x958831e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376831ef, SUC_lexicon_tags_28 },
    { 0xb6e831f0, SUC_lexicon_tags_1 },
    { 0xc06831f1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12831fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8831fd, SUC_lexicon_tags_25 },
    { 0x506831fd, SUC_lexicon_tags_25 },
    { 0xeb2831fd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1483203, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf083207, SUC_lexicon_tags_7 },
    { 0x48c83208, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2908320a, SUC_lexicon_tags_37 },
    { 0xc908320b, SUC_lexicon_tags_37 },
    { 0xe0e8320a, SUC_lexicon_tags_14 },
    { 0x2368320d, SUC_lexicon_tags_2 },
    { 0x7388320e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8f683210, SUC_lexicon_tags_29 },
    { 0x3bc83211, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a083214, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e83218, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc708321a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a8321d, SUC_lexicon_tags_6 },
    { 0x9a28321d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5283220, SUC_lexicon_tags_11 },
    { 0x43e83221, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483226, SUC_lexicon_tags_27 },
    { 0xa3083227, SUC_lexicon_tags_7 },
    { 0x94083228, SUC_lexicon_tags_33 },
    { 0x39683229, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2408322b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c28322f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb483238, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8323b, SUC_lexicon_tags_32 },
    { 0xd068323c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e83240, SUC_lexicon_tags_26 },
    { 0x83241, SUC_lexicon_tags_26 },
    { 0x8883241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b28324b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c83252, SUC_lexicon_tags_38 },
    { 0xde483252, SUC_lexicon_tags_17 },
    { 0x32483254, SUC_lexicon_tags_7 },
    { 0x39683255, SUC_lexicon_tags_11 },
    { 0x45683255, SUC_lexicon_tags_1 },
    { 0x69883257, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e8325b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61e8325d, SUC_lexicon_tags_47 },
    { 0xd988325e, SUC_lexicon_tags_17 },
    { 0x5868325f, SUC_lexicon_tags_21 },
    { 0xf368325f, SUC_lexicon_tags_42 },
    { 0xaa68325f, SUC_lexicon_tags_38 },
    { 0xb883262, SUC_lexicon_tags_11 },
    { 0x84283263, SUC_lexicon_tags_30 },
    { 0xfc483262, SUC_lexicon_tags_88 },
    { 0xe3a83264, SUC_lexicon_tags_7 },
    { 0xc1c83262, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668326d, SUC_lexicon_tags_0 },
    { 0xa788326e, SUC_lexicon_tags_0 },
    { 0xe028326d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c83274, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57c83276, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da8327a, SUC_lexicon_tags_38 },
    { 0xae48327a, SUC_lexicon_tags_12 },
    { 0x49c8327a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8c8327e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdd083280, SUC_lexicon_tags_30 },
    { 0x75283281, SUC_lexicon_tags_37 },
    { 0xb8483280, SUC_lexicon_tags_63 },
    { 0x46083283, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x95883285, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4308328e, SUC_lexicon_tags_47 },
    { 0xab28328e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xcfe83291, SUC_lexicon_tags_11 },
    { 0x50683291, SUC_lexicon_tags_32 },
    { 0xb6083293, SUC_lexicon_tags_34 },
    { 0x2283293, SUC_lexicon_tags_37 },
    { 0x8ba83293, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c83299, SUC_lexicon_tags_40 },
    { 0x9368329a, SUC_lexicon_tags_11 },
    { 0xb9283299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0832a0, SUC_lexicon_tags_31 },
    { 0xf7e832a0, SUC_lexicon_tags_40 },
    { 0xa20832a2, SUC_lexicon_tags_10 },
    { 0x9ba832a3, SUC_lexicon_tags_7 },
    { 0x22e832a1, SUC_lexicon_tags_8 },
    { 0xf46832a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67c832a7, SUC_lexicon_tags_12 },
    { 0xbba832a8, SUC_lexicon_tags_25 },
    { 0x5d2832a9, SUC_lexicon_tags_15 },
    { 0x258832a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7da832ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14832b7, SUC_lexicon_tags_8 },
    { 0x49a832b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e832c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e832c3, SUC_lexicon_tags_575 },
    { 0x97e832c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76832c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c832cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa06832cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0832d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca832d6, SUC_lexicon_tags_25 },
    { 0x6f8832d7, SUC_lexicon_tags_101 },
    { 0xfde832d8, SUC_lexicon_tags_11 },
    { 0xb86832d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7ae832db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78832df, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x44a832e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x114832e4, SUC_lexicon_tags_37 },
    { 0x218832e5, SUC_lexicon_tags_29 },
    { 0x73c832e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb66832e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c832f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8a832f4, SUC_lexicon_tags_44 },
    { 0x4f2832f5, SUC_lexicon_tags_17 },
    { 0xfd6832f6, SUC_lexicon_tags_79 },
    { 0x88c832f7, SUC_lexicon_tags_33 },
    { 0x38e832f5, SUC_lexicon_tags_17 },
    { 0xc7e832f9, SUC_lexicon_tags_33 },
    { 0xc18832f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246832fe, SUC_lexicon_tags_60 },
    { 0x442832ff, SUC_lexicon_tags_15 },
    { 0xdb083300, SUC_lexicon_tags_11 },
    { 0x9e283301, SUC_lexicon_tags_23 },
    { 0xfec83300, SUC_lexicon_tags_26 },
    { 0xa7483303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff883308, SUC_lexicon_tags_0 },
    { 0xae083309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4508330e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e83314, SUC_lexicon_tags_14 },
    { 0x6de83315, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68331a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6068331c, SUC_lexicon_tags_87 },
    { 0xc2e8331d, SUC_lexicon_tags_7 },
    { 0x1808331d, SUC_lexicon_tags_37 },
    { 0xf6e8331f, SUC_lexicon_tags_4 },
    { 0x57483320, SUC_lexicon_tags_15 },
    { 0xd7483321, SUC_lexicon_tags_33 },
    { 0x24283320, SUC_lexicon_tags_22 },
    { 0x5283323, SUC_lexicon_tags_23 },
    { 0xc48331c, SUC_lexicon_tags_87 },
    { 0x7568331f, SUC_lexicon_tags_1 },
    { 0x1f883326, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e083329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4448332c, SUC_lexicon_tags_7 },
    { 0x4388332c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2283331, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d283334, SUC_lexicon_tags_11 },
    { 0x98083335, SUC_lexicon_tags_2 },
    { 0x5ee83334, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e83339, SUC_lexicon_tags_59 },
    { 0x1cc83339, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9ac8333c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46a8333e, SUC_lexicon_tags_1 },
    { 0xcf28333f, SUC_lexicon_tags_8 },
    { 0xfee83340, SUC_lexicon_tags_19 },
    { 0xaee83341, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x60883344, SUC_lexicon_tags_12 },
    { 0x29e83345, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a483349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8168334b, SUC_lexicon_tags_7 },
    { 0x79c8334c, SUC_lexicon_tags_63 },
    { 0x17a8334d, SUC_lexicon_tags_27 },
    { 0x1dc8334d, SUC_lexicon_tags_11 },
    { 0x3568334d, SUC_lexicon_tags_27 },
    { 0xdd883350, SUC_lexicon_tags_2 },
    { 0x75e83351, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x60483353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d683358, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d08335c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5228335e, SUC_lexicon_tags_17 },
    { 0x7a88335f, SUC_lexicon_tags_9 },
    { 0x84a83360, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e483362, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x18883365, SUC_lexicon_tags_105 },
    { 0xfe483366, SUC_lexicon_tags_47 },
    { 0xef483367, SUC_lexicon_tags_30 },
    { 0xd5083368, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c8336b, SUC_lexicon_tags_12 },
    { 0x40e8336c, SUC_lexicon_tags_82 },
    { 0xcb88336d, SUC_lexicon_tags_9 },
    { 0x8708336d, SUC_lexicon_tags_7 },
    { 0x5a88336f, SUC_lexicon_tags_49 },
    { 0xee283370, SUC_lexicon_tags_12 },
    { 0xc028336f, SUC_lexicon_tags_23 },
    { 0x76483371, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b083374, SUC_lexicon_tags_45 },
    { 0xe4883375, SUC_lexicon_tags_2 },
    { 0x2b883375, SUC_lexicon_tags_7 },
    { 0xb9683375, SUC_lexicon_tags_11 },
    { 0xc0a83376, SUC_lexicon_tags_92 },
    { 0xde083379, SUC_lexicon_tags_17 },
    { 0xd1683377, SUC_lexicon_tags_38 },
    { 0xa9a8337b, SUC_lexicon_tags_167 },
    { 0xb548337c, SUC_lexicon_tags_42 },
    { 0x1dc8337d, SUC_lexicon_tags_19 },
    { 0xc748337c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeec83380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89883385, SUC_lexicon_tags_15 },
    { 0x32c83386, SUC_lexicon_tags_31 },
    { 0x3e883387, SUC_lexicon_tags_25 },
    { 0x35883388, SUC_lexicon_tags_33 },
    { 0xec683387, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f88338c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28338f, SUC_lexicon_tags_15 },
    { 0x7c83390, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2a83392, SUC_lexicon_tags_12 },
    { 0x5c883393, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x80883395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf483398, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2528339e, SUC_lexicon_tags_38 },
    { 0xdcc8339f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x696833a1, SUC_lexicon_tags_15 },
    { 0x14a833a1, SUC_lexicon_tags_1084 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec833a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8833ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdaa833af, SUC_lexicon_tags_11 },
    { 0x3bc833b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6833b4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x8a0833b6, SUC_lexicon_tags_85 },
    { 0x9ae833b7, SUC_lexicon_tags_29 },
    { 0xd2c833b8, SUC_lexicon_tags_87 },
    { 0x534833b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ea833bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60833ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6833d6, SUC_lexicon_tags_985 },
    { 0xf76833d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x682833d9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe94833db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeec833dd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa833e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246833e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc56833e8, SUC_lexicon_tags_12 },
    { 0x3c0833e8, SUC_lexicon_tags_6 },
    { 0xa74833e8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7d2833ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4833f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea833f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e833f8, SUC_lexicon_tags_104 },
    { 0x924833f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x104833fb, SUC_lexicon_tags_1376 },
    { 0, NULL },
    { 0x218833fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a283400, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f083404, SUC_lexicon_tags_37 },
    { 0x79a83405, SUC_lexicon_tags_15 },
    { 0x38683406, SUC_lexicon_tags_12 },
    { 0xab283407, SUC_lexicon_tags_39 },
    { 0x5da83407, SUC_lexicon_tags_60 },
    { 0x44a83406, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e8340c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea83410, SUC_lexicon_tags_37 },
    { 0xda83411, SUC_lexicon_tags_38 },
    { 0x5083412, SUC_lexicon_tags_11 },
    { 0x54c83413, SUC_lexicon_tags_29 },
    { 0xb3883413, SUC_lexicon_tags_1 },
    { 0xa4883414, SUC_lexicon_tags_45 },
    { 0x22083414, SUC_lexicon_tags_2 },
    { 0xfc083417, SUC_lexicon_tags_0 },
    { 0x5e883417, SUC_lexicon_tags_31 },
    { 0x1b683412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a8341c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8368341e, SUC_lexicon_tags_23 },
    { 0x4128341f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13883423, SUC_lexicon_tags_31 },
    { 0x34083423, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x30a83426, SUC_lexicon_tags_2 },
    { 0x57e83427, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe8342b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d883433, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0483436, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c68343a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca8343e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5b283440, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5683448, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc8344b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd708344d, SUC_lexicon_tags_2 },
    { 0x6a08344e, SUC_lexicon_tags_29 },
    { 0x8c08344f, SUC_lexicon_tags_8 },
    { 0x75a8344f, SUC_lexicon_tags_2 },
    { 0xc048344e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c83454, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa683456, SUC_lexicon_tags_37 },
    { 0xe1a83457, SUC_lexicon_tags_133 },
    { 0x7c483458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ea8345a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace8345e, SUC_lexicon_tags_82 },
    { 0x9148345f, SUC_lexicon_tags_29 },
    { 0xe0e8345f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85083464, SUC_lexicon_tags_19 },
    { 0xf8e83465, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c83468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc648346a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1808346c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5788346e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc083470, SUC_lexicon_tags_15 },
    { 0x7a83470, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17283476, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf683479, SUC_lexicon_tags_63 },
    { 0x3528347a, SUC_lexicon_tags_25 },
    { 0xb888347b, SUC_lexicon_tags_12 },
    { 0x2268347c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec28347e, SUC_lexicon_tags_15 },
    { 0xfc08347f, SUC_lexicon_tags_19 },
    { 0xb9e83480, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75c83482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e483487, SUC_lexicon_tags_25 },
    { 0x3a483488, SUC_lexicon_tags_25 },
    { 0x2b483489, SUC_lexicon_tags_7 },
    { 0xf2883488, SUC_lexicon_tags_1 },
    { 0xca48348b, SUC_lexicon_tags_39 },
    { 0x20e8348b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e883492, SUC_lexicon_tags_45 },
    { 0xfe83493, SUC_lexicon_tags_2 },
    { 0xee683492, SUC_lexicon_tags_13 },
    { 0x62883495, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30483497, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd08349c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xad88349e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69a834a0, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8834a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6834a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388834ad, SUC_lexicon_tags_61 },
    { 0xb22834ae, SUC_lexicon_tags_32 },
    { 0x294834af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c2834b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0834b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c834b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc834bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506834c1, SUC_lexicon_tags_17 },
    { 0x5fe834c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3e0834c4, SUC_lexicon_tags_30 },
    { 0xd7e834c5, SUC_lexicon_tags_0 },
    { 0x970834c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc3c834c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e834d0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x312834d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x94a834d4, SUC_lexicon_tags_23 },
    { 0xa28834d5, SUC_lexicon_tags_3 },
    { 0x17a834d5, SUC_lexicon_tags_37 },
    { 0x594834d7, SUC_lexicon_tags_57 },
    { 0x72e834d4, SUC_lexicon_tags_25 },
    { 0x3f8834d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x12834dc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2834e0, SUC_lexicon_tags_2 },
    { 0x81e834e1, SUC_lexicon_tags_12 },
    { 0xb40834e2, SUC_lexicon_tags_25 },
    { 0x7d6834e2, SUC_lexicon_tags_42 },
    { 0xa1a834e4, SUC_lexicon_tags_17 },
    { 0x948834e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x604834e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x596834ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec834ef, SUC_lexicon_tags_33 },
    { 0x658834f0, SUC_lexicon_tags_17 },
    { 0x3aa834f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x19a834f3, SUC_lexicon_tags_43 },
    { 0x566834f3, SUC_lexicon_tags_40 },
    { 0x68c834f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4834f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e834fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21883506, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd848350a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90283515, SUC_lexicon_tags_32 },
    { 0x6c283515, SUC_lexicon_tags_15 },
    { 0xf7883517, SUC_lexicon_tags_37 },
    { 0xe1083517, SUC_lexicon_tags_37 },
    { 0xbca83519, SUC_lexicon_tags_15 },
    { 0xb8883519, SUC_lexicon_tags_2 },
    { 0x4ac8351a, SUC_lexicon_tags_10 },
    { 0x27483516, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c8351f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5083523, SUC_lexicon_tags_56 },
    { 0xf5283524, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba83528, SUC_lexicon_tags_13 },
    { 0xfe83529, SUC_lexicon_tags_11 },
    { 0xeb28352a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec8352d, SUC_lexicon_tags_2 },
    { 0x6728352e, SUC_lexicon_tags_63 },
    { 0x9a8352f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35483534, SUC_lexicon_tags_3 },
    { 0x17683534, SUC_lexicon_tags_31 },
    { 0x9a083536, SUC_lexicon_tags_21 },
    { 0x1ca83535, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8353a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c8353c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75a8353e, SUC_lexicon_tags_1 },
    { 0xa4e8353f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c83542, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29483545, SUC_lexicon_tags_47 },
    { 0xd1c83546, SUC_lexicon_tags_0 },
    { 0xc9a83546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9283550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b083557, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f88355a, SUC_lexicon_tags_17 },
    { 0x6d28355b, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe8355e, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0xfba83560, SUC_lexicon_tags_0 },
    { 0x3e483561, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc88356d, SUC_lexicon_tags_39 },
    { 0x2a8356e, SUC_lexicon_tags_11 },
    { 0x5568356f, SUC_lexicon_tags_25 },
    { 0xf1683570, SUC_lexicon_tags_12 },
    { 0xeee83570, SUC_lexicon_tags_37 },
    { 0xd988356d, SUC_lexicon_tags_37 },
    { 0xe9a83573, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d683575, SUC_lexicon_tags_33 },
    { 0x7683576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae68357b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb488357d, SUC_lexicon_tags_57 },
    { 0xb1a8357d, SUC_lexicon_tags_28 },
    { 0x7dc8357f, SUC_lexicon_tags_47 },
    { 0xada83580, SUC_lexicon_tags_11 },
    { 0x56483581, SUC_lexicon_tags_25 },
    { 0xa2a83581, SUC_lexicon_tags_16 },
    { 0x328357d, SUC_lexicon_tags_31 },
    { 0x3283580, SUC_lexicon_tags_2 },
    { 0xc7483585, SUC_lexicon_tags_12 },
    { 0xef883586, SUC_lexicon_tags_7 },
    { 0x6ac83587, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2528358a, SUC_lexicon_tags_25 },
    { 0x3708358b, SUC_lexicon_tags_4 },
    { 0xcec8358c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b08358e, SUC_lexicon_tags_12 },
    { 0x3c88358e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6283592, SUC_lexicon_tags_23 },
    { 0xaea83593, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da83596, SUC_lexicon_tags_12 },
    { 0xdd883596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ee83599, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaf68359b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5068359d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8835a3, SUC_lexicon_tags_1 },
    { 0x1f8835a4, SUC_lexicon_tags_10 },
    { 0x1d0835a5, SUC_lexicon_tags_11 },
    { 0xe3a835a6, SUC_lexicon_tags_38 },
    { 0x2fe835a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x218835a9, SUC_lexicon_tags_17 },
    { 0x262835a9, SUC_lexicon_tags_1 },
    { 0xaba835ab, SUC_lexicon_tags_43 },
    { 0x28835ab, SUC_lexicon_tags_47 },
    { 0x350835ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x170835af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca835b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x598835bb, SUC_lexicon_tags_25 },
    { 0x7ba835bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be835bf, SUC_lexicon_tags_69 },
    { 0x2d6835c0, SUC_lexicon_tags_10 },
    { 0x88c835c1, SUC_lexicon_tags_38 },
    { 0xaf0835c1, SUC_lexicon_tags_15 },
    { 0xa72835c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x530835c6, SUC_lexicon_tags_25 },
    { 0x9fa835c6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a835ca, SUC_lexicon_tags_11 },
    { 0x886835ca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2835d0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce835d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02835db, SUC_lexicon_tags_25 },
    { 0xc44835db, SUC_lexicon_tags_38 },
    { 0x924835dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450835e2, SUC_lexicon_tags_25 },
    { 0xf80835e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9a0835e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c835e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x182835ea, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x4d6835ec, SUC_lexicon_tags_14 },
    { 0x7ae835ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x978835ef, SUC_lexicon_tags_9 },
    { 0x680835f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2835f3, SUC_lexicon_tags_4 },
    { 0xa08835f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6835fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda6835fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7683605, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59883608, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2668360a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23683610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a83613, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8748361a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8568361c, SUC_lexicon_tags_10 },
    { 0xd908361c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94083622, SUC_lexicon_tags_11 },
    { 0xcb683622, SUC_lexicon_tags_0 },
    { 0x54683624, SUC_lexicon_tags_1 },
    { 0x8f883625, SUC_lexicon_tags_12 },
    { 0x2aa83624, SUC_lexicon_tags_35 },
    { 0x8e883627, SUC_lexicon_tags_2 },
    { 0xc8c83628, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b48362c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5068362e, SUC_lexicon_tags_12 },
    { 0x7f68362e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda83637, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9788363a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce8363f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c83642, SUC_lexicon_tags_37 },
    { 0xb0a83643, SUC_lexicon_tags_11 },
    { 0x97a83642, SUC_lexicon_tags_11 },
    { 0x73883645, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c28364d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7683650, SUC_lexicon_tags_87 },
    { 0xbc83651, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde83655, SUC_lexicon_tags_30 },
    { 0xefc83656, SUC_lexicon_tags_2 },
    { 0x70a83657, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a8365a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c08365e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a83664, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0x20483667, SUC_lexicon_tags_17 },
    { 0x1b283667, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c88366b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e83672, SUC_lexicon_tags_11 },
    { 0xbb283673, SUC_lexicon_tags_12 },
    { 0x6e683674, SUC_lexicon_tags_14 },
    { 0xc4283672, SUC_lexicon_tags_43 },
    { 0x5d083673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57883678, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a08367c, SUC_lexicon_tags_77 },
    { 0x1568367d, SUC_lexicon_tags_7 },
    { 0x62e8367c, SUC_lexicon_tags_104 },
    { 0x75c8367f, SUC_lexicon_tags_116 },
    { 0xb4283680, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2e83682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a483684, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f88368e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x59683691, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbee83693, SUC_lexicon_tags_9 },
    { 0x86a83693, SUC_lexicon_tags_11 },
    { 0x2a483695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19483698, SUC_lexicon_tags_6 },
    { 0x29e83699, SUC_lexicon_tags_11 },
    { 0x3c083699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd108369e, SUC_lexicon_tags_17 },
    { 0x2e88369f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xae4836a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e836a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa836a9, SUC_lexicon_tags_584 },
    { 0, NULL },
    { 0xc7c836ab, SUC_lexicon_tags_11 },
    { 0xd7e836ac, SUC_lexicon_tags_60 },
    { 0xd4e836ab, SUC_lexicon_tags_7 },
    { 0x14c836ab, SUC_lexicon_tags_77 },
    { 0xade836af, SUC_lexicon_tags_26 },
    { 0x33c836b0, SUC_lexicon_tags_37 },
    { 0xb60836ae, SUC_lexicon_tags_60 },
    { 0xfe8836b2, SUC_lexicon_tags_26 },
    { 0xddc836b0, SUC_lexicon_tags_28 },
    { 0x29a836b4, SUC_lexicon_tags_10 },
    { 0x1c2836b5, SUC_lexicon_tags_15 },
    { 0xfac836b4, SUC_lexicon_tags_2 },
    { 0xb00836b7, SUC_lexicon_tags_32 },
    { 0x47a836b7, SUC_lexicon_tags_33 },
    { 0xcaa836b8, SUC_lexicon_tags_88 },
    { 0xf1e836b9, SUC_lexicon_tags_2 },
    { 0xd38836af, SUC_lexicon_tags_2 },
    { 0xe04836ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8836c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196836cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6836d0, SUC_lexicon_tags_66 },
    { 0xee0836d1, SUC_lexicon_tags_1 },
    { 0x26a836d2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9e2836d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x652836d6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc5c836d8, SUC_lexicon_tags_7 },
    { 0x514836d9, SUC_lexicon_tags_15 },
    { 0xa4a836da, SUC_lexicon_tags_1629 },
    { 0xdda836db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28836df, SUC_lexicon_tags_12 },
    { 0xfd2836e0, SUC_lexicon_tags_33 },
    { 0xbca836e1, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0xb20836e3, SUC_lexicon_tags_60 },
    { 0x5d2836e4, SUC_lexicon_tags_38 },
    { 0x158836e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c836e8, SUC_lexicon_tags_1 },
    { 0xdfa836e9, SUC_lexicon_tags_53 },
    { 0x3ce836ea, SUC_lexicon_tags_29 },
    { 0xf20836eb, SUC_lexicon_tags_30 },
    { 0x5f0836eb, SUC_lexicon_tags_46 },
    { 0x43e836ed, SUC_lexicon_tags_12 },
    { 0x880836eb, SUC_lexicon_tags_2 },
    { 0x1a6836ef, SUC_lexicon_tags_34 },
    { 0x5b4836ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88836f4, SUC_lexicon_tags_17 },
    { 0x862836f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e836f9, SUC_lexicon_tags_19 },
    { 0xd44836f9, SUC_lexicon_tags_10 },
    { 0x35a836fb, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xcf4836fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x432836ff, SUC_lexicon_tags_65 },
    { 0x53e836ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4e83702, SUC_lexicon_tags_63 },
    { 0xf4e83703, SUC_lexicon_tags_47 },
    { 0xd6683704, SUC_lexicon_tags_28 },
    { 0xeae83702, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ec83707, SUC_lexicon_tags_12 },
    { 0xd2c83708, SUC_lexicon_tags_26 },
    { 0x5d883707, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x37e8370b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41c8370d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8068370f, SUC_lexicon_tags_12 },
    { 0xef283710, SUC_lexicon_tags_83 },
    { 0xa6683711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8283715, SUC_lexicon_tags_1 },
    { 0x22c83715, SUC_lexicon_tags_7 },
    { 0xfd883717, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xda683719, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88371e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc83723, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x70883725, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38083729, SUC_lexicon_tags_37 },
    { 0xb083729, SUC_lexicon_tags_11 },
    { 0x7388372b, SUC_lexicon_tags_31 },
    { 0xe008372c, SUC_lexicon_tags_10 },
    { 0xc7e8372d, SUC_lexicon_tags_25 },
    { 0x7548372e, SUC_lexicon_tags_7 },
    { 0x2448372d, SUC_lexicon_tags_25 },
    { 0x9448372d, SUC_lexicon_tags_15 },
    { 0xc788372c, SUC_lexicon_tags_8 },
    { 0x12c8372c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60683735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a483738, SUC_lexicon_tags_30 },
    { 0x33c83739, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe8373c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2e8373e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc6483740, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e83745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf083749, SUC_lexicon_tags_1 },
    { 0x9dc83749, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e48374f, SUC_lexicon_tags_7 },
    { 0x53483750, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x68283753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e083756, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e8375a, SUC_lexicon_tags_17 },
    { 0x5608375a, SUC_lexicon_tags_26 },
    { 0x2da8375c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf483761, SUC_lexicon_tags_30 },
    { 0xf2483762, SUC_lexicon_tags_9 },
    { 0x73c83763, SUC_lexicon_tags_15 },
    { 0xf8e83764, SUC_lexicon_tags_23 },
    { 0xd1483765, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e8376a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0e8376c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x77a8376e, SUC_lexicon_tags_0 },
    { 0x89c8376f, SUC_lexicon_tags_1 },
    { 0x81883770, SUC_lexicon_tags_12 },
    { 0xf9c83771, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6083774, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xace83776, SUC_lexicon_tags_10 },
    { 0x6b883777, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddc83779, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x308377b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa8377f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd683781, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc283788, SUC_lexicon_tags_17 },
    { 0xd6683789, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56c8378b, SUC_lexicon_tags_12 },
    { 0x5a88378b, SUC_lexicon_tags_85 },
    { 0xf648378d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c83790, SUC_lexicon_tags_27 },
    { 0x97a83790, SUC_lexicon_tags_45 },
    { 0x97c83790, SUC_lexicon_tags_12 },
    { 0x24883792, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f483796, SUC_lexicon_tags_12 },
    { 0x94e83796, SUC_lexicon_tags_11 },
    { 0xdb483798, SUC_lexicon_tags_10 },
    { 0xe5483799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2837a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x560837a3, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc837ab, SUC_lexicon_tags_87 },
    { 0xfb2837ac, SUC_lexicon_tags_6 },
    { 0x34a837ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bc837af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x194837b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a837bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e6837be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc6837c0, SUC_lexicon_tags_17 },
    { 0xb42837c0, SUC_lexicon_tags_1 },
    { 0xebc837c2, SUC_lexicon_tags_37 },
    { 0xeb6837c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65c837c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0e837c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67e837c9, SUC_lexicon_tags_30 },
    { 0x80a837ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c837cd, SUC_lexicon_tags_15 },
    { 0x1f0837cd, SUC_lexicon_tags_25 },
    { 0x10a837cd, SUC_lexicon_tags_61 },
    { 0x1b2837d0, SUC_lexicon_tags_4 },
    { 0xb04837d1, SUC_lexicon_tags_14 },
    { 0xeb0837d0, SUC_lexicon_tags_2 },
    { 0xc22837d3, SUC_lexicon_tags_37 },
    { 0x488837d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d8837d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6837dc, SUC_lexicon_tags_17 },
    { 0xdbe837dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf90837df, SUC_lexicon_tags_25 },
    { 0x38e837e0, SUC_lexicon_tags_17 },
    { 0xe38837e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x456837e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4837e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a837eb, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0x106837ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x388837ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98837f3, SUC_lexicon_tags_2 },
    { 0xd12837f3, SUC_lexicon_tags_14 },
    { 0xfae837f5, SUC_lexicon_tags_14 },
    { 0xd00837f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be837fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff8837fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d483802, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec83806, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6c83808, SUC_lexicon_tags_13 },
    { 0xcec83809, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb108380e, SUC_lexicon_tags_29 },
    { 0x1c8380f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea83813, SUC_lexicon_tags_17 },
    { 0x44a83814, SUC_lexicon_tags_2 },
    { 0xd7683813, SUC_lexicon_tags_47 },
    { 0x47083816, SUC_lexicon_tags_14 },
    { 0x64e83816, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e283819, SUC_lexicon_tags_65 },
    { 0xe928381a, SUC_lexicon_tags_37 },
    { 0xcf28381a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb708381f, SUC_lexicon_tags_31 },
    { 0x12a83820, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1083824, SUC_lexicon_tags_45 },
    { 0x46883825, SUC_lexicon_tags_11 },
    { 0xad483824, SUC_lexicon_tags_0 },
    { 0xa1083824, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf08382c, SUC_lexicon_tags_36 },
    { 0xbe28382d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3508382f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a83832, SUC_lexicon_tags_68 },
    { 0x51283833, SUC_lexicon_tags_49 },
    { 0x39683832, SUC_lexicon_tags_47 },
    { 0x35e83835, SUC_lexicon_tags_11 },
    { 0x32283836, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x16e83838, SUC_lexicon_tags_14 },
    { 0xfee83839, SUC_lexicon_tags_87 },
    { 0xa9c8383a, SUC_lexicon_tags_60 },
    { 0x70a8383a, SUC_lexicon_tags_7 },
    { 0xe648383c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c08383e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x39483840, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x40683843, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e83848, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1388384a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5268384c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8a8384e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff883850, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99483854, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc83859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf988385b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd308385e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62c83860, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc0683862, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb483867, SUC_lexicon_tags_19 },
    { 0xbb083868, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9328386e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e483874, SUC_lexicon_tags_33 },
    { 0xe4e83875, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c083881, SUC_lexicon_tags_23 },
    { 0xa4483882, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89083886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75a83888, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c8388d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52283890, SUC_lexicon_tags_68 },
    { 0x32283891, SUC_lexicon_tags_45 },
    { 0x8aa83891, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe83896, SUC_lexicon_tags_65 },
    { 0xbc883897, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f48389c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326838a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x998838a3, SUC_lexicon_tags_38 },
    { 0xfe2838a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248838ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe44838ad, SUC_lexicon_tags_38 },
    { 0x8d2838ad, SUC_lexicon_tags_27 },
    { 0xe18838af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484838b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe96838b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x63e838b7, SUC_lexicon_tags_43 },
    { 0xa1c838b8, SUC_lexicon_tags_12 },
    { 0xafc838b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x132838bb, SUC_lexicon_tags_37 },
    { 0x4d4838bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ea838be, SUC_lexicon_tags_37 },
    { 0xe78838bf, SUC_lexicon_tags_1 },
    { 0xd80838bf, SUC_lexicon_tags_87 },
    { 0x8c4838c0, SUC_lexicon_tags_77 },
    { 0xff0838c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c838c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680838c9, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326838cd, SUC_lexicon_tags_7 },
    { 0xf7a838ce, SUC_lexicon_tags_11 },
    { 0x5e0838cf, SUC_lexicon_tags_96 },
    { 0xd7c838d0, SUC_lexicon_tags_7 },
    { 0xdb8838d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc838d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a838d7, SUC_lexicon_tags_37 },
    { 0x616838d8, SUC_lexicon_tags_14 },
    { 0xd1c838d9, SUC_lexicon_tags_11 },
    { 0x318838d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee838df, SUC_lexicon_tags_84 },
    { 0x796838e0, SUC_lexicon_tags_13 },
    { 0xc5a838e1, SUC_lexicon_tags_43 },
    { 0x7c0838e2, SUC_lexicon_tags_15 },
    { 0x282838e3, SUC_lexicon_tags_1 },
    { 0x3f6838e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4838e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x748838e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6838ed, SUC_lexicon_tags_12 },
    { 0x8d8838ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac838f2, SUC_lexicon_tags_11 },
    { 0x7ea838f3, SUC_lexicon_tags_23 },
    { 0xb28838f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac838fd, SUC_lexicon_tags_7 },
    { 0xeae838fe, SUC_lexicon_tags_61 },
    { 0x5bc838ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87083901, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e83904, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe988390c, SUC_lexicon_tags_3 },
    { 0x9288390c, SUC_lexicon_tags_117 },
    { 0x3688390c, SUC_lexicon_tags_33 },
    { 0xe368390d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc5883911, SUC_lexicon_tags_10 },
    { 0x39483912, SUC_lexicon_tags_13 },
    { 0x62483913, SUC_lexicon_tags_37 },
    { 0xd3483913, SUC_lexicon_tags_12 },
    { 0x45c83914, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a83918, SUC_lexicon_tags_39 },
    { 0x36083918, SUC_lexicon_tags_48 },
    { 0x88c8391a, SUC_lexicon_tags_25 },
    { 0xccc8391a, SUC_lexicon_tags_11 },
    { 0xa7283919, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80683924, SUC_lexicon_tags_3 },
    { 0x6ea83925, SUC_lexicon_tags_7 },
    { 0x6b883926, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4248392a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2008392e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2a483930, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac83932, SUC_lexicon_tags_28 },
    { 0xde083933, SUC_lexicon_tags_15 },
    { 0xe5083934, SUC_lexicon_tags_2 },
    { 0x73c83935, SUC_lexicon_tags_7 },
    { 0xde483936, SUC_lexicon_tags_19 },
    { 0x74a83933, SUC_lexicon_tags_11 },
    { 0xeaa83938, SUC_lexicon_tags_1 },
    { 0x5083939, SUC_lexicon_tags_7 },
    { 0x9c8393a, SUC_lexicon_tags_21 },
    { 0x2dc83937, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b88393f, SUC_lexicon_tags_30 },
    { 0xc4e83940, SUC_lexicon_tags_2 },
    { 0xf148393f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e83944, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52083946, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d48394a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c8394d, SUC_lexicon_tags_85 },
    { 0x728394e, SUC_lexicon_tags_49 },
    { 0xf068394f, SUC_lexicon_tags_37 },
    { 0xc0483950, SUC_lexicon_tags_1 },
    { 0x8e283951, SUC_lexicon_tags_14 },
    { 0x4da83952, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6283955, SUC_lexicon_tags_29 },
    { 0xc5c83955, SUC_lexicon_tags_2 },
    { 0x8d283957, SUC_lexicon_tags_15 },
    { 0x1e483958, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8868395c, SUC_lexicon_tags_65 },
    { 0xf048395d, SUC_lexicon_tags_15 },
    { 0xcc28395e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd683960, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb7283962, SUC_lexicon_tags_37 },
    { 0x8ee83963, SUC_lexicon_tags_11 },
    { 0xc3483963, SUC_lexicon_tags_26 },
    { 0x8bc83965, SUC_lexicon_tags_129 },
    { 0x9d283966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1883968, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6068396a, SUC_lexicon_tags_49 },
    { 0xddc8396a, SUC_lexicon_tags_70 },
    { 0x2f08396a, SUC_lexicon_tags_5 },
    { 0x9548396b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e83973, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4283975, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa1283977, SUC_lexicon_tags_47 },
    { 0xd9083977, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1ae8397a, SUC_lexicon_tags_12 },
    { 0x57c8397b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf668397e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e83984, SUC_lexicon_tags_14 },
    { 0xe8483985, SUC_lexicon_tags_19 },
    { 0x45083985, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x89a83989, SUC_lexicon_tags_10 },
    { 0x2468398a, SUC_lexicon_tags_8 },
    { 0x59e83989, SUC_lexicon_tags_11 },
    { 0xbe48398c, SUC_lexicon_tags_66 },
    { 0x3828398d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4508398f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xac083992, SUC_lexicon_tags_6 },
    { 0xf5483993, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b683998, SUC_lexicon_tags_25 },
    { 0x62a83999, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc839a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716839a7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe36839a9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x378839ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdc8839ad, SUC_lexicon_tags_15 },
    { 0xf80839ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xebe839b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x638839b3, SUC_lexicon_tags_166 },
    { 0, NULL },
    { 0xd1e839b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16839bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46839be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6839c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464839c7, SUC_lexicon_tags_25 },
    { 0x388839c8, SUC_lexicon_tags_14 },
    { 0xab6839c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee8839cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa20839cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c839d0, SUC_lexicon_tags_7 },
    { 0xea4839d1, SUC_lexicon_tags_29 },
    { 0x51c839d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x762839d5, SUC_lexicon_tags_25 },
    { 0x36a839d6, SUC_lexicon_tags_25 },
    { 0x928839d6, SUC_lexicon_tags_14 },
    { 0x852839d8, SUC_lexicon_tags_9 },
    { 0xf1c839d8, SUC_lexicon_tags_48 },
    { 0x852839d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf8e839dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6839df, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c839e5, SUC_lexicon_tags_37 },
    { 0x642839e6, SUC_lexicon_tags_1 },
    { 0x97a839e7, SUC_lexicon_tags_37 },
    { 0x8a4839e6, SUC_lexicon_tags_10 },
    { 0xb4839e9, SUC_lexicon_tags_34 },
    { 0x4dc839e8, SUC_lexicon_tags_1 },
    { 0x346839e5, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790839f1, SUC_lexicon_tags_83 },
    { 0x402839f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x740839f4, SUC_lexicon_tags_7 },
    { 0xa18839f5, SUC_lexicon_tags_7 },
    { 0xce2839f6, SUC_lexicon_tags_15 },
    { 0x21e839f6, SUC_lexicon_tags_31 },
    { 0x6f4839f8, SUC_lexicon_tags_37 },
    { 0x898839f6, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a83a00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e883a02, SUC_lexicon_tags_38 },
    { 0xf8083a03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e83a07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc83a0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1483a12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5083a19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3683a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d283a1e, SUC_lexicon_tags_25 },
    { 0x1f683a1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x20283a21, SUC_lexicon_tags_26 },
    { 0xb1c83a21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f883a25, SUC_lexicon_tags_14 },
    { 0xd4e83a26, SUC_lexicon_tags_3 },
    { 0xbf683a27, SUC_lexicon_tags_43 },
    { 0x25683a27, SUC_lexicon_tags_378 },
    { 0xb8283a25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93083a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64683a40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2683a43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c83a46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c483a4c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x79083a4e, SUC_lexicon_tags_1 },
    { 0x74083a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8083a51, SUC_lexicon_tags_27 },
    { 0x53a83a52, SUC_lexicon_tags_2 },
    { 0xaee83a52, SUC_lexicon_tags_14 },
    { 0xe8283a54, SUC_lexicon_tags_32 },
    { 0xca683a52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9d683a57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51c83a59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4083a5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb083a5e, SUC_lexicon_tags_29 },
    { 0xd1883a5f, SUC_lexicon_tags_275 },
    { 0x6f683a5f, SUC_lexicon_tags_63 },
    { 0x3dc83a5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da83a64, SUC_lexicon_tags_15 },
    { 0xffa83a64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4683a6c, SUC_lexicon_tags_7 },
    { 0x58083a6c, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c83a75, SUC_lexicon_tags_26 },
    { 0xc8083a76, SUC_lexicon_tags_282 },
    { 0x76283a77, SUC_lexicon_tags_6 },
    { 0xde883a76, SUC_lexicon_tags_25 },
    { 0xd4a83a78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbce83a7b, SUC_lexicon_tags_2 },
    { 0x5ea83a7b, SUC_lexicon_tags_19 },
    { 0x45683a7c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x56083a80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0a83a82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba83a85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e83a8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c83a90, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x46083a93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e483a99, SUC_lexicon_tags_15 },
    { 0x18c83a9a, SUC_lexicon_tags_33 },
    { 0x59c83a9b, SUC_lexicon_tags_15 },
    { 0x9e283a9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe83a9f, SUC_lexicon_tags_31 },
    { 0x21e83aa0, SUC_lexicon_tags_17 },
    { 0x4cc83aa1, SUC_lexicon_tags_15 },
    { 0xf3883a9f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff283aa6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x61c83aa8, SUC_lexicon_tags_43 },
    { 0x23e83aa9, SUC_lexicon_tags_15 },
    { 0x1c883aaa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae83aaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61683ab3, SUC_lexicon_tags_12 },
    { 0x10c83ab3, SUC_lexicon_tags_37 },
    { 0x25e83ab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c83ab8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb083ac0, SUC_lexicon_tags_11 },
    { 0x99683ac1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c83ac4, SUC_lexicon_tags_12 },
    { 0xad083ac5, SUC_lexicon_tags_70 },
    { 0x8d083ac6, SUC_lexicon_tags_2 },
    { 0x50883ac7, SUC_lexicon_tags_11 },
    { 0x6ce83ac7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e883acc, SUC_lexicon_tags_11 },
    { 0x34e83acd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a83ad1, SUC_lexicon_tags_47 },
    { 0xe4683ad1, SUC_lexicon_tags_25 },
    { 0x52483ad3, SUC_lexicon_tags_43 },
    { 0xd4083ad1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafa83ad6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x39e83ad8, SUC_lexicon_tags_27 },
    { 0xde083ad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe883ade, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x33883ae0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xabe83ae2, SUC_lexicon_tags_25 },
    { 0x49a83ae3, SUC_lexicon_tags_11 },
    { 0x4ae83ae4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdba83ae6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65283aea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25483aef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e83af7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88283af9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e83afd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7283b07, SUC_lexicon_tags_47 },
    { 0x51a83b07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90483b0a, SUC_lexicon_tags_19 },
    { 0x68683b0a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x40483b0e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56083b10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38283b16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3f683b18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea83b1c, SUC_lexicon_tags_9 },
    { 0x5fa83b1d, SUC_lexicon_tags_19 },
    { 0xa683b1d, SUC_lexicon_tags_17 },
    { 0xbd883b1f, SUC_lexicon_tags_93 },
    { 0x63683b20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc083b26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9083b28, SUC_lexicon_tags_15 },
    { 0x50083b29, SUC_lexicon_tags_17 },
    { 0xb4c83b2a, SUC_lexicon_tags_12 },
    { 0x7a683b2a, SUC_lexicon_tags_7 },
    { 0x3f083b2c, SUC_lexicon_tags_9 },
    { 0x4bc83b2d, SUC_lexicon_tags_11 },
    { 0x61c83b2d, SUC_lexicon_tags_25 },
    { 0x6ec83b2f, SUC_lexicon_tags_25 },
    { 0x1ea83b2f, SUC_lexicon_tags_7 },
    { 0xe3883b31, SUC_lexicon_tags_37 },
    { 0x8d283b30, SUC_lexicon_tags_1 },
    { 0x2fa83b33, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5683b36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74283b3f, SUC_lexicon_tags_14 },
    { 0x1b683b3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfd683b42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x17083b44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91883b46, SUC_lexicon_tags_2 },
    { 0xad883b47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c83b4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0683b4c, SUC_lexicon_tags_28 },
    { 0xccc83b4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x59683b4f, SUC_lexicon_tags_15 },
    { 0xf7c83b50, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e83b57, SUC_lexicon_tags_49 },
    { 0x40483b58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc683b5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc83b60, SUC_lexicon_tags_12 },
    { 0x64c83b61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x72e83b63, SUC_lexicon_tags_60 },
    { 0x61e83b63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e83b71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84283b78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a083b7a, SUC_lexicon_tags_13 },
    { 0x6d083b7a, SUC_lexicon_tags_23 },
    { 0x9c83b7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebc83b7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f883b84, SUC_lexicon_tags_12 },
    { 0xb1883b85, SUC_lexicon_tags_21 },
    { 0xfb883b86, SUC_lexicon_tags_30 },
    { 0xfd083b85, SUC_lexicon_tags_45 },
    { 0x10083b87, SUC_lexicon_tags_17 },
    { 0xcfc83b84, SUC_lexicon_tags_37 },
    { 0xba483b86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33e83b8c, SUC_lexicon_tags_38 },
    { 0xea883b8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x81683b8f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e83b92, SUC_lexicon_tags_9 },
    { 0x8a683b93, SUC_lexicon_tags_11 },
    { 0xb2083b94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98283ba3, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0xc3083ba5, SUC_lexicon_tags_48 },
    { 0x44283ba6, SUC_lexicon_tags_25 },
    { 0x7e283ba6, SUC_lexicon_tags_113 },
    { 0x3d083ba5, SUC_lexicon_tags_11 },
    { 0x98c83ba8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc3083bab, SUC_lexicon_tags_17 },
    { 0xb683bac, SUC_lexicon_tags_11 },
    { 0x10a83bab, SUC_lexicon_tags_37 },
    { 0xffa83bab, SUC_lexicon_tags_17 },
    { 0xd0c83baf, SUC_lexicon_tags_45 },
    { 0x3d683bab, SUC_lexicon_tags_11 },
    { 0xb5c83bb1, SUC_lexicon_tags_28 },
    { 0x47c83bac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a83bb9, SUC_lexicon_tags_25 },
    { 0xf7883bba, SUC_lexicon_tags_15 },
    { 0x37e83bbb, SUC_lexicon_tags_17 },
    { 0x5b483bbb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e283bc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7883bc9, SUC_lexicon_tags_23 },
    { 0xf4083bc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e83bcd, SUC_lexicon_tags_65 },
    { 0x40a83bcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0283bd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a883bd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x48083bd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41a83bda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9283bdd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78283be2, SUC_lexicon_tags_17 },
    { 0x42a83be2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba483be6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x36283be8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb0083bea, SUC_lexicon_tags_9 },
    { 0x5b883beb, SUC_lexicon_tags_2 },
    { 0x2fc83beb, SUC_lexicon_tags_25 },
    { 0xa5683bed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c83bef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a83bf2, SUC_lexicon_tags_26 },
    { 0x98883bf3, SUC_lexicon_tags_7 },
    { 0xd5a83bf4, SUC_lexicon_tags_9 },
    { 0x1ac83bf3, SUC_lexicon_tags_12 },
    { 0xd7883bf3, SUC_lexicon_tags_15 },
    { 0x6e883bf3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c83bfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12883c05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb683c0a, SUC_lexicon_tags_27 },
    { 0x69e83c0b, SUC_lexicon_tags_17 },
    { 0x4ba83c0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a883c10, SUC_lexicon_tags_27 },
    { 0xb0683c11, SUC_lexicon_tags_5 },
    { 0x6e683c11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6283c18, SUC_lexicon_tags_11 },
    { 0x8a683c19, SUC_lexicon_tags_17 },
    { 0x63e83c1a, SUC_lexicon_tags_70 },
    { 0x7283c1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7883c1e, SUC_lexicon_tags_17 },
    { 0xb7083c1f, SUC_lexicon_tags_23 },
    { 0xdd283c20, SUC_lexicon_tags_55 },
    { 0xd4e83c21, SUC_lexicon_tags_238 },
    { 0x88483c1e, SUC_lexicon_tags_25 },
    { 0x21483c23, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcf883c25, SUC_lexicon_tags_12 },
    { 0x28883c26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4883c2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e83c2d, SUC_lexicon_tags_12 },
    { 0xf1c83c2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9283c30, SUC_lexicon_tags_12 },
    { 0x61483c31, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9483c35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41683c38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa883c3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55c83c41, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7883c44, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa8683c46, SUC_lexicon_tags_33 },
    { 0x5b283c47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e83c4b, SUC_lexicon_tags_6 },
    { 0xbb883c4b, SUC_lexicon_tags_6 },
    { 0x5dc83c4d, SUC_lexicon_tags_28 },
    { 0x36083c4e, SUC_lexicon_tags_4 },
    { 0xd6a83c4f, SUC_lexicon_tags_49 },
    { 0x2a883c50, SUC_lexicon_tags_33 },
    { 0x3b883c50, SUC_lexicon_tags_46 },
    { 0xc2c83c50, SUC_lexicon_tags_25 },
    { 0xfec83c4f, SUC_lexicon_tags_7 },
    { 0x2fe83c54, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9d683c56, SUC_lexicon_tags_19 },
    { 0xfce83c56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb483c5a, SUC_lexicon_tags_7 },
    { 0x93483c5b, SUC_lexicon_tags_25 },
    { 0x4e083c5b, SUC_lexicon_tags_7 },
    { 0xef483c5b, SUC_lexicon_tags_11 },
    { 0x67483c5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12483c61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c283c66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a83c68, SUC_lexicon_tags_17 },
    { 0x31883c69, SUC_lexicon_tags_45 },
    { 0x22c83c69, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b883c6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20283c73, SUC_lexicon_tags_17 },
    { 0x83283c74, SUC_lexicon_tags_56 },
    { 0xc4483c74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a83c78, SUC_lexicon_tags_2 },
    { 0xba483c79, SUC_lexicon_tags_15 },
    { 0x10083c78, SUC_lexicon_tags_12 },
    { 0xdc083c78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda883c7f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3f283c81, SUC_lexicon_tags_25 },
    { 0x20a83c82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a83c88, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x75e83c8a, SUC_lexicon_tags_11 },
    { 0x6e683c8b, SUC_lexicon_tags_14 },
    { 0xac083c8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0483c92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf883c96, SUC_lexicon_tags_17 },
    { 0x1be83c97, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa7483c99, SUC_lexicon_tags_12 },
    { 0x8ce83c9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3083ca2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6083ca5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe483cab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x69883cae, SUC_lexicon_tags_37 },
    { 0xfea83caf, SUC_lexicon_tags_7 },
    { 0xbb483caf, SUC_lexicon_tags_43 },
    { 0x90683caf, SUC_lexicon_tags_46 },
    { 0x45c83cb1, SUC_lexicon_tags_45 },
    { 0x2883cb2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f683cb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0083cb8, SUC_lexicon_tags_23 },
    { 0xb5883cb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdda83cbb, SUC_lexicon_tags_43 },
    { 0x3d683cbc, SUC_lexicon_tags_12 },
    { 0xcfa83cbb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa5a83cbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52483cc1, SUC_lexicon_tags_37 },
    { 0x9b883cc1, SUC_lexicon_tags_15 },
    { 0x70e83cc3, SUC_lexicon_tags_33 },
    { 0x1a483cc3, SUC_lexicon_tags_27 },
    { 0xa9283cc5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c83ccf, SUC_lexicon_tags_10 },
    { 0xc8883cd0, SUC_lexicon_tags_33 },
    { 0xc2683cd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xde883cd4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29483cda, SUC_lexicon_tags_1 },
    { 0x7ce83cda, SUC_lexicon_tags_11 },
    { 0xfc083cdc, SUC_lexicon_tags_1 },
    { 0xafc83cdd, SUC_lexicon_tags_17 },
    { 0x1da83cda, SUC_lexicon_tags_65 },
    { 0xf1683cdf, SUC_lexicon_tags_25 },
    { 0x54c83cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ca83ce2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa83ce5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x91283ce8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31883cec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee83cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d283cfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93483cfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c83d01, SUC_lexicon_tags_11 },
    { 0x1e483d01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe83d05, SUC_lexicon_tags_26 },
    { 0x1c83d05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85883d09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31883d0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x13883d11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a83d15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b083d1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb0483d1d, SUC_lexicon_tags_4 },
    { 0x35083d1e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x89083d20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3683d22, SUC_lexicon_tags_64 },
    { 0x2ee83d23, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a83d28, SUC_lexicon_tags_56 },
    { 0x88e83d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6083d2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xccc83d30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0483d32, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3a683d34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e83d3b, SUC_lexicon_tags_27 },
    { 0x68083d3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcb683d3e, SUC_lexicon_tags_28 },
    { 0x65083d3f, SUC_lexicon_tags_37 },
    { 0xda683d3f, SUC_lexicon_tags_49 },
    { 0xa4483d40, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda83d47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65083d4c, SUC_lexicon_tags_9 },
    { 0xb2283d4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96483d52, SUC_lexicon_tags_30 },
    { 0x4b683d53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4283d55, SUC_lexicon_tags_37 },
    { 0x14c83d56, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x73c83d58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf8283d5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2e83d5c, SUC_lexicon_tags_37 },
    { 0xc3e83d5c, SUC_lexicon_tags_37 },
    { 0x1c483d5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec83d62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae83d65, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1e83d67, SUC_lexicon_tags_25 },
    { 0x83e83d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51483d6f, SUC_lexicon_tags_71 },
    { 0x6c483d70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b083d74, SUC_lexicon_tags_29 },
    { 0x683d75, SUC_lexicon_tags_12 },
    { 0xda83d76, SUC_lexicon_tags_17 },
    { 0xec483d77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc883d7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0883d7d, SUC_lexicon_tags_7 },
    { 0x6e483d7e, SUC_lexicon_tags_25 },
    { 0x73a83d7f, SUC_lexicon_tags_38 },
    { 0x3c883d80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3683d85, SUC_lexicon_tags_1 },
    { 0x92c83d86, SUC_lexicon_tags_25 },
    { 0x4aa83d87, SUC_lexicon_tags_2 },
    { 0x2b083d87, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb8c83d8a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10683d8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f683d95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35883da2, SUC_lexicon_tags_19 },
    { 0xe683da2, SUC_lexicon_tags_12 },
    { 0x6f083da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e683dab, SUC_lexicon_tags_31 },
    { 0x89a83dac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4a483dae, SUC_lexicon_tags_34 },
    { 0x7c083daf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e83dbf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6d483dc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c083dc3, SUC_lexicon_tags_25 },
    { 0x1283dc4, SUC_lexicon_tags_6 },
    { 0xd8e83dc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc83dc8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5283dcd, SUC_lexicon_tags_33 },
    { 0xc5683dcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ae83dd0, SUC_lexicon_tags_7 },
    { 0x65a83dd1, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97083dd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60883dde, SUC_lexicon_tags_49 },
    { 0x29283dde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec83de2, SUC_lexicon_tags_25 },
    { 0x70883de3, SUC_lexicon_tags_48 },
    { 0xbd083de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20083de7, SUC_lexicon_tags_6 },
    { 0x15883de8, SUC_lexicon_tags_12 },
    { 0x17e83de7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64683dec, SUC_lexicon_tags_2 },
    { 0x44283ded, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc83df1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbfa83df3, SUC_lexicon_tags_17 },
    { 0xafc83df3, SUC_lexicon_tags_17 },
    { 0x14c83df5, SUC_lexicon_tags_32 },
    { 0xc7283df6, SUC_lexicon_tags_17 },
    { 0xaa83df5, SUC_lexicon_tags_38 },
    { 0xf6a83df8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19083e02, SUC_lexicon_tags_23 },
    { 0x21a83e02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe683e07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf683e09, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4bc83e0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea83e0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0a83e11, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd7283e13, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6683e17, SUC_lexicon_tags_63 },
    { 0xc8283e17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f083e1f, SUC_lexicon_tags_77 },
    { 0x13a83e20, SUC_lexicon_tags_0 },
    { 0x27883e21, SUC_lexicon_tags_28 },
    { 0x68c83e22, SUC_lexicon_tags_31 },
    { 0xb2c83e23, SUC_lexicon_tags_2 },
    { 0x5c883e24, SUC_lexicon_tags_2 },
    { 0xbfc83e25, SUC_lexicon_tags_23 },
    { 0xa1683e26, SUC_lexicon_tags_1 },
    { 0xb6a83e27, SUC_lexicon_tags_11 },
    { 0xa1c83e28, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0483e2e, SUC_lexicon_tags_23 },
    { 0xbbe83e2f, SUC_lexicon_tags_12 },
    { 0xc0883e30, SUC_lexicon_tags_60 },
    { 0x62683e31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd283e36, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x61483e39, SUC_lexicon_tags_7 },
    { 0xb9e83e39, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba683e3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1e83e3e, SUC_lexicon_tags_30 },
    { 0xa2283e3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa83e42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c83e45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b083e47, SUC_lexicon_tags_33 },
    { 0xef083e47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a83e4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e683e54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26883e56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc7883e58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd883e5b, SUC_lexicon_tags_11 },
    { 0xee483e5b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3e483e5e, SUC_lexicon_tags_2 },
    { 0x5e683e5e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8083e67, SUC_lexicon_tags_12 },
    { 0xaf483e68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab283e6c, SUC_lexicon_tags_15 },
    { 0x61283e6c, SUC_lexicon_tags_11 },
    { 0x6e83e6e, SUC_lexicon_tags_23 },
    { 0x1683e6f, SUC_lexicon_tags_2 },
    { 0x1ac83e70, SUC_lexicon_tags_14 },
    { 0x5c283e6f, SUC_lexicon_tags_17 },
    { 0x25e83e71, SUC_lexicon_tags_1 },
    { 0xae883e73, SUC_lexicon_tags_69 },
    { 0xb3283e73, SUC_lexicon_tags_26 },
    { 0xbd083e73, SUC_lexicon_tags_17 },
    { 0xdee83e76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b083e78, SUC_lexicon_tags_37 },
    { 0xc8883e78, SUC_lexicon_tags_21 },
    { 0x19e83e7a, SUC_lexicon_tags_7 },
    { 0x67483e7a, SUC_lexicon_tags_33 },
    { 0x6fc83e79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77683e83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a83e88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc83e8c, SUC_lexicon_tags_37 },
    { 0x32283e8d, SUC_lexicon_tags_7 },
    { 0x51883e8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32683e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ea83e92, SUC_lexicon_tags_17 },
    { 0x1ca83e93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9683e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57883e9e, SUC_lexicon_tags_43 },
    { 0x4883e9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b883ea2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd683ea5, SUC_lexicon_tags_107 },
    { 0xa0483ea6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a483ead, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c83eb2, SUC_lexicon_tags_60 },
    { 0x37483eb2, SUC_lexicon_tags_38 },
    { 0xff483eb3, SUC_lexicon_tags_12 },
    { 0xf3c83eb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21483ebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c283ec4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4083eca, SUC_lexicon_tags_19 },
    { 0x88883ecb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f083ece, SUC_lexicon_tags_1003 },
    { 0x13483ecf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28883ed4, SUC_lexicon_tags_37 },
    { 0xe683ed4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc83edc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e83ee0, SUC_lexicon_tags_8 },
    { 0x44c83ee0, SUC_lexicon_tags_31 },
    { 0x6de83ee2, SUC_lexicon_tags_60 },
    { 0x1ee83ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x283ee5, SUC_lexicon_tags_37 },
    { 0xb2883ee6, SUC_lexicon_tags_17 },
    { 0x6e083ee7, SUC_lexicon_tags_47 },
    { 0x1c83ee8, SUC_lexicon_tags_25 },
    { 0x68883ee7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa83eed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c83ef0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4283ef3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10483ef5, SUC_lexicon_tags_15 },
    { 0x48083ef5, SUC_lexicon_tags_17 },
    { 0x60a83ef7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e83efb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8283efe, SUC_lexicon_tags_33 },
    { 0x8d683eff, SUC_lexicon_tags_27 },
    { 0x9ca83f00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b883f08, SUC_lexicon_tags_37 },
    { 0xd3683f09, SUC_lexicon_tags_12 },
    { 0x8be83f09, SUC_lexicon_tags_13 },
    { 0x35c83f0b, SUC_lexicon_tags_7 },
    { 0xfc483f0c, SUC_lexicon_tags_84 },
    { 0xbac83f09, SUC_lexicon_tags_66 },
    { 0x97683f0d, SUC_lexicon_tags_37 },
    { 0xd9a83f0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3883f12, SUC_lexicon_tags_46 },
    { 0xcb483f12, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd083f16, SUC_lexicon_tags_12 },
    { 0x5c83f16, SUC_lexicon_tags_23 },
    { 0x88683f18, SUC_lexicon_tags_25 },
    { 0xfc83f19, SUC_lexicon_tags_0 },
    { 0xad483f1a, SUC_lexicon_tags_45 },
    { 0x92083f1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95883f20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a83f25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61e83f27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9a083f29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88c83f2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a83f30, SUC_lexicon_tags_14 },
    { 0xd5283f30, SUC_lexicon_tags_12 },
    { 0x85483f32, SUC_lexicon_tags_23 },
    { 0xc8283f33, SUC_lexicon_tags_7 },
    { 0xd0e83f31, SUC_lexicon_tags_47 },
    { 0x89e83f35, SUC_lexicon_tags_17 },
    { 0xf5883f36, SUC_lexicon_tags_11 },
    { 0xa8c83f35, SUC_lexicon_tags_63 },
    { 0x69c83f37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87283f3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9283f3d, SUC_lexicon_tags_11 },
    { 0xf8a83f3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3683f41, SUC_lexicon_tags_48 },
    { 0x84883f41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf283f44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0083f47, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdec83f49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28483f50, SUC_lexicon_tags_15 },
    { 0x4f883f50, SUC_lexicon_tags_23 },
    { 0x12883f52, SUC_lexicon_tags_37 },
    { 0x4ce83f52, SUC_lexicon_tags_28 },
    { 0x1e283f54, SUC_lexicon_tags_82 },
    { 0x94483f54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61083f59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c83f5c, SUC_lexicon_tags_27 },
    { 0x37c83f5d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x73e83f5f, SUC_lexicon_tags_11 },
    { 0x45083f60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95483f63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76683f65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc83f6c, SUC_lexicon_tags_47 },
    { 0x6c683f6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc83f74, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada83f78, SUC_lexicon_tags_30 },
    { 0xfe083f79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a83f7c, SUC_lexicon_tags_43 },
    { 0x77683f7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xada83f7f, SUC_lexicon_tags_45 },
    { 0x1d083f7f, SUC_lexicon_tags_23 },
    { 0x3083f7f, SUC_lexicon_tags_1 },
    { 0xd8883f82, SUC_lexicon_tags_7 },
    { 0xe2883f83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94683f86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed283f8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb283f8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca883f93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51683f96, SUC_lexicon_tags_17 },
    { 0x56083f97, SUC_lexicon_tags_0 },
    { 0xea83f97, SUC_lexicon_tags_31 },
    { 0xa7a83f99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c83f9c, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a83fa0, SUC_lexicon_tags_2 },
    { 0x8d683fa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc83fa4, SUC_lexicon_tags_12 },
    { 0x8ce83fa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1083fa7, SUC_lexicon_tags_10 },
    { 0x54a83fa8, SUC_lexicon_tags_17 },
    { 0x68c83fa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63883fac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2083faf, SUC_lexicon_tags_47 },
    { 0xe9e83fb0, SUC_lexicon_tags_12 },
    { 0x10083fb0, SUC_lexicon_tags_31 },
    { 0xb4083fb2, SUC_lexicon_tags_15 },
    { 0xcf483fb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c83fb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x14083fbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x34283fbd, SUC_lexicon_tags_37 },
    { 0x45083fbe, SUC_lexicon_tags_12 },
    { 0xcf283fbf, SUC_lexicon_tags_11 },
    { 0x76c83fbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe83fc7, SUC_lexicon_tags_6 },
    { 0x49a83fc7, SUC_lexicon_tags_31 },
    { 0x33683fc9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe9683fcb, SUC_lexicon_tags_2 },
    { 0x4c283fcb, SUC_lexicon_tags_47 },
    { 0xcb283fcd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc83fd0, SUC_lexicon_tags_20 },
    { 0x1ca83fd1, SUC_lexicon_tags_960 },
    { 0x8b483fd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3e83fd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a83fd7, SUC_lexicon_tags_25 },
    { 0x65883fd8, SUC_lexicon_tags_7 },
    { 0x5ba83fd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2e83fdb, SUC_lexicon_tags_17 },
    { 0x43a83fdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c083fe2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x13483fe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11683fe8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81c83fea, SUC_lexicon_tags_15 },
    { 0x78a83feb, SUC_lexicon_tags_45 },
    { 0x6ca83fec, SUC_lexicon_tags_38 },
    { 0xc3083fea, SUC_lexicon_tags_14 },
    { 0x2c683fee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe483ff0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb883ff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f883ff8, SUC_lexicon_tags_11 },
    { 0x5fe83ff9, SUC_lexicon_tags_45 },
    { 0x85883ff8, SUC_lexicon_tags_61 },
    { 0xebe83ff8, SUC_lexicon_tags_66 },
    { 0x69083ffc, SUC_lexicon_tags_10 },
    { 0xca83ffd, SUC_lexicon_tags_6 },
    { 0xd2483ff8, SUC_lexicon_tags_33 },
    { 0x63483fff, SUC_lexicon_tags_661 },
    { 0x9ee84000, SUC_lexicon_tags_65 },
    { 0xc2083ffc, SUC_lexicon_tags_31 },
    { 0xd683ffb, SUC_lexicon_tags_5 },
    { 0x99e84003, SUC_lexicon_tags_7 },
    { 0x60283ffe, SUC_lexicon_tags_25 },
    { 0x9cc84005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6228400a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5328400d, SUC_lexicon_tags_27 },
    { 0x2868400e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ba84010, SUC_lexicon_tags_29 },
    { 0x30c84011, SUC_lexicon_tags_11 },
    { 0xb8884012, SUC_lexicon_tags_19 },
    { 0x32684013, SUC_lexicon_tags_27 },
    { 0xe7684012, SUC_lexicon_tags_35 },
    { 0xaa684014, SUC_lexicon_tags_22 },
    { 0xbfc84011, SUC_lexicon_tags_37 },
    { 0xcc484012, SUC_lexicon_tags_12 },
    { 0x22e84018, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd408401c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea8401f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde284021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2284024, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe128402b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c28402d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb0c8402f, SUC_lexicon_tags_15 },
    { 0x4e084030, SUC_lexicon_tags_7 },
    { 0x52484031, SUC_lexicon_tags_233 },
    { 0xb7c84031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c84035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d484037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87884040, SUC_lexicon_tags_6 },
    { 0x9884041, SUC_lexicon_tags_63 },
    { 0x8d484042, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2668404b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42e8404d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3284051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a84055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa84058, SUC_lexicon_tags_25 },
    { 0xd3484059, SUC_lexicon_tags_17 },
    { 0xb4c8405a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83884060, SUC_lexicon_tags_17 },
    { 0xdec84060, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa84065, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a84068, SUC_lexicon_tags_27 },
    { 0x4c884068, SUC_lexicon_tags_84 },
    { 0x6388406a, SUC_lexicon_tags_0 },
    { 0xe248406b, SUC_lexicon_tags_9 },
    { 0xc5084069, SUC_lexicon_tags_19 },
    { 0x9fc8406d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76c8406f, SUC_lexicon_tags_37 },
    { 0xc7284070, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4884073, SUC_lexicon_tags_489 },
    { 0x31e84074, SUC_lexicon_tags_29 },
    { 0x3b284075, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a484077, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9748407c, SUC_lexicon_tags_17 },
    { 0x2ec8407d, SUC_lexicon_tags_44 },
    { 0x1c88407e, SUC_lexicon_tags_47 },
    { 0x2248407f, SUC_lexicon_tags_1 },
    { 0xae28407d, SUC_lexicon_tags_14 },
    { 0x3084081, SUC_lexicon_tags_1 },
    { 0xbcc8407e, SUC_lexicon_tags_47 },
    { 0x59a8407d, SUC_lexicon_tags_30 },
    { 0x6a084084, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e84089, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e84093, SUC_lexicon_tags_33 },
    { 0xb6284093, SUC_lexicon_tags_27 },
    { 0x85284093, SUC_lexicon_tags_11 },
    { 0xe6284096, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x14a84098, SUC_lexicon_tags_12 },
    { 0x6f684098, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132840a1, SUC_lexicon_tags_7 },
    { 0x880840a1, SUC_lexicon_tags_15 },
    { 0x514840a3, SUC_lexicon_tags_27 },
    { 0x928840a3, SUC_lexicon_tags_11 },
    { 0xdca840a2, SUC_lexicon_tags_12 },
    { 0x1a0840a2, SUC_lexicon_tags_61 },
    { 0xdee840a7, SUC_lexicon_tags_12 },
    { 0x824840a7, SUC_lexicon_tags_21 },
    { 0xa90840a9, SUC_lexicon_tags_32 },
    { 0xd00840a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36840ac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38840b1, SUC_lexicon_tags_2 },
    { 0x832840b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd0e840b4, SUC_lexicon_tags_15 },
    { 0xa36840b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe5c840b7, SUC_lexicon_tags_249 },
    { 0, NULL },
    { 0xe62840b9, SUC_lexicon_tags_47 },
    { 0xac2840ba, SUC_lexicon_tags_8 },
    { 0x86e840b9, SUC_lexicon_tags_2 },
    { 0xdae840b9, SUC_lexicon_tags_5 },
    { 0xb84840bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0840c7, SUC_lexicon_tags_5 },
    { 0xbe840c7, SUC_lexicon_tags_1 },
    { 0xd9a840c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4c840cb, SUC_lexicon_tags_6 },
    { 0xe46840cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336840d0, SUC_lexicon_tags_1 },
    { 0xc8840d1, SUC_lexicon_tags_30 },
    { 0xd4e840d2, SUC_lexicon_tags_1 },
    { 0xed6840d3, SUC_lexicon_tags_30 },
    { 0x682840d4, SUC_lexicon_tags_7 },
    { 0x9e8840d3, SUC_lexicon_tags_25 },
    { 0x67c840d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e840db, SUC_lexicon_tags_25 },
    { 0xa90840dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8840e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2840e7, SUC_lexicon_tags_25 },
    { 0xfbe840e8, SUC_lexicon_tags_14 },
    { 0xad0840e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190840ed, SUC_lexicon_tags_551 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82840f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc840f3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2840f7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa840fd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0084100, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59084103, SUC_lexicon_tags_10 },
    { 0x88a84103, SUC_lexicon_tags_19 },
    { 0xae084105, SUC_lexicon_tags_80 },
    { 0xe2684104, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e484108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28410b, SUC_lexicon_tags_30 },
    { 0x96c8410c, SUC_lexicon_tags_10 },
    { 0xa548410c, SUC_lexicon_tags_6 },
    { 0x6048410e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41084117, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x43884119, SUC_lexicon_tags_37 },
    { 0xe9284119, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488411f, SUC_lexicon_tags_8 },
    { 0x5da8411f, SUC_lexicon_tags_37 },
    { 0x10c84121, SUC_lexicon_tags_27 },
    { 0xfe88411f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf684124, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e484127, SUC_lexicon_tags_0 },
    { 0x95c84128, SUC_lexicon_tags_1 },
    { 0x64a84129, SUC_lexicon_tags_17 },
    { 0xaee84128, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4328412d, SUC_lexicon_tags_37 },
    { 0xb588412e, SUC_lexicon_tags_17 },
    { 0xe508412d, SUC_lexicon_tags_37 },
    { 0xc9884130, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79484134, SUC_lexicon_tags_112 },
    { 0x75884135, SUC_lexicon_tags_2 },
    { 0xef084136, SUC_lexicon_tags_48 },
    { 0xe6e84137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76884139, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c88413c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c84141, SUC_lexicon_tags_30 },
    { 0x2a084141, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2f684144, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x44684146, SUC_lexicon_tags_11 },
    { 0xebc84147, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c8414d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ee8414f, SUC_lexicon_tags_12 },
    { 0x90c84150, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x684153, SUC_lexicon_tags_9 },
    { 0x71284154, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e8415b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4e8415d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6884162, SUC_lexicon_tags_218 },
    { 0xb0084163, SUC_lexicon_tags_17 },
    { 0xdce84162, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcda84166, SUC_lexicon_tags_25 },
    { 0x14e84167, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e84171, SUC_lexicon_tags_16 },
    { 0xedc84171, SUC_lexicon_tags_25 },
    { 0x6a084173, SUC_lexicon_tags_2 },
    { 0x4b884172, SUC_lexicon_tags_11 },
    { 0x17884171, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc84178, SUC_lexicon_tags_25 },
    { 0xc1484178, SUC_lexicon_tags_47 },
    { 0xd8c84178, SUC_lexicon_tags_17 },
    { 0xe6c84179, SUC_lexicon_tags_27 },
    { 0x4c8417c, SUC_lexicon_tags_33 },
    { 0xa8884178, SUC_lexicon_tags_6 },
    { 0x12a8417e, SUC_lexicon_tags_12 },
    { 0x52a84178, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c84182, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d484184, SUC_lexicon_tags_37 },
    { 0xe8484185, SUC_lexicon_tags_15 },
    { 0xcec84185, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0684189, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5108418c, SUC_lexicon_tags_10 },
    { 0x6028418d, SUC_lexicon_tags_56 },
    { 0xaa28418c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xde684190, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xab884193, SUC_lexicon_tags_27 },
    { 0x43084194, SUC_lexicon_tags_1 },
    { 0x72a84193, SUC_lexicon_tags_4 },
    { 0xdc084196, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25a84198, SUC_lexicon_tags_47 },
    { 0xda084198, SUC_lexicon_tags_0 },
    { 0x1008419a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c8419e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17a841a0, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a841a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x870841a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2841a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x688841ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2841b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4841b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54841bc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x966841be, SUC_lexicon_tags_66 },
    { 0x804841bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x644841c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e841c6, SUC_lexicon_tags_48 },
    { 0xc70841c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8841ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524841d1, SUC_lexicon_tags_17 },
    { 0xa84841d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1c841d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8841d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c841db, SUC_lexicon_tags_45 },
    { 0xbf8841dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e841df, SUC_lexicon_tags_2 },
    { 0x926841e0, SUC_lexicon_tags_0 },
    { 0x886841e0, SUC_lexicon_tags_11 },
    { 0x2c6841e1, SUC_lexicon_tags_28 },
    { 0xd14841e3, SUC_lexicon_tags_27 },
    { 0x22841e4, SUC_lexicon_tags_2 },
    { 0x51c841e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x566841e7, SUC_lexicon_tags_15 },
    { 0xd5a841e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80841ee, SUC_lexicon_tags_60 },
    { 0xea0841ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5c2841f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd08841f3, SUC_lexicon_tags_17 },
    { 0x9e2841f4, SUC_lexicon_tags_77 },
    { 0xdb2841f3, SUC_lexicon_tags_2 },
    { 0x93e841f5, SUC_lexicon_tags_17 },
    { 0xe94841f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a841fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x724841fc, SUC_lexicon_tags_19 },
    { 0x6ee841fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a8841ff, SUC_lexicon_tags_25 },
    { 0x31e84200, SUC_lexicon_tags_19 },
    { 0x60484201, SUC_lexicon_tags_26 },
    { 0xa8484202, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ca84204, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4484208, SUC_lexicon_tags_21 },
    { 0xdda84208, SUC_lexicon_tags_33 },
    { 0xfbe8420a, SUC_lexicon_tags_55 },
    { 0x3468420a, SUC_lexicon_tags_7 },
    { 0x5708420c, SUC_lexicon_tags_36 },
    { 0x1c68420d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c84212, SUC_lexicon_tags_12 },
    { 0xac684213, SUC_lexicon_tags_12 },
    { 0x89684212, SUC_lexicon_tags_27 },
    { 0xe5c84215, SUC_lexicon_tags_17 },
    { 0x7bc84216, SUC_lexicon_tags_21 },
    { 0x12e84214, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc8421a, SUC_lexicon_tags_45 },
    { 0x4e68421b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08421f, SUC_lexicon_tags_16 },
    { 0xd3084220, SUC_lexicon_tags_4 },
    { 0x26084221, SUC_lexicon_tags_4 },
    { 0xbe84220, SUC_lexicon_tags_17 },
    { 0x3f684223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ae84225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab48422b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda84230, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea84239, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee8423c, SUC_lexicon_tags_19 },
    { 0xcde8423d, SUC_lexicon_tags_49 },
    { 0x1b68423d, SUC_lexicon_tags_21 },
    { 0xba8423f, SUC_lexicon_tags_1 },
    { 0xe68423e, SUC_lexicon_tags_92 },
    { 0xa1e8423e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe0c84243, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8424c, SUC_lexicon_tags_17 },
    { 0xfe8424d, SUC_lexicon_tags_25 },
    { 0x168424c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x99484250, SUC_lexicon_tags_491 },
    { 0, NULL },
    { 0xe484252, SUC_lexicon_tags_4 },
    { 0xcde84253, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b084258, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x628425a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6788425f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32084261, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea84264, SUC_lexicon_tags_31 },
    { 0x15084265, SUC_lexicon_tags_0 },
    { 0x9b284266, SUC_lexicon_tags_7 },
    { 0x7e684267, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e8426e, SUC_lexicon_tags_17 },
    { 0xee68426e, SUC_lexicon_tags_11 },
    { 0xd268426e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d28427b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc608427e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52084280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43884284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce84287, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8ac84289, SUC_lexicon_tags_146 },
    { 0x4bc8428a, SUC_lexicon_tags_0 },
    { 0x58c8428a, SUC_lexicon_tags_11 },
    { 0x8ec8428b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56884290, SUC_lexicon_tags_114 },
    { 0x76e84291, SUC_lexicon_tags_65 },
    { 0xc1684291, SUC_lexicon_tags_15 },
    { 0xa3284293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e8429c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0842a1, SUC_lexicon_tags_2 },
    { 0x796842a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8842aa, SUC_lexicon_tags_66 },
    { 0x144842ab, SUC_lexicon_tags_24 },
    { 0xab4842ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a842b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x420842b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c842b4, SUC_lexicon_tags_60 },
    { 0x38842b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50842b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada842be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518842c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f0842c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x530842c7, SUC_lexicon_tags_30 },
    { 0x5e8842c8, SUC_lexicon_tags_17 },
    { 0xa1e842c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6842d0, SUC_lexicon_tags_11 },
    { 0x34a842d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x900842d4, SUC_lexicon_tags_10 },
    { 0x868842d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce842df, SUC_lexicon_tags_113 },
    { 0x4b6842e0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd0842e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0842e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeac842ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34842ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x942842ee, SUC_lexicon_tags_14 },
    { 0x9c0842ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8842f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa842fa, SUC_lexicon_tags_15 },
    { 0x1e8842fb, SUC_lexicon_tags_62 },
    { 0xa66842fc, SUC_lexicon_tags_25 },
    { 0x6c0842fd, SUC_lexicon_tags_21 },
    { 0xa0a842fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98684301, SUC_lexicon_tags_35 },
    { 0xef484302, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd0484304, SUC_lexicon_tags_25 },
    { 0xeb284305, SUC_lexicon_tags_11 },
    { 0xf6e84306, SUC_lexicon_tags_79 },
    { 0x68084304, SUC_lexicon_tags_26 },
    { 0xaa084308, SUC_lexicon_tags_14 },
    { 0x9b684309, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf308430b, SUC_lexicon_tags_6 },
    { 0xd8a8430c, SUC_lexicon_tags_46 },
    { 0xe568430d, SUC_lexicon_tags_55 },
    { 0xc388430d, SUC_lexicon_tags_15 },
    { 0x25c8430b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee8431e, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c84322, SUC_lexicon_tags_2 },
    { 0x4de84322, SUC_lexicon_tags_14 },
    { 0xc4684323, SUC_lexicon_tags_12 },
    { 0xa9084325, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e84329, SUC_lexicon_tags_11 },
    { 0x71e8432a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a8432d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3348432f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4e84331, SUC_lexicon_tags_38 },
    { 0x2ba84332, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e84335, SUC_lexicon_tags_46 },
    { 0xd4a84335, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48884339, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc8433e, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xbb484340, SUC_lexicon_tags_5 },
    { 0xdae84340, SUC_lexicon_tags_10 },
    { 0x6884342, SUC_lexicon_tags_56 },
    { 0xee84342, SUC_lexicon_tags_14 },
    { 0xf5284344, SUC_lexicon_tags_37 },
    { 0xb8684345, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4d884347, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x488434a, SUC_lexicon_tags_15 },
    { 0x4ce8434a, SUC_lexicon_tags_21 },
    { 0x37e8434c, SUC_lexicon_tags_15 },
    { 0x8b08434c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xca88434f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56684352, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c8435c, SUC_lexicon_tags_14 },
    { 0x7928435d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80684364, SUC_lexicon_tags_15 },
    { 0xac884365, SUC_lexicon_tags_37 },
    { 0xc0084364, SUC_lexicon_tags_45 },
    { 0x19684367, SUC_lexicon_tags_40 },
    { 0x4d684367, SUC_lexicon_tags_63 },
    { 0xc5e84368, SUC_lexicon_tags_37 },
    { 0xaf284367, SUC_lexicon_tags_25 },
    { 0x4128436b, SUC_lexicon_tags_2 },
    { 0x25c84369, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd684371, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2484374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff884376, SUC_lexicon_tags_23 },
    { 0x5a684376, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228437e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17284380, SUC_lexicon_tags_27 },
    { 0x5c684381, SUC_lexicon_tags_17 },
    { 0x8e484380, SUC_lexicon_tags_49 },
    { 0x76484383, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e84388, SUC_lexicon_tags_33 },
    { 0x3fa84389, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb68438b, SUC_lexicon_tags_8 },
    { 0x688438c, SUC_lexicon_tags_2 },
    { 0xae88438d, SUC_lexicon_tags_25 },
    { 0x8508438e, SUC_lexicon_tags_37 },
    { 0xa748438f, SUC_lexicon_tags_44 },
    { 0x30084390, SUC_lexicon_tags_49 },
    { 0x78284391, SUC_lexicon_tags_11 },
    { 0xada8438c, SUC_lexicon_tags_7 },
    { 0x7f08438f, SUC_lexicon_tags_60 },
    { 0x8e8438f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x67484396, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb484398, SUC_lexicon_tags_37 },
    { 0xca484399, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce8439d, SUC_lexicon_tags_23 },
    { 0xbb48439e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7e843a0, SUC_lexicon_tags_37 },
    { 0x8b6843a1, SUC_lexicon_tags_9 },
    { 0x1b8843a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90843a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe32843a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba843ab, SUC_lexicon_tags_25 },
    { 0x6a0843ac, SUC_lexicon_tags_9 },
    { 0x9aa843ad, SUC_lexicon_tags_0 },
    { 0x60c843ab, SUC_lexicon_tags_15 },
    { 0xff4843af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15c843b1, SUC_lexicon_tags_19 },
    { 0x138843b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1c843b4, SUC_lexicon_tags_11 },
    { 0x1c4843b5, SUC_lexicon_tags_15 },
    { 0x938843b5, SUC_lexicon_tags_37 },
    { 0xdd8843b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4843be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x766843c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34843c6, SUC_lexicon_tags_25 },
    { 0x7b8843c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2f2843c9, SUC_lexicon_tags_29 },
    { 0x94a843c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e843ce, SUC_lexicon_tags_9 },
    { 0x70e843cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e843d2, SUC_lexicon_tags_17 },
    { 0x43a843d3, SUC_lexicon_tags_15 },
    { 0x45e843d3, SUC_lexicon_tags_60 },
    { 0x704843d5, SUC_lexicon_tags_12 },
    { 0xa8a843d6, SUC_lexicon_tags_17 },
    { 0xd42843d7, SUC_lexicon_tags_4 },
    { 0x93a843d8, SUC_lexicon_tags_27 },
    { 0x34843d8, SUC_lexicon_tags_45 },
    { 0xc74843d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e843e3, SUC_lexicon_tags_27 },
    { 0xfc2843e4, SUC_lexicon_tags_0 },
    { 0xa54843e4, SUC_lexicon_tags_15 },
    { 0x510843e6, SUC_lexicon_tags_26 },
    { 0x280843e7, SUC_lexicon_tags_11 },
    { 0x312843e8, SUC_lexicon_tags_2 },
    { 0x972843e9, SUC_lexicon_tags_30 },
    { 0xf44843e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcea843ec, SUC_lexicon_tags_12 },
    { 0xd1a843ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f8843ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x462843f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592843f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x386843fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8ca843fc, SUC_lexicon_tags_66 },
    { 0x83e843fc, SUC_lexicon_tags_10 },
    { 0x94e843fc, SUC_lexicon_tags_12 },
    { 0x4ba843ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce084403, SUC_lexicon_tags_32 },
    { 0x42284404, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd484406, SUC_lexicon_tags_4 },
    { 0xf5a84407, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb508440b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5e8440d, SUC_lexicon_tags_16 },
    { 0xd548440e, SUC_lexicon_tags_75 },
    { 0x3de8440f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3884411, SUC_lexicon_tags_12 },
    { 0x7d884411, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c84415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34884418, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf7a8441a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62a8441c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe208441f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xac684422, SUC_lexicon_tags_11 },
    { 0xbba84422, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe484425, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63484427, SUC_lexicon_tags_14 },
    { 0x8d284428, SUC_lexicon_tags_25 },
    { 0x75484427, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a8442e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32484433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e284435, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38c84437, SUC_lexicon_tags_25 },
    { 0xe4684438, SUC_lexicon_tags_60 },
    { 0x45c84437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62a8443b, SUC_lexicon_tags_15 },
    { 0xfcc8443b, SUC_lexicon_tags_30 },
    { 0x7948443d, SUC_lexicon_tags_27 },
    { 0xa6a8443d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c84441, SUC_lexicon_tags_63 },
    { 0xb0a84441, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x18484445, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43c84447, SUC_lexicon_tags_23 },
    { 0x7884447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf68444b, SUC_lexicon_tags_45 },
    { 0x7cc8444b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd848444e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48884451, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b484453, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e84456, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a8445f, SUC_lexicon_tags_37 },
    { 0x43c84460, SUC_lexicon_tags_11 },
    { 0x1de84460, SUC_lexicon_tags_16 },
    { 0x15a84462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2284465, SUC_lexicon_tags_55 },
    { 0x89e84466, SUC_lexicon_tags_60 },
    { 0x98484467, SUC_lexicon_tags_37 },
    { 0xfa684468, SUC_lexicon_tags_1 },
    { 0xa1484469, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b68446d, SUC_lexicon_tags_15 },
    { 0xbcc8446d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa84473, SUC_lexicon_tags_2 },
    { 0x3aa84474, SUC_lexicon_tags_29 },
    { 0xbf884475, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f884478, SUC_lexicon_tags_27 },
    { 0x59284479, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc84480, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a884482, SUC_lexicon_tags_19 },
    { 0x7c684483, SUC_lexicon_tags_26 },
    { 0x2b084483, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9368448a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4884493, SUC_lexicon_tags_37 },
    { 0xae684494, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb328449a, SUC_lexicon_tags_1 },
    { 0x9b48449b, SUC_lexicon_tags_15 },
    { 0x4888449c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16844a4, SUC_lexicon_tags_6 },
    { 0xfbe844a5, SUC_lexicon_tags_28 },
    { 0xd26844a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x230844a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2844ab, SUC_lexicon_tags_77 },
    { 0xa8844ac, SUC_lexicon_tags_0 },
    { 0x95a844ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x19e844b0, SUC_lexicon_tags_60 },
    { 0xe30844b1, SUC_lexicon_tags_3 },
    { 0x12e844b0, SUC_lexicon_tags_17 },
    { 0x4fe844b3, SUC_lexicon_tags_49 },
    { 0xa2a844b4, SUC_lexicon_tags_0 },
    { 0x502844b5, SUC_lexicon_tags_7 },
    { 0xe1e844b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e844ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a844be, SUC_lexicon_tags_7 },
    { 0x796844bf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8c4844c1, SUC_lexicon_tags_63 },
    { 0x9b8844c2, SUC_lexicon_tags_47 },
    { 0x342844c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e844c6, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8844cd, SUC_lexicon_tags_1 },
    { 0xc86844ce, SUC_lexicon_tags_27 },
    { 0xcd6844cf, SUC_lexicon_tags_7 },
    { 0x54844d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208844d4, SUC_lexicon_tags_82 },
    { 0xb22844d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c844dd, SUC_lexicon_tags_0 },
    { 0xef8844dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c844e7, SUC_lexicon_tags_15 },
    { 0x72e844e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6844ef, SUC_lexicon_tags_27 },
    { 0x570844f0, SUC_lexicon_tags_25 },
    { 0x22a844f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a844fc, SUC_lexicon_tags_7 },
    { 0x67c844fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdca844ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x78884501, SUC_lexicon_tags_0 },
    { 0x1ec84502, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32084504, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b68450b, SUC_lexicon_tags_25 },
    { 0xdfe8450b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a68450f, SUC_lexicon_tags_11 },
    { 0x7c684510, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58e84512, SUC_lexicon_tags_35 },
    { 0xf5a84513, SUC_lexicon_tags_7 },
    { 0x5e084513, SUC_lexicon_tags_14 },
    { 0xe4e84512, SUC_lexicon_tags_19 },
    { 0x98484513, SUC_lexicon_tags_17 },
    { 0xc9084517, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa788451a, SUC_lexicon_tags_33 },
    { 0x5fe8451b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c84521, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb88452a, SUC_lexicon_tags_29 },
    { 0x90a8452a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf548452d, SUC_lexicon_tags_11 },
    { 0x9d28452e, SUC_lexicon_tags_25 },
    { 0x57c8452d, SUC_lexicon_tags_31 },
    { 0x76284530, SUC_lexicon_tags_32 },
    { 0xf4884530, SUC_lexicon_tags_27 },
    { 0xa4a84532, SUC_lexicon_tags_12 },
    { 0x47684533, SUC_lexicon_tags_12 },
    { 0xe9c84534, SUC_lexicon_tags_1 },
    { 0x4ee84534, SUC_lexicon_tags_23 },
    { 0x2da84532, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe368453c, SUC_lexicon_tags_27 },
    { 0x7b68453d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12084544, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a884549, SUC_lexicon_tags_15 },
    { 0x19e8454a, SUC_lexicon_tags_2 },
    { 0xada8454a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7484551, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba84554, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59284558, SUC_lexicon_tags_2 },
    { 0xbe284558, SUC_lexicon_tags_7 },
    { 0xf208455a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bc8455c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8084563, SUC_lexicon_tags_70 },
    { 0x24c84564, SUC_lexicon_tags_37 },
    { 0x3e684565, SUC_lexicon_tags_9 },
    { 0x10484566, SUC_lexicon_tags_11 },
    { 0xa484563, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xca48456a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabe8456c, SUC_lexicon_tags_38 },
    { 0x9b08456d, SUC_lexicon_tags_30 },
    { 0x57a8456d, SUC_lexicon_tags_17 },
    { 0x21a8456f, SUC_lexicon_tags_37 },
    { 0x58284570, SUC_lexicon_tags_37 },
    { 0x48e8456f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d28457b, SUC_lexicon_tags_17 },
    { 0x2fe8457b, SUC_lexicon_tags_23 },
    { 0xb428457b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xada8457f, SUC_lexicon_tags_15 },
    { 0x9368457f, SUC_lexicon_tags_60 },
    { 0x4da8457f, SUC_lexicon_tags_29 },
    { 0x48c84580, SUC_lexicon_tags_38 },
    { 0x66c84583, SUC_lexicon_tags_42 },
    { 0x54884584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a84587, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8484589, SUC_lexicon_tags_25 },
    { 0x3c48458a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d88458e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ea84590, SUC_lexicon_tags_3 },
    { 0xb6084591, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a8459a, SUC_lexicon_tags_1 },
    { 0xfc28459b, SUC_lexicon_tags_12 },
    { 0xc608459b, SUC_lexicon_tags_37 },
    { 0x51a8459c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a845a3, SUC_lexicon_tags_9 },
    { 0x8b4845a4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde845a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe40845a9, SUC_lexicon_tags_8 },
    { 0xdb8845aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7e845ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe845b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e845bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xda8845be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4d6845c0, SUC_lexicon_tags_19 },
    { 0x24c845c0, SUC_lexicon_tags_38 },
    { 0xbac845c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e845cb, SUC_lexicon_tags_25 },
    { 0x88c845cc, SUC_lexicon_tags_15 },
    { 0x2e2845cd, SUC_lexicon_tags_107 },
    { 0xfb6845cd, SUC_lexicon_tags_12 },
    { 0x820845cf, SUC_lexicon_tags_11 },
    { 0xbe2845d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956845d4, SUC_lexicon_tags_11 },
    { 0x9e8845d5, SUC_lexicon_tags_45 },
    { 0x94a845d4, SUC_lexicon_tags_55 },
    { 0x22845d5, SUC_lexicon_tags_9 },
    { 0xc9a845d8, SUC_lexicon_tags_1 },
    { 0x116845d5, SUC_lexicon_tags_21 },
    { 0x784845d4, SUC_lexicon_tags_37 },
    { 0x8e6845db, SUC_lexicon_tags_12 },
    { 0x7a8845dc, SUC_lexicon_tags_28 },
    { 0xbda845db, SUC_lexicon_tags_12 },
    { 0x4de845de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba845e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34845e4, SUC_lexicon_tags_49 },
    { 0x438845e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67c845e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8c845e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46845f1, SUC_lexicon_tags_0 },
    { 0x7ca845f1, SUC_lexicon_tags_12 },
    { 0xd94845f3, SUC_lexicon_tags_1 },
    { 0x568845f4, SUC_lexicon_tags_7 },
    { 0x4d8845f3, SUC_lexicon_tags_12 },
    { 0x554845f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be845f9, SUC_lexicon_tags_29 },
    { 0x894845fa, SUC_lexicon_tags_7 },
    { 0x90845fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f484600, SUC_lexicon_tags_25 },
    { 0x38484601, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91484606, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2684609, SUC_lexicon_tags_2 },
    { 0xb3884609, SUC_lexicon_tags_27 },
    { 0xb2e8460b, SUC_lexicon_tags_37 },
    { 0x3e08460b, SUC_lexicon_tags_43 },
    { 0xb128460d, SUC_lexicon_tags_37 },
    { 0x7248460d, SUC_lexicon_tags_56 },
    { 0x95484609, SUC_lexicon_tags_6 },
    { 0xc2a84610, SUC_lexicon_tags_25 },
    { 0x2ac84611, SUC_lexicon_tags_7 },
    { 0xa3884612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e84619, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2228461b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf148461f, SUC_lexicon_tags_43 },
    { 0x2348461f, SUC_lexicon_tags_11 },
    { 0xe7684621, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f284625, SUC_lexicon_tags_17 },
    { 0x8ea84626, SUC_lexicon_tags_68 },
    { 0xca484625, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4528462b, SUC_lexicon_tags_15 },
    { 0xe948462c, SUC_lexicon_tags_33 },
    { 0x38e8462d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3284631, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4e84633, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da84637, SUC_lexicon_tags_27 },
    { 0x23284637, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4a8463a, SUC_lexicon_tags_41 },
    { 0x3168463a, SUC_lexicon_tags_17 },
    { 0x2be8463c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56c8463e, SUC_lexicon_tags_30 },
    { 0xbfe8463e, SUC_lexicon_tags_25 },
    { 0xb4a84640, SUC_lexicon_tags_14 },
    { 0x69a8463f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe284644, SUC_lexicon_tags_2 },
    { 0x5a684645, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ee84647, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x98084649, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf588464c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaae8464e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad084651, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6684654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63684656, SUC_lexicon_tags_49 },
    { 0xe7a84657, SUC_lexicon_tags_2 },
    { 0x5f284658, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3728465b, SUC_lexicon_tags_37 },
    { 0xa8e8465b, SUC_lexicon_tags_29 },
    { 0xb888465b, SUC_lexicon_tags_47 },
    { 0x4028465e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f484663, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4884665, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa84667, SUC_lexicon_tags_17 },
    { 0xc7084667, SUC_lexicon_tags_12 },
    { 0xa4084669, SUC_lexicon_tags_17 },
    { 0x4ce8466a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb168466d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3b88466f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ce84671, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a84674, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a68467c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4228467f, SUC_lexicon_tags_26 },
    { 0x6cc84680, SUC_lexicon_tags_30 },
    { 0x3ca84680, SUC_lexicon_tags_45 },
    { 0x59c84681, SUC_lexicon_tags_37 },
    { 0x26884683, SUC_lexicon_tags_12 },
    { 0x31484684, SUC_lexicon_tags_46 },
    { 0xdde84685, SUC_lexicon_tags_27 },
    { 0x73884686, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c8468a, SUC_lexicon_tags_45 },
    { 0xf2c8468b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e08468e, SUC_lexicon_tags_43 },
    { 0x6948468e, SUC_lexicon_tags_37 },
    { 0xde88468f, SUC_lexicon_tags_43 },
    { 0xc6a84691, SUC_lexicon_tags_37 },
    { 0xdd884692, SUC_lexicon_tags_25 },
    { 0xf4884693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea684695, SUC_lexicon_tags_37 },
    { 0xe0484695, SUC_lexicon_tags_1 },
    { 0xcaa84696, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b48469a, SUC_lexicon_tags_1 },
    { 0x51a8469b, SUC_lexicon_tags_18 },
    { 0x9d28469a, SUC_lexicon_tags_25 },
    { 0x9188469d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662846b1, SUC_lexicon_tags_2 },
    { 0x532846b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878846bd, SUC_lexicon_tags_12 },
    { 0xcac846bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xefc846c0, SUC_lexicon_tags_60 },
    { 0x94e846c1, SUC_lexicon_tags_10 },
    { 0x600846c1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x61e846c4, SUC_lexicon_tags_27 },
    { 0x7f4846c5, SUC_lexicon_tags_6 },
    { 0x332846c6, SUC_lexicon_tags_70 },
    { 0x21c846c7, SUC_lexicon_tags_17 },
    { 0xd84846c8, SUC_lexicon_tags_45 },
    { 0x88a846c9, SUC_lexicon_tags_18 },
    { 0x798846ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x260846cd, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982846dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x886846de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6846e1, SUC_lexicon_tags_25 },
    { 0x468846e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90846e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36846eb, SUC_lexicon_tags_2 },
    { 0xd34846eb, SUC_lexicon_tags_84 },
    { 0x84e846ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e846f5, SUC_lexicon_tags_45 },
    { 0x4e846f6, SUC_lexicon_tags_2 },
    { 0xae846f6, SUC_lexicon_tags_47 },
    { 0xeac846f6, SUC_lexicon_tags_15 },
    { 0x184846f6, SUC_lexicon_tags_65 },
    { 0x558846f5, SUC_lexicon_tags_17 },
    { 0x188846fb, SUC_lexicon_tags_8 },
    { 0xf6e846fc, SUC_lexicon_tags_14 },
    { 0x45a846fc, SUC_lexicon_tags_11 },
    { 0x716846fc, SUC_lexicon_tags_17 },
    { 0x23e846f9, SUC_lexicon_tags_1554 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc084704, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x93084706, SUC_lexicon_tags_15 },
    { 0xc6884707, SUC_lexicon_tags_33 },
    { 0x4d084708, SUC_lexicon_tags_19 },
    { 0xc084707, SUC_lexicon_tags_25 },
    { 0x3b68470a, SUC_lexicon_tags_68 },
    { 0x3608470b, SUC_lexicon_tags_25 },
    { 0xba48470b, SUC_lexicon_tags_0 },
    { 0xdfc8470c, SUC_lexicon_tags_38 },
    { 0x948470c, SUC_lexicon_tags_49 },
    { 0xab48470b, SUC_lexicon_tags_14 },
    { 0x3fa84710, SUC_lexicon_tags_47 },
    { 0xee484708, SUC_lexicon_tags_45 },
    { 0x9b284712, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c84715, SUC_lexicon_tags_1 },
    { 0x7ae84716, SUC_lexicon_tags_49 },
    { 0xbec84717, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa848471c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c28471f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x35884722, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8284727, SUC_lexicon_tags_37 },
    { 0x4d884728, SUC_lexicon_tags_25 },
    { 0x8b884729, SUC_lexicon_tags_55 },
    { 0xf788472a, SUC_lexicon_tags_15 },
    { 0x15284729, SUC_lexicon_tags_26 },
    { 0xf2c8472c, SUC_lexicon_tags_26 },
    { 0xd4e8472d, SUC_lexicon_tags_47 },
    { 0xa2c8472e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87c84730, SUC_lexicon_tags_29 },
    { 0x4ac84730, SUC_lexicon_tags_9 },
    { 0xda084730, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a84736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3188473f, SUC_lexicon_tags_37 },
    { 0x10084740, SUC_lexicon_tags_17 },
    { 0xe5684741, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc84745, SUC_lexicon_tags_37 },
    { 0x27e84746, SUC_lexicon_tags_15 },
    { 0x22684747, SUC_lexicon_tags_33 },
    { 0x7c684748, SUC_lexicon_tags_15 },
    { 0x99684749, SUC_lexicon_tags_0 },
    { 0x31a84748, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba88474e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43084755, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe988475a, SUC_lexicon_tags_0 },
    { 0xab48475b, SUC_lexicon_tags_11 },
    { 0x4e48475b, SUC_lexicon_tags_17 },
    { 0xd628475b, SUC_lexicon_tags_54 },
    { 0xa1c8475e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55884760, SUC_lexicon_tags_10 },
    { 0x6fa84761, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf684769, SUC_lexicon_tags_1 },
    { 0x72684769, SUC_lexicon_tags_63 },
    { 0x11a8476b, SUC_lexicon_tags_11 },
    { 0x8128476a, SUC_lexicon_tags_45 },
    { 0x4048476d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xca68476f, SUC_lexicon_tags_38 },
    { 0x5708476f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c84773, SUC_lexicon_tags_28 },
    { 0x5a484773, SUC_lexicon_tags_25 },
    { 0x4884775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb084778, SUC_lexicon_tags_68 },
    { 0x9f484779, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a8477d, SUC_lexicon_tags_21 },
    { 0x1608477d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e84785, SUC_lexicon_tags_37 },
    { 0x8ec84785, SUC_lexicon_tags_37 },
    { 0xcb884785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c884789, SUC_lexicon_tags_45 },
    { 0xb288478a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf808478c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a8478f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c84792, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc84797, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3ee84799, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c08479f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc20847a1, SUC_lexicon_tags_105 },
    { 0xb02847a2, SUC_lexicon_tags_65 },
    { 0xa8c847a2, SUC_lexicon_tags_11 },
    { 0x590847a4, SUC_lexicon_tags_17 },
    { 0x4c8847a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e847ab, SUC_lexicon_tags_9 },
    { 0xd04847ab, SUC_lexicon_tags_15 },
    { 0x960847ad, SUC_lexicon_tags_17 },
    { 0xfe2847ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c847b2, SUC_lexicon_tags_17 },
    { 0xefe847b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a847b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe7c847b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec847be, SUC_lexicon_tags_11 },
    { 0x288847be, SUC_lexicon_tags_10 },
    { 0xae0847c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740847cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x244847cf, SUC_lexicon_tags_23 },
    { 0xb30847cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf22847d2, SUC_lexicon_tags_235 },
    { 0x480847d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe4847d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa847d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12e847db, SUC_lexicon_tags_12 },
    { 0xcf4847dc, SUC_lexicon_tags_235 },
    { 0x810847dd, SUC_lexicon_tags_136 },
    { 0x2dc847dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12847e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x14847e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e6847ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc7a847ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0847ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd36847f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae847f5, SUC_lexicon_tags_49 },
    { 0xc16847f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c847fd, SUC_lexicon_tags_26 },
    { 0xa1a847fe, SUC_lexicon_tags_15 },
    { 0x8f4847fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xed684802, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8884805, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508480d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e84810, SUC_lexicon_tags_40 },
    { 0x2084811, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7684813, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x53c84815, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc488481b, SUC_lexicon_tags_17 },
    { 0x7ba8481b, SUC_lexicon_tags_15 },
    { 0x2e28481c, SUC_lexicon_tags_49 },
    { 0xdf28481d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xee884820, SUC_lexicon_tags_66 },
    { 0xebe84821, SUC_lexicon_tags_43 },
    { 0x18e84820, SUC_lexicon_tags_43 },
    { 0x5a484823, SUC_lexicon_tags_15 },
    { 0xbb484821, SUC_lexicon_tags_7 },
    { 0xd6c84825, SUC_lexicon_tags_30 },
    { 0x6c84820, SUC_lexicon_tags_5 },
    { 0xf1c84827, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2548482b, SUC_lexicon_tags_12 },
    { 0xa3a8482c, SUC_lexicon_tags_25 },
    { 0x73e8482c, SUC_lexicon_tags_14 },
    { 0x968482d, SUC_lexicon_tags_10 },
    { 0x8448482d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c84836, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e8483a, SUC_lexicon_tags_45 },
    { 0xbe68483b, SUC_lexicon_tags_33 },
    { 0xeec8483c, SUC_lexicon_tags_28 },
    { 0x7128483b, SUC_lexicon_tags_1 },
    { 0x29e8483e, SUC_lexicon_tags_65 },
    { 0x7668483f, SUC_lexicon_tags_12 },
    { 0x1008483c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe84843, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62884847, SUC_lexicon_tags_11 },
    { 0x8f084848, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9f68484a, SUC_lexicon_tags_63 },
    { 0xcbe8484b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccc8484d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7dc8484f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a84857, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5c484859, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a8485d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x30e8485f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e84869, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a28486c, SUC_lexicon_tags_8 },
    { 0xcbe8486d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8168486f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x21484871, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e84874, SUC_lexicon_tags_37 },
    { 0x85484875, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ca84877, SUC_lexicon_tags_15 },
    { 0xe2484877, SUC_lexicon_tags_17 },
    { 0x60884877, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7348487d, SUC_lexicon_tags_40 },
    { 0xaf08487d, SUC_lexicon_tags_82 },
    { 0xb08487f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0684881, SUC_lexicon_tags_17 },
    { 0x95e84882, SUC_lexicon_tags_15 },
    { 0xc484881, SUC_lexicon_tags_9 },
    { 0x89a84881, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe3084886, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82284888, SUC_lexicon_tags_47 },
    { 0xa1684888, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b88488e, SUC_lexicon_tags_31 },
    { 0x5e08488f, SUC_lexicon_tags_12 },
    { 0x27284890, SUC_lexicon_tags_43 },
    { 0x1fa84891, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8884897, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4284899, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b88489d, SUC_lexicon_tags_14 },
    { 0x3208489e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8848ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a8848ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0848b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdc4848b8, SUC_lexicon_tags_87 },
    { 0xc64848b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x822848bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb8848bd, SUC_lexicon_tags_18 },
    { 0x5ca848be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98848c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c848c7, SUC_lexicon_tags_47 },
    { 0x12a848c7, SUC_lexicon_tags_31 },
    { 0x49a848c9, SUC_lexicon_tags_77 },
    { 0x412848ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40848ce, SUC_lexicon_tags_33 },
    { 0x33c848cf, SUC_lexicon_tags_84 },
    { 0x3de848cf, SUC_lexicon_tags_39 },
    { 0x3e2848d1, SUC_lexicon_tags_2 },
    { 0xf7e848d1, SUC_lexicon_tags_1 },
    { 0xea2848d3, SUC_lexicon_tags_154 },
    { 0x748848d4, SUC_lexicon_tags_15 },
    { 0xbf0848d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x496848d7, SUC_lexicon_tags_17 },
    { 0xb5c848d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358848dc, SUC_lexicon_tags_15 },
    { 0x5c4848dd, SUC_lexicon_tags_11 },
    { 0x86a848de, SUC_lexicon_tags_14 },
    { 0x2e0848df, SUC_lexicon_tags_25 },
    { 0x7e2848e0, SUC_lexicon_tags_12 },
    { 0x5de848e1, SUC_lexicon_tags_3 },
    { 0x8dc848df, SUC_lexicon_tags_37 },
    { 0x3a4848dd, SUC_lexicon_tags_32 },
    { 0x40c848e3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48848e7, SUC_lexicon_tags_17 },
    { 0xe0e848e8, SUC_lexicon_tags_34 },
    { 0xa68848e8, SUC_lexicon_tags_11 },
    { 0xbc8848e8, SUC_lexicon_tags_15 },
    { 0x572848e9, SUC_lexicon_tags_43 },
    { 0xe74848ec, SUC_lexicon_tags_47 },
    { 0xdae848ed, SUC_lexicon_tags_15 },
    { 0xc9c848ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248848f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6c848f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c848fe, SUC_lexicon_tags_6 },
    { 0x4a8848ff, SUC_lexicon_tags_21 },
    { 0x488848fe, SUC_lexicon_tags_25 },
    { 0x2ea848fe, SUC_lexicon_tags_26 },
    { 0xf3c84900, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e84906, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xed484908, SUC_lexicon_tags_11 },
    { 0xaea84909, SUC_lexicon_tags_6 },
    { 0xcfc84909, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2928490c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf368490f, SUC_lexicon_tags_66 },
    { 0x54884910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be84919, SUC_lexicon_tags_82 },
    { 0x3a88491a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc8491d, SUC_lexicon_tags_19 },
    { 0xcd68491e, SUC_lexicon_tags_11 },
    { 0xa3a8491f, SUC_lexicon_tags_15 },
    { 0xdba84920, SUC_lexicon_tags_47 },
    { 0x4fe84920, SUC_lexicon_tags_33 },
    { 0x308491f, SUC_lexicon_tags_39 },
    { 0xa1484923, SUC_lexicon_tags_34 },
    { 0x18284924, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb88492d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac84930, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf684934, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf1c84936, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x32e84938, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4688493b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ac8493d, SUC_lexicon_tags_2 },
    { 0xa9e8493e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a84941, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c684949, SUC_lexicon_tags_2 },
    { 0xe248494a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66284950, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e84953, SUC_lexicon_tags_2 },
    { 0xa5e84954, SUC_lexicon_tags_15 },
    { 0x1b084954, SUC_lexicon_tags_37 },
    { 0x16c84954, SUC_lexicon_tags_108 },
    { 0x64884957, SUC_lexicon_tags_29 },
    { 0xfa484958, SUC_lexicon_tags_37 },
    { 0x84484959, SUC_lexicon_tags_48 },
    { 0xf3084959, SUC_lexicon_tags_6 },
    { 0xb2084958, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e88495d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a84962, SUC_lexicon_tags_14 },
    { 0xeaa84963, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62684967, SUC_lexicon_tags_31 },
    { 0xf6284968, SUC_lexicon_tags_27 },
    { 0x2ce84967, SUC_lexicon_tags_37 },
    { 0xdbe84969, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc28496c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37e8496e, SUC_lexicon_tags_19 },
    { 0x9188496e, SUC_lexicon_tags_60 },
    { 0x2888496f, SUC_lexicon_tags_4 },
    { 0xc6684971, SUC_lexicon_tags_37 },
    { 0xd3a84971, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21484975, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0484979, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a8497d, SUC_lexicon_tags_12 },
    { 0x5fe8497d, SUC_lexicon_tags_64 },
    { 0x9a88497d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51484983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa884986, SUC_lexicon_tags_11 },
    { 0x7284986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f284989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2408498c, SUC_lexicon_tags_9 },
    { 0x7828498d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe484990, SUC_lexicon_tags_8 },
    { 0x96c84991, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57484993, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8484995, SUC_lexicon_tags_34 },
    { 0x7c284996, SUC_lexicon_tags_12 },
    { 0x22084997, SUC_lexicon_tags_43 },
    { 0x8c484997, SUC_lexicon_tags_11 },
    { 0x4c284997, SUC_lexicon_tags_48 },
    { 0x4988499a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd68499f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32849a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9c849a7, SUC_lexicon_tags_28 },
    { 0xbc0849a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x600849aa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2849af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84849b5, SUC_lexicon_tags_11 },
    { 0xcf4849b5, SUC_lexicon_tags_23 },
    { 0xaa2849b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38849bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa2e849bd, SUC_lexicon_tags_7 },
    { 0x584849bd, SUC_lexicon_tags_30 },
    { 0x7e8849bf, SUC_lexicon_tags_47 },
    { 0xb24849c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572849c4, SUC_lexicon_tags_12 },
    { 0xc32849c5, SUC_lexicon_tags_25 },
    { 0x522849c4, SUC_lexicon_tags_33 },
    { 0x7e2849c7, SUC_lexicon_tags_25 },
    { 0xb20849c8, SUC_lexicon_tags_14 },
    { 0xb72849c9, SUC_lexicon_tags_15 },
    { 0xb58849ca, SUC_lexicon_tags_19 },
    { 0x268849cb, SUC_lexicon_tags_44 },
    { 0xebc849c9, SUC_lexicon_tags_26 },
    { 0xa22849cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe2c849cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f6849d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf84849d3, SUC_lexicon_tags_17 },
    { 0x340849d4, SUC_lexicon_tags_35 },
    { 0x246849d5, SUC_lexicon_tags_10 },
    { 0x574849d6, SUC_lexicon_tags_87 },
    { 0x6e849d7, SUC_lexicon_tags_84 },
    { 0x212849d4, SUC_lexicon_tags_11 },
    { 0x2e4849d9, SUC_lexicon_tags_39 },
    { 0xc72849d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94849e0, SUC_lexicon_tags_33 },
    { 0x3ac849e1, SUC_lexicon_tags_48 },
    { 0xc00849e1, SUC_lexicon_tags_2 },
    { 0x728849e1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10849e6, SUC_lexicon_tags_12 },
    { 0xaa2849e7, SUC_lexicon_tags_32 },
    { 0xf88849e8, SUC_lexicon_tags_39 },
    { 0x526849e8, SUC_lexicon_tags_17 },
    { 0xe4a849ea, SUC_lexicon_tags_43 },
    { 0x8a0849eb, SUC_lexicon_tags_63 },
    { 0x834849ea, SUC_lexicon_tags_44 },
    { 0x6b4849ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56849f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x834849f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e849f8, SUC_lexicon_tags_17 },
    { 0x938849f9, SUC_lexicon_tags_10 },
    { 0xcc2849fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e849fe, SUC_lexicon_tags_60 },
    { 0x7ae849fe, SUC_lexicon_tags_569 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b884a04, SUC_lexicon_tags_11 },
    { 0xc3884a05, SUC_lexicon_tags_17 },
    { 0x5c84a05, SUC_lexicon_tags_37 },
    { 0x80284a04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7284a0a, SUC_lexicon_tags_32 },
    { 0xe4084a0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac84a11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c84a16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x42e84a18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4884a1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11a84a1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93084a1f, SUC_lexicon_tags_29 },
    { 0x11c84a1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a884a26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f284a2a, SUC_lexicon_tags_18 },
    { 0xc9a84a2b, SUC_lexicon_tags_25 },
    { 0xb5884a2a, SUC_lexicon_tags_15 },
    { 0x76684a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51284a2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5684a35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa084a39, SUC_lexicon_tags_29 },
    { 0x31084a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86084a3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e84a43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2284a47, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6684a4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e84a4f, SUC_lexicon_tags_25 },
    { 0x1f084a50, SUC_lexicon_tags_2 },
    { 0xa084a50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc5484a53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29c84a55, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x30284a57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf484a5d, SUC_lexicon_tags_25 },
    { 0xe084a5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e84a61, SUC_lexicon_tags_37 },
    { 0xeec84a62, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x19284a65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cc84a67, SUC_lexicon_tags_208 },
    { 0x68284a68, SUC_lexicon_tags_47 },
    { 0x28684a69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23e84a6b, SUC_lexicon_tags_12 },
    { 0xa7884a6c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x12284a6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde84a72, SUC_lexicon_tags_14 },
    { 0xa1484a72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e84a78, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c84a7e, SUC_lexicon_tags_14 },
    { 0x82484a7f, SUC_lexicon_tags_16 },
    { 0x55084a7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c84a83, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e84a86, SUC_lexicon_tags_66 },
    { 0xb6c84a87, SUC_lexicon_tags_38 },
    { 0x78c84a87, SUC_lexicon_tags_6 },
    { 0xcb884a89, SUC_lexicon_tags_25 },
    { 0xd3e84a8a, SUC_lexicon_tags_46 },
    { 0x38284a8a, SUC_lexicon_tags_29 },
    { 0xcfa84a8c, SUC_lexicon_tags_45 },
    { 0xbba84a8d, SUC_lexicon_tags_34 },
    { 0x88084a8d, SUC_lexicon_tags_0 },
    { 0x8c484a8d, SUC_lexicon_tags_1 },
    { 0x2284a8a, SUC_lexicon_tags_7 },
    { 0xd9a84a8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5484a94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a284a97, SUC_lexicon_tags_15 },
    { 0xd8684a98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84084a9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b884aa6, SUC_lexicon_tags_8 },
    { 0x14884aa7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5fe84aa9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8684aac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1084ab1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98484ab6, SUC_lexicon_tags_55 },
    { 0x2a084ab7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc84aba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87484abc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x82084abf, SUC_lexicon_tags_25 },
    { 0xc7c84ac0, SUC_lexicon_tags_9 },
    { 0x1fc84ac1, SUC_lexicon_tags_7 },
    { 0x2fa84ac2, SUC_lexicon_tags_74 },
    { 0xbdc84ac0, SUC_lexicon_tags_255 },
    { 0x74e84ac2, SUC_lexicon_tags_37 },
    { 0xb2c84ac4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdea84ac7, SUC_lexicon_tags_25 },
    { 0x9c884ac8, SUC_lexicon_tags_43 },
    { 0x82284ac7, SUC_lexicon_tags_951 },
    { 0x24884ac9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a884ad0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83484ad4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce84adb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde684ade, SUC_lexicon_tags_2 },
    { 0x7be84adf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc084ae1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0284ae5, SUC_lexicon_tags_14 },
    { 0x79c84ae6, SUC_lexicon_tags_349 },
    { 0xfc484ae6, SUC_lexicon_tags_14 },
    { 0xa8684ae6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c84af4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c84af8, SUC_lexicon_tags_1 },
    { 0x8ea84af9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79084afd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e84b01, SUC_lexicon_tags_25 },
    { 0xbb484b02, SUC_lexicon_tags_57 },
    { 0xf8c84b03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4484b05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe484b08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95284b12, SUC_lexicon_tags_1 },
    { 0x64884b13, SUC_lexicon_tags_15 },
    { 0x7884b14, SUC_lexicon_tags_87 },
    { 0x87a84b15, SUC_lexicon_tags_30 },
    { 0x50884b16, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc84b1b, SUC_lexicon_tags_17 },
    { 0x6a684b1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72484b1f, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2084b23, SUC_lexicon_tags_3 },
    { 0x5d484b24, SUC_lexicon_tags_23 },
    { 0xf0284b24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90084b2a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a84b2d, SUC_lexicon_tags_39 },
    { 0xdb284b2e, SUC_lexicon_tags_11 },
    { 0x2f084b2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0884b33, SUC_lexicon_tags_29 },
    { 0x65c84b34, SUC_lexicon_tags_54 },
    { 0x5b284b34, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d084b38, SUC_lexicon_tags_15 },
    { 0x79484b39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d284b3c, SUC_lexicon_tags_12 },
    { 0x36684b3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c084b40, SUC_lexicon_tags_21 },
    { 0xc5e84b41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57684b44, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdf084b46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a84b4a, SUC_lexicon_tags_7 },
    { 0x59884b4b, SUC_lexicon_tags_2 },
    { 0xd0084b4c, SUC_lexicon_tags_39 },
    { 0x6a684b4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ca84b4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd884b56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a084b58, SUC_lexicon_tags_37 },
    { 0xc9884b59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x20e84b5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x59284b5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x14884b60, SUC_lexicon_tags_37 },
    { 0x8a484b61, SUC_lexicon_tags_4 },
    { 0xb8684b62, SUC_lexicon_tags_538 },
    { 0x66684b61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6084b65, SUC_lexicon_tags_15 },
    { 0x9e084b66, SUC_lexicon_tags_216 },
    { 0xf084b67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xece84b6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa284b6c, SUC_lexicon_tags_23 },
    { 0x62684b6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94084b70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x39c84b72, SUC_lexicon_tags_27 },
    { 0x87c84b72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9284b76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed484b78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e84b7e, SUC_lexicon_tags_23 },
    { 0x1a684b7f, SUC_lexicon_tags_9 },
    { 0xf3484b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a084b85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d084b87, SUC_lexicon_tags_49 },
    { 0xc8e84b88, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4fa84b8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35084b8d, SUC_lexicon_tags_10 },
    { 0x2a484b8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de84b92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52084b95, SUC_lexicon_tags_12 },
    { 0xa7e84b95, SUC_lexicon_tags_1 },
    { 0x40084b97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30084b9d, SUC_lexicon_tags_25 },
    { 0x94c84b9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe84ba2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad884ba4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4284bac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a84baf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54484bb1, SUC_lexicon_tags_1 },
    { 0x42c84bb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d484bb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70684bb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84684bbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x30484bc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82284bc4, SUC_lexicon_tags_43 },
    { 0xabc84bc5, SUC_lexicon_tags_38 },
    { 0xb6684bc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2684bcb, SUC_lexicon_tags_1 },
    { 0x94684bcc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc684bd0, SUC_lexicon_tags_12 },
    { 0x60684bd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c84bd4, SUC_lexicon_tags_28 },
    { 0xd5a84bd4, SUC_lexicon_tags_9 },
    { 0x92084bd6, SUC_lexicon_tags_37 },
    { 0xe884bd7, SUC_lexicon_tags_45 },
    { 0x15884bd7, SUC_lexicon_tags_26 },
    { 0x14e84bd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62c84bdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87884bdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e84be1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a84be6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x86884be8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe5884bea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c84bed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf684bf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x72484bf4, SUC_lexicon_tags_49 },
    { 0xf2884bf5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e84bfa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85484c00, SUC_lexicon_tags_11 },
    { 0xc4084c01, SUC_lexicon_tags_33 },
    { 0xf0884c02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8484c06, SUC_lexicon_tags_7 },
    { 0x3c884c07, SUC_lexicon_tags_38 },
    { 0x13484c08, SUC_lexicon_tags_10 },
    { 0x2ec84c09, SUC_lexicon_tags_7 },
    { 0x3dc84c06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70684c0f, SUC_lexicon_tags_23 },
    { 0x27484c10, SUC_lexicon_tags_25 },
    { 0x29884c11, SUC_lexicon_tags_7 },
    { 0xd8884c10, SUC_lexicon_tags_27 },
    { 0x9ae84c13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12c84c15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaa84c17, SUC_lexicon_tags_14 },
    { 0x8f484c18, SUC_lexicon_tags_1 },
    { 0x5ee84c19, SUC_lexicon_tags_22 },
    { 0x68884c19, SUC_lexicon_tags_17 },
    { 0x9d284c1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc84c23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a84c26, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e84c29, SUC_lexicon_tags_38 },
    { 0x8c284c29, SUC_lexicon_tags_48 },
    { 0xcbe84c2a, SUC_lexicon_tags_43 },
    { 0x9e884c2c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3484c31, SUC_lexicon_tags_25 },
    { 0xb1884c32, SUC_lexicon_tags_26 },
    { 0x87884c33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37284c35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22484c41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e84c45, SUC_lexicon_tags_27 },
    { 0x67c84c46, SUC_lexicon_tags_1 },
    { 0x4be84c47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e84c4a, SUC_lexicon_tags_15 },
    { 0xd6684c4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc884c4d, SUC_lexicon_tags_2 },
    { 0x44084c4d, SUC_lexicon_tags_10 },
    { 0x3d084c4e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c484c52, SUC_lexicon_tags_29 },
    { 0xb2284c52, SUC_lexicon_tags_11 },
    { 0xc5a84c53, SUC_lexicon_tags_134 },
    { 0xe1e84c55, SUC_lexicon_tags_7 },
    { 0xaec84c54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4484c58, SUC_lexicon_tags_68 },
    { 0x56084c59, SUC_lexicon_tags_25 },
    { 0x16a84c58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4284c5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb484c60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7684c65, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb084c68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a84c6b, SUC_lexicon_tags_29 },
    { 0x3da84c6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0484c70, SUC_lexicon_tags_0 },
    { 0xa7a84c71, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xdbe84c73, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8be84c75, SUC_lexicon_tags_10 },
    { 0xf9c84c76, SUC_lexicon_tags_12 },
    { 0x7a484c77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17484c7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e84c81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20684c87, SUC_lexicon_tags_11 },
    { 0xf3884c88, SUC_lexicon_tags_15 },
    { 0xc3684c88, SUC_lexicon_tags_13 },
    { 0xe1084c8a, SUC_lexicon_tags_25 },
    { 0x16a84c8b, SUC_lexicon_tags_25 },
    { 0x61a84c87, SUC_lexicon_tags_10 },
    { 0x37284c8d, SUC_lexicon_tags_7 },
    { 0xf2484c8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e84c94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de84c9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29084c9c, SUC_lexicon_tags_37 },
    { 0x9ac84c9d, SUC_lexicon_tags_47 },
    { 0x9ea84c9e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e84ca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c484ca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69284ca8, SUC_lexicon_tags_37 },
    { 0xdba84ca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9e84cab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50284cb3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x58084cb6, SUC_lexicon_tags_39 },
    { 0xcc084cb7, SUC_lexicon_tags_17 },
    { 0x2ac84cb8, SUC_lexicon_tags_118 },
    { 0xff284cb8, SUC_lexicon_tags_5 },
    { 0x8e084cba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a84cbd, SUC_lexicon_tags_11 },
    { 0xbb284cbe, SUC_lexicon_tags_31 },
    { 0x2de84cbe, SUC_lexicon_tags_2 },
    { 0x48484cbf, SUC_lexicon_tags_45 },
    { 0x39484cc1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x56e84cc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67684cc7, SUC_lexicon_tags_45 },
    { 0x7aa84cc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e84ccc, SUC_lexicon_tags_2 },
    { 0xe2e84ccc, SUC_lexicon_tags_37 },
    { 0x4c084ccc, SUC_lexicon_tags_26 },
    { 0xc7484ccf, SUC_lexicon_tags_48 },
    { 0x4684cd0, SUC_lexicon_tags_11 },
    { 0x22884ccf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x66a84cd3, SUC_lexicon_tags_12 },
    { 0x14a84cd4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x26284cd7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2284cda, SUC_lexicon_tags_17 },
    { 0xf3684cda, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a84cdf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8684ce2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc84ce8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x99c84cea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64684cee, SUC_lexicon_tags_25 },
    { 0x43284cee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a84cf4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1884cf7, SUC_lexicon_tags_12 },
    { 0x82084cf8, SUC_lexicon_tags_34 },
    { 0x2cc84cf9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade84cfd, SUC_lexicon_tags_28 },
    { 0xa1084cfd, SUC_lexicon_tags_2 },
    { 0xcc484cfe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb884d04, SUC_lexicon_tags_27 },
    { 0x90a84d05, SUC_lexicon_tags_38 },
    { 0x37c84d04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92084d08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f684d0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e84d19, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x84284d1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfd684d1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49a84d1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3684d21, SUC_lexicon_tags_17 },
    { 0x9ca84d21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a84d26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f684d28, SUC_lexicon_tags_15 },
    { 0x6b684d29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e484d2b, SUC_lexicon_tags_82 },
    { 0x39c84d2c, SUC_lexicon_tags_1 },
    { 0x4a684d2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c84d33, SUC_lexicon_tags_17 },
    { 0x1a84d33, SUC_lexicon_tags_37 },
    { 0xebe84d35, SUC_lexicon_tags_17 },
    { 0x87684d34, SUC_lexicon_tags_40 },
    { 0xfee84d33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d884d3e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac84d41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee84d46, SUC_lexicon_tags_11 },
    { 0x53084d47, SUC_lexicon_tags_63 },
    { 0xd7a84d47, SUC_lexicon_tags_137 },
    { 0x48484d48, SUC_lexicon_tags_43 },
    { 0x43884d4a, SUC_lexicon_tags_12 },
    { 0x16c84d48, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe84d4e, SUC_lexicon_tags_9 },
    { 0x13684d4f, SUC_lexicon_tags_68 },
    { 0xa3484d4f, SUC_lexicon_tags_32 },
    { 0x48684d51, SUC_lexicon_tags_23 },
    { 0xa7c84d52, SUC_lexicon_tags_12 },
    { 0xdbc84d52, SUC_lexicon_tags_63 },
    { 0x96084d53, SUC_lexicon_tags_22 },
    { 0xed084d55, SUC_lexicon_tags_6 },
    { 0xcb484d54, SUC_lexicon_tags_30 },
    { 0x15884d57, SUC_lexicon_tags_28 },
    { 0x26284d57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c284d5c, SUC_lexicon_tags_26 },
    { 0xac684d5d, SUC_lexicon_tags_28 },
    { 0x90c84d5d, SUC_lexicon_tags_25 },
    { 0x82c84d5f, SUC_lexicon_tags_14 },
    { 0x78284d5f, SUC_lexicon_tags_38 },
    { 0x84c84d61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c84d6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c84d71, SUC_lexicon_tags_25 },
    { 0xde084d72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9084d77, SUC_lexicon_tags_37 },
    { 0xe5284d78, SUC_lexicon_tags_15 },
    { 0x8fa84d79, SUC_lexicon_tags_35 },
    { 0x74284d7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e84d7f, SUC_lexicon_tags_11 },
    { 0xfa884d7f, SUC_lexicon_tags_6 },
    { 0xb6c84d80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69884d8a, SUC_lexicon_tags_37 },
    { 0x82484d8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc84d90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa2084d92, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a84d99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5084d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b484d9f, SUC_lexicon_tags_47 },
    { 0x3ea84da0, SUC_lexicon_tags_7 },
    { 0x2f884da1, SUC_lexicon_tags_29 },
    { 0x39084d9f, SUC_lexicon_tags_84 },
    { 0xaf884da3, SUC_lexicon_tags_7 },
    { 0x81684da3, SUC_lexicon_tags_47 },
    { 0x21a84da5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3084da8, SUC_lexicon_tags_41 },
    { 0xa7884da9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea84dac, SUC_lexicon_tags_19 },
    { 0x92884dac, SUC_lexicon_tags_27 },
    { 0xd4684dae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf684db7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x54284db9, SUC_lexicon_tags_45 },
    { 0xdbc84dba, SUC_lexicon_tags_21 },
    { 0x96c84dbb, SUC_lexicon_tags_14 },
    { 0x38284db9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6484dc4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc8e84dc6, SUC_lexicon_tags_0 },
    { 0xf1884dc7, SUC_lexicon_tags_0 },
    { 0xeaa84dc7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94084dcf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd4684dd1, SUC_lexicon_tags_17 },
    { 0xd684dd2, SUC_lexicon_tags_31 },
    { 0xea484dd2, SUC_lexicon_tags_15 },
    { 0xb1884dd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f084dd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e84ddc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee84de0, SUC_lexicon_tags_32 },
    { 0x51a84de1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xde84de4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53884df0, SUC_lexicon_tags_6 },
    { 0x16c84df1, SUC_lexicon_tags_0 },
    { 0x46c84df1, SUC_lexicon_tags_100 },
    { 0x28884df3, SUC_lexicon_tags_1 },
    { 0xf3e84df1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x82284df6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62a84df8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e084dfa, SUC_lexicon_tags_3 },
    { 0x7fc84dfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da84dfe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c84e03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a84e0b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa7e84e0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52484e13, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1484e16, SUC_lexicon_tags_47 },
    { 0xfdc84e17, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6484e1a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46884e1f, SUC_lexicon_tags_35 },
    { 0x12e84e20, SUC_lexicon_tags_61 },
    { 0x9084e20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8684e25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d484e28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc8a84e2a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x16c84e2c, SUC_lexicon_tags_15 },
    { 0x48a84e2d, SUC_lexicon_tags_2 },
    { 0x73084e2e, SUC_lexicon_tags_26 },
    { 0x89c84e2e, SUC_lexicon_tags_23 },
    { 0xd6884e2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34884e33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf8484e35, SUC_lexicon_tags_6 },
    { 0x7be84e36, SUC_lexicon_tags_45 },
    { 0x9b684e36, SUC_lexicon_tags_15 },
    { 0xf5c84e35, SUC_lexicon_tags_1 },
    { 0x94c84e36, SUC_lexicon_tags_7 },
    { 0x3b084e3a, SUC_lexicon_tags_17 },
    { 0xf5484e3b, SUC_lexicon_tags_38 },
    { 0x48884e3c, SUC_lexicon_tags_2 },
    { 0xb9484e3d, SUC_lexicon_tags_43 },
    { 0x75884e3d, SUC_lexicon_tags_65 },
    { 0x35684e3a, SUC_lexicon_tags_2 },
    { 0xa9c84e3e, SUC_lexicon_tags_23 },
    { 0x53684e3f, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be84e45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x83084e48, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x93a84e4a, SUC_lexicon_tags_46 },
    { 0x9b884e4b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28884e54, SUC_lexicon_tags_12 },
    { 0xe2c84e54, SUC_lexicon_tags_11 },
    { 0xa1a84e55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x53884e59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa84e5f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x39684e61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x76684e63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a84e66, SUC_lexicon_tags_65 },
    { 0x96a84e67, SUC_lexicon_tags_17 },
    { 0xffe84e68, SUC_lexicon_tags_17 },
    { 0x6ec84e69, SUC_lexicon_tags_17 },
    { 0xc4084e69, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa84e6e, SUC_lexicon_tags_15 },
    { 0x99684e6f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa6284e71, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf084e77, SUC_lexicon_tags_23 },
    { 0x1e484e78, SUC_lexicon_tags_1 },
    { 0x6ba84e78, SUC_lexicon_tags_25 },
    { 0x39684e78, SUC_lexicon_tags_1 },
    { 0xf0884e79, SUC_lexicon_tags_25 },
    { 0x6d684e7a, SUC_lexicon_tags_14 },
    { 0xb0a84e7d, SUC_lexicon_tags_25 },
    { 0x91884e7e, SUC_lexicon_tags_49 },
    { 0x42c84e7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe284e83, SUC_lexicon_tags_1 },
    { 0xea484e83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec84e89, SUC_lexicon_tags_69 },
    { 0x8e884e89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f484e8c, SUC_lexicon_tags_127 },
    { 0x80c84e8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6084e93, SUC_lexicon_tags_17 },
    { 0xd9484e94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f084e97, SUC_lexicon_tags_7 },
    { 0x9ba84e98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a84e9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad484ea3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x24684ea5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b684ea9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4e84eab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b084ead, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8684eb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8c84eb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfc84eb4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee84eba, SUC_lexicon_tags_31 },
    { 0x7da84eba, SUC_lexicon_tags_25 },
    { 0x90e84ebc, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7484ebf, SUC_lexicon_tags_18 },
    { 0xb8084ec0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e84ec3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91e84ec5, SUC_lexicon_tags_37 },
    { 0x68884ec6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1284ecd, SUC_lexicon_tags_60 },
    { 0x4f084ecd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda84ed5, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0x62484ed7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78e84ed9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a884edb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad684edd, SUC_lexicon_tags_11 },
    { 0xe3c84edd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31284ee5, SUC_lexicon_tags_3 },
    { 0x77284ee6, SUC_lexicon_tags_37 },
    { 0x1b684ee7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe484eeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d884eee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76484ef4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da84ef7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2684eff, SUC_lexicon_tags_15 },
    { 0x45284f00, SUC_lexicon_tags_1 },
    { 0x7b684eff, SUC_lexicon_tags_1 },
    { 0x93684f02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0284f07, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc84f0b, SUC_lexicon_tags_15 },
    { 0x1d884f0b, SUC_lexicon_tags_17 },
    { 0x95c84f0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc84f12, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd684f17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeca84f19, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xffe84f1b, SUC_lexicon_tags_2 },
    { 0x61e84f1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e884f1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e484f21, SUC_lexicon_tags_15 },
    { 0x18284f22, SUC_lexicon_tags_7 },
    { 0x2f284f23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5084f26, SUC_lexicon_tags_39 },
    { 0x3fc84f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a484f2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e84f30, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xeba84f32, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a84f35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89884f3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf684f40, SUC_lexicon_tags_33 },
    { 0x44684f40, SUC_lexicon_tags_45 },
    { 0xf5484f41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e84f48, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c84f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce084f4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5284f52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25284f55, SUC_lexicon_tags_60 },
    { 0xf8284f56, SUC_lexicon_tags_31 },
    { 0xb8e84f57, SUC_lexicon_tags_55 },
    { 0x6e084f58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ee84f5a, SUC_lexicon_tags_27 },
    { 0x9fc84f5b, SUC_lexicon_tags_37 },
    { 0xab484f5b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6e084f5e, SUC_lexicon_tags_12 },
    { 0x4d484f5f, SUC_lexicon_tags_57 },
    { 0x87684f5e, SUC_lexicon_tags_10 },
    { 0xb6084f61, SUC_lexicon_tags_23 },
    { 0x60a84f61, SUC_lexicon_tags_74 },
    { 0xb5484f62, SUC_lexicon_tags_37 },
    { 0xb1484f63, SUC_lexicon_tags_52 },
    { 0xcae84f65, SUC_lexicon_tags_79 },
    { 0x6c284f65, SUC_lexicon_tags_565 },
    { 0x85e84f64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc84f6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3084f6f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8e84f71, SUC_lexicon_tags_27 },
    { 0x9c84f71, SUC_lexicon_tags_26 },
    { 0x9b084f72, SUC_lexicon_tags_0 },
    { 0x4d884f71, SUC_lexicon_tags_11 },
    { 0xa8084f72, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbae84f77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93284f81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0484f83, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6684f8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb684f96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68284f99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76684f9f, SUC_lexicon_tags_7 },
    { 0x684fa0, SUC_lexicon_tags_25 },
    { 0x9c484fa1, SUC_lexicon_tags_23 },
    { 0xab484fa2, SUC_lexicon_tags_30 },
    { 0x13284fa3, SUC_lexicon_tags_8 },
    { 0x8ba84fa4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3e484fa6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x14684fa8, SUC_lexicon_tags_15 },
    { 0xab084fa9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4fc84fab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b084fae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd484fb0, SUC_lexicon_tags_9 },
    { 0x24884fb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75884fb3, SUC_lexicon_tags_27 },
    { 0x64884fb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74a84fb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39684fbe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x66884fc1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b484fc8, SUC_lexicon_tags_25 },
    { 0xeae84fc9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6dc84fcb, SUC_lexicon_tags_25 },
    { 0x51084fcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e84fd0, SUC_lexicon_tags_37 },
    { 0xac684fd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f884fd3, SUC_lexicon_tags_15 },
    { 0x60c84fd4, SUC_lexicon_tags_43 },
    { 0x45e84fd5, SUC_lexicon_tags_26 },
    { 0x9f084fd6, SUC_lexicon_tags_37 },
    { 0x86684fd7, SUC_lexicon_tags_37 },
    { 0xeca84fd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc84fe1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8a684fe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a84fe7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e84fed, SUC_lexicon_tags_15 },
    { 0xe5884fee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79084ff4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7684ff7, SUC_lexicon_tags_37 },
    { 0xae84ff7, SUC_lexicon_tags_47 },
    { 0x26884ff8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac84ffd, SUC_lexicon_tags_47 },
    { 0x2cc84ffd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf085002, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46285006, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92485008, SUC_lexicon_tags_45 },
    { 0x1c285009, SUC_lexicon_tags_25 },
    { 0x7ec8500a, SUC_lexicon_tags_25 },
    { 0xa0a8500b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d88500d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0485011, SUC_lexicon_tags_47 },
    { 0x9dc85011, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c85019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc88501c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f485021, SUC_lexicon_tags_25 },
    { 0xdda85021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa85026, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd08502c, SUC_lexicon_tags_7 },
    { 0x9928502d, SUC_lexicon_tags_11 },
    { 0xf5a8502d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a85031, SUC_lexicon_tags_43 },
    { 0xbc685032, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xab885035, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7988503b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e68503d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc308503f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6285043, SUC_lexicon_tags_56 },
    { 0x14285044, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcdc85046, SUC_lexicon_tags_40 },
    { 0x95085047, SUC_lexicon_tags_14 },
    { 0xc0a85048, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba8504c, SUC_lexicon_tags_17 },
    { 0x81c8504d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec885055, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc85059, SUC_lexicon_tags_0 },
    { 0x6268505a, SUC_lexicon_tags_6 },
    { 0x9e8505a, SUC_lexicon_tags_16 },
    { 0x9de85059, SUC_lexicon_tags_45 },
    { 0xc8e8505d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab68505f, SUC_lexicon_tags_0 },
    { 0x9d88505f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc85064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88485066, SUC_lexicon_tags_25 },
    { 0xcae85067, SUC_lexicon_tags_37 },
    { 0x57885068, SUC_lexicon_tags_14 },
    { 0xc8a85069, SUC_lexicon_tags_14 },
    { 0x6148506a, SUC_lexicon_tags_49 },
    { 0x9f485069, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7168506d, SUC_lexicon_tags_2 },
    { 0x3a48506e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e485070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3485073, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2885076, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b885079, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9de8507b, SUC_lexicon_tags_25 },
    { 0xc3a8507c, SUC_lexicon_tags_15 },
    { 0xe628507c, SUC_lexicon_tags_9 },
    { 0x2588507e, SUC_lexicon_tags_11 },
    { 0x5fa8507d, SUC_lexicon_tags_11 },
    { 0xbe08507c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x59285083, SUC_lexicon_tags_14 },
    { 0xa1285084, SUC_lexicon_tags_15 },
    { 0x9ce85085, SUC_lexicon_tags_1 },
    { 0x5da85086, SUC_lexicon_tags_21 },
    { 0xf9885083, SUC_lexicon_tags_25 },
    { 0x39c85084, SUC_lexicon_tags_37 },
    { 0x7da85088, SUC_lexicon_tags_26 },
    { 0x3d88508a, SUC_lexicon_tags_17 },
    { 0x6c8508a, SUC_lexicon_tags_2 },
    { 0xc168508c, SUC_lexicon_tags_81 },
    { 0xb9685087, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc85092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de85095, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c28509a, SUC_lexicon_tags_3 },
    { 0x6ec8509b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe668509d, SUC_lexicon_tags_15 },
    { 0xe768509e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec850a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58850a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c850ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea850b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0850b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782850be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe08850c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58850c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892850cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92c850ce, SUC_lexicon_tags_16 },
    { 0x384850cf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454850d3, SUC_lexicon_tags_66 },
    { 0x7d0850d3, SUC_lexicon_tags_27 },
    { 0x592850d5, SUC_lexicon_tags_14 },
    { 0xd2e850d5, SUC_lexicon_tags_1 },
    { 0x388850d5, SUC_lexicon_tags_25 },
    { 0xf2e850d8, SUC_lexicon_tags_38 },
    { 0xd0a850d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c850dc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbcc850de, SUC_lexicon_tags_1393 },
    { 0x9ea850df, SUC_lexicon_tags_17 },
    { 0xfce850df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70850e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4850f0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72850f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c850fa, SUC_lexicon_tags_43 },
    { 0x410850fa, SUC_lexicon_tags_68 },
    { 0xc9e850fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac85106, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x21285108, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb188510a, SUC_lexicon_tags_7 },
    { 0xcf48510b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc8510f, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xa6e85111, SUC_lexicon_tags_1 },
    { 0x3f685112, SUC_lexicon_tags_1 },
    { 0x50e85113, SUC_lexicon_tags_15 },
    { 0xbd685114, SUC_lexicon_tags_2 },
    { 0x1c285114, SUC_lexicon_tags_11 },
    { 0xe3085114, SUC_lexicon_tags_10 },
    { 0xeea85117, SUC_lexicon_tags_32 },
    { 0x9e685116, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf988511a, SUC_lexicon_tags_15 },
    { 0x9c88511a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76885121, SUC_lexicon_tags_37 },
    { 0xb685121, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908512a, SUC_lexicon_tags_83 },
    { 0x10c8512a, SUC_lexicon_tags_11 },
    { 0x2b88512a, SUC_lexicon_tags_25 },
    { 0xb5e8512d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49685131, SUC_lexicon_tags_160 },
    { 0x79885131, SUC_lexicon_tags_84 },
    { 0xb685132, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8a685135, SUC_lexicon_tags_55 },
    { 0xdf085136, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f68513b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d28513d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x45485140, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x44685142, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c085146, SUC_lexicon_tags_57 },
    { 0xe0c85147, SUC_lexicon_tags_61 },
    { 0xa3a85147, SUC_lexicon_tags_17 },
    { 0x6f285148, SUC_lexicon_tags_37 },
    { 0x9e88514a, SUC_lexicon_tags_0 },
    { 0xa5c8514b, SUC_lexicon_tags_2 },
    { 0xd308514b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb08514f, SUC_lexicon_tags_65 },
    { 0x4bc8514f, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xca885152, SUC_lexicon_tags_17 },
    { 0x7fc85153, SUC_lexicon_tags_37 },
    { 0x69885152, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86285158, SUC_lexicon_tags_25 },
    { 0x55685159, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x928515b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6108515e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69885165, SUC_lexicon_tags_65 },
    { 0x54c85166, SUC_lexicon_tags_45 },
    { 0x49e85167, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x87e85169, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc168516e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f485177, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb48517c, SUC_lexicon_tags_0 },
    { 0xd308517c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4085181, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12285183, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x46085185, SUC_lexicon_tags_56 },
    { 0x6dc85186, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5085189, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9308518c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5008518e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86685192, SUC_lexicon_tags_39 },
    { 0xc0685193, SUC_lexicon_tags_1 },
    { 0xff885194, SUC_lexicon_tags_12 },
    { 0x4de85195, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28485199, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1128519b, SUC_lexicon_tags_77 },
    { 0x2048519b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4851a4, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea851a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73c851ab, SUC_lexicon_tags_3 },
    { 0xf9a851ac, SUC_lexicon_tags_10 },
    { 0xcfa851ab, SUC_lexicon_tags_15 },
    { 0x990851ad, SUC_lexicon_tags_63 },
    { 0x2a4851af, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4851b2, SUC_lexicon_tags_2 },
    { 0xc7a851b3, SUC_lexicon_tags_3 },
    { 0xb1a851b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x808851b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440851bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa851bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x560851c1, SUC_lexicon_tags_15 },
    { 0xfd6851c2, SUC_lexicon_tags_33 },
    { 0x19e851c1, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xbb0851c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a851c8, SUC_lexicon_tags_43 },
    { 0x988851c9, SUC_lexicon_tags_14 },
    { 0x748851c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6851d2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb72851d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4851d7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2851da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c851df, SUC_lexicon_tags_87 },
    { 0xc7a851e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c851e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe20851e8, SUC_lexicon_tags_17 },
    { 0xde8851e9, SUC_lexicon_tags_4 },
    { 0xf7a851e8, SUC_lexicon_tags_1 },
    { 0xcda851eb, SUC_lexicon_tags_12 },
    { 0xc60851ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e851f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d6851f2, SUC_lexicon_tags_31 },
    { 0x6da851f3, SUC_lexicon_tags_11 },
    { 0xa58851f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc851f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x722851fc, SUC_lexicon_tags_15 },
    { 0x22c851fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x74285200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3085203, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x29e85205, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa85207, SUC_lexicon_tags_7 },
    { 0x93885208, SUC_lexicon_tags_11 },
    { 0xb2885209, SUC_lexicon_tags_17 },
    { 0x3a285209, SUC_lexicon_tags_37 },
    { 0x248520b, SUC_lexicon_tags_23 },
    { 0xb1e8520c, SUC_lexicon_tags_25 },
    { 0x908520c, SUC_lexicon_tags_40 },
    { 0x6108520c, SUC_lexicon_tags_25 },
    { 0x6068520a, SUC_lexicon_tags_39 },
    { 0x5e285210, SUC_lexicon_tags_15 },
    { 0x7588520e, SUC_lexicon_tags_0 },
    { 0x7948520d, SUC_lexicon_tags_17 },
    { 0x60085213, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe485216, SUC_lexicon_tags_15 },
    { 0x32885217, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea8521b, SUC_lexicon_tags_37 },
    { 0x98a8521c, SUC_lexicon_tags_14 },
    { 0xa108521d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9285220, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74685225, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc85228, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6f08522a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb428522c, SUC_lexicon_tags_9 },
    { 0x8228522d, SUC_lexicon_tags_38 },
    { 0x7168522e, SUC_lexicon_tags_23 },
    { 0x3c08522e, SUC_lexicon_tags_6 },
    { 0x45c85230, SUC_lexicon_tags_15 },
    { 0xda68522d, SUC_lexicon_tags_15 },
    { 0xf3e85231, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf908523d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6e8523f, SUC_lexicon_tags_11 },
    { 0xdca85240, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36e85242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a8524c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5885251, SUC_lexicon_tags_22 },
    { 0x87485252, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8e85254, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a85259, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb08525b, SUC_lexicon_tags_15 },
    { 0x708525c, SUC_lexicon_tags_25 },
    { 0x7088525c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe485261, SUC_lexicon_tags_17 },
    { 0xe885262, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x35885264, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93885268, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc948526a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb708526c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5d68526e, SUC_lexicon_tags_37 },
    { 0xea8526f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11a85271, SUC_lexicon_tags_60 },
    { 0x3c685272, SUC_lexicon_tags_68 },
    { 0xa3085272, SUC_lexicon_tags_11 },
    { 0xc2685272, SUC_lexicon_tags_25 },
    { 0x7c485275, SUC_lexicon_tags_1 },
    { 0xd8885272, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a85279, SUC_lexicon_tags_28 },
    { 0x8d085279, SUC_lexicon_tags_17 },
    { 0x7b28527b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x468527d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a085282, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3085286, SUC_lexicon_tags_37 },
    { 0x60685287, SUC_lexicon_tags_11 },
    { 0x6e285288, SUC_lexicon_tags_12 },
    { 0xca85289, SUC_lexicon_tags_7 },
    { 0x92885289, SUC_lexicon_tags_12 },
    { 0x3cc85289, SUC_lexicon_tags_15 },
    { 0x228528c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce85290, SUC_lexicon_tags_129 },
    { 0xdd085291, SUC_lexicon_tags_9 },
    { 0xd2285290, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a285295, SUC_lexicon_tags_11 },
    { 0x3ea85296, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a8529b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84852a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x766852a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60852a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210852ac, SUC_lexicon_tags_14 },
    { 0x222852ac, SUC_lexicon_tags_37 },
    { 0xd08852ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb42852b0, SUC_lexicon_tags_23 },
    { 0xb22852b0, SUC_lexicon_tags_60 },
    { 0xdd4852b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36852b6, SUC_lexicon_tags_12 },
    { 0x860852b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c852ba, SUC_lexicon_tags_25 },
    { 0x1f0852ba, SUC_lexicon_tags_77 },
    { 0x62c852bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872852c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c852c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274852cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x356852ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8852d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6852d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaa852d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2fc852da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2852e2, SUC_lexicon_tags_23 },
    { 0x360852e3, SUC_lexicon_tags_7 },
    { 0x21c852e4, SUC_lexicon_tags_2 },
    { 0xd3c852e5, SUC_lexicon_tags_7 },
    { 0xb0c852e3, SUC_lexicon_tags_37 },
    { 0x276852e7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a852ec, SUC_lexicon_tags_6 },
    { 0x22852ed, SUC_lexicon_tags_25 },
    { 0x948852ed, SUC_lexicon_tags_60 },
    { 0xbf6852ef, SUC_lexicon_tags_9 },
    { 0xdc0852f0, SUC_lexicon_tags_17 },
    { 0xef0852f0, SUC_lexicon_tags_27 },
    { 0xf78852f1, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda852f6, SUC_lexicon_tags_43 },
    { 0xc5c852f7, SUC_lexicon_tags_12 },
    { 0x6b0852f8, SUC_lexicon_tags_62 },
    { 0xf0a852f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8852fe, SUC_lexicon_tags_37 },
    { 0x3b4852fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x41885301, SUC_lexicon_tags_25 },
    { 0xd8c85302, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f685306, SUC_lexicon_tags_37 },
    { 0xb1c85306, SUC_lexicon_tags_23 },
    { 0x45a85308, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc5a8530a, SUC_lexicon_tags_19 },
    { 0xd568530a, SUC_lexicon_tags_230 },
    { 0x2468530c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d285312, SUC_lexicon_tags_14 },
    { 0x55485313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b885318, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d28531d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e85325, SUC_lexicon_tags_19 },
    { 0xf8e85326, SUC_lexicon_tags_37 },
    { 0xc7285325, SUC_lexicon_tags_25 },
    { 0xb3485328, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4088532a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6388532c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa85330, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb685338, SUC_lexicon_tags_26 },
    { 0xea685338, SUC_lexicon_tags_8 },
    { 0xed28533a, SUC_lexicon_tags_17 },
    { 0xf188533b, SUC_lexicon_tags_77 },
    { 0x87a8533b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf48533f, SUC_lexicon_tags_140 },
    { 0x99c85340, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9d285342, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xd6c85344, SUC_lexicon_tags_1 },
    { 0x11c85345, SUC_lexicon_tags_188 },
    { 0x34e85344, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c8534c, SUC_lexicon_tags_9 },
    { 0xcb88534d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61485351, SUC_lexicon_tags_38 },
    { 0x12885352, SUC_lexicon_tags_26 },
    { 0x48485352, SUC_lexicon_tags_11 },
    { 0xac485354, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x54a85356, SUC_lexicon_tags_7 },
    { 0x36285357, SUC_lexicon_tags_21 },
    { 0xcd685357, SUC_lexicon_tags_47 },
    { 0x95685359, SUC_lexicon_tags_15 },
    { 0x3668535a, SUC_lexicon_tags_1 },
    { 0x3588535a, SUC_lexicon_tags_29 },
    { 0xd768535c, SUC_lexicon_tags_522 },
    { 0xd1a8535b, SUC_lexicon_tags_26 },
    { 0xa08535e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e885360, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69685363, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e85369, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18885372, SUC_lexicon_tags_28 },
    { 0x2a85373, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xde85376, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x70a85378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9888537b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd488537d, SUC_lexicon_tags_37 },
    { 0x59e8537d, SUC_lexicon_tags_35 },
    { 0xf728537d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93885381, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e85385, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c8538a, SUC_lexicon_tags_37 },
    { 0x7048538b, SUC_lexicon_tags_15 },
    { 0x3a88538b, SUC_lexicon_tags_8 },
    { 0xc008538d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x46685390, SUC_lexicon_tags_15 },
    { 0x8b285390, SUC_lexicon_tags_25 },
    { 0x19a85392, SUC_lexicon_tags_45 },
    { 0x98085391, SUC_lexicon_tags_25 },
    { 0xbba85391, SUC_lexicon_tags_15 },
    { 0x17485395, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a85397, SUC_lexicon_tags_21 },
    { 0x9cc85398, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc1a8539a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf088539e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42853a1, SUC_lexicon_tags_79 },
    { 0xf3a853a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2853ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e853b1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xce853b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x68e853b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca853b9, SUC_lexicon_tags_38 },
    { 0xa0e853ba, SUC_lexicon_tags_10 },
    { 0xe02853bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x730853bd, SUC_lexicon_tags_38 },
    { 0xc80853be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c853c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634853c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ce853ca, SUC_lexicon_tags_0 },
    { 0xfb0853cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf16853cd, SUC_lexicon_tags_1 },
    { 0x6a2853ce, SUC_lexicon_tags_37 },
    { 0xb0853ce, SUC_lexicon_tags_8 },
    { 0xc3e853d0, SUC_lexicon_tags_32 },
    { 0x88c853d0, SUC_lexicon_tags_1 },
    { 0x48a853d2, SUC_lexicon_tags_87 },
    { 0x96853d3, SUC_lexicon_tags_15 },
    { 0xf5a853d2, SUC_lexicon_tags_21 },
    { 0x5aa853d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6853d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2853dc, SUC_lexicon_tags_47 },
    { 0x5a2853dd, SUC_lexicon_tags_13 },
    { 0x182853de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74c853e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e853e3, SUC_lexicon_tags_1 },
    { 0x7f8853e4, SUC_lexicon_tags_11 },
    { 0xafe853e4, SUC_lexicon_tags_19 },
    { 0x8c6853e4, SUC_lexicon_tags_91 },
    { 0xea0853e7, SUC_lexicon_tags_17 },
    { 0xa5e853e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e853eb, SUC_lexicon_tags_11 },
    { 0x7a4853eb, SUC_lexicon_tags_11 },
    { 0xcd0853ed, SUC_lexicon_tags_8 },
    { 0x7c853eb, SUC_lexicon_tags_12 },
    { 0x9e8853eb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78853f4, SUC_lexicon_tags_15 },
    { 0xe0c853f5, SUC_lexicon_tags_17 },
    { 0xece853f6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcaa853f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e885403, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43485407, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e08540c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99085415, SUC_lexicon_tags_51 },
    { 0x5e485415, SUC_lexicon_tags_2 },
    { 0x67085416, SUC_lexicon_tags_27 },
    { 0x75885418, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed68541f, SUC_lexicon_tags_2 },
    { 0xeb28541f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4885424, SUC_lexicon_tags_27 },
    { 0xc4685424, SUC_lexicon_tags_0 },
    { 0x85885426, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41485429, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5628542f, SUC_lexicon_tags_28 },
    { 0x43085430, SUC_lexicon_tags_13 },
    { 0x24c85431, SUC_lexicon_tags_25 },
    { 0xf1e85431, SUC_lexicon_tags_15 },
    { 0xa4685433, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xff085436, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cc85438, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed68543a, SUC_lexicon_tags_47 },
    { 0x7b68543b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5068543e, SUC_lexicon_tags_25 },
    { 0x1668543f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x97485441, SUC_lexicon_tags_1 },
    { 0xd9a85442, SUC_lexicon_tags_63 },
    { 0x33c85442, SUC_lexicon_tags_37 },
    { 0xfc685444, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xad885447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6348544a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff28544e, SUC_lexicon_tags_13 },
    { 0x14a8544e, SUC_lexicon_tags_26 },
    { 0x3928544e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61685452, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b68545b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2a8545d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c485469, SUC_lexicon_tags_37 },
    { 0xb6485469, SUC_lexicon_tags_38 },
    { 0xf128546b, SUC_lexicon_tags_0 },
    { 0xf128546c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45685472, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x65685474, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6685479, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb688547b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e8547f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef885483, SUC_lexicon_tags_4 },
    { 0x3e285483, SUC_lexicon_tags_37 },
    { 0x1c85485, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52885488, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e8548c, SUC_lexicon_tags_37 },
    { 0xcf08548d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36085494, SUC_lexicon_tags_1 },
    { 0xb9c85495, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e85498, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a8549d, SUC_lexicon_tags_7 },
    { 0x6088549d, SUC_lexicon_tags_2 },
    { 0xc3a8549f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e854a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a854a5, SUC_lexicon_tags_1 },
    { 0x420854a6, SUC_lexicon_tags_8 },
    { 0xb9c854a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2854aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a854ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52854b0, SUC_lexicon_tags_37 },
    { 0x14c854b1, SUC_lexicon_tags_0 },
    { 0x554854b2, SUC_lexicon_tags_30 },
    { 0x432854b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6854bc, SUC_lexicon_tags_37 },
    { 0xe8a854bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c854c0, SUC_lexicon_tags_15 },
    { 0x7f0854c0, SUC_lexicon_tags_37 },
    { 0x6e0854c1, SUC_lexicon_tags_1 },
    { 0xd6c854c3, SUC_lexicon_tags_47 },
    { 0x340854c4, SUC_lexicon_tags_2 },
    { 0x802854c5, SUC_lexicon_tags_12 },
    { 0x27c854c5, SUC_lexicon_tags_26 },
    { 0x2f2854c3, SUC_lexicon_tags_2 },
    { 0x76854c6, SUC_lexicon_tags_65 },
    { 0x8e0854c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4854d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe10854d4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2c4854d6, SUC_lexicon_tags_19 },
    { 0xc94854d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36854da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e854de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2854e1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04854e7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xfc8854e9, SUC_lexicon_tags_27 },
    { 0x3ae854ea, SUC_lexicon_tags_25 },
    { 0xfcc854ea, SUC_lexicon_tags_25 },
    { 0x7a0854ec, SUC_lexicon_tags_17 },
    { 0x900854ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0c854ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x606854f1, SUC_lexicon_tags_10 },
    { 0xcac854f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3de854f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2854f9, SUC_lexicon_tags_12 },
    { 0xb6e854f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c485501, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c85506, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a85509, SUC_lexicon_tags_66 },
    { 0xab48550a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a8550d, SUC_lexicon_tags_77 },
    { 0x7708550e, SUC_lexicon_tags_7 },
    { 0xff88550e, SUC_lexicon_tags_9 },
    { 0xc4c8550d, SUC_lexicon_tags_1 },
    { 0xf7a85511, SUC_lexicon_tags_7 },
    { 0x11085511, SUC_lexicon_tags_12 },
    { 0x5728550e, SUC_lexicon_tags_26 },
    { 0x1e48550d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8685519, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4968551d, SUC_lexicon_tags_159 },
    { 0xbc28551e, SUC_lexicon_tags_2 },
    { 0xafc8551f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a85529, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca8552d, SUC_lexicon_tags_12 },
    { 0x488552e, SUC_lexicon_tags_26 },
    { 0xabc8552f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84285531, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c85536, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x48485539, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e8553c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e85542, SUC_lexicon_tags_15 },
    { 0xfce85543, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28e85545, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x4fe85547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c8554a, SUC_lexicon_tags_63 },
    { 0x3528554b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e85551, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f285553, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c85559, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcce8555b, SUC_lexicon_tags_37 },
    { 0xe1e8555c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23085565, SUC_lexicon_tags_7 },
    { 0x52885565, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2285569, SUC_lexicon_tags_17 },
    { 0x40a85569, SUC_lexicon_tags_9 },
    { 0xef08556a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c85574, SUC_lexicon_tags_25 },
    { 0xe1c85575, SUC_lexicon_tags_15 },
    { 0xf2a85576, SUC_lexicon_tags_0 },
    { 0xffa85575, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdac85579, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae8557c, SUC_lexicon_tags_45 },
    { 0xf948557d, SUC_lexicon_tags_37 },
    { 0x6128557c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50085580, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc485584, SUC_lexicon_tags_63 },
    { 0x3e285585, SUC_lexicon_tags_49 },
    { 0xafc85586, SUC_lexicon_tags_23 },
    { 0x33485586, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f08558a, SUC_lexicon_tags_47 },
    { 0xea28558a, SUC_lexicon_tags_2 },
    { 0xeb48558c, SUC_lexicon_tags_8 },
    { 0x2d88558d, SUC_lexicon_tags_9 },
    { 0x748558c, SUC_lexicon_tags_12 },
    { 0x62a8558f, SUC_lexicon_tags_29 },
    { 0x76c85590, SUC_lexicon_tags_17 },
    { 0xdcc8558f, SUC_lexicon_tags_27 },
    { 0x3fe85592, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4685596, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x20c85598, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8559b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42855a1, SUC_lexicon_tags_42 },
    { 0x52c855a2, SUC_lexicon_tags_0 },
    { 0xc40855a3, SUC_lexicon_tags_36 },
    { 0x30855a2, SUC_lexicon_tags_26 },
    { 0x120855a5, SUC_lexicon_tags_8 },
    { 0xc0855a1, SUC_lexicon_tags_17 },
    { 0x474855a7, SUC_lexicon_tags_19 },
    { 0x60c855a6, SUC_lexicon_tags_27 },
    { 0x894855a9, SUC_lexicon_tags_17 },
    { 0x8f0855a3, SUC_lexicon_tags_29 },
    { 0x1ec855ab, SUC_lexicon_tags_40 },
    { 0x56855ac, SUC_lexicon_tags_39 },
    { 0xada855ab, SUC_lexicon_tags_29 },
    { 0xc44855ad, SUC_lexicon_tags_19 },
    { 0xd5c855af, SUC_lexicon_tags_11 },
    { 0x8e855b0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2855b3, SUC_lexicon_tags_25 },
    { 0x3e4855b4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0855b9, SUC_lexicon_tags_46 },
    { 0x620855ba, SUC_lexicon_tags_107 },
    { 0xca4855b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58a855bd, SUC_lexicon_tags_6 },
    { 0x66a855be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25e855c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e855c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08855c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x346855c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96855cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2855ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644855d5, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e855d9, SUC_lexicon_tags_25 },
    { 0x8a8855d9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x558855dd, SUC_lexicon_tags_1 },
    { 0x16a855de, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x72855e0, SUC_lexicon_tags_6 },
    { 0x314855e1, SUC_lexicon_tags_2 },
    { 0xca4855e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c8855e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x728855e6, SUC_lexicon_tags_15 },
    { 0xda2855e7, SUC_lexicon_tags_32 },
    { 0xe82855e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a855eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec855ee, SUC_lexicon_tags_11 },
    { 0xbca855ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c855f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c855f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3285600, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11885604, SUC_lexicon_tags_11 },
    { 0x49885605, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1588560a, SUC_lexicon_tags_12 },
    { 0x28e8560b, SUC_lexicon_tags_7 },
    { 0xaec8560b, SUC_lexicon_tags_1 },
    { 0x2408560a, SUC_lexicon_tags_37 },
    { 0xcce8560e, SUC_lexicon_tags_236 },
    { 0xea88560f, SUC_lexicon_tags_79 },
    { 0x91085610, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x68085613, SUC_lexicon_tags_17 },
    { 0x9a885614, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7b085616, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x39a85618, SUC_lexicon_tags_38 },
    { 0xc9285619, SUC_lexicon_tags_15 },
    { 0x1448561a, SUC_lexicon_tags_60 },
    { 0x66c8561a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2648561d, SUC_lexicon_tags_2 },
    { 0x1388561e, SUC_lexicon_tags_25 },
    { 0xa6a8561f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0085621, SUC_lexicon_tags_2 },
    { 0x6b885622, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82285627, SUC_lexicon_tags_7 },
    { 0x57c85628, SUC_lexicon_tags_30 },
    { 0x34085629, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe548562b, SUC_lexicon_tags_28 },
    { 0x7c8562c, SUC_lexicon_tags_4 },
    { 0xd8a8562d, SUC_lexicon_tags_37 },
    { 0xdc68562e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ce85630, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65085632, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4085635, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd885637, SUC_lexicon_tags_56 },
    { 0x3aa85638, SUC_lexicon_tags_65 },
    { 0x39485639, SUC_lexicon_tags_15 },
    { 0xf9285639, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdc8563c, SUC_lexicon_tags_7 },
    { 0x4cc8563d, SUC_lexicon_tags_30 },
    { 0x2b88563e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b685643, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x23a85645, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x75085647, SUC_lexicon_tags_37 },
    { 0x7b085648, SUC_lexicon_tags_34 },
    { 0x19c85649, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f08564b, SUC_lexicon_tags_27 },
    { 0xb188564c, SUC_lexicon_tags_17 },
    { 0xe68564b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c85650, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc85653, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c85656, SUC_lexicon_tags_47 },
    { 0xd2885657, SUC_lexicon_tags_65 },
    { 0x4285658, SUC_lexicon_tags_37 },
    { 0x51e85659, SUC_lexicon_tags_11 },
    { 0x82a85659, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b68565d, SUC_lexicon_tags_15 },
    { 0x6268565e, SUC_lexicon_tags_17 },
    { 0xd5c8565e, SUC_lexicon_tags_47 },
    { 0xace8565d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd885663, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a85666, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a685669, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8108566c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef88566f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75285671, SUC_lexicon_tags_25 },
    { 0xd1285671, SUC_lexicon_tags_384 },
    { 0, NULL },
    { 0, NULL },
    { 0xf285675, SUC_lexicon_tags_74 },
    { 0x77e85676, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e085679, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6b88567b, SUC_lexicon_tags_37 },
    { 0xdd48567c, SUC_lexicon_tags_23 },
    { 0xe508567c, SUC_lexicon_tags_25 },
    { 0x5d28567e, SUC_lexicon_tags_15 },
    { 0xe7a8567c, SUC_lexicon_tags_25 },
    { 0x5c08567d, SUC_lexicon_tags_25 },
    { 0x5ca85680, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67285686, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a8568c, SUC_lexicon_tags_23 },
    { 0xea88568d, SUC_lexicon_tags_49 },
    { 0x2ca8568d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e85691, SUC_lexicon_tags_11 },
    { 0x82685692, SUC_lexicon_tags_27 },
    { 0x69885693, SUC_lexicon_tags_12 },
    { 0xb7a85693, SUC_lexicon_tags_17 },
    { 0x12485695, SUC_lexicon_tags_25 },
    { 0x92485693, SUC_lexicon_tags_5 },
    { 0x5a485697, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28856a1, SUC_lexicon_tags_11 },
    { 0x6a2856a1, SUC_lexicon_tags_32 },
    { 0x676856a1, SUC_lexicon_tags_0 },
    { 0xcb0856a4, SUC_lexicon_tags_7 },
    { 0xf02856a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398856a9, SUC_lexicon_tags_1 },
    { 0x8d8856a9, SUC_lexicon_tags_9 },
    { 0xc72856ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba856ae, SUC_lexicon_tags_37 },
    { 0x8da856af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe856b2, SUC_lexicon_tags_64 },
    { 0xd8c856b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x646856b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf3c856b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6856bb, SUC_lexicon_tags_25 },
    { 0xb32856bc, SUC_lexicon_tags_47 },
    { 0xabe856bb, SUC_lexicon_tags_43 },
    { 0x3c8856be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x162856c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718856c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524856cc, SUC_lexicon_tags_15 },
    { 0x258856cc, SUC_lexicon_tags_37 },
    { 0x3b0856ce, SUC_lexicon_tags_91 },
    { 0x16a856cf, SUC_lexicon_tags_25 },
    { 0x134856cf, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e856d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4856d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6856dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6c856de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4856e2, SUC_lexicon_tags_31 },
    { 0x2b4856e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdfc856e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef4856e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xca4856e9, SUC_lexicon_tags_60 },
    { 0x290856e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfb6856ec, SUC_lexicon_tags_14 },
    { 0x604856ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600856f1, SUC_lexicon_tags_55 },
    { 0x634856f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x258856f4, SUC_lexicon_tags_11 },
    { 0xeb2856f4, SUC_lexicon_tags_1 },
    { 0x434856f6, SUC_lexicon_tags_11 },
    { 0xd58856f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0856fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66885709, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3a8570b, SUC_lexicon_tags_60 },
    { 0x90c8570c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa2a8570e, SUC_lexicon_tags_17 },
    { 0x1f48570f, SUC_lexicon_tags_25 },
    { 0x68085710, SUC_lexicon_tags_43 },
    { 0x43885711, SUC_lexicon_tags_9 },
    { 0xb3a85712, SUC_lexicon_tags_45 },
    { 0xc8c85712, SUC_lexicon_tags_34 },
    { 0xf8c85714, SUC_lexicon_tags_26 },
    { 0x8e285715, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f28571a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e885725, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40a85727, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88c85729, SUC_lexicon_tags_60 },
    { 0x7408572a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79885731, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c485737, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c08573d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf485741, SUC_lexicon_tags_6 },
    { 0x41885742, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70285746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52285748, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41c8574a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b68574d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x80e8574f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d485751, SUC_lexicon_tags_19 },
    { 0x42a85751, SUC_lexicon_tags_43 },
    { 0x11885753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3685755, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e85759, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5528575c, SUC_lexicon_tags_13 },
    { 0x7568575d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b08575f, SUC_lexicon_tags_17 },
    { 0x828575f, SUC_lexicon_tags_1371 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c85766, SUC_lexicon_tags_25 },
    { 0x43e85767, SUC_lexicon_tags_65 },
    { 0xf2c85768, SUC_lexicon_tags_19 },
    { 0x66e85769, SUC_lexicon_tags_14 },
    { 0xc0085767, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1828576f, SUC_lexicon_tags_7 },
    { 0x89285770, SUC_lexicon_tags_17 },
    { 0xe6c85770, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9285779, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac28577b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8577e, SUC_lexicon_tags_30 },
    { 0x4da8577f, SUC_lexicon_tags_29 },
    { 0x96c85780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3085788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d68578b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a8578f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36285792, SUC_lexicon_tags_17 },
    { 0x7cc85793, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9685797, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c8579d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c857a4, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac857a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb1a857ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e857ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7d0857b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400857b6, SUC_lexicon_tags_15 },
    { 0x1d8857b7, SUC_lexicon_tags_30 },
    { 0x2a0857b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbbe857ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31c857bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x530857bf, SUC_lexicon_tags_17 },
    { 0xc68857bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c857c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a857ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e857cc, SUC_lexicon_tags_2 },
    { 0x1cc857cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8857d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44857d7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca857db, SUC_lexicon_tags_2 },
    { 0xe5e857db, SUC_lexicon_tags_53 },
    { 0xb78857db, SUC_lexicon_tags_37 },
    { 0x70a857dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x528857e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04857e3, SUC_lexicon_tags_25 },
    { 0x174857e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x857e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4d4857e8, SUC_lexicon_tags_7 },
    { 0xc2a857e9, SUC_lexicon_tags_7 },
    { 0xda857e9, SUC_lexicon_tags_12 },
    { 0x466857eb, SUC_lexicon_tags_7 },
    { 0x652857ec, SUC_lexicon_tags_65 },
    { 0x406857ed, SUC_lexicon_tags_48 },
    { 0x232857ee, SUC_lexicon_tags_7 },
    { 0x342857ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a8857f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x672857f3, SUC_lexicon_tags_14 },
    { 0x5d8857f4, SUC_lexicon_tags_7 },
    { 0xdf2857f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ce857f7, SUC_lexicon_tags_2 },
    { 0xad8857f8, SUC_lexicon_tags_8 },
    { 0xc96857f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b0857fb, SUC_lexicon_tags_49 },
    { 0xbc6857fc, SUC_lexicon_tags_25 },
    { 0x498857fd, SUC_lexicon_tags_17 },
    { 0x186857fe, SUC_lexicon_tags_11 },
    { 0x530857fe, SUC_lexicon_tags_25 },
    { 0x87885800, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea85806, SUC_lexicon_tags_15 },
    { 0xd4e85807, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a8580d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e85814, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6be85816, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a8581e, SUC_lexicon_tags_68 },
    { 0xa348581f, SUC_lexicon_tags_31 },
    { 0xda085820, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f885822, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x58085824, SUC_lexicon_tags_10 },
    { 0x58c85825, SUC_lexicon_tags_60 },
    { 0x9e485826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1485829, SUC_lexicon_tags_2 },
    { 0x71285829, SUC_lexicon_tags_11 },
    { 0xc068582b, SUC_lexicon_tags_35 },
    { 0xbda8582c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5688582f, SUC_lexicon_tags_37 },
    { 0x19885830, SUC_lexicon_tags_27 },
    { 0x65285830, SUC_lexicon_tags_6 },
    { 0x31485830, SUC_lexicon_tags_49 },
    { 0x94885833, SUC_lexicon_tags_7 },
    { 0x4e485833, SUC_lexicon_tags_17 },
    { 0x1ce85835, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcee85837, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29285839, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c8583f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4e085841, SUC_lexicon_tags_7 },
    { 0xca885841, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5885845, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c85849, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9548584b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8c8584d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xc188584f, SUC_lexicon_tags_17 },
    { 0x1ac8584f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ca85852, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe85857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32485860, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba85864, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c85867, SUC_lexicon_tags_25 },
    { 0x8b685867, SUC_lexicon_tags_33 },
    { 0x17885869, SUC_lexicon_tags_33 },
    { 0xc1a85869, SUC_lexicon_tags_173 },
    { 0xc428586b, SUC_lexicon_tags_19 },
    { 0xc948586a, SUC_lexicon_tags_8 },
    { 0xf4c8586c, SUC_lexicon_tags_6 },
    { 0x8b48586b, SUC_lexicon_tags_25 },
    { 0x57c8586c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe3285871, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a85874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468587a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a08587e, SUC_lexicon_tags_134 },
    { 0x1768587f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4685884, SUC_lexicon_tags_7 },
    { 0xd7885885, SUC_lexicon_tags_26 },
    { 0x1f285886, SUC_lexicon_tags_17 },
    { 0x8485885, SUC_lexicon_tags_1 },
    { 0xc1c85888, SUC_lexicon_tags_5 },
    { 0x8d085886, SUC_lexicon_tags_60 },
    { 0xde68588a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9568588d, SUC_lexicon_tags_15 },
    { 0x1b28588d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1685891, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d885896, SUC_lexicon_tags_60 },
    { 0xbc85897, SUC_lexicon_tags_7 },
    { 0x3b285898, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a88589b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb088589d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x792858a0, SUC_lexicon_tags_15 },
    { 0x9e8858a1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4858a4, SUC_lexicon_tags_30 },
    { 0x310858a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442858a9, SUC_lexicon_tags_14 },
    { 0xcb0858aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e858ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x682858af, SUC_lexicon_tags_33 },
    { 0x4a858b0, SUC_lexicon_tags_17 },
    { 0xa42858b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdde858b3, SUC_lexicon_tags_2 },
    { 0x5c8858b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea858b9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd5a858bb, SUC_lexicon_tags_23 },
    { 0x6be858bc, SUC_lexicon_tags_6 },
    { 0xa3e858bd, SUC_lexicon_tags_8 },
    { 0x526858be, SUC_lexicon_tags_27 },
    { 0x6d4858bb, SUC_lexicon_tags_17 },
    { 0xf7a858c0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24858c3, SUC_lexicon_tags_43 },
    { 0x458858c4, SUC_lexicon_tags_25 },
    { 0xdc6858c4, SUC_lexicon_tags_37 },
    { 0x61a858c5, SUC_lexicon_tags_7 },
    { 0x74a858c7, SUC_lexicon_tags_17 },
    { 0x462858c8, SUC_lexicon_tags_12 },
    { 0x13a858c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2858ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10858d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x644858d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdc858d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c858e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2858e3, SUC_lexicon_tags_8 },
    { 0xce858e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34858e8, SUC_lexicon_tags_25 },
    { 0x2f4858e9, SUC_lexicon_tags_14 },
    { 0xeae858ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5cc858ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x278858ee, SUC_lexicon_tags_61 },
    { 0x2fa858ee, SUC_lexicon_tags_37 },
    { 0xf0c858ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4858f3, SUC_lexicon_tags_15 },
    { 0x7a8858f4, SUC_lexicon_tags_15 },
    { 0x352858f4, SUC_lexicon_tags_25 },
    { 0x208858f6, SUC_lexicon_tags_0 },
    { 0x536858f6, SUC_lexicon_tags_11 },
    { 0xf6858f4, SUC_lexicon_tags_15 },
    { 0x286858f8, SUC_lexicon_tags_37 },
    { 0x46e858fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc858fd, SUC_lexicon_tags_11 },
    { 0xdc6858fe, SUC_lexicon_tags_37 },
    { 0x21e858fd, SUC_lexicon_tags_8 },
    { 0xf9e85900, SUC_lexicon_tags_7 },
    { 0xa8e85901, SUC_lexicon_tags_7 },
    { 0x6ca85901, SUC_lexicon_tags_17 },
    { 0x4ce85903, SUC_lexicon_tags_5 },
    { 0x15e85902, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54e85906, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8a85908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc8590f, SUC_lexicon_tags_60 },
    { 0xfde85910, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbba85912, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc85916, SUC_lexicon_tags_140 },
    { 0xa0c85916, SUC_lexicon_tags_11 },
    { 0x6b485917, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6928591f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba85923, SUC_lexicon_tags_56 },
    { 0xc5285924, SUC_lexicon_tags_49 },
    { 0x93685925, SUC_lexicon_tags_62 },
    { 0x89685924, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6c85928, SUC_lexicon_tags_17 },
    { 0xa5685929, SUC_lexicon_tags_11 },
    { 0x30a85929, SUC_lexicon_tags_17 },
    { 0xe8685928, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c85932, SUC_lexicon_tags_23 },
    { 0x83e85933, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf285935, SUC_lexicon_tags_63 },
    { 0x7ae85935, SUC_lexicon_tags_29 },
    { 0xd1c85937, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc8593f, SUC_lexicon_tags_12 },
    { 0xba68593f, SUC_lexicon_tags_37 },
    { 0x82485941, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5fe85943, SUC_lexicon_tags_11 },
    { 0x2f085944, SUC_lexicon_tags_1 },
    { 0x5f685945, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd368594c, SUC_lexicon_tags_15 },
    { 0x20a8594d, SUC_lexicon_tags_47 },
    { 0x1968594d, SUC_lexicon_tags_25 },
    { 0xdc88594f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d885955, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8085957, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99085959, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65485968, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c85973, SUC_lexicon_tags_33 },
    { 0x12285974, SUC_lexicon_tags_1 },
    { 0xac285973, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae85980, SUC_lexicon_tags_37 },
    { 0x6c85981, SUC_lexicon_tags_31 },
    { 0x7d085980, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae85985, SUC_lexicon_tags_37 },
    { 0x61485986, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b48598a, SUC_lexicon_tags_6 },
    { 0x7348598b, SUC_lexicon_tags_14 },
    { 0xaa88598a, SUC_lexicon_tags_230 },
    { 0x62a8598c, SUC_lexicon_tags_37 },
    { 0x5ec8598d, SUC_lexicon_tags_68 },
    { 0x8d68598d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b685991, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62285995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2e85997, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb988599b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa9a8599d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe308599f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e859a4, SUC_lexicon_tags_49 },
    { 0x9a4859a5, SUC_lexicon_tags_29 },
    { 0x96c859a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xacc859a8, SUC_lexicon_tags_15 },
    { 0xd50859a8, SUC_lexicon_tags_37 },
    { 0xe78859aa, SUC_lexicon_tags_87 },
    { 0xcba859ab, SUC_lexicon_tags_17 },
    { 0xaaa859a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x274859af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x134859b1, SUC_lexicon_tags_10 },
    { 0xc92859b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a859b6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x566859b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00859bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88859c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x16a859c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a859ca, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x724859cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x690859cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8859d2, SUC_lexicon_tags_14 },
    { 0x33c859d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0859dc, SUC_lexicon_tags_19 },
    { 0x110859dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928859e2, SUC_lexicon_tags_1 },
    { 0xa4e859e3, SUC_lexicon_tags_7 },
    { 0x5f4859e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc859e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2859ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6859f4, SUC_lexicon_tags_19 },
    { 0x2ca859f5, SUC_lexicon_tags_23 },
    { 0x18859f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ec859f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa80859fa, SUC_lexicon_tags_63 },
    { 0xfa0859fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1a859fd, SUC_lexicon_tags_17 },
    { 0xc00859fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc85a05, SUC_lexicon_tags_37 },
    { 0xc5485a06, SUC_lexicon_tags_0 },
    { 0xa7885a07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee685a0a, SUC_lexicon_tags_37 },
    { 0x33685a0b, SUC_lexicon_tags_94 },
    { 0xf6285a0b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff285a10, SUC_lexicon_tags_1 },
    { 0x14c85a10, SUC_lexicon_tags_15 },
    { 0x31e85a12, SUC_lexicon_tags_8 },
    { 0xf885a10, SUC_lexicon_tags_7 },
    { 0x58285a13, SUC_lexicon_tags_25 },
    { 0x3e285a11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1e85a17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42085a1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc85a1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a285a25, SUC_lexicon_tags_0 },
    { 0x1f285a26, SUC_lexicon_tags_37 },
    { 0x33685a27, SUC_lexicon_tags_12 },
    { 0x8a85a28, SUC_lexicon_tags_9 },
    { 0x86085a28, SUC_lexicon_tags_11 },
    { 0x55685a28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b285a2d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88285a36, SUC_lexicon_tags_11 },
    { 0x41885a36, SUC_lexicon_tags_15 },
    { 0x79485a38, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x58685a3b, SUC_lexicon_tags_11 },
    { 0x35685a3b, SUC_lexicon_tags_29 },
    { 0x4e685a3b, SUC_lexicon_tags_37 },
    { 0x10485a3e, SUC_lexicon_tags_25 },
    { 0xff685a3f, SUC_lexicon_tags_15 },
    { 0xd7c85a40, SUC_lexicon_tags_0 },
    { 0x32885a3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98085a43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa85a49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4685a4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83885a4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa85a53, SUC_lexicon_tags_25 },
    { 0x3c885a54, SUC_lexicon_tags_37 },
    { 0x22085a55, SUC_lexicon_tags_38 },
    { 0x56885a56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23085a5a, SUC_lexicon_tags_1 },
    { 0x68885a5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b885a61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7485a63, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xc6885a65, SUC_lexicon_tags_14 },
    { 0x63a85a66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7485a6e, SUC_lexicon_tags_83 },
    { 0x1cc85a6e, SUC_lexicon_tags_37 },
    { 0x2d085a70, SUC_lexicon_tags_11 },
    { 0x34c85a71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88085a74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89085a76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a85a84, SUC_lexicon_tags_6 },
    { 0x18685a85, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee85a88, SUC_lexicon_tags_56 },
    { 0x34085a89, SUC_lexicon_tags_15 },
    { 0xa8c85a8a, SUC_lexicon_tags_15 },
    { 0x90285a8b, SUC_lexicon_tags_1 },
    { 0xdb885a8b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e885a8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90a85a91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10285a9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c85aa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18285aa8, SUC_lexicon_tags_34 },
    { 0xbea85aa8, SUC_lexicon_tags_29 },
    { 0x88485aaa, SUC_lexicon_tags_25 },
    { 0xa3085aab, SUC_lexicon_tags_27 },
    { 0xbee85aaa, SUC_lexicon_tags_32 },
    { 0xa6c85aa8, SUC_lexicon_tags_25 },
    { 0x1aa85aae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca85ab1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff485abc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7e85abe, SUC_lexicon_tags_0 },
    { 0x9aa85abf, SUC_lexicon_tags_43 },
    { 0x37485ac0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a85ac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6485ac4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xde85ac7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a85acc, SUC_lexicon_tags_37 },
    { 0x9f285acd, SUC_lexicon_tags_55 },
    { 0x24685acc, SUC_lexicon_tags_107 },
    { 0x34e85acc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x79485ad1, SUC_lexicon_tags_11 },
    { 0xfc685ad1, SUC_lexicon_tags_37 },
    { 0x33885ad3, SUC_lexicon_tags_37 },
    { 0xbec85ad4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d085ad7, SUC_lexicon_tags_46 },
    { 0xeb085ad8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0485ae1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x20685ae4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1c085ae6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3c85ae8, SUC_lexicon_tags_7 },
    { 0xe6e85ae9, SUC_lexicon_tags_30 },
    { 0x53285aea, SUC_lexicon_tags_33 },
    { 0x31085aea, SUC_lexicon_tags_23 },
    { 0x41c85aea, SUC_lexicon_tags_87 },
    { 0x6a85aea, SUC_lexicon_tags_38 },
    { 0xa4c85aee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c85af2, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0285af6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x54485af9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x7285afc, SUC_lexicon_tags_25 },
    { 0x9a285afd, SUC_lexicon_tags_85 },
    { 0xd8485afc, SUC_lexicon_tags_33 },
    { 0xbb285aff, SUC_lexicon_tags_11 },
    { 0x17885b00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba85b06, SUC_lexicon_tags_11 },
    { 0xea085b07, SUC_lexicon_tags_29 },
    { 0x14485b07, SUC_lexicon_tags_19 },
    { 0x38685b07, SUC_lexicon_tags_37 },
    { 0xcd485b0a, SUC_lexicon_tags_37 },
    { 0x9da85b0a, SUC_lexicon_tags_1 },
    { 0x84285b0b, SUC_lexicon_tags_25 },
    { 0x8aa85b0d, SUC_lexicon_tags_31 },
    { 0xb5685b0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc085b11, SUC_lexicon_tags_81 },
    { 0xe6485b11, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4a685b14, SUC_lexicon_tags_25 },
    { 0x41485b15, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x82685b17, SUC_lexicon_tags_37 },
    { 0xe2885b18, SUC_lexicon_tags_37 },
    { 0x38e85b19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f685b1d, SUC_lexicon_tags_11 },
    { 0x10a85b1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac285b22, SUC_lexicon_tags_60 },
    { 0xeea85b23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x88685b26, SUC_lexicon_tags_25 },
    { 0x10e85b27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60885b29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2885b2c, SUC_lexicon_tags_7 },
    { 0x45c85b2c, SUC_lexicon_tags_12 },
    { 0xfc85b2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d685b31, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34285b39, SUC_lexicon_tags_2 },
    { 0xbe685b3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b085b41, SUC_lexicon_tags_37 },
    { 0x6d685b42, SUC_lexicon_tags_16 },
    { 0x24e85b43, SUC_lexicon_tags_2 },
    { 0x5c885b44, SUC_lexicon_tags_1 },
    { 0x6b685b44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88485b47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c685b4d, SUC_lexicon_tags_2 },
    { 0x15c85b4e, SUC_lexicon_tags_3 },
    { 0x63085b4e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x17485b51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c085b53, SUC_lexicon_tags_25 },
    { 0xa485b54, SUC_lexicon_tags_55 },
    { 0x5b485b55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88485b57, SUC_lexicon_tags_47 },
    { 0xb8685b57, SUC_lexicon_tags_19 },
    { 0x22285b59, SUC_lexicon_tags_31 },
    { 0x3c285b5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17685b5c, SUC_lexicon_tags_19 },
    { 0x83885b5d, SUC_lexicon_tags_39 },
    { 0xa9085b5e, SUC_lexicon_tags_47 },
    { 0x17c85b5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86485b69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e85b6f, SUC_lexicon_tags_2 },
    { 0xdb285b70, SUC_lexicon_tags_77 },
    { 0xac085b71, SUC_lexicon_tags_63 },
    { 0xf1a85b71, SUC_lexicon_tags_8 },
    { 0x87285b71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c85b77, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x95685b79, SUC_lexicon_tags_15 },
    { 0x6f685b7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6285b85, SUC_lexicon_tags_1 },
    { 0x9885b86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbd085b88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcfc85b8a, SUC_lexicon_tags_17 },
    { 0xe4485b8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6685b90, SUC_lexicon_tags_68 },
    { 0x67685b90, SUC_lexicon_tags_49 },
    { 0xb6085b92, SUC_lexicon_tags_2 },
    { 0xa8085b91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8485b98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e85b9e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca85ba1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c85ba9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37085bab, SUC_lexicon_tags_25 },
    { 0xb3e85bac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885bb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a085bb4, SUC_lexicon_tags_11 },
    { 0x59685bb5, SUC_lexicon_tags_38 },
    { 0xe8085bb4, SUC_lexicon_tags_47 },
    { 0xdc885bb7, SUC_lexicon_tags_2 },
    { 0xcb285bb4, SUC_lexicon_tags_48 },
    { 0x41085bb8, SUC_lexicon_tags_14 },
    { 0x35c85bba, SUC_lexicon_tags_13 },
    { 0xaea85bb8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaca85bbd, SUC_lexicon_tags_12 },
    { 0xe7c85bbe, SUC_lexicon_tags_23 },
    { 0x8bc85bbf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3885bc5, SUC_lexicon_tags_25 },
    { 0x8ac85bc6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51085bca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83e85bcc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a85bd6, SUC_lexicon_tags_15 },
    { 0x60a85bd6, SUC_lexicon_tags_7 },
    { 0x67e85bd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe885bda, SUC_lexicon_tags_45 },
    { 0x69085bdb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x13a85bdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a85be0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2085be3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6485be8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63285bea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9dc85bec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa85bef, SUC_lexicon_tags_17 },
    { 0x16885bf0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8485bf3, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c85bf7, SUC_lexicon_tags_6 },
    { 0xcc085bf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22e85bfa, SUC_lexicon_tags_47 },
    { 0xc1a85bfb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a85bff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b485c01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91e85c03, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4885c07, SUC_lexicon_tags_25 },
    { 0x1685c08, SUC_lexicon_tags_4 },
    { 0xc8685c08, SUC_lexicon_tags_160 },
    { 0xe0c85c07, SUC_lexicon_tags_25 },
    { 0xf1c85c07, SUC_lexicon_tags_37 },
    { 0x33e85c09, SUC_lexicon_tags_1 },
    { 0x14c85c09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e85c10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea085c12, SUC_lexicon_tags_12 },
    { 0xacc85c13, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcda85c15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xae485c18, SUC_lexicon_tags_37 },
    { 0x7a685c19, SUC_lexicon_tags_10 },
    { 0x62c85c1a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x42885c1d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22085c21, SUC_lexicon_tags_1 },
    { 0xec085c22, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xfa485c24, SUC_lexicon_tags_33 },
    { 0xa8885c25, SUC_lexicon_tags_1 },
    { 0xc4e85c26, SUC_lexicon_tags_2 },
    { 0x3b485c27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d085c29, SUC_lexicon_tags_8 },
    { 0xa1c85c2a, SUC_lexicon_tags_7 },
    { 0x5b685c2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea85c2f, SUC_lexicon_tags_0 },
    { 0xe9a85c30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfc85c32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a85c35, SUC_lexicon_tags_25 },
    { 0x7e85c36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb085c3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5685c40, SUC_lexicon_tags_11 },
    { 0x23285c40, SUC_lexicon_tags_2 },
    { 0xb0485c40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1685c45, SUC_lexicon_tags_0 },
    { 0x33e85c46, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3085c4b, SUC_lexicon_tags_7 },
    { 0x42e85c4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f685c4f, SUC_lexicon_tags_17 },
    { 0x15085c50, SUC_lexicon_tags_1 },
    { 0x83285c51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54685c55, SUC_lexicon_tags_27 },
    { 0x7f085c55, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea85c5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xffe85c5f, SUC_lexicon_tags_65 },
    { 0x48285c5f, SUC_lexicon_tags_15 },
    { 0x63c85c60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc285c64, SUC_lexicon_tags_37 },
    { 0xc3285c65, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b285c6c, SUC_lexicon_tags_17 },
    { 0x2f085c6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c85c70, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4ae85c72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x22885c74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a85c78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3285c7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17885c81, SUC_lexicon_tags_11 },
    { 0x7a85c82, SUC_lexicon_tags_25 },
    { 0x3ba85c83, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbc85c85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b885c8d, SUC_lexicon_tags_151 },
    { 0x92c85c8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec85c91, SUC_lexicon_tags_148 },
    { 0x15a85c91, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd0a85c94, SUC_lexicon_tags_37 },
    { 0xaa485c94, SUC_lexicon_tags_33 },
    { 0x95085c96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x83285c99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x35485c9c, SUC_lexicon_tags_11 },
    { 0x87885c9d, SUC_lexicon_tags_7 },
    { 0x60e85c9d, SUC_lexicon_tags_37 },
    { 0xca485c9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c85ca3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8885ca7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8685cac, SUC_lexicon_tags_42 },
    { 0x8da85cad, SUC_lexicon_tags_29 },
    { 0x80085cad, SUC_lexicon_tags_15 },
    { 0xf9685caf, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8685cb2, SUC_lexicon_tags_43 },
    { 0x7a485cb3, SUC_lexicon_tags_27 },
    { 0x21085cb2, SUC_lexicon_tags_12 },
    { 0x8be85cb5, SUC_lexicon_tags_2 },
    { 0x49c85cb5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfac85cb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc85cbb, SUC_lexicon_tags_136 },
    { 0xf6085cbc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe85cbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e85cc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc285cc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e85cd0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf0285cd2, SUC_lexicon_tags_27 },
    { 0x49c85cd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf085cd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32085cdc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6285cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8885ce3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71285ce7, SUC_lexicon_tags_15 },
    { 0x3f885ce8, SUC_lexicon_tags_37 },
    { 0x88e85ce7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99685ceb, SUC_lexicon_tags_15 },
    { 0x67885cec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b285cef, SUC_lexicon_tags_30 },
    { 0x88085cef, SUC_lexicon_tags_25 },
    { 0x7f485cef, SUC_lexicon_tags_63 },
    { 0xcd285cf2, SUC_lexicon_tags_0 },
    { 0x48e85cf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39285cf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c85cf9, SUC_lexicon_tags_15 },
    { 0x8b285cfa, SUC_lexicon_tags_6 },
    { 0x1c85cf9, SUC_lexicon_tags_1 },
    { 0xe0685cfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac485d00, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf685d08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd085d0c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c85d13, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xfbe85d15, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe9085d17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3685d19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8085d1b, SUC_lexicon_tags_17 },
    { 0xfee85d1c, SUC_lexicon_tags_12 },
    { 0xacc85d1b, SUC_lexicon_tags_49 },
    { 0x45a85d1c, SUC_lexicon_tags_37 },
    { 0xff485d1e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f285d23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9685d26, SUC_lexicon_tags_37 },
    { 0x9aa85d27, SUC_lexicon_tags_6 },
    { 0x23885d27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b085d2c, SUC_lexicon_tags_27 },
    { 0x6e485d2c, SUC_lexicon_tags_95 },
    { 0x13485d2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa85d38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92085d3a, SUC_lexicon_tags_2 },
    { 0x3ba85d3a, SUC_lexicon_tags_19 },
    { 0x42085d3c, SUC_lexicon_tags_8 },
    { 0x32285d3b, SUC_lexicon_tags_25 },
    { 0x5b285d3e, SUC_lexicon_tags_21 },
    { 0xdbc85d3d, SUC_lexicon_tags_19 },
    { 0x8fa85d40, SUC_lexicon_tags_32 },
    { 0x3e285d3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x81885d44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe85d47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31285d4d, SUC_lexicon_tags_12 },
    { 0x6885d4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3285d50, SUC_lexicon_tags_14 },
    { 0x6e885d51, SUC_lexicon_tags_7 },
    { 0x18285d52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd085d56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c85d59, SUC_lexicon_tags_37 },
    { 0x1ae85d59, SUC_lexicon_tags_38 },
    { 0xf5685d5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23085d5d, SUC_lexicon_tags_21 },
    { 0xcb685d5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf485d60, SUC_lexicon_tags_17 },
    { 0x1a885d60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a885d66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a85d6c, SUC_lexicon_tags_23 },
    { 0x83e85d6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c85d72, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e85d76, SUC_lexicon_tags_15 },
    { 0xb2285d77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a485d79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35c85d7b, SUC_lexicon_tags_1 },
    { 0xe9a85d7c, SUC_lexicon_tags_68 },
    { 0xe0285d7b, SUC_lexicon_tags_29 },
    { 0x4c885d7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf6685d80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8285d82, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x75e85d84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3885d87, SUC_lexicon_tags_16 },
    { 0x50c85d88, SUC_lexicon_tags_17 },
    { 0xcba85d87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4085d8c, SUC_lexicon_tags_9 },
    { 0x70e85d8d, SUC_lexicon_tags_15 },
    { 0xdc285d8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb285d90, SUC_lexicon_tags_37 },
    { 0xace85d90, SUC_lexicon_tags_7 },
    { 0x1da85d92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6485d97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50485d9e, SUC_lexicon_tags_17 },
    { 0xd485d9e, SUC_lexicon_tags_17 },
    { 0x41085da0, SUC_lexicon_tags_49 },
    { 0xaa685d9f, SUC_lexicon_tags_0 },
    { 0x78685da1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4685da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba85da8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8085daa, SUC_lexicon_tags_1 },
    { 0x1bc85dab, SUC_lexicon_tags_37 },
    { 0x84285dab, SUC_lexicon_tags_12 },
    { 0x7c685dab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b485daf, SUC_lexicon_tags_11 },
    { 0xaa285db0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c885db2, SUC_lexicon_tags_47 },
    { 0x86085db3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4085db5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c85dba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x54a85dbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e85dc0, SUC_lexicon_tags_12 },
    { 0x2dc85dc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f085dc6, SUC_lexicon_tags_37 },
    { 0xe7c85dc7, SUC_lexicon_tags_7 },
    { 0xd1c85dc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad685dd2, SUC_lexicon_tags_23 },
    { 0x7d685dd3, SUC_lexicon_tags_0 },
    { 0xde685dd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda285dd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485ddf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe3c85de1, SUC_lexicon_tags_26 },
    { 0x32085de2, SUC_lexicon_tags_7 },
    { 0xd5685de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a85de8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c85dee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e85df0, SUC_lexicon_tags_218 },
    { 0x3c685df1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda85df4, SUC_lexicon_tags_25 },
    { 0xf5a85df5, SUC_lexicon_tags_10 },
    { 0x85085df5, SUC_lexicon_tags_25 },
    { 0xec485df6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce85dfa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0685dff, SUC_lexicon_tags_37 },
    { 0xdc685e00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x23285e03, SUC_lexicon_tags_31 },
    { 0x22885e04, SUC_lexicon_tags_11 },
    { 0x88a85e05, SUC_lexicon_tags_17 },
    { 0x99485e06, SUC_lexicon_tags_49 },
    { 0xace85e07, SUC_lexicon_tags_33 },
    { 0xbea85e08, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55285e11, SUC_lexicon_tags_37 },
    { 0xf1e85e12, SUC_lexicon_tags_9 },
    { 0xffe85e13, SUC_lexicon_tags_11 },
    { 0x8485e12, SUC_lexicon_tags_25 },
    { 0x87c85e13, SUC_lexicon_tags_33 },
    { 0xc2285e16, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb3085e18, SUC_lexicon_tags_15 },
    { 0xbaa85e18, SUC_lexicon_tags_118 },
    { 0x50085e1a, SUC_lexicon_tags_12 },
    { 0x67085e1b, SUC_lexicon_tags_25 },
    { 0x17885e1a, SUC_lexicon_tags_19 },
    { 0x685e18, SUC_lexicon_tags_22 },
    { 0xbfe85e1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72285e25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a85e27, SUC_lexicon_tags_9 },
    { 0xe085e28, SUC_lexicon_tags_45 },
    { 0x38285e28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a85e2d, SUC_lexicon_tags_15 },
    { 0x90a85e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3c85e30, SUC_lexicon_tags_60 },
    { 0x84285e30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61485e33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc285e35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17c85e37, SUC_lexicon_tags_34 },
    { 0xbe685e38, SUC_lexicon_tags_61 },
    { 0x1d485e39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91285e3b, SUC_lexicon_tags_23 },
    { 0x44085e3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8085e3e, SUC_lexicon_tags_252 },
    { 0x44485e3f, SUC_lexicon_tags_3 },
    { 0x8ae85e3f, SUC_lexicon_tags_4 },
    { 0x82c85e3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c85e46, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x65485e48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc85e4b, SUC_lexicon_tags_25 },
    { 0x29285e4c, SUC_lexicon_tags_2 },
    { 0xe1285e4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91a85e4f, SUC_lexicon_tags_4 },
    { 0x90285e4f, SUC_lexicon_tags_30 },
    { 0x9c685e51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2685e53, SUC_lexicon_tags_11 },
    { 0x67685e53, SUC_lexicon_tags_11 },
    { 0x10a85e54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c85e58, SUC_lexicon_tags_33 },
    { 0x92e85e59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x93a85e5b, SUC_lexicon_tags_25 },
    { 0xb1885e5c, SUC_lexicon_tags_33 },
    { 0xfec85e5d, SUC_lexicon_tags_60 },
    { 0x66085e5d, SUC_lexicon_tags_44 },
    { 0xaaa85e5f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc85e63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2485e67, SUC_lexicon_tags_6 },
    { 0x5ca85e68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c85e6c, SUC_lexicon_tags_68 },
    { 0x16a85e6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7e85e6f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e85e74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f885e79, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a85e7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28885e82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a85e8a, SUC_lexicon_tags_1145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6485e8f, SUC_lexicon_tags_39 },
    { 0x38285e90, SUC_lexicon_tags_11 },
    { 0xd9285e91, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x85285e93, SUC_lexicon_tags_25 },
    { 0x2ee85e94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2f485e96, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaea85e98, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x11685e9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1885e9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea85e9f, SUC_lexicon_tags_25 },
    { 0xd4a85e9f, SUC_lexicon_tags_48 },
    { 0x88a85ea1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc85ea5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac85eb0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81085eb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf685eb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcc85eba, SUC_lexicon_tags_3 },
    { 0xb6e85eba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85685ec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24c85ec4, SUC_lexicon_tags_29 },
    { 0x4a085ec5, SUC_lexicon_tags_4 },
    { 0x1fa85ec6, SUC_lexicon_tags_11 },
    { 0x32485ec7, SUC_lexicon_tags_0 },
    { 0xec285ec7, SUC_lexicon_tags_47 },
    { 0xc6285ec7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa0885ecb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a85ece, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e85ed1, SUC_lexicon_tags_33 },
    { 0xc0885ed1, SUC_lexicon_tags_65 },
    { 0x28285ed3, SUC_lexicon_tags_37 },
    { 0x5c285ed3, SUC_lexicon_tags_6 },
    { 0xd3285ed1, SUC_lexicon_tags_1 },
    { 0x91085ed1, SUC_lexicon_tags_7 },
    { 0x2b285ed7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec85edb, SUC_lexicon_tags_8 },
    { 0xab685edb, SUC_lexicon_tags_12 },
    { 0xf4285edc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3285ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c85ee4, SUC_lexicon_tags_31 },
    { 0xb8485ee5, SUC_lexicon_tags_1 },
    { 0x2f485ee6, SUC_lexicon_tags_27 },
    { 0x15885ee7, SUC_lexicon_tags_1 },
    { 0x2ca85ee8, SUC_lexicon_tags_15 },
    { 0x40c85ee6, SUC_lexicon_tags_2 },
    { 0x95285ee5, SUC_lexicon_tags_23 },
    { 0xc6085ee5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc485ef3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec285ef7, SUC_lexicon_tags_1 },
    { 0xb8e85ef8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae85eff, SUC_lexicon_tags_3 },
    { 0xbb685f00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65085f04, SUC_lexicon_tags_17 },
    { 0x12e85f04, SUC_lexicon_tags_12 },
    { 0x95085f06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6a85f08, SUC_lexicon_tags_23 },
    { 0xae885f08, SUC_lexicon_tags_129 },
    { 0x2ba85f08, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x47685f0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3285f11, SUC_lexicon_tags_25 },
    { 0x86e85f11, SUC_lexicon_tags_38 },
    { 0xdd685f12, SUC_lexicon_tags_2 },
    { 0x90e85f14, SUC_lexicon_tags_17 },
    { 0xab885f12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1685f18, SUC_lexicon_tags_7 },
    { 0x3aa85f19, SUC_lexicon_tags_60 },
    { 0x5a485f19, SUC_lexicon_tags_31 },
    { 0x5a085f1b, SUC_lexicon_tags_11 },
    { 0x49e85f1c, SUC_lexicon_tags_1 },
    { 0x28a85f1a, SUC_lexicon_tags_23 },
    { 0x71885f1b, SUC_lexicon_tags_53 },
    { 0x1c685f1c, SUC_lexicon_tags_37 },
    { 0xf9085f1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x55885f23, SUC_lexicon_tags_46 },
    { 0xef485f24, SUC_lexicon_tags_23 },
    { 0x38e85f25, SUC_lexicon_tags_37 },
    { 0x28c85f26, SUC_lexicon_tags_65 },
    { 0x63285f27, SUC_lexicon_tags_7 },
    { 0x5f285f26, SUC_lexicon_tags_12 },
    { 0x47885f28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99285f2e, SUC_lexicon_tags_15 },
    { 0xa3085f2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5485f32, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf685f36, SUC_lexicon_tags_2 },
    { 0x55c85f37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb485f3a, SUC_lexicon_tags_15 },
    { 0xb6685f3a, SUC_lexicon_tags_14 },
    { 0xc7485f3b, SUC_lexicon_tags_39 },
    { 0x6c085f3c, SUC_lexicon_tags_17 },
    { 0xab685f3e, SUC_lexicon_tags_4 },
    { 0x13c85f3e, SUC_lexicon_tags_49 },
    { 0xb0485f3e, SUC_lexicon_tags_33 },
    { 0xd0e85f3d, SUC_lexicon_tags_19 },
    { 0xf5885f3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a85f48, SUC_lexicon_tags_11 },
    { 0xd7885f48, SUC_lexicon_tags_63 },
    { 0xcb485f4a, SUC_lexicon_tags_37 },
    { 0x8be85f4b, SUC_lexicon_tags_37 },
    { 0x9285f48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42885f53, SUC_lexicon_tags_15 },
    { 0x8c285f54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78085f5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e85f5f, SUC_lexicon_tags_65 },
    { 0x56885f60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x12685f63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a85f69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3e85f6b, SUC_lexicon_tags_23 },
    { 0x82e85f6c, SUC_lexicon_tags_8 },
    { 0x3c085f6d, SUC_lexicon_tags_60 },
    { 0xdae85f6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c85f71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x41885f73, SUC_lexicon_tags_43 },
    { 0x5bc85f73, SUC_lexicon_tags_0 },
    { 0x73a85f75, SUC_lexicon_tags_12 },
    { 0xe6c85f76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe085f78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15285f7b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c485f7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b885f81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6885f83, SUC_lexicon_tags_38 },
    { 0xd7c85f84, SUC_lexicon_tags_7 },
    { 0xf6885f85, SUC_lexicon_tags_15 },
    { 0x26285f86, SUC_lexicon_tags_8 },
    { 0xc8a85f87, SUC_lexicon_tags_17 },
    { 0x69885f87, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3685f8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29e85f8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e85f91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc485f93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e85f96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5085f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6685f9a, SUC_lexicon_tags_7 },
    { 0xe2a85f9a, SUC_lexicon_tags_7 },
    { 0xa8485f9c, SUC_lexicon_tags_65 },
    { 0x66885f9d, SUC_lexicon_tags_17 },
    { 0x8a285f9d, SUC_lexicon_tags_11 },
    { 0x52685f9e, SUC_lexicon_tags_14 },
    { 0x4a485fa0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb285fa2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa85fa9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4885fae, SUC_lexicon_tags_1 },
    { 0x13a85fae, SUC_lexicon_tags_11 },
    { 0x36285fb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5ee85fb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x35685fb5, SUC_lexicon_tags_27 },
    { 0x12a85fb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5c85fb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80a85fba, SUC_lexicon_tags_105 },
    { 0x4f885fbb, SUC_lexicon_tags_31 },
    { 0x2b685fba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e85fc5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1cc85fc7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e885fcc, SUC_lexicon_tags_27 },
    { 0xb6885fcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e85fd0, SUC_lexicon_tags_25 },
    { 0x485fd1, SUC_lexicon_tags_2 },
    { 0xa3885fd0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd685fd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf285fda, SUC_lexicon_tags_31 },
    { 0xd5c85fdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c685fdd, SUC_lexicon_tags_16 },
    { 0x3b885fde, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xda285fe0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2485fe3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe85fea, SUC_lexicon_tags_39 },
    { 0x72e85feb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0685fed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86485ff3, SUC_lexicon_tags_25 },
    { 0xa0e85ff4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46485ffa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd486000, SUC_lexicon_tags_15 },
    { 0x23086001, SUC_lexicon_tags_23 },
    { 0x72286002, SUC_lexicon_tags_9 },
    { 0x94486001, SUC_lexicon_tags_47 },
    { 0xcb686000, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e8600b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e8600e, SUC_lexicon_tags_4 },
    { 0xdca8600f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5c286011, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50686019, SUC_lexicon_tags_27 },
    { 0xae08601a, SUC_lexicon_tags_17 },
    { 0x4b686019, SUC_lexicon_tags_37 },
    { 0x4e28601a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7608602b, SUC_lexicon_tags_37 },
    { 0xc2e8602b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea8602f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x90686031, SUC_lexicon_tags_9 },
    { 0x3486031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1486035, SUC_lexicon_tags_12 },
    { 0x8be86035, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x29286038, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f88603a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x88a8603c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e8603f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd9a86041, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae86048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba8604c, SUC_lexicon_tags_47 },
    { 0x3808604d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5048604f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd686054, SUC_lexicon_tags_15 },
    { 0x11486054, SUC_lexicon_tags_38 },
    { 0x2cc86056, SUC_lexicon_tags_25 },
    { 0x4b286057, SUC_lexicon_tags_17 },
    { 0x79086056, SUC_lexicon_tags_11 },
    { 0x66a86058, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48605e, SUC_lexicon_tags_45 },
    { 0xf7a8605e, SUC_lexicon_tags_11 },
    { 0x7c28605e, SUC_lexicon_tags_14 },
    { 0xdea86061, SUC_lexicon_tags_65 },
    { 0x77286061, SUC_lexicon_tags_27 },
    { 0x9ec86062, SUC_lexicon_tags_2 },
    { 0x66a86063, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e8606b, SUC_lexicon_tags_15 },
    { 0x4048606c, SUC_lexicon_tags_30 },
    { 0xf7c8606c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x53486070, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53c86072, SUC_lexicon_tags_17 },
    { 0xdc686072, SUC_lexicon_tags_2 },
    { 0xf6086072, SUC_lexicon_tags_43 },
    { 0x7486075, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda686078, SUC_lexicon_tags_47 },
    { 0x31486079, SUC_lexicon_tags_69 },
    { 0x6948607a, SUC_lexicon_tags_91 },
    { 0xde68607b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd968607e, SUC_lexicon_tags_37 },
    { 0x7d08607f, SUC_lexicon_tags_49 },
    { 0x6dc8607f, SUC_lexicon_tags_60 },
    { 0x96686080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e86084, SUC_lexicon_tags_14 },
    { 0x70886085, SUC_lexicon_tags_11 },
    { 0xa5286085, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1886089, SUC_lexicon_tags_48 },
    { 0x2f88608a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e8608c, SUC_lexicon_tags_15 },
    { 0xa148608d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7286090, SUC_lexicon_tags_25 },
    { 0xf0286090, SUC_lexicon_tags_39 },
    { 0x4aa86092, SUC_lexicon_tags_38 },
    { 0x32086093, SUC_lexicon_tags_45 },
    { 0x2ec86094, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3086096, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98c86098, SUC_lexicon_tags_29 },
    { 0xa9286099, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e860a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c860a4, SUC_lexicon_tags_63 },
    { 0x7d2860a4, SUC_lexicon_tags_15 },
    { 0x9c6860a6, SUC_lexicon_tags_38 },
    { 0x64c860a6, SUC_lexicon_tags_33 },
    { 0x9fe860a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22860ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e860af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa28860b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec860b7, SUC_lexicon_tags_37 },
    { 0x384860b8, SUC_lexicon_tags_12 },
    { 0x9ba860b9, SUC_lexicon_tags_43 },
    { 0xae860b9, SUC_lexicon_tags_10 },
    { 0xf20860bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8860c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8860cb, SUC_lexicon_tags_2 },
    { 0xfda860cc, SUC_lexicon_tags_19 },
    { 0xf60860cd, SUC_lexicon_tags_25 },
    { 0x17c860cc, SUC_lexicon_tags_68 },
    { 0x796860cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x92e860d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a860d5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6860db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80a860dd, SUC_lexicon_tags_10 },
    { 0x270860dd, SUC_lexicon_tags_1 },
    { 0x1b2860df, SUC_lexicon_tags_14 },
    { 0x890860df, SUC_lexicon_tags_15 },
    { 0x7d0860e1, SUC_lexicon_tags_29 },
    { 0x3ce860de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xec0860e4, SUC_lexicon_tags_15 },
    { 0x8a2860e5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4a860e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14860ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4b4860ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e860f0, SUC_lexicon_tags_12 },
    { 0xe8c860f1, SUC_lexicon_tags_15 },
    { 0xece860f1, SUC_lexicon_tags_2 },
    { 0x9c8860f3, SUC_lexicon_tags_11 },
    { 0x7d4860f4, SUC_lexicon_tags_25 },
    { 0x3ea860f5, SUC_lexicon_tags_12 },
    { 0x9f8860f6, SUC_lexicon_tags_65 },
    { 0xdaa860f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c0860f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70860fd, SUC_lexicon_tags_21 },
    { 0x236860fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce86100, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48286102, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac86105, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c286108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a86111, SUC_lexicon_tags_29 },
    { 0x74c86111, SUC_lexicon_tags_12 },
    { 0x9d286113, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17686115, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd2886117, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9208611b, SUC_lexicon_tags_30 },
    { 0x268611b, SUC_lexicon_tags_63 },
    { 0x71c8611d, SUC_lexicon_tags_14 },
    { 0xbd68611c, SUC_lexicon_tags_2 },
    { 0x6a28611f, SUC_lexicon_tags_35 },
    { 0xf3086120, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24886126, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e8612b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4108612f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3086132, SUC_lexicon_tags_27 },
    { 0xd3886133, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9686138, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1c08613a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe068613c, SUC_lexicon_tags_33 },
    { 0xa708613d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba86142, SUC_lexicon_tags_22 },
    { 0x4b086143, SUC_lexicon_tags_38 },
    { 0xa8c86144, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a8614a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5368614e, SUC_lexicon_tags_17 },
    { 0x7aa8614f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2886152, SUC_lexicon_tags_46 },
    { 0xbb886153, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1486159, SUC_lexicon_tags_57 },
    { 0xf7286159, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe1e8615c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e86160, SUC_lexicon_tags_2 },
    { 0xb0a86161, SUC_lexicon_tags_65 },
    { 0xfd086160, SUC_lexicon_tags_1 },
    { 0xb3c86160, SUC_lexicon_tags_2 },
    { 0x29286160, SUC_lexicon_tags_47 },
    { 0x42c86163, SUC_lexicon_tags_25 },
    { 0x52086160, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc486169, SUC_lexicon_tags_63 },
    { 0xbba86169, SUC_lexicon_tags_15 },
    { 0x3768616b, SUC_lexicon_tags_37 },
    { 0x8828616c, SUC_lexicon_tags_17 },
    { 0xeea8616d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4568616f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee86176, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb108617a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e8617d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a08617f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8286186, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8486188, SUC_lexicon_tags_37 },
    { 0xd2086189, SUC_lexicon_tags_37 },
    { 0xecc8618a, SUC_lexicon_tags_15 },
    { 0xbf68618b, SUC_lexicon_tags_11 },
    { 0x56c8618c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5aa8618e, SUC_lexicon_tags_65 },
    { 0x8ae8618e, SUC_lexicon_tags_49 },
    { 0x56e86190, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a86195, SUC_lexicon_tags_11 },
    { 0x9d086195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e486198, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe668619a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c08619c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb608619f, SUC_lexicon_tags_27 },
    { 0x78e861a0, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce861aa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd2a861ac, SUC_lexicon_tags_7 },
    { 0x48c861ac, SUC_lexicon_tags_2 },
    { 0xfa0861ae, SUC_lexicon_tags_11 },
    { 0xc4861af, SUC_lexicon_tags_19 },
    { 0xdd0861b0, SUC_lexicon_tags_27 },
    { 0xa3c861ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108861b6, SUC_lexicon_tags_3 },
    { 0xf52861b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae861bd, SUC_lexicon_tags_10 },
    { 0x3d2861be, SUC_lexicon_tags_15 },
    { 0xd00861bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8861c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x602861c5, SUC_lexicon_tags_48 },
    { 0x52e861c5, SUC_lexicon_tags_31 },
    { 0xd00861c7, SUC_lexicon_tags_68 },
    { 0x1a4861c7, SUC_lexicon_tags_1 },
    { 0xec8861c9, SUC_lexicon_tags_7 },
    { 0x4a6861c9, SUC_lexicon_tags_30 },
    { 0xe78861c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598861d0, SUC_lexicon_tags_30 },
    { 0xe98861d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66e861d3, SUC_lexicon_tags_49 },
    { 0x922861d4, SUC_lexicon_tags_1 },
    { 0x310861d5, SUC_lexicon_tags_23 },
    { 0x3d0861d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3e861d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a861dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9a861df, SUC_lexicon_tags_12 },
    { 0xa3a861df, SUC_lexicon_tags_17 },
    { 0x642861e1, SUC_lexicon_tags_17 },
    { 0x9a8861e2, SUC_lexicon_tags_30 },
    { 0xa6e861e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc861eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc861f0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae861f5, SUC_lexicon_tags_29 },
    { 0x83e861f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a861fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fe861fc, SUC_lexicon_tags_23 },
    { 0x72861fc, SUC_lexicon_tags_17 },
    { 0x5fc861fe, SUC_lexicon_tags_82 },
    { 0x32e861ff, SUC_lexicon_tags_2 },
    { 0x23086200, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2286202, SUC_lexicon_tags_2 },
    { 0xd2a86203, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb7286205, SUC_lexicon_tags_0 },
    { 0x11c86206, SUC_lexicon_tags_12 },
    { 0x85a86207, SUC_lexicon_tags_2 },
    { 0x78286208, SUC_lexicon_tags_49 },
    { 0x11086209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b88620e, SUC_lexicon_tags_29 },
    { 0xb5a8620e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac86214, SUC_lexicon_tags_0 },
    { 0x26c86215, SUC_lexicon_tags_37 },
    { 0x72286216, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x88086219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7686227, SUC_lexicon_tags_1 },
    { 0x3ba86228, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a88622b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc888622d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf286234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x31486237, SUC_lexicon_tags_27 },
    { 0x2d886238, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x39e8623a, SUC_lexicon_tags_323 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5828623e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbc286240, SUC_lexicon_tags_12 },
    { 0x72086241, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c86247, SUC_lexicon_tags_37 },
    { 0x35e86248, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a8624c, SUC_lexicon_tags_11 },
    { 0xfca8624d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a86250, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a686255, SUC_lexicon_tags_12 },
    { 0x16a86255, SUC_lexicon_tags_15 },
    { 0x5b086257, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe208625b, SUC_lexicon_tags_25 },
    { 0x3588625c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c86260, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x94286263, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc86269, SUC_lexicon_tags_25 },
    { 0x46486269, SUC_lexicon_tags_33 },
    { 0xd088626a, SUC_lexicon_tags_11 },
    { 0x77c8626c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5886271, SUC_lexicon_tags_12 },
    { 0x25686272, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58286274, SUC_lexicon_tags_23 },
    { 0xb5a86275, SUC_lexicon_tags_11 },
    { 0xa7086275, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e86279, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba8627c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a28627f, SUC_lexicon_tags_35 },
    { 0xdc086280, SUC_lexicon_tags_37 },
    { 0xbae86281, SUC_lexicon_tags_8 },
    { 0xc886281, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x35086284, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x22286286, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdce86288, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc268628b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9888628d, SUC_lexicon_tags_10 },
    { 0x9788628e, SUC_lexicon_tags_9 },
    { 0xb9e8628d, SUC_lexicon_tags_19 },
    { 0xfb686290, SUC_lexicon_tags_11 },
    { 0x1ea86291, SUC_lexicon_tags_3 },
    { 0x9548628f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68886294, SUC_lexicon_tags_38 },
    { 0xd7286295, SUC_lexicon_tags_3 },
    { 0x26286296, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa286299, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe028629d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a862a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x324862a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd68862a4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80862a8, SUC_lexicon_tags_17 },
    { 0xcd4862a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x708862ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x900862ae, SUC_lexicon_tags_25 },
    { 0x1e6862af, SUC_lexicon_tags_37 },
    { 0xe0e862b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa94862b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce862b5, SUC_lexicon_tags_14 },
    { 0x942862b6, SUC_lexicon_tags_62 },
    { 0x42a862b7, SUC_lexicon_tags_14 },
    { 0x292862b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa46862ba, SUC_lexicon_tags_30 },
    { 0x2c0862bb, SUC_lexicon_tags_15 },
    { 0xaa2862bb, SUC_lexicon_tags_8 },
    { 0xe0862bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56862c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714862ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850862d0, SUC_lexicon_tags_15 },
    { 0xca8862d1, SUC_lexicon_tags_8 },
    { 0x120862d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbec862d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c862d7, SUC_lexicon_tags_46 },
    { 0xdf6862d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6862db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc862de, SUC_lexicon_tags_12 },
    { 0xcd8862de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4862e2, SUC_lexicon_tags_0 },
    { 0xc20862e3, SUC_lexicon_tags_7 },
    { 0xf5c862e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x756862e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98862ed, SUC_lexicon_tags_15 },
    { 0x3ca862ed, SUC_lexicon_tags_14 },
    { 0x428862ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e862f3, SUC_lexicon_tags_12 },
    { 0x7ee862f4, SUC_lexicon_tags_11 },
    { 0x87c862f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2862fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2862fd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87886303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63086305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0c86307, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae8630c, SUC_lexicon_tags_37 },
    { 0x348630c, SUC_lexicon_tags_15 },
    { 0xa8e8630c, SUC_lexicon_tags_14 },
    { 0x4a08630f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29086311, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99086315, SUC_lexicon_tags_14 },
    { 0xf3e86316, SUC_lexicon_tags_27 },
    { 0xd9886317, SUC_lexicon_tags_15 },
    { 0x38286318, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38a8631e, SUC_lexicon_tags_25 },
    { 0xbb88631f, SUC_lexicon_tags_25 },
    { 0xb9a8631e, SUC_lexicon_tags_11 },
    { 0xdaa86321, SUC_lexicon_tags_12 },
    { 0x9a86321, SUC_lexicon_tags_11 },
    { 0x8d486321, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x23a86325, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda86328, SUC_lexicon_tags_7 },
    { 0xc4286328, SUC_lexicon_tags_56 },
    { 0xe2a86329, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea8632d, SUC_lexicon_tags_27 },
    { 0xa488632d, SUC_lexicon_tags_28 },
    { 0xc168632f, SUC_lexicon_tags_5 },
    { 0x6f086330, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a86333, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46286335, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e86338, SUC_lexicon_tags_30 },
    { 0x18e86338, SUC_lexicon_tags_590 },
    { 0x40a8633a, SUC_lexicon_tags_85 },
    { 0x35486338, SUC_lexicon_tags_12 },
    { 0x5128633c, SUC_lexicon_tags_57 },
    { 0x6808633d, SUC_lexicon_tags_65 },
    { 0xefa8633a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9686344, SUC_lexicon_tags_42 },
    { 0xb086345, SUC_lexicon_tags_48 },
    { 0x4a86346, SUC_lexicon_tags_13 },
    { 0x76e86347, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x10686349, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec8634c, SUC_lexicon_tags_2 },
    { 0xd328634d, SUC_lexicon_tags_47 },
    { 0x2a68634d, SUC_lexicon_tags_37 },
    { 0x6aa8634f, SUC_lexicon_tags_15 },
    { 0x75c86350, SUC_lexicon_tags_84 },
    { 0x90486350, SUC_lexicon_tags_25 },
    { 0xd5c86350, SUC_lexicon_tags_11 },
    { 0x5c486351, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc86356, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d686359, SUC_lexicon_tags_14 },
    { 0x96a8635a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb828635d, SUC_lexicon_tags_17 },
    { 0x2728635e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfce86360, SUC_lexicon_tags_179 },
    { 0x1a486360, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55886367, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3a86369, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a8636e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee86373, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe86379, SUC_lexicon_tags_29 },
    { 0xa8c8637a, SUC_lexicon_tags_14 },
    { 0x4588637b, SUC_lexicon_tags_1 },
    { 0xbe886379, SUC_lexicon_tags_60 },
    { 0x38a8637d, SUC_lexicon_tags_77 },
    { 0x6f08637e, SUC_lexicon_tags_15 },
    { 0xc688637e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cc86381, SUC_lexicon_tags_19 },
    { 0x56886381, SUC_lexicon_tags_1330 },
    { 0xdc886383, SUC_lexicon_tags_25 },
    { 0xfe486384, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c86387, SUC_lexicon_tags_18 },
    { 0x8d486388, SUC_lexicon_tags_1 },
    { 0x9c86387, SUC_lexicon_tags_25 },
    { 0xa4a86387, SUC_lexicon_tags_23 },
    { 0x148638b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12286391, SUC_lexicon_tags_23 },
    { 0x2f486391, SUC_lexicon_tags_4 },
    { 0x5f686391, SUC_lexicon_tags_39 },
    { 0x45a86394, SUC_lexicon_tags_42 },
    { 0x30886395, SUC_lexicon_tags_33 },
    { 0xd8286394, SUC_lexicon_tags_15 },
    { 0x35c86396, SUC_lexicon_tags_4 },
    { 0x96e86397, SUC_lexicon_tags_49 },
    { 0x28a86397, SUC_lexicon_tags_37 },
    { 0xa3c8639a, SUC_lexicon_tags_45 },
    { 0x628639a, SUC_lexicon_tags_2 },
    { 0x8ea86396, SUC_lexicon_tags_12 },
    { 0xfd886392, SUC_lexicon_tags_25 },
    { 0xc308639c, SUC_lexicon_tags_2 },
    { 0x40886398, SUC_lexicon_tags_47 },
    { 0x348639f, SUC_lexicon_tags_66 },
    { 0x93e8639a, SUC_lexicon_tags_32 },
    { 0xfea86395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc52863a4, SUC_lexicon_tags_25 },
    { 0xec0863a5, SUC_lexicon_tags_29 },
    { 0xb12863a5, SUC_lexicon_tags_17 },
    { 0x6ec863a7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x990863a9, SUC_lexicon_tags_11 },
    { 0x17a863aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x61c863ac, SUC_lexicon_tags_47 },
    { 0x39a863ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6e863af, SUC_lexicon_tags_12 },
    { 0x12e863b0, SUC_lexicon_tags_14 },
    { 0x348863b0, SUC_lexicon_tags_12 },
    { 0x2863b2, SUC_lexicon_tags_49 },
    { 0x5aa863b0, SUC_lexicon_tags_37 },
    { 0x298863b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd2863b6, SUC_lexicon_tags_1 },
    { 0x20863b6, SUC_lexicon_tags_15 },
    { 0x67c863b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x332863bb, SUC_lexicon_tags_80 },
    { 0x15a863bc, SUC_lexicon_tags_12 },
    { 0x94a863bd, SUC_lexicon_tags_170 },
    { 0x2d2863be, SUC_lexicon_tags_47 },
    { 0xfb4863bf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2863c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x252863ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe8a863cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e863cf, SUC_lexicon_tags_17 },
    { 0x36863cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x656863d3, SUC_lexicon_tags_2 },
    { 0xa4863d4, SUC_lexicon_tags_8 },
    { 0xbf6863d3, SUC_lexicon_tags_4 },
    { 0x814863d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82863da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8863dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea863e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc863ed, SUC_lexicon_tags_33 },
    { 0xac863ee, SUC_lexicon_tags_15 },
    { 0xd02863ee, SUC_lexicon_tags_15 },
    { 0x232863ef, SUC_lexicon_tags_37 },
    { 0x7d4863f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb84863f3, SUC_lexicon_tags_10 },
    { 0x93c863f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8863f7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e863fa, SUC_lexicon_tags_25 },
    { 0x30a863fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x452863fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f686400, SUC_lexicon_tags_0 },
    { 0x90486400, SUC_lexicon_tags_37 },
    { 0xcea86402, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b486404, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c86407, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa628640a, SUC_lexicon_tags_32 },
    { 0x6528640b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba86410, SUC_lexicon_tags_25 },
    { 0x5c086410, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f886413, SUC_lexicon_tags_37 },
    { 0x42c86413, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf486416, SUC_lexicon_tags_26 },
    { 0xb9086417, SUC_lexicon_tags_14 },
    { 0x64086418, SUC_lexicon_tags_61 },
    { 0x66886416, SUC_lexicon_tags_104 },
    { 0xc9c8641a, SUC_lexicon_tags_60 },
    { 0x20a86419, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8928641d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea86420, SUC_lexicon_tags_19 },
    { 0x9dc86420, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba86425, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc8642c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4328642e, SUC_lexicon_tags_17 },
    { 0xde48642f, SUC_lexicon_tags_12 },
    { 0x6e886430, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42886434, SUC_lexicon_tags_25 },
    { 0x89c86435, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83e86437, SUC_lexicon_tags_7 },
    { 0x70886438, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d28643d, SUC_lexicon_tags_14 },
    { 0x74e8643e, SUC_lexicon_tags_2 },
    { 0xb3c8643f, SUC_lexicon_tags_47 },
    { 0xe3c8643d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a86443, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e86449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1188644f, SUC_lexicon_tags_11 },
    { 0xc8644f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c86453, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78886456, SUC_lexicon_tags_12 },
    { 0xdce86457, SUC_lexicon_tags_25 },
    { 0xd7686456, SUC_lexicon_tags_27 },
    { 0x2286456, SUC_lexicon_tags_77 },
    { 0x3608645a, SUC_lexicon_tags_56 },
    { 0xa4e8645b, SUC_lexicon_tags_12 },
    { 0x5e28645c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f08645e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb1e86460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55286467, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf868646b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c48646e, SUC_lexicon_tags_16 },
    { 0x5848646e, SUC_lexicon_tags_17 },
    { 0xf788646f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3d086472, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6886475, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a88647b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e8647e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f286481, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa3e86483, SUC_lexicon_tags_25 },
    { 0x4fe86484, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8686486, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32686488, SUC_lexicon_tags_7 },
    { 0x39086488, SUC_lexicon_tags_38 },
    { 0x2568648a, SUC_lexicon_tags_23 },
    { 0x26a8648b, SUC_lexicon_tags_39 },
    { 0x3288648c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ce8648e, SUC_lexicon_tags_60 },
    { 0xd208648f, SUC_lexicon_tags_5 },
    { 0x21286490, SUC_lexicon_tags_23 },
    { 0x17e86491, SUC_lexicon_tags_27 },
    { 0xcea86491, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc286495, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e86499, SUC_lexicon_tags_17 },
    { 0xa2e8649a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa8649e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48864a2, SUC_lexicon_tags_1 },
    { 0xa4c864a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8864a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92864aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x990864ad, SUC_lexicon_tags_25 },
    { 0x228864ae, SUC_lexicon_tags_0 },
    { 0xd66864af, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x724864b2, SUC_lexicon_tags_2 },
    { 0xb2e864b3, SUC_lexicon_tags_62 },
    { 0x8c6864b3, SUC_lexicon_tags_12 },
    { 0x2a8864b2, SUC_lexicon_tags_12 },
    { 0x1a2864b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x162864b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848864bf, SUC_lexicon_tags_38 },
    { 0xada864c0, SUC_lexicon_tags_60 },
    { 0xb50864c1, SUC_lexicon_tags_25 },
    { 0xe02864c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6e864c4, SUC_lexicon_tags_28 },
    { 0xabc864c5, SUC_lexicon_tags_31 },
    { 0x706864c6, SUC_lexicon_tags_23 },
    { 0x794864c4, SUC_lexicon_tags_62 },
    { 0xc82864c8, SUC_lexicon_tags_1 },
    { 0x492864c9, SUC_lexicon_tags_17 },
    { 0x76a864c5, SUC_lexicon_tags_37 },
    { 0xc42864c7, SUC_lexicon_tags_27 },
    { 0x612864cc, SUC_lexicon_tags_8 },
    { 0x98e864cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982864d2, SUC_lexicon_tags_23 },
    { 0x732864d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800864d9, SUC_lexicon_tags_12 },
    { 0xffa864da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de864dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x830864e0, SUC_lexicon_tags_26 },
    { 0xb48864e1, SUC_lexicon_tags_17 },
    { 0x804864e2, SUC_lexicon_tags_11 },
    { 0x748864e3, SUC_lexicon_tags_4 },
    { 0xce6864e2, SUC_lexicon_tags_2 },
    { 0x624864e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc864ee, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xddc864f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8864f5, SUC_lexicon_tags_45 },
    { 0xb4c864f6, SUC_lexicon_tags_25 },
    { 0x9e6864f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2864fa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e864fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99486506, SUC_lexicon_tags_7 },
    { 0xfa086507, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a8650c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb708650e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b086514, SUC_lexicon_tags_31 },
    { 0x8ac86515, SUC_lexicon_tags_0 },
    { 0x18a86514, SUC_lexicon_tags_15 },
    { 0x24086517, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x79286519, SUC_lexicon_tags_38 },
    { 0xe008651a, SUC_lexicon_tags_17 },
    { 0x2028651b, SUC_lexicon_tags_15 },
    { 0x7948651c, SUC_lexicon_tags_1 },
    { 0x6548651b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x76a8651f, SUC_lexicon_tags_25 },
    { 0xd0886520, SUC_lexicon_tags_84 },
    { 0xd6886520, SUC_lexicon_tags_77 },
    { 0xcae86522, SUC_lexicon_tags_6 },
    { 0xec286523, SUC_lexicon_tags_33 },
    { 0x57086522, SUC_lexicon_tags_104 },
    { 0xc686525, SUC_lexicon_tags_31 },
    { 0xb6e86520, SUC_lexicon_tags_82 },
    { 0x18886527, SUC_lexicon_tags_49 },
    { 0xd2a86528, SUC_lexicon_tags_31 },
    { 0x65a86529, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xbf48652b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5ae8652d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7868652f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e86534, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27086536, SUC_lexicon_tags_77 },
    { 0xf6886537, SUC_lexicon_tags_11 },
    { 0x8f486537, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a8653e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10286540, SUC_lexicon_tags_1 },
    { 0x6686541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3886543, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb48654a, SUC_lexicon_tags_15 },
    { 0xe6e8654b, SUC_lexicon_tags_25 },
    { 0x2028654c, SUC_lexicon_tags_1 },
    { 0x7a8654c, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x71886550, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7c86552, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa86556, SUC_lexicon_tags_25 },
    { 0x97886557, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd208655b, SUC_lexicon_tags_33 },
    { 0x7da8655c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c686562, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc86568, SUC_lexicon_tags_17 },
    { 0xcf086569, SUC_lexicon_tags_2 },
    { 0xc568656a, SUC_lexicon_tags_84 },
    { 0xde68656b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a86571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a86574, SUC_lexicon_tags_7 },
    { 0x86086575, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8228657f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab286588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa488658f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a86597, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7c686599, SUC_lexicon_tags_49 },
    { 0xedc8659a, SUC_lexicon_tags_9 },
    { 0x3868659b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe308659d, SUC_lexicon_tags_98 },
    { 0xee88659e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa34865a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3c865a2, SUC_lexicon_tags_1 },
    { 0xd48865a3, SUC_lexicon_tags_33 },
    { 0xb6e865a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x458865a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x736865a8, SUC_lexicon_tags_2 },
    { 0xcfc865a9, SUC_lexicon_tags_37 },
    { 0xf22865a9, SUC_lexicon_tags_7 },
    { 0x1be865ab, SUC_lexicon_tags_61 },
    { 0x68e865a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x98865ae, SUC_lexicon_tags_9 },
    { 0xa48865ae, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe58865b1, SUC_lexicon_tags_8 },
    { 0x3be865b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba865b5, SUC_lexicon_tags_198 },
    { 0x3d2865b6, SUC_lexicon_tags_9 },
    { 0x73c865b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77a865b9, SUC_lexicon_tags_1 },
    { 0xd66865ba, SUC_lexicon_tags_25 },
    { 0xdb8865bb, SUC_lexicon_tags_1 },
    { 0x80865ba, SUC_lexicon_tags_46 },
    { 0xf74865b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x650865bf, SUC_lexicon_tags_7 },
    { 0xfc2865c0, SUC_lexicon_tags_37 },
    { 0xab8865c1, SUC_lexicon_tags_14 },
    { 0x45e865c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacc865c4, SUC_lexicon_tags_6 },
    { 0x5e6865c4, SUC_lexicon_tags_37 },
    { 0xc6865c6, SUC_lexicon_tags_18 },
    { 0x22a865c7, SUC_lexicon_tags_37 },
    { 0x42e865c8, SUC_lexicon_tags_7 },
    { 0x9c8865c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec865cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e865d2, SUC_lexicon_tags_14 },
    { 0x708865d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e865d7, SUC_lexicon_tags_15 },
    { 0xab0865d7, SUC_lexicon_tags_25 },
    { 0xaa8865d9, SUC_lexicon_tags_30 },
    { 0xef2865da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08865e1, SUC_lexicon_tags_37 },
    { 0xe5a865e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0865e6, SUC_lexicon_tags_17 },
    { 0x68c865e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82865eb, SUC_lexicon_tags_1 },
    { 0x3aa865eb, SUC_lexicon_tags_6 },
    { 0xcae865ec, SUC_lexicon_tags_0 },
    { 0xe78865ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x99c865f0, SUC_lexicon_tags_32 },
    { 0x8e4865f1, SUC_lexicon_tags_65 },
    { 0x384865f2, SUC_lexicon_tags_6 },
    { 0x398865f3, SUC_lexicon_tags_30 },
    { 0xaf2865f4, SUC_lexicon_tags_15 },
    { 0x4b4865f5, SUC_lexicon_tags_7 },
    { 0x1c0865f6, SUC_lexicon_tags_0 },
    { 0xd32865f1, SUC_lexicon_tags_0 },
    { 0x42c865f6, SUC_lexicon_tags_0 },
    { 0x162865f3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a865fd, SUC_lexicon_tags_47 },
    { 0x68865fe, SUC_lexicon_tags_37 },
    { 0xe94865fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce86603, SUC_lexicon_tags_33 },
    { 0x44486604, SUC_lexicon_tags_12 },
    { 0x44286605, SUC_lexicon_tags_55 },
    { 0xd3886605, SUC_lexicon_tags_66 },
    { 0xe6a86605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7908660a, SUC_lexicon_tags_37 },
    { 0x748660b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb928660f, SUC_lexicon_tags_17 },
    { 0xde88660f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50286613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0886616, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3ee86618, SUC_lexicon_tags_49 },
    { 0x15886619, SUC_lexicon_tags_12 },
    { 0x8e08661a, SUC_lexicon_tags_47 },
    { 0x5fa8661b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e8661f, SUC_lexicon_tags_37 },
    { 0xaf086620, SUC_lexicon_tags_15 },
    { 0x50e86620, SUC_lexicon_tags_1 },
    { 0xb7486620, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8d686624, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e086626, SUC_lexicon_tags_79 },
    { 0x49a86627, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8888662b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a8662f, SUC_lexicon_tags_11 },
    { 0xea086630, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb686638, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c86640, SUC_lexicon_tags_1 },
    { 0xa4c86641, SUC_lexicon_tags_29 },
    { 0xdd686642, SUC_lexicon_tags_17 },
    { 0xa3c86642, SUC_lexicon_tags_6 },
    { 0x97a86642, SUC_lexicon_tags_37 },
    { 0x41486645, SUC_lexicon_tags_14 },
    { 0x36286645, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x39286648, SUC_lexicon_tags_124 },
    { 0x4486649, SUC_lexicon_tags_45 },
    { 0x3fa8664a, SUC_lexicon_tags_49 },
    { 0x2048664b, SUC_lexicon_tags_25 },
    { 0xee88664b, SUC_lexicon_tags_37 },
    { 0x468664d, SUC_lexicon_tags_45 },
    { 0x5548664e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24086654, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea86656, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba286658, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e48665f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d886662, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a86665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c86672, SUC_lexicon_tags_60 },
    { 0x12086673, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de86677, SUC_lexicon_tags_17 },
    { 0xc686678, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48667c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0486680, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf686686, SUC_lexicon_tags_47 },
    { 0x6486687, SUC_lexicon_tags_7 },
    { 0xd9286686, SUC_lexicon_tags_21 },
    { 0x46a86688, SUC_lexicon_tags_25 },
    { 0x8a48668a, SUC_lexicon_tags_37 },
    { 0x99886689, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71086691, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd286693, SUC_lexicon_tags_48 },
    { 0x1e86694, SUC_lexicon_tags_12 },
    { 0xd6c86693, SUC_lexicon_tags_23 },
    { 0xdfe86696, SUC_lexicon_tags_11 },
    { 0x27a86697, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c48669c, SUC_lexicon_tags_17 },
    { 0x2f28669c, SUC_lexicon_tags_25 },
    { 0xfd28669e, SUC_lexicon_tags_25 },
    { 0xbb48669e, SUC_lexicon_tags_11 },
    { 0x446866a0, SUC_lexicon_tags_37 },
    { 0x652866a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x750866a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdee866a5, SUC_lexicon_tags_15 },
    { 0x572866a6, SUC_lexicon_tags_14 },
    { 0xbd2866a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x976866aa, SUC_lexicon_tags_16 },
    { 0x69c866ab, SUC_lexicon_tags_19 },
    { 0xd6866ac, SUC_lexicon_tags_17 },
    { 0xbd8866ad, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14866b2, SUC_lexicon_tags_49 },
    { 0x65c866b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbbc866b5, SUC_lexicon_tags_25 },
    { 0xe2c866b6, SUC_lexicon_tags_3 },
    { 0x33c866b5, SUC_lexicon_tags_1 },
    { 0x7b4866b5, SUC_lexicon_tags_17 },
    { 0xeca866b6, SUC_lexicon_tags_6 },
    { 0xe4c866b6, SUC_lexicon_tags_21 },
    { 0x29c866bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4866bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e866c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a866ca, SUC_lexicon_tags_65 },
    { 0xa40866cb, SUC_lexicon_tags_649 },
    { 0x76866cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e866d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0866da, SUC_lexicon_tags_4 },
    { 0x8e0866db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc5e866dd, SUC_lexicon_tags_14 },
    { 0x85c866de, SUC_lexicon_tags_0 },
    { 0x6d8866df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1ba866e1, SUC_lexicon_tags_30 },
    { 0xe74866e2, SUC_lexicon_tags_37 },
    { 0xd5e866e2, SUC_lexicon_tags_2 },
    { 0xeee866e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d8866e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e866ea, SUC_lexicon_tags_15 },
    { 0x104866eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8866f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xece866f9, SUC_lexicon_tags_31 },
    { 0xeba866fa, SUC_lexicon_tags_21 },
    { 0x868866fa, SUC_lexicon_tags_21 },
    { 0x51e866fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x87c866fe, SUC_lexicon_tags_47 },
    { 0x47c866ff, SUC_lexicon_tags_37 },
    { 0x8fa86700, SUC_lexicon_tags_27 },
    { 0x16c86701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35886706, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8286709, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb68670b, SUC_lexicon_tags_10 },
    { 0xdaa8670b, SUC_lexicon_tags_2 },
    { 0xd048670b, SUC_lexicon_tags_63 },
    { 0x2e8670c, SUC_lexicon_tags_0 },
    { 0xe108670f, SUC_lexicon_tags_11 },
    { 0x64486710, SUC_lexicon_tags_17 },
    { 0x4486711, SUC_lexicon_tags_0 },
    { 0xaf086711, SUC_lexicon_tags_11 },
    { 0x62686712, SUC_lexicon_tags_17 },
    { 0xacc86714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77486718, SUC_lexicon_tags_25 },
    { 0xe5886719, SUC_lexicon_tags_27 },
    { 0x5286719, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3088671c, SUC_lexicon_tags_6 },
    { 0xb908671d, SUC_lexicon_tags_11 },
    { 0x8868671d, SUC_lexicon_tags_11 },
    { 0x6528671d, SUC_lexicon_tags_2 },
    { 0x7e48671c, SUC_lexicon_tags_33 },
    { 0x55e8671c, SUC_lexicon_tags_27 },
    { 0xe1a86722, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xda686724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1348672b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88c8672d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7368672f, SUC_lexicon_tags_1 },
    { 0xa0086730, SUC_lexicon_tags_29 },
    { 0xf4086730, SUC_lexicon_tags_22 },
    { 0x1428672f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f286737, SUC_lexicon_tags_15 },
    { 0x61086738, SUC_lexicon_tags_9 },
    { 0x30686739, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a8673d, SUC_lexicon_tags_1 },
    { 0xc928673e, SUC_lexicon_tags_7 },
    { 0x7228673d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c886749, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6828674b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca8674e, SUC_lexicon_tags_17 },
    { 0xc528674f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c686752, SUC_lexicon_tags_1 },
    { 0x86886753, SUC_lexicon_tags_4 },
    { 0xdf486753, SUC_lexicon_tags_20 },
    { 0x80286754, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808675a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c86762, SUC_lexicon_tags_60 },
    { 0x27486762, SUC_lexicon_tags_15 },
    { 0x52686764, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c86769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb828676f, SUC_lexicon_tags_30 },
    { 0x4086770, SUC_lexicon_tags_15 },
    { 0x4d086771, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14486774, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e86778, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8668677b, SUC_lexicon_tags_26 },
    { 0x80a8677c, SUC_lexicon_tags_15 },
    { 0x3d28677b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa748677f, SUC_lexicon_tags_12 },
    { 0xae28677f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e886785, SUC_lexicon_tags_37 },
    { 0x79286786, SUC_lexicon_tags_26 },
    { 0xdf886786, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xde486789, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf348678d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc86790, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa7286792, SUC_lexicon_tags_11 },
    { 0x73e86792, SUC_lexicon_tags_1 },
    { 0x94086794, SUC_lexicon_tags_60 },
    { 0x98a86795, SUC_lexicon_tags_57 },
    { 0xb4c86796, SUC_lexicon_tags_29 },
    { 0x60286797, SUC_lexicon_tags_57 },
    { 0x90c86798, SUC_lexicon_tags_25 },
    { 0x59c86798, SUC_lexicon_tags_11 },
    { 0x79686798, SUC_lexicon_tags_11 },
    { 0x42686797, SUC_lexicon_tags_108 },
    { 0x9968679a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce867a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0867a7, SUC_lexicon_tags_11 },
    { 0x638867a7, SUC_lexicon_tags_29 },
    { 0xb9a867a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590867ad, SUC_lexicon_tags_37 },
    { 0xe3e867ae, SUC_lexicon_tags_0 },
    { 0x464867ad, SUC_lexicon_tags_32 },
    { 0x566867af, SUC_lexicon_tags_45 },
    { 0x788867b1, SUC_lexicon_tags_33 },
    { 0x7f8867b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14e867b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c867b7, SUC_lexicon_tags_2 },
    { 0x868867b8, SUC_lexicon_tags_17 },
    { 0x7b8867b8, SUC_lexicon_tags_25 },
    { 0x6e867b9, SUC_lexicon_tags_37 },
    { 0xc72867bb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8867c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa867c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac867c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x726867cc, SUC_lexicon_tags_26 },
    { 0xf8e867cd, SUC_lexicon_tags_12 },
    { 0xbda867ce, SUC_lexicon_tags_25 },
    { 0x3e867cf, SUC_lexicon_tags_17 },
    { 0xee2867d0, SUC_lexicon_tags_39 },
    { 0xb08867d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x558867d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe867d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782867da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14867dd, SUC_lexicon_tags_17 },
    { 0xc4a867de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8867e3, SUC_lexicon_tags_543 },
    { 0xfb6867e3, SUC_lexicon_tags_11 },
    { 0xdbe867e3, SUC_lexicon_tags_11 },
    { 0xc48867e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x198867e9, SUC_lexicon_tags_1 },
    { 0x68c867ea, SUC_lexicon_tags_2 },
    { 0x6f8867eb, SUC_lexicon_tags_84 },
    { 0xe867ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394867f1, SUC_lexicon_tags_17 },
    { 0x6be867f2, SUC_lexicon_tags_17 },
    { 0x81e867f3, SUC_lexicon_tags_63 },
    { 0x1a4867f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x280867f7, SUC_lexicon_tags_37 },
    { 0xfee867f8, SUC_lexicon_tags_32 },
    { 0x830867f7, SUC_lexicon_tags_29 },
    { 0x978867f8, SUC_lexicon_tags_27 },
    { 0xbe6867f9, SUC_lexicon_tags_19 },
    { 0xbf4867f9, SUC_lexicon_tags_23 },
    { 0x11a867f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5686801, SUC_lexicon_tags_27 },
    { 0xbfe86802, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4886804, SUC_lexicon_tags_2 },
    { 0xe0886805, SUC_lexicon_tags_31 },
    { 0x4ba86806, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f886809, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9548680d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a286812, SUC_lexicon_tags_43 },
    { 0x8ea86813, SUC_lexicon_tags_17 },
    { 0xc5886813, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x44886817, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7086819, SUC_lexicon_tags_15 },
    { 0xa828681a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e8681e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c286822, SUC_lexicon_tags_64 },
    { 0xe4486822, SUC_lexicon_tags_37 },
    { 0x15c86824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c86827, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x48a86829, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62c8682b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc48682e, SUC_lexicon_tags_14 },
    { 0x2e88682f, SUC_lexicon_tags_23 },
    { 0xf668682f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53286833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe886837, SUC_lexicon_tags_0 },
    { 0x78086837, SUC_lexicon_tags_29 },
    { 0xae686839, SUC_lexicon_tags_61 },
    { 0x2608683a, SUC_lexicon_tags_47 },
    { 0xda686839, SUC_lexicon_tags_27 },
    { 0x288683c, SUC_lexicon_tags_8 },
    { 0x1de8683d, SUC_lexicon_tags_25 },
    { 0xfd686839, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad086840, SUC_lexicon_tags_82 },
    { 0x14c86840, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb086845, SUC_lexicon_tags_16 },
    { 0xcc86846, SUC_lexicon_tags_25 },
    { 0x23c86846, SUC_lexicon_tags_37 },
    { 0x19c86846, SUC_lexicon_tags_54 },
    { 0xd486845, SUC_lexicon_tags_41 },
    { 0x22886848, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c8684f, SUC_lexicon_tags_43 },
    { 0x7988684f, SUC_lexicon_tags_25 },
    { 0x1a286851, SUC_lexicon_tags_63 },
    { 0x2e286851, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c86856, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd886858, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xace8685b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca68685e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52486862, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4686865, SUC_lexicon_tags_37 },
    { 0x17486866, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8486868, SUC_lexicon_tags_4 },
    { 0x63286868, SUC_lexicon_tags_41 },
    { 0xc1486868, SUC_lexicon_tags_1 },
    { 0xb488686b, SUC_lexicon_tags_17 },
    { 0x31e8686c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0486873, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72886877, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e8687a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc86880, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2886882, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c686884, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14686889, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf688688d, SUC_lexicon_tags_17 },
    { 0x9fa8688e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc686890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0286896, SUC_lexicon_tags_4 },
    { 0xf6e86897, SUC_lexicon_tags_15 },
    { 0x19486897, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3028689b, SUC_lexicon_tags_14 },
    { 0xc548689c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb1c8689e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x90868a1, SUC_lexicon_tags_14 },
    { 0xe5a868a2, SUC_lexicon_tags_37 },
    { 0x284868a3, SUC_lexicon_tags_37 },
    { 0xb2868a4, SUC_lexicon_tags_23 },
    { 0xe1c868a5, SUC_lexicon_tags_25 },
    { 0x862868a3, SUC_lexicon_tags_1 },
    { 0xb6c868a1, SUC_lexicon_tags_25 },
    { 0x73c868a6, SUC_lexicon_tags_17 },
    { 0x858868a2, SUC_lexicon_tags_37 },
    { 0x2e8868aa, SUC_lexicon_tags_25 },
    { 0x51c868ab, SUC_lexicon_tags_0 },
    { 0x2e4868a1, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x3e6868ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x676868b1, SUC_lexicon_tags_2 },
    { 0x522868b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee868b5, SUC_lexicon_tags_7 },
    { 0xee8868b5, SUC_lexicon_tags_37 },
    { 0x8de868b7, SUC_lexicon_tags_22 },
    { 0x7c0868b6, SUC_lexicon_tags_45 },
    { 0xac4868b9, SUC_lexicon_tags_34 },
    { 0x178868b6, SUC_lexicon_tags_114 },
    { 0xa6a868bb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x470868bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc868c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44868c8, SUC_lexicon_tags_13 },
    { 0x1d2868c9, SUC_lexicon_tags_2 },
    { 0x984868c8, SUC_lexicon_tags_11 },
    { 0x364868c9, SUC_lexicon_tags_15 },
    { 0xdd0868cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8868cf, SUC_lexicon_tags_23 },
    { 0xf3a868cf, SUC_lexicon_tags_30 },
    { 0x9b0868d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0868d8, SUC_lexicon_tags_65 },
    { 0x7b4868d8, SUC_lexicon_tags_28 },
    { 0xb7e868da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa0868dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x262868df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2868e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8868e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee868ed, SUC_lexicon_tags_63 },
    { 0x7c8868ed, SUC_lexicon_tags_10 },
    { 0xf06868ee, SUC_lexicon_tags_46 },
    { 0xd12868ed, SUC_lexicon_tags_37 },
    { 0xe96868f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2868f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc868fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e868fd, SUC_lexicon_tags_17 },
    { 0xcf2868fe, SUC_lexicon_tags_45 },
    { 0x5fc868ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa86903, SUC_lexicon_tags_17 },
    { 0xda086904, SUC_lexicon_tags_43 },
    { 0x3de86903, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac886907, SUC_lexicon_tags_37 },
    { 0x55e86907, SUC_lexicon_tags_10 },
    { 0x91686907, SUC_lexicon_tags_107 },
    { 0xac88690a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa488690c, SUC_lexicon_tags_46 },
    { 0xe6c8690d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d686913, SUC_lexicon_tags_49 },
    { 0xae486914, SUC_lexicon_tags_37 },
    { 0x1d286915, SUC_lexicon_tags_34 },
    { 0x28086915, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e8691a, SUC_lexicon_tags_25 },
    { 0xd728691a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa86922, SUC_lexicon_tags_60 },
    { 0x33086922, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25286929, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba8692e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd286933, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75c86935, SUC_lexicon_tags_0 },
    { 0x91a86936, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e86939, SUC_lexicon_tags_38 },
    { 0x29686939, SUC_lexicon_tags_47 },
    { 0x4c28693b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4686940, SUC_lexicon_tags_21 },
    { 0x7be86941, SUC_lexicon_tags_10 },
    { 0x27e86941, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x31086944, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65486947, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc708694b, SUC_lexicon_tags_33 },
    { 0xed88694c, SUC_lexicon_tags_15 },
    { 0xce8694b, SUC_lexicon_tags_15 },
    { 0x85c8694e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4286954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6886956, SUC_lexicon_tags_37 },
    { 0x8ac86957, SUC_lexicon_tags_63 },
    { 0x8da86957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd488695b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf848695d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc28695f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x13086962, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e86967, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee8696a, SUC_lexicon_tags_59 },
    { 0xde88696b, SUC_lexicon_tags_2 },
    { 0xf6e8696c, SUC_lexicon_tags_7 },
    { 0x14e8696c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xefc8696f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee86975, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe008697a, SUC_lexicon_tags_15 },
    { 0x74e8697b, SUC_lexicon_tags_627 },
    { 0x6f28697c, SUC_lexicon_tags_12 },
    { 0x6d48697d, SUC_lexicon_tags_7 },
    { 0xfbc8697c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe86982, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee886984, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a86987, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5068698b, SUC_lexicon_tags_12 },
    { 0x9a88698c, SUC_lexicon_tags_9 },
    { 0x9148698b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf886991, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba086997, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8d486999, SUC_lexicon_tags_27 },
    { 0xe228699a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x70c8699c, SUC_lexicon_tags_31 },
    { 0x55a8699d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb968699f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60869a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0869a9, SUC_lexicon_tags_0 },
    { 0x566869aa, SUC_lexicon_tags_37 },
    { 0xd96869ab, SUC_lexicon_tags_9 },
    { 0x2b2869ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18869b2, SUC_lexicon_tags_11 },
    { 0xbaa869b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf2869b5, SUC_lexicon_tags_43 },
    { 0x502869b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc0869b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c869bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be869c1, SUC_lexicon_tags_11 },
    { 0x982869c2, SUC_lexicon_tags_47 },
    { 0x730869c1, SUC_lexicon_tags_2 },
    { 0xff6869c3, SUC_lexicon_tags_23 },
    { 0xe92869c4, SUC_lexicon_tags_11 },
    { 0x8b8869c6, SUC_lexicon_tags_46 },
    { 0xd5a869c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ae869c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4da869cb, SUC_lexicon_tags_55 },
    { 0x3ba869cb, SUC_lexicon_tags_0 },
    { 0xf1c869cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2869d1, SUC_lexicon_tags_6 },
    { 0x8e869d1, SUC_lexicon_tags_37 },
    { 0xaa0869d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08869d8, SUC_lexicon_tags_15 },
    { 0x23c869d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc0869db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc869e1, SUC_lexicon_tags_25 },
    { 0x8a2869e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e869ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e869f0, SUC_lexicon_tags_12 },
    { 0x6b8869f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a869f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x652869f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd96869f8, SUC_lexicon_tags_28 },
    { 0xcca869f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22869fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51686a01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9086a08, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x80686a0b, SUC_lexicon_tags_0 },
    { 0xdfc86a0c, SUC_lexicon_tags_17 },
    { 0xd6486a0c, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba86a11, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0686a16, SUC_lexicon_tags_28 },
    { 0xcc486a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41086a1e, SUC_lexicon_tags_25 },
    { 0xc5086a1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e86a2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61886a2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e86a33, SUC_lexicon_tags_107 },
    { 0xe9686a34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86a86a36, SUC_lexicon_tags_29 },
    { 0xc0e86a36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74686a3c, SUC_lexicon_tags_25 },
    { 0xc086a3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbb686a3f, SUC_lexicon_tags_37 },
    { 0xa7c86a40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75c86a42, SUC_lexicon_tags_36 },
    { 0x76286a43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb086a48, SUC_lexicon_tags_19 },
    { 0xb3e86a48, SUC_lexicon_tags_28 },
    { 0x29286a4a, SUC_lexicon_tags_11 },
    { 0x2a86a4a, SUC_lexicon_tags_9 },
    { 0x8086a4a, SUC_lexicon_tags_20 },
    { 0x9ca86a4d, SUC_lexicon_tags_17 },
    { 0x39c86a4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec86a51, SUC_lexicon_tags_17 },
    { 0xace86a52, SUC_lexicon_tags_15 },
    { 0x6d486a53, SUC_lexicon_tags_15 },
    { 0x58a86a54, SUC_lexicon_tags_27 },
    { 0x43086a55, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe686a59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1e86a5b, SUC_lexicon_tags_15 },
    { 0xd2486a5c, SUC_lexicon_tags_12 },
    { 0x71e86a5c, SUC_lexicon_tags_0 },
    { 0x91486a5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa3a86a60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c886a64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2286a69, SUC_lexicon_tags_17 },
    { 0xe6286a69, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9286a74, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe86a7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x97c86a7f, SUC_lexicon_tags_63 },
    { 0xec086a80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c86a85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c86a8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x96086a90, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a86a93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1486a96, SUC_lexicon_tags_7 },
    { 0x4ec86a96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e86a9b, SUC_lexicon_tags_37 },
    { 0xb4286a9c, SUC_lexicon_tags_38 },
    { 0xf6a86a9c, SUC_lexicon_tags_19 },
    { 0x89886a9c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb086aa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a86aa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c86aae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62086ab0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c86ab3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcc286ab5, SUC_lexicon_tags_19 },
    { 0xce486ab5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c86ab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a86ac5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc86ac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d486acc, SUC_lexicon_tags_28 },
    { 0xdbc86acd, SUC_lexicon_tags_37 },
    { 0xc1c86ace, SUC_lexicon_tags_63 },
    { 0x72086acf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x59c86ad1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e086ad3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x96086ad5, SUC_lexicon_tags_41 },
    { 0x20286ad6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc86ad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d486adb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e86ae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f886ae4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9286aec, SUC_lexicon_tags_65 },
    { 0x70086aed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x73286af0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a86af3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11886afc, SUC_lexicon_tags_65 },
    { 0xbee86afd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63286b03, SUC_lexicon_tags_2 },
    { 0xfd686b03, SUC_lexicon_tags_11 },
    { 0xd4486b05, SUC_lexicon_tags_42 },
    { 0xdc486b03, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x73686b08, SUC_lexicon_tags_11 },
    { 0xa4a86b08, SUC_lexicon_tags_43 },
    { 0x14086b0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17086b0e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35286b14, SUC_lexicon_tags_65 },
    { 0x60486b15, SUC_lexicon_tags_11 },
    { 0xbd486b16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb086b1a, SUC_lexicon_tags_11 },
    { 0xeb486b1a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6ee86b1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64686b23, SUC_lexicon_tags_2 },
    { 0x22686b23, SUC_lexicon_tags_14 },
    { 0x9d086b25, SUC_lexicon_tags_63 },
    { 0xf3e86b25, SUC_lexicon_tags_31 },
    { 0x18e86b27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35486b2b, SUC_lexicon_tags_37 },
    { 0x7e486b2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1286b2e, SUC_lexicon_tags_94 },
    { 0x62486b2f, SUC_lexicon_tags_23 },
    { 0xf4686b30, SUC_lexicon_tags_11 },
    { 0x10486b2f, SUC_lexicon_tags_25 },
    { 0x6e886b32, SUC_lexicon_tags_12 },
    { 0xb7886b32, SUC_lexicon_tags_14 },
    { 0x82086b2e, SUC_lexicon_tags_37 },
    { 0xb2286b35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e86b38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb686b3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc86b43, SUC_lexicon_tags_15 },
    { 0x55286b43, SUC_lexicon_tags_37 },
    { 0x94886b45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc886b4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a86b4d, SUC_lexicon_tags_17 },
    { 0x9d486b4e, SUC_lexicon_tags_63 },
    { 0x66486b4f, SUC_lexicon_tags_7 },
    { 0x4fe86b4f, SUC_lexicon_tags_2 },
    { 0xd0486b51, SUC_lexicon_tags_0 },
    { 0x8d686b51, SUC_lexicon_tags_17 },
    { 0x83086b53, SUC_lexicon_tags_7 },
    { 0xca086b4f, SUC_lexicon_tags_7 },
    { 0xca286b4e, SUC_lexicon_tags_26 },
    { 0x2c686b56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe886b5a, SUC_lexicon_tags_31 },
    { 0xa6886b5a, SUC_lexicon_tags_0 },
    { 0xcc086b5c, SUC_lexicon_tags_25 },
    { 0x3dc86b5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5686b61, SUC_lexicon_tags_14 },
    { 0x1aa86b61, SUC_lexicon_tags_47 },
    { 0x48e86b63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa886b67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c86b6f, SUC_lexicon_tags_35 },
    { 0xfce86b70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b486b75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c86b79, SUC_lexicon_tags_7 },
    { 0x16e86b79, SUC_lexicon_tags_15 },
    { 0x7a86b7b, SUC_lexicon_tags_9 },
    { 0xa7086b7c, SUC_lexicon_tags_60 },
    { 0xfa686b7d, SUC_lexicon_tags_7 },
    { 0x59886b79, SUC_lexicon_tags_82 },
    { 0x9c86b7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87086b81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e86b85, SUC_lexicon_tags_15 },
    { 0xf8c86b85, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x34086b88, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xfd286b8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2a86b8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca86b92, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f286b96, SUC_lexicon_tags_2 },
    { 0x58e86b96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb286b9b, SUC_lexicon_tags_25 },
    { 0x6ce86b9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68086ba1, SUC_lexicon_tags_17 },
    { 0x23686ba1, SUC_lexicon_tags_42 },
    { 0xc2086ba2, SUC_lexicon_tags_11 },
    { 0xbe086ba4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c686ba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0686bac, SUC_lexicon_tags_37 },
    { 0x4ce86bad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c86bb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e86bb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2ba86bbb, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be86bbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76886bc2, SUC_lexicon_tags_27 },
    { 0x51686bc3, SUC_lexicon_tags_11 },
    { 0x7a886bc4, SUC_lexicon_tags_43 },
    { 0x88486bc4, SUC_lexicon_tags_7 },
    { 0x38a86bc6, SUC_lexicon_tags_47 },
    { 0xbcc86bc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82686bce, SUC_lexicon_tags_74 },
    { 0xc7886bce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58c86bd1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3f286bd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96286bd9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x49a86bdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a486bdd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x21686bdf, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c86be5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9de86be8, SUC_lexicon_tags_82 },
    { 0x1c486be9, SUC_lexicon_tags_34 },
    { 0x35c86bea, SUC_lexicon_tags_60 },
    { 0x8ea86be9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79a86bed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf686bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc86bff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99886c05, SUC_lexicon_tags_23 },
    { 0x75886c06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e286c0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x46c86c0c, SUC_lexicon_tags_14 },
    { 0x63086c0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a686c11, SUC_lexicon_tags_1 },
    { 0xd486c11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61e86c14, SUC_lexicon_tags_7 },
    { 0x5286c15, SUC_lexicon_tags_12 },
    { 0x8ea86c16, SUC_lexicon_tags_12 },
    { 0xe7486c17, SUC_lexicon_tags_25 },
    { 0x26686c16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a86c1b, SUC_lexicon_tags_25 },
    { 0xfb286c1b, SUC_lexicon_tags_31 },
    { 0xea486c1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92086c25, SUC_lexicon_tags_60 },
    { 0xada86c25, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a86c29, SUC_lexicon_tags_27 },
    { 0xc3086c2a, SUC_lexicon_tags_25 },
    { 0x2ec86c29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe86c2e, SUC_lexicon_tags_1 },
    { 0x63a86c2f, SUC_lexicon_tags_14 },
    { 0xa0286c30, SUC_lexicon_tags_1 },
    { 0x80a86c30, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1286c37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a86c3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8086c45, SUC_lexicon_tags_47 },
    { 0xf5486c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37c86c48, SUC_lexicon_tags_30 },
    { 0xd1486c49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79086c4e, SUC_lexicon_tags_6 },
    { 0x8fa86c4e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x46686c51, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x30886c53, SUC_lexicon_tags_7 },
    { 0xa3486c53, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2486c56, SUC_lexicon_tags_1 },
    { 0x55c86c57, SUC_lexicon_tags_12 },
    { 0x64c86c58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeda86c5b, SUC_lexicon_tags_31 },
    { 0x35c86c5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x39486c5f, SUC_lexicon_tags_16 },
    { 0xc286c60, SUC_lexicon_tags_32 },
    { 0x39886c61, SUC_lexicon_tags_15 },
    { 0x42486c61, SUC_lexicon_tags_17 },
    { 0x7c286c61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa86c6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a86c6f, SUC_lexicon_tags_0 },
    { 0xe8486c70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6086c78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd0686c7a, SUC_lexicon_tags_1 },
    { 0xb4e86c7a, SUC_lexicon_tags_12 },
    { 0x2f286c7c, SUC_lexicon_tags_2 },
    { 0x21886c7d, SUC_lexicon_tags_23 },
    { 0x9a86c7e, SUC_lexicon_tags_27 },
    { 0x7ca86c7f, SUC_lexicon_tags_12 },
    { 0xfde86c80, SUC_lexicon_tags_2 },
    { 0x37c86c81, SUC_lexicon_tags_37 },
    { 0x65486c81, SUC_lexicon_tags_37 },
    { 0x7ee86c82, SUC_lexicon_tags_82 },
    { 0xf3e86c84, SUC_lexicon_tags_148 },
    { 0xff886c83, SUC_lexicon_tags_117 },
    { 0xbc086c82, SUC_lexicon_tags_6 },
    { 0x80e86c87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c86c8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6c86c8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e86c91, SUC_lexicon_tags_9 },
    { 0xf3286c91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7486c97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69886c99, SUC_lexicon_tags_30 },
    { 0x65a86c9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda086c9c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de86c9f, SUC_lexicon_tags_87 },
    { 0x3f686ca0, SUC_lexicon_tags_10 },
    { 0x60886ca1, SUC_lexicon_tags_124 },
    { 0x5bc86ca0, SUC_lexicon_tags_1 },
    { 0x13686ca0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ca86ca5, SUC_lexicon_tags_1 },
    { 0x9b486ca6, SUC_lexicon_tags_27 },
    { 0x8c286ca7, SUC_lexicon_tags_12 },
    { 0x8fe86ca5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c486cad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d886cb0, SUC_lexicon_tags_26 },
    { 0xd5886cb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe86cbd, SUC_lexicon_tags_11 },
    { 0x35086cbd, SUC_lexicon_tags_30 },
    { 0x16c86cbf, SUC_lexicon_tags_69 },
    { 0x43a86cc0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c86cca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7886ccd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1086cd3, SUC_lexicon_tags_6 },
    { 0x11686cd3, SUC_lexicon_tags_45 },
    { 0x60886cd5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a86cdc, SUC_lexicon_tags_2 },
    { 0x79286cdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1086ce1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d486cea, SUC_lexicon_tags_7 },
    { 0x6486cea, SUC_lexicon_tags_0 },
    { 0x26e86cec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15e86cee, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec86cf5, SUC_lexicon_tags_1 },
    { 0x80e86cf6, SUC_lexicon_tags_11 },
    { 0xce086cf6, SUC_lexicon_tags_12 },
    { 0xab286cf8, SUC_lexicon_tags_12 },
    { 0x5e86cf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2086cfc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb886cfe, SUC_lexicon_tags_37 },
    { 0xb6886cff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c86d02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6886d04, SUC_lexicon_tags_27 },
    { 0x23086d05, SUC_lexicon_tags_1 },
    { 0x9d086d05, SUC_lexicon_tags_17 },
    { 0xae886d07, SUC_lexicon_tags_15 },
    { 0x86e86d08, SUC_lexicon_tags_38 },
    { 0xc4086d08, SUC_lexicon_tags_14 },
    { 0xd9486d05, SUC_lexicon_tags_15 },
    { 0xa6686d0b, SUC_lexicon_tags_19 },
    { 0xeea86d0c, SUC_lexicon_tags_15 },
    { 0x1c686d0d, SUC_lexicon_tags_43 },
    { 0xac86d0b, SUC_lexicon_tags_4 },
    { 0x71a86d0f, SUC_lexicon_tags_45 },
    { 0x56e86d0b, SUC_lexicon_tags_9 },
    { 0xbaa86d11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c886d13, SUC_lexicon_tags_6 },
    { 0x59486d13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10886d16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c86d19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93c86d1b, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d086d1e, SUC_lexicon_tags_4 },
    { 0x6e886d1f, SUC_lexicon_tags_10 },
    { 0x7a086d1e, SUC_lexicon_tags_17 },
    { 0x2f486d21, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e86d26, SUC_lexicon_tags_17 },
    { 0xb8a86d27, SUC_lexicon_tags_13 },
    { 0x86086d28, SUC_lexicon_tags_15 },
    { 0x68c86d29, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e86d2c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x38e86d2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27286d33, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c286d3c, SUC_lexicon_tags_66 },
    { 0xee886d3d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x71886d3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8886d41, SUC_lexicon_tags_12 },
    { 0xdb686d41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc86d49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd286d51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa86d54, SUC_lexicon_tags_25 },
    { 0x29286d54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa86d5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c86d5f, SUC_lexicon_tags_55 },
    { 0xe6686d5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90c86d62, SUC_lexicon_tags_42 },
    { 0xd1486d63, SUC_lexicon_tags_46 },
    { 0xaa286d62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e86d69, SUC_lexicon_tags_50 },
    { 0x88086d6a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x45086d6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc686d6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf886d78, SUC_lexicon_tags_25 },
    { 0xf8e86d79, SUC_lexicon_tags_11 },
    { 0x32486d7a, SUC_lexicon_tags_14 },
    { 0x1ba86d7b, SUC_lexicon_tags_15 },
    { 0x5b086d7c, SUC_lexicon_tags_4 },
    { 0xdd086d7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae686d80, SUC_lexicon_tags_2 },
    { 0x93a86d81, SUC_lexicon_tags_15 },
    { 0xf4086d81, SUC_lexicon_tags_27 },
    { 0x5f886d83, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff086d87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb886d8a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc6486d8c, SUC_lexicon_tags_79 },
    { 0x8b886d8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc486d92, SUC_lexicon_tags_2 },
    { 0xb3286d92, SUC_lexicon_tags_6 },
    { 0x57886d92, SUC_lexicon_tags_15 },
    { 0x96486d93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe5e86d97, SUC_lexicon_tags_1 },
    { 0xc2e86d98, SUC_lexicon_tags_17 },
    { 0xda086d97, SUC_lexicon_tags_233 },
    { 0xcda86d97, SUC_lexicon_tags_2 },
    { 0xb0486d9b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x2f286d9d, SUC_lexicon_tags_37 },
    { 0xb3c86d9e, SUC_lexicon_tags_7 },
    { 0xe3a86d9d, SUC_lexicon_tags_17 },
    { 0xdde86d9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c886da3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8686da8, SUC_lexicon_tags_1 },
    { 0x6ea86da8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc86dac, SUC_lexicon_tags_1 },
    { 0x40686dac, SUC_lexicon_tags_63 },
    { 0xa6086dac, SUC_lexicon_tags_39 },
    { 0x82886daf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a86db6, SUC_lexicon_tags_12 },
    { 0x8c686db7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3686dbe, SUC_lexicon_tags_31 },
    { 0x62886dbf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a86dc2, SUC_lexicon_tags_14 },
    { 0xf0a86dc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c86dca, SUC_lexicon_tags_19 },
    { 0xc5086dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ea86dcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78a86dcf, SUC_lexicon_tags_15 },
    { 0xa1c86dd0, SUC_lexicon_tags_63 },
    { 0x1c686dd1, SUC_lexicon_tags_38 },
    { 0xb6a86dd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbe86dd4, SUC_lexicon_tags_46 },
    { 0xa9886dd5, SUC_lexicon_tags_5 },
    { 0x2e686dd4, SUC_lexicon_tags_2 },
    { 0xdb086dd7, SUC_lexicon_tags_60 },
    { 0x52686dd6, SUC_lexicon_tags_63 },
    { 0xa6a86dd9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc686ddb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe886ddf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e86de2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95c86de4, SUC_lexicon_tags_25 },
    { 0x40486de5, SUC_lexicon_tags_11 },
    { 0x3a886de5, SUC_lexicon_tags_17 },
    { 0x13a86de7, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x90686de9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc86dec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9686df0, SUC_lexicon_tags_1 },
    { 0xdba86df0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf486df4, SUC_lexicon_tags_39 },
    { 0xf0c86df4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c286df9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x91486dfc, SUC_lexicon_tags_55 },
    { 0xbc686dfc, SUC_lexicon_tags_37 },
    { 0x96086dfc, SUC_lexicon_tags_2 },
    { 0xe4086dff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3886e01, SUC_lexicon_tags_25 },
    { 0xcf886e02, SUC_lexicon_tags_12 },
    { 0x79c86e02, SUC_lexicon_tags_14 },
    { 0xe6c86e04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17486e0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64286e0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x49a86e11, SUC_lexicon_tags_17 },
    { 0xc6a86e11, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a86e15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a086e18, SUC_lexicon_tags_1449 },
    { 0xf0086e19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf086e1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c86e22, SUC_lexicon_tags_12 },
    { 0x41486e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d286e25, SUC_lexicon_tags_37 },
    { 0x1e486e26, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4686e28, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a86e2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1fe86e2f, SUC_lexicon_tags_75 },
    { 0x64c86e2f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8f686e32, SUC_lexicon_tags_64 },
    { 0xc2e86e32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c86e36, SUC_lexicon_tags_883 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be86e3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa86e3d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c086e40, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x30686e42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1086e44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5886e47, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6f286e49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fc86e4b, SUC_lexicon_tags_10 },
    { 0x6a886e4c, SUC_lexicon_tags_8 },
    { 0x6f286e4c, SUC_lexicon_tags_17 },
    { 0xf7886e4b, SUC_lexicon_tags_37 },
    { 0x74486e4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6c86e51, SUC_lexicon_tags_7 },
    { 0xc2e86e52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda86e55, SUC_lexicon_tags_37 },
    { 0x9c86e56, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0486e5a, SUC_lexicon_tags_17 },
    { 0xb8c86e5b, SUC_lexicon_tags_15 },
    { 0xaf486e5c, SUC_lexicon_tags_1 },
    { 0x26086e5c, SUC_lexicon_tags_149 },
    { 0xea086e5e, SUC_lexicon_tags_11 },
    { 0x99a86e5a, SUC_lexicon_tags_16 },
    { 0xbce86e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea86e64, SUC_lexicon_tags_45 },
    { 0xd2a86e65, SUC_lexicon_tags_25 },
    { 0xa5286e66, SUC_lexicon_tags_2 },
    { 0x14e86e64, SUC_lexicon_tags_11 },
    { 0x63e86e68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0086e6b, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0xc5686e6d, SUC_lexicon_tags_26 },
    { 0x4e686e6d, SUC_lexicon_tags_37 },
    { 0x47886e6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6086e72, SUC_lexicon_tags_23 },
    { 0x27886e72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf086e78, SUC_lexicon_tags_36 },
    { 0xdf286e79, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a686e7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7486e7e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a686e80, SUC_lexicon_tags_25 },
    { 0x12486e81, SUC_lexicon_tags_7 },
    { 0x85686e80, SUC_lexicon_tags_33 },
    { 0x85086e83, SUC_lexicon_tags_2 },
    { 0xa4486e84, SUC_lexicon_tags_30 },
    { 0xe286e85, SUC_lexicon_tags_37 },
    { 0x37e86e86, SUC_lexicon_tags_12 },
    { 0x3b886e87, SUC_lexicon_tags_37 },
    { 0x8b086e88, SUC_lexicon_tags_55 },
    { 0x95686e89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a86e8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xec686e8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd486e92, SUC_lexicon_tags_21 },
    { 0x8be86e93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56a86e95, SUC_lexicon_tags_49 },
    { 0xa4086e96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca286e9a, SUC_lexicon_tags_16 },
    { 0xc2286e9a, SUC_lexicon_tags_29 },
    { 0xb6886e9b, SUC_lexicon_tags_87 },
    { 0x25486e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11286e9f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e686ea4, SUC_lexicon_tags_0 },
    { 0x6dc86ea4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41686eaa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x31c86eac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4486eb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc886eb8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4486ebb, SUC_lexicon_tags_23 },
    { 0x22c86ebb, SUC_lexicon_tags_28 },
    { 0x16c86ebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffc86ebf, SUC_lexicon_tags_19 },
    { 0x8ea86ec0, SUC_lexicon_tags_49 },
    { 0x73486ec1, SUC_lexicon_tags_45 },
    { 0x3f086ec2, SUC_lexicon_tags_45 },
    { 0x4486ec2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba86ec7, SUC_lexicon_tags_17 },
    { 0x76886ec8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd086ecd, SUC_lexicon_tags_21 },
    { 0x4fc86ece, SUC_lexicon_tags_2 },
    { 0x38286ecf, SUC_lexicon_tags_38 },
    { 0xd2486ed0, SUC_lexicon_tags_19 },
    { 0xf8e86ed1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x58486ed3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee086ed7, SUC_lexicon_tags_43 },
    { 0xaa686ed7, SUC_lexicon_tags_11 },
    { 0xe1086ed9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c286edc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdc686ede, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd6c86ee0, SUC_lexicon_tags_141 },
    { 0x65886ee0, SUC_lexicon_tags_15 },
    { 0xd5086ee2, SUC_lexicon_tags_65 },
    { 0x3e286ee2, SUC_lexicon_tags_31 },
    { 0x81886ee3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e86ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1c86eea, SUC_lexicon_tags_11 },
    { 0x3ce86eeb, SUC_lexicon_tags_37 },
    { 0x5686eec, SUC_lexicon_tags_42 },
    { 0xfac86eed, SUC_lexicon_tags_19 },
    { 0x57886eec, SUC_lexicon_tags_14 },
    { 0x31e86eef, SUC_lexicon_tags_25 },
    { 0x42086ef0, SUC_lexicon_tags_47 },
    { 0x4c486ef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca886ef3, SUC_lexicon_tags_11 },
    { 0x3d286ef4, SUC_lexicon_tags_25 },
    { 0xfec86ef4, SUC_lexicon_tags_23 },
    { 0xdf886ef4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45a86ef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c86efb, SUC_lexicon_tags_13 },
    { 0xcc486efc, SUC_lexicon_tags_10 },
    { 0x71886efc, SUC_lexicon_tags_33 },
    { 0x50686efe, SUC_lexicon_tags_25 },
    { 0x94a86efb, SUC_lexicon_tags_45 },
    { 0x85686efd, SUC_lexicon_tags_68 },
    { 0xef686f01, SUC_lexicon_tags_14 },
    { 0x5e86f01, SUC_lexicon_tags_55 },
    { 0xe6e86f01, SUC_lexicon_tags_65 },
    { 0xdb086f04, SUC_lexicon_tags_60 },
    { 0x58086f05, SUC_lexicon_tags_38 },
    { 0xc7686f05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38a86f08, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71886f0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d286f11, SUC_lexicon_tags_38 },
    { 0xcb486f11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42486f15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67486f17, SUC_lexicon_tags_2 },
    { 0xa4a86f17, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdc486f1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa886f1f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2286f22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9486f2b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe686f30, SUC_lexicon_tags_15 },
    { 0x21e86f30, SUC_lexicon_tags_63 },
    { 0x15c86f32, SUC_lexicon_tags_4 },
    { 0x9886f32, SUC_lexicon_tags_7 },
    { 0xe0086f34, SUC_lexicon_tags_23 },
    { 0x9ce86f34, SUC_lexicon_tags_17 },
    { 0x5ac86f36, SUC_lexicon_tags_1 },
    { 0x45c86f37, SUC_lexicon_tags_70 },
    { 0x38686f33, SUC_lexicon_tags_1 },
    { 0x3d486f39, SUC_lexicon_tags_6 },
    { 0x82a86f3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc86f46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x22486f48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c86f4d, SUC_lexicon_tags_47 },
    { 0x72e86f4e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb886f53, SUC_lexicon_tags_60 },
    { 0x21886f53, SUC_lexicon_tags_1 },
    { 0x26886f55, SUC_lexicon_tags_15 },
    { 0x77a86f55, SUC_lexicon_tags_25 },
    { 0x4a286f57, SUC_lexicon_tags_33 },
    { 0xa1686f58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x26086f5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a086f5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a86f5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf686f62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e86f67, SUC_lexicon_tags_1 },
    { 0xd8686f68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce886f6d, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x65086f70, SUC_lexicon_tags_1 },
    { 0x2f286f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e86f77, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2086f7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc86f7e, SUC_lexicon_tags_30 },
    { 0x57e86f7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21c86f81, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33686f85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc86f88, SUC_lexicon_tags_42 },
    { 0xf8e86f88, SUC_lexicon_tags_15 },
    { 0x35a86f8a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x67c86f8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69686f8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a86f91, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb2686f93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c686f96, SUC_lexicon_tags_27 },
    { 0x3f686f97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d086f99, SUC_lexicon_tags_8 },
    { 0x10286f99, SUC_lexicon_tags_63 },
    { 0xa6a86f99, SUC_lexicon_tags_46 },
    { 0x36e86f9c, SUC_lexicon_tags_13 },
    { 0x85c86f9d, SUC_lexicon_tags_16 },
    { 0x7a686f9a, SUC_lexicon_tags_27 },
    { 0x7b086f9e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x23086fa1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4486fa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcce86fa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50286faa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5886faf, SUC_lexicon_tags_19 },
    { 0xbc086fb0, SUC_lexicon_tags_14 },
    { 0xe1e86fb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ac86fb3, SUC_lexicon_tags_11 },
    { 0x35686fb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57886fb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3686fb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6a86fba, SUC_lexicon_tags_46 },
    { 0xd5486fbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x31286fbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81686fc1, SUC_lexicon_tags_14 },
    { 0xbf886fc2, SUC_lexicon_tags_9 },
    { 0x22a86fc3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe1486fc5, SUC_lexicon_tags_63 },
    { 0x80a86fc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1086fc8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a86fcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45886fcf, SUC_lexicon_tags_63 },
    { 0x4ee86fcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba86fd3, SUC_lexicon_tags_11 },
    { 0xd5886fd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65486fd9, SUC_lexicon_tags_617 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a86fdd, SUC_lexicon_tags_43 },
    { 0xe7686fde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f086fe8, SUC_lexicon_tags_45 },
    { 0xdf486fe9, SUC_lexicon_tags_37 },
    { 0x2e886fea, SUC_lexicon_tags_21 },
    { 0x63486feb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3c86fed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76686ff0, SUC_lexicon_tags_42 },
    { 0xbc486ff1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e86ff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde286ff9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfc286ffb, SUC_lexicon_tags_17 },
    { 0x9286ffc, SUC_lexicon_tags_11 },
    { 0x63486ffd, SUC_lexicon_tags_1 },
    { 0x17886ffe, SUC_lexicon_tags_37 },
    { 0x6e686ffd, SUC_lexicon_tags_6 },
    { 0x74a86ffc, SUC_lexicon_tags_131 },
    { 0xc1e86ffc, SUC_lexicon_tags_5 },
    { 0x11686fff, SUC_lexicon_tags_1 },
    { 0x4ca87003, SUC_lexicon_tags_47 },
    { 0xca087000, SUC_lexicon_tags_17 },
    { 0x59087004, SUC_lexicon_tags_21 },
    { 0x9c86ffb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28700b, SUC_lexicon_tags_92 },
    { 0x8508700c, SUC_lexicon_tags_37 },
    { 0xfb68700d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8268700f, SUC_lexicon_tags_8 },
    { 0xdd487010, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x78487012, SUC_lexicon_tags_23 },
    { 0x20887013, SUC_lexicon_tags_30 },
    { 0xf4887013, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41287016, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52087018, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4128701d, SUC_lexicon_tags_14 },
    { 0x8c08701e, SUC_lexicon_tags_16 },
    { 0xd748701f, SUC_lexicon_tags_10 },
    { 0x93e87020, SUC_lexicon_tags_11 },
    { 0xe3287021, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29487026, SUC_lexicon_tags_37 },
    { 0x36887026, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96887029, SUC_lexicon_tags_79 },
    { 0x5068702a, SUC_lexicon_tags_45 },
    { 0x8ec8702a, SUC_lexicon_tags_49 },
    { 0x4dc8702c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e8702f, SUC_lexicon_tags_7 },
    { 0x32e87030, SUC_lexicon_tags_23 },
    { 0x65087031, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2287036, SUC_lexicon_tags_25 },
    { 0x92687037, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8088703a, SUC_lexicon_tags_23 },
    { 0x10e8703b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbf48703d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc87044, SUC_lexicon_tags_60 },
    { 0xa9e87045, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31687049, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf88704b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c48704d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x73c8704f, SUC_lexicon_tags_4 },
    { 0xd188704f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15287055, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaee87057, SUC_lexicon_tags_49 },
    { 0x66087057, SUC_lexicon_tags_43 },
    { 0x3bc87059, SUC_lexicon_tags_2 },
    { 0xe6687059, SUC_lexicon_tags_17 },
    { 0xdca8705b, SUC_lexicon_tags_57 },
    { 0x5fc87059, SUC_lexicon_tags_87 },
    { 0xffa8705d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7628705f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc087062, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff087066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c8706a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcdc8706c, SUC_lexicon_tags_12 },
    { 0xdfe8706d, SUC_lexicon_tags_37 },
    { 0x3248706d, SUC_lexicon_tags_85 },
    { 0xe7a8706f, SUC_lexicon_tags_57 },
    { 0x72287070, SUC_lexicon_tags_3 },
    { 0x81c87071, SUC_lexicon_tags_17 },
    { 0x96687072, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4f487074, SUC_lexicon_tags_17 },
    { 0xe2087074, SUC_lexicon_tags_15 },
    { 0xc4887075, SUC_lexicon_tags_66 },
    { 0x81e87077, SUC_lexicon_tags_62 },
    { 0xd0c87078, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb88707a, SUC_lexicon_tags_57 },
    { 0xd3c8707b, SUC_lexicon_tags_91 },
    { 0x2648707a, SUC_lexicon_tags_5 },
    { 0x1bc8707d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31e8707f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c87083, SUC_lexicon_tags_22 },
    { 0xf4887084, SUC_lexicon_tags_2 },
    { 0x69c87084, SUC_lexicon_tags_14 },
    { 0xffe87085, SUC_lexicon_tags_9 },
    { 0x93c87087, SUC_lexicon_tags_16 },
    { 0x23687087, SUC_lexicon_tags_45 },
    { 0x35687089, SUC_lexicon_tags_17 },
    { 0xc6a87089, SUC_lexicon_tags_38 },
    { 0xe228708b, SUC_lexicon_tags_26 },
    { 0xe8c8708b, SUC_lexicon_tags_1 },
    { 0xdec87084, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6bc8708f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb087091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e87094, SUC_lexicon_tags_17 },
    { 0xe8887095, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c087098, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4908709f, SUC_lexicon_tags_15 },
    { 0xb5c870a0, SUC_lexicon_tags_2 },
    { 0x454870a1, SUC_lexicon_tags_15 },
    { 0x18a870a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e870a9, SUC_lexicon_tags_14 },
    { 0x1ac870aa, SUC_lexicon_tags_11 },
    { 0xeac870aa, SUC_lexicon_tags_17 },
    { 0xca8870a9, SUC_lexicon_tags_11 },
    { 0x15e870ad, SUC_lexicon_tags_14 },
    { 0x23a870ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x872870b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8870b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a870b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774870bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x374870bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562870c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c870cc, SUC_lexicon_tags_2 },
    { 0x2d6870cd, SUC_lexicon_tags_60 },
    { 0xe9e870ce, SUC_lexicon_tags_15 },
    { 0x1fc870cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50870d8, SUC_lexicon_tags_27 },
    { 0xcc870d9, SUC_lexicon_tags_30 },
    { 0xa40870da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c870df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c870ec, SUC_lexicon_tags_10 },
    { 0xeb0870ed, SUC_lexicon_tags_25 },
    { 0x7dc870ed, SUC_lexicon_tags_17 },
    { 0xdb0870ed, SUC_lexicon_tags_15 },
    { 0xe2e870f0, SUC_lexicon_tags_174 },
    { 0x2c0870ef, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a870f7, SUC_lexicon_tags_2 },
    { 0x9de870f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x248870fa, SUC_lexicon_tags_30 },
    { 0xae4870fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x362870fe, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x91e87100, SUC_lexicon_tags_14 },
    { 0x55087100, SUC_lexicon_tags_63 },
    { 0x12e87101, SUC_lexicon_tags_45 },
    { 0x94087103, SUC_lexicon_tags_49 },
    { 0x15c87104, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c8710f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee8711e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c87121, SUC_lexicon_tags_15 },
    { 0xa3287122, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41087124, SUC_lexicon_tags_23 },
    { 0x20287124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e87128, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e88712d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5708712f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e087133, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde87136, SUC_lexicon_tags_55 },
    { 0x1fc87137, SUC_lexicon_tags_31 },
    { 0x80287138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ae8713a, SUC_lexicon_tags_87 },
    { 0x20c8713b, SUC_lexicon_tags_25 },
    { 0x6c68713c, SUC_lexicon_tags_62 },
    { 0xcf48713d, SUC_lexicon_tags_64 },
    { 0x2848713d, SUC_lexicon_tags_15 },
    { 0x56a8713d, SUC_lexicon_tags_27 },
    { 0x4ca8713d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fe87142, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc287146, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a87149, SUC_lexicon_tags_38 },
    { 0xe5c87149, SUC_lexicon_tags_39 },
    { 0xb7287149, SUC_lexicon_tags_0 },
    { 0x2488714c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4408714e, SUC_lexicon_tags_49 },
    { 0x21a8714e, SUC_lexicon_tags_17 },
    { 0x5e08714f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc87152, SUC_lexicon_tags_11 },
    { 0xa9487153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc487156, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9808715f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3687162, SUC_lexicon_tags_198 },
    { 0x30a87163, SUC_lexicon_tags_25 },
    { 0x53c87163, SUC_lexicon_tags_14 },
    { 0x41e87165, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x22487167, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a8716a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x42a8716c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c687171, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa3287173, SUC_lexicon_tags_1 },
    { 0x27c87174, SUC_lexicon_tags_17 },
    { 0x6d487174, SUC_lexicon_tags_14 },
    { 0x57287176, SUC_lexicon_tags_9 },
    { 0x9b087174, SUC_lexicon_tags_12 },
    { 0xf2887178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d68717a, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xea48717c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e687180, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36487184, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d487186, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e8718b, SUC_lexicon_tags_11 },
    { 0xb2a8718b, SUC_lexicon_tags_29 },
    { 0x1928718d, SUC_lexicon_tags_12 },
    { 0x8368718e, SUC_lexicon_tags_23 },
    { 0x4ca8718d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea687193, SUC_lexicon_tags_37 },
    { 0x44687194, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2687196, SUC_lexicon_tags_11 },
    { 0x25287196, SUC_lexicon_tags_11 },
    { 0xcee87196, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaa68719a, SUC_lexicon_tags_19 },
    { 0x2928719b, SUC_lexicon_tags_27 },
    { 0xaea8719c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6388719e, SUC_lexicon_tags_77 },
    { 0xb9a8719e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188871ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd8871af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x544871b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0871b5, SUC_lexicon_tags_12 },
    { 0x39a871b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x370871b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe871ba, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee871bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e871c1, SUC_lexicon_tags_64 },
    { 0xb0e871c1, SUC_lexicon_tags_31 },
    { 0xf42871c3, SUC_lexicon_tags_14 },
    { 0xb18871c4, SUC_lexicon_tags_28 },
    { 0x948871c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x610871c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e871cd, SUC_lexicon_tags_12 },
    { 0x44871ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a0871d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4871e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc66871e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c8871e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c871eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332871f3, SUC_lexicon_tags_15 },
    { 0x4ea871f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba871f7, SUC_lexicon_tags_30 },
    { 0x8e871f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc87203, SUC_lexicon_tags_2 },
    { 0xa5487204, SUC_lexicon_tags_7 },
    { 0xc2287205, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61287210, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc68721d, SUC_lexicon_tags_30 },
    { 0xac08721e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e687221, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e87224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7487227, SUC_lexicon_tags_37 },
    { 0x1e487228, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e28722a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc228722c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x948722f, SUC_lexicon_tags_38 },
    { 0x65687230, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a887234, SUC_lexicon_tags_22 },
    { 0xbf087234, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b487237, SUC_lexicon_tags_7 },
    { 0xfc687238, SUC_lexicon_tags_63 },
    { 0x3e087238, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c68723d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xba88723f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc87245, SUC_lexicon_tags_47 },
    { 0x4ee87246, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb948724c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2287250, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3e887252, SUC_lexicon_tags_11 },
    { 0x67087253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae87257, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b28725a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a8725e, SUC_lexicon_tags_7 },
    { 0xede8725e, SUC_lexicon_tags_35 },
    { 0x1ae87260, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe487263, SUC_lexicon_tags_21 },
    { 0xd887264, SUC_lexicon_tags_15 },
    { 0x67a87264, SUC_lexicon_tags_14 },
    { 0xe487264, SUC_lexicon_tags_104 },
    { 0x55487263, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x228726a, SUC_lexicon_tags_11 },
    { 0x348726b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e8726f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc087273, SUC_lexicon_tags_27 },
    { 0xfcc87273, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18087279, SUC_lexicon_tags_7 },
    { 0x7448727a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec8727d, SUC_lexicon_tags_45 },
    { 0x3dc8727e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17e87280, SUC_lexicon_tags_0 },
    { 0x6c087281, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ee87283, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d687286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7487289, SUC_lexicon_tags_0 },
    { 0x9908728a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa8728f, SUC_lexicon_tags_27 },
    { 0x8be8728f, SUC_lexicon_tags_26 },
    { 0x60e87291, SUC_lexicon_tags_23 },
    { 0x7aa87292, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8287296, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec08729b, SUC_lexicon_tags_80 },
    { 0xa2a8729c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7588729e, SUC_lexicon_tags_79 },
    { 0xa888729f, SUC_lexicon_tags_11 },
    { 0xfa4872a0, SUC_lexicon_tags_11 },
    { 0xeb4872a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x942872a3, SUC_lexicon_tags_7 },
    { 0x14872a4, SUC_lexicon_tags_8 },
    { 0x3da872a3, SUC_lexicon_tags_0 },
    { 0x5e8872a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a872a9, SUC_lexicon_tags_1 },
    { 0xf78872a9, SUC_lexicon_tags_25 },
    { 0xffc872aa, SUC_lexicon_tags_65 },
    { 0x41a872ac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x876872ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8872b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144872b7, SUC_lexicon_tags_28 },
    { 0xd80872b8, SUC_lexicon_tags_45 },
    { 0xc9e872b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a872bc, SUC_lexicon_tags_25 },
    { 0x268872bc, SUC_lexicon_tags_2 },
    { 0xbe0872be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e872c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c872c6, SUC_lexicon_tags_37 },
    { 0x1e4872c7, SUC_lexicon_tags_4 },
    { 0xf7e872c8, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0x524872cb, SUC_lexicon_tags_45 },
    { 0xf5c872cc, SUC_lexicon_tags_28 },
    { 0x72e872cc, SUC_lexicon_tags_83 },
    { 0xf6e872ce, SUC_lexicon_tags_7 },
    { 0xf0e872cf, SUC_lexicon_tags_17 },
    { 0xf0a872cf, SUC_lexicon_tags_8 },
    { 0x7a8872d1, SUC_lexicon_tags_2 },
    { 0x31e872d2, SUC_lexicon_tags_25 },
    { 0x42c872d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x876872d5, SUC_lexicon_tags_17 },
    { 0xbea872d6, SUC_lexicon_tags_37 },
    { 0xf0e872d6, SUC_lexicon_tags_25 },
    { 0x670872d8, SUC_lexicon_tags_17 },
    { 0x2f8872d8, SUC_lexicon_tags_25 },
    { 0xdfc872d6, SUC_lexicon_tags_25 },
    { 0x1c8872da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb34872dd, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x590872df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa3e872e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x544872e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae872ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe22872ec, SUC_lexicon_tags_25 },
    { 0x640872ed, SUC_lexicon_tags_17 },
    { 0x7e0872ed, SUC_lexicon_tags_17 },
    { 0x84a872ee, SUC_lexicon_tags_15 },
    { 0xac0872ef, SUC_lexicon_tags_17 },
    { 0x2b2872f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9a6872f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x66872f5, SUC_lexicon_tags_37 },
    { 0x274872f6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc872f9, SUC_lexicon_tags_55 },
    { 0x14872fa, SUC_lexicon_tags_15 },
    { 0xa6872fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e4872fd, SUC_lexicon_tags_38 },
    { 0x21a872fe, SUC_lexicon_tags_10 },
    { 0xb7e872ff, SUC_lexicon_tags_10 },
    { 0x12872ff, SUC_lexicon_tags_12 },
    { 0x2487301, SUC_lexicon_tags_2 },
    { 0xd887302, SUC_lexicon_tags_25 },
    { 0x97487302, SUC_lexicon_tags_28 },
    { 0x6ec87304, SUC_lexicon_tags_1 },
    { 0x34a87303, SUC_lexicon_tags_63 },
    { 0x16287300, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48730d, SUC_lexicon_tags_17 },
    { 0x988730d, SUC_lexicon_tags_31 },
    { 0x6a88730f, SUC_lexicon_tags_2 },
    { 0xe4c87310, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2c887312, SUC_lexicon_tags_21 },
    { 0x23c87313, SUC_lexicon_tags_25 },
    { 0xfa687313, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeac87316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1488731a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c8731e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6887323, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c887334, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57087336, SUC_lexicon_tags_27 },
    { 0xc8287336, SUC_lexicon_tags_33 },
    { 0xe2887337, SUC_lexicon_tags_63 },
    { 0xb8c87339, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3487340, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae87344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48487347, SUC_lexicon_tags_11 },
    { 0x72e87348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8828734a, SUC_lexicon_tags_63 },
    { 0x6d08734b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c8734f, SUC_lexicon_tags_30 },
    { 0xf3c87350, SUC_lexicon_tags_11 },
    { 0x44c87351, SUC_lexicon_tags_11 },
    { 0x45c87352, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83c87354, SUC_lexicon_tags_14 },
    { 0xf3487354, SUC_lexicon_tags_37 },
    { 0x7d087356, SUC_lexicon_tags_16 },
    { 0xc1687355, SUC_lexicon_tags_2 },
    { 0xa3887358, SUC_lexicon_tags_17 },
    { 0x59887359, SUC_lexicon_tags_46 },
    { 0xb908735a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a8735f, SUC_lexicon_tags_11 },
    { 0x5b28735f, SUC_lexicon_tags_63 },
    { 0x61a87360, SUC_lexicon_tags_17 },
    { 0xf5487362, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea87366, SUC_lexicon_tags_70 },
    { 0xe3c87366, SUC_lexicon_tags_23 },
    { 0x88087366, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca8736b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57087370, SUC_lexicon_tags_12 },
    { 0x46887371, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd687375, SUC_lexicon_tags_17 },
    { 0x2d287375, SUC_lexicon_tags_19 },
    { 0x24487377, SUC_lexicon_tags_37 },
    { 0xdc887376, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e8737c, SUC_lexicon_tags_113 },
    { 0xafa8737d, SUC_lexicon_tags_27 },
    { 0xa0c8737e, SUC_lexicon_tags_27 },
    { 0x4408737e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2687381, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9287385, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a8738c, SUC_lexicon_tags_17 },
    { 0x4728738d, SUC_lexicon_tags_19 },
    { 0x89c8738c, SUC_lexicon_tags_0 },
    { 0xd5a8738d, SUC_lexicon_tags_597 },
    { 0x1e287390, SUC_lexicon_tags_79 },
    { 0xb5a8738d, SUC_lexicon_tags_2 },
    { 0xe4887392, SUC_lexicon_tags_13 },
    { 0xb5e87392, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce87396, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8739b, SUC_lexicon_tags_0 },
    { 0x56a8739b, SUC_lexicon_tags_14 },
    { 0x16a8739d, SUC_lexicon_tags_45 },
    { 0x7568739c, SUC_lexicon_tags_6 },
    { 0xf0a8739f, SUC_lexicon_tags_27 },
    { 0x5e4873a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8873a7, SUC_lexicon_tags_16 },
    { 0xfe8873a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfc873aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfd2873ac, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510873b1, SUC_lexicon_tags_47 },
    { 0xb50873b1, SUC_lexicon_tags_11 },
    { 0xfa2873b1, SUC_lexicon_tags_16 },
    { 0xf02873b4, SUC_lexicon_tags_43 },
    { 0xae2873b5, SUC_lexicon_tags_15 },
    { 0xfe873b5, SUC_lexicon_tags_25 },
    { 0x764873b7, SUC_lexicon_tags_19 },
    { 0x3fa873b8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1a873ba, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa873bd, SUC_lexicon_tags_79 },
    { 0x376873be, SUC_lexicon_tags_11 },
    { 0x884873bf, SUC_lexicon_tags_15 },
    { 0xde8873c0, SUC_lexicon_tags_37 },
    { 0xb4c873c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52873c5, SUC_lexicon_tags_14 },
    { 0xad4873c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x188873c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0873d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa4873d2, SUC_lexicon_tags_25 },
    { 0xe2c873d2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x968873d6, SUC_lexicon_tags_14 },
    { 0xba4873d7, SUC_lexicon_tags_8 },
    { 0x9a8873d7, SUC_lexicon_tags_47 },
    { 0x48a873d9, SUC_lexicon_tags_87 },
    { 0xee0873d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44873e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc873e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04873ef, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2de873f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x396873f3, SUC_lexicon_tags_46 },
    { 0xc60873f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf94873f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa08873f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25887400, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e8740b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x808740d, SUC_lexicon_tags_28 },
    { 0x51a8740d, SUC_lexicon_tags_11 },
    { 0x1c08740e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6087412, SUC_lexicon_tags_17 },
    { 0x8ea87412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e87416, SUC_lexicon_tags_47 },
    { 0xd3c87417, SUC_lexicon_tags_37 },
    { 0xca287417, SUC_lexicon_tags_1 },
    { 0xa0e87419, SUC_lexicon_tags_8 },
    { 0x8028741a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a68741e, SUC_lexicon_tags_14 },
    { 0xa108741e, SUC_lexicon_tags_87 },
    { 0x12e87420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8087423, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c687426, SUC_lexicon_tags_17 },
    { 0xc6e87427, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf108742b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9588742d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd548742f, SUC_lexicon_tags_17 },
    { 0x888742f, SUC_lexicon_tags_4 },
    { 0x99487431, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c87434, SUC_lexicon_tags_17 },
    { 0xe7e87435, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2a87437, SUC_lexicon_tags_42 },
    { 0xf2e87437, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x38a8743a, SUC_lexicon_tags_45 },
    { 0xffa8743a, SUC_lexicon_tags_201 },
    { 0xf148743c, SUC_lexicon_tags_37 },
    { 0xe848743b, SUC_lexicon_tags_12 },
    { 0x2988743b, SUC_lexicon_tags_1 },
    { 0x308743f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94287443, SUC_lexicon_tags_61 },
    { 0x1487444, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x31a87446, SUC_lexicon_tags_7 },
    { 0xfec87446, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e8744a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9048744c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb0c8744e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc1c87450, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f287454, SUC_lexicon_tags_28 },
    { 0xd4c87455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2087459, SUC_lexicon_tags_17 },
    { 0x42a8745a, SUC_lexicon_tags_12 },
    { 0x14a8745b, SUC_lexicon_tags_28 },
    { 0xc828745b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1cc8745e, SUC_lexicon_tags_17 },
    { 0xfb28745f, SUC_lexicon_tags_46 },
    { 0xc548745f, SUC_lexicon_tags_17 },
    { 0x8728745f, SUC_lexicon_tags_0 },
    { 0xe1c87461, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4b287464, SUC_lexicon_tags_1278 },
    { 0x5aa87465, SUC_lexicon_tags_0 },
    { 0x1b487464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b28746a, SUC_lexicon_tags_32 },
    { 0x76e8746b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe528746e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa87473, SUC_lexicon_tags_0 },
    { 0x1a887473, SUC_lexicon_tags_11 },
    { 0x9cc87475, SUC_lexicon_tags_17 },
    { 0x6e087475, SUC_lexicon_tags_45 },
    { 0xfac87477, SUC_lexicon_tags_2 },
    { 0xb1a87478, SUC_lexicon_tags_25 },
    { 0x17e87479, SUC_lexicon_tags_23 },
    { 0x4f887473, SUC_lexicon_tags_15 },
    { 0x63487477, SUC_lexicon_tags_29 },
    { 0x91e8747c, SUC_lexicon_tags_7 },
    { 0xdda8747d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3188747f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a87481, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4887485, SUC_lexicon_tags_37 },
    { 0xba087486, SUC_lexicon_tags_7 },
    { 0x10887486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1e87489, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c8748c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c87491, SUC_lexicon_tags_46 },
    { 0x32e87491, SUC_lexicon_tags_113 },
    { 0x3d687492, SUC_lexicon_tags_11 },
    { 0x80087494, SUC_lexicon_tags_37 },
    { 0x33087495, SUC_lexicon_tags_31 },
    { 0xe1c87496, SUC_lexicon_tags_6 },
    { 0x4fe87496, SUC_lexicon_tags_7 },
    { 0xcc087496, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a8749c, SUC_lexicon_tags_12 },
    { 0xd6a8749c, SUC_lexicon_tags_25 },
    { 0x55c8749e, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8874a1, SUC_lexicon_tags_17 },
    { 0xf30874a2, SUC_lexicon_tags_25 },
    { 0xe0c874a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc874a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12874ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5e874ae, SUC_lexicon_tags_7 },
    { 0xf66874ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e874b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c874b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8874bb, SUC_lexicon_tags_11 },
    { 0xac0874bb, SUC_lexicon_tags_66 },
    { 0xaa2874bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52874c1, SUC_lexicon_tags_11 },
    { 0x10c874c1, SUC_lexicon_tags_25 },
    { 0xb5a874c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae874c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee874d2, SUC_lexicon_tags_9 },
    { 0xc8874d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8874d6, SUC_lexicon_tags_12 },
    { 0xdd4874d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0874db, SUC_lexicon_tags_6 },
    { 0x2c4874db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20874e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274874e4, SUC_lexicon_tags_30 },
    { 0x7a4874e5, SUC_lexicon_tags_27 },
    { 0x8d6874e5, SUC_lexicon_tags_17 },
    { 0x3b6874e7, SUC_lexicon_tags_25 },
    { 0xf48874e8, SUC_lexicon_tags_11 },
    { 0x60e874e7, SUC_lexicon_tags_11 },
    { 0x5a0874ea, SUC_lexicon_tags_11 },
    { 0x4ba874ea, SUC_lexicon_tags_25 },
    { 0xc90874eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938874f0, SUC_lexicon_tags_49 },
    { 0x1e874f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb6874f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac874f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6874f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee87500, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f087502, SUC_lexicon_tags_23 },
    { 0x37287503, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae87508, SUC_lexicon_tags_25 },
    { 0xc6687509, SUC_lexicon_tags_170 },
    { 0x91687508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2548750e, SUC_lexicon_tags_37 },
    { 0x7f88750f, SUC_lexicon_tags_25 },
    { 0x53c87510, SUC_lexicon_tags_14 },
    { 0x53487511, SUC_lexicon_tags_39 },
    { 0xef287512, SUC_lexicon_tags_1 },
    { 0xa7c87511, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2087515, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7808751b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc548751d, SUC_lexicon_tags_14 },
    { 0x4b48751e, SUC_lexicon_tags_56 },
    { 0x5f28751f, SUC_lexicon_tags_64 },
    { 0x8ba87520, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31087529, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3d68752b, SUC_lexicon_tags_7 },
    { 0xad88752b, SUC_lexicon_tags_7 },
    { 0x4ae8752c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea87530, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67087533, SUC_lexicon_tags_22 },
    { 0xcbc87534, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x52c87537, SUC_lexicon_tags_7 },
    { 0xf5c87538, SUC_lexicon_tags_14 },
    { 0xb0c87539, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e8753c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a087545, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c8754a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a87553, SUC_lexicon_tags_11 },
    { 0x42687554, SUC_lexicon_tags_12 },
    { 0x30287554, SUC_lexicon_tags_37 },
    { 0x71287556, SUC_lexicon_tags_32 },
    { 0x22087557, SUC_lexicon_tags_27 },
    { 0x76687556, SUC_lexicon_tags_25 },
    { 0x4d287556, SUC_lexicon_tags_12 },
    { 0x2f287555, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a87560, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6087563, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0287566, SUC_lexicon_tags_38 },
    { 0xe887567, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24287574, SUC_lexicon_tags_94 },
    { 0x7087575, SUC_lexicon_tags_37 },
    { 0x7b487574, SUC_lexicon_tags_47 },
    { 0x57687577, SUC_lexicon_tags_19 },
    { 0xc9a87578, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8a88757a, SUC_lexicon_tags_47 },
    { 0xc008757b, SUC_lexicon_tags_17 },
    { 0x74a8757c, SUC_lexicon_tags_23 },
    { 0x87e8757c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f487580, SUC_lexicon_tags_7 },
    { 0x37287580, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c87584, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1cc87586, SUC_lexicon_tags_15 },
    { 0xd5087586, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc868758b, SUC_lexicon_tags_9 },
    { 0xefe8758b, SUC_lexicon_tags_7 },
    { 0x53e8758d, SUC_lexicon_tags_11 },
    { 0x8b28758d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca87591, SUC_lexicon_tags_21 },
    { 0xece87592, SUC_lexicon_tags_10 },
    { 0x1f687591, SUC_lexicon_tags_37 },
    { 0x58e87592, SUC_lexicon_tags_87 },
    { 0xe2c87594, SUC_lexicon_tags_63 },
    { 0x4a487591, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6e87598, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8628759b, SUC_lexicon_tags_25 },
    { 0x788759c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0875a1, SUC_lexicon_tags_26 },
    { 0xc86875a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfa2875a4, SUC_lexicon_tags_14 },
    { 0x5f8875a5, SUC_lexicon_tags_0 },
    { 0xf8c875a4, SUC_lexicon_tags_38 },
    { 0x464875a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c875ab, SUC_lexicon_tags_35 },
    { 0xcc6875ac, SUC_lexicon_tags_33 },
    { 0xa4875ad, SUC_lexicon_tags_66 },
    { 0x6fc875ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78875bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a875c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34875c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0875cc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x460875ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc62875d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28875d4, SUC_lexicon_tags_29 },
    { 0x470875d5, SUC_lexicon_tags_0 },
    { 0x8be875d4, SUC_lexicon_tags_16 },
    { 0xfa2875d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x562875da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0875de, SUC_lexicon_tags_7 },
    { 0x486875df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc875e2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8875e6, SUC_lexicon_tags_12 },
    { 0x634875e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244875ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xec8875ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x88a875f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e875f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230875fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9087606, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a87609, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb568760c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb988760e, SUC_lexicon_tags_17 },
    { 0xc3e8760f, SUC_lexicon_tags_47 },
    { 0x8b687610, SUC_lexicon_tags_41 },
    { 0x1c48760e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95687614, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8487618, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb968761c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6e68761e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x16487620, SUC_lexicon_tags_14 },
    { 0x49e87621, SUC_lexicon_tags_2 },
    { 0x8ba87622, SUC_lexicon_tags_25 },
    { 0x3fe87623, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3887626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c87629, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6108762c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f08762f, SUC_lexicon_tags_37 },
    { 0xbd687630, SUC_lexicon_tags_25 },
    { 0x26687631, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36e87633, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e87637, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd48763b, SUC_lexicon_tags_15 },
    { 0xf768763c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc348763f, SUC_lexicon_tags_17 },
    { 0x81c87640, SUC_lexicon_tags_12 },
    { 0xfee87640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb887643, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8287645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28764b, SUC_lexicon_tags_37 },
    { 0x3da8764c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1687652, SUC_lexicon_tags_6 },
    { 0x49687653, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2887656, SUC_lexicon_tags_1 },
    { 0x5ca87657, SUC_lexicon_tags_11 },
    { 0x9ec87658, SUC_lexicon_tags_30 },
    { 0x6ba87659, SUC_lexicon_tags_29 },
    { 0xc5487658, SUC_lexicon_tags_31 },
    { 0x1088765b, SUC_lexicon_tags_10 },
    { 0xeaa8765b, SUC_lexicon_tags_38 },
    { 0xee48765a, SUC_lexicon_tags_77 },
    { 0xcdc8765e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x60287660, SUC_lexicon_tags_14 },
    { 0x4fe87661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1687669, SUC_lexicon_tags_136 },
    { 0xaca8766a, SUC_lexicon_tags_11 },
    { 0x2a88766b, SUC_lexicon_tags_11 },
    { 0x15887669, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1dc8766e, SUC_lexicon_tags_25 },
    { 0x7c68766f, SUC_lexicon_tags_25 },
    { 0x7a087670, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8287675, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d887679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb28767d, SUC_lexicon_tags_19 },
    { 0x4da8767e, SUC_lexicon_tags_7 },
    { 0x10c8767f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a87682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e87688, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708768c, SUC_lexicon_tags_27 },
    { 0x8fc8768c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ec8768f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53087697, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8769c, SUC_lexicon_tags_17 },
    { 0x4a08769d, SUC_lexicon_tags_11 },
    { 0x2188769c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a876a1, SUC_lexicon_tags_87 },
    { 0xec4876a2, SUC_lexicon_tags_23 },
    { 0x830876a3, SUC_lexicon_tags_4 },
    { 0xb4c876a4, SUC_lexicon_tags_65 },
    { 0x476876a4, SUC_lexicon_tags_25 },
    { 0x6f4876a6, SUC_lexicon_tags_27 },
    { 0xb56876a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc876ac, SUC_lexicon_tags_87 },
    { 0xef4876ad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x588876af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc876b2, SUC_lexicon_tags_37 },
    { 0xeb4876b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75a876b5, SUC_lexicon_tags_11 },
    { 0x3b8876b5, SUC_lexicon_tags_25 },
    { 0x498876b6, SUC_lexicon_tags_11 },
    { 0xa88876b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0876bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2876c3, SUC_lexicon_tags_26 },
    { 0xd52876c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x46c876c6, SUC_lexicon_tags_2 },
    { 0x8aa876c7, SUC_lexicon_tags_14 },
    { 0xef6876c8, SUC_lexicon_tags_32 },
    { 0xd9a876c7, SUC_lexicon_tags_28 },
    { 0xaa6876ca, SUC_lexicon_tags_1 },
    { 0x464876cb, SUC_lexicon_tags_12 },
    { 0xc92876ca, SUC_lexicon_tags_35 },
    { 0x6ee876ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36876d2, SUC_lexicon_tags_55 },
    { 0x4b6876d3, SUC_lexicon_tags_1 },
    { 0x652876d2, SUC_lexicon_tags_11 },
    { 0x81e876d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4876db, SUC_lexicon_tags_82 },
    { 0x46c876dc, SUC_lexicon_tags_12 },
    { 0x6876db, SUC_lexicon_tags_14 },
    { 0xa04876dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6876e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbfa876e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646876ec, SUC_lexicon_tags_19 },
    { 0xd6a876ed, SUC_lexicon_tags_15 },
    { 0x61a876ec, SUC_lexicon_tags_17 },
    { 0x9ba876ef, SUC_lexicon_tags_23 },
    { 0x38e876f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a876f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c876f7, SUC_lexicon_tags_17 },
    { 0x178876f7, SUC_lexicon_tags_37 },
    { 0x99e876f8, SUC_lexicon_tags_12 },
    { 0x180876fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66487700, SUC_lexicon_tags_19 },
    { 0x1687700, SUC_lexicon_tags_14 },
    { 0xa9887701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5887704, SUC_lexicon_tags_31 },
    { 0xe6087704, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe9687707, SUC_lexicon_tags_107 },
    { 0xa8e87708, SUC_lexicon_tags_2 },
    { 0xc487709, SUC_lexicon_tags_10 },
    { 0x8d887709, SUC_lexicon_tags_2 },
    { 0x1a08770b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e87711, SUC_lexicon_tags_11 },
    { 0x94887712, SUC_lexicon_tags_48 },
    { 0x24687713, SUC_lexicon_tags_9 },
    { 0xe9687714, SUC_lexicon_tags_11 },
    { 0x40c87715, SUC_lexicon_tags_27 },
    { 0xea087713, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc87719, SUC_lexicon_tags_37 },
    { 0xf368771a, SUC_lexicon_tags_37 },
    { 0x33e8771b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf168771e, SUC_lexicon_tags_47 },
    { 0xcbe8771e, SUC_lexicon_tags_12 },
    { 0xf7287720, SUC_lexicon_tags_42 },
    { 0xf728771e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d287727, SUC_lexicon_tags_60 },
    { 0x8b287728, SUC_lexicon_tags_28 },
    { 0xbe687729, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e8772e, SUC_lexicon_tags_11 },
    { 0xdbc8772f, SUC_lexicon_tags_14 },
    { 0x4608772e, SUC_lexicon_tags_6 },
    { 0x2d487731, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c87736, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b08773c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8608773f, SUC_lexicon_tags_37 },
    { 0x19287740, SUC_lexicon_tags_15 },
    { 0xb908773f, SUC_lexicon_tags_37 },
    { 0x5c87742, SUC_lexicon_tags_22 },
    { 0xa8287743, SUC_lexicon_tags_17 },
    { 0xb5e87744, SUC_lexicon_tags_9 },
    { 0xbda87744, SUC_lexicon_tags_25 },
    { 0x7fe87746, SUC_lexicon_tags_33 },
    { 0xff287747, SUC_lexicon_tags_47 },
    { 0xac487745, SUC_lexicon_tags_7 },
    { 0xc1c87749, SUC_lexicon_tags_0 },
    { 0x2608774a, SUC_lexicon_tags_11 },
    { 0x6768774a, SUC_lexicon_tags_2 },
    { 0xfde8774b, SUC_lexicon_tags_65 },
    { 0x2628774d, SUC_lexicon_tags_37 },
    { 0x8ce87749, SUC_lexicon_tags_11 },
    { 0xf3c8774f, SUC_lexicon_tags_11 },
    { 0x44a87741, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc87754, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55687756, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6087759, SUC_lexicon_tags_7 },
    { 0xcd687759, SUC_lexicon_tags_25 },
    { 0xf588775b, SUC_lexicon_tags_11 },
    { 0xfde8775b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5e8775e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c87761, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63e87763, SUC_lexicon_tags_37 },
    { 0x9aa87764, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be87767, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2a87769, SUC_lexicon_tags_7 },
    { 0x6d687769, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9388776e, SUC_lexicon_tags_35 },
    { 0xf5a8776f, SUC_lexicon_tags_57 },
    { 0x3d087770, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe87774, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd687777, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8087779, SUC_lexicon_tags_37 },
    { 0x108777a, SUC_lexicon_tags_11 },
    { 0x2928777a, SUC_lexicon_tags_1 },
    { 0x8748777c, SUC_lexicon_tags_16 },
    { 0x408777d, SUC_lexicon_tags_28 },
    { 0xabe8777c, SUC_lexicon_tags_25 },
    { 0xfbc8777f, SUC_lexicon_tags_2 },
    { 0x91c8777f, SUC_lexicon_tags_2 },
    { 0x83a87781, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde87789, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7368778b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f48778e, SUC_lexicon_tags_16 },
    { 0xdb68778f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf2287791, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd287793, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a687799, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b28779d, SUC_lexicon_tags_11 },
    { 0xba28779e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4877a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16877a5, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe877ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6877b4, SUC_lexicon_tags_49 },
    { 0x2dc877b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda8877b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f0877b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0877be, SUC_lexicon_tags_32 },
    { 0x116877be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e2877c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e877c3, SUC_lexicon_tags_32 },
    { 0x9a6877c3, SUC_lexicon_tags_151 },
    { 0xee4877c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a877c8, SUC_lexicon_tags_17 },
    { 0xbb4877c9, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2877cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa877d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5c877d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56877d8, SUC_lexicon_tags_12 },
    { 0x10e877d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa36877db, SUC_lexicon_tags_11 },
    { 0xac4877dc, SUC_lexicon_tags_7 },
    { 0x98e877dc, SUC_lexicon_tags_60 },
    { 0x1d6877dc, SUC_lexicon_tags_2 },
    { 0x5aa877df, SUC_lexicon_tags_17 },
    { 0x8e8877e0, SUC_lexicon_tags_16 },
    { 0x3d0877e1, SUC_lexicon_tags_57 },
    { 0xf70877df, SUC_lexicon_tags_39 },
    { 0x8f8877dd, SUC_lexicon_tags_25 },
    { 0x936877e4, SUC_lexicon_tags_0 },
    { 0x3d6877e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc877ea, SUC_lexicon_tags_17 },
    { 0x192877ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26877f2, SUC_lexicon_tags_60 },
    { 0x538877f3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x32e877f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6f8877f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef4877f9, SUC_lexicon_tags_12 },
    { 0x95e877fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf84877fc, SUC_lexicon_tags_49 },
    { 0x54e877fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x110877ff, SUC_lexicon_tags_2 },
    { 0xd3e87800, SUC_lexicon_tags_70 },
    { 0x9c087800, SUC_lexicon_tags_13 },
    { 0xde087802, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5487805, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcae87807, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2f087809, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa87811, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf5e87813, SUC_lexicon_tags_82 },
    { 0x93487813, SUC_lexicon_tags_25 },
    { 0xc6e87815, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c87818, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4a28781a, SUC_lexicon_tags_14 },
    { 0xf208781b, SUC_lexicon_tags_11 },
    { 0x2048781c, SUC_lexicon_tags_25 },
    { 0x6588781d, SUC_lexicon_tags_23 },
    { 0xd9c8781d, SUC_lexicon_tags_30 },
    { 0x50a8781c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa487826, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6687828, SUC_lexicon_tags_45 },
    { 0xf3087829, SUC_lexicon_tags_2 },
    { 0xc0487829, SUC_lexicon_tags_198 },
    { 0x4ec8782b, SUC_lexicon_tags_65 },
    { 0x6d48782c, SUC_lexicon_tags_6 },
    { 0x9548782b, SUC_lexicon_tags_33 },
    { 0x8d48782d, SUC_lexicon_tags_25 },
    { 0x2668782f, SUC_lexicon_tags_38 },
    { 0x65687830, SUC_lexicon_tags_28 },
    { 0x8ee87831, SUC_lexicon_tags_31 },
    { 0x75e87830, SUC_lexicon_tags_28 },
    { 0xf0c8782b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde87836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba8783a, SUC_lexicon_tags_45 },
    { 0x4868783b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xed48783d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a87841, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c287843, SUC_lexicon_tags_12 },
    { 0x95487843, SUC_lexicon_tags_65 },
    { 0x25e87844, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3a87847, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe8784a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd048784d, SUC_lexicon_tags_55 },
    { 0xa648784d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb087850, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a87853, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x83687856, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fc87858, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7608785d, SUC_lexicon_tags_12 },
    { 0xee28785e, SUC_lexicon_tags_11 },
    { 0x9a28785d, SUC_lexicon_tags_1 },
    { 0xeda8785d, SUC_lexicon_tags_25 },
    { 0xbc687861, SUC_lexicon_tags_14 },
    { 0xcdc87861, SUC_lexicon_tags_45 },
    { 0x7ec8785f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e87866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4287869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x388786c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6087872, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea87877, SUC_lexicon_tags_19 },
    { 0x63487878, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1248787f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c87885, SUC_lexicon_tags_11 },
    { 0x9a87886, SUC_lexicon_tags_25 },
    { 0x99c87885, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff08788b, SUC_lexicon_tags_1 },
    { 0x88e8788c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6328788f, SUC_lexicon_tags_1 },
    { 0x42287890, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6fa87892, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x94887895, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a487897, SUC_lexicon_tags_55 },
    { 0x8a287898, SUC_lexicon_tags_38 },
    { 0x64687897, SUC_lexicon_tags_2 },
    { 0x7a08789a, SUC_lexicon_tags_43 },
    { 0xec087899, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1888789d, SUC_lexicon_tags_238 },
    { 0xd468789e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ce878a0, SUC_lexicon_tags_33 },
    { 0xdde878a1, SUC_lexicon_tags_29 },
    { 0x6ec878a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc878a7, SUC_lexicon_tags_12 },
    { 0x1e6878a8, SUC_lexicon_tags_1 },
    { 0xd02878a9, SUC_lexicon_tags_37 },
    { 0x52878aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc878ae, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e878b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbe878b3, SUC_lexicon_tags_49 },
    { 0x68878b4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86878b8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22878c1, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58878c4, SUC_lexicon_tags_235 },
    { 0xbd0878c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4878cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0878cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea878d3, SUC_lexicon_tags_8 },
    { 0x172878d4, SUC_lexicon_tags_33 },
    { 0x464878d5, SUC_lexicon_tags_7 },
    { 0x9b8878d5, SUC_lexicon_tags_2 },
    { 0x590878d7, SUC_lexicon_tags_37 },
    { 0xf6c878d8, SUC_lexicon_tags_17 },
    { 0xa98878d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c878dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x94878de, SUC_lexicon_tags_25 },
    { 0xed2878df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02878e4, SUC_lexicon_tags_16 },
    { 0x684878e5, SUC_lexicon_tags_2 },
    { 0x8e6878e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfde878e8, SUC_lexicon_tags_17 },
    { 0x6b2878e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7be878eb, SUC_lexicon_tags_38 },
    { 0xe0e878ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2878ef, SUC_lexicon_tags_1 },
    { 0x4e4878ef, SUC_lexicon_tags_7 },
    { 0x40e878f1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966878f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28878f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6878fc, SUC_lexicon_tags_17 },
    { 0x9b2878fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e87902, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43687907, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x58a87909, SUC_lexicon_tags_38 },
    { 0xbd28790a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa328790f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e87912, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x99087914, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc5a87916, SUC_lexicon_tags_27 },
    { 0x22887917, SUC_lexicon_tags_15 },
    { 0x20087917, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5108791a, SUC_lexicon_tags_6 },
    { 0xc6c8791b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf687920, SUC_lexicon_tags_11 },
    { 0xbb687921, SUC_lexicon_tags_14 },
    { 0x73887920, SUC_lexicon_tags_19 },
    { 0xbe287923, SUC_lexicon_tags_29 },
    { 0x8f887924, SUC_lexicon_tags_31 },
    { 0x5f087925, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7368792a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc87933, SUC_lexicon_tags_30 },
    { 0x2e087934, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c8793a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61e8793c, SUC_lexicon_tags_1 },
    { 0xf88793c, SUC_lexicon_tags_9 },
    { 0x5408793e, SUC_lexicon_tags_37 },
    { 0xc968793f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5887943, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8887945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad487948, SUC_lexicon_tags_31 },
    { 0x79087948, SUC_lexicon_tags_25 },
    { 0x57c87949, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea8794f, SUC_lexicon_tags_3 },
    { 0x59a87950, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b887960, SUC_lexicon_tags_45 },
    { 0xc2687960, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a87965, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b687967, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd588796a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6948796c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be87972, SUC_lexicon_tags_2 },
    { 0x70a87973, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c87976, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf868797a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da8797e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a8798a, SUC_lexicon_tags_12 },
    { 0x21e8798b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef48798f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46487991, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5b287993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad287995, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9fe87997, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908799d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x898879a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec879a3, SUC_lexicon_tags_11 },
    { 0x3ec879a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24c879a6, SUC_lexicon_tags_60 },
    { 0xf48879a7, SUC_lexicon_tags_15 },
    { 0xfc6879a7, SUC_lexicon_tags_7 },
    { 0x310879a9, SUC_lexicon_tags_42 },
    { 0x7a6879a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2879af, SUC_lexicon_tags_15 },
    { 0xdf8879b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2879b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a879b6, SUC_lexicon_tags_15 },
    { 0x8fa879b7, SUC_lexicon_tags_39 },
    { 0xe0a879b8, SUC_lexicon_tags_27 },
    { 0xa8879b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x62a879bc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312879c2, SUC_lexicon_tags_116 },
    { 0xdc6879c2, SUC_lexicon_tags_14 },
    { 0xaf0879c4, SUC_lexicon_tags_19 },
    { 0xe6879c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa879ca, SUC_lexicon_tags_2 },
    { 0xcf2879cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04879ce, SUC_lexicon_tags_46 },
    { 0xb46879cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x934879d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c879d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562879df, SUC_lexicon_tags_47 },
    { 0x2b4879e0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0879eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a879ee, SUC_lexicon_tags_60 },
    { 0x1e8879ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0879f3, SUC_lexicon_tags_108 },
    { 0x478879f4, SUC_lexicon_tags_4 },
    { 0x510879f4, SUC_lexicon_tags_61 },
    { 0xa1e879f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c879fa, SUC_lexicon_tags_15 },
    { 0x1e6879fb, SUC_lexicon_tags_14 },
    { 0x7c4879fc, SUC_lexicon_tags_15 },
    { 0xd4c879fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89687a00, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13087a0a, SUC_lexicon_tags_37 },
    { 0x6bc87a0b, SUC_lexicon_tags_33 },
    { 0x42e87a0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a087a14, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x64a87a16, SUC_lexicon_tags_31 },
    { 0x5287a17, SUC_lexicon_tags_17 },
    { 0xc7487a18, SUC_lexicon_tags_17 },
    { 0xc1287a18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d887a1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x94887a1e, SUC_lexicon_tags_28 },
    { 0x2e487a1e, SUC_lexicon_tags_2 },
    { 0xab687a20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77487a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20287a2b, SUC_lexicon_tags_16 },
    { 0x48e87a2b, SUC_lexicon_tags_37 },
    { 0xa9487a2d, SUC_lexicon_tags_37 },
    { 0xcb887a2d, SUC_lexicon_tags_27 },
    { 0x3f887a2f, SUC_lexicon_tags_39 },
    { 0x2a287a2f, SUC_lexicon_tags_9 },
    { 0x27487a2e, SUC_lexicon_tags_15 },
    { 0xc6887a2d, SUC_lexicon_tags_17 },
    { 0xf4e87a2c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b287a38, SUC_lexicon_tags_160 },
    { 0xa7687a39, SUC_lexicon_tags_1 },
    { 0x3fc87a38, SUC_lexicon_tags_47 },
    { 0xd1a87a3b, SUC_lexicon_tags_38 },
    { 0xb9c87a3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xace87a3e, SUC_lexicon_tags_14 },
    { 0x46887a3f, SUC_lexicon_tags_33 },
    { 0x2f687a3f, SUC_lexicon_tags_15 },
    { 0x57e87a3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb687a48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67c87a4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x37a87a4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fc87a4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a287a50, SUC_lexicon_tags_46 },
    { 0xa4687a51, SUC_lexicon_tags_31 },
    { 0x7e087a50, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x53e87a54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f487a56, SUC_lexicon_tags_37 },
    { 0x4dc87a57, SUC_lexicon_tags_27 },
    { 0x4fc87a58, SUC_lexicon_tags_37 },
    { 0x20e87a58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2887a60, SUC_lexicon_tags_11 },
    { 0x1c687a60, SUC_lexicon_tags_0 },
    { 0xe0287a61, SUC_lexicon_tags_37 },
    { 0x8b087a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c87a6b, SUC_lexicon_tags_18 },
    { 0xa6487a6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcda87a6e, SUC_lexicon_tags_12 },
    { 0xa287a6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb0a87a71, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca887a76, SUC_lexicon_tags_15 },
    { 0x38a87a77, SUC_lexicon_tags_77 },
    { 0xc9887a77, SUC_lexicon_tags_26 },
    { 0x9e87a79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd887a7b, SUC_lexicon_tags_37 },
    { 0x6ea87a7c, SUC_lexicon_tags_26 },
    { 0x66887a7d, SUC_lexicon_tags_82 },
    { 0x22c87a7e, SUC_lexicon_tags_0 },
    { 0xa0a87a7d, SUC_lexicon_tags_43 },
    { 0xda687a7f, SUC_lexicon_tags_15 },
    { 0x7ea87a81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c487a87, SUC_lexicon_tags_17 },
    { 0x74e87a88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3287a8a, SUC_lexicon_tags_7 },
    { 0xf1a87a8b, SUC_lexicon_tags_46 },
    { 0x6de87a8a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5287a90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x17e87a93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9c87a95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f487a99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb687a9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e87aa1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8487aa4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b687aad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa9887aaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d087ab7, SUC_lexicon_tags_46 },
    { 0xf3287ab7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47687aba, SUC_lexicon_tags_11 },
    { 0xefe87aba, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xcaa87abd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf487ac0, SUC_lexicon_tags_11 },
    { 0x52887ac0, SUC_lexicon_tags_19 },
    { 0x90087ac0, SUC_lexicon_tags_7 },
    { 0xf2c87ac3, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6087ac7, SUC_lexicon_tags_1157 },
    { 0x1b087ac8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x77a87aca, SUC_lexicon_tags_49 },
    { 0x9e487acb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4487acf, SUC_lexicon_tags_37 },
    { 0xacc87ad0, SUC_lexicon_tags_7 },
    { 0xfc087ad1, SUC_lexicon_tags_6 },
    { 0x48c87ad2, SUC_lexicon_tags_63 },
    { 0x8dc87ad1, SUC_lexicon_tags_11 },
    { 0x6c287ad4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b087ad8, SUC_lexicon_tags_19 },
    { 0x8a687ad9, SUC_lexicon_tags_0 },
    { 0xc687ada, SUC_lexicon_tags_16 },
    { 0xd2487adb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24487add, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdd687adf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6087ae9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e87aed, SUC_lexicon_tags_37 },
    { 0x91087aee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7487af4, SUC_lexicon_tags_37 },
    { 0xf7487af5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60087af9, SUC_lexicon_tags_15 },
    { 0x11287afa, SUC_lexicon_tags_17 },
    { 0xefc87afb, SUC_lexicon_tags_27 },
    { 0x43687afc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc487b01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d287b04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9c87b06, SUC_lexicon_tags_14 },
    { 0x28c87b07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a87b0c, SUC_lexicon_tags_11 },
    { 0x4aa87b0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf487b12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec87b17, SUC_lexicon_tags_14 },
    { 0xa4087b18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a87b1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31487b1e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbdc87b20, SUC_lexicon_tags_14 },
    { 0x1e87b21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc487b26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1887b2d, SUC_lexicon_tags_17 },
    { 0x5287b2e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xba887b30, SUC_lexicon_tags_55 },
    { 0x30287b30, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf087b35, SUC_lexicon_tags_16 },
    { 0xd6e87b35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc0c87b38, SUC_lexicon_tags_71 },
    { 0xefa87b39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c87b3c, SUC_lexicon_tags_2 },
    { 0x5d687b3c, SUC_lexicon_tags_87 },
    { 0x9be87b3e, SUC_lexicon_tags_87 },
    { 0xfe487b3f, SUC_lexicon_tags_1 },
    { 0x80a87b40, SUC_lexicon_tags_141 },
    { 0x85287b3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x59887b44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c87b47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a87b4d, SUC_lexicon_tags_15 },
    { 0x63287b4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1287b52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19087b57, SUC_lexicon_tags_9 },
    { 0xf3887b57, SUC_lexicon_tags_129 },
    { 0x8a687b58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6287b5c, SUC_lexicon_tags_2 },
    { 0x63c87b5d, SUC_lexicon_tags_17 },
    { 0x7e287b5d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x83287b60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7687b66, SUC_lexicon_tags_27 },
    { 0x2c487b67, SUC_lexicon_tags_8 },
    { 0x1c687b68, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc87b6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc287b73, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac287b78, SUC_lexicon_tags_12 },
    { 0xee287b78, SUC_lexicon_tags_25 },
    { 0x8fc87b7a, SUC_lexicon_tags_10 },
    { 0xca887b7b, SUC_lexicon_tags_2 },
    { 0xb5a87b7a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x30287b7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a87b86, SUC_lexicon_tags_25 },
    { 0xe8287b87, SUC_lexicon_tags_60 },
    { 0x12687b87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4487b8c, SUC_lexicon_tags_28 },
    { 0x61c87b8d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b687b91, SUC_lexicon_tags_37 },
    { 0x4b487b92, SUC_lexicon_tags_2 },
    { 0xef087b93, SUC_lexicon_tags_17 },
    { 0x4aa87b94, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7887b98, SUC_lexicon_tags_10 },
    { 0xe7a87b98, SUC_lexicon_tags_39 },
    { 0x78a87b98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x86887b9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3287ba3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb5487ba5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbdc87ba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce87bac, SUC_lexicon_tags_54 },
    { 0x5ce87bad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba87bb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9487bbd, SUC_lexicon_tags_296 },
    { 0xf1487bbd, SUC_lexicon_tags_46 },
    { 0x32887bbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2087bc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d287bc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b287bca, SUC_lexicon_tags_17 },
    { 0x6c287bcb, SUC_lexicon_tags_87 },
    { 0x3c087bcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3687bcf, SUC_lexicon_tags_60 },
    { 0x7d887bcf, SUC_lexicon_tags_11 },
    { 0x7d287bd0, SUC_lexicon_tags_106 },
    { 0xf3087bcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0687bd6, SUC_lexicon_tags_60 },
    { 0x86487bd7, SUC_lexicon_tags_11 },
    { 0x1ca87bd8, SUC_lexicon_tags_17 },
    { 0xe5287bd8, SUC_lexicon_tags_17 },
    { 0x66287bda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21487bdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe4687bde, SUC_lexicon_tags_31 },
    { 0x61087bdf, SUC_lexicon_tags_19 },
    { 0x33687bdf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5887be7, SUC_lexicon_tags_17 },
    { 0x85e87be8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98287bea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a487bed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c887bf0, SUC_lexicon_tags_0 },
    { 0x8be87bf1, SUC_lexicon_tags_12 },
    { 0x37687bf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d887bf4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a87bfa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e87c03, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a487c0c, SUC_lexicon_tags_14 },
    { 0x62487c0d, SUC_lexicon_tags_14 },
    { 0x12487c0e, SUC_lexicon_tags_47 },
    { 0xf9287c0f, SUC_lexicon_tags_63 },
    { 0x6f087c0d, SUC_lexicon_tags_5 },
    { 0xe5c87c11, SUC_lexicon_tags_60 },
    { 0xd7287c11, SUC_lexicon_tags_12 },
    { 0xdb687c0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97287c16, SUC_lexicon_tags_38 },
    { 0x22e87c17, SUC_lexicon_tags_14 },
    { 0x8d687c17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b087c1a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e87c1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de87c25, SUC_lexicon_tags_15 },
    { 0xe1487c25, SUC_lexicon_tags_12 },
    { 0x44a87c27, SUC_lexicon_tags_49 },
    { 0x10687c26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc87c38, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x36887c3b, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce687c40, SUC_lexicon_tags_25 },
    { 0xce87c40, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da87c47, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a287c4a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c87c4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xff087c4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26087c57, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5bc87c59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde87c5d, SUC_lexicon_tags_101 },
    { 0xc87c5e, SUC_lexicon_tags_49 },
    { 0xc3487c5d, SUC_lexicon_tags_12 },
    { 0xd2487c60, SUC_lexicon_tags_14 },
    { 0x84687c5e, SUC_lexicon_tags_11 },
    { 0xc3887c62, SUC_lexicon_tags_15 },
    { 0xfb087c63, SUC_lexicon_tags_11 },
    { 0xb0487c64, SUC_lexicon_tags_14 },
    { 0x24087c62, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c87c69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97287c6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba87c6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb287c71, SUC_lexicon_tags_1 },
    { 0xf8887c71, SUC_lexicon_tags_47 },
    { 0xca487c73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe87c76, SUC_lexicon_tags_14 },
    { 0xc8487c77, SUC_lexicon_tags_45 },
    { 0xed087c76, SUC_lexicon_tags_37 },
    { 0x4ea87c79, SUC_lexicon_tags_17 },
    { 0xd1e87c78, SUC_lexicon_tags_26 },
    { 0xbe287c7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7887c7d, SUC_lexicon_tags_25 },
    { 0xc487c7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe487c80, SUC_lexicon_tags_63 },
    { 0x82887c81, SUC_lexicon_tags_26 },
    { 0x3b087c82, SUC_lexicon_tags_12 },
    { 0xb2687c83, SUC_lexicon_tags_43 },
    { 0x15487c82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2887c88, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xafe87c8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae087c8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2a87c8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ee87c90, SUC_lexicon_tags_15 },
    { 0xe7c87c91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85a87c93, SUC_lexicon_tags_25 },
    { 0xf0687c94, SUC_lexicon_tags_14 },
    { 0x3687c94, SUC_lexicon_tags_48 },
    { 0x6c287c96, SUC_lexicon_tags_17 },
    { 0xec887c97, SUC_lexicon_tags_17 },
    { 0x80287c94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x51687c9a, SUC_lexicon_tags_6 },
    { 0xaa087c9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca87ca1, SUC_lexicon_tags_17 },
    { 0xc3287ca2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec87ca5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a87cad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a487cb5, SUC_lexicon_tags_33 },
    { 0x82e87cb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0287cba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1887cc2, SUC_lexicon_tags_11 },
    { 0x7c087cc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b487cca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x60e87ccc, SUC_lexicon_tags_11 },
    { 0x72687ccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c287cd3, SUC_lexicon_tags_37 },
    { 0xb8a87cd4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x27e87cd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48887cd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x28e87cda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1887cdd, SUC_lexicon_tags_42 },
    { 0x38487cde, SUC_lexicon_tags_25 },
    { 0xba87cdf, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0xdd087ce1, SUC_lexicon_tags_63 },
    { 0x2f887ce1, SUC_lexicon_tags_65 },
    { 0x14a87ce3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda487cec, SUC_lexicon_tags_1534 },
    { 0xa1087ced, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a87cf0, SUC_lexicon_tags_1 },
    { 0xbc87cf1, SUC_lexicon_tags_0 },
    { 0xd3087cf1, SUC_lexicon_tags_25 },
    { 0x2de87cf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93287cf5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4887cf9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x56287cfb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9b687cfd, SUC_lexicon_tags_1 },
    { 0x61e87cfe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70287d02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac87d04, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c887d07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c887d0a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x72087d0d, SUC_lexicon_tags_11 },
    { 0x2c487d0e, SUC_lexicon_tags_38 },
    { 0x2a287d0f, SUC_lexicon_tags_25 },
    { 0xec087d10, SUC_lexicon_tags_12 },
    { 0xce287d11, SUC_lexicon_tags_54 },
    { 0xbe287d11, SUC_lexicon_tags_29 },
    { 0x88887d0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14087d1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c87d1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96c87d1f, SUC_lexicon_tags_13 },
    { 0xf6487d20, SUC_lexicon_tags_37 },
    { 0x57687d21, SUC_lexicon_tags_1 },
    { 0xa0a87d22, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de87d25, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c87d28, SUC_lexicon_tags_25 },
    { 0x33c87d29, SUC_lexicon_tags_11 },
    { 0x47887d2a, SUC_lexicon_tags_255 },
    { 0x20a87d2a, SUC_lexicon_tags_0 },
    { 0xa7287d2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14a87d2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e87d32, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a487d36, SUC_lexicon_tags_183 },
    { 0x6de87d36, SUC_lexicon_tags_38 },
    { 0xbb687d37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4887d3c, SUC_lexicon_tags_17 },
    { 0x41e87d3c, SUC_lexicon_tags_23 },
    { 0xff687d3c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7ee87d40, SUC_lexicon_tags_27 },
    { 0x39a87d41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2487d43, SUC_lexicon_tags_25 },
    { 0x1d887d44, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd1287d46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae87d4c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x20287d4e, SUC_lexicon_tags_31 },
    { 0x73a87d4f, SUC_lexicon_tags_25 },
    { 0xf0c87d50, SUC_lexicon_tags_17 },
    { 0xb7687d4e, SUC_lexicon_tags_56 },
    { 0xa7487d4e, SUC_lexicon_tags_25 },
    { 0xa8487d53, SUC_lexicon_tags_8 },
    { 0xc9c87d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13a87d56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88887d5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde287d62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e87d65, SUC_lexicon_tags_15 },
    { 0x49e87d66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e87d69, SUC_lexicon_tags_25 },
    { 0x42287d6a, SUC_lexicon_tags_9 },
    { 0xc9487d6b, SUC_lexicon_tags_37 },
    { 0x1be87d69, SUC_lexicon_tags_38 },
    { 0x67e87d6a, SUC_lexicon_tags_11 },
    { 0x2f087d6e, SUC_lexicon_tags_10 },
    { 0xfde87d6c, SUC_lexicon_tags_71 },
    { 0x39a87d6f, SUC_lexicon_tags_6 },
    { 0x7f687d6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd5c87d73, SUC_lexicon_tags_7 },
    { 0x91c87d73, SUC_lexicon_tags_60 },
    { 0x5e87d75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3687d78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc87d7b, SUC_lexicon_tags_25 },
    { 0xf7087d7c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62287d82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99887d87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ba87d89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10887d92, SUC_lexicon_tags_63 },
    { 0x5ac87d92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc87d96, SUC_lexicon_tags_27 },
    { 0xd6487d97, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6687d9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45887d9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x64287da1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e87dab, SUC_lexicon_tags_5 },
    { 0xa887dab, SUC_lexicon_tags_28 },
    { 0x90487dad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac87db2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x36487db5, SUC_lexicon_tags_19 },
    { 0x44687db5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c87dba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa87dc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3687dc4, SUC_lexicon_tags_15 },
    { 0x68087dc5, SUC_lexicon_tags_170 },
    { 0x4887dc6, SUC_lexicon_tags_4 },
    { 0x65a87dc7, SUC_lexicon_tags_0 },
    { 0x12087dc8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57287dce, SUC_lexicon_tags_11 },
    { 0xd8287dcf, SUC_lexicon_tags_2 },
    { 0x3b287dd0, SUC_lexicon_tags_2 },
    { 0xaf887dd1, SUC_lexicon_tags_26 },
    { 0x90a87dd1, SUC_lexicon_tags_15 },
    { 0x71687dd3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd7887dd5, SUC_lexicon_tags_43 },
    { 0x3dc87dd6, SUC_lexicon_tags_1 },
    { 0x64e87dd7, SUC_lexicon_tags_27 },
    { 0x96e87dd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30887ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de87de1, SUC_lexicon_tags_32 },
    { 0x1e87de1, SUC_lexicon_tags_1 },
    { 0x66887de1, SUC_lexicon_tags_68 },
    { 0xbb087de1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32287de6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6887de8, SUC_lexicon_tags_37 },
    { 0xe487de9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3e887deb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7087dee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd487df0, SUC_lexicon_tags_56 },
    { 0x7ba87df1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c87df5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dc87df7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35087dfa, SUC_lexicon_tags_15 },
    { 0xefc87dfa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c87e00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d687e06, SUC_lexicon_tags_11 },
    { 0xb1487e07, SUC_lexicon_tags_66 },
    { 0x88a87e06, SUC_lexicon_tags_15 },
    { 0xe887e07, SUC_lexicon_tags_15 },
    { 0xd9a87e0a, SUC_lexicon_tags_66 },
    { 0x81287e0a, SUC_lexicon_tags_87 },
    { 0xe5887e0c, SUC_lexicon_tags_32 },
    { 0x1687e0c, SUC_lexicon_tags_1 },
    { 0x4d687e0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71487e10, SUC_lexicon_tags_56 },
    { 0xb7e87e10, SUC_lexicon_tags_1 },
    { 0x8f487e12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a87e16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c087e18, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32087e1c, SUC_lexicon_tags_37 },
    { 0xbd487e1c, SUC_lexicon_tags_32 },
    { 0x23287e1e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c87e21, SUC_lexicon_tags_42 },
    { 0x14687e21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19487e27, SUC_lexicon_tags_37 },
    { 0x99087e27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c87e2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x25287e30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34287e34, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e87e38, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xbd487e3a, SUC_lexicon_tags_30 },
    { 0x10287e3b, SUC_lexicon_tags_11 },
    { 0xb6e87e3c, SUC_lexicon_tags_27 },
    { 0xeb487e3c, SUC_lexicon_tags_11 },
    { 0xf0087e3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79087e40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeec87e42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e087e44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f887e46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd687e4b, SUC_lexicon_tags_1 },
    { 0x55c87e4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5d287e4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e87e51, SUC_lexicon_tags_7 },
    { 0x39687e52, SUC_lexicon_tags_1 },
    { 0x57487e53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c87e57, SUC_lexicon_tags_15 },
    { 0x19e87e58, SUC_lexicon_tags_0 },
    { 0x11287e59, SUC_lexicon_tags_29 },
    { 0xc9087e57, SUC_lexicon_tags_12 },
    { 0x26e87e5a, SUC_lexicon_tags_30 },
    { 0x3b287e5c, SUC_lexicon_tags_23 },
    { 0xa687e5d, SUC_lexicon_tags_11 },
    { 0xdaa87e57, SUC_lexicon_tags_17 },
    { 0xdfc87e5b, SUC_lexicon_tags_31 },
    { 0x60687e5b, SUC_lexicon_tags_17 },
    { 0xa7e87e61, SUC_lexicon_tags_2 },
    { 0xa1687e62, SUC_lexicon_tags_65 },
    { 0xe5287e63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3887e65, SUC_lexicon_tags_77 },
    { 0xacc87e66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33c87e68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e87e6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf1a87e6f, SUC_lexicon_tags_23 },
    { 0x75c87e6f, SUC_lexicon_tags_11 },
    { 0x2a087e71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78287e73, SUC_lexicon_tags_1 },
    { 0x3a287e73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb287e76, SUC_lexicon_tags_31 },
    { 0x3de87e77, SUC_lexicon_tags_2 },
    { 0xe5e87e76, SUC_lexicon_tags_30 },
    { 0xc2c87e76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88087e7e, SUC_lexicon_tags_17 },
    { 0x49687e7f, SUC_lexicon_tags_15 },
    { 0x6ec87e80, SUC_lexicon_tags_27 },
    { 0xabc87e80, SUC_lexicon_tags_17 },
    { 0x32a87e80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c687e89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29887e8d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d287e94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x46a87e96, SUC_lexicon_tags_37 },
    { 0x72687e97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2287e9c, SUC_lexicon_tags_37 },
    { 0x8b687e9d, SUC_lexicon_tags_17 },
    { 0x8cc87e9e, SUC_lexicon_tags_37 },
    { 0x3f687e9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e687ea3, SUC_lexicon_tags_17 },
    { 0xbb687ea3, SUC_lexicon_tags_0 },
    { 0xb7487ea3, SUC_lexicon_tags_47 },
    { 0xfbc87ea3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x50887ea8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c87eac, SUC_lexicon_tags_29 },
    { 0xfc887ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8287eaf, SUC_lexicon_tags_25 },
    { 0xe0687eb0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x74687eb2, SUC_lexicon_tags_12 },
    { 0xff487eb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b687eb6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc7a87eb8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0887ebc, SUC_lexicon_tags_15 },
    { 0x1c287ebd, SUC_lexicon_tags_9 },
    { 0xc5a87ebd, SUC_lexicon_tags_0 },
    { 0x3e287ebf, SUC_lexicon_tags_37 },
    { 0xbac87ebd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c87ec3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e87ec6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd487ec8, SUC_lexicon_tags_23 },
    { 0xdca87ec9, SUC_lexicon_tags_12 },
    { 0x21487ec9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d687ece, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7c87ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43487ed4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5287ed6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2ba87ed8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e87edc, SUC_lexicon_tags_25 },
    { 0x78287edd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc687ee0, SUC_lexicon_tags_25 },
    { 0x11887ee1, SUC_lexicon_tags_25 },
    { 0xf6a87ee2, SUC_lexicon_tags_12 },
    { 0xc2c87ee1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb287ee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c87eed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x40487eef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x52087ef1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b487ef4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c87ef8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc887eff, SUC_lexicon_tags_37 },
    { 0xb6687eff, SUC_lexicon_tags_20 },
    { 0xdf887f01, SUC_lexicon_tags_0 },
    { 0xb9a87f02, SUC_lexicon_tags_2 },
    { 0x7ee87f03, SUC_lexicon_tags_29 },
    { 0x46e87f04, SUC_lexicon_tags_0 },
    { 0x3bc87f05, SUC_lexicon_tags_12 },
    { 0x6a487f05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe4e87f08, SUC_lexicon_tags_37 },
    { 0x887f09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3487f0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0a87f0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7be87f0f, SUC_lexicon_tags_27 },
    { 0xcf087f10, SUC_lexicon_tags_87 },
    { 0x37e87f0f, SUC_lexicon_tags_1529 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9887f18, SUC_lexicon_tags_17 },
    { 0x44087f19, SUC_lexicon_tags_7 },
    { 0xc4687f19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a87f1d, SUC_lexicon_tags_4 },
    { 0xfb287f1e, SUC_lexicon_tags_0 },
    { 0x25a87f1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e687f23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5887f30, SUC_lexicon_tags_47 },
    { 0x54087f31, SUC_lexicon_tags_30 },
    { 0xe9087f32, SUC_lexicon_tags_11 },
    { 0x43887f32, SUC_lexicon_tags_7 },
    { 0x5b487f34, SUC_lexicon_tags_27 },
    { 0xdf487f35, SUC_lexicon_tags_1 },
    { 0x19a87f35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc087f39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94a87f3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ac87f3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0487f47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc87f4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3687f52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfba87f54, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e287f58, SUC_lexicon_tags_17 },
    { 0x73e87f59, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6687f5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91287f5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc87f61, SUC_lexicon_tags_10 },
    { 0x57687f62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c87f65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7e87f67, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a287f6e, SUC_lexicon_tags_61 },
    { 0xe4887f6e, SUC_lexicon_tags_68 },
    { 0x14087f6e, SUC_lexicon_tags_83 },
    { 0xf5c87f71, SUC_lexicon_tags_15 },
    { 0x90887f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc87f78, SUC_lexicon_tags_19 },
    { 0x1c087f79, SUC_lexicon_tags_12 },
    { 0x91087f7a, SUC_lexicon_tags_11 },
    { 0xd4c87f79, SUC_lexicon_tags_892 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f087f7e, SUC_lexicon_tags_17 },
    { 0xd8087f7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a087f85, SUC_lexicon_tags_5 },
    { 0xe0a87f86, SUC_lexicon_tags_336 },
    { 0xbd487f85, SUC_lexicon_tags_25 },
    { 0xd3087f87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71487f8d, SUC_lexicon_tags_17 },
    { 0x6cc87f8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23887f93, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a87f97, SUC_lexicon_tags_26 },
    { 0x72887f98, SUC_lexicon_tags_13 },
    { 0x46087f99, SUC_lexicon_tags_17 },
    { 0xdf887f99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb487f9d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x94887fa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1887fa7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3b487fa9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a87fac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4087faf, SUC_lexicon_tags_1 },
    { 0xa3487fb0, SUC_lexicon_tags_1 },
    { 0x6d287fb0, SUC_lexicon_tags_11 },
    { 0x2e287fb1, SUC_lexicon_tags_17 },
    { 0xd0287fb1, SUC_lexicon_tags_39 },
    { 0x36e87fb2, SUC_lexicon_tags_15 },
    { 0xffa87fb5, SUC_lexicon_tags_17 },
    { 0x74a87fb1, SUC_lexicon_tags_5 },
    { 0xe3c87fb5, SUC_lexicon_tags_38 },
    { 0x84287fb8, SUC_lexicon_tags_9 },
    { 0xe6887fb9, SUC_lexicon_tags_37 },
    { 0x31e87fb8, SUC_lexicon_tags_1 },
    { 0x63287fbb, SUC_lexicon_tags_37 },
    { 0xaf487fbb, SUC_lexicon_tags_3 },
    { 0x4087fbd, SUC_lexicon_tags_7 },
    { 0x4a887fbd, SUC_lexicon_tags_7 },
    { 0xf3887fbc, SUC_lexicon_tags_11 },
    { 0xb1a87fc0, SUC_lexicon_tags_47 },
    { 0x4a087fc1, SUC_lexicon_tags_7 },
    { 0xa1887fc2, SUC_lexicon_tags_15 },
    { 0xc1a87fb8, SUC_lexicon_tags_38 },
    { 0xaec87fbf, SUC_lexicon_tags_25 },
    { 0x29887fc2, SUC_lexicon_tags_25 },
    { 0x8c087fc6, SUC_lexicon_tags_17 },
    { 0x21887fc6, SUC_lexicon_tags_12 },
    { 0x8287fc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xada87fcb, SUC_lexicon_tags_43 },
    { 0x7d287fcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2087fce, SUC_lexicon_tags_26 },
    { 0xf3487fce, SUC_lexicon_tags_14 },
    { 0x6ec87fcf, SUC_lexicon_tags_34 },
    { 0x63887fcf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbb087fd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d287fd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x75a87fd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95687fda, SUC_lexicon_tags_25 },
    { 0x7c287fdb, SUC_lexicon_tags_27 },
    { 0xb0487fdb, SUC_lexicon_tags_167 },
    { 0x88e87fdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b687fdf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29887fe4, SUC_lexicon_tags_19 },
    { 0xba287fe4, SUC_lexicon_tags_60 },
    { 0x9f687fe6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf2487fe8, SUC_lexicon_tags_15 },
    { 0x2687fe8, SUC_lexicon_tags_37 },
    { 0x43087fe8, SUC_lexicon_tags_12 },
    { 0x9b887fe9, SUC_lexicon_tags_25 },
    { 0x87887feb, SUC_lexicon_tags_25 },
    { 0x21887fed, SUC_lexicon_tags_37 },
    { 0xa6887fed, SUC_lexicon_tags_0 },
    { 0x48487fee, SUC_lexicon_tags_11 },
    { 0x7a487ff0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb287ff5, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c87ff8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0487ffd, SUC_lexicon_tags_66 },
    { 0xa1e87ffe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16088005, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c888008, SUC_lexicon_tags_32 },
    { 0xa0688008, SUC_lexicon_tags_85 },
    { 0x56088009, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9368800d, SUC_lexicon_tags_1 },
    { 0x4268800e, SUC_lexicon_tags_37 },
    { 0x7448800f, SUC_lexicon_tags_1 },
    { 0x9a28800f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x82a88012, SUC_lexicon_tags_33 },
    { 0x62888012, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c88018, SUC_lexicon_tags_17 },
    { 0x3e88018, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa628801e, SUC_lexicon_tags_11 },
    { 0x1c48801e, SUC_lexicon_tags_15 },
    { 0x81288020, SUC_lexicon_tags_37 },
    { 0xc548801e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be88027, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xbbe88029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd548802b, SUC_lexicon_tags_2 },
    { 0x8628802c, SUC_lexicon_tags_37 },
    { 0xe9a8802d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba488031, SUC_lexicon_tags_81 },
    { 0xf0688031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea288034, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6288037, SUC_lexicon_tags_29 },
    { 0x1688038, SUC_lexicon_tags_0 },
    { 0xc4488038, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4928803d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85a8803f, SUC_lexicon_tags_47 },
    { 0xe8088040, SUC_lexicon_tags_47 },
    { 0xfa688040, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ec88043, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5688049, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57e8804b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe388804e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x50e88050, SUC_lexicon_tags_2 },
    { 0x9d888051, SUC_lexicon_tags_43 },
    { 0xa9c88051, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc88055, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d888058, SUC_lexicon_tags_1 },
    { 0x8e688059, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48805d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e48805f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41488064, SUC_lexicon_tags_24 },
    { 0x65488064, SUC_lexicon_tags_38 },
    { 0xe3888066, SUC_lexicon_tags_17 },
    { 0x59888067, SUC_lexicon_tags_0 },
    { 0xeac88068, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca8806f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa88075, SUC_lexicon_tags_15 },
    { 0x81c88075, SUC_lexicon_tags_14 },
    { 0x78888076, SUC_lexicon_tags_43 },
    { 0x2c88078, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6048807a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a8807d, SUC_lexicon_tags_15 },
    { 0xa428807e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f088080, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x45088082, SUC_lexicon_tags_14 },
    { 0x40888083, SUC_lexicon_tags_39 },
    { 0x37c88084, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe888087, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac8808b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2168808f, SUC_lexicon_tags_1 },
    { 0x4908808f, SUC_lexicon_tags_94 },
    { 0xd9488091, SUC_lexicon_tags_26 },
    { 0xe0488090, SUC_lexicon_tags_79 },
    { 0xbb488093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4488096, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a8809f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf92880a1, SUC_lexicon_tags_14 },
    { 0x402880a1, SUC_lexicon_tags_2 },
    { 0xd0a880a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ae880a5, SUC_lexicon_tags_64 },
    { 0x5a8880a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfe880a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e880ab, SUC_lexicon_tags_26 },
    { 0xfc6880ac, SUC_lexicon_tags_25 },
    { 0x6d6880ad, SUC_lexicon_tags_35 },
    { 0xdf8880ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e880b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x416880b3, SUC_lexicon_tags_11 },
    { 0x3ba880b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x902880b6, SUC_lexicon_tags_79 },
    { 0x5e6880b6, SUC_lexicon_tags_25 },
    { 0x312880b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdc8880ba, SUC_lexicon_tags_11 },
    { 0x118880bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x57e880bd, SUC_lexicon_tags_15 },
    { 0xeb2880bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ec880c0, SUC_lexicon_tags_1 },
    { 0x880c0, SUC_lexicon_tags_17 },
    { 0x7a880c0, SUC_lexicon_tags_33 },
    { 0x642880c1, SUC_lexicon_tags_1 },
    { 0xf5e880c0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4880c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f0880ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x930880cc, SUC_lexicon_tags_7 },
    { 0x2880cc, SUC_lexicon_tags_676 },
    { 0, NULL },
    { 0xb36880cf, SUC_lexicon_tags_37 },
    { 0xaec880d0, SUC_lexicon_tags_19 },
    { 0x51c880d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0880d4, SUC_lexicon_tags_12 },
    { 0xc86880d5, SUC_lexicon_tags_12 },
    { 0x3c6880d6, SUC_lexicon_tags_25 },
    { 0x16a880d6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0880de, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8880e5, SUC_lexicon_tags_32 },
    { 0x5c2880e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0880e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x398880ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x160880ee, SUC_lexicon_tags_15 },
    { 0x78a880ee, SUC_lexicon_tags_21 },
    { 0xd26880ee, SUC_lexicon_tags_7 },
    { 0xb90880ee, SUC_lexicon_tags_1 },
    { 0xf0e880f2, SUC_lexicon_tags_47 },
    { 0x5d8880ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2880f7, SUC_lexicon_tags_11 },
    { 0xe48880f8, SUC_lexicon_tags_17 },
    { 0x2da880f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c880fc, SUC_lexicon_tags_63 },
    { 0x5b4880fd, SUC_lexicon_tags_1 },
    { 0x94a880fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17688101, SUC_lexicon_tags_15 },
    { 0x6d888101, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1288104, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x43888107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e088109, SUC_lexicon_tags_17 },
    { 0xfd88810a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c8810d, SUC_lexicon_tags_87 },
    { 0x9bc8810e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf688111, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x53a88113, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1088119, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87a8811b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc8811e, SUC_lexicon_tags_33 },
    { 0x56e8811e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6688121, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e88125, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3c88127, SUC_lexicon_tags_22 },
    { 0xe9e88127, SUC_lexicon_tags_25 },
    { 0x47088128, SUC_lexicon_tags_7 },
    { 0xef88812a, SUC_lexicon_tags_25 },
    { 0x82c8812b, SUC_lexicon_tags_60 },
    { 0xee88812c, SUC_lexicon_tags_0 },
    { 0xeae88129, SUC_lexicon_tags_37 },
    { 0xf268812e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd688131, SUC_lexicon_tags_37 },
    { 0xce488132, SUC_lexicon_tags_28 },
    { 0x47e88133, SUC_lexicon_tags_14 },
    { 0xb3c88133, SUC_lexicon_tags_17 },
    { 0xb9888133, SUC_lexicon_tags_15 },
    { 0x6b088131, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a8813b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb88813f, SUC_lexicon_tags_14 },
    { 0x54c8813f, SUC_lexicon_tags_17 },
    { 0x7b488141, SUC_lexicon_tags_31 },
    { 0x72088142, SUC_lexicon_tags_60 },
    { 0x70688143, SUC_lexicon_tags_26 },
    { 0x8888144, SUC_lexicon_tags_27 },
    { 0xd488143, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08814a, SUC_lexicon_tags_11 },
    { 0xff68814b, SUC_lexicon_tags_605 },
    { 0x1cc8814c, SUC_lexicon_tags_39 },
    { 0xc568814b, SUC_lexicon_tags_25 },
    { 0xbe28814d, SUC_lexicon_tags_45 },
    { 0x95a8814e, SUC_lexicon_tags_5 },
    { 0x8f288150, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xed288153, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe888155, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54488158, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba08815c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b888160, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c88165, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf048816a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3628816c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf508816e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4088175, SUC_lexicon_tags_14 },
    { 0xb8688176, SUC_lexicon_tags_10 },
    { 0x78888175, SUC_lexicon_tags_12 },
    { 0x70688178, SUC_lexicon_tags_38 },
    { 0x25688179, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3628817b, SUC_lexicon_tags_30 },
    { 0x2e88817c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb968817e, SUC_lexicon_tags_37 },
    { 0x7648817f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62e88181, SUC_lexicon_tags_31 },
    { 0xb6288181, SUC_lexicon_tags_37 },
    { 0x15a88183, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a88187, SUC_lexicon_tags_11 },
    { 0xd2e88188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa488818b, SUC_lexicon_tags_27 },
    { 0xda68818c, SUC_lexicon_tags_7 },
    { 0x4888818d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c88190, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f288192, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c88195, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e088198, SUC_lexicon_tags_14 },
    { 0xaa488198, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0c8819b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47e8819d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba881a1, SUC_lexicon_tags_33 },
    { 0xe52881a2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48881a6, SUC_lexicon_tags_367 },
    { 0x3f2881a7, SUC_lexicon_tags_12 },
    { 0x4b4881a8, SUC_lexicon_tags_61 },
    { 0x842881a9, SUC_lexicon_tags_47 },
    { 0xf8a881a8, SUC_lexicon_tags_38 },
    { 0x3e8881a9, SUC_lexicon_tags_27 },
    { 0x358881a7, SUC_lexicon_tags_12 },
    { 0xbb8881ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c881b1, SUC_lexicon_tags_961 },
    { 0xc30881b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94881b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e881bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x81a881c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a881c5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf1e881c7, SUC_lexicon_tags_15 },
    { 0xc9c881c7, SUC_lexicon_tags_55 },
    { 0x4ea881c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8881ce, SUC_lexicon_tags_160 },
    { 0x6f6881ce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214881d3, SUC_lexicon_tags_1 },
    { 0xe6e881d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4881d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852881dd, SUC_lexicon_tags_25 },
    { 0xb00881dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6881e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0881eb, SUC_lexicon_tags_37 },
    { 0x2de881ec, SUC_lexicon_tags_17 },
    { 0x312881eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4a881ef, SUC_lexicon_tags_37 },
    { 0x1b2881f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc3e881f2, SUC_lexicon_tags_1 },
    { 0x632881f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58c881f5, SUC_lexicon_tags_123 },
    { 0xe10881f6, SUC_lexicon_tags_12 },
    { 0x132881f7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfc8881f9, SUC_lexicon_tags_29 },
    { 0x41a881f9, SUC_lexicon_tags_29 },
    { 0xb30881fb, SUC_lexicon_tags_25 },
    { 0x7a6881f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe18881fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe2c88200, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x77c88202, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9688207, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xfc488209, SUC_lexicon_tags_27 },
    { 0x4708820a, SUC_lexicon_tags_66 },
    { 0x8ce8820b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba88214, SUC_lexicon_tags_15 },
    { 0x8ae88215, SUC_lexicon_tags_11 },
    { 0xbae88216, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5f488218, SUC_lexicon_tags_65 },
    { 0x1a88218, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x59a8821b, SUC_lexicon_tags_25 },
    { 0xb6c8821c, SUC_lexicon_tags_6 },
    { 0x5448821d, SUC_lexicon_tags_53 },
    { 0xeac8821e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a88221, SUC_lexicon_tags_1 },
    { 0xa0088222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2408822b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a48822e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc88233, SUC_lexicon_tags_17 },
    { 0x22e88233, SUC_lexicon_tags_4 },
    { 0x79488235, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1da88237, SUC_lexicon_tags_17 },
    { 0xafe88238, SUC_lexicon_tags_25 },
    { 0x49a88239, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa308823e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c88241, SUC_lexicon_tags_14 },
    { 0xbd688241, SUC_lexicon_tags_33 },
    { 0x46e88241, SUC_lexicon_tags_21 },
    { 0x39088244, SUC_lexicon_tags_11 },
    { 0xef288244, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8288247, SUC_lexicon_tags_47 },
    { 0xc0a88248, SUC_lexicon_tags_11 },
    { 0x6c888248, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97688252, SUC_lexicon_tags_63 },
    { 0x34e88253, SUC_lexicon_tags_4 },
    { 0xcda88253, SUC_lexicon_tags_38 },
    { 0x17088254, SUC_lexicon_tags_13 },
    { 0x29e88256, SUC_lexicon_tags_11 },
    { 0xede88255, SUC_lexicon_tags_11 },
    { 0xe7a88258, SUC_lexicon_tags_38 },
    { 0x30288257, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7028825b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c68825f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f488262, SUC_lexicon_tags_33 },
    { 0x7d688262, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa888265, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x91888267, SUC_lexicon_tags_91 },
    { 0x79288268, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca48826c, SUC_lexicon_tags_45 },
    { 0x4ca8826d, SUC_lexicon_tags_0 },
    { 0x21e8826e, SUC_lexicon_tags_57 },
    { 0x19c8826f, SUC_lexicon_tags_92 },
    { 0xa348826f, SUC_lexicon_tags_43 },
    { 0xa88826e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9de88273, SUC_lexicon_tags_37 },
    { 0x9bc88273, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17c88276, SUC_lexicon_tags_11 },
    { 0xd288276, SUC_lexicon_tags_47 },
    { 0xa5c88277, SUC_lexicon_tags_25 },
    { 0x26888279, SUC_lexicon_tags_84 },
    { 0xd2a8827a, SUC_lexicon_tags_16 },
    { 0xa0e8827b, SUC_lexicon_tags_1 },
    { 0x7de8827b, SUC_lexicon_tags_37 },
    { 0xb368827b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca88281, SUC_lexicon_tags_9 },
    { 0x5d488281, SUC_lexicon_tags_23 },
    { 0xf2e88283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc08828a, SUC_lexicon_tags_30 },
    { 0x7808828a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xce88828d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a88291, SUC_lexicon_tags_31 },
    { 0xf0688292, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a8829c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686882a0, SUC_lexicon_tags_14 },
    { 0xe6882a1, SUC_lexicon_tags_1 },
    { 0x5b0882a1, SUC_lexicon_tags_40 },
    { 0x526882a2, SUC_lexicon_tags_17 },
    { 0xe7a882a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb0882a6, SUC_lexicon_tags_29 },
    { 0x6b8882a7, SUC_lexicon_tags_37 },
    { 0x80882a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e882ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x988882ad, SUC_lexicon_tags_1 },
    { 0x6aa882ae, SUC_lexicon_tags_17 },
    { 0x36a882af, SUC_lexicon_tags_38 },
    { 0x508882b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a882b5, SUC_lexicon_tags_63 },
    { 0x6c6882b6, SUC_lexicon_tags_138 },
    { 0xe88882b6, SUC_lexicon_tags_15 },
    { 0x838882b8, SUC_lexicon_tags_3 },
    { 0x8aa882b9, SUC_lexicon_tags_38 },
    { 0x6b2882ba, SUC_lexicon_tags_48 },
    { 0x6aa882bb, SUC_lexicon_tags_23 },
    { 0xf9e882bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x320882bf, SUC_lexicon_tags_82 },
    { 0xeb0882bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca6882c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x286882c4, SUC_lexicon_tags_32 },
    { 0x672882c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ce882c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcce882ca, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xbba882cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x780882ce, SUC_lexicon_tags_46 },
    { 0xe42882ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0882d2, SUC_lexicon_tags_47 },
    { 0x8fe882d2, SUC_lexicon_tags_29 },
    { 0x906882d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaa0882d6, SUC_lexicon_tags_60 },
    { 0x756882d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c882dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c882df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a882e2, SUC_lexicon_tags_49 },
    { 0x98c882e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x54882e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4882eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764882f0, SUC_lexicon_tags_17 },
    { 0x4ec882f0, SUC_lexicon_tags_92 },
    { 0x720882f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e882f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x722882f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfac882f9, SUC_lexicon_tags_14 },
    { 0x70c882f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c882fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18888300, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2a88302, SUC_lexicon_tags_25 },
    { 0x67888303, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc88307, SUC_lexicon_tags_13 },
    { 0x1c088308, SUC_lexicon_tags_33 },
    { 0xf6488309, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c48830c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf688830e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d888313, SUC_lexicon_tags_45 },
    { 0x7ac88314, SUC_lexicon_tags_60 },
    { 0x1ac88314, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e88319, SUC_lexicon_tags_15 },
    { 0xe088319, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a88329, SUC_lexicon_tags_15 },
    { 0x86488329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c88332, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53088334, SUC_lexicon_tags_2 },
    { 0xab888334, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be88339, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e88340, SUC_lexicon_tags_46 },
    { 0xe9e88341, SUC_lexicon_tags_38 },
    { 0x2ce88342, SUC_lexicon_tags_37 },
    { 0x43088343, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf088346, SUC_lexicon_tags_68 },
    { 0x31288346, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72a88349, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3648834d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b88834f, SUC_lexicon_tags_23 },
    { 0xb3688350, SUC_lexicon_tags_17 },
    { 0x87888350, SUC_lexicon_tags_30 },
    { 0x9a888351, SUC_lexicon_tags_2 },
    { 0x26a88352, SUC_lexicon_tags_38 },
    { 0xdfa88354, SUC_lexicon_tags_15 },
    { 0x6ce88355, SUC_lexicon_tags_37 },
    { 0xb5c88356, SUC_lexicon_tags_25 },
    { 0x69e88356, SUC_lexicon_tags_8 },
    { 0xb1c88357, SUC_lexicon_tags_170 },
    { 0xe0888354, SUC_lexicon_tags_7 },
    { 0xba688359, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa428835c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa628835f, SUC_lexicon_tags_30 },
    { 0x32488360, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c88363, SUC_lexicon_tags_49 },
    { 0xb0c88364, SUC_lexicon_tags_16 },
    { 0xb3c88363, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b288368, SUC_lexicon_tags_45 },
    { 0x3a488369, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17288373, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a88377, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc8837b, SUC_lexicon_tags_28 },
    { 0x4408837c, SUC_lexicon_tags_8 },
    { 0xd588837b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca88382, SUC_lexicon_tags_11 },
    { 0xb2488382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828838a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1f68838c, SUC_lexicon_tags_14 },
    { 0x5a28838d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c88390, SUC_lexicon_tags_13 },
    { 0x95288391, SUC_lexicon_tags_2 },
    { 0x41888391, SUC_lexicon_tags_11 },
    { 0xfd288391, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcb488395, SUC_lexicon_tags_7 },
    { 0x8a088396, SUC_lexicon_tags_37 },
    { 0x6de88397, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1288399, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb728839b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc8839e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e883a3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28883a8, SUC_lexicon_tags_50 },
    { 0xb86883a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4883ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a883b2, SUC_lexicon_tags_25 },
    { 0xb52883b3, SUC_lexicon_tags_26 },
    { 0x54c883b3, SUC_lexicon_tags_46 },
    { 0x444883b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x760883b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec883bb, SUC_lexicon_tags_25 },
    { 0xc9e883bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594883c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf38883c5, SUC_lexicon_tags_47 },
    { 0xb48883c6, SUC_lexicon_tags_12 },
    { 0xf54883c7, SUC_lexicon_tags_25 },
    { 0x88a883c7, SUC_lexicon_tags_30 },
    { 0x7e0883c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa2883cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4d8883cd, SUC_lexicon_tags_17 },
    { 0xf46883cd, SUC_lexicon_tags_11 },
    { 0x73c883cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26883d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x544883d5, SUC_lexicon_tags_43 },
    { 0x4c883d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x620883d8, SUC_lexicon_tags_27 },
    { 0x3de883d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2883dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160883e2, SUC_lexicon_tags_38 },
    { 0x358883e3, SUC_lexicon_tags_6 },
    { 0x270883e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc883e6, SUC_lexicon_tags_7 },
    { 0x8dc883e7, SUC_lexicon_tags_15 },
    { 0x85c883e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf6883ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68883ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52883f1, SUC_lexicon_tags_6 },
    { 0xe3e883f1, SUC_lexicon_tags_106 },
    { 0x902883f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x118883f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x986883f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0883fc, SUC_lexicon_tags_1 },
    { 0x330883fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x302883ff, SUC_lexicon_tags_68 },
    { 0x81c88400, SUC_lexicon_tags_9 },
    { 0x9a688401, SUC_lexicon_tags_13 },
    { 0xde4883ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc88407, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae48840a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe848840c, SUC_lexicon_tags_11 },
    { 0x6008840d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a88410, SUC_lexicon_tags_37 },
    { 0x60688411, SUC_lexicon_tags_104 },
    { 0xdbe88412, SUC_lexicon_tags_7 },
    { 0x1da88411, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc4688415, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d288418, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb28841a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f28841e, SUC_lexicon_tags_2 },
    { 0x1e48841f, SUC_lexicon_tags_7 },
    { 0x27e88420, SUC_lexicon_tags_10 },
    { 0x79288421, SUC_lexicon_tags_37 },
    { 0xa368841f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc88428, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x5e8842a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c8842d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b888430, SUC_lexicon_tags_2 },
    { 0x81088431, SUC_lexicon_tags_62 },
    { 0xfa288431, SUC_lexicon_tags_25 },
    { 0xcdc88433, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a88843a, SUC_lexicon_tags_34 },
    { 0x5068843b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c08843d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2288843f, SUC_lexicon_tags_11 },
    { 0x4668843f, SUC_lexicon_tags_14 },
    { 0xfec88441, SUC_lexicon_tags_11 },
    { 0x6b488442, SUC_lexicon_tags_27 },
    { 0x70088443, SUC_lexicon_tags_37 },
    { 0x92888443, SUC_lexicon_tags_293 },
    { 0x7ac88441, SUC_lexicon_tags_1 },
    { 0x9cc88446, SUC_lexicon_tags_12 },
    { 0x90a88447, SUC_lexicon_tags_14 },
    { 0xf0888442, SUC_lexicon_tags_8 },
    { 0x84c88449, SUC_lexicon_tags_37 },
    { 0x9b08844a, SUC_lexicon_tags_5 },
    { 0x91e8844b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8888456, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da88459, SUC_lexicon_tags_2 },
    { 0xf248845a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc8845e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c88462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6688464, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2a688466, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88469, SUC_lexicon_tags_28 },
    { 0x77e8846a, SUC_lexicon_tags_10 },
    { 0x3348846a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a48846e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77288470, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc88474, SUC_lexicon_tags_10 },
    { 0x9da88475, SUC_lexicon_tags_7 },
    { 0x6f888476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e88479, SUC_lexicon_tags_37 },
    { 0xc088479, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc8847e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbaa88480, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15288487, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e8848b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28848e, SUC_lexicon_tags_12 },
    { 0x1768848f, SUC_lexicon_tags_15 },
    { 0x34488490, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35488494, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b288497, SUC_lexicon_tags_129 },
    { 0xfea88497, SUC_lexicon_tags_17 },
    { 0xabc88498, SUC_lexicon_tags_37 },
    { 0xf828849a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7768849f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06884a4, SUC_lexicon_tags_33 },
    { 0x5ac884a4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e884a8, SUC_lexicon_tags_26 },
    { 0xc5c884a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac884ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e884b0, SUC_lexicon_tags_1 },
    { 0xcae884b1, SUC_lexicon_tags_5 },
    { 0x41e884b2, SUC_lexicon_tags_63 },
    { 0x938884b3, SUC_lexicon_tags_1 },
    { 0x67c884b1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3c0884b6, SUC_lexicon_tags_25 },
    { 0x6c0884b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x994884b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x784884bb, SUC_lexicon_tags_49 },
    { 0x81a884bb, SUC_lexicon_tags_171 },
    { 0x702884bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbce884bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x362884c2, SUC_lexicon_tags_37 },
    { 0xd64884c3, SUC_lexicon_tags_37 },
    { 0x1b8884c4, SUC_lexicon_tags_11 },
    { 0x4884c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a884c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x228884cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc884d7, SUC_lexicon_tags_0 },
    { 0x888884d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4884df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb00884e1, SUC_lexicon_tags_6 },
    { 0x4f6884e1, SUC_lexicon_tags_15 },
    { 0x26a884e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20884e8, SUC_lexicon_tags_77 },
    { 0xe00884e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ca884eb, SUC_lexicon_tags_11 },
    { 0x16884ec, SUC_lexicon_tags_1 },
    { 0xdf0884eb, SUC_lexicon_tags_61 },
    { 0x8c4884ee, SUC_lexicon_tags_43 },
    { 0x672884ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72e884f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x484884f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce884f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c884fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7088502, SUC_lexicon_tags_17 },
    { 0x21088503, SUC_lexicon_tags_2 },
    { 0x4d088504, SUC_lexicon_tags_1 },
    { 0x2f288504, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe3888507, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a8850a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x17e8850c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94c8850e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xba088510, SUC_lexicon_tags_28 },
    { 0x70688511, SUC_lexicon_tags_9 },
    { 0x1d288510, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd2c88514, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x64288517, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84488520, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac488523, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb488527, SUC_lexicon_tags_6 },
    { 0x6dc88528, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c8852b, SUC_lexicon_tags_1 },
    { 0xd408852c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdc8852e, SUC_lexicon_tags_456 },
    { 0, NULL },
    { 0, NULL },
    { 0x4088531, SUC_lexicon_tags_31 },
    { 0x1a688532, SUC_lexicon_tags_2 },
    { 0x72088532, SUC_lexicon_tags_32 },
    { 0xdea88533, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed488539, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2888541, SUC_lexicon_tags_106 },
    { 0x9ec88542, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc888545, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x48888547, SUC_lexicon_tags_49 },
    { 0x5088547, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd08854a, SUC_lexicon_tags_16 },
    { 0x4b28854a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda68854d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c88550, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f488552, SUC_lexicon_tags_11 },
    { 0xc1a88553, SUC_lexicon_tags_2 },
    { 0xab288552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27c88556, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47a88558, SUC_lexicon_tags_11 },
    { 0xa488559, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f88855f, SUC_lexicon_tags_45 },
    { 0xf1e88560, SUC_lexicon_tags_31 },
    { 0x67688560, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4308856a, SUC_lexicon_tags_2 },
    { 0xefa8856b, SUC_lexicon_tags_6 },
    { 0x9188856c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaac8856e, SUC_lexicon_tags_15 },
    { 0x2ea8856f, SUC_lexicon_tags_25 },
    { 0x8ee88570, SUC_lexicon_tags_1 },
    { 0xe088571, SUC_lexicon_tags_37 },
    { 0x9c088572, SUC_lexicon_tags_37 },
    { 0x98288572, SUC_lexicon_tags_6 },
    { 0xfb288574, SUC_lexicon_tags_7 },
    { 0xfd688575, SUC_lexicon_tags_37 },
    { 0xea088571, SUC_lexicon_tags_33 },
    { 0xae888577, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5be88579, SUC_lexicon_tags_42 },
    { 0xe1a8857a, SUC_lexicon_tags_79 },
    { 0x7928857b, SUC_lexicon_tags_23 },
    { 0x2008857b, SUC_lexicon_tags_61 },
    { 0x9c08857d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f888580, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0488583, SUC_lexicon_tags_17 },
    { 0x4ca88584, SUC_lexicon_tags_1 },
    { 0x3be88585, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae88588, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e8858f, SUC_lexicon_tags_17 },
    { 0x51c88590, SUC_lexicon_tags_2 },
    { 0xf7088591, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60088597, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a8859b, SUC_lexicon_tags_70 },
    { 0xd428859b, SUC_lexicon_tags_14 },
    { 0x4088859b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4108859f, SUC_lexicon_tags_25 },
    { 0x5c6885a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2885a7, SUC_lexicon_tags_0 },
    { 0x2a6885a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950885ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830885b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0885b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4885bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc885bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c885c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0e885c5, SUC_lexicon_tags_66 },
    { 0xb2885c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92885cb, SUC_lexicon_tags_8 },
    { 0xfe885cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1f8885ce, SUC_lexicon_tags_47 },
    { 0xe50885cf, SUC_lexicon_tags_37 },
    { 0xa06885d0, SUC_lexicon_tags_37 },
    { 0x52a885d0, SUC_lexicon_tags_65 },
    { 0xd12885cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c885da, SUC_lexicon_tags_2 },
    { 0x7c6885db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x460885dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde4885e0, SUC_lexicon_tags_14 },
    { 0xecc885e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444885e5, SUC_lexicon_tags_7 },
    { 0xbe6885e6, SUC_lexicon_tags_9 },
    { 0x8fa885e7, SUC_lexicon_tags_31 },
    { 0xfda885e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8885ed, SUC_lexicon_tags_1 },
    { 0x14e885ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48885f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e885f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e885ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a88606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fa88608, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb48860b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe88613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda888615, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e288621, SUC_lexicon_tags_43 },
    { 0xd9288622, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63088629, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e8862d, SUC_lexicon_tags_41 },
    { 0x7128862e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d488631, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x71088634, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e8863b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e88640, SUC_lexicon_tags_19 },
    { 0xff088641, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a8864a, SUC_lexicon_tags_1 },
    { 0xb668864b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35288654, SUC_lexicon_tags_17 },
    { 0x8c488655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa948865b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46a8865d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x45288660, SUC_lexicon_tags_1 },
    { 0x61088661, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39888663, SUC_lexicon_tags_17 },
    { 0x44088664, SUC_lexicon_tags_37 },
    { 0x27c88664, SUC_lexicon_tags_27 },
    { 0x24488666, SUC_lexicon_tags_11 },
    { 0x23c88666, SUC_lexicon_tags_29 },
    { 0x76288666, SUC_lexicon_tags_696 },
    { 0x85a88668, SUC_lexicon_tags_17 },
    { 0x8408866a, SUC_lexicon_tags_0 },
    { 0xf428866b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb568866d, SUC_lexicon_tags_65 },
    { 0xeb28866e, SUC_lexicon_tags_77 },
    { 0x38a8866f, SUC_lexicon_tags_1 },
    { 0xc488670, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9688679, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x9e08867b, SUC_lexicon_tags_2 },
    { 0x9508867c, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0x8e48867e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8a088680, SUC_lexicon_tags_21 },
    { 0x95688680, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a288684, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20088686, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a88689, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x70c8868b, SUC_lexicon_tags_12 },
    { 0xe4a8868c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa468868e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xb5c88690, SUC_lexicon_tags_16 },
    { 0x22a88691, SUC_lexicon_tags_32 },
    { 0x5ea88692, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda288695, SUC_lexicon_tags_12 },
    { 0x3e288695, SUC_lexicon_tags_43 },
    { 0xc2488697, SUC_lexicon_tags_17 },
    { 0x5ae88698, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef88869d, SUC_lexicon_tags_221 },
    { 0xc488869d, SUC_lexicon_tags_0 },
    { 0x23e8869e, SUC_lexicon_tags_65 },
    { 0x4e2886a0, SUC_lexicon_tags_87 },
    { 0xa748869f, SUC_lexicon_tags_12 },
    { 0x308886a2, SUC_lexicon_tags_23 },
    { 0x378886a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee886a6, SUC_lexicon_tags_103 },
    { 0x9d8886a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1a886a9, SUC_lexicon_tags_47 },
    { 0x800886aa, SUC_lexicon_tags_27 },
    { 0x71a886ab, SUC_lexicon_tags_160 },
    { 0x9ce886ac, SUC_lexicon_tags_12 },
    { 0x936886ab, SUC_lexicon_tags_15 },
    { 0x5d8886ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x61a886b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x278886b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x674886b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2e886b7, SUC_lexicon_tags_43 },
    { 0x4a4886b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde886bc, SUC_lexicon_tags_49 },
    { 0xa24886bd, SUC_lexicon_tags_38 },
    { 0xbf2886be, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc3c886c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2886c3, SUC_lexicon_tags_61 },
    { 0x3b2886c4, SUC_lexicon_tags_37 },
    { 0x17e886c5, SUC_lexicon_tags_11 },
    { 0xe3e886c6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9f2886c8, SUC_lexicon_tags_7 },
    { 0x4d4886c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc886cc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944886d4, SUC_lexicon_tags_0 },
    { 0xe2886d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36886dc, SUC_lexicon_tags_17 },
    { 0x13c886dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0886e7, SUC_lexicon_tags_28 },
    { 0xa82886e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x722886eb, SUC_lexicon_tags_7 },
    { 0xcf8886ec, SUC_lexicon_tags_0 },
    { 0xc9a886ed, SUC_lexicon_tags_11 },
    { 0xe9c886ed, SUC_lexicon_tags_0 },
    { 0xa20886ec, SUC_lexicon_tags_37 },
    { 0x8d4886f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16886f3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe886f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c2886fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e288703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe88709, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5488711, SUC_lexicon_tags_10 },
    { 0x9c288712, SUC_lexicon_tags_11 },
    { 0x1c88713, SUC_lexicon_tags_2 },
    { 0xd3c88713, SUC_lexicon_tags_46 },
    { 0xaac88715, SUC_lexicon_tags_34 },
    { 0xa5a88712, SUC_lexicon_tags_250 },
    { 0x5de88713, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a8871f, SUC_lexicon_tags_37 },
    { 0x5d088720, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60088723, SUC_lexicon_tags_32 },
    { 0x90a88724, SUC_lexicon_tags_60 },
    { 0xb6c88725, SUC_lexicon_tags_12 },
    { 0x3488724, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a888728, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e88872a, SUC_lexicon_tags_69 },
    { 0x4908872b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x308872e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30888733, SUC_lexicon_tags_15 },
    { 0x28488734, SUC_lexicon_tags_37 },
    { 0x39c88735, SUC_lexicon_tags_12 },
    { 0xa0e88734, SUC_lexicon_tags_27 },
    { 0xad088737, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c8873a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf788873c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa488873e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83488742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa8874a, SUC_lexicon_tags_25 },
    { 0xb0a8874b, SUC_lexicon_tags_12 },
    { 0xc368874c, SUC_lexicon_tags_47 },
    { 0xb3e8874d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14888754, SUC_lexicon_tags_25 },
    { 0x54288755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3688757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50a88759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d888760, SUC_lexicon_tags_28 },
    { 0x8cc88761, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70c88763, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x50c88765, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d488769, SUC_lexicon_tags_43 },
    { 0xa628876a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc708876f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a88772, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c288776, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b08877a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92e8877c, SUC_lexicon_tags_16 },
    { 0x5588877d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c88780, SUC_lexicon_tags_27 },
    { 0x46a88780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1688784, SUC_lexicon_tags_49 },
    { 0x12e88785, SUC_lexicon_tags_45 },
    { 0xb3e88786, SUC_lexicon_tags_2 },
    { 0x73a88784, SUC_lexicon_tags_17 },
    { 0xbcc88788, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28e8878a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d48878d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe8e8878f, SUC_lexicon_tags_7 },
    { 0xa9088790, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e488795, SUC_lexicon_tags_8 },
    { 0x5d288795, SUC_lexicon_tags_25 },
    { 0xb688796, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3388879a, SUC_lexicon_tags_64 },
    { 0xe0c8879b, SUC_lexicon_tags_81 },
    { 0x3d28879c, SUC_lexicon_tags_66 },
    { 0xb148879d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c887a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa887a8, SUC_lexicon_tags_15 },
    { 0x640887a9, SUC_lexicon_tags_12 },
    { 0xa1c887a9, SUC_lexicon_tags_25 },
    { 0xb0887ab, SUC_lexicon_tags_1 },
    { 0x7e887ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590887b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8c0887b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890887bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4c887bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f4887bf, SUC_lexicon_tags_27 },
    { 0x264887c0, SUC_lexicon_tags_37 },
    { 0x73c887c1, SUC_lexicon_tags_12 },
    { 0x5d6887c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee887c7, SUC_lexicon_tags_27 },
    { 0xbb4887c8, SUC_lexicon_tags_1 },
    { 0xa0e887c9, SUC_lexicon_tags_27 },
    { 0x96887ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae887cd, SUC_lexicon_tags_12 },
    { 0x564887ce, SUC_lexicon_tags_25 },
    { 0xec887cf, SUC_lexicon_tags_17 },
    { 0xd4887d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2da887d2, SUC_lexicon_tags_11 },
    { 0xd92887d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a887d8, SUC_lexicon_tags_15 },
    { 0x21e887d9, SUC_lexicon_tags_15 },
    { 0xa18887d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390887de, SUC_lexicon_tags_31 },
    { 0xbc6887df, SUC_lexicon_tags_0 },
    { 0x1f0887e0, SUC_lexicon_tags_97 },
    { 0x484887e1, SUC_lexicon_tags_37 },
    { 0xbba887de, SUC_lexicon_tags_27 },
    { 0x3c6887e3, SUC_lexicon_tags_29 },
    { 0x70c887e4, SUC_lexicon_tags_47 },
    { 0xeb6887e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2887e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x206887eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc887ee, SUC_lexicon_tags_11 },
    { 0xdae887ef, SUC_lexicon_tags_1 },
    { 0xa50887f0, SUC_lexicon_tags_27 },
    { 0xa80887f0, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa887f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288887fe, SUC_lexicon_tags_9 },
    { 0xbc8887ff, SUC_lexicon_tags_17 },
    { 0x9e088800, SUC_lexicon_tags_11 },
    { 0x528887fe, SUC_lexicon_tags_11 },
    { 0x6888801, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce88805, SUC_lexicon_tags_37 },
    { 0x4c088806, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5188880c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa28880e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fa88810, SUC_lexicon_tags_29 },
    { 0x6ea88811, SUC_lexicon_tags_15 },
    { 0x99c88812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e88816, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a8881b, SUC_lexicon_tags_2 },
    { 0x1248881b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee88822, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11088826, SUC_lexicon_tags_25 },
    { 0x2c088826, SUC_lexicon_tags_14 },
    { 0x7de88828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf408882c, SUC_lexicon_tags_25 },
    { 0x2c68882d, SUC_lexicon_tags_14 },
    { 0xa9a8882d, SUC_lexicon_tags_26 },
    { 0x6a08882f, SUC_lexicon_tags_14 },
    { 0x5468882f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a88833, SUC_lexicon_tags_17 },
    { 0xb1488834, SUC_lexicon_tags_11 },
    { 0xf3a88835, SUC_lexicon_tags_15 },
    { 0xf4888833, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e8883a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8088840, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e8884b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37288852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52888854, SUC_lexicon_tags_9 },
    { 0xc1688855, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8885c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf488860, SUC_lexicon_tags_43 },
    { 0xa2288861, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a88864, SUC_lexicon_tags_70 },
    { 0xcde88864, SUC_lexicon_tags_14 },
    { 0xde688865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c888869, SUC_lexicon_tags_57 },
    { 0x2bc88869, SUC_lexicon_tags_43 },
    { 0x40a8886b, SUC_lexicon_tags_43 },
    { 0xc5688869, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e88871, SUC_lexicon_tags_2 },
    { 0x8e088872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2688875, SUC_lexicon_tags_1 },
    { 0x2c488875, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb4088878, SUC_lexicon_tags_25 },
    { 0x88688879, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6668887d, SUC_lexicon_tags_29 },
    { 0x80a8887e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec88882, SUC_lexicon_tags_17 },
    { 0x3e888883, SUC_lexicon_tags_48 },
    { 0x29e88884, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a8888c, SUC_lexicon_tags_15 },
    { 0x5ac8888d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a088890, SUC_lexicon_tags_9 },
    { 0x7be88890, SUC_lexicon_tags_21 },
    { 0xffc88891, SUC_lexicon_tags_15 },
    { 0x6c688891, SUC_lexicon_tags_0 },
    { 0xda888893, SUC_lexicon_tags_11 },
    { 0xa6488895, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x52888897, SUC_lexicon_tags_23 },
    { 0x8488897, SUC_lexicon_tags_46 },
    { 0x76488899, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2188889c, SUC_lexicon_tags_21 },
    { 0x7288889d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe888a0, SUC_lexicon_tags_0 },
    { 0x54e888a1, SUC_lexicon_tags_43 },
    { 0xbe2888a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e888a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1f4888aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e888ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc888b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x95e888b5, SUC_lexicon_tags_4 },
    { 0xdf4888b6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6888bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32888c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x388888c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16888cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63c888cd, SUC_lexicon_tags_16 },
    { 0x6fa888ce, SUC_lexicon_tags_9 },
    { 0xd2888cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684888da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4888dd, SUC_lexicon_tags_17 },
    { 0xbe6888dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8cc888e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa888e4, SUC_lexicon_tags_7 },
    { 0x538888e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58a888e7, SUC_lexicon_tags_33 },
    { 0x160888e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4888ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43e888ee, SUC_lexicon_tags_16 },
    { 0x7bc888ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2888f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0888f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2ea888f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf90888fb, SUC_lexicon_tags_37 },
    { 0x3fc888fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8888ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb488904, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac488906, SUC_lexicon_tags_1 },
    { 0xa5088907, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8108890a, SUC_lexicon_tags_92 },
    { 0xf528890b, SUC_lexicon_tags_7 },
    { 0x3228890b, SUC_lexicon_tags_11 },
    { 0x59c8890d, SUC_lexicon_tags_31 },
    { 0x4348890d, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x3f888910, SUC_lexicon_tags_2 },
    { 0x9ac88910, SUC_lexicon_tags_0 },
    { 0x86c88911, SUC_lexicon_tags_0 },
    { 0x2be88911, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99e88915, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d688917, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e88891b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c8891f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x83888921, SUC_lexicon_tags_19 },
    { 0x9c088921, SUC_lexicon_tags_12 },
    { 0x15c88923, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a88926, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8648892a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9768892c, SUC_lexicon_tags_7 },
    { 0x6ce8892c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca28892f, SUC_lexicon_tags_37 },
    { 0xefc8892f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5888933, SUC_lexicon_tags_7 },
    { 0xff688934, SUC_lexicon_tags_7 },
    { 0x61288935, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6da88937, SUC_lexicon_tags_12 },
    { 0x93088937, SUC_lexicon_tags_66 },
    { 0xe9688939, SUC_lexicon_tags_48 },
    { 0x93e88939, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb808893e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd088941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9c88943, SUC_lexicon_tags_65 },
    { 0x7b288943, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3348894d, SUC_lexicon_tags_12 },
    { 0x27e8894d, SUC_lexicon_tags_26 },
    { 0x3c88894f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c088951, SUC_lexicon_tags_25 },
    { 0x50e88951, SUC_lexicon_tags_21 },
    { 0x2b288952, SUC_lexicon_tags_25 },
    { 0xb1088954, SUC_lexicon_tags_25 },
    { 0xca88954, SUC_lexicon_tags_47 },
    { 0x9e288954, SUC_lexicon_tags_17 },
    { 0x8288951, SUC_lexicon_tags_31 },
    { 0x27288953, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa08895e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d288961, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa8896c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e08896e, SUC_lexicon_tags_47 },
    { 0x5928896f, SUC_lexicon_tags_743 },
    { 0x9228896e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2888972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec88978, SUC_lexicon_tags_3 },
    { 0x59688979, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e08897c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc008897f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e088985, SUC_lexicon_tags_11 },
    { 0x43c88986, SUC_lexicon_tags_37 },
    { 0x80e88987, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97a88989, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c8898e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c888991, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb688993, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d688998, SUC_lexicon_tags_46 },
    { 0x90088999, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e8899c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1588899f, SUC_lexicon_tags_7 },
    { 0x2c2889a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0889a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec889a6, SUC_lexicon_tags_48 },
    { 0x752889a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e889ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee889ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbc889b0, SUC_lexicon_tags_32 },
    { 0x7ae889b1, SUC_lexicon_tags_30 },
    { 0x9a2889b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab8889b4, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e889b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c889bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e889c0, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8889c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe4889c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f6889c8, SUC_lexicon_tags_47 },
    { 0xe9a889c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c889cd, SUC_lexicon_tags_1062 },
    { 0x70889cd, SUC_lexicon_tags_7 },
    { 0xd7a889ce, SUC_lexicon_tags_31 },
    { 0x58889cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8889d4, SUC_lexicon_tags_25 },
    { 0x47e889d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1cc889d7, SUC_lexicon_tags_37 },
    { 0xc9e889d8, SUC_lexicon_tags_66 },
    { 0xebc889d9, SUC_lexicon_tags_96 },
    { 0x610889da, SUC_lexicon_tags_160 },
    { 0xaaa889da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x378889de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e889e1, SUC_lexicon_tags_23 },
    { 0x9a0889e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8889e5, SUC_lexicon_tags_27 },
    { 0x286889e5, SUC_lexicon_tags_12 },
    { 0x4ca889e7, SUC_lexicon_tags_21 },
    { 0x8de889e8, SUC_lexicon_tags_25 },
    { 0x6a4889e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae889ef, SUC_lexicon_tags_33 },
    { 0x460889f0, SUC_lexicon_tags_43 },
    { 0x718889f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a889f3, SUC_lexicon_tags_48 },
    { 0xdac889f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2889f7, SUC_lexicon_tags_47 },
    { 0x10e889f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86889fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc88a03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17488a05, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f088a07, SUC_lexicon_tags_8 },
    { 0x68688a07, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81688a11, SUC_lexicon_tags_32 },
    { 0xde88a12, SUC_lexicon_tags_8 },
    { 0xd2488a13, SUC_lexicon_tags_65 },
    { 0x5fe88a14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2288a17, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa6c88a19, SUC_lexicon_tags_9 },
    { 0xbde88a1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a88a24, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb2e88a26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8e88a28, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x50e88a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca88a34, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8b688a36, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c88a3c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7088a3f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c88a44, SUC_lexicon_tags_26 },
    { 0x6a288a45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34288a4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a888a52, SUC_lexicon_tags_8 },
    { 0x19888a53, SUC_lexicon_tags_66 },
    { 0x58a88a53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5e88a56, SUC_lexicon_tags_28 },
    { 0xe5088a56, SUC_lexicon_tags_87 },
    { 0x34888a58, SUC_lexicon_tags_162 },
    { 0xf9488a58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9688a5b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbb688a5d, SUC_lexicon_tags_1 },
    { 0xb2a88a5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0088a60, SUC_lexicon_tags_17 },
    { 0xdfc88a60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea88a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44088a69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e88a6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd688a6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x62c88a71, SUC_lexicon_tags_17 },
    { 0x8aa88a72, SUC_lexicon_tags_63 },
    { 0x36288a73, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11488a78, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f088a7b, SUC_lexicon_tags_28 },
    { 0x7c688a7c, SUC_lexicon_tags_15 },
    { 0x78c88a7d, SUC_lexicon_tags_14 },
    { 0x45c88a7b, SUC_lexicon_tags_17 },
    { 0x90e88a7f, SUC_lexicon_tags_39 },
    { 0xc7a88a80, SUC_lexicon_tags_66 },
    { 0xb0e88a81, SUC_lexicon_tags_33 },
    { 0x1ce88a7e, SUC_lexicon_tags_15 },
    { 0x93088a83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ee88a85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x48888a88, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3aa88a8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c88a8d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e88a90, SUC_lexicon_tags_16 },
    { 0x4aa88a90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0e88a93, SUC_lexicon_tags_2 },
    { 0x57688a94, SUC_lexicon_tags_23 },
    { 0x14488a95, SUC_lexicon_tags_25 },
    { 0xffc88a95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a88a9a, SUC_lexicon_tags_46 },
    { 0x66088a9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63488a9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcea88a9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70288aa4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x92088aa6, SUC_lexicon_tags_17 },
    { 0x67e88aa6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe288aa9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe6088aab, SUC_lexicon_tags_7 },
    { 0xc9288aac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa88aaf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6d488ab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f488ab8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde88abe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac88ac8, SUC_lexicon_tags_63 },
    { 0x1e088ac8, SUC_lexicon_tags_40 },
    { 0x7c688ac9, SUC_lexicon_tags_12 },
    { 0xe2888aca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13688acf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1488adf, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0xede88ae2, SUC_lexicon_tags_14 },
    { 0x40888ae2, SUC_lexicon_tags_15 },
    { 0x8f288ae4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0688ae7, SUC_lexicon_tags_23 },
    { 0x81a88ae8, SUC_lexicon_tags_31 },
    { 0xb5c88ae8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa88aec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4a88aee, SUC_lexicon_tags_45 },
    { 0x37288aee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3488af3, SUC_lexicon_tags_27 },
    { 0xc1488af4, SUC_lexicon_tags_37 },
    { 0xc8c88af5, SUC_lexicon_tags_47 },
    { 0x8088af5, SUC_lexicon_tags_27 },
    { 0x37888af7, SUC_lexicon_tags_15 },
    { 0xe9688af4, SUC_lexicon_tags_6 },
    { 0xd0088af3, SUC_lexicon_tags_17 },
    { 0x4b288af8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x82688afc, SUC_lexicon_tags_25 },
    { 0x9e888afd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3888b00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe6c88b02, SUC_lexicon_tags_62 },
    { 0x88a88b02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5488b08, SUC_lexicon_tags_31 },
    { 0x3d488b09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c88b0d, SUC_lexicon_tags_15 },
    { 0x10e88b0e, SUC_lexicon_tags_6 },
    { 0xdec88b0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea488b12, SUC_lexicon_tags_15 },
    { 0x46488b13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca288b15, SUC_lexicon_tags_1 },
    { 0xafa88b16, SUC_lexicon_tags_33 },
    { 0x47a88b17, SUC_lexicon_tags_12 },
    { 0x79488b18, SUC_lexicon_tags_55 },
    { 0xc3e88b19, SUC_lexicon_tags_9 },
    { 0x68088b19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33688b1c, SUC_lexicon_tags_37 },
    { 0x1088b1d, SUC_lexicon_tags_9 },
    { 0x33088b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e88b28, SUC_lexicon_tags_37 },
    { 0xdba88b29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5088b2b, SUC_lexicon_tags_2 },
    { 0x15888b2b, SUC_lexicon_tags_29 },
    { 0xc7288b2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d888b32, SUC_lexicon_tags_28 },
    { 0x30a88b33, SUC_lexicon_tags_15 },
    { 0xfd088b33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73288b36, SUC_lexicon_tags_33 },
    { 0x73488b37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc8688b39, SUC_lexicon_tags_1 },
    { 0x46888b3a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e88b3d, SUC_lexicon_tags_23 },
    { 0x65288b3e, SUC_lexicon_tags_15 },
    { 0xdb088b3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41088b42, SUC_lexicon_tags_2 },
    { 0xe88b43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5288b48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea88b4b, SUC_lexicon_tags_82 },
    { 0x7288b4b, SUC_lexicon_tags_87 },
    { 0xbde88b4d, SUC_lexicon_tags_68 },
    { 0x97a88b4d, SUC_lexicon_tags_60 },
    { 0x12488b4e, SUC_lexicon_tags_19 },
    { 0x8f888b50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x31488b53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d488b55, SUC_lexicon_tags_7 },
    { 0x5a088b56, SUC_lexicon_tags_45 },
    { 0x54a88b57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e88b5a, SUC_lexicon_tags_33 },
    { 0x9a088b5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x88088b5d, SUC_lexicon_tags_10 },
    { 0x60088b5d, SUC_lexicon_tags_0 },
    { 0xba488b5f, SUC_lexicon_tags_38 },
    { 0xc9088b60, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8888b65, SUC_lexicon_tags_12 },
    { 0xac88b65, SUC_lexicon_tags_3 },
    { 0x91688b66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x84288b69, SUC_lexicon_tags_25 },
    { 0x37288b6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25688b6c, SUC_lexicon_tags_14 },
    { 0xa8088b6d, SUC_lexicon_tags_7 },
    { 0x6b488b6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0688b71, SUC_lexicon_tags_31 },
    { 0xa6a88b71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38088b74, SUC_lexicon_tags_12 },
    { 0x61288b74, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e088b79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53888b7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf5c88b7d, SUC_lexicon_tags_1 },
    { 0x3ac88b7e, SUC_lexicon_tags_84 },
    { 0xa2488b7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c88b82, SUC_lexicon_tags_26 },
    { 0xcd488b82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c88b86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c288b8e, SUC_lexicon_tags_68 },
    { 0xc9a88b8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd2a88b91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0288b94, SUC_lexicon_tags_35 },
    { 0xe2088b95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38288b97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c88b9a, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xf488b9d, SUC_lexicon_tags_1 },
    { 0x85688b9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c88ba3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa9688ba5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5088bab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x32688bae, SUC_lexicon_tags_2 },
    { 0x72888baf, SUC_lexicon_tags_27 },
    { 0x9b288bb0, SUC_lexicon_tags_17 },
    { 0xb1888baf, SUC_lexicon_tags_19 },
    { 0x2be88bae, SUC_lexicon_tags_47 },
    { 0x5b688bb0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7a88bb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce88bb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3688bbb, SUC_lexicon_tags_27 },
    { 0x9ea88bbb, SUC_lexicon_tags_43 },
    { 0xfc88bbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7e88bbf, SUC_lexicon_tags_18 },
    { 0xe3088bc0, SUC_lexicon_tags_37 },
    { 0x8b888bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85888bc3, SUC_lexicon_tags_12 },
    { 0xe5888bc3, SUC_lexicon_tags_23 },
    { 0xddc88bc4, SUC_lexicon_tags_7 },
    { 0xd1288bc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ba88bc8, SUC_lexicon_tags_33 },
    { 0x12a88bc9, SUC_lexicon_tags_0 },
    { 0x32c88bc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d688bcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6488bd1, SUC_lexicon_tags_15 },
    { 0x10488bd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c288bd4, SUC_lexicon_tags_17 },
    { 0x9f888bd4, SUC_lexicon_tags_27 },
    { 0xeae88bd6, SUC_lexicon_tags_16 },
    { 0x29088bd6, SUC_lexicon_tags_15 },
    { 0xc1c88bd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb288bdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e888be1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcc88be3, SUC_lexicon_tags_27 },
    { 0x4ac88be4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aa88be6, SUC_lexicon_tags_11 },
    { 0x12a88be7, SUC_lexicon_tags_77 },
    { 0xffe88be7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3088beb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55a88bed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab288bef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac88bf6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a88bfa, SUC_lexicon_tags_38 },
    { 0xdc88bfb, SUC_lexicon_tags_25 },
    { 0x9a688bfb, SUC_lexicon_tags_33 },
    { 0xee088bfd, SUC_lexicon_tags_47 },
    { 0x36288bfa, SUC_lexicon_tags_29 },
    { 0x99888bff, SUC_lexicon_tags_12 },
    { 0xa0888bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49e88c02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bc88c04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5488c07, SUC_lexicon_tags_27 },
    { 0x94a88c08, SUC_lexicon_tags_12 },
    { 0x18288c09, SUC_lexicon_tags_61 },
    { 0x29e88c07, SUC_lexicon_tags_12 },
    { 0x36088c0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6688c0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1c88c11, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c88c14, SUC_lexicon_tags_15 },
    { 0x89288c14, SUC_lexicon_tags_2 },
    { 0x96e88c16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcbe88c18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23c88c1a, SUC_lexicon_tags_12 },
    { 0x9b888c1b, SUC_lexicon_tags_46 },
    { 0xc6a88c1c, SUC_lexicon_tags_15 },
    { 0x3c288c1d, SUC_lexicon_tags_11 },
    { 0x31c88c1e, SUC_lexicon_tags_31 },
    { 0x79a88c1d, SUC_lexicon_tags_25 },
    { 0xe8c88c20, SUC_lexicon_tags_66 },
    { 0x59a88c21, SUC_lexicon_tags_38 },
    { 0xb6488c21, SUC_lexicon_tags_25 },
    { 0x7d488c23, SUC_lexicon_tags_15 },
    { 0xdae88c20, SUC_lexicon_tags_7 },
    { 0xfae88c25, SUC_lexicon_tags_17 },
    { 0x19888c26, SUC_lexicon_tags_1 },
    { 0x9688c27, SUC_lexicon_tags_9 },
    { 0x2f888c26, SUC_lexicon_tags_17 },
    { 0x39688c29, SUC_lexicon_tags_13 },
    { 0xc7288c1e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x16e88c2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffa88c2e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6ea88c30, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90888c35, SUC_lexicon_tags_23 },
    { 0xfd288c36, SUC_lexicon_tags_8 },
    { 0xa1488c37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xca488c39, SUC_lexicon_tags_15 },
    { 0xe4688c39, SUC_lexicon_tags_17 },
    { 0xfb688c39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1288c3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c688c41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x45688c44, SUC_lexicon_tags_0 },
    { 0x8a488c44, SUC_lexicon_tags_49 },
    { 0x97688c46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85888c49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17688c4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1fe88c4f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb3c88c51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75888c53, SUC_lexicon_tags_12 },
    { 0xa5088c53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc88c56, SUC_lexicon_tags_34 },
    { 0x8d688c57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89e88c59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ce88c5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1488c5f, SUC_lexicon_tags_28 },
    { 0x74488c60, SUC_lexicon_tags_27 },
    { 0xc8488c60, SUC_lexicon_tags_1 },
    { 0x5d888c60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xafc88c64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac88c68, SUC_lexicon_tags_25 },
    { 0xa0488c69, SUC_lexicon_tags_11 },
    { 0x6cc88c6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2288c6d, SUC_lexicon_tags_49 },
    { 0x76088c6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9888c71, SUC_lexicon_tags_25 },
    { 0xd2c88c72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94688c79, SUC_lexicon_tags_331 },
    { 0xcd488c79, SUC_lexicon_tags_46 },
    { 0xf4688c7b, SUC_lexicon_tags_9 },
    { 0xf0488c7c, SUC_lexicon_tags_104 },
    { 0x62a88c7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc288c7f, SUC_lexicon_tags_38 },
    { 0x63a88c80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c88c90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9688c93, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xed488c96, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc88c9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e88ca1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16488ca6, SUC_lexicon_tags_47 },
    { 0xde488ca6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x49288ca9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66088cae, SUC_lexicon_tags_11 },
    { 0x2f688caf, SUC_lexicon_tags_37 },
    { 0x92888cb0, SUC_lexicon_tags_14 },
    { 0x58688cb1, SUC_lexicon_tags_10 },
    { 0xbb288cb2, SUC_lexicon_tags_12 },
    { 0xab288cb3, SUC_lexicon_tags_49 },
    { 0x39a88cb4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b888cb7, SUC_lexicon_tags_23 },
    { 0x16888cb7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c88cbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6fc88cbe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c288cc3, SUC_lexicon_tags_15 },
    { 0xb5288cc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c288cc6, SUC_lexicon_tags_12 },
    { 0x5ec88cc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c88cca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a88cce, SUC_lexicon_tags_60 },
    { 0xe7088cce, SUC_lexicon_tags_19 },
    { 0xebe88cd0, SUC_lexicon_tags_23 },
    { 0x6f688cd1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7a688cd3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe9e88cd5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee88cd8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e488ce0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9de88ce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae88ce9, SUC_lexicon_tags_21 },
    { 0xd5088cea, SUC_lexicon_tags_15 },
    { 0xf6e88ceb, SUC_lexicon_tags_37 },
    { 0x64888ce9, SUC_lexicon_tags_17 },
    { 0xf9088ced, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x57488cef, SUC_lexicon_tags_45 },
    { 0xdae88cf0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1488cf6, SUC_lexicon_tags_61 },
    { 0x9c088cf6, SUC_lexicon_tags_2 },
    { 0xf2c88cf8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc88cfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29888d00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81288d02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1688d08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31088d12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa688d15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca688d18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1688d1b, SUC_lexicon_tags_30 },
    { 0xc7488d1c, SUC_lexicon_tags_2 },
    { 0xd0888d1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d888d21, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x10088d24, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c88d2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb288d34, SUC_lexicon_tags_84 },
    { 0x76a88d35, SUC_lexicon_tags_120 },
    { 0x37288d36, SUC_lexicon_tags_87 },
    { 0xb288d37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e88d3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d288d40, SUC_lexicon_tags_1 },
    { 0x67288d41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2688d44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1c688d46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcee88d48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a88d4b, SUC_lexicon_tags_49 },
    { 0x3ec88d4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc88d58, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e88d5f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa9088d61, SUC_lexicon_tags_46 },
    { 0x9b888d62, SUC_lexicon_tags_37 },
    { 0xe1c88d63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe88d6a, SUC_lexicon_tags_66 },
    { 0x9e888d6a, SUC_lexicon_tags_7 },
    { 0x58a88d6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7288d6e, SUC_lexicon_tags_43 },
    { 0x93088d6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9088d72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5888d75, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xde288d77, SUC_lexicon_tags_68 },
    { 0xa3888d78, SUC_lexicon_tags_14 },
    { 0x44c88d77, SUC_lexicon_tags_1 },
    { 0x82e88d78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5488d7e, SUC_lexicon_tags_9 },
    { 0x8c88d7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea488d83, SUC_lexicon_tags_257 },
    { 0x73e88d84, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe88d89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a88d97, SUC_lexicon_tags_38 },
    { 0xc4088d97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2c488d9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a88d9d, SUC_lexicon_tags_15 },
    { 0xe0888d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa288da0, SUC_lexicon_tags_27 },
    { 0x56888da1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42488da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94288da7, SUC_lexicon_tags_25 },
    { 0x44488da8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2f888daa, SUC_lexicon_tags_17 },
    { 0xec088dab, SUC_lexicon_tags_37 },
    { 0x19a88daa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe9c88dae, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac88db2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee88db5, SUC_lexicon_tags_26 },
    { 0xcc088db5, SUC_lexicon_tags_12 },
    { 0xc2288db5, SUC_lexicon_tags_12 },
    { 0x80888db8, SUC_lexicon_tags_48 },
    { 0x40a88db8, SUC_lexicon_tags_15 },
    { 0xd4288dba, SUC_lexicon_tags_4 },
    { 0x71c88dbb, SUC_lexicon_tags_25 },
    { 0xc2c88db8, SUC_lexicon_tags_37 },
    { 0x30e88dbd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1888dc0, SUC_lexicon_tags_33 },
    { 0x8ec88dc1, SUC_lexicon_tags_25 },
    { 0x3e688dc2, SUC_lexicon_tags_2 },
    { 0xb1688dc1, SUC_lexicon_tags_9 },
    { 0x2be88dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18a88dc6, SUC_lexicon_tags_38 },
    { 0x99488dc6, SUC_lexicon_tags_82 },
    { 0x9c688dc8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd3888dca, SUC_lexicon_tags_11 },
    { 0xbe888dca, SUC_lexicon_tags_7 },
    { 0x60088dcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13288dcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19288dd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa88dd6, SUC_lexicon_tags_2 },
    { 0xa7888dd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1488dda, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc88dde, SUC_lexicon_tags_37 },
    { 0x69088ddf, SUC_lexicon_tags_4 },
    { 0xcd888de0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de88de3, SUC_lexicon_tags_16 },
    { 0x5b688de3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6688de6, SUC_lexicon_tags_87 },
    { 0x20288de7, SUC_lexicon_tags_31 },
    { 0xf1c88de8, SUC_lexicon_tags_32 },
    { 0xff488de8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a888dec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc488dee, SUC_lexicon_tags_7 },
    { 0xfa688def, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6688df4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e88df7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e88dfa, SUC_lexicon_tags_37 },
    { 0xe5288dfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa88dfd, SUC_lexicon_tags_25 },
    { 0xa2688dfe, SUC_lexicon_tags_15 },
    { 0xec288dfe, SUC_lexicon_tags_1 },
    { 0xc288e00, SUC_lexicon_tags_27 },
    { 0x9fc88e01, SUC_lexicon_tags_15 },
    { 0x8c888e02, SUC_lexicon_tags_17 },
    { 0x13c88e02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc488e09, SUC_lexicon_tags_25 },
    { 0xf0e88e0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9088e10, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x63288e13, SUC_lexicon_tags_7 },
    { 0xb7488e13, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xca488e16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83088e18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae88e1b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x72888e1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5888e21, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x86088e23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e488e26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0a88e28, SUC_lexicon_tags_43 },
    { 0x67e88e28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e88e31, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde088e37, SUC_lexicon_tags_57 },
    { 0xa4488e38, SUC_lexicon_tags_17 },
    { 0xf5688e37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde88e40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24288e42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0288e45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac88e4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62088e4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71088e52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e888e55, SUC_lexicon_tags_45 },
    { 0x4c288e55, SUC_lexicon_tags_46 },
    { 0x77a88e55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d088e59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10288e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea88e64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa488e67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488e6b, SUC_lexicon_tags_26 },
    { 0xb088e6b, SUC_lexicon_tags_77 },
    { 0xca888e6d, SUC_lexicon_tags_14 },
    { 0x62c88e6d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf2a88e70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc88e73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30688e75, SUC_lexicon_tags_25 },
    { 0x1c888e76, SUC_lexicon_tags_42 },
    { 0x95c88e76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40288e79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6088e7d, SUC_lexicon_tags_15 },
    { 0xf1088e7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b488e81, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2888e87, SUC_lexicon_tags_15 },
    { 0xf6a88e88, SUC_lexicon_tags_11 },
    { 0x1c288e88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c88e94, SUC_lexicon_tags_37 },
    { 0xec088e95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c288e99, SUC_lexicon_tags_37 },
    { 0x62e88e9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce88e9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c88ea4, SUC_lexicon_tags_17 },
    { 0x12688ea4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa88ea8, SUC_lexicon_tags_17 },
    { 0xe5c88ea8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d088eab, SUC_lexicon_tags_60 },
    { 0x41488eab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8088eb1, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0x52488eb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84088eb9, SUC_lexicon_tags_12 },
    { 0x8088eba, SUC_lexicon_tags_61 },
    { 0xfd888eb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91488ebd, SUC_lexicon_tags_15 },
    { 0xcfa88ebe, SUC_lexicon_tags_15 },
    { 0xf9688ebd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9088ec1, SUC_lexicon_tags_25 },
    { 0x73688ec1, SUC_lexicon_tags_37 },
    { 0x80e88ec3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84688ec9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e88ecd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee88ed2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbec88ed4, SUC_lexicon_tags_2 },
    { 0xd4088ed4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee88edb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c488edd, SUC_lexicon_tags_25 },
    { 0x17288edd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee88ee1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0288ee5, SUC_lexicon_tags_25 },
    { 0x63088ee6, SUC_lexicon_tags_12 },
    { 0x3d488ee7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb688eea, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7088ef0, SUC_lexicon_tags_17 },
    { 0xa3c88ef1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96088ef5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e88efa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1e88efc, SUC_lexicon_tags_56 },
    { 0x9c088efd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4dc88eff, SUC_lexicon_tags_25 },
    { 0xa7288f00, SUC_lexicon_tags_29 },
    { 0x7d288f00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a688f09, SUC_lexicon_tags_9 },
    { 0x8ba88f0a, SUC_lexicon_tags_25 },
    { 0x18a88f0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x41e88f0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc88f11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe88f18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87688f29, SUC_lexicon_tags_2 },
    { 0xd2a88f2a, SUC_lexicon_tags_48 },
    { 0x51c88f2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac88f2f, SUC_lexicon_tags_14 },
    { 0x45888f30, SUC_lexicon_tags_23 },
    { 0x16888f30, SUC_lexicon_tags_0 },
    { 0x43c88f2f, SUC_lexicon_tags_2 },
    { 0x44288f33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9a88f35, SUC_lexicon_tags_9 },
    { 0x50888f36, SUC_lexicon_tags_38 },
    { 0x7fa88f35, SUC_lexicon_tags_2 },
    { 0x4d488f38, SUC_lexicon_tags_1 },
    { 0xeb288f39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e888f3e, SUC_lexicon_tags_77 },
    { 0xc7a88f3f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x81088f42, SUC_lexicon_tags_43 },
    { 0xe5288f42, SUC_lexicon_tags_25 },
    { 0x5f888f43, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74288f4a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de88f54, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b288f5a, SUC_lexicon_tags_37 },
    { 0xdf688f5b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1f888f5d, SUC_lexicon_tags_11 },
    { 0x32888f5d, SUC_lexicon_tags_0 },
    { 0xa2c88f5e, SUC_lexicon_tags_25 },
    { 0x1d688f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b288f62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4f688f64, SUC_lexicon_tags_31 },
    { 0x7d088f65, SUC_lexicon_tags_19 },
    { 0xd0088f66, SUC_lexicon_tags_37 },
    { 0x1c488f66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c088f6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73a88f6c, SUC_lexicon_tags_7 },
    { 0x69e88f6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c88f70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a088f73, SUC_lexicon_tags_37 },
    { 0xa1688f74, SUC_lexicon_tags_37 },
    { 0x3a888f75, SUC_lexicon_tags_84 },
    { 0xe8488f76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x28c88f78, SUC_lexicon_tags_12 },
    { 0xf4e88f79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69e88f7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a88f7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x97888f81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a88f87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55e88f89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a688f90, SUC_lexicon_tags_15 },
    { 0xe8a88f91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8be88f93, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae888f95, SUC_lexicon_tags_28 },
    { 0x24688f96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44688f9a, SUC_lexicon_tags_0 },
    { 0x9f088f9b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x74e88f9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a688fa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2c88fa2, SUC_lexicon_tags_25 },
    { 0x46a88fa3, SUC_lexicon_tags_43 },
    { 0x98488fa4, SUC_lexicon_tags_17 },
    { 0x8f488fa5, SUC_lexicon_tags_296 },
    { 0xbde88fa6, SUC_lexicon_tags_1 },
    { 0xaf888fa6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c88fac, SUC_lexicon_tags_15 },
    { 0x95088fad, SUC_lexicon_tags_37 },
    { 0xa5088fae, SUC_lexicon_tags_25 },
    { 0xc9488faf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22288fb4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe088fba, SUC_lexicon_tags_28 },
    { 0xab488fbb, SUC_lexicon_tags_23 },
    { 0xc5c88fbb, SUC_lexicon_tags_17 },
    { 0xaa288fbc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaca88fbf, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34488fc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6288fcc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc88fcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6c88fd1, SUC_lexicon_tags_25 },
    { 0x488fd2, SUC_lexicon_tags_25 },
    { 0x42088fd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e088fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0288fdc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e88fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8888fe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb088fed, SUC_lexicon_tags_7 },
    { 0x57488fee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc88ff1, SUC_lexicon_tags_11 },
    { 0x1a488ff2, SUC_lexicon_tags_11 },
    { 0x26888ff2, SUC_lexicon_tags_15 },
    { 0x7ca88ff2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1488ff6, SUC_lexicon_tags_11 },
    { 0x1f688ff6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1088ffb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c889000, SUC_lexicon_tags_2 },
    { 0x80689001, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x8ec89003, SUC_lexicon_tags_16 },
    { 0x57889003, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e89008, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92a8900a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8888900c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e889014, SUC_lexicon_tags_30 },
    { 0x24e89015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6689018, SUC_lexicon_tags_7 },
    { 0xfd289019, SUC_lexicon_tags_141 },
    { 0xdae8901a, SUC_lexicon_tags_28 },
    { 0x22e8901a, SUC_lexicon_tags_19 },
    { 0x51a8901c, SUC_lexicon_tags_6 },
    { 0x9128901c, SUC_lexicon_tags_19 },
    { 0x9d68901e, SUC_lexicon_tags_27 },
    { 0xc128901b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1a489021, SUC_lexicon_tags_84 },
    { 0x60889021, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34689026, SUC_lexicon_tags_45 },
    { 0xaa289027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c8902b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63e8902d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd368902f, SUC_lexicon_tags_64 },
    { 0xf7489030, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbee89032, SUC_lexicon_tags_2 },
    { 0xf9689033, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x45a89035, SUC_lexicon_tags_25 },
    { 0xf3089036, SUC_lexicon_tags_43 },
    { 0xb2c89036, SUC_lexicon_tags_3 },
    { 0xb5889036, SUC_lexicon_tags_32 },
    { 0xe5689039, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2048903c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e89043, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a89047, SUC_lexicon_tags_11 },
    { 0x4ec89048, SUC_lexicon_tags_82 },
    { 0x86089049, SUC_lexicon_tags_25 },
    { 0xb5c89049, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe7e8904c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a8904f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7a89051, SUC_lexicon_tags_7 },
    { 0xaaa89052, SUC_lexicon_tags_0 },
    { 0x41889053, SUC_lexicon_tags_25 },
    { 0xf5e89051, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5a289056, SUC_lexicon_tags_10 },
    { 0xb9a89056, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7708905c, SUC_lexicon_tags_45 },
    { 0xcea8905d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64e8905f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x96689061, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab889067, SUC_lexicon_tags_2 },
    { 0x47e89068, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21289072, SUC_lexicon_tags_0 },
    { 0x69e89072, SUC_lexicon_tags_15 },
    { 0x46689073, SUC_lexicon_tags_6 },
    { 0x54c89075, SUC_lexicon_tags_6 },
    { 0x8bc89072, SUC_lexicon_tags_25 },
    { 0x95489076, SUC_lexicon_tags_37 },
    { 0x1e689078, SUC_lexicon_tags_31 },
    { 0x46a89079, SUC_lexicon_tags_11 },
    { 0x7228907a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c89081, SUC_lexicon_tags_61 },
    { 0xd0289082, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8689086, SUC_lexicon_tags_34 },
    { 0x4f089086, SUC_lexicon_tags_2 },
    { 0xd3e89086, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c8908b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea8908f, SUC_lexicon_tags_60 },
    { 0xa288908f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c489093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98889096, SUC_lexicon_tags_25 },
    { 0xcc289097, SUC_lexicon_tags_15 },
    { 0xc1e89097, SUC_lexicon_tags_1 },
    { 0xc3689099, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9848909c, SUC_lexicon_tags_57 },
    { 0x3f68909c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc588909f, SUC_lexicon_tags_27 },
    { 0xef68909f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78890a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe890a6, SUC_lexicon_tags_11 },
    { 0x3ae890a6, SUC_lexicon_tags_37 },
    { 0x49e890a6, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x660890ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be890b0, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984890b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc890b7, SUC_lexicon_tags_14 },
    { 0xf6a890b8, SUC_lexicon_tags_14 },
    { 0x50e890b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0890c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4890c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c890c6, SUC_lexicon_tags_17 },
    { 0xcc890c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50890ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94890cd, SUC_lexicon_tags_49 },
    { 0x6e0890cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4b2890d0, SUC_lexicon_tags_66 },
    { 0xddc890d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ec890d3, SUC_lexicon_tags_82 },
    { 0x71e890d4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x68890d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2dc890d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9de890db, SUC_lexicon_tags_216 },
    { 0xd06890dc, SUC_lexicon_tags_32 },
    { 0x14e890dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc890e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26890ec, SUC_lexicon_tags_14 },
    { 0x77c890ed, SUC_lexicon_tags_47 },
    { 0xc14890ee, SUC_lexicon_tags_49 },
    { 0x82890ef, SUC_lexicon_tags_37 },
    { 0xa4890ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x854890f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x866890f5, SUC_lexicon_tags_32 },
    { 0x710890f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda0890f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c890fb, SUC_lexicon_tags_1 },
    { 0xe9a890fc, SUC_lexicon_tags_37 },
    { 0x764890fd, SUC_lexicon_tags_15 },
    { 0xbe2890fb, SUC_lexicon_tags_1 },
    { 0x904890ff, SUC_lexicon_tags_84 },
    { 0xd6089100, SUC_lexicon_tags_38 },
    { 0xe8e89101, SUC_lexicon_tags_32 },
    { 0xade89101, SUC_lexicon_tags_1 },
    { 0x88689102, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1289108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a8910c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf48910f, SUC_lexicon_tags_39 },
    { 0x8768910f, SUC_lexicon_tags_19 },
    { 0xfc89111, SUC_lexicon_tags_32 },
    { 0x55889111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6889115, SUC_lexicon_tags_38 },
    { 0x36489115, SUC_lexicon_tags_21 },
    { 0xec89117, SUC_lexicon_tags_15 },
    { 0x67289117, SUC_lexicon_tags_15 },
    { 0x46e89117, SUC_lexicon_tags_63 },
    { 0x3b489118, SUC_lexicon_tags_37 },
    { 0x5fc89115, SUC_lexicon_tags_30 },
    { 0xffa8911b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e8911f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e89122, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x21289124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe89129, SUC_lexicon_tags_19 },
    { 0x9828912a, SUC_lexicon_tags_8 },
    { 0xb148912a, SUC_lexicon_tags_11 },
    { 0x2048912a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9489130, SUC_lexicon_tags_25 },
    { 0x1da89130, SUC_lexicon_tags_47 },
    { 0xc7289132, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c89135, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd28913a, SUC_lexicon_tags_4 },
    { 0x9fc8913a, SUC_lexicon_tags_12 },
    { 0x73c8913c, SUC_lexicon_tags_17 },
    { 0xfa68913c, SUC_lexicon_tags_15 },
    { 0x7fa8913a, SUC_lexicon_tags_25 },
    { 0x8068913f, SUC_lexicon_tags_43 },
    { 0x1c08913e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc89144, SUC_lexicon_tags_57 },
    { 0xa5e89144, SUC_lexicon_tags_12 },
    { 0x4ce89145, SUC_lexicon_tags_15 },
    { 0xf6e89147, SUC_lexicon_tags_1 },
    { 0x6489146, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8914c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8488914e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba289150, SUC_lexicon_tags_14 },
    { 0x9fe89150, SUC_lexicon_tags_28 },
    { 0xcea89152, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24089156, SUC_lexicon_tags_7 },
    { 0xd3089157, SUC_lexicon_tags_15 },
    { 0x16c89157, SUC_lexicon_tags_1 },
    { 0x47489159, SUC_lexicon_tags_38 },
    { 0x38e89158, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce08915d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba89160, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf689162, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c89165, SUC_lexicon_tags_21 },
    { 0x51289166, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69489168, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21e8916a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x968916c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6889172, SUC_lexicon_tags_33 },
    { 0x62889173, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8917e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b089180, SUC_lexicon_tags_7 },
    { 0xfd289180, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1289183, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe328918a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec89190, SUC_lexicon_tags_15 },
    { 0xd6689191, SUC_lexicon_tags_9 },
    { 0x78089192, SUC_lexicon_tags_28 },
    { 0x15689193, SUC_lexicon_tags_41 },
    { 0x9b089191, SUC_lexicon_tags_12 },
    { 0xaf489190, SUC_lexicon_tags_100 },
    { 0x1fa89196, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be891a1, SUC_lexicon_tags_17 },
    { 0x78891a2, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x1d8891a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c891a7, SUC_lexicon_tags_17 },
    { 0xcc4891a8, SUC_lexicon_tags_77 },
    { 0xce6891a8, SUC_lexicon_tags_30 },
    { 0xf28891aa, SUC_lexicon_tags_37 },
    { 0x4dc891ab, SUC_lexicon_tags_25 },
    { 0x32c891ac, SUC_lexicon_tags_62 },
    { 0x458891a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a891b1, SUC_lexicon_tags_30 },
    { 0x25c891b2, SUC_lexicon_tags_2 },
    { 0xc46891b3, SUC_lexicon_tags_31 },
    { 0xf26891b4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c891b8, SUC_lexicon_tags_1 },
    { 0x2dc891b9, SUC_lexicon_tags_0 },
    { 0xed2891ba, SUC_lexicon_tags_62 },
    { 0x66e891b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da891c0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96891c4, SUC_lexicon_tags_14 },
    { 0xf54891c5, SUC_lexicon_tags_49 },
    { 0xd04891c6, SUC_lexicon_tags_66 },
    { 0x452891c6, SUC_lexicon_tags_17 },
    { 0x636891c8, SUC_lexicon_tags_9 },
    { 0x59c891c9, SUC_lexicon_tags_25 },
    { 0x230891c8, SUC_lexicon_tags_19 },
    { 0xaec891ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786891d0, SUC_lexicon_tags_15 },
    { 0x82e891d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac891d7, SUC_lexicon_tags_37 },
    { 0x494891d8, SUC_lexicon_tags_65 },
    { 0xd6891d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda891dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdd2891de, SUC_lexicon_tags_53 },
    { 0x862891df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44a891e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa891e4, SUC_lexicon_tags_2 },
    { 0x35a891e5, SUC_lexicon_tags_7 },
    { 0xcc6891e6, SUC_lexicon_tags_37 },
    { 0x3c8891e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724891ef, SUC_lexicon_tags_11 },
    { 0x6c4891f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52891f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56891f8, SUC_lexicon_tags_1 },
    { 0xd42891f9, SUC_lexicon_tags_12 },
    { 0xa9a891fa, SUC_lexicon_tags_28 },
    { 0x534891fb, SUC_lexicon_tags_19 },
    { 0x45c891f9, SUC_lexicon_tags_17 },
    { 0x3e8891fb, SUC_lexicon_tags_12 },
    { 0x89c891fe, SUC_lexicon_tags_39 },
    { 0xe4e891fe, SUC_lexicon_tags_11 },
    { 0xc7e891fe, SUC_lexicon_tags_17 },
    { 0x2b0891ff, SUC_lexicon_tags_175 },
    { 0x43c891fd, SUC_lexicon_tags_179 },
    { 0xe0689203, SUC_lexicon_tags_17 },
    { 0x27089204, SUC_lexicon_tags_37 },
    { 0xe289205, SUC_lexicon_tags_31 },
    { 0x4f689204, SUC_lexicon_tags_8 },
    { 0xa0e89205, SUC_lexicon_tags_23 },
    { 0xe0289208, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe388920c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20e8920e, SUC_lexicon_tags_79 },
    { 0x3e08920e, SUC_lexicon_tags_45 },
    { 0xf0689210, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x81889213, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9b889215, SUC_lexicon_tags_29 },
    { 0x3f689216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84a89218, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08921c, SUC_lexicon_tags_8 },
    { 0xc0e8921d, SUC_lexicon_tags_37 },
    { 0xe0e8921d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96089221, SUC_lexicon_tags_38 },
    { 0x8d889222, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb889224, SUC_lexicon_tags_10 },
    { 0xc9c89225, SUC_lexicon_tags_34 },
    { 0xda89226, SUC_lexicon_tags_80 },
    { 0x3bc89226, SUC_lexicon_tags_19 },
    { 0x71089228, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e68922a, SUC_lexicon_tags_33 },
    { 0x19e8922b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a89234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c489238, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e8923b, SUC_lexicon_tags_45 },
    { 0x6c68923c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf828923f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1c689241, SUC_lexicon_tags_15 },
    { 0xe7489242, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa689245, SUC_lexicon_tags_17 },
    { 0xc1889246, SUC_lexicon_tags_34 },
    { 0xb0689247, SUC_lexicon_tags_1 },
    { 0xed289248, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x47a8924a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6108924c, SUC_lexicon_tags_9 },
    { 0x8808924d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9b28924f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x78289251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34489254, SUC_lexicon_tags_18 },
    { 0xdc689255, SUC_lexicon_tags_28 },
    { 0x4ee89256, SUC_lexicon_tags_15 },
    { 0x9f289256, SUC_lexicon_tags_39 },
    { 0x65e89257, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe308925e, SUC_lexicon_tags_70 },
    { 0x9b08925e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c489263, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c89266, SUC_lexicon_tags_48 },
    { 0x34089267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1689275, SUC_lexicon_tags_26 },
    { 0x80289275, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5808927d, SUC_lexicon_tags_23 },
    { 0xd8c8927d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a89283, SUC_lexicon_tags_12 },
    { 0x81c89283, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc089289, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc289293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e089299, SUC_lexicon_tags_0 },
    { 0x4f68929a, SUC_lexicon_tags_68 },
    { 0x7088929b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf28929d, SUC_lexicon_tags_23 },
    { 0x4168929e, SUC_lexicon_tags_37 },
    { 0xea88929e, SUC_lexicon_tags_9 },
    { 0xc6a892a0, SUC_lexicon_tags_17 },
    { 0x73c892a1, SUC_lexicon_tags_17 },
    { 0x168892a2, SUC_lexicon_tags_7 },
    { 0xd92892a0, SUC_lexicon_tags_77 },
    { 0x710892a2, SUC_lexicon_tags_28 },
    { 0x7188929f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec892ab, SUC_lexicon_tags_1 },
    { 0x1ac892ab, SUC_lexicon_tags_7 },
    { 0x874892ac, SUC_lexicon_tags_84 },
    { 0xcf4892ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e892b8, SUC_lexicon_tags_9 },
    { 0xd84892b9, SUC_lexicon_tags_37 },
    { 0x7f2892b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x846892bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc892bf, SUC_lexicon_tags_32 },
    { 0x6ba892bf, SUC_lexicon_tags_8 },
    { 0x402892c0, SUC_lexicon_tags_25 },
    { 0x910892c0, SUC_lexicon_tags_8 },
    { 0x66c892c2, SUC_lexicon_tags_11 },
    { 0x6d6892c4, SUC_lexicon_tags_11 },
    { 0x4b6892c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x922892c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108892cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x90e892ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2892d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8892d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c892da, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572892de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x556892e1, SUC_lexicon_tags_7 },
    { 0xfee892e1, SUC_lexicon_tags_8 },
    { 0x13a892e3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x254892e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e892ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d0892ee, SUC_lexicon_tags_14 },
    { 0xaae892ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c892f5, SUC_lexicon_tags_11 },
    { 0x154892f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a892fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29c892fd, SUC_lexicon_tags_29 },
    { 0x274892fe, SUC_lexicon_tags_17 },
    { 0xf50892ff, SUC_lexicon_tags_23 },
    { 0x2ea89300, SUC_lexicon_tags_17 },
    { 0x73e89301, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x65289303, SUC_lexicon_tags_2 },
    { 0x3c289304, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e289307, SUC_lexicon_tags_11 },
    { 0x48889308, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd08930c, SUC_lexicon_tags_2 },
    { 0x8ba8930d, SUC_lexicon_tags_114 },
    { 0x3ca8930d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a89316, SUC_lexicon_tags_27 },
    { 0x58e89317, SUC_lexicon_tags_10 },
    { 0x7ee89316, SUC_lexicon_tags_15 },
    { 0xf0489319, SUC_lexicon_tags_7 },
    { 0x47689316, SUC_lexicon_tags_25 },
    { 0xf2289319, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc8931e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23289322, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc89326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a08932b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc8932e, SUC_lexicon_tags_13 },
    { 0x87a8932e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9289336, SUC_lexicon_tags_25 },
    { 0xdea89337, SUC_lexicon_tags_8 },
    { 0x26689338, SUC_lexicon_tags_12 },
    { 0x1d489337, SUC_lexicon_tags_2 },
    { 0xee289339, SUC_lexicon_tags_25 },
    { 0xa7e8933b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8489342, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f889345, SUC_lexicon_tags_37 },
    { 0x90089346, SUC_lexicon_tags_61 },
    { 0x5e89347, SUC_lexicon_tags_1 },
    { 0x50289346, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9548934d, SUC_lexicon_tags_25 },
    { 0xe1e8934e, SUC_lexicon_tags_37 },
    { 0x27c8934e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8889351, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x70289354, SUC_lexicon_tags_63 },
    { 0x56c89354, SUC_lexicon_tags_17 },
    { 0xbe889356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60a89358, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29689363, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a89366, SUC_lexicon_tags_12 },
    { 0x26289367, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4088936a, SUC_lexicon_tags_38 },
    { 0x8048936b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7a8936d, SUC_lexicon_tags_27 },
    { 0xc208936e, SUC_lexicon_tags_37 },
    { 0xb588936e, SUC_lexicon_tags_11 },
    { 0x53689370, SUC_lexicon_tags_31 },
    { 0x62c89370, SUC_lexicon_tags_40 },
    { 0x72e89372, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ac89374, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc8937b, SUC_lexicon_tags_6 },
    { 0x8de8937b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a8937f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e889381, SUC_lexicon_tags_37 },
    { 0x4f889382, SUC_lexicon_tags_25 },
    { 0x12289383, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3da89386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b289389, SUC_lexicon_tags_11 },
    { 0x83e8938a, SUC_lexicon_tags_17 },
    { 0x9268938a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f689390, SUC_lexicon_tags_1 },
    { 0xd0a89391, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcf089393, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08939a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c88939f, SUC_lexicon_tags_12 },
    { 0xbb68939f, SUC_lexicon_tags_34 },
    { 0x82a893a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec893a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x538893a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212893ae, SUC_lexicon_tags_27 },
    { 0x1cc893ae, SUC_lexicon_tags_112 },
    { 0x2d4893b0, SUC_lexicon_tags_31 },
    { 0x64a893b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224893be, SUC_lexicon_tags_17 },
    { 0xa6e893bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6893c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x806893c4, SUC_lexicon_tags_31 },
    { 0x1e6893c4, SUC_lexicon_tags_17 },
    { 0x246893c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x314893c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa8893ca, SUC_lexicon_tags_37 },
    { 0x382893cb, SUC_lexicon_tags_47 },
    { 0x554893cc, SUC_lexicon_tags_30 },
    { 0x7d8893cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb52893cf, SUC_lexicon_tags_23 },
    { 0xce0893cf, SUC_lexicon_tags_1 },
    { 0x908893d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0893d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf60893d6, SUC_lexicon_tags_2 },
    { 0x6e2893d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x294893da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x462893dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392893e0, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2893e6, SUC_lexicon_tags_26 },
    { 0x64e893e6, SUC_lexicon_tags_4 },
    { 0xb62893e7, SUC_lexicon_tags_38 },
    { 0xb64893e7, SUC_lexicon_tags_77 },
    { 0xf3c893ea, SUC_lexicon_tags_14 },
    { 0x12893e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe76893ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7c893ef, SUC_lexicon_tags_12 },
    { 0xff0893ef, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbf4893f2, SUC_lexicon_tags_37 },
    { 0xcb2893f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e893f9, SUC_lexicon_tags_70 },
    { 0x4b8893f9, SUC_lexicon_tags_27 },
    { 0x188893fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2893fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b289403, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe1889405, SUC_lexicon_tags_12 },
    { 0xf3289406, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a8940a, SUC_lexicon_tags_7 },
    { 0xd088940a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf48940f, SUC_lexicon_tags_19 },
    { 0xd0e8940f, SUC_lexicon_tags_13 },
    { 0x90089411, SUC_lexicon_tags_37 },
    { 0x6ce89412, SUC_lexicon_tags_15 },
    { 0xab089413, SUC_lexicon_tags_33 },
    { 0x7a889414, SUC_lexicon_tags_11 },
    { 0x46689415, SUC_lexicon_tags_23 },
    { 0xc8e89416, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a8941a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6908941f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa4089421, SUC_lexicon_tags_43 },
    { 0x6c089422, SUC_lexicon_tags_48 },
    { 0x70689423, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x64689426, SUC_lexicon_tags_17 },
    { 0x14889427, SUC_lexicon_tags_25 },
    { 0x94e89427, SUC_lexicon_tags_27 },
    { 0xa2c89429, SUC_lexicon_tags_37 },
    { 0xd2289428, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6148942e, SUC_lexicon_tags_25 },
    { 0x5fa8942f, SUC_lexicon_tags_7 },
    { 0x8708942f, SUC_lexicon_tags_43 },
    { 0x8d28942f, SUC_lexicon_tags_17 },
    { 0xf6889432, SUC_lexicon_tags_26 },
    { 0x2e489433, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa468943b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8943d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e89440, SUC_lexicon_tags_24 },
    { 0x65889440, SUC_lexicon_tags_35 },
    { 0x15089440, SUC_lexicon_tags_9 },
    { 0xa8a89440, SUC_lexicon_tags_37 },
    { 0xa3489444, SUC_lexicon_tags_37 },
    { 0x66289445, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa8944a, SUC_lexicon_tags_2 },
    { 0x10e8944a, SUC_lexicon_tags_235 },
    { 0xe148944c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38889453, SUC_lexicon_tags_33 },
    { 0x2b489454, SUC_lexicon_tags_4 },
    { 0xea289454, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32289459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e8945e, SUC_lexicon_tags_38 },
    { 0x8b68945f, SUC_lexicon_tags_25 },
    { 0x6fa89460, SUC_lexicon_tags_47 },
    { 0xf3c8945f, SUC_lexicon_tags_30 },
    { 0x64089461, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e89465, SUC_lexicon_tags_33 },
    { 0x61889466, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa489469, SUC_lexicon_tags_37 },
    { 0xac289469, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48946d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c89470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2e89472, SUC_lexicon_tags_26 },
    { 0x1f889472, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52089475, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c89478, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee28947e, SUC_lexicon_tags_15 },
    { 0x7c8947f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x26689481, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a89487, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a8948a, SUC_lexicon_tags_47 },
    { 0xe688948b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7428948f, SUC_lexicon_tags_2 },
    { 0x2f88948f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96889498, SUC_lexicon_tags_2 },
    { 0x88489499, SUC_lexicon_tags_12 },
    { 0x2fe89498, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd248949c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c894a5, SUC_lexicon_tags_4 },
    { 0x51a894a5, SUC_lexicon_tags_77 },
    { 0x230894a7, SUC_lexicon_tags_17 },
    { 0xa96894a7, SUC_lexicon_tags_4 },
    { 0x448894a6, SUC_lexicon_tags_17 },
    { 0x2a0894a7, SUC_lexicon_tags_17 },
    { 0x9a2894ab, SUC_lexicon_tags_25 },
    { 0x11c894ac, SUC_lexicon_tags_17 },
    { 0x240894a6, SUC_lexicon_tags_53 },
    { 0x744894a7, SUC_lexicon_tags_23 },
    { 0x42c894ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26894b3, SUC_lexicon_tags_72 },
    { 0xb2a894b4, SUC_lexicon_tags_37 },
    { 0xa96894b3, SUC_lexicon_tags_17 },
    { 0x780894b5, SUC_lexicon_tags_0 },
    { 0x5fc894b7, SUC_lexicon_tags_6 },
    { 0x9a894b7, SUC_lexicon_tags_664 },
    { 0x872894b7, SUC_lexicon_tags_28 },
    { 0x46c894ba, SUC_lexicon_tags_17 },
    { 0x298894b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8894bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa70894bf, SUC_lexicon_tags_60 },
    { 0xd58894c0, SUC_lexicon_tags_0 },
    { 0x646894c0, SUC_lexicon_tags_63 },
    { 0x130894c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30894c7, SUC_lexicon_tags_7 },
    { 0x7d8894c7, SUC_lexicon_tags_23 },
    { 0x296894c7, SUC_lexicon_tags_38 },
    { 0x56a894ca, SUC_lexicon_tags_55 },
    { 0x994894cb, SUC_lexicon_tags_2 },
    { 0x984894c8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c894cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b4894d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32894d5, SUC_lexicon_tags_0 },
    { 0x1e0894d5, SUC_lexicon_tags_60 },
    { 0x3a6894d6, SUC_lexicon_tags_141 },
    { 0x252894d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x828894db, SUC_lexicon_tags_15 },
    { 0xc40894dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea894e0, SUC_lexicon_tags_15 },
    { 0x3de894e1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xee8894e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa894e9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x764894ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f4894ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c894f2, SUC_lexicon_tags_7 },
    { 0x7d4894f2, SUC_lexicon_tags_10 },
    { 0xe0a894f4, SUC_lexicon_tags_37 },
    { 0x8da894f5, SUC_lexicon_tags_116 },
    { 0x24c894f5, SUC_lexicon_tags_37 },
    { 0x53c894f2, SUC_lexicon_tags_33 },
    { 0x69e894f8, SUC_lexicon_tags_2 },
    { 0xee6894f9, SUC_lexicon_tags_17 },
    { 0x3c2894f8, SUC_lexicon_tags_46 },
    { 0xcc894fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c894fe, SUC_lexicon_tags_54 },
    { 0xf00894ff, SUC_lexicon_tags_25 },
    { 0x93089500, SUC_lexicon_tags_25 },
    { 0xa6089501, SUC_lexicon_tags_15 },
    { 0x86489502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38489508, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e8950c, SUC_lexicon_tags_14 },
    { 0x6b88950d, SUC_lexicon_tags_31 },
    { 0x13e8950e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd48951a, SUC_lexicon_tags_25 },
    { 0xb788951b, SUC_lexicon_tags_43 },
    { 0xa308951a, SUC_lexicon_tags_87 },
    { 0xebc8951d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78889524, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc089528, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9b48952a, SUC_lexicon_tags_25 },
    { 0x5488952a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x7508952e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23889531, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x46689534, SUC_lexicon_tags_17 },
    { 0xa89535, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x35689537, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1148953b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5a8953d, SUC_lexicon_tags_37 },
    { 0x5d88953d, SUC_lexicon_tags_14 },
    { 0xfd68953f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc089545, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a89549, SUC_lexicon_tags_61 },
    { 0xb288954a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1488954c, SUC_lexicon_tags_23 },
    { 0xbcc8954d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc89552, SUC_lexicon_tags_27 },
    { 0x97889553, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a89556, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8955f, SUC_lexicon_tags_12 },
    { 0x76889560, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e89564, SUC_lexicon_tags_29 },
    { 0x2c889564, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a89568, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848956f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c289576, SUC_lexicon_tags_7 },
    { 0x99089577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e28957c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81289581, SUC_lexicon_tags_15 },
    { 0xad289582, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a89589, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8068958b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde68958d, SUC_lexicon_tags_7 },
    { 0x14c8958e, SUC_lexicon_tags_49 },
    { 0xaf88958f, SUC_lexicon_tags_1 },
    { 0x6d089590, SUC_lexicon_tags_2 },
    { 0x79c89591, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39889595, SUC_lexicon_tags_2 },
    { 0xd5689596, SUC_lexicon_tags_46 },
    { 0x9fe89597, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622895a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986895a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xede895aa, SUC_lexicon_tags_27 },
    { 0x46c895aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c895af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400895b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0895c1, SUC_lexicon_tags_7 },
    { 0x218895c1, SUC_lexicon_tags_0 },
    { 0xd70895c2, SUC_lexicon_tags_28 },
    { 0x8c6895c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe68895c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e895ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a895d1, SUC_lexicon_tags_25 },
    { 0xcfc895d2, SUC_lexicon_tags_7 },
    { 0x56a895d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78895d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x998895d9, SUC_lexicon_tags_25 },
    { 0xbc895da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43e895dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8895e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02895e8, SUC_lexicon_tags_15 },
    { 0x17e895e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe10895eb, SUC_lexicon_tags_7 },
    { 0x3a0895ec, SUC_lexicon_tags_11 },
    { 0xdb2895ec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa895f1, SUC_lexicon_tags_63 },
    { 0x1a2895f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530895f7, SUC_lexicon_tags_454 },
    { 0, NULL },
    { 0x514895f9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x74a895fb, SUC_lexicon_tags_17 },
    { 0x6c6895fc, SUC_lexicon_tags_2 },
    { 0x312895fd, SUC_lexicon_tags_38 },
    { 0x51e895fd, SUC_lexicon_tags_2 },
    { 0xf54895ff, SUC_lexicon_tags_11 },
    { 0xc48895fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e89605, SUC_lexicon_tags_77 },
    { 0x31489606, SUC_lexicon_tags_11 },
    { 0xd8c89606, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a88960a, SUC_lexicon_tags_13 },
    { 0xff08960b, SUC_lexicon_tags_46 },
    { 0x8208960c, SUC_lexicon_tags_0 },
    { 0x6e88960d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3089612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c89615, SUC_lexicon_tags_27 },
    { 0x15e89616, SUC_lexicon_tags_15 },
    { 0x91689615, SUC_lexicon_tags_33 },
    { 0xbd889618, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3748961a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf388961c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe89623, SUC_lexicon_tags_2 },
    { 0x8c489624, SUC_lexicon_tags_10 },
    { 0x8c689625, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x72c89627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb68962b, SUC_lexicon_tags_11 },
    { 0x7f88962c, SUC_lexicon_tags_28 },
    { 0x11a8962d, SUC_lexicon_tags_15 },
    { 0x58c8962e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94489632, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad889636, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4889638, SUC_lexicon_tags_18 },
    { 0x7e089639, SUC_lexicon_tags_30 },
    { 0xa148963a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x34a8963c, SUC_lexicon_tags_37 },
    { 0xb508963c, SUC_lexicon_tags_27 },
    { 0x93a8963c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9889642, SUC_lexicon_tags_47 },
    { 0x21689643, SUC_lexicon_tags_15 },
    { 0xc6489642, SUC_lexicon_tags_23 },
    { 0x9a289645, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea89649, SUC_lexicon_tags_11 },
    { 0xcf689649, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7768964c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e89650, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac89653, SUC_lexicon_tags_15 },
    { 0x77489654, SUC_lexicon_tags_897 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae89658, SUC_lexicon_tags_17 },
    { 0xa7489658, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa268965c, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4689663, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe4289665, SUC_lexicon_tags_2 },
    { 0xfce89665, SUC_lexicon_tags_2 },
    { 0x1fe89666, SUC_lexicon_tags_25 },
    { 0x95e89667, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da8966d, SUC_lexicon_tags_1 },
    { 0xc2c8966d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25089676, SUC_lexicon_tags_37 },
    { 0xee089676, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xebe89679, SUC_lexicon_tags_17 },
    { 0x4448967a, SUC_lexicon_tags_30 },
    { 0xb8c8967b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecc8967d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c89681, SUC_lexicon_tags_49 },
    { 0xe5489682, SUC_lexicon_tags_12 },
    { 0x1e489683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7289685, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad489688, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4c8968a, SUC_lexicon_tags_17 },
    { 0xa648968b, SUC_lexicon_tags_68 },
    { 0x29e8968c, SUC_lexicon_tags_6 },
    { 0xec8968a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e889691, SUC_lexicon_tags_7 },
    { 0x27e89692, SUC_lexicon_tags_17 },
    { 0x4b289693, SUC_lexicon_tags_37 },
    { 0x46089693, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x98289697, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2588969c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21a8969e, SUC_lexicon_tags_7 },
    { 0xf868969f, SUC_lexicon_tags_11 },
    { 0x49e8969f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882896a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4896aa, SUC_lexicon_tags_23 },
    { 0x4ee896ab, SUC_lexicon_tags_25 },
    { 0x794896aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xece896ae, SUC_lexicon_tags_7 },
    { 0x778896ae, SUC_lexicon_tags_2 },
    { 0xb9a896b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e896b3, SUC_lexicon_tags_30 },
    { 0xdf6896b4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xed2896b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x906896b9, SUC_lexicon_tags_47 },
    { 0x68c896ba, SUC_lexicon_tags_48 },
    { 0x50a896bb, SUC_lexicon_tags_15 },
    { 0xce0896bc, SUC_lexicon_tags_33 },
    { 0xa5e896bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa32896bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe896c7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26896cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x738896cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ea896cf, SUC_lexicon_tags_37 },
    { 0xe06896d0, SUC_lexicon_tags_29 },
    { 0x58e896cf, SUC_lexicon_tags_32 },
    { 0x3b8896d2, SUC_lexicon_tags_11 },
    { 0x41a896d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278896d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc896e0, SUC_lexicon_tags_49 },
    { 0x2c0896e0, SUC_lexicon_tags_7 },
    { 0xe1e896e2, SUC_lexicon_tags_37 },
    { 0x3f2896e3, SUC_lexicon_tags_66 },
    { 0xb82896e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2896e9, SUC_lexicon_tags_25 },
    { 0x584896e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2896ed, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64896f0, SUC_lexicon_tags_25 },
    { 0x576896f1, SUC_lexicon_tags_60 },
    { 0x1ee896f2, SUC_lexicon_tags_92 },
    { 0x600896f3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c896ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9689701, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba8970c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde48970e, SUC_lexicon_tags_37 },
    { 0xe308970e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x19089711, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9689714, SUC_lexicon_tags_26 },
    { 0x16a89715, SUC_lexicon_tags_33 },
    { 0xc0889716, SUC_lexicon_tags_19 },
    { 0xf9a89714, SUC_lexicon_tags_17 },
    { 0x6cc89715, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8988971b, SUC_lexicon_tags_12 },
    { 0xc868971c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5689726, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x98289729, SUC_lexicon_tags_49 },
    { 0xb489729, SUC_lexicon_tags_7 },
    { 0xd8689729, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfea8972d, SUC_lexicon_tags_39 },
    { 0x52a8972e, SUC_lexicon_tags_14 },
    { 0x2d08972f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1489737, SUC_lexicon_tags_60 },
    { 0x42c89738, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5748973a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1a8973c, SUC_lexicon_tags_12 },
    { 0x2708973d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4289740, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x39289743, SUC_lexicon_tags_1 },
    { 0xb9a89744, SUC_lexicon_tags_1 },
    { 0x3a889745, SUC_lexicon_tags_7 },
    { 0x70689746, SUC_lexicon_tags_77 },
    { 0xd5a89746, SUC_lexicon_tags_45 },
    { 0x20289743, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7888974a, SUC_lexicon_tags_19 },
    { 0x8d08974b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfb88974d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c89759, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf08975c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5889760, SUC_lexicon_tags_30 },
    { 0x90c89761, SUC_lexicon_tags_14 },
    { 0x47889760, SUC_lexicon_tags_26 },
    { 0xbde89763, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec89766, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a89768, SUC_lexicon_tags_0 },
    { 0xc0489769, SUC_lexicon_tags_16 },
    { 0xd7889769, SUC_lexicon_tags_11 },
    { 0x86289769, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d48976e, SUC_lexicon_tags_37 },
    { 0xeae8976f, SUC_lexicon_tags_28 },
    { 0x43e89770, SUC_lexicon_tags_25 },
    { 0xe9e89770, SUC_lexicon_tags_1 },
    { 0xd0489772, SUC_lexicon_tags_49 },
    { 0x3b089773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3489775, SUC_lexicon_tags_1 },
    { 0x78889776, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f08977a, SUC_lexicon_tags_33 },
    { 0xf08977b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1ca8977d, SUC_lexicon_tags_55 },
    { 0xefc8977d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf0289780, SUC_lexicon_tags_8 },
    { 0x8289780, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e489785, SUC_lexicon_tags_29 },
    { 0x42a89786, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6328978a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca689792, SUC_lexicon_tags_29 },
    { 0x39e89793, SUC_lexicon_tags_11 },
    { 0xa8a89793, SUC_lexicon_tags_13 },
    { 0xff289792, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa89799, SUC_lexicon_tags_17 },
    { 0x1808979a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e897a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x534897a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec897a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0897af, SUC_lexicon_tags_23 },
    { 0x60e897b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x816897b2, SUC_lexicon_tags_23 },
    { 0xf04897b3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a897ba, SUC_lexicon_tags_11 },
    { 0x312897bb, SUC_lexicon_tags_11 },
    { 0xd04897bc, SUC_lexicon_tags_11 },
    { 0xe5e897bd, SUC_lexicon_tags_1 },
    { 0xa9e897ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e897c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd94897c3, SUC_lexicon_tags_11 },
    { 0x664897c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaf0897c6, SUC_lexicon_tags_6 },
    { 0x1d6897c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6897cc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6897d1, SUC_lexicon_tags_23 },
    { 0x698897d2, SUC_lexicon_tags_39 },
    { 0x2a8897d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a897d8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34897e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c897e3, SUC_lexicon_tags_66 },
    { 0x45e897e4, SUC_lexicon_tags_56 },
    { 0x244897e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244897ea, SUC_lexicon_tags_37 },
    { 0xbb4897eb, SUC_lexicon_tags_9 },
    { 0xdac897ec, SUC_lexicon_tags_37 },
    { 0x418897eb, SUC_lexicon_tags_84 },
    { 0xb18897eb, SUC_lexicon_tags_29 },
    { 0x4a4897ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e897f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa897f6, SUC_lexicon_tags_151 },
    { 0xe00897f7, SUC_lexicon_tags_47 },
    { 0x826897f8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0897fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f8897fe, SUC_lexicon_tags_11 },
    { 0xfa897fe, SUC_lexicon_tags_14 },
    { 0x648897ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d089805, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x75889807, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd3c89809, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf7e8980b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa728980d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a89811, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x60489814, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf289817, SUC_lexicon_tags_61 },
    { 0xede89818, SUC_lexicon_tags_28 },
    { 0x2da89817, SUC_lexicon_tags_27 },
    { 0x2f08981a, SUC_lexicon_tags_23 },
    { 0xd768981b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd428981f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9289823, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc89827, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c48982a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a8982d, SUC_lexicon_tags_9 },
    { 0x2ba8982e, SUC_lexicon_tags_49 },
    { 0xb0e8982f, SUC_lexicon_tags_66 },
    { 0x8f889830, SUC_lexicon_tags_11 },
    { 0x9c489831, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde89836, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38489838, SUC_lexicon_tags_17 },
    { 0x5e689839, SUC_lexicon_tags_60 },
    { 0x7e88983a, SUC_lexicon_tags_30 },
    { 0x95e89838, SUC_lexicon_tags_25 },
    { 0x51a89839, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29289844, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc89849, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6548984b, SUC_lexicon_tags_28 },
    { 0x55a8984b, SUC_lexicon_tags_107 },
    { 0xe788984c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63289859, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a8985c, SUC_lexicon_tags_2 },
    { 0x7fa8985c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4889860, SUC_lexicon_tags_17 },
    { 0x68e89861, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7a89863, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1289867, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9248986f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88987a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x14c8987c, SUC_lexicon_tags_37 },
    { 0x7488987c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7c8987f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3089885, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa89889, SUC_lexicon_tags_3 },
    { 0x9dc8988a, SUC_lexicon_tags_43 },
    { 0x5528988a, SUC_lexicon_tags_12 },
    { 0x5ea8988b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec89890, SUC_lexicon_tags_15 },
    { 0x97689890, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xea489893, SUC_lexicon_tags_46 },
    { 0xc5489894, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60289899, SUC_lexicon_tags_60 },
    { 0x60c8989a, SUC_lexicon_tags_15 },
    { 0x1ac8989b, SUC_lexicon_tags_105 },
    { 0xbe28989c, SUC_lexicon_tags_17 },
    { 0xc168989d, SUC_lexicon_tags_13 },
    { 0xc088989e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c898a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a898a9, SUC_lexicon_tags_10 },
    { 0x9f6898aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a898b5, SUC_lexicon_tags_16 },
    { 0x764898b5, SUC_lexicon_tags_11 },
    { 0xd8a898b7, SUC_lexicon_tags_66 },
    { 0x504898b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c898bc, SUC_lexicon_tags_2 },
    { 0x1ae898bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6898c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2c2898c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ae898c6, SUC_lexicon_tags_8 },
    { 0x27c898c7, SUC_lexicon_tags_68 },
    { 0x2d4898c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216898d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6c898d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8de898d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x830898d9, SUC_lexicon_tags_26 },
    { 0x21a898d9, SUC_lexicon_tags_30 },
    { 0x336898db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdfe898dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fc898df, SUC_lexicon_tags_2 },
    { 0xc38898df, SUC_lexicon_tags_7 },
    { 0xef6898e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a898e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6898e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c898f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32898f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba898f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72898fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee89900, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10889906, SUC_lexicon_tags_27 },
    { 0xe6289907, SUC_lexicon_tags_0 },
    { 0x50089908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b48990e, SUC_lexicon_tags_47 },
    { 0x148990f, SUC_lexicon_tags_25 },
    { 0xd8889910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b089913, SUC_lexicon_tags_33 },
    { 0xee289913, SUC_lexicon_tags_30 },
    { 0xc7e89915, SUC_lexicon_tags_2 },
    { 0x64a89915, SUC_lexicon_tags_37 },
    { 0x82e89914, SUC_lexicon_tags_9 },
    { 0xb5689918, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7128991d, SUC_lexicon_tags_60 },
    { 0xd408991e, SUC_lexicon_tags_15 },
    { 0xcc68991f, SUC_lexicon_tags_7 },
    { 0xea8991f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x39889923, SUC_lexicon_tags_17 },
    { 0xe9889924, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a89927, SUC_lexicon_tags_66 },
    { 0x5ba89928, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce8992b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4889930, SUC_lexicon_tags_15 },
    { 0x5b689931, SUC_lexicon_tags_1 },
    { 0x91e89932, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c89935, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ae89937, SUC_lexicon_tags_14 },
    { 0x96089938, SUC_lexicon_tags_12 },
    { 0xf6c89938, SUC_lexicon_tags_7 },
    { 0xb3c89938, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e8993f, SUC_lexicon_tags_14 },
    { 0xe6489940, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcca89942, SUC_lexicon_tags_27 },
    { 0x6ee89942, SUC_lexicon_tags_1 },
    { 0x1e889944, SUC_lexicon_tags_12 },
    { 0x23e89945, SUC_lexicon_tags_5 },
    { 0x1cc89944, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8089949, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9728994d, SUC_lexicon_tags_37 },
    { 0xd768994d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x77489950, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5489953, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x39289955, SUC_lexicon_tags_8 },
    { 0x9fe89956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6e89958, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6289961, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0x4bc89963, SUC_lexicon_tags_17 },
    { 0x21e89963, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e089968, SUC_lexicon_tags_60 },
    { 0xe1289969, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4068996c, SUC_lexicon_tags_49 },
    { 0x4c28996c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcc8996f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x22c89971, SUC_lexicon_tags_19 },
    { 0x39489972, SUC_lexicon_tags_1 },
    { 0x4b489973, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e89976, SUC_lexicon_tags_11 },
    { 0x7fc89977, SUC_lexicon_tags_25 },
    { 0x95889978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb168997a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf488997d, SUC_lexicon_tags_113 },
    { 0xea8997e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21c89980, SUC_lexicon_tags_29 },
    { 0x5aa89980, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa5689983, SUC_lexicon_tags_7 },
    { 0xe5089984, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8998d, SUC_lexicon_tags_12 },
    { 0xce8998d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a89994, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0889999, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xff08999c, SUC_lexicon_tags_12 },
    { 0x248999d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e899a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f8899a2, SUC_lexicon_tags_87 },
    { 0xe0e899a2, SUC_lexicon_tags_12 },
    { 0xd4899a2, SUC_lexicon_tags_17 },
    { 0x608899a5, SUC_lexicon_tags_9 },
    { 0x3ac899a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e899aa, SUC_lexicon_tags_17 },
    { 0xda0899ab, SUC_lexicon_tags_37 },
    { 0xb36899ac, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a899b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa18899b5, SUC_lexicon_tags_1 },
    { 0xdd4899b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4ac899b8, SUC_lexicon_tags_8 },
    { 0x986899b9, SUC_lexicon_tags_25 },
    { 0x52c899ba, SUC_lexicon_tags_38 },
    { 0x22a899b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x250899bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa7c899bf, SUC_lexicon_tags_37 },
    { 0x24c899c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c899c5, SUC_lexicon_tags_16 },
    { 0x79c899c5, SUC_lexicon_tags_31 },
    { 0x12e899c7, SUC_lexicon_tags_38 },
    { 0xf7a899c5, SUC_lexicon_tags_1 },
    { 0xef0899c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e4899cb, SUC_lexicon_tags_8 },
    { 0x552899cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14899d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x556899d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa899d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e899dc, SUC_lexicon_tags_37 },
    { 0x36a899dd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62899e2, SUC_lexicon_tags_7 },
    { 0x228899e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa899ec, SUC_lexicon_tags_9 },
    { 0x1be899ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496899f8, SUC_lexicon_tags_19 },
    { 0x1de899f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6de899fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96899fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x50289a01, SUC_lexicon_tags_33 },
    { 0xfa889a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f089a06, SUC_lexicon_tags_37 },
    { 0x8d689a07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd089a09, SUC_lexicon_tags_43 },
    { 0x76c89a0a, SUC_lexicon_tags_12 },
    { 0xf0689a0b, SUC_lexicon_tags_63 },
    { 0xf4c89a0c, SUC_lexicon_tags_12 },
    { 0x81889a0d, SUC_lexicon_tags_25 },
    { 0x4d889a0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e889a14, SUC_lexicon_tags_15 },
    { 0xb7e89a15, SUC_lexicon_tags_15 },
    { 0x39089a15, SUC_lexicon_tags_7 },
    { 0xe5489a15, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a89a24, SUC_lexicon_tags_26 },
    { 0x52889a25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfc89a27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc3089a29, SUC_lexicon_tags_2 },
    { 0xe7a89a2a, SUC_lexicon_tags_23 },
    { 0x7c689a2b, SUC_lexicon_tags_37 },
    { 0x3ac89a2c, SUC_lexicon_tags_7 },
    { 0xdaa89a2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a889a34, SUC_lexicon_tags_28 },
    { 0x68689a35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c489a37, SUC_lexicon_tags_4 },
    { 0x3f289a37, SUC_lexicon_tags_37 },
    { 0x40a89a38, SUC_lexicon_tags_7 },
    { 0x2d689a39, SUC_lexicon_tags_15 },
    { 0xdf689a3b, SUC_lexicon_tags_38 },
    { 0x79e89a39, SUC_lexicon_tags_8 },
    { 0xc9289a3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcea89a3f, SUC_lexicon_tags_1 },
    { 0xb6a89a40, SUC_lexicon_tags_28 },
    { 0x22a89a40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0689a46, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x62c89a48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0889a4c, SUC_lexicon_tags_7 },
    { 0x19889a4c, SUC_lexicon_tags_23 },
    { 0xaa289a4e, SUC_lexicon_tags_32 },
    { 0x7d489a4f, SUC_lexicon_tags_30 },
    { 0xde289a4f, SUC_lexicon_tags_41 },
    { 0xa9889a4f, SUC_lexicon_tags_87 },
    { 0xe4289a52, SUC_lexicon_tags_35 },
    { 0xd889a4e, SUC_lexicon_tags_19 },
    { 0x38689a54, SUC_lexicon_tags_2 },
    { 0x50a89a55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc2289a57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee89a5b, SUC_lexicon_tags_37 },
    { 0x22289a5c, SUC_lexicon_tags_37 },
    { 0x89689a5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee89a62, SUC_lexicon_tags_12 },
    { 0x44e89a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef089a6b, SUC_lexicon_tags_17 },
    { 0xb5289a6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe489a6f, SUC_lexicon_tags_37 },
    { 0x44489a70, SUC_lexicon_tags_25 },
    { 0x8dc89a71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90889a74, SUC_lexicon_tags_12 },
    { 0x85289a75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c89a7a, SUC_lexicon_tags_65 },
    { 0x66a89a7b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52689a80, SUC_lexicon_tags_12 },
    { 0xea489a81, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3289a86, SUC_lexicon_tags_38 },
    { 0xc2489a87, SUC_lexicon_tags_15 },
    { 0x46489a86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c289a90, SUC_lexicon_tags_42 },
    { 0x6ca89a90, SUC_lexicon_tags_4 },
    { 0x60689a90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5289a94, SUC_lexicon_tags_7 },
    { 0xa0e89a95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6489a97, SUC_lexicon_tags_68 },
    { 0x26689a98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x91289a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc89a9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c89aa2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6ee89aa4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee89aa8, SUC_lexicon_tags_1 },
    { 0x42489aa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79289aad, SUC_lexicon_tags_0 },
    { 0x34c89aae, SUC_lexicon_tags_2 },
    { 0x2b689aad, SUC_lexicon_tags_0 },
    { 0x6b489ab0, SUC_lexicon_tags_37 },
    { 0xb6089ab1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6489ab5, SUC_lexicon_tags_160 },
    { 0x7f089ab5, SUC_lexicon_tags_7 },
    { 0x63c89ab7, SUC_lexicon_tags_66 },
    { 0x49e89ab6, SUC_lexicon_tags_5 },
    { 0x5b089ab9, SUC_lexicon_tags_25 },
    { 0x39a89ab9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd889abe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2889ac0, SUC_lexicon_tags_12 },
    { 0xcca89ac0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45489ac4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa3889ac6, SUC_lexicon_tags_45 },
    { 0xaba89ac7, SUC_lexicon_tags_33 },
    { 0x7bc89ac8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55c89aca, SUC_lexicon_tags_43 },
    { 0x23689aca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f889acd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e889ad0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe89ad3, SUC_lexicon_tags_9 },
    { 0xa4a89ad4, SUC_lexicon_tags_185 },
    { 0xf4889ad3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9f089ad7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c89ada, SUC_lexicon_tags_17 },
    { 0x9d089adb, SUC_lexicon_tags_7 },
    { 0xdb889adc, SUC_lexicon_tags_15 },
    { 0x89689add, SUC_lexicon_tags_84 },
    { 0xd0689ade, SUC_lexicon_tags_76 },
    { 0x67689adf, SUC_lexicon_tags_11 },
    { 0x8c89ae0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b289ae3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xada89ae5, SUC_lexicon_tags_15 },
    { 0x1a889ae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37089ae9, SUC_lexicon_tags_15 },
    { 0x15a89aea, SUC_lexicon_tags_55 },
    { 0x73489ae9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5cc89aed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc89af1, SUC_lexicon_tags_27 },
    { 0xf3089af2, SUC_lexicon_tags_25 },
    { 0x4c89af3, SUC_lexicon_tags_44 },
    { 0xf4689af4, SUC_lexicon_tags_15 },
    { 0x3ee89af5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c89afc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3289b00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39489b03, SUC_lexicon_tags_0 },
    { 0x5689b04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90089b06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e89b0b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x47e89b0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd089b12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3489b16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9489b1a, SUC_lexicon_tags_49 },
    { 0xb9689b1b, SUC_lexicon_tags_11 },
    { 0xe6c89b1a, SUC_lexicon_tags_1 },
    { 0x12889b1d, SUC_lexicon_tags_3 },
    { 0x40489b1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38489b23, SUC_lexicon_tags_25 },
    { 0x2889b24, SUC_lexicon_tags_38 },
    { 0x87489b25, SUC_lexicon_tags_17 },
    { 0x85689b25, SUC_lexicon_tags_37 },
    { 0x93c89b27, SUC_lexicon_tags_1 },
    { 0x8a489b25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe89b2d, SUC_lexicon_tags_17 },
    { 0xb0289b2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3289b34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7489b36, SUC_lexicon_tags_35 },
    { 0xfae89b37, SUC_lexicon_tags_97 },
    { 0xffa89b36, SUC_lexicon_tags_13 },
    { 0xf2c89b38, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a89b3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0e89b40, SUC_lexicon_tags_27 },
    { 0x4c289b41, SUC_lexicon_tags_60 },
    { 0xcae89b42, SUC_lexicon_tags_11 },
    { 0xb5289b42, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x10c89b45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87a89b47, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x73489b49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a89b4c, SUC_lexicon_tags_2 },
    { 0xd4889b4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a89b52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce89b56, SUC_lexicon_tags_43 },
    { 0xd8a89b57, SUC_lexicon_tags_10 },
    { 0xdf689b58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c89b5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e889b5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x36889b61, SUC_lexicon_tags_15 },
    { 0xb9289b61, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x14689b64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f889b6f, SUC_lexicon_tags_37 },
    { 0x83a89b70, SUC_lexicon_tags_37 },
    { 0x20289b71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74889b73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1089b75, SUC_lexicon_tags_43 },
    { 0xd8489b75, SUC_lexicon_tags_33 },
    { 0x53489b77, SUC_lexicon_tags_48 },
    { 0x81489b77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a89b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4a89b7e, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0xd5c89b80, SUC_lexicon_tags_25 },
    { 0xd4a89b81, SUC_lexicon_tags_7 },
    { 0x9289b80, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea89b85, SUC_lexicon_tags_38 },
    { 0xa9889b86, SUC_lexicon_tags_25 },
    { 0x10289b87, SUC_lexicon_tags_63 },
    { 0xfa889b85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9889b8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70689b91, SUC_lexicon_tags_1 },
    { 0x52e89b92, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3489b96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72089b9b, SUC_lexicon_tags_1 },
    { 0x9c289b9b, SUC_lexicon_tags_37 },
    { 0x3a489b9b, SUC_lexicon_tags_45 },
    { 0x53c89b9c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5489ba1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd889ba4, SUC_lexicon_tags_45 },
    { 0xd1489ba4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3889ba9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xce089bac, SUC_lexicon_tags_11 },
    { 0xe6889bad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe689baf, SUC_lexicon_tags_23 },
    { 0xa8289bb0, SUC_lexicon_tags_11 },
    { 0x7089bb1, SUC_lexicon_tags_68 },
    { 0x4d089bb1, SUC_lexicon_tags_38 },
    { 0x4da89bb3, SUC_lexicon_tags_15 },
    { 0x97689bb4, SUC_lexicon_tags_28 },
    { 0xd8e89bb5, SUC_lexicon_tags_11 },
    { 0x7e689bb6, SUC_lexicon_tags_16 },
    { 0x12089bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f889bba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1889bbd, SUC_lexicon_tags_57 },
    { 0x10c89bbd, SUC_lexicon_tags_1 },
    { 0xac889bbf, SUC_lexicon_tags_49 },
    { 0x70889bbf, SUC_lexicon_tags_85 },
    { 0xf1089bc1, SUC_lexicon_tags_47 },
    { 0x31a89bc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9689bc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc489bcb, SUC_lexicon_tags_47 },
    { 0xe1489bcc, SUC_lexicon_tags_25 },
    { 0xbc089bcd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20889bd1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e689bd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51a89bd5, SUC_lexicon_tags_56 },
    { 0xfd289bd6, SUC_lexicon_tags_1 },
    { 0xde89bd7, SUC_lexicon_tags_13 },
    { 0x9b089bd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a89bdb, SUC_lexicon_tags_47 },
    { 0x1fa89bdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e89be0, SUC_lexicon_tags_17 },
    { 0xcc489be1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8289be4, SUC_lexicon_tags_17 },
    { 0x89be4, SUC_lexicon_tags_7 },
    { 0x3489be6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe89beb, SUC_lexicon_tags_74 },
    { 0x5f689bec, SUC_lexicon_tags_17 },
    { 0xc6289beb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x33489bf0, SUC_lexicon_tags_14 },
    { 0x9b489bf1, SUC_lexicon_tags_37 },
    { 0x5f289bf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15089bf4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x98689bf6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c89bf9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa2289bfb, SUC_lexicon_tags_48 },
    { 0x19089bfc, SUC_lexicon_tags_23 },
    { 0x70089bfd, SUC_lexicon_tags_11 },
    { 0xb2289bfd, SUC_lexicon_tags_23 },
    { 0xf8089bff, SUC_lexicon_tags_49 },
    { 0xdaa89c00, SUC_lexicon_tags_33 },
    { 0xe3e89c01, SUC_lexicon_tags_34 },
    { 0x68489c01, SUC_lexicon_tags_17 },
    { 0x84a89c03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55489c06, SUC_lexicon_tags_11 },
    { 0xc0689c06, SUC_lexicon_tags_49 },
    { 0xddc89c08, SUC_lexicon_tags_1 },
    { 0xc2689c08, SUC_lexicon_tags_60 },
    { 0xe5089c0a, SUC_lexicon_tags_37 },
    { 0x1e889c09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95289c0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e89c13, SUC_lexicon_tags_49 },
    { 0x72e89c13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc089c16, SUC_lexicon_tags_194 },
    { 0xc4289c17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbda89c19, SUC_lexicon_tags_26 },
    { 0xcae89c1a, SUC_lexicon_tags_12 },
    { 0x62489c1a, SUC_lexicon_tags_17 },
    { 0x18889c1c, SUC_lexicon_tags_1 },
    { 0xaaa89c1d, SUC_lexicon_tags_11 },
    { 0xa6c89c1c, SUC_lexicon_tags_37 },
    { 0x6c689c1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a289c21, SUC_lexicon_tags_6 },
    { 0x7c89c21, SUC_lexicon_tags_55 },
    { 0x13c89c22, SUC_lexicon_tags_1 },
    { 0x45689c24, SUC_lexicon_tags_37 },
    { 0x33889c21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ac89c27, SUC_lexicon_tags_88 },
    { 0x3f289c28, SUC_lexicon_tags_27 },
    { 0x42289c29, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xae689c2c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4689c31, SUC_lexicon_tags_11 },
    { 0xc2c89c31, SUC_lexicon_tags_25 },
    { 0xbe89c33, SUC_lexicon_tags_4 },
    { 0xdae89c34, SUC_lexicon_tags_60 },
    { 0x4089c34, SUC_lexicon_tags_60 },
    { 0x6d689c36, SUC_lexicon_tags_7 },
    { 0xf089c33, SUC_lexicon_tags_10 },
    { 0x55289c38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4689c3a, SUC_lexicon_tags_11 },
    { 0x5ba89c3b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6c289c3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10489c40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7889c44, SUC_lexicon_tags_21 },
    { 0x75a89c45, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c89c4c, SUC_lexicon_tags_2 },
    { 0xa9689c4d, SUC_lexicon_tags_14 },
    { 0x27e89c4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d689c50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e89c56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c89c61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x34489c63, SUC_lexicon_tags_37 },
    { 0xf5289c64, SUC_lexicon_tags_1 },
    { 0x33289c65, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa9089c67, SUC_lexicon_tags_96 },
    { 0xce289c68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c89c6e, SUC_lexicon_tags_63 },
    { 0x84a89c6f, SUC_lexicon_tags_10 },
    { 0x31089c70, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x65289c73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a89c79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1689c7b, SUC_lexicon_tags_37 },
    { 0x5d889c7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a89c7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a889c84, SUC_lexicon_tags_8 },
    { 0x4f289c85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f689c89, SUC_lexicon_tags_11 },
    { 0x44e89c89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5289c8c, SUC_lexicon_tags_7 },
    { 0x2b689c8d, SUC_lexicon_tags_13 },
    { 0x35e89c8e, SUC_lexicon_tags_47 },
    { 0x77489c8d, SUC_lexicon_tags_27 },
    { 0x56489c90, SUC_lexicon_tags_2 },
    { 0x72889c8d, SUC_lexicon_tags_45 },
    { 0xb1089c92, SUC_lexicon_tags_8 },
    { 0xa3889c92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a889c96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1da89c98, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e89c9d, SUC_lexicon_tags_172 },
    { 0xfa889c9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc889ca5, SUC_lexicon_tags_25 },
    { 0x3f489ca6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e489ca8, SUC_lexicon_tags_37 },
    { 0x81289ca9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53a89cab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e89cb1, SUC_lexicon_tags_14 },
    { 0x47489cb2, SUC_lexicon_tags_6 },
    { 0x6f289cb3, SUC_lexicon_tags_49 },
    { 0xa2889cb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e689cb6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8289cb9, SUC_lexicon_tags_45 },
    { 0xa6489cba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8c89cbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a089cc3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1e089cc5, SUC_lexicon_tags_2 },
    { 0x6a289cc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58089cc9, SUC_lexicon_tags_2 },
    { 0xc0c89cca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c89ccf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69889cd2, SUC_lexicon_tags_1 },
    { 0xe9689cd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb289cd5, SUC_lexicon_tags_6 },
    { 0x62a89cd5, SUC_lexicon_tags_17 },
    { 0x17c89cd7, SUC_lexicon_tags_14 },
    { 0xd0889cd8, SUC_lexicon_tags_37 },
    { 0x2a089cd8, SUC_lexicon_tags_49 },
    { 0xb1689cda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd689cdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a89ce0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62889ce2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe89ce5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90089ce8, SUC_lexicon_tags_16 },
    { 0xf4489ce9, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xbf689ceb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4089cf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89289cf6, SUC_lexicon_tags_17 },
    { 0x1b289cf7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee89cfb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce89d01, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5089d07, SUC_lexicon_tags_26 },
    { 0x2fc89d08, SUC_lexicon_tags_1106 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca89d0b, SUC_lexicon_tags_8 },
    { 0x59489d0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5689d0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74089d10, SUC_lexicon_tags_1 },
    { 0xe8e89d11, SUC_lexicon_tags_1 },
    { 0x47289d11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c89d15, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x66a89d17, SUC_lexicon_tags_63 },
    { 0xa8689d18, SUC_lexicon_tags_19 },
    { 0x57489d17, SUC_lexicon_tags_25 },
    { 0x89089d1a, SUC_lexicon_tags_49 },
    { 0x93a89d17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97e89d1d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc5c89d1f, SUC_lexicon_tags_37 },
    { 0x13c89d1f, SUC_lexicon_tags_25 },
    { 0x55089d21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54289d24, SUC_lexicon_tags_37 },
    { 0xe089d25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8489d2a, SUC_lexicon_tags_61 },
    { 0x80c89d2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae89d2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28289d32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe89d38, SUC_lexicon_tags_107 },
    { 0x41a89d39, SUC_lexicon_tags_63 },
    { 0xba089d3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x62889d3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a89d41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda289d43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36689d47, SUC_lexicon_tags_12 },
    { 0xfdc89d48, SUC_lexicon_tags_1 },
    { 0x60a89d49, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27889d4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e89d54, SUC_lexicon_tags_12 },
    { 0x3e289d54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5889d57, SUC_lexicon_tags_63 },
    { 0xbb889d58, SUC_lexicon_tags_25 },
    { 0xc7e89d59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e89d60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54489d64, SUC_lexicon_tags_7 },
    { 0x1e489d65, SUC_lexicon_tags_18 },
    { 0x9689d65, SUC_lexicon_tags_25 },
    { 0xea889d67, SUC_lexicon_tags_12 },
    { 0x52489d64, SUC_lexicon_tags_37 },
    { 0x71689d69, SUC_lexicon_tags_1 },
    { 0xd5c89d6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ba89d6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a489d70, SUC_lexicon_tags_38 },
    { 0x8f289d71, SUC_lexicon_tags_17 },
    { 0xdde89d71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a89d75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b489d77, SUC_lexicon_tags_17 },
    { 0xfb289d78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x43489d7b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdf489d7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e89d83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f689d87, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf0289d89, SUC_lexicon_tags_25 },
    { 0xbec89d8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa89d8e, SUC_lexicon_tags_0 },
    { 0x85289d8f, SUC_lexicon_tags_11 },
    { 0x82089d90, SUC_lexicon_tags_1 },
    { 0x47489d91, SUC_lexicon_tags_1 },
    { 0x6f089d92, SUC_lexicon_tags_37 },
    { 0xc1089d92, SUC_lexicon_tags_10 },
    { 0x8ba89d92, SUC_lexicon_tags_21 },
    { 0x43a89d92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45889d9c, SUC_lexicon_tags_0 },
    { 0xe889d9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e889da2, SUC_lexicon_tags_59 },
    { 0xa9889da2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd889da5, SUC_lexicon_tags_37 },
    { 0xa089da6, SUC_lexicon_tags_8 },
    { 0x89089da7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1289daa, SUC_lexicon_tags_25 },
    { 0x57489dab, SUC_lexicon_tags_7 },
    { 0x4d889dac, SUC_lexicon_tags_33 },
    { 0xe89dac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb0e89daf, SUC_lexicon_tags_11 },
    { 0x36489db0, SUC_lexicon_tags_45 },
    { 0xaac89db1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4089db6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f689dbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e089dbe, SUC_lexicon_tags_49 },
    { 0xd3089dbf, SUC_lexicon_tags_60 },
    { 0x95089dbf, SUC_lexicon_tags_1 },
    { 0x3d689dc1, SUC_lexicon_tags_31 },
    { 0xd4089dbe, SUC_lexicon_tags_158 },
    { 0xa4e89dc3, SUC_lexicon_tags_25 },
    { 0xc3089dc4, SUC_lexicon_tags_17 },
    { 0x8f289dc4, SUC_lexicon_tags_2 },
    { 0xca289dc6, SUC_lexicon_tags_63 },
    { 0x88889dc7, SUC_lexicon_tags_33 },
    { 0xff089dc8, SUC_lexicon_tags_39 },
    { 0x48c89dc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89dcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29489dd2, SUC_lexicon_tags_17 },
    { 0xac89dd3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd089dd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c89dda, SUC_lexicon_tags_17 },
    { 0x53c89dda, SUC_lexicon_tags_6 },
    { 0x6a289ddc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d889de1, SUC_lexicon_tags_63 },
    { 0x1f689de2, SUC_lexicon_tags_107 },
    { 0x93489de1, SUC_lexicon_tags_11 },
    { 0x18489de2, SUC_lexicon_tags_109 },
    { 0xa7e89de2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5d289de7, SUC_lexicon_tags_26 },
    { 0x36a89de8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc089deb, SUC_lexicon_tags_27 },
    { 0x40289dec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7089df0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63089df4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c89df8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76489dfc, SUC_lexicon_tags_14 },
    { 0xcaa89dfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b489dff, SUC_lexicon_tags_27 },
    { 0xf0e89e00, SUC_lexicon_tags_7 },
    { 0x67c89dff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a89e0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7889e0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0889e10, SUC_lexicon_tags_60 },
    { 0xf7089e11, SUC_lexicon_tags_172 },
    { 0x6e889e10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2889e15, SUC_lexicon_tags_11 },
    { 0x50689e16, SUC_lexicon_tags_17 },
    { 0x60a89e17, SUC_lexicon_tags_8 },
    { 0xc5289e18, SUC_lexicon_tags_14 },
    { 0xde689e19, SUC_lexicon_tags_26 },
    { 0xe7089e1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x50089e1c, SUC_lexicon_tags_21 },
    { 0x57289e1d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x52889e1f, SUC_lexicon_tags_60 },
    { 0x7fe89e20, SUC_lexicon_tags_0 },
    { 0x94a89e1f, SUC_lexicon_tags_15 },
    { 0x8b689e21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b089e24, SUC_lexicon_tags_17 },
    { 0x6d289e25, SUC_lexicon_tags_4 },
    { 0x82c89e26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22a89e28, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x94889e2b, SUC_lexicon_tags_49 },
    { 0x24089e2c, SUC_lexicon_tags_28 },
    { 0xde889e2d, SUC_lexicon_tags_14 },
    { 0xda489e2e, SUC_lexicon_tags_11 },
    { 0xa7e89e2e, SUC_lexicon_tags_0 },
    { 0xdac89e30, SUC_lexicon_tags_14 },
    { 0x96a89e2d, SUC_lexicon_tags_0 },
    { 0xe3289e2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e89e36, SUC_lexicon_tags_17 },
    { 0xe6e89e37, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xac889e3a, SUC_lexicon_tags_35 },
    { 0xe4089e3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c89e3e, SUC_lexicon_tags_32 },
    { 0x889e3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80c89e41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca89e45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb089e47, SUC_lexicon_tags_29 },
    { 0x6c89e47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be89e4f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a689e53, SUC_lexicon_tags_31 },
    { 0x76089e53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57289e56, SUC_lexicon_tags_15 },
    { 0x98089e57, SUC_lexicon_tags_2 },
    { 0xd7889e58, SUC_lexicon_tags_26 },
    { 0x63889e58, SUC_lexicon_tags_26 },
    { 0x3b289e5a, SUC_lexicon_tags_13 },
    { 0xd7a89e56, SUC_lexicon_tags_61 },
    { 0xfda89e57, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d889e60, SUC_lexicon_tags_1 },
    { 0x68689e60, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3889e64, SUC_lexicon_tags_49 },
    { 0xb4e89e65, SUC_lexicon_tags_77 },
    { 0xc6e89e66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61889e68, SUC_lexicon_tags_17 },
    { 0xd7489e69, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6cc89e6b, SUC_lexicon_tags_178 },
    { 0xfda89e6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e89e70, SUC_lexicon_tags_7 },
    { 0x16e89e71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1dc89e73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69489e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc089e7c, SUC_lexicon_tags_9 },
    { 0x76289e7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8489e7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9089e85, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e89e8c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2089e94, SUC_lexicon_tags_91 },
    { 0xafa89e95, SUC_lexicon_tags_11 },
    { 0x77c89e96, SUC_lexicon_tags_25 },
    { 0xf9a89e97, SUC_lexicon_tags_19 },
    { 0xf2489e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec89e9c, SUC_lexicon_tags_42 },
    { 0xa6489e9d, SUC_lexicon_tags_7 },
    { 0x58689e9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd689ea1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbbe89ea3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xace89ea6, SUC_lexicon_tags_37 },
    { 0x88e89ea6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1689eab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7a89ead, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc89eb0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a89eb3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d889eb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45889eb8, SUC_lexicon_tags_11 },
    { 0x10889eb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6a89ebb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17289ebd, SUC_lexicon_tags_37 },
    { 0xd7889ebe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8089ec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d089ec3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb689ec9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f689ed7, SUC_lexicon_tags_29 },
    { 0x5a89ed7, SUC_lexicon_tags_48 },
    { 0x50c89ed9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc89ede, SUC_lexicon_tags_7 },
    { 0x90689edf, SUC_lexicon_tags_29 },
    { 0xad289ede, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e489ef0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17a89ef2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4089efa, SUC_lexicon_tags_17 },
    { 0xaac89efb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a689efd, SUC_lexicon_tags_11 },
    { 0x1b489efd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf689f02, SUC_lexicon_tags_47 },
    { 0xaae89f03, SUC_lexicon_tags_37 },
    { 0xd3689f02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f289f10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0289f13, SUC_lexicon_tags_2 },
    { 0x53489f14, SUC_lexicon_tags_17 },
    { 0x30889f14, SUC_lexicon_tags_25 },
    { 0xb7e89f14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc89f1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65689f22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a89f26, SUC_lexicon_tags_11 },
    { 0x78a89f27, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6089f2c, SUC_lexicon_tags_5 },
    { 0x81e89f2d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4889f33, SUC_lexicon_tags_25 },
    { 0x4b689f33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x84489f36, SUC_lexicon_tags_7 },
    { 0x16489f37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a689f42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a689f47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f089f4a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d089f51, SUC_lexicon_tags_15 },
    { 0x16c89f52, SUC_lexicon_tags_113 },
    { 0x8f489f53, SUC_lexicon_tags_17 },
    { 0x29489f52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa889f57, SUC_lexicon_tags_39 },
    { 0x55289f57, SUC_lexicon_tags_2 },
    { 0x2de89f57, SUC_lexicon_tags_79 },
    { 0xe889f58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec089f5c, SUC_lexicon_tags_17 },
    { 0xf8e89f5d, SUC_lexicon_tags_1 },
    { 0x79e89f5e, SUC_lexicon_tags_66 },
    { 0xf0489f5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c89f62, SUC_lexicon_tags_2 },
    { 0x40a89f63, SUC_lexicon_tags_7 },
    { 0x6d489f63, SUC_lexicon_tags_27 },
    { 0x85089f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2289f69, SUC_lexicon_tags_2 },
    { 0xd5089f69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e89f6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87889f75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c889f78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d489f7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a889f89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee489f8d, SUC_lexicon_tags_30 },
    { 0xe9089f8e, SUC_lexicon_tags_5 },
    { 0xeb089f8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2d889f91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f889f99, SUC_lexicon_tags_1 },
    { 0x49489f9a, SUC_lexicon_tags_65 },
    { 0xdd289f9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x97489f9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63689fa0, SUC_lexicon_tags_19 },
    { 0x57889fa1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9489fa3, SUC_lexicon_tags_1 },
    { 0x65e89fa3, SUC_lexicon_tags_33 },
    { 0xa6889fa3, SUC_lexicon_tags_40 },
    { 0x78c89fa6, SUC_lexicon_tags_26 },
    { 0xb0489fa7, SUC_lexicon_tags_194 },
    { 0xde289fa6, SUC_lexicon_tags_2 },
    { 0xa4489fa5, SUC_lexicon_tags_45 },
    { 0xf5e89faa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa89faf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10289fb1, SUC_lexicon_tags_27 },
    { 0x18489fb2, SUC_lexicon_tags_29 },
    { 0x66289fb3, SUC_lexicon_tags_1 },
    { 0xde289fb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f289fb6, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc89fbb, SUC_lexicon_tags_60 },
    { 0x47e89fbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65289fbf, SUC_lexicon_tags_15 },
    { 0xada89fc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc889fc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc889fc6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x92089fc8, SUC_lexicon_tags_68 },
    { 0xa5889fc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x46a89fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c489fcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64889fd1, SUC_lexicon_tags_37 },
    { 0x53c89fd2, SUC_lexicon_tags_27 },
    { 0x56889fd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b089fd8, SUC_lexicon_tags_43 },
    { 0x71689fd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x68689fdc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2c89fde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7ee89fe0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1089fe4, SUC_lexicon_tags_26 },
    { 0xc8e89fe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a89fe9, SUC_lexicon_tags_48 },
    { 0x38289fea, SUC_lexicon_tags_25 },
    { 0x40489feb, SUC_lexicon_tags_17 },
    { 0x8e89feb, SUC_lexicon_tags_63 },
    { 0x6889feb, SUC_lexicon_tags_31 },
    { 0x6cc89fed, SUC_lexicon_tags_3 },
    { 0xf1689fee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66e89ff1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6089ff3, SUC_lexicon_tags_1 },
    { 0xbd689ff4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14489ff8, SUC_lexicon_tags_6 },
    { 0x1ec89ff8, SUC_lexicon_tags_38 },
    { 0xa0289ffa, SUC_lexicon_tags_2 },
    { 0x14689ffa, SUC_lexicon_tags_19 },
    { 0xe5889ff8, SUC_lexicon_tags_37 },
    { 0x92a89ffd, SUC_lexicon_tags_31 },
    { 0xa6089ffd, SUC_lexicon_tags_1 },
    { 0x3b889fff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45a8a001, SUC_lexicon_tags_17 },
    { 0xfc08a002, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b28a005, SUC_lexicon_tags_14 },
    { 0xf528a006, SUC_lexicon_tags_11 },
    { 0x3688a007, SUC_lexicon_tags_63 },
    { 0x108a008, SUC_lexicon_tags_11 },
    { 0xe7e8a009, SUC_lexicon_tags_37 },
    { 0x8448a00a, SUC_lexicon_tags_1 },
    { 0x9b48a00b, SUC_lexicon_tags_65 },
    { 0x5788a00c, SUC_lexicon_tags_61 },
    { 0x96a8a00d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7628a010, SUC_lexicon_tags_8 },
    { 0xc2c8a010, SUC_lexicon_tags_8 },
    { 0x8a28a012, SUC_lexicon_tags_12 },
    { 0x5c28a013, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8e8a015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6948a01a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f08a01c, SUC_lexicon_tags_23 },
    { 0x8a08a01d, SUC_lexicon_tags_2 },
    { 0x288a01e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd528a028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d28a034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f68a039, SUC_lexicon_tags_7 },
    { 0xbda8a03a, SUC_lexicon_tags_0 },
    { 0xc028a03a, SUC_lexicon_tags_63 },
    { 0x7228a03c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba8a041, SUC_lexicon_tags_6 },
    { 0xaa88a041, SUC_lexicon_tags_23 },
    { 0x78a8a042, SUC_lexicon_tags_38 },
    { 0x44a8a044, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae68a04d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc8a054, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848a058, SUC_lexicon_tags_11 },
    { 0x40a8a059, SUC_lexicon_tags_37 },
    { 0x2b68a059, SUC_lexicon_tags_34 },
    { 0x8728a05b, SUC_lexicon_tags_37 },
    { 0xde48a05b, SUC_lexicon_tags_7 },
    { 0xb8e8a05d, SUC_lexicon_tags_37 },
    { 0xa028a05e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef28a062, SUC_lexicon_tags_8 },
    { 0xfec8a063, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce8a067, SUC_lexicon_tags_11 },
    { 0x69a8a068, SUC_lexicon_tags_37 },
    { 0xcea8a067, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c8a06d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x78a8a06f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa48a072, SUC_lexicon_tags_17 },
    { 0x2048a073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd588a075, SUC_lexicon_tags_46 },
    { 0x53a8a075, SUC_lexicon_tags_1 },
    { 0xac48a077, SUC_lexicon_tags_49 },
    { 0x47a8a077, SUC_lexicon_tags_11 },
    { 0x8c68a078, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac8a07c, SUC_lexicon_tags_48 },
    { 0x288a07d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c8a081, SUC_lexicon_tags_23 },
    { 0xb48a082, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8688a085, SUC_lexicon_tags_69 },
    { 0x5868a086, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8a089, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd068a08c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc88a098, SUC_lexicon_tags_60 },
    { 0x1d88a098, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e8a09c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d08a09f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e8a0a4, SUC_lexicon_tags_45 },
    { 0x9908a0a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b28a0a7, SUC_lexicon_tags_43 },
    { 0xbfa8a0a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe008a0aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe548a0b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcc8a0b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f08a0b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c8a0b8, SUC_lexicon_tags_68 },
    { 0x7ba8a0b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828a0bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb768a0c3, SUC_lexicon_tags_39 },
    { 0xe9a8a0c3, SUC_lexicon_tags_34 },
    { 0xc008a0c5, SUC_lexicon_tags_2 },
    { 0x1f08a0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8a0d1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e8a0d7, SUC_lexicon_tags_13 },
    { 0x88a8a0d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa808a0e2, SUC_lexicon_tags_7 },
    { 0xe4a8a0e3, SUC_lexicon_tags_79 },
    { 0x2608a0e4, SUC_lexicon_tags_11 },
    { 0xfbe8a0e5, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xf408a0e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e8a0ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2128a0ef, SUC_lexicon_tags_2 },
    { 0x428a0f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa628a0f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8a0f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4048a0f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc8a0ff, SUC_lexicon_tags_79 },
    { 0x1ea8a100, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe908a103, SUC_lexicon_tags_49 },
    { 0x5ce8a103, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8a109, SUC_lexicon_tags_12 },
    { 0x9ce8a10a, SUC_lexicon_tags_38 },
    { 0xe808a109, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb248a117, SUC_lexicon_tags_2 },
    { 0x2408a118, SUC_lexicon_tags_10 },
    { 0x268a119, SUC_lexicon_tags_103 },
    { 0x128a118, SUC_lexicon_tags_11 },
    { 0x6fe8a11b, SUC_lexicon_tags_11 },
    { 0x6d68a11b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3688a11e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c8a121, SUC_lexicon_tags_11 },
    { 0xf0e8a122, SUC_lexicon_tags_57 },
    { 0xb8e8a123, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc268a129, SUC_lexicon_tags_40 },
    { 0xa5c8a12a, SUC_lexicon_tags_25 },
    { 0xa4c8a12b, SUC_lexicon_tags_48 },
    { 0xaf88a12c, SUC_lexicon_tags_37 },
    { 0x9d08a12d, SUC_lexicon_tags_7 },
    { 0x61a8a12e, SUC_lexicon_tags_25 },
    { 0xbe8a12b, SUC_lexicon_tags_2 },
    { 0xf748a12b, SUC_lexicon_tags_1 },
    { 0xdf68a131, SUC_lexicon_tags_11 },
    { 0xb488a132, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8cc8a134, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe768a13b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8068a13d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a8a141, SUC_lexicon_tags_25 },
    { 0xba8a142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80c8a144, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b68a14c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d68a14f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a68a152, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e8a158, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6908a15a, SUC_lexicon_tags_45 },
    { 0xb5c8a15b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e8a166, SUC_lexicon_tags_29 },
    { 0x36a8a167, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5808a16b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1008a16d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec8a170, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec28a179, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc8a17c, SUC_lexicon_tags_7 },
    { 0x9f28a17d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d28a180, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c8a188, SUC_lexicon_tags_47 },
    { 0x72e8a189, SUC_lexicon_tags_7 },
    { 0xb848a189, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd48a18d, SUC_lexicon_tags_25 },
    { 0x8948a18e, SUC_lexicon_tags_12 },
    { 0xc128a18f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3028a198, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe8a19e, SUC_lexicon_tags_25 },
    { 0x8c8a19e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7048a1a1, SUC_lexicon_tags_69 },
    { 0x7d68a1a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a8a1a5, SUC_lexicon_tags_27 },
    { 0xd5c8a1a5, SUC_lexicon_tags_1 },
    { 0xea48a1a7, SUC_lexicon_tags_17 },
    { 0xc888a1a5, SUC_lexicon_tags_1 },
    { 0x9108a1a9, SUC_lexicon_tags_37 },
    { 0xba08a1a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e48a1ac, SUC_lexicon_tags_43 },
    { 0x87c8a1ad, SUC_lexicon_tags_40 },
    { 0xf268a1ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8a1b1, SUC_lexicon_tags_13 },
    { 0x4488a1b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2868a1b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9628a1b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74a8a1bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d28a1c0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x188a1c2, SUC_lexicon_tags_19 },
    { 0x10c8a1c3, SUC_lexicon_tags_37 },
    { 0xac8a1c3, SUC_lexicon_tags_68 },
    { 0x5168a1c5, SUC_lexicon_tags_12 },
    { 0xbe68a1c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x95a8a1c8, SUC_lexicon_tags_34 },
    { 0x46e8a1c9, SUC_lexicon_tags_12 },
    { 0xff88a1ca, SUC_lexicon_tags_25 },
    { 0xcde8a1cb, SUC_lexicon_tags_25 },
    { 0x2808a1ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda08a1d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5108a1d4, SUC_lexicon_tags_124 },
    { 0x2008a1d4, SUC_lexicon_tags_12 },
    { 0x1108a1d6, SUC_lexicon_tags_17 },
    { 0x1808a1d7, SUC_lexicon_tags_27 },
    { 0x7608a1d8, SUC_lexicon_tags_37 },
    { 0xdf68a1d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5cc8a1db, SUC_lexicon_tags_40 },
    { 0x4868a1dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf68a1de, SUC_lexicon_tags_33 },
    { 0xf988a1de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc568a1e3, SUC_lexicon_tags_25 },
    { 0xe588a1e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2048a1e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548a1ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaba8a1f1, SUC_lexicon_tags_2 },
    { 0x3988a1f2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe928a1f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe708a1fa, SUC_lexicon_tags_45 },
    { 0xd708a1fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba8a1fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf668a203, SUC_lexicon_tags_15 },
    { 0xc7a8a204, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f48a20c, SUC_lexicon_tags_14 },
    { 0xdc48a20c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x848a20f, SUC_lexicon_tags_11 },
    { 0x4a8a210, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x2928a212, SUC_lexicon_tags_4 },
    { 0xc5c8a213, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f28a216, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a8a219, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e8a21c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc068a21e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6e48a220, SUC_lexicon_tags_1 },
    { 0xdf28a221, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc8a228, SUC_lexicon_tags_11 },
    { 0xcb48a229, SUC_lexicon_tags_32 },
    { 0x4ce8a22a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf488a22d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb248a230, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2528a236, SUC_lexicon_tags_56 },
    { 0xa288a237, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c28a23a, SUC_lexicon_tags_47 },
    { 0x5468a23a, SUC_lexicon_tags_15 },
    { 0x53a8a23c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e8a23f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a8a245, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc8a249, SUC_lexicon_tags_11 },
    { 0xa2a8a249, SUC_lexicon_tags_8 },
    { 0x5088a24b, SUC_lexicon_tags_33 },
    { 0xbf48a24c, SUC_lexicon_tags_2 },
    { 0xd548a24d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd748a254, SUC_lexicon_tags_2 },
    { 0x5668a254, SUC_lexicon_tags_12 },
    { 0x6b48a256, SUC_lexicon_tags_43 },
    { 0x3968a255, SUC_lexicon_tags_7 },
    { 0xeba8a256, SUC_lexicon_tags_1 },
    { 0x8d68a259, SUC_lexicon_tags_45 },
    { 0xbe28a258, SUC_lexicon_tags_33 },
    { 0x3fc8a259, SUC_lexicon_tags_33 },
    { 0xaca8a25c, SUC_lexicon_tags_1 },
    { 0xfa8a25d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fc8a25f, SUC_lexicon_tags_4 },
    { 0xbbe8a260, SUC_lexicon_tags_2 },
    { 0x228a261, SUC_lexicon_tags_423 },
    { 0, NULL },
    { 0x75e8a263, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c8a26b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac68a26e, SUC_lexicon_tags_10 },
    { 0xd1c8a26e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb448a277, SUC_lexicon_tags_46 },
    { 0xf1c8a277, SUC_lexicon_tags_6 },
    { 0xdfc8a278, SUC_lexicon_tags_2 },
    { 0xe508a27a, SUC_lexicon_tags_7 },
    { 0xe5e8a27b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e8a27f, SUC_lexicon_tags_68 },
    { 0x3ae8a280, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc8a287, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cc8a289, SUC_lexicon_tags_47 },
    { 0x60c8a289, SUC_lexicon_tags_11 },
    { 0x4148a28b, SUC_lexicon_tags_23 },
    { 0x6128a28c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb788a291, SUC_lexicon_tags_8 },
    { 0xb1c8a292, SUC_lexicon_tags_14 },
    { 0xa6e8a292, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a8a296, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x27c8a298, SUC_lexicon_tags_12 },
    { 0x788a299, SUC_lexicon_tags_1 },
    { 0xa888a29a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb268a29c, SUC_lexicon_tags_2 },
    { 0x75c8a29d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4828a29f, SUC_lexicon_tags_23 },
    { 0x1948a2a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9288a2a7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2148a2a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7768a2ab, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb508a2ad, SUC_lexicon_tags_11 },
    { 0x1a88a2ae, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a48a2b1, SUC_lexicon_tags_45 },
    { 0x3608a2b2, SUC_lexicon_tags_2 },
    { 0xe648a2b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe88a2be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a8a2c1, SUC_lexicon_tags_4 },
    { 0xf828a2c2, SUC_lexicon_tags_66 },
    { 0xf028a2c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a8a2c6, SUC_lexicon_tags_38 },
    { 0xa008a2c7, SUC_lexicon_tags_43 },
    { 0x3ac8a2c8, SUC_lexicon_tags_84 },
    { 0xd9c8a2c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaca8a2cb, SUC_lexicon_tags_2 },
    { 0xac08a2cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6308a2ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa048a2d3, SUC_lexicon_tags_17 },
    { 0x1e08a2d4, SUC_lexicon_tags_68 },
    { 0x9c48a2d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e28a2da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f08a2df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a8a2e4, SUC_lexicon_tags_43 },
    { 0x8fc8a2e4, SUC_lexicon_tags_11 },
    { 0xa608a2e6, SUC_lexicon_tags_17 },
    { 0x2708a2e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f08a2e9, SUC_lexicon_tags_7 },
    { 0x5ec8a2e9, SUC_lexicon_tags_68 },
    { 0x8ca8a2e9, SUC_lexicon_tags_14 },
    { 0x1b08a2ec, SUC_lexicon_tags_15 },
    { 0x1428a2ed, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xcfa8a2ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6388a2f3, SUC_lexicon_tags_354 },
    { 0x3f48a2f4, SUC_lexicon_tags_37 },
    { 0xd4e8a2f5, SUC_lexicon_tags_6 },
    { 0x4fc8a2f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4608a2f9, SUC_lexicon_tags_19 },
    { 0x58a8a2f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9888a302, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba8a305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa968a309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb328a30b, SUC_lexicon_tags_23 },
    { 0x2968a30c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a48a311, SUC_lexicon_tags_11 },
    { 0xd3a8a312, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6948a314, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe408a319, SUC_lexicon_tags_17 },
    { 0xd3c8a31a, SUC_lexicon_tags_16 },
    { 0x57c8a31b, SUC_lexicon_tags_7 },
    { 0xd9a8a31a, SUC_lexicon_tags_2 },
    { 0x5ee8a31d, SUC_lexicon_tags_14 },
    { 0x9868a31b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2828a321, SUC_lexicon_tags_37 },
    { 0x5d68a322, SUC_lexicon_tags_60 },
    { 0x8168a323, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9228a326, SUC_lexicon_tags_61 },
    { 0x2bc8a326, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee8a32b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc308a32d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a8a335, SUC_lexicon_tags_9 },
    { 0xec08a336, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e8a33c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8d08a33e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde8a340, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68a34a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb68a34c, SUC_lexicon_tags_11 },
    { 0xafa8a34d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb168a350, SUC_lexicon_tags_43 },
    { 0x7348a350, SUC_lexicon_tags_21 },
    { 0x6c28a350, SUC_lexicon_tags_14 },
    { 0xc108a353, SUC_lexicon_tags_1 },
    { 0x3388a354, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd648a359, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffa8a35b, SUC_lexicon_tags_12 },
    { 0x9f48a35c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ee8a35e, SUC_lexicon_tags_1 },
    { 0x6088a35f, SUC_lexicon_tags_9 },
    { 0x1468a35e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef08a366, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf28a368, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3e48a36a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9fe8a36c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c08a36f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a48a374, SUC_lexicon_tags_39 },
    { 0x50e8a375, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a8a378, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4448a37b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd028a37f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4528a382, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe548a385, SUC_lexicon_tags_25 },
    { 0x3548a386, SUC_lexicon_tags_11 },
    { 0x9f08a386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa148a38c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf108a391, SUC_lexicon_tags_48 },
    { 0xba8a392, SUC_lexicon_tags_12 },
    { 0x5dc8a392, SUC_lexicon_tags_2 },
    { 0x3208a394, SUC_lexicon_tags_17 },
    { 0x75a8a392, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1228a399, SUC_lexicon_tags_6 },
    { 0x6f48a399, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d48a39c, SUC_lexicon_tags_38 },
    { 0x2ec8a39d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c8a3a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed28a3a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c8a3ac, SUC_lexicon_tags_1 },
    { 0xbe88a3ad, SUC_lexicon_tags_47 },
    { 0x7808a3ae, SUC_lexicon_tags_148 },
    { 0xe8c8a3ae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe888a3b1, SUC_lexicon_tags_0 },
    { 0xac68a3b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa8a3b8, SUC_lexicon_tags_17 },
    { 0x1668a3b9, SUC_lexicon_tags_37 },
    { 0x2d88a3b9, SUC_lexicon_tags_29 },
    { 0xec88a3b9, SUC_lexicon_tags_21 },
    { 0x7408a3ba, SUC_lexicon_tags_0 },
    { 0x988a3bb, SUC_lexicon_tags_25 },
    { 0x99a8a3be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x848a3c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e8a3c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c08a3c9, SUC_lexicon_tags_21 },
    { 0xb5c8a3c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6668a3cc, SUC_lexicon_tags_2 },
    { 0xcea8a3cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da8a3d2, SUC_lexicon_tags_1 },
    { 0xbd68a3d3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb548a3d7, SUC_lexicon_tags_25 },
    { 0xd628a3d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f28a3db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2ea8a3dd, SUC_lexicon_tags_55 },
    { 0x7268a3dd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa628a3e3, SUC_lexicon_tags_23 },
    { 0x1fa8a3e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x10c8a3e6, SUC_lexicon_tags_14 },
    { 0xd988a3e6, SUC_lexicon_tags_37 },
    { 0x2128a3e7, SUC_lexicon_tags_1309 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a8a3ed, SUC_lexicon_tags_45 },
    { 0x3128a3ee, SUC_lexicon_tags_25 },
    { 0x1c08a3ef, SUC_lexicon_tags_11 },
    { 0xfaa8a3f0, SUC_lexicon_tags_32 },
    { 0xd4c8a3f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b68a3f3, SUC_lexicon_tags_14 },
    { 0xea28a3f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2968a3f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a8a3fa, SUC_lexicon_tags_37 },
    { 0x4e28a3fb, SUC_lexicon_tags_49 },
    { 0x6f08a3fc, SUC_lexicon_tags_6 },
    { 0x9528a3fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c8a404, SUC_lexicon_tags_12 },
    { 0x7028a405, SUC_lexicon_tags_1 },
    { 0x6e28a405, SUC_lexicon_tags_37 },
    { 0x4c28a407, SUC_lexicon_tags_0 },
    { 0x80a8a408, SUC_lexicon_tags_9 },
    { 0xf248a409, SUC_lexicon_tags_2 },
    { 0xc0e8a405, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x448a40d, SUC_lexicon_tags_36 },
    { 0xfb68a40e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7088a413, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9dc8a415, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368a423, SUC_lexicon_tags_37 },
    { 0xe168a424, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3768a428, SUC_lexicon_tags_37 },
    { 0xc428a429, SUC_lexicon_tags_7 },
    { 0xb188a42a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8668a42c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6468a430, SUC_lexicon_tags_55 },
    { 0x57c8a431, SUC_lexicon_tags_2 },
    { 0xc108a432, SUC_lexicon_tags_25 },
    { 0xba28a430, SUC_lexicon_tags_10 },
    { 0x3b28a433, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b48a437, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8a43d, SUC_lexicon_tags_47 },
    { 0x5108a43d, SUC_lexicon_tags_7 },
    { 0x2068a43f, SUC_lexicon_tags_6 },
    { 0x85a8a43e, SUC_lexicon_tags_25 },
    { 0xd2e8a441, SUC_lexicon_tags_33 },
    { 0xcfa8a442, SUC_lexicon_tags_12 },
    { 0x6ee8a442, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4b88a445, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa188a44c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8148a450, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb448a452, SUC_lexicon_tags_53 },
    { 0x9128a452, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce8a455, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc668a457, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88a45f, SUC_lexicon_tags_39 },
    { 0x6168a45f, SUC_lexicon_tags_27 },
    { 0x5c68a460, SUC_lexicon_tags_11 },
    { 0x8448a461, SUC_lexicon_tags_25 },
    { 0x2908a463, SUC_lexicon_tags_31 },
    { 0x4e68a45f, SUC_lexicon_tags_31 },
    { 0xf868a465, SUC_lexicon_tags_2 },
    { 0x4aa8a462, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2768a469, SUC_lexicon_tags_17 },
    { 0x85a8a46a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a8a46f, SUC_lexicon_tags_9 },
    { 0xc1e8a46f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac8a476, SUC_lexicon_tags_11 },
    { 0xa748a477, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafc8a479, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2168a47d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd88a480, SUC_lexicon_tags_47 },
    { 0xdcc8a480, SUC_lexicon_tags_38 },
    { 0xb368a482, SUC_lexicon_tags_37 },
    { 0x7da8a480, SUC_lexicon_tags_10 },
    { 0xc8c8a484, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c28a487, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e8a48b, SUC_lexicon_tags_19 },
    { 0xfbe8a48c, SUC_lexicon_tags_63 },
    { 0x41a8a48c, SUC_lexicon_tags_17 },
    { 0x3088a48e, SUC_lexicon_tags_24 },
    { 0xb2e8a48c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe3c8a491, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a8a494, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd48a497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf048a49a, SUC_lexicon_tags_61 },
    { 0xd868a49b, SUC_lexicon_tags_15 },
    { 0x3448a49a, SUC_lexicon_tags_0 },
    { 0x3308a49c, SUC_lexicon_tags_11 },
    { 0xd828a49e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2368a4ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ec8a4ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c8a4b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4988a4b4, SUC_lexicon_tags_35 },
    { 0x12e8a4b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a28a4b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c08a4c6, SUC_lexicon_tags_25 },
    { 0x5a88a4c7, SUC_lexicon_tags_15 },
    { 0xb6e8a4c8, SUC_lexicon_tags_4 },
    { 0xf588a4c8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e8a4cd, SUC_lexicon_tags_127 },
    { 0x7048a4ce, SUC_lexicon_tags_60 },
    { 0x61e8a4ce, SUC_lexicon_tags_3 },
    { 0x8348a4ce, SUC_lexicon_tags_8 },
    { 0x4088a4cd, SUC_lexicon_tags_0 },
    { 0xee68a4d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5968a4d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ea8a4d7, SUC_lexicon_tags_27 },
    { 0xaf88a4d8, SUC_lexicon_tags_49 },
    { 0x468a4d9, SUC_lexicon_tags_10 },
    { 0x3ec8a4da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a8a4df, SUC_lexicon_tags_34 },
    { 0xedc8a4e0, SUC_lexicon_tags_118 },
    { 0x1568a4e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61c8a4e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb348a4e9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee8a4ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e8a4f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f28a4f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc228a4f5, SUC_lexicon_tags_2 },
    { 0xac08a4f6, SUC_lexicon_tags_19 },
    { 0x92e8a4f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee8a4fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2728a501, SUC_lexicon_tags_36 },
    { 0x6408a502, SUC_lexicon_tags_8 },
    { 0x4b48a502, SUC_lexicon_tags_27 },
    { 0xcba8a502, SUC_lexicon_tags_57 },
    { 0xdb28a502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff08a509, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa208a50b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c8a511, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f28a516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8a51c, SUC_lexicon_tags_39 },
    { 0xe8a8a51d, SUC_lexicon_tags_1 },
    { 0x5328a51d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc228a522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf28a524, SUC_lexicon_tags_44 },
    { 0xd9a8a525, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f08a52f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa168a533, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f28a538, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9208a53c, SUC_lexicon_tags_64 },
    { 0x4088a53d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a8a543, SUC_lexicon_tags_1 },
    { 0xb388a543, SUC_lexicon_tags_31 },
    { 0x2728a545, SUC_lexicon_tags_37 },
    { 0xc08a546, SUC_lexicon_tags_47 },
    { 0xecc8a543, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8f88a549, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5808a54e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc8a550, SUC_lexicon_tags_87 },
    { 0xdb88a550, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a8a555, SUC_lexicon_tags_87 },
    { 0xa068a556, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x6988a558, SUC_lexicon_tags_11 },
    { 0x7828a559, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef08a560, SUC_lexicon_tags_9 },
    { 0x9e08a560, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae8a564, SUC_lexicon_tags_47 },
    { 0xca68a565, SUC_lexicon_tags_38 },
    { 0x6628a566, SUC_lexicon_tags_1 },
    { 0xfc88a566, SUC_lexicon_tags_37 },
    { 0xabe8a567, SUC_lexicon_tags_12 },
    { 0x59c8a568, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a8a56e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9928a570, SUC_lexicon_tags_12 },
    { 0x8c68a570, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb88a57a, SUC_lexicon_tags_2 },
    { 0x82c8a57b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e8a57e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc6c8a580, SUC_lexicon_tags_0 },
    { 0x37e8a581, SUC_lexicon_tags_7 },
    { 0xd808a582, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2068a584, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x33e8a586, SUC_lexicon_tags_57 },
    { 0xaa68a587, SUC_lexicon_tags_37 },
    { 0x1e68a587, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9608a594, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e8a59a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6788a5a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3be8a5a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa208a5a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e8a5ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce8a5b1, SUC_lexicon_tags_11 },
    { 0xa028a5b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf908a5b6, SUC_lexicon_tags_6 },
    { 0xd7e8a5b7, SUC_lexicon_tags_12 },
    { 0x9548a5b8, SUC_lexicon_tags_11 },
    { 0x12c8a5b9, SUC_lexicon_tags_1 },
    { 0xcbe8a5ba, SUC_lexicon_tags_49 },
    { 0x18c8a5b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c8a5be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf068a5c2, SUC_lexicon_tags_1 },
    { 0x6c8a5c3, SUC_lexicon_tags_7 },
    { 0x3e8a5c3, SUC_lexicon_tags_1 },
    { 0xe488a5c5, SUC_lexicon_tags_37 },
    { 0x37e8a5c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4968a5cc, SUC_lexicon_tags_68 },
    { 0xa3e8a5cd, SUC_lexicon_tags_17 },
    { 0xf7c8a5ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c8a5d4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd748a5dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc908a5de, SUC_lexicon_tags_15 },
    { 0x8728a5df, SUC_lexicon_tags_2 },
    { 0x9d28a5e0, SUC_lexicon_tags_14 },
    { 0x9aa8a5e1, SUC_lexicon_tags_87 },
    { 0xdac8a5e2, SUC_lexicon_tags_14 },
    { 0x85e8a5df, SUC_lexicon_tags_25 },
    { 0xa488a5de, SUC_lexicon_tags_38 },
    { 0x708a5e1, SUC_lexicon_tags_38 },
    { 0xf128a5e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4e88a5e8, SUC_lexicon_tags_33 },
    { 0x64c8a5e8, SUC_lexicon_tags_7 },
    { 0x21e8a5ea, SUC_lexicon_tags_10 },
    { 0x5268a5ea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e68a5ef, SUC_lexicon_tags_0 },
    { 0x7688a5f0, SUC_lexicon_tags_17 },
    { 0x5708a5f0, SUC_lexicon_tags_15 },
    { 0xb3c8a5f0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c8a5f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae68a5f7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x17e8a5f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1708a5fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaec8a600, SUC_lexicon_tags_15 },
    { 0x1548a600, SUC_lexicon_tags_35 },
    { 0xd988a602, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c8a60c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c8a60f, SUC_lexicon_tags_33 },
    { 0x2d28a610, SUC_lexicon_tags_47 },
    { 0x5c28a60f, SUC_lexicon_tags_49 },
    { 0xfea8a612, SUC_lexicon_tags_12 },
    { 0x7108a613, SUC_lexicon_tags_1 },
    { 0x8888a610, SUC_lexicon_tags_572 },
    { 0x2748a612, SUC_lexicon_tags_7 },
    { 0x468a616, SUC_lexicon_tags_37 },
    { 0x4428a616, SUC_lexicon_tags_2 },
    { 0xbbe8a618, SUC_lexicon_tags_12 },
    { 0x8788a615, SUC_lexicon_tags_77 },
    { 0x3048a61a, SUC_lexicon_tags_1 },
    { 0x1ee8a61b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a8a61e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c8a626, SUC_lexicon_tags_14 },
    { 0x1928a627, SUC_lexicon_tags_25 },
    { 0xfdc8a626, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c8a62e, SUC_lexicon_tags_29 },
    { 0x9ac8a62f, SUC_lexicon_tags_46 },
    { 0xb568a62f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec8a633, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57c8a635, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce48a63e, SUC_lexicon_tags_1 },
    { 0x4b48a63e, SUC_lexicon_tags_37 },
    { 0xaa88a63e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb48a648, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb248a64a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe808a64c, SUC_lexicon_tags_11 },
    { 0xf5a8a64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a68a64f, SUC_lexicon_tags_37 },
    { 0x6228a650, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f88a653, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba28a657, SUC_lexicon_tags_49 },
    { 0x9628a658, SUC_lexicon_tags_57 },
    { 0x2368a659, SUC_lexicon_tags_37 },
    { 0x47e8a65a, SUC_lexicon_tags_17 },
    { 0x1408a65b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf548a65d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b88a660, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4548a664, SUC_lexicon_tags_0 },
    { 0x4888a665, SUC_lexicon_tags_1 },
    { 0x5c8a665, SUC_lexicon_tags_65 },
    { 0xd6a8a664, SUC_lexicon_tags_25 },
    { 0x8a68a666, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fa8a66a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8c8a66c, SUC_lexicon_tags_37 },
    { 0xae28a66d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdaa8a66f, SUC_lexicon_tags_63 },
    { 0x9e48a66f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9a8a672, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8a676, SUC_lexicon_tags_49 },
    { 0x5f68a677, SUC_lexicon_tags_37 },
    { 0xbaa8a678, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff08a67a, SUC_lexicon_tags_201 },
    { 0xbb88a67a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a8a67f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf1e8a681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xada8a684, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba8a689, SUC_lexicon_tags_46 },
    { 0x428a689, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c8a68d, SUC_lexicon_tags_2 },
    { 0x7e28a68e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968a693, SUC_lexicon_tags_2 },
    { 0xc08a694, SUC_lexicon_tags_1 },
    { 0x8488a693, SUC_lexicon_tags_15 },
    { 0x7fc8a696, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2de8a698, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x348a69a, SUC_lexicon_tags_17 },
    { 0x2c68a69b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe628a69d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x608a69f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec8a6a2, SUC_lexicon_tags_7 },
    { 0xabc8a6a2, SUC_lexicon_tags_29 },
    { 0xa728a6a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1688a6a6, SUC_lexicon_tags_25 },
    { 0x108a6a7, SUC_lexicon_tags_25 },
    { 0x8168a6a8, SUC_lexicon_tags_37 },
    { 0x24e8a6a7, SUC_lexicon_tags_17 },
    { 0xac08a6a6, SUC_lexicon_tags_11 },
    { 0x1e08a6a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1208a6ae, SUC_lexicon_tags_63 },
    { 0x4a8a6af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa968a6b2, SUC_lexicon_tags_15 },
    { 0xeb48a6b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf288a6b7, SUC_lexicon_tags_8 },
    { 0x8c48a6b7, SUC_lexicon_tags_29 },
    { 0xdb68a6b8, SUC_lexicon_tags_17 },
    { 0x8728a6b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe8a6bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a6c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a8a6c4, SUC_lexicon_tags_15 },
    { 0xdbe8a6c4, SUC_lexicon_tags_15 },
    { 0x4ce8a6c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaa48a6c8, SUC_lexicon_tags_47 },
    { 0xe4e8a6c8, SUC_lexicon_tags_43 },
    { 0xcb08a6ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdda8a6cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8a6d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1fc8a6d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7728a6d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8328a6e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9968a6e3, SUC_lexicon_tags_33 },
    { 0xc408a6e4, SUC_lexicon_tags_38 },
    { 0xa348a6e4, SUC_lexicon_tags_37 },
    { 0x6268a6e4, SUC_lexicon_tags_37 },
    { 0x3588a6e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe8a6eb, SUC_lexicon_tags_47 },
    { 0x7568a6ec, SUC_lexicon_tags_17 },
    { 0xd708a6ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf48a6f0, SUC_lexicon_tags_53 },
    { 0x9f88a6f1, SUC_lexicon_tags_15 },
    { 0xf908a6f2, SUC_lexicon_tags_23 },
    { 0xc708a6f2, SUC_lexicon_tags_15 },
    { 0x8168a6f2, SUC_lexicon_tags_9 },
    { 0x2248a6f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1fa8a6f7, SUC_lexicon_tags_16 },
    { 0x80c8a6f8, SUC_lexicon_tags_19 },
    { 0x8248a6f9, SUC_lexicon_tags_7 },
    { 0x5fc8a6f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf6c8a6fc, SUC_lexicon_tags_8 },
    { 0x2b88a6fd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xea48a700, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb728a703, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb128a705, SUC_lexicon_tags_27 },
    { 0x9908a706, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1788a70e, SUC_lexicon_tags_1 },
    { 0x3a48a70f, SUC_lexicon_tags_17 },
    { 0xbfa8a710, SUC_lexicon_tags_66 },
    { 0xe468a70f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2488a714, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9168a716, SUC_lexicon_tags_11 },
    { 0xd0a8a717, SUC_lexicon_tags_46 },
    { 0x4148a718, SUC_lexicon_tags_23 },
    { 0x24a8a719, SUC_lexicon_tags_25 },
    { 0x4688a71a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e8a71d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4308a71f, SUC_lexicon_tags_47 },
    { 0xbee8a720, SUC_lexicon_tags_37 },
    { 0x1428a721, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae8a725, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb28a729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc748a72b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe1e8a72d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2608a732, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9dc8a734, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ca8a736, SUC_lexicon_tags_11 },
    { 0x97a8a737, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48a73b, SUC_lexicon_tags_11 },
    { 0xfea8a73b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f08a73f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x75a8a741, SUC_lexicon_tags_12 },
    { 0x3d88a741, SUC_lexicon_tags_0 },
    { 0xf668a743, SUC_lexicon_tags_2 },
    { 0xc768a744, SUC_lexicon_tags_79 },
    { 0xac68a745, SUC_lexicon_tags_2 },
    { 0x2b88a746, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92e8a748, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6188a74c, SUC_lexicon_tags_25 },
    { 0x7fe8a74d, SUC_lexicon_tags_12 },
    { 0x4d48a74e, SUC_lexicon_tags_32 },
    { 0x43a8a74d, SUC_lexicon_tags_68 },
    { 0x8568a74e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb228a752, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5508a754, SUC_lexicon_tags_37 },
    { 0xa668a755, SUC_lexicon_tags_9 },
    { 0x59e8a756, SUC_lexicon_tags_2 },
    { 0x5c68a754, SUC_lexicon_tags_1 },
    { 0xc7c8a756, SUC_lexicon_tags_49 },
    { 0x2648a759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c48a75b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1d08a75d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c8a762, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc268a764, SUC_lexicon_tags_11 },
    { 0x9e28a765, SUC_lexicon_tags_0 },
    { 0x79a8a764, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c68a76b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc988a76f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6888a774, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb88a776, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48a77a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e28a77c, SUC_lexicon_tags_26 },
    { 0x5368a77d, SUC_lexicon_tags_35 },
    { 0x7b28a77d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3488a781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x628a783, SUC_lexicon_tags_7 },
    { 0x868a784, SUC_lexicon_tags_25 },
    { 0xe608a785, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de8a788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad08a78b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7348a78e, SUC_lexicon_tags_54 },
    { 0x2548a78e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee8a793, SUC_lexicon_tags_32 },
    { 0x8c8a793, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdc28a796, SUC_lexicon_tags_919 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc348a79c, SUC_lexicon_tags_37 },
    { 0x1e08a79c, SUC_lexicon_tags_11 },
    { 0xef68a79e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8a7a2, SUC_lexicon_tags_12 },
    { 0x6888a7a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca48a7ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf748a7b0, SUC_lexicon_tags_19 },
    { 0xfe28a7b1, SUC_lexicon_tags_4 },
    { 0x56e8a7b1, SUC_lexicon_tags_64 },
    { 0x97a8a7b0, SUC_lexicon_tags_26 },
    { 0xd8c8a7b4, SUC_lexicon_tags_6 },
    { 0x4828a7b4, SUC_lexicon_tags_29 },
    { 0x4fa8a7b5, SUC_lexicon_tags_262 },
    { 0x9288a7b5, SUC_lexicon_tags_11 },
    { 0xc328a7b8, SUC_lexicon_tags_11 },
    { 0x46e8a7b1, SUC_lexicon_tags_49 },
    { 0x3168a7ba, SUC_lexicon_tags_70 },
    { 0x55e8a7bb, SUC_lexicon_tags_19 },
    { 0x1f28a7bc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x8128a7be, SUC_lexicon_tags_30 },
    { 0x42c8a7bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb588a7c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55a8a7c3, SUC_lexicon_tags_4 },
    { 0x3c48a7c4, SUC_lexicon_tags_33 },
    { 0x9ba8a7c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe988a7c8, SUC_lexicon_tags_66 },
    { 0x3388a7c8, SUC_lexicon_tags_7 },
    { 0xcbe8a7ca, SUC_lexicon_tags_17 },
    { 0xc068a7cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa8a7cf, SUC_lexicon_tags_47 },
    { 0x1348a7d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c28a7d2, SUC_lexicon_tags_27 },
    { 0x2e68a7d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdd28a7d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x62a8a7d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6168a7dc, SUC_lexicon_tags_39 },
    { 0x4488a7dd, SUC_lexicon_tags_21 },
    { 0xeaa8a7dc, SUC_lexicon_tags_0 },
    { 0x2f68a7df, SUC_lexicon_tags_63 },
    { 0xcb88a7dc, SUC_lexicon_tags_38 },
    { 0x3748a7de, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xe4e8a7e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a8a7e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5388a7ec, SUC_lexicon_tags_37 },
    { 0x37c8a7ec, SUC_lexicon_tags_8 },
    { 0x7c8a7ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc8a7f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf668a7f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3988a7f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce8a7fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c8a800, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc68a802, SUC_lexicon_tags_63 },
    { 0x4b28a802, SUC_lexicon_tags_56 },
    { 0x8b48a804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe388a807, SUC_lexicon_tags_37 },
    { 0xb1c8a807, SUC_lexicon_tags_17 },
    { 0x8f08a807, SUC_lexicon_tags_12 },
    { 0x4e68a809, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefc8a80c, SUC_lexicon_tags_47 },
    { 0x25e8a80d, SUC_lexicon_tags_35 },
    { 0xaaa8a80e, SUC_lexicon_tags_15 },
    { 0x6b08a80f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc8a81a, SUC_lexicon_tags_27 },
    { 0x1548a81a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca8a822, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da8a826, SUC_lexicon_tags_25 },
    { 0xab48a826, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85c8a829, SUC_lexicon_tags_24 },
    { 0xa9c8a82a, SUC_lexicon_tags_14 },
    { 0x9a48a82b, SUC_lexicon_tags_0 },
    { 0x56e8a829, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8528a82e, SUC_lexicon_tags_198 },
    { 0xd268a82f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1628a831, SUC_lexicon_tags_12 },
    { 0xd5e8a832, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6528a834, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a8a838, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e8a843, SUC_lexicon_tags_7 },
    { 0x3f88a844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xace8a846, SUC_lexicon_tags_43 },
    { 0x8268a846, SUC_lexicon_tags_2 },
    { 0x1488a847, SUC_lexicon_tags_26 },
    { 0xb248a849, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf208a84c, SUC_lexicon_tags_11 },
    { 0xd048a84c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2188a851, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x248a853, SUC_lexicon_tags_7 },
    { 0xa828a854, SUC_lexicon_tags_47 },
    { 0x64e8a855, SUC_lexicon_tags_37 },
    { 0x21c8a856, SUC_lexicon_tags_32 },
    { 0x39a8a857, SUC_lexicon_tags_12 },
    { 0x8348a857, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab88a85f, SUC_lexicon_tags_39 },
    { 0x31a8a85f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e8a863, SUC_lexicon_tags_84 },
    { 0x8748a864, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb928a867, SUC_lexicon_tags_107 },
    { 0x9468a867, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb188a86c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d68a86f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6808a871, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0x988a873, SUC_lexicon_tags_30 },
    { 0xa6a8a874, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e8a877, SUC_lexicon_tags_11 },
    { 0x21c8a878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2988a87b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96c8a87d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9028a880, SUC_lexicon_tags_37 },
    { 0xf068a881, SUC_lexicon_tags_63 },
    { 0x248a881, SUC_lexicon_tags_25 },
    { 0xe648a883, SUC_lexicon_tags_12 },
    { 0x59c8a881, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed28a886, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5268a88a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8548a88c, SUC_lexicon_tags_9 },
    { 0xe588a88d, SUC_lexicon_tags_12 },
    { 0x3fc8a88c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3608a893, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b08a899, SUC_lexicon_tags_19 },
    { 0x4c08a89a, SUC_lexicon_tags_7 },
    { 0x9d88a89a, SUC_lexicon_tags_23 },
    { 0x9288a89c, SUC_lexicon_tags_13 },
    { 0x4428a89d, SUC_lexicon_tags_1 },
    { 0xe888a89e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c8a8a1, SUC_lexicon_tags_35 },
    { 0x3568a8a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc8a8a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb3c8a8a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe728a8ae, SUC_lexicon_tags_37 },
    { 0xc068a8af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97a8a8b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe828a8b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8a8b7, SUC_lexicon_tags_33 },
    { 0xd3e8a8b8, SUC_lexicon_tags_18 },
    { 0x8dc8a8b8, SUC_lexicon_tags_12 },
    { 0x65e8a8ba, SUC_lexicon_tags_83 },
    { 0xa4e8a8b8, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0x5cc8a8bd, SUC_lexicon_tags_27 },
    { 0xaa08a8bd, SUC_lexicon_tags_8 },
    { 0xe6e8a8bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf808a8c1, SUC_lexicon_tags_33 },
    { 0x5c48a8c2, SUC_lexicon_tags_7 },
    { 0x1c28a8c3, SUC_lexicon_tags_1 },
    { 0x2448a8c4, SUC_lexicon_tags_87 },
    { 0x7368a8c4, SUC_lexicon_tags_7 },
    { 0xf048a8c1, SUC_lexicon_tags_14 },
    { 0xdae8a8c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdf28a8c9, SUC_lexicon_tags_2 },
    { 0x8968a8ca, SUC_lexicon_tags_28 },
    { 0x44c8a8c9, SUC_lexicon_tags_1 },
    { 0xde28a8cc, SUC_lexicon_tags_7 },
    { 0x768a8ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbd08a8cf, SUC_lexicon_tags_25 },
    { 0xb448a8cf, SUC_lexicon_tags_1 },
    { 0xd88a8cf, SUC_lexicon_tags_11 },
    { 0x99c8a8d2, SUC_lexicon_tags_77 },
    { 0x73c8a8d3, SUC_lexicon_tags_37 },
    { 0xa1a8a8d4, SUC_lexicon_tags_11 },
    { 0x59c8a8d4, SUC_lexicon_tags_27 },
    { 0xfd28a8d6, SUC_lexicon_tags_65 },
    { 0x9f88a8d7, SUC_lexicon_tags_0 },
    { 0xdf48a8d4, SUC_lexicon_tags_15 },
    { 0x6fe8a8d9, SUC_lexicon_tags_21 },
    { 0xc3a8a8d4, SUC_lexicon_tags_9 },
    { 0x5fe8a8db, SUC_lexicon_tags_31 },
    { 0x5d88a8db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de8a8df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368a8e4, SUC_lexicon_tags_33 },
    { 0xf2e8a8e5, SUC_lexicon_tags_27 },
    { 0x8848a8e6, SUC_lexicon_tags_17 },
    { 0x7a68a8e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd868a8e9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2628a8ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c8a8f5, SUC_lexicon_tags_37 },
    { 0xbce8a8f5, SUC_lexicon_tags_25 },
    { 0x1d08a8f6, SUC_lexicon_tags_17 },
    { 0x9988a8f8, SUC_lexicon_tags_7 },
    { 0x5da8a8f9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5948a8fb, SUC_lexicon_tags_7 },
    { 0xf0e8a8fb, SUC_lexicon_tags_27 },
    { 0xa8c8a8fb, SUC_lexicon_tags_25 },
    { 0x84c8a8fd, SUC_lexicon_tags_11 },
    { 0xb168a8ff, SUC_lexicon_tags_17 },
    { 0xfd28a8fd, SUC_lexicon_tags_15 },
    { 0xffc8a901, SUC_lexicon_tags_13 },
    { 0x1cc8a902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a08a904, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c8a907, SUC_lexicon_tags_27 },
    { 0x1388a908, SUC_lexicon_tags_31 },
    { 0xc6c8a908, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c8a90e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x59e8a910, SUC_lexicon_tags_28 },
    { 0xc8a8a911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4188a914, SUC_lexicon_tags_7 },
    { 0x5ea8a915, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a8a91a, SUC_lexicon_tags_16 },
    { 0x168a91b, SUC_lexicon_tags_170 },
    { 0x2c68a91c, SUC_lexicon_tags_7 },
    { 0x8c48a91b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6f88a91f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa8a922, SUC_lexicon_tags_136 },
    { 0x4d48a923, SUC_lexicon_tags_17 },
    { 0x6428a924, SUC_lexicon_tags_37 },
    { 0x23a8a925, SUC_lexicon_tags_17 },
    { 0x4308a926, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x3028a928, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8a92b, SUC_lexicon_tags_47 },
    { 0xdda8a92c, SUC_lexicon_tags_38 },
    { 0xa4a8a92b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda28a931, SUC_lexicon_tags_25 },
    { 0xb4a8a931, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf88a935, SUC_lexicon_tags_37 },
    { 0x1f08a935, SUC_lexicon_tags_1 },
    { 0xa328a935, SUC_lexicon_tags_16 },
    { 0x38c8a937, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d88a93d, SUC_lexicon_tags_46 },
    { 0x8d48a93e, SUC_lexicon_tags_38 },
    { 0x3768a93e, SUC_lexicon_tags_17 },
    { 0x3e28a940, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x3488a943, SUC_lexicon_tags_33 },
    { 0xb008a944, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b08a949, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1c8a94b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a8a94f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa6c8a951, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6c8a953, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x48a955, SUC_lexicon_tags_25 },
    { 0xfd88a955, SUC_lexicon_tags_47 },
    { 0x548a957, SUC_lexicon_tags_23 },
    { 0x2a68a958, SUC_lexicon_tags_38 },
    { 0x5b28a958, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x548a95c, SUC_lexicon_tags_55 },
    { 0x43e8a95c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd888a95f, SUC_lexicon_tags_25 },
    { 0x7c08a960, SUC_lexicon_tags_37 },
    { 0xea88a960, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2488a96b, SUC_lexicon_tags_39 },
    { 0x6aa8a96c, SUC_lexicon_tags_17 },
    { 0x47c8a96b, SUC_lexicon_tags_37 },
    { 0xec28a96d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x208a970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebc8a972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5888a975, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef28a977, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe8a97f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a8a984, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b08a986, SUC_lexicon_tags_9 },
    { 0x22a8a987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb48a98a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a68a98e, SUC_lexicon_tags_45 },
    { 0xd488a98e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc028a993, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0xf908a995, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee8a99b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa228a9a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da8a9a4, SUC_lexicon_tags_23 },
    { 0x81e8a9a4, SUC_lexicon_tags_11 },
    { 0x8a68a9a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb88a9a9, SUC_lexicon_tags_31 },
    { 0x83e8a9aa, SUC_lexicon_tags_28 },
    { 0x3a48a9ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d08a9ad, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3908a9b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2728a9b6, SUC_lexicon_tags_37 },
    { 0x7028a9b7, SUC_lexicon_tags_15 },
    { 0x3cc8a9b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1588a9bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4a8a9bd, SUC_lexicon_tags_2 },
    { 0x79a8a9bd, SUC_lexicon_tags_25 },
    { 0xc548a9bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1208a9c6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x24c8a9c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e8a9cf, SUC_lexicon_tags_15 },
    { 0xa8a9d0, SUC_lexicon_tags_19 },
    { 0x1ea8a9d1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6848a9d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe988a9de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4108a9e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf588a9e2, SUC_lexicon_tags_39 },
    { 0xc5a8a9e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3a8a9e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c8a9ec, SUC_lexicon_tags_39 },
    { 0xb208a9ed, SUC_lexicon_tags_7 },
    { 0xeda8a9ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4948a9f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d28a9f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb9a8a9f6, SUC_lexicon_tags_17 },
    { 0x8ea8a9f6, SUC_lexicon_tags_38 },
    { 0xa8a8a9f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1108a9fa, SUC_lexicon_tags_12 },
    { 0x23c8a9fb, SUC_lexicon_tags_28 },
    { 0x5888a9fc, SUC_lexicon_tags_63 },
    { 0x7ec8a9fc, SUC_lexicon_tags_6 },
    { 0x7dc8a9fd, SUC_lexicon_tags_7 },
    { 0x4908a9ff, SUC_lexicon_tags_2 },
    { 0xfbc8a9ff, SUC_lexicon_tags_46 },
    { 0x9e68a9fc, SUC_lexicon_tags_30 },
    { 0x1608aa02, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e88aa0a, SUC_lexicon_tags_12 },
    { 0xb188aa0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc8aa10, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e8aa13, SUC_lexicon_tags_8 },
    { 0xec68aa14, SUC_lexicon_tags_68 },
    { 0xb548aa14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc288aa19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb88aa1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69a8aa20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3868aa23, SUC_lexicon_tags_117 },
    { 0x4468aa24, SUC_lexicon_tags_26 },
    { 0x4ae8aa25, SUC_lexicon_tags_7 },
    { 0xc988aa23, SUC_lexicon_tags_19 },
    { 0x8b48aa24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2948aa2b, SUC_lexicon_tags_7 },
    { 0xc2c8aa2c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e8aa30, SUC_lexicon_tags_6 },
    { 0xa5c8aa31, SUC_lexicon_tags_43 },
    { 0x86e8aa32, SUC_lexicon_tags_17 },
    { 0x17a8aa32, SUC_lexicon_tags_7 },
    { 0xf148aa31, SUC_lexicon_tags_49 },
    { 0xb628aa30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c68aa37, SUC_lexicon_tags_25 },
    { 0x9348aa38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a8aa3c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff08aa40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf148aa42, SUC_lexicon_tags_63 },
    { 0x5428aa43, SUC_lexicon_tags_82 },
    { 0xe688aa42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e8aa49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e8aa4d, SUC_lexicon_tags_17 },
    { 0x28c8aa4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3888aa57, SUC_lexicon_tags_11 },
    { 0xce28aa58, SUC_lexicon_tags_49 },
    { 0x2088aa58, SUC_lexicon_tags_2 },
    { 0x5a48aa57, SUC_lexicon_tags_37 },
    { 0x1c28aa57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b08aa62, SUC_lexicon_tags_38 },
    { 0xf728aa62, SUC_lexicon_tags_63 },
    { 0xf0e8aa64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a8aa69, SUC_lexicon_tags_6 },
    { 0xcc08aa6a, SUC_lexicon_tags_1 },
    { 0xf448aa6a, SUC_lexicon_tags_25 },
    { 0x9468aa6c, SUC_lexicon_tags_60 },
    { 0x4f68aa6c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c8aa76, SUC_lexicon_tags_2 },
    { 0x6de8aa77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbe8aa79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c8aa7c, SUC_lexicon_tags_13 },
    { 0x7b28aa7d, SUC_lexicon_tags_34 },
    { 0x1e8aa7c, SUC_lexicon_tags_60 },
    { 0x2ea8aa7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x64c8aa81, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8788aa83, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5ba8aa85, SUC_lexicon_tags_27 },
    { 0x6f68aa86, SUC_lexicon_tags_15 },
    { 0x75c8aa87, SUC_lexicon_tags_30 },
    { 0xa208aa86, SUC_lexicon_tags_9 },
    { 0xbc28aa87, SUC_lexicon_tags_37 },
    { 0x1468aa87, SUC_lexicon_tags_56 },
    { 0x39e8aa86, SUC_lexicon_tags_27 },
    { 0xb568aa86, SUC_lexicon_tags_11 },
    { 0x1d68aa8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe48aa8f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef48aa93, SUC_lexicon_tags_46 },
    { 0x92c8aa94, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe4c8aa96, SUC_lexicon_tags_31 },
    { 0x64a8aa97, SUC_lexicon_tags_25 },
    { 0x2648aa98, SUC_lexicon_tags_37 },
    { 0xf7e8aa98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c8aa9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4e68aa9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e8aaa1, SUC_lexicon_tags_23 },
    { 0x9308aaa1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5528aaa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb768aaa7, SUC_lexicon_tags_33 },
    { 0x6c08aaa8, SUC_lexicon_tags_60 },
    { 0x1428aaa8, SUC_lexicon_tags_37 },
    { 0xa8aaaa, SUC_lexicon_tags_39 },
    { 0xe308aaab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb668aaae, SUC_lexicon_tags_0 },
    { 0xada8aaae, SUC_lexicon_tags_30 },
    { 0x2848aab0, SUC_lexicon_tags_11 },
    { 0x11e8aab1, SUC_lexicon_tags_14 },
    { 0x44c8aaaf, SUC_lexicon_tags_12 },
    { 0xd0a8aab3, SUC_lexicon_tags_7 },
    { 0x2028aab4, SUC_lexicon_tags_7 },
    { 0x7108aab5, SUC_lexicon_tags_28 },
    { 0x5ba8aab1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a8aabf, SUC_lexicon_tags_46 },
    { 0x2708aac0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4928aac3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1108aac7, SUC_lexicon_tags_26 },
    { 0x6b28aac7, SUC_lexicon_tags_25 },
    { 0x89a8aac9, SUC_lexicon_tags_17 },
    { 0x9cc8aac7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a8aad0, SUC_lexicon_tags_17 },
    { 0xb708aad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3368aad4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a8aad9, SUC_lexicon_tags_12 },
    { 0x5408aada, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d28aade, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ec8aae0, SUC_lexicon_tags_47 },
    { 0xe4c8aae1, SUC_lexicon_tags_15 },
    { 0xaa48aae1, SUC_lexicon_tags_28 },
    { 0x46c8aae3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5048aae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa8aaeb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x408aaed, SUC_lexicon_tags_15 },
    { 0xdb88aaee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x75e8aaf0, SUC_lexicon_tags_30 },
    { 0x6fc8aaf1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7348aaf6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88aafb, SUC_lexicon_tags_60 },
    { 0xb588aafc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d68aafe, SUC_lexicon_tags_15 },
    { 0xf888aaff, SUC_lexicon_tags_30 },
    { 0xd568ab00, SUC_lexicon_tags_53 },
    { 0xb268ab00, SUC_lexicon_tags_1 },
    { 0xf8c8aaff, SUC_lexicon_tags_17 },
    { 0xda88aaff, SUC_lexicon_tags_27 },
    { 0xa488ab04, SUC_lexicon_tags_15 },
    { 0x5428ab05, SUC_lexicon_tags_48 },
    { 0xa1a8ab05, SUC_lexicon_tags_25 },
    { 0xc8c8ab07, SUC_lexicon_tags_14 },
    { 0x3708ab08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce8ab0e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8ab13, SUC_lexicon_tags_2 },
    { 0x29c8ab14, SUC_lexicon_tags_1 },
    { 0xd6c8ab15, SUC_lexicon_tags_45 },
    { 0x90a8ab16, SUC_lexicon_tags_35 },
    { 0x20e8ab17, SUC_lexicon_tags_11 },
    { 0x9be8ab18, SUC_lexicon_tags_1 },
    { 0x5a68ab16, SUC_lexicon_tags_65 },
    { 0xaa08ab13, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb68ab1e, SUC_lexicon_tags_11 },
    { 0xb288ab1f, SUC_lexicon_tags_17 },
    { 0xed88ab1e, SUC_lexicon_tags_1 },
    { 0x3c88ab21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6708ab23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a8ab25, SUC_lexicon_tags_1 },
    { 0xbea8ab26, SUC_lexicon_tags_1 },
    { 0x1728ab27, SUC_lexicon_tags_30 },
    { 0x3908ab28, SUC_lexicon_tags_25 },
    { 0x68c8ab29, SUC_lexicon_tags_17 },
    { 0xf628ab2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e88ab2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5508ab31, SUC_lexicon_tags_53 },
    { 0x3708ab32, SUC_lexicon_tags_37 },
    { 0xb88ab32, SUC_lexicon_tags_12 },
    { 0x1068ab33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c8ab3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e8ab46, SUC_lexicon_tags_34 },
    { 0xb808ab47, SUC_lexicon_tags_45 },
    { 0x9a68ab47, SUC_lexicon_tags_26 },
    { 0x1ce8ab48, SUC_lexicon_tags_25 },
    { 0x8688ab4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc8ab54, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x23e8ab56, SUC_lexicon_tags_28 },
    { 0xa7a8ab56, SUC_lexicon_tags_77 },
    { 0x4ee8ab56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe8ab5b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc428ab5e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa508ab60, SUC_lexicon_tags_35 },
    { 0x4d68ab60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca8ab68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce08ab6a, SUC_lexicon_tags_1 },
    { 0x8428ab6b, SUC_lexicon_tags_8 },
    { 0x2c48ab6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9708ab6e, SUC_lexicon_tags_11 },
    { 0xb388ab6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9288ab7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff08ab80, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd1c8ab82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4208ab85, SUC_lexicon_tags_12 },
    { 0x8ba8ab85, SUC_lexicon_tags_23 },
    { 0xc4c8ab87, SUC_lexicon_tags_33 },
    { 0xc088ab88, SUC_lexicon_tags_17 },
    { 0xc968ab87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a8ab91, SUC_lexicon_tags_47 },
    { 0xc5a8ab92, SUC_lexicon_tags_45 },
    { 0x5c68ab92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5388ab95, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8b48ab97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c88ab99, SUC_lexicon_tags_0 },
    { 0x7748ab9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac8ab9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c8aba1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5a8aba3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd428abad, SUC_lexicon_tags_45 },
    { 0x6748abae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa728abb1, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc8abbb, SUC_lexicon_tags_15 },
    { 0xaca8abbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac8abc5, SUC_lexicon_tags_15 },
    { 0xda68abc6, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0xd408abc9, SUC_lexicon_tags_31 },
    { 0xcf68abc9, SUC_lexicon_tags_61 },
    { 0x3728abc9, SUC_lexicon_tags_485 },
    { 0xa548abc9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc428abd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa448abd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd28abda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7dc8abdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda8abde, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a8abe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62a8abe4, SUC_lexicon_tags_65 },
    { 0x2be8abe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c8abe8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a08abeb, SUC_lexicon_tags_5 },
    { 0x90c8abec, SUC_lexicon_tags_68 },
    { 0xf448abed, SUC_lexicon_tags_43 },
    { 0x38c8abee, SUC_lexicon_tags_39 },
    { 0xf0c8abef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc8abf6, SUC_lexicon_tags_25 },
    { 0xe5a8abf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x888abfa, SUC_lexicon_tags_27 },
    { 0xa908abfa, SUC_lexicon_tags_49 },
    { 0x8708abfb, SUC_lexicon_tags_3 },
    { 0xf9e8abfd, SUC_lexicon_tags_7 },
    { 0x9f68abfd, SUC_lexicon_tags_27 },
    { 0xc4c8abfb, SUC_lexicon_tags_11 },
    { 0xae08ac00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e68ac03, SUC_lexicon_tags_92 },
    { 0x8bc8ac04, SUC_lexicon_tags_38 },
    { 0xf0e8ac05, SUC_lexicon_tags_4 },
    { 0x1a48ac05, SUC_lexicon_tags_82 },
    { 0x7748ac07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a28ac09, SUC_lexicon_tags_92 },
    { 0xf888ac0a, SUC_lexicon_tags_17 },
    { 0xcbe8ac0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c8ac0f, SUC_lexicon_tags_28 },
    { 0x64c8ac10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ea8ac12, SUC_lexicon_tags_45 },
    { 0xe788ac12, SUC_lexicon_tags_95 },
    { 0x3d68ac14, SUC_lexicon_tags_60 },
    { 0xdd08ac14, SUC_lexicon_tags_29 },
    { 0x7608ac14, SUC_lexicon_tags_17 },
    { 0xe708ac13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26c8ac19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce8ac20, SUC_lexicon_tags_35 },
    { 0xa308ac20, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x47e8ac23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3408ac27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3808ac29, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc08ac2d, SUC_lexicon_tags_45 },
    { 0xc128ac2e, SUC_lexicon_tags_25 },
    { 0x5688ac2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168ac37, SUC_lexicon_tags_26 },
    { 0x86c8ac37, SUC_lexicon_tags_43 },
    { 0x33a8ac39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1348ac3d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xb868ac3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xca8ac41, SUC_lexicon_tags_19 },
    { 0x55c8ac42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da8ac47, SUC_lexicon_tags_19 },
    { 0xc908ac48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd008ac50, SUC_lexicon_tags_11 },
    { 0xf7c8ac50, SUC_lexicon_tags_17 },
    { 0xf348ac51, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1a08ac54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5468ac5b, SUC_lexicon_tags_7 },
    { 0x1808ac5c, SUC_lexicon_tags_37 },
    { 0xfaa8ac5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c8ac63, SUC_lexicon_tags_11 },
    { 0xa8c8ac63, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1788ac66, SUC_lexicon_tags_9 },
    { 0x6b68ac67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1988ac6a, SUC_lexicon_tags_0 },
    { 0x53c8ac6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7788ac6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b68ac71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4868ac73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f28ac78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7548ac81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd348ac83, SUC_lexicon_tags_17 },
    { 0xf308ac84, SUC_lexicon_tags_5 },
    { 0xb888ac85, SUC_lexicon_tags_37 },
    { 0x528ac85, SUC_lexicon_tags_17 },
    { 0xdba8ac86, SUC_lexicon_tags_9 },
    { 0xe748ac87, SUC_lexicon_tags_45 },
    { 0xd528ac87, SUC_lexicon_tags_25 },
    { 0xc3c8ac8a, SUC_lexicon_tags_7 },
    { 0x2408ac8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5b48ac8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf9a8ac8f, SUC_lexicon_tags_12 },
    { 0xd88ac90, SUC_lexicon_tags_14 },
    { 0x32e8ac91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f88ac94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4808ac98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe308ac9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec48aca2, SUC_lexicon_tags_2 },
    { 0x2e28aca2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f68aca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9348aca8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdce8acaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad68acad, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c8acb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad88acb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab68acb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe388acb8, SUC_lexicon_tags_19 },
    { 0xbe08acb9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdce8acbc, SUC_lexicon_tags_84 },
    { 0x6d88acbd, SUC_lexicon_tags_15 },
    { 0x6608acbc, SUC_lexicon_tags_79 },
    { 0x2e68acbf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x188acc1, SUC_lexicon_tags_37 },
    { 0x73e8acc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c8acc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c08acc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f28accb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ac8accd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa788acd2, SUC_lexicon_tags_17 },
    { 0xe288acd3, SUC_lexicon_tags_10 },
    { 0xe548acd4, SUC_lexicon_tags_0 },
    { 0x14a8acd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc968acd9, SUC_lexicon_tags_13 },
    { 0x5f48acd9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4788acdc, SUC_lexicon_tags_37 },
    { 0x7188acdc, SUC_lexicon_tags_4 },
    { 0x2508acdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa968ace0, SUC_lexicon_tags_25 },
    { 0x9b08ace1, SUC_lexicon_tags_26 },
    { 0xf328ace2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8008ace4, SUC_lexicon_tags_43 },
    { 0x5388ace4, SUC_lexicon_tags_1 },
    { 0x36e8ace6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc928acec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6608acee, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c68acf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5928acfb, SUC_lexicon_tags_56 },
    { 0x5788acfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc8ad00, SUC_lexicon_tags_151 },
    { 0xab28ad01, SUC_lexicon_tags_77 },
    { 0x50c8ad01, SUC_lexicon_tags_60 },
    { 0xc6e8ad03, SUC_lexicon_tags_14 },
    { 0x2ae8ad04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a48ad06, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd908ad08, SUC_lexicon_tags_27 },
    { 0xe948ad09, SUC_lexicon_tags_4 },
    { 0x74c8ad0a, SUC_lexicon_tags_12 },
    { 0x5fe8ad0b, SUC_lexicon_tags_1 },
    { 0x3fa8ad0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xde68ad0e, SUC_lexicon_tags_49 },
    { 0x1b48ad0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa008ad11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfce8ad13, SUC_lexicon_tags_63 },
    { 0xe428ad14, SUC_lexicon_tags_26 },
    { 0x52e8ad15, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x13a8ad17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4048ad19, SUC_lexicon_tags_17 },
    { 0xd928ad1a, SUC_lexicon_tags_2 },
    { 0x10e8ad1b, SUC_lexicon_tags_37 },
    { 0x92c8ad19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce8ad21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac8ad25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e88ad28, SUC_lexicon_tags_28 },
    { 0xc608ad29, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2068ad2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e08ad30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdac8ad32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27e8ad34, SUC_lexicon_tags_55 },
    { 0xfe8ad35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6968ad3a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa328ad3e, SUC_lexicon_tags_11 },
    { 0x7248ad3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca68ad42, SUC_lexicon_tags_16 },
    { 0x9968ad43, SUC_lexicon_tags_14 },
    { 0x1fc8ad44, SUC_lexicon_tags_46 },
    { 0xbae8ad45, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb288ad48, SUC_lexicon_tags_15 },
    { 0x5e28ad48, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b08ad4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7508ad51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6528ad56, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e68ad5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e68ad66, SUC_lexicon_tags_28 },
    { 0x8da8ad67, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdb68ad69, SUC_lexicon_tags_11 },
    { 0xe9e8ad69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59c8ad6c, SUC_lexicon_tags_28 },
    { 0xb3c8ad6c, SUC_lexicon_tags_7 },
    { 0x4108ad6e, SUC_lexicon_tags_19 },
    { 0x468ad6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f88ad77, SUC_lexicon_tags_1 },
    { 0xbd08ad78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5808ad7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf88ad7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c8ad80, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4748ad85, SUC_lexicon_tags_2 },
    { 0xf1c8ad86, SUC_lexicon_tags_15 },
    { 0x7108ad87, SUC_lexicon_tags_68 },
    { 0x3b68ad88, SUC_lexicon_tags_15 },
    { 0xc968ad87, SUC_lexicon_tags_37 },
    { 0x4168ad8a, SUC_lexicon_tags_43 },
    { 0xfd28ad8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3928ad8f, SUC_lexicon_tags_11 },
    { 0x9ae8ad90, SUC_lexicon_tags_6 },
    { 0x7068ad91, SUC_lexicon_tags_61 },
    { 0x7a68ad92, SUC_lexicon_tags_2 },
    { 0x85e8ad93, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa8ad96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc268ad9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d08ada0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6308ada8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf08adaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a8adad, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1408adbb, SUC_lexicon_tags_14 },
    { 0x1b88adbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd548adbe, SUC_lexicon_tags_82 },
    { 0xce08adbf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d28adc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e8adc7, SUC_lexicon_tags_2 },
    { 0xa648adc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb528adca, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e8add0, SUC_lexicon_tags_12 },
    { 0xa548add1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1808add4, SUC_lexicon_tags_60 },
    { 0x55c8add5, SUC_lexicon_tags_57 },
    { 0xcc68add5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7e8add8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad28addd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc8ade2, SUC_lexicon_tags_7 },
    { 0x1028ade2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3968ade8, SUC_lexicon_tags_23 },
    { 0x4d08ade8, SUC_lexicon_tags_15 },
    { 0x87a8adea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad68adf3, SUC_lexicon_tags_49 },
    { 0xef28adf4, SUC_lexicon_tags_29 },
    { 0x70a8adf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9028adf7, SUC_lexicon_tags_11 },
    { 0x828adf7, SUC_lexicon_tags_37 },
    { 0xf708adf9, SUC_lexicon_tags_7 },
    { 0xe188adf8, SUC_lexicon_tags_12 },
    { 0xec8adf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de8ae02, SUC_lexicon_tags_37 },
    { 0xf7c8ae02, SUC_lexicon_tags_7 },
    { 0x42a8ae03, SUC_lexicon_tags_27 },
    { 0x7468ae05, SUC_lexicon_tags_37 },
    { 0xba08ae06, SUC_lexicon_tags_26 },
    { 0x7b48ae06, SUC_lexicon_tags_1 },
    { 0xf4c8ae08, SUC_lexicon_tags_1 },
    { 0xb2e8ae08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88e8ae0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c8ae10, SUC_lexicon_tags_10 },
    { 0xa128ae11, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b08ae16, SUC_lexicon_tags_74 },
    { 0x1108ae17, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf9e8ae19, SUC_lexicon_tags_39 },
    { 0x3608ae19, SUC_lexicon_tags_63 },
    { 0x41e8ae1b, SUC_lexicon_tags_37 },
    { 0x8bc8ae1c, SUC_lexicon_tags_27 },
    { 0xd468ae1a, SUC_lexicon_tags_49 },
    { 0xe8ae1e, SUC_lexicon_tags_33 },
    { 0xd428ae1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce48ae23, SUC_lexicon_tags_33 },
    { 0xd5c8ae24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a88ae27, SUC_lexicon_tags_4 },
    { 0xd388ae28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a8ae2d, SUC_lexicon_tags_30 },
    { 0xc708ae2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a8ae33, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e8ae36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7508ae3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28ae42, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a8ae45, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c8ae4a, SUC_lexicon_tags_34 },
    { 0x4ce8ae4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed88ae4d, SUC_lexicon_tags_17 },
    { 0x2408ae4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf608ae51, SUC_lexicon_tags_4 },
    { 0x1c48ae52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f68ae55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc48ae57, SUC_lexicon_tags_87 },
    { 0x7a88ae58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b48ae5c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xde8ae5e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc8a8ae60, SUC_lexicon_tags_1 },
    { 0xe188ae60, SUC_lexicon_tags_25 },
    { 0x8928ae60, SUC_lexicon_tags_84 },
    { 0xa488ae63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a8ae67, SUC_lexicon_tags_30 },
    { 0x6728ae68, SUC_lexicon_tags_29 },
    { 0x25a8ae67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f88ae6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf728ae6f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfdc8ae71, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3088ae73, SUC_lexicon_tags_2 },
    { 0xece8ae73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa968ae7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c8ae7d, SUC_lexicon_tags_15 },
    { 0xff08ae7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c8ae83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1c8ae85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc28ae8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa968ae8d, SUC_lexicon_tags_49 },
    { 0x2c08ae8e, SUC_lexicon_tags_37 },
    { 0x37e8ae8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa448ae91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6428ae95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d48ae9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9608ae9e, SUC_lexicon_tags_3 },
    { 0x8f88ae9f, SUC_lexicon_tags_1 },
    { 0x6508aea0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a48aeaa, SUC_lexicon_tags_1 },
    { 0xafc8aeab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3aa8aead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c8aeb3, SUC_lexicon_tags_812 },
    { 0xd048aeb4, SUC_lexicon_tags_11 },
    { 0x7d08aeb4, SUC_lexicon_tags_6 },
    { 0x9fe8aeb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfea8aeb8, SUC_lexicon_tags_11 },
    { 0x4e68aeb9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x888aebb, SUC_lexicon_tags_14 },
    { 0x61a8aebc, SUC_lexicon_tags_7 },
    { 0x6288aebc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c8aec0, SUC_lexicon_tags_37 },
    { 0xcf48aec1, SUC_lexicon_tags_12 },
    { 0x6dc8aec1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x98c8aec5, SUC_lexicon_tags_37 },
    { 0x2c08aec6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7bc8aec8, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x15a8aecb, SUC_lexicon_tags_47 },
    { 0x2b88aecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe908aed1, SUC_lexicon_tags_13 },
    { 0x4f08aed1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68aed9, SUC_lexicon_tags_30 },
    { 0x87a8aeda, SUC_lexicon_tags_38 },
    { 0x37c8aed9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228aedf, SUC_lexicon_tags_0 },
    { 0x8248aee0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe388aeec, SUC_lexicon_tags_20 },
    { 0x3ec8aeed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xea48aef0, SUC_lexicon_tags_60 },
    { 0x7408aef1, SUC_lexicon_tags_25 },
    { 0x5048aef2, SUC_lexicon_tags_84 },
    { 0x6fa8aef2, SUC_lexicon_tags_37 },
    { 0x8668aef2, SUC_lexicon_tags_65 },
    { 0xce88aef3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd4a8aef7, SUC_lexicon_tags_25 },
    { 0x6928aef8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x86e8aefa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb808aefd, SUC_lexicon_tags_30 },
    { 0x2a68aefe, SUC_lexicon_tags_30 },
    { 0x5648aeff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce8af03, SUC_lexicon_tags_10 },
    { 0x2dc8af04, SUC_lexicon_tags_1 },
    { 0x7aa8af05, SUC_lexicon_tags_26 },
    { 0xe008af03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa48af0b, SUC_lexicon_tags_248 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd08af0e, SUC_lexicon_tags_23 },
    { 0x5088af0f, SUC_lexicon_tags_42 },
    { 0xaea8af10, SUC_lexicon_tags_47 },
    { 0xfba8af0f, SUC_lexicon_tags_43 },
    { 0xc148af12, SUC_lexicon_tags_49 },
    { 0x988af13, SUC_lexicon_tags_33 },
    { 0x8008af0e, SUC_lexicon_tags_25 },
    { 0x5d68af15, SUC_lexicon_tags_3 },
    { 0x20a8af15, SUC_lexicon_tags_0 },
    { 0x6068af16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb28af1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0a8af1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f48af1f, SUC_lexicon_tags_25 },
    { 0x25e8af1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2588af22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc768af27, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4048af29, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb128af2b, SUC_lexicon_tags_15 },
    { 0xbb88af2c, SUC_lexicon_tags_6 },
    { 0x5d28af2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5e8af2f, SUC_lexicon_tags_70 },
    { 0xe1c8af30, SUC_lexicon_tags_68 },
    { 0xefe8af30, SUC_lexicon_tags_9 },
    { 0x90c8af32, SUC_lexicon_tags_19 },
    { 0x4928af31, SUC_lexicon_tags_2 },
    { 0x7868af34, SUC_lexicon_tags_46 },
    { 0x8ce8af2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce8af3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf28af47, SUC_lexicon_tags_19 },
    { 0x9508af48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd208af4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1628af4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de8af53, SUC_lexicon_tags_11 },
    { 0xab68af53, SUC_lexicon_tags_25 },
    { 0x8268af55, SUC_lexicon_tags_10 },
    { 0x9608af56, SUC_lexicon_tags_31 },
    { 0x7a48af57, SUC_lexicon_tags_17 },
    { 0xeb28af58, SUC_lexicon_tags_11 },
    { 0x72e8af59, SUC_lexicon_tags_37 },
    { 0xfba8af59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7448af5e, SUC_lexicon_tags_1 },
    { 0x8e68af5f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688af63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e8af6c, SUC_lexicon_tags_49 },
    { 0x93c8af6d, SUC_lexicon_tags_19 },
    { 0x8ae8af6e, SUC_lexicon_tags_27 },
    { 0x58c8af6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c8af73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac8af76, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe88af7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe088af80, SUC_lexicon_tags_32 },
    { 0xff08af81, SUC_lexicon_tags_47 },
    { 0x55c8af82, SUC_lexicon_tags_43 },
    { 0xc5e8af83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5068af87, SUC_lexicon_tags_27 },
    { 0x4748af88, SUC_lexicon_tags_79 },
    { 0x41a8af88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d88af8b, SUC_lexicon_tags_7 },
    { 0x7648af8b, SUC_lexicon_tags_25 },
    { 0xee28af8d, SUC_lexicon_tags_60 },
    { 0x7ea8af8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c8af91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac8af9f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbd08afa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e8afa6, SUC_lexicon_tags_47 },
    { 0xa108afa6, SUC_lexicon_tags_12 },
    { 0xcf08afa8, SUC_lexicon_tags_47 },
    { 0xaae8afa9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c8afae, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c08afb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b28afb7, SUC_lexicon_tags_0 },
    { 0x89c8afb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf668afba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be8afbd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa488afc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4708afc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcdc8afc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2748afcd, SUC_lexicon_tags_9 },
    { 0x248afce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf868afd0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x328afd2, SUC_lexicon_tags_25 },
    { 0x52e8afd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5288afd6, SUC_lexicon_tags_14 },
    { 0xc1a8afd7, SUC_lexicon_tags_37 },
    { 0xb68afd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa708afdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d28afe1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x46c8afe3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8288afe8, SUC_lexicon_tags_2 },
    { 0x8468afe8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c8afec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc8aff1, SUC_lexicon_tags_27 },
    { 0xce48aff2, SUC_lexicon_tags_2 },
    { 0xc108aff3, SUC_lexicon_tags_1 },
    { 0xffa8aff4, SUC_lexicon_tags_25 },
    { 0xca68aff1, SUC_lexicon_tags_11 },
    { 0x5b28aff6, SUC_lexicon_tags_43 },
    { 0x988aff7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e28affa, SUC_lexicon_tags_2 },
    { 0xe8c8affa, SUC_lexicon_tags_11 },
    { 0xc2e8affc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc8b000, SUC_lexicon_tags_11 },
    { 0xaba8b001, SUC_lexicon_tags_29 },
    { 0xb868b002, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7208b004, SUC_lexicon_tags_27 },
    { 0xcd28b004, SUC_lexicon_tags_11 },
    { 0xffc8b006, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4948b009, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa28b00b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c8b00f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd568b018, SUC_lexicon_tags_87 },
    { 0xeba8b019, SUC_lexicon_tags_17 },
    { 0xa548b018, SUC_lexicon_tags_16 },
    { 0xc28b019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6648b01d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe088b027, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5b28b029, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa08b02b, SUC_lexicon_tags_38 },
    { 0x1928b02b, SUC_lexicon_tags_56 },
    { 0xf48b02d, SUC_lexicon_tags_7 },
    { 0x4828b02d, SUC_lexicon_tags_207 },
    { 0x8d68b02f, SUC_lexicon_tags_15 },
    { 0x69a8b030, SUC_lexicon_tags_14 },
    { 0xca08b02f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f28b035, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e8b03a, SUC_lexicon_tags_25 },
    { 0x8168b03a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1048b03d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xea68b040, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e8b043, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a8b049, SUC_lexicon_tags_25 },
    { 0xae28b04a, SUC_lexicon_tags_25 },
    { 0xd788b04b, SUC_lexicon_tags_11 },
    { 0x6fa8b04c, SUC_lexicon_tags_49 },
    { 0xfb88b04d, SUC_lexicon_tags_49 },
    { 0x2c88b04e, SUC_lexicon_tags_7 },
    { 0x6388b04f, SUC_lexicon_tags_82 },
    { 0x4848b050, SUC_lexicon_tags_6 },
    { 0xd9e8b051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c8b055, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47a8b057, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c8b05c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b88b060, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e88b064, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f48b067, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd848b069, SUC_lexicon_tags_28 },
    { 0xc4a8b06a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e8b06d, SUC_lexicon_tags_13 },
    { 0xa328b06e, SUC_lexicon_tags_2 },
    { 0x1008b06f, SUC_lexicon_tags_46 },
    { 0x7328b070, SUC_lexicon_tags_30 },
    { 0x3cc8b06e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb988b073, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5ea8b075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a68b081, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5cc8b083, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7508b087, SUC_lexicon_tags_37 },
    { 0xdae8b088, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fe8b08a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2a8b08c, SUC_lexicon_tags_44 },
    { 0x9d28b08d, SUC_lexicon_tags_37 },
    { 0xe388b08c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a8b091, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4e8b093, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f68b098, SUC_lexicon_tags_45 },
    { 0x1648b099, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8808b09b, SUC_lexicon_tags_37 },
    { 0x87c8b09c, SUC_lexicon_tags_14 },
    { 0xeb08b09d, SUC_lexicon_tags_43 },
    { 0x6d88b09e, SUC_lexicon_tags_160 },
    { 0xc4a8b09f, SUC_lexicon_tags_4 },
    { 0x4a28b0a0, SUC_lexicon_tags_15 },
    { 0x41e8b0a0, SUC_lexicon_tags_12 },
    { 0xb4e8b0a1, SUC_lexicon_tags_17 },
    { 0x6e28b09e, SUC_lexicon_tags_3 },
    { 0x8408b0a4, SUC_lexicon_tags_17 },
    { 0xadc8b0a3, SUC_lexicon_tags_15 },
    { 0xe008b0a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea8b0ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e8b0ae, SUC_lexicon_tags_11 },
    { 0x5a48b0af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c8b0b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc8b0b9, SUC_lexicon_tags_2 },
    { 0xa608b0ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33a8b0bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29c8b0be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca8b0c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768b0ca, SUC_lexicon_tags_30 },
    { 0xb388b0cb, SUC_lexicon_tags_32 },
    { 0x6468b0cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6828b0ce, SUC_lexicon_tags_39 },
    { 0xcd28b0cf, SUC_lexicon_tags_25 },
    { 0x9348b0cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xab68b0d3, SUC_lexicon_tags_11 },
    { 0xeca8b0d4, SUC_lexicon_tags_28 },
    { 0xab68b0d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9e8b0d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2148b0d9, SUC_lexicon_tags_37 },
    { 0xcc88b0da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5148b0dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6788b0e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc8b0e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e8b0ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a8b0ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2368b0f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1488b0f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8b0fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a8b0fe, SUC_lexicon_tags_47 },
    { 0xbc8b0ff, SUC_lexicon_tags_8 },
    { 0x29a8b0ff, SUC_lexicon_tags_25 },
    { 0xb1a8b0ff, SUC_lexicon_tags_47 },
    { 0xa728b0ff, SUC_lexicon_tags_11 },
    { 0x5688b0ff, SUC_lexicon_tags_87 },
    { 0x8028b100, SUC_lexicon_tags_37 },
    { 0x8788b102, SUC_lexicon_tags_12 },
    { 0x2bc8b104, SUC_lexicon_tags_17 },
    { 0xe068b107, SUC_lexicon_tags_17 },
    { 0xd808b108, SUC_lexicon_tags_7 },
    { 0x8128b103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48b10c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f68b111, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff28b113, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca8b122, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a8b127, SUC_lexicon_tags_1 },
    { 0x4a88b127, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2308b12a, SUC_lexicon_tags_11 },
    { 0x8f08b12a, SUC_lexicon_tags_30 },
    { 0x14e8b12b, SUC_lexicon_tags_7 },
    { 0xf2c8b12d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a8b130, SUC_lexicon_tags_34 },
    { 0x5c48b131, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2d08b133, SUC_lexicon_tags_16 },
    { 0x6fe8b134, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1168b138, SUC_lexicon_tags_2 },
    { 0x2028b139, SUC_lexicon_tags_37 },
    { 0xde28b13a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec48b144, SUC_lexicon_tags_37 },
    { 0xf3a8b145, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f68b149, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e8b14d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8468b14f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a8b159, SUC_lexicon_tags_136 },
    { 0x7f88b159, SUC_lexicon_tags_7 },
    { 0xdf48b15b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf808b15d, SUC_lexicon_tags_11 },
    { 0xc2e8b15d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f28b162, SUC_lexicon_tags_15 },
    { 0xb668b162, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc1a8b165, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c8b168, SUC_lexicon_tags_12 },
    { 0x1ac8b169, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe928b16d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42c8b16f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7688b174, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5868b179, SUC_lexicon_tags_11 },
    { 0xaae8b179, SUC_lexicon_tags_29 },
    { 0x3428b17b, SUC_lexicon_tags_2 },
    { 0xc648b17b, SUC_lexicon_tags_11 },
    { 0xf168b17d, SUC_lexicon_tags_15 },
    { 0x9188b17d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc428b181, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa88b185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb88b187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb888b189, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd388b194, SUC_lexicon_tags_31 },
    { 0x97c8b195, SUC_lexicon_tags_14 },
    { 0x6488b196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa8b19b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5288b19e, SUC_lexicon_tags_1 },
    { 0x7fa8b19e, SUC_lexicon_tags_25 },
    { 0x8e48b1a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ce8b1a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3668b1a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe188b1ac, SUC_lexicon_tags_12 },
    { 0xefe8b1ad, SUC_lexicon_tags_31 },
    { 0x7288b1ad, SUC_lexicon_tags_65 },
    { 0xdac8b1ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c8b1b6, SUC_lexicon_tags_14 },
    { 0x9cc8b1b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a8b1ba, SUC_lexicon_tags_1 },
    { 0xb6e8b1ba, SUC_lexicon_tags_49 },
    { 0x92c8b1bc, SUC_lexicon_tags_15 },
    { 0x6d28b1bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe368b1c0, SUC_lexicon_tags_37 },
    { 0x1dc8b1c0, SUC_lexicon_tags_47 },
    { 0x5488b1c2, SUC_lexicon_tags_1338 },
    { 0x27c8b1c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x40e8b1c5, SUC_lexicon_tags_37 },
    { 0xbf48b1c6, SUC_lexicon_tags_7 },
    { 0x7ca8b1c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1988b1ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c08b1d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5788b1d5, SUC_lexicon_tags_39 },
    { 0x5768b1d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9948b1d9, SUC_lexicon_tags_25 },
    { 0xd348b1da, SUC_lexicon_tags_15 },
    { 0xa9c8b1da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x768b1dd, SUC_lexicon_tags_15 },
    { 0xb468b1dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa068b1e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae8b1e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa708b1ea, SUC_lexicon_tags_1 },
    { 0xe228b1eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc48b1ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa68b1f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2068b1f5, SUC_lexicon_tags_17 },
    { 0x2e8b1f6, SUC_lexicon_tags_7 },
    { 0xa768b1f6, SUC_lexicon_tags_2 },
    { 0x948b1f6, SUC_lexicon_tags_16 },
    { 0x88b1f9, SUC_lexicon_tags_53 },
    { 0xe4a8b1fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08b1fd, SUC_lexicon_tags_29 },
    { 0xe948b1fe, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x5908b201, SUC_lexicon_tags_9 },
    { 0xb508b202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e08b208, SUC_lexicon_tags_79 },
    { 0xd008b208, SUC_lexicon_tags_1 },
    { 0x9828b20a, SUC_lexicon_tags_26 },
    { 0xda28b20a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8348b20e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a8b212, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4748b227, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1688b229, SUC_lexicon_tags_21 },
    { 0x4b08b22a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf448b230, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb728b232, SUC_lexicon_tags_150 },
    { 0x3d08b233, SUC_lexicon_tags_1 },
    { 0x7ce8b234, SUC_lexicon_tags_6 },
    { 0x6428b233, SUC_lexicon_tags_2 },
    { 0xe0e8b236, SUC_lexicon_tags_799 },
    { 0x60e8b235, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5848b23a, SUC_lexicon_tags_44 },
    { 0xb568b23b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c08b240, SUC_lexicon_tags_1 },
    { 0xf148b240, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae8b244, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48b249, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c8b24c, SUC_lexicon_tags_12 },
    { 0x8308b24d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9668b252, SUC_lexicon_tags_60 },
    { 0xb848b253, SUC_lexicon_tags_1 },
    { 0x1ea8b254, SUC_lexicon_tags_6 },
    { 0x448b253, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea8b259, SUC_lexicon_tags_2 },
    { 0x6588b25a, SUC_lexicon_tags_23 },
    { 0x8648b25b, SUC_lexicon_tags_6 },
    { 0x6348b25a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc108b25e, SUC_lexicon_tags_87 },
    { 0x7308b25f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb768b261, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa8b264, SUC_lexicon_tags_47 },
    { 0x1908b265, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd928b26a, SUC_lexicon_tags_61 },
    { 0xb1e8b26b, SUC_lexicon_tags_17 },
    { 0x70c8b26c, SUC_lexicon_tags_15 },
    { 0x368b26d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73a8b26f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x868b271, SUC_lexicon_tags_45 },
    { 0xeb88b272, SUC_lexicon_tags_11 },
    { 0xba88b271, SUC_lexicon_tags_23 },
    { 0x65c8b273, SUC_lexicon_tags_53 },
    { 0xac88b271, SUC_lexicon_tags_65 },
    { 0x5e68b276, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa8b279, SUC_lexicon_tags_25 },
    { 0x4b48b27a, SUC_lexicon_tags_1 },
    { 0x5aa8b27b, SUC_lexicon_tags_5 },
    { 0x57e8b27b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3188b27e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf848b282, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68b289, SUC_lexicon_tags_63 },
    { 0x9888b28a, SUC_lexicon_tags_25 },
    { 0xc128b28b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec8b293, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a8b297, SUC_lexicon_tags_6 },
    { 0xc348b297, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8808b29a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5008b29c, SUC_lexicon_tags_1 },
    { 0xa68b29d, SUC_lexicon_tags_84 },
    { 0x5fc8b29e, SUC_lexicon_tags_39 },
    { 0x25c8b29e, SUC_lexicon_tags_7 },
    { 0x2748b29d, SUC_lexicon_tags_12 },
    { 0xf0c8b2a1, SUC_lexicon_tags_65 },
    { 0xe488b2a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e8b2a6, SUC_lexicon_tags_21 },
    { 0x15e8b2a7, SUC_lexicon_tags_33 },
    { 0x63c8b2a8, SUC_lexicon_tags_7 },
    { 0x51a8b2a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a8b2ac, SUC_lexicon_tags_12 },
    { 0xfe88b2ac, SUC_lexicon_tags_15 },
    { 0xb7a8b2ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe108b2b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc8b2b6, SUC_lexicon_tags_48 },
    { 0x788b2b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6788b2bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4a8b2bf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xeaa8b2c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48a8b2c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4008b2c9, SUC_lexicon_tags_37 },
    { 0xcd08b2ca, SUC_lexicon_tags_13 },
    { 0x8448b2cb, SUC_lexicon_tags_2 },
    { 0x1788b2cb, SUC_lexicon_tags_15 },
    { 0xf368b2ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c8b2d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf888b2d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd88b2d8, SUC_lexicon_tags_87 },
    { 0xea08b2d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8b2dc, SUC_lexicon_tags_60 },
    { 0xdb48b2dd, SUC_lexicon_tags_2 },
    { 0x83e8b2de, SUC_lexicon_tags_63 },
    { 0x6d48b2de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26c8b2e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08b2e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcc8b2e6, SUC_lexicon_tags_10 },
    { 0x1868b2e7, SUC_lexicon_tags_7 },
    { 0x6ba8b2e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c88b2ed, SUC_lexicon_tags_146 },
    { 0x5ee8b2ed, SUC_lexicon_tags_19 },
    { 0x9f48b2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc8b2f3, SUC_lexicon_tags_43 },
    { 0x2b88b2f3, SUC_lexicon_tags_66 },
    { 0x9608b2f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6fc8b2f7, SUC_lexicon_tags_26 },
    { 0xd148b2f8, SUC_lexicon_tags_10 },
    { 0x7bc8b2f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4768b2fe, SUC_lexicon_tags_17 },
    { 0xb028b2ff, SUC_lexicon_tags_9 },
    { 0xb5c8b2fe, SUC_lexicon_tags_7 },
    { 0xb7a8b301, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe768b306, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1b88b308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb88b30a, SUC_lexicon_tags_12 },
    { 0xa728b30b, SUC_lexicon_tags_39 },
    { 0xec48b30c, SUC_lexicon_tags_45 },
    { 0x2ac8b30c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a8b310, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x41e8b312, SUC_lexicon_tags_15 },
    { 0x2048b313, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe708b31a, SUC_lexicon_tags_49 },
    { 0x8028b31b, SUC_lexicon_tags_17 },
    { 0x8628b31c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x41c8b31e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf248b320, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f08b326, SUC_lexicon_tags_1 },
    { 0x5d88b327, SUC_lexicon_tags_47 },
    { 0x8508b327, SUC_lexicon_tags_17 },
    { 0x6c68b328, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c8b32c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8be8b32e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9408b330, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa8b334, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6448b336, SUC_lexicon_tags_49 },
    { 0xddc8b336, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb228b33d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88b340, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a8b348, SUC_lexicon_tags_17 },
    { 0xdac8b349, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a8b34c, SUC_lexicon_tags_15 },
    { 0xf6a8b34d, SUC_lexicon_tags_25 },
    { 0xb468b34c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba08b353, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc28b359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a8b361, SUC_lexicon_tags_55 },
    { 0x8128b362, SUC_lexicon_tags_55 },
    { 0x53c8b362, SUC_lexicon_tags_25 },
    { 0x41a8b364, SUC_lexicon_tags_84 },
    { 0x3868b364, SUC_lexicon_tags_12 },
    { 0x888b366, SUC_lexicon_tags_17 },
    { 0xc7e8b367, SUC_lexicon_tags_15 },
    { 0x2488b368, SUC_lexicon_tags_28 },
    { 0xca88b369, SUC_lexicon_tags_43 },
    { 0x3728b36a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d28b370, SUC_lexicon_tags_12 },
    { 0x6728b371, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cc8b373, SUC_lexicon_tags_30 },
    { 0xbcc8b374, SUC_lexicon_tags_12 },
    { 0x8e68b374, SUC_lexicon_tags_2 },
    { 0xbb28b375, SUC_lexicon_tags_12 },
    { 0x5ae8b377, SUC_lexicon_tags_15 },
    { 0xd2a8b375, SUC_lexicon_tags_0 },
    { 0xa7e8b379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c8b37b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6768b37d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb748b37f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce8b385, SUC_lexicon_tags_25 },
    { 0x8e88b385, SUC_lexicon_tags_9 },
    { 0x2e8b387, SUC_lexicon_tags_8 },
    { 0xc508b386, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc8b393, SUC_lexicon_tags_25 },
    { 0xf9e8b393, SUC_lexicon_tags_58 },
    { 0x1bc8b393, SUC_lexicon_tags_43 },
    { 0x6fa8b396, SUC_lexicon_tags_32 },
    { 0xdfc8b396, SUC_lexicon_tags_112 },
    { 0x8368b398, SUC_lexicon_tags_33 },
    { 0x65a8b398, SUC_lexicon_tags_25 },
    { 0x4c08b39a, SUC_lexicon_tags_23 },
    { 0xbde8b399, SUC_lexicon_tags_11 },
    { 0xdf48b39c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c8b39f, SUC_lexicon_tags_25 },
    { 0xe908b39f, SUC_lexicon_tags_14 },
    { 0x6dc8b3a0, SUC_lexicon_tags_2 },
    { 0x98e8b3a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28b3a4, SUC_lexicon_tags_27 },
    { 0x73e8b3a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5f48b3a7, SUC_lexicon_tags_31 },
    { 0xe2c8b3a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd408b3aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2468b3ac, SUC_lexicon_tags_17 },
    { 0x3368b3ad, SUC_lexicon_tags_65 },
    { 0x4ae8b3ae, SUC_lexicon_tags_15 },
    { 0x3748b3ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b28b3b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce68b3b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc908b3be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa68b3c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828b3cf, SUC_lexicon_tags_2 },
    { 0xefe8b3d0, SUC_lexicon_tags_9 },
    { 0xce68b3cf, SUC_lexicon_tags_37 },
    { 0x5108b3d0, SUC_lexicon_tags_39 },
    { 0x83c8b3cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8e8b3d5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1948b3d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd088b3dc, SUC_lexicon_tags_4 },
    { 0xfe48b3dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd3a8b3df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa08b3e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7c8b3e8, SUC_lexicon_tags_40 },
    { 0x7d08b3e9, SUC_lexicon_tags_7 },
    { 0xbce8b3ea, SUC_lexicon_tags_47 },
    { 0xa108b3e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f68b3ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c48b3f2, SUC_lexicon_tags_27 },
    { 0xf3a8b3f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e48b3f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d28b3fa, SUC_lexicon_tags_23 },
    { 0x99a8b3fa, SUC_lexicon_tags_60 },
    { 0x4c08b3fc, SUC_lexicon_tags_8 },
    { 0xc468b3fd, SUC_lexicon_tags_5 },
    { 0x7208b3fe, SUC_lexicon_tags_1 },
    { 0x448b3ff, SUC_lexicon_tags_4 },
    { 0x5068b3fe, SUC_lexicon_tags_14 },
    { 0x9ec8b401, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4368b403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe928b405, SUC_lexicon_tags_64 },
    { 0x9d28b405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c48b40a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x50a8b40c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c8b40f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7388b413, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a8b416, SUC_lexicon_tags_15 },
    { 0x9ce8b417, SUC_lexicon_tags_1 },
    { 0x5468b418, SUC_lexicon_tags_47 },
    { 0x5528b419, SUC_lexicon_tags_33 },
    { 0xd788b41a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1008b41c, SUC_lexicon_tags_10 },
    { 0x4a48b41d, SUC_lexicon_tags_7 },
    { 0x5b88b41e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe088b426, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3948b431, SUC_lexicon_tags_19 },
    { 0x7bc8b432, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac8b437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4468b43c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xf168b43e, SUC_lexicon_tags_32 },
    { 0xebc8b43f, SUC_lexicon_tags_33 },
    { 0xd668b43f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe328b442, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14c8b444, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf68b449, SUC_lexicon_tags_28 },
    { 0x488b44a, SUC_lexicon_tags_57 },
    { 0xbbc8b44a, SUC_lexicon_tags_43 },
    { 0x2e48b44c, SUC_lexicon_tags_11 },
    { 0xd308b44c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a08b454, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9388b458, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c88b45c, SUC_lexicon_tags_6 },
    { 0x1148b45c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2588b460, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b467, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa8b46e, SUC_lexicon_tags_23 },
    { 0xb188b46e, SUC_lexicon_tags_25 },
    { 0xcc68b470, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe28b473, SUC_lexicon_tags_17 },
    { 0xb8e8b473, SUC_lexicon_tags_29 },
    { 0xda88b475, SUC_lexicon_tags_16 },
    { 0x79c8b476, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xab28b478, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb788b47d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c8b480, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad68b484, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0x16e8b486, SUC_lexicon_tags_11 },
    { 0x75e8b487, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c88b48c, SUC_lexicon_tags_2 },
    { 0x8e8b48d, SUC_lexicon_tags_30 },
    { 0x5468b48c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57c8b490, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d68b492, SUC_lexicon_tags_6 },
    { 0x7b68b492, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe8b49d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc108b4a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7e8b4a4, SUC_lexicon_tags_11 },
    { 0x3cc8b4a5, SUC_lexicon_tags_12 },
    { 0xb328b4a5, SUC_lexicon_tags_11 },
    { 0x2048b4a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe308b4aa, SUC_lexicon_tags_15 },
    { 0xca08b4aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab08b4b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb148b4b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f68b4b7, SUC_lexicon_tags_8 },
    { 0x8da8b4b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da8b4bb, SUC_lexicon_tags_26 },
    { 0x6aa8b4bc, SUC_lexicon_tags_2 },
    { 0x2088b4bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a8b4cd, SUC_lexicon_tags_25 },
    { 0xb1c8b4cd, SUC_lexicon_tags_3 },
    { 0x588b4cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa248b4d2, SUC_lexicon_tags_4 },
    { 0x2928b4d3, SUC_lexicon_tags_18 },
    { 0x23c8b4d3, SUC_lexicon_tags_27 },
    { 0x6228b4d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9608b4d7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd308b4da, SUC_lexicon_tags_17 },
    { 0x4228b4db, SUC_lexicon_tags_27 },
    { 0x14e8b4da, SUC_lexicon_tags_15 },
    { 0xbe8b4db, SUC_lexicon_tags_25 },
    { 0x1ae8b4de, SUC_lexicon_tags_63 },
    { 0xf108b4df, SUC_lexicon_tags_19 },
    { 0xe9e8b4dd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc8b4e9, SUC_lexicon_tags_136 },
    { 0x8d08b4ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7828b4ec, SUC_lexicon_tags_25 },
    { 0x2d08b4ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2a8b4ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc8b4f3, SUC_lexicon_tags_15 },
    { 0x61c8b4f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc308b4f6, SUC_lexicon_tags_48 },
    { 0x5548b4f6, SUC_lexicon_tags_12 },
    { 0x4428b4f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf848b4fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d88b4ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd768b503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5088b505, SUC_lexicon_tags_23 },
    { 0x71c8b505, SUC_lexicon_tags_55 },
    { 0x2688b507, SUC_lexicon_tags_17 },
    { 0xda08b508, SUC_lexicon_tags_19 },
    { 0xfbc8b509, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f88b50c, SUC_lexicon_tags_60 },
    { 0x6428b50d, SUC_lexicon_tags_32 },
    { 0xcec8b50e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd688b512, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x20e8b514, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf428b518, SUC_lexicon_tags_60 },
    { 0x2e08b519, SUC_lexicon_tags_15 },
    { 0x3ee8b518, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b48b51e, SUC_lexicon_tags_7 },
    { 0xe1a8b51f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb088b526, SUC_lexicon_tags_11 },
    { 0x1e88b526, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x68c8b529, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf148b52b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1248b530, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e8b533, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd648b53b, SUC_lexicon_tags_12 },
    { 0xf2c8b53c, SUC_lexicon_tags_15 },
    { 0x3fa8b53d, SUC_lexicon_tags_47 },
    { 0xf988b53e, SUC_lexicon_tags_37 },
    { 0xbbc8b53e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8288b542, SUC_lexicon_tags_17 },
    { 0xaee8b543, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee8b546, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e68b549, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348b54e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd508b550, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe608b552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b88b554, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc8b558, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c8b55c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e68b55f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e08b56a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe8b56f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec08b572, SUC_lexicon_tags_15 },
    { 0x1288b573, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5128b57c, SUC_lexicon_tags_15 },
    { 0x1fa8b57c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5468b581, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc68b584, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb1e8b586, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4328b589, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc88b58b, SUC_lexicon_tags_47 },
    { 0x20e8b58c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa808b58e, SUC_lexicon_tags_5 },
    { 0x3868b58f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1728b591, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa8b593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8b597, SUC_lexicon_tags_11 },
    { 0x55a8b598, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x92a8b59a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e8b5a6, SUC_lexicon_tags_6 },
    { 0x2788b5a7, SUC_lexicon_tags_61 },
    { 0xb1c8b5a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6728b5ad, SUC_lexicon_tags_26 },
    { 0xab28b5ad, SUC_lexicon_tags_9 },
    { 0x7668b5ad, SUC_lexicon_tags_28 },
    { 0xaa68b5ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd148b5b4, SUC_lexicon_tags_57 },
    { 0x1d28b5b5, SUC_lexicon_tags_57 },
    { 0x5248b5b6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c88b5b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe908b5bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e8b5c0, SUC_lexicon_tags_17 },
    { 0x3288b5c0, SUC_lexicon_tags_1 },
    { 0x5f88b5c2, SUC_lexicon_tags_47 },
    { 0xb3c8b5c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc388b5c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d68b5cb, SUC_lexicon_tags_14 },
    { 0xa368b5cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe908b5cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac8b5d2, SUC_lexicon_tags_65 },
    { 0x1f68b5d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee8b5d9, SUC_lexicon_tags_7 },
    { 0xf668b5da, SUC_lexicon_tags_49 },
    { 0x6468b5db, SUC_lexicon_tags_45 },
    { 0xe848b5d9, SUC_lexicon_tags_27 },
    { 0xe248b5dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc288b5e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xab68b5ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c48b5ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba08b5f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0c8b5f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x588b5f4, SUC_lexicon_tags_37 },
    { 0x9e8b5f5, SUC_lexicon_tags_60 },
    { 0x8868b5f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc308b5fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc88b5fc, SUC_lexicon_tags_15 },
    { 0x8de8b5fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2008b600, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7428b604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a08b611, SUC_lexicon_tags_38 },
    { 0x6628b611, SUC_lexicon_tags_17 },
    { 0xb608b613, SUC_lexicon_tags_54 },
    { 0x1f48b612, SUC_lexicon_tags_7 },
    { 0xa568b615, SUC_lexicon_tags_2 },
    { 0x41a8b615, SUC_lexicon_tags_15 },
    { 0x7948b615, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a8b61c, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc8b620, SUC_lexicon_tags_26 },
    { 0x7bc8b621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa68b625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc88b629, SUC_lexicon_tags_1 },
    { 0x43e8b62a, SUC_lexicon_tags_2 },
    { 0x5208b62b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6428b62d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e28b630, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa748b634, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3548b639, SUC_lexicon_tags_45 },
    { 0xe188b63a, SUC_lexicon_tags_37 },
    { 0xdfa8b63b, SUC_lexicon_tags_19 },
    { 0xf5c8b63c, SUC_lexicon_tags_37 },
    { 0xa5e8b639, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa8b645, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c8b648, SUC_lexicon_tags_1 },
    { 0x8828b648, SUC_lexicon_tags_14 },
    { 0xb768b64a, SUC_lexicon_tags_17 },
    { 0xd488b64b, SUC_lexicon_tags_4 },
    { 0x1f88b64b, SUC_lexicon_tags_17 },
    { 0x58c8b64d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa128b655, SUC_lexicon_tags_17 },
    { 0x5848b656, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd5a8b658, SUC_lexicon_tags_46 },
    { 0x448b658, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a8b65d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb28b662, SUC_lexicon_tags_69 },
    { 0xc128b663, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc8b666, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8de8b668, SUC_lexicon_tags_7 },
    { 0x1c68b669, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc8b670, SUC_lexicon_tags_1 },
    { 0x8568b671, SUC_lexicon_tags_25 },
    { 0x8e68b672, SUC_lexicon_tags_124 },
    { 0x4d28b670, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c8b677, SUC_lexicon_tags_0 },
    { 0xcd48b677, SUC_lexicon_tags_37 },
    { 0xcbc8b679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c8b67c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc308b681, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c8b685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c8b689, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5948b691, SUC_lexicon_tags_2 },
    { 0xd5e8b692, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa008b696, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4948b698, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xca48b69b, SUC_lexicon_tags_1 },
    { 0x6568b69c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd648b6a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe408b6a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3128b6a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8848b6ae, SUC_lexicon_tags_1 },
    { 0xa808b6ae, SUC_lexicon_tags_47 },
    { 0x9168b6af, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b48b6b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf328b6b5, SUC_lexicon_tags_7 },
    { 0xc3a8b6b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd28b6b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb828b6ba, SUC_lexicon_tags_75 },
    { 0x5da8b6bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1948b6be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72c8b6c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c28b6c6, SUC_lexicon_tags_7 },
    { 0x1128b6c6, SUC_lexicon_tags_11 },
    { 0x6b28b6c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50a8b6ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8b6cd, SUC_lexicon_tags_33 },
    { 0xae48b6ce, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca68b6d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3328b6d8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6788b6da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2068b6de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8b6e1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a48b6e4, SUC_lexicon_tags_37 },
    { 0x1e08b6e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e8b6f2, SUC_lexicon_tags_8 },
    { 0x5fa8b6f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868b6f8, SUC_lexicon_tags_33 },
    { 0x5368b6f8, SUC_lexicon_tags_37 },
    { 0xda68b6fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8568b6ff, SUC_lexicon_tags_2 },
    { 0x36c8b700, SUC_lexicon_tags_11 },
    { 0x9328b701, SUC_lexicon_tags_60 },
    { 0x2928b6ff, SUC_lexicon_tags_33 },
    { 0x2928b703, SUC_lexicon_tags_17 },
    { 0xbae8b703, SUC_lexicon_tags_15 },
    { 0x6608b705, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e8b709, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3888b70c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc8b710, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c8b712, SUC_lexicon_tags_47 },
    { 0x5848b713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe948b715, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8f68b717, SUC_lexicon_tags_11 },
    { 0x3968b717, SUC_lexicon_tags_14 },
    { 0xe168b717, SUC_lexicon_tags_7 },
    { 0x168b71a, SUC_lexicon_tags_25 },
    { 0xb988b71b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc868b71d, SUC_lexicon_tags_124 },
    { 0x5748b71d, SUC_lexicon_tags_11 },
    { 0x1c08b71f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8628b722, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c8b727, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89a8b729, SUC_lexicon_tags_82 },
    { 0xab28b729, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x86e8b72c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc988b72e, SUC_lexicon_tags_22 },
    { 0x9168b72e, SUC_lexicon_tags_15 },
    { 0xb4e8b730, SUC_lexicon_tags_11 },
    { 0xd328b72e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x58e8b733, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c8b736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3048b738, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4408b73a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44a8b73c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc8b73e, SUC_lexicon_tags_7 },
    { 0xcc28b73f, SUC_lexicon_tags_37 },
    { 0x91a8b740, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b88b742, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf08b744, SUC_lexicon_tags_68 },
    { 0x6c48b744, SUC_lexicon_tags_4 },
    { 0x9948b745, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c48b748, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea8b74e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb648b751, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e8b756, SUC_lexicon_tags_27 },
    { 0x3c28b757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf488b75b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xda08b75e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7e8b760, SUC_lexicon_tags_12 },
    { 0x9c88b760, SUC_lexicon_tags_32 },
    { 0xd488b762, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xde28b764, SUC_lexicon_tags_17 },
    { 0x8fa8b765, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6948b767, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb48b76a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae8b76d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc88b771, SUC_lexicon_tags_27 },
    { 0xae48b771, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd268b779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc888b77b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c88b77f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc8b782, SUC_lexicon_tags_49 },
    { 0x9c88b783, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a8b793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda8b797, SUC_lexicon_tags_31 },
    { 0x97c8b798, SUC_lexicon_tags_27 },
    { 0x508b799, SUC_lexicon_tags_17 },
    { 0x2d08b797, SUC_lexicon_tags_17 },
    { 0xe1c8b79b, SUC_lexicon_tags_25 },
    { 0xbc68b797, SUC_lexicon_tags_26 },
    { 0xd9a8b79b, SUC_lexicon_tags_57 },
    { 0xe328b79b, SUC_lexicon_tags_23 },
    { 0x4d28b79f, SUC_lexicon_tags_37 },
    { 0xa9a8b7a0, SUC_lexicon_tags_39 },
    { 0xe08b7a1, SUC_lexicon_tags_124 },
    { 0x1ec8b7a1, SUC_lexicon_tags_2 },
    { 0xb8c8b7a3, SUC_lexicon_tags_124 },
    { 0x7888b7a2, SUC_lexicon_tags_33 },
    { 0xa128b79d, SUC_lexicon_tags_27 },
    { 0x7c8b79e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f28b7a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7ec8b7ab, SUC_lexicon_tags_17 },
    { 0x8cc8b7ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c8b7af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb168b7b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84c8b7b7, SUC_lexicon_tags_14 },
    { 0x5428b7b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3408b7bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5b88b7bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa588b7c0, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de8b7c5, SUC_lexicon_tags_106 },
    { 0x8a48b7c5, SUC_lexicon_tags_27 },
    { 0x10a8b7c7, SUC_lexicon_tags_38 },
    { 0xdb88b7c6, SUC_lexicon_tags_17 },
    { 0x3a08b7c9, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c8b7cc, SUC_lexicon_tags_65 },
    { 0xe968b7cd, SUC_lexicon_tags_27 },
    { 0xafe8b7cd, SUC_lexicon_tags_23 },
    { 0x20e8b7cd, SUC_lexicon_tags_652 },
    { 0xc188b7d0, SUC_lexicon_tags_17 },
    { 0xe1e8b7cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c8b7d8, SUC_lexicon_tags_37 },
    { 0xd368b7d9, SUC_lexicon_tags_25 },
    { 0x1fa8b7da, SUC_lexicon_tags_87 },
    { 0x1148b7db, SUC_lexicon_tags_10 },
    { 0x5dc8b7dc, SUC_lexicon_tags_47 },
    { 0x1c88b7dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc08b7df, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c8b7e2, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce68b7e9, SUC_lexicon_tags_12 },
    { 0x8988b7e9, SUC_lexicon_tags_43 },
    { 0xcac8b7eb, SUC_lexicon_tags_11 },
    { 0xfc8b7eb, SUC_lexicon_tags_66 },
    { 0xcc8b7ec, SUC_lexicon_tags_25 },
    { 0x6b48b7ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e8b7f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde08b7f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e8b7fe, SUC_lexicon_tags_25 },
    { 0x1d28b7ff, SUC_lexicon_tags_14 },
    { 0x848b800, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3e8b802, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48b805, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa8b80a, SUC_lexicon_tags_29 },
    { 0x8fa8b80a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c68b80f, SUC_lexicon_tags_12 },
    { 0xf5e8b810, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d08b813, SUC_lexicon_tags_27 },
    { 0xdac8b814, SUC_lexicon_tags_47 },
    { 0x5d48b815, SUC_lexicon_tags_47 },
    { 0x2de8b816, SUC_lexicon_tags_37 },
    { 0x2868b813, SUC_lexicon_tags_14 },
    { 0xeac8b817, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xef08b81a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1668b81c, SUC_lexicon_tags_11 },
    { 0xc788b81d, SUC_lexicon_tags_68 },
    { 0xdf08b81d, SUC_lexicon_tags_17 },
    { 0xeca8b81c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x248b821, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f68b823, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc608b827, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb488b829, SUC_lexicon_tags_12 },
    { 0x4448b829, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e8b832, SUC_lexicon_tags_27 },
    { 0x1748b832, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23c8b835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb08b837, SUC_lexicon_tags_1 },
    { 0xb1c8b838, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68b844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5388b849, SUC_lexicon_tags_4 },
    { 0x6d48b84a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc528b84e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36e8b850, SUC_lexicon_tags_15 },
    { 0x71e8b851, SUC_lexicon_tags_25 },
    { 0x7228b852, SUC_lexicon_tags_63 },
    { 0xbaa8b850, SUC_lexicon_tags_37 },
    { 0xdc28b854, SUC_lexicon_tags_4 },
    { 0x7128b855, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a28b85a, SUC_lexicon_tags_63 },
    { 0x5ba8b85b, SUC_lexicon_tags_37 },
    { 0x46a8b85a, SUC_lexicon_tags_10 },
    { 0x9c08b85b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8b860, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7108b866, SUC_lexicon_tags_37 },
    { 0xbf08b866, SUC_lexicon_tags_47 },
    { 0x2de8b868, SUC_lexicon_tags_65 },
    { 0xbc08b867, SUC_lexicon_tags_2 },
    { 0xe0c8b86a, SUC_lexicon_tags_7 },
    { 0x4f08b86b, SUC_lexicon_tags_37 },
    { 0x9a8b86b, SUC_lexicon_tags_37 },
    { 0xc768b86c, SUC_lexicon_tags_2 },
    { 0x2208b868, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7028b874, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8068b876, SUC_lexicon_tags_15 },
    { 0xa48b877, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee8b87c, SUC_lexicon_tags_37 },
    { 0x308b87d, SUC_lexicon_tags_2 },
    { 0x3848b87e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d88b881, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5668b885, SUC_lexicon_tags_0 },
    { 0x6868b886, SUC_lexicon_tags_37 },
    { 0xb768b887, SUC_lexicon_tags_47 },
    { 0x5048b888, SUC_lexicon_tags_1 },
    { 0x6ba8b885, SUC_lexicon_tags_45 },
    { 0x17c8b88a, SUC_lexicon_tags_29 },
    { 0x4908b887, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a8b895, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c88b897, SUC_lexicon_tags_1 },
    { 0x79c8b898, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd2e8b89a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ca8b89c, SUC_lexicon_tags_25 },
    { 0xbd68b89c, SUC_lexicon_tags_100 },
    { 0x71e8b89c, SUC_lexicon_tags_17 },
    { 0x76e8b89c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e8b8a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d28b8ab, SUC_lexicon_tags_29 },
    { 0x47a8b8ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8b8af, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbc48b8b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd748b8b4, SUC_lexicon_tags_37 },
    { 0x8fc8b8b5, SUC_lexicon_tags_17 },
    { 0x2928b8b6, SUC_lexicon_tags_27 },
    { 0x5648b8b7, SUC_lexicon_tags_45 },
    { 0x5348b8b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd28b8bb, SUC_lexicon_tags_25 },
    { 0xe5e8b8bc, SUC_lexicon_tags_10 },
    { 0x168b8bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a8b8c7, SUC_lexicon_tags_33 },
    { 0xb768b8c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d48b8cc, SUC_lexicon_tags_1 },
    { 0xd548b8cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7468b8cf, SUC_lexicon_tags_47 },
    { 0xdfe8b8d0, SUC_lexicon_tags_26 },
    { 0xe828b8d1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e8b8d4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1208b8d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae08b8d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe508b8dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e8b8de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf88b8e2, SUC_lexicon_tags_32 },
    { 0xc168b8e3, SUC_lexicon_tags_34 },
    { 0xa668b8e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75a8b8e6, SUC_lexicon_tags_234 },
    { 0xf768b8e7, SUC_lexicon_tags_1 },
    { 0x7f28b8e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf28b8f3, SUC_lexicon_tags_7 },
    { 0xea08b8f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb708b8f7, SUC_lexicon_tags_1 },
    { 0x4348b8f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4888b8fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f88b8fe, SUC_lexicon_tags_39 },
    { 0x5e88b8ff, SUC_lexicon_tags_113 },
    { 0x5f48b900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5648b902, SUC_lexicon_tags_26 },
    { 0xc2c8b903, SUC_lexicon_tags_11 },
    { 0xdd28b904, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x31a8b906, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1668b909, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c8b90d, SUC_lexicon_tags_16 },
    { 0x5228b90e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1008b915, SUC_lexicon_tags_29 },
    { 0x2528b915, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf68b91a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c68b91e, SUC_lexicon_tags_16 },
    { 0x9ec8b91f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcec8b921, SUC_lexicon_tags_33 },
    { 0x7d08b922, SUC_lexicon_tags_2 },
    { 0xc6c8b922, SUC_lexicon_tags_37 },
    { 0x6308b923, SUC_lexicon_tags_27 },
    { 0xe448b922, SUC_lexicon_tags_2 },
    { 0xd408b921, SUC_lexicon_tags_1 },
    { 0x6448b927, SUC_lexicon_tags_63 },
    { 0x5a28b928, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb48b92a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc8b92e, SUC_lexicon_tags_37 },
    { 0x8b88b92f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc608b935, SUC_lexicon_tags_37 },
    { 0xfb68b936, SUC_lexicon_tags_38 },
    { 0xdc28b935, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9848b943, SUC_lexicon_tags_28 },
    { 0x3d68b944, SUC_lexicon_tags_12 },
    { 0x20a8b945, SUC_lexicon_tags_25 },
    { 0x3f68b946, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ea8b948, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3e8b94a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4208b94d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c8b950, SUC_lexicon_tags_27 },
    { 0x6a88b950, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab28b955, SUC_lexicon_tags_60 },
    { 0x9b28b956, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c8b95a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe828b967, SUC_lexicon_tags_12 },
    { 0xcb28b968, SUC_lexicon_tags_35 },
    { 0xf6c8b967, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4768b96d, SUC_lexicon_tags_33 },
    { 0x7968b96e, SUC_lexicon_tags_30 },
    { 0x85e8b96e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d68b971, SUC_lexicon_tags_15 },
    { 0xca88b971, SUC_lexicon_tags_3 },
    { 0x4ea8b972, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa428b976, SUC_lexicon_tags_15 },
    { 0x2128b977, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1868b979, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdec8b97b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a08b984, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b88b98c, SUC_lexicon_tags_33 },
    { 0x7128b98d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c8b998, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3be8b99a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc8b99e, SUC_lexicon_tags_7 },
    { 0xb9a8b99e, SUC_lexicon_tags_1 },
    { 0x1348b9a0, SUC_lexicon_tags_19 },
    { 0x528b99e, SUC_lexicon_tags_7 },
    { 0xbcc8b99f, SUC_lexicon_tags_11 },
    { 0xb808b9a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8b9a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x648b9aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c8b9ac, SUC_lexicon_tags_37 },
    { 0x5ea8b9ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb68b9b0, SUC_lexicon_tags_63 },
    { 0x8ba8b9b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf48b9b3, SUC_lexicon_tags_1 },
    { 0x9e8b9b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a68b9b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8248b9bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec8b9c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71a8b9c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf928b9c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2268b9c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc88b9cc, SUC_lexicon_tags_17 },
    { 0xa3e8b9cc, SUC_lexicon_tags_23 },
    { 0xdc88b9cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ee8b9d0, SUC_lexicon_tags_0 },
    { 0xa7c8b9d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a8b9d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2528b9d8, SUC_lexicon_tags_37 },
    { 0x3a08b9d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b48b9db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28b9df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c8b9e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe068b9e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5768b9eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3208b9ef, SUC_lexicon_tags_19 },
    { 0x928b9f0, SUC_lexicon_tags_4 },
    { 0x5888b9ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4008b9f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a8b9f7, SUC_lexicon_tags_19 },
    { 0xe1a8b9f8, SUC_lexicon_tags_37 },
    { 0x8648b9f9, SUC_lexicon_tags_45 },
    { 0xa088b9fa, SUC_lexicon_tags_43 },
    { 0xb9a8b9fb, SUC_lexicon_tags_2 },
    { 0x8028b9fb, SUC_lexicon_tags_12 },
    { 0xcd68b9f8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8ba01, SUC_lexicon_tags_33 },
    { 0x78c8ba02, SUC_lexicon_tags_11 },
    { 0xde68ba02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1fa8ba05, SUC_lexicon_tags_12 },
    { 0xa088ba05, SUC_lexicon_tags_43 },
    { 0xbfc8ba06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc8ba0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c68ba11, SUC_lexicon_tags_1 },
    { 0xfb28ba11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x24a8ba14, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35a8ba16, SUC_lexicon_tags_38 },
    { 0xf768ba16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7b28ba19, SUC_lexicon_tags_17 },
    { 0xca88ba1a, SUC_lexicon_tags_19 },
    { 0x3c8ba1a, SUC_lexicon_tags_37 },
    { 0x9f08ba1c, SUC_lexicon_tags_11 },
    { 0x8668ba1c, SUC_lexicon_tags_68 },
    { 0x9288ba1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd588ba20, SUC_lexicon_tags_77 },
    { 0x75c8ba21, SUC_lexicon_tags_45 },
    { 0x2be8ba22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9728ba29, SUC_lexicon_tags_33 },
    { 0xc148ba2a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xccc8ba2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ba30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe268ba3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xac68ba3e, SUC_lexicon_tags_8 },
    { 0xba88ba3e, SUC_lexicon_tags_19 },
    { 0xc388ba3f, SUC_lexicon_tags_19 },
    { 0x6628ba41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2548ba48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f68ba4b, SUC_lexicon_tags_32 },
    { 0x1988ba4c, SUC_lexicon_tags_29 },
    { 0x1628ba4c, SUC_lexicon_tags_2 },
    { 0x2fa8ba4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a8ba51, SUC_lexicon_tags_32 },
    { 0x86e8ba52, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x308ba55, SUC_lexicon_tags_12 },
    { 0x7dc8ba55, SUC_lexicon_tags_11 },
    { 0xefc8ba57, SUC_lexicon_tags_7 },
    { 0xdd88ba58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe588ba5c, SUC_lexicon_tags_7 },
    { 0x5d08ba5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1808ba5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19e8ba61, SUC_lexicon_tags_63 },
    { 0x3368ba61, SUC_lexicon_tags_13 },
    { 0x84e8ba63, SUC_lexicon_tags_17 },
    { 0x6d28ba64, SUC_lexicon_tags_87 },
    { 0x1a48ba65, SUC_lexicon_tags_5 },
    { 0x5428ba64, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c48ba69, SUC_lexicon_tags_35 },
    { 0xe788ba6a, SUC_lexicon_tags_10 },
    { 0xbc08ba6a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a8ba72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c8ba77, SUC_lexicon_tags_30 },
    { 0x58c8ba78, SUC_lexicon_tags_15 },
    { 0x4a08ba79, SUC_lexicon_tags_9 },
    { 0xe388ba79, SUC_lexicon_tags_63 },
    { 0x7108ba78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9048ba7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa508ba81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d68ba83, SUC_lexicon_tags_30 },
    { 0x4928ba84, SUC_lexicon_tags_6 },
    { 0xb7a8ba85, SUC_lexicon_tags_63 },
    { 0x5ce8ba84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe308ba89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea8ba93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6c48ba95, SUC_lexicon_tags_30 },
    { 0x56e8ba96, SUC_lexicon_tags_2 },
    { 0x81c8ba97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e8baa5, SUC_lexicon_tags_29 },
    { 0x9668baa6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3bc8baa8, SUC_lexicon_tags_29 },
    { 0xefa8baa9, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0xeec8baab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48bab0, SUC_lexicon_tags_23 },
    { 0x4748bab0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c8bab8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988babc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb728babf, SUC_lexicon_tags_55 },
    { 0x5a88babf, SUC_lexicon_tags_34 },
    { 0xba8bac1, SUC_lexicon_tags_17 },
    { 0xf8a8bac2, SUC_lexicon_tags_6 },
    { 0x27a8babf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e8bac7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef68baca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4908bace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e8bad1, SUC_lexicon_tags_15 },
    { 0x5ac8bad2, SUC_lexicon_tags_12 },
    { 0x5648bad2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8928bad6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb488badd, SUC_lexicon_tags_46 },
    { 0x3e28bade, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9368bae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a8baea, SUC_lexicon_tags_27 },
    { 0xd7c8baeb, SUC_lexicon_tags_37 },
    { 0x4e88baeb, SUC_lexicon_tags_37 },
    { 0x3308baec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a88baef, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x4728baf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f68baf4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc8baf7, SUC_lexicon_tags_13 },
    { 0xb1e8baf8, SUC_lexicon_tags_7 },
    { 0x5e48baf9, SUC_lexicon_tags_7 },
    { 0x1a28bafa, SUC_lexicon_tags_43 },
    { 0x56c8bafa, SUC_lexicon_tags_17 },
    { 0x6ea8bafc, SUC_lexicon_tags_10 },
    { 0x6b48bafd, SUC_lexicon_tags_21 },
    { 0x8e08bafe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93c8bb00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da8bb05, SUC_lexicon_tags_17 },
    { 0x5e28bb06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf788bb0a, SUC_lexicon_tags_2 },
    { 0xb408bb0b, SUC_lexicon_tags_11 },
    { 0xc2a8bb0b, SUC_lexicon_tags_25 },
    { 0x3d08bb0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40a8bb0f, SUC_lexicon_tags_47 },
    { 0xce8bb10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a8bb14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb28bb17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1308bb20, SUC_lexicon_tags_11 },
    { 0x8e48bb21, SUC_lexicon_tags_25 },
    { 0x3ca8bb21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8128bb24, SUC_lexicon_tags_37 },
    { 0x2c8bb24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528bb29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc148bb2b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7d88bb2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a68bb33, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9088bb35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4428bb37, SUC_lexicon_tags_21 },
    { 0x2ba8bb37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee08bb3a, SUC_lexicon_tags_2 },
    { 0x6768bb3b, SUC_lexicon_tags_12 },
    { 0xf0c8bb3b, SUC_lexicon_tags_23 },
    { 0x7508bb3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1568bb40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c8bb4a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da8bb51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6648bb54, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e8bb58, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7388bb5f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb368bb62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c8bb65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff48bb71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4508bb73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7148bb75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1628bb7c, SUC_lexicon_tags_8 },
    { 0xc0a8bb7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2508bb7f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xad08bb82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc08bb84, SUC_lexicon_tags_2 },
    { 0x7ec8bb85, SUC_lexicon_tags_37 },
    { 0x7828bb86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c28bb8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c8bb8d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c8bb90, SUC_lexicon_tags_38 },
    { 0x17e8bb91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef88bb95, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x408bb97, SUC_lexicon_tags_12 },
    { 0x27a8bb98, SUC_lexicon_tags_4 },
    { 0x4f68bb99, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee68bb9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe168bb9f, SUC_lexicon_tags_17 },
    { 0xee48bb9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7668bba2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf428bba6, SUC_lexicon_tags_21 },
    { 0xdf48bba7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaa08bba9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b48bbac, SUC_lexicon_tags_11 },
    { 0x5168bbac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7668bbb2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b88bbb4, SUC_lexicon_tags_8 },
    { 0xc708bbb4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe8bbb8, SUC_lexicon_tags_23 },
    { 0x6a48bbb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3888bbbc, SUC_lexicon_tags_47 },
    { 0x4508bbbc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8928bbc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528bbcb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe8bbce, SUC_lexicon_tags_12 },
    { 0x7d08bbce, SUC_lexicon_tags_37 },
    { 0x8668bbd0, SUC_lexicon_tags_55 },
    { 0x10a8bbd1, SUC_lexicon_tags_49 },
    { 0xfc88bbd2, SUC_lexicon_tags_11 },
    { 0xcd88bbd1, SUC_lexicon_tags_1 },
    { 0xcb48bbd1, SUC_lexicon_tags_21 },
    { 0xabe8bbd5, SUC_lexicon_tags_18 },
    { 0x1da8bbd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5108bbda, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7d68bbdc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xec28bbde, SUC_lexicon_tags_32 },
    { 0x31a8bbdf, SUC_lexicon_tags_1 },
    { 0x64e8bbe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a8bbe4, SUC_lexicon_tags_12 },
    { 0x388bbe5, SUC_lexicon_tags_7 },
    { 0xc0e8bbe6, SUC_lexicon_tags_37 },
    { 0xf608bbe7, SUC_lexicon_tags_127 },
    { 0x1d88bbe7, SUC_lexicon_tags_15 },
    { 0xe328bbe6, SUC_lexicon_tags_38 },
    { 0xd5c8bbea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd88bbf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d68bbf5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a8bbfc, SUC_lexicon_tags_7 },
    { 0x4f48bbfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc148bc01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc08bc03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a8bc07, SUC_lexicon_tags_1 },
    { 0x10e8bc08, SUC_lexicon_tags_32 },
    { 0x8fa8bc08, SUC_lexicon_tags_13 },
    { 0x3ea8bc0a, SUC_lexicon_tags_15 },
    { 0xdaa8bc0b, SUC_lexicon_tags_179 },
    { 0x2f88bc08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2568bc0f, SUC_lexicon_tags_37 },
    { 0x9408bc10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa928bc12, SUC_lexicon_tags_33 },
    { 0x9a28bc12, SUC_lexicon_tags_17 },
    { 0x6828bc14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd28bc18, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5be8bc1a, SUC_lexicon_tags_12 },
    { 0xe4a8bc1b, SUC_lexicon_tags_17 },
    { 0xf448bc1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb568bc24, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc728bc28, SUC_lexicon_tags_120 },
    { 0xde68bc29, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e28bc2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f88bc2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce28bc30, SUC_lexicon_tags_38 },
    { 0xc5a8bc31, SUC_lexicon_tags_29 },
    { 0xcfe8bc31, SUC_lexicon_tags_46 },
    { 0xbdc8bc31, SUC_lexicon_tags_21 },
    { 0x9b68bc33, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2588bc3a, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc8bc3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7288bc44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3028bc46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeda8bc48, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc8bc4b, SUC_lexicon_tags_37 },
    { 0xfbe8bc4c, SUC_lexicon_tags_9 },
    { 0xafc8bc4d, SUC_lexicon_tags_63 },
    { 0x4928bc4e, SUC_lexicon_tags_56 },
    { 0x3ce8bc4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc568bc51, SUC_lexicon_tags_46 },
    { 0xd8c8bc52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1488bc55, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb0a8bc57, SUC_lexicon_tags_6 },
    { 0x9c68bc58, SUC_lexicon_tags_28 },
    { 0x448bc59, SUC_lexicon_tags_31 },
    { 0x5f08bc59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c8bc5d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc8bc63, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdb28bc65, SUC_lexicon_tags_141 },
    { 0xbec8bc66, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x2628bc68, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e8bc6b, SUC_lexicon_tags_9 },
    { 0xc0c8bc6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e8bc6f, SUC_lexicon_tags_37 },
    { 0x83a8bc70, SUC_lexicon_tags_10 },
    { 0xaf68bc71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbda8bc73, SUC_lexicon_tags_27 },
    { 0x9de8bc73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcca8bc76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2248bc7b, SUC_lexicon_tags_23 },
    { 0x5c28bc7c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa8bc83, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a8bc86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc168bc88, SUC_lexicon_tags_25 },
    { 0x76a8bc89, SUC_lexicon_tags_57 },
    { 0xf3e8bc8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80e8bc8c, SUC_lexicon_tags_17 },
    { 0xfa88bc8c, SUC_lexicon_tags_79 },
    { 0xdd28bc8e, SUC_lexicon_tags_11 },
    { 0x67a8bc8e, SUC_lexicon_tags_21 },
    { 0xb788bc8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a28bc94, SUC_lexicon_tags_2 },
    { 0x5fa8bc95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff28bc97, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7188bc99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf48bc9b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x868bc9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1928bca2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6388bca4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x72c8bca6, SUC_lexicon_tags_47 },
    { 0x848bca7, SUC_lexicon_tags_4 },
    { 0x4e28bca8, SUC_lexicon_tags_15 },
    { 0x65a8bca7, SUC_lexicon_tags_8 },
    { 0xbe48bca9, SUC_lexicon_tags_57 },
    { 0x1aa8bca7, SUC_lexicon_tags_48 },
    { 0xd288bcac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7068bcaf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca08bcb3, SUC_lexicon_tags_26 },
    { 0x5a48bcb3, SUC_lexicon_tags_23 },
    { 0x69a8bcb5, SUC_lexicon_tags_15 },
    { 0xc228bcb6, SUC_lexicon_tags_35 },
    { 0x5a48bcb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb708bcba, SUC_lexicon_tags_2 },
    { 0xff08bcba, SUC_lexicon_tags_16 },
    { 0xed28bcbc, SUC_lexicon_tags_23 },
    { 0xf0a8bcba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7928bcc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e8bcca, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2008bcd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4888bcdc, SUC_lexicon_tags_46 },
    { 0xe7a8bcdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe8bce0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27c8bce3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd528bce5, SUC_lexicon_tags_37 },
    { 0xaee8bce6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe608bce8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e8bceb, SUC_lexicon_tags_12 },
    { 0xb0e8bcec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3028bcf2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc468bcf9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7928bcfb, SUC_lexicon_tags_37 },
    { 0x9208bcfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2948bd10, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9068bd1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x888bd1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc8bd20, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc008bd22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8bd28, SUC_lexicon_tags_17 },
    { 0xc3a8bd29, SUC_lexicon_tags_25 },
    { 0x6c28bd2a, SUC_lexicon_tags_15 },
    { 0x1a48bd2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0a8bd2d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa8bd32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf48bd34, SUC_lexicon_tags_14 },
    { 0xc088bd35, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da8bd38, SUC_lexicon_tags_17 },
    { 0x2e8bd39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe908bd3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca8bd42, SUC_lexicon_tags_30 },
    { 0x3268bd42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x508bd45, SUC_lexicon_tags_2 },
    { 0xac08bd45, SUC_lexicon_tags_7 },
    { 0x6608bd47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce8bd4b, SUC_lexicon_tags_87 },
    { 0xed28bd4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd468bd51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2288bd54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe8bd57, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e88bd5a, SUC_lexicon_tags_77 },
    { 0xd768bd5b, SUC_lexicon_tags_17 },
    { 0x27c8bd5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8208bd5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14e8bd60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e8bd63, SUC_lexicon_tags_38 },
    { 0xdc88bd64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb0c8bd66, SUC_lexicon_tags_37 },
    { 0xaa08bd67, SUC_lexicon_tags_2 },
    { 0x6948bd68, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x80c8bd6a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x1888bd6d, SUC_lexicon_tags_7 },
    { 0x33c8bd6e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x9428bd71, SUC_lexicon_tags_8 },
    { 0x7328bd72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a8bd77, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd128bd79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9328bd7c, SUC_lexicon_tags_17 },
    { 0xe0c8bd7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5c8bd7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4568bd86, SUC_lexicon_tags_55 },
    { 0x4a48bd87, SUC_lexicon_tags_48 },
    { 0xbfe8bd88, SUC_lexicon_tags_11 },
    { 0xd748bd89, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a8bd8f, SUC_lexicon_tags_11 },
    { 0x9628bd8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe868bd92, SUC_lexicon_tags_0 },
    { 0xd0a8bd92, SUC_lexicon_tags_64 },
    { 0x9788bd94, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1888bd98, SUC_lexicon_tags_63 },
    { 0xf308bd98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e8bd9c, SUC_lexicon_tags_30 },
    { 0x8368bd9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c28bd9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1588bda1, SUC_lexicon_tags_2 },
    { 0xa6e8bda1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a8bda5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92e8bda7, SUC_lexicon_tags_19 },
    { 0xa808bda7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x928bdaa, SUC_lexicon_tags_583 },
    { 0xd3c8bdab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9048bdaf, SUC_lexicon_tags_14 },
    { 0xe728bdb0, SUC_lexicon_tags_19 },
    { 0xe988bdb1, SUC_lexicon_tags_2 },
    { 0xf1a8bdb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b68bdb5, SUC_lexicon_tags_23 },
    { 0xb0c8bdb6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4e8bdb8, SUC_lexicon_tags_7 },
    { 0xf6e8bdb8, SUC_lexicon_tags_13 },
    { 0x15e8bdb8, SUC_lexicon_tags_11 },
    { 0xed48bdb9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5448bdbd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c8bdc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac28bdc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d88bdcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c88bdd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a8bdd4, SUC_lexicon_tags_37 },
    { 0xc168bdd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b28bdd7, SUC_lexicon_tags_45 },
    { 0xba48bdd8, SUC_lexicon_tags_26 },
    { 0x5e88bdd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a68bdde, SUC_lexicon_tags_18 },
    { 0x96a8bdde, SUC_lexicon_tags_47 },
    { 0x48e8bddf, SUC_lexicon_tags_7 },
    { 0xe7c8bdde, SUC_lexicon_tags_12 },
    { 0x8b28bde2, SUC_lexicon_tags_25 },
    { 0xd0a8bde3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x10c8bde5, SUC_lexicon_tags_25 },
    { 0x68a8bde6, SUC_lexicon_tags_8 },
    { 0xcb28bde7, SUC_lexicon_tags_45 },
    { 0x2f08bde7, SUC_lexicon_tags_23 },
    { 0x5608bde9, SUC_lexicon_tags_37 },
    { 0xdde8bde6, SUC_lexicon_tags_65 },
    { 0x8988bdeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a8bdee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde88bdf1, SUC_lexicon_tags_17 },
    { 0xc4a8bdf2, SUC_lexicon_tags_17 },
    { 0x28bdf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2928bdf5, SUC_lexicon_tags_30 },
    { 0xac8bdf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e8be08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd28be0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a8be13, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xce28be15, SUC_lexicon_tags_82 },
    { 0x4d68be16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2288be18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e8be1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c28be1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce8be23, SUC_lexicon_tags_2 },
    { 0xc7a8be23, SUC_lexicon_tags_12 },
    { 0x25c8be24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x38e8be27, SUC_lexicon_tags_35 },
    { 0x8ee8be28, SUC_lexicon_tags_1 },
    { 0xba48be28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e8be30, SUC_lexicon_tags_8 },
    { 0x5ba8be31, SUC_lexicon_tags_38 },
    { 0x63c8be32, SUC_lexicon_tags_15 },
    { 0xe148be33, SUC_lexicon_tags_66 },
    { 0x448be34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e8be38, SUC_lexicon_tags_12 },
    { 0x2588be38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6148be3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c28be3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4968be42, SUC_lexicon_tags_37 },
    { 0x9ec8be43, SUC_lexicon_tags_37 },
    { 0xb848be42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3928be47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb648be49, SUC_lexicon_tags_19 },
    { 0x5448be4a, SUC_lexicon_tags_205 },
    { 0x78a8be4a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3708be52, SUC_lexicon_tags_12 },
    { 0xec28be53, SUC_lexicon_tags_14 },
    { 0x7068be54, SUC_lexicon_tags_1 },
    { 0xbac8be55, SUC_lexicon_tags_11 },
    { 0x1f48be55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188be5c, SUC_lexicon_tags_14 },
    { 0xee88be5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa88be61, SUC_lexicon_tags_7 },
    { 0x74a8be62, SUC_lexicon_tags_0 },
    { 0x1908be61, SUC_lexicon_tags_17 },
    { 0xc348be63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8288be66, SUC_lexicon_tags_37 },
    { 0x4348be66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf08be6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4988be71, SUC_lexicon_tags_37 },
    { 0x2a08be72, SUC_lexicon_tags_14 },
    { 0xd188be73, SUC_lexicon_tags_27 },
    { 0x7568be74, SUC_lexicon_tags_14 },
    { 0x13e8be73, SUC_lexicon_tags_10 },
    { 0x6408be72, SUC_lexicon_tags_27 },
    { 0xd068be77, SUC_lexicon_tags_63 },
    { 0x8188be72, SUC_lexicon_tags_27 },
    { 0xd368be78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1668be7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c8be7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9468be81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5428be85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc188be87, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x95a8be89, SUC_lexicon_tags_2 },
    { 0x8a88be89, SUC_lexicon_tags_29 },
    { 0x6e28be89, SUC_lexicon_tags_12 },
    { 0x5fc8be8c, SUC_lexicon_tags_15 },
    { 0x8ea8be8d, SUC_lexicon_tags_19 },
    { 0xbdc8be8a, SUC_lexicon_tags_1 },
    { 0xf728be8c, SUC_lexicon_tags_2 },
    { 0x77a8be90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5de8be92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a8be97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb08be9b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4508be9e, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x3fa8bea0, SUC_lexicon_tags_37 },
    { 0xe28bea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6c8bea3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf88bea8, SUC_lexicon_tags_15 },
    { 0x9688bea9, SUC_lexicon_tags_2 },
    { 0xdea8beaa, SUC_lexicon_tags_32 },
    { 0x4508beab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2a8bead, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6608beb4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc908beb8, SUC_lexicon_tags_14 },
    { 0xd468beb9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a08bebc, SUC_lexicon_tags_57 },
    { 0xb0c8bebc, SUC_lexicon_tags_82 },
    { 0x7de8bebe, SUC_lexicon_tags_66 },
    { 0xe9e8bebf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa568bec4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c8bec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c8beca, SUC_lexicon_tags_7 },
    { 0x1c48becb, SUC_lexicon_tags_12 },
    { 0x10c8becc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e68bece, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd828bed0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc8bed4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa428bed8, SUC_lexicon_tags_7 },
    { 0x13e8bed9, SUC_lexicon_tags_17 },
    { 0xb2e8beda, SUC_lexicon_tags_47 },
    { 0x95a8bedb, SUC_lexicon_tags_33 },
    { 0x3048bedc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac68bede, SUC_lexicon_tags_37 },
    { 0x5128bede, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7768bee1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3a08bee3, SUC_lexicon_tags_1 },
    { 0x208bee3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf388bee7, SUC_lexicon_tags_40 },
    { 0x6348bee8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d48beee, SUC_lexicon_tags_23 },
    { 0xe2c8beef, SUC_lexicon_tags_0 },
    { 0x2248beef, SUC_lexicon_tags_37 },
    { 0x3e8bef1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd448bef7, SUC_lexicon_tags_25 },
    { 0xa208bef8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c8befc, SUC_lexicon_tags_25 },
    { 0x44c8befc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c28bf00, SUC_lexicon_tags_138 },
    { 0xb9c8bf01, SUC_lexicon_tags_6 },
    { 0xe328bf00, SUC_lexicon_tags_7 },
    { 0x60c8bf03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc768bf0b, SUC_lexicon_tags_27 },
    { 0xf128bf0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb608bf0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4528bf12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe8bf17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd128bf1b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe748bf1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51e8bf20, SUC_lexicon_tags_53 },
    { 0x5e8bf21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9808bf25, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08bf28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2108bf2a, SUC_lexicon_tags_7 },
    { 0x6828bf2a, SUC_lexicon_tags_46 },
    { 0xe568bf2c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8068bf30, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1908bf32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a48bf3c, SUC_lexicon_tags_11 },
    { 0xe9c8bf3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c8bf43, SUC_lexicon_tags_29 },
    { 0x8088bf44, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e8bf48, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5088bf4d, SUC_lexicon_tags_21 },
    { 0x6a88bf4e, SUC_lexicon_tags_15 },
    { 0xaf08bf4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x128bf51, SUC_lexicon_tags_37 },
    { 0xdca8bf51, SUC_lexicon_tags_87 },
    { 0x8748bf53, SUC_lexicon_tags_2 },
    { 0xd668bf53, SUC_lexicon_tags_27 },
    { 0x6e68bf55, SUC_lexicon_tags_17 },
    { 0xaa08bf55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec8bf5c, SUC_lexicon_tags_7 },
    { 0xad28bf5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc8bf62, SUC_lexicon_tags_37 },
    { 0x3d28bf63, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd728bf67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4648bf6d, SUC_lexicon_tags_48 },
    { 0x5348bf6d, SUC_lexicon_tags_33 },
    { 0x8488bf6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc8bf77, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c8bf7a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a8bf7f, SUC_lexicon_tags_25 },
    { 0x6608bf80, SUC_lexicon_tags_16 },
    { 0xfb28bf81, SUC_lexicon_tags_12 },
    { 0x5728bf82, SUC_lexicon_tags_15 },
    { 0x2948bf83, SUC_lexicon_tags_32 },
    { 0xa9e8bf83, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a8bf87, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce08bf8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc8bf8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e8bf92, SUC_lexicon_tags_11 },
    { 0xc048bf93, SUC_lexicon_tags_158 },
    { 0xc008bf94, SUC_lexicon_tags_7 },
    { 0x87e8bf95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8bf99, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8bf9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1a8bf9e, SUC_lexicon_tags_29 },
    { 0xcb48bf9f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c8bfa2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b88bfa7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbec8bfa9, SUC_lexicon_tags_63 },
    { 0x3d48bfaa, SUC_lexicon_tags_21 },
    { 0x83e8bfab, SUC_lexicon_tags_1 },
    { 0x6aa8bfac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7348bfae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d08bfb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7388bfb7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c08bfbb, SUC_lexicon_tags_2 },
    { 0x5568bfbc, SUC_lexicon_tags_11 },
    { 0xa068bfbc, SUC_lexicon_tags_47 },
    { 0x9ae8bfbe, SUC_lexicon_tags_17 },
    { 0xdaa8bfbf, SUC_lexicon_tags_63 },
    { 0xa2c8bfc0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1d28bfc2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac8bfc6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c8bfc9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48bfcf, SUC_lexicon_tags_55 },
    { 0x3ac8bfd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b88bfd6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6928bfd8, SUC_lexicon_tags_15 },
    { 0xcca8bfd8, SUC_lexicon_tags_17 },
    { 0x81c8bfda, SUC_lexicon_tags_56 },
    { 0x5248bfda, SUC_lexicon_tags_11 },
    { 0x7068bfdc, SUC_lexicon_tags_9 },
    { 0x97c8bfdc, SUC_lexicon_tags_16 },
    { 0x21a8bfde, SUC_lexicon_tags_7 },
    { 0x968bfde, SUC_lexicon_tags_11 },
    { 0x4a48bfe0, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x3328bfe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa388bfe4, SUC_lexicon_tags_11 },
    { 0x4248bfe4, SUC_lexicon_tags_468 },
    { 0x8608bfe6, SUC_lexicon_tags_14 },
    { 0xf808bfe6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfc48bfe9, SUC_lexicon_tags_8 },
    { 0x4728bfe9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc448bfee, SUC_lexicon_tags_17 },
    { 0xbdc8bfef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb888bff6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf228bffa, SUC_lexicon_tags_7 },
    { 0x2988bffb, SUC_lexicon_tags_15 },
    { 0x4a88bffc, SUC_lexicon_tags_64 },
    { 0x6d68bffd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6648c001, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d48c004, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e88c006, SUC_lexicon_tags_282 },
    { 0xd588c006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c8c00b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa808c016, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08c01b, SUC_lexicon_tags_158 },
    { 0x9768c01b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4428c022, SUC_lexicon_tags_11 },
    { 0xee88c023, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9648c026, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae8c02b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec8c02e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3408c032, SUC_lexicon_tags_17 },
    { 0x6408c033, SUC_lexicon_tags_63 },
    { 0x4ca8c034, SUC_lexicon_tags_1 },
    { 0x70e8c034, SUC_lexicon_tags_254 },
    { 0xbe48c032, SUC_lexicon_tags_87 },
    { 0xb9a8c035, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228c03f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1aa8c041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea68c044, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5248c046, SUC_lexicon_tags_37 },
    { 0xc88c047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3548c049, SUC_lexicon_tags_31 },
    { 0xe468c04a, SUC_lexicon_tags_46 },
    { 0xa328c04b, SUC_lexicon_tags_11 },
    { 0xee8c04b, SUC_lexicon_tags_43 },
    { 0x35c8c049, SUC_lexicon_tags_74 },
    { 0x5748c04e, SUC_lexicon_tags_16 },
    { 0x5de8c04f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c08c051, SUC_lexicon_tags_33 },
    { 0x1d08c052, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e8c059, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc8c05c, SUC_lexicon_tags_0 },
    { 0x7528c05d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8c061, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7268c064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd08c066, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a8c06a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd728c06d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2468c070, SUC_lexicon_tags_2 },
    { 0xe548c071, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a8c074, SUC_lexicon_tags_15 },
    { 0x18c8c075, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f28c077, SUC_lexicon_tags_63 },
    { 0xdbc8c078, SUC_lexicon_tags_13 },
    { 0xd868c077, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c8c07c, SUC_lexicon_tags_25 },
    { 0xbc88c07d, SUC_lexicon_tags_56 },
    { 0x1ca8c07e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6a8c080, SUC_lexicon_tags_11 },
    { 0x3628c080, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7968c089, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a8c08c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xac88c08f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc48c092, SUC_lexicon_tags_33 },
    { 0x7988c092, SUC_lexicon_tags_63 },
    { 0xf808c092, SUC_lexicon_tags_1 },
    { 0x4bc8c095, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdd28c097, SUC_lexicon_tags_160 },
    { 0xa8a8c097, SUC_lexicon_tags_8 },
    { 0xe468c099, SUC_lexicon_tags_7 },
    { 0x3328c099, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd28c09e, SUC_lexicon_tags_43 },
    { 0xf7c8c09e, SUC_lexicon_tags_26 },
    { 0x5e68c0a0, SUC_lexicon_tags_27 },
    { 0xc5a8c09f, SUC_lexicon_tags_11 },
    { 0xaf88c0a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de8c0a7, SUC_lexicon_tags_29 },
    { 0x43c8c0a8, SUC_lexicon_tags_2 },
    { 0x5208c0a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c48c0ac, SUC_lexicon_tags_49 },
    { 0xfdc8c0ac, SUC_lexicon_tags_27 },
    { 0x5e48c0ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc8c0b4, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xfd48c0b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec8c0b9, SUC_lexicon_tags_60 },
    { 0x26c8c0ba, SUC_lexicon_tags_19 },
    { 0x29a8c0bb, SUC_lexicon_tags_7 },
    { 0x2588c0bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4508c0c2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede8c0c7, SUC_lexicon_tags_37 },
    { 0x5c8c0c8, SUC_lexicon_tags_0 },
    { 0x1b68c0c8, SUC_lexicon_tags_12 },
    { 0xbf68c0c9, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce8c0cd, SUC_lexicon_tags_63 },
    { 0x6208c0cd, SUC_lexicon_tags_30 },
    { 0x59a8c0cf, SUC_lexicon_tags_60 },
    { 0x9428c0d0, SUC_lexicon_tags_11 },
    { 0xdf28c0d1, SUC_lexicon_tags_15 },
    { 0x54c8c0cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6928c0d8, SUC_lexicon_tags_4 },
    { 0xa648c0d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xee68c0dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e8c0e0, SUC_lexicon_tags_25 },
    { 0x8f68c0e1, SUC_lexicon_tags_87 },
    { 0x8fc8c0e0, SUC_lexicon_tags_11 },
    { 0xcb08c0e3, SUC_lexicon_tags_11 },
    { 0xa108c0e4, SUC_lexicon_tags_66 },
    { 0x9108c0e5, SUC_lexicon_tags_12 },
    { 0xb888c0e2, SUC_lexicon_tags_49 },
    { 0xc0e8c0e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2628c0ed, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c8c0f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf848c0f4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8ae8c0f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91c8c0f8, SUC_lexicon_tags_33 },
    { 0x2108c0f9, SUC_lexicon_tags_60 },
    { 0x4cc8c0f8, SUC_lexicon_tags_26 },
    { 0x8928c0fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c68c100, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xa828c102, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1228c105, SUC_lexicon_tags_17 },
    { 0x5e8c105, SUC_lexicon_tags_7 },
    { 0xb768c107, SUC_lexicon_tags_191 },
    { 0x4068c108, SUC_lexicon_tags_15 },
    { 0xe928c107, SUC_lexicon_tags_14 },
    { 0x19c8c10a, SUC_lexicon_tags_80 },
    { 0x23e8c106, SUC_lexicon_tags_10 },
    { 0x1de8c10a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8308c10e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x908c110, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd568c113, SUC_lexicon_tags_11 },
    { 0xfec8c113, SUC_lexicon_tags_17 },
    { 0x7e88c115, SUC_lexicon_tags_60 },
    { 0x72a8c116, SUC_lexicon_tags_10 },
    { 0x57c8c116, SUC_lexicon_tags_33 },
    { 0xf648c116, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe168c11a, SUC_lexicon_tags_15 },
    { 0x3648c11b, SUC_lexicon_tags_91 },
    { 0xa708c11c, SUC_lexicon_tags_55 },
    { 0x41a8c11d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e8c120, SUC_lexicon_tags_10 },
    { 0xe428c121, SUC_lexicon_tags_23 },
    { 0x72a8c120, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4028c128, SUC_lexicon_tags_1 },
    { 0xb4c8c128, SUC_lexicon_tags_19 },
    { 0xb6c8c12a, SUC_lexicon_tags_0 },
    { 0xf728c128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8808c130, SUC_lexicon_tags_26 },
    { 0xb208c131, SUC_lexicon_tags_737 },
    { 0x908c131, SUC_lexicon_tags_7 },
    { 0xdaa8c133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f28c139, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc68c13c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca8c143, SUC_lexicon_tags_11 },
    { 0x3b08c144, SUC_lexicon_tags_17 },
    { 0xcd48c145, SUC_lexicon_tags_27 },
    { 0xe7e8c146, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e08c148, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd268c14a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa8c150, SUC_lexicon_tags_118 },
    { 0xd928c150, SUC_lexicon_tags_37 },
    { 0x69e8c152, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb868c157, SUC_lexicon_tags_11 },
    { 0x4428c158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2aa8c15a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a8c15e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa8c161, SUC_lexicon_tags_63 },
    { 0x7fc8c161, SUC_lexicon_tags_60 },
    { 0x2b88c163, SUC_lexicon_tags_29 },
    { 0x7c68c161, SUC_lexicon_tags_56 },
    { 0x1e88c161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c8c167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec8c16a, SUC_lexicon_tags_9 },
    { 0xa5c8c16b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc48c16e, SUC_lexicon_tags_37 },
    { 0x9a8c16e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1868c172, SUC_lexicon_tags_27 },
    { 0xfc28c173, SUC_lexicon_tags_14 },
    { 0x3ee8c173, SUC_lexicon_tags_15 },
    { 0x8e08c173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9688c177, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x40a8c179, SUC_lexicon_tags_11 },
    { 0x6d48c179, SUC_lexicon_tags_37 },
    { 0x6908c179, SUC_lexicon_tags_14 },
    { 0x5908c17c, SUC_lexicon_tags_1 },
    { 0x5a28c17d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfda8c17f, SUC_lexicon_tags_6 },
    { 0x9f68c180, SUC_lexicon_tags_11 },
    { 0xaca8c181, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c8c184, SUC_lexicon_tags_23 },
    { 0x388c185, SUC_lexicon_tags_31 },
    { 0xa668c186, SUC_lexicon_tags_37 },
    { 0xba8c186, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8f08c189, SUC_lexicon_tags_49 },
    { 0xb228c189, SUC_lexicon_tags_25 },
    { 0xab08c18a, SUC_lexicon_tags_6 },
    { 0xca28c189, SUC_lexicon_tags_19 },
    { 0x3048c18d, SUC_lexicon_tags_27 },
    { 0x8208c18d, SUC_lexicon_tags_33 },
    { 0x708c18f, SUC_lexicon_tags_15 },
    { 0x2048c18a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd68c192, SUC_lexicon_tags_11 },
    { 0x2c88c193, SUC_lexicon_tags_8 },
    { 0x2668c194, SUC_lexicon_tags_43 },
    { 0x5388c195, SUC_lexicon_tags_25 },
    { 0x4708c196, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9328c19d, SUC_lexicon_tags_11 },
    { 0x7b88c19e, SUC_lexicon_tags_31 },
    { 0xf68c19f, SUC_lexicon_tags_37 },
    { 0xe148c19f, SUC_lexicon_tags_1 },
    { 0x7748c19e, SUC_lexicon_tags_9 },
    { 0xcba8c19e, SUC_lexicon_tags_47 },
    { 0x69c8c1a3, SUC_lexicon_tags_11 },
    { 0x9e8c1a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8228c1a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb948c1ae, SUC_lexicon_tags_141 },
    { 0xfa08c1ae, SUC_lexicon_tags_25 },
    { 0x7068c1b0, SUC_lexicon_tags_37 },
    { 0xc1a8c1b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf888c1b3, SUC_lexicon_tags_26 },
    { 0x8ae8c1b4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2068c1b6, SUC_lexicon_tags_1 },
    { 0x8868c1b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf848c1bc, SUC_lexicon_tags_65 },
    { 0xb228c1bd, SUC_lexicon_tags_37 },
    { 0x6e08c1be, SUC_lexicon_tags_63 },
    { 0x3368c1bd, SUC_lexicon_tags_68 },
    { 0x1948c1be, SUC_lexicon_tags_68 },
    { 0x39e8c1bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd88c1c4, SUC_lexicon_tags_15 },
    { 0xd788c1c5, SUC_lexicon_tags_17 },
    { 0x8388c1c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8608c1c8, SUC_lexicon_tags_15 },
    { 0x7f88c1c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c8c1cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee68c1ce, SUC_lexicon_tags_35 },
    { 0x6a8c1ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6788c1d1, SUC_lexicon_tags_7 },
    { 0xa268c1d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x928c1d5, SUC_lexicon_tags_129 },
    { 0x4348c1d6, SUC_lexicon_tags_65 },
    { 0x43a8c1d6, SUC_lexicon_tags_79 },
    { 0x3368c1d8, SUC_lexicon_tags_25 },
    { 0x7be8c1d7, SUC_lexicon_tags_19 },
    { 0xbf48c1da, SUC_lexicon_tags_60 },
    { 0x3308c1db, SUC_lexicon_tags_37 },
    { 0x24c8c1d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd868c1e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e8c1e5, SUC_lexicon_tags_43 },
    { 0x5b68c1e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc68c1e9, SUC_lexicon_tags_37 },
    { 0x3f68c1ea, SUC_lexicon_tags_33 },
    { 0x77c8c1ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc28c1ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a8c1f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbd28c1f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2948c1fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d08c1ff, SUC_lexicon_tags_1 },
    { 0xb688c200, SUC_lexicon_tags_97 },
    { 0xd88c200, SUC_lexicon_tags_33 },
    { 0xa9a8c202, SUC_lexicon_tags_11 },
    { 0xe9a8c200, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfe8c205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba8c208, SUC_lexicon_tags_30 },
    { 0xb188c208, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a8c20e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee8c211, SUC_lexicon_tags_40 },
    { 0xf9a8c211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb08c215, SUC_lexicon_tags_25 },
    { 0x9d08c216, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45e8c218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd568c21d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf8a8c21f, SUC_lexicon_tags_60 },
    { 0xb888c220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a08c222, SUC_lexicon_tags_25 },
    { 0x98a8c222, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb948c226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4088c228, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68c22c, SUC_lexicon_tags_15 },
    { 0x43c8c22d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a08c233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac8c236, SUC_lexicon_tags_2 },
    { 0x3ee8c237, SUC_lexicon_tags_100 },
    { 0x74a8c238, SUC_lexicon_tags_25 },
    { 0x1008c239, SUC_lexicon_tags_25 },
    { 0x6308c23a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac8c240, SUC_lexicon_tags_27 },
    { 0xdae8c241, SUC_lexicon_tags_14 },
    { 0x2228c241, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x248c245, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8fe8c247, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c68c24b, SUC_lexicon_tags_37 },
    { 0x5fa8c24c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe548c251, SUC_lexicon_tags_2 },
    { 0xb348c252, SUC_lexicon_tags_38 },
    { 0xfcc8c252, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6028c25b, SUC_lexicon_tags_25 },
    { 0xe308c25c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e8c261, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb28c269, SUC_lexicon_tags_45 },
    { 0xf7a8c26a, SUC_lexicon_tags_11 },
    { 0x1928c269, SUC_lexicon_tags_7 },
    { 0x9188c26c, SUC_lexicon_tags_7 },
    { 0xd168c26c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8c273, SUC_lexicon_tags_6 },
    { 0xa728c274, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1288c276, SUC_lexicon_tags_4 },
    { 0x3588c277, SUC_lexicon_tags_25 },
    { 0x2288c278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42e8c27a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa8c27d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf388c280, SUC_lexicon_tags_33 },
    { 0x4ee8c281, SUC_lexicon_tags_8 },
    { 0xc508c282, SUC_lexicon_tags_15 },
    { 0x8648c281, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf88c285, SUC_lexicon_tags_7 },
    { 0x8248c286, SUC_lexicon_tags_7 },
    { 0x8648c287, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x5b88c289, SUC_lexicon_tags_63 },
    { 0xfa88c28a, SUC_lexicon_tags_11 },
    { 0x8348c28b, SUC_lexicon_tags_0 },
    { 0x4e8c28b, SUC_lexicon_tags_3 },
    { 0x3608c28d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe8c28f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c8c293, SUC_lexicon_tags_19 },
    { 0xe888c294, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcbc8c296, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a48c29a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c8c29f, SUC_lexicon_tags_23 },
    { 0x4588c2a0, SUC_lexicon_tags_7 },
    { 0xb828c2a1, SUC_lexicon_tags_9 },
    { 0xb968c2a0, SUC_lexicon_tags_39 },
    { 0x5048c2a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e8c2a6, SUC_lexicon_tags_60 },
    { 0xa2a8c2a6, SUC_lexicon_tags_0 },
    { 0x9e68c2a8, SUC_lexicon_tags_87 },
    { 0xc948c2a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e48c2ab, SUC_lexicon_tags_17 },
    { 0x7ae8c2ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a8c2b0, SUC_lexicon_tags_37 },
    { 0x1b68c2b1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8228c2bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4868c2bf, SUC_lexicon_tags_10 },
    { 0xef08c2bf, SUC_lexicon_tags_0 },
    { 0x24a8c2c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb28c2c5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e8c2cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe008c2cd, SUC_lexicon_tags_68 },
    { 0x4c88c2cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e8c2dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x648c2de, SUC_lexicon_tags_529 },
    { 0xece8c2de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4b68c2e1, SUC_lexicon_tags_87 },
    { 0xa8c2e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f28c2e5, SUC_lexicon_tags_60 },
    { 0xf348c2e6, SUC_lexicon_tags_12 },
    { 0x9e88c2e6, SUC_lexicon_tags_39 },
    { 0xaca8c2e7, SUC_lexicon_tags_7 },
    { 0xe5a8c2e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e8c2ec, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c8c2f1, SUC_lexicon_tags_17 },
    { 0x48e8c2f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9788c2f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d88c2fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f68c2ff, SUC_lexicon_tags_1 },
    { 0x58c8c300, SUC_lexicon_tags_39 },
    { 0x3e08c301, SUC_lexicon_tags_87 },
    { 0xe888c302, SUC_lexicon_tags_1 },
    { 0x81c8c2ff, SUC_lexicon_tags_45 },
    { 0x42c8c301, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa948c307, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de8c30e, SUC_lexicon_tags_11 },
    { 0x1e88c30f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba28c311, SUC_lexicon_tags_11 },
    { 0x19e8c312, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68c315, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9988c317, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28a8c319, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a08c31c, SUC_lexicon_tags_22 },
    { 0x3e08c31d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea8c32c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x92e8c32e, SUC_lexicon_tags_29 },
    { 0x35e8c32f, SUC_lexicon_tags_30 },
    { 0x61c8c32e, SUC_lexicon_tags_47 },
    { 0xaba8c32f, SUC_lexicon_tags_17 },
    { 0xa068c32e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb968c334, SUC_lexicon_tags_37 },
    { 0x3c88c335, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88c338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf28c33a, SUC_lexicon_tags_61 },
    { 0xd3c8c33a, SUC_lexicon_tags_47 },
    { 0x828c33c, SUC_lexicon_tags_38 },
    { 0xdbc8c33d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf68c343, SUC_lexicon_tags_43 },
    { 0x8608c343, SUC_lexicon_tags_8 },
    { 0xe728c343, SUC_lexicon_tags_46 },
    { 0xdba8c346, SUC_lexicon_tags_19 },
    { 0x4208c346, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9828c349, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d88c34c, SUC_lexicon_tags_49 },
    { 0xdfc8c34d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x21c8c34f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2a8c351, SUC_lexicon_tags_49 },
    { 0x8208c351, SUC_lexicon_tags_46 },
    { 0x6808c353, SUC_lexicon_tags_10 },
    { 0xfc28c354, SUC_lexicon_tags_77 },
    { 0xb208c355, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c8c358, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd988c35a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x55a8c35c, SUC_lexicon_tags_25 },
    { 0x32a8c35c, SUC_lexicon_tags_17 },
    { 0x1c08c35e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2608c363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6148c367, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaae8c369, SUC_lexicon_tags_27 },
    { 0x3288c36a, SUC_lexicon_tags_43 },
    { 0x6848c36a, SUC_lexicon_tags_25 },
    { 0x568c36c, SUC_lexicon_tags_12 },
    { 0x6c08c36d, SUC_lexicon_tags_49 },
    { 0x8528c36e, SUC_lexicon_tags_11 },
    { 0xb9c8c36f, SUC_lexicon_tags_45 },
    { 0x25a8c36f, SUC_lexicon_tags_14 },
    { 0xf688c371, SUC_lexicon_tags_66 },
    { 0xd428c370, SUC_lexicon_tags_20 },
    { 0xc408c373, SUC_lexicon_tags_11 },
    { 0x1ec8c371, SUC_lexicon_tags_12 },
    { 0x2628c375, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1488c378, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e8c382, SUC_lexicon_tags_45 },
    { 0x77a8c383, SUC_lexicon_tags_17 },
    { 0x45a8c384, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c8c389, SUC_lexicon_tags_23 },
    { 0xa008c38a, SUC_lexicon_tags_1 },
    { 0xa048c389, SUC_lexicon_tags_87 },
    { 0xdb08c38c, SUC_lexicon_tags_12 },
    { 0xd0c8c38d, SUC_lexicon_tags_61 },
    { 0x9ca8c38b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8408c390, SUC_lexicon_tags_17 },
    { 0xe588c391, SUC_lexicon_tags_27 },
    { 0x9468c392, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf48c396, SUC_lexicon_tags_27 },
    { 0xf648c396, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1988c39d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5428c3a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcae8c3a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a8c3a7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9848c3a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d68c3ab, SUC_lexicon_tags_32 },
    { 0xbd08c3ac, SUC_lexicon_tags_17 },
    { 0x9d28c3ac, SUC_lexicon_tags_15 },
    { 0x77c8c3ad, SUC_lexicon_tags_26 },
    { 0x34a8c3af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d28c3b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12e8c3b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x488c3b9, SUC_lexicon_tags_7 },
    { 0xf88c3b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c8c3bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc988c3c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d88c3cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8728c3d1, SUC_lexicon_tags_2 },
    { 0x9948c3d2, SUC_lexicon_tags_19 },
    { 0x2ae8c3d2, SUC_lexicon_tags_16 },
    { 0x8888c3d4, SUC_lexicon_tags_47 },
    { 0xb08c3d2, SUC_lexicon_tags_138 },
    { 0xd5e8c3d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x24e8c3d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe688c3da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc108c3de, SUC_lexicon_tags_101 },
    { 0x6808c3df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa908c3e2, SUC_lexicon_tags_17 },
    { 0xa388c3e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c28c3e9, SUC_lexicon_tags_11 },
    { 0x26c8c3ea, SUC_lexicon_tags_62 },
    { 0x6ea8c3eb, SUC_lexicon_tags_1 },
    { 0xfe28c3eb, SUC_lexicon_tags_25 },
    { 0x7068c3eb, SUC_lexicon_tags_31 },
    { 0xb9a8c3ee, SUC_lexicon_tags_34 },
    { 0xe028c3ea, SUC_lexicon_tags_65 },
    { 0x4388c3ed, SUC_lexicon_tags_1 },
    { 0x2188c3eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2408c3f8, SUC_lexicon_tags_19 },
    { 0xf6a8c3f8, SUC_lexicon_tags_37 },
    { 0xd588c3f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd008c3fd, SUC_lexicon_tags_0 },
    { 0xcee8c3fe, SUC_lexicon_tags_30 },
    { 0x5368c3fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e8c404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3748c408, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa248c40b, SUC_lexicon_tags_39 },
    { 0x4e8c40c, SUC_lexicon_tags_55 },
    { 0xf2a8c40b, SUC_lexicon_tags_37 },
    { 0xe848c40e, SUC_lexicon_tags_0 },
    { 0xca8c40f, SUC_lexicon_tags_37 },
    { 0x5408c40f, SUC_lexicon_tags_60 },
    { 0x33e8c40d, SUC_lexicon_tags_26 },
    { 0xd5c8c40e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8c41e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a8c424, SUC_lexicon_tags_21 },
    { 0xe388c424, SUC_lexicon_tags_1 },
    { 0xbce8c426, SUC_lexicon_tags_85 },
    { 0xc048c427, SUC_lexicon_tags_17 },
    { 0x308c427, SUC_lexicon_tags_17 },
    { 0x8668c426, SUC_lexicon_tags_34 },
    { 0x8588c42a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9a8c42c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8c432, SUC_lexicon_tags_1 },
    { 0xcd88c433, SUC_lexicon_tags_28 },
    { 0x5b28c434, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a08c43a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x748c43c, SUC_lexicon_tags_43 },
    { 0x8dc8c43d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e8c441, SUC_lexicon_tags_27 },
    { 0x488c442, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc8c446, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c8c449, SUC_lexicon_tags_9 },
    { 0x8528c44a, SUC_lexicon_tags_33 },
    { 0x8c8c44b, SUC_lexicon_tags_7 },
    { 0x9388c44b, SUC_lexicon_tags_17 },
    { 0xafa8c44c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x12c8c44f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5208c452, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc28c457, SUC_lexicon_tags_38 },
    { 0xe668c457, SUC_lexicon_tags_8 },
    { 0xe568c459, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xdb48c45b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x328c45d, SUC_lexicon_tags_46 },
    { 0x47a8c45d, SUC_lexicon_tags_7 },
    { 0x22a8c45f, SUC_lexicon_tags_11 },
    { 0x2288c45f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6448c462, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a8c467, SUC_lexicon_tags_2 },
    { 0x2e48c468, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd028c475, SUC_lexicon_tags_321 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e8c47a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb08c480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31e8c482, SUC_lexicon_tags_6 },
    { 0x27a8c483, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd968c486, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4848c488, SUC_lexicon_tags_17 },
    { 0xdc48c488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba8c48e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26c8c490, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb248c494, SUC_lexicon_tags_0 },
    { 0x9c8c495, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8908c49a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c28c49d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a08c49f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e8c4a4, SUC_lexicon_tags_28 },
    { 0xd368c4a5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc408c4aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c8c4ae, SUC_lexicon_tags_2 },
    { 0xd328c4ae, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a8c4b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73e8c4b6, SUC_lexicon_tags_1 },
    { 0x16e8c4b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48c4bb, SUC_lexicon_tags_25 },
    { 0x7b08c4bc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf348c4be, SUC_lexicon_tags_1 },
    { 0xd748c4bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa228c4c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2328c4c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd68c4cd, SUC_lexicon_tags_1 },
    { 0x76e8c4ce, SUC_lexicon_tags_17 },
    { 0xd0c8c4ce, SUC_lexicon_tags_9 },
    { 0x5208c4d0, SUC_lexicon_tags_17 },
    { 0xcd68c4d1, SUC_lexicon_tags_45 },
    { 0x60a8c4ce, SUC_lexicon_tags_37 },
    { 0xd888c4d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd308c4d7, SUC_lexicon_tags_2 },
    { 0xc508c4d7, SUC_lexicon_tags_46 },
    { 0x3908c4d9, SUC_lexicon_tags_2 },
    { 0xf728c4d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e88c4df, SUC_lexicon_tags_45 },
    { 0xd648c4e0, SUC_lexicon_tags_25 },
    { 0x5fa8c4df, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e48c4e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e8c4ea, SUC_lexicon_tags_7 },
    { 0x6ec8c4eb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x8988c4ee, SUC_lexicon_tags_12 },
    { 0xa7c8c4ef, SUC_lexicon_tags_14 },
    { 0x8d68c4ef, SUC_lexicon_tags_27 },
    { 0xf4e8c4f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6c08c4f3, SUC_lexicon_tags_68 },
    { 0xeca8c4f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb328c4f9, SUC_lexicon_tags_14 },
    { 0x20a8c4fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a8c50b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e8c50f, SUC_lexicon_tags_14 },
    { 0xdc48c510, SUC_lexicon_tags_12 },
    { 0xe728c511, SUC_lexicon_tags_107 },
    { 0x448c50f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xd088c515, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9648c517, SUC_lexicon_tags_17 },
    { 0xbc68c518, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c8c51c, SUC_lexicon_tags_37 },
    { 0x27e8c51c, SUC_lexicon_tags_45 },
    { 0x9e88c51d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d28c520, SUC_lexicon_tags_7 },
    { 0xf768c521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c8c529, SUC_lexicon_tags_25 },
    { 0xb8c8c52a, SUC_lexicon_tags_1 },
    { 0x1928c52a, SUC_lexicon_tags_43 },
    { 0x7d68c52c, SUC_lexicon_tags_25 },
    { 0xd868c52b, SUC_lexicon_tags_37 },
    { 0xa3a8c52d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4708c533, SUC_lexicon_tags_11 },
    { 0x1948c533, SUC_lexicon_tags_37 },
    { 0x12c8c535, SUC_lexicon_tags_38 },
    { 0x6dc8c535, SUC_lexicon_tags_84 },
    { 0xdc68c537, SUC_lexicon_tags_46 },
    { 0x8288c537, SUC_lexicon_tags_68 },
    { 0xe208c536, SUC_lexicon_tags_11 },
    { 0x74c8c53a, SUC_lexicon_tags_7 },
    { 0x7868c534, SUC_lexicon_tags_1 },
    { 0x2568c53c, SUC_lexicon_tags_41 },
    { 0x3808c53c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e88c53f, SUC_lexicon_tags_187 },
    { 0xa68c540, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3228c542, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c8c546, SUC_lexicon_tags_7 },
    { 0x5188c546, SUC_lexicon_tags_0 },
    { 0x5d68c547, SUC_lexicon_tags_17 },
    { 0x40c8c548, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e48c54c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c8c54f, SUC_lexicon_tags_47 },
    { 0xda08c550, SUC_lexicon_tags_15 },
    { 0x5788c550, SUC_lexicon_tags_56 },
    { 0xb808c552, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4b88c554, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6788c556, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa08c55a, SUC_lexicon_tags_62 },
    { 0xbb48c55b, SUC_lexicon_tags_25 },
    { 0x8c28c55c, SUC_lexicon_tags_14 },
    { 0x4e28c55b, SUC_lexicon_tags_15 },
    { 0x1b68c55a, SUC_lexicon_tags_0 },
    { 0xfe28c55c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc8c562, SUC_lexicon_tags_25 },
    { 0x3888c563, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48c567, SUC_lexicon_tags_8 },
    { 0x1848c568, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5688c56c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e8c574, SUC_lexicon_tags_83 },
    { 0x328c575, SUC_lexicon_tags_43 },
    { 0x2b68c575, SUC_lexicon_tags_27 },
    { 0xf88c577, SUC_lexicon_tags_17 },
    { 0xd88c577, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xda08c57a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xba48c57d, SUC_lexicon_tags_18 },
    { 0x328c57e, SUC_lexicon_tags_2 },
    { 0x3c68c57e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d88c583, SUC_lexicon_tags_17 },
    { 0xba28c583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a8c58c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e08c58f, SUC_lexicon_tags_11 },
    { 0xbd88c590, SUC_lexicon_tags_26 },
    { 0x4128c591, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c28c593, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe548c597, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xd4a8c599, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8328c59c, SUC_lexicon_tags_25 },
    { 0xee88c59d, SUC_lexicon_tags_28 },
    { 0x2c08c59e, SUC_lexicon_tags_8 },
    { 0x99a8c59f, SUC_lexicon_tags_14 },
    { 0x108c59f, SUC_lexicon_tags_19 },
    { 0x5dc8c59c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdaa8c5a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d28c5a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd588c5b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x388c5b4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbfe8c5b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5008c5b8, SUC_lexicon_tags_17 },
    { 0x8228c5b9, SUC_lexicon_tags_43 },
    { 0x5328c5ba, SUC_lexicon_tags_14 },
    { 0x4028c5b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9388c5bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c8c5c2, SUC_lexicon_tags_12 },
    { 0xf328c5c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7028c5c8, SUC_lexicon_tags_38 },
    { 0x88c5c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1a28c5cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c8c5d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a28c5d9, SUC_lexicon_tags_66 },
    { 0x6b88c5da, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x75c8c5dc, SUC_lexicon_tags_16 },
    { 0xbde8c5dc, SUC_lexicon_tags_37 },
    { 0xd108c5de, SUC_lexicon_tags_160 },
    { 0xc1a8c5de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f48c5e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf68c5e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c8c5ec, SUC_lexicon_tags_9 },
    { 0x308c5ec, SUC_lexicon_tags_237 },
    { 0x1108c5ed, SUC_lexicon_tags_31 },
    { 0x6f28c5ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e8c5f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf28c5fb, SUC_lexicon_tags_21 },
    { 0x94a8c5fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f48c5fe, SUC_lexicon_tags_1 },
    { 0xdd88c5ff, SUC_lexicon_tags_27 },
    { 0xbca8c600, SUC_lexicon_tags_6 },
    { 0x7f08c601, SUC_lexicon_tags_29 },
    { 0xa2e8c602, SUC_lexicon_tags_1 },
    { 0xa208c600, SUC_lexicon_tags_1 },
    { 0x48c601, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa048c609, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7848c60d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc468c60f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ca8c611, SUC_lexicon_tags_14 },
    { 0xca68c612, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa008c617, SUC_lexicon_tags_1 },
    { 0x8a08c618, SUC_lexicon_tags_92 },
    { 0x248c619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c8c61e, SUC_lexicon_tags_30 },
    { 0x29a8c61f, SUC_lexicon_tags_28 },
    { 0xbe48c620, SUC_lexicon_tags_87 },
    { 0x1828c620, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2148c625, SUC_lexicon_tags_11 },
    { 0x46c8c626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c8c629, SUC_lexicon_tags_35 },
    { 0xc88c629, SUC_lexicon_tags_2 },
    { 0xc3e8c62b, SUC_lexicon_tags_21 },
    { 0xbc48c62c, SUC_lexicon_tags_60 },
    { 0xa728c629, SUC_lexicon_tags_25 },
    { 0x5688c62e, SUC_lexicon_tags_11 },
    { 0x9a88c62e, SUC_lexicon_tags_0 },
    { 0xb0e8c630, SUC_lexicon_tags_21 },
    { 0x2908c630, SUC_lexicon_tags_17 },
    { 0x6068c62b, SUC_lexicon_tags_7 },
    { 0x8528c633, SUC_lexicon_tags_27 },
    { 0x7268c62b, SUC_lexicon_tags_33 },
    { 0xf788c635, SUC_lexicon_tags_15 },
    { 0x9e28c636, SUC_lexicon_tags_11 },
    { 0xb4a8c637, SUC_lexicon_tags_2 },
    { 0x5528c637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9868c63b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8108c63e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f08c644, SUC_lexicon_tags_65 },
    { 0xd2c8c645, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488c64a, SUC_lexicon_tags_37 },
    { 0xd008c64b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbba8c64d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9528c657, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xad68c65a, SUC_lexicon_tags_37 },
    { 0x7d48c65a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfec8c65d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbba8c65f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a88c661, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf488c669, SUC_lexicon_tags_25 },
    { 0x7de8c66a, SUC_lexicon_tags_42 },
    { 0xf948c669, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4328c66d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb448c674, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7748c678, SUC_lexicon_tags_11 },
    { 0xe568c678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8c67c, SUC_lexicon_tags_25 },
    { 0x2c88c67d, SUC_lexicon_tags_0 },
    { 0xa148c67d, SUC_lexicon_tags_28 },
    { 0x9d08c67d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3968c682, SUC_lexicon_tags_1109 },
    { 0x8d68c683, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1968c685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c8c689, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d28c68d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8c696, SUC_lexicon_tags_48 },
    { 0x3628c696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c8c69a, SUC_lexicon_tags_1 },
    { 0x1ee8c69b, SUC_lexicon_tags_25 },
    { 0xf108c69c, SUC_lexicon_tags_37 },
    { 0xb248c69c, SUC_lexicon_tags_28 },
    { 0xca68c69c, SUC_lexicon_tags_12 },
    { 0x3b08c69f, SUC_lexicon_tags_17 },
    { 0x3488c6a0, SUC_lexicon_tags_26 },
    { 0x7a28c69f, SUC_lexicon_tags_60 },
    { 0x2d48c69f, SUC_lexicon_tags_25 },
    { 0xfe28c69b, SUC_lexicon_tags_28 },
    { 0xd108c6a2, SUC_lexicon_tags_23 },
    { 0x5988c6a5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7508c6ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac8c6b6, SUC_lexicon_tags_17 },
    { 0x58e8c6b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4a8c6b9, SUC_lexicon_tags_10 },
    { 0xdc88c6b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc408c6be, SUC_lexicon_tags_17 },
    { 0xa088c6be, SUC_lexicon_tags_32 },
    { 0xd1e8c6c0, SUC_lexicon_tags_2 },
    { 0xad88c6c1, SUC_lexicon_tags_0 },
    { 0xe608c6c1, SUC_lexicon_tags_0 },
    { 0xa508c6c0, SUC_lexicon_tags_37 },
    { 0x9788c6c4, SUC_lexicon_tags_25 },
    { 0xc8c8c6c4, SUC_lexicon_tags_6 },
    { 0x2b68c6c6, SUC_lexicon_tags_11 },
    { 0x95e8c6c7, SUC_lexicon_tags_12 },
    { 0x1c68c6c8, SUC_lexicon_tags_26 },
    { 0xab28c6c8, SUC_lexicon_tags_11 },
    { 0x7be8c6ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148c6d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff68c6d4, SUC_lexicon_tags_14 },
    { 0x91c8c6d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d48c6d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba8c6dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8c6e1, SUC_lexicon_tags_7 },
    { 0x6ba8c6e2, SUC_lexicon_tags_7 },
    { 0x75e8c6e1, SUC_lexicon_tags_17 },
    { 0x5228c6e4, SUC_lexicon_tags_11 },
    { 0xe8a8c6e2, SUC_lexicon_tags_19 },
    { 0x9ba8c6e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc8c6e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc168c6eb, SUC_lexicon_tags_17 },
    { 0x808c6ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x748c6ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc6c8c6f1, SUC_lexicon_tags_47 },
    { 0x9a48c6f1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c68c6f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9768c6fe, SUC_lexicon_tags_17 },
    { 0xa7a8c6ff, SUC_lexicon_tags_25 },
    { 0x75a8c6fe, SUC_lexicon_tags_37 },
    { 0xec68c701, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c68c705, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2588c707, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc808c70a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1c8c70c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c8c70f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x16a8c711, SUC_lexicon_tags_11 },
    { 0x2348c712, SUC_lexicon_tags_7 },
    { 0x4188c713, SUC_lexicon_tags_1 },
    { 0x2328c714, SUC_lexicon_tags_15 },
    { 0x3a48c714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9168c718, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3848c71a, SUC_lexicon_tags_30 },
    { 0x1688c71b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac88c71e, SUC_lexicon_tags_27 },
    { 0x9c88c71f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf28c722, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce8c726, SUC_lexicon_tags_17 },
    { 0xf228c727, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe88c729, SUC_lexicon_tags_77 },
    { 0x7e88c72a, SUC_lexicon_tags_12 },
    { 0xf908c72b, SUC_lexicon_tags_37 },
    { 0x4108c72a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x248c72e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff28c737, SUC_lexicon_tags_12 },
    { 0x5148c738, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e48c73a, SUC_lexicon_tags_28 },
    { 0x4ae8c73a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8288c73e, SUC_lexicon_tags_27 },
    { 0xd728c73f, SUC_lexicon_tags_7 },
    { 0xecc8c740, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a8c746, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7368c748, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9088c74b, SUC_lexicon_tags_15 },
    { 0x9ac8c74c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8c74f, SUC_lexicon_tags_49 },
    { 0x4ee8c750, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b88c756, SUC_lexicon_tags_15 },
    { 0xc5c8c757, SUC_lexicon_tags_37 },
    { 0x5b68c758, SUC_lexicon_tags_11 },
    { 0xba68c758, SUC_lexicon_tags_37 },
    { 0x9bc8c758, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae8c761, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf708c765, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a88c768, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec8c76e, SUC_lexicon_tags_12 },
    { 0xa448c76f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7588c771, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb488c774, SUC_lexicon_tags_15 },
    { 0xde68c774, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc8c77a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e8c77f, SUC_lexicon_tags_49 },
    { 0x8208c780, SUC_lexicon_tags_21 },
    { 0x4b28c781, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b08c787, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc8c78c, SUC_lexicon_tags_32 },
    { 0x3e68c78d, SUC_lexicon_tags_7 },
    { 0x4588c78e, SUC_lexicon_tags_12 },
    { 0xb6a8c78f, SUC_lexicon_tags_17 },
    { 0x5f08c78f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x39e8c792, SUC_lexicon_tags_1 },
    { 0xf2c8c793, SUC_lexicon_tags_17 },
    { 0x2668c794, SUC_lexicon_tags_19 },
    { 0x7488c795, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f08c79b, SUC_lexicon_tags_60 },
    { 0x3c08c79b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57e8c79e, SUC_lexicon_tags_17 },
    { 0x8a88c79f, SUC_lexicon_tags_32 },
    { 0x3428c7a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e68c7a2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc4c8c7a4, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb48c7aa, SUC_lexicon_tags_83 },
    { 0xd8e8c7ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc8c7ae, SUC_lexicon_tags_34 },
    { 0x26c8c7af, SUC_lexicon_tags_6 },
    { 0x5488c7b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c28c7b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c8c7bb, SUC_lexicon_tags_2 },
    { 0x8828c7bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e8c7c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a48c7ca, SUC_lexicon_tags_11 },
    { 0xdd68c7ca, SUC_lexicon_tags_1 },
    { 0xcf28c7cc, SUC_lexicon_tags_207 },
    { 0x74c8c7cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9128c7d1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1508c7d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa968c7d8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa168c7db, SUC_lexicon_tags_8 },
    { 0xf368c7dc, SUC_lexicon_tags_8 },
    { 0x3488c7dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc188c7e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c68c7e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9008c7e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5268c7e8, SUC_lexicon_tags_155 },
    { 0x81e8c7e9, SUC_lexicon_tags_17 },
    { 0x8ec8c7ea, SUC_lexicon_tags_17 },
    { 0xa6e8c7e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9328c7ed, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e8c7f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f08c7f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc8c7fa, SUC_lexicon_tags_15 },
    { 0xd08c7fb, SUC_lexicon_tags_1 },
    { 0x8148c7fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa868c7fe, SUC_lexicon_tags_45 },
    { 0xed08c7ff, SUC_lexicon_tags_9 },
    { 0x1008c7ff, SUC_lexicon_tags_46 },
    { 0x6308c800, SUC_lexicon_tags_32 },
    { 0xa2e8c800, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9c8c804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2388c80c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee8c80f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a8c812, SUC_lexicon_tags_19 },
    { 0x8688c812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb588c818, SUC_lexicon_tags_17 },
    { 0x40c8c818, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7348c81b, SUC_lexicon_tags_12 },
    { 0x3ae8c81c, SUC_lexicon_tags_46 },
    { 0x5fc8c81d, SUC_lexicon_tags_31 },
    { 0xcd88c81e, SUC_lexicon_tags_46 },
    { 0xfde8c81c, SUC_lexicon_tags_12 },
    { 0xa548c81f, SUC_lexicon_tags_37 },
    { 0x3f68c81e, SUC_lexicon_tags_60 },
    { 0x4b28c822, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9988c82a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b88c82d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa568c830, SUC_lexicon_tags_28 },
    { 0xe1a8c830, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79e8c833, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c68c835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c8c838, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d88c83a, SUC_lexicon_tags_33 },
    { 0xaf28c83b, SUC_lexicon_tags_10 },
    { 0xf8a8c83c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54e8c83e, SUC_lexicon_tags_37 },
    { 0xf4a8c83e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf848c845, SUC_lexicon_tags_25 },
    { 0xdd08c846, SUC_lexicon_tags_37 },
    { 0x3088c846, SUC_lexicon_tags_15 },
    { 0xea48c848, SUC_lexicon_tags_37 },
    { 0x7c48c846, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc948c84d, SUC_lexicon_tags_66 },
    { 0xe8c8c84e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa8c851, SUC_lexicon_tags_16 },
    { 0xc08c852, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc8c857, SUC_lexicon_tags_55 },
    { 0x53e8c858, SUC_lexicon_tags_25 },
    { 0xdb68c858, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x288c85b, SUC_lexicon_tags_43 },
    { 0x3088c85b, SUC_lexicon_tags_12 },
    { 0xb0e8c85c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d48c861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc8c868, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8c86f, SUC_lexicon_tags_35 },
    { 0xac08c870, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa368c877, SUC_lexicon_tags_23 },
    { 0x56c8c878, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29a8c87a, SUC_lexicon_tags_45 },
    { 0x2628c87b, SUC_lexicon_tags_57 },
    { 0x5428c87c, SUC_lexicon_tags_12 },
    { 0xe448c87d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b28c880, SUC_lexicon_tags_15 },
    { 0xec28c881, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c8c887, SUC_lexicon_tags_2 },
    { 0xfce8c888, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb4e8c88a, SUC_lexicon_tags_25 },
    { 0x9628c88a, SUC_lexicon_tags_66 },
    { 0xdd68c88c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc8c88f, SUC_lexicon_tags_25 },
    { 0x628c88f, SUC_lexicon_tags_60 },
    { 0x3f08c891, SUC_lexicon_tags_87 },
    { 0xfaa8c890, SUC_lexicon_tags_49 },
    { 0x7048c893, SUC_lexicon_tags_25 },
    { 0x8488c88f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb28c89a, SUC_lexicon_tags_2 },
    { 0xd8e8c89b, SUC_lexicon_tags_7 },
    { 0x3908c89b, SUC_lexicon_tags_19 },
    { 0x39a8c89b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a8c8a0, SUC_lexicon_tags_25 },
    { 0x8128c8a0, SUC_lexicon_tags_1 },
    { 0xb6a8c8a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be8c8a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a8c8ad, SUC_lexicon_tags_83 },
    { 0xa288c8ad, SUC_lexicon_tags_34 },
    { 0x8528c8af, SUC_lexicon_tags_27 },
    { 0x34e8c8b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de8c8b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f48c8bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46c8c8be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2bc8c8c0, SUC_lexicon_tags_19 },
    { 0x6768c8c1, SUC_lexicon_tags_10 },
    { 0x388c8c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xae88c8c5, SUC_lexicon_tags_63 },
    { 0xa0c8c8c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd988c8c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7508c8ca, SUC_lexicon_tags_60 },
    { 0xb168c8cb, SUC_lexicon_tags_2 },
    { 0xc0e8c8ca, SUC_lexicon_tags_27 },
    { 0x5188c8cd, SUC_lexicon_tags_7 },
    { 0xb468c8ce, SUC_lexicon_tags_43 },
    { 0xec28c8cc, SUC_lexicon_tags_2 },
    { 0x3d88c8d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c8c8d5, SUC_lexicon_tags_12 },
    { 0xe168c8d6, SUC_lexicon_tags_1 },
    { 0xcc28c8d5, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x53e8c8d9, SUC_lexicon_tags_11 },
    { 0x45c8c8da, SUC_lexicon_tags_56 },
    { 0x1088c8d9, SUC_lexicon_tags_25 },
    { 0xa588c8db, SUC_lexicon_tags_46 },
    { 0x3aa8c8dd, SUC_lexicon_tags_12 },
    { 0xd9e8c8de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a8c8e4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6268c8e6, SUC_lexicon_tags_1 },
    { 0xb648c8e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f48c8ef, SUC_lexicon_tags_25 },
    { 0xf808c8f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa128c8f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6728c8fa, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc848c8fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd148c903, SUC_lexicon_tags_1 },
    { 0x9e08c903, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96a8c906, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4808c90e, SUC_lexicon_tags_25 },
    { 0xfc08c90f, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x6588c912, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5088c915, SUC_lexicon_tags_37 },
    { 0x5ba8c916, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce8c91c, SUC_lexicon_tags_14 },
    { 0x93c8c91c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1e8c91f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3e8c921, SUC_lexicon_tags_11 },
    { 0xb3c8c921, SUC_lexicon_tags_1 },
    { 0xcda8c923, SUC_lexicon_tags_37 },
    { 0x3448c923, SUC_lexicon_tags_11 },
    { 0x59e8c925, SUC_lexicon_tags_37 },
    { 0x8548c921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a8c92a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e8c935, SUC_lexicon_tags_23 },
    { 0xc0a8c936, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac8c93a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4688c942, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf188c945, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93e8c947, SUC_lexicon_tags_25 },
    { 0xc268c947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4708c94a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c48c94c, SUC_lexicon_tags_6 },
    { 0x3828c94c, SUC_lexicon_tags_11 },
    { 0x97c8c94e, SUC_lexicon_tags_25 },
    { 0x7f28c94f, SUC_lexicon_tags_37 },
    { 0xd208c94d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb928c955, SUC_lexicon_tags_32 },
    { 0xaf88c955, SUC_lexicon_tags_29 },
    { 0xcc68c955, SUC_lexicon_tags_7 },
    { 0xf008c958, SUC_lexicon_tags_31 },
    { 0xa888c957, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xa508c95b, SUC_lexicon_tags_17 },
    { 0x9a28c95b, SUC_lexicon_tags_37 },
    { 0xde8c95c, SUC_lexicon_tags_2 },
    { 0xaa8c95e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x84c8c960, SUC_lexicon_tags_0 },
    { 0x8f48c961, SUC_lexicon_tags_9 },
    { 0x61e8c962, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a68c965, SUC_lexicon_tags_1 },
    { 0x21a8c966, SUC_lexicon_tags_37 },
    { 0xe0a8c967, SUC_lexicon_tags_37 },
    { 0x7008c967, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5608c96a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe348c96c, SUC_lexicon_tags_29 },
    { 0xccc8c96d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e88c96f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde08c971, SUC_lexicon_tags_27 },
    { 0x1d48c972, SUC_lexicon_tags_66 },
    { 0x468c971, SUC_lexicon_tags_171 },
    { 0x46c8c974, SUC_lexicon_tags_37 },
    { 0xab28c975, SUC_lexicon_tags_1 },
    { 0x7528c976, SUC_lexicon_tags_14 },
    { 0x7ec8c975, SUC_lexicon_tags_38 },
    { 0x75e8c978, SUC_lexicon_tags_11 },
    { 0x8fa8c975, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb248c97b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88c980, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e88c983, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a8c986, SUC_lexicon_tags_7 },
    { 0x4a48c986, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa68c98b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f48c990, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ca8c992, SUC_lexicon_tags_87 },
    { 0xd728c993, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xd528c996, SUC_lexicon_tags_30 },
    { 0x1e68c996, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e8c99b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a8c99e, SUC_lexicon_tags_34 },
    { 0xc9a8c99e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e8c9a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c28c9a7, SUC_lexicon_tags_30 },
    { 0x8ba8c9a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e8c9ad, SUC_lexicon_tags_23 },
    { 0xd648c9ae, SUC_lexicon_tags_127 },
    { 0xcb28c9ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28c9b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xee28c9bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x92a8c9be, SUC_lexicon_tags_45 },
    { 0x7108c9be, SUC_lexicon_tags_17 },
    { 0xca8c9c0, SUC_lexicon_tags_25 },
    { 0x56a8c9be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f48c9c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8c9cb, SUC_lexicon_tags_37 },
    { 0x4868c9cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b88c9cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8828c9d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf448c9d5, SUC_lexicon_tags_31 },
    { 0xae88c9d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b68c9d8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec08c9e6, SUC_lexicon_tags_15 },
    { 0x47c8c9e6, SUC_lexicon_tags_63 },
    { 0xe488c9e8, SUC_lexicon_tags_27 },
    { 0xc88c9e8, SUC_lexicon_tags_1 },
    { 0x8108c9ea, SUC_lexicon_tags_37 },
    { 0x98c8c9eb, SUC_lexicon_tags_33 },
    { 0x4988c9ea, SUC_lexicon_tags_19 },
    { 0xaf88c9ec, SUC_lexicon_tags_77 },
    { 0x1a8c9ee, SUC_lexicon_tags_10 },
    { 0xaee8c9ef, SUC_lexicon_tags_15 },
    { 0x1468c9ef, SUC_lexicon_tags_17 },
    { 0xf708c9f1, SUC_lexicon_tags_23 },
    { 0xb688c9f2, SUC_lexicon_tags_30 },
    { 0x4a28c9f1, SUC_lexicon_tags_40 },
    { 0xb808c9f0, SUC_lexicon_tags_17 },
    { 0x91e8c9ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea8c9f9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc8c9fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac8ca00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6808ca06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3068ca0a, SUC_lexicon_tags_38 },
    { 0x4948ca0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e8ca10, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa868ca13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90a8ca15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba8ca18, SUC_lexicon_tags_11 },
    { 0x768ca19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc468ca1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe48ca1d, SUC_lexicon_tags_1 },
    { 0x2fa8ca1d, SUC_lexicon_tags_14 },
    { 0xd848ca1f, SUC_lexicon_tags_17 },
    { 0x3368ca20, SUC_lexicon_tags_2 },
    { 0xcd28ca20, SUC_lexicon_tags_1 },
    { 0xcf68ca21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7148ca29, SUC_lexicon_tags_0 },
    { 0x20e8ca29, SUC_lexicon_tags_38 },
    { 0xe328ca2b, SUC_lexicon_tags_31 },
    { 0xb828ca2c, SUC_lexicon_tags_29 },
    { 0x5948ca2c, SUC_lexicon_tags_44 },
    { 0x2648ca2b, SUC_lexicon_tags_36 },
    { 0x96a8ca2f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3808ca35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb368ca3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e8ca42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac8ca45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c8ca47, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x9848ca4a, SUC_lexicon_tags_12 },
    { 0xf288ca4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c68ca4e, SUC_lexicon_tags_14 },
    { 0xfe48ca4f, SUC_lexicon_tags_11 },
    { 0x7f68ca50, SUC_lexicon_tags_15 },
    { 0xb668ca4f, SUC_lexicon_tags_7 },
    { 0xdee8ca51, SUC_lexicon_tags_6 },
    { 0x21c8ca53, SUC_lexicon_tags_15 },
    { 0xd408ca54, SUC_lexicon_tags_9 },
    { 0xbfe8ca55, SUC_lexicon_tags_65 },
    { 0x4148ca54, SUC_lexicon_tags_7 },
    { 0x27a8ca55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac8ca5c, SUC_lexicon_tags_28 },
    { 0x7168ca5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c8ca60, SUC_lexicon_tags_12 },
    { 0xfe48ca61, SUC_lexicon_tags_106 },
    { 0x2cc8ca61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1868ca65, SUC_lexicon_tags_38 },
    { 0x7c68ca66, SUC_lexicon_tags_47 },
    { 0x5828ca67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7048ca6c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea8ca6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf168ca73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8088ca75, SUC_lexicon_tags_49 },
    { 0xd368ca75, SUC_lexicon_tags_6 },
    { 0x4b08ca75, SUC_lexicon_tags_1 },
    { 0xad48ca78, SUC_lexicon_tags_17 },
    { 0x86e8ca78, SUC_lexicon_tags_48 },
    { 0x3568ca7a, SUC_lexicon_tags_12 },
    { 0x4e88ca75, SUC_lexicon_tags_85 },
    { 0x5348ca7c, SUC_lexicon_tags_19 },
    { 0xdf48ca7d, SUC_lexicon_tags_6 },
    { 0x6168ca7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa8ca85, SUC_lexicon_tags_45 },
    { 0x5668ca86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb88ca88, SUC_lexicon_tags_12 },
    { 0x9008ca89, SUC_lexicon_tags_140 },
    { 0x3688ca8a, SUC_lexicon_tags_65 },
    { 0x7308ca8b, SUC_lexicon_tags_27 },
    { 0x88e8ca8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce8ca91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb88ca96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdc88ca98, SUC_lexicon_tags_66 },
    { 0xbe68ca98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67c8ca9b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f68ca9e, SUC_lexicon_tags_17 },
    { 0x70a8ca9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa2e8caa1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x248caa4, SUC_lexicon_tags_77 },
    { 0xe08caa5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e8caa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb928caae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae08cabd, SUC_lexicon_tags_2 },
    { 0x40c8cabe, SUC_lexicon_tags_37 },
    { 0x15e8cabe, SUC_lexicon_tags_38 },
    { 0xb968cac0, SUC_lexicon_tags_57 },
    { 0x8f28cac0, SUC_lexicon_tags_1 },
    { 0xc808cac0, SUC_lexicon_tags_32 },
    { 0x99a8cabf, SUC_lexicon_tags_17 },
    { 0xd988cac4, SUC_lexicon_tags_17 },
    { 0x2d68cac4, SUC_lexicon_tags_38 },
    { 0xd688cac3, SUC_lexicon_tags_14 },
    { 0xd7a8cac7, SUC_lexicon_tags_114 },
    { 0x93c8cabf, SUC_lexicon_tags_23 },
    { 0xc388cac9, SUC_lexicon_tags_47 },
    { 0x12c8cac9, SUC_lexicon_tags_0 },
    { 0x1008cacb, SUC_lexicon_tags_37 },
    { 0xab08cacc, SUC_lexicon_tags_25 },
    { 0x3408cacd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08cad5, SUC_lexicon_tags_1 },
    { 0x55c8cad6, SUC_lexicon_tags_37 },
    { 0x7aa8cad6, SUC_lexicon_tags_11 },
    { 0x1aa8cad7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa188cada, SUC_lexicon_tags_27 },
    { 0x2148cada, SUC_lexicon_tags_60 },
    { 0x8208cadc, SUC_lexicon_tags_45 },
    { 0x51e8cadc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e8cae1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6348cae4, SUC_lexicon_tags_12 },
    { 0x36a8cae5, SUC_lexicon_tags_63 },
    { 0x2668cae6, SUC_lexicon_tags_33 },
    { 0xcc08cae7, SUC_lexicon_tags_25 },
    { 0x3ac8cae8, SUC_lexicon_tags_8 },
    { 0x3588cae9, SUC_lexicon_tags_39 },
    { 0xa7a8caea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c8caf3, SUC_lexicon_tags_1 },
    { 0xc368caf4, SUC_lexicon_tags_0 },
    { 0xe248caf5, SUC_lexicon_tags_2 },
    { 0xf268caf3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce8cafb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d88cb00, SUC_lexicon_tags_26 },
    { 0x56c8cb01, SUC_lexicon_tags_14 },
    { 0x70a8cb01, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8cb06, SUC_lexicon_tags_25 },
    { 0x4a8cb07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf368cb09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2908cb0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb508cb12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f08cb17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc488cb1a, SUC_lexicon_tags_15 },
    { 0xe968cb1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e28cb24, SUC_lexicon_tags_37 },
    { 0x14e8cb25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8708cb27, SUC_lexicon_tags_25 },
    { 0xb908cb28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de8cb2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae8cb31, SUC_lexicon_tags_11 },
    { 0x5a48cb31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b88cb34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9048cb36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc8cb3b, SUC_lexicon_tags_77 },
    { 0xe2c8cb3c, SUC_lexicon_tags_1 },
    { 0xcd88cb3d, SUC_lexicon_tags_160 },
    { 0x4de8cb3e, SUC_lexicon_tags_17 },
    { 0xdd68cb3d, SUC_lexicon_tags_45 },
    { 0xbbe8cb3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec48cb42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c8cb45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a68cb47, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec8cb4e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3568cb5b, SUC_lexicon_tags_17 },
    { 0x5648cb5b, SUC_lexicon_tags_1 },
    { 0x7ec8cb5c, SUC_lexicon_tags_17 },
    { 0x6608cb5e, SUC_lexicon_tags_29 },
    { 0x4a8cb5f, SUC_lexicon_tags_11 },
    { 0x5008cb5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c08cb66, SUC_lexicon_tags_21 },
    { 0xbdc8cb66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e8cb6b, SUC_lexicon_tags_30 },
    { 0x96a8cb6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d68cb6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda8cb72, SUC_lexicon_tags_0 },
    { 0x6a68cb73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e8cb76, SUC_lexicon_tags_77 },
    { 0x4b08cb77, SUC_lexicon_tags_17 },
    { 0x79c8cb78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a8cb7b, SUC_lexicon_tags_25 },
    { 0x4e88cb7c, SUC_lexicon_tags_70 },
    { 0x44e8cb7d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7628cb7f, SUC_lexicon_tags_17 },
    { 0x748cb7f, SUC_lexicon_tags_179 },
    { 0x6628cb7f, SUC_lexicon_tags_2 },
    { 0x5c08cb82, SUC_lexicon_tags_15 },
    { 0x1308cb83, SUC_lexicon_tags_33 },
    { 0xb928cb84, SUC_lexicon_tags_8 },
    { 0x8d08cb83, SUC_lexicon_tags_11 },
    { 0x6b28cb86, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c8cb90, SUC_lexicon_tags_11 },
    { 0xa7c8cb90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa88cb95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a48cb98, SUC_lexicon_tags_23 },
    { 0x7928cb99, SUC_lexicon_tags_37 },
    { 0x8208cb9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda8cba3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7628cbad, SUC_lexicon_tags_141 },
    { 0xfd08cbae, SUC_lexicon_tags_2 },
    { 0xaee8cbaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a8cbb2, SUC_lexicon_tags_53 },
    { 0xf668cbb3, SUC_lexicon_tags_8 },
    { 0xdd08cbb4, SUC_lexicon_tags_91 },
    { 0xb868cbb2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de8cbbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5828cbbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e8cbc4, SUC_lexicon_tags_2 },
    { 0xdf48cbc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8cbc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea8cbcc, SUC_lexicon_tags_30 },
    { 0x4e28cbcd, SUC_lexicon_tags_17 },
    { 0x9ac8cbcd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c8cbd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa388cbd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4248cbdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1648cbde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1248cbe0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xede8cbe3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x21a8cbe5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xffe8cbe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3528cbe9, SUC_lexicon_tags_54 },
    { 0xe008cbea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ce8cbec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4148cbee, SUC_lexicon_tags_8 },
    { 0x1dc8cbef, SUC_lexicon_tags_60 },
    { 0xf268cbf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd628cbfc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a8cc00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688cc05, SUC_lexicon_tags_33 },
    { 0x4e8cc05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c8cc09, SUC_lexicon_tags_1 },
    { 0x2288cc0a, SUC_lexicon_tags_29 },
    { 0x1308cc0b, SUC_lexicon_tags_37 },
    { 0x98a8cc0b, SUC_lexicon_tags_40 },
    { 0xc08cc0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7e48cc0f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8568cc15, SUC_lexicon_tags_26 },
    { 0xec88cc16, SUC_lexicon_tags_25 },
    { 0x13e8cc17, SUC_lexicon_tags_49 },
    { 0xb6e8cc16, SUC_lexicon_tags_17 },
    { 0x25e8cc15, SUC_lexicon_tags_33 },
    { 0x4b88cc1a, SUC_lexicon_tags_7 },
    { 0x47e8cc1b, SUC_lexicon_tags_26 },
    { 0xeec8cc16, SUC_lexicon_tags_65 },
    { 0x2848cc1d, SUC_lexicon_tags_28 },
    { 0xf0a8cc19, SUC_lexicon_tags_39 },
    { 0x1108cc15, SUC_lexicon_tags_49 },
    { 0xa928cc20, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x79c8cc22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6248cc27, SUC_lexicon_tags_15 },
    { 0xa588cc27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e08cc2a, SUC_lexicon_tags_30 },
    { 0xe288cc2a, SUC_lexicon_tags_17 },
    { 0x5f28cc2a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe8cc30, SUC_lexicon_tags_1 },
    { 0xda08cc31, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae8cc35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec8cc37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3808cc3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb348cc40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d68cc44, SUC_lexicon_tags_46 },
    { 0x2f08cc45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1488cc4a, SUC_lexicon_tags_17 },
    { 0x3728cc4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe868cc4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9628cc53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x908cc55, SUC_lexicon_tags_25 },
    { 0x8708cc56, SUC_lexicon_tags_27 },
    { 0x12a8cc56, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e8cc5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3748cc61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9a8cc63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2648cc68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76c8cc6a, SUC_lexicon_tags_12 },
    { 0xe6e8cc6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a8cc6d, SUC_lexicon_tags_15 },
    { 0xada8cc6e, SUC_lexicon_tags_17 },
    { 0x4d48cc6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4088cc76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c28cc7a, SUC_lexicon_tags_7 },
    { 0x4128cc7a, SUC_lexicon_tags_8 },
    { 0x6708cc7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc788cc80, SUC_lexicon_tags_12 },
    { 0xa328cc80, SUC_lexicon_tags_7 },
    { 0x5148cc82, SUC_lexicon_tags_32 },
    { 0x85c8cc83, SUC_lexicon_tags_14 },
    { 0x8a08cc84, SUC_lexicon_tags_2 },
    { 0xdae8cc81, SUC_lexicon_tags_11 },
    { 0x2dc8cc86, SUC_lexicon_tags_46 },
    { 0x7048cc86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c8cc8b, SUC_lexicon_tags_60 },
    { 0x7448cc8b, SUC_lexicon_tags_37 },
    { 0xa28cc8d, SUC_lexicon_tags_15 },
    { 0x8e08cc8e, SUC_lexicon_tags_23 },
    { 0xb68cc8f, SUC_lexicon_tags_27 },
    { 0x4ac8cc90, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948cc94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e8cc9a, SUC_lexicon_tags_26 },
    { 0x4368cc9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b28cca0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf028cca4, SUC_lexicon_tags_43 },
    { 0x8688cca5, SUC_lexicon_tags_48 },
    { 0xb888cca5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a8ccab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3168ccad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8ccb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8648ccb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c8ccb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e8ccc1, SUC_lexicon_tags_12 },
    { 0x1ee8ccc2, SUC_lexicon_tags_15 },
    { 0x6fe8ccc3, SUC_lexicon_tags_978 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7948ccc7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa168ccca, SUC_lexicon_tags_9 },
    { 0xd968cccb, SUC_lexicon_tags_60 },
    { 0x1b88cccb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2588ccd0, SUC_lexicon_tags_11 },
    { 0x6e88ccd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1c8ccd3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8628ccdb, SUC_lexicon_tags_37 },
    { 0x4a08ccdb, SUC_lexicon_tags_4 },
    { 0xa768ccdd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cce4, SUC_lexicon_tags_17 },
    { 0xf2e8cce5, SUC_lexicon_tags_0 },
    { 0x5e8cce5, SUC_lexicon_tags_10 },
    { 0xa728cce4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35e8cce9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd68ccf0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2a8ccf2, SUC_lexicon_tags_25 },
    { 0xada8ccf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b08ccfa, SUC_lexicon_tags_1 },
    { 0x25e8ccfa, SUC_lexicon_tags_11 },
    { 0xdc68ccfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11a8ccfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd008cd00, SUC_lexicon_tags_11 },
    { 0x3348cd00, SUC_lexicon_tags_170 },
    { 0x7fc8cd02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa88cd04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc68cd0c, SUC_lexicon_tags_71 },
    { 0x1ae8cd0d, SUC_lexicon_tags_63 },
    { 0xdbc8cd0d, SUC_lexicon_tags_65 },
    { 0xd4a8cd0f, SUC_lexicon_tags_193 },
    { 0x8268cd0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3748cd14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3548cd18, SUC_lexicon_tags_37 },
    { 0xeaa8cd19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4148cd1e, SUC_lexicon_tags_106 },
    { 0x2968cd1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe48cd25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5488cd2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6428cd2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe688cd37, SUC_lexicon_tags_60 },
    { 0x5d68cd38, SUC_lexicon_tags_48 },
    { 0x9488cd39, SUC_lexicon_tags_15 },
    { 0x1da8cd3a, SUC_lexicon_tags_30 },
    { 0xa288cd3b, SUC_lexicon_tags_37 },
    { 0x5968cd3c, SUC_lexicon_tags_37 },
    { 0x3f08cd3d, SUC_lexicon_tags_2 },
    { 0xb568cd37, SUC_lexicon_tags_6 },
    { 0x7288cd39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb68cd42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12a8cd44, SUC_lexicon_tags_32 },
    { 0xe68cd45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2068cd49, SUC_lexicon_tags_11 },
    { 0x3b88cd49, SUC_lexicon_tags_45 },
    { 0x1848cd4b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad08cd4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd088cd53, SUC_lexicon_tags_2 },
    { 0xf9e8cd53, SUC_lexicon_tags_8 },
    { 0xeaa8cd55, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xbca8cd57, SUC_lexicon_tags_11 },
    { 0x7a28cd58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9e8cd5a, SUC_lexicon_tags_9 },
    { 0x8048cd5b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8cd61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e88cd72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xea88cd74, SUC_lexicon_tags_21 },
    { 0x3a28cd75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a8cd78, SUC_lexicon_tags_11 },
    { 0xea28cd78, SUC_lexicon_tags_7 },
    { 0xa528cd78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb988cd7e, SUC_lexicon_tags_12 },
    { 0xf848cd7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd708cd81, SUC_lexicon_tags_65 },
    { 0x2268cd82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8cd85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e8cd88, SUC_lexicon_tags_31 },
    { 0x30c8cd89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1508cd8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xece8cd8f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf448cd91, SUC_lexicon_tags_23 },
    { 0xb188cd91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd28cd94, SUC_lexicon_tags_631 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8cd9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa8cda6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7568cda9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd28cdab, SUC_lexicon_tags_30 },
    { 0xcea8cdac, SUC_lexicon_tags_55 },
    { 0xdee8cdad, SUC_lexicon_tags_2 },
    { 0x6388cdad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d08cdb6, SUC_lexicon_tags_39 },
    { 0xfd08cdb7, SUC_lexicon_tags_9 },
    { 0xe7c8cdb6, SUC_lexicon_tags_23 },
    { 0x3a8cdb9, SUC_lexicon_tags_27 },
    { 0xdda8cdb8, SUC_lexicon_tags_47 },
    { 0x7bc8cdbb, SUC_lexicon_tags_1 },
    { 0xe748cdbc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5ac8cdbe, SUC_lexicon_tags_17 },
    { 0x8d88cdbf, SUC_lexicon_tags_37 },
    { 0x6728cdbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b48cdc3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c8cdc6, SUC_lexicon_tags_1 },
    { 0x3488cdc6, SUC_lexicon_tags_25 },
    { 0x6908cdc8, SUC_lexicon_tags_6 },
    { 0x5f48cdc9, SUC_lexicon_tags_14 },
    { 0xae08cdca, SUC_lexicon_tags_12 },
    { 0x8588cdc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f88cdce, SUC_lexicon_tags_29 },
    { 0x8b28cdcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128cdd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee8cdda, SUC_lexicon_tags_15 },
    { 0x9448cdda, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe908cddd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a48cddf, SUC_lexicon_tags_37 },
    { 0xf488cde0, SUC_lexicon_tags_207 },
    { 0xff48cde0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8cde4, SUC_lexicon_tags_0 },
    { 0x5e28cde5, SUC_lexicon_tags_15 },
    { 0x3808cde6, SUC_lexicon_tags_17 },
    { 0xefa8cde7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdce8cde9, SUC_lexicon_tags_27 },
    { 0x2f08cde9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1768cdec, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be8cdf0, SUC_lexicon_tags_49 },
    { 0x9588cdf1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e28cdf5, SUC_lexicon_tags_37 },
    { 0xd3c8cdf5, SUC_lexicon_tags_14 },
    { 0x5e88cdf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2148cdf9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd408cdfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae88ce00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb048ce06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2548ce0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc8ce0f, SUC_lexicon_tags_15 },
    { 0xaa88ce10, SUC_lexicon_tags_12 },
    { 0xb3c8ce11, SUC_lexicon_tags_25 },
    { 0x2408ce12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a8ce1c, SUC_lexicon_tags_27 },
    { 0xa0c8ce1d, SUC_lexicon_tags_23 },
    { 0xf9a8ce1e, SUC_lexicon_tags_77 },
    { 0x5f48ce1e, SUC_lexicon_tags_27 },
    { 0x3d48ce20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e8ce26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e8ce2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6008ce2d, SUC_lexicon_tags_81 },
    { 0x9fa8ce2e, SUC_lexicon_tags_27 },
    { 0xb108ce2f, SUC_lexicon_tags_17 },
    { 0xc228ce2f, SUC_lexicon_tags_12 },
    { 0x14e8ce31, SUC_lexicon_tags_15 },
    { 0x9d88ce32, SUC_lexicon_tags_25 },
    { 0x9b08ce33, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xcd28ce35, SUC_lexicon_tags_37 },
    { 0x5628ce36, SUC_lexicon_tags_5 },
    { 0xeea8ce37, SUC_lexicon_tags_37 },
    { 0xd68ce38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5788ce3a, SUC_lexicon_tags_25 },
    { 0x5e08ce3a, SUC_lexicon_tags_7 },
    { 0x6108ce3b, SUC_lexicon_tags_84 },
    { 0x9ec8ce3c, SUC_lexicon_tags_1 },
    { 0xb388ce3d, SUC_lexicon_tags_11 },
    { 0x8fc8ce3f, SUC_lexicon_tags_2 },
    { 0xbe88ce40, SUC_lexicon_tags_17 },
    { 0x1788ce41, SUC_lexicon_tags_9 },
    { 0xcd88ce3f, SUC_lexicon_tags_15 },
    { 0xad48ce43, SUC_lexicon_tags_25 },
    { 0x6608ce43, SUC_lexicon_tags_26 },
    { 0x3708ce44, SUC_lexicon_tags_7 },
    { 0xe328ce46, SUC_lexicon_tags_70 },
    { 0x71a8ce45, SUC_lexicon_tags_19 },
    { 0x65a8ce47, SUC_lexicon_tags_1 },
    { 0x9488ce49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a48ce4b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9368ce50, SUC_lexicon_tags_11 },
    { 0x8768ce51, SUC_lexicon_tags_57 },
    { 0x5128ce52, SUC_lexicon_tags_8 },
    { 0xd988ce50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1148ce5a, SUC_lexicon_tags_27 },
    { 0x5348ce5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25e8ce5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1288ce65, SUC_lexicon_tags_30 },
    { 0x7ce8ce65, SUC_lexicon_tags_7 },
    { 0x40a8ce67, SUC_lexicon_tags_33 },
    { 0x8488ce65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc408ce6c, SUC_lexicon_tags_15 },
    { 0xcb68ce6c, SUC_lexicon_tags_12 },
    { 0xef68ce6e, SUC_lexicon_tags_37 },
    { 0xd6a8ce6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc8ce74, SUC_lexicon_tags_17 },
    { 0x63a8ce74, SUC_lexicon_tags_33 },
    { 0xd5e8ce75, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe828ce7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x948ce7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fa8ce80, SUC_lexicon_tags_32 },
    { 0x1288ce80, SUC_lexicon_tags_5 },
    { 0x5448ce81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c88ce87, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe68ce8b, SUC_lexicon_tags_6 },
    { 0xbae8ce8c, SUC_lexicon_tags_7 },
    { 0x5c28ce8d, SUC_lexicon_tags_1 },
    { 0x568ce8c, SUC_lexicon_tags_63 },
    { 0x3e68ce8e, SUC_lexicon_tags_2 },
    { 0x9008ce90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3688ce92, SUC_lexicon_tags_65 },
    { 0xf168ce93, SUC_lexicon_tags_25 },
    { 0x4ea8ce94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c8ce9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a8ce9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c8cea2, SUC_lexicon_tags_13 },
    { 0xf3e8cea3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e8cea7, SUC_lexicon_tags_37 },
    { 0x6608cea8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1a8ceaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe028cead, SUC_lexicon_tags_19 },
    { 0x9ba8ceae, SUC_lexicon_tags_10 },
    { 0xa8e8ceaf, SUC_lexicon_tags_17 },
    { 0xb608ceae, SUC_lexicon_tags_25 },
    { 0x11e8ceb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2328ceb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc188ceb6, SUC_lexicon_tags_45 },
    { 0x6648ceb6, SUC_lexicon_tags_14 },
    { 0xc8c8ceb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb88cebe, SUC_lexicon_tags_36 },
    { 0x1448cebf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb88cec1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1a8cec3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9088cece, SUC_lexicon_tags_19 },
    { 0xdb08cecf, SUC_lexicon_tags_14 },
    { 0xec68ced0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e8ced7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc8cedc, SUC_lexicon_tags_23 },
    { 0x1f48cedc, SUC_lexicon_tags_7 },
    { 0xaf68cede, SUC_lexicon_tags_66 },
    { 0xa068cedf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c8cee5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb508cee7, SUC_lexicon_tags_11 },
    { 0x8388cee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbda8ceea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc1e8ceec, SUC_lexicon_tags_2 },
    { 0x66a8ceed, SUC_lexicon_tags_60 },
    { 0x1c8ceec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8668cef2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc08cef6, SUC_lexicon_tags_56 },
    { 0xbf88cef7, SUC_lexicon_tags_3 },
    { 0xcc48cef8, SUC_lexicon_tags_12 },
    { 0xd168cef8, SUC_lexicon_tags_49 },
    { 0xfd08cefa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf448cefe, SUC_lexicon_tags_17 },
    { 0x2488cefe, SUC_lexicon_tags_37 },
    { 0xb5a8cefe, SUC_lexicon_tags_37 },
    { 0x72a8cf01, SUC_lexicon_tags_17 },
    { 0xa0c8cf01, SUC_lexicon_tags_11 },
    { 0x47c8cf03, SUC_lexicon_tags_17 },
    { 0x8ec8cf00, SUC_lexicon_tags_10 },
    { 0x60c8cf00, SUC_lexicon_tags_60 },
    { 0x2ac8cf06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97c8cf08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5fe8cf0a, SUC_lexicon_tags_31 },
    { 0xda88cf0a, SUC_lexicon_tags_27 },
    { 0x2b88cf0a, SUC_lexicon_tags_37 },
    { 0x5768cf0b, SUC_lexicon_tags_60 },
    { 0x8768cf0e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x5788cf10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5288cf15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bc8cf17, SUC_lexicon_tags_17 },
    { 0x7ca8cf18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c88cf1b, SUC_lexicon_tags_28 },
    { 0xb988cf1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b28cf1f, SUC_lexicon_tags_25 },
    { 0x3388cf20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8de8cf22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x508cf24, SUC_lexicon_tags_25 },
    { 0xb4a8cf25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc68cf2c, SUC_lexicon_tags_7 },
    { 0x68a8cf2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf88cf30, SUC_lexicon_tags_14 },
    { 0x8c8cf31, SUC_lexicon_tags_4 },
    { 0x5568cf32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfa8cf34, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5948cf38, SUC_lexicon_tags_11 },
    { 0x9708cf38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a08cf3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9468cf3e, SUC_lexicon_tags_15 },
    { 0xf1a8cf3f, SUC_lexicon_tags_60 },
    { 0x43a8cf40, SUC_lexicon_tags_16 },
    { 0x9a68cf3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb348cf44, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec8cf48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e8cf4e, SUC_lexicon_tags_17 },
    { 0xd5a8cf4e, SUC_lexicon_tags_37 },
    { 0x4048cf4e, SUC_lexicon_tags_10 },
    { 0x468cf51, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a68cf56, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e08cf5a, SUC_lexicon_tags_68 },
    { 0xfe88cf5b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28cf5e, SUC_lexicon_tags_120 },
    { 0xf3a8cf5e, SUC_lexicon_tags_8 },
    { 0x9a48cf5e, SUC_lexicon_tags_8 },
    { 0xb048cf61, SUC_lexicon_tags_55 },
    { 0x7d88cf61, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x708cf64, SUC_lexicon_tags_25 },
    { 0x5868cf64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc8cf6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4828cf6d, SUC_lexicon_tags_55 },
    { 0x76a8cf6e, SUC_lexicon_tags_37 },
    { 0xca8cf6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3748cf73, SUC_lexicon_tags_2 },
    { 0x8188cf74, SUC_lexicon_tags_43 },
    { 0x61a8cf75, SUC_lexicon_tags_43 },
    { 0xa128cf76, SUC_lexicon_tags_72 },
    { 0xb688cf77, SUC_lexicon_tags_11 },
    { 0xae48cf77, SUC_lexicon_tags_17 },
    { 0xe0a8cf77, SUC_lexicon_tags_5 },
    { 0x97c8cf78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c8cf80, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3e88cf82, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b88cf86, SUC_lexicon_tags_65 },
    { 0xc368cf87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2848cf92, SUC_lexicon_tags_17 },
    { 0x6728cf92, SUC_lexicon_tags_14 },
    { 0xe808cf94, SUC_lexicon_tags_47 },
    { 0x9508cf95, SUC_lexicon_tags_64 },
    { 0xbc28cf96, SUC_lexicon_tags_2 },
    { 0xeb08cf95, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xad88cf99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508cf9e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x68a8cfa0, SUC_lexicon_tags_2 },
    { 0xde88cfa1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x96e8cfa4, SUC_lexicon_tags_27 },
    { 0xf808cfa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa788cfaa, SUC_lexicon_tags_7 },
    { 0xba8cfaa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac8cfb2, SUC_lexicon_tags_70 },
    { 0xbb28cfb2, SUC_lexicon_tags_27 },
    { 0x1088cfb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b28cfbd, SUC_lexicon_tags_33 },
    { 0x94e8cfbe, SUC_lexicon_tags_1 },
    { 0xed28cfbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5bc8cfc1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2188cfc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe088cfcf, SUC_lexicon_tags_11 },
    { 0xf0e8cfd0, SUC_lexicon_tags_7 },
    { 0xc308cfd1, SUC_lexicon_tags_23 },
    { 0xd2a8cfcf, SUC_lexicon_tags_47 },
    { 0x3008cfd3, SUC_lexicon_tags_25 },
    { 0x1b48cfd3, SUC_lexicon_tags_49 },
    { 0x1128cfd4, SUC_lexicon_tags_14 },
    { 0x6328cfd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf708cfda, SUC_lexicon_tags_12 },
    { 0x9048cfda, SUC_lexicon_tags_7 },
    { 0x3528cfdb, SUC_lexicon_tags_45 },
    { 0x9fa8cfdd, SUC_lexicon_tags_14 },
    { 0x3dc8cfdd, SUC_lexicon_tags_2 },
    { 0xfb68cfdf, SUC_lexicon_tags_37 },
    { 0x73c8cfdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24e8cfe2, SUC_lexicon_tags_12 },
    { 0x2c48cfe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1528cfe5, SUC_lexicon_tags_19 },
    { 0x9968cfe5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb188cfe9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb168cfef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a8cff2, SUC_lexicon_tags_32 },
    { 0x1928cff3, SUC_lexicon_tags_7 },
    { 0x1fe8cff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c8cff7, SUC_lexicon_tags_45 },
    { 0x44a8cff8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4968cffc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a8d005, SUC_lexicon_tags_53 },
    { 0x7ae8d006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed08d00f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9048d011, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48d01c, SUC_lexicon_tags_45 },
    { 0x9228d01d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788d021, SUC_lexicon_tags_21 },
    { 0x23e8d022, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb208d025, SUC_lexicon_tags_12 },
    { 0x3f28d026, SUC_lexicon_tags_17 },
    { 0x2fa8d027, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5428d029, SUC_lexicon_tags_6 },
    { 0x13a8d02a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd28d031, SUC_lexicon_tags_79 },
    { 0x7128d032, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3de8d038, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd68d03a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea8d03d, SUC_lexicon_tags_17 },
    { 0x1c8d03e, SUC_lexicon_tags_2 },
    { 0xa708d03f, SUC_lexicon_tags_8 },
    { 0xe2a8d040, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea68d042, SUC_lexicon_tags_19 },
    { 0xc6e8d042, SUC_lexicon_tags_37 },
    { 0x7428d044, SUC_lexicon_tags_140 },
    { 0x93e8d044, SUC_lexicon_tags_25 },
    { 0xe8d046, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc488d049, SUC_lexicon_tags_4 },
    { 0x9bc8d04a, SUC_lexicon_tags_1 },
    { 0x9f48d04b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2dc8d04d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6e8d04f, SUC_lexicon_tags_84 },
    { 0x8cc8d050, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca8d057, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1428d059, SUC_lexicon_tags_17 },
    { 0xdf28d059, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf468d05c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc8d060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1588d062, SUC_lexicon_tags_37 },
    { 0x2d28d063, SUC_lexicon_tags_33 },
    { 0x648d064, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae8d06e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe88d071, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e8d074, SUC_lexicon_tags_1 },
    { 0x3ca8d075, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7048d077, SUC_lexicon_tags_17 },
    { 0x84c8d078, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd08d07b, SUC_lexicon_tags_37 },
    { 0x2de8d07b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc8d080, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba8d084, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9108d088, SUC_lexicon_tags_23 },
    { 0x2388d088, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a48d08b, SUC_lexicon_tags_43 },
    { 0x11c8d08b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a28d092, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc8d096, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc8d099, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4748d09f, SUC_lexicon_tags_19 },
    { 0x91a8d09f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb3c8d0a2, SUC_lexicon_tags_37 },
    { 0xa2a8d0a2, SUC_lexicon_tags_27 },
    { 0xd7a8d0a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f48d0a7, SUC_lexicon_tags_17 },
    { 0xc5e8d0a8, SUC_lexicon_tags_68 },
    { 0x59e8d0a7, SUC_lexicon_tags_34 },
    { 0x6628d0aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf88d0ae, SUC_lexicon_tags_29 },
    { 0x868d0ae, SUC_lexicon_tags_37 },
    { 0x6608d0b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88d0b8, SUC_lexicon_tags_25 },
    { 0x6148d0b9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc8d0be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3a8d0c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe48d0c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9a88d0c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a8d0c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6888d0c9, SUC_lexicon_tags_28 },
    { 0xfda8d0ca, SUC_lexicon_tags_77 },
    { 0x5d68d0cb, SUC_lexicon_tags_12 },
    { 0x6b08d0cc, SUC_lexicon_tags_7 },
    { 0x5108d0cb, SUC_lexicon_tags_7 },
    { 0xd3c8d0cd, SUC_lexicon_tags_13 },
    { 0x81c8d0ce, SUC_lexicon_tags_12 },
    { 0x8088d0cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27c8d0d2, SUC_lexicon_tags_36 },
    { 0xece8d0d3, SUC_lexicon_tags_37 },
    { 0xe928d0d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb28d0d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d88d0dc, SUC_lexicon_tags_12 },
    { 0x6c68d0dc, SUC_lexicon_tags_60 },
    { 0x4748d0dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da8d0e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6908d0e3, SUC_lexicon_tags_33 },
    { 0x1048d0e4, SUC_lexicon_tags_4 },
    { 0x1528d0e4, SUC_lexicon_tags_25 },
    { 0x9ec8d0e4, SUC_lexicon_tags_17 },
    { 0xcf48d0e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf868d0ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c8d0ee, SUC_lexicon_tags_9 },
    { 0xace8d0ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a8d0f4, SUC_lexicon_tags_15 },
    { 0xf768d0f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4568d0f7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1068d0ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1708d104, SUC_lexicon_tags_11 },
    { 0xfba8d105, SUC_lexicon_tags_1 },
    { 0x8888d106, SUC_lexicon_tags_1 },
    { 0x4668d107, SUC_lexicon_tags_39 },
    { 0x4c68d108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7388d10b, SUC_lexicon_tags_23 },
    { 0x5548d10b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x1068d10f, SUC_lexicon_tags_5 },
    { 0xc8a8d10f, SUC_lexicon_tags_2 },
    { 0xee28d111, SUC_lexicon_tags_25 },
    { 0xb7a8d112, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47c8d114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4a8d116, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4428d118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78e8d11a, SUC_lexicon_tags_47 },
    { 0x46e8d11b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae08d11f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e28d121, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb48d124, SUC_lexicon_tags_7 },
    { 0x9e68d125, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ac8d127, SUC_lexicon_tags_16 },
    { 0xef48d128, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa088d12a, SUC_lexicon_tags_7 },
    { 0x5748d12a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a8d138, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e68d13a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec68d13c, SUC_lexicon_tags_17 },
    { 0x22a8d13d, SUC_lexicon_tags_15 },
    { 0x1568d13e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea68d143, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa48d146, SUC_lexicon_tags_0 },
    { 0x5b28d147, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc608d14a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6148d14c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee8d153, SUC_lexicon_tags_238 },
    { 0x23a8d154, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c8d159, SUC_lexicon_tags_7 },
    { 0x5cc8d159, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a8d162, SUC_lexicon_tags_25 },
    { 0xd9e8d162, SUC_lexicon_tags_2 },
    { 0xd368d162, SUC_lexicon_tags_1235 },
    { 0x2668d165, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc488d169, SUC_lexicon_tags_11 },
    { 0xb348d16a, SUC_lexicon_tags_25 },
    { 0xbb88d16b, SUC_lexicon_tags_14 },
    { 0x78a8d16a, SUC_lexicon_tags_23 },
    { 0xa68d16d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6808d170, SUC_lexicon_tags_30 },
    { 0x2868d171, SUC_lexicon_tags_26 },
    { 0xcfc8d171, SUC_lexicon_tags_60 },
    { 0x31e8d170, SUC_lexicon_tags_37 },
    { 0x1148d170, SUC_lexicon_tags_28 },
    { 0xa5a8d174, SUC_lexicon_tags_2 },
    { 0xffa8d176, SUC_lexicon_tags_27 },
    { 0x9688d175, SUC_lexicon_tags_26 },
    { 0x1568d177, SUC_lexicon_tags_49 },
    { 0x4688d179, SUC_lexicon_tags_25 },
    { 0x6628d17a, SUC_lexicon_tags_11 },
    { 0x76a8d17a, SUC_lexicon_tags_30 },
    { 0xc068d174, SUC_lexicon_tags_11 },
    { 0x2b28d174, SUC_lexicon_tags_87 },
    { 0x4b88d17e, SUC_lexicon_tags_9 },
    { 0xf008d174, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfa88d181, SUC_lexicon_tags_12 },
    { 0x5f08d182, SUC_lexicon_tags_31 },
    { 0x3188d181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde8d186, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2648d18a, SUC_lexicon_tags_25 },
    { 0xc8c8d18a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7788d190, SUC_lexicon_tags_11 },
    { 0x348d191, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6288d195, SUC_lexicon_tags_37 },
    { 0x4c28d196, SUC_lexicon_tags_56 },
    { 0x6608d197, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca8d1a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc388d1a8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08d1ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bc8d1b0, SUC_lexicon_tags_17 },
    { 0xbda8d1b1, SUC_lexicon_tags_45 },
    { 0x71c8d1b0, SUC_lexicon_tags_47 },
    { 0x2a68d1b1, SUC_lexicon_tags_38 },
    { 0x4be8d1b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8de8d1b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c08d1b8, SUC_lexicon_tags_63 },
    { 0xfd28d1b8, SUC_lexicon_tags_37 },
    { 0xd248d1b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf228d1c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb388d1cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b08d1ce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a68d1d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48d1d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac08d1da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9248d1dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa508d1e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd08d1e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba8d1e7, SUC_lexicon_tags_19 },
    { 0x4908d1e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb68d1eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c8d1f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e8d1f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c8d1f8, SUC_lexicon_tags_43 },
    { 0x6728d1f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c8d1fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d48d1ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6628d203, SUC_lexicon_tags_11 },
    { 0xea48d204, SUC_lexicon_tags_63 },
    { 0xe0c8d205, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8088d208, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc8d20b, SUC_lexicon_tags_25 },
    { 0x6888d20b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb968d20e, SUC_lexicon_tags_49 },
    { 0x9ac8d20f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc548d215, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3f28d217, SUC_lexicon_tags_11 },
    { 0xac48d218, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6068d21a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7888d21c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf788d227, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb08d22a, SUC_lexicon_tags_626 },
    { 0xbfc8d22a, SUC_lexicon_tags_25 },
    { 0xe9c8d22a, SUC_lexicon_tags_37 },
    { 0x4548d22d, SUC_lexicon_tags_2 },
    { 0xe728d22a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa608d230, SUC_lexicon_tags_25 },
    { 0x50c8d230, SUC_lexicon_tags_71 },
    { 0xe008d232, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68d238, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8d241, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde8d244, SUC_lexicon_tags_47 },
    { 0x60c8d244, SUC_lexicon_tags_61 },
    { 0x9a8d246, SUC_lexicon_tags_2 },
    { 0xa7e8d244, SUC_lexicon_tags_60 },
    { 0x87e8d245, SUC_lexicon_tags_7 },
    { 0xfa48d247, SUC_lexicon_tags_63 },
    { 0x94c8d248, SUC_lexicon_tags_17 },
    { 0xc148d24b, SUC_lexicon_tags_11 },
    { 0x3848d24c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e8d253, SUC_lexicon_tags_9 },
    { 0x6a8d253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a08d256, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa9e8d258, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf128d25b, SUC_lexicon_tags_23 },
    { 0x7a88d25c, SUC_lexicon_tags_31 },
    { 0xa3a8d25b, SUC_lexicon_tags_17 },
    { 0x2d08d25e, SUC_lexicon_tags_11 },
    { 0x8e28d25e, SUC_lexicon_tags_0 },
    { 0x5828d25e, SUC_lexicon_tags_0 },
    { 0x1248d261, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3788d263, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5a8d265, SUC_lexicon_tags_39 },
    { 0x7fc8d266, SUC_lexicon_tags_45 },
    { 0x9948d267, SUC_lexicon_tags_45 },
    { 0x2388d267, SUC_lexicon_tags_1591 },
    { 0x3c08d269, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa728d26d, SUC_lexicon_tags_17 },
    { 0x8948d26e, SUC_lexicon_tags_46 },
    { 0xfcc8d26f, SUC_lexicon_tags_12 },
    { 0xea8d26d, SUC_lexicon_tags_12 },
    { 0x73a8d271, SUC_lexicon_tags_4 },
    { 0x688d272, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6968d276, SUC_lexicon_tags_12 },
    { 0x56e8d276, SUC_lexicon_tags_11 },
    { 0xffe8d276, SUC_lexicon_tags_15 },
    { 0x13c8d277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a68d287, SUC_lexicon_tags_8 },
    { 0xb328d287, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6268d28f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x428d292, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b28d294, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b48d297, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8a8d299, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xbda8d29b, SUC_lexicon_tags_68 },
    { 0x72e8d29c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69a8d29e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3328d2ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1188d2b0, SUC_lexicon_tags_37 },
    { 0x3528d2b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e8d2b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3a8d2ba, SUC_lexicon_tags_9 },
    { 0x1728d2bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19c8d2bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6808d2bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8d2c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf28d2cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5348d2d1, SUC_lexicon_tags_37 },
    { 0x4648d2d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ec8d2d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3228d2df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d28d2e2, SUC_lexicon_tags_8 },
    { 0x1128d2e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7868d2e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x488d2e8, SUC_lexicon_tags_60 },
    { 0x4308d2e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c8d2ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x79c8d2ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1408d2f0, SUC_lexicon_tags_37 },
    { 0x3d48d2f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c8d2f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7e8d2f6, SUC_lexicon_tags_15 },
    { 0xd288d2f7, SUC_lexicon_tags_27 },
    { 0x8e8d2f8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e88d2fe, SUC_lexicon_tags_7 },
    { 0xe828d2fe, SUC_lexicon_tags_6 },
    { 0xcba8d300, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e48d302, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec8d306, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x788d308, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4548d30a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1268d312, SUC_lexicon_tags_21 },
    { 0x2dc8d313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba8d317, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9928d31a, SUC_lexicon_tags_27 },
    { 0x9a68d31a, SUC_lexicon_tags_2 },
    { 0xfc88d31c, SUC_lexicon_tags_31 },
    { 0x8b88d31d, SUC_lexicon_tags_37 },
    { 0xfea8d31d, SUC_lexicon_tags_0 },
    { 0xdd68d31f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5668d323, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92a8d325, SUC_lexicon_tags_33 },
    { 0x7ec8d326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba8d329, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc468d32c, SUC_lexicon_tags_26 },
    { 0x328d32d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4448d32f, SUC_lexicon_tags_26 },
    { 0xaee8d330, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e68d334, SUC_lexicon_tags_27 },
    { 0x79c8d335, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac8d339, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc868d33b, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x1868d33d, SUC_lexicon_tags_37 },
    { 0xe188d33e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa648d342, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe8d347, SUC_lexicon_tags_60 },
    { 0xee48d347, SUC_lexicon_tags_70 },
    { 0x7d68d349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7528d350, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e8d354, SUC_lexicon_tags_2 },
    { 0xd4c8d355, SUC_lexicon_tags_112 },
    { 0x4d08d354, SUC_lexicon_tags_9 },
    { 0xa488d357, SUC_lexicon_tags_17 },
    { 0x7aa8d357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a8d35b, SUC_lexicon_tags_19 },
    { 0xff48d35c, SUC_lexicon_tags_47 },
    { 0x2988d35c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e48d363, SUC_lexicon_tags_11 },
    { 0x16c8d364, SUC_lexicon_tags_1 },
    { 0x1048d365, SUC_lexicon_tags_26 },
    { 0xeb28d366, SUC_lexicon_tags_7 },
    { 0x9f28d367, SUC_lexicon_tags_23 },
    { 0x2f88d366, SUC_lexicon_tags_30 },
    { 0x2d48d369, SUC_lexicon_tags_14 },
    { 0xce48d366, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b48d36d, SUC_lexicon_tags_30 },
    { 0x4d48d36e, SUC_lexicon_tags_45 },
    { 0x8de8d36f, SUC_lexicon_tags_19 },
    { 0xfaa8d36d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7548d374, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab68d37a, SUC_lexicon_tags_19 },
    { 0x14c8d37b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe148d381, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd88d384, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba8d388, SUC_lexicon_tags_103 },
    { 0xd3e8d389, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4788d38b, SUC_lexicon_tags_20 },
    { 0xf688d38c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6988d38f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a8d39a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f48d3a0, SUC_lexicon_tags_57 },
    { 0x5d88d3a0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9268d3a5, SUC_lexicon_tags_37 },
    { 0x1108d3a6, SUC_lexicon_tags_45 },
    { 0xcb48d3a7, SUC_lexicon_tags_25 },
    { 0x4188d3a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fe8d3aa, SUC_lexicon_tags_38 },
    { 0x4b08d3aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c8d3af, SUC_lexicon_tags_7 },
    { 0x39c8d3af, SUC_lexicon_tags_15 },
    { 0x4ec8d3af, SUC_lexicon_tags_7 },
    { 0xb028d3b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x92c8d3b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f68d3ba, SUC_lexicon_tags_32 },
    { 0xacc8d3ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe148d3be, SUC_lexicon_tags_60 },
    { 0x4e8d3bf, SUC_lexicon_tags_7 },
    { 0xf7a8d3c0, SUC_lexicon_tags_15 },
    { 0x2e8d3c1, SUC_lexicon_tags_19 },
    { 0x8d68d3c1, SUC_lexicon_tags_38 },
    { 0xc708d3bf, SUC_lexicon_tags_37 },
    { 0x9ac8d3c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8568d3c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3468d3cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3548d3d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8d3d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5788d3db, SUC_lexicon_tags_7 },
    { 0xf348d3dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8508d3e1, SUC_lexicon_tags_12 },
    { 0x7da8d3e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb168d3e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a8d3f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a8d3f7, SUC_lexicon_tags_14 },
    { 0x6de8d3f8, SUC_lexicon_tags_28 },
    { 0x7848d3f7, SUC_lexicon_tags_23 },
    { 0x6b88d3fa, SUC_lexicon_tags_14 },
    { 0x1508d3f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97e8d3fd, SUC_lexicon_tags_17 },
    { 0x3ca8d3fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc648d401, SUC_lexicon_tags_19 },
    { 0x1148d402, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1508d404, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5268d406, SUC_lexicon_tags_37 },
    { 0xa128d406, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe688d40a, SUC_lexicon_tags_85 },
    { 0xe5e8d40a, SUC_lexicon_tags_39 },
    { 0x91c8d40c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4bc8d40e, SUC_lexicon_tags_49 },
    { 0x7168d40e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe08d417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe308d41a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaaa8d41c, SUC_lexicon_tags_15 },
    { 0xc1a8d41c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c8d423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88d42b, SUC_lexicon_tags_0 },
    { 0x54e8d42c, SUC_lexicon_tags_37 },
    { 0x2f68d42d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9528d431, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac88d434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d88d437, SUC_lexicon_tags_14 },
    { 0xf3c8d437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec28d43a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa468d43e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee8d445, SUC_lexicon_tags_46 },
    { 0xc588d446, SUC_lexicon_tags_12 },
    { 0xa8d447, SUC_lexicon_tags_21 },
    { 0x8928d448, SUC_lexicon_tags_37 },
    { 0xb288d447, SUC_lexicon_tags_47 },
    { 0x2768d44a, SUC_lexicon_tags_25 },
    { 0x6e68d44b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e8d44f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e8d452, SUC_lexicon_tags_4 },
    { 0x8108d452, SUC_lexicon_tags_17 },
    { 0x29c8d452, SUC_lexicon_tags_25 },
    { 0xd088d455, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a8d45c, SUC_lexicon_tags_14 },
    { 0xfbe8d45d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8528d45f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf28d466, SUC_lexicon_tags_37 },
    { 0xa908d467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc948d46d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb48d471, SUC_lexicon_tags_6 },
    { 0x7608d472, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a8d476, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa208d478, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8da8d47a, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0x1068d47d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe408d480, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc748d482, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b88d484, SUC_lexicon_tags_17 },
    { 0x3c88d484, SUC_lexicon_tags_68 },
    { 0x8ce8d486, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf008d488, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc208d48a, SUC_lexicon_tags_47 },
    { 0xf148d48b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x808d48d, SUC_lexicon_tags_63 },
    { 0xe828d48e, SUC_lexicon_tags_2 },
    { 0xf268d48f, SUC_lexicon_tags_37 },
    { 0x19a8d48d, SUC_lexicon_tags_27 },
    { 0xee48d48e, SUC_lexicon_tags_68 },
    { 0xa688d492, SUC_lexicon_tags_0 },
    { 0x1628d48f, SUC_lexicon_tags_2 },
    { 0xc048d494, SUC_lexicon_tags_37 },
    { 0xe5e8d495, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e8d499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5e8d49b, SUC_lexicon_tags_12 },
    { 0xb0a8d49c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde48d4a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24a8d4a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55e8d4a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f28d4ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6828d4b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1168d4b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x688d4ba, SUC_lexicon_tags_17 },
    { 0x7ca8d4ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc8d4c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e8d4c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3608d4ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda8d4ce, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2088d4d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a8d4d5, SUC_lexicon_tags_26 },
    { 0xd0e8d4d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b28d4dd, SUC_lexicon_tags_23 },
    { 0x2608d4de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ee8d4e0, SUC_lexicon_tags_1 },
    { 0x23e8d4e1, SUC_lexicon_tags_25 },
    { 0x9d28d4e0, SUC_lexicon_tags_8 },
    { 0xc208d4e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa328d4e6, SUC_lexicon_tags_27 },
    { 0xa7a8d4e7, SUC_lexicon_tags_77 },
    { 0xd08d4e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf68d4ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc68d4f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e08d4f2, SUC_lexicon_tags_187 },
    { 0xf9a8d4f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac08d4f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2148d4fa, SUC_lexicon_tags_14 },
    { 0xe928d4fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf068d4fe, SUC_lexicon_tags_25 },
    { 0x47e8d4ff, SUC_lexicon_tags_29 },
    { 0xad68d500, SUC_lexicon_tags_2 },
    { 0x2548d4fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a68d506, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c28d509, SUC_lexicon_tags_14 },
    { 0xda28d509, SUC_lexicon_tags_11 },
    { 0x7528d50b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2608d50d, SUC_lexicon_tags_18 },
    { 0xbf88d50d, SUC_lexicon_tags_15 },
    { 0x25e8d50f, SUC_lexicon_tags_11 },
    { 0xa048d510, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9908d512, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7088d518, SUC_lexicon_tags_19 },
    { 0xe188d519, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd08d51b, SUC_lexicon_tags_68 },
    { 0xe028d51c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1868d51e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8348d520, SUC_lexicon_tags_7 },
    { 0x98e8d521, SUC_lexicon_tags_37 },
    { 0xfb48d521, SUC_lexicon_tags_38 },
    { 0x788d523, SUC_lexicon_tags_172 },
    { 0xf028d523, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee8d529, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x90a8d52b, SUC_lexicon_tags_43 },
    { 0xe068d52b, SUC_lexicon_tags_15 },
    { 0xd348d52c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe28d52f, SUC_lexicon_tags_68 },
    { 0xe6e8d530, SUC_lexicon_tags_1 },
    { 0xeae8d531, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7648d537, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7288d53f, SUC_lexicon_tags_0 },
    { 0x128d540, SUC_lexicon_tags_26 },
    { 0x8c08d541, SUC_lexicon_tags_0 },
    { 0xc108d540, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2048d544, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa608d548, SUC_lexicon_tags_33 },
    { 0x4e08d548, SUC_lexicon_tags_1 },
    { 0xe128d548, SUC_lexicon_tags_15 },
    { 0xc908d54b, SUC_lexicon_tags_19 },
    { 0x20e8d54c, SUC_lexicon_tags_63 },
    { 0xaa08d54b, SUC_lexicon_tags_30 },
    { 0xd708d54b, SUC_lexicon_tags_3 },
    { 0x5c68d54f, SUC_lexicon_tags_0 },
    { 0x8d54b, SUC_lexicon_tags_47 },
    { 0xbfc8d54f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848d555, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc28d55c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3368d55e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2e8d560, SUC_lexicon_tags_14 },
    { 0x2b08d561, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb88d565, SUC_lexicon_tags_66 },
    { 0x7fc8d566, SUC_lexicon_tags_2 },
    { 0x1fc8d567, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be8d56d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe848d571, SUC_lexicon_tags_37 },
    { 0xbc08d572, SUC_lexicon_tags_15 },
    { 0x37c8d573, SUC_lexicon_tags_16 },
    { 0x67e8d574, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6e8d576, SUC_lexicon_tags_25 },
    { 0x4848d576, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcce8d579, SUC_lexicon_tags_42 },
    { 0x46e8d57a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fa8d57c, SUC_lexicon_tags_612 },
    { 0xef28d57d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d587, SUC_lexicon_tags_7 },
    { 0x1a08d588, SUC_lexicon_tags_39 },
    { 0xf408d587, SUC_lexicon_tags_3 },
    { 0x828d58a, SUC_lexicon_tags_7 },
    { 0xf228d58b, SUC_lexicon_tags_15 },
    { 0x6d48d58c, SUC_lexicon_tags_2 },
    { 0x448d58d, SUC_lexicon_tags_37 },
    { 0xcba8d58d, SUC_lexicon_tags_19 },
    { 0x1ea8d58d, SUC_lexicon_tags_12 },
    { 0x7968d590, SUC_lexicon_tags_1 },
    { 0x6148d58d, SUC_lexicon_tags_31 },
    { 0x2a28d592, SUC_lexicon_tags_29 },
    { 0x90c8d588, SUC_lexicon_tags_42 },
    { 0x4fa8d594, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e8d598, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c8d5a1, SUC_lexicon_tags_63 },
    { 0x7328d5a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62c8d5a4, SUC_lexicon_tags_1 },
    { 0xf328d5a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36c8d5a7, SUC_lexicon_tags_11 },
    { 0x21a8d5a8, SUC_lexicon_tags_1 },
    { 0x11e8d5a9, SUC_lexicon_tags_31 },
    { 0x2b68d5aa, SUC_lexicon_tags_25 },
    { 0xf5a8d5ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a8d5b0, SUC_lexicon_tags_1 },
    { 0x8828d5b1, SUC_lexicon_tags_56 },
    { 0x688d5b1, SUC_lexicon_tags_25 },
    { 0xe8e8d5b3, SUC_lexicon_tags_17 },
    { 0xdac8d5b3, SUC_lexicon_tags_45 },
    { 0x9428d5b5, SUC_lexicon_tags_15 },
    { 0x1fe8d5b2, SUC_lexicon_tags_26 },
    { 0x7648d5b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92a8d5b9, SUC_lexicon_tags_2 },
    { 0x3688d5ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd948d5bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc8d5c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c8d5c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2828d5cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd568d5cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e8d5d6, SUC_lexicon_tags_23 },
    { 0xca8d5d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8e8d5d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2128d5db, SUC_lexicon_tags_19 },
    { 0xd2c8d5db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42c8d5de, SUC_lexicon_tags_27 },
    { 0x7a68d5df, SUC_lexicon_tags_3 },
    { 0x4928d5e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e8d5e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8888d5e6, SUC_lexicon_tags_12 },
    { 0xc228d5e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9408d5eb, SUC_lexicon_tags_33 },
    { 0x2dc8d5eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2768d5f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea48d5f6, SUC_lexicon_tags_37 },
    { 0xb108d5f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b88d5fc, SUC_lexicon_tags_39 },
    { 0x70c8d5fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x288d5ff, SUC_lexicon_tags_11 },
    { 0x32e8d5ff, SUC_lexicon_tags_13 },
    { 0xad88d600, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b28d609, SUC_lexicon_tags_29 },
    { 0x5928d60a, SUC_lexicon_tags_7 },
    { 0x7ac8d60b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da8d612, SUC_lexicon_tags_17 },
    { 0x60e8d613, SUC_lexicon_tags_6 },
    { 0xb148d614, SUC_lexicon_tags_7 },
    { 0x1648d615, SUC_lexicon_tags_25 },
    { 0x808d613, SUC_lexicon_tags_17 },
    { 0x98a8d614, SUC_lexicon_tags_15 },
    { 0x8688d618, SUC_lexicon_tags_37 },
    { 0x45a8d619, SUC_lexicon_tags_45 },
    { 0x3968d619, SUC_lexicon_tags_37 },
    { 0x5d88d618, SUC_lexicon_tags_1 },
    { 0x5e88d61c, SUC_lexicon_tags_25 },
    { 0x3b48d619, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb868d61f, SUC_lexicon_tags_25 },
    { 0xeae8d620, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x17c8d622, SUC_lexicon_tags_1 },
    { 0xd448d623, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c8d62a, SUC_lexicon_tags_10 },
    { 0xeb08d62b, SUC_lexicon_tags_17 },
    { 0x42c8d62b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb8c8d62e, SUC_lexicon_tags_29 },
    { 0xd908d62e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228d636, SUC_lexicon_tags_27 },
    { 0x3968d637, SUC_lexicon_tags_15 },
    { 0x3e48d637, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad08d640, SUC_lexicon_tags_6 },
    { 0xf7c8d641, SUC_lexicon_tags_30 },
    { 0x2f48d640, SUC_lexicon_tags_1 },
    { 0x1d88d643, SUC_lexicon_tags_32 },
    { 0xcf08d640, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe8d647, SUC_lexicon_tags_34 },
    { 0xe8c8d648, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb48d64f, SUC_lexicon_tags_29 },
    { 0x7b28d650, SUC_lexicon_tags_37 },
    { 0x2e08d651, SUC_lexicon_tags_234 },
    { 0x1fa8d651, SUC_lexicon_tags_43 },
    { 0x89e8d653, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8d656, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58e8d658, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88d65c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa3c8d65e, SUC_lexicon_tags_17 },
    { 0x8a08d65f, SUC_lexicon_tags_1 },
    { 0xe728d65f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc8d663, SUC_lexicon_tags_17 },
    { 0x2a48d664, SUC_lexicon_tags_11 },
    { 0xb028d664, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a8d66c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c8d671, SUC_lexicon_tags_5 },
    { 0x3dc8d672, SUC_lexicon_tags_28 },
    { 0xe5c8d673, SUC_lexicon_tags_1 },
    { 0x19e8d673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1808d678, SUC_lexicon_tags_8 },
    { 0xdae8d678, SUC_lexicon_tags_47 },
    { 0xcd48d67a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x26c8d67c, SUC_lexicon_tags_0 },
    { 0x8c68d67d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a8d682, SUC_lexicon_tags_65 },
    { 0xd3c8d682, SUC_lexicon_tags_28 },
    { 0xe2e8d684, SUC_lexicon_tags_4 },
    { 0x34a8d683, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc48d689, SUC_lexicon_tags_49 },
    { 0x91e8d68a, SUC_lexicon_tags_15 },
    { 0x3de8d68b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d08d690, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e08d695, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa288d697, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x50c8d699, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe868d69e, SUC_lexicon_tags_1 },
    { 0x7788d69f, SUC_lexicon_tags_12 },
    { 0x11e8d6a0, SUC_lexicon_tags_26 },
    { 0xca48d6a1, SUC_lexicon_tags_26 },
    { 0x6268d6a2, SUC_lexicon_tags_35 },
    { 0x2ac8d6a2, SUC_lexicon_tags_47 },
    { 0xc5a8d6a3, SUC_lexicon_tags_25 },
    { 0x70e8d6a4, SUC_lexicon_tags_9 },
    { 0x9fe8d6a6, SUC_lexicon_tags_1 },
    { 0xc48d6a7, SUC_lexicon_tags_30 },
    { 0x4528d6a3, SUC_lexicon_tags_25 },
    { 0x5728d6a3, SUC_lexicon_tags_17 },
    { 0x2ec8d6a1, SUC_lexicon_tags_47 },
    { 0x6ea8d6ab, SUC_lexicon_tags_15 },
    { 0x6908d6ab, SUC_lexicon_tags_23 },
    { 0x3de8d69e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf68d6b1, SUC_lexicon_tags_38 },
    { 0xafa8d6b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6448d6b6, SUC_lexicon_tags_23 },
    { 0x3728d6b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608d6bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeea8d6bf, SUC_lexicon_tags_47 },
    { 0x83e8d6c0, SUC_lexicon_tags_6 },
    { 0xbec8d6c1, SUC_lexicon_tags_45 },
    { 0x608d6c1, SUC_lexicon_tags_17 },
    { 0xc168d6c3, SUC_lexicon_tags_0 },
    { 0xe768d6c3, SUC_lexicon_tags_15 },
    { 0x5a68d6c2, SUC_lexicon_tags_38 },
    { 0x5468d6c6, SUC_lexicon_tags_27 },
    { 0x8c28d6c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb548d6d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9368d6d6, SUC_lexicon_tags_27 },
    { 0x1f48d6d7, SUC_lexicon_tags_25 },
    { 0x8da8d6d8, SUC_lexicon_tags_10 },
    { 0x1c48d6d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b68d6db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb008d6de, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b68d6e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd68d6e5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a28d6ec, SUC_lexicon_tags_12 },
    { 0x5fa8d6ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4248d6f0, SUC_lexicon_tags_37 },
    { 0x14a8d6f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c8d6f5, SUC_lexicon_tags_6 },
    { 0x6e88d6f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b08d6fc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe5a8d6fe, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x3788d700, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc8d704, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e8d70a, SUC_lexicon_tags_46 },
    { 0x868d70b, SUC_lexicon_tags_23 },
    { 0xbfc8d70c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a8d70f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa48d716, SUC_lexicon_tags_1 },
    { 0xdea8d717, SUC_lexicon_tags_45 },
    { 0x21a8d718, SUC_lexicon_tags_1 },
    { 0x4f68d719, SUC_lexicon_tags_17 },
    { 0x7c08d719, SUC_lexicon_tags_15 },
    { 0x4b08d719, SUC_lexicon_tags_0 },
    { 0x36e8d719, SUC_lexicon_tags_25 },
    { 0x1c28d718, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de8d720, SUC_lexicon_tags_37 },
    { 0x49c8d721, SUC_lexicon_tags_4 },
    { 0xd388d722, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xc428d724, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb328d728, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7948d72b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc8d72d, SUC_lexicon_tags_14 },
    { 0x1c08d72e, SUC_lexicon_tags_14 },
    { 0x1d68d72f, SUC_lexicon_tags_83 },
    { 0xaa68d72d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc308d732, SUC_lexicon_tags_39 },
    { 0x95c8d732, SUC_lexicon_tags_2 },
    { 0x4dc8d732, SUC_lexicon_tags_4 },
    { 0xffc8d734, SUC_lexicon_tags_25 },
    { 0x5308d732, SUC_lexicon_tags_65 },
    { 0xe6a8d737, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7be8d739, SUC_lexicon_tags_11 },
    { 0x7ce8d739, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a8d73d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf048d744, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42e8d746, SUC_lexicon_tags_11 },
    { 0x5ee8d746, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2468d74a, SUC_lexicon_tags_7 },
    { 0x1bc8d74b, SUC_lexicon_tags_0 },
    { 0x9188d74c, SUC_lexicon_tags_12 },
    { 0x29e8d74c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x67e8d74f, SUC_lexicon_tags_25 },
    { 0xd028d74f, SUC_lexicon_tags_10 },
    { 0xf68d751, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1c8d753, SUC_lexicon_tags_37 },
    { 0x29c8d754, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca8d758, SUC_lexicon_tags_39 },
    { 0xc6a8d759, SUC_lexicon_tags_63 },
    { 0xe808d75a, SUC_lexicon_tags_29 },
    { 0x9068d75b, SUC_lexicon_tags_15 },
    { 0xdc88d75a, SUC_lexicon_tags_11 },
    { 0xe808d75d, SUC_lexicon_tags_12 },
    { 0xaa88d75a, SUC_lexicon_tags_11 },
    { 0x4ce8d75f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc0c8d761, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a8d764, SUC_lexicon_tags_12 },
    { 0xd8a8d764, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9b28d767, SUC_lexicon_tags_8 },
    { 0x26c8d768, SUC_lexicon_tags_7 },
    { 0x80e8d768, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e8d76c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5988d76f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfc8d771, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7368d773, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84e8d775, SUC_lexicon_tags_36 },
    { 0x92c8d775, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e08d778, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f48d77d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec8d782, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e48d785, SUC_lexicon_tags_49 },
    { 0x7b68d786, SUC_lexicon_tags_31 },
    { 0x7ca8d785, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a48d789, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c8d78d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2468d793, SUC_lexicon_tags_12 },
    { 0x5308d793, SUC_lexicon_tags_1 },
    { 0x2368d795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c48d798, SUC_lexicon_tags_12 },
    { 0xc008d799, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57e8d7a0, SUC_lexicon_tags_8 },
    { 0x9b48d7a1, SUC_lexicon_tags_27 },
    { 0x4b08d7a2, SUC_lexicon_tags_33 },
    { 0x9e8d7a2, SUC_lexicon_tags_47 },
    { 0x9d88d7a1, SUC_lexicon_tags_2 },
    { 0x12a8d7a5, SUC_lexicon_tags_35 },
    { 0xb8e8d7a2, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xe628d7a8, SUC_lexicon_tags_37 },
    { 0x5bc8d7a9, SUC_lexicon_tags_33 },
    { 0xb568d7aa, SUC_lexicon_tags_37 },
    { 0x5588d7ab, SUC_lexicon_tags_25 },
    { 0x8a08d7a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e8d7b0, SUC_lexicon_tags_10 },
    { 0xcba8d7b1, SUC_lexicon_tags_1 },
    { 0x7588d7b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9408d7b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbda8d7b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4028d7c2, SUC_lexicon_tags_16 },
    { 0x3268d7c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa028d7c6, SUC_lexicon_tags_104 },
    { 0x21e8d7c6, SUC_lexicon_tags_65 },
    { 0x76c8d7c8, SUC_lexicon_tags_10 },
    { 0xfb28d7c8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x99a8d7cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a48d7cf, SUC_lexicon_tags_12 },
    { 0xb7c8d7cf, SUC_lexicon_tags_30 },
    { 0xbd48d7d1, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe928d7d7, SUC_lexicon_tags_82 },
    { 0xdf68d7d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6448d7da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc8d7e0, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0xef08d7e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e8d7e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf88d7e8, SUC_lexicon_tags_4 },
    { 0x5c28d7e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc148d7eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeae8d7ed, SUC_lexicon_tags_14 },
    { 0x1088d7ed, SUC_lexicon_tags_0 },
    { 0xf4c8d7ef, SUC_lexicon_tags_46 },
    { 0xb748d7ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8d7f5, SUC_lexicon_tags_14 },
    { 0x1d08d7f6, SUC_lexicon_tags_17 },
    { 0xbac8d7f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc108d7fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08d804, SUC_lexicon_tags_1 },
    { 0x4f28d805, SUC_lexicon_tags_25 },
    { 0x9228d806, SUC_lexicon_tags_1 },
    { 0xe5e8d806, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c8d80c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9c8d80e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3248d810, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc868d812, SUC_lexicon_tags_25 },
    { 0x4a8d813, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb68d819, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc8d81c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a88d81f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba8d822, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608d829, SUC_lexicon_tags_23 },
    { 0xee08d82a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8cc8d82c, SUC_lexicon_tags_25 },
    { 0x1048d82d, SUC_lexicon_tags_2 },
    { 0xa188d82e, SUC_lexicon_tags_37 },
    { 0xa768d82f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe808d832, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb308d834, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9768d83b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d28d83e, SUC_lexicon_tags_1 },
    { 0x34a8d83f, SUC_lexicon_tags_25 },
    { 0x7ba8d840, SUC_lexicon_tags_1 },
    { 0x8648d83f, SUC_lexicon_tags_7 },
    { 0x51c8d842, SUC_lexicon_tags_10 },
    { 0x8408d843, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8768d84a, SUC_lexicon_tags_33 },
    { 0xfe48d84b, SUC_lexicon_tags_27 },
    { 0x3568d84b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a8d850, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb808d853, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5428d856, SUC_lexicon_tags_11 },
    { 0x97a8d857, SUC_lexicon_tags_25 },
    { 0xc928d858, SUC_lexicon_tags_12 },
    { 0x4bc8d856, SUC_lexicon_tags_13 },
    { 0x5e8d85a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd508d85e, SUC_lexicon_tags_85 },
    { 0x2028d85f, SUC_lexicon_tags_0 },
    { 0xdaa8d85f, SUC_lexicon_tags_2 },
    { 0xf868d861, SUC_lexicon_tags_8 },
    { 0x51e8d862, SUC_lexicon_tags_11 },
    { 0x6768d862, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x8568d865, SUC_lexicon_tags_37 },
    { 0x2208d866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3e8d868, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe968d86d, SUC_lexicon_tags_11 },
    { 0xe5e8d86d, SUC_lexicon_tags_29 },
    { 0xe4c8d86f, SUC_lexicon_tags_31 },
    { 0xbb88d86d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb88d873, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e8d876, SUC_lexicon_tags_48 },
    { 0x608d877, SUC_lexicon_tags_37 },
    { 0x69e8d877, SUC_lexicon_tags_11 },
    { 0xd3c8d879, SUC_lexicon_tags_27 },
    { 0x66c8d879, SUC_lexicon_tags_29 },
    { 0x6808d87a, SUC_lexicon_tags_37 },
    { 0xe488d87c, SUC_lexicon_tags_2 },
    { 0x1c28d87d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c8d881, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf2c8d883, SUC_lexicon_tags_7 },
    { 0xe628d883, SUC_lexicon_tags_60 },
    { 0x6aa8d884, SUC_lexicon_tags_17 },
    { 0x8868d886, SUC_lexicon_tags_2 },
    { 0x3aa8d887, SUC_lexicon_tags_25 },
    { 0xa08d887, SUC_lexicon_tags_37 },
    { 0x2f28d883, SUC_lexicon_tags_48 },
    { 0x6208d886, SUC_lexicon_tags_22 },
    { 0xef88d88a, SUC_lexicon_tags_11 },
    { 0x4de8d88c, SUC_lexicon_tags_33 },
    { 0xa108d88d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d88d894, SUC_lexicon_tags_15 },
    { 0x5208d895, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe488d898, SUC_lexicon_tags_34 },
    { 0xa7e8d899, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9388d89f, SUC_lexicon_tags_19 },
    { 0x3ec8d89f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7928d8a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3268d8a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c8d8a8, SUC_lexicon_tags_37 },
    { 0x78c8d8a9, SUC_lexicon_tags_30 },
    { 0x8be8d8aa, SUC_lexicon_tags_11 },
    { 0x5e48d8a9, SUC_lexicon_tags_11 },
    { 0x9cc8d8aa, SUC_lexicon_tags_12 },
    { 0x11a8d8ad, SUC_lexicon_tags_15 },
    { 0x85e8d8ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce8d8b1, SUC_lexicon_tags_25 },
    { 0x73c8d8b2, SUC_lexicon_tags_9 },
    { 0x28a8d8b1, SUC_lexicon_tags_37 },
    { 0xefc8d8b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c28d8ba, SUC_lexicon_tags_25 },
    { 0x60e8d8bb, SUC_lexicon_tags_7 },
    { 0xae28d8bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2588d8be, SUC_lexicon_tags_37 },
    { 0x8668d8be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a8d8c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a68d8cd, SUC_lexicon_tags_45 },
    { 0xffa8d8ce, SUC_lexicon_tags_10 },
    { 0x9308d8ce, SUC_lexicon_tags_7 },
    { 0x7868d8d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5468d8d2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x9b08d8d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c8d8dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef88d8e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c8d8e3, SUC_lexicon_tags_17 },
    { 0xac88d8e3, SUC_lexicon_tags_37 },
    { 0x64e8d8e5, SUC_lexicon_tags_26 },
    { 0xbc88d8e5, SUC_lexicon_tags_15 },
    { 0x49e8d8e7, SUC_lexicon_tags_17 },
    { 0x3ea8d8e8, SUC_lexicon_tags_14 },
    { 0x3ee8d8e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb08d8ec, SUC_lexicon_tags_9 },
    { 0x1468d8ec, SUC_lexicon_tags_45 },
    { 0x3228d8ed, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa908d8f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff68d8f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e48d8fe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d28d904, SUC_lexicon_tags_37 },
    { 0x48c8d905, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe2c8d907, SUC_lexicon_tags_11 },
    { 0xc6e8d907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6a8d90a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a8d90d, SUC_lexicon_tags_1 },
    { 0xca28d90e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1408d914, SUC_lexicon_tags_49 },
    { 0x5d68d915, SUC_lexicon_tags_17 },
    { 0xe748d916, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xada8d918, SUC_lexicon_tags_18 },
    { 0x9388d919, SUC_lexicon_tags_39 },
    { 0x8fc8d91a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34e8d91c, SUC_lexicon_tags_1 },
    { 0xf7e8d91c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7148d922, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a8d928, SUC_lexicon_tags_38 },
    { 0xeee8d928, SUC_lexicon_tags_37 },
    { 0x73e8d92a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c8d92d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec8d930, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b88d933, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6408d939, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc28d93c, SUC_lexicon_tags_12 },
    { 0xa668d93c, SUC_lexicon_tags_12 },
    { 0xa1a8d93e, SUC_lexicon_tags_8 },
    { 0x72e8d93f, SUC_lexicon_tags_11 },
    { 0x1128d940, SUC_lexicon_tags_12 },
    { 0xc1a8d93c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xee08d943, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8d94a, SUC_lexicon_tags_15 },
    { 0x8288d94a, SUC_lexicon_tags_17 },
    { 0x2be8d94c, SUC_lexicon_tags_19 },
    { 0x87e8d94c, SUC_lexicon_tags_60 },
    { 0xa928d94d, SUC_lexicon_tags_30 },
    { 0x9228d94f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf008d951, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57a8d953, SUC_lexicon_tags_1 },
    { 0xac48d953, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd268d956, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6508d959, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa888d960, SUC_lexicon_tags_21 },
    { 0x4048d961, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe88d964, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8748d967, SUC_lexicon_tags_25 },
    { 0xad28d968, SUC_lexicon_tags_37 },
    { 0xc8e8d969, SUC_lexicon_tags_12 },
    { 0x6bc8d96a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6c8d96c, SUC_lexicon_tags_23 },
    { 0xa288d96d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xcfa8d96f, SUC_lexicon_tags_29 },
    { 0x22e8d970, SUC_lexicon_tags_19 },
    { 0x8188d970, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61e8d973, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a48d978, SUC_lexicon_tags_31 },
    { 0xf88d978, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a8d97f, SUC_lexicon_tags_6 },
    { 0x48e8d980, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c8d983, SUC_lexicon_tags_17 },
    { 0x2308d984, SUC_lexicon_tags_55 },
    { 0x6d28d985, SUC_lexicon_tags_10 },
    { 0x96a8d985, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x948d988, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d98b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6748d991, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xed08d994, SUC_lexicon_tags_14 },
    { 0x5e48d995, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7008d999, SUC_lexicon_tags_30 },
    { 0xcba8d999, SUC_lexicon_tags_15 },
    { 0x8708d999, SUC_lexicon_tags_83 },
    { 0x7088d99c, SUC_lexicon_tags_37 },
    { 0x44c8d999, SUC_lexicon_tags_37 },
    { 0xdb68d99e, SUC_lexicon_tags_19 },
    { 0x32e8d99e, SUC_lexicon_tags_68 },
    { 0x1a8d99f, SUC_lexicon_tags_3 },
    { 0x5f48d9a1, SUC_lexicon_tags_11 },
    { 0xada8d9a1, SUC_lexicon_tags_0 },
    { 0x19a8d9a3, SUC_lexicon_tags_6 },
    { 0x5e48d9a4, SUC_lexicon_tags_45 },
    { 0x1208d9a4, SUC_lexicon_tags_45 },
    { 0xba88d9a4, SUC_lexicon_tags_48 },
    { 0x9868d9a6, SUC_lexicon_tags_92 },
    { 0x29a8d9a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcca8d9aa, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x3408d9ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaba8d9af, SUC_lexicon_tags_8 },
    { 0x7408d9b0, SUC_lexicon_tags_37 },
    { 0xab88d9b0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd908d9b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d88d9b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd508d9bb, SUC_lexicon_tags_1 },
    { 0x748d9bc, SUC_lexicon_tags_9 },
    { 0x1088d9bb, SUC_lexicon_tags_12 },
    { 0x1908d9bb, SUC_lexicon_tags_12 },
    { 0x328d9bf, SUC_lexicon_tags_11 },
    { 0xa328d9c0, SUC_lexicon_tags_2 },
    { 0x2c88d9c1, SUC_lexicon_tags_9 },
    { 0x6988d9bd, SUC_lexicon_tags_15 },
    { 0xe608d9c2, SUC_lexicon_tags_43 },
    { 0x7528d9bb, SUC_lexicon_tags_11 },
    { 0x4fa8d9bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa068d9c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7108d9cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe88d9d0, SUC_lexicon_tags_0 },
    { 0xc528d9d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e28d9d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e8d9d5, SUC_lexicon_tags_19 },
    { 0xca48d9d6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e48d9d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2128d9de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba8d9e6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa6a8d9e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x988d9ea, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x6428d9ec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa8d9f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ea8d9f3, SUC_lexicon_tags_65 },
    { 0xdd08d9f4, SUC_lexicon_tags_23 },
    { 0x91c8d9f3, SUC_lexicon_tags_17 },
    { 0x1aa8d9f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6ac8d9f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc9c8d9fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8da01, SUC_lexicon_tags_12 },
    { 0x2ec8da02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ba8da04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf508da06, SUC_lexicon_tags_38 },
    { 0x3da8da06, SUC_lexicon_tags_46 },
    { 0xaa8da08, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x11a8da0a, SUC_lexicon_tags_28 },
    { 0x3548da0b, SUC_lexicon_tags_19 },
    { 0x8848da0c, SUC_lexicon_tags_12 },
    { 0x1028da0a, SUC_lexicon_tags_19 },
    { 0x72e8da0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e8da12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa548da18, SUC_lexicon_tags_45 },
    { 0xb6c8da19, SUC_lexicon_tags_25 },
    { 0x6a28da1a, SUC_lexicon_tags_6 },
    { 0xd168da18, SUC_lexicon_tags_25 },
    { 0xede8da1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4228da1e, SUC_lexicon_tags_37 },
    { 0xe088da1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c8da22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6a8da24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa408da27, SUC_lexicon_tags_70 },
    { 0xef08da28, SUC_lexicon_tags_7 },
    { 0xee48da29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4d48da2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6928da2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc48da30, SUC_lexicon_tags_1 },
    { 0xb8c8da31, SUC_lexicon_tags_15 },
    { 0x86a8da32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa88da3b, SUC_lexicon_tags_7 },
    { 0xa8da3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8da41, SUC_lexicon_tags_27 },
    { 0x4ee8da41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a8da47, SUC_lexicon_tags_17 },
    { 0x7b08da48, SUC_lexicon_tags_0 },
    { 0x7e28da49, SUC_lexicon_tags_49 },
    { 0x44a8da4a, SUC_lexicon_tags_65 },
    { 0xef68da4b, SUC_lexicon_tags_21 },
    { 0x4388da4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8e08da4e, SUC_lexicon_tags_1 },
    { 0x3fa8da4f, SUC_lexicon_tags_32 },
    { 0x79c8da50, SUC_lexicon_tags_24 },
    { 0x4ec8da4f, SUC_lexicon_tags_1 },
    { 0xc68da4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf748da55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a8da58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x78a8da5b, SUC_lexicon_tags_11 },
    { 0xcfc8da5c, SUC_lexicon_tags_49 },
    { 0x7948da5b, SUC_lexicon_tags_43 },
    { 0x8ba8da5e, SUC_lexicon_tags_15 },
    { 0xcae8da5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c28da66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c8da6e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb68da75, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc788da77, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e8da7b, SUC_lexicon_tags_26 },
    { 0x21c8da7b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a8da7f, SUC_lexicon_tags_1 },
    { 0x3a8da80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb608da86, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a68da88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc8da8c, SUC_lexicon_tags_37 },
    { 0x72e8da8d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a8da95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2908da99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7da8da9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd928da9d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1a68da9f, SUC_lexicon_tags_10 },
    { 0xbae8da9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe328daa2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37a8daa4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1628daa6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x828daa8, SUC_lexicon_tags_126 },
    { 0x7128daa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89e8daab, SUC_lexicon_tags_11 },
    { 0x2948daac, SUC_lexicon_tags_49 },
    { 0xf988daad, SUC_lexicon_tags_48 },
    { 0x1928daae, SUC_lexicon_tags_65 },
    { 0x4268daad, SUC_lexicon_tags_23 },
    { 0xa328daae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a08dab7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc28daba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea8dabd, SUC_lexicon_tags_43 },
    { 0x9e48dabe, SUC_lexicon_tags_7 },
    { 0x37e8dabe, SUC_lexicon_tags_39 },
    { 0x5a68dabf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb988dac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25e8dac7, SUC_lexicon_tags_1 },
    { 0xc648dac8, SUC_lexicon_tags_49 },
    { 0x5008dac9, SUC_lexicon_tags_12 },
    { 0x54a8daca, SUC_lexicon_tags_57 },
    { 0xf128dacb, SUC_lexicon_tags_37 },
    { 0xeac8dac9, SUC_lexicon_tags_15 },
    { 0xd728dacd, SUC_lexicon_tags_9 },
    { 0x9168dace, SUC_lexicon_tags_12 },
    { 0xb728dacd, SUC_lexicon_tags_37 },
    { 0xa3a8dacf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e8dad4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5228dad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8dadb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x23a8dadd, SUC_lexicon_tags_47 },
    { 0x1fe8dade, SUC_lexicon_tags_118 },
    { 0xe668dadf, SUC_lexicon_tags_15 },
    { 0x7ce8dadd, SUC_lexicon_tags_17 },
    { 0x7cc8dae1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5c8dae3, SUC_lexicon_tags_12 },
    { 0xca68dae3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbfc8dae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c08daf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe08daf3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6928daf6, SUC_lexicon_tags_13 },
    { 0xd888daf7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3968dafb, SUC_lexicon_tags_11 },
    { 0x1fe8dafc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b28dafe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15c8db00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedc8db02, SUC_lexicon_tags_25 },
    { 0x6a48db02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c8db0a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2188db0e, SUC_lexicon_tags_0 },
    { 0xad88db0f, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8db17, SUC_lexicon_tags_11 },
    { 0xb3c8db18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8568db1e, SUC_lexicon_tags_32 },
    { 0x9fc8db1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace8db27, SUC_lexicon_tags_21 },
    { 0x4da8db27, SUC_lexicon_tags_9 },
    { 0xf268db29, SUC_lexicon_tags_22 },
    { 0x9348db2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec08db34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe548db39, SUC_lexicon_tags_37 },
    { 0x1768db3a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1448db3e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388db43, SUC_lexicon_tags_7 },
    { 0x85a8db43, SUC_lexicon_tags_9 },
    { 0x8c88db45, SUC_lexicon_tags_38 },
    { 0x20a8db46, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e8db49, SUC_lexicon_tags_25 },
    { 0x6da8db4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc768db4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68db54, SUC_lexicon_tags_11 },
    { 0x68a8db54, SUC_lexicon_tags_26 },
    { 0x2348db56, SUC_lexicon_tags_30 },
    { 0xd9a8db57, SUC_lexicon_tags_17 },
    { 0xb0a8db57, SUC_lexicon_tags_25 },
    { 0x1428db55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd048db5b, SUC_lexicon_tags_16 },
    { 0xf6a8db5c, SUC_lexicon_tags_498 },
    { 0, NULL },
    { 0xb348db5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e8db61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe248db63, SUC_lexicon_tags_37 },
    { 0xae08db64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93c8db66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22c8db68, SUC_lexicon_tags_46 },
    { 0x5b48db68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad48db6b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6488db70, SUC_lexicon_tags_15 },
    { 0x8ec8db70, SUC_lexicon_tags_11 },
    { 0x2c88db72, SUC_lexicon_tags_9 },
    { 0x21c8db73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7908db76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c28db78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e48db7c, SUC_lexicon_tags_0 },
    { 0xfca8db7d, SUC_lexicon_tags_30 },
    { 0x6f88db7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe248db82, SUC_lexicon_tags_43 },
    { 0x64a8db83, SUC_lexicon_tags_45 },
    { 0xd0c8db82, SUC_lexicon_tags_16 },
    { 0x60e8db85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5728db87, SUC_lexicon_tags_1 },
    { 0xb0e8db88, SUC_lexicon_tags_17 },
    { 0x3588db89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2228db8f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c8db95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6308db9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79a8db9d, SUC_lexicon_tags_1 },
    { 0xdf48db9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa488dba1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a8dba7, SUC_lexicon_tags_62 },
    { 0x6388dba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c8dbac, SUC_lexicon_tags_2 },
    { 0xdee8dbad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf268dbbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb88dbbf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e8dbc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3c8dbc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1988dbca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57e8dbcc, SUC_lexicon_tags_9 },
    { 0x17a8dbcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1108dbd0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2308dbd2, SUC_lexicon_tags_19 },
    { 0xbbc8dbd3, SUC_lexicon_tags_60 },
    { 0xf08dbd2, SUC_lexicon_tags_68 },
    { 0xac48dbd5, SUC_lexicon_tags_15 },
    { 0x63a8dbd5, SUC_lexicon_tags_0 },
    { 0xe88dbd7, SUC_lexicon_tags_37 },
    { 0x7688dbd7, SUC_lexicon_tags_60 },
    { 0x6568dbd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2488dbdc, SUC_lexicon_tags_37 },
    { 0xe868dbdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba8dbe0, SUC_lexicon_tags_37 },
    { 0x5288dbe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc88dbe7, SUC_lexicon_tags_43 },
    { 0xe208dbe8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b08dbeb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3aa8dbed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3228dbf3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a8dbf6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf6e8dbf8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb8e8dbfa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b68dc00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08dc03, SUC_lexicon_tags_63 },
    { 0x61a8dc03, SUC_lexicon_tags_17 },
    { 0x8ee8dc05, SUC_lexicon_tags_63 },
    { 0xfac8dc05, SUC_lexicon_tags_25 },
    { 0xcdc8dc04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b68dc09, SUC_lexicon_tags_14 },
    { 0x2748dc0a, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a8dc0d, SUC_lexicon_tags_15 },
    { 0xb3e8dc0d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9748dc14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd428dc18, SUC_lexicon_tags_37 },
    { 0x96e8dc19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb408dc1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcea8dc1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d88dc21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79a8dc23, SUC_lexicon_tags_25 },
    { 0x8548dc24, SUC_lexicon_tags_37 },
    { 0x7e48dc23, SUC_lexicon_tags_23 },
    { 0x8a48dc26, SUC_lexicon_tags_47 },
    { 0xc5e8dc26, SUC_lexicon_tags_46 },
    { 0x57a8dc28, SUC_lexicon_tags_17 },
    { 0x75c8dc29, SUC_lexicon_tags_47 },
    { 0x39a8dc2a, SUC_lexicon_tags_87 },
    { 0x71c8dc2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac8dc2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4308dc32, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6088dc34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e8dc3e, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb928dc42, SUC_lexicon_tags_37 },
    { 0x2ca8dc43, SUC_lexicon_tags_15 },
    { 0x8008dc43, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x8868dc46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc68dc49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8dc54, SUC_lexicon_tags_138 },
    { 0xf228dc55, SUC_lexicon_tags_17 },
    { 0x3128dc55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc788dc58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8dc5c, SUC_lexicon_tags_1 },
    { 0x6988dc5d, SUC_lexicon_tags_32 },
    { 0x9008dc5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6228dc60, SUC_lexicon_tags_21 },
    { 0xbbe8dc61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c8dc64, SUC_lexicon_tags_63 },
    { 0xf4a8dc65, SUC_lexicon_tags_1 },
    { 0xd628dc66, SUC_lexicon_tags_15 },
    { 0xf8e8dc67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8868dc6d, SUC_lexicon_tags_14 },
    { 0xf5a8dc6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4168dc72, SUC_lexicon_tags_65 },
    { 0x2008dc73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e8dc79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b88dc7b, SUC_lexicon_tags_26 },
    { 0xe368dc7b, SUC_lexicon_tags_38 },
    { 0xc108dc7b, SUC_lexicon_tags_1 },
    { 0x3fe8dc7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a8dc82, SUC_lexicon_tags_1 },
    { 0x2aa8dc82, SUC_lexicon_tags_23 },
    { 0x3a08dc84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7248dc89, SUC_lexicon_tags_5 },
    { 0x48a8dc89, SUC_lexicon_tags_27 },
    { 0xce28dc8b, SUC_lexicon_tags_14 },
    { 0x5f48dc8a, SUC_lexicon_tags_17 },
    { 0xb168dc8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba8dc93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c8dc98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7028dc9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e68dc9e, SUC_lexicon_tags_15 },
    { 0x6508dc9f, SUC_lexicon_tags_1 },
    { 0xd128dca0, SUC_lexicon_tags_27 },
    { 0x1ae8dca1, SUC_lexicon_tags_37 },
    { 0x7208dca2, SUC_lexicon_tags_47 },
    { 0xc048dca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76a8dca5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad28dca8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a8dcb3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68dcb7, SUC_lexicon_tags_15 },
    { 0xd3a8dcb8, SUC_lexicon_tags_63 },
    { 0x4008dcb8, SUC_lexicon_tags_37 },
    { 0x5b88dcba, SUC_lexicon_tags_37 },
    { 0xa2e8dcba, SUC_lexicon_tags_14 },
    { 0xfe08dcbc, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0x8188dcbf, SUC_lexicon_tags_14 },
    { 0x3488dcbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf08dcc3, SUC_lexicon_tags_2 },
    { 0x6168dcc4, SUC_lexicon_tags_77 },
    { 0x79e8dcc4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dccb, SUC_lexicon_tags_17 },
    { 0xbea8dccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e8dcd3, SUC_lexicon_tags_85 },
    { 0xf448dcd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9e8dcd6, SUC_lexicon_tags_17 },
    { 0xeee8dcd6, SUC_lexicon_tags_0 },
    { 0xf108dcd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5888dcdc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5728dce1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc888dce3, SUC_lexicon_tags_17 },
    { 0x7b88dce3, SUC_lexicon_tags_7 },
    { 0xfac8dce4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e8dce8, SUC_lexicon_tags_82 },
    { 0xc808dce8, SUC_lexicon_tags_15 },
    { 0x5c88dcea, SUC_lexicon_tags_19 },
    { 0x85c8dce8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6a28dced, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e08dcf0, SUC_lexicon_tags_12 },
    { 0x1588dcf1, SUC_lexicon_tags_21 },
    { 0x48c8dcf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6568dcf5, SUC_lexicon_tags_9 },
    { 0xb3a8dcf6, SUC_lexicon_tags_14 },
    { 0xfda8dcf6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa388dcfa, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e8dcff, SUC_lexicon_tags_11 },
    { 0x9408dcff, SUC_lexicon_tags_1 },
    { 0xf28dd01, SUC_lexicon_tags_7 },
    { 0x92c8dd01, SUC_lexicon_tags_37 },
    { 0xfa08dd02, SUC_lexicon_tags_1621 },
    { 0x4308dd04, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6528dd06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab28dd0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc8dd0e, SUC_lexicon_tags_11 },
    { 0x4ac8dd0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6768dd11, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x1e28dd13, SUC_lexicon_tags_46 },
    { 0xf328dd14, SUC_lexicon_tags_84 },
    { 0x9868dd13, SUC_lexicon_tags_45 },
    { 0x3588dd16, SUC_lexicon_tags_15 },
    { 0xdaa8dd17, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d88dd1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3528dd1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d88dd1f, SUC_lexicon_tags_2 },
    { 0xd988dd1f, SUC_lexicon_tags_16 },
    { 0x7de8dd1f, SUC_lexicon_tags_1 },
    { 0xaa08dd22, SUC_lexicon_tags_32 },
    { 0x3c48dd21, SUC_lexicon_tags_27 },
    { 0x3f28dd24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a88dd26, SUC_lexicon_tags_16 },
    { 0xc008dd27, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x9c08dd29, SUC_lexicon_tags_2 },
    { 0xd588dd2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa88dd2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf188dd2f, SUC_lexicon_tags_15 },
    { 0x2e48dd2f, SUC_lexicon_tags_63 },
    { 0x2dc8dd30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe428dd35, SUC_lexicon_tags_2 },
    { 0x1b08dd36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c8dd39, SUC_lexicon_tags_28 },
    { 0x1928dd39, SUC_lexicon_tags_37 },
    { 0xac08dd3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9448dd3d, SUC_lexicon_tags_15 },
    { 0x6c68dd3e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf68dd40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8dd43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf3a8dd45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc8dd4b, SUC_lexicon_tags_19 },
    { 0x1f08dd4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba8dd54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc408dd5e, SUC_lexicon_tags_27 },
    { 0xa7c8dd5f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc888dd66, SUC_lexicon_tags_60 },
    { 0x2528dd67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa508dd6a, SUC_lexicon_tags_77 },
    { 0x3f48dd6a, SUC_lexicon_tags_15 },
    { 0xd328dd6c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbc48dd6e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc8dd71, SUC_lexicon_tags_25 },
    { 0x15e8dd71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe8dd77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b68dd7c, SUC_lexicon_tags_14 },
    { 0xf788dd7d, SUC_lexicon_tags_48 },
    { 0xaec8dd7e, SUC_lexicon_tags_6 },
    { 0x27c8dd7f, SUC_lexicon_tags_46 },
    { 0x3728dd80, SUC_lexicon_tags_60 },
    { 0x4e28dd81, SUC_lexicon_tags_25 },
    { 0xa8a8dd82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd768dd8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe8dd8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11e8dd91, SUC_lexicon_tags_23 },
    { 0xf9e8dd92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd1a8dd94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7728dd96, SUC_lexicon_tags_87 },
    { 0xb528dd97, SUC_lexicon_tags_26 },
    { 0x26c8dd96, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe648dd9a, SUC_lexicon_tags_0 },
    { 0x98a8dd9b, SUC_lexicon_tags_7 },
    { 0x72a8dd9b, SUC_lexicon_tags_37 },
    { 0xdc48dd9d, SUC_lexicon_tags_69 },
    { 0x15e8dd9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e8dda1, SUC_lexicon_tags_21 },
    { 0x86a8dda2, SUC_lexicon_tags_60 },
    { 0xf448dda1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec8dda7, SUC_lexicon_tags_0 },
    { 0xb468dda8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68ddad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a68ddaf, SUC_lexicon_tags_57 },
    { 0x8b88ddb0, SUC_lexicon_tags_49 },
    { 0xada8ddb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc8ddb4, SUC_lexicon_tags_31 },
    { 0xba08ddb5, SUC_lexicon_tags_4 },
    { 0x1aa8ddb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc8ddb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde8ddc1, SUC_lexicon_tags_351 },
    { 0x7dc8ddc1, SUC_lexicon_tags_15 },
    { 0xa508ddc3, SUC_lexicon_tags_48 },
    { 0xb028ddc4, SUC_lexicon_tags_17 },
    { 0x1b28ddc5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6708ddc7, SUC_lexicon_tags_0 },
    { 0xc148ddc7, SUC_lexicon_tags_29 },
    { 0x7208ddc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x368ddcb, SUC_lexicon_tags_7 },
    { 0x4ca8ddcc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xba48ddcf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c8ddd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e08ddd6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x16a8ddd8, SUC_lexicon_tags_63 },
    { 0x9a48ddd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec8dddc, SUC_lexicon_tags_35 },
    { 0x12c8dddd, SUC_lexicon_tags_17 },
    { 0x828ddde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb228dde0, SUC_lexicon_tags_37 },
    { 0x1de8dde1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa108ddec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8288ddf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb08ddf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e08ddfb, SUC_lexicon_tags_11 },
    { 0xb208ddfb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4148de03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a88de09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308de0d, SUC_lexicon_tags_8 },
    { 0x1988de0d, SUC_lexicon_tags_28 },
    { 0x5f08de0f, SUC_lexicon_tags_0 },
    { 0xb7a8de10, SUC_lexicon_tags_29 },
    { 0xd248de0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a8de16, SUC_lexicon_tags_113 },
    { 0x5668de16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f48de1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84e8de21, SUC_lexicon_tags_37 },
    { 0xe5a8de22, SUC_lexicon_tags_23 },
    { 0x1b48de21, SUC_lexicon_tags_70 },
    { 0x55e8de24, SUC_lexicon_tags_37 },
    { 0xcd28de25, SUC_lexicon_tags_15 },
    { 0x1a28de25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54e8de28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf908de2b, SUC_lexicon_tags_16 },
    { 0x8548de2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b28de30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfea8de32, SUC_lexicon_tags_38 },
    { 0x9e8de33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa8de36, SUC_lexicon_tags_11 },
    { 0x8b88de37, SUC_lexicon_tags_10 },
    { 0xcaa8de38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2908de3a, SUC_lexicon_tags_25 },
    { 0x9f88de3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d88de3f, SUC_lexicon_tags_70 },
    { 0xfa48de40, SUC_lexicon_tags_16 },
    { 0x4468de41, SUC_lexicon_tags_43 },
    { 0xd288de42, SUC_lexicon_tags_37 },
    { 0xd048de42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe268de46, SUC_lexicon_tags_35 },
    { 0x3388de47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x68a8de49, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae8de4c, SUC_lexicon_tags_37 },
    { 0xe688de4d, SUC_lexicon_tags_45 },
    { 0xca68de4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4588de50, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2128de53, SUC_lexicon_tags_1671 },
    { 0x3448de54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd968de58, SUC_lexicon_tags_19 },
    { 0xae48de59, SUC_lexicon_tags_23 },
    { 0x5828de59, SUC_lexicon_tags_7 },
    { 0xe528de59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2468de5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe348de64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4628de6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc048de6f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108de76, SUC_lexicon_tags_47 },
    { 0x8ba8de77, SUC_lexicon_tags_12 },
    { 0x6b88de78, SUC_lexicon_tags_32 },
    { 0x9768de79, SUC_lexicon_tags_12 },
    { 0xb2a8de77, SUC_lexicon_tags_11 },
    { 0x64a8de7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c8de7e, SUC_lexicon_tags_60 },
    { 0xbea8de7f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xada8de82, SUC_lexicon_tags_25 },
    { 0xb4a8de83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6c8de85, SUC_lexicon_tags_37 },
    { 0x8d28de85, SUC_lexicon_tags_7 },
    { 0x5de8de87, SUC_lexicon_tags_37 },
    { 0xc0e8de86, SUC_lexicon_tags_25 },
    { 0xe768de86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3848de8b, SUC_lexicon_tags_2 },
    { 0x6188de8c, SUC_lexicon_tags_92 },
    { 0xd328de8d, SUC_lexicon_tags_45 },
    { 0x4508de8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc308de91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ec8de93, SUC_lexicon_tags_65 },
    { 0x2848de94, SUC_lexicon_tags_10 },
    { 0x67e8de94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f48de98, SUC_lexicon_tags_25 },
    { 0xd008de98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe208de9d, SUC_lexicon_tags_11 },
    { 0xfd68de9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb48dea1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e48dea7, SUC_lexicon_tags_29 },
    { 0xf3a8dea8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be8deaf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b08deb3, SUC_lexicon_tags_19 },
    { 0x6fe8deb4, SUC_lexicon_tags_27 },
    { 0xf28deb4, SUC_lexicon_tags_27 },
    { 0xd248deb6, SUC_lexicon_tags_3 },
    { 0xcf08deb6, SUC_lexicon_tags_60 },
    { 0x1f88deb6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x6c88deba, SUC_lexicon_tags_25 },
    { 0x9568debb, SUC_lexicon_tags_14 },
    { 0xa748deba, SUC_lexicon_tags_26 },
    { 0x4608debd, SUC_lexicon_tags_11 },
    { 0x7ae8debe, SUC_lexicon_tags_7 },
    { 0x7ee8debf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede8dec3, SUC_lexicon_tags_11 },
    { 0x1888dec4, SUC_lexicon_tags_0 },
    { 0x94a8dec4, SUC_lexicon_tags_15 },
    { 0x5f68dec6, SUC_lexicon_tags_43 },
    { 0xbc8dec4, SUC_lexicon_tags_43 },
    { 0x40c8dec7, SUC_lexicon_tags_351 },
    { 0xd528dec3, SUC_lexicon_tags_0 },
    { 0x4288dec8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8788ded0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee8ded3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa168ded8, SUC_lexicon_tags_2 },
    { 0x4fa8ded8, SUC_lexicon_tags_12 },
    { 0x2248ded9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x34c8dedc, SUC_lexicon_tags_1 },
    { 0xeca8dedd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb88dedf, SUC_lexicon_tags_81 },
    { 0x8468dee0, SUC_lexicon_tags_1 },
    { 0xece8dee0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21a8dee3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf728deea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8168deef, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xa568def1, SUC_lexicon_tags_6 },
    { 0x44e8def1, SUC_lexicon_tags_23 },
    { 0xe388def3, SUC_lexicon_tags_5 },
    { 0x1348def4, SUC_lexicon_tags_4 },
    { 0x4728def5, SUC_lexicon_tags_28 },
    { 0xe1c8def4, SUC_lexicon_tags_35 },
    { 0x3148def6, SUC_lexicon_tags_17 },
    { 0xf7c8def8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaee8defa, SUC_lexicon_tags_11 },
    { 0x7c8defb, SUC_lexicon_tags_19 },
    { 0x1688defc, SUC_lexicon_tags_7 },
    { 0xe128defd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5ce8deff, SUC_lexicon_tags_38 },
    { 0xce88df00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5188df02, SUC_lexicon_tags_38 },
    { 0x5ce8df03, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8ca8df05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8288df07, SUC_lexicon_tags_21 },
    { 0xdf08df07, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x67a8df0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc088df0e, SUC_lexicon_tags_79 },
    { 0xcd68df0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc8df14, SUC_lexicon_tags_12 },
    { 0xd548df15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6268df17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc88df19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x65a8df1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd28df1f, SUC_lexicon_tags_44 },
    { 0xeaa8df20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a8df29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a8df2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f48df35, SUC_lexicon_tags_45 },
    { 0x1948df36, SUC_lexicon_tags_31 },
    { 0x508df37, SUC_lexicon_tags_17 },
    { 0xc828df36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ce8df3a, SUC_lexicon_tags_17 },
    { 0xc28df3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf048df3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd708df43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdac8df45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b68df4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe8c8df4c, SUC_lexicon_tags_47 },
    { 0x44a8df4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9988df50, SUC_lexicon_tags_2 },
    { 0x49c8df51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1148df55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe8df59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3148df5f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb608df61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92c8df63, SUC_lexicon_tags_166 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c8df68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2988df6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9908df6d, SUC_lexicon_tags_79 },
    { 0x1e28df6e, SUC_lexicon_tags_60 },
    { 0xe5e8df6f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a8df74, SUC_lexicon_tags_15 },
    { 0x5ec8df74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfb48df77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8df7b, SUC_lexicon_tags_2 },
    { 0xa088df7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9728df7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec08df81, SUC_lexicon_tags_6 },
    { 0x8328df81, SUC_lexicon_tags_11 },
    { 0xcdc8df81, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf528df89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e88df8c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc8df91, SUC_lexicon_tags_12 },
    { 0xb08df92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28df95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe8df98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5188df9c, SUC_lexicon_tags_25 },
    { 0x8f68df9c, SUC_lexicon_tags_8 },
    { 0xaa48df9e, SUC_lexicon_tags_9 },
    { 0x8008df9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f68dfa2, SUC_lexicon_tags_39 },
    { 0x4068dfa3, SUC_lexicon_tags_22 },
    { 0x4c68dfa2, SUC_lexicon_tags_37 },
    { 0xbea8dfa5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce8dfab, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec8dfaf, SUC_lexicon_tags_31 },
    { 0xb028dfb0, SUC_lexicon_tags_7 },
    { 0xaa48dfb1, SUC_lexicon_tags_17 },
    { 0x1c8dfb0, SUC_lexicon_tags_43 },
    { 0xca88dfb0, SUC_lexicon_tags_25 },
    { 0xf1e8dfb2, SUC_lexicon_tags_17 },
    { 0xeac8dfb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba8dfba, SUC_lexicon_tags_91 },
    { 0x48e8dfbb, SUC_lexicon_tags_2 },
    { 0x3a28dfba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a88dfbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad28dfc1, SUC_lexicon_tags_60 },
    { 0x8b88dfc1, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0xb408dfc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5208dfc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd928dfc9, SUC_lexicon_tags_23 },
    { 0x3aa8dfc9, SUC_lexicon_tags_15 },
    { 0x2068dfcb, SUC_lexicon_tags_11 },
    { 0x2b08dfca, SUC_lexicon_tags_4 },
    { 0xed68dfcd, SUC_lexicon_tags_34 },
    { 0x308dfcd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd748dfd0, SUC_lexicon_tags_26 },
    { 0x5be8dfd1, SUC_lexicon_tags_25 },
    { 0xeac8dfd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc8dfd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a88dfd8, SUC_lexicon_tags_55 },
    { 0x7ce8dfd9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7908dfdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce48dfe1, SUC_lexicon_tags_13 },
    { 0xe728dfe2, SUC_lexicon_tags_1 },
    { 0xa268dfe2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5448dfe6, SUC_lexicon_tags_25 },
    { 0x5968dfe6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce8dfea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a88dfee, SUC_lexicon_tags_40 },
    { 0xd9a8dfef, SUC_lexicon_tags_65 },
    { 0xfc48dff0, SUC_lexicon_tags_4 },
    { 0x6f08dff1, SUC_lexicon_tags_49 },
    { 0x9ac8dff1, SUC_lexicon_tags_33 },
    { 0xbae8dff3, SUC_lexicon_tags_17 },
    { 0x908dff3, SUC_lexicon_tags_47 },
    { 0x708dff4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5668e000, SUC_lexicon_tags_25 },
    { 0xf568e000, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1548e005, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e015, SUC_lexicon_tags_45 },
    { 0xeda8e016, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5728e018, SUC_lexicon_tags_44 },
    { 0x6368e019, SUC_lexicon_tags_10 },
    { 0x81a8e01a, SUC_lexicon_tags_27 },
    { 0xa6a8e018, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d48e01f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe408e023, SUC_lexicon_tags_66 },
    { 0x8a28e024, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9708e028, SUC_lexicon_tags_0 },
    { 0x4fe8e029, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c8e02c, SUC_lexicon_tags_17 },
    { 0x2788e02d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b08e030, SUC_lexicon_tags_31 },
    { 0xc08e031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd788e035, SUC_lexicon_tags_11 },
    { 0x86c8e036, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xeda8e038, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc8e03b, SUC_lexicon_tags_9 },
    { 0xd668e03c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb28e03e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a88e044, SUC_lexicon_tags_43 },
    { 0xdb48e044, SUC_lexicon_tags_39 },
    { 0x6308e046, SUC_lexicon_tags_151 },
    { 0x8f08e044, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd88e04b, SUC_lexicon_tags_23 },
    { 0xb908e04c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8908e053, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x528e056, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3dc8e058, SUC_lexicon_tags_25 },
    { 0xaae8e059, SUC_lexicon_tags_0 },
    { 0x5fc8e05a, SUC_lexicon_tags_17 },
    { 0xb188e05b, SUC_lexicon_tags_12 },
    { 0x3488e059, SUC_lexicon_tags_17 },
    { 0x1a68e05d, SUC_lexicon_tags_37 },
    { 0x1a8e05d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a8e062, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x608e065, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c68e06a, SUC_lexicon_tags_63 },
    { 0x6a88e06b, SUC_lexicon_tags_7 },
    { 0x11a8e06a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7508e06e, SUC_lexicon_tags_82 },
    { 0x17c8e06f, SUC_lexicon_tags_15 },
    { 0x7248e070, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2048e076, SUC_lexicon_tags_15 },
    { 0xffc8e077, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e8e07d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8e082, SUC_lexicon_tags_55 },
    { 0xaf68e083, SUC_lexicon_tags_0 },
    { 0xefe8e084, SUC_lexicon_tags_17 },
    { 0x1168e083, SUC_lexicon_tags_26 },
    { 0x9508e086, SUC_lexicon_tags_26 },
    { 0x7ae8e087, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c8e08a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca8e08d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x14c8e08f, SUC_lexicon_tags_37 },
    { 0x12a8e090, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x23c8e092, SUC_lexicon_tags_0 },
    { 0xc4a8e092, SUC_lexicon_tags_14 },
    { 0x5628e094, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc48e09a, SUC_lexicon_tags_1 },
    { 0xf448e09a, SUC_lexicon_tags_25 },
    { 0xa888e09a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4928e09e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568e0ab, SUC_lexicon_tags_11 },
    { 0xb768e0ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe8e0bc, SUC_lexicon_tags_52 },
    { 0xbe08e0bc, SUC_lexicon_tags_106 },
    { 0x46e8e0be, SUC_lexicon_tags_63 },
    { 0xe5a8e0bf, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc8e0c4, SUC_lexicon_tags_45 },
    { 0xaec8e0c5, SUC_lexicon_tags_1 },
    { 0x208e0c6, SUC_lexicon_tags_11 },
    { 0x5fc8e0c7, SUC_lexicon_tags_12 },
    { 0x6cc8e0c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc908e0cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3848e0d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2c8e0da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf508e0dd, SUC_lexicon_tags_25 },
    { 0x5368e0dd, SUC_lexicon_tags_46 },
    { 0xdde8e0df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe208e0eb, SUC_lexicon_tags_29 },
    { 0xcbe8e0ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7488e0ee, SUC_lexicon_tags_34 },
    { 0xdc08e0ef, SUC_lexicon_tags_15 },
    { 0x2b48e0ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb748e0f2, SUC_lexicon_tags_37 },
    { 0x328e0f2, SUC_lexicon_tags_25 },
    { 0xa568e0f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7708e0f6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x19c8e0f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee8e0fb, SUC_lexicon_tags_37 },
    { 0x99e8e0fb, SUC_lexicon_tags_37 },
    { 0x6b28e0fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9628e100, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c8e104, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e28e106, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae8e10a, SUC_lexicon_tags_11 },
    { 0xcd08e10a, SUC_lexicon_tags_31 },
    { 0xee28e10a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf508e10e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8e111, SUC_lexicon_tags_56 },
    { 0xf828e112, SUC_lexicon_tags_25 },
    { 0xa988e112, SUC_lexicon_tags_55 },
    { 0xf7e8e114, SUC_lexicon_tags_25 },
    { 0xec28e115, SUC_lexicon_tags_6 },
    { 0x4c08e112, SUC_lexicon_tags_0 },
    { 0x9068e111, SUC_lexicon_tags_7 },
    { 0xf0a8e118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc48e125, SUC_lexicon_tags_33 },
    { 0xa28e126, SUC_lexicon_tags_68 },
    { 0x9b48e127, SUC_lexicon_tags_15 },
    { 0x9b08e127, SUC_lexicon_tags_43 },
    { 0xcfa8e129, SUC_lexicon_tags_29 },
    { 0xba68e12a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc8e12e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8788e132, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c8e135, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a8e137, SUC_lexicon_tags_6 },
    { 0xe5c8e137, SUC_lexicon_tags_10 },
    { 0x7748e139, SUC_lexicon_tags_25 },
    { 0x54a8e13a, SUC_lexicon_tags_38 },
    { 0xcd88e13b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e28e13e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a88e142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c28e145, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7848e147, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa8e14a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc488e14f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce8e152, SUC_lexicon_tags_11 },
    { 0x79a8e153, SUC_lexicon_tags_1 },
    { 0x5a08e154, SUC_lexicon_tags_47 },
    { 0xb248e152, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4588e163, SUC_lexicon_tags_8 },
    { 0xcba8e164, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c8e169, SUC_lexicon_tags_38 },
    { 0x4b68e16a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x48a8e16d, SUC_lexicon_tags_11 },
    { 0x84a8e16d, SUC_lexicon_tags_28 },
    { 0x90c8e16d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a8e174, SUC_lexicon_tags_11 },
    { 0x2ee8e174, SUC_lexicon_tags_25 },
    { 0x2908e176, SUC_lexicon_tags_39 },
    { 0x4ea8e177, SUC_lexicon_tags_13 },
    { 0x7c48e177, SUC_lexicon_tags_33 },
    { 0x2548e179, SUC_lexicon_tags_81 },
    { 0x90a8e178, SUC_lexicon_tags_77 },
    { 0xb5a8e17b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd08e17e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe428e180, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8aa8e182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5708e187, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde8e18a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3008e18f, SUC_lexicon_tags_37 },
    { 0xbde8e18f, SUC_lexicon_tags_49 },
    { 0x68c8e18f, SUC_lexicon_tags_37 },
    { 0xc248e192, SUC_lexicon_tags_47 },
    { 0x7288e193, SUC_lexicon_tags_37 },
    { 0xc3c8e193, SUC_lexicon_tags_7 },
    { 0xdc8e194, SUC_lexicon_tags_15 },
    { 0x7208e196, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c8e19c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed88e1ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e8e1b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb748e1b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb68e1b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4088e1bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeda8e1bd, SUC_lexicon_tags_65 },
    { 0x4268e1be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1188e1c1, SUC_lexicon_tags_27 },
    { 0xf2a8e1c1, SUC_lexicon_tags_46 },
    { 0x708e1c3, SUC_lexicon_tags_2 },
    { 0x4f08e1c4, SUC_lexicon_tags_29 },
    { 0x4ce8e1c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd368e1c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c8e1ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d68e1d2, SUC_lexicon_tags_9 },
    { 0x4e48e1d2, SUC_lexicon_tags_25 },
    { 0x5bc8e1d3, SUC_lexicon_tags_12 },
    { 0x6328e1d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e28e1d9, SUC_lexicon_tags_84 },
    { 0xff48e1da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4e48e1dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a8e1e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a8e1e3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7c48e1e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a88e1e9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a8e1f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb08e1f3, SUC_lexicon_tags_28 },
    { 0x2328e1f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c8e1f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca8e1fb, SUC_lexicon_tags_19 },
    { 0xc888e1fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8388e204, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a8e207, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb268e20c, SUC_lexicon_tags_55 },
    { 0xc8a8e20d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa568e211, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2be8e213, SUC_lexicon_tags_25 },
    { 0x16e8e213, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7808e217, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2bc8e219, SUC_lexicon_tags_27 },
    { 0x8be8e219, SUC_lexicon_tags_65 },
    { 0x7ae8e21b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3168e21f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5468e221, SUC_lexicon_tags_37 },
    { 0x9928e221, SUC_lexicon_tags_10 },
    { 0x9508e223, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea8e229, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1908e22f, SUC_lexicon_tags_28 },
    { 0xdbe8e230, SUC_lexicon_tags_43 },
    { 0x4ea8e231, SUC_lexicon_tags_15 },
    { 0xd5a8e232, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3008e237, SUC_lexicon_tags_12 },
    { 0xf288e238, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9de8e23a, SUC_lexicon_tags_53 },
    { 0x4a68e23b, SUC_lexicon_tags_12 },
    { 0x1f68e23b, SUC_lexicon_tags_15 },
    { 0x7d28e23d, SUC_lexicon_tags_0 },
    { 0x9148e23e, SUC_lexicon_tags_11 },
    { 0x5d48e23f, SUC_lexicon_tags_60 },
    { 0x7568e240, SUC_lexicon_tags_23 },
    { 0xc0e8e241, SUC_lexicon_tags_45 },
    { 0xd548e23f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c8e245, SUC_lexicon_tags_9 },
    { 0x8be8e246, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a8e24c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1928e253, SUC_lexicon_tags_27 },
    { 0xe3a8e253, SUC_lexicon_tags_28 },
    { 0xa348e255, SUC_lexicon_tags_66 },
    { 0x4dc8e255, SUC_lexicon_tags_33 },
    { 0xd7e8e254, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7448e25b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc8e265, SUC_lexicon_tags_7 },
    { 0x6448e266, SUC_lexicon_tags_7 },
    { 0x928e266, SUC_lexicon_tags_11 },
    { 0x4408e267, SUC_lexicon_tags_19 },
    { 0x568e269, SUC_lexicon_tags_82 },
    { 0x7ac8e265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6348e26c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe8e270, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa9a8e272, SUC_lexicon_tags_129 },
    { 0x64e8e273, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7788e276, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x85e8e278, SUC_lexicon_tags_27 },
    { 0x6628e279, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b08e27c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e8e27f, SUC_lexicon_tags_2 },
    { 0xaf28e27f, SUC_lexicon_tags_15 },
    { 0x2968e280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5108e283, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f88e28f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2508e296, SUC_lexicon_tags_2 },
    { 0xc68e296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a8e29e, SUC_lexicon_tags_1 },
    { 0x23a8e29f, SUC_lexicon_tags_32 },
    { 0xc168e29f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5888e2a2, SUC_lexicon_tags_2 },
    { 0xe8a8e2a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f48e2a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf568e2a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c48e2aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2c8e2ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xba08e2af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfb08e2b1, SUC_lexicon_tags_79 },
    { 0x2d88e2b2, SUC_lexicon_tags_34 },
    { 0x288e2b3, SUC_lexicon_tags_31 },
    { 0x3308e2b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5248e2b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb928e2b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6108e2bc, SUC_lexicon_tags_37 },
    { 0x92c8e2bd, SUC_lexicon_tags_2 },
    { 0xb668e2be, SUC_lexicon_tags_25 },
    { 0x50c8e2bf, SUC_lexicon_tags_15 },
    { 0xc7a8e2bf, SUC_lexicon_tags_37 },
    { 0x1968e2be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe8e2cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7e8e2ce, SUC_lexicon_tags_2 },
    { 0x1ec8e2cf, SUC_lexicon_tags_11 },
    { 0xff88e2d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08e2d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe8e2d9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d28e2dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2cc8e2df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1508e2e1, SUC_lexicon_tags_49 },
    { 0x6b88e2e2, SUC_lexicon_tags_29 },
    { 0x5c68e2e3, SUC_lexicon_tags_37 },
    { 0x22a8e2e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2368e2e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd88e2ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a8e2ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb948e2f1, SUC_lexicon_tags_34 },
    { 0xd508e2f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a88e2f5, SUC_lexicon_tags_16 },
    { 0x2dc8e2f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe048e2f8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1408e300, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcbe8e302, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7568e306, SUC_lexicon_tags_15 },
    { 0x6e68e306, SUC_lexicon_tags_15 },
    { 0x1228e308, SUC_lexicon_tags_14 },
    { 0x66c8e308, SUC_lexicon_tags_25 },
    { 0xda8e30a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc08e30c, SUC_lexicon_tags_11 },
    { 0x3888e30d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2128e30f, SUC_lexicon_tags_0 },
    { 0x90c8e310, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc568e312, SUC_lexicon_tags_21 },
    { 0xc68e313, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9868e315, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1228e319, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc8e31c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe228e31f, SUC_lexicon_tags_9 },
    { 0xbca8e320, SUC_lexicon_tags_7 },
    { 0x6e8e321, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa8e326, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe888e329, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8908e32b, SUC_lexicon_tags_12 },
    { 0x55c8e32c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8488e32f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a48e335, SUC_lexicon_tags_37 },
    { 0x2b88e335, SUC_lexicon_tags_2 },
    { 0x7928e336, SUC_lexicon_tags_17 },
    { 0xb028e338, SUC_lexicon_tags_37 },
    { 0xaca8e336, SUC_lexicon_tags_15 },
    { 0xb028e33a, SUC_lexicon_tags_1 },
    { 0xb1c8e33b, SUC_lexicon_tags_15 },
    { 0xbb08e33b, SUC_lexicon_tags_23 },
    { 0x5148e33d, SUC_lexicon_tags_17 },
    { 0x84a8e33e, SUC_lexicon_tags_45 },
    { 0x7de8e33f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa248e343, SUC_lexicon_tags_15 },
    { 0x7bc8e344, SUC_lexicon_tags_25 },
    { 0x46c8e345, SUC_lexicon_tags_37 },
    { 0x1fe8e346, SUC_lexicon_tags_12 },
    { 0x2e88e347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa188e34b, SUC_lexicon_tags_12 },
    { 0xdea8e34c, SUC_lexicon_tags_63 },
    { 0x1708e34c, SUC_lexicon_tags_31 },
    { 0x6088e34d, SUC_lexicon_tags_45 },
    { 0x85e8e34b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc8e355, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc88e35a, SUC_lexicon_tags_46 },
    { 0x3468e35b, SUC_lexicon_tags_38 },
    { 0x288e35c, SUC_lexicon_tags_49 },
    { 0x8148e35d, SUC_lexicon_tags_6 },
    { 0xd988e35e, SUC_lexicon_tags_14 },
    { 0xb6a8e35e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e8e368, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4408e36d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf628e36f, SUC_lexicon_tags_42 },
    { 0x8328e36f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa368e373, SUC_lexicon_tags_31 },
    { 0xcc88e374, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d68e378, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec8e37b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608e38a, SUC_lexicon_tags_17 },
    { 0xdd28e38b, SUC_lexicon_tags_45 },
    { 0x9cc8e38c, SUC_lexicon_tags_45 },
    { 0xba48e38d, SUC_lexicon_tags_15 },
    { 0xfb68e38c, SUC_lexicon_tags_19 },
    { 0x4f48e38f, SUC_lexicon_tags_42 },
    { 0x7d88e390, SUC_lexicon_tags_42 },
    { 0x6368e391, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf808e393, SUC_lexicon_tags_212 },
    { 0xef88e394, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc8e397, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd788e399, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9348e3a1, SUC_lexicon_tags_31 },
    { 0x37e8e3a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8e3a5, SUC_lexicon_tags_12 },
    { 0xd928e3a5, SUC_lexicon_tags_31 },
    { 0x1588e3a5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c8e3ab, SUC_lexicon_tags_84 },
    { 0x5ec8e3ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f28e3ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1188e3b0, SUC_lexicon_tags_37 },
    { 0x54a8e3b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd988e3b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8828e3b9, SUC_lexicon_tags_11 },
    { 0x2768e3ba, SUC_lexicon_tags_17 },
    { 0x9d08e3bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf228e3c0, SUC_lexicon_tags_1 },
    { 0xd6c8e3c1, SUC_lexicon_tags_19 },
    { 0x3c8e3c2, SUC_lexicon_tags_47 },
    { 0x7728e3c3, SUC_lexicon_tags_11 },
    { 0x8468e3c4, SUC_lexicon_tags_1 },
    { 0xb288e3c5, SUC_lexicon_tags_12 },
    { 0xf668e3c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d28e3c8, SUC_lexicon_tags_1 },
    { 0x96e8e3c9, SUC_lexicon_tags_23 },
    { 0x68c8e3ca, SUC_lexicon_tags_8 },
    { 0x2ce8e3ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf408e3cd, SUC_lexicon_tags_17 },
    { 0xca88e3ce, SUC_lexicon_tags_37 },
    { 0x68a8e3cf, SUC_lexicon_tags_7 },
    { 0x7788e3d0, SUC_lexicon_tags_32 },
    { 0x7b88e3d0, SUC_lexicon_tags_2 },
    { 0xb088e3d0, SUC_lexicon_tags_28 },
    { 0x56c8e3d3, SUC_lexicon_tags_25 },
    { 0xc488e3ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a8e3d7, SUC_lexicon_tags_30 },
    { 0xb2a8e3d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1108e3da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e8e3e2, SUC_lexicon_tags_77 },
    { 0xcc68e3e3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdce8e3e5, SUC_lexicon_tags_116 },
    { 0x4c8e3e5, SUC_lexicon_tags_0 },
    { 0x3588e3e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf048e3e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x47a8e3eb, SUC_lexicon_tags_14 },
    { 0xd2c8e3ec, SUC_lexicon_tags_37 },
    { 0xcac8e3eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67e8e3ef, SUC_lexicon_tags_27 },
    { 0x7f68e3f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf368e3f2, SUC_lexicon_tags_7 },
    { 0xb888e3f3, SUC_lexicon_tags_2 },
    { 0xb4c8e3f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x368e3f6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x82e8e3f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2828e3fa, SUC_lexicon_tags_60 },
    { 0x5ac8e3fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd788e3fd, SUC_lexicon_tags_12 },
    { 0x7908e3fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d68e40b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4c48e40d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x74e8e40f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca8e413, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d08e417, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf408e41c, SUC_lexicon_tags_31 },
    { 0x4088e41c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5268e41f, SUC_lexicon_tags_15 },
    { 0x6e08e420, SUC_lexicon_tags_23 },
    { 0xad28e421, SUC_lexicon_tags_7 },
    { 0xc2e8e422, SUC_lexicon_tags_19 },
    { 0x7ee8e420, SUC_lexicon_tags_17 },
    { 0xfe08e420, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa648e428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92a8e42a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e8e42e, SUC_lexicon_tags_11 },
    { 0x9d28e42f, SUC_lexicon_tags_108 },
    { 0x87c8e42f, SUC_lexicon_tags_105 },
    { 0xb6a8e42f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc8e434, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa8e438, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc368e43c, SUC_lexicon_tags_17 },
    { 0x3408e43d, SUC_lexicon_tags_7 },
    { 0xa3c8e43e, SUC_lexicon_tags_3 },
    { 0xb768e43e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xae08e441, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x948e444, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1828e447, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7848e44c, SUC_lexicon_tags_17 },
    { 0x9168e44c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a8e453, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f68e456, SUC_lexicon_tags_25 },
    { 0xa088e457, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a68e460, SUC_lexicon_tags_15 },
    { 0x7f08e461, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x2028e463, SUC_lexicon_tags_56 },
    { 0xb6e8e464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa648e466, SUC_lexicon_tags_25 },
    { 0x6a8e466, SUC_lexicon_tags_11 },
    { 0x1c88e468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb988e46c, SUC_lexicon_tags_8 },
    { 0xe5a8e46d, SUC_lexicon_tags_12 },
    { 0xed88e46e, SUC_lexicon_tags_23 },
    { 0x1668e46f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6bc8e471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e8e477, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c8e47a, SUC_lexicon_tags_8 },
    { 0x6448e47b, SUC_lexicon_tags_43 },
    { 0x64a8e47c, SUC_lexicon_tags_11 },
    { 0xad48e47d, SUC_lexicon_tags_14 },
    { 0xc6e8e47e, SUC_lexicon_tags_17 },
    { 0x4e88e47b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe328e487, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x71e8e489, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08e48c, SUC_lexicon_tags_0 },
    { 0x5788e48d, SUC_lexicon_tags_1 },
    { 0x17e8e48e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a8e492, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc708e494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fa8e496, SUC_lexicon_tags_14 },
    { 0x7c48e497, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4228e49d, SUC_lexicon_tags_37 },
    { 0xfd48e49e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a8e4a1, SUC_lexicon_tags_27 },
    { 0xf708e4a2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac8e4ab, SUC_lexicon_tags_52 },
    { 0x7f28e4ac, SUC_lexicon_tags_28 },
    { 0xd968e4ac, SUC_lexicon_tags_37 },
    { 0x57a8e4ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed08e4b1, SUC_lexicon_tags_45 },
    { 0x5a48e4b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5928e4b6, SUC_lexicon_tags_6 },
    { 0x4b88e4b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3468e4b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c08e4c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a8e4c5, SUC_lexicon_tags_21 },
    { 0x6d48e4c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed48e4c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6408e4d1, SUC_lexicon_tags_15 },
    { 0x89e8e4d1, SUC_lexicon_tags_28 },
    { 0x32e8e4d3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8e4d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbf88e4d8, SUC_lexicon_tags_11 },
    { 0xa208e4d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9808e4dd, SUC_lexicon_tags_23 },
    { 0xd528e4de, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c68e4e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4128e4e5, SUC_lexicon_tags_12 },
    { 0xa7e8e4e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d88e4e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52c8e4eb, SUC_lexicon_tags_11 },
    { 0xbbc8e4ec, SUC_lexicon_tags_87 },
    { 0xb68e4ed, SUC_lexicon_tags_29 },
    { 0x5128e4ee, SUC_lexicon_tags_0 },
    { 0xdd68e4ef, SUC_lexicon_tags_183 },
    { 0xd508e4f0, SUC_lexicon_tags_17 },
    { 0x8248e4f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a8e4f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe48e4fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc8e4ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4488e503, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe448e505, SUC_lexicon_tags_25 },
    { 0xcc28e505, SUC_lexicon_tags_11 },
    { 0xfce8e507, SUC_lexicon_tags_49 },
    { 0x7a28e507, SUC_lexicon_tags_136 },
    { 0xd888e507, SUC_lexicon_tags_25 },
    { 0xa188e507, SUC_lexicon_tags_12 },
    { 0xa48e50b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63e8e50d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c08e510, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd148e512, SUC_lexicon_tags_31 },
    { 0x8c08e512, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f68e51a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba68e51f, SUC_lexicon_tags_30 },
    { 0x6288e520, SUC_lexicon_tags_1 },
    { 0xe068e51f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3228e524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb08e527, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8808e52a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec8e52f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a8e538, SUC_lexicon_tags_17 },
    { 0xcda8e539, SUC_lexicon_tags_2 },
    { 0x67c8e539, SUC_lexicon_tags_33 },
    { 0x2ec8e53b, SUC_lexicon_tags_7 },
    { 0x5f88e53c, SUC_lexicon_tags_837 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e540, SUC_lexicon_tags_10 },
    { 0x5528e540, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d68e546, SUC_lexicon_tags_14 },
    { 0xd768e547, SUC_lexicon_tags_54 },
    { 0x9528e546, SUC_lexicon_tags_30 },
    { 0xa4e8e546, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc8e54d, SUC_lexicon_tags_25 },
    { 0x53c8e54e, SUC_lexicon_tags_15 },
    { 0xc268e54e, SUC_lexicon_tags_12 },
    { 0x6b88e550, SUC_lexicon_tags_15 },
    { 0x4b48e551, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xac68e554, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca8e557, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbaa8e559, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e8e55c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e8e565, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a8e56a, SUC_lexicon_tags_49 },
    { 0x2f68e56b, SUC_lexicon_tags_0 },
    { 0xc808e56c, SUC_lexicon_tags_26 },
    { 0x5c48e56c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8188e56f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc68e573, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf48e576, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7888e579, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe548e57f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a8e582, SUC_lexicon_tags_69 },
    { 0xc9c8e583, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e8e586, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2a8e588, SUC_lexicon_tags_15 },
    { 0xd988e588, SUC_lexicon_tags_12 },
    { 0x78a8e588, SUC_lexicon_tags_17 },
    { 0xac88e58b, SUC_lexicon_tags_85 },
    { 0x49e8e58b, SUC_lexicon_tags_26 },
    { 0x28e58c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7528e590, SUC_lexicon_tags_127 },
    { 0xa68e591, SUC_lexicon_tags_37 },
    { 0x62a8e592, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a8e595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b68e598, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e8e59c, SUC_lexicon_tags_25 },
    { 0xcee8e59c, SUC_lexicon_tags_17 },
    { 0xa8a8e59c, SUC_lexicon_tags_25 },
    { 0x2868e59f, SUC_lexicon_tags_65 },
    { 0x64a8e59e, SUC_lexicon_tags_1 },
    { 0x8128e59e, SUC_lexicon_tags_15 },
    { 0x2308e5a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1a8e5a4, SUC_lexicon_tags_27 },
    { 0x2188e5a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeee8e5a7, SUC_lexicon_tags_12 },
    { 0xa68e5a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ca8e5aa, SUC_lexicon_tags_61 },
    { 0x1b28e5ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbbe8e5ad, SUC_lexicon_tags_79 },
    { 0xfec8e5ae, SUC_lexicon_tags_37 },
    { 0xf808e5ae, SUC_lexicon_tags_12 },
    { 0x4c28e5b0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5488e5b6, SUC_lexicon_tags_15 },
    { 0x2748e5b7, SUC_lexicon_tags_1 },
    { 0xa008e5b8, SUC_lexicon_tags_9 },
    { 0x4e28e5b8, SUC_lexicon_tags_17 },
    { 0xf928e5ba, SUC_lexicon_tags_77 },
    { 0x89c8e5bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c48e5be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce8e5c2, SUC_lexicon_tags_30 },
    { 0x428e5c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd608e5c7, SUC_lexicon_tags_45 },
    { 0xbd28e5c8, SUC_lexicon_tags_5 },
    { 0x4088e5c9, SUC_lexicon_tags_65 },
    { 0x1468e5c9, SUC_lexicon_tags_27 },
    { 0xf388e5c9, SUC_lexicon_tags_60 },
    { 0xba8e5cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7228e5ce, SUC_lexicon_tags_22 },
    { 0xf0c8e5cf, SUC_lexicon_tags_2 },
    { 0x5fc8e5d0, SUC_lexicon_tags_37 },
    { 0xdd88e5d1, SUC_lexicon_tags_7 },
    { 0xca88e5d2, SUC_lexicon_tags_17 },
    { 0x6308e5d3, SUC_lexicon_tags_25 },
    { 0x9588e5d4, SUC_lexicon_tags_12 },
    { 0xfa28e5d5, SUC_lexicon_tags_15 },
    { 0xbca8e5d5, SUC_lexicon_tags_27 },
    { 0xd688e5d1, SUC_lexicon_tags_15 },
    { 0xf508e5d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d88e5db, SUC_lexicon_tags_17 },
    { 0x8348e5db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac8e5df, SUC_lexicon_tags_25 },
    { 0xe9a8e5e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c8e5e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a48e5eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a28e5ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a8e5f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x50e8e5f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c88e5fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6568e5fd, SUC_lexicon_tags_15 },
    { 0x4c88e5fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe488e601, SUC_lexicon_tags_17 },
    { 0x5c28e602, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a68e607, SUC_lexicon_tags_25 },
    { 0x23c8e608, SUC_lexicon_tags_7 },
    { 0x5a08e608, SUC_lexicon_tags_14 },
    { 0x6768e607, SUC_lexicon_tags_15 },
    { 0x4a08e60b, SUC_lexicon_tags_2 },
    { 0x2748e609, SUC_lexicon_tags_7 },
    { 0xce68e60a, SUC_lexicon_tags_12 },
    { 0x55c8e60e, SUC_lexicon_tags_28 },
    { 0x6d88e60f, SUC_lexicon_tags_17 },
    { 0x8048e610, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9088e616, SUC_lexicon_tags_43 },
    { 0xb808e617, SUC_lexicon_tags_37 },
    { 0xf788e618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb388e61c, SUC_lexicon_tags_11 },
    { 0x2ee8e61c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ce8e61f, SUC_lexicon_tags_11 },
    { 0xd568e61f, SUC_lexicon_tags_38 },
    { 0xdf08e621, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c8e625, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xeca8e628, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf108e62d, SUC_lexicon_tags_11 },
    { 0x4fc8e62d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe648e630, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e08e633, SUC_lexicon_tags_60 },
    { 0x5cc8e633, SUC_lexicon_tags_8 },
    { 0x7a8e635, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e8e63b, SUC_lexicon_tags_0 },
    { 0x8708e63b, SUC_lexicon_tags_38 },
    { 0xeb48e63d, SUC_lexicon_tags_57 },
    { 0xc568e63e, SUC_lexicon_tags_8 },
    { 0xaf28e63f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc8e643, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bc8e645, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148e64b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc728e64f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc28e652, SUC_lexicon_tags_27 },
    { 0xdd08e652, SUC_lexicon_tags_4 },
    { 0x8188e654, SUC_lexicon_tags_47 },
    { 0x7b08e653, SUC_lexicon_tags_65 },
    { 0xf008e656, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a8e659, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c8e65d, SUC_lexicon_tags_17 },
    { 0x4188e65e, SUC_lexicon_tags_5 },
    { 0x12c8e65d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x248e661, SUC_lexicon_tags_4 },
    { 0xe408e661, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe168e667, SUC_lexicon_tags_11 },
    { 0xcf08e668, SUC_lexicon_tags_19 },
    { 0x60c8e667, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa48e66b, SUC_lexicon_tags_6 },
    { 0x5d08e66c, SUC_lexicon_tags_45 },
    { 0x2288e66b, SUC_lexicon_tags_37 },
    { 0xa328e66e, SUC_lexicon_tags_33 },
    { 0x37a8e66f, SUC_lexicon_tags_8 },
    { 0xce28e66f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf0a8e672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac08e678, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e08e67a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda8e67d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c08e67f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe68e683, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2928e689, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad08e698, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xebc8e69a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7be8e69c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8188e69f, SUC_lexicon_tags_106 },
    { 0x9a48e6a0, SUC_lexicon_tags_33 },
    { 0xff88e69f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf288e6a4, SUC_lexicon_tags_27 },
    { 0x7248e6a5, SUC_lexicon_tags_23 },
    { 0x8988e6a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a8e6aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb168e6b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaba8e6ba, SUC_lexicon_tags_17 },
    { 0x96a8e6bb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf068e6be, SUC_lexicon_tags_25 },
    { 0x4628e6bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1c88e6c1, SUC_lexicon_tags_463 },
    { 0, NULL },
    { 0xca88e6c3, SUC_lexicon_tags_37 },
    { 0x4148e6c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8e88e6c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a28e6ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9be8e6d0, SUC_lexicon_tags_7 },
    { 0xfde8e6d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c8e6d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8528e6d7, SUC_lexicon_tags_250 },
    { 0x56e8e6d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28e6db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b88e6dd, SUC_lexicon_tags_6 },
    { 0xd9a8e6dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe288e6ea, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa088e6f2, SUC_lexicon_tags_2 },
    { 0x62e8e6f3, SUC_lexicon_tags_25 },
    { 0x1388e6f2, SUC_lexicon_tags_62 },
    { 0x1868e6f2, SUC_lexicon_tags_17 },
    { 0x3e48e6f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08e6fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce8e6fd, SUC_lexicon_tags_15 },
    { 0x8688e6fe, SUC_lexicon_tags_55 },
    { 0x6c68e6fd, SUC_lexicon_tags_14 },
    { 0x7e48e700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c8e703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb48e707, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9928e70f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a8e714, SUC_lexicon_tags_17 },
    { 0xcca8e715, SUC_lexicon_tags_62 },
    { 0xef28e716, SUC_lexicon_tags_2 },
    { 0x3548e714, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7168e719, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e8e71d, SUC_lexicon_tags_49 },
    { 0xd528e71d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x83a8e721, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56a8e723, SUC_lexicon_tags_25 },
    { 0x23c8e724, SUC_lexicon_tags_17 },
    { 0x96c8e725, SUC_lexicon_tags_2 },
    { 0x7628e726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca8e72a, SUC_lexicon_tags_32 },
    { 0x2be8e72b, SUC_lexicon_tags_11 },
    { 0x8648e72c, SUC_lexicon_tags_1 },
    { 0x6dc8e72d, SUC_lexicon_tags_25 },
    { 0xe3e8e72c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a8e731, SUC_lexicon_tags_15 },
    { 0xb868e731, SUC_lexicon_tags_25 },
    { 0xca28e733, SUC_lexicon_tags_37 },
    { 0x5ea8e734, SUC_lexicon_tags_23 },
    { 0x1d48e733, SUC_lexicon_tags_29 },
    { 0xc028e734, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d48e739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa8e73c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xff48e73f, SUC_lexicon_tags_27 },
    { 0xbbc8e73f, SUC_lexicon_tags_24 },
    { 0x8428e73f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe8e751, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x44e8e753, SUC_lexicon_tags_25 },
    { 0x1808e754, SUC_lexicon_tags_39 },
    { 0xe848e755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45a8e757, SUC_lexicon_tags_49 },
    { 0xa848e757, SUC_lexicon_tags_11 },
    { 0x57c8e758, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e08e75e, SUC_lexicon_tags_25 },
    { 0x7f68e75f, SUC_lexicon_tags_37 },
    { 0x8328e75e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6468e763, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6548e767, SUC_lexicon_tags_7 },
    { 0x808e767, SUC_lexicon_tags_25 },
    { 0xf428e768, SUC_lexicon_tags_28 },
    { 0x5228e76a, SUC_lexicon_tags_7 },
    { 0xc7a8e76a, SUC_lexicon_tags_12 },
    { 0x2188e76c, SUC_lexicon_tags_28 },
    { 0xe0e8e76d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb08e76f, SUC_lexicon_tags_6 },
    { 0x78e8e76f, SUC_lexicon_tags_27 },
    { 0x5828e76f, SUC_lexicon_tags_84 },
    { 0xbae8e770, SUC_lexicon_tags_37 },
    { 0x3908e772, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a8e777, SUC_lexicon_tags_25 },
    { 0xfe8e778, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1768e77b, SUC_lexicon_tags_29 },
    { 0x8788e77c, SUC_lexicon_tags_25 },
    { 0xd7a8e77d, SUC_lexicon_tags_7 },
    { 0x8ca8e77d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a28e782, SUC_lexicon_tags_57 },
    { 0xcbe8e783, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf488e78a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4928e78e, SUC_lexicon_tags_66 },
    { 0xbf08e78e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a8e794, SUC_lexicon_tags_69 },
    { 0x60a8e795, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0xefa8e797, SUC_lexicon_tags_11 },
    { 0x2348e797, SUC_lexicon_tags_42 },
    { 0x2548e799, SUC_lexicon_tags_1 },
    { 0x9248e798, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4068e79c, SUC_lexicon_tags_7 },
    { 0xbae8e79d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x188e7a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d08e7a4, SUC_lexicon_tags_1 },
    { 0x1d68e7a4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x72e8e7a7, SUC_lexicon_tags_1 },
    { 0xb428e7a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd008e7ab, SUC_lexicon_tags_51 },
    { 0x2348e7ab, SUC_lexicon_tags_49 },
    { 0xc688e7ab, SUC_lexicon_tags_56 },
    { 0x6788e7ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2f68e7b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b48e7b3, SUC_lexicon_tags_7 },
    { 0xa788e7b4, SUC_lexicon_tags_37 },
    { 0x49a8e7b5, SUC_lexicon_tags_12 },
    { 0xd388e7b3, SUC_lexicon_tags_16 },
    { 0x7968e7b5, SUC_lexicon_tags_27 },
    { 0x1548e7b8, SUC_lexicon_tags_30 },
    { 0x6448e7b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb288e7bd, SUC_lexicon_tags_11 },
    { 0xff68e7be, SUC_lexicon_tags_26 },
    { 0x45e8e7bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc8e7c2, SUC_lexicon_tags_11 },
    { 0x6e68e7c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8588e7c5, SUC_lexicon_tags_30 },
    { 0x9d28e7c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc8e7cd, SUC_lexicon_tags_15 },
    { 0x1968e7cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e28e7d0, SUC_lexicon_tags_1 },
    { 0xfbe8e7d0, SUC_lexicon_tags_148 },
    { 0xa5a8e7d2, SUC_lexicon_tags_11 },
    { 0x8748e7d0, SUC_lexicon_tags_7 },
    { 0x2948e7d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c88e7db, SUC_lexicon_tags_42 },
    { 0xdea8e7dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaa88e7de, SUC_lexicon_tags_12 },
    { 0x8128e7df, SUC_lexicon_tags_25 },
    { 0x94a8e7df, SUC_lexicon_tags_11 },
    { 0x6648e7df, SUC_lexicon_tags_14 },
    { 0x88e8e7e2, SUC_lexicon_tags_7 },
    { 0x4808e7e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e8e7e8, SUC_lexicon_tags_17 },
    { 0xeee8e7e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf608e7ed, SUC_lexicon_tags_0 },
    { 0x6e28e7ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a08e7f2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1048e7f4, SUC_lexicon_tags_7 },
    { 0x7de8e7f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d68e7fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe288e7fd, SUC_lexicon_tags_27 },
    { 0x9ae8e7fe, SUC_lexicon_tags_43 },
    { 0xe6e8e7ff, SUC_lexicon_tags_38 },
    { 0xea48e7ff, SUC_lexicon_tags_38 },
    { 0x3a48e801, SUC_lexicon_tags_17 },
    { 0x76c8e802, SUC_lexicon_tags_45 },
    { 0x4328e803, SUC_lexicon_tags_1 },
    { 0x26a8e801, SUC_lexicon_tags_84 },
    { 0xf788e805, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8248e808, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe408e80a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8e811, SUC_lexicon_tags_15 },
    { 0x89c8e812, SUC_lexicon_tags_19 },
    { 0x93e8e813, SUC_lexicon_tags_7 },
    { 0x7788e814, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c88e817, SUC_lexicon_tags_27 },
    { 0x1c8e818, SUC_lexicon_tags_25 },
    { 0xf288e819, SUC_lexicon_tags_25 },
    { 0x8348e81a, SUC_lexicon_tags_12 },
    { 0xd4c8e819, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc68e822, SUC_lexicon_tags_17 },
    { 0xe1e8e822, SUC_lexicon_tags_60 },
    { 0x3b28e822, SUC_lexicon_tags_0 },
    { 0x2be8e823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e88e833, SUC_lexicon_tags_33 },
    { 0xa8a8e834, SUC_lexicon_tags_32 },
    { 0xdc68e835, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c8e83e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe788e842, SUC_lexicon_tags_35 },
    { 0x1a88e842, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e8e84c, SUC_lexicon_tags_60 },
    { 0x6d88e84d, SUC_lexicon_tags_25 },
    { 0xa7c8e84e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5648e850, SUC_lexicon_tags_13 },
    { 0xe4c8e851, SUC_lexicon_tags_28 },
    { 0x7168e851, SUC_lexicon_tags_6 },
    { 0xfe48e853, SUC_lexicon_tags_47 },
    { 0xbe08e851, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4148e856, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb088e858, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c8e85d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec88e861, SUC_lexicon_tags_17 },
    { 0x628e862, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3688e868, SUC_lexicon_tags_204 },
    { 0x16e8e869, SUC_lexicon_tags_87 },
    { 0x5ea8e869, SUC_lexicon_tags_38 },
    { 0x8008e86b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd308e86d, SUC_lexicon_tags_15 },
    { 0xcda8e86e, SUC_lexicon_tags_14 },
    { 0xe328e86d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x17c8e871, SUC_lexicon_tags_17 },
    { 0xb548e871, SUC_lexicon_tags_2 },
    { 0x58a8e873, SUC_lexicon_tags_23 },
    { 0xede8e873, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da8e880, SUC_lexicon_tags_31 },
    { 0x8ea8e881, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca8e888, SUC_lexicon_tags_47 },
    { 0x7a48e889, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c8e88c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2868e891, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c8e89c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe588e89e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5768e8a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5628e8a4, SUC_lexicon_tags_17 },
    { 0x5208e8a5, SUC_lexicon_tags_29 },
    { 0xb568e8a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7108e8a8, SUC_lexicon_tags_37 },
    { 0xacc8e8a8, SUC_lexicon_tags_12 },
    { 0xc828e8aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c28e8ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa848e8af, SUC_lexicon_tags_12 },
    { 0xce8e8b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1948e8b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688e8b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a88e8bc, SUC_lexicon_tags_11 },
    { 0x90a8e8bd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e8e8c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf08e8c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e8c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c8e8cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6848e8d1, SUC_lexicon_tags_29 },
    { 0x6708e8d2, SUC_lexicon_tags_1 },
    { 0xa088e8d2, SUC_lexicon_tags_31 },
    { 0xc428e8d3, SUC_lexicon_tags_33 },
    { 0x7fe8e8d5, SUC_lexicon_tags_37 },
    { 0x7c28e8d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c28e8d9, SUC_lexicon_tags_21 },
    { 0x5128e8da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7888e8de, SUC_lexicon_tags_38 },
    { 0x8e8e8de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c8e8e3, SUC_lexicon_tags_4 },
    { 0xaa68e8e3, SUC_lexicon_tags_16 },
    { 0xf7e8e8e5, SUC_lexicon_tags_60 },
    { 0x9728e8e4, SUC_lexicon_tags_17 },
    { 0xbb28e8e7, SUC_lexicon_tags_14 },
    { 0xd788e8e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa828e8ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3808e8ec, SUC_lexicon_tags_1 },
    { 0xf28e8ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86a8e8ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe28e8f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd6c8e8f3, SUC_lexicon_tags_32 },
    { 0x1e68e8f4, SUC_lexicon_tags_1 },
    { 0x7788e8f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8e8e8f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a8e8fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8328e8ff, SUC_lexicon_tags_49 },
    { 0x5928e900, SUC_lexicon_tags_65 },
    { 0x12a8e901, SUC_lexicon_tags_12 },
    { 0x168e901, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8e905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7a8e907, SUC_lexicon_tags_37 },
    { 0xc628e908, SUC_lexicon_tags_56 },
    { 0xff28e909, SUC_lexicon_tags_8 },
    { 0x55a8e908, SUC_lexicon_tags_3 },
    { 0x8e08e90b, SUC_lexicon_tags_12 },
    { 0x2c08e90c, SUC_lexicon_tags_92 },
    { 0x3968e908, SUC_lexicon_tags_25 },
    { 0xe2e8e90d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e8e913, SUC_lexicon_tags_39 },
    { 0xfa28e914, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x328e916, SUC_lexicon_tags_60 },
    { 0xf0c8e917, SUC_lexicon_tags_8 },
    { 0xaa48e917, SUC_lexicon_tags_2 },
    { 0x3e68e919, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd88e91b, SUC_lexicon_tags_0 },
    { 0x5588e91b, SUC_lexicon_tags_3 },
    { 0xfda8e91d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228e921, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe588e926, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf68e929, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe48e92c, SUC_lexicon_tags_25 },
    { 0x3be8e92c, SUC_lexicon_tags_16 },
    { 0xeda8e92e, SUC_lexicon_tags_9 },
    { 0xb48e92f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e8e934, SUC_lexicon_tags_14 },
    { 0xfe68e935, SUC_lexicon_tags_2 },
    { 0xe068e934, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a88e939, SUC_lexicon_tags_37 },
    { 0xea68e93a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdac8e93c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9168e93f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a48e942, SUC_lexicon_tags_7 },
    { 0xb848e942, SUC_lexicon_tags_15 },
    { 0x3488e944, SUC_lexicon_tags_11 },
    { 0x78e8e945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb68e952, SUC_lexicon_tags_11 },
    { 0xb188e953, SUC_lexicon_tags_65 },
    { 0x33a8e954, SUC_lexicon_tags_45 },
    { 0x59a8e954, SUC_lexicon_tags_19 },
    { 0x7028e956, SUC_lexicon_tags_42 },
    { 0x8988e956, SUC_lexicon_tags_12 },
    { 0x2a88e958, SUC_lexicon_tags_29 },
    { 0x19e8e956, SUC_lexicon_tags_25 },
    { 0x58c8e958, SUC_lexicon_tags_37 },
    { 0x1268e95b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7808e964, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3f68e966, SUC_lexicon_tags_11 },
    { 0x83c8e967, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8e96c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d28e971, SUC_lexicon_tags_2 },
    { 0xb3a8e972, SUC_lexicon_tags_15 },
    { 0xc308e973, SUC_lexicon_tags_26 },
    { 0x4a88e974, SUC_lexicon_tags_47 },
    { 0xe168e975, SUC_lexicon_tags_0 },
    { 0x1de8e975, SUC_lexicon_tags_19 },
    { 0x3f68e972, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8e982, SUC_lexicon_tags_37 },
    { 0xa8e983, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7448e985, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5a8e987, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc8e98a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7408e990, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe428e992, SUC_lexicon_tags_12 },
    { 0x4b08e993, SUC_lexicon_tags_105 },
    { 0x9d08e994, SUC_lexicon_tags_37 },
    { 0x5fa8e994, SUC_lexicon_tags_25 },
    { 0x8be8e994, SUC_lexicon_tags_2 },
    { 0xb108e997, SUC_lexicon_tags_29 },
    { 0x3ba8e992, SUC_lexicon_tags_38 },
    { 0x51e8e999, SUC_lexicon_tags_17 },
    { 0x9aa8e99a, SUC_lexicon_tags_8 },
    { 0x8588e996, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1588e99f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d48e9a1, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xb028e9a3, SUC_lexicon_tags_38 },
    { 0x2868e9a4, SUC_lexicon_tags_25 },
    { 0xa648e9a3, SUC_lexicon_tags_17 },
    { 0x91a8e9a6, SUC_lexicon_tags_19 },
    { 0x788e9a6, SUC_lexicon_tags_12 },
    { 0x85a8e9a8, SUC_lexicon_tags_26 },
    { 0x7668e9a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb768e9ab, SUC_lexicon_tags_2 },
    { 0x5628e9ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5c68e9ae, SUC_lexicon_tags_35 },
    { 0x6b28e9af, SUC_lexicon_tags_29 },
    { 0xb3a8e9b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8548e9b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2208e9c0, SUC_lexicon_tags_33 },
    { 0x3f68e9c1, SUC_lexicon_tags_15 },
    { 0xbb88e9c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb28e9c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b08e9c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7368e9d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc308e9d4, SUC_lexicon_tags_68 },
    { 0xa308e9d5, SUC_lexicon_tags_55 },
    { 0xbde8e9d6, SUC_lexicon_tags_33 },
    { 0xbdc8e9d5, SUC_lexicon_tags_87 },
    { 0x6388e9d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a8e9db, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe08e9df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a8e9e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa068e9e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ee8e9e9, SUC_lexicon_tags_4 },
    { 0xbd28e9e9, SUC_lexicon_tags_24 },
    { 0xa328e9ea, SUC_lexicon_tags_11 },
    { 0x47a8e9eb, SUC_lexicon_tags_11 },
    { 0x2308e9ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf088e9ef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a8e9f7, SUC_lexicon_tags_31 },
    { 0x6ae8e9f8, SUC_lexicon_tags_63 },
    { 0x80c8e9f9, SUC_lexicon_tags_31 },
    { 0xad88e9fa, SUC_lexicon_tags_49 },
    { 0x51a8e9fa, SUC_lexicon_tags_17 },
    { 0x8a28e9fb, SUC_lexicon_tags_7 },
    { 0xdbe8e9fd, SUC_lexicon_tags_1 },
    { 0xed28e9fe, SUC_lexicon_tags_65 },
    { 0x2e48e9fe, SUC_lexicon_tags_61 },
    { 0x2288e9fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8ea05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7a8ea07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f68ea0a, SUC_lexicon_tags_53 },
    { 0xc4c8ea0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2468ea0d, SUC_lexicon_tags_27 },
    { 0x13c8ea0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf868ea19, SUC_lexicon_tags_15 },
    { 0x1b28ea19, SUC_lexicon_tags_25 },
    { 0xb608ea1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e8ea21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1d68ea23, SUC_lexicon_tags_0 },
    { 0x1f88ea23, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a8ea29, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a08ea2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5388ea32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7988ea34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7c8ea36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc448ea43, SUC_lexicon_tags_27 },
    { 0xe788ea43, SUC_lexicon_tags_17 },
    { 0x29e8ea43, SUC_lexicon_tags_1 },
    { 0xf288ea43, SUC_lexicon_tags_17 },
    { 0x81a8ea44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4388ea49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c8ea4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7288ea4e, SUC_lexicon_tags_25 },
    { 0xa9e8ea4e, SUC_lexicon_tags_1 },
    { 0x7808ea50, SUC_lexicon_tags_19 },
    { 0x67a8ea50, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaac8ea54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc068ea56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c8ea5d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b68ea61, SUC_lexicon_tags_4 },
    { 0xeca8ea61, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2868ea68, SUC_lexicon_tags_25 },
    { 0xd4c8ea68, SUC_lexicon_tags_17 },
    { 0x83e8ea6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1668ea6e, SUC_lexicon_tags_30 },
    { 0xfd28ea6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4688ea73, SUC_lexicon_tags_32 },
    { 0x4208ea73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8648ea7c, SUC_lexicon_tags_37 },
    { 0x3748ea7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xefe8ea7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67e8ea81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2c8ea83, SUC_lexicon_tags_25 },
    { 0x368ea83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3a8ea86, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9748ea88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa68ea8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08ea96, SUC_lexicon_tags_3 },
    { 0xe968ea96, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9d08ea99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f88ea9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ac8ea9e, SUC_lexicon_tags_15 },
    { 0xf608ea9e, SUC_lexicon_tags_7 },
    { 0x7108eaa0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb948eaa6, SUC_lexicon_tags_25 },
    { 0xb788eaa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c8eaab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa88eaae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1568eab0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c8eab3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe588eab7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b48eabb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd028eabd, SUC_lexicon_tags_11 },
    { 0x4d68eabd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf88eac1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c8eac9, SUC_lexicon_tags_68 },
    { 0x65c8eaca, SUC_lexicon_tags_37 },
    { 0x71c8eacb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd528eacd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd608ead1, SUC_lexicon_tags_12 },
    { 0xd2c8ead2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84c8ead4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc08eada, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ec8eadc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1588eade, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6168eae2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd368eae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308eaec, SUC_lexicon_tags_2 },
    { 0xc348eaed, SUC_lexicon_tags_7 },
    { 0x6c48eaee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf648eaf0, SUC_lexicon_tags_11 },
    { 0x3f68eaf1, SUC_lexicon_tags_10 },
    { 0x9388eaf2, SUC_lexicon_tags_62 },
    { 0xff88eaf2, SUC_lexicon_tags_30 },
    { 0x5408eaf2, SUC_lexicon_tags_27 },
    { 0xb668eaf1, SUC_lexicon_tags_2 },
    { 0xdf68eaf2, SUC_lexicon_tags_27 },
    { 0x44c8eaf2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e8eafc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d68eaff, SUC_lexicon_tags_0 },
    { 0xd928eaff, SUC_lexicon_tags_0 },
    { 0xe728eaff, SUC_lexicon_tags_55 },
    { 0x3b48eb02, SUC_lexicon_tags_15 },
    { 0x7bc8eb03, SUC_lexicon_tags_65 },
    { 0xaa88eb04, SUC_lexicon_tags_1 },
    { 0x3ac8eb05, SUC_lexicon_tags_2 },
    { 0xc9e8eb05, SUC_lexicon_tags_11 },
    { 0x1be8eb03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a28eb09, SUC_lexicon_tags_11 },
    { 0x6068eb0a, SUC_lexicon_tags_25 },
    { 0x3868eb0a, SUC_lexicon_tags_37 },
    { 0x9fa8eb0b, SUC_lexicon_tags_1 },
    { 0xab88eb0a, SUC_lexicon_tags_25 },
    { 0xd788eb0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x148eb11, SUC_lexicon_tags_77 },
    { 0x788eb11, SUC_lexicon_tags_80 },
    { 0x2388eb13, SUC_lexicon_tags_25 },
    { 0x8428eb14, SUC_lexicon_tags_15 },
    { 0xcbe8eb12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8428eb17, SUC_lexicon_tags_7 },
    { 0x2888eb17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb88eb1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe588eb27, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de8eb32, SUC_lexicon_tags_49 },
    { 0xbae8eb33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf388eb35, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e08eb3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3688eb41, SUC_lexicon_tags_11 },
    { 0x6a28eb42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e8eb4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca8eb51, SUC_lexicon_tags_45 },
    { 0x7be8eb52, SUC_lexicon_tags_19 },
    { 0x1988eb53, SUC_lexicon_tags_17 },
    { 0xcd68eb54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae8eb58, SUC_lexicon_tags_10 },
    { 0xee88eb58, SUC_lexicon_tags_37 },
    { 0xdde8eb5a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc68eb5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e08eb62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9748eb67, SUC_lexicon_tags_11 },
    { 0xbc8eb67, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a48eb70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb48eb74, SUC_lexicon_tags_66 },
    { 0xf7c8eb75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfe28eb77, SUC_lexicon_tags_63 },
    { 0x2d28eb77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fe8eb7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5148eb7d, SUC_lexicon_tags_0 },
    { 0x5d48eb7e, SUC_lexicon_tags_9 },
    { 0x2fc8eb7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x468eb82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf08eb87, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe928eb91, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa488eb96, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1908eb9c, SUC_lexicon_tags_2 },
    { 0x2a48eb9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a8eba0, SUC_lexicon_tags_7 },
    { 0x75c8eba0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee8eba7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c28eba9, SUC_lexicon_tags_43 },
    { 0x8448eba9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7408ebaf, SUC_lexicon_tags_26 },
    { 0xeda8ebb0, SUC_lexicon_tags_7 },
    { 0xaa08ebb1, SUC_lexicon_tags_2 },
    { 0xc4c8ebb0, SUC_lexicon_tags_61 },
    { 0x9e68ebb3, SUC_lexicon_tags_5 },
    { 0xb1c8ebb0, SUC_lexicon_tags_37 },
    { 0xf548ebb0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa788ebbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa8ebc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c8ebc5, SUC_lexicon_tags_6 },
    { 0x9a8ebc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e08ebc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e8ebd5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c8ebd8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e8ebdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da8ebe1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d28ebe3, SUC_lexicon_tags_29 },
    { 0xdb08ebe4, SUC_lexicon_tags_57 },
    { 0x5f88ebe5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a48ebec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe4c8ebee, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xde48ebf0, SUC_lexicon_tags_1 },
    { 0xb268ebf1, SUC_lexicon_tags_37 },
    { 0x2568ebf2, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6728ebf6, SUC_lexicon_tags_17 },
    { 0x3608ebf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6308ebfe, SUC_lexicon_tags_12 },
    { 0x8f48ebff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a48ec01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd28ec04, SUC_lexicon_tags_4 },
    { 0x9da8ec05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4808ec08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15c8ec0a, SUC_lexicon_tags_14 },
    { 0x2f48ec0b, SUC_lexicon_tags_6 },
    { 0x9ac8ec0a, SUC_lexicon_tags_60 },
    { 0x4228ec0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7c8ec0f, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188ec14, SUC_lexicon_tags_11 },
    { 0xa4e8ec14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f68ec17, SUC_lexicon_tags_30 },
    { 0x1b08ec18, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1988ec1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8288ec26, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a28ec2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb188ec30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4408ec35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e8ec39, SUC_lexicon_tags_1 },
    { 0x48a8ec3a, SUC_lexicon_tags_14 },
    { 0xfa28ec3a, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0xdbc8ec3d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4988ec41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c8ec44, SUC_lexicon_tags_50 },
    { 0xd9c8ec45, SUC_lexicon_tags_79 },
    { 0xd88ec45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6188ec4c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c8ec4f, SUC_lexicon_tags_56 },
    { 0xd5e8ec50, SUC_lexicon_tags_37 },
    { 0x8528ec51, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf1a8ec53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a8ec58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x128ec5b, SUC_lexicon_tags_37 },
    { 0xe8a8ec5c, SUC_lexicon_tags_37 },
    { 0x82c8ec5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d48ec60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7c8ec62, SUC_lexicon_tags_28 },
    { 0x4f48ec62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xea48ec65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc348ec67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5c8ec69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf728ec6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748ec71, SUC_lexicon_tags_30 },
    { 0x88c8ec72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c8ec75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88ec79, SUC_lexicon_tags_63 },
    { 0x4f28ec79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4548ec7d, SUC_lexicon_tags_29 },
    { 0x1a68ec7e, SUC_lexicon_tags_79 },
    { 0xa5c8ec7f, SUC_lexicon_tags_53 },
    { 0x7228ec7e, SUC_lexicon_tags_25 },
    { 0x5168ec81, SUC_lexicon_tags_37 },
    { 0x6948ec82, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e8ec86, SUC_lexicon_tags_77 },
    { 0x92c8ec86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8628ec89, SUC_lexicon_tags_9 },
    { 0x5a08ec8a, SUC_lexicon_tags_43 },
    { 0xf68ec8b, SUC_lexicon_tags_17 },
    { 0xa028ec8b, SUC_lexicon_tags_11 },
    { 0x2ac8ec8d, SUC_lexicon_tags_45 },
    { 0xe988ec8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9188ec90, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0xdd28ec92, SUC_lexicon_tags_1 },
    { 0x4348ec92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c8ec97, SUC_lexicon_tags_23 },
    { 0x8a08ec97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xef88ec9a, SUC_lexicon_tags_17 },
    { 0x6e8ec9b, SUC_lexicon_tags_2 },
    { 0x6c28ec9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2dc8ec9e, SUC_lexicon_tags_27 },
    { 0x8448ec9f, SUC_lexicon_tags_27 },
    { 0xaa48eca0, SUC_lexicon_tags_22 },
    { 0x26c8eca1, SUC_lexicon_tags_1 },
    { 0x88e8ec9e, SUC_lexicon_tags_14 },
    { 0x61c8eca3, SUC_lexicon_tags_27 },
    { 0x3a28eca4, SUC_lexicon_tags_0 },
    { 0xc0c8eca3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6828ecad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f08ecb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x468ecb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf28ecb7, SUC_lexicon_tags_45 },
    { 0x9fc8ecb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb528ecbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb48ecbf, SUC_lexicon_tags_1 },
    { 0x2ec8ecc0, SUC_lexicon_tags_28 },
    { 0xbc08ecc1, SUC_lexicon_tags_43 },
    { 0xd0c8ecc2, SUC_lexicon_tags_1 },
    { 0xf9a8ecc3, SUC_lexicon_tags_6 },
    { 0x4d68ecc4, SUC_lexicon_tags_0 },
    { 0x7d08ecc4, SUC_lexicon_tags_25 },
    { 0x65c8ecc6, SUC_lexicon_tags_17 },
    { 0xbbe8ecc6, SUC_lexicon_tags_46 },
    { 0x9468ecc2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1c8ecca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfde8eccc, SUC_lexicon_tags_2 },
    { 0x37a8eccd, SUC_lexicon_tags_1 },
    { 0xf908ecce, SUC_lexicon_tags_79 },
    { 0x7da8ecce, SUC_lexicon_tags_31 },
    { 0x5788eccf, SUC_lexicon_tags_12 },
    { 0x43c8ecd1, SUC_lexicon_tags_45 },
    { 0xfba8eccc, SUC_lexicon_tags_1 },
    { 0x548ecd2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xac88ecd6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6068ecd8, SUC_lexicon_tags_32 },
    { 0x9f48ecd9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c8ece4, SUC_lexicon_tags_17 },
    { 0x8008ece5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2bc8ece7, SUC_lexicon_tags_12 },
    { 0x9a28ece8, SUC_lexicon_tags_65 },
    { 0x3948ece9, SUC_lexicon_tags_37 },
    { 0xc388ece9, SUC_lexicon_tags_49 },
    { 0xcda8eceb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf588ecef, SUC_lexicon_tags_33 },
    { 0x2888ecef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89a8ecf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9628ecf6, SUC_lexicon_tags_1 },
    { 0xa1e8ecf7, SUC_lexicon_tags_25 },
    { 0x2b08ecf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e8ecfb, SUC_lexicon_tags_87 },
    { 0xb928ecfc, SUC_lexicon_tags_22 },
    { 0x7008ecfc, SUC_lexicon_tags_1 },
    { 0x3a88ecfc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2588ed00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c8ed04, SUC_lexicon_tags_54 },
    { 0xd728ed05, SUC_lexicon_tags_65 },
    { 0xbd28ed06, SUC_lexicon_tags_87 },
    { 0x9348ed07, SUC_lexicon_tags_30 },
    { 0xaea8ed06, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x21a8ed0a, SUC_lexicon_tags_1 },
    { 0xadc8ed0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e8ed0e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a8ed13, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4188ed1b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7708ed1d, SUC_lexicon_tags_43 },
    { 0x7148ed1d, SUC_lexicon_tags_25 },
    { 0xb3e8ed1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe848ed21, SUC_lexicon_tags_23 },
    { 0xa5a8ed22, SUC_lexicon_tags_15 },
    { 0xf5c8ed22, SUC_lexicon_tags_37 },
    { 0xef08ed22, SUC_lexicon_tags_17 },
    { 0xd908ed25, SUC_lexicon_tags_31 },
    { 0x73e8ed23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6a8ed28, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa8ed2c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8388ed2e, SUC_lexicon_tags_23 },
    { 0xe1a8ed2f, SUC_lexicon_tags_2 },
    { 0xd628ed2f, SUC_lexicon_tags_7 },
    { 0x14c8ed2f, SUC_lexicon_tags_23 },
    { 0x3c48ed32, SUC_lexicon_tags_64 },
    { 0xdbe8ed33, SUC_lexicon_tags_66 },
    { 0x8c68ed33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd348ed36, SUC_lexicon_tags_34 },
    { 0xd48ed36, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8ce8ed39, SUC_lexicon_tags_15 },
    { 0xae8ed3a, SUC_lexicon_tags_1 },
    { 0xb188ed3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4488ed3d, SUC_lexicon_tags_25 },
    { 0x22c8ed3e, SUC_lexicon_tags_11 },
    { 0x18c8ed3f, SUC_lexicon_tags_87 },
    { 0x1408ed3d, SUC_lexicon_tags_49 },
    { 0xe3c8ed40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x468ed43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7808ed45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd68ed47, SUC_lexicon_tags_37 },
    { 0x7888ed48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c48ed4a, SUC_lexicon_tags_38 },
    { 0x328ed4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c8ed5f, SUC_lexicon_tags_17 },
    { 0x3a28ed60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a8ed67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf48ed6b, SUC_lexicon_tags_11 },
    { 0x1408ed6c, SUC_lexicon_tags_45 },
    { 0x8428ed6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf908ed70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f28ed72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8748ed7c, SUC_lexicon_tags_37 },
    { 0xefa8ed7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0a8ed7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb68ed81, SUC_lexicon_tags_25 },
    { 0xe4e8ed82, SUC_lexicon_tags_37 },
    { 0x1088ed82, SUC_lexicon_tags_15 },
    { 0xaec8ed82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1828ed8d, SUC_lexicon_tags_11 },
    { 0x8928ed8e, SUC_lexicon_tags_7 },
    { 0x1888ed8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe088ed93, SUC_lexicon_tags_60 },
    { 0x73e8ed94, SUC_lexicon_tags_25 },
    { 0x2b88ed95, SUC_lexicon_tags_9 },
    { 0x7ce8ed96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2a88ed98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea8ed9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f48eda1, SUC_lexicon_tags_15 },
    { 0xa0a8eda2, SUC_lexicon_tags_25 },
    { 0x13a8eda2, SUC_lexicon_tags_25 },
    { 0x8668eda4, SUC_lexicon_tags_29 },
    { 0xa5e8eda4, SUC_lexicon_tags_7 },
    { 0x25c8eda4, SUC_lexicon_tags_1 },
    { 0xe648eda6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d48eda9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a8edac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd328edb0, SUC_lexicon_tags_14 },
    { 0x2748edb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed48edb5, SUC_lexicon_tags_17 },
    { 0x6228edb6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6728edb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68edbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe8edc4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe88edc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83e8edc8, SUC_lexicon_tags_53 },
    { 0xf668edc9, SUC_lexicon_tags_23 },
    { 0x6e28edc9, SUC_lexicon_tags_38 },
    { 0x1188edcb, SUC_lexicon_tags_47 },
    { 0x41e8edcc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xedc8edce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c8edd1, SUC_lexicon_tags_13 },
    { 0x5808edd2, SUC_lexicon_tags_87 },
    { 0xecc8edd1, SUC_lexicon_tags_38 },
    { 0xb748edd4, SUC_lexicon_tags_37 },
    { 0xeee8edd4, SUC_lexicon_tags_19 },
    { 0xfac8edd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac8edd9, SUC_lexicon_tags_17 },
    { 0xa6c8edda, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c8eddd, SUC_lexicon_tags_20 },
    { 0xbfc8edde, SUC_lexicon_tags_37 },
    { 0xb88eddf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c8ede6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e48ede8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbda8edea, SUC_lexicon_tags_28 },
    { 0x268edeb, SUC_lexicon_tags_0 },
    { 0x2628edec, SUC_lexicon_tags_11 },
    { 0x64a8edec, SUC_lexicon_tags_17 },
    { 0x3248edee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc8edf1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9568edf6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xccc8edf8, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x7188edfa, SUC_lexicon_tags_53 },
    { 0x4b28edfb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68ee01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21c8ee03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc8ee0c, SUC_lexicon_tags_17 },
    { 0x3a68ee0d, SUC_lexicon_tags_37 },
    { 0x4da8ee0e, SUC_lexicon_tags_16 },
    { 0xa9c8ee0f, SUC_lexicon_tags_82 },
    { 0xf428ee0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7de8ee12, SUC_lexicon_tags_15 },
    { 0x1548ee13, SUC_lexicon_tags_17 },
    { 0xe008ee14, SUC_lexicon_tags_15 },
    { 0x2908ee15, SUC_lexicon_tags_66 },
    { 0xa248ee16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa608ee18, SUC_lexicon_tags_25 },
    { 0x1168ee18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a8ee1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff88ee1e, SUC_lexicon_tags_15 },
    { 0x2ac8ee1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ae8ee21, SUC_lexicon_tags_49 },
    { 0x9888ee21, SUC_lexicon_tags_14 },
    { 0x1008ee23, SUC_lexicon_tags_15 },
    { 0xf88ee24, SUC_lexicon_tags_7 },
    { 0x5fc8ee25, SUC_lexicon_tags_45 },
    { 0x948ee24, SUC_lexicon_tags_12 },
    { 0xa988ee25, SUC_lexicon_tags_19 },
    { 0x2148ee28, SUC_lexicon_tags_82 },
    { 0x77c8ee23, SUC_lexicon_tags_1 },
    { 0xeb48ee2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf428ee2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3c8ee2e, SUC_lexicon_tags_11 },
    { 0xd968ee2f, SUC_lexicon_tags_46 },
    { 0x828ee2f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e8ee35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd08ee37, SUC_lexicon_tags_44 },
    { 0x7628ee38, SUC_lexicon_tags_3 },
    { 0x99c8ee39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xacc8ee3b, SUC_lexicon_tags_46 },
    { 0xd988ee3c, SUC_lexicon_tags_11 },
    { 0x75c8ee3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a8ee44, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e8ee48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4a8ee4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1568ee4f, SUC_lexicon_tags_66 },
    { 0x2148ee50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d88ee53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2848ee58, SUC_lexicon_tags_148 },
    { 0xad08ee59, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae8ee5c, SUC_lexicon_tags_15 },
    { 0xc3e8ee5c, SUC_lexicon_tags_0 },
    { 0xc948ee5d, SUC_lexicon_tags_14 },
    { 0x6048ee5f, SUC_lexicon_tags_66 },
    { 0x4148ee5f, SUC_lexicon_tags_1 },
    { 0xdd28ee5c, SUC_lexicon_tags_191 },
    { 0x29e8ee62, SUC_lexicon_tags_32 },
    { 0x3be8ee63, SUC_lexicon_tags_23 },
    { 0x5f48ee64, SUC_lexicon_tags_39 },
    { 0x4988ee65, SUC_lexicon_tags_25 },
    { 0xf848ee66, SUC_lexicon_tags_25 },
    { 0x1c48ee67, SUC_lexicon_tags_12 },
    { 0xd128ee61, SUC_lexicon_tags_39 },
    { 0x5bc8ee69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3e08ee6b, SUC_lexicon_tags_31 },
    { 0x1908ee6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa048ee70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb048ee75, SUC_lexicon_tags_15 },
    { 0xcc08ee76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c68ee7c, SUC_lexicon_tags_7 },
    { 0x9528ee7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9728ee7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6308ee8a, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7128ee8e, SUC_lexicon_tags_23 },
    { 0x50c8ee8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec8ee93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c88ee97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf28ee99, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc328ee9c, SUC_lexicon_tags_37 },
    { 0x4ee8ee9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c8eea2, SUC_lexicon_tags_11 },
    { 0xd348eea2, SUC_lexicon_tags_15 },
    { 0x8308eea4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb628eea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e8eeaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x77a8eeac, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e88eeaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5e8eeb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1388eeb5, SUC_lexicon_tags_38 },
    { 0xdca8eeb6, SUC_lexicon_tags_7 },
    { 0x5168eeb6, SUC_lexicon_tags_1 },
    { 0x16e8eeb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c8eebb, SUC_lexicon_tags_14 },
    { 0x6668eebb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc728eec2, SUC_lexicon_tags_30 },
    { 0x9b48eec3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e8eec9, SUC_lexicon_tags_25 },
    { 0x9da8eec9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc488eed1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x908eed4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e8eed8, SUC_lexicon_tags_45 },
    { 0x52c8eed9, SUC_lexicon_tags_63 },
    { 0x1b28eeda, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x248eedc, SUC_lexicon_tags_14 },
    { 0xff68eedc, SUC_lexicon_tags_11 },
    { 0xede8eedc, SUC_lexicon_tags_25 },
    { 0x7488eedd, SUC_lexicon_tags_25 },
    { 0x9508eedf, SUC_lexicon_tags_11 },
    { 0xf3e8eedd, SUC_lexicon_tags_61 },
    { 0x4d08eedc, SUC_lexicon_tags_7 },
    { 0xd268eee3, SUC_lexicon_tags_63 },
    { 0xa7a8eedf, SUC_lexicon_tags_7 },
    { 0xb208eee5, SUC_lexicon_tags_33 },
    { 0x5a48eee6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x16a8eee8, SUC_lexicon_tags_45 },
    { 0x708eee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9428eeeb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6948eeed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7808eeef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da8eef2, SUC_lexicon_tags_1415 },
    { 0x9a68eef3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfa8eef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7a8eef7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6648eefa, SUC_lexicon_tags_45 },
    { 0x6e88eefb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd668eeff, SUC_lexicon_tags_14 },
    { 0x75a8ef00, SUC_lexicon_tags_12 },
    { 0xdf88ef01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8608ef07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68ef0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e8ef11, SUC_lexicon_tags_37 },
    { 0x4188ef12, SUC_lexicon_tags_23 },
    { 0xc9a8ef13, SUC_lexicon_tags_21 },
    { 0xae68ef11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd708ef1b, SUC_lexicon_tags_804 },
    { 0x5268ef1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x25c8ef1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd08ef22, SUC_lexicon_tags_25 },
    { 0x2f48ef23, SUC_lexicon_tags_7 },
    { 0x5c68ef23, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x4648ef26, SUC_lexicon_tags_49 },
    { 0xf748ef26, SUC_lexicon_tags_14 },
    { 0x6528ef27, SUC_lexicon_tags_17 },
    { 0xdec8ef29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe408ef2f, SUC_lexicon_tags_25 },
    { 0xaf48ef2f, SUC_lexicon_tags_7 },
    { 0xbbc8ef2f, SUC_lexicon_tags_25 },
    { 0x25e8ef2f, SUC_lexicon_tags_25 },
    { 0xa688ef33, SUC_lexicon_tags_12 },
    { 0x2748ef34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9148ef3a, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd428ef49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8988ef4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ef50, SUC_lexicon_tags_37 },
    { 0xd2e8ef51, SUC_lexicon_tags_12 },
    { 0x4ea8ef50, SUC_lexicon_tags_83 },
    { 0x9908ef50, SUC_lexicon_tags_28 },
    { 0xf5e8ef50, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4c8ef56, SUC_lexicon_tags_17 },
    { 0xa5e8ef57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc28ef5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa248ef5e, SUC_lexicon_tags_46 },
    { 0x96e8ef5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7628ef62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e8ef6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4688ef6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf648ef71, SUC_lexicon_tags_37 },
    { 0x34c8ef71, SUC_lexicon_tags_17 },
    { 0x6968ef71, SUC_lexicon_tags_49 },
    { 0x6288ef74, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee8ef7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec88ef7e, SUC_lexicon_tags_25 },
    { 0x7ac8ef7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08ef82, SUC_lexicon_tags_2 },
    { 0x4fc8ef82, SUC_lexicon_tags_172 },
    { 0x3b68ef83, SUC_lexicon_tags_70 },
    { 0xa848ef85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc68ef88, SUC_lexicon_tags_2 },
    { 0xd28ef89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e8ef8c, SUC_lexicon_tags_1 },
    { 0xd648ef8d, SUC_lexicon_tags_27 },
    { 0x7488ef8e, SUC_lexicon_tags_0 },
    { 0x7308ef8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e8ef93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e8ef99, SUC_lexicon_tags_45 },
    { 0x4fa8ef9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd48ef9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d08ef9f, SUC_lexicon_tags_37 },
    { 0x4568ef9f, SUC_lexicon_tags_17 },
    { 0x9808efa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8efa8, SUC_lexicon_tags_31 },
    { 0x40c8efa9, SUC_lexicon_tags_60 },
    { 0xa368efa8, SUC_lexicon_tags_26 },
    { 0x6528efaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf028efae, SUC_lexicon_tags_28 },
    { 0x6148efae, SUC_lexicon_tags_14 },
    { 0x66a8efae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2728efb2, SUC_lexicon_tags_17 },
    { 0xc9a8efb2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c68efb5, SUC_lexicon_tags_15 },
    { 0xf8a8efb6, SUC_lexicon_tags_45 },
    { 0x61e8efb6, SUC_lexicon_tags_25 },
    { 0x2f28efb5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf288efbd, SUC_lexicon_tags_43 },
    { 0x9a08efbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9668efc1, SUC_lexicon_tags_37 },
    { 0xea88efc2, SUC_lexicon_tags_4 },
    { 0xe748efc3, SUC_lexicon_tags_7 },
    { 0x5b28efc4, SUC_lexicon_tags_27 },
    { 0x7608efc5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc88efc8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf28efcd, SUC_lexicon_tags_66 },
    { 0x928efce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf2c8efd0, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e8efd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b68efd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a8efda, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3968efdc, SUC_lexicon_tags_37 },
    { 0xdc88efdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1528efdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a8efe3, SUC_lexicon_tags_65 },
    { 0x148efe3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb408efe7, SUC_lexicon_tags_1021 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a8efeb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1808eff2, SUC_lexicon_tags_25 },
    { 0x82e8eff3, SUC_lexicon_tags_15 },
    { 0x2068eff2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4408eff6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec8effa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3488efff, SUC_lexicon_tags_7 },
    { 0xe148f000, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xae28f002, SUC_lexicon_tags_17 },
    { 0xa9a8f002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed48f007, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2468f00b, SUC_lexicon_tags_17 },
    { 0x6fc8f00c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4168f00f, SUC_lexicon_tags_9 },
    { 0x3788f010, SUC_lexicon_tags_45 },
    { 0xe808f011, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf188f013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8828f015, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c8f019, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3608f01b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f68f024, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5428f02c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e68f02f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b68f035, SUC_lexicon_tags_12 },
    { 0x8488f036, SUC_lexicon_tags_45 },
    { 0xeec8f035, SUC_lexicon_tags_25 },
    { 0x3ac8f038, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4508f03a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c8f040, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4868f043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3328f046, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a8f04c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e08f052, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb968f054, SUC_lexicon_tags_43 },
    { 0xa608f055, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e8f05d, SUC_lexicon_tags_19 },
    { 0xf048f05e, SUC_lexicon_tags_49 },
    { 0xf268f05e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a8f062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe808f065, SUC_lexicon_tags_11 },
    { 0x39c8f065, SUC_lexicon_tags_4 },
    { 0x8888f065, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a48f06f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed08f07a, SUC_lexicon_tags_17 },
    { 0xb848f07b, SUC_lexicon_tags_45 },
    { 0x2a48f07b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57e8f07f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe008f089, SUC_lexicon_tags_12 },
    { 0x60c8f08a, SUC_lexicon_tags_11 },
    { 0x2768f08b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8f097, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb08f09a, SUC_lexicon_tags_63 },
    { 0x5108f09a, SUC_lexicon_tags_7 },
    { 0xf488f09b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fa8f09e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa768f0a0, SUC_lexicon_tags_25 },
    { 0x9a48f0a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5b48f0a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9ce8f0a5, SUC_lexicon_tags_172 },
    { 0x4248f0a6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9968f0aa, SUC_lexicon_tags_24 },
    { 0xa548f0aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd48f0ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e88f0b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8f0b3, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a8f0b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a8f0bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9768f0bf, SUC_lexicon_tags_2 },
    { 0x2168f0c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbbe8f0c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3128f0c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3468f0c8, SUC_lexicon_tags_2 },
    { 0x1b88f0c8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2888f0cc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6cc8f0ce, SUC_lexicon_tags_60 },
    { 0xa568f0cf, SUC_lexicon_tags_25 },
    { 0x3128f0ce, SUC_lexicon_tags_66 },
    { 0x33e8f0ce, SUC_lexicon_tags_14 },
    { 0x4568f0d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e8f0d9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe988f0db, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec8f0de, SUC_lexicon_tags_1 },
    { 0xe908f0df, SUC_lexicon_tags_17 },
    { 0xd828f0e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe528f0e7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f0f0, SUC_lexicon_tags_37 },
    { 0x7348f0f1, SUC_lexicon_tags_17 },
    { 0x9b48f0f0, SUC_lexicon_tags_37 },
    { 0xd4e8f0f3, SUC_lexicon_tags_19 },
    { 0x4d88f0f3, SUC_lexicon_tags_49 },
    { 0xf768f0f4, SUC_lexicon_tags_60 },
    { 0x3948f0f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x51c8f0f8, SUC_lexicon_tags_37 },
    { 0xa428f0f8, SUC_lexicon_tags_2 },
    { 0x7968f0f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf908f0fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d08f102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8168f104, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc8f108, SUC_lexicon_tags_108 },
    { 0xd0c8f109, SUC_lexicon_tags_60 },
    { 0x5ae8f108, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb068f10c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748f112, SUC_lexicon_tags_27 },
    { 0x5368f113, SUC_lexicon_tags_14 },
    { 0x6e08f113, SUC_lexicon_tags_27 },
    { 0x9128f113, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc8f117, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be8f11b, SUC_lexicon_tags_2 },
    { 0xa168f11c, SUC_lexicon_tags_1 },
    { 0xdfa8f11c, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7a8f121, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95e8f123, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a8f12b, SUC_lexicon_tags_7 },
    { 0x8a08f12c, SUC_lexicon_tags_27 },
    { 0xb28f12d, SUC_lexicon_tags_2 },
    { 0x2328f12b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c8f138, SUC_lexicon_tags_63 },
    { 0xdac8f139, SUC_lexicon_tags_25 },
    { 0x15c8f139, SUC_lexicon_tags_14 },
    { 0x7c28f138, SUC_lexicon_tags_106 },
    { 0x1ba8f13c, SUC_lexicon_tags_27 },
    { 0xc068f13d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a8f142, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd848f147, SUC_lexicon_tags_37 },
    { 0xf048f148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a8f14b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xb8a8f14d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d48f150, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9208f152, SUC_lexicon_tags_7 },
    { 0x14c8f153, SUC_lexicon_tags_31 },
    { 0x56c8f154, SUC_lexicon_tags_79 },
    { 0x2168f153, SUC_lexicon_tags_19 },
    { 0x51a8f154, SUC_lexicon_tags_84 },
    { 0x4fe8f157, SUC_lexicon_tags_17 },
    { 0x7e08f158, SUC_lexicon_tags_49 },
    { 0xfb68f152, SUC_lexicon_tags_47 },
    { 0xabe8f15a, SUC_lexicon_tags_15 },
    { 0x8aa8f15b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6628f160, SUC_lexicon_tags_16 },
    { 0x6328f161, SUC_lexicon_tags_37 },
    { 0xbac8f161, SUC_lexicon_tags_37 },
    { 0xb4e8f163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad08f167, SUC_lexicon_tags_46 },
    { 0x4b68f168, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da8f16c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5688f172, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5908f174, SUC_lexicon_tags_16 },
    { 0x4828f175, SUC_lexicon_tags_25 },
    { 0xf848f176, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb268f178, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6328f17c, SUC_lexicon_tags_45 },
    { 0x8108f17d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xecc8f17f, SUC_lexicon_tags_12 },
    { 0x1528f17f, SUC_lexicon_tags_11 },
    { 0x1608f17f, SUC_lexicon_tags_63 },
    { 0x3088f182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec48f184, SUC_lexicon_tags_45 },
    { 0x66a8f185, SUC_lexicon_tags_9 },
    { 0x58c8f186, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a88f189, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3588f18b, SUC_lexicon_tags_12 },
    { 0x5528f18b, SUC_lexicon_tags_11 },
    { 0xa908f18d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd388f190, SUC_lexicon_tags_10 },
    { 0x9ee8f190, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a8f194, SUC_lexicon_tags_33 },
    { 0x5868f195, SUC_lexicon_tags_11 },
    { 0x8268f195, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d08f19a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71a8f19c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8168f19f, SUC_lexicon_tags_18 },
    { 0xd668f1a0, SUC_lexicon_tags_23 },
    { 0x7b48f19f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8f1a4, SUC_lexicon_tags_66 },
    { 0xea88f1a5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9168f1a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e8f1ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbb28f1ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd08f1b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4128f1b6, SUC_lexicon_tags_7 },
    { 0xaf68f1b7, SUC_lexicon_tags_7 },
    { 0xa48f1b8, SUC_lexicon_tags_37 },
    { 0x428f1b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea8f1bc, SUC_lexicon_tags_12 },
    { 0xd688f1bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe368f1c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8e8f1c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a8f1c5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b68f1c9, SUC_lexicon_tags_11 },
    { 0x61a8f1ca, SUC_lexicon_tags_68 },
    { 0xa788f1ca, SUC_lexicon_tags_11 },
    { 0x1888f1cc, SUC_lexicon_tags_27 },
    { 0x2fe8f1cb, SUC_lexicon_tags_23 },
    { 0xcd08f1ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e28f1d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9be8f1d3, SUC_lexicon_tags_2 },
    { 0x3ac8f1d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9a8f1d6, SUC_lexicon_tags_63 },
    { 0x31e8f1d7, SUC_lexicon_tags_2 },
    { 0x11e8f1d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe008f1db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2148f1df, SUC_lexicon_tags_17 },
    { 0x2dc8f1df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f08f1e3, SUC_lexicon_tags_13 },
    { 0xff48f1e4, SUC_lexicon_tags_8 },
    { 0x4a48f1e3, SUC_lexicon_tags_26 },
    { 0xcf88f1e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e8f1ed, SUC_lexicon_tags_66 },
    { 0x76e8f1ee, SUC_lexicon_tags_37 },
    { 0x3ae8f1ef, SUC_lexicon_tags_48 },
    { 0xc5c8f1f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc328f1f2, SUC_lexicon_tags_77 },
    { 0xfc8f1f3, SUC_lexicon_tags_37 },
    { 0x2e8f1f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08f1f9, SUC_lexicon_tags_25 },
    { 0x2508f1f9, SUC_lexicon_tags_47 },
    { 0xdac8f1f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e8f1fd, SUC_lexicon_tags_27 },
    { 0x4d68f1fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a8f201, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8488f203, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde08f205, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2568f209, SUC_lexicon_tags_65 },
    { 0xfaa8f20a, SUC_lexicon_tags_14 },
    { 0x1b48f20a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c8f210, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2868f213, SUC_lexicon_tags_61 },
    { 0x9488f214, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e68f217, SUC_lexicon_tags_2 },
    { 0x1f28f218, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c8f21c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7468f21f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a8f223, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x66e8f225, SUC_lexicon_tags_23 },
    { 0xf6e8f226, SUC_lexicon_tags_37 },
    { 0x61a8f226, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba8f22a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce48f22c, SUC_lexicon_tags_6 },
    { 0x19a8f22c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48f231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab68f238, SUC_lexicon_tags_25 },
    { 0x8668f239, SUC_lexicon_tags_98 },
    { 0x7908f239, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1408f23c, SUC_lexicon_tags_2 },
    { 0xa4e8f23d, SUC_lexicon_tags_1 },
    { 0xe2c8f23e, SUC_lexicon_tags_30 },
    { 0xffa8f23c, SUC_lexicon_tags_7 },
    { 0x5008f240, SUC_lexicon_tags_9 },
    { 0x9a08f241, SUC_lexicon_tags_14 },
    { 0xb908f240, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb6c8f244, SUC_lexicon_tags_27 },
    { 0x5de8f245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa8f248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a68f24c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3ac8f24e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b08f252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb08f255, SUC_lexicon_tags_17 },
    { 0x1b48f256, SUC_lexicon_tags_4 },
    { 0x4988f257, SUC_lexicon_tags_1 },
    { 0x7ee8f258, SUC_lexicon_tags_32 },
    { 0xfbe8f255, SUC_lexicon_tags_17 },
    { 0x41e8f25a, SUC_lexicon_tags_87 },
    { 0xd548f25a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c8f263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe28f265, SUC_lexicon_tags_60 },
    { 0x3d68f266, SUC_lexicon_tags_12 },
    { 0x7ea8f266, SUC_lexicon_tags_47 },
    { 0xef08f266, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ce8f26a, SUC_lexicon_tags_12 },
    { 0x3be8f26a, SUC_lexicon_tags_23 },
    { 0xd308f26b, SUC_lexicon_tags_25 },
    { 0xe6c8f26d, SUC_lexicon_tags_0 },
    { 0x56c8f26c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e88f276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1748f278, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6268f27e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4208f288, SUC_lexicon_tags_5 },
    { 0xb168f289, SUC_lexicon_tags_47 },
    { 0xb788f288, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad68f291, SUC_lexicon_tags_14 },
    { 0x4148f292, SUC_lexicon_tags_12 },
    { 0x868f291, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c68f295, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3ba8f297, SUC_lexicon_tags_47 },
    { 0xb208f297, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe168f29c, SUC_lexicon_tags_17 },
    { 0x3d48f29c, SUC_lexicon_tags_14 },
    { 0x3288f29c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd628f2a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa88f2a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e8f2a7, SUC_lexicon_tags_15 },
    { 0x9488f2a7, SUC_lexicon_tags_23 },
    { 0xbae8f2a9, SUC_lexicon_tags_7 },
    { 0x4168f2aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468f2af, SUC_lexicon_tags_45 },
    { 0x1368f2af, SUC_lexicon_tags_29 },
    { 0xc888f2b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c8f2b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a88f2b5, SUC_lexicon_tags_56 },
    { 0x2128f2b6, SUC_lexicon_tags_38 },
    { 0x36c8f2b5, SUC_lexicon_tags_25 },
    { 0xe288f2b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9148f2bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9908f2bf, SUC_lexicon_tags_6 },
    { 0x1068f2c0, SUC_lexicon_tags_17 },
    { 0x848f2c0, SUC_lexicon_tags_24 },
    { 0x7f08f2c2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xa3a8f2c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x708f2c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfa48f2c8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5708f2ca, SUC_lexicon_tags_49 },
    { 0xee28f2cb, SUC_lexicon_tags_60 },
    { 0x91e8f2cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa488f2d7, SUC_lexicon_tags_15 },
    { 0x88c8f2d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa668f2db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa268f2dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf28f2df, SUC_lexicon_tags_32 },
    { 0x34a8f2df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba88f2e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7028f2ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc8f2f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc2c8f2f2, SUC_lexicon_tags_47 },
    { 0x9948f2f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb268f2f8, SUC_lexicon_tags_30 },
    { 0xd8c8f2f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4aa8f2fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf28f2fd, SUC_lexicon_tags_12 },
    { 0x8688f2fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1208f300, SUC_lexicon_tags_27 },
    { 0xe3a8f301, SUC_lexicon_tags_33 },
    { 0x8e68f302, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3148f307, SUC_lexicon_tags_37 },
    { 0xa4c8f307, SUC_lexicon_tags_17 },
    { 0xb68f307, SUC_lexicon_tags_2 },
    { 0xc228f307, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x75c8f30c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa268f30e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e8f311, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a8f314, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5288f317, SUC_lexicon_tags_2 },
    { 0x7ca8f317, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8f31b, SUC_lexicon_tags_47 },
    { 0xe148f31b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec28f31e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11e8f320, SUC_lexicon_tags_11 },
    { 0xede8f320, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e88f323, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a8f326, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26c8f328, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe8f32c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3888f331, SUC_lexicon_tags_17 },
    { 0x4968f332, SUC_lexicon_tags_26 },
    { 0x9748f332, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd028f336, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c08f33a, SUC_lexicon_tags_57 },
    { 0xd0a8f33b, SUC_lexicon_tags_25 },
    { 0x9a08f33c, SUC_lexicon_tags_25 },
    { 0xde48f33c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87c8f33f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e68f346, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb528f348, SUC_lexicon_tags_1 },
    { 0xd1e8f349, SUC_lexicon_tags_77 },
    { 0x6608f34a, SUC_lexicon_tags_8 },
    { 0x6068f348, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd748f34f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5968f351, SUC_lexicon_tags_7 },
    { 0x3228f352, SUC_lexicon_tags_7 },
    { 0xdb68f353, SUC_lexicon_tags_45 },
    { 0xdf08f352, SUC_lexicon_tags_17 },
    { 0x7068f353, SUC_lexicon_tags_9 },
    { 0x5048f356, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a8f35d, SUC_lexicon_tags_37 },
    { 0x33e8f35e, SUC_lexicon_tags_7 },
    { 0xd548f35f, SUC_lexicon_tags_22 },
    { 0xc128f35e, SUC_lexicon_tags_60 },
    { 0x1ac8f35e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e8f365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93a8f367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3948f36a, SUC_lexicon_tags_10 },
    { 0x1da8f36b, SUC_lexicon_tags_65 },
    { 0x4a88f36b, SUC_lexicon_tags_1 },
    { 0x9688f36d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5908f374, SUC_lexicon_tags_23 },
    { 0x23a8f374, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xede8f378, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8888f37b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf68f37d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8f380, SUC_lexicon_tags_15 },
    { 0x92e8f380, SUC_lexicon_tags_11 },
    { 0x3e48f382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a28f385, SUC_lexicon_tags_11 },
    { 0xe008f385, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8248f388, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3b88f38a, SUC_lexicon_tags_37 },
    { 0xfd68f38b, SUC_lexicon_tags_11 },
    { 0x85a8f38b, SUC_lexicon_tags_7 },
    { 0x23a8f38a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28f390, SUC_lexicon_tags_37 },
    { 0xe88f391, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe68f394, SUC_lexicon_tags_12 },
    { 0xa188f395, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae8f398, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba88f39c, SUC_lexicon_tags_6 },
    { 0x8088f39c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c8f3a0, SUC_lexicon_tags_63 },
    { 0xe1e8f3a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f48f3a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e8f3a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9248f3a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb588f3ab, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5428f3ad, SUC_lexicon_tags_38 },
    { 0xb9c8f3ae, SUC_lexicon_tags_17 },
    { 0x6d48f3af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a88f3b1, SUC_lexicon_tags_47 },
    { 0x2788f3b2, SUC_lexicon_tags_14 },
    { 0x7bc8f3b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c28f3bb, SUC_lexicon_tags_15 },
    { 0x3ee8f3bb, SUC_lexicon_tags_1 },
    { 0x7f28f3bd, SUC_lexicon_tags_81 },
    { 0x1808f3bc, SUC_lexicon_tags_0 },
    { 0x21c8f3bf, SUC_lexicon_tags_15 },
    { 0xcc8f3be, SUC_lexicon_tags_15 },
    { 0x8a88f3bf, SUC_lexicon_tags_60 },
    { 0xfd48f3c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8608f3ca, SUC_lexicon_tags_29 },
    { 0x2068f3cb, SUC_lexicon_tags_12 },
    { 0x5c28f3cb, SUC_lexicon_tags_87 },
    { 0x7528f3cd, SUC_lexicon_tags_158 },
    { 0xfc48f3cc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef48f3d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5548f3d9, SUC_lexicon_tags_55 },
    { 0xe48f3da, SUC_lexicon_tags_25 },
    { 0xba88f3da, SUC_lexicon_tags_108 },
    { 0xd628f3dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2e8f3de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2108f3e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca8f3e4, SUC_lexicon_tags_17 },
    { 0x94c8f3e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a88f3e9, SUC_lexicon_tags_15 },
    { 0xef68f3e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d08f3f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8e8f3f3, SUC_lexicon_tags_27 },
    { 0x4d48f3f4, SUC_lexicon_tags_1 },
    { 0x7d68f3f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e8f3f8, SUC_lexicon_tags_48 },
    { 0x9f68f3f9, SUC_lexicon_tags_41 },
    { 0xb888f3fa, SUC_lexicon_tags_11 },
    { 0x2708f3fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7568f3ff, SUC_lexicon_tags_43 },
    { 0x31a8f400, SUC_lexicon_tags_32 },
    { 0xbda8f401, SUC_lexicon_tags_41 },
    { 0xf7e8f402, SUC_lexicon_tags_77 },
    { 0x55a8f401, SUC_lexicon_tags_95 },
    { 0x6a48f403, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b68f409, SUC_lexicon_tags_23 },
    { 0x3048f40a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b48f40c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b68f40e, SUC_lexicon_tags_30 },
    { 0x2e88f40e, SUC_lexicon_tags_45 },
    { 0x6ca8f410, SUC_lexicon_tags_23 },
    { 0xae08f411, SUC_lexicon_tags_11 },
    { 0x5008f412, SUC_lexicon_tags_136 },
    { 0xc4a8f40f, SUC_lexicon_tags_92 },
    { 0x5468f414, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc68f416, SUC_lexicon_tags_56 },
    { 0xee28f416, SUC_lexicon_tags_14 },
    { 0x6fe8f418, SUC_lexicon_tags_25 },
    { 0x25c8f419, SUC_lexicon_tags_82 },
    { 0x4d48f417, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3cc8f41c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f28f41f, SUC_lexicon_tags_9 },
    { 0xe1a8f41f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e88f424, SUC_lexicon_tags_30 },
    { 0x1de8f425, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf608f42e, SUC_lexicon_tags_14 },
    { 0x1ac8f42f, SUC_lexicon_tags_46 },
    { 0x71c8f430, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28f433, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f68f43d, SUC_lexicon_tags_85 },
    { 0xf868f43e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe928f446, SUC_lexicon_tags_5 },
    { 0x5188f447, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf08f449, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7208f44c, SUC_lexicon_tags_25 },
    { 0xf328f44c, SUC_lexicon_tags_7 },
    { 0xba8f44e, SUC_lexicon_tags_2 },
    { 0x5388f44d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac8f456, SUC_lexicon_tags_23 },
    { 0x79c8f457, SUC_lexicon_tags_37 },
    { 0x1e48f458, SUC_lexicon_tags_30 },
    { 0x748f458, SUC_lexicon_tags_12 },
    { 0x908f45a, SUC_lexicon_tags_2 },
    { 0xc228f45b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e28f45f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf8a8f461, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b88f465, SUC_lexicon_tags_65 },
    { 0xc0a8f465, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8f46a, SUC_lexicon_tags_11 },
    { 0x8a68f46b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2248f46e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1908f470, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf408f472, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c8f476, SUC_lexicon_tags_2 },
    { 0xe3c8f476, SUC_lexicon_tags_5 },
    { 0x1e8f478, SUC_lexicon_tags_31 },
    { 0x1c08f478, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd868f480, SUC_lexicon_tags_47 },
    { 0x62c8f481, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb68f488, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32c8f48a, SUC_lexicon_tags_37 },
    { 0x4508f48b, SUC_lexicon_tags_1 },
    { 0x78c8f48a, SUC_lexicon_tags_33 },
    { 0x8028f48d, SUC_lexicon_tags_30 },
    { 0xaaa8f48c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba08f493, SUC_lexicon_tags_12 },
    { 0x9c88f493, SUC_lexicon_tags_37 },
    { 0xf988f495, SUC_lexicon_tags_30 },
    { 0x7788f496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e48f49a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a8f4a4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2228f4a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a8f4aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c8f4ad, SUC_lexicon_tags_17 },
    { 0xa968f4ae, SUC_lexicon_tags_85 },
    { 0x3048f4ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab48f4b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c88f4b9, SUC_lexicon_tags_25 },
    { 0xed68f4ba, SUC_lexicon_tags_1 },
    { 0x4888f4b9, SUC_lexicon_tags_21 },
    { 0xefa8f4bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4188f4c0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a8f4c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1448f4c5, SUC_lexicon_tags_8 },
    { 0x13e8f4c5, SUC_lexicon_tags_38 },
    { 0x37a8f4c5, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6568f4cc, SUC_lexicon_tags_48 },
    { 0xa348f4cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa788f4cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0c8f4d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c8f4d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x75a8f4d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9748f4d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6c8f4db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3788f4dd, SUC_lexicon_tags_1 },
    { 0x2ec8f4de, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e8f4e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf488f4e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1328f4ea, SUC_lexicon_tags_23 },
    { 0x9208f4eb, SUC_lexicon_tags_68 },
    { 0xc5a8f4ec, SUC_lexicon_tags_0 },
    { 0x5048f4ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1728f4f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1488f4f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8c8f4f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe8f4f7, SUC_lexicon_tags_62 },
    { 0xf268f4f7, SUC_lexicon_tags_9 },
    { 0xe0c8f4f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a8f4fc, SUC_lexicon_tags_52 },
    { 0x88f4fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae08f501, SUC_lexicon_tags_2 },
    { 0x4a88f501, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f28f504, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc68f507, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xba28f509, SUC_lexicon_tags_59 },
    { 0x1ec8f50a, SUC_lexicon_tags_37 },
    { 0xef88f50b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc8f50f, SUC_lexicon_tags_8 },
    { 0x26e8f510, SUC_lexicon_tags_35 },
    { 0xf968f511, SUC_lexicon_tags_49 },
    { 0xea88f512, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f68f515, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf428f518, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c8f51b, SUC_lexicon_tags_8 },
    { 0xd368f51b, SUC_lexicon_tags_0 },
    { 0x27c8f51d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8568f520, SUC_lexicon_tags_19 },
    { 0x42e8f521, SUC_lexicon_tags_37 },
    { 0x4ce8f521, SUC_lexicon_tags_17 },
    { 0x45e8f522, SUC_lexicon_tags_2 },
    { 0x1ce8f521, SUC_lexicon_tags_18 },
    { 0x9b28f525, SUC_lexicon_tags_49 },
    { 0x5ca8f526, SUC_lexicon_tags_12 },
    { 0xe108f527, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe5c8f529, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9888f52d, SUC_lexicon_tags_47 },
    { 0x348f52e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e8f531, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd048f533, SUC_lexicon_tags_37 },
    { 0x3908f533, SUC_lexicon_tags_25 },
    { 0x1e08f535, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2da8f537, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa08f53a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9788f53e, SUC_lexicon_tags_83 },
    { 0xa908f53e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3ee8f541, SUC_lexicon_tags_6 },
    { 0x5cc8f542, SUC_lexicon_tags_46 },
    { 0x61a8f543, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9428f549, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c28f54c, SUC_lexicon_tags_60 },
    { 0xf128f54d, SUC_lexicon_tags_1 },
    { 0xb928f54e, SUC_lexicon_tags_49 },
    { 0x9f68f54e, SUC_lexicon_tags_37 },
    { 0x74c8f550, SUC_lexicon_tags_45 },
    { 0xaa48f550, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdae8f553, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87c8f555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3148f558, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd28f55b, SUC_lexicon_tags_37 },
    { 0x5ea8f55c, SUC_lexicon_tags_24 },
    { 0xe0a8f55d, SUC_lexicon_tags_30 },
    { 0xfd28f55e, SUC_lexicon_tags_12 },
    { 0xf7a8f55e, SUC_lexicon_tags_7 },
    { 0x1988f560, SUC_lexicon_tags_35 },
    { 0x61a8f55c, SUC_lexicon_tags_47 },
    { 0x78e8f562, SUC_lexicon_tags_1 },
    { 0xb228f562, SUC_lexicon_tags_15 },
    { 0x3668f563, SUC_lexicon_tags_26 },
    { 0x6948f565, SUC_lexicon_tags_15 },
    { 0x1428f565, SUC_lexicon_tags_336 },
    { 0x9c68f564, SUC_lexicon_tags_8 },
    { 0xcba8f55b, SUC_lexicon_tags_151 },
    { 0x5268f569, SUC_lexicon_tags_66 },
    { 0xb5c8f569, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368f56f, SUC_lexicon_tags_98 },
    { 0x2e28f570, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe328f573, SUC_lexicon_tags_0 },
    { 0x6848f574, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6928f578, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8348f57a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e8f57d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8408f57f, SUC_lexicon_tags_15 },
    { 0xef48f57f, SUC_lexicon_tags_5 },
    { 0xe848f581, SUC_lexicon_tags_46 },
    { 0xadc8f582, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe28f584, SUC_lexicon_tags_1 },
    { 0xbb88f584, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74e8f587, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1f28f589, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35c8f58b, SUC_lexicon_tags_69 },
    { 0xe2c8f58b, SUC_lexicon_tags_1 },
    { 0xaf08f58c, SUC_lexicon_tags_15 },
    { 0x6e8f58e, SUC_lexicon_tags_30 },
    { 0x9388f58e, SUC_lexicon_tags_43 },
    { 0x2b48f590, SUC_lexicon_tags_56 },
    { 0x2128f590, SUC_lexicon_tags_25 },
    { 0xe508f591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea8f598, SUC_lexicon_tags_84 },
    { 0x7468f599, SUC_lexicon_tags_19 },
    { 0x8c8f59a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2248f59f, SUC_lexicon_tags_31 },
    { 0x4268f59f, SUC_lexicon_tags_7 },
    { 0x58c8f5a0, SUC_lexicon_tags_33 },
    { 0x3408f5a2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe328f5a7, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a68f5aa, SUC_lexicon_tags_26 },
    { 0x9628f5aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xffe8f5ad, SUC_lexicon_tags_25 },
    { 0xf9c8f5ae, SUC_lexicon_tags_29 },
    { 0x2e28f5af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdfa8f5b1, SUC_lexicon_tags_19 },
    { 0xeb48f5b1, SUC_lexicon_tags_2 },
    { 0xfae8f5b1, SUC_lexicon_tags_37 },
    { 0xdce8f5b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608f5b9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf168f5bb, SUC_lexicon_tags_0 },
    { 0xaaa8f5bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbac8f5be, SUC_lexicon_tags_63 },
    { 0x26e8f5bf, SUC_lexicon_tags_11 },
    { 0xd648f5c0, SUC_lexicon_tags_60 },
    { 0xe668f5c0, SUC_lexicon_tags_37 },
    { 0xb448f5c1, SUC_lexicon_tags_19 },
    { 0xe248f5c1, SUC_lexicon_tags_47 },
    { 0x6a48f5c3, SUC_lexicon_tags_7 },
    { 0x2728f5c5, SUC_lexicon_tags_7 },
    { 0xcc08f5c6, SUC_lexicon_tags_29 },
    { 0x71c8f5c5, SUC_lexicon_tags_15 },
    { 0xa828f5be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56a8f5ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac8f5d0, SUC_lexicon_tags_35 },
    { 0x3948f5d1, SUC_lexicon_tags_33 },
    { 0xbfe8f5d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6ae8f5d4, SUC_lexicon_tags_34 },
    { 0xc1c8f5d5, SUC_lexicon_tags_29 },
    { 0x9708f5d4, SUC_lexicon_tags_10 },
    { 0x4008f5d7, SUC_lexicon_tags_49 },
    { 0xca8f5d4, SUC_lexicon_tags_36 },
    { 0x1ea8f5d7, SUC_lexicon_tags_63 },
    { 0x5168f5d4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe508f5e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2908f5e6, SUC_lexicon_tags_1 },
    { 0x8208f5e6, SUC_lexicon_tags_7 },
    { 0x9bc8f5e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1748f5f2, SUC_lexicon_tags_7 },
    { 0x12a8f5f3, SUC_lexicon_tags_1 },
    { 0x7248f5f4, SUC_lexicon_tags_7 },
    { 0x5508f5f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8428f5fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a08f604, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c8f607, SUC_lexicon_tags_28 },
    { 0xc728f607, SUC_lexicon_tags_2 },
    { 0xca68f609, SUC_lexicon_tags_19 },
    { 0x8de8f607, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e8f60d, SUC_lexicon_tags_30 },
    { 0x28a8f60e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5c8f610, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb368f612, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8f61a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b28f61c, SUC_lexicon_tags_11 },
    { 0xe828f61d, SUC_lexicon_tags_12 },
    { 0x53c8f61e, SUC_lexicon_tags_23 },
    { 0xf728f61e, SUC_lexicon_tags_37 },
    { 0xca48f620, SUC_lexicon_tags_11 },
    { 0xce28f621, SUC_lexicon_tags_68 },
    { 0x65c8f61e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae8f627, SUC_lexicon_tags_60 },
    { 0xa108f627, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca8f62e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee8f631, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9428f638, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9d08f63a, SUC_lexicon_tags_37 },
    { 0xe5a8f63a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1428f63f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe8f641, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8888f643, SUC_lexicon_tags_12 },
    { 0xd648f644, SUC_lexicon_tags_37 },
    { 0x9288f645, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e8f649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fa8f64b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b88f64d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8528f64f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d68f651, SUC_lexicon_tags_10 },
    { 0x4248f652, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa48f654, SUC_lexicon_tags_9 },
    { 0xbc88f655, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8f659, SUC_lexicon_tags_2 },
    { 0x508f659, SUC_lexicon_tags_37 },
    { 0xc6a8f65a, SUC_lexicon_tags_60 },
    { 0x2d68f65b, SUC_lexicon_tags_11 },
    { 0xd5a8f65d, SUC_lexicon_tags_1 },
    { 0xaa08f65d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5608f661, SUC_lexicon_tags_45 },
    { 0xf268f662, SUC_lexicon_tags_12 },
    { 0x4bc8f662, SUC_lexicon_tags_6 },
    { 0x1ea8f664, SUC_lexicon_tags_2 },
    { 0xc6a8f663, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe068f668, SUC_lexicon_tags_83 },
    { 0x7c48f669, SUC_lexicon_tags_47 },
    { 0x8f08f66a, SUC_lexicon_tags_46 },
    { 0xcde8f668, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e68f66e, SUC_lexicon_tags_66 },
    { 0x2e48f66f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a8f674, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a8f677, SUC_lexicon_tags_25 },
    { 0x1688f677, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5a8f67a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5248f67c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe788f683, SUC_lexicon_tags_11 },
    { 0x7e28f684, SUC_lexicon_tags_27 },
    { 0xc588f685, SUC_lexicon_tags_65 },
    { 0x9c88f684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e8f68c, SUC_lexicon_tags_21 },
    { 0x6e88f68d, SUC_lexicon_tags_1104 },
    { 0xaea8f68c, SUC_lexicon_tags_0 },
    { 0x4da8f68f, SUC_lexicon_tags_32 },
    { 0xf448f690, SUC_lexicon_tags_30 },
    { 0xc928f68c, SUC_lexicon_tags_27 },
    { 0x9c08f68f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc8f695, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc8f69b, SUC_lexicon_tags_47 },
    { 0xb608f69c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd948f6a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb268f6a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a88f6aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae8f6b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4008f6b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae8f6b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc68f6b8, SUC_lexicon_tags_87 },
    { 0x6a48f6b8, SUC_lexicon_tags_1 },
    { 0xdde8f6b8, SUC_lexicon_tags_19 },
    { 0xef08f6bb, SUC_lexicon_tags_33 },
    { 0x4808f6bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2668f6c3, SUC_lexicon_tags_87 },
    { 0x7808f6c3, SUC_lexicon_tags_11 },
    { 0x52c8f6c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a8f6ca, SUC_lexicon_tags_49 },
    { 0xa688f6cb, SUC_lexicon_tags_27 },
    { 0xb5e8f6cc, SUC_lexicon_tags_36 },
    { 0xb868f6cd, SUC_lexicon_tags_1 },
    { 0xefa8f6cd, SUC_lexicon_tags_38 },
    { 0x4908f6cd, SUC_lexicon_tags_25 },
    { 0x35c8f6d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9e8f6d2, SUC_lexicon_tags_7 },
    { 0xa308f6d3, SUC_lexicon_tags_8 },
    { 0x3608f6d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e8f6d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a8f6db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99e8f6dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca8f6e8, SUC_lexicon_tags_11 },
    { 0x4f88f6e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9748f6ed, SUC_lexicon_tags_28 },
    { 0xe948f6ee, SUC_lexicon_tags_63 },
    { 0x9f48f6ef, SUC_lexicon_tags_1 },
    { 0x5548f6f0, SUC_lexicon_tags_57 },
    { 0x2728f6ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c8f6f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd088f6f9, SUC_lexicon_tags_43 },
    { 0xc0c8f6fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xba08f6fd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xc8f6ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd688f705, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4ea8f707, SUC_lexicon_tags_15 },
    { 0x8a48f708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7aa8f70a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e8f70d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x48e8f70f, SUC_lexicon_tags_40 },
    { 0x1a88f710, SUC_lexicon_tags_37 },
    { 0xd248f710, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf748f714, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xede8f716, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23e8f718, SUC_lexicon_tags_10 },
    { 0xd528f719, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3aa8f71b, SUC_lexicon_tags_2 },
    { 0xa208f71b, SUC_lexicon_tags_1 },
    { 0xfa48f71d, SUC_lexicon_tags_39 },
    { 0xbe28f71e, SUC_lexicon_tags_2 },
    { 0xf628f71d, SUC_lexicon_tags_37 },
    { 0x2808f720, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce08f724, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcc68f726, SUC_lexicon_tags_17 },
    { 0xdbc8f727, SUC_lexicon_tags_12 },
    { 0xd088f726, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc728f72e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6448f730, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c8f736, SUC_lexicon_tags_17 },
    { 0xa508f737, SUC_lexicon_tags_15 },
    { 0x94c8f737, SUC_lexicon_tags_38 },
    { 0xe008f739, SUC_lexicon_tags_31 },
    { 0x20a8f73a, SUC_lexicon_tags_25 },
    { 0xf848f738, SUC_lexicon_tags_12 },
    { 0xe5e8f73c, SUC_lexicon_tags_46 },
    { 0xf9e8f73c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da8f741, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c8f745, SUC_lexicon_tags_37 },
    { 0xac08f745, SUC_lexicon_tags_25 },
    { 0xbb88f747, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c8f74f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3388f752, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb428f75c, SUC_lexicon_tags_7 },
    { 0x3ea8f75d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb928f75f, SUC_lexicon_tags_11 },
    { 0x2e68f75f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8f766, SUC_lexicon_tags_47 },
    { 0xef68f766, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc028f76a, SUC_lexicon_tags_49 },
    { 0x5fe8f76b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea8f771, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdce8f773, SUC_lexicon_tags_1 },
    { 0x1a28f773, SUC_lexicon_tags_3 },
    { 0xaf28f775, SUC_lexicon_tags_38 },
    { 0x7ca8f775, SUC_lexicon_tags_17 },
    { 0x5d28f776, SUC_lexicon_tags_12 },
    { 0x7108f776, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488f783, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4f88f785, SUC_lexicon_tags_84 },
    { 0xacc8f786, SUC_lexicon_tags_13 },
    { 0x4668f785, SUC_lexicon_tags_30 },
    { 0xd608f788, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe348f78a, SUC_lexicon_tags_25 },
    { 0x8a88f78b, SUC_lexicon_tags_12 },
    { 0x2a28f78a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b08f78e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2e8f790, SUC_lexicon_tags_16 },
    { 0x3508f791, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x28e8f793, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf248f797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a8f79b, SUC_lexicon_tags_15 },
    { 0xc668f79b, SUC_lexicon_tags_84 },
    { 0x688f79c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea8f7a4, SUC_lexicon_tags_23 },
    { 0xcfe8f7a5, SUC_lexicon_tags_1 },
    { 0x57a8f7a6, SUC_lexicon_tags_2 },
    { 0x28f7a4, SUC_lexicon_tags_37 },
    { 0xd688f7a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5668f7ac, SUC_lexicon_tags_62 },
    { 0x1ea8f7ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2748f7af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e8f7b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4908f7b4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x688f7b7, SUC_lexicon_tags_15 },
    { 0xf1c8f7b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18e8f7ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c8f7c1, SUC_lexicon_tags_23 },
    { 0x7cc8f7c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc308f7c4, SUC_lexicon_tags_19 },
    { 0xbde8f7c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd88f7cd, SUC_lexicon_tags_29 },
    { 0x4948f7ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8f7d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6e8f7d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea8f7d7, SUC_lexicon_tags_9 },
    { 0x8b28f7d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe8f7dd, SUC_lexicon_tags_144 },
    { 0x52a8f7de, SUC_lexicon_tags_8 },
    { 0x6208f7df, SUC_lexicon_tags_49 },
    { 0x2b68f7de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfe8f7e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4188f7e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e8f7e9, SUC_lexicon_tags_11 },
    { 0x8f08f7ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc8f7ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3168f7f4, SUC_lexicon_tags_0 },
    { 0xefc8f7f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d88f7f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe88f7f9, SUC_lexicon_tags_16 },
    { 0x1b68f7fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed08f800, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b28f805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a8f80c, SUC_lexicon_tags_26 },
    { 0x78e8f80d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae48f811, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e48f818, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a8f81b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xedc8f81d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x168f820, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa888f822, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20c8f824, SUC_lexicon_tags_7 },
    { 0x5be8f824, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9268f828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6368f82a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa468f82d, SUC_lexicon_tags_37 },
    { 0xa7c8f82d, SUC_lexicon_tags_2 },
    { 0xe8e8f82d, SUC_lexicon_tags_27 },
    { 0x8d28f82d, SUC_lexicon_tags_23 },
    { 0xf9a8f831, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb288f833, SUC_lexicon_tags_4 },
    { 0x2448f833, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea8f837, SUC_lexicon_tags_81 },
    { 0xcda8f838, SUC_lexicon_tags_2 },
    { 0x25a8f839, SUC_lexicon_tags_23 },
    { 0x2a28f83a, SUC_lexicon_tags_43 },
    { 0x9508f839, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce8f845, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb08f848, SUC_lexicon_tags_40 },
    { 0xd428f849, SUC_lexicon_tags_25 },
    { 0xef48f848, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e8f84d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a8f856, SUC_lexicon_tags_1 },
    { 0x73e8f857, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe48f85c, SUC_lexicon_tags_19 },
    { 0xe968f85c, SUC_lexicon_tags_11 },
    { 0x788f85e, SUC_lexicon_tags_37 },
    { 0x2668f85c, SUC_lexicon_tags_25 },
    { 0x4868f860, SUC_lexicon_tags_57 },
    { 0xf328f861, SUC_lexicon_tags_47 },
    { 0xf6e8f862, SUC_lexicon_tags_12 },
    { 0x1188f85e, SUC_lexicon_tags_11 },
    { 0x3f08f864, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e68f866, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c8f869, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xac48f86c, SUC_lexicon_tags_17 },
    { 0x1dc8f86d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8a28f86f, SUC_lexicon_tags_33 },
    { 0x6748f86f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe768f877, SUC_lexicon_tags_9 },
    { 0x4528f878, SUC_lexicon_tags_0 },
    { 0x1448f879, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea8f87c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb68f880, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7888f882, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1788f886, SUC_lexicon_tags_15 },
    { 0xa7c8f887, SUC_lexicon_tags_12 },
    { 0x6d28f886, SUC_lexicon_tags_1 },
    { 0x57c8f888, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e8f890, SUC_lexicon_tags_7 },
    { 0xa2e8f891, SUC_lexicon_tags_38 },
    { 0xddc8f892, SUC_lexicon_tags_47 },
    { 0x5048f893, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f28f896, SUC_lexicon_tags_25 },
    { 0xf088f896, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e8f89a, SUC_lexicon_tags_47 },
    { 0xdea8f89a, SUC_lexicon_tags_30 },
    { 0xaec8f89c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4408f89f, SUC_lexicon_tags_28 },
    { 0xcf08f8a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c8f8a6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8f8a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb328f8ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5bc8f8ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa68f8b0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf3e8f8b2, SUC_lexicon_tags_23 },
    { 0x73e8f8b3, SUC_lexicon_tags_31 },
    { 0x60e8f8b2, SUC_lexicon_tags_7 },
    { 0xb968f8b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a8f8b8, SUC_lexicon_tags_25 },
    { 0x4ca8f8b9, SUC_lexicon_tags_15 },
    { 0xe7e8f8b9, SUC_lexicon_tags_11 },
    { 0xf988f8b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8988f8bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd208f8c2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x588f8c5, SUC_lexicon_tags_27 },
    { 0x4d08f8c6, SUC_lexicon_tags_0 },
    { 0x7d48f8c7, SUC_lexicon_tags_35 },
    { 0xdba8f8c7, SUC_lexicon_tags_0 },
    { 0x8708f8c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe8f8cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7168f8cf, SUC_lexicon_tags_37 },
    { 0xcbc8f8d0, SUC_lexicon_tags_25 },
    { 0xa908f8d1, SUC_lexicon_tags_19 },
    { 0x33a8f8d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b68f8d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c8f8d7, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf068f8db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e8f8e3, SUC_lexicon_tags_5 },
    { 0xb168f8e4, SUC_lexicon_tags_16 },
    { 0x55e8f8e4, SUC_lexicon_tags_16 },
    { 0x2588f8e6, SUC_lexicon_tags_37 },
    { 0x9dc8f8e7, SUC_lexicon_tags_1563 },
    { 0x5688f8e8, SUC_lexicon_tags_9 },
    { 0x2e08f8e8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x91a8f8eb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xede8f8ee, SUC_lexicon_tags_14 },
    { 0x9d28f8ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc708f8f8, SUC_lexicon_tags_49 },
    { 0xdae8f8f9, SUC_lexicon_tags_0 },
    { 0x8708f8fa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa228f8ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48f905, SUC_lexicon_tags_15 },
    { 0x7008f906, SUC_lexicon_tags_29 },
    { 0x3d48f906, SUC_lexicon_tags_15 },
    { 0x2fe8f908, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a48f90b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb08f90d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a8f912, SUC_lexicon_tags_30 },
    { 0xf6e8f913, SUC_lexicon_tags_37 },
    { 0x4988f914, SUC_lexicon_tags_45 },
    { 0x79a8f915, SUC_lexicon_tags_31 },
    { 0xdb28f914, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e8f919, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a8f921, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1968f926, SUC_lexicon_tags_25 },
    { 0x2da8f927, SUC_lexicon_tags_23 },
    { 0x42a8f928, SUC_lexicon_tags_61 },
    { 0xaee8f929, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb88f92b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8688f92e, SUC_lexicon_tags_38 },
    { 0x1368f92e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa128f933, SUC_lexicon_tags_21 },
    { 0x4ae8f933, SUC_lexicon_tags_37 },
    { 0xb7c8f933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a8f939, SUC_lexicon_tags_37 },
    { 0xa108f939, SUC_lexicon_tags_30 },
    { 0xfe08f93b, SUC_lexicon_tags_25 },
    { 0xe528f93c, SUC_lexicon_tags_7 },
    { 0xb788f93c, SUC_lexicon_tags_127 },
    { 0x148f93d, SUC_lexicon_tags_7 },
    { 0x4c08f93d, SUC_lexicon_tags_25 },
    { 0xea68f939, SUC_lexicon_tags_14 },
    { 0xea08f941, SUC_lexicon_tags_30 },
    { 0xa948f942, SUC_lexicon_tags_45 },
    { 0xa3a8f943, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x668f946, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a8f949, SUC_lexicon_tags_37 },
    { 0xc1a8f94a, SUC_lexicon_tags_5 },
    { 0xd848f94b, SUC_lexicon_tags_12 },
    { 0x16c8f94b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be8f94f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c8f952, SUC_lexicon_tags_11 },
    { 0xafe8f953, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3308f955, SUC_lexicon_tags_17 },
    { 0xb28f956, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe048f959, SUC_lexicon_tags_45 },
    { 0xaba8f959, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93c8f95c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e8f961, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea8f964, SUC_lexicon_tags_11 },
    { 0xbc88f964, SUC_lexicon_tags_68 },
    { 0xbd88f966, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb708f975, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4948f978, SUC_lexicon_tags_63 },
    { 0x18c8f978, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c8f97b, SUC_lexicon_tags_2 },
    { 0xb588f97c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa728f980, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8008f987, SUC_lexicon_tags_17 },
    { 0x3e28f987, SUC_lexicon_tags_0 },
    { 0xa888f989, SUC_lexicon_tags_11 },
    { 0xd308f989, SUC_lexicon_tags_1 },
    { 0x2e08f98b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf3e8f98d, SUC_lexicon_tags_17 },
    { 0xcf68f98e, SUC_lexicon_tags_74 },
    { 0xfe08f98f, SUC_lexicon_tags_63 },
    { 0x2c88f98f, SUC_lexicon_tags_14 },
    { 0x1c28f991, SUC_lexicon_tags_63 },
    { 0x36a8f992, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e8f995, SUC_lexicon_tags_8 },
    { 0xa2e8f996, SUC_lexicon_tags_8 },
    { 0x6ec8f997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da8f99a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa208f99c, SUC_lexicon_tags_7 },
    { 0x9848f99d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c8f9a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e8f9a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f88f9a8, SUC_lexicon_tags_12 },
    { 0x2d08f9a9, SUC_lexicon_tags_12 },
    { 0xf988f9aa, SUC_lexicon_tags_25 },
    { 0x28a8f9a9, SUC_lexicon_tags_55 },
    { 0xb408f9aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa8f9ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d48f9b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5bc8f9b2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b88f9b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f28f9b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6a68f9ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb768f9be, SUC_lexicon_tags_25 },
    { 0x97e8f9bf, SUC_lexicon_tags_30 },
    { 0x9828f9c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88c8f9c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1588f9c5, SUC_lexicon_tags_12 },
    { 0xde08f9c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa8f9c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad08f9ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab28f9cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b88f9d3, SUC_lexicon_tags_17 },
    { 0x98a8f9d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa868f9d7, SUC_lexicon_tags_46 },
    { 0x4688f9d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a68f9dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c88f9e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6388f9e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8108f9eb, SUC_lexicon_tags_17 },
    { 0x28e8f9eb, SUC_lexicon_tags_63 },
    { 0xd948f9eb, SUC_lexicon_tags_25 },
    { 0xc948f9ee, SUC_lexicon_tags_87 },
    { 0x6f88f9ef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6208f9fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a28fa02, SUC_lexicon_tags_7 },
    { 0xb2e8fa03, SUC_lexicon_tags_49 },
    { 0xbca8fa03, SUC_lexicon_tags_48 },
    { 0xae48fa05, SUC_lexicon_tags_4 },
    { 0x2568fa04, SUC_lexicon_tags_11 },
    { 0x2e48fa07, SUC_lexicon_tags_31 },
    { 0xd468fa07, SUC_lexicon_tags_27 },
    { 0x96a8fa07, SUC_lexicon_tags_23 },
    { 0xec8fa07, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e8fa0f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4348fa11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e8fa17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4568fa1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a8fa1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a68fa22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa8fa25, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x9588fa27, SUC_lexicon_tags_3 },
    { 0x3048fa28, SUC_lexicon_tags_32 },
    { 0x3ba8fa28, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb428fa32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c8fa38, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3ee8fa3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbcc8fa3c, SUC_lexicon_tags_27 },
    { 0xd4c8fa3c, SUC_lexicon_tags_37 },
    { 0x9aa8fa3e, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea68fa52, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b88fa54, SUC_lexicon_tags_170 },
    { 0x7108fa55, SUC_lexicon_tags_57 },
    { 0xce08fa56, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d28fa5d, SUC_lexicon_tags_21 },
    { 0x49a8fa5e, SUC_lexicon_tags_26 },
    { 0x9e48fa5e, SUC_lexicon_tags_47 },
    { 0x1c68fa60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc608fa65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc748fa69, SUC_lexicon_tags_14 },
    { 0x4e68fa6a, SUC_lexicon_tags_9 },
    { 0x8408fa6b, SUC_lexicon_tags_37 },
    { 0x6248fa6b, SUC_lexicon_tags_37 },
    { 0xa6e8fa6d, SUC_lexicon_tags_27 },
    { 0xc648fa6d, SUC_lexicon_tags_47 },
    { 0xa368fa6f, SUC_lexicon_tags_17 },
    { 0x4fe8fa6b, SUC_lexicon_tags_70 },
    { 0xeb68fa6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48fa74, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9068fa7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e8fa7e, SUC_lexicon_tags_7 },
    { 0x2728fa7f, SUC_lexicon_tags_14 },
    { 0xbdc8fa80, SUC_lexicon_tags_55 },
    { 0xb8c8fa7e, SUC_lexicon_tags_15 },
    { 0xa028fa82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa8fa85, SUC_lexicon_tags_63 },
    { 0x7648fa86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a28fa8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe308fa93, SUC_lexicon_tags_17 },
    { 0x7068fa93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33c8fa96, SUC_lexicon_tags_40 },
    { 0xecc8fa96, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b48fa9c, SUC_lexicon_tags_306 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a8faa0, SUC_lexicon_tags_65 },
    { 0x6ba8faa0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1208faa4, SUC_lexicon_tags_0 },
    { 0x1e08faa4, SUC_lexicon_tags_0 },
    { 0x3648faa6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d28faa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe428faab, SUC_lexicon_tags_25 },
    { 0xd808faab, SUC_lexicon_tags_1 },
    { 0xa648faac, SUC_lexicon_tags_9 },
    { 0x3488faad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1408fab0, SUC_lexicon_tags_12 },
    { 0x468fab1, SUC_lexicon_tags_25 },
    { 0x9768fab2, SUC_lexicon_tags_66 },
    { 0x67a8fab3, SUC_lexicon_tags_37 },
    { 0x1d08fab4, SUC_lexicon_tags_8 },
    { 0xc028fab2, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0, NULL },
    { 0x4148fab8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d08fabc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c8fabf, SUC_lexicon_tags_31 },
    { 0x3408fac0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb528fac4, SUC_lexicon_tags_40 },
    { 0x2e28fac5, SUC_lexicon_tags_25 },
    { 0x6d68fac6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5528facb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c28face, SUC_lexicon_tags_4 },
    { 0xcdc8face, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7468fad1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9048fad7, SUC_lexicon_tags_23 },
    { 0xe4e8fad7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa8fadd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69c8fadf, SUC_lexicon_tags_7 },
    { 0xf6a8fadf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xea48fae3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x83e8fae5, SUC_lexicon_tags_15 },
    { 0x3628fae6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a8fae8, SUC_lexicon_tags_37 },
    { 0xe808fae9, SUC_lexicon_tags_28 },
    { 0xe2a8fae9, SUC_lexicon_tags_47 },
    { 0x4908faeb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde28faf0, SUC_lexicon_tags_33 },
    { 0xd0c8faf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf108faf9, SUC_lexicon_tags_1 },
    { 0x5e8fafa, SUC_lexicon_tags_43 },
    { 0xafc8fafb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd708fb01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe708fb03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc8fb07, SUC_lexicon_tags_27 },
    { 0x1828fb08, SUC_lexicon_tags_2 },
    { 0x8fa8fb07, SUC_lexicon_tags_188 },
    { 0x16e8fb0a, SUC_lexicon_tags_2 },
    { 0x5da8fb0b, SUC_lexicon_tags_37 },
    { 0xa8c8fb0c, SUC_lexicon_tags_15 },
    { 0x5848fb0d, SUC_lexicon_tags_11 },
    { 0xdfe8fb0d, SUC_lexicon_tags_12 },
    { 0x2c68fb0f, SUC_lexicon_tags_60 },
    { 0x3f88fb0d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f88fb14, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a8fb17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a8fb1a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe628fb1e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x288fb20, SUC_lexicon_tags_12 },
    { 0x7ee8fb20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c8fb2d, SUC_lexicon_tags_7 },
    { 0xa468fb2d, SUC_lexicon_tags_63 },
    { 0x3148fb2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a8fb37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8248fb3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f88fb45, SUC_lexicon_tags_15 },
    { 0x9b88fb46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf068fb4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6548fb4e, SUC_lexicon_tags_29 },
    { 0x8308fb4f, SUC_lexicon_tags_43 },
    { 0x3ce8fb50, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd028fb56, SUC_lexicon_tags_27 },
    { 0xc08fb57, SUC_lexicon_tags_26 },
    { 0xaae8fb58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9668fb5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c48fb61, SUC_lexicon_tags_37 },
    { 0xe8a8fb61, SUC_lexicon_tags_38 },
    { 0xe1c8fb63, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb148fb68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e8fb6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc8fb6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5348fb76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708fb7a, SUC_lexicon_tags_23 },
    { 0x6948fb7b, SUC_lexicon_tags_7 },
    { 0xd488fb7c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa688fb7e, SUC_lexicon_tags_7 },
    { 0xefc8fb7f, SUC_lexicon_tags_37 },
    { 0x9508fb7e, SUC_lexicon_tags_92 },
    { 0x7908fb81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf68fb84, SUC_lexicon_tags_15 },
    { 0x7c88fb84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c28fb88, SUC_lexicon_tags_16 },
    { 0x8708fb88, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9828fb8b, SUC_lexicon_tags_60 },
    { 0x2a68fb8c, SUC_lexicon_tags_2 },
    { 0xb408fb8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a28fb91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a8fb99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf68fb9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9808fba2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe788fba7, SUC_lexicon_tags_37 },
    { 0xc088fba7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc8fbab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd228fbad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x85e8fbaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e88fbb2, SUC_lexicon_tags_15 },
    { 0x88a8fbb3, SUC_lexicon_tags_37 },
    { 0x9ca8fbb2, SUC_lexicon_tags_17 },
    { 0x6e28fbb4, SUC_lexicon_tags_1 },
    { 0x73c8fbb3, SUC_lexicon_tags_15 },
    { 0xbc68fbb6, SUC_lexicon_tags_17 },
    { 0x51a8fbb5, SUC_lexicon_tags_8 },
    { 0x4e88fbb9, SUC_lexicon_tags_47 },
    { 0xa2e8fbba, SUC_lexicon_tags_17 },
    { 0x22c8fbbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c8fbbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88fbc3, SUC_lexicon_tags_27 },
    { 0x48c8fbc4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d48fbc8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2488fbce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4608fbd1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e8fbd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e8fbd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5948fbdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27e8fbdd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce28fbe2, SUC_lexicon_tags_17 },
    { 0x2628fbe3, SUC_lexicon_tags_45 },
    { 0x3b68fbe4, SUC_lexicon_tags_62 },
    { 0x4da8fbe5, SUC_lexicon_tags_21 },
    { 0xa788fbe6, SUC_lexicon_tags_7 },
    { 0x3a28fbe7, SUC_lexicon_tags_17 },
    { 0xb228fbe6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7108fbea, SUC_lexicon_tags_1 },
    { 0x3a68fbea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba68fbf4, SUC_lexicon_tags_38 },
    { 0x2088fbf5, SUC_lexicon_tags_25 },
    { 0x6808fbf4, SUC_lexicon_tags_23 },
    { 0x2308fbf7, SUC_lexicon_tags_17 },
    { 0xf3e8fbf7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe8fbfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9008fc01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe008fc04, SUC_lexicon_tags_23 },
    { 0x5908fc05, SUC_lexicon_tags_17 },
    { 0x5288fc06, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4008fc09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7388fc0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce8fc0f, SUC_lexicon_tags_25 },
    { 0xbe68fc0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4088fc14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a8fc18, SUC_lexicon_tags_25 },
    { 0x4f28fc19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8bc8fc1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8fc22, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0xd28fc24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8a8fc26, SUC_lexicon_tags_11 },
    { 0x1fa8fc26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1a8fc29, SUC_lexicon_tags_17 },
    { 0xd748fc2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2e8fc2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf448fc32, SUC_lexicon_tags_27 },
    { 0x6b68fc33, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2848fc35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e8fc39, SUC_lexicon_tags_15 },
    { 0x8aa8fc3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x388fc3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc48fc44, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e8fc47, SUC_lexicon_tags_30 },
    { 0xaa68fc48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb528fc4d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5448fc51, SUC_lexicon_tags_23 },
    { 0x25a8fc52, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e8fc57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8808fc5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x728fc5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde28fc61, SUC_lexicon_tags_65 },
    { 0x9448fc62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c8fc66, SUC_lexicon_tags_11 },
    { 0xd1a8fc66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa8fc6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c8fc72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728fc79, SUC_lexicon_tags_15 },
    { 0x5f88fc79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3668fc7c, SUC_lexicon_tags_12 },
    { 0x91e8fc7c, SUC_lexicon_tags_112 },
    { 0xd268fc7e, SUC_lexicon_tags_2 },
    { 0xfc8fc7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x948fc81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd88fc83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c08fc87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a8fc8a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e8fc8f, SUC_lexicon_tags_17 },
    { 0x70e8fc90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe788fc92, SUC_lexicon_tags_35 },
    { 0x5248fc93, SUC_lexicon_tags_45 },
    { 0xac8fc94, SUC_lexicon_tags_46 },
    { 0xd4e8fc92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada8fc9e, SUC_lexicon_tags_2 },
    { 0x13a8fc9f, SUC_lexicon_tags_38 },
    { 0x9088fc9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f28fca2, SUC_lexicon_tags_11 },
    { 0x43a8fca2, SUC_lexicon_tags_38 },
    { 0x57e8fca4, SUC_lexicon_tags_37 },
    { 0xbd48fca5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb88fca8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e8fcab, SUC_lexicon_tags_17 },
    { 0xfe68fcac, SUC_lexicon_tags_13 },
    { 0xd148fcad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6888fcaf, SUC_lexicon_tags_45 },
    { 0x7808fcb0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3288fcb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda8fcb7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de8fcbc, SUC_lexicon_tags_7 },
    { 0x5f28fcbd, SUC_lexicon_tags_31 },
    { 0xcc68fcbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1668fcc3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc8fcc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e8fccb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x40e8fccd, SUC_lexicon_tags_101 },
    { 0xb2c8fccd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bc8fcd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a8fcd5, SUC_lexicon_tags_9 },
    { 0x7c48fcd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea8fcda, SUC_lexicon_tags_6 },
    { 0x9248fcda, SUC_lexicon_tags_15 },
    { 0x14e8fcdc, SUC_lexicon_tags_37 },
    { 0x22c8fcdd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4148fcdf, SUC_lexicon_tags_11 },
    { 0x4b88fcdf, SUC_lexicon_tags_25 },
    { 0xcf28fce1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd48fce5, SUC_lexicon_tags_2 },
    { 0xe1c8fce6, SUC_lexicon_tags_12 },
    { 0xb948fce6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d68fcef, SUC_lexicon_tags_2 },
    { 0x8cc8fcf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e8fcf3, SUC_lexicon_tags_47 },
    { 0xdd88fcf4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe588fcf6, SUC_lexicon_tags_23 },
    { 0xd808fcf6, SUC_lexicon_tags_25 },
    { 0x4168fcf8, SUC_lexicon_tags_15 },
    { 0x5ce8fcf9, SUC_lexicon_tags_1 },
    { 0xfe28fcfa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xae28fcfc, SUC_lexicon_tags_17 },
    { 0x92c8fcfd, SUC_lexicon_tags_4 },
    { 0xcc08fcfe, SUC_lexicon_tags_44 },
    { 0x4868fcfc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e08fd04, SUC_lexicon_tags_25 },
    { 0x91a8fd05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1568fd09, SUC_lexicon_tags_63 },
    { 0xb448fd0a, SUC_lexicon_tags_6 },
    { 0xdc28fd09, SUC_lexicon_tags_9 },
    { 0x9388fd0c, SUC_lexicon_tags_37 },
    { 0xa488fd0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e88fd13, SUC_lexicon_tags_2 },
    { 0xac68fd14, SUC_lexicon_tags_11 },
    { 0x9888fd14, SUC_lexicon_tags_4 },
    { 0x73e8fd16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48c8fd19, SUC_lexicon_tags_25 },
    { 0xef08fd19, SUC_lexicon_tags_77 },
    { 0xbe48fd1b, SUC_lexicon_tags_2 },
    { 0x8108fd1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f48fd1e, SUC_lexicon_tags_77 },
    { 0x4648fd1f, SUC_lexicon_tags_25 },
    { 0x3968fd20, SUC_lexicon_tags_25 },
    { 0x6308fd1f, SUC_lexicon_tags_19 },
    { 0xd208fd21, SUC_lexicon_tags_49 },
    { 0x1b08fd22, SUC_lexicon_tags_63 },
    { 0x5928fd24, SUC_lexicon_tags_87 },
    { 0xdfc8fd24, SUC_lexicon_tags_61 },
    { 0xfa8fd20, SUC_lexicon_tags_45 },
    { 0xd6e8fd24, SUC_lexicon_tags_0 },
    { 0xf9c8fd28, SUC_lexicon_tags_15 },
    { 0xad88fd29, SUC_lexicon_tags_12 },
    { 0x5ea8fd28, SUC_lexicon_tags_15 },
    { 0xc4e8fd29, SUC_lexicon_tags_37 },
    { 0xaa08fd28, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf088fd2e, SUC_lexicon_tags_63 },
    { 0xbd08fd2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c68fd35, SUC_lexicon_tags_15 },
    { 0x8648fd36, SUC_lexicon_tags_37 },
    { 0x2708fd37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8728fd3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6008fd3d, SUC_lexicon_tags_31 },
    { 0xdc48fd3d, SUC_lexicon_tags_29 },
    { 0xb1a8fd3e, SUC_lexicon_tags_7 },
    { 0x20e8fd40, SUC_lexicon_tags_12 },
    { 0x47e8fd40, SUC_lexicon_tags_12 },
    { 0x648fd42, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a8fd47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdb48fd49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1148fd4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a88fd4f, SUC_lexicon_tags_7 },
    { 0xb3e8fd4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa908fd52, SUC_lexicon_tags_41 },
    { 0xc5a8fd53, SUC_lexicon_tags_25 },
    { 0xd768fd54, SUC_lexicon_tags_62 },
    { 0x15a8fd55, SUC_lexicon_tags_261 },
    { 0x2fa8fd52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4588fd5b, SUC_lexicon_tags_7 },
    { 0x6ca8fd5b, SUC_lexicon_tags_66 },
    { 0x9868fd5b, SUC_lexicon_tags_15 },
    { 0xd828fd5e, SUC_lexicon_tags_25 },
    { 0xada8fd5e, SUC_lexicon_tags_4 },
    { 0xcaa8fd5e, SUC_lexicon_tags_37 },
    { 0xfa88fd61, SUC_lexicon_tags_19 },
    { 0xc048fd61, SUC_lexicon_tags_27 },
    { 0x4928fd61, SUC_lexicon_tags_80 },
    { 0xb1e8fd64, SUC_lexicon_tags_7 },
    { 0x7028fd65, SUC_lexicon_tags_48 },
    { 0x3a8fd66, SUC_lexicon_tags_17 },
    { 0xb648fd66, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0xdbc8fd69, SUC_lexicon_tags_11 },
    { 0xf028fd69, SUC_lexicon_tags_15 },
    { 0x14c8fd6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6988fd6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe708fd6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7248fd71, SUC_lexicon_tags_8 },
    { 0x6368fd72, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e8fd7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf468fd81, SUC_lexicon_tags_7 },
    { 0x1f28fd82, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec8fd88, SUC_lexicon_tags_17 },
    { 0x2b48fd89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf208fd9c, SUC_lexicon_tags_15 },
    { 0x7ae8fd9d, SUC_lexicon_tags_6 },
    { 0x82a8fd9e, SUC_lexicon_tags_30 },
    { 0x1a8fd9f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e8fda5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8748fdaa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd48fdad, SUC_lexicon_tags_45 },
    { 0x23c8fdae, SUC_lexicon_tags_23 },
    { 0xf408fdaf, SUC_lexicon_tags_37 },
    { 0x3788fdb0, SUC_lexicon_tags_139 },
    { 0xd68fdb0, SUC_lexicon_tags_49 },
    { 0x9be8fdb2, SUC_lexicon_tags_11 },
    { 0xad68fdb3, SUC_lexicon_tags_2 },
    { 0xbc8fdb4, SUC_lexicon_tags_6 },
    { 0x1088fdb5, SUC_lexicon_tags_39 },
    { 0x5368fdb4, SUC_lexicon_tags_87 },
    { 0xcc68fdaf, SUC_lexicon_tags_23 },
    { 0xd708fdb8, SUC_lexicon_tags_8 },
    { 0x7868fdb6, SUC_lexicon_tags_39 },
    { 0x1f88fdba, SUC_lexicon_tags_17 },
    { 0xf7c8fdb9, SUC_lexicon_tags_17 },
    { 0x1cc8fdae, SUC_lexicon_tags_15 },
    { 0x31e8fdbd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8168fdc0, SUC_lexicon_tags_12 },
    { 0x8b48fdc1, SUC_lexicon_tags_45 },
    { 0x23a8fdc2, SUC_lexicon_tags_19 },
    { 0xcca8fdc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6e8fdc5, SUC_lexicon_tags_43 },
    { 0x1aa8fdc6, SUC_lexicon_tags_148 },
    { 0x97a8fdc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba8fdcb, SUC_lexicon_tags_30 },
    { 0xcbc8fdcc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe988fdcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6908fdd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc8fdda, SUC_lexicon_tags_1 },
    { 0xbfe8fdda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3088fde1, SUC_lexicon_tags_29 },
    { 0x928fde2, SUC_lexicon_tags_14 },
    { 0x1fe8fde3, SUC_lexicon_tags_2 },
    { 0xa208fde4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a8fde6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe248fde9, SUC_lexicon_tags_11 },
    { 0x8d28fde9, SUC_lexicon_tags_25 },
    { 0xe828fdeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a8fdee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec28fdf2, SUC_lexicon_tags_23 },
    { 0x368fdf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6748fdfa, SUC_lexicon_tags_23 },
    { 0x27e8fdfb, SUC_lexicon_tags_17 },
    { 0x4aa8fdfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5828fdff, SUC_lexicon_tags_37 },
    { 0x6408fe00, SUC_lexicon_tags_60 },
    { 0x6908fe00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7088fe03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca8fe0b, SUC_lexicon_tags_2 },
    { 0xb448fe0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e8fe0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7b68fe11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c8fe17, SUC_lexicon_tags_19 },
    { 0xe2e8fe17, SUC_lexicon_tags_29 },
    { 0xa288fe19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca8fe1c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08fe1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a8fe22, SUC_lexicon_tags_19 },
    { 0xa628fe22, SUC_lexicon_tags_9 },
    { 0x2d28fe24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5088fe2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb48fe2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a8fe33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7748fe38, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc8fe3e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc8fe44, SUC_lexicon_tags_0 },
    { 0x7808fe45, SUC_lexicon_tags_32 },
    { 0x5608fe44, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a8fe4c, SUC_lexicon_tags_25 },
    { 0xd008fe4c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8388fe4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1168fe51, SUC_lexicon_tags_17 },
    { 0x3948fe52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6168fe55, SUC_lexicon_tags_25 },
    { 0x2928fe56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6568fe5a, SUC_lexicon_tags_65 },
    { 0x15c8fe5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb708fe5d, SUC_lexicon_tags_37 },
    { 0xcec8fe5d, SUC_lexicon_tags_34 },
    { 0xbf88fe5f, SUC_lexicon_tags_37 },
    { 0x7dc8fe5f, SUC_lexicon_tags_37 },
    { 0x4be8fe60, SUC_lexicon_tags_31 },
    { 0x5ae8fe62, SUC_lexicon_tags_8 },
    { 0x58e8fe63, SUC_lexicon_tags_17 },
    { 0xdbc8fe62, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a8fe6a, SUC_lexicon_tags_2 },
    { 0x4148fe6a, SUC_lexicon_tags_25 },
    { 0x9e48fe6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb48fe6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a8fe73, SUC_lexicon_tags_32 },
    { 0x9668fe74, SUC_lexicon_tags_45 },
    { 0x76a8fe73, SUC_lexicon_tags_17 },
    { 0x3e8fe76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d48fe7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b28fe81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c68fe86, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e8fe90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4928fe93, SUC_lexicon_tags_27 },
    { 0xe2e8fe94, SUC_lexicon_tags_2 },
    { 0xd008fe95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae8fe9c, SUC_lexicon_tags_7 },
    { 0x7bc8fe9d, SUC_lexicon_tags_43 },
    { 0x8c8fe9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb888fea0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e8fea5, SUC_lexicon_tags_17 },
    { 0x8548fea6, SUC_lexicon_tags_6 },
    { 0xab28fea7, SUC_lexicon_tags_25 },
    { 0x4188fea7, SUC_lexicon_tags_14 },
    { 0x3328fea8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x65e8feab, SUC_lexicon_tags_47 },
    { 0xd4a8feab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbbc8feae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e8feb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8feb4, SUC_lexicon_tags_6 },
    { 0x7d68feb5, SUC_lexicon_tags_25 },
    { 0xda48feb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b48feb8, SUC_lexicon_tags_4 },
    { 0xe888feb8, SUC_lexicon_tags_19 },
    { 0xf28feba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cc8febc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fec3, SUC_lexicon_tags_31 },
    { 0xbca8fec4, SUC_lexicon_tags_25 },
    { 0x5488fec5, SUC_lexicon_tags_33 },
    { 0xba68fec6, SUC_lexicon_tags_7 },
    { 0x57a8fec7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c8fece, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9808fed3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2468fed6, SUC_lexicon_tags_1 },
    { 0x7248fed7, SUC_lexicon_tags_19 },
    { 0xb8e8fed7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf408fedf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd068fee7, SUC_lexicon_tags_65 },
    { 0x9148fee8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3408feec, SUC_lexicon_tags_7 },
    { 0x3c8feed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8fef4, SUC_lexicon_tags_17 },
    { 0x8b48fef5, SUC_lexicon_tags_47 },
    { 0x48fef5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e8fefc, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea8ff04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72e8ff07, SUC_lexicon_tags_60 },
    { 0x28ff07, SUC_lexicon_tags_613 },
    { 0x82c8ff07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3268ff0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d08ff10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e88ff16, SUC_lexicon_tags_38 },
    { 0x7fa8ff16, SUC_lexicon_tags_11 },
    { 0xeb88ff18, SUC_lexicon_tags_37 },
    { 0x75e8ff19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95a8ff1b, SUC_lexicon_tags_31 },
    { 0xffa8ff1c, SUC_lexicon_tags_23 },
    { 0x8d28ff1c, SUC_lexicon_tags_2 },
    { 0x8f48ff1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5dc8ff20, SUC_lexicon_tags_11 },
    { 0x4e28ff21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb88ff23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc428ff28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd408ff2a, SUC_lexicon_tags_38 },
    { 0x9b68ff2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d08ff2f, SUC_lexicon_tags_35 },
    { 0x1868ff30, SUC_lexicon_tags_234 },
    { 0xe0e8ff31, SUC_lexicon_tags_25 },
    { 0xfba8ff30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe68ff34, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x91e8ff36, SUC_lexicon_tags_30 },
    { 0xd7c8ff37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde8ff3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45a8ff40, SUC_lexicon_tags_23 },
    { 0xd5e8ff40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc8ff44, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2608ff4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68ff4d, SUC_lexicon_tags_15 },
    { 0xc008ff4d, SUC_lexicon_tags_38 },
    { 0x6f28ff4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc8ff54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e8ff58, SUC_lexicon_tags_37 },
    { 0xa7a8ff58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf768ff5b, SUC_lexicon_tags_37 },
    { 0x9828ff5c, SUC_lexicon_tags_12 },
    { 0x1a48ff5b, SUC_lexicon_tags_28 },
    { 0xe888ff5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a8ff61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7348ff63, SUC_lexicon_tags_47 },
    { 0xfd28ff64, SUC_lexicon_tags_21 },
    { 0xbdc8ff63, SUC_lexicon_tags_37 },
    { 0x26e8ff66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe108ff69, SUC_lexicon_tags_0 },
    { 0xa0a8ff6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb108ff6d, SUC_lexicon_tags_31 },
    { 0x7ee8ff6d, SUC_lexicon_tags_32 },
    { 0xad28ff6f, SUC_lexicon_tags_1 },
    { 0xa1a8ff70, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x9388ff72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a88ff74, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a8ff79, SUC_lexicon_tags_25 },
    { 0xa928ff7a, SUC_lexicon_tags_43 },
    { 0x8fa8ff7b, SUC_lexicon_tags_14 },
    { 0xbfc8ff7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e28ff7f, SUC_lexicon_tags_27 },
    { 0xfb68ff7f, SUC_lexicon_tags_17 },
    { 0xd108ff80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08ff87, SUC_lexicon_tags_7 },
    { 0xc1e8ff88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d48ff8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa288ff92, SUC_lexicon_tags_30 },
    { 0x3f28ff92, SUC_lexicon_tags_19 },
    { 0xade8ff94, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf948ff9c, SUC_lexicon_tags_1 },
    { 0x5328ff9d, SUC_lexicon_tags_65 },
    { 0x9fe8ff9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf48ffa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5c8ffa2, SUC_lexicon_tags_2 },
    { 0xb4c8ffa3, SUC_lexicon_tags_11 },
    { 0xe848ffa4, SUC_lexicon_tags_37 },
    { 0xea68ffa4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x51c8ffa7, SUC_lexicon_tags_25 },
    { 0x5a48ffa8, SUC_lexicon_tags_2 },
    { 0xabc8ffa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fe8ffab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc88ffaf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c8ffb2, SUC_lexicon_tags_17 },
    { 0x9d88ffb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde88ffb5, SUC_lexicon_tags_55 },
    { 0x7148ffb5, SUC_lexicon_tags_2 },
    { 0x32e8ffb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59e8ffbc, SUC_lexicon_tags_8 },
    { 0xdf88ffbd, SUC_lexicon_tags_37 },
    { 0xd988ffbe, SUC_lexicon_tags_5 },
    { 0x5dc8ffbf, SUC_lexicon_tags_37 },
    { 0x2328ffbf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xda68ffc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da8ffc6, SUC_lexicon_tags_27 },
    { 0x7928ffc7, SUC_lexicon_tags_2 },
    { 0xabc8ffc8, SUC_lexicon_tags_27 },
    { 0x448ffc9, SUC_lexicon_tags_37 },
    { 0x35c8ffc6, SUC_lexicon_tags_9 },
    { 0xf568ffcb, SUC_lexicon_tags_35 },
    { 0x9368ffcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6068ffd2, SUC_lexicon_tags_17 },
    { 0xb4c8ffd2, SUC_lexicon_tags_25 },
    { 0x6968ffd4, SUC_lexicon_tags_99 },
    { 0x9dc8ffd5, SUC_lexicon_tags_14 },
    { 0x66e8ffd6, SUC_lexicon_tags_28 },
    { 0xedc8ffd7, SUC_lexicon_tags_38 },
    { 0xea88ffd8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7368ffdb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ffde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8868ffe3, SUC_lexicon_tags_15 },
    { 0x6de8ffe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a28ffe8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa648ffeb, SUC_lexicon_tags_37 },
    { 0x6068ffec, SUC_lexicon_tags_15 },
    { 0xe5e8ffed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4448fffb, SUC_lexicon_tags_32 },
    { 0x4b28fffc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a90009, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc9000e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae90012, SUC_lexicon_tags_27 },
    { 0x81890012, SUC_lexicon_tags_45 },
    { 0xc7490014, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84490016, SUC_lexicon_tags_28 },
    { 0x48e90017, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e9001d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb409001f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe4490021, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e690027, SUC_lexicon_tags_43 },
    { 0x18e90027, SUC_lexicon_tags_116 },
    { 0x39c90029, SUC_lexicon_tags_25 },
    { 0xa0a9002a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4309002f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc90032, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b490034, SUC_lexicon_tags_25 },
    { 0xad490034, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee9003b, SUC_lexicon_tags_25 },
    { 0x6d29003c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490041, SUC_lexicon_tags_25 },
    { 0x5fc90042, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbca90044, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac690048, SUC_lexicon_tags_60 },
    { 0x41090049, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad49004b, SUC_lexicon_tags_17 },
    { 0x28a9004c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe9004f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c90054, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba290059, SUC_lexicon_tags_2 },
    { 0xb1c90059, SUC_lexicon_tags_8 },
    { 0xc7e90059, SUC_lexicon_tags_11 },
    { 0x509005c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c890060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a90064, SUC_lexicon_tags_62 },
    { 0xa7690065, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8a90067, SUC_lexicon_tags_64 },
    { 0x2c290067, SUC_lexicon_tags_45 },
    { 0x85890067, SUC_lexicon_tags_12 },
    { 0xd0490067, SUC_lexicon_tags_37 },
    { 0x8ae90069, SUC_lexicon_tags_60 },
    { 0xf2e90067, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7490070, SUC_lexicon_tags_8 },
    { 0x20090070, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb7290073, SUC_lexicon_tags_38 },
    { 0x9290074, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68490079, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70e9007b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2e9007d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8289007f, SUC_lexicon_tags_21 },
    { 0xd8290080, SUC_lexicon_tags_37 },
    { 0x71890081, SUC_lexicon_tags_12 },
    { 0xcba90082, SUC_lexicon_tags_17 },
    { 0x64490080, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba890086, SUC_lexicon_tags_29 },
    { 0x9cc90086, SUC_lexicon_tags_33 },
    { 0x18890088, SUC_lexicon_tags_12 },
    { 0x8f690089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a9008d, SUC_lexicon_tags_46 },
    { 0xf369008e, SUC_lexicon_tags_8 },
    { 0x7229008f, SUC_lexicon_tags_37 },
    { 0x8c090090, SUC_lexicon_tags_46 },
    { 0x27090090, SUC_lexicon_tags_39 },
    { 0x6129008e, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6189009b, SUC_lexicon_tags_11 },
    { 0xb2a9009c, SUC_lexicon_tags_25 },
    { 0x9149009d, SUC_lexicon_tags_12 },
    { 0x7a89009e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2900a8, SUC_lexicon_tags_7 },
    { 0x9b0900a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x410900ab, SUC_lexicon_tags_12 },
    { 0xfda900ab, SUC_lexicon_tags_7 },
    { 0x6a6900ab, SUC_lexicon_tags_37 },
    { 0x1ce900ad, SUC_lexicon_tags_17 },
    { 0x78900af, SUC_lexicon_tags_15 },
    { 0x3b4900ac, SUC_lexicon_tags_37 },
    { 0x3a6900ad, SUC_lexicon_tags_1 },
    { 0x268900b2, SUC_lexicon_tags_7 },
    { 0x3ba900b3, SUC_lexicon_tags_28 },
    { 0x394900b4, SUC_lexicon_tags_28 },
    { 0x224900b4, SUC_lexicon_tags_23 },
    { 0x33c900b6, SUC_lexicon_tags_2 },
    { 0x9c2900b5, SUC_lexicon_tags_11 },
    { 0xd7e900b8, SUC_lexicon_tags_17 },
    { 0xc16900ab, SUC_lexicon_tags_30 },
    { 0xc52900ba, SUC_lexicon_tags_25 },
    { 0x30a900bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4900bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x624900c1, SUC_lexicon_tags_13 },
    { 0x16c900c2, SUC_lexicon_tags_37 },
    { 0x2fe900c1, SUC_lexicon_tags_1 },
    { 0x438900c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae8900c6, SUC_lexicon_tags_27 },
    { 0x34c900c7, SUC_lexicon_tags_15 },
    { 0x982900c7, SUC_lexicon_tags_1 },
    { 0x6e2900c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4c900cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2900cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x14a900d2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8e6900d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc900d7, SUC_lexicon_tags_47 },
    { 0x39e900d8, SUC_lexicon_tags_43 },
    { 0xac8900d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa10900db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe900df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xba0900e1, SUC_lexicon_tags_28 },
    { 0x51a900e2, SUC_lexicon_tags_11 },
    { 0xe44900e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460900ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2900f2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb5a900f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x83a900f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x914900f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb58900fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e900fd, SUC_lexicon_tags_25 },
    { 0xa6e900fd, SUC_lexicon_tags_23 },
    { 0xe66900ff, SUC_lexicon_tags_12 },
    { 0xd1e90100, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc8a90102, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0xac690104, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8290107, SUC_lexicon_tags_10 },
    { 0xe8c90108, SUC_lexicon_tags_87 },
    { 0xbe90109, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3be9010b, SUC_lexicon_tags_34 },
    { 0x5709010b, SUC_lexicon_tags_37 },
    { 0xcbe9010d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90490112, SUC_lexicon_tags_38 },
    { 0x74c90113, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb490117, SUC_lexicon_tags_9 },
    { 0xa5e90117, SUC_lexicon_tags_7 },
    { 0x26290117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa589011d, SUC_lexicon_tags_989 },
    { 0, NULL },
    { 0x2f49011f, SUC_lexicon_tags_2 },
    { 0x4a490120, SUC_lexicon_tags_33 },
    { 0x4290120, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5090124, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5490126, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbda90128, SUC_lexicon_tags_12 },
    { 0xbac90129, SUC_lexicon_tags_1 },
    { 0xa949012a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11a9012c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c90131, SUC_lexicon_tags_77 },
    { 0x2e290131, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67290138, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x729013a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6749013e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1c90140, SUC_lexicon_tags_33 },
    { 0x74890140, SUC_lexicon_tags_26 },
    { 0x93e90142, SUC_lexicon_tags_12 },
    { 0x4ac90140, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x21a90145, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53290149, SUC_lexicon_tags_19 },
    { 0xdfe9014a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a090151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70090155, SUC_lexicon_tags_15 },
    { 0xff890156, SUC_lexicon_tags_42 },
    { 0x7b890157, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8609015a, SUC_lexicon_tags_127 },
    { 0xc929015b, SUC_lexicon_tags_2 },
    { 0x85c9015b, SUC_lexicon_tags_87 },
    { 0x75c9015a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x85090160, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7290163, SUC_lexicon_tags_25 },
    { 0x8a290164, SUC_lexicon_tags_5 },
    { 0x7ac90165, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c90167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1649016b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd290170, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9290176, SUC_lexicon_tags_17 },
    { 0x5c290177, SUC_lexicon_tags_17 },
    { 0x8c890176, SUC_lexicon_tags_79 },
    { 0xb9a90176, SUC_lexicon_tags_60 },
    { 0xd2a9017a, SUC_lexicon_tags_37 },
    { 0xf290178, SUC_lexicon_tags_43 },
    { 0x20090176, SUC_lexicon_tags_15 },
    { 0x3fe9017d, SUC_lexicon_tags_42 },
    { 0x5149017e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1890181, SUC_lexicon_tags_8 },
    { 0xd2290182, SUC_lexicon_tags_29 },
    { 0x85690182, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31490187, SUC_lexicon_tags_14 },
    { 0x2a890188, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e9018b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82c9018d, SUC_lexicon_tags_48 },
    { 0x7a69018e, SUC_lexicon_tags_45 },
    { 0x8fc9018f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82490192, SUC_lexicon_tags_45 },
    { 0xe1690193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85490196, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d29019a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e901a0, SUC_lexicon_tags_47 },
    { 0xc0901a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6901a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc901a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e0901ab, SUC_lexicon_tags_1 },
    { 0xd36901ac, SUC_lexicon_tags_55 },
    { 0x280901ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x18901b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0901b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc901ba, SUC_lexicon_tags_60 },
    { 0xe68901ba, SUC_lexicon_tags_17 },
    { 0x2ac901bc, SUC_lexicon_tags_15 },
    { 0x338901bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba901c1, SUC_lexicon_tags_29 },
    { 0x7aa901c2, SUC_lexicon_tags_65 },
    { 0xfc901c1, SUC_lexicon_tags_29 },
    { 0xa7a901c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c901c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b2901c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8a901cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2901d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be901d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e901d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa901db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30901df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc901e4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce901eb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2901ee, SUC_lexicon_tags_45 },
    { 0x3a8901ee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x10901f1, SUC_lexicon_tags_25 },
    { 0x898901f1, SUC_lexicon_tags_37 },
    { 0xd52901f2, SUC_lexicon_tags_15 },
    { 0x98e901f2, SUC_lexicon_tags_17 },
    { 0x486901f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2c4901f7, SUC_lexicon_tags_15 },
    { 0x1e901f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62901fb, SUC_lexicon_tags_37 },
    { 0x2a4901fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a90202, SUC_lexicon_tags_8 },
    { 0x44490202, SUC_lexicon_tags_2 },
    { 0x8b890204, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc9020a, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd409020e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e90214, SUC_lexicon_tags_7 },
    { 0xea90215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb549021b, SUC_lexicon_tags_12 },
    { 0x409021c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x27e9021e, SUC_lexicon_tags_7 },
    { 0x2f89021e, SUC_lexicon_tags_17 },
    { 0x3a89021e, SUC_lexicon_tags_43 },
    { 0x9b29021e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa90224, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad290226, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc9090228, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8489022a, SUC_lexicon_tags_7 },
    { 0x6369022a, SUC_lexicon_tags_16 },
    { 0xdde9022b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69890230, SUC_lexicon_tags_82 },
    { 0x9890231, SUC_lexicon_tags_11 },
    { 0x4a490232, SUC_lexicon_tags_39 },
    { 0xd5290233, SUC_lexicon_tags_53 },
    { 0xfee90234, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2090237, SUC_lexicon_tags_17 },
    { 0xb0490237, SUC_lexicon_tags_11 },
    { 0xeba90239, SUC_lexicon_tags_12 },
    { 0x7e9023a, SUC_lexicon_tags_33 },
    { 0xc09023b, SUC_lexicon_tags_79 },
    { 0xcee9023b, SUC_lexicon_tags_77 },
    { 0xea9023b, SUC_lexicon_tags_7 },
    { 0x6ea9023c, SUC_lexicon_tags_0 },
    { 0xec090238, SUC_lexicon_tags_11 },
    { 0x13c90240, SUC_lexicon_tags_0 },
    { 0x40690241, SUC_lexicon_tags_6 },
    { 0xe90242, SUC_lexicon_tags_1 },
    { 0x1669023a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e90246, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2969024a, SUC_lexicon_tags_37 },
    { 0xb529024b, SUC_lexicon_tags_2 },
    { 0x4ba9024a, SUC_lexicon_tags_38 },
    { 0x2b29024b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c90250, SUC_lexicon_tags_45 },
    { 0xda90251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d290253, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea90256, SUC_lexicon_tags_2 },
    { 0x99890257, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba90261, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x45e90263, SUC_lexicon_tags_17 },
    { 0xa3890264, SUC_lexicon_tags_17 },
    { 0x97690264, SUC_lexicon_tags_63 },
    { 0x41690266, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b290269, SUC_lexicon_tags_30 },
    { 0x4c49026a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9909026c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68490272, SUC_lexicon_tags_17 },
    { 0x7fc90273, SUC_lexicon_tags_38 },
    { 0xee090274, SUC_lexicon_tags_37 },
    { 0xa3e90275, SUC_lexicon_tags_696 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec09027b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c90282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf090286, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c9028b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee89028e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb2290290, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a90293, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7290296, SUC_lexicon_tags_35 },
    { 0xffe90297, SUC_lexicon_tags_31 },
    { 0x1d490298, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa389029c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2902a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc902a5, SUC_lexicon_tags_33 },
    { 0x5d0902a6, SUC_lexicon_tags_7 },
    { 0xc62902a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x774902a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe08902ac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc0e902ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x794902b0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e902b5, SUC_lexicon_tags_7 },
    { 0xf24902b6, SUC_lexicon_tags_7 },
    { 0x4da902b7, SUC_lexicon_tags_39 },
    { 0xbd6902b8, SUC_lexicon_tags_46 },
    { 0xea0902b9, SUC_lexicon_tags_1 },
    { 0xff0902b8, SUC_lexicon_tags_7 },
    { 0x8a902ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc902c3, SUC_lexicon_tags_2 },
    { 0x988902c3, SUC_lexicon_tags_64 },
    { 0xbec902c5, SUC_lexicon_tags_1 },
    { 0xb18902c3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda902c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2902ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbe902d0, SUC_lexicon_tags_15 },
    { 0x30c902d1, SUC_lexicon_tags_25 },
    { 0x820902d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76902d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c902dd, SUC_lexicon_tags_19 },
    { 0xd4e902dd, SUC_lexicon_tags_37 },
    { 0xbb0902de, SUC_lexicon_tags_25 },
    { 0x170902e0, SUC_lexicon_tags_7 },
    { 0x258902e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc56902e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc902e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0902e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c902ed, SUC_lexicon_tags_34 },
    { 0x616902ed, SUC_lexicon_tags_16 },
    { 0x56902ee, SUC_lexicon_tags_45 },
    { 0xcc0902f0, SUC_lexicon_tags_0 },
    { 0xe6c902f1, SUC_lexicon_tags_60 },
    { 0xbc902f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x908902f4, SUC_lexicon_tags_40 },
    { 0x644902f4, SUC_lexicon_tags_19 },
    { 0xd50902f5, SUC_lexicon_tags_12 },
    { 0xc4a902f5, SUC_lexicon_tags_16 },
    { 0x22a902f8, SUC_lexicon_tags_6 },
    { 0x59c902f4, SUC_lexicon_tags_29 },
    { 0x342902fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbe902fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x632902fe, SUC_lexicon_tags_11 },
    { 0x980902fe, SUC_lexicon_tags_9 },
    { 0x62890300, SUC_lexicon_tags_6 },
    { 0x51890301, SUC_lexicon_tags_37 },
    { 0x49290302, SUC_lexicon_tags_0 },
    { 0x31090301, SUC_lexicon_tags_17 },
    { 0x35090304, SUC_lexicon_tags_1 },
    { 0xf7c90304, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f09030a, SUC_lexicon_tags_68 },
    { 0x4ea9030b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e9030e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a90312, SUC_lexicon_tags_30 },
    { 0xf5690313, SUC_lexicon_tags_19 },
    { 0xb0a90314, SUC_lexicon_tags_7 },
    { 0x1a890313, SUC_lexicon_tags_25 },
    { 0xc2290316, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4690318, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3649031d, SUC_lexicon_tags_1 },
    { 0xd3e9031d, SUC_lexicon_tags_11 },
    { 0x2ec9031f, SUC_lexicon_tags_2 },
    { 0xcee9031f, SUC_lexicon_tags_2 },
    { 0xe529031e, SUC_lexicon_tags_34 },
    { 0x2f890322, SUC_lexicon_tags_8 },
    { 0xf7e9031d, SUC_lexicon_tags_17 },
    { 0x92490324, SUC_lexicon_tags_49 },
    { 0xff690325, SUC_lexicon_tags_63 },
    { 0xcde90326, SUC_lexicon_tags_43 },
    { 0x71c9031e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa5a90329, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea9032d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xea49032f, SUC_lexicon_tags_17 },
    { 0xbcc9032f, SUC_lexicon_tags_23 },
    { 0xf4c90331, SUC_lexicon_tags_15 },
    { 0x34690332, SUC_lexicon_tags_25 },
    { 0x5a890332, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99c90335, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c690338, SUC_lexicon_tags_33 },
    { 0xd9490338, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5429033c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e89033f, SUC_lexicon_tags_2 },
    { 0xee90340, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c90344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74090346, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44890348, SUC_lexicon_tags_11 },
    { 0x6c290349, SUC_lexicon_tags_1 },
    { 0xe4890349, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x5ea9034c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a9034f, SUC_lexicon_tags_32 },
    { 0xa7490350, SUC_lexicon_tags_12 },
    { 0x3f09034f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6b890353, SUC_lexicon_tags_61 },
    { 0x21490354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde490359, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1129035b, SUC_lexicon_tags_47 },
    { 0x4aa9035b, SUC_lexicon_tags_25 },
    { 0x2229035d, SUC_lexicon_tags_68 },
    { 0x9a9035b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc90360, SUC_lexicon_tags_25 },
    { 0xb1490361, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa90365, SUC_lexicon_tags_1 },
    { 0xfec90365, SUC_lexicon_tags_31 },
    { 0x56890367, SUC_lexicon_tags_30 },
    { 0x7f290368, SUC_lexicon_tags_1 },
    { 0x8a290368, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbb69036b, SUC_lexicon_tags_7 },
    { 0xd8c9036c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2f29036e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc6c90370, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5d290372, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7fe90374, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58290378, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba9037d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a690385, SUC_lexicon_tags_2 },
    { 0x83090386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad89038a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea090390, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6809039b, SUC_lexicon_tags_8 },
    { 0x4cc9039c, SUC_lexicon_tags_101 },
    { 0x1669039c, SUC_lexicon_tags_27 },
    { 0x6f89039e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542903a2, SUC_lexicon_tags_12 },
    { 0xf16903a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac903ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x144903ae, SUC_lexicon_tags_25 },
    { 0xa22903af, SUC_lexicon_tags_17 },
    { 0xcac903b0, SUC_lexicon_tags_37 },
    { 0x188903b0, SUC_lexicon_tags_45 },
    { 0x91a903b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb12903b4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a903b8, SUC_lexicon_tags_192 },
    { 0x660903b8, SUC_lexicon_tags_11 },
    { 0xba4903ba, SUC_lexicon_tags_21 },
    { 0xdba903ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c903be, SUC_lexicon_tags_35 },
    { 0x9b8903bf, SUC_lexicon_tags_14 },
    { 0x94e903c0, SUC_lexicon_tags_25 },
    { 0xf3e903c0, SUC_lexicon_tags_37 },
    { 0x192903c0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0903c5, SUC_lexicon_tags_43 },
    { 0x1ce903c6, SUC_lexicon_tags_1 },
    { 0x330903c7, SUC_lexicon_tags_7 },
    { 0x196903c8, SUC_lexicon_tags_49 },
    { 0x1a4903c9, SUC_lexicon_tags_13 },
    { 0xccc903ca, SUC_lexicon_tags_29 },
    { 0xd8a903c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a903ce, SUC_lexicon_tags_23 },
    { 0x894903cf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a903d3, SUC_lexicon_tags_1 },
    { 0x42c903d4, SUC_lexicon_tags_37 },
    { 0xdfe903d5, SUC_lexicon_tags_2 },
    { 0x662903d4, SUC_lexicon_tags_1 },
    { 0x66c903d3, SUC_lexicon_tags_37 },
    { 0x684903d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x758903da, SUC_lexicon_tags_2 },
    { 0xc0903db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2903dd, SUC_lexicon_tags_37 },
    { 0x738903de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32903e3, SUC_lexicon_tags_38 },
    { 0x3a8903e4, SUC_lexicon_tags_11 },
    { 0x156903e4, SUC_lexicon_tags_29 },
    { 0x7c4903e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe5e903e8, SUC_lexicon_tags_17 },
    { 0x114903e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda903ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c903f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12903f4, SUC_lexicon_tags_55 },
    { 0x9ac903f4, SUC_lexicon_tags_11 },
    { 0x4d2903f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56903fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea90402, SUC_lexicon_tags_60 },
    { 0x88e90403, SUC_lexicon_tags_53 },
    { 0x36090404, SUC_lexicon_tags_57 },
    { 0x2bc90403, SUC_lexicon_tags_8 },
    { 0xba90406, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x91290409, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b29040c, SUC_lexicon_tags_3 },
    { 0xf7a9040c, SUC_lexicon_tags_11 },
    { 0x509040e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e90413, SUC_lexicon_tags_12 },
    { 0xc8c90414, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x62090417, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c9041a, SUC_lexicon_tags_2 },
    { 0x8ac9041b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea90421, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaf890423, SUC_lexicon_tags_45 },
    { 0xbe690424, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc90428, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1dc9042a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b290431, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xed490433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f090435, SUC_lexicon_tags_60 },
    { 0x50a90436, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3969043a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b49043d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e290440, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97090447, SUC_lexicon_tags_10 },
    { 0x97690448, SUC_lexicon_tags_17 },
    { 0x9b290449, SUC_lexicon_tags_26 },
    { 0xf669044a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf090451, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x13690453, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a9045b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de9045e, SUC_lexicon_tags_29 },
    { 0x7d49045f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc90462, SUC_lexicon_tags_90 },
    { 0, NULL },
    { 0x4690464, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x26890466, SUC_lexicon_tags_49 },
    { 0xc6890467, SUC_lexicon_tags_11 },
    { 0xad490466, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22690472, SUC_lexicon_tags_6 },
    { 0x8de90473, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e90479, SUC_lexicon_tags_47 },
    { 0x3c09047a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac9047e, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f290481, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68a90483, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca90488, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fe9048a, SUC_lexicon_tags_32 },
    { 0x23a9048a, SUC_lexicon_tags_21 },
    { 0x1b49048c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d89048f, SUC_lexicon_tags_19 },
    { 0xf949048f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c490496, SUC_lexicon_tags_30 },
    { 0x41890497, SUC_lexicon_tags_45 },
    { 0x1f690498, SUC_lexicon_tags_27 },
    { 0x4de90499, SUC_lexicon_tags_37 },
    { 0xbec90499, SUC_lexicon_tags_4 },
    { 0x2da9049b, SUC_lexicon_tags_45 },
    { 0x9209049b, SUC_lexicon_tags_37 },
    { 0x7fa9049d, SUC_lexicon_tags_33 },
    { 0x3529049c, SUC_lexicon_tags_17 },
    { 0xca09049c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e904a3, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c904a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfcc904a8, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8904ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40904b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74904b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa904bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab8904be, SUC_lexicon_tags_48 },
    { 0xc72904be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c904c4, SUC_lexicon_tags_37 },
    { 0x18c904c4, SUC_lexicon_tags_1 },
    { 0x3e6904c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe904ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260904cf, SUC_lexicon_tags_14 },
    { 0xbc0904d0, SUC_lexicon_tags_12 },
    { 0xa68904d0, SUC_lexicon_tags_0 },
    { 0x44a904d2, SUC_lexicon_tags_11 },
    { 0x86a904d1, SUC_lexicon_tags_37 },
    { 0xccc904d4, SUC_lexicon_tags_8 },
    { 0xd38904d4, SUC_lexicon_tags_17 },
    { 0xb46904d5, SUC_lexicon_tags_14 },
    { 0x258904d7, SUC_lexicon_tags_15 },
    { 0xa24904d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x630904da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00904e0, SUC_lexicon_tags_17 },
    { 0x72904e1, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be904e5, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a904e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a904ed, SUC_lexicon_tags_61 },
    { 0x69e904ed, SUC_lexicon_tags_11 },
    { 0x48c904ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86904f3, SUC_lexicon_tags_87 },
    { 0x712904f3, SUC_lexicon_tags_0 },
    { 0x80c904f5, SUC_lexicon_tags_84 },
    { 0xc4904f6, SUC_lexicon_tags_45 },
    { 0xb10904f4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510904fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57890500, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29290505, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5890509, SUC_lexicon_tags_12 },
    { 0x78c9050a, SUC_lexicon_tags_14 },
    { 0x1929050a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c9050f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x40490512, SUC_lexicon_tags_27 },
    { 0x36890513, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf490518, SUC_lexicon_tags_60 },
    { 0x40490519, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2090524, SUC_lexicon_tags_38 },
    { 0x5b490524, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c49052b, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec9052f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f490531, SUC_lexicon_tags_37 },
    { 0xc7a90532, SUC_lexicon_tags_68 },
    { 0x17690532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec090536, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a9053a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb869053d, SUC_lexicon_tags_17 },
    { 0x1669053d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c90542, SUC_lexicon_tags_60 },
    { 0xc3890542, SUC_lexicon_tags_47 },
    { 0xaa90542, SUC_lexicon_tags_66 },
    { 0x58890545, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e090549, SUC_lexicon_tags_60 },
    { 0x32e90549, SUC_lexicon_tags_14 },
    { 0xe709054b, SUC_lexicon_tags_35 },
    { 0xb569054c, SUC_lexicon_tags_7 },
    { 0x71c9054d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1690550, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6b690552, SUC_lexicon_tags_39 },
    { 0xbec90552, SUC_lexicon_tags_14 },
    { 0x27090553, SUC_lexicon_tags_8 },
    { 0x8fe90554, SUC_lexicon_tags_9 },
    { 0x3d890554, SUC_lexicon_tags_11 },
    { 0x2c890557, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70090559, SUC_lexicon_tags_25 },
    { 0xa789055a, SUC_lexicon_tags_19 },
    { 0xe1a90559, SUC_lexicon_tags_37 },
    { 0x92e9055b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c09055e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80690560, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf2690562, SUC_lexicon_tags_29 },
    { 0xa8890563, SUC_lexicon_tags_55 },
    { 0x60e90563, SUC_lexicon_tags_12 },
    { 0x7be90565, SUC_lexicon_tags_2 },
    { 0xf090566, SUC_lexicon_tags_7 },
    { 0x7da90564, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac69056d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa690573, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2890577, SUC_lexicon_tags_0 },
    { 0x4e490578, SUC_lexicon_tags_1 },
    { 0xf6a90578, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x87a9057b, SUC_lexicon_tags_0 },
    { 0x8609057c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a69057f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b690585, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27a90587, SUC_lexicon_tags_37 },
    { 0x92490588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x289058b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4290590, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35690592, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57690594, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2090598, SUC_lexicon_tags_2 },
    { 0xbe90598, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a49059c, SUC_lexicon_tags_25 },
    { 0x7a29059d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0905a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4905a9, SUC_lexicon_tags_1 },
    { 0xc5c905aa, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38905ad, SUC_lexicon_tags_68 },
    { 0x1da905ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x962905b0, SUC_lexicon_tags_25 },
    { 0xee6905b0, SUC_lexicon_tags_33 },
    { 0x8cc905b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fe905b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4905bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208905c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4905c6, SUC_lexicon_tags_12 },
    { 0xcd8905c6, SUC_lexicon_tags_2 },
    { 0xb50905c8, SUC_lexicon_tags_30 },
    { 0x94c905c9, SUC_lexicon_tags_11 },
    { 0x53c905c7, SUC_lexicon_tags_37 },
    { 0xbfc905cb, SUC_lexicon_tags_12 },
    { 0x76c905cb, SUC_lexicon_tags_57 },
    { 0x182905cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62905d4, SUC_lexicon_tags_66 },
    { 0x9a905d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x980905d7, SUC_lexicon_tags_1 },
    { 0xc96905d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6905db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4905e0, SUC_lexicon_tags_25 },
    { 0x904905e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b6905e3, SUC_lexicon_tags_31 },
    { 0x9c2905e4, SUC_lexicon_tags_15 },
    { 0xf14905e3, SUC_lexicon_tags_299 },
    { 0xae2905e6, SUC_lexicon_tags_49 },
    { 0x28905e7, SUC_lexicon_tags_37 },
    { 0x8d2905e5, SUC_lexicon_tags_63 },
    { 0xed6905e7, SUC_lexicon_tags_15 },
    { 0xdd4905e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x734905ed, SUC_lexicon_tags_15 },
    { 0x33a905ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x830905f0, SUC_lexicon_tags_38 },
    { 0x1b8905f1, SUC_lexicon_tags_17 },
    { 0x8ba905f2, SUC_lexicon_tags_27 },
    { 0xc28905f3, SUC_lexicon_tags_25 },
    { 0xb40905f4, SUC_lexicon_tags_2 },
    { 0x73c905f1, SUC_lexicon_tags_26 },
    { 0xde0905f6, SUC_lexicon_tags_47 },
    { 0x304905f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c905fb, SUC_lexicon_tags_84 },
    { 0x23a905fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2090605, SUC_lexicon_tags_39 },
    { 0x52290606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcac90608, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab29060d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f890614, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x53690617, SUC_lexicon_tags_2 },
    { 0x3ae90618, SUC_lexicon_tags_38 },
    { 0xca690619, SUC_lexicon_tags_26 },
    { 0xdc090618, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b29061e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7290620, SUC_lexicon_tags_40 },
    { 0x52890621, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5890625, SUC_lexicon_tags_15 },
    { 0xf6890626, SUC_lexicon_tags_15 },
    { 0x5d090627, SUC_lexicon_tags_26 },
    { 0x32290628, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8649062a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2ec9062c, SUC_lexicon_tags_14 },
    { 0x229062d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1189062f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe090633, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1490635, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7490637, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9229063a, SUC_lexicon_tags_27 },
    { 0x5e9063a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec9063e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3e90640, SUC_lexicon_tags_63 },
    { 0xea490641, SUC_lexicon_tags_37 },
    { 0xdb490640, SUC_lexicon_tags_1 },
    { 0x54490642, SUC_lexicon_tags_17 },
    { 0x74090640, SUC_lexicon_tags_7 },
    { 0x4890642, SUC_lexicon_tags_23 },
    { 0x60290642, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2290648, SUC_lexicon_tags_13 },
    { 0xa3290648, SUC_lexicon_tags_15 },
    { 0x3b690649, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4c29064c, SUC_lexicon_tags_23 },
    { 0x77e9064d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc90652, SUC_lexicon_tags_12 },
    { 0x94a90653, SUC_lexicon_tags_1336 },
    { 0x55090654, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a290657, SUC_lexicon_tags_31 },
    { 0xc5290658, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c9065f, SUC_lexicon_tags_14 },
    { 0x57690660, SUC_lexicon_tags_10 },
    { 0xbb490661, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9c290663, SUC_lexicon_tags_1 },
    { 0xdda90664, SUC_lexicon_tags_21 },
    { 0x19090663, SUC_lexicon_tags_39 },
    { 0x77a90666, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee090669, SUC_lexicon_tags_83 },
    { 0x27a90669, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x33e9066c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xae69066e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e90674, SUC_lexicon_tags_12 },
    { 0xab490675, SUC_lexicon_tags_1 },
    { 0x4a090675, SUC_lexicon_tags_11 },
    { 0xd8690677, SUC_lexicon_tags_9 },
    { 0x8f890674, SUC_lexicon_tags_7 },
    { 0x80890679, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebc9067b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5a09067d, SUC_lexicon_tags_28 },
    { 0x27e9067e, SUC_lexicon_tags_60 },
    { 0x80c9067e, SUC_lexicon_tags_38 },
    { 0xbb890680, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1da90682, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69890687, SUC_lexicon_tags_56 },
    { 0xee890688, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc9068c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a290690, SUC_lexicon_tags_14 },
    { 0x9ac90691, SUC_lexicon_tags_37 },
    { 0xea890691, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xda690694, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1490697, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68690699, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa049069c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0906a2, SUC_lexicon_tags_77 },
    { 0x106906a3, SUC_lexicon_tags_15 },
    { 0x7e2906a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e906aa, SUC_lexicon_tags_20 },
    { 0x71e906aa, SUC_lexicon_tags_11 },
    { 0x49e906ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a906b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x526906b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98a906b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc04906b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf08906ba, SUC_lexicon_tags_66 },
    { 0x6ee906bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ea906bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc14906bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb8906c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13e906c6, SUC_lexicon_tags_11 },
    { 0xc94906c7, SUC_lexicon_tags_1 },
    { 0x188906c8, SUC_lexicon_tags_43 },
    { 0x8ee906c7, SUC_lexicon_tags_61 },
    { 0xa4906ca, SUC_lexicon_tags_30 },
    { 0x932906c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e906cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0906d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780906d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36906db, SUC_lexicon_tags_47 },
    { 0x9f6906dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8906df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86906e5, SUC_lexicon_tags_33 },
    { 0x4ea906e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x822906e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x180906ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c906ef, SUC_lexicon_tags_17 },
    { 0x330906f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372906f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98906fa, SUC_lexicon_tags_15 },
    { 0x26a906fb, SUC_lexicon_tags_31 },
    { 0xc9c906fc, SUC_lexicon_tags_17 },
    { 0x414906fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f890702, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a490704, SUC_lexicon_tags_80 },
    { 0xf9890705, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd649070a, SUC_lexicon_tags_23 },
    { 0xed49070b, SUC_lexicon_tags_4 },
    { 0xba9070c, SUC_lexicon_tags_35 },
    { 0x4c69070d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31090711, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xabe90713, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e9071e, SUC_lexicon_tags_12 },
    { 0x21a9071f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8490721, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xa2e90723, SUC_lexicon_tags_25 },
    { 0xea690724, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x96890726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb29072d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a90732, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c90735, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a90738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e9073c, SUC_lexicon_tags_25 },
    { 0xc589073c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb2c9073f, SUC_lexicon_tags_148 },
    { 0x2a89073f, SUC_lexicon_tags_23 },
    { 0x39290741, SUC_lexicon_tags_136 },
    { 0xcec90742, SUC_lexicon_tags_15 },
    { 0x63290740, SUC_lexicon_tags_82 },
    { 0x49e90744, SUC_lexicon_tags_12 },
    { 0x6f290745, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f89074c, SUC_lexicon_tags_1 },
    { 0x6909074d, SUC_lexicon_tags_37 },
    { 0x7ce9074e, SUC_lexicon_tags_55 },
    { 0x5d69074f, SUC_lexicon_tags_12 },
    { 0x8f090750, SUC_lexicon_tags_36 },
    { 0x53690751, SUC_lexicon_tags_11 },
    { 0x9f090752, SUC_lexicon_tags_63 },
    { 0xe8890752, SUC_lexicon_tags_11 },
    { 0xa2c90754, SUC_lexicon_tags_11 },
    { 0xeb890754, SUC_lexicon_tags_0 },
    { 0xeb290756, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab09075a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2690761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99490764, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4490766, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b29076b, SUC_lexicon_tags_56 },
    { 0x51c9076c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9290772, SUC_lexicon_tags_10 },
    { 0x3490772, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93690777, SUC_lexicon_tags_9 },
    { 0xca890777, SUC_lexicon_tags_7 },
    { 0xddc90779, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc09077e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x82490780, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13890784, SUC_lexicon_tags_17 },
    { 0x9e490784, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62e90789, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8049078e, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x70490790, SUC_lexicon_tags_63 },
    { 0x17890790, SUC_lexicon_tags_1 },
    { 0x8be90790, SUC_lexicon_tags_17 },
    { 0x2f890790, SUC_lexicon_tags_37 },
    { 0xf2690794, SUC_lexicon_tags_37 },
    { 0xe9a90794, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ac90797, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6209079b, SUC_lexicon_tags_45 },
    { 0x9249079c, SUC_lexicon_tags_15 },
    { 0x4e09079d, SUC_lexicon_tags_47 },
    { 0xfe89079e, SUC_lexicon_tags_87 },
    { 0x9d09079e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x360907a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2907ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x134907ae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30907b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c8907b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x646907ba, SUC_lexicon_tags_25 },
    { 0x2f0907bb, SUC_lexicon_tags_17 },
    { 0xf42907bc, SUC_lexicon_tags_17 },
    { 0x620907bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868907c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ec907c9, SUC_lexicon_tags_12 },
    { 0x2c0907c9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558907ce, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4907d5, SUC_lexicon_tags_14 },
    { 0xef4907d6, SUC_lexicon_tags_63 },
    { 0x270907d6, SUC_lexicon_tags_11 },
    { 0xfe2907d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x662907db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e4907dd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x64907df, SUC_lexicon_tags_15 },
    { 0xc60907df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0907e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8907ea, SUC_lexicon_tags_43 },
    { 0x8f2907ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2907f2, SUC_lexicon_tags_16 },
    { 0x584907f2, SUC_lexicon_tags_63 },
    { 0xb9c907f2, SUC_lexicon_tags_68 },
    { 0xa3a907f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528907f9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x852907fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a907ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ce90801, SUC_lexicon_tags_39 },
    { 0x1c090802, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xad490804, SUC_lexicon_tags_37 },
    { 0xa1a90805, SUC_lexicon_tags_11 },
    { 0xe7490806, SUC_lexicon_tags_9 },
    { 0x8c090807, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d890809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb369080c, SUC_lexicon_tags_47 },
    { 0x9f09080d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5e9080f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a490812, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xea090815, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff290818, SUC_lexicon_tags_43 },
    { 0xbd490819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9dc9081b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1609081d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa489081f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f690822, SUC_lexicon_tags_61 },
    { 0xfea90823, SUC_lexicon_tags_37 },
    { 0x7be90822, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d090826, SUC_lexicon_tags_25 },
    { 0x69e90827, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a9082c, SUC_lexicon_tags_63 },
    { 0x3629082d, SUC_lexicon_tags_141 },
    { 0xc189082e, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xb9090830, SUC_lexicon_tags_21 },
    { 0x15090831, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6490834, SUC_lexicon_tags_34 },
    { 0xeac90835, SUC_lexicon_tags_60 },
    { 0x7d490836, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5ca90838, SUC_lexicon_tags_15 },
    { 0xd8a90839, SUC_lexicon_tags_47 },
    { 0xea9083a, SUC_lexicon_tags_33 },
    { 0xba9083a, SUC_lexicon_tags_4 },
    { 0x3409083c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x41c9083e, SUC_lexicon_tags_0 },
    { 0x2de9083e, SUC_lexicon_tags_17 },
    { 0x57e90840, SUC_lexicon_tags_12 },
    { 0xc709083e, SUC_lexicon_tags_2 },
    { 0xa7e90842, SUC_lexicon_tags_43 },
    { 0x17c90843, SUC_lexicon_tags_49 },
    { 0x51890842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc689084f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71490852, SUC_lexicon_tags_12 },
    { 0xfb890853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b49085c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba09085e, SUC_lexicon_tags_15 },
    { 0x7d49085e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb690864, SUC_lexicon_tags_160 },
    { 0x65490865, SUC_lexicon_tags_19 },
    { 0x36090865, SUC_lexicon_tags_68 },
    { 0xb2c90865, SUC_lexicon_tags_107 },
    { 0xa4a90868, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa09086a, SUC_lexicon_tags_43 },
    { 0x7b89086b, SUC_lexicon_tags_84 },
    { 0x9b69086c, SUC_lexicon_tags_6 },
    { 0x3049086c, SUC_lexicon_tags_136 },
    { 0x1209086e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2a90870, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xec090872, SUC_lexicon_tags_1 },
    { 0xbc690873, SUC_lexicon_tags_5 },
    { 0x62c90872, SUC_lexicon_tags_12 },
    { 0xb6490875, SUC_lexicon_tags_37 },
    { 0x6d690874, SUC_lexicon_tags_47 },
    { 0x86a90877, SUC_lexicon_tags_15 },
    { 0x6ea90878, SUC_lexicon_tags_45 },
    { 0xa0c90878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa9087c, SUC_lexicon_tags_28 },
    { 0xb8e9087d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc929087f, SUC_lexicon_tags_11 },
    { 0xa3e90880, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34490889, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbf09088b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2529088f, SUC_lexicon_tags_87 },
    { 0xadc90890, SUC_lexicon_tags_37 },
    { 0x27290891, SUC_lexicon_tags_17 },
    { 0xf5e90892, SUC_lexicon_tags_2 },
    { 0xabe90891, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4090895, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5c690897, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3749089a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1d69089c, SUC_lexicon_tags_27 },
    { 0x3589089d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742908a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8908a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83e908a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc908ac, SUC_lexicon_tags_7 },
    { 0x2ee908ad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x820908af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x98e908b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x98e908b3, SUC_lexicon_tags_37 },
    { 0xb72908b3, SUC_lexicon_tags_47 },
    { 0x5d8908b4, SUC_lexicon_tags_27 },
    { 0x414908b6, SUC_lexicon_tags_2 },
    { 0x88a908b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a908ba, SUC_lexicon_tags_10 },
    { 0x210908bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e908bf, SUC_lexicon_tags_47 },
    { 0xff0908bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4908c3, SUC_lexicon_tags_1 },
    { 0x362908c4, SUC_lexicon_tags_37 },
    { 0x9d8908c5, SUC_lexicon_tags_12 },
    { 0x7da908c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a908ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda908cf, SUC_lexicon_tags_16 },
    { 0x1be908d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdec908d2, SUC_lexicon_tags_17 },
    { 0x9fa908d2, SUC_lexicon_tags_34 },
    { 0x55e908d4, SUC_lexicon_tags_2 },
    { 0xbbe908d5, SUC_lexicon_tags_2 },
    { 0xa78908d6, SUC_lexicon_tags_25 },
    { 0xca908d6, SUC_lexicon_tags_37 },
    { 0x8a2908d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc0908da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8c8908dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e908df, SUC_lexicon_tags_352 },
    { 0x728908df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x176908e3, SUC_lexicon_tags_38 },
    { 0xc32908e4, SUC_lexicon_tags_65 },
    { 0xc6908e4, SUC_lexicon_tags_23 },
    { 0xc908e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb82908e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ac908ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x474908ec, SUC_lexicon_tags_19 },
    { 0x1ca908ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0908f0, SUC_lexicon_tags_46 },
    { 0x4be908f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x490908f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc908f7, SUC_lexicon_tags_7 },
    { 0xd04908f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x932908fa, SUC_lexicon_tags_33 },
    { 0xfba908fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe90900, SUC_lexicon_tags_46 },
    { 0x97890901, SUC_lexicon_tags_65 },
    { 0x7d490901, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3da90904, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda90908, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde9090b, SUC_lexicon_tags_12 },
    { 0x42c9090c, SUC_lexicon_tags_15 },
    { 0x5109090d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc069090f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a890912, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8349091a, SUC_lexicon_tags_68 },
    { 0x59a9091b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x46e9091d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a90921, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3290923, SUC_lexicon_tags_22 },
    { 0x2e290923, SUC_lexicon_tags_17 },
    { 0x55690925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda89092a, SUC_lexicon_tags_31 },
    { 0x1229092b, SUC_lexicon_tags_25 },
    { 0xb029092a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75e9092e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d890934, SUC_lexicon_tags_9 },
    { 0xcaa90935, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94090938, SUC_lexicon_tags_5 },
    { 0x4890938, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f89093b, SUC_lexicon_tags_12 },
    { 0x1e69093c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a90941, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x66290943, SUC_lexicon_tags_38 },
    { 0xea890944, SUC_lexicon_tags_2 },
    { 0x52a90945, SUC_lexicon_tags_37 },
    { 0x3f690944, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac9094d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3029095b, SUC_lexicon_tags_25 },
    { 0xabe9095c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf2e9095e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25490965, SUC_lexicon_tags_7 },
    { 0xd8090966, SUC_lexicon_tags_226 },
    { 0x37c90967, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9269096c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49e90970, SUC_lexicon_tags_49 },
    { 0x98490971, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33090974, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e90979, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd309097c, SUC_lexicon_tags_56 },
    { 0xb209097d, SUC_lexicon_tags_39 },
    { 0x5d49097e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24290980, SUC_lexicon_tags_14 },
    { 0xfc90981, SUC_lexicon_tags_16 },
    { 0x6f490982, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7890984, SUC_lexicon_tags_7 },
    { 0xc7890984, SUC_lexicon_tags_46 },
    { 0x9a690986, SUC_lexicon_tags_11 },
    { 0xada90987, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce90990, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5090992, SUC_lexicon_tags_60 },
    { 0x23a90993, SUC_lexicon_tags_12 },
    { 0x36690994, SUC_lexicon_tags_25 },
    { 0xa2c90992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e90998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276909a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c909a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a909ad, SUC_lexicon_tags_11 },
    { 0xe28909ad, SUC_lexicon_tags_11 },
    { 0xdae909af, SUC_lexicon_tags_12 },
    { 0x2f2909af, SUC_lexicon_tags_0 },
    { 0x65e909ad, SUC_lexicon_tags_23 },
    { 0x294909b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xddc909b4, SUC_lexicon_tags_8 },
    { 0x45c909b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156909bb, SUC_lexicon_tags_17 },
    { 0x70a909bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a909c6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd78909c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168909cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x936909ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48909d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66909db, SUC_lexicon_tags_84 },
    { 0x9f8909dc, SUC_lexicon_tags_12 },
    { 0xa8c909dd, SUC_lexicon_tags_25 },
    { 0xfd2909db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518909e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4909e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c909f3, SUC_lexicon_tags_23 },
    { 0xe78909f3, SUC_lexicon_tags_79 },
    { 0x7d8909f5, SUC_lexicon_tags_2 },
    { 0xf9a909f5, SUC_lexicon_tags_38 },
    { 0xf78909f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x99c909f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a90a01, SUC_lexicon_tags_2 },
    { 0x43890a02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7090a04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e090a07, SUC_lexicon_tags_21 },
    { 0x71690a08, SUC_lexicon_tags_0 },
    { 0x8be90a08, SUC_lexicon_tags_63 },
    { 0xf0a90a08, SUC_lexicon_tags_37 },
    { 0x2f890a08, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44890a0f, SUC_lexicon_tags_15 },
    { 0xdfa90a0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c90a19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5890a1b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a290a21, SUC_lexicon_tags_0 },
    { 0x9ee90a22, SUC_lexicon_tags_35 },
    { 0x2a90a23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a90a26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1490a29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e90a30, SUC_lexicon_tags_8 },
    { 0x7a290a31, SUC_lexicon_tags_6 },
    { 0x33090a31, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc90a38, SUC_lexicon_tags_25 },
    { 0xdaa90a38, SUC_lexicon_tags_7 },
    { 0xddc90a3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa090a3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13a90a3e, SUC_lexicon_tags_2 },
    { 0xa6290a3e, SUC_lexicon_tags_37 },
    { 0x3e890a3f, SUC_lexicon_tags_1 },
    { 0x24090a40, SUC_lexicon_tags_55 },
    { 0x5d490a3f, SUC_lexicon_tags_0 },
    { 0x6c90a40, SUC_lexicon_tags_27 },
    { 0x3ce90a3f, SUC_lexicon_tags_31 },
    { 0x7d690a45, SUC_lexicon_tags_17 },
    { 0x5ca90a44, SUC_lexicon_tags_26 },
    { 0xe4290a3e, SUC_lexicon_tags_25 },
    { 0xaba90a43, SUC_lexicon_tags_31 },
    { 0xfb090a49, SUC_lexicon_tags_66 },
    { 0x2490a4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a90a53, SUC_lexicon_tags_1 },
    { 0x3c490a54, SUC_lexicon_tags_11 },
    { 0x14290a55, SUC_lexicon_tags_17 },
    { 0x85c90a55, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60490a5b, SUC_lexicon_tags_38 },
    { 0x33690a5c, SUC_lexicon_tags_17 },
    { 0x6f490a5d, SUC_lexicon_tags_30 },
    { 0x42490a5e, SUC_lexicon_tags_15 },
    { 0xeb290a5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c890a65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91490a67, SUC_lexicon_tags_15 },
    { 0x41a90a67, SUC_lexicon_tags_23 },
    { 0x48690a68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e490a6d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c90a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97690a76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea90a79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf290a81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd8c90a83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2090a89, SUC_lexicon_tags_11 },
    { 0x32890a89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x39690a8c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xae290a8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6890a92, SUC_lexicon_tags_7 },
    { 0xfec90a92, SUC_lexicon_tags_26 },
    { 0x98290a94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d890a97, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x64890a99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x90a90a9b, SUC_lexicon_tags_27 },
    { 0x7f890a9c, SUC_lexicon_tags_11 },
    { 0xc9c90a9c, SUC_lexicon_tags_11 },
    { 0xa8e90a9e, SUC_lexicon_tags_30 },
    { 0xfac90a9e, SUC_lexicon_tags_9 },
    { 0xea290a9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c90aa5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x96290aa7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x94090aa9, SUC_lexicon_tags_15 },
    { 0x19c90aaa, SUC_lexicon_tags_2 },
    { 0xfa690aab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b690aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c90ab7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x39490ab9, SUC_lexicon_tags_11 },
    { 0x8ca90aba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c90abe, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc690ac1, SUC_lexicon_tags_60 },
    { 0x41890ac1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x65090ac5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb1a90ac7, SUC_lexicon_tags_19 },
    { 0xa6e90ac8, SUC_lexicon_tags_37 },
    { 0x85690ac9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeb090acb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0c90acd, SUC_lexicon_tags_25 },
    { 0xc6a90ace, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca90ad3, SUC_lexicon_tags_65 },
    { 0xda690ad4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc7a90ad6, SUC_lexicon_tags_38 },
    { 0xf6a90ad7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe6290ad9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x23490adc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4690ae1, SUC_lexicon_tags_25 },
    { 0xee690ae2, SUC_lexicon_tags_45 },
    { 0xa5890ae2, SUC_lexicon_tags_11 },
    { 0x51690ae3, SUC_lexicon_tags_7 },
    { 0xd8490ae5, SUC_lexicon_tags_46 },
    { 0x38c90ae6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa90aea, SUC_lexicon_tags_11 },
    { 0x94490aeb, SUC_lexicon_tags_19 },
    { 0xfa490aeb, SUC_lexicon_tags_22 },
    { 0x61a90aed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b490af1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0090af9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe890b00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0890b03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf8c90b05, SUC_lexicon_tags_49 },
    { 0xf090b06, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c90b09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa90b0d, SUC_lexicon_tags_2 },
    { 0x96290b0e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac90b12, SUC_lexicon_tags_32 },
    { 0x91c90b13, SUC_lexicon_tags_134 },
    { 0x75090b14, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x17c90b16, SUC_lexicon_tags_7 },
    { 0xfe90b16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd890b1c, SUC_lexicon_tags_25 },
    { 0x1b090b1d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x92290b20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc7e90b22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a90b26, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c90b2c, SUC_lexicon_tags_2 },
    { 0x94e90b2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0290b32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2c90b34, SUC_lexicon_tags_17 },
    { 0x36a90b34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd090b37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d490b39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e690b3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81890b44, SUC_lexicon_tags_7 },
    { 0x30290b44, SUC_lexicon_tags_2 },
    { 0x70e90b46, SUC_lexicon_tags_1 },
    { 0xba690b44, SUC_lexicon_tags_30 },
    { 0x6de90b48, SUC_lexicon_tags_12 },
    { 0xb3690b49, SUC_lexicon_tags_47 },
    { 0x99490b46, SUC_lexicon_tags_60 },
    { 0x3d690b4b, SUC_lexicon_tags_2 },
    { 0x63090b4c, SUC_lexicon_tags_15 },
    { 0x39090b45, SUC_lexicon_tags_38 },
    { 0x9a290b44, SUC_lexicon_tags_12 },
    { 0xaa490b4f, SUC_lexicon_tags_60 },
    { 0xcde90b50, SUC_lexicon_tags_62 },
    { 0xee290b4b, SUC_lexicon_tags_11 },
    { 0xfa290b4b, SUC_lexicon_tags_34 },
    { 0xf4c90b4b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x73490b55, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce90b59, SUC_lexicon_tags_29 },
    { 0x5a290b5a, SUC_lexicon_tags_25 },
    { 0xe6890b5a, SUC_lexicon_tags_2 },
    { 0x9de90b5a, SUC_lexicon_tags_0 },
    { 0x12a90b5c, SUC_lexicon_tags_37 },
    { 0xd6890b5c, SUC_lexicon_tags_17 },
    { 0xe4290b5b, SUC_lexicon_tags_37 },
    { 0xc1c90b59, SUC_lexicon_tags_25 },
    { 0x7ec90b61, SUC_lexicon_tags_65 },
    { 0x4aa90b61, SUC_lexicon_tags_43 },
    { 0x74490b5f, SUC_lexicon_tags_14 },
    { 0x4c90b5f, SUC_lexicon_tags_19 },
    { 0x2f890b62, SUC_lexicon_tags_23 },
    { 0x87290b66, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5090b6a, SUC_lexicon_tags_33 },
    { 0x2c890b6b, SUC_lexicon_tags_15 },
    { 0x51a90b6c, SUC_lexicon_tags_17 },
    { 0xb7890b6d, SUC_lexicon_tags_91 },
    { 0xea290b6d, SUC_lexicon_tags_19 },
    { 0xdae90b6d, SUC_lexicon_tags_23 },
    { 0x3490b6f, SUC_lexicon_tags_38 },
    { 0xd090b71, SUC_lexicon_tags_18 },
    { 0xe4490b71, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x49490b74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe90b78, SUC_lexicon_tags_60 },
    { 0x56a90b79, SUC_lexicon_tags_84 },
    { 0x90690b7a, SUC_lexicon_tags_30 },
    { 0xba890b7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6490b81, SUC_lexicon_tags_15 },
    { 0x46090b82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73690b8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e90b90, SUC_lexicon_tags_10 },
    { 0x6d490b90, SUC_lexicon_tags_17 },
    { 0x5f490b92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaca90b94, SUC_lexicon_tags_1 },
    { 0x81890b95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5490b9a, SUC_lexicon_tags_11 },
    { 0xb9690b9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd890b9d, SUC_lexicon_tags_63 },
    { 0x69e90b9d, SUC_lexicon_tags_38 },
    { 0x23890b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c090ba1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37890ba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c90ba9, SUC_lexicon_tags_17 },
    { 0xd7690baa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8290bac, SUC_lexicon_tags_7 },
    { 0xd7e90bad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe90bb2, SUC_lexicon_tags_1 },
    { 0x68290bb3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x75090bb6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf2690bb8, SUC_lexicon_tags_26 },
    { 0x88c90bb9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcd290bbb, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c890bbf, SUC_lexicon_tags_11 },
    { 0x5a690bc0, SUC_lexicon_tags_37 },
    { 0x64a90bbf, SUC_lexicon_tags_23 },
    { 0x85490bbf, SUC_lexicon_tags_10 },
    { 0xeea90bc3, SUC_lexicon_tags_46 },
    { 0x76290bc4, SUC_lexicon_tags_25 },
    { 0x1aa90bc1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xde90bc8, SUC_lexicon_tags_118 },
    { 0x92890bc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29290bcb, SUC_lexicon_tags_53 },
    { 0x31a90bcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed090bce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9a490bd0, SUC_lexicon_tags_57 },
    { 0x97290bd0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa9290bd3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2ee90bd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5ce90bd7, SUC_lexicon_tags_25 },
    { 0x8b290bd8, SUC_lexicon_tags_25 },
    { 0x5d490bd9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6f290bdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f090bdf, SUC_lexicon_tags_33 },
    { 0x85a90be0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc90be3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69a90be5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22490be8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca90bee, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x46090bf1, SUC_lexicon_tags_27 },
    { 0x81290bf2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x47290bf4, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0x57e90bf6, SUC_lexicon_tags_47 },
    { 0x60090bf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6a90bf9, SUC_lexicon_tags_55 },
    { 0x37690bf9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a690bff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d690c02, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54090c06, SUC_lexicon_tags_38 },
    { 0xaea90c07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b290c0d, SUC_lexicon_tags_49 },
    { 0x56e90c0d, SUC_lexicon_tags_25 },
    { 0xd8e90c0f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7c090c11, SUC_lexicon_tags_65 },
    { 0x15290c12, SUC_lexicon_tags_11 },
    { 0x5e690c13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9690c17, SUC_lexicon_tags_15 },
    { 0xce490c18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a90c1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d490c21, SUC_lexicon_tags_8 },
    { 0x3e290c22, SUC_lexicon_tags_7 },
    { 0x5d290c23, SUC_lexicon_tags_37 },
    { 0x44e90c21, SUC_lexicon_tags_25 },
    { 0x7fc90c21, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46890c29, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac90c2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee90c2f, SUC_lexicon_tags_17 },
    { 0x1ac90c30, SUC_lexicon_tags_25 },
    { 0xe3490c31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87090c35, SUC_lexicon_tags_26 },
    { 0x5ec90c35, SUC_lexicon_tags_12 },
    { 0x63090c35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecc90c39, SUC_lexicon_tags_33 },
    { 0xe490c3a, SUC_lexicon_tags_17 },
    { 0xb3690c3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e490c3d, SUC_lexicon_tags_64 },
    { 0x46290c3e, SUC_lexicon_tags_60 },
    { 0xd6e90c3e, SUC_lexicon_tags_25 },
    { 0xcfe90c40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xada90c42, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4090c46, SUC_lexicon_tags_25 },
    { 0xa0290c47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45490c49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e90c52, SUC_lexicon_tags_7 },
    { 0x3aa90c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c90c58, SUC_lexicon_tags_43 },
    { 0xfd090c59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3490c5e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25490c66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x96690c69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c890c6b, SUC_lexicon_tags_27 },
    { 0x84690c6c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x20290c6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3290c79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a90c81, SUC_lexicon_tags_218 },
    { 0x64890c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad290c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce090c86, SUC_lexicon_tags_26 },
    { 0x82090c87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b490c8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0290c90, SUC_lexicon_tags_37 },
    { 0x30890c91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4e90c93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67490c98, SUC_lexicon_tags_27 },
    { 0x21e90c99, SUC_lexicon_tags_8 },
    { 0x1ca90c9a, SUC_lexicon_tags_11 },
    { 0x55490c9a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c90ca0, SUC_lexicon_tags_37 },
    { 0x59290ca1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e90ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf490cab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91290cad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73a90caf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xedc90cb1, SUC_lexicon_tags_64 },
    { 0xde490cb2, SUC_lexicon_tags_60 },
    { 0x46e90cb3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x83a90cb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a90cb9, SUC_lexicon_tags_35 },
    { 0x1f890cb9, SUC_lexicon_tags_63 },
    { 0xc1c90cbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c90cc2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x12c90cc4, SUC_lexicon_tags_30 },
    { 0x18c90cc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf290cc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76690cd4, SUC_lexicon_tags_25 },
    { 0xa690cd5, SUC_lexicon_tags_25 },
    { 0x80690cd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84a90cd8, SUC_lexicon_tags_77 },
    { 0xdfc90cd9, SUC_lexicon_tags_30 },
    { 0x36c90cda, SUC_lexicon_tags_25 },
    { 0x99690cdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac90ce0, SUC_lexicon_tags_37 },
    { 0x1b490ce1, SUC_lexicon_tags_25 },
    { 0x9e290ce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e90ce9, SUC_lexicon_tags_27 },
    { 0xa5490cea, SUC_lexicon_tags_43 },
    { 0x43090ce9, SUC_lexicon_tags_12 },
    { 0x3e490ce9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7890cf1, SUC_lexicon_tags_37 },
    { 0xadc90cf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed490cf9, SUC_lexicon_tags_15 },
    { 0x40a90cfa, SUC_lexicon_tags_11 },
    { 0x3e890cf9, SUC_lexicon_tags_15 },
    { 0xfe890cfc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a90cff, SUC_lexicon_tags_15 },
    { 0xcf290d00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xad490d02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c290d04, SUC_lexicon_tags_17 },
    { 0x8fe90d05, SUC_lexicon_tags_25 },
    { 0x48090d04, SUC_lexicon_tags_19 },
    { 0x96490d07, SUC_lexicon_tags_38 },
    { 0x52890d08, SUC_lexicon_tags_57 },
    { 0xdf090d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde890d0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0290d12, SUC_lexicon_tags_1 },
    { 0x1fa90d12, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdf090d15, SUC_lexicon_tags_12 },
    { 0x57090d15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e90d1a, SUC_lexicon_tags_11 },
    { 0xe8a90d1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb490d23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c90d26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84890d2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8290d2e, SUC_lexicon_tags_0 },
    { 0xc3e90d2e, SUC_lexicon_tags_2 },
    { 0xd490d2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd290d33, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xb2290d35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a90d38, SUC_lexicon_tags_56 },
    { 0x5e90d39, SUC_lexicon_tags_25 },
    { 0x47c90d38, SUC_lexicon_tags_47 },
    { 0x7e690d3b, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0x69c90d3d, SUC_lexicon_tags_32 },
    { 0x3c490d3d, SUC_lexicon_tags_37 },
    { 0x92a90d3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ca90d41, SUC_lexicon_tags_1 },
    { 0x97e90d41, SUC_lexicon_tags_10 },
    { 0x12890d43, SUC_lexicon_tags_11 },
    { 0xdf690d44, SUC_lexicon_tags_2 },
    { 0xeb290d43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82290d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca90d4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a90d54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26490d57, SUC_lexicon_tags_19 },
    { 0x75c90d57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1090d5d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18490d62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe290d65, SUC_lexicon_tags_32 },
    { 0xaf490d66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc90d6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x10890d6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a90d6f, SUC_lexicon_tags_38 },
    { 0xa9a90d70, SUC_lexicon_tags_25 },
    { 0x14690d71, SUC_lexicon_tags_33 },
    { 0x35090d70, SUC_lexicon_tags_19 },
    { 0x71e90d73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd290d77, SUC_lexicon_tags_77 },
    { 0xdcc90d78, SUC_lexicon_tags_0 },
    { 0xa1490d78, SUC_lexicon_tags_11 },
    { 0xb6e90d7a, SUC_lexicon_tags_28 },
    { 0x93290d7b, SUC_lexicon_tags_17 },
    { 0x8090d7a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c290d81, SUC_lexicon_tags_37 },
    { 0xa1a90d81, SUC_lexicon_tags_11 },
    { 0xa1e90d83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe90d89, SUC_lexicon_tags_14 },
    { 0xb4e90d8a, SUC_lexicon_tags_60 },
    { 0x15290d8b, SUC_lexicon_tags_25 },
    { 0x62090d8b, SUC_lexicon_tags_25 },
    { 0x24a90d8d, SUC_lexicon_tags_37 },
    { 0xfe890d8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76a90d90, SUC_lexicon_tags_37 },
    { 0xe1890d90, SUC_lexicon_tags_29 },
    { 0x38090d92, SUC_lexicon_tags_29 },
    { 0x89490d93, SUC_lexicon_tags_11 },
    { 0x1ea90d91, SUC_lexicon_tags_19 },
    { 0xd2290d93, SUC_lexicon_tags_25 },
    { 0xda690d96, SUC_lexicon_tags_30 },
    { 0x9fa90d97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea90d9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e90d9d, SUC_lexicon_tags_2 },
    { 0xdc090d9e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf090da1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x35290da4, SUC_lexicon_tags_2 },
    { 0x98690da5, SUC_lexicon_tags_43 },
    { 0x6f090da5, SUC_lexicon_tags_26 },
    { 0x58890da7, SUC_lexicon_tags_0 },
    { 0xea490da8, SUC_lexicon_tags_68 },
    { 0x41c90da9, SUC_lexicon_tags_65 },
    { 0x71490da6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b090daf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be90db5, SUC_lexicon_tags_65 },
    { 0x5be90db6, SUC_lexicon_tags_17 },
    { 0xa5490db6, SUC_lexicon_tags_37 },
    { 0xdc490db8, SUC_lexicon_tags_11 },
    { 0x51290db9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e90dbd, SUC_lexicon_tags_37 },
    { 0x5c490dbd, SUC_lexicon_tags_63 },
    { 0x8ee90dbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8fa90dc1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xd3890dc3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdbe90dc5, SUC_lexicon_tags_45 },
    { 0xef690dc6, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0xba490dc8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97490dcc, SUC_lexicon_tags_49 },
    { 0xc7a90dcc, SUC_lexicon_tags_38 },
    { 0xae890dcd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x93490dd1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f490dd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49690dd8, SUC_lexicon_tags_39 },
    { 0x65c90dd8, SUC_lexicon_tags_8 },
    { 0x48490dd9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb490ddc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60890de0, SUC_lexicon_tags_381 },
    { 0x92490de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72490de8, SUC_lexicon_tags_6 },
    { 0x78a90de9, SUC_lexicon_tags_1 },
    { 0x4c690dea, SUC_lexicon_tags_38 },
    { 0xd7690deb, SUC_lexicon_tags_12 },
    { 0xd1490deb, SUC_lexicon_tags_30 },
    { 0xb3690dec, SUC_lexicon_tags_1 },
    { 0x26e90dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a90df1, SUC_lexicon_tags_33 },
    { 0xfc890df2, SUC_lexicon_tags_38 },
    { 0x38890df3, SUC_lexicon_tags_17 },
    { 0x55a90df3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee90df8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10890dfc, SUC_lexicon_tags_17 },
    { 0x43890dfd, SUC_lexicon_tags_37 },
    { 0xea890dfd, SUC_lexicon_tags_2 },
    { 0xf0890dff, SUC_lexicon_tags_63 },
    { 0x12490e00, SUC_lexicon_tags_26 },
    { 0x16490e01, SUC_lexicon_tags_27 },
    { 0xd690e02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e490e05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec90e08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4890e11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52890e13, SUC_lexicon_tags_27 },
    { 0xfdc90e14, SUC_lexicon_tags_0 },
    { 0x5b090e14, SUC_lexicon_tags_37 },
    { 0xf1290e16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde90e1a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4490e1c, SUC_lexicon_tags_17 },
    { 0xcf290e1c, SUC_lexicon_tags_27 },
    { 0x18290e1e, SUC_lexicon_tags_11 },
    { 0x29490e1f, SUC_lexicon_tags_2 },
    { 0x5ee90e20, SUC_lexicon_tags_11 },
    { 0x90c90e21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96890e25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa7290e27, SUC_lexicon_tags_12 },
    { 0xc90e28, SUC_lexicon_tags_23 },
    { 0x95090e27, SUC_lexicon_tags_2 },
    { 0xe2690e2a, SUC_lexicon_tags_25 },
    { 0xff690e2b, SUC_lexicon_tags_10 },
    { 0x41c90e2c, SUC_lexicon_tags_11 },
    { 0x5f690e2b, SUC_lexicon_tags_11 },
    { 0x5ba90e2b, SUC_lexicon_tags_12 },
    { 0x7890e2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe90e33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda290e3a, SUC_lexicon_tags_37 },
    { 0xc2290e3a, SUC_lexicon_tags_11 },
    { 0xcee90e3c, SUC_lexicon_tags_43 },
    { 0xc0490e3d, SUC_lexicon_tags_0 },
    { 0x3b290e3d, SUC_lexicon_tags_23 },
    { 0xb4090e3f, SUC_lexicon_tags_17 },
    { 0x37a90e40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82690e42, SUC_lexicon_tags_18 },
    { 0x11a90e43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5e90e45, SUC_lexicon_tags_59 },
    { 0x22490e46, SUC_lexicon_tags_15 },
    { 0xfcc90e47, SUC_lexicon_tags_5 },
    { 0x2b090e48, SUC_lexicon_tags_17 },
    { 0xe1890e49, SUC_lexicon_tags_127 },
    { 0x48e90e46, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4290e4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x21890e4f, SUC_lexicon_tags_14 },
    { 0x19e90e50, SUC_lexicon_tags_15 },
    { 0xfa090e51, SUC_lexicon_tags_45 },
    { 0x92a90e52, SUC_lexicon_tags_17 },
    { 0x7490e52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30890e55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8290e59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf290e5b, SUC_lexicon_tags_0 },
    { 0x9c890e5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c90e63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc9090e65, SUC_lexicon_tags_27 },
    { 0xe6090e66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda890e69, SUC_lexicon_tags_14 },
    { 0x6dc90e6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x76c90e6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbe090e6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a90e74, SUC_lexicon_tags_47 },
    { 0x75690e75, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7490e7b, SUC_lexicon_tags_12 },
    { 0x13090e7c, SUC_lexicon_tags_9 },
    { 0xe3a90e7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc90e80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25290e83, SUC_lexicon_tags_1 },
    { 0x23490e83, SUC_lexicon_tags_146 },
    { 0x53e90e85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8690e8a, SUC_lexicon_tags_7 },
    { 0x91c90e8b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb6890e8d, SUC_lexicon_tags_7 },
    { 0x89890e8d, SUC_lexicon_tags_0 },
    { 0xc4490e8d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x48290e91, SUC_lexicon_tags_27 },
    { 0xa9290e92, SUC_lexicon_tags_2 },
    { 0x2c890e93, SUC_lexicon_tags_14 },
    { 0x35290e94, SUC_lexicon_tags_4 },
    { 0x60290e91, SUC_lexicon_tags_1 },
    { 0x48490e93, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74090e9a, SUC_lexicon_tags_9 },
    { 0xd5090e9b, SUC_lexicon_tags_32 },
    { 0x5d290e9a, SUC_lexicon_tags_12 },
    { 0x8a690e9d, SUC_lexicon_tags_25 },
    { 0x71490e9c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe2490ea0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e90ea3, SUC_lexicon_tags_25 },
    { 0xee290ea3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3290ea9, SUC_lexicon_tags_12 },
    { 0x81890eaa, SUC_lexicon_tags_19 },
    { 0xc8a90eab, SUC_lexicon_tags_37 },
    { 0x16c90eac, SUC_lexicon_tags_32 },
    { 0xb8090eac, SUC_lexicon_tags_12 },
    { 0x5bc90eaa, SUC_lexicon_tags_11 },
    { 0x35c90eaf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3890eb5, SUC_lexicon_tags_11 },
    { 0x4e490eb6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x58890eb9, SUC_lexicon_tags_15 },
    { 0x2090eb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab690ebd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd690ec4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2090ec8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c490eca, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d690ecd, SUC_lexicon_tags_15 },
    { 0xdf290ece, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe290ed9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e90edd, SUC_lexicon_tags_7 },
    { 0xc4690ede, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a90ee7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f490ee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35690eec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f290ef8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xda290efa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e90f00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6890f04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3490f0d, SUC_lexicon_tags_45 },
    { 0x6fc90f0d, SUC_lexicon_tags_37 },
    { 0x7a90f0e, SUC_lexicon_tags_11 },
    { 0x3d690f10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23890f15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f890f1e, SUC_lexicon_tags_23 },
    { 0x70690f1f, SUC_lexicon_tags_8 },
    { 0x32890f20, SUC_lexicon_tags_11 },
    { 0xd0c90f1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81490f23, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0xeb090f25, SUC_lexicon_tags_35 },
    { 0xa0690f25, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6b090f28, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x75890f2b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70090f30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b490f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b490f37, SUC_lexicon_tags_17 },
    { 0x41a90f38, SUC_lexicon_tags_2 },
    { 0x56a90f39, SUC_lexicon_tags_28 },
    { 0x69890f3a, SUC_lexicon_tags_265 },
    { 0x96c90f37, SUC_lexicon_tags_12 },
    { 0x18e90f3c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x81290f3e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe2a90f40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc90f46, SUC_lexicon_tags_25 },
    { 0x28c90f47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde490f4c, SUC_lexicon_tags_17 },
    { 0xae090f4c, SUC_lexicon_tags_308 },
    { 0x31490f4e, SUC_lexicon_tags_15 },
    { 0x80490f4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be90f5a, SUC_lexicon_tags_31 },
    { 0xdcc90f5b, SUC_lexicon_tags_37 },
    { 0xc9c90f5c, SUC_lexicon_tags_37 },
    { 0xf890f5d, SUC_lexicon_tags_57 },
    { 0xa7090f5c, SUC_lexicon_tags_47 },
    { 0xaa290f5e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce90f63, SUC_lexicon_tags_25 },
    { 0x70c90f64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x46e90f66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90290f6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd090f6d, SUC_lexicon_tags_11 },
    { 0x14690f6d, SUC_lexicon_tags_36 },
    { 0x8c690f6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a90f72, SUC_lexicon_tags_30 },
    { 0x1a690f72, SUC_lexicon_tags_7 },
    { 0x57490f72, SUC_lexicon_tags_43 },
    { 0x87c90f75, SUC_lexicon_tags_55 },
    { 0xe0c90f75, SUC_lexicon_tags_33 },
    { 0xeac90f77, SUC_lexicon_tags_7 },
    { 0x81290f74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29090f7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a90f7d, SUC_lexicon_tags_1 },
    { 0xa7290f7e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xdb290f80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc0c90f82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c90f86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2290f8a, SUC_lexicon_tags_17 },
    { 0xff690f8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e90f8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe490f91, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a90f99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3a90f9b, SUC_lexicon_tags_118 },
    { 0x6ca90f9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97890fa0, SUC_lexicon_tags_37 },
    { 0xc9890fa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3c90fa3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e90fa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a690faa, SUC_lexicon_tags_1 },
    { 0xb4090faa, SUC_lexicon_tags_11 },
    { 0xe6090fac, SUC_lexicon_tags_1 },
    { 0x8de90fac, SUC_lexicon_tags_16 },
    { 0x1b490fae, SUC_lexicon_tags_25 },
    { 0xd9c90faa, SUC_lexicon_tags_43 },
    { 0x8da90fae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a490fb4, SUC_lexicon_tags_15 },
    { 0xfda90fb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ae90fb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x76290fb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e90fbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2690fc2, SUC_lexicon_tags_47 },
    { 0xb9890fc3, SUC_lexicon_tags_17 },
    { 0x3290fc3, SUC_lexicon_tags_17 },
    { 0x5e290fc2, SUC_lexicon_tags_17 },
    { 0x76290fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8090fca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87690fce, SUC_lexicon_tags_37 },
    { 0xc6e90fcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60090fd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71c90fd8, SUC_lexicon_tags_27 },
    { 0x37e90fd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece90fdf, SUC_lexicon_tags_37 },
    { 0x10890fe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c90fe3, SUC_lexicon_tags_29 },
    { 0x32c90fe4, SUC_lexicon_tags_37 },
    { 0x56c90fe5, SUC_lexicon_tags_13 },
    { 0xfe690fe6, SUC_lexicon_tags_11 },
    { 0xb2a90fe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc90feb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc90fee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb4e90ff0, SUC_lexicon_tags_1 },
    { 0x94690ff1, SUC_lexicon_tags_17 },
    { 0x71490ff2, SUC_lexicon_tags_0 },
    { 0xe9e90ff3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e290ff6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97a90ff8, SUC_lexicon_tags_7 },
    { 0x87e90ff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84490ffb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e91000, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3d091002, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd369100d, SUC_lexicon_tags_63 },
    { 0x4d29100e, SUC_lexicon_tags_11 },
    { 0x3769100f, SUC_lexicon_tags_63 },
    { 0xa5091010, SUC_lexicon_tags_12 },
    { 0xdf09100f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e9101a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd09101d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f89101f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x44a91021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94691023, SUC_lexicon_tags_82 },
    { 0x27a91024, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5c491026, SUC_lexicon_tags_49 },
    { 0x3cc91026, SUC_lexicon_tags_15 },
    { 0xbaa91028, SUC_lexicon_tags_30 },
    { 0xdce91029, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9789102c, SUC_lexicon_tags_1 },
    { 0x62a9102d, SUC_lexicon_tags_29 },
    { 0x74e9102e, SUC_lexicon_tags_6 },
    { 0x8d69102e, SUC_lexicon_tags_1 },
    { 0xd9091030, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xca091032, SUC_lexicon_tags_63 },
    { 0xaf691033, SUC_lexicon_tags_27 },
    { 0x52291033, SUC_lexicon_tags_82 },
    { 0xaba91034, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac69103c, SUC_lexicon_tags_63 },
    { 0xc169103d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdb69103f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4f891041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28691044, SUC_lexicon_tags_12 },
    { 0xdb691045, SUC_lexicon_tags_63 },
    { 0x28891046, SUC_lexicon_tags_14 },
    { 0xaa691047, SUC_lexicon_tags_7 },
    { 0x1891045, SUC_lexicon_tags_12 },
    { 0xcaa91049, SUC_lexicon_tags_31 },
    { 0x8689104a, SUC_lexicon_tags_30 },
    { 0x7849104b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca91051, SUC_lexicon_tags_37 },
    { 0xdb491052, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba91059, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5289105b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a9105f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda491065, SUC_lexicon_tags_53 },
    { 0x42291066, SUC_lexicon_tags_37 },
    { 0x8ee91066, SUC_lexicon_tags_30 },
    { 0x50c91068, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae9106d, SUC_lexicon_tags_14 },
    { 0xbee9106e, SUC_lexicon_tags_30 },
    { 0x4049106e, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x35e91071, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa2691073, SUC_lexicon_tags_10 },
    { 0xfd691074, SUC_lexicon_tags_9 },
    { 0xe0291075, SUC_lexicon_tags_21 },
    { 0x8ea91076, SUC_lexicon_tags_11 },
    { 0x97c91075, SUC_lexicon_tags_37 },
    { 0xe1e91077, SUC_lexicon_tags_2 },
    { 0x97a91079, SUC_lexicon_tags_28 },
    { 0xa789107a, SUC_lexicon_tags_0 },
    { 0xe6891077, SUC_lexicon_tags_8 },
    { 0x40e9107c, SUC_lexicon_tags_87 },
    { 0xea49107d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd709107f, SUC_lexicon_tags_45 },
    { 0xe4a91080, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a91083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4691085, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc589108b, SUC_lexicon_tags_31 },
    { 0xb09108b, SUC_lexicon_tags_31 },
    { 0x2129108d, SUC_lexicon_tags_2 },
    { 0x9209108c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x26291090, SUC_lexicon_tags_38 },
    { 0xd7291091, SUC_lexicon_tags_7 },
    { 0x49c91091, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcae91094, SUC_lexicon_tags_68 },
    { 0xadc91094, SUC_lexicon_tags_118 },
    { 0x64c91096, SUC_lexicon_tags_11 },
    { 0x16a91097, SUC_lexicon_tags_7 },
    { 0xf7e91098, SUC_lexicon_tags_29 },
    { 0xec291099, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14910a1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de910ab, SUC_lexicon_tags_15 },
    { 0x89a910ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de910b0, SUC_lexicon_tags_25 },
    { 0x24e910b1, SUC_lexicon_tags_21 },
    { 0xd5e910b2, SUC_lexicon_tags_2 },
    { 0xa42910b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a2910b5, SUC_lexicon_tags_26 },
    { 0x2c8910b6, SUC_lexicon_tags_25 },
    { 0xe96910b6, SUC_lexicon_tags_68 },
    { 0x30a910b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e910be, SUC_lexicon_tags_23 },
    { 0xbec910bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e910c2, SUC_lexicon_tags_26 },
    { 0x6e6910c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2910c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4910c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7e910cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaf4910cd, SUC_lexicon_tags_19 },
    { 0x578910ce, SUC_lexicon_tags_48 },
    { 0xe18910cf, SUC_lexicon_tags_19 },
    { 0xfb8910cf, SUC_lexicon_tags_25 },
    { 0xde2910d1, SUC_lexicon_tags_15 },
    { 0x790910d2, SUC_lexicon_tags_2 },
    { 0x8d4910d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08910d9, SUC_lexicon_tags_47 },
    { 0x9a6910da, SUC_lexicon_tags_16 },
    { 0xa6c910db, SUC_lexicon_tags_2 },
    { 0xf70910dc, SUC_lexicon_tags_32 },
    { 0xf32910dd, SUC_lexicon_tags_13 },
    { 0x5a6910da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70910e2, SUC_lexicon_tags_25 },
    { 0x9bc910e2, SUC_lexicon_tags_7 },
    { 0xe0c910e2, SUC_lexicon_tags_233 },
    { 0x752910e3, SUC_lexicon_tags_23 },
    { 0x7e0910e6, SUC_lexicon_tags_1 },
    { 0xb02910e6, SUC_lexicon_tags_9 },
    { 0x9ee910e4, SUC_lexicon_tags_63 },
    { 0x7c6910e9, SUC_lexicon_tags_25 },
    { 0xacc910e7, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x288910ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56910f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c910f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x300910f6, SUC_lexicon_tags_25 },
    { 0x1ac910f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x682910fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a91102, SUC_lexicon_tags_27 },
    { 0x61e91103, SUC_lexicon_tags_837 },
    { 0x2a91103, SUC_lexicon_tags_207 },
    { 0xb3c91104, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a691109, SUC_lexicon_tags_17 },
    { 0xe8c9110a, SUC_lexicon_tags_37 },
    { 0x8429110b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a89110d, SUC_lexicon_tags_47 },
    { 0xa549110d, SUC_lexicon_tags_63 },
    { 0x78a9110f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c291111, SUC_lexicon_tags_48 },
    { 0xdca91112, SUC_lexicon_tags_47 },
    { 0xa2491111, SUC_lexicon_tags_68 },
    { 0xa3691114, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc91117, SUC_lexicon_tags_35 },
    { 0x1f891117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb629111a, SUC_lexicon_tags_91 },
    { 0x49e9111b, SUC_lexicon_tags_11 },
    { 0x1469111b, SUC_lexicon_tags_25 },
    { 0x1a49111c, SUC_lexicon_tags_37 },
    { 0xd3c9111d, SUC_lexicon_tags_7 },
    { 0x7749111c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x57a91122, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0091124, SUC_lexicon_tags_14 },
    { 0xe9491125, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1fa91127, SUC_lexicon_tags_43 },
    { 0x9fe91128, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f89112a, SUC_lexicon_tags_33 },
    { 0xaa89112b, SUC_lexicon_tags_0 },
    { 0xbc9112c, SUC_lexicon_tags_11 },
    { 0xee29112d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe891131, SUC_lexicon_tags_47 },
    { 0xa8a91132, SUC_lexicon_tags_7 },
    { 0x7fc91133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a91136, SUC_lexicon_tags_47 },
    { 0xeda91136, SUC_lexicon_tags_15 },
    { 0xa3691138, SUC_lexicon_tags_19 },
    { 0x21c91139, SUC_lexicon_tags_15 },
    { 0x3891139, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe69113c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb089113e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd291141, SUC_lexicon_tags_63 },
    { 0x8491141, SUC_lexicon_tags_37 },
    { 0x46891143, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e91147, SUC_lexicon_tags_25 },
    { 0x18a91147, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe969114a, SUC_lexicon_tags_21 },
    { 0x3049114b, SUC_lexicon_tags_17 },
    { 0x3b69114c, SUC_lexicon_tags_4 },
    { 0xd5a9114d, SUC_lexicon_tags_25 },
    { 0x4289114d, SUC_lexicon_tags_85 },
    { 0x5f09114b, SUC_lexicon_tags_151 },
    { 0x86e9114e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a91153, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70691157, SUC_lexicon_tags_57 },
    { 0xabc91158, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e491162, SUC_lexicon_tags_17 },
    { 0xb2a91163, SUC_lexicon_tags_0 },
    { 0x2de91163, SUC_lexicon_tags_37 },
    { 0xad091163, SUC_lexicon_tags_25 },
    { 0xeaa91166, SUC_lexicon_tags_56 },
    { 0xe091166, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4bc91169, SUC_lexicon_tags_38 },
    { 0x7209116a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf29116f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xff891172, SUC_lexicon_tags_0 },
    { 0x9a491173, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19891177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61691179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e9117e, SUC_lexicon_tags_23 },
    { 0xa289117f, SUC_lexicon_tags_16 },
    { 0x7be91180, SUC_lexicon_tags_19 },
    { 0x4d891181, SUC_lexicon_tags_29 },
    { 0xc2891182, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc891185, SUC_lexicon_tags_63 },
    { 0x7c891186, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18491191, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca291193, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6891195, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e91199, SUC_lexicon_tags_31 },
    { 0x7949119a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c29119d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3c9119f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60911a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c911a8, SUC_lexicon_tags_2 },
    { 0xda911a8, SUC_lexicon_tags_23 },
    { 0x250911a8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8911ad, SUC_lexicon_tags_12 },
    { 0x528911ae, SUC_lexicon_tags_7 },
    { 0x546911ae, SUC_lexicon_tags_15 },
    { 0x386911ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ac911b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2911b5, SUC_lexicon_tags_63 },
    { 0x8d2911b5, SUC_lexicon_tags_21 },
    { 0xb56911b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba911ba, SUC_lexicon_tags_2 },
    { 0xce6911bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0911c1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb8911c3, SUC_lexicon_tags_37 },
    { 0xc2911c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x742911c6, SUC_lexicon_tags_25 },
    { 0x634911c7, SUC_lexicon_tags_25 },
    { 0x226911c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ee911ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x588911cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec911cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c911d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce4911d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e911db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa5c911dd, SUC_lexicon_tags_17 },
    { 0xc2911dd, SUC_lexicon_tags_12 },
    { 0x55e911dd, SUC_lexicon_tags_23 },
    { 0x812911e0, SUC_lexicon_tags_17 },
    { 0x66911dd, SUC_lexicon_tags_40 },
    { 0x382911e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb8911e4, SUC_lexicon_tags_13 },
    { 0xeba911e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2911ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1cc911ee, SUC_lexicon_tags_23 },
    { 0x32911ef, SUC_lexicon_tags_2 },
    { 0x160911f0, SUC_lexicon_tags_74 },
    { 0xc6a911f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a911f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe7e911f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x204911fa, SUC_lexicon_tags_16 },
    { 0xf9c911fb, SUC_lexicon_tags_25 },
    { 0xafa911fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c91205, SUC_lexicon_tags_27 },
    { 0xce891205, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb091208, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf809120a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e9120f, SUC_lexicon_tags_12 },
    { 0xd449120f, SUC_lexicon_tags_23 },
    { 0xfbe91211, SUC_lexicon_tags_31 },
    { 0x5b291210, SUC_lexicon_tags_48 },
    { 0x4049120f, SUC_lexicon_tags_17 },
    { 0x8f491214, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x14891217, SUC_lexicon_tags_19 },
    { 0x8f891218, SUC_lexicon_tags_49 },
    { 0xe4091219, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c9121c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e91226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f091229, SUC_lexicon_tags_6 },
    { 0xa409122a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfea9122c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc91236, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29123a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf829123f, SUC_lexicon_tags_7 },
    { 0x83c91240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfa91242, SUC_lexicon_tags_37 },
    { 0x4fe91242, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc91248, SUC_lexicon_tags_45 },
    { 0xbe91249, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde69124b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5a89124d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d291251, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe91255, SUC_lexicon_tags_15 },
    { 0x77291256, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac91259, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a9125f, SUC_lexicon_tags_25 },
    { 0x9891260, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20291265, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb69126a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc469126e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a91271, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb091273, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a891276, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac891278, SUC_lexicon_tags_1 },
    { 0x7de91279, SUC_lexicon_tags_14 },
    { 0x5f89127a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e49127e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e91281, SUC_lexicon_tags_56 },
    { 0x8d291282, SUC_lexicon_tags_37 },
    { 0xfbe91283, SUC_lexicon_tags_6 },
    { 0x9d691284, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f091287, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97091289, SUC_lexicon_tags_43 },
    { 0x12e9128a, SUC_lexicon_tags_1 },
    { 0xea091289, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd1c9128d, SUC_lexicon_tags_19 },
    { 0xc689128e, SUC_lexicon_tags_47 },
    { 0xf0c9128f, SUC_lexicon_tags_11 },
    { 0x96c91290, SUC_lexicon_tags_37 },
    { 0xc7891291, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa7c91293, SUC_lexicon_tags_63 },
    { 0x63691294, SUC_lexicon_tags_47 },
    { 0x87891295, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33691299, SUC_lexicon_tags_29 },
    { 0x8029129a, SUC_lexicon_tags_11 },
    { 0x3129129a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6a9129d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932912a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7a912a6, SUC_lexicon_tags_12 },
    { 0x6d8912a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6912ab, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xa0912ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81c912af, SUC_lexicon_tags_31 },
    { 0xc9c912b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a912b4, SUC_lexicon_tags_14 },
    { 0x732912b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a912bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244912c3, SUC_lexicon_tags_10 },
    { 0xae4912c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29c912c6, SUC_lexicon_tags_60 },
    { 0x752912c6, SUC_lexicon_tags_17 },
    { 0xf1e912c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa912cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92912d5, SUC_lexicon_tags_6 },
    { 0x4c6912d6, SUC_lexicon_tags_12 },
    { 0xf74912d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x150912d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a912dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c912e1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf0c912e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca912e7, SUC_lexicon_tags_68 },
    { 0x470912e8, SUC_lexicon_tags_10 },
    { 0x1a6912e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd2912eb, SUC_lexicon_tags_33 },
    { 0x998912eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x966912ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e912f2, SUC_lexicon_tags_55 },
    { 0xe70912f3, SUC_lexicon_tags_14 },
    { 0x7d0912f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x38912f7, SUC_lexicon_tags_0 },
    { 0xec4912f8, SUC_lexicon_tags_48 },
    { 0x69a912f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596912ff, SUC_lexicon_tags_45 },
    { 0x51691300, SUC_lexicon_tags_30 },
    { 0x7c891301, SUC_lexicon_tags_37 },
    { 0xca691302, SUC_lexicon_tags_60 },
    { 0xc8a91303, SUC_lexicon_tags_17 },
    { 0xaf291303, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5291308, SUC_lexicon_tags_45 },
    { 0x41a91308, SUC_lexicon_tags_68 },
    { 0x9e49130a, SUC_lexicon_tags_15 },
    { 0x9ec9130b, SUC_lexicon_tags_7 },
    { 0xb249130b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51691310, SUC_lexicon_tags_25 },
    { 0xd9891311, SUC_lexicon_tags_6 },
    { 0x5e691311, SUC_lexicon_tags_43 },
    { 0xf2691313, SUC_lexicon_tags_1 },
    { 0xf4091314, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e9131b, SUC_lexicon_tags_23 },
    { 0x12c9131c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe729131f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7091322, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9291326, SUC_lexicon_tags_1 },
    { 0xf4891327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be9132e, SUC_lexicon_tags_38 },
    { 0x3d69132e, SUC_lexicon_tags_8 },
    { 0x2329132e, SUC_lexicon_tags_47 },
    { 0xa0c9132f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41091337, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd8491339, SUC_lexicon_tags_47 },
    { 0x3609133a, SUC_lexicon_tags_37 },
    { 0xf469133b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92c9133e, SUC_lexicon_tags_15 },
    { 0x9809133f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd291342, SUC_lexicon_tags_23 },
    { 0x5d291342, SUC_lexicon_tags_15 },
    { 0xa2891342, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x73291347, SUC_lexicon_tags_33 },
    { 0x53c91348, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c9134c, SUC_lexicon_tags_7 },
    { 0x1609134d, SUC_lexicon_tags_37 },
    { 0xffa9134e, SUC_lexicon_tags_79 },
    { 0xbc09134d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22691352, SUC_lexicon_tags_14 },
    { 0x7de91353, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2c91355, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef891359, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec9135c, SUC_lexicon_tags_37 },
    { 0x5409135c, SUC_lexicon_tags_11 },
    { 0xcb49135c, SUC_lexicon_tags_25 },
    { 0x8e49135d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16e91361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee91364, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b09136a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb89136f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47c91371, SUC_lexicon_tags_15 },
    { 0x8ca91372, SUC_lexicon_tags_15 },
    { 0xcd691373, SUC_lexicon_tags_14 },
    { 0x9091374, SUC_lexicon_tags_12 },
    { 0x52291374, SUC_lexicon_tags_7 },
    { 0x45291374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45691379, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb491380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32691388, SUC_lexicon_tags_63 },
    { 0xafe91389, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6249138b, SUC_lexicon_tags_11 },
    { 0x8ce9138c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4769139d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6913a3, SUC_lexicon_tags_25 },
    { 0xee0913a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6913a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0913ac, SUC_lexicon_tags_54 },
    { 0xa9a913ad, SUC_lexicon_tags_14 },
    { 0xa96913ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc48913b0, SUC_lexicon_tags_23 },
    { 0x72a913b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8ba913b3, SUC_lexicon_tags_33 },
    { 0xd30913b4, SUC_lexicon_tags_2 },
    { 0xaa4913b5, SUC_lexicon_tags_25 },
    { 0xafa913b5, SUC_lexicon_tags_37 },
    { 0xf40913b5, SUC_lexicon_tags_39 },
    { 0xc8e913b5, SUC_lexicon_tags_26 },
    { 0xa7a913b5, SUC_lexicon_tags_17 },
    { 0xee913ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe8c913bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2de913be, SUC_lexicon_tags_1 },
    { 0x146913be, SUC_lexicon_tags_12 },
    { 0x59e913bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae913c4, SUC_lexicon_tags_17 },
    { 0xe34913c5, SUC_lexicon_tags_11 },
    { 0x57c913c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4913c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf24913cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe4e913cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e913d3, SUC_lexicon_tags_66 },
    { 0x97c913d4, SUC_lexicon_tags_7 },
    { 0x29c913d5, SUC_lexicon_tags_335 },
    { 0x9e8913d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ae913d8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd3a913da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8913de, SUC_lexicon_tags_31 },
    { 0x994913df, SUC_lexicon_tags_34 },
    { 0xb3c913e0, SUC_lexicon_tags_1 },
    { 0x930913e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c913e4, SUC_lexicon_tags_13 },
    { 0x8ce913e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6913ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba913f2, SUC_lexicon_tags_27 },
    { 0x502913f3, SUC_lexicon_tags_37 },
    { 0x588913f4, SUC_lexicon_tags_158 },
    { 0x6e6913f4, SUC_lexicon_tags_43 },
    { 0x6fe913f4, SUC_lexicon_tags_38 },
    { 0xc22913f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0913fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e2913fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a291403, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf089140d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0491418, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2269141a, SUC_lexicon_tags_29 },
    { 0x449141a, SUC_lexicon_tags_23 },
    { 0x12e9141c, SUC_lexicon_tags_37 },
    { 0x85c9141c, SUC_lexicon_tags_14 },
    { 0x4929141c, SUC_lexicon_tags_60 },
    { 0x8e69141f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x77691421, SUC_lexicon_tags_2 },
    { 0xa1091421, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4491426, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec69142d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3529142f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc1091431, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48891433, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f69143d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec91442, SUC_lexicon_tags_17 },
    { 0x99c91442, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x59891445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1809144a, SUC_lexicon_tags_155 },
    { 0, NULL },
    { 0x9ee9144c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x6389144e, SUC_lexicon_tags_15 },
    { 0xdea9144f, SUC_lexicon_tags_0 },
    { 0x56291450, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3091453, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea91456, SUC_lexicon_tags_82 },
    { 0x57e91457, SUC_lexicon_tags_15 },
    { 0x92291458, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e9145b, SUC_lexicon_tags_4 },
    { 0x2c49145b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c491460, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b891462, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91464, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc91468, SUC_lexicon_tags_62 },
    { 0xd5291468, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e91470, SUC_lexicon_tags_21 },
    { 0x5a691470, SUC_lexicon_tags_9 },
    { 0x7d091472, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1891474, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d691477, SUC_lexicon_tags_179 },
    { 0x1c891478, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7289147b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e9147d, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc91484, SUC_lexicon_tags_1 },
    { 0x8491485, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55a91487, SUC_lexicon_tags_45 },
    { 0x6be91488, SUC_lexicon_tags_37 },
    { 0x57e91489, SUC_lexicon_tags_47 },
    { 0xa089148a, SUC_lexicon_tags_31 },
    { 0xd7891488, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae9148e, SUC_lexicon_tags_17 },
    { 0x8ca9148f, SUC_lexicon_tags_33 },
    { 0xe0891490, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x65891492, SUC_lexicon_tags_23 },
    { 0xb3491492, SUC_lexicon_tags_63 },
    { 0x3e291494, SUC_lexicon_tags_7 },
    { 0xb4091492, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d691498, SUC_lexicon_tags_49 },
    { 0x42a91499, SUC_lexicon_tags_27 },
    { 0x8c69149a, SUC_lexicon_tags_38 },
    { 0xfc9149b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a914a1, SUC_lexicon_tags_45 },
    { 0x176914a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeac914a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc914a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x650914aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830914af, SUC_lexicon_tags_25 },
    { 0x212914b0, SUC_lexicon_tags_19 },
    { 0xaa0914af, SUC_lexicon_tags_4 },
    { 0xd7a914b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92914bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x358914c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8914c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56914c9, SUC_lexicon_tags_12 },
    { 0xbae914ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x966914cd, SUC_lexicon_tags_63 },
    { 0xbaa914cd, SUC_lexicon_tags_48 },
    { 0x1a8914ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c914d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd48914d8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x70914db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230914df, SUC_lexicon_tags_17 },
    { 0xbc8914e0, SUC_lexicon_tags_1 },
    { 0xdbe914e0, SUC_lexicon_tags_25 },
    { 0x522914e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8914e5, SUC_lexicon_tags_87 },
    { 0x58a914e5, SUC_lexicon_tags_38 },
    { 0x1c4914e7, SUC_lexicon_tags_14 },
    { 0xb5a914e8, SUC_lexicon_tags_17 },
    { 0xcf2914e9, SUC_lexicon_tags_238 },
    { 0x546914e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca4914ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfd6914ee, SUC_lexicon_tags_64 },
    { 0x946914ef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x296914f1, SUC_lexicon_tags_0 },
    { 0xb54914f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0914f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112914fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb891500, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5491504, SUC_lexicon_tags_26 },
    { 0xc291505, SUC_lexicon_tags_11 },
    { 0xf8091506, SUC_lexicon_tags_12 },
    { 0x3e091506, SUC_lexicon_tags_0 },
    { 0xcb891508, SUC_lexicon_tags_19 },
    { 0xdb891509, SUC_lexicon_tags_33 },
    { 0x48291506, SUC_lexicon_tags_15 },
    { 0x82c91506, SUC_lexicon_tags_46 },
    { 0xd9c9150c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491511, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x62291514, SUC_lexicon_tags_26 },
    { 0xc0891515, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8149151a, SUC_lexicon_tags_2 },
    { 0xba09151a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd489151f, SUC_lexicon_tags_14 },
    { 0x61491520, SUC_lexicon_tags_38 },
    { 0x75091521, SUC_lexicon_tags_43 },
    { 0xdf291521, SUC_lexicon_tags_25 },
    { 0xf9891521, SUC_lexicon_tags_12 },
    { 0x36091524, SUC_lexicon_tags_25 },
    { 0x47a91525, SUC_lexicon_tags_11 },
    { 0xc4291525, SUC_lexicon_tags_12 },
    { 0x1ec91525, SUC_lexicon_tags_38 },
    { 0x90a91528, SUC_lexicon_tags_45 },
    { 0xb5c91527, SUC_lexicon_tags_30 },
    { 0xc669152a, SUC_lexicon_tags_25 },
    { 0xed891525, SUC_lexicon_tags_22 },
    { 0x7549152c, SUC_lexicon_tags_37 },
    { 0x10e9152d, SUC_lexicon_tags_10 },
    { 0x50491526, SUC_lexicon_tags_15 },
    { 0xe0c9152e, SUC_lexicon_tags_19 },
    { 0xaa091525, SUC_lexicon_tags_0 },
    { 0x9da91530, SUC_lexicon_tags_19 },
    { 0x94c91527, SUC_lexicon_tags_92 },
    { 0xfb291533, SUC_lexicon_tags_25 },
    { 0x18491533, SUC_lexicon_tags_17 },
    { 0xa691534, SUC_lexicon_tags_29 },
    { 0xa5291536, SUC_lexicon_tags_14 },
    { 0x491536, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd409153b, SUC_lexicon_tags_7 },
    { 0x9189153c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc891540, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48e91542, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xac291544, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71491546, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x14491548, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2869154c, SUC_lexicon_tags_1 },
    { 0x6949154d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ee9154f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6291551, SUC_lexicon_tags_11 },
    { 0xae091551, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6491555, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe091558, SUC_lexicon_tags_29 },
    { 0x69891559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e9155c, SUC_lexicon_tags_17 },
    { 0x70e9155d, SUC_lexicon_tags_19 },
    { 0x1aa9155e, SUC_lexicon_tags_0 },
    { 0x71c9155f, SUC_lexicon_tags_25 },
    { 0xb1c91560, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa91565, SUC_lexicon_tags_1 },
    { 0xae891565, SUC_lexicon_tags_11 },
    { 0x18891566, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x2029156a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc91574, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c691579, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea9157c, SUC_lexicon_tags_37 },
    { 0x3d69157d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b291580, SUC_lexicon_tags_15 },
    { 0x28691581, SUC_lexicon_tags_7 },
    { 0xe0491581, SUC_lexicon_tags_23 },
    { 0x10291582, SUC_lexicon_tags_58 },
    { 0xe91584, SUC_lexicon_tags_0 },
    { 0xc6691584, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9a91587, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe3891589, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a9158d, SUC_lexicon_tags_37 },
    { 0x23a9158e, SUC_lexicon_tags_1 },
    { 0x9bc9158f, SUC_lexicon_tags_37 },
    { 0x1489158f, SUC_lexicon_tags_0 },
    { 0xd8891591, SUC_lexicon_tags_17 },
    { 0x25e91591, SUC_lexicon_tags_15 },
    { 0xdac91593, SUC_lexicon_tags_37 },
    { 0x5c491593, SUC_lexicon_tags_12 },
    { 0x92491594, SUC_lexicon_tags_25 },
    { 0x51691596, SUC_lexicon_tags_6 },
    { 0x74491597, SUC_lexicon_tags_15 },
    { 0xa6891591, SUC_lexicon_tags_40 },
    { 0xbe291596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca9159d, SUC_lexicon_tags_65 },
    { 0x3829159e, SUC_lexicon_tags_56 },
    { 0xad09159f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd08915a1, SUC_lexicon_tags_28 },
    { 0x4c6915a2, SUC_lexicon_tags_11 },
    { 0xaae915a3, SUC_lexicon_tags_1 },
    { 0x690915a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b8915a6, SUC_lexicon_tags_7 },
    { 0x87c915a6, SUC_lexicon_tags_15 },
    { 0xf9c915a8, SUC_lexicon_tags_11 },
    { 0x9fa915a9, SUC_lexicon_tags_43 },
    { 0xaae915aa, SUC_lexicon_tags_19 },
    { 0x8915a9, SUC_lexicon_tags_11 },
    { 0x2ee915a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x382915ae, SUC_lexicon_tags_1 },
    { 0x66915af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0915b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91a915b6, SUC_lexicon_tags_7 },
    { 0x95c915b7, SUC_lexicon_tags_14 },
    { 0xe7e915b7, SUC_lexicon_tags_15 },
    { 0xd86915b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x604915bc, SUC_lexicon_tags_29 },
    { 0x9a4915bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x900915bf, SUC_lexicon_tags_21 },
    { 0xb98915c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94915c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd12915c4, SUC_lexicon_tags_12 },
    { 0xf04915c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb14915c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa34915c9, SUC_lexicon_tags_60 },
    { 0x718915ca, SUC_lexicon_tags_37 },
    { 0x6da915cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc915d0, SUC_lexicon_tags_434 },
    { 0x4d6915d1, SUC_lexicon_tags_33 },
    { 0x502915d2, SUC_lexicon_tags_25 },
    { 0x82e915d1, SUC_lexicon_tags_15 },
    { 0xbf4915d1, SUC_lexicon_tags_65 },
    { 0xb3c915d2, SUC_lexicon_tags_68 },
    { 0x362915d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c915dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834915e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf0c915e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0c915e6, SUC_lexicon_tags_23 },
    { 0x4c2915e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x492915ea, SUC_lexicon_tags_19 },
    { 0x568915eb, SUC_lexicon_tags_7 },
    { 0x41c915ec, SUC_lexicon_tags_32 },
    { 0x6e8915ed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e915f0, SUC_lexicon_tags_45 },
    { 0xdde915f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16915f5, SUC_lexicon_tags_11 },
    { 0xf0915f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2915fa, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6915ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4291604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8869160a, SUC_lexicon_tags_7 },
    { 0x7a49160b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea49160e, SUC_lexicon_tags_14 },
    { 0x8029160f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a91615, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa7e91617, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0891619, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab09161d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x37691620, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e691628, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c69162e, SUC_lexicon_tags_64 },
    { 0xc329162f, SUC_lexicon_tags_7 },
    { 0xfea91630, SUC_lexicon_tags_25 },
    { 0xf9891631, SUC_lexicon_tags_13 },
    { 0x2b291632, SUC_lexicon_tags_7 },
    { 0xb0291633, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2291636, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab691638, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3349163b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a91644, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb769164c, SUC_lexicon_tags_25 },
    { 0xb169164d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef291653, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a9165b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11291661, SUC_lexicon_tags_17 },
    { 0x40491662, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b891668, SUC_lexicon_tags_12 },
    { 0x6ea91669, SUC_lexicon_tags_16 },
    { 0x5de91668, SUC_lexicon_tags_37 },
    { 0xbde91668, SUC_lexicon_tags_49 },
    { 0xe489166a, SUC_lexicon_tags_7 },
    { 0x33a9166d, SUC_lexicon_tags_1109 },
    { 0x1d69166d, SUC_lexicon_tags_43 },
    { 0xe149166f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc91673, SUC_lexicon_tags_806 },
    { 0x45291674, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc91679, SUC_lexicon_tags_2 },
    { 0x8dc9167a, SUC_lexicon_tags_21 },
    { 0xd0691679, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc91680, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e91686, SUC_lexicon_tags_45 },
    { 0xa6691687, SUC_lexicon_tags_12 },
    { 0x91091688, SUC_lexicon_tags_45 },
    { 0xa7e91688, SUC_lexicon_tags_17 },
    { 0x1d49168a, SUC_lexicon_tags_152 },
    { 0xf491688, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3ae9168d, SUC_lexicon_tags_28 },
    { 0xf369168e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac891690, SUC_lexicon_tags_26 },
    { 0x5ca91691, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab091695, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf029169a, SUC_lexicon_tags_38 },
    { 0x4ac9169b, SUC_lexicon_tags_25 },
    { 0xcc9169c, SUC_lexicon_tags_23 },
    { 0x1509169d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfec9169f, SUC_lexicon_tags_15 },
    { 0x9fe916a0, SUC_lexicon_tags_43 },
    { 0x8ca916a0, SUC_lexicon_tags_37 },
    { 0x9d6916a2, SUC_lexicon_tags_37 },
    { 0x590916a1, SUC_lexicon_tags_8 },
    { 0xdd4916a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcca916a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc916a9, SUC_lexicon_tags_38 },
    { 0xd12916aa, SUC_lexicon_tags_21 },
    { 0x47e916ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276916b1, SUC_lexicon_tags_12 },
    { 0x7ec916b2, SUC_lexicon_tags_84 },
    { 0xa20916b3, SUC_lexicon_tags_11 },
    { 0x558916b4, SUC_lexicon_tags_15 },
    { 0x70c916b4, SUC_lexicon_tags_17 },
    { 0x19c916b5, SUC_lexicon_tags_25 },
    { 0x4ba916b7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd36916b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e916bc, SUC_lexicon_tags_14 },
    { 0xeaa916bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c916c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67a916c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd78916c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4916c9, SUC_lexicon_tags_14 },
    { 0xc8a916ca, SUC_lexicon_tags_7 },
    { 0x428916cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c916ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30916d2, SUC_lexicon_tags_1 },
    { 0xd00916d3, SUC_lexicon_tags_85 },
    { 0xfd6916d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x904916d7, SUC_lexicon_tags_834 },
    { 0x188916d8, SUC_lexicon_tags_22 },
    { 0xd9c916d9, SUC_lexicon_tags_30 },
    { 0x92a916da, SUC_lexicon_tags_12 },
    { 0x29a916db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x60916de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846916e4, SUC_lexicon_tags_70 },
    { 0x6a6916e4, SUC_lexicon_tags_174 },
    { 0x3f6916e6, SUC_lexicon_tags_15 },
    { 0xbf4916e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfbe916e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8fc916eb, SUC_lexicon_tags_61 },
    { 0x794916eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x20e916ee, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4916f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70916f4, SUC_lexicon_tags_1 },
    { 0x8f4916f5, SUC_lexicon_tags_27 },
    { 0x91c916f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618916fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2691701, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a091705, SUC_lexicon_tags_14 },
    { 0x55c91705, SUC_lexicon_tags_37 },
    { 0xf8291707, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9170a, SUC_lexicon_tags_17 },
    { 0x3009170b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe91710, SUC_lexicon_tags_2 },
    { 0xed091710, SUC_lexicon_tags_23 },
    { 0x51891711, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc49171b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c91721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44291723, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea91728, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe09172d, SUC_lexicon_tags_23 },
    { 0xc289172d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24291731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e91734, SUC_lexicon_tags_7 },
    { 0x80e91734, SUC_lexicon_tags_15 },
    { 0xd5891736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3669173a, SUC_lexicon_tags_60 },
    { 0x3c89173a, SUC_lexicon_tags_2 },
    { 0xa069173a, SUC_lexicon_tags_25 },
    { 0x8b69173d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc9173f, SUC_lexicon_tags_10 },
    { 0xf5a91740, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72491742, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf491744, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56291748, SUC_lexicon_tags_2 },
    { 0x23491749, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1369174b, SUC_lexicon_tags_38 },
    { 0xea49174c, SUC_lexicon_tags_2 },
    { 0xe929174c, SUC_lexicon_tags_30 },
    { 0x5629174b, SUC_lexicon_tags_25 },
    { 0xe509174d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c91753, SUC_lexicon_tags_7 },
    { 0x94e91753, SUC_lexicon_tags_1 },
    { 0xde91754, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0091758, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b09175d, SUC_lexicon_tags_7 },
    { 0x7b49175e, SUC_lexicon_tags_17 },
    { 0xd869175f, SUC_lexicon_tags_15 },
    { 0x8569175f, SUC_lexicon_tags_15 },
    { 0x58e9175d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a91764, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e91767, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69176b, SUC_lexicon_tags_7 },
    { 0x5749176c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x42c9176e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e91771, SUC_lexicon_tags_11 },
    { 0x72e91771, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa91776, SUC_lexicon_tags_17 },
    { 0xfda91776, SUC_lexicon_tags_0 },
    { 0x55e91778, SUC_lexicon_tags_15 },
    { 0x18291777, SUC_lexicon_tags_60 },
    { 0x12a91777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce9177e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a91783, SUC_lexicon_tags_1 },
    { 0xd5c91784, SUC_lexicon_tags_23 },
    { 0x21491785, SUC_lexicon_tags_17 },
    { 0xa4a91786, SUC_lexicon_tags_49 },
    { 0x43291786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8091789, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35091791, SUC_lexicon_tags_37 },
    { 0xffc91792, SUC_lexicon_tags_11 },
    { 0x6b891793, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c691795, SUC_lexicon_tags_32 },
    { 0x2691795, SUC_lexicon_tags_23 },
    { 0x6e291797, SUC_lexicon_tags_63 },
    { 0xcf091798, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89179b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb929179f, SUC_lexicon_tags_23 },
    { 0x78917a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36917a3, SUC_lexicon_tags_47 },
    { 0x4be917a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa917a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x26e917a9, SUC_lexicon_tags_25 },
    { 0xd58917aa, SUC_lexicon_tags_14 },
    { 0x84e917aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1a917ad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x634917b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2917b3, SUC_lexicon_tags_1 },
    { 0x968917b4, SUC_lexicon_tags_11 },
    { 0x972917b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c917b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2917bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588917c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60917cf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x38a917d1, SUC_lexicon_tags_37 },
    { 0x5dc917d1, SUC_lexicon_tags_26 },
    { 0xe30917d3, SUC_lexicon_tags_15 },
    { 0xfca917d3, SUC_lexicon_tags_12 },
    { 0x640917d5, SUC_lexicon_tags_44 },
    { 0xed0917d6, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240917df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c917e4, SUC_lexicon_tags_15 },
    { 0x44c917e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786917ec, SUC_lexicon_tags_12 },
    { 0x472917ed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42917fb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e917ff, SUC_lexicon_tags_38 },
    { 0x64291800, SUC_lexicon_tags_42 },
    { 0x23491800, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8091804, SUC_lexicon_tags_15 },
    { 0x93491804, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb149180d, SUC_lexicon_tags_30 },
    { 0x3749180e, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb491811, SUC_lexicon_tags_2 },
    { 0xa2691812, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13291817, SUC_lexicon_tags_15 },
    { 0x79c91818, SUC_lexicon_tags_27 },
    { 0xa2a91819, SUC_lexicon_tags_23 },
    { 0xfd491819, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xefa9181c, SUC_lexicon_tags_16 },
    { 0xe109181c, SUC_lexicon_tags_10 },
    { 0xed69181e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xbf691820, SUC_lexicon_tags_2 },
    { 0x19091821, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3e691823, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfca91825, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6291827, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef69182b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e9182f, SUC_lexicon_tags_19 },
    { 0xd9a91830, SUC_lexicon_tags_12 },
    { 0x56491831, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c91834, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc091836, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa91839, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f09183b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xa289183e, SUC_lexicon_tags_28 },
    { 0x11e9183e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32091841, SUC_lexicon_tags_25 },
    { 0x61091841, SUC_lexicon_tags_33 },
    { 0xd8a91842, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf9491845, SUC_lexicon_tags_23 },
    { 0x1de91845, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7949184e, SUC_lexicon_tags_108 },
    { 0xf289184f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5491853, SUC_lexicon_tags_25 },
    { 0xe1091854, SUC_lexicon_tags_33 },
    { 0x85091855, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22091859, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb569185b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2491860, SUC_lexicon_tags_37 },
    { 0x8dc91860, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d291867, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62c91869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b09186e, SUC_lexicon_tags_11 },
    { 0x5869186e, SUC_lexicon_tags_21 },
    { 0x28a91870, SUC_lexicon_tags_45 },
    { 0x45c91871, SUC_lexicon_tags_178 },
    { 0x23291872, SUC_lexicon_tags_38 },
    { 0xe0891873, SUC_lexicon_tags_0 },
    { 0x34a9186e, SUC_lexicon_tags_56 },
    { 0xad491870, SUC_lexicon_tags_25 },
    { 0x5b691872, SUC_lexicon_tags_47 },
    { 0x35c91877, SUC_lexicon_tags_87 },
    { 0x5f691875, SUC_lexicon_tags_71 },
    { 0x12e9186e, SUC_lexicon_tags_33 },
    { 0xa8a9187a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d49187e, SUC_lexicon_tags_28 },
    { 0x6189187f, SUC_lexicon_tags_17 },
    { 0xcd891880, SUC_lexicon_tags_27 },
    { 0xa649187f, SUC_lexicon_tags_39 },
    { 0x3a891882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76891889, SUC_lexicon_tags_33 },
    { 0x3589188a, SUC_lexicon_tags_12 },
    { 0x4849188a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f09188e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x80691890, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x39091893, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7ce91895, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x64291897, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3029189c, SUC_lexicon_tags_35 },
    { 0x9869189d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2918a6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x240918a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbc4918aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638918ae, SUC_lexicon_tags_30 },
    { 0xa20918ae, SUC_lexicon_tags_15 },
    { 0xbae918b0, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x5ae918b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb94918b4, SUC_lexicon_tags_19 },
    { 0xd08918b5, SUC_lexicon_tags_43 },
    { 0xad8918b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc26918b8, SUC_lexicon_tags_23 },
    { 0xb56918b9, SUC_lexicon_tags_27 },
    { 0x68c918ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a918bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb2a918bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6918c2, SUC_lexicon_tags_46 },
    { 0x9e6918c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72c918c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b2918c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x13a918c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x338918cb, SUC_lexicon_tags_8 },
    { 0x816918cc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e918d1, SUC_lexicon_tags_48 },
    { 0x5da918d2, SUC_lexicon_tags_92 },
    { 0x5fe918d2, SUC_lexicon_tags_19 },
    { 0x384918d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0e918d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e918db, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8918e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534918f1, SUC_lexicon_tags_66 },
    { 0x250918f2, SUC_lexicon_tags_11 },
    { 0x800918f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x524918f6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4918fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c091900, SUC_lexicon_tags_39 },
    { 0x6e091901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e91904, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0091906, SUC_lexicon_tags_55 },
    { 0xa4691907, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a091909, SUC_lexicon_tags_11 },
    { 0x2629190a, SUC_lexicon_tags_11 },
    { 0x3009190b, SUC_lexicon_tags_37 },
    { 0x7409190c, SUC_lexicon_tags_45 },
    { 0x7d09190c, SUC_lexicon_tags_2 },
    { 0xdb49190d, SUC_lexicon_tags_11 },
    { 0xab69190f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91491911, SUC_lexicon_tags_37 },
    { 0xc0691912, SUC_lexicon_tags_0 },
    { 0x71a91913, SUC_lexicon_tags_33 },
    { 0x33691914, SUC_lexicon_tags_37 },
    { 0x26091914, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9191b, SUC_lexicon_tags_1 },
    { 0xf529191b, SUC_lexicon_tags_63 },
    { 0x8229191b, SUC_lexicon_tags_37 },
    { 0x26a9191e, SUC_lexicon_tags_43 },
    { 0x2069191f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5891924, SUC_lexicon_tags_263 },
    { 0xb091925, SUC_lexicon_tags_46 },
    { 0x56691924, SUC_lexicon_tags_11 },
    { 0x42091927, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a91929, SUC_lexicon_tags_38 },
    { 0x37e91929, SUC_lexicon_tags_9 },
    { 0x5e49192b, SUC_lexicon_tags_11 },
    { 0xcf49192c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb609192e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e91934, SUC_lexicon_tags_46 },
    { 0x27a91934, SUC_lexicon_tags_12 },
    { 0xcc891936, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a91939, SUC_lexicon_tags_66 },
    { 0x1409193a, SUC_lexicon_tags_7 },
    { 0x1f69193a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3409193f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac91945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe869194f, SUC_lexicon_tags_1 },
    { 0x56e91950, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c91955, SUC_lexicon_tags_12 },
    { 0x82691956, SUC_lexicon_tags_1 },
    { 0x12e91957, SUC_lexicon_tags_23 },
    { 0xa891958, SUC_lexicon_tags_37 },
    { 0xe4c91959, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae9195e, SUC_lexicon_tags_6 },
    { 0x9c89195e, SUC_lexicon_tags_37 },
    { 0xec09195f, SUC_lexicon_tags_65 },
    { 0x3e491960, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6891965, SUC_lexicon_tags_36 },
    { 0xd2091965, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd89196b, SUC_lexicon_tags_56 },
    { 0xae49196c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4491975, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f69197a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa9197d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a91983, SUC_lexicon_tags_15 },
    { 0x99e91984, SUC_lexicon_tags_107 },
    { 0x3c291984, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9629198a, SUC_lexicon_tags_10 },
    { 0x17a9198b, SUC_lexicon_tags_25 },
    { 0xb629198c, SUC_lexicon_tags_37 },
    { 0xe3c9198c, SUC_lexicon_tags_2 },
    { 0x7e29198e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x36491990, SUC_lexicon_tags_12 },
    { 0xcb491990, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x89091994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32c91996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24691999, SUC_lexicon_tags_14 },
    { 0x93c9199a, SUC_lexicon_tags_2 },
    { 0x96c9199a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae9199f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766919a5, SUC_lexicon_tags_15 },
    { 0xcb0919a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x994919a9, SUC_lexicon_tags_27 },
    { 0x568919aa, SUC_lexicon_tags_101 },
    { 0x800919aa, SUC_lexicon_tags_35 },
    { 0x168919ac, SUC_lexicon_tags_19 },
    { 0xf14919ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c919b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d2919b4, SUC_lexicon_tags_11 },
    { 0x6ce919b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x944919b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0919ba, SUC_lexicon_tags_17 },
    { 0x2ae919bb, SUC_lexicon_tags_37 },
    { 0xf9a919bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a919c5, SUC_lexicon_tags_68 },
    { 0x4b4919c6, SUC_lexicon_tags_31 },
    { 0xcd8919c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0919cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4919cf, SUC_lexicon_tags_15 },
    { 0x8ac919d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x304919d2, SUC_lexicon_tags_17 },
    { 0xd88919d3, SUC_lexicon_tags_38 },
    { 0xf74919d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70919d7, SUC_lexicon_tags_28 },
    { 0xcf6919d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52919dc, SUC_lexicon_tags_1 },
    { 0x44e919dd, SUC_lexicon_tags_17 },
    { 0xeca919de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da919e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x674919e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428919ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcd6919ec, SUC_lexicon_tags_27 },
    { 0x7a4919ed, SUC_lexicon_tags_83 },
    { 0x4e6919ee, SUC_lexicon_tags_25 },
    { 0x30919ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x652919f1, SUC_lexicon_tags_15 },
    { 0xc82919f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c919f6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x902919f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164919ff, SUC_lexicon_tags_25 },
    { 0x15691a00, SUC_lexicon_tags_19 },
    { 0xbdc91a01, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xfde91a03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88091a0e, SUC_lexicon_tags_30 },
    { 0xad491a0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa691a13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc291a15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa2491a17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e91a1a, SUC_lexicon_tags_17 },
    { 0x62a91a1b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e491a1f, SUC_lexicon_tags_43 },
    { 0x5f291a20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c91a23, SUC_lexicon_tags_85 },
    { 0x24291a24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7091a26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88a91a28, SUC_lexicon_tags_92 },
    { 0x76691a29, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36491a32, SUC_lexicon_tags_59 },
    { 0x42891a33, SUC_lexicon_tags_37 },
    { 0xcf491a33, SUC_lexicon_tags_15 },
    { 0x8fc91a33, SUC_lexicon_tags_14 },
    { 0x13491a34, SUC_lexicon_tags_1 },
    { 0x2e691a37, SUC_lexicon_tags_12 },
    { 0x81291a38, SUC_lexicon_tags_28 },
    { 0x85891a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ee91a3b, SUC_lexicon_tags_37 },
    { 0x88e91a3c, SUC_lexicon_tags_7 },
    { 0x5d091a3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1491a41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ec91a43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34a91a45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e91a4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x15091a4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae91a52, SUC_lexicon_tags_47 },
    { 0x8e891a52, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x36691a55, SUC_lexicon_tags_30 },
    { 0xd2891a56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f891a5b, SUC_lexicon_tags_14 },
    { 0x59e91a5c, SUC_lexicon_tags_1 },
    { 0xbaa91a5d, SUC_lexicon_tags_50 },
    { 0x8b891a5c, SUC_lexicon_tags_63 },
    { 0x3d691a5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14691a62, SUC_lexicon_tags_48 },
    { 0x37891a63, SUC_lexicon_tags_25 },
    { 0xc2291a64, SUC_lexicon_tags_17 },
    { 0xf2c91a64, SUC_lexicon_tags_2 },
    { 0x7491a66, SUC_lexicon_tags_25 },
    { 0x7e691a65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9491a69, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xac091a6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ea91a6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aa91a6f, SUC_lexicon_tags_27 },
    { 0xe4e91a70, SUC_lexicon_tags_11 },
    { 0xe7691a6f, SUC_lexicon_tags_11 },
    { 0x13091a72, SUC_lexicon_tags_55 },
    { 0x42291a72, SUC_lexicon_tags_17 },
    { 0xc9091a74, SUC_lexicon_tags_12 },
    { 0xfa291a75, SUC_lexicon_tags_25 },
    { 0x8e691a75, SUC_lexicon_tags_7 },
    { 0x8ce91a77, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa9291a79, SUC_lexicon_tags_17 },
    { 0xe9a91a7a, SUC_lexicon_tags_27 },
    { 0x23891a7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a691a7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dc91a80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b491a86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d691a88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc491a8f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1a91a91, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc91a94, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa91a98, SUC_lexicon_tags_37 },
    { 0xd1891a99, SUC_lexicon_tags_14 },
    { 0x4891a99, SUC_lexicon_tags_43 },
    { 0x76e91a99, SUC_lexicon_tags_11 },
    { 0x2ec91a9c, SUC_lexicon_tags_5 },
    { 0xb0091a98, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x4c891a9f, SUC_lexicon_tags_11 },
    { 0xd9c91aa0, SUC_lexicon_tags_2 },
    { 0x3cc91aa1, SUC_lexicon_tags_38 },
    { 0x70e91a9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xae691aa5, SUC_lexicon_tags_1 },
    { 0xb8e91aa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23a91aa8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6691aab, SUC_lexicon_tags_16 },
    { 0x2d691aac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7a91aae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x87491ab1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b491ab5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe091ab8, SUC_lexicon_tags_7 },
    { 0x2c291ab9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3691ac0, SUC_lexicon_tags_37 },
    { 0xa91ac1, SUC_lexicon_tags_2 },
    { 0xb4091ac2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90091ac8, SUC_lexicon_tags_55 },
    { 0x9ce91ac9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb5091acb, SUC_lexicon_tags_12 },
    { 0x39c91acc, SUC_lexicon_tags_297 },
    { 0x5e91acb, SUC_lexicon_tags_1 },
    { 0x1fa91ace, SUC_lexicon_tags_27 },
    { 0xaa091ace, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e291ad1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9491ad5, SUC_lexicon_tags_37 },
    { 0xbe691ad6, SUC_lexicon_tags_11 },
    { 0x1a91ad6, SUC_lexicon_tags_7 },
    { 0x72891ad8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e291adc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a91ae5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76891ae8, SUC_lexicon_tags_51 },
    { 0x7b891ae9, SUC_lexicon_tags_15 },
    { 0xfb091ae9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1fa91aec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb291af2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22c91af4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9291af7, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae291afb, SUC_lexicon_tags_29 },
    { 0x22c91afc, SUC_lexicon_tags_79 },
    { 0x24891afd, SUC_lexicon_tags_2 },
    { 0x45291afc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97491b05, SUC_lexicon_tags_10 },
    { 0xaca91b05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce91b09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5aa91b0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b691b11, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc91b16, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0491b1c, SUC_lexicon_tags_9 },
    { 0x3f691b1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89891b1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc91b24, SUC_lexicon_tags_17 },
    { 0xa2091b25, SUC_lexicon_tags_57 },
    { 0xdc91b24, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f291b29, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49491b2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc91b31, SUC_lexicon_tags_23 },
    { 0x29291b32, SUC_lexicon_tags_45 },
    { 0x50a91b32, SUC_lexicon_tags_17 },
    { 0x79091b31, SUC_lexicon_tags_37 },
    { 0xf7e91b34, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14291b3d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x50091b3f, SUC_lexicon_tags_10 },
    { 0xd2491b3f, SUC_lexicon_tags_9 },
    { 0x13091b3f, SUC_lexicon_tags_31 },
    { 0xc91b41, SUC_lexicon_tags_17 },
    { 0x84691b40, SUC_lexicon_tags_65 },
    { 0xef091b44, SUC_lexicon_tags_7 },
    { 0xc3691b40, SUC_lexicon_tags_27 },
    { 0xc9a91b46, SUC_lexicon_tags_27 },
    { 0xdec91b47, SUC_lexicon_tags_6 },
    { 0x13091b48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd891b4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf291b4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x97a91b4f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae91b53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18891b57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x13491b5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d291b5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e91b65, SUC_lexicon_tags_16 },
    { 0x64a91b65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6091b6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a291b6f, SUC_lexicon_tags_38 },
    { 0xfda91b70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35291b72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a91b76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b691b7c, SUC_lexicon_tags_11 },
    { 0x30891b7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb091b81, SUC_lexicon_tags_17 },
    { 0xf5c91b82, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xd4891b84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde91b87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc291b8c, SUC_lexicon_tags_1 },
    { 0xbde91b8d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4bc91b8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72691b97, SUC_lexicon_tags_31 },
    { 0x49e91b98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30691b9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c491b9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c91ba7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5291ba9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa91bae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba91bb1, SUC_lexicon_tags_4 },
    { 0x1e491bb1, SUC_lexicon_tags_15 },
    { 0x6e491bb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe91bb6, SUC_lexicon_tags_37 },
    { 0x7ea91bb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5aa91bb9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee91bbe, SUC_lexicon_tags_25 },
    { 0xab691bbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c91bc5, SUC_lexicon_tags_1 },
    { 0x63a91bc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf291bc8, SUC_lexicon_tags_43 },
    { 0x8ce91bc8, SUC_lexicon_tags_1 },
    { 0xb9291bca, SUC_lexicon_tags_37 },
    { 0xabe91bc8, SUC_lexicon_tags_12 },
    { 0x52291bcc, SUC_lexicon_tags_12 },
    { 0x6b291bcd, SUC_lexicon_tags_12 },
    { 0xd7291bce, SUC_lexicon_tags_17 },
    { 0x11e91bcf, SUC_lexicon_tags_19 },
    { 0x26e91bd0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa6691bd2, SUC_lexicon_tags_267 },
    { 0xd6091bd3, SUC_lexicon_tags_17 },
    { 0xb4491bd4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0491bd7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6291bdb, SUC_lexicon_tags_29 },
    { 0x6de91bdb, SUC_lexicon_tags_7 },
    { 0x64091bdb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f691be0, SUC_lexicon_tags_12 },
    { 0x62891be0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x87691be3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a91bea, SUC_lexicon_tags_55 },
    { 0x97e91bea, SUC_lexicon_tags_46 },
    { 0x8ca91bec, SUC_lexicon_tags_1 },
    { 0x11691bed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba91bf0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc091bf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0691bf8, SUC_lexicon_tags_27 },
    { 0x8ca91bf9, SUC_lexicon_tags_25 },
    { 0x67e91bfa, SUC_lexicon_tags_43 },
    { 0x7da91bfb, SUC_lexicon_tags_11 },
    { 0x3e891bfa, SUC_lexicon_tags_25 },
    { 0x92e91bfd, SUC_lexicon_tags_19 },
    { 0x2091bfb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c91c09, SUC_lexicon_tags_25 },
    { 0xbe91c0a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x99c91c0c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93091c10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x71491c12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8691c1c, SUC_lexicon_tags_151 },
    { 0xb2691c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb491c25, SUC_lexicon_tags_1 },
    { 0x55891c25, SUC_lexicon_tags_19 },
    { 0x65a91c27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a91c2a, SUC_lexicon_tags_43 },
    { 0xe0091c2a, SUC_lexicon_tags_28 },
    { 0x84491c2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa91c2f, SUC_lexicon_tags_17 },
    { 0xfea91c2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd291c32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e91c36, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf0691c38, SUC_lexicon_tags_1 },
    { 0xe3491c38, SUC_lexicon_tags_25 },
    { 0xb1a91c39, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73691c3f, SUC_lexicon_tags_23 },
    { 0xad491c40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1891c44, SUC_lexicon_tags_45 },
    { 0x9091c45, SUC_lexicon_tags_49 },
    { 0x60491c46, SUC_lexicon_tags_45 },
    { 0x7f091c47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c891c4a, SUC_lexicon_tags_107 },
    { 0x89491c4b, SUC_lexicon_tags_85 },
    { 0x5a691c4b, SUC_lexicon_tags_29 },
    { 0xb5a91c4d, SUC_lexicon_tags_7 },
    { 0x3c891c4d, SUC_lexicon_tags_12 },
    { 0x46a91c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfae91c51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e91c54, SUC_lexicon_tags_11 },
    { 0x9ca91c55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99691c59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e91c5f, SUC_lexicon_tags_14 },
    { 0xdd491c5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a91c64, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb6491c66, SUC_lexicon_tags_18 },
    { 0x65e91c66, SUC_lexicon_tags_47 },
    { 0x68c91c68, SUC_lexicon_tags_7 },
    { 0xaa91c69, SUC_lexicon_tags_15 },
    { 0x90c91c6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae91c6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x60891c70, SUC_lexicon_tags_32 },
    { 0xf8091c70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a691c79, SUC_lexicon_tags_87 },
    { 0x82091c7a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71091c80, SUC_lexicon_tags_23 },
    { 0x40491c81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36491c83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb1091c85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85891c87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17691c93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71291c95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c091c97, SUC_lexicon_tags_17 },
    { 0x5f091c98, SUC_lexicon_tags_77 },
    { 0xe6091c99, SUC_lexicon_tags_31 },
    { 0x17691c99, SUC_lexicon_tags_3 },
    { 0x30291c99, SUC_lexicon_tags_91 },
    { 0x4c891c9c, SUC_lexicon_tags_11 },
    { 0xb6491c9d, SUC_lexicon_tags_43 },
    { 0x58491c9e, SUC_lexicon_tags_43 },
    { 0x52e91c9c, SUC_lexicon_tags_7 },
    { 0xdd291c9d, SUC_lexicon_tags_19 },
    { 0xb7c91ca1, SUC_lexicon_tags_6 },
    { 0x9b891ca1, SUC_lexicon_tags_9 },
    { 0x13a91ca3, SUC_lexicon_tags_21 },
    { 0xee491ca4, SUC_lexicon_tags_3 },
    { 0xab691ca2, SUC_lexicon_tags_38 },
    { 0xfbe91c9a, SUC_lexicon_tags_1 },
    { 0xf0891ca7, SUC_lexicon_tags_37 },
    { 0x59c91ca3, SUC_lexicon_tags_85 },
    { 0x9ea91c9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc291cae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6c91cb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac91cb4, SUC_lexicon_tags_0 },
    { 0x76491cb5, SUC_lexicon_tags_4 },
    { 0xad291cb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c91cbc, SUC_lexicon_tags_17 },
    { 0xde691cbc, SUC_lexicon_tags_23 },
    { 0xf4c91cbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd091cc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5891cc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46091cc5, SUC_lexicon_tags_30 },
    { 0xe7291cc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x99e91cc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe91cce, SUC_lexicon_tags_46 },
    { 0x7a691ccf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x55091cd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb891cd3, SUC_lexicon_tags_45 },
    { 0x9f491cd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e91cd8, SUC_lexicon_tags_14 },
    { 0xfb891cd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62891cdb, SUC_lexicon_tags_87 },
    { 0xb3891cdb, SUC_lexicon_tags_37 },
    { 0x11891cdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a91ce0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9891ce4, SUC_lexicon_tags_30 },
    { 0xd4e91ce5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99091cea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e91cf0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36291cf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9291cfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9491d02, SUC_lexicon_tags_12 },
    { 0xf2891d03, SUC_lexicon_tags_82 },
    { 0x29a91d04, SUC_lexicon_tags_4 },
    { 0xc1e91d04, SUC_lexicon_tags_63 },
    { 0x67491d04, SUC_lexicon_tags_60 },
    { 0xe7c91d07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83891d0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfcc91d0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6891d13, SUC_lexicon_tags_14 },
    { 0xafe91d14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e91d17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56491d19, SUC_lexicon_tags_13 },
    { 0x77a91d19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf891d20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f291d23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e891d2d, SUC_lexicon_tags_47 },
    { 0xae91d2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70491d31, SUC_lexicon_tags_27 },
    { 0xc6c91d32, SUC_lexicon_tags_17 },
    { 0x72891d33, SUC_lexicon_tags_68 },
    { 0x3a291d34, SUC_lexicon_tags_15 },
    { 0x42c91d33, SUC_lexicon_tags_11 },
    { 0xe4891d33, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1891d39, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb9091d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca91d3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac91d42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4091d44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a91d4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c091d4f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a91d53, SUC_lexicon_tags_48 },
    { 0xe5091d53, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c91d57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91e91d59, SUC_lexicon_tags_17 },
    { 0x69091d5a, SUC_lexicon_tags_14 },
    { 0x93a91d5b, SUC_lexicon_tags_0 },
    { 0xb3291d5c, SUC_lexicon_tags_17 },
    { 0x4f091d5b, SUC_lexicon_tags_15 },
    { 0xee91d5e, SUC_lexicon_tags_8 },
    { 0x2891d5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8691d64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5491d67, SUC_lexicon_tags_37 },
    { 0x4ae91d68, SUC_lexicon_tags_2 },
    { 0x13091d69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6be91d6b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a91d6e, SUC_lexicon_tags_28 },
    { 0xac091d6f, SUC_lexicon_tags_49 },
    { 0x7cc91d6f, SUC_lexicon_tags_17 },
    { 0x2da91d71, SUC_lexicon_tags_17 },
    { 0x56891d72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11091d77, SUC_lexicon_tags_1 },
    { 0x4c091d77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8091d7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18291d86, SUC_lexicon_tags_66 },
    { 0x91e91d87, SUC_lexicon_tags_4 },
    { 0xeda91d88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76691d8a, SUC_lexicon_tags_12 },
    { 0xc8a91d8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a91d94, SUC_lexicon_tags_37 },
    { 0x6b091d95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe91da1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c691da6, SUC_lexicon_tags_47 },
    { 0x91291da7, SUC_lexicon_tags_34 },
    { 0xcec91da6, SUC_lexicon_tags_12 },
    { 0x2f291da9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce91dae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39491db0, SUC_lexicon_tags_15 },
    { 0x20291db1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5691db9, SUC_lexicon_tags_14 },
    { 0xc4691dba, SUC_lexicon_tags_25 },
    { 0xc0891db9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc1e91dbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33891dbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83691dc3, SUC_lexicon_tags_14 },
    { 0x2d291dc4, SUC_lexicon_tags_15 },
    { 0x27e91dc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b691dc7, SUC_lexicon_tags_11 },
    { 0xdbe91dc7, SUC_lexicon_tags_37 },
    { 0xe7491dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51291dce, SUC_lexicon_tags_14 },
    { 0x26891dcf, SUC_lexicon_tags_0 },
    { 0xaa091dd0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0691dd5, SUC_lexicon_tags_60 },
    { 0x10291dd5, SUC_lexicon_tags_32 },
    { 0x891dd7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71491ddc, SUC_lexicon_tags_56 },
    { 0x98a91ddd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x70a91ddf, SUC_lexicon_tags_19 },
    { 0x68c91de0, SUC_lexicon_tags_14 },
    { 0x2ea91de0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86e91de3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2fa91de5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10491de7, SUC_lexicon_tags_34 },
    { 0x91491de8, SUC_lexicon_tags_12 },
    { 0x2a291de7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a91dec, SUC_lexicon_tags_17 },
    { 0xe4491ded, SUC_lexicon_tags_28 },
    { 0x32a91dee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ce91df0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x62491df2, SUC_lexicon_tags_2 },
    { 0xbf491df3, SUC_lexicon_tags_1 },
    { 0xf6291df4, SUC_lexicon_tags_65 },
    { 0xf1a91df5, SUC_lexicon_tags_26 },
    { 0x79c91df5, SUC_lexicon_tags_23 },
    { 0x75691df6, SUC_lexicon_tags_11 },
    { 0xb1a91df8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x14c91dfb, SUC_lexicon_tags_27 },
    { 0x96691dfb, SUC_lexicon_tags_19 },
    { 0x31291dfb, SUC_lexicon_tags_12 },
    { 0x1691dfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc691e03, SUC_lexicon_tags_12 },
    { 0x4da91e04, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x77491e07, SUC_lexicon_tags_14 },
    { 0x5fa91e07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c891e0c, SUC_lexicon_tags_45 },
    { 0xede91e0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac91e13, SUC_lexicon_tags_25 },
    { 0xffe91e14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11691e16, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1091e22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c091e25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c91e29, SUC_lexicon_tags_72 },
    { 0x3291e2a, SUC_lexicon_tags_11 },
    { 0x5fe91e2b, SUC_lexicon_tags_2 },
    { 0x58491e2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fa91e2e, SUC_lexicon_tags_25 },
    { 0x28e91e2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e91e34, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x63691e37, SUC_lexicon_tags_27 },
    { 0x87c91e38, SUC_lexicon_tags_53 },
    { 0x81691e39, SUC_lexicon_tags_17 },
    { 0xc1a91e38, SUC_lexicon_tags_8 },
    { 0x36e91e37, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc91e3e, SUC_lexicon_tags_11 },
    { 0x5ea91e3f, SUC_lexicon_tags_11 },
    { 0x59491e3f, SUC_lexicon_tags_45 },
    { 0xddc91e41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4691e45, SUC_lexicon_tags_53 },
    { 0x2091e46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66491e4a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x62491e4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xefe91e4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3891e51, SUC_lexicon_tags_49 },
    { 0xfb891e52, SUC_lexicon_tags_21 },
    { 0xcf291e52, SUC_lexicon_tags_28 },
    { 0xc6a91e52, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c91e58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a91e5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20691e5f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8b291e61, SUC_lexicon_tags_43 },
    { 0x2a891e62, SUC_lexicon_tags_0 },
    { 0x34a91e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41491e6b, SUC_lexicon_tags_8 },
    { 0xf9091e6b, SUC_lexicon_tags_11 },
    { 0xdc891e6b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa91e75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa91e78, SUC_lexicon_tags_2 },
    { 0x1fe91e79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e91e7c, SUC_lexicon_tags_26 },
    { 0x3bc91e7d, SUC_lexicon_tags_2 },
    { 0xa6091e7e, SUC_lexicon_tags_81 },
    { 0x11291e7f, SUC_lexicon_tags_45 },
    { 0x18891e7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddc91e82, SUC_lexicon_tags_7 },
    { 0x5ea91e83, SUC_lexicon_tags_37 },
    { 0xc1a91e83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5691e87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90691e8f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27091e94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2891e96, SUC_lexicon_tags_25 },
    { 0x7a491e96, SUC_lexicon_tags_79 },
    { 0x96091e98, SUC_lexicon_tags_7 },
    { 0xb9291e98, SUC_lexicon_tags_30 },
    { 0xddc91e99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a91e9e, SUC_lexicon_tags_37 },
    { 0xdc691e9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c91ea4, SUC_lexicon_tags_32 },
    { 0xd8291ea5, SUC_lexicon_tags_11 },
    { 0xea091ea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28891ea9, SUC_lexicon_tags_46 },
    { 0x31691ea9, SUC_lexicon_tags_11 },
    { 0x2be91eab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49491eb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe91eb3, SUC_lexicon_tags_12 },
    { 0xcfc91eb4, SUC_lexicon_tags_1 },
    { 0xe3291eb5, SUC_lexicon_tags_257 },
    { 0x6f291eb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf691ebb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde91ebe, SUC_lexicon_tags_26 },
    { 0x49091ebf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c491ec1, SUC_lexicon_tags_42 },
    { 0x7b891ec2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43691ec4, SUC_lexicon_tags_17 },
    { 0x5e091ec4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6691ec9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8291ecc, SUC_lexicon_tags_26 },
    { 0x7d491ecd, SUC_lexicon_tags_25 },
    { 0x3c491ecd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xadc91ed0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfa91ed2, SUC_lexicon_tags_25 },
    { 0x6ae91ed3, SUC_lexicon_tags_38 },
    { 0x78a91ed4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc91ed7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4c291ed9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e91edc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34891ee1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e691ee4, SUC_lexicon_tags_4 },
    { 0xe6691ee5, SUC_lexicon_tags_0 },
    { 0xfb091ee4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80291eed, SUC_lexicon_tags_18 },
    { 0xada91eee, SUC_lexicon_tags_7 },
    { 0xc2e91eef, SUC_lexicon_tags_46 },
    { 0x3e291eee, SUC_lexicon_tags_1 },
    { 0x6e291eed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57291ef6, SUC_lexicon_tags_25 },
    { 0xb2891ef7, SUC_lexicon_tags_17 },
    { 0xb0291ef8, SUC_lexicon_tags_2 },
    { 0x3ac91ef9, SUC_lexicon_tags_15 },
    { 0x35891ef8, SUC_lexicon_tags_25 },
    { 0xf4291efb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a91efe, SUC_lexicon_tags_17 },
    { 0xf8e91eff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84091f05, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69291f09, SUC_lexicon_tags_29 },
    { 0xc9091f0a, SUC_lexicon_tags_9 },
    { 0xba091f09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38291f0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2491f0f, SUC_lexicon_tags_38 },
    { 0x91691f10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25491f18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e91f1c, SUC_lexicon_tags_31 },
    { 0x1f491f1d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d491f20, SUC_lexicon_tags_25 },
    { 0x57a91f20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c891f23, SUC_lexicon_tags_4 },
    { 0x3891f24, SUC_lexicon_tags_2 },
    { 0xe9091f24, SUC_lexicon_tags_2 },
    { 0x45891f24, SUC_lexicon_tags_45 },
    { 0x3ec91f27, SUC_lexicon_tags_19 },
    { 0xa4291f28, SUC_lexicon_tags_46 },
    { 0xb6491f25, SUC_lexicon_tags_2 },
    { 0x53891f27, SUC_lexicon_tags_17 },
    { 0x76091f2b, SUC_lexicon_tags_9 },
    { 0x84291f2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba91f35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56691f39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaee91f3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x22c91f3e, SUC_lexicon_tags_37 },
    { 0x20091f3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c291f42, SUC_lexicon_tags_12 },
    { 0x6e91f43, SUC_lexicon_tags_60 },
    { 0x37a91f43, SUC_lexicon_tags_25 },
    { 0x48091f45, SUC_lexicon_tags_77 },
    { 0x21491f45, SUC_lexicon_tags_8 },
    { 0x3a491f47, SUC_lexicon_tags_51 },
    { 0xea291f48, SUC_lexicon_tags_6 },
    { 0xf291f45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c91f4d, SUC_lexicon_tags_1 },
    { 0x9b291f4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98291f50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba91f53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e491f58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe91f5b, SUC_lexicon_tags_63 },
    { 0x2b291f5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcba91f5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec91f61, SUC_lexicon_tags_79 },
    { 0x2d091f61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79091f64, SUC_lexicon_tags_7 },
    { 0x5d891f65, SUC_lexicon_tags_27 },
    { 0xf291f66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74291f69, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1891f71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28091f73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4691f75, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c91f79, SUC_lexicon_tags_15 },
    { 0x3ce91f7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x78491f7d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c91f81, SUC_lexicon_tags_12 },
    { 0x5491f82, SUC_lexicon_tags_25 },
    { 0x39491f83, SUC_lexicon_tags_60 },
    { 0x78a91f84, SUC_lexicon_tags_92 },
    { 0x2b491f84, SUC_lexicon_tags_3 },
    { 0x97e91f86, SUC_lexicon_tags_23 },
    { 0xf891f85, SUC_lexicon_tags_1 },
    { 0xd6291f84, SUC_lexicon_tags_31 },
    { 0xa5a91f86, SUC_lexicon_tags_2 },
    { 0xae291f8a, SUC_lexicon_tags_15 },
    { 0x14891f89, SUC_lexicon_tags_1 },
    { 0xaba91f82, SUC_lexicon_tags_43 },
    { 0xe6091f83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c91f90, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x77091f92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x90091f94, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x30291f96, SUC_lexicon_tags_3 },
    { 0x4b891f97, SUC_lexicon_tags_49 },
    { 0x13091f98, SUC_lexicon_tags_22 },
    { 0x16a91f98, SUC_lexicon_tags_11 },
    { 0xe3a91f99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99291f9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28491f9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a091fa1, SUC_lexicon_tags_5 },
    { 0x2d891fa2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e691fa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f491fb1, SUC_lexicon_tags_37 },
    { 0x28091fb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4691fbb, SUC_lexicon_tags_47 },
    { 0x6291fbc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf1a91fbe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b291fc2, SUC_lexicon_tags_27 },
    { 0x28891fc3, SUC_lexicon_tags_0 },
    { 0x5b091fc4, SUC_lexicon_tags_15 },
    { 0xba091fc4, SUC_lexicon_tags_29 },
    { 0xd3891fc6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbac91fc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ec91fca, SUC_lexicon_tags_32 },
    { 0xf5891fcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76291fd3, SUC_lexicon_tags_15 },
    { 0xea291fd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc491fd6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd891fdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c91fe1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93a91fe3, SUC_lexicon_tags_7 },
    { 0x66891fe4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38a91fe6, SUC_lexicon_tags_28 },
    { 0xd2e91fe7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d091ff1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e91ff4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e91ff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22491ff9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc5a91ffb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e91ffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d892002, SUC_lexicon_tags_7 },
    { 0xf7292002, SUC_lexicon_tags_537 },
    { 0, NULL },
    { 0xd1c92005, SUC_lexicon_tags_23 },
    { 0x59892006, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3f492008, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x789200a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1989200d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3e9200f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf092019, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdde9201d, SUC_lexicon_tags_4 },
    { 0xa149201d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8292022, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c92027, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36892029, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6949202f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e92033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8492037, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3389203c, SUC_lexicon_tags_0 },
    { 0x7d89203d, SUC_lexicon_tags_25 },
    { 0xc7a9203c, SUC_lexicon_tags_38 },
    { 0xc169203f, SUC_lexicon_tags_47 },
    { 0x10e92040, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53492043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75492045, SUC_lexicon_tags_33 },
    { 0x5be92046, SUC_lexicon_tags_49 },
    { 0x44c92047, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e09204a, SUC_lexicon_tags_60 },
    { 0xd329204b, SUC_lexicon_tags_4 },
    { 0x9449204c, SUC_lexicon_tags_26 },
    { 0x9c9204c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e92050, SUC_lexicon_tags_25 },
    { 0xf2c92050, SUC_lexicon_tags_11 },
    { 0x8ac92051, SUC_lexicon_tags_234 },
    { 0xf5c92053, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e92058, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb909205a, SUC_lexicon_tags_45 },
    { 0x4329205a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd292060, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c492063, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbac92065, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d492067, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2692069, SUC_lexicon_tags_25 },
    { 0x1b492069, SUC_lexicon_tags_60 },
    { 0xa0492069, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3892072, SUC_lexicon_tags_129 },
    { 0x5e092073, SUC_lexicon_tags_7 },
    { 0xbba92074, SUC_lexicon_tags_11 },
    { 0xd4492075, SUC_lexicon_tags_23 },
    { 0xa9092075, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c9207b, SUC_lexicon_tags_12 },
    { 0x6729207c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6292081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd292083, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94492087, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49208c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xada9208f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1a92091, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1a92093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3892098, SUC_lexicon_tags_45 },
    { 0xe5492099, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd1e9209b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef49209d, SUC_lexicon_tags_15 },
    { 0x57e9209d, SUC_lexicon_tags_47 },
    { 0x8269209f, SUC_lexicon_tags_14 },
    { 0x49e920a0, SUC_lexicon_tags_25 },
    { 0x5f8920a0, SUC_lexicon_tags_43 },
    { 0xc86920a1, SUC_lexicon_tags_39 },
    { 0xa48920a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6920a8, SUC_lexicon_tags_31 },
    { 0xcae920a9, SUC_lexicon_tags_63 },
    { 0x2ba920a8, SUC_lexicon_tags_63 },
    { 0xf60920a9, SUC_lexicon_tags_30 },
    { 0x782920ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xec0920ae, SUC_lexicon_tags_37 },
    { 0xde920af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa920b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178920b9, SUC_lexicon_tags_49 },
    { 0x75a920ba, SUC_lexicon_tags_45 },
    { 0x650920bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14920c0, SUC_lexicon_tags_10 },
    { 0x88e920c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2920c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412920cb, SUC_lexicon_tags_15 },
    { 0x4a8920cc, SUC_lexicon_tags_9 },
    { 0xdce920cb, SUC_lexicon_tags_15 },
    { 0xf66920ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x596920d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c920d3, SUC_lexicon_tags_40 },
    { 0xda8920d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e920d7, SUC_lexicon_tags_12 },
    { 0x45c920d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fc920da, SUC_lexicon_tags_1 },
    { 0x9e8920db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552920e2, SUC_lexicon_tags_0 },
    { 0x990920e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec920e7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702920ed, SUC_lexicon_tags_23 },
    { 0x5be920ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28920f1, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6920f4, SUC_lexicon_tags_38 },
    { 0x5b8920f4, SUC_lexicon_tags_18 },
    { 0xea920f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc920fa, SUC_lexicon_tags_5 },
    { 0x662920fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a920ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec92107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca9210a, SUC_lexicon_tags_2 },
    { 0xac89210b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7909210f, SUC_lexicon_tags_1367 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52892113, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x60092115, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c9211a, SUC_lexicon_tags_2 },
    { 0xc4c9211a, SUC_lexicon_tags_11 },
    { 0xa6e9211c, SUC_lexicon_tags_1 },
    { 0xfbe9211d, SUC_lexicon_tags_43 },
    { 0xaa9211d, SUC_lexicon_tags_26 },
    { 0xd1a9211c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x68a92121, SUC_lexicon_tags_25 },
    { 0x4d492121, SUC_lexicon_tags_17 },
    { 0xfec92123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e692126, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b49212c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef092130, SUC_lexicon_tags_2 },
    { 0x2ea92131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42092135, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1692139, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4969213e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15292142, SUC_lexicon_tags_25 },
    { 0x692143, SUC_lexicon_tags_37 },
    { 0xa8092144, SUC_lexicon_tags_2 },
    { 0xf1092144, SUC_lexicon_tags_62 },
    { 0x85292146, SUC_lexicon_tags_25 },
    { 0xedc92147, SUC_lexicon_tags_31 },
    { 0x6fa92146, SUC_lexicon_tags_25 },
    { 0xa092143, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7469214c, SUC_lexicon_tags_25 },
    { 0xd3a9214d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x67492150, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c92154, SUC_lexicon_tags_21 },
    { 0x492154, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12c92157, SUC_lexicon_tags_25 },
    { 0xc6e92158, SUC_lexicon_tags_12 },
    { 0xd0092158, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe069215b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee92160, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f092162, SUC_lexicon_tags_11 },
    { 0xc4a92163, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfe492165, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e92168, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a69216e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f092171, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14292173, SUC_lexicon_tags_37 },
    { 0x29c92173, SUC_lexicon_tags_25 },
    { 0x99e92174, SUC_lexicon_tags_11 },
    { 0xc2692176, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa092179, SUC_lexicon_tags_6 },
    { 0x4f092179, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9217d, SUC_lexicon_tags_20 },
    { 0xd5c9217e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9292181, SUC_lexicon_tags_12 },
    { 0x7c492182, SUC_lexicon_tags_38 },
    { 0x28692183, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x44e92185, SUC_lexicon_tags_37 },
    { 0xace92186, SUC_lexicon_tags_243 },
    { 0x3c892187, SUC_lexicon_tags_3 },
    { 0x37292188, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e9218c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb292190, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92194, SUC_lexicon_tags_27 },
    { 0x15c92194, SUC_lexicon_tags_42 },
    { 0x38e92196, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad892198, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c89219c, SUC_lexicon_tags_198 },
    { 0x9ce9219c, SUC_lexicon_tags_98 },
    { 0x7b09219e, SUC_lexicon_tags_37 },
    { 0xe6e9219f, SUC_lexicon_tags_55 },
    { 0x3b0921a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf6921a2, SUC_lexicon_tags_15 },
    { 0x21a921a3, SUC_lexicon_tags_12 },
    { 0x28e921a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf8921a6, SUC_lexicon_tags_37 },
    { 0x6f4921a7, SUC_lexicon_tags_11 },
    { 0x3ac921a7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a921ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfea921af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8fe921b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4921b5, SUC_lexicon_tags_37 },
    { 0x8ca921b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb0e921b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fc921ba, SUC_lexicon_tags_25 },
    { 0x6a2921bb, SUC_lexicon_tags_6 },
    { 0x5be921bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50921c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x682921c2, SUC_lexicon_tags_29 },
    { 0xdb2921c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8921d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x528921d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68c921d9, SUC_lexicon_tags_25 },
    { 0xe26921da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a921df, SUC_lexicon_tags_1 },
    { 0x62c921e0, SUC_lexicon_tags_12 },
    { 0xffc921e0, SUC_lexicon_tags_12 },
    { 0x284921df, SUC_lexicon_tags_12 },
    { 0x484921e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad2921e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8921e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c921ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfde921ee, SUC_lexicon_tags_37 },
    { 0xe1c921ef, SUC_lexicon_tags_14 },
    { 0x796921ee, SUC_lexicon_tags_25 },
    { 0x682921ee, SUC_lexicon_tags_88 },
    { 0x7a6921ef, SUC_lexicon_tags_12 },
    { 0xe6e921f3, SUC_lexicon_tags_37 },
    { 0x614921f4, SUC_lexicon_tags_11 },
    { 0xfae921f4, SUC_lexicon_tags_27 },
    { 0xf56921f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2921fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc921fe, SUC_lexicon_tags_37 },
    { 0xfe0921fe, SUC_lexicon_tags_156 },
    { 0x620921ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9949220c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ec9220e, SUC_lexicon_tags_14 },
    { 0xa9220f, SUC_lexicon_tags_37 },
    { 0x4c29220e, SUC_lexicon_tags_37 },
    { 0x54692211, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb492214, SUC_lexicon_tags_26 },
    { 0x28c92215, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e9221a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5649221c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c92223, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2692227, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xba892229, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x15a9222b, SUC_lexicon_tags_3 },
    { 0xf629222b, SUC_lexicon_tags_37 },
    { 0xe3a9222d, SUC_lexicon_tags_15 },
    { 0xee89222e, SUC_lexicon_tags_13 },
    { 0x2e69222b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c092231, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c9223d, SUC_lexicon_tags_37 },
    { 0xa9e9223e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1292244, SUC_lexicon_tags_16 },
    { 0xb6892245, SUC_lexicon_tags_17 },
    { 0xeda92246, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9224a, SUC_lexicon_tags_37 },
    { 0xd569224b, SUC_lexicon_tags_16 },
    { 0x4a09224b, SUC_lexicon_tags_17 },
    { 0xcaa9224d, SUC_lexicon_tags_35 },
    { 0x35e9224d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca92253, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8492255, SUC_lexicon_tags_4 },
    { 0x5b892256, SUC_lexicon_tags_12 },
    { 0xa8892256, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac09225e, SUC_lexicon_tags_40 },
    { 0xf329225e, SUC_lexicon_tags_1 },
    { 0x75c9225e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85c92262, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18892266, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c892269, SUC_lexicon_tags_7 },
    { 0xa409226a, SUC_lexicon_tags_77 },
    { 0x8f69226b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c29226f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9492273, SUC_lexicon_tags_37 },
    { 0x52a92273, SUC_lexicon_tags_12 },
    { 0x79a92274, SUC_lexicon_tags_38 },
    { 0x78e92276, SUC_lexicon_tags_14 },
    { 0xbb692276, SUC_lexicon_tags_49 },
    { 0x21692273, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe669227d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c92280, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc0e92282, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf092286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35e92288, SUC_lexicon_tags_18 },
    { 0x9e692289, SUC_lexicon_tags_11 },
    { 0x12e92289, SUC_lexicon_tags_11 },
    { 0xcb09228b, SUC_lexicon_tags_37 },
    { 0x7809228b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3892290, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a92295, SUC_lexicon_tags_27 },
    { 0x79a92296, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa189229d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972922a3, SUC_lexicon_tags_38 },
    { 0xfd6922a4, SUC_lexicon_tags_43 },
    { 0x48a922a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c922a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e922ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x848922af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac922b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1d0922b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c8922b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e922b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x940922bc, SUC_lexicon_tags_17 },
    { 0xd76922bd, SUC_lexicon_tags_17 },
    { 0x3ee922bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62922c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e922c4, SUC_lexicon_tags_37 },
    { 0xc9c922c5, SUC_lexicon_tags_17 },
    { 0x258922c6, SUC_lexicon_tags_87 },
    { 0xe24922c6, SUC_lexicon_tags_48 },
    { 0x782922c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52922cf, SUC_lexicon_tags_25 },
    { 0x18922d0, SUC_lexicon_tags_33 },
    { 0x618922cf, SUC_lexicon_tags_33 },
    { 0x166922d2, SUC_lexicon_tags_16 },
    { 0xc06922cf, SUC_lexicon_tags_15 },
    { 0x5e6922d4, SUC_lexicon_tags_70 },
    { 0x558922d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda922d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a922de, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e922e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x31e922e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a922e7, SUC_lexicon_tags_29 },
    { 0x376922e7, SUC_lexicon_tags_70 },
    { 0x93e922e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae6922eb, SUC_lexicon_tags_25 },
    { 0xae922ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6922ef, SUC_lexicon_tags_38 },
    { 0x99a922f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x822922f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x792922f4, SUC_lexicon_tags_7 },
    { 0x232922f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324922f9, SUC_lexicon_tags_11 },
    { 0xc10922fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8922fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae92300, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a692303, SUC_lexicon_tags_31 },
    { 0xd7092304, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6092308, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c9230b, SUC_lexicon_tags_4 },
    { 0x8e09230c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8269230e, SUC_lexicon_tags_15 },
    { 0x1e29230f, SUC_lexicon_tags_11 },
    { 0x9da9230e, SUC_lexicon_tags_19 },
    { 0x87092311, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f892313, SUC_lexicon_tags_19 },
    { 0xeb092313, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac92317, SUC_lexicon_tags_27 },
    { 0xcaa92318, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc9231f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28c92321, SUC_lexicon_tags_46 },
    { 0x44692322, SUC_lexicon_tags_45 },
    { 0x89692323, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c92327, SUC_lexicon_tags_15 },
    { 0x89892328, SUC_lexicon_tags_7 },
    { 0xf0c92329, SUC_lexicon_tags_26 },
    { 0xa329232a, SUC_lexicon_tags_17 },
    { 0x60c9232b, SUC_lexicon_tags_31 },
    { 0x1b69232c, SUC_lexicon_tags_40 },
    { 0xa1a9232d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff69232f, SUC_lexicon_tags_31 },
    { 0x7d092330, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff092332, SUC_lexicon_tags_1 },
    { 0xd4492332, SUC_lexicon_tags_105 },
    { 0x93a92333, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88692338, SUC_lexicon_tags_6 },
    { 0xbd892338, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea9233c, SUC_lexicon_tags_37 },
    { 0x7f29233d, SUC_lexicon_tags_0 },
    { 0xbf89233e, SUC_lexicon_tags_37 },
    { 0x2e9233d, SUC_lexicon_tags_25 },
    { 0x81e92340, SUC_lexicon_tags_9 },
    { 0x64892341, SUC_lexicon_tags_16 },
    { 0x23e92342, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc92345, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x64e92347, SUC_lexicon_tags_19 },
    { 0x56a92347, SUC_lexicon_tags_15 },
    { 0x2c492347, SUC_lexicon_tags_25 },
    { 0x2c892348, SUC_lexicon_tags_15 },
    { 0x9969234b, SUC_lexicon_tags_47 },
    { 0x51c92349, SUC_lexicon_tags_37 },
    { 0xd9492349, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce92358, SUC_lexicon_tags_32 },
    { 0xf8692359, SUC_lexicon_tags_25 },
    { 0xaca9235a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cc9235c, SUC_lexicon_tags_12 },
    { 0xd3a9235c, SUC_lexicon_tags_32 },
    { 0x8a29235e, SUC_lexicon_tags_14 },
    { 0xada9235f, SUC_lexicon_tags_65 },
    { 0x3b69235e, SUC_lexicon_tags_14 },
    { 0x4949235e, SUC_lexicon_tags_0 },
    { 0x16892360, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e92364, SUC_lexicon_tags_84 },
    { 0x63092364, SUC_lexicon_tags_56 },
    { 0xe9692366, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba92369, SUC_lexicon_tags_4 },
    { 0xcde9236a, SUC_lexicon_tags_11 },
    { 0xf6a9236a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x8829236e, SUC_lexicon_tags_9 },
    { 0xe8e9236f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37c92371, SUC_lexicon_tags_1 },
    { 0xcfc92372, SUC_lexicon_tags_12 },
    { 0x8ec92373, SUC_lexicon_tags_11 },
    { 0xd1092374, SUC_lexicon_tags_19 },
    { 0x23492371, SUC_lexicon_tags_28 },
    { 0x55092371, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0292380, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xde892382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f292384, SUC_lexicon_tags_194 },
    { 0x57e92385, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e9238a, SUC_lexicon_tags_11 },
    { 0x45c9238a, SUC_lexicon_tags_87 },
    { 0xf029238c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17692391, SUC_lexicon_tags_17 },
    { 0xfba92392, SUC_lexicon_tags_1 },
    { 0x35492391, SUC_lexicon_tags_33 },
    { 0xf9092394, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8da92396, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xef092398, SUC_lexicon_tags_60 },
    { 0x76092399, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45c9239c, SUC_lexicon_tags_10 },
    { 0xe549239c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88923a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4923a9, SUC_lexicon_tags_65 },
    { 0xb5e923a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26923af, SUC_lexicon_tags_87 },
    { 0x530923af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0923b8, SUC_lexicon_tags_60 },
    { 0x49c923b9, SUC_lexicon_tags_25 },
    { 0x38a923b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca923c1, SUC_lexicon_tags_7 },
    { 0x1b6923c2, SUC_lexicon_tags_14 },
    { 0xdc0923c3, SUC_lexicon_tags_17 },
    { 0xa30923c4, SUC_lexicon_tags_11 },
    { 0x43e923c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x182923c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32923ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x904923d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa923d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x814923d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2923d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0a923db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe74923dd, SUC_lexicon_tags_7 },
    { 0x12e923de, SUC_lexicon_tags_47 },
    { 0x696923df, SUC_lexicon_tags_47 },
    { 0x7f8923de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5a923e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6923e9, SUC_lexicon_tags_10 },
    { 0x1d6923ea, SUC_lexicon_tags_1 },
    { 0x724923eb, SUC_lexicon_tags_12 },
    { 0xb70923ea, SUC_lexicon_tags_6 },
    { 0x368923ed, SUC_lexicon_tags_15 },
    { 0x172923ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4923f7, SUC_lexicon_tags_12 },
    { 0xcd8923f8, SUC_lexicon_tags_27 },
    { 0x86c923f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e923fd, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xb06923ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e292401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17e92403, SUC_lexicon_tags_22 },
    { 0xccc92403, SUC_lexicon_tags_11 },
    { 0x47e92405, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce892408, SUC_lexicon_tags_14 },
    { 0x3cc92409, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a9240c, SUC_lexicon_tags_15 },
    { 0x1209240c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fa9240f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca92413, SUC_lexicon_tags_11 },
    { 0xedc92414, SUC_lexicon_tags_25 },
    { 0x91692415, SUC_lexicon_tags_173 },
    { 0xc9e92416, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82092420, SUC_lexicon_tags_25 },
    { 0xa3892420, SUC_lexicon_tags_84 },
    { 0x26a92421, SUC_lexicon_tags_43 },
    { 0xebc92421, SUC_lexicon_tags_57 },
    { 0xb7a92424, SUC_lexicon_tags_26 },
    { 0x20092425, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c92427, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1aa92429, SUC_lexicon_tags_17 },
    { 0x6469242a, SUC_lexicon_tags_25 },
    { 0xc5e9242b, SUC_lexicon_tags_25 },
    { 0xf729242c, SUC_lexicon_tags_1 },
    { 0x6d89242d, SUC_lexicon_tags_45 },
    { 0xdf09242e, SUC_lexicon_tags_27 },
    { 0xd629242b, SUC_lexicon_tags_48 },
    { 0x589242b, SUC_lexicon_tags_1 },
    { 0x87e92431, SUC_lexicon_tags_10 },
    { 0xc9492432, SUC_lexicon_tags_38 },
    { 0x51492433, SUC_lexicon_tags_25 },
    { 0x66692434, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xafc92437, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7449243b, SUC_lexicon_tags_29 },
    { 0x449243c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x34a9243e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c92441, SUC_lexicon_tags_47 },
    { 0x9f292442, SUC_lexicon_tags_8 },
    { 0xaba92443, SUC_lexicon_tags_0 },
    { 0x74092444, SUC_lexicon_tags_39 },
    { 0xe6e92445, SUC_lexicon_tags_26 },
    { 0xcb492443, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46692448, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8be9244a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e9244f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e92452, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3292456, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa89245f, SUC_lexicon_tags_28 },
    { 0xbb49245f, SUC_lexicon_tags_120 },
    { 0x6c49245f, SUC_lexicon_tags_38 },
    { 0x77092460, SUC_lexicon_tags_11 },
    { 0x3ac92463, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x75892465, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce9246c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc649246e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x65892470, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a92473, SUC_lexicon_tags_3 },
    { 0xa6292474, SUC_lexicon_tags_12 },
    { 0x35692475, SUC_lexicon_tags_9 },
    { 0xd2a92474, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc9247b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3cc9247d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc492481, SUC_lexicon_tags_17 },
    { 0xfb292481, SUC_lexicon_tags_1 },
    { 0x76e92483, SUC_lexicon_tags_87 },
    { 0x72a92484, SUC_lexicon_tags_57 },
    { 0x60e92482, SUC_lexicon_tags_38 },
    { 0x96e92486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93692489, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c9248c, SUC_lexicon_tags_1 },
    { 0x4969248d, SUC_lexicon_tags_37 },
    { 0x5829248d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1092491, SUC_lexicon_tags_66 },
    { 0x6492492, SUC_lexicon_tags_44 },
    { 0xbca92492, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b692497, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe309249b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbde9249d, SUC_lexicon_tags_47 },
    { 0xcc49249e, SUC_lexicon_tags_35 },
    { 0x80c9249e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe924a3, SUC_lexicon_tags_31 },
    { 0x24c924a4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc924a8, SUC_lexicon_tags_60 },
    { 0xdba924a9, SUC_lexicon_tags_201 },
    { 0x1ae924a9, SUC_lexicon_tags_47 },
    { 0x7ae924ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c6924ad, SUC_lexicon_tags_8 },
    { 0x2d4924ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2924b2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe924b5, SUC_lexicon_tags_42 },
    { 0x2a4924b5, SUC_lexicon_tags_2 },
    { 0x3f8924b7, SUC_lexicon_tags_7 },
    { 0x60924b8, SUC_lexicon_tags_12 },
    { 0xfb4924b9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930924c1, SUC_lexicon_tags_37 },
    { 0xe38924c2, SUC_lexicon_tags_30 },
    { 0x290924c2, SUC_lexicon_tags_33 },
    { 0xbd6924c4, SUC_lexicon_tags_37 },
    { 0xd50924c5, SUC_lexicon_tags_29 },
    { 0x8d6924c4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4924cc, SUC_lexicon_tags_65 },
    { 0x72a924cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ae924cf, SUC_lexicon_tags_33 },
    { 0xd50924d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10924d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0924d8, SUC_lexicon_tags_27 },
    { 0x7a0924d8, SUC_lexicon_tags_47 },
    { 0x6fc924da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc924df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae924e3, SUC_lexicon_tags_5 },
    { 0x500924e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40924ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf84924ec, SUC_lexicon_tags_2 },
    { 0x3ee924ed, SUC_lexicon_tags_17 },
    { 0x160924ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa924f5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08924f9, SUC_lexicon_tags_37 },
    { 0xdd0924fa, SUC_lexicon_tags_30 },
    { 0xaaa924f9, SUC_lexicon_tags_25 },
    { 0x8ba924fb, SUC_lexicon_tags_17 },
    { 0xf98924fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91692500, SUC_lexicon_tags_37 },
    { 0xb4292501, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea92504, SUC_lexicon_tags_12 },
    { 0xa5292505, SUC_lexicon_tags_47 },
    { 0x7a292506, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f49250b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe309250d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b292512, SUC_lexicon_tags_33 },
    { 0x4e292513, SUC_lexicon_tags_68 },
    { 0xa2492514, SUC_lexicon_tags_37 },
    { 0xdac92514, SUC_lexicon_tags_11 },
    { 0xa8c92516, SUC_lexicon_tags_77 },
    { 0x26c92512, SUC_lexicon_tags_14 },
    { 0x2bc92515, SUC_lexicon_tags_27 },
    { 0x72c92519, SUC_lexicon_tags_17 },
    { 0xed29251a, SUC_lexicon_tags_17 },
    { 0xa169251a, SUC_lexicon_tags_37 },
    { 0xe2292519, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2109251e, SUC_lexicon_tags_37 },
    { 0x2789251f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x35692521, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa492528, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f09252b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e92530, SUC_lexicon_tags_1 },
    { 0x73e92531, SUC_lexicon_tags_25 },
    { 0xa0092532, SUC_lexicon_tags_63 },
    { 0x30c92531, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x33092535, SUC_lexicon_tags_17 },
    { 0xf2492536, SUC_lexicon_tags_37 },
    { 0x3a092537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6da92539, SUC_lexicon_tags_15 },
    { 0x3569253a, SUC_lexicon_tags_37 },
    { 0x97a9253b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c9253e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e892543, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c92548, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea9254b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2de9254d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe92552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11092557, SUC_lexicon_tags_13 },
    { 0xd4492558, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff09255e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa492560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ac92562, SUC_lexicon_tags_25 },
    { 0x90e92563, SUC_lexicon_tags_25 },
    { 0x9092564, SUC_lexicon_tags_6 },
    { 0x80692565, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a92569, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86092573, SUC_lexicon_tags_15 },
    { 0xda892574, SUC_lexicon_tags_7 },
    { 0x35492575, SUC_lexicon_tags_7 },
    { 0xa9892576, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x6c92578, SUC_lexicon_tags_33 },
    { 0xcca92579, SUC_lexicon_tags_33 },
    { 0x1cc9257a, SUC_lexicon_tags_2 },
    { 0xda69257a, SUC_lexicon_tags_11 },
    { 0xdf09257c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9809257f, SUC_lexicon_tags_17 },
    { 0xf5c92580, SUC_lexicon_tags_29 },
    { 0x85692581, SUC_lexicon_tags_33 },
    { 0xc492582, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16092584, SUC_lexicon_tags_19 },
    { 0x8b892585, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1292588, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5209258b, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e92591, SUC_lexicon_tags_17 },
    { 0xa0092592, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c92595, SUC_lexicon_tags_25 },
    { 0x31e92595, SUC_lexicon_tags_17 },
    { 0x83c92597, SUC_lexicon_tags_0 },
    { 0x9b692598, SUC_lexicon_tags_7 },
    { 0x3ca92595, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de9259d, SUC_lexicon_tags_12 },
    { 0x10e9259d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea925a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904925a7, SUC_lexicon_tags_31 },
    { 0xaa925a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x690925aa, SUC_lexicon_tags_12 },
    { 0x562925aa, SUC_lexicon_tags_1 },
    { 0xc72925aa, SUC_lexicon_tags_33 },
    { 0xd58925ad, SUC_lexicon_tags_25 },
    { 0x62e925ab, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x54a925b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e925b4, SUC_lexicon_tags_30 },
    { 0xa7a925b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6925c1, SUC_lexicon_tags_19 },
    { 0x254925c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x864925c5, SUC_lexicon_tags_25 },
    { 0xcca925c6, SUC_lexicon_tags_46 },
    { 0x764925c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ec925c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be925d2, SUC_lexicon_tags_70 },
    { 0x670925d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be925d7, SUC_lexicon_tags_37 },
    { 0xa46925d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7fe925da, SUC_lexicon_tags_13 },
    { 0x706925db, SUC_lexicon_tags_2 },
    { 0xa0925dc, SUC_lexicon_tags_30 },
    { 0xd08925dc, SUC_lexicon_tags_287 },
    { 0xae6925de, SUC_lexicon_tags_49 },
    { 0x39a925df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62925e2, SUC_lexicon_tags_11 },
    { 0xb02925e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde925e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a925ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c925f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x980925f8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe68925fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a925fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ce925ff, SUC_lexicon_tags_6 },
    { 0x8da92600, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e92604, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51692606, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd292608, SUC_lexicon_tags_12 },
    { 0x3e692609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a9260f, SUC_lexicon_tags_37 },
    { 0x3089260f, SUC_lexicon_tags_2 },
    { 0xd8e92611, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3292616, SUC_lexicon_tags_49 },
    { 0xed892617, SUC_lexicon_tags_93 },
    { 0xa1a92618, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1cc9261a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6409261c, SUC_lexicon_tags_37 },
    { 0x3a49261c, SUC_lexicon_tags_7 },
    { 0x44e9261d, SUC_lexicon_tags_31 },
    { 0xb09261c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5492621, SUC_lexicon_tags_25 },
    { 0x96092621, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd0692624, SUC_lexicon_tags_65 },
    { 0x21a92625, SUC_lexicon_tags_16 },
    { 0x16892624, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf909262d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc009262f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90092631, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c92634, SUC_lexicon_tags_1 },
    { 0x95c92635, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd649263c, SUC_lexicon_tags_1 },
    { 0xf929263c, SUC_lexicon_tags_27 },
    { 0x1a9263e, SUC_lexicon_tags_47 },
    { 0x6d49263d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8292641, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba92645, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34692647, SUC_lexicon_tags_25 },
    { 0x3da92647, SUC_lexicon_tags_37 },
    { 0x8d092649, SUC_lexicon_tags_49 },
    { 0xaf492649, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf292652, SUC_lexicon_tags_55 },
    { 0x7aa92653, SUC_lexicon_tags_2 },
    { 0xa1092654, SUC_lexicon_tags_25 },
    { 0x93892652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c29265b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb1e9265d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6889265f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13892665, SUC_lexicon_tags_62 },
    { 0x8ee92665, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52892668, SUC_lexicon_tags_19 },
    { 0x46e92669, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbfe9266b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf49266f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73c92671, SUC_lexicon_tags_4 },
    { 0x53e92671, SUC_lexicon_tags_37 },
    { 0xb0e92671, SUC_lexicon_tags_37 },
    { 0xe9092674, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c892679, SUC_lexicon_tags_11 },
    { 0x5ce92679, SUC_lexicon_tags_19 },
    { 0x31e9267b, SUC_lexicon_tags_31 },
    { 0x6769267c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36892680, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x89892683, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25292685, SUC_lexicon_tags_39 },
    { 0x59c92686, SUC_lexicon_tags_25 },
    { 0xfd692686, SUC_lexicon_tags_37 },
    { 0xcd692688, SUC_lexicon_tags_35 },
    { 0xd0a92689, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c49268c, SUC_lexicon_tags_1 },
    { 0x2989268d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e92690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ac92692, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68292694, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x58e92697, SUC_lexicon_tags_26 },
    { 0x57492697, SUC_lexicon_tags_11 },
    { 0x10492699, SUC_lexicon_tags_9 },
    { 0xa589269a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8769269c, SUC_lexicon_tags_87 },
    { 0x8e89269c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc926a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226926a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb2926a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2926ac, SUC_lexicon_tags_29 },
    { 0xa82926ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x476926af, SUC_lexicon_tags_23 },
    { 0x72926b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e6926b2, SUC_lexicon_tags_11 },
    { 0xf22926b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946926bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4926c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6926c6, SUC_lexicon_tags_19 },
    { 0x5e4926c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0926d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e926d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50926dd, SUC_lexicon_tags_11 },
    { 0xc4a926de, SUC_lexicon_tags_30 },
    { 0xfe8926df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be926e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec926e7, SUC_lexicon_tags_37 },
    { 0xc28926e8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x596926eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x662926ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x358926f0, SUC_lexicon_tags_65 },
    { 0xf3e926f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64926f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6926f8, SUC_lexicon_tags_17 },
    { 0xc6c926f9, SUC_lexicon_tags_46 },
    { 0xffc926fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968926ff, SUC_lexicon_tags_49 },
    { 0x5c292700, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2292702, SUC_lexicon_tags_43 },
    { 0x492703, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7b092705, SUC_lexicon_tags_19 },
    { 0x50692706, SUC_lexicon_tags_7 },
    { 0x37e92706, SUC_lexicon_tags_65 },
    { 0xa2c92705, SUC_lexicon_tags_47 },
    { 0xdf692709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc9270f, SUC_lexicon_tags_6 },
    { 0x9289270f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8692715, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3092718, SUC_lexicon_tags_24 },
    { 0x21c92719, SUC_lexicon_tags_37 },
    { 0x3bc9271a, SUC_lexicon_tags_30 },
    { 0xc429271b, SUC_lexicon_tags_11 },
    { 0x22c9271b, SUC_lexicon_tags_57 },
    { 0xb8e9271a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57492721, SUC_lexicon_tags_2 },
    { 0xbf492722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77692728, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a9272e, SUC_lexicon_tags_64 },
    { 0xf189272e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e92735, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d692738, SUC_lexicon_tags_61 },
    { 0xe2292739, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8cc9273b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe9273e, SUC_lexicon_tags_87 },
    { 0xf3a9273f, SUC_lexicon_tags_10 },
    { 0x6d492740, SUC_lexicon_tags_68 },
    { 0x60a92741, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36892747, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x36092749, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3569274c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c9274f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a92752, SUC_lexicon_tags_82 },
    { 0x4c092753, SUC_lexicon_tags_23 },
    { 0x7ee92754, SUC_lexicon_tags_12 },
    { 0x40c92755, SUC_lexicon_tags_14 },
    { 0x5de92753, SUC_lexicon_tags_46 },
    { 0x7c092753, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b49275e, SUC_lexicon_tags_46 },
    { 0x9749275f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe92763, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x96a92766, SUC_lexicon_tags_26 },
    { 0x8e892767, SUC_lexicon_tags_23 },
    { 0x3c92768, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5849276c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe292771, SUC_lexicon_tags_37 },
    { 0xf1092772, SUC_lexicon_tags_39 },
    { 0x32292773, SUC_lexicon_tags_37 },
    { 0x99a92774, SUC_lexicon_tags_21 },
    { 0xf3692775, SUC_lexicon_tags_7 },
    { 0xb4e92776, SUC_lexicon_tags_45 },
    { 0x5de92777, SUC_lexicon_tags_1 },
    { 0x4c092778, SUC_lexicon_tags_15 },
    { 0x9a492777, SUC_lexicon_tags_2 },
    { 0x92778, SUC_lexicon_tags_25 },
    { 0x9949277b, SUC_lexicon_tags_15 },
    { 0xe0692773, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c92780, SUC_lexicon_tags_8 },
    { 0xff692781, SUC_lexicon_tags_21 },
    { 0xde92782, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d092785, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5292791, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7c892793, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e92799, SUC_lexicon_tags_17 },
    { 0x5669279a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58927a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac927a5, SUC_lexicon_tags_37 },
    { 0x654927a6, SUC_lexicon_tags_37 },
    { 0x8c4927a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808927b7, SUC_lexicon_tags_25 },
    { 0x90e927b7, SUC_lexicon_tags_69 },
    { 0x29a927b8, SUC_lexicon_tags_23 },
    { 0xb40927b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c927c1, SUC_lexicon_tags_1 },
    { 0xe2c927c2, SUC_lexicon_tags_37 },
    { 0xfa4927c1, SUC_lexicon_tags_12 },
    { 0x4d4927c1, SUC_lexicon_tags_37 },
    { 0x970927c5, SUC_lexicon_tags_11 },
    { 0xe7c927c5, SUC_lexicon_tags_11 },
    { 0x8ec927c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x968927ca, SUC_lexicon_tags_1 },
    { 0xdda927cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a927ce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x180927d0, SUC_lexicon_tags_25 },
    { 0x336927d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc927d6, SUC_lexicon_tags_127 },
    { 0xfa2927d7, SUC_lexicon_tags_17 },
    { 0xeb8927d8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526927dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46927df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc927e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604927e7, SUC_lexicon_tags_11 },
    { 0x9e8927e8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xef2927ea, SUC_lexicon_tags_15 },
    { 0x1c8927eb, SUC_lexicon_tags_25 },
    { 0x6de927eb, SUC_lexicon_tags_19 },
    { 0x2ba927eb, SUC_lexicon_tags_25 },
    { 0x768927ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58927f5, SUC_lexicon_tags_9 },
    { 0xf20927f5, SUC_lexicon_tags_7 },
    { 0xb94927f7, SUC_lexicon_tags_21 },
    { 0xe68927f5, SUC_lexicon_tags_9 },
    { 0xd40927f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e927fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c92802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f692808, SUC_lexicon_tags_63 },
    { 0x2c92809, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba9280d, SUC_lexicon_tags_37 },
    { 0xc169280e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x90092810, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0x9ee92812, SUC_lexicon_tags_11 },
    { 0xd692812, SUC_lexicon_tags_2 },
    { 0xdd292814, SUC_lexicon_tags_92 },
    { 0x8a492814, SUC_lexicon_tags_15 },
    { 0x1fa92815, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e9281d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9e9281f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe1092821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c92825, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x72c92827, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6129282e, SUC_lexicon_tags_19 },
    { 0x1ba9282f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f092831, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae692835, SUC_lexicon_tags_7 },
    { 0x95292836, SUC_lexicon_tags_12 },
    { 0x12292837, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd149283a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9329283e, SUC_lexicon_tags_8 },
    { 0xb629283e, SUC_lexicon_tags_37 },
    { 0x8c692840, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e92846, SUC_lexicon_tags_17 },
    { 0x14892846, SUC_lexicon_tags_43 },
    { 0xe2092847, SUC_lexicon_tags_38 },
    { 0xe3c92849, SUC_lexicon_tags_2 },
    { 0xf449284a, SUC_lexicon_tags_11 },
    { 0xa309284a, SUC_lexicon_tags_23 },
    { 0x4ee9284a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5689284e, SUC_lexicon_tags_25 },
    { 0x34e9284f, SUC_lexicon_tags_11 },
    { 0x2489284f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e92854, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0492856, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a9285f, SUC_lexicon_tags_8 },
    { 0x5fa92860, SUC_lexicon_tags_1 },
    { 0xb9292861, SUC_lexicon_tags_2 },
    { 0x62692861, SUC_lexicon_tags_32 },
    { 0x2b492863, SUC_lexicon_tags_17 },
    { 0x2a692863, SUC_lexicon_tags_17 },
    { 0x1c692865, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc92867, SUC_lexicon_tags_12 },
    { 0xa0092868, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca9286b, SUC_lexicon_tags_11 },
    { 0x2049286c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4209286f, SUC_lexicon_tags_27 },
    { 0x8a892870, SUC_lexicon_tags_17 },
    { 0xc9892871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69692873, SUC_lexicon_tags_23 },
    { 0xa7692874, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe092877, SUC_lexicon_tags_23 },
    { 0xc0092878, SUC_lexicon_tags_21 },
    { 0x59a92878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4709287c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7b29287e, SUC_lexicon_tags_65 },
    { 0x409287f, SUC_lexicon_tags_25 },
    { 0xe4892880, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5692882, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15092886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8da92888, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa629288e, SUC_lexicon_tags_13 },
    { 0x3549288e, SUC_lexicon_tags_11 },
    { 0xc9292890, SUC_lexicon_tags_12 },
    { 0x94492891, SUC_lexicon_tags_19 },
    { 0xe509288f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98092894, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8692896, SUC_lexicon_tags_49 },
    { 0x24c92897, SUC_lexicon_tags_60 },
    { 0x79892897, SUC_lexicon_tags_11 },
    { 0x33892899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789289d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e928a0, SUC_lexicon_tags_37 },
    { 0xb60928a1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x966928a3, SUC_lexicon_tags_16 },
    { 0x1fc928a3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca928a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x902928ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672928b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce928b4, SUC_lexicon_tags_15 },
    { 0x180928b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x706928b8, SUC_lexicon_tags_26 },
    { 0x58928b9, SUC_lexicon_tags_27 },
    { 0xd92928ba, SUC_lexicon_tags_19 },
    { 0xef4928b8, SUC_lexicon_tags_6 },
    { 0xdd6928bb, SUC_lexicon_tags_15 },
    { 0x622928bb, SUC_lexicon_tags_106 },
    { 0x264928be, SUC_lexicon_tags_95 },
    { 0xdbc928be, SUC_lexicon_tags_19 },
    { 0xbe2928c0, SUC_lexicon_tags_37 },
    { 0x64e928be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d6928c3, SUC_lexicon_tags_17 },
    { 0xb92928c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c2928c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc928ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x858928cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a6928ce, SUC_lexicon_tags_26 },
    { 0x8ea928cf, SUC_lexicon_tags_33 },
    { 0x6da928d0, SUC_lexicon_tags_87 },
    { 0xb90928d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6928d9, SUC_lexicon_tags_34 },
    { 0x26e928d9, SUC_lexicon_tags_4 },
    { 0xe20928db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c928dd, SUC_lexicon_tags_47 },
    { 0xdc2928dd, SUC_lexicon_tags_77 },
    { 0x418928df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94928e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c928e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcea928e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a928ec, SUC_lexicon_tags_16 },
    { 0x900928ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98928f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcac928f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc928f8, SUC_lexicon_tags_37 },
    { 0x5cc928f9, SUC_lexicon_tags_14 },
    { 0xee928f9, SUC_lexicon_tags_8 },
    { 0x102928fa, SUC_lexicon_tags_15 },
    { 0xede928fb, SUC_lexicon_tags_14 },
    { 0xab2928fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x95c928ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3692909, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc9290d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc249290f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92913, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc92915, SUC_lexicon_tags_1 },
    { 0x82892915, SUC_lexicon_tags_11 },
    { 0x2c92917, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5b292919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3fe9291b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x13c9291d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7f09291f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a92922, SUC_lexicon_tags_3 },
    { 0xfec92922, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b092928, SUC_lexicon_tags_0 },
    { 0x45e92928, SUC_lexicon_tags_31 },
    { 0x8709292a, SUC_lexicon_tags_1073 },
    { 0xee49292b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4189292d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe92930, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd709293b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8909293f, SUC_lexicon_tags_23 },
    { 0x6b092940, SUC_lexicon_tags_12 },
    { 0xcfa92940, SUC_lexicon_tags_27 },
    { 0xb3a92942, SUC_lexicon_tags_37 },
    { 0x3e892941, SUC_lexicon_tags_60 },
    { 0x1e92941, SUC_lexicon_tags_55 },
    { 0x4e892945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8692947, SUC_lexicon_tags_276 },
    { 0xe492948, SUC_lexicon_tags_7 },
    { 0x4c892949, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a9294c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c92950, SUC_lexicon_tags_60 },
    { 0xf8092951, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcd692953, SUC_lexicon_tags_1 },
    { 0xdd692953, SUC_lexicon_tags_21 },
    { 0xc92955, SUC_lexicon_tags_11 },
    { 0x48692956, SUC_lexicon_tags_15 },
    { 0x6be92957, SUC_lexicon_tags_7 },
    { 0x8cc92956, SUC_lexicon_tags_26 },
    { 0x11892958, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c9295d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcdc9295f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2692963, SUC_lexicon_tags_12 },
    { 0x93692963, SUC_lexicon_tags_28 },
    { 0x3b492965, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2169296a, SUC_lexicon_tags_17 },
    { 0x5e89296a, SUC_lexicon_tags_13 },
    { 0xe389296c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd009296e, SUC_lexicon_tags_11 },
    { 0x2f89296e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97692971, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x20092973, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e9297b, SUC_lexicon_tags_15 },
    { 0xbc9297c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a92984, SUC_lexicon_tags_17 },
    { 0x34892985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4e92987, SUC_lexicon_tags_7 },
    { 0x1de92987, SUC_lexicon_tags_7 },
    { 0x37e92989, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2429298b, SUC_lexicon_tags_171 },
    { 0x6d49298c, SUC_lexicon_tags_37 },
    { 0x1c29298d, SUC_lexicon_tags_23 },
    { 0x15a9298e, SUC_lexicon_tags_37 },
    { 0xc1e9298f, SUC_lexicon_tags_15 },
    { 0xac89298e, SUC_lexicon_tags_47 },
    { 0x35892991, SUC_lexicon_tags_25 },
    { 0x90692992, SUC_lexicon_tags_38 },
    { 0xec92993, SUC_lexicon_tags_15 },
    { 0xa3692994, SUC_lexicon_tags_2 },
    { 0x1a492995, SUC_lexicon_tags_11 },
    { 0x79c92996, SUC_lexicon_tags_9 },
    { 0x9e89298d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63e92999, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc9299c, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8929a0, SUC_lexicon_tags_46 },
    { 0x828929a0, SUC_lexicon_tags_92 },
    { 0x718929a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc929a8, SUC_lexicon_tags_30 },
    { 0x308929a9, SUC_lexicon_tags_15 },
    { 0x170929a9, SUC_lexicon_tags_0 },
    { 0xfc6929a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x724929ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfec929b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x332929b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e6929b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac929b7, SUC_lexicon_tags_23 },
    { 0xed4929b8, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240929bc, SUC_lexicon_tags_17 },
    { 0x416929bd, SUC_lexicon_tags_2 },
    { 0xba929bc, SUC_lexicon_tags_37 },
    { 0xcec929bf, SUC_lexicon_tags_12 },
    { 0x974929c0, SUC_lexicon_tags_27 },
    { 0x9c6929bf, SUC_lexicon_tags_1 },
    { 0x6a0929c2, SUC_lexicon_tags_118 },
    { 0xcde929c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe929c6, SUC_lexicon_tags_25 },
    { 0xa34929c7, SUC_lexicon_tags_38 },
    { 0xc7e929c7, SUC_lexicon_tags_19 },
    { 0xaf2929c9, SUC_lexicon_tags_60 },
    { 0x4e2929c9, SUC_lexicon_tags_25 },
    { 0x5aa929cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670929d1, SUC_lexicon_tags_34 },
    { 0x78e929d2, SUC_lexicon_tags_2 },
    { 0x708929d3, SUC_lexicon_tags_47 },
    { 0xaca929d2, SUC_lexicon_tags_48 },
    { 0xc3c929d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0929d7, SUC_lexicon_tags_17 },
    { 0x36c929d8, SUC_lexicon_tags_60 },
    { 0x484929d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4a929db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd64929dd, SUC_lexicon_tags_10 },
    { 0xfe0929de, SUC_lexicon_tags_7 },
    { 0xdca929df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36929e3, SUC_lexicon_tags_25 },
    { 0x880929e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea929e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae929ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46929ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90929f3, SUC_lexicon_tags_1 },
    { 0x8f4929f4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede929f9, SUC_lexicon_tags_49 },
    { 0x5b2929f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a929fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e92a00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c692a03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48892a06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61492a08, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x28692a0a, SUC_lexicon_tags_7 },
    { 0xb0c92a0b, SUC_lexicon_tags_1 },
    { 0xac92a0b, SUC_lexicon_tags_0 },
    { 0xf892a0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb892a11, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd2292a13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7092a17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e92a1b, SUC_lexicon_tags_11 },
    { 0xfd492a1b, SUC_lexicon_tags_56 },
    { 0x6d692a1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26e92a1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c92a22, SUC_lexicon_tags_9 },
    { 0x2e892a23, SUC_lexicon_tags_12 },
    { 0x70c92a22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x90c92a26, SUC_lexicon_tags_1 },
    { 0xc6e92a27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c92a2b, SUC_lexicon_tags_287 },
    { 0x24892a2b, SUC_lexicon_tags_11 },
    { 0x9f092a2d, SUC_lexicon_tags_14 },
    { 0x2f292a2e, SUC_lexicon_tags_11 },
    { 0xd8092a2c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6e892a31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a092a33, SUC_lexicon_tags_1 },
    { 0x16a92a34, SUC_lexicon_tags_17 },
    { 0xd2692a34, SUC_lexicon_tags_59 },
    { 0xe892a36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d492a38, SUC_lexicon_tags_29 },
    { 0x80092a38, SUC_lexicon_tags_33 },
    { 0xbc292a3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c92a3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc292a40, SUC_lexicon_tags_87 },
    { 0xe7892a40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32a92a43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac92a4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec92a50, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x69692a52, SUC_lexicon_tags_7 },
    { 0x23492a52, SUC_lexicon_tags_12 },
    { 0xb1092a52, SUC_lexicon_tags_46 },
    { 0x12a92a54, SUC_lexicon_tags_15 },
    { 0x33892a56, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x37892a58, SUC_lexicon_tags_323 },
    { 0x66492a59, SUC_lexicon_tags_160 },
    { 0x28892a59, SUC_lexicon_tags_15 },
    { 0x91492a5b, SUC_lexicon_tags_30 },
    { 0x1b092a5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc092a5e, SUC_lexicon_tags_17 },
    { 0xcda92a5f, SUC_lexicon_tags_19 },
    { 0x34292a5f, SUC_lexicon_tags_2 },
    { 0xec692a61, SUC_lexicon_tags_45 },
    { 0xdac92a62, SUC_lexicon_tags_29 },
    { 0x86a92a63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc3a92a65, SUC_lexicon_tags_9 },
    { 0x57c92a66, SUC_lexicon_tags_17 },
    { 0xce092a66, SUC_lexicon_tags_17 },
    { 0xaee92a68, SUC_lexicon_tags_9 },
    { 0x1ac92a67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34a92a6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a492a6f, SUC_lexicon_tags_37 },
    { 0x55292a70, SUC_lexicon_tags_2 },
    { 0x40292a70, SUC_lexicon_tags_0 },
    { 0xa8492a71, SUC_lexicon_tags_53 },
    { 0x3892a70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea892a75, SUC_lexicon_tags_37 },
    { 0x1f492a76, SUC_lexicon_tags_17 },
    { 0x4ee92a75, SUC_lexicon_tags_25 },
    { 0x28e92a78, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec92a84, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x40c92a86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d492a88, SUC_lexicon_tags_15 },
    { 0x1e692a89, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa5892a8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e92a90, SUC_lexicon_tags_2 },
    { 0x72092a91, SUC_lexicon_tags_30 },
    { 0x5492a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a92a95, SUC_lexicon_tags_12 },
    { 0x2f692a96, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x33492a98, SUC_lexicon_tags_1 },
    { 0xdac92a98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda692a9d, SUC_lexicon_tags_87 },
    { 0x2bc92a9d, SUC_lexicon_tags_11 },
    { 0x88692a9f, SUC_lexicon_tags_12 },
    { 0x24492aa0, SUC_lexicon_tags_13 },
    { 0x15a92a9f, SUC_lexicon_tags_68 },
    { 0x97e92aa2, SUC_lexicon_tags_2 },
    { 0xcd892aa3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xc3c92aa5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf7292aa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55892aa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x53092aac, SUC_lexicon_tags_87 },
    { 0xb9692aad, SUC_lexicon_tags_37 },
    { 0x82292aac, SUC_lexicon_tags_31 },
    { 0x95892aad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3892ab5, SUC_lexicon_tags_70 },
    { 0xf5692ab6, SUC_lexicon_tags_19 },
    { 0xc0c92ab6, SUC_lexicon_tags_11 },
    { 0x9da92ab6, SUC_lexicon_tags_37 },
    { 0x78e92ab6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x12892abc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8d092abe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x20892ac1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbea92ac3, SUC_lexicon_tags_37 },
    { 0x6d692ac4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c92ac7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa92ace, SUC_lexicon_tags_23 },
    { 0xc3c92acf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a92ad2, SUC_lexicon_tags_7 },
    { 0x34e92ad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc92ad7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92adb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b692add, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec92ae5, SUC_lexicon_tags_47 },
    { 0xaaa92ae6, SUC_lexicon_tags_37 },
    { 0xca92ae7, SUC_lexicon_tags_15 },
    { 0x57692ae8, SUC_lexicon_tags_8 },
    { 0x30c92ae5, SUC_lexicon_tags_11 },
    { 0xb2092aea, SUC_lexicon_tags_25 },
    { 0xcc892aea, SUC_lexicon_tags_21 },
    { 0xf9292aeb, SUC_lexicon_tags_18 },
    { 0x58892aec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce892af0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee92af3, SUC_lexicon_tags_2 },
    { 0xa8e92af3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb2092af6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a92afc, SUC_lexicon_tags_8 },
    { 0x66092afd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c92b06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e692b10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9a92b12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64092b17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabe92b19, SUC_lexicon_tags_7 },
    { 0xa8a92b1a, SUC_lexicon_tags_9 },
    { 0xe0692b1b, SUC_lexicon_tags_17 },
    { 0x9c092b1c, SUC_lexicon_tags_14 },
    { 0x90a92b1a, SUC_lexicon_tags_0 },
    { 0x5f292b1b, SUC_lexicon_tags_60 },
    { 0xc8292b1f, SUC_lexicon_tags_15 },
    { 0x81292b20, SUC_lexicon_tags_118 },
    { 0xc4092b21, SUC_lexicon_tags_8 },
    { 0x3aa92b22, SUC_lexicon_tags_1 },
    { 0x77092b23, SUC_lexicon_tags_34 },
    { 0x6f092b22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63492b26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e92b2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf292b2d, SUC_lexicon_tags_2 },
    { 0x7a092b2e, SUC_lexicon_tags_1 },
    { 0x45a92b2f, SUC_lexicon_tags_37 },
    { 0xc6c92b30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c092b34, SUC_lexicon_tags_37 },
    { 0x8c492b35, SUC_lexicon_tags_45 },
    { 0x8fa92b36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26092b39, SUC_lexicon_tags_23 },
    { 0xf1492b3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbc892b3c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d092b44, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x2e092b46, SUC_lexicon_tags_57 },
    { 0x23c92b47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8492b49, SUC_lexicon_tags_19 },
    { 0x9092b4a, SUC_lexicon_tags_25 },
    { 0xabc92b49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22292b4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d092b51, SUC_lexicon_tags_2 },
    { 0xf0a92b51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfa92b54, SUC_lexicon_tags_17 },
    { 0xc8092b55, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a692b5c, SUC_lexicon_tags_4 },
    { 0x4b092b5d, SUC_lexicon_tags_60 },
    { 0x60c92b5d, SUC_lexicon_tags_15 },
    { 0x49292b5f, SUC_lexicon_tags_14 },
    { 0xbba92b5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f092b62, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c492b67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43492b6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8092b71, SUC_lexicon_tags_7 },
    { 0x9aa92b72, SUC_lexicon_tags_17 },
    { 0xb0e92b72, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x28292b75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26692b77, SUC_lexicon_tags_49 },
    { 0xc4692b77, SUC_lexicon_tags_12 },
    { 0x67892b79, SUC_lexicon_tags_27 },
    { 0x1b692b7a, SUC_lexicon_tags_31 },
    { 0x41c92b78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43092b84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8692b86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca92b8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5092b8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe492b91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe492b97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce92b9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d492b9d, SUC_lexicon_tags_8 },
    { 0xe7a92b9e, SUC_lexicon_tags_12 },
    { 0xb7292b9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x12692ba2, SUC_lexicon_tags_43 },
    { 0x7292ba2, SUC_lexicon_tags_68 },
    { 0xb4492ba3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x72492ba7, SUC_lexicon_tags_14 },
    { 0xc2892ba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c92bab, SUC_lexicon_tags_12 },
    { 0xca692bac, SUC_lexicon_tags_25 },
    { 0xe6492bad, SUC_lexicon_tags_422 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc92bb0, SUC_lexicon_tags_17 },
    { 0x56a92bb1, SUC_lexicon_tags_38 },
    { 0x11e92bb2, SUC_lexicon_tags_31 },
    { 0x7d092bb3, SUC_lexicon_tags_38 },
    { 0x5a692bb4, SUC_lexicon_tags_17 },
    { 0x58292bb4, SUC_lexicon_tags_37 },
    { 0x6d492bb2, SUC_lexicon_tags_65 },
    { 0x3092bb0, SUC_lexicon_tags_120 },
    { 0xa4092bb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed892bbd, SUC_lexicon_tags_11 },
    { 0xbe92bbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a92bc5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc6292bc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72092bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf492bd2, SUC_lexicon_tags_91 },
    { 0x6de92bd2, SUC_lexicon_tags_47 },
    { 0x8a692bd2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf692bd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe492bda, SUC_lexicon_tags_19 },
    { 0x62892bdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39a92bdd, SUC_lexicon_tags_7 },
    { 0xd6e92bde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3892be5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98092beb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5692bee, SUC_lexicon_tags_34 },
    { 0xf8292bee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ca92bf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9092bf4, SUC_lexicon_tags_7 },
    { 0x2f292bf5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2892bf8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2692bfd, SUC_lexicon_tags_43 },
    { 0xcc692bfe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e892c00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc92c04, SUC_lexicon_tags_16 },
    { 0x17492c04, SUC_lexicon_tags_2 },
    { 0xe9a92c05, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a92c0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1092c0e, SUC_lexicon_tags_8 },
    { 0x9f892c0f, SUC_lexicon_tags_43 },
    { 0x67692c0f, SUC_lexicon_tags_15 },
    { 0x1b092c11, SUC_lexicon_tags_14 },
    { 0x3e892c10, SUC_lexicon_tags_25 },
    { 0x27692c12, SUC_lexicon_tags_11 },
    { 0x35492c14, SUC_lexicon_tags_9 },
    { 0x71492c12, SUC_lexicon_tags_7 },
    { 0x10892c0e, SUC_lexicon_tags_43 },
    { 0x79e92c17, SUC_lexicon_tags_11 },
    { 0x53692c13, SUC_lexicon_tags_91 },
    { 0xb4492c14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda92c1c, SUC_lexicon_tags_6 },
    { 0x3a292c1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba92c22, SUC_lexicon_tags_1 },
    { 0x69292c22, SUC_lexicon_tags_25 },
    { 0x40292c23, SUC_lexicon_tags_31 },
    { 0x69e92c22, SUC_lexicon_tags_37 },
    { 0xe4692c22, SUC_lexicon_tags_70 },
    { 0xd8292c25, SUC_lexicon_tags_25 },
    { 0xb4492c28, SUC_lexicon_tags_4 },
    { 0xbd092c28, SUC_lexicon_tags_87 },
    { 0xa3092c2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdd292c2c, SUC_lexicon_tags_66 },
    { 0xdc92c2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32492c2f, SUC_lexicon_tags_27 },
    { 0xe8892c2f, SUC_lexicon_tags_2 },
    { 0x89692c31, SUC_lexicon_tags_2 },
    { 0xe1a92c31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35a92c34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a92c3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba92c3f, SUC_lexicon_tags_11 },
    { 0x91c92c40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee92c45, SUC_lexicon_tags_2 },
    { 0xe1692c46, SUC_lexicon_tags_48 },
    { 0xb6092c47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d892c4c, SUC_lexicon_tags_49 },
    { 0x52692c4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a092c53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde92c55, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46692c5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69892c62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c92c66, SUC_lexicon_tags_0 },
    { 0xbbe92c67, SUC_lexicon_tags_17 },
    { 0xba892c68, SUC_lexicon_tags_2 },
    { 0x18492c69, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1492c6e, SUC_lexicon_tags_8 },
    { 0xe3092c6f, SUC_lexicon_tags_12 },
    { 0x56092c6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x29692c73, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x93892c76, SUC_lexicon_tags_12 },
    { 0xb0092c77, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x18492c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30e92c7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15e92c7e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xde892c80, SUC_lexicon_tags_38 },
    { 0x77292c81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a92c86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c892c88, SUC_lexicon_tags_17 },
    { 0x80892c88, SUC_lexicon_tags_15 },
    { 0x89892c8a, SUC_lexicon_tags_49 },
    { 0x66c92c8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9092c91, SUC_lexicon_tags_37 },
    { 0x43892c92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a92c95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ac92c97, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf3092c99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2292c9d, SUC_lexicon_tags_20 },
    { 0x57692c9e, SUC_lexicon_tags_37 },
    { 0xf0492c9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29092ca6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7fa92ca8, SUC_lexicon_tags_47 },
    { 0x64292ca8, SUC_lexicon_tags_12 },
    { 0xef892ca9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc2a92cac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc92caf, SUC_lexicon_tags_13 },
    { 0xdc292cb0, SUC_lexicon_tags_25 },
    { 0x18692cb1, SUC_lexicon_tags_61 },
    { 0x5f892cb2, SUC_lexicon_tags_4 },
    { 0x72e92cb2, SUC_lexicon_tags_12 },
    { 0x9c492cb1, SUC_lexicon_tags_14 },
    { 0x66292cb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x69e92cb7, SUC_lexicon_tags_9 },
    { 0xa5092cb8, SUC_lexicon_tags_35 },
    { 0xec092cb8, SUC_lexicon_tags_0 },
    { 0xef092cba, SUC_lexicon_tags_25 },
    { 0xeb092cb9, SUC_lexicon_tags_37 },
    { 0xf7a92cbc, SUC_lexicon_tags_2 },
    { 0xa292cbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd492cbf, SUC_lexicon_tags_37 },
    { 0xbec92cc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce292cc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab292cc6, SUC_lexicon_tags_47 },
    { 0x38692cc7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23892ccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe492ccf, SUC_lexicon_tags_2 },
    { 0x38c92cd0, SUC_lexicon_tags_43 },
    { 0x22292cd1, SUC_lexicon_tags_15 },
    { 0x3de92cd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54a92cd4, SUC_lexicon_tags_37 },
    { 0xc0892cd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79892cda, SUC_lexicon_tags_69 },
    { 0xa0292cda, SUC_lexicon_tags_11 },
    { 0xba892cdc, SUC_lexicon_tags_39 },
    { 0x292cdd, SUC_lexicon_tags_6 },
    { 0x60092cde, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x29092ce0, SUC_lexicon_tags_7 },
    { 0x97492ce1, SUC_lexicon_tags_78 },
    { 0x88e92ce1, SUC_lexicon_tags_56 },
    { 0x8cc92ce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca492ce6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd092cef, SUC_lexicon_tags_61 },
    { 0xf1692cef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c92cfc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f092d00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a92d04, SUC_lexicon_tags_29 },
    { 0xb4c92d05, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82292d09, SUC_lexicon_tags_0 },
    { 0x22a92d0a, SUC_lexicon_tags_1 },
    { 0xae092d0b, SUC_lexicon_tags_0 },
    { 0x7c492d09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c92d19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe92d1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1692d20, SUC_lexicon_tags_6 },
    { 0x5e092d21, SUC_lexicon_tags_0 },
    { 0x63a92d22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44c92d24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27492d26, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0892d2b, SUC_lexicon_tags_37 },
    { 0xe8892d2c, SUC_lexicon_tags_23 },
    { 0x51a92d2d, SUC_lexicon_tags_47 },
    { 0x1b892d2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e92d31, SUC_lexicon_tags_12 },
    { 0x9492d32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a92d37, SUC_lexicon_tags_475 },
    { 0x8b092d38, SUC_lexicon_tags_17 },
    { 0x39092d38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40492d3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a692d42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35092d44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e292d47, SUC_lexicon_tags_31 },
    { 0xb1e92d47, SUC_lexicon_tags_27 },
    { 0xef492d48, SUC_lexicon_tags_35 },
    { 0x98092d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72692d52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x61692d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38692d58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf1692d5a, SUC_lexicon_tags_68 },
    { 0x33092d5b, SUC_lexicon_tags_63 },
    { 0x27492d5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28692d61, SUC_lexicon_tags_6 },
    { 0xcea92d62, SUC_lexicon_tags_12 },
    { 0x31492d63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3be92d65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a92d68, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc4e92d6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37a92d6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2492d70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e92d74, SUC_lexicon_tags_14 },
    { 0x4cc92d75, SUC_lexicon_tags_1 },
    { 0x89c92d75, SUC_lexicon_tags_15 },
    { 0x1e492d74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c92d7c, SUC_lexicon_tags_1 },
    { 0xae892d7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc92d81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da92d86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6092d88, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57492d90, SUC_lexicon_tags_19 },
    { 0x9aa92d91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36292d97, SUC_lexicon_tags_1 },
    { 0x8e492d98, SUC_lexicon_tags_26 },
    { 0xe4892d99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae92da2, SUC_lexicon_tags_11 },
    { 0xcc292da3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63c92da5, SUC_lexicon_tags_33 },
    { 0x9c692da6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d892da8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92dac, SUC_lexicon_tags_56 },
    { 0xa092dad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0292db3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c092db5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea892dbb, SUC_lexicon_tags_37 },
    { 0x80292dbc, SUC_lexicon_tags_40 },
    { 0xd1892dbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57892dbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5c92dc1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x96e92dc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7892dc7, SUC_lexicon_tags_37 },
    { 0x98692dc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c92dcc, SUC_lexicon_tags_8 },
    { 0x4f692dcd, SUC_lexicon_tags_2 },
    { 0x7692dce, SUC_lexicon_tags_7 },
    { 0xf5492dcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ac92dd1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e92dd5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf4292dd7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x35092dda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c92dde, SUC_lexicon_tags_25 },
    { 0x3f692ddf, SUC_lexicon_tags_17 },
    { 0x5c292de0, SUC_lexicon_tags_37 },
    { 0x3e092de1, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7892de6, SUC_lexicon_tags_57 },
    { 0xb9892de6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf492dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22892dee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9292df0, SUC_lexicon_tags_37 },
    { 0xc4692df0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49492df5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a92df8, SUC_lexicon_tags_37 },
    { 0xdf292df9, SUC_lexicon_tags_17 },
    { 0x6f092dfa, SUC_lexicon_tags_25 },
    { 0x1ca92dfb, SUC_lexicon_tags_49 },
    { 0xc9892df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7092dfe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe3092e00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x33492e03, SUC_lexicon_tags_37 },
    { 0xebc92e04, SUC_lexicon_tags_10 },
    { 0x38a92e04, SUC_lexicon_tags_11 },
    { 0xed092e05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb692e0a, SUC_lexicon_tags_0 },
    { 0xafa92e0b, SUC_lexicon_tags_14 },
    { 0xb4292e0c, SUC_lexicon_tags_60 },
    { 0x6dc92e0b, SUC_lexicon_tags_25 },
    { 0x30492e0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e892e16, SUC_lexicon_tags_16 },
    { 0xf1e92e17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c92e1c, SUC_lexicon_tags_70 },
    { 0x56e92e1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c92e20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4892e24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27092e27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2292e2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c092e32, SUC_lexicon_tags_3 },
    { 0x9ec92e33, SUC_lexicon_tags_19 },
    { 0x84292e34, SUC_lexicon_tags_12 },
    { 0x28892e34, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e3c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7c492e3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c92e41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x57492e44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7292e4c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x45292e4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbc92e51, SUC_lexicon_tags_23 },
    { 0x8da92e52, SUC_lexicon_tags_15 },
    { 0x40892e52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51092e56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c92e59, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3492e62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fc92e64, SUC_lexicon_tags_10 },
    { 0x73292e64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa492e68, SUC_lexicon_tags_1 },
    { 0x28092e68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7c92e6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1692e6d, SUC_lexicon_tags_0 },
    { 0x7fa92e6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1292e73, SUC_lexicon_tags_7 },
    { 0xb4092e74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce92e78, SUC_lexicon_tags_37 },
    { 0xeb892e79, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcbe92e7b, SUC_lexicon_tags_106 },
    { 0xac692e7c, SUC_lexicon_tags_42 },
    { 0x1ae92e7d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99892e84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a92e8d, SUC_lexicon_tags_28 },
    { 0xf1692e8d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac892e92, SUC_lexicon_tags_23 },
    { 0x75892e92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c92e98, SUC_lexicon_tags_21 },
    { 0xf0892e98, SUC_lexicon_tags_60 },
    { 0xd5292e9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84892e9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c892e9e, SUC_lexicon_tags_7 },
    { 0x70492e9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28892ea1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88692eac, SUC_lexicon_tags_7 },
    { 0x77e92eac, SUC_lexicon_tags_6 },
    { 0x2c492ead, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11092eb2, SUC_lexicon_tags_27 },
    { 0x6c092eb3, SUC_lexicon_tags_10 },
    { 0xc3c92eb4, SUC_lexicon_tags_15 },
    { 0x7b292eb5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc0a92eb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4892eba, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x36292ebc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa9292ebe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e292ec0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6d292ec2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x53892ec5, SUC_lexicon_tags_25 },
    { 0x20092ec6, SUC_lexicon_tags_37 },
    { 0x7492ec7, SUC_lexicon_tags_11 },
    { 0xc6092ec7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac92ecc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17292ed9, SUC_lexicon_tags_23 },
    { 0xf8c92eda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce92edd, SUC_lexicon_tags_15 },
    { 0x9c892ede, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f692ee7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe092eec, SUC_lexicon_tags_15 },
    { 0x92c92eed, SUC_lexicon_tags_33 },
    { 0xbf292eed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a92ef1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x81c92ef3, SUC_lexicon_tags_47 },
    { 0x42292ef4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e92ef8, SUC_lexicon_tags_47 },
    { 0x20092ef9, SUC_lexicon_tags_11 },
    { 0x3492ef8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56692efc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b292eff, SUC_lexicon_tags_66 },
    { 0xbc492f00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5a492f02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x75292f05, SUC_lexicon_tags_15 },
    { 0xca092f06, SUC_lexicon_tags_10 },
    { 0xa1c92f07, SUC_lexicon_tags_27 },
    { 0x1fe92f06, SUC_lexicon_tags_2 },
    { 0x53292f09, SUC_lexicon_tags_12 },
    { 0x1d692f07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e892f0c, SUC_lexicon_tags_17 },
    { 0x6c92f0d, SUC_lexicon_tags_19 },
    { 0xa4292f0e, SUC_lexicon_tags_27 },
    { 0xd3292f0c, SUC_lexicon_tags_5 },
    { 0x49a92f10, SUC_lexicon_tags_7 },
    { 0x38492f11, SUC_lexicon_tags_15 },
    { 0x15692f0e, SUC_lexicon_tags_31 },
    { 0x10692f13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69a92f15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa92f18, SUC_lexicon_tags_27 },
    { 0x3e092f19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c692f1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf0892f1d, SUC_lexicon_tags_9 },
    { 0x64e92f1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17492f20, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe6892f22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a92f27, SUC_lexicon_tags_12 },
    { 0x42e92f28, SUC_lexicon_tags_17 },
    { 0xd2692f29, SUC_lexicon_tags_46 },
    { 0xbf892f2a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c92f2e, SUC_lexicon_tags_27 },
    { 0xc2492f2e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33292f34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c892f37, SUC_lexicon_tags_33 },
    { 0x5ec92f38, SUC_lexicon_tags_39 },
    { 0xce892f37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbe492f3b, SUC_lexicon_tags_2 },
    { 0x9a492f3c, SUC_lexicon_tags_151 },
    { 0x69a92f3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e92f42, SUC_lexicon_tags_60 },
    { 0xfba92f43, SUC_lexicon_tags_33 },
    { 0x59692f44, SUC_lexicon_tags_33 },
    { 0xb0692f45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee92f49, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e92f4c, SUC_lexicon_tags_17 },
    { 0x88892f4c, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xa1a92f4f, SUC_lexicon_tags_194 },
    { 0xbd292f50, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36092f52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x39492f55, SUC_lexicon_tags_17 },
    { 0x12c92f56, SUC_lexicon_tags_25 },
    { 0xa0492f57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf492f59, SUC_lexicon_tags_56 },
    { 0x59692f5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75092f5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xabe92f61, SUC_lexicon_tags_30 },
    { 0x78692f61, SUC_lexicon_tags_12 },
    { 0xd2c92f63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x79292f65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d092f69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdec92f6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20492f73, SUC_lexicon_tags_7 },
    { 0x2ca92f74, SUC_lexicon_tags_1 },
    { 0xaac92f74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5492f7c, SUC_lexicon_tags_15 },
    { 0xf9092f7d, SUC_lexicon_tags_25 },
    { 0xd1892f7e, SUC_lexicon_tags_45 },
    { 0x50a92f7f, SUC_lexicon_tags_37 },
    { 0xf5c92f80, SUC_lexicon_tags_15 },
    { 0xfde92f81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c92f85, SUC_lexicon_tags_15 },
    { 0x34692f86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67692f89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cc92f8b, SUC_lexicon_tags_25 },
    { 0xf5292f8c, SUC_lexicon_tags_12 },
    { 0xfb892f8c, SUC_lexicon_tags_7 },
    { 0x45492f8e, SUC_lexicon_tags_6 },
    { 0x7ec92f8e, SUC_lexicon_tags_66 },
    { 0xb4e92f90, SUC_lexicon_tags_18 },
    { 0x6c692f8e, SUC_lexicon_tags_165 },
    { 0x32892f8d, SUC_lexicon_tags_49 },
    { 0x10a92f8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e92f99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e092f9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf292fa1, SUC_lexicon_tags_11 },
    { 0x96292fa1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc892fa9, SUC_lexicon_tags_17 },
    { 0xc892fa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21a92fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0492fae, SUC_lexicon_tags_38 },
    { 0x13892fae, SUC_lexicon_tags_11 },
    { 0x15292fae, SUC_lexicon_tags_37 },
    { 0xf4092fb1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b892fb5, SUC_lexicon_tags_11 },
    { 0xb6692fb6, SUC_lexicon_tags_62 },
    { 0x5e492fb7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c092fba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73492fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b892fc5, SUC_lexicon_tags_21 },
    { 0xc2492fc6, SUC_lexicon_tags_7 },
    { 0x43a92fc6, SUC_lexicon_tags_17 },
    { 0xbec92fc8, SUC_lexicon_tags_28 },
    { 0x8da92fc9, SUC_lexicon_tags_17 },
    { 0x3e092fca, SUC_lexicon_tags_12 },
    { 0x3b292fcb, SUC_lexicon_tags_1 },
    { 0xf6e92fcb, SUC_lexicon_tags_14 },
    { 0x9492fcd, SUC_lexicon_tags_17 },
    { 0x94892fcd, SUC_lexicon_tags_170 },
    { 0x6e292fcd, SUC_lexicon_tags_13 },
    { 0x1b492fcc, SUC_lexicon_tags_17 },
    { 0x69c92fc7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xde492fd4, SUC_lexicon_tags_28 },
    { 0x30e92fd5, SUC_lexicon_tags_12 },
    { 0x87a92fd5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13292fdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97292fdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5692fe1, SUC_lexicon_tags_17 },
    { 0xac92fe1, SUC_lexicon_tags_11 },
    { 0x54292fe3, SUC_lexicon_tags_15 },
    { 0xfee92fe4, SUC_lexicon_tags_25 },
    { 0x73c92fe4, SUC_lexicon_tags_1 },
    { 0x8892fe5, SUC_lexicon_tags_17 },
    { 0x57c92fe7, SUC_lexicon_tags_79 },
    { 0x8fa92fe1, SUC_lexicon_tags_1 },
    { 0x2ae92fe1, SUC_lexicon_tags_151 },
    { 0x2f692fe7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x72292fed, SUC_lexicon_tags_2 },
    { 0x18692fee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xefe92ff0, SUC_lexicon_tags_35 },
    { 0xc4892ff1, SUC_lexicon_tags_124 },
    { 0x95c92ff0, SUC_lexicon_tags_14 },
    { 0x23492ff3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a92ff6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a92ffd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc93001, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e093003, SUC_lexicon_tags_29 },
    { 0x60a93004, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf893006, SUC_lexicon_tags_60 },
    { 0xec493007, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7f493009, SUC_lexicon_tags_2 },
    { 0xd4e9300a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96a9300c, SUC_lexicon_tags_15 },
    { 0x269300c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94493012, SUC_lexicon_tags_14 },
    { 0xf093013, SUC_lexicon_tags_46 },
    { 0x96893014, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5893018, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73693022, SUC_lexicon_tags_43 },
    { 0xb1e93023, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf693028, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb129302a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30a9302c, SUC_lexicon_tags_84 },
    { 0x249302d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e493031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9e93033, SUC_lexicon_tags_47 },
    { 0xe2893034, SUC_lexicon_tags_2 },
    { 0xbfe93034, SUC_lexicon_tags_2 },
    { 0xab493036, SUC_lexicon_tags_31 },
    { 0xb293037, SUC_lexicon_tags_2 },
    { 0xa9893034, SUC_lexicon_tags_30 },
    { 0x8bc93039, SUC_lexicon_tags_2 },
    { 0x3e29303a, SUC_lexicon_tags_28 },
    { 0x2ec9303b, SUC_lexicon_tags_43 },
    { 0xa109303b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2a09303e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x52493040, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x78093043, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae93046, SUC_lexicon_tags_37 },
    { 0x26093047, SUC_lexicon_tags_23 },
    { 0xc2293048, SUC_lexicon_tags_45 },
    { 0x7693048, SUC_lexicon_tags_19 },
    { 0xb8a93046, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9493053, SUC_lexicon_tags_19 },
    { 0xe2093054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52893057, SUC_lexicon_tags_17 },
    { 0xb7e93058, SUC_lexicon_tags_7 },
    { 0xa3c93059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc669305e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5493063, SUC_lexicon_tags_17 },
    { 0xa6c93064, SUC_lexicon_tags_27 },
    { 0x62a93063, SUC_lexicon_tags_14 },
    { 0x17c93063, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca9306a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd493074, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e93077, SUC_lexicon_tags_0 },
    { 0xe6a93078, SUC_lexicon_tags_1 },
    { 0x25893079, SUC_lexicon_tags_39 },
    { 0x65693078, SUC_lexicon_tags_15 },
    { 0x24a9307b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d69307f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x13893082, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39493086, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c9308e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe93091, SUC_lexicon_tags_7 },
    { 0x7f693091, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66893094, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9293098, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b89309c, SUC_lexicon_tags_37 },
    { 0xda29309d, SUC_lexicon_tags_69 },
    { 0x5309309e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce930a2, SUC_lexicon_tags_29 },
    { 0x556930a2, SUC_lexicon_tags_17 },
    { 0x7b4930a4, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xbbe930a6, SUC_lexicon_tags_15 },
    { 0xed0930a6, SUC_lexicon_tags_25 },
    { 0xb84930a8, SUC_lexicon_tags_15 },
    { 0x36e930a9, SUC_lexicon_tags_2 },
    { 0x2c930aa, SUC_lexicon_tags_2 },
    { 0x4cc930aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0930ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae930b1, SUC_lexicon_tags_21 },
    { 0xd0930b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe78930b4, SUC_lexicon_tags_46 },
    { 0x55e930b4, SUC_lexicon_tags_25 },
    { 0x5d2930b4, SUC_lexicon_tags_10 },
    { 0xe30930b4, SUC_lexicon_tags_28 },
    { 0xb0930b6, SUC_lexicon_tags_17 },
    { 0xc52930b9, SUC_lexicon_tags_11 },
    { 0xc9c930b4, SUC_lexicon_tags_11 },
    { 0xd70930bb, SUC_lexicon_tags_55 },
    { 0xac2930bc, SUC_lexicon_tags_11 },
    { 0x53c930bd, SUC_lexicon_tags_66 },
    { 0x850930be, SUC_lexicon_tags_25 },
    { 0xf62930bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e930c5, SUC_lexicon_tags_37 },
    { 0xc32930c6, SUC_lexicon_tags_12 },
    { 0x526930c6, SUC_lexicon_tags_19 },
    { 0xc58930c8, SUC_lexicon_tags_12 },
    { 0x780930c9, SUC_lexicon_tags_43 },
    { 0x2e0930ca, SUC_lexicon_tags_43 },
    { 0x39e930c8, SUC_lexicon_tags_15 },
    { 0x5fc930c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba930d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xee0930d2, SUC_lexicon_tags_12 },
    { 0x9d8930d3, SUC_lexicon_tags_26 },
    { 0xaf2930d3, SUC_lexicon_tags_56 },
    { 0xb86930d5, SUC_lexicon_tags_25 },
    { 0x54e930d5, SUC_lexicon_tags_25 },
    { 0x488930d6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a930da, SUC_lexicon_tags_2 },
    { 0xf16930da, SUC_lexicon_tags_37 },
    { 0x5b2930dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c930e6, SUC_lexicon_tags_11 },
    { 0x814930e7, SUC_lexicon_tags_7 },
    { 0xc10930e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc930eb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76930ee, SUC_lexicon_tags_25 },
    { 0xcce930ef, SUC_lexicon_tags_31 },
    { 0x900930f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204930f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594930fd, SUC_lexicon_tags_2 },
    { 0x98c930fd, SUC_lexicon_tags_146 },
    { 0xb64930fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa293104, SUC_lexicon_tags_29 },
    { 0xd6c93105, SUC_lexicon_tags_32 },
    { 0x5b893106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a93109, SUC_lexicon_tags_33 },
    { 0xb349310a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82493111, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81093115, SUC_lexicon_tags_96 },
    { 0x33293116, SUC_lexicon_tags_15 },
    { 0x7a693117, SUC_lexicon_tags_17 },
    { 0x17093118, SUC_lexicon_tags_1 },
    { 0xc9893118, SUC_lexicon_tags_37 },
    { 0xf7e9311a, SUC_lexicon_tags_17 },
    { 0xd9e9311a, SUC_lexicon_tags_45 },
    { 0x9a29311c, SUC_lexicon_tags_9 },
    { 0xc6a93116, SUC_lexicon_tags_719 },
    { 0xb2c93119, SUC_lexicon_tags_15 },
    { 0x76293118, SUC_lexicon_tags_47 },
    { 0x56693117, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4093126, SUC_lexicon_tags_19 },
    { 0x13093127, SUC_lexicon_tags_66 },
    { 0x30493127, SUC_lexicon_tags_43 },
    { 0x5c293128, SUC_lexicon_tags_17 },
    { 0xd293126, SUC_lexicon_tags_6 },
    { 0x4789312b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2529312f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb093132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c93136, SUC_lexicon_tags_11 },
    { 0xe1693136, SUC_lexicon_tags_25 },
    { 0xaf293138, SUC_lexicon_tags_43 },
    { 0x2293138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4093140, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e93144, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93693148, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5989314d, SUC_lexicon_tags_61 },
    { 0xc0e9314e, SUC_lexicon_tags_12 },
    { 0xd769314d, SUC_lexicon_tags_47 },
    { 0xbf693150, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14693157, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9315b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa389315d, SUC_lexicon_tags_15 },
    { 0xc1c9315e, SUC_lexicon_tags_25 },
    { 0xdce9315f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7093164, SUC_lexicon_tags_19 },
    { 0x8bc93165, SUC_lexicon_tags_17 },
    { 0xfc93166, SUC_lexicon_tags_7 },
    { 0x43c93167, SUC_lexicon_tags_37 },
    { 0xd0093164, SUC_lexicon_tags_35 },
    { 0xb8693166, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e9316e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d493171, SUC_lexicon_tags_7 },
    { 0xc0293171, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c693178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f49317a, SUC_lexicon_tags_2 },
    { 0xde9317b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1569317d, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e293186, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f49318a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7a9318c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa893191, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a893195, SUC_lexicon_tags_23 },
    { 0xec93195, SUC_lexicon_tags_17 },
    { 0x51293196, SUC_lexicon_tags_11 },
    { 0x22493198, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x45a9319a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2931a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2931a4, SUC_lexicon_tags_17 },
    { 0x9c0931a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8931aa, SUC_lexicon_tags_27 },
    { 0x488931ab, SUC_lexicon_tags_88 },
    { 0x9c6931ab, SUC_lexicon_tags_37 },
    { 0xc10931ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce931af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c8931b1, SUC_lexicon_tags_26 },
    { 0x586931b2, SUC_lexicon_tags_25 },
    { 0x976931b3, SUC_lexicon_tags_49 },
    { 0x844931b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50e931b6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88931bc, SUC_lexicon_tags_7 },
    { 0x54e931bd, SUC_lexicon_tags_14 },
    { 0xf0e931bc, SUC_lexicon_tags_19 },
    { 0xec8931bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda931c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78931ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x286931cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8931d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2931d6, SUC_lexicon_tags_47 },
    { 0x5cc931d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10931e3, SUC_lexicon_tags_15 },
    { 0xbe0931e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0931e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e931ee, SUC_lexicon_tags_37 },
    { 0x8a4931ee, SUC_lexicon_tags_37 },
    { 0x7ae931f0, SUC_lexicon_tags_27 },
    { 0xec8931f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70931f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68931f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c931fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e931fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb693201, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e93207, SUC_lexicon_tags_17 },
    { 0x69293207, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa749320d, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac9321d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3693220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58293222, SUC_lexicon_tags_26 },
    { 0x65893223, SUC_lexicon_tags_28 },
    { 0x21093224, SUC_lexicon_tags_27 },
    { 0x19893225, SUC_lexicon_tags_38 },
    { 0xcca93224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf409322a, SUC_lexicon_tags_27 },
    { 0x1009322b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed493233, SUC_lexicon_tags_7 },
    { 0xef693233, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a93239, SUC_lexicon_tags_1 },
    { 0x5869323a, SUC_lexicon_tags_77 },
    { 0x6849323b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd49323d, SUC_lexicon_tags_12 },
    { 0x5449323d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x16693240, SUC_lexicon_tags_29 },
    { 0x94893241, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x58893243, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96493249, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c9324e, SUC_lexicon_tags_13 },
    { 0xae9324f, SUC_lexicon_tags_63 },
    { 0x9689324e, SUC_lexicon_tags_65 },
    { 0x4f89324e, SUC_lexicon_tags_17 },
    { 0x7c893252, SUC_lexicon_tags_11 },
    { 0xe6c93253, SUC_lexicon_tags_37 },
    { 0xc5e93252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f493256, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f293259, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbea9325b, SUC_lexicon_tags_6 },
    { 0x88e9325c, SUC_lexicon_tags_4 },
    { 0x4809325c, SUC_lexicon_tags_9 },
    { 0xf249325b, SUC_lexicon_tags_2 },
    { 0x1f29325e, SUC_lexicon_tags_11 },
    { 0x21c93260, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff493265, SUC_lexicon_tags_25 },
    { 0x31493266, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac93269, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd89326b, SUC_lexicon_tags_17 },
    { 0xb2c9326b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e93271, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e93274, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe689327b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x749327d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1293281, SUC_lexicon_tags_5 },
    { 0xdc693281, SUC_lexicon_tags_39 },
    { 0xa3e93282, SUC_lexicon_tags_47 },
    { 0x88e93284, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35293289, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc49328c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1e49328e, SUC_lexicon_tags_19 },
    { 0x7f09328e, SUC_lexicon_tags_1 },
    { 0xc9c93290, SUC_lexicon_tags_11 },
    { 0xf0093290, SUC_lexicon_tags_37 },
    { 0x17c93292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4293295, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba93299, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66932a1, SUC_lexicon_tags_29 },
    { 0x960932a2, SUC_lexicon_tags_25 },
    { 0x6e6932a3, SUC_lexicon_tags_0 },
    { 0x57c932a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb70932a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac932ad, SUC_lexicon_tags_49 },
    { 0x3e2932ae, SUC_lexicon_tags_37 },
    { 0xfee932ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x776932b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766932b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650932bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca932be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x108932c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa6932c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x67e932c5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6932c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7aa932ca, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8932d1, SUC_lexicon_tags_26 },
    { 0x500932d2, SUC_lexicon_tags_43 },
    { 0xe48932d1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122932d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4932dd, SUC_lexicon_tags_25 },
    { 0x28c932de, SUC_lexicon_tags_35 },
    { 0xe72932df, SUC_lexicon_tags_17 },
    { 0xd56932de, SUC_lexicon_tags_65 },
    { 0x31a932e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4932e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164932ea, SUC_lexicon_tags_25 },
    { 0xd40932ea, SUC_lexicon_tags_37 },
    { 0x850932ec, SUC_lexicon_tags_29 },
    { 0xea932ed, SUC_lexicon_tags_25 },
    { 0x30932ee, SUC_lexicon_tags_17 },
    { 0xa80932ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbac932f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a932f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dc932f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c932fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x77a932fd, SUC_lexicon_tags_1 },
    { 0xae4932fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x34c93300, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x53693302, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa93305, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5293308, SUC_lexicon_tags_25 },
    { 0xfce93309, SUC_lexicon_tags_17 },
    { 0x449330a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4389330c, SUC_lexicon_tags_23 },
    { 0x3f09330d, SUC_lexicon_tags_11 },
    { 0xe5a9330d, SUC_lexicon_tags_11 },
    { 0x7b89330c, SUC_lexicon_tags_11 },
    { 0x6be93310, SUC_lexicon_tags_29 },
    { 0xe9330c, SUC_lexicon_tags_15 },
    { 0xefa93312, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42493315, SUC_lexicon_tags_26 },
    { 0x42c93316, SUC_lexicon_tags_1 },
    { 0x99693317, SUC_lexicon_tags_25 },
    { 0x6293316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a9331b, SUC_lexicon_tags_37 },
    { 0x9709331c, SUC_lexicon_tags_66 },
    { 0xed89331d, SUC_lexicon_tags_15 },
    { 0x589331c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6293322, SUC_lexicon_tags_17 },
    { 0x8cc93322, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8093329, SUC_lexicon_tags_37 },
    { 0x6e29332a, SUC_lexicon_tags_11 },
    { 0xd269332b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a9332f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29293331, SUC_lexicon_tags_26 },
    { 0xf2093331, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47093335, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c93338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a9333b, SUC_lexicon_tags_12 },
    { 0xd889333c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2869333e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4493342, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac93345, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9929334b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a93350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a9335f, SUC_lexicon_tags_21 },
    { 0x4693360, SUC_lexicon_tags_1 },
    { 0x32e93361, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff893367, SUC_lexicon_tags_19 },
    { 0xb9893368, SUC_lexicon_tags_37 },
    { 0x8b893367, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a9336f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea93373, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa93379, SUC_lexicon_tags_33 },
    { 0xa0693379, SUC_lexicon_tags_37 },
    { 0x8989337b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x1149337e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea93382, SUC_lexicon_tags_14 },
    { 0x8f893383, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99693387, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcbe93389, SUC_lexicon_tags_1 },
    { 0x4329338a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf249338d, SUC_lexicon_tags_11 },
    { 0xeea9338e, SUC_lexicon_tags_4 },
    { 0x22a9338f, SUC_lexicon_tags_5 },
    { 0x9f29338d, SUC_lexicon_tags_62 },
    { 0x37e9338d, SUC_lexicon_tags_11 },
    { 0xd4e9338d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2c093394, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x27c93396, SUC_lexicon_tags_7 },
    { 0x99a93396, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbfa93399, SUC_lexicon_tags_30 },
    { 0x5c93399, SUC_lexicon_tags_28 },
    { 0x2ce9339b, SUC_lexicon_tags_10 },
    { 0x1a29339c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e49339f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x318933a1, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x933a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x842933a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2933aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6933ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624933b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e933b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd50933bb, SUC_lexicon_tags_43 },
    { 0x2ea933bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c933bf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72933c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfe933c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2933ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134933ce, SUC_lexicon_tags_1 },
    { 0xa36933ce, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf98933d1, SUC_lexicon_tags_17 },
    { 0x782933d1, SUC_lexicon_tags_12 },
    { 0xb8e933d1, SUC_lexicon_tags_11 },
    { 0x7f4933d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48e933d6, SUC_lexicon_tags_33 },
    { 0x4e8933d7, SUC_lexicon_tags_11 },
    { 0xd0c933d8, SUC_lexicon_tags_17 },
    { 0xb0e933d9, SUC_lexicon_tags_35 },
    { 0xcc4933d8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a933e3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca933e7, SUC_lexicon_tags_25 },
    { 0x40933e8, SUC_lexicon_tags_11 },
    { 0x328933e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24933ee, SUC_lexicon_tags_11 },
    { 0xc7c933ee, SUC_lexicon_tags_25 },
    { 0x4e2933f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40933f3, SUC_lexicon_tags_27 },
    { 0x3f4933f4, SUC_lexicon_tags_7 },
    { 0x910933f5, SUC_lexicon_tags_12 },
    { 0xbdc933f6, SUC_lexicon_tags_37 },
    { 0xadc933f7, SUC_lexicon_tags_2 },
    { 0x28933f3, SUC_lexicon_tags_37 },
    { 0xb98933f9, SUC_lexicon_tags_25 },
    { 0x3a0933f9, SUC_lexicon_tags_49 },
    { 0xfc6933f9, SUC_lexicon_tags_9 },
    { 0x59e933fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac893401, SUC_lexicon_tags_37 },
    { 0x8b493402, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3c93404, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x55693407, SUC_lexicon_tags_27 },
    { 0x5c893408, SUC_lexicon_tags_11 },
    { 0xb4893409, SUC_lexicon_tags_12 },
    { 0xf7093409, SUC_lexicon_tags_1 },
    { 0xe3c9340b, SUC_lexicon_tags_25 },
    { 0x609340c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c93410, SUC_lexicon_tags_31 },
    { 0x2d893411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4893416, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc9341f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e93426, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb493429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe849342d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6093432, SUC_lexicon_tags_6 },
    { 0xae693433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2893435, SUC_lexicon_tags_23 },
    { 0x7a493436, SUC_lexicon_tags_55 },
    { 0xda693436, SUC_lexicon_tags_7 },
    { 0x39093438, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6789343c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b893442, SUC_lexicon_tags_11 },
    { 0x25493443, SUC_lexicon_tags_6 },
    { 0x7ba93443, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24893446, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc9344a, SUC_lexicon_tags_99 },
    { 0xf369344b, SUC_lexicon_tags_7 },
    { 0x45e9344c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c93450, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc093454, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xab893457, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xa3093459, SUC_lexicon_tags_37 },
    { 0x19a9345a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa9345f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86693469, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ac9346b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xb8c9346d, SUC_lexicon_tags_103 },
    { 0xb8c9346d, SUC_lexicon_tags_17 },
    { 0x2dc9346e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x78e93471, SUC_lexicon_tags_15 },
    { 0x38293472, SUC_lexicon_tags_63 },
    { 0x90a93473, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6693475, SUC_lexicon_tags_188 },
    { 0x86693476, SUC_lexicon_tags_21 },
    { 0x1d693477, SUC_lexicon_tags_37 },
    { 0x5be93478, SUC_lexicon_tags_31 },
    { 0x31093478, SUC_lexicon_tags_87 },
    { 0x5d09347a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6109347f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72693483, SUC_lexicon_tags_4 },
    { 0x9a893484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe9348d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb493493, SUC_lexicon_tags_14 },
    { 0x1ae93494, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xed693497, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc09349c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7309349e, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc934a2, SUC_lexicon_tags_4 },
    { 0x2ac934a2, SUC_lexicon_tags_46 },
    { 0x29e934a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e934ab, SUC_lexicon_tags_2 },
    { 0x6aa934ac, SUC_lexicon_tags_27 },
    { 0xfc934ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc934b0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xc90934b2, SUC_lexicon_tags_2 },
    { 0x6d8934b2, SUC_lexicon_tags_2 },
    { 0x6c934b4, SUC_lexicon_tags_1 },
    { 0xd6934b5, SUC_lexicon_tags_49 },
    { 0x576934b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdcc934b8, SUC_lexicon_tags_29 },
    { 0x29c934b9, SUC_lexicon_tags_11 },
    { 0xca6934b9, SUC_lexicon_tags_14 },
    { 0x60e934b8, SUC_lexicon_tags_77 },
    { 0x17c934bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc20934be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c934c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a934ce, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x276934d0, SUC_lexicon_tags_14 },
    { 0x47c934d0, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e934d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa9a934d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c934e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c934e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56934ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ce934f0, SUC_lexicon_tags_37 },
    { 0x5da934f0, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0x4ac934f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0934f6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6934f9, SUC_lexicon_tags_77 },
    { 0xc34934f9, SUC_lexicon_tags_118 },
    { 0x836934f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e93500, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97c93502, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0493508, SUC_lexicon_tags_17 },
    { 0xfba93509, SUC_lexicon_tags_12 },
    { 0xd4c93509, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a69350d, SUC_lexicon_tags_23 },
    { 0x1869350e, SUC_lexicon_tags_7 },
    { 0xaa69350e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d493511, SUC_lexicon_tags_11 },
    { 0x92293511, SUC_lexicon_tags_7 },
    { 0x6d693513, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a93517, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4789351b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1549351d, SUC_lexicon_tags_67 },
    { 0x1d09351e, SUC_lexicon_tags_48 },
    { 0x4269351e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa93525, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7493528, SUC_lexicon_tags_31 },
    { 0x30693528, SUC_lexicon_tags_37 },
    { 0x70c9352a, SUC_lexicon_tags_28 },
    { 0x41e9352b, SUC_lexicon_tags_17 },
    { 0x55a93529, SUC_lexicon_tags_0 },
    { 0x4389352d, SUC_lexicon_tags_17 },
    { 0x24a9352a, SUC_lexicon_tags_0 },
    { 0x8c89352f, SUC_lexicon_tags_25 },
    { 0xaae9352f, SUC_lexicon_tags_2 },
    { 0x19c9352d, SUC_lexicon_tags_17 },
    { 0x2909352e, SUC_lexicon_tags_17 },
    { 0x8a93533, SUC_lexicon_tags_77 },
    { 0x79093528, SUC_lexicon_tags_19 },
    { 0x3d09352a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe493538, SUC_lexicon_tags_37 },
    { 0x78a93539, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88093543, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0693548, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3289354b, SUC_lexicon_tags_15 },
    { 0x4369354c, SUC_lexicon_tags_39 },
    { 0x7a69354d, SUC_lexicon_tags_12 },
    { 0x86e9354b, SUC_lexicon_tags_25 },
    { 0x9509354c, SUC_lexicon_tags_9 },
    { 0x24293550, SUC_lexicon_tags_25 },
    { 0xdb69354f, SUC_lexicon_tags_12 },
    { 0x63893552, SUC_lexicon_tags_17 },
    { 0xfc293553, SUC_lexicon_tags_7 },
    { 0x34093554, SUC_lexicon_tags_254 },
    { 0xb6893551, SUC_lexicon_tags_17 },
    { 0x22293556, SUC_lexicon_tags_2 },
    { 0x3de93556, SUC_lexicon_tags_37 },
    { 0x54893556, SUC_lexicon_tags_47 },
    { 0x43093556, SUC_lexicon_tags_1 },
    { 0x3293559, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a9355e, SUC_lexicon_tags_82 },
    { 0x1589355e, SUC_lexicon_tags_17 },
    { 0x1049355f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a93564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde29356e, SUC_lexicon_tags_17 },
    { 0x50e9356f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x86293572, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f093574, SUC_lexicon_tags_31 },
    { 0xbd493574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77a93577, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd5a93579, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc93584, SUC_lexicon_tags_8 },
    { 0x72a93585, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc93592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b093595, SUC_lexicon_tags_334 },
    { 0x67693595, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x74893598, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd189359a, SUC_lexicon_tags_1 },
    { 0x8629359b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a9359e, SUC_lexicon_tags_1 },
    { 0x229359f, SUC_lexicon_tags_2 },
    { 0xbac935a0, SUC_lexicon_tags_0 },
    { 0x956935a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16935a5, SUC_lexicon_tags_11 },
    { 0x9e8935a6, SUC_lexicon_tags_12 },
    { 0x2aa935a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd32935a9, SUC_lexicon_tags_19 },
    { 0xdf6935aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5aa935ac, SUC_lexicon_tags_7 },
    { 0x972935ac, SUC_lexicon_tags_14 },
    { 0xa2c935ac, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0xe78935b0, SUC_lexicon_tags_61 },
    { 0xec2935b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be935b7, SUC_lexicon_tags_25 },
    { 0xb26935b8, SUC_lexicon_tags_11 },
    { 0x31e935b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2935c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c935d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdb4935d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0935d8, SUC_lexicon_tags_30 },
    { 0xc20935d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a935df, SUC_lexicon_tags_45 },
    { 0xba2935df, SUC_lexicon_tags_1 },
    { 0xac0935e1, SUC_lexicon_tags_63 },
    { 0x4c2935e2, SUC_lexicon_tags_5 },
    { 0x8e6935df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288935e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2935ea, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x2c8935ec, SUC_lexicon_tags_1 },
    { 0x208935ed, SUC_lexicon_tags_37 },
    { 0xe86935ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c935f7, SUC_lexicon_tags_29 },
    { 0xbe4935f7, SUC_lexicon_tags_47 },
    { 0xb70935f9, SUC_lexicon_tags_27 },
    { 0xf0a935f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f093602, SUC_lexicon_tags_25 },
    { 0xbf693603, SUC_lexicon_tags_47 },
    { 0xb4a93604, SUC_lexicon_tags_1 },
    { 0x78493602, SUC_lexicon_tags_18 },
    { 0xa1a93606, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d89360d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf189360f, SUC_lexicon_tags_17 },
    { 0x77c9360f, SUC_lexicon_tags_25 },
    { 0xf6893610, SUC_lexicon_tags_57 },
    { 0x61a93612, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a493615, SUC_lexicon_tags_25 },
    { 0x4ac93615, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f693618, SUC_lexicon_tags_11 },
    { 0xba93618, SUC_lexicon_tags_65 },
    { 0x1689361a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d493620, SUC_lexicon_tags_5 },
    { 0xa4893620, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e93624, SUC_lexicon_tags_12 },
    { 0x2ce93624, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb289362a, SUC_lexicon_tags_84 },
    { 0x3809362a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f29362f, SUC_lexicon_tags_11 },
    { 0x929362f, SUC_lexicon_tags_0 },
    { 0xeaa93631, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a93639, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87293645, SUC_lexicon_tags_25 },
    { 0x893645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9a93648, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae93657, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd6093659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c09365b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb69365e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c93661, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8493664, SUC_lexicon_tags_19 },
    { 0x10293665, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5529366b, SUC_lexicon_tags_25 },
    { 0xa89366c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f893670, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20c93672, SUC_lexicon_tags_2 },
    { 0xb6e93672, SUC_lexicon_tags_25 },
    { 0x2693674, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c93677, SUC_lexicon_tags_14 },
    { 0xedc93678, SUC_lexicon_tags_37 },
    { 0x8e493678, SUC_lexicon_tags_0 },
    { 0xbf49367a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe069367d, SUC_lexicon_tags_1 },
    { 0x6509367e, SUC_lexicon_tags_26 },
    { 0x99a9367f, SUC_lexicon_tags_37 },
    { 0x72e93680, SUC_lexicon_tags_26 },
    { 0xd4693681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2493683, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4893686, SUC_lexicon_tags_9 },
    { 0x3b293687, SUC_lexicon_tags_1 },
    { 0xa93688, SUC_lexicon_tags_2 },
    { 0xc2a93689, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb729368b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce93694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c893696, SUC_lexicon_tags_1 },
    { 0x61e93697, SUC_lexicon_tags_1 },
    { 0xfaa93698, SUC_lexicon_tags_60 },
    { 0xe93697, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9d49369b, SUC_lexicon_tags_0 },
    { 0x2ac9369c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59e9369e, SUC_lexicon_tags_37 },
    { 0x5529369e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e936a3, SUC_lexicon_tags_2 },
    { 0x1a936a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c936a7, SUC_lexicon_tags_9 },
    { 0x5d8936a7, SUC_lexicon_tags_0 },
    { 0x26936a9, SUC_lexicon_tags_25 },
    { 0xd16936a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6936ad, SUC_lexicon_tags_7 },
    { 0xe16936ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac936b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba8936b3, SUC_lexicon_tags_12 },
    { 0xf24936b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x482936b7, SUC_lexicon_tags_1 },
    { 0x818936b8, SUC_lexicon_tags_30 },
    { 0xc7e936b8, SUC_lexicon_tags_25 },
    { 0x91a936b8, SUC_lexicon_tags_14 },
    { 0x160936bb, SUC_lexicon_tags_14 },
    { 0x67a936bc, SUC_lexicon_tags_47 },
    { 0x28a936bc, SUC_lexicon_tags_31 },
    { 0xde936be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc936c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2e8936c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2a936c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772936cd, SUC_lexicon_tags_23 },
    { 0xbac936ce, SUC_lexicon_tags_170 },
    { 0xd0c936cf, SUC_lexicon_tags_12 },
    { 0x7c8936cd, SUC_lexicon_tags_29 },
    { 0x244936d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0936d5, SUC_lexicon_tags_63 },
    { 0xd8e936d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4b6936d8, SUC_lexicon_tags_28 },
    { 0xbb2936d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x74936dc, SUC_lexicon_tags_38 },
    { 0xf70936dc, SUC_lexicon_tags_37 },
    { 0x916936de, SUC_lexicon_tags_8 },
    { 0x96a936de, SUC_lexicon_tags_23 },
    { 0x752936e0, SUC_lexicon_tags_28 },
    { 0xf9e936e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430936e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0936e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xad4936eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa98936ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e936f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4936f8, SUC_lexicon_tags_700 },
    { 0, NULL },
    { 0, NULL },
    { 0x802936fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8936ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xade93701, SUC_lexicon_tags_65 },
    { 0xd5493702, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2893706, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a493713, SUC_lexicon_tags_25 },
    { 0x9f093713, SUC_lexicon_tags_23 },
    { 0xacc93715, SUC_lexicon_tags_28 },
    { 0x3e493716, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c9371a, SUC_lexicon_tags_29 },
    { 0x3e9371a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba49371d, SUC_lexicon_tags_29 },
    { 0xa3c9371e, SUC_lexicon_tags_2 },
    { 0x6569371e, SUC_lexicon_tags_11 },
    { 0x7709371d, SUC_lexicon_tags_66 },
    { 0xb689371e, SUC_lexicon_tags_38 },
    { 0xe569371f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafc93724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac9372b, SUC_lexicon_tags_6 },
    { 0x5049372b, SUC_lexicon_tags_16 },
    { 0x8a09372d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e93731, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99293736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63493738, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c9373d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec93741, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6093748, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda69374a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e9374e, SUC_lexicon_tags_1 },
    { 0x4189374f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xffe93751, SUC_lexicon_tags_12 },
    { 0x1093752, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa4893754, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c93759, SUC_lexicon_tags_0 },
    { 0x5c9375a, SUC_lexicon_tags_12 },
    { 0x5a09375b, SUC_lexicon_tags_11 },
    { 0x4bc93759, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x829375f, SUC_lexicon_tags_12 },
    { 0x9a89375f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43493764, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea93767, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d49376b, SUC_lexicon_tags_7 },
    { 0x33a9376b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce69376e, SUC_lexicon_tags_32 },
    { 0x5f89376e, SUC_lexicon_tags_43 },
    { 0x97e93770, SUC_lexicon_tags_1 },
    { 0x1b493770, SUC_lexicon_tags_108 },
    { 0xe6e93771, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7493774, SUC_lexicon_tags_11 },
    { 0x30a93774, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc9377c, SUC_lexicon_tags_49 },
    { 0x69a9377d, SUC_lexicon_tags_9 },
    { 0x569377e, SUC_lexicon_tags_1 },
    { 0xb489377f, SUC_lexicon_tags_29 },
    { 0x71e93780, SUC_lexicon_tags_29 },
    { 0x76e9377e, SUC_lexicon_tags_1 },
    { 0x36493782, SUC_lexicon_tags_27 },
    { 0xcb293780, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x68e93785, SUC_lexicon_tags_10 },
    { 0x63893786, SUC_lexicon_tags_8 },
    { 0x4b493786, SUC_lexicon_tags_16 },
    { 0x24a93785, SUC_lexicon_tags_27 },
    { 0x9e893787, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9909378f, SUC_lexicon_tags_55 },
    { 0xcc893790, SUC_lexicon_tags_72 },
    { 0xe289378f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49693795, SUC_lexicon_tags_60 },
    { 0x12893796, SUC_lexicon_tags_30 },
    { 0xb5e93797, SUC_lexicon_tags_15 },
    { 0xb7c93796, SUC_lexicon_tags_23 },
    { 0x7093799, SUC_lexicon_tags_11 },
    { 0x12093799, SUC_lexicon_tags_47 },
    { 0x2069379b, SUC_lexicon_tags_47 },
    { 0x7ce9379c, SUC_lexicon_tags_449 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30937a5, SUC_lexicon_tags_65 },
    { 0x4f2937a6, SUC_lexicon_tags_33 },
    { 0x330937a5, SUC_lexicon_tags_29 },
    { 0x696937a8, SUC_lexicon_tags_11 },
    { 0xcc8937a8, SUC_lexicon_tags_59 },
    { 0x7da937a6, SUC_lexicon_tags_11 },
    { 0x2ec937ab, SUC_lexicon_tags_63 },
    { 0x8ee937a6, SUC_lexicon_tags_38 },
    { 0xbaa937a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c937b3, SUC_lexicon_tags_14 },
    { 0xa1e937b4, SUC_lexicon_tags_6 },
    { 0xd5c937b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6937bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce937c0, SUC_lexicon_tags_8 },
    { 0xf2937c0, SUC_lexicon_tags_2 },
    { 0x868937c2, SUC_lexicon_tags_8 },
    { 0x2fc937c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x896937c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2a2937c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208937cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee937d0, SUC_lexicon_tags_1 },
    { 0x610937d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7a937d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6937d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61e937db, SUC_lexicon_tags_1 },
    { 0xae2937dc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0937df, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68937e2, SUC_lexicon_tags_19 },
    { 0x362937e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x32937e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee937e8, SUC_lexicon_tags_1 },
    { 0x894937e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x40a937eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c937f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c937f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce0937f5, SUC_lexicon_tags_19 },
    { 0x9d4937f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78937fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64937ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf2a93801, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xafa93803, SUC_lexicon_tags_25 },
    { 0x2b293804, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa9380e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a93813, SUC_lexicon_tags_26 },
    { 0xee093814, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6b893816, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c9381a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1709381c, SUC_lexicon_tags_61 },
    { 0x529381d, SUC_lexicon_tags_11 },
    { 0xc729381d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe93825, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a9382a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd093831, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc693834, SUC_lexicon_tags_0 },
    { 0x64693835, SUC_lexicon_tags_37 },
    { 0x20093836, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4609383e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4893841, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9693843, SUC_lexicon_tags_1 },
    { 0x1d693844, SUC_lexicon_tags_14 },
    { 0xc9c93845, SUC_lexicon_tags_17 },
    { 0xfb293846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e9384a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91a9384c, SUC_lexicon_tags_23 },
    { 0x6de9384d, SUC_lexicon_tags_37 },
    { 0x18c9384d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x87693850, SUC_lexicon_tags_37 },
    { 0xc8c93851, SUC_lexicon_tags_27 },
    { 0x1c493852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c9385d, SUC_lexicon_tags_12 },
    { 0x28a9385e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef293860, SUC_lexicon_tags_37 },
    { 0x66e93861, SUC_lexicon_tags_25 },
    { 0x3cc93861, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xef293865, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30093869, SUC_lexicon_tags_46 },
    { 0x9ac9386a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48493872, SUC_lexicon_tags_1 },
    { 0x17093873, SUC_lexicon_tags_63 },
    { 0x93893873, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f493877, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c9387b, SUC_lexicon_tags_56 },
    { 0x26c9387c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e93886, SUC_lexicon_tags_1 },
    { 0x42693887, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x289388a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb229388d, SUC_lexicon_tags_25 },
    { 0x53a9388e, SUC_lexicon_tags_26 },
    { 0x83e9388f, SUC_lexicon_tags_17 },
    { 0x48a93890, SUC_lexicon_tags_37 },
    { 0x67e93891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69893895, SUC_lexicon_tags_38 },
    { 0x7ce93896, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa9389b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee9389e, SUC_lexicon_tags_77 },
    { 0xf269389e, SUC_lexicon_tags_17 },
    { 0x300938a0, SUC_lexicon_tags_0 },
    { 0x6269389f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718938a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93a938a7, SUC_lexicon_tags_37 },
    { 0xff8938a8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc938ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2938b5, SUC_lexicon_tags_11 },
    { 0xf1c938b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88938bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0938be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2938c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee938c3, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0xb9c938c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc8938c7, SUC_lexicon_tags_10 },
    { 0xe3e938c8, SUC_lexicon_tags_37 },
    { 0xd66938c7, SUC_lexicon_tags_29 },
    { 0xb4e938c8, SUC_lexicon_tags_38 },
    { 0x720938cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4aa938cd, SUC_lexicon_tags_40 },
    { 0xd9e938ce, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a938d4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826938d8, SUC_lexicon_tags_17 },
    { 0x61e938d9, SUC_lexicon_tags_49 },
    { 0xea4938d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0938e0, SUC_lexicon_tags_23 },
    { 0xe0c938e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xede938e4, SUC_lexicon_tags_37 },
    { 0x68938e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352938ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918938ee, SUC_lexicon_tags_28 },
    { 0xbd6938ef, SUC_lexicon_tags_7 },
    { 0x7cc938f0, SUC_lexicon_tags_63 },
    { 0x21e938ee, SUC_lexicon_tags_113 },
    { 0x514938ef, SUC_lexicon_tags_0 },
    { 0x10c938f2, SUC_lexicon_tags_82 },
    { 0x190938f4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70938f8, SUC_lexicon_tags_47 },
    { 0x716938f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec93901, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc9693903, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19093905, SUC_lexicon_tags_26 },
    { 0x72a93905, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d293909, SUC_lexicon_tags_1 },
    { 0xa09390a, SUC_lexicon_tags_106 },
    { 0x49390a, SUC_lexicon_tags_7 },
    { 0xe009390c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9969390f, SUC_lexicon_tags_12 },
    { 0xda09390f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2893912, SUC_lexicon_tags_25 },
    { 0xe6a93912, SUC_lexicon_tags_1 },
    { 0x91e93914, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94893919, SUC_lexicon_tags_11 },
    { 0xfac93919, SUC_lexicon_tags_30 },
    { 0x2f89391b, SUC_lexicon_tags_1 },
    { 0x7f69391c, SUC_lexicon_tags_40 },
    { 0xaf09391d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43093920, SUC_lexicon_tags_28 },
    { 0x4e093920, SUC_lexicon_tags_0 },
    { 0xd0093920, SUC_lexicon_tags_25 },
    { 0x49693921, SUC_lexicon_tags_11 },
    { 0x72293924, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4693927, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6729392d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b893930, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xdec93933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5093938, SUC_lexicon_tags_40 },
    { 0x1493938, SUC_lexicon_tags_32 },
    { 0xc49393a, SUC_lexicon_tags_61 },
    { 0xb629393b, SUC_lexicon_tags_6 },
    { 0x36e9393b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc93942, SUC_lexicon_tags_25 },
    { 0x12693943, SUC_lexicon_tags_1 },
    { 0xb1e93943, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38293947, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3093949, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3009394c, SUC_lexicon_tags_46 },
    { 0x4e9394d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79293953, SUC_lexicon_tags_104 },
    { 0xa7293954, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82093958, SUC_lexicon_tags_26 },
    { 0x2ac93959, SUC_lexicon_tags_9 },
    { 0x9e89395a, SUC_lexicon_tags_46 },
    { 0xce29395a, SUC_lexicon_tags_30 },
    { 0x1ae93959, SUC_lexicon_tags_49 },
    { 0x6f89395d, SUC_lexicon_tags_17 },
    { 0x1ea93958, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e93961, SUC_lexicon_tags_12 },
    { 0x79a93962, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55693964, SUC_lexicon_tags_37 },
    { 0x5a293965, SUC_lexicon_tags_32 },
    { 0x9e493966, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c9396a, SUC_lexicon_tags_15 },
    { 0xbd29396b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x569396d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x13493970, SUC_lexicon_tags_25 },
    { 0xcc093971, SUC_lexicon_tags_1 },
    { 0xcc693970, SUC_lexicon_tags_46 },
    { 0x57693973, SUC_lexicon_tags_23 },
    { 0xda293974, SUC_lexicon_tags_11 },
    { 0x4ca93972, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da9397a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a9397e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6da93980, SUC_lexicon_tags_27 },
    { 0x39693981, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb6493983, SUC_lexicon_tags_63 },
    { 0x14693983, SUC_lexicon_tags_26 },
    { 0xaf693985, SUC_lexicon_tags_13 },
    { 0xf9a93986, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4229398a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f093996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25493999, SUC_lexicon_tags_11 },
    { 0x84a9399a, SUC_lexicon_tags_49 },
    { 0x3ec9399b, SUC_lexicon_tags_2 },
    { 0x6909399c, SUC_lexicon_tags_31 },
    { 0xe709399c, SUC_lexicon_tags_33 },
    { 0xb529399e, SUC_lexicon_tags_45 },
    { 0x61e9399f, SUC_lexicon_tags_2 },
    { 0x1f69399f, SUC_lexicon_tags_15 },
    { 0x9149399a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x370939a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a939a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x946939a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6939aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x59e939ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb98939ae, SUC_lexicon_tags_17 },
    { 0xc3c939ae, SUC_lexicon_tags_18 },
    { 0x90a939ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf16939b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c939b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72939ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984939be, SUC_lexicon_tags_27 },
    { 0x3c6939be, SUC_lexicon_tags_25 },
    { 0xfe2939be, SUC_lexicon_tags_2 },
    { 0x6c2939c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0939c5, SUC_lexicon_tags_87 },
    { 0xed6939c5, SUC_lexicon_tags_32 },
    { 0xc6e939c6, SUC_lexicon_tags_15 },
    { 0xfe0939c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb08939ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc939d0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x262939d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0e939d4, SUC_lexicon_tags_23 },
    { 0x43a939d5, SUC_lexicon_tags_7 },
    { 0x2a2939d6, SUC_lexicon_tags_25 },
    { 0x338939d7, SUC_lexicon_tags_14 },
    { 0x8ea939d6, SUC_lexicon_tags_12 },
    { 0x864939d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b4939db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12939e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b4939e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee939ee, SUC_lexicon_tags_43 },
    { 0xc4a939ef, SUC_lexicon_tags_37 },
    { 0x26939ee, SUC_lexicon_tags_108 },
    { 0x954939f1, SUC_lexicon_tags_7 },
    { 0xf0a939f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94939f8, SUC_lexicon_tags_29 },
    { 0x562939f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00939fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a939ff, SUC_lexicon_tags_37 },
    { 0x37693a00, SUC_lexicon_tags_15 },
    { 0x74c93a01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3293a03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7893a06, SUC_lexicon_tags_15 },
    { 0xfe293a07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ae93a09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a93a0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5093a0f, SUC_lexicon_tags_12 },
    { 0x73c93a10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25693a16, SUC_lexicon_tags_6 },
    { 0x89e93a17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb7293a19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde693a1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb293a23, SUC_lexicon_tags_27 },
    { 0x3f093a24, SUC_lexicon_tags_3 },
    { 0xa8e93a23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e93a2b, SUC_lexicon_tags_15 },
    { 0x9f293a2c, SUC_lexicon_tags_2 },
    { 0xdbc93a2d, SUC_lexicon_tags_11 },
    { 0xde893a2e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b693a30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a93a35, SUC_lexicon_tags_15 },
    { 0x61293a35, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a93a3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa93a3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c93a40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce293a43, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c93a49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde893a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c93a4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e93a51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86093a55, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e93a59, SUC_lexicon_tags_25 },
    { 0xb2c93a5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb893a5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde293a61, SUC_lexicon_tags_2 },
    { 0x3ac93a62, SUC_lexicon_tags_30 },
    { 0x6c893a63, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a293a6a, SUC_lexicon_tags_37 },
    { 0x60093a6b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc9c93a6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0493a70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfc93a72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7693a76, SUC_lexicon_tags_47 },
    { 0xc6493a77, SUC_lexicon_tags_43 },
    { 0x80693a78, SUC_lexicon_tags_7 },
    { 0x39693a79, SUC_lexicon_tags_1 },
    { 0xa6c93a7a, SUC_lexicon_tags_17 },
    { 0x28e93a79, SUC_lexicon_tags_49 },
    { 0x81293a7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3293a81, SUC_lexicon_tags_5 },
    { 0xfc093a82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c93a85, SUC_lexicon_tags_0 },
    { 0xa7c93a86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec93a8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb693a8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2293a93, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6293a9f, SUC_lexicon_tags_33 },
    { 0xc2293a9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11893aa2, SUC_lexicon_tags_12 },
    { 0xbae93aa3, SUC_lexicon_tags_68 },
    { 0xcc493aa4, SUC_lexicon_tags_37 },
    { 0xb7693aa5, SUC_lexicon_tags_19 },
    { 0xc4e93aa6, SUC_lexicon_tags_11 },
    { 0x2be93aa6, SUC_lexicon_tags_12 },
    { 0xb8c93aa5, SUC_lexicon_tags_55 },
    { 0x52093aa6, SUC_lexicon_tags_33 },
    { 0xf1c93aaa, SUC_lexicon_tags_15 },
    { 0xaf093aab, SUC_lexicon_tags_26 },
    { 0x2b893aab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x71893aaf, SUC_lexicon_tags_11 },
    { 0x47893ab0, SUC_lexicon_tags_21 },
    { 0x27a93ab1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88c93ab3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35e93ab6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9693aba, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5893ac3, SUC_lexicon_tags_47 },
    { 0x52493ac4, SUC_lexicon_tags_0 },
    { 0xc6c93ac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14893ac7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x25893aca, SUC_lexicon_tags_1 },
    { 0x56293acb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c93ace, SUC_lexicon_tags_65 },
    { 0x3c693acf, SUC_lexicon_tags_17 },
    { 0x8fc93ad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9693ad5, SUC_lexicon_tags_87 },
    { 0x6ae93ad6, SUC_lexicon_tags_47 },
    { 0x24493ad5, SUC_lexicon_tags_13 },
    { 0xea493ad8, SUC_lexicon_tags_45 },
    { 0x2f293ad8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda93adf, SUC_lexicon_tags_9 },
    { 0x26093ae0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8e93ae2, SUC_lexicon_tags_43 },
    { 0xa93ae3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a93ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc93ae9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x43093aeb, SUC_lexicon_tags_7 },
    { 0x69293aec, SUC_lexicon_tags_25 },
    { 0xb4893aec, SUC_lexicon_tags_11 },
    { 0x19a93aee, SUC_lexicon_tags_0 },
    { 0x91893aef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9a93af1, SUC_lexicon_tags_25 },
    { 0x74293af1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6693af5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb8c93af7, SUC_lexicon_tags_12 },
    { 0xe4e93af8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a93afb, SUC_lexicon_tags_1 },
    { 0x31093afc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x49a93afe, SUC_lexicon_tags_25 },
    { 0xf6a93afe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89693b01, SUC_lexicon_tags_1 },
    { 0xeba93b01, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb6893b04, SUC_lexicon_tags_12 },
    { 0x1a893b05, SUC_lexicon_tags_19 },
    { 0xd1293b05, SUC_lexicon_tags_17 },
    { 0x47093b07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd8693b09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5093b14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a93b19, SUC_lexicon_tags_2 },
    { 0x1ea93b1a, SUC_lexicon_tags_17 },
    { 0xa2893b1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49a93b1d, SUC_lexicon_tags_17 },
    { 0x89693b1e, SUC_lexicon_tags_25 },
    { 0x81c93b1f, SUC_lexicon_tags_0 },
    { 0xf893b1d, SUC_lexicon_tags_43 },
    { 0xd7693b21, SUC_lexicon_tags_45 },
    { 0x27493b22, SUC_lexicon_tags_47 },
    { 0x1dc93b1f, SUC_lexicon_tags_47 },
    { 0x7a493b24, SUC_lexicon_tags_16 },
    { 0xe9493b23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc693b28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe093b30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc93b33, SUC_lexicon_tags_37 },
    { 0x5ae93b34, SUC_lexicon_tags_22 },
    { 0x7e693b33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee93b3d, SUC_lexicon_tags_37 },
    { 0x33e93b3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc9493b40, SUC_lexicon_tags_23 },
    { 0x46e93b41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72293b43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c93b45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea893b48, SUC_lexicon_tags_12 },
    { 0xcc293b48, SUC_lexicon_tags_19 },
    { 0x8a893b4a, SUC_lexicon_tags_37 },
    { 0x48c93b48, SUC_lexicon_tags_17 },
    { 0x71493b49, SUC_lexicon_tags_26 },
    { 0x14693b48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d693b4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1be93b51, SUC_lexicon_tags_16 },
    { 0x2c493b51, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xde893b54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a93b56, SUC_lexicon_tags_46 },
    { 0x93693b57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e93b5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fe93b5c, SUC_lexicon_tags_1 },
    { 0x55a93b5d, SUC_lexicon_tags_29 },
    { 0x1ae93b5e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a93b64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65293b66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74693b6c, SUC_lexicon_tags_12 },
    { 0xe8a93b6d, SUC_lexicon_tags_21 },
    { 0xc0293b6c, SUC_lexicon_tags_79 },
    { 0xcb293b6f, SUC_lexicon_tags_47 },
    { 0xa7893b70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1493b77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2493b79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9e93b7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1693b7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46293b7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e493b83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69293b85, SUC_lexicon_tags_1 },
    { 0xc4693b85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8093b88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79093b8c, SUC_lexicon_tags_98 },
    { 0x37693b8d, SUC_lexicon_tags_15 },
    { 0xf4893b8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdd893b90, SUC_lexicon_tags_6 },
    { 0xb4493b91, SUC_lexicon_tags_8 },
    { 0x46e93b92, SUC_lexicon_tags_43 },
    { 0x78293b91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0693b96, SUC_lexicon_tags_100 },
    { 0x90693b97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93a93b9a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0493ba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c293ba2, SUC_lexicon_tags_17 },
    { 0x62c93ba3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x81a93ba5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda093ba8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff693baa, SUC_lexicon_tags_106 },
    { 0x20e93bab, SUC_lexicon_tags_17 },
    { 0x8b093bab, SUC_lexicon_tags_6 },
    { 0x95293bac, SUC_lexicon_tags_14 },
    { 0xebc93bae, SUC_lexicon_tags_37 },
    { 0x20293bab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc293bb2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xad093bb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57493bb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79893bc0, SUC_lexicon_tags_25 },
    { 0x76e93bc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c693bcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd0c93bcf, SUC_lexicon_tags_26 },
    { 0x83a93bd0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc5093bd2, SUC_lexicon_tags_7 },
    { 0x4ea93bd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1893bd5, SUC_lexicon_tags_46 },
    { 0x91693bd6, SUC_lexicon_tags_68 },
    { 0xa3c93bd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8cc93bd9, SUC_lexicon_tags_12 },
    { 0x94a93bd9, SUC_lexicon_tags_47 },
    { 0x73093bdb, SUC_lexicon_tags_8 },
    { 0x28a93bdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5e93bde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc293be3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c93beb, SUC_lexicon_tags_0 },
    { 0xc9a93bec, SUC_lexicon_tags_17 },
    { 0x6c893bec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc93bf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a93bf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31293bf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xea493bfc, SUC_lexicon_tags_22 },
    { 0x87893bfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc493c01, SUC_lexicon_tags_84 },
    { 0xdaa93c02, SUC_lexicon_tags_12 },
    { 0x72c93c02, SUC_lexicon_tags_25 },
    { 0x1893c04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43493c07, SUC_lexicon_tags_43 },
    { 0x96693c07, SUC_lexicon_tags_25 },
    { 0x7d493c07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23693c0b, SUC_lexicon_tags_11 },
    { 0xdc893c0c, SUC_lexicon_tags_1 },
    { 0x3a093c0c, SUC_lexicon_tags_25 },
    { 0x1c493c0e, SUC_lexicon_tags_178 },
    { 0x91893c0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a93c12, SUC_lexicon_tags_8 },
    { 0x26c93c12, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc93c17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6693c1a, SUC_lexicon_tags_38 },
    { 0xa3e93c1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f693c1e, SUC_lexicon_tags_23 },
    { 0xd4a93c1f, SUC_lexicon_tags_7 },
    { 0x2a493c20, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8493c2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d893c2c, SUC_lexicon_tags_12 },
    { 0x56493c2d, SUC_lexicon_tags_1 },
    { 0x7ea93c2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a93c31, SUC_lexicon_tags_11 },
    { 0x9a893c32, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xed293c35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e93c3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60693c47, SUC_lexicon_tags_29 },
    { 0xae93c48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x98093c4a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb4093c4c, SUC_lexicon_tags_15 },
    { 0x5ea93c4d, SUC_lexicon_tags_0 },
    { 0x1aa93c4e, SUC_lexicon_tags_7 },
    { 0xfa293c4f, SUC_lexicon_tags_2 },
    { 0x9493c50, SUC_lexicon_tags_32 },
    { 0x6c893c51, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2493c55, SUC_lexicon_tags_7 },
    { 0x1a893c56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a93c5b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6893c61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70293c67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e93c6d, SUC_lexicon_tags_11 },
    { 0xd8493c6e, SUC_lexicon_tags_37 },
    { 0x55493c6d, SUC_lexicon_tags_29 },
    { 0xe1493c70, SUC_lexicon_tags_11 },
    { 0x7a93c71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60493c74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42493c79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0893c7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66493c80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0493c82, SUC_lexicon_tags_30 },
    { 0x86293c83, SUC_lexicon_tags_1 },
    { 0x6ec93c83, SUC_lexicon_tags_41 },
    { 0x38e93c83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e293c8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8893c91, SUC_lexicon_tags_25 },
    { 0xd7493c91, SUC_lexicon_tags_7 },
    { 0xbbc93c91, SUC_lexicon_tags_25 },
    { 0x3b293c94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab093c97, SUC_lexicon_tags_17 },
    { 0x96693c97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e293c9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c093c9c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x57493c9e, SUC_lexicon_tags_19 },
    { 0x42c93c9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d293ca1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac93ca5, SUC_lexicon_tags_1 },
    { 0x68893ca5, SUC_lexicon_tags_12 },
    { 0xd9a93ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7493ca9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea93caf, SUC_lexicon_tags_63 },
    { 0x5bc93cb0, SUC_lexicon_tags_2 },
    { 0xe493cb1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54693cb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27893cb8, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe93cbb, SUC_lexicon_tags_43 },
    { 0x27693cbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c93cbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a93cc8, SUC_lexicon_tags_16 },
    { 0x76c93cc9, SUC_lexicon_tags_16 },
    { 0x77e93cca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde93ccd, SUC_lexicon_tags_2 },
    { 0x41293ccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d093cd1, SUC_lexicon_tags_49 },
    { 0x2a893cd2, SUC_lexicon_tags_75 },
    { 0x95693cd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e93cda, SUC_lexicon_tags_11 },
    { 0x31493cdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac93ce0, SUC_lexicon_tags_2 },
    { 0x35093ce1, SUC_lexicon_tags_21 },
    { 0xf4293ce2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb493ce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f693ce9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19293ced, SUC_lexicon_tags_35 },
    { 0xe6693cee, SUC_lexicon_tags_214 },
    { 0x52e93cef, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x78a93cf1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc293cf4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a93cf7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c93cfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8d693cfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98893d03, SUC_lexicon_tags_25 },
    { 0xc1893d03, SUC_lexicon_tags_47 },
    { 0x44493d05, SUC_lexicon_tags_27 },
    { 0x8ea93d05, SUC_lexicon_tags_68 },
    { 0xb1693d06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22893d09, SUC_lexicon_tags_15 },
    { 0x62a93d09, SUC_lexicon_tags_7 },
    { 0x17693d0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7f693d0d, SUC_lexicon_tags_1 },
    { 0x7ec93d0d, SUC_lexicon_tags_14 },
    { 0x67693d0f, SUC_lexicon_tags_28 },
    { 0x98c93d0f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd093d13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc093d15, SUC_lexicon_tags_37 },
    { 0x25c93d16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45693d19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75293d1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e93d1f, SUC_lexicon_tags_37 },
    { 0x6d093d20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e93d24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa293d29, SUC_lexicon_tags_63 },
    { 0xe6893d2a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf093d2d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2e293d2f, SUC_lexicon_tags_1 },
    { 0x17a93d30, SUC_lexicon_tags_15 },
    { 0x1a893d31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7c93d33, SUC_lexicon_tags_38 },
    { 0xf2693d34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaf093d36, SUC_lexicon_tags_37 },
    { 0xe1893d37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda93d3b, SUC_lexicon_tags_11 },
    { 0xa1293d3c, SUC_lexicon_tags_26 },
    { 0x11893d3d, SUC_lexicon_tags_25 },
    { 0x5e093d3d, SUC_lexicon_tags_1 },
    { 0x9d093d3d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x57893d42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a93d45, SUC_lexicon_tags_43 },
    { 0x82293d45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64893d51, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce93d54, SUC_lexicon_tags_118 },
    { 0xf7e93d55, SUC_lexicon_tags_14 },
    { 0x54a93d56, SUC_lexicon_tags_33 },
    { 0x26293d55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92893d59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a93d5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc93d5e, SUC_lexicon_tags_11 },
    { 0x6ca93d5f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9693d62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac93d65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8693d6b, SUC_lexicon_tags_17 },
    { 0xfd293d6c, SUC_lexicon_tags_136 },
    { 0xeae93d6c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x64493d6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb693d71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x56a93d73, SUC_lexicon_tags_11 },
    { 0xa7293d74, SUC_lexicon_tags_17 },
    { 0xefc93d75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20a93d77, SUC_lexicon_tags_46 },
    { 0x1dc93d77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce93d81, SUC_lexicon_tags_32 },
    { 0x80693d81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c693d84, SUC_lexicon_tags_84 },
    { 0x2f093d85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a693d8a, SUC_lexicon_tags_33 },
    { 0xf3a93d8b, SUC_lexicon_tags_11 },
    { 0x2ce93d8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2093d93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a093d95, SUC_lexicon_tags_15 },
    { 0xfc093d95, SUC_lexicon_tags_28 },
    { 0x9093d97, SUC_lexicon_tags_23 },
    { 0x8d893d98, SUC_lexicon_tags_17 },
    { 0xabc93d98, SUC_lexicon_tags_7 },
    { 0xc5093d9a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea93d9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c093da1, SUC_lexicon_tags_8 },
    { 0x41893da1, SUC_lexicon_tags_68 },
    { 0x1f293da3, SUC_lexicon_tags_39 },
    { 0xbfa93da3, SUC_lexicon_tags_12 },
    { 0xe5093da3, SUC_lexicon_tags_2 },
    { 0x6aa93da6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc893da9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4493dac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x87093dae, SUC_lexicon_tags_49 },
    { 0xe4e93daf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9893db6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58493dba, SUC_lexicon_tags_14 },
    { 0x64893dbb, SUC_lexicon_tags_11 },
    { 0x14e93dbc, SUC_lexicon_tags_12 },
    { 0x19693dbc, SUC_lexicon_tags_39 },
    { 0xc0e93dbd, SUC_lexicon_tags_0 },
    { 0xef293dbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b893dc2, SUC_lexicon_tags_9 },
    { 0x3d093dc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b493dc5, SUC_lexicon_tags_11 },
    { 0xff693dc5, SUC_lexicon_tags_37 },
    { 0x70c93dc5, SUC_lexicon_tags_45 },
    { 0x12493dc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23293dcb, SUC_lexicon_tags_47 },
    { 0x54893dcc, SUC_lexicon_tags_15 },
    { 0xb0493dcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2693dd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90c93dd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e493dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37093ddc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b893ddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51e93de1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c93de7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe293dec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33093dee, SUC_lexicon_tags_11 },
    { 0x2e893dee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb093df1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21293df9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36093dfb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x70093dfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa093e00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0693e0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51693e14, SUC_lexicon_tags_11 },
    { 0x88693e14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae93e1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd493e1c, SUC_lexicon_tags_31 },
    { 0x72693e1c, SUC_lexicon_tags_15 },
    { 0xcdc93e1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31093e21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7a93e23, SUC_lexicon_tags_31 },
    { 0xbba93e23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56493e2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x71493e2e, SUC_lexicon_tags_17 },
    { 0x1f493e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3493e32, SUC_lexicon_tags_32 },
    { 0xec93e33, SUC_lexicon_tags_47 },
    { 0x63693e33, SUC_lexicon_tags_17 },
    { 0x1a93e32, SUC_lexicon_tags_17 },
    { 0xd2693e34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c293e39, SUC_lexicon_tags_4 },
    { 0x77693e39, SUC_lexicon_tags_0 },
    { 0x28e93e3b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x82c93e3d, SUC_lexicon_tags_7 },
    { 0x3d893e3e, SUC_lexicon_tags_66 },
    { 0x1ac93e3e, SUC_lexicon_tags_15 },
    { 0xaaa93e3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac93e43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e93e48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x22893e4b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c93e4e, SUC_lexicon_tags_47 },
    { 0x78e93e4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca93e52, SUC_lexicon_tags_31 },
    { 0x9c93e53, SUC_lexicon_tags_54 },
    { 0x54893e54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a93e5a, SUC_lexicon_tags_8 },
    { 0xbe93e5b, SUC_lexicon_tags_27 },
    { 0xb8693e5a, SUC_lexicon_tags_49 },
    { 0xc1e93e5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc493e61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb293e6a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xec493e6d, SUC_lexicon_tags_37 },
    { 0x1e93e6e, SUC_lexicon_tags_1 },
    { 0xa0c93e6f, SUC_lexicon_tags_7 },
    { 0x76e93e70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba93e74, SUC_lexicon_tags_2 },
    { 0xd6493e75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c93e79, SUC_lexicon_tags_17 },
    { 0xc7a93e7a, SUC_lexicon_tags_113 },
    { 0xd9e93e7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbec93e7d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc293e83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x48493e85, SUC_lexicon_tags_25 },
    { 0x14e93e86, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec93e8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61693e95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34293e99, SUC_lexicon_tags_13 },
    { 0x73693e9a, SUC_lexicon_tags_37 },
    { 0x36693e9b, SUC_lexicon_tags_27 },
    { 0xf293e9c, SUC_lexicon_tags_21 },
    { 0x26693e9d, SUC_lexicon_tags_6 },
    { 0x75093e99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8f493ea0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d293ea2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6a693ea4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c93ea8, SUC_lexicon_tags_39 },
    { 0xd0493ea9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5693eab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xca893ead, SUC_lexicon_tags_45 },
    { 0x65e93eae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec93eb2, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49293eb6, SUC_lexicon_tags_19 },
    { 0x83693eb6, SUC_lexicon_tags_17 },
    { 0x35893eb8, SUC_lexicon_tags_7 },
    { 0x70e93eb9, SUC_lexicon_tags_12 },
    { 0x48693eb8, SUC_lexicon_tags_15 },
    { 0x41c93ebb, SUC_lexicon_tags_87 },
    { 0x5c093ebc, SUC_lexicon_tags_12 },
    { 0x45093eba, SUC_lexicon_tags_17 },
    { 0x19693eb8, SUC_lexicon_tags_43 },
    { 0xeee93ebf, SUC_lexicon_tags_19 },
    { 0xf1493ebf, SUC_lexicon_tags_31 },
    { 0x28493eb8, SUC_lexicon_tags_0 },
    { 0x5c093eb9, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa93ec5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x67493ec7, SUC_lexicon_tags_14 },
    { 0xfdc93ec8, SUC_lexicon_tags_1 },
    { 0x87493ec9, SUC_lexicon_tags_11 },
    { 0x7c93eca, SUC_lexicon_tags_0 },
    { 0x46493eca, SUC_lexicon_tags_17 },
    { 0xb2093ecc, SUC_lexicon_tags_26 },
    { 0xe293ecd, SUC_lexicon_tags_56 },
    { 0xcca93ece, SUC_lexicon_tags_38 },
    { 0xbe293ecf, SUC_lexicon_tags_0 },
    { 0x4e093ece, SUC_lexicon_tags_27 },
    { 0xc6c93ece, SUC_lexicon_tags_70 },
    { 0xed293ecf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6493ed7, SUC_lexicon_tags_7 },
    { 0x79e93ed7, SUC_lexicon_tags_9 },
    { 0xf0893ed9, SUC_lexicon_tags_25 },
    { 0x8c293eda, SUC_lexicon_tags_2 },
    { 0x57c93edb, SUC_lexicon_tags_47 },
    { 0xbd693eda, SUC_lexicon_tags_12 },
    { 0x4f493ed7, SUC_lexicon_tags_13 },
    { 0xc6893ed9, SUC_lexicon_tags_45 },
    { 0x1f093edb, SUC_lexicon_tags_0 },
    { 0x8ee93ee0, SUC_lexicon_tags_37 },
    { 0xe8293ee0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36093ee6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22493eeb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x41e93eed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21493eef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x22893ef1, SUC_lexicon_tags_25 },
    { 0xb6693ef2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2e93ef4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0293ef7, SUC_lexicon_tags_11 },
    { 0xc4693ef8, SUC_lexicon_tags_56 },
    { 0x3a93ef9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x58293efc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba493efe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x84493f00, SUC_lexicon_tags_14 },
    { 0xb4693f01, SUC_lexicon_tags_47 },
    { 0x23e93f02, SUC_lexicon_tags_84 },
    { 0x14293f03, SUC_lexicon_tags_11 },
    { 0xe1493f02, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8293f08, SUC_lexicon_tags_11 },
    { 0x1fa93f09, SUC_lexicon_tags_11 },
    { 0xde093f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc93f0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c93f15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5093f18, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5093f1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18693f27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8493f2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87c93f2c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e93f32, SUC_lexicon_tags_33 },
    { 0xbe693f33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb6093f35, SUC_lexicon_tags_25 },
    { 0x74293f36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a93f39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c93f3d, SUC_lexicon_tags_15 },
    { 0x25a93f3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c293f45, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e93f4b, SUC_lexicon_tags_12 },
    { 0x9c693f4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59e93f4e, SUC_lexicon_tags_18 },
    { 0xcca93f4f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcd693f51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d493f5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe3c93f60, SUC_lexicon_tags_47 },
    { 0x8b093f61, SUC_lexicon_tags_55 },
    { 0x70c93f60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3f093f64, SUC_lexicon_tags_77 },
    { 0x54e93f65, SUC_lexicon_tags_49 },
    { 0x5c493f66, SUC_lexicon_tags_34 },
    { 0x66893f67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ba93f69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47293f6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99493f75, SUC_lexicon_tags_87 },
    { 0xec293f75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13693f7b, SUC_lexicon_tags_28 },
    { 0xe0c93f7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x97493f7f, SUC_lexicon_tags_7 },
    { 0x18a93f80, SUC_lexicon_tags_27 },
    { 0x76293f7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x43093f84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c93f87, SUC_lexicon_tags_45 },
    { 0xeb293f88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c093f8e, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f693f96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3893f98, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a93f9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce93fa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90693fa5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6493faa, SUC_lexicon_tags_198 },
    { 0x3ae93fab, SUC_lexicon_tags_21 },
    { 0x4e693faa, SUC_lexicon_tags_2 },
    { 0xf2493fad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff093faf, SUC_lexicon_tags_0 },
    { 0x1de93faf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74093fb2, SUC_lexicon_tags_23 },
    { 0x85a93fb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7293fb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4493fb9, SUC_lexicon_tags_37 },
    { 0xade93fba, SUC_lexicon_tags_12 },
    { 0xd9693fba, SUC_lexicon_tags_2 },
    { 0x2fc93fbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97093fbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c93fc4, SUC_lexicon_tags_17 },
    { 0x61293fc5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x98093fc7, SUC_lexicon_tags_15 },
    { 0xfc893fc7, SUC_lexicon_tags_45 },
    { 0xb093fc8, SUC_lexicon_tags_17 },
    { 0x3ea93fca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee93fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63893fd9, SUC_lexicon_tags_19 },
    { 0xf0093fda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae93fdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38e93fdf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xaa93fe1, SUC_lexicon_tags_15 },
    { 0xb3c93fe1, SUC_lexicon_tags_25 },
    { 0xae93fe1, SUC_lexicon_tags_31 },
    { 0x75093fe2, SUC_lexicon_tags_49 },
    { 0x63693fe5, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x44293fe7, SUC_lexicon_tags_87 },
    { 0x10893fe8, SUC_lexicon_tags_10 },
    { 0xcca93fe9, SUC_lexicon_tags_34 },
    { 0x8f693fea, SUC_lexicon_tags_7 },
    { 0x3c493feb, SUC_lexicon_tags_25 },
    { 0xc6c93fea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42093ff0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa893ff4, SUC_lexicon_tags_104 },
    { 0x66493ff5, SUC_lexicon_tags_12 },
    { 0xf3893ff6, SUC_lexicon_tags_2 },
    { 0xcf893ff7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19e93ff9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea94000, SUC_lexicon_tags_15 },
    { 0x7de94001, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x81e94003, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf9294005, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c9400a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc269400c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99294011, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1294017, SUC_lexicon_tags_38 },
    { 0xdbc94017, SUC_lexicon_tags_66 },
    { 0x31e94019, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5109401b, SUC_lexicon_tags_9 },
    { 0x2209401c, SUC_lexicon_tags_28 },
    { 0x6849401d, SUC_lexicon_tags_26 },
    { 0xfc69401e, SUC_lexicon_tags_25 },
    { 0xf389401d, SUC_lexicon_tags_37 },
    { 0xaba94020, SUC_lexicon_tags_14 },
    { 0xa6e9401f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac294027, SUC_lexicon_tags_33 },
    { 0xabe94028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89402c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9049402f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a94033, SUC_lexicon_tags_140 },
    { 0xd0094034, SUC_lexicon_tags_2 },
    { 0x18094035, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x63e94037, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe069403d, SUC_lexicon_tags_2 },
    { 0xaae9403e, SUC_lexicon_tags_17 },
    { 0x69e9403f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c94045, SUC_lexicon_tags_11 },
    { 0x69e94046, SUC_lexicon_tags_43 },
    { 0xd1a94047, SUC_lexicon_tags_31 },
    { 0x36894048, SUC_lexicon_tags_43 },
    { 0x20a94049, SUC_lexicon_tags_25 },
    { 0xe8094047, SUC_lexicon_tags_2 },
    { 0x9ce9404b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa709404d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71294051, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce494055, SUC_lexicon_tags_14 },
    { 0x1be94056, SUC_lexicon_tags_33 },
    { 0x25894057, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e9405e, SUC_lexicon_tags_67 },
    { 0x1689405f, SUC_lexicon_tags_63 },
    { 0x2c94060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92494064, SUC_lexicon_tags_38 },
    { 0xcbe94065, SUC_lexicon_tags_1 },
    { 0xa6894065, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a9406a, SUC_lexicon_tags_26 },
    { 0x1169406b, SUC_lexicon_tags_37 },
    { 0x3109406a, SUC_lexicon_tags_99 },
    { 0xa769406c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f09406f, SUC_lexicon_tags_12 },
    { 0x6969406f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e94076, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76094078, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2749407a, SUC_lexicon_tags_37 },
    { 0xa2e9407b, SUC_lexicon_tags_53 },
    { 0x9a49407c, SUC_lexicon_tags_38 },
    { 0xe09407d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e94084, SUC_lexicon_tags_11 },
    { 0xd7a94085, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a9408d, SUC_lexicon_tags_12 },
    { 0x6829408e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x17294090, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c94094, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea94098, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf309409c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3249409e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xefc940a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68940a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe52940a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2940a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7e0940ab, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa940ae, SUC_lexicon_tags_16 },
    { 0xfd4940ae, SUC_lexicon_tags_49 },
    { 0xb04940b0, SUC_lexicon_tags_79 },
    { 0x3e0940b1, SUC_lexicon_tags_27 },
    { 0xb3e940b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce940bc, SUC_lexicon_tags_37 },
    { 0x648940bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e940c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa8940c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x286940c7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9a4940c9, SUC_lexicon_tags_14 },
    { 0x6d8940c9, SUC_lexicon_tags_27 },
    { 0x348940ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e940ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74940d1, SUC_lexicon_tags_146 },
    { 0x7da940d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6940de, SUC_lexicon_tags_12 },
    { 0xed4940de, SUC_lexicon_tags_0 },
    { 0xc5c940e0, SUC_lexicon_tags_37 },
    { 0x448940e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746940e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd18940e7, SUC_lexicon_tags_19 },
    { 0x52c940e7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee940ec, SUC_lexicon_tags_62 },
    { 0x8dc940ec, SUC_lexicon_tags_1 },
    { 0xb8e940ee, SUC_lexicon_tags_25 },
    { 0x230940ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc940f3, SUC_lexicon_tags_12 },
    { 0xf1e940f4, SUC_lexicon_tags_17 },
    { 0xf90940f5, SUC_lexicon_tags_7 },
    { 0x108940f6, SUC_lexicon_tags_27 },
    { 0x910940f7, SUC_lexicon_tags_27 },
    { 0xeca940f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x700940fa, SUC_lexicon_tags_33 },
    { 0x6ec940fa, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0x43c940fd, SUC_lexicon_tags_68 },
    { 0x118940fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbfa94100, SUC_lexicon_tags_12 },
    { 0xea294100, SUC_lexicon_tags_60 },
    { 0xad094102, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5589410a, SUC_lexicon_tags_1 },
    { 0x25e9410b, SUC_lexicon_tags_2 },
    { 0xd789410c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d494110, SUC_lexicon_tags_12 },
    { 0x5f094110, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e694113, SUC_lexicon_tags_2 },
    { 0xb4494113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb494116, SUC_lexicon_tags_17 },
    { 0x51a94116, SUC_lexicon_tags_37 },
    { 0x1c94116, SUC_lexicon_tags_46 },
    { 0x52294119, SUC_lexicon_tags_47 },
    { 0x9be94119, SUC_lexicon_tags_46 },
    { 0xd389411b, SUC_lexicon_tags_25 },
    { 0x8e09411b, SUC_lexicon_tags_15 },
    { 0x269411c, SUC_lexicon_tags_35 },
    { 0xe4a9411b, SUC_lexicon_tags_17 },
    { 0x7d89411e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1894125, SUC_lexicon_tags_12 },
    { 0x99694126, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e9412a, SUC_lexicon_tags_65 },
    { 0x8c69412b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55694138, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xac49413a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea94145, SUC_lexicon_tags_19 },
    { 0x85c94146, SUC_lexicon_tags_56 },
    { 0xac694147, SUC_lexicon_tags_17 },
    { 0x64c94148, SUC_lexicon_tags_25 },
    { 0x74a94149, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e9414f, SUC_lexicon_tags_7 },
    { 0xb6a94150, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41694155, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0294159, SUC_lexicon_tags_53 },
    { 0xa3c9415a, SUC_lexicon_tags_63 },
    { 0xdea9415b, SUC_lexicon_tags_7 },
    { 0xb889415b, SUC_lexicon_tags_15 },
    { 0x1b29415c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9094165, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58694168, SUC_lexicon_tags_45 },
    { 0x88294168, SUC_lexicon_tags_26 },
    { 0x6c09416a, SUC_lexicon_tags_14 },
    { 0xb949416b, SUC_lexicon_tags_31 },
    { 0x5349416c, SUC_lexicon_tags_47 },
    { 0x7149416c, SUC_lexicon_tags_55 },
    { 0x1e69416e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a894172, SUC_lexicon_tags_37 },
    { 0xfa894173, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19094179, SUC_lexicon_tags_7 },
    { 0x81494179, SUC_lexicon_tags_62 },
    { 0x8429417b, SUC_lexicon_tags_17 },
    { 0xc3a9417b, SUC_lexicon_tags_47 },
    { 0xb829417d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4b09417f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e94183, SUC_lexicon_tags_11 },
    { 0xb6694184, SUC_lexicon_tags_29 },
    { 0x3d294184, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9909418a, SUC_lexicon_tags_2 },
    { 0xd589418a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b09418f, SUC_lexicon_tags_1 },
    { 0x3a49418f, SUC_lexicon_tags_25 },
    { 0xfb094191, SUC_lexicon_tags_2 },
    { 0x30694192, SUC_lexicon_tags_1 },
    { 0xf6494190, SUC_lexicon_tags_38 },
    { 0x7ba94194, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3009419c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c941a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x306941a6, SUC_lexicon_tags_1 },
    { 0xc76941a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36941ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2a941ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0941b1, SUC_lexicon_tags_63 },
    { 0x7ec941b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50941ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc50941bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e941c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e941ca, SUC_lexicon_tags_14 },
    { 0xc4a941ca, SUC_lexicon_tags_15 },
    { 0x776941ca, SUC_lexicon_tags_23 },
    { 0xe56941cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e941d0, SUC_lexicon_tags_37 },
    { 0xdb6941d1, SUC_lexicon_tags_6 },
    { 0x24c941d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8941d6, SUC_lexicon_tags_39 },
    { 0xabe941d7, SUC_lexicon_tags_33 },
    { 0x9f8941d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfba941db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6941de, SUC_lexicon_tags_6 },
    { 0x78941df, SUC_lexicon_tags_15 },
    { 0xec941e0, SUC_lexicon_tags_7 },
    { 0xe42941e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2941e4, SUC_lexicon_tags_2 },
    { 0x452941e4, SUC_lexicon_tags_12 },
    { 0x2b8941e6, SUC_lexicon_tags_108 },
    { 0x8de941e7, SUC_lexicon_tags_63 },
    { 0xb90941e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722941f5, SUC_lexicon_tags_38 },
    { 0x962941f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a941fa, SUC_lexicon_tags_1 },
    { 0x498941fa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x306941fd, SUC_lexicon_tags_29 },
    { 0x9a6941fe, SUC_lexicon_tags_60 },
    { 0x7cc941ff, SUC_lexicon_tags_60 },
    { 0xd56941ff, SUC_lexicon_tags_17 },
    { 0xfc094201, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b294205, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e94209, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea9420d, SUC_lexicon_tags_29 },
    { 0x8a69420e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab494213, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb094218, SUC_lexicon_tags_25 },
    { 0x84294219, SUC_lexicon_tags_66 },
    { 0xd269421a, SUC_lexicon_tags_2 },
    { 0x7ac9421b, SUC_lexicon_tags_22 },
    { 0x5ac9421c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61094225, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e094233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0894237, SUC_lexicon_tags_23 },
    { 0xfe894237, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3469423a, SUC_lexicon_tags_55 },
    { 0xd6c9423a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef09423f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e94243, SUC_lexicon_tags_39 },
    { 0xf9494244, SUC_lexicon_tags_9 },
    { 0x33a94245, SUC_lexicon_tags_87 },
    { 0x5a494243, SUC_lexicon_tags_25 },
    { 0x7da94247, SUC_lexicon_tags_38 },
    { 0xdb494247, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a9424e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91e94252, SUC_lexicon_tags_34 },
    { 0xe6694253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e9425a, SUC_lexicon_tags_38 },
    { 0xe3e9425b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a69425f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6dc94261, SUC_lexicon_tags_37 },
    { 0xb9294262, SUC_lexicon_tags_12 },
    { 0xa5894263, SUC_lexicon_tags_65 },
    { 0x9c94263, SUC_lexicon_tags_26 },
    { 0x84a94265, SUC_lexicon_tags_26 },
    { 0xde094266, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdd494268, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaec9426a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab49426c, SUC_lexicon_tags_19 },
    { 0x9289426d, SUC_lexicon_tags_1 },
    { 0xa69426e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55294270, SUC_lexicon_tags_47 },
    { 0x48294270, SUC_lexicon_tags_14 },
    { 0x18a94272, SUC_lexicon_tags_11 },
    { 0x49094273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96894276, SUC_lexicon_tags_7 },
    { 0x84894276, SUC_lexicon_tags_37 },
    { 0x96894278, SUC_lexicon_tags_46 },
    { 0xc8294278, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7709427b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea9427f, SUC_lexicon_tags_37 },
    { 0xc5294280, SUC_lexicon_tags_2 },
    { 0xd0a94280, SUC_lexicon_tags_6 },
    { 0xe909427f, SUC_lexicon_tags_60 },
    { 0xcfe94281, SUC_lexicon_tags_17 },
    { 0x89c94283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94294287, SUC_lexicon_tags_1 },
    { 0xb5894287, SUC_lexicon_tags_15 },
    { 0xef894289, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9428e, SUC_lexicon_tags_14 },
    { 0x1aa9428f, SUC_lexicon_tags_31 },
    { 0xa0494290, SUC_lexicon_tags_11 },
    { 0xb2a94290, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3dc94293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d694295, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e9429a, SUC_lexicon_tags_15 },
    { 0x1389429b, SUC_lexicon_tags_31 },
    { 0x8189429b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c9429f, SUC_lexicon_tags_79 },
    { 0xe70942a0, SUC_lexicon_tags_11 },
    { 0xb00942a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364942a7, SUC_lexicon_tags_27 },
    { 0xeb8942a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a942ac, SUC_lexicon_tags_25 },
    { 0xd7c942ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6942b0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcbc942b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30942b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c942b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e942bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4942c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae942c3, SUC_lexicon_tags_63 },
    { 0xa00942c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994942ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf50942cc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66942d2, SUC_lexicon_tags_13 },
    { 0xdbe942d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e942da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06942e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2942e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x546942e6, SUC_lexicon_tags_12 },
    { 0x40942e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x788942ea, SUC_lexicon_tags_11 },
    { 0x4e4942eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8942ee, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40942f6, SUC_lexicon_tags_61 },
    { 0xa3e942f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3a942f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb2942fb, SUC_lexicon_tags_11 },
    { 0x384942fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0942ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a94304, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffa94308, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c9430b, SUC_lexicon_tags_10 },
    { 0xb7c9430c, SUC_lexicon_tags_12 },
    { 0x3329430d, SUC_lexicon_tags_39 },
    { 0x5249430e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29094313, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x28494316, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x51094319, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe369431b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa0c9431d, SUC_lexicon_tags_79 },
    { 0xcd29431d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8c94320, SUC_lexicon_tags_14 },
    { 0xf9294321, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a94325, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x73c94328, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a9432b, SUC_lexicon_tags_48 },
    { 0x8ae9432c, SUC_lexicon_tags_15 },
    { 0xd129432d, SUC_lexicon_tags_66 },
    { 0x9289432e, SUC_lexicon_tags_2 },
    { 0x1789432f, SUC_lexicon_tags_33 },
    { 0xed09432d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x30494332, SUC_lexicon_tags_22 },
    { 0xfa494333, SUC_lexicon_tags_15 },
    { 0xc7694334, SUC_lexicon_tags_29 },
    { 0xe5694332, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6494337, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf894339, SUC_lexicon_tags_8 },
    { 0xc729433a, SUC_lexicon_tags_8 },
    { 0x7e89433b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f294341, SUC_lexicon_tags_107 },
    { 0xbfc94341, SUC_lexicon_tags_62 },
    { 0x73294343, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x92c94345, SUC_lexicon_tags_15 },
    { 0x6bc94346, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a9434c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32694352, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29435a, SUC_lexicon_tags_11 },
    { 0xd8e9435b, SUC_lexicon_tags_37 },
    { 0xdca9435b, SUC_lexicon_tags_23 },
    { 0x5729435d, SUC_lexicon_tags_26 },
    { 0x8dc9435d, SUC_lexicon_tags_31 },
    { 0xcc69435f, SUC_lexicon_tags_25 },
    { 0x7469435d, SUC_lexicon_tags_46 },
    { 0xadc94361, SUC_lexicon_tags_46 },
    { 0x41a94362, SUC_lexicon_tags_0 },
    { 0xdcc9435a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7694365, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec94368, SUC_lexicon_tags_2 },
    { 0xc5e94369, SUC_lexicon_tags_38 },
    { 0x1049436a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e94371, SUC_lexicon_tags_29 },
    { 0xafa94372, SUC_lexicon_tags_43 },
    { 0xb1894373, SUC_lexicon_tags_27 },
    { 0x93694374, SUC_lexicon_tags_943 },
    { 0x8fc94375, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x74494378, SUC_lexicon_tags_17 },
    { 0xc0694378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe549437c, SUC_lexicon_tags_30 },
    { 0xda89437c, SUC_lexicon_tags_37 },
    { 0x8f29437c, SUC_lexicon_tags_57 },
    { 0xa9c9437f, SUC_lexicon_tags_43 },
    { 0x3e494380, SUC_lexicon_tags_8 },
    { 0xd7e94381, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf0894383, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e94388, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4149438a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec94397, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe9439b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e943a7, SUC_lexicon_tags_25 },
    { 0xa6c943a7, SUC_lexicon_tags_10 },
    { 0xabe943a9, SUC_lexicon_tags_91 },
    { 0x4a0943aa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54943af, SUC_lexicon_tags_7 },
    { 0x79c943b0, SUC_lexicon_tags_60 },
    { 0x2b0943b0, SUC_lexicon_tags_40 },
    { 0xc8a943b1, SUC_lexicon_tags_4 },
    { 0x616943b3, SUC_lexicon_tags_37 },
    { 0xb24943b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9b6943b6, SUC_lexicon_tags_12 },
    { 0x42943b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2943ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a943c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0943c6, SUC_lexicon_tags_38 },
    { 0xb44943c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ba943c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7ee943cb, SUC_lexicon_tags_2 },
    { 0x2f2943cc, SUC_lexicon_tags_33 },
    { 0xc84943cd, SUC_lexicon_tags_1 },
    { 0xe84943cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbc943d0, SUC_lexicon_tags_1 },
    { 0xae8943d0, SUC_lexicon_tags_87 },
    { 0xa8a943d2, SUC_lexicon_tags_178 },
    { 0x5aa943d3, SUC_lexicon_tags_2 },
    { 0xb2943d1, SUC_lexicon_tags_15 },
    { 0x896943d5, SUC_lexicon_tags_10 },
    { 0xb58943d5, SUC_lexicon_tags_43 },
    { 0xa5a943d7, SUC_lexicon_tags_63 },
    { 0x798943d8, SUC_lexicon_tags_17 },
    { 0x91a943d9, SUC_lexicon_tags_31 },
    { 0x58e943d8, SUC_lexicon_tags_17 },
    { 0x574943da, SUC_lexicon_tags_37 },
    { 0x978943d8, SUC_lexicon_tags_8 },
    { 0x780943d8, SUC_lexicon_tags_387 },
    { 0xdce943de, SUC_lexicon_tags_6 },
    { 0x464943de, SUC_lexicon_tags_12 },
    { 0xa06943df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0943e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a943e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c943eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x402943ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c943f3, SUC_lexicon_tags_14 },
    { 0xc4a943f4, SUC_lexicon_tags_27 },
    { 0xf4a943f5, SUC_lexicon_tags_33 },
    { 0x594943f6, SUC_lexicon_tags_12 },
    { 0x176943f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e943fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778943fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5894402, SUC_lexicon_tags_23 },
    { 0x6a694403, SUC_lexicon_tags_2 },
    { 0xafc94404, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d29440a, SUC_lexicon_tags_35 },
    { 0x8509440a, SUC_lexicon_tags_65 },
    { 0xa069440c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7849440f, SUC_lexicon_tags_15 },
    { 0xfc294410, SUC_lexicon_tags_9 },
    { 0xb3094410, SUC_lexicon_tags_15 },
    { 0x3094411, SUC_lexicon_tags_27 },
    { 0xb549440f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c94416, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfae94418, SUC_lexicon_tags_15 },
    { 0x2ce94418, SUC_lexicon_tags_38 },
    { 0x54894418, SUC_lexicon_tags_5 },
    { 0x6809441b, SUC_lexicon_tags_37 },
    { 0x49c9441c, SUC_lexicon_tags_38 },
    { 0x3269441d, SUC_lexicon_tags_15 },
    { 0xe5a9441d, SUC_lexicon_tags_85 },
    { 0x4f49441f, SUC_lexicon_tags_38 },
    { 0x27494418, SUC_lexicon_tags_17 },
    { 0x6c49441d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a94426, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda894428, SUC_lexicon_tags_433 },
    { 0, NULL },
    { 0x67c9442a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8329442c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec94430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b294432, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65894439, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4949443c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1189443f, SUC_lexicon_tags_92 },
    { 0xf6894440, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe494442, SUC_lexicon_tags_7 },
    { 0x57494443, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce9444f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a94452, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e94458, SUC_lexicon_tags_12 },
    { 0xc8294459, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a94461, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce94467, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c9446b, SUC_lexicon_tags_15 },
    { 0x7f69446c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xf909446f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18694472, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x44494475, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca94478, SUC_lexicon_tags_77 },
    { 0x8d894479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa9447c, SUC_lexicon_tags_34 },
    { 0x1269447c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc529447f, SUC_lexicon_tags_25 },
    { 0x28494480, SUC_lexicon_tags_49 },
    { 0x45694481, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9094484, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa94488, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9069448a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29448d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32894493, SUC_lexicon_tags_900 },
    { 0x57894494, SUC_lexicon_tags_11 },
    { 0xc5e94493, SUC_lexicon_tags_47 },
    { 0x3ec94496, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6294499, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab89449c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3249449e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572944a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872944a8, SUC_lexicon_tags_7 },
    { 0x630944a9, SUC_lexicon_tags_15 },
    { 0x3b6944aa, SUC_lexicon_tags_27 },
    { 0x98a944ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da944b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0944b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88944b8, SUC_lexicon_tags_68 },
    { 0x27c944b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46944bd, SUC_lexicon_tags_12 },
    { 0x620944be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3e8944c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76944ca, SUC_lexicon_tags_25 },
    { 0x622944ca, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xafc944cd, SUC_lexicon_tags_7 },
    { 0x90c944cd, SUC_lexicon_tags_35 },
    { 0x5da944cf, SUC_lexicon_tags_56 },
    { 0xcbc944cf, SUC_lexicon_tags_68 },
    { 0x398944cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e2944d3, SUC_lexicon_tags_17 },
    { 0x7a4944d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592944db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8944e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x804944e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4944e8, SUC_lexicon_tags_14 },
    { 0x580944e9, SUC_lexicon_tags_683 },
    { 0xf1e944ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90944ee, SUC_lexicon_tags_87 },
    { 0xaa0944ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104944f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140944fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6094500, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xd0e94502, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f894506, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e9450c, SUC_lexicon_tags_45 },
    { 0xc9450c, SUC_lexicon_tags_30 },
    { 0xc889450e, SUC_lexicon_tags_0 },
    { 0x4c09450f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec94516, SUC_lexicon_tags_7 },
    { 0x58894517, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1649451c, SUC_lexicon_tags_49 },
    { 0x1b89451c, SUC_lexicon_tags_14 },
    { 0x9a69451e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47494526, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x86e94528, SUC_lexicon_tags_12 },
    { 0x9d094528, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd109452b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf094530, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe294533, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96294538, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2729453a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1749453d, SUC_lexicon_tags_2 },
    { 0xce29453d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7894545, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a94548, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e9454b, SUC_lexicon_tags_11 },
    { 0x3b49454b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1189454e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa94553, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a29455d, SUC_lexicon_tags_1 },
    { 0xbf69455e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9ae94560, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93c94562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a294565, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe1294567, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d89456a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c694570, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80894572, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a9457f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc894581, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4694587, SUC_lexicon_tags_28 },
    { 0xa1c94588, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56894590, SUC_lexicon_tags_11 },
    { 0x1b294590, SUC_lexicon_tags_37 },
    { 0xd6894591, SUC_lexicon_tags_82 },
    { 0x8ac94590, SUC_lexicon_tags_0 },
    { 0x51094593, SUC_lexicon_tags_9 },
    { 0x3d694595, SUC_lexicon_tags_12 },
    { 0x9c294592, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5494599, SUC_lexicon_tags_27 },
    { 0x14094599, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xec89459c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0945a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcd2945a5, SUC_lexicon_tags_2 },
    { 0x9d6945a5, SUC_lexicon_tags_1 },
    { 0x3da945a7, SUC_lexicon_tags_63 },
    { 0x352945a8, SUC_lexicon_tags_17 },
    { 0x5d4945a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x516945ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc945b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4945b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf46945b7, SUC_lexicon_tags_87 },
    { 0x502945b7, SUC_lexicon_tags_17 },
    { 0x14e945b7, SUC_lexicon_tags_47 },
    { 0xf58945b7, SUC_lexicon_tags_17 },
    { 0x4a8945ba, SUC_lexicon_tags_11 },
    { 0x742945ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c945c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba945c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c945ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d6945cc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10945cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x480945d1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x816945d4, SUC_lexicon_tags_63 },
    { 0xff6945d4, SUC_lexicon_tags_63 },
    { 0x9d0945d5, SUC_lexicon_tags_25 },
    { 0x692945d7, SUC_lexicon_tags_37 },
    { 0xe70945d5, SUC_lexicon_tags_11 },
    { 0x65a945d9, SUC_lexicon_tags_17 },
    { 0xc94945da, SUC_lexicon_tags_2 },
    { 0x696945db, SUC_lexicon_tags_39 },
    { 0x81e945da, SUC_lexicon_tags_47 },
    { 0x488945dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8e945df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfba945e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc945e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd0945e9, SUC_lexicon_tags_61 },
    { 0xfcc945e9, SUC_lexicon_tags_7 },
    { 0x488945e9, SUC_lexicon_tags_26 },
    { 0x756945e9, SUC_lexicon_tags_25 },
    { 0x882945ed, SUC_lexicon_tags_37 },
    { 0x684945ed, SUC_lexicon_tags_63 },
    { 0x360945ee, SUC_lexicon_tags_2 },
    { 0xc6c945f0, SUC_lexicon_tags_127 },
    { 0x176945f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8945f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6945fa, SUC_lexicon_tags_7 },
    { 0x4d6945fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xef6945fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6a945ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3694601, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcee94603, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc1494605, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7494607, SUC_lexicon_tags_37 },
    { 0x15c94607, SUC_lexicon_tags_27 },
    { 0x36094608, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf769460c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d49460f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15894618, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc069461a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf09461d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c94623, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91894625, SUC_lexicon_tags_11 },
    { 0xbce94626, SUC_lexicon_tags_25 },
    { 0x8a494627, SUC_lexicon_tags_79 },
    { 0x2aa94626, SUC_lexicon_tags_23 },
    { 0x5494626, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0094630, SUC_lexicon_tags_17 },
    { 0xe1094630, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x27694634, SUC_lexicon_tags_11 },
    { 0xa5a94635, SUC_lexicon_tags_31 },
    { 0x1ce94636, SUC_lexicon_tags_17 },
    { 0x99a94636, SUC_lexicon_tags_37 },
    { 0x28894634, SUC_lexicon_tags_25 },
    { 0x21a94639, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9729463d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52e9463f, SUC_lexicon_tags_82 },
    { 0xa2c94640, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6094643, SUC_lexicon_tags_7 },
    { 0x92294644, SUC_lexicon_tags_25 },
    { 0x25e94645, SUC_lexicon_tags_37 },
    { 0xdbc94644, SUC_lexicon_tags_17 },
    { 0xb894647, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb849464b, SUC_lexicon_tags_25 },
    { 0xd409464c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9894654, SUC_lexicon_tags_23 },
    { 0x29094655, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x7e94657, SUC_lexicon_tags_23 },
    { 0x7ac94657, SUC_lexicon_tags_17 },
    { 0x6bc94659, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe789465b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb1e9465d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c94660, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe94663, SUC_lexicon_tags_17 },
    { 0xa4294664, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3494667, SUC_lexicon_tags_30 },
    { 0x10e94668, SUC_lexicon_tags_65 },
    { 0xc5494669, SUC_lexicon_tags_15 },
    { 0x41c9466a, SUC_lexicon_tags_12 },
    { 0xca694668, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9466f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9467b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x40c9467d, SUC_lexicon_tags_37 },
    { 0x7b69467e, SUC_lexicon_tags_12 },
    { 0x10a9467e, SUC_lexicon_tags_15 },
    { 0x6ea94680, SUC_lexicon_tags_9 },
    { 0xde494681, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x83694683, SUC_lexicon_tags_17 },
    { 0x81894683, SUC_lexicon_tags_47 },
    { 0xf9894685, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3de94687, SUC_lexicon_tags_16 },
    { 0xe8094687, SUC_lexicon_tags_1 },
    { 0xac094689, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x32a9468b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4de9468f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3e94691, SUC_lexicon_tags_28 },
    { 0x3be94691, SUC_lexicon_tags_30 },
    { 0x4a294691, SUC_lexicon_tags_63 },
    { 0xdd694693, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee94697, SUC_lexicon_tags_0 },
    { 0x6b294698, SUC_lexicon_tags_26 },
    { 0xf6894699, SUC_lexicon_tags_38 },
    { 0x1c94698, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb49469f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4946a2, SUC_lexicon_tags_65 },
    { 0x374946a2, SUC_lexicon_tags_17 },
    { 0xa92946a3, SUC_lexicon_tags_14 },
    { 0x4c946a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198946a9, SUC_lexicon_tags_7 },
    { 0x2be946aa, SUC_lexicon_tags_7 },
    { 0xd90946a9, SUC_lexicon_tags_6 },
    { 0xf50946ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7a946ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c946b1, SUC_lexicon_tags_25 },
    { 0x3c8946b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928946bc, SUC_lexicon_tags_9 },
    { 0x84c946bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6946c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x28c946c2, SUC_lexicon_tags_49 },
    { 0xfba946c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x932946c5, SUC_lexicon_tags_140 },
    { 0x6fe946c6, SUC_lexicon_tags_72 },
    { 0xdae946c7, SUC_lexicon_tags_89 },
    { 0x6ec946c7, SUC_lexicon_tags_120 },
    { 0xb24946c7, SUC_lexicon_tags_12 },
    { 0x13c946ca, SUC_lexicon_tags_15 },
    { 0xdb0946ca, SUC_lexicon_tags_37 },
    { 0x802946cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354946d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb12946d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x502946d5, SUC_lexicon_tags_11 },
    { 0x83a946d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x248946d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100946dc, SUC_lexicon_tags_31 },
    { 0xd1e946dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34946e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926946ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x226946ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x264946ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834946f3, SUC_lexicon_tags_36 },
    { 0xb8e946f4, SUC_lexicon_tags_38 },
    { 0x2f8946f4, SUC_lexicon_tags_17 },
    { 0x5c6946f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a2946f8, SUC_lexicon_tags_19 },
    { 0x56c946f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe946fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c94707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a9470f, SUC_lexicon_tags_39 },
    { 0xee094710, SUC_lexicon_tags_45 },
    { 0x7a494711, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8494715, SUC_lexicon_tags_37 },
    { 0x42694716, SUC_lexicon_tags_60 },
    { 0xd7494717, SUC_lexicon_tags_4 },
    { 0x8a094717, SUC_lexicon_tags_61 },
    { 0xb6a94719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc569471b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9471f, SUC_lexicon_tags_19 },
    { 0xaf694720, SUC_lexicon_tags_4 },
    { 0x2989471f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60294723, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4694725, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc94729, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2389472e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe9473a, SUC_lexicon_tags_433 },
    { 0xa7c9473a, SUC_lexicon_tags_2 },
    { 0x90c9473a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc89473e, SUC_lexicon_tags_17 },
    { 0x1ae9473f, SUC_lexicon_tags_47 },
    { 0xc5894740, SUC_lexicon_tags_9 },
    { 0x6d09473e, SUC_lexicon_tags_6 },
    { 0x81494740, SUC_lexicon_tags_54 },
    { 0xa6e94743, SUC_lexicon_tags_9 },
    { 0x1694743, SUC_lexicon_tags_17 },
    { 0xbe94745, SUC_lexicon_tags_14 },
    { 0x88494744, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf894748, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b89474c, SUC_lexicon_tags_7 },
    { 0xc149474d, SUC_lexicon_tags_19 },
    { 0xeb69474e, SUC_lexicon_tags_12 },
    { 0xd729474f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x28c94751, SUC_lexicon_tags_26 },
    { 0x56694751, SUC_lexicon_tags_49 },
    { 0x21a94752, SUC_lexicon_tags_34 },
    { 0xbfc94754, SUC_lexicon_tags_151 },
    { 0xad894754, SUC_lexicon_tags_19 },
    { 0x59c94756, SUC_lexicon_tags_27 },
    { 0x77894755, SUC_lexicon_tags_28 },
    { 0x92a94751, SUC_lexicon_tags_17 },
    { 0x97c94755, SUC_lexicon_tags_7 },
    { 0x3c69475a, SUC_lexicon_tags_37 },
    { 0x12e94753, SUC_lexicon_tags_84 },
    { 0x7989475c, SUC_lexicon_tags_2 },
    { 0x33e9475d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c94762, SUC_lexicon_tags_0 },
    { 0x32694763, SUC_lexicon_tags_6 },
    { 0xe4a94762, SUC_lexicon_tags_14 },
    { 0xb8a94763, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5294768, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd549476d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58894771, SUC_lexicon_tags_19 },
    { 0xf0494771, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0294775, SUC_lexicon_tags_47 },
    { 0xa7494776, SUC_lexicon_tags_37 },
    { 0xd0e94777, SUC_lexicon_tags_594 },
    { 0x4494778, SUC_lexicon_tags_2 },
    { 0x8294776, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5869477b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea49477e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x494780, SUC_lexicon_tags_63 },
    { 0x2da94780, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5494787, SUC_lexicon_tags_39 },
    { 0x12894787, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2349478b, SUC_lexicon_tags_31 },
    { 0x2189478c, SUC_lexicon_tags_15 },
    { 0xb709478c, SUC_lexicon_tags_38 },
    { 0x73c9478b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2294790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94494792, SUC_lexicon_tags_12 },
    { 0x39a94793, SUC_lexicon_tags_9 },
    { 0x2894794, SUC_lexicon_tags_15 },
    { 0xea094794, SUC_lexicon_tags_17 },
    { 0x93094792, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2094799, SUC_lexicon_tags_48 },
    { 0xaec9479a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e49479d, SUC_lexicon_tags_30 },
    { 0x36a9479e, SUC_lexicon_tags_15 },
    { 0x36a9479f, SUC_lexicon_tags_17 },
    { 0xe6e9479d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd02947a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10947aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a947af, SUC_lexicon_tags_2 },
    { 0xdba947af, SUC_lexicon_tags_37 },
    { 0x948947b1, SUC_lexicon_tags_2 },
    { 0xd32947b2, SUC_lexicon_tags_0 },
    { 0xc30947b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca947ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828947c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884947cb, SUC_lexicon_tags_68 },
    { 0x862947cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6947cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2947d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x942947d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916947df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8947e3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c947e6, SUC_lexicon_tags_17 },
    { 0x7dc947e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e947ee, SUC_lexicon_tags_1 },
    { 0x91e947ef, SUC_lexicon_tags_14 },
    { 0x540947ee, SUC_lexicon_tags_38 },
    { 0x4a8947f1, SUC_lexicon_tags_47 },
    { 0x5d4947f2, SUC_lexicon_tags_11 },
    { 0xd9c947f3, SUC_lexicon_tags_25 },
    { 0x9b0947f3, SUC_lexicon_tags_65 },
    { 0x31c947ee, SUC_lexicon_tags_3 },
    { 0x962947f6, SUC_lexicon_tags_12 },
    { 0xc0a947f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba0947f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd6e947fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2947ff, SUC_lexicon_tags_59 },
    { 0x1ae94800, SUC_lexicon_tags_23 },
    { 0x17494801, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc9480d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a94810, SUC_lexicon_tags_22 },
    { 0xf7294810, SUC_lexicon_tags_29 },
    { 0xed294810, SUC_lexicon_tags_14 },
    { 0x12e94813, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24494815, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb769481d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3b49481f, SUC_lexicon_tags_29 },
    { 0xc1094820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1294822, SUC_lexicon_tags_27 },
    { 0x26894823, SUC_lexicon_tags_37 },
    { 0xffc94822, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10694829, SUC_lexicon_tags_12 },
    { 0x17094829, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe149482c, SUC_lexicon_tags_21 },
    { 0xbf89482d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a49482f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3094832, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea094834, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf669483d, SUC_lexicon_tags_7 },
    { 0x5b49483d, SUC_lexicon_tags_39 },
    { 0x2d49483d, SUC_lexicon_tags_25 },
    { 0x30e94840, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73494845, SUC_lexicon_tags_10 },
    { 0xce894846, SUC_lexicon_tags_60 },
    { 0xe0c94845, SUC_lexicon_tags_14 },
    { 0x85294848, SUC_lexicon_tags_25 },
    { 0xfd094849, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c94851, SUC_lexicon_tags_12 },
    { 0x76694851, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaea94854, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e9485e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32494860, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc094864, SUC_lexicon_tags_4 },
    { 0x54694865, SUC_lexicon_tags_2 },
    { 0xcec94865, SUC_lexicon_tags_25 },
    { 0xb7094866, SUC_lexicon_tags_19 },
    { 0x94094865, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a9486c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32a9486e, SUC_lexicon_tags_37 },
    { 0xa909486e, SUC_lexicon_tags_15 },
    { 0xe1494870, SUC_lexicon_tags_84 },
    { 0xaf494871, SUC_lexicon_tags_46 },
    { 0xa169486f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca94875, SUC_lexicon_tags_7 },
    { 0xa9494876, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c9487b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa94882, SUC_lexicon_tags_37 },
    { 0xc4c94883, SUC_lexicon_tags_65 },
    { 0x90294884, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64494889, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9509488b, SUC_lexicon_tags_43 },
    { 0xa449488c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c94890, SUC_lexicon_tags_17 },
    { 0x5fa94891, SUC_lexicon_tags_17 },
    { 0x46694890, SUC_lexicon_tags_45 },
    { 0xcda94890, SUC_lexicon_tags_48 },
    { 0x4ae94892, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce894897, SUC_lexicon_tags_8 },
    { 0xa8294898, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdb89489a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7409489c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a948a0, SUC_lexicon_tags_6 },
    { 0xd84948a0, SUC_lexicon_tags_43 },
    { 0x2e6948a0, SUC_lexicon_tags_15 },
    { 0x8ae948a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf8948a5, SUC_lexicon_tags_25 },
    { 0xb1e948a6, SUC_lexicon_tags_12 },
    { 0x1e8948a5, SUC_lexicon_tags_4 },
    { 0x3e8948a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486948ad, SUC_lexicon_tags_15 },
    { 0x61e948ae, SUC_lexicon_tags_53 },
    { 0xae2948ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332948b3, SUC_lexicon_tags_12 },
    { 0x78948b4, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574948b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6948ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c948be, SUC_lexicon_tags_15 },
    { 0xbca948be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x214948c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48948c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8fc948c7, SUC_lexicon_tags_17 },
    { 0xe26948c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638948cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x462948d0, SUC_lexicon_tags_9 },
    { 0x11c948d1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42948d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2948d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6c948d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2948de, SUC_lexicon_tags_11 },
    { 0xab0948de, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50948e2, SUC_lexicon_tags_80 },
    { 0x786948e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x904948e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e948e9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x376948ec, SUC_lexicon_tags_63 },
    { 0x5b4948ed, SUC_lexicon_tags_17 },
    { 0x780948ec, SUC_lexicon_tags_9 },
    { 0xf10948ed, SUC_lexicon_tags_28 },
    { 0xb10948f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa948f2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436948f6, SUC_lexicon_tags_12 },
    { 0x6f0948f7, SUC_lexicon_tags_47 },
    { 0x46c948f6, SUC_lexicon_tags_7 },
    { 0x26a948f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62948fc, SUC_lexicon_tags_45 },
    { 0x710948fd, SUC_lexicon_tags_25 },
    { 0xc60948fd, SUC_lexicon_tags_33 },
    { 0x18c948fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90094901, SUC_lexicon_tags_87 },
    { 0x25e94902, SUC_lexicon_tags_1 },
    { 0xc3e94903, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a694905, SUC_lexicon_tags_27 },
    { 0x5f694906, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44694908, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf389490f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x98094911, SUC_lexicon_tags_12 },
    { 0x17e94911, SUC_lexicon_tags_77 },
    { 0x80494913, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a9491b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb249491f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c94923, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13294928, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a9492b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e9492f, SUC_lexicon_tags_11 },
    { 0x8d094930, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a94933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c69493a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c29493c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a29493e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a94941, SUC_lexicon_tags_11 },
    { 0x10694942, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x58694945, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4694948, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc049494a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe109494c, SUC_lexicon_tags_45 },
    { 0x5069494c, SUC_lexicon_tags_17 },
    { 0x8149494e, SUC_lexicon_tags_25 },
    { 0x2f29494d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90694952, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27694956, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8b494958, SUC_lexicon_tags_12 },
    { 0xfa294959, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9495d, SUC_lexicon_tags_716 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e94961, SUC_lexicon_tags_63 },
    { 0x34c94962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c94970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c94976, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e9497d, SUC_lexicon_tags_4 },
    { 0x8ee9497e, SUC_lexicon_tags_1 },
    { 0x7909497f, SUC_lexicon_tags_23 },
    { 0xe289497f, SUC_lexicon_tags_12 },
    { 0x29c94981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd569498a, SUC_lexicon_tags_15 },
    { 0xcde9498a, SUC_lexicon_tags_19 },
    { 0x5929498b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f09498f, SUC_lexicon_tags_185 },
    { 0xaf49498f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8e94992, SUC_lexicon_tags_23 },
    { 0x9c894993, SUC_lexicon_tags_23 },
    { 0x32894994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b094997, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8349499c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc9499f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4949a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0949a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52949b3, SUC_lexicon_tags_16 },
    { 0x34a949b4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c949b9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c949bc, SUC_lexicon_tags_7 },
    { 0xa86949bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd6949bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0949c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe34949c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8ba949c6, SUC_lexicon_tags_7 },
    { 0x92949c7, SUC_lexicon_tags_7 },
    { 0xa28949c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0949cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec949db, SUC_lexicon_tags_25 },
    { 0x548949dc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaa0949de, SUC_lexicon_tags_37 },
    { 0x6c6949df, SUC_lexicon_tags_63 },
    { 0xad4949e0, SUC_lexicon_tags_32 },
    { 0xb42949e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e949e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2949e7, SUC_lexicon_tags_45 },
    { 0xe8a949e8, SUC_lexicon_tags_0 },
    { 0xf80949e9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x370949eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce4949ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586949f2, SUC_lexicon_tags_15 },
    { 0x4bc949f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x65a949f5, SUC_lexicon_tags_14 },
    { 0x158949f5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1b6949f8, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c949fc, SUC_lexicon_tags_25 },
    { 0xa949fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdea949ff, SUC_lexicon_tags_40 },
    { 0x91894a00, SUC_lexicon_tags_17 },
    { 0xde94a01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea694a07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3694a0b, SUC_lexicon_tags_26 },
    { 0x22294a0b, SUC_lexicon_tags_46 },
    { 0x2e294a0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de94a18, SUC_lexicon_tags_69 },
    { 0x14094a19, SUC_lexicon_tags_72 },
    { 0xd1094a1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a694a24, SUC_lexicon_tags_11 },
    { 0xea694a25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8294a2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a494a2e, SUC_lexicon_tags_88 },
    { 0x11a94a2f, SUC_lexicon_tags_16 },
    { 0xc7894a30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d494a37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96694a3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9294a41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5694a4b, SUC_lexicon_tags_37 },
    { 0x3dc94a4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2294a4e, SUC_lexicon_tags_6 },
    { 0x6cc94a4f, SUC_lexicon_tags_43 },
    { 0xb0494a50, SUC_lexicon_tags_7 },
    { 0xf2e94a50, SUC_lexicon_tags_1 },
    { 0x98094a52, SUC_lexicon_tags_15 },
    { 0x6ba94a52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be94a56, SUC_lexicon_tags_65 },
    { 0x5b294a56, SUC_lexicon_tags_27 },
    { 0x65694a58, SUC_lexicon_tags_25 },
    { 0x45494a57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25294a5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa94a63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12294a66, SUC_lexicon_tags_45 },
    { 0x8aa94a67, SUC_lexicon_tags_82 },
    { 0x85c94a68, SUC_lexicon_tags_60 },
    { 0x76294a68, SUC_lexicon_tags_37 },
    { 0x78494a6a, SUC_lexicon_tags_15 },
    { 0x7c894a6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95894a71, SUC_lexicon_tags_46 },
    { 0x89694a71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c94a75, SUC_lexicon_tags_12 },
    { 0xf8a94a75, SUC_lexicon_tags_68 },
    { 0x43894a77, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b494a7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c94a7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d894a85, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f894a89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49294a8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f094a93, SUC_lexicon_tags_26 },
    { 0x694a94, SUC_lexicon_tags_7 },
    { 0x8b294a95, SUC_lexicon_tags_46 },
    { 0x2f894a93, SUC_lexicon_tags_37 },
    { 0xa4094a97, SUC_lexicon_tags_63 },
    { 0x2e094a98, SUC_lexicon_tags_1 },
    { 0xa9094a94, SUC_lexicon_tags_2 },
    { 0x2c294a9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22494a9f, SUC_lexicon_tags_19 },
    { 0xb094aa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1894aa5, SUC_lexicon_tags_25 },
    { 0xcca94aa6, SUC_lexicon_tags_21 },
    { 0x22894aa7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b894aae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77e94ab0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60294ab3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x17a94ab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76294abf, SUC_lexicon_tags_17 },
    { 0x2d094abf, SUC_lexicon_tags_4 },
    { 0x86a94ac1, SUC_lexicon_tags_7 },
    { 0xb5094ac2, SUC_lexicon_tags_31 },
    { 0xf8094ac2, SUC_lexicon_tags_38 },
    { 0xcca94ac4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d094ac7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51494acc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2094acf, SUC_lexicon_tags_4 },
    { 0x3ce94ad0, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c94ad4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0894adc, SUC_lexicon_tags_16 },
    { 0x19a94add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c494adf, SUC_lexicon_tags_7 },
    { 0x6ec94ae0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x13e94ae2, SUC_lexicon_tags_37 },
    { 0xa3a94ae3, SUC_lexicon_tags_8 },
    { 0x3d494ae4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x694ae7, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x96e94ae9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda94aed, SUC_lexicon_tags_6 },
    { 0x41294aee, SUC_lexicon_tags_45 },
    { 0xa7494aef, SUC_lexicon_tags_60 },
    { 0xe5494af0, SUC_lexicon_tags_25 },
    { 0x54694aed, SUC_lexicon_tags_12 },
    { 0xf6c94af2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16694af7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de94afa, SUC_lexicon_tags_37 },
    { 0x8ae94afb, SUC_lexicon_tags_11 },
    { 0x32294afb, SUC_lexicon_tags_39 },
    { 0x66894afd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x51894b00, SUC_lexicon_tags_0 },
    { 0x52094b00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84094b04, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x24094b06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ce94b0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e94b13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1494b1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec94b1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d294b21, SUC_lexicon_tags_12 },
    { 0x9d694b22, SUC_lexicon_tags_17 },
    { 0x47494b21, SUC_lexicon_tags_33 },
    { 0x99294b24, SUC_lexicon_tags_19 },
    { 0xe4494b25, SUC_lexicon_tags_43 },
    { 0x71694b26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xff894b29, SUC_lexicon_tags_12 },
    { 0xb2e94b2a, SUC_lexicon_tags_2 },
    { 0x91e94b2b, SUC_lexicon_tags_2 },
    { 0x9d294b2c, SUC_lexicon_tags_38 },
    { 0x61a94b2a, SUC_lexicon_tags_7 },
    { 0xd4094b2e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x32694b31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0894b33, SUC_lexicon_tags_23 },
    { 0x5a94b34, SUC_lexicon_tags_1 },
    { 0xc2094b35, SUC_lexicon_tags_12 },
    { 0xb5c94b34, SUC_lexicon_tags_15 },
    { 0x8d294b37, SUC_lexicon_tags_38 },
    { 0xf9894b35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x13094b3b, SUC_lexicon_tags_1 },
    { 0x50094b3c, SUC_lexicon_tags_15 },
    { 0x23294b3c, SUC_lexicon_tags_127 },
    { 0x9e294b3e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x50694b40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a494b46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7894b48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e894b4a, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85094b53, SUC_lexicon_tags_37 },
    { 0xb6c94b54, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f694b58, SUC_lexicon_tags_37 },
    { 0xc0494b59, SUC_lexicon_tags_11 },
    { 0xe3694b5a, SUC_lexicon_tags_32 },
    { 0x49a94b5b, SUC_lexicon_tags_11 },
    { 0xc0c94b5c, SUC_lexicon_tags_56 },
    { 0xce294b5d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe94b60, SUC_lexicon_tags_25 },
    { 0xf4694b61, SUC_lexicon_tags_25 },
    { 0x3cc94b60, SUC_lexicon_tags_174 },
    { 0xf8094b63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e94b66, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0294b6a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6da94b6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdc494b6e, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x91894b70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5294b72, SUC_lexicon_tags_27 },
    { 0x76294b73, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c494b76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7694b78, SUC_lexicon_tags_11 },
    { 0xc1094b79, SUC_lexicon_tags_2 },
    { 0x17294b7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f894b7c, SUC_lexicon_tags_26 },
    { 0x6d694b7d, SUC_lexicon_tags_6 },
    { 0x7894b7d, SUC_lexicon_tags_12 },
    { 0x96e94b7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a94b82, SUC_lexicon_tags_113 },
    { 0x18294b83, SUC_lexicon_tags_4 },
    { 0x50a94b82, SUC_lexicon_tags_23 },
    { 0x4a694b85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec94b90, SUC_lexicon_tags_11 },
    { 0x26694b91, SUC_lexicon_tags_11 },
    { 0x30694b90, SUC_lexicon_tags_63 },
    { 0xc6494b90, SUC_lexicon_tags_63 },
    { 0x23a94b93, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34694b98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12894b9b, SUC_lexicon_tags_25 },
    { 0xea094b9c, SUC_lexicon_tags_28 },
    { 0x1094b9d, SUC_lexicon_tags_5 },
    { 0x5f094b9c, SUC_lexicon_tags_25 },
    { 0x8fc94b9e, SUC_lexicon_tags_38 },
    { 0xcb894b9c, SUC_lexicon_tags_57 },
    { 0x1b894b9d, SUC_lexicon_tags_7 },
    { 0xd4c94b9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce94bac, SUC_lexicon_tags_1 },
    { 0xe7094bad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfb294baf, SUC_lexicon_tags_25 },
    { 0x70a94bb0, SUC_lexicon_tags_63 },
    { 0x69e94bb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a94bb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85294bb7, SUC_lexicon_tags_7 },
    { 0xfa094bb8, SUC_lexicon_tags_1 },
    { 0xe694bb9, SUC_lexicon_tags_10 },
    { 0x36c94bba, SUC_lexicon_tags_37 },
    { 0xb8294bba, SUC_lexicon_tags_84 },
    { 0xb7294bba, SUC_lexicon_tags_25 },
    { 0xc2694bb7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83494bc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2294bc8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22894bca, SUC_lexicon_tags_37 },
    { 0xb2694bcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc94bd1, SUC_lexicon_tags_0 },
    { 0xcf294bd2, SUC_lexicon_tags_17 },
    { 0x88494bd3, SUC_lexicon_tags_8 },
    { 0x35094bd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa94bd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd894bda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7294bde, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47094be2, SUC_lexicon_tags_37 },
    { 0x26e94be3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13894be7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e494bf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b494bf2, SUC_lexicon_tags_7 },
    { 0x80494bf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc294bf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e94c04, SUC_lexicon_tags_43 },
    { 0xa5c94c05, SUC_lexicon_tags_11 },
    { 0x4c094c06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2c94c08, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x82694c0b, SUC_lexicon_tags_28 },
    { 0x17494c0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70294c10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d894c16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a94c1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95894c22, SUC_lexicon_tags_44 },
    { 0x81494c22, SUC_lexicon_tags_6 },
    { 0xd8e94c24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23694c2b, SUC_lexicon_tags_37 },
    { 0x84e94c2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e94c30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba94c33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3694c35, SUC_lexicon_tags_174 },
    { 0x89894c35, SUC_lexicon_tags_0 },
    { 0x87e94c37, SUC_lexicon_tags_33 },
    { 0x93c94c38, SUC_lexicon_tags_17 },
    { 0x97e94c35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b094c3b, SUC_lexicon_tags_43 },
    { 0x82494c3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31494c3f, SUC_lexicon_tags_4 },
    { 0x78494c3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3e294c42, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc94c47, SUC_lexicon_tags_37 },
    { 0xc7694c48, SUC_lexicon_tags_37 },
    { 0x8f094c49, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa2694c4b, SUC_lexicon_tags_1 },
    { 0x33e94c4c, SUC_lexicon_tags_7 },
    { 0xc7894c4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae94c51, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa9a94c53, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x30e94c55, SUC_lexicon_tags_23 },
    { 0xf6894c55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72894c5a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd5694c5c, SUC_lexicon_tags_37 },
    { 0xe6e94c5d, SUC_lexicon_tags_37 },
    { 0xba094c5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2694c60, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde494c67, SUC_lexicon_tags_7 },
    { 0xb0294c67, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44294c6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41494c71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11094c73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a94c76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd9094c78, SUC_lexicon_tags_11 },
    { 0xcb094c79, SUC_lexicon_tags_63 },
    { 0x9d294c7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f094c80, SUC_lexicon_tags_29 },
    { 0x4f294c81, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e94c84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e94c87, SUC_lexicon_tags_95 },
    { 0x87894c88, SUC_lexicon_tags_32 },
    { 0xb6e94c88, SUC_lexicon_tags_43 },
    { 0x62e94c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea94c8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61a94c8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f294c91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbc094c93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694c9c, SUC_lexicon_tags_35 },
    { 0xcb894c9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc94ca0, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae94ca3, SUC_lexicon_tags_17 },
    { 0x2fa94ca3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a94ca7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38894ca9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5294caf, SUC_lexicon_tags_53 },
    { 0x44494cb0, SUC_lexicon_tags_17 },
    { 0x1a694caf, SUC_lexicon_tags_37 },
    { 0x1a294cb2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8c94cb4, SUC_lexicon_tags_17 },
    { 0xfaa94cb5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcf694cb7, SUC_lexicon_tags_87 },
    { 0xbe94cb8, SUC_lexicon_tags_7 },
    { 0x1a294cb8, SUC_lexicon_tags_9 },
    { 0x53e94cb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc494cbc, SUC_lexicon_tags_14 },
    { 0x7fa94cbd, SUC_lexicon_tags_15 },
    { 0x11294cbe, SUC_lexicon_tags_12 },
    { 0x40e94cbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc94cc3, SUC_lexicon_tags_11 },
    { 0xa3094cc4, SUC_lexicon_tags_37 },
    { 0xd3a94cc5, SUC_lexicon_tags_15 },
    { 0xc694cc4, SUC_lexicon_tags_12 },
    { 0xcb494cc3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e94ccc, SUC_lexicon_tags_19 },
    { 0xd3294ccd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x53694cd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d694cd5, SUC_lexicon_tags_33 },
    { 0x83a94cd6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6a094cd8, SUC_lexicon_tags_48 },
    { 0x14494cd9, SUC_lexicon_tags_55 },
    { 0xff494cda, SUC_lexicon_tags_12 },
    { 0x95494cdb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x33a94cdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc94ce4, SUC_lexicon_tags_17 },
    { 0xf0894ce5, SUC_lexicon_tags_60 },
    { 0x5c494ce6, SUC_lexicon_tags_11 },
    { 0x46694ce7, SUC_lexicon_tags_12 },
    { 0x73a94ce8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c94ceb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72094ced, SUC_lexicon_tags_105 },
    { 0x81294cee, SUC_lexicon_tags_64 },
    { 0x2494cef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc894cf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa094cfa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca94cfe, SUC_lexicon_tags_1 },
    { 0x89c94cff, SUC_lexicon_tags_10 },
    { 0xb3894d00, SUC_lexicon_tags_7 },
    { 0x93094cfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51894d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a94d08, SUC_lexicon_tags_37 },
    { 0x6a694d09, SUC_lexicon_tags_37 },
    { 0x18294d0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2494d10, SUC_lexicon_tags_25 },
    { 0x29694d10, SUC_lexicon_tags_2 },
    { 0x11694d10, SUC_lexicon_tags_25 },
    { 0xa1294d13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3e94d15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbac94d17, SUC_lexicon_tags_17 },
    { 0x57294d18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19294d1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc94d23, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8894d26, SUC_lexicon_tags_39 },
    { 0x65294d26, SUC_lexicon_tags_11 },
    { 0xabe94d28, SUC_lexicon_tags_38 },
    { 0xe9e94d29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb9c94d2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6294d2d, SUC_lexicon_tags_17 },
    { 0x49094d2d, SUC_lexicon_tags_68 },
    { 0x77094d2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74694d31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdaa94d33, SUC_lexicon_tags_25 },
    { 0x2a494d34, SUC_lexicon_tags_26 },
    { 0x39494d33, SUC_lexicon_tags_47 },
    { 0xba694d34, SUC_lexicon_tags_31 },
    { 0xffc94d37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x62e94d39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb494d3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44494d3d, SUC_lexicon_tags_10 },
    { 0xaf094d3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9494d46, SUC_lexicon_tags_33 },
    { 0xe4a94d47, SUC_lexicon_tags_25 },
    { 0xe0094d47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5f894d4a, SUC_lexicon_tags_47 },
    { 0x75c94d4b, SUC_lexicon_tags_2 },
    { 0x51494d4c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f694d4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63094d53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e894d5a, SUC_lexicon_tags_28 },
    { 0x70494d5b, SUC_lexicon_tags_19 },
    { 0x33a94d5c, SUC_lexicon_tags_17 },
    { 0x8f494d5d, SUC_lexicon_tags_25 },
    { 0xf8694d5b, SUC_lexicon_tags_25 },
    { 0x23294d5d, SUC_lexicon_tags_15 },
    { 0x83494d60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf494d63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb294d67, SUC_lexicon_tags_21 },
    { 0x2694d68, SUC_lexicon_tags_17 },
    { 0x74494d69, SUC_lexicon_tags_259 },
    { 0x54294d6a, SUC_lexicon_tags_70 },
    { 0x6ee94d67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4c94d6d, SUC_lexicon_tags_47 },
    { 0x1fe94d6e, SUC_lexicon_tags_79 },
    { 0x18e94d6f, SUC_lexicon_tags_15 },
    { 0x6c894d6f, SUC_lexicon_tags_47 },
    { 0x6c094d6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac94d7e, SUC_lexicon_tags_29 },
    { 0xcca94d7e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd6094d81, SUC_lexicon_tags_17 },
    { 0x34a94d82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32494d86, SUC_lexicon_tags_17 },
    { 0x1a494d87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb894d8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a94d8e, SUC_lexicon_tags_37 },
    { 0xfdc94d8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66294d91, SUC_lexicon_tags_296 },
    { 0xfce94d92, SUC_lexicon_tags_7 },
    { 0x1694d93, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a94d97, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xed294d9a, SUC_lexicon_tags_32 },
    { 0xc4a94d9b, SUC_lexicon_tags_14 },
    { 0x6a694d9a, SUC_lexicon_tags_17 },
    { 0x59a94d9d, SUC_lexicon_tags_7 },
    { 0x77294d9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e94da1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e694da5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e94da9, SUC_lexicon_tags_1 },
    { 0xda694daa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2494dad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda894daf, SUC_lexicon_tags_37 },
    { 0xd3494daf, SUC_lexicon_tags_17 },
    { 0xad094db1, SUC_lexicon_tags_23 },
    { 0x31a94db2, SUC_lexicon_tags_43 },
    { 0xff494db2, SUC_lexicon_tags_27 },
    { 0x71894daf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc694db6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e94dbc, SUC_lexicon_tags_37 },
    { 0x6b294dbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad694dc2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd894dc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65294dc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa894dcd, SUC_lexicon_tags_11 },
    { 0xa6494dce, SUC_lexicon_tags_27 },
    { 0x2a094dcf, SUC_lexicon_tags_48 },
    { 0x14e94dd0, SUC_lexicon_tags_7 },
    { 0x61c94dcd, SUC_lexicon_tags_37 },
    { 0x49894dd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x64894dd4, SUC_lexicon_tags_8 },
    { 0xc5e94dd5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d294dd8, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a94ddb, SUC_lexicon_tags_17 },
    { 0x39e94ddc, SUC_lexicon_tags_12 },
    { 0x5b094ddd, SUC_lexicon_tags_39 },
    { 0x51494dde, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91694de4, SUC_lexicon_tags_2 },
    { 0x73a94de5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a894de8, SUC_lexicon_tags_65 },
    { 0x9ac94de8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8a294deb, SUC_lexicon_tags_11 },
    { 0xd4294dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2894dee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd094df0, SUC_lexicon_tags_1 },
    { 0x3b494df1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39694df3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a94dfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda94dff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x46294e01, SUC_lexicon_tags_32 },
    { 0xc0a94e02, SUC_lexicon_tags_163 },
    { 0xef894e01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30e94e05, SUC_lexicon_tags_47 },
    { 0x4d294e06, SUC_lexicon_tags_11 },
    { 0xee894e07, SUC_lexicon_tags_49 },
    { 0xce494e08, SUC_lexicon_tags_14 },
    { 0x43c94e08, SUC_lexicon_tags_43 },
    { 0xaf694e08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc94e0f, SUC_lexicon_tags_25 },
    { 0xb8294e10, SUC_lexicon_tags_34 },
    { 0x9cc94e11, SUC_lexicon_tags_113 },
    { 0x35a94e11, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85694e17, SUC_lexicon_tags_65 },
    { 0x9a694e17, SUC_lexicon_tags_43 },
    { 0xfd294e18, SUC_lexicon_tags_11 },
    { 0x99694e1a, SUC_lexicon_tags_63 },
    { 0xdda94e1b, SUC_lexicon_tags_25 },
    { 0x91a94e1b, SUC_lexicon_tags_37 },
    { 0xac94e1d, SUC_lexicon_tags_23 },
    { 0xd8294e1e, SUC_lexicon_tags_11 },
    { 0x3e294e1e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x48c94e21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a494e26, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc094e28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc94e2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3694e31, SUC_lexicon_tags_23 },
    { 0x11894e32, SUC_lexicon_tags_11 },
    { 0x61694e33, SUC_lexicon_tags_9 },
    { 0x5d294e33, SUC_lexicon_tags_2 },
    { 0xec894e32, SUC_lexicon_tags_63 },
    { 0xc5494e36, SUC_lexicon_tags_19 },
    { 0x9f094e37, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d294e3d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b294e47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d494e4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19894e4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e94e50, SUC_lexicon_tags_26 },
    { 0xfb494e51, SUC_lexicon_tags_28 },
    { 0x96094e51, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8be94e55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda094e5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9894e5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6094e62, SUC_lexicon_tags_23 },
    { 0xd1294e62, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb1094e65, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7da94e67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15494e6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb094e72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65494e77, SUC_lexicon_tags_38 },
    { 0x1b494e77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a94e7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e94e82, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58694e87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a894e89, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x89294e8b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b494e92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11694e94, SUC_lexicon_tags_10 },
    { 0x1a294e95, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a894e98, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73094e9d, SUC_lexicon_tags_15 },
    { 0x6fa94e9d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdf494ea0, SUC_lexicon_tags_43 },
    { 0x54294ea1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae94ea4, SUC_lexicon_tags_37 },
    { 0x14a94ea5, SUC_lexicon_tags_9 },
    { 0xcfc94ea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c94eab, SUC_lexicon_tags_49 },
    { 0x6c94eab, SUC_lexicon_tags_12 },
    { 0xf6894ead, SUC_lexicon_tags_33 },
    { 0xdec94eae, SUC_lexicon_tags_2 },
    { 0xab894eaf, SUC_lexicon_tags_45 },
    { 0x6d894eb0, SUC_lexicon_tags_9 },
    { 0xfa094eaf, SUC_lexicon_tags_15 },
    { 0x42694eb2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x20094eb5, SUC_lexicon_tags_1 },
    { 0xa3694eb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x96894eb9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8694ebf, SUC_lexicon_tags_37 },
    { 0xe7494ec0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6494ec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84694ec6, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38894eca, SUC_lexicon_tags_87 },
    { 0x98a94ecb, SUC_lexicon_tags_10 },
    { 0x57094ecb, SUC_lexicon_tags_2 },
    { 0xad694ecc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60094ed1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc494ed4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96694ed6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e94ed9, SUC_lexicon_tags_15 },
    { 0xe6294ed9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9c94edc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc94ee1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe9694ee3, SUC_lexicon_tags_14 },
    { 0x44894ee4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xefc94ee6, SUC_lexicon_tags_14 },
    { 0xd4e94ee7, SUC_lexicon_tags_7 },
    { 0x56694ee8, SUC_lexicon_tags_37 },
    { 0x37694ee9, SUC_lexicon_tags_66 },
    { 0x25094eea, SUC_lexicon_tags_27 },
    { 0xf0c94ee8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a94ef0, SUC_lexicon_tags_25 },
    { 0x72694ef0, SUC_lexicon_tags_63 },
    { 0x1b094ef0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da94ef5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a94eff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2694f02, SUC_lexicon_tags_19 },
    { 0x8b094f02, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4a894f05, SUC_lexicon_tags_21 },
    { 0x91294f05, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0094f10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4094f17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc494f1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x27694f1d, SUC_lexicon_tags_17 },
    { 0xefa94f1e, SUC_lexicon_tags_9 },
    { 0x54e94f1f, SUC_lexicon_tags_47 },
    { 0xf9a94f20, SUC_lexicon_tags_17 },
    { 0x26694f20, SUC_lexicon_tags_56 },
    { 0x9b294f22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25894f25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1a94f27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a094f2e, SUC_lexicon_tags_7 },
    { 0xa0a94f2f, SUC_lexicon_tags_19 },
    { 0x76c94f30, SUC_lexicon_tags_75 },
    { 0x9aa94f31, SUC_lexicon_tags_38 },
    { 0x7c094f32, SUC_lexicon_tags_55 },
    { 0x50a94f31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a94f3d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c94f43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc494f46, SUC_lexicon_tags_7 },
    { 0x99e94f47, SUC_lexicon_tags_32 },
    { 0x22494f48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae94f4f, SUC_lexicon_tags_35 },
    { 0x7fa94f4f, SUC_lexicon_tags_1 },
    { 0xd5694f51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe694f57, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e94f5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8894f5d, SUC_lexicon_tags_43 },
    { 0x28c94f5e, SUC_lexicon_tags_17 },
    { 0xb0494f5f, SUC_lexicon_tags_65 },
    { 0x62094f60, SUC_lexicon_tags_23 },
    { 0xee494f61, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a094f64, SUC_lexicon_tags_37 },
    { 0xe7c94f64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4094f67, SUC_lexicon_tags_37 },
    { 0x3a694f68, SUC_lexicon_tags_15 },
    { 0x32294f69, SUC_lexicon_tags_74 },
    { 0x66494f68, SUC_lexicon_tags_60 },
    { 0x87a94f6b, SUC_lexicon_tags_11 },
    { 0xc0294f6c, SUC_lexicon_tags_27 },
    { 0xa894f6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef894f73, SUC_lexicon_tags_15 },
    { 0x37494f74, SUC_lexicon_tags_16 },
    { 0xd7494f75, SUC_lexicon_tags_37 },
    { 0x4c094f75, SUC_lexicon_tags_43 },
    { 0x34e94f73, SUC_lexicon_tags_12 },
    { 0x5a894f74, SUC_lexicon_tags_25 },
    { 0xbea94f79, SUC_lexicon_tags_25 },
    { 0x30894f74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c94f7d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55094f81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82894f84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e094f88, SUC_lexicon_tags_25 },
    { 0xe1494f89, SUC_lexicon_tags_31 },
    { 0xf7894f88, SUC_lexicon_tags_43 },
    { 0xcb494f8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8b094f8d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x39094f90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9094f92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4294f96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a094f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52e94f9a, SUC_lexicon_tags_7 },
    { 0x62e94f9b, SUC_lexicon_tags_47 },
    { 0xc8c94f9b, SUC_lexicon_tags_15 },
    { 0x1f294f9c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x90c94f9f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5294fa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50094fab, SUC_lexicon_tags_37 },
    { 0xfbc94fab, SUC_lexicon_tags_37 },
    { 0x14094fad, SUC_lexicon_tags_46 },
    { 0xe7694fae, SUC_lexicon_tags_30 },
    { 0xa894fad, SUC_lexicon_tags_7 },
    { 0x69894fad, SUC_lexicon_tags_12 },
    { 0x9c894fb1, SUC_lexicon_tags_28 },
    { 0x23694fb1, SUC_lexicon_tags_11 },
    { 0x61a94fb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd894fb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34294fbf, SUC_lexicon_tags_49 },
    { 0x2fe94fc0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x63294fc3, SUC_lexicon_tags_63 },
    { 0x8a694fc4, SUC_lexicon_tags_23 },
    { 0x97e94fc5, SUC_lexicon_tags_30 },
    { 0x72294fc6, SUC_lexicon_tags_60 },
    { 0x20094fc7, SUC_lexicon_tags_6 },
    { 0xce094fc4, SUC_lexicon_tags_38 },
    { 0x14494fc9, SUC_lexicon_tags_29 },
    { 0xb6694fca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b894fcd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e94fd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x74494fd3, SUC_lexicon_tags_12 },
    { 0xc4294fd4, SUC_lexicon_tags_37 },
    { 0x1b094fd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac94fd8, SUC_lexicon_tags_38 },
    { 0x1ba94fd8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37094fde, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x55e94fe0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x82894fe2, SUC_lexicon_tags_17 },
    { 0x98e94fe3, SUC_lexicon_tags_25 },
    { 0xe5094fe4, SUC_lexicon_tags_7 },
    { 0x94094fe5, SUC_lexicon_tags_7 },
    { 0x12294fe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98894fec, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c94fef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c294ff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xda894ff5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12294ff7, SUC_lexicon_tags_28 },
    { 0x6894ff7, SUC_lexicon_tags_17 },
    { 0x37894ff8, SUC_lexicon_tags_39 },
    { 0xc1294ffa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75294ffc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4da94ffe, SUC_lexicon_tags_17 },
    { 0x7e94ffe, SUC_lexicon_tags_0 },
    { 0xbba94ffe, SUC_lexicon_tags_25 },
    { 0x79495001, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c95004, SUC_lexicon_tags_4 },
    { 0x36295004, SUC_lexicon_tags_37 },
    { 0xe0295006, SUC_lexicon_tags_2 },
    { 0x80495006, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ee95009, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a49500f, SUC_lexicon_tags_47 },
    { 0x76295010, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81295016, SUC_lexicon_tags_12 },
    { 0xce695017, SUC_lexicon_tags_1 },
    { 0xf3a95018, SUC_lexicon_tags_14 },
    { 0xc0c95017, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c895023, SUC_lexicon_tags_12 },
    { 0x92095024, SUC_lexicon_tags_35 },
    { 0xe5695024, SUC_lexicon_tags_39 },
    { 0xef095026, SUC_lexicon_tags_12 },
    { 0xd4295027, SUC_lexicon_tags_37 },
    { 0x9e495027, SUC_lexicon_tags_47 },
    { 0xea895023, SUC_lexicon_tags_11 },
    { 0x4ba95024, SUC_lexicon_tags_17 },
    { 0xd095024, SUC_lexicon_tags_17 },
    { 0xd949502c, SUC_lexicon_tags_29 },
    { 0xe629502d, SUC_lexicon_tags_17 },
    { 0xc409502e, SUC_lexicon_tags_0 },
    { 0x2329502f, SUC_lexicon_tags_49 },
    { 0x90695030, SUC_lexicon_tags_37 },
    { 0xe5a95031, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc95035, SUC_lexicon_tags_25 },
    { 0x1be95035, SUC_lexicon_tags_7 },
    { 0x99e95037, SUC_lexicon_tags_29 },
    { 0xc5c95038, SUC_lexicon_tags_15 },
    { 0xb0095036, SUC_lexicon_tags_47 },
    { 0x15a9503a, SUC_lexicon_tags_37 },
    { 0xf4c9503a, SUC_lexicon_tags_178 },
    { 0xd4695037, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa295040, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b295046, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc295049, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f49504f, SUC_lexicon_tags_1 },
    { 0x5589504f, SUC_lexicon_tags_2 },
    { 0x62495051, SUC_lexicon_tags_1 },
    { 0x44095051, SUC_lexicon_tags_7 },
    { 0x4e095052, SUC_lexicon_tags_11 },
    { 0xccc95051, SUC_lexicon_tags_0 },
    { 0x84495055, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d495059, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c95064, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe95068, SUC_lexicon_tags_19 },
    { 0xe9c95069, SUC_lexicon_tags_11 },
    { 0x48895069, SUC_lexicon_tags_11 },
    { 0xe029506b, SUC_lexicon_tags_25 },
    { 0x5b29506c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e9506f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc295071, SUC_lexicon_tags_27 },
    { 0x91695072, SUC_lexicon_tags_25 },
    { 0xa3c95073, SUC_lexicon_tags_25 },
    { 0xf2e95072, SUC_lexicon_tags_29 },
    { 0xf3a95074, SUC_lexicon_tags_38 },
    { 0x45295073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da9507a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4e69507c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa9507f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a95081, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa95084, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc95088, SUC_lexicon_tags_0 },
    { 0xb895089, SUC_lexicon_tags_14 },
    { 0xcb29508a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda9508e, SUC_lexicon_tags_63 },
    { 0x6629508e, SUC_lexicon_tags_11 },
    { 0x9fe9508f, SUC_lexicon_tags_106 },
    { 0x7c095091, SUC_lexicon_tags_48 },
    { 0x8269508e, SUC_lexicon_tags_8 },
    { 0x89a95093, SUC_lexicon_tags_1 },
    { 0x45a95094, SUC_lexicon_tags_11 },
    { 0x329508e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4e95097, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x41895099, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2109509b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8769509e, SUC_lexicon_tags_13 },
    { 0xdc49509f, SUC_lexicon_tags_16 },
    { 0x2709509e, SUC_lexicon_tags_37 },
    { 0x3a6950a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580950a6, SUC_lexicon_tags_17 },
    { 0x7c4950a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a950ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e950b0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64950b3, SUC_lexicon_tags_6 },
    { 0x52950b4, SUC_lexicon_tags_1 },
    { 0xe4c950b5, SUC_lexicon_tags_47 },
    { 0xd40950b6, SUC_lexicon_tags_70 },
    { 0xbb2950b7, SUC_lexicon_tags_37 },
    { 0xd0e950b4, SUC_lexicon_tags_48 },
    { 0x3f8950b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e950be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec950c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c950c8, SUC_lexicon_tags_2 },
    { 0x2c6950c8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb14950cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0950d0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3ea950d2, SUC_lexicon_tags_25 },
    { 0xe42950d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2cc950d5, SUC_lexicon_tags_12 },
    { 0xcca950d6, SUC_lexicon_tags_30 },
    { 0x4f6950d6, SUC_lexicon_tags_17 },
    { 0x150950d8, SUC_lexicon_tags_21 },
    { 0x282950d9, SUC_lexicon_tags_15 },
    { 0x6b0950d5, SUC_lexicon_tags_37 },
    { 0xca2950db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5ae950dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5b4950df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b4950e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd5e950e3, SUC_lexicon_tags_25 },
    { 0x256950e4, SUC_lexicon_tags_47 },
    { 0xedc950e4, SUC_lexicon_tags_17 },
    { 0xc0e950e6, SUC_lexicon_tags_25 },
    { 0x794950e7, SUC_lexicon_tags_37 },
    { 0xa44950e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa86950ea, SUC_lexicon_tags_37 },
    { 0x548950eb, SUC_lexicon_tags_14 },
    { 0x76a950ea, SUC_lexicon_tags_16 },
    { 0x700950ed, SUC_lexicon_tags_25 },
    { 0x736950eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x750950f1, SUC_lexicon_tags_65 },
    { 0xf66950f1, SUC_lexicon_tags_13 },
    { 0x78c950f3, SUC_lexicon_tags_12 },
    { 0x730950f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4950fb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38950ff, SUC_lexicon_tags_28 },
    { 0x6895100, SUC_lexicon_tags_60 },
    { 0xcb495100, SUC_lexicon_tags_39 },
    { 0x28a950ff, SUC_lexicon_tags_19 },
    { 0x80295101, SUC_lexicon_tags_17 },
    { 0xc3295104, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x26095107, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb6095109, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada95112, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad695119, SUC_lexicon_tags_2 },
    { 0x4f095119, SUC_lexicon_tags_1 },
    { 0xea49511a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5329511f, SUC_lexicon_tags_1 },
    { 0x4a09511f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee95123, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4495128, SUC_lexicon_tags_49 },
    { 0x28c95129, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6489512b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6495132, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f695136, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa95139, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9513c, SUC_lexicon_tags_37 },
    { 0xcc89513d, SUC_lexicon_tags_25 },
    { 0x8309513d, SUC_lexicon_tags_17 },
    { 0x5649513f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39495144, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcfe95146, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e9514b, SUC_lexicon_tags_17 },
    { 0x2049514b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e495155, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a95159, SUC_lexicon_tags_28 },
    { 0xaac9515a, SUC_lexicon_tags_11 },
    { 0xe8e9515b, SUC_lexicon_tags_11 },
    { 0x8b69515c, SUC_lexicon_tags_11 },
    { 0x8089515b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4909515f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e95164, SUC_lexicon_tags_4 },
    { 0x39095165, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf695167, SUC_lexicon_tags_2 },
    { 0x37095168, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd095170, SUC_lexicon_tags_12 },
    { 0x66095171, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3695173, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d295177, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a9517a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x76c9517c, SUC_lexicon_tags_11 },
    { 0xc849517d, SUC_lexicon_tags_57 },
    { 0xd189517d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d095184, SUC_lexicon_tags_26 },
    { 0x10095185, SUC_lexicon_tags_14 },
    { 0x98895186, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc09518a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xce69518d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91c9518f, SUC_lexicon_tags_14 },
    { 0xca695190, SUC_lexicon_tags_25 },
    { 0x38e95191, SUC_lexicon_tags_22 },
    { 0x57295191, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48095197, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6089519b, SUC_lexicon_tags_1 },
    { 0x4fc9519c, SUC_lexicon_tags_37 },
    { 0x47a9519d, SUC_lexicon_tags_37 },
    { 0xde9519e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0951a3, SUC_lexicon_tags_47 },
    { 0xf74951a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38c951a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5dc951a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4951ae, SUC_lexicon_tags_40 },
    { 0x6f2951af, SUC_lexicon_tags_1 },
    { 0x4c6951b0, SUC_lexicon_tags_27 },
    { 0x844951b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa951ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56951bf, SUC_lexicon_tags_47 },
    { 0xba4951c0, SUC_lexicon_tags_17 },
    { 0x2bc951c1, SUC_lexicon_tags_25 },
    { 0x882951c0, SUC_lexicon_tags_26 },
    { 0xf92951c3, SUC_lexicon_tags_25 },
    { 0x14951c4, SUC_lexicon_tags_12 },
    { 0x7ba951c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45a951c8, SUC_lexicon_tags_33 },
    { 0xf90951c9, SUC_lexicon_tags_31 },
    { 0xdea951ca, SUC_lexicon_tags_55 },
    { 0x26a951cb, SUC_lexicon_tags_45 },
    { 0xe34951c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612951d0, SUC_lexicon_tags_38 },
    { 0x814951d1, SUC_lexicon_tags_31 },
    { 0x8a8951d2, SUC_lexicon_tags_37 },
    { 0x288951d0, SUC_lexicon_tags_63 },
    { 0xf68951d0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb62951d6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x728951d9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550951e2, SUC_lexicon_tags_12 },
    { 0xf06951e3, SUC_lexicon_tags_8 },
    { 0x8d8951e4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4951e8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304951ec, SUC_lexicon_tags_84 },
    { 0x36e951ec, SUC_lexicon_tags_60 },
    { 0xbfa951ee, SUC_lexicon_tags_25 },
    { 0x85a951ef, SUC_lexicon_tags_27 },
    { 0xbf0951f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe951f3, SUC_lexicon_tags_15 },
    { 0x818951f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b2951f6, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xa7e951f8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2951fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x942951fd, SUC_lexicon_tags_17 },
    { 0xbd6951fd, SUC_lexicon_tags_2 },
    { 0xfc2951fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7695201, SUC_lexicon_tags_519 },
    { 0xe2895201, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23a95204, SUC_lexicon_tags_28 },
    { 0x5e095204, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xea295207, SUC_lexicon_tags_17 },
    { 0x3fa95208, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a9520c, SUC_lexicon_tags_37 },
    { 0x4e29520d, SUC_lexicon_tags_14 },
    { 0xaa9520e, SUC_lexicon_tags_165 },
    { 0xf4a9520e, SUC_lexicon_tags_23 },
    { 0xa9a95210, SUC_lexicon_tags_25 },
    { 0x96a95211, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee295216, SUC_lexicon_tags_763 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a9521a, SUC_lexicon_tags_25 },
    { 0x3fe9521b, SUC_lexicon_tags_25 },
    { 0x4029521c, SUC_lexicon_tags_19 },
    { 0xf8e9521d, SUC_lexicon_tags_7 },
    { 0xafe9521e, SUC_lexicon_tags_19 },
    { 0xba49521c, SUC_lexicon_tags_23 },
    { 0x55e95220, SUC_lexicon_tags_57 },
    { 0x74495221, SUC_lexicon_tags_28 },
    { 0xc729521a, SUC_lexicon_tags_77 },
    { 0xee695223, SUC_lexicon_tags_7 },
    { 0x3009521a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a95228, SUC_lexicon_tags_12 },
    { 0xbf095228, SUC_lexicon_tags_11 },
    { 0xa789522a, SUC_lexicon_tags_7 },
    { 0xb4e9522b, SUC_lexicon_tags_10 },
    { 0xfac9522c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a29522f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53295231, SUC_lexicon_tags_37 },
    { 0xda495232, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3695235, SUC_lexicon_tags_2 },
    { 0x95235, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b495238, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x589523a, SUC_lexicon_tags_17 },
    { 0x1b69523a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ec9523d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74c9523f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3895241, SUC_lexicon_tags_1 },
    { 0x5a295241, SUC_lexicon_tags_31 },
    { 0xcfa95243, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6609524c, SUC_lexicon_tags_3 },
    { 0x5f89524c, SUC_lexicon_tags_13 },
    { 0x9969524e, SUC_lexicon_tags_11 },
    { 0x75c9524d, SUC_lexicon_tags_29 },
    { 0x96a95250, SUC_lexicon_tags_12 },
    { 0x4ea95251, SUC_lexicon_tags_37 },
    { 0x44895252, SUC_lexicon_tags_15 },
    { 0xe0295253, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70095257, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e9525a, SUC_lexicon_tags_1 },
    { 0x3ee9525b, SUC_lexicon_tags_45 },
    { 0xb909525a, SUC_lexicon_tags_12 },
    { 0x7ae9525d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee495260, SUC_lexicon_tags_30 },
    { 0x85095260, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbca95263, SUC_lexicon_tags_37 },
    { 0xe1a95263, SUC_lexicon_tags_2 },
    { 0xac295265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9695268, SUC_lexicon_tags_9 },
    { 0xad895268, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc549526d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c9526f, SUC_lexicon_tags_26 },
    { 0x5b895270, SUC_lexicon_tags_37 },
    { 0x4d895271, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb8a95273, SUC_lexicon_tags_37 },
    { 0xd1095274, SUC_lexicon_tags_25 },
    { 0xb0695274, SUC_lexicon_tags_25 },
    { 0xc3295276, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xba695279, SUC_lexicon_tags_0 },
    { 0x1469527a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4229527e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc495281, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c9528d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19095290, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55695293, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8495295, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xba695298, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb229529b, SUC_lexicon_tags_12 },
    { 0x72a9529c, SUC_lexicon_tags_45 },
    { 0xb4a9529d, SUC_lexicon_tags_6 },
    { 0x189529b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98952a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa1c952a5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c952a8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4952ac, SUC_lexicon_tags_39 },
    { 0xd48952ad, SUC_lexicon_tags_7 },
    { 0x9d8952ae, SUC_lexicon_tags_27 },
    { 0xed0952af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69a952b1, SUC_lexicon_tags_25 },
    { 0x422952b2, SUC_lexicon_tags_17 },
    { 0x5b2952b3, SUC_lexicon_tags_27 },
    { 0xa30952b4, SUC_lexicon_tags_95 },
    { 0xfae952b5, SUC_lexicon_tags_17 },
    { 0x1b0952b5, SUC_lexicon_tags_31 },
    { 0xd92952b2, SUC_lexicon_tags_15 },
    { 0x7d2952b5, SUC_lexicon_tags_17 },
    { 0x930952b5, SUC_lexicon_tags_17 },
    { 0x6c6952ba, SUC_lexicon_tags_37 },
    { 0xdc0952bb, SUC_lexicon_tags_2 },
    { 0xafa952ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940952c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c952c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38952c8, SUC_lexicon_tags_37 },
    { 0x2d0952c9, SUC_lexicon_tags_46 },
    { 0x888952c9, SUC_lexicon_tags_37 },
    { 0xce6952c8, SUC_lexicon_tags_7 },
    { 0xe56952ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefe952ce, SUC_lexicon_tags_27 },
    { 0xe64952cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88952d2, SUC_lexicon_tags_16 },
    { 0x1a4952d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58e952d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca952d9, SUC_lexicon_tags_12 },
    { 0x248952da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x53c952dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55c952de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e952e1, SUC_lexicon_tags_12 },
    { 0x82e952e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x978952e5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40952eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec952f0, SUC_lexicon_tags_43 },
    { 0x91a952f0, SUC_lexicon_tags_7 },
    { 0xb08952f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78952f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba952fb, SUC_lexicon_tags_23 },
    { 0x7d0952fc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x598952fe, SUC_lexicon_tags_47 },
    { 0xfe2952ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f495305, SUC_lexicon_tags_21 },
    { 0xc8c95306, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2095309, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x92e9530b, SUC_lexicon_tags_15 },
    { 0x10e9530c, SUC_lexicon_tags_47 },
    { 0xbc49530b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53695311, SUC_lexicon_tags_12 },
    { 0x3d495311, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da9531a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9089531c, SUC_lexicon_tags_37 },
    { 0x5989531c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1895323, SUC_lexicon_tags_19 },
    { 0x32095324, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41895326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9e95328, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d695330, SUC_lexicon_tags_27 },
    { 0x87c95331, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f295335, SUC_lexicon_tags_2 },
    { 0x63a95336, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e9533c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf069533f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1495341, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe2495343, SUC_lexicon_tags_37 },
    { 0x26895343, SUC_lexicon_tags_17 },
    { 0xe6695345, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c95348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1869534b, SUC_lexicon_tags_11 },
    { 0x1d49534c, SUC_lexicon_tags_1 },
    { 0xa909534d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f295350, SUC_lexicon_tags_37 },
    { 0xa0e95351, SUC_lexicon_tags_1 },
    { 0xa3695352, SUC_lexicon_tags_12 },
    { 0x95e95353, SUC_lexicon_tags_23 },
    { 0x6b295352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e95357, SUC_lexicon_tags_25 },
    { 0x89895358, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a89535b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49535e, SUC_lexicon_tags_2 },
    { 0xc609535f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62c95361, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74095364, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a95367, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8c95369, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84c9536b, SUC_lexicon_tags_0 },
    { 0x9409536c, SUC_lexicon_tags_1 },
    { 0x36c9536d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd895372, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6895377, SUC_lexicon_tags_27 },
    { 0x8fe95378, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7269537a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c9537f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c95383, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83495387, SUC_lexicon_tags_37 },
    { 0x4295388, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea95391, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x99a95393, SUC_lexicon_tags_45 },
    { 0xaee95394, SUC_lexicon_tags_38 },
    { 0x65495395, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706953a6, SUC_lexicon_tags_29 },
    { 0x4a0953a7, SUC_lexicon_tags_26 },
    { 0x506953a8, SUC_lexicon_tags_2 },
    { 0x808953a9, SUC_lexicon_tags_35 },
    { 0x93e953aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a953ae, SUC_lexicon_tags_17 },
    { 0x8c8953af, SUC_lexicon_tags_47 },
    { 0x340953b0, SUC_lexicon_tags_38 },
    { 0x53a953af, SUC_lexicon_tags_38 },
    { 0xa4a953b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ba953b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c953b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb60953b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a953ca, SUC_lexicon_tags_25 },
    { 0x8d0953cb, SUC_lexicon_tags_26 },
    { 0x5b8953cc, SUC_lexicon_tags_25 },
    { 0x1e0953cc, SUC_lexicon_tags_21 },
    { 0xe28953cb, SUC_lexicon_tags_11 },
    { 0x5fa953cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00953d2, SUC_lexicon_tags_15 },
    { 0x664953d3, SUC_lexicon_tags_23 },
    { 0xe52953d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcea953d6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c953d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e953e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6953e7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcc0953e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac953ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x446953ef, SUC_lexicon_tags_29 },
    { 0x962953f0, SUC_lexicon_tags_12 },
    { 0x51a953ef, SUC_lexicon_tags_12 },
    { 0x66c953f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718953f9, SUC_lexicon_tags_41 },
    { 0xcc4953fa, SUC_lexicon_tags_17 },
    { 0x62c953fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0953ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7ba95401, SUC_lexicon_tags_11 },
    { 0x8dc95402, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23095404, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x8095407, SUC_lexicon_tags_1 },
    { 0xc5295408, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe469540c, SUC_lexicon_tags_83 },
    { 0xe209540c, SUC_lexicon_tags_11 },
    { 0xaf09540e, SUC_lexicon_tags_25 },
    { 0x6569540c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e95412, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e95423, SUC_lexicon_tags_15 },
    { 0xbb695424, SUC_lexicon_tags_25 },
    { 0x74095424, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10c95427, SUC_lexicon_tags_27 },
    { 0x9b895427, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee9542b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd209542d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa889542f, SUC_lexicon_tags_33 },
    { 0x50e95430, SUC_lexicon_tags_2 },
    { 0xd295430, SUC_lexicon_tags_12 },
    { 0x77c95432, SUC_lexicon_tags_49 },
    { 0x77295432, SUC_lexicon_tags_14 },
    { 0x35895434, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x49895436, SUC_lexicon_tags_1 },
    { 0x1895437, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf89543b, SUC_lexicon_tags_27 },
    { 0xdd89543c, SUC_lexicon_tags_33 },
    { 0x3549543c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78895441, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c95444, SUC_lexicon_tags_25 },
    { 0x64095445, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x72495447, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a9544a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52695450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5295452, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c95455, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95095457, SUC_lexicon_tags_60 },
    { 0xd9895458, SUC_lexicon_tags_17 },
    { 0xefe95458, SUC_lexicon_tags_11 },
    { 0xa089545a, SUC_lexicon_tags_7 },
    { 0x53e95458, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e95461, SUC_lexicon_tags_19 },
    { 0xf0895462, SUC_lexicon_tags_35 },
    { 0xa1895461, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d29546f, SUC_lexicon_tags_50 },
    { 0x4c495470, SUC_lexicon_tags_21 },
    { 0x6f495471, SUC_lexicon_tags_27 },
    { 0xd1295471, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6e095474, SUC_lexicon_tags_7 },
    { 0x92695475, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78095480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cc95482, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea95487, SUC_lexicon_tags_17 },
    { 0x9e895488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37e9548a, SUC_lexicon_tags_10 },
    { 0x6b49548b, SUC_lexicon_tags_2 },
    { 0x94c9548b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a9548f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9895492, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbc95494, SUC_lexicon_tags_7 },
    { 0x1a095494, SUC_lexicon_tags_1 },
    { 0xaa095496, SUC_lexicon_tags_0 },
    { 0xfd295494, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de9549b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdc29549d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60c9549f, SUC_lexicon_tags_15 },
    { 0x23e954a0, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc954a3, SUC_lexicon_tags_33 },
    { 0xfba954a4, SUC_lexicon_tags_1 },
    { 0x15a954a4, SUC_lexicon_tags_77 },
    { 0x85a954a4, SUC_lexicon_tags_28 },
    { 0x2ce954a5, SUC_lexicon_tags_25 },
    { 0xaa954a6, SUC_lexicon_tags_33 },
    { 0xc954a9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa954ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218954b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4954bc, SUC_lexicon_tags_8 },
    { 0x74c954bd, SUC_lexicon_tags_79 },
    { 0x7e8954be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2a954c0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x102954c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca954c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb38954c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4954cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836954d5, SUC_lexicon_tags_37 },
    { 0x1c0954d6, SUC_lexicon_tags_17 },
    { 0xa7a954d7, SUC_lexicon_tags_9 },
    { 0x1dc954d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a954df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0954e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72954f2, SUC_lexicon_tags_27 },
    { 0x56e954f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4954f6, SUC_lexicon_tags_37 },
    { 0xe2c954f6, SUC_lexicon_tags_12 },
    { 0x6bc954f8, SUC_lexicon_tags_185 },
    { 0xfcc954f6, SUC_lexicon_tags_15 },
    { 0x8a0954f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf82954fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67495504, SUC_lexicon_tags_38 },
    { 0xec495504, SUC_lexicon_tags_15 },
    { 0xb9495506, SUC_lexicon_tags_29 },
    { 0xeee95506, SUC_lexicon_tags_49 },
    { 0x43895508, SUC_lexicon_tags_9 },
    { 0xd8495509, SUC_lexicon_tags_32 },
    { 0x43895506, SUC_lexicon_tags_11 },
    { 0x1c09550b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63e9550d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb495510, SUC_lexicon_tags_55 },
    { 0x31495510, SUC_lexicon_tags_47 },
    { 0xf5e95512, SUC_lexicon_tags_11 },
    { 0x59695513, SUC_lexicon_tags_2 },
    { 0xdec95514, SUC_lexicon_tags_1 },
    { 0xffe95515, SUC_lexicon_tags_15 },
    { 0xe895516, SUC_lexicon_tags_17 },
    { 0x9a895517, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63e95519, SUC_lexicon_tags_25 },
    { 0xb3a95519, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x34c9551c, SUC_lexicon_tags_29 },
    { 0x10c9551d, SUC_lexicon_tags_61 },
    { 0xab89551d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf495521, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3fe95523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdc95525, SUC_lexicon_tags_37 },
    { 0xe9495526, SUC_lexicon_tags_25 },
    { 0x88a95526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c495529, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a9552f, SUC_lexicon_tags_25 },
    { 0x6c89552f, SUC_lexicon_tags_11 },
    { 0xfe95531, SUC_lexicon_tags_37 },
    { 0x1895532, SUC_lexicon_tags_9 },
    { 0xad095533, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e095536, SUC_lexicon_tags_25 },
    { 0xb2695537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd549553e, SUC_lexicon_tags_43 },
    { 0x8029553f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc95543, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae095546, SUC_lexicon_tags_29 },
    { 0x22295546, SUC_lexicon_tags_47 },
    { 0x83a95546, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9554b, SUC_lexicon_tags_12 },
    { 0xd6e9554b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66a9554e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8495551, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9095559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6949555c, SUC_lexicon_tags_37 },
    { 0x3149555c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c95565, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbac95567, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x97895569, SUC_lexicon_tags_28 },
    { 0x7ae9556a, SUC_lexicon_tags_37 },
    { 0xfc69556a, SUC_lexicon_tags_25 },
    { 0xdf49556c, SUC_lexicon_tags_11 },
    { 0x1a89556d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e95570, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a95574, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x85495577, SUC_lexicon_tags_25 },
    { 0x3695578, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa109557a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xad89557c, SUC_lexicon_tags_15 },
    { 0xeb49557d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88895580, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a95583, SUC_lexicon_tags_17 },
    { 0x64295584, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e95589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c29558c, SUC_lexicon_tags_63 },
    { 0x2469558d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee95591, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71295598, SUC_lexicon_tags_33 },
    { 0xa1095599, SUC_lexicon_tags_27 },
    { 0x97e95599, SUC_lexicon_tags_7 },
    { 0xcf895598, SUC_lexicon_tags_28 },
    { 0xcf69559c, SUC_lexicon_tags_1 },
    { 0x9429559b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2955a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e955a6, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xa6955a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe02955aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c955b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca955b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x760955b7, SUC_lexicon_tags_344 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64955bc, SUC_lexicon_tags_17 },
    { 0x768955bd, SUC_lexicon_tags_10 },
    { 0x180955bd, SUC_lexicon_tags_12 },
    { 0xbe6955bf, SUC_lexicon_tags_19 },
    { 0xa80955c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80e955c2, SUC_lexicon_tags_38 },
    { 0x54a955c3, SUC_lexicon_tags_49 },
    { 0xaaa955c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9e955c6, SUC_lexicon_tags_23 },
    { 0x180955c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x530955ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x37c955cc, SUC_lexicon_tags_1 },
    { 0x950955cd, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02955d0, SUC_lexicon_tags_25 },
    { 0x8c8955d1, SUC_lexicon_tags_14 },
    { 0xfd8955d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06955d6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6955da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xada955dd, SUC_lexicon_tags_32 },
    { 0xdee955dd, SUC_lexicon_tags_2 },
    { 0x96e955de, SUC_lexicon_tags_55 },
    { 0xaa6955e0, SUC_lexicon_tags_1 },
    { 0x736955df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4955e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36955ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92955f0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2dc955f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb2955f4, SUC_lexicon_tags_40 },
    { 0x4ce955f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd60955f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a955fa, SUC_lexicon_tags_0 },
    { 0x3a4955fb, SUC_lexicon_tags_27 },
    { 0xa18955fb, SUC_lexicon_tags_126 },
    { 0xb88955fc, SUC_lexicon_tags_61 },
    { 0x100955fe, SUC_lexicon_tags_14 },
    { 0x5cc955fb, SUC_lexicon_tags_30 },
    { 0xf6955ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44895602, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc2095604, SUC_lexicon_tags_39 },
    { 0x8ac95605, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x18095608, SUC_lexicon_tags_77 },
    { 0x7ae95608, SUC_lexicon_tags_2 },
    { 0xf5c95608, SUC_lexicon_tags_19 },
    { 0x7ac9560b, SUC_lexicon_tags_34 },
    { 0x68c9560b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc349560f, SUC_lexicon_tags_12 },
    { 0xaae95610, SUC_lexicon_tags_11 },
    { 0x4a495611, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b695616, SUC_lexicon_tags_11 },
    { 0x83695616, SUC_lexicon_tags_7 },
    { 0x73a95618, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f29561c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1b89561e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5295620, SUC_lexicon_tags_19 },
    { 0x33095621, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67695625, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b895628, SUC_lexicon_tags_1 },
    { 0x34095628, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa609562c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7ec9562e, SUC_lexicon_tags_14 },
    { 0x1769562f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f095636, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e9563a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4095640, SUC_lexicon_tags_29 },
    { 0xa9a95641, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42495644, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac95648, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce9564d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x41c9564f, SUC_lexicon_tags_62 },
    { 0xbf49564f, SUC_lexicon_tags_39 },
    { 0xed29564f, SUC_lexicon_tags_23 },
    { 0x55a95652, SUC_lexicon_tags_37 },
    { 0x63495653, SUC_lexicon_tags_48 },
    { 0x5f095654, SUC_lexicon_tags_42 },
    { 0x5a895655, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d09565d, SUC_lexicon_tags_31 },
    { 0x1249565e, SUC_lexicon_tags_17 },
    { 0x389565d, SUC_lexicon_tags_45 },
    { 0xf695660, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5295662, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8495666, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca9566b, SUC_lexicon_tags_10 },
    { 0xcae9566c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe949566f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1c95671, SUC_lexicon_tags_11 },
    { 0xec495671, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5aa95674, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb0895676, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd389567b, SUC_lexicon_tags_1 },
    { 0xbb89567b, SUC_lexicon_tags_14 },
    { 0x4969567d, SUC_lexicon_tags_87 },
    { 0x1729567e, SUC_lexicon_tags_15 },
    { 0xbc09567e, SUC_lexicon_tags_10 },
    { 0xd3c95680, SUC_lexicon_tags_25 },
    { 0x6ce95680, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9495685, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa8495687, SUC_lexicon_tags_25 },
    { 0xe2095687, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6909568c, SUC_lexicon_tags_25 },
    { 0xc8a9568d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f695692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa95695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4949569e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x200956a0, SUC_lexicon_tags_25 },
    { 0x456956a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6956a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90956a8, SUC_lexicon_tags_25 },
    { 0xc62956a8, SUC_lexicon_tags_15 },
    { 0x18956a8, SUC_lexicon_tags_70 },
    { 0x7e8956aa, SUC_lexicon_tags_25 },
    { 0xc3e956a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd3c956ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8956b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86956b4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662956b9, SUC_lexicon_tags_2 },
    { 0x29e956ba, SUC_lexicon_tags_2 },
    { 0xc9a956bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc956c2, SUC_lexicon_tags_25 },
    { 0x250956c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfc8956c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8956c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x222956ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc956cd, SUC_lexicon_tags_1333 },
    { 0, NULL },
    { 0x344956cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d6956d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e956d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5c956d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bc956d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa956df, SUC_lexicon_tags_15 },
    { 0xe12956e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a956e4, SUC_lexicon_tags_47 },
    { 0x5c6956e5, SUC_lexicon_tags_25 },
    { 0x44e956e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a956ea, SUC_lexicon_tags_21 },
    { 0xa1c956eb, SUC_lexicon_tags_30 },
    { 0x5d2956eb, SUC_lexicon_tags_15 },
    { 0xb04956ed, SUC_lexicon_tags_47 },
    { 0x3a2956ee, SUC_lexicon_tags_15 },
    { 0xa76956ed, SUC_lexicon_tags_2 },
    { 0x8c956f0, SUC_lexicon_tags_17 },
    { 0xd9c956f1, SUC_lexicon_tags_40 },
    { 0x5ea956f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e956f7, SUC_lexicon_tags_10 },
    { 0xb70956f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x978956fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xece956fe, SUC_lexicon_tags_27 },
    { 0x2de956ff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb1e95701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91e95703, SUC_lexicon_tags_47 },
    { 0xc9a95704, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c95708, SUC_lexicon_tags_11 },
    { 0x51095709, SUC_lexicon_tags_17 },
    { 0x1449570a, SUC_lexicon_tags_37 },
    { 0xd2a95709, SUC_lexicon_tags_12 },
    { 0xdee95708, SUC_lexicon_tags_12 },
    { 0x4e29570d, SUC_lexicon_tags_8 },
    { 0x45c9570e, SUC_lexicon_tags_12 },
    { 0x3a49570f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2895713, SUC_lexicon_tags_7 },
    { 0x9ec95714, SUC_lexicon_tags_63 },
    { 0xae695713, SUC_lexicon_tags_56 },
    { 0xebc95716, SUC_lexicon_tags_17 },
    { 0xd5e95717, SUC_lexicon_tags_15 },
    { 0x8b895714, SUC_lexicon_tags_1 },
    { 0x10e95719, SUC_lexicon_tags_2 },
    { 0x61c9571a, SUC_lexicon_tags_32 },
    { 0x1fc9571b, SUC_lexicon_tags_11 },
    { 0x70e95719, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56a9571e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13c95720, SUC_lexicon_tags_9 },
    { 0x48295721, SUC_lexicon_tags_23 },
    { 0xa7a95722, SUC_lexicon_tags_17 },
    { 0x15c95722, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89695725, SUC_lexicon_tags_29 },
    { 0xce495725, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a9572f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61695737, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc95739, SUC_lexicon_tags_7 },
    { 0x13a95739, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa9573f, SUC_lexicon_tags_49 },
    { 0x2495740, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c95743, SUC_lexicon_tags_41 },
    { 0x1ea95743, SUC_lexicon_tags_11 },
    { 0x12a95744, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be95748, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac9574b, SUC_lexicon_tags_47 },
    { 0x12a9574c, SUC_lexicon_tags_51 },
    { 0xa309574d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa309574f, SUC_lexicon_tags_38 },
    { 0x67e9574f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4095757, SUC_lexicon_tags_61 },
    { 0x38895757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd369575a, SUC_lexicon_tags_23 },
    { 0x8229575b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c89575d, SUC_lexicon_tags_15 },
    { 0x8aa9575e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a95761, SUC_lexicon_tags_27 },
    { 0xd3c95762, SUC_lexicon_tags_37 },
    { 0xddc95762, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0495766, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xc2295768, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1895773, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a69577c, SUC_lexicon_tags_17 },
    { 0x5809577d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24895783, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede95787, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb789578a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc949578c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb095790, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0895796, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2895799, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d09579e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78957a3, SUC_lexicon_tags_0 },
    { 0x81e957a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4d8957a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a957a9, SUC_lexicon_tags_15 },
    { 0xda4957aa, SUC_lexicon_tags_49 },
    { 0x738957aa, SUC_lexicon_tags_17 },
    { 0xa2e957aa, SUC_lexicon_tags_0 },
    { 0x95a957ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa8e957af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe957b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e957bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e957c3, SUC_lexicon_tags_23 },
    { 0xede957c4, SUC_lexicon_tags_68 },
    { 0xfe2957c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a957cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94e957cd, SUC_lexicon_tags_63 },
    { 0x766957cd, SUC_lexicon_tags_3 },
    { 0xa5a957cd, SUC_lexicon_tags_47 },
    { 0x5b2957d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e957d9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x11a957dc, SUC_lexicon_tags_1 },
    { 0xf1c957dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548957e1, SUC_lexicon_tags_37 },
    { 0xf26957e2, SUC_lexicon_tags_2 },
    { 0x48e957e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6957e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508957f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45e957f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54e957f6, SUC_lexicon_tags_12 },
    { 0x5ba957f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6a957f9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae957fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x438957ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5095805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c95809, SUC_lexicon_tags_63 },
    { 0x50a95809, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e9580e, SUC_lexicon_tags_25 },
    { 0x1e69580f, SUC_lexicon_tags_0 },
    { 0x19695810, SUC_lexicon_tags_11 },
    { 0x5e495810, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd095813, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38e9581b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c95822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53495826, SUC_lexicon_tags_12 },
    { 0xe5a95827, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xde095829, SUC_lexicon_tags_15 },
    { 0x6d295829, SUC_lexicon_tags_8 },
    { 0x4309582b, SUC_lexicon_tags_25 },
    { 0x7ec9582c, SUC_lexicon_tags_1 },
    { 0xf49582c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a89582f, SUC_lexicon_tags_55 },
    { 0x2c095830, SUC_lexicon_tags_14 },
    { 0x5aa95830, SUC_lexicon_tags_10 },
    { 0x2de9582f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf695834, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30295836, SUC_lexicon_tags_47 },
    { 0x43695837, SUC_lexicon_tags_1 },
    { 0x53c95838, SUC_lexicon_tags_14 },
    { 0xda695839, SUC_lexicon_tags_12 },
    { 0x2bc95839, SUC_lexicon_tags_45 },
    { 0xcbe95837, SUC_lexicon_tags_95 },
    { 0x7869583c, SUC_lexicon_tags_15 },
    { 0xd329583d, SUC_lexicon_tags_378 },
    { 0x2ce9583c, SUC_lexicon_tags_15 },
    { 0x9769583e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c95843, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad295846, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce895848, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x57a9584a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b29584d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d895851, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7c695853, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4895857, SUC_lexicon_tags_12 },
    { 0xaaa95857, SUC_lexicon_tags_15 },
    { 0x16895858, SUC_lexicon_tags_63 },
    { 0x5d69585a, SUC_lexicon_tags_2 },
    { 0x6ae9585b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6149585f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a695863, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x34495866, SUC_lexicon_tags_34 },
    { 0x23695866, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75295869, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb09586e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4095876, SUC_lexicon_tags_47 },
    { 0x29695877, SUC_lexicon_tags_6 },
    { 0xbfa95877, SUC_lexicon_tags_25 },
    { 0x61095878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b29587c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8de9587e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x70495881, SUC_lexicon_tags_30 },
    { 0x4e495882, SUC_lexicon_tags_7 },
    { 0x21e95882, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd495889, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee9588c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5895894, SUC_lexicon_tags_24 },
    { 0x14095894, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a95898, SUC_lexicon_tags_15 },
    { 0xf3495898, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6589589f, SUC_lexicon_tags_7 },
    { 0x872958a0, SUC_lexicon_tags_2 },
    { 0xfe49589f, SUC_lexicon_tags_2 },
    { 0xfea9589f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8958a5, SUC_lexicon_tags_38 },
    { 0x4aa958a5, SUC_lexicon_tags_10 },
    { 0xba0958a7, SUC_lexicon_tags_25 },
    { 0x57e958a8, SUC_lexicon_tags_19 },
    { 0xfae958a9, SUC_lexicon_tags_1 },
    { 0x5c6958a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e958ae, SUC_lexicon_tags_38 },
    { 0x1a2958af, SUC_lexicon_tags_55 },
    { 0xfbc958b0, SUC_lexicon_tags_15 },
    { 0x6c6958b1, SUC_lexicon_tags_37 },
    { 0x434958ae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62958b5, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x1ba958b7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284958bb, SUC_lexicon_tags_17 },
    { 0xb66958bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8958bf, SUC_lexicon_tags_60 },
    { 0x916958c0, SUC_lexicon_tags_17 },
    { 0x584958bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514958c6, SUC_lexicon_tags_25 },
    { 0x564958c6, SUC_lexicon_tags_39 },
    { 0x958c8, SUC_lexicon_tags_15 },
    { 0x91c958c7, SUC_lexicon_tags_62 },
    { 0xea0958ca, SUC_lexicon_tags_17 },
    { 0xae958c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772958d2, SUC_lexicon_tags_41 },
    { 0xbf2958d3, SUC_lexicon_tags_11 },
    { 0x50e958d2, SUC_lexicon_tags_1 },
    { 0xc60958d5, SUC_lexicon_tags_19 },
    { 0x18c958d5, SUC_lexicon_tags_3 },
    { 0x46e958d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a958db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x596958dd, SUC_lexicon_tags_1 },
    { 0x17e958de, SUC_lexicon_tags_2 },
    { 0x6e958dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736958e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b0958e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198958eb, SUC_lexicon_tags_7 },
    { 0x650958ec, SUC_lexicon_tags_1 },
    { 0x640958ed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x214958f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe4958f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca958f4, SUC_lexicon_tags_63 },
    { 0xbea958f4, SUC_lexicon_tags_7 },
    { 0xc2e958f4, SUC_lexicon_tags_19 },
    { 0xfa8958f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866958fb, SUC_lexicon_tags_65 },
    { 0x73e958fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x860958fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83295900, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e95905, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca69590c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b69590e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcda95910, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe95918, SUC_lexicon_tags_37 },
    { 0x18095919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a69591b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77095921, SUC_lexicon_tags_47 },
    { 0xcce95922, SUC_lexicon_tags_56 },
    { 0x295922, SUC_lexicon_tags_8 },
    { 0xb2a95924, SUC_lexicon_tags_10 },
    { 0xa0095925, SUC_lexicon_tags_63 },
    { 0x6de95926, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b295929, SUC_lexicon_tags_45 },
    { 0xabc9592a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a29592d, SUC_lexicon_tags_2 },
    { 0x3949592d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58095932, SUC_lexicon_tags_25 },
    { 0xaa895933, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc95937, SUC_lexicon_tags_46 },
    { 0x79a95938, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee95940, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc495944, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b095946, SUC_lexicon_tags_12 },
    { 0xd3095947, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d29594e, SUC_lexicon_tags_18 },
    { 0x9c49594e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64695953, SUC_lexicon_tags_15 },
    { 0x45a95954, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1c95956, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2a95958, SUC_lexicon_tags_27 },
    { 0x7e95959, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a9595c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd489595f, SUC_lexicon_tags_60 },
    { 0xbe9595f, SUC_lexicon_tags_16 },
    { 0xce695961, SUC_lexicon_tags_47 },
    { 0xc3895962, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16895966, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c69596c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x16a9596e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe095970, SUC_lexicon_tags_1 },
    { 0x4aa95971, SUC_lexicon_tags_19 },
    { 0x43695972, SUC_lexicon_tags_11 },
    { 0xf5095971, SUC_lexicon_tags_14 },
    { 0x70a95974, SUC_lexicon_tags_63 },
    { 0xc8295971, SUC_lexicon_tags_27 },
    { 0xb2c95976, SUC_lexicon_tags_12 },
    { 0xe5495977, SUC_lexicon_tags_160 },
    { 0xc6895978, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ba9597a, SUC_lexicon_tags_11 },
    { 0xafc9597b, SUC_lexicon_tags_7 },
    { 0xd949597b, SUC_lexicon_tags_9 },
    { 0xaf69597d, SUC_lexicon_tags_49 },
    { 0x3ce9597d, SUC_lexicon_tags_38 },
    { 0x43c9597e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca95982, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a95985, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a095988, SUC_lexicon_tags_11 },
    { 0xd695989, SUC_lexicon_tags_11 },
    { 0x8889598a, SUC_lexicon_tags_4 },
    { 0x8c69598b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a9598e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9295990, SUC_lexicon_tags_38 },
    { 0x35e95990, SUC_lexicon_tags_11 },
    { 0x54895992, SUC_lexicon_tags_2 },
    { 0x53495991, SUC_lexicon_tags_15 },
    { 0x39e95994, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1f895996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0295999, SUC_lexicon_tags_3 },
    { 0x10495999, SUC_lexicon_tags_66 },
    { 0x9409599b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8829599f, SUC_lexicon_tags_30 },
    { 0xd36959a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea959a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2c959a5, SUC_lexicon_tags_14 },
    { 0x318959a6, SUC_lexicon_tags_83 },
    { 0x6e8959a7, SUC_lexicon_tags_7 },
    { 0x28c959a7, SUC_lexicon_tags_19 },
    { 0xe92959a7, SUC_lexicon_tags_8 },
    { 0x7a4959a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562959ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6959b3, SUC_lexicon_tags_19 },
    { 0x280959b4, SUC_lexicon_tags_37 },
    { 0x530959b4, SUC_lexicon_tags_9 },
    { 0x946959b5, SUC_lexicon_tags_12 },
    { 0x14e959b7, SUC_lexicon_tags_955 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54959c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c959c6, SUC_lexicon_tags_17 },
    { 0x4dc959c6, SUC_lexicon_tags_46 },
    { 0x590959c8, SUC_lexicon_tags_37 },
    { 0xf4a959c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc959ce, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc959d4, SUC_lexicon_tags_34 },
    { 0xaa2959d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0959da, SUC_lexicon_tags_60 },
    { 0x440959db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x798959dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f8959df, SUC_lexicon_tags_25 },
    { 0x8f4959df, SUC_lexicon_tags_2 },
    { 0xfc8959e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24959e3, SUC_lexicon_tags_19 },
    { 0x70e959e4, SUC_lexicon_tags_1 },
    { 0x8be959e5, SUC_lexicon_tags_11 },
    { 0xd1c959e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96959e9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a959ef, SUC_lexicon_tags_37 },
    { 0xb7a959f0, SUC_lexicon_tags_37 },
    { 0xa6c959f1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x27c959f3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e959f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa959fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4959fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13695a00, SUC_lexicon_tags_114 },
    { 0x93a95a01, SUC_lexicon_tags_37 },
    { 0x45295a02, SUC_lexicon_tags_11 },
    { 0x8fe95a03, SUC_lexicon_tags_47 },
    { 0x38095a01, SUC_lexicon_tags_28 },
    { 0x83495a05, SUC_lexicon_tags_7 },
    { 0xb7e95a05, SUC_lexicon_tags_47 },
    { 0xed895a07, SUC_lexicon_tags_25 },
    { 0x84e95a06, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcd695a0a, SUC_lexicon_tags_0 },
    { 0x33a95a0b, SUC_lexicon_tags_42 },
    { 0xbc495a0c, SUC_lexicon_tags_55 },
    { 0x76495a0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9695a0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f895a11, SUC_lexicon_tags_68 },
    { 0x3f695a12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2695a15, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x78e95a17, SUC_lexicon_tags_31 },
    { 0x75095a18, SUC_lexicon_tags_7 },
    { 0x26c95a17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97e95a1b, SUC_lexicon_tags_84 },
    { 0xa6695a1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81e95a1e, SUC_lexicon_tags_47 },
    { 0x7e095a1f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e95a28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d095a2a, SUC_lexicon_tags_33 },
    { 0x97895a2b, SUC_lexicon_tags_17 },
    { 0x75c95a2c, SUC_lexicon_tags_60 },
    { 0x52c95a2c, SUC_lexicon_tags_37 },
    { 0xba95a2c, SUC_lexicon_tags_61 },
    { 0xdfe95a2f, SUC_lexicon_tags_68 },
    { 0x45695a2f, SUC_lexicon_tags_8 },
    { 0xe3095a2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e95a34, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x13a95a36, SUC_lexicon_tags_24 },
    { 0xb2895a37, SUC_lexicon_tags_27 },
    { 0xf3e95a37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4895a3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x42295a3d, SUC_lexicon_tags_15 },
    { 0xd2e95a3e, SUC_lexicon_tags_25 },
    { 0x74295a3e, SUC_lexicon_tags_37 },
    { 0x8c695a40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a95a45, SUC_lexicon_tags_37 },
    { 0xeb495a46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1a95a48, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc95a4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba95a4f, SUC_lexicon_tags_0 },
    { 0x6ae95a4f, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba895a55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1695a5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e095a5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62a95a5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8695a62, SUC_lexicon_tags_63 },
    { 0xa4a95a63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47695a67, SUC_lexicon_tags_7 },
    { 0xd5095a68, SUC_lexicon_tags_37 },
    { 0x4aa95a69, SUC_lexicon_tags_1 },
    { 0xfc95a67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f895a6e, SUC_lexicon_tags_15 },
    { 0x9a095a6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35295a74, SUC_lexicon_tags_0 },
    { 0x9a895a75, SUC_lexicon_tags_33 },
    { 0xe7c95a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a495a7c, SUC_lexicon_tags_31 },
    { 0x97895a7d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc095a83, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc695a86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa895a90, SUC_lexicon_tags_34 },
    { 0xfc695a90, SUC_lexicon_tags_17 },
    { 0xe4c95a92, SUC_lexicon_tags_30 },
    { 0x69a95a91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3295a9c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ec95a9e, SUC_lexicon_tags_47 },
    { 0x73495a9e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b695aa3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5895aa8, SUC_lexicon_tags_37 },
    { 0x4095aa9, SUC_lexicon_tags_28 },
    { 0x3695aaa, SUC_lexicon_tags_38 },
    { 0xf1095aaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d895ab0, SUC_lexicon_tags_11 },
    { 0xa1c95ab1, SUC_lexicon_tags_12 },
    { 0x2b695ab2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe95ab5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa3095ab7, SUC_lexicon_tags_27 },
    { 0xdb895ab8, SUC_lexicon_tags_60 },
    { 0x5c95ab9, SUC_lexicon_tags_105 },
    { 0xa0295ab8, SUC_lexicon_tags_17 },
    { 0xac695abb, SUC_lexicon_tags_8 },
    { 0x48495abb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5295ac6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a495ace, SUC_lexicon_tags_68 },
    { 0xb9095ace, SUC_lexicon_tags_25 },
    { 0x58c95ad0, SUC_lexicon_tags_7 },
    { 0x86895ad1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a95add, SUC_lexicon_tags_37 },
    { 0xb0c95ade, SUC_lexicon_tags_15 },
    { 0x3695adf, SUC_lexicon_tags_0 },
    { 0xb2c95ae0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac95ae3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47095ae6, SUC_lexicon_tags_77 },
    { 0x47695ae7, SUC_lexicon_tags_17 },
    { 0x22495ae8, SUC_lexicon_tags_25 },
    { 0xc4295ae9, SUC_lexicon_tags_6 },
    { 0xed695ae8, SUC_lexicon_tags_0 },
    { 0x83495ae7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6095af0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24095af8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c495afc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e895b01, SUC_lexicon_tags_12 },
    { 0x60895b02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1895b06, SUC_lexicon_tags_23 },
    { 0xf9295b07, SUC_lexicon_tags_27 },
    { 0xc8295b07, SUC_lexicon_tags_25 },
    { 0xe2e95b08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9695b0c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c95b0f, SUC_lexicon_tags_2 },
    { 0x55895b10, SUC_lexicon_tags_47 },
    { 0x6b895b0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72295b16, SUC_lexicon_tags_15 },
    { 0x82295b17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89a95b19, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76095b1d, SUC_lexicon_tags_25 },
    { 0xb1295b1e, SUC_lexicon_tags_31 },
    { 0xb9e95b1f, SUC_lexicon_tags_16 },
    { 0x7f295b1f, SUC_lexicon_tags_49 },
    { 0x39695b21, SUC_lexicon_tags_25 },
    { 0x27095b21, SUC_lexicon_tags_47 },
    { 0xf3095b23, SUC_lexicon_tags_2 },
    { 0x30a95b24, SUC_lexicon_tags_0 },
    { 0xa5695b25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35695b27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29c95b2b, SUC_lexicon_tags_11 },
    { 0xa3695b2c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c295b30, SUC_lexicon_tags_25 },
    { 0x17095b30, SUC_lexicon_tags_25 },
    { 0xe7695b31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69095b34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5095b39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8095b3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c95b40, SUC_lexicon_tags_12 },
    { 0x77c95b41, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e95b44, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd5695b46, SUC_lexicon_tags_1 },
    { 0xf6895b46, SUC_lexicon_tags_42 },
    { 0x94095b47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc895b4a, SUC_lexicon_tags_6 },
    { 0x68695b4b, SUC_lexicon_tags_17 },
    { 0xf695b4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc9095b4e, SUC_lexicon_tags_0 },
    { 0x86295b4f, SUC_lexicon_tags_11 },
    { 0x13e95b50, SUC_lexicon_tags_46 },
    { 0xa1c95b51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3695b53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86695b59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe495b5d, SUC_lexicon_tags_30 },
    { 0x89295b5d, SUC_lexicon_tags_33 },
    { 0xe0e95b5d, SUC_lexicon_tags_2 },
    { 0xc1095b60, SUC_lexicon_tags_29 },
    { 0x81695b5d, SUC_lexicon_tags_26 },
    { 0xfd895b62, SUC_lexicon_tags_12 },
    { 0xa5495b63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53495b69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4c95b6b, SUC_lexicon_tags_37 },
    { 0xb6895b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3be95b6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7c95b70, SUC_lexicon_tags_37 },
    { 0xa4095b71, SUC_lexicon_tags_7 },
    { 0xf4895b70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3bc95b74, SUC_lexicon_tags_13 },
    { 0xffc95b74, SUC_lexicon_tags_7 },
    { 0xce95b74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea695b79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be95b85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c95b88, SUC_lexicon_tags_46 },
    { 0xf3895b88, SUC_lexicon_tags_33 },
    { 0x50c95b89, SUC_lexicon_tags_265 },
    { 0xf0895b8b, SUC_lexicon_tags_37 },
    { 0x44a95b8c, SUC_lexicon_tags_23 },
    { 0x24a95b8d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x9ea95b8f, SUC_lexicon_tags_77 },
    { 0x7c495b8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33c95b92, SUC_lexicon_tags_12 },
    { 0x5ec95b93, SUC_lexicon_tags_25 },
    { 0xa4e95b94, SUC_lexicon_tags_17 },
    { 0x47295b94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa0095b97, SUC_lexicon_tags_11 },
    { 0x5fe95b97, SUC_lexicon_tags_12 },
    { 0x32c95b97, SUC_lexicon_tags_12 },
    { 0xc1695b97, SUC_lexicon_tags_17 },
    { 0x59695b9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be95b9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97095ba3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1495ba6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x33095ba8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe0295baa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96495baf, SUC_lexicon_tags_12 },
    { 0xa1895baf, SUC_lexicon_tags_19 },
    { 0xd6a95baf, SUC_lexicon_tags_9 },
    { 0x4e695bb2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdd295bb4, SUC_lexicon_tags_12 },
    { 0x34295bb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x93495bb8, SUC_lexicon_tags_47 },
    { 0xb0495bb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5695bbd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a95bc1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x10e95bc3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79095bcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21e95bcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4a95bcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a895bd4, SUC_lexicon_tags_11 },
    { 0x7f295bd4, SUC_lexicon_tags_1 },
    { 0xa7e95bd6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0895bd9, SUC_lexicon_tags_14 },
    { 0x98295bda, SUC_lexicon_tags_259 },
    { 0x75095bd9, SUC_lexicon_tags_14 },
    { 0xb7895bdc, SUC_lexicon_tags_0 },
    { 0x53c95bdc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x54695bdf, SUC_lexicon_tags_6 },
    { 0xb6c95be0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95495be7, SUC_lexicon_tags_46 },
    { 0x54295be8, SUC_lexicon_tags_30 },
    { 0xece95be8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x43695bec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6095bee, SUC_lexicon_tags_63 },
    { 0xc2e95bef, SUC_lexicon_tags_1 },
    { 0x1b495bf0, SUC_lexicon_tags_43 },
    { 0x8d695bee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84695bf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77895bf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70295bfc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x90a95bfe, SUC_lexicon_tags_262 },
    { 0x22c95bfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce95c02, SUC_lexicon_tags_19 },
    { 0x40c95c02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d295c0a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c95c0e, SUC_lexicon_tags_37 },
    { 0xc0c95c0e, SUC_lexicon_tags_37 },
    { 0x55895c0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a295c14, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c495c18, SUC_lexicon_tags_25 },
    { 0x62095c19, SUC_lexicon_tags_63 },
    { 0xe3a95c1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75495c1d, SUC_lexicon_tags_77 },
    { 0x3e895c1d, SUC_lexicon_tags_63 },
    { 0x40495c1f, SUC_lexicon_tags_1 },
    { 0xd1695c20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3895c22, SUC_lexicon_tags_47 },
    { 0x1b095c23, SUC_lexicon_tags_63 },
    { 0x90095c23, SUC_lexicon_tags_84 },
    { 0x23895c25, SUC_lexicon_tags_10 },
    { 0xeb695c26, SUC_lexicon_tags_25 },
    { 0xd8a95c27, SUC_lexicon_tags_37 },
    { 0x3c495c28, SUC_lexicon_tags_12 },
    { 0x8ba95c29, SUC_lexicon_tags_1 },
    { 0x12295c26, SUC_lexicon_tags_48 },
    { 0x1fc95c22, SUC_lexicon_tags_0 },
    { 0x65295c24, SUC_lexicon_tags_37 },
    { 0xdc695c27, SUC_lexicon_tags_23 },
    { 0xeb695c2e, SUC_lexicon_tags_3 },
    { 0x81295c23, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca95c38, SUC_lexicon_tags_6 },
    { 0x5c095c39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec95c3c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1095c40, SUC_lexicon_tags_15 },
    { 0x73295c41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3895c44, SUC_lexicon_tags_1 },
    { 0x34695c44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ea95c47, SUC_lexicon_tags_37 },
    { 0x90a95c48, SUC_lexicon_tags_6 },
    { 0x1f895c49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fc95c4b, SUC_lexicon_tags_37 },
    { 0xe5695c4c, SUC_lexicon_tags_77 },
    { 0x25295c4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec495c4f, SUC_lexicon_tags_1 },
    { 0x3c495c50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70095c53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30895c5c, SUC_lexicon_tags_8 },
    { 0x43e95c5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4295c5f, SUC_lexicon_tags_61 },
    { 0x8295c60, SUC_lexicon_tags_64 },
    { 0x9ae95c61, SUC_lexicon_tags_6 },
    { 0x82895c61, SUC_lexicon_tags_2 },
    { 0x2f095c5f, SUC_lexicon_tags_37 },
    { 0x80e95c60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a95c6a, SUC_lexicon_tags_17 },
    { 0x88695c6b, SUC_lexicon_tags_23 },
    { 0xff695c6c, SUC_lexicon_tags_68 },
    { 0x7f895c6d, SUC_lexicon_tags_45 },
    { 0x58c95c6e, SUC_lexicon_tags_1 },
    { 0xa2295c6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7095c79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x79895c7b, SUC_lexicon_tags_37 },
    { 0x36895c7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee95c7f, SUC_lexicon_tags_17 },
    { 0xc1295c7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca295c83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x23295c85, SUC_lexicon_tags_509 },
    { 0, NULL },
    { 0x58a95c87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4c695c89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9095c8d, SUC_lexicon_tags_12 },
    { 0x77c95c8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a95c91, SUC_lexicon_tags_37 },
    { 0xa7295c92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee95c97, SUC_lexicon_tags_39 },
    { 0xd8295c97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x13c95c9a, SUC_lexicon_tags_82 },
    { 0xef695c9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66895ca0, SUC_lexicon_tags_2 },
    { 0x71a95ca1, SUC_lexicon_tags_37 },
    { 0x29e95ca2, SUC_lexicon_tags_21 },
    { 0x98c95ca3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee95ca6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95695ca8, SUC_lexicon_tags_28 },
    { 0x9a495ca9, SUC_lexicon_tags_19 },
    { 0xa6c95caa, SUC_lexicon_tags_19 },
    { 0x21a95cab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x41695cad, SUC_lexicon_tags_44 },
    { 0xa9c95cae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a895cb0, SUC_lexicon_tags_46 },
    { 0x54e95cb0, SUC_lexicon_tags_7 },
    { 0x5da95cb0, SUC_lexicon_tags_45 },
    { 0x6b895cb3, SUC_lexicon_tags_82 },
    { 0x12095cb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43095cbb, SUC_lexicon_tags_14 },
    { 0xad495cbc, SUC_lexicon_tags_47 },
    { 0x70095cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46295cc1, SUC_lexicon_tags_63 },
    { 0x29095cc1, SUC_lexicon_tags_17 },
    { 0x6c95cc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec95cc5, SUC_lexicon_tags_1 },
    { 0xa7695cc5, SUC_lexicon_tags_49 },
    { 0x99495cc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28695cc9, SUC_lexicon_tags_25 },
    { 0x3ea95cca, SUC_lexicon_tags_0 },
    { 0xb8095ccb, SUC_lexicon_tags_10 },
    { 0x8695ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe95cd4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca95cd7, SUC_lexicon_tags_25 },
    { 0x2d295cd8, SUC_lexicon_tags_11 },
    { 0xce95cd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xabe95cdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a95ce1, SUC_lexicon_tags_12 },
    { 0x8ee95ce1, SUC_lexicon_tags_11 },
    { 0x9ca95ce2, SUC_lexicon_tags_61 },
    { 0x43c95ce4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x63295ce7, SUC_lexicon_tags_7 },
    { 0x8e495ce8, SUC_lexicon_tags_17 },
    { 0x94a95ce9, SUC_lexicon_tags_12 },
    { 0x9ce95ce8, SUC_lexicon_tags_23 },
    { 0xd9295ceb, SUC_lexicon_tags_11 },
    { 0xeac95cec, SUC_lexicon_tags_29 },
    { 0x94095ceb, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5495cf2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7895cf8, SUC_lexicon_tags_7 },
    { 0x82095cf8, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xed495cfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd295cfd, SUC_lexicon_tags_1 },
    { 0x52895cfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa95d01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7095d03, SUC_lexicon_tags_11 },
    { 0x25295d03, SUC_lexicon_tags_38 },
    { 0xbb895d05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa95d09, SUC_lexicon_tags_11 },
    { 0x4ee95d09, SUC_lexicon_tags_2 },
    { 0xbfc95d0b, SUC_lexicon_tags_75 },
    { 0x2c295d09, SUC_lexicon_tags_47 },
    { 0x34895d0c, SUC_lexicon_tags_107 },
    { 0xd4295d0e, SUC_lexicon_tags_48 },
    { 0xb5295d0f, SUC_lexicon_tags_65 },
    { 0x60295d0f, SUC_lexicon_tags_21 },
    { 0xe3695d11, SUC_lexicon_tags_37 },
    { 0x81495d12, SUC_lexicon_tags_37 },
    { 0xd4895d13, SUC_lexicon_tags_1 },
    { 0x2bc95d14, SUC_lexicon_tags_38 },
    { 0x70c95d13, SUC_lexicon_tags_1 },
    { 0xefc95d13, SUC_lexicon_tags_32 },
    { 0x2f895d17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62095d19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f895d1c, SUC_lexicon_tags_17 },
    { 0xd0e95d1d, SUC_lexicon_tags_34 },
    { 0x1dc95d1e, SUC_lexicon_tags_11 },
    { 0xe3895d1f, SUC_lexicon_tags_2 },
    { 0x39a95d20, SUC_lexicon_tags_21 },
    { 0x8a095d21, SUC_lexicon_tags_19 },
    { 0xf6e95d22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e95d25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17695d27, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6695d39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42695d3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e95d42, SUC_lexicon_tags_15 },
    { 0x97495d43, SUC_lexicon_tags_10 },
    { 0x1e095d44, SUC_lexicon_tags_25 },
    { 0x22e95d43, SUC_lexicon_tags_37 },
    { 0x25295d44, SUC_lexicon_tags_12 },
    { 0x4a095d47, SUC_lexicon_tags_37 },
    { 0x7f695d48, SUC_lexicon_tags_19 },
    { 0xa7295d45, SUC_lexicon_tags_12 },
    { 0x81a95d4a, SUC_lexicon_tags_11 },
    { 0xbea95d49, SUC_lexicon_tags_66 },
    { 0x8a95d44, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95895d53, SUC_lexicon_tags_79 },
    { 0x21c95d54, SUC_lexicon_tags_65 },
    { 0x1cc95d55, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84095d59, SUC_lexicon_tags_7 },
    { 0xc1a95d59, SUC_lexicon_tags_0 },
    { 0xbd495d5b, SUC_lexicon_tags_37 },
    { 0xcfe95d5c, SUC_lexicon_tags_17 },
    { 0x8d895d5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x71495d60, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1695d63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96a95d65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea895d69, SUC_lexicon_tags_15 },
    { 0x8095d6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd695d6c, SUC_lexicon_tags_0 },
    { 0xaa95d6c, SUC_lexicon_tags_23 },
    { 0xc3295d6c, SUC_lexicon_tags_1 },
    { 0x45a95d6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc695d71, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe8295d73, SUC_lexicon_tags_1 },
    { 0x1c095d74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3695d78, SUC_lexicon_tags_11 },
    { 0xf4a95d79, SUC_lexicon_tags_19 },
    { 0x8c095d7a, SUC_lexicon_tags_37 },
    { 0x5ca95d7b, SUC_lexicon_tags_38 },
    { 0xe0095d7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20495d7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c95d81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f695d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0295d85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc95d8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b495d90, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x95695d93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75895d95, SUC_lexicon_tags_1 },
    { 0xdf295d96, SUC_lexicon_tags_60 },
    { 0x56e95d96, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b095d9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e95da0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d495da2, SUC_lexicon_tags_26 },
    { 0x8fa95da2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc7295da5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c295da8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaea95daa, SUC_lexicon_tags_14 },
    { 0x69495daa, SUC_lexicon_tags_14 },
    { 0xfe095dac, SUC_lexicon_tags_47 },
    { 0x29c95dac, SUC_lexicon_tags_0 },
    { 0xa7895dad, SUC_lexicon_tags_11 },
    { 0x86c95dab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6695db2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47495db5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb295dbf, SUC_lexicon_tags_26 },
    { 0x85e95dc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73295dc2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfda95dc4, SUC_lexicon_tags_15 },
    { 0xa1e95dc5, SUC_lexicon_tags_49 },
    { 0x59295dc5, SUC_lexicon_tags_30 },
    { 0x9d895dc7, SUC_lexicon_tags_11 },
    { 0x36c95dc5, SUC_lexicon_tags_25 },
    { 0xa7895dc8, SUC_lexicon_tags_7 },
    { 0xc2495dca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21695dce, SUC_lexicon_tags_5 },
    { 0xee295dce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54095dd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3295dd6, SUC_lexicon_tags_527 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa695dd9, SUC_lexicon_tags_0 },
    { 0xc6e95dd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31095de3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62495de5, SUC_lexicon_tags_37 },
    { 0x17695de5, SUC_lexicon_tags_65 },
    { 0xc2695de6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4495ded, SUC_lexicon_tags_68 },
    { 0xd0e95dee, SUC_lexicon_tags_17 },
    { 0xd0895def, SUC_lexicon_tags_127 },
    { 0x39e95ded, SUC_lexicon_tags_11 },
    { 0x9f695df1, SUC_lexicon_tags_26 },
    { 0x74695dee, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa95df5, SUC_lexicon_tags_65 },
    { 0xbd095df6, SUC_lexicon_tags_2 },
    { 0x5c95df7, SUC_lexicon_tags_12 },
    { 0x2d295df6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x295dfb, SUC_lexicon_tags_7 },
    { 0xcc895dfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0695dfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24495e04, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe895e06, SUC_lexicon_tags_12 },
    { 0xc4295e07, SUC_lexicon_tags_2 },
    { 0x83e95e08, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f895e14, SUC_lexicon_tags_31 },
    { 0xf7c95e15, SUC_lexicon_tags_37 },
    { 0x56e95e16, SUC_lexicon_tags_743 },
    { 0x90495e16, SUC_lexicon_tags_12 },
    { 0xa4095e16, SUC_lexicon_tags_25 },
    { 0x47695e19, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x48895e1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x82495e1e, SUC_lexicon_tags_62 },
    { 0xd0a95e1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a695e22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a95e25, SUC_lexicon_tags_63 },
    { 0x4f695e26, SUC_lexicon_tags_32 },
    { 0xd8495e26, SUC_lexicon_tags_15 },
    { 0xa1c95e28, SUC_lexicon_tags_25 },
    { 0x64a95e28, SUC_lexicon_tags_3 },
    { 0xace95e28, SUC_lexicon_tags_0 },
    { 0xa2495e2b, SUC_lexicon_tags_33 },
    { 0xaac95e2a, SUC_lexicon_tags_10 },
    { 0xd6a95e29, SUC_lexicon_tags_160 },
    { 0x44e95e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c895e32, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97695e36, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c295e39, SUC_lexicon_tags_11 },
    { 0xa895e3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa1095e3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee95e46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a895e51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ee95e53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6095e57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b295e59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e95e5e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe0695e60, SUC_lexicon_tags_17 },
    { 0x34895e60, SUC_lexicon_tags_9 },
    { 0xc4695e62, SUC_lexicon_tags_14 },
    { 0x1c95e63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a695e67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4695e6b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa7295e6d, SUC_lexicon_tags_14 },
    { 0xf1c95e6e, SUC_lexicon_tags_54 },
    { 0x88295e6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x93e95e71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff695e7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a95e81, SUC_lexicon_tags_11 },
    { 0x7ae95e81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5a95e84, SUC_lexicon_tags_387 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c295e88, SUC_lexicon_tags_14 },
    { 0x3de95e89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b495e8b, SUC_lexicon_tags_53 },
    { 0x71095e8b, SUC_lexicon_tags_11 },
    { 0x94c95e8c, SUC_lexicon_tags_7 },
    { 0x5b495e8b, SUC_lexicon_tags_177 },
    { 0x15a95e8d, SUC_lexicon_tags_38 },
    { 0x91095e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d695e93, SUC_lexicon_tags_15 },
    { 0xa5295e94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50e95e96, SUC_lexicon_tags_170 },
    { 0x50a95e97, SUC_lexicon_tags_60 },
    { 0x99a95e98, SUC_lexicon_tags_82 },
    { 0x36095e96, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e95ea7, SUC_lexicon_tags_7 },
    { 0x9b895ea8, SUC_lexicon_tags_0 },
    { 0x24a95ea8, SUC_lexicon_tags_35 },
    { 0xab095eaa, SUC_lexicon_tags_54 },
    { 0x75e95eab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32095eaf, SUC_lexicon_tags_750 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e95eb3, SUC_lexicon_tags_37 },
    { 0x75695eb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29c95eb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f895eba, SUC_lexicon_tags_49 },
    { 0xf6895ebb, SUC_lexicon_tags_26 },
    { 0xd2295ebc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c95ebe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf0895ec0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e95ec5, SUC_lexicon_tags_26 },
    { 0xa9895ec6, SUC_lexicon_tags_25 },
    { 0x5e695ec7, SUC_lexicon_tags_39 },
    { 0xb1095ec6, SUC_lexicon_tags_15 },
    { 0x59495ec9, SUC_lexicon_tags_12 },
    { 0x14695eca, SUC_lexicon_tags_2 },
    { 0x3f895eca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5e295ecd, SUC_lexicon_tags_11 },
    { 0x26e95ece, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f095ed1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6e95ed3, SUC_lexicon_tags_32 },
    { 0x59095ed4, SUC_lexicon_tags_47 },
    { 0x22295ed3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90095eda, SUC_lexicon_tags_37 },
    { 0xafc95edb, SUC_lexicon_tags_1 },
    { 0x71895edc, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0xcba95ede, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd495ee1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5095ee8, SUC_lexicon_tags_8 },
    { 0xb1e95ee8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ba95eeb, SUC_lexicon_tags_26 },
    { 0xa9295eec, SUC_lexicon_tags_11 },
    { 0xe4a95eed, SUC_lexicon_tags_38 },
    { 0x3ee95eee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2d495ef0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec95ef5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfe295ef7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b895ef9, SUC_lexicon_tags_33 },
    { 0xe2e95ef9, SUC_lexicon_tags_35 },
    { 0x4a895efa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17095f00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf895f04, SUC_lexicon_tags_11 },
    { 0x69495f04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27095f07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6495f09, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2d095f0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e95f10, SUC_lexicon_tags_1 },
    { 0xaac95f10, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7295f15, SUC_lexicon_tags_9 },
    { 0xada95f16, SUC_lexicon_tags_37 },
    { 0x1a295f17, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d495f21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0095f23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef495f25, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75495f30, SUC_lexicon_tags_37 },
    { 0xf9c95f31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb495f33, SUC_lexicon_tags_2 },
    { 0x2d295f33, SUC_lexicon_tags_74 },
    { 0x3a895f35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x82695f38, SUC_lexicon_tags_60 },
    { 0x14895f39, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x73295f3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7095f3d, SUC_lexicon_tags_12 },
    { 0x62c95f3d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64095f47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2c95f49, SUC_lexicon_tags_29 },
    { 0x92e95f4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d695f4f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d695f53, SUC_lexicon_tags_4 },
    { 0x2f495f53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1295f58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1895f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66295f5c, SUC_lexicon_tags_11 },
    { 0x3de95f5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f295f61, SUC_lexicon_tags_150 },
    { 0xeec95f62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d895f64, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x21895f67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20895f6d, SUC_lexicon_tags_11 },
    { 0x2ca95f6e, SUC_lexicon_tags_47 },
    { 0x4c95f6f, SUC_lexicon_tags_27 },
    { 0x1d295f6f, SUC_lexicon_tags_37 },
    { 0xd5695f71, SUC_lexicon_tags_12 },
    { 0x75895f71, SUC_lexicon_tags_17 },
    { 0x78095f73, SUC_lexicon_tags_10 },
    { 0x72a95f70, SUC_lexicon_tags_47 },
    { 0x70a95f75, SUC_lexicon_tags_37 },
    { 0x63895f75, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f895f7a, SUC_lexicon_tags_2 },
    { 0xd5295f7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74695f7f, SUC_lexicon_tags_31 },
    { 0xcec95f7f, SUC_lexicon_tags_15 },
    { 0x51a95f81, SUC_lexicon_tags_2 },
    { 0x4e495f82, SUC_lexicon_tags_1 },
    { 0x5a095f81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x46a95f85, SUC_lexicon_tags_3 },
    { 0x2d295f86, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69695f8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c95f93, SUC_lexicon_tags_17 },
    { 0x14895f94, SUC_lexicon_tags_9 },
    { 0x6e695f94, SUC_lexicon_tags_17 },
    { 0xc6095f96, SUC_lexicon_tags_92 },
    { 0x2ca95f97, SUC_lexicon_tags_15 },
    { 0x9c495f98, SUC_lexicon_tags_27 },
    { 0x54095f94, SUC_lexicon_tags_30 },
    { 0x2c095f98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25a95f9c, SUC_lexicon_tags_47 },
    { 0xa2495f9c, SUC_lexicon_tags_194 },
    { 0x67095f9e, SUC_lexicon_tags_17 },
    { 0xf7a95f9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3695fa2, SUC_lexicon_tags_4 },
    { 0xc7895fa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x37695fa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4895fa9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x57095fab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c95faf, SUC_lexicon_tags_42 },
    { 0xb6095fb0, SUC_lexicon_tags_27 },
    { 0xdfa95faf, SUC_lexicon_tags_12 },
    { 0xfe95fb2, SUC_lexicon_tags_2 },
    { 0x74495fb3, SUC_lexicon_tags_60 },
    { 0x54e95fb3, SUC_lexicon_tags_45 },
    { 0x81a95fb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b295fb8, SUC_lexicon_tags_15 },
    { 0xea295fb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x81695fbc, SUC_lexicon_tags_1 },
    { 0xfda95fbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20495fc1, SUC_lexicon_tags_37 },
    { 0xf695fc1, SUC_lexicon_tags_55 },
    { 0x81295fc3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x57295fc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5495fc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e95fcb, SUC_lexicon_tags_48 },
    { 0x76095fcc, SUC_lexicon_tags_15 },
    { 0x2dc95fcc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e95fd3, SUC_lexicon_tags_13 },
    { 0x4495fd4, SUC_lexicon_tags_11 },
    { 0x86895fd4, SUC_lexicon_tags_2 },
    { 0x91295fd6, SUC_lexicon_tags_7 },
    { 0xe1a95fd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e95fe0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54695fe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54a95fe9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75095fed, SUC_lexicon_tags_23 },
    { 0xa0895fed, SUC_lexicon_tags_15 },
    { 0x29895fef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d495ff3, SUC_lexicon_tags_37 },
    { 0xcec95ff4, SUC_lexicon_tags_17 },
    { 0xbc295ff5, SUC_lexicon_tags_35 },
    { 0x36295ff5, SUC_lexicon_tags_6 },
    { 0x3be95ff3, SUC_lexicon_tags_49 },
    { 0x1b295ff5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61895ffa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8295ffd, SUC_lexicon_tags_27 },
    { 0xf8695ffd, SUC_lexicon_tags_11 },
    { 0xe9695fff, SUC_lexicon_tags_14 },
    { 0x3b495fff, SUC_lexicon_tags_14 },
    { 0x89a95ffd, SUC_lexicon_tags_30 },
    { 0xf2a95ffd, SUC_lexicon_tags_21 },
    { 0xe0495ffd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d696006, SUC_lexicon_tags_29 },
    { 0xdee96007, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6509600b, SUC_lexicon_tags_15 },
    { 0x32c9600b, SUC_lexicon_tags_14 },
    { 0xf309600b, SUC_lexicon_tags_25 },
    { 0xea69600e, SUC_lexicon_tags_12 },
    { 0x6609600e, SUC_lexicon_tags_17 },
    { 0xbc096010, SUC_lexicon_tags_0 },
    { 0x6c9600e, SUC_lexicon_tags_22 },
    { 0x83096012, SUC_lexicon_tags_37 },
    { 0xfd896012, SUC_lexicon_tags_25 },
    { 0x96c96013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f496016, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f896019, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x849601c, SUC_lexicon_tags_12 },
    { 0x3169601d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a96021, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfbe96023, SUC_lexicon_tags_28 },
    { 0x42a96023, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d09602b, SUC_lexicon_tags_27 },
    { 0x2f29602b, SUC_lexicon_tags_37 },
    { 0xf389602d, SUC_lexicon_tags_17 },
    { 0x1c09602d, SUC_lexicon_tags_25 },
    { 0x4809602f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc9a96031, SUC_lexicon_tags_17 },
    { 0x93296032, SUC_lexicon_tags_17 },
    { 0xdfc96033, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x56a96035, SUC_lexicon_tags_42 },
    { 0x41e96036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e696038, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1629603f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa49604b, SUC_lexicon_tags_37 },
    { 0x7fe9604c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa9604f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4896053, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab896059, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9489605b, SUC_lexicon_tags_68 },
    { 0x2129605b, SUC_lexicon_tags_918 },
    { 0xac09605d, SUC_lexicon_tags_25 },
    { 0xbe49605d, SUC_lexicon_tags_31 },
    { 0xe689605b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d096064, SUC_lexicon_tags_60 },
    { 0x40696064, SUC_lexicon_tags_25 },
    { 0x41296065, SUC_lexicon_tags_30 },
    { 0x9896066, SUC_lexicon_tags_37 },
    { 0x92c96068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca96073, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19c96075, SUC_lexicon_tags_33 },
    { 0xfe96076, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e96079, SUC_lexicon_tags_33 },
    { 0x7d09607a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2269607e, SUC_lexicon_tags_106 },
    { 0x2669607e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86296086, SUC_lexicon_tags_15 },
    { 0xf7696087, SUC_lexicon_tags_30 },
    { 0xe5896088, SUC_lexicon_tags_0 },
    { 0xfd096089, SUC_lexicon_tags_27 },
    { 0x69296086, SUC_lexicon_tags_14 },
    { 0xcb29608b, SUC_lexicon_tags_1 },
    { 0x689608c, SUC_lexicon_tags_11 },
    { 0x6ca9608c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7296090, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1096097, SUC_lexicon_tags_235 },
    { 0x1f496097, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xb289609b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764960a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf06960a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e960a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6960a9, SUC_lexicon_tags_64 },
    { 0x3a6960aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36960af, SUC_lexicon_tags_15 },
    { 0x962960af, SUC_lexicon_tags_32 },
    { 0xa8a960b1, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x6ca960b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c960b8, SUC_lexicon_tags_2 },
    { 0xc2a960b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56a960bb, SUC_lexicon_tags_14 },
    { 0xdc2960bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdec960be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x282960c0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x34a960c2, SUC_lexicon_tags_69 },
    { 0x43c960c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce960c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a960c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18960cc, SUC_lexicon_tags_12 },
    { 0x9e6960cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe960cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9d6960d1, SUC_lexicon_tags_3 },
    { 0x986960d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10960d5, SUC_lexicon_tags_33 },
    { 0xca6960d6, SUC_lexicon_tags_3 },
    { 0x7e8960d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c960db, SUC_lexicon_tags_44 },
    { 0x336960db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0960e2, SUC_lexicon_tags_15 },
    { 0xcde960e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6960e9, SUC_lexicon_tags_15 },
    { 0x67c960ea, SUC_lexicon_tags_1 },
    { 0x9a2960e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2960ee, SUC_lexicon_tags_8 },
    { 0x53e960ee, SUC_lexicon_tags_2 },
    { 0x650960f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0960f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4960f9, SUC_lexicon_tags_15 },
    { 0xe44960fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e960ff, SUC_lexicon_tags_17 },
    { 0xe8496100, SUC_lexicon_tags_37 },
    { 0xf6296100, SUC_lexicon_tags_32 },
    { 0x96096102, SUC_lexicon_tags_37 },
    { 0x95296102, SUC_lexicon_tags_17 },
    { 0x29c96104, SUC_lexicon_tags_23 },
    { 0xfbc96102, SUC_lexicon_tags_12 },
    { 0x67e96102, SUC_lexicon_tags_26 },
    { 0x11c96102, SUC_lexicon_tags_8 },
    { 0xc4296104, SUC_lexicon_tags_27 },
    { 0xbf696108, SUC_lexicon_tags_16 },
    { 0x57896106, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1096111, SUC_lexicon_tags_11 },
    { 0x37296112, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec96117, SUC_lexicon_tags_31 },
    { 0xa7c96117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1309611b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3649611d, SUC_lexicon_tags_0 },
    { 0x6709611e, SUC_lexicon_tags_47 },
    { 0xf9e9611d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x83696122, SUC_lexicon_tags_12 },
    { 0x1f696123, SUC_lexicon_tags_1 },
    { 0x83c96124, SUC_lexicon_tags_9 },
    { 0x2e496125, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c49612a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1096132, SUC_lexicon_tags_84 },
    { 0xd7696132, SUC_lexicon_tags_11 },
    { 0x5ca96134, SUC_lexicon_tags_43 },
    { 0xf5096135, SUC_lexicon_tags_1 },
    { 0x8896136, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc296139, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b49613d, SUC_lexicon_tags_83 },
    { 0x8d09613d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xce296141, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb096143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e96146, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6896148, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f09614a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe429614d, SUC_lexicon_tags_11 },
    { 0xd329614d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c696153, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3de96155, SUC_lexicon_tags_10 },
    { 0x22296155, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd3e96158, SUC_lexicon_tags_26 },
    { 0x4ca96159, SUC_lexicon_tags_14 },
    { 0xcfe9615a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8389615e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9696160, SUC_lexicon_tags_31 },
    { 0xe9096161, SUC_lexicon_tags_12 },
    { 0x8fa96162, SUC_lexicon_tags_15 },
    { 0xf6c96163, SUC_lexicon_tags_1 },
    { 0x20a96164, SUC_lexicon_tags_25 },
    { 0x3ce96160, SUC_lexicon_tags_3 },
    { 0x6da96160, SUC_lexicon_tags_87 },
    { 0xd4e96167, SUC_lexicon_tags_12 },
    { 0x13c96167, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9c9616a, SUC_lexicon_tags_60 },
    { 0x5969616a, SUC_lexicon_tags_17 },
    { 0xb6a9616c, SUC_lexicon_tags_33 },
    { 0x5ec9616d, SUC_lexicon_tags_19 },
    { 0x7e09616b, SUC_lexicon_tags_33 },
    { 0xe569616f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e96172, SUC_lexicon_tags_47 },
    { 0xee096173, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59496177, SUC_lexicon_tags_14 },
    { 0x66096178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb589617b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf589617e, SUC_lexicon_tags_11 },
    { 0xe9e9617e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x33096181, SUC_lexicon_tags_45 },
    { 0x90e96181, SUC_lexicon_tags_12 },
    { 0x77296182, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e96186, SUC_lexicon_tags_12 },
    { 0xf9096186, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9c96189, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba96190, SUC_lexicon_tags_1 },
    { 0x17c96191, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46496198, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6129619c, SUC_lexicon_tags_45 },
    { 0x55a9619d, SUC_lexicon_tags_9 },
    { 0xf709619d, SUC_lexicon_tags_28 },
    { 0xe749619f, SUC_lexicon_tags_23 },
    { 0x100961a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828961a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c0961a6, SUC_lexicon_tags_60 },
    { 0x3bc961a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a2961a9, SUC_lexicon_tags_7 },
    { 0x5d6961aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98961ae, SUC_lexicon_tags_38 },
    { 0xb2961af, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x2b2961b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c961b5, SUC_lexicon_tags_68 },
    { 0x12a961b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc961b9, SUC_lexicon_tags_63 },
    { 0x358961b9, SUC_lexicon_tags_14 },
    { 0xe70961bb, SUC_lexicon_tags_8 },
    { 0x528961bb, SUC_lexicon_tags_28 },
    { 0xe8a961bd, SUC_lexicon_tags_11 },
    { 0x292961bc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x594961c0, SUC_lexicon_tags_14 },
    { 0xa82961c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54961cb, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa961cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92a961d1, SUC_lexicon_tags_30 },
    { 0x130961d1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace961d8, SUC_lexicon_tags_6 },
    { 0xf10961d8, SUC_lexicon_tags_7 },
    { 0x104961d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52961dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x12961df, SUC_lexicon_tags_15 },
    { 0x8c6961e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a961e4, SUC_lexicon_tags_37 },
    { 0xd2961e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a961ee, SUC_lexicon_tags_4 },
    { 0xc4a961ef, SUC_lexicon_tags_25 },
    { 0x8f4961ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98961f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a961fb, SUC_lexicon_tags_38 },
    { 0x2e8961fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e96209, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb29620e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4696211, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa96219, SUC_lexicon_tags_1 },
    { 0xba49621a, SUC_lexicon_tags_15 },
    { 0x3d09621b, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1696222, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee96225, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47e96227, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a96233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc696235, SUC_lexicon_tags_37 },
    { 0xdd096236, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6896238, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1689623a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bc9623c, SUC_lexicon_tags_37 },
    { 0x7329623c, SUC_lexicon_tags_31 },
    { 0x9149623c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x67696241, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a09624b, SUC_lexicon_tags_0 },
    { 0xb449624c, SUC_lexicon_tags_0 },
    { 0x2089624d, SUC_lexicon_tags_25 },
    { 0x16c9624d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b696253, SUC_lexicon_tags_15 },
    { 0xbe496254, SUC_lexicon_tags_17 },
    { 0xfe696255, SUC_lexicon_tags_8 },
    { 0x4e696254, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95c96258, SUC_lexicon_tags_2 },
    { 0x3ae96259, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e89625c, SUC_lexicon_tags_17 },
    { 0xe229625d, SUC_lexicon_tags_6 },
    { 0x6409625e, SUC_lexicon_tags_2 },
    { 0x6829625e, SUC_lexicon_tags_26 },
    { 0x3f096260, SUC_lexicon_tags_11 },
    { 0xc9896260, SUC_lexicon_tags_38 },
    { 0xca89625e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe629626a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7169626d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c96271, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e896274, SUC_lexicon_tags_17 },
    { 0xa296275, SUC_lexicon_tags_17 },
    { 0xc9096275, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba96279, SUC_lexicon_tags_47 },
    { 0xd9a9627a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdc9627c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbc49627e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce96286, SUC_lexicon_tags_7 },
    { 0x5fa96287, SUC_lexicon_tags_12 },
    { 0x90e96288, SUC_lexicon_tags_19 },
    { 0xdf696289, SUC_lexicon_tags_12 },
    { 0xf5096288, SUC_lexicon_tags_17 },
    { 0xf69628b, SUC_lexicon_tags_12 },
    { 0xa2896289, SUC_lexicon_tags_4 },
    { 0x5c496288, SUC_lexicon_tags_37 },
    { 0x9449628e, SUC_lexicon_tags_14 },
    { 0x4529628c, SUC_lexicon_tags_15 },
    { 0x9c96290, SUC_lexicon_tags_14 },
    { 0xe2896290, SUC_lexicon_tags_23 },
    { 0x76296292, SUC_lexicon_tags_48 },
    { 0x9fc9628d, SUC_lexicon_tags_60 },
    { 0xaa696294, SUC_lexicon_tags_11 },
    { 0x91896294, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c9629b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf789629e, SUC_lexicon_tags_10 },
    { 0xd6c9629f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xae0962a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858962b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6962b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50962bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf12962bd, SUC_lexicon_tags_37 },
    { 0x228962bd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce962c3, SUC_lexicon_tags_25 },
    { 0x55a962c4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xac4962c6, SUC_lexicon_tags_55 },
    { 0x826962c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2962ca, SUC_lexicon_tags_47 },
    { 0xac6962cb, SUC_lexicon_tags_38 },
    { 0x78a962cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94962cf, SUC_lexicon_tags_49 },
    { 0x99e962cf, SUC_lexicon_tags_33 },
    { 0x8962d1, SUC_lexicon_tags_15 },
    { 0xd7a962cf, SUC_lexicon_tags_583 },
    { 0x2aa962d3, SUC_lexicon_tags_15 },
    { 0x1e6962d4, SUC_lexicon_tags_11 },
    { 0xaa8962d5, SUC_lexicon_tags_7 },
    { 0x7f6962d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a962db, SUC_lexicon_tags_17 },
    { 0x438962dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a962df, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6962e5, SUC_lexicon_tags_17 },
    { 0x160962e5, SUC_lexicon_tags_38 },
    { 0x374962e7, SUC_lexicon_tags_56 },
    { 0x69a962e8, SUC_lexicon_tags_4 },
    { 0x80e962e9, SUC_lexicon_tags_15 },
    { 0xd1c962e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc24962ec, SUC_lexicon_tags_25 },
    { 0xc38962ec, SUC_lexicon_tags_37 },
    { 0xe962ee, SUC_lexicon_tags_47 },
    { 0xde4962ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x480962f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4962f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x948962f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a962fc, SUC_lexicon_tags_37 },
    { 0xfb8962fd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x1ca962ff, SUC_lexicon_tags_33 },
    { 0xcb6962ff, SUC_lexicon_tags_55 },
    { 0x7c496301, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a096304, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa96308, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee29630a, SUC_lexicon_tags_14 },
    { 0xc909630b, SUC_lexicon_tags_11 },
    { 0x3109630c, SUC_lexicon_tags_38 },
    { 0x44a9630d, SUC_lexicon_tags_0 },
    { 0xb89630d, SUC_lexicon_tags_2 },
    { 0x33e9630a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb696312, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x36296315, SUC_lexicon_tags_60 },
    { 0xf5896316, SUC_lexicon_tags_2 },
    { 0x32296317, SUC_lexicon_tags_33 },
    { 0x72e96318, SUC_lexicon_tags_17 },
    { 0x3ae96317, SUC_lexicon_tags_15 },
    { 0xbfa9631a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf769631f, SUC_lexicon_tags_1234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a496326, SUC_lexicon_tags_47 },
    { 0x5f096326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc9632c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb296332, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4096338, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1449633f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae96347, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a9634a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x43e9634c, SUC_lexicon_tags_25 },
    { 0xeae9634d, SUC_lexicon_tags_15 },
    { 0x4c89634d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be96353, SUC_lexicon_tags_45 },
    { 0x16c96354, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86896359, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c9635e, SUC_lexicon_tags_37 },
    { 0xe0a9635f, SUC_lexicon_tags_44 },
    { 0x5e9635f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92096366, SUC_lexicon_tags_49 },
    { 0x63496367, SUC_lexicon_tags_2 },
    { 0xfc296367, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xafe9636a, SUC_lexicon_tags_17 },
    { 0x7069636b, SUC_lexicon_tags_87 },
    { 0x6c9636b, SUC_lexicon_tags_7 },
    { 0xc269636b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9696370, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36096375, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f296377, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb909637a, SUC_lexicon_tags_37 },
    { 0x20c9637a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4429637d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3696382, SUC_lexicon_tags_2 },
    { 0x80696382, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a96388, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc549638b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x88c9638d, SUC_lexicon_tags_11 },
    { 0xc09638e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1896390, SUC_lexicon_tags_19 },
    { 0x1b296391, SUC_lexicon_tags_7 },
    { 0xbee96392, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3189639c, SUC_lexicon_tags_2 },
    { 0xf1a9639d, SUC_lexicon_tags_15 },
    { 0xeac9639d, SUC_lexicon_tags_63 },
    { 0xb669639f, SUC_lexicon_tags_28 },
    { 0xe28963a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56963a8, SUC_lexicon_tags_49 },
    { 0xfae963a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e963ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c963b4, SUC_lexicon_tags_1172 },
    { 0xc0c963b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x176963b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ba963b9, SUC_lexicon_tags_47 },
    { 0x176963ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6963be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08963c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x814963c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e963ca, SUC_lexicon_tags_43 },
    { 0x93e963cb, SUC_lexicon_tags_17 },
    { 0x176963cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee963d1, SUC_lexicon_tags_7 },
    { 0xaa6963d1, SUC_lexicon_tags_1 },
    { 0x246963d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70963d8, SUC_lexicon_tags_37 },
    { 0xa50963d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8963de, SUC_lexicon_tags_0 },
    { 0xffa963df, SUC_lexicon_tags_11 },
    { 0xf96963de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40963e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e963eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74963ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8963f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66963f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e963fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c963fd, SUC_lexicon_tags_12 },
    { 0x746963fe, SUC_lexicon_tags_8 },
    { 0x398963ff, SUC_lexicon_tags_65 },
    { 0xf9496400, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf0696402, SUC_lexicon_tags_60 },
    { 0x5fc96402, SUC_lexicon_tags_49 },
    { 0x96296402, SUC_lexicon_tags_28 },
    { 0x45c96405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc589640e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e96412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a96415, SUC_lexicon_tags_40 },
    { 0xf696416, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb8e96418, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x20c9641a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe0c9641c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ba9641e, SUC_lexicon_tags_28 },
    { 0x9c69641f, SUC_lexicon_tags_7 },
    { 0xb7096420, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce896422, SUC_lexicon_tags_23 },
    { 0xf3296423, SUC_lexicon_tags_170 },
    { 0x56a96424, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20296429, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4909642c, SUC_lexicon_tags_77 },
    { 0x5be9642d, SUC_lexicon_tags_15 },
    { 0x8049642e, SUC_lexicon_tags_44 },
    { 0xe149642f, SUC_lexicon_tags_84 },
    { 0x15a96430, SUC_lexicon_tags_68 },
    { 0xa3896430, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c96438, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c9643c, SUC_lexicon_tags_25 },
    { 0x9d09643d, SUC_lexicon_tags_75 },
    { 0xfec9643e, SUC_lexicon_tags_26 },
    { 0x43c9643d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf896443, SUC_lexicon_tags_37 },
    { 0x27496444, SUC_lexicon_tags_33 },
    { 0x6a496445, SUC_lexicon_tags_17 },
    { 0x14296446, SUC_lexicon_tags_0 },
    { 0x6d296444, SUC_lexicon_tags_2 },
    { 0x70a96446, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa829644c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d09644e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5496450, SUC_lexicon_tags_11 },
    { 0x20096450, SUC_lexicon_tags_25 },
    { 0x89496452, SUC_lexicon_tags_11 },
    { 0xcce96453, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a296457, SUC_lexicon_tags_2 },
    { 0x31296457, SUC_lexicon_tags_39 },
    { 0x15c96459, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa049645b, SUC_lexicon_tags_204 },
    { 0x4ce9645b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe296461, SUC_lexicon_tags_8 },
    { 0xa1c96462, SUC_lexicon_tags_27 },
    { 0xd5c96461, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b696466, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c96469, SUC_lexicon_tags_12 },
    { 0xf8c9646a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3109646e, SUC_lexicon_tags_37 },
    { 0xeba9646f, SUC_lexicon_tags_30 },
    { 0x9489646f, SUC_lexicon_tags_12 },
    { 0x8089646e, SUC_lexicon_tags_7 },
    { 0x7a096472, SUC_lexicon_tags_25 },
    { 0xe6696473, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11096475, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf989647c, SUC_lexicon_tags_831 },
    { 0x66e9647d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x88096480, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd6296482, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73696486, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e9648b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a9648f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2696492, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6896495, SUC_lexicon_tags_63 },
    { 0xbac96496, SUC_lexicon_tags_39 },
    { 0x84096495, SUC_lexicon_tags_37 },
    { 0xdf096497, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3d09649a, SUC_lexicon_tags_65 },
    { 0x6d09649a, SUC_lexicon_tags_7 },
    { 0xc069649c, SUC_lexicon_tags_38 },
    { 0x1f49649d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8964a0, SUC_lexicon_tags_17 },
    { 0x314964a0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2964a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34964aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a964b5, SUC_lexicon_tags_14 },
    { 0x19e964b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b8964b8, SUC_lexicon_tags_1 },
    { 0x150964b9, SUC_lexicon_tags_13 },
    { 0x7cc964ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x586964bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2964c0, SUC_lexicon_tags_45 },
    { 0x2de964c1, SUC_lexicon_tags_25 },
    { 0xeac964c0, SUC_lexicon_tags_14 },
    { 0xcba964c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0964c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a964ca, SUC_lexicon_tags_25 },
    { 0xfd4964cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0964ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2964d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc964d8, SUC_lexicon_tags_11 },
    { 0x8c964d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988964dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e964e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c964e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x60a964e6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf88964e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48964ea, SUC_lexicon_tags_60 },
    { 0x5a4964eb, SUC_lexicon_tags_27 },
    { 0x828964ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea964ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560964f3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8964f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x64964f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2964ff, SUC_lexicon_tags_30 },
    { 0x818964ff, SUC_lexicon_tags_45 },
    { 0xa1296501, SUC_lexicon_tags_8 },
    { 0x93896502, SUC_lexicon_tags_15 },
    { 0xec8964ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64096509, SUC_lexicon_tags_8 },
    { 0x5329650a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09650d, SUC_lexicon_tags_25 },
    { 0x149650d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e96511, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7096516, SUC_lexicon_tags_29 },
    { 0xfee96517, SUC_lexicon_tags_47 },
    { 0x7d696518, SUC_lexicon_tags_38 },
    { 0xdd096518, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4989651c, SUC_lexicon_tags_25 },
    { 0x8049651d, SUC_lexicon_tags_37 },
    { 0xc729651e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xd3e96520, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xba896522, SUC_lexicon_tags_15 },
    { 0xcbc96523, SUC_lexicon_tags_40 },
    { 0xa8496523, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcfc96526, SUC_lexicon_tags_29 },
    { 0x78496527, SUC_lexicon_tags_12 },
    { 0xaca96527, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b096537, SUC_lexicon_tags_79 },
    { 0x14096537, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3d69653a, SUC_lexicon_tags_25 },
    { 0xda89653a, SUC_lexicon_tags_37 },
    { 0x9de9653a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f296544, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31296546, SUC_lexicon_tags_17 },
    { 0xd0c96546, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x99c96549, SUC_lexicon_tags_39 },
    { 0x1529654a, SUC_lexicon_tags_15 },
    { 0xc4e9654a, SUC_lexicon_tags_47 },
    { 0x1e09654a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x71c9654e, SUC_lexicon_tags_11 },
    { 0x4509654f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc96556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9049655f, SUC_lexicon_tags_25 },
    { 0x23a9655f, SUC_lexicon_tags_43 },
    { 0xdaa96561, SUC_lexicon_tags_0 },
    { 0x14496562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e96566, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae9656a, SUC_lexicon_tags_23 },
    { 0xd049656b, SUC_lexicon_tags_65 },
    { 0x58a9656c, SUC_lexicon_tags_1 },
    { 0xfaa9656d, SUC_lexicon_tags_43 },
    { 0x65a9656d, SUC_lexicon_tags_9 },
    { 0xec09656b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78e96571, SUC_lexicon_tags_45 },
    { 0xc8096572, SUC_lexicon_tags_11 },
    { 0xbc696573, SUC_lexicon_tags_55 },
    { 0xe96574, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e96576, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a9657a, SUC_lexicon_tags_12 },
    { 0x66e9657a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce96580, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6696587, SUC_lexicon_tags_60 },
    { 0x3296588, SUC_lexicon_tags_17 },
    { 0x3ca96589, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b49658c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b496592, SUC_lexicon_tags_11 },
    { 0xc9296593, SUC_lexicon_tags_60 },
    { 0xe496594, SUC_lexicon_tags_15 },
    { 0x63e96595, SUC_lexicon_tags_9 },
    { 0xf7496594, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x55696598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214965a0, SUC_lexicon_tags_11 },
    { 0x93c965a1, SUC_lexicon_tags_37 },
    { 0x12c965a2, SUC_lexicon_tags_58 },
    { 0x220965a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x880965a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66e965a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68e965aa, SUC_lexicon_tags_17 },
    { 0x4c965aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a965ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb22965b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e965b7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x28e965b9, SUC_lexicon_tags_23 },
    { 0x5e6965ba, SUC_lexicon_tags_19 },
    { 0x358965bb, SUC_lexicon_tags_49 },
    { 0x2e0965bc, SUC_lexicon_tags_25 },
    { 0xc5e965b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2965c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a965c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a2965c6, SUC_lexicon_tags_37 },
    { 0x352965c7, SUC_lexicon_tags_11 },
    { 0xea8965c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe44965ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880965ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408965d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c965d5, SUC_lexicon_tags_1 },
    { 0xf8965d5, SUC_lexicon_tags_12 },
    { 0xaa4965d7, SUC_lexicon_tags_23 },
    { 0x582965d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x93c965da, SUC_lexicon_tags_23 },
    { 0x84c965da, SUC_lexicon_tags_1 },
    { 0x5e6965dc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360965e0, SUC_lexicon_tags_1 },
    { 0x5f2965e1, SUC_lexicon_tags_28 },
    { 0xb6a965e2, SUC_lexicon_tags_64 },
    { 0xce6965e3, SUC_lexicon_tags_43 },
    { 0x888965e0, SUC_lexicon_tags_35 },
    { 0x758965e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34965e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x840965ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6965f2, SUC_lexicon_tags_15 },
    { 0xd2a965f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c965f6, SUC_lexicon_tags_61 },
    { 0xe24965f6, SUC_lexicon_tags_25 },
    { 0x998965f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e965fb, SUC_lexicon_tags_60 },
    { 0x8a2965fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a96601, SUC_lexicon_tags_8 },
    { 0x60a96602, SUC_lexicon_tags_68 },
    { 0x50896603, SUC_lexicon_tags_33 },
    { 0x29496604, SUC_lexicon_tags_47 },
    { 0x65e96605, SUC_lexicon_tags_296 },
    { 0xf3a96606, SUC_lexicon_tags_8 },
    { 0xf5496605, SUC_lexicon_tags_37 },
    { 0x43496608, SUC_lexicon_tags_46 },
    { 0x47c96607, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3989660b, SUC_lexicon_tags_2 },
    { 0x6d49660c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2769660e, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16896613, SUC_lexicon_tags_127 },
    { 0x1b896613, SUC_lexicon_tags_40 },
    { 0x37c96615, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57696618, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4509661b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7a9661d, SUC_lexicon_tags_7 },
    { 0x4069661e, SUC_lexicon_tags_17 },
    { 0x6049661e, SUC_lexicon_tags_27 },
    { 0x94c9661d, SUC_lexicon_tags_17 },
    { 0xece96621, SUC_lexicon_tags_2 },
    { 0xe3496622, SUC_lexicon_tags_15 },
    { 0x5c296623, SUC_lexicon_tags_45 },
    { 0xaec96621, SUC_lexicon_tags_29 },
    { 0xc9696625, SUC_lexicon_tags_7 },
    { 0x26e96626, SUC_lexicon_tags_15 },
    { 0x98a96627, SUC_lexicon_tags_92 },
    { 0x22a96625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a9662e, SUC_lexicon_tags_2 },
    { 0x32c9662f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28096631, SUC_lexicon_tags_11 },
    { 0x29896631, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0x93096635, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x45896637, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7149663d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa96645, SUC_lexicon_tags_25 },
    { 0x2b696645, SUC_lexicon_tags_48 },
    { 0xe9496647, SUC_lexicon_tags_37 },
    { 0x12a96648, SUC_lexicon_tags_17 },
    { 0x17e96649, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a09664c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2089664e, SUC_lexicon_tags_26 },
    { 0xf6e9664e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x88896651, SUC_lexicon_tags_27 },
    { 0x7bc96652, SUC_lexicon_tags_23 },
    { 0xaa096652, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcee96655, SUC_lexicon_tags_11 },
    { 0xb8696655, SUC_lexicon_tags_27 },
    { 0xc1e96657, SUC_lexicon_tags_7 },
    { 0xf5e96656, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be9665b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60c9665d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc729665f, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7496663, SUC_lexicon_tags_47 },
    { 0xbf896664, SUC_lexicon_tags_19 },
    { 0xe7c96664, SUC_lexicon_tags_11 },
    { 0x59096666, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde89666e, SUC_lexicon_tags_47 },
    { 0xb349666f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19096672, SUC_lexicon_tags_4 },
    { 0xdc896673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1296678, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x64a9667a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9296680, SUC_lexicon_tags_31 },
    { 0x8ae96681, SUC_lexicon_tags_4 },
    { 0xbfa96682, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb296687, SUC_lexicon_tags_6 },
    { 0x69296687, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2669668b, SUC_lexicon_tags_2 },
    { 0x9dc9668c, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa69668f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24096694, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xede96697, SUC_lexicon_tags_28 },
    { 0x27c96698, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2029669a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa49669c, SUC_lexicon_tags_7 },
    { 0x8c9669d, SUC_lexicon_tags_43 },
    { 0xaca9669e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86966a1, SUC_lexicon_tags_14 },
    { 0x71e966a2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4966a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15a966aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0966ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c966b0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de966b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e966b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c0966b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcb0966bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4966bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8966c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa0966c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00966c7, SUC_lexicon_tags_13 },
    { 0xcfa966c8, SUC_lexicon_tags_48 },
    { 0x5ca966c8, SUC_lexicon_tags_37 },
    { 0x29c966ca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x160966cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27e966cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9bc966d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0966d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62966d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2966db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x198966dd, SUC_lexicon_tags_45 },
    { 0x974966dd, SUC_lexicon_tags_39 },
    { 0x2c4966df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x454966e2, SUC_lexicon_tags_11 },
    { 0xd54966e3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb30966e5, SUC_lexicon_tags_14 },
    { 0x1e0966e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e966ec, SUC_lexicon_tags_14 },
    { 0x958966ed, SUC_lexicon_tags_35 },
    { 0xa4966ed, SUC_lexicon_tags_9 },
    { 0xf4a966ee, SUC_lexicon_tags_1 },
    { 0xcc0966f0, SUC_lexicon_tags_26 },
    { 0x1ce966ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf22966f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2966ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a96705, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf096709, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x95c9670b, SUC_lexicon_tags_7 },
    { 0x95c9670c, SUC_lexicon_tags_25 },
    { 0x5929670d, SUC_lexicon_tags_37 },
    { 0xda49670c, SUC_lexicon_tags_10 },
    { 0x2a69670e, SUC_lexicon_tags_2 },
    { 0x95c96710, SUC_lexicon_tags_77 },
    { 0xc429670f, SUC_lexicon_tags_39 },
    { 0xe3096711, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6096715, SUC_lexicon_tags_17 },
    { 0x98096715, SUC_lexicon_tags_33 },
    { 0xe3696717, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe549671d, SUC_lexicon_tags_7 },
    { 0xe749671e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe496724, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49c96726, SUC_lexicon_tags_131 },
    { 0xa3096727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7149672b, SUC_lexicon_tags_25 },
    { 0xa149672b, SUC_lexicon_tags_28 },
    { 0x7e49672d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d696730, SUC_lexicon_tags_66 },
    { 0x28c96731, SUC_lexicon_tags_19 },
    { 0xe3e96730, SUC_lexicon_tags_12 },
    { 0x1fc96732, SUC_lexicon_tags_46 },
    { 0x1e696734, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5096737, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe9673a, SUC_lexicon_tags_43 },
    { 0xe249673b, SUC_lexicon_tags_23 },
    { 0x1dc9673c, SUC_lexicon_tags_13 },
    { 0xf2c9673d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b896741, SUC_lexicon_tags_26 },
    { 0x16896742, SUC_lexicon_tags_37 },
    { 0xcec96743, SUC_lexicon_tags_19 },
    { 0x85496744, SUC_lexicon_tags_8 },
    { 0x66c96744, SUC_lexicon_tags_15 },
    { 0x8d296746, SUC_lexicon_tags_10 },
    { 0x1ac96747, SUC_lexicon_tags_23 },
    { 0x6f896745, SUC_lexicon_tags_10 },
    { 0x5fa96749, SUC_lexicon_tags_0 },
    { 0x2ea9674a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9674f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7696752, SUC_lexicon_tags_47 },
    { 0x1f496752, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c9675a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c896764, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8496767, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf89676a, SUC_lexicon_tags_1 },
    { 0xf929676b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da9676e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7496776, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9189677a, SUC_lexicon_tags_47 },
    { 0x9a09677b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75496782, SUC_lexicon_tags_15 },
    { 0x7c296783, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc96786, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfd496788, SUC_lexicon_tags_39 },
    { 0xfa096789, SUC_lexicon_tags_37 },
    { 0x9ca9678a, SUC_lexicon_tags_37 },
    { 0x37a96788, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc9678f, SUC_lexicon_tags_85 },
    { 0x4fe96790, SUC_lexicon_tags_25 },
    { 0x33c96790, SUC_lexicon_tags_63 },
    { 0xcbe96790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57a96794, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ec96796, SUC_lexicon_tags_116 },
    { 0x58496797, SUC_lexicon_tags_7 },
    { 0xeec96798, SUC_lexicon_tags_2 },
    { 0xd2096799, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95c9679c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a967a0, SUC_lexicon_tags_29 },
    { 0xc66967a1, SUC_lexicon_tags_15 },
    { 0x930967a2, SUC_lexicon_tags_37 },
    { 0x1d8967a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd0967a5, SUC_lexicon_tags_92 },
    { 0xb2967a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550967aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeb8967ac, SUC_lexicon_tags_8 },
    { 0x18a967ac, SUC_lexicon_tags_26 },
    { 0x8ce967ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e967b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0967ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24967c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda967cb, SUC_lexicon_tags_5 },
    { 0xbec967cc, SUC_lexicon_tags_1 },
    { 0x980967cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364967d3, SUC_lexicon_tags_17 },
    { 0x398967d3, SUC_lexicon_tags_7 },
    { 0x3d2967d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44967d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdcc967db, SUC_lexicon_tags_15 },
    { 0xa6e967dc, SUC_lexicon_tags_37 },
    { 0x5c8967dd, SUC_lexicon_tags_77 },
    { 0xb78967db, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78967e1, SUC_lexicon_tags_34 },
    { 0x52e967e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0967e7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde967eb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e967f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90967f3, SUC_lexicon_tags_61 },
    { 0x8fa967f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee967f7, SUC_lexicon_tags_7 },
    { 0xa4967f8, SUC_lexicon_tags_38 },
    { 0x4c2967f7, SUC_lexicon_tags_37 },
    { 0x116967f8, SUC_lexicon_tags_46 },
    { 0x5f8967f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a96801, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfd696803, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38296806, SUC_lexicon_tags_61 },
    { 0x24496806, SUC_lexicon_tags_12 },
    { 0x2bc96808, SUC_lexicon_tags_15 },
    { 0xac896806, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3869680c, SUC_lexicon_tags_80 },
    { 0x6ac9680c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba96810, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb7e96812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a496814, SUC_lexicon_tags_107 },
    { 0xadc96815, SUC_lexicon_tags_15 },
    { 0x9a96816, SUC_lexicon_tags_24 },
    { 0xcce96817, SUC_lexicon_tags_0 },
    { 0x6a696817, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20a9681a, SUC_lexicon_tags_6 },
    { 0xa049681b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd829681f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2da96821, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda96829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf69682b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc09682d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a9682f, SUC_lexicon_tags_43 },
    { 0x1ae96830, SUC_lexicon_tags_17 },
    { 0x4b296831, SUC_lexicon_tags_11 },
    { 0x4ba96830, SUC_lexicon_tags_17 },
    { 0xe6896831, SUC_lexicon_tags_34 },
    { 0xd3a96834, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e9683c, SUC_lexicon_tags_49 },
    { 0xa9683d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace96841, SUC_lexicon_tags_50 },
    { 0xe9e96841, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e96847, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf896849, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa369684b, SUC_lexicon_tags_54 },
    { 0x6729684b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8229684f, SUC_lexicon_tags_31 },
    { 0xbf29684f, SUC_lexicon_tags_37 },
    { 0x3c896850, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa96856, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c9685f, SUC_lexicon_tags_15 },
    { 0xf5a96860, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2696863, SUC_lexicon_tags_49 },
    { 0x40e96864, SUC_lexicon_tags_4 },
    { 0xd9e96865, SUC_lexicon_tags_47 },
    { 0xb9e96866, SUC_lexicon_tags_25 },
    { 0x3b696866, SUC_lexicon_tags_25 },
    { 0xa2896868, SUC_lexicon_tags_11 },
    { 0x81096868, SUC_lexicon_tags_151 },
    { 0xf7a96868, SUC_lexicon_tags_64 },
    { 0x8a296867, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e96870, SUC_lexicon_tags_14 },
    { 0xe2c96871, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea96874, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xca696876, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6f896878, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8a9687a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3969687c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59896883, SUC_lexicon_tags_19 },
    { 0xbc696884, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x50e96886, SUC_lexicon_tags_1 },
    { 0xc6c96886, SUC_lexicon_tags_25 },
    { 0xa8496888, SUC_lexicon_tags_63 },
    { 0x4c296886, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c9688d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe5c9688f, SUC_lexicon_tags_46 },
    { 0xe249688f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x44496892, SUC_lexicon_tags_9 },
    { 0xede96893, SUC_lexicon_tags_15 },
    { 0xbd296892, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b296898, SUC_lexicon_tags_0 },
    { 0xb3496898, SUC_lexicon_tags_2 },
    { 0x5be96899, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc9689f, SUC_lexicon_tags_14 },
    { 0xe9689f, SUC_lexicon_tags_79 },
    { 0x6f69689f, SUC_lexicon_tags_0 },
    { 0xfd4968a0, SUC_lexicon_tags_55 },
    { 0x94c9689f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce968a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76968aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20968b0, SUC_lexicon_tags_7 },
    { 0xb3c968b1, SUC_lexicon_tags_12 },
    { 0x1ec968b1, SUC_lexicon_tags_3 },
    { 0xede968b3, SUC_lexicon_tags_19 },
    { 0xbb2968b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68968b9, SUC_lexicon_tags_27 },
    { 0x604968b9, SUC_lexicon_tags_47 },
    { 0xe9a968ba, SUC_lexicon_tags_7 },
    { 0xa96968bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822968c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x362968c7, SUC_lexicon_tags_1 },
    { 0x334968c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12968cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9c4968ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e968d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6968d4, SUC_lexicon_tags_63 },
    { 0x8e6968d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a6968d7, SUC_lexicon_tags_17 },
    { 0x140968d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb0968da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c968dd, SUC_lexicon_tags_43 },
    { 0x3dc968dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a968e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2968e5, SUC_lexicon_tags_15 },
    { 0x454968e5, SUC_lexicon_tags_25 },
    { 0x400968e6, SUC_lexicon_tags_259 },
    { 0xdee968e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda968eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a968f0, SUC_lexicon_tags_45 },
    { 0x6b6968f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc968f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x792968f7, SUC_lexicon_tags_25 },
    { 0xac4968f8, SUC_lexicon_tags_25 },
    { 0xf1e968f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c2968fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec968fe, SUC_lexicon_tags_25 },
    { 0x42968fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdca96901, SUC_lexicon_tags_17 },
    { 0xe6a96901, SUC_lexicon_tags_1 },
    { 0x87c96903, SUC_lexicon_tags_37 },
    { 0x41696902, SUC_lexicon_tags_37 },
    { 0x8c696905, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf0896907, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7289690b, SUC_lexicon_tags_4 },
    { 0xe6c9690b, SUC_lexicon_tags_63 },
    { 0x5929690d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13696910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec96913, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e96916, SUC_lexicon_tags_12 },
    { 0x35c96917, SUC_lexicon_tags_10 },
    { 0x9aa96916, SUC_lexicon_tags_2 },
    { 0xade96919, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc9691e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e96922, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb296926, SUC_lexicon_tags_37 },
    { 0xc296927, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa849692e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55696930, SUC_lexicon_tags_9 },
    { 0xbd896930, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e9693a, SUC_lexicon_tags_11 },
    { 0x93a9693b, SUC_lexicon_tags_2 },
    { 0x129693b, SUC_lexicon_tags_27 },
    { 0x2009693d, SUC_lexicon_tags_63 },
    { 0x1c9693e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x53296940, SUC_lexicon_tags_38 },
    { 0x5ea96941, SUC_lexicon_tags_25 },
    { 0x24896941, SUC_lexicon_tags_2 },
    { 0x16c96943, SUC_lexicon_tags_38 },
    { 0x3f096943, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd7a96946, SUC_lexicon_tags_2 },
    { 0x10a96946, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e9694a, SUC_lexicon_tags_37 },
    { 0xe89694b, SUC_lexicon_tags_15 },
    { 0xd009694c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1549694e, SUC_lexicon_tags_0 },
    { 0xf5e9694f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d096952, SUC_lexicon_tags_48 },
    { 0x3be96952, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9096957, SUC_lexicon_tags_60 },
    { 0xbac96958, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1f89695a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce9695e, SUC_lexicon_tags_0 },
    { 0x669695f, SUC_lexicon_tags_27 },
    { 0xa349695e, SUC_lexicon_tags_25 },
    { 0x98c96961, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f696967, SUC_lexicon_tags_60 },
    { 0x14496967, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5069696a, SUC_lexicon_tags_138 },
    { 0xc989696a, SUC_lexicon_tags_60 },
    { 0x3be9696c, SUC_lexicon_tags_2 },
    { 0x7d89696d, SUC_lexicon_tags_17 },
    { 0xa109696d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd0496970, SUC_lexicon_tags_17 },
    { 0x76c96971, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25a96973, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46696976, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3409697d, SUC_lexicon_tags_12 },
    { 0x99c9697d, SUC_lexicon_tags_11 },
    { 0x9fe9697f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf1e96981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa896987, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa709698a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e49698e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c96995, SUC_lexicon_tags_3 },
    { 0x64e96995, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd69699e, SUC_lexicon_tags_136 },
    { 0xac89699f, SUC_lexicon_tags_17 },
    { 0x768969a0, SUC_lexicon_tags_8 },
    { 0xd50969a1, SUC_lexicon_tags_47 },
    { 0xad4969a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70969ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb14969ad, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe969b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb26969b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314969b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce969bd, SUC_lexicon_tags_12 },
    { 0xb8a969be, SUC_lexicon_tags_17 },
    { 0xf86969be, SUC_lexicon_tags_27 },
    { 0xa74969c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872969ca, SUC_lexicon_tags_37 },
    { 0x33c969cb, SUC_lexicon_tags_2 },
    { 0x6f4969cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632969d6, SUC_lexicon_tags_6 },
    { 0x486969d7, SUC_lexicon_tags_45 },
    { 0x292969d6, SUC_lexicon_tags_15 },
    { 0x106969d9, SUC_lexicon_tags_14 },
    { 0x84c969da, SUC_lexicon_tags_37 },
    { 0x30c969db, SUC_lexicon_tags_17 },
    { 0xbb8969d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c0969de, SUC_lexicon_tags_135 },
    { 0x320969de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x624969e2, SUC_lexicon_tags_6 },
    { 0x220969e2, SUC_lexicon_tags_7 },
    { 0x166969e4, SUC_lexicon_tags_33 },
    { 0x276969e4, SUC_lexicon_tags_7 },
    { 0xd3a969e4, SUC_lexicon_tags_87 },
    { 0x9da969e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836969eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46969ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0969f6, SUC_lexicon_tags_25 },
    { 0x606969f6, SUC_lexicon_tags_1663 },
    { 0xfc6969f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x452969fa, SUC_lexicon_tags_8 },
    { 0xeb2969fa, SUC_lexicon_tags_16 },
    { 0x3f4969fc, SUC_lexicon_tags_23 },
    { 0x654969fb, SUC_lexicon_tags_37 },
    { 0xb70969fe, SUC_lexicon_tags_6 },
    { 0x876969fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c96a05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x34096a07, SUC_lexicon_tags_49 },
    { 0xd8e96a08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca96a0b, SUC_lexicon_tags_84 },
    { 0x61096a0c, SUC_lexicon_tags_15 },
    { 0x93496a0d, SUC_lexicon_tags_11 },
    { 0x15496a0e, SUC_lexicon_tags_11 },
    { 0xb8096a0e, SUC_lexicon_tags_25 },
    { 0x8fc96a10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3496a12, SUC_lexicon_tags_19 },
    { 0x45096a13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c696a15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5896a1c, SUC_lexicon_tags_43 },
    { 0x3c696a1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c96a23, SUC_lexicon_tags_58 },
    { 0x55696a23, SUC_lexicon_tags_25 },
    { 0x69896a25, SUC_lexicon_tags_11 },
    { 0x39c96a26, SUC_lexicon_tags_473 },
    { 0xda696a25, SUC_lexicon_tags_11 },
    { 0x1a696a23, SUC_lexicon_tags_14 },
    { 0x35e96a29, SUC_lexicon_tags_14 },
    { 0x3696a26, SUC_lexicon_tags_238 },
    { 0x5fe96a2b, SUC_lexicon_tags_25 },
    { 0x45296a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9096a2e, SUC_lexicon_tags_37 },
    { 0xb5496a2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe896a34, SUC_lexicon_tags_43 },
    { 0xdf696a34, SUC_lexicon_tags_69 },
    { 0x94c96a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1896a38, SUC_lexicon_tags_23 },
    { 0xda96a39, SUC_lexicon_tags_17 },
    { 0xc3496a3a, SUC_lexicon_tags_33 },
    { 0xdc496a38, SUC_lexicon_tags_79 },
    { 0xe96a3c, SUC_lexicon_tags_14 },
    { 0xcba96a3c, SUC_lexicon_tags_0 },
    { 0xf296a3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93896a41, SUC_lexicon_tags_25 },
    { 0xa6a96a42, SUC_lexicon_tags_11 },
    { 0x2e096a43, SUC_lexicon_tags_33 },
    { 0x50096a44, SUC_lexicon_tags_31 },
    { 0xe7696a45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd696a47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86496a4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68296a51, SUC_lexicon_tags_17 },
    { 0xa2696a51, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7896a62, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1f296a64, SUC_lexicon_tags_66 },
    { 0x11c96a64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a96a69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac96a6e, SUC_lexicon_tags_37 },
    { 0xe4096a6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8fa96a71, SUC_lexicon_tags_8 },
    { 0xdfa96a72, SUC_lexicon_tags_37 },
    { 0x57696a71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xef296a75, SUC_lexicon_tags_38 },
    { 0xb3496a76, SUC_lexicon_tags_87 },
    { 0xc8496a77, SUC_lexicon_tags_23 },
    { 0xffe96a77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd896a7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a096a84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31496a8a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c96a8d, SUC_lexicon_tags_37 },
    { 0x88e96a8e, SUC_lexicon_tags_7 },
    { 0x49c96a8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba96a94, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5896a98, SUC_lexicon_tags_11 },
    { 0x90096a99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39696aa0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a96aa6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec96aaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45496ab8, SUC_lexicon_tags_17 },
    { 0x3896ab8, SUC_lexicon_tags_14 },
    { 0x70496ab8, SUC_lexicon_tags_32 },
    { 0x30a96abb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2496abe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce96aca, SUC_lexicon_tags_17 },
    { 0x78696acb, SUC_lexicon_tags_25 },
    { 0x42696acc, SUC_lexicon_tags_49 },
    { 0x8a296acd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43296ad1, SUC_lexicon_tags_11 },
    { 0xa7a96ad1, SUC_lexicon_tags_6 },
    { 0x5b496ad2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd9296ad5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c96ad8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x43296ada, SUC_lexicon_tags_37 },
    { 0x84896adb, SUC_lexicon_tags_25 },
    { 0x9ec96ada, SUC_lexicon_tags_15 },
    { 0x58e96adc, SUC_lexicon_tags_15 },
    { 0x5496ade, SUC_lexicon_tags_15 },
    { 0x3cc96adf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6696ae2, SUC_lexicon_tags_33 },
    { 0xb3a96ae3, SUC_lexicon_tags_15 },
    { 0x75496ae3, SUC_lexicon_tags_37 },
    { 0xc0296ae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc96aeb, SUC_lexicon_tags_27 },
    { 0x31296aeb, SUC_lexicon_tags_49 },
    { 0xa9a96aec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x57a96aef, SUC_lexicon_tags_14 },
    { 0x46096aef, SUC_lexicon_tags_2 },
    { 0xb4496af1, SUC_lexicon_tags_25 },
    { 0xf8096af2, SUC_lexicon_tags_17 },
    { 0xc296af1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x51096af6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39696afa, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x8d896afc, SUC_lexicon_tags_19 },
    { 0xa1896afd, SUC_lexicon_tags_70 },
    { 0x9a496afe, SUC_lexicon_tags_58 },
    { 0xd5896afe, SUC_lexicon_tags_11 },
    { 0x65c96b00, SUC_lexicon_tags_9 },
    { 0xf6696aff, SUC_lexicon_tags_37 },
    { 0xfec96afd, SUC_lexicon_tags_29 },
    { 0xd6896b03, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf5496b05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e96b0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6696b11, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e896b1a, SUC_lexicon_tags_17 },
    { 0x8a096b1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f696b20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20296b24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5896b27, SUC_lexicon_tags_43 },
    { 0x46e96b28, SUC_lexicon_tags_8 },
    { 0x6d896b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e96b2c, SUC_lexicon_tags_32 },
    { 0x87a96b2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe8a96b2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52496b32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83896b37, SUC_lexicon_tags_37 },
    { 0x95696b37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e96b3e, SUC_lexicon_tags_15 },
    { 0x77696b3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba96b46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa96b4f, SUC_lexicon_tags_1 },
    { 0xc5e96b50, SUC_lexicon_tags_32 },
    { 0x61c96b50, SUC_lexicon_tags_12 },
    { 0xdf496b50, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2096b54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a96b58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c96b60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7496b65, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7e296b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76896b6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c96b6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc96b71, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e96b79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a96b7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2496b80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e96b83, SUC_lexicon_tags_162 },
    { 0xfaa96b83, SUC_lexicon_tags_55 },
    { 0x21696b85, SUC_lexicon_tags_22 },
    { 0x9ba96b85, SUC_lexicon_tags_124 },
    { 0x37496b87, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c296b8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95a96b8c, SUC_lexicon_tags_10 },
    { 0xaea96b8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e96b91, SUC_lexicon_tags_63 },
    { 0x2fa96b91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0c96b94, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a96b98, SUC_lexicon_tags_17 },
    { 0xce296b98, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99296b9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c96b9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89696ba2, SUC_lexicon_tags_37 },
    { 0xeb096ba3, SUC_lexicon_tags_25 },
    { 0x23e96ba3, SUC_lexicon_tags_28 },
    { 0x1ae96ba5, SUC_lexicon_tags_15 },
    { 0xad696ba6, SUC_lexicon_tags_9 },
    { 0x7ca96ba6, SUC_lexicon_tags_25 },
    { 0xdd496ba4, SUC_lexicon_tags_136 },
    { 0x7a896ba9, SUC_lexicon_tags_21 },
    { 0x67896ba6, SUC_lexicon_tags_25 },
    { 0xf3e96bab, SUC_lexicon_tags_2 },
    { 0xbc496bac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc96baf, SUC_lexicon_tags_60 },
    { 0x93a96baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c96bb3, SUC_lexicon_tags_25 },
    { 0x1c296bb4, SUC_lexicon_tags_37 },
    { 0x49e96bb4, SUC_lexicon_tags_12 },
    { 0xe6896bb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1296bb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0896bba, SUC_lexicon_tags_12 },
    { 0x58496bbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44a96bbd, SUC_lexicon_tags_37 },
    { 0xc0296bbe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbca96bc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x66e96bc4, SUC_lexicon_tags_38 },
    { 0xc6a96bc4, SUC_lexicon_tags_9 },
    { 0xb5696bc6, SUC_lexicon_tags_46 },
    { 0xacc96bc7, SUC_lexicon_tags_25 },
    { 0x1b496bc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e96bcb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x16096bce, SUC_lexicon_tags_25 },
    { 0xcb696bcf, SUC_lexicon_tags_35 },
    { 0x9c96bd0, SUC_lexicon_tags_15 },
    { 0xd4096bd1, SUC_lexicon_tags_19 },
    { 0xc9e96bd0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8896bdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a96be2, SUC_lexicon_tags_15 },
    { 0x9e496be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x94096be6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70896bec, SUC_lexicon_tags_45 },
    { 0x5dc96bed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28496bef, SUC_lexicon_tags_198 },
    { 0x40a96bf0, SUC_lexicon_tags_11 },
    { 0xcce96bf0, SUC_lexicon_tags_0 },
    { 0x99296bf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8096bf5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe2096bf7, SUC_lexicon_tags_1 },
    { 0x73a96bf8, SUC_lexicon_tags_47 },
    { 0x1fc96bf8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7fe96bfb, SUC_lexicon_tags_37 },
    { 0xaf696bfc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d296bff, SUC_lexicon_tags_15 },
    { 0x2fc96c00, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0696c07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14096c09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f096c0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a496c10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2696c12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbc096c14, SUC_lexicon_tags_37 },
    { 0x19696c14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4296c17, SUC_lexicon_tags_43 },
    { 0x8d096c18, SUC_lexicon_tags_2 },
    { 0xd8e96c19, SUC_lexicon_tags_37 },
    { 0x5ca96c18, SUC_lexicon_tags_7 },
    { 0x87a96c19, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36496c23, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9a496c25, SUC_lexicon_tags_27 },
    { 0x81e96c26, SUC_lexicon_tags_9 },
    { 0xf1e96c26, SUC_lexicon_tags_31 },
    { 0xfc896c28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf3896c2a, SUC_lexicon_tags_65 },
    { 0x88a96c2b, SUC_lexicon_tags_47 },
    { 0xabe96c2b, SUC_lexicon_tags_31 },
    { 0x7c296c2d, SUC_lexicon_tags_15 },
    { 0x9ca96c2e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b096c31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83e96c33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99896c35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6696c3a, SUC_lexicon_tags_14 },
    { 0x46696c3b, SUC_lexicon_tags_25 },
    { 0xba696c3c, SUC_lexicon_tags_12 },
    { 0xc2096c3b, SUC_lexicon_tags_25 },
    { 0x35096c3e, SUC_lexicon_tags_37 },
    { 0x52e96c3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b496c42, SUC_lexicon_tags_33 },
    { 0x80496c43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb296c4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1a96c4c, SUC_lexicon_tags_28 },
    { 0xe1696c4c, SUC_lexicon_tags_11 },
    { 0x92896c4e, SUC_lexicon_tags_14 },
    { 0x7a296c4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96c57, SUC_lexicon_tags_37 },
    { 0x5d896c58, SUC_lexicon_tags_15 },
    { 0x77696c57, SUC_lexicon_tags_2 },
    { 0x59c96c59, SUC_lexicon_tags_27 },
    { 0x40c96c58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23896c62, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4096c66, SUC_lexicon_tags_60 },
    { 0x97c96c66, SUC_lexicon_tags_15 },
    { 0xdca96c68, SUC_lexicon_tags_15 },
    { 0x1a96c68, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2496c6d, SUC_lexicon_tags_29 },
    { 0xc5c96c6e, SUC_lexicon_tags_37 },
    { 0x4aa96c6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8b896c71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa096c74, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x61696c76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc96c7b, SUC_lexicon_tags_25 },
    { 0x98096c7b, SUC_lexicon_tags_53 },
    { 0xeba96c7d, SUC_lexicon_tags_2 },
    { 0x4ec96c7b, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a496c82, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x80696c84, SUC_lexicon_tags_113 },
    { 0xa6a96c84, SUC_lexicon_tags_30 },
    { 0x38e96c86, SUC_lexicon_tags_11 },
    { 0xde696c86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89496c8e, SUC_lexicon_tags_25 },
    { 0x89a96c8e, SUC_lexicon_tags_27 },
    { 0x3e96c90, SUC_lexicon_tags_0 },
    { 0x56496c91, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14096c96, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76496ca5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70c96ca7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x89096caa, SUC_lexicon_tags_11 },
    { 0xe5296caa, SUC_lexicon_tags_37 },
    { 0x35896cac, SUC_lexicon_tags_37 },
    { 0xa6c96cad, SUC_lexicon_tags_37 },
    { 0xcfc96cae, SUC_lexicon_tags_27 },
    { 0x75496caf, SUC_lexicon_tags_14 },
    { 0x40496cae, SUC_lexicon_tags_11 },
    { 0x7ba96caf, SUC_lexicon_tags_6 },
    { 0xdbc96cb2, SUC_lexicon_tags_45 },
    { 0x4a096cb0, SUC_lexicon_tags_17 },
    { 0xaa696cab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1096cb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a96cbb, SUC_lexicon_tags_37 },
    { 0xb2896cbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88096cc2, SUC_lexicon_tags_37 },
    { 0xd1296cc3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6296cc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76096ccb, SUC_lexicon_tags_27 },
    { 0x27896ccb, SUC_lexicon_tags_32 },
    { 0x31296ccc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc96cd3, SUC_lexicon_tags_34 },
    { 0x90496cd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab096cd8, SUC_lexicon_tags_40 },
    { 0x93096cd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44096cdd, SUC_lexicon_tags_14 },
    { 0x1e096cde, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7096ce1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d496cea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb496cf3, SUC_lexicon_tags_2 },
    { 0x9ea96cf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc96cf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc96d01, SUC_lexicon_tags_11 },
    { 0x16e96d01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb696d04, SUC_lexicon_tags_47 },
    { 0x6fa96d04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a96d08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x31096d0a, SUC_lexicon_tags_15 },
    { 0xe3496d0b, SUC_lexicon_tags_25 },
    { 0xa5496d0b, SUC_lexicon_tags_7 },
    { 0xf8096d0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2296d17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa96d1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87696d1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda96d23, SUC_lexicon_tags_37 },
    { 0xc8096d24, SUC_lexicon_tags_1 },
    { 0x3b096d25, SUC_lexicon_tags_39 },
    { 0xe3896d26, SUC_lexicon_tags_1 },
    { 0x9f296d23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0496d30, SUC_lexicon_tags_23 },
    { 0x16e96d30, SUC_lexicon_tags_56 },
    { 0x66496d32, SUC_lexicon_tags_17 },
    { 0x50896d33, SUC_lexicon_tags_82 },
    { 0x9ae96d30, SUC_lexicon_tags_11 },
    { 0xd1c96d30, SUC_lexicon_tags_37 },
    { 0xd3896d32, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e96d3d, SUC_lexicon_tags_17 },
    { 0x1ee96d3e, SUC_lexicon_tags_27 },
    { 0xfea96d3f, SUC_lexicon_tags_7 },
    { 0x93696d3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41096d44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496d4e, SUC_lexicon_tags_2 },
    { 0x68e96d4f, SUC_lexicon_tags_17 },
    { 0x2f496d50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11296d54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa896d57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee96d5a, SUC_lexicon_tags_71 },
    { 0x3c696d5a, SUC_lexicon_tags_26 },
    { 0x9b496d5b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x6fa96d5e, SUC_lexicon_tags_77 },
    { 0x9a096d5f, SUC_lexicon_tags_28 },
    { 0xb3696d60, SUC_lexicon_tags_31 },
    { 0x2c296d61, SUC_lexicon_tags_33 },
    { 0x4be96d62, SUC_lexicon_tags_25 },
    { 0x9896d60, SUC_lexicon_tags_55 },
    { 0x5d496d64, SUC_lexicon_tags_1 },
    { 0xfba96d65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18296d6a, SUC_lexicon_tags_156 },
    { 0x29a96d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac96d6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b896d73, SUC_lexicon_tags_45 },
    { 0x4bc96d74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32096d78, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x29296d7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5296d81, SUC_lexicon_tags_15 },
    { 0x8c496d82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec96d85, SUC_lexicon_tags_63 },
    { 0x3ee96d85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18096d8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f096d90, SUC_lexicon_tags_39 },
    { 0x26c96d91, SUC_lexicon_tags_2 },
    { 0xe4a96d92, SUC_lexicon_tags_66 },
    { 0x45e96d92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a696d9b, SUC_lexicon_tags_14 },
    { 0xac96d9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1be96d9f, SUC_lexicon_tags_12 },
    { 0xfba96da0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a96da4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba896daa, SUC_lexicon_tags_17 },
    { 0xa6096daa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9c696dad, SUC_lexicon_tags_14 },
    { 0x27c96dae, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb496db1, SUC_lexicon_tags_38 },
    { 0xa0a96db1, SUC_lexicon_tags_15 },
    { 0xb2696db3, SUC_lexicon_tags_17 },
    { 0x66496db2, SUC_lexicon_tags_61 },
    { 0x58296db5, SUC_lexicon_tags_48 },
    { 0xacc96db6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21c96db8, SUC_lexicon_tags_33 },
    { 0x50c96db8, SUC_lexicon_tags_23 },
    { 0x2fe96dba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c96dbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c96dc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e96dc5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe9c96dc7, SUC_lexicon_tags_14 },
    { 0x29496dc8, SUC_lexicon_tags_12 },
    { 0xe5896dc9, SUC_lexicon_tags_26 },
    { 0xad096dca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe1296dcc, SUC_lexicon_tags_25 },
    { 0xfd296dcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f696dcf, SUC_lexicon_tags_37 },
    { 0x23896dd0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c96dd3, SUC_lexicon_tags_32 },
    { 0xe4496dd3, SUC_lexicon_tags_15 },
    { 0x89296dd5, SUC_lexicon_tags_27 },
    { 0xb2c96dd3, SUC_lexicon_tags_14 },
    { 0xefa96dd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d296dda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9dc96ddc, SUC_lexicon_tags_17 },
    { 0x88a96ddc, SUC_lexicon_tags_7 },
    { 0xfd096dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80496de0, SUC_lexicon_tags_61 },
    { 0xb1496de0, SUC_lexicon_tags_419 },
    { 0xdd096de2, SUC_lexicon_tags_5 },
    { 0x696de3, SUC_lexicon_tags_2 },
    { 0xb2c96de4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2b496de6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec96de9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd2a96deb, SUC_lexicon_tags_45 },
    { 0x24096deb, SUC_lexicon_tags_25 },
    { 0x86296deb, SUC_lexicon_tags_18 },
    { 0x42696deb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70696df1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee96df5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23296df7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6896dfa, SUC_lexicon_tags_11 },
    { 0x40c96dfb, SUC_lexicon_tags_0 },
    { 0xd7a96dfa, SUC_lexicon_tags_25 },
    { 0x24896dfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58296dff, SUC_lexicon_tags_170 },
    { 0x13c96dff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2f296e02, SUC_lexicon_tags_23 },
    { 0x76696e03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a496e06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e96e0a, SUC_lexicon_tags_163 },
    { 0xbe696e0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e96e10, SUC_lexicon_tags_28 },
    { 0xd0a96e11, SUC_lexicon_tags_0 },
    { 0x45c96e12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bc96e14, SUC_lexicon_tags_19 },
    { 0xb7896e15, SUC_lexicon_tags_11 },
    { 0xea096e15, SUC_lexicon_tags_11 },
    { 0x3e496e17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfc96e19, SUC_lexicon_tags_48 },
    { 0x4a096e19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc5a96e1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e96e22, SUC_lexicon_tags_30 },
    { 0x30c96e23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ee96e25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a496e27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28096e2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4296e35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0e96e37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59896e3b, SUC_lexicon_tags_17 },
    { 0xed896e3c, SUC_lexicon_tags_275 },
    { 0xf7896e3c, SUC_lexicon_tags_43 },
    { 0x88896e3e, SUC_lexicon_tags_48 },
    { 0xed496e3f, SUC_lexicon_tags_11 },
    { 0x5c896e3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93096e44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4096e46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca96e49, SUC_lexicon_tags_61 },
    { 0xcc496e4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e096e4c, SUC_lexicon_tags_33 },
    { 0xe3296e4d, SUC_lexicon_tags_6 },
    { 0x84e96e4e, SUC_lexicon_tags_6 },
    { 0xca496e4f, SUC_lexicon_tags_32 },
    { 0x47a96e4f, SUC_lexicon_tags_15 },
    { 0xa5696e51, SUC_lexicon_tags_47 },
    { 0x7a96e52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16696e55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a96e58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e96e5b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x41a96e5d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9b896e5f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc96e64, SUC_lexicon_tags_45 },
    { 0x42296e65, SUC_lexicon_tags_8 },
    { 0x6c096e66, SUC_lexicon_tags_43 },
    { 0xd6896e67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34096e6c, SUC_lexicon_tags_33 },
    { 0x4896e6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3096e75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25496e78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2896e7f, SUC_lexicon_tags_26 },
    { 0x6ca96e80, SUC_lexicon_tags_14 },
    { 0x87696e81, SUC_lexicon_tags_1358 },
    { 0x24a96e82, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x80896e85, SUC_lexicon_tags_14 },
    { 0xa496e86, SUC_lexicon_tags_33 },
    { 0xc0e96e87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a96e8c, SUC_lexicon_tags_226 },
    { 0xcb296e8d, SUC_lexicon_tags_7 },
    { 0x7ca96e8e, SUC_lexicon_tags_10 },
    { 0x99496e8e, SUC_lexicon_tags_38 },
    { 0x41896e90, SUC_lexicon_tags_11 },
    { 0x35a96e8e, SUC_lexicon_tags_15 },
    { 0x5d296e92, SUC_lexicon_tags_69 },
    { 0x4cc96e93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e96e96, SUC_lexicon_tags_31 },
    { 0x67a96e97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45896e9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e496e9c, SUC_lexicon_tags_17 },
    { 0x2e496e9d, SUC_lexicon_tags_19 },
    { 0x68096e9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7896ea2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90e96ea4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bc96ea6, SUC_lexicon_tags_10 },
    { 0x71696ea7, SUC_lexicon_tags_10 },
    { 0xdb496ea6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c96eac, SUC_lexicon_tags_33 },
    { 0x38496ead, SUC_lexicon_tags_104 },
    { 0xa2496eae, SUC_lexicon_tags_12 },
    { 0x5a096eaf, SUC_lexicon_tags_14 },
    { 0x32496eaf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0096eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79496eb8, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x4f296eba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75096ebe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa96ec2, SUC_lexicon_tags_38 },
    { 0x2c296ec3, SUC_lexicon_tags_15 },
    { 0x9ca96ec4, SUC_lexicon_tags_11 },
    { 0xec296ec5, SUC_lexicon_tags_10 },
    { 0xee296ec2, SUC_lexicon_tags_1 },
    { 0x85a96ec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99496eca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71296ed0, SUC_lexicon_tags_7 },
    { 0xc6a96ed0, SUC_lexicon_tags_116 },
    { 0x87c96ed2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc096ed5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9496ed8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8296edc, SUC_lexicon_tags_85 },
    { 0x2aa96edd, SUC_lexicon_tags_14 },
    { 0x4ba96edc, SUC_lexicon_tags_79 },
    { 0x6be96edd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8096ee4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41696eef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f896ef1, SUC_lexicon_tags_23 },
    { 0x85496ef2, SUC_lexicon_tags_15 },
    { 0x7de96ef3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8a96ef5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25096efb, SUC_lexicon_tags_27 },
    { 0xea696efb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e96f02, SUC_lexicon_tags_25 },
    { 0x21e96f03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a96f06, SUC_lexicon_tags_39 },
    { 0xbec96f07, SUC_lexicon_tags_48 },
    { 0x4d296f08, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb496f11, SUC_lexicon_tags_0 },
    { 0x52896f11, SUC_lexicon_tags_17 },
    { 0xa7c96f12, SUC_lexicon_tags_17 },
    { 0x2a096f14, SUC_lexicon_tags_31 },
    { 0x49096f15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24696f18, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe096f1d, SUC_lexicon_tags_25 },
    { 0x29096f1e, SUC_lexicon_tags_63 },
    { 0x19096f1f, SUC_lexicon_tags_55 },
    { 0x76e96f1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c96f23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1296f26, SUC_lexicon_tags_55 },
    { 0x30496f26, SUC_lexicon_tags_64 },
    { 0x33e96f28, SUC_lexicon_tags_47 },
    { 0xc5296f29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c296f2e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52a96f30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e96f33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9a96f35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd496f37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad696f39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f496f3b, SUC_lexicon_tags_24 },
    { 0xbf096f3c, SUC_lexicon_tags_0 },
    { 0x9b896f3d, SUC_lexicon_tags_12 },
    { 0x44096f3d, SUC_lexicon_tags_49 },
    { 0xa0a96f3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xce696f42, SUC_lexicon_tags_56 },
    { 0x40e96f43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39296f45, SUC_lexicon_tags_37 },
    { 0xd6096f46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86696f48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87096f4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda96f51, SUC_lexicon_tags_11 },
    { 0x3c296f52, SUC_lexicon_tags_17 },
    { 0x8fc96f53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfee96f55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e96f59, SUC_lexicon_tags_0 },
    { 0x97496f5a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca96f5d, SUC_lexicon_tags_7 },
    { 0xf8e96f5e, SUC_lexicon_tags_14 },
    { 0x25896f5f, SUC_lexicon_tags_17 },
    { 0xace96f60, SUC_lexicon_tags_19 },
    { 0xce96f60, SUC_lexicon_tags_25 },
    { 0xae96f62, SUC_lexicon_tags_21 },
    { 0x4696f5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d696f67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12496f6c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3896f71, SUC_lexicon_tags_30 },
    { 0x92496f72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27696f79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8296f7e, SUC_lexicon_tags_46 },
    { 0x2e296f7f, SUC_lexicon_tags_19 },
    { 0x4a496f80, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93296f84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26096f88, SUC_lexicon_tags_38 },
    { 0x33896f89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37496f8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x77896f8e, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xca696f91, SUC_lexicon_tags_37 },
    { 0x41a96f92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x97696f95, SUC_lexicon_tags_46 },
    { 0x43496f96, SUC_lexicon_tags_37 },
    { 0x4b896f96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8696f9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f496f9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc5896f9e, SUC_lexicon_tags_11 },
    { 0x94c96f9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2896fa2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc96fa5, SUC_lexicon_tags_55 },
    { 0x50a96fa6, SUC_lexicon_tags_23 },
    { 0x8ae96fa6, SUC_lexicon_tags_17 },
    { 0x2f096fa8, SUC_lexicon_tags_37 },
    { 0xce096fa8, SUC_lexicon_tags_15 },
    { 0xd1c96fa8, SUC_lexicon_tags_33 },
    { 0xa7c96fa5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x56296fad, SUC_lexicon_tags_15 },
    { 0x65096fad, SUC_lexicon_tags_1 },
    { 0xe1696faf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c96fb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf696fb6, SUC_lexicon_tags_28 },
    { 0x47e96fb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5696fbb, SUC_lexicon_tags_27 },
    { 0xeaa96fbc, SUC_lexicon_tags_1 },
    { 0x2d096fbd, SUC_lexicon_tags_15 },
    { 0xc696fbe, SUC_lexicon_tags_79 },
    { 0x10e96fbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f296fc2, SUC_lexicon_tags_15 },
    { 0x3c96fc3, SUC_lexicon_tags_23 },
    { 0x98296fc4, SUC_lexicon_tags_49 },
    { 0x71296fc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a96fca, SUC_lexicon_tags_9 },
    { 0x9b496fca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2896fcd, SUC_lexicon_tags_7 },
    { 0xfd296fcd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ee96fd0, SUC_lexicon_tags_11 },
    { 0x90c96fd0, SUC_lexicon_tags_8 },
    { 0xc6296fd2, SUC_lexicon_tags_15 },
    { 0x4c696fd2, SUC_lexicon_tags_65 },
    { 0xb8696fd0, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xb1a96fd6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e696fd9, SUC_lexicon_tags_68 },
    { 0x30096fd9, SUC_lexicon_tags_23 },
    { 0xa6496fdb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e496fe1, SUC_lexicon_tags_79 },
    { 0xc1e96fe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc096fe5, SUC_lexicon_tags_46 },
    { 0x3ec96fe5, SUC_lexicon_tags_26 },
    { 0x79a96fe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc96fea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbc296fec, SUC_lexicon_tags_63 },
    { 0x8de96fec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9896ff4, SUC_lexicon_tags_2 },
    { 0x6e896ff5, SUC_lexicon_tags_43 },
    { 0xcc896ff6, SUC_lexicon_tags_11 },
    { 0x23c96ff4, SUC_lexicon_tags_17 },
    { 0xf8096ff8, SUC_lexicon_tags_47 },
    { 0xa2a96ff6, SUC_lexicon_tags_17 },
    { 0x9b096ffa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a096fff, SUC_lexicon_tags_27 },
    { 0x9fc97000, SUC_lexicon_tags_11 },
    { 0xc0497001, SUC_lexicon_tags_39 },
    { 0x5f497000, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x69e97004, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e897007, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e697009, SUC_lexicon_tags_4 },
    { 0xcc89700a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68897011, SUC_lexicon_tags_25 },
    { 0x4fe97012, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f497014, SUC_lexicon_tags_37 },
    { 0xc6297015, SUC_lexicon_tags_17 },
    { 0x9a297016, SUC_lexicon_tags_37 },
    { 0xe6897017, SUC_lexicon_tags_11 },
    { 0x3c97016, SUC_lexicon_tags_37 },
    { 0xf1e97019, SUC_lexicon_tags_12 },
    { 0x45e97016, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda89701e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80697024, SUC_lexicon_tags_1 },
    { 0x2e697025, SUC_lexicon_tags_7 },
    { 0xf8297025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdba97029, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9702d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5929702f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25697032, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8409703d, SUC_lexicon_tags_30 },
    { 0xba09703e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c97046, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc7c97048, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xd1c9704a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8069704e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54097051, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x62897054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2669705e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c97063, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbb897065, SUC_lexicon_tags_25 },
    { 0x5cc97066, SUC_lexicon_tags_7 },
    { 0x2d097065, SUC_lexicon_tags_17 },
    { 0x96697066, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee9706e, SUC_lexicon_tags_14 },
    { 0xdd09706f, SUC_lexicon_tags_68 },
    { 0x6f697070, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e97074, SUC_lexicon_tags_1 },
    { 0xe3e97074, SUC_lexicon_tags_17 },
    { 0xfe097075, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb497079, SUC_lexicon_tags_37 },
    { 0xbba97079, SUC_lexicon_tags_43 },
    { 0x4da9707b, SUC_lexicon_tags_17 },
    { 0xc389707c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5c09707e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13497080, SUC_lexicon_tags_37 },
    { 0x87097081, SUC_lexicon_tags_2 },
    { 0xbb497081, SUC_lexicon_tags_12 },
    { 0x53297081, SUC_lexicon_tags_17 },
    { 0xc8e97084, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869708b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb897099, SUC_lexicon_tags_49 },
    { 0x1229709a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x909709c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdcc9709e, SUC_lexicon_tags_35 },
    { 0xc609709f, SUC_lexicon_tags_63 },
    { 0x7a4970a0, SUC_lexicon_tags_6 },
    { 0xfd2970a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c970a4, SUC_lexicon_tags_37 },
    { 0xc12970a5, SUC_lexicon_tags_12 },
    { 0x4c0970a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e970a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8970ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73e970af, SUC_lexicon_tags_12 },
    { 0x89e970b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x468970b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20a970b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c970b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad4970b9, SUC_lexicon_tags_37 },
    { 0x6a0970ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588970be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x896970c1, SUC_lexicon_tags_28 },
    { 0xd90970c2, SUC_lexicon_tags_12 },
    { 0x5f6970c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3a4970c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf4970c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc970cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe970d0, SUC_lexicon_tags_3 },
    { 0xaa970d0, SUC_lexicon_tags_15 },
    { 0x894970d0, SUC_lexicon_tags_45 },
    { 0xb0970d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x686970d6, SUC_lexicon_tags_70 },
    { 0xc28970d6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x890970da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae970dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x50c970df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58970e2, SUC_lexicon_tags_37 },
    { 0x480970e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee970e5, SUC_lexicon_tags_39 },
    { 0x374970e6, SUC_lexicon_tags_11 },
    { 0x95e970e7, SUC_lexicon_tags_15 },
    { 0x8c8970e8, SUC_lexicon_tags_45 },
    { 0x3c4970e9, SUC_lexicon_tags_117 },
    { 0xc3a970e9, SUC_lexicon_tags_49 },
    { 0xbe6970e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc0970ed, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c970f1, SUC_lexicon_tags_40 },
    { 0x51a970f2, SUC_lexicon_tags_19 },
    { 0xfe2970f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2970f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc970fa, SUC_lexicon_tags_38 },
    { 0x92970fb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a97101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e97104, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c697106, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68497108, SUC_lexicon_tags_15 },
    { 0x4c897109, SUC_lexicon_tags_215 },
    { 0x1d697108, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c9710e, SUC_lexicon_tags_1 },
    { 0x8309710f, SUC_lexicon_tags_60 },
    { 0xf4a9710e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c97114, SUC_lexicon_tags_14 },
    { 0xd7497114, SUC_lexicon_tags_31 },
    { 0xe8097114, SUC_lexicon_tags_44 },
    { 0x9f097116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc9711a, SUC_lexicon_tags_12 },
    { 0x4e29711b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x1789711d, SUC_lexicon_tags_16 },
    { 0x8d69711e, SUC_lexicon_tags_12 },
    { 0x4409711e, SUC_lexicon_tags_11 },
    { 0xc1e97120, SUC_lexicon_tags_15 },
    { 0x6b697121, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b097129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa469712b, SUC_lexicon_tags_68 },
    { 0xcbe9712c, SUC_lexicon_tags_11 },
    { 0x8fa9712b, SUC_lexicon_tags_25 },
    { 0x7e9712e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x3297130, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb697132, SUC_lexicon_tags_42 },
    { 0xf5a97133, SUC_lexicon_tags_7 },
    { 0x33097133, SUC_lexicon_tags_17 },
    { 0x53a97132, SUC_lexicon_tags_11 },
    { 0x74e97134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9713a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfca9713d, SUC_lexicon_tags_14 },
    { 0x3ec9713d, SUC_lexicon_tags_37 },
    { 0xf809713f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb249714a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff29714c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec97151, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf897153, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa497155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17a97157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69715b, SUC_lexicon_tags_7 },
    { 0x7ec9715b, SUC_lexicon_tags_7 },
    { 0xe1e9715c, SUC_lexicon_tags_1 },
    { 0x47a9715e, SUC_lexicon_tags_33 },
    { 0x83a9715f, SUC_lexicon_tags_2 },
    { 0xc0897160, SUC_lexicon_tags_49 },
    { 0xdcc97160, SUC_lexicon_tags_37 },
    { 0x28c97162, SUC_lexicon_tags_25 },
    { 0x35497160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e097165, SUC_lexicon_tags_14 },
    { 0xba097165, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe229716d, SUC_lexicon_tags_33 },
    { 0x1629716e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec297170, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f297172, SUC_lexicon_tags_25 },
    { 0x41097173, SUC_lexicon_tags_26 },
    { 0xa9c97173, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce97178, SUC_lexicon_tags_23 },
    { 0x16897179, SUC_lexicon_tags_17 },
    { 0xa269717a, SUC_lexicon_tags_1 },
    { 0xfe49717b, SUC_lexicon_tags_27 },
    { 0x79a9717c, SUC_lexicon_tags_1 },
    { 0xd809717a, SUC_lexicon_tags_37 },
    { 0xdaa9717b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7897180, SUC_lexicon_tags_43 },
    { 0x61e97181, SUC_lexicon_tags_37 },
    { 0xf8697182, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6ae97184, SUC_lexicon_tags_38 },
    { 0xc7c97184, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c97189, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe9718e, SUC_lexicon_tags_6 },
    { 0x2009718f, SUC_lexicon_tags_2 },
    { 0xe009718f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c97193, SUC_lexicon_tags_17 },
    { 0x23c97194, SUC_lexicon_tags_10 },
    { 0x53c97194, SUC_lexicon_tags_57 },
    { 0x1f097196, SUC_lexicon_tags_1 },
    { 0xdda97197, SUC_lexicon_tags_30 },
    { 0xcc297198, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7669719b, SUC_lexicon_tags_2 },
    { 0xa69719c, SUC_lexicon_tags_17 },
    { 0x2129719c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe971a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0971a6, SUC_lexicon_tags_19 },
    { 0x85e971a7, SUC_lexicon_tags_40 },
    { 0x18971a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c971ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2971b1, SUC_lexicon_tags_55 },
    { 0xa2971b2, SUC_lexicon_tags_17 },
    { 0xde0971b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52971be, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56971c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96971c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60c971c7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc971cb, SUC_lexicon_tags_15 },
    { 0x7a6971cb, SUC_lexicon_tags_1 },
    { 0x9a6971cd, SUC_lexicon_tags_17 },
    { 0x308971ce, SUC_lexicon_tags_15 },
    { 0xd7c971ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa6971d1, SUC_lexicon_tags_38 },
    { 0xf0a971d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e971d9, SUC_lexicon_tags_25 },
    { 0xff2971da, SUC_lexicon_tags_26 },
    { 0xa5e971d9, SUC_lexicon_tags_1 },
    { 0x4a971db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x75c971de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a971e2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x462971e4, SUC_lexicon_tags_17 },
    { 0xb32971e5, SUC_lexicon_tags_23 },
    { 0x700971e6, SUC_lexicon_tags_16 },
    { 0x300971e7, SUC_lexicon_tags_0 },
    { 0x102971e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2971ec, SUC_lexicon_tags_23 },
    { 0x5ea971ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2971f0, SUC_lexicon_tags_10 },
    { 0xdc2971f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd10971f3, SUC_lexicon_tags_2 },
    { 0x5c2971f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x952971f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba971fc, SUC_lexicon_tags_37 },
    { 0xde971fd, SUC_lexicon_tags_5 },
    { 0x46c971fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97697205, SUC_lexicon_tags_37 },
    { 0x6e697206, SUC_lexicon_tags_79 },
    { 0xefc97207, SUC_lexicon_tags_15 },
    { 0x58097206, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfe29720a, SUC_lexicon_tags_17 },
    { 0x1ee9720b, SUC_lexicon_tags_21 },
    { 0x3f29720c, SUC_lexicon_tags_12 },
    { 0x4769720d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21297211, SUC_lexicon_tags_49 },
    { 0xbb097211, SUC_lexicon_tags_39 },
    { 0x39097213, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a97219, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc509721b, SUC_lexicon_tags_7 },
    { 0x3ce9721c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c9721f, SUC_lexicon_tags_2 },
    { 0xe429721f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829722a, SUC_lexicon_tags_60 },
    { 0x2029722b, SUC_lexicon_tags_25 },
    { 0x14a9722a, SUC_lexicon_tags_82 },
    { 0x6f69722d, SUC_lexicon_tags_29 },
    { 0x30a9722e, SUC_lexicon_tags_7 },
    { 0x3fa9722f, SUC_lexicon_tags_38 },
    { 0xb4297230, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69097234, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbe697236, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe9723a, SUC_lexicon_tags_14 },
    { 0xe929723b, SUC_lexicon_tags_33 },
    { 0xe249723c, SUC_lexicon_tags_25 },
    { 0x2669723a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66897242, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0097248, SUC_lexicon_tags_45 },
    { 0x6097249, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7789724d, SUC_lexicon_tags_25 },
    { 0x2669724e, SUC_lexicon_tags_12 },
    { 0x18e9724d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3097254, SUC_lexicon_tags_2 },
    { 0x6d097255, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fe97257, SUC_lexicon_tags_11 },
    { 0x42c97258, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e9725a, SUC_lexicon_tags_1 },
    { 0x2269725b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41c9725d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a97261, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a97264, SUC_lexicon_tags_17 },
    { 0xe6497265, SUC_lexicon_tags_104 },
    { 0x22c97266, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75c97268, SUC_lexicon_tags_17 },
    { 0xbfe97269, SUC_lexicon_tags_63 },
    { 0x47497269, SUC_lexicon_tags_45 },
    { 0xf349726b, SUC_lexicon_tags_37 },
    { 0xf9c9726c, SUC_lexicon_tags_27 },
    { 0x2929726d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb809726f, SUC_lexicon_tags_1 },
    { 0x5fe97270, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c97275, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe7e97277, SUC_lexicon_tags_9 },
    { 0x4da97278, SUC_lexicon_tags_84 },
    { 0x3c297279, SUC_lexicon_tags_2 },
    { 0x2e09727a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8f09727c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86697281, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54497283, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc897285, SUC_lexicon_tags_0 },
    { 0x35697286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac9728a, SUC_lexicon_tags_8 },
    { 0xb5a9728b, SUC_lexicon_tags_27 },
    { 0xbf49728c, SUC_lexicon_tags_27 },
    { 0xda69728d, SUC_lexicon_tags_43 },
    { 0x6049728d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16897290, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb897293, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x38897295, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea97298, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa709729f, SUC_lexicon_tags_29 },
    { 0xb1a972a0, SUC_lexicon_tags_35 },
    { 0xfb29729f, SUC_lexicon_tags_26 },
    { 0x2929729f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x566972a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0972a8, SUC_lexicon_tags_37 },
    { 0x4d4972a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe6e972ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc972ae, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c972b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede972ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16972c0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb4a972c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x576972c5, SUC_lexicon_tags_87 },
    { 0x960972c6, SUC_lexicon_tags_9 },
    { 0x1e6972c7, SUC_lexicon_tags_30 },
    { 0x28a972c7, SUC_lexicon_tags_37 },
    { 0xd30972c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6972cd, SUC_lexicon_tags_12 },
    { 0xcc6972cd, SUC_lexicon_tags_2 },
    { 0x85a972cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6972d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6972d5, SUC_lexicon_tags_7 },
    { 0x6c6972d6, SUC_lexicon_tags_12 },
    { 0x348972d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba972da, SUC_lexicon_tags_2 },
    { 0x2dc972da, SUC_lexicon_tags_0 },
    { 0x1be972dc, SUC_lexicon_tags_37 },
    { 0x8d0972db, SUC_lexicon_tags_124 },
    { 0xf3e972de, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40972e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206972e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa972e8, SUC_lexicon_tags_45 },
    { 0x886972e9, SUC_lexicon_tags_695 },
    { 0xa5c972ea, SUC_lexicon_tags_12 },
    { 0xde4972eb, SUC_lexicon_tags_37 },
    { 0xc62972ea, SUC_lexicon_tags_17 },
    { 0xca6972ed, SUC_lexicon_tags_7 },
    { 0x748972ee, SUC_lexicon_tags_11 },
    { 0xb7a972e8, SUC_lexicon_tags_37 },
    { 0x164972f0, SUC_lexicon_tags_112 },
    { 0x898972f1, SUC_lexicon_tags_87 },
    { 0xa5c972f1, SUC_lexicon_tags_14 },
    { 0xa82972f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3fc972f5, SUC_lexicon_tags_187 },
    { 0x5a4972f6, SUC_lexicon_tags_47 },
    { 0x6f4972f6, SUC_lexicon_tags_29 },
    { 0x8dc972f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300972fc, SUC_lexicon_tags_17 },
    { 0x828972fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a697301, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3ca97303, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb097307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9730a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b497310, SUC_lexicon_tags_17 },
    { 0xc6497311, SUC_lexicon_tags_2 },
    { 0xaa097312, SUC_lexicon_tags_17 },
    { 0xf8e97310, SUC_lexicon_tags_32 },
    { 0xb3097314, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4697316, SUC_lexicon_tags_15 },
    { 0x7bc97316, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89297319, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6469731b, SUC_lexicon_tags_15 },
    { 0x8229731c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a9731f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0097322, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6297326, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbd897328, SUC_lexicon_tags_30 },
    { 0xc1097329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe269732b, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x3849732e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64497332, SUC_lexicon_tags_85 },
    { 0x41e97332, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76697337, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb949733f, SUC_lexicon_tags_7 },
    { 0xcfa97340, SUC_lexicon_tags_37 },
    { 0xd8a97341, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0097344, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec97348, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8509734c, SUC_lexicon_tags_29 },
    { 0xa789734d, SUC_lexicon_tags_37 },
    { 0xe609734e, SUC_lexicon_tags_11 },
    { 0xcde9734f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11097356, SUC_lexicon_tags_2 },
    { 0x4ee97356, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c09735a, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c9735d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd729735f, SUC_lexicon_tags_17 },
    { 0x4ec97360, SUC_lexicon_tags_65 },
    { 0xcc297361, SUC_lexicon_tags_40 },
    { 0xdb897360, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe097366, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8897368, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c9736b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3fc9736d, SUC_lexicon_tags_8 },
    { 0xe489736e, SUC_lexicon_tags_11 },
    { 0x3d69736f, SUC_lexicon_tags_15 },
    { 0x6989736f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f297373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a97376, SUC_lexicon_tags_27 },
    { 0xb8a97377, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6897379, SUC_lexicon_tags_10 },
    { 0x9689737a, SUC_lexicon_tags_15 },
    { 0x93c9737b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62a9737d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea97381, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9897386, SUC_lexicon_tags_11 },
    { 0x73c97387, SUC_lexicon_tags_7 },
    { 0x23297387, SUC_lexicon_tags_2 },
    { 0xa2097389, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2689738d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2697390, SUC_lexicon_tags_7 },
    { 0xd7297391, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb5a97393, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8897395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d497398, SUC_lexicon_tags_37 },
    { 0x20297398, SUC_lexicon_tags_13 },
    { 0xd009739a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2329739f, SUC_lexicon_tags_2 },
    { 0xbd09739f, SUC_lexicon_tags_0 },
    { 0x68e973a1, SUC_lexicon_tags_82 },
    { 0xa4e973a1, SUC_lexicon_tags_27 },
    { 0x174973a3, SUC_lexicon_tags_87 },
    { 0x706973a4, SUC_lexicon_tags_46 },
    { 0xdc2973a5, SUC_lexicon_tags_17 },
    { 0x88c9739f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946973aa, SUC_lexicon_tags_47 },
    { 0x28a973ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec973b0, SUC_lexicon_tags_31 },
    { 0x464973b1, SUC_lexicon_tags_4 },
    { 0xee2973b2, SUC_lexicon_tags_15 },
    { 0x4f4973b3, SUC_lexicon_tags_14 },
    { 0x7c4973b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x746973b7, SUC_lexicon_tags_45 },
    { 0xdf6973b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a973bb, SUC_lexicon_tags_44 },
    { 0x46a973bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948973c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a973c7, SUC_lexicon_tags_25 },
    { 0xdc0973c7, SUC_lexicon_tags_12 },
    { 0x618973c7, SUC_lexicon_tags_11 },
    { 0x91e973ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10973ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c973d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x554973d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x152973d6, SUC_lexicon_tags_43 },
    { 0x37e973d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x148973d9, SUC_lexicon_tags_11 },
    { 0xd82973da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44973dd, SUC_lexicon_tags_37 },
    { 0x288973dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34973e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x750973e4, SUC_lexicon_tags_12 },
    { 0x474973e5, SUC_lexicon_tags_38 },
    { 0x680973e5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x180973e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6973ec, SUC_lexicon_tags_1 },
    { 0x2ce973ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeaa973ef, SUC_lexicon_tags_26 },
    { 0x43e973f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3e973f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a973ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb7497401, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5097408, SUC_lexicon_tags_23 },
    { 0xc2497409, SUC_lexicon_tags_27 },
    { 0x8069740a, SUC_lexicon_tags_11 },
    { 0x30a9740b, SUC_lexicon_tags_12 },
    { 0x2e29740c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c9740f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe5a97411, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca97414, SUC_lexicon_tags_82 },
    { 0x66697415, SUC_lexicon_tags_25 },
    { 0x21897414, SUC_lexicon_tags_15 },
    { 0x7c097415, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x189741a, SUC_lexicon_tags_77 },
    { 0x5f29741b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc89741d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e97422, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4697425, SUC_lexicon_tags_29 },
    { 0xdb697426, SUC_lexicon_tags_16 },
    { 0x86c97425, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc9742b, SUC_lexicon_tags_11 },
    { 0xaf69742b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a97431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdc97433, SUC_lexicon_tags_37 },
    { 0x7297433, SUC_lexicon_tags_45 },
    { 0xdba97435, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da97439, SUC_lexicon_tags_49 },
    { 0x11a9743a, SUC_lexicon_tags_25 },
    { 0x5b49743b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc97441, SUC_lexicon_tags_17 },
    { 0x5c697442, SUC_lexicon_tags_11 },
    { 0xdce97443, SUC_lexicon_tags_33 },
    { 0xfc297442, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4a97446, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25697448, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee69744a, SUC_lexicon_tags_6 },
    { 0xa9c9744a, SUC_lexicon_tags_9 },
    { 0xcfc9744c, SUC_lexicon_tags_12 },
    { 0x4a49744d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf769744f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xec497452, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a097454, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b097457, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e097459, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1189745c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7349745f, SUC_lexicon_tags_55 },
    { 0x1aa97460, SUC_lexicon_tags_15 },
    { 0x94497460, SUC_lexicon_tags_26 },
    { 0x51097462, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa5697464, SUC_lexicon_tags_22 },
    { 0xfda97464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e9746b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4e69746d, SUC_lexicon_tags_40 },
    { 0x789746e, SUC_lexicon_tags_15 },
    { 0x26e9746e, SUC_lexicon_tags_11 },
    { 0x6ca97470, SUC_lexicon_tags_49 },
    { 0xece9746e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1897474, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a9747b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f69747e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb697488, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb849748a, SUC_lexicon_tags_12 },
    { 0xf969748b, SUC_lexicon_tags_33 },
    { 0x7c9748c, SUC_lexicon_tags_57 },
    { 0xfe9748d, SUC_lexicon_tags_12 },
    { 0xe209748b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xda497491, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a697495, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba9749f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe82974a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4974a4, SUC_lexicon_tags_35 },
    { 0xa7c974a5, SUC_lexicon_tags_12 },
    { 0x8ca974a6, SUC_lexicon_tags_29 },
    { 0x61a974a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x472974a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ba974ab, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a974af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800974b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4974b7, SUC_lexicon_tags_9 },
    { 0x170974b7, SUC_lexicon_tags_11 },
    { 0x100974b9, SUC_lexicon_tags_25 },
    { 0x978974ba, SUC_lexicon_tags_11 },
    { 0xedc974b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64974be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a974c3, SUC_lexicon_tags_43 },
    { 0xda974c4, SUC_lexicon_tags_17 },
    { 0xdc6974c5, SUC_lexicon_tags_2 },
    { 0x3f4974c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a2974c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x502974ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0974d7, SUC_lexicon_tags_25 },
    { 0x1e4974d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570974e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x252974e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x154974e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a974e9, SUC_lexicon_tags_14 },
    { 0x970974ea, SUC_lexicon_tags_61 },
    { 0xf0974eb, SUC_lexicon_tags_11 },
    { 0x6c2974e9, SUC_lexicon_tags_11 },
    { 0xb90974ed, SUC_lexicon_tags_1 },
    { 0xc0e974ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a974f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38974f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92974fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe8c974fd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29750a, SUC_lexicon_tags_45 },
    { 0xed29750b, SUC_lexicon_tags_8 },
    { 0xa789750c, SUC_lexicon_tags_35 },
    { 0x1009750b, SUC_lexicon_tags_17 },
    { 0xa329750e, SUC_lexicon_tags_28 },
    { 0xfca9750d, SUC_lexicon_tags_80 },
    { 0xd5097510, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xefc97512, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99697519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7669751b, SUC_lexicon_tags_0 },
    { 0x2169751c, SUC_lexicon_tags_2 },
    { 0xe889751d, SUC_lexicon_tags_26 },
    { 0xc249751e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82a97520, SUC_lexicon_tags_15 },
    { 0x71697520, SUC_lexicon_tags_15 },
    { 0x22897522, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98297524, SUC_lexicon_tags_57 },
    { 0xb097525, SUC_lexicon_tags_62 },
    { 0xbee97525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce897528, SUC_lexicon_tags_43 },
    { 0x48697529, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1129752c, SUC_lexicon_tags_17 },
    { 0xa429752c, SUC_lexicon_tags_7 },
    { 0xe669752e, SUC_lexicon_tags_17 },
    { 0x6989752f, SUC_lexicon_tags_25 },
    { 0x39897530, SUC_lexicon_tags_2 },
    { 0x35097530, SUC_lexicon_tags_14 },
    { 0x6c97532, SUC_lexicon_tags_25 },
    { 0xc6a97532, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68497535, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36e97537, SUC_lexicon_tags_25 },
    { 0x3ce97538, SUC_lexicon_tags_37 },
    { 0xae97538, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e09753c, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xf549753e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72897541, SUC_lexicon_tags_327 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b697544, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd897547, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a49754b, SUC_lexicon_tags_17 },
    { 0x9fc9754b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa97555, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf9297557, SUC_lexicon_tags_77 },
    { 0xa0097558, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xca49755b, SUC_lexicon_tags_13 },
    { 0xce89755c, SUC_lexicon_tags_23 },
    { 0xa1e9755c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c97562, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7097564, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf897566, SUC_lexicon_tags_66 },
    { 0x3e097567, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9209756e, SUC_lexicon_tags_30 },
    { 0xea9756f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf697571, SUC_lexicon_tags_37 },
    { 0xb5097572, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c97579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49497585, SUC_lexicon_tags_4 },
    { 0x7da97586, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x13697589, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c9758d, SUC_lexicon_tags_19 },
    { 0x3e9758d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897592, SUC_lexicon_tags_2 },
    { 0x85c97593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e497597, SUC_lexicon_tags_47 },
    { 0xb5697597, SUC_lexicon_tags_23 },
    { 0xf2897599, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf369759c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe229759f, SUC_lexicon_tags_19 },
    { 0xe975a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb64975a2, SUC_lexicon_tags_11 },
    { 0xf0a975a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x122975a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8975a9, SUC_lexicon_tags_26 },
    { 0xb9e975aa, SUC_lexicon_tags_15 },
    { 0x94975ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x93e975ad, SUC_lexicon_tags_7 },
    { 0x6d4975ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e975b1, SUC_lexicon_tags_1 },
    { 0xefa975b1, SUC_lexicon_tags_17 },
    { 0xbfc975b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf58975b5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6975b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2975c0, SUC_lexicon_tags_49 },
    { 0xe975c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c975c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96975cb, SUC_lexicon_tags_23 },
    { 0x160975cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x392975cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78975d7, SUC_lexicon_tags_12 },
    { 0x166975d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8975db, SUC_lexicon_tags_25 },
    { 0xa4a975db, SUC_lexicon_tags_27 },
    { 0xc1c975dd, SUC_lexicon_tags_1 },
    { 0x99a975de, SUC_lexicon_tags_60 },
    { 0xe48975db, SUC_lexicon_tags_9 },
    { 0x492975dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0975e3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a975e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6975eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c975ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2975f1, SUC_lexicon_tags_88 },
    { 0x810975f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594975fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc97601, SUC_lexicon_tags_33 },
    { 0xf9697602, SUC_lexicon_tags_1 },
    { 0x73697601, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3697606, SUC_lexicon_tags_43 },
    { 0x50497607, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e69760c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa9760f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd097612, SUC_lexicon_tags_11 },
    { 0x91297613, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4897618, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e9761c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4a29761e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e97627, SUC_lexicon_tags_60 },
    { 0x9c097628, SUC_lexicon_tags_17 },
    { 0xcdc97627, SUC_lexicon_tags_37 },
    { 0x2de9762a, SUC_lexicon_tags_15 },
    { 0xbc89762b, SUC_lexicon_tags_2 },
    { 0xa529762c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3a9762e, SUC_lexicon_tags_79 },
    { 0x27e9762f, SUC_lexicon_tags_1 },
    { 0xcc897630, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be97633, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e97636, SUC_lexicon_tags_34 },
    { 0xd1097637, SUC_lexicon_tags_32 },
    { 0x79097637, SUC_lexicon_tags_25 },
    { 0x4b297637, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8489763b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaf49763d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec49763f, SUC_lexicon_tags_39 },
    { 0x82697640, SUC_lexicon_tags_42 },
    { 0xc4e97641, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e97645, SUC_lexicon_tags_11 },
    { 0x8d497646, SUC_lexicon_tags_28 },
    { 0x897647, SUC_lexicon_tags_15 },
    { 0x70697648, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf329764c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9029764e, SUC_lexicon_tags_2 },
    { 0xfe9764f, SUC_lexicon_tags_12 },
    { 0x2cc97650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba697652, SUC_lexicon_tags_11 },
    { 0x64497653, SUC_lexicon_tags_68 },
    { 0x7be97652, SUC_lexicon_tags_87 },
    { 0x7ca97655, SUC_lexicon_tags_11 },
    { 0x1697656, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87297659, SUC_lexicon_tags_43 },
    { 0x7aa9765a, SUC_lexicon_tags_23 },
    { 0x62c9765b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a09765e, SUC_lexicon_tags_4 },
    { 0xbb89765f, SUC_lexicon_tags_17 },
    { 0xe9a9765e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39297666, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x93e97668, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x86e9766a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c9766f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x21e97671, SUC_lexicon_tags_49 },
    { 0xadc97672, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x62497675, SUC_lexicon_tags_9 },
    { 0x1ea97676, SUC_lexicon_tags_7 },
    { 0x1c097677, SUC_lexicon_tags_15 },
    { 0x5e497678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf129767a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae09767c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89097682, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a97685, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb909768a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa9768e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10897690, SUC_lexicon_tags_71 },
    { 0x85a97691, SUC_lexicon_tags_25 },
    { 0x7097690, SUC_lexicon_tags_46 },
    { 0x9c97693, SUC_lexicon_tags_37 },
    { 0xddc97694, SUC_lexicon_tags_30 },
    { 0xc9897694, SUC_lexicon_tags_7 },
    { 0x8a897696, SUC_lexicon_tags_17 },
    { 0x61097693, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc69769c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2169769f, SUC_lexicon_tags_65 },
    { 0xe68976a0, SUC_lexicon_tags_54 },
    { 0x8e4976a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2976a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x336976a7, SUC_lexicon_tags_35 },
    { 0x1b0976a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9d0976aa, SUC_lexicon_tags_11 },
    { 0x6dc976ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32976ae, SUC_lexicon_tags_12 },
    { 0xee8976af, SUC_lexicon_tags_33 },
    { 0x9f4976af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e976b4, SUC_lexicon_tags_14 },
    { 0x268976b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a976ba, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882976bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5a976c1, SUC_lexicon_tags_14 },
    { 0x6d4976c2, SUC_lexicon_tags_8 },
    { 0x336976c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2976c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa976c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692976d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcfc976d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60976d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa20976db, SUC_lexicon_tags_7 },
    { 0xe88976dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a976df, SUC_lexicon_tags_1 },
    { 0x7c6976e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8976ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x86c976ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x742976ef, SUC_lexicon_tags_37 },
    { 0x8e976f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0976f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x772976f5, SUC_lexicon_tags_23 },
    { 0xa7e976f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816976fc, SUC_lexicon_tags_37 },
    { 0x38a976fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x800976ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c97703, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a97708, SUC_lexicon_tags_25 },
    { 0x6f697708, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a9770d, SUC_lexicon_tags_34 },
    { 0x9289770e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ba97710, SUC_lexicon_tags_38 },
    { 0xa4897711, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49897713, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6b697715, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34897721, SUC_lexicon_tags_40 },
    { 0xec097722, SUC_lexicon_tags_15 },
    { 0x87697722, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65c9772a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4289772d, SUC_lexicon_tags_0 },
    { 0x70c9772e, SUC_lexicon_tags_11 },
    { 0x9de9772d, SUC_lexicon_tags_23 },
    { 0xf429772e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb297735, SUC_lexicon_tags_43 },
    { 0x2fa97736, SUC_lexicon_tags_65 },
    { 0x1e497736, SUC_lexicon_tags_15 },
    { 0x64c97738, SUC_lexicon_tags_7 },
    { 0x5ec97739, SUC_lexicon_tags_59 },
    { 0x6b497736, SUC_lexicon_tags_20 },
    { 0x32e97737, SUC_lexicon_tags_7 },
    { 0x81e97737, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfca9773e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc97744, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd3e97746, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6dc97748, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9169774d, SUC_lexicon_tags_68 },
    { 0x9609774e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21097752, SUC_lexicon_tags_1 },
    { 0xb0a97752, SUC_lexicon_tags_37 },
    { 0x30e97754, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce97759, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f69775c, SUC_lexicon_tags_0 },
    { 0xe09775d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x80a9775f, SUC_lexicon_tags_7 },
    { 0xd9697760, SUC_lexicon_tags_15 },
    { 0x8297760, SUC_lexicon_tags_2 },
    { 0x17297762, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x16297764, SUC_lexicon_tags_17 },
    { 0x8697764, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2297767, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde69776b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a97770, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x93297772, SUC_lexicon_tags_8 },
    { 0xce897773, SUC_lexicon_tags_402 },
    { 0xebe97773, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79297776, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x22097779, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4029777d, SUC_lexicon_tags_7 },
    { 0xd5e9777e, SUC_lexicon_tags_25 },
    { 0xf0c9777f, SUC_lexicon_tags_14 },
    { 0x37097780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42897784, SUC_lexicon_tags_60 },
    { 0x86297785, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2497787, SUC_lexicon_tags_37 },
    { 0x8d497788, SUC_lexicon_tags_47 },
    { 0xf5a97788, SUC_lexicon_tags_1 },
    { 0x2009778a, SUC_lexicon_tags_16 },
    { 0x5709778b, SUC_lexicon_tags_45 },
    { 0xa749778c, SUC_lexicon_tags_19 },
    { 0xd6a9778d, SUC_lexicon_tags_4 },
    { 0x8189778e, SUC_lexicon_tags_18 },
    { 0xc909778f, SUC_lexicon_tags_15 },
    { 0x20697790, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d297792, SUC_lexicon_tags_14 },
    { 0xc4497792, SUC_lexicon_tags_2 },
    { 0xf097794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca297799, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e49779f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4977a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca977a6, SUC_lexicon_tags_39 },
    { 0xece977a7, SUC_lexicon_tags_7 },
    { 0xbea977a6, SUC_lexicon_tags_25 },
    { 0xc0c977a6, SUC_lexicon_tags_63 },
    { 0x48e977a7, SUC_lexicon_tags_22 },
    { 0x264977aa, SUC_lexicon_tags_45 },
    { 0xc66977ac, SUC_lexicon_tags_12 },
    { 0x8a4977ac, SUC_lexicon_tags_15 },
    { 0x7a2977a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638977b8, SUC_lexicon_tags_11 },
    { 0x71c977b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd06977bb, SUC_lexicon_tags_45 },
    { 0xe50977bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2977c3, SUC_lexicon_tags_31 },
    { 0x86a977c4, SUC_lexicon_tags_11 },
    { 0xeb2977c4, SUC_lexicon_tags_0 },
    { 0x176977c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14977cb, SUC_lexicon_tags_37 },
    { 0x8ae977cb, SUC_lexicon_tags_7 },
    { 0x3dc977cd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12977d0, SUC_lexicon_tags_11 },
    { 0xac977d1, SUC_lexicon_tags_48 },
    { 0x25a977d0, SUC_lexicon_tags_2 },
    { 0x7ea977d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb8977d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc977df, SUC_lexicon_tags_10 },
    { 0x200977e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0977e3, SUC_lexicon_tags_27 },
    { 0x880977e4, SUC_lexicon_tags_8 },
    { 0x366977e5, SUC_lexicon_tags_9 },
    { 0x16e977e5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94977ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd6c977ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36977f2, SUC_lexicon_tags_31 },
    { 0x832977f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f2977f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41e977f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc88977f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf74977fb, SUC_lexicon_tags_669 },
    { 0x8d6977fc, SUC_lexicon_tags_49 },
    { 0x82977fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa14977ff, SUC_lexicon_tags_7 },
    { 0xa70977ff, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0897805, SUC_lexicon_tags_14 },
    { 0x2f297805, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6c97808, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc69780a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a9780d, SUC_lexicon_tags_25 },
    { 0x7de9780e, SUC_lexicon_tags_25 },
    { 0xd449780e, SUC_lexicon_tags_17 },
    { 0x7f29780f, SUC_lexicon_tags_14 },
    { 0xcec9780d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc97814, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7697816, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c9781a, SUC_lexicon_tags_10 },
    { 0x3549781a, SUC_lexicon_tags_37 },
    { 0x9b69781b, SUC_lexicon_tags_87 },
    { 0x2669781c, SUC_lexicon_tags_26 },
    { 0x7489781a, SUC_lexicon_tags_25 },
    { 0x8de9781a, SUC_lexicon_tags_47 },
    { 0x6ac9781e, SUC_lexicon_tags_82 },
    { 0xb489781e, SUC_lexicon_tags_29 },
    { 0x9897820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18497825, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3909782e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafc97830, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a97835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb297837, SUC_lexicon_tags_11 },
    { 0xab497837, SUC_lexicon_tags_26 },
    { 0x72497839, SUC_lexicon_tags_2 },
    { 0x33c97839, SUC_lexicon_tags_1567 },
    { 0x7c697839, SUC_lexicon_tags_2 },
    { 0xb3e9783c, SUC_lexicon_tags_7 },
    { 0xb169783c, SUC_lexicon_tags_106 },
    { 0x57e9783e, SUC_lexicon_tags_79 },
    { 0xb0c9783e, SUC_lexicon_tags_0 },
    { 0xc049783c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf297845, SUC_lexicon_tags_12 },
    { 0xc3897846, SUC_lexicon_tags_26 },
    { 0xe4c97845, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e097849, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcde9784c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5e9784e, SUC_lexicon_tags_8 },
    { 0x85a9784f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6e697851, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7297854, SUC_lexicon_tags_7 },
    { 0x55097854, SUC_lexicon_tags_2 },
    { 0xe1a97855, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8497859, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e9785f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6097868, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d49786c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e097873, SUC_lexicon_tags_9 },
    { 0xdac97874, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb069787c, SUC_lexicon_tags_9 },
    { 0x77c9787d, SUC_lexicon_tags_4 },
    { 0x94c9787c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39297884, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74a97886, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71097888, SUC_lexicon_tags_87 },
    { 0x48297889, SUC_lexicon_tags_26 },
    { 0x7b697888, SUC_lexicon_tags_25 },
    { 0x4c497888, SUC_lexicon_tags_16 },
    { 0x60a9788c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd109788f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6e97891, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50978a6, SUC_lexicon_tags_37 },
    { 0x6da978a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94978aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x718978ac, SUC_lexicon_tags_11 },
    { 0x592978ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x978af, SUC_lexicon_tags_31 },
    { 0xd76978b0, SUC_lexicon_tags_38 },
    { 0xe30978b1, SUC_lexicon_tags_66 },
    { 0x70978b1, SUC_lexicon_tags_47 },
    { 0xe46978b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb46978b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x474978b7, SUC_lexicon_tags_7 },
    { 0x63c978b7, SUC_lexicon_tags_33 },
    { 0xe8c978b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406978bd, SUC_lexicon_tags_0 },
    { 0xc0a978be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44978c2, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x464978c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x376978c8, SUC_lexicon_tags_40 },
    { 0xa5a978c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x138978cb, SUC_lexicon_tags_12 },
    { 0x330978cc, SUC_lexicon_tags_7 },
    { 0x35e978cc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6c8978cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0978d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c978d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x274978d9, SUC_lexicon_tags_47 },
    { 0x66e978da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcc978dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xde978df, SUC_lexicon_tags_33 },
    { 0xa20978df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea978e5, SUC_lexicon_tags_63 },
    { 0x198978e6, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xb3a978e8, SUC_lexicon_tags_15 },
    { 0x468978e9, SUC_lexicon_tags_47 },
    { 0xef8978ea, SUC_lexicon_tags_27 },
    { 0x6cc978ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba978ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa978f2, SUC_lexicon_tags_26 },
    { 0x6f0978f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c978f9, SUC_lexicon_tags_17 },
    { 0xbee978fa, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xc0c978fc, SUC_lexicon_tags_34 },
    { 0xdd0978fd, SUC_lexicon_tags_43 },
    { 0x3fc978fe, SUC_lexicon_tags_49 },
    { 0x8d8978ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x52697902, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7897905, SUC_lexicon_tags_19 },
    { 0xbd897906, SUC_lexicon_tags_30 },
    { 0x12497906, SUC_lexicon_tags_61 },
    { 0x5b297907, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6609790d, SUC_lexicon_tags_10 },
    { 0xa489790d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x28897911, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb8697913, SUC_lexicon_tags_57 },
    { 0x14c97914, SUC_lexicon_tags_17 },
    { 0x7aa97915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fa97917, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb89791c, SUC_lexicon_tags_47 },
    { 0x72a9791d, SUC_lexicon_tags_11 },
    { 0x5569791e, SUC_lexicon_tags_15 },
    { 0x1249791d, SUC_lexicon_tags_30 },
    { 0x4ee9791e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8297923, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a97928, SUC_lexicon_tags_15 },
    { 0x28897929, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe529792b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2829792e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da97932, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd9c97934, SUC_lexicon_tags_37 },
    { 0xd5e97935, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a97939, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee29793b, SUC_lexicon_tags_9 },
    { 0x69a9793c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc29793e, SUC_lexicon_tags_17 },
    { 0xe89793e, SUC_lexicon_tags_14 },
    { 0x4f89793f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde497944, SUC_lexicon_tags_64 },
    { 0xde497944, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x95097947, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e49794a, SUC_lexicon_tags_17 },
    { 0xacc9794b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62c9794d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e97955, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6297957, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a97959, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x85e9795b, SUC_lexicon_tags_63 },
    { 0xade9795c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a97960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x297963, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25097967, SUC_lexicon_tags_63 },
    { 0x77097968, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6729796a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7697970, SUC_lexicon_tags_48 },
    { 0x27a97970, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25497975, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd3697977, SUC_lexicon_tags_17 },
    { 0xad897978, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb809797b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9797f, SUC_lexicon_tags_23 },
    { 0xe3297980, SUC_lexicon_tags_0 },
    { 0x1a09797f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94697987, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a9798a, SUC_lexicon_tags_38 },
    { 0x24a9798a, SUC_lexicon_tags_5 },
    { 0x4a29798c, SUC_lexicon_tags_1 },
    { 0xd149798d, SUC_lexicon_tags_1 },
    { 0xd8e9798c, SUC_lexicon_tags_63 },
    { 0x1ae9798f, SUC_lexicon_tags_37 },
    { 0x59e9798f, SUC_lexicon_tags_47 },
    { 0xdd897991, SUC_lexicon_tags_15 },
    { 0xc5697992, SUC_lexicon_tags_38 },
    { 0xb9497993, SUC_lexicon_tags_17 },
    { 0xc469798f, SUC_lexicon_tags_0 },
    { 0xd8897995, SUC_lexicon_tags_70 },
    { 0x61e97996, SUC_lexicon_tags_11 },
    { 0x5b697996, SUC_lexicon_tags_47 },
    { 0x55097991, SUC_lexicon_tags_33 },
    { 0xf0497999, SUC_lexicon_tags_7 },
    { 0x7989799a, SUC_lexicon_tags_7 },
    { 0xa169799b, SUC_lexicon_tags_11 },
    { 0xd6c9799b, SUC_lexicon_tags_63 },
    { 0x59e9799d, SUC_lexicon_tags_0 },
    { 0x6529799e, SUC_lexicon_tags_39 },
    { 0xe809799c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16979a5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa2e979a7, SUC_lexicon_tags_15 },
    { 0xc9a979a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x242979ab, SUC_lexicon_tags_37 },
    { 0x2ca979ac, SUC_lexicon_tags_25 },
    { 0x1c6979ad, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0xd4e979af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12979b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6e979b4, SUC_lexicon_tags_60 },
    { 0xe08979b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8979b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16979be, SUC_lexicon_tags_14 },
    { 0x128979bf, SUC_lexicon_tags_17 },
    { 0x8fc979be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4979c3, SUC_lexicon_tags_1 },
    { 0x956979c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x378979c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46979cd, SUC_lexicon_tags_8 },
    { 0x2c8979ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x106979d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2979d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8979d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x57c979db, SUC_lexicon_tags_17 },
    { 0x2979db, SUC_lexicon_tags_27 },
    { 0x490979db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6979e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4979e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xade979e6, SUC_lexicon_tags_37 },
    { 0x7de979e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ac979e9, SUC_lexicon_tags_23 },
    { 0xc2a979ea, SUC_lexicon_tags_25 },
    { 0xe2a979eb, SUC_lexicon_tags_29 },
    { 0x416979ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc979f1, SUC_lexicon_tags_37 },
    { 0x6ce979f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc979f6, SUC_lexicon_tags_17 },
    { 0x982979f7, SUC_lexicon_tags_12 },
    { 0x77e979f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35097a01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67097a03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4497a0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd697a13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ba97a15, SUC_lexicon_tags_2 },
    { 0x75497a15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a97a1b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x5f497a1d, SUC_lexicon_tags_13 },
    { 0xd9e97a1d, SUC_lexicon_tags_6 },
    { 0x8fe97a1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37497a24, SUC_lexicon_tags_15 },
    { 0x78e97a24, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f297a2a, SUC_lexicon_tags_43 },
    { 0x17c97a2b, SUC_lexicon_tags_14 },
    { 0x97e97a2c, SUC_lexicon_tags_3 },
    { 0x7f097a2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c97a31, SUC_lexicon_tags_15 },
    { 0xa7697a31, SUC_lexicon_tags_6 },
    { 0x9c97a31, SUC_lexicon_tags_19 },
    { 0x2bc97a33, SUC_lexicon_tags_64 },
    { 0x1ca97a35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54897a37, SUC_lexicon_tags_17 },
    { 0x8a097a38, SUC_lexicon_tags_28 },
    { 0xfcc97a39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x77297a3c, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x94a97a3e, SUC_lexicon_tags_1 },
    { 0xbaa97a3f, SUC_lexicon_tags_2 },
    { 0x65a97a40, SUC_lexicon_tags_121 },
    { 0xd5897a40, SUC_lexicon_tags_6 },
    { 0xa2497a3f, SUC_lexicon_tags_26 },
    { 0x80497a43, SUC_lexicon_tags_47 },
    { 0x27e97a3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x56897a47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82697a4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2097a52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46497a54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa697a58, SUC_lexicon_tags_17 },
    { 0x7b297a59, SUC_lexicon_tags_25 },
    { 0xd1697a58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe097a63, SUC_lexicon_tags_17 },
    { 0x3497a64, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3297a6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27297a6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a097a6f, SUC_lexicon_tags_30 },
    { 0xe4e97a70, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf897a77, SUC_lexicon_tags_37 },
    { 0x55e97a77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe97a7b, SUC_lexicon_tags_11 },
    { 0x2297a7c, SUC_lexicon_tags_25 },
    { 0xf5e97a7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a97a81, SUC_lexicon_tags_17 },
    { 0x23e97a82, SUC_lexicon_tags_2 },
    { 0xa8297a83, SUC_lexicon_tags_19 },
    { 0x75497a84, SUC_lexicon_tags_56 },
    { 0x76097a85, SUC_lexicon_tags_57 },
    { 0x2a097a86, SUC_lexicon_tags_11 },
    { 0x98697a87, SUC_lexicon_tags_29 },
    { 0xb2497a81, SUC_lexicon_tags_45 },
    { 0x80297a89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9297a8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d097a92, SUC_lexicon_tags_37 },
    { 0x67897a92, SUC_lexicon_tags_17 },
    { 0x87e97a92, SUC_lexicon_tags_12 },
    { 0x42897a95, SUC_lexicon_tags_25 },
    { 0x29697a96, SUC_lexicon_tags_19 },
    { 0xc9697a94, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x91c97a99, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39897a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3897aa1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc97aa7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea697aab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b497ab5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a97abc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99e97abe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e97ac6, SUC_lexicon_tags_8 },
    { 0x55a97ac6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x79097aca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe897acc, SUC_lexicon_tags_11 },
    { 0xe0697acc, SUC_lexicon_tags_47 },
    { 0x89697ace, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76697ad3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d697ad7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54c97ad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80897adb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86697ae3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a97ae6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4fa97ae8, SUC_lexicon_tags_47 },
    { 0xe4097ae8, SUC_lexicon_tags_15 },
    { 0x84897aea, SUC_lexicon_tags_8 },
    { 0xfe297aeb, SUC_lexicon_tags_37 },
    { 0x73697aec, SUC_lexicon_tags_37 },
    { 0x6b497aed, SUC_lexicon_tags_23 },
    { 0xc9097aee, SUC_lexicon_tags_23 },
    { 0xb2297aee, SUC_lexicon_tags_18 },
    { 0x2a497af0, SUC_lexicon_tags_37 },
    { 0x3b897af1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e297af4, SUC_lexicon_tags_23 },
    { 0xc1097af4, SUC_lexicon_tags_15 },
    { 0x6a897af5, SUC_lexicon_tags_0 },
    { 0x6f097af7, SUC_lexicon_tags_15 },
    { 0x24897af8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x85697afb, SUC_lexicon_tags_14 },
    { 0x5a97afc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5497b06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24097b0a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8be97b0c, SUC_lexicon_tags_60 },
    { 0xd5297b0c, SUC_lexicon_tags_47 },
    { 0xbc297b0e, SUC_lexicon_tags_0 },
    { 0x11a97b0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c097b12, SUC_lexicon_tags_28 },
    { 0x5097b12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x32c97b15, SUC_lexicon_tags_11 },
    { 0xb0097b16, SUC_lexicon_tags_37 },
    { 0xfcc97b16, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0697b1d, SUC_lexicon_tags_2 },
    { 0x4d097b1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fe97b20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76697b24, SUC_lexicon_tags_47 },
    { 0x7e97b25, SUC_lexicon_tags_31 },
    { 0xd2c97b26, SUC_lexicon_tags_11 },
    { 0xcf897b24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68697b29, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x40c97b2b, SUC_lexicon_tags_103 },
    { 0xbf497b2b, SUC_lexicon_tags_17 },
    { 0x9c897b2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4097b39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a897b3e, SUC_lexicon_tags_45 },
    { 0xaca97b3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f897b41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5097b43, SUC_lexicon_tags_2 },
    { 0x78a97b44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6697b48, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38097b51, SUC_lexicon_tags_14 },
    { 0xc3297b51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc97b56, SUC_lexicon_tags_37 },
    { 0x75697b57, SUC_lexicon_tags_51 },
    { 0xbc897b56, SUC_lexicon_tags_10 },
    { 0x77c97b58, SUC_lexicon_tags_33 },
    { 0x5ea97b57, SUC_lexicon_tags_29 },
    { 0x43897b57, SUC_lexicon_tags_16 },
    { 0x40897b5c, SUC_lexicon_tags_11 },
    { 0xae497b5c, SUC_lexicon_tags_14 },
    { 0xe97b5e, SUC_lexicon_tags_15 },
    { 0xf4497b5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8297b64, SUC_lexicon_tags_92 },
    { 0xc8897b65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11c97b67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3497b69, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec97b6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42c97b70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e897b72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc97b75, SUC_lexicon_tags_25 },
    { 0xbde97b76, SUC_lexicon_tags_7 },
    { 0x87a97b75, SUC_lexicon_tags_11 },
    { 0x7a297b78, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16697b7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee97b7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8c97b85, SUC_lexicon_tags_7 },
    { 0xe2897b86, SUC_lexicon_tags_19 },
    { 0xa8a97b87, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x96e97b89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb297b8c, SUC_lexicon_tags_30 },
    { 0x8d897b8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0897b90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5a297b92, SUC_lexicon_tags_19 },
    { 0x23c97b93, SUC_lexicon_tags_49 },
    { 0xa97b92, SUC_lexicon_tags_45 },
    { 0xf8c97b92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xea697b97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d497b9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64497ba2, SUC_lexicon_tags_1 },
    { 0x9fe97ba3, SUC_lexicon_tags_12 },
    { 0x1ce97ba4, SUC_lexicon_tags_15 },
    { 0xd8697ba5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6be97ba7, SUC_lexicon_tags_21 },
    { 0x5a697ba8, SUC_lexicon_tags_17 },
    { 0x46097ba8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c97bae, SUC_lexicon_tags_1 },
    { 0xbfa97baf, SUC_lexicon_tags_11 },
    { 0xcac97baf, SUC_lexicon_tags_45 },
    { 0x90c97baf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e97bb4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c897bb8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c297bbc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d697bbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5697bc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e97bcc, SUC_lexicon_tags_111 },
    { 0xf9297bcd, SUC_lexicon_tags_9 },
    { 0xc497bcc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f097bd1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x33297bd4, SUC_lexicon_tags_0 },
    { 0x71297bd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17e97bd7, SUC_lexicon_tags_17 },
    { 0x23897bd8, SUC_lexicon_tags_21 },
    { 0xe0297bd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39297bdb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da97be0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e97be4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2697be7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c97bec, SUC_lexicon_tags_42 },
    { 0x6a297bec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe9297bef, SUC_lexicon_tags_23 },
    { 0x9e097bef, SUC_lexicon_tags_14 },
    { 0x4ac97bf0, SUC_lexicon_tags_1 },
    { 0xa8097bef, SUC_lexicon_tags_14 },
    { 0x97a97bf3, SUC_lexicon_tags_10 },
    { 0x90e97bf4, SUC_lexicon_tags_37 },
    { 0x6de97bf3, SUC_lexicon_tags_37 },
    { 0x3ea97bf6, SUC_lexicon_tags_17 },
    { 0x55097bf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c97bfa, SUC_lexicon_tags_27 },
    { 0xe9097bfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72e97bfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98297bff, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe97c02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc897c04, SUC_lexicon_tags_49 },
    { 0x54297c05, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0297c09, SUC_lexicon_tags_37 },
    { 0xb7897c0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a097c10, SUC_lexicon_tags_2 },
    { 0xf9297c11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4497c1e, SUC_lexicon_tags_11 },
    { 0xb6897c1e, SUC_lexicon_tags_11 },
    { 0x88c97c20, SUC_lexicon_tags_1 },
    { 0xd7497c21, SUC_lexicon_tags_9 },
    { 0xa7c97c22, SUC_lexicon_tags_12 },
    { 0x52e97c23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5697c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d697c30, SUC_lexicon_tags_43 },
    { 0xe2897c31, SUC_lexicon_tags_39 },
    { 0xcc497c30, SUC_lexicon_tags_61 },
    { 0x49897c33, SUC_lexicon_tags_14 },
    { 0x11497c30, SUC_lexicon_tags_39 },
    { 0xb4097c35, SUC_lexicon_tags_17 },
    { 0x28697c35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x97097c39, SUC_lexicon_tags_26 },
    { 0x17e97c3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x96897c3d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x40897c3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c497c43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a297c4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d497c4f, SUC_lexicon_tags_37 },
    { 0x6f897c50, SUC_lexicon_tags_1 },
    { 0xd7297c51, SUC_lexicon_tags_27 },
    { 0x73c97c52, SUC_lexicon_tags_10 },
    { 0x58497c52, SUC_lexicon_tags_28 },
    { 0x3c497c4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6e97c56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c097c59, SUC_lexicon_tags_25 },
    { 0xc2297c5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x70697c5d, SUC_lexicon_tags_2 },
    { 0xf5297c5e, SUC_lexicon_tags_19 },
    { 0xbc897c5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ba97c61, SUC_lexicon_tags_10 },
    { 0x5f897c62, SUC_lexicon_tags_37 },
    { 0xd3297c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c97c68, SUC_lexicon_tags_25 },
    { 0x55a97c69, SUC_lexicon_tags_6 },
    { 0x3c697c68, SUC_lexicon_tags_53 },
    { 0x4d097c68, SUC_lexicon_tags_62 },
    { 0x6b897c6a, SUC_lexicon_tags_28 },
    { 0x9f697c68, SUC_lexicon_tags_38 },
    { 0xe4e97c69, SUC_lexicon_tags_0 },
    { 0xd8c97c6f, SUC_lexicon_tags_11 },
    { 0xf4297c70, SUC_lexicon_tags_43 },
    { 0xb2097c71, SUC_lexicon_tags_42 },
    { 0xfca97c6f, SUC_lexicon_tags_38 },
    { 0x51297c6d, SUC_lexicon_tags_19 },
    { 0xd497c73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3c97c76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde897c7e, SUC_lexicon_tags_1 },
    { 0x30e97c7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea297c88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e97c90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x34e97c92, SUC_lexicon_tags_55 },
    { 0x3697c92, SUC_lexicon_tags_19 },
    { 0x8bc97c94, SUC_lexicon_tags_12 },
    { 0xb5297c94, SUC_lexicon_tags_7 },
    { 0xc7497c92, SUC_lexicon_tags_163 },
    { 0x1e097c92, SUC_lexicon_tags_19 },
    { 0x15097c95, SUC_lexicon_tags_37 },
    { 0xd8c97c95, SUC_lexicon_tags_15 },
    { 0xccc97c9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c97c9f, SUC_lexicon_tags_32 },
    { 0x40697ca0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a297ca5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7897cab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3297cb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e97cc1, SUC_lexicon_tags_60 },
    { 0xa5897cc2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57497cc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85297cc9, SUC_lexicon_tags_29 },
    { 0x8b497cca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48297cd0, SUC_lexicon_tags_55 },
    { 0x1f897cd1, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x50497cd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9897cd7, SUC_lexicon_tags_8 },
    { 0x3e897cd7, SUC_lexicon_tags_17 },
    { 0x6897cd9, SUC_lexicon_tags_17 },
    { 0xc4a97cd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee697ce4, SUC_lexicon_tags_5 },
    { 0xe697ce5, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c97ce8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa7c97cea, SUC_lexicon_tags_14 },
    { 0x8b497cea, SUC_lexicon_tags_47 },
    { 0x91c97cec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf097cee, SUC_lexicon_tags_52 },
    { 0xaf897cef, SUC_lexicon_tags_63 },
    { 0xe3a97cf0, SUC_lexicon_tags_23 },
    { 0x30497cf1, SUC_lexicon_tags_37 },
    { 0xb2c97cf0, SUC_lexicon_tags_19 },
    { 0x66097cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa97cf6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa897cf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6097d03, SUC_lexicon_tags_11 },
    { 0xfb897d04, SUC_lexicon_tags_17 },
    { 0x87297d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71697d07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f097d0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68097d0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7697d13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc897d17, SUC_lexicon_tags_55 },
    { 0xb6297d17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83897d1c, SUC_lexicon_tags_46 },
    { 0xf5897d1d, SUC_lexicon_tags_1 },
    { 0xb7897d1e, SUC_lexicon_tags_45 },
    { 0x88297d1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xec97d22, SUC_lexicon_tags_2 },
    { 0xba897d23, SUC_lexicon_tags_15 },
    { 0x53497d24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46e97d29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c297d30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a697d34, SUC_lexicon_tags_15 },
    { 0xc6897d35, SUC_lexicon_tags_9 },
    { 0x72497d34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20497d3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe897d40, SUC_lexicon_tags_17 },
    { 0x29a97d41, SUC_lexicon_tags_21 },
    { 0xbc897d42, SUC_lexicon_tags_1 },
    { 0x9d697d43, SUC_lexicon_tags_1 },
    { 0x77e97d44, SUC_lexicon_tags_25 },
    { 0x7ce97d45, SUC_lexicon_tags_25 },
    { 0x67897d46, SUC_lexicon_tags_37 },
    { 0xaf497d42, SUC_lexicon_tags_25 },
    { 0x46897d48, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5697d4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96097d4f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xade97d51, SUC_lexicon_tags_14 },
    { 0xf4e97d52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2297d57, SUC_lexicon_tags_2 },
    { 0xe9897d57, SUC_lexicon_tags_62 },
    { 0xe1897d57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83097d61, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d097d64, SUC_lexicon_tags_30 },
    { 0x1f897d64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9497d68, SUC_lexicon_tags_87 },
    { 0x15097d69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5497d6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0097d6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde97d72, SUC_lexicon_tags_11 },
    { 0xb1097d73, SUC_lexicon_tags_34 },
    { 0x2e897d73, SUC_lexicon_tags_37 },
    { 0x53497d75, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e97d7a, SUC_lexicon_tags_29 },
    { 0xb0697d7a, SUC_lexicon_tags_33 },
    { 0xa4e97d7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xda297d7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0e97d81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce97d87, SUC_lexicon_tags_27 },
    { 0xefe97d87, SUC_lexicon_tags_11 },
    { 0x12e97d87, SUC_lexicon_tags_68 },
    { 0xae97d8a, SUC_lexicon_tags_17 },
    { 0x80a97d89, SUC_lexicon_tags_197 },
    { 0x2c097d8a, SUC_lexicon_tags_60 },
    { 0xa5c97d8d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e97d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23297d95, SUC_lexicon_tags_26 },
    { 0x22697d95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0497d99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86297d9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2297da1, SUC_lexicon_tags_7 },
    { 0x1ec97da2, SUC_lexicon_tags_10 },
    { 0x74697da2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78697da5, SUC_lexicon_tags_33 },
    { 0x43097da5, SUC_lexicon_tags_11 },
    { 0xc4097da7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c97dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47697db4, SUC_lexicon_tags_29 },
    { 0x6697db5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a97db8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34097dbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0497dc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc97dc8, SUC_lexicon_tags_15 },
    { 0xd697dc9, SUC_lexicon_tags_0 },
    { 0x14c97dca, SUC_lexicon_tags_14 },
    { 0x3097dcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e097dcd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9697dd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc297dd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e97de3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29697de6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9e897de8, SUC_lexicon_tags_38 },
    { 0x76497de9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8497dec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce97def, SUC_lexicon_tags_15 },
    { 0x7fc97def, SUC_lexicon_tags_38 },
    { 0x6f497df1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8e97df3, SUC_lexicon_tags_23 },
    { 0xcea97df4, SUC_lexicon_tags_17 },
    { 0x3c497df5, SUC_lexicon_tags_37 },
    { 0x5d497df6, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd097dfa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x69097dfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c97e09, SUC_lexicon_tags_38 },
    { 0x77c97e0a, SUC_lexicon_tags_7 },
    { 0xf5297e0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7697e0e, SUC_lexicon_tags_37 },
    { 0x53c97e0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeaa97e11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c297e15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53097e1f, SUC_lexicon_tags_113 },
    { 0x36497e1f, SUC_lexicon_tags_37 },
    { 0x28097e21, SUC_lexicon_tags_21 },
    { 0x5e497e22, SUC_lexicon_tags_2 },
    { 0xc2497e21, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x49697e25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4bc97e27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c97e2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e697e30, SUC_lexicon_tags_2 },
    { 0xec097e31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a97e36, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9e297e38, SUC_lexicon_tags_37 },
    { 0xc0897e39, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9097e3c, SUC_lexicon_tags_11 },
    { 0xbe897e3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4097e40, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0497e4a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x44497e4c, SUC_lexicon_tags_60 },
    { 0xf0697e4c, SUC_lexicon_tags_1 },
    { 0xf5e97e4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e97e51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51297e57, SUC_lexicon_tags_63 },
    { 0x60e97e58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb897e5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xef497e60, SUC_lexicon_tags_38 },
    { 0x23c97e61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e97e68, SUC_lexicon_tags_0 },
    { 0x3ac97e68, SUC_lexicon_tags_106 },
    { 0x6fe97e6a, SUC_lexicon_tags_31 },
    { 0x1a097e68, SUC_lexicon_tags_1 },
    { 0xdf697e6c, SUC_lexicon_tags_41 },
    { 0xbc297e6d, SUC_lexicon_tags_17 },
    { 0x99e97e6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4497e72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62297e74, SUC_lexicon_tags_31 },
    { 0x2297e75, SUC_lexicon_tags_5 },
    { 0xb9a97e76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf8e97e78, SUC_lexicon_tags_37 },
    { 0xc8097e79, SUC_lexicon_tags_1 },
    { 0x2c497e78, SUC_lexicon_tags_61 },
    { 0x43297e7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x83297e7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18497e80, SUC_lexicon_tags_7 },
    { 0x51497e80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6a97e83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd097e85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c97e8b, SUC_lexicon_tags_8 },
    { 0x15c97e8c, SUC_lexicon_tags_37 },
    { 0x5ce97e8c, SUC_lexicon_tags_12 },
    { 0x5de97e8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe6697e90, SUC_lexicon_tags_60 },
    { 0xda97e90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec697e95, SUC_lexicon_tags_0 },
    { 0xb9697e96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf897e9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x78897e9d, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x17697e9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71e97ea2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac697ea4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3897ea6, SUC_lexicon_tags_7 },
    { 0x76497ea6, SUC_lexicon_tags_15 },
    { 0x86697ea8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe97eac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc97eaf, SUC_lexicon_tags_28 },
    { 0xdfa97eb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d897eb4, SUC_lexicon_tags_7 },
    { 0x92097eb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23097eb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e97ebb, SUC_lexicon_tags_38 },
    { 0xe4697ebb, SUC_lexicon_tags_81 },
    { 0x73a97ebd, SUC_lexicon_tags_19 },
    { 0xcc97ebe, SUC_lexicon_tags_0 },
    { 0xc497ebf, SUC_lexicon_tags_1 },
    { 0x66c97ebc, SUC_lexicon_tags_10 },
    { 0xa6297ec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e97ec4, SUC_lexicon_tags_10 },
    { 0x5fe97ec5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e697ec7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc97ece, SUC_lexicon_tags_0 },
    { 0x66697ecf, SUC_lexicon_tags_4 },
    { 0x47497ed0, SUC_lexicon_tags_17 },
    { 0x3a497ece, SUC_lexicon_tags_25 },
    { 0x1d097ed2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa697ed5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a97ed8, SUC_lexicon_tags_2 },
    { 0x8ce97ed9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63097ede, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55897ee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f497ee7, SUC_lexicon_tags_49 },
    { 0x92697ee8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63297eea, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e497ef0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20297ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9697efc, SUC_lexicon_tags_15 },
    { 0x2f297efc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dc97eff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf897f02, SUC_lexicon_tags_10 },
    { 0xdb497f03, SUC_lexicon_tags_4 },
    { 0x70497f04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a97f07, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7f697f09, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c97f0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48697f0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33697f14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xca97f16, SUC_lexicon_tags_27 },
    { 0xe8e97f17, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8e497f19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10497f1c, SUC_lexicon_tags_45 },
    { 0x8b497f1d, SUC_lexicon_tags_38 },
    { 0x22a97f1d, SUC_lexicon_tags_1 },
    { 0xc497f1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc97f24, SUC_lexicon_tags_22 },
    { 0x9e297f25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f097f2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac97f32, SUC_lexicon_tags_2 },
    { 0x22097f33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a97f37, SUC_lexicon_tags_7 },
    { 0xa5897f37, SUC_lexicon_tags_2 },
    { 0xa7497f39, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd097f3d, SUC_lexicon_tags_6 },
    { 0x297f3e, SUC_lexicon_tags_28 },
    { 0x94a97f3f, SUC_lexicon_tags_7 },
    { 0xffc97f3f, SUC_lexicon_tags_12 },
    { 0x59097f41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5097f43, SUC_lexicon_tags_70 },
    { 0x21c97f44, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x53697f46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca497f4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d097f4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6897f4f, SUC_lexicon_tags_26 },
    { 0x50497f50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1da97f52, SUC_lexicon_tags_67 },
    { 0x52097f53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafe97f55, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d297f58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1297f5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2297f62, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea97f66, SUC_lexicon_tags_37 },
    { 0xc4497f66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67497f69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e97f6d, SUC_lexicon_tags_15 },
    { 0x2a97f6e, SUC_lexicon_tags_17 },
    { 0xb0697f6d, SUC_lexicon_tags_37 },
    { 0xd5297f70, SUC_lexicon_tags_11 },
    { 0xf0a97f6e, SUC_lexicon_tags_27 },
    { 0x71897f72, SUC_lexicon_tags_15 },
    { 0x90897f73, SUC_lexicon_tags_43 },
    { 0xb2097f6f, SUC_lexicon_tags_33 },
    { 0xa3097f75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x83097f78, SUC_lexicon_tags_27 },
    { 0xc6297f79, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfaa97f7b, SUC_lexicon_tags_10 },
    { 0x21297f7b, SUC_lexicon_tags_39 },
    { 0x13097f7c, SUC_lexicon_tags_62 },
    { 0x85697f7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x92497f81, SUC_lexicon_tags_7 },
    { 0xd4c97f81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe297f85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed697f87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21a97f89, SUC_lexicon_tags_27 },
    { 0x95e97f8a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b697f93, SUC_lexicon_tags_7 },
    { 0x85497f93, SUC_lexicon_tags_37 },
    { 0xdea97f94, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x64697f97, SUC_lexicon_tags_6 },
    { 0xc4497f98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87297f9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e97fa2, SUC_lexicon_tags_15 },
    { 0x7a697fa3, SUC_lexicon_tags_1 },
    { 0x69c97fa4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaee97fa6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a097fab, SUC_lexicon_tags_12 },
    { 0x34a97fac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8097faf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf6497fb1, SUC_lexicon_tags_12 },
    { 0x39497fb1, SUC_lexicon_tags_11 },
    { 0x90497fb2, SUC_lexicon_tags_17 },
    { 0x4f097fb4, SUC_lexicon_tags_16 },
    { 0xb2297fb5, SUC_lexicon_tags_12 },
    { 0x92097fb5, SUC_lexicon_tags_14 },
    { 0x50297fb4, SUC_lexicon_tags_15 },
    { 0xd5497fb8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea97fbb, SUC_lexicon_tags_15 },
    { 0x4f097fbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d297fbf, SUC_lexicon_tags_49 },
    { 0xc4297fc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29897fc2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c497fc6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x71497fc8, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e697fcc, SUC_lexicon_tags_37 },
    { 0xa9e97fcd, SUC_lexicon_tags_7 },
    { 0x5ec97fcc, SUC_lexicon_tags_43 },
    { 0x6a097fcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba097fd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25097fd9, SUC_lexicon_tags_5 },
    { 0x96497fda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x29497fdc, SUC_lexicon_tags_17 },
    { 0xbe097fdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb297fe4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc97fe9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e97fed, SUC_lexicon_tags_63 },
    { 0x8e297fee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5697ff0, SUC_lexicon_tags_27 },
    { 0xae497ff1, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4697ff4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x93897ff6, SUC_lexicon_tags_24 },
    { 0x80297ff6, SUC_lexicon_tags_12 },
    { 0x55497ff8, SUC_lexicon_tags_29 },
    { 0x4f897ff9, SUC_lexicon_tags_33 },
    { 0xab097ffa, SUC_lexicon_tags_12 },
    { 0xf5c97ffb, SUC_lexicon_tags_15 },
    { 0x5be97ffc, SUC_lexicon_tags_7 },
    { 0x38897ffd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7697fff, SUC_lexicon_tags_82 },
    { 0x84e97fff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c98003, SUC_lexicon_tags_7 },
    { 0x39e98004, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ac98006, SUC_lexicon_tags_37 },
    { 0x4f098007, SUC_lexicon_tags_2 },
    { 0xefe98007, SUC_lexicon_tags_26 },
    { 0xcd898009, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe98011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca98015, SUC_lexicon_tags_11 },
    { 0x47c98016, SUC_lexicon_tags_25 },
    { 0xe9498017, SUC_lexicon_tags_37 },
    { 0x6e898017, SUC_lexicon_tags_11 },
    { 0x7d898019, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x30a9801b, SUC_lexicon_tags_34 },
    { 0x6c9801c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26698021, SUC_lexicon_tags_14 },
    { 0x85698022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4098025, SUC_lexicon_tags_25 },
    { 0x24e98026, SUC_lexicon_tags_36 },
    { 0x2e098027, SUC_lexicon_tags_11 },
    { 0x37a98028, SUC_lexicon_tags_7 },
    { 0x9f698025, SUC_lexicon_tags_14 },
    { 0x1809802a, SUC_lexicon_tags_12 },
    { 0xefc98026, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cc9802d, SUC_lexicon_tags_31 },
    { 0xf289802e, SUC_lexicon_tags_23 },
    { 0x67c9802e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79298034, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2898038, SUC_lexicon_tags_25 },
    { 0x2fc98039, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e89803d, SUC_lexicon_tags_30 },
    { 0x4f09803e, SUC_lexicon_tags_7 },
    { 0xab49803f, SUC_lexicon_tags_33 },
    { 0x1b698040, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa98045, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc98048, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb329804e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaaa98050, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9a98052, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29498056, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e9805a, SUC_lexicon_tags_25 },
    { 0x2a49805b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8129805e, SUC_lexicon_tags_12 },
    { 0x7f89805e, SUC_lexicon_tags_27 },
    { 0x17a98060, SUC_lexicon_tags_45 },
    { 0x68298060, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8298065, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f898067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9789806a, SUC_lexicon_tags_7 },
    { 0xbd09806b, SUC_lexicon_tags_15 },
    { 0xc089806b, SUC_lexicon_tags_38 },
    { 0x9a89806a, SUC_lexicon_tags_1 },
    { 0xdf89806c, SUC_lexicon_tags_37 },
    { 0xb449806b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb5c98071, SUC_lexicon_tags_23 },
    { 0xf1098072, SUC_lexicon_tags_12 },
    { 0x9298073, SUC_lexicon_tags_16 },
    { 0x53a98074, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4de98076, SUC_lexicon_tags_37 },
    { 0x4fa98077, SUC_lexicon_tags_11 },
    { 0x95698078, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1e89807a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda9807d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a69807f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x48698081, SUC_lexicon_tags_37 },
    { 0xe7c98081, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa8e98084, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x63498087, SUC_lexicon_tags_11 },
    { 0xa4c98088, SUC_lexicon_tags_1 },
    { 0xe2c98087, SUC_lexicon_tags_47 },
    { 0x7a69808a, SUC_lexicon_tags_1 },
    { 0x6bc9808b, SUC_lexicon_tags_63 },
    { 0x6369808c, SUC_lexicon_tags_0 },
    { 0xc7a9808b, SUC_lexicon_tags_9 },
    { 0x27c9808e, SUC_lexicon_tags_19 },
    { 0x6409808f, SUC_lexicon_tags_1 },
    { 0x97298090, SUC_lexicon_tags_2 },
    { 0xf1498088, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8fc98093, SUC_lexicon_tags_28 },
    { 0xf7e98093, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c098098, SUC_lexicon_tags_29 },
    { 0x27498098, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f29809e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a980a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x676980a5, SUC_lexicon_tags_29 },
    { 0xa84980a6, SUC_lexicon_tags_27 },
    { 0xcce980a7, SUC_lexicon_tags_21 },
    { 0xfae980a7, SUC_lexicon_tags_12 },
    { 0x5da980a9, SUC_lexicon_tags_7 },
    { 0x542980aa, SUC_lexicon_tags_45 },
    { 0xb2c980a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e980ae, SUC_lexicon_tags_26 },
    { 0xd04980af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548980b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2980b5, SUC_lexicon_tags_12 },
    { 0x7e2980b6, SUC_lexicon_tags_57 },
    { 0xe22980b5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4980bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa22980bf, SUC_lexicon_tags_49 },
    { 0x65e980bf, SUC_lexicon_tags_8 },
    { 0x2980c0, SUC_lexicon_tags_63 },
    { 0xba6980c2, SUC_lexicon_tags_10 },
    { 0x55a980c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec980c7, SUC_lexicon_tags_38 },
    { 0xa54980c8, SUC_lexicon_tags_11 },
    { 0xa0e980c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7ba980cb, SUC_lexicon_tags_2 },
    { 0x504980cc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0980d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9c980d5, SUC_lexicon_tags_11 },
    { 0xdc2980d6, SUC_lexicon_tags_10 },
    { 0x83c980d6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8980da, SUC_lexicon_tags_8 },
    { 0xa2c980db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64c980dd, SUC_lexicon_tags_0 },
    { 0x9b0980de, SUC_lexicon_tags_1 },
    { 0x144980df, SUC_lexicon_tags_25 },
    { 0xb78980de, SUC_lexicon_tags_7 },
    { 0x752980e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250980e6, SUC_lexicon_tags_11 },
    { 0x7d8980e7, SUC_lexicon_tags_17 },
    { 0xba6980e7, SUC_lexicon_tags_23 },
    { 0xd56980e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2980ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbc980ef, SUC_lexicon_tags_0 },
    { 0x8a6980ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a6980f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c980f7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xba4980f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a980fd, SUC_lexicon_tags_2 },
    { 0xcee980fd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x20498100, SUC_lexicon_tags_23 },
    { 0xfc098101, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7298107, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3098109, SUC_lexicon_tags_38 },
    { 0x66a9810a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a9810d, SUC_lexicon_tags_38 },
    { 0xcde9810d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ea98110, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3898112, SUC_lexicon_tags_10 },
    { 0xdde98113, SUC_lexicon_tags_34 },
    { 0xefc98114, SUC_lexicon_tags_15 },
    { 0xa0e98115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44098117, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa9811b, SUC_lexicon_tags_1 },
    { 0xb309811b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd49811f, SUC_lexicon_tags_38 },
    { 0xae09811f, SUC_lexicon_tags_16 },
    { 0xc2a98121, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7298123, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x90a98125, SUC_lexicon_tags_14 },
    { 0xc4098125, SUC_lexicon_tags_14 },
    { 0x66a98127, SUC_lexicon_tags_27 },
    { 0x32698126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83c9812b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb1a9812d, SUC_lexicon_tags_15 },
    { 0xb569812e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff898132, SUC_lexicon_tags_31 },
    { 0xb9a98133, SUC_lexicon_tags_65 },
    { 0x9898134, SUC_lexicon_tags_6 },
    { 0x3ac98135, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c98138, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a29813a, SUC_lexicon_tags_8 },
    { 0xaa89813a, SUC_lexicon_tags_15 },
    { 0x54e9813c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e29813e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c98141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6298145, SUC_lexicon_tags_6 },
    { 0x42298146, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e9814a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc129814e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e98151, SUC_lexicon_tags_3 },
    { 0x4d898151, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd898154, SUC_lexicon_tags_27 },
    { 0x63898155, SUC_lexicon_tags_9 },
    { 0x2f298155, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b498159, SUC_lexicon_tags_11 },
    { 0xb8c9815a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc9815f, SUC_lexicon_tags_8 },
    { 0xb9098160, SUC_lexicon_tags_82 },
    { 0x79498160, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec98165, SUC_lexicon_tags_45 },
    { 0x27a98166, SUC_lexicon_tags_25 },
    { 0xdc498165, SUC_lexicon_tags_9 },
    { 0x50898166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac9816b, SUC_lexicon_tags_39 },
    { 0xa509816b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd609816e, SUC_lexicon_tags_18 },
    { 0x3d69816f, SUC_lexicon_tags_151 },
    { 0x4a098170, SUC_lexicon_tags_66 },
    { 0x47a98171, SUC_lexicon_tags_19 },
    { 0x8f498172, SUC_lexicon_tags_3 },
    { 0x54e98173, SUC_lexicon_tags_17 },
    { 0x66c9816f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78a98176, SUC_lexicon_tags_2 },
    { 0x62098176, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4049817b, SUC_lexicon_tags_30 },
    { 0x88a9817c, SUC_lexicon_tags_21 },
    { 0xf189817b, SUC_lexicon_tags_7 },
    { 0x25e9817e, SUC_lexicon_tags_7 },
    { 0xd349817e, SUC_lexicon_tags_7 },
    { 0x9329817e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7698183, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf49818d, SUC_lexicon_tags_37 },
    { 0x91a9818e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfde98190, SUC_lexicon_tags_30 },
    { 0xb1a98190, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x19a98193, SUC_lexicon_tags_49 },
    { 0x6298194, SUC_lexicon_tags_8 },
    { 0x82698194, SUC_lexicon_tags_33 },
    { 0xe4298196, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe698199, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99e9819e, SUC_lexicon_tags_7 },
    { 0x9be9819f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc981a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe68981a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d2981a6, SUC_lexicon_tags_103 },
    { 0x77e981a6, SUC_lexicon_tags_15 },
    { 0x90e981a7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e981ab, SUC_lexicon_tags_11 },
    { 0xbc4981ab, SUC_lexicon_tags_8 },
    { 0xb62981ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854981b1, SUC_lexicon_tags_12 },
    { 0x11e981b2, SUC_lexicon_tags_14 },
    { 0xe6a981b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfbc981b5, SUC_lexicon_tags_9 },
    { 0x9c0981b6, SUC_lexicon_tags_28 },
    { 0x77c981b7, SUC_lexicon_tags_17 },
    { 0x204981b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c981bc, SUC_lexicon_tags_12 },
    { 0x474981bc, SUC_lexicon_tags_33 },
    { 0x3d4981be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be981c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20981c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcdc981c6, SUC_lexicon_tags_66 },
    { 0x290981c7, SUC_lexicon_tags_28 },
    { 0x7d2981c8, SUC_lexicon_tags_27 },
    { 0x566981c6, SUC_lexicon_tags_37 },
    { 0x4dc981ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e4981cc, SUC_lexicon_tags_12 },
    { 0x3ec981cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90981cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb24981d1, SUC_lexicon_tags_26 },
    { 0x39a981d2, SUC_lexicon_tags_47 },
    { 0x48a981d1, SUC_lexicon_tags_47 },
    { 0x48c981d3, SUC_lexicon_tags_82 },
    { 0x80981d5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e981d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x88e981db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x684981de, SUC_lexicon_tags_65 },
    { 0x2fc981de, SUC_lexicon_tags_25 },
    { 0xba0981e0, SUC_lexicon_tags_0 },
    { 0xc6981e1, SUC_lexicon_tags_6 },
    { 0xc48981e2, SUC_lexicon_tags_14 },
    { 0xd8c981e3, SUC_lexicon_tags_17 },
    { 0xa78981e4, SUC_lexicon_tags_79 },
    { 0xbd8981e2, SUC_lexicon_tags_37 },
    { 0xac8981e6, SUC_lexicon_tags_79 },
    { 0xf40981df, SUC_lexicon_tags_45 },
    { 0xa5e981e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x406981eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe12981ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae981f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdfa981f8, SUC_lexicon_tags_37 },
    { 0x6dc981f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7c981fb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x852981fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2709820c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb69820f, SUC_lexicon_tags_13 },
    { 0x4fc98210, SUC_lexicon_tags_34 },
    { 0x1c498210, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a98217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad89821c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8769821f, SUC_lexicon_tags_2 },
    { 0xbae98220, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a98223, SUC_lexicon_tags_37 },
    { 0xcea98224, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x10898226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a9822c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd29822e, SUC_lexicon_tags_1 },
    { 0xb1e9822e, SUC_lexicon_tags_6 },
    { 0x2c498230, SUC_lexicon_tags_25 },
    { 0xc2c98231, SUC_lexicon_tags_1 },
    { 0x5aa98231, SUC_lexicon_tags_2 },
    { 0x93a98232, SUC_lexicon_tags_6 },
    { 0x54498230, SUC_lexicon_tags_94 },
    { 0xf3698230, SUC_lexicon_tags_19 },
    { 0x5a69822e, SUC_lexicon_tags_46 },
    { 0x88e98237, SUC_lexicon_tags_1 },
    { 0x54e98238, SUC_lexicon_tags_28 },
    { 0x1ca98238, SUC_lexicon_tags_12 },
    { 0x5ee98237, SUC_lexicon_tags_98 },
    { 0xc8898238, SUC_lexicon_tags_2 },
    { 0x669823a, SUC_lexicon_tags_32 },
    { 0xf329823d, SUC_lexicon_tags_24 },
    { 0xfcc9823c, SUC_lexicon_tags_15 },
    { 0x96c9823f, SUC_lexicon_tags_33 },
    { 0xcf298237, SUC_lexicon_tags_7 },
    { 0x28098241, SUC_lexicon_tags_25 },
    { 0xc2298240, SUC_lexicon_tags_25 },
    { 0xfce98235, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c98249, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xece9824b, SUC_lexicon_tags_43 },
    { 0x1369824c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa249824f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13498254, SUC_lexicon_tags_12 },
    { 0x86498255, SUC_lexicon_tags_27 },
    { 0xa8e98255, SUC_lexicon_tags_1 },
    { 0x8b298257, SUC_lexicon_tags_17 },
    { 0x35898258, SUC_lexicon_tags_11 },
    { 0xc7898259, SUC_lexicon_tags_19 },
    { 0xd749825a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb689825c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc089825e, SUC_lexicon_tags_15 },
    { 0xd549825f, SUC_lexicon_tags_14 },
    { 0xc9825f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3498262, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56898267, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb789826c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f698271, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68498273, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1a98275, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4309827e, SUC_lexicon_tags_17 },
    { 0x609827f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e098285, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc98288, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd449828e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde498290, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4098293, SUC_lexicon_tags_25 },
    { 0x8ba98293, SUC_lexicon_tags_33 },
    { 0x30a98295, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe9829a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa329829e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32982a1, SUC_lexicon_tags_30 },
    { 0x73e982a2, SUC_lexicon_tags_28 },
    { 0xe8982a3, SUC_lexicon_tags_9 },
    { 0xa8a982a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8c2982a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80982aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf70982ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac8982ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8982b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x476982b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376982b8, SUC_lexicon_tags_17 },
    { 0xfcc982b8, SUC_lexicon_tags_7 },
    { 0x440982ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846982bf, SUC_lexicon_tags_7 },
    { 0x666982c0, SUC_lexicon_tags_37 },
    { 0xe4a982c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe42982c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ba982c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2982c8, SUC_lexicon_tags_7 },
    { 0x63a982c9, SUC_lexicon_tags_11 },
    { 0xef2982ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91e982cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x166982ce, SUC_lexicon_tags_32 },
    { 0x762982cf, SUC_lexicon_tags_47 },
    { 0x100982d0, SUC_lexicon_tags_29 },
    { 0xdf2982d0, SUC_lexicon_tags_28 },
    { 0x52982d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da982d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2982da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6982dd, SUC_lexicon_tags_63 },
    { 0xe94982de, SUC_lexicon_tags_32 },
    { 0x8de982df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30982e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xecc982e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18a982e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102982ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8982f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb06982f2, SUC_lexicon_tags_14 },
    { 0xe02982f3, SUC_lexicon_tags_60 },
    { 0x630982f2, SUC_lexicon_tags_7 },
    { 0xbe6982f5, SUC_lexicon_tags_37 },
    { 0x6aa982f6, SUC_lexicon_tags_17 },
    { 0xe94982f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12982fa, SUC_lexicon_tags_31 },
    { 0xde2982fb, SUC_lexicon_tags_1 },
    { 0x912982fc, SUC_lexicon_tags_14 },
    { 0xd88982fa, SUC_lexicon_tags_37 },
    { 0xdb4982fe, SUC_lexicon_tags_32 },
    { 0xc34982fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0498302, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x74498304, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1b298306, SUC_lexicon_tags_0 },
    { 0x7b498306, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeca98309, SUC_lexicon_tags_15 },
    { 0xa9a98309, SUC_lexicon_tags_0 },
    { 0xd4c98309, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec9830e, SUC_lexicon_tags_28 },
    { 0x8389830e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5698312, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6289831a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c69831e, SUC_lexicon_tags_5 },
    { 0x1a89831f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x23098321, SUC_lexicon_tags_17 },
    { 0x53098321, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9498324, SUC_lexicon_tags_14 },
    { 0xae298324, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa049832b, SUC_lexicon_tags_23 },
    { 0x5e69832c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e69832f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68098338, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2c9833a, SUC_lexicon_tags_860 },
    { 0x8029833b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee98340, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80098342, SUC_lexicon_tags_2 },
    { 0x50298343, SUC_lexicon_tags_17 },
    { 0x67298343, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7098347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf49834a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0e9834c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb898352, SUC_lexicon_tags_60 },
    { 0xca698353, SUC_lexicon_tags_12 },
    { 0xf7098354, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e098357, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c9835b, SUC_lexicon_tags_33 },
    { 0xb3e9835c, SUC_lexicon_tags_16 },
    { 0x6549835d, SUC_lexicon_tags_63 },
    { 0x59e9835e, SUC_lexicon_tags_37 },
    { 0xd169835d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0298361, SUC_lexicon_tags_27 },
    { 0xcc898362, SUC_lexicon_tags_60 },
    { 0x37a98363, SUC_lexicon_tags_584 },
    { 0x5b898364, SUC_lexicon_tags_11 },
    { 0xb1498365, SUC_lexicon_tags_48 },
    { 0xa5498364, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe649836a, SUC_lexicon_tags_29 },
    { 0x7269836b, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9098370, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d898373, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe9837a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5898382, SUC_lexicon_tags_51 },
    { 0xf0098382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d298385, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e298388, SUC_lexicon_tags_30 },
    { 0x7b298388, SUC_lexicon_tags_12 },
    { 0x7789838a, SUC_lexicon_tags_55 },
    { 0xdee98389, SUC_lexicon_tags_32 },
    { 0x10e9838c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e98390, SUC_lexicon_tags_11 },
    { 0x54698390, SUC_lexicon_tags_92 },
    { 0x53698392, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e98397, SUC_lexicon_tags_47 },
    { 0x9c498398, SUC_lexicon_tags_8 },
    { 0x1b298399, SUC_lexicon_tags_12 },
    { 0x58a9839a, SUC_lexicon_tags_35 },
    { 0x2a89839b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc149839d, SUC_lexicon_tags_7 },
    { 0xbf09839e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e6983a0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd5e983a2, SUC_lexicon_tags_17 },
    { 0xf80983a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe66983a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71a983b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed0983b4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3d4983b6, SUC_lexicon_tags_37 },
    { 0x2ac983b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x510983b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x704983bc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c983c1, SUC_lexicon_tags_37 },
    { 0xb94983c2, SUC_lexicon_tags_63 },
    { 0xf0e983c1, SUC_lexicon_tags_7 },
    { 0x58983c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98e983c6, SUC_lexicon_tags_17 },
    { 0x5e4983c7, SUC_lexicon_tags_11 },
    { 0x738983c8, SUC_lexicon_tags_37 },
    { 0x36a983c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4983ce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc84983d0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0983d4, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xb8c983d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x826983d8, SUC_lexicon_tags_1 },
    { 0x722983d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438983de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392983e4, SUC_lexicon_tags_17 },
    { 0x790983e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16983e7, SUC_lexicon_tags_37 },
    { 0xb96983e8, SUC_lexicon_tags_27 },
    { 0x962983e9, SUC_lexicon_tags_17 },
    { 0xaec983e7, SUC_lexicon_tags_40 },
    { 0xbb2983eb, SUC_lexicon_tags_33 },
    { 0x3a8983ec, SUC_lexicon_tags_40 },
    { 0x354983ed, SUC_lexicon_tags_7 },
    { 0xda6983ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944983f6, SUC_lexicon_tags_15 },
    { 0x940983f7, SUC_lexicon_tags_17 },
    { 0x464983f8, SUC_lexicon_tags_17 },
    { 0xaac983f8, SUC_lexicon_tags_11 },
    { 0x398983f6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a98403, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a498406, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x88498408, SUC_lexicon_tags_2 },
    { 0xdb498408, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc729840d, SUC_lexicon_tags_0 },
    { 0x449840e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x61698411, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x39e98414, SUC_lexicon_tags_34 },
    { 0xab498415, SUC_lexicon_tags_45 },
    { 0x72c98416, SUC_lexicon_tags_25 },
    { 0xa0698417, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c49841b, SUC_lexicon_tags_37 },
    { 0x3b69841c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3409841e, SUC_lexicon_tags_61 },
    { 0x2349841f, SUC_lexicon_tags_2 },
    { 0x88498420, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50298424, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e98428, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa6a9842a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa709842c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55298437, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb389843a, SUC_lexicon_tags_4 },
    { 0xce49843b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a98440, SUC_lexicon_tags_0 },
    { 0x6498440, SUC_lexicon_tags_133 },
    { 0xbb498442, SUC_lexicon_tags_31 },
    { 0xc4298440, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59098447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c9844b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f89844f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c98452, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2d898454, SUC_lexicon_tags_32 },
    { 0x1c98455, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92498458, SUC_lexicon_tags_4 },
    { 0x15698458, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe9c9845b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7709845d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e98461, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x70698463, SUC_lexicon_tags_11 },
    { 0xa8c98464, SUC_lexicon_tags_15 },
    { 0xff298464, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be9846a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbca9846c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c9846f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31e98471, SUC_lexicon_tags_11 },
    { 0x3e498471, SUC_lexicon_tags_17 },
    { 0xaf698473, SUC_lexicon_tags_12 },
    { 0x31c98473, SUC_lexicon_tags_4 },
    { 0x3c298475, SUC_lexicon_tags_15 },
    { 0x5d298476, SUC_lexicon_tags_4 },
    { 0xa6c98477, SUC_lexicon_tags_33 },
    { 0x57298476, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda98480, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a98483, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e98486, SUC_lexicon_tags_17 },
    { 0xdd298486, SUC_lexicon_tags_1 },
    { 0x40098488, SUC_lexicon_tags_10 },
    { 0xab298489, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54298490, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a98494, SUC_lexicon_tags_16 },
    { 0x83e98494, SUC_lexicon_tags_12 },
    { 0x54e98496, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9298499, SUC_lexicon_tags_66 },
    { 0xaa698499, SUC_lexicon_tags_16 },
    { 0x9789849b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa189849e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc984a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48984a7, SUC_lexicon_tags_27 },
    { 0x632984a8, SUC_lexicon_tags_37 },
    { 0xbde984a8, SUC_lexicon_tags_27 },
    { 0x68a984aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776984b0, SUC_lexicon_tags_11 },
    { 0xb6c984b0, SUC_lexicon_tags_37 },
    { 0x8d0984b0, SUC_lexicon_tags_65 },
    { 0x7cc984b3, SUC_lexicon_tags_12 },
    { 0x89c984b1, SUC_lexicon_tags_8 },
    { 0x89a984b1, SUC_lexicon_tags_17 },
    { 0x188984b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca984b9, SUC_lexicon_tags_37 },
    { 0x24c984ba, SUC_lexicon_tags_15 },
    { 0xf80984bb, SUC_lexicon_tags_30 },
    { 0x604984b9, SUC_lexicon_tags_37 },
    { 0xf30984ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a984c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15e984c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd48984c4, SUC_lexicon_tags_60 },
    { 0x8a2984c5, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0xbf0984c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32984ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a984cd, SUC_lexicon_tags_17 },
    { 0xa984ce, SUC_lexicon_tags_15 },
    { 0xee0984ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf1c984d1, SUC_lexicon_tags_27 },
    { 0x8bc984d2, SUC_lexicon_tags_12 },
    { 0xdcc984d3, SUC_lexicon_tags_1 },
    { 0x9f8984d3, SUC_lexicon_tags_11 },
    { 0xa6c984d5, SUC_lexicon_tags_17 },
    { 0xe66984d4, SUC_lexicon_tags_28 },
    { 0xe98984d5, SUC_lexicon_tags_18 },
    { 0xe8e984d8, SUC_lexicon_tags_47 },
    { 0xe46984d1, SUC_lexicon_tags_14 },
    { 0x4d4984da, SUC_lexicon_tags_63 },
    { 0xfb2984db, SUC_lexicon_tags_4 },
    { 0x3e4984dc, SUC_lexicon_tags_63 },
    { 0x96c984dc, SUC_lexicon_tags_37 },
    { 0xe02984db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2984e6, SUC_lexicon_tags_49 },
    { 0xf1e984e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e984ec, SUC_lexicon_tags_19 },
    { 0x7ce984ec, SUC_lexicon_tags_43 },
    { 0x162984ee, SUC_lexicon_tags_23 },
    { 0xf52984ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x58a984f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84984f4, SUC_lexicon_tags_29 },
    { 0x762984f4, SUC_lexicon_tags_17 },
    { 0xed8984f6, SUC_lexicon_tags_11 },
    { 0x212984f4, SUC_lexicon_tags_7 },
    { 0x1d0984f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8984fb, SUC_lexicon_tags_27 },
    { 0x450984fc, SUC_lexicon_tags_55 },
    { 0x9ac984fc, SUC_lexicon_tags_38 },
    { 0xf1e984fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4898502, SUC_lexicon_tags_0 },
    { 0x12098503, SUC_lexicon_tags_11 },
    { 0xb2898503, SUC_lexicon_tags_7 },
    { 0x91698504, SUC_lexicon_tags_27 },
    { 0xa7698504, SUC_lexicon_tags_10 },
    { 0x77898504, SUC_lexicon_tags_15 },
    { 0x26098503, SUC_lexicon_tags_16 },
    { 0x29698509, SUC_lexicon_tags_43 },
    { 0xc6c9850a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5e9850c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x66a9850f, SUC_lexicon_tags_37 },
    { 0xf0a9850f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b498513, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb098518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7409851a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61c9851c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd629851e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8898521, SUC_lexicon_tags_19 },
    { 0x65c98522, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6898526, SUC_lexicon_tags_37 },
    { 0xd7898527, SUC_lexicon_tags_38 },
    { 0xbc898527, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24a9852a, SUC_lexicon_tags_17 },
    { 0x7aa9852b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea98530, SUC_lexicon_tags_28 },
    { 0x9ac98530, SUC_lexicon_tags_37 },
    { 0x1ba98532, SUC_lexicon_tags_11 },
    { 0xfd098533, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3698535, SUC_lexicon_tags_12 },
    { 0x6de98536, SUC_lexicon_tags_65 },
    { 0xfd098537, SUC_lexicon_tags_2 },
    { 0xe1c98536, SUC_lexicon_tags_11 },
    { 0xef698537, SUC_lexicon_tags_11 },
    { 0xc1298539, SUC_lexicon_tags_12 },
    { 0x5cc9853b, SUC_lexicon_tags_17 },
    { 0x4309853c, SUC_lexicon_tags_15 },
    { 0xf609853d, SUC_lexicon_tags_2 },
    { 0x7869853b, SUC_lexicon_tags_87 },
    { 0x48a9853f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e98547, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1e9854a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29698552, SUC_lexicon_tags_11 },
    { 0xfea98552, SUC_lexicon_tags_37 },
    { 0xb2898553, SUC_lexicon_tags_1 },
    { 0xd9c98554, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbba98557, SUC_lexicon_tags_37 },
    { 0x6ca98557, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x76a9855a, SUC_lexicon_tags_38 },
    { 0x1009855b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd498560, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d298563, SUC_lexicon_tags_15 },
    { 0x35c98563, SUC_lexicon_tags_15 },
    { 0x6a698564, SUC_lexicon_tags_0 },
    { 0xd0498563, SUC_lexicon_tags_25 },
    { 0xf7298563, SUC_lexicon_tags_14 },
    { 0x39298568, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea98573, SUC_lexicon_tags_35 },
    { 0x30e98573, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a898577, SUC_lexicon_tags_1 },
    { 0x94a98578, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee9857a, SUC_lexicon_tags_8 },
    { 0x5f69857a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73298582, SUC_lexicon_tags_28 },
    { 0x65898582, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a98586, SUC_lexicon_tags_2 },
    { 0xb298587, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5729858c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cc9858e, SUC_lexicon_tags_17 },
    { 0x3589858e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29e98592, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9ca98594, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4098597, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb498599, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xec9859b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd929859d, SUC_lexicon_tags_26 },
    { 0x3d49859d, SUC_lexicon_tags_60 },
    { 0x7ea9859f, SUC_lexicon_tags_15 },
    { 0xfc2985a0, SUC_lexicon_tags_31 },
    { 0x8389859d, SUC_lexicon_tags_7 },
    { 0x822985a0, SUC_lexicon_tags_1 },
    { 0x776985a3, SUC_lexicon_tags_27 },
    { 0xe20985a3, SUC_lexicon_tags_87 },
    { 0x6649859d, SUC_lexicon_tags_37 },
    { 0xe7c985a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75e985a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x590985aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2985ae, SUC_lexicon_tags_1 },
    { 0x760985ae, SUC_lexicon_tags_16 },
    { 0xce0985b0, SUC_lexicon_tags_47 },
    { 0x412985b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe985b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x414985b6, SUC_lexicon_tags_7 },
    { 0xb86985b7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec985ba, SUC_lexicon_tags_61 },
    { 0x3ba985bb, SUC_lexicon_tags_8 },
    { 0x4b0985bc, SUC_lexicon_tags_37 },
    { 0x99c985bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a985c0, SUC_lexicon_tags_11 },
    { 0x466985c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1a985c3, SUC_lexicon_tags_87 },
    { 0xe2985c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76985c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba985ca, SUC_lexicon_tags_0 },
    { 0xabc985cb, SUC_lexicon_tags_60 },
    { 0x8a0985cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0985d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe985d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae985d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa985db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa985df, SUC_lexicon_tags_17 },
    { 0xd68985e0, SUC_lexicon_tags_11 },
    { 0x820985e1, SUC_lexicon_tags_7 },
    { 0x844985e1, SUC_lexicon_tags_23 },
    { 0x9fa985e3, SUC_lexicon_tags_37 },
    { 0x1ba985df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x576985e6, SUC_lexicon_tags_30 },
    { 0x208985e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6985ea, SUC_lexicon_tags_1 },
    { 0x584985ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdb8985ed, SUC_lexicon_tags_27 },
    { 0xe12985ee, SUC_lexicon_tags_64 },
    { 0x618985ee, SUC_lexicon_tags_14 },
    { 0x852985ee, SUC_lexicon_tags_7 },
    { 0x7b8985f0, SUC_lexicon_tags_12 },
    { 0x4a0985f2, SUC_lexicon_tags_12 },
    { 0xf3e985f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2e985f5, SUC_lexicon_tags_25 },
    { 0x6ea985f6, SUC_lexicon_tags_2 },
    { 0x498985f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2985f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc94985fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc98600, SUC_lexicon_tags_25 },
    { 0xe9298600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb498603, SUC_lexicon_tags_27 },
    { 0x98604, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9a098606, SUC_lexicon_tags_11 },
    { 0xefa98606, SUC_lexicon_tags_25 },
    { 0x3dc98608, SUC_lexicon_tags_25 },
    { 0xde498609, SUC_lexicon_tags_87 },
    { 0x3189860a, SUC_lexicon_tags_14 },
    { 0x3709860b, SUC_lexicon_tags_83 },
    { 0xa9860b, SUC_lexicon_tags_38 },
    { 0xe7c98608, SUC_lexicon_tags_42 },
    { 0xf2098607, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c98614, SUC_lexicon_tags_11 },
    { 0xa5e98614, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x10898618, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e9861e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ac98620, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xab298623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa98626, SUC_lexicon_tags_34 },
    { 0xb1a98627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1498629, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6e9862b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31498633, SUC_lexicon_tags_17 },
    { 0x67498634, SUC_lexicon_tags_63 },
    { 0x36698635, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeb698637, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc809863d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e98644, SUC_lexicon_tags_10 },
    { 0x38098644, SUC_lexicon_tags_28 },
    { 0xafc98644, SUC_lexicon_tags_9 },
    { 0x79498647, SUC_lexicon_tags_4 },
    { 0x20a98648, SUC_lexicon_tags_17 },
    { 0x36098648, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1069864e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e498652, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26698655, SUC_lexicon_tags_26 },
    { 0x79a98656, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15898664, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae9866b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98674, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36098678, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed89867d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a698680, SUC_lexicon_tags_23 },
    { 0xf5698680, SUC_lexicon_tags_1 },
    { 0x28e98681, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5898685, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc98689, SUC_lexicon_tags_23 },
    { 0x30898689, SUC_lexicon_tags_11 },
    { 0xcfc98689, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8a89868d, SUC_lexicon_tags_37 },
    { 0x8e09868e, SUC_lexicon_tags_63 },
    { 0x6de9868f, SUC_lexicon_tags_0 },
    { 0x1ac98690, SUC_lexicon_tags_65 },
    { 0x1c498691, SUC_lexicon_tags_17 },
    { 0x3d29868f, SUC_lexicon_tags_45 },
    { 0xbb898693, SUC_lexicon_tags_46 },
    { 0x33698694, SUC_lexicon_tags_14 },
    { 0xfc9868d, SUC_lexicon_tags_17 },
    { 0x24498696, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6609869b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c9869f, SUC_lexicon_tags_28 },
    { 0x34c9869f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a986a3, SUC_lexicon_tags_25 },
    { 0x4c4986a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x992986a6, SUC_lexicon_tags_11 },
    { 0xc5c986a7, SUC_lexicon_tags_21 },
    { 0xc3e986a7, SUC_lexicon_tags_2 },
    { 0x7a4986a9, SUC_lexicon_tags_262 },
    { 0xe78986a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666986af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdfe986b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc26986b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76986b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ee986b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf0986ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92986bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542986c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0986c6, SUC_lexicon_tags_0 },
    { 0xe2c986c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30986cc, SUC_lexicon_tags_12 },
    { 0x5fc986cd, SUC_lexicon_tags_79 },
    { 0xb92986ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66986d1, SUC_lexicon_tags_17 },
    { 0x7bc986d2, SUC_lexicon_tags_0 },
    { 0x54986d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x356986d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c6986d7, SUC_lexicon_tags_15 },
    { 0xcde986d8, SUC_lexicon_tags_19 },
    { 0x24a986d8, SUC_lexicon_tags_82 },
    { 0x5fa986d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4d4986dc, SUC_lexicon_tags_14 },
    { 0x4ea986dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda986ea, SUC_lexicon_tags_8 },
    { 0x2e8986ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70986ee, SUC_lexicon_tags_7 },
    { 0xfc0986ef, SUC_lexicon_tags_31 },
    { 0x3a0986ef, SUC_lexicon_tags_91 },
    { 0x530986f1, SUC_lexicon_tags_87 },
    { 0x562986f1, SUC_lexicon_tags_17 },
    { 0x7a0986f3, SUC_lexicon_tags_8 },
    { 0xbfc986f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa8986f6, SUC_lexicon_tags_25 },
    { 0xa70986f7, SUC_lexicon_tags_16 },
    { 0x24e986f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x184986fb, SUC_lexicon_tags_17 },
    { 0xd96986fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42986ff, SUC_lexicon_tags_37 },
    { 0x73898700, SUC_lexicon_tags_29 },
    { 0xf9c98701, SUC_lexicon_tags_37 },
    { 0x87498702, SUC_lexicon_tags_1 },
    { 0xbc898702, SUC_lexicon_tags_17 },
    { 0x1ee98704, SUC_lexicon_tags_21 },
    { 0xd8498701, SUC_lexicon_tags_29 },
    { 0x53098706, SUC_lexicon_tags_172 },
    { 0x14698707, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42298709, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8049870c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe9870f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68098714, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c98717, SUC_lexicon_tags_7 },
    { 0x29c98718, SUC_lexicon_tags_47 },
    { 0x59a98719, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x94a9871b, SUC_lexicon_tags_37 },
    { 0xfde9871c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x63e9871f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc898724, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x94a98726, SUC_lexicon_tags_12 },
    { 0x66098727, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b29872a, SUC_lexicon_tags_27 },
    { 0xb709872a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89898730, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf698735, SUC_lexicon_tags_136 },
    { 0xdac98736, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3498738, SUC_lexicon_tags_82 },
    { 0x8298739, SUC_lexicon_tags_7 },
    { 0x44498738, SUC_lexicon_tags_56 },
    { 0x7ec9873b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c89873d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3a9873f, SUC_lexicon_tags_0 },
    { 0x4ce98740, SUC_lexicon_tags_91 },
    { 0x82e98740, SUC_lexicon_tags_17 },
    { 0xb0698742, SUC_lexicon_tags_21 },
    { 0x7c898743, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a98749, SUC_lexicon_tags_33 },
    { 0xd5c98749, SUC_lexicon_tags_1 },
    { 0x3049874b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea9874d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e98752, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e98757, SUC_lexicon_tags_37 },
    { 0xd1c98758, SUC_lexicon_tags_12 },
    { 0xbfc98759, SUC_lexicon_tags_43 },
    { 0xaa09875a, SUC_lexicon_tags_17 },
    { 0x2d09875b, SUC_lexicon_tags_37 },
    { 0xcf69875c, SUC_lexicon_tags_43 },
    { 0xc1a9875b, SUC_lexicon_tags_31 },
    { 0x5689875d, SUC_lexicon_tags_16 },
    { 0xc6c9875d, SUC_lexicon_tags_11 },
    { 0x68c9875e, SUC_lexicon_tags_47 },
    { 0xd9898761, SUC_lexicon_tags_7 },
    { 0x77c98760, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x70e98764, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae98768, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa9876d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ac9876f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8e98771, SUC_lexicon_tags_47 },
    { 0x1b298772, SUC_lexicon_tags_17 },
    { 0x1ec98773, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x63a98775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7a98777, SUC_lexicon_tags_33 },
    { 0x28698777, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c09877a, SUC_lexicon_tags_37 },
    { 0x80e9877a, SUC_lexicon_tags_47 },
    { 0xda29877b, SUC_lexicon_tags_38 },
    { 0xb949877c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a98780, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf6498782, SUC_lexicon_tags_25 },
    { 0xaa298782, SUC_lexicon_tags_0 },
    { 0xadc98784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe049878a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe889878f, SUC_lexicon_tags_1 },
    { 0x50c98790, SUC_lexicon_tags_12 },
    { 0xfd09878f, SUC_lexicon_tags_37 },
    { 0xfd298792, SUC_lexicon_tags_1 },
    { 0x3e98793, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a98796, SUC_lexicon_tags_43 },
    { 0x42c98797, SUC_lexicon_tags_10 },
    { 0x82e98797, SUC_lexicon_tags_68 },
    { 0x56498799, SUC_lexicon_tags_25 },
    { 0xa3698797, SUC_lexicon_tags_12 },
    { 0x7509879b, SUC_lexicon_tags_1 },
    { 0x10e9879c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a9879f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede987a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8987b1, SUC_lexicon_tags_25 },
    { 0x168987b2, SUC_lexicon_tags_38 },
    { 0x7ee987b2, SUC_lexicon_tags_4 },
    { 0x6dc987b1, SUC_lexicon_tags_6 },
    { 0xbcc987b5, SUC_lexicon_tags_11 },
    { 0xa52987b6, SUC_lexicon_tags_17 },
    { 0x4e8987b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd32987b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaec987bb, SUC_lexicon_tags_8 },
    { 0x59c987bc, SUC_lexicon_tags_55 },
    { 0xedc987bd, SUC_lexicon_tags_6 },
    { 0x6e2987be, SUC_lexicon_tags_33 },
    { 0x70a987bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412987c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8987cc, SUC_lexicon_tags_15 },
    { 0x35e987cd, SUC_lexicon_tags_2 },
    { 0xd9a987cc, SUC_lexicon_tags_37 },
    { 0x798987cf, SUC_lexicon_tags_47 },
    { 0x9d0987cf, SUC_lexicon_tags_17 },
    { 0x9c987d1, SUC_lexicon_tags_25 },
    { 0x9e6987d1, SUC_lexicon_tags_61 },
    { 0x7ae987ce, SUC_lexicon_tags_49 },
    { 0xd76987d4, SUC_lexicon_tags_85 },
    { 0x6d0987d5, SUC_lexicon_tags_15 },
    { 0x572987d6, SUC_lexicon_tags_47 },
    { 0x8b4987d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeb4987d9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x258987db, SUC_lexicon_tags_37 },
    { 0x556987dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8be987de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638987e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x680987e4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x580987e7, SUC_lexicon_tags_63 },
    { 0xc8e987e7, SUC_lexicon_tags_13 },
    { 0x1d4987e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a987ec, SUC_lexicon_tags_17 },
    { 0x320987ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06987f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbf4987f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8987f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc987fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66987ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24898801, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0e98803, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a498807, SUC_lexicon_tags_7 },
    { 0x85698808, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd629880c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89098810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5098815, SUC_lexicon_tags_47 },
    { 0x75898816, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0c98818, SUC_lexicon_tags_37 },
    { 0x49298819, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f69881e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaca98820, SUC_lexicon_tags_7 },
    { 0x9a98820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe098828, SUC_lexicon_tags_8 },
    { 0x39898828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6698830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba98835, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a098837, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27a98839, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7e9883b, SUC_lexicon_tags_15 },
    { 0xd529883c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c98845, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45098848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1629884a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7be9884c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c9884f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8c98851, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xede98854, SUC_lexicon_tags_37 },
    { 0xd9898854, SUC_lexicon_tags_17 },
    { 0x3d098855, SUC_lexicon_tags_19 },
    { 0x5dc98857, SUC_lexicon_tags_77 },
    { 0xd5498858, SUC_lexicon_tags_18 },
    { 0xbe498857, SUC_lexicon_tags_38 },
    { 0xd6c98857, SUC_lexicon_tags_26 },
    { 0x99298859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a9885e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a98861, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b298864, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e98867, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf29886d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89898870, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26498874, SUC_lexicon_tags_55 },
    { 0xdda98874, SUC_lexicon_tags_46 },
    { 0x34098876, SUC_lexicon_tags_10 },
    { 0x82298877, SUC_lexicon_tags_14 },
    { 0x33698877, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1529887a, SUC_lexicon_tags_11 },
    { 0x5aa9887b, SUC_lexicon_tags_15 },
    { 0xdb29887c, SUC_lexicon_tags_11 },
    { 0x63c9887a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f898880, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83e98882, SUC_lexicon_tags_37 },
    { 0x67e98883, SUC_lexicon_tags_45 },
    { 0xd498884, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c98887, SUC_lexicon_tags_61 },
    { 0xf4098888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e9888c, SUC_lexicon_tags_6 },
    { 0x4869888d, SUC_lexicon_tags_37 },
    { 0xd9a9888d, SUC_lexicon_tags_12 },
    { 0xd8e9888d, SUC_lexicon_tags_15 },
    { 0x56c9888d, SUC_lexicon_tags_11 },
    { 0x6ba98890, SUC_lexicon_tags_8 },
    { 0x6909888e, SUC_lexicon_tags_37 },
    { 0xe1a98893, SUC_lexicon_tags_15 },
    { 0x17e98893, SUC_lexicon_tags_46 },
    { 0xfa298895, SUC_lexicon_tags_25 },
    { 0x89e98896, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e9889b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcbe9889d, SUC_lexicon_tags_60 },
    { 0x1649889e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0988a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a988a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b6988a8, SUC_lexicon_tags_11 },
    { 0x6aa988a9, SUC_lexicon_tags_28 },
    { 0x498988aa, SUC_lexicon_tags_25 },
    { 0x69c988a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaca988ad, SUC_lexicon_tags_613 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc988b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3bc988b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf34988b4, SUC_lexicon_tags_9 },
    { 0x3e6988b5, SUC_lexicon_tags_43 },
    { 0xf08988b6, SUC_lexicon_tags_25 },
    { 0xe3e988b7, SUC_lexicon_tags_7 },
    { 0xf0988b7, SUC_lexicon_tags_18 },
    { 0x5c6988b5, SUC_lexicon_tags_25 },
    { 0x178988ba, SUC_lexicon_tags_1 },
    { 0xe66988bb, SUC_lexicon_tags_15 },
    { 0x330988bc, SUC_lexicon_tags_8 },
    { 0x94c988bd, SUC_lexicon_tags_7 },
    { 0x6fc988be, SUC_lexicon_tags_1 },
    { 0x332988b9, SUC_lexicon_tags_15 },
    { 0x194988c0, SUC_lexicon_tags_17 },
    { 0xc94988c1, SUC_lexicon_tags_434 },
    { 0xd8e988c2, SUC_lexicon_tags_37 },
    { 0xd3e988ba, SUC_lexicon_tags_68 },
    { 0xf98988b5, SUC_lexicon_tags_15 },
    { 0x2c2988c5, SUC_lexicon_tags_14 },
    { 0xacc988c6, SUC_lexicon_tags_17 },
    { 0x7f2988c7, SUC_lexicon_tags_65 },
    { 0xd68988c8, SUC_lexicon_tags_15 },
    { 0x706988c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8988cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x166988cf, SUC_lexicon_tags_85 },
    { 0x236988cf, SUC_lexicon_tags_65 },
    { 0x588988d0, SUC_lexicon_tags_14 },
    { 0xbc4988cf, SUC_lexicon_tags_15 },
    { 0x76a988d3, SUC_lexicon_tags_60 },
    { 0xfb6988d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x39a988d6, SUC_lexicon_tags_7 },
    { 0x830988d7, SUC_lexicon_tags_107 },
    { 0x5d0988d8, SUC_lexicon_tags_37 },
    { 0x5d4988d9, SUC_lexicon_tags_37 },
    { 0x52c988d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb6988dc, SUC_lexicon_tags_79 },
    { 0x7ae988dd, SUC_lexicon_tags_37 },
    { 0xbfe988de, SUC_lexicon_tags_29 },
    { 0x372988dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaca988e1, SUC_lexicon_tags_28 },
    { 0xab2988e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e988e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5a2988e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a988ed, SUC_lexicon_tags_37 },
    { 0x9cc988ee, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e988f6, SUC_lexicon_tags_61 },
    { 0x16a988f7, SUC_lexicon_tags_16 },
    { 0x6c988f6, SUC_lexicon_tags_31 },
    { 0x18e988f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24988fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb0988fd, SUC_lexicon_tags_27 },
    { 0xf4a988fe, SUC_lexicon_tags_3 },
    { 0x6fa988ff, SUC_lexicon_tags_45 },
    { 0x7fc988ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8498902, SUC_lexicon_tags_45 },
    { 0x10498903, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e98906, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x98298908, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2498911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c98915, SUC_lexicon_tags_17 },
    { 0x4ec98916, SUC_lexicon_tags_25 },
    { 0x71a98916, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe5298919, SUC_lexicon_tags_1 },
    { 0x3129891a, SUC_lexicon_tags_26 },
    { 0xef49891a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31898922, SUC_lexicon_tags_14 },
    { 0x56898923, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0698925, SUC_lexicon_tags_1 },
    { 0x43e98926, SUC_lexicon_tags_19 },
    { 0x78298926, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89898929, SUC_lexicon_tags_47 },
    { 0x44698929, SUC_lexicon_tags_29 },
    { 0x27a9892a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6809892d, SUC_lexicon_tags_17 },
    { 0xeac9892d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e098932, SUC_lexicon_tags_1 },
    { 0x6f298933, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52698939, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8869893b, SUC_lexicon_tags_33 },
    { 0x7f69893b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c98941, SUC_lexicon_tags_37 },
    { 0xddc98942, SUC_lexicon_tags_15 },
    { 0xd0298943, SUC_lexicon_tags_17 },
    { 0x5c098944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdba98949, SUC_lexicon_tags_15 },
    { 0x4f49894a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe898951, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf898958, SUC_lexicon_tags_46 },
    { 0xc0498958, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5749895d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdee9895f, SUC_lexicon_tags_7 },
    { 0xd1098960, SUC_lexicon_tags_0 },
    { 0xb9498961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1898963, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76898965, SUC_lexicon_tags_31 },
    { 0x49298965, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0498968, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6169896c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a698974, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea98977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc698979, SUC_lexicon_tags_1 },
    { 0x6ae9897a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa5a9897c, SUC_lexicon_tags_11 },
    { 0x1809897d, SUC_lexicon_tags_19 },
    { 0x1a69897c, SUC_lexicon_tags_26 },
    { 0xae09897f, SUC_lexicon_tags_47 },
    { 0xe7c98980, SUC_lexicon_tags_17 },
    { 0xcf09897d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6349898a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd249898c, SUC_lexicon_tags_7 },
    { 0x2049898d, SUC_lexicon_tags_16 },
    { 0xc529898e, SUC_lexicon_tags_12 },
    { 0x469898f, SUC_lexicon_tags_15 },
    { 0x1049898c, SUC_lexicon_tags_15 },
    { 0x71e9898f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46898993, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7c898995, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d89899c, SUC_lexicon_tags_17 },
    { 0x2869899c, SUC_lexicon_tags_48 },
    { 0xa549899e, SUC_lexicon_tags_17 },
    { 0xa629899c, SUC_lexicon_tags_45 },
    { 0x13c9899e, SUC_lexicon_tags_37 },
    { 0x758989a1, SUC_lexicon_tags_2 },
    { 0x9c6989a2, SUC_lexicon_tags_27 },
    { 0x54989a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x350989a5, SUC_lexicon_tags_11 },
    { 0x452989a5, SUC_lexicon_tags_70 },
    { 0xc22989a7, SUC_lexicon_tags_23 },
    { 0xf26989a8, SUC_lexicon_tags_14 },
    { 0xc14989a5, SUC_lexicon_tags_17 },
    { 0x650989a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb52989ac, SUC_lexicon_tags_25 },
    { 0xfe6989ac, SUC_lexicon_tags_11 },
    { 0xcb2989ae, SUC_lexicon_tags_104 },
    { 0xe64989af, SUC_lexicon_tags_59 },
    { 0xb10989af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97a989b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30989b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc26989b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472989be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4989c3, SUC_lexicon_tags_11 },
    { 0xbac989c4, SUC_lexicon_tags_17 },
    { 0x82e989c3, SUC_lexicon_tags_25 },
    { 0xf0989c3, SUC_lexicon_tags_17 },
    { 0x4b8989c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x196989c9, SUC_lexicon_tags_2 },
    { 0x672989ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4989d4, SUC_lexicon_tags_55 },
    { 0x302989d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x500989d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280989db, SUC_lexicon_tags_47 },
    { 0xb42989dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28c989de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8989e2, SUC_lexicon_tags_7 },
    { 0x4f4989e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac989eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10989ee, SUC_lexicon_tags_25 },
    { 0xe9e989ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18989f2, SUC_lexicon_tags_82 },
    { 0x908989f3, SUC_lexicon_tags_10 },
    { 0xe5c989f2, SUC_lexicon_tags_14 },
    { 0x6a989f5, SUC_lexicon_tags_7 },
    { 0x28e989f6, SUC_lexicon_tags_37 },
    { 0xcc2989f7, SUC_lexicon_tags_1 },
    { 0x58e989f4, SUC_lexicon_tags_40 },
    { 0x154989f9, SUC_lexicon_tags_10 },
    { 0x7c2989fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d498a00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea98a06, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7b698a08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e98a0d, SUC_lexicon_tags_17 },
    { 0xe6a98a0e, SUC_lexicon_tags_28 },
    { 0x57298a0d, SUC_lexicon_tags_37 },
    { 0xf3098a0d, SUC_lexicon_tags_0 },
    { 0xe2a98a0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8098a19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e98a1c, SUC_lexicon_tags_43 },
    { 0x63a98a1c, SUC_lexicon_tags_33 },
    { 0x1d898a1e, SUC_lexicon_tags_11 },
    { 0x22698a1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a98a25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a98a28, SUC_lexicon_tags_48 },
    { 0x36298a29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x36498a2c, SUC_lexicon_tags_65 },
    { 0x58898a2c, SUC_lexicon_tags_2 },
    { 0xb7a98a2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb498a31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93298a35, SUC_lexicon_tags_30 },
    { 0x23e98a36, SUC_lexicon_tags_45 },
    { 0x3ec98a36, SUC_lexicon_tags_17 },
    { 0x9f298a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba98a3e, SUC_lexicon_tags_83 },
    { 0xb1a98a3f, SUC_lexicon_tags_7 },
    { 0x31898a3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55098a43, SUC_lexicon_tags_25 },
    { 0x27098a44, SUC_lexicon_tags_26 },
    { 0xc6a98a45, SUC_lexicon_tags_25 },
    { 0xa0c98a45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c98a49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f898a4c, SUC_lexicon_tags_45 },
    { 0xd1698a4c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e098a51, SUC_lexicon_tags_2 },
    { 0xc3698a51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x13898a54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x17498a57, SUC_lexicon_tags_6 },
    { 0xc298a58, SUC_lexicon_tags_15 },
    { 0x25c98a59, SUC_lexicon_tags_7 },
    { 0xa9898a58, SUC_lexicon_tags_12 },
    { 0xde098a59, SUC_lexicon_tags_15 },
    { 0xdf898a5a, SUC_lexicon_tags_11 },
    { 0xba098a5d, SUC_lexicon_tags_61 },
    { 0x96a98a58, SUC_lexicon_tags_10 },
    { 0x8b698a5f, SUC_lexicon_tags_69 },
    { 0x61098a5f, SUC_lexicon_tags_1 },
    { 0x2fe98a5f, SUC_lexicon_tags_15 },
    { 0x5c498a5b, SUC_lexicon_tags_16 },
    { 0x98a63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7298a67, SUC_lexicon_tags_15 },
    { 0x9a98a68, SUC_lexicon_tags_14 },
    { 0x32698a69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ee98a6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c898a72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa98a76, SUC_lexicon_tags_17 },
    { 0xad698a77, SUC_lexicon_tags_9 },
    { 0x9a298a76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc3098a7a, SUC_lexicon_tags_1 },
    { 0xb1698a7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51098a7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77298a85, SUC_lexicon_tags_87 },
    { 0xf5e98a86, SUC_lexicon_tags_27 },
    { 0x3e898a87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1898a91, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6298a94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a498a96, SUC_lexicon_tags_10 },
    { 0x3f298a97, SUC_lexicon_tags_66 },
    { 0xe7c98a98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4698a9c, SUC_lexicon_tags_32 },
    { 0xc3098a9c, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5098aa4, SUC_lexicon_tags_13 },
    { 0x72698aa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e98aa8, SUC_lexicon_tags_37 },
    { 0x66c98aa9, SUC_lexicon_tags_49 },
    { 0xaae98aaa, SUC_lexicon_tags_45 },
    { 0x34898aa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b698aae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6098ab6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8698ab8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83098aba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b298ac0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a98ac3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4098ac8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8298ace, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3dc98ad0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88e98ad2, SUC_lexicon_tags_11 },
    { 0x84298ad3, SUC_lexicon_tags_37 },
    { 0xbc098ad3, SUC_lexicon_tags_1 },
    { 0xe498ad2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8898ad8, SUC_lexicon_tags_14 },
    { 0x50a98ad9, SUC_lexicon_tags_25 },
    { 0xba98ada, SUC_lexicon_tags_37 },
    { 0xd2098ada, SUC_lexicon_tags_11 },
    { 0x9d298ad9, SUC_lexicon_tags_13 },
    { 0x29498add, SUC_lexicon_tags_25 },
    { 0x87098ade, SUC_lexicon_tags_26 },
    { 0x2b098ade, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c98ae2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xed098ae5, SUC_lexicon_tags_0 },
    { 0xd1098ae5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe98aeb, SUC_lexicon_tags_437 },
    { 0x2a898aeb, SUC_lexicon_tags_11 },
    { 0xba898aed, SUC_lexicon_tags_37 },
    { 0xf4a98aec, SUC_lexicon_tags_0 },
    { 0xec098aef, SUC_lexicon_tags_23 },
    { 0xbb498aef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54298af5, SUC_lexicon_tags_12 },
    { 0x3bc98af6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cc98af8, SUC_lexicon_tags_39 },
    { 0xb4c98af8, SUC_lexicon_tags_63 },
    { 0x41898afa, SUC_lexicon_tags_1 },
    { 0x14298afb, SUC_lexicon_tags_846 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9898b02, SUC_lexicon_tags_2 },
    { 0x4ac98b03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7898b06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6098b0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da98b10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6498b13, SUC_lexicon_tags_1 },
    { 0x7ee98b14, SUC_lexicon_tags_1 },
    { 0xe2e98b13, SUC_lexicon_tags_1 },
    { 0xca898b16, SUC_lexicon_tags_8 },
    { 0xf8498b16, SUC_lexicon_tags_1 },
    { 0xf9c98b18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3898b1a, SUC_lexicon_tags_25 },
    { 0x8ae98b1b, SUC_lexicon_tags_37 },
    { 0x93698b1c, SUC_lexicon_tags_12 },
    { 0xfe498b1c, SUC_lexicon_tags_15 },
    { 0x5de98b1b, SUC_lexicon_tags_0 },
    { 0x86698b1d, SUC_lexicon_tags_15 },
    { 0x79298b20, SUC_lexicon_tags_6 },
    { 0xe9498b21, SUC_lexicon_tags_38 },
    { 0x3dc98b1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x99e98b24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc98b2b, SUC_lexicon_tags_15 },
    { 0x9e698b2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97698b30, SUC_lexicon_tags_33 },
    { 0x8ca98b31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f898b33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcda98b35, SUC_lexicon_tags_47 },
    { 0xd0498b36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff298b38, SUC_lexicon_tags_25 },
    { 0x56e98b38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0e98b3b, SUC_lexicon_tags_7 },
    { 0x8b298b3c, SUC_lexicon_tags_27 },
    { 0xed298b3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5e98b3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d098b41, SUC_lexicon_tags_15 },
    { 0x6ec98b42, SUC_lexicon_tags_9 },
    { 0x72898b43, SUC_lexicon_tags_4 },
    { 0x80c98b44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98b46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x42a98b48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee98b4e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a698b50, SUC_lexicon_tags_1 },
    { 0x31498b51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1098b54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6098b57, SUC_lexicon_tags_33 },
    { 0x22698b57, SUC_lexicon_tags_7 },
    { 0x22698b58, SUC_lexicon_tags_27 },
    { 0xeae98b5a, SUC_lexicon_tags_27 },
    { 0x8fa98b5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb1c98b5d, SUC_lexicon_tags_8 },
    { 0x80a98b5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c98b62, SUC_lexicon_tags_19 },
    { 0xeec98b62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73c98b65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17e98b67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fe98b69, SUC_lexicon_tags_173 },
    { 0x9f898b6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x56e98b6c, SUC_lexicon_tags_63 },
    { 0xd7098b6d, SUC_lexicon_tags_12 },
    { 0xbc98b6e, SUC_lexicon_tags_1 },
    { 0xee098b6d, SUC_lexicon_tags_15 },
    { 0x14898b6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4698b73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca98b78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6498b7a, SUC_lexicon_tags_37 },
    { 0x56098b7b, SUC_lexicon_tags_8 },
    { 0x61c98b7a, SUC_lexicon_tags_2 },
    { 0xc8698b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6898b82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95098b86, SUC_lexicon_tags_33 },
    { 0xf3498b87, SUC_lexicon_tags_26 },
    { 0x8ac98b88, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa498b8a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0498b8d, SUC_lexicon_tags_12 },
    { 0xb3a98b8e, SUC_lexicon_tags_29 },
    { 0x45498b8f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c898b94, SUC_lexicon_tags_14 },
    { 0xe4c98b95, SUC_lexicon_tags_1 },
    { 0xa4498b96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6498b9a, SUC_lexicon_tags_63 },
    { 0x9e298b9b, SUC_lexicon_tags_11 },
    { 0x5ce98b9c, SUC_lexicon_tags_15 },
    { 0x10a98b9b, SUC_lexicon_tags_12 },
    { 0xf5e98b9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f698ba3, SUC_lexicon_tags_25 },
    { 0x85498ba4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf698ba6, SUC_lexicon_tags_9 },
    { 0x27898ba6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x77498baa, SUC_lexicon_tags_29 },
    { 0x7e498baa, SUC_lexicon_tags_15 },
    { 0x21898bab, SUC_lexicon_tags_9 },
    { 0xf4298bad, SUC_lexicon_tags_85 },
    { 0x81c98baa, SUC_lexicon_tags_32 },
    { 0x21c98baf, SUC_lexicon_tags_14 },
    { 0x7c098bb0, SUC_lexicon_tags_7 },
    { 0x76898baf, SUC_lexicon_tags_11 },
    { 0x1fa98bb2, SUC_lexicon_tags_28 },
    { 0x11a98bb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a98bb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d898bb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c98bbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2098bc2, SUC_lexicon_tags_16 },
    { 0x3d898bc2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x96898bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e98bc8, SUC_lexicon_tags_47 },
    { 0x44e98bc8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a98bcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca698bcf, SUC_lexicon_tags_7 },
    { 0xb1698bd0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6498bd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dc98bd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55298bd8, SUC_lexicon_tags_2 },
    { 0x6fa98bd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda498bdb, SUC_lexicon_tags_4 },
    { 0x71298bdb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb698bdf, SUC_lexicon_tags_29 },
    { 0x60498be0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea98be3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6298be9, SUC_lexicon_tags_8 },
    { 0xfdc98bea, SUC_lexicon_tags_43 },
    { 0x87898beb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c98bef, SUC_lexicon_tags_23 },
    { 0x94a98bef, SUC_lexicon_tags_17 },
    { 0x5d498bef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c98bf7, SUC_lexicon_tags_34 },
    { 0x85a98bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad098bfa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac98bfe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae98c03, SUC_lexicon_tags_38 },
    { 0x4b098c04, SUC_lexicon_tags_65 },
    { 0xa6e98c05, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da98c0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da98c0e, SUC_lexicon_tags_31 },
    { 0xfc498c0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77498c12, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e98c1a, SUC_lexicon_tags_12 },
    { 0xd7698c1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3298c1f, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe698c26, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x33498c29, SUC_lexicon_tags_79 },
    { 0x2c898c2a, SUC_lexicon_tags_17 },
    { 0x45698c2a, SUC_lexicon_tags_87 },
    { 0xe1a98c2c, SUC_lexicon_tags_63 },
    { 0x94698c2d, SUC_lexicon_tags_2 },
    { 0xb1898c29, SUC_lexicon_tags_70 },
    { 0xda698c2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79e98c31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c98c38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac98c3d, SUC_lexicon_tags_47 },
    { 0xc8a98c3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1698c42, SUC_lexicon_tags_25 },
    { 0xb0898c43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x82498c45, SUC_lexicon_tags_43 },
    { 0x79098c45, SUC_lexicon_tags_25 },
    { 0xa3698c47, SUC_lexicon_tags_25 },
    { 0xcf898c48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96698c4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21498c4c, SUC_lexicon_tags_28 },
    { 0x70698c4c, SUC_lexicon_tags_43 },
    { 0x7f698c4e, SUC_lexicon_tags_29 },
    { 0xbbe98c4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc98c54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x68e98c56, SUC_lexicon_tags_17 },
    { 0x28c98c57, SUC_lexicon_tags_25 },
    { 0x79a98c58, SUC_lexicon_tags_1 },
    { 0xe6298c59, SUC_lexicon_tags_8 },
    { 0xc5098c58, SUC_lexicon_tags_12 },
    { 0xfc498c59, SUC_lexicon_tags_7 },
    { 0xb3a98c5c, SUC_lexicon_tags_37 },
    { 0xe6498c5c, SUC_lexicon_tags_89 },
    { 0xab298c5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xab298c61, SUC_lexicon_tags_15 },
    { 0x6ac98c61, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdf698c64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3298c6b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd7298c6d, SUC_lexicon_tags_82 },
    { 0xb6a98c6d, SUC_lexicon_tags_8 },
    { 0xee098c6f, SUC_lexicon_tags_30 },
    { 0x72298c70, SUC_lexicon_tags_7 },
    { 0xa4098c6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3098c73, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b298c77, SUC_lexicon_tags_45 },
    { 0x40c98c78, SUC_lexicon_tags_17 },
    { 0xa7298c79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce98c7d, SUC_lexicon_tags_29 },
    { 0x1de98c7d, SUC_lexicon_tags_48 },
    { 0xa7898c7d, SUC_lexicon_tags_23 },
    { 0xdcc98c7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e98c83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47898c89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf498c8d, SUC_lexicon_tags_25 },
    { 0xade98c8e, SUC_lexicon_tags_183 },
    { 0x30098c8f, SUC_lexicon_tags_45 },
    { 0x41698c90, SUC_lexicon_tags_15 },
    { 0x95498c91, SUC_lexicon_tags_17 },
    { 0x24698c92, SUC_lexicon_tags_7 },
    { 0x5c098c92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x60498c95, SUC_lexicon_tags_27 },
    { 0x97c98c96, SUC_lexicon_tags_37 },
    { 0xb7898c97, SUC_lexicon_tags_27 },
    { 0x96098c98, SUC_lexicon_tags_32 },
    { 0x87a98c99, SUC_lexicon_tags_12 },
    { 0x32e98c98, SUC_lexicon_tags_17 },
    { 0xe4298c9b, SUC_lexicon_tags_13 },
    { 0x5a298c9c, SUC_lexicon_tags_75 },
    { 0xf8498c9d, SUC_lexicon_tags_14 },
    { 0x2e498c9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e98ca1, SUC_lexicon_tags_2 },
    { 0xb3498ca2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef898ca8, SUC_lexicon_tags_37 },
    { 0x3b498ca9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25298cb3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc98cb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f898cb9, SUC_lexicon_tags_25 },
    { 0xe7098cb9, SUC_lexicon_tags_14 },
    { 0x9f298cbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe1498cbd, SUC_lexicon_tags_14 },
    { 0xd0e98cbd, SUC_lexicon_tags_11 },
    { 0x2da98cbf, SUC_lexicon_tags_773 },
    { 0xaea98cc0, SUC_lexicon_tags_7 },
    { 0x99e98cbf, SUC_lexicon_tags_11 },
    { 0xd4498cc0, SUC_lexicon_tags_23 },
    { 0x8c898cc3, SUC_lexicon_tags_12 },
    { 0xf9698cc3, SUC_lexicon_tags_21 },
    { 0x8e298cc4, SUC_lexicon_tags_37 },
    { 0x79898cc6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xcf098cc8, SUC_lexicon_tags_11 },
    { 0x76098cc9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6ba98ccb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc098ccf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a98cd3, SUC_lexicon_tags_11 },
    { 0x40698cd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d498ce5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29098ce7, SUC_lexicon_tags_37 },
    { 0x8a298ce8, SUC_lexicon_tags_1 },
    { 0xe5498ce9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75698ceb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35898cf2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6898cf5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x13898cf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cc98cfa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x46498cfd, SUC_lexicon_tags_1 },
    { 0xf4c98cfe, SUC_lexicon_tags_908 },
    { 0, NULL },
    { 0xa0098d00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69298d04, SUC_lexicon_tags_77 },
    { 0x18698d04, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x61498d07, SUC_lexicon_tags_15 },
    { 0x7b898d07, SUC_lexicon_tags_2 },
    { 0xb1098d09, SUC_lexicon_tags_11 },
    { 0x17298d09, SUC_lexicon_tags_6 },
    { 0x19c98d0b, SUC_lexicon_tags_11 },
    { 0xf8e98d09, SUC_lexicon_tags_10 },
    { 0xb4698d07, SUC_lexicon_tags_19 },
    { 0xf5698d0e, SUC_lexicon_tags_74 },
    { 0x95098d0e, SUC_lexicon_tags_17 },
    { 0x74e98d10, SUC_lexicon_tags_1 },
    { 0xaf098d11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71898d15, SUC_lexicon_tags_11 },
    { 0xe9498d15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbe98d18, SUC_lexicon_tags_14 },
    { 0xbec98d18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5298d1b, SUC_lexicon_tags_47 },
    { 0xa2c98d1c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23698d2b, SUC_lexicon_tags_69 },
    { 0x51298d2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6698d2e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7ce98d30, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf4298d32, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc98d39, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f498d3c, SUC_lexicon_tags_29 },
    { 0xd7698d3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64a98d40, SUC_lexicon_tags_11 },
    { 0x78c98d41, SUC_lexicon_tags_0 },
    { 0x7cc98d41, SUC_lexicon_tags_11 },
    { 0x40a98d43, SUC_lexicon_tags_83 },
    { 0xd9098d44, SUC_lexicon_tags_33 },
    { 0x99a98d45, SUC_lexicon_tags_1 },
    { 0xb7698d46, SUC_lexicon_tags_4 },
    { 0x3d098d45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38a98d49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae498d4b, SUC_lexicon_tags_14 },
    { 0x9f298d4c, SUC_lexicon_tags_25 },
    { 0x40698d4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72c98d4f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27898d53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f898d57, SUC_lexicon_tags_25 },
    { 0x5ac98d57, SUC_lexicon_tags_33 },
    { 0x30898d59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47298d5c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc5298d5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc98d62, SUC_lexicon_tags_13 },
    { 0x37c98d62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea898d68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x43298d6b, SUC_lexicon_tags_28 },
    { 0x1fa98d6b, SUC_lexicon_tags_9 },
    { 0x6b098d6d, SUC_lexicon_tags_69 },
    { 0x54e98d6b, SUC_lexicon_tags_7 },
    { 0x5c698d6f, SUC_lexicon_tags_25 },
    { 0xc98d6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca98d78, SUC_lexicon_tags_47 },
    { 0x96298d78, SUC_lexicon_tags_2 },
    { 0x18e98d7a, SUC_lexicon_tags_11 },
    { 0x22698d7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6098d84, SUC_lexicon_tags_37 },
    { 0xbb298d84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3bc98d87, SUC_lexicon_tags_47 },
    { 0xfa898d88, SUC_lexicon_tags_12 },
    { 0xcec98d89, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xab098d8b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe1e98d8d, SUC_lexicon_tags_2 },
    { 0x7a098d8e, SUC_lexicon_tags_32 },
    { 0x73c98d8d, SUC_lexicon_tags_2 },
    { 0x44698d90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c98d93, SUC_lexicon_tags_60 },
    { 0x39698d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8298d98, SUC_lexicon_tags_43 },
    { 0xbd098d98, SUC_lexicon_tags_15 },
    { 0x94898d9a, SUC_lexicon_tags_28 },
    { 0xd6a98d98, SUC_lexicon_tags_14 },
    { 0x9a98d9c, SUC_lexicon_tags_7 },
    { 0xa7498d9d, SUC_lexicon_tags_63 },
    { 0xaca98d9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf698da5, SUC_lexicon_tags_37 },
    { 0xfa498da6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7298dad, SUC_lexicon_tags_33 },
    { 0x71498dae, SUC_lexicon_tags_55 },
    { 0xc7a98daf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x62898db1, SUC_lexicon_tags_1 },
    { 0x47698db2, SUC_lexicon_tags_25 },
    { 0xa9a98db3, SUC_lexicon_tags_25 },
    { 0x44898db3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e98db9, SUC_lexicon_tags_11 },
    { 0x4e898dba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23098dbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a98dc4, SUC_lexicon_tags_17 },
    { 0x9a098dc4, SUC_lexicon_tags_12 },
    { 0x23698dc6, SUC_lexicon_tags_7 },
    { 0xe2898dc6, SUC_lexicon_tags_12 },
    { 0x46c98dc8, SUC_lexicon_tags_60 },
    { 0xf8098dc9, SUC_lexicon_tags_12 },
    { 0x85698dc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8298dce, SUC_lexicon_tags_15 },
    { 0x97a98dcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20498dd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92498dd6, SUC_lexicon_tags_23 },
    { 0x3be98dd7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3aa98dd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd298ddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37298de2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d698de4, SUC_lexicon_tags_2 },
    { 0x82498de4, SUC_lexicon_tags_85 },
    { 0xce698de6, SUC_lexicon_tags_6 },
    { 0xa1298de5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe098de9, SUC_lexicon_tags_65 },
    { 0x7b298de9, SUC_lexicon_tags_8 },
    { 0xa9a98deb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1498dee, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xae098df0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc0a98df2, SUC_lexicon_tags_49 },
    { 0x56c98df2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7298df5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf9e98df7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0898df9, SUC_lexicon_tags_15 },
    { 0x9da98dfa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe3898dfc, SUC_lexicon_tags_18 },
    { 0xd8898dfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5298dff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee98e07, SUC_lexicon_tags_29 },
    { 0x9a698e07, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b098e0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f298e0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0698e13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e898e18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a98e1d, SUC_lexicon_tags_17 },
    { 0xdec98e1d, SUC_lexicon_tags_34 },
    { 0xf4698e1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f498e23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a98e28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc098e2a, SUC_lexicon_tags_15 },
    { 0x7e898e2b, SUC_lexicon_tags_17 },
    { 0xf3898e2c, SUC_lexicon_tags_71 },
    { 0x34c98e2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2898e32, SUC_lexicon_tags_11 },
    { 0x91698e33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3298e37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0298e3a, SUC_lexicon_tags_25 },
    { 0xfa898e3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d898e3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a98e41, SUC_lexicon_tags_63 },
    { 0x66e98e41, SUC_lexicon_tags_204 },
    { 0x89298e43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53098e48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe098e4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59c98e4c, SUC_lexicon_tags_12 },
    { 0xc5898e4d, SUC_lexicon_tags_7 },
    { 0x17498e4e, SUC_lexicon_tags_11 },
    { 0xabe98e4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69098e55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa98e58, SUC_lexicon_tags_12 },
    { 0xbc098e59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a98e5c, SUC_lexicon_tags_25 },
    { 0xda898e5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66498e61, SUC_lexicon_tags_31 },
    { 0xff898e62, SUC_lexicon_tags_15 },
    { 0x6ce98e63, SUC_lexicon_tags_43 },
    { 0xc1698e62, SUC_lexicon_tags_17 },
    { 0x38a98e65, SUC_lexicon_tags_68 },
    { 0xee698e65, SUC_lexicon_tags_25 },
    { 0x5a98e67, SUC_lexicon_tags_37 },
    { 0x25698e65, SUC_lexicon_tags_30 },
    { 0xd2698e66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94298e6c, SUC_lexicon_tags_17 },
    { 0xf4698e6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6498e6f, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xd4098e71, SUC_lexicon_tags_6 },
    { 0xd9698e71, SUC_lexicon_tags_37 },
    { 0x43e98e73, SUC_lexicon_tags_4 },
    { 0x12e98e73, SUC_lexicon_tags_15 },
    { 0xccc98e72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e98e7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5bc98e7c, SUC_lexicon_tags_26 },
    { 0xb9c98e7d, SUC_lexicon_tags_37 },
    { 0x72a98e7e, SUC_lexicon_tags_37 },
    { 0x3ec98e7f, SUC_lexicon_tags_21 },
    { 0x1aa98e7f, SUC_lexicon_tags_46 },
    { 0x15898e81, SUC_lexicon_tags_12 },
    { 0x31898e7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x74298e85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a498e8d, SUC_lexicon_tags_37 },
    { 0x14e98e8e, SUC_lexicon_tags_1 },
    { 0xe2898e8f, SUC_lexicon_tags_15 },
    { 0xc8298e8e, SUC_lexicon_tags_108 },
    { 0x53e98e8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e698e96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2da98e98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x67698e9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae98e9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2a98ea0, SUC_lexicon_tags_34 },
    { 0x8ba98ea1, SUC_lexicon_tags_39 },
    { 0xe7898ea2, SUC_lexicon_tags_33 },
    { 0x2a898ea3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xae898ea5, SUC_lexicon_tags_39 },
    { 0x19c98ea5, SUC_lexicon_tags_129 },
    { 0x6e498ea6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2a498ea9, SUC_lexicon_tags_2 },
    { 0x1aa98eaa, SUC_lexicon_tags_8 },
    { 0xe3898eab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83898eaf, SUC_lexicon_tags_33 },
    { 0x6e098eaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc698eb2, SUC_lexicon_tags_10 },
    { 0x23098eb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7298eba, SUC_lexicon_tags_1 },
    { 0x58e98eba, SUC_lexicon_tags_1 },
    { 0x1aa98eba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34498ec0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6098ec2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c98ecc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec98ed4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa298ed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed498ed9, SUC_lexicon_tags_37 },
    { 0xc6098ed9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26898ede, SUC_lexicon_tags_158 },
    { 0x6a898ede, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef498ee5, SUC_lexicon_tags_47 },
    { 0x21298ee6, SUC_lexicon_tags_7 },
    { 0xe3298ee7, SUC_lexicon_tags_8 },
    { 0x33898ee8, SUC_lexicon_tags_1 },
    { 0x19898ee5, SUC_lexicon_tags_87 },
    { 0xbfa98eea, SUC_lexicon_tags_37 },
    { 0xd6c98eeb, SUC_lexicon_tags_19 },
    { 0x82098ee6, SUC_lexicon_tags_34 },
    { 0xbce98eeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f898eef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde698ef1, SUC_lexicon_tags_1 },
    { 0xc6e98ef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80698ef4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17498ef9, SUC_lexicon_tags_25 },
    { 0x5c898efa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf898efd, SUC_lexicon_tags_5 },
    { 0x11498efe, SUC_lexicon_tags_13 },
    { 0x40098efd, SUC_lexicon_tags_10 },
    { 0xf7e98efd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3498f05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc298f08, SUC_lexicon_tags_23 },
    { 0x1c298f08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcee98f0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25898f0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ac98f0f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda98f12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8e898f14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36098f18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b498f1e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76498f29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ea98f2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c98f2e, SUC_lexicon_tags_23 },
    { 0xcfe98f2f, SUC_lexicon_tags_17 },
    { 0x65298f30, SUC_lexicon_tags_107 },
    { 0x59c98f31, SUC_lexicon_tags_37 },
    { 0xace98f31, SUC_lexicon_tags_38 },
    { 0xc6098f33, SUC_lexicon_tags_45 },
    { 0x46898f34, SUC_lexicon_tags_11 },
    { 0x55a98f32, SUC_lexicon_tags_37 },
    { 0x2a698f36, SUC_lexicon_tags_4 },
    { 0xe2898f2f, SUC_lexicon_tags_38 },
    { 0x16898f38, SUC_lexicon_tags_63 },
    { 0xba298f39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a698f3c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc98f3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc98f42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3a98f44, SUC_lexicon_tags_7 },
    { 0x8c098f45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1e98f47, SUC_lexicon_tags_25 },
    { 0x42698f48, SUC_lexicon_tags_19 },
    { 0x55498f48, SUC_lexicon_tags_17 },
    { 0x5c698f4a, SUC_lexicon_tags_43 },
    { 0xae898f4b, SUC_lexicon_tags_38 },
    { 0xdee98f4c, SUC_lexicon_tags_16 },
    { 0xd8898f4c, SUC_lexicon_tags_23 },
    { 0xc0298f4e, SUC_lexicon_tags_148 },
    { 0xc4498f49, SUC_lexicon_tags_25 },
    { 0x8ce98f4d, SUC_lexicon_tags_27 },
    { 0x13e98f51, SUC_lexicon_tags_25 },
    { 0xbb098f52, SUC_lexicon_tags_63 },
    { 0x63898f53, SUC_lexicon_tags_37 },
    { 0x2c098f4e, SUC_lexicon_tags_27 },
    { 0x17298f55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3898f59, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc98f5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d698f64, SUC_lexicon_tags_25 },
    { 0x4b498f65, SUC_lexicon_tags_11 },
    { 0x4dc98f66, SUC_lexicon_tags_25 },
    { 0x4698f64, SUC_lexicon_tags_17 },
    { 0xbf098f68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6498f6b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xab698f6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e98f70, SUC_lexicon_tags_47 },
    { 0x54c98f71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e98f75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1498f79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c98f7c, SUC_lexicon_tags_15 },
    { 0xaa098f7d, SUC_lexicon_tags_2 },
    { 0x1c498f7d, SUC_lexicon_tags_14 },
    { 0x75098f7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffe98f81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29098f83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15e98f89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6898f8f, SUC_lexicon_tags_7 },
    { 0xab898f8f, SUC_lexicon_tags_43 },
    { 0x59098f91, SUC_lexicon_tags_45 },
    { 0x73e98f91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78898f96, SUC_lexicon_tags_11 },
    { 0x67a98f96, SUC_lexicon_tags_26 },
    { 0x88898f97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24a98f9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78698f9e, SUC_lexicon_tags_49 },
    { 0x8aa98f9f, SUC_lexicon_tags_4 },
    { 0x26698f9f, SUC_lexicon_tags_66 },
    { 0xd8a98fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51698fa3, SUC_lexicon_tags_7 },
    { 0x98c98fa3, SUC_lexicon_tags_7 },
    { 0x4c098fa5, SUC_lexicon_tags_19 },
    { 0x3e98fa6, SUC_lexicon_tags_15 },
    { 0x81298fa5, SUC_lexicon_tags_32 },
    { 0x9cc98fa8, SUC_lexicon_tags_25 },
    { 0x6fe98fa7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3098fad, SUC_lexicon_tags_65 },
    { 0xb6c98fae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a98fb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e98fb3, SUC_lexicon_tags_21 },
    { 0x25c98fb4, SUC_lexicon_tags_14 },
    { 0xb9c98fb5, SUC_lexicon_tags_7 },
    { 0x4a898fb6, SUC_lexicon_tags_14 },
    { 0xd5898fb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa9a98fb9, SUC_lexicon_tags_12 },
    { 0x70e98fba, SUC_lexicon_tags_7 },
    { 0xb9e98fbb, SUC_lexicon_tags_7 },
    { 0x7b298fbb, SUC_lexicon_tags_37 },
    { 0xc4e98fbd, SUC_lexicon_tags_25 },
    { 0x96498fbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb7698fc0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x27298fc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94498fc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd498fcc, SUC_lexicon_tags_25 },
    { 0xc1898fcd, SUC_lexicon_tags_12 },
    { 0xdcc98fce, SUC_lexicon_tags_87 },
    { 0x77898fcf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea98fd3, SUC_lexicon_tags_1 },
    { 0xe9098fd4, SUC_lexicon_tags_37 },
    { 0x63a98fd3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa98fd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfba98fda, SUC_lexicon_tags_37 },
    { 0xde498fda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4498fdd, SUC_lexicon_tags_59 },
    { 0x36c98fdd, SUC_lexicon_tags_60 },
    { 0xc5c98fdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22098fe1, SUC_lexicon_tags_25 },
    { 0x7e898fe2, SUC_lexicon_tags_7 },
    { 0x35098fe3, SUC_lexicon_tags_29 },
    { 0x27698fe2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad498feb, SUC_lexicon_tags_138 },
    { 0x2ae98fec, SUC_lexicon_tags_28 },
    { 0x1a098fed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd698ff0, SUC_lexicon_tags_16 },
    { 0x6898ff1, SUC_lexicon_tags_23 },
    { 0xd3a98ff2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x24e98ff4, SUC_lexicon_tags_48 },
    { 0x92698ff4, SUC_lexicon_tags_37 },
    { 0xc8098ff4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c098ffb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc99004, SUC_lexicon_tags_136 },
    { 0xba099004, SUC_lexicon_tags_46 },
    { 0xe4499006, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf29900c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93899010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe299012, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e99014, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1209901d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f89901f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa299023, SUC_lexicon_tags_17 },
    { 0x7d299024, SUC_lexicon_tags_25 },
    { 0xe9c99023, SUC_lexicon_tags_161 },
    { 0xdd699024, SUC_lexicon_tags_22 },
    { 0xa099023, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc109902e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16099030, SUC_lexicon_tags_29 },
    { 0x8cc99031, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfde99033, SUC_lexicon_tags_1 },
    { 0x90299034, SUC_lexicon_tags_45 },
    { 0x62299034, SUC_lexicon_tags_14 },
    { 0xd5899034, SUC_lexicon_tags_15 },
    { 0x72099037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9903f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3499041, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede99048, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1849904c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf069904f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e99056, SUC_lexicon_tags_16 },
    { 0x3b899056, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47c99059, SUC_lexicon_tags_55 },
    { 0x8249905a, SUC_lexicon_tags_37 },
    { 0xb6e9905a, SUC_lexicon_tags_11 },
    { 0xf569905c, SUC_lexicon_tags_25 },
    { 0xfee9905d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9389905f, SUC_lexicon_tags_8 },
    { 0x7509905f, SUC_lexicon_tags_60 },
    { 0xa0699061, SUC_lexicon_tags_462 },
    { 0xe8099062, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96299066, SUC_lexicon_tags_31 },
    { 0x61e99067, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a9906c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x95e9906e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c099070, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xad299072, SUC_lexicon_tags_21 },
    { 0xccc99073, SUC_lexicon_tags_208 },
    { 0xfda99073, SUC_lexicon_tags_7 },
    { 0x7f299073, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaba99077, SUC_lexicon_tags_1 },
    { 0xb9c99077, SUC_lexicon_tags_16 },
    { 0xa8099079, SUC_lexicon_tags_27 },
    { 0x349907a, SUC_lexicon_tags_29 },
    { 0xe2c9907a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x52e9907e, SUC_lexicon_tags_29 },
    { 0xa7a9907e, SUC_lexicon_tags_1 },
    { 0x42a9907e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c499086, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2c99088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3789908a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c29908f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae99094, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbbe99096, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a9909c, SUC_lexicon_tags_7 },
    { 0x7c09909d, SUC_lexicon_tags_2 },
    { 0x63c9909c, SUC_lexicon_tags_49 },
    { 0xa549909f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb32990a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe990a6, SUC_lexicon_tags_85 },
    { 0x95c990a6, SUC_lexicon_tags_25 },
    { 0x56e990a8, SUC_lexicon_tags_53 },
    { 0x464990a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x122990ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x976990ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72990b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c990b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x434990ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2990bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5e8990bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc990c2, SUC_lexicon_tags_5 },
    { 0xf70990c3, SUC_lexicon_tags_30 },
    { 0xaa4990c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0990c9, SUC_lexicon_tags_17 },
    { 0x7aa990ca, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c990cd, SUC_lexicon_tags_60 },
    { 0x948990cd, SUC_lexicon_tags_14 },
    { 0xedc990ce, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4990d4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e990d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e6990d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4a990db, SUC_lexicon_tags_17 },
    { 0xa0990dc, SUC_lexicon_tags_47 },
    { 0xd8c990dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c990e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4990e4, SUC_lexicon_tags_1 },
    { 0x8ba990e5, SUC_lexicon_tags_7 },
    { 0x9c0990e6, SUC_lexicon_tags_11 },
    { 0x320990e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6990eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x626990ee, SUC_lexicon_tags_0 },
    { 0x72a990ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82990f7, SUC_lexicon_tags_63 },
    { 0xd90990f7, SUC_lexicon_tags_12 },
    { 0x1d0990f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x136990fb, SUC_lexicon_tags_42 },
    { 0xf0c990fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8990ff, SUC_lexicon_tags_37 },
    { 0xbe899100, SUC_lexicon_tags_33 },
    { 0x94c990ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc1e99103, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4aa99105, SUC_lexicon_tags_12 },
    { 0xefe99105, SUC_lexicon_tags_107 },
    { 0xa3299107, SUC_lexicon_tags_46 },
    { 0xeb899108, SUC_lexicon_tags_11 },
    { 0x16499109, SUC_lexicon_tags_11 },
    { 0xba09910a, SUC_lexicon_tags_9 },
    { 0x8d899108, SUC_lexicon_tags_11 },
    { 0x5e69910a, SUC_lexicon_tags_7 },
    { 0xf7699107, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabc9910f, SUC_lexicon_tags_37 },
    { 0xa4c9910f, SUC_lexicon_tags_37 },
    { 0xc4c99111, SUC_lexicon_tags_11 },
    { 0x55099111, SUC_lexicon_tags_17 },
    { 0xef099113, SUC_lexicon_tags_1 },
    { 0x73499113, SUC_lexicon_tags_26 },
    { 0xaa899113, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x13a99117, SUC_lexicon_tags_68 },
    { 0xab099118, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5349911b, SUC_lexicon_tags_7 },
    { 0xa249911c, SUC_lexicon_tags_17 },
    { 0x9c69911d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ae9911f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf499121, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21899129, SUC_lexicon_tags_66 },
    { 0x19a9912a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a099131, SUC_lexicon_tags_47 },
    { 0x75499132, SUC_lexicon_tags_9 },
    { 0x5cc99133, SUC_lexicon_tags_65 },
    { 0x2d499134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39e99136, SUC_lexicon_tags_17 },
    { 0x5e699137, SUC_lexicon_tags_57 },
    { 0x46499137, SUC_lexicon_tags_47 },
    { 0x62299138, SUC_lexicon_tags_33 },
    { 0xfcc9913a, SUC_lexicon_tags_1 },
    { 0xe589913b, SUC_lexicon_tags_603 },
    { 0x30c9913c, SUC_lexicon_tags_15 },
    { 0x72299139, SUC_lexicon_tags_27 },
    { 0x4c9913e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x48a99140, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16099142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c89914a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc9914e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5699153, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x32a99155, SUC_lexicon_tags_12 },
    { 0x10899156, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c9915b, SUC_lexicon_tags_12 },
    { 0x3489915c, SUC_lexicon_tags_11 },
    { 0x6169915c, SUC_lexicon_tags_9 },
    { 0xf769915e, SUC_lexicon_tags_11 },
    { 0xcc49915e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4699161, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be99164, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d299168, SUC_lexicon_tags_8 },
    { 0x81899169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb909916e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe099171, SUC_lexicon_tags_14 },
    { 0xd2a99172, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a99175, SUC_lexicon_tags_0 },
    { 0x3bc99176, SUC_lexicon_tags_26 },
    { 0x99c99177, SUC_lexicon_tags_15 },
    { 0x42499178, SUC_lexicon_tags_17 },
    { 0x3f299179, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc889917b, SUC_lexicon_tags_17 },
    { 0x6809917c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a99182, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d299184, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb699188, SUC_lexicon_tags_1 },
    { 0x1d099189, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529918e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de99191, SUC_lexicon_tags_129 },
    { 0xaa299192, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2899196, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8d699198, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc9919f, SUC_lexicon_tags_8 },
    { 0xfca991a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc0991a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50991a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde991a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32991aa, SUC_lexicon_tags_11 },
    { 0x270991ab, SUC_lexicon_tags_61 },
    { 0x37a991ac, SUC_lexicon_tags_12 },
    { 0x45c991ad, SUC_lexicon_tags_1 },
    { 0xc36991ac, SUC_lexicon_tags_33 },
    { 0xadc991aa, SUC_lexicon_tags_23 },
    { 0x562991ae, SUC_lexicon_tags_42 },
    { 0x89a991b1, SUC_lexicon_tags_47 },
    { 0x52991af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4991b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c8991b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88991be, SUC_lexicon_tags_25 },
    { 0xc24991be, SUC_lexicon_tags_15 },
    { 0xb42991bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bc991c2, SUC_lexicon_tags_37 },
    { 0x9ae991c3, SUC_lexicon_tags_15 },
    { 0xe22991c4, SUC_lexicon_tags_37 },
    { 0xba8991c5, SUC_lexicon_tags_7 },
    { 0x92e991c2, SUC_lexicon_tags_8 },
    { 0xaa991c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e991cb, SUC_lexicon_tags_31 },
    { 0xf4c991cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52991d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e991d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926991db, SUC_lexicon_tags_5 },
    { 0xc34991dc, SUC_lexicon_tags_23 },
    { 0x11a991dd, SUC_lexicon_tags_39 },
    { 0x19a991de, SUC_lexicon_tags_64 },
    { 0x46c991dc, SUC_lexicon_tags_8 },
    { 0xf2a991e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82991e3, SUC_lexicon_tags_1 },
    { 0x426991e3, SUC_lexicon_tags_37 },
    { 0xa72991e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa991e8, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c991ec, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x5d4991ee, SUC_lexicon_tags_2 },
    { 0xe66991ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10991f2, SUC_lexicon_tags_17 },
    { 0xa8a991f3, SUC_lexicon_tags_1 },
    { 0xb6991f4, SUC_lexicon_tags_33 },
    { 0x716991f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x154991f7, SUC_lexicon_tags_1 },
    { 0x32c991f8, SUC_lexicon_tags_23 },
    { 0x670991f8, SUC_lexicon_tags_66 },
    { 0xde2991fa, SUC_lexicon_tags_0 },
    { 0xea2991f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a2991fd, SUC_lexicon_tags_60 },
    { 0xa6c991fe, SUC_lexicon_tags_23 },
    { 0x162991ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x86a99201, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28299203, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36299205, SUC_lexicon_tags_16 },
    { 0x72499205, SUC_lexicon_tags_39 },
    { 0x13499206, SUC_lexicon_tags_35 },
    { 0xc7a99208, SUC_lexicon_tags_17 },
    { 0xbe099209, SUC_lexicon_tags_25 },
    { 0xeaa9920a, SUC_lexicon_tags_11 },
    { 0xffa99209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc9920e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8699215, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49921a, SUC_lexicon_tags_45 },
    { 0x9069921b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec99227, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a9922a, SUC_lexicon_tags_11 },
    { 0x1c49922b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8699231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31299233, SUC_lexicon_tags_60 },
    { 0x66e99234, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x50899236, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a9923d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c699241, SUC_lexicon_tags_31 },
    { 0xea499241, SUC_lexicon_tags_38 },
    { 0x7dc99243, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c99247, SUC_lexicon_tags_2 },
    { 0xae699247, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe29924b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4229924f, SUC_lexicon_tags_49 },
    { 0x52c9924f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d499256, SUC_lexicon_tags_25 },
    { 0x49699257, SUC_lexicon_tags_7 },
    { 0x54c99256, SUC_lexicon_tags_0 },
    { 0x42499259, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e9925c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc89925f, SUC_lexicon_tags_1 },
    { 0x93099260, SUC_lexicon_tags_2 },
    { 0x4dc99261, SUC_lexicon_tags_66 },
    { 0xe8899262, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5299265, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfec99267, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc769926a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcba9926d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c99274, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a9927b, SUC_lexicon_tags_9 },
    { 0xa29927b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea69927e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a99281, SUC_lexicon_tags_17 },
    { 0xc3299281, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e99287, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8489928d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbdc9928f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9929a, SUC_lexicon_tags_1 },
    { 0x37a9929b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf369929d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e992a3, SUC_lexicon_tags_27 },
    { 0x918992a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc992a9, SUC_lexicon_tags_24 },
    { 0x7f6992a9, SUC_lexicon_tags_12 },
    { 0x582992ab, SUC_lexicon_tags_2 },
    { 0x4b0992ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6992b0, SUC_lexicon_tags_25 },
    { 0x1c0992b1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2992b5, SUC_lexicon_tags_11 },
    { 0x9a2992b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcee992b8, SUC_lexicon_tags_47 },
    { 0x6ae992b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbc992bb, SUC_lexicon_tags_14 },
    { 0x6f6992bc, SUC_lexicon_tags_12 },
    { 0xc5a992bd, SUC_lexicon_tags_17 },
    { 0x7da992be, SUC_lexicon_tags_17 },
    { 0xde0992bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390992c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378992ca, SUC_lexicon_tags_2 },
    { 0x3c992cb, SUC_lexicon_tags_42 },
    { 0xaf8992ca, SUC_lexicon_tags_89 },
    { 0x28e992cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x758992cf, SUC_lexicon_tags_5 },
    { 0x24c992d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a992d4, SUC_lexicon_tags_31 },
    { 0x732992d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672992d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72992e4, SUC_lexicon_tags_2 },
    { 0x91c992e5, SUC_lexicon_tags_26 },
    { 0x262992e5, SUC_lexicon_tags_13 },
    { 0xde6992e7, SUC_lexicon_tags_25 },
    { 0x9e992e7, SUC_lexicon_tags_38 },
    { 0x428992e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0992f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x508992f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08992f9, SUC_lexicon_tags_25 },
    { 0x1d8992fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868992fe, SUC_lexicon_tags_17 },
    { 0x984992ff, SUC_lexicon_tags_45 },
    { 0x47499300, SUC_lexicon_tags_17 },
    { 0xcd2992fe, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x92899304, SUC_lexicon_tags_37 },
    { 0xa9c99305, SUC_lexicon_tags_46 },
    { 0xfba99306, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x27899308, SUC_lexicon_tags_37 },
    { 0xc2e99309, SUC_lexicon_tags_37 },
    { 0x1569930a, SUC_lexicon_tags_29 },
    { 0x94c99308, SUC_lexicon_tags_12 },
    { 0xcf89930c, SUC_lexicon_tags_49 },
    { 0xc129930c, SUC_lexicon_tags_7 },
    { 0x69c9930e, SUC_lexicon_tags_2 },
    { 0xf969930e, SUC_lexicon_tags_25 },
    { 0xdee99310, SUC_lexicon_tags_46 },
    { 0x3409930d, SUC_lexicon_tags_11 },
    { 0x43299312, SUC_lexicon_tags_57 },
    { 0x51699313, SUC_lexicon_tags_43 },
    { 0xb5699313, SUC_lexicon_tags_7 },
    { 0x7189930b, SUC_lexicon_tags_57 },
    { 0xe1c99316, SUC_lexicon_tags_68 },
    { 0x6d299316, SUC_lexicon_tags_17 },
    { 0x31099316, SUC_lexicon_tags_68 },
    { 0xa509930f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b49931f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b499322, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1699326, SUC_lexicon_tags_19 },
    { 0xb0699327, SUC_lexicon_tags_0 },
    { 0x16899326, SUC_lexicon_tags_7 },
    { 0xe6499329, SUC_lexicon_tags_65 },
    { 0x7a499329, SUC_lexicon_tags_46 },
    { 0xc5299326, SUC_lexicon_tags_43 },
    { 0x29e9932a, SUC_lexicon_tags_15 },
    { 0xacc99326, SUC_lexicon_tags_15 },
    { 0xfba9932a, SUC_lexicon_tags_33 },
    { 0x1329932f, SUC_lexicon_tags_27 },
    { 0xa3c9932b, SUC_lexicon_tags_17 },
    { 0xa5c99331, SUC_lexicon_tags_27 },
    { 0x2899328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac99336, SUC_lexicon_tags_19 },
    { 0x1c299336, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9933b, SUC_lexicon_tags_12 },
    { 0x3a29933c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43099341, SUC_lexicon_tags_1 },
    { 0xe299341, SUC_lexicon_tags_15 },
    { 0x18e99342, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x66099346, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e299348, SUC_lexicon_tags_33 },
    { 0xf3699348, SUC_lexicon_tags_46 },
    { 0x6aa9934a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd699350, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc99356, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c49935b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d89935e, SUC_lexicon_tags_37 },
    { 0x7ce9935e, SUC_lexicon_tags_38 },
    { 0x6969935e, SUC_lexicon_tags_62 },
    { 0xc0699361, SUC_lexicon_tags_33 },
    { 0xda499362, SUC_lexicon_tags_1 },
    { 0xb4e99363, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c99366, SUC_lexicon_tags_62 },
    { 0xe99366, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b69936b, SUC_lexicon_tags_37 },
    { 0x2b49936c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7149936e, SUC_lexicon_tags_7 },
    { 0xcda9936f, SUC_lexicon_tags_27 },
    { 0xb2699370, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85299376, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8a99378, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e9937b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7299380, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a99385, SUC_lexicon_tags_23 },
    { 0xa0099385, SUC_lexicon_tags_1 },
    { 0x4b499385, SUC_lexicon_tags_37 },
    { 0xc5699388, SUC_lexicon_tags_14 },
    { 0xa1499388, SUC_lexicon_tags_63 },
    { 0xc69938a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f69938c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42099391, SUC_lexicon_tags_0 },
    { 0xdcc99392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7c99394, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcae99396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1129939a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b49939c, SUC_lexicon_tags_12 },
    { 0xafe9939c, SUC_lexicon_tags_68 },
    { 0xd949939e, SUC_lexicon_tags_11 },
    { 0xf2c9939d, SUC_lexicon_tags_2 },
    { 0xa5a993a0, SUC_lexicon_tags_45 },
    { 0xd96993a1, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8993a7, SUC_lexicon_tags_2 },
    { 0x986993a8, SUC_lexicon_tags_25 },
    { 0xb10993a9, SUC_lexicon_tags_15 },
    { 0xe80993a9, SUC_lexicon_tags_20 },
    { 0x8c993ab, SUC_lexicon_tags_21 },
    { 0x78a993ac, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdd0993ae, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648993b2, SUC_lexicon_tags_37 },
    { 0x154993b3, SUC_lexicon_tags_29 },
    { 0x316993b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60993b7, SUC_lexicon_tags_565 },
    { 0xf7c993b8, SUC_lexicon_tags_37 },
    { 0x4c0993b8, SUC_lexicon_tags_0 },
    { 0x392993ba, SUC_lexicon_tags_6 },
    { 0xa9c993bb, SUC_lexicon_tags_17 },
    { 0xb56993bc, SUC_lexicon_tags_47 },
    { 0x404993bc, SUC_lexicon_tags_17 },
    { 0xa40993bc, SUC_lexicon_tags_7 },
    { 0xe4993bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7c993c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a993c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x342993c8, SUC_lexicon_tags_37 },
    { 0x732993c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae993cc, SUC_lexicon_tags_11 },
    { 0xf8c993cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2993d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x94a993d5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2993da, SUC_lexicon_tags_17 },
    { 0xd1c993db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea993de, SUC_lexicon_tags_12 },
    { 0xbc4993df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x426993e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8993e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x596993e8, SUC_lexicon_tags_29 },
    { 0x72e993e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980993f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x35e993f7, SUC_lexicon_tags_11 },
    { 0xb18993f8, SUC_lexicon_tags_37 },
    { 0x7c993f9, SUC_lexicon_tags_25 },
    { 0x9ce993fa, SUC_lexicon_tags_1 },
    { 0xac8993f8, SUC_lexicon_tags_27 },
    { 0xd90993f8, SUC_lexicon_tags_8 },
    { 0xf06993fd, SUC_lexicon_tags_3 },
    { 0xc38993fe, SUC_lexicon_tags_28 },
    { 0xafe993ff, SUC_lexicon_tags_33 },
    { 0xbbc993ff, SUC_lexicon_tags_17 },
    { 0x13899401, SUC_lexicon_tags_63 },
    { 0x218993fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2a99407, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a9940a, SUC_lexicon_tags_7 },
    { 0x86a9940a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1649940d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b09940f, SUC_lexicon_tags_37 },
    { 0x94099410, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbc099412, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa99416, SUC_lexicon_tags_23 },
    { 0xb6899416, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc9941a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cc9941c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca99423, SUC_lexicon_tags_68 },
    { 0x1a299424, SUC_lexicon_tags_11 },
    { 0x3c99424, SUC_lexicon_tags_11 },
    { 0xd1699426, SUC_lexicon_tags_25 },
    { 0xd6099424, SUC_lexicon_tags_25 },
    { 0xcce99425, SUC_lexicon_tags_29 },
    { 0xa0699429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97e9942b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a499435, SUC_lexicon_tags_12 },
    { 0x59e99436, SUC_lexicon_tags_0 },
    { 0xf4a99437, SUC_lexicon_tags_12 },
    { 0xb3a99437, SUC_lexicon_tags_31 },
    { 0xb8099439, SUC_lexicon_tags_30 },
    { 0xcd899439, SUC_lexicon_tags_1 },
    { 0x5e69943b, SUC_lexicon_tags_14 },
    { 0x50c9943b, SUC_lexicon_tags_9 },
    { 0x5b299439, SUC_lexicon_tags_37 },
    { 0xd909943e, SUC_lexicon_tags_25 },
    { 0x3049943b, SUC_lexicon_tags_1 },
    { 0x2929943f, SUC_lexicon_tags_11 },
    { 0x21899440, SUC_lexicon_tags_12 },
    { 0xeb099440, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8499446, SUC_lexicon_tags_2 },
    { 0x40a99447, SUC_lexicon_tags_37 },
    { 0x33099448, SUC_lexicon_tags_13 },
    { 0x44899449, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac9944c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ee9944e, SUC_lexicon_tags_49 },
    { 0x59a9944f, SUC_lexicon_tags_25 },
    { 0x1d49944e, SUC_lexicon_tags_43 },
    { 0x7769944e, SUC_lexicon_tags_38 },
    { 0xe6499451, SUC_lexicon_tags_47 },
    { 0x55a99453, SUC_lexicon_tags_7 },
    { 0xe2899454, SUC_lexicon_tags_1 },
    { 0xdc099454, SUC_lexicon_tags_11 },
    { 0x97099456, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a9945c, SUC_lexicon_tags_2 },
    { 0xee9945c, SUC_lexicon_tags_12 },
    { 0xf129945e, SUC_lexicon_tags_15 },
    { 0xc209945f, SUC_lexicon_tags_25 },
    { 0xdf499460, SUC_lexicon_tags_8 },
    { 0xfe099461, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87a99465, SUC_lexicon_tags_11 },
    { 0xf9499466, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb589946c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa69946e, SUC_lexicon_tags_21 },
    { 0x1629946f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea99472, SUC_lexicon_tags_417 },
    { 0xfaa99472, SUC_lexicon_tags_56 },
    { 0x5f499472, SUC_lexicon_tags_38 },
    { 0x4ce99473, SUC_lexicon_tags_25 },
    { 0xac99473, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe849947f, SUC_lexicon_tags_23 },
    { 0xc9e99480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d099483, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x62c99485, SUC_lexicon_tags_25 },
    { 0x40099486, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x71299488, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc699496, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc9949e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x292994a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x340994a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ea994a4, SUC_lexicon_tags_4 },
    { 0x4e0994a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c994ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac994b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a994b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d2994b6, SUC_lexicon_tags_7 },
    { 0x5f4994b7, SUC_lexicon_tags_7 },
    { 0x30a994b8, SUC_lexicon_tags_37 },
    { 0x606994b9, SUC_lexicon_tags_10 },
    { 0x7d6994ba, SUC_lexicon_tags_2 },
    { 0xb64994bb, SUC_lexicon_tags_60 },
    { 0x962994bb, SUC_lexicon_tags_25 },
    { 0xa92994bb, SUC_lexicon_tags_8 },
    { 0x202994be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20a994c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e994c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb70994c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec994c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x366994cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65e994ce, SUC_lexicon_tags_63 },
    { 0xbe6994cf, SUC_lexicon_tags_0 },
    { 0x1c4994ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4994da, SUC_lexicon_tags_14 },
    { 0x9c994da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2994e1, SUC_lexicon_tags_17 },
    { 0x68e994e1, SUC_lexicon_tags_37 },
    { 0x9de994e3, SUC_lexicon_tags_15 },
    { 0xc54994e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92994e7, SUC_lexicon_tags_37 },
    { 0x5c8994e7, SUC_lexicon_tags_25 },
    { 0x4c4994e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98e994eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2e994ed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c994f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ee994f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9c8994f5, SUC_lexicon_tags_17 },
    { 0x842994f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x526994f8, SUC_lexicon_tags_15 },
    { 0xca6994f9, SUC_lexicon_tags_23 },
    { 0xc58994fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8994fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6994ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55499505, SUC_lexicon_tags_46 },
    { 0xc6099506, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6699509, SUC_lexicon_tags_26 },
    { 0x2e299509, SUC_lexicon_tags_49 },
    { 0x28a9950b, SUC_lexicon_tags_63 },
    { 0xe029950b, SUC_lexicon_tags_37 },
    { 0x1f89950d, SUC_lexicon_tags_27 },
    { 0x4d69950e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4299512, SUC_lexicon_tags_8 },
    { 0xdfa99513, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a99518, SUC_lexicon_tags_14 },
    { 0x5f499518, SUC_lexicon_tags_31 },
    { 0xc1899518, SUC_lexicon_tags_25 },
    { 0x8d69951a, SUC_lexicon_tags_29 },
    { 0x9d69951c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22499525, SUC_lexicon_tags_38 },
    { 0x2de99526, SUC_lexicon_tags_8 },
    { 0x6c699527, SUC_lexicon_tags_12 },
    { 0xeb299528, SUC_lexicon_tags_42 },
    { 0x5499528, SUC_lexicon_tags_24 },
    { 0xf7e9952a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c9952f, SUC_lexicon_tags_63 },
    { 0xc6299530, SUC_lexicon_tags_30 },
    { 0x1d099531, SUC_lexicon_tags_25 },
    { 0x74499530, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1c099534, SUC_lexicon_tags_25 },
    { 0x3ea99535, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x67e99537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb149953a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b69953e, SUC_lexicon_tags_12 },
    { 0xb49953f, SUC_lexicon_tags_7 },
    { 0x86099540, SUC_lexicon_tags_4 },
    { 0x33c99541, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1899546, SUC_lexicon_tags_1 },
    { 0xd7c99547, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1129954b, SUC_lexicon_tags_10 },
    { 0x3249954c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80a9954e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e499555, SUC_lexicon_tags_8 },
    { 0x4de99555, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a699559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1869955f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c99563, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a99566, SUC_lexicon_tags_7 },
    { 0x1299567, SUC_lexicon_tags_25 },
    { 0xc4499568, SUC_lexicon_tags_7 },
    { 0xbe99567, SUC_lexicon_tags_7 },
    { 0x6409956a, SUC_lexicon_tags_3 },
    { 0x32299568, SUC_lexicon_tags_60 },
    { 0x31e99567, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc509956f, SUC_lexicon_tags_25 },
    { 0x14c9956f, SUC_lexicon_tags_37 },
    { 0xc3a9956f, SUC_lexicon_tags_25 },
    { 0xfe099571, SUC_lexicon_tags_37 },
    { 0x90c99571, SUC_lexicon_tags_8 },
    { 0x62299574, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3aa99576, SUC_lexicon_tags_12 },
    { 0x5a299577, SUC_lexicon_tags_38 },
    { 0x45c99578, SUC_lexicon_tags_25 },
    { 0xb9899579, SUC_lexicon_tags_29 },
    { 0xb7c99578, SUC_lexicon_tags_29 },
    { 0x3429957b, SUC_lexicon_tags_43 },
    { 0xe8499578, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c99581, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x14299583, SUC_lexicon_tags_14 },
    { 0x7c299584, SUC_lexicon_tags_16 },
    { 0x89e99584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba09958c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e99592, SUC_lexicon_tags_15 },
    { 0x71c99592, SUC_lexicon_tags_12 },
    { 0x11a99594, SUC_lexicon_tags_1 },
    { 0x10e99595, SUC_lexicon_tags_25 },
    { 0x82899595, SUC_lexicon_tags_7 },
    { 0xa1699597, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb29959c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc995a3, SUC_lexicon_tags_178 },
    { 0x61c995a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e995a7, SUC_lexicon_tags_49 },
    { 0x744995a7, SUC_lexicon_tags_25 },
    { 0xcc8995a9, SUC_lexicon_tags_37 },
    { 0x27a995aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840995b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2995b3, SUC_lexicon_tags_7 },
    { 0x6dc995b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a995b7, SUC_lexicon_tags_6 },
    { 0xf00995b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xba995bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c995c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922995c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x718995c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1de995cf, SUC_lexicon_tags_7 },
    { 0x8c995d0, SUC_lexicon_tags_2 },
    { 0x642995cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90995d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e995d7, SUC_lexicon_tags_25 },
    { 0x876995d8, SUC_lexicon_tags_37 },
    { 0x910995d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8995df, SUC_lexicon_tags_3 },
    { 0xae2995e0, SUC_lexicon_tags_32 },
    { 0x636995e1, SUC_lexicon_tags_152 },
    { 0xeba995e2, SUC_lexicon_tags_37 },
    { 0xeb0995e3, SUC_lexicon_tags_1 },
    { 0xa0a995e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa56995e6, SUC_lexicon_tags_17 },
    { 0x6b6995e7, SUC_lexicon_tags_27 },
    { 0x592995e8, SUC_lexicon_tags_15 },
    { 0xc52995e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x648995ec, SUC_lexicon_tags_94 },
    { 0x89e995ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x56995f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x920995f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe4995f4, SUC_lexicon_tags_46 },
    { 0x97e995f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa995f8, SUC_lexicon_tags_1 },
    { 0x580995f9, SUC_lexicon_tags_25 },
    { 0x8e2995f9, SUC_lexicon_tags_49 },
    { 0x6d2995fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256995ff, SUC_lexicon_tags_12 },
    { 0x308995ff, SUC_lexicon_tags_14 },
    { 0x98c99601, SUC_lexicon_tags_15 },
    { 0xce299601, SUC_lexicon_tags_17 },
    { 0x93499601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cc99605, SUC_lexicon_tags_23 },
    { 0x6fa99606, SUC_lexicon_tags_21 },
    { 0x71299607, SUC_lexicon_tags_13 },
    { 0xab499607, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6409960c, SUC_lexicon_tags_55 },
    { 0xc829960c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc09960f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace99614, SUC_lexicon_tags_11 },
    { 0x4cc99614, SUC_lexicon_tags_11 },
    { 0x7c499615, SUC_lexicon_tags_46 },
    { 0x9ac99617, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6099619, SUC_lexicon_tags_19 },
    { 0x1849961a, SUC_lexicon_tags_15 },
    { 0x64e9961b, SUC_lexicon_tags_39 },
    { 0xddc9961c, SUC_lexicon_tags_7 },
    { 0x6ba9961a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c99621, SUC_lexicon_tags_87 },
    { 0x32299622, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e9962d, SUC_lexicon_tags_2 },
    { 0x7fe9962d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc299631, SUC_lexicon_tags_48 },
    { 0x10299632, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c299635, SUC_lexicon_tags_0 },
    { 0x21a99636, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43699639, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1769963f, SUC_lexicon_tags_55 },
    { 0xa8699640, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f699643, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd899647, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11499649, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec9964d, SUC_lexicon_tags_11 },
    { 0x4229964d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88299652, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ea99654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbbc99656, SUC_lexicon_tags_37 },
    { 0x50099656, SUC_lexicon_tags_37 },
    { 0xee299656, SUC_lexicon_tags_11 },
    { 0x40099659, SUC_lexicon_tags_47 },
    { 0x4d899659, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x2a49965c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79499661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e99665, SUC_lexicon_tags_0 },
    { 0x4e299666, SUC_lexicon_tags_7 },
    { 0x51899666, SUC_lexicon_tags_69 },
    { 0x70499668, SUC_lexicon_tags_26 },
    { 0x84299669, SUC_lexicon_tags_31 },
    { 0xc8499668, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37e9966d, SUC_lexicon_tags_17 },
    { 0xab29966e, SUC_lexicon_tags_17 },
    { 0x9bc9966e, SUC_lexicon_tags_13 },
    { 0xd0099670, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc99675, SUC_lexicon_tags_15 },
    { 0xe9e99675, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e99679, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xab29967c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7a9967e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82c99681, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc99684, SUC_lexicon_tags_8 },
    { 0x62699684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a9968a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf209968c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e899690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c899693, SUC_lexicon_tags_7 },
    { 0x4e99694, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a9969b, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0xf749969e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5a996a0, SUC_lexicon_tags_12 },
    { 0x64996a0, SUC_lexicon_tags_12 },
    { 0xda0996a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2b4996a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc2e996a6, SUC_lexicon_tags_63 },
    { 0xd30996a7, SUC_lexicon_tags_1 },
    { 0x11a996a7, SUC_lexicon_tags_15 },
    { 0xcc8996a8, SUC_lexicon_tags_37 },
    { 0x766996aa, SUC_lexicon_tags_17 },
    { 0x572996ab, SUC_lexicon_tags_43 },
    { 0x310996aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b2996ae, SUC_lexicon_tags_108 },
    { 0x690996af, SUC_lexicon_tags_63 },
    { 0xae996af, SUC_lexicon_tags_1 },
    { 0xe3e996ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588996bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c996bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72996c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb84996cb, SUC_lexicon_tags_15 },
    { 0x21a996cc, SUC_lexicon_tags_8 },
    { 0x1f2996cd, SUC_lexicon_tags_10 },
    { 0xe88996ce, SUC_lexicon_tags_48 },
    { 0xafe996cf, SUC_lexicon_tags_116 },
    { 0xf42996ce, SUC_lexicon_tags_1 },
    { 0xca4996ce, SUC_lexicon_tags_26 },
    { 0x23a996ce, SUC_lexicon_tags_37 },
    { 0xe24996cd, SUC_lexicon_tags_15 },
    { 0xe42996cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62996d9, SUC_lexicon_tags_46 },
    { 0x672996d9, SUC_lexicon_tags_11 },
    { 0xc5e996db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x152996de, SUC_lexicon_tags_60 },
    { 0x69a996df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c996e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc996eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb7c996ed, SUC_lexicon_tags_17 },
    { 0xff0996ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd5a996f0, SUC_lexicon_tags_37 },
    { 0x8996f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf6a996f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2996f7, SUC_lexicon_tags_55 },
    { 0xcce996f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32996fd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda99701, SUC_lexicon_tags_66 },
    { 0x10c99702, SUC_lexicon_tags_37 },
    { 0xd8a99702, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e99707, SUC_lexicon_tags_38 },
    { 0x66c99707, SUC_lexicon_tags_12 },
    { 0x38299708, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xef09970c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf49970e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1899714, SUC_lexicon_tags_23 },
    { 0xa2299715, SUC_lexicon_tags_127 },
    { 0xe2299716, SUC_lexicon_tags_37 },
    { 0xdd499717, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c99719, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6fa9971b, SUC_lexicon_tags_6 },
    { 0x3c9971c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68299724, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0899726, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7029972e, SUC_lexicon_tags_15 },
    { 0xd789972f, SUC_lexicon_tags_19 },
    { 0xcb09972f, SUC_lexicon_tags_0 },
    { 0xd3699731, SUC_lexicon_tags_63 },
    { 0x47c99732, SUC_lexicon_tags_25 },
    { 0x55c99732, SUC_lexicon_tags_45 },
    { 0x59299732, SUC_lexicon_tags_15 },
    { 0x7989972f, SUC_lexicon_tags_7 },
    { 0x2ba9972f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e9973a, SUC_lexicon_tags_65 },
    { 0x93e9973b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1569973d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1469973f, SUC_lexicon_tags_37 },
    { 0xa849973f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4299744, SUC_lexicon_tags_19 },
    { 0x1499745, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xda99747, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d89974d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a99751, SUC_lexicon_tags_21 },
    { 0xd6099751, SUC_lexicon_tags_2 },
    { 0x34c99753, SUC_lexicon_tags_1 },
    { 0x23899754, SUC_lexicon_tags_31 },
    { 0xa2699753, SUC_lexicon_tags_9 },
    { 0x9c099756, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d299758, SUC_lexicon_tags_45 },
    { 0xfec99759, SUC_lexicon_tags_30 },
    { 0x3489975a, SUC_lexicon_tags_16 },
    { 0x51e9975b, SUC_lexicon_tags_17 },
    { 0xdc99759, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa889975f, SUC_lexicon_tags_7 },
    { 0xfae9975f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1e99762, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1699765, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97899769, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4a9976b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5699770, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcf299772, SUC_lexicon_tags_0 },
    { 0x1aa99772, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81499775, SUC_lexicon_tags_25 },
    { 0xa0699776, SUC_lexicon_tags_12 },
    { 0x50499776, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4249977c, SUC_lexicon_tags_14 },
    { 0x1649977d, SUC_lexicon_tags_35 },
    { 0x9da9977d, SUC_lexicon_tags_48 },
    { 0x2749977d, SUC_lexicon_tags_23 },
    { 0x27e99780, SUC_lexicon_tags_63 },
    { 0x4a499781, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a9978a, SUC_lexicon_tags_49 },
    { 0x87c9978b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c99793, SUC_lexicon_tags_11 },
    { 0xc5899794, SUC_lexicon_tags_33 },
    { 0x72699794, SUC_lexicon_tags_19 },
    { 0x7899796, SUC_lexicon_tags_37 },
    { 0xf0099794, SUC_lexicon_tags_37 },
    { 0xd299797, SUC_lexicon_tags_47 },
    { 0x65899799, SUC_lexicon_tags_14 },
    { 0xf2c9979a, SUC_lexicon_tags_1 },
    { 0xc49979b, SUC_lexicon_tags_16 },
    { 0xaf09979c, SUC_lexicon_tags_11 },
    { 0xbbc9979d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8997a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x510997aa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e997b1, SUC_lexicon_tags_0 },
    { 0xec0997b2, SUC_lexicon_tags_14 },
    { 0xd2e997b3, SUC_lexicon_tags_15 },
    { 0xb00997b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34997b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06997be, SUC_lexicon_tags_11 },
    { 0xd92997bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62997c6, SUC_lexicon_tags_2 },
    { 0xcca997c7, SUC_lexicon_tags_21 },
    { 0x4cc997c8, SUC_lexicon_tags_7 },
    { 0xa54997c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x430997cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x398997ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x606997d1, SUC_lexicon_tags_29 },
    { 0xa34997d2, SUC_lexicon_tags_1 },
    { 0xa40997d3, SUC_lexicon_tags_1 },
    { 0x266997d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924997d8, SUC_lexicon_tags_25 },
    { 0xbca997d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0997dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e997df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1ce997e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2997e5, SUC_lexicon_tags_49 },
    { 0xe0e997e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x862997e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e997eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70a997ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf00997ef, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x594997f1, SUC_lexicon_tags_45 },
    { 0x866997f1, SUC_lexicon_tags_10 },
    { 0xe2a997f2, SUC_lexicon_tags_79 },
    { 0x6ac997f4, SUC_lexicon_tags_11 },
    { 0x9d8997f2, SUC_lexicon_tags_37 },
    { 0xc88997f4, SUC_lexicon_tags_22 },
    { 0xbb8997f3, SUC_lexicon_tags_11 },
    { 0x750997f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x260997fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4c997fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab499801, SUC_lexicon_tags_12 },
    { 0xbc099801, SUC_lexicon_tags_31 },
    { 0xc5299801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de99806, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1da99808, SUC_lexicon_tags_45 },
    { 0x80a99809, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2649980b, SUC_lexicon_tags_38 },
    { 0xb009980b, SUC_lexicon_tags_25 },
    { 0x1369980c, SUC_lexicon_tags_17 },
    { 0xf569980e, SUC_lexicon_tags_25 },
    { 0xc6c9980f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e99812, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f69981a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff09981e, SUC_lexicon_tags_30 },
    { 0xcf69981f, SUC_lexicon_tags_15 },
    { 0x3f69981f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb9c99822, SUC_lexicon_tags_29 },
    { 0x9dc99822, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45099829, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1149982c, SUC_lexicon_tags_39 },
    { 0xd3a9982c, SUC_lexicon_tags_47 },
    { 0xda49982e, SUC_lexicon_tags_49 },
    { 0xc6e9982e, SUC_lexicon_tags_11 },
    { 0xdc699830, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x42499833, SUC_lexicon_tags_10 },
    { 0x73899834, SUC_lexicon_tags_6 },
    { 0xf9499835, SUC_lexicon_tags_7 },
    { 0x10099836, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e899839, SUC_lexicon_tags_8 },
    { 0x3469983a, SUC_lexicon_tags_37 },
    { 0xef499839, SUC_lexicon_tags_100 },
    { 0xe1c9983c, SUC_lexicon_tags_768 },
    { 0x8e49983d, SUC_lexicon_tags_15 },
    { 0x1189983e, SUC_lexicon_tags_68 },
    { 0xc8a9983f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0499841, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e99844, SUC_lexicon_tags_16 },
    { 0xda699845, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c99848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a99850, SUC_lexicon_tags_43 },
    { 0x90099850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36e99853, SUC_lexicon_tags_46 },
    { 0x18e99853, SUC_lexicon_tags_61 },
    { 0x84e99854, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e9985d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16a9985f, SUC_lexicon_tags_47 },
    { 0x2569985f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a99864, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4899867, SUC_lexicon_tags_12 },
    { 0x10a99868, SUC_lexicon_tags_15 },
    { 0x9ea99868, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe9986d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70699871, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaf699873, SUC_lexicon_tags_29 },
    { 0xe4299874, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a99878, SUC_lexicon_tags_15 },
    { 0xab299878, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e9987e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa99882, SUC_lexicon_tags_37 },
    { 0x94c99883, SUC_lexicon_tags_17 },
    { 0x9f899883, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a99888, SUC_lexicon_tags_25 },
    { 0xa099888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb509988b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b49988e, SUC_lexicon_tags_12 },
    { 0x9769988e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99899892, SUC_lexicon_tags_26 },
    { 0xb5699893, SUC_lexicon_tags_37 },
    { 0xfd899894, SUC_lexicon_tags_47 },
    { 0x74c99893, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa689989d, SUC_lexicon_tags_11 },
    { 0x1ae9989e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26998a1, SUC_lexicon_tags_12 },
    { 0x22a998a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec998a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64e998aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990998af, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa66998b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf38998b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912998b7, SUC_lexicon_tags_21 },
    { 0xeae998b7, SUC_lexicon_tags_11 },
    { 0xf84998b7, SUC_lexicon_tags_2 },
    { 0x3f6998ba, SUC_lexicon_tags_17 },
    { 0x70998ba, SUC_lexicon_tags_160 },
    { 0x318998ba, SUC_lexicon_tags_141 },
    { 0x16998b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e998c5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4998ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e998d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdb6998d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x914998d5, SUC_lexicon_tags_23 },
    { 0xfc0998d6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x272998d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48998dc, SUC_lexicon_tags_27 },
    { 0xd20998dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e998e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd06998e2, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34998e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6998f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400998fd, SUC_lexicon_tags_38 },
    { 0x6c6998fd, SUC_lexicon_tags_25 },
    { 0x7d0998ff, SUC_lexicon_tags_12 },
    { 0x94a99900, SUC_lexicon_tags_2 },
    { 0xdaa99901, SUC_lexicon_tags_37 },
    { 0x77a99902, SUC_lexicon_tags_12 },
    { 0x1fe99903, SUC_lexicon_tags_43 },
    { 0x3cc99904, SUC_lexicon_tags_34 },
    { 0x76499905, SUC_lexicon_tags_8 },
    { 0xcd899903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e9990b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a89990f, SUC_lexicon_tags_18 },
    { 0xfca9990f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e499913, SUC_lexicon_tags_39 },
    { 0x6e99914, SUC_lexicon_tags_17 },
    { 0x9d899914, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcce99917, SUC_lexicon_tags_7 },
    { 0x3d299917, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6809991d, SUC_lexicon_tags_65 },
    { 0xa249991e, SUC_lexicon_tags_43 },
    { 0x9f09991f, SUC_lexicon_tags_48 },
    { 0x3169991d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1899922, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c99925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e99928, SUC_lexicon_tags_26 },
    { 0xc7899929, SUC_lexicon_tags_12 },
    { 0x68e99928, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1749992d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5099931, SUC_lexicon_tags_17 },
    { 0x65899931, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e99936, SUC_lexicon_tags_33 },
    { 0x6f699937, SUC_lexicon_tags_77 },
    { 0x87299938, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a9993d, SUC_lexicon_tags_37 },
    { 0xcd69993e, SUC_lexicon_tags_23 },
    { 0x2449993d, SUC_lexicon_tags_37 },
    { 0x8da99940, SUC_lexicon_tags_2 },
    { 0xe5e99941, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xcd099943, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc99946, SUC_lexicon_tags_28 },
    { 0xdf099946, SUC_lexicon_tags_19 },
    { 0xc6c99947, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69c9994a, SUC_lexicon_tags_59 },
    { 0x1909994b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f69994e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ac99950, SUC_lexicon_tags_23 },
    { 0x66499951, SUC_lexicon_tags_7 },
    { 0x87c99952, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea299955, SUC_lexicon_tags_7 },
    { 0x56c99955, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7129995b, SUC_lexicon_tags_12 },
    { 0xc729995c, SUC_lexicon_tags_31 },
    { 0xb729995c, SUC_lexicon_tags_57 },
    { 0xa3a9995b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63699964, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a9996b, SUC_lexicon_tags_0 },
    { 0x6f09996c, SUC_lexicon_tags_87 },
    { 0x51e9996c, SUC_lexicon_tags_12 },
    { 0x66c9996e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c99971, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd1899973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65499977, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x65699979, SUC_lexicon_tags_29 },
    { 0xbc69997a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdd29997c, SUC_lexicon_tags_31 },
    { 0x949997c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb299988, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9009998a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4949998e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9609999d, SUC_lexicon_tags_65 },
    { 0x26c9999e, SUC_lexicon_tags_2 },
    { 0x4229999e, SUC_lexicon_tags_47 },
    { 0x252999a0, SUC_lexicon_tags_31 },
    { 0x532999a1, SUC_lexicon_tags_15 },
    { 0xdfc999a1, SUC_lexicon_tags_6 },
    { 0xe3c999a3, SUC_lexicon_tags_66 },
    { 0xe22999a4, SUC_lexicon_tags_38 },
    { 0x5de999a1, SUC_lexicon_tags_15 },
    { 0xd32999a6, SUC_lexicon_tags_23 },
    { 0x804999a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac999aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x796999ac, SUC_lexicon_tags_55 },
    { 0x982999ac, SUC_lexicon_tags_10 },
    { 0x104999ad, SUC_lexicon_tags_23 },
    { 0x296999af, SUC_lexicon_tags_17 },
    { 0xd86999ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x814999b3, SUC_lexicon_tags_14 },
    { 0x4e2999b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c999b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x694999ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3de999bc, SUC_lexicon_tags_0 },
    { 0x56999bd, SUC_lexicon_tags_29 },
    { 0x3a999bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa999c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c999c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4999c8, SUC_lexicon_tags_37 },
    { 0x3ae999c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec999cc, SUC_lexicon_tags_12 },
    { 0xd5a999cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd88999cf, SUC_lexicon_tags_37 },
    { 0x60e999d0, SUC_lexicon_tags_4 },
    { 0x20e999d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24999d5, SUC_lexicon_tags_37 },
    { 0xdbe999d6, SUC_lexicon_tags_31 },
    { 0x6ac999d6, SUC_lexicon_tags_9 },
    { 0x2ec999d6, SUC_lexicon_tags_60 },
    { 0x584999d7, SUC_lexicon_tags_45 },
    { 0x278999da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850999de, SUC_lexicon_tags_10 },
    { 0xb72999df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x484999e1, SUC_lexicon_tags_17 },
    { 0x8a8999e1, SUC_lexicon_tags_12 },
    { 0x3ee999e3, SUC_lexicon_tags_61 },
    { 0x564999e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdce999e8, SUC_lexicon_tags_10 },
    { 0x50a999e9, SUC_lexicon_tags_12 },
    { 0x70c999ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a999f0, SUC_lexicon_tags_39 },
    { 0xa4a999f1, SUC_lexicon_tags_1 },
    { 0x9cc999f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54999f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe999fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62099a01, SUC_lexicon_tags_17 },
    { 0x96e99a01, SUC_lexicon_tags_88 },
    { 0x1a299a03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc99a0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56499a0c, SUC_lexicon_tags_25 },
    { 0xbc99a0d, SUC_lexicon_tags_11 },
    { 0x56c99a0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd499a1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6e99a20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99899a25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60c99a28, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4a499a2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a99a2e, SUC_lexicon_tags_17 },
    { 0x3ea99a2f, SUC_lexicon_tags_45 },
    { 0xd2c99a2f, SUC_lexicon_tags_16 },
    { 0x6f299a2f, SUC_lexicon_tags_43 },
    { 0x1f899a32, SUC_lexicon_tags_60 },
    { 0xc3099a30, SUC_lexicon_tags_29 },
    { 0x69699a34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4499a39, SUC_lexicon_tags_9 },
    { 0xfd299a3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a99a3f, SUC_lexicon_tags_6 },
    { 0x77699a40, SUC_lexicon_tags_33 },
    { 0x16a99a41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10e99a43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a499a45, SUC_lexicon_tags_23 },
    { 0xcca99a45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8c99a48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e99a4c, SUC_lexicon_tags_11 },
    { 0x90e99a4d, SUC_lexicon_tags_7 },
    { 0x87499a4d, SUC_lexicon_tags_15 },
    { 0x7b899a4f, SUC_lexicon_tags_12 },
    { 0x76e99a50, SUC_lexicon_tags_56 },
    { 0x2f499a50, SUC_lexicon_tags_37 },
    { 0x1d699a52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7a699a54, SUC_lexicon_tags_14 },
    { 0x81a99a55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25099a59, SUC_lexicon_tags_17 },
    { 0x33699a59, SUC_lexicon_tags_37 },
    { 0x45499a59, SUC_lexicon_tags_2 },
    { 0xb1099a5c, SUC_lexicon_tags_21 },
    { 0xc4a99a5c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x44899a60, SUC_lexicon_tags_12 },
    { 0xc3c99a60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa99a64, SUC_lexicon_tags_16 },
    { 0xb9899a65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e99a68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6899a6d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x88299a70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51099a74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e99a77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc299a79, SUC_lexicon_tags_2 },
    { 0xbde99a7a, SUC_lexicon_tags_15 },
    { 0x67a99a7b, SUC_lexicon_tags_23 },
    { 0x97a99a7b, SUC_lexicon_tags_7 },
    { 0x80e99a7c, SUC_lexicon_tags_2 },
    { 0x86699a7e, SUC_lexicon_tags_11 },
    { 0xdf099a7f, SUC_lexicon_tags_66 },
    { 0xddc99a7f, SUC_lexicon_tags_33 },
    { 0xcfa99a81, SUC_lexicon_tags_39 },
    { 0xe5c99a80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec99a86, SUC_lexicon_tags_1 },
    { 0xb9299a86, SUC_lexicon_tags_37 },
    { 0xf8699a88, SUC_lexicon_tags_1 },
    { 0x77499a88, SUC_lexicon_tags_15 },
    { 0x28299a8a, SUC_lexicon_tags_32 },
    { 0x7d699a8b, SUC_lexicon_tags_37 },
    { 0x69c99a86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x15699a8f, SUC_lexicon_tags_9 },
    { 0xa899a8f, SUC_lexicon_tags_29 },
    { 0x9ca99a91, SUC_lexicon_tags_823 },
    { 0, NULL },
    { 0x8a99a93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c099a96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a99a9c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x27c99a9e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e99aa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb699aa3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xef299aa6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a99aaa, SUC_lexicon_tags_33 },
    { 0xdc699aab, SUC_lexicon_tags_28 },
    { 0xd3699aac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6499aaf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x65a99ab1, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e99ab4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc99ab7, SUC_lexicon_tags_6 },
    { 0x88c99ab7, SUC_lexicon_tags_7 },
    { 0xb3699ab9, SUC_lexicon_tags_17 },
    { 0x39899aba, SUC_lexicon_tags_7 },
    { 0x5b499ab8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22699ac0, SUC_lexicon_tags_25 },
    { 0x6e899ac0, SUC_lexicon_tags_37 },
    { 0x27a99ac2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x28899ac4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a099ac7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33a99aca, SUC_lexicon_tags_63 },
    { 0x62299acb, SUC_lexicon_tags_80 },
    { 0x93299acb, SUC_lexicon_tags_45 },
    { 0xa299acb, SUC_lexicon_tags_7 },
    { 0x2a299acd, SUC_lexicon_tags_7 },
    { 0x8fc99ace, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e299ad3, SUC_lexicon_tags_31 },
    { 0x71899ad4, SUC_lexicon_tags_9 },
    { 0x45299ad4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94a99ad7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x27c99ad9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae99adc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e99ae2, SUC_lexicon_tags_7 },
    { 0xd0099ae3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xec699ae5, SUC_lexicon_tags_15 },
    { 0x4099ae6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a99aec, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc99aef, SUC_lexicon_tags_26 },
    { 0x5e099aef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc499af7, SUC_lexicon_tags_11 },
    { 0x2c899af8, SUC_lexicon_tags_15 },
    { 0xcb299af8, SUC_lexicon_tags_17 },
    { 0x11099afa, SUC_lexicon_tags_27 },
    { 0x2e099afb, SUC_lexicon_tags_37 },
    { 0xe6699afa, SUC_lexicon_tags_6 },
    { 0x2ca99afd, SUC_lexicon_tags_43 },
    { 0xdf299afe, SUC_lexicon_tags_37 },
    { 0xc4c99afc, SUC_lexicon_tags_1 },
    { 0xe499b00, SUC_lexicon_tags_4 },
    { 0x3d299afe, SUC_lexicon_tags_17 },
    { 0x45e99b02, SUC_lexicon_tags_27 },
    { 0x69c99afe, SUC_lexicon_tags_17 },
    { 0x14899b03, SUC_lexicon_tags_39 },
    { 0xc0299b05, SUC_lexicon_tags_25 },
    { 0xc8e99b06, SUC_lexicon_tags_68 },
    { 0x5c099b06, SUC_lexicon_tags_45 },
    { 0xcd099afa, SUC_lexicon_tags_15 },
    { 0x10c99b00, SUC_lexicon_tags_11 },
    { 0x5f899b0a, SUC_lexicon_tags_25 },
    { 0x3d299b0b, SUC_lexicon_tags_17 },
    { 0x62499b0a, SUC_lexicon_tags_7 },
    { 0x53a99b0d, SUC_lexicon_tags_2 },
    { 0x17299b0e, SUC_lexicon_tags_14 },
    { 0x62699af8, SUC_lexicon_tags_55 },
    { 0xaa499b10, SUC_lexicon_tags_37 },
    { 0x8c899b10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x36899b14, SUC_lexicon_tags_17 },
    { 0xf6499b15, SUC_lexicon_tags_0 },
    { 0x78c99b16, SUC_lexicon_tags_37 },
    { 0xcd499b17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f699b1a, SUC_lexicon_tags_28 },
    { 0x84e99b1a, SUC_lexicon_tags_37 },
    { 0x6ea99b1a, SUC_lexicon_tags_15 },
    { 0x76c99b1d, SUC_lexicon_tags_29 },
    { 0xfbc99b1e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4b899b20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2099b22, SUC_lexicon_tags_37 },
    { 0xd5099b22, SUC_lexicon_tags_56 },
    { 0x17499b22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7f899b26, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e99b2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4499b30, SUC_lexicon_tags_23 },
    { 0x2be99b30, SUC_lexicon_tags_37 },
    { 0x4d099b32, SUC_lexicon_tags_45 },
    { 0xbae99b33, SUC_lexicon_tags_1 },
    { 0xfac99b34, SUC_lexicon_tags_25 },
    { 0x64a99b32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e99b38, SUC_lexicon_tags_16 },
    { 0xf5499b39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9899b3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73699b3e, SUC_lexicon_tags_15 },
    { 0xfc899b3f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3d699b41, SUC_lexicon_tags_1 },
    { 0x8f499b41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0499b49, SUC_lexicon_tags_14 },
    { 0x12e99b4a, SUC_lexicon_tags_17 },
    { 0x84499b4b, SUC_lexicon_tags_3 },
    { 0x99699b4a, SUC_lexicon_tags_26 },
    { 0xbca99b4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5a99b4f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe1699b51, SUC_lexicon_tags_17 },
    { 0x1c699b51, SUC_lexicon_tags_1 },
    { 0xc8a99b53, SUC_lexicon_tags_10 },
    { 0x31699b54, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c99b5a, SUC_lexicon_tags_11 },
    { 0x5299b5b, SUC_lexicon_tags_0 },
    { 0x27499b5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40499b61, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xbfa99b63, SUC_lexicon_tags_27 },
    { 0x69a99b64, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xddc99b66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5699b73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49299b75, SUC_lexicon_tags_12 },
    { 0xe0099b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0699b7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5899b7c, SUC_lexicon_tags_24 },
    { 0x89499b7d, SUC_lexicon_tags_62 },
    { 0xe3e99b7d, SUC_lexicon_tags_77 },
    { 0xfd699b7d, SUC_lexicon_tags_1 },
    { 0x77a99b80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79499b82, SUC_lexicon_tags_11 },
    { 0x4a499b83, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60099b8a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x79099b8d, SUC_lexicon_tags_4 },
    { 0x66a99b8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c99b91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95899b9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec99b9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedc99ba0, SUC_lexicon_tags_27 },
    { 0x63299ba1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be99ba9, SUC_lexicon_tags_6 },
    { 0x6cc99baa, SUC_lexicon_tags_68 },
    { 0x4ba99ba9, SUC_lexicon_tags_34 },
    { 0xd9099bac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23899bb0, SUC_lexicon_tags_47 },
    { 0xf1e99bb1, SUC_lexicon_tags_11 },
    { 0x45899bb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62299bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ba99bb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37499bbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae99bbf, SUC_lexicon_tags_10 },
    { 0x1a899bc0, SUC_lexicon_tags_17 },
    { 0xa0299bc0, SUC_lexicon_tags_38 },
    { 0xbce99bc1, SUC_lexicon_tags_27 },
    { 0xc1299bc3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae99bc7, SUC_lexicon_tags_12 },
    { 0xa7099bc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce99bcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6a99bcf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade99bd4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e99bd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5899be4, SUC_lexicon_tags_12 },
    { 0x91499be5, SUC_lexicon_tags_29 },
    { 0xa5099be6, SUC_lexicon_tags_7 },
    { 0x52499be7, SUC_lexicon_tags_25 },
    { 0x6f499be5, SUC_lexicon_tags_14 },
    { 0xee299be9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36e99beb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49499bf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba99bfb, SUC_lexicon_tags_13 },
    { 0xe0e99bfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a99c03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef899c06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea99c0a, SUC_lexicon_tags_27 },
    { 0x40699c0b, SUC_lexicon_tags_11 },
    { 0xea99c0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2499c0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0299c14, SUC_lexicon_tags_12 },
    { 0x499c15, SUC_lexicon_tags_1 },
    { 0xd9299c15, SUC_lexicon_tags_4 },
    { 0xc4699c17, SUC_lexicon_tags_2 },
    { 0x6ac99c14, SUC_lexicon_tags_2 },
    { 0xe8899c19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e099c1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c99c22, SUC_lexicon_tags_45 },
    { 0x8b699c23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7be99c25, SUC_lexicon_tags_23 },
    { 0xa4a99c26, SUC_lexicon_tags_17 },
    { 0x26099c27, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71899c2b, SUC_lexicon_tags_14 },
    { 0x3fa99c2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d899c2f, SUC_lexicon_tags_11 },
    { 0xa6099c30, SUC_lexicon_tags_37 },
    { 0x8a299c30, SUC_lexicon_tags_6 },
    { 0xc2299c31, SUC_lexicon_tags_7 },
    { 0x64299c33, SUC_lexicon_tags_37 },
    { 0x59699c34, SUC_lexicon_tags_0 },
    { 0xea699c30, SUC_lexicon_tags_7 },
    { 0x80c99c32, SUC_lexicon_tags_49 },
    { 0xd4699c34, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29699c3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a899c3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6a99c41, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe299c45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9499c48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa99c4c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x1fa99c4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd099c53, SUC_lexicon_tags_37 },
    { 0xeda99c54, SUC_lexicon_tags_25 },
    { 0x2cc99c55, SUC_lexicon_tags_25 },
    { 0x44699c56, SUC_lexicon_tags_32 },
    { 0x29699c57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15e99c59, SUC_lexicon_tags_136 },
    { 0x25499c5a, SUC_lexicon_tags_45 },
    { 0xc1499c5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x16099c5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6299c5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e99c63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5ca99c65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a99c67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7499c69, SUC_lexicon_tags_74 },
    { 0x5b499c6a, SUC_lexicon_tags_15 },
    { 0xe0e99c6a, SUC_lexicon_tags_15 },
    { 0xba899c6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb699c6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22a99c71, SUC_lexicon_tags_37 },
    { 0x9ac99c72, SUC_lexicon_tags_170 },
    { 0x5c099c73, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3299c76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdd099c78, SUC_lexicon_tags_33 },
    { 0xc4499c78, SUC_lexicon_tags_37 },
    { 0x36a99c78, SUC_lexicon_tags_1 },
    { 0x3499c7b, SUC_lexicon_tags_60 },
    { 0x13a99c7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e99c7f, SUC_lexicon_tags_11 },
    { 0xe1e99c7f, SUC_lexicon_tags_7 },
    { 0x83699c7f, SUC_lexicon_tags_27 },
    { 0x45699c82, SUC_lexicon_tags_11 },
    { 0x2fe99c83, SUC_lexicon_tags_17 },
    { 0x9f299c81, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c99c88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca99c8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2a99c8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdea99c90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a899c92, SUC_lexicon_tags_45 },
    { 0xe8699c93, SUC_lexicon_tags_15 },
    { 0x12c99c93, SUC_lexicon_tags_15 },
    { 0x1e899c95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ba99c97, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfcc99c99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x64499c9c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d299ca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab699ca3, SUC_lexicon_tags_15 },
    { 0x1d299ca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe699caa, SUC_lexicon_tags_29 },
    { 0xc6299cab, SUC_lexicon_tags_37 },
    { 0xee699cac, SUC_lexicon_tags_14 },
    { 0x8f499cad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97899cb3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63499cb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28299cb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf299cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe99cc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e499cc2, SUC_lexicon_tags_63 },
    { 0x7f099cc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a99cc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e99ccd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd099cd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f099cdb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99499ce3, SUC_lexicon_tags_4 },
    { 0x65699ce4, SUC_lexicon_tags_91 },
    { 0x86499ce4, SUC_lexicon_tags_17 },
    { 0xed899ce6, SUC_lexicon_tags_32 },
    { 0xb2499ce7, SUC_lexicon_tags_82 },
    { 0x26699ce8, SUC_lexicon_tags_17 },
    { 0xbca99ce4, SUC_lexicon_tags_77 },
    { 0x56299cea, SUC_lexicon_tags_22 },
    { 0x15499ce7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9f899ced, SUC_lexicon_tags_1 },
    { 0xbb299cee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x92899cf1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbca99cf3, SUC_lexicon_tags_17 },
    { 0x26699cf4, SUC_lexicon_tags_39 },
    { 0x30699cf5, SUC_lexicon_tags_12 },
    { 0xa3699cf6, SUC_lexicon_tags_1 },
    { 0x2b299cf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c99cfb, SUC_lexicon_tags_37 },
    { 0xc7899cfc, SUC_lexicon_tags_48 },
    { 0xb8c99cfd, SUC_lexicon_tags_19 },
    { 0xf6299cfe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b299d07, SUC_lexicon_tags_63 },
    { 0x4c99d08, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc299d0d, SUC_lexicon_tags_11 },
    { 0xae699d0e, SUC_lexicon_tags_11 },
    { 0x6a099d0f, SUC_lexicon_tags_16 },
    { 0xe2499d0f, SUC_lexicon_tags_12 },
    { 0x26c99d11, SUC_lexicon_tags_37 },
    { 0xaa299d12, SUC_lexicon_tags_32 },
    { 0xf7c99d13, SUC_lexicon_tags_19 },
    { 0xb7c99d0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46299d17, SUC_lexicon_tags_56 },
    { 0x71c99d18, SUC_lexicon_tags_39 },
    { 0xa8499d19, SUC_lexicon_tags_17 },
    { 0xb2899d17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95899d1c, SUC_lexicon_tags_8 },
    { 0x7d099d1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfe99d1f, SUC_lexicon_tags_19 },
    { 0xfc499d1f, SUC_lexicon_tags_60 },
    { 0x39299d21, SUC_lexicon_tags_102 },
    { 0xcc99d21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79499d24, SUC_lexicon_tags_2 },
    { 0x48099d24, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28499d2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7299d2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a99d32, SUC_lexicon_tags_1 },
    { 0xbd299d33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d099d35, SUC_lexicon_tags_4 },
    { 0xa3a99d36, SUC_lexicon_tags_0 },
    { 0x55a99d37, SUC_lexicon_tags_43 },
    { 0x56899d38, SUC_lexicon_tags_9 },
    { 0x68899d36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c499d3c, SUC_lexicon_tags_23 },
    { 0x45899d3c, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x2fc99d3f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e99d42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93699d46, SUC_lexicon_tags_2 },
    { 0xd8a99d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1699d49, SUC_lexicon_tags_41 },
    { 0x4899d49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49099d4e, SUC_lexicon_tags_47 },
    { 0x9b899d4f, SUC_lexicon_tags_2 },
    { 0xe4299d4f, SUC_lexicon_tags_37 },
    { 0xd7e99d4f, SUC_lexicon_tags_2 },
    { 0x9fa99d52, SUC_lexicon_tags_29 },
    { 0x69c99d53, SUC_lexicon_tags_13 },
    { 0x4a299d52, SUC_lexicon_tags_15 },
    { 0x8de99d55, SUC_lexicon_tags_23 },
    { 0x34c99d56, SUC_lexicon_tags_17 },
    { 0x3c899d57, SUC_lexicon_tags_26 },
    { 0x98499d57, SUC_lexicon_tags_32 },
    { 0x79099d59, SUC_lexicon_tags_33 },
    { 0x46299d59, SUC_lexicon_tags_2 },
    { 0xc0699d5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe499d60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88a99d62, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfb099d64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6699d66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc699d69, SUC_lexicon_tags_47 },
    { 0x55c99d69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70899d70, SUC_lexicon_tags_28 },
    { 0x87099d71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e899d75, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb8a99d77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e899d7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f699d7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0099d7f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc99d83, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7499d86, SUC_lexicon_tags_28 },
    { 0x8a899d86, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x40e99d89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc99d8b, SUC_lexicon_tags_33 },
    { 0x3f099d8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c899d92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2099d9b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa4099d9d, SUC_lexicon_tags_0 },
    { 0x9ec99d9d, SUC_lexicon_tags_15 },
    { 0xe0a99d9f, SUC_lexicon_tags_68 },
    { 0x89a99da0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a99da3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6b299da5, SUC_lexicon_tags_14 },
    { 0xf9099da5, SUC_lexicon_tags_80 },
    { 0x60499da7, SUC_lexicon_tags_12 },
    { 0x9d299da8, SUC_lexicon_tags_18 },
    { 0x33499da9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c99dac, SUC_lexicon_tags_11 },
    { 0xf9499dad, SUC_lexicon_tags_17 },
    { 0xd7899dae, SUC_lexicon_tags_66 },
    { 0xbfc99daf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x53099db1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0299db7, SUC_lexicon_tags_11 },
    { 0x41499db7, SUC_lexicon_tags_2 },
    { 0x67e99db9, SUC_lexicon_tags_15 },
    { 0xd0a99db7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf899dbf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xac099dc2, SUC_lexicon_tags_47 },
    { 0x92899dc3, SUC_lexicon_tags_39 },
    { 0xa8499dc4, SUC_lexicon_tags_43 },
    { 0xc9099dc5, SUC_lexicon_tags_37 },
    { 0x85899dc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54e99dc8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xff299dcb, SUC_lexicon_tags_47 },
    { 0x5f699dcc, SUC_lexicon_tags_10 },
    { 0x5ac99dcd, SUC_lexicon_tags_351 },
    { 0x5ea99dcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12499dd2, SUC_lexicon_tags_15 },
    { 0x17499dd3, SUC_lexicon_tags_17 },
    { 0x69a99dd3, SUC_lexicon_tags_31 },
    { 0x87899dd3, SUC_lexicon_tags_9 },
    { 0x77499dd6, SUC_lexicon_tags_1 },
    { 0xcb299dd6, SUC_lexicon_tags_37 },
    { 0xd3499dd6, SUC_lexicon_tags_60 },
    { 0xc3099dd9, SUC_lexicon_tags_66 },
    { 0x59499dd9, SUC_lexicon_tags_15 },
    { 0x20099dda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec99ddf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe899de1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x90099de3, SUC_lexicon_tags_63 },
    { 0x15499de4, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e899de7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53899dea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbae99dec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf2699dee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3299df0, SUC_lexicon_tags_2 },
    { 0x9e899df0, SUC_lexicon_tags_28 },
    { 0xa299df2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb699df4, SUC_lexicon_tags_31 },
    { 0x63c99df5, SUC_lexicon_tags_17 },
    { 0x45e99df5, SUC_lexicon_tags_8 },
    { 0x56899df7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5899dfb, SUC_lexicon_tags_179 },
    { 0xdb299dfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83299dff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46099e05, SUC_lexicon_tags_126 },
    { 0x92699e06, SUC_lexicon_tags_84 },
    { 0xfee99e07, SUC_lexicon_tags_15 },
    { 0x17499e05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x61099e0a, SUC_lexicon_tags_47 },
    { 0x74499e0b, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5099e12, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4899e16, SUC_lexicon_tags_7 },
    { 0xf5099e16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace99e1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba99e27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1299e29, SUC_lexicon_tags_16 },
    { 0x43a99e2a, SUC_lexicon_tags_37 },
    { 0x6e499e29, SUC_lexicon_tags_4 },
    { 0x8fc99e2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1299e30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac99e35, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b499e38, SUC_lexicon_tags_17 },
    { 0x51699e38, SUC_lexicon_tags_17 },
    { 0xb8699e3a, SUC_lexicon_tags_11 },
    { 0xbd699e39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47099e3d, SUC_lexicon_tags_61 },
    { 0x51099e3d, SUC_lexicon_tags_19 },
    { 0xb7299e3e, SUC_lexicon_tags_38 },
    { 0x3c699e40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d699e44, SUC_lexicon_tags_35 },
    { 0xffe99e45, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87899e47, SUC_lexicon_tags_68 },
    { 0xc0e99e48, SUC_lexicon_tags_12 },
    { 0xbe299e48, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e99e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81299e52, SUC_lexicon_tags_7 },
    { 0xba99e53, SUC_lexicon_tags_40 },
    { 0x1d299e52, SUC_lexicon_tags_2 },
    { 0xdc899e55, SUC_lexicon_tags_8 },
    { 0xf4299e55, SUC_lexicon_tags_14 },
    { 0xf0499e57, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0xe3e99e59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66299e5b, SUC_lexicon_tags_65 },
    { 0x58099e5c, SUC_lexicon_tags_37 },
    { 0x1299e5c, SUC_lexicon_tags_37 },
    { 0x23c99e5e, SUC_lexicon_tags_45 },
    { 0x3f899e5f, SUC_lexicon_tags_25 },
    { 0xfa699e60, SUC_lexicon_tags_49 },
    { 0x9ae99e61, SUC_lexicon_tags_15 },
    { 0x69899e5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17e99e64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5299e69, SUC_lexicon_tags_17 },
    { 0x26899e6a, SUC_lexicon_tags_2 },
    { 0x3c699e69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a899e6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65899e77, SUC_lexicon_tags_17 },
    { 0x8fc99e78, SUC_lexicon_tags_25 },
    { 0x5f899e79, SUC_lexicon_tags_33 },
    { 0x4da99e7a, SUC_lexicon_tags_19 },
    { 0x3499e7b, SUC_lexicon_tags_37 },
    { 0xbe899e78, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77099e80, SUC_lexicon_tags_28 },
    { 0xa3c99e80, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfee99e84, SUC_lexicon_tags_10 },
    { 0x6c699e85, SUC_lexicon_tags_33 },
    { 0x53899e86, SUC_lexicon_tags_46 },
    { 0xee699e85, SUC_lexicon_tags_2 },
    { 0x32c99e88, SUC_lexicon_tags_12 },
    { 0x78c99e88, SUC_lexicon_tags_39 },
    { 0xab499e88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70699e8d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xde699e90, SUC_lexicon_tags_1 },
    { 0x64699e91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a99e95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67299e97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c99e9a, SUC_lexicon_tags_7 },
    { 0x8c99e9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81099ea1, SUC_lexicon_tags_17 },
    { 0xe9699ea2, SUC_lexicon_tags_39 },
    { 0xb2c99ea3, SUC_lexicon_tags_12 },
    { 0x3099ea4, SUC_lexicon_tags_47 },
    { 0x62699ea3, SUC_lexicon_tags_11 },
    { 0x84e99ea2, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a99ea9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6899eb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe99eb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b699eb5, SUC_lexicon_tags_37 },
    { 0x56a99eb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75899ebd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8099ebf, SUC_lexicon_tags_15 },
    { 0x89899ebf, SUC_lexicon_tags_11 },
    { 0x37899ec0, SUC_lexicon_tags_23 },
    { 0x89499ec0, SUC_lexicon_tags_25 },
    { 0x48699ec3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68499ec5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ac99ec7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a99ed1, SUC_lexicon_tags_30 },
    { 0x78e99ed2, SUC_lexicon_tags_49 },
    { 0xe6699ed3, SUC_lexicon_tags_25 },
    { 0xcf299ed2, SUC_lexicon_tags_25 },
    { 0xcfc99ed3, SUC_lexicon_tags_23 },
    { 0xa1299ed6, SUC_lexicon_tags_26 },
    { 0x7a499ed5, SUC_lexicon_tags_0 },
    { 0x54099ed8, SUC_lexicon_tags_234 },
    { 0xfcc99ed8, SUC_lexicon_tags_14 },
    { 0xc8699ed4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f699edd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6499ee4, SUC_lexicon_tags_2 },
    { 0x8099ee5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc0c99ee7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93099eec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a99eef, SUC_lexicon_tags_27 },
    { 0xf0899eef, SUC_lexicon_tags_52 },
    { 0xa5a99ef1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef899ef7, SUC_lexicon_tags_9 },
    { 0x2ea99ef8, SUC_lexicon_tags_1 },
    { 0xa4c99ef9, SUC_lexicon_tags_37 },
    { 0x8aa99efa, SUC_lexicon_tags_26 },
    { 0xc4c99efb, SUC_lexicon_tags_25 },
    { 0x16a99efb, SUC_lexicon_tags_21 },
    { 0xeb899efb, SUC_lexicon_tags_1 },
    { 0x28c99efe, SUC_lexicon_tags_15 },
    { 0xf2c99eff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a99f07, SUC_lexicon_tags_15 },
    { 0x30c99f07, SUC_lexicon_tags_17 },
    { 0x4ca99f09, SUC_lexicon_tags_12 },
    { 0xafa99f0a, SUC_lexicon_tags_33 },
    { 0x84499f0b, SUC_lexicon_tags_38 },
    { 0x60099f0c, SUC_lexicon_tags_15 },
    { 0x37e99f0b, SUC_lexicon_tags_33 },
    { 0x36e99f0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd699f11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x98899f13, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e299f16, SUC_lexicon_tags_39 },
    { 0x3b899f17, SUC_lexicon_tags_17 },
    { 0x7d699f18, SUC_lexicon_tags_17 },
    { 0x37699f18, SUC_lexicon_tags_65 },
    { 0x7ec99f16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f499f1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x41099f20, SUC_lexicon_tags_4 },
    { 0x93c99f21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaea99f23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57699f25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32699f28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a99f2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc99f32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9499f37, SUC_lexicon_tags_61 },
    { 0x64499f38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e099f42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e699f45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x70699f47, SUC_lexicon_tags_63 },
    { 0x4c499f48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x44c99f4a, SUC_lexicon_tags_25 },
    { 0x7cc99f4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9899f51, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x68c99f53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcb299f55, SUC_lexicon_tags_25 },
    { 0x3e899f55, SUC_lexicon_tags_9 },
    { 0x41099f55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3899f5c, SUC_lexicon_tags_25 },
    { 0x1299f5c, SUC_lexicon_tags_49 },
    { 0x98499f5e, SUC_lexicon_tags_27 },
    { 0x96099f5f, SUC_lexicon_tags_2 },
    { 0xa8c99f5f, SUC_lexicon_tags_39 },
    { 0x79699f60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6099f63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c99f68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b499f70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x56699f73, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca99f76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6299f79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c299f80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43899f83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71899f86, SUC_lexicon_tags_38 },
    { 0x75899f86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76499f8b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf6a99f8d, SUC_lexicon_tags_13 },
    { 0xbbc99f8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca099f90, SUC_lexicon_tags_12 },
    { 0xa6c99f90, SUC_lexicon_tags_12 },
    { 0xdf299f92, SUC_lexicon_tags_25 },
    { 0x95a99f93, SUC_lexicon_tags_37 },
    { 0x75e99f91, SUC_lexicon_tags_25 },
    { 0x2a699f95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e99f9b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba99f9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad099fa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f699fa7, SUC_lexicon_tags_15 },
    { 0x80e99fa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2699fab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xace99fad, SUC_lexicon_tags_11 },
    { 0xde99fae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4499fb8, SUC_lexicon_tags_12 },
    { 0x38699fb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20699fbb, SUC_lexicon_tags_1 },
    { 0xb5099fbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3699fbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c99fc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce99fc9, SUC_lexicon_tags_8 },
    { 0xcdc99fca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1699fcf, SUC_lexicon_tags_117 },
    { 0xd1e99fd0, SUC_lexicon_tags_30 },
    { 0xf6899fd0, SUC_lexicon_tags_0 },
    { 0xbe099fd2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c99fe2, SUC_lexicon_tags_2 },
    { 0xef899fe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60299fe5, SUC_lexicon_tags_5 },
    { 0x19099fe6, SUC_lexicon_tags_11 },
    { 0xfde99fe7, SUC_lexicon_tags_37 },
    { 0xb8699fe8, SUC_lexicon_tags_25 },
    { 0x4d099fe8, SUC_lexicon_tags_14 },
    { 0x8f099fe8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x17699fed, SUC_lexicon_tags_17 },
    { 0x5fc99fed, SUC_lexicon_tags_17 },
    { 0x1fa99fef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd1699ff1, SUC_lexicon_tags_45 },
    { 0x7aa99ff1, SUC_lexicon_tags_27 },
    { 0xc5a99ff2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb299ff6, SUC_lexicon_tags_12 },
    { 0xfe699ff6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7299fff, SUC_lexicon_tags_19 },
    { 0xdf89a000, SUC_lexicon_tags_47 },
    { 0x1dc9a000, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1fe9a003, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc9a008, SUC_lexicon_tags_16 },
    { 0xbf69a009, SUC_lexicon_tags_46 },
    { 0x25e9a009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8e9a00c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3169a012, SUC_lexicon_tags_2 },
    { 0xa3e9a013, SUC_lexicon_tags_31 },
    { 0xcfa9a013, SUC_lexicon_tags_1 },
    { 0x6009a014, SUC_lexicon_tags_15 },
    { 0x6ae9a012, SUC_lexicon_tags_79 },
    { 0x52a9a017, SUC_lexicon_tags_1 },
    { 0x9049a018, SUC_lexicon_tags_1 },
    { 0x8b09a015, SUC_lexicon_tags_25 },
    { 0x1729a01a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5d29a01c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0c9a01e, SUC_lexicon_tags_30 },
    { 0xc5a9a01f, SUC_lexicon_tags_37 },
    { 0x2049a01e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e9a024, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x49a9a026, SUC_lexicon_tags_26 },
    { 0x2fa9a026, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa9a02b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e9a02e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2109a031, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa309a037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef89a039, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfd89a03b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4449a03d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa289a04d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8449a055, SUC_lexicon_tags_18 },
    { 0x4a89a056, SUC_lexicon_tags_17 },
    { 0x1789a057, SUC_lexicon_tags_37 },
    { 0xe909a056, SUC_lexicon_tags_49 },
    { 0x429a055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1da9a05c, SUC_lexicon_tags_12 },
    { 0xcc09a05d, SUC_lexicon_tags_33 },
    { 0x2ca9a05c, SUC_lexicon_tags_27 },
    { 0x6ea9a05f, SUC_lexicon_tags_46 },
    { 0xe869a05c, SUC_lexicon_tags_2 },
    { 0x71a9a060, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae9a065, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2e49a067, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb8a9a069, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4969a06e, SUC_lexicon_tags_7 },
    { 0x9949a06e, SUC_lexicon_tags_46 },
    { 0x609a070, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xace9a072, SUC_lexicon_tags_97 },
    { 0x45a9a073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2529a076, SUC_lexicon_tags_7 },
    { 0xee29a076, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc589a079, SUC_lexicon_tags_25 },
    { 0xff29a07a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef69a087, SUC_lexicon_tags_44 },
    { 0x2349a088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb29a08e, SUC_lexicon_tags_1 },
    { 0x6609a08f, SUC_lexicon_tags_37 },
    { 0x7ea9a090, SUC_lexicon_tags_12 },
    { 0x6709a091, SUC_lexicon_tags_47 },
    { 0xe4c9a092, SUC_lexicon_tags_17 },
    { 0x8d89a093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3e9a095, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee09a099, SUC_lexicon_tags_38 },
    { 0xdee9a09a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be9a0a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f49a0a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xee29a0a6, SUC_lexicon_tags_15 },
    { 0x9089a0a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e49a0ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b69a0ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f09a0af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7649a0b1, SUC_lexicon_tags_7 },
    { 0x4f89a0b1, SUC_lexicon_tags_28 },
    { 0x3de9a0b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee9a0b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf369a0b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5549a0bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec49a0be, SUC_lexicon_tags_23 },
    { 0x57e9a0bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8889a0c3, SUC_lexicon_tags_25 },
    { 0x35a9a0c3, SUC_lexicon_tags_37 },
    { 0xca69a0c5, SUC_lexicon_tags_11 },
    { 0xab89a0c5, SUC_lexicon_tags_37 },
    { 0x1029a0c7, SUC_lexicon_tags_2 },
    { 0xc169a0c7, SUC_lexicon_tags_12 },
    { 0x9809a0c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf929a0cd, SUC_lexicon_tags_2 },
    { 0xd629a0cd, SUC_lexicon_tags_17 },
    { 0xf449a0cd, SUC_lexicon_tags_14 },
    { 0xd69a0d0, SUC_lexicon_tags_1 },
    { 0x1929a0ce, SUC_lexicon_tags_32 },
    { 0xe4e9a0d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ec9a0d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0e9a0d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa649a0d8, SUC_lexicon_tags_31 },
    { 0xb989a0d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6749a0dc, SUC_lexicon_tags_29 },
    { 0x1dc9a0dd, SUC_lexicon_tags_23 },
    { 0x1ce9a0dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5889a0e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfd69a0e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8489a0e8, SUC_lexicon_tags_39 },
    { 0x7e69a0e9, SUC_lexicon_tags_15 },
    { 0x4ca9a0ea, SUC_lexicon_tags_45 },
    { 0x72c9a0eb, SUC_lexicon_tags_1 },
    { 0x34a9a0eb, SUC_lexicon_tags_37 },
    { 0xb749a0ec, SUC_lexicon_tags_17 },
    { 0xf089a0ea, SUC_lexicon_tags_39 },
    { 0xb8e9a0ef, SUC_lexicon_tags_6 },
    { 0xa169a0f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa049a0f5, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be9a0f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a49a0fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8fc9a0fd, SUC_lexicon_tags_7 },
    { 0x729a0fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e9a101, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f49a105, SUC_lexicon_tags_25 },
    { 0x2fe9a106, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e9a10f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c9a117, SUC_lexicon_tags_31 },
    { 0x5ce9a118, SUC_lexicon_tags_39 },
    { 0xca89a118, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xd689a11b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee89a121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b09a123, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa689a125, SUC_lexicon_tags_17 },
    { 0xd569a126, SUC_lexicon_tags_11 },
    { 0x6649a127, SUC_lexicon_tags_33 },
    { 0xf629a128, SUC_lexicon_tags_25 },
    { 0x8c9a127, SUC_lexicon_tags_2 },
    { 0xe1c9a129, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09a12d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9f09a12f, SUC_lexicon_tags_31 },
    { 0xe789a130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ec9a132, SUC_lexicon_tags_25 },
    { 0x4009a133, SUC_lexicon_tags_37 },
    { 0x36a9a134, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a9a13c, SUC_lexicon_tags_11 },
    { 0x2d89a13c, SUC_lexicon_tags_28 },
    { 0x30e9a13e, SUC_lexicon_tags_17 },
    { 0x6709a13f, SUC_lexicon_tags_11 },
    { 0x86e9a140, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee29a146, SUC_lexicon_tags_7 },
    { 0xce89a147, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3789a14b, SUC_lexicon_tags_35 },
    { 0x6589a14c, SUC_lexicon_tags_38 },
    { 0xc149a14d, SUC_lexicon_tags_17 },
    { 0x31c9a14d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5c9a150, SUC_lexicon_tags_15 },
    { 0x3ec9a151, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4929a153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f29a157, SUC_lexicon_tags_38 },
    { 0xb2c9a158, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2309a15d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5689a161, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd209a164, SUC_lexicon_tags_37 },
    { 0x1709a164, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1f89a167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4729a169, SUC_lexicon_tags_10 },
    { 0x7b69a169, SUC_lexicon_tags_15 },
    { 0x8429a16a, SUC_lexicon_tags_60 },
    { 0x48e9a16c, SUC_lexicon_tags_37 },
    { 0xf569a16d, SUC_lexicon_tags_7 },
    { 0xfc89a16a, SUC_lexicon_tags_15 },
    { 0x6e09a16f, SUC_lexicon_tags_12 },
    { 0xd0a9a170, SUC_lexicon_tags_37 },
    { 0xe6a9a171, SUC_lexicon_tags_8 },
    { 0x8ba9a171, SUC_lexicon_tags_68 },
    { 0x4e89a173, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x46a9a176, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c9a17a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa49a17c, SUC_lexicon_tags_15 },
    { 0xe8e9a17d, SUC_lexicon_tags_7 },
    { 0x7cc9a17c, SUC_lexicon_tags_8 },
    { 0xe89a17f, SUC_lexicon_tags_23 },
    { 0x7429a17e, SUC_lexicon_tags_9 },
    { 0x7789a181, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb749a186, SUC_lexicon_tags_7 },
    { 0xb789a187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d49a18a, SUC_lexicon_tags_46 },
    { 0x6209a18b, SUC_lexicon_tags_17 },
    { 0xcce9a18b, SUC_lexicon_tags_33 },
    { 0x41a9a18b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4829a190, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xef89a193, SUC_lexicon_tags_17 },
    { 0xcce9a194, SUC_lexicon_tags_29 },
    { 0xa7e9a195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4389a198, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2449a1a0, SUC_lexicon_tags_12 },
    { 0x1989a1a1, SUC_lexicon_tags_14 },
    { 0x1d49a1a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4da9a1a4, SUC_lexicon_tags_15 },
    { 0x15c9a1a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a9a1a8, SUC_lexicon_tags_8 },
    { 0x8849a1a8, SUC_lexicon_tags_15 },
    { 0xebc9a1aa, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x53e9a1ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c9a1b2, SUC_lexicon_tags_48 },
    { 0x8f69a1b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a89a1b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9729a1b8, SUC_lexicon_tags_14 },
    { 0xee89a1b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2229a1bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe249a1be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef49a1c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e9a1c7, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xfda9a1c9, SUC_lexicon_tags_65 },
    { 0xc8c9a1ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4929a1cd, SUC_lexicon_tags_15 },
    { 0xc829a1ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef49a1d0, SUC_lexicon_tags_45 },
    { 0x5a69a1d0, SUC_lexicon_tags_17 },
    { 0xfce9a1d1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x65a9a1d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe449a1d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3509a1df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7669a1e1, SUC_lexicon_tags_63 },
    { 0xf0e9a1e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c9a1e6, SUC_lexicon_tags_11 },
    { 0xad29a1e6, SUC_lexicon_tags_0 },
    { 0xdc09a1e8, SUC_lexicon_tags_82 },
    { 0x8fa9a1e9, SUC_lexicon_tags_10 },
    { 0xf029a1e9, SUC_lexicon_tags_17 },
    { 0x61c9a1eb, SUC_lexicon_tags_15 },
    { 0x4989a1ea, SUC_lexicon_tags_14 },
    { 0xe49a1e9, SUC_lexicon_tags_15 },
    { 0x9fc9a1ed, SUC_lexicon_tags_43 },
    { 0xba69a1e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5429a1f1, SUC_lexicon_tags_1 },
    { 0xef29a1f1, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0x4269a1f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1e09a1f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b69a1f9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc129a1fc, SUC_lexicon_tags_47 },
    { 0xdee9a1fd, SUC_lexicon_tags_38 },
    { 0xbe69a1fe, SUC_lexicon_tags_43 },
    { 0x9589a1ff, SUC_lexicon_tags_39 },
    { 0xdc69a200, SUC_lexicon_tags_2 },
    { 0x48c9a201, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xacc9a203, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc9a206, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x31c9a208, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf889a20b, SUC_lexicon_tags_27 },
    { 0x9fe9a20c, SUC_lexicon_tags_31 },
    { 0x7249a20d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8089a20f, SUC_lexicon_tags_17 },
    { 0x8de9a210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe249a212, SUC_lexicon_tags_1 },
    { 0x4f89a213, SUC_lexicon_tags_11 },
    { 0xeda9a213, SUC_lexicon_tags_1 },
    { 0xb69a215, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xec09a218, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a9a21c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e9a220, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf49a223, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6789a227, SUC_lexicon_tags_27 },
    { 0xc129a227, SUC_lexicon_tags_29 },
    { 0x98c9a229, SUC_lexicon_tags_2 },
    { 0xf1e9a22a, SUC_lexicon_tags_47 },
    { 0xa089a22a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfe9a22d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8689a237, SUC_lexicon_tags_4 },
    { 0x9ae9a238, SUC_lexicon_tags_11 },
    { 0xfc09a239, SUC_lexicon_tags_38 },
    { 0x9b69a237, SUC_lexicon_tags_69 },
    { 0x8ca9a23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7149a23e, SUC_lexicon_tags_26 },
    { 0x5889a23f, SUC_lexicon_tags_15 },
    { 0x8789a23e, SUC_lexicon_tags_17 },
    { 0xb029a241, SUC_lexicon_tags_37 },
    { 0x81e9a242, SUC_lexicon_tags_33 },
    { 0x64a9a240, SUC_lexicon_tags_64 },
    { 0x4a49a241, SUC_lexicon_tags_8 },
    { 0x77a9a245, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc909a248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de9a24e, SUC_lexicon_tags_63 },
    { 0x74e9a24f, SUC_lexicon_tags_15 },
    { 0xc4a9a250, SUC_lexicon_tags_30 },
    { 0x6b89a251, SUC_lexicon_tags_12 },
    { 0x9329a24f, SUC_lexicon_tags_27 },
    { 0x2629a24e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe089a255, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a9a258, SUC_lexicon_tags_1 },
    { 0x54e9a258, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e9a25e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86c9a260, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1169a262, SUC_lexicon_tags_17 },
    { 0xca69a263, SUC_lexicon_tags_57 },
    { 0x6289a263, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d09a269, SUC_lexicon_tags_84 },
    { 0x39c9a26a, SUC_lexicon_tags_43 },
    { 0x76e9a26b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d29a270, SUC_lexicon_tags_27 },
    { 0x7309a270, SUC_lexicon_tags_17 },
    { 0x4829a272, SUC_lexicon_tags_16 },
    { 0xf129a272, SUC_lexicon_tags_63 },
    { 0xa049a274, SUC_lexicon_tags_14 },
    { 0xc5e9a274, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba9a279, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd9c9a27b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb29a27e, SUC_lexicon_tags_30 },
    { 0xf9c9a27f, SUC_lexicon_tags_25 },
    { 0x8ea9a280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c9a284, SUC_lexicon_tags_11 },
    { 0x9a09a285, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f29a28a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e9a28e, SUC_lexicon_tags_10 },
    { 0x4a9a28f, SUC_lexicon_tags_38 },
    { 0xe349a290, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1169a293, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe989a299, SUC_lexicon_tags_17 },
    { 0xb789a29a, SUC_lexicon_tags_65 },
    { 0xf7e9a29a, SUC_lexicon_tags_13 },
    { 0x41a9a29c, SUC_lexicon_tags_11 },
    { 0x66c9a29d, SUC_lexicon_tags_17 },
    { 0x53e9a29c, SUC_lexicon_tags_60 },
    { 0x9829a299, SUC_lexicon_tags_47 },
    { 0x7489a2a0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c9a2a7, SUC_lexicon_tags_37 },
    { 0x7209a2a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc69a2aa, SUC_lexicon_tags_12 },
    { 0xb6e9a2aa, SUC_lexicon_tags_15 },
    { 0xca69a2ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3509a2b0, SUC_lexicon_tags_15 },
    { 0x7cc9a2b1, SUC_lexicon_tags_1 },
    { 0x8769a2b2, SUC_lexicon_tags_31 },
    { 0xe269a2b3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc189a2b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3009a2b7, SUC_lexicon_tags_7 },
    { 0x7509a2b8, SUC_lexicon_tags_25 },
    { 0xe929a2b8, SUC_lexicon_tags_3 },
    { 0xc1a9a2ba, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec9a2c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x24a9a2c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe729a2c6, SUC_lexicon_tags_0 },
    { 0xca69a2c6, SUC_lexicon_tags_17 },
    { 0x24c9a2c6, SUC_lexicon_tags_7 },
    { 0xe1a9a2c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xee09a2cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1209a2cd, SUC_lexicon_tags_48 },
    { 0x8e09a2cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7a9a2d0, SUC_lexicon_tags_21 },
    { 0xb789a2d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc49a2d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f09a2e1, SUC_lexicon_tags_12 },
    { 0xc749a2e2, SUC_lexicon_tags_11 },
    { 0xdfa9a2e3, SUC_lexicon_tags_49 },
    { 0xc5a9a2e4, SUC_lexicon_tags_23 },
    { 0x8c9a2e3, SUC_lexicon_tags_15 },
    { 0x4469a2e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde9a2e8, SUC_lexicon_tags_37 },
    { 0x6309a2e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f49a2f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5569a2f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe09a2f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9529a2fe, SUC_lexicon_tags_15 },
    { 0xf1a9a2ff, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb769a309, SUC_lexicon_tags_2 },
    { 0x18e9a30a, SUC_lexicon_tags_0 },
    { 0xd169a30b, SUC_lexicon_tags_47 },
    { 0x5ac9a30c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c9a310, SUC_lexicon_tags_198 },
    { 0x3a09a311, SUC_lexicon_tags_28 },
    { 0x6de9a311, SUC_lexicon_tags_11 },
    { 0x28e9a312, SUC_lexicon_tags_17 },
    { 0xf789a314, SUC_lexicon_tags_2 },
    { 0xa729a311, SUC_lexicon_tags_15 },
    { 0xff49a316, SUC_lexicon_tags_25 },
    { 0x3de9a314, SUC_lexicon_tags_11 },
    { 0xc989a318, SUC_lexicon_tags_17 },
    { 0xcb09a319, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e9a31d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89a325, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3cc9a327, SUC_lexicon_tags_60 },
    { 0xc669a328, SUC_lexicon_tags_15 },
    { 0x7fe9a329, SUC_lexicon_tags_33 },
    { 0x11c9a32a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9a32d, SUC_lexicon_tags_1 },
    { 0x5b49a32d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c89a330, SUC_lexicon_tags_9 },
    { 0x1f49a331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e9a335, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92c9a337, SUC_lexicon_tags_56 },
    { 0x45a9a338, SUC_lexicon_tags_38 },
    { 0xa469a338, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb889a33b, SUC_lexicon_tags_37 },
    { 0xd629a33c, SUC_lexicon_tags_1 },
    { 0x23e9a33b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf089a33f, SUC_lexicon_tags_40 },
    { 0x56c9a340, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe8e9a342, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4f69a344, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f49a347, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f89a34d, SUC_lexicon_tags_2 },
    { 0x7ba9a34e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6029a350, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13e9a352, SUC_lexicon_tags_25 },
    { 0xba09a353, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b89a355, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc469a357, SUC_lexicon_tags_47 },
    { 0x6e09a358, SUC_lexicon_tags_23 },
    { 0x5209a359, SUC_lexicon_tags_14 },
    { 0x2909a357, SUC_lexicon_tags_69 },
    { 0xa4e9a35a, SUC_lexicon_tags_15 },
    { 0x6689a35c, SUC_lexicon_tags_8 },
    { 0x9a89a358, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29a360, SUC_lexicon_tags_23 },
    { 0x4849a360, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca09a363, SUC_lexicon_tags_3 },
    { 0x7be9a364, SUC_lexicon_tags_7 },
    { 0xebc9a365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3e9a367, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xb689a36a, SUC_lexicon_tags_16 },
    { 0xcde9a36b, SUC_lexicon_tags_37 },
    { 0x9129a36a, SUC_lexicon_tags_11 },
    { 0x6429a36d, SUC_lexicon_tags_37 },
    { 0xf329a36e, SUC_lexicon_tags_0 },
    { 0xabc9a36f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5e9a371, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19e9a373, SUC_lexicon_tags_2 },
    { 0x3a9a374, SUC_lexicon_tags_27 },
    { 0x8fe9a375, SUC_lexicon_tags_31 },
    { 0x6769a373, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f69a379, SUC_lexicon_tags_25 },
    { 0x1b49a379, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb269a37c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c9a380, SUC_lexicon_tags_29 },
    { 0x71e9a381, SUC_lexicon_tags_1464 },
    { 0xc609a382, SUC_lexicon_tags_48 },
    { 0x3d29a383, SUC_lexicon_tags_10 },
    { 0xca29a384, SUC_lexicon_tags_17 },
    { 0xde29a383, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb309a387, SUC_lexicon_tags_19 },
    { 0x2549a388, SUC_lexicon_tags_15 },
    { 0x4589a389, SUC_lexicon_tags_215 },
    { 0x2a29a38a, SUC_lexicon_tags_66 },
    { 0x1109a38b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c9a38f, SUC_lexicon_tags_13 },
    { 0xec69a38f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8489a392, SUC_lexicon_tags_62 },
    { 0xe689a393, SUC_lexicon_tags_2 },
    { 0xf9a9a394, SUC_lexicon_tags_4 },
    { 0xbe49a392, SUC_lexicon_tags_12 },
    { 0xf529a396, SUC_lexicon_tags_1 },
    { 0xa0c9a396, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4969a39e, SUC_lexicon_tags_14 },
    { 0x2b89a39e, SUC_lexicon_tags_31 },
    { 0x2089a39f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e9a3a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7649a3ac, SUC_lexicon_tags_17 },
    { 0x4949a3ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3109a3b1, SUC_lexicon_tags_37 },
    { 0x6a29a3b2, SUC_lexicon_tags_19 },
    { 0xcf69a3b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a49a3ba, SUC_lexicon_tags_8 },
    { 0xf0a9a3bb, SUC_lexicon_tags_25 },
    { 0x1829a3bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4de9a3bf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde9a3c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca9a3cf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9089a3d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc9a3de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfdc9a3e0, SUC_lexicon_tags_16 },
    { 0x6509a3e1, SUC_lexicon_tags_25 },
    { 0xf509a3e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5789a3e4, SUC_lexicon_tags_11 },
    { 0xc569a3e5, SUC_lexicon_tags_2 },
    { 0xf329a3e6, SUC_lexicon_tags_33 },
    { 0xc269a3e6, SUC_lexicon_tags_15 },
    { 0x3d29a3e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc089a3ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c9a3f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc329a3f3, SUC_lexicon_tags_31 },
    { 0xbb09a3f4, SUC_lexicon_tags_27 },
    { 0xc09a3f4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4449a3f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda09a3fc, SUC_lexicon_tags_6 },
    { 0x76a9a3fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4089a3ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f49a401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6829a404, SUC_lexicon_tags_7 },
    { 0xae89a405, SUC_lexicon_tags_35 },
    { 0x2de9a406, SUC_lexicon_tags_4 },
    { 0xfb69a407, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8689a409, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d69a40d, SUC_lexicon_tags_63 },
    { 0x9b49a40e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5da9a413, SUC_lexicon_tags_17 },
    { 0x9709a414, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8829a416, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4669a41b, SUC_lexicon_tags_97 },
    { 0x91c9a41b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda9a424, SUC_lexicon_tags_2 },
    { 0xbe09a425, SUC_lexicon_tags_1 },
    { 0x8a09a426, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2ce9a428, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb89a42e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa9a431, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfbc9a433, SUC_lexicon_tags_38 },
    { 0xfb69a434, SUC_lexicon_tags_11 },
    { 0x98e9a433, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x8a09a437, SUC_lexicon_tags_37 },
    { 0xc549a438, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fe9a43c, SUC_lexicon_tags_11 },
    { 0xd7e9a43c, SUC_lexicon_tags_37 },
    { 0x51a9a43c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb409a441, SUC_lexicon_tags_87 },
    { 0xae09a442, SUC_lexicon_tags_2 },
    { 0x50c9a443, SUC_lexicon_tags_67 },
    { 0x9e9a443, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8829a446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5649a448, SUC_lexicon_tags_63 },
    { 0xf889a449, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2729a452, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x5549a455, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce9a457, SUC_lexicon_tags_49 },
    { 0x6289a457, SUC_lexicon_tags_8 },
    { 0x77a9a459, SUC_lexicon_tags_28 },
    { 0x6309a459, SUC_lexicon_tags_19 },
    { 0x129a45b, SUC_lexicon_tags_11 },
    { 0x3929a45c, SUC_lexicon_tags_2 },
    { 0xcec9a45b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d89a462, SUC_lexicon_tags_50 },
    { 0x229a463, SUC_lexicon_tags_11 },
    { 0x2069a463, SUC_lexicon_tags_12 },
    { 0x6829a465, SUC_lexicon_tags_32 },
    { 0x28e9a466, SUC_lexicon_tags_17 },
    { 0xdf69a467, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d49a46c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6409a46f, SUC_lexicon_tags_7 },
    { 0x51c9a470, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94a9a472, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e9a476, SUC_lexicon_tags_11 },
    { 0x5229a477, SUC_lexicon_tags_62 },
    { 0x9069a477, SUC_lexicon_tags_68 },
    { 0xd909a477, SUC_lexicon_tags_31 },
    { 0x7849a47a, SUC_lexicon_tags_35 },
    { 0x1769a47a, SUC_lexicon_tags_17 },
    { 0xf369a47b, SUC_lexicon_tags_31 },
    { 0xa229a47b, SUC_lexicon_tags_152 },
    { 0x6429a47e, SUC_lexicon_tags_8 },
    { 0x70c9a47f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xade9a481, SUC_lexicon_tags_15 },
    { 0x3e89a481, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce09a485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d49a487, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a9a48d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6409a490, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6e9a492, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd669a49f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3169a4a2, SUC_lexicon_tags_19 },
    { 0x9009a4a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9a4a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3329a4ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd829a4ae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2849a4b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369a4ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x929a4bc, SUC_lexicon_tags_7 },
    { 0xca9a4bc, SUC_lexicon_tags_29 },
    { 0x4a69a4be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb549a4c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e49a4c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a9a4c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb909a4d1, SUC_lexicon_tags_35 },
    { 0xc189a4d1, SUC_lexicon_tags_12 },
    { 0x7829a4d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a9a4d6, SUC_lexicon_tags_7 },
    { 0x8649a4d7, SUC_lexicon_tags_14 },
    { 0xf669a4d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f09a4db, SUC_lexicon_tags_25 },
    { 0xcf09a4db, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xc8e9a4de, SUC_lexicon_tags_11 },
    { 0x3309a4df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd49a4e3, SUC_lexicon_tags_1089 },
    { 0xb7c9a4e4, SUC_lexicon_tags_70 },
    { 0x3049a4e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf529a4e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f09a4f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c09a4f9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc369a4fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb849a4fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c9a501, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae09a504, SUC_lexicon_tags_29 },
    { 0x7b89a505, SUC_lexicon_tags_26 },
    { 0xa409a506, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b49a50a, SUC_lexicon_tags_30 },
    { 0x56a9a50a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e9a50e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37c9a510, SUC_lexicon_tags_37 },
    { 0x1029a511, SUC_lexicon_tags_1 },
    { 0x11a9a512, SUC_lexicon_tags_2 },
    { 0x8a89a511, SUC_lexicon_tags_37 },
    { 0xd1c9a514, SUC_lexicon_tags_27 },
    { 0xa8a9a514, SUC_lexicon_tags_31 },
    { 0xecc9a513, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c9a519, SUC_lexicon_tags_23 },
    { 0xb289a51a, SUC_lexicon_tags_13 },
    { 0xd329a519, SUC_lexicon_tags_0 },
    { 0xe9a51c, SUC_lexicon_tags_11 },
    { 0x3549a51c, SUC_lexicon_tags_2 },
    { 0x3e29a51e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc049a521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d29a526, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c69a52b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa29a530, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x609a532, SUC_lexicon_tags_11 },
    { 0xd2e9a533, SUC_lexicon_tags_7 },
    { 0xab69a532, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6029a53a, SUC_lexicon_tags_37 },
    { 0x2e29a53b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7949a53e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9b69a540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89a544, SUC_lexicon_tags_17 },
    { 0xe769a545, SUC_lexicon_tags_194 },
    { 0x89a544, SUC_lexicon_tags_7 },
    { 0xf609a547, SUC_lexicon_tags_39 },
    { 0x66a9a548, SUC_lexicon_tags_15 },
    { 0x9109a544, SUC_lexicon_tags_21 },
    { 0xa369a549, SUC_lexicon_tags_17 },
    { 0xd589a54b, SUC_lexicon_tags_68 },
    { 0xb4c9a54c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9a54f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa9a552, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14e9a554, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e9a557, SUC_lexicon_tags_11 },
    { 0x7c29a558, SUC_lexicon_tags_17 },
    { 0xaa29a558, SUC_lexicon_tags_0 },
    { 0x7929a55a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9a55d, SUC_lexicon_tags_25 },
    { 0x8149a55e, SUC_lexicon_tags_61 },
    { 0x9a69a55e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8309a566, SUC_lexicon_tags_37 },
    { 0x8a49a567, SUC_lexicon_tags_45 },
    { 0x90e9a568, SUC_lexicon_tags_33 },
    { 0xb969a569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc809a56d, SUC_lexicon_tags_2 },
    { 0x3b89a56d, SUC_lexicon_tags_60 },
    { 0x3ca9a56e, SUC_lexicon_tags_17 },
    { 0xe849a56e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15e9a572, SUC_lexicon_tags_8 },
    { 0xa909a573, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7d69a575, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9a583, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61e9a585, SUC_lexicon_tags_37 },
    { 0x6729a586, SUC_lexicon_tags_17 },
    { 0xd469a587, SUC_lexicon_tags_29 },
    { 0x8589a588, SUC_lexicon_tags_8 },
    { 0x93c9a589, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe969a58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb809a58f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa589a591, SUC_lexicon_tags_14 },
    { 0xab49a592, SUC_lexicon_tags_49 },
    { 0x37a9a593, SUC_lexicon_tags_37 },
    { 0x8589a594, SUC_lexicon_tags_98 },
    { 0x5189a595, SUC_lexicon_tags_47 },
    { 0x6ae9a596, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x97e9a598, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e49a59a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749a5a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1e9a5a3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a69a5a7, SUC_lexicon_tags_2 },
    { 0x9049a5a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24a9a5aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71e9a5ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x88c9a5af, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8b29a5b1, SUC_lexicon_tags_37 },
    { 0xe089a5b1, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xd549a5b4, SUC_lexicon_tags_1 },
    { 0xf0c9a5b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6389a5b9, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0xe489a5bc, SUC_lexicon_tags_11 },
    { 0x2289a5bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c9a5c1, SUC_lexicon_tags_30 },
    { 0x42e9a5c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f89a5c4, SUC_lexicon_tags_27 },
    { 0xd729a5c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2169a5c9, SUC_lexicon_tags_12 },
    { 0x2fe9a5ca, SUC_lexicon_tags_26 },
    { 0x7809a5cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4a49a5cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b49a5cf, SUC_lexicon_tags_37 },
    { 0x94c9a5cf, SUC_lexicon_tags_7 },
    { 0xed49a5d0, SUC_lexicon_tags_8 },
    { 0x1ec9a5d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc909a5d4, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe709a5df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe329a5e3, SUC_lexicon_tags_17 },
    { 0x2009a5e4, SUC_lexicon_tags_17 },
    { 0x9829a5e3, SUC_lexicon_tags_47 },
    { 0xe7a9a5e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae9a5ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a9a5f1, SUC_lexicon_tags_64 },
    { 0xf329a5f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2309a5f4, SUC_lexicon_tags_1037 },
    { 0xf309a5f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4649a5f8, SUC_lexicon_tags_16 },
    { 0xed29a5f8, SUC_lexicon_tags_49 },
    { 0x69e9a5fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8769a5fe, SUC_lexicon_tags_15 },
    { 0x4a69a5fe, SUC_lexicon_tags_17 },
    { 0xb1c9a600, SUC_lexicon_tags_100 },
    { 0x1689a601, SUC_lexicon_tags_17 },
    { 0x7f49a602, SUC_lexicon_tags_9 },
    { 0xeb89a603, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf9e9a605, SUC_lexicon_tags_77 },
    { 0xdbc9a606, SUC_lexicon_tags_60 },
    { 0xda49a606, SUC_lexicon_tags_12 },
    { 0xec09a608, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8249a60b, SUC_lexicon_tags_11 },
    { 0x4de9a60b, SUC_lexicon_tags_39 },
    { 0x1a9a60d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf29a620, SUC_lexicon_tags_12 },
    { 0x8c69a621, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c09a623, SUC_lexicon_tags_95 },
    { 0xcae9a623, SUC_lexicon_tags_10 },
    { 0xb8e9a625, SUC_lexicon_tags_37 },
    { 0x2c09a623, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e9a62c, SUC_lexicon_tags_60 },
    { 0x14c9a62d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e9a630, SUC_lexicon_tags_17 },
    { 0xd929a630, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcda9a633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5209a635, SUC_lexicon_tags_65 },
    { 0xd0e9a636, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f69a639, SUC_lexicon_tags_37 },
    { 0x64a9a63a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe169a63c, SUC_lexicon_tags_15 },
    { 0xc809a63d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc769a641, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4349a643, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa9a648, SUC_lexicon_tags_43 },
    { 0xccc9a648, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2be9a64b, SUC_lexicon_tags_37 },
    { 0x15e9a64c, SUC_lexicon_tags_37 },
    { 0x8f69a64d, SUC_lexicon_tags_37 },
    { 0x7ea9a64e, SUC_lexicon_tags_26 },
    { 0x6589a64e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1909a652, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba9a656, SUC_lexicon_tags_37 },
    { 0x3709a656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6929a659, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7d69a65b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e9a65e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e69a664, SUC_lexicon_tags_45 },
    { 0xb069a665, SUC_lexicon_tags_43 },
    { 0x589a666, SUC_lexicon_tags_17 },
    { 0xad49a664, SUC_lexicon_tags_3 },
    { 0xd4c9a666, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5fe9a66a, SUC_lexicon_tags_7 },
    { 0xb5c9a66a, SUC_lexicon_tags_12 },
    { 0xc209a66b, SUC_lexicon_tags_82 },
    { 0xff09a66d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe929a670, SUC_lexicon_tags_47 },
    { 0x8489a671, SUC_lexicon_tags_2 },
    { 0x4649a671, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5f69a674, SUC_lexicon_tags_87 },
    { 0xd789a674, SUC_lexicon_tags_34 },
    { 0xbce9a675, SUC_lexicon_tags_2 },
    { 0xdbe9a676, SUC_lexicon_tags_37 },
    { 0x1709a678, SUC_lexicon_tags_30 },
    { 0x4509a677, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe569a67c, SUC_lexicon_tags_37 },
    { 0xea89a67d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1269a67f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc969a682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf209a686, SUC_lexicon_tags_25 },
    { 0x1169a686, SUC_lexicon_tags_0 },
    { 0x7449a686, SUC_lexicon_tags_45 },
    { 0xf1a9a686, SUC_lexicon_tags_28 },
    { 0x6d89a68a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e9a68e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7d89a690, SUC_lexicon_tags_1 },
    { 0x9e09a691, SUC_lexicon_tags_49 },
    { 0x73a9a691, SUC_lexicon_tags_39 },
    { 0x59e9a693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3829a698, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b69a69a, SUC_lexicon_tags_2 },
    { 0x4089a69b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3369a69d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe29a6a1, SUC_lexicon_tags_8 },
    { 0x8b29a6a2, SUC_lexicon_tags_43 },
    { 0xe2a9a6a1, SUC_lexicon_tags_11 },
    { 0x28a9a6a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2e9a6a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee9a6b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe5e9a6b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5889a6bc, SUC_lexicon_tags_158 },
    { 0x67e9a6bd, SUC_lexicon_tags_25 },
    { 0xe529a6bd, SUC_lexicon_tags_12 },
    { 0xa869a6bf, SUC_lexicon_tags_15 },
    { 0xcd49a6c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4029a6c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c49a6cc, SUC_lexicon_tags_17 },
    { 0x6329a6cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8509a6cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a6d3, SUC_lexicon_tags_15 },
    { 0xbd69a6d4, SUC_lexicon_tags_14 },
    { 0xf489a6d4, SUC_lexicon_tags_2 },
    { 0xbb29a6d3, SUC_lexicon_tags_34 },
    { 0x149a6d7, SUC_lexicon_tags_19 },
    { 0xf309a6d5, SUC_lexicon_tags_38 },
    { 0xe3a9a6d9, SUC_lexicon_tags_45 },
    { 0xa829a6d7, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x6c69a6dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa929a6de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb249a6e1, SUC_lexicon_tags_14 },
    { 0x62e9a6e2, SUC_lexicon_tags_24 },
    { 0x5b29a6e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfc29a6e5, SUC_lexicon_tags_43 },
    { 0xfd89a6e6, SUC_lexicon_tags_10 },
    { 0x3829a6e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de9a6eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84a9a6ee, SUC_lexicon_tags_49 },
    { 0x35e9a6ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade9a6f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe789a6fb, SUC_lexicon_tags_30 },
    { 0xd149a6fc, SUC_lexicon_tags_7 },
    { 0xaf69a6fd, SUC_lexicon_tags_11 },
    { 0x2309a6fe, SUC_lexicon_tags_43 },
    { 0xcd89a6fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6709a701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a29a703, SUC_lexicon_tags_0 },
    { 0x50c9a704, SUC_lexicon_tags_9 },
    { 0x8209a705, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa309a70a, SUC_lexicon_tags_17 },
    { 0xeb49a70b, SUC_lexicon_tags_26 },
    { 0x48a9a70c, SUC_lexicon_tags_12 },
    { 0xde69a70a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649a713, SUC_lexicon_tags_8 },
    { 0xedc9a713, SUC_lexicon_tags_38 },
    { 0xe149a715, SUC_lexicon_tags_29 },
    { 0x269a715, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa189a719, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba9a71c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6429a71e, SUC_lexicon_tags_25 },
    { 0xd8a9a71e, SUC_lexicon_tags_17 },
    { 0x48e9a71e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e9a724, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c29a72a, SUC_lexicon_tags_37 },
    { 0xa389a72b, SUC_lexicon_tags_25 },
    { 0x15a9a72c, SUC_lexicon_tags_37 },
    { 0xb769a72d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4049a72f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd69a731, SUC_lexicon_tags_38 },
    { 0x3bc9a732, SUC_lexicon_tags_9 },
    { 0x8549a732, SUC_lexicon_tags_15 },
    { 0xedc9a734, SUC_lexicon_tags_15 },
    { 0x1a89a735, SUC_lexicon_tags_0 },
    { 0x8229a735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1969a73a, SUC_lexicon_tags_17 },
    { 0xbcc9a73a, SUC_lexicon_tags_4 },
    { 0x26c9a73b, SUC_lexicon_tags_7 },
    { 0x7549a73c, SUC_lexicon_tags_17 },
    { 0xbb29a73a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2fa9a740, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c9a743, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9409a745, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c9a749, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xee69a74b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd909a74d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8329a753, SUC_lexicon_tags_37 },
    { 0x7249a753, SUC_lexicon_tags_64 },
    { 0x6f49a753, SUC_lexicon_tags_33 },
    { 0x2cc9a756, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a9a75a, SUC_lexicon_tags_63 },
    { 0xd1e9a75b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x65c9a75d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xac29a760, SUC_lexicon_tags_2 },
    { 0xc2c9a761, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc029a76d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x2e9a76f, SUC_lexicon_tags_17 },
    { 0x58e9a76f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfec9a772, SUC_lexicon_tags_37 },
    { 0x9f49a773, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2549a776, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75a9a778, SUC_lexicon_tags_4 },
    { 0x3a9a778, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a89a77d, SUC_lexicon_tags_19 },
    { 0x75a9a77e, SUC_lexicon_tags_9 },
    { 0x8fc9a77e, SUC_lexicon_tags_65 },
    { 0x5509a77f, SUC_lexicon_tags_77 },
    { 0x1429a781, SUC_lexicon_tags_25 },
    { 0x10a9a782, SUC_lexicon_tags_17 },
    { 0x33e9a781, SUC_lexicon_tags_25 },
    { 0x4e29a784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ec9a786, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc9a78a, SUC_lexicon_tags_6 },
    { 0x5729a78b, SUC_lexicon_tags_25 },
    { 0x5e89a78c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c9a790, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ca9a792, SUC_lexicon_tags_37 },
    { 0xb949a793, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c9a799, SUC_lexicon_tags_27 },
    { 0x8189a79a, SUC_lexicon_tags_25 },
    { 0x7ee9a799, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe869a7a1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89a7a4, SUC_lexicon_tags_63 },
    { 0x2409a7a5, SUC_lexicon_tags_19 },
    { 0xb229a7a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2549a7a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa9a7ad, SUC_lexicon_tags_17 },
    { 0x90e9a7ae, SUC_lexicon_tags_17 },
    { 0x7fe9a7af, SUC_lexicon_tags_60 },
    { 0x3849a7af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc9a9a7b2, SUC_lexicon_tags_33 },
    { 0x8249a7b3, SUC_lexicon_tags_64 },
    { 0x81a9a7b4, SUC_lexicon_tags_2 },
    { 0xfe89a7b5, SUC_lexicon_tags_13 },
    { 0xf629a7b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf969a7ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde9a7c0, SUC_lexicon_tags_16 },
    { 0x6c49a7c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2029a7c3, SUC_lexicon_tags_15 },
    { 0xcc49a7c4, SUC_lexicon_tags_0 },
    { 0x3389a7c3, SUC_lexicon_tags_37 },
    { 0xa5a9a7c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9a7cc, SUC_lexicon_tags_16 },
    { 0x3b29a7cc, SUC_lexicon_tags_30 },
    { 0xd0e9a7cd, SUC_lexicon_tags_0 },
    { 0x3689a7cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe189a7d4, SUC_lexicon_tags_0 },
    { 0xf5c9a7d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe689a7d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6069a7e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f09a7ed, SUC_lexicon_tags_19 },
    { 0x89a7ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5549a7f3, SUC_lexicon_tags_11 },
    { 0x8f69a7f3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb49a7f6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f69a7fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe929a7fe, SUC_lexicon_tags_13 },
    { 0xa349a7fe, SUC_lexicon_tags_25 },
    { 0x5749a800, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6949a802, SUC_lexicon_tags_66 },
    { 0xe6c9a803, SUC_lexicon_tags_65 },
    { 0x1569a804, SUC_lexicon_tags_220 },
    { 0x84a9a803, SUC_lexicon_tags_11 },
    { 0x5709a806, SUC_lexicon_tags_1 },
    { 0xc569a806, SUC_lexicon_tags_82 },
    { 0x6049a802, SUC_lexicon_tags_35 },
    { 0x2d29a808, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee9a80c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e89a80e, SUC_lexicon_tags_11 },
    { 0x26c9a80f, SUC_lexicon_tags_15 },
    { 0xdbe9a810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa89a814, SUC_lexicon_tags_79 },
    { 0x36e9a815, SUC_lexicon_tags_9 },
    { 0x8209a816, SUC_lexicon_tags_179 },
    { 0x6ac9a814, SUC_lexicon_tags_38 },
    { 0x6f09a818, SUC_lexicon_tags_12 },
    { 0x269a819, SUC_lexicon_tags_9 },
    { 0x2c69a819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9309a81c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb949a81e, SUC_lexicon_tags_61 },
    { 0x1129a81f, SUC_lexicon_tags_46 },
    { 0x269a81f, SUC_lexicon_tags_15 },
    { 0xbe9a821, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249a82a, SUC_lexicon_tags_12 },
    { 0x14c9a82b, SUC_lexicon_tags_46 },
    { 0x3b69a82b, SUC_lexicon_tags_9 },
    { 0xf5c9a82d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xece9a831, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a83c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae9a840, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c9a844, SUC_lexicon_tags_68 },
    { 0xd509a845, SUC_lexicon_tags_37 },
    { 0x989a844, SUC_lexicon_tags_37 },
    { 0x5a09a847, SUC_lexicon_tags_34 },
    { 0xa7c9a847, SUC_lexicon_tags_33 },
    { 0x5029a848, SUC_lexicon_tags_27 },
    { 0xa5a9a844, SUC_lexicon_tags_17 },
    { 0xdda9a84b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca9a84f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b69a856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf49a85c, SUC_lexicon_tags_48 },
    { 0xea89a85c, SUC_lexicon_tags_29 },
    { 0xfee9a85c, SUC_lexicon_tags_48 },
    { 0x50a9a85f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6649a862, SUC_lexicon_tags_14 },
    { 0x3c49a863, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa9a867, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcda9a869, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6529a86b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e9a86e, SUC_lexicon_tags_1 },
    { 0xf3c9a86f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5569a871, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a9a874, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c9a877, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4c29a879, SUC_lexicon_tags_31 },
    { 0x6389a87a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd169a87d, SUC_lexicon_tags_2 },
    { 0x8e09a87d, SUC_lexicon_tags_12 },
    { 0xa789a87f, SUC_lexicon_tags_60 },
    { 0x13c9a87d, SUC_lexicon_tags_29 },
    { 0xbda9a881, SUC_lexicon_tags_38 },
    { 0xfa09a87d, SUC_lexicon_tags_47 },
    { 0x8209a883, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e89a886, SUC_lexicon_tags_9 },
    { 0xb069a886, SUC_lexicon_tags_166 },
    { 0x9da9a887, SUC_lexicon_tags_35 },
    { 0xfce9a889, SUC_lexicon_tags_47 },
    { 0xde69a88a, SUC_lexicon_tags_14 },
    { 0xd689a88b, SUC_lexicon_tags_15 },
    { 0x6209a887, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd849a88f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3f09a891, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xaac9a893, SUC_lexicon_tags_0 },
    { 0x4a29a894, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5049a896, SUC_lexicon_tags_27 },
    { 0x3e69a896, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a9a89a, SUC_lexicon_tags_21 },
    { 0xf29a89b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xddc9a89d, SUC_lexicon_tags_39 },
    { 0x4729a89e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb509a8a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9149a8a7, SUC_lexicon_tags_12 },
    { 0x7f89a8a7, SUC_lexicon_tags_60 },
    { 0xc6c9a8a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63c9a8ab, SUC_lexicon_tags_15 },
    { 0xb5a9a8ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8129a8b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb29a8b5, SUC_lexicon_tags_39 },
    { 0x61a9a8b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf169a8b8, SUC_lexicon_tags_11 },
    { 0x1da9a8b9, SUC_lexicon_tags_49 },
    { 0x28e9a8ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce9a8bc, SUC_lexicon_tags_0 },
    { 0xe09a8bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6929a8c0, SUC_lexicon_tags_15 },
    { 0x89a8c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d29a8c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9a8c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a9a8cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9109a8ce, SUC_lexicon_tags_1 },
    { 0xa6e9a8cf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7829a8d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa929a8d9, SUC_lexicon_tags_10 },
    { 0x77e9a8d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2929a8dd, SUC_lexicon_tags_32 },
    { 0x249a8de, SUC_lexicon_tags_60 },
    { 0xfaa9a8df, SUC_lexicon_tags_63 },
    { 0xfdc9a8df, SUC_lexicon_tags_12 },
    { 0x2689a8dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5229a8e3, SUC_lexicon_tags_13 },
    { 0xae29a8e4, SUC_lexicon_tags_15 },
    { 0x1fe9a8e4, SUC_lexicon_tags_0 },
    { 0xd09a8e4, SUC_lexicon_tags_16 },
    { 0xb029a8e7, SUC_lexicon_tags_12 },
    { 0xb609a8e8, SUC_lexicon_tags_28 },
    { 0x1209a8e8, SUC_lexicon_tags_45 },
    { 0x8169a8e3, SUC_lexicon_tags_17 },
    { 0x4909a8eb, SUC_lexicon_tags_15 },
    { 0xe589a8ec, SUC_lexicon_tags_28 },
    { 0x6189a8ec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc9a8f2, SUC_lexicon_tags_37 },
    { 0x21e9a8f3, SUC_lexicon_tags_17 },
    { 0xa409a8f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c9a8f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f29a8fb, SUC_lexicon_tags_37 },
    { 0x49a9a8fc, SUC_lexicon_tags_25 },
    { 0x1d49a8fb, SUC_lexicon_tags_7 },
    { 0x7389a8fc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec9a901, SUC_lexicon_tags_25 },
    { 0xb9a9a901, SUC_lexicon_tags_15 },
    { 0x3609a903, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14e9a905, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2c9a907, SUC_lexicon_tags_19 },
    { 0x7049a907, SUC_lexicon_tags_55 },
    { 0xcdc9a909, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x29e9a90b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ea9a90d, SUC_lexicon_tags_82 },
    { 0x7329a90d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6229a912, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a9a915, SUC_lexicon_tags_25 },
    { 0x3389a916, SUC_lexicon_tags_37 },
    { 0xc9a9a915, SUC_lexicon_tags_63 },
    { 0x1c9a915, SUC_lexicon_tags_48 },
    { 0xd9c9a915, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a9a923, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee9a926, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x20c9a928, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7029a92a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a9a92e, SUC_lexicon_tags_20 },
    { 0x1649a92e, SUC_lexicon_tags_6 },
    { 0x3b09a930, SUC_lexicon_tags_17 },
    { 0x9809a92e, SUC_lexicon_tags_38 },
    { 0x6ee9a932, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e9a935, SUC_lexicon_tags_28 },
    { 0xf349a936, SUC_lexicon_tags_12 },
    { 0xb469a936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51a9a939, SUC_lexicon_tags_37 },
    { 0xb789a93a, SUC_lexicon_tags_57 },
    { 0x2b29a939, SUC_lexicon_tags_15 },
    { 0x3829a93b, SUC_lexicon_tags_7 },
    { 0x6349a93d, SUC_lexicon_tags_11 },
    { 0x6589a93e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2129a945, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7249a94a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e29a94f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8e9a951, SUC_lexicon_tags_39 },
    { 0xaa69a952, SUC_lexicon_tags_11 },
    { 0x9ba9a953, SUC_lexicon_tags_0 },
    { 0xd729a952, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae9a957, SUC_lexicon_tags_1 },
    { 0x3ea9a957, SUC_lexicon_tags_8 },
    { 0xe469a958, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa9a95e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289a962, SUC_lexicon_tags_23 },
    { 0x1f49a962, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9d89a965, SUC_lexicon_tags_3 },
    { 0xcf89a965, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x769a969, SUC_lexicon_tags_17 },
    { 0xfb49a969, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8109a96e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e9a973, SUC_lexicon_tags_12 },
    { 0x4e29a974, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8929a976, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9a979, SUC_lexicon_tags_11 },
    { 0xc569a97a, SUC_lexicon_tags_2 },
    { 0xb769a97a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a89a981, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf029a983, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7529a986, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b89a98b, SUC_lexicon_tags_17 },
    { 0xa29a98b, SUC_lexicon_tags_45 },
    { 0x609a98d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a9a993, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x13e9a995, SUC_lexicon_tags_7 },
    { 0x1f69a996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec9a99a, SUC_lexicon_tags_15 },
    { 0xb189a99b, SUC_lexicon_tags_7 },
    { 0xf5e9a99a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad89a99f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1609a9a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb29a9a6, SUC_lexicon_tags_17 },
    { 0xd789a9a7, SUC_lexicon_tags_37 },
    { 0x5749a9a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd489a9ad, SUC_lexicon_tags_29 },
    { 0xfb09a9ae, SUC_lexicon_tags_15 },
    { 0x2829a9af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e9a9b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf249a9b4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce89a9b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc369a9bc, SUC_lexicon_tags_11 },
    { 0xd1a9a9bd, SUC_lexicon_tags_37 },
    { 0xf3a9a9be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca9a9c1, SUC_lexicon_tags_4 },
    { 0x5a69a9c2, SUC_lexicon_tags_19 },
    { 0x4f69a9c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e89a9c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44a9a9c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5649a9ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8be9a9cc, SUC_lexicon_tags_25 },
    { 0x18a9a9cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc269a9d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc709a9d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8469a9db, SUC_lexicon_tags_19 },
    { 0x8089a9dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf349a9de, SUC_lexicon_tags_1 },
    { 0xd6a9a9df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e29a9e1, SUC_lexicon_tags_11 },
    { 0xb729a9e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2469a9e7, SUC_lexicon_tags_11 },
    { 0x64e9a9e7, SUC_lexicon_tags_7 },
    { 0xd9e9a9e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a9a9ec, SUC_lexicon_tags_43 },
    { 0x8ee9a9ed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9a9f1, SUC_lexicon_tags_32 },
    { 0xd4a9a9f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c9a9f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x54a9a9fb, SUC_lexicon_tags_61 },
    { 0xd2e9a9fb, SUC_lexicon_tags_25 },
    { 0xb189a9fb, SUC_lexicon_tags_29 },
    { 0xb0a9a9fb, SUC_lexicon_tags_7 },
    { 0xc349a9ff, SUC_lexicon_tags_49 },
    { 0xc929aa00, SUC_lexicon_tags_16 },
    { 0xb2c9aa01, SUC_lexicon_tags_7 },
    { 0x6649aa01, SUC_lexicon_tags_11 },
    { 0xe6c9aa03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6729aa0a, SUC_lexicon_tags_19 },
    { 0x1729aa0b, SUC_lexicon_tags_11 },
    { 0xe109aa0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2589aa0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc149aa17, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa629aa1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf289aa1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ea9aa1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a9aa25, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7249aa27, SUC_lexicon_tags_15 },
    { 0x1a49aa27, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x769aa2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c9aa2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53c9aa30, SUC_lexicon_tags_27 },
    { 0x7a09aa31, SUC_lexicon_tags_1 },
    { 0x3969aa32, SUC_lexicon_tags_23 },
    { 0xc409aa33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2e9aa35, SUC_lexicon_tags_17 },
    { 0x85c9aa36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3869aa3a, SUC_lexicon_tags_7 },
    { 0x97a9aa3b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c9aa3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f89aa43, SUC_lexicon_tags_11 },
    { 0x2829aa44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ae9aa46, SUC_lexicon_tags_12 },
    { 0x13a9aa46, SUC_lexicon_tags_2 },
    { 0x5689aa47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea9aa4c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c9aa4f, SUC_lexicon_tags_60 },
    { 0xd229aa4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbe9aa52, SUC_lexicon_tags_11 },
    { 0xff49aa52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfa9aa55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9aa58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9aa60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78e9aa62, SUC_lexicon_tags_14 },
    { 0xf089aa63, SUC_lexicon_tags_14 },
    { 0x7da9aa64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c29aa66, SUC_lexicon_tags_25 },
    { 0x8a49aa67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3689aa72, SUC_lexicon_tags_39 },
    { 0x7d09aa72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa789aa76, SUC_lexicon_tags_12 },
    { 0xf749aa77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a9aa7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd49aa87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7949aa8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c89aa8e, SUC_lexicon_tags_14 },
    { 0xaba9aa8f, SUC_lexicon_tags_33 },
    { 0x47c9aa90, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x39a9aa92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e09aa95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc9aaa1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ca9aaa3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ea9aaa5, SUC_lexicon_tags_2 },
    { 0xda09aaa6, SUC_lexicon_tags_43 },
    { 0x6dc9aaa7, SUC_lexicon_tags_7 },
    { 0xaa09aaa7, SUC_lexicon_tags_16 },
    { 0xa169aaa9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3209aaab, SUC_lexicon_tags_47 },
    { 0xa6c9aaac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3b09aaae, SUC_lexicon_tags_48 },
    { 0xca9aaae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe9aab3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea29aab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf449aab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72c9aabd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b49aac0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa609aac4, SUC_lexicon_tags_2 },
    { 0xc9aac5, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc49aac9, SUC_lexicon_tags_1 },
    { 0x7c89aac9, SUC_lexicon_tags_49 },
    { 0x5809aacb, SUC_lexicon_tags_66 },
    { 0x4a89aacc, SUC_lexicon_tags_2 },
    { 0xa7e9aacc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a9aad0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9aad4, SUC_lexicon_tags_17 },
    { 0x9ce9aad5, SUC_lexicon_tags_59 },
    { 0xa5c9aad6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e9aadd, SUC_lexicon_tags_26 },
    { 0xcec9aade, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2229aae1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3e89aae3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9aae7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe929aae9, SUC_lexicon_tags_37 },
    { 0xc589aaea, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a9aaed, SUC_lexicon_tags_68 },
    { 0x8f29aaed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e69aaf1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x35c9aaf3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc49aafc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc9aaff, SUC_lexicon_tags_17 },
    { 0xcbe9ab00, SUC_lexicon_tags_25 },
    { 0x289ab01, SUC_lexicon_tags_33 },
    { 0xb949ab02, SUC_lexicon_tags_49 },
    { 0x20c9aaff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e09ab05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89ab0b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a9ab13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a9ab18, SUC_lexicon_tags_17 },
    { 0xae09ab19, SUC_lexicon_tags_23 },
    { 0x69c9ab19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c9ab1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee9ab22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9ab25, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2f49ab27, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e9ab2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b69ab2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7429ab32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3da9ab34, SUC_lexicon_tags_14 },
    { 0x1349ab35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2569ab37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe889ab3e, SUC_lexicon_tags_19 },
    { 0xd569ab3f, SUC_lexicon_tags_47 },
    { 0x329ab3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1689ab43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7369ab45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee49ab4a, SUC_lexicon_tags_15 },
    { 0x55a9ab4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee9ab50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f89ab57, SUC_lexicon_tags_37 },
    { 0x6309ab57, SUC_lexicon_tags_81 },
    { 0x14c9ab58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb69ab60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe749ab68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x849ab6b, SUC_lexicon_tags_43 },
    { 0x9729ab6c, SUC_lexicon_tags_5 },
    { 0x9fc9ab6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa2a9ab6f, SUC_lexicon_tags_17 },
    { 0x1369ab6f, SUC_lexicon_tags_38 },
    { 0x5509ab71, SUC_lexicon_tags_11 },
    { 0xfe09ab70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b29ab84, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8089ab86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf989ab89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5429ab8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1c9ab8d, SUC_lexicon_tags_12 },
    { 0x3ea9ab8e, SUC_lexicon_tags_69 },
    { 0x73e9ab8f, SUC_lexicon_tags_80 },
    { 0x6449ab90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c9ab96, SUC_lexicon_tags_6 },
    { 0x8ca9ab97, SUC_lexicon_tags_1 },
    { 0x3229ab98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba9ab9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4409ab9e, SUC_lexicon_tags_15 },
    { 0x2e49ab9f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf029aba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4069aba3, SUC_lexicon_tags_60 },
    { 0xb509aba3, SUC_lexicon_tags_47 },
    { 0xa629aba5, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0xc6a9aba7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf7a9aba9, SUC_lexicon_tags_0 },
    { 0x3289abaa, SUC_lexicon_tags_15 },
    { 0x6f69abab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2909abae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1cc9abb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20c9abb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12c9abb4, SUC_lexicon_tags_2 },
    { 0x86c9abb4, SUC_lexicon_tags_16 },
    { 0xfa09abb6, SUC_lexicon_tags_1 },
    { 0x9a49abb7, SUC_lexicon_tags_33 },
    { 0xe009abb7, SUC_lexicon_tags_15 },
    { 0x6be9abb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a69abbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ce9abbf, SUC_lexicon_tags_47 },
    { 0x8229abbf, SUC_lexicon_tags_37 },
    { 0x9ce9abc0, SUC_lexicon_tags_37 },
    { 0x81e9abc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab09abc4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5ca9abc6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x37c9abc8, SUC_lexicon_tags_2 },
    { 0x9fe9abc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1609abcc, SUC_lexicon_tags_0 },
    { 0xf4a9abcc, SUC_lexicon_tags_10 },
    { 0x7fa9abcc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e69abd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf469abd6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40e9abd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a89abda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e9abe0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c9abe3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a09abe9, SUC_lexicon_tags_32 },
    { 0xaaa9abe9, SUC_lexicon_tags_68 },
    { 0x7fa9abea, SUC_lexicon_tags_9 },
    { 0xa469abec, SUC_lexicon_tags_12 },
    { 0xbc69abec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2669abef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc929abf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa309abf3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x19a9abf5, SUC_lexicon_tags_12 },
    { 0x8de9abf6, SUC_lexicon_tags_37 },
    { 0x2589abf7, SUC_lexicon_tags_7 },
    { 0x7de9abf7, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0x33e9abfa, SUC_lexicon_tags_15 },
    { 0x6a29abfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd969ac00, SUC_lexicon_tags_25 },
    { 0xbae9ac01, SUC_lexicon_tags_17 },
    { 0xbb69ac02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a9ac06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09ac09, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f49ac0d, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6209ac14, SUC_lexicon_tags_37 },
    { 0x2509ac15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8029ac17, SUC_lexicon_tags_25 },
    { 0xc4c9ac18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4929ac20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91c9ac22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5289ac29, SUC_lexicon_tags_37 },
    { 0x3369ac2a, SUC_lexicon_tags_28 },
    { 0x5bc9ac2b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5549ac2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad49ac35, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x3489ac37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be9ac3f, SUC_lexicon_tags_25 },
    { 0x4349ac40, SUC_lexicon_tags_11 },
    { 0x5dc9ac41, SUC_lexicon_tags_37 },
    { 0x25c9ac42, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef69ac47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e9ac4e, SUC_lexicon_tags_26 },
    { 0x1989ac4e, SUC_lexicon_tags_38 },
    { 0xf4e9ac4e, SUC_lexicon_tags_55 },
    { 0x37c9ac51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d29ac53, SUC_lexicon_tags_25 },
    { 0x9d69ac54, SUC_lexicon_tags_6 },
    { 0xed69ac55, SUC_lexicon_tags_27 },
    { 0xf1a9ac56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc9ac5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcee9ac5c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749ac64, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfb09ac66, SUC_lexicon_tags_17 },
    { 0xb989ac67, SUC_lexicon_tags_25 },
    { 0x4749ac67, SUC_lexicon_tags_14 },
    { 0xf489ac69, SUC_lexicon_tags_55 },
    { 0x2029ac66, SUC_lexicon_tags_32 },
    { 0xf1a9ac6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e9ac6e, SUC_lexicon_tags_61 },
    { 0x5c49ac6e, SUC_lexicon_tags_2 },
    { 0x7c29ac70, SUC_lexicon_tags_37 },
    { 0xc069ac71, SUC_lexicon_tags_28 },
    { 0x7409ac71, SUC_lexicon_tags_25 },
    { 0x669ac72, SUC_lexicon_tags_60 },
    { 0xfb49ac6f, SUC_lexicon_tags_37 },
    { 0x7869ac75, SUC_lexicon_tags_2 },
    { 0x2c89ac72, SUC_lexicon_tags_15 },
    { 0x3789ac77, SUC_lexicon_tags_6 },
    { 0x4d09ac78, SUC_lexicon_tags_30 },
    { 0x87a9ac77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8569ac80, SUC_lexicon_tags_17 },
    { 0xe5c9ac80, SUC_lexicon_tags_47 },
    { 0xdfc9ac82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e9ac87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1889ac8c, SUC_lexicon_tags_25 },
    { 0x7a9ac8d, SUC_lexicon_tags_60 },
    { 0x2349ac8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43a9ac90, SUC_lexicon_tags_7 },
    { 0xcc29ac91, SUC_lexicon_tags_17 },
    { 0xb869ac91, SUC_lexicon_tags_34 },
    { 0xc949ac93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a9ac9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bc9ac9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e9aca5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa289acac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc9acb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec9acb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6249acb7, SUC_lexicon_tags_30 },
    { 0x1a69acb8, SUC_lexicon_tags_7 },
    { 0x6fa9acb8, SUC_lexicon_tags_25 },
    { 0xdee9acb7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c49acc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f49acca, SUC_lexicon_tags_17 },
    { 0x8ae9acca, SUC_lexicon_tags_12 },
    { 0x3a9acca, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4729acce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2329acd5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xafe9acd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a89acd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74e9acdb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b09acdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce89ace7, SUC_lexicon_tags_56 },
    { 0xb3a9ace8, SUC_lexicon_tags_11 },
    { 0x2d89ace9, SUC_lexicon_tags_65 },
    { 0x7929ace9, SUC_lexicon_tags_46 },
    { 0x3429ace7, SUC_lexicon_tags_15 },
    { 0xb649acec, SUC_lexicon_tags_15 },
    { 0x2949ace8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x67c9acef, SUC_lexicon_tags_27 },
    { 0xda09acef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f29acf2, SUC_lexicon_tags_6 },
    { 0xbd29acf3, SUC_lexicon_tags_137 },
    { 0x6229acf2, SUC_lexicon_tags_13 },
    { 0x2b89acf4, SUC_lexicon_tags_1 },
    { 0x9e49acf2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2109acfc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3929ad00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7da9ad02, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x15e9ad04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f49ad0a, SUC_lexicon_tags_37 },
    { 0x1489ad0b, SUC_lexicon_tags_47 },
    { 0x1309ad0c, SUC_lexicon_tags_37 },
    { 0xc09ad0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb169ad15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf949ad19, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7709ad1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde9ad21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8709ad24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e9ad28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x42e9ad2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb2a9ad2c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a9ad2f, SUC_lexicon_tags_14 },
    { 0xbec9ad30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b69ad32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a9ad3a, SUC_lexicon_tags_1 },
    { 0x9d69ad3b, SUC_lexicon_tags_8 },
    { 0xf809ad3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c9ad40, SUC_lexicon_tags_6 },
    { 0x8269ad41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a9ad45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9729ad4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e89ad52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5629ad5a, SUC_lexicon_tags_15 },
    { 0xf1a9ad5a, SUC_lexicon_tags_38 },
    { 0x1069ad5c, SUC_lexicon_tags_15 },
    { 0x4be9ad5d, SUC_lexicon_tags_14 },
    { 0x91e9ad5e, SUC_lexicon_tags_9 },
    { 0xbfc9ad5e, SUC_lexicon_tags_23 },
    { 0x95e9ad60, SUC_lexicon_tags_27 },
    { 0xe1e9ad5e, SUC_lexicon_tags_60 },
    { 0x1c49ad62, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd389ad6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a89ad6c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e9ad70, SUC_lexicon_tags_15 },
    { 0x729ad71, SUC_lexicon_tags_40 },
    { 0x60c9ad72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc49ad76, SUC_lexicon_tags_43 },
    { 0xfc09ad77, SUC_lexicon_tags_33 },
    { 0x64e9ad78, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca9ad7f, SUC_lexicon_tags_61 },
    { 0x11a9ad80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c49ad83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf369ad85, SUC_lexicon_tags_15 },
    { 0x12e9ad85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea9ad92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb69ad95, SUC_lexicon_tags_9 },
    { 0x3b09ad96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d49ad9a, SUC_lexicon_tags_63 },
    { 0x1169ad9b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8709ad9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c9ada1, SUC_lexicon_tags_11 },
    { 0x1e49ada2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa69ada4, SUC_lexicon_tags_63 },
    { 0x85e9ada5, SUC_lexicon_tags_45 },
    { 0x8b89ada6, SUC_lexicon_tags_33 },
    { 0x3c09ada7, SUC_lexicon_tags_37 },
    { 0x5349ada4, SUC_lexicon_tags_25 },
    { 0x96a9ada4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e9adaf, SUC_lexicon_tags_11 },
    { 0xc589adaf, SUC_lexicon_tags_19 },
    { 0xc729adb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7be9adb3, SUC_lexicon_tags_48 },
    { 0xa629adb3, SUC_lexicon_tags_37 },
    { 0x2829adb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4109adb8, SUC_lexicon_tags_19 },
    { 0x9ba9adb9, SUC_lexicon_tags_11 },
    { 0xbe89adb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1dc9adbc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a09adbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2589adc1, SUC_lexicon_tags_41 },
    { 0x15a9adc2, SUC_lexicon_tags_60 },
    { 0x52a9adc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ba9adc5, SUC_lexicon_tags_13 },
    { 0x6169adc6, SUC_lexicon_tags_4 },
    { 0x7869adc5, SUC_lexicon_tags_65 },
    { 0x9e69adc8, SUC_lexicon_tags_91 },
    { 0x9309adc9, SUC_lexicon_tags_11 },
    { 0x2669adca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8f29adcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c9add2, SUC_lexicon_tags_12 },
    { 0xd8e9add3, SUC_lexicon_tags_1 },
    { 0x9bc9add4, SUC_lexicon_tags_39 },
    { 0x9749add2, SUC_lexicon_tags_70 },
    { 0x2789add6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1309addb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e29ade7, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x6ce9ade9, SUC_lexicon_tags_4 },
    { 0x6089adea, SUC_lexicon_tags_17 },
    { 0x5a9adea, SUC_lexicon_tags_0 },
    { 0xa489ade9, SUC_lexicon_tags_13 },
    { 0xdf89adea, SUC_lexicon_tags_17 },
    { 0xd049adeb, SUC_lexicon_tags_1 },
    { 0x1f89adef, SUC_lexicon_tags_68 },
    { 0x2c9adf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb549adf4, SUC_lexicon_tags_26 },
    { 0x89c9adf5, SUC_lexicon_tags_48 },
    { 0xe689adf5, SUC_lexicon_tags_17 },
    { 0x7e49adf7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe9adfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2029ae01, SUC_lexicon_tags_2 },
    { 0xc949ae02, SUC_lexicon_tags_49 },
    { 0xe909ae02, SUC_lexicon_tags_19 },
    { 0x68a9ae04, SUC_lexicon_tags_61 },
    { 0x8b09ae05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3549ae08, SUC_lexicon_tags_15 },
    { 0x72e9ae09, SUC_lexicon_tags_32 },
    { 0x129ae09, SUC_lexicon_tags_9 },
    { 0x56c9ae0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e9ae10, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3d29ae12, SUC_lexicon_tags_7 },
    { 0xb649ae13, SUC_lexicon_tags_11 },
    { 0xaba9ae13, SUC_lexicon_tags_61 },
    { 0x3189ae13, SUC_lexicon_tags_12 },
    { 0x2f09ae16, SUC_lexicon_tags_2 },
    { 0x39a9ae17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1309ae19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ac9ae1b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2d29ae1d, SUC_lexicon_tags_2 },
    { 0xd009ae1d, SUC_lexicon_tags_32 },
    { 0x5889ae1f, SUC_lexicon_tags_28 },
    { 0x61a9ae20, SUC_lexicon_tags_14 },
    { 0xdec9ae1f, SUC_lexicon_tags_29 },
    { 0xe269ae22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe089ae27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafc9ae29, SUC_lexicon_tags_37 },
    { 0x4e49ae29, SUC_lexicon_tags_1 },
    { 0x4689ae2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae9ae2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc9ae32, SUC_lexicon_tags_49 },
    { 0xf09ae32, SUC_lexicon_tags_17 },
    { 0xb489ae34, SUC_lexicon_tags_85 },
    { 0xfb89ae34, SUC_lexicon_tags_198 },
    { 0x1369ae36, SUC_lexicon_tags_70 },
    { 0x4a49ae37, SUC_lexicon_tags_17 },
    { 0xb129ae37, SUC_lexicon_tags_2 },
    { 0x7389ae37, SUC_lexicon_tags_11 },
    { 0xc2c9ae34, SUC_lexicon_tags_12 },
    { 0xf309ae3b, SUC_lexicon_tags_1 },
    { 0x5649ae3c, SUC_lexicon_tags_33 },
    { 0xbaa9ae3b, SUC_lexicon_tags_9 },
    { 0x3b49ae3b, SUC_lexicon_tags_37 },
    { 0x8829ae36, SUC_lexicon_tags_37 },
    { 0x8f29ae36, SUC_lexicon_tags_16 },
    { 0xd6a9ae35, SUC_lexicon_tags_6 },
    { 0x73c9ae42, SUC_lexicon_tags_27 },
    { 0x3ca9ae42, SUC_lexicon_tags_47 },
    { 0xd569ae35, SUC_lexicon_tags_17 },
    { 0x36c9ae33, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb189ae49, SUC_lexicon_tags_37 },
    { 0x7ec9ae4a, SUC_lexicon_tags_1 },
    { 0xc4a9ae4a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9649ae4e, SUC_lexicon_tags_34 },
    { 0x47a9ae4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ae53, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5609ae56, SUC_lexicon_tags_26 },
    { 0xe69ae57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2e9ae59, SUC_lexicon_tags_11 },
    { 0x97c9ae59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189ae5f, SUC_lexicon_tags_30 },
    { 0x549ae60, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c9ae65, SUC_lexicon_tags_11 },
    { 0xfc9ae65, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x48c9ae68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea49ae6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf609ae71, SUC_lexicon_tags_26 },
    { 0x6d89ae72, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa509ae74, SUC_lexicon_tags_26 },
    { 0xbcc9ae75, SUC_lexicon_tags_205 },
    { 0x74e9ae76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1389ae7b, SUC_lexicon_tags_21 },
    { 0xd989ae7c, SUC_lexicon_tags_17 },
    { 0xb8a9ae7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5769ae81, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9069ae83, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2669ae88, SUC_lexicon_tags_3 },
    { 0x86e9ae88, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c9ae93, SUC_lexicon_tags_11 },
    { 0xe409ae94, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba49ae9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3869ae9f, SUC_lexicon_tags_11 },
    { 0xfda9ae9f, SUC_lexicon_tags_15 },
    { 0x5a09aea1, SUC_lexicon_tags_37 },
    { 0xc929aea1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55c9aea7, SUC_lexicon_tags_43 },
    { 0x109aea8, SUC_lexicon_tags_44 },
    { 0xf6e9aea9, SUC_lexicon_tags_26 },
    { 0x2ee9aea8, SUC_lexicon_tags_1 },
    { 0xcb49aeaa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c9aeb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9629aeb3, SUC_lexicon_tags_1 },
    { 0x8ec9aeb4, SUC_lexicon_tags_38 },
    { 0xbf69aeb4, SUC_lexicon_tags_28 },
    { 0x7ec9aeb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfea9aeb9, SUC_lexicon_tags_1 },
    { 0xb849aeb9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f89aec2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae9aec5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf749aec9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca9aecf, SUC_lexicon_tags_5 },
    { 0x3849aecf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9729aed3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3669aed7, SUC_lexicon_tags_17 },
    { 0x9f89aed8, SUC_lexicon_tags_7 },
    { 0x8589aed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40a9aedb, SUC_lexicon_tags_24 },
    { 0xa4e9aedb, SUC_lexicon_tags_79 },
    { 0x4589aedd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a9aee0, SUC_lexicon_tags_37 },
    { 0x7a29aee1, SUC_lexicon_tags_17 },
    { 0x1c69aee1, SUC_lexicon_tags_11 },
    { 0x2f49aee3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3049aee6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa049aee9, SUC_lexicon_tags_7 },
    { 0x5309aeea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f09aeec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc929aeee, SUC_lexicon_tags_82 },
    { 0xdb29aeef, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be9aef4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c9aef8, SUC_lexicon_tags_3 },
    { 0xc1c9aef9, SUC_lexicon_tags_37 },
    { 0x7cc9aefa, SUC_lexicon_tags_7 },
    { 0x32a9aef9, SUC_lexicon_tags_17 },
    { 0x9709aefa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62c9aefe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf749af06, SUC_lexicon_tags_11 },
    { 0x5389af07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e69af0a, SUC_lexicon_tags_88 },
    { 0x3389af0b, SUC_lexicon_tags_23 },
    { 0xfc9af0c, SUC_lexicon_tags_17 },
    { 0x61e9af0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7049af0f, SUC_lexicon_tags_20 },
    { 0xbb29af0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5609af12, SUC_lexicon_tags_17 },
    { 0x77e9af13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc689af15, SUC_lexicon_tags_1 },
    { 0x1709af16, SUC_lexicon_tags_19 },
    { 0xd909af17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9af1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5069af1e, SUC_lexicon_tags_17 },
    { 0x1c29af1f, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea9af22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd989af29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf669af2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d49af2f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a9af32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8b29af34, SUC_lexicon_tags_15 },
    { 0x21e9af35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb89af39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ec9af3b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5509af49, SUC_lexicon_tags_15 },
    { 0xdb29af4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93a9af4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a89af4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c9af51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e9af57, SUC_lexicon_tags_1 },
    { 0x4f89af58, SUC_lexicon_tags_12 },
    { 0xea9af58, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1789af65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d69af68, SUC_lexicon_tags_12 },
    { 0x6669af69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4789af6b, SUC_lexicon_tags_27 },
    { 0x2029af6c, SUC_lexicon_tags_38 },
    { 0xd869af6d, SUC_lexicon_tags_12 },
    { 0xd929af6c, SUC_lexicon_tags_27 },
    { 0x8569af6f, SUC_lexicon_tags_82 },
    { 0x6ac9af70, SUC_lexicon_tags_1 },
    { 0x4509af71, SUC_lexicon_tags_0 },
    { 0x4729af6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e49af7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa529af7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a9af86, SUC_lexicon_tags_37 },
    { 0x5389af86, SUC_lexicon_tags_27 },
    { 0xf4c9af88, SUC_lexicon_tags_12 },
    { 0xb6e9af86, SUC_lexicon_tags_15 },
    { 0x7789af8a, SUC_lexicon_tags_2 },
    { 0x14c9af86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf309af91, SUC_lexicon_tags_43 },
    { 0x4e9af92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde69af94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f89af9a, SUC_lexicon_tags_27 },
    { 0x6509af9a, SUC_lexicon_tags_14 },
    { 0x5ce9af9b, SUC_lexicon_tags_0 },
    { 0x6949af9a, SUC_lexicon_tags_30 },
    { 0x1129af9e, SUC_lexicon_tags_68 },
    { 0x89a9af9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf089afa5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6909afa7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc9afad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2629afaf, SUC_lexicon_tags_11 },
    { 0xa469afaf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe829afb8, SUC_lexicon_tags_25 },
    { 0xd289afb8, SUC_lexicon_tags_1 },
    { 0x789afb9, SUC_lexicon_tags_7 },
    { 0x4669afb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea9afc2, SUC_lexicon_tags_11 },
    { 0x82c9afc2, SUC_lexicon_tags_17 },
    { 0xb869afc3, SUC_lexicon_tags_37 },
    { 0x5da9afc4, SUC_lexicon_tags_74 },
    { 0x29c9afc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5409afc9, SUC_lexicon_tags_37 },
    { 0xf769afc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde89afce, SUC_lexicon_tags_0 },
    { 0x2ec9afcf, SUC_lexicon_tags_37 },
    { 0xdba9afcf, SUC_lexicon_tags_54 },
    { 0x7729afcf, SUC_lexicon_tags_28 },
    { 0x3a29afd1, SUC_lexicon_tags_11 },
    { 0x2649afcf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2209afd6, SUC_lexicon_tags_27 },
    { 0xdc89afd6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5729afda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a9afe1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c09afe4, SUC_lexicon_tags_11 },
    { 0xd629afe5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf589afe8, SUC_lexicon_tags_2 },
    { 0xdfa9afe9, SUC_lexicon_tags_12 },
    { 0x1609afea, SUC_lexicon_tags_14 },
    { 0x67c9afeb, SUC_lexicon_tags_31 },
    { 0xa29afe9, SUC_lexicon_tags_15 },
    { 0xd9c9afed, SUC_lexicon_tags_31 },
    { 0x1c89afe8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe529aff2, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x8e09aff4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca9affa, SUC_lexicon_tags_11 },
    { 0xd4a9affb, SUC_lexicon_tags_38 },
    { 0x7869affc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6589afff, SUC_lexicon_tags_11 },
    { 0x5129afff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1449b002, SUC_lexicon_tags_7 },
    { 0x2849b003, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7529b009, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf49b00b, SUC_lexicon_tags_55 },
    { 0x2629b00c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e9b011, SUC_lexicon_tags_31 },
    { 0xe069b012, SUC_lexicon_tags_10 },
    { 0x82a9b013, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc9b016, SUC_lexicon_tags_11 },
    { 0xb909b016, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x44c9b019, SUC_lexicon_tags_6 },
    { 0x45a9b01a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x369b01c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe169b020, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca9b026, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9a9b028, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf69b02a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf49b02d, SUC_lexicon_tags_26 },
    { 0x3389b02e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc9b032, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcec9b035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc9b03b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9e9b03d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb49b040, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaba9b042, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c9b045, SUC_lexicon_tags_34 },
    { 0xcd29b046, SUC_lexicon_tags_1 },
    { 0x3989b047, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafc9b049, SUC_lexicon_tags_14 },
    { 0xf2e9b049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1889b04f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa809b052, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69b056, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be9b05b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2429b05e, SUC_lexicon_tags_11 },
    { 0xbaa9b05f, SUC_lexicon_tags_5 },
    { 0x9de9b060, SUC_lexicon_tags_17 },
    { 0x1a89b05f, SUC_lexicon_tags_7 },
    { 0x6b09b062, SUC_lexicon_tags_38 },
    { 0xed89b063, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbb69b065, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa9b069, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae89b06c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc7c9b06e, SUC_lexicon_tags_11 },
    { 0x9ea9b06f, SUC_lexicon_tags_1 },
    { 0x5309b070, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xea29b073, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0c9b075, SUC_lexicon_tags_37 },
    { 0xa89b076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae9b079, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b69b080, SUC_lexicon_tags_11 },
    { 0xfe9b081, SUC_lexicon_tags_1 },
    { 0x21a9b081, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d49b085, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c9b089, SUC_lexicon_tags_14 },
    { 0xd089b089, SUC_lexicon_tags_31 },
    { 0x4149b08a, SUC_lexicon_tags_33 },
    { 0xe409b089, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf689b090, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9889b092, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1329b095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3a9b097, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c89b09c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc929b09e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a9b0a6, SUC_lexicon_tags_1 },
    { 0xbb09b0a6, SUC_lexicon_tags_25 },
    { 0x8949b0a8, SUC_lexicon_tags_1 },
    { 0xca89b0a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7809b0ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe929b0b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3a9b0b6, SUC_lexicon_tags_29 },
    { 0xc729b0b7, SUC_lexicon_tags_28 },
    { 0x1509b0b8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe369b0ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b69b0c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x929b0c3, SUC_lexicon_tags_37 },
    { 0xfe49b0c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e9b0c7, SUC_lexicon_tags_47 },
    { 0x85a9b0c8, SUC_lexicon_tags_47 },
    { 0x4a09b0c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc029b0cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5929b0cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e9b0d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c69b0d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff09b0dc, SUC_lexicon_tags_46 },
    { 0x6289b0dd, SUC_lexicon_tags_37 },
    { 0x5be9b0de, SUC_lexicon_tags_15 },
    { 0x4189b0df, SUC_lexicon_tags_40 },
    { 0x36a9b0df, SUC_lexicon_tags_14 },
    { 0x5029b0e0, SUC_lexicon_tags_55 },
    { 0x4069b0df, SUC_lexicon_tags_17 },
    { 0x6929b0e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde9b0e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4669b0ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa409b0f0, SUC_lexicon_tags_37 },
    { 0xb509b0f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbde9b0f3, SUC_lexicon_tags_17 },
    { 0xbb89b0f4, SUC_lexicon_tags_12 },
    { 0x12e9b0f5, SUC_lexicon_tags_37 },
    { 0x2f49b0f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c9b0fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6649b0fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc769b104, SUC_lexicon_tags_77 },
    { 0x9689b104, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e9b10a, SUC_lexicon_tags_63 },
    { 0xad09b10b, SUC_lexicon_tags_2 },
    { 0xfba9b10c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e9b10f, SUC_lexicon_tags_12 },
    { 0xd529b110, SUC_lexicon_tags_37 },
    { 0x2829b111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9169b114, SUC_lexicon_tags_2 },
    { 0xc309b115, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7769b119, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x20e9b11b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7129b11d, SUC_lexicon_tags_1 },
    { 0x9389b11e, SUC_lexicon_tags_11 },
    { 0xdc09b11f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf729b125, SUC_lexicon_tags_60 },
    { 0x3069b125, SUC_lexicon_tags_60 },
    { 0x49c9b127, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6949b12b, SUC_lexicon_tags_17 },
    { 0x10e9b12c, SUC_lexicon_tags_0 },
    { 0x15e9b12b, SUC_lexicon_tags_0 },
    { 0x4369b12e, SUC_lexicon_tags_13 },
    { 0x2909b12e, SUC_lexicon_tags_4 },
    { 0xf169b130, SUC_lexicon_tags_37 },
    { 0x4609b12e, SUC_lexicon_tags_37 },
    { 0x7969b12f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69b137, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf229b13a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae9b143, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c9b147, SUC_lexicon_tags_14 },
    { 0x4d49b148, SUC_lexicon_tags_27 },
    { 0x42a9b149, SUC_lexicon_tags_39 },
    { 0xad49b148, SUC_lexicon_tags_81 },
    { 0x9809b14b, SUC_lexicon_tags_7 },
    { 0x9a9b14a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e9b152, SUC_lexicon_tags_11 },
    { 0x8309b153, SUC_lexicon_tags_0 },
    { 0x6709b153, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9049b156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf69b158, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd949b15c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e9b15f, SUC_lexicon_tags_7 },
    { 0x7369b15f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x509b163, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8469b167, SUC_lexicon_tags_31 },
    { 0x4749b168, SUC_lexicon_tags_11 },
    { 0x98a9b169, SUC_lexicon_tags_23 },
    { 0x3429b16a, SUC_lexicon_tags_32 },
    { 0x6d49b169, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b89b16d, SUC_lexicon_tags_1 },
    { 0x8e89b16e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb089b170, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f09b173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d89b175, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be9b179, SUC_lexicon_tags_36 },
    { 0x4449b17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff49b180, SUC_lexicon_tags_25 },
    { 0x2109b181, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6589b183, SUC_lexicon_tags_1 },
    { 0x7d49b184, SUC_lexicon_tags_45 },
    { 0xa629b185, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac9b188, SUC_lexicon_tags_28 },
    { 0x1949b188, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa6a9b18b, SUC_lexicon_tags_12 },
    { 0x1d89b18c, SUC_lexicon_tags_2 },
    { 0x31a9b18c, SUC_lexicon_tags_47 },
    { 0xb889b18e, SUC_lexicon_tags_55 },
    { 0xf009b18e, SUC_lexicon_tags_61 },
    { 0x3049b190, SUC_lexicon_tags_11 },
    { 0x86a9b18b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e9b194, SUC_lexicon_tags_15 },
    { 0x289b194, SUC_lexicon_tags_1 },
    { 0x2449b195, SUC_lexicon_tags_22 },
    { 0x3d29b196, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9b19c, SUC_lexicon_tags_65 },
    { 0x5e69b19c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe009b1a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd689b1a3, SUC_lexicon_tags_15 },
    { 0x5629b1a4, SUC_lexicon_tags_37 },
    { 0xdac9b1a3, SUC_lexicon_tags_28 },
    { 0xb069b1a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5869b1a8, SUC_lexicon_tags_37 },
    { 0x8129b1a9, SUC_lexicon_tags_1 },
    { 0xfb49b1a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e9b1b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c9b1bd, SUC_lexicon_tags_4 },
    { 0x48c9b1bd, SUC_lexicon_tags_45 },
    { 0xe429b1bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd09b1c2, SUC_lexicon_tags_38 },
    { 0x1da9b1c3, SUC_lexicon_tags_11 },
    { 0xf0e9b1c3, SUC_lexicon_tags_0 },
    { 0xf449b1c5, SUC_lexicon_tags_8 },
    { 0xa5a9b1c2, SUC_lexicon_tags_25 },
    { 0x3e29b1c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9369b1c9, SUC_lexicon_tags_0 },
    { 0x5dc9b1ca, SUC_lexicon_tags_13 },
    { 0x53c9b1ca, SUC_lexicon_tags_17 },
    { 0xaa9b1cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69b1cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5249b1d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69b1db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5669b1dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9b1e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6829b1e6, SUC_lexicon_tags_66 },
    { 0xb6e9b1e7, SUC_lexicon_tags_100 },
    { 0x5269b1e8, SUC_lexicon_tags_16 },
    { 0xb869b1e9, SUC_lexicon_tags_23 },
    { 0xaee9b1ea, SUC_lexicon_tags_27 },
    { 0x8e9b1ea, SUC_lexicon_tags_14 },
    { 0xae49b1ea, SUC_lexicon_tags_63 },
    { 0x34c9b1ed, SUC_lexicon_tags_14 },
    { 0x66a9b1e7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e89b1f1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a9b1f5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa489b1f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4869b1f9, SUC_lexicon_tags_11 },
    { 0x8a29b1fa, SUC_lexicon_tags_2 },
    { 0x229b1fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29b1fd, SUC_lexicon_tags_49 },
    { 0xeba9b1fe, SUC_lexicon_tags_11 },
    { 0x1d29b1ff, SUC_lexicon_tags_12 },
    { 0xd949b1fd, SUC_lexicon_tags_12 },
    { 0x25c9b201, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e9b205, SUC_lexicon_tags_4 },
    { 0xb0a9b205, SUC_lexicon_tags_14 },
    { 0xe069b207, SUC_lexicon_tags_6 },
    { 0x8989b208, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf309b20e, SUC_lexicon_tags_37 },
    { 0x1dc9b20f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa989b211, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x569b213, SUC_lexicon_tags_15 },
    { 0x1389b214, SUC_lexicon_tags_33 },
    { 0x4d69b215, SUC_lexicon_tags_17 },
    { 0x36c9b215, SUC_lexicon_tags_23 },
    { 0xc1a9b214, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb249b21b, SUC_lexicon_tags_38 },
    { 0x4249b21c, SUC_lexicon_tags_103 },
    { 0x7589b21d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1309b221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca29b223, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9b226, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9909b228, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8009b22c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5309b22e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac49b230, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd989b236, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1ee9b238, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd169b23a, SUC_lexicon_tags_59 },
    { 0x9d89b23b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8209b23d, SUC_lexicon_tags_19 },
    { 0xd309b23e, SUC_lexicon_tags_7 },
    { 0x349b23e, SUC_lexicon_tags_109 },
    { 0x6909b23f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3889b244, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d29b247, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xc389b24a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b89b250, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8609b252, SUC_lexicon_tags_7 },
    { 0xc989b253, SUC_lexicon_tags_37 },
    { 0xe469b254, SUC_lexicon_tags_27 },
    { 0xdfa9b253, SUC_lexicon_tags_45 },
    { 0x3e69b255, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcde9b258, SUC_lexicon_tags_37 },
    { 0x4f49b258, SUC_lexicon_tags_1 },
    { 0x6ee9b259, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9b25e, SUC_lexicon_tags_25 },
    { 0x7ca9b25e, SUC_lexicon_tags_7 },
    { 0x23c9b25f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe209b262, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e69b266, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a9b26a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d09b26d, SUC_lexicon_tags_1 },
    { 0xd509b26d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6229b270, SUC_lexicon_tags_15 },
    { 0xc09b271, SUC_lexicon_tags_7 },
    { 0x3529b270, SUC_lexicon_tags_23 },
    { 0xefc9b271, SUC_lexicon_tags_15 },
    { 0x9289b274, SUC_lexicon_tags_47 },
    { 0x8b49b275, SUC_lexicon_tags_1 },
    { 0x449b275, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf69b278, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4909b27a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x83e9b27c, SUC_lexicon_tags_25 },
    { 0xb4e9b27c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a9b285, SUC_lexicon_tags_11 },
    { 0x8129b286, SUC_lexicon_tags_27 },
    { 0x2329b286, SUC_lexicon_tags_32 },
    { 0xfe29b288, SUC_lexicon_tags_12 },
    { 0x8569b286, SUC_lexicon_tags_17 },
    { 0x79a9b288, SUC_lexicon_tags_47 },
    { 0x1e69b289, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6869b28e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f69b291, SUC_lexicon_tags_7 },
    { 0xcf49b292, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d09b294, SUC_lexicon_tags_68 },
    { 0xb5e9b295, SUC_lexicon_tags_1 },
    { 0xd449b296, SUC_lexicon_tags_12 },
    { 0x56c9b297, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4929b29b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc29b29e, SUC_lexicon_tags_37 },
    { 0x2609b29f, SUC_lexicon_tags_25 },
    { 0x1ee9b29f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x229b2a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65a9b2a5, SUC_lexicon_tags_31 },
    { 0x7f09b2a5, SUC_lexicon_tags_11 },
    { 0xd449b2a7, SUC_lexicon_tags_7 },
    { 0xc329b2a8, SUC_lexicon_tags_3 },
    { 0xa8a9b2a9, SUC_lexicon_tags_17 },
    { 0x86c9b2aa, SUC_lexicon_tags_87 },
    { 0x4649b2a9, SUC_lexicon_tags_82 },
    { 0x7b49b2ac, SUC_lexicon_tags_30 },
    { 0x9909b2ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa249b2b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a49b2b5, SUC_lexicon_tags_62 },
    { 0x4749b2b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e9b2b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2be9b2bb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e9b2bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f89b2c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfde9b2c6, SUC_lexicon_tags_25 },
    { 0x11c9b2c6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a89b2ca, SUC_lexicon_tags_17 },
    { 0xb5a9b2cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd829b2cd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56c9b2d1, SUC_lexicon_tags_55 },
    { 0x47a9b2d2, SUC_lexicon_tags_49 },
    { 0x1da9b2d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x849b2d5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3589b2d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5489b2db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f89b2de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6969b2e0, SUC_lexicon_tags_29 },
    { 0xcdc9b2e1, SUC_lexicon_tags_170 },
    { 0xbac9b2e0, SUC_lexicon_tags_12 },
    { 0xf009b2e3, SUC_lexicon_tags_26 },
    { 0xcd09b2e4, SUC_lexicon_tags_1 },
    { 0xea9b2e5, SUC_lexicon_tags_27 },
    { 0x98e9b2e6, SUC_lexicon_tags_43 },
    { 0xd209b2e6, SUC_lexicon_tags_15 },
    { 0xb9c9b2e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee9b2ea, SUC_lexicon_tags_4 },
    { 0x6ca9b2eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8349b2f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b69b2f6, SUC_lexicon_tags_11 },
    { 0xe409b2f7, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a9b2ff, SUC_lexicon_tags_29 },
    { 0xcbc9b300, SUC_lexicon_tags_37 },
    { 0x5209b301, SUC_lexicon_tags_47 },
    { 0x24c9b300, SUC_lexicon_tags_12 },
    { 0xf269b303, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa649b307, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f29b309, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7849b30c, SUC_lexicon_tags_8 },
    { 0xc69b30d, SUC_lexicon_tags_12 },
    { 0x8389b30d, SUC_lexicon_tags_46 },
    { 0xe9a9b30e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfe9b311, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe589b315, SUC_lexicon_tags_17 },
    { 0x5dc9b316, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8fa9b318, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4149b31b, SUC_lexicon_tags_60 },
    { 0xbee9b31c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e9b32c, SUC_lexicon_tags_7 },
    { 0xb769b32c, SUC_lexicon_tags_65 },
    { 0xa7a9b32e, SUC_lexicon_tags_28 },
    { 0x9889b32f, SUC_lexicon_tags_12 },
    { 0x1bc9b330, SUC_lexicon_tags_27 },
    { 0x4889b330, SUC_lexicon_tags_15 },
    { 0xa729b330, SUC_lexicon_tags_28 },
    { 0x5929b333, SUC_lexicon_tags_91 },
    { 0xc449b334, SUC_lexicon_tags_16 },
    { 0xf869b335, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e9b33b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f89b33e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7269b341, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49b344, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa1c9b346, SUC_lexicon_tags_108 },
    { 0x9509b347, SUC_lexicon_tags_25 },
    { 0x6449b348, SUC_lexicon_tags_5 },
    { 0xb769b349, SUC_lexicon_tags_12 },
    { 0x77c9b348, SUC_lexicon_tags_12 },
    { 0x7de9b34b, SUC_lexicon_tags_12 },
    { 0x64c9b346, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789b350, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b69b353, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe089b355, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4149b35b, SUC_lexicon_tags_2 },
    { 0x8969b35c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b89b35e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5549b366, SUC_lexicon_tags_62 },
    { 0xafe9b367, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e9b36a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd469b36e, SUC_lexicon_tags_87 },
    { 0xae49b36f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8049b37a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa289b37e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a69b388, SUC_lexicon_tags_25 },
    { 0xe0e9b388, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa4e9b38b, SUC_lexicon_tags_33 },
    { 0x2009b38b, SUC_lexicon_tags_17 },
    { 0x6fa9b38d, SUC_lexicon_tags_77 },
    { 0x8609b38c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9909b390, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc829b394, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e9b397, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3669b39c, SUC_lexicon_tags_45 },
    { 0xeb49b39d, SUC_lexicon_tags_2 },
    { 0x7de9b39d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3389b3a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9b3a4, SUC_lexicon_tags_12 },
    { 0xb8a9b3a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd649b3aa, SUC_lexicon_tags_49 },
    { 0xe6e9b3aa, SUC_lexicon_tags_37 },
    { 0x2d89b3ac, SUC_lexicon_tags_1 },
    { 0x6b29b3ad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5ec9b3af, SUC_lexicon_tags_47 },
    { 0xcf89b3af, SUC_lexicon_tags_1 },
    { 0x2249b3b1, SUC_lexicon_tags_37 },
    { 0x5889b3af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69b3b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e9b3be, SUC_lexicon_tags_23 },
    { 0x27c9b3bf, SUC_lexicon_tags_47 },
    { 0x96e9b3c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a9b3c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849b3cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5629b3ce, SUC_lexicon_tags_37 },
    { 0x65a9b3cf, SUC_lexicon_tags_37 },
    { 0x649b3d0, SUC_lexicon_tags_33 },
    { 0x6489b3d1, SUC_lexicon_tags_3 },
    { 0x96c9b3d2, SUC_lexicon_tags_49 },
    { 0x2f49b3d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e9b3d6, SUC_lexicon_tags_19 },
    { 0x81a9b3d7, SUC_lexicon_tags_47 },
    { 0x5fe9b3d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e49b3da, SUC_lexicon_tags_57 },
    { 0xb9c9b3db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9749b3de, SUC_lexicon_tags_23 },
    { 0x9769b3df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e29b3e1, SUC_lexicon_tags_172 },
    { 0x2209b3e1, SUC_lexicon_tags_34 },
    { 0xe89b3e1, SUC_lexicon_tags_26 },
    { 0x5d89b3e4, SUC_lexicon_tags_21 },
    { 0x8689b3e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a09b3e9, SUC_lexicon_tags_1600 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c9b3ec, SUC_lexicon_tags_3 },
    { 0xec49b3ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf409b3ef, SUC_lexicon_tags_7 },
    { 0x16c9b3f0, SUC_lexicon_tags_31 },
    { 0x65c9b3f1, SUC_lexicon_tags_53 },
    { 0x6809b3f2, SUC_lexicon_tags_23 },
    { 0xa149b3f0, SUC_lexicon_tags_2 },
    { 0x2949b3f4, SUC_lexicon_tags_63 },
    { 0x8dc9b3f4, SUC_lexicon_tags_63 },
    { 0x489b3f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd409b3f8, SUC_lexicon_tags_43 },
    { 0x95e9b3f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb109b3fb, SUC_lexicon_tags_17 },
    { 0xdf49b3fb, SUC_lexicon_tags_15 },
    { 0xdae9b3fc, SUC_lexicon_tags_27 },
    { 0x7ae9b3fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a9b401, SUC_lexicon_tags_79 },
    { 0xef89b402, SUC_lexicon_tags_7 },
    { 0x3649b403, SUC_lexicon_tags_11 },
    { 0x5ca9b402, SUC_lexicon_tags_14 },
    { 0x7f29b405, SUC_lexicon_tags_45 },
    { 0xaa9b406, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1089b408, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c89b40b, SUC_lexicon_tags_26 },
    { 0xae49b40c, SUC_lexicon_tags_37 },
    { 0x5ee9b40c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e9b411, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb69b414, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc709b41b, SUC_lexicon_tags_47 },
    { 0x7469b41c, SUC_lexicon_tags_31 },
    { 0x9969b41d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca9b420, SUC_lexicon_tags_17 },
    { 0x2fc9b420, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c9b425, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d09b427, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5369b42c, SUC_lexicon_tags_47 },
    { 0x8249b42d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a9b431, SUC_lexicon_tags_37 },
    { 0xa009b431, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7be9b437, SUC_lexicon_tags_136 },
    { 0x2829b438, SUC_lexicon_tags_38 },
    { 0xdbc9b439, SUC_lexicon_tags_57 },
    { 0x6409b43a, SUC_lexicon_tags_15 },
    { 0x9b439, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8869b43f, SUC_lexicon_tags_19 },
    { 0x1bc9b440, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1469b443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1749b446, SUC_lexicon_tags_24 },
    { 0x8189b446, SUC_lexicon_tags_17 },
    { 0x5a89b448, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf049b44b, SUC_lexicon_tags_29 },
    { 0xd169b44c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe549b451, SUC_lexicon_tags_37 },
    { 0xb8e9b452, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe9b457, SUC_lexicon_tags_8 },
    { 0x4d69b458, SUC_lexicon_tags_17 },
    { 0xa149b458, SUC_lexicon_tags_37 },
    { 0x989b45a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1689b45f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf09b464, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda89b466, SUC_lexicon_tags_15 },
    { 0x2f49b466, SUC_lexicon_tags_11 },
    { 0x2a69b468, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2649b46f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09b473, SUC_lexicon_tags_2 },
    { 0xca09b474, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac9b479, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc409b47b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc9b480, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x449b482, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6189b484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e09b486, SUC_lexicon_tags_31 },
    { 0xaea9b487, SUC_lexicon_tags_33 },
    { 0x5e9b488, SUC_lexicon_tags_7 },
    { 0xcc89b487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae9b48c, SUC_lexicon_tags_11 },
    { 0x79c9b48c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec9b491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8269b493, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2f89b495, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x65c9b497, SUC_lexicon_tags_66 },
    { 0xc829b497, SUC_lexicon_tags_45 },
    { 0x35c9b499, SUC_lexicon_tags_47 },
    { 0x40a9b49a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5a9b49c, SUC_lexicon_tags_13 },
    { 0x1c89b49c, SUC_lexicon_tags_37 },
    { 0x4889b49d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69b4a3, SUC_lexicon_tags_7 },
    { 0xdba9b4a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf949b4a8, SUC_lexicon_tags_25 },
    { 0xc569b4a8, SUC_lexicon_tags_1 },
    { 0x3249b4aa, SUC_lexicon_tags_12 },
    { 0xd489b4a9, SUC_lexicon_tags_47 },
    { 0x6e09b4aa, SUC_lexicon_tags_15 },
    { 0x4ea9b4ad, SUC_lexicon_tags_11 },
    { 0x2d09b4ab, SUC_lexicon_tags_62 },
    { 0xc7c9b4af, SUC_lexicon_tags_38 },
    { 0x19a9b4af, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe449b4b4, SUC_lexicon_tags_27 },
    { 0xd449b4b5, SUC_lexicon_tags_15 },
    { 0xc8e9b4b6, SUC_lexicon_tags_11 },
    { 0x3a29b4b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf89b4bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce9b4c1, SUC_lexicon_tags_106 },
    { 0xb7a9b4c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8529b4c4, SUC_lexicon_tags_60 },
    { 0xa8a9b4c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf529b4cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa49b4ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa9b4d5, SUC_lexicon_tags_4 },
    { 0xdec9b4d6, SUC_lexicon_tags_27 },
    { 0x1829b4d7, SUC_lexicon_tags_77 },
    { 0xbe69b4d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x509b4da, SUC_lexicon_tags_11 },
    { 0x3429b4db, SUC_lexicon_tags_19 },
    { 0xdda9b4dc, SUC_lexicon_tags_26 },
    { 0x4389b4dd, SUC_lexicon_tags_33 },
    { 0x7709b4dd, SUC_lexicon_tags_32 },
    { 0x96c9b4df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3929b4e3, SUC_lexicon_tags_37 },
    { 0x1d09b4e4, SUC_lexicon_tags_17 },
    { 0x7d09b4e4, SUC_lexicon_tags_37 },
    { 0xa2e9b4e6, SUC_lexicon_tags_37 },
    { 0x8009b4e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0c9b4e9, SUC_lexicon_tags_7 },
    { 0x7869b4ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2569b4ec, SUC_lexicon_tags_12 },
    { 0x5989b4ed, SUC_lexicon_tags_15 },
    { 0x62c9b4ee, SUC_lexicon_tags_11 },
    { 0x2a9b4ee, SUC_lexicon_tags_11 },
    { 0x4769b4ef, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1049b4f2, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a9b4f5, SUC_lexicon_tags_10 },
    { 0xd909b4f6, SUC_lexicon_tags_25 },
    { 0xc149b4f7, SUC_lexicon_tags_23 },
    { 0xdc29b4f8, SUC_lexicon_tags_9 },
    { 0xb229b4f9, SUC_lexicon_tags_23 },
    { 0xc429b4fa, SUC_lexicon_tags_30 },
    { 0x5009b4fa, SUC_lexicon_tags_12 },
    { 0xd269b4fc, SUC_lexicon_tags_74 },
    { 0x6029b4f9, SUC_lexicon_tags_25 },
    { 0xad69b4fe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49b501, SUC_lexicon_tags_12 },
    { 0xb729b502, SUC_lexicon_tags_47 },
    { 0x4ac9b503, SUC_lexicon_tags_29 },
    { 0xdb09b503, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e69b50b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a9b510, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7e9b512, SUC_lexicon_tags_45 },
    { 0xaf89b512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fe9b515, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb09b51c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd509b51e, SUC_lexicon_tags_6 },
    { 0x3229b51f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe5a9b521, SUC_lexicon_tags_2 },
    { 0x7a89b521, SUC_lexicon_tags_45 },
    { 0xc149b523, SUC_lexicon_tags_19 },
    { 0x6ec9b523, SUC_lexicon_tags_17 },
    { 0xeae9b522, SUC_lexicon_tags_75 },
    { 0x1389b523, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe09b529, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3169b530, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5889b532, SUC_lexicon_tags_1 },
    { 0x1a29b532, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4089b535, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e9b539, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x82a9b53b, SUC_lexicon_tags_10 },
    { 0x5549b53c, SUC_lexicon_tags_33 },
    { 0xb689b53c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x5929b53f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf309b541, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4709b546, SUC_lexicon_tags_33 },
    { 0x5349b547, SUC_lexicon_tags_7 },
    { 0x66e9b548, SUC_lexicon_tags_7 },
    { 0x2989b548, SUC_lexicon_tags_14 },
    { 0xfba9b549, SUC_lexicon_tags_0 },
    { 0xc329b54b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba69b54e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a9b552, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca9b556, SUC_lexicon_tags_6 },
    { 0x3c09b557, SUC_lexicon_tags_6 },
    { 0x1e89b558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3e9b55a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe709b55c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef09b560, SUC_lexicon_tags_12 },
    { 0x7fe9b561, SUC_lexicon_tags_19 },
    { 0x5489b561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4569b565, SUC_lexicon_tags_30 },
    { 0x7e69b565, SUC_lexicon_tags_32 },
    { 0x6849b565, SUC_lexicon_tags_7 },
    { 0x5e89b565, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e9b56c, SUC_lexicon_tags_11 },
    { 0x27e9b56d, SUC_lexicon_tags_14 },
    { 0x7969b56d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb49b57c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e09b582, SUC_lexicon_tags_34 },
    { 0x9d49b583, SUC_lexicon_tags_37 },
    { 0xc149b584, SUC_lexicon_tags_27 },
    { 0xcda9b585, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6929b587, SUC_lexicon_tags_4 },
    { 0x69e9b588, SUC_lexicon_tags_38 },
    { 0xafc9b589, SUC_lexicon_tags_12 },
    { 0x2b49b588, SUC_lexicon_tags_14 },
    { 0x62e9b58b, SUC_lexicon_tags_11 },
    { 0xdd89b58c, SUC_lexicon_tags_47 },
    { 0xcce9b58c, SUC_lexicon_tags_15 },
    { 0x8529b58b, SUC_lexicon_tags_32 },
    { 0xeba9b58f, SUC_lexicon_tags_4 },
    { 0xa3c9b58f, SUC_lexicon_tags_29 },
    { 0x6149b58b, SUC_lexicon_tags_1 },
    { 0x2609b592, SUC_lexicon_tags_63 },
    { 0x4ba9b593, SUC_lexicon_tags_48 },
    { 0xee89b58c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9b599, SUC_lexicon_tags_39 },
    { 0x8169b59a, SUC_lexicon_tags_7 },
    { 0x3209b59a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a9b59f, SUC_lexicon_tags_49 },
    { 0xa9b5a0, SUC_lexicon_tags_31 },
    { 0x1d29b5a1, SUC_lexicon_tags_27 },
    { 0x1c29b5a2, SUC_lexicon_tags_15 },
    { 0xfe29b5a3, SUC_lexicon_tags_85 },
    { 0xe709b5a4, SUC_lexicon_tags_47 },
    { 0xffa9b5a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2a9b5a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b5af, SUC_lexicon_tags_0 },
    { 0x8129b5b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a9b5b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d09b5b8, SUC_lexicon_tags_25 },
    { 0x94a9b5b9, SUC_lexicon_tags_2 },
    { 0xdb09b5ba, SUC_lexicon_tags_7 },
    { 0x24e9b5bb, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xde69b5bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a9b5c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf849b5c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b09b5c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a9b5cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e9b5ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3049b5d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7969b5d3, SUC_lexicon_tags_39 },
    { 0x7709b5d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4169b5d9, SUC_lexicon_tags_19 },
    { 0xe029b5d9, SUC_lexicon_tags_13 },
    { 0xe009b5db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4169b5df, SUC_lexicon_tags_179 },
    { 0xfa29b5e0, SUC_lexicon_tags_15 },
    { 0x1629b5e1, SUC_lexicon_tags_2 },
    { 0x7109b5e0, SUC_lexicon_tags_15 },
    { 0xb529b5e0, SUC_lexicon_tags_61 },
    { 0x2189b5e4, SUC_lexicon_tags_29 },
    { 0x2749b5e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e9b5eb, SUC_lexicon_tags_39 },
    { 0xf269b5ec, SUC_lexicon_tags_289 },
    { 0xe469b5eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac9b5f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7289b5f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949b5fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0e9b5fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ee9b600, SUC_lexicon_tags_14 },
    { 0x2349b600, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e9b604, SUC_lexicon_tags_49 },
    { 0xdd29b604, SUC_lexicon_tags_12 },
    { 0x9b606, SUC_lexicon_tags_38 },
    { 0x2209b606, SUC_lexicon_tags_49 },
    { 0xb129b608, SUC_lexicon_tags_37 },
    { 0x2889b609, SUC_lexicon_tags_10 },
    { 0x4289b606, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e49b60d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbea9b60f, SUC_lexicon_tags_27 },
    { 0xee89b60f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45a9b612, SUC_lexicon_tags_4 },
    { 0xb5a9b612, SUC_lexicon_tags_12 },
    { 0x989b613, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa9b618, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca9b61b, SUC_lexicon_tags_17 },
    { 0xf7a9b61c, SUC_lexicon_tags_0 },
    { 0xe009b61d, SUC_lexicon_tags_2 },
    { 0x3f49b61b, SUC_lexicon_tags_43 },
    { 0x7169b61e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe229b622, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9b628, SUC_lexicon_tags_104 },
    { 0xf7e9b629, SUC_lexicon_tags_17 },
    { 0xb089b62a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9449b62c, SUC_lexicon_tags_14 },
    { 0x3f49b62d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb89b631, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad69b634, SUC_lexicon_tags_35 },
    { 0x3f49b635, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a09b639, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5569b63b, SUC_lexicon_tags_37 },
    { 0xb1e9b63c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf89b641, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a9b644, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfcc9b646, SUC_lexicon_tags_60 },
    { 0xc7e9b647, SUC_lexicon_tags_10 },
    { 0xfa69b647, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8349b64b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c9b64f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6549b652, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9909b654, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1449b65b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee89b65f, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x3929b662, SUC_lexicon_tags_37 },
    { 0xc3c9b663, SUC_lexicon_tags_2 },
    { 0xfe9b664, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee9b668, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e9b66b, SUC_lexicon_tags_7 },
    { 0xa649b66c, SUC_lexicon_tags_63 },
    { 0x3b89b66d, SUC_lexicon_tags_33 },
    { 0x1e29b66d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5749b672, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca9b676, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a9b679, SUC_lexicon_tags_60 },
    { 0x3849b679, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e9b680, SUC_lexicon_tags_15 },
    { 0x6c9b681, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc9b684, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e9b687, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9b68b, SUC_lexicon_tags_12 },
    { 0x7f29b68b, SUC_lexicon_tags_43 },
    { 0x44e9b68c, SUC_lexicon_tags_12 },
    { 0x4749b68e, SUC_lexicon_tags_45 },
    { 0x19e9b68f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xba29b692, SUC_lexicon_tags_15 },
    { 0xd189b692, SUC_lexicon_tags_11 },
    { 0x2ce9b694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb89b696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd409b699, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea9b69e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2209b6a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5e49b6a2, SUC_lexicon_tags_15 },
    { 0x6d69b6a3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a9b6a6, SUC_lexicon_tags_25 },
    { 0xb329b6a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c29b6a9, SUC_lexicon_tags_12 },
    { 0xab89b6a9, SUC_lexicon_tags_60 },
    { 0xa949b6aa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1589b6b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1949b6b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8749b6b5, SUC_lexicon_tags_6 },
    { 0x8669b6b5, SUC_lexicon_tags_39 },
    { 0x4b69b6b7, SUC_lexicon_tags_48 },
    { 0xf209b6b7, SUC_lexicon_tags_37 },
    { 0x4c89b6b9, SUC_lexicon_tags_15 },
    { 0x1729b6b7, SUC_lexicon_tags_33 },
    { 0xa649b6bb, SUC_lexicon_tags_15 },
    { 0x5b89b6bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e9b6c0, SUC_lexicon_tags_0 },
    { 0x209b6c1, SUC_lexicon_tags_25 },
    { 0x6a9b6c2, SUC_lexicon_tags_109 },
    { 0xe9b6c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf29b6d0, SUC_lexicon_tags_25 },
    { 0x7d29b6d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf629b6d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb589b6d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa389b6d7, SUC_lexicon_tags_12 },
    { 0xf169b6d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf29b6e3, SUC_lexicon_tags_12 },
    { 0x4909b6e4, SUC_lexicon_tags_93 },
    { 0x5f09b6e5, SUC_lexicon_tags_12 },
    { 0xce9b6e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32a9b6e8, SUC_lexicon_tags_17 },
    { 0xcaa9b6e9, SUC_lexicon_tags_2 },
    { 0xeac9b6ea, SUC_lexicon_tags_4 },
    { 0xada9b6ea, SUC_lexicon_tags_40 },
    { 0xbea9b6e9, SUC_lexicon_tags_37 },
    { 0x8a09b6ed, SUC_lexicon_tags_11 },
    { 0xafc9b6ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c89b6f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e89b6f3, SUC_lexicon_tags_38 },
    { 0xee29b6f3, SUC_lexicon_tags_17 },
    { 0x3529b6f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c9b6f8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f69b6fc, SUC_lexicon_tags_10 },
    { 0xc589b6fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf69b6ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab49b703, SUC_lexicon_tags_49 },
    { 0x4f49b704, SUC_lexicon_tags_19 },
    { 0x8089b703, SUC_lexicon_tags_25 },
    { 0xe49b706, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e9b709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6589b711, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xeec9b713, SUC_lexicon_tags_45 },
    { 0xe3c9b714, SUC_lexicon_tags_1 },
    { 0xf4e9b713, SUC_lexicon_tags_24 },
    { 0xf9a9b716, SUC_lexicon_tags_15 },
    { 0x46e9b716, SUC_lexicon_tags_16 },
    { 0xd1e9b718, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf0a9b71a, SUC_lexicon_tags_17 },
    { 0x5f09b71a, SUC_lexicon_tags_23 },
    { 0x229b71c, SUC_lexicon_tags_33 },
    { 0x89e9b71d, SUC_lexicon_tags_35 },
    { 0x7409b71a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e9b722, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e89b728, SUC_lexicon_tags_47 },
    { 0x9929b728, SUC_lexicon_tags_60 },
    { 0xa4e9b72a, SUC_lexicon_tags_31 },
    { 0xdfc9b72b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609b730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1429b732, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc9b734, SUC_lexicon_tags_11 },
    { 0xc369b735, SUC_lexicon_tags_38 },
    { 0x8e69b736, SUC_lexicon_tags_11 },
    { 0xfc69b735, SUC_lexicon_tags_7 },
    { 0x5a9b735, SUC_lexicon_tags_30 },
    { 0x2fe9b739, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be9b743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f49b74b, SUC_lexicon_tags_23 },
    { 0x5aa9b74c, SUC_lexicon_tags_28 },
    { 0x8989b74d, SUC_lexicon_tags_29 },
    { 0xfe89b74d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4809b751, SUC_lexicon_tags_68 },
    { 0x5569b752, SUC_lexicon_tags_6 },
    { 0x1ca9b753, SUC_lexicon_tags_60 },
    { 0x1fc9b754, SUC_lexicon_tags_19 },
    { 0x2dc9b753, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89b760, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x46e9b762, SUC_lexicon_tags_29 },
    { 0x3449b762, SUC_lexicon_tags_19 },
    { 0xb7c9b764, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdac9b766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ac9b768, SUC_lexicon_tags_12 },
    { 0x9869b769, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc29b76c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e9b76f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a09b77a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8309b77c, SUC_lexicon_tags_0 },
    { 0xe7e9b77d, SUC_lexicon_tags_77 },
    { 0x4b09b77e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8509b781, SUC_lexicon_tags_19 },
    { 0x5369b782, SUC_lexicon_tags_82 },
    { 0xc89b782, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c9b786, SUC_lexicon_tags_6 },
    { 0x10c9b786, SUC_lexicon_tags_0 },
    { 0x7509b788, SUC_lexicon_tags_14 },
    { 0x22a9b788, SUC_lexicon_tags_7 },
    { 0xa889b788, SUC_lexicon_tags_7 },
    { 0xce69b78b, SUC_lexicon_tags_49 },
    { 0x8a09b78a, SUC_lexicon_tags_56 },
    { 0x43c9b78b, SUC_lexicon_tags_9 },
    { 0x7a9b78e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a9b792, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9b795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c9b79b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa969b79d, SUC_lexicon_tags_60 },
    { 0x809b79e, SUC_lexicon_tags_37 },
    { 0xd989b79e, SUC_lexicon_tags_33 },
    { 0xb309b7a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7609b7a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x37c9b7a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabe9b7a8, SUC_lexicon_tags_17 },
    { 0xaf49b7a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe9b7ae, SUC_lexicon_tags_7 },
    { 0xbba9b7ae, SUC_lexicon_tags_33 },
    { 0x6809b7b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc269b7b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be9b7c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2e9b7c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5b89b7c4, SUC_lexicon_tags_33 },
    { 0xbfe9b7c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b69b7c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x79a9b7ca, SUC_lexicon_tags_1 },
    { 0xb09b7cb, SUC_lexicon_tags_25 },
    { 0x3ce9b7cb, SUC_lexicon_tags_17 },
    { 0x6569b7cd, SUC_lexicon_tags_2 },
    { 0x32e9b7ce, SUC_lexicon_tags_47 },
    { 0x2a09b7cf, SUC_lexicon_tags_6 },
    { 0x6a9b7ca, SUC_lexicon_tags_48 },
    { 0x96a9b7ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc9b7d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a49b7db, SUC_lexicon_tags_478 },
    { 0x77c9b7db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac69b7e0, SUC_lexicon_tags_80 },
    { 0x9b69b7e1, SUC_lexicon_tags_47 },
    { 0xc6e9b7e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b89b7e5, SUC_lexicon_tags_25 },
    { 0x6b69b7e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b49b7e9, SUC_lexicon_tags_8 },
    { 0xb149b7ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8989b7ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc9b7f0, SUC_lexicon_tags_875 },
    { 0xce49b7f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb09b7f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d09b7f6, SUC_lexicon_tags_60 },
    { 0xc129b7f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3009b7f9, SUC_lexicon_tags_17 },
    { 0x2329b7fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee9b7fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa309b803, SUC_lexicon_tags_11 },
    { 0x3809b803, SUC_lexicon_tags_1 },
    { 0x78c9b804, SUC_lexicon_tags_56 },
    { 0x6309b803, SUC_lexicon_tags_282 },
    { 0x4e09b806, SUC_lexicon_tags_33 },
    { 0x5209b808, SUC_lexicon_tags_23 },
    { 0x7069b809, SUC_lexicon_tags_7 },
    { 0x6569b80a, SUC_lexicon_tags_37 },
    { 0x9009b80b, SUC_lexicon_tags_37 },
    { 0x4ba9b80c, SUC_lexicon_tags_11 },
    { 0x8f89b80c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b29b813, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa989b817, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f89b81b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdc9b81d, SUC_lexicon_tags_43 },
    { 0xbda9b81e, SUC_lexicon_tags_79 },
    { 0x3aa9b81f, SUC_lexicon_tags_25 },
    { 0xee69b820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca9b824, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c9b82b, SUC_lexicon_tags_39 },
    { 0xf729b82b, SUC_lexicon_tags_37 },
    { 0x48e9b82d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4149b830, SUC_lexicon_tags_25 },
    { 0x2709b831, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a9b836, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9269b83a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbc9b83c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2029b83f, SUC_lexicon_tags_2 },
    { 0x5cc9b840, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a9b844, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf89b847, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdc9b849, SUC_lexicon_tags_47 },
    { 0xbae9b849, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389b84f, SUC_lexicon_tags_17 },
    { 0x589b850, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd09b853, SUC_lexicon_tags_15 },
    { 0x3c29b853, SUC_lexicon_tags_27 },
    { 0x7f69b855, SUC_lexicon_tags_14 },
    { 0xebc9b853, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac69b85b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc249b867, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3029b86a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb549b86c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b69b86e, SUC_lexicon_tags_322 },
    { 0x2e89b86e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb69b874, SUC_lexicon_tags_17 },
    { 0xb4e9b875, SUC_lexicon_tags_23 },
    { 0xcd09b874, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x509b878, SUC_lexicon_tags_37 },
    { 0xec09b879, SUC_lexicon_tags_25 },
    { 0x17e9b87a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe849b87c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc9b87f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a9b882, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09b889, SUC_lexicon_tags_37 },
    { 0x1689b88a, SUC_lexicon_tags_37 },
    { 0xee49b88a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe69b88d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb029b892, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf209b896, SUC_lexicon_tags_17 },
    { 0x7629b897, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9b89c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a9b8a2, SUC_lexicon_tags_9 },
    { 0xdc29b8a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc189b8ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8809b8b2, SUC_lexicon_tags_33 },
    { 0xd129b8b2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd209b8b5, SUC_lexicon_tags_66 },
    { 0xb7a9b8b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4e9b8b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9089b8c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c9b8c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e9b8c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe289b8cc, SUC_lexicon_tags_25 },
    { 0xae49b8cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe7c9b8cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6889b8d1, SUC_lexicon_tags_11 },
    { 0x7329b8d1, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2509b8da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd529b8dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa9b8e0, SUC_lexicon_tags_15 },
    { 0x3f89b8e1, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d89b8e6, SUC_lexicon_tags_11 },
    { 0xd549b8e7, SUC_lexicon_tags_12 },
    { 0x69a9b8e8, SUC_lexicon_tags_37 },
    { 0xa729b8e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26a9b8eb, SUC_lexicon_tags_37 },
    { 0xd329b8eb, SUC_lexicon_tags_2 },
    { 0xf349b8ed, SUC_lexicon_tags_16 },
    { 0xed69b8ee, SUC_lexicon_tags_2 },
    { 0xc3e9b8ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4269b8f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdc9b8f4, SUC_lexicon_tags_19 },
    { 0x5ee9b8f4, SUC_lexicon_tags_25 },
    { 0x6ba9b8f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6529b8f9, SUC_lexicon_tags_9 },
    { 0x65a9b8f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa969b8ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe69b903, SUC_lexicon_tags_68 },
    { 0xcea9b904, SUC_lexicon_tags_16 },
    { 0xa869b903, SUC_lexicon_tags_25 },
    { 0x4e69b906, SUC_lexicon_tags_11 },
    { 0x1309b907, SUC_lexicon_tags_23 },
    { 0x3d49b907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9769b90b, SUC_lexicon_tags_2 },
    { 0xf329b90c, SUC_lexicon_tags_1 },
    { 0x1f69b90d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c9b911, SUC_lexicon_tags_45 },
    { 0xe889b912, SUC_lexicon_tags_1 },
    { 0x53a9b913, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x91c9b915, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489b919, SUC_lexicon_tags_38 },
    { 0x12c9b919, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e9b925, SUC_lexicon_tags_87 },
    { 0xeb89b926, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ca9b928, SUC_lexicon_tags_15 },
    { 0xfa9b929, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f69b92e, SUC_lexicon_tags_6 },
    { 0xbec9b92e, SUC_lexicon_tags_3 },
    { 0x9f49b930, SUC_lexicon_tags_66 },
    { 0xdcc9b92f, SUC_lexicon_tags_43 },
    { 0x14c9b932, SUC_lexicon_tags_30 },
    { 0x3aa9b933, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dc9b935, SUC_lexicon_tags_17 },
    { 0x9869b935, SUC_lexicon_tags_11 },
    { 0xce9b937, SUC_lexicon_tags_8 },
    { 0xc869b937, SUC_lexicon_tags_38 },
    { 0xb2c9b939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7329b93c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8509b93e, SUC_lexicon_tags_1 },
    { 0x1689b93e, SUC_lexicon_tags_0 },
    { 0xbdc9b93f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd309b943, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec9b946, SUC_lexicon_tags_1 },
    { 0x4749b946, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e9b94b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7309b94d, SUC_lexicon_tags_1 },
    { 0x149b94e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de9b951, SUC_lexicon_tags_12 },
    { 0x2249b952, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6889b959, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed49b95c, SUC_lexicon_tags_14 },
    { 0xeb89b95d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xae9b95f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c9b964, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc889b96c, SUC_lexicon_tags_3 },
    { 0x4189b96c, SUC_lexicon_tags_11 },
    { 0x2b49b96e, SUC_lexicon_tags_19 },
    { 0x6e9b96f, SUC_lexicon_tags_79 },
    { 0x5869b970, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9469b973, SUC_lexicon_tags_17 },
    { 0xf8a9b973, SUC_lexicon_tags_25 },
    { 0x8309b975, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5e09b977, SUC_lexicon_tags_11 },
    { 0x9389b978, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15a9b97a, SUC_lexicon_tags_1 },
    { 0x61c9b97b, SUC_lexicon_tags_7 },
    { 0x7da9b97b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2489b97e, SUC_lexicon_tags_60 },
    { 0xdca9b97f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8f09b981, SUC_lexicon_tags_87 },
    { 0x169b982, SUC_lexicon_tags_27 },
    { 0xca89b983, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf729b985, SUC_lexicon_tags_58 },
    { 0x1d49b986, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b69b988, SUC_lexicon_tags_13 },
    { 0x8149b988, SUC_lexicon_tags_7 },
    { 0xafa9b98a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a29b98e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b49b993, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de9b997, SUC_lexicon_tags_38 },
    { 0xc7e9b997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9b99d, SUC_lexicon_tags_11 },
    { 0xe209b99d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2629b9a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b69b9a5, SUC_lexicon_tags_12 },
    { 0xf8e9b9a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb69b9a9, SUC_lexicon_tags_6 },
    { 0x2e89b9a9, SUC_lexicon_tags_49 },
    { 0xddc9b9a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4409b9ad, SUC_lexicon_tags_28 },
    { 0xa7a9b9ad, SUC_lexicon_tags_1 },
    { 0x6c89b9af, SUC_lexicon_tags_27 },
    { 0x1749b9af, SUC_lexicon_tags_59 },
    { 0xda29b9b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9949b9b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x47a9b9b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc689b9bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec9b9c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c9b9c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5329b9c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2969b9c8, SUC_lexicon_tags_60 },
    { 0xe629b9c9, SUC_lexicon_tags_18 },
    { 0x5289b9c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd049b9cd, SUC_lexicon_tags_46 },
    { 0xe0a9b9ce, SUC_lexicon_tags_37 },
    { 0xb0e9b9cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1229b9d2, SUC_lexicon_tags_9 },
    { 0x2369b9d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6749b9d5, SUC_lexicon_tags_11 },
    { 0xe8e9b9d6, SUC_lexicon_tags_35 },
    { 0x2ce9b9d7, SUC_lexicon_tags_8 },
    { 0xba09b9d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb09b9dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5269b9de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e9b9e4, SUC_lexicon_tags_70 },
    { 0xf289b9e5, SUC_lexicon_tags_23 },
    { 0x1989b9e4, SUC_lexicon_tags_0 },
    { 0x7b49b9e5, SUC_lexicon_tags_85 },
    { 0x7e29b9e8, SUC_lexicon_tags_49 },
    { 0x3e49b9e9, SUC_lexicon_tags_87 },
    { 0x1f89b9e9, SUC_lexicon_tags_45 },
    { 0xbfe9b9e9, SUC_lexicon_tags_30 },
    { 0x9c29b9eb, SUC_lexicon_tags_49 },
    { 0x4fc9b9e4, SUC_lexicon_tags_33 },
    { 0x4ea9b9e4, SUC_lexicon_tags_27 },
    { 0xc769b9ef, SUC_lexicon_tags_47 },
    { 0x5509b9ee, SUC_lexicon_tags_7 },
    { 0xf3c9b9f1, SUC_lexicon_tags_25 },
    { 0x149b9ee, SUC_lexicon_tags_224 },
    { 0xdf29b9f3, SUC_lexicon_tags_2 },
    { 0xbc29b9f4, SUC_lexicon_tags_10 },
    { 0x7969b9f5, SUC_lexicon_tags_62 },
    { 0x78a9b9f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae89b9fe, SUC_lexicon_tags_33 },
    { 0x50c9b9ff, SUC_lexicon_tags_2 },
    { 0x3369ba00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a9ba03, SUC_lexicon_tags_28 },
    { 0xf689ba04, SUC_lexicon_tags_1 },
    { 0xb789ba03, SUC_lexicon_tags_16 },
    { 0x9a69ba06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949ba11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a9ba14, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf09ba1f, SUC_lexicon_tags_7 },
    { 0x13a9ba20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace9ba28, SUC_lexicon_tags_12 },
    { 0x5ca9ba28, SUC_lexicon_tags_4 },
    { 0x8509ba29, SUC_lexicon_tags_19 },
    { 0x3569ba29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea9ba2e, SUC_lexicon_tags_11 },
    { 0x45a9ba2e, SUC_lexicon_tags_12 },
    { 0x5449ba2e, SUC_lexicon_tags_43 },
    { 0xf8e9ba31, SUC_lexicon_tags_25 },
    { 0xe329ba32, SUC_lexicon_tags_14 },
    { 0x1789ba2e, SUC_lexicon_tags_31 },
    { 0xf369ba34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c9ba37, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e9ba3f, SUC_lexicon_tags_12 },
    { 0x4bc9ba3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e9ba44, SUC_lexicon_tags_37 },
    { 0x16c9ba45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb89ba47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78a9ba49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a9ba4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b29ba50, SUC_lexicon_tags_6 },
    { 0xd29ba51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a89ba54, SUC_lexicon_tags_204 },
    { 0x3189ba54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6889ba57, SUC_lexicon_tags_16 },
    { 0xeae9ba58, SUC_lexicon_tags_37 },
    { 0x6729ba59, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a9ba5d, SUC_lexicon_tags_47 },
    { 0xb029ba5e, SUC_lexicon_tags_47 },
    { 0x3a29ba5e, SUC_lexicon_tags_17 },
    { 0x4709ba60, SUC_lexicon_tags_32 },
    { 0x6a29ba61, SUC_lexicon_tags_6 },
    { 0xd8a9ba62, SUC_lexicon_tags_113 },
    { 0x129ba5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c9ba6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9169ba6c, SUC_lexicon_tags_11 },
    { 0xa4e9ba6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf749ba73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb849ba7b, SUC_lexicon_tags_26 },
    { 0x96c9ba7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3629ba7e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa5a9ba80, SUC_lexicon_tags_47 },
    { 0x6749ba81, SUC_lexicon_tags_529 },
    { 0xdaa9ba81, SUC_lexicon_tags_1 },
    { 0x2609ba82, SUC_lexicon_tags_8 },
    { 0x949ba81, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee9ba89, SUC_lexicon_tags_7 },
    { 0xe409ba8a, SUC_lexicon_tags_3 },
    { 0xff09ba89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeea9ba8d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb889ba8f, SUC_lexicon_tags_38 },
    { 0xdb69ba90, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xc429ba93, SUC_lexicon_tags_63 },
    { 0xca09ba94, SUC_lexicon_tags_43 },
    { 0x80a9ba95, SUC_lexicon_tags_165 },
    { 0xb689ba93, SUC_lexicon_tags_7 },
    { 0xbd09ba96, SUC_lexicon_tags_17 },
    { 0x6b69ba95, SUC_lexicon_tags_11 },
    { 0x2689ba99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b29ba9d, SUC_lexicon_tags_86 },
    { 0x3149ba9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdea9baa0, SUC_lexicon_tags_15 },
    { 0x1369baa0, SUC_lexicon_tags_19 },
    { 0x3a29baa0, SUC_lexicon_tags_4 },
    { 0xb4c9baa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa629baa5, SUC_lexicon_tags_2 },
    { 0xd969baa5, SUC_lexicon_tags_48 },
    { 0x2909baa7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c89bab0, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5349bab4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea9bab7, SUC_lexicon_tags_37 },
    { 0x57e9bab8, SUC_lexicon_tags_19 },
    { 0x1169bab9, SUC_lexicon_tags_12 },
    { 0x5ca9baba, SUC_lexicon_tags_45 },
    { 0x1b29baba, SUC_lexicon_tags_17 },
    { 0xa4a9babc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9509babf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa09bac1, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a9bac5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc449bac7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e9baca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a9bacd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8089bacf, SUC_lexicon_tags_14 },
    { 0xb649bad0, SUC_lexicon_tags_4 },
    { 0xce49bad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb09bad6, SUC_lexicon_tags_27 },
    { 0x4d89bad7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x59c9bad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a9badf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1909bae2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6249bae4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6689bae6, SUC_lexicon_tags_26 },
    { 0x3e29bae7, SUC_lexicon_tags_37 },
    { 0xc69bae6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9ce9baea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229baf1, SUC_lexicon_tags_11 },
    { 0xae49baf2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xeaa9baf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3169baf7, SUC_lexicon_tags_11 },
    { 0x5f49baf7, SUC_lexicon_tags_29 },
    { 0x42c9baf9, SUC_lexicon_tags_25 },
    { 0x3349baf9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1409bafc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97e9bb05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60c9bb0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6169bb0e, SUC_lexicon_tags_33 },
    { 0x5129bb0f, SUC_lexicon_tags_15 },
    { 0xa89bb0f, SUC_lexicon_tags_28 },
    { 0xdd69bb11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b89bb13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef29bb1b, SUC_lexicon_tags_18 },
    { 0x1e29bb1c, SUC_lexicon_tags_47 },
    { 0xda09bb1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c09bb20, SUC_lexicon_tags_25 },
    { 0xb3c9bb21, SUC_lexicon_tags_45 },
    { 0xbac9bb22, SUC_lexicon_tags_14 },
    { 0x72e9bb22, SUC_lexicon_tags_30 },
    { 0x3629bb24, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc9bb28, SUC_lexicon_tags_37 },
    { 0x4fe9bb29, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c9bb2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549bb32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1229bb37, SUC_lexicon_tags_11 },
    { 0x3de9bb38, SUC_lexicon_tags_15 },
    { 0xab49bb39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9bb40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc49bb42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e09bb45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x909bb47, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6409bb4a, SUC_lexicon_tags_1 },
    { 0xae09bb4a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89bb50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42e9bb52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2fc9bb54, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8089bb59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7709bb5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa29bb5d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb89bb63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8849bb68, SUC_lexicon_tags_15 },
    { 0x8889bb69, SUC_lexicon_tags_14 },
    { 0x7769bb6a, SUC_lexicon_tags_26 },
    { 0xf849bb68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ba9bb6d, SUC_lexicon_tags_37 },
    { 0xf7c9bb6e, SUC_lexicon_tags_12 },
    { 0xe249bb6f, SUC_lexicon_tags_11 },
    { 0xa8a9bb70, SUC_lexicon_tags_2 },
    { 0xfd29bb70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc669bb74, SUC_lexicon_tags_11 },
    { 0x41c9bb74, SUC_lexicon_tags_17 },
    { 0xa069bb76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c9bb7a, SUC_lexicon_tags_7 },
    { 0x4f89bb7b, SUC_lexicon_tags_26 },
    { 0x37a9bb7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c89bb80, SUC_lexicon_tags_17 },
    { 0xf5a9bb80, SUC_lexicon_tags_65 },
    { 0x44e9bb81, SUC_lexicon_tags_17 },
    { 0xef69bb81, SUC_lexicon_tags_1 },
    { 0x5689bb83, SUC_lexicon_tags_12 },
    { 0x84a9bb80, SUC_lexicon_tags_37 },
    { 0x7789bb86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4329bb88, SUC_lexicon_tags_45 },
    { 0x4b09bb89, SUC_lexicon_tags_6 },
    { 0x5c09bb89, SUC_lexicon_tags_1 },
    { 0xd9e9bb89, SUC_lexicon_tags_14 },
    { 0x5a49bb8c, SUC_lexicon_tags_37 },
    { 0xb0e9bb8d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e9bb91, SUC_lexicon_tags_31 },
    { 0x52e9bb92, SUC_lexicon_tags_11 },
    { 0xee09bb93, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9589bb95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c9bb9b, SUC_lexicon_tags_12 },
    { 0x6549bb9c, SUC_lexicon_tags_37 },
    { 0x689bb9c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc449bba8, SUC_lexicon_tags_170 },
    { 0x42e9bba8, SUC_lexicon_tags_48 },
    { 0xcca9bba8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e9bbb3, SUC_lexicon_tags_27 },
    { 0x2ee9bbb4, SUC_lexicon_tags_17 },
    { 0x31e9bbb5, SUC_lexicon_tags_13 },
    { 0x73e9bbb6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeca9bbb8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8769bbbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa649bbc3, SUC_lexicon_tags_82 },
    { 0x97a9bbc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be9bbc8, SUC_lexicon_tags_25 },
    { 0xa629bbc9, SUC_lexicon_tags_65 },
    { 0x4889bbca, SUC_lexicon_tags_15 },
    { 0xcd09bbca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b09bbcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2649bbd0, SUC_lexicon_tags_17 },
    { 0x109bbd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce9bbd5, SUC_lexicon_tags_47 },
    { 0xba89bbd6, SUC_lexicon_tags_8 },
    { 0xa789bbd6, SUC_lexicon_tags_8 },
    { 0xa869bbd8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e49bbdd, SUC_lexicon_tags_29 },
    { 0x19c9bbdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1189bbe1, SUC_lexicon_tags_2 },
    { 0x13e9bbe1, SUC_lexicon_tags_25 },
    { 0xfe09bbe3, SUC_lexicon_tags_25 },
    { 0x2a89bbe4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf69bbe6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c9bbea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6249bbf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd649bbf2, SUC_lexicon_tags_15 },
    { 0x8249bbf3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbc09bbf5, SUC_lexicon_tags_7 },
    { 0xada9bbf6, SUC_lexicon_tags_26 },
    { 0xffe9bbf7, SUC_lexicon_tags_17 },
    { 0xde29bbf7, SUC_lexicon_tags_47 },
    { 0xfbe9bbf8, SUC_lexicon_tags_185 },
    { 0xc529bbfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a9bbff, SUC_lexicon_tags_95 },
    { 0xf5a9bc00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e9bc06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2609bc09, SUC_lexicon_tags_52 },
    { 0x3249bc0a, SUC_lexicon_tags_15 },
    { 0x2189bc0b, SUC_lexicon_tags_37 },
    { 0x8e69bc09, SUC_lexicon_tags_36 },
    { 0x73c9bc0d, SUC_lexicon_tags_26 },
    { 0x589bc0e, SUC_lexicon_tags_29 },
    { 0x1129bc0f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc449bc11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de9bc14, SUC_lexicon_tags_25 },
    { 0x60a9bc14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x309bc17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa29bc19, SUC_lexicon_tags_19 },
    { 0x9ea9bc19, SUC_lexicon_tags_66 },
    { 0xf1a9bc1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa4e9bc1d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa869bc20, SUC_lexicon_tags_9 },
    { 0x109bc21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da9bc26, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e9bc29, SUC_lexicon_tags_46 },
    { 0x4a89bc2a, SUC_lexicon_tags_15 },
    { 0xea89bc2a, SUC_lexicon_tags_275 },
    { 0xe669bc2c, SUC_lexicon_tags_17 },
    { 0x6589bc2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc29bc31, SUC_lexicon_tags_17 },
    { 0x8c69bc32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2b89bc34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c9bc37, SUC_lexicon_tags_356 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf229bc3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d69bc40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa9bc44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e69bc47, SUC_lexicon_tags_12 },
    { 0x59e9bc48, SUC_lexicon_tags_25 },
    { 0x65c9bc49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4a69bc4b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a9bc50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1da9bc52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba9bc54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a49bc57, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xab89bc59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c9bc61, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4729bc69, SUC_lexicon_tags_29 },
    { 0x2d09bc6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5669bc6d, SUC_lexicon_tags_60 },
    { 0x4d89bc6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5929bc71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e9bc74, SUC_lexicon_tags_37 },
    { 0x18c9bc74, SUC_lexicon_tags_17 },
    { 0x4729bc75, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a9bc7b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a9bc7e, SUC_lexicon_tags_7 },
    { 0xa4e9bc7f, SUC_lexicon_tags_2 },
    { 0x5669bc80, SUC_lexicon_tags_25 },
    { 0x7da9bc81, SUC_lexicon_tags_2 },
    { 0x6889bc82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce29bc8a, SUC_lexicon_tags_45 },
    { 0x22e9bc8b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c69bc9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e9bc9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf829bc9f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf529bca1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4189bca3, SUC_lexicon_tags_12 },
    { 0x2389bca4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe9bcab, SUC_lexicon_tags_1 },
    { 0x2a69bcac, SUC_lexicon_tags_7 },
    { 0x3889bcad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb729bcb0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x65a9bcb3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x80a9bcb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31a9bcb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91c9bcb9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3889bcbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b29bcc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c89bcc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a89bccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7689bcd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf489bcd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec9bcd6, SUC_lexicon_tags_17 },
    { 0xcc29bcd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2709bcdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e9bce0, SUC_lexicon_tags_15 },
    { 0xdd29bce1, SUC_lexicon_tags_7 },
    { 0xeb09bce0, SUC_lexicon_tags_11 },
    { 0xb689bce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe949bce6, SUC_lexicon_tags_25 },
    { 0xb149bce7, SUC_lexicon_tags_37 },
    { 0x98c9bce8, SUC_lexicon_tags_15 },
    { 0x1c09bce9, SUC_lexicon_tags_6 },
    { 0x6ee9bce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc669bcee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe249bcf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc9bcf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5009bcf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7849bcfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2369bcfe, SUC_lexicon_tags_11 },
    { 0xe529bcfe, SUC_lexicon_tags_16 },
    { 0x29c9bd00, SUC_lexicon_tags_63 },
    { 0x969bd00, SUC_lexicon_tags_43 },
    { 0xda49bcfe, SUC_lexicon_tags_27 },
    { 0x3e29bcff, SUC_lexicon_tags_17 },
    { 0x1309bd04, SUC_lexicon_tags_1 },
    { 0x4189bd05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ee9bd07, SUC_lexicon_tags_2 },
    { 0xd409bd08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf729bd0b, SUC_lexicon_tags_11 },
    { 0xc609bd0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e9bd10, SUC_lexicon_tags_12 },
    { 0xa529bd11, SUC_lexicon_tags_16 },
    { 0x4989bd11, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae9bd15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a89bd17, SUC_lexicon_tags_2 },
    { 0x5fa9bd17, SUC_lexicon_tags_43 },
    { 0x5469bd19, SUC_lexicon_tags_7 },
    { 0x5329bd1a, SUC_lexicon_tags_26 },
    { 0xd869bd1b, SUC_lexicon_tags_63 },
    { 0x6909bd17, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe069bd20, SUC_lexicon_tags_56 },
    { 0x1849bd21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3109bd27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a29bd2c, SUC_lexicon_tags_15 },
    { 0x4629bd2d, SUC_lexicon_tags_17 },
    { 0x93e9bd2e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x209bd31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2909bd3c, SUC_lexicon_tags_43 },
    { 0x9f89bd3c, SUC_lexicon_tags_6 },
    { 0x3409bd3c, SUC_lexicon_tags_33 },
    { 0x209bd3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8969bd44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9bd47, SUC_lexicon_tags_15 },
    { 0x6929bd47, SUC_lexicon_tags_45 },
    { 0xaa09bd49, SUC_lexicon_tags_2 },
    { 0xe049bd47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb589bd4d, SUC_lexicon_tags_14 },
    { 0x2f89bd4e, SUC_lexicon_tags_12 },
    { 0x1089bd4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1e9bd51, SUC_lexicon_tags_12 },
    { 0x24e9bd52, SUC_lexicon_tags_23 },
    { 0xbba9bd53, SUC_lexicon_tags_0 },
    { 0xe829bd53, SUC_lexicon_tags_33 },
    { 0xeb29bd55, SUC_lexicon_tags_63 },
    { 0xe6e9bd56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c9bd59, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3fc9bd5b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c9bd60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee9bd63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bc9bd65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a9bd6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea9bd6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fa9bd70, SUC_lexicon_tags_9 },
    { 0xd309bd71, SUC_lexicon_tags_47 },
    { 0x1329bd71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ba9bd74, SUC_lexicon_tags_87 },
    { 0x9429bd75, SUC_lexicon_tags_47 },
    { 0x7e69bd76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a9bd79, SUC_lexicon_tags_47 },
    { 0x9569bd79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a9bd7f, SUC_lexicon_tags_25 },
    { 0xed89bd7f, SUC_lexicon_tags_26 },
    { 0x1d09bd81, SUC_lexicon_tags_12 },
    { 0x5ae9bd81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e9bd85, SUC_lexicon_tags_37 },
    { 0xf769bd86, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbfa9bd88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8109bd8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69bd91, SUC_lexicon_tags_66 },
    { 0x3fa9bd91, SUC_lexicon_tags_85 },
    { 0x9289bd93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e9bd99, SUC_lexicon_tags_45 },
    { 0x72a9bd99, SUC_lexicon_tags_188 },
    { 0x1d69bd99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde9bd9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb389bda5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc89bda8, SUC_lexicon_tags_2 },
    { 0x3dc9bda9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4829bdae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5729bdb2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe849bdb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4749bdb7, SUC_lexicon_tags_11 },
    { 0xff49bdb8, SUC_lexicon_tags_106 },
    { 0xc369bdb8, SUC_lexicon_tags_79 },
    { 0x5069bdba, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8909bdc0, SUC_lexicon_tags_10 },
    { 0x5069bdc1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe009bdc4, SUC_lexicon_tags_14 },
    { 0x6429bdc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47a9bdc7, SUC_lexicon_tags_2 },
    { 0x1e29bdc7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x8de9bdca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a9bdd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeac9bdd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7a9bdd6, SUC_lexicon_tags_26 },
    { 0x8f69bdd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfa9bdd9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xab69bddb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe9bdde, SUC_lexicon_tags_60 },
    { 0x2e09bdde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe49bde1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e9bde4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e9bde8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f09bdea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f69bdf2, SUC_lexicon_tags_23 },
    { 0xac9bdf3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e69bdf6, SUC_lexicon_tags_264 },
    { 0xe049bdf6, SUC_lexicon_tags_27 },
    { 0x1e9bdf8, SUC_lexicon_tags_25 },
    { 0xfbc9bdf8, SUC_lexicon_tags_17 },
    { 0xa9e9bdfa, SUC_lexicon_tags_17 },
    { 0x7ee9bdf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc569be00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1009be06, SUC_lexicon_tags_32 },
    { 0x63c9be07, SUC_lexicon_tags_1 },
    { 0xd5a9be08, SUC_lexicon_tags_17 },
    { 0x6469be09, SUC_lexicon_tags_7 },
    { 0x6a69be09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7489be0c, SUC_lexicon_tags_25 },
    { 0x51e9be0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c9be11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3749be14, SUC_lexicon_tags_16 },
    { 0x16e9be14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e9be18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9be1a, SUC_lexicon_tags_11 },
    { 0x2a29be1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca09be21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa449be23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189be28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9dc9be2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec09be2e, SUC_lexicon_tags_79 },
    { 0x6aa9be2f, SUC_lexicon_tags_57 },
    { 0x13c9be30, SUC_lexicon_tags_9 },
    { 0x13a9be31, SUC_lexicon_tags_31 },
    { 0x7f29be32, SUC_lexicon_tags_402 },
    { 0x5589be32, SUC_lexicon_tags_31 },
    { 0x7a29be30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd229be3b, SUC_lexicon_tags_8 },
    { 0x44e9be3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfe9be3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8e9be40, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf1a9be42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3069be4e, SUC_lexicon_tags_11 },
    { 0xda09be4f, SUC_lexicon_tags_31 },
    { 0x9729be4f, SUC_lexicon_tags_25 },
    { 0x4aa9be50, SUC_lexicon_tags_19 },
    { 0x2a29be50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x41e9be54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa709be56, SUC_lexicon_tags_45 },
    { 0xa529be56, SUC_lexicon_tags_17 },
    { 0x45c9be58, SUC_lexicon_tags_47 },
    { 0x17c9be56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ce9be5b, SUC_lexicon_tags_7 },
    { 0x3389be5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ec9be5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e9be62, SUC_lexicon_tags_14 },
    { 0xbfe9be63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c9be67, SUC_lexicon_tags_30 },
    { 0xb949be67, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa889be6b, SUC_lexicon_tags_27 },
    { 0xf029be6c, SUC_lexicon_tags_17 },
    { 0xa6c9be6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad09be71, SUC_lexicon_tags_25 },
    { 0x1789be72, SUC_lexicon_tags_15 },
    { 0xe349be73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e9be78, SUC_lexicon_tags_19 },
    { 0x9849be79, SUC_lexicon_tags_45 },
    { 0xefe9be79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe609be80, SUC_lexicon_tags_45 },
    { 0x30c9be81, SUC_lexicon_tags_40 },
    { 0x8e89be82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4789be85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea9be8f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b09be98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a9be9e, SUC_lexicon_tags_141 },
    { 0x51e9be9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe9bea2, SUC_lexicon_tags_60 },
    { 0xf669bea3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d49bea6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc949beac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e9beaf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7569beb2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9beb5, SUC_lexicon_tags_87 },
    { 0x5a49beb6, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xc949beb8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9729bebb, SUC_lexicon_tags_33 },
    { 0xed49bebb, SUC_lexicon_tags_38 },
    { 0xad09bebd, SUC_lexicon_tags_1 },
    { 0x2e09bebe, SUC_lexicon_tags_0 },
    { 0xc3c9bebf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa89bec1, SUC_lexicon_tags_2 },
    { 0x1769bec2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7a9bec4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b89bec7, SUC_lexicon_tags_19 },
    { 0xbea9bec8, SUC_lexicon_tags_1 },
    { 0x7da9bec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c9becd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf269becf, SUC_lexicon_tags_15 },
    { 0x9129becf, SUC_lexicon_tags_25 },
    { 0xa389bed0, SUC_lexicon_tags_10 },
    { 0xe49bed1, SUC_lexicon_tags_1 },
    { 0x3da9bed3, SUC_lexicon_tags_35 },
    { 0xa4e9bed3, SUC_lexicon_tags_11 },
    { 0xe949bed5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc209bed9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4149bedc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ce9bede, SUC_lexicon_tags_56 },
    { 0xc9e9bedf, SUC_lexicon_tags_17 },
    { 0x3d09bee0, SUC_lexicon_tags_26 },
    { 0xc6e9bee1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3809bee5, SUC_lexicon_tags_15 },
    { 0x5189bee6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c29bee8, SUC_lexicon_tags_66 },
    { 0x69a9bee9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb889beec, SUC_lexicon_tags_7 },
    { 0x2489beed, SUC_lexicon_tags_11 },
    { 0x6229beee, SUC_lexicon_tags_0 },
    { 0x4dc9beef, SUC_lexicon_tags_56 },
    { 0xc2c9beee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e9bef5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4289bef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e9bf04, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd809bf06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e9bf0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649bf0e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x789bf10, SUC_lexicon_tags_12 },
    { 0x90a9bf11, SUC_lexicon_tags_12 },
    { 0x7769bf11, SUC_lexicon_tags_15 },
    { 0xb9c9bf11, SUC_lexicon_tags_7 },
    { 0x5369bf14, SUC_lexicon_tags_47 },
    { 0x5369bf15, SUC_lexicon_tags_25 },
    { 0x3669bf16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e9bf19, SUC_lexicon_tags_11 },
    { 0xfc29bf1a, SUC_lexicon_tags_27 },
    { 0x1c29bf1b, SUC_lexicon_tags_7 },
    { 0x3229bf1b, SUC_lexicon_tags_45 },
    { 0x3689bf1d, SUC_lexicon_tags_15 },
    { 0x8949bf1e, SUC_lexicon_tags_12 },
    { 0x2c9bf1f, SUC_lexicon_tags_43 },
    { 0xf829bf1b, SUC_lexicon_tags_37 },
    { 0x3629bf20, SUC_lexicon_tags_33 },
    { 0x6549bf22, SUC_lexicon_tags_37 },
    { 0x44a9bf23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab49bf29, SUC_lexicon_tags_6 },
    { 0x15c9bf2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2849bf31, SUC_lexicon_tags_26 },
    { 0xa209bf32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a69bf34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c9bf38, SUC_lexicon_tags_61 },
    { 0xe829bf39, SUC_lexicon_tags_7 },
    { 0x1109bf38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x11a9bf3c, SUC_lexicon_tags_57 },
    { 0x6529bf3d, SUC_lexicon_tags_17 },
    { 0x1e29bf3d, SUC_lexicon_tags_133 },
    { 0x3249bf3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c69bf43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1349bf46, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e49bf4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd69bf51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa809bf59, SUC_lexicon_tags_17 },
    { 0x41c9bf59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95c9bf5c, SUC_lexicon_tags_7 },
    { 0x29bf5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae9bf60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9bf65, SUC_lexicon_tags_65 },
    { 0x9589bf65, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3509bf6b, SUC_lexicon_tags_15 },
    { 0xb849bf6b, SUC_lexicon_tags_29 },
    { 0x65e9bf6d, SUC_lexicon_tags_45 },
    { 0x8f89bf6e, SUC_lexicon_tags_65 },
    { 0xc3e9bf6e, SUC_lexicon_tags_12 },
    { 0xff69bf6e, SUC_lexicon_tags_49 },
    { 0x7409bf6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf49bf77, SUC_lexicon_tags_17 },
    { 0xebe9bf78, SUC_lexicon_tags_7 },
    { 0x3d29bf79, SUC_lexicon_tags_7 },
    { 0xa929bf7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6469bf7c, SUC_lexicon_tags_17 },
    { 0xf749bf7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd89bf7f, SUC_lexicon_tags_37 },
    { 0xe249bf7f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c9bf83, SUC_lexicon_tags_87 },
    { 0x689bf84, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1aa9bf86, SUC_lexicon_tags_7 },
    { 0x88a9bf87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd789bf8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8069bf8f, SUC_lexicon_tags_11 },
    { 0x9b49bf90, SUC_lexicon_tags_7 },
    { 0xb9e9bf90, SUC_lexicon_tags_37 },
    { 0xe7a9bf92, SUC_lexicon_tags_17 },
    { 0x9d69bf93, SUC_lexicon_tags_31 },
    { 0x3e89bf93, SUC_lexicon_tags_14 },
    { 0xafa9bf92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9389bf97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a69bfa4, SUC_lexicon_tags_177 },
    { 0xa469bfa5, SUC_lexicon_tags_2 },
    { 0x1e09bfa4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcb49bfa8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc269bfad, SUC_lexicon_tags_16 },
    { 0x8ba9bfad, SUC_lexicon_tags_12 },
    { 0xf5c9bfaf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe349bfb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x389bfb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2069bfb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc049bfbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe189bfc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a9bfcb, SUC_lexicon_tags_17 },
    { 0x1c29bfcb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c9bfcf, SUC_lexicon_tags_7 },
    { 0x30e9bfcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e89bfd3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd809bfd8, SUC_lexicon_tags_1 },
    { 0xdcc9bfd8, SUC_lexicon_tags_54 },
    { 0x7749bfda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bfdd, SUC_lexicon_tags_65 },
    { 0x4549bfde, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa309bfe2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c9bfe6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x68e9bfe9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1a9bfeb, SUC_lexicon_tags_15 },
    { 0x63c9bfeb, SUC_lexicon_tags_45 },
    { 0x9e89bfed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf69bff0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2989bff7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c49bffa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa729bffc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f29c006, SUC_lexicon_tags_1 },
    { 0x2e9c006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c29c009, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be9c00c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2209c00e, SUC_lexicon_tags_17 },
    { 0x1809c00f, SUC_lexicon_tags_37 },
    { 0x98a9c010, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xeee9c013, SUC_lexicon_tags_55 },
    { 0x3da9c014, SUC_lexicon_tags_1 },
    { 0x64a9c014, SUC_lexicon_tags_1 },
    { 0xde49c013, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc89c019, SUC_lexicon_tags_45 },
    { 0x49a9c01a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec9c01e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc929c025, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9409c028, SUC_lexicon_tags_103 },
    { 0xa509c029, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2769c02d, SUC_lexicon_tags_19 },
    { 0x29c02e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xed29c031, SUC_lexicon_tags_47 },
    { 0xdb29c032, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc9c037, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8549c03a, SUC_lexicon_tags_2 },
    { 0xcee9c03a, SUC_lexicon_tags_15 },
    { 0x36c9c03c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4689c03f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2829c041, SUC_lexicon_tags_30 },
    { 0x7c89c041, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa129c044, SUC_lexicon_tags_33 },
    { 0x47e9c045, SUC_lexicon_tags_1 },
    { 0x6769c046, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e9c04b, SUC_lexicon_tags_14 },
    { 0x5ca9c04c, SUC_lexicon_tags_32 },
    { 0x6ee9c04d, SUC_lexicon_tags_69 },
    { 0x3ec9c04e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e9c054, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a9c059, SUC_lexicon_tags_1 },
    { 0xb789c05a, SUC_lexicon_tags_23 },
    { 0xbf69c05b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf729c05d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe069c05f, SUC_lexicon_tags_27 },
    { 0x6489c05f, SUC_lexicon_tags_11 },
    { 0x5f29c061, SUC_lexicon_tags_19 },
    { 0x1829c062, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec9c064, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d09c068, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729c06d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc209c070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe789c085, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc469c092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c9c094, SUC_lexicon_tags_2 },
    { 0x1c89c095, SUC_lexicon_tags_16 },
    { 0xc29c096, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd609c098, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a9c09c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3609c0a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9349c0a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b89c0aa, SUC_lexicon_tags_2 },
    { 0x2ce9c0ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2729c0ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde49c0b0, SUC_lexicon_tags_11 },
    { 0x7369c0b1, SUC_lexicon_tags_37 },
    { 0x8789c0b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa9c0b6, SUC_lexicon_tags_1 },
    { 0xf289c0b7, SUC_lexicon_tags_19 },
    { 0x829c0b7, SUC_lexicon_tags_23 },
    { 0xe1e9c0b8, SUC_lexicon_tags_28 },
    { 0x5d29c0b9, SUC_lexicon_tags_15 },
    { 0x8849c0b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8529c0c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c9c0c4, SUC_lexicon_tags_2 },
    { 0x6d29c0c4, SUC_lexicon_tags_10 },
    { 0x269c0c6, SUC_lexicon_tags_12 },
    { 0x1789c0c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcc49c0c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9c0ce, SUC_lexicon_tags_1 },
    { 0xc549c0cf, SUC_lexicon_tags_15 },
    { 0xf029c0ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a89c0d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2a9c0d4, SUC_lexicon_tags_14 },
    { 0x4389c0d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc9c0d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc09c0df, SUC_lexicon_tags_25 },
    { 0x729c0e0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c29c0e3, SUC_lexicon_tags_11 },
    { 0x9da9c0e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x11c9c0e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x82e9c0e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e29c0ed, SUC_lexicon_tags_1 },
    { 0x3349c0ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe429c0f6, SUC_lexicon_tags_45 },
    { 0x41c9c0f7, SUC_lexicon_tags_37 },
    { 0x2c89c0f8, SUC_lexicon_tags_37 },
    { 0x2909c0f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe949c0fe, SUC_lexicon_tags_28 },
    { 0xb609c0fe, SUC_lexicon_tags_11 },
    { 0x26a9c0ff, SUC_lexicon_tags_8 },
    { 0xa8a9c0ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc9c106, SUC_lexicon_tags_1 },
    { 0x21a9c106, SUC_lexicon_tags_8 },
    { 0x9b49c108, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9189c10c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9409c10e, SUC_lexicon_tags_12 },
    { 0x48e9c10e, SUC_lexicon_tags_15 },
    { 0xe09c110, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ea9c112, SUC_lexicon_tags_2 },
    { 0xafa9c113, SUC_lexicon_tags_37 },
    { 0x1509c112, SUC_lexicon_tags_17 },
    { 0xb5c9c115, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c69c117, SUC_lexicon_tags_17 },
    { 0xfe49c117, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea9c11c, SUC_lexicon_tags_194 },
    { 0x1ee9c11d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5829c120, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5809c122, SUC_lexicon_tags_28 },
    { 0xd329c122, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d69c128, SUC_lexicon_tags_30 },
    { 0x6009c129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d69c12b, SUC_lexicon_tags_0 },
    { 0xb329c12c, SUC_lexicon_tags_45 },
    { 0x8b49c12d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf549c12f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a9c135, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b29c138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5689c13a, SUC_lexicon_tags_26 },
    { 0x8ec9c13b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc69c13d, SUC_lexicon_tags_43 },
    { 0xde09c13d, SUC_lexicon_tags_19 },
    { 0x9ba9c13f, SUC_lexicon_tags_12 },
    { 0x2489c140, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1449c144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a9c148, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf709c14d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6609c153, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8ac9c155, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89c159, SUC_lexicon_tags_32 },
    { 0x9e9c15a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1149c15d, SUC_lexicon_tags_47 },
    { 0xcec9c15e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66c9c161, SUC_lexicon_tags_2 },
    { 0xff89c162, SUC_lexicon_tags_15 },
    { 0xa989c161, SUC_lexicon_tags_25 },
    { 0x9289c163, SUC_lexicon_tags_0 },
    { 0x5ba9c162, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a9c169, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe209c16e, SUC_lexicon_tags_39 },
    { 0x14c9c16f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1909c171, SUC_lexicon_tags_7 },
    { 0x29c9c172, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8969c175, SUC_lexicon_tags_151 },
    { 0x1f69c176, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e9c17b, SUC_lexicon_tags_19 },
    { 0xca29c17b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a9c189, SUC_lexicon_tags_14 },
    { 0x14a9c18a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a9c18e, SUC_lexicon_tags_12 },
    { 0xd929c18f, SUC_lexicon_tags_11 },
    { 0xe649c18f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe9c197, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31a9c199, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7989c1a1, SUC_lexicon_tags_8 },
    { 0x3e49c1a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3e9c1a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa09c1a6, SUC_lexicon_tags_2 },
    { 0xb3a9c1a6, SUC_lexicon_tags_37 },
    { 0xc5c9c1a6, SUC_lexicon_tags_87 },
    { 0x76a9c1a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x66c9c1ab, SUC_lexicon_tags_37 },
    { 0x5dc9c1ac, SUC_lexicon_tags_47 },
    { 0x71e9c1ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a9c1b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3569c1b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a9c1bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6569c1c0, SUC_lexicon_tags_15 },
    { 0x2889c1c1, SUC_lexicon_tags_30 },
    { 0x67c9c1c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf489c1c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa529c1cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa349c1d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a09c1de, SUC_lexicon_tags_13 },
    { 0x6789c1df, SUC_lexicon_tags_7 },
    { 0xf2e9c1e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca9c1ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf09c1ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd149c1f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea9c1f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b29c1ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe29c202, SUC_lexicon_tags_23 },
    { 0xa049c203, SUC_lexicon_tags_12 },
    { 0x2f29c204, SUC_lexicon_tags_30 },
    { 0xe129c204, SUC_lexicon_tags_17 },
    { 0x3429c205, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc809c20a, SUC_lexicon_tags_68 },
    { 0x7a89c20b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9169c20f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26e9c211, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a9c21b, SUC_lexicon_tags_69 },
    { 0x1d29c21c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e09c21e, SUC_lexicon_tags_37 },
    { 0x57e9c21e, SUC_lexicon_tags_68 },
    { 0x3b09c220, SUC_lexicon_tags_12 },
    { 0x19c9c221, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe909c223, SUC_lexicon_tags_57 },
    { 0x6b49c224, SUC_lexicon_tags_11 },
    { 0x6c69c225, SUC_lexicon_tags_45 },
    { 0x9209c226, SUC_lexicon_tags_37 },
    { 0xfa49c225, SUC_lexicon_tags_47 },
    { 0x5da9c228, SUC_lexicon_tags_23 },
    { 0x2c29c228, SUC_lexicon_tags_107 },
    { 0x3b29c22a, SUC_lexicon_tags_32 },
    { 0x3469c22a, SUC_lexicon_tags_8 },
    { 0xbe29c22c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5889c231, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf69c233, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee49c235, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1849c23a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac9c23d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb789c240, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb689c249, SUC_lexicon_tags_1 },
    { 0xad09c249, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc29c24c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4069c250, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8349c257, SUC_lexicon_tags_0 },
    { 0xc3c9c257, SUC_lexicon_tags_60 },
    { 0xe409c259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9c25c, SUC_lexicon_tags_12 },
    { 0x9e89c25c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2849c25f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff49c261, SUC_lexicon_tags_17 },
    { 0x2969c261, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4009c264, SUC_lexicon_tags_37 },
    { 0xf909c265, SUC_lexicon_tags_15 },
    { 0xaba9c266, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d49c269, SUC_lexicon_tags_15 },
    { 0x2e9c26a, SUC_lexicon_tags_11 },
    { 0xc929c26b, SUC_lexicon_tags_25 },
    { 0xc69c26c, SUC_lexicon_tags_1 },
    { 0x3b29c26d, SUC_lexicon_tags_23 },
    { 0xd89c26e, SUC_lexicon_tags_26 },
    { 0xb1e9c26d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee9c274, SUC_lexicon_tags_1 },
    { 0xe1c9c274, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2549c277, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa6c9c279, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1389c27c, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0xb9c9c27e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4cc9c280, SUC_lexicon_tags_27 },
    { 0x3289c281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889c287, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1749c289, SUC_lexicon_tags_3 },
    { 0x9589c289, SUC_lexicon_tags_43 },
    { 0x7c69c28b, SUC_lexicon_tags_10 },
    { 0x9669c28c, SUC_lexicon_tags_27 },
    { 0x8ba9c28d, SUC_lexicon_tags_11 },
    { 0x80e9c28e, SUC_lexicon_tags_39 },
    { 0xa669c28f, SUC_lexicon_tags_46 },
    { 0xa8c9c28c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e9c293, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8a09c295, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d49c299, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc9c29d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d69c2a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb089c2a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b89c2a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b49c2a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a9c2ab, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9c2b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a29c2b3, SUC_lexicon_tags_25 },
    { 0xc029c2b3, SUC_lexicon_tags_15 },
    { 0x5409c2b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc9c2b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd549c2bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf349c2ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3829c2cd, SUC_lexicon_tags_47 },
    { 0x8e49c2cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d29c2d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a9c2dc, SUC_lexicon_tags_12 },
    { 0x4869c2dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0c9c2df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3fe9c2e1, SUC_lexicon_tags_12 },
    { 0x1c29c2e2, SUC_lexicon_tags_39 },
    { 0x7a29c2e3, SUC_lexicon_tags_9 },
    { 0x8de9c2e4, SUC_lexicon_tags_12 },
    { 0x8d09c2e5, SUC_lexicon_tags_27 },
    { 0x1049c2e5, SUC_lexicon_tags_28 },
    { 0xd369c2e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2149c2ed, SUC_lexicon_tags_25 },
    { 0x41a9c2ee, SUC_lexicon_tags_44 },
    { 0xac69c2ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8609c2f1, SUC_lexicon_tags_11 },
    { 0x7649c2f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a9c2f6, SUC_lexicon_tags_5 },
    { 0xee09c2f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb89c2ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe769c304, SUC_lexicon_tags_60 },
    { 0x709c304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e9c309, SUC_lexicon_tags_15 },
    { 0xca9c309, SUC_lexicon_tags_17 },
    { 0x969c309, SUC_lexicon_tags_17 },
    { 0x3869c309, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc69c313, SUC_lexicon_tags_30 },
    { 0x6da9c314, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc729c318, SUC_lexicon_tags_57 },
    { 0x6b49c318, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb09c31c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f29c320, SUC_lexicon_tags_37 },
    { 0xa349c320, SUC_lexicon_tags_60 },
    { 0x1ce9c320, SUC_lexicon_tags_15 },
    { 0x6e49c323, SUC_lexicon_tags_19 },
    { 0xa889c324, SUC_lexicon_tags_31 },
    { 0x80e9c325, SUC_lexicon_tags_16 },
    { 0x9ba9c324, SUC_lexicon_tags_25 },
    { 0x4869c322, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe729c329, SUC_lexicon_tags_60 },
    { 0x4ea9c329, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3729c32d, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x2029c330, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c9c338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a9c33b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c9c344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0c9c346, SUC_lexicon_tags_14 },
    { 0x28c9c346, SUC_lexicon_tags_19 },
    { 0xa1a9c346, SUC_lexicon_tags_25 },
    { 0x1009c347, SUC_lexicon_tags_15 },
    { 0xa5e9c34a, SUC_lexicon_tags_9 },
    { 0x9fc9c34a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb2a9c34d, SUC_lexicon_tags_14 },
    { 0xe2e9c34d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c29c351, SUC_lexicon_tags_25 },
    { 0x19a9c351, SUC_lexicon_tags_30 },
    { 0xc749c352, SUC_lexicon_tags_14 },
    { 0x72a9c352, SUC_lexicon_tags_25 },
    { 0x7669c355, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9069c359, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa309c35d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x11e9c35f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe9c365, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a49c368, SUC_lexicon_tags_63 },
    { 0x8889c369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4589c36e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa09c375, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88e9c377, SUC_lexicon_tags_14 },
    { 0x1e89c377, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0a9c37a, SUC_lexicon_tags_19 },
    { 0x4ae9c37b, SUC_lexicon_tags_60 },
    { 0x7769c37b, SUC_lexicon_tags_25 },
    { 0xfdc9c37d, SUC_lexicon_tags_27 },
    { 0x4769c37d, SUC_lexicon_tags_1 },
    { 0xa7a9c37f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8da9c381, SUC_lexicon_tags_47 },
    { 0xdc09c381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5809c387, SUC_lexicon_tags_37 },
    { 0x7aa9c388, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b89c390, SUC_lexicon_tags_25 },
    { 0xa429c391, SUC_lexicon_tags_39 },
    { 0x5d29c391, SUC_lexicon_tags_12 },
    { 0xed69c393, SUC_lexicon_tags_11 },
    { 0x4e69c393, SUC_lexicon_tags_2 },
    { 0x5789c395, SUC_lexicon_tags_15 },
    { 0x3d89c393, SUC_lexicon_tags_28 },
    { 0x95a9c397, SUC_lexicon_tags_11 },
    { 0x48a9c398, SUC_lexicon_tags_15 },
    { 0x6509c395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe589c39b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4009c3a1, SUC_lexicon_tags_15 },
    { 0xce29c3a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb629c3a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe589c3a9, SUC_lexicon_tags_2 },
    { 0xeb89c3a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1889c3ad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc269c3b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xff09c3b5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa3e9c3b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa589c3be, SUC_lexicon_tags_37 },
    { 0x809c3bf, SUC_lexicon_tags_17 },
    { 0x1c9c3c0, SUC_lexicon_tags_37 },
    { 0xe049c3c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe9e9c3c3, SUC_lexicon_tags_25 },
    { 0xed49c3c4, SUC_lexicon_tags_25 },
    { 0x2de9c3c3, SUC_lexicon_tags_37 },
    { 0x9169c3c6, SUC_lexicon_tags_7 },
    { 0xea09c3c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f09c3cc, SUC_lexicon_tags_7 },
    { 0x8dc9c3cc, SUC_lexicon_tags_7 },
    { 0x47a9c3cd, SUC_lexicon_tags_11 },
    { 0xdf49c3cf, SUC_lexicon_tags_31 },
    { 0xba9c3d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe829c3d2, SUC_lexicon_tags_12 },
    { 0x6969c3d3, SUC_lexicon_tags_68 },
    { 0x1349c3d3, SUC_lexicon_tags_65 },
    { 0x4ee9c3d5, SUC_lexicon_tags_34 },
    { 0x1209c3d5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb7e9c3d8, SUC_lexicon_tags_49 },
    { 0x1469c3d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b69c3dc, SUC_lexicon_tags_47 },
    { 0x8909c3dc, SUC_lexicon_tags_37 },
    { 0x1989c3dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a9c3e3, SUC_lexicon_tags_12 },
    { 0x5dc9c3e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd89c3e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52a9c3e8, SUC_lexicon_tags_87 },
    { 0x1669c3e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b49c3ec, SUC_lexicon_tags_11 },
    { 0x4909c3ed, SUC_lexicon_tags_17 },
    { 0xf549c3ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf89c3f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c9c3fd, SUC_lexicon_tags_37 },
    { 0xbc9c3fd, SUC_lexicon_tags_17 },
    { 0x9089c3ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6369c401, SUC_lexicon_tags_87 },
    { 0xfc69c402, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcba9c404, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a9c409, SUC_lexicon_tags_65 },
    { 0xfcc9c409, SUC_lexicon_tags_40 },
    { 0xab29c40b, SUC_lexicon_tags_7 },
    { 0x2109c40b, SUC_lexicon_tags_45 },
    { 0x9b89c40b, SUC_lexicon_tags_17 },
    { 0x9ca9c40e, SUC_lexicon_tags_37 },
    { 0x6ce9c40c, SUC_lexicon_tags_61 },
    { 0x1829c410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac9c415, SUC_lexicon_tags_25 },
    { 0x6f49c416, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c9c41b, SUC_lexicon_tags_63 },
    { 0xd9e9c41b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c9c420, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e9c424, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf769c42a, SUC_lexicon_tags_30 },
    { 0x4a49c42b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb89c42e, SUC_lexicon_tags_11 },
    { 0xb449c42e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xad09c432, SUC_lexicon_tags_2 },
    { 0x9809c433, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7829c435, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea69c439, SUC_lexicon_tags_17 },
    { 0xbc29c439, SUC_lexicon_tags_3 },
    { 0xb889c43b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa849c43d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2609c441, SUC_lexicon_tags_37 },
    { 0x67c9c442, SUC_lexicon_tags_38 },
    { 0x2e49c443, SUC_lexicon_tags_37 },
    { 0xbb09c441, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f09c449, SUC_lexicon_tags_25 },
    { 0xa6a9c44a, SUC_lexicon_tags_27 },
    { 0x9329c44b, SUC_lexicon_tags_1 },
    { 0xd629c44c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be9c451, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xea69c453, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8929c455, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb349c457, SUC_lexicon_tags_63 },
    { 0x3849c458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c9c45e, SUC_lexicon_tags_1 },
    { 0x349c45f, SUC_lexicon_tags_23 },
    { 0xfce9c45e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd069c462, SUC_lexicon_tags_12 },
    { 0x9f89c463, SUC_lexicon_tags_27 },
    { 0x9ca9c463, SUC_lexicon_tags_7 },
    { 0x54e9c464, SUC_lexicon_tags_27 },
    { 0xa569c464, SUC_lexicon_tags_35 },
    { 0x9e9c467, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86e9c46a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16a9c46d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9529c46f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea9c47b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4109c47e, SUC_lexicon_tags_47 },
    { 0xd929c47e, SUC_lexicon_tags_11 },
    { 0x6aa9c47e, SUC_lexicon_tags_1 },
    { 0x3269c481, SUC_lexicon_tags_11 },
    { 0x30c9c481, SUC_lexicon_tags_17 },
    { 0x8d69c480, SUC_lexicon_tags_30 },
    { 0x6c49c484, SUC_lexicon_tags_63 },
    { 0x7a9c47e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b29c488, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa49c48c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c9c491, SUC_lexicon_tags_49 },
    { 0xd009c492, SUC_lexicon_tags_29 },
    { 0x89c9c493, SUC_lexicon_tags_23 },
    { 0x2a09c494, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a89c497, SUC_lexicon_tags_11 },
    { 0xa929c498, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9749c49d, SUC_lexicon_tags_17 },
    { 0x3669c49e, SUC_lexicon_tags_8 },
    { 0x53a9c49f, SUC_lexicon_tags_1 },
    { 0x9b49c4a0, SUC_lexicon_tags_15 },
    { 0x7e49c4a1, SUC_lexicon_tags_60 },
    { 0xa829c4a2, SUC_lexicon_tags_29 },
    { 0xaba9c4a1, SUC_lexicon_tags_60 },
    { 0x3d09c4a4, SUC_lexicon_tags_25 },
    { 0x7629c4a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2969c4a9, SUC_lexicon_tags_11 },
    { 0x8e89c4a9, SUC_lexicon_tags_30 },
    { 0x9089c4a9, SUC_lexicon_tags_17 },
    { 0x7549c4a9, SUC_lexicon_tags_40 },
    { 0xfe49c4ad, SUC_lexicon_tags_92 },
    { 0xec89c4ad, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xa009c4b0, SUC_lexicon_tags_14 },
    { 0x85a9c4b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6829c4b7, SUC_lexicon_tags_17 },
    { 0x8e89c4b8, SUC_lexicon_tags_11 },
    { 0xf029c4b8, SUC_lexicon_tags_6 },
    { 0x3ac9c4ba, SUC_lexicon_tags_11 },
    { 0x7109c4bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5589c4bd, SUC_lexicon_tags_1 },
    { 0x46c9c4be, SUC_lexicon_tags_8 },
    { 0x5d49c4be, SUC_lexicon_tags_0 },
    { 0x2c49c4bd, SUC_lexicon_tags_37 },
    { 0x75c9c4be, SUC_lexicon_tags_28 },
    { 0xe29c4c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf469c4c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb69c4cc, SUC_lexicon_tags_9 },
    { 0x769c4cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4089c4cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f49c4d2, SUC_lexicon_tags_15 },
    { 0x2bc9c4d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb029c4d5, SUC_lexicon_tags_26 },
    { 0x8329c4d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d29c4d8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b69c4dc, SUC_lexicon_tags_1 },
    { 0x3aa9c4dd, SUC_lexicon_tags_25 },
    { 0x1089c4de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe9c4e7, SUC_lexicon_tags_9 },
    { 0x24e9c4e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe489c4ed, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5949c4ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c9c4fa, SUC_lexicon_tags_28 },
    { 0x1249c4fb, SUC_lexicon_tags_2 },
    { 0x1409c4fa, SUC_lexicon_tags_14 },
    { 0x2009c4fd, SUC_lexicon_tags_33 },
    { 0xc309c4fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe849c500, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a9c504, SUC_lexicon_tags_72 },
    { 0x4b69c504, SUC_lexicon_tags_26 },
    { 0x61c9c504, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c9c508, SUC_lexicon_tags_48 },
    { 0x6289c508, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe069c50d, SUC_lexicon_tags_15 },
    { 0xba69c50d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d09c518, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa0a9c51a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa929c51c, SUC_lexicon_tags_17 },
    { 0x4f69c51c, SUC_lexicon_tags_6 },
    { 0x7269c51e, SUC_lexicon_tags_17 },
    { 0x1c29c51d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a9c524, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe629c52d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3a9c52f, SUC_lexicon_tags_11 },
    { 0xaba9c530, SUC_lexicon_tags_12 },
    { 0x1e9c530, SUC_lexicon_tags_15 },
    { 0x8c49c532, SUC_lexicon_tags_2 },
    { 0x7969c533, SUC_lexicon_tags_25 },
    { 0xce49c534, SUC_lexicon_tags_17 },
    { 0x4ba9c534, SUC_lexicon_tags_33 },
    { 0x1c09c534, SUC_lexicon_tags_4 },
    { 0xcd49c537, SUC_lexicon_tags_2 },
    { 0x16e9c535, SUC_lexicon_tags_68 },
    { 0xc69c539, SUC_lexicon_tags_4 },
    { 0x4b49c534, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd529c53d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9209c540, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed49c549, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4089c54d, SUC_lexicon_tags_45 },
    { 0x90e9c54e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8809c550, SUC_lexicon_tags_37 },
    { 0xeb49c551, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f49c55b, SUC_lexicon_tags_1 },
    { 0x64e9c55c, SUC_lexicon_tags_17 },
    { 0x85e9c55d, SUC_lexicon_tags_4 },
    { 0x2609c55b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa469c562, SUC_lexicon_tags_28 },
    { 0x7ae9c562, SUC_lexicon_tags_45 },
    { 0xfac9c564, SUC_lexicon_tags_1 },
    { 0x25c9c565, SUC_lexicon_tags_15 },
    { 0x83a9c565, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc9c56b, SUC_lexicon_tags_37 },
    { 0xbe9c56b, SUC_lexicon_tags_1 },
    { 0x6069c56c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa309c570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd649c573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6249c577, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2949c579, SUC_lexicon_tags_18 },
    { 0x8629c579, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a89c57d, SUC_lexicon_tags_7 },
    { 0xe829c57e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5309c580, SUC_lexicon_tags_2 },
    { 0xe89c580, SUC_lexicon_tags_12 },
    { 0x2789c582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9c585, SUC_lexicon_tags_12 },
    { 0xf8e9c585, SUC_lexicon_tags_63 },
    { 0x1ea9c585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8449c58b, SUC_lexicon_tags_39 },
    { 0x7fe9c58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e9c591, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x35c9c593, SUC_lexicon_tags_2 },
    { 0xdae9c594, SUC_lexicon_tags_11 },
    { 0xcaa9c595, SUC_lexicon_tags_33 },
    { 0x97c9c593, SUC_lexicon_tags_1 },
    { 0x5469c596, SUC_lexicon_tags_25 },
    { 0x8b29c596, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec29c59d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789c5a3, SUC_lexicon_tags_2 },
    { 0xe6c9c5a3, SUC_lexicon_tags_17 },
    { 0xe4c9c5a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88a9c5a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ce9c5a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a29c5ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87e9c5af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2529c5b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a9c5b6, SUC_lexicon_tags_25 },
    { 0xcba9c5b7, SUC_lexicon_tags_107 },
    { 0xa7e9c5b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f09c5c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a29c5c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf69c5c7, SUC_lexicon_tags_73 },
    { 0x8a49c5c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac9c5d1, SUC_lexicon_tags_17 },
    { 0x6ae9c5d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ba9c5d4, SUC_lexicon_tags_15 },
    { 0x9ee9c5d4, SUC_lexicon_tags_25 },
    { 0x9f29c5d6, SUC_lexicon_tags_1 },
    { 0x7629c5d7, SUC_lexicon_tags_14 },
    { 0xcac9c5d6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6e29c5da, SUC_lexicon_tags_1 },
    { 0x2ea9c5db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc409c5e1, SUC_lexicon_tags_37 },
    { 0xdd49c5e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ee9c5e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8989c5e7, SUC_lexicon_tags_8 },
    { 0x4769c5e8, SUC_lexicon_tags_37 },
    { 0x2329c5e9, SUC_lexicon_tags_25 },
    { 0xbae9c5ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3fe9c5ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a89c5ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8289c5f4, SUC_lexicon_tags_27 },
    { 0x8a09c5f5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8649c5f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4569c600, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc849c603, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e69c608, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e89c60a, SUC_lexicon_tags_1 },
    { 0x929c60b, SUC_lexicon_tags_37 },
    { 0x5529c60c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc9c60f, SUC_lexicon_tags_37 },
    { 0x4249c610, SUC_lexicon_tags_30 },
    { 0xa1e9c611, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xab09c613, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1469c618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa09c61a, SUC_lexicon_tags_26 },
    { 0x49a9c61b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2089c621, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa9c625, SUC_lexicon_tags_1 },
    { 0xd8a9c626, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa49c630, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8fe9c632, SUC_lexicon_tags_12 },
    { 0x2529c633, SUC_lexicon_tags_25 },
    { 0x9da9c632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec69c637, SUC_lexicon_tags_10 },
    { 0x8169c638, SUC_lexicon_tags_28 },
    { 0x2229c637, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x99c9c63b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf629c640, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc9c644, SUC_lexicon_tags_15 },
    { 0x5c89c644, SUC_lexicon_tags_6 },
    { 0x9b09c645, SUC_lexicon_tags_15 },
    { 0x6449c647, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca9c64c, SUC_lexicon_tags_25 },
    { 0x2b29c64d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2a89c64f, SUC_lexicon_tags_23 },
    { 0xc8c9c650, SUC_lexicon_tags_28 },
    { 0x52c9c651, SUC_lexicon_tags_1 },
    { 0x89c652, SUC_lexicon_tags_66 },
    { 0xe49c653, SUC_lexicon_tags_12 },
    { 0xcc69c64f, SUC_lexicon_tags_8 },
    { 0xa6e9c653, SUC_lexicon_tags_45 },
    { 0xbda9c654, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe49c658, SUC_lexicon_tags_43 },
    { 0xf5e9c659, SUC_lexicon_tags_43 },
    { 0x3469c65a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c9c65f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf829c666, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec09c669, SUC_lexicon_tags_112 },
    { 0x5aa9c66a, SUC_lexicon_tags_16 },
    { 0x4149c66b, SUC_lexicon_tags_46 },
    { 0xa509c66c, SUC_lexicon_tags_15 },
    { 0x9849c669, SUC_lexicon_tags_14 },
    { 0xd129c66e, SUC_lexicon_tags_15 },
    { 0x52a9c66f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda69c673, SUC_lexicon_tags_12 },
    { 0x7149c673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c9c679, SUC_lexicon_tags_45 },
    { 0x3ec9c67a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ac9c67c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a9c681, SUC_lexicon_tags_2 },
    { 0xfe49c681, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcf29c684, SUC_lexicon_tags_49 },
    { 0x69c9c685, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf289c687, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c89c689, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9869c68e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f29c693, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe429c69a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe09c69d, SUC_lexicon_tags_11 },
    { 0xd269c69d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7f49c6a0, SUC_lexicon_tags_17 },
    { 0x2309c6a1, SUC_lexicon_tags_11 },
    { 0xb429c6a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f89c6a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09c6a9, SUC_lexicon_tags_29 },
    { 0xa869c6a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa649c6ac, SUC_lexicon_tags_37 },
    { 0x71a9c6ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fc9c6af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a29c6b5, SUC_lexicon_tags_15 },
    { 0x5269c6b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7a49c6b8, SUC_lexicon_tags_2 },
    { 0xfca9c6b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7729c6bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccc9c6be, SUC_lexicon_tags_27 },
    { 0x4329c6bf, SUC_lexicon_tags_12 },
    { 0x2f69c6c0, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xb169c6c3, SUC_lexicon_tags_2 },
    { 0x6de9c6c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac9c6cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x209c6ce, SUC_lexicon_tags_3 },
    { 0xaee9c6cf, SUC_lexicon_tags_69 },
    { 0x2489c6d0, SUC_lexicon_tags_31 },
    { 0x56a9c6ce, SUC_lexicon_tags_3 },
    { 0xf529c6d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc9c6d6, SUC_lexicon_tags_15 },
    { 0x1ea9c6d7, SUC_lexicon_tags_17 },
    { 0x5b09c6d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea9c6db, SUC_lexicon_tags_12 },
    { 0x7709c6dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc469c6de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc9c6e1, SUC_lexicon_tags_87 },
    { 0xafa9c6e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a9c6e6, SUC_lexicon_tags_17 },
    { 0x7429c6e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d89c6e9, SUC_lexicon_tags_45 },
    { 0x9b89c6ea, SUC_lexicon_tags_17 },
    { 0xe429c6ea, SUC_lexicon_tags_7 },
    { 0x4e09c6ea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9c6f1, SUC_lexicon_tags_43 },
    { 0x3f69c6f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4629c6f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8669c6f8, SUC_lexicon_tags_14 },
    { 0xd729c6f8, SUC_lexicon_tags_30 },
    { 0x2729c6f8, SUC_lexicon_tags_27 },
    { 0xc3c9c6f9, SUC_lexicon_tags_9 },
    { 0x889c6fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9289c700, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee49c702, SUC_lexicon_tags_37 },
    { 0x9569c702, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf169c705, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca9c70f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc769c71a, SUC_lexicon_tags_2 },
    { 0xc909c71b, SUC_lexicon_tags_37 },
    { 0xad89c71a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41a9c720, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb89c724, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7949c728, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf909c72d, SUC_lexicon_tags_14 },
    { 0x7f89c72e, SUC_lexicon_tags_31 },
    { 0xb449c72f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x78a9c731, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf3e9c733, SUC_lexicon_tags_60 },
    { 0x1249c733, SUC_lexicon_tags_7 },
    { 0x9109c735, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc9c73d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c49c744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f69c74a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe629c751, SUC_lexicon_tags_2 },
    { 0xd7c9c752, SUC_lexicon_tags_7 },
    { 0x2d89c752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e29c756, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1c09c758, SUC_lexicon_tags_37 },
    { 0xafc9c759, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe6a9c75b, SUC_lexicon_tags_2 },
    { 0x66c9c75c, SUC_lexicon_tags_12 },
    { 0x9a9c75c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1329c75f, SUC_lexicon_tags_17 },
    { 0x97e9c760, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xab29c763, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c9c768, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a9c76d, SUC_lexicon_tags_32 },
    { 0x8c29c76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5009c771, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c29c77b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8229c77d, SUC_lexicon_tags_15 },
    { 0x6cc9c77d, SUC_lexicon_tags_11 },
    { 0x7309c77f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1489c78f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8329c793, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6149c797, SUC_lexicon_tags_8 },
    { 0x7e09c798, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8669c79c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de9c79f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b29c7a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9329c7a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5289c7ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1769c7b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfde9c7b7, SUC_lexicon_tags_65 },
    { 0x5aa9c7b8, SUC_lexicon_tags_12 },
    { 0xbd29c7b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6629c7bc, SUC_lexicon_tags_1 },
    { 0x92a9c7bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8ee9c7bf, SUC_lexicon_tags_7 },
    { 0xb4c9c7bf, SUC_lexicon_tags_17 },
    { 0x4029c7c1, SUC_lexicon_tags_38 },
    { 0xfb49c7c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e9c7c7, SUC_lexicon_tags_49 },
    { 0xe869c7c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7c49c7ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc009c7cf, SUC_lexicon_tags_27 },
    { 0x51a9c7cf, SUC_lexicon_tags_2 },
    { 0xd7c9c7cf, SUC_lexicon_tags_1 },
    { 0x12a9c7d0, SUC_lexicon_tags_2 },
    { 0xa189c7d3, SUC_lexicon_tags_17 },
    { 0xdd69c7d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9649c7d7, SUC_lexicon_tags_207 },
    { 0x2169c7d8, SUC_lexicon_tags_3 },
    { 0xe229c7d9, SUC_lexicon_tags_19 },
    { 0x1429c7da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2249c7dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e69c7e0, SUC_lexicon_tags_6 },
    { 0x9de9c7e1, SUC_lexicon_tags_15 },
    { 0x6109c7e2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1fc9c7e4, SUC_lexicon_tags_8 },
    { 0xcec9c7e5, SUC_lexicon_tags_109 },
    { 0x37a9c7e4, SUC_lexicon_tags_23 },
    { 0x1769c7e7, SUC_lexicon_tags_17 },
    { 0x3549c7e8, SUC_lexicon_tags_25 },
    { 0x2629c7e8, SUC_lexicon_tags_6 },
    { 0xeaa9c7e8, SUC_lexicon_tags_11 },
    { 0x1be9c7eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6c9c7ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be9c7f3, SUC_lexicon_tags_2 },
    { 0xaf49c7f4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7fc9c7f6, SUC_lexicon_tags_38 },
    { 0xc389c7f6, SUC_lexicon_tags_15 },
    { 0x2849c7f7, SUC_lexicon_tags_2 },
    { 0xf009c7f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a9c7fc, SUC_lexicon_tags_47 },
    { 0x9369c7fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1fc9c7ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ac9c801, SUC_lexicon_tags_28 },
    { 0xec89c801, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa969c809, SUC_lexicon_tags_14 },
    { 0xa3c9c809, SUC_lexicon_tags_38 },
    { 0x4e49c80b, SUC_lexicon_tags_47 },
    { 0x5449c80c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9809c80e, SUC_lexicon_tags_4 },
    { 0x7e69c80f, SUC_lexicon_tags_47 },
    { 0xd309c80e, SUC_lexicon_tags_8 },
    { 0xaee9c811, SUC_lexicon_tags_37 },
    { 0x8689c812, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9729c81b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c9c81e, SUC_lexicon_tags_32 },
    { 0xb809c81f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a9c828, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe69c82d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a09c82f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29c832, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x6769c834, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe9c838, SUC_lexicon_tags_1 },
    { 0x8fa9c838, SUC_lexicon_tags_11 },
    { 0x1489c839, SUC_lexicon_tags_2 },
    { 0xab09c83a, SUC_lexicon_tags_37 },
    { 0x8969c83c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7789c83e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9c841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf369c847, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb089c84c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf249c854, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9c857, SUC_lexicon_tags_17 },
    { 0x3c29c857, SUC_lexicon_tags_38 },
    { 0x3b09c859, SUC_lexicon_tags_15 },
    { 0xb589c85a, SUC_lexicon_tags_27 },
    { 0xa1e9c85b, SUC_lexicon_tags_11 },
    { 0x86c9c85a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbd69c85e, SUC_lexicon_tags_17 },
    { 0x7129c85f, SUC_lexicon_tags_66 },
    { 0x7dc9c85f, SUC_lexicon_tags_15 },
    { 0xb369c861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c9c868, SUC_lexicon_tags_27 },
    { 0xe89c868, SUC_lexicon_tags_19 },
    { 0x76c9c868, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee9c871, SUC_lexicon_tags_34 },
    { 0x2589c872, SUC_lexicon_tags_11 },
    { 0x7d89c873, SUC_lexicon_tags_25 },
    { 0x989c874, SUC_lexicon_tags_0 },
    { 0x9ae9c873, SUC_lexicon_tags_69 },
    { 0x2329c874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda49c87a, SUC_lexicon_tags_25 },
    { 0xa949c87a, SUC_lexicon_tags_49 },
    { 0x59e9c87b, SUC_lexicon_tags_11 },
    { 0xba09c87c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3529c890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8809c897, SUC_lexicon_tags_77 },
    { 0xac9c898, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e49c8a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e9c8ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de9c8ae, SUC_lexicon_tags_27 },
    { 0x1969c8ae, SUC_lexicon_tags_63 },
    { 0xa7a9c8b0, SUC_lexicon_tags_29 },
    { 0x1c9c8b1, SUC_lexicon_tags_70 },
    { 0x7c09c8b1, SUC_lexicon_tags_2 },
    { 0xd4a9c8b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d69c8b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e89c8b8, SUC_lexicon_tags_37 },
    { 0xed69c8b9, SUC_lexicon_tags_184 },
    { 0x1a89c8ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f89c8bd, SUC_lexicon_tags_2 },
    { 0x2229c8bd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4ce9c8c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a9c8c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5e9c8c9, SUC_lexicon_tags_2 },
    { 0x5b49c8ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e89c8cd, SUC_lexicon_tags_60 },
    { 0xfbe9c8ce, SUC_lexicon_tags_9 },
    { 0x44a9c8cd, SUC_lexicon_tags_10 },
    { 0x1cc9c8d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3469c8d2, SUC_lexicon_tags_27 },
    { 0x5a69c8d2, SUC_lexicon_tags_7 },
    { 0x9289c8d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa849c8d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969c8da, SUC_lexicon_tags_25 },
    { 0x1d09c8db, SUC_lexicon_tags_80 },
    { 0x4d09c8db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1629c8df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce49c8e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec9c8e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7e49c8e8, SUC_lexicon_tags_45 },
    { 0x4cc9c8e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb889c8f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3c9c8f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf69c8f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x23a9c8f9, SUC_lexicon_tags_68 },
    { 0x2189c8fa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef29c8fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc009c904, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda89c907, SUC_lexicon_tags_2 },
    { 0xf469c907, SUC_lexicon_tags_47 },
    { 0x36a9c907, SUC_lexicon_tags_33 },
    { 0xc0a9c90a, SUC_lexicon_tags_1 },
    { 0xb049c90a, SUC_lexicon_tags_32 },
    { 0x71a9c90b, SUC_lexicon_tags_49 },
    { 0xda29c90d, SUC_lexicon_tags_146 },
    { 0xc5c9c90d, SUC_lexicon_tags_33 },
    { 0xd6e9c90f, SUC_lexicon_tags_25 },
    { 0x9ee9c90c, SUC_lexicon_tags_11 },
    { 0x55e9c911, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27e9c913, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd09c916, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa69c918, SUC_lexicon_tags_37 },
    { 0xddc9c918, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c9c91c, SUC_lexicon_tags_25 },
    { 0xdca9c91c, SUC_lexicon_tags_11 },
    { 0x1509c91e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369c928, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ca9c92a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ec9c92c, SUC_lexicon_tags_17 },
    { 0x3ba9c92d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2049c92f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6709c934, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xebe9c936, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c9c93f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b09c942, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5769c944, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce9c948, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e9c94b, SUC_lexicon_tags_47 },
    { 0x8b09c94c, SUC_lexicon_tags_38 },
    { 0x1f29c94d, SUC_lexicon_tags_1 },
    { 0x4849c94d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3869c952, SUC_lexicon_tags_85 },
    { 0xf169c953, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe769c95b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3049c95e, SUC_lexicon_tags_37 },
    { 0x2b89c95f, SUC_lexicon_tags_17 },
    { 0xdfc9c95f, SUC_lexicon_tags_7 },
    { 0x56c9c95e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8489c966, SUC_lexicon_tags_16 },
    { 0x9ee9c967, SUC_lexicon_tags_29 },
    { 0xc6c9c967, SUC_lexicon_tags_14 },
    { 0x3809c966, SUC_lexicon_tags_27 },
    { 0x82a9c96a, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x5729c96d, SUC_lexicon_tags_15 },
    { 0x31e9c96e, SUC_lexicon_tags_29 },
    { 0x7b09c96f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda29c971, SUC_lexicon_tags_60 },
    { 0x9529c972, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e9c977, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb1c9c979, SUC_lexicon_tags_11 },
    { 0xbb69c979, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa9c97e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca9c983, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c987, SUC_lexicon_tags_57 },
    { 0xe4a9c988, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c9c994, SUC_lexicon_tags_30 },
    { 0x7969c995, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8249c997, SUC_lexicon_tags_37 },
    { 0xb129c997, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c9c99e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c09c9a2, SUC_lexicon_tags_17 },
    { 0x609c9a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d69c9a9, SUC_lexicon_tags_29 },
    { 0xed09c9a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e49c9ae, SUC_lexicon_tags_11 },
    { 0x7ee9c9af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a29c9b3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f09c9b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5169c9bb, SUC_lexicon_tags_11 },
    { 0x9fc9c9bc, SUC_lexicon_tags_17 },
    { 0x5829c9bd, SUC_lexicon_tags_9 },
    { 0x93c9c9bb, SUC_lexicon_tags_17 },
    { 0x2f89c9bf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a9c9c6, SUC_lexicon_tags_55 },
    { 0x6229c9c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e9c9d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b89c9db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfca9c9dd, SUC_lexicon_tags_1383 },
    { 0xda49c9dd, SUC_lexicon_tags_40 },
    { 0xa0c9c9de, SUC_lexicon_tags_0 },
    { 0x5969c9de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc129c9e4, SUC_lexicon_tags_37 },
    { 0xed49c9e4, SUC_lexicon_tags_14 },
    { 0x90e9c9e6, SUC_lexicon_tags_63 },
    { 0xa2a9c9e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4589c9ea, SUC_lexicon_tags_39 },
    { 0x989c9eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcfe9c9ed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda49c9f1, SUC_lexicon_tags_37 },
    { 0xd409c9f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7269c9f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2089c9f8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb69c9fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a9ca04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3269ca07, SUC_lexicon_tags_7 },
    { 0xe809ca07, SUC_lexicon_tags_64 },
    { 0x6009ca09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc389ca0b, SUC_lexicon_tags_10 },
    { 0xb549ca0c, SUC_lexicon_tags_37 },
    { 0x7889ca0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c9ca10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc749ca13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be9ca17, SUC_lexicon_tags_61 },
    { 0xc409ca18, SUC_lexicon_tags_84 },
    { 0xdb09ca17, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2709ca1e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1669ca21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bc9ca23, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6c49ca25, SUC_lexicon_tags_17 },
    { 0xdd29ca26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd09ca28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e9ca2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9649ca31, SUC_lexicon_tags_7 },
    { 0xc4a9ca31, SUC_lexicon_tags_25 },
    { 0x67a9ca33, SUC_lexicon_tags_38 },
    { 0xd549ca33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9aa9ca36, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2969ca38, SUC_lexicon_tags_11 },
    { 0xaa09ca39, SUC_lexicon_tags_17 },
    { 0xbd29ca39, SUC_lexicon_tags_15 },
    { 0xb2a9ca3b, SUC_lexicon_tags_0 },
    { 0x9129ca3b, SUC_lexicon_tags_17 },
    { 0x7029ca3d, SUC_lexicon_tags_37 },
    { 0x5069ca3e, SUC_lexicon_tags_2 },
    { 0x2ea9ca3f, SUC_lexicon_tags_0 },
    { 0x3b09ca40, SUC_lexicon_tags_13 },
    { 0x6909ca40, SUC_lexicon_tags_11 },
    { 0x8029ca40, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3049ca44, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d89ca47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac9ca4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc009ca55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe589ca57, SUC_lexicon_tags_27 },
    { 0xa8a9ca58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa709ca5e, SUC_lexicon_tags_15 },
    { 0xec09ca5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1e9ca61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5869ca65, SUC_lexicon_tags_2 },
    { 0x8449ca65, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xa889ca68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f89ca6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcaa9ca6c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd289ca6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda9ca75, SUC_lexicon_tags_25 },
    { 0xa269ca76, SUC_lexicon_tags_15 },
    { 0xcaa9ca77, SUC_lexicon_tags_2 },
    { 0x9089ca77, SUC_lexicon_tags_2 },
    { 0x46e9ca79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b09ca7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f29ca7d, SUC_lexicon_tags_17 },
    { 0xc29ca7e, SUC_lexicon_tags_25 },
    { 0xe8c9ca7f, SUC_lexicon_tags_15 },
    { 0xdde9ca80, SUC_lexicon_tags_2 },
    { 0x729ca7f, SUC_lexicon_tags_37 },
    { 0xd5e9ca82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c9ca85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfe9ca87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf29ca8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf369ca8e, SUC_lexicon_tags_29 },
    { 0x43a9ca8e, SUC_lexicon_tags_37 },
    { 0xcbc9ca8f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2729ca92, SUC_lexicon_tags_15 },
    { 0xf249ca92, SUC_lexicon_tags_19 },
    { 0xaee9ca93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6329ca98, SUC_lexicon_tags_29 },
    { 0x57c9ca99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8589ca9f, SUC_lexicon_tags_12 },
    { 0x9ec9ca9f, SUC_lexicon_tags_0 },
    { 0x4b89caa1, SUC_lexicon_tags_1 },
    { 0x729caa2, SUC_lexicon_tags_37 },
    { 0x8ba9caa3, SUC_lexicon_tags_33 },
    { 0x51e9caa1, SUC_lexicon_tags_8 },
    { 0xae09caa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9429caa7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c09caae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb569cab1, SUC_lexicon_tags_37 },
    { 0x63a9cab1, SUC_lexicon_tags_12 },
    { 0xa1a9cab2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a9cab8, SUC_lexicon_tags_2 },
    { 0x1109cab8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd29cabc, SUC_lexicon_tags_49 },
    { 0xe709cabc, SUC_lexicon_tags_12 },
    { 0xf169cabe, SUC_lexicon_tags_12 },
    { 0x3389cabf, SUC_lexicon_tags_12 },
    { 0xa429cabc, SUC_lexicon_tags_63 },
    { 0x4f29cac1, SUC_lexicon_tags_17 },
    { 0x2f69cabe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce9cac6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a9cad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfde9cad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b49cad8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa49cadb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3249cade, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7069cae1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a9cae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4409cae7, SUC_lexicon_tags_17 },
    { 0xe5a9cae8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a9caeb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3449caee, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xcca9caf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e9caf3, SUC_lexicon_tags_14 },
    { 0xd869caf3, SUC_lexicon_tags_57 },
    { 0xcc69caf5, SUC_lexicon_tags_2 },
    { 0x4f09caf3, SUC_lexicon_tags_63 },
    { 0x7329caf7, SUC_lexicon_tags_1 },
    { 0xb8c9caf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc9cafb, SUC_lexicon_tags_43 },
    { 0x1be9cafc, SUC_lexicon_tags_8 },
    { 0x109cafb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd689cb01, SUC_lexicon_tags_63 },
    { 0xb1c9cb02, SUC_lexicon_tags_26 },
    { 0x2029cb03, SUC_lexicon_tags_64 },
    { 0x7849cb04, SUC_lexicon_tags_11 },
    { 0x7029cb05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d69cb0b, SUC_lexicon_tags_11 },
    { 0x62e9cb0c, SUC_lexicon_tags_38 },
    { 0x25e9cb0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf089cb10, SUC_lexicon_tags_55 },
    { 0x92e9cb11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5689cb16, SUC_lexicon_tags_10 },
    { 0x9289cb17, SUC_lexicon_tags_38 },
    { 0x4129cb18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc129cb1e, SUC_lexicon_tags_2 },
    { 0x4129cb1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f09cb21, SUC_lexicon_tags_25 },
    { 0xe9c9cb21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1149cb26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a49cb2a, SUC_lexicon_tags_11 },
    { 0xbca9cb2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd849cb30, SUC_lexicon_tags_17 },
    { 0x9169cb31, SUC_lexicon_tags_18 },
    { 0xce69cb30, SUC_lexicon_tags_12 },
    { 0x3c29cb32, SUC_lexicon_tags_11 },
    { 0x5609cb34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f89cb36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa989cb38, SUC_lexicon_tags_0 },
    { 0x33a9cb39, SUC_lexicon_tags_7 },
    { 0x45a9cb38, SUC_lexicon_tags_5 },
    { 0x29c9cb3b, SUC_lexicon_tags_28 },
    { 0xc969cb3c, SUC_lexicon_tags_87 },
    { 0xd6c9cb39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e9cb43, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeec9cb45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf89cb48, SUC_lexicon_tags_37 },
    { 0x2a69cb48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4009cb50, SUC_lexicon_tags_5 },
    { 0x81c9cb50, SUC_lexicon_tags_63 },
    { 0x4989cb52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a9cb56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9629cb59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8829cb5b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9249cb5d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c9cb64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e9cb6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ae9cb6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99c9cb71, SUC_lexicon_tags_27 },
    { 0x4529cb72, SUC_lexicon_tags_2 },
    { 0xb3c9cb72, SUC_lexicon_tags_37 },
    { 0x8f29cb73, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe9cb82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c9cb86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe9cb8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xff89cb8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a9cb91, SUC_lexicon_tags_23 },
    { 0x5509cb92, SUC_lexicon_tags_60 },
    { 0xb989cb93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8089cb96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf989cb9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e9cb9f, SUC_lexicon_tags_5 },
    { 0x1869cba0, SUC_lexicon_tags_11 },
    { 0x5409cba1, SUC_lexicon_tags_43 },
    { 0x2fc9cba1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de9cba6, SUC_lexicon_tags_37 },
    { 0x8089cba6, SUC_lexicon_tags_15 },
    { 0xf549cba8, SUC_lexicon_tags_19 },
    { 0xe9a9cba9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9849cbab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6689cbad, SUC_lexicon_tags_27 },
    { 0xfba9cbae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4049cbb2, SUC_lexicon_tags_63 },
    { 0xb529cbb3, SUC_lexicon_tags_15 },
    { 0xf3c9cbb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c9cbb9, SUC_lexicon_tags_47 },
    { 0x5189cbba, SUC_lexicon_tags_2 },
    { 0x4209cbba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2e9cbbd, SUC_lexicon_tags_21 },
    { 0x5529cbbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9309cbc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489cbc6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9cbcd, SUC_lexicon_tags_49 },
    { 0x2b89cbce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d49cbd1, SUC_lexicon_tags_3 },
    { 0x9109cbd2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1969cbdb, SUC_lexicon_tags_26 },
    { 0x4ca9cbdc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9929cbde, SUC_lexicon_tags_11 },
    { 0x9c89cbdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4e9cbe1, SUC_lexicon_tags_11 },
    { 0x5bc9cbe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d89cbe4, SUC_lexicon_tags_11 },
    { 0xb3e9cbe4, SUC_lexicon_tags_27 },
    { 0x2769cbe6, SUC_lexicon_tags_38 },
    { 0x3a09cbe6, SUC_lexicon_tags_27 },
    { 0xe7a9cbe8, SUC_lexicon_tags_77 },
    { 0xf749cbe9, SUC_lexicon_tags_7 },
    { 0x9649cbea, SUC_lexicon_tags_27 },
    { 0x5589cbeb, SUC_lexicon_tags_2 },
    { 0x8989cbe9, SUC_lexicon_tags_1 },
    { 0x4289cbe9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x58c9cbef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa889cbf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7269cbf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e9cbfd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4929cc03, SUC_lexicon_tags_26 },
    { 0x5fc9cc04, SUC_lexicon_tags_8 },
    { 0x6769cc05, SUC_lexicon_tags_60 },
    { 0xf69cc06, SUC_lexicon_tags_37 },
    { 0x5c09cc05, SUC_lexicon_tags_9 },
    { 0xa29cc08, SUC_lexicon_tags_23 },
    { 0xf169cc09, SUC_lexicon_tags_12 },
    { 0x42a9cc09, SUC_lexicon_tags_25 },
    { 0x5be9cc0b, SUC_lexicon_tags_25 },
    { 0x5549cc0b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8889cc10, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d89cc13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf129cc16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a49cc18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23e9cc22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b29cc27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b49cc2a, SUC_lexicon_tags_47 },
    { 0x2249cc2b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x33a9cc2d, SUC_lexicon_tags_2 },
    { 0xdc89cc2e, SUC_lexicon_tags_5 },
    { 0x4029cc2d, SUC_lexicon_tags_7 },
    { 0x4229cc30, SUC_lexicon_tags_2 },
    { 0x9849cc2e, SUC_lexicon_tags_112 },
    { 0x7d89cc32, SUC_lexicon_tags_14 },
    { 0x5729cc33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e9cc37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f09cc39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1489cc3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589cc40, SUC_lexicon_tags_1 },
    { 0x9c9cc40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be9cc44, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x3aa9cc46, SUC_lexicon_tags_17 },
    { 0x6109cc47, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa9cc4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22c9cc4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe1c9cc50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa89cc53, SUC_lexicon_tags_49 },
    { 0x49e9cc54, SUC_lexicon_tags_27 },
    { 0x2f29cc55, SUC_lexicon_tags_11 },
    { 0xee69cc54, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8809cc58, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c9cc5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa109cc64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3929cc67, SUC_lexicon_tags_15 },
    { 0x1fc9cc68, SUC_lexicon_tags_11 },
    { 0xae29cc69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b69cc6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7409cc73, SUC_lexicon_tags_17 },
    { 0xb849cc73, SUC_lexicon_tags_11 },
    { 0x6549cc75, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea9cc79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7869cc82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5509cc84, SUC_lexicon_tags_87 },
    { 0xd709cc85, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cc88, SUC_lexicon_tags_64 },
    { 0x6dc9cc89, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x2549cc8b, SUC_lexicon_tags_34 },
    { 0xd489cc8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa189cc8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb29cc92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad29cc94, SUC_lexicon_tags_28 },
    { 0x68c9cc95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfda9cc97, SUC_lexicon_tags_11 },
    { 0x6809cc98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe369cca0, SUC_lexicon_tags_66 },
    { 0x1769cca1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac29cca3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae9cca6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1809cca8, SUC_lexicon_tags_17 },
    { 0xab49cca8, SUC_lexicon_tags_37 },
    { 0x6689ccaa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e9ccad, SUC_lexicon_tags_17 },
    { 0x9ea9ccad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb369ccb3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc009ccb8, SUC_lexicon_tags_47 },
    { 0x7869ccb9, SUC_lexicon_tags_2 },
    { 0x64c9ccba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1609ccbc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78c9ccc0, SUC_lexicon_tags_44 },
    { 0xc2a9ccc1, SUC_lexicon_tags_1 },
    { 0xf709ccc0, SUC_lexicon_tags_2 },
    { 0x5e89ccc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe9cccd, SUC_lexicon_tags_17 },
    { 0x59a9ccce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfa9ccd0, SUC_lexicon_tags_25 },
    { 0xf449ccd1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa9e9ccd3, SUC_lexicon_tags_23 },
    { 0x7c9ccd4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc809ccda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce9cce0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909cce7, SUC_lexicon_tags_1 },
    { 0x7c69cce8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5a49ccea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da9ccef, SUC_lexicon_tags_23 },
    { 0xeee9ccf0, SUC_lexicon_tags_30 },
    { 0xba29ccf1, SUC_lexicon_tags_21 },
    { 0x54a9ccf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5409ccf4, SUC_lexicon_tags_8 },
    { 0x30c9ccf5, SUC_lexicon_tags_15 },
    { 0xed49ccf5, SUC_lexicon_tags_15 },
    { 0xf049ccf7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79c9ccf9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2149ccfc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xba9ccfe, SUC_lexicon_tags_1 },
    { 0x88a9ccfe, SUC_lexicon_tags_15 },
    { 0xc6e9cd00, SUC_lexicon_tags_11 },
    { 0xe2c9cd01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a9cd09, SUC_lexicon_tags_7 },
    { 0x9c69cd09, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8309cd13, SUC_lexicon_tags_9 },
    { 0xfb09cd14, SUC_lexicon_tags_6 },
    { 0xf3c9cd15, SUC_lexicon_tags_14 },
    { 0x3909cd13, SUC_lexicon_tags_55 },
    { 0xcce9cd17, SUC_lexicon_tags_2 },
    { 0x2469cd17, SUC_lexicon_tags_25 },
    { 0x54a9cd19, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e09cd1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3329cd27, SUC_lexicon_tags_17 },
    { 0x8c49cd27, SUC_lexicon_tags_37 },
    { 0x9749cd28, SUC_lexicon_tags_14 },
    { 0x9309cd2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb709cd32, SUC_lexicon_tags_7 },
    { 0x61e9cd33, SUC_lexicon_tags_3 },
    { 0x3809cd34, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c29cd3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b09cd42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4769cd44, SUC_lexicon_tags_7 },
    { 0xe5a9cd45, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd989cd48, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc529cd52, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf489cd5b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c29cd5e, SUC_lexicon_tags_25 },
    { 0x8a89cd5f, SUC_lexicon_tags_27 },
    { 0xb3c9cd60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb89cd6c, SUC_lexicon_tags_47 },
    { 0x2c9cd6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e69cd6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4aa9cd71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3389cd75, SUC_lexicon_tags_29 },
    { 0x76c9cd76, SUC_lexicon_tags_14 },
    { 0x3109cd77, SUC_lexicon_tags_27 },
    { 0x7a29cd77, SUC_lexicon_tags_14 },
    { 0x7329cd76, SUC_lexicon_tags_6 },
    { 0x2c89cd7a, SUC_lexicon_tags_10 },
    { 0xd889cd77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1729cd82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c49cd87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe49cd8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e9cd94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b89cd9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc89cd9d, SUC_lexicon_tags_17 },
    { 0x5a9cd9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81a9cda0, SUC_lexicon_tags_61 },
    { 0x9969cda1, SUC_lexicon_tags_0 },
    { 0x2929cda2, SUC_lexicon_tags_11 },
    { 0xc3c9cda0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa69cda8, SUC_lexicon_tags_11 },
    { 0xcaa9cda9, SUC_lexicon_tags_37 },
    { 0xce9cdaa, SUC_lexicon_tags_56 },
    { 0x5989cdab, SUC_lexicon_tags_12 },
    { 0x9da9cdaa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x569cdae, SUC_lexicon_tags_11 },
    { 0xad49cdaf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce09cdb3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb8e9cdb5, SUC_lexicon_tags_37 },
    { 0x7769cdb5, SUC_lexicon_tags_41 },
    { 0x2a9cdb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb809cdba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe629cdc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b89cdc2, SUC_lexicon_tags_19 },
    { 0x8a09cdc3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6389cdc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c9cdc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e49cdcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e9cdd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf729cddb, SUC_lexicon_tags_63 },
    { 0x5509cddb, SUC_lexicon_tags_97 },
    { 0x1689cddb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc8e9cddf, SUC_lexicon_tags_0 },
    { 0x1789cde0, SUC_lexicon_tags_15 },
    { 0x8be9cde1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe649cde4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1729cde9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb649cdeb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb09cded, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa609cdf4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f29cdf8, SUC_lexicon_tags_30 },
    { 0x2f89cdf9, SUC_lexicon_tags_27 },
    { 0xbd69cdf9, SUC_lexicon_tags_17 },
    { 0xd1a9cdfa, SUC_lexicon_tags_63 },
    { 0x4509cdfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8e9cdfe, SUC_lexicon_tags_27 },
    { 0xc09cdff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f09ce02, SUC_lexicon_tags_7 },
    { 0xc089ce02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd709ce07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa089ce0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a69ce0e, SUC_lexicon_tags_38 },
    { 0x94e9ce0f, SUC_lexicon_tags_37 },
    { 0xcc89ce0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8729ce13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf69ce15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xad69ce18, SUC_lexicon_tags_7 },
    { 0x4269ce19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa589ce1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4aa9ce20, SUC_lexicon_tags_11 },
    { 0xed89ce21, SUC_lexicon_tags_2 },
    { 0xe3a9ce22, SUC_lexicon_tags_45 },
    { 0xe349ce20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1969ce2a, SUC_lexicon_tags_2 },
    { 0x53a9ce2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7009ce2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb49ce33, SUC_lexicon_tags_43 },
    { 0x7de9ce34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c9ce3d, SUC_lexicon_tags_49 },
    { 0xf6c9ce3e, SUC_lexicon_tags_15 },
    { 0x83c9ce3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709ce45, SUC_lexicon_tags_2 },
    { 0xa09ce46, SUC_lexicon_tags_15 },
    { 0xd329ce46, SUC_lexicon_tags_12 },
    { 0xf409ce48, SUC_lexicon_tags_45 },
    { 0x4ae9ce46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1449ce51, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4609ce55, SUC_lexicon_tags_49 },
    { 0xd389ce56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77c9ce58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd329ce63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f09ce6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4089ce6e, SUC_lexicon_tags_60 },
    { 0xd049ce6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ce72, SUC_lexicon_tags_63 },
    { 0x2ba9ce73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd889ce76, SUC_lexicon_tags_17 },
    { 0x649ce77, SUC_lexicon_tags_15 },
    { 0x7fc9ce77, SUC_lexicon_tags_37 },
    { 0xf2e9ce79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb09ce7c, SUC_lexicon_tags_30 },
    { 0xee69ce7d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc169ce7f, SUC_lexicon_tags_63 },
    { 0x80c9ce80, SUC_lexicon_tags_11 },
    { 0xd2c9ce81, SUC_lexicon_tags_28 },
    { 0xa3e9ce82, SUC_lexicon_tags_17 },
    { 0xa3e9ce80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5a9ce85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d09ce88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a69ce8a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba9ce8e, SUC_lexicon_tags_7 },
    { 0xb789ce8f, SUC_lexicon_tags_17 },
    { 0x1d69ce90, SUC_lexicon_tags_1 },
    { 0x9569ce8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389ce97, SUC_lexicon_tags_13 },
    { 0x5129ce97, SUC_lexicon_tags_26 },
    { 0x6a89ce99, SUC_lexicon_tags_1 },
    { 0x95c9ce9a, SUC_lexicon_tags_25 },
    { 0x4949ce9a, SUC_lexicon_tags_29 },
    { 0x8c89ce98, SUC_lexicon_tags_4 },
    { 0x39c9ce9d, SUC_lexicon_tags_40 },
    { 0x4749ce9e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xb329cea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e9ceaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a9ceb7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a9cebd, SUC_lexicon_tags_23 },
    { 0xbfe9cebe, SUC_lexicon_tags_65 },
    { 0x96c9cebf, SUC_lexicon_tags_37 },
    { 0x22a9cebd, SUC_lexicon_tags_11 },
    { 0x37c9cec1, SUC_lexicon_tags_32 },
    { 0x10c9cec1, SUC_lexicon_tags_45 },
    { 0x6c49cec2, SUC_lexicon_tags_14 },
    { 0xeec9cec3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbae9cec6, SUC_lexicon_tags_12 },
    { 0x7829cec7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc9cecd, SUC_lexicon_tags_25 },
    { 0x5569cece, SUC_lexicon_tags_55 },
    { 0xdb49cece, SUC_lexicon_tags_56 },
    { 0xa149cecf, SUC_lexicon_tags_55 },
    { 0xfe49ced1, SUC_lexicon_tags_11 },
    { 0x21a9ced1, SUC_lexicon_tags_23 },
    { 0x3ca9ced2, SUC_lexicon_tags_26 },
    { 0xb1a9cecf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40a9ced6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde09ceda, SUC_lexicon_tags_40 },
    { 0x27c9cedb, SUC_lexicon_tags_9 },
    { 0x2589cedc, SUC_lexicon_tags_17 },
    { 0x11c9cedc, SUC_lexicon_tags_15 },
    { 0x8ca9cedd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cc9cee0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e9cee3, SUC_lexicon_tags_2 },
    { 0xc269cee4, SUC_lexicon_tags_33 },
    { 0xba49cee4, SUC_lexicon_tags_29 },
    { 0x28c9cee4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x72a9cee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbc9ceeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7469cef5, SUC_lexicon_tags_43 },
    { 0xc5a9cef6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e9cef8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x589cefa, SUC_lexicon_tags_17 },
    { 0x9c9cefb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c9cefe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9e29cf00, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbde9cf02, SUC_lexicon_tags_61 },
    { 0x3469cf03, SUC_lexicon_tags_11 },
    { 0xdcc9cf04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10c9cf06, SUC_lexicon_tags_65 },
    { 0x7be9cf07, SUC_lexicon_tags_49 },
    { 0x5f89cf08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca9cf10, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe549cf19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c9cf1c, SUC_lexicon_tags_17 },
    { 0xd09cf1d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa189cf1f, SUC_lexicon_tags_79 },
    { 0xf929cf1f, SUC_lexicon_tags_33 },
    { 0x2d09cf21, SUC_lexicon_tags_12 },
    { 0xf8a9cf22, SUC_lexicon_tags_117 },
    { 0xafc9cf21, SUC_lexicon_tags_37 },
    { 0x8829cf24, SUC_lexicon_tags_2 },
    { 0xa3a9cf24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7349cf28, SUC_lexicon_tags_32 },
    { 0x46a9cf29, SUC_lexicon_tags_33 },
    { 0x4d49cf2a, SUC_lexicon_tags_27 },
    { 0x4309cf29, SUC_lexicon_tags_7 },
    { 0xffe9cf2c, SUC_lexicon_tags_17 },
    { 0x1ae9cf2d, SUC_lexicon_tags_38 },
    { 0xb409cf2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6669cf32, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd589cf38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdf49cf3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf589cf3d, SUC_lexicon_tags_11 },
    { 0xf89cf3e, SUC_lexicon_tags_15 },
    { 0xa769cf3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd409cf45, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xda49cf47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec9cf4b, SUC_lexicon_tags_19 },
    { 0x89a9cf4b, SUC_lexicon_tags_13 },
    { 0x3249cf4d, SUC_lexicon_tags_37 },
    { 0x69cf4c, SUC_lexicon_tags_48 },
    { 0x3189cf4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce9cf54, SUC_lexicon_tags_17 },
    { 0x2789cf54, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x9fa9cf57, SUC_lexicon_tags_49 },
    { 0x61a9cf58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd1c9cf5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a9cf5d, SUC_lexicon_tags_11 },
    { 0xd529cf5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8569cf64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a29cf67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38a9cf69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3049cf6f, SUC_lexicon_tags_37 },
    { 0xfb89cf70, SUC_lexicon_tags_15 },
    { 0x1289cf71, SUC_lexicon_tags_25 },
    { 0xd1e9cf72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a9cf7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3589cf80, SUC_lexicon_tags_77 },
    { 0xeaa9cf80, SUC_lexicon_tags_2 },
    { 0x58c9cf82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a29cf85, SUC_lexicon_tags_4 },
    { 0xa529cf86, SUC_lexicon_tags_12 },
    { 0x9f29cf86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb29cf8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae9cf90, SUC_lexicon_tags_87 },
    { 0x5e09cf90, SUC_lexicon_tags_17 },
    { 0xa369cf90, SUC_lexicon_tags_1 },
    { 0xe3c9cf93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8629cf95, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe49cf9b, SUC_lexicon_tags_62 },
    { 0xf689cf9c, SUC_lexicon_tags_49 },
    { 0xc2c9cf9d, SUC_lexicon_tags_27 },
    { 0x5f89cf9e, SUC_lexicon_tags_37 },
    { 0xc4e9cf9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d69cfa2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac9cfa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a29cfa8, SUC_lexicon_tags_9 },
    { 0x2629cfa9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x849cfab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1cc9cfad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d89cfb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c29cfb8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa929cfbb, SUC_lexicon_tags_37 },
    { 0x3849cfbc, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e9cfc0, SUC_lexicon_tags_43 },
    { 0x4da9cfc1, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x5769cfc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2249cfc5, SUC_lexicon_tags_12 },
    { 0x38c9cfc5, SUC_lexicon_tags_66 },
    { 0xc729cfc7, SUC_lexicon_tags_26 },
    { 0x6949cfc8, SUC_lexicon_tags_2 },
    { 0x73a9cfc9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c9cfd4, SUC_lexicon_tags_48 },
    { 0xf5c9cfd5, SUC_lexicon_tags_14 },
    { 0x7349cfd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2be9cfd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9cfdd, SUC_lexicon_tags_15 },
    { 0xca09cfdd, SUC_lexicon_tags_2 },
    { 0xe5e9cfdf, SUC_lexicon_tags_12 },
    { 0x2509cfdf, SUC_lexicon_tags_12 },
    { 0xfec9cfde, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb389cfe5, SUC_lexicon_tags_21 },
    { 0xda49cfe6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd1a9cfe8, SUC_lexicon_tags_39 },
    { 0xed49cfe8, SUC_lexicon_tags_17 },
    { 0x9cfe9, SUC_lexicon_tags_11 },
    { 0x9469cfeb, SUC_lexicon_tags_43 },
    { 0x14e9cfe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5329cfef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd149cff3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf409cff6, SUC_lexicon_tags_11 },
    { 0x35a9cff7, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x4d69cff9, SUC_lexicon_tags_7 },
    { 0x13e9cffa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1489cffc, SUC_lexicon_tags_37 },
    { 0x7bc9cffd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeae9d000, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c69d004, SUC_lexicon_tags_14 },
    { 0xc8a9d005, SUC_lexicon_tags_514 },
    { 0x7fe9d006, SUC_lexicon_tags_37 },
    { 0xcc9d007, SUC_lexicon_tags_11 },
    { 0xc169d008, SUC_lexicon_tags_92 },
    { 0x61a9d009, SUC_lexicon_tags_45 },
    { 0x7ec9d008, SUC_lexicon_tags_37 },
    { 0x3209d005, SUC_lexicon_tags_4 },
    { 0x3d49d00c, SUC_lexicon_tags_63 },
    { 0x9149d008, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a9d010, SUC_lexicon_tags_15 },
    { 0xb309d011, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8869d015, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb069d020, SUC_lexicon_tags_31 },
    { 0x19e9d021, SUC_lexicon_tags_12 },
    { 0xe049d022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929d029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a89d030, SUC_lexicon_tags_46 },
    { 0x71a9d031, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa389d033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15c9d035, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb89d037, SUC_lexicon_tags_25 },
    { 0x54a9d038, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc09d03c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469d041, SUC_lexicon_tags_14 },
    { 0xc989d041, SUC_lexicon_tags_11 },
    { 0xdfa9d043, SUC_lexicon_tags_43 },
    { 0x6e9d044, SUC_lexicon_tags_35 },
    { 0xb909d041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce9d048, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7229d04e, SUC_lexicon_tags_553 },
    { 0x6769d04e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe569d052, SUC_lexicon_tags_11 },
    { 0xce29d053, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7849d059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf469d05f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x99e9d061, SUC_lexicon_tags_1 },
    { 0xb69d061, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba89d067, SUC_lexicon_tags_0 },
    { 0x21c9d068, SUC_lexicon_tags_17 },
    { 0x75a9d068, SUC_lexicon_tags_37 },
    { 0xfa09d06a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e49d075, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c9d079, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf89d07b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6609d07d, SUC_lexicon_tags_45 },
    { 0xfdc9d07d, SUC_lexicon_tags_25 },
    { 0x86a9d07f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8fa9d081, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeb69d083, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdd09d085, SUC_lexicon_tags_17 },
    { 0xb469d086, SUC_lexicon_tags_30 },
    { 0x5b69d087, SUC_lexicon_tags_30 },
    { 0x82c9d086, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09d08b, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1749d08f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e9d094, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09d09d, SUC_lexicon_tags_1 },
    { 0x3649d09d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7549d0a0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa689d0a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e9d0a6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec9d0ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd709d0ae, SUC_lexicon_tags_17 },
    { 0xf469d0af, SUC_lexicon_tags_17 },
    { 0x3089d0b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa369d0b2, SUC_lexicon_tags_15 },
    { 0xe489d0b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57a9d0b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa149d0b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc629d0bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf889d0c3, SUC_lexicon_tags_14 },
    { 0x69a9d0c3, SUC_lexicon_tags_33 },
    { 0x43e9d0c5, SUC_lexicon_tags_68 },
    { 0x1289d0c6, SUC_lexicon_tags_23 },
    { 0xc329d0c7, SUC_lexicon_tags_37 },
    { 0x9749d0c6, SUC_lexicon_tags_23 },
    { 0x2229d0c9, SUC_lexicon_tags_1 },
    { 0xf8c9d0c7, SUC_lexicon_tags_37 },
    { 0x8129d0c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9d0d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a9d0d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeec9d0d9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf649d0e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9069d0e3, SUC_lexicon_tags_15 },
    { 0x7069d0e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4449d0ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d0ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb609d0f0, SUC_lexicon_tags_500 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa589d0f7, SUC_lexicon_tags_17 },
    { 0x6ba9d0f8, SUC_lexicon_tags_71 },
    { 0x2fc9d0f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa9d0fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a9d100, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a9d103, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e89d109, SUC_lexicon_tags_25 },
    { 0x1369d109, SUC_lexicon_tags_37 },
    { 0x4f49d10b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9789d10e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e9d113, SUC_lexicon_tags_11 },
    { 0xf0c9d114, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a09d119, SUC_lexicon_tags_35 },
    { 0xb2e9d119, SUC_lexicon_tags_7 },
    { 0x24e9d119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae9d11e, SUC_lexicon_tags_30 },
    { 0xa149d11f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a9d122, SUC_lexicon_tags_23 },
    { 0x5ec9d123, SUC_lexicon_tags_11 },
    { 0x3c29d124, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f09d129, SUC_lexicon_tags_13 },
    { 0x9da9d12a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a9d130, SUC_lexicon_tags_37 },
    { 0x529d130, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb49d136, SUC_lexicon_tags_11 },
    { 0xb6a9d137, SUC_lexicon_tags_57 },
    { 0xbde9d138, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e9d13f, SUC_lexicon_tags_12 },
    { 0xb09d140, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2229d143, SUC_lexicon_tags_0 },
    { 0xf3e9d144, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc009d14d, SUC_lexicon_tags_17 },
    { 0xda9d14e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ba9d150, SUC_lexicon_tags_29 },
    { 0x2789d150, SUC_lexicon_tags_12 },
    { 0xbb89d152, SUC_lexicon_tags_37 },
    { 0xcca9d153, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8a9d155, SUC_lexicon_tags_79 },
    { 0xcf09d156, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5169d15a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3229d15c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c9d163, SUC_lexicon_tags_1538 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe089d167, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xad89d169, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d89d16c, SUC_lexicon_tags_31 },
    { 0xf309d16d, SUC_lexicon_tags_8 },
    { 0x3649d16d, SUC_lexicon_tags_17 },
    { 0x1bc9d16f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c9d172, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf249d174, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e9d179, SUC_lexicon_tags_6 },
    { 0x8129d179, SUC_lexicon_tags_6 },
    { 0x3869d17b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7349d180, SUC_lexicon_tags_11 },
    { 0x77e9d180, SUC_lexicon_tags_25 },
    { 0x8709d181, SUC_lexicon_tags_30 },
    { 0x40c9d183, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a9d18b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6749d18d, SUC_lexicon_tags_37 },
    { 0xe0e9d18e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82e9d193, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e9d196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4809d19a, SUC_lexicon_tags_30 },
    { 0x22c9d19b, SUC_lexicon_tags_31 },
    { 0x3089d19a, SUC_lexicon_tags_37 },
    { 0x3029d19b, SUC_lexicon_tags_25 },
    { 0x7e69d19d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9d1a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc9d1b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa9d1bd, SUC_lexicon_tags_19 },
    { 0xa749d1bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86e9d1c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde9d1cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf89d1d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c9d1db, SUC_lexicon_tags_37 },
    { 0xaa89d1dc, SUC_lexicon_tags_9 },
    { 0xba09d1dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d89d1df, SUC_lexicon_tags_37 },
    { 0x1ce9d1e0, SUC_lexicon_tags_45 },
    { 0x5089d1e1, SUC_lexicon_tags_27 },
    { 0xb929d1e2, SUC_lexicon_tags_1 },
    { 0x4b69d1e3, SUC_lexicon_tags_27 },
    { 0xd4c9d1e3, SUC_lexicon_tags_17 },
    { 0xefc9d1e5, SUC_lexicon_tags_10 },
    { 0x2509d1e4, SUC_lexicon_tags_31 },
    { 0xb2e9d1e2, SUC_lexicon_tags_87 },
    { 0x2349d1e8, SUC_lexicon_tags_7 },
    { 0x46c9d1e1, SUC_lexicon_tags_46 },
    { 0xe3a9d1ea, SUC_lexicon_tags_2 },
    { 0xd049d1ea, SUC_lexicon_tags_12 },
    { 0x2e69d1eb, SUC_lexicon_tags_17 },
    { 0x8a89d1ed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7549d1ef, SUC_lexicon_tags_33 },
    { 0xbce9d1f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10c9d1f4, SUC_lexicon_tags_26 },
    { 0x87e9d1f5, SUC_lexicon_tags_21 },
    { 0xafe9d1f6, SUC_lexicon_tags_43 },
    { 0x95e9d1f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6429d1fc, SUC_lexicon_tags_7 },
    { 0x1769d1fd, SUC_lexicon_tags_56 },
    { 0x4f69d1fd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1409d201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d69d203, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf809d206, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4169d209, SUC_lexicon_tags_116 },
    { 0x4569d209, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa149d20c, SUC_lexicon_tags_60 },
    { 0x7d49d20c, SUC_lexicon_tags_37 },
    { 0x1e69d20e, SUC_lexicon_tags_11 },
    { 0x30a9d20d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d69d211, SUC_lexicon_tags_47 },
    { 0xcc89d212, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a9d216, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9529d21a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd069d21c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2289d21e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca9d227, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98a9d22c, SUC_lexicon_tags_12 },
    { 0xae09d22c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d235, SUC_lexicon_tags_0 },
    { 0xe749d236, SUC_lexicon_tags_27 },
    { 0xeae9d237, SUC_lexicon_tags_17 },
    { 0xa789d237, SUC_lexicon_tags_15 },
    { 0xdc9d239, SUC_lexicon_tags_4 },
    { 0xd6c9d239, SUC_lexicon_tags_9 },
    { 0x7629d23a, SUC_lexicon_tags_29 },
    { 0xbf49d23c, SUC_lexicon_tags_7 },
    { 0xbd29d23d, SUC_lexicon_tags_12 },
    { 0x5c69d23d, SUC_lexicon_tags_7 },
    { 0x7769d23f, SUC_lexicon_tags_9 },
    { 0x3109d23e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe9d248, SUC_lexicon_tags_45 },
    { 0x6d09d248, SUC_lexicon_tags_16 },
    { 0x5d29d248, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c89d24d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f29d251, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff09d256, SUC_lexicon_tags_27 },
    { 0x77a9d257, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a9d25e, SUC_lexicon_tags_17 },
    { 0x14a9d25f, SUC_lexicon_tags_17 },
    { 0xfde9d25f, SUC_lexicon_tags_2 },
    { 0xc329d25f, SUC_lexicon_tags_10 },
    { 0xf269d262, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea9d265, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50e9d267, SUC_lexicon_tags_7 },
    { 0x5849d268, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8169d26a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42e9d26c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd09d26e, SUC_lexicon_tags_9 },
    { 0xd249d26f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5089d272, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4609d275, SUC_lexicon_tags_50 },
    { 0x61a9d276, SUC_lexicon_tags_37 },
    { 0xdee9d277, SUC_lexicon_tags_60 },
    { 0x9129d278, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe549d27a, SUC_lexicon_tags_49 },
    { 0x31c9d27b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2709d27d, SUC_lexicon_tags_11 },
    { 0xdce9d27d, SUC_lexicon_tags_11 },
    { 0xe589d27e, SUC_lexicon_tags_37 },
    { 0xb6e9d280, SUC_lexicon_tags_11 },
    { 0x929d27f, SUC_lexicon_tags_11 },
    { 0x4c69d281, SUC_lexicon_tags_83 },
    { 0x8309d27e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7f49d285, SUC_lexicon_tags_25 },
    { 0xeee9d286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4309d288, SUC_lexicon_tags_65 },
    { 0xea69d289, SUC_lexicon_tags_37 },
    { 0xd009d28a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7949d290, SUC_lexicon_tags_46 },
    { 0xc389d290, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd049d29d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd29d29f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2809d2a1, SUC_lexicon_tags_37 },
    { 0xd7c9d2a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b29d2a7, SUC_lexicon_tags_27 },
    { 0xee29d2a7, SUC_lexicon_tags_25 },
    { 0xdb49d2a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc0a9d2ab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc9d2b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46a9d2b3, SUC_lexicon_tags_26 },
    { 0xa689d2b4, SUC_lexicon_tags_11 },
    { 0xce09d2b5, SUC_lexicon_tags_37 },
    { 0xb3e9d2b6, SUC_lexicon_tags_1 },
    { 0xb629d2b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3429d2ba, SUC_lexicon_tags_63 },
    { 0x9ae9d2bb, SUC_lexicon_tags_17 },
    { 0x47c9d2bc, SUC_lexicon_tags_14 },
    { 0xda69d2bd, SUC_lexicon_tags_11 },
    { 0xc749d2be, SUC_lexicon_tags_141 },
    { 0x7129d2bc, SUC_lexicon_tags_1 },
    { 0x85a9d2c0, SUC_lexicon_tags_25 },
    { 0x5ac9d2be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf49d2c3, SUC_lexicon_tags_27 },
    { 0x3c9d2c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa789d2c9, SUC_lexicon_tags_25 },
    { 0xa289d2ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a9d2d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe889d2d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8329d2d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7329d2d9, SUC_lexicon_tags_13 },
    { 0xdf49d2da, SUC_lexicon_tags_1 },
    { 0x2389d2d9, SUC_lexicon_tags_1 },
    { 0x8ae9d2dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe529d2de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd029d2e2, SUC_lexicon_tags_37 },
    { 0xa9c9d2e3, SUC_lexicon_tags_28 },
    { 0xab49d2e2, SUC_lexicon_tags_28 },
    { 0x3109d2e5, SUC_lexicon_tags_26 },
    { 0x73e9d2e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7709d2eb, SUC_lexicon_tags_63 },
    { 0x989d2ec, SUC_lexicon_tags_2 },
    { 0xa9c9d2ed, SUC_lexicon_tags_43 },
    { 0x6489d2eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c49d2f5, SUC_lexicon_tags_2 },
    { 0xeb49d2f6, SUC_lexicon_tags_30 },
    { 0xd109d2f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd4e9d2f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a09d2fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfe29d2fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc89d301, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e69d309, SUC_lexicon_tags_17 },
    { 0xfda9d309, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e9d30f, SUC_lexicon_tags_11 },
    { 0xe189d30f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52c9d317, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6269d31a, SUC_lexicon_tags_17 },
    { 0x609d31a, SUC_lexicon_tags_37 },
    { 0x4fe9d31c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc9d322, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc9d328, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c49d32a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3729d32c, SUC_lexicon_tags_0 },
    { 0xb309d32d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c29d330, SUC_lexicon_tags_19 },
    { 0x7a09d331, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1189d333, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5289d335, SUC_lexicon_tags_46 },
    { 0x31c9d336, SUC_lexicon_tags_17 },
    { 0xefe9d336, SUC_lexicon_tags_25 },
    { 0xaca9d338, SUC_lexicon_tags_8 },
    { 0xe909d336, SUC_lexicon_tags_21 },
    { 0x2bc9d335, SUC_lexicon_tags_37 },
    { 0x4209d33b, SUC_lexicon_tags_37 },
    { 0x56a9d33c, SUC_lexicon_tags_66 },
    { 0x4f49d33d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e9d341, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d344, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8309d349, SUC_lexicon_tags_15 },
    { 0xd129d34a, SUC_lexicon_tags_116 },
    { 0x4be9d34b, SUC_lexicon_tags_28 },
    { 0xca9d34c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16c9d34e, SUC_lexicon_tags_6 },
    { 0xe089d34f, SUC_lexicon_tags_27 },
    { 0x5a89d34e, SUC_lexicon_tags_45 },
    { 0x9c89d350, SUC_lexicon_tags_17 },
    { 0xda09d34e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa249d356, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d29d358, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6749d360, SUC_lexicon_tags_18 },
    { 0x4d29d360, SUC_lexicon_tags_37 },
    { 0x57e9d362, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8729d368, SUC_lexicon_tags_30 },
    { 0x9369d368, SUC_lexicon_tags_17 },
    { 0xe289d36a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d29d36e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1789d372, SUC_lexicon_tags_30 },
    { 0x1369d372, SUC_lexicon_tags_17 },
    { 0xed49d373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd109d376, SUC_lexicon_tags_39 },
    { 0x309d377, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94e9d379, SUC_lexicon_tags_1 },
    { 0xca09d37a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e49d37e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e29d383, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xea89d385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9d38d, SUC_lexicon_tags_8 },
    { 0x76a9d38e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3929d390, SUC_lexicon_tags_15 },
    { 0x3c29d390, SUC_lexicon_tags_60 },
    { 0x87a9d392, SUC_lexicon_tags_25 },
    { 0x2889d392, SUC_lexicon_tags_37 },
    { 0xdbe9d393, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f09d397, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9349d399, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3009d39c, SUC_lexicon_tags_15 },
    { 0x4f69d39c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1209d3a2, SUC_lexicon_tags_15 },
    { 0xbe49d3a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3669d3a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b89d3aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf509d3ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xabe9d3af, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xec69d3b1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbdc9d3b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5029d3b6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xcb29d3b8, SUC_lexicon_tags_12 },
    { 0x7329d3b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc449d3bd, SUC_lexicon_tags_23 },
    { 0xd09d3bd, SUC_lexicon_tags_1 },
    { 0x3449d3bf, SUC_lexicon_tags_53 },
    { 0x9089d3c0, SUC_lexicon_tags_25 },
    { 0xc469d3bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xede9d3c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb49d3c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd0c9d3c9, SUC_lexicon_tags_0 },
    { 0x3329d3ca, SUC_lexicon_tags_88 },
    { 0x3689d3ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7869d3cf, SUC_lexicon_tags_49 },
    { 0xfee9d3d0, SUC_lexicon_tags_47 },
    { 0x4c69d3d0, SUC_lexicon_tags_167 },
    { 0x9909d3d2, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e49d3d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e9d3dd, SUC_lexicon_tags_1 },
    { 0xa6c9d3dd, SUC_lexicon_tags_46 },
    { 0x7ce9d3df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2629d3e3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb409d3ea, SUC_lexicon_tags_27 },
    { 0xce9d3eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b49d3f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5e9d3f3, SUC_lexicon_tags_27 },
    { 0xc029d3f4, SUC_lexicon_tags_12 },
    { 0x4dc9d3f5, SUC_lexicon_tags_12 },
    { 0x6de9d3f5, SUC_lexicon_tags_17 },
    { 0x9ae9d3f5, SUC_lexicon_tags_37 },
    { 0x1de9d3f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a29d3fb, SUC_lexicon_tags_1 },
    { 0xc689d3fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4569d3ff, SUC_lexicon_tags_49 },
    { 0xe5c9d3ff, SUC_lexicon_tags_4 },
    { 0xc9a9d401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e9d404, SUC_lexicon_tags_7 },
    { 0x1d49d405, SUC_lexicon_tags_20 },
    { 0xe4e9d406, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x98e9d409, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8269d40b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f69d40e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61e9d410, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7289d413, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9329d417, SUC_lexicon_tags_61 },
    { 0x8109d418, SUC_lexicon_tags_25 },
    { 0x87c9d417, SUC_lexicon_tags_0 },
    { 0x30e9d41a, SUC_lexicon_tags_21 },
    { 0xbb89d41b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaee9d41d, SUC_lexicon_tags_33 },
    { 0x5ea9d41d, SUC_lexicon_tags_25 },
    { 0xd3a9d41e, SUC_lexicon_tags_11 },
    { 0x8b69d420, SUC_lexicon_tags_25 },
    { 0x4369d421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc09d423, SUC_lexicon_tags_8 },
    { 0xa369d424, SUC_lexicon_tags_1 },
    { 0x23e9d425, SUC_lexicon_tags_5 },
    { 0xf29d426, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a9d42b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe9d42f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6489d431, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa709d434, SUC_lexicon_tags_2 },
    { 0xbf89d434, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1789d439, SUC_lexicon_tags_46 },
    { 0x6fc9d43a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8a69d43c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1069d43e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a9d443, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac9d44a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2969d44d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de9d453, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc49d457, SUC_lexicon_tags_14 },
    { 0x2d69d458, SUC_lexicon_tags_12 },
    { 0x6ae9d459, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd689d45e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x13e9d460, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8189d462, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd69d46a, SUC_lexicon_tags_25 },
    { 0xfbe9d46b, SUC_lexicon_tags_33 },
    { 0x65c9d46c, SUC_lexicon_tags_9 },
    { 0xd749d46b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b49d471, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa469d474, SUC_lexicon_tags_15 },
    { 0x30e9d474, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1649d480, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec9d488, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7349d48e, SUC_lexicon_tags_26 },
    { 0x21e9d48f, SUC_lexicon_tags_43 },
    { 0x1de9d48f, SUC_lexicon_tags_46 },
    { 0x5169d490, SUC_lexicon_tags_2 },
    { 0x28c9d490, SUC_lexicon_tags_15 },
    { 0x1529d493, SUC_lexicon_tags_80 },
    { 0xce89d494, SUC_lexicon_tags_10 },
    { 0x31c9d493, SUC_lexicon_tags_53 },
    { 0x8369d496, SUC_lexicon_tags_47 },
    { 0x5c09d497, SUC_lexicon_tags_38 },
    { 0x2e89d497, SUC_lexicon_tags_7 },
    { 0xe4e9d492, SUC_lexicon_tags_21 },
    { 0x8589d49a, SUC_lexicon_tags_17 },
    { 0x8469d49b, SUC_lexicon_tags_47 },
    { 0xfd69d49c, SUC_lexicon_tags_22 },
    { 0xe909d49d, SUC_lexicon_tags_14 },
    { 0x6809d49c, SUC_lexicon_tags_37 },
    { 0xc649d49f, SUC_lexicon_tags_29 },
    { 0x98c9d49f, SUC_lexicon_tags_37 },
    { 0x9a09d4a1, SUC_lexicon_tags_84 },
    { 0x5409d4a2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x68c9d4a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeb09d4a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c09d4a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb869d4aa, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc89d4ac, SUC_lexicon_tags_87 },
    { 0x9749d4ad, SUC_lexicon_tags_38 },
    { 0x29e9d4ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb449d4b2, SUC_lexicon_tags_2 },
    { 0x5669d4b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63a9d4b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d29d4b7, SUC_lexicon_tags_8 },
    { 0xe6a9d4b8, SUC_lexicon_tags_148 },
    { 0xd409d4b9, SUC_lexicon_tags_37 },
    { 0x7a69d4ba, SUC_lexicon_tags_7 },
    { 0xa729d4b9, SUC_lexicon_tags_11 },
    { 0x3549d4b9, SUC_lexicon_tags_4 },
    { 0x7229d4b9, SUC_lexicon_tags_11 },
    { 0xece9d4be, SUC_lexicon_tags_11 },
    { 0x5a09d4b9, SUC_lexicon_tags_63 },
    { 0x1549d4c0, SUC_lexicon_tags_15 },
    { 0x4349d4c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb29d4c3, SUC_lexicon_tags_0 },
    { 0x6289d4c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd789d4ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc949d4d0, SUC_lexicon_tags_12 },
    { 0x9ce9d4d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe9d4d4, SUC_lexicon_tags_15 },
    { 0x12c9d4d4, SUC_lexicon_tags_60 },
    { 0xf289d4d5, SUC_lexicon_tags_37 },
    { 0x33c9d4d5, SUC_lexicon_tags_35 },
    { 0xae49d4d4, SUC_lexicon_tags_25 },
    { 0xcb89d4d9, SUC_lexicon_tags_31 },
    { 0x2b89d4da, SUC_lexicon_tags_1 },
    { 0xfd89d4da, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x1009d4dd, SUC_lexicon_tags_15 },
    { 0x3829d4de, SUC_lexicon_tags_17 },
    { 0x6d89d4de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57c9d4e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa389d4e3, SUC_lexicon_tags_17 },
    { 0xa8a9d4e4, SUC_lexicon_tags_3 },
    { 0x9fc9d4e4, SUC_lexicon_tags_17 },
    { 0x8ac9d4e3, SUC_lexicon_tags_17 },
    { 0x9ee9d4e7, SUC_lexicon_tags_15 },
    { 0x5e09d4e5, SUC_lexicon_tags_23 },
    { 0x2689d4e9, SUC_lexicon_tags_15 },
    { 0x8609d4ea, SUC_lexicon_tags_1 },
    { 0xfea9d4eb, SUC_lexicon_tags_0 },
    { 0xac89d4eb, SUC_lexicon_tags_0 },
    { 0x96c9d4ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd09d4f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc49d4f6, SUC_lexicon_tags_17 },
    { 0xb3a9d4f6, SUC_lexicon_tags_27 },
    { 0x5129d4f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e9d4fc, SUC_lexicon_tags_37 },
    { 0x3c09d4fc, SUC_lexicon_tags_10 },
    { 0xe029d4fd, SUC_lexicon_tags_11 },
    { 0x5889d4ff, SUC_lexicon_tags_37 },
    { 0xd249d4ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d29d502, SUC_lexicon_tags_47 },
    { 0x94c9d503, SUC_lexicon_tags_37 },
    { 0x52e9d503, SUC_lexicon_tags_37 },
    { 0xeba9d504, SUC_lexicon_tags_7 },
    { 0x10c9d506, SUC_lexicon_tags_0 },
    { 0x7da9d507, SUC_lexicon_tags_12 },
    { 0xe509d508, SUC_lexicon_tags_23 },
    { 0xdf69d508, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2469d50d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a9d518, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53e9d51c, SUC_lexicon_tags_47 },
    { 0x9cc9d51c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8149d51f, SUC_lexicon_tags_25 },
    { 0xea49d520, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1629d525, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4109d527, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6549d52b, SUC_lexicon_tags_32 },
    { 0xdc29d52b, SUC_lexicon_tags_60 },
    { 0xdb89d52d, SUC_lexicon_tags_82 },
    { 0xdda9d52e, SUC_lexicon_tags_0 },
    { 0xaf89d52d, SUC_lexicon_tags_0 },
    { 0xef29d52e, SUC_lexicon_tags_15 },
    { 0x8089d530, SUC_lexicon_tags_29 },
    { 0xd349d530, SUC_lexicon_tags_45 },
    { 0xd129d533, SUC_lexicon_tags_2 },
    { 0x3a49d534, SUC_lexicon_tags_31 },
    { 0x2109d52f, SUC_lexicon_tags_11 },
    { 0x64e9d536, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8129d539, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf909d53c, SUC_lexicon_tags_47 },
    { 0x5b89d53d, SUC_lexicon_tags_2 },
    { 0xc629d53e, SUC_lexicon_tags_11 },
    { 0x61a9d53f, SUC_lexicon_tags_61 },
    { 0x4e09d540, SUC_lexicon_tags_57 },
    { 0x98e9d541, SUC_lexicon_tags_68 },
    { 0x8ce9d542, SUC_lexicon_tags_32 },
    { 0xe009d543, SUC_lexicon_tags_4 },
    { 0x9529d540, SUC_lexicon_tags_61 },
    { 0xcf49d544, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909d54b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf89d54d, SUC_lexicon_tags_1 },
    { 0x4269d54e, SUC_lexicon_tags_31 },
    { 0xb569d54f, SUC_lexicon_tags_66 },
    { 0xe4e9d550, SUC_lexicon_tags_25 },
    { 0x2d09d54f, SUC_lexicon_tags_19 },
    { 0x9009d54f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5389d55b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa189d563, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f69d569, SUC_lexicon_tags_12 },
    { 0x59a9d56a, SUC_lexicon_tags_43 },
    { 0x2409d56a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f29d570, SUC_lexicon_tags_66 },
    { 0xadc9d571, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc889d579, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc49d57b, SUC_lexicon_tags_47 },
    { 0x509d57b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3349d580, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a9d583, SUC_lexicon_tags_12 },
    { 0x8329d583, SUC_lexicon_tags_11 },
    { 0x2549d585, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5569d587, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x82c9d589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de9d58f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9129d591, SUC_lexicon_tags_2 },
    { 0xa489d591, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c49d59f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4a9d5a1, SUC_lexicon_tags_37 },
    { 0x2f09d5a2, SUC_lexicon_tags_167 },
    { 0x73c9d5a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4389d5a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5609d5ad, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe649d5b1, SUC_lexicon_tags_30 },
    { 0x8da9d5b2, SUC_lexicon_tags_60 },
    { 0xa09d5b3, SUC_lexicon_tags_3 },
    { 0xae09d5b4, SUC_lexicon_tags_4 },
    { 0xaee9d5b5, SUC_lexicon_tags_9 },
    { 0x67a9d5b6, SUC_lexicon_tags_14 },
    { 0x4309d5b6, SUC_lexicon_tags_7 },
    { 0x1dc9d5b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f49d5ba, SUC_lexicon_tags_49 },
    { 0xbca9d5bb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5649d5c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c9d5ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f49d5cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6fe9d5d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e9d5d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c9d5d8, SUC_lexicon_tags_12 },
    { 0xbfc9d5d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e9d5dc, SUC_lexicon_tags_161 },
    { 0x4289d5dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9d5e3, SUC_lexicon_tags_25 },
    { 0x4c9d5e4, SUC_lexicon_tags_14 },
    { 0x8ce9d5e3, SUC_lexicon_tags_3 },
    { 0x2129d5e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9d5ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea9d5ee, SUC_lexicon_tags_38 },
    { 0x2d49d5ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66c9d5f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc309d5fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf569d5fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a9d602, SUC_lexicon_tags_11 },
    { 0x829d602, SUC_lexicon_tags_37 },
    { 0x31c9d604, SUC_lexicon_tags_107 },
    { 0x6849d604, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3489d607, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe529d60a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b29d611, SUC_lexicon_tags_47 },
    { 0x6c29d612, SUC_lexicon_tags_25 },
    { 0x6229d611, SUC_lexicon_tags_4 },
    { 0xd3e9d612, SUC_lexicon_tags_47 },
    { 0xab49d615, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xadc9d617, SUC_lexicon_tags_47 },
    { 0x6fe9d618, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60a9d61a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81c9d61c, SUC_lexicon_tags_6 },
    { 0x769d61d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf49d621, SUC_lexicon_tags_23 },
    { 0x1689d622, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7689d628, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a9d62b, SUC_lexicon_tags_9 },
    { 0x38a9d62b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9989d62e, SUC_lexicon_tags_17 },
    { 0xa3c9d62f, SUC_lexicon_tags_2 },
    { 0x5a69d62f, SUC_lexicon_tags_25 },
    { 0x2849d631, SUC_lexicon_tags_23 },
    { 0xc369d62e, SUC_lexicon_tags_15 },
    { 0x5829d632, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c9d637, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac9d63a, SUC_lexicon_tags_31 },
    { 0x4f49d63b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9169d63e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdb09d640, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6629d642, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa969d644, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c9d648, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9c9d64a, SUC_lexicon_tags_17 },
    { 0xeaa9d64b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xab09d64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa9d655, SUC_lexicon_tags_17 },
    { 0xf1c9d656, SUC_lexicon_tags_6 },
    { 0x5209d657, SUC_lexicon_tags_48 },
    { 0x60a9d656, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc9d65b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b09d661, SUC_lexicon_tags_15 },
    { 0x8b09d661, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a9d666, SUC_lexicon_tags_29 },
    { 0xc329d667, SUC_lexicon_tags_11 },
    { 0x3489d666, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c9d66b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7829d66e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc829d671, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef49d673, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf369d675, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6c9d677, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a49d67a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4569d680, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e9d683, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb509d689, SUC_lexicon_tags_63 },
    { 0xae29d68a, SUC_lexicon_tags_7 },
    { 0x269d68b, SUC_lexicon_tags_14 },
    { 0xa689d68a, SUC_lexicon_tags_2 },
    { 0xf4e9d68d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe29d691, SUC_lexicon_tags_20 },
    { 0x50c9d692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca9d696, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a09d69b, SUC_lexicon_tags_15 },
    { 0x8ec9d69b, SUC_lexicon_tags_12 },
    { 0x92c9d69d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa9d6a0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c9d6a8, SUC_lexicon_tags_15 },
    { 0x8a89d6a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4889d6ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaae9d6ad, SUC_lexicon_tags_1 },
    { 0xe889d6ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6769d6b0, SUC_lexicon_tags_60 },
    { 0x3049d6b1, SUC_lexicon_tags_31 },
    { 0x39e9d6b2, SUC_lexicon_tags_30 },
    { 0xe049d6b3, SUC_lexicon_tags_23 },
    { 0xb029d6b3, SUC_lexicon_tags_63 },
    { 0xfdc9d6b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc49d6bb, SUC_lexicon_tags_1 },
    { 0x9d09d6bb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e9d6c2, SUC_lexicon_tags_28 },
    { 0x7fe9d6c3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c9d6c8, SUC_lexicon_tags_7 },
    { 0xcd69d6c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e89d6cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee89d6d4, SUC_lexicon_tags_2 },
    { 0x7b89d6d5, SUC_lexicon_tags_6 },
    { 0x2d69d6d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf349d6da, SUC_lexicon_tags_46 },
    { 0x4fe9d6db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1a9d6dd, SUC_lexicon_tags_7 },
    { 0x53c9d6de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c9d6e2, SUC_lexicon_tags_15 },
    { 0x32c9d6e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd89d6e5, SUC_lexicon_tags_33 },
    { 0xde9d6e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9529d6ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe9d6f0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc09d6f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa9d6f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda09d6f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9009d6fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c9d701, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61c9d703, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd49d705, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c9d709, SUC_lexicon_tags_21 },
    { 0x61c9d70a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4349d70e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e9d711, SUC_lexicon_tags_17 },
    { 0x7029d711, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde69d716, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69d719, SUC_lexicon_tags_15 },
    { 0xe449d719, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c29d71e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb49d721, SUC_lexicon_tags_16 },
    { 0xb569d721, SUC_lexicon_tags_15 },
    { 0xa089d723, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e89d726, SUC_lexicon_tags_37 },
    { 0xb189d727, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3509d730, SUC_lexicon_tags_17 },
    { 0xd849d730, SUC_lexicon_tags_12 },
    { 0x4489d732, SUC_lexicon_tags_21 },
    { 0x129d733, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e9d736, SUC_lexicon_tags_68 },
    { 0x5b69d737, SUC_lexicon_tags_23 },
    { 0x8669d738, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6a9d73a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1bc9d73c, SUC_lexicon_tags_17 },
    { 0xb69d73d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d89d741, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4789d743, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8fc9d745, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f69d74a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a49d74d, SUC_lexicon_tags_55 },
    { 0xa009d74d, SUC_lexicon_tags_11 },
    { 0xfe89d74f, SUC_lexicon_tags_26 },
    { 0xf929d74d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb109d754, SUC_lexicon_tags_38 },
    { 0x2ee9d755, SUC_lexicon_tags_19 },
    { 0x3989d754, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa9d75c, SUC_lexicon_tags_24 },
    { 0x3fc9d75d, SUC_lexicon_tags_11 },
    { 0x3e49d75d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee9d762, SUC_lexicon_tags_37 },
    { 0x109d763, SUC_lexicon_tags_12 },
    { 0xb6c9d764, SUC_lexicon_tags_29 },
    { 0x4069d765, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec9d76a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e9d76e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e9d771, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe969d773, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf609d775, SUC_lexicon_tags_1 },
    { 0xfc09d776, SUC_lexicon_tags_4 },
    { 0xe8c9d776, SUC_lexicon_tags_6 },
    { 0x7489d778, SUC_lexicon_tags_6 },
    { 0xb9e9d776, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c9d77e, SUC_lexicon_tags_17 },
    { 0xdf89d77f, SUC_lexicon_tags_100 },
    { 0xc369d780, SUC_lexicon_tags_39 },
    { 0x2e09d781, SUC_lexicon_tags_38 },
    { 0x2e69d77e, SUC_lexicon_tags_33 },
    { 0x67c9d781, SUC_lexicon_tags_22 },
    { 0xe049d784, SUC_lexicon_tags_17 },
    { 0x6869d785, SUC_lexicon_tags_27 },
    { 0x8dc9d784, SUC_lexicon_tags_49 },
    { 0x98c9d77f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc629d789, SUC_lexicon_tags_12 },
    { 0x6529d78a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xede9d78d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d69d791, SUC_lexicon_tags_4 },
    { 0xd529d792, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4ce9d794, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbb89d796, SUC_lexicon_tags_37 },
    { 0xa089d796, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e9d79d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5469d79f, SUC_lexicon_tags_21 },
    { 0x6be9d7a0, SUC_lexicon_tags_7 },
    { 0xbce9d7a0, SUC_lexicon_tags_12 },
    { 0x9c9d7a2, SUC_lexicon_tags_33 },
    { 0x1ce9d7a3, SUC_lexicon_tags_35 },
    { 0xbb29d7a4, SUC_lexicon_tags_19 },
    { 0x4149d7a4, SUC_lexicon_tags_37 },
    { 0x1a89d7a4, SUC_lexicon_tags_25 },
    { 0x7369d7a2, SUC_lexicon_tags_37 },
    { 0xefa9d7a6, SUC_lexicon_tags_11 },
    { 0x82e9d7a7, SUC_lexicon_tags_12 },
    { 0x1889d7a3, SUC_lexicon_tags_11 },
    { 0x2c49d7ab, SUC_lexicon_tags_9 },
    { 0xd609d7a7, SUC_lexicon_tags_1 },
    { 0x8349d7a0, SUC_lexicon_tags_1 },
    { 0xc89d7a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6269d7b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489d7b6, SUC_lexicon_tags_14 },
    { 0xaa69d7b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa9d7bd, SUC_lexicon_tags_60 },
    { 0xaf89d7bd, SUC_lexicon_tags_55 },
    { 0x8ac9d7bd, SUC_lexicon_tags_0 },
    { 0xd869d7bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe609d7c6, SUC_lexicon_tags_8 },
    { 0x589d7c7, SUC_lexicon_tags_19 },
    { 0x9749d7c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5729d7cb, SUC_lexicon_tags_1 },
    { 0x6349d7cc, SUC_lexicon_tags_10 },
    { 0x41c9d7cd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d49d7d0, SUC_lexicon_tags_23 },
    { 0x1c49d7d1, SUC_lexicon_tags_77 },
    { 0x88e9d7d1, SUC_lexicon_tags_19 },
    { 0xae29d7d1, SUC_lexicon_tags_12 },
    { 0x9a09d7d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e29d7d8, SUC_lexicon_tags_12 },
    { 0xd429d7d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a9d7de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3169d7e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabc9d7e3, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a09d7ea, SUC_lexicon_tags_11 },
    { 0xaca9d7eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7029d7f0, SUC_lexicon_tags_26 },
    { 0x3409d7f1, SUC_lexicon_tags_25 },
    { 0xb429d7f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd049d7f4, SUC_lexicon_tags_15 },
    { 0x5649d7f4, SUC_lexicon_tags_6 },
    { 0xa4a9d7f6, SUC_lexicon_tags_14 },
    { 0x76e9d7f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe849d7fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a9d803, SUC_lexicon_tags_19 },
    { 0xea89d804, SUC_lexicon_tags_1 },
    { 0xeb89d803, SUC_lexicon_tags_15 },
    { 0x9c69d804, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a9d80b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f29d80d, SUC_lexicon_tags_17 },
    { 0xb2a9d80e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9989d810, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34c9d819, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf669d81b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x749d81e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2269d820, SUC_lexicon_tags_14 },
    { 0xf6a9d821, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd409d823, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a9d827, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c9d82a, SUC_lexicon_tags_9 },
    { 0x74a9d82b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c49d82e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f69d831, SUC_lexicon_tags_19 },
    { 0xfd89d832, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5689d836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc89d838, SUC_lexicon_tags_12 },
    { 0x1889d838, SUC_lexicon_tags_25 },
    { 0xb609d83a, SUC_lexicon_tags_27 },
    { 0x9109d838, SUC_lexicon_tags_17 },
    { 0xc789d838, SUC_lexicon_tags_12 },
    { 0xdf49d83d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa3c9d83f, SUC_lexicon_tags_1 },
    { 0x1869d840, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5a09d842, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8c9d848, SUC_lexicon_tags_37 },
    { 0x3ba9d849, SUC_lexicon_tags_25 },
    { 0xf8a9d84a, SUC_lexicon_tags_21 },
    { 0x21e9d84a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31c9d84d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd709d84f, SUC_lexicon_tags_6 },
    { 0xfea9d850, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x3b89d852, SUC_lexicon_tags_11 },
    { 0x4029d853, SUC_lexicon_tags_19 },
    { 0xe7e9d852, SUC_lexicon_tags_6 },
    { 0xe8e9d852, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7249d858, SUC_lexicon_tags_1 },
    { 0x4869d858, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd089d85c, SUC_lexicon_tags_63 },
    { 0xa969d85d, SUC_lexicon_tags_11 },
    { 0x5b89d85d, SUC_lexicon_tags_43 },
    { 0x2469d85e, SUC_lexicon_tags_8 },
    { 0x5b49d85f, SUC_lexicon_tags_25 },
    { 0x22e9d860, SUC_lexicon_tags_68 },
    { 0x23a9d862, SUC_lexicon_tags_8 },
    { 0x2209d861, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb109d866, SUC_lexicon_tags_17 },
    { 0x6f89d867, SUC_lexicon_tags_2 },
    { 0x94c9d868, SUC_lexicon_tags_33 },
    { 0x7589d866, SUC_lexicon_tags_44 },
    { 0x6c09d868, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e9d86d, SUC_lexicon_tags_35 },
    { 0x8dc9d86e, SUC_lexicon_tags_17 },
    { 0xa569d86f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde9d871, SUC_lexicon_tags_77 },
    { 0xebc9d871, SUC_lexicon_tags_14 },
    { 0x1629d873, SUC_lexicon_tags_17 },
    { 0xaea9d874, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc49d877, SUC_lexicon_tags_14 },
    { 0x7349d878, SUC_lexicon_tags_12 },
    { 0x85a9d879, SUC_lexicon_tags_38 },
    { 0xeca9d878, SUC_lexicon_tags_81 },
    { 0x989d878, SUC_lexicon_tags_0 },
    { 0xa09d87a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc769d87e, SUC_lexicon_tags_14 },
    { 0x3089d87e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e9d883, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5fa9d885, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c9d888, SUC_lexicon_tags_11 },
    { 0x1309d889, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa89d88d, SUC_lexicon_tags_0 },
    { 0x9d89d88e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce9d896, SUC_lexicon_tags_9 },
    { 0xa109d897, SUC_lexicon_tags_11 },
    { 0x589d896, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb969d89b, SUC_lexicon_tags_8 },
    { 0xc069d89c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1409d89e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca9d8a4, SUC_lexicon_tags_39 },
    { 0xe09d8a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8949d8ab, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee9d8ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x389d8b0, SUC_lexicon_tags_1 },
    { 0xd0a9d8b1, SUC_lexicon_tags_45 },
    { 0xbf09d8b1, SUC_lexicon_tags_15 },
    { 0x1829d8b2, SUC_lexicon_tags_8 },
    { 0x5dc9d8b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9e9d8b6, SUC_lexicon_tags_39 },
    { 0x1689d8b7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9449d8ba, SUC_lexicon_tags_37 },
    { 0xd3c9d8ba, SUC_lexicon_tags_14 },
    { 0xe189d8bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa9d8c1, SUC_lexicon_tags_7 },
    { 0x5129d8c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc709d8c4, SUC_lexicon_tags_17 },
    { 0xe529d8c4, SUC_lexicon_tags_41 },
    { 0x98a9d8c6, SUC_lexicon_tags_1 },
    { 0x7309d8c4, SUC_lexicon_tags_7 },
    { 0x6a89d8c5, SUC_lexicon_tags_28 },
    { 0x8889d8c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe9d8cb, SUC_lexicon_tags_0 },
    { 0x249d8cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2309d8ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9c9d8d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcd29d8d2, SUC_lexicon_tags_30 },
    { 0xe1e9d8d3, SUC_lexicon_tags_0 },
    { 0xab49d8d4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7869d8d7, SUC_lexicon_tags_11 },
    { 0x4f09d8d7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8d69d8da, SUC_lexicon_tags_7 },
    { 0xd809d8db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x81a9d8dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7089d8e1, SUC_lexicon_tags_60 },
    { 0x2e29d8e1, SUC_lexicon_tags_1152 },
    { 0x6e49d8e3, SUC_lexicon_tags_31 },
    { 0x1d49d8e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x47a9d8e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e9d8e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc1e9d8eb, SUC_lexicon_tags_33 },
    { 0x32c9d8ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4509d8f0, SUC_lexicon_tags_26 },
    { 0x5ea9d8f1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec29d8f7, SUC_lexicon_tags_19 },
    { 0xb069d8f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68c9d8fa, SUC_lexicon_tags_55 },
    { 0x2e49d8fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc089d902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d69d904, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d69d907, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfda9d909, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11e9d90c, SUC_lexicon_tags_79 },
    { 0xfea9d90d, SUC_lexicon_tags_6 },
    { 0x79c9d90e, SUC_lexicon_tags_12 },
    { 0xb89d90f, SUC_lexicon_tags_27 },
    { 0x8089d910, SUC_lexicon_tags_12 },
    { 0x8cc9d90d, SUC_lexicon_tags_19 },
    { 0x2009d912, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c9d919, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5f09d91b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7609d921, SUC_lexicon_tags_61 },
    { 0x4d49d922, SUC_lexicon_tags_6 },
    { 0xa49d923, SUC_lexicon_tags_37 },
    { 0x8309d924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcea9d926, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e89d929, SUC_lexicon_tags_66 },
    { 0x6589d92a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xca29d92c, SUC_lexicon_tags_63 },
    { 0x2989d92c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe9d930, SUC_lexicon_tags_56 },
    { 0x8a89d931, SUC_lexicon_tags_37 },
    { 0xbc29d932, SUC_lexicon_tags_9 },
    { 0x9369d933, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da9d944, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84c9d946, SUC_lexicon_tags_37 },
    { 0x2ce9d947, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab89d94b, SUC_lexicon_tags_16 },
    { 0x7fc9d94b, SUC_lexicon_tags_25 },
    { 0x9e89d94d, SUC_lexicon_tags_49 },
    { 0x3f69d94d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe029d950, SUC_lexicon_tags_8 },
    { 0xa129d951, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a9d957, SUC_lexicon_tags_13 },
    { 0xdbc9d958, SUC_lexicon_tags_37 },
    { 0x3769d958, SUC_lexicon_tags_15 },
    { 0x2ee9d959, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de9d95f, SUC_lexicon_tags_0 },
    { 0x14e9d960, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe969d966, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba9d96f, SUC_lexicon_tags_1 },
    { 0xd389d970, SUC_lexicon_tags_63 },
    { 0x3d69d96f, SUC_lexicon_tags_11 },
    { 0x3469d972, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb349d977, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc669d97c, SUC_lexicon_tags_45 },
    { 0xdb49d97c, SUC_lexicon_tags_15 },
    { 0x2f29d97c, SUC_lexicon_tags_19 },
    { 0xdf09d97f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb029d981, SUC_lexicon_tags_62 },
    { 0xb8a9d981, SUC_lexicon_tags_23 },
    { 0xafe9d981, SUC_lexicon_tags_1 },
    { 0xb369d981, SUC_lexicon_tags_37 },
    { 0xf449d983, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5829d987, SUC_lexicon_tags_27 },
    { 0x4b29d988, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c9d98b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c9d98e, SUC_lexicon_tags_15 },
    { 0xa449d98e, SUC_lexicon_tags_17 },
    { 0x4069d98e, SUC_lexicon_tags_11 },
    { 0x6e29d98e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5fa9d993, SUC_lexicon_tags_61 },
    { 0xab69d994, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d09d999, SUC_lexicon_tags_11 },
    { 0xd1a9d99a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6589d99f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80e9d9a1, SUC_lexicon_tags_7 },
    { 0xace9d9a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7769d9a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd49d9a9, SUC_lexicon_tags_17 },
    { 0x9209d9aa, SUC_lexicon_tags_29 },
    { 0xb5a9d9aa, SUC_lexicon_tags_26 },
    { 0x5209d9ab, SUC_lexicon_tags_12 },
    { 0x8e09d9ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1329d9b0, SUC_lexicon_tags_11 },
    { 0x7ec9d9b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cc9d9b6, SUC_lexicon_tags_25 },
    { 0xcb29d9b7, SUC_lexicon_tags_79 },
    { 0xca09d9b6, SUC_lexicon_tags_25 },
    { 0x44c9d9b9, SUC_lexicon_tags_48 },
    { 0xa049d9ba, SUC_lexicon_tags_49 },
    { 0xe369d9bb, SUC_lexicon_tags_79 },
    { 0x3fa9d9bb, SUC_lexicon_tags_51 },
    { 0xc689d9bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea9d9c2, SUC_lexicon_tags_37 },
    { 0xae29d9c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xed89d9c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d29d9c7, SUC_lexicon_tags_8 },
    { 0x2909d9c7, SUC_lexicon_tags_15 },
    { 0x5d89d9c8, SUC_lexicon_tags_11 },
    { 0x8f29d9ca, SUC_lexicon_tags_25 },
    { 0xbc29d9c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8889d9cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55a9d9d1, SUC_lexicon_tags_26 },
    { 0xf029d9d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9d9db, SUC_lexicon_tags_49 },
    { 0xfac9d9db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb169d9e0, SUC_lexicon_tags_17 },
    { 0x70c9d9e1, SUC_lexicon_tags_46 },
    { 0xfa09d9e1, SUC_lexicon_tags_17 },
    { 0x2f89d9e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d09d9e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd409d9e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf149d9eb, SUC_lexicon_tags_11 },
    { 0x80a9d9ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc889d9f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7729d9f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e89d9fa, SUC_lexicon_tags_63 },
    { 0xb369d9fb, SUC_lexicon_tags_27 },
    { 0xaa09d9fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c9d9ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2569da01, SUC_lexicon_tags_46 },
    { 0x7a89da02, SUC_lexicon_tags_42 },
    { 0x7d69da02, SUC_lexicon_tags_49 },
    { 0xdee9da04, SUC_lexicon_tags_47 },
    { 0x2489da02, SUC_lexicon_tags_2 },
    { 0x4589da06, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a89da0d, SUC_lexicon_tags_7 },
    { 0xa069da0e, SUC_lexicon_tags_15 },
    { 0x489da0f, SUC_lexicon_tags_10 },
    { 0x1129da10, SUC_lexicon_tags_65 },
    { 0x9949da11, SUC_lexicon_tags_37 },
    { 0x4409da10, SUC_lexicon_tags_29 },
    { 0x66c9da10, SUC_lexicon_tags_12 },
    { 0x4b09da0e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e89da1a, SUC_lexicon_tags_33 },
    { 0x79c9da1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a9da1e, SUC_lexicon_tags_9 },
    { 0x1129da1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de9da23, SUC_lexicon_tags_15 },
    { 0x9d69da24, SUC_lexicon_tags_31 },
    { 0x70c9da24, SUC_lexicon_tags_1 },
    { 0x86e9da26, SUC_lexicon_tags_12 },
    { 0x2aa9da25, SUC_lexicon_tags_17 },
    { 0xe09da27, SUC_lexicon_tags_11 },
    { 0x1a89da29, SUC_lexicon_tags_25 },
    { 0xd249da2a, SUC_lexicon_tags_2 },
    { 0xdc9da2b, SUC_lexicon_tags_17 },
    { 0x7ce9da2c, SUC_lexicon_tags_2 },
    { 0xb0c9da2d, SUC_lexicon_tags_39 },
    { 0xa469da27, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3289da33, SUC_lexicon_tags_29 },
    { 0x8709da34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7ac9da36, SUC_lexicon_tags_11 },
    { 0x9849da37, SUC_lexicon_tags_12 },
    { 0x2ee9da37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae9da3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc509da3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfe9da3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac29da49, SUC_lexicon_tags_25 },
    { 0x7169da4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9da4e, SUC_lexicon_tags_45 },
    { 0xdd89da4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe09da52, SUC_lexicon_tags_2 },
    { 0x49a9da53, SUC_lexicon_tags_47 },
    { 0x3f89da53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb09da5a, SUC_lexicon_tags_48 },
    { 0x97e9da5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa709da5f, SUC_lexicon_tags_2 },
    { 0x9629da60, SUC_lexicon_tags_37 },
    { 0x9189da60, SUC_lexicon_tags_7 },
    { 0x2509da60, SUC_lexicon_tags_45 },
    { 0x5e49da61, SUC_lexicon_tags_12 },
    { 0x4fe9da63, SUC_lexicon_tags_12 },
    { 0xe5e9da60, SUC_lexicon_tags_7 },
    { 0x1829da66, SUC_lexicon_tags_53 },
    { 0x3e09da65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa29da6a, SUC_lexicon_tags_39 },
    { 0xc3a9da6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5029da6d, SUC_lexicon_tags_109 },
    { 0xbde9da6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a09da70, SUC_lexicon_tags_8 },
    { 0xe829da70, SUC_lexicon_tags_10 },
    { 0xf769da72, SUC_lexicon_tags_77 },
    { 0x3609da73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc509da75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc3c9da77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5e9da79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f89da7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab89da7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2469da7f, SUC_lexicon_tags_38 },
    { 0xd029da80, SUC_lexicon_tags_37 },
    { 0xd5e9da81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c9da8a, SUC_lexicon_tags_21 },
    { 0x4209da8b, SUC_lexicon_tags_7 },
    { 0x5269da8c, SUC_lexicon_tags_12 },
    { 0x2469da8d, SUC_lexicon_tags_66 },
    { 0x11c9da8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5589da9a, SUC_lexicon_tags_42 },
    { 0xa769da9b, SUC_lexicon_tags_11 },
    { 0x2029da9a, SUC_lexicon_tags_14 },
    { 0x9fc9da9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fa9da9f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc9daa9, SUC_lexicon_tags_4 },
    { 0xf7e9daaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa009daad, SUC_lexicon_tags_14 },
    { 0xd909daae, SUC_lexicon_tags_2 },
    { 0xee09daaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc209dab1, SUC_lexicon_tags_19 },
    { 0x3449dab2, SUC_lexicon_tags_43 },
    { 0x1809dab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369dab8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529dabd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e69dac7, SUC_lexicon_tags_35 },
    { 0x7ee9dac8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7569daca, SUC_lexicon_tags_19 },
    { 0x3cc9dacb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x25e9dacd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e29dad0, SUC_lexicon_tags_16 },
    { 0x9629dad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7889dad4, SUC_lexicon_tags_10 },
    { 0xbdc9dad5, SUC_lexicon_tags_37 },
    { 0x6e69dad5, SUC_lexicon_tags_17 },
    { 0xa129dad7, SUC_lexicon_tags_16 },
    { 0xfc9dad5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7389dada, SUC_lexicon_tags_25 },
    { 0x90e9dadb, SUC_lexicon_tags_63 },
    { 0xc569dada, SUC_lexicon_tags_43 },
    { 0xc389dadd, SUC_lexicon_tags_37 },
    { 0x20c9dadb, SUC_lexicon_tags_82 },
    { 0xc789dadf, SUC_lexicon_tags_0 },
    { 0xfe69dadf, SUC_lexicon_tags_30 },
    { 0xa309dadf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2989dae4, SUC_lexicon_tags_17 },
    { 0x369dae4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b29daec, SUC_lexicon_tags_16 },
    { 0x2c69daed, SUC_lexicon_tags_9 },
    { 0x65a9daee, SUC_lexicon_tags_29 },
    { 0x1069daef, SUC_lexicon_tags_12 },
    { 0xc49daef, SUC_lexicon_tags_44 },
    { 0xd649daf1, SUC_lexicon_tags_46 },
    { 0x1629daed, SUC_lexicon_tags_74 },
    { 0x53e9daee, SUC_lexicon_tags_27 },
    { 0x2dc9daf2, SUC_lexicon_tags_38 },
    { 0xfe69daf1, SUC_lexicon_tags_15 },
    { 0x3229daf6, SUC_lexicon_tags_40 },
    { 0xc209daf7, SUC_lexicon_tags_60 },
    { 0xabc9daec, SUC_lexicon_tags_65 },
    { 0xed09daf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb449dafc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ce9dafe, SUC_lexicon_tags_37 },
    { 0xda9daff, SUC_lexicon_tags_12 },
    { 0xa5e9daff, SUC_lexicon_tags_1 },
    { 0x28a9db01, SUC_lexicon_tags_15 },
    { 0x6529db02, SUC_lexicon_tags_47 },
    { 0x1549db00, SUC_lexicon_tags_68 },
    { 0xabc9db04, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9869db06, SUC_lexicon_tags_40 },
    { 0xde49db07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4e9db0b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229db10, SUC_lexicon_tags_0 },
    { 0x7349db11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7689db18, SUC_lexicon_tags_235 },
    { 0x61a9db18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a9db21, SUC_lexicon_tags_3 },
    { 0xdba9db22, SUC_lexicon_tags_12 },
    { 0xdde9db22, SUC_lexicon_tags_13 },
    { 0xee49db24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e49db2c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2529db33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a29db35, SUC_lexicon_tags_10 },
    { 0xe089db35, SUC_lexicon_tags_12 },
    { 0xf8a9db37, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba9db3f, SUC_lexicon_tags_68 },
    { 0x35e9db40, SUC_lexicon_tags_55 },
    { 0x9fe9db41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3469db44, SUC_lexicon_tags_3 },
    { 0x6bc9db45, SUC_lexicon_tags_24 },
    { 0xada9db44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9129db48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa069db4d, SUC_lexicon_tags_11 },
    { 0x2569db4d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49db53, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e9db57, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee9db5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a49db60, SUC_lexicon_tags_25 },
    { 0x829db61, SUC_lexicon_tags_10 },
    { 0xbd29db62, SUC_lexicon_tags_77 },
    { 0xa909db63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6089db65, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x9a69db67, SUC_lexicon_tags_2 },
    { 0xd309db68, SUC_lexicon_tags_49 },
    { 0x5489db67, SUC_lexicon_tags_17 },
    { 0x7069db6a, SUC_lexicon_tags_45 },
    { 0xddc9db6b, SUC_lexicon_tags_63 },
    { 0x9f89db68, SUC_lexicon_tags_33 },
    { 0x1569db6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8489db70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4f89db72, SUC_lexicon_tags_23 },
    { 0xcd09db72, SUC_lexicon_tags_30 },
    { 0x41e9db74, SUC_lexicon_tags_11 },
    { 0x21c9db74, SUC_lexicon_tags_60 },
    { 0x7009db72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc829db79, SUC_lexicon_tags_45 },
    { 0xa569db7a, SUC_lexicon_tags_38 },
    { 0xf3e9db7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e29db7e, SUC_lexicon_tags_9 },
    { 0x6389db7e, SUC_lexicon_tags_38 },
    { 0xb889db80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea9db85, SUC_lexicon_tags_37 },
    { 0x4d49db86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8069db88, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x21c9db8b, SUC_lexicon_tags_14 },
    { 0x1089db8b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ac9db92, SUC_lexicon_tags_25 },
    { 0x9c89db92, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5009db95, SUC_lexicon_tags_11 },
    { 0x6c9db96, SUC_lexicon_tags_63 },
    { 0x7989db96, SUC_lexicon_tags_16 },
    { 0x7c89db98, SUC_lexicon_tags_12 },
    { 0xf209db98, SUC_lexicon_tags_15 },
    { 0xfda9db95, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xae9db9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe29dba3, SUC_lexicon_tags_30 },
    { 0xd269dba4, SUC_lexicon_tags_4 },
    { 0x8829dba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ee9dba7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc949dbb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1109dbb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3c9dbb7, SUC_lexicon_tags_87 },
    { 0xe0c9dbb7, SUC_lexicon_tags_33 },
    { 0x5309dbb7, SUC_lexicon_tags_12 },
    { 0xec29dbba, SUC_lexicon_tags_11 },
    { 0xbca9dbb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8889dbc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce9dbcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e9dbd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e9dbd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4be9dbd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x949dbd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ea9dbda, SUC_lexicon_tags_11 },
    { 0x7709dbdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29dbdd, SUC_lexicon_tags_59 },
    { 0xf789dbde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e9dbe4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd129dbe8, SUC_lexicon_tags_5 },
    { 0x3029dbe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9229dbeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd69dbee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c9dbf1, SUC_lexicon_tags_7 },
    { 0x9b09dbf1, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b09dbf5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc269dbf8, SUC_lexicon_tags_14 },
    { 0xcd29dbf9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5869dbfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd229dbfd, SUC_lexicon_tags_1 },
    { 0x9ec9dbfd, SUC_lexicon_tags_43 },
    { 0xd549dbff, SUC_lexicon_tags_16 },
    { 0x10e9dc00, SUC_lexicon_tags_3 },
    { 0x5589dc01, SUC_lexicon_tags_23 },
    { 0xb549dc02, SUC_lexicon_tags_32 },
    { 0x1a49dbff, SUC_lexicon_tags_25 },
    { 0x9909dbfe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb109dc06, SUC_lexicon_tags_47 },
    { 0x32a9dc06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9dc0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6869dc0e, SUC_lexicon_tags_66 },
    { 0x5d29dc0f, SUC_lexicon_tags_57 },
    { 0x5ea9dc10, SUC_lexicon_tags_37 },
    { 0x8969dc10, SUC_lexicon_tags_19 },
    { 0x8109dc10, SUC_lexicon_tags_17 },
    { 0x8029dc13, SUC_lexicon_tags_1426 },
    { 0x1389dc13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69dc19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c29dc1b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc009dc1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce9dc20, SUC_lexicon_tags_6 },
    { 0xc9c9dc20, SUC_lexicon_tags_17 },
    { 0x7b49dc22, SUC_lexicon_tags_17 },
    { 0xff49dc23, SUC_lexicon_tags_26 },
    { 0x8089dc24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43c9dc27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5089dc2d, SUC_lexicon_tags_33 },
    { 0xa49dc2d, SUC_lexicon_tags_10 },
    { 0x2f09dc2e, SUC_lexicon_tags_9 },
    { 0x4349dc2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3569dc39, SUC_lexicon_tags_39 },
    { 0xd289dc39, SUC_lexicon_tags_11 },
    { 0x7129dc3b, SUC_lexicon_tags_29 },
    { 0x6ce9dc3c, SUC_lexicon_tags_19 },
    { 0x72c9dc3b, SUC_lexicon_tags_12 },
    { 0x39a9dc3e, SUC_lexicon_tags_10 },
    { 0x4909dc3f, SUC_lexicon_tags_17 },
    { 0x9ca9dc3f, SUC_lexicon_tags_17 },
    { 0x4be9dc41, SUC_lexicon_tags_23 },
    { 0x7409dc42, SUC_lexicon_tags_46 },
    { 0x69a9dc41, SUC_lexicon_tags_60 },
    { 0x11e9dc39, SUC_lexicon_tags_100 },
    { 0xf509dc39, SUC_lexicon_tags_14 },
    { 0x1969dc46, SUC_lexicon_tags_48 },
    { 0xd6e9dc47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6789dc49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c9dc4c, SUC_lexicon_tags_11 },
    { 0xda9dc4c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c9dc51, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b89dc54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3809dc56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0c9dc58, SUC_lexicon_tags_1 },
    { 0x8789dc59, SUC_lexicon_tags_38 },
    { 0xd1e9dc59, SUC_lexicon_tags_37 },
    { 0xc609dc59, SUC_lexicon_tags_7 },
    { 0x5249dc5c, SUC_lexicon_tags_33 },
    { 0x38a9dc5c, SUC_lexicon_tags_47 },
    { 0x9769dc59, SUC_lexicon_tags_75 },
    { 0x7929dc59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0e9dc61, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc9dc67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe389dc69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f49dc6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5229dc70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb509dc73, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f69dc79, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3729dc7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa9dc81, SUC_lexicon_tags_16 },
    { 0xb869dc82, SUC_lexicon_tags_27 },
    { 0x1869dc82, SUC_lexicon_tags_15 },
    { 0xb4c9dc81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdac9dc86, SUC_lexicon_tags_15 },
    { 0xdda9dc87, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a89dc8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c9dc96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc689dc98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb249dc9a, SUC_lexicon_tags_25 },
    { 0xb289dc9a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c9dca4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569dca9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dc9dcab, SUC_lexicon_tags_17 },
    { 0x1f09dcac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e49dcb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc9dcba, SUC_lexicon_tags_45 },
    { 0xa609dcbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x94a9dcbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9869dcbf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x62e9dcc2, SUC_lexicon_tags_11 },
    { 0x95c9dcc3, SUC_lexicon_tags_9 },
    { 0x5769dcc4, SUC_lexicon_tags_15 },
    { 0xb29dcc5, SUC_lexicon_tags_28 },
    { 0xb089dcc6, SUC_lexicon_tags_11 },
    { 0xd749dcc4, SUC_lexicon_tags_15 },
    { 0xe309dcc8, SUC_lexicon_tags_63 },
    { 0x3e69dcc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549dccf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x11c9dcd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1429dcd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3309dcd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91c9dcdf, SUC_lexicon_tags_30 },
    { 0xed29dce0, SUC_lexicon_tags_11 },
    { 0x3149dcdf, SUC_lexicon_tags_2 },
    { 0xeec9dce2, SUC_lexicon_tags_1 },
    { 0xf5a9dce3, SUC_lexicon_tags_37 },
    { 0xcc9dce4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5be9dce6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b09dce9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe9dcec, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff49dcf1, SUC_lexicon_tags_33 },
    { 0x14c9dcf2, SUC_lexicon_tags_24 },
    { 0xc7a9dcf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc9dcf6, SUC_lexicon_tags_37 },
    { 0x5269dcf6, SUC_lexicon_tags_1 },
    { 0xc3e9dcf7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2f89dcfa, SUC_lexicon_tags_63 },
    { 0xf7c9dcfb, SUC_lexicon_tags_47 },
    { 0x67a9dcfb, SUC_lexicon_tags_996 },
    { 0, NULL },
    { 0xbc09dcfe, SUC_lexicon_tags_3 },
    { 0x25e9dcfe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e9dd04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a9dd08, SUC_lexicon_tags_21 },
    { 0x73e9dd09, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc969dd10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3189dd13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b29dd15, SUC_lexicon_tags_7 },
    { 0x90c9dd15, SUC_lexicon_tags_27 },
    { 0x93e9dd16, SUC_lexicon_tags_17 },
    { 0x18c9dd15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac89dd1d, SUC_lexicon_tags_37 },
    { 0xece9dd1e, SUC_lexicon_tags_275 },
    { 0xbac9dd1f, SUC_lexicon_tags_12 },
    { 0x6149dd1f, SUC_lexicon_tags_15 },
    { 0xca09dd20, SUC_lexicon_tags_9 },
    { 0x3aa9dd22, SUC_lexicon_tags_10 },
    { 0x84a9dd22, SUC_lexicon_tags_15 },
    { 0x9b69dd24, SUC_lexicon_tags_31 },
    { 0xea9dd24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a9dd2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae9dd30, SUC_lexicon_tags_11 },
    { 0x1689dd31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49dd33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a9dd38, SUC_lexicon_tags_27 },
    { 0x8629dd39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc289dd3d, SUC_lexicon_tags_16 },
    { 0xa2c9dd3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb269dd41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf29dd43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa709dd45, SUC_lexicon_tags_17 },
    { 0xc889dd46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37e9dd48, SUC_lexicon_tags_15 },
    { 0x95e9dd49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4429dd4d, SUC_lexicon_tags_32 },
    { 0xa9e9dd4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8589dd51, SUC_lexicon_tags_31 },
    { 0x8189dd52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8409dd56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a9dd59, SUC_lexicon_tags_30 },
    { 0xb689dd5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c29dd5e, SUC_lexicon_tags_0 },
    { 0xa709dd5e, SUC_lexicon_tags_17 },
    { 0xe9a9dd5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa1a9dd62, SUC_lexicon_tags_25 },
    { 0xea69dd63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c9dd6c, SUC_lexicon_tags_14 },
    { 0xc589dd6c, SUC_lexicon_tags_84 },
    { 0x5e09dd6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa009dd72, SUC_lexicon_tags_1 },
    { 0x2649dd72, SUC_lexicon_tags_13 },
    { 0xca69dd74, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1389dd76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9969dd79, SUC_lexicon_tags_2 },
    { 0x71c9dd7a, SUC_lexicon_tags_15 },
    { 0xd129dd79, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x7129dd7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9509dd83, SUC_lexicon_tags_1 },
    { 0xac29dd84, SUC_lexicon_tags_17 },
    { 0x7a69dd84, SUC_lexicon_tags_11 },
    { 0x9229dd86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe9dd88, SUC_lexicon_tags_0 },
    { 0x9349dd89, SUC_lexicon_tags_15 },
    { 0x1309dd8a, SUC_lexicon_tags_17 },
    { 0x1969dd88, SUC_lexicon_tags_7 },
    { 0x1e29dd8c, SUC_lexicon_tags_26 },
    { 0x2ca9dd8d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7609dd93, SUC_lexicon_tags_11 },
    { 0x6949dd94, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e69dd96, SUC_lexicon_tags_7 },
    { 0x6ec9dd97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79e9dd9b, SUC_lexicon_tags_2 },
    { 0x49e9dd9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a69dd9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd09dda1, SUC_lexicon_tags_9 },
    { 0xc889dda1, SUC_lexicon_tags_1 },
    { 0x2b89dda3, SUC_lexicon_tags_7 },
    { 0x4b09dda4, SUC_lexicon_tags_62 },
    { 0x1449dda5, SUC_lexicon_tags_1 },
    { 0xeb29dda5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc9ddac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3f89ddae, SUC_lexicon_tags_12 },
    { 0x949ddae, SUC_lexicon_tags_2 },
    { 0x54a9ddb0, SUC_lexicon_tags_2 },
    { 0xa309ddb1, SUC_lexicon_tags_12 },
    { 0x19c9ddb2, SUC_lexicon_tags_26 },
    { 0x9c9ddb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac89ddb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc49ddbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb09ddbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4469ddc0, SUC_lexicon_tags_103 },
    { 0xbea9ddc1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5469ddc6, SUC_lexicon_tags_33 },
    { 0x7bc9ddc6, SUC_lexicon_tags_17 },
    { 0xe1c9ddc6, SUC_lexicon_tags_18 },
    { 0x6de9ddc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d29ddcc, SUC_lexicon_tags_19 },
    { 0xcb09ddcd, SUC_lexicon_tags_37 },
    { 0x2f09ddce, SUC_lexicon_tags_38 },
    { 0xe6e9ddcd, SUC_lexicon_tags_47 },
    { 0x1069ddd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3689ddda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae9dde3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe629dde7, SUC_lexicon_tags_27 },
    { 0x83a9dde8, SUC_lexicon_tags_32 },
    { 0xb3c9dde8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee69ddf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2089ddf9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3009de00, SUC_lexicon_tags_11 },
    { 0x55e9de01, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe309de08, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c9de16, SUC_lexicon_tags_0 },
    { 0x48e9de16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69de1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x53c9de1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a29de31, SUC_lexicon_tags_49 },
    { 0xfd49de32, SUC_lexicon_tags_77 },
    { 0xb049de33, SUC_lexicon_tags_49 },
    { 0x49c9de32, SUC_lexicon_tags_11 },
    { 0xd0c9de35, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2dc9de37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f89de3a, SUC_lexicon_tags_14 },
    { 0xd2e9de3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf149de43, SUC_lexicon_tags_23 },
    { 0xcf29de43, SUC_lexicon_tags_47 },
    { 0x9149de45, SUC_lexicon_tags_5 },
    { 0x22a9de45, SUC_lexicon_tags_61 },
    { 0xc749de44, SUC_lexicon_tags_87 },
    { 0xeb89de45, SUC_lexicon_tags_18 },
    { 0x1de9de49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8129de4e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf29de56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd769de5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2529de5d, SUC_lexicon_tags_15 },
    { 0x67c9de5e, SUC_lexicon_tags_17 },
    { 0x7c49de5e, SUC_lexicon_tags_25 },
    { 0xde9de5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc589de63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1269de65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce9de6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c49de6f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d69de74, SUC_lexicon_tags_35 },
    { 0xb3e9de75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc5a9de77, SUC_lexicon_tags_25 },
    { 0x6cc9de78, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x62a9de7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa089de7c, SUC_lexicon_tags_0 },
    { 0xbc09de7d, SUC_lexicon_tags_11 },
    { 0x26c9de7d, SUC_lexicon_tags_2 },
    { 0x9c69de7f, SUC_lexicon_tags_13 },
    { 0x11e9de7f, SUC_lexicon_tags_17 },
    { 0x4329de7c, SUC_lexicon_tags_37 },
    { 0xfe29de82, SUC_lexicon_tags_31 },
    { 0x7309de83, SUC_lexicon_tags_1 },
    { 0xd089de84, SUC_lexicon_tags_25 },
    { 0x2249de85, SUC_lexicon_tags_47 },
    { 0xeb49de86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50a9de88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4c9de8a, SUC_lexicon_tags_17 },
    { 0xf1a9de8a, SUC_lexicon_tags_43 },
    { 0x1f29de8c, SUC_lexicon_tags_17 },
    { 0x72e9de8d, SUC_lexicon_tags_25 },
    { 0x83e9de8e, SUC_lexicon_tags_7 },
    { 0xb4a9de8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb729de92, SUC_lexicon_tags_12 },
    { 0x569de93, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b49de9a, SUC_lexicon_tags_7 },
    { 0x60a9de9b, SUC_lexicon_tags_38 },
    { 0x39a9de9c, SUC_lexicon_tags_37 },
    { 0xcfe9de9d, SUC_lexicon_tags_7 },
    { 0x5269de9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fe9dea0, SUC_lexicon_tags_25 },
    { 0xcfa9dea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb469dea3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69dea6, SUC_lexicon_tags_31 },
    { 0xdf49dea7, SUC_lexicon_tags_12 },
    { 0x4869dea8, SUC_lexicon_tags_65 },
    { 0x30e9dea6, SUC_lexicon_tags_7 },
    { 0x13c9deaa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2429deb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd649deb9, SUC_lexicon_tags_2 },
    { 0xd7c9deba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae89debc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa749debe, SUC_lexicon_tags_70 },
    { 0x8529debf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e9dec4, SUC_lexicon_tags_28 },
    { 0xeba9dec4, SUC_lexicon_tags_32 },
    { 0xabc9dec4, SUC_lexicon_tags_40 },
    { 0x9249dec5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4189dec9, SUC_lexicon_tags_1 },
    { 0xf469deca, SUC_lexicon_tags_61 },
    { 0x7ae9decb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f89dece, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6889ded1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e9ded5, SUC_lexicon_tags_7 },
    { 0x8fc9ded6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5a9ded8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da9dedb, SUC_lexicon_tags_15 },
    { 0x9b69dedc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcd49dede, SUC_lexicon_tags_2 },
    { 0xf1e9dedf, SUC_lexicon_tags_11 },
    { 0xee69dedf, SUC_lexicon_tags_20 },
    { 0xf329dee1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1209dee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc789deee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd829def5, SUC_lexicon_tags_17 },
    { 0xfd29def6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb189defe, SUC_lexicon_tags_26 },
    { 0xe369defe, SUC_lexicon_tags_23 },
    { 0xe7c9deff, SUC_lexicon_tags_11 },
    { 0xa9e9df01, SUC_lexicon_tags_12 },
    { 0x5c29df02, SUC_lexicon_tags_15 },
    { 0x51e9df03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1589df08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7989df0f, SUC_lexicon_tags_12 },
    { 0xf649df0f, SUC_lexicon_tags_6 },
    { 0xc8a9df0f, SUC_lexicon_tags_16 },
    { 0xc2a9df0f, SUC_lexicon_tags_49 },
    { 0x3529df13, SUC_lexicon_tags_1 },
    { 0x6469df12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2769df17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa9df1a, SUC_lexicon_tags_55 },
    { 0xa69df1b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9049df20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc449df26, SUC_lexicon_tags_15 },
    { 0x88c9df27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc849df2c, SUC_lexicon_tags_25 },
    { 0xb669df2c, SUC_lexicon_tags_108 },
    { 0x81e9df2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c89df33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe569df35, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb429df3b, SUC_lexicon_tags_15 },
    { 0x3e29df3b, SUC_lexicon_tags_17 },
    { 0x3d89df3d, SUC_lexicon_tags_25 },
    { 0xb1a9df3e, SUC_lexicon_tags_47 },
    { 0x2909df3f, SUC_lexicon_tags_17 },
    { 0x5969df3f, SUC_lexicon_tags_15 },
    { 0xa09df3f, SUC_lexicon_tags_0 },
    { 0x1c69df42, SUC_lexicon_tags_32 },
    { 0xb009df43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e9df47, SUC_lexicon_tags_39 },
    { 0x25a9df48, SUC_lexicon_tags_2 },
    { 0xcb49df49, SUC_lexicon_tags_1 },
    { 0x6249df49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc9df4f, SUC_lexicon_tags_7 },
    { 0x4649df50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba89df54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe909df56, SUC_lexicon_tags_14 },
    { 0x7069df57, SUC_lexicon_tags_12 },
    { 0xd429df57, SUC_lexicon_tags_35 },
    { 0xe9a9df56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda9df5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c9df5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb7c9df61, SUC_lexicon_tags_28 },
    { 0x36a9df62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafc9df64, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a9df67, SUC_lexicon_tags_14 },
    { 0xa7a9df68, SUC_lexicon_tags_2 },
    { 0xa949df68, SUC_lexicon_tags_33 },
    { 0xb629df6a, SUC_lexicon_tags_38 },
    { 0xf1a9df69, SUC_lexicon_tags_65 },
    { 0x54e9df6c, SUC_lexicon_tags_1 },
    { 0xe009df6b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe709df6f, SUC_lexicon_tags_31 },
    { 0x9449df6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe169df72, SUC_lexicon_tags_37 },
    { 0xbb29df73, SUC_lexicon_tags_29 },
    { 0xc909df73, SUC_lexicon_tags_15 },
    { 0x2929df75, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x20e9df77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26e9df79, SUC_lexicon_tags_60 },
    { 0x3aa9df7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeec9df7c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5049df7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8709df83, SUC_lexicon_tags_1 },
    { 0x7969df83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa9df87, SUC_lexicon_tags_1 },
    { 0x9809df87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x909df8a, SUC_lexicon_tags_23 },
    { 0x5929df8b, SUC_lexicon_tags_2 },
    { 0x4269df8c, SUC_lexicon_tags_27 },
    { 0x2f29df8b, SUC_lexicon_tags_15 },
    { 0xd809df8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6089df91, SUC_lexicon_tags_17 },
    { 0x49a9df92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5909df95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89df98, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4909df9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x11a9df9c, SUC_lexicon_tags_47 },
    { 0x6a9df9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8e9df9f, SUC_lexicon_tags_37 },
    { 0x9ce9df9f, SUC_lexicon_tags_57 },
    { 0x6c9dfa1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xcee9dfa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a9dfaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa869dfad, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d69dfb0, SUC_lexicon_tags_55 },
    { 0xdc69dfb1, SUC_lexicon_tags_77 },
    { 0x1ea9dfb1, SUC_lexicon_tags_15 },
    { 0x59c9dfb1, SUC_lexicon_tags_4 },
    { 0x8cc9dfb1, SUC_lexicon_tags_49 },
    { 0x8629dfb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac9dfbb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x91a9dfbd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1749dfbf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c9dfc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e9dfc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c9dfcd, SUC_lexicon_tags_11 },
    { 0x8fc9dfce, SUC_lexicon_tags_8 },
    { 0x1649dfcd, SUC_lexicon_tags_60 },
    { 0xe149dfd0, SUC_lexicon_tags_55 },
    { 0xb489dfd0, SUC_lexicon_tags_11 },
    { 0xd669dfd2, SUC_lexicon_tags_27 },
    { 0x94e9dfd2, SUC_lexicon_tags_639 },
    { 0x3829dfd4, SUC_lexicon_tags_14 },
    { 0xa69dfcf, SUC_lexicon_tags_15 },
    { 0xdfa9dfd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c69dfdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a9dfe1, SUC_lexicon_tags_33 },
    { 0x6e09dfe2, SUC_lexicon_tags_47 },
    { 0xe489dfe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4729dfe5, SUC_lexicon_tags_12 },
    { 0x6029dfe6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c69dfe8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x8569dfea, SUC_lexicon_tags_37 },
    { 0x8c49dfeb, SUC_lexicon_tags_9 },
    { 0x81c9dfeb, SUC_lexicon_tags_29 },
    { 0x38c9dfec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69dff6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe29dffc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac09e006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a9e009, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba49e00b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ae9e00d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81c9e00f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe09e011, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09e015, SUC_lexicon_tags_17 },
    { 0x4fc9e015, SUC_lexicon_tags_27 },
    { 0x8489e015, SUC_lexicon_tags_33 },
    { 0xf7a9e017, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1529e01a, SUC_lexicon_tags_2 },
    { 0xe2c9e01b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5709e01d, SUC_lexicon_tags_11 },
    { 0xd589e01d, SUC_lexicon_tags_39 },
    { 0x3129e01f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc109e022, SUC_lexicon_tags_63 },
    { 0x86c9e022, SUC_lexicon_tags_37 },
    { 0x1e89e024, SUC_lexicon_tags_13 },
    { 0x6f89e023, SUC_lexicon_tags_19 },
    { 0x5289e026, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xf429e028, SUC_lexicon_tags_12 },
    { 0x3069e028, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb289e02e, SUC_lexicon_tags_1 },
    { 0x2a49e02e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xee9e031, SUC_lexicon_tags_171 },
    { 0x4c69e032, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7189e036, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5989e03a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e9e03d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafc9e03f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc269e041, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f09e046, SUC_lexicon_tags_47 },
    { 0x82c9e046, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae49e04b, SUC_lexicon_tags_37 },
    { 0xe669e04c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a49e04e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcac9e050, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b09e054, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf69e057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7209e05f, SUC_lexicon_tags_1 },
    { 0x269e060, SUC_lexicon_tags_7 },
    { 0x8109e061, SUC_lexicon_tags_1 },
    { 0x74c9e060, SUC_lexicon_tags_70 },
    { 0xbe29e062, SUC_lexicon_tags_15 },
    { 0xffa9e064, SUC_lexicon_tags_16 },
    { 0xade9e062, SUC_lexicon_tags_47 },
    { 0x4d29e062, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe909e06a, SUC_lexicon_tags_1 },
    { 0x3049e06b, SUC_lexicon_tags_1 },
    { 0xdc69e06c, SUC_lexicon_tags_230 },
    { 0xfde9e06d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1469e070, SUC_lexicon_tags_11 },
    { 0x76e9e071, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x90a9e073, SUC_lexicon_tags_15 },
    { 0x4f89e074, SUC_lexicon_tags_37 },
    { 0x8209e073, SUC_lexicon_tags_1 },
    { 0xdac9e076, SUC_lexicon_tags_7 },
    { 0x8289e077, SUC_lexicon_tags_43 },
    { 0x2909e077, SUC_lexicon_tags_25 },
    { 0x65e9e079, SUC_lexicon_tags_23 },
    { 0xe209e077, SUC_lexicon_tags_25 },
    { 0xdf69e07b, SUC_lexicon_tags_23 },
    { 0xa749e07c, SUC_lexicon_tags_43 },
    { 0xea49e07c, SUC_lexicon_tags_16 },
    { 0x8de9e07e, SUC_lexicon_tags_1 },
    { 0x23e9e07e, SUC_lexicon_tags_2 },
    { 0x9889e080, SUC_lexicon_tags_7 },
    { 0xd0e9e081, SUC_lexicon_tags_0 },
    { 0x3429e082, SUC_lexicon_tags_15 },
    { 0xd89e083, SUC_lexicon_tags_23 },
    { 0xd3e9e083, SUC_lexicon_tags_17 },
    { 0x5c9e07b, SUC_lexicon_tags_1 },
    { 0xcce9e086, SUC_lexicon_tags_12 },
    { 0xbc49e086, SUC_lexicon_tags_47 },
    { 0x1e89e088, SUC_lexicon_tags_37 },
    { 0x8029e07e, SUC_lexicon_tags_11 },
    { 0xb49e07d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe689e08e, SUC_lexicon_tags_34 },
    { 0xf149e08f, SUC_lexicon_tags_4 },
    { 0xe8a9e08e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e9e096, SUC_lexicon_tags_69 },
    { 0x9d69e097, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9e09a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1249e09f, SUC_lexicon_tags_9 },
    { 0xab09e0a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3889e0a3, SUC_lexicon_tags_29 },
    { 0x7629e0a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcac9e0a7, SUC_lexicon_tags_37 },
    { 0x5b49e0a8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89e0ac, SUC_lexicon_tags_47 },
    { 0x6ca9e0ad, SUC_lexicon_tags_12 },
    { 0xbd29e0ae, SUC_lexicon_tags_23 },
    { 0xb149e0af, SUC_lexicon_tags_201 },
    { 0xf189e0b0, SUC_lexicon_tags_12 },
    { 0x7889e0b1, SUC_lexicon_tags_45 },
    { 0xfba9e0b2, SUC_lexicon_tags_15 },
    { 0x3989e0b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc89e0b5, SUC_lexicon_tags_101 },
    { 0xff69e0b6, SUC_lexicon_tags_12 },
    { 0xbbc9e0b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b09e0c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3889e0c2, SUC_lexicon_tags_9 },
    { 0x3bc9e0c2, SUC_lexicon_tags_16 },
    { 0x8b89e0c4, SUC_lexicon_tags_37 },
    { 0x5429e0c5, SUC_lexicon_tags_31 },
    { 0x5969e0c3, SUC_lexicon_tags_23 },
    { 0x9789e0c6, SUC_lexicon_tags_43 },
    { 0xd7a9e0c6, SUC_lexicon_tags_27 },
    { 0x9b69e0c8, SUC_lexicon_tags_35 },
    { 0x8d09e0ca, SUC_lexicon_tags_11 },
    { 0xf29e0cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a9e0d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d49e0dc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7469e0df, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa8e9e0e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc29e0e7, SUC_lexicon_tags_7 },
    { 0x66e9e0e7, SUC_lexicon_tags_92 },
    { 0xf409e0e9, SUC_lexicon_tags_14 },
    { 0xc949e0e7, SUC_lexicon_tags_6 },
    { 0x7609e0eb, SUC_lexicon_tags_11 },
    { 0x2129e0e7, SUC_lexicon_tags_19 },
    { 0x8ba9e0e7, SUC_lexicon_tags_7 },
    { 0xaf29e0ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ea9e0f0, SUC_lexicon_tags_57 },
    { 0x38c9e0f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a9e0f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b09e0fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a9e101, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4849e104, SUC_lexicon_tags_19 },
    { 0xfea9e104, SUC_lexicon_tags_27 },
    { 0x509e105, SUC_lexicon_tags_37 },
    { 0x9e104, SUC_lexicon_tags_47 },
    { 0x9ee9e104, SUC_lexicon_tags_15 },
    { 0x4e09e106, SUC_lexicon_tags_49 },
    { 0x43a9e10a, SUC_lexicon_tags_116 },
    { 0xdd09e10b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3929e10d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa9e110, SUC_lexicon_tags_916 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe9e113, SUC_lexicon_tags_37 },
    { 0x9409e114, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x289e116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5509e11d, SUC_lexicon_tags_26 },
    { 0x3f29e11d, SUC_lexicon_tags_15 },
    { 0xdb29e11f, SUC_lexicon_tags_37 },
    { 0x3ac9e120, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x47a9e122, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e9e125, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c9e128, SUC_lexicon_tags_17 },
    { 0xb329e129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1e9e12b, SUC_lexicon_tags_1 },
    { 0xeb89e12c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c9e12f, SUC_lexicon_tags_68 },
    { 0x9d89e130, SUC_lexicon_tags_108 },
    { 0xcea9e12f, SUC_lexicon_tags_37 },
    { 0x5889e132, SUC_lexicon_tags_25 },
    { 0x729e133, SUC_lexicon_tags_8 },
    { 0xbb29e134, SUC_lexicon_tags_68 },
    { 0x3009e134, SUC_lexicon_tags_31 },
    { 0xf0e9e134, SUC_lexicon_tags_47 },
    { 0x4549e137, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8109e13d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x9009e140, SUC_lexicon_tags_1 },
    { 0x5749e141, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9369e144, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd369e146, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc809e14c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca9e150, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5309e156, SUC_lexicon_tags_17 },
    { 0x1969e156, SUC_lexicon_tags_8 },
    { 0xcae9e158, SUC_lexicon_tags_37 },
    { 0x3e69e159, SUC_lexicon_tags_15 },
    { 0x66a9e15a, SUC_lexicon_tags_1 },
    { 0xeee9e157, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b49e15d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5989e15f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4e49e161, SUC_lexicon_tags_25 },
    { 0x65c9e162, SUC_lexicon_tags_37 },
    { 0xc329e163, SUC_lexicon_tags_15 },
    { 0x3029e163, SUC_lexicon_tags_33 },
    { 0xcce9e165, SUC_lexicon_tags_60 },
    { 0xae29e161, SUC_lexicon_tags_31 },
    { 0x8c49e167, SUC_lexicon_tags_15 },
    { 0xc569e168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa669e16a, SUC_lexicon_tags_80 },
    { 0x6689e16b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9969e172, SUC_lexicon_tags_15 },
    { 0xc8c9e172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5209e175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce09e177, SUC_lexicon_tags_100 },
    { 0xaf49e177, SUC_lexicon_tags_25 },
    { 0x2109e179, SUC_lexicon_tags_2 },
    { 0x5f89e17a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e17d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae9e182, SUC_lexicon_tags_37 },
    { 0xd889e183, SUC_lexicon_tags_1 },
    { 0xd969e183, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4729e186, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e9e18a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa629e18d, SUC_lexicon_tags_12 },
    { 0x4a49e18d, SUC_lexicon_tags_11 },
    { 0x66c9e18f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1829e192, SUC_lexicon_tags_11 },
    { 0x2da9e193, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c89e198, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e9e19b, SUC_lexicon_tags_12 },
    { 0x39a9e19c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed29e19e, SUC_lexicon_tags_4 },
    { 0x3429e19f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd269e1a2, SUC_lexicon_tags_65 },
    { 0x83c9e1a2, SUC_lexicon_tags_11 },
    { 0xf49e1a4, SUC_lexicon_tags_15 },
    { 0xb889e1a2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x91c9e1a7, SUC_lexicon_tags_21 },
    { 0x6409e1a8, SUC_lexicon_tags_47 },
    { 0xef49e1a9, SUC_lexicon_tags_2 },
    { 0x3cc9e1a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2769e1af, SUC_lexicon_tags_16 },
    { 0xfc29e1b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a9e1b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c09e1b7, SUC_lexicon_tags_43 },
    { 0x6989e1b7, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a9e1bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x78c9e1be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a9e1c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe9e1c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc849e1ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e69e1d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7789e1da, SUC_lexicon_tags_34 },
    { 0xa5e9e1db, SUC_lexicon_tags_37 },
    { 0xb0a9e1da, SUC_lexicon_tags_1 },
    { 0x7549e1dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe349e1df, SUC_lexicon_tags_2 },
    { 0x9d49e1e0, SUC_lexicon_tags_19 },
    { 0x9809e1e1, SUC_lexicon_tags_63 },
    { 0x3469e1e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6249e1f2, SUC_lexicon_tags_65 },
    { 0x2e69e1f3, SUC_lexicon_tags_2 },
    { 0x7cc9e1f4, SUC_lexicon_tags_47 },
    { 0x3029e1f5, SUC_lexicon_tags_12 },
    { 0x7049e1f5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7729e1f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7989e1fb, SUC_lexicon_tags_19 },
    { 0xc529e1fb, SUC_lexicon_tags_46 },
    { 0xa89e1fd, SUC_lexicon_tags_15 },
    { 0x6f89e1fd, SUC_lexicon_tags_0 },
    { 0xd749e1fe, SUC_lexicon_tags_31 },
    { 0x7ba9e200, SUC_lexicon_tags_8 },
    { 0x45c9e201, SUC_lexicon_tags_7 },
    { 0x4f89e1fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe309e204, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6329e209, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa209e20c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba89e216, SUC_lexicon_tags_0 },
    { 0x9149e217, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e9e21a, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc9e21d, SUC_lexicon_tags_37 },
    { 0x72a9e21e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe089e226, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc9e22c, SUC_lexicon_tags_21 },
    { 0xbf69e22d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc449e231, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c9e238, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b89e23a, SUC_lexicon_tags_101 },
    { 0xf109e23b, SUC_lexicon_tags_27 },
    { 0x6889e23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c89e23e, SUC_lexicon_tags_23 },
    { 0x7069e23f, SUC_lexicon_tags_37 },
    { 0xa5e9e240, SUC_lexicon_tags_1 },
    { 0xc409e241, SUC_lexicon_tags_12 },
    { 0x3269e241, SUC_lexicon_tags_2 },
    { 0xf889e243, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7789e249, SUC_lexicon_tags_15 },
    { 0xaa9e24a, SUC_lexicon_tags_15 },
    { 0x1a9e24a, SUC_lexicon_tags_13 },
    { 0x5e89e24c, SUC_lexicon_tags_17 },
    { 0x1a89e24b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d09e251, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a9e254, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde89e256, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de9e25a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a9e25e, SUC_lexicon_tags_0 },
    { 0x5209e25f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7089e268, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a29e26a, SUC_lexicon_tags_15 },
    { 0x5389e26b, SUC_lexicon_tags_38 },
    { 0xcb89e26c, SUC_lexicon_tags_14 },
    { 0x2689e26d, SUC_lexicon_tags_15 },
    { 0x9089e26e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a9e276, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9849e27d, SUC_lexicon_tags_7 },
    { 0x8709e27d, SUC_lexicon_tags_82 },
    { 0xed69e27f, SUC_lexicon_tags_6 },
    { 0x4069e27f, SUC_lexicon_tags_32 },
    { 0x52c9e27d, SUC_lexicon_tags_82 },
    { 0x3e09e282, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa9e28b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c9e291, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12e9e293, SUC_lexicon_tags_25 },
    { 0x19a9e294, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c9e297, SUC_lexicon_tags_47 },
    { 0x5669e297, SUC_lexicon_tags_37 },
    { 0xad49e299, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xed49e29b, SUC_lexicon_tags_17 },
    { 0x8c09e29b, SUC_lexicon_tags_11 },
    { 0xe149e29b, SUC_lexicon_tags_25 },
    { 0xd289e29b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd749e2a1, SUC_lexicon_tags_29 },
    { 0xf29e2a2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c9e2a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8149e2a8, SUC_lexicon_tags_33 },
    { 0xeda9e2a9, SUC_lexicon_tags_14 },
    { 0xd909e2aa, SUC_lexicon_tags_46 },
    { 0xf309e2a9, SUC_lexicon_tags_11 },
    { 0xfda9e2a9, SUC_lexicon_tags_28 },
    { 0xbde9e2ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2149e2af, SUC_lexicon_tags_43 },
    { 0xc5a9e2b0, SUC_lexicon_tags_1024 },
    { 0, NULL },
    { 0, NULL },
    { 0x2769e2b3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269e2b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3749e2bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e9e2c0, SUC_lexicon_tags_37 },
    { 0x48e9e2c0, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xe4a9e2c3, SUC_lexicon_tags_31 },
    { 0xfde9e2c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c9e2c8, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d29e2cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49a9e2ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f29e2d1, SUC_lexicon_tags_0 },
    { 0x87c9e2d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6609e2db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c9e2df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7229e2e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5309e2e5, SUC_lexicon_tags_12 },
    { 0x50c9e2e6, SUC_lexicon_tags_12 },
    { 0x3449e2e7, SUC_lexicon_tags_12 },
    { 0xe729e2e7, SUC_lexicon_tags_87 },
    { 0x7a9e2e9, SUC_lexicon_tags_17 },
    { 0x7409e2ea, SUC_lexicon_tags_17 },
    { 0xd849e2e9, SUC_lexicon_tags_25 },
    { 0x5709e2e9, SUC_lexicon_tags_26 },
    { 0x40a9e2eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c9e2f1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b29e2f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c89e2fe, SUC_lexicon_tags_82 },
    { 0x97e9e2ff, SUC_lexicon_tags_25 },
    { 0xf8e9e300, SUC_lexicon_tags_19 },
    { 0x21e9e300, SUC_lexicon_tags_12 },
    { 0x4a09e302, SUC_lexicon_tags_25 },
    { 0x4449e303, SUC_lexicon_tags_37 },
    { 0xe209e304, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6069e308, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2ca9e30a, SUC_lexicon_tags_51 },
    { 0x9f69e30b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba9e30e, SUC_lexicon_tags_30 },
    { 0x8ae9e30f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5189e317, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3289e31a, SUC_lexicon_tags_1 },
    { 0x6749e31a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c9e31e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72c9e320, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c9e324, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb009e32d, SUC_lexicon_tags_16 },
    { 0xe289e32d, SUC_lexicon_tags_17 },
    { 0xc289e32e, SUC_lexicon_tags_0 },
    { 0x60a9e32f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe049e333, SUC_lexicon_tags_37 },
    { 0xbec9e333, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a9e337, SUC_lexicon_tags_31 },
    { 0x9309e338, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc489e342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x389e344, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd769e346, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa149e348, SUC_lexicon_tags_55 },
    { 0xc69e348, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd2a9e34b, SUC_lexicon_tags_39 },
    { 0x9d89e34c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c69e359, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a49e35b, SUC_lexicon_tags_14 },
    { 0xc9e35c, SUC_lexicon_tags_37 },
    { 0xb69e35b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc29e361, SUC_lexicon_tags_27 },
    { 0x2669e362, SUC_lexicon_tags_37 },
    { 0x55e9e362, SUC_lexicon_tags_25 },
    { 0xeba9e361, SUC_lexicon_tags_11 },
    { 0x9f89e365, SUC_lexicon_tags_17 },
    { 0x4ea9e366, SUC_lexicon_tags_6 },
    { 0xfda9e366, SUC_lexicon_tags_3 },
    { 0xef69e367, SUC_lexicon_tags_65 },
    { 0xdf29e369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5369e36b, SUC_lexicon_tags_39 },
    { 0xdf29e36b, SUC_lexicon_tags_201 },
    { 0xd2c9e36c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a9e371, SUC_lexicon_tags_19 },
    { 0x9ae9e371, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1c69e374, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb009e37b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb049e380, SUC_lexicon_tags_11 },
    { 0x5e89e380, SUC_lexicon_tags_49 },
    { 0xdd09e382, SUC_lexicon_tags_27 },
    { 0xaee9e382, SUC_lexicon_tags_33 },
    { 0x19c9e384, SUC_lexicon_tags_17 },
    { 0x8649e384, SUC_lexicon_tags_30 },
    { 0x50a9e386, SUC_lexicon_tags_37 },
    { 0x71c9e387, SUC_lexicon_tags_27 },
    { 0x449e380, SUC_lexicon_tags_14 },
    { 0xc0e9e389, SUC_lexicon_tags_12 },
    { 0xaaa9e389, SUC_lexicon_tags_11 },
    { 0x4b49e38b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9e38f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6889e391, SUC_lexicon_tags_15 },
    { 0x3b69e391, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa609e395, SUC_lexicon_tags_3 },
    { 0x6769e396, SUC_lexicon_tags_27 },
    { 0xaac9e396, SUC_lexicon_tags_11 },
    { 0xb609e398, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c9e39b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x61a9e39d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6069e3a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc669e3a3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a9e3a8, SUC_lexicon_tags_1 },
    { 0xb169e3a9, SUC_lexicon_tags_30 },
    { 0x9449e3aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29a9e3ad, SUC_lexicon_tags_37 },
    { 0xc7e9e3ae, SUC_lexicon_tags_13 },
    { 0x5b29e3af, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a9e3b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee9e3be, SUC_lexicon_tags_37 },
    { 0x3c09e3bf, SUC_lexicon_tags_7 },
    { 0xfe9e3c0, SUC_lexicon_tags_27 },
    { 0x37e9e3c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d29e3c3, SUC_lexicon_tags_30 },
    { 0x4749e3c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc9e9e3c6, SUC_lexicon_tags_23 },
    { 0xebc9e3c6, SUC_lexicon_tags_30 },
    { 0x76c9e3c8, SUC_lexicon_tags_7 },
    { 0x5309e3c7, SUC_lexicon_tags_7 },
    { 0x7849e3ca, SUC_lexicon_tags_7 },
    { 0x6b49e3cb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9749e3ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc9e3d2, SUC_lexicon_tags_37 },
    { 0xe909e3d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa689e3d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe689e3d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4709e3db, SUC_lexicon_tags_27 },
    { 0x6809e3dc, SUC_lexicon_tags_16 },
    { 0xa4c9e3dd, SUC_lexicon_tags_2 },
    { 0x5b69e3dc, SUC_lexicon_tags_25 },
    { 0xa5e9e3de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xabc9e3e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41c9e3e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed69e3e8, SUC_lexicon_tags_13 },
    { 0x84a9e3e8, SUC_lexicon_tags_60 },
    { 0xc529e3ea, SUC_lexicon_tags_8 },
    { 0x2be9e3eb, SUC_lexicon_tags_47 },
    { 0x19c9e3ec, SUC_lexicon_tags_62 },
    { 0x4c29e3ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc89e3f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9609e3fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d69e400, SUC_lexicon_tags_17 },
    { 0xd229e400, SUC_lexicon_tags_0 },
    { 0x1229e402, SUC_lexicon_tags_37 },
    { 0x9629e402, SUC_lexicon_tags_46 },
    { 0x28e9e404, SUC_lexicon_tags_413 },
    { 0xd829e401, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3089e408, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc189e40c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x24a9e40e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c29e410, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e9e414, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb549e416, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da9e422, SUC_lexicon_tags_52 },
    { 0x28c9e423, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc9e426, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85a9e42a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9769e42f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4869e433, SUC_lexicon_tags_21 },
    { 0x1389e433, SUC_lexicon_tags_63 },
    { 0x9ea9e434, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9649e437, SUC_lexicon_tags_2 },
    { 0x9389e438, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1209e43a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49e43e, SUC_lexicon_tags_12 },
    { 0x3da9e43f, SUC_lexicon_tags_7 },
    { 0x8709e43e, SUC_lexicon_tags_10 },
    { 0x5349e43f, SUC_lexicon_tags_30 },
    { 0xf889e440, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc9e445, SUC_lexicon_tags_33 },
    { 0x6269e446, SUC_lexicon_tags_9 },
    { 0x1489e447, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5249e450, SUC_lexicon_tags_37 },
    { 0x6b49e451, SUC_lexicon_tags_37 },
    { 0xf3e9e450, SUC_lexicon_tags_29 },
    { 0x6b89e453, SUC_lexicon_tags_7 },
    { 0x3f69e453, SUC_lexicon_tags_10 },
    { 0xb8a9e453, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c9e45a, SUC_lexicon_tags_19 },
    { 0x1309e45b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea9e45f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c9e462, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x429e465, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e9e468, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92e9e46e, SUC_lexicon_tags_9 },
    { 0x7609e46f, SUC_lexicon_tags_12 },
    { 0xaea9e470, SUC_lexicon_tags_1397 },
    { 0, NULL },
    { 0x47c9e472, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbe29e474, SUC_lexicon_tags_12 },
    { 0x70a9e475, SUC_lexicon_tags_17 },
    { 0xa29e476, SUC_lexicon_tags_103 },
    { 0x2089e477, SUC_lexicon_tags_45 },
    { 0x6429e478, SUC_lexicon_tags_2 },
    { 0x3669e478, SUC_lexicon_tags_25 },
    { 0xa8c9e478, SUC_lexicon_tags_45 },
    { 0x54a9e479, SUC_lexicon_tags_38 },
    { 0x1589e47a, SUC_lexicon_tags_25 },
    { 0x4629e47d, SUC_lexicon_tags_37 },
    { 0x4649e479, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a9e482, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd189e487, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a9e48c, SUC_lexicon_tags_8 },
    { 0xaa89e48d, SUC_lexicon_tags_7 },
    { 0x5589e48d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x169e490, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb669e492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7e9e494, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7009e498, SUC_lexicon_tags_11 },
    { 0x45c9e499, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52e9e49f, SUC_lexicon_tags_2 },
    { 0x7649e49f, SUC_lexicon_tags_30 },
    { 0x4049e4a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb009e4a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb269e4a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc9e4ab, SUC_lexicon_tags_45 },
    { 0xc729e4ab, SUC_lexicon_tags_1 },
    { 0x66a9e4ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4209e4af, SUC_lexicon_tags_46 },
    { 0x93a9e4af, SUC_lexicon_tags_1 },
    { 0xabe9e4b1, SUC_lexicon_tags_39 },
    { 0x229e4b1, SUC_lexicon_tags_17 },
    { 0x9409e4b2, SUC_lexicon_tags_7 },
    { 0x4969e4b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a9e4b7, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x1329e4b9, SUC_lexicon_tags_37 },
    { 0xd49e4b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3389e4be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb609e4c0, SUC_lexicon_tags_230 },
    { 0x8009e4c1, SUC_lexicon_tags_2 },
    { 0x8189e4c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869e4ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e49e4ce, SUC_lexicon_tags_37 },
    { 0x6ae9e4ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcc89e4d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4049e4d3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba9e4d6, SUC_lexicon_tags_25 },
    { 0x9e9e4d7, SUC_lexicon_tags_21 },
    { 0xaf29e4d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7589e4df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x22e9e4e2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3d89e4e4, SUC_lexicon_tags_38 },
    { 0xf469e4e5, SUC_lexicon_tags_23 },
    { 0x76e9e4e6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf69e4e9, SUC_lexicon_tags_0 },
    { 0x2309e4ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa09e4ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e9e4f1, SUC_lexicon_tags_12 },
    { 0x4da9e4f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6549e4f5, SUC_lexicon_tags_21 },
    { 0x3c29e4f6, SUC_lexicon_tags_39 },
    { 0x8e69e4f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d89e4f9, SUC_lexicon_tags_15 },
    { 0x95c9e4f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf09e4fe, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf329e500, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb069e507, SUC_lexicon_tags_9 },
    { 0x649e508, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x28c9e50a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa6c9e50c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee9e510, SUC_lexicon_tags_7 },
    { 0x489e510, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf469e513, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e9e518, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x49a9e51a, SUC_lexicon_tags_17 },
    { 0x8869e51b, SUC_lexicon_tags_1 },
    { 0x2f69e51c, SUC_lexicon_tags_31 },
    { 0xd389e51a, SUC_lexicon_tags_15 },
    { 0x4a49e51e, SUC_lexicon_tags_17 },
    { 0x8de9e51e, SUC_lexicon_tags_27 },
    { 0x2049e51f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7129e523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4389e525, SUC_lexicon_tags_15 },
    { 0xf369e525, SUC_lexicon_tags_2 },
    { 0x4bc9e527, SUC_lexicon_tags_34 },
    { 0x6529e525, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a09e52a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e9e532, SUC_lexicon_tags_87 },
    { 0x1449e533, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29e536, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa1a9e538, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc209e53a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e9e540, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7c89e542, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa9e545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce9e54a, SUC_lexicon_tags_11 },
    { 0xc3a9e54a, SUC_lexicon_tags_13 },
    { 0xf3a9e54c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c9e551, SUC_lexicon_tags_1 },
    { 0x8fa9e551, SUC_lexicon_tags_11 },
    { 0x49e9e551, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfac9e555, SUC_lexicon_tags_43 },
    { 0x77e9e556, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc129e55c, SUC_lexicon_tags_11 },
    { 0x9849e55c, SUC_lexicon_tags_26 },
    { 0x5989e55e, SUC_lexicon_tags_4 },
    { 0xe69e55f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6289e563, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb649e565, SUC_lexicon_tags_34 },
    { 0xe649e566, SUC_lexicon_tags_17 },
    { 0x9fe9e566, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2209e56b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4489e56e, SUC_lexicon_tags_15 },
    { 0x6769e56e, SUC_lexicon_tags_27 },
    { 0xde69e570, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2bc9e572, SUC_lexicon_tags_15 },
    { 0x38c9e573, SUC_lexicon_tags_27 },
    { 0x7889e572, SUC_lexicon_tags_21 },
    { 0xe009e575, SUC_lexicon_tags_29 },
    { 0x3d89e576, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58a9e578, SUC_lexicon_tags_20 },
    { 0x1549e578, SUC_lexicon_tags_1 },
    { 0x6f49e578, SUC_lexicon_tags_11 },
    { 0x2089e579, SUC_lexicon_tags_0 },
    { 0x3829e57c, SUC_lexicon_tags_43 },
    { 0xc2e9e57d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd649e57f, SUC_lexicon_tags_16 },
    { 0xcdc9e580, SUC_lexicon_tags_15 },
    { 0xee09e581, SUC_lexicon_tags_2 },
    { 0xe4c9e582, SUC_lexicon_tags_29 },
    { 0x22c9e583, SUC_lexicon_tags_12 },
    { 0x27e9e581, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd869e586, SUC_lexicon_tags_65 },
    { 0x5e69e586, SUC_lexicon_tags_1 },
    { 0x9f89e587, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1089e58a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6d29e58c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1589e592, SUC_lexicon_tags_94 },
    { 0x1ec9e593, SUC_lexicon_tags_43 },
    { 0x45e9e592, SUC_lexicon_tags_31 },
    { 0x2c89e593, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28e9e597, SUC_lexicon_tags_15 },
    { 0x1369e598, SUC_lexicon_tags_17 },
    { 0xf669e598, SUC_lexicon_tags_138 },
    { 0x24c9e59a, SUC_lexicon_tags_7 },
    { 0x5f69e59b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa289e59f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3429e5a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ac9e5a9, SUC_lexicon_tags_2 },
    { 0xb189e5aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5a49e5ac, SUC_lexicon_tags_37 },
    { 0x55c9e5ac, SUC_lexicon_tags_21 },
    { 0x9009e5ae, SUC_lexicon_tags_22 },
    { 0xd6e9e5ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8929e5b4, SUC_lexicon_tags_43 },
    { 0x5269e5b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e29e5b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d69e5ba, SUC_lexicon_tags_21 },
    { 0xc6a9e5bb, SUC_lexicon_tags_33 },
    { 0x8869e5ba, SUC_lexicon_tags_29 },
    { 0xd6c9e5bd, SUC_lexicon_tags_167 },
    { 0x409e5ba, SUC_lexicon_tags_61 },
    { 0x5be9e5bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9869e5c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb29e5c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a9e5cc, SUC_lexicon_tags_12 },
    { 0x5469e5cd, SUC_lexicon_tags_16 },
    { 0xec9e5cc, SUC_lexicon_tags_8 },
    { 0xc9a9e5cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa349e5db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3869e5e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc529e5e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7749e5f0, SUC_lexicon_tags_66 },
    { 0x2309e5f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd929e5f7, SUC_lexicon_tags_87 },
    { 0xd2e9e5f8, SUC_lexicon_tags_27 },
    { 0x61c9e5f9, SUC_lexicon_tags_7 },
    { 0x58e9e5fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5069e605, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d89e609, SUC_lexicon_tags_25 },
    { 0xe7c9e60a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be9e60d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e9e611, SUC_lexicon_tags_1 },
    { 0xdc89e612, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x1949e614, SUC_lexicon_tags_11 },
    { 0x6ae9e615, SUC_lexicon_tags_25 },
    { 0x1869e614, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc829e618, SUC_lexicon_tags_39 },
    { 0xab69e619, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd49e61c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d09e61f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5c9e621, SUC_lexicon_tags_32 },
    { 0xcac9e621, SUC_lexicon_tags_10 },
    { 0x1e69e623, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e9e62d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc029e632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace9e63b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c9e640, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b49e642, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c9e645, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b29e649, SUC_lexicon_tags_14 },
    { 0x9409e64a, SUC_lexicon_tags_6 },
    { 0xffe9e64a, SUC_lexicon_tags_28 },
    { 0xcfa9e649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a9e64f, SUC_lexicon_tags_8 },
    { 0xc4e9e650, SUC_lexicon_tags_25 },
    { 0x7689e651, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70e9e653, SUC_lexicon_tags_12 },
    { 0xd6a9e654, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e9e658, SUC_lexicon_tags_11 },
    { 0x4da9e659, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x99e9e65b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be9e65e, SUC_lexicon_tags_23 },
    { 0xa889e65f, SUC_lexicon_tags_8 },
    { 0xffe9e65f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaa89e662, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e9e667, SUC_lexicon_tags_83 },
    { 0x3ca9e668, SUC_lexicon_tags_26 },
    { 0x48e9e669, SUC_lexicon_tags_25 },
    { 0x5e09e669, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe269e66d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd229e671, SUC_lexicon_tags_11 },
    { 0x289e672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebe9e674, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8289e677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc289e67a, SUC_lexicon_tags_65 },
    { 0xbc29e67a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb909e67d, SUC_lexicon_tags_2 },
    { 0xfac9e67e, SUC_lexicon_tags_25 },
    { 0xd769e67f, SUC_lexicon_tags_27 },
    { 0xdac9e680, SUC_lexicon_tags_21 },
    { 0xc8c9e67d, SUC_lexicon_tags_23 },
    { 0x9dc9e682, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2c9e684, SUC_lexicon_tags_11 },
    { 0xdea9e684, SUC_lexicon_tags_21 },
    { 0xfae9e686, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1229e68a, SUC_lexicon_tags_25 },
    { 0xcdc9e68b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ec9e68e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe069e690, SUC_lexicon_tags_47 },
    { 0x47a9e691, SUC_lexicon_tags_25 },
    { 0x8ec9e692, SUC_lexicon_tags_63 },
    { 0x54e9e693, SUC_lexicon_tags_1 },
    { 0xb4a9e693, SUC_lexicon_tags_45 },
    { 0x8009e690, SUC_lexicon_tags_37 },
    { 0x22e9e696, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd589e699, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9869e69d, SUC_lexicon_tags_14 },
    { 0x1ca9e69e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x329e6a0, SUC_lexicon_tags_38 },
    { 0x1b09e6a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f09e6a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe529e6a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa69e6a8, SUC_lexicon_tags_28 },
    { 0x4529e6a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7e9e6ab, SUC_lexicon_tags_15 },
    { 0x7a9e6ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf89e6b0, SUC_lexicon_tags_56 },
    { 0x28a9e6b0, SUC_lexicon_tags_103 },
    { 0x89e9e6b2, SUC_lexicon_tags_1 },
    { 0x3f89e6b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14e9e6b7, SUC_lexicon_tags_15 },
    { 0x32e9e6b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb689e6c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac9e6c6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe9e6c9, SUC_lexicon_tags_43 },
    { 0xda49e6c9, SUC_lexicon_tags_25 },
    { 0x4369e6ca, SUC_lexicon_tags_28 },
    { 0x85a9e6c9, SUC_lexicon_tags_28 },
    { 0xa849e6cd, SUC_lexicon_tags_0 },
    { 0x2269e6cd, SUC_lexicon_tags_15 },
    { 0xbd89e6cb, SUC_lexicon_tags_12 },
    { 0x5f49e6cb, SUC_lexicon_tags_12 },
    { 0x7669e6d1, SUC_lexicon_tags_17 },
    { 0x62e9e6cf, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x44c9e6d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4609e6d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce9e6dd, SUC_lexicon_tags_7 },
    { 0xc2a9e6de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6da9e6e0, SUC_lexicon_tags_11 },
    { 0x3589e6e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2389e6ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc169e6ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7c9e6ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc589e6f2, SUC_lexicon_tags_10 },
    { 0x56e9e6f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff49e6f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9949e6fa, SUC_lexicon_tags_47 },
    { 0x3569e6fb, SUC_lexicon_tags_3 },
    { 0x5d89e6fc, SUC_lexicon_tags_25 },
    { 0x9909e6fb, SUC_lexicon_tags_25 },
    { 0x9c69e6fe, SUC_lexicon_tags_1 },
    { 0xd0a9e6fe, SUC_lexicon_tags_31 },
    { 0x1029e700, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3d29e702, SUC_lexicon_tags_2 },
    { 0x8389e703, SUC_lexicon_tags_35 },
    { 0x4c09e702, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdbc9e706, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2829e70a, SUC_lexicon_tags_25 },
    { 0xb0c9e70a, SUC_lexicon_tags_23 },
    { 0x7d89e70a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb809e71f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb229e724, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc9e727, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf29e72a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9829e72c, SUC_lexicon_tags_39 },
    { 0xc469e72d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa9e730, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e09e738, SUC_lexicon_tags_2 },
    { 0x6389e739, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xece9e73b, SUC_lexicon_tags_33 },
    { 0x4fc9e73c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcae9e73e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f69e740, SUC_lexicon_tags_2 },
    { 0x6a49e741, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82e9e743, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6529e745, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1809e747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8209e74a, SUC_lexicon_tags_59 },
    { 0xdfa9e74b, SUC_lexicon_tags_2 },
    { 0x8249e74a, SUC_lexicon_tags_21 },
    { 0xe629e74d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b49e74f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9809e751, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb189e754, SUC_lexicon_tags_85 },
    { 0x1c9e754, SUC_lexicon_tags_17 },
    { 0x19e9e755, SUC_lexicon_tags_21 },
    { 0x57a9e757, SUC_lexicon_tags_17 },
    { 0xd5c9e757, SUC_lexicon_tags_63 },
    { 0x3889e758, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd789e75c, SUC_lexicon_tags_27 },
    { 0xb749e75c, SUC_lexicon_tags_23 },
    { 0x6129e75e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a9e763, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2609e766, SUC_lexicon_tags_85 },
    { 0xc489e767, SUC_lexicon_tags_1 },
    { 0x3329e768, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1369e76b, SUC_lexicon_tags_60 },
    { 0x8a9e76c, SUC_lexicon_tags_17 },
    { 0xf9c9e76d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa9e9e76f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc909e771, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc69e778, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc9e77b, SUC_lexicon_tags_91 },
    { 0x6489e77b, SUC_lexicon_tags_485 },
    { 0x62e9e77c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf09e788, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de9e78d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c9e794, SUC_lexicon_tags_6 },
    { 0x6889e795, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8469e798, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c9e7a6, SUC_lexicon_tags_14 },
    { 0xf529e7a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x79c9e7a9, SUC_lexicon_tags_92 },
    { 0x7fa9e7a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xad09e7ac, SUC_lexicon_tags_39 },
    { 0xd329e7ac, SUC_lexicon_tags_11 },
    { 0x3ea9e7ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a29e7b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf669e7b4, SUC_lexicon_tags_11 },
    { 0x9ce9e7b4, SUC_lexicon_tags_14 },
    { 0xb3c9e7b5, SUC_lexicon_tags_33 },
    { 0x3be9e7b7, SUC_lexicon_tags_17 },
    { 0xad89e7b8, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a29e7bb, SUC_lexicon_tags_15 },
    { 0x1389e7bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22e9e7be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9189e7c2, SUC_lexicon_tags_10 },
    { 0x7349e7c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe349e7c7, SUC_lexicon_tags_107 },
    { 0x1de9e7c8, SUC_lexicon_tags_0 },
    { 0x7f49e7c8, SUC_lexicon_tags_14 },
    { 0x2529e7ca, SUC_lexicon_tags_15 },
    { 0x6529e7ca, SUC_lexicon_tags_6 },
    { 0xa869e7cc, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3729e7d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c9e7d3, SUC_lexicon_tags_31 },
    { 0x8529e7d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c9e7d8, SUC_lexicon_tags_1 },
    { 0x169e7d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1369e7df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3869e7e2, SUC_lexicon_tags_23 },
    { 0xc9e9e7e3, SUC_lexicon_tags_11 },
    { 0x9649e7e4, SUC_lexicon_tags_11 },
    { 0x44e9e7e2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7569e7ec, SUC_lexicon_tags_17 },
    { 0xa809e7ed, SUC_lexicon_tags_9 },
    { 0xcb89e7ed, SUC_lexicon_tags_25 },
    { 0x4129e7ef, SUC_lexicon_tags_151 },
    { 0x8309e7f0, SUC_lexicon_tags_37 },
    { 0xeb09e7f1, SUC_lexicon_tags_15 },
    { 0x9829e7f0, SUC_lexicon_tags_15 },
    { 0x5769e7f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41c9e7f5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xbac9e7f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a49e7fd, SUC_lexicon_tags_11 },
    { 0x7069e7fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc9e800, SUC_lexicon_tags_17 },
    { 0xe9c9e801, SUC_lexicon_tags_11 },
    { 0x3ea9e802, SUC_lexicon_tags_11 },
    { 0xcdc9e801, SUC_lexicon_tags_0 },
    { 0x6329e804, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6329e806, SUC_lexicon_tags_60 },
    { 0xe369e806, SUC_lexicon_tags_14 },
    { 0xb09e807, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea9e80b, SUC_lexicon_tags_11 },
    { 0x5629e80b, SUC_lexicon_tags_15 },
    { 0xd4a9e80d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc329e810, SUC_lexicon_tags_80 },
    { 0xfbc9e811, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf949e813, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb49e815, SUC_lexicon_tags_31 },
    { 0x5c69e816, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf549e81b, SUC_lexicon_tags_12 },
    { 0x7349e81b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3309e820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4c9e822, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d69e825, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafc9e827, SUC_lexicon_tags_15 },
    { 0x6b09e828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc29e82c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8e09e82e, SUC_lexicon_tags_29 },
    { 0xd969e82f, SUC_lexicon_tags_37 },
    { 0x2fe9e830, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83e9e833, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb649e836, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd929e83c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3be9e83f, SUC_lexicon_tags_14 },
    { 0x2ea9e840, SUC_lexicon_tags_60 },
    { 0xf969e840, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e69e848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a9e84b, SUC_lexicon_tags_37 },
    { 0xbb09e84c, SUC_lexicon_tags_27 },
    { 0x2269e84b, SUC_lexicon_tags_0 },
    { 0x6949e84e, SUC_lexicon_tags_27 },
    { 0x4589e84e, SUC_lexicon_tags_37 },
    { 0xc5a9e850, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e9e853, SUC_lexicon_tags_65 },
    { 0xe3a9e854, SUC_lexicon_tags_12 },
    { 0xd409e853, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c89e858, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8929e85c, SUC_lexicon_tags_82 },
    { 0xdfa9e85c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1429e861, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5389e866, SUC_lexicon_tags_37 },
    { 0x6249e867, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c9e86a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc369e871, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c9e874, SUC_lexicon_tags_37 },
    { 0x3489e875, SUC_lexicon_tags_1 },
    { 0xd4c9e876, SUC_lexicon_tags_37 },
    { 0xa5c9e874, SUC_lexicon_tags_38 },
    { 0xe909e878, SUC_lexicon_tags_4 },
    { 0xe629e878, SUC_lexicon_tags_27 },
    { 0xdf89e87a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6469e87d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a9e883, SUC_lexicon_tags_2 },
    { 0x669e884, SUC_lexicon_tags_10 },
    { 0x7769e885, SUC_lexicon_tags_9 },
    { 0x2e69e884, SUC_lexicon_tags_14 },
    { 0xde89e886, SUC_lexicon_tags_25 },
    { 0x59e9e888, SUC_lexicon_tags_15 },
    { 0xcc49e888, SUC_lexicon_tags_55 },
    { 0x4729e888, SUC_lexicon_tags_47 },
    { 0xfde9e889, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3529e88d, SUC_lexicon_tags_15 },
    { 0x8c49e88e, SUC_lexicon_tags_15 },
    { 0x9f89e88d, SUC_lexicon_tags_15 },
    { 0xd4e9e88d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90a9e893, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1289e895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7689e899, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a49e89b, SUC_lexicon_tags_12 },
    { 0x3ae9e89b, SUC_lexicon_tags_46 },
    { 0xf9a9e89d, SUC_lexicon_tags_4 },
    { 0x9909e89e, SUC_lexicon_tags_16 },
    { 0x3c49e89f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf549e8a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc9e8ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5e9e8ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f69e8b3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x569e8b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xffc9e8b7, SUC_lexicon_tags_7 },
    { 0x1b09e8b7, SUC_lexicon_tags_13 },
    { 0xa09e8b7, SUC_lexicon_tags_15 },
    { 0x1a49e8b9, SUC_lexicon_tags_63 },
    { 0xe29e8b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2ec9e8bd, SUC_lexicon_tags_28 },
    { 0x4a69e8be, SUC_lexicon_tags_15 },
    { 0x9149e8be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac9e8c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4529e8c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xec29e8c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e9e8ce, SUC_lexicon_tags_60 },
    { 0x5809e8cf, SUC_lexicon_tags_63 },
    { 0xd8a9e8d0, SUC_lexicon_tags_27 },
    { 0xefe9e8d0, SUC_lexicon_tags_2 },
    { 0x9f09e8ce, SUC_lexicon_tags_1 },
    { 0xba49e8ce, SUC_lexicon_tags_2 },
    { 0x6329e8d0, SUC_lexicon_tags_25 },
    { 0x4409e8d5, SUC_lexicon_tags_30 },
    { 0xf369e8d6, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb609e8db, SUC_lexicon_tags_40 },
    { 0x2be9e8db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda89e8de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9e8e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5389e8e4, SUC_lexicon_tags_29 },
    { 0x64a9e8e4, SUC_lexicon_tags_17 },
    { 0x409e8e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e9e8eb, SUC_lexicon_tags_48 },
    { 0xac9e8ec, SUC_lexicon_tags_39 },
    { 0xf4c9e8ed, SUC_lexicon_tags_87 },
    { 0xcfc9e8ee, SUC_lexicon_tags_7 },
    { 0x76c9e8ee, SUC_lexicon_tags_47 },
    { 0xce09e8ef, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x349e8f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8429e8f4, SUC_lexicon_tags_30 },
    { 0x6ca9e8f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ec9e8f7, SUC_lexicon_tags_38 },
    { 0xe049e8f7, SUC_lexicon_tags_8 },
    { 0x6429e8f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d29e8fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xca49e900, SUC_lexicon_tags_15 },
    { 0x21c9e901, SUC_lexicon_tags_45 },
    { 0x1389e900, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa569e908, SUC_lexicon_tags_25 },
    { 0x5ea9e909, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c9e90d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa9e911, SUC_lexicon_tags_64 },
    { 0x1029e912, SUC_lexicon_tags_79 },
    { 0x4fa9e913, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c9e916, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc649e918, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9689e91a, SUC_lexicon_tags_11 },
    { 0x7809e91b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4329e91d, SUC_lexicon_tags_12 },
    { 0x5749e91e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae9e926, SUC_lexicon_tags_6 },
    { 0x94a9e927, SUC_lexicon_tags_2 },
    { 0xc629e928, SUC_lexicon_tags_17 },
    { 0x9169e928, SUC_lexicon_tags_21 },
    { 0x7be9e928, SUC_lexicon_tags_282 },
    { 0x3f89e92b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be9e92e, SUC_lexicon_tags_25 },
    { 0xd4e9e92e, SUC_lexicon_tags_8 },
    { 0xbbc9e92f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x33e9e932, SUC_lexicon_tags_108 },
    { 0x1529e933, SUC_lexicon_tags_37 },
    { 0xc929e933, SUC_lexicon_tags_1 },
    { 0x3889e935, SUC_lexicon_tags_7 },
    { 0x3349e936, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8409e939, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb269e93d, SUC_lexicon_tags_29 },
    { 0xd689e93e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf369e941, SUC_lexicon_tags_25 },
    { 0x1609e942, SUC_lexicon_tags_0 },
    { 0x1909e942, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa449e946, SUC_lexicon_tags_66 },
    { 0xa209e947, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8409e949, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9c9e94b, SUC_lexicon_tags_37 },
    { 0x73c9e94c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc369e950, SUC_lexicon_tags_25 },
    { 0x5249e951, SUC_lexicon_tags_17 },
    { 0x56a9e950, SUC_lexicon_tags_60 },
    { 0x5109e953, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2669e958, SUC_lexicon_tags_1 },
    { 0x18c9e958, SUC_lexicon_tags_66 },
    { 0x53a9e958, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x429e95d, SUC_lexicon_tags_1 },
    { 0x5a89e95e, SUC_lexicon_tags_12 },
    { 0xf7e9e95f, SUC_lexicon_tags_14 },
    { 0xe9e95e, SUC_lexicon_tags_65 },
    { 0x5769e960, SUC_lexicon_tags_7 },
    { 0x74c9e962, SUC_lexicon_tags_48 },
    { 0x649e962, SUC_lexicon_tags_25 },
    { 0x99a9e95f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc9e967, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5769e969, SUC_lexicon_tags_15 },
    { 0xa709e96a, SUC_lexicon_tags_2 },
    { 0xd389e96a, SUC_lexicon_tags_60 },
    { 0x5f89e96c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5609e970, SUC_lexicon_tags_19 },
    { 0x1de9e971, SUC_lexicon_tags_60 },
    { 0xcd09e971, SUC_lexicon_tags_63 },
    { 0xbe89e971, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c9e97a, SUC_lexicon_tags_1 },
    { 0x1729e97a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d69e989, SUC_lexicon_tags_23 },
    { 0xb869e989, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b29e98d, SUC_lexicon_tags_17 },
    { 0x8f69e98e, SUC_lexicon_tags_17 },
    { 0x71a9e98d, SUC_lexicon_tags_21 },
    { 0xa9e98e, SUC_lexicon_tags_15 },
    { 0xce09e991, SUC_lexicon_tags_33 },
    { 0xce49e991, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa09e995, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe4a9e997, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1829e99a, SUC_lexicon_tags_26 },
    { 0x2fa9e99b, SUC_lexicon_tags_47 },
    { 0x1509e99a, SUC_lexicon_tags_27 },
    { 0x4c29e99b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc749e99f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe389e9a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca9e9a5, SUC_lexicon_tags_47 },
    { 0x5989e9a6, SUC_lexicon_tags_58 },
    { 0x9d89e9a6, SUC_lexicon_tags_0 },
    { 0xc7a9e9a5, SUC_lexicon_tags_47 },
    { 0x7909e9a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa069e9ac, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2029e9b1, SUC_lexicon_tags_37 },
    { 0xb5a9e9b2, SUC_lexicon_tags_1 },
    { 0x1009e9b3, SUC_lexicon_tags_11 },
    { 0xd4e9e9b4, SUC_lexicon_tags_37 },
    { 0xa229e9b2, SUC_lexicon_tags_55 },
    { 0x409e9b6, SUC_lexicon_tags_12 },
    { 0x8149e9b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbfc9e9b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7949e9be, SUC_lexicon_tags_17 },
    { 0x67a9e9bf, SUC_lexicon_tags_25 },
    { 0x7ea9e9c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc029e9c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e9e9c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4a9e9c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9549e9ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b49e9d0, SUC_lexicon_tags_25 },
    { 0xb49e9d1, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d89e9d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c9e9da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c89e9e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e9e9e3, SUC_lexicon_tags_43 },
    { 0x5049e9e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c89e9e8, SUC_lexicon_tags_23 },
    { 0xa989e9e9, SUC_lexicon_tags_1 },
    { 0x21e9e9e9, SUC_lexicon_tags_29 },
    { 0x2129e9e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9689e9f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1249e9f4, SUC_lexicon_tags_17 },
    { 0xfa9e9f4, SUC_lexicon_tags_25 },
    { 0x209e9f6, SUC_lexicon_tags_17 },
    { 0xdfa9e9f7, SUC_lexicon_tags_10 },
    { 0x4c29e9f8, SUC_lexicon_tags_17 },
    { 0xf5c9e9f8, SUC_lexicon_tags_17 },
    { 0x7a49e9fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2349e9fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9ea05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4449ea0d, SUC_lexicon_tags_56 },
    { 0x52e9ea0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdb69ea10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8aa9ea12, SUC_lexicon_tags_9 },
    { 0xad09ea13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9709ea15, SUC_lexicon_tags_11 },
    { 0xa229ea15, SUC_lexicon_tags_12 },
    { 0xf229ea17, SUC_lexicon_tags_28 },
    { 0x449ea18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4589ea1b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe769ea20, SUC_lexicon_tags_49 },
    { 0xe749ea21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb489ea2d, SUC_lexicon_tags_60 },
    { 0xd729ea2e, SUC_lexicon_tags_59 },
    { 0xf289ea2f, SUC_lexicon_tags_43 },
    { 0xb129ea2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29c9ea32, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5969ea35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8a9ea37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb09ea39, SUC_lexicon_tags_7 },
    { 0xb669ea3a, SUC_lexicon_tags_45 },
    { 0x93a9ea39, SUC_lexicon_tags_992 },
    { 0, NULL },
    { 0xfde9ea3d, SUC_lexicon_tags_11 },
    { 0xf669ea3e, SUC_lexicon_tags_11 },
    { 0xe069ea3d, SUC_lexicon_tags_10 },
    { 0xf4c9ea40, SUC_lexicon_tags_14 },
    { 0xae9ea41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4649ea45, SUC_lexicon_tags_62 },
    { 0xe749ea46, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x249ea48, SUC_lexicon_tags_37 },
    { 0xd349ea49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7509ea4c, SUC_lexicon_tags_12 },
    { 0x1a09ea4d, SUC_lexicon_tags_106 },
    { 0x78c9ea4e, SUC_lexicon_tags_46 },
    { 0x349ea4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29ea53, SUC_lexicon_tags_21 },
    { 0xb889ea54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca9ea5a, SUC_lexicon_tags_38 },
    { 0x60e9ea5a, SUC_lexicon_tags_0 },
    { 0x2a89ea5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec89ea61, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x61a9ea63, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9909ea66, SUC_lexicon_tags_45 },
    { 0x7909ea67, SUC_lexicon_tags_17 },
    { 0x7509ea68, SUC_lexicon_tags_68 },
    { 0xbc29ea69, SUC_lexicon_tags_25 },
    { 0x1029ea6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec9ea70, SUC_lexicon_tags_65 },
    { 0x6829ea71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca9ea76, SUC_lexicon_tags_25 },
    { 0xb449ea77, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69ea7c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1549ea80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce29ea88, SUC_lexicon_tags_14 },
    { 0xbdc9ea89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e9ea92, SUC_lexicon_tags_47 },
    { 0x9bc9ea93, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9369ea95, SUC_lexicon_tags_37 },
    { 0x4609ea96, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f69ea99, SUC_lexicon_tags_13 },
    { 0x7809ea9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2569ea9d, SUC_lexicon_tags_38 },
    { 0xb29ea9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13a9eaa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7069eaaa, SUC_lexicon_tags_31 },
    { 0x269eaaa, SUC_lexicon_tags_1 },
    { 0x3a89eaac, SUC_lexicon_tags_15 },
    { 0x9769eaaa, SUC_lexicon_tags_40 },
    { 0xd609eaae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a9eab0, SUC_lexicon_tags_6 },
    { 0xff49eab0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa09eab6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba49eab8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a69eaba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9449eabe, SUC_lexicon_tags_9 },
    { 0xd329eabf, SUC_lexicon_tags_32 },
    { 0x5ee9eac0, SUC_lexicon_tags_15 },
    { 0xe6e9eabf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b89eac8, SUC_lexicon_tags_47 },
    { 0x4fe9eac9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c9eacc, SUC_lexicon_tags_15 },
    { 0x9489eacc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa409ead2, SUC_lexicon_tags_21 },
    { 0xa9ead2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3109eadb, SUC_lexicon_tags_15 },
    { 0x8109eadb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb549eae0, SUC_lexicon_tags_32 },
    { 0xc9eae1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5589eae3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c9eaee, SUC_lexicon_tags_77 },
    { 0xe2a9eaef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69eaf4, SUC_lexicon_tags_15 },
    { 0x8ea9eaf5, SUC_lexicon_tags_47 },
    { 0x869eaf6, SUC_lexicon_tags_3 },
    { 0xdf29eaf4, SUC_lexicon_tags_60 },
    { 0x449eaf8, SUC_lexicon_tags_29 },
    { 0x42a9eaf8, SUC_lexicon_tags_37 },
    { 0xefa9eaf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede9eb04, SUC_lexicon_tags_23 },
    { 0x5109eb05, SUC_lexicon_tags_1 },
    { 0xd929eb06, SUC_lexicon_tags_17 },
    { 0xb969eb05, SUC_lexicon_tags_19 },
    { 0x1049eb08, SUC_lexicon_tags_63 },
    { 0x2f29eb09, SUC_lexicon_tags_29 },
    { 0xbc89eb04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe649eb0d, SUC_lexicon_tags_25 },
    { 0xb8c9eb0e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x23c9eb10, SUC_lexicon_tags_1 },
    { 0x14e9eb10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a9eb19, SUC_lexicon_tags_32 },
    { 0x4509eb1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd229eb1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21a9eb1e, SUC_lexicon_tags_82 },
    { 0x65e9eb1f, SUC_lexicon_tags_33 },
    { 0x8a09eb20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe989eb22, SUC_lexicon_tags_0 },
    { 0x3e09eb23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2329eb25, SUC_lexicon_tags_37 },
    { 0x3669eb26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5189eb2b, SUC_lexicon_tags_15 },
    { 0x6c9eb2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a09eb2f, SUC_lexicon_tags_9 },
    { 0x48a9eb30, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae9eb34, SUC_lexicon_tags_170 },
    { 0x7a69eb35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc9eb38, SUC_lexicon_tags_107 },
    { 0xcde9eb39, SUC_lexicon_tags_555 },
    { 0x7a69eb3a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e9eb3d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9cc9eb3f, SUC_lexicon_tags_2 },
    { 0x829eb40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba49eb47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3269eb4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9eb54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a9eb57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c9eb5c, SUC_lexicon_tags_1 },
    { 0xc209eb5c, SUC_lexicon_tags_77 },
    { 0x3a9eb5c, SUC_lexicon_tags_12 },
    { 0x5929eb5c, SUC_lexicon_tags_37 },
    { 0x3d29eb60, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b49eb63, SUC_lexicon_tags_7 },
    { 0xfc89eb63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe289eb6b, SUC_lexicon_tags_6 },
    { 0x7a09eb6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da9eb6f, SUC_lexicon_tags_23 },
    { 0x7fc9eb6f, SUC_lexicon_tags_49 },
    { 0xecc9eb71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa369eb73, SUC_lexicon_tags_12 },
    { 0xca29eb74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a9eb78, SUC_lexicon_tags_11 },
    { 0x9d89eb78, SUC_lexicon_tags_38 },
    { 0xf9a9eb7a, SUC_lexicon_tags_9 },
    { 0x2e9eb7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ac9eb7d, SUC_lexicon_tags_1 },
    { 0x17a9eb7e, SUC_lexicon_tags_19 },
    { 0x4a49eb7e, SUC_lexicon_tags_37 },
    { 0xa4e9eb7f, SUC_lexicon_tags_2 },
    { 0xfb69eb81, SUC_lexicon_tags_7 },
    { 0x929eb80, SUC_lexicon_tags_13 },
    { 0x8e49eb83, SUC_lexicon_tags_47 },
    { 0xc429eb82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb849eb87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe349eb93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf869eb9a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa9a9eb9c, SUC_lexicon_tags_34 },
    { 0x6909eb9d, SUC_lexicon_tags_46 },
    { 0x5309eb9e, SUC_lexicon_tags_37 },
    { 0xe0a9eb9f, SUC_lexicon_tags_17 },
    { 0x6dc9eba0, SUC_lexicon_tags_49 },
    { 0x8449eb9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec9eba4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c49eba7, SUC_lexicon_tags_7 },
    { 0x9849eba7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e09ebb1, SUC_lexicon_tags_17 },
    { 0xed49ebb1, SUC_lexicon_tags_11 },
    { 0xff89ebb3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c9ebba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a9ebbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8469ebbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc529ebc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc9ebc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf89ebc7, SUC_lexicon_tags_11 },
    { 0x3d89ebc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2a9ebca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe649ebcf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf169ebd4, SUC_lexicon_tags_11 },
    { 0xd2a9ebd5, SUC_lexicon_tags_15 },
    { 0x4b09ebd5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ca9ebdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a9ebe5, SUC_lexicon_tags_1 },
    { 0x8f49ebe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d69ebe8, SUC_lexicon_tags_9 },
    { 0xdde9ebe9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9649ebed, SUC_lexicon_tags_68 },
    { 0x6e9ebee, SUC_lexicon_tags_43 },
    { 0x4fc9ebef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd49ebf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe869ebf7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f49ebfe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3029ec0a, SUC_lexicon_tags_8 },
    { 0x3209ec0b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6c29ec0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6949ec10, SUC_lexicon_tags_25 },
    { 0x529ec11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2789ec13, SUC_lexicon_tags_66 },
    { 0x5e29ec14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5369ec16, SUC_lexicon_tags_28 },
    { 0xbde9ec17, SUC_lexicon_tags_8 },
    { 0x1549ec18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcac9ec1a, SUC_lexicon_tags_15 },
    { 0xba09ec1b, SUC_lexicon_tags_43 },
    { 0x7829ec1c, SUC_lexicon_tags_26 },
    { 0x3b29ec1d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c9ec23, SUC_lexicon_tags_11 },
    { 0x5309ec24, SUC_lexicon_tags_23 },
    { 0xe749ec25, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf49ec28, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa289ec2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9049ec2e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7369ec31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff49ec36, SUC_lexicon_tags_8 },
    { 0x449ec37, SUC_lexicon_tags_21 },
    { 0xa409ec36, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8409ec3b, SUC_lexicon_tags_23 },
    { 0xac89ec3c, SUC_lexicon_tags_27 },
    { 0x4ee9ec3c, SUC_lexicon_tags_47 },
    { 0xf909ec3e, SUC_lexicon_tags_29 },
    { 0x8869ec3f, SUC_lexicon_tags_54 },
    { 0xf049ec3d, SUC_lexicon_tags_113 },
    { 0x9469ec3b, SUC_lexicon_tags_26 },
    { 0xc469ec42, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e9ec45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd769ec4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd749ec4e, SUC_lexicon_tags_37 },
    { 0x5d29ec4e, SUC_lexicon_tags_33 },
    { 0xe09ec4f, SUC_lexicon_tags_41 },
    { 0x4369ec50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7809ec53, SUC_lexicon_tags_19 },
    { 0xe9ec54, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4469ec58, SUC_lexicon_tags_9 },
    { 0xdc09ec58, SUC_lexicon_tags_11 },
    { 0xe289ec5a, SUC_lexicon_tags_1 },
    { 0x2ec9ec5b, SUC_lexicon_tags_61 },
    { 0xe9ec5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc809ec68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39a9ec6a, SUC_lexicon_tags_25 },
    { 0x9e49ec6b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa229ec70, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3929ec73, SUC_lexicon_tags_25 },
    { 0x4709ec74, SUC_lexicon_tags_8 },
    { 0x2c9ec74, SUC_lexicon_tags_12 },
    { 0xfd29ec76, SUC_lexicon_tags_14 },
    { 0x1549ec77, SUC_lexicon_tags_7 },
    { 0xf109ec77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9649ec7b, SUC_lexicon_tags_38 },
    { 0xb229ec7c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a9ec83, SUC_lexicon_tags_14 },
    { 0x2629ec84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa9ec8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc149ec8e, SUC_lexicon_tags_37 },
    { 0x1829ec8f, SUC_lexicon_tags_17 },
    { 0x329ec90, SUC_lexicon_tags_12 },
    { 0x57a9ec8e, SUC_lexicon_tags_63 },
    { 0x2ee9ec92, SUC_lexicon_tags_87 },
    { 0xf849ec93, SUC_lexicon_tags_53 },
    { 0xd509ec94, SUC_lexicon_tags_95 },
    { 0xad09ec91, SUC_lexicon_tags_63 },
    { 0xdf69ec96, SUC_lexicon_tags_37 },
    { 0x3729ec96, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x9d89ec99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe049ec9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b89ec9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a29eca2, SUC_lexicon_tags_17 },
    { 0x5809eca3, SUC_lexicon_tags_24 },
    { 0xf7e9eca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c9eca7, SUC_lexicon_tags_13 },
    { 0x76a9eca8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa09ecab, SUC_lexicon_tags_11 },
    { 0x5d29ecac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b89ecb3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc209ecb7, SUC_lexicon_tags_10 },
    { 0x3e49ecb7, SUC_lexicon_tags_17 },
    { 0xc1c9ecb8, SUC_lexicon_tags_29 },
    { 0xeae9ecba, SUC_lexicon_tags_25 },
    { 0x83c9ecbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58a9ecbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xec09ecbf, SUC_lexicon_tags_7 },
    { 0x6029ecc0, SUC_lexicon_tags_37 },
    { 0x1369ecc1, SUC_lexicon_tags_37 },
    { 0xedc9ecbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6269ecc7, SUC_lexicon_tags_57 },
    { 0xc149ecc8, SUC_lexicon_tags_12 },
    { 0x8389ecc9, SUC_lexicon_tags_1 },
    { 0x5a69ecc8, SUC_lexicon_tags_14 },
    { 0x99c9eccb, SUC_lexicon_tags_37 },
    { 0x7ac9ecc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc2a9ecce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56a9ecd0, SUC_lexicon_tags_38 },
    { 0x2509ecd1, SUC_lexicon_tags_25 },
    { 0x57c9ecd2, SUC_lexicon_tags_7 },
    { 0x27c9ecd3, SUC_lexicon_tags_11 },
    { 0x4869ecd2, SUC_lexicon_tags_25 },
    { 0x8e49ecd5, SUC_lexicon_tags_17 },
    { 0x2d69ecd6, SUC_lexicon_tags_11 },
    { 0x7809ecd7, SUC_lexicon_tags_27 },
    { 0x6409ecd8, SUC_lexicon_tags_11 },
    { 0x4be9ecd9, SUC_lexicon_tags_7 },
    { 0x2e9ecd8, SUC_lexicon_tags_38 },
    { 0x5349ecdb, SUC_lexicon_tags_25 },
    { 0x5489ecdc, SUC_lexicon_tags_63 },
    { 0x209ecd7, SUC_lexicon_tags_65 },
    { 0xbf69ecde, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889ece4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8909ece6, SUC_lexicon_tags_26 },
    { 0xffe9ece7, SUC_lexicon_tags_1 },
    { 0x35a9ece8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa569ecec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a9ecf0, SUC_lexicon_tags_30 },
    { 0x5dc9ecf1, SUC_lexicon_tags_15 },
    { 0xf0c9ecf2, SUC_lexicon_tags_47 },
    { 0x93a9ecf3, SUC_lexicon_tags_14 },
    { 0x5589ecf2, SUC_lexicon_tags_17 },
    { 0x19a9ecf5, SUC_lexicon_tags_25 },
    { 0xa709ecf0, SUC_lexicon_tags_25 },
    { 0x5109ecf7, SUC_lexicon_tags_40 },
    { 0xbae9ecf7, SUC_lexicon_tags_45 },
    { 0xe229ecf7, SUC_lexicon_tags_23 },
    { 0xdc29ecfa, SUC_lexicon_tags_11 },
    { 0x58a9ecfb, SUC_lexicon_tags_29 },
    { 0x89ecf9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c89ecfe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe949ed04, SUC_lexicon_tags_38 },
    { 0x40c9ed05, SUC_lexicon_tags_12 },
    { 0x6769ed06, SUC_lexicon_tags_34 },
    { 0xfee9ed07, SUC_lexicon_tags_15 },
    { 0x2489ed07, SUC_lexicon_tags_15 },
    { 0x5429ed07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a29ed0b, SUC_lexicon_tags_14 },
    { 0x4189ed0c, SUC_lexicon_tags_66 },
    { 0xf229ed0c, SUC_lexicon_tags_11 },
    { 0x4e89ed0e, SUC_lexicon_tags_35 },
    { 0x7469ed0c, SUC_lexicon_tags_33 },
    { 0x1f49ed10, SUC_lexicon_tags_21 },
    { 0x2e29ed10, SUC_lexicon_tags_27 },
    { 0xe529ed0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd269ed14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f89ed1c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9069ed1f, SUC_lexicon_tags_15 },
    { 0xdc69ed1f, SUC_lexicon_tags_26 },
    { 0xc9e9ed21, SUC_lexicon_tags_17 },
    { 0xc8c9ed21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f29ed25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e9ed2c, SUC_lexicon_tags_21 },
    { 0x6a29ed2d, SUC_lexicon_tags_63 },
    { 0x5289ed2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc9ed32, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3289ed35, SUC_lexicon_tags_15 },
    { 0xee49ed36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c9ed3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef09ed3e, SUC_lexicon_tags_2 },
    { 0x7fa9ed3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac9ed43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9e49ed45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef29ed49, SUC_lexicon_tags_1 },
    { 0xa8e9ed4a, SUC_lexicon_tags_12 },
    { 0x64e9ed4b, SUC_lexicon_tags_37 },
    { 0x86c9ed4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3e9ed4e, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a9ed55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e9ed59, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5329ed5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8429ed5f, SUC_lexicon_tags_1 },
    { 0xc3e9ed60, SUC_lexicon_tags_16 },
    { 0x8889ed5f, SUC_lexicon_tags_38 },
    { 0x7d29ed62, SUC_lexicon_tags_45 },
    { 0x1ba9ed63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf69ed6a, SUC_lexicon_tags_27 },
    { 0xd8e9ed6b, SUC_lexicon_tags_1 },
    { 0x20e9ed6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28a9ed6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd09ed75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b09ed80, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c9ed83, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x27a9ed85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e9ed8b, SUC_lexicon_tags_3 },
    { 0xea69ed8c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeea9ed90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc929ed94, SUC_lexicon_tags_19 },
    { 0xc009ed95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6509ed98, SUC_lexicon_tags_138 },
    { 0x989ed99, SUC_lexicon_tags_33 },
    { 0xdaa9ed98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3c9ed9d, SUC_lexicon_tags_30 },
    { 0xc7c9ed9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4089eda2, SUC_lexicon_tags_2 },
    { 0xeea9eda3, SUC_lexicon_tags_37 },
    { 0xf289eda4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb29eda9, SUC_lexicon_tags_57 },
    { 0x3ec9edaa, SUC_lexicon_tags_43 },
    { 0xf2c9edab, SUC_lexicon_tags_38 },
    { 0xd849edac, SUC_lexicon_tags_70 },
    { 0xe9e9edac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8309edaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f49edb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1189edb7, SUC_lexicon_tags_12 },
    { 0xe869edb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a29edbe, SUC_lexicon_tags_17 },
    { 0xa969edbf, SUC_lexicon_tags_15 },
    { 0x3c69edc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a9edcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6529edcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8189edd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2769edd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9509edd7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb389eddb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b09eddd, SUC_lexicon_tags_28 },
    { 0xa0c9edde, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c69ede2, SUC_lexicon_tags_37 },
    { 0x4b89ede3, SUC_lexicon_tags_1 },
    { 0x4ba9ede3, SUC_lexicon_tags_1 },
    { 0x9469ede5, SUC_lexicon_tags_7 },
    { 0xf8c9ede6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x429ede9, SUC_lexicon_tags_48 },
    { 0x5e49edea, SUC_lexicon_tags_27 },
    { 0x2b89ede9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a9edee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe649edf4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4849edf6, SUC_lexicon_tags_25 },
    { 0xdbc9edf7, SUC_lexicon_tags_37 },
    { 0x9969edf7, SUC_lexicon_tags_11 },
    { 0x3749edf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a29edff, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc9ee02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c9ee05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x97e9ee08, SUC_lexicon_tags_2 },
    { 0x75a9ee09, SUC_lexicon_tags_30 },
    { 0xfae9ee09, SUC_lexicon_tags_65 },
    { 0x69ee0b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc9ee10, SUC_lexicon_tags_12 },
    { 0x2f09ee11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x42c9ee14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9ee17, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3009ee19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e9ee21, SUC_lexicon_tags_27 },
    { 0x34c9ee21, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f29ee25, SUC_lexicon_tags_48 },
    { 0xe4c9ee26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c9ee29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69ee2d, SUC_lexicon_tags_9 },
    { 0xcb29ee2d, SUC_lexicon_tags_47 },
    { 0x5789ee2d, SUC_lexicon_tags_57 },
    { 0xd569ee2d, SUC_lexicon_tags_25 },
    { 0x8b29ee31, SUC_lexicon_tags_8 },
    { 0x7ec9ee2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4809ee35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x37e9ee37, SUC_lexicon_tags_65 },
    { 0xf349ee38, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbfc9ee3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7229ee3c, SUC_lexicon_tags_11 },
    { 0x4469ee3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3cc9ee3f, SUC_lexicon_tags_19 },
    { 0xccc9ee40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55e9ee42, SUC_lexicon_tags_83 },
    { 0xacc9ee42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb429ee45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1929ee47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc549ee49, SUC_lexicon_tags_55 },
    { 0xf7c9ee4a, SUC_lexicon_tags_17 },
    { 0xa6a9ee49, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xec89ee4e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe549ee54, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc569ee56, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc49ee59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc4a9ee5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe229ee60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c89ee65, SUC_lexicon_tags_23 },
    { 0x8209ee65, SUC_lexicon_tags_17 },
    { 0xcf49ee67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7769ee6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb669ee6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfe9ee6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeca9ee71, SUC_lexicon_tags_12 },
    { 0xf109ee72, SUC_lexicon_tags_1 },
    { 0x4589ee73, SUC_lexicon_tags_26 },
    { 0x76a9ee73, SUC_lexicon_tags_27 },
    { 0x8449ee74, SUC_lexicon_tags_53 },
    { 0xf5e9ee73, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2389ee78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42c9ee7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5309ee7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f49ee81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c69ee83, SUC_lexicon_tags_9 },
    { 0xf069ee84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x94a9ee87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c9ee8a, SUC_lexicon_tags_15 },
    { 0x8289ee8a, SUC_lexicon_tags_12 },
    { 0x7589ee8b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b09ee91, SUC_lexicon_tags_7 },
    { 0x5829ee92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe769ee94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a09ee9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8709eea1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd29eea4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3489eea7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c9eeaa, SUC_lexicon_tags_84 },
    { 0x4ae9eeaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24a9eeae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a9eeb2, SUC_lexicon_tags_43 },
    { 0x8ac9eeb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d09eeb6, SUC_lexicon_tags_2 },
    { 0x8d09eeb7, SUC_lexicon_tags_6 },
    { 0x5a9eeb8, SUC_lexicon_tags_27 },
    { 0x1889eeb8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5689eebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7669eec0, SUC_lexicon_tags_37 },
    { 0x3149eec0, SUC_lexicon_tags_10 },
    { 0x7cc9eec2, SUC_lexicon_tags_25 },
    { 0xb789eec3, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6109eecb, SUC_lexicon_tags_27 },
    { 0x4a09eecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec69eed2, SUC_lexicon_tags_17 },
    { 0x9809eed3, SUC_lexicon_tags_15 },
    { 0x3d49eed4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2969eedf, SUC_lexicon_tags_0 },
    { 0x6f09eedf, SUC_lexicon_tags_32 },
    { 0x9a9eedf, SUC_lexicon_tags_14 },
    { 0x2d89eee1, SUC_lexicon_tags_1 },
    { 0xe6a9eee2, SUC_lexicon_tags_15 },
    { 0x41c9eee4, SUC_lexicon_tags_2 },
    { 0xa9a9eee3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe729eeec, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc649eeee, SUC_lexicon_tags_9 },
    { 0xcba9eeee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3229eef1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6209eef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3649eef8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd329eefd, SUC_lexicon_tags_66 },
    { 0x8e9eefe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f89ef04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f29ef0a, SUC_lexicon_tags_1 },
    { 0x2569ef0b, SUC_lexicon_tags_19 },
    { 0xc509ef0c, SUC_lexicon_tags_45 },
    { 0x7509ef0d, SUC_lexicon_tags_37 },
    { 0x3989ef0c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x8089ef11, SUC_lexicon_tags_33 },
    { 0x8109ef12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7849ef16, SUC_lexicon_tags_46 },
    { 0xa9e9ef17, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1009ef1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd889ef20, SUC_lexicon_tags_208 },
    { 0xee69ef21, SUC_lexicon_tags_9 },
    { 0x4489ef22, SUC_lexicon_tags_17 },
    { 0xb2c9ef22, SUC_lexicon_tags_14 },
    { 0x1fa9ef22, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x76a9ef26, SUC_lexicon_tags_15 },
    { 0x1969ef27, SUC_lexicon_tags_25 },
    { 0x71c9ef27, SUC_lexicon_tags_17 },
    { 0x2769ef26, SUC_lexicon_tags_1 },
    { 0xb689ef2a, SUC_lexicon_tags_25 },
    { 0x6aa9ef28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe249ef2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9ef30, SUC_lexicon_tags_3 },
    { 0x649ef31, SUC_lexicon_tags_7 },
    { 0xcee9ef32, SUC_lexicon_tags_25 },
    { 0xa049ef33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a9ef36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb389ef38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a9ef3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca9ef40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd789ef43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7409ef47, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a9ef4b, SUC_lexicon_tags_2 },
    { 0xb149ef4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb989ef4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a9ef52, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2389ef54, SUC_lexicon_tags_12 },
    { 0x83c9ef54, SUC_lexicon_tags_14 },
    { 0xeca9ef56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb09ef5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2749ef5e, SUC_lexicon_tags_23 },
    { 0xac49ef5f, SUC_lexicon_tags_15 },
    { 0x69c9ef60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f49ef65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4ea9ef67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8649ef6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x36a9ef6d, SUC_lexicon_tags_1 },
    { 0x2749ef6e, SUC_lexicon_tags_15 },
    { 0xfc49ef6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc9ef74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef29ef76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x20c9ef78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b89ef7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd389ef7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeac9ef7f, SUC_lexicon_tags_10 },
    { 0x7fa9ef7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a9ef83, SUC_lexicon_tags_9 },
    { 0x6e89ef84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e69ef86, SUC_lexicon_tags_12 },
    { 0x73c9ef87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad69ef8a, SUC_lexicon_tags_25 },
    { 0x6549ef8b, SUC_lexicon_tags_37 },
    { 0x62e9ef8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4849ef8e, SUC_lexicon_tags_17 },
    { 0x9c29ef8f, SUC_lexicon_tags_11 },
    { 0x5609ef90, SUC_lexicon_tags_11 },
    { 0xdf89ef90, SUC_lexicon_tags_16 },
    { 0x9e49ef92, SUC_lexicon_tags_37 },
    { 0xcfa9ef93, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc9ef98, SUC_lexicon_tags_11 },
    { 0xd49ef99, SUC_lexicon_tags_37 },
    { 0x6c69ef99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7429ef9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c9efa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc9efa2, SUC_lexicon_tags_37 },
    { 0x7e29efa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf669efa5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe49efaa, SUC_lexicon_tags_25 },
    { 0x7249efab, SUC_lexicon_tags_37 },
    { 0x96e9efab, SUC_lexicon_tags_37 },
    { 0xfa9efab, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9efb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8a9efbd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3789efbf, SUC_lexicon_tags_60 },
    { 0x2109efc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8149efc2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4d29efc4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d09efc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a09efcd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf389efd1, SUC_lexicon_tags_7 },
    { 0x7b69efd1, SUC_lexicon_tags_25 },
    { 0x42e9efd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47a9efd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb49efe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7489efe5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa949efe7, SUC_lexicon_tags_17 },
    { 0xc6a9efe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc689efed, SUC_lexicon_tags_28 },
    { 0xd89efee, SUC_lexicon_tags_49 },
    { 0x8109efed, SUC_lexicon_tags_12 },
    { 0x5709eff0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf469eff2, SUC_lexicon_tags_6 },
    { 0xb8a9eff3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf09eff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d29eff7, SUC_lexicon_tags_11 },
    { 0x4849eff8, SUC_lexicon_tags_25 },
    { 0xef89eff8, SUC_lexicon_tags_68 },
    { 0x7aa9eff9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf449effe, SUC_lexicon_tags_7 },
    { 0x9349efff, SUC_lexicon_tags_0 },
    { 0xfd69f000, SUC_lexicon_tags_47 },
    { 0x4549f000, SUC_lexicon_tags_27 },
    { 0xe109efff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1409f007, SUC_lexicon_tags_17 },
    { 0x6049f007, SUC_lexicon_tags_7 },
    { 0xf169f009, SUC_lexicon_tags_23 },
    { 0x5489f00a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc49f00d, SUC_lexicon_tags_65 },
    { 0x1569f00e, SUC_lexicon_tags_14 },
    { 0x35e9f00e, SUC_lexicon_tags_17 },
    { 0x1cc9f010, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d09f017, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9049f01b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb549f01d, SUC_lexicon_tags_31 },
    { 0xe5a9f01e, SUC_lexicon_tags_26 },
    { 0x9fc9f01d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc349f026, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xeee9f028, SUC_lexicon_tags_6 },
    { 0xd409f028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7789f02c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde09f033, SUC_lexicon_tags_25 },
    { 0x3ac9f034, SUC_lexicon_tags_14 },
    { 0xd6a9f033, SUC_lexicon_tags_2 },
    { 0x8a9f033, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e9f03e, SUC_lexicon_tags_27 },
    { 0xfa29f03f, SUC_lexicon_tags_25 },
    { 0x7ee9f03f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2dc9f042, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb269f047, SUC_lexicon_tags_17 },
    { 0x8429f048, SUC_lexicon_tags_7 },
    { 0xfca9f049, SUC_lexicon_tags_2 },
    { 0xf569f048, SUC_lexicon_tags_51 },
    { 0x1c89f049, SUC_lexicon_tags_37 },
    { 0xda29f04c, SUC_lexicon_tags_13 },
    { 0xa3e9f047, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca9f04f, SUC_lexicon_tags_25 },
    { 0x4269f04f, SUC_lexicon_tags_37 },
    { 0xfcc9f04f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5969f055, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe489f058, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd009f063, SUC_lexicon_tags_2 },
    { 0x5689f064, SUC_lexicon_tags_15 },
    { 0x49c9f065, SUC_lexicon_tags_0 },
    { 0xf969f066, SUC_lexicon_tags_46 },
    { 0xef69f067, SUC_lexicon_tags_17 },
    { 0x5b69f064, SUC_lexicon_tags_17 },
    { 0xa1c9f066, SUC_lexicon_tags_87 },
    { 0xb4a9f06a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4b09f06c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e49f06e, SUC_lexicon_tags_37 },
    { 0x32e9f06f, SUC_lexicon_tags_0 },
    { 0x7f89f06f, SUC_lexicon_tags_37 },
    { 0xaae9f071, SUC_lexicon_tags_25 },
    { 0xdee9f072, SUC_lexicon_tags_2 },
    { 0x2189f070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9069f076, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe49f078, SUC_lexicon_tags_45 },
    { 0x8969f079, SUC_lexicon_tags_7 },
    { 0xa289f07a, SUC_lexicon_tags_27 },
    { 0xd4a9f07a, SUC_lexicon_tags_15 },
    { 0xe8c9f07c, SUC_lexicon_tags_37 },
    { 0x3349f07a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8969f082, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f09f086, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd49f089, SUC_lexicon_tags_25 },
    { 0x9689f08a, SUC_lexicon_tags_31 },
    { 0x7a29f089, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac9f08e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b69f091, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e69f096, SUC_lexicon_tags_55 },
    { 0xf069f097, SUC_lexicon_tags_39 },
    { 0xe149f098, SUC_lexicon_tags_16 },
    { 0xca9f096, SUC_lexicon_tags_11 },
    { 0x5589f099, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc9f09e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe049f0a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c89f0a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a9f0ad, SUC_lexicon_tags_60 },
    { 0xe9c9f0ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa9f0b2, SUC_lexicon_tags_17 },
    { 0x3ec9f0b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a29f0b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e9f0ba, SUC_lexicon_tags_48 },
    { 0xd6e9f0bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7689f0bf, SUC_lexicon_tags_43 },
    { 0xac29f0c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e9f0c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e49f0c8, SUC_lexicon_tags_11 },
    { 0x1289f0c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1249f0cc, SUC_lexicon_tags_37 },
    { 0x9f09f0cc, SUC_lexicon_tags_15 },
    { 0x9909f0ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b69f0d0, SUC_lexicon_tags_28 },
    { 0xb589f0d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf029f0d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bc9f0d6, SUC_lexicon_tags_1 },
    { 0x3b09f0d6, SUC_lexicon_tags_38 },
    { 0xb609f0d8, SUC_lexicon_tags_66 },
    { 0x5ea9f0d9, SUC_lexicon_tags_26 },
    { 0xb869f0da, SUC_lexicon_tags_26 },
    { 0x8289f0d9, SUC_lexicon_tags_77 },
    { 0xcd69f0d6, SUC_lexicon_tags_15 },
    { 0x39e9f0da, SUC_lexicon_tags_43 },
    { 0x7809f0dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb289f0e1, SUC_lexicon_tags_17 },
    { 0x4229f0e2, SUC_lexicon_tags_1 },
    { 0xd809f0e3, SUC_lexicon_tags_26 },
    { 0x86a9f0e4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x64c9f0e6, SUC_lexicon_tags_8 },
    { 0xbde9f0e7, SUC_lexicon_tags_12 },
    { 0x2ce9f0e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4269f0eb, SUC_lexicon_tags_11 },
    { 0x29f0ec, SUC_lexicon_tags_37 },
    { 0x69a9f0ed, SUC_lexicon_tags_11 },
    { 0x3669f0ec, SUC_lexicon_tags_25 },
    { 0xcda9f0ed, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xeac9f0f1, SUC_lexicon_tags_19 },
    { 0x7709f0f2, SUC_lexicon_tags_11 },
    { 0x1869f0f3, SUC_lexicon_tags_25 },
    { 0x9209f0f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a9f0fa, SUC_lexicon_tags_11 },
    { 0xebc9f0fa, SUC_lexicon_tags_18 },
    { 0xca69f0fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3609f0fe, SUC_lexicon_tags_8 },
    { 0xd89f0ff, SUC_lexicon_tags_7 },
    { 0x17e9f0ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e9f103, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b29f107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe009f10a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d89f10d, SUC_lexicon_tags_37 },
    { 0x4809f10e, SUC_lexicon_tags_14 },
    { 0xea49f10d, SUC_lexicon_tags_7 },
    { 0x4e09f110, SUC_lexicon_tags_12 },
    { 0x7829f111, SUC_lexicon_tags_68 },
    { 0xa0e9f112, SUC_lexicon_tags_8 },
    { 0x709f111, SUC_lexicon_tags_13 },
    { 0xfd69f112, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9f11b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdee9f11d, SUC_lexicon_tags_27 },
    { 0x8b49f11e, SUC_lexicon_tags_17 },
    { 0x5f89f11f, SUC_lexicon_tags_17 },
    { 0x3ea9f11e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32e9f123, SUC_lexicon_tags_17 },
    { 0x5c29f124, SUC_lexicon_tags_11 },
    { 0xd329f124, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5229f129, SUC_lexicon_tags_84 },
    { 0x78a9f129, SUC_lexicon_tags_23 },
    { 0xff29f12b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa189f12f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7369f137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69f13b, SUC_lexicon_tags_30 },
    { 0x11c9f13c, SUC_lexicon_tags_15 },
    { 0x7749f13d, SUC_lexicon_tags_33 },
    { 0xfc49f13e, SUC_lexicon_tags_15 },
    { 0x18a9f13e, SUC_lexicon_tags_46 },
    { 0x9f09f140, SUC_lexicon_tags_1 },
    { 0xd1a9f141, SUC_lexicon_tags_23 },
    { 0xfc49f142, SUC_lexicon_tags_37 },
    { 0x79c9f143, SUC_lexicon_tags_7 },
    { 0x3249f142, SUC_lexicon_tags_8 },
    { 0x2dc9f145, SUC_lexicon_tags_12 },
    { 0x1509f146, SUC_lexicon_tags_25 },
    { 0x8e69f147, SUC_lexicon_tags_23 },
    { 0x38e9f148, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2549f14c, SUC_lexicon_tags_15 },
    { 0x4789f14d, SUC_lexicon_tags_15 },
    { 0x8ce9f14e, SUC_lexicon_tags_191 },
    { 0x6e09f14f, SUC_lexicon_tags_1 },
    { 0x92e9f150, SUC_lexicon_tags_12 },
    { 0x6fc9f151, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb89f153, SUC_lexicon_tags_7 },
    { 0x6ba9f154, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd809f156, SUC_lexicon_tags_19 },
    { 0x9ee9f157, SUC_lexicon_tags_15 },
    { 0x9849f158, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd49f15f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc9f163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c9f168, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce29f170, SUC_lexicon_tags_8 },
    { 0x5d09f171, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81e9f173, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fc9f175, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6f69f177, SUC_lexicon_tags_30 },
    { 0xda09f177, SUC_lexicon_tags_23 },
    { 0x9ac9f178, SUC_lexicon_tags_68 },
    { 0x8109f17a, SUC_lexicon_tags_23 },
    { 0x17e9f17a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb709f17d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x629f180, SUC_lexicon_tags_23 },
    { 0x2469f181, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f49f184, SUC_lexicon_tags_15 },
    { 0xfa69f185, SUC_lexicon_tags_92 },
    { 0xc109f185, SUC_lexicon_tags_39 },
    { 0x32a9f185, SUC_lexicon_tags_47 },
    { 0x9bc9f188, SUC_lexicon_tags_43 },
    { 0x9569f189, SUC_lexicon_tags_40 },
    { 0xb169f186, SUC_lexicon_tags_14 },
    { 0x8c29f18b, SUC_lexicon_tags_15 },
    { 0x6fa9f18c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc169f190, SUC_lexicon_tags_14 },
    { 0xa1e9f190, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd289f193, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9f199, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e9f1a5, SUC_lexicon_tags_11 },
    { 0xb269f1a6, SUC_lexicon_tags_15 },
    { 0x12e9f1a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca9f1aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4a89f1ac, SUC_lexicon_tags_11 },
    { 0xc649f1ad, SUC_lexicon_tags_7 },
    { 0xcda9f1ad, SUC_lexicon_tags_37 },
    { 0xf849f1af, SUC_lexicon_tags_23 },
    { 0xe349f1af, SUC_lexicon_tags_60 },
    { 0x1369f1af, SUC_lexicon_tags_40 },
    { 0x3729f1b2, SUC_lexicon_tags_19 },
    { 0xd729f1af, SUC_lexicon_tags_30 },
    { 0xbd89f1b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8409f1b7, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x9949f1b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd949f1bc, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9289f1c7, SUC_lexicon_tags_9 },
    { 0x7be9f1c8, SUC_lexicon_tags_63 },
    { 0xf29f1c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7949f1cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe449f1ce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1649f1d0, SUC_lexicon_tags_7 },
    { 0xcd09f1d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb69f1d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f29f1d6, SUC_lexicon_tags_35 },
    { 0x4229f1d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a09f1da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b89f1dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe9f1e0, SUC_lexicon_tags_18 },
    { 0xa349f1e1, SUC_lexicon_tags_11 },
    { 0x18a9f1e2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xef69f1e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489f1ea, SUC_lexicon_tags_65 },
    { 0x12a9f1ea, SUC_lexicon_tags_49 },
    { 0x75a9f1ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf2a9f1ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81a9f1f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6709f1f6, SUC_lexicon_tags_56 },
    { 0x24e9f1f7, SUC_lexicon_tags_18 },
    { 0x9609f1f8, SUC_lexicon_tags_47 },
    { 0x4fe9f1f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb89f1fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a09f206, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a9f20a, SUC_lexicon_tags_4 },
    { 0xc369f20a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e9f212, SUC_lexicon_tags_27 },
    { 0xfd09f213, SUC_lexicon_tags_37 },
    { 0xdda9f214, SUC_lexicon_tags_25 },
    { 0xe8e9f215, SUC_lexicon_tags_37 },
    { 0xc729f215, SUC_lexicon_tags_12 },
    { 0xed29f216, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf49f21a, SUC_lexicon_tags_7 },
    { 0x63c9f21b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2889f21e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2cc9f220, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc9f228, SUC_lexicon_tags_34 },
    { 0xe0e9f229, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4609f231, SUC_lexicon_tags_15 },
    { 0xe389f232, SUC_lexicon_tags_7 },
    { 0xf309f233, SUC_lexicon_tags_77 },
    { 0xf8e9f233, SUC_lexicon_tags_7 },
    { 0x1a69f235, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c89f239, SUC_lexicon_tags_25 },
    { 0xe629f23a, SUC_lexicon_tags_25 },
    { 0x65e9f23b, SUC_lexicon_tags_25 },
    { 0xc089f23a, SUC_lexicon_tags_2 },
    { 0xe3e9f23d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee9f240, SUC_lexicon_tags_48 },
    { 0x6329f241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac9f246, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa9f24d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29e9f251, SUC_lexicon_tags_37 },
    { 0x51e9f252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf449f254, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7969f257, SUC_lexicon_tags_6 },
    { 0x3fc9f258, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe749f25a, SUC_lexicon_tags_45 },
    { 0x5de9f25b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6a09f25d, SUC_lexicon_tags_15 },
    { 0x97c9f25e, SUC_lexicon_tags_25 },
    { 0x1c89f25d, SUC_lexicon_tags_38 },
    { 0xa489f25e, SUC_lexicon_tags_37 },
    { 0x8a9f261, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdae9f263, SUC_lexicon_tags_33 },
    { 0xb9a9f264, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad29f267, SUC_lexicon_tags_1 },
    { 0x37a9f268, SUC_lexicon_tags_6 },
    { 0x6449f269, SUC_lexicon_tags_25 },
    { 0x6ae9f269, SUC_lexicon_tags_26 },
    { 0x51e9f26b, SUC_lexicon_tags_11 },
    { 0xea09f26c, SUC_lexicon_tags_17 },
    { 0xcd29f26c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa49f26f, SUC_lexicon_tags_37 },
    { 0xd169f270, SUC_lexicon_tags_54 },
    { 0x4389f271, SUC_lexicon_tags_31 },
    { 0xf4a9f271, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf129f274, SUC_lexicon_tags_70 },
    { 0x8109f274, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869f279, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc9f27d, SUC_lexicon_tags_7 },
    { 0xa689f27e, SUC_lexicon_tags_43 },
    { 0x2cc9f27e, SUC_lexicon_tags_11 },
    { 0xe129f280, SUC_lexicon_tags_11 },
    { 0x4b89f281, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc69f284, SUC_lexicon_tags_65 },
    { 0x2729f284, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c9f28a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa789f28d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae9f297, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c9f29a, SUC_lexicon_tags_47 },
    { 0x5c29f29b, SUC_lexicon_tags_11 },
    { 0x6629f29c, SUC_lexicon_tags_25 },
    { 0x8da9f29c, SUC_lexicon_tags_43 },
    { 0x2989f29e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a69f2a4, SUC_lexicon_tags_33 },
    { 0xdea9f2a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e9f2aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb7e9f2ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc109f2ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de9f2b3, SUC_lexicon_tags_27 },
    { 0x45e9f2b4, SUC_lexicon_tags_11 },
    { 0x79a9f2b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1929f2b7, SUC_lexicon_tags_107 },
    { 0x7109f2b8, SUC_lexicon_tags_7 },
    { 0x30a9f2b9, SUC_lexicon_tags_33 },
    { 0x10e9f2ba, SUC_lexicon_tags_17 },
    { 0xb29f2ba, SUC_lexicon_tags_11 },
    { 0x369f2b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3169f2be, SUC_lexicon_tags_31 },
    { 0x8e69f2bf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5609f2c3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d29f2c6, SUC_lexicon_tags_17 },
    { 0x4469f2c7, SUC_lexicon_tags_1 },
    { 0x609f2c8, SUC_lexicon_tags_65 },
    { 0x10a9f2c8, SUC_lexicon_tags_7 },
    { 0x1d49f2c7, SUC_lexicon_tags_1 },
    { 0x6069f2c7, SUC_lexicon_tags_0 },
    { 0xbae9f2cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc689f2cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc729f2d2, SUC_lexicon_tags_11 },
    { 0xa5a9f2d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcea9f2d6, SUC_lexicon_tags_17 },
    { 0xd609f2d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69c9f2d9, SUC_lexicon_tags_33 },
    { 0xa209f2da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ba9f2dc, SUC_lexicon_tags_0 },
    { 0x9c69f2dd, SUC_lexicon_tags_1 },
    { 0x7e89f2dd, SUC_lexicon_tags_12 },
    { 0x65e9f2dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c49f2e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1349f2e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3049f2e9, SUC_lexicon_tags_29 },
    { 0xfae9f2e9, SUC_lexicon_tags_33 },
    { 0x7209f2e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1549f2f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee9f2f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e9f2ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75a9f301, SUC_lexicon_tags_6 },
    { 0x4b49f302, SUC_lexicon_tags_37 },
    { 0x3229f301, SUC_lexicon_tags_12 },
    { 0x2969f302, SUC_lexicon_tags_69 },
    { 0xa129f305, SUC_lexicon_tags_31 },
    { 0xf189f306, SUC_lexicon_tags_39 },
    { 0x1569f307, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee9f30a, SUC_lexicon_tags_1 },
    { 0xdee9f30b, SUC_lexicon_tags_27 },
    { 0xc4a9f30c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fa9f30f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe9f314, SUC_lexicon_tags_37 },
    { 0x75a9f314, SUC_lexicon_tags_12 },
    { 0xf5c9f316, SUC_lexicon_tags_1 },
    { 0x9b89f317, SUC_lexicon_tags_11 },
    { 0x749f318, SUC_lexicon_tags_14 },
    { 0xbae9f319, SUC_lexicon_tags_6 },
    { 0x1549f319, SUC_lexicon_tags_287 },
    { 0x8da9f318, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc549f31f, SUC_lexicon_tags_37 },
    { 0x6629f320, SUC_lexicon_tags_17 },
    { 0xbf29f320, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa889f323, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7549f325, SUC_lexicon_tags_17 },
    { 0x6e89f326, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b89f329, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7929f32c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa769f32f, SUC_lexicon_tags_15 },
    { 0x5489f32f, SUC_lexicon_tags_43 },
    { 0xd249f331, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa989f334, SUC_lexicon_tags_28 },
    { 0x61a9f335, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd249f337, SUC_lexicon_tags_30 },
    { 0x2229f338, SUC_lexicon_tags_53 },
    { 0xe6a9f339, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8449f33e, SUC_lexicon_tags_7 },
    { 0xc709f33f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xade9f341, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe7a9f343, SUC_lexicon_tags_2 },
    { 0x1489f344, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9f348, SUC_lexicon_tags_12 },
    { 0xbe69f348, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5609f350, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8509f354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6909f358, SUC_lexicon_tags_2 },
    { 0x4869f359, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9f35c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5929f35e, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x7c09f360, SUC_lexicon_tags_23 },
    { 0x309f361, SUC_lexicon_tags_13 },
    { 0x2509f361, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61a9f364, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd49f36f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8089f372, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x50a9f374, SUC_lexicon_tags_2 },
    { 0xa029f375, SUC_lexicon_tags_14 },
    { 0x2a9f375, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa529f37a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8989f37e, SUC_lexicon_tags_12 },
    { 0x6de9f37f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2349f382, SUC_lexicon_tags_34 },
    { 0x66c9f383, SUC_lexicon_tags_11 },
    { 0x1c29f384, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f89f388, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10a9f38b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca09f38f, SUC_lexicon_tags_60 },
    { 0x57a9f38f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x55e9f392, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3da9f394, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0xef09f396, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1449f399, SUC_lexicon_tags_12 },
    { 0x3c69f399, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x7789f39c, SUC_lexicon_tags_12 },
    { 0xca69f39d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf589f39f, SUC_lexicon_tags_87 },
    { 0x49f39f, SUC_lexicon_tags_900 },
    { 0x2dc9f3a0, SUC_lexicon_tags_96 },
    { 0x5f69f3a1, SUC_lexicon_tags_17 },
    { 0x5ac9f3a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfc9f3a5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x56e9f3a7, SUC_lexicon_tags_104 },
    { 0xf129f3a8, SUC_lexicon_tags_47 },
    { 0xca49f3a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca9f3ad, SUC_lexicon_tags_31 },
    { 0x2c09f3ae, SUC_lexicon_tags_17 },
    { 0xf369f3ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf09f3b1, SUC_lexicon_tags_68 },
    { 0x72a9f3b1, SUC_lexicon_tags_15 },
    { 0x2ba9f3b2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9269f3b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f89f3b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c9f3bf, SUC_lexicon_tags_7 },
    { 0xe5a9f3bf, SUC_lexicon_tags_60 },
    { 0x31e9f3c1, SUC_lexicon_tags_42 },
    { 0x5f89f3c2, SUC_lexicon_tags_12 },
    { 0x6069f3c3, SUC_lexicon_tags_81 },
    { 0x4489f3c4, SUC_lexicon_tags_80 },
    { 0x6f09f3c5, SUC_lexicon_tags_25 },
    { 0x8ba9f3c5, SUC_lexicon_tags_17 },
    { 0x669f3c2, SUC_lexicon_tags_12 },
    { 0x9d29f3c7, SUC_lexicon_tags_26 },
    { 0xd4e9f3c9, SUC_lexicon_tags_4 },
    { 0x6dc9f3c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a9f3ce, SUC_lexicon_tags_11 },
    { 0x6fe9f3ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6169f3d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd069f3d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9509f3d7, SUC_lexicon_tags_63 },
    { 0x5149f3d7, SUC_lexicon_tags_37 },
    { 0xa7c9f3d9, SUC_lexicon_tags_12 },
    { 0xf789f3da, SUC_lexicon_tags_87 },
    { 0x40a9f3db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e9f3de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ee9f3e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xda49f3e2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1329f3e4, SUC_lexicon_tags_2 },
    { 0xf0e9f3e4, SUC_lexicon_tags_7 },
    { 0xc29f3e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa869f3eb, SUC_lexicon_tags_72 },
    { 0x8bc9f3ec, SUC_lexicon_tags_17 },
    { 0x90c9f3ed, SUC_lexicon_tags_82 },
    { 0x1649f3eb, SUC_lexicon_tags_12 },
    { 0x4a09f3ef, SUC_lexicon_tags_30 },
    { 0x3bc9f3f0, SUC_lexicon_tags_57 },
    { 0xbde9f3f0, SUC_lexicon_tags_1 },
    { 0xdca9f3f2, SUC_lexicon_tags_11 },
    { 0x11c9f3ee, SUC_lexicon_tags_2 },
    { 0x80a9f3ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e9f3f9, SUC_lexicon_tags_27 },
    { 0xb069f3fa, SUC_lexicon_tags_2 },
    { 0x8489f3fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb409f3ff, SUC_lexicon_tags_0 },
    { 0xbf29f400, SUC_lexicon_tags_31 },
    { 0x4d69f3ff, SUC_lexicon_tags_10 },
    { 0xf729f402, SUC_lexicon_tags_29 },
    { 0xe9a9f403, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33e9f405, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe589f40a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6069f40c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x909f40e, SUC_lexicon_tags_12 },
    { 0xc7a9f40e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c9f412, SUC_lexicon_tags_14 },
    { 0xa689f413, SUC_lexicon_tags_11 },
    { 0xd189f413, SUC_lexicon_tags_45 },
    { 0x3049f415, SUC_lexicon_tags_15 },
    { 0x4429f413, SUC_lexicon_tags_0 },
    { 0xc9f412, SUC_lexicon_tags_17 },
    { 0xe609f418, SUC_lexicon_tags_9 },
    { 0x5989f419, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc9f41b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde9f41e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf149f423, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1269f427, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c9f42c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea49f42f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc109f431, SUC_lexicon_tags_197 },
    { 0x72e9f432, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669f438, SUC_lexicon_tags_37 },
    { 0x2b29f439, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3569f43e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x269f441, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5629f444, SUC_lexicon_tags_65 },
    { 0x8369f444, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce9f44c, SUC_lexicon_tags_148 },
    { 0x5009f44d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4849f454, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb549f45a, SUC_lexicon_tags_82 },
    { 0x7129f45b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa9f45f, SUC_lexicon_tags_11 },
    { 0x2de9f45f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9849f46b, SUC_lexicon_tags_63 },
    { 0xe1e9f46c, SUC_lexicon_tags_11 },
    { 0xb269f46d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba9f471, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9f476, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1649f478, SUC_lexicon_tags_25 },
    { 0x73e9f478, SUC_lexicon_tags_13 },
    { 0x1989f47a, SUC_lexicon_tags_38 },
    { 0x58a9f47a, SUC_lexicon_tags_23 },
    { 0x7589f478, SUC_lexicon_tags_36 },
    { 0xe649f47d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x48e9f480, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e9f483, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9609f488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b09f48b, SUC_lexicon_tags_65 },
    { 0x4aa9f48b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa9f492, SUC_lexicon_tags_61 },
    { 0xb889f493, SUC_lexicon_tags_43 },
    { 0x7349f494, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a9f49a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4cc9f49c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2b09f49e, SUC_lexicon_tags_29 },
    { 0xafc9f49f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8da9f4a1, SUC_lexicon_tags_103 },
    { 0xf749f4a2, SUC_lexicon_tags_60 },
    { 0xd4c9f4a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe9f4a5, SUC_lexicon_tags_11 },
    { 0xac49f4a6, SUC_lexicon_tags_28 },
    { 0x3529f4a7, SUC_lexicon_tags_15 },
    { 0x9609f4a6, SUC_lexicon_tags_6 },
    { 0x8189f4a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3fe9f4ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e9f4ae, SUC_lexicon_tags_27 },
    { 0x8649f4af, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5569f4b2, SUC_lexicon_tags_7 },
    { 0xb2a9f4b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee9f4b9, SUC_lexicon_tags_47 },
    { 0x4589f4ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be9f4bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a9f4c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2889f4c8, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1749f4cf, SUC_lexicon_tags_19 },
    { 0xf689f4d0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb009f4d6, SUC_lexicon_tags_49 },
    { 0x96e9f4d6, SUC_lexicon_tags_19 },
    { 0x2ac9f4d8, SUC_lexicon_tags_25 },
    { 0xcb69f4d9, SUC_lexicon_tags_83 },
    { 0x7769f4da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb969f4dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7269f4e2, SUC_lexicon_tags_14 },
    { 0x289f4e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa089f4e6, SUC_lexicon_tags_1 },
    { 0x46e9f4e7, SUC_lexicon_tags_37 },
    { 0x9309f4e8, SUC_lexicon_tags_17 },
    { 0x38c9f4e8, SUC_lexicon_tags_47 },
    { 0x8a89f4ea, SUC_lexicon_tags_11 },
    { 0xc069f4eb, SUC_lexicon_tags_43 },
    { 0xe969f4e9, SUC_lexicon_tags_56 },
    { 0xeea9f4ed, SUC_lexicon_tags_4 },
    { 0x1a69f4ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b09f4f1, SUC_lexicon_tags_1 },
    { 0x9f4f1, SUC_lexicon_tags_11 },
    { 0x3409f4f3, SUC_lexicon_tags_55 },
    { 0xf3a9f4f4, SUC_lexicon_tags_25 },
    { 0xcec9f4f4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd109f4fa, SUC_lexicon_tags_49 },
    { 0x6a89f4fb, SUC_lexicon_tags_63 },
    { 0x39a9f4fb, SUC_lexicon_tags_63 },
    { 0x9649f4fd, SUC_lexicon_tags_6 },
    { 0x27c9f4fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5a9f500, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf89f502, SUC_lexicon_tags_49 },
    { 0xe169f503, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbc9f505, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb89f50a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc329f50f, SUC_lexicon_tags_14 },
    { 0x5449f50f, SUC_lexicon_tags_63 },
    { 0xf6a9f511, SUC_lexicon_tags_2 },
    { 0x8349f511, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce9f516, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9769f519, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb29f51c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b29f522, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a9f525, SUC_lexicon_tags_63 },
    { 0xaa29f525, SUC_lexicon_tags_11 },
    { 0xfe69f526, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f69f529, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa969f52d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9889f52f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8329f534, SUC_lexicon_tags_0 },
    { 0x1f09f535, SUC_lexicon_tags_29 },
    { 0xa169f536, SUC_lexicon_tags_11 },
    { 0x1f69f537, SUC_lexicon_tags_66 },
    { 0x8789f537, SUC_lexicon_tags_32 },
    { 0xfe29f539, SUC_lexicon_tags_4 },
    { 0x2369f539, SUC_lexicon_tags_2 },
    { 0x6829f539, SUC_lexicon_tags_61 },
    { 0x13a9f536, SUC_lexicon_tags_137 },
    { 0xe369f53d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a9f546, SUC_lexicon_tags_47 },
    { 0x31c9f547, SUC_lexicon_tags_13 },
    { 0xeb69f548, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2369f54c, SUC_lexicon_tags_25 },
    { 0xe189f54c, SUC_lexicon_tags_12 },
    { 0xa5e9f54e, SUC_lexicon_tags_22 },
    { 0x4f69f54f, SUC_lexicon_tags_15 },
    { 0x56a9f550, SUC_lexicon_tags_11 },
    { 0xae49f550, SUC_lexicon_tags_55 },
    { 0x8dc9f54f, SUC_lexicon_tags_77 },
    { 0xe769f553, SUC_lexicon_tags_17 },
    { 0x5aa9f554, SUC_lexicon_tags_23 },
    { 0x5a9f554, SUC_lexicon_tags_12 },
    { 0x2949f550, SUC_lexicon_tags_31 },
    { 0x1d89f552, SUC_lexicon_tags_8 },
    { 0x36a9f552, SUC_lexicon_tags_15 },
    { 0xf0e9f559, SUC_lexicon_tags_63 },
    { 0x63c9f552, SUC_lexicon_tags_7 },
    { 0x3f69f553, SUC_lexicon_tags_28 },
    { 0x62a9f55c, SUC_lexicon_tags_11 },
    { 0x8f69f556, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xa789f55f, SUC_lexicon_tags_39 },
    { 0x1d29f560, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad69f562, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc929f565, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a9f568, SUC_lexicon_tags_7 },
    { 0xbaa9f569, SUC_lexicon_tags_45 },
    { 0xa109f569, SUC_lexicon_tags_25 },
    { 0x8ec9f56b, SUC_lexicon_tags_22 },
    { 0x289f569, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfae9f56f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba9f573, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf909f575, SUC_lexicon_tags_17 },
    { 0x4dc9f576, SUC_lexicon_tags_7 },
    { 0xb09f577, SUC_lexicon_tags_47 },
    { 0xd909f577, SUC_lexicon_tags_26 },
    { 0xd4c9f575, SUC_lexicon_tags_60 },
    { 0x6f89f57a, SUC_lexicon_tags_25 },
    { 0xa049f57a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8789f57d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d89f583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe849f585, SUC_lexicon_tags_29 },
    { 0x54e9f586, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x79a9f588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf249f58b, SUC_lexicon_tags_32 },
    { 0x7ba9f58c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4bc9f58e, SUC_lexicon_tags_7 },
    { 0x96e9f58e, SUC_lexicon_tags_49 },
    { 0x7509f58f, SUC_lexicon_tags_47 },
    { 0x91c9f590, SUC_lexicon_tags_65 },
    { 0xa309f592, SUC_lexicon_tags_29 },
    { 0x7b09f592, SUC_lexicon_tags_11 },
    { 0x2729f594, SUC_lexicon_tags_25 },
    { 0x42a9f594, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a9f598, SUC_lexicon_tags_71 },
    { 0x5789f599, SUC_lexicon_tags_31 },
    { 0x13a9f59a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa549f59c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a9f5ab, SUC_lexicon_tags_17 },
    { 0x14a9f5ac, SUC_lexicon_tags_11 },
    { 0x2d89f5ad, SUC_lexicon_tags_31 },
    { 0xb729f5ab, SUC_lexicon_tags_1 },
    { 0x3829f5ac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9e9f5b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x69a9f5b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f29f5b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a29f5ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e89f5bf, SUC_lexicon_tags_534 },
    { 0x71e9f5bf, SUC_lexicon_tags_17 },
    { 0x2e89f5c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xee49f5c4, SUC_lexicon_tags_14 },
    { 0x81c9f5c5, SUC_lexicon_tags_12 },
    { 0x13e9f5c6, SUC_lexicon_tags_7 },
    { 0x8ac9f5c7, SUC_lexicon_tags_23 },
    { 0xe269f5c8, SUC_lexicon_tags_22 },
    { 0xdcc9f5c6, SUC_lexicon_tags_27 },
    { 0xd089f5c8, SUC_lexicon_tags_1 },
    { 0x80c9f5cb, SUC_lexicon_tags_0 },
    { 0x5d29f5cc, SUC_lexicon_tags_37 },
    { 0xba09f5cc, SUC_lexicon_tags_9 },
    { 0x1dc9f5c9, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xd369f5d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5889f5d6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1709f5d9, SUC_lexicon_tags_79 },
    { 0xc449f5da, SUC_lexicon_tags_37 },
    { 0xaac9f5db, SUC_lexicon_tags_12 },
    { 0x29c9f5db, SUC_lexicon_tags_17 },
    { 0x3449f5db, SUC_lexicon_tags_17 },
    { 0xd549f5db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd189f5e0, SUC_lexicon_tags_0 },
    { 0x4909f5e0, SUC_lexicon_tags_1 },
    { 0x9569f5e0, SUC_lexicon_tags_85 },
    { 0xd4a9f5e3, SUC_lexicon_tags_26 },
    { 0xb0a9f5e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68e9f5e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3449f5eb, SUC_lexicon_tags_14 },
    { 0x9889f5ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xce09f5ee, SUC_lexicon_tags_11 },
    { 0x2ee9f5ee, SUC_lexicon_tags_19 },
    { 0x17a9f5ef, SUC_lexicon_tags_38 },
    { 0x5fa9f5f1, SUC_lexicon_tags_31 },
    { 0xfb09f5f1, SUC_lexicon_tags_42 },
    { 0x2e09f5f1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe209f5fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8ec9f600, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c9f604, SUC_lexicon_tags_17 },
    { 0x12e9f604, SUC_lexicon_tags_6 },
    { 0xb889f606, SUC_lexicon_tags_37 },
    { 0xd1e9f605, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f09f60a, SUC_lexicon_tags_19 },
    { 0xa289f60b, SUC_lexicon_tags_138 },
    { 0xd349f60c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1529f611, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa169f619, SUC_lexicon_tags_26 },
    { 0x52a9f61a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45a9f61c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9f61f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4349f622, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e9f625, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c9f628, SUC_lexicon_tags_37 },
    { 0x5989f628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c09f62c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc5c9f62e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c29f636, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc489f63a, SUC_lexicon_tags_31 },
    { 0x82a9f63b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e9f643, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe409f646, SUC_lexicon_tags_65 },
    { 0xaf89f646, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b89f64b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4709f64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bc9f64f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9669f655, SUC_lexicon_tags_25 },
    { 0x3669f656, SUC_lexicon_tags_28 },
    { 0x41e9f657, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c89f65a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6609f660, SUC_lexicon_tags_2 },
    { 0x19a9f661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc9f664, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e9f66a, SUC_lexicon_tags_37 },
    { 0x7c69f66b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6269f66e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe349f671, SUC_lexicon_tags_17 },
    { 0x6389f672, SUC_lexicon_tags_6 },
    { 0x24a9f673, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda9f677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c9f67c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a9f681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc09f684, SUC_lexicon_tags_6 },
    { 0xdde9f685, SUC_lexicon_tags_151 },
    { 0x4069f686, SUC_lexicon_tags_8 },
    { 0x7809f687, SUC_lexicon_tags_43 },
    { 0xed49f688, SUC_lexicon_tags_1061 },
    { 0x3489f684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a9f68c, SUC_lexicon_tags_28 },
    { 0x2309f68c, SUC_lexicon_tags_11 },
    { 0x1b49f68d, SUC_lexicon_tags_9 },
    { 0xfe89f68f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe0c9f691, SUC_lexicon_tags_56 },
    { 0xaca9f691, SUC_lexicon_tags_24 },
    { 0xc1a9f693, SUC_lexicon_tags_33 },
    { 0x9c69f694, SUC_lexicon_tags_39 },
    { 0x8e29f695, SUC_lexicon_tags_1 },
    { 0xa29f696, SUC_lexicon_tags_8 },
    { 0xa209f697, SUC_lexicon_tags_82 },
    { 0xfb49f695, SUC_lexicon_tags_2 },
    { 0x1889f694, SUC_lexicon_tags_11 },
    { 0x81a9f69a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9f69e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6869f6ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ae9f6af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb249f6b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a9f6bb, SUC_lexicon_tags_7 },
    { 0x15e9f6bc, SUC_lexicon_tags_12 },
    { 0x9ca9f6bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30c9f6bf, SUC_lexicon_tags_8 },
    { 0x8909f6bf, SUC_lexicon_tags_28 },
    { 0x6f89f6c1, SUC_lexicon_tags_25 },
    { 0xa309f6c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeba9f6c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9c69f6c9, SUC_lexicon_tags_42 },
    { 0x9f29f6ca, SUC_lexicon_tags_17 },
    { 0x4b09f6cb, SUC_lexicon_tags_37 },
    { 0x4e69f6cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5669f6d0, SUC_lexicon_tags_47 },
    { 0x9e49f6d1, SUC_lexicon_tags_84 },
    { 0xc829f6d2, SUC_lexicon_tags_48 },
    { 0x10a9f6d0, SUC_lexicon_tags_15 },
    { 0x2f89f6d2, SUC_lexicon_tags_14 },
    { 0x2069f6d5, SUC_lexicon_tags_62 },
    { 0x7b89f6d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e9f6dd, SUC_lexicon_tags_47 },
    { 0x16e9f6de, SUC_lexicon_tags_29 },
    { 0xe129f6df, SUC_lexicon_tags_54 },
    { 0x4109f6e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7889f6e2, SUC_lexicon_tags_47 },
    { 0x9e29f6e2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a9f6e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c9f6ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a09f6ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49f6f6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c9f6f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57e9f6fb, SUC_lexicon_tags_47 },
    { 0x12e9f6fb, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa509f702, SUC_lexicon_tags_7 },
    { 0x1809f702, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb049f70c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xea9f70e, SUC_lexicon_tags_38 },
    { 0xe889f70f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa9f712, SUC_lexicon_tags_37 },
    { 0x8f69f713, SUC_lexicon_tags_1 },
    { 0xede9f714, SUC_lexicon_tags_37 },
    { 0xbae9f714, SUC_lexicon_tags_17 },
    { 0xbee9f712, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e29f718, SUC_lexicon_tags_37 },
    { 0xd089f719, SUC_lexicon_tags_25 },
    { 0x5929f71a, SUC_lexicon_tags_55 },
    { 0xab69f71b, SUC_lexicon_tags_12 },
    { 0x5b49f71c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d09f71e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda9f723, SUC_lexicon_tags_37 },
    { 0x6a29f724, SUC_lexicon_tags_25 },
    { 0xac49f724, SUC_lexicon_tags_17 },
    { 0x2e89f723, SUC_lexicon_tags_11 },
    { 0x6289f727, SUC_lexicon_tags_12 },
    { 0x1ba9f727, SUC_lexicon_tags_47 },
    { 0x9fa9f723, SUC_lexicon_tags_27 },
    { 0xc6c9f72a, SUC_lexicon_tags_17 },
    { 0x1249f729, SUC_lexicon_tags_49 },
    { 0xce29f726, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x529f72f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2bc9f731, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b89f733, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a9f737, SUC_lexicon_tags_92 },
    { 0x9fc9f737, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb749f73a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7889f745, SUC_lexicon_tags_45 },
    { 0x61a9f745, SUC_lexicon_tags_11 },
    { 0xc5e9f745, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c49f74a, SUC_lexicon_tags_36 },
    { 0xbd49f74b, SUC_lexicon_tags_11 },
    { 0xde29f74b, SUC_lexicon_tags_1 },
    { 0xcf69f74b, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0xa869f750, SUC_lexicon_tags_45 },
    { 0x8829f750, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b29f755, SUC_lexicon_tags_33 },
    { 0x4869f756, SUC_lexicon_tags_25 },
    { 0xd349f757, SUC_lexicon_tags_0 },
    { 0x3169f756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8a9f75a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce9f76a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5489f772, SUC_lexicon_tags_0 },
    { 0xa9f773, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a9f777, SUC_lexicon_tags_12 },
    { 0xc949f778, SUC_lexicon_tags_23 },
    { 0xab49f779, SUC_lexicon_tags_15 },
    { 0xf9e9f777, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcee9f77c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd6a9f77e, SUC_lexicon_tags_27 },
    { 0x1a29f77f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9469f784, SUC_lexicon_tags_25 },
    { 0xcec9f784, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8649f789, SUC_lexicon_tags_17 },
    { 0x8249f78a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc189f78e, SUC_lexicon_tags_37 },
    { 0xdb09f78e, SUC_lexicon_tags_3 },
    { 0x7cc9f790, SUC_lexicon_tags_27 },
    { 0x4969f790, SUC_lexicon_tags_49 },
    { 0xb709f792, SUC_lexicon_tags_47 },
    { 0x2929f793, SUC_lexicon_tags_25 },
    { 0x9469f793, SUC_lexicon_tags_11 },
    { 0x2589f794, SUC_lexicon_tags_23 },
    { 0xa7e9f793, SUC_lexicon_tags_25 },
    { 0xc3e9f796, SUC_lexicon_tags_30 },
    { 0xe189f796, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f69f79f, SUC_lexicon_tags_11 },
    { 0xf329f7a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d49f7a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c69f7a6, SUC_lexicon_tags_31 },
    { 0x1289f7a7, SUC_lexicon_tags_11 },
    { 0x8ba9f7a8, SUC_lexicon_tags_27 },
    { 0xd049f7a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4689f7ac, SUC_lexicon_tags_25 },
    { 0xde9f7ad, SUC_lexicon_tags_6 },
    { 0xb5c9f7ad, SUC_lexicon_tags_46 },
    { 0x4c09f7af, SUC_lexicon_tags_45 },
    { 0x47a9f7b0, SUC_lexicon_tags_9 },
    { 0x33c9f7b1, SUC_lexicon_tags_3 },
    { 0xe509f7b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf229f7ba, SUC_lexicon_tags_37 },
    { 0x7069f7ba, SUC_lexicon_tags_17 },
    { 0xbbe9f7ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3049f7bf, SUC_lexicon_tags_65 },
    { 0x2649f7bf, SUC_lexicon_tags_392 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69f7c6, SUC_lexicon_tags_19 },
    { 0x7769f7c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ee9f7c9, SUC_lexicon_tags_11 },
    { 0x8e29f7c9, SUC_lexicon_tags_49 },
    { 0xb009f7c9, SUC_lexicon_tags_23 },
    { 0xc969f7cc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xda49f7cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c29f7d4, SUC_lexicon_tags_60 },
    { 0xe129f7d5, SUC_lexicon_tags_15 },
    { 0x3d29f7d5, SUC_lexicon_tags_25 },
    { 0x9789f7d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd129f7db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7c29f7dd, SUC_lexicon_tags_23 },
    { 0xa309f7de, SUC_lexicon_tags_12 },
    { 0x2e09f7df, SUC_lexicon_tags_61 },
    { 0x549f7e0, SUC_lexicon_tags_25 },
    { 0x2c89f7e1, SUC_lexicon_tags_1 },
    { 0x5329f7df, SUC_lexicon_tags_12 },
    { 0x809f7e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c09f7e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c9f7e8, SUC_lexicon_tags_11 },
    { 0x8a89f7e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc329f7eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9389f7ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf49f7f6, SUC_lexicon_tags_15 },
    { 0x4bc9f7f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c9f7fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a9f7fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x649f7ff, SUC_lexicon_tags_28 },
    { 0x6689f800, SUC_lexicon_tags_17 },
    { 0xdea9f7ff, SUC_lexicon_tags_43 },
    { 0xad09f7ff, SUC_lexicon_tags_11 },
    { 0x73c9f803, SUC_lexicon_tags_15 },
    { 0x35e9f804, SUC_lexicon_tags_19 },
    { 0xe8e9f805, SUC_lexicon_tags_25 },
    { 0xd949f805, SUC_lexicon_tags_10 },
    { 0x89e9f804, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b89f809, SUC_lexicon_tags_43 },
    { 0x5ea9f809, SUC_lexicon_tags_34 },
    { 0x7f69f809, SUC_lexicon_tags_1 },
    { 0x40e9f80b, SUC_lexicon_tags_37 },
    { 0x5cc9f809, SUC_lexicon_tags_63 },
    { 0xe3a9f80d, SUC_lexicon_tags_43 },
    { 0xd6a9f809, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c89f813, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c49f821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1169f823, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe49f825, SUC_lexicon_tags_28 },
    { 0xf849f825, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7669f829, SUC_lexicon_tags_11 },
    { 0x49a9f82a, SUC_lexicon_tags_29 },
    { 0x8469f82b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7129f82d, SUC_lexicon_tags_6 },
    { 0x23e9f82e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd89f830, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c69f833, SUC_lexicon_tags_12 },
    { 0x2f69f834, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf49f837, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f29f83b, SUC_lexicon_tags_43 },
    { 0xfb69f83c, SUC_lexicon_tags_7 },
    { 0xde09f83b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c9f843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3009f846, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f84a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf809f84c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6949f84e, SUC_lexicon_tags_25 },
    { 0xe809f84e, SUC_lexicon_tags_87 },
    { 0xec69f84f, SUC_lexicon_tags_87 },
    { 0x8f09f851, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9349f853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e9f855, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85e9f858, SUC_lexicon_tags_49 },
    { 0x18a9f859, SUC_lexicon_tags_14 },
    { 0xc569f859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa49f85c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309f861, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47a9f865, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5929f868, SUC_lexicon_tags_38 },
    { 0xc449f869, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5f29f86b, SUC_lexicon_tags_45 },
    { 0x8a69f86c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9289f86f, SUC_lexicon_tags_15 },
    { 0xcf69f86f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x17a9f872, SUC_lexicon_tags_15 },
    { 0xfb29f872, SUC_lexicon_tags_33 },
    { 0x4709f874, SUC_lexicon_tags_12 },
    { 0x9b09f875, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf769f87a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1889f880, SUC_lexicon_tags_37 },
    { 0xb9e9f881, SUC_lexicon_tags_29 },
    { 0x1ea9f881, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa749f885, SUC_lexicon_tags_33 },
    { 0x2e29f885, SUC_lexicon_tags_33 },
    { 0xfce9f885, SUC_lexicon_tags_66 },
    { 0xfe09f888, SUC_lexicon_tags_15 },
    { 0x4869f889, SUC_lexicon_tags_19 },
    { 0x4f69f886, SUC_lexicon_tags_37 },
    { 0x8f29f888, SUC_lexicon_tags_47 },
    { 0x3cc9f88c, SUC_lexicon_tags_35 },
    { 0x2d09f88c, SUC_lexicon_tags_10 },
    { 0xa189f887, SUC_lexicon_tags_23 },
    { 0xe1e9f88d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf389f894, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a9f899, SUC_lexicon_tags_37 },
    { 0x3969f899, SUC_lexicon_tags_2 },
    { 0x3589f899, SUC_lexicon_tags_15 },
    { 0x7e09f899, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a9f8a0, SUC_lexicon_tags_37 },
    { 0xa2e9f8a1, SUC_lexicon_tags_2 },
    { 0x7629f8a2, SUC_lexicon_tags_11 },
    { 0x64a9f8a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1e9f8a6, SUC_lexicon_tags_18 },
    { 0xc149f8a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb509f8a9, SUC_lexicon_tags_14 },
    { 0x1309f8a9, SUC_lexicon_tags_27 },
    { 0x8569f8a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c49f8ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x69e9f8b0, SUC_lexicon_tags_36 },
    { 0x72e9f8b0, SUC_lexicon_tags_7 },
    { 0x4609f8b2, SUC_lexicon_tags_19 },
    { 0xfb49f8b3, SUC_lexicon_tags_17 },
    { 0xfae9f8b4, SUC_lexicon_tags_14 },
    { 0x5649f8b5, SUC_lexicon_tags_29 },
    { 0x95e9f8b6, SUC_lexicon_tags_1 },
    { 0x6a89f8b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb109f8bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c9f8c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a09f8c2, SUC_lexicon_tags_0 },
    { 0xf789f8c3, SUC_lexicon_tags_7 },
    { 0xb8a9f8c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4469f8ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc9f8cd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689f8d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc69f8db, SUC_lexicon_tags_19 },
    { 0x60c9f8dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2429f8de, SUC_lexicon_tags_7 },
    { 0xf629f8df, SUC_lexicon_tags_10 },
    { 0xd109f8e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a9f8ea, SUC_lexicon_tags_7 },
    { 0xd7c9f8ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae9f8ee, SUC_lexicon_tags_140 },
    { 0xf6a9f8ef, SUC_lexicon_tags_43 },
    { 0xcba9f8ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb309f8f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa29f8f5, SUC_lexicon_tags_6 },
    { 0xa309f8f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x86c9f8f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd09f8fb, SUC_lexicon_tags_23 },
    { 0xba09f8fc, SUC_lexicon_tags_14 },
    { 0x2269f8fb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8989f8ff, SUC_lexicon_tags_11 },
    { 0xf369f900, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa749f904, SUC_lexicon_tags_10 },
    { 0xb209f905, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfac9f90c, SUC_lexicon_tags_1 },
    { 0x1729f90d, SUC_lexicon_tags_14 },
    { 0x83e9f90c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c9f914, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5569f917, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xedc9f919, SUC_lexicon_tags_25 },
    { 0xf069f91a, SUC_lexicon_tags_6 },
    { 0x49c9f91b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa669f91f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3ac9f921, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec9f927, SUC_lexicon_tags_1 },
    { 0x4869f928, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a9f92c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a9f938, SUC_lexicon_tags_70 },
    { 0x84a9f939, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2929f93e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5dc9f940, SUC_lexicon_tags_19 },
    { 0xebc9f941, SUC_lexicon_tags_60 },
    { 0x9e49f942, SUC_lexicon_tags_1 },
    { 0x41e9f943, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c9f948, SUC_lexicon_tags_19 },
    { 0x2b89f948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c9f94d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d49f94f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea9f956, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c9f95b, SUC_lexicon_tags_1 },
    { 0x1369f95c, SUC_lexicon_tags_27 },
    { 0x4209f95d, SUC_lexicon_tags_30 },
    { 0x4c89f95e, SUC_lexicon_tags_3 },
    { 0x7e9f95d, SUC_lexicon_tags_37 },
    { 0xc8a9f960, SUC_lexicon_tags_23 },
    { 0xdde9f95f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba9f968, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc9f96d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee9f973, SUC_lexicon_tags_23 },
    { 0x4189f974, SUC_lexicon_tags_37 },
    { 0x8a09f975, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e69f978, SUC_lexicon_tags_11 },
    { 0xd09f979, SUC_lexicon_tags_57 },
    { 0x13c9f978, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc9a9f97c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda9f97f, SUC_lexicon_tags_4 },
    { 0x7169f980, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c9f983, SUC_lexicon_tags_11 },
    { 0x9d09f983, SUC_lexicon_tags_13 },
    { 0x17e9f985, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25c9f989, SUC_lexicon_tags_12 },
    { 0x5969f989, SUC_lexicon_tags_17 },
    { 0x8d49f98b, SUC_lexicon_tags_25 },
    { 0xd169f989, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bc9f990, SUC_lexicon_tags_17 },
    { 0x9949f991, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfc69f993, SUC_lexicon_tags_38 },
    { 0xdec9f994, SUC_lexicon_tags_7 },
    { 0x9f49f993, SUC_lexicon_tags_1 },
    { 0xeb09f994, SUC_lexicon_tags_14 },
    { 0x3ba9f997, SUC_lexicon_tags_23 },
    { 0x2c09f994, SUC_lexicon_tags_15 },
    { 0xc709f998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc969f99c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3729f99e, SUC_lexicon_tags_45 },
    { 0x50a9f99e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5c9f9a1, SUC_lexicon_tags_38 },
    { 0x9529f9a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe5c9f9a4, SUC_lexicon_tags_37 },
    { 0xe0c9f9a5, SUC_lexicon_tags_37 },
    { 0x8629f9a5, SUC_lexicon_tags_37 },
    { 0xfe9f9a5, SUC_lexicon_tags_60 },
    { 0xf4a9f9a8, SUC_lexicon_tags_49 },
    { 0xbca9f9a9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc2a9f9ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1469f9ad, SUC_lexicon_tags_12 },
    { 0x46c9f9ae, SUC_lexicon_tags_17 },
    { 0xd8a9f9ae, SUC_lexicon_tags_38 },
    { 0x2ea9f9b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd049f9b4, SUC_lexicon_tags_35 },
    { 0x20a9f9b5, SUC_lexicon_tags_37 },
    { 0xaf69f9b6, SUC_lexicon_tags_10 },
    { 0x2c89f9b7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1309f9ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf649f9bc, SUC_lexicon_tags_38 },
    { 0x4889f9bd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5989f9c3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xece9f9c5, SUC_lexicon_tags_38 },
    { 0xc5c9f9c6, SUC_lexicon_tags_26 },
    { 0xf949f9c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5369f9c9, SUC_lexicon_tags_45 },
    { 0xac89f9c9, SUC_lexicon_tags_47 },
    { 0xdbe9f9cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa9f9cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfe9f9d1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd8c9f9d3, SUC_lexicon_tags_77 },
    { 0xdc29f9d4, SUC_lexicon_tags_29 },
    { 0xeb09f9d4, SUC_lexicon_tags_55 },
    { 0xaf49f9d5, SUC_lexicon_tags_7 },
    { 0xc6e9f9d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2409f9d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7c9f9db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd009f9dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c29f9e2, SUC_lexicon_tags_5 },
    { 0xbb89f9e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e9f9eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdbc9f9ed, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fe9f9f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa9f9f4, SUC_lexicon_tags_62 },
    { 0xc89f9f5, SUC_lexicon_tags_70 },
    { 0xdd29f9f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc9f9fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac9fa01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4909fa04, SUC_lexicon_tags_68 },
    { 0x4229fa05, SUC_lexicon_tags_23 },
    { 0xade9fa06, SUC_lexicon_tags_11 },
    { 0x3629fa05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd29fa0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a9fa12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29fa16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc929fa19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc9fa1c, SUC_lexicon_tags_33 },
    { 0xbba9fa1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3849fa20, SUC_lexicon_tags_63 },
    { 0x9ac9fa21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d49fa25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e9fa2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa2a9fa2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea69fa31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c49fa33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2769fa39, SUC_lexicon_tags_49 },
    { 0x5b89fa39, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70e9fa3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc049fa44, SUC_lexicon_tags_25 },
    { 0x47e9fa45, SUC_lexicon_tags_98 },
    { 0x3929fa46, SUC_lexicon_tags_56 },
    { 0xc7c9fa46, SUC_lexicon_tags_68 },
    { 0x3d49fa48, SUC_lexicon_tags_45 },
    { 0xaa69fa49, SUC_lexicon_tags_27 },
    { 0x72c9fa48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1709fa4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa509fa53, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2109fa56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a29fa5b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3329fa5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9469fa62, SUC_lexicon_tags_138 },
    { 0x6e69fa63, SUC_lexicon_tags_7 },
    { 0xdb49fa64, SUC_lexicon_tags_1 },
    { 0x7789fa65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3da9fa67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8e9fa69, SUC_lexicon_tags_17 },
    { 0xc4e9fa69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a69fa75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf929fa7a, SUC_lexicon_tags_12 },
    { 0x3809fa7b, SUC_lexicon_tags_63 },
    { 0x6349fa7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5469fa80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce9fa84, SUC_lexicon_tags_62 },
    { 0x7709fa85, SUC_lexicon_tags_7 },
    { 0xdec9fa84, SUC_lexicon_tags_29 },
    { 0xbe49fa84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3609fa89, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c9fa8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c9fa8f, SUC_lexicon_tags_17 },
    { 0x1029fa90, SUC_lexicon_tags_4 },
    { 0x2c89fa91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xada9fa93, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c49fa96, SUC_lexicon_tags_28 },
    { 0xde9fa97, SUC_lexicon_tags_49 },
    { 0x9729fa98, SUC_lexicon_tags_17 },
    { 0x6089fa99, SUC_lexicon_tags_7 },
    { 0x2a89fa97, SUC_lexicon_tags_17 },
    { 0x2a89fa98, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee29faa0, SUC_lexicon_tags_2 },
    { 0x4b49faa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a9faa6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d29faaa, SUC_lexicon_tags_34 },
    { 0xc569faab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd169faae, SUC_lexicon_tags_1 },
    { 0x5cc9faae, SUC_lexicon_tags_7 },
    { 0xeca9faaf, SUC_lexicon_tags_29 },
    { 0xd849fab1, SUC_lexicon_tags_37 },
    { 0xb0a9fab1, SUC_lexicon_tags_15 },
    { 0x3f29fab3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5e89fab5, SUC_lexicon_tags_30 },
    { 0x2869fab5, SUC_lexicon_tags_65 },
    { 0x10e9fab7, SUC_lexicon_tags_2 },
    { 0x2509fab7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d29fac0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x71c9fac3, SUC_lexicon_tags_9 },
    { 0x8009fac4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3a9fac6, SUC_lexicon_tags_6 },
    { 0x1ea9fac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd89fac9, SUC_lexicon_tags_77 },
    { 0x37a9faca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da9fad0, SUC_lexicon_tags_13 },
    { 0xa1e9fad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8029fad4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8a49fad6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3229fadc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc9fadf, SUC_lexicon_tags_2 },
    { 0xdde9fadf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7229fae3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209fae8, SUC_lexicon_tags_9 },
    { 0x5009fae8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe709faed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x26a9faef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15a9faf1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f89faf5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb6a9faf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd109fafb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74a9fafd, SUC_lexicon_tags_12 },
    { 0x8489fafe, SUC_lexicon_tags_19 },
    { 0x5349fafe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf189fb01, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb349fb05, SUC_lexicon_tags_14 },
    { 0xaa49fb06, SUC_lexicon_tags_31 },
    { 0x99a9fb07, SUC_lexicon_tags_79 },
    { 0x3409fb07, SUC_lexicon_tags_7 },
    { 0x1229fb07, SUC_lexicon_tags_23 },
    { 0x4e09fb0a, SUC_lexicon_tags_96 },
    { 0x30c9fb0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c9fb10, SUC_lexicon_tags_37 },
    { 0x7309fb11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e9fb16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a9fb19, SUC_lexicon_tags_16 },
    { 0x8909fb19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc2e9fb1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a69fb1e, SUC_lexicon_tags_25 },
    { 0x7649fb1f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a9fb24, SUC_lexicon_tags_13 },
    { 0xde89fb24, SUC_lexicon_tags_2 },
    { 0xc389fb26, SUC_lexicon_tags_17 },
    { 0xace9fb24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e09fb29, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde9fb2c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa9fb34, SUC_lexicon_tags_8 },
    { 0x89fb34, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2ba9fb37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa9fb3c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a9fb3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae9fb43, SUC_lexicon_tags_7 },
    { 0xb1a9fb44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9ba9fb46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c49fb49, SUC_lexicon_tags_31 },
    { 0x69a9fb4a, SUC_lexicon_tags_60 },
    { 0x5829fb4a, SUC_lexicon_tags_79 },
    { 0xf569fb4c, SUC_lexicon_tags_17 },
    { 0xccc9fb4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xede9fb50, SUC_lexicon_tags_30 },
    { 0x4f49fb51, SUC_lexicon_tags_8 },
    { 0x9409fb52, SUC_lexicon_tags_16 },
    { 0xd289fb52, SUC_lexicon_tags_31 },
    { 0xe169fb51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e69fb59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6349fb5d, SUC_lexicon_tags_37 },
    { 0x6c49fb5e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c29fb64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0e9fb66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9229fb68, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xeee9fb6a, SUC_lexicon_tags_29 },
    { 0x60a9fb6b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4c29fb6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729fb71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9fb78, SUC_lexicon_tags_8 },
    { 0x2cc9fb78, SUC_lexicon_tags_1 },
    { 0x5d69fb7a, SUC_lexicon_tags_1 },
    { 0x4fc9fb78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf49fb81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2029fb83, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad09fb87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc8c9fb89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9f09fb8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11c9fb8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae09fb8f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be9fb94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x29fb96, SUC_lexicon_tags_37 },
    { 0xb169fb97, SUC_lexicon_tags_0 },
    { 0x1249fb98, SUC_lexicon_tags_14 },
    { 0xdf69fb99, SUC_lexicon_tags_7 },
    { 0x569fb9a, SUC_lexicon_tags_10 },
    { 0x8bc9fb9b, SUC_lexicon_tags_32 },
    { 0xa5c9fb9c, SUC_lexicon_tags_46 },
    { 0xb649fb9c, SUC_lexicon_tags_63 },
    { 0xfa69fb9e, SUC_lexicon_tags_11 },
    { 0xba09fb96, SUC_lexicon_tags_47 },
    { 0x42c9fba0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3649fba2, SUC_lexicon_tags_1 },
    { 0x7329fba3, SUC_lexicon_tags_12 },
    { 0xabe9fba4, SUC_lexicon_tags_12 },
    { 0xcac9fba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5469fba7, SUC_lexicon_tags_7 },
    { 0x5609fba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3449fbae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7669fbb0, SUC_lexicon_tags_23 },
    { 0x73a9fbb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e9fbb5, SUC_lexicon_tags_33 },
    { 0xf689fbb6, SUC_lexicon_tags_87 },
    { 0x1789fbb7, SUC_lexicon_tags_37 },
    { 0xb729fbb8, SUC_lexicon_tags_43 },
    { 0x289fbb9, SUC_lexicon_tags_8 },
    { 0xdd49fbba, SUC_lexicon_tags_38 },
    { 0xf089fbbb, SUC_lexicon_tags_43 },
    { 0x5909fbbc, SUC_lexicon_tags_39 },
    { 0x5c9fbbb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x89a9fbbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a69fbc4, SUC_lexicon_tags_43 },
    { 0xc029fbc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229fbd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa29fbd5, SUC_lexicon_tags_28 },
    { 0xbe29fbd5, SUC_lexicon_tags_17 },
    { 0x4fc9fbd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4789fbdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa009fbe5, SUC_lexicon_tags_1 },
    { 0x39a9fbe6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c9fbe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729fbed, SUC_lexicon_tags_16 },
    { 0xc3a9fbee, SUC_lexicon_tags_4 },
    { 0x7e69fbef, SUC_lexicon_tags_12 },
    { 0x6f29fbed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc9fbf3, SUC_lexicon_tags_14 },
    { 0x9949fbf3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b49fbf9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8329fbfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e9fc04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2349fc08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x79e9fc0b, SUC_lexicon_tags_37 },
    { 0x8869fc0b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3189fc11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7809fc13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a9fc17, SUC_lexicon_tags_31 },
    { 0x9149fc17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc9fc22, SUC_lexicon_tags_53 },
    { 0xaf09fc23, SUC_lexicon_tags_12 },
    { 0x7769fc24, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe509fc29, SUC_lexicon_tags_25 },
    { 0x3069fc29, SUC_lexicon_tags_43 },
    { 0xe9fc2b, SUC_lexicon_tags_23 },
    { 0x2c89fc2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5049fc2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfae9fc30, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x749fc32, SUC_lexicon_tags_60 },
    { 0x1989fc33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x35a9fc36, SUC_lexicon_tags_43 },
    { 0x2709fc36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c69fc3c, SUC_lexicon_tags_25 },
    { 0x8449fc3d, SUC_lexicon_tags_26 },
    { 0x9ba9fc3e, SUC_lexicon_tags_92 },
    { 0x56a9fc3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88a9fc41, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac09fc43, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d09fc48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe949fc4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a9fc50, SUC_lexicon_tags_68 },
    { 0x8c29fc51, SUC_lexicon_tags_29 },
    { 0xcb29fc51, SUC_lexicon_tags_4 },
    { 0x8fa9fc53, SUC_lexicon_tags_37 },
    { 0x5ca9fc51, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeac9fc56, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3c29fc58, SUC_lexicon_tags_1 },
    { 0x649fc59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7989fc5b, SUC_lexicon_tags_17 },
    { 0x8849fc5c, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x8ac9fc5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8349fc60, SUC_lexicon_tags_16 },
    { 0x529fc61, SUC_lexicon_tags_17 },
    { 0x2ee9fc62, SUC_lexicon_tags_38 },
    { 0xdc09fc60, SUC_lexicon_tags_12 },
    { 0x39a9fc64, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e9fc68, SUC_lexicon_tags_37 },
    { 0x4b49fc68, SUC_lexicon_tags_10 },
    { 0x6029fc6a, SUC_lexicon_tags_11 },
    { 0xd1c9fc6b, SUC_lexicon_tags_66 },
    { 0xdc29fc68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99e9fc6e, SUC_lexicon_tags_26 },
    { 0x6ac9fc6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a89fc78, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd949fc7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509fc7f, SUC_lexicon_tags_38 },
    { 0xe469fc7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a69fc85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa689fc8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e9fc93, SUC_lexicon_tags_45 },
    { 0x2c9fc94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd089fc9a, SUC_lexicon_tags_37 },
    { 0x7189fc9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4649fc9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1849fca0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbda9fca3, SUC_lexicon_tags_0 },
    { 0xee09fca4, SUC_lexicon_tags_83 },
    { 0x8f09fca4, SUC_lexicon_tags_25 },
    { 0x789fca6, SUC_lexicon_tags_37 },
    { 0x34c9fca7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a9fca9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5849fcae, SUC_lexicon_tags_16 },
    { 0x3a9fcae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2be9fcb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd849fcb6, SUC_lexicon_tags_11 },
    { 0x1de9fcb7, SUC_lexicon_tags_1 },
    { 0xbc9fcb8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xc349fcba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf509fcbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b69fcc1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe1c9fcc3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x449fcc5, SUC_lexicon_tags_15 },
    { 0xfde9fcc6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9be9fcc8, SUC_lexicon_tags_37 },
    { 0x2749fcc8, SUC_lexicon_tags_140 },
    { 0xb989fcc9, SUC_lexicon_tags_7 },
    { 0xc6e9fccb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9649fccf, SUC_lexicon_tags_13 },
    { 0xac09fcd0, SUC_lexicon_tags_79 },
    { 0xd869fcd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9469fcd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc9fcd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9109fcdc, SUC_lexicon_tags_27 },
    { 0xf329fcdd, SUC_lexicon_tags_15 },
    { 0x49fcdc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3489fce9, SUC_lexicon_tags_27 },
    { 0x97c9fcea, SUC_lexicon_tags_2 },
    { 0x53e9fceb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd369fcf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4389fcf2, SUC_lexicon_tags_37 },
    { 0xde69fcf2, SUC_lexicon_tags_33 },
    { 0x1789fcf3, SUC_lexicon_tags_848 },
    { 0xede9fcf3, SUC_lexicon_tags_47 },
    { 0x2969fcf2, SUC_lexicon_tags_48 },
    { 0xf969fcf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc429fcfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40e9fd07, SUC_lexicon_tags_12 },
    { 0xf1a9fd08, SUC_lexicon_tags_37 },
    { 0x8989fd08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5069fd0c, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xf449fd0e, SUC_lexicon_tags_0 },
    { 0x1da9fd0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14e9fd11, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x72a9fd13, SUC_lexicon_tags_37 },
    { 0x90a9fd14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb009fd16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4709fd1a, SUC_lexicon_tags_37 },
    { 0x9a49fd1b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb29fd1f, SUC_lexicon_tags_1 },
    { 0xecc9fd1f, SUC_lexicon_tags_1 },
    { 0xb929fd21, SUC_lexicon_tags_6 },
    { 0xb9e9fd21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf469fd26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x78c9fd29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9369fd2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba9fd2f, SUC_lexicon_tags_37 },
    { 0x37e9fd30, SUC_lexicon_tags_23 },
    { 0xce09fd30, SUC_lexicon_tags_39 },
    { 0x5449fd2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe1e9fd34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67a9fd37, SUC_lexicon_tags_7 },
    { 0x9d29fd38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe689fd3a, SUC_lexicon_tags_12 },
    { 0xd989fd3b, SUC_lexicon_tags_37 },
    { 0x26a9fd3b, SUC_lexicon_tags_26 },
    { 0x39e9fd3d, SUC_lexicon_tags_25 },
    { 0x7689fd3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47e9fd42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9149fd47, SUC_lexicon_tags_54 },
    { 0x8249fd48, SUC_lexicon_tags_33 },
    { 0xc449fd49, SUC_lexicon_tags_49 },
    { 0xd5c9fd4a, SUC_lexicon_tags_11 },
    { 0x2d69fd49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1669fd4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc069fd51, SUC_lexicon_tags_15 },
    { 0xce49fd52, SUC_lexicon_tags_15 },
    { 0x55a9fd51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb069fd59, SUC_lexicon_tags_1 },
    { 0xaf29fd5a, SUC_lexicon_tags_2 },
    { 0xb429fd5b, SUC_lexicon_tags_26 },
    { 0xcac9fd5c, SUC_lexicon_tags_25 },
    { 0x6ae9fd5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1169fd66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3129fd6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef69fd6f, SUC_lexicon_tags_25 },
    { 0x2629fd6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa9fd77, SUC_lexicon_tags_2 },
    { 0xce89fd78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25e9fd7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e9fd81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a9fd87, SUC_lexicon_tags_95 },
    { 0xd6e9fd88, SUC_lexicon_tags_12 },
    { 0x2829fd87, SUC_lexicon_tags_2 },
    { 0xbe49fd89, SUC_lexicon_tags_1086 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd769fd8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e49fd92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3869fd96, SUC_lexicon_tags_11 },
    { 0x2269fd97, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2329fd99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e9fd9f, SUC_lexicon_tags_17 },
    { 0x6c89fda0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e9fda3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe309fda5, SUC_lexicon_tags_37 },
    { 0xd5a9fda6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x24e9fda8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c9fdac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d89fdb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5a9fdb3, SUC_lexicon_tags_7 },
    { 0x3b69fdb4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4969fdba, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e9fdbd, SUC_lexicon_tags_18 },
    { 0x64a9fdbe, SUC_lexicon_tags_2 },
    { 0x6909fdbf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa4e9fdc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc769fdc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2069fdca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd9c9fdcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22e9fdd0, SUC_lexicon_tags_39 },
    { 0xbb89fdd1, SUC_lexicon_tags_3 },
    { 0xa289fdd2, SUC_lexicon_tags_2 },
    { 0x3c9fdd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb889fdd6, SUC_lexicon_tags_47 },
    { 0x2fe9fdd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5049fdd9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1029fddd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa129fddf, SUC_lexicon_tags_27 },
    { 0xfac9fddf, SUC_lexicon_tags_25 },
    { 0x46a9fde1, SUC_lexicon_tags_12 },
    { 0x7189fde0, SUC_lexicon_tags_27 },
    { 0xa4e9fde3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b29fde9, SUC_lexicon_tags_7 },
    { 0xbf49fdea, SUC_lexicon_tags_23 },
    { 0xd29fdeb, SUC_lexicon_tags_19 },
    { 0x1629fdec, SUC_lexicon_tags_37 },
    { 0x87e9fdec, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9fdf0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe7e9fdf2, SUC_lexicon_tags_10 },
    { 0x4d09fdf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf409fdf8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x54e9fdfa, SUC_lexicon_tags_17 },
    { 0xc1e9fdfb, SUC_lexicon_tags_8 },
    { 0x2529fdfc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e9fe00, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb809fe04, SUC_lexicon_tags_47 },
    { 0x6849fe05, SUC_lexicon_tags_6 },
    { 0x2649fe05, SUC_lexicon_tags_84 },
    { 0x8f49fe04, SUC_lexicon_tags_25 },
    { 0x2a49fe07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e09fe0a, SUC_lexicon_tags_15 },
    { 0x3189fe0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf969fe0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2229fe12, SUC_lexicon_tags_12 },
    { 0x8269fe13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29fe1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec9fe21, SUC_lexicon_tags_27 },
    { 0x17a9fe22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb129fe25, SUC_lexicon_tags_77 },
    { 0x1769fe26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4409fe28, SUC_lexicon_tags_11 },
    { 0x3f09fe28, SUC_lexicon_tags_60 },
    { 0xe649fe2a, SUC_lexicon_tags_18 },
    { 0x41a9fe2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc729fe2d, SUC_lexicon_tags_29 },
    { 0xd849fe2e, SUC_lexicon_tags_5 },
    { 0x46c9fe2d, SUC_lexicon_tags_25 },
    { 0x2fc9fe2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca09fe32, SUC_lexicon_tags_21 },
    { 0x4929fe33, SUC_lexicon_tags_12 },
    { 0xdca9fe34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe69fe37, SUC_lexicon_tags_25 },
    { 0xd09fe37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e69fe3d, SUC_lexicon_tags_63 },
    { 0x4a69fe3e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9fe41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e9fe4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2689fe51, SUC_lexicon_tags_17 },
    { 0x7649fe52, SUC_lexicon_tags_30 },
    { 0x3de9fe52, SUC_lexicon_tags_96 },
    { 0x1c89fe54, SUC_lexicon_tags_16 },
    { 0x3a29fe51, SUC_lexicon_tags_35 },
    { 0x5c09fe54, SUC_lexicon_tags_16 },
    { 0x72a9fe57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e9fe5e, SUC_lexicon_tags_67 },
    { 0xde29fe5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf29fe61, SUC_lexicon_tags_28 },
    { 0x29c9fe61, SUC_lexicon_tags_15 },
    { 0xeb49fe63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2729fe65, SUC_lexicon_tags_14 },
    { 0x1c29fe66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbea9fe68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd69fe6a, SUC_lexicon_tags_31 },
    { 0x349fe6b, SUC_lexicon_tags_17 },
    { 0x7fe9fe6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8409fe6f, SUC_lexicon_tags_62 },
    { 0x3c9fe70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e9fe76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8189fe78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77a9fe7a, SUC_lexicon_tags_37 },
    { 0xe309fe7b, SUC_lexicon_tags_1 },
    { 0x9e49fe7c, SUC_lexicon_tags_17 },
    { 0x3fa9fe7d, SUC_lexicon_tags_63 },
    { 0x5629fe7e, SUC_lexicon_tags_19 },
    { 0x12e9fe7f, SUC_lexicon_tags_2 },
    { 0xb609fe80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2149fe82, SUC_lexicon_tags_68 },
    { 0xeea9fe83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20a9fe86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1889fe8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a29fe90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd49fe92, SUC_lexicon_tags_0 },
    { 0x5c49fe93, SUC_lexicon_tags_29 },
    { 0x85c9fe94, SUC_lexicon_tags_12 },
    { 0x3e69fe94, SUC_lexicon_tags_65 },
    { 0x9809fe95, SUC_lexicon_tags_23 },
    { 0xbfa9fe97, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9fea1, SUC_lexicon_tags_43 },
    { 0xcbe9fea2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74a9fea7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3409feaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea29feb2, SUC_lexicon_tags_42 },
    { 0xee49feb3, SUC_lexicon_tags_17 },
    { 0x2709feb4, SUC_lexicon_tags_16 },
    { 0xbf49feb5, SUC_lexicon_tags_43 },
    { 0xd8a9feb3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf9c9feb8, SUC_lexicon_tags_11 },
    { 0x9389feb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d89febb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab29fec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc529fec5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4a9fecb, SUC_lexicon_tags_12 },
    { 0xa549fecb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb189fecf, SUC_lexicon_tags_7 },
    { 0x4b29fed0, SUC_lexicon_tags_37 },
    { 0x9c69fed1, SUC_lexicon_tags_11 },
    { 0x6989fed1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f49feda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5869fedf, SUC_lexicon_tags_47 },
    { 0x2689fee0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec9fee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f09fee8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xac9feea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc229fef4, SUC_lexicon_tags_2 },
    { 0xc2a9fef5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f49fef7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc469ff0b, SUC_lexicon_tags_34 },
    { 0x53a9ff0c, SUC_lexicon_tags_38 },
    { 0xae49ff0d, SUC_lexicon_tags_10 },
    { 0x8be9ff0b, SUC_lexicon_tags_37 },
    { 0x9be9ff0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6949ff12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5729ff17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49ff20, SUC_lexicon_tags_11 },
    { 0x1709ff21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09ff26, SUC_lexicon_tags_25 },
    { 0x2949ff27, SUC_lexicon_tags_49 },
    { 0x3769ff26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5089ff2d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e9ff30, SUC_lexicon_tags_33 },
    { 0x4109ff30, SUC_lexicon_tags_25 },
    { 0xd7e9ff31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4549ff39, SUC_lexicon_tags_11 },
    { 0x49ff3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7de9ff3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89ff3f, SUC_lexicon_tags_10 },
    { 0xfec9ff40, SUC_lexicon_tags_25 },
    { 0x7ce9ff41, SUC_lexicon_tags_33 },
    { 0x6e69ff41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4609ff4a, SUC_lexicon_tags_60 },
    { 0x2ce9ff4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a09ff51, SUC_lexicon_tags_46 },
    { 0x12a9ff52, SUC_lexicon_tags_37 },
    { 0x69e9ff53, SUC_lexicon_tags_17 },
    { 0x87c9ff51, SUC_lexicon_tags_61 },
    { 0x3949ff55, SUC_lexicon_tags_29 },
    { 0xf5e9ff52, SUC_lexicon_tags_7 },
    { 0xaf29ff55, SUC_lexicon_tags_7 },
    { 0xe089ff58, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4949ff5a, SUC_lexicon_tags_27 },
    { 0xb09ff5b, SUC_lexicon_tags_7 },
    { 0xb829ff5a, SUC_lexicon_tags_77 },
    { 0x4969ff5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf049ff69, SUC_lexicon_tags_17 },
    { 0x36c9ff6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16c9ff6c, SUC_lexicon_tags_34 },
    { 0x2289ff6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9429ff6f, SUC_lexicon_tags_13 },
    { 0xc3c9ff6f, SUC_lexicon_tags_17 },
    { 0xc309ff70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549ff7e, SUC_lexicon_tags_0 },
    { 0x20c9ff7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c69ff81, SUC_lexicon_tags_14 },
    { 0x7c09ff81, SUC_lexicon_tags_25 },
    { 0x46e9ff82, SUC_lexicon_tags_13 },
    { 0x9249ff82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9ff87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c9ff8b, SUC_lexicon_tags_7 },
    { 0xba09ff8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf509ff8e, SUC_lexicon_tags_1 },
    { 0x4049ff8e, SUC_lexicon_tags_13 },
    { 0xf889ff90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc9ff92, SUC_lexicon_tags_23 },
    { 0x5969ff93, SUC_lexicon_tags_74 },
    { 0x889ff94, SUC_lexicon_tags_0 },
    { 0x8a09ff94, SUC_lexicon_tags_61 },
    { 0xf409ff92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e9ff99, SUC_lexicon_tags_92 },
    { 0x6cc9ff9a, SUC_lexicon_tags_23 },
    { 0x1de9ff99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8909ffa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19c9ffa6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc729ffa9, SUC_lexicon_tags_15 },
    { 0xb6a9ffa9, SUC_lexicon_tags_2 },
    { 0x2289ffab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4949ffad, SUC_lexicon_tags_6 },
    { 0x5389ffae, SUC_lexicon_tags_27 },
    { 0xee09ffae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9409ffb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d09ffb3, SUC_lexicon_tags_17 },
    { 0x1209ffb3, SUC_lexicon_tags_8 },
    { 0x3629ffb5, SUC_lexicon_tags_77 },
    { 0xac09ffb6, SUC_lexicon_tags_151 },
    { 0xe829ffb4, SUC_lexicon_tags_37 },
    { 0xc529ffb3, SUC_lexicon_tags_7 },
    { 0xb4e9ffb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409ffbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3849ffbf, SUC_lexicon_tags_84 },
    { 0x30c9ffc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2869ffc5, SUC_lexicon_tags_7 },
    { 0x6569ffc5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd649ffcd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ba9ffcf, SUC_lexicon_tags_25 },
    { 0x1da9ffcf, SUC_lexicon_tags_25 },
    { 0xf629ffd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa169ffd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe769ffd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd229ffdc, SUC_lexicon_tags_1 },
    { 0xd969ffdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3629ffe3, SUC_lexicon_tags_0 },
    { 0x4d69ffe4, SUC_lexicon_tags_77 },
    { 0x9409ffe5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7209ffed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4809fff6, SUC_lexicon_tags_2 },
    { 0x7949fff7, SUC_lexicon_tags_25 },
    { 0x26a9fff6, SUC_lexicon_tags_79 },
    { 0x5a69fff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c89fffc, SUC_lexicon_tags_17 },
    { 0xabc9fffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1cc9ffff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bea0002, SUC_lexicon_tags_15 },
    { 0x854a0003, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ea0009, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaa000c, SUC_lexicon_tags_61 },
    { 0xdaca000d, SUC_lexicon_tags_1 },
    { 0x200a000e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ca0012, SUC_lexicon_tags_17 },
    { 0x386a0013, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12a0019, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ea001e, SUC_lexicon_tags_2 },
    { 0x986a001f, SUC_lexicon_tags_47 },
    { 0x76ca001f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90a0024, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11ea0026, SUC_lexicon_tags_25 },
    { 0x816a0027, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26a002e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2a0031, SUC_lexicon_tags_21 },
    { 0xa02a0032, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6a0035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4a0038, SUC_lexicon_tags_1 },
    { 0xb0aa0039, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f4a003b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb48a003d, SUC_lexicon_tags_15 },
    { 0x7c8a003e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a0041, SUC_lexicon_tags_47 },
    { 0x4b6a0041, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a0047, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ca004b, SUC_lexicon_tags_35 },
    { 0xe3ca004c, SUC_lexicon_tags_17 },
    { 0x69aa004b, SUC_lexicon_tags_7 },
    { 0xa0aa004d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aa0054, SUC_lexicon_tags_60 },
    { 0xa80a0054, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeaa0057, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4a005d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe56a005f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ca0064, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e2a0066, SUC_lexicon_tags_38 },
    { 0x64ca0067, SUC_lexicon_tags_27 },
    { 0xc22a0066, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478a0070, SUC_lexicon_tags_37 },
    { 0x8f4a0070, SUC_lexicon_tags_0 },
    { 0x8faa0070, SUC_lexicon_tags_7 },
    { 0xd96a0073, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294a0077, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212a007d, SUC_lexicon_tags_33 },
    { 0x1aca007e, SUC_lexicon_tags_29 },
    { 0x5daa007f, SUC_lexicon_tags_87 },
    { 0xf90a0080, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eca0083, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2a008c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ea008f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x190a0092, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88a0094, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x654a0096, SUC_lexicon_tags_31 },
    { 0x322a0097, SUC_lexicon_tags_11 },
    { 0x62ea0098, SUC_lexicon_tags_31 },
    { 0x5aea0099, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xec0a009b, SUC_lexicon_tags_27 },
    { 0xceaa009c, SUC_lexicon_tags_25 },
    { 0xacea009d, SUC_lexicon_tags_2 },
    { 0xbc6a009e, SUC_lexicon_tags_37 },
    { 0x64aa009f, SUC_lexicon_tags_37 },
    { 0x748a00a0, SUC_lexicon_tags_4 },
    { 0xb7ea009f, SUC_lexicon_tags_1 },
    { 0x600a009d, SUC_lexicon_tags_17 },
    { 0xe18a00a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x544a00a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x612a00a7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a00aa, SUC_lexicon_tags_7 },
    { 0xf8ea00ab, SUC_lexicon_tags_22 },
    { 0x6eea00ab, SUC_lexicon_tags_43 },
    { 0xa02a00ad, SUC_lexicon_tags_25 },
    { 0xf5ea00ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x818a00b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660a00b5, SUC_lexicon_tags_32 },
    { 0x5d6a00b6, SUC_lexicon_tags_28 },
    { 0x15aa00b6, SUC_lexicon_tags_25 },
    { 0x134a00b8, SUC_lexicon_tags_46 },
    { 0xabca00b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2f8a00bb, SUC_lexicon_tags_2 },
    { 0x81ea00bb, SUC_lexicon_tags_1387 },
    { 0xcc4a00bd, SUC_lexicon_tags_8 },
    { 0x4ca00bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x618a00c0, SUC_lexicon_tags_7 },
    { 0x5cea00c1, SUC_lexicon_tags_15 },
    { 0x708a00c1, SUC_lexicon_tags_56 },
    { 0xd4ca00c1, SUC_lexicon_tags_0 },
    { 0xceea00c0, SUC_lexicon_tags_46 },
    { 0x99ca00c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x794a00c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x378a00ca, SUC_lexicon_tags_14 },
    { 0x2a4a00cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460a00cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38a00d4, SUC_lexicon_tags_7 },
    { 0xdc4a00d5, SUC_lexicon_tags_37 },
    { 0x23aa00d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x386a00d8, SUC_lexicon_tags_16 },
    { 0x83ca00d9, SUC_lexicon_tags_66 },
    { 0x964a00da, SUC_lexicon_tags_66 },
    { 0x9faa00db, SUC_lexicon_tags_18 },
    { 0xe10a00dc, SUC_lexicon_tags_9 },
    { 0x648a00d8, SUC_lexicon_tags_7 },
    { 0x3e8a00d8, SUC_lexicon_tags_1 },
    { 0x430a00db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8a00e3, SUC_lexicon_tags_2 },
    { 0xdbaa00e4, SUC_lexicon_tags_1 },
    { 0xd36a00e4, SUC_lexicon_tags_1 },
    { 0x85aa00e6, SUC_lexicon_tags_43 },
    { 0xda2a00e5, SUC_lexicon_tags_1451 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2a00eb, SUC_lexicon_tags_2 },
    { 0xcfaa00eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eaa00ee, SUC_lexicon_tags_1 },
    { 0xb92a00ef, SUC_lexicon_tags_234 },
    { 0xeaea00ee, SUC_lexicon_tags_51 },
    { 0x1baa00f1, SUC_lexicon_tags_26 },
    { 0x764a00f2, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca00f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf6ca00fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520a00ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0101, SUC_lexicon_tags_53 },
    { 0x77ea0102, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa54a0104, SUC_lexicon_tags_0 },
    { 0x9f2a0105, SUC_lexicon_tags_11 },
    { 0x910a0105, SUC_lexicon_tags_29 },
    { 0xa56a0107, SUC_lexicon_tags_37 },
    { 0x5a4a0104, SUC_lexicon_tags_17 },
    { 0x104a0109, SUC_lexicon_tags_2 },
    { 0xe74a010a, SUC_lexicon_tags_2 },
    { 0x67ea0107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f0a010d, SUC_lexicon_tags_31 },
    { 0x10ca010e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x698a0110, SUC_lexicon_tags_26 },
    { 0x22aa0111, SUC_lexicon_tags_19 },
    { 0x884a0112, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0aa0119, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x47aa011c, SUC_lexicon_tags_12 },
    { 0x70aa011c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ca0121, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2a0124, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca0127, SUC_lexicon_tags_12 },
    { 0x1c8a0128, SUC_lexicon_tags_0 },
    { 0x270a0129, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8baa012c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdc0a012e, SUC_lexicon_tags_19 },
    { 0xbaea012e, SUC_lexicon_tags_62 },
    { 0x264a0130, SUC_lexicon_tags_1 },
    { 0xa1ea0131, SUC_lexicon_tags_19 },
    { 0xf2ea0132, SUC_lexicon_tags_47 },
    { 0xe66a0133, SUC_lexicon_tags_2 },
    { 0x68a0134, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ea0137, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0a013c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea013f, SUC_lexicon_tags_23 },
    { 0x88aa0140, SUC_lexicon_tags_23 },
    { 0x3f0a0141, SUC_lexicon_tags_7 },
    { 0x652a0140, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2a0146, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cea014a, SUC_lexicon_tags_2 },
    { 0x43ca014b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ea0152, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4a0155, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aa0159, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ca015e, SUC_lexicon_tags_38 },
    { 0x88aa015f, SUC_lexicon_tags_2 },
    { 0x234a0160, SUC_lexicon_tags_26 },
    { 0x618a015f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x85ca0163, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2aa016b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf0a016d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154a0173, SUC_lexicon_tags_43 },
    { 0xa78a0173, SUC_lexicon_tags_17 },
    { 0x648a0174, SUC_lexicon_tags_15 },
    { 0xadea0176, SUC_lexicon_tags_37 },
    { 0x276a0177, SUC_lexicon_tags_1 },
    { 0x648a0176, SUC_lexicon_tags_37 },
    { 0x198a0176, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xba8a017b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x332a017d, SUC_lexicon_tags_63 },
    { 0xecaa017e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a8a0180, SUC_lexicon_tags_25 },
    { 0x472a0180, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ca0186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6a0189, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aa0192, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x192a0194, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf0a0196, SUC_lexicon_tags_15 },
    { 0x66aa0197, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ca019a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdca019c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6a01a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48ea01a4, SUC_lexicon_tags_12 },
    { 0xc74a01a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x22aa01a8, SUC_lexicon_tags_13 },
    { 0xb0ea01a9, SUC_lexicon_tags_49 },
    { 0x724a01aa, SUC_lexicon_tags_26 },
    { 0xc58a01ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2a01ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34a01b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54a01b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x384a01b7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ca01bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x14ea01bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd0ea01bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670a01c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54a01c9, SUC_lexicon_tags_2 },
    { 0xb0aa01ca, SUC_lexicon_tags_1 },
    { 0xee6a01ca, SUC_lexicon_tags_17 },
    { 0xbdea01cc, SUC_lexicon_tags_0 },
    { 0xd22a01cc, SUC_lexicon_tags_8 },
    { 0x518a01cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2a01d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x612a01d6, SUC_lexicon_tags_27 },
    { 0x2c6a01d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x28ca01d9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc8aa01db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ea01e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffea01e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6a01f0, SUC_lexicon_tags_17 },
    { 0xff8a01f1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74aa01f6, SUC_lexicon_tags_2 },
    { 0xd10a01f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7aa01fe, SUC_lexicon_tags_15 },
    { 0x5aca01fe, SUC_lexicon_tags_11 },
    { 0xfb8a01fe, SUC_lexicon_tags_7 },
    { 0x350a01ff, SUC_lexicon_tags_31 },
    { 0x6f0a01fe, SUC_lexicon_tags_43 },
    { 0x752a0203, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x320a0205, SUC_lexicon_tags_7 },
    { 0x7f4a0205, SUC_lexicon_tags_12 },
    { 0xc5aa0205, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf0ea0209, SUC_lexicon_tags_23 },
    { 0xf2ea020a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16a020d, SUC_lexicon_tags_37 },
    { 0x88ea020e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x606a0211, SUC_lexicon_tags_45 },
    { 0x732a0211, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x466a0214, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8a0217, SUC_lexicon_tags_23 },
    { 0x762a0217, SUC_lexicon_tags_7 },
    { 0x884a0217, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x3bea021b, SUC_lexicon_tags_9 },
    { 0x2faa021c, SUC_lexicon_tags_63 },
    { 0x3ea021d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72a0221, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4a0224, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900a022c, SUC_lexicon_tags_1 },
    { 0xfeaa022c, SUC_lexicon_tags_635 },
    { 0, NULL },
    { 0x950a022f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd36a0231, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc68a0233, SUC_lexicon_tags_1048 },
    { 0, NULL },
    { 0x9eaa0235, SUC_lexicon_tags_17 },
    { 0x6caa0236, SUC_lexicon_tags_37 },
    { 0x4f4a0235, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee2a0239, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5caa023d, SUC_lexicon_tags_4 },
    { 0xe76a023e, SUC_lexicon_tags_47 },
    { 0x360a023e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674a0243, SUC_lexicon_tags_476 },
    { 0xa62a0244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6ea0246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868a024b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6a024e, SUC_lexicon_tags_2 },
    { 0xceca024e, SUC_lexicon_tags_17 },
    { 0x508a0250, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaa0253, SUC_lexicon_tags_7 },
    { 0x676a0254, SUC_lexicon_tags_2 },
    { 0x96aa0255, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6a0258, SUC_lexicon_tags_49 },
    { 0x1a8a0258, SUC_lexicon_tags_11 },
    { 0x61ea025a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4a025d, SUC_lexicon_tags_33 },
    { 0xf9ea025d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa0268, SUC_lexicon_tags_65 },
    { 0xa42a0269, SUC_lexicon_tags_15 },
    { 0x3bea0269, SUC_lexicon_tags_1 },
    { 0x6dea026b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x754a026d, SUC_lexicon_tags_15 },
    { 0x36aa026d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51ca0270, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592a0277, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x188a027a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x412a027c, SUC_lexicon_tags_2 },
    { 0x33ca027d, SUC_lexicon_tags_7 },
    { 0x6eea027e, SUC_lexicon_tags_1 },
    { 0x46ca027f, SUC_lexicon_tags_7 },
    { 0x5d8a0280, SUC_lexicon_tags_45 },
    { 0xea2a0281, SUC_lexicon_tags_27 },
    { 0xddca0282, SUC_lexicon_tags_31 },
    { 0xec4a0283, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26a0286, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4a028b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4a028f, SUC_lexicon_tags_25 },
    { 0x314a0290, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x412a0293, SUC_lexicon_tags_19 },
    { 0x2cca0294, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44a0297, SUC_lexicon_tags_1 },
    { 0x5f6a0298, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174a029f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400a02a4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8e6a02a6, SUC_lexicon_tags_15 },
    { 0x134a02a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ea02ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470a02b4, SUC_lexicon_tags_62 },
    { 0xd54a02b5, SUC_lexicon_tags_68 },
    { 0x7f4a02b4, SUC_lexicon_tags_12 },
    { 0xf06a02b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04a02ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9b0a02bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0a02c2, SUC_lexicon_tags_37 },
    { 0x122a02c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc18a02c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8a02c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc40a02ca, SUC_lexicon_tags_11 },
    { 0x63ca02cb, SUC_lexicon_tags_15 },
    { 0xdeca02cc, SUC_lexicon_tags_37 },
    { 0x34ca02cd, SUC_lexicon_tags_6 },
    { 0x232a02ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ca02d1, SUC_lexicon_tags_12 },
    { 0x99aa02d2, SUC_lexicon_tags_17 },
    { 0xa76a02d3, SUC_lexicon_tags_6 },
    { 0xea4a02d3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a02dd, SUC_lexicon_tags_21 },
    { 0xa5aa02de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986a02e3, SUC_lexicon_tags_32 },
    { 0x698a02e4, SUC_lexicon_tags_37 },
    { 0x1f6a02e5, SUC_lexicon_tags_17 },
    { 0x53aa02e6, SUC_lexicon_tags_23 },
    { 0x846a02e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x346a02e9, SUC_lexicon_tags_13 },
    { 0xbea02e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacca02ec, SUC_lexicon_tags_38 },
    { 0xe9aa02ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774a02f1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x950a02f3, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8a02f7, SUC_lexicon_tags_1 },
    { 0xe16a02f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ca0304, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x392a0306, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6a030a, SUC_lexicon_tags_2 },
    { 0x8d4a030b, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678a0311, SUC_lexicon_tags_8 },
    { 0x86ca0312, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0a0314, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ca031b, SUC_lexicon_tags_15 },
    { 0x456a031b, SUC_lexicon_tags_2 },
    { 0x85ea031d, SUC_lexicon_tags_0 },
    { 0x81ca031e, SUC_lexicon_tags_37 },
    { 0x26ca031f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53aa0324, SUC_lexicon_tags_23 },
    { 0x17aa0325, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x408a0327, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2a032f, SUC_lexicon_tags_37 },
    { 0xc1aa032f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4a0335, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64a0338, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xffea033a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2ca033c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x858a033e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372a0342, SUC_lexicon_tags_26 },
    { 0x70a0343, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca0346, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcdaa0348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x786a034a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4a034d, SUC_lexicon_tags_1 },
    { 0xc46a034e, SUC_lexicon_tags_21 },
    { 0x2b4a034e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4a0352, SUC_lexicon_tags_33 },
    { 0x676a0352, SUC_lexicon_tags_14 },
    { 0xabaa0354, SUC_lexicon_tags_16 },
    { 0xfa6a0355, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124a035f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902a0363, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256a036b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeca036e, SUC_lexicon_tags_11 },
    { 0x938a036e, SUC_lexicon_tags_19 },
    { 0xd7ca0370, SUC_lexicon_tags_11 },
    { 0x956a0370, SUC_lexicon_tags_15 },
    { 0xa88a0372, SUC_lexicon_tags_15 },
    { 0x474a0372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4aa0375, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ca0379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd62a037b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5faa0380, SUC_lexicon_tags_1 },
    { 0x9aea0381, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02a0384, SUC_lexicon_tags_87 },
    { 0xcc4a0385, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92a038a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ea038f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ca0393, SUC_lexicon_tags_33 },
    { 0xd46a0393, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a8a0396, SUC_lexicon_tags_15 },
    { 0xdb6a0397, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24a039a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc80a039c, SUC_lexicon_tags_2 },
    { 0x2cca039d, SUC_lexicon_tags_82 },
    { 0xac2a039e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a8a03a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab8a03a2, SUC_lexicon_tags_45 },
    { 0x192a03a3, SUC_lexicon_tags_1 },
    { 0xe10a03a3, SUC_lexicon_tags_43 },
    { 0xf88a03a5, SUC_lexicon_tags_7 },
    { 0x9b4a03a6, SUC_lexicon_tags_107 },
    { 0x216a03a5, SUC_lexicon_tags_450 },
    { 0x576a03a6, SUC_lexicon_tags_14 },
    { 0x5aea03a6, SUC_lexicon_tags_35 },
    { 0xef8a03a2, SUC_lexicon_tags_15 },
    { 0x92ca03ab, SUC_lexicon_tags_37 },
    { 0x554a03ac, SUC_lexicon_tags_68 },
    { 0x4aa03ac, SUC_lexicon_tags_62 },
    { 0x7dca03ad, SUC_lexicon_tags_13 },
    { 0xceea03a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494a03b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8a03ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe2a03bc, SUC_lexicon_tags_15 },
    { 0xe66a03bd, SUC_lexicon_tags_72 },
    { 0x5a4a03be, SUC_lexicon_tags_45 },
    { 0x69ea03bf, SUC_lexicon_tags_27 },
    { 0x466a03bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x782a03c3, SUC_lexicon_tags_63 },
    { 0x544a03c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x878a03c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3aa03c9, SUC_lexicon_tags_59 },
    { 0xc3aa03ca, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xaf4a03cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464a03d4, SUC_lexicon_tags_14 },
    { 0xbbca03d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca03d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8ca03db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ea03de, SUC_lexicon_tags_62 },
    { 0x81ca03df, SUC_lexicon_tags_2 },
    { 0x32ca03df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76aa03e4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x516a03e6, SUC_lexicon_tags_42 },
    { 0xd3ea03e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560a03ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0a03ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92a03f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00a03f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x664a03fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x306a03fe, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0x7f6a0400, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26a0404, SUC_lexicon_tags_47 },
    { 0x682a0404, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0a040c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x554a040e, SUC_lexicon_tags_37 },
    { 0xa4a040f, SUC_lexicon_tags_11 },
    { 0x5dea0410, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a0415, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6a0419, SUC_lexicon_tags_63 },
    { 0x6a2a0419, SUC_lexicon_tags_47 },
    { 0x882a041b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306a041f, SUC_lexicon_tags_28 },
    { 0x436a0420, SUC_lexicon_tags_25 },
    { 0x8aca0421, SUC_lexicon_tags_37 },
    { 0xe9aa0421, SUC_lexicon_tags_25 },
    { 0x4a6a0423, SUC_lexicon_tags_35 },
    { 0x28ca0423, SUC_lexicon_tags_8 },
    { 0xb8ca0425, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ea042a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca042e, SUC_lexicon_tags_19 },
    { 0x28ea042f, SUC_lexicon_tags_1 },
    { 0x85ea0430, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47ea0432, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3ea0434, SUC_lexicon_tags_15 },
    { 0xf0aa0435, SUC_lexicon_tags_23 },
    { 0x9c2a0436, SUC_lexicon_tags_7 },
    { 0x55ea0436, SUC_lexicon_tags_11 },
    { 0xd4ea0437, SUC_lexicon_tags_28 },
    { 0x65ea0437, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafca043b, SUC_lexicon_tags_17 },
    { 0x284a043b, SUC_lexicon_tags_63 },
    { 0x6d2a043d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9aa043f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe00a0441, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe64a0443, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74a0446, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x832a0448, SUC_lexicon_tags_3 },
    { 0x84ea0448, SUC_lexicon_tags_63 },
    { 0x5aca0449, SUC_lexicon_tags_25 },
    { 0x2eea044b, SUC_lexicon_tags_9 },
    { 0x5c4a044c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ea0452, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a0457, SUC_lexicon_tags_3 },
    { 0xc0a0458, SUC_lexicon_tags_37 },
    { 0x470a0459, SUC_lexicon_tags_37 },
    { 0x932a045a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ca0463, SUC_lexicon_tags_37 },
    { 0x42a0464, SUC_lexicon_tags_0 },
    { 0xf5ca0463, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccca0469, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xafaa046b, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea0470, SUC_lexicon_tags_27 },
    { 0x8f2a0470, SUC_lexicon_tags_1 },
    { 0x4a8a0472, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe80a0474, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ca047b, SUC_lexicon_tags_11 },
    { 0x952a047c, SUC_lexicon_tags_2 },
    { 0x166a047d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ca0483, SUC_lexicon_tags_17 },
    { 0xe00a0483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28a0488, SUC_lexicon_tags_11 },
    { 0xb78a0489, SUC_lexicon_tags_27 },
    { 0xc5ea048a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0a0490, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba6a0492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8aa0494, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x730a0496, SUC_lexicon_tags_48 },
    { 0xf0aa0497, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8a049b, SUC_lexicon_tags_1 },
    { 0x152a049c, SUC_lexicon_tags_7 },
    { 0x22ea049b, SUC_lexicon_tags_91 },
    { 0xc0aa049d, SUC_lexicon_tags_25 },
    { 0xddca049f, SUC_lexicon_tags_27 },
    { 0x150a049d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754a04ad, SUC_lexicon_tags_42 },
    { 0x752a04ad, SUC_lexicon_tags_8 },
    { 0x378a04af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa04b2, SUC_lexicon_tags_10 },
    { 0xc38a04b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x952a04b5, SUC_lexicon_tags_9 },
    { 0x48a04b5, SUC_lexicon_tags_38 },
    { 0x2f2a04b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4a04b9, SUC_lexicon_tags_27 },
    { 0x11ca04ba, SUC_lexicon_tags_48 },
    { 0x774a04bb, SUC_lexicon_tags_9 },
    { 0xc9ea04bc, SUC_lexicon_tags_84 },
    { 0xc3ca04bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aca04c1, SUC_lexicon_tags_1 },
    { 0x1ca04c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468a04c6, SUC_lexicon_tags_25 },
    { 0x946a04c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x268a04ca, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xc50a04cc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8a04d0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaa04d3, SUC_lexicon_tags_37 },
    { 0xaa0a04d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aa04d9, SUC_lexicon_tags_33 },
    { 0x4faa04d9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834a04e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa04e7, SUC_lexicon_tags_10 },
    { 0xce6a04e8, SUC_lexicon_tags_25 },
    { 0xde4a04e8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x12ca04eb, SUC_lexicon_tags_12 },
    { 0x4d6a04ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82aa04f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6a04f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220a04fc, SUC_lexicon_tags_43 },
    { 0xb92a04fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2dea04ff, SUC_lexicon_tags_38 },
    { 0x4dca04ff, SUC_lexicon_tags_55 },
    { 0xb3ea0501, SUC_lexicon_tags_17 },
    { 0x570a0500, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x220a0504, SUC_lexicon_tags_25 },
    { 0xe02a0505, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684a050b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ca050e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26a0511, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x952a0513, SUC_lexicon_tags_25 },
    { 0x672a0514, SUC_lexicon_tags_3 },
    { 0xd4aa0515, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaa0519, SUC_lexicon_tags_37 },
    { 0x3a8a051a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ca051f, SUC_lexicon_tags_17 },
    { 0x176a0520, SUC_lexicon_tags_60 },
    { 0x1c8a0520, SUC_lexicon_tags_1 },
    { 0x6e4a0522, SUC_lexicon_tags_0 },
    { 0x692a0523, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc86a0525, SUC_lexicon_tags_1 },
    { 0x666a0526, SUC_lexicon_tags_65 },
    { 0xa28a0526, SUC_lexicon_tags_45 },
    { 0x80aa0526, SUC_lexicon_tags_2 },
    { 0xf10a0529, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6a052c, SUC_lexicon_tags_47 },
    { 0xbc0a052d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bea0536, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe06a0538, SUC_lexicon_tags_11 },
    { 0x180a0538, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x774a053b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x792a053e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86aa0540, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68a0545, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x896a0547, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x934a0549, SUC_lexicon_tags_26 },
    { 0xcc8a054a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea054d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacea0551, SUC_lexicon_tags_47 },
    { 0x556a0552, SUC_lexicon_tags_5 },
    { 0xd6aa0552, SUC_lexicon_tags_1 },
    { 0x672a0552, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd76a0556, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2a055b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe38a055d, SUC_lexicon_tags_70 },
    { 0x452a055d, SUC_lexicon_tags_11 },
    { 0x560a055f, SUC_lexicon_tags_17 },
    { 0xa20a0560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ea0567, SUC_lexicon_tags_0 },
    { 0xc86a0567, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8a056e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaea0573, SUC_lexicon_tags_60 },
    { 0xa54a0573, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf12a0576, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a057a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x454a057c, SUC_lexicon_tags_47 },
    { 0x356a057c, SUC_lexicon_tags_30 },
    { 0x16aa057d, SUC_lexicon_tags_11 },
    { 0xfb0a057e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdca0585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a0589, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaf6a058b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcd6a058d, SUC_lexicon_tags_12 },
    { 0xd9ea058d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd66a0590, SUC_lexicon_tags_82 },
    { 0x406a0590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96a0596, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4a059a, SUC_lexicon_tags_87 },
    { 0x4e2a059b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2aaa059d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31ea059f, SUC_lexicon_tags_27 },
    { 0xaa8a05a0, SUC_lexicon_tags_55 },
    { 0x81aa05a1, SUC_lexicon_tags_37 },
    { 0x50ca05a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x492a05a4, SUC_lexicon_tags_37 },
    { 0xbfaa05a4, SUC_lexicon_tags_25 },
    { 0xd62a05a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x818a05a8, SUC_lexicon_tags_47 },
    { 0x6cea05a8, SUC_lexicon_tags_9 },
    { 0x522a05aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x260a05ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88aa05b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ea05b6, SUC_lexicon_tags_21 },
    { 0x716a05b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x864a05b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8a05c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74ca05c4, SUC_lexicon_tags_56 },
    { 0x61ea05c5, SUC_lexicon_tags_23 },
    { 0xe2a05c6, SUC_lexicon_tags_66 },
    { 0x9d6a05c7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80a05ca, SUC_lexicon_tags_7 },
    { 0x8aea05ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538a05cf, SUC_lexicon_tags_7 },
    { 0xe22a05d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4a05da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8a05dd, SUC_lexicon_tags_15 },
    { 0x432a05de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88a05e4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8a05eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dea05f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca0601, SUC_lexicon_tags_11 },
    { 0x598a0602, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5aea0604, SUC_lexicon_tags_3 },
    { 0xaa6a0604, SUC_lexicon_tags_11 },
    { 0xce2a0606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25aa0608, SUC_lexicon_tags_61 },
    { 0x672a0609, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x932a060c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2a060f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6a0612, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x958a0614, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x472a0616, SUC_lexicon_tags_61 },
    { 0x130a0617, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x816a0619, SUC_lexicon_tags_79 },
    { 0x1f0a061a, SUC_lexicon_tags_12 },
    { 0x5d4a061b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72a0620, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08a0623, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc72a0625, SUC_lexicon_tags_23 },
    { 0xe26a0626, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c6a0628, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaa062c, SUC_lexicon_tags_66 },
    { 0x56ca062d, SUC_lexicon_tags_11 },
    { 0x140a062e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0a0634, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaa0637, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x388a063a, SUC_lexicon_tags_29 },
    { 0x5d0a063b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32ca063d, SUC_lexicon_tags_17 },
    { 0x4faa063d, SUC_lexicon_tags_31 },
    { 0x26ea063e, SUC_lexicon_tags_103 },
    { 0x42a0640, SUC_lexicon_tags_40 },
    { 0xe04a0640, SUC_lexicon_tags_1 },
    { 0x8aa0642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aa0647, SUC_lexicon_tags_46 },
    { 0x27aa0648, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e8a064a, SUC_lexicon_tags_7 },
    { 0x1f6a064b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbca0653, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x390a0656, SUC_lexicon_tags_282 },
    { 0xb7ea0656, SUC_lexicon_tags_1 },
    { 0x1bca0658, SUC_lexicon_tags_13 },
    { 0xeea0659, SUC_lexicon_tags_33 },
    { 0x12a0656, SUC_lexicon_tags_37 },
    { 0x944a0659, SUC_lexicon_tags_2 },
    { 0xd72a065c, SUC_lexicon_tags_25 },
    { 0x69aa065d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52a0661, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104a0665, SUC_lexicon_tags_17 },
    { 0xf6a0665, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x826a0668, SUC_lexicon_tags_31 },
    { 0x870a0669, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76a066d, SUC_lexicon_tags_19 },
    { 0x32ca066d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaa0671, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618a0677, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12aa0679, SUC_lexicon_tags_27 },
    { 0x14a0679, SUC_lexicon_tags_37 },
    { 0x6f0a067a, SUC_lexicon_tags_16 },
    { 0xef2a067c, SUC_lexicon_tags_14 },
    { 0x7dea067d, SUC_lexicon_tags_61 },
    { 0xb38a067d, SUC_lexicon_tags_6 },
    { 0x9caa067c, SUC_lexicon_tags_31 },
    { 0x8ca0680, SUC_lexicon_tags_43 },
    { 0x94ca067f, SUC_lexicon_tags_68 },
    { 0x912a0682, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59aa0687, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56aa068c, SUC_lexicon_tags_25 },
    { 0xcea068c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134a0691, SUC_lexicon_tags_18 },
    { 0x6eaa0691, SUC_lexicon_tags_138 },
    { 0xecea0691, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4eaa0695, SUC_lexicon_tags_69 },
    { 0x95ea0696, SUC_lexicon_tags_17 },
    { 0x22aa0696, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef0a0699, SUC_lexicon_tags_15 },
    { 0xc36a0699, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2a06a2, SUC_lexicon_tags_44 },
    { 0x262a06a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe42a06a5, SUC_lexicon_tags_38 },
    { 0x64aa06a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20a06ac, SUC_lexicon_tags_15 },
    { 0xeca06ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ca06b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b4a06b5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xddea06b7, SUC_lexicon_tags_33 },
    { 0x35aa06b8, SUC_lexicon_tags_108 },
    { 0x93ca06b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210a06bf, SUC_lexicon_tags_27 },
    { 0xeaea06c0, SUC_lexicon_tags_11 },
    { 0x990a06bf, SUC_lexicon_tags_172 },
    { 0x47ca06c2, SUC_lexicon_tags_17 },
    { 0x896a06c3, SUC_lexicon_tags_10 },
    { 0x4cca06c3, SUC_lexicon_tags_27 },
    { 0x47ca06c4, SUC_lexicon_tags_12 },
    { 0x7a0a06c6, SUC_lexicon_tags_559 },
    { 0x3aa06c3, SUC_lexicon_tags_47 },
    { 0x864a06c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ea06cb, SUC_lexicon_tags_19 },
    { 0x436a06cc, SUC_lexicon_tags_76 },
    { 0xffaa06cd, SUC_lexicon_tags_15 },
    { 0x4eca06cc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73aa06d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c4a06d6, SUC_lexicon_tags_0 },
    { 0x528a06d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ca06dd, SUC_lexicon_tags_0 },
    { 0x1a8a06de, SUC_lexicon_tags_12 },
    { 0x6f4a06de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6a06e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcdea06e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bea06e7, SUC_lexicon_tags_45 },
    { 0xb80a06e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e0a06ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18aa06ed, SUC_lexicon_tags_17 },
    { 0xa2aa06ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x280a06f1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x83ea06f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ca06f6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38a06f9, SUC_lexicon_tags_2 },
    { 0x984a06f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6d6a06fc, SUC_lexicon_tags_28 },
    { 0xac8a06fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaca0701, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9aa0707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6a070b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed4a070d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca0712, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaa0716, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbea0718, SUC_lexicon_tags_7 },
    { 0x76ea0719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x166a071b, SUC_lexicon_tags_151 },
    { 0xd94a071c, SUC_lexicon_tags_29 },
    { 0x7aca071d, SUC_lexicon_tags_47 },
    { 0xd0ea071d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e8a0720, SUC_lexicon_tags_30 },
    { 0x5e6a0721, SUC_lexicon_tags_11 },
    { 0x9d4a0722, SUC_lexicon_tags_17 },
    { 0x2a0a0723, SUC_lexicon_tags_1 },
    { 0x3b0a0724, SUC_lexicon_tags_23 },
    { 0x22a0720, SUC_lexicon_tags_15 },
    { 0x968a0725, SUC_lexicon_tags_26 },
    { 0x9e0a0723, SUC_lexicon_tags_54 },
    { 0xcc6a0728, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ea072a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e0a072c, SUC_lexicon_tags_45 },
    { 0xd4aa072d, SUC_lexicon_tags_45 },
    { 0xa0aa072c, SUC_lexicon_tags_63 },
    { 0x19ca072d, SUC_lexicon_tags_30 },
    { 0x2f0a072c, SUC_lexicon_tags_151 },
    { 0xd02a0730, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x402a0734, SUC_lexicon_tags_25 },
    { 0xaeea0735, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6a0739, SUC_lexicon_tags_2 },
    { 0x3b8a073a, SUC_lexicon_tags_6 },
    { 0x71ea073b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dea073e, SUC_lexicon_tags_37 },
    { 0x72ca073e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54a0744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386a0748, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x858a074a, SUC_lexicon_tags_68 },
    { 0x890a074a, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0xa36a074d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32a0750, SUC_lexicon_tags_148 },
    { 0x9eea0751, SUC_lexicon_tags_29 },
    { 0xe74a0750, SUC_lexicon_tags_45 },
    { 0x6d0a0753, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32a0757, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04a075c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ca075f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x678a0761, SUC_lexicon_tags_4 },
    { 0x776a0762, SUC_lexicon_tags_47 },
    { 0x16ea0761, SUC_lexicon_tags_231 },
    { 0xea8a0761, SUC_lexicon_tags_35 },
    { 0x464a0762, SUC_lexicon_tags_1 },
    { 0x8b8a0766, SUC_lexicon_tags_56 },
    { 0x734a0767, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0a076b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54aa076d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1ca076f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaca0774, SUC_lexicon_tags_8 },
    { 0x882a0774, SUC_lexicon_tags_23 },
    { 0xf40a0776, SUC_lexicon_tags_17 },
    { 0xc08a0774, SUC_lexicon_tags_38 },
    { 0x7ca0778, SUC_lexicon_tags_11 },
    { 0x1d6a0778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x424a077c, SUC_lexicon_tags_2 },
    { 0xb6ca077d, SUC_lexicon_tags_11 },
    { 0xcb2a077d, SUC_lexicon_tags_2 },
    { 0xc76a077f, SUC_lexicon_tags_43 },
    { 0x5eea077f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eea0783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334a0788, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0x82aa078a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c8a078c, SUC_lexicon_tags_38 },
    { 0x760a078d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aa0794, SUC_lexicon_tags_10 },
    { 0x242a0795, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x730a0798, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4a079c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6a07a3, SUC_lexicon_tags_38 },
    { 0x98aa07a4, SUC_lexicon_tags_17 },
    { 0xa3aa07a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ea07a9, SUC_lexicon_tags_61 },
    { 0xa70a07a9, SUC_lexicon_tags_7 },
    { 0x480a07a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76aa07af, SUC_lexicon_tags_47 },
    { 0xde2a07af, SUC_lexicon_tags_60 },
    { 0xed0a07b1, SUC_lexicon_tags_37 },
    { 0x1f6a07b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06a07b7, SUC_lexicon_tags_47 },
    { 0x9eea07b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x890a07ba, SUC_lexicon_tags_1 },
    { 0x8dea07bb, SUC_lexicon_tags_11 },
    { 0x90a07bb, SUC_lexicon_tags_77 },
    { 0x26ea07ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbea07c0, SUC_lexicon_tags_9 },
    { 0xb68a07c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948a07c6, SUC_lexicon_tags_37 },
    { 0xb40a07c7, SUC_lexicon_tags_25 },
    { 0xe9ca07c8, SUC_lexicon_tags_12 },
    { 0x210a07c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ea07cc, SUC_lexicon_tags_1 },
    { 0x20ea07cd, SUC_lexicon_tags_65 },
    { 0x2eca07ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9aa07d0, SUC_lexicon_tags_19 },
    { 0x872a07d1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10a07dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aa07e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdc2a07e3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5b4a07e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304a07ec, SUC_lexicon_tags_53 },
    { 0x410a07ed, SUC_lexicon_tags_8 },
    { 0xe3ca07ee, SUC_lexicon_tags_37 },
    { 0xf82a07ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4a07fb, SUC_lexicon_tags_23 },
    { 0xd00a07fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7eea07fe, SUC_lexicon_tags_21 },
    { 0x994a07fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2ca0801, SUC_lexicon_tags_38 },
    { 0x43ea0802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fca0804, SUC_lexicon_tags_63 },
    { 0xcaca0804, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x316a0808, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedaa080b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x528a080d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ea0812, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4a0817, SUC_lexicon_tags_44 },
    { 0xe4ea0818, SUC_lexicon_tags_4 },
    { 0xa2ca0819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x250a081b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca081e, SUC_lexicon_tags_32 },
    { 0x894a081f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca0823, SUC_lexicon_tags_15 },
    { 0xf68a0823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaaa0826, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x224a0828, SUC_lexicon_tags_48 },
    { 0x626a0829, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6a082d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea0834, SUC_lexicon_tags_1 },
    { 0xfe0a0835, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ea0838, SUC_lexicon_tags_26 },
    { 0x3c8a0839, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb0ea083b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x778a083e, SUC_lexicon_tags_12 },
    { 0x416a083f, SUC_lexicon_tags_33 },
    { 0x85ca0840, SUC_lexicon_tags_6 },
    { 0xa2aa0841, SUC_lexicon_tags_23 },
    { 0xed8a083f, SUC_lexicon_tags_19 },
    { 0xabaa0843, SUC_lexicon_tags_25 },
    { 0xc96a0844, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4caa0846, SUC_lexicon_tags_63 },
    { 0x79aa0847, SUC_lexicon_tags_138 },
    { 0xcf4a0847, SUC_lexicon_tags_1 },
    { 0xf4ca0849, SUC_lexicon_tags_68 },
    { 0x3aaa0849, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0a084f, SUC_lexicon_tags_11 },
    { 0x22aa0850, SUC_lexicon_tags_37 },
    { 0x6b0a0851, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8eaa0853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe4a0855, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494a085c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0aa085e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78a0862, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaa0866, SUC_lexicon_tags_12 },
    { 0xa0aa0866, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ca0870, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460a0875, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccca0881, SUC_lexicon_tags_43 },
    { 0xb6aa0882, SUC_lexicon_tags_43 },
    { 0x910a0882, SUC_lexicon_tags_1409 },
    { 0xa6aa0884, SUC_lexicon_tags_2 },
    { 0xe22a0885, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7caa088c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8cea088e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886a089b, SUC_lexicon_tags_1 },
    { 0x816a089c, SUC_lexicon_tags_1 },
    { 0xb72a089d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x740a08a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60a08a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d4a08a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d8a08a9, SUC_lexicon_tags_14 },
    { 0xcfea08aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaa08ad, SUC_lexicon_tags_15 },
    { 0x65ea08ad, SUC_lexicon_tags_12 },
    { 0x9e2a08af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ca08b3, SUC_lexicon_tags_17 },
    { 0x1daa08b4, SUC_lexicon_tags_49 },
    { 0x750a08b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x266a08b7, SUC_lexicon_tags_35 },
    { 0x492a08b8, SUC_lexicon_tags_29 },
    { 0x124a08b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d2a08bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2a08bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ea08c0, SUC_lexicon_tags_15 },
    { 0x9daa08c1, SUC_lexicon_tags_72 },
    { 0x8b8a08c2, SUC_lexicon_tags_37 },
    { 0x114a08c2, SUC_lexicon_tags_7 },
    { 0xc78a08c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a08c7, SUC_lexicon_tags_61 },
    { 0x8daa08c8, SUC_lexicon_tags_2 },
    { 0x578a08c9, SUC_lexicon_tags_6 },
    { 0x768a08c8, SUC_lexicon_tags_11 },
    { 0x530a08cb, SUC_lexicon_tags_25 },
    { 0x56aa08cc, SUC_lexicon_tags_65 },
    { 0xba2a08cd, SUC_lexicon_tags_2 },
    { 0x936a08ce, SUC_lexicon_tags_7 },
    { 0xfd0a08ca, SUC_lexicon_tags_0 },
    { 0xb2a08c8, SUC_lexicon_tags_6 },
    { 0xc92a08d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaa08d4, SUC_lexicon_tags_43 },
    { 0xd8aa08d5, SUC_lexicon_tags_31 },
    { 0xe48a08d6, SUC_lexicon_tags_17 },
    { 0x436a08d7, SUC_lexicon_tags_19 },
    { 0x432a08d7, SUC_lexicon_tags_17 },
    { 0xcb0a08d7, SUC_lexicon_tags_1 },
    { 0xac2a08da, SUC_lexicon_tags_35 },
    { 0x19ea08db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbb0a08dd, SUC_lexicon_tags_10 },
    { 0x37aa08de, SUC_lexicon_tags_17 },
    { 0x16ea08df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18a08e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50a08e9, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90a08ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2a08f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a4a08f3, SUC_lexicon_tags_2 },
    { 0x528a08f4, SUC_lexicon_tags_25 },
    { 0xb36a08f5, SUC_lexicon_tags_17 },
    { 0x6c6a08f5, SUC_lexicon_tags_25 },
    { 0x49aa08f7, SUC_lexicon_tags_2 },
    { 0x1c0a08f8, SUC_lexicon_tags_11 },
    { 0x6a08f6, SUC_lexicon_tags_3 },
    { 0x7cca08f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320a0902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6a090f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eea0912, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1aa091a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6fca091c, SUC_lexicon_tags_37 },
    { 0x116a091c, SUC_lexicon_tags_11 },
    { 0x602a091c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0a0925, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2eaa0927, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5daa092a, SUC_lexicon_tags_15 },
    { 0xdf2a092b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ca092f, SUC_lexicon_tags_27 },
    { 0x372a092f, SUC_lexicon_tags_33 },
    { 0xe84a0930, SUC_lexicon_tags_37 },
    { 0xd30a0931, SUC_lexicon_tags_31 },
    { 0xf46a0932, SUC_lexicon_tags_27 },
    { 0xca8a0934, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53aa0936, SUC_lexicon_tags_30 },
    { 0x15ca0937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974a093d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb2ea093f, SUC_lexicon_tags_19 },
    { 0xc9ca0940, SUC_lexicon_tags_27 },
    { 0x6aa0941, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aa0945, SUC_lexicon_tags_1 },
    { 0x2f4a0946, SUC_lexicon_tags_37 },
    { 0xe26a0947, SUC_lexicon_tags_37 },
    { 0x670a0946, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ca094d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4a0952, SUC_lexicon_tags_49 },
    { 0xe34a0953, SUC_lexicon_tags_66 },
    { 0x782a0954, SUC_lexicon_tags_15 },
    { 0xdd4a0955, SUC_lexicon_tags_26 },
    { 0x8bea0954, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0a095c, SUC_lexicon_tags_7 },
    { 0x4c4a095c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaca0963, SUC_lexicon_tags_53 },
    { 0x128a0963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9aa0966, SUC_lexicon_tags_15 },
    { 0xa60a0967, SUC_lexicon_tags_6 },
    { 0x1fca0968, SUC_lexicon_tags_37 },
    { 0x88a0967, SUC_lexicon_tags_2 },
    { 0xc0ea096a, SUC_lexicon_tags_14 },
    { 0x476a096b, SUC_lexicon_tags_7 },
    { 0x41ca096c, SUC_lexicon_tags_61 },
    { 0xadaa096d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca0973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x208a0975, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886a097a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6ca097c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6a097f, SUC_lexicon_tags_19 },
    { 0x500a0980, SUC_lexicon_tags_6 },
    { 0x594a0981, SUC_lexicon_tags_15 },
    { 0x2c4a0982, SUC_lexicon_tags_38 },
    { 0xccea0982, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120a0987, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea098d, SUC_lexicon_tags_33 },
    { 0xaf0a098d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x752a0990, SUC_lexicon_tags_15 },
    { 0x252a0991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb64a0993, SUC_lexicon_tags_15 },
    { 0xa8a0994, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a0998, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x872a099a, SUC_lexicon_tags_15 },
    { 0x922a099b, SUC_lexicon_tags_57 },
    { 0x4f8a099a, SUC_lexicon_tags_19 },
    { 0xc7ea099a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fca09a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe48a09a5, SUC_lexicon_tags_17 },
    { 0x4ca09a5, SUC_lexicon_tags_49 },
    { 0xe00a09a6, SUC_lexicon_tags_2 },
    { 0xd32a09a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2a09ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ca09b0, SUC_lexicon_tags_31 },
    { 0xf82a09b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2a09b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaa09bb, SUC_lexicon_tags_37 },
    { 0x91ca09bc, SUC_lexicon_tags_17 },
    { 0x30ca09bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x600a09c0, SUC_lexicon_tags_60 },
    { 0x472a09c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45aa09c6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52a09ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4a09cd, SUC_lexicon_tags_25 },
    { 0x332a09cd, SUC_lexicon_tags_15 },
    { 0xf8aa09cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180a09d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf88a09d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ca09d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ea09df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7ca09e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ca09e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0a09ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80a09f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x434a09fb, SUC_lexicon_tags_60 },
    { 0x506a09fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x346a09ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4a0a04, SUC_lexicon_tags_37 },
    { 0x52ca0a05, SUC_lexicon_tags_68 },
    { 0xcfea0a05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcca0a0b, SUC_lexicon_tags_72 },
    { 0xdb4a0a0b, SUC_lexicon_tags_62 },
    { 0x478a0a0d, SUC_lexicon_tags_7 },
    { 0x5b8a0a0b, SUC_lexicon_tags_27 },
    { 0x64ca0a0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca0a12, SUC_lexicon_tags_47 },
    { 0xa6a0a12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaca0a16, SUC_lexicon_tags_7 },
    { 0x51aa0a17, SUC_lexicon_tags_356 },
    { 0x410a0a16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4caa0a1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf50a0a1c, SUC_lexicon_tags_91 },
    { 0x1caa0a1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x764a0a1f, SUC_lexicon_tags_2 },
    { 0xc0aa0a20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c8a0a22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344a0a2c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9a4a0a2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2aca0a30, SUC_lexicon_tags_24 },
    { 0xb88a0a30, SUC_lexicon_tags_29 },
    { 0x66ca0a32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x962a0a34, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeb6a0a36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x668a0a38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bea0a40, SUC_lexicon_tags_0 },
    { 0x51ca0a41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96a0a4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224a0a52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ca0a57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce0a0a59, SUC_lexicon_tags_63 },
    { 0x156a0a59, SUC_lexicon_tags_25 },
    { 0xffaa0a5b, SUC_lexicon_tags_9 },
    { 0xdbca0a5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x536a0a5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56a0a67, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x638a0a69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81ca0a6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b2a0a6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6a0a75, SUC_lexicon_tags_47 },
    { 0xd6ca0a75, SUC_lexicon_tags_38 },
    { 0x654a0a77, SUC_lexicon_tags_8 },
    { 0x582a0a78, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdaca0a7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804a0a80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea0a86, SUC_lexicon_tags_14 },
    { 0x3b0a0a86, SUC_lexicon_tags_4 },
    { 0xee2a0a88, SUC_lexicon_tags_19 },
    { 0x13ca0a89, SUC_lexicon_tags_25 },
    { 0xdb6a0a8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154a0a90, SUC_lexicon_tags_1 },
    { 0x954a0a91, SUC_lexicon_tags_351 },
    { 0x378a0a92, SUC_lexicon_tags_9 },
    { 0xe4a0a93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50a0a96, SUC_lexicon_tags_17 },
    { 0x282a0a97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06a0a9a, SUC_lexicon_tags_28 },
    { 0xb6ea0a9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216a0a9f, SUC_lexicon_tags_8 },
    { 0xbd2a0a9f, SUC_lexicon_tags_9 },
    { 0xb78a0aa1, SUC_lexicon_tags_37 },
    { 0xddca0a9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x346a0aa4, SUC_lexicon_tags_11 },
    { 0x4c4a0aa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaa0aa9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ca0aac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x130a0aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x478a0ab1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x960a0ab4, SUC_lexicon_tags_28 },
    { 0x280a0ab4, SUC_lexicon_tags_38 },
    { 0x4f2a0ab6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306a0abf, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9daa0ac1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ea0ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3aa0ac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aea0ac9, SUC_lexicon_tags_11 },
    { 0x3f0a0ac9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ca0acd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeaca0acf, SUC_lexicon_tags_1 },
    { 0x42a0ad0, SUC_lexicon_tags_37 },
    { 0x61aa0acf, SUC_lexicon_tags_47 },
    { 0x85ea0acf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe98a0ad4, SUC_lexicon_tags_37 },
    { 0x11aa0ad5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe86a0ad7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0a0adc, SUC_lexicon_tags_6 },
    { 0xc8a0add, SUC_lexicon_tags_1 },
    { 0xc1aa0adc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc24a0ae0, SUC_lexicon_tags_33 },
    { 0xab4a0ae1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xddea0ae4, SUC_lexicon_tags_45 },
    { 0xa00a0ae4, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8a0ae8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30a0aeb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556a0af0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x568a0af2, SUC_lexicon_tags_19 },
    { 0xe6aa0af3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe96a0af5, SUC_lexicon_tags_17 },
    { 0xcb8a0af6, SUC_lexicon_tags_7 },
    { 0x782a0af7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee8a0af9, SUC_lexicon_tags_2 },
    { 0xedaa0afa, SUC_lexicon_tags_2 },
    { 0xa30a0afb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6a0b00, SUC_lexicon_tags_14 },
    { 0x518a0b01, SUC_lexicon_tags_25 },
    { 0x19aa0b02, SUC_lexicon_tags_60 },
    { 0xc72a0b02, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716a0b09, SUC_lexicon_tags_646 },
    { 0xd16a0b0a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76a0b0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4a0b10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca0b17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10a0b1b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x346a0b1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdaa0b1f, SUC_lexicon_tags_12 },
    { 0xacca0b20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dea0b26, SUC_lexicon_tags_15 },
    { 0x456a0b27, SUC_lexicon_tags_1 },
    { 0x1aaa0b26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3ea0b2a, SUC_lexicon_tags_12 },
    { 0x2dea0b2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6ca0b2d, SUC_lexicon_tags_69 },
    { 0xb0ea0b2e, SUC_lexicon_tags_79 },
    { 0x88a0b2f, SUC_lexicon_tags_65 },
    { 0xa32a0b2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe72a0b32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf6ea0b34, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x688a0b36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x278a0b38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafea0b3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0a0b47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8caa0b49, SUC_lexicon_tags_30 },
    { 0x1a0a0b4a, SUC_lexicon_tags_17 },
    { 0xcea0b4b, SUC_lexicon_tags_17 },
    { 0x472a0b4c, SUC_lexicon_tags_14 },
    { 0xdb8a0b4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6a0b50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02a0b56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486a0b5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10a0b5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x44aa0b61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6a0b64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59ca0b66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fca0b68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a0b6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858a0b77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x758a0b7a, SUC_lexicon_tags_1 },
    { 0xb84a0b7b, SUC_lexicon_tags_15 },
    { 0x680a0b7b, SUC_lexicon_tags_5 },
    { 0x9f0a0b7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4a0b81, SUC_lexicon_tags_39 },
    { 0xe4ca0b82, SUC_lexicon_tags_15 },
    { 0x44ea0b83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x206a0b86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e4a0b88, SUC_lexicon_tags_23 },
    { 0xeeaa0b89, SUC_lexicon_tags_1 },
    { 0x51ea0b89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4a0b8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca0b92, SUC_lexicon_tags_9 },
    { 0x124a0b93, SUC_lexicon_tags_45 },
    { 0x628a0b93, SUC_lexicon_tags_17 },
    { 0x4a2a0b95, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12a0b98, SUC_lexicon_tags_47 },
    { 0xc50a0b98, SUC_lexicon_tags_30 },
    { 0xbdaa0b9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ca0ba6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8a6a0ba8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ca0bab, SUC_lexicon_tags_23 },
    { 0x784a0bac, SUC_lexicon_tags_13 },
    { 0xe1ca0bac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aea0bb1, SUC_lexicon_tags_43 },
    { 0x342a0bb2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646a0bb7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc48a0bb9, SUC_lexicon_tags_25 },
    { 0x234a0bb9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6a0bbd, SUC_lexicon_tags_17 },
    { 0x208a0bbe, SUC_lexicon_tags_28 },
    { 0x358a0bbf, SUC_lexicon_tags_11 },
    { 0xd60a0bc0, SUC_lexicon_tags_12 },
    { 0x60aa0bbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc8a0bc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946a0bc9, SUC_lexicon_tags_42 },
    { 0xb7ca0bca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x274a0bcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6a0bd2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca0bd9, SUC_lexicon_tags_77 },
    { 0x68ca0bd9, SUC_lexicon_tags_11 },
    { 0x992a0bdb, SUC_lexicon_tags_17 },
    { 0xd18a0bdb, SUC_lexicon_tags_7 },
    { 0xecea0bdd, SUC_lexicon_tags_2 },
    { 0x76a0bde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b0a0be0, SUC_lexicon_tags_15 },
    { 0xcca0be1, SUC_lexicon_tags_35 },
    { 0xd58a0be2, SUC_lexicon_tags_96 },
    { 0xd7aa0be0, SUC_lexicon_tags_1 },
    { 0xfca0be4, SUC_lexicon_tags_7 },
    { 0xffea0be5, SUC_lexicon_tags_11 },
    { 0x8f6a0be3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a0bec, SUC_lexicon_tags_25 },
    { 0x60a0bed, SUC_lexicon_tags_2 },
    { 0x688a0bed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x578a0bf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4a0bf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aa0bf7, SUC_lexicon_tags_48 },
    { 0x80a0bf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a0bfc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x790a0bff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4a0c02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x666a0c04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x36ea0c06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd4ca0c08, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ca0c0c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x664a0c0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x582a0c10, SUC_lexicon_tags_14 },
    { 0xbfaa0c10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ea0c14, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490a0c19, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0a0c1c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x30ea0c1e, SUC_lexicon_tags_1 },
    { 0x2c0a0c1f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x930a0c22, SUC_lexicon_tags_17 },
    { 0xdbca0c23, SUC_lexicon_tags_37 },
    { 0x394a0c23, SUC_lexicon_tags_14 },
    { 0xacca0c24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa36a0c27, SUC_lexicon_tags_56 },
    { 0xd6a0c28, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x912a0c2a, SUC_lexicon_tags_12 },
    { 0xe16a0c2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbfea0c2d, SUC_lexicon_tags_82 },
    { 0xfdea0c2e, SUC_lexicon_tags_105 },
    { 0xfc2a0c2e, SUC_lexicon_tags_25 },
    { 0xc7ca0c2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ea0c33, SUC_lexicon_tags_47 },
    { 0xbc8a0c34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc12a0c36, SUC_lexicon_tags_6 },
    { 0xabaa0c37, SUC_lexicon_tags_7 },
    { 0x9ca0c38, SUC_lexicon_tags_27 },
    { 0x89ea0c37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744a0c3d, SUC_lexicon_tags_11 },
    { 0xbf0a0c3d, SUC_lexicon_tags_7 },
    { 0x9fea0c3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x734a0c41, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878a0c48, SUC_lexicon_tags_62 },
    { 0x252a0c49, SUC_lexicon_tags_47 },
    { 0xd92a0c49, SUC_lexicon_tags_8 },
    { 0xfaa0c4a, SUC_lexicon_tags_56 },
    { 0x758a0c4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44a0c53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea0c57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6a0c5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5aa0c60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ea0c64, SUC_lexicon_tags_2 },
    { 0xf8aa0c65, SUC_lexicon_tags_2 },
    { 0x92ea0c64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd58a0c68, SUC_lexicon_tags_45 },
    { 0x45aa0c69, SUC_lexicon_tags_31 },
    { 0x79ca0c6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca0c6e, SUC_lexicon_tags_45 },
    { 0xec0a0c6f, SUC_lexicon_tags_43 },
    { 0xcb0a0c70, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1c8a0c72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50a0c75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfbca0c77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856a0c7c, SUC_lexicon_tags_37 },
    { 0xf78a0c7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca0c80, SUC_lexicon_tags_1 },
    { 0xebca0c80, SUC_lexicon_tags_25 },
    { 0x78ea0c82, SUC_lexicon_tags_15 },
    { 0xde0a0c82, SUC_lexicon_tags_1 },
    { 0xe58a0c83, SUC_lexicon_tags_57 },
    { 0xe98a0c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6ea0c87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x386a0c89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7b4a0c8b, SUC_lexicon_tags_12 },
    { 0x126a0c8b, SUC_lexicon_tags_1 },
    { 0x610a0c8d, SUC_lexicon_tags_437 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7faa0c91, SUC_lexicon_tags_85 },
    { 0xe10a0c91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dea0c97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56a0c9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4a0c9e, SUC_lexicon_tags_45 },
    { 0x2aaa0c9f, SUC_lexicon_tags_45 },
    { 0x14a0ca0, SUC_lexicon_tags_47 },
    { 0x4e4a0ca1, SUC_lexicon_tags_12 },
    { 0x870a0ca0, SUC_lexicon_tags_46 },
    { 0x728a0ca0, SUC_lexicon_tags_17 },
    { 0xe2ea0c9f, SUC_lexicon_tags_12 },
    { 0xad0a0ca5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6a0cab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2a0cb5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ea0cb9, SUC_lexicon_tags_25 },
    { 0x7c0a0cb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c8a0cbc, SUC_lexicon_tags_66 },
    { 0x504a0cbd, SUC_lexicon_tags_25 },
    { 0x10ca0cbd, SUC_lexicon_tags_8 },
    { 0x6aea0cbf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf56a0cc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ca0cc3, SUC_lexicon_tags_56 },
    { 0x6eaa0cc4, SUC_lexicon_tags_17 },
    { 0xc5ca0cc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49aa0cc7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c4a0cc9, SUC_lexicon_tags_3 },
    { 0x640a0cc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x436a0ccd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ea0cd1, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xa42a0cd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92a0cd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2a0cd8, SUC_lexicon_tags_25 },
    { 0xfe4a0cd9, SUC_lexicon_tags_25 },
    { 0x1caa0cd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbca0cdd, SUC_lexicon_tags_49 },
    { 0xffaa0cde, SUC_lexicon_tags_35 },
    { 0x940a0cdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6a0ce3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe78a0ce5, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x334a0ce7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94a0cea, SUC_lexicon_tags_653 },
    { 0xb8a0ceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43aa0cee, SUC_lexicon_tags_7 },
    { 0x278a0cee, SUC_lexicon_tags_2 },
    { 0xd66a0cf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaca0cf5, SUC_lexicon_tags_28 },
    { 0xda6a0cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ca0cfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaa0d02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96a0d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248a0d09, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x636a0d0c, SUC_lexicon_tags_49 },
    { 0xe20a0d0d, SUC_lexicon_tags_11 },
    { 0xd58a0d0d, SUC_lexicon_tags_55 },
    { 0xf40a0d0f, SUC_lexicon_tags_15 },
    { 0x724a0d0f, SUC_lexicon_tags_39 },
    { 0xe40a0d11, SUC_lexicon_tags_33 },
    { 0x950a0d11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36a0d17, SUC_lexicon_tags_1 },
    { 0x956a0d18, SUC_lexicon_tags_27 },
    { 0xe0a0d18, SUC_lexicon_tags_48 },
    { 0x308a0d1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd50a0d1c, SUC_lexicon_tags_39 },
    { 0xd28a0d1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74a0d22, SUC_lexicon_tags_8 },
    { 0xe58a0d23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x676a0d25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x250a0d28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bca0d33, SUC_lexicon_tags_1 },
    { 0x99ea0d33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d6a0d36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804a0d3a, SUC_lexicon_tags_4 },
    { 0x96ca0d3b, SUC_lexicon_tags_15 },
    { 0x158a0d3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x314a0d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaa0d47, SUC_lexicon_tags_25 },
    { 0x5cea0d47, SUC_lexicon_tags_62 },
    { 0x74aa0d48, SUC_lexicon_tags_47 },
    { 0x346a0d47, SUC_lexicon_tags_38 },
    { 0x158a0d4b, SUC_lexicon_tags_14 },
    { 0x1c6a0d4b, SUC_lexicon_tags_25 },
    { 0x100a0d4d, SUC_lexicon_tags_37 },
    { 0x666a0d4e, SUC_lexicon_tags_43 },
    { 0x826a0d4f, SUC_lexicon_tags_17 },
    { 0xd52a0d50, SUC_lexicon_tags_17 },
    { 0xd0aa0d50, SUC_lexicon_tags_14 },
    { 0xaa2a0d52, SUC_lexicon_tags_12 },
    { 0xdca0d50, SUC_lexicon_tags_37 },
    { 0x248a0d4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1ca0d56, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aa0d5a, SUC_lexicon_tags_15 },
    { 0x282a0d5a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x274a0d5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192a0d63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0a0d66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x810a0d69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27ea0d6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120a0d6f, SUC_lexicon_tags_9 },
    { 0x28ea0d70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592a0d74, SUC_lexicon_tags_4 },
    { 0xcb8a0d74, SUC_lexicon_tags_25 },
    { 0xec4a0d76, SUC_lexicon_tags_49 },
    { 0xebaa0d77, SUC_lexicon_tags_12 },
    { 0x226a0d76, SUC_lexicon_tags_47 },
    { 0xe36a0d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ea0d7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94a0d7f, SUC_lexicon_tags_84 },
    { 0xc8ca0d80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x282a0d82, SUC_lexicon_tags_65 },
    { 0x51aa0d83, SUC_lexicon_tags_63 },
    { 0x554a0d83, SUC_lexicon_tags_14 },
    { 0xcd2a0d85, SUC_lexicon_tags_11 },
    { 0xe44a0d85, SUC_lexicon_tags_1 },
    { 0x65ea0d86, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8f4a0d89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x364a0d8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0a0d8f, SUC_lexicon_tags_1 },
    { 0x67ca0d90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe34a0d92, SUC_lexicon_tags_7 },
    { 0x53aa0d92, SUC_lexicon_tags_7 },
    { 0x3c6a0d94, SUC_lexicon_tags_17 },
    { 0x560a0d94, SUC_lexicon_tags_17 },
    { 0x19ea0d92, SUC_lexicon_tags_1 },
    { 0xc90a0d92, SUC_lexicon_tags_2 },
    { 0x714a0d98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa32a0d9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728a0da2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x146a0da5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd3aa0da7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406a0dad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ca0db0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4a0dbb, SUC_lexicon_tags_2 },
    { 0x3e6a0dbc, SUC_lexicon_tags_7 },
    { 0x84ca0dbc, SUC_lexicon_tags_37 },
    { 0x4eaa0dbd, SUC_lexicon_tags_28 },
    { 0xf92a0dbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfea0dc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x948a0dc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2a0dc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x60ea0dca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dca0dd0, SUC_lexicon_tags_15 },
    { 0x9b2a0dd0, SUC_lexicon_tags_49 },
    { 0x462a0dd2, SUC_lexicon_tags_12 },
    { 0xd6a0dd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88a0ddb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x512a0ddd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa52a0ddf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x824a0de1, SUC_lexicon_tags_8 },
    { 0xdaa0de2, SUC_lexicon_tags_30 },
    { 0x216a0de3, SUC_lexicon_tags_15 },
    { 0x65ea0de4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664a0deb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6a0df9, SUC_lexicon_tags_11 },
    { 0x772a0dfa, SUC_lexicon_tags_17 },
    { 0x968a0dfb, SUC_lexicon_tags_11 },
    { 0xf7aa0dfc, SUC_lexicon_tags_7 },
    { 0x7e4a0dfd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xb8ca0dff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c8a0e01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e2a0e03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a0e07, SUC_lexicon_tags_43 },
    { 0xcf6a0e08, SUC_lexicon_tags_25 },
    { 0x6e6a0e09, SUC_lexicon_tags_17 },
    { 0x778a0e0a, SUC_lexicon_tags_56 },
    { 0x5c2a0e09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ea0e0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2a0e13, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd0a0e15, SUC_lexicon_tags_67 },
    { 0x24ea0e15, SUC_lexicon_tags_2 },
    { 0x23ca0e17, SUC_lexicon_tags_92 },
    { 0x7a4a0e18, SUC_lexicon_tags_29 },
    { 0xca0a0e15, SUC_lexicon_tags_69 },
    { 0xabea0e15, SUC_lexicon_tags_11 },
    { 0xe6a0e1b, SUC_lexicon_tags_12 },
    { 0xdf6a0e1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2a0e1f, SUC_lexicon_tags_12 },
    { 0x94a0e1f, SUC_lexicon_tags_1 },
    { 0x1caa0e1f, SUC_lexicon_tags_17 },
    { 0xfa2a0e22, SUC_lexicon_tags_11 },
    { 0x16ca0e22, SUC_lexicon_tags_45 },
    { 0x1e6a0e20, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08a0e27, SUC_lexicon_tags_5 },
    { 0x80ea0e28, SUC_lexicon_tags_11 },
    { 0xd60a0e29, SUC_lexicon_tags_8 },
    { 0xa44a0e2a, SUC_lexicon_tags_15 },
    { 0x1e6a0e2b, SUC_lexicon_tags_15 },
    { 0x8aa0e28, SUC_lexicon_tags_25 },
    { 0x566a0e2d, SUC_lexicon_tags_0 },
    { 0xf22a0e2d, SUC_lexicon_tags_28 },
    { 0x232a0e2e, SUC_lexicon_tags_25 },
    { 0x9eca0e30, SUC_lexicon_tags_56 },
    { 0x16aa0e29, SUC_lexicon_tags_47 },
    { 0x9aaa0e30, SUC_lexicon_tags_1 },
    { 0xd46a0e29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x156a0e35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8a0e38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60aa0e3e, SUC_lexicon_tags_27 },
    { 0xcf4a0e3f, SUC_lexicon_tags_38 },
    { 0x20a0e40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x580a0e42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06a0e4a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94a0e4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6baa0e50, SUC_lexicon_tags_38 },
    { 0x260a0e51, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848a0e59, SUC_lexicon_tags_17 },
    { 0x272a0e5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaea0e5f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74a0e71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa0e75, SUC_lexicon_tags_60 },
    { 0xb3ea0e76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a0e7a, SUC_lexicon_tags_25 },
    { 0x38a0e7b, SUC_lexicon_tags_1 },
    { 0xa4ea0e7c, SUC_lexicon_tags_12 },
    { 0x6eea0e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4a0e80, SUC_lexicon_tags_31 },
    { 0x3eaa0e81, SUC_lexicon_tags_60 },
    { 0xd58a0e82, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6a0e85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ca0e89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34a0e92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74a0e9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92aa0ead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaea0eb1, SUC_lexicon_tags_11 },
    { 0x86a0eb2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ca0eb8, SUC_lexicon_tags_45 },
    { 0x294a0eb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92a0ebe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20a0ec1, SUC_lexicon_tags_3 },
    { 0x46a0ec1, SUC_lexicon_tags_8 },
    { 0x1a8a0ec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x950a0ec6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3fea0ec8, SUC_lexicon_tags_11 },
    { 0x31ca0ec9, SUC_lexicon_tags_49 },
    { 0x59ea0ec8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460a0ecf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78a0edb, SUC_lexicon_tags_11 },
    { 0xc78a0edc, SUC_lexicon_tags_23 },
    { 0x934a0edd, SUC_lexicon_tags_15 },
    { 0x76a0ede, SUC_lexicon_tags_17 },
    { 0x21ea0edf, SUC_lexicon_tags_65 },
    { 0x8d8a0edc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8a0ee9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712a0ef1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ea0ef4, SUC_lexicon_tags_45 },
    { 0xb60a0ef5, SUC_lexicon_tags_15 },
    { 0xeeaa0ef6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x472a0ef9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x65aa0efc, SUC_lexicon_tags_28 },
    { 0xc36a0efd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314a0f02, SUC_lexicon_tags_1 },
    { 0xd8ea0f02, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2a0f09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc18a0f0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b0a0f0d, SUC_lexicon_tags_17 },
    { 0x1aa0f0e, SUC_lexicon_tags_151 },
    { 0xe08a0f0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0a0f12, SUC_lexicon_tags_12 },
    { 0xcca0f12, SUC_lexicon_tags_17 },
    { 0x1e0a0f14, SUC_lexicon_tags_25 },
    { 0xb7aa0f14, SUC_lexicon_tags_8 },
    { 0x44ea0f14, SUC_lexicon_tags_37 },
    { 0xebea0f17, SUC_lexicon_tags_30 },
    { 0xa2aa0f18, SUC_lexicon_tags_9 },
    { 0xe62a0f19, SUC_lexicon_tags_16 },
    { 0xa4ca0f14, SUC_lexicon_tags_17 },
    { 0x840a0f12, SUC_lexicon_tags_15 },
    { 0x5f2a0f1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10a0f1f, SUC_lexicon_tags_22 },
    { 0xe62a0f20, SUC_lexicon_tags_63 },
    { 0x6a0a0f21, SUC_lexicon_tags_15 },
    { 0x5c2a0f1f, SUC_lexicon_tags_14 },
    { 0xae2a0f1f, SUC_lexicon_tags_7 },
    { 0x418a0f24, SUC_lexicon_tags_39 },
    { 0x890a0f24, SUC_lexicon_tags_25 },
    { 0xfb6a0f25, SUC_lexicon_tags_11 },
    { 0xd56a0f26, SUC_lexicon_tags_47 },
    { 0x3e6a0f27, SUC_lexicon_tags_12 },
    { 0xc6ca0f20, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ca0f2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaa0f2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2a0f35, SUC_lexicon_tags_31 },
    { 0xb48a0f36, SUC_lexicon_tags_3 },
    { 0xacca0f37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2a0f3a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aa0f3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc08a0f40, SUC_lexicon_tags_17 },
    { 0xc0a0f41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41aa0f43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aa0f49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe4a0f4b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc26a0f4d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52a0f50, SUC_lexicon_tags_12 },
    { 0xf44a0f50, SUC_lexicon_tags_113 },
    { 0x6eea0f52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474a0f57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce6a0f59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x480a0f5b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728a0f64, SUC_lexicon_tags_1 },
    { 0xddaa0f65, SUC_lexicon_tags_27 },
    { 0x1eea0f66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a0f69, SUC_lexicon_tags_2 },
    { 0x436a0f6a, SUC_lexicon_tags_25 },
    { 0x70ca0f69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508a0f72, SUC_lexicon_tags_31 },
    { 0xd7ca0f73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x964a0f75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8caa0f77, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6a0f7a, SUC_lexicon_tags_7 },
    { 0xf4ca0f7a, SUC_lexicon_tags_14 },
    { 0xd4a0f7b, SUC_lexicon_tags_14 },
    { 0x69aa0f7d, SUC_lexicon_tags_17 },
    { 0xc3ca0f7e, SUC_lexicon_tags_10 },
    { 0x9c8a0f7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74a0f84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696a0f88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8a0f8b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28a0f8f, SUC_lexicon_tags_8 },
    { 0x802a0f90, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ea0f94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ea0f97, SUC_lexicon_tags_27 },
    { 0x572a0f97, SUC_lexicon_tags_9 },
    { 0x4d2a0f97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa64a0f9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ea0f9f, SUC_lexicon_tags_45 },
    { 0x67ea0fa0, SUC_lexicon_tags_37 },
    { 0x33ea0fa1, SUC_lexicon_tags_19 },
    { 0x9baa0f9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ca0fa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80a0fa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a0fb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aaa0fb5, SUC_lexicon_tags_27 },
    { 0x138a0fb6, SUC_lexicon_tags_118 },
    { 0xb52a0fb7, SUC_lexicon_tags_8 },
    { 0x940a0fb7, SUC_lexicon_tags_33 },
    { 0xdb0a0fb9, SUC_lexicon_tags_25 },
    { 0x8f0a0fba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd44a0fbc, SUC_lexicon_tags_25 },
    { 0x80a0fbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ea0fc3, SUC_lexicon_tags_9 },
    { 0xe34a0fc3, SUC_lexicon_tags_0 },
    { 0xbc8a0fc4, SUC_lexicon_tags_0 },
    { 0x408a0fc6, SUC_lexicon_tags_25 },
    { 0x264a0fc7, SUC_lexicon_tags_37 },
    { 0x640a0fc8, SUC_lexicon_tags_65 },
    { 0x22aa0fc8, SUC_lexicon_tags_68 },
    { 0x4cea0fca, SUC_lexicon_tags_10 },
    { 0x5c2a0fcb, SUC_lexicon_tags_38 },
    { 0xd4ca0fca, SUC_lexicon_tags_17 },
    { 0x710a0fcd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ea0fd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x746a0fd2, SUC_lexicon_tags_57 },
    { 0x9b6a0fd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87ca0fd5, SUC_lexicon_tags_16 },
    { 0x4f0a0fd6, SUC_lexicon_tags_17 },
    { 0xacca0fd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31ea0fd9, SUC_lexicon_tags_25 },
    { 0x3a4a0fda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60a0fdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd0a0fdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6a0fe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x346a0fe6, SUC_lexicon_tags_17 },
    { 0x5f0a0fe7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd56a0fe9, SUC_lexicon_tags_19 },
    { 0x256a0fea, SUC_lexicon_tags_37 },
    { 0xfe6a0feb, SUC_lexicon_tags_2 },
    { 0x7eea0fec, SUC_lexicon_tags_7 },
    { 0xd4ea0fed, SUC_lexicon_tags_49 },
    { 0xe40a0fee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a0ff3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180a0ff9, SUC_lexicon_tags_37 },
    { 0x2faa0ffa, SUC_lexicon_tags_11 },
    { 0x9aaa0ffb, SUC_lexicon_tags_37 },
    { 0x2aa0ffa, SUC_lexicon_tags_0 },
    { 0xf0a0ffc, SUC_lexicon_tags_37 },
    { 0x19ea0ffc, SUC_lexicon_tags_32 },
    { 0x7f4a0fff, SUC_lexicon_tags_63 },
    { 0xa5aa1000, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ea1003, SUC_lexicon_tags_55 },
    { 0x49ea1003, SUC_lexicon_tags_12 },
    { 0x592a1005, SUC_lexicon_tags_25 },
    { 0xfdea1006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802a100b, SUC_lexicon_tags_22 },
    { 0xda4a100c, SUC_lexicon_tags_14 },
    { 0xc34a100c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12a1013, SUC_lexicon_tags_0 },
    { 0xb76a1014, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640a1018, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04a101c, SUC_lexicon_tags_35 },
    { 0xfeaa101c, SUC_lexicon_tags_11 },
    { 0xdd6a101e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ca1021, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4a1024, SUC_lexicon_tags_33 },
    { 0x542a1025, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04a1028, SUC_lexicon_tags_9 },
    { 0xc7aa1029, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaaca102b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a103a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1aa103c, SUC_lexicon_tags_28 },
    { 0x492a103d, SUC_lexicon_tags_35 },
    { 0x6d0a103c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x352a1040, SUC_lexicon_tags_31 },
    { 0xf66a1040, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952a1045, SUC_lexicon_tags_380 },
    { 0x784a1046, SUC_lexicon_tags_30 },
    { 0xb12a1047, SUC_lexicon_tags_8 },
    { 0x3aea1048, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc36a104a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfaea104c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8a1050, SUC_lexicon_tags_38 },
    { 0x29ea1051, SUC_lexicon_tags_53 },
    { 0x52a1052, SUC_lexicon_tags_14 },
    { 0xad4a1053, SUC_lexicon_tags_2 },
    { 0x5eaa1054, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca1057, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43aa105c, SUC_lexicon_tags_37 },
    { 0x3faa105d, SUC_lexicon_tags_10 },
    { 0xd2aa105e, SUC_lexicon_tags_28 },
    { 0xcc6a105c, SUC_lexicon_tags_17 },
    { 0x196a1060, SUC_lexicon_tags_7 },
    { 0x2eca105c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x600a1063, SUC_lexicon_tags_14 },
    { 0xb68a1064, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x56a1066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeeea1068, SUC_lexicon_tags_87 },
    { 0x3f4a1069, SUC_lexicon_tags_11 },
    { 0xefea1069, SUC_lexicon_tags_26 },
    { 0x6f8a106b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8a1070, SUC_lexicon_tags_30 },
    { 0x9d2a1070, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6a1078, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fca107e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324a1085, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678a1089, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x912a108b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77ca108d, SUC_lexicon_tags_67 },
    { 0x694a108d, SUC_lexicon_tags_1 },
    { 0xdc4a108d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfca1092, SUC_lexicon_tags_11 },
    { 0x8bca1092, SUC_lexicon_tags_27 },
    { 0x928a1094, SUC_lexicon_tags_83 },
    { 0xfeca1095, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6a1098, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaa10a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31ca10a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac4a10a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1faa10ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a10ae, SUC_lexicon_tags_4 },
    { 0xfcaa10af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8a10b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0a10ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x670a10bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32a10c3, SUC_lexicon_tags_26 },
    { 0x312a10c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8a10c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6aa10cc, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0x56aa10ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ea10d3, SUC_lexicon_tags_4 },
    { 0xcf0a10d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112a10da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6a10df, SUC_lexicon_tags_27 },
    { 0x852a10e0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x424a10e2, SUC_lexicon_tags_37 },
    { 0xc0ca10e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70ca10e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b6a10e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4a10eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccca10f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb62a10f2, SUC_lexicon_tags_7 },
    { 0x76ca10f3, SUC_lexicon_tags_14 },
    { 0x1e8a10f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca10f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6a10fc, SUC_lexicon_tags_26 },
    { 0x142a10fd, SUC_lexicon_tags_87 },
    { 0x208a10fd, SUC_lexicon_tags_2 },
    { 0x2dea10ff, SUC_lexicon_tags_10 },
    { 0xeb4a1100, SUC_lexicon_tags_6 },
    { 0xeea10fd, SUC_lexicon_tags_7 },
    { 0x982a10fe, SUC_lexicon_tags_151 },
    { 0x4d4a1103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a1107, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184a110b, SUC_lexicon_tags_0 },
    { 0x1d8a110c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3eca110e, SUC_lexicon_tags_4 },
    { 0x408a110f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x358a1111, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04a1114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x986a1116, SUC_lexicon_tags_48 },
    { 0xaf4a1116, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc4a1119, SUC_lexicon_tags_43 },
    { 0xbb6a111a, SUC_lexicon_tags_6 },
    { 0x2faa111b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ea111e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4cca1120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdca1123, SUC_lexicon_tags_17 },
    { 0x4d4a1124, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a1127, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548a112d, SUC_lexicon_tags_30 },
    { 0xb64a112e, SUC_lexicon_tags_33 },
    { 0x772a112e, SUC_lexicon_tags_1040 },
    { 0x952a112f, SUC_lexicon_tags_74 },
    { 0xbc6a1131, SUC_lexicon_tags_31 },
    { 0x9f6a1132, SUC_lexicon_tags_160 },
    { 0xc18a1133, SUC_lexicon_tags_9 },
    { 0x174a112d, SUC_lexicon_tags_49 },
    { 0x3e2a1132, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x922a1138, SUC_lexicon_tags_37 },
    { 0x3f0a1139, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0a113e, SUC_lexicon_tags_17 },
    { 0x84ea113f, SUC_lexicon_tags_7 },
    { 0xbd4a1140, SUC_lexicon_tags_15 },
    { 0xa4ca1141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x318a1143, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd56a1145, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x940a1148, SUC_lexicon_tags_64 },
    { 0x36ea1149, SUC_lexicon_tags_7 },
    { 0x9a0a114a, SUC_lexicon_tags_104 },
    { 0x66ea114b, SUC_lexicon_tags_1 },
    { 0xcfca114c, SUC_lexicon_tags_7 },
    { 0x7d0a114c, SUC_lexicon_tags_7 },
    { 0x724a1149, SUC_lexicon_tags_33 },
    { 0x8e6a114d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2a1151, SUC_lexicon_tags_11 },
    { 0x974a1152, SUC_lexicon_tags_273 },
    { 0xedea1152, SUC_lexicon_tags_37 },
    { 0x2e4a1154, SUC_lexicon_tags_82 },
    { 0x9e2a1154, SUC_lexicon_tags_33 },
    { 0x566a1153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394a115c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0a1160, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80a1163, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaa1166, SUC_lexicon_tags_64 },
    { 0x84a1167, SUC_lexicon_tags_114 },
    { 0xb20a1168, SUC_lexicon_tags_25 },
    { 0x8a8a1167, SUC_lexicon_tags_14 },
    { 0xe64a116a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278a116e, SUC_lexicon_tags_7 },
    { 0xe1ea116f, SUC_lexicon_tags_15 },
    { 0xbf0a116f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6a1173, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2a1176, SUC_lexicon_tags_12 },
    { 0x5b2a1177, SUC_lexicon_tags_31 },
    { 0xfa8a1178, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca117c, SUC_lexicon_tags_12 },
    { 0x92aa117d, SUC_lexicon_tags_29 },
    { 0x9f8a117e, SUC_lexicon_tags_6 },
    { 0x976a117f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06a1182, SUC_lexicon_tags_17 },
    { 0xecea1183, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa82a1185, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118a118a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52a1191, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x452a1194, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x282a1196, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78a119a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87aa11a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a11a8, SUC_lexicon_tags_23 },
    { 0xc60a11a9, SUC_lexicon_tags_37 },
    { 0xe50a11a9, SUC_lexicon_tags_187 },
    { 0x8c6a11a8, SUC_lexicon_tags_0 },
    { 0xfd6a11ab, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4a11b0, SUC_lexicon_tags_9 },
    { 0xf04a11b0, SUC_lexicon_tags_79 },
    { 0xf9ca11b2, SUC_lexicon_tags_27 },
    { 0xbeaa11b3, SUC_lexicon_tags_15 },
    { 0x238a11b4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a11ba, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29aa11bf, SUC_lexicon_tags_600 },
    { 0x486a11c0, SUC_lexicon_tags_2 },
    { 0x5eea11c1, SUC_lexicon_tags_12 },
    { 0x4f6a11c0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0a11c9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10a11d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x894a11d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca11d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca11da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaea11df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714a11e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82a11e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ea11f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2a11fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aa11ff, SUC_lexicon_tags_30 },
    { 0x60ea1200, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04a1205, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2a120a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaa1212, SUC_lexicon_tags_37 },
    { 0xc5aa1212, SUC_lexicon_tags_430 },
    { 0, NULL },
    { 0xbb8a1215, SUC_lexicon_tags_0 },
    { 0xd6ca1215, SUC_lexicon_tags_23 },
    { 0xf8a1217, SUC_lexicon_tags_37 },
    { 0x30a1218, SUC_lexicon_tags_12 },
    { 0x838a1216, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76a121f, SUC_lexicon_tags_17 },
    { 0x9c4a121f, SUC_lexicon_tags_1 },
    { 0x54ea1220, SUC_lexicon_tags_25 },
    { 0xa42a1222, SUC_lexicon_tags_65 },
    { 0x7c0a1223, SUC_lexicon_tags_28 },
    { 0x128a121f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x840a1227, SUC_lexicon_tags_37 },
    { 0x742a1227, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x526a122a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982a122e, SUC_lexicon_tags_6 },
    { 0x46aa122f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x118a1231, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaa1235, SUC_lexicon_tags_60 },
    { 0x522a1235, SUC_lexicon_tags_39 },
    { 0x4faa1237, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aca123b, SUC_lexicon_tags_47 },
    { 0x394a123c, SUC_lexicon_tags_11 },
    { 0x9eaa123c, SUC_lexicon_tags_48 },
    { 0xb92a123e, SUC_lexicon_tags_11 },
    { 0xc22a123f, SUC_lexicon_tags_79 },
    { 0x156a123e, SUC_lexicon_tags_11 },
    { 0x55aa123e, SUC_lexicon_tags_15 },
    { 0x14aa1242, SUC_lexicon_tags_28 },
    { 0x44aa1241, SUC_lexicon_tags_31 },
    { 0xf44a1244, SUC_lexicon_tags_17 },
    { 0x742a1245, SUC_lexicon_tags_33 },
    { 0x2b4a1246, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92a124b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812a124f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccea1253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x932a1255, SUC_lexicon_tags_45 },
    { 0x198a1256, SUC_lexicon_tags_17 },
    { 0x818a1257, SUC_lexicon_tags_2 },
    { 0xd1ca1258, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x356a125b, SUC_lexicon_tags_12 },
    { 0xa5ea125c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76a125f, SUC_lexicon_tags_17 },
    { 0x98aa1260, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6a1263, SUC_lexicon_tags_13 },
    { 0x3cca1264, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ca1267, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x956a1269, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bca126f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278a1276, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ca127c, SUC_lexicon_tags_25 },
    { 0x60ea127d, SUC_lexicon_tags_29 },
    { 0x236a127e, SUC_lexicon_tags_47 },
    { 0x36a127c, SUC_lexicon_tags_16 },
    { 0xcbca1280, SUC_lexicon_tags_46 },
    { 0x6e6a127e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710a1285, SUC_lexicon_tags_26 },
    { 0x7faa1285, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca128c, SUC_lexicon_tags_66 },
    { 0xd44a128c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4a1294, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852a129b, SUC_lexicon_tags_37 },
    { 0x7b0a129b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e0a129e, SUC_lexicon_tags_17 },
    { 0xf6aa129f, SUC_lexicon_tags_7 },
    { 0xf7ca129f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aa12a6, SUC_lexicon_tags_15 },
    { 0x364a12a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574a12ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444a12b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20a12b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e2a12b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedea12bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aca12c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd54a12c2, SUC_lexicon_tags_14 },
    { 0xc5aa12c3, SUC_lexicon_tags_4 },
    { 0xad0a12c4, SUC_lexicon_tags_2 },
    { 0xc44a12c5, SUC_lexicon_tags_45 },
    { 0xcaea12c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x836a12c8, SUC_lexicon_tags_11 },
    { 0x364a12c9, SUC_lexicon_tags_14 },
    { 0xd5aa12ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc46a12cc, SUC_lexicon_tags_47 },
    { 0x646a12cd, SUC_lexicon_tags_29 },
    { 0x814a12ce, SUC_lexicon_tags_25 },
    { 0x134a12cf, SUC_lexicon_tags_45 },
    { 0xcbca12cf, SUC_lexicon_tags_29 },
    { 0x4f4a12cf, SUC_lexicon_tags_2 },
    { 0xbe4a12ce, SUC_lexicon_tags_1 },
    { 0x7fca12d3, SUC_lexicon_tags_37 },
    { 0x998a12cd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcb2a12d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xad2a12d8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeea12df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754a12e3, SUC_lexicon_tags_23 },
    { 0x1f0a12e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3eca12e6, SUC_lexicon_tags_10 },
    { 0xa42a12e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x246a12ea, SUC_lexicon_tags_37 },
    { 0xa1ea12eb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd2a12ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ea12f1, SUC_lexicon_tags_37 },
    { 0x460a12f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0a12f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a4a12f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918a12fe, SUC_lexicon_tags_26 },
    { 0xc6aa12ff, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8a1302, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4a1306, SUC_lexicon_tags_14 },
    { 0x87aa1307, SUC_lexicon_tags_30 },
    { 0xee8a1306, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4a130b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44a130e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x476a1310, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbca1316, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6a1319, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ea131d, SUC_lexicon_tags_23 },
    { 0xd3ca131e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe78a1320, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ea1323, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378a132a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfea132c, SUC_lexicon_tags_28 },
    { 0xdf2a132c, SUC_lexicon_tags_23 },
    { 0xbc0a132e, SUC_lexicon_tags_38 },
    { 0xff6a132d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa02a1331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206a1337, SUC_lexicon_tags_38 },
    { 0xcc8a1337, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cea133c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fea1341, SUC_lexicon_tags_17 },
    { 0xe1ea1341, SUC_lexicon_tags_30 },
    { 0xc0ea1342, SUC_lexicon_tags_14 },
    { 0x1d2a1344, SUC_lexicon_tags_10 },
    { 0x598a1345, SUC_lexicon_tags_12 },
    { 0x1aaa1346, SUC_lexicon_tags_37 },
    { 0xf90a1347, SUC_lexicon_tags_11 },
    { 0x926a1348, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x398a134b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd4a134d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x672a134f, SUC_lexicon_tags_11 },
    { 0xa3ea1350, SUC_lexicon_tags_27 },
    { 0xb3ea1351, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628a1355, SUC_lexicon_tags_7 },
    { 0xb32a1356, SUC_lexicon_tags_9 },
    { 0xfc4a1355, SUC_lexicon_tags_42 },
    { 0xb00a1358, SUC_lexicon_tags_55 },
    { 0x57aa1359, SUC_lexicon_tags_62 },
    { 0x5aaa1356, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdca135c, SUC_lexicon_tags_27 },
    { 0xfdca135d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294a136a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540a136f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a1375, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ca1379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc8a137b, SUC_lexicon_tags_7 },
    { 0xbf2a137b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6a137f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4d6a1381, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92aa1384, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x95ca1386, SUC_lexicon_tags_30 },
    { 0xb66a1387, SUC_lexicon_tags_25 },
    { 0x2c4a1388, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa74a138a, SUC_lexicon_tags_14 },
    { 0x540a138b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2a1394, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5caa1398, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19ca139a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa0a139c, SUC_lexicon_tags_27 },
    { 0x784a139d, SUC_lexicon_tags_45 },
    { 0xf26a139e, SUC_lexicon_tags_15 },
    { 0xd6ea139c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2a13a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x468a13a5, SUC_lexicon_tags_13 },
    { 0x8dca13a5, SUC_lexicon_tags_33 },
    { 0xed0a13a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9ca13a9, SUC_lexicon_tags_28 },
    { 0x386a13aa, SUC_lexicon_tags_25 },
    { 0x188a13aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8a13ae, SUC_lexicon_tags_25 },
    { 0x6b0a13af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf0aa13b1, SUC_lexicon_tags_41 },
    { 0x300a13b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8a13bb, SUC_lexicon_tags_33 },
    { 0xa94a13bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5daa13be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x178a13c0, SUC_lexicon_tags_40 },
    { 0x30aa13c1, SUC_lexicon_tags_12 },
    { 0x994a13c1, SUC_lexicon_tags_15 },
    { 0xca2a13c1, SUC_lexicon_tags_27 },
    { 0xc62a13c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50a13c6, SUC_lexicon_tags_11 },
    { 0x4c0a13c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfa6a13c9, SUC_lexicon_tags_2 },
    { 0xd7ea13ca, SUC_lexicon_tags_287 },
    { 0xa48a13ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eea13ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f4a13d0, SUC_lexicon_tags_47 },
    { 0x24a13d0, SUC_lexicon_tags_1 },
    { 0x22a13d0, SUC_lexicon_tags_12 },
    { 0x87aa13d3, SUC_lexicon_tags_0 },
    { 0xc08a13d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8a13d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512a13dd, SUC_lexicon_tags_2 },
    { 0x666a13de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126a13e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2a13e5, SUC_lexicon_tags_14 },
    { 0x4eaa13e6, SUC_lexicon_tags_11 },
    { 0xce4a13e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32a13ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78a13ef, SUC_lexicon_tags_2 },
    { 0x41ea13f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56aa13f2, SUC_lexicon_tags_37 },
    { 0xf26a13f3, SUC_lexicon_tags_37 },
    { 0x1cea13f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x11ca13f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0a13fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684a13fe, SUC_lexicon_tags_0 },
    { 0xdc4a13ff, SUC_lexicon_tags_19 },
    { 0xa82a1400, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa1404, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8a1408, SUC_lexicon_tags_12 },
    { 0x6e8a1409, SUC_lexicon_tags_1 },
    { 0xd0ca140a, SUC_lexicon_tags_14 },
    { 0x5aa1409, SUC_lexicon_tags_28 },
    { 0x6d2a140a, SUC_lexicon_tags_43 },
    { 0x3c6a1409, SUC_lexicon_tags_2 },
    { 0x818a140e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa1411, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x510a1414, SUC_lexicon_tags_69 },
    { 0x374a1415, SUC_lexicon_tags_12 },
    { 0x238a1416, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4a1419, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02a141c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x37ca141e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a1422, SUC_lexicon_tags_11 },
    { 0x990a1422, SUC_lexicon_tags_27 },
    { 0x43ca1424, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0aa1429, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x438a142c, SUC_lexicon_tags_188 },
    { 0xf4ca142d, SUC_lexicon_tags_1 },
    { 0x5eea142e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8a1431, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbaaa1433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ca1437, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x858a1439, SUC_lexicon_tags_12 },
    { 0xea0a143a, SUC_lexicon_tags_25 },
    { 0x98a143b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaea143e, SUC_lexicon_tags_32 },
    { 0x2c4a143f, SUC_lexicon_tags_33 },
    { 0x3a8a1440, SUC_lexicon_tags_80 },
    { 0x9aca1441, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x966a1443, SUC_lexicon_tags_37 },
    { 0x35ca1444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8a1447, SUC_lexicon_tags_47 },
    { 0xfd2a1447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814a1453, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62a1459, SUC_lexicon_tags_0 },
    { 0x108a1459, SUC_lexicon_tags_92 },
    { 0xb9ca1459, SUC_lexicon_tags_12 },
    { 0x79ea145c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2a145f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42a1465, SUC_lexicon_tags_23 },
    { 0x7c0a1466, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0x822a1468, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa146b, SUC_lexicon_tags_25 },
    { 0xe9aa146c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742a1470, SUC_lexicon_tags_43 },
    { 0x536a1471, SUC_lexicon_tags_155 },
    { 0x3cea1472, SUC_lexicon_tags_11 },
    { 0xaaea1473, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc78a1475, SUC_lexicon_tags_10 },
    { 0x3c8a1476, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98aa147a, SUC_lexicon_tags_37 },
    { 0xe68a147b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95ea147d, SUC_lexicon_tags_1 },
    { 0x92ea147e, SUC_lexicon_tags_28 },
    { 0x124a147f, SUC_lexicon_tags_47 },
    { 0x51ea147f, SUC_lexicon_tags_25 },
    { 0xf32a147d, SUC_lexicon_tags_27 },
    { 0x49ea1481, SUC_lexicon_tags_39 },
    { 0x268a147d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822a1487, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ea148a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0a148e, SUC_lexicon_tags_30 },
    { 0xd6a148f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed2a1491, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50a1494, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xccca1496, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468a149d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x688a149f, SUC_lexicon_tags_43 },
    { 0x962a14a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x404a14a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdd2a14a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55aa14a7, SUC_lexicon_tags_37 },
    { 0x874a14a8, SUC_lexicon_tags_860 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6a14ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498a14b1, SUC_lexicon_tags_15 },
    { 0x80a14b2, SUC_lexicon_tags_23 },
    { 0x8e2a14b3, SUC_lexicon_tags_92 },
    { 0x35aa14b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6a14bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaeea14bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62a14c0, SUC_lexicon_tags_47 },
    { 0x91aa14c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0a14c6, SUC_lexicon_tags_25 },
    { 0x4a8a14c6, SUC_lexicon_tags_37 },
    { 0x14ca14c8, SUC_lexicon_tags_11 },
    { 0x396a14c9, SUC_lexicon_tags_12 },
    { 0x2f8a14c8, SUC_lexicon_tags_10 },
    { 0x2b6a14cb, SUC_lexicon_tags_25 },
    { 0x3caa14cc, SUC_lexicon_tags_17 },
    { 0x2f8a14cd, SUC_lexicon_tags_1 },
    { 0x916a14ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8a14d6, SUC_lexicon_tags_17 },
    { 0xf06a14d7, SUC_lexicon_tags_28 },
    { 0x1daa14d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x91aa14db, SUC_lexicon_tags_6 },
    { 0x220a14dc, SUC_lexicon_tags_23 },
    { 0x6a4a14dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20a14e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x642a14e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe6a14e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0a14f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ca14f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724a14fc, SUC_lexicon_tags_87 },
    { 0x97ca14fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48a1500, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ea1504, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe0a1506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4a150c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x404a150e, SUC_lexicon_tags_15 },
    { 0x7c2a150f, SUC_lexicon_tags_1 },
    { 0x94ea1510, SUC_lexicon_tags_37 },
    { 0xc3ca1510, SUC_lexicon_tags_2 },
    { 0x9e2a150f, SUC_lexicon_tags_12 },
    { 0x90aa1511, SUC_lexicon_tags_26 },
    { 0x790a1514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ea1517, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296a151b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x172a151d, SUC_lexicon_tags_7 },
    { 0x1caa151e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0a1526, SUC_lexicon_tags_10 },
    { 0x68a1526, SUC_lexicon_tags_29 },
    { 0xeaa1528, SUC_lexicon_tags_49 },
    { 0x7c0a1527, SUC_lexicon_tags_63 },
    { 0x6c4a152a, SUC_lexicon_tags_11 },
    { 0x3d8a152b, SUC_lexicon_tags_26 },
    { 0x1dca152a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe90a152e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a1535, SUC_lexicon_tags_17 },
    { 0xacca1535, SUC_lexicon_tags_2 },
    { 0xb90a1536, SUC_lexicon_tags_6 },
    { 0x154a1536, SUC_lexicon_tags_16 },
    { 0x43aa1536, SUC_lexicon_tags_7 },
    { 0x3ca153a, SUC_lexicon_tags_39 },
    { 0x20ca1537, SUC_lexicon_tags_18 },
    { 0x434a153c, SUC_lexicon_tags_11 },
    { 0x98ca1539, SUC_lexicon_tags_57 },
    { 0x918a153e, SUC_lexicon_tags_17 },
    { 0x83ca1538, SUC_lexicon_tags_11 },
    { 0xea0a153d, SUC_lexicon_tags_15 },
    { 0x66a153a, SUC_lexicon_tags_43 },
    { 0x2bea1542, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0a1549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6a154f, SUC_lexicon_tags_1 },
    { 0x2d4a1550, SUC_lexicon_tags_11 },
    { 0x698a1550, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4a1558, SUC_lexicon_tags_37 },
    { 0xae8a1559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x350a155b, SUC_lexicon_tags_61 },
    { 0x49ea155b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x892a155e, SUC_lexicon_tags_14 },
    { 0xbe8a155f, SUC_lexicon_tags_23 },
    { 0x94ea1560, SUC_lexicon_tags_2 },
    { 0x25ea155f, SUC_lexicon_tags_40 },
    { 0x1c2a1562, SUC_lexicon_tags_15 },
    { 0xb38a1563, SUC_lexicon_tags_33 },
    { 0x860a1561, SUC_lexicon_tags_7 },
    { 0xa6ca1560, SUC_lexicon_tags_15 },
    { 0xffca1566, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb8a1568, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x202a156b, SUC_lexicon_tags_17 },
    { 0xd3ca156b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae2a156e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8a1574, SUC_lexicon_tags_7 },
    { 0x7a4a1575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64a157f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x572a1582, SUC_lexicon_tags_25 },
    { 0x318a1583, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29ca1585, SUC_lexicon_tags_11 },
    { 0xc0a1586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630a158a, SUC_lexicon_tags_63 },
    { 0xffca158b, SUC_lexicon_tags_49 },
    { 0xc64a158c, SUC_lexicon_tags_17 },
    { 0x164a158c, SUC_lexicon_tags_1 },
    { 0xd44a158e, SUC_lexicon_tags_2 },
    { 0xe46a158b, SUC_lexicon_tags_19 },
    { 0x6a1590, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a4a1592, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aa1595, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9baa1597, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04a159a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3b4a159c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2a159f, SUC_lexicon_tags_0 },
    { 0xfc0a15a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6a15a4, SUC_lexicon_tags_12 },
    { 0xce0a15a5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46a15ad, SUC_lexicon_tags_31 },
    { 0x698a15ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbea15b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6a15ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ca15bd, SUC_lexicon_tags_39 },
    { 0xac8a15bd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372a15c4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21aa15d2, SUC_lexicon_tags_12 },
    { 0xe50a15d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58a15d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4a15d8, SUC_lexicon_tags_19 },
    { 0x646a15d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ca15dc, SUC_lexicon_tags_26 },
    { 0x190a15dd, SUC_lexicon_tags_62 },
    { 0x886a15de, SUC_lexicon_tags_2 },
    { 0x1f6a15de, SUC_lexicon_tags_49 },
    { 0xaa6a15e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe36a15e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x90aa15e5, SUC_lexicon_tags_8 },
    { 0x7f8a15e6, SUC_lexicon_tags_17 },
    { 0x9b2a15e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x744a15e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x93aa15ec, SUC_lexicon_tags_27 },
    { 0x888a15ec, SUC_lexicon_tags_33 },
    { 0x838a15ee, SUC_lexicon_tags_56 },
    { 0x63aa15ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58a15f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x464a15fa, SUC_lexicon_tags_60 },
    { 0xca15fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51ca15fd, SUC_lexicon_tags_82 },
    { 0x16aa15fe, SUC_lexicon_tags_2 },
    { 0xa32a15fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732a1605, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ea1608, SUC_lexicon_tags_17 },
    { 0x7d2a1609, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9fca160b, SUC_lexicon_tags_12 },
    { 0x444a160b, SUC_lexicon_tags_1 },
    { 0xcdea160d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6ea160f, SUC_lexicon_tags_50 },
    { 0x158a1610, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a1615, SUC_lexicon_tags_6 },
    { 0x8a4a1616, SUC_lexicon_tags_63 },
    { 0xce2a1616, SUC_lexicon_tags_68 },
    { 0xadca1618, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cca161e, SUC_lexicon_tags_37 },
    { 0x68ca161f, SUC_lexicon_tags_108 },
    { 0x868a1620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42a1622, SUC_lexicon_tags_26 },
    { 0x8eaa1623, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ca1628, SUC_lexicon_tags_25 },
    { 0x23ea1628, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5aca162b, SUC_lexicon_tags_8 },
    { 0x894a162c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222a1632, SUC_lexicon_tags_14 },
    { 0x57ea1632, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x716a1635, SUC_lexicon_tags_7 },
    { 0x1a4a1636, SUC_lexicon_tags_37 },
    { 0x22a1637, SUC_lexicon_tags_5 },
    { 0xb02a1635, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8caa163a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27ea163c, SUC_lexicon_tags_9 },
    { 0x5f6a163d, SUC_lexicon_tags_19 },
    { 0xb5aa163c, SUC_lexicon_tags_25 },
    { 0xe16a163e, SUC_lexicon_tags_11 },
    { 0xa3aa1640, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aa1645, SUC_lexicon_tags_2 },
    { 0x8ca1645, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12a164c, SUC_lexicon_tags_16 },
    { 0x42ea164d, SUC_lexicon_tags_37 },
    { 0xdf0a164c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4faa1650, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x514a1653, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252a1659, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6a165d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43ca165f, SUC_lexicon_tags_25 },
    { 0xc0a1660, SUC_lexicon_tags_49 },
    { 0x820a1661, SUC_lexicon_tags_37 },
    { 0x836a1662, SUC_lexicon_tags_30 },
    { 0x1a6a1661, SUC_lexicon_tags_12 },
    { 0x3b2a1662, SUC_lexicon_tags_15 },
    { 0x8dca1665, SUC_lexicon_tags_31 },
    { 0xa4ea1666, SUC_lexicon_tags_45 },
    { 0xfdca1666, SUC_lexicon_tags_9 },
    { 0xf46a1667, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xf72a166a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684a1671, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62a1674, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2a1679, SUC_lexicon_tags_12 },
    { 0xfcca1679, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ca1684, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708a1688, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872a168c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20aa1691, SUC_lexicon_tags_29 },
    { 0x5ea1691, SUC_lexicon_tags_0 },
    { 0x3f4a1693, SUC_lexicon_tags_0 },
    { 0x2dea1692, SUC_lexicon_tags_257 },
    { 0x1e0a1693, SUC_lexicon_tags_16 },
    { 0xa1ca1696, SUC_lexicon_tags_19 },
    { 0x2bea1695, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x4a1699, SUC_lexicon_tags_12 },
    { 0x9a8a1699, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdeea169c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14a169f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x248a16a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2ca16a3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x694a16a5, SUC_lexicon_tags_23 },
    { 0x9cca16a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54a16b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6fea16b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe40a16b6, SUC_lexicon_tags_6 },
    { 0x476a16b6, SUC_lexicon_tags_37 },
    { 0x70ca16b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ea16bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x952a16be, SUC_lexicon_tags_43 },
    { 0x92a16bf, SUC_lexicon_tags_12 },
    { 0x8f0a16c0, SUC_lexicon_tags_15 },
    { 0x6a2a16c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x256a16c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956a16c9, SUC_lexicon_tags_37 },
    { 0x83ca16ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd28a16cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x860a16cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f2a16d1, SUC_lexicon_tags_15 },
    { 0x6e6a16d2, SUC_lexicon_tags_26 },
    { 0xdd0a16d2, SUC_lexicon_tags_37 },
    { 0x9aa16d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafaa16d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ea16d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a16dd, SUC_lexicon_tags_11 },
    { 0xc8a16de, SUC_lexicon_tags_47 },
    { 0x578a16dd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952a16e4, SUC_lexicon_tags_16 },
    { 0x472a16e4, SUC_lexicon_tags_45 },
    { 0x4e6a16e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x438a16e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8a16eb, SUC_lexicon_tags_38 },
    { 0xaf8a16ec, SUC_lexicon_tags_17 },
    { 0x640a16eb, SUC_lexicon_tags_9 },
    { 0x30ca16ee, SUC_lexicon_tags_18 },
    { 0x256a16ef, SUC_lexicon_tags_17 },
    { 0x8b0a16ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cca16f2, SUC_lexicon_tags_11 },
    { 0xde6a16f3, SUC_lexicon_tags_12 },
    { 0xa72a16f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x722a16f6, SUC_lexicon_tags_37 },
    { 0x3baa16f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ea16fa, SUC_lexicon_tags_27 },
    { 0x602a16fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ca1703, SUC_lexicon_tags_19 },
    { 0xf72a1704, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44a1708, SUC_lexicon_tags_17 },
    { 0xa70a1709, SUC_lexicon_tags_1 },
    { 0xfca1709, SUC_lexicon_tags_0 },
    { 0x450a170b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x434a170d, SUC_lexicon_tags_60 },
    { 0x22ca170d, SUC_lexicon_tags_11 },
    { 0xa80a170d, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a1715, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cea1718, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca171c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bca1721, SUC_lexicon_tags_15 },
    { 0x4b6a1722, SUC_lexicon_tags_8 },
    { 0xbbaa1723, SUC_lexicon_tags_17 },
    { 0xa34a1724, SUC_lexicon_tags_12 },
    { 0x20a1724, SUC_lexicon_tags_15 },
    { 0xcf4a1725, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0a172f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b0a1731, SUC_lexicon_tags_27 },
    { 0x1bca1731, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8a1738, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590a173d, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140a1745, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a174c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4a1750, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x668a1752, SUC_lexicon_tags_47 },
    { 0x366a1753, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988a1758, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9aca175a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920a175e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02a1762, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ca176a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x58aa176c, SUC_lexicon_tags_16 },
    { 0x1fca176c, SUC_lexicon_tags_38 },
    { 0x9cea176c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ca1772, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2a1775, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x274a1777, SUC_lexicon_tags_66 },
    { 0xa08a1777, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aa177e, SUC_lexicon_tags_0 },
    { 0x196a177f, SUC_lexicon_tags_7 },
    { 0x180a177e, SUC_lexicon_tags_1 },
    { 0x276a1781, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x844a1784, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a1787, SUC_lexicon_tags_9 },
    { 0x15ca1788, SUC_lexicon_tags_6 },
    { 0x1e0a1789, SUC_lexicon_tags_17 },
    { 0xb84a178a, SUC_lexicon_tags_45 },
    { 0x644a178a, SUC_lexicon_tags_2 },
    { 0x922a178c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26a178f, SUC_lexicon_tags_12 },
    { 0x5a2a1790, SUC_lexicon_tags_2 },
    { 0x5a4a1791, SUC_lexicon_tags_47 },
    { 0xe40a1792, SUC_lexicon_tags_64 },
    { 0x128a1791, SUC_lexicon_tags_17 },
    { 0x22a1794, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a1798, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76a179e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac6a17a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x314a17a3, SUC_lexicon_tags_1 },
    { 0x920a17a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98aa17a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaea17ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952a17bb, SUC_lexicon_tags_14 },
    { 0x184a17bc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x92aa17bf, SUC_lexicon_tags_68 },
    { 0xbdca17bf, SUC_lexicon_tags_25 },
    { 0xc64a17c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x396a17c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6a17ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe2ea17cc, SUC_lexicon_tags_17 },
    { 0x536a17cc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ca17d1, SUC_lexicon_tags_48 },
    { 0x44a17d1, SUC_lexicon_tags_17 },
    { 0x4ca17d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4a17d7, SUC_lexicon_tags_43 },
    { 0x9e2a17d7, SUC_lexicon_tags_15 },
    { 0x7a8a17d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21aa17dc, SUC_lexicon_tags_7 },
    { 0xa6aa17dc, SUC_lexicon_tags_31 },
    { 0x798a17dd, SUC_lexicon_tags_2 },
    { 0x658a17df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322a17e3, SUC_lexicon_tags_11 },
    { 0x376a17e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08a17e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ca17ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4a17f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eca17fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560a17ff, SUC_lexicon_tags_60 },
    { 0x992a1800, SUC_lexicon_tags_33 },
    { 0xe9ca17ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb88a1803, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ea1808, SUC_lexicon_tags_39 },
    { 0x11ca1809, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46a180c, SUC_lexicon_tags_7 },
    { 0x758a180c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x910a180f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0a1812, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xddaa1815, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ea1819, SUC_lexicon_tags_17 },
    { 0xaa2a181a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aca181d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64a1820, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaa1823, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68a1826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8a182b, SUC_lexicon_tags_37 },
    { 0xfaca182b, SUC_lexicon_tags_7 },
    { 0x7ea182b, SUC_lexicon_tags_60 },
    { 0x27aa182e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8a1832, SUC_lexicon_tags_26 },
    { 0xc48a1833, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a1840, SUC_lexicon_tags_68 },
    { 0x480a1841, SUC_lexicon_tags_47 },
    { 0xd48a1840, SUC_lexicon_tags_1 },
    { 0x4c0a1843, SUC_lexicon_tags_18 },
    { 0x778a1843, SUC_lexicon_tags_0 },
    { 0xbeca1845, SUC_lexicon_tags_2 },
    { 0x9aca1842, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x782a1848, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc70a184a, SUC_lexicon_tags_55 },
    { 0xaa0a184b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeeaa184d, SUC_lexicon_tags_25 },
    { 0x9c8a184d, SUC_lexicon_tags_1 },
    { 0x354a184d, SUC_lexicon_tags_2 },
    { 0xa60a184e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77ea1852, SUC_lexicon_tags_14 },
    { 0x18aa1852, SUC_lexicon_tags_7 },
    { 0xa06a1854, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20a1857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06a185a, SUC_lexicon_tags_37 },
    { 0xefaa185b, SUC_lexicon_tags_15 },
    { 0xcaea185c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d4a185e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728a1863, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ca1866, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68a186a, SUC_lexicon_tags_1 },
    { 0xa7ca186b, SUC_lexicon_tags_4 },
    { 0xb84a186c, SUC_lexicon_tags_25 },
    { 0x896a186c, SUC_lexicon_tags_21 },
    { 0x824a186d, SUC_lexicon_tags_30 },
    { 0xba4a186f, SUC_lexicon_tags_46 },
    { 0x9a8a1870, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ea1878, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2a187b, SUC_lexicon_tags_141 },
    { 0xf2ca187c, SUC_lexicon_tags_17 },
    { 0xf6ea187d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xad6a187f, SUC_lexicon_tags_87 },
    { 0x896a1880, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e6a1882, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350a188c, SUC_lexicon_tags_30 },
    { 0x75aa188d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd8a188f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa20a1891, SUC_lexicon_tags_27 },
    { 0x99ca1891, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68a1895, SUC_lexicon_tags_49 },
    { 0xedea1895, SUC_lexicon_tags_0 },
    { 0x458a1895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260a189e, SUC_lexicon_tags_17 },
    { 0x522a189f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x872a18a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaa18a5, SUC_lexicon_tags_27 },
    { 0xb0aa18a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ca18a9, SUC_lexicon_tags_17 },
    { 0x750a18a9, SUC_lexicon_tags_11 },
    { 0xd72a18ab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbe4a18ad, SUC_lexicon_tags_29 },
    { 0xf7aa18ae, SUC_lexicon_tags_40 },
    { 0xad0a18ad, SUC_lexicon_tags_57 },
    { 0xae4a18b0, SUC_lexicon_tags_7 },
    { 0xb64a18b1, SUC_lexicon_tags_26 },
    { 0x2b6a18b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aa18b6, SUC_lexicon_tags_7 },
    { 0xfb0a18b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814a18bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x214a18c2, SUC_lexicon_tags_17 },
    { 0x96a18c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc96a18c5, SUC_lexicon_tags_55 },
    { 0x81ca18c6, SUC_lexicon_tags_11 },
    { 0xd2a18c5, SUC_lexicon_tags_12 },
    { 0x2c2a18c6, SUC_lexicon_tags_38 },
    { 0xc24a18c9, SUC_lexicon_tags_1 },
    { 0x392a18ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21ca18cc, SUC_lexicon_tags_11 },
    { 0xc4aa18cd, SUC_lexicon_tags_37 },
    { 0x816a18cc, SUC_lexicon_tags_68 },
    { 0x2e2a18cf, SUC_lexicon_tags_1 },
    { 0xd00a18cf, SUC_lexicon_tags_8 },
    { 0xffca18d1, SUC_lexicon_tags_28 },
    { 0xb5ea18d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64aa18d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0a18d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6a18dd, SUC_lexicon_tags_63 },
    { 0xa66a18de, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88a18e3, SUC_lexicon_tags_15 },
    { 0x766a18e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x306a18e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x480a18e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9caa18eb, SUC_lexicon_tags_49 },
    { 0x51ea18eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12a18f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ea18f9, SUC_lexicon_tags_11 },
    { 0xfb2a18fa, SUC_lexicon_tags_33 },
    { 0xe5aa18fb, SUC_lexicon_tags_15 },
    { 0x786a18fa, SUC_lexicon_tags_69 },
    { 0xc4aa18fc, SUC_lexicon_tags_14 },
    { 0x586a18fe, SUC_lexicon_tags_63 },
    { 0x132a18ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7ea1901, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab6a1903, SUC_lexicon_tags_55 },
    { 0xf4ea1904, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x86aa1906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aca190a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a8a190c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4caa1910, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bca1913, SUC_lexicon_tags_46 },
    { 0x30aa1914, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5aca1916, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6a191a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0a191f, SUC_lexicon_tags_82 },
    { 0x4dea191f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cea1923, SUC_lexicon_tags_2 },
    { 0xcdaa1924, SUC_lexicon_tags_15 },
    { 0xe1ea1924, SUC_lexicon_tags_2 },
    { 0xb04a1926, SUC_lexicon_tags_15 },
    { 0x686a1923, SUC_lexicon_tags_26 },
    { 0x522a1928, SUC_lexicon_tags_14 },
    { 0x5aca1929, SUC_lexicon_tags_19 },
    { 0xd9ca1929, SUC_lexicon_tags_6 },
    { 0xa4aa1928, SUC_lexicon_tags_14 },
    { 0xd5ca192c, SUC_lexicon_tags_28 },
    { 0x978a192d, SUC_lexicon_tags_134 },
    { 0xab2a1926, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4a1933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5aa1935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ea193b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8a193f, SUC_lexicon_tags_1 },
    { 0xff8a1940, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x858a1942, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x720a1944, SUC_lexicon_tags_1 },
    { 0x288a1945, SUC_lexicon_tags_25 },
    { 0x402a1946, SUC_lexicon_tags_16 },
    { 0x296a1944, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02a194c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdea1953, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x704a1955, SUC_lexicon_tags_37 },
    { 0xe6a1956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ca1959, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfa0a195b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ca195e, SUC_lexicon_tags_11 },
    { 0x6f0a195f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1b8a1961, SUC_lexicon_tags_38 },
    { 0xfeaa1962, SUC_lexicon_tags_15 },
    { 0x838a1963, SUC_lexicon_tags_11 },
    { 0xc56a1964, SUC_lexicon_tags_2 },
    { 0x66a1965, SUC_lexicon_tags_4 },
    { 0xb40a1966, SUC_lexicon_tags_14 },
    { 0x1caa1963, SUC_lexicon_tags_84 },
    { 0xfbca1963, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ea196d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa60a196f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6a1972, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0a1976, SUC_lexicon_tags_55 },
    { 0x5e2a1977, SUC_lexicon_tags_10 },
    { 0xf2aa1976, SUC_lexicon_tags_6 },
    { 0x762a1978, SUC_lexicon_tags_25 },
    { 0x51ea197a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6a1980, SUC_lexicon_tags_2 },
    { 0xb06a1980, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ca1988, SUC_lexicon_tags_22 },
    { 0x442a1989, SUC_lexicon_tags_37 },
    { 0x968a198a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a198e, SUC_lexicon_tags_17 },
    { 0xd34a198e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592a1993, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fea1996, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a1999, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca199c, SUC_lexicon_tags_11 },
    { 0xf8aa199c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x176a199f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc4a19a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6ea19a3, SUC_lexicon_tags_33 },
    { 0x514a19a3, SUC_lexicon_tags_37 },
    { 0x1e2a19a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe4ea19a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2a6a19a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aa19ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x96aa19b0, SUC_lexicon_tags_47 },
    { 0x746a19b1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454a19b6, SUC_lexicon_tags_11 },
    { 0x862a19b6, SUC_lexicon_tags_49 },
    { 0xedaa19b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xba4a19ba, SUC_lexicon_tags_37 },
    { 0x9d2a19bb, SUC_lexicon_tags_87 },
    { 0x3e4a19bc, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4a19bf, SUC_lexicon_tags_63 },
    { 0x3faa19c0, SUC_lexicon_tags_11 },
    { 0x514a19bf, SUC_lexicon_tags_1 },
    { 0x54aa19bf, SUC_lexicon_tags_38 },
    { 0xefea19c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78a19c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa72a19ca, SUC_lexicon_tags_9 },
    { 0x51ca19cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14ca19cd, SUC_lexicon_tags_17 },
    { 0x48ea19ce, SUC_lexicon_tags_68 },
    { 0x48ca19ce, SUC_lexicon_tags_17 },
    { 0x684a19ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ca19d4, SUC_lexicon_tags_77 },
    { 0x7aea19d5, SUC_lexicon_tags_15 },
    { 0x4f0a19d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16aa19de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66aa19e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab4a19e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66a19e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ea19ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd06a19ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eca19f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf44a19f4, SUC_lexicon_tags_17 },
    { 0xd20a19f5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x14ca19f7, SUC_lexicon_tags_36 },
    { 0x796a19f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5dea19fa, SUC_lexicon_tags_12 },
    { 0x6eaa19fa, SUC_lexicon_tags_4 },
    { 0x26a19fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaa1a03, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5d8a1a05, SUC_lexicon_tags_13 },
    { 0x300a1a06, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x698a1a08, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4b2a1a0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca1a0d, SUC_lexicon_tags_8 },
    { 0xb24a1a0e, SUC_lexicon_tags_17 },
    { 0x854a1a0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92a1a14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x198a1a16, SUC_lexicon_tags_31 },
    { 0x95aa1a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd10a1a19, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6a1a1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65aa1a23, SUC_lexicon_tags_14 },
    { 0xfd8a1a24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b4a1a26, SUC_lexicon_tags_15 },
    { 0xe7ca1a27, SUC_lexicon_tags_54 },
    { 0x256a1a27, SUC_lexicon_tags_17 },
    { 0x774a1a26, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90aa1a31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a1a38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76a1a3c, SUC_lexicon_tags_68 },
    { 0xdc0a1a3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08a1a49, SUC_lexicon_tags_17 },
    { 0xe74a1a4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1aa1a4c, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614a1a50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42a1a54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaa1a58, SUC_lexicon_tags_84 },
    { 0xa2ea1a59, SUC_lexicon_tags_25 },
    { 0xadca1a5a, SUC_lexicon_tags_26 },
    { 0xfbea1a5a, SUC_lexicon_tags_37 },
    { 0x8d6a1a58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3aa1a61, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ea1a64, SUC_lexicon_tags_17 },
    { 0x41aa1a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10ea1a67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4a1a6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34a1a72, SUC_lexicon_tags_14 },
    { 0x52aa1a73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212a1a77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114a1a7c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8a1a80, SUC_lexicon_tags_60 },
    { 0x85ca1a80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106a1a8b, SUC_lexicon_tags_11 },
    { 0x700a1a8b, SUC_lexicon_tags_77 },
    { 0x328a1a8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x64ca1a8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe84a1a91, SUC_lexicon_tags_25 },
    { 0x642a1a91, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4a1a97, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x5faa1a99, SUC_lexicon_tags_43 },
    { 0x68ca1a9a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x552a1a9c, SUC_lexicon_tags_16 },
    { 0x53ea1a9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362a1aa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaa1aa4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600a1aad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x156a1ab0, SUC_lexicon_tags_15 },
    { 0x8c4a1ab1, SUC_lexicon_tags_65 },
    { 0xb84a1ab2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x43aa1ab5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2a1ac0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ea1ac3, SUC_lexicon_tags_25 },
    { 0x32aa1ac4, SUC_lexicon_tags_38 },
    { 0x99aa1ac4, SUC_lexicon_tags_33 },
    { 0x582a1ac6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42a1ac9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x886a1acb, SUC_lexicon_tags_11 },
    { 0xff6a1acc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2a1ae0, SUC_lexicon_tags_17 },
    { 0x1c2a1ae1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aa1ae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e8a1aeb, SUC_lexicon_tags_43 },
    { 0xc40a1aec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dea1aef, SUC_lexicon_tags_21 },
    { 0xa4ca1af0, SUC_lexicon_tags_11 },
    { 0xd7ea1af0, SUC_lexicon_tags_17 },
    { 0x79ea1af1, SUC_lexicon_tags_4 },
    { 0xbaca1aef, SUC_lexicon_tags_25 },
    { 0x5daa1af4, SUC_lexicon_tags_45 },
    { 0x838a1af5, SUC_lexicon_tags_49 },
    { 0x8bea1af6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2aa1afb, SUC_lexicon_tags_33 },
    { 0x8dea1afc, SUC_lexicon_tags_37 },
    { 0x4aca1afc, SUC_lexicon_tags_55 },
    { 0x3daa1afe, SUC_lexicon_tags_4 },
    { 0x55ea1aff, SUC_lexicon_tags_25 },
    { 0x4faa1b00, SUC_lexicon_tags_48 },
    { 0x2a4a1b01, SUC_lexicon_tags_23 },
    { 0x836a1b02, SUC_lexicon_tags_2 },
    { 0x210a1b02, SUC_lexicon_tags_19 },
    { 0x114a1b04, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68a1b07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0a1b0e, SUC_lexicon_tags_30 },
    { 0x77ca1b0e, SUC_lexicon_tags_49 },
    { 0xcbca1b10, SUC_lexicon_tags_39 },
    { 0xb4a1b10, SUC_lexicon_tags_27 },
    { 0x748a1b10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04a1b1a, SUC_lexicon_tags_66 },
    { 0xedca1b1b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416a1b1f, SUC_lexicon_tags_17 },
    { 0x800a1b1f, SUC_lexicon_tags_38 },
    { 0x9e8a1b21, SUC_lexicon_tags_37 },
    { 0xae2a1b20, SUC_lexicon_tags_25 },
    { 0xa7ca1b20, SUC_lexicon_tags_10 },
    { 0xb20a1b21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ca1b2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ea1b2d, SUC_lexicon_tags_25 },
    { 0x3eca1b2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7ea1b30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684a1b35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa1b38, SUC_lexicon_tags_45 },
    { 0xa1b39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ea1b3e, SUC_lexicon_tags_7 },
    { 0xb06a1b3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e6a1b41, SUC_lexicon_tags_27 },
    { 0x964a1b41, SUC_lexicon_tags_15 },
    { 0x972a1b42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766a1b47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb34a1b49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a1b4d, SUC_lexicon_tags_45 },
    { 0xc8ca1b4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bea1b50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x83aa1b53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ca1b57, SUC_lexicon_tags_4 },
    { 0xfeca1b58, SUC_lexicon_tags_1 },
    { 0x87ea1b58, SUC_lexicon_tags_49 },
    { 0x84a1b5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cca1b5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2a1b61, SUC_lexicon_tags_17 },
    { 0xec6a1b61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218a1b66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a1b69, SUC_lexicon_tags_6 },
    { 0xa46a1b6a, SUC_lexicon_tags_25 },
    { 0xba2a1b6b, SUC_lexicon_tags_25 },
    { 0x62aa1b6c, SUC_lexicon_tags_1 },
    { 0x53ca1b69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46ea1b6f, SUC_lexicon_tags_14 },
    { 0x502a1b70, SUC_lexicon_tags_63 },
    { 0xec4a1b71, SUC_lexicon_tags_10 },
    { 0xe52a1b72, SUC_lexicon_tags_45 },
    { 0xab4a1b73, SUC_lexicon_tags_11 },
    { 0xd52a1b70, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580a1b79, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980a1b81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162a1b8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ea1b91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0a1b94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaa1b99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5ca1b9b, SUC_lexicon_tags_1 },
    { 0x280a1b9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2a1b9f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaa1ba3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a1ba9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37aa1bac, SUC_lexicon_tags_2 },
    { 0xb74a1bad, SUC_lexicon_tags_33 },
    { 0x624a1bae, SUC_lexicon_tags_1 },
    { 0x10ea1baf, SUC_lexicon_tags_2 },
    { 0x64ea1baf, SUC_lexicon_tags_28 },
    { 0xffaa1bb0, SUC_lexicon_tags_2 },
    { 0x336a1bb2, SUC_lexicon_tags_26 },
    { 0x404a1bb2, SUC_lexicon_tags_521 },
    { 0xf64a1bb4, SUC_lexicon_tags_106 },
    { 0x942a1bb3, SUC_lexicon_tags_17 },
    { 0x7c4a1bb2, SUC_lexicon_tags_11 },
    { 0x37aa1bb1, SUC_lexicon_tags_37 },
    { 0x45ca1bb8, SUC_lexicon_tags_91 },
    { 0x6e0a1bad, SUC_lexicon_tags_1 },
    { 0xfa0a1bb4, SUC_lexicon_tags_45 },
    { 0x6d8a1bbb, SUC_lexicon_tags_2 },
    { 0xa1bbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7ca1bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55aa1bc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6a1bc3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ea1bc6, SUC_lexicon_tags_37 },
    { 0xdaa1bc6, SUC_lexicon_tags_17 },
    { 0xca4a1bc8, SUC_lexicon_tags_25 },
    { 0x6a8a1bc8, SUC_lexicon_tags_11 },
    { 0x960a1bca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4a1bcd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79aa1bcf, SUC_lexicon_tags_26 },
    { 0xd48a1bd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f6a1bd2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46a1bd7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26a1bdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaeaa1bde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6a1be5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x778a1be7, SUC_lexicon_tags_37 },
    { 0x812a1be8, SUC_lexicon_tags_11 },
    { 0x7eea1be9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95ca1beb, SUC_lexicon_tags_27 },
    { 0x74ca1bec, SUC_lexicon_tags_84 },
    { 0xce4a1bed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7daa1bef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa94a1bf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb8a1bf3, SUC_lexicon_tags_6 },
    { 0xff6a1bf3, SUC_lexicon_tags_23 },
    { 0x1d8a1bf3, SUC_lexicon_tags_2 },
    { 0x816a1bf5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60aa1bfd, SUC_lexicon_tags_33 },
    { 0xd84a1bfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952a1c02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8a1c07, SUC_lexicon_tags_17 },
    { 0x156a1c07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xddea1c0b, SUC_lexicon_tags_25 },
    { 0xdaca1c0b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5a4a1c0e, SUC_lexicon_tags_39 },
    { 0x7caa1c0e, SUC_lexicon_tags_191 },
    { 0x180a1c10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x650a1c13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ea1c20, SUC_lexicon_tags_13 },
    { 0x636a1c20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914a1c2b, SUC_lexicon_tags_25 },
    { 0x35ea1c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc98a1c2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd60a1c30, SUC_lexicon_tags_194 },
    { 0x960a1c31, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8a1c3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54a1c40, SUC_lexicon_tags_31 },
    { 0xbca1c41, SUC_lexicon_tags_8 },
    { 0x268a1c42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382a1c4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a1c4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca1c52, SUC_lexicon_tags_11 },
    { 0xd54a1c53, SUC_lexicon_tags_23 },
    { 0x7eca1c53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a1c59, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x2bca1c5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0a1c5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aa1c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78a1c66, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x218a1c68, SUC_lexicon_tags_84 },
    { 0xef6a1c69, SUC_lexicon_tags_45 },
    { 0xe50a1c68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0ea1c6c, SUC_lexicon_tags_11 },
    { 0x3f6a1c6d, SUC_lexicon_tags_33 },
    { 0x2b0a1c6e, SUC_lexicon_tags_8 },
    { 0x2dca1c6c, SUC_lexicon_tags_28 },
    { 0x3d6a1c70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0a1c73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde6a1c75, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2a1c7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7eca1c7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304a1c83, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ea1c86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478a1c8b, SUC_lexicon_tags_23 },
    { 0x162a1c8b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90a1c90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2a1c93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e8a1c95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52ca1c97, SUC_lexicon_tags_8 },
    { 0xa70a1c98, SUC_lexicon_tags_12 },
    { 0xcb6a1c99, SUC_lexicon_tags_30 },
    { 0xa58a1c98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaa1c9d, SUC_lexicon_tags_2 },
    { 0x57ea1c9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6aa1ca1, SUC_lexicon_tags_0 },
    { 0xaf4a1ca2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720a1ca7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea1cab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70a1cb5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2a1cba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38a1cbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16a1cc2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x536a1cc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaa1cce, SUC_lexicon_tags_2 },
    { 0xfdca1ccf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8a1cd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3aa1cd4, SUC_lexicon_tags_6 },
    { 0x616a1cd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b2a1cd7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308a1cdb, SUC_lexicon_tags_15 },
    { 0x90ca1cdc, SUC_lexicon_tags_12 },
    { 0xdbea1cdb, SUC_lexicon_tags_37 },
    { 0xae4a1cde, SUC_lexicon_tags_63 },
    { 0x3e6a1cdc, SUC_lexicon_tags_0 },
    { 0x854a1ce0, SUC_lexicon_tags_1462 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a1ce3, SUC_lexicon_tags_17 },
    { 0x7bca1ce3, SUC_lexicon_tags_14 },
    { 0x2e4a1ce5, SUC_lexicon_tags_34 },
    { 0x46ea1ce5, SUC_lexicon_tags_27 },
    { 0xd22a1ce6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaa1cea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caa1cf2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x600a1cf4, SUC_lexicon_tags_15 },
    { 0xe10a1cf5, SUC_lexicon_tags_39 },
    { 0xba2a1cf6, SUC_lexicon_tags_49 },
    { 0xa26a1cf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ca1cfa, SUC_lexicon_tags_17 },
    { 0xf0a1cfb, SUC_lexicon_tags_17 },
    { 0x2b6a1cfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a2a1cfe, SUC_lexicon_tags_65 },
    { 0xe78a1cfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x224a1d01, SUC_lexicon_tags_2 },
    { 0xd6aa1d02, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x158a1d05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc2a1d07, SUC_lexicon_tags_27 },
    { 0xa24a1d08, SUC_lexicon_tags_11 },
    { 0x84aa1d09, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bea1d0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82a1d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ea1d16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98a1d19, SUC_lexicon_tags_17 },
    { 0x6c8a1d1a, SUC_lexicon_tags_29 },
    { 0xb1ca1d1b, SUC_lexicon_tags_8 },
    { 0xda0a1d1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274a1d20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd6a1d22, SUC_lexicon_tags_32 },
    { 0x228a1d23, SUC_lexicon_tags_4 },
    { 0x66aa1d22, SUC_lexicon_tags_17 },
    { 0x2baa1d22, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88a1d29, SUC_lexicon_tags_44 },
    { 0x4dca1d29, SUC_lexicon_tags_17 },
    { 0x19ea1d2b, SUC_lexicon_tags_7 },
    { 0x182a1d2c, SUC_lexicon_tags_17 },
    { 0x42ca1d29, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb5ea1d2f, SUC_lexicon_tags_2 },
    { 0x838a1d30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2a1d33, SUC_lexicon_tags_95 },
    { 0x7cea1d33, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a1d39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ca1d3e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6a1d41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bca1d47, SUC_lexicon_tags_0 },
    { 0x8d8a1d48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d2a1d4a, SUC_lexicon_tags_12 },
    { 0xbfea1d4b, SUC_lexicon_tags_17 },
    { 0x8c6a1d4b, SUC_lexicon_tags_27 },
    { 0xe96a1d4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a6a1d4f, SUC_lexicon_tags_1 },
    { 0x13ea1d4f, SUC_lexicon_tags_37 },
    { 0x5a4a1d51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86a1d54, SUC_lexicon_tags_30 },
    { 0x702a1d55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fea1d5a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedca1d5f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf6a1d61, SUC_lexicon_tags_15 },
    { 0xc3ea1d62, SUC_lexicon_tags_82 },
    { 0x7cea1d62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x54a1d65, SUC_lexicon_tags_92 },
    { 0xe54a1d66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66a1d6b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58a1d6e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x6aca1d70, SUC_lexicon_tags_11 },
    { 0x48aa1d71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8a1d75, SUC_lexicon_tags_0 },
    { 0x10ea1d75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x958a1d79, SUC_lexicon_tags_15 },
    { 0x8f4a1d7a, SUC_lexicon_tags_28 },
    { 0xfca1d7a, SUC_lexicon_tags_25 },
    { 0x128a1d7c, SUC_lexicon_tags_1 },
    { 0x89ea1d7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc14a1d7f, SUC_lexicon_tags_10 },
    { 0x5f8a1d80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69ea1d82, SUC_lexicon_tags_17 },
    { 0x162a1d83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x510a1d85, SUC_lexicon_tags_56 },
    { 0x5b0a1d86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4a1d90, SUC_lexicon_tags_25 },
    { 0x9eaa1d91, SUC_lexicon_tags_11 },
    { 0x782a1d90, SUC_lexicon_tags_46 },
    { 0xe8a1d93, SUC_lexicon_tags_27 },
    { 0xe5ca1d91, SUC_lexicon_tags_16 },
    { 0xcb0a1d92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x51aa1d98, SUC_lexicon_tags_45 },
    { 0xaf4a1d99, SUC_lexicon_tags_7 },
    { 0xb54a1d99, SUC_lexicon_tags_37 },
    { 0x106a1d9b, SUC_lexicon_tags_23 },
    { 0x290a1d9c, SUC_lexicon_tags_11 },
    { 0xd92a1d9d, SUC_lexicon_tags_45 },
    { 0xd54a1d9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4a1da1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84a1da6, SUC_lexicon_tags_25 },
    { 0x464a1da7, SUC_lexicon_tags_66 },
    { 0x9f0a1da8, SUC_lexicon_tags_253 },
    { 0x702a1da9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322a1db1, SUC_lexicon_tags_11 },
    { 0xf6aa1db2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ea1dba, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x516a1dbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aa1dc1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbd8a1dc3, SUC_lexicon_tags_7 },
    { 0x486a1dc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfc4a1dc6, SUC_lexicon_tags_29 },
    { 0x3cca1dc7, SUC_lexicon_tags_17 },
    { 0xcfaa1dc8, SUC_lexicon_tags_45 },
    { 0x5b6a1dc9, SUC_lexicon_tags_144 },
    { 0xfdaa1dca, SUC_lexicon_tags_43 },
    { 0x4e0a1dc8, SUC_lexicon_tags_7 },
    { 0xb5ea1dcc, SUC_lexicon_tags_14 },
    { 0x318a1dc6, SUC_lexicon_tags_29 },
    { 0x10aa1dc7, SUC_lexicon_tags_4 },
    { 0xef0a1dcf, SUC_lexicon_tags_82 },
    { 0xf74a1dcf, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4faa1dd5, SUC_lexicon_tags_11 },
    { 0x260a1dd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdaca1dd8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00a1de1, SUC_lexicon_tags_15 },
    { 0xd02a1de1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc22a1de4, SUC_lexicon_tags_77 },
    { 0x234a1de4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ea1df2, SUC_lexicon_tags_28 },
    { 0x414a1df3, SUC_lexicon_tags_37 },
    { 0x2f6a1df4, SUC_lexicon_tags_49 },
    { 0x4daa1df3, SUC_lexicon_tags_13 },
    { 0x290a1df6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2a1dfb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2a1dff, SUC_lexicon_tags_12 },
    { 0x2c8a1e00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dca1e04, SUC_lexicon_tags_23 },
    { 0x7e8a1e04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eea1e09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x398a1e0c, SUC_lexicon_tags_3 },
    { 0x314a1e0c, SUC_lexicon_tags_25 },
    { 0xb0ea1e0e, SUC_lexicon_tags_27 },
    { 0xa3ea1e0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ea1e16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92a1e21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6a1e25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x380a1e28, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2a1e2b, SUC_lexicon_tags_16 },
    { 0x72aa1e2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6a1e30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96a1e32, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a1e36, SUC_lexicon_tags_48 },
    { 0x49ca1e36, SUC_lexicon_tags_43 },
    { 0xfd6a1e38, SUC_lexicon_tags_12 },
    { 0xdfca1e36, SUC_lexicon_tags_95 },
    { 0x41ea1e3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8a1e3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa1e42, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908a1e4a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x89aa1e4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aa1e52, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68a1e59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x34aa1e5b, SUC_lexicon_tags_17 },
    { 0x7bca1e5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128a1e63, SUC_lexicon_tags_19 },
    { 0xa2ea1e64, SUC_lexicon_tags_17 },
    { 0x6aea1e64, SUC_lexicon_tags_37 },
    { 0xe20a1e66, SUC_lexicon_tags_1 },
    { 0x972a1e65, SUC_lexicon_tags_1 },
    { 0xedaa1e68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x554a1e6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x214a1e6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ca1e71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaa1e78, SUC_lexicon_tags_60 },
    { 0x906a1e79, SUC_lexicon_tags_65 },
    { 0x46ea1e78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34a1e7f, SUC_lexicon_tags_37 },
    { 0x510a1e7f, SUC_lexicon_tags_2 },
    { 0x5c8a1e81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64a1e84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64aa1e8a, SUC_lexicon_tags_17 },
    { 0x58ca1e8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3baa1e8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x940a1e90, SUC_lexicon_tags_26 },
    { 0xe2ea1e91, SUC_lexicon_tags_11 },
    { 0x43aa1e92, SUC_lexicon_tags_22 },
    { 0x4f6a1e93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9eaa1e95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x126a1e98, SUC_lexicon_tags_38 },
    { 0xeb6a1e99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceea1e9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b2a1e9f, SUC_lexicon_tags_24 },
    { 0x10aa1e9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60aa1ea2, SUC_lexicon_tags_2 },
    { 0xeb0a1ea3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dca1ea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eea1eab, SUC_lexicon_tags_204 },
    { 0xb0ca1eac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510a1eb0, SUC_lexicon_tags_25 },
    { 0xf02a1eb1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfe8a1eb3, SUC_lexicon_tags_33 },
    { 0xb7ea1eb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdea1ebb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236a1ec0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaca1ec5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40a1ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ea1eca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702a1ecf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16aa1ed3, SUC_lexicon_tags_27 },
    { 0x66a1ed4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36a1ede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea1ee1, SUC_lexicon_tags_6 },
    { 0x842a1ee2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a1ee6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8a1ee8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9c0a1eea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74a1ef2, SUC_lexicon_tags_7 },
    { 0xb9aa1ef2, SUC_lexicon_tags_26 },
    { 0xc30a1ef4, SUC_lexicon_tags_77 },
    { 0xccaa1ef3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0a1ef9, SUC_lexicon_tags_12 },
    { 0x556a1efa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dca1efd, SUC_lexicon_tags_55 },
    { 0x6baa1efe, SUC_lexicon_tags_45 },
    { 0x16a1eff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12aa1f06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ea1f09, SUC_lexicon_tags_55 },
    { 0x4b4a1f09, SUC_lexicon_tags_15 },
    { 0xe2ca1f0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872a1f11, SUC_lexicon_tags_2 },
    { 0x6f8a1f12, SUC_lexicon_tags_37 },
    { 0x1d6a1f12, SUC_lexicon_tags_11 },
    { 0x4b4a1f14, SUC_lexicon_tags_43 },
    { 0xe94a1f11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72a1f18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58ea1f1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4a1f1d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ea1f20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa70a1f22, SUC_lexicon_tags_66 },
    { 0x8aaa1f23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1f25, SUC_lexicon_tags_15 },
    { 0xa9ea1f26, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2a1f29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d0a1f2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x434a1f2d, SUC_lexicon_tags_69 },
    { 0xa62a1f2d, SUC_lexicon_tags_39 },
    { 0x356a1f2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfcca1f31, SUC_lexicon_tags_11 },
    { 0x3b4a1f32, SUC_lexicon_tags_21 },
    { 0x33ca1f32, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedea1f3b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668a1f45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2a1f51, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeaa1f53, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204a1f57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8baa1f59, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aa1f5d, SUC_lexicon_tags_14 },
    { 0x2baa1f5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc2aa1f60, SUC_lexicon_tags_12 },
    { 0x638a1f61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ea1f6a, SUC_lexicon_tags_66 },
    { 0xb00a1f6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc2a1f6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x88aa1f6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aca1f76, SUC_lexicon_tags_195 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6a1f7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dca1f7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xffca1f81, SUC_lexicon_tags_7 },
    { 0xe76a1f82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb9ca1f84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc12a1f86, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaa1f8a, SUC_lexicon_tags_140 },
    { 0x6aaa1f8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4a1f90, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cca1f94, SUC_lexicon_tags_49 },
    { 0xc0ea1f95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a1f99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56a1f9c, SUC_lexicon_tags_1 },
    { 0x8a1f9d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ca1fa2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x212a1fa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0a1fa8, SUC_lexicon_tags_12 },
    { 0x6cea1fa9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa72a1fab, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbe2a1fad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4daa1fb0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a1fb3, SUC_lexicon_tags_7 },
    { 0x7daa1fb3, SUC_lexicon_tags_37 },
    { 0x4fea1fb5, SUC_lexicon_tags_56 },
    { 0x714a1fb4, SUC_lexicon_tags_24 },
    { 0x3b8a1fb5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbaa1fb9, SUC_lexicon_tags_33 },
    { 0xfbaa1fba, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xe6aa1fbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cca1fbe, SUC_lexicon_tags_9 },
    { 0x636a1fbf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0a1fc3, SUC_lexicon_tags_37 },
    { 0x7d2a1fc3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8a1fc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2a1fc9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd6a1fcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96a1fce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb2a1fd0, SUC_lexicon_tags_37 },
    { 0xd46a1fd1, SUC_lexicon_tags_25 },
    { 0xfaea1fd2, SUC_lexicon_tags_15 },
    { 0x9caa1fd3, SUC_lexicon_tags_1 },
    { 0x940a1fd0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fca1fd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50ca1fd9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a1fdd, SUC_lexicon_tags_37 },
    { 0xd46a1fdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eca1fe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde0a1fe3, SUC_lexicon_tags_2 },
    { 0xca0a1fe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0a1fe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bea1fea, SUC_lexicon_tags_39 },
    { 0xe12a1feb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132a1ff0, SUC_lexicon_tags_19 },
    { 0x118a1ff0, SUC_lexicon_tags_30 },
    { 0x526a1ff0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378a1ff9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdea1ffc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8a1fff, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4faa2009, SUC_lexicon_tags_4 },
    { 0xef0a200a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa200d, SUC_lexicon_tags_57 },
    { 0x490a200d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdca2012, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52a2015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x270a2017, SUC_lexicon_tags_25 },
    { 0x17ca2018, SUC_lexicon_tags_8 },
    { 0x5e8a2019, SUC_lexicon_tags_26 },
    { 0x464a2019, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bea2022, SUC_lexicon_tags_15 },
    { 0xa2023, SUC_lexicon_tags_17 },
    { 0x90a2024, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ea2030, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420a2036, SUC_lexicon_tags_37 },
    { 0x304a2037, SUC_lexicon_tags_37 },
    { 0x1e2a2038, SUC_lexicon_tags_32 },
    { 0x32ea2036, SUC_lexicon_tags_28 },
    { 0xb02a2038, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe8ca203c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0a2043, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4a8a2045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15aa2047, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eca2053, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22a2057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x422a2059, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x706a205b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a205f, SUC_lexicon_tags_33 },
    { 0x8aca2060, SUC_lexicon_tags_12 },
    { 0xc16a2060, SUC_lexicon_tags_11 },
    { 0xeaaa2061, SUC_lexicon_tags_8 },
    { 0x924a2063, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30a2068, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24ca206a, SUC_lexicon_tags_17 },
    { 0x7e4a206a, SUC_lexicon_tags_11 },
    { 0x802a206b, SUC_lexicon_tags_17 },
    { 0xcc6a206d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x1a8a206f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a2072, SUC_lexicon_tags_7 },
    { 0x390a2073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf8a2075, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2a207b, SUC_lexicon_tags_29 },
    { 0xbdca207c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c2a207e, SUC_lexicon_tags_43 },
    { 0x688a207f, SUC_lexicon_tags_31 },
    { 0x482a2080, SUC_lexicon_tags_45 },
    { 0xc34a2081, SUC_lexicon_tags_14 },
    { 0x5d4a207f, SUC_lexicon_tags_15 },
    { 0x276a207e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdeca2085, SUC_lexicon_tags_37 },
    { 0x7aca2086, SUC_lexicon_tags_17 },
    { 0x2aea2087, SUC_lexicon_tags_15 },
    { 0xbcca2088, SUC_lexicon_tags_25 },
    { 0x668a2089, SUC_lexicon_tags_12 },
    { 0x432a2089, SUC_lexicon_tags_12 },
    { 0xa88a208a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6a2090, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aa2095, SUC_lexicon_tags_7 },
    { 0xe30a2096, SUC_lexicon_tags_9 },
    { 0xf9ca2095, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x502a209a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b0a209c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b8a209e, SUC_lexicon_tags_37 },
    { 0x4cca209e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ca20a4, SUC_lexicon_tags_37 },
    { 0x980a20a5, SUC_lexicon_tags_79 },
    { 0x4caa20a6, SUC_lexicon_tags_27 },
    { 0x29ca20a5, SUC_lexicon_tags_17 },
    { 0x62a20a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aa20ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x536a20ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724a20b5, SUC_lexicon_tags_28 },
    { 0x4e4a20b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4a20b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7caa20bb, SUC_lexicon_tags_31 },
    { 0x9e4a20bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ea20bf, SUC_lexicon_tags_2 },
    { 0x244a20c0, SUC_lexicon_tags_45 },
    { 0x5d2a20c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x576a20c4, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8aa20c7, SUC_lexicon_tags_33 },
    { 0x4b0a20c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeca20cb, SUC_lexicon_tags_37 },
    { 0xaea20cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x430a20ce, SUC_lexicon_tags_29 },
    { 0xbb4a20cf, SUC_lexicon_tags_32 },
    { 0xbb6a20cf, SUC_lexicon_tags_23 },
    { 0xe52a20d1, SUC_lexicon_tags_47 },
    { 0x3a4a20d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaca20d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x390a20d8, SUC_lexicon_tags_60 },
    { 0xbf6a20d8, SUC_lexicon_tags_61 },
    { 0x228a20d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1ea20dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2caa20de, SUC_lexicon_tags_38 },
    { 0xcb2a20df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x110a20e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a20e5, SUC_lexicon_tags_33 },
    { 0xd2ea20e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63aa20e8, SUC_lexicon_tags_14 },
    { 0x920a20e9, SUC_lexicon_tags_46 },
    { 0xd70a20ea, SUC_lexicon_tags_23 },
    { 0x740a20ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe2ea20ed, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0xe5aa20ef, SUC_lexicon_tags_27 },
    { 0x20aa20f0, SUC_lexicon_tags_41 },
    { 0xb14a20f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc4a20f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452a20f7, SUC_lexicon_tags_43 },
    { 0x7daa20f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xba4a20fa, SUC_lexicon_tags_32 },
    { 0xc52a20fb, SUC_lexicon_tags_11 },
    { 0xc8ca20fa, SUC_lexicon_tags_43 },
    { 0xcaea20fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee6a20ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70a2104, SUC_lexicon_tags_1 },
    { 0xcf8a2104, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xddea2108, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa3aa210a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x7d0a210c, SUC_lexicon_tags_25 },
    { 0xe16a210c, SUC_lexicon_tags_11 },
    { 0x96ea210e, SUC_lexicon_tags_10 },
    { 0x75ca210d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x240a2111, SUC_lexicon_tags_29 },
    { 0xaaca2112, SUC_lexicon_tags_17 },
    { 0x382a2113, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0aa2119, SUC_lexicon_tags_17 },
    { 0x48ea211a, SUC_lexicon_tags_45 },
    { 0x42aa2119, SUC_lexicon_tags_2 },
    { 0x892a211b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ca211f, SUC_lexicon_tags_56 },
    { 0xb6ca2120, SUC_lexicon_tags_15 },
    { 0x322a2121, SUC_lexicon_tags_11 },
    { 0x510a2121, SUC_lexicon_tags_23 },
    { 0x562a2123, SUC_lexicon_tags_59 },
    { 0x308a2121, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852a2134, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2a2137, SUC_lexicon_tags_42 },
    { 0xf4aa2138, SUC_lexicon_tags_45 },
    { 0x5c4a2137, SUC_lexicon_tags_31 },
    { 0x1e4a213a, SUC_lexicon_tags_23 },
    { 0x902a213a, SUC_lexicon_tags_28 },
    { 0xadea213b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bea213e, SUC_lexicon_tags_1 },
    { 0x4d6a213e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x300a2141, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x702a2143, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6a2149, SUC_lexicon_tags_9 },
    { 0xe36a214a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb6aa214c, SUC_lexicon_tags_17 },
    { 0x572a214c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6a2150, SUC_lexicon_tags_37 },
    { 0xe1aa2150, SUC_lexicon_tags_46 },
    { 0xf28a2152, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x280a2155, SUC_lexicon_tags_57 },
    { 0x464a2156, SUC_lexicon_tags_137 },
    { 0xa9ea2156, SUC_lexicon_tags_25 },
    { 0xc98a2158, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b2a215a, SUC_lexicon_tags_47 },
    { 0x686a215a, SUC_lexicon_tags_47 },
    { 0xbf0a215b, SUC_lexicon_tags_34 },
    { 0xfb2a215d, SUC_lexicon_tags_11 },
    { 0x534a215e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8a2164, SUC_lexicon_tags_7 },
    { 0x9d6a2165, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dea216c, SUC_lexicon_tags_32 },
    { 0x728a216c, SUC_lexicon_tags_38 },
    { 0xe86a216e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x982a2171, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5ea2173, SUC_lexicon_tags_25 },
    { 0xad0a2174, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6a2176, SUC_lexicon_tags_8 },
    { 0x222a2177, SUC_lexicon_tags_23 },
    { 0xcd8a2178, SUC_lexicon_tags_15 },
    { 0xe5ca2179, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ca217c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ea2182, SUC_lexicon_tags_12 },
    { 0xf2ea2183, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4baa2185, SUC_lexicon_tags_26 },
    { 0x620a2186, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ea218a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x264a218d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4a2192, SUC_lexicon_tags_11 },
    { 0x8fea2193, SUC_lexicon_tags_19 },
    { 0xd00a2193, SUC_lexicon_tags_38 },
    { 0x87aa2195, SUC_lexicon_tags_30 },
    { 0x9bca2194, SUC_lexicon_tags_25 },
    { 0x67ca2195, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ca219a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6a219e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d6a21a0, SUC_lexicon_tags_4 },
    { 0x3f6a21a1, SUC_lexicon_tags_37 },
    { 0x996a21a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9aea21a4, SUC_lexicon_tags_40 },
    { 0xe0ca21a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88aa21aa, SUC_lexicon_tags_25 },
    { 0xcbca21ab, SUC_lexicon_tags_23 },
    { 0xa92a21aa, SUC_lexicon_tags_12 },
    { 0x8cca21ad, SUC_lexicon_tags_12 },
    { 0xfc4a21ad, SUC_lexicon_tags_22 },
    { 0xe20a21ae, SUC_lexicon_tags_7 },
    { 0xce2a21ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x98ea21b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x972a21b5, SUC_lexicon_tags_2 },
    { 0x4d0a21b6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x290a21b8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc40a21ba, SUC_lexicon_tags_17 },
    { 0xf9aa21ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd9ca21bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744a21c1, SUC_lexicon_tags_19 },
    { 0x42ca21c2, SUC_lexicon_tags_14 },
    { 0xfa6a21c2, SUC_lexicon_tags_16 },
    { 0x30ea21c2, SUC_lexicon_tags_47 },
    { 0x460a21c1, SUC_lexicon_tags_37 },
    { 0xa5ca21c5, SUC_lexicon_tags_14 },
    { 0xcc2a21c3, SUC_lexicon_tags_17 },
    { 0xfd2a21c7, SUC_lexicon_tags_5 },
    { 0xf52a21c2, SUC_lexicon_tags_25 },
    { 0x10ea21c1, SUC_lexicon_tags_43 },
    { 0x8e4a21cb, SUC_lexicon_tags_5 },
    { 0xf6aa21c7, SUC_lexicon_tags_6 },
    { 0xef4a21cd, SUC_lexicon_tags_14 },
    { 0x2aa21ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c4a21d0, SUC_lexicon_tags_23 },
    { 0xab4a21d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x222a21d3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bca21d6, SUC_lexicon_tags_14 },
    { 0x962a21d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86a21da, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ca21dd, SUC_lexicon_tags_25 },
    { 0x8f6a21dd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660a21e2, SUC_lexicon_tags_49 },
    { 0x858a21e3, SUC_lexicon_tags_15 },
    { 0x7e4a21e3, SUC_lexicon_tags_37 },
    { 0xeeea21e5, SUC_lexicon_tags_63 },
    { 0xad6a21e4, SUC_lexicon_tags_48 },
    { 0x44ca21e3, SUC_lexicon_tags_37 },
    { 0x706a21e6, SUC_lexicon_tags_45 },
    { 0xd64a21e6, SUC_lexicon_tags_87 },
    { 0x7c6a21e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x62ea21ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e0a21ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe10a21f0, SUC_lexicon_tags_31 },
    { 0xf8a21f1, SUC_lexicon_tags_1 },
    { 0x2f6a21f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50a21f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326a2203, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4a2207, SUC_lexicon_tags_415 },
    { 0, NULL },
    { 0xff2a2209, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6a220c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca2215, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788a221c, SUC_lexicon_tags_60 },
    { 0x150a221d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x37aa221f, SUC_lexicon_tags_49 },
    { 0x492a2220, SUC_lexicon_tags_16 },
    { 0xaca2221, SUC_lexicon_tags_7 },
    { 0xb1aa2221, SUC_lexicon_tags_27 },
    { 0x312a2220, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4a222a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10a222e, SUC_lexicon_tags_45 },
    { 0x9dea222e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60a2231, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x602a2233, SUC_lexicon_tags_11 },
    { 0xdd8a2234, SUC_lexicon_tags_0 },
    { 0x894a2235, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34a223b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb38a223d, SUC_lexicon_tags_11 },
    { 0x456a223e, SUC_lexicon_tags_30 },
    { 0xa2ca223f, SUC_lexicon_tags_17 },
    { 0xb74a2240, SUC_lexicon_tags_27 },
    { 0xc5ea2240, SUC_lexicon_tags_17 },
    { 0xc10a2242, SUC_lexicon_tags_10 },
    { 0x54a2241, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x220a2245, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x294a2247, SUC_lexicon_tags_63 },
    { 0xb5ca2248, SUC_lexicon_tags_13 },
    { 0xf6aa2249, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaea224e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdca2254, SUC_lexicon_tags_47 },
    { 0x9f4a2255, SUC_lexicon_tags_27 },
    { 0x6baa2256, SUC_lexicon_tags_17 },
    { 0x318a2257, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x740a2259, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4a2260, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cea2265, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3aa226a, SUC_lexicon_tags_37 },
    { 0xa94a226b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948a226f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6a2272, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb44a2274, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4a227a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82a227e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94a228f, SUC_lexicon_tags_21 },
    { 0xaca2290, SUC_lexicon_tags_17 },
    { 0x470a228f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54aa2295, SUC_lexicon_tags_28 },
    { 0x1e2a2295, SUC_lexicon_tags_1 },
    { 0x912a2295, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a6a2299, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa48a229b, SUC_lexicon_tags_45 },
    { 0xbfea229c, SUC_lexicon_tags_11 },
    { 0x23aa229c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32a22a0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57aa22a5, SUC_lexicon_tags_17 },
    { 0x3b2a22a6, SUC_lexicon_tags_25 },
    { 0x4c2a22a7, SUC_lexicon_tags_11 },
    { 0xf68a22a8, SUC_lexicon_tags_1 },
    { 0x898a22a9, SUC_lexicon_tags_57 },
    { 0xe70a22a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6a6a22ac, SUC_lexicon_tags_15 },
    { 0x6b0a22ad, SUC_lexicon_tags_55 },
    { 0x9a4a22ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694a22b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a8a22b4, SUC_lexicon_tags_27 },
    { 0xa96a22b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80a22b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486a22be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ca22c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2a22cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822a22d2, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x24ca22d4, SUC_lexicon_tags_26 },
    { 0xa36a22d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x778a22d8, SUC_lexicon_tags_12 },
    { 0x5d6a22d9, SUC_lexicon_tags_45 },
    { 0x8c8a22da, SUC_lexicon_tags_7 },
    { 0x95ea22d8, SUC_lexicon_tags_14 },
    { 0xc96a22dc, SUC_lexicon_tags_71 },
    { 0x8a22dd, SUC_lexicon_tags_37 },
    { 0x6d2a22dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x670a22e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4a22e5, SUC_lexicon_tags_11 },
    { 0xf4ca22e6, SUC_lexicon_tags_25 },
    { 0x57ca22e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x194a22e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x648a22ec, SUC_lexicon_tags_11 },
    { 0xf04a22ec, SUC_lexicon_tags_12 },
    { 0xaa0a22ed, SUC_lexicon_tags_60 },
    { 0xdfca22ee, SUC_lexicon_tags_11 },
    { 0xda4a22ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x952a22f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6a22f8, SUC_lexicon_tags_179 },
    { 0xd72a22f9, SUC_lexicon_tags_30 },
    { 0xf18a22f9, SUC_lexicon_tags_1 },
    { 0x4ca22fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8a2300, SUC_lexicon_tags_7 },
    { 0x826a2301, SUC_lexicon_tags_4 },
    { 0x8dca2300, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ca2309, SUC_lexicon_tags_10 },
    { 0xffca230a, SUC_lexicon_tags_3 },
    { 0x51ea230b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x740a230e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cca2311, SUC_lexicon_tags_0 },
    { 0xbb2a2311, SUC_lexicon_tags_12 },
    { 0x8a0a2313, SUC_lexicon_tags_21 },
    { 0xa90a2313, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552a2318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4a231e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2a232a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x818a232c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x578a232e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ca2331, SUC_lexicon_tags_17 },
    { 0x388a2331, SUC_lexicon_tags_18 },
    { 0xfb0a2333, SUC_lexicon_tags_12 },
    { 0x420a2334, SUC_lexicon_tags_2 },
    { 0x1a6a2332, SUC_lexicon_tags_7 },
    { 0x5a2a2336, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8a233a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638a2341, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a2347, SUC_lexicon_tags_46 },
    { 0x3cea2348, SUC_lexicon_tags_70 },
    { 0xc1ca2349, SUC_lexicon_tags_6 },
    { 0xb6aa2349, SUC_lexicon_tags_1 },
    { 0xf72a234b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192a234f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ca2352, SUC_lexicon_tags_31 },
    { 0xdaaa2353, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe3aa2355, SUC_lexicon_tags_12 },
    { 0xccca2356, SUC_lexicon_tags_0 },
    { 0x900a2357, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eea235b, SUC_lexicon_tags_47 },
    { 0x12ca235c, SUC_lexicon_tags_11 },
    { 0x3e0a235b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe90a235f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232a2363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2a2366, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x84a2368, SUC_lexicon_tags_28 },
    { 0x662a2369, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28a2371, SUC_lexicon_tags_25 },
    { 0xd08a2372, SUC_lexicon_tags_70 },
    { 0x40ea2371, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2a2376, SUC_lexicon_tags_26 },
    { 0xef6a2377, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8a237c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x42ea237e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xde6a2380, SUC_lexicon_tags_38 },
    { 0xec4a2381, SUC_lexicon_tags_92 },
    { 0x1eca2382, SUC_lexicon_tags_38 },
    { 0x2dca2383, SUC_lexicon_tags_47 },
    { 0xc0a2381, SUC_lexicon_tags_2 },
    { 0x850a2385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xadca2387, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x92ea2389, SUC_lexicon_tags_27 },
    { 0xe6ea238a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x212a238d, SUC_lexicon_tags_30 },
    { 0x1aca238e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6a2394, SUC_lexicon_tags_25 },
    { 0xbd4a2395, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbdaa2397, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xae4a2399, SUC_lexicon_tags_2 },
    { 0xd72a2399, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808a239f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xfd4a23a1, SUC_lexicon_tags_0 },
    { 0x8a4a23a2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74a23a5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ea23a8, SUC_lexicon_tags_17 },
    { 0xa23a9, SUC_lexicon_tags_33 },
    { 0x632a23a8, SUC_lexicon_tags_37 },
    { 0x80a23a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x694a23ad, SUC_lexicon_tags_48 },
    { 0xa94a23ad, SUC_lexicon_tags_11 },
    { 0x6cea23ae, SUC_lexicon_tags_87 },
    { 0x190a23b0, SUC_lexicon_tags_42 },
    { 0x6daa23b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866a23b6, SUC_lexicon_tags_1 },
    { 0xe3aa23b7, SUC_lexicon_tags_2 },
    { 0x40a23b8, SUC_lexicon_tags_1146 },
    { 0xb34a23b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6a23bd, SUC_lexicon_tags_25 },
    { 0x7b4a23be, SUC_lexicon_tags_29 },
    { 0x386a23bf, SUC_lexicon_tags_25 },
    { 0x7ca23be, SUC_lexicon_tags_37 },
    { 0xdb8a23c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x674a23c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x950a23c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6a23c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ea23cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4a23cf, SUC_lexicon_tags_68 },
    { 0x734a23d0, SUC_lexicon_tags_55 },
    { 0xb0aa23d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2a23d4, SUC_lexicon_tags_63 },
    { 0x3f2a23d4, SUC_lexicon_tags_5 },
    { 0x652a23d6, SUC_lexicon_tags_17 },
    { 0xa06a23d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8a23e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab8a23e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x200a23e5, SUC_lexicon_tags_35 },
    { 0x686a23e6, SUC_lexicon_tags_37 },
    { 0x80ea23e7, SUC_lexicon_tags_49 },
    { 0x512a23e8, SUC_lexicon_tags_25 },
    { 0xb0ea23e8, SUC_lexicon_tags_38 },
    { 0xc58a23e9, SUC_lexicon_tags_16 },
    { 0x428a23eb, SUC_lexicon_tags_27 },
    { 0x946a23ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0a23ef, SUC_lexicon_tags_7 },
    { 0x212a23f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea23f5, SUC_lexicon_tags_7 },
    { 0x550a23f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38a23fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aa2400, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4a2406, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472a240c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaa2414, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaa241f, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ea2422, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed4a2424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a2427, SUC_lexicon_tags_8 },
    { 0x1faa2427, SUC_lexicon_tags_47 },
    { 0x2f0a2427, SUC_lexicon_tags_8 },
    { 0xaeaa2428, SUC_lexicon_tags_13 },
    { 0xd5ca242b, SUC_lexicon_tags_44 },
    { 0x70ca2428, SUC_lexicon_tags_19 },
    { 0xe94a242d, SUC_lexicon_tags_19 },
    { 0x7d8a242b, SUC_lexicon_tags_47 },
    { 0x20ca242f, SUC_lexicon_tags_63 },
    { 0xf52a2430, SUC_lexicon_tags_0 },
    { 0x1d2a242f, SUC_lexicon_tags_37 },
    { 0x3faa2432, SUC_lexicon_tags_25 },
    { 0x640a2433, SUC_lexicon_tags_7 },
    { 0x314a2431, SUC_lexicon_tags_2 },
    { 0x32ca2435, SUC_lexicon_tags_12 },
    { 0x7c0a2436, SUC_lexicon_tags_45 },
    { 0x7c8a2435, SUC_lexicon_tags_4 },
    { 0x56aa2438, SUC_lexicon_tags_8 },
    { 0x748a2439, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x94aa243b, SUC_lexicon_tags_7 },
    { 0xc9aa243c, SUC_lexicon_tags_68 },
    { 0x80ca243c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4e0a243f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab8a2441, SUC_lexicon_tags_1 },
    { 0x28ea2442, SUC_lexicon_tags_27 },
    { 0xa06a2443, SUC_lexicon_tags_11 },
    { 0x5caa2444, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ca2450, SUC_lexicon_tags_27 },
    { 0xafca2450, SUC_lexicon_tags_2 },
    { 0xf8ca2450, SUC_lexicon_tags_18 },
    { 0x704a2451, SUC_lexicon_tags_26 },
    { 0xae2a2453, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54a2457, SUC_lexicon_tags_14 },
    { 0xb18a2458, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60aa245a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6a245d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x70a245f, SUC_lexicon_tags_2 },
    { 0x804a2460, SUC_lexicon_tags_47 },
    { 0xd94a2461, SUC_lexicon_tags_11 },
    { 0x856a245f, SUC_lexicon_tags_9 },
    { 0xb36a2463, SUC_lexicon_tags_0 },
    { 0x5aca245f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8a2468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa246b, SUC_lexicon_tags_100 },
    { 0xbea246c, SUC_lexicon_tags_38 },
    { 0x91ea246c, SUC_lexicon_tags_15 },
    { 0x100a246e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x694a2471, SUC_lexicon_tags_25 },
    { 0x714a2471, SUC_lexicon_tags_2 },
    { 0xf00a2473, SUC_lexicon_tags_1 },
    { 0x15aa2474, SUC_lexicon_tags_63 },
    { 0x200a2475, SUC_lexicon_tags_8 },
    { 0x32ca2475, SUC_lexicon_tags_11 },
    { 0xacea2475, SUC_lexicon_tags_1 },
    { 0x9b8a2472, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x822a247a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc84a247c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaa2481, SUC_lexicon_tags_17 },
    { 0xbcaa2482, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x93aa2484, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a8a2486, SUC_lexicon_tags_0 },
    { 0x3f8a2486, SUC_lexicon_tags_12 },
    { 0xb38a2488, SUC_lexicon_tags_30 },
    { 0xf00a2487, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ea248d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8a2490, SUC_lexicon_tags_14 },
    { 0x6aa2490, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x952a2493, SUC_lexicon_tags_40 },
    { 0x70ca2494, SUC_lexicon_tags_47 },
    { 0x596a2495, SUC_lexicon_tags_49 },
    { 0x316a2496, SUC_lexicon_tags_63 },
    { 0x554a2494, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6c6a2499, SUC_lexicon_tags_12 },
    { 0xf56a249a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546a249e, SUC_lexicon_tags_43 },
    { 0xeeaa249e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8a24a5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x41ca24a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb4a24a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8aa24af, SUC_lexicon_tags_27 },
    { 0xc88a24b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd68a24b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x324a24b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14aa24bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa0ea24bd, SUC_lexicon_tags_19 },
    { 0x87ca24be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42a24c5, SUC_lexicon_tags_11 },
    { 0xcc4a24c6, SUC_lexicon_tags_7 },
    { 0x786a24c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304a24d8, SUC_lexicon_tags_47 },
    { 0x980a24d9, SUC_lexicon_tags_26 },
    { 0x602a24da, SUC_lexicon_tags_57 },
    { 0x9b0a24db, SUC_lexicon_tags_30 },
    { 0x93aa24da, SUC_lexicon_tags_63 },
    { 0xb28a24da, SUC_lexicon_tags_25 },
    { 0x8c2a24de, SUC_lexicon_tags_11 },
    { 0x938a24de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4a24e2, SUC_lexicon_tags_23 },
    { 0xfeea24e2, SUC_lexicon_tags_7 },
    { 0x6f6a24e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74a24e7, SUC_lexicon_tags_37 },
    { 0xd1ea24e7, SUC_lexicon_tags_44 },
    { 0x5eca24e8, SUC_lexicon_tags_7 },
    { 0xe14a24e9, SUC_lexicon_tags_12 },
    { 0xc40a24eb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eca24ee, SUC_lexicon_tags_38 },
    { 0x8eea24ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaa24f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6caa24fd, SUC_lexicon_tags_37 },
    { 0xda2a24fe, SUC_lexicon_tags_24 },
    { 0x9a2a24ff, SUC_lexicon_tags_6 },
    { 0xe2aa2500, SUC_lexicon_tags_15 },
    { 0x27ea2500, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24aa250b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaca250e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1caa2510, SUC_lexicon_tags_14 },
    { 0xf5ea2511, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600a2519, SUC_lexicon_tags_14 },
    { 0xd9ea251a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x40aa251d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc82a251f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45aa2521, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250a2525, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ca252d, SUC_lexicon_tags_13 },
    { 0x6f0a252e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ea2531, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d4a2533, SUC_lexicon_tags_7 },
    { 0x4a0a2533, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8a2537, SUC_lexicon_tags_17 },
    { 0x6a4a2538, SUC_lexicon_tags_38 },
    { 0xeea2539, SUC_lexicon_tags_55 },
    { 0x4ea253a, SUC_lexicon_tags_31 },
    { 0x5faa2539, SUC_lexicon_tags_47 },
    { 0x9eaa253b, SUC_lexicon_tags_15 },
    { 0x43ea253d, SUC_lexicon_tags_17 },
    { 0xe10a253e, SUC_lexicon_tags_12 },
    { 0x8e8a253f, SUC_lexicon_tags_37 },
    { 0xf58a2540, SUC_lexicon_tags_12 },
    { 0x242a2541, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4a2549, SUC_lexicon_tags_38 },
    { 0x76ca254a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842a254e, SUC_lexicon_tags_37 },
    { 0x110a254e, SUC_lexicon_tags_24 },
    { 0x9e0a254e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ea2555, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa58a2557, SUC_lexicon_tags_12 },
    { 0xf46a2558, SUC_lexicon_tags_26 },
    { 0x182a2559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12ca255b, SUC_lexicon_tags_25 },
    { 0x262a255b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ca2561, SUC_lexicon_tags_1 },
    { 0x5b2a2561, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x750a2564, SUC_lexicon_tags_63 },
    { 0xe8ea2565, SUC_lexicon_tags_14 },
    { 0xee6a2564, SUC_lexicon_tags_37 },
    { 0x5eea2564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a256d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8ea256f, SUC_lexicon_tags_10 },
    { 0x67ea256f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c2a2572, SUC_lexicon_tags_12 },
    { 0x8e8a2572, SUC_lexicon_tags_33 },
    { 0xacca2574, SUC_lexicon_tags_10 },
    { 0x6f6a2572, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x598a2577, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa68a2579, SUC_lexicon_tags_2 },
    { 0x75ea257a, SUC_lexicon_tags_687 },
    { 0xc6a257b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80a257f, SUC_lexicon_tags_6 },
    { 0xb04a2580, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eea2584, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe68a2586, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x656a2589, SUC_lexicon_tags_39 },
    { 0x4bca258a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81aa2597, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6a25a1, SUC_lexicon_tags_38 },
    { 0x98a25a2, SUC_lexicon_tags_19 },
    { 0x8d8a25a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6a25ae, SUC_lexicon_tags_15 },
    { 0x56a25af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fea25b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634a25b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20ea25b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35aa25bd, SUC_lexicon_tags_23 },
    { 0x778a25be, SUC_lexicon_tags_151 },
    { 0xc70a25be, SUC_lexicon_tags_1 },
    { 0x56ea25bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ea25c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x536a25c8, SUC_lexicon_tags_9 },
    { 0xe38a25c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94a25cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ea25d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12a25d5, SUC_lexicon_tags_37 },
    { 0x838a25d6, SUC_lexicon_tags_1 },
    { 0x6b0a25d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aa25db, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x658a25de, SUC_lexicon_tags_6 },
    { 0x8f0a25de, SUC_lexicon_tags_12 },
    { 0xda2a25e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93aa25e4, SUC_lexicon_tags_33 },
    { 0xb54a25e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdaaa25e7, SUC_lexicon_tags_7 },
    { 0x266a25e8, SUC_lexicon_tags_37 },
    { 0x43aa25e9, SUC_lexicon_tags_1 },
    { 0x586a25e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x496a25ec, SUC_lexicon_tags_19 },
    { 0xdd6a25ed, SUC_lexicon_tags_10 },
    { 0x924a25ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25aa25f4, SUC_lexicon_tags_33 },
    { 0x12ea25f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6a25fa, SUC_lexicon_tags_14 },
    { 0x278a25fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54ca25fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ea2601, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83aa2608, SUC_lexicon_tags_17 },
    { 0x8c2a2609, SUC_lexicon_tags_7 },
    { 0xfaaa2609, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfaa260f, SUC_lexicon_tags_48 },
    { 0x9aa260f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8a2613, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8a2617, SUC_lexicon_tags_8 },
    { 0xee4a2618, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88a261e, SUC_lexicon_tags_30 },
    { 0x9e8a261f, SUC_lexicon_tags_38 },
    { 0x518a2620, SUC_lexicon_tags_25 },
    { 0x57aa2621, SUC_lexicon_tags_107 },
    { 0x3aea2622, SUC_lexicon_tags_45 },
    { 0x432a2623, SUC_lexicon_tags_39 },
    { 0xbbea2622, SUC_lexicon_tags_60 },
    { 0x91aa2621, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ca2628, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480a262e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbca2631, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4a2637, SUC_lexicon_tags_15 },
    { 0x262a2637, SUC_lexicon_tags_40 },
    { 0x2ea2637, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x90ca263b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96a2646, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa86a2648, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74a264b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x924a264e, SUC_lexicon_tags_19 },
    { 0x1dca264e, SUC_lexicon_tags_19 },
    { 0x10ca2650, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8fca2652, SUC_lexicon_tags_60 },
    { 0x43aa2652, SUC_lexicon_tags_46 },
    { 0x7faa2654, SUC_lexicon_tags_95 },
    { 0x8fea2654, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94a265b, SUC_lexicon_tags_15 },
    { 0x974a265c, SUC_lexicon_tags_25 },
    { 0xd7aa265c, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98a2660, SUC_lexicon_tags_15 },
    { 0xaeea2661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94a2664, SUC_lexicon_tags_1 },
    { 0x54ca2665, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4a2667, SUC_lexicon_tags_25 },
    { 0x768a2667, SUC_lexicon_tags_1 },
    { 0x792a2668, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ca2670, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x352a2672, SUC_lexicon_tags_14 },
    { 0x9caa2672, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6b2a2675, SUC_lexicon_tags_2 },
    { 0xd2a2676, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x570a2678, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4a267b, SUC_lexicon_tags_38 },
    { 0xb86a267c, SUC_lexicon_tags_40 },
    { 0xb6aa267d, SUC_lexicon_tags_37 },
    { 0x2b8a267d, SUC_lexicon_tags_1 },
    { 0xf2ea267f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9d4a2681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcaa2683, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aea268a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4a268d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x614a2690, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd5aa2692, SUC_lexicon_tags_27 },
    { 0x772a2693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6a2697, SUC_lexicon_tags_12 },
    { 0xb16a2698, SUC_lexicon_tags_17 },
    { 0x1b4a2698, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8a26a6, SUC_lexicon_tags_6 },
    { 0x440a26a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14ca26a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2a26ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc34a26ae, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8a26b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x346a26bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2a26be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x204a26c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea26c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a0a26c6, SUC_lexicon_tags_32 },
    { 0xd8aa26c7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0a26cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ea26cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x630a26d1, SUC_lexicon_tags_13 },
    { 0x3ca26d1, SUC_lexicon_tags_0 },
    { 0x7a0a26d3, SUC_lexicon_tags_60 },
    { 0x7c4a26d3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf1ea26d6, SUC_lexicon_tags_60 },
    { 0x742a26d7, SUC_lexicon_tags_12 },
    { 0xe7ca26d7, SUC_lexicon_tags_17 },
    { 0x4e6a26d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa26dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a26df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ca26e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb62a26e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aca26e9, SUC_lexicon_tags_10 },
    { 0x174a26ea, SUC_lexicon_tags_25 },
    { 0x946a26e9, SUC_lexicon_tags_0 },
    { 0x4cea26ec, SUC_lexicon_tags_25 },
    { 0x2e6a26e9, SUC_lexicon_tags_47 },
    { 0x790a26ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538a2701, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6a2705, SUC_lexicon_tags_77 },
    { 0x59ea2706, SUC_lexicon_tags_25 },
    { 0xad2a2706, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aa270b, SUC_lexicon_tags_11 },
    { 0x43ea270c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x182a270f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aa2712, SUC_lexicon_tags_23 },
    { 0x3b0a2713, SUC_lexicon_tags_39 },
    { 0x2caa2713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34a2717, SUC_lexicon_tags_6 },
    { 0xb1aa2717, SUC_lexicon_tags_7 },
    { 0xe2aa2719, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782a271f, SUC_lexicon_tags_4 },
    { 0x13ea271f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaa2725, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45ea2727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x232a2729, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bea272b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbe6a272d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x828a272f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2a2733, SUC_lexicon_tags_14 },
    { 0x1caa2734, SUC_lexicon_tags_1 },
    { 0x1dca2735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6a2738, SUC_lexicon_tags_17 },
    { 0xebea2738, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730a273d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4a2740, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6a2743, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10aa2746, SUC_lexicon_tags_2 },
    { 0x964a2746, SUC_lexicon_tags_6 },
    { 0xabaa2748, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0a274a, SUC_lexicon_tags_63 },
    { 0xa2ca274b, SUC_lexicon_tags_21 },
    { 0x918a274c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x858a274f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33ca2751, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x762a2753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26ea2755, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650a275a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90a2760, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40a2764, SUC_lexicon_tags_14 },
    { 0xdb4a2765, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a276b, SUC_lexicon_tags_25 },
    { 0x4fea276b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x162a276e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8a2771, SUC_lexicon_tags_37 },
    { 0x424a2772, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ea2776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8a2779, SUC_lexicon_tags_3 },
    { 0x6ea277a, SUC_lexicon_tags_15 },
    { 0x764a277b, SUC_lexicon_tags_27 },
    { 0x656a2779, SUC_lexicon_tags_53 },
    { 0xf28a277d, SUC_lexicon_tags_25 },
    { 0x69aa277d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc2ea2780, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540a2784, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00a2787, SUC_lexicon_tags_61 },
    { 0xe4aa2787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15ca278a, SUC_lexicon_tags_68 },
    { 0x14ca278b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba0a278d, SUC_lexicon_tags_70 },
    { 0x188a278e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ea2791, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x97ea2793, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x368a2795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2a2799, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x576a279b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb70a279d, SUC_lexicon_tags_1 },
    { 0x5d8a279d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x728a27a0, SUC_lexicon_tags_29 },
    { 0x116a27a1, SUC_lexicon_tags_4 },
    { 0x3c2a27a2, SUC_lexicon_tags_45 },
    { 0xac8a27a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1ca27a5, SUC_lexicon_tags_16 },
    { 0x98aa27a6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0a27a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02a27b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460a27bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6a27c2, SUC_lexicon_tags_12 },
    { 0xb6a27c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38aa27c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32a27c8, SUC_lexicon_tags_7 },
    { 0xeb4a27c8, SUC_lexicon_tags_23 },
    { 0xdd2a27c9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf5ea27cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaa27d2, SUC_lexicon_tags_2 },
    { 0xc4ea27d3, SUC_lexicon_tags_17 },
    { 0xee2a27d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x658a27d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafaa27d8, SUC_lexicon_tags_37 },
    { 0xc0a27d9, SUC_lexicon_tags_9 },
    { 0x424a27da, SUC_lexicon_tags_11 },
    { 0xc4ca27d8, SUC_lexicon_tags_17 },
    { 0x468a27dc, SUC_lexicon_tags_25 },
    { 0xfa0a27da, SUC_lexicon_tags_170 },
    { 0x8cea27dc, SUC_lexicon_tags_1 },
    { 0x4c2a27de, SUC_lexicon_tags_1 },
    { 0xd20a27db, SUC_lexicon_tags_43 },
    { 0x490a27e1, SUC_lexicon_tags_25 },
    { 0x46a27e1, SUC_lexicon_tags_25 },
    { 0x774a27e3, SUC_lexicon_tags_29 },
    { 0x9ca27e3, SUC_lexicon_tags_31 },
    { 0xcb6a27e5, SUC_lexicon_tags_11 },
    { 0xe82a27e6, SUC_lexicon_tags_27 },
    { 0x91ca27e7, SUC_lexicon_tags_37 },
    { 0xf64a27e5, SUC_lexicon_tags_80 },
    { 0x75aa27dc, SUC_lexicon_tags_29 },
    { 0xceea27db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00a27ed, SUC_lexicon_tags_49 },
    { 0x86ea27ed, SUC_lexicon_tags_47 },
    { 0x688a27ed, SUC_lexicon_tags_27 },
    { 0x68ea27f0, SUC_lexicon_tags_25 },
    { 0x4f4a27ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6ea27f3, SUC_lexicon_tags_37 },
    { 0x39ca27f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc02a27f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca27f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaa27fc, SUC_lexicon_tags_17 },
    { 0x73aa27fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8a2802, SUC_lexicon_tags_17 },
    { 0xeea2803, SUC_lexicon_tags_48 },
    { 0xc2ea2804, SUC_lexicon_tags_14 },
    { 0xd08a2802, SUC_lexicon_tags_15 },
    { 0xac8a2806, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0a280c, SUC_lexicon_tags_15 },
    { 0x2d8a280d, SUC_lexicon_tags_1 },
    { 0x5c2a280e, SUC_lexicon_tags_21 },
    { 0x59ca280f, SUC_lexicon_tags_26 },
    { 0x926a280e, SUC_lexicon_tags_17 },
    { 0xec2a2811, SUC_lexicon_tags_49 },
    { 0x982a2812, SUC_lexicon_tags_29 },
    { 0x106a280f, SUC_lexicon_tags_45 },
    { 0xa06a280f, SUC_lexicon_tags_25 },
    { 0xc3aa2815, SUC_lexicon_tags_25 },
    { 0x96aa280c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbb0a2818, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3cea281a, SUC_lexicon_tags_25 },
    { 0x992a281a, SUC_lexicon_tags_57 },
    { 0xceea281c, SUC_lexicon_tags_37 },
    { 0xe64a281d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x354a281f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x748a2821, SUC_lexicon_tags_6 },
    { 0x868a2821, SUC_lexicon_tags_12 },
    { 0x9cca2822, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870a2828, SUC_lexicon_tags_26 },
    { 0xabaa2829, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4faa282b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x438a282e, SUC_lexicon_tags_14 },
    { 0xe88a282e, SUC_lexicon_tags_92 },
    { 0x2e4a282f, SUC_lexicon_tags_49 },
    { 0x9f0a282e, SUC_lexicon_tags_23 },
    { 0xbcca2832, SUC_lexicon_tags_27 },
    { 0x8c8a2832, SUC_lexicon_tags_12 },
    { 0xbaca2834, SUC_lexicon_tags_15 },
    { 0x4e6a2835, SUC_lexicon_tags_47 },
    { 0xb0a2832, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8a283d, SUC_lexicon_tags_12 },
    { 0x3d0a283d, SUC_lexicon_tags_51 },
    { 0x3aca283d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32ea2841, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ea2845, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb76a2847, SUC_lexicon_tags_419 },
    { 0x1f2a2847, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5faa284c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6a2850, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfe6a2852, SUC_lexicon_tags_49 },
    { 0x748a2853, SUC_lexicon_tags_47 },
    { 0x3d4a2853, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb04a2856, SUC_lexicon_tags_9 },
    { 0x64ea2856, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x200a285a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ea2863, SUC_lexicon_tags_25 },
    { 0xb1aa2864, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2ca2866, SUC_lexicon_tags_80 },
    { 0xe4a2867, SUC_lexicon_tags_45 },
    { 0x5f2a2867, SUC_lexicon_tags_75 },
    { 0x18ca2868, SUC_lexicon_tags_25 },
    { 0x842a286a, SUC_lexicon_tags_31 },
    { 0xb78a286b, SUC_lexicon_tags_55 },
    { 0x568a286c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d6a286e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x640a2870, SUC_lexicon_tags_21 },
    { 0x466a2871, SUC_lexicon_tags_7 },
    { 0x9c8a2871, SUC_lexicon_tags_25 },
    { 0x174a2873, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cca2876, SUC_lexicon_tags_65 },
    { 0x40ca2877, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04a287b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0a2880, SUC_lexicon_tags_25 },
    { 0x3c0a2881, SUC_lexicon_tags_2 },
    { 0xfea2882, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a2888, SUC_lexicon_tags_11 },
    { 0xfa6a2888, SUC_lexicon_tags_46 },
    { 0x7d8a2889, SUC_lexicon_tags_0 },
    { 0x92a288a, SUC_lexicon_tags_84 },
    { 0x560a2889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe90a288e, SUC_lexicon_tags_37 },
    { 0x102a288e, SUC_lexicon_tags_25 },
    { 0x750a288f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aa2895, SUC_lexicon_tags_66 },
    { 0x7d6a2896, SUC_lexicon_tags_32 },
    { 0x83ca2896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ea289a, SUC_lexicon_tags_12 },
    { 0x41aa289a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x880a289d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdaaa289f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644a28a3, SUC_lexicon_tags_49 },
    { 0xaa6a28a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b6a28a6, SUC_lexicon_tags_17 },
    { 0x638a28a6, SUC_lexicon_tags_17 },
    { 0xf8aa28a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0a28ab, SUC_lexicon_tags_1 },
    { 0x4eaa28ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ca28b0, SUC_lexicon_tags_17 },
    { 0x882a28b1, SUC_lexicon_tags_25 },
    { 0x7c0a28b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x38a28b5, SUC_lexicon_tags_18 },
    { 0x7eea28b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x858a28b9, SUC_lexicon_tags_82 },
    { 0xa42a28b9, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ea28bd, SUC_lexicon_tags_32 },
    { 0x416a28be, SUC_lexicon_tags_17 },
    { 0x7c6a28be, SUC_lexicon_tags_8 },
    { 0x9aaa28c0, SUC_lexicon_tags_25 },
    { 0x5d0a28bd, SUC_lexicon_tags_37 },
    { 0xa92a28c2, SUC_lexicon_tags_17 },
    { 0x5b0a28c0, SUC_lexicon_tags_1 },
    { 0xd40a28c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4a28cf, SUC_lexicon_tags_1 },
    { 0xd7ea28d0, SUC_lexicon_tags_37 },
    { 0x628a28cf, SUC_lexicon_tags_2 },
    { 0xe20a28d2, SUC_lexicon_tags_19 },
    { 0xd16a28d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694a28d9, SUC_lexicon_tags_19 },
    { 0x852a28d9, SUC_lexicon_tags_11 },
    { 0x99ea28db, SUC_lexicon_tags_11 },
    { 0xbe2a28db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5cea28de, SUC_lexicon_tags_17 },
    { 0x6b0a28df, SUC_lexicon_tags_17 },
    { 0x49ea28de, SUC_lexicon_tags_66 },
    { 0xb38a28e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ca28e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96a28e7, SUC_lexicon_tags_4 },
    { 0xdd8a28e7, SUC_lexicon_tags_11 },
    { 0x260a28e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x720a28eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0a28f2, SUC_lexicon_tags_37 },
    { 0x1f4a28f2, SUC_lexicon_tags_26 },
    { 0x272a28f4, SUC_lexicon_tags_37 },
    { 0xb0ca28f5, SUC_lexicon_tags_17 },
    { 0x6f8a28f2, SUC_lexicon_tags_15 },
    { 0x782a28f7, SUC_lexicon_tags_10 },
    { 0xb8a28f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd8a28fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0a28fd, SUC_lexicon_tags_39 },
    { 0x93ca28fe, SUC_lexicon_tags_43 },
    { 0xadaa28ff, SUC_lexicon_tags_23 },
    { 0xb5ea2900, SUC_lexicon_tags_1 },
    { 0xb38a2901, SUC_lexicon_tags_29 },
    { 0x26aa2902, SUC_lexicon_tags_1 },
    { 0x1b2a2901, SUC_lexicon_tags_33 },
    { 0xa98a2904, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x906a2906, SUC_lexicon_tags_60 },
    { 0x86ea2907, SUC_lexicon_tags_63 },
    { 0x7c8a2908, SUC_lexicon_tags_17 },
    { 0xb90a2908, SUC_lexicon_tags_12 },
    { 0x526a290a, SUC_lexicon_tags_37 },
    { 0xa4ea290b, SUC_lexicon_tags_38 },
    { 0x70aa290c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x574a290e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1d8a2910, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x802a2912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x890a2915, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x126a2918, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe28a291a, SUC_lexicon_tags_1 },
    { 0x414a291b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb8a291d, SUC_lexicon_tags_7 },
    { 0xb80a291d, SUC_lexicon_tags_29 },
    { 0x726a291e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9aa2922, SUC_lexicon_tags_30 },
    { 0x8eca2922, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ea2925, SUC_lexicon_tags_23 },
    { 0xa62a2926, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfc8a2928, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca292b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558a2930, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf0aa2932, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aea2937, SUC_lexicon_tags_26 },
    { 0x61ca2938, SUC_lexicon_tags_77 },
    { 0x27ca2939, SUC_lexicon_tags_7 },
    { 0xaf4a293a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88a293d, SUC_lexicon_tags_7 },
    { 0x562a293e, SUC_lexicon_tags_7 },
    { 0x108a293d, SUC_lexicon_tags_7 },
    { 0x81ea293e, SUC_lexicon_tags_37 },
    { 0xc6a293d, SUC_lexicon_tags_33 },
    { 0xf22a2942, SUC_lexicon_tags_56 },
    { 0x350a2940, SUC_lexicon_tags_19 },
    { 0x6d8a2942, SUC_lexicon_tags_37 },
    { 0xe3ca2945, SUC_lexicon_tags_11 },
    { 0xf4ca2946, SUC_lexicon_tags_4 },
    { 0x240a2946, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6a294b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x264a294d, SUC_lexicon_tags_7 },
    { 0x87ca294e, SUC_lexicon_tags_39 },
    { 0xf3ea294e, SUC_lexicon_tags_37 },
    { 0x824a294f, SUC_lexicon_tags_37 },
    { 0xeca294e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a6a2953, SUC_lexicon_tags_218 },
    { 0x392a2954, SUC_lexicon_tags_12 },
    { 0x132a2955, SUC_lexicon_tags_60 },
    { 0xb68a2956, SUC_lexicon_tags_27 },
    { 0x8b4a2957, SUC_lexicon_tags_1 },
    { 0xc00a2956, SUC_lexicon_tags_7 },
    { 0xaaaa2959, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34a295d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78aa295f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaa2963, SUC_lexicon_tags_1 },
    { 0x1f6a2964, SUC_lexicon_tags_12 },
    { 0x974a2965, SUC_lexicon_tags_17 },
    { 0x458a2965, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd22a2968, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6a296d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6aa296f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfca2972, SUC_lexicon_tags_15 },
    { 0x558a2973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc24a2975, SUC_lexicon_tags_31 },
    { 0xa58a2975, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564a297a, SUC_lexicon_tags_32 },
    { 0x590a297b, SUC_lexicon_tags_63 },
    { 0x5f0a297b, SUC_lexicon_tags_38 },
    { 0xb72a297b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x13aa2980, SUC_lexicon_tags_0 },
    { 0xcd4a2980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x856a2984, SUC_lexicon_tags_23 },
    { 0xa50a2984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92a2988, SUC_lexicon_tags_44 },
    { 0x9dca2988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8a298e, SUC_lexicon_tags_26 },
    { 0x138a298e, SUC_lexicon_tags_26 },
    { 0xf24a2990, SUC_lexicon_tags_2 },
    { 0x562a2990, SUC_lexicon_tags_14 },
    { 0x25aa2990, SUC_lexicon_tags_19 },
    { 0x250a2993, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79aa2995, SUC_lexicon_tags_16 },
    { 0xf2a2996, SUC_lexicon_tags_6 },
    { 0xc56a2997, SUC_lexicon_tags_38 },
    { 0xb2a2998, SUC_lexicon_tags_2 },
    { 0x6caa2995, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842a299e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aca29a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ea29a8, SUC_lexicon_tags_1 },
    { 0xacea29a9, SUC_lexicon_tags_45 },
    { 0xf04a29aa, SUC_lexicon_tags_37 },
    { 0x4b4a29ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x810a29ae, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2a29b1, SUC_lexicon_tags_17 },
    { 0x300a29b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdea29b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58aa29b8, SUC_lexicon_tags_8 },
    { 0xdf6a29b9, SUC_lexicon_tags_4 },
    { 0x686a29ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2a29c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70a29ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x226a29cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3f4a29ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2a29d4, SUC_lexicon_tags_7 },
    { 0xa3ea29d5, SUC_lexicon_tags_63 },
    { 0x92ea29d6, SUC_lexicon_tags_25 },
    { 0xf44a29d7, SUC_lexicon_tags_15 },
    { 0x73aa29d6, SUC_lexicon_tags_7 },
    { 0xd20a29d5, SUC_lexicon_tags_12 },
    { 0x192a29d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a29dd, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xba6a29df, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04a29e3, SUC_lexicon_tags_43 },
    { 0xa6ca29e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xefaa29e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8a29e9, SUC_lexicon_tags_7 },
    { 0x4c0a29e9, SUC_lexicon_tags_11 },
    { 0xc0a29eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3faa29ed, SUC_lexicon_tags_27 },
    { 0x780a29ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x584a29f0, SUC_lexicon_tags_15 },
    { 0x7faa29f1, SUC_lexicon_tags_12 },
    { 0xb90a29f2, SUC_lexicon_tags_55 },
    { 0x596a29f3, SUC_lexicon_tags_31 },
    { 0x256a29f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ea29f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4a29fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ea29ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8a2a07, SUC_lexicon_tags_37 },
    { 0x73ca2a08, SUC_lexicon_tags_37 },
    { 0xf90a2a07, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602a2a0d, SUC_lexicon_tags_7 },
    { 0x90a2a0e, SUC_lexicon_tags_37 },
    { 0x28ca2a0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16a2a13, SUC_lexicon_tags_68 },
    { 0x264a2a14, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4aa2a16, SUC_lexicon_tags_37 },
    { 0xb66a2a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92a2a1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ea2a1d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x164a2a20, SUC_lexicon_tags_35 },
    { 0x614a2a21, SUC_lexicon_tags_37 },
    { 0x3e0a2a21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaa2a25, SUC_lexicon_tags_38 },
    { 0x7eaa2a26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53aa2a2a, SUC_lexicon_tags_9 },
    { 0xc24a2a2a, SUC_lexicon_tags_25 },
    { 0xa64a2a2c, SUC_lexicon_tags_1 },
    { 0x98ca2a2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0a2a31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ea2a37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556a2a3d, SUC_lexicon_tags_77 },
    { 0x622a2a3e, SUC_lexicon_tags_4 },
    { 0x9b2a2a3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaa2a49, SUC_lexicon_tags_11 },
    { 0x538a2a49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498a2a52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68a2a57, SUC_lexicon_tags_19 },
    { 0x178a2a57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900a2a5c, SUC_lexicon_tags_30 },
    { 0xc04a2a5d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16a2a65, SUC_lexicon_tags_15 },
    { 0xdfea2a66, SUC_lexicon_tags_47 },
    { 0xab8a2a67, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa0ea2a69, SUC_lexicon_tags_1 },
    { 0x916a2a69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268a2a6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16a2a76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeb4a2a78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ea2a7b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4dea2a7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x140a2a80, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x504a2a83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ea2a86, SUC_lexicon_tags_2 },
    { 0x6aa2a86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf04a2a89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaa2a8f, SUC_lexicon_tags_17 },
    { 0x5faa2a90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb2a2a92, SUC_lexicon_tags_21 },
    { 0x4aa2a93, SUC_lexicon_tags_15 },
    { 0x86aa2a93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30a2a98, SUC_lexicon_tags_15 },
    { 0x882a2a98, SUC_lexicon_tags_27 },
    { 0x8c0a2a98, SUC_lexicon_tags_26 },
    { 0x28ca2a9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302a2aa1, SUC_lexicon_tags_812 },
    { 0xb28a2aa2, SUC_lexicon_tags_103 },
    { 0x9d6a2aa3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ca2aa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362a2aad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ea2ab0, SUC_lexicon_tags_11 },
    { 0x936a2ab1, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ea2ab4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4a2ab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaa2ac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34a2ac6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x12ea2ac8, SUC_lexicon_tags_14 },
    { 0xd54a2ac9, SUC_lexicon_tags_77 },
    { 0x994a2aca, SUC_lexicon_tags_59 },
    { 0x5b6a2acb, SUC_lexicon_tags_0 },
    { 0xb8ea2acc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bca2ad1, SUC_lexicon_tags_27 },
    { 0xdcaa2ad2, SUC_lexicon_tags_10 },
    { 0x42a2ad3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00a2ad6, SUC_lexicon_tags_9 },
    { 0x91ca2ad6, SUC_lexicon_tags_17 },
    { 0x972a2ad7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaa2ade, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2a2ae2, SUC_lexicon_tags_2 },
    { 0xf08a2ae3, SUC_lexicon_tags_10 },
    { 0xda0a2ae4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbe0a2ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ea2ae8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aa2af6, SUC_lexicon_tags_7 },
    { 0xa2af6, SUC_lexicon_tags_14 },
    { 0x6aca2af8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684a2afe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1aaa2b00, SUC_lexicon_tags_151 },
    { 0x610a2b00, SUC_lexicon_tags_17 },
    { 0x75ca2b02, SUC_lexicon_tags_25 },
    { 0xd18a2b00, SUC_lexicon_tags_45 },
    { 0x104a2b04, SUC_lexicon_tags_3 },
    { 0xbea2b04, SUC_lexicon_tags_37 },
    { 0x62ea2b04, SUC_lexicon_tags_4 },
    { 0x686a2b07, SUC_lexicon_tags_60 },
    { 0x978a2b07, SUC_lexicon_tags_28 },
    { 0x8a2a2b09, SUC_lexicon_tags_100 },
    { 0x3f8a2b00, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x628a2b0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x32ca2b0f, SUC_lexicon_tags_17 },
    { 0xd50a2b10, SUC_lexicon_tags_22 },
    { 0x7eca2b11, SUC_lexicon_tags_23 },
    { 0xf80a2b10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0a2b18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8a2b1b, SUC_lexicon_tags_11 },
    { 0x31ca2b1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6aea2b1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158a2b23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf10a2b25, SUC_lexicon_tags_17 },
    { 0xfaca2b25, SUC_lexicon_tags_6 },
    { 0x6ca2b25, SUC_lexicon_tags_1 },
    { 0x4daa2b26, SUC_lexicon_tags_17 },
    { 0x4aca2b27, SUC_lexicon_tags_63 },
    { 0x394a2b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa60a2b2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ea2b2f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298a2b33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66a2b3f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a2b46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34a2b4a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20aa2b4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92a2b51, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a2b5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x892a2b5d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8a6a2b5f, SUC_lexicon_tags_11 },
    { 0xb9ca2b60, SUC_lexicon_tags_14 },
    { 0xfca2b61, SUC_lexicon_tags_7 },
    { 0xffca2b61, SUC_lexicon_tags_37 },
    { 0x51ca2b63, SUC_lexicon_tags_0 },
    { 0xf56a2b61, SUC_lexicon_tags_15 },
    { 0xc4a2b65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47ea2b67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fea2b6e, SUC_lexicon_tags_37 },
    { 0x65ea2b6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59aa2b72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62a2b76, SUC_lexicon_tags_9 },
    { 0x7a6a2b77, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x3cca2b79, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2a2b7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf1ca2b7f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49aa2b83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x982a2b86, SUC_lexicon_tags_35 },
    { 0x92ca2b87, SUC_lexicon_tags_7 },
    { 0xa44a2b87, SUC_lexicon_tags_19 },
    { 0x684a2b87, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62a2b8e, SUC_lexicon_tags_15 },
    { 0x10ca2b8f, SUC_lexicon_tags_7 },
    { 0xe8aa2b8e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8a2b94, SUC_lexicon_tags_28 },
    { 0xb76a2b95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x940a2b98, SUC_lexicon_tags_15 },
    { 0x606a2b98, SUC_lexicon_tags_17 },
    { 0xcf2a2b9a, SUC_lexicon_tags_15 },
    { 0xb98a2b9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25aa2b9d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8a2ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdca2baa, SUC_lexicon_tags_45 },
    { 0x866a2bab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cea2bb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c0a2bb6, SUC_lexicon_tags_61 },
    { 0x1e2a2bb7, SUC_lexicon_tags_14 },
    { 0x8e2a2bb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2a2bbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x594a2bbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaea2bc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624a2bca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x948a2bcc, SUC_lexicon_tags_45 },
    { 0x426a2bcd, SUC_lexicon_tags_37 },
    { 0x542a2bce, SUC_lexicon_tags_14 },
    { 0x79ca2bcf, SUC_lexicon_tags_160 },
    { 0x8daa2bd0, SUC_lexicon_tags_17 },
    { 0xa32a2bd1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18a2bd5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206a2bd9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3e0a2bdb, SUC_lexicon_tags_11 },
    { 0x93ea2bdc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950a2be8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248a2bec, SUC_lexicon_tags_17 },
    { 0xe32a2bed, SUC_lexicon_tags_47 },
    { 0x504a2bee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aa2bf3, SUC_lexicon_tags_1 },
    { 0x90a2bf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6a2bf7, SUC_lexicon_tags_8 },
    { 0x898a2bf7, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ea2bfc, SUC_lexicon_tags_27 },
    { 0xd3ca2bfd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08a2c01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x174a2c04, SUC_lexicon_tags_56 },
    { 0x44ea2c05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4a2c09, SUC_lexicon_tags_26 },
    { 0x54a2c09, SUC_lexicon_tags_3 },
    { 0x30ea2c0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50a2c0e, SUC_lexicon_tags_23 },
    { 0x50ca2c0e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ca2c13, SUC_lexicon_tags_12 },
    { 0xe3aa2c14, SUC_lexicon_tags_17 },
    { 0x960a2c15, SUC_lexicon_tags_33 },
    { 0x906a2c16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ea2c19, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6a2c20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39aa2c23, SUC_lexicon_tags_993 },
    { 0x6fca2c24, SUC_lexicon_tags_14 },
    { 0xd7aa2c23, SUC_lexicon_tags_75 },
    { 0x814a2c26, SUC_lexicon_tags_25 },
    { 0x494a2c27, SUC_lexicon_tags_12 },
    { 0xf90a2c23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a2c2f, SUC_lexicon_tags_8 },
    { 0xd76a2c30, SUC_lexicon_tags_16 },
    { 0x8f6a2c31, SUC_lexicon_tags_4 },
    { 0x5daa2c32, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26a2c35, SUC_lexicon_tags_7 },
    { 0xee0a2c36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e4a2c38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5ea2c3a, SUC_lexicon_tags_38 },
    { 0xcc6a2c3b, SUC_lexicon_tags_12 },
    { 0xca4a2c3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46a2c41, SUC_lexicon_tags_65 },
    { 0x1f8a2c41, SUC_lexicon_tags_47 },
    { 0xbcca2c42, SUC_lexicon_tags_37 },
    { 0xe7ca2c42, SUC_lexicon_tags_23 },
    { 0x6f2a2c42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaca2c48, SUC_lexicon_tags_2 },
    { 0x8e6a2c48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46a2c50, SUC_lexicon_tags_23 },
    { 0xb96a2c51, SUC_lexicon_tags_47 },
    { 0xfeca2c50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174a2c56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20aa2c62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ea2c65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2daa2c6b, SUC_lexicon_tags_11 },
    { 0x3b0a2c6b, SUC_lexicon_tags_7 },
    { 0xae6a2c6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x518a2c70, SUC_lexicon_tags_8 },
    { 0x5dca2c71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08a2c78, SUC_lexicon_tags_45 },
    { 0x430a2c79, SUC_lexicon_tags_5 },
    { 0x37ea2c79, SUC_lexicon_tags_12 },
    { 0x9faa2c7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26aa2c85, SUC_lexicon_tags_17 },
    { 0x23ca2c86, SUC_lexicon_tags_37 },
    { 0x4e2a2c87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520a2c8d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0a2c92, SUC_lexicon_tags_15 },
    { 0x636a2c92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618a2c97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478a2c9c, SUC_lexicon_tags_15 },
    { 0x20aa2c9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b2a2c9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8a2ca3, SUC_lexicon_tags_37 },
    { 0x860a2ca3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8a2ca7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa28a2caa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2ca2cac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48a2cb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dca2cb7, SUC_lexicon_tags_48 },
    { 0xf30a2cb8, SUC_lexicon_tags_45 },
    { 0x9a8a2cb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd22a2cbb, SUC_lexicon_tags_17 },
    { 0x94a2cbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbca2cc0, SUC_lexicon_tags_7 },
    { 0xa3ca2cc1, SUC_lexicon_tags_12 },
    { 0xb2ca2cc2, SUC_lexicon_tags_11 },
    { 0x9c2a2cc3, SUC_lexicon_tags_11 },
    { 0xfd0a2cc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc8a2cc6, SUC_lexicon_tags_61 },
    { 0x7b8a2cc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e4a2cc9, SUC_lexicon_tags_17 },
    { 0x190a2cc9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48a2ccd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ca2cd0, SUC_lexicon_tags_63 },
    { 0x86aa2cd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ea2cd5, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2cd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43ca2cdb, SUC_lexicon_tags_2 },
    { 0x5bea2cdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe1aa2cde, SUC_lexicon_tags_7 },
    { 0x59ca2cdf, SUC_lexicon_tags_37 },
    { 0xed8a2ce0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dca2ce5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62a2cea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x454a2cec, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ea2cf1, SUC_lexicon_tags_15 },
    { 0xc32a2cf1, SUC_lexicon_tags_15 },
    { 0x7eca2cf3, SUC_lexicon_tags_47 },
    { 0xce0a2cf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ea2cfc, SUC_lexicon_tags_17 },
    { 0xf4ea2cfd, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x348a2cff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ea2d03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13aa2d07, SUC_lexicon_tags_38 },
    { 0xc14a2d08, SUC_lexicon_tags_33 },
    { 0x55aa2d09, SUC_lexicon_tags_79 },
    { 0xa7ca2d0a, SUC_lexicon_tags_17 },
    { 0x6f6a2d08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc98a2d0d, SUC_lexicon_tags_4 },
    { 0x86ca2d0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd28a2d10, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xaf2a2d12, SUC_lexicon_tags_25 },
    { 0x5eea2d13, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2a2d17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0a2d1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaa2d2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4a2d2f, SUC_lexicon_tags_38 },
    { 0x42ca2d30, SUC_lexicon_tags_25 },
    { 0x3a8a2d2f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeea2d35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75ca2d37, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x614a2d39, SUC_lexicon_tags_11 },
    { 0x82ca2d3a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ea2d3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84a2d42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aa2d45, SUC_lexicon_tags_68 },
    { 0xdb0a2d46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6a2d49, SUC_lexicon_tags_1 },
    { 0xa80a2d4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddea2d4f, SUC_lexicon_tags_33 },
    { 0x574a2d4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190a2d54, SUC_lexicon_tags_0 },
    { 0xef6a2d55, SUC_lexicon_tags_8 },
    { 0x1e2a2d55, SUC_lexicon_tags_27 },
    { 0x442a2d54, SUC_lexicon_tags_37 },
    { 0xb66a2d58, SUC_lexicon_tags_37 },
    { 0x36ea2d59, SUC_lexicon_tags_47 },
    { 0xf3ea2d5a, SUC_lexicon_tags_39 },
    { 0x46ca2d58, SUC_lexicon_tags_37 },
    { 0x684a2d59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7aaa2d5e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ca2d61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0aa2d63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x94a2d65, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ea2d68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daa2d70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x352a2d72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fea2d74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc74a2d76, SUC_lexicon_tags_45 },
    { 0xaeca2d77, SUC_lexicon_tags_17 },
    { 0x104a2d78, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaa2d7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86a2d7f, SUC_lexicon_tags_220 },
    { 0xff2a2d80, SUC_lexicon_tags_37 },
    { 0xbcea2d81, SUC_lexicon_tags_38 },
    { 0x648a2d82, SUC_lexicon_tags_60 },
    { 0xb48a2d83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb50a2d85, SUC_lexicon_tags_32 },
    { 0x4c8a2d86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792a2d8f, SUC_lexicon_tags_29 },
    { 0xe52a2d90, SUC_lexicon_tags_10 },
    { 0xf06a2d91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6a2d94, SUC_lexicon_tags_38 },
    { 0x632a2d95, SUC_lexicon_tags_23 },
    { 0x38aa2d96, SUC_lexicon_tags_27 },
    { 0x570a2d97, SUC_lexicon_tags_38 },
    { 0x190a2d97, SUC_lexicon_tags_1 },
    { 0x73ca2d99, SUC_lexicon_tags_25 },
    { 0xa12a2d9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2a2da0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x590a2da3, SUC_lexicon_tags_37 },
    { 0xfd2a2da4, SUC_lexicon_tags_66 },
    { 0xd08a2da5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbfea2da7, SUC_lexicon_tags_37 },
    { 0x334a2da8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4e6a2daa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbca2dad, SUC_lexicon_tags_37 },
    { 0x9a2a2dae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6ea2db0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x372a2db2, SUC_lexicon_tags_13 },
    { 0xfd6a2db3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x926a2db6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x606a2db9, SUC_lexicon_tags_47 },
    { 0x5a2a2db9, SUC_lexicon_tags_34 },
    { 0x25ea2dbb, SUC_lexicon_tags_15 },
    { 0x8e2a2dbc, SUC_lexicon_tags_27 },
    { 0x2e6a2dbd, SUC_lexicon_tags_11 },
    { 0xd3ca2dbe, SUC_lexicon_tags_17 },
    { 0x31ea2db9, SUC_lexicon_tags_11 },
    { 0xdbca2dc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa70a2dc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ca2dca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7aa2dcc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1e0a2dce, SUC_lexicon_tags_6 },
    { 0x6f2a2dce, SUC_lexicon_tags_31 },
    { 0xe58a2dd0, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744a2dda, SUC_lexicon_tags_17 },
    { 0xa0ca2ddb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42a2de2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeea2de5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x110a2de8, SUC_lexicon_tags_33 },
    { 0x9eca2de8, SUC_lexicon_tags_26 },
    { 0xc8a2de8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x148a2ded, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20a2df1, SUC_lexicon_tags_26 },
    { 0x28ca2df1, SUC_lexicon_tags_29 },
    { 0x60ca2df2, SUC_lexicon_tags_28 },
    { 0xc9aa2df2, SUC_lexicon_tags_17 },
    { 0x48aa2df3, SUC_lexicon_tags_65 },
    { 0xbaea2df2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2a2df9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19aa2e03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe70a2e05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x76a2e08, SUC_lexicon_tags_2 },
    { 0xfbca2e08, SUC_lexicon_tags_33 },
    { 0x9cca2e08, SUC_lexicon_tags_17 },
    { 0xa10a2e0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ea2e11, SUC_lexicon_tags_25 },
    { 0xa2ea2e11, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ea2e18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x124a2e1a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1aea2e1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0a2e1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24a2e23, SUC_lexicon_tags_60 },
    { 0xea4a2e24, SUC_lexicon_tags_23 },
    { 0xcbea2e24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc74a2e27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540a2e2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ca2e30, SUC_lexicon_tags_56 },
    { 0xda2a2e31, SUC_lexicon_tags_13 },
    { 0x1cca2e30, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd58a2e34, SUC_lexicon_tags_32 },
    { 0xfdaa2e34, SUC_lexicon_tags_15 },
    { 0x14a2e36, SUC_lexicon_tags_11 },
    { 0x266a2e35, SUC_lexicon_tags_17 },
    { 0x21ea2e38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeeaa2e3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a2e40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e8a2e42, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x842a2e44, SUC_lexicon_tags_103 },
    { 0x9bca2e45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560a2e49, SUC_lexicon_tags_37 },
    { 0x1c4a2e4a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844a2e50, SUC_lexicon_tags_45 },
    { 0x254a2e51, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x120a2e54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8a2e58, SUC_lexicon_tags_1 },
    { 0xde4a2e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x342a2e5b, SUC_lexicon_tags_37 },
    { 0x95aa2e5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2fea2e5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31ca2e60, SUC_lexicon_tags_43 },
    { 0xa1ea2e61, SUC_lexicon_tags_15 },
    { 0xeb8a2e60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ca2e65, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ea2e68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb42a2e6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aa2e75, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706a2e7b, SUC_lexicon_tags_56 },
    { 0x6c4a2e7c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xb9ca2e7e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xedaa2e80, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8a2e83, SUC_lexicon_tags_66 },
    { 0x96ca2e83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f6a2e86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70a2e89, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x562a2e8c, SUC_lexicon_tags_35 },
    { 0xa24a2e8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb8a2e8f, SUC_lexicon_tags_19 },
    { 0xbd8a2e90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeca2e92, SUC_lexicon_tags_25 },
    { 0x7f6a2e92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8a2e9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9ea2e9c, SUC_lexicon_tags_38 },
    { 0x74ea2e9c, SUC_lexicon_tags_49 },
    { 0xbb8a2e9c, SUC_lexicon_tags_1 },
    { 0x6aea2e9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04a2ea3, SUC_lexicon_tags_17 },
    { 0x7eaa2ea4, SUC_lexicon_tags_15 },
    { 0x42a2ea4, SUC_lexicon_tags_37 },
    { 0xa18a2ea3, SUC_lexicon_tags_25 },
    { 0x496a2ea6, SUC_lexicon_tags_25 },
    { 0x4a6a2ea5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x822a2eaa, SUC_lexicon_tags_96 },
    { 0x474a2eab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10ea2ead, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6a2eb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296a2eb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a2ebb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2a2ebf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7b2a2ec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8a2ec4, SUC_lexicon_tags_33 },
    { 0xee6a2ec5, SUC_lexicon_tags_25 },
    { 0xdf4a2ec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2a2ecd, SUC_lexicon_tags_12 },
    { 0xfbea2ece, SUC_lexicon_tags_56 },
    { 0xe6aa2ecf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea2ed6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28ca2ed8, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ea2edb, SUC_lexicon_tags_7 },
    { 0x404a2edc, SUC_lexicon_tags_11 },
    { 0xe54a2edc, SUC_lexicon_tags_9 },
    { 0xc34a2ede, SUC_lexicon_tags_14 },
    { 0x93ea2ede, SUC_lexicon_tags_10 },
    { 0x6c2a2ee0, SUC_lexicon_tags_11 },
    { 0xf3aa2ee1, SUC_lexicon_tags_29 },
    { 0x502a2ee1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaea2ee9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aa2eec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28a2ef1, SUC_lexicon_tags_83 },
    { 0x814a2ef1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240a2ef7, SUC_lexicon_tags_57 },
    { 0x5a8a2ef8, SUC_lexicon_tags_33 },
    { 0xe8aa2ef8, SUC_lexicon_tags_47 },
    { 0x26aa2efa, SUC_lexicon_tags_17 },
    { 0x90ea2ef7, SUC_lexicon_tags_38 },
    { 0x306a2efc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x144a2efe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64a2f02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd72a2f04, SUC_lexicon_tags_29 },
    { 0x17aa2f05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa2f0c, SUC_lexicon_tags_47 },
    { 0xa9ca2f0d, SUC_lexicon_tags_27 },
    { 0x36aa2f0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc6a2f10, SUC_lexicon_tags_38 },
    { 0x6eca2f10, SUC_lexicon_tags_42 },
    { 0xa22a2f11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228a2f16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc74a2f18, SUC_lexicon_tags_31 },
    { 0x81aa2f19, SUC_lexicon_tags_25 },
    { 0xb0ca2f1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ea2f1d, SUC_lexicon_tags_6 },
    { 0xca4a2f1e, SUC_lexicon_tags_32 },
    { 0xb22a2f1f, SUC_lexicon_tags_12 },
    { 0xc10a2f20, SUC_lexicon_tags_17 },
    { 0x9d6a2f21, SUC_lexicon_tags_31 },
    { 0xda8a2f22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce2a2f24, SUC_lexicon_tags_12 },
    { 0x4dea2f25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ea2f28, SUC_lexicon_tags_1 },
    { 0x9c6a2f29, SUC_lexicon_tags_18 },
    { 0x7dea2f2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x344a2f2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88a2f31, SUC_lexicon_tags_8 },
    { 0x780a2f31, SUC_lexicon_tags_43 },
    { 0x75aa2f33, SUC_lexicon_tags_25 },
    { 0x6aca2f33, SUC_lexicon_tags_45 },
    { 0xe40a2f35, SUC_lexicon_tags_55 },
    { 0xb9ea2f36, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5daa2f3b, SUC_lexicon_tags_65 },
    { 0xbd0a2f3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x628a2f3e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ea2f41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ca2f44, SUC_lexicon_tags_16 },
    { 0xc6ca2f45, SUC_lexicon_tags_1188 },
    { 0xc02a2f46, SUC_lexicon_tags_47 },
    { 0x23aa2f47, SUC_lexicon_tags_6 },
    { 0x8eaa2f48, SUC_lexicon_tags_9 },
    { 0xc2ca2f48, SUC_lexicon_tags_2 },
    { 0xbd2a2f47, SUC_lexicon_tags_11 },
    { 0xff6a2f4b, SUC_lexicon_tags_6 },
    { 0x972a2f4a, SUC_lexicon_tags_43 },
    { 0x3b0a2f48, SUC_lexicon_tags_14 },
    { 0xc92a2f46, SUC_lexicon_tags_11 },
    { 0x16ea2f4f, SUC_lexicon_tags_30 },
    { 0x992a2f4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4a2f57, SUC_lexicon_tags_14 },
    { 0xc68a2f58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadea2f5f, SUC_lexicon_tags_27 },
    { 0xbc0a2f60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aa2f64, SUC_lexicon_tags_37 },
    { 0x9c2a2f65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdca2f6a, SUC_lexicon_tags_38 },
    { 0xa9ea2f6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116a2f6f, SUC_lexicon_tags_8 },
    { 0x6e0a2f6f, SUC_lexicon_tags_12 },
    { 0xc1ca2f70, SUC_lexicon_tags_83 },
    { 0xc28a2f72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970a2f77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde6a2f79, SUC_lexicon_tags_77 },
    { 0x422a2f7a, SUC_lexicon_tags_9 },
    { 0x322a2f7b, SUC_lexicon_tags_37 },
    { 0xb28a2f79, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x27aa2f7e, SUC_lexicon_tags_25 },
    { 0x45aa2f7f, SUC_lexicon_tags_47 },
    { 0x30ea2f80, SUC_lexicon_tags_37 },
    { 0x94ea2f7e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x9c2a2f83, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344a2f8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ca2f92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0a2f95, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4a2f99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8a2fa0, SUC_lexicon_tags_15 },
    { 0x678a2fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaa2fa4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x864a2fa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2a2fa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x66aa2fab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fca2fb0, SUC_lexicon_tags_12 },
    { 0x3a4a2fb1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf0ca2fb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6a2fb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2a8a2fb9, SUC_lexicon_tags_1 },
    { 0x798a2fb9, SUC_lexicon_tags_29 },
    { 0x6ca2fbb, SUC_lexicon_tags_61 },
    { 0xddca2fbc, SUC_lexicon_tags_0 },
    { 0xedca2fb9, SUC_lexicon_tags_60 },
    { 0x3e8a2fbe, SUC_lexicon_tags_45 },
    { 0xbc8a2fb9, SUC_lexicon_tags_14 },
    { 0x294a2fc0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194a2fca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444a2fd0, SUC_lexicon_tags_17 },
    { 0x50ca2fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cca2fd6, SUC_lexicon_tags_32 },
    { 0x6e2a2fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ea2fdc, SUC_lexicon_tags_17 },
    { 0xf80a2fdd, SUC_lexicon_tags_64 },
    { 0x5fca2fdc, SUC_lexicon_tags_17 },
    { 0xd4aa2fdf, SUC_lexicon_tags_14 },
    { 0x852a2fdd, SUC_lexicon_tags_12 },
    { 0xa28a2fdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa2fe5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x34aa2fe7, SUC_lexicon_tags_60 },
    { 0xf68a2fe8, SUC_lexicon_tags_2 },
    { 0xa48a2fe9, SUC_lexicon_tags_11 },
    { 0x4b0a2fea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x838a2fec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x210a2fef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6a2ff6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x360a2ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b6a2ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186a2fff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4eaa3001, SUC_lexicon_tags_16 },
    { 0x56ca3002, SUC_lexicon_tags_0 },
    { 0x530a3003, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ea3006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fca300d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa88a300f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1aca3011, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x340a3013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12a3018, SUC_lexicon_tags_11 },
    { 0x5b6a3019, SUC_lexicon_tags_47 },
    { 0x470a3018, SUC_lexicon_tags_37 },
    { 0xd02a301b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ea301f, SUC_lexicon_tags_23 },
    { 0x7d4a3020, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eca3027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ea302d, SUC_lexicon_tags_47 },
    { 0x178a302e, SUC_lexicon_tags_37 },
    { 0xe38a302e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08a3032, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ea3038, SUC_lexicon_tags_70 },
    { 0xec2a3039, SUC_lexicon_tags_39 },
    { 0xca3039, SUC_lexicon_tags_45 },
    { 0x4aaa303b, SUC_lexicon_tags_32 },
    { 0x67aa303b, SUC_lexicon_tags_16 },
    { 0x874a303c, SUC_lexicon_tags_1 },
    { 0xfeaa303e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x920a3040, SUC_lexicon_tags_12 },
    { 0x46ea3041, SUC_lexicon_tags_1 },
    { 0x95ca3042, SUC_lexicon_tags_41 },
    { 0xb9ca3043, SUC_lexicon_tags_17 },
    { 0xca8a3040, SUC_lexicon_tags_43 },
    { 0x456a3045, SUC_lexicon_tags_1 },
    { 0x58ea3046, SUC_lexicon_tags_37 },
    { 0x2f2a3047, SUC_lexicon_tags_17 },
    { 0x3a2a3048, SUC_lexicon_tags_31 },
    { 0x616a3047, SUC_lexicon_tags_0 },
    { 0x126a3049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcaa3051, SUC_lexicon_tags_10 },
    { 0x410a3052, SUC_lexicon_tags_38 },
    { 0x87ea3053, SUC_lexicon_tags_12 },
    { 0x74aa3052, SUC_lexicon_tags_61 },
    { 0xb42a3052, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x298a3058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4a305c, SUC_lexicon_tags_102 },
    { 0x6f2a305d, SUC_lexicon_tags_25 },
    { 0xd28a305c, SUC_lexicon_tags_49 },
    { 0xc84a305e, SUC_lexicon_tags_37 },
    { 0x3d0a3060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x660a3063, SUC_lexicon_tags_12 },
    { 0x890a3063, SUC_lexicon_tags_23 },
    { 0x34aa3065, SUC_lexicon_tags_17 },
    { 0x950a3065, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02a306a, SUC_lexicon_tags_17 },
    { 0x464a306a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x998a306d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4a3075, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x748a3078, SUC_lexicon_tags_49 },
    { 0x57ca3078, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38ea307b, SUC_lexicon_tags_19 },
    { 0xa7ca307c, SUC_lexicon_tags_27 },
    { 0xd48a307c, SUC_lexicon_tags_38 },
    { 0x10ca307c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950a3084, SUC_lexicon_tags_33 },
    { 0x632a3085, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87ea3087, SUC_lexicon_tags_0 },
    { 0xdf0a3087, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x970a308a, SUC_lexicon_tags_11 },
    { 0x442a308a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x928a308d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92a30a1, SUC_lexicon_tags_11 },
    { 0xf0ea30a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x972a30a4, SUC_lexicon_tags_27 },
    { 0x462a30a4, SUC_lexicon_tags_21 },
    { 0xedea30a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2a30ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42aa30ad, SUC_lexicon_tags_6 },
    { 0x33ea30ae, SUC_lexicon_tags_1 },
    { 0x172a30ae, SUC_lexicon_tags_171 },
    { 0xfeca30b0, SUC_lexicon_tags_37 },
    { 0x56ea30af, SUC_lexicon_tags_15 },
    { 0xd8ca30b2, SUC_lexicon_tags_48 },
    { 0xc8ea30b1, SUC_lexicon_tags_27 },
    { 0x6f6a30b3, SUC_lexicon_tags_37 },
    { 0xe28a30ad, SUC_lexicon_tags_63 },
    { 0x872a30b6, SUC_lexicon_tags_21 },
    { 0xae0a30b7, SUC_lexicon_tags_3 },
    { 0x964a30b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacea30bc, SUC_lexicon_tags_23 },
    { 0x32a30bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ca30c0, SUC_lexicon_tags_3 },
    { 0xc26a30c1, SUC_lexicon_tags_13 },
    { 0x128a30c0, SUC_lexicon_tags_230 },
    { 0x4cca30c0, SUC_lexicon_tags_46 },
    { 0x89ca30c0, SUC_lexicon_tags_70 },
    { 0x8fca30c5, SUC_lexicon_tags_62 },
    { 0xdd2a30c6, SUC_lexicon_tags_69 },
    { 0x94ca30c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a30d3, SUC_lexicon_tags_0 },
    { 0xa4ea30d4, SUC_lexicon_tags_34 },
    { 0x81ca30d3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa6ea30d7, SUC_lexicon_tags_17 },
    { 0x46a30d7, SUC_lexicon_tags_0 },
    { 0x3b8a30d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ea30e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622a30e4, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6a30e7, SUC_lexicon_tags_29 },
    { 0xd1ea30e8, SUC_lexicon_tags_33 },
    { 0x444a30e8, SUC_lexicon_tags_13 },
    { 0x67ca30ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x770a30ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2a30f2, SUC_lexicon_tags_15 },
    { 0xde6a30f2, SUC_lexicon_tags_7 },
    { 0x1daa30f4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb82a30f6, SUC_lexicon_tags_37 },
    { 0xc38a30f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfeea30f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512a30fe, SUC_lexicon_tags_63 },
    { 0xd5aa30ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd34a3101, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ca310a, SUC_lexicon_tags_33 },
    { 0xdd8a310b, SUC_lexicon_tags_25 },
    { 0xeaa310c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x600a310e, SUC_lexicon_tags_33 },
    { 0xd8aa310f, SUC_lexicon_tags_37 },
    { 0x3faa310f, SUC_lexicon_tags_10 },
    { 0x780a3111, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58a3115, SUC_lexicon_tags_65 },
    { 0xbcca3116, SUC_lexicon_tags_0 },
    { 0x390a3117, SUC_lexicon_tags_0 },
    { 0xec6a3117, SUC_lexicon_tags_38 },
    { 0xcf2a3117, SUC_lexicon_tags_6 },
    { 0x300a3115, SUC_lexicon_tags_7 },
    { 0x81ea3117, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8ea311d, SUC_lexicon_tags_15 },
    { 0xa78a311e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59ea3120, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf04a3122, SUC_lexicon_tags_31 },
    { 0x524a3123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16a3129, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4a312e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0a3133, SUC_lexicon_tags_33 },
    { 0x88ea3134, SUC_lexicon_tags_27 },
    { 0x4d4a3135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76a3138, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ca313b, SUC_lexicon_tags_81 },
    { 0xff6a313c, SUC_lexicon_tags_38 },
    { 0x358a313d, SUC_lexicon_tags_10 },
    { 0xf72a313c, SUC_lexicon_tags_23 },
    { 0x8a2a313f, SUC_lexicon_tags_15 },
    { 0x3aca3140, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x232a3142, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58a3145, SUC_lexicon_tags_25 },
    { 0xe7aa3146, SUC_lexicon_tags_47 },
    { 0xfbaa3145, SUC_lexicon_tags_19 },
    { 0x8e4a3148, SUC_lexicon_tags_98 },
    { 0x5f6a3149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52a314b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ea3150, SUC_lexicon_tags_13 },
    { 0x9c4a3150, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa3154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18a3158, SUC_lexicon_tags_61 },
    { 0x2b0a3158, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28a3160, SUC_lexicon_tags_29 },
    { 0x9aca3160, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fea3164, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x86aa3167, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212a316f, SUC_lexicon_tags_25 },
    { 0x712a3170, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x594a3173, SUC_lexicon_tags_47 },
    { 0x2e4a3174, SUC_lexicon_tags_47 },
    { 0xc72a3175, SUC_lexicon_tags_65 },
    { 0x650a3173, SUC_lexicon_tags_14 },
    { 0x488a3174, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a317a, SUC_lexicon_tags_48 },
    { 0x3eca317a, SUC_lexicon_tags_12 },
    { 0xbaea317c, SUC_lexicon_tags_14 },
    { 0xc12a317c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa317f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x48ea3181, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ca3186, SUC_lexicon_tags_1 },
    { 0x4f4a3187, SUC_lexicon_tags_12 },
    { 0x8e2a3186, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6a318b, SUC_lexicon_tags_37 },
    { 0xdb6a318c, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90a3193, SUC_lexicon_tags_17 },
    { 0xdfea3193, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x476a3196, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0a3199, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcca319d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x33ca319f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2a31a3, SUC_lexicon_tags_30 },
    { 0x58aa31a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42a31a7, SUC_lexicon_tags_15 },
    { 0x4e6a31a8, SUC_lexicon_tags_37 },
    { 0xe28a31a9, SUC_lexicon_tags_14 },
    { 0xbcca31a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dca31b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2a31b6, SUC_lexicon_tags_14 },
    { 0x510a31b7, SUC_lexicon_tags_25 },
    { 0xceaa31b6, SUC_lexicon_tags_17 },
    { 0x7a2a31b6, SUC_lexicon_tags_37 },
    { 0xfb6a31ba, SUC_lexicon_tags_53 },
    { 0x386a31ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23aa31bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89ca31c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484a31c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x142a31c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2a31cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8a31d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6a31d9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8a31dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baa31e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88a31e7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ca31ef, SUC_lexicon_tags_49 },
    { 0xd04a31f0, SUC_lexicon_tags_33 },
    { 0xbb8a31f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8a31f4, SUC_lexicon_tags_1 },
    { 0xc28a31f5, SUC_lexicon_tags_7 },
    { 0xaa6a31f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448a31fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x388a31fc, SUC_lexicon_tags_17 },
    { 0xe1ca31fd, SUC_lexicon_tags_17 },
    { 0x136a31fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaa3201, SUC_lexicon_tags_66 },
    { 0xf76a3201, SUC_lexicon_tags_38 },
    { 0x966a3203, SUC_lexicon_tags_37 },
    { 0xbc2a3204, SUC_lexicon_tags_23 },
    { 0xed2a3204, SUC_lexicon_tags_7 },
    { 0x7d6a3206, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4a320d, SUC_lexicon_tags_12 },
    { 0x5b6a320e, SUC_lexicon_tags_68 },
    { 0x6cea320e, SUC_lexicon_tags_12 },
    { 0x98ca3210, SUC_lexicon_tags_87 },
    { 0x230a3211, SUC_lexicon_tags_23 },
    { 0xbc4a3212, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748a3219, SUC_lexicon_tags_46 },
    { 0x97ca321a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaaca321c, SUC_lexicon_tags_7 },
    { 0x79ea321d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ea3220, SUC_lexicon_tags_6 },
    { 0x6eea3221, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826a3225, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xff6a3227, SUC_lexicon_tags_39 },
    { 0xceea3228, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x672a322a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ea322d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf2a322f, SUC_lexicon_tags_12 },
    { 0xc56a3230, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38a3233, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6a3236, SUC_lexicon_tags_14 },
    { 0x770a3237, SUC_lexicon_tags_6 },
    { 0x8b8a3238, SUC_lexicon_tags_25 },
    { 0x8cca3239, SUC_lexicon_tags_17 },
    { 0x7b4a3236, SUC_lexicon_tags_7 },
    { 0x2e0a3238, SUC_lexicon_tags_1 },
    { 0xf5ca3237, SUC_lexicon_tags_11 },
    { 0x98aa323d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe0a323f, SUC_lexicon_tags_27 },
    { 0xe58a323f, SUC_lexicon_tags_17 },
    { 0xe28a3241, SUC_lexicon_tags_17 },
    { 0x990a3242, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ca3245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf22a3247, SUC_lexicon_tags_0 },
    { 0xa7ea3248, SUC_lexicon_tags_91 },
    { 0x38aa3248, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aea324c, SUC_lexicon_tags_21 },
    { 0x824a324c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x276a324f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48a3253, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf70a3255, SUC_lexicon_tags_17 },
    { 0x6d2a3255, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8a3259, SUC_lexicon_tags_37 },
    { 0x5f0a3259, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bea325e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88a3261, SUC_lexicon_tags_29 },
    { 0x24ea3262, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200a3267, SUC_lexicon_tags_1 },
    { 0x8a4a3268, SUC_lexicon_tags_9 },
    { 0x5bea3269, SUC_lexicon_tags_27 },
    { 0x8ca3269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbeca326c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38aa326e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2a3273, SUC_lexicon_tags_33 },
    { 0x376a3274, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae2a3276, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216a327a, SUC_lexicon_tags_31 },
    { 0xe94a327b, SUC_lexicon_tags_49 },
    { 0xfb0a327b, SUC_lexicon_tags_15 },
    { 0x170a327d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ca3280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedea3286, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xab2a3288, SUC_lexicon_tags_57 },
    { 0xeaea3288, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ca3290, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44a3293, SUC_lexicon_tags_17 },
    { 0x81ca3293, SUC_lexicon_tags_60 },
    { 0x9e2a3295, SUC_lexicon_tags_23 },
    { 0xd14a3295, SUC_lexicon_tags_7 },
    { 0xea8a3297, SUC_lexicon_tags_29 },
    { 0x7d2a3297, SUC_lexicon_tags_13 },
    { 0x442a3298, SUC_lexicon_tags_46 },
    { 0x94ca329a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc0aa329c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0a329f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4b8a32a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefea32a3, SUC_lexicon_tags_2 },
    { 0xf1aa32a4, SUC_lexicon_tags_23 },
    { 0x18a32a5, SUC_lexicon_tags_15 },
    { 0xf58a32a5, SUC_lexicon_tags_39 },
    { 0x88a32a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a32aa, SUC_lexicon_tags_31 },
    { 0x71ca32ab, SUC_lexicon_tags_14 },
    { 0x30aa32ac, SUC_lexicon_tags_1 },
    { 0xf36a32ac, SUC_lexicon_tags_63 },
    { 0x1f2a32ad, SUC_lexicon_tags_21 },
    { 0x3b6a32ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4a32b2, SUC_lexicon_tags_25 },
    { 0x6c2a32b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd3ca32b5, SUC_lexicon_tags_12 },
    { 0x9a6a32b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742a32bf, SUC_lexicon_tags_17 },
    { 0x63ca32c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ca32c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6a32ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9aa32d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9eea32d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8a32d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0a32d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6faa32db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76a32de, SUC_lexicon_tags_2 },
    { 0xaaa32df, SUC_lexicon_tags_11 },
    { 0x308a32e0, SUC_lexicon_tags_38 },
    { 0xe9aa32e1, SUC_lexicon_tags_45 },
    { 0x18ea32e2, SUC_lexicon_tags_26 },
    { 0x7ca32e3, SUC_lexicon_tags_87 },
    { 0xd84a32e4, SUC_lexicon_tags_87 },
    { 0xdb2a32e3, SUC_lexicon_tags_17 },
    { 0xb5ca32e6, SUC_lexicon_tags_14 },
    { 0xc9ea32e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aa32ec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x78aa32ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf50a32f0, SUC_lexicon_tags_1 },
    { 0xe6ca32f1, SUC_lexicon_tags_10 },
    { 0x920a32f2, SUC_lexicon_tags_12 },
    { 0xabea32f1, SUC_lexicon_tags_19 },
    { 0x906a32f2, SUC_lexicon_tags_15 },
    { 0x31ea32f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf28a32f7, SUC_lexicon_tags_87 },
    { 0x4c0a32f8, SUC_lexicon_tags_17 },
    { 0xb84a32f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c2a32fb, SUC_lexicon_tags_55 },
    { 0x726a32fc, SUC_lexicon_tags_12 },
    { 0xaf2a32fd, SUC_lexicon_tags_11 },
    { 0x310a32fe, SUC_lexicon_tags_7 },
    { 0xc4a32ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x654a3301, SUC_lexicon_tags_12 },
    { 0x94a3302, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a3309, SUC_lexicon_tags_56 },
    { 0xf78a330a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bea330c, SUC_lexicon_tags_47 },
    { 0xfeaa330d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x464a3310, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404a3315, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ea3318, SUC_lexicon_tags_14 },
    { 0x590a3319, SUC_lexicon_tags_55 },
    { 0xb80a3319, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47aa331c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6a3321, SUC_lexicon_tags_15 },
    { 0x4dca3322, SUC_lexicon_tags_12 },
    { 0x916a3323, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf8a3325, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4a332a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eea332d, SUC_lexicon_tags_12 },
    { 0x9a0a332e, SUC_lexicon_tags_23 },
    { 0xe56a332d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaa3333, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60a3336, SUC_lexicon_tags_2 },
    { 0x490a3336, SUC_lexicon_tags_26 },
    { 0x3b0a3338, SUC_lexicon_tags_17 },
    { 0xdd2a3339, SUC_lexicon_tags_1 },
    { 0x6c2a3339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276a333f, SUC_lexicon_tags_11 },
    { 0x4b2a3340, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3ca3342, SUC_lexicon_tags_25 },
    { 0x370a3342, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ea3347, SUC_lexicon_tags_8 },
    { 0x4c8a3348, SUC_lexicon_tags_60 },
    { 0x112a3349, SUC_lexicon_tags_28 },
    { 0x7a6a3349, SUC_lexicon_tags_31 },
    { 0x834a334a, SUC_lexicon_tags_30 },
    { 0x38aa334c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4a3353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ca3356, SUC_lexicon_tags_2 },
    { 0x24ea3356, SUC_lexicon_tags_2 },
    { 0xb6ea3358, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63ca335a, SUC_lexicon_tags_2 },
    { 0x7dca335b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114a3364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32a3367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6aa3369, SUC_lexicon_tags_11 },
    { 0x47ea336a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6a3371, SUC_lexicon_tags_59 },
    { 0x1ea3372, SUC_lexicon_tags_38 },
    { 0x726a3371, SUC_lexicon_tags_6 },
    { 0x156a3374, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0a337c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafca337e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bca3381, SUC_lexicon_tags_28 },
    { 0xb38a3382, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x59aa3384, SUC_lexicon_tags_15 },
    { 0x944a3385, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6a338a, SUC_lexicon_tags_27 },
    { 0x39aa338b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94a3392, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x380a3395, SUC_lexicon_tags_12 },
    { 0x4ca3396, SUC_lexicon_tags_43 },
    { 0xb22a3397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192a33a4, SUC_lexicon_tags_66 },
    { 0x322a33a5, SUC_lexicon_tags_17 },
    { 0x17ca33a6, SUC_lexicon_tags_2 },
    { 0x6aaa33a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94a33ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02a33af, SUC_lexicon_tags_63 },
    { 0xe08a33af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1ea33b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aa33b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9aa33b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaa33bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87ca33c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x488a33c3, SUC_lexicon_tags_13 },
    { 0x5bca33c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10a33c8, SUC_lexicon_tags_12 },
    { 0x396a33c8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x71ea33cb, SUC_lexicon_tags_15 },
    { 0xb28a33cc, SUC_lexicon_tags_17 },
    { 0xaa0a33cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x696a33d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16a33d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x274a33d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ca33dd, SUC_lexicon_tags_12 },
    { 0x1e6a33de, SUC_lexicon_tags_1 },
    { 0xcea33dd, SUC_lexicon_tags_38 },
    { 0x592a33de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd0a33e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a33e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22a33e8, SUC_lexicon_tags_37 },
    { 0x180a33e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6a33f3, SUC_lexicon_tags_23 },
    { 0x222a33f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddca33f9, SUC_lexicon_tags_27 },
    { 0x4a2a33fa, SUC_lexicon_tags_33 },
    { 0x9b6a33fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4faa3401, SUC_lexicon_tags_19 },
    { 0xfe8a3401, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4faa3409, SUC_lexicon_tags_0 },
    { 0xaf8a3409, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x258a340c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fca3410, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342a3416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80a3419, SUC_lexicon_tags_45 },
    { 0x5c4a341a, SUC_lexicon_tags_38 },
    { 0xdeca341b, SUC_lexicon_tags_62 },
    { 0xd5ea3419, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2a341f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ca3424, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120a3429, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ca342e, SUC_lexicon_tags_35 },
    { 0xc16a342f, SUC_lexicon_tags_47 },
    { 0xbc2a3430, SUC_lexicon_tags_23 },
    { 0xf54a342f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230a3435, SUC_lexicon_tags_9 },
    { 0xb16a3436, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960a343b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96a343e, SUC_lexicon_tags_46 },
    { 0x570a343f, SUC_lexicon_tags_64 },
    { 0x626a3440, SUC_lexicon_tags_1 },
    { 0x728a3440, SUC_lexicon_tags_26 },
    { 0xe34a3442, SUC_lexicon_tags_9 },
    { 0xca4a3441, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfc0a3445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8a344f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49ea3451, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1aa3454, SUC_lexicon_tags_33 },
    { 0xa3ca3455, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6a3458, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00a345b, SUC_lexicon_tags_59 },
    { 0x2f8a345b, SUC_lexicon_tags_1 },
    { 0x7e2a345c, SUC_lexicon_tags_253 },
    { 0xf6a345e, SUC_lexicon_tags_32 },
    { 0x392a345f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59aa3465, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb54a3467, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc22a3469, SUC_lexicon_tags_77 },
    { 0x994a346a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d0a346c, SUC_lexicon_tags_25 },
    { 0xe58a346c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4a3472, SUC_lexicon_tags_10 },
    { 0x84aa3473, SUC_lexicon_tags_28 },
    { 0x27ea3474, SUC_lexicon_tags_165 },
    { 0xb36a3475, SUC_lexicon_tags_2 },
    { 0x60a3473, SUC_lexicon_tags_15 },
    { 0x91ca3473, SUC_lexicon_tags_33 },
    { 0x93aa3478, SUC_lexicon_tags_19 },
    { 0xb9aa3474, SUC_lexicon_tags_17 },
    { 0x65ea347a, SUC_lexicon_tags_58 },
    { 0x64aa347b, SUC_lexicon_tags_57 },
    { 0x50a347c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdca3480, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36a3488, SUC_lexicon_tags_47 },
    { 0x912a3488, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x41ea348b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6a3493, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x24aa3495, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a349b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274a34a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43aa34a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad2a34a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ca34a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdca34ae, SUC_lexicon_tags_1645 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ea34b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78a34b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00a34ba, SUC_lexicon_tags_6 },
    { 0xa3ca34bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faa34c5, SUC_lexicon_tags_61 },
    { 0x15ca34c5, SUC_lexicon_tags_15 },
    { 0x3a8a34c7, SUC_lexicon_tags_27 },
    { 0x16ca34c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe56a34ca, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xbc8a34cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1dea34ce, SUC_lexicon_tags_7 },
    { 0x982a34cf, SUC_lexicon_tags_13 },
    { 0xf26a34cf, SUC_lexicon_tags_17 },
    { 0xe3ca34cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ea34d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x306a34d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00a34d9, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0x58a34dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8aa34de, SUC_lexicon_tags_15 },
    { 0xba6a34de, SUC_lexicon_tags_25 },
    { 0x9b2a34df, SUC_lexicon_tags_31 },
    { 0x292a34e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x744a34e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1daa34e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944a34ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02a34f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x126a34f5, SUC_lexicon_tags_19 },
    { 0xe74a34f5, SUC_lexicon_tags_12 },
    { 0x138a34f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354a34fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0a3504, SUC_lexicon_tags_60 },
    { 0xdf0a3504, SUC_lexicon_tags_11 },
    { 0xffca3506, SUC_lexicon_tags_12 },
    { 0x30a3505, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9dca3509, SUC_lexicon_tags_6 },
    { 0xe4a350a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ca350d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ca3511, SUC_lexicon_tags_25 },
    { 0x3a6a3512, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x176a3514, SUC_lexicon_tags_17 },
    { 0xec6a3515, SUC_lexicon_tags_40 },
    { 0x6caa3516, SUC_lexicon_tags_26 },
    { 0x108a3517, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44aa351a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13ea351c, SUC_lexicon_tags_25 },
    { 0x3a4a351d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ca3520, SUC_lexicon_tags_64 },
    { 0x104a3521, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x920a3523, SUC_lexicon_tags_55 },
    { 0xad4a3524, SUC_lexicon_tags_7 },
    { 0x632a3524, SUC_lexicon_tags_17 },
    { 0xb64a3524, SUC_lexicon_tags_25 },
    { 0x9a2a3524, SUC_lexicon_tags_57 },
    { 0x6eca3528, SUC_lexicon_tags_100 },
    { 0x3a6a3525, SUC_lexicon_tags_7 },
    { 0xf8ca352a, SUC_lexicon_tags_53 },
    { 0x5dea352b, SUC_lexicon_tags_25 },
    { 0x356a352c, SUC_lexicon_tags_8 },
    { 0x612a352d, SUC_lexicon_tags_17 },
    { 0x4a6a352e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ea3531, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ea3536, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2a353b, SUC_lexicon_tags_7 },
    { 0x82aa353c, SUC_lexicon_tags_69 },
    { 0xcf0a353d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbfca353f, SUC_lexicon_tags_13 },
    { 0x7f4a3540, SUC_lexicon_tags_77 },
    { 0x24ea3541, SUC_lexicon_tags_42 },
    { 0xd06a3542, SUC_lexicon_tags_9 },
    { 0xae6a3542, SUC_lexicon_tags_63 },
    { 0xd98a3540, SUC_lexicon_tags_9 },
    { 0xef8a3543, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2a3549, SUC_lexicon_tags_26 },
    { 0x672a354a, SUC_lexicon_tags_66 },
    { 0x95ea3549, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc76a354d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ca3550, SUC_lexicon_tags_26 },
    { 0xb16a3550, SUC_lexicon_tags_47 },
    { 0xfd0a3552, SUC_lexicon_tags_15 },
    { 0x2ea3551, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x272a3556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x376a3558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6a355c, SUC_lexicon_tags_6 },
    { 0x94aa355c, SUC_lexicon_tags_118 },
    { 0x514a355d, SUC_lexicon_tags_28 },
    { 0x514a355e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938a3563, SUC_lexicon_tags_0 },
    { 0x148a3563, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0x69aa3566, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x418a3568, SUC_lexicon_tags_12 },
    { 0x18aa3569, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca356c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x22ea356e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa34a3570, SUC_lexicon_tags_19 },
    { 0x98ca3571, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caa3577, SUC_lexicon_tags_49 },
    { 0x7a6a3577, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66a3583, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec0a3585, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b4a3587, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ea358c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ea3591, SUC_lexicon_tags_25 },
    { 0xc48a3591, SUC_lexicon_tags_63 },
    { 0x452a3593, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea35a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fca35a6, SUC_lexicon_tags_12 },
    { 0x1b8a35a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd2a35a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82a35ac, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4a35b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x840a35b3, SUC_lexicon_tags_1 },
    { 0xbd2a35b4, SUC_lexicon_tags_29 },
    { 0x3f4a35b5, SUC_lexicon_tags_17 },
    { 0xf24a35b3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a35b9, SUC_lexicon_tags_2 },
    { 0xf32a35ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8a35bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ca35c0, SUC_lexicon_tags_68 },
    { 0x4b6a35c1, SUC_lexicon_tags_6 },
    { 0xe8ca35c1, SUC_lexicon_tags_37 },
    { 0x154a35c3, SUC_lexicon_tags_87 },
    { 0xa66a35c3, SUC_lexicon_tags_15 },
    { 0x7aea35c3, SUC_lexicon_tags_14 },
    { 0x53ea35c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48a35cb, SUC_lexicon_tags_14 },
    { 0x890a35cc, SUC_lexicon_tags_7 },
    { 0x99ea35cd, SUC_lexicon_tags_6 },
    { 0x3eaa35ce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a35d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a35d5, SUC_lexicon_tags_631 },
    { 0, NULL },
    { 0x6a8a35d7, SUC_lexicon_tags_37 },
    { 0x5f8a35d8, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xb1aa35da, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe7ca35dc, SUC_lexicon_tags_17 },
    { 0xa8ca35dc, SUC_lexicon_tags_0 },
    { 0xaea35de, SUC_lexicon_tags_28 },
    { 0x1e4a35dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97aa35e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ea35e5, SUC_lexicon_tags_2 },
    { 0x27ea35e6, SUC_lexicon_tags_37 },
    { 0xc28a35e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe80a35e9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x90aa35ec, SUC_lexicon_tags_17 },
    { 0xa32a35ed, SUC_lexicon_tags_60 },
    { 0x7c4a35ee, SUC_lexicon_tags_12 },
    { 0x466a35ed, SUC_lexicon_tags_11 },
    { 0x9e0a35f0, SUC_lexicon_tags_37 },
    { 0xd1ea35ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930a35f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2a35fc, SUC_lexicon_tags_49 },
    { 0x5a6a35fc, SUC_lexicon_tags_34 },
    { 0x898a35fe, SUC_lexicon_tags_11 },
    { 0xad6a35ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aea3605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59ca3607, SUC_lexicon_tags_37 },
    { 0x1c0a3608, SUC_lexicon_tags_11 },
    { 0xfa4a3607, SUC_lexicon_tags_25 },
    { 0x434a360a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x192a360c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a3610, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc36a3612, SUC_lexicon_tags_1 },
    { 0x566a3613, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x288a3616, SUC_lexicon_tags_0 },
    { 0x16ea3616, SUC_lexicon_tags_21 },
    { 0xadca3617, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaa361c, SUC_lexicon_tags_1 },
    { 0xab0a361c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4fea361f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2faa3622, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98a3630, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf88a3632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x374a3634, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x4faa3636, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520a363a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ea363d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12a3641, SUC_lexicon_tags_2 },
    { 0x5b4a3642, SUC_lexicon_tags_0 },
    { 0xfcca3642, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94a3649, SUC_lexicon_tags_1 },
    { 0x18a364a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5aa364d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0a3654, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98a3657, SUC_lexicon_tags_34 },
    { 0xf6ea3658, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994a365c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8a3664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdea3666, SUC_lexicon_tags_23 },
    { 0xa26a3666, SUC_lexicon_tags_63 },
    { 0x424a3668, SUC_lexicon_tags_39 },
    { 0x620a3669, SUC_lexicon_tags_9 },
    { 0x58a366a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6a366e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x864a3670, SUC_lexicon_tags_43 },
    { 0x154a3671, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4a3673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664a3678, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc62a367a, SUC_lexicon_tags_38 },
    { 0x560a367b, SUC_lexicon_tags_74 },
    { 0x9e0a367b, SUC_lexicon_tags_12 },
    { 0x7a8a367d, SUC_lexicon_tags_6 },
    { 0xa74a367e, SUC_lexicon_tags_23 },
    { 0x50aa367a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6dea3681, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4a3684, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x134a3687, SUC_lexicon_tags_26 },
    { 0xf68a3687, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606a368c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4a368f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a3693, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x768a3696, SUC_lexicon_tags_10 },
    { 0x886a3697, SUC_lexicon_tags_15 },
    { 0x5f6a3698, SUC_lexicon_tags_25 },
    { 0x55ca3699, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6ca369b, SUC_lexicon_tags_47 },
    { 0xe6a369b, SUC_lexicon_tags_33 },
    { 0x8faa369b, SUC_lexicon_tags_28 },
    { 0x714a369c, SUC_lexicon_tags_12 },
    { 0x756a369f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x928a36a2, SUC_lexicon_tags_19 },
    { 0xea8a36a3, SUC_lexicon_tags_10 },
    { 0xb4ea36a2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb00a36a6, SUC_lexicon_tags_7 },
    { 0x3c6a36a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x972a36a9, SUC_lexicon_tags_19 },
    { 0xceca36a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0a36ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde0a36af, SUC_lexicon_tags_4 },
    { 0x136a36b0, SUC_lexicon_tags_165 },
    { 0x8f2a36b1, SUC_lexicon_tags_37 },
    { 0x6c4a36b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x690a36b4, SUC_lexicon_tags_0 },
    { 0xcfaa36b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ea36bb, SUC_lexicon_tags_12 },
    { 0x24ca36bb, SUC_lexicon_tags_15 },
    { 0xd00a36bd, SUC_lexicon_tags_14 },
    { 0xae6a36be, SUC_lexicon_tags_47 },
    { 0x1e2a36bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00a36c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x516a36c7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9baa36c9, SUC_lexicon_tags_17 },
    { 0xf10a36c9, SUC_lexicon_tags_29 },
    { 0x2b2a36cb, SUC_lexicon_tags_11 },
    { 0x39aa36cc, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dea36d1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8a36d5, SUC_lexicon_tags_35 },
    { 0x97ca36d6, SUC_lexicon_tags_23 },
    { 0x9baa36d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4a36da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x370a36dc, SUC_lexicon_tags_17 },
    { 0xfb0a36dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd68a36df, SUC_lexicon_tags_48 },
    { 0xe9aa36e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0a36e4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1fca36e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a36e9, SUC_lexicon_tags_57 },
    { 0x23aa36e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf12a36ec, SUC_lexicon_tags_25 },
    { 0xa68a36ed, SUC_lexicon_tags_141 },
    { 0xaaaa36ec, SUC_lexicon_tags_61 },
    { 0x7dca36ec, SUC_lexicon_tags_38 },
    { 0xd5aa36f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x476a36f2, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xb32a36f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50ea36f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacca36f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ea36fd, SUC_lexicon_tags_19 },
    { 0x176a36fe, SUC_lexicon_tags_104 },
    { 0x860a36fe, SUC_lexicon_tags_65 },
    { 0x6faa3700, SUC_lexicon_tags_1 },
    { 0x93aa3700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68a3704, SUC_lexicon_tags_47 },
    { 0x268a3705, SUC_lexicon_tags_25 },
    { 0x606a3706, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c4a3708, SUC_lexicon_tags_7 },
    { 0x78ca3709, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29aa3710, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x228a3713, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fea3715, SUC_lexicon_tags_12 },
    { 0xd0ca3716, SUC_lexicon_tags_23 },
    { 0xb7ea3717, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4a371a, SUC_lexicon_tags_9 },
    { 0xb2a371a, SUC_lexicon_tags_0 },
    { 0x10aa371c, SUC_lexicon_tags_15 },
    { 0x6d2a371d, SUC_lexicon_tags_23 },
    { 0x3b4a371d, SUC_lexicon_tags_15 },
    { 0x9f0a371d, SUC_lexicon_tags_17 },
    { 0x7dea3720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x804a3723, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0a3726, SUC_lexicon_tags_38 },
    { 0xf0aa3727, SUC_lexicon_tags_14 },
    { 0x22aa3727, SUC_lexicon_tags_38 },
    { 0x5bea3727, SUC_lexicon_tags_14 },
    { 0xa0a3726, SUC_lexicon_tags_25 },
    { 0x740a372b, SUC_lexicon_tags_12 },
    { 0xe78a372c, SUC_lexicon_tags_2 },
    { 0xb22a372d, SUC_lexicon_tags_2 },
    { 0xe08a372e, SUC_lexicon_tags_23 },
    { 0x40a372f, SUC_lexicon_tags_65 },
    { 0xcaca372d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc0a3732, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ca3737, SUC_lexicon_tags_26 },
    { 0x588a3737, SUC_lexicon_tags_66 },
    { 0xe76a3739, SUC_lexicon_tags_38 },
    { 0xb34a3737, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0a373e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0a3742, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfbaa3744, SUC_lexicon_tags_11 },
    { 0xf00a3745, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x5daa3748, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2faa374a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x146a374c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0a3751, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8aaa3753, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52a3756, SUC_lexicon_tags_3 },
    { 0xd90a3756, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36a375a, SUC_lexicon_tags_77 },
    { 0xa24a375b, SUC_lexicon_tags_11 },
    { 0x5fea375c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb58a375e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xad4a3760, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71aa3762, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbeea3764, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8a3767, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfea376d, SUC_lexicon_tags_15 },
    { 0x90ca376e, SUC_lexicon_tags_49 },
    { 0x2ca376f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54a3772, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa60a3775, SUC_lexicon_tags_7 },
    { 0x84a3775, SUC_lexicon_tags_11 },
    { 0x218a3776, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xedca377a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8a3782, SUC_lexicon_tags_15 },
    { 0x3bca3783, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62a3788, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe3aa378a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aaa378c, SUC_lexicon_tags_9 },
    { 0x738a378d, SUC_lexicon_tags_83 },
    { 0x3eca378e, SUC_lexicon_tags_0 },
    { 0xa64a378f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x384a3791, SUC_lexicon_tags_11 },
    { 0x444a3792, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aa379a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaa379d, SUC_lexicon_tags_37 },
    { 0xd6ea379e, SUC_lexicon_tags_60 },
    { 0xce4a379f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa68a37a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca37a6, SUC_lexicon_tags_23 },
    { 0x4ea37a7, SUC_lexicon_tags_64 },
    { 0x4aa37a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5aa37aa, SUC_lexicon_tags_7 },
    { 0x5e6a37ab, SUC_lexicon_tags_11 },
    { 0xbbea37ac, SUC_lexicon_tags_11 },
    { 0xb0ca37ac, SUC_lexicon_tags_107 },
    { 0x152a37ab, SUC_lexicon_tags_39 },
    { 0x9a2a37ad, SUC_lexicon_tags_11 },
    { 0x89aa37ad, SUC_lexicon_tags_37 },
    { 0x586a37ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe12a37b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x138a37b5, SUC_lexicon_tags_23 },
    { 0x202a37b6, SUC_lexicon_tags_63 },
    { 0x93ca37b7, SUC_lexicon_tags_65 },
    { 0x98aa37b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526a37bc, SUC_lexicon_tags_26 },
    { 0xe5aa37bd, SUC_lexicon_tags_16 },
    { 0xe84a37bc, SUC_lexicon_tags_27 },
    { 0xb92a37be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c6a37c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baa37cc, SUC_lexicon_tags_38 },
    { 0x712a37cd, SUC_lexicon_tags_9 },
    { 0xc62a37cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaca37d2, SUC_lexicon_tags_7 },
    { 0xdd0a37d3, SUC_lexicon_tags_49 },
    { 0xafca37d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fca37d9, SUC_lexicon_tags_25 },
    { 0xf40a37d9, SUC_lexicon_tags_49 },
    { 0x3eea37db, SUC_lexicon_tags_47 },
    { 0x440a37dc, SUC_lexicon_tags_83 },
    { 0x176a37dd, SUC_lexicon_tags_9 },
    { 0xcb4a37de, SUC_lexicon_tags_32 },
    { 0x8d0a37d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daa37e9, SUC_lexicon_tags_27 },
    { 0x630a37ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceca37ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8a37f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6daa37f2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ea37f5, SUC_lexicon_tags_1 },
    { 0x66aa37f5, SUC_lexicon_tags_15 },
    { 0x394a37f6, SUC_lexicon_tags_0 },
    { 0xec4a37f7, SUC_lexicon_tags_1 },
    { 0x2ea37f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952a37fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f6a3800, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8a3805, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66a3810, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18a3819, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300a3823, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196a382a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29aa382c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ea3831, SUC_lexicon_tags_47 },
    { 0xb60a3831, SUC_lexicon_tags_37 },
    { 0x36aa3833, SUC_lexicon_tags_45 },
    { 0x822a3833, SUC_lexicon_tags_19 },
    { 0x356a3835, SUC_lexicon_tags_37 },
    { 0xeeea3834, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ea383b, SUC_lexicon_tags_4 },
    { 0xcc4a383c, SUC_lexicon_tags_31 },
    { 0xdbea383b, SUC_lexicon_tags_33 },
    { 0x8aaa383c, SUC_lexicon_tags_65 },
    { 0xe0ca383f, SUC_lexicon_tags_12 },
    { 0x720a3840, SUC_lexicon_tags_25 },
    { 0x652a3841, SUC_lexicon_tags_23 },
    { 0xcca3841, SUC_lexicon_tags_14 },
    { 0x7d4a3841, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0a3846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0a3849, SUC_lexicon_tags_23 },
    { 0xe40a3849, SUC_lexicon_tags_38 },
    { 0x452a384a, SUC_lexicon_tags_69 },
    { 0x51ca384c, SUC_lexicon_tags_68 },
    { 0x9caa384b, SUC_lexicon_tags_87 },
    { 0x80a384e, SUC_lexicon_tags_1 },
    { 0x8d4a384d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dea3854, SUC_lexicon_tags_37 },
    { 0x5aca3855, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ea385b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52aa385f, SUC_lexicon_tags_10 },
    { 0xd2a3860, SUC_lexicon_tags_38 },
    { 0x832a3861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfeaa3863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ea3866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x512a3869, SUC_lexicon_tags_2 },
    { 0x82ea3869, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24a386f, SUC_lexicon_tags_7 },
    { 0xd7aa386f, SUC_lexicon_tags_26 },
    { 0x670a3871, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x148a3874, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x518a3876, SUC_lexicon_tags_19 },
    { 0xb3ea3877, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378a387b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2a387d, SUC_lexicon_tags_14 },
    { 0x806a387e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ca3882, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8a388e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe48a3890, SUC_lexicon_tags_47 },
    { 0xd90a3890, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6aa3893, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcf8a3895, SUC_lexicon_tags_27 },
    { 0x9ca3896, SUC_lexicon_tags_14 },
    { 0xd1aa3897, SUC_lexicon_tags_37 },
    { 0xae0a3897, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90a389c, SUC_lexicon_tags_37 },
    { 0xf7ca389d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68a38a6, SUC_lexicon_tags_13 },
    { 0x9fea38a7, SUC_lexicon_tags_15 },
    { 0x14ca38a8, SUC_lexicon_tags_49 },
    { 0x150a38a9, SUC_lexicon_tags_7 },
    { 0x2fca38a8, SUC_lexicon_tags_19 },
    { 0x46ea38a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x768a38ad, SUC_lexicon_tags_4 },
    { 0x608a38ae, SUC_lexicon_tags_10 },
    { 0xdd2a38af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94a38b6, SUC_lexicon_tags_21 },
    { 0x514a38b7, SUC_lexicon_tags_40 },
    { 0x62ea38b8, SUC_lexicon_tags_2 },
    { 0x78ca38b8, SUC_lexicon_tags_96 },
    { 0x2d2a38b8, SUC_lexicon_tags_55 },
    { 0xe7aa38b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bca38bd, SUC_lexicon_tags_12 },
    { 0x638a38bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ca38c2, SUC_lexicon_tags_11 },
    { 0xe2a38c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfaa38c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x660a38c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a38cc, SUC_lexicon_tags_25 },
    { 0x904a38cd, SUC_lexicon_tags_25 },
    { 0x6aaa38ce, SUC_lexicon_tags_30 },
    { 0xb66a38cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab8a38d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x930a38d4, SUC_lexicon_tags_11 },
    { 0x4ea38d4, SUC_lexicon_tags_15 },
    { 0xcaaa38d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d4a38d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa14a38da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0a38de, SUC_lexicon_tags_61 },
    { 0x292a38df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6a38e5, SUC_lexicon_tags_37 },
    { 0x1b4a38e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910a38ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f0a38ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0a38f7, SUC_lexicon_tags_38 },
    { 0x398a38f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ea38fc, SUC_lexicon_tags_12 },
    { 0x418a38fd, SUC_lexicon_tags_49 },
    { 0x808a38fe, SUC_lexicon_tags_8 },
    { 0xf4ea38fc, SUC_lexicon_tags_0 },
    { 0xcd6a3900, SUC_lexicon_tags_25 },
    { 0xb44a3901, SUC_lexicon_tags_26 },
    { 0x8daa3901, SUC_lexicon_tags_31 },
    { 0x688a38fc, SUC_lexicon_tags_11 },
    { 0x5eca3901, SUC_lexicon_tags_23 },
    { 0x1e0a3905, SUC_lexicon_tags_15 },
    { 0x1f4a3906, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda4a3908, SUC_lexicon_tags_15 },
    { 0xc50a3909, SUC_lexicon_tags_45 },
    { 0x8f6a3909, SUC_lexicon_tags_28 },
    { 0xf72a390b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2a3916, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04a391b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x750a391d, SUC_lexicon_tags_33 },
    { 0x732a391e, SUC_lexicon_tags_44 },
    { 0xd68a391f, SUC_lexicon_tags_37 },
    { 0x3d8a3920, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2eea3922, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd8a3924, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2ea3926, SUC_lexicon_tags_6 },
    { 0x718a3927, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fea392b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64a392d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x394a392f, SUC_lexicon_tags_2 },
    { 0xf0a3930, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbaca3932, SUC_lexicon_tags_38 },
    { 0x514a3933, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860a3939, SUC_lexicon_tags_12 },
    { 0x24a393a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x738a393d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ea3943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e2a3945, SUC_lexicon_tags_12 },
    { 0x686a3946, SUC_lexicon_tags_15 },
    { 0x1b0a3947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x318a394a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x578a394c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440a3950, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe14a3952, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ea3956, SUC_lexicon_tags_30 },
    { 0x8eaa3956, SUC_lexicon_tags_33 },
    { 0xadca3956, SUC_lexicon_tags_11 },
    { 0x4b0a3959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3ea395b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8a3960, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ca3967, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a396b, SUC_lexicon_tags_3 },
    { 0xe4ea396c, SUC_lexicon_tags_32 },
    { 0x286a396b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a3970, SUC_lexicon_tags_7 },
    { 0x28a3970, SUC_lexicon_tags_32 },
    { 0x53aa3972, SUC_lexicon_tags_7 },
    { 0x7b2a3973, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5caa3975, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ea3978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcca397d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52a398a, SUC_lexicon_tags_43 },
    { 0x682a398b, SUC_lexicon_tags_16 },
    { 0x510a398c, SUC_lexicon_tags_17 },
    { 0x164a398d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ca3992, SUC_lexicon_tags_32 },
    { 0x59aa3992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x380a3995, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0a3998, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ca39a1, SUC_lexicon_tags_16 },
    { 0x476a39a2, SUC_lexicon_tags_1 },
    { 0x3a8a39a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x568a39a5, SUC_lexicon_tags_2 },
    { 0xd8aa39a5, SUC_lexicon_tags_9 },
    { 0x28ca39a5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf4aa39a9, SUC_lexicon_tags_454 },
    { 0xed2a39aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4aaa39ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ea39b1, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca39b5, SUC_lexicon_tags_27 },
    { 0x8cca39b5, SUC_lexicon_tags_7 },
    { 0x258a39b6, SUC_lexicon_tags_50 },
    { 0xf72a39b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x474a39ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2fea39bc, SUC_lexicon_tags_60 },
    { 0xb0aa39bd, SUC_lexicon_tags_37 },
    { 0xceca39bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8ca39c0, SUC_lexicon_tags_17 },
    { 0x68a39c0, SUC_lexicon_tags_8 },
    { 0x750a39c1, SUC_lexicon_tags_14 },
    { 0xf58a39c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x526a39c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27ea39c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0a39d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30a39d9, SUC_lexicon_tags_63 },
    { 0x698a39da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950a39e2, SUC_lexicon_tags_17 },
    { 0x676a39e3, SUC_lexicon_tags_45 },
    { 0xba2a39e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ca39ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ea39ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd20a39f0, SUC_lexicon_tags_2 },
    { 0x3dea39f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6a39fb, SUC_lexicon_tags_17 },
    { 0xf28a39fc, SUC_lexicon_tags_55 },
    { 0xa44a39fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6a3a01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a3a08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ea3a0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984a3a11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26a3a13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x586a3a16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38a3a1a, SUC_lexicon_tags_25 },
    { 0xd62a3a1b, SUC_lexicon_tags_37 },
    { 0xfaea3a1c, SUC_lexicon_tags_25 },
    { 0x6aa3a1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0a3a20, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x956a3a22, SUC_lexicon_tags_31 },
    { 0xc6a3a23, SUC_lexicon_tags_15 },
    { 0xa42a3a24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30a3a2f, SUC_lexicon_tags_66 },
    { 0x33ea3a2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ca3a35, SUC_lexicon_tags_39 },
    { 0x4baa3a36, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x22a3a38, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aa3a3c, SUC_lexicon_tags_33 },
    { 0xf42a3a3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x758a3a40, SUC_lexicon_tags_2 },
    { 0x542a3a40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13ea3a43, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506a3a4d, SUC_lexicon_tags_25 },
    { 0x9e6a3a4e, SUC_lexicon_tags_25 },
    { 0x722a3a4f, SUC_lexicon_tags_10 },
    { 0x702a3a50, SUC_lexicon_tags_17 },
    { 0x2c0a3a4e, SUC_lexicon_tags_7 },
    { 0x46ea3a52, SUC_lexicon_tags_47 },
    { 0x896a3a52, SUC_lexicon_tags_37 },
    { 0xfa2a3a53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0a3a58, SUC_lexicon_tags_11 },
    { 0xb12a3a58, SUC_lexicon_tags_2 },
    { 0x528a3a5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd72a3a5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bea3a5f, SUC_lexicon_tags_46 },
    { 0x814a3a60, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20aa3a66, SUC_lexicon_tags_46 },
    { 0x76a3a66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4a3a6f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2a3a73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x250a3a76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39ea3a78, SUC_lexicon_tags_37 },
    { 0x92a3a79, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4a3a7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x364a3a81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8a3a86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e2a3a88, SUC_lexicon_tags_25 },
    { 0xeeaa3a89, SUC_lexicon_tags_32 },
    { 0x8e2a3a8a, SUC_lexicon_tags_33 },
    { 0x2cca3a8b, SUC_lexicon_tags_25 },
    { 0x42a3a8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71ca3a8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a2a3a90, SUC_lexicon_tags_37 },
    { 0xd4aa3a90, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x940a3a94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75ea3a96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x148a3a99, SUC_lexicon_tags_33 },
    { 0x5faa3a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670a3a9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b2a3aa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2a3aa5, SUC_lexicon_tags_1 },
    { 0x1fea3aa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2a3aa9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792a3aaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2ca3ab1, SUC_lexicon_tags_17 },
    { 0xf7ea3ab2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ea3ab6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc24a3ab8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4a3ac1, SUC_lexicon_tags_60 },
    { 0x12aa3ac2, SUC_lexicon_tags_2 },
    { 0x242a3ac3, SUC_lexicon_tags_353 },
    { 0x44ea3ac3, SUC_lexicon_tags_26 },
    { 0x80ca3ac4, SUC_lexicon_tags_47 },
    { 0x5c6a3ac6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6cca3ac8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8a3acb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x566a3acd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x566a3acf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76a3ad3, SUC_lexicon_tags_7 },
    { 0x222a3ad4, SUC_lexicon_tags_14 },
    { 0xc6aa3ad4, SUC_lexicon_tags_56 },
    { 0x498a3ad5, SUC_lexicon_tags_23 },
    { 0x304a3ad5, SUC_lexicon_tags_17 },
    { 0xa54a3ad8, SUC_lexicon_tags_49 },
    { 0xf78a3ad9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6e8a3adb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6a3ade, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aaa3ae2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfea3ae6, SUC_lexicon_tags_0 },
    { 0x98aa3ae6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35aa3aec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a3aef, SUC_lexicon_tags_31 },
    { 0x260a3af0, SUC_lexicon_tags_31 },
    { 0xdb6a3af0, SUC_lexicon_tags_6 },
    { 0x9c6a3aef, SUC_lexicon_tags_2 },
    { 0x318a3af0, SUC_lexicon_tags_87 },
    { 0x33ea3af4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2a3afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8a3b01, SUC_lexicon_tags_1 },
    { 0xaaaa3b02, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbd8a3b04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0a3b08, SUC_lexicon_tags_47 },
    { 0x9b2a3b09, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe80a3b0b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62a3b14, SUC_lexicon_tags_77 },
    { 0xa18a3b14, SUC_lexicon_tags_27 },
    { 0x876a3b16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x588a3b18, SUC_lexicon_tags_60 },
    { 0x5e0a3b19, SUC_lexicon_tags_1 },
    { 0xd8ca3b19, SUC_lexicon_tags_38 },
    { 0x8c8a3b19, SUC_lexicon_tags_7 },
    { 0x14aa3b1c, SUC_lexicon_tags_11 },
    { 0xf02a3b1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ea3b20, SUC_lexicon_tags_7 },
    { 0xc2ea3b20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf98a3b23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea3b2b, SUC_lexicon_tags_231 },
    { 0x6fea3b2b, SUC_lexicon_tags_38 },
    { 0x3aca3b2c, SUC_lexicon_tags_13 },
    { 0x27ea3b2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96a3b31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6a3b34, SUC_lexicon_tags_37 },
    { 0xe64a3b35, SUC_lexicon_tags_41 },
    { 0xe0ea3b36, SUC_lexicon_tags_2 },
    { 0xd02a3b35, SUC_lexicon_tags_9 },
    { 0xdeaa3b38, SUC_lexicon_tags_2 },
    { 0x9aaa3b38, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6d6a3b3b, SUC_lexicon_tags_23 },
    { 0x49ca3b3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8a3b3f, SUC_lexicon_tags_28 },
    { 0xae2a3b40, SUC_lexicon_tags_47 },
    { 0xe70a3b41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcea3b44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde4a3b47, SUC_lexicon_tags_45 },
    { 0x3cca3b48, SUC_lexicon_tags_25 },
    { 0x3a4a3b48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa00a3b4b, SUC_lexicon_tags_0 },
    { 0x574a3b4c, SUC_lexicon_tags_1 },
    { 0x9cea3b4c, SUC_lexicon_tags_15 },
    { 0xea0a3b4b, SUC_lexicon_tags_60 },
    { 0xd92a3b4f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cea3b56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752a3b5d, SUC_lexicon_tags_15 },
    { 0xb56a3b5e, SUC_lexicon_tags_46 },
    { 0x982a3b5f, SUC_lexicon_tags_39 },
    { 0xdaea3b5f, SUC_lexicon_tags_17 },
    { 0x258a3b5e, SUC_lexicon_tags_31 },
    { 0xb78a3b62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe6ea3b64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca3b67, SUC_lexicon_tags_7 },
    { 0xf6aa3b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76a3b6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31aa3b6c, SUC_lexicon_tags_49 },
    { 0x58aa3b6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ca3b72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x228a3b74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46a3b81, SUC_lexicon_tags_65 },
    { 0xaf2a3b81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cca3b89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x892a3b8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa3b93, SUC_lexicon_tags_35 },
    { 0x1c4a3b94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aa3b99, SUC_lexicon_tags_348 },
    { 0x98ea3b99, SUC_lexicon_tags_29 },
    { 0xc04a3b9a, SUC_lexicon_tags_11 },
    { 0xcc8a3b9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ea3b9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47aa3ba8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aa3bad, SUC_lexicon_tags_28 },
    { 0x9baa3bae, SUC_lexicon_tags_14 },
    { 0x12a3baf, SUC_lexicon_tags_113 },
    { 0x6b0a3baf, SUC_lexicon_tags_45 },
    { 0x8f4a3baf, SUC_lexicon_tags_42 },
    { 0x916a3bb2, SUC_lexicon_tags_60 },
    { 0xf1aa3bb2, SUC_lexicon_tags_15 },
    { 0x1f8a3bad, SUC_lexicon_tags_14 },
    { 0x83aa3bb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0a3bb8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71ca3bba, SUC_lexicon_tags_15 },
    { 0x8c8a3bba, SUC_lexicon_tags_8 },
    { 0x11aa3bbc, SUC_lexicon_tags_25 },
    { 0x464a3bbd, SUC_lexicon_tags_17 },
    { 0xcf0a3bbe, SUC_lexicon_tags_7 },
    { 0x108a3bbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ca3bc2, SUC_lexicon_tags_20 },
    { 0x142a3bc3, SUC_lexicon_tags_27 },
    { 0x384a3bc3, SUC_lexicon_tags_25 },
    { 0xfaaa3bc5, SUC_lexicon_tags_14 },
    { 0x99ca3bc3, SUC_lexicon_tags_14 },
    { 0x2d2a3bc7, SUC_lexicon_tags_1 },
    { 0xb66a3bc8, SUC_lexicon_tags_4 },
    { 0x810a3bc2, SUC_lexicon_tags_37 },
    { 0x182a3bca, SUC_lexicon_tags_28 },
    { 0xdf2a3bc5, SUC_lexicon_tags_19 },
    { 0x788a3bcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11ca3bce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714a3bd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326a3bd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67aa3bd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x272a3bdb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x88ea3bdd, SUC_lexicon_tags_11 },
    { 0x804a3bdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976a3be4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2a3bea, SUC_lexicon_tags_182 },
    { 0xa36a3beb, SUC_lexicon_tags_60 },
    { 0x5fca3bec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd4a3bee, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xfc8a3bf0, SUC_lexicon_tags_30 },
    { 0x850a3bf1, SUC_lexicon_tags_63 },
    { 0xba8a3bf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4a3bf7, SUC_lexicon_tags_0 },
    { 0xf92a3bf8, SUC_lexicon_tags_0 },
    { 0x338a3bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ca3c00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6a3c05, SUC_lexicon_tags_17 },
    { 0x1faa3c06, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ea3c0b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ca3c16, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16a3c19, SUC_lexicon_tags_35 },
    { 0x1ca3c1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700a3c20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25ca3c22, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14a3c2b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x346a3c2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x788a3c30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c4a3c32, SUC_lexicon_tags_17 },
    { 0x2f4a3c33, SUC_lexicon_tags_17 },
    { 0x1fea3c33, SUC_lexicon_tags_1 },
    { 0x630a3c34, SUC_lexicon_tags_29 },
    { 0xb86a3c36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccca3c3a, SUC_lexicon_tags_1 },
    { 0x7eea3c3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x618a3c3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6a3c42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58a3c46, SUC_lexicon_tags_26 },
    { 0x2caa3c47, SUC_lexicon_tags_11 },
    { 0xd30a3c48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96a3c4d, SUC_lexicon_tags_11 },
    { 0xd56a3c4e, SUC_lexicon_tags_91 },
    { 0xae2a3c4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8c8a3c51, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xfb0a3c53, SUC_lexicon_tags_11 },
    { 0x24ca3c53, SUC_lexicon_tags_17 },
    { 0xc52a3c55, SUC_lexicon_tags_13 },
    { 0xedaa3c56, SUC_lexicon_tags_82 },
    { 0x80a3c55, SUC_lexicon_tags_9 },
    { 0xc22a3c57, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xef8a3c5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a3c61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494a3c65, SUC_lexicon_tags_17 },
    { 0x5f0a3c66, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714a3c6a, SUC_lexicon_tags_38 },
    { 0xb16a3c6a, SUC_lexicon_tags_29 },
    { 0x2d8a3c6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa78a3c6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ea3c73, SUC_lexicon_tags_0 },
    { 0xa10a3c74, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x482a3c76, SUC_lexicon_tags_37 },
    { 0x59aa3c77, SUC_lexicon_tags_43 },
    { 0xf36a3c77, SUC_lexicon_tags_37 },
    { 0xb54a3c76, SUC_lexicon_tags_45 },
    { 0x430a3c7a, SUC_lexicon_tags_21 },
    { 0x910a3c7a, SUC_lexicon_tags_42 },
    { 0x67ea3c7c, SUC_lexicon_tags_82 },
    { 0x5d2a3c7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x194a3c7f, SUC_lexicon_tags_11 },
    { 0xb6ca3c80, SUC_lexicon_tags_21 },
    { 0xf68a3c80, SUC_lexicon_tags_11 },
    { 0x52ea3c82, SUC_lexicon_tags_7 },
    { 0x1a8a3c83, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ea3c86, SUC_lexicon_tags_14 },
    { 0x540a3c87, SUC_lexicon_tags_16 },
    { 0x870a3c87, SUC_lexicon_tags_38 },
    { 0xcb0a3c87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cca3c8b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x488a3c8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xef8a3c8f, SUC_lexicon_tags_11 },
    { 0xaf4a3c90, SUC_lexicon_tags_29 },
    { 0x76aa3c91, SUC_lexicon_tags_37 },
    { 0x7eca3c90, SUC_lexicon_tags_27 },
    { 0x8c4a3c91, SUC_lexicon_tags_10 },
    { 0xf6a3c94, SUC_lexicon_tags_15 },
    { 0x570a3c95, SUC_lexicon_tags_8 },
    { 0x7d8a3c96, SUC_lexicon_tags_25 },
    { 0x5eea3c96, SUC_lexicon_tags_27 },
    { 0x25aa3c97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14a3c9b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ea3ca1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a6a3ca3, SUC_lexicon_tags_62 },
    { 0x2d8a3ca4, SUC_lexicon_tags_15 },
    { 0x862a3ca3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2a3ca7, SUC_lexicon_tags_133 },
    { 0x168a3ca8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1faa3cab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282a3cb1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60a3cb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4a3cbd, SUC_lexicon_tags_12 },
    { 0x996a3cbd, SUC_lexicon_tags_19 },
    { 0xa98a3cbd, SUC_lexicon_tags_28 },
    { 0x924a3cbf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca3cc7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x10aa3cca, SUC_lexicon_tags_13 },
    { 0x98aa3ccb, SUC_lexicon_tags_6 },
    { 0x9e4a3cca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbea3cd0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84a3cd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74aa3cd8, SUC_lexicon_tags_178 },
    { 0x37aa3cd9, SUC_lexicon_tags_28 },
    { 0x51ca3cda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0a3cde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6a3ce1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xefaa3ce3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x972a3ce6, SUC_lexicon_tags_13 },
    { 0x68a3ce7, SUC_lexicon_tags_1 },
    { 0x2c0a3ce7, SUC_lexicon_tags_37 },
    { 0x8ca3ce9, SUC_lexicon_tags_37 },
    { 0xf16a3cea, SUC_lexicon_tags_2 },
    { 0xc0ca3cea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ca3cee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8a3cf6, SUC_lexicon_tags_1 },
    { 0xa10a3cf6, SUC_lexicon_tags_31 },
    { 0x772a3cf8, SUC_lexicon_tags_14 },
    { 0xa0ea3cf8, SUC_lexicon_tags_27 },
    { 0x16aa3cf9, SUC_lexicon_tags_4 },
    { 0xc1ca3cfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2a3d00, SUC_lexicon_tags_30 },
    { 0x8e4a3d01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174a3d05, SUC_lexicon_tags_60 },
    { 0xbeea3d05, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238a3d0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0a3d12, SUC_lexicon_tags_17 },
    { 0x7cca3d12, SUC_lexicon_tags_31 },
    { 0xac0a3d14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ca3d1a, SUC_lexicon_tags_15 },
    { 0x316a3d1b, SUC_lexicon_tags_35 },
    { 0xb34a3d1c, SUC_lexicon_tags_7 },
    { 0xd80a3d1c, SUC_lexicon_tags_6 },
    { 0x17aa3d1e, SUC_lexicon_tags_62 },
    { 0x8f8a3d1c, SUC_lexicon_tags_32 },
    { 0xccca3d20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf62a3d22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ca3d25, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ca3d29, SUC_lexicon_tags_12 },
    { 0x6c2a3d2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2a3d2e, SUC_lexicon_tags_25 },
    { 0xc22a3d2f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9a8a3d31, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x810a3d33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x198a3d35, SUC_lexicon_tags_17 },
    { 0x93ca3d36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x926a3d39, SUC_lexicon_tags_66 },
    { 0xae0a3d3a, SUC_lexicon_tags_11 },
    { 0xc2ca3d3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ea3d3f, SUC_lexicon_tags_43 },
    { 0x51ea3d3f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cea3d48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706a3d4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ca3d4e, SUC_lexicon_tags_23 },
    { 0x9aca3d4e, SUC_lexicon_tags_33 },
    { 0x644a3d4f, SUC_lexicon_tags_6 },
    { 0x500a3d50, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982a3d56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x198a3d58, SUC_lexicon_tags_11 },
    { 0x888a3d58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51aa3d5b, SUC_lexicon_tags_1 },
    { 0xbdaa3d5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572a3d61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2a3d65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fea3d67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840a3d6e, SUC_lexicon_tags_17 },
    { 0x110a3d6e, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0xadea3d72, SUC_lexicon_tags_68 },
    { 0xf96a3d73, SUC_lexicon_tags_6 },
    { 0x520a3d74, SUC_lexicon_tags_49 },
    { 0xfd4a3d75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1eca3d77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4a3d7b, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x5f4a3d7d, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xc16a3d7f, SUC_lexicon_tags_44 },
    { 0x4dea3d80, SUC_lexicon_tags_37 },
    { 0x9d4a3d80, SUC_lexicon_tags_9 },
    { 0xcdca3d82, SUC_lexicon_tags_26 },
    { 0x90ea3d80, SUC_lexicon_tags_17 },
    { 0xd58a3d7f, SUC_lexicon_tags_15 },
    { 0x97ea3d82, SUC_lexicon_tags_37 },
    { 0x9dea3d86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x298a3d88, SUC_lexicon_tags_15 },
    { 0x3faa3d88, SUC_lexicon_tags_42 },
    { 0xd78a3d88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50a3d8e, SUC_lexicon_tags_207 },
    { 0x30ea3d8f, SUC_lexicon_tags_15 },
    { 0xf36a3d8e, SUC_lexicon_tags_17 },
    { 0x694a3d91, SUC_lexicon_tags_38 },
    { 0xe1ea3d92, SUC_lexicon_tags_7 },
    { 0xc7ca3d92, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938a3d9c, SUC_lexicon_tags_11 },
    { 0xd5ea3d9d, SUC_lexicon_tags_12 },
    { 0x594a3d9e, SUC_lexicon_tags_43 },
    { 0xac8a3d9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc6ca3da1, SUC_lexicon_tags_34 },
    { 0x79ea3da2, SUC_lexicon_tags_47 },
    { 0x590a3da2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266a3da7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0ca3da9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x600a3dab, SUC_lexicon_tags_8 },
    { 0x9c4a3dac, SUC_lexicon_tags_31 },
    { 0xddaa3dad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740a3db4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x606a3db6, SUC_lexicon_tags_0 },
    { 0xb08a3db6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ca3dba, SUC_lexicon_tags_12 },
    { 0xd1ca3dbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4a3dc0, SUC_lexicon_tags_11 },
    { 0x608a3dc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ca3dc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6a3dc8, SUC_lexicon_tags_5 },
    { 0x8bca3dc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d2a3dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8a3dcf, SUC_lexicon_tags_37 },
    { 0x72ea3dd0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x320a3dd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ca3dda, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x842a3ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd16a3dde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36a3de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb96a3de4, SUC_lexicon_tags_17 },
    { 0x8d0a3de5, SUC_lexicon_tags_17 },
    { 0x194a3de5, SUC_lexicon_tags_14 },
    { 0xa4aa3de7, SUC_lexicon_tags_11 },
    { 0x594a3de5, SUC_lexicon_tags_7 },
    { 0x8aa3de5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x70aa3dec, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0a3df0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf80a3df2, SUC_lexicon_tags_65 },
    { 0xdd4a3df2, SUC_lexicon_tags_43 },
    { 0x15ea3df3, SUC_lexicon_tags_66 },
    { 0x2dca3df5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca3df9, SUC_lexicon_tags_11 },
    { 0xd38a3df9, SUC_lexicon_tags_1 },
    { 0xa12a3dfb, SUC_lexicon_tags_15 },
    { 0xf0ca3dfb, SUC_lexicon_tags_38 },
    { 0x648a3dfd, SUC_lexicon_tags_17 },
    { 0xf6a3dfe, SUC_lexicon_tags_138 },
    { 0xb64a3dfb, SUC_lexicon_tags_66 },
    { 0x2b4a3dff, SUC_lexicon_tags_31 },
    { 0x2a0a3dff, SUC_lexicon_tags_140 },
    { 0x354a3dff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeb2a3e04, SUC_lexicon_tags_7 },
    { 0xae0a3e05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6a3e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18a3e0c, SUC_lexicon_tags_37 },
    { 0x67ca3e0d, SUC_lexicon_tags_7 },
    { 0xac6a3e0e, SUC_lexicon_tags_30 },
    { 0x44ca3e0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9aa3e13, SUC_lexicon_tags_25 },
    { 0x9a0a3e14, SUC_lexicon_tags_33 },
    { 0x8a8a3e14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x924a3e17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2a3e1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c6a3e1d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x354a3e1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x494a3e22, SUC_lexicon_tags_37 },
    { 0xd8a3e22, SUC_lexicon_tags_7 },
    { 0x60a3e22, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6a3e2a, SUC_lexicon_tags_30 },
    { 0xeea3e2a, SUC_lexicon_tags_12 },
    { 0xca4a3e2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bca3e2e, SUC_lexicon_tags_3 },
    { 0xba4a3e2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ca3e34, SUC_lexicon_tags_47 },
    { 0x5c8a3e35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96ca3e37, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5d4a3e39, SUC_lexicon_tags_60 },
    { 0x5b2a3e3a, SUC_lexicon_tags_12 },
    { 0x1c6a3e3a, SUC_lexicon_tags_27 },
    { 0x6bca3e39, SUC_lexicon_tags_33 },
    { 0x7a6a3e3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90a3e46, SUC_lexicon_tags_37 },
    { 0x21ea3e46, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58a3e4d, SUC_lexicon_tags_11 },
    { 0x540a3e4e, SUC_lexicon_tags_11 },
    { 0xdc4a3e4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x186a3e52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x82a3e55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x466a3e57, SUC_lexicon_tags_8 },
    { 0x6f8a3e58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc80a3e5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc88a3e5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a3e5f, SUC_lexicon_tags_60 },
    { 0x7ea3e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf50a3e62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52a3e69, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xbe6a3e6b, SUC_lexicon_tags_37 },
    { 0x7e4a3e6c, SUC_lexicon_tags_37 },
    { 0x834a3e6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8ea3e6f, SUC_lexicon_tags_49 },
    { 0xb8ea3e70, SUC_lexicon_tags_34 },
    { 0xec6a3e71, SUC_lexicon_tags_43 },
    { 0x440a3e72, SUC_lexicon_tags_0 },
    { 0x718a3e71, SUC_lexicon_tags_49 },
    { 0x58ea3e72, SUC_lexicon_tags_32 },
    { 0xf4ca3e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce6a3e77, SUC_lexicon_tags_26 },
    { 0x46aa3e78, SUC_lexicon_tags_25 },
    { 0x6b4a3e79, SUC_lexicon_tags_39 },
    { 0xdfca3e7a, SUC_lexicon_tags_15 },
    { 0x84ca3e77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40aa3e80, SUC_lexicon_tags_27 },
    { 0x988a3e81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ea3e87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x828a3e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ca3e8c, SUC_lexicon_tags_49 },
    { 0xfaaa3e8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ca3e93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faa3e98, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ea3e9d, SUC_lexicon_tags_16 },
    { 0x652a3e9d, SUC_lexicon_tags_23 },
    { 0x59ca3e9f, SUC_lexicon_tags_2 },
    { 0xc6ca3ea0, SUC_lexicon_tags_10 },
    { 0xc20a3ea1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c8a3ea3, SUC_lexicon_tags_19 },
    { 0x196a3ea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2a3ea8, SUC_lexicon_tags_66 },
    { 0xec4a3ea9, SUC_lexicon_tags_10 },
    { 0x85ea3eaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaa3ead, SUC_lexicon_tags_12 },
    { 0x996a3eae, SUC_lexicon_tags_30 },
    { 0xe4a3eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6a3eb3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x98aa3eb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a3ebd, SUC_lexicon_tags_17 },
    { 0xdc0a3ebd, SUC_lexicon_tags_34 },
    { 0x752a3ebe, SUC_lexicon_tags_63 },
    { 0x44aa3ebf, SUC_lexicon_tags_12 },
    { 0x8b2a3ec0, SUC_lexicon_tags_33 },
    { 0xc7ca3ec2, SUC_lexicon_tags_17 },
    { 0x4bca3ebf, SUC_lexicon_tags_38 },
    { 0xdd8a3ec4, SUC_lexicon_tags_17 },
    { 0xde2a3ebe, SUC_lexicon_tags_25 },
    { 0xe1aa3ec3, SUC_lexicon_tags_25 },
    { 0x5e8a3ec6, SUC_lexicon_tags_15 },
    { 0x562a3ec8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a3ecc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x306a3ece, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x30ea3ed0, SUC_lexicon_tags_45 },
    { 0x206a3ed0, SUC_lexicon_tags_25 },
    { 0x2bea3ed2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2aa3ed4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca3ed7, SUC_lexicon_tags_43 },
    { 0xca3ed8, SUC_lexicon_tags_63 },
    { 0x644a3ed9, SUC_lexicon_tags_37 },
    { 0x3cea3eda, SUC_lexicon_tags_12 },
    { 0xbf4a3ed9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x448a3ede, SUC_lexicon_tags_23 },
    { 0xb30a3edf, SUC_lexicon_tags_30 },
    { 0x38ea3ee0, SUC_lexicon_tags_38 },
    { 0x198a3ee0, SUC_lexicon_tags_61 },
    { 0xc42a3ee1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66a3ee5, SUC_lexicon_tags_61 },
    { 0xc68a3ee6, SUC_lexicon_tags_4 },
    { 0x338a3ee7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa26a3ee9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934a3eed, SUC_lexicon_tags_15 },
    { 0x76ca3eed, SUC_lexicon_tags_14 },
    { 0x102a3eed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ea3ef2, SUC_lexicon_tags_66 },
    { 0xc22a3ef3, SUC_lexicon_tags_16 },
    { 0x3d6a3ef4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbeca3ef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa6a3ef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744a3efd, SUC_lexicon_tags_1 },
    { 0x7c4a3efe, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54a3f01, SUC_lexicon_tags_12 },
    { 0xbe0a3f01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f4a3f04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc94a3f06, SUC_lexicon_tags_30 },
    { 0x2d4a3f06, SUC_lexicon_tags_11 },
    { 0xba0a3f08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40a3f0d, SUC_lexicon_tags_43 },
    { 0xec8a3f0e, SUC_lexicon_tags_19 },
    { 0xc4aa3f0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0a3f18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218a3f1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44a3f23, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x826a3f26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x476a3f28, SUC_lexicon_tags_63 },
    { 0x71ca3f28, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x310a3f2c, SUC_lexicon_tags_26 },
    { 0x74aa3f2c, SUC_lexicon_tags_25 },
    { 0xf6ca3f2e, SUC_lexicon_tags_38 },
    { 0xd84a3f2f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x1b0a3f31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aca3f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81aa3f42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4a3f48, SUC_lexicon_tags_30 },
    { 0xa8ea3f49, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18a3f50, SUC_lexicon_tags_23 },
    { 0xa88a3f51, SUC_lexicon_tags_44 },
    { 0x3d0a3f52, SUC_lexicon_tags_38 },
    { 0x588a3f50, SUC_lexicon_tags_23 },
    { 0x698a3f50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94a3f58, SUC_lexicon_tags_33 },
    { 0x2b0a3f59, SUC_lexicon_tags_19 },
    { 0xa56a3f5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0a3f62, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6a3f65, SUC_lexicon_tags_11 },
    { 0x5d8a3f66, SUC_lexicon_tags_10 },
    { 0x956a3f67, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a3f6c, SUC_lexicon_tags_6 },
    { 0xc38a3f6c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca3f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1eca3f73, SUC_lexicon_tags_11 },
    { 0xf4aa3f73, SUC_lexicon_tags_1 },
    { 0x64aa3f75, SUC_lexicon_tags_82 },
    { 0x246a3f73, SUC_lexicon_tags_9 },
    { 0x7a6a3f77, SUC_lexicon_tags_11 },
    { 0x31ca3f76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894a3f81, SUC_lexicon_tags_2 },
    { 0x8b2a3f82, SUC_lexicon_tags_37 },
    { 0x97aa3f83, SUC_lexicon_tags_0 },
    { 0xce0a3f84, SUC_lexicon_tags_1 },
    { 0x40ea3f82, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfc8a3f87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc16a3f89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ca3f8e, SUC_lexicon_tags_1 },
    { 0x98a3f8e, SUC_lexicon_tags_37 },
    { 0xd8ca3f90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eca3f94, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bea3f97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984a3f9b, SUC_lexicon_tags_25 },
    { 0xed2a3f9b, SUC_lexicon_tags_32 },
    { 0x496a3f9b, SUC_lexicon_tags_182 },
    { 0x49ea3f9d, SUC_lexicon_tags_39 },
    { 0x988a3f9f, SUC_lexicon_tags_8 },
    { 0xd7aa3fa0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ea3fa3, SUC_lexicon_tags_34 },
    { 0x4a3fa3, SUC_lexicon_tags_31 },
    { 0xa84a3fa3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862a3faf, SUC_lexicon_tags_11 },
    { 0xe48a3faf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6aa3fb3, SUC_lexicon_tags_7 },
    { 0xa6aa3fb4, SUC_lexicon_tags_33 },
    { 0x8c4a3fb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23ea3fb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ca3fbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc6a3fbe, SUC_lexicon_tags_36 },
    { 0x988a3fbe, SUC_lexicon_tags_17 },
    { 0xd46a3fc0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718a3fc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0aa3fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26a3fcf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8a3fd2, SUC_lexicon_tags_12 },
    { 0x202a3fd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15aa3fd6, SUC_lexicon_tags_11 },
    { 0x622a3fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaea3fda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x274a3fdd, SUC_lexicon_tags_63 },
    { 0xd1aa3fde, SUC_lexicon_tags_16 },
    { 0x4a3fdf, SUC_lexicon_tags_27 },
    { 0x8f8a3fe0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742a3fe7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a3feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ca3fee, SUC_lexicon_tags_151 },
    { 0xf6aa3fee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190a3ff5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xba8a3ff7, SUC_lexicon_tags_4 },
    { 0x9dea3ff8, SUC_lexicon_tags_15 },
    { 0xe46a3ff9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4a3ffd, SUC_lexicon_tags_60 },
    { 0x51ca3ffe, SUC_lexicon_tags_1 },
    { 0x6dca3ffe, SUC_lexicon_tags_25 },
    { 0x1eea3fff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ea4004, SUC_lexicon_tags_26 },
    { 0xf86a4005, SUC_lexicon_tags_25 },
    { 0xb88a4006, SUC_lexicon_tags_44 },
    { 0xda2a4006, SUC_lexicon_tags_39 },
    { 0x162a4006, SUC_lexicon_tags_37 },
    { 0xc9ea4009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x652a400c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4a4010, SUC_lexicon_tags_29 },
    { 0x906a4011, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6a4014, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738a401b, SUC_lexicon_tags_45 },
    { 0x72ca401c, SUC_lexicon_tags_12 },
    { 0x34ea401d, SUC_lexicon_tags_38 },
    { 0x1caa401e, SUC_lexicon_tags_14 },
    { 0x764a401f, SUC_lexicon_tags_12 },
    { 0x906a401e, SUC_lexicon_tags_17 },
    { 0x654a4021, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544a4025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a402b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6a402e, SUC_lexicon_tags_25 },
    { 0x72ca402e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2a4035, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6a403a, SUC_lexicon_tags_2 },
    { 0x994a403b, SUC_lexicon_tags_15 },
    { 0x3b2a403c, SUC_lexicon_tags_29 },
    { 0xc6ea403c, SUC_lexicon_tags_0 },
    { 0x896a403c, SUC_lexicon_tags_17 },
    { 0xce4a403b, SUC_lexicon_tags_3 },
    { 0xbf4a403f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x35aa4042, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52a404f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0a4052, SUC_lexicon_tags_45 },
    { 0x29ea4053, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aca405b, SUC_lexicon_tags_10 },
    { 0x36ca405c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716a4064, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0a406a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaa406f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2a4076, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cea407c, SUC_lexicon_tags_60 },
    { 0x120a407d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84a408a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe3aa408c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ea408f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1aa4095, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc10a4097, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0a409c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3e6a409e, SUC_lexicon_tags_63 },
    { 0x7eca409e, SUC_lexicon_tags_23 },
    { 0x33ea40a0, SUC_lexicon_tags_4 },
    { 0x748a40a1, SUC_lexicon_tags_11 },
    { 0x368a40a2, SUC_lexicon_tags_37 },
    { 0x5e0a40a2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a40a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116a40ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa60a40ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x78a40af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aa40b3, SUC_lexicon_tags_60 },
    { 0x2faa40b4, SUC_lexicon_tags_59 },
    { 0xb32a40b5, SUC_lexicon_tags_37 },
    { 0x862a40b6, SUC_lexicon_tags_10 },
    { 0x632a40b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a0a40b9, SUC_lexicon_tags_1 },
    { 0x808a40ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9daa40bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340a40c0, SUC_lexicon_tags_4 },
    { 0x368a40c0, SUC_lexicon_tags_26 },
    { 0x9f8a40c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7aa40c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xab8a40c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7eaa40c8, SUC_lexicon_tags_5 },
    { 0xf12a40c8, SUC_lexicon_tags_15 },
    { 0xba6a40c8, SUC_lexicon_tags_15 },
    { 0xaf8a40c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ca40ce, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0x54ca40d0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe52a40d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854a40d6, SUC_lexicon_tags_11 },
    { 0x680a40d6, SUC_lexicon_tags_2 },
    { 0x124a40d8, SUC_lexicon_tags_40 },
    { 0xcaca40d8, SUC_lexicon_tags_10 },
    { 0x66ea40d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8a40e1, SUC_lexicon_tags_15 },
    { 0xe64a40e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4a0a40e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ca40e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbea40e9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x18ca40eb, SUC_lexicon_tags_38 },
    { 0x15aa40ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ca40f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdeaa40f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdea40f5, SUC_lexicon_tags_29 },
    { 0x9caa40f6, SUC_lexicon_tags_11 },
    { 0x140a40f7, SUC_lexicon_tags_37 },
    { 0x5c6a40f7, SUC_lexicon_tags_17 },
    { 0x4fea40f5, SUC_lexicon_tags_12 },
    { 0x728a40fa, SUC_lexicon_tags_47 },
    { 0x3e4a40fb, SUC_lexicon_tags_38 },
    { 0x748a40f5, SUC_lexicon_tags_25 },
    { 0xe74a40f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ea410b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca4116, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x354a4118, SUC_lexicon_tags_8 },
    { 0x43aa4119, SUC_lexicon_tags_44 },
    { 0x7bca4119, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdc8a411c, SUC_lexicon_tags_16 },
    { 0xa8aa411d, SUC_lexicon_tags_92 },
    { 0x78ca411d, SUC_lexicon_tags_57 },
    { 0xb2aa411d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cea4122, SUC_lexicon_tags_2 },
    { 0x8fca4123, SUC_lexicon_tags_14 },
    { 0x35ea4124, SUC_lexicon_tags_55 },
    { 0xb74a4125, SUC_lexicon_tags_14 },
    { 0x20ea4125, SUC_lexicon_tags_708 },
    { 0xa5ca4126, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ea412c, SUC_lexicon_tags_17 },
    { 0xdb6a412c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ca4131, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x64aa4134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf98a4136, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ea413d, SUC_lexicon_tags_25 },
    { 0xf94a413d, SUC_lexicon_tags_27 },
    { 0x7eea413d, SUC_lexicon_tags_23 },
    { 0xad2a4140, SUC_lexicon_tags_25 },
    { 0x846a413e, SUC_lexicon_tags_27 },
    { 0x2c4a4142, SUC_lexicon_tags_846 },
    { 0, NULL },
    { 0x9aea4144, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ca4148, SUC_lexicon_tags_106 },
    { 0x1f4a4148, SUC_lexicon_tags_45 },
    { 0xcd2a414a, SUC_lexicon_tags_1 },
    { 0x278a414b, SUC_lexicon_tags_14 },
    { 0xcd8a414c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x242a414e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a4a4150, SUC_lexicon_tags_20 },
    { 0xa7ea4151, SUC_lexicon_tags_7 },
    { 0x74ca4152, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230a4158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94a415b, SUC_lexicon_tags_104 },
    { 0xbbea415c, SUC_lexicon_tags_15 },
    { 0xd42a415d, SUC_lexicon_tags_37 },
    { 0xbd8a415d, SUC_lexicon_tags_47 },
    { 0x8c2a415c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04a4163, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaa4166, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4a416c, SUC_lexicon_tags_0 },
    { 0x3bca416c, SUC_lexicon_tags_12 },
    { 0x82ea416d, SUC_lexicon_tags_80 },
    { 0xea8a416f, SUC_lexicon_tags_1 },
    { 0x9baa416c, SUC_lexicon_tags_37 },
    { 0xf5ea416f, SUC_lexicon_tags_17 },
    { 0xb94a416e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc5aa4174, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4a4177, SUC_lexicon_tags_14 },
    { 0xb8ca4178, SUC_lexicon_tags_38 },
    { 0xd4aa4179, SUC_lexicon_tags_63 },
    { 0x88ea4178, SUC_lexicon_tags_1 },
    { 0x8aa4179, SUC_lexicon_tags_430 },
    { 0x6eca417c, SUC_lexicon_tags_23 },
    { 0x1d2a417b, SUC_lexicon_tags_47 },
    { 0xfc4a417d, SUC_lexicon_tags_0 },
    { 0x620a417b, SUC_lexicon_tags_17 },
    { 0x3d2a417f, SUC_lexicon_tags_7 },
    { 0x108a4180, SUC_lexicon_tags_37 },
    { 0xb72a4182, SUC_lexicon_tags_37 },
    { 0xd7ca4182, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x938a4185, SUC_lexicon_tags_37 },
    { 0x2eea4186, SUC_lexicon_tags_55 },
    { 0xfeca4185, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ea4193, SUC_lexicon_tags_28 },
    { 0xbeea4193, SUC_lexicon_tags_19 },
    { 0xf2ea4195, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8aa4197, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704a41a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286a41a5, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1aa41aa, SUC_lexicon_tags_0 },
    { 0xeb4a41aa, SUC_lexicon_tags_1 },
    { 0xeb8a41ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ca41b3, SUC_lexicon_tags_17 },
    { 0xed8a41b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xccca41b7, SUC_lexicon_tags_19 },
    { 0x4eea41b8, SUC_lexicon_tags_1 },
    { 0x27ca41b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814a41be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8a41c1, SUC_lexicon_tags_43 },
    { 0xc64a41c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd34a41c4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5aa41c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0a41cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xceca41cd, SUC_lexicon_tags_47 },
    { 0xeb2a41ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aa41d1, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80a41d5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8a2a41d7, SUC_lexicon_tags_28 },
    { 0x632a41d8, SUC_lexicon_tags_30 },
    { 0x89aa41d8, SUC_lexicon_tags_40 },
    { 0x472a41da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca41de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2a41e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc56a41e3, SUC_lexicon_tags_25 },
    { 0xe80a41e3, SUC_lexicon_tags_7 },
    { 0x982a41e4, SUC_lexicon_tags_17 },
    { 0xbcca41e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78a41e9, SUC_lexicon_tags_9 },
    { 0x2d4a41e9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56a41ee, SUC_lexicon_tags_3 },
    { 0x39ea41ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97aa41f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e0a41f3, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x43aa41f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64a41fd, SUC_lexicon_tags_49 },
    { 0x9e6a41fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2a4203, SUC_lexicon_tags_12 },
    { 0x906a4204, SUC_lexicon_tags_48 },
    { 0x696a4205, SUC_lexicon_tags_12 },
    { 0xaa8a4203, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaa420c, SUC_lexicon_tags_47 },
    { 0x978a420d, SUC_lexicon_tags_9 },
    { 0x5aca420c, SUC_lexicon_tags_43 },
    { 0x11ca420f, SUC_lexicon_tags_30 },
    { 0xe82a420f, SUC_lexicon_tags_23 },
    { 0x1e6a420f, SUC_lexicon_tags_37 },
    { 0x82aa420c, SUC_lexicon_tags_9 },
    { 0x30aa420c, SUC_lexicon_tags_92 },
    { 0x716a4214, SUC_lexicon_tags_15 },
    { 0x3c8a420c, SUC_lexicon_tags_160 },
    { 0x76ea4216, SUC_lexicon_tags_11 },
    { 0x806a4217, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea421e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x420a4221, SUC_lexicon_tags_24 },
    { 0x550a4222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcca4224, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafca4229, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2a422c, SUC_lexicon_tags_31 },
    { 0x91ca422d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04a4232, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bca4234, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78aa4239, SUC_lexicon_tags_19 },
    { 0x7e6a423a, SUC_lexicon_tags_4 },
    { 0xb1aa423b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954a4241, SUC_lexicon_tags_35 },
    { 0xa76a4242, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ca4246, SUC_lexicon_tags_25 },
    { 0xbaa4246, SUC_lexicon_tags_37 },
    { 0x330a4248, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c6a424a, SUC_lexicon_tags_11 },
    { 0x460a424b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x458a424d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5a8a424f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f0a4251, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0a4254, SUC_lexicon_tags_30 },
    { 0xf50a4255, SUC_lexicon_tags_15 },
    { 0xab8a4256, SUC_lexicon_tags_12 },
    { 0x1d8a4255, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa28a425a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x920a425c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2a4261, SUC_lexicon_tags_17 },
    { 0x2d8a4261, SUC_lexicon_tags_7 },
    { 0x9dea4263, SUC_lexicon_tags_14 },
    { 0x9eaa4264, SUC_lexicon_tags_0 },
    { 0x2a6a4265, SUC_lexicon_tags_129 },
    { 0x1d8a4266, SUC_lexicon_tags_17 },
    { 0xb02a4267, SUC_lexicon_tags_17 },
    { 0x766a4268, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4a4270, SUC_lexicon_tags_7 },
    { 0x940a4271, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9aa4275, SUC_lexicon_tags_9 },
    { 0x44ca4276, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aa427a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xca2a427c, SUC_lexicon_tags_25 },
    { 0x462a427c, SUC_lexicon_tags_37 },
    { 0x7dea427e, SUC_lexicon_tags_2 },
    { 0xadea427f, SUC_lexicon_tags_19 },
    { 0xd6aa427f, SUC_lexicon_tags_37 },
    { 0x7e8a427f, SUC_lexicon_tags_29 },
    { 0x3b0a4282, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2a4286, SUC_lexicon_tags_17 },
    { 0x79ca4287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a428a, SUC_lexicon_tags_14 },
    { 0xba0a428b, SUC_lexicon_tags_8 },
    { 0x604a428c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xca428e, SUC_lexicon_tags_25 },
    { 0xf3ea428f, SUC_lexicon_tags_7 },
    { 0x64aa4290, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ea4295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376a429f, SUC_lexicon_tags_27 },
    { 0x204a42a0, SUC_lexicon_tags_29 },
    { 0xcd6a42a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbea42a3, SUC_lexicon_tags_2 },
    { 0x950a42a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38a42aa, SUC_lexicon_tags_31 },
    { 0x98ca42ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe72a42ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x212a42af, SUC_lexicon_tags_17 },
    { 0x740a42af, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00a42b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dea42bc, SUC_lexicon_tags_25 },
    { 0xcbca42bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756a42c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x278a42c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aca42cb, SUC_lexicon_tags_35 },
    { 0x6eea42cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76a42d2, SUC_lexicon_tags_11 },
    { 0xaf8a42d3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaca42d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x580a42da, SUC_lexicon_tags_30 },
    { 0x736a42db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60aa42e1, SUC_lexicon_tags_2 },
    { 0xba4a42e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7fca42e4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8eaa42e6, SUC_lexicon_tags_63 },
    { 0x2a4a42e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654a42ec, SUC_lexicon_tags_84 },
    { 0x5e4a42ed, SUC_lexicon_tags_636 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770a42f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x846a42f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf6a42f6, SUC_lexicon_tags_124 },
    { 0xc7ea42f7, SUC_lexicon_tags_14 },
    { 0x2d2a42f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ca42fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca42ff, SUC_lexicon_tags_37 },
    { 0x524a4300, SUC_lexicon_tags_12 },
    { 0xaaca4300, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eea4304, SUC_lexicon_tags_3 },
    { 0xdb0a4305, SUC_lexicon_tags_63 },
    { 0xac4a4304, SUC_lexicon_tags_1 },
    { 0xbdea4304, SUC_lexicon_tags_25 },
    { 0x776a4308, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c4a430a, SUC_lexicon_tags_19 },
    { 0x246a430b, SUC_lexicon_tags_30 },
    { 0x400a430c, SUC_lexicon_tags_49 },
    { 0xe1ea430c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31aa4310, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x358a4313, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe32a4315, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x50a4318, SUC_lexicon_tags_11 },
    { 0x742a4319, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292a431f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156a4325, SUC_lexicon_tags_11 },
    { 0x5cca4325, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dea4328, SUC_lexicon_tags_11 },
    { 0xd24a4329, SUC_lexicon_tags_1 },
    { 0x394a4329, SUC_lexicon_tags_28 },
    { 0xd06a4328, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf0ea432d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdea4333, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8a4338, SUC_lexicon_tags_57 },
    { 0x9d2a4338, SUC_lexicon_tags_63 },
    { 0x7d4a433a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42a433d, SUC_lexicon_tags_25 },
    { 0xdb2a433e, SUC_lexicon_tags_48 },
    { 0xebaa433e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x630a4342, SUC_lexicon_tags_7 },
    { 0x524a4343, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xde8a4345, SUC_lexicon_tags_13 },
    { 0xd2ca4345, SUC_lexicon_tags_39 },
    { 0xf28a4346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98ea4349, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x484a434b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06a4350, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb2ca4352, SUC_lexicon_tags_37 },
    { 0x70ea4353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x674a4356, SUC_lexicon_tags_7 },
    { 0xe2a4356, SUC_lexicon_tags_107 },
    { 0xb1ea4358, SUC_lexicon_tags_21 },
    { 0xe18a4358, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12a435d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x478a435f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20aa4365, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x52ea4367, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x95ca4369, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x110a436b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60a436e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ca4373, SUC_lexicon_tags_151 },
    { 0xb78a4374, SUC_lexicon_tags_15 },
    { 0x3dea4375, SUC_lexicon_tags_25 },
    { 0xa2aa4373, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8a437d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54a4380, SUC_lexicon_tags_37 },
    { 0x1c0a4381, SUC_lexicon_tags_11 },
    { 0x18ea4381, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x4eca4384, SUC_lexicon_tags_11 },
    { 0x212a4385, SUC_lexicon_tags_27 },
    { 0xc62a4386, SUC_lexicon_tags_21 },
    { 0x532a4387, SUC_lexicon_tags_18 },
    { 0xf2aa4387, SUC_lexicon_tags_12 },
    { 0x746a4385, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x974a438c, SUC_lexicon_tags_15 },
    { 0x2faa438d, SUC_lexicon_tags_11 },
    { 0x252a438e, SUC_lexicon_tags_27 },
    { 0xe9ea438f, SUC_lexicon_tags_7 },
    { 0x434a438e, SUC_lexicon_tags_37 },
    { 0xb5ca4391, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0xf32a4393, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb20a4395, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x170a4398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6a439c, SUC_lexicon_tags_61 },
    { 0x934a439d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7aa439f, SUC_lexicon_tags_12 },
    { 0xf50a43a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70ca43a2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aa43a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf66a43ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdca43b1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd7ca43b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684a43b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454a43be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8a43c2, SUC_lexicon_tags_33 },
    { 0xe50a43c2, SUC_lexicon_tags_2 },
    { 0x8f4a43c3, SUC_lexicon_tags_82 },
    { 0xfd0a43c5, SUC_lexicon_tags_7 },
    { 0xd40a43c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260a43ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4a43cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2a43d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x564a43d8, SUC_lexicon_tags_1 },
    { 0xc74a43d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xccea43dc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca43e0, SUC_lexicon_tags_33 },
    { 0xdc0a43e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a43e4, SUC_lexicon_tags_55 },
    { 0xf90a43e4, SUC_lexicon_tags_2 },
    { 0xa9ca43e5, SUC_lexicon_tags_8 },
    { 0x42ea43e7, SUC_lexicon_tags_11 },
    { 0x98aa43e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122a43ec, SUC_lexicon_tags_63 },
    { 0x826a43ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x268a43ef, SUC_lexicon_tags_1 },
    { 0x628a43ef, SUC_lexicon_tags_37 },
    { 0xc1ca43f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe34a43f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaa43f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724a43fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8fea43ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad2a4401, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdcea4403, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcca4406, SUC_lexicon_tags_9 },
    { 0xf9aa4407, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe54a4409, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe24a440b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ea4415, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486a4419, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26a441f, SUC_lexicon_tags_6 },
    { 0x8b8a4420, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x714a4422, SUC_lexicon_tags_1 },
    { 0x3e6a4422, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1fca4425, SUC_lexicon_tags_28 },
    { 0xeb0a4426, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2aca4428, SUC_lexicon_tags_98 },
    { 0x3a0a4428, SUC_lexicon_tags_42 },
    { 0x284a442a, SUC_lexicon_tags_45 },
    { 0x748a4428, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2a442e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aca4435, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x762a4438, SUC_lexicon_tags_28 },
    { 0x1faa4439, SUC_lexicon_tags_15 },
    { 0x2d4a443a, SUC_lexicon_tags_27 },
    { 0xbbaa4438, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63aa4445, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x55ca4447, SUC_lexicon_tags_47 },
    { 0x110a4448, SUC_lexicon_tags_14 },
    { 0xd38a4447, SUC_lexicon_tags_48 },
    { 0x436a444a, SUC_lexicon_tags_10 },
    { 0x832a444a, SUC_lexicon_tags_43 },
    { 0x2a4a444b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10ca444e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfca4453, SUC_lexicon_tags_37 },
    { 0x37ca4454, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978a4458, SUC_lexicon_tags_12 },
    { 0xc5aa4458, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8a445c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fca4461, SUC_lexicon_tags_46 },
    { 0x88ea4461, SUC_lexicon_tags_11 },
    { 0xd6ca4461, SUC_lexicon_tags_11 },
    { 0xd9ca4461, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86a446a, SUC_lexicon_tags_15 },
    { 0xeaca446b, SUC_lexicon_tags_37 },
    { 0x8c6a446c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x57aa446f, SUC_lexicon_tags_26 },
    { 0x1f2a4470, SUC_lexicon_tags_14 },
    { 0xe22a4471, SUC_lexicon_tags_9 },
    { 0xe6ea4471, SUC_lexicon_tags_21 },
    { 0x62ea4470, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aa4479, SUC_lexicon_tags_23 },
    { 0x33ca447a, SUC_lexicon_tags_48 },
    { 0x596a447b, SUC_lexicon_tags_1 },
    { 0xef6a447a, SUC_lexicon_tags_37 },
    { 0x3b2a447d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772a4482, SUC_lexicon_tags_37 },
    { 0xd12a4483, SUC_lexicon_tags_26 },
    { 0xbbca4483, SUC_lexicon_tags_27 },
    { 0x378a4485, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8eaa4487, SUC_lexicon_tags_25 },
    { 0x30a4487, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc88a448a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2a448e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9d6a4490, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ca4494, SUC_lexicon_tags_2 },
    { 0xc8a4494, SUC_lexicon_tags_12 },
    { 0x540a4496, SUC_lexicon_tags_26 },
    { 0xe8ca4496, SUC_lexicon_tags_38 },
    { 0x100a4498, SUC_lexicon_tags_17 },
    { 0x570a4498, SUC_lexicon_tags_25 },
    { 0xbc6a449a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd6a449c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24ca449e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120a44a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17aa44a4, SUC_lexicon_tags_47 },
    { 0x738a44a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17ca44a7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3a0a44a9, SUC_lexicon_tags_25 },
    { 0xc70a44aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x45ea44ac, SUC_lexicon_tags_47 },
    { 0xbeea44ad, SUC_lexicon_tags_75 },
    { 0xf5ea44ac, SUC_lexicon_tags_15 },
    { 0x7aa44ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea44b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4a44b5, SUC_lexicon_tags_28 },
    { 0xd90a44b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0a44b9, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x9bea44bb, SUC_lexicon_tags_11 },
    { 0x46ca44bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2a44c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2a44c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f0a44c5, SUC_lexicon_tags_25 },
    { 0xedca44c6, SUC_lexicon_tags_55 },
    { 0x75aa44c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16a44cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x17ea44ce, SUC_lexicon_tags_12 },
    { 0x9aca44cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb50a44d1, SUC_lexicon_tags_11 },
    { 0x2c2a44d2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea44d6, SUC_lexicon_tags_39 },
    { 0x3cea44d7, SUC_lexicon_tags_37 },
    { 0x528a44d6, SUC_lexicon_tags_60 },
    { 0xdbea44d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256a44dd, SUC_lexicon_tags_1 },
    { 0x356a44de, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xde6a44e0, SUC_lexicon_tags_2 },
    { 0x3eaa44e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a44e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7aa44e6, SUC_lexicon_tags_5 },
    { 0x3b6a44e7, SUC_lexicon_tags_30 },
    { 0xefaa44e8, SUC_lexicon_tags_1 },
    { 0xd1ea44e9, SUC_lexicon_tags_37 },
    { 0x944a44ea, SUC_lexicon_tags_47 },
    { 0xad0a44e8, SUC_lexicon_tags_1 },
    { 0x4e6a44ec, SUC_lexicon_tags_37 },
    { 0xe2ea44ed, SUC_lexicon_tags_49 },
    { 0x2a0a44ec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x27ca44f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d4a44f2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7a0a44f4, SUC_lexicon_tags_39 },
    { 0x7baa44f4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8a44fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aca4509, SUC_lexicon_tags_31 },
    { 0x72aa450a, SUC_lexicon_tags_21 },
    { 0xa30a4509, SUC_lexicon_tags_37 },
    { 0xb34a450c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44aa450f, SUC_lexicon_tags_108 },
    { 0xea6a450f, SUC_lexicon_tags_11 },
    { 0x796a4510, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41aa4517, SUC_lexicon_tags_0 },
    { 0x770a4518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaa451b, SUC_lexicon_tags_11 },
    { 0x9e4a451c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4a451f, SUC_lexicon_tags_33 },
    { 0x4fea451f, SUC_lexicon_tags_15 },
    { 0x716a451f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x592a4523, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x36aa4526, SUC_lexicon_tags_63 },
    { 0xdbca4526, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xd18a4529, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc5ea452b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x72aa452e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5aa4531, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2a4534, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x6e0a4536, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1aa4538, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeca453d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6a4543, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8a4547, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x87ca4549, SUC_lexicon_tags_15 },
    { 0xb8ca454a, SUC_lexicon_tags_14 },
    { 0xd9ea4549, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4a454e, SUC_lexicon_tags_12 },
    { 0x8d4a454f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbc0a4551, SUC_lexicon_tags_7 },
    { 0x72a4552, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00a4557, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2a455b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7aa455d, SUC_lexicon_tags_31 },
    { 0x56ea455d, SUC_lexicon_tags_114 },
    { 0xd3ca455f, SUC_lexicon_tags_43 },
    { 0x546a455d, SUC_lexicon_tags_1 },
    { 0x154a4561, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca4564, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6ca4566, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08a456c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ea4571, SUC_lexicon_tags_49 },
    { 0x46a4572, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882a457b, SUC_lexicon_tags_15 },
    { 0x5aa457c, SUC_lexicon_tags_1 },
    { 0xc7ea457c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13ca457f, SUC_lexicon_tags_80 },
    { 0xcc0a457f, SUC_lexicon_tags_15 },
    { 0xe58a457f, SUC_lexicon_tags_15 },
    { 0x632a4582, SUC_lexicon_tags_32 },
    { 0xdf0a4583, SUC_lexicon_tags_37 },
    { 0x8e2a457f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84a4587, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ea458e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xec6a4591, SUC_lexicon_tags_23 },
    { 0xeaea4592, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6a4595, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44a459f, SUC_lexicon_tags_1 },
    { 0x4aca459f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94aa45a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660a45aa, SUC_lexicon_tags_10 },
    { 0xfea45aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2a45af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ca45b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aa45b7, SUC_lexicon_tags_7 },
    { 0xeb0a45b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76a45bb, SUC_lexicon_tags_1 },
    { 0x9b0a45bc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94a45bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae4a45c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87aa45c7, SUC_lexicon_tags_48 },
    { 0xe28a45c8, SUC_lexicon_tags_17 },
    { 0x31aa45c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2a45cd, SUC_lexicon_tags_45 },
    { 0x3aca45ce, SUC_lexicon_tags_31 },
    { 0x776a45cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8a45d3, SUC_lexicon_tags_1 },
    { 0xb1aa45d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaa45da, SUC_lexicon_tags_14 },
    { 0x50ea45db, SUC_lexicon_tags_25 },
    { 0x32a45dc, SUC_lexicon_tags_1 },
    { 0x5d2a45dd, SUC_lexicon_tags_91 },
    { 0x51aa45dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96a45e1, SUC_lexicon_tags_65 },
    { 0x864a45e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a45e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x822a45e8, SUC_lexicon_tags_58 },
    { 0xa00a45e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf6a45eb, SUC_lexicon_tags_15 },
    { 0x76ca45ec, SUC_lexicon_tags_37 },
    { 0xb7ea45ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x252a45f0, SUC_lexicon_tags_27 },
    { 0x2e8a45f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482a45fd, SUC_lexicon_tags_17 },
    { 0x412a45fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x640a4601, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9b2a4603, SUC_lexicon_tags_37 },
    { 0x752a4603, SUC_lexicon_tags_8 },
    { 0x80ea4605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4a4608, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66a460b, SUC_lexicon_tags_2 },
    { 0xd10a460b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6a4610, SUC_lexicon_tags_3 },
    { 0xc9ea4610, SUC_lexicon_tags_29 },
    { 0xb76a4612, SUC_lexicon_tags_0 },
    { 0x9a6a4613, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca4617, SUC_lexicon_tags_0 },
    { 0x7a0a4618, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a461e, SUC_lexicon_tags_23 },
    { 0x890a461f, SUC_lexicon_tags_25 },
    { 0xbaaa4620, SUC_lexicon_tags_17 },
    { 0x940a4620, SUC_lexicon_tags_7 },
    { 0x686a461f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aea4629, SUC_lexicon_tags_47 },
    { 0x6eca4629, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe56a462c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36ea462e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a4a4630, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4a4635, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ea4638, SUC_lexicon_tags_37 },
    { 0xab4a4639, SUC_lexicon_tags_6 },
    { 0xf3aa4639, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ca463e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaca4642, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff0a4644, SUC_lexicon_tags_15 },
    { 0x7eea4645, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450a4649, SUC_lexicon_tags_49 },
    { 0x6cca464a, SUC_lexicon_tags_5 },
    { 0x19aa464b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x338a464d, SUC_lexicon_tags_1 },
    { 0xaa6a464e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aca4651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf74a4653, SUC_lexicon_tags_12 },
    { 0x8aa4654, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x866a4656, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbe6a4658, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb1aa465a, SUC_lexicon_tags_23 },
    { 0x92ca465b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4a465e, SUC_lexicon_tags_60 },
    { 0xd44a465f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98a4662, SUC_lexicon_tags_11 },
    { 0x90ca4662, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x150a4666, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf76a4668, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x70a466a, SUC_lexicon_tags_46 },
    { 0x5e6a466b, SUC_lexicon_tags_0 },
    { 0xe8ea466a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xeea466f, SUC_lexicon_tags_30 },
    { 0xce2a4670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ea4674, SUC_lexicon_tags_16 },
    { 0x844a4674, SUC_lexicon_tags_41 },
    { 0x5a0a4676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cca4678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6a467c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x80a467e, SUC_lexicon_tags_17 },
    { 0x1dca467f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394a4688, SUC_lexicon_tags_33 },
    { 0x3eca4688, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2aa468d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ea4690, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xab2a4692, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d0a4694, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaca4697, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aea469d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08a46a1, SUC_lexicon_tags_63 },
    { 0x376a46a2, SUC_lexicon_tags_37 },
    { 0xf88a46a3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4a46ad, SUC_lexicon_tags_15 },
    { 0x950a46ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x690a46b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ca46b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ea46b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0a46b9, SUC_lexicon_tags_25 },
    { 0x14ea46ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36a46c2, SUC_lexicon_tags_48 },
    { 0x33ca46c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea46ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x394a46cd, SUC_lexicon_tags_37 },
    { 0x1e6a46cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c0a46d0, SUC_lexicon_tags_216 },
    { 0xea46d0, SUC_lexicon_tags_8 },
    { 0x20aa46d2, SUC_lexicon_tags_2 },
    { 0xe64a46d3, SUC_lexicon_tags_17 },
    { 0xab8a46d4, SUC_lexicon_tags_11 },
    { 0x818a46d5, SUC_lexicon_tags_11 },
    { 0xa50a46d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5eca46d8, SUC_lexicon_tags_37 },
    { 0x54ea46d9, SUC_lexicon_tags_9 },
    { 0x598a46da, SUC_lexicon_tags_3 },
    { 0xa82a46db, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ca46de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x774a46e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed2a46e2, SUC_lexicon_tags_10 },
    { 0x394a46e2, SUC_lexicon_tags_29 },
    { 0x420a46e4, SUC_lexicon_tags_19 },
    { 0xd92a46e5, SUC_lexicon_tags_25 },
    { 0x85aa46e6, SUC_lexicon_tags_16 },
    { 0xc4ea46e7, SUC_lexicon_tags_30 },
    { 0xbe8a46e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aa46eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x488a46ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x764a46f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x548a46f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbfca46f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2a46f8, SUC_lexicon_tags_13 },
    { 0x9e4a46f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97aa46fb, SUC_lexicon_tags_30 },
    { 0xe32a46fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17aa4700, SUC_lexicon_tags_39 },
    { 0x84ca4701, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xae2a4703, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99ca4705, SUC_lexicon_tags_38 },
    { 0xa84a4706, SUC_lexicon_tags_11 },
    { 0x838a4706, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4a470a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182a470e, SUC_lexicon_tags_16 },
    { 0x93ca470f, SUC_lexicon_tags_12 },
    { 0xb1ca4710, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d8a4712, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca8a4715, SUC_lexicon_tags_21 },
    { 0x7a4a4715, SUC_lexicon_tags_11 },
    { 0x4ca4717, SUC_lexicon_tags_15 },
    { 0x804a4717, SUC_lexicon_tags_68 },
    { 0x7eca4715, SUC_lexicon_tags_47 },
    { 0x526a471a, SUC_lexicon_tags_27 },
    { 0xb62a471a, SUC_lexicon_tags_11 },
    { 0x2f6a471a, SUC_lexicon_tags_37 },
    { 0x654a471d, SUC_lexicon_tags_49 },
    { 0xa38a471e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x868a4720, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912a4724, SUC_lexicon_tags_37 },
    { 0x5b2a4724, SUC_lexicon_tags_15 },
    { 0x686a4726, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fca4728, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ea472d, SUC_lexicon_tags_37 },
    { 0x39ca472d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504a4733, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa50a4735, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ca4739, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ca4740, SUC_lexicon_tags_7 },
    { 0x302a4740, SUC_lexicon_tags_60 },
    { 0x790a4741, SUC_lexicon_tags_30 },
    { 0xceea4742, SUC_lexicon_tags_11 },
    { 0x88ea4744, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x488a4746, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x694a4748, SUC_lexicon_tags_30 },
    { 0x5ca4748, SUC_lexicon_tags_39 },
    { 0xe0a4748, SUC_lexicon_tags_60 },
    { 0x860a474a, SUC_lexicon_tags_27 },
    { 0xa80a474c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x86a474e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4a4751, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ea4755, SUC_lexicon_tags_47 },
    { 0x538a4755, SUC_lexicon_tags_11 },
    { 0x442a4757, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46aa475d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8a4760, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a4763, SUC_lexicon_tags_6 },
    { 0x7f0a4764, SUC_lexicon_tags_38 },
    { 0xdbea4765, SUC_lexicon_tags_27 },
    { 0xaf0a4766, SUC_lexicon_tags_14 },
    { 0xed8a4766, SUC_lexicon_tags_19 },
    { 0x78aa4768, SUC_lexicon_tags_50 },
    { 0xe4a4768, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1baa476e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ea4771, SUC_lexicon_tags_46 },
    { 0x58ea4771, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958a4779, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x444a477c, SUC_lexicon_tags_17 },
    { 0xe78a477d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x80ca477f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ca4782, SUC_lexicon_tags_47 },
    { 0xd4aa4783, SUC_lexicon_tags_151 },
    { 0xf48a4783, SUC_lexicon_tags_11 },
    { 0x46ea4782, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58a478a, SUC_lexicon_tags_39 },
    { 0x7b6a478b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc02a478d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408a4791, SUC_lexicon_tags_23 },
    { 0xad4a4792, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8a4795, SUC_lexicon_tags_12 },
    { 0x276a4796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26ea4798, SUC_lexicon_tags_23 },
    { 0x210a4799, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0a479e, SUC_lexicon_tags_27 },
    { 0x38ea479f, SUC_lexicon_tags_38 },
    { 0x9a2a47a0, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0a47a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4a47a6, SUC_lexicon_tags_30 },
    { 0x14ca47a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5eea47a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328a47ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62a47b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a47b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d2a47b6, SUC_lexicon_tags_15 },
    { 0xa8ca47b7, SUC_lexicon_tags_12 },
    { 0x124a47b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb64a47ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eea47bd, SUC_lexicon_tags_15 },
    { 0xb12a47be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7aa47c0, SUC_lexicon_tags_65 },
    { 0x392a47c1, SUC_lexicon_tags_11 },
    { 0x784a47c2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a47c6, SUC_lexicon_tags_25 },
    { 0x7eca47c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc38a47c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea6a47cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x846a47ce, SUC_lexicon_tags_15 },
    { 0x6a2a47cf, SUC_lexicon_tags_45 },
    { 0xd6ca47cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe96a47d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ca47dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x930a47de, SUC_lexicon_tags_31 },
    { 0x8e2a47df, SUC_lexicon_tags_40 },
    { 0x2e4a47e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3caa47e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1dea47e5, SUC_lexicon_tags_17 },
    { 0xf88a47e6, SUC_lexicon_tags_37 },
    { 0xdf4a47e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640a47ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bca47ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752a47f4, SUC_lexicon_tags_38 },
    { 0x42ea47f5, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xc16a47f7, SUC_lexicon_tags_2 },
    { 0xcb0a47f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45ea47fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ea47fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa44a47ff, SUC_lexicon_tags_27 },
    { 0x9dca4800, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfea4802, SUC_lexicon_tags_43 },
    { 0xb38a4802, SUC_lexicon_tags_7 },
    { 0xf2a4802, SUC_lexicon_tags_12 },
    { 0xe0ea4803, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x574a4808, SUC_lexicon_tags_38 },
    { 0x6e8a4809, SUC_lexicon_tags_19 },
    { 0xf8aa4809, SUC_lexicon_tags_49 },
    { 0x9ca480b, SUC_lexicon_tags_37 },
    { 0xc66a4808, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x720a480f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x888a4811, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacca4813, SUC_lexicon_tags_31 },
    { 0x698a4813, SUC_lexicon_tags_38 },
    { 0x640a4815, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbaaa4817, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a481b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506a4821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678a4825, SUC_lexicon_tags_25 },
    { 0xc22a4826, SUC_lexicon_tags_19 },
    { 0x18aa4827, SUC_lexicon_tags_49 },
    { 0xeaca4827, SUC_lexicon_tags_6 },
    { 0x402a4825, SUC_lexicon_tags_46 },
    { 0x3cca4828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc36a482c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x588a482e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc2a4830, SUC_lexicon_tags_31 },
    { 0x74ea4831, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60a4835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8a483c, SUC_lexicon_tags_0 },
    { 0xfa4a483d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeeea483f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb04a4841, SUC_lexicon_tags_15 },
    { 0xd92a4842, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8a4845, SUC_lexicon_tags_37 },
    { 0xc84a4846, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x292a4848, SUC_lexicon_tags_68 },
    { 0x312a4848, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea484e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7aa4850, SUC_lexicon_tags_37 },
    { 0x9e0a4851, SUC_lexicon_tags_26 },
    { 0x612a4852, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baa4856, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x142a4858, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746a485c, SUC_lexicon_tags_33 },
    { 0x96a485c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x884a485f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ca4862, SUC_lexicon_tags_1 },
    { 0x206a4863, SUC_lexicon_tags_17 },
    { 0x8eea4863, SUC_lexicon_tags_16 },
    { 0x644a4865, SUC_lexicon_tags_38 },
    { 0x2b2a4866, SUC_lexicon_tags_30 },
    { 0x49ca4867, SUC_lexicon_tags_26 },
    { 0x976a4868, SUC_lexicon_tags_15 },
    { 0x182a4866, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xf2aa486b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa16a486d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86a4871, SUC_lexicon_tags_49 },
    { 0xa18a4871, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9aa4875, SUC_lexicon_tags_37 },
    { 0xc74a4875, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dea487a, SUC_lexicon_tags_33 },
    { 0x564a487a, SUC_lexicon_tags_52 },
    { 0xa9ca487c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ca4880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6c4a4882, SUC_lexicon_tags_30 },
    { 0x518a4883, SUC_lexicon_tags_0 },
    { 0xd3ea4883, SUC_lexicon_tags_56 },
    { 0xe48a4885, SUC_lexicon_tags_17 },
    { 0x63aa4886, SUC_lexicon_tags_12 },
    { 0xae6a4887, SUC_lexicon_tags_87 },
    { 0x7e4a4888, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2c2a488a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aea4890, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea4894, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x524a4896, SUC_lexicon_tags_2 },
    { 0x6d8a4897, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54ca4899, SUC_lexicon_tags_2 },
    { 0x82a489a, SUC_lexicon_tags_42 },
    { 0x3e0a489a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x734a489e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54a48a2, SUC_lexicon_tags_37 },
    { 0x996a48a2, SUC_lexicon_tags_35 },
    { 0x46a48a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x966a48a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30a48a9, SUC_lexicon_tags_47 },
    { 0xa18a48aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x36aa48ad, SUC_lexicon_tags_38 },
    { 0x374a48ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x986a48b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaca48b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74ea48b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4a48c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666a48c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330a48ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x162a48cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8a48d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74ca48d2, SUC_lexicon_tags_17 },
    { 0xfb0a48d3, SUC_lexicon_tags_2 },
    { 0x7dca48d4, SUC_lexicon_tags_12 },
    { 0x8d2a48d4, SUC_lexicon_tags_1 },
    { 0x70a48d6, SUC_lexicon_tags_14 },
    { 0xedea48d6, SUC_lexicon_tags_26 },
    { 0xdc8a48d5, SUC_lexicon_tags_1 },
    { 0xea0a48d9, SUC_lexicon_tags_11 },
    { 0x3eca48d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xae4a48dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ca48e3, SUC_lexicon_tags_12 },
    { 0x120a48e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a4a48e6, SUC_lexicon_tags_2 },
    { 0x27ea48e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8a48eb, SUC_lexicon_tags_5 },
    { 0x87ca48ec, SUC_lexicon_tags_14 },
    { 0x1b2a48ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe60a48ef, SUC_lexicon_tags_1 },
    { 0xf4ea48f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x444a48f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96a48f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0a48fc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68a4900, SUC_lexicon_tags_12 },
    { 0xa32a4901, SUC_lexicon_tags_43 },
    { 0x312a4901, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4a490a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352a490e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cca4910, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50a4919, SUC_lexicon_tags_15 },
    { 0xa88a491a, SUC_lexicon_tags_1 },
    { 0xa68a491a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6a491f, SUC_lexicon_tags_0 },
    { 0xa54a4920, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xab2a4922, SUC_lexicon_tags_3 },
    { 0xeaaa4923, SUC_lexicon_tags_60 },
    { 0xb2aa4924, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eca492d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x680a492f, SUC_lexicon_tags_9 },
    { 0xb60a4930, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ea4934, SUC_lexicon_tags_61 },
    { 0x11ca4935, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4a4938, SUC_lexicon_tags_63 },
    { 0xdd6a4939, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec2a493b, SUC_lexicon_tags_0 },
    { 0x34ea493c, SUC_lexicon_tags_1 },
    { 0xe20a493d, SUC_lexicon_tags_47 },
    { 0x704a493e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2a4941, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x566a4943, SUC_lexicon_tags_19 },
    { 0xbeea4944, SUC_lexicon_tags_15 },
    { 0x480a4943, SUC_lexicon_tags_17 },
    { 0xb5ea4944, SUC_lexicon_tags_17 },
    { 0x16ca4945, SUC_lexicon_tags_11 },
    { 0x3c8a4948, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ea494e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490a4953, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18a4957, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30a495a, SUC_lexicon_tags_15 },
    { 0xd50a495b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116a4960, SUC_lexicon_tags_10 },
    { 0x45aa4961, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4daa4964, SUC_lexicon_tags_2 },
    { 0xa2ea4964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7aa4967, SUC_lexicon_tags_38 },
    { 0xb4ea4968, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd2a496a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xca8a496c, SUC_lexicon_tags_15 },
    { 0xe72a496c, SUC_lexicon_tags_43 },
    { 0xd74a496e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290a4976, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8a497b, SUC_lexicon_tags_19 },
    { 0xe7ea497b, SUC_lexicon_tags_27 },
    { 0xc5aa497d, SUC_lexicon_tags_2 },
    { 0x686a497b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd6ea4980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34a4984, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780a4988, SUC_lexicon_tags_43 },
    { 0x870a4989, SUC_lexicon_tags_37 },
    { 0x67ca4989, SUC_lexicon_tags_13 },
    { 0x9aea498b, SUC_lexicon_tags_1 },
    { 0x6cea498c, SUC_lexicon_tags_15 },
    { 0x47ea498d, SUC_lexicon_tags_2 },
    { 0x624a498e, SUC_lexicon_tags_60 },
    { 0x270a498f, SUC_lexicon_tags_8 },
    { 0x106a4990, SUC_lexicon_tags_2 },
    { 0x26a4991, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ea4998, SUC_lexicon_tags_26 },
    { 0xca8a4998, SUC_lexicon_tags_23 },
    { 0xdf8a499a, SUC_lexicon_tags_25 },
    { 0xbb6a499b, SUC_lexicon_tags_12 },
    { 0x760a499c, SUC_lexicon_tags_27 },
    { 0xc60a499c, SUC_lexicon_tags_54 },
    { 0xe80a499e, SUC_lexicon_tags_8 },
    { 0xf66a499f, SUC_lexicon_tags_7 },
    { 0x70aa4998, SUC_lexicon_tags_0 },
    { 0xb4ca49a1, SUC_lexicon_tags_1 },
    { 0x58ca49a1, SUC_lexicon_tags_34 },
    { 0x4dca49a2, SUC_lexicon_tags_39 },
    { 0xad4a49a4, SUC_lexicon_tags_42 },
    { 0xe26a49a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6a49aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36a49ad, SUC_lexicon_tags_65 },
    { 0xaeaa49ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d6a49b0, SUC_lexicon_tags_11 },
    { 0x36ca49b1, SUC_lexicon_tags_70 },
    { 0x27aa49b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ca49b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62a49b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0a49be, SUC_lexicon_tags_63 },
    { 0xfaa49bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54a49c2, SUC_lexicon_tags_15 },
    { 0x106a49c2, SUC_lexicon_tags_17 },
    { 0x964a49c4, SUC_lexicon_tags_37 },
    { 0xa7ca49c5, SUC_lexicon_tags_26 },
    { 0xdcea49c6, SUC_lexicon_tags_17 },
    { 0x7f0a49c7, SUC_lexicon_tags_6 },
    { 0xf8ca49c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x958a49ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ea49d2, SUC_lexicon_tags_4 },
    { 0x8c0a49d2, SUC_lexicon_tags_9 },
    { 0x35ea49d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6a0a49d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x368a49d8, SUC_lexicon_tags_19 },
    { 0x3d2a49d9, SUC_lexicon_tags_11 },
    { 0x388a49d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd0a49dc, SUC_lexicon_tags_11 },
    { 0xbdca49dd, SUC_lexicon_tags_63 },
    { 0x90ca49de, SUC_lexicon_tags_44 },
    { 0x28aa49de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472a49e4, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0xdcaa49e6, SUC_lexicon_tags_37 },
    { 0x16aa49e7, SUC_lexicon_tags_0 },
    { 0x5f2a49e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19aa49ea, SUC_lexicon_tags_23 },
    { 0x152a49eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10a49ee, SUC_lexicon_tags_25 },
    { 0xc46a49ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5daa49f3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xba0a49f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0a49f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38a49fc, SUC_lexicon_tags_19 },
    { 0xeaa49fd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc2a49ff, SUC_lexicon_tags_21 },
    { 0x7c4a49ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28a4a06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fca4a0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aca4a10, SUC_lexicon_tags_43 },
    { 0x24ea4a11, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4a15, SUC_lexicon_tags_7 },
    { 0xf1ea4a15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eea4a1b, SUC_lexicon_tags_37 },
    { 0x8e4a4a1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ea4a1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cca4a24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962a4a29, SUC_lexicon_tags_8 },
    { 0x270a4a29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ea4a2e, SUC_lexicon_tags_2 },
    { 0x8bca4a2f, SUC_lexicon_tags_2 },
    { 0x8b6a4a30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabaa4a33, SUC_lexicon_tags_25 },
    { 0xa06a4a34, SUC_lexicon_tags_15 },
    { 0x316a4a35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06a4a3a, SUC_lexicon_tags_38 },
    { 0x9fea4a3a, SUC_lexicon_tags_25 },
    { 0x70aa4a3a, SUC_lexicon_tags_47 },
    { 0xa0ca4a3d, SUC_lexicon_tags_7 },
    { 0x28aa4a3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48a4a45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ca4a48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82a4a4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x526a4a4e, SUC_lexicon_tags_38 },
    { 0x86aa4a4f, SUC_lexicon_tags_17 },
    { 0x7d0a4a4e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2a4a56, SUC_lexicon_tags_17 },
    { 0x77ca4a56, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bca4a5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ea4a5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x928a4a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960a4a67, SUC_lexicon_tags_16 },
    { 0x65ea4a68, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162a4a6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbe2a4a71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x638a4a74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912a4a79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4a4a7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaa4a80, SUC_lexicon_tags_2 },
    { 0xad6a4a81, SUC_lexicon_tags_25 },
    { 0xa80a4a80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4a4a86, SUC_lexicon_tags_23 },
    { 0xc1ea4a86, SUC_lexicon_tags_13 },
    { 0xbb6a4a88, SUC_lexicon_tags_25 },
    { 0x18a4a88, SUC_lexicon_tags_8 },
    { 0xc5ca4a8a, SUC_lexicon_tags_1 },
    { 0x466a4a8b, SUC_lexicon_tags_11 },
    { 0xeeaa4a8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5aa4a91, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4a4a96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32aa4a98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64a4a9b, SUC_lexicon_tags_234 },
    { 0x1f4a4a9b, SUC_lexicon_tags_43 },
    { 0x932a4a9d, SUC_lexicon_tags_15 },
    { 0x178a4a9e, SUC_lexicon_tags_1 },
    { 0x74a4a9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0ea4aa1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x164a4aa3, SUC_lexicon_tags_2 },
    { 0x3e8a4aa4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100a4aa9, SUC_lexicon_tags_5 },
    { 0x332a4aaa, SUC_lexicon_tags_38 },
    { 0x49ca4aaa, SUC_lexicon_tags_14 },
    { 0xd0ea4aac, SUC_lexicon_tags_33 },
    { 0xfeaa4aac, SUC_lexicon_tags_7 },
    { 0x12aa4aaa, SUC_lexicon_tags_43 },
    { 0xa88a4aaf, SUC_lexicon_tags_37 },
    { 0x8bea4ab0, SUC_lexicon_tags_37 },
    { 0x9e6a4ab1, SUC_lexicon_tags_31 },
    { 0x90aa4ab2, SUC_lexicon_tags_11 },
    { 0x1e8a4ab2, SUC_lexicon_tags_66 },
    { 0x59ca4aac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74a4abd, SUC_lexicon_tags_33 },
    { 0x14aa4abe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ea4ac4, SUC_lexicon_tags_23 },
    { 0xdeea4ac5, SUC_lexicon_tags_14 },
    { 0xa88a4ac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fca4ac8, SUC_lexicon_tags_27 },
    { 0x9e6a4ac9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2a4acd, SUC_lexicon_tags_1 },
    { 0xf6ca4ace, SUC_lexicon_tags_44 },
    { 0xc6ea4acf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb1aa4ad1, SUC_lexicon_tags_15 },
    { 0xde2a4ad2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43aa4ad4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb24a4ad6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ca4ada, SUC_lexicon_tags_30 },
    { 0xbe8a4adb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x250a4add, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ca4ae0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6a4ae4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9ca4ae6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22aa4af1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7aea4af3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ca4af6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460a4afa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca4aff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaa4b03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32a4b0c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedea4b10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8a4b14, SUC_lexicon_tags_12 },
    { 0xa8ea4b15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4a4b1b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x79aa4b1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaa4b21, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x41aa4b23, SUC_lexicon_tags_49 },
    { 0xc28a4b24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ca4b27, SUC_lexicon_tags_37 },
    { 0x3aa4b28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54a4b2a, SUC_lexicon_tags_49 },
    { 0xcd0a4b2b, SUC_lexicon_tags_1 },
    { 0x2c4a4b2c, SUC_lexicon_tags_2 },
    { 0x23aa4b2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862a4b31, SUC_lexicon_tags_43 },
    { 0xcb6a4b32, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x772a4b34, SUC_lexicon_tags_11 },
    { 0x93ca4b35, SUC_lexicon_tags_87 },
    { 0xf02a4b36, SUC_lexicon_tags_33 },
    { 0x6daa4b35, SUC_lexicon_tags_19 },
    { 0xc88a4b38, SUC_lexicon_tags_4 },
    { 0x964a4b36, SUC_lexicon_tags_28 },
    { 0x432a4b3a, SUC_lexicon_tags_31 },
    { 0x4d0a4b3b, SUC_lexicon_tags_0 },
    { 0x768a4b34, SUC_lexicon_tags_63 },
    { 0xfcea4b3a, SUC_lexicon_tags_27 },
    { 0x7cca4b3e, SUC_lexicon_tags_32 },
    { 0x750a4b3e, SUC_lexicon_tags_12 },
    { 0xae6a4b3e, SUC_lexicon_tags_47 },
    { 0xe94a4b41, SUC_lexicon_tags_0 },
    { 0x2eaa4b42, SUC_lexicon_tags_37 },
    { 0x504a4b3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed4a4b45, SUC_lexicon_tags_15 },
    { 0xb7ca4b46, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5d6a4b48, SUC_lexicon_tags_77 },
    { 0xd5ea4b48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72a4b4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6caa4b55, SUC_lexicon_tags_37 },
    { 0x43aa4b56, SUC_lexicon_tags_68 },
    { 0xc46a4b55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42ea4b59, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x46ea4b5b, SUC_lexicon_tags_15 },
    { 0x3f6a4b5c, SUC_lexicon_tags_37 },
    { 0x8bca4b5d, SUC_lexicon_tags_0 },
    { 0x95ea4b5e, SUC_lexicon_tags_2 },
    { 0x9fea4b5f, SUC_lexicon_tags_11 },
    { 0x278a4b5f, SUC_lexicon_tags_37 },
    { 0xdcea4b61, SUC_lexicon_tags_17 },
    { 0x1aea4b5c, SUC_lexicon_tags_37 },
    { 0x398a4b63, SUC_lexicon_tags_13 },
    { 0x798a4b60, SUC_lexicon_tags_60 },
    { 0xaeea4b64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ea4b6c, SUC_lexicon_tags_2 },
    { 0x9dca4b6d, SUC_lexicon_tags_0 },
    { 0xf2aa4b6d, SUC_lexicon_tags_16 },
    { 0x110a4b6e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x376a4b71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x240a4b73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92a4b79, SUC_lexicon_tags_48 },
    { 0x478a4b79, SUC_lexicon_tags_11 },
    { 0x3f0a4b7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b2a4b7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ca4b80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12a4b8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e8a4b8d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8a4b91, SUC_lexicon_tags_14 },
    { 0xca4a4b91, SUC_lexicon_tags_22 },
    { 0x78a4b92, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x186a4b95, SUC_lexicon_tags_27 },
    { 0xa44a4b96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aa4b99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb26a4b9b, SUC_lexicon_tags_11 },
    { 0x940a4b9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0a4ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78a4ba3, SUC_lexicon_tags_37 },
    { 0x894a4ba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85aa4ba7, SUC_lexicon_tags_14 },
    { 0x3faa4ba7, SUC_lexicon_tags_37 },
    { 0xcaa4ba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x920a4bac, SUC_lexicon_tags_7 },
    { 0x5eaa4bad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73aa4bb0, SUC_lexicon_tags_63 },
    { 0x18ea4bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62a4bb8, SUC_lexicon_tags_7 },
    { 0x47aa4bb9, SUC_lexicon_tags_0 },
    { 0x374a4bba, SUC_lexicon_tags_16 },
    { 0x8d6a4bba, SUC_lexicon_tags_19 },
    { 0xb8a4bbc, SUC_lexicon_tags_49 },
    { 0x264a4bbb, SUC_lexicon_tags_108 },
    { 0xd8ea4bb8, SUC_lexicon_tags_133 },
    { 0x6faa4bbf, SUC_lexicon_tags_17 },
    { 0x1daa4bba, SUC_lexicon_tags_0 },
    { 0x48ca4bc1, SUC_lexicon_tags_92 },
    { 0x8a2a4bc2, SUC_lexicon_tags_15 },
    { 0xb70a4bc3, SUC_lexicon_tags_8 },
    { 0x400a4bc1, SUC_lexicon_tags_37 },
    { 0x1d2a4bc4, SUC_lexicon_tags_8 },
    { 0xd08a4bb8, SUC_lexicon_tags_13 },
    { 0xc5ea4bb9, SUC_lexicon_tags_35 },
    { 0xde4a4bc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x456a4bca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58ca4bcc, SUC_lexicon_tags_49 },
    { 0xde8a4bcd, SUC_lexicon_tags_35 },
    { 0x448a4bcd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2a4bd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ea4bd8, SUC_lexicon_tags_31 },
    { 0x1aaa4bd9, SUC_lexicon_tags_7 },
    { 0xbbaa4bd9, SUC_lexicon_tags_12 },
    { 0x536a4bdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b4a4bdd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a4be0, SUC_lexicon_tags_29 },
    { 0x444a4be1, SUC_lexicon_tags_51 },
    { 0xc80a4be2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaa4be7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2a4bec, SUC_lexicon_tags_65 },
    { 0x604a4bed, SUC_lexicon_tags_17 },
    { 0x4eaa4bed, SUC_lexicon_tags_27 },
    { 0x6baa4bef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aaa4bf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68a4bfa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xad2a4bfc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2baa4c06, SUC_lexicon_tags_37 },
    { 0x3cea4c06, SUC_lexicon_tags_10 },
    { 0x238a4c08, SUC_lexicon_tags_38 },
    { 0xd08a4c06, SUC_lexicon_tags_6 },
    { 0x63ea4c0a, SUC_lexicon_tags_25 },
    { 0x96ca4c0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320a4c13, SUC_lexicon_tags_12 },
    { 0x928a4c13, SUC_lexicon_tags_14 },
    { 0x1cca4c15, SUC_lexicon_tags_17 },
    { 0x6b2a4c16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x742a4c18, SUC_lexicon_tags_39 },
    { 0x324a4c18, SUC_lexicon_tags_4 },
    { 0xf66a4c1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8a4c22, SUC_lexicon_tags_9 },
    { 0x39ea4c22, SUC_lexicon_tags_2 },
    { 0xe42a4c24, SUC_lexicon_tags_10 },
    { 0xe60a4c25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe28a4c27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984a4c2b, SUC_lexicon_tags_15 },
    { 0x51ea4c2c, SUC_lexicon_tags_1 },
    { 0xee8a4c2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea4c34, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8a4c38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd4a4c3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06a4c43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ca4c47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52a4c50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57ea4c52, SUC_lexicon_tags_17 },
    { 0xcd6a4c53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x93ea4c55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6a4c5a, SUC_lexicon_tags_26 },
    { 0xcd0a4c5b, SUC_lexicon_tags_37 },
    { 0x86ea4c5c, SUC_lexicon_tags_2 },
    { 0x4bea4c5d, SUC_lexicon_tags_37 },
    { 0xe66a4c5e, SUC_lexicon_tags_1 },
    { 0xc8aa4c5b, SUC_lexicon_tags_23 },
    { 0xef4a4c60, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaa4c63, SUC_lexicon_tags_48 },
    { 0xb26a4c64, SUC_lexicon_tags_28 },
    { 0x812a4c64, SUC_lexicon_tags_11 },
    { 0x64a4c63, SUC_lexicon_tags_55 },
    { 0x714a4c67, SUC_lexicon_tags_11 },
    { 0x6b4a4c68, SUC_lexicon_tags_7 },
    { 0x460a4c68, SUC_lexicon_tags_7 },
    { 0x5b6a4c68, SUC_lexicon_tags_12 },
    { 0x5f0a4c69, SUC_lexicon_tags_25 },
    { 0xfb4a4c6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x73ea4c6e, SUC_lexicon_tags_63 },
    { 0x2fea4c6f, SUC_lexicon_tags_12 },
    { 0xc4ca4c70, SUC_lexicon_tags_275 },
    { 0xd9ea4c70, SUC_lexicon_tags_1 },
    { 0x60ea4c6f, SUC_lexicon_tags_87 },
    { 0xe92a4c6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8ea4c75, SUC_lexicon_tags_2 },
    { 0x786a4c75, SUC_lexicon_tags_15 },
    { 0xe7ca4c77, SUC_lexicon_tags_11 },
    { 0x8ea4c78, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2a4c7d, SUC_lexicon_tags_11 },
    { 0xfaea4c7e, SUC_lexicon_tags_25 },
    { 0x836a4c7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x236a4c82, SUC_lexicon_tags_7 },
    { 0x54ea4c83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e4a4c85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca4c88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ca4c8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8a4c8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5caa4c91, SUC_lexicon_tags_1 },
    { 0xb0aa4c91, SUC_lexicon_tags_25 },
    { 0x1a2a4c93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeeaa4c95, SUC_lexicon_tags_48 },
    { 0x692a4c96, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ca4c9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74a4ca1, SUC_lexicon_tags_25 },
    { 0x168a4ca2, SUC_lexicon_tags_2 },
    { 0x5c0a4ca3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca4ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cca4ca9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeea4cad, SUC_lexicon_tags_60 },
    { 0x540a4cad, SUC_lexicon_tags_0 },
    { 0x8f2a4caf, SUC_lexicon_tags_60 },
    { 0xea0a4cb0, SUC_lexicon_tags_2 },
    { 0x8cea4caf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34a4cb5, SUC_lexicon_tags_19 },
    { 0x386a4cb6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b4a4cb8, SUC_lexicon_tags_37 },
    { 0xa82a4cb9, SUC_lexicon_tags_8 },
    { 0x5d6a4cb9, SUC_lexicon_tags_37 },
    { 0x952a4cb9, SUC_lexicon_tags_25 },
    { 0x2eea4cb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d4a4cbe, SUC_lexicon_tags_11 },
    { 0x3f6a4cbf, SUC_lexicon_tags_23 },
    { 0xf2aa4cc0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ea4cc5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x86aa4cc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47aa4cc9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562a4cce, SUC_lexicon_tags_48 },
    { 0xf36a4ccf, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xa8ca4cd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592a4cd5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x298a4cd8, SUC_lexicon_tags_17 },
    { 0xe7aa4cd8, SUC_lexicon_tags_12 },
    { 0x236a4cda, SUC_lexicon_tags_34 },
    { 0x80ea4cdb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x250a4cde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38a4ce1, SUC_lexicon_tags_15 },
    { 0xf6a4ce2, SUC_lexicon_tags_11 },
    { 0x25ca4ce3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268a4cee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13aa4cf2, SUC_lexicon_tags_46 },
    { 0x9e8a4cf3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedaa4cf8, SUC_lexicon_tags_23 },
    { 0xd42a4cf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aa4d00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960a4d05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x788a4d07, SUC_lexicon_tags_6 },
    { 0x592a4d07, SUC_lexicon_tags_15 },
    { 0x85aa4d09, SUC_lexicon_tags_11 },
    { 0xf32a4d0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x220a4d0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ea4d0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ea4d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ea4d15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ea4d1e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x754a4d21, SUC_lexicon_tags_17 },
    { 0x876a4d22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9caa4d24, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76a4d27, SUC_lexicon_tags_6 },
    { 0x2d0a4d28, SUC_lexicon_tags_7 },
    { 0x26ca4d29, SUC_lexicon_tags_19 },
    { 0xb2ca4d2a, SUC_lexicon_tags_15 },
    { 0x8a8a4d29, SUC_lexicon_tags_17 },
    { 0xc6ea4d2b, SUC_lexicon_tags_6 },
    { 0xdfea4d29, SUC_lexicon_tags_5 },
    { 0xe26a4d2d, SUC_lexicon_tags_11 },
    { 0x884a4d2d, SUC_lexicon_tags_17 },
    { 0xea6a4d2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12a4d33, SUC_lexicon_tags_29 },
    { 0xfa4a4d34, SUC_lexicon_tags_79 },
    { 0xda8a4d34, SUC_lexicon_tags_25 },
    { 0xce4a4d36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x918a4d38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ea4d3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4d41, SUC_lexicon_tags_12 },
    { 0x286a4d41, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ea4d47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccea4d50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2a4d53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33aa4d58, SUC_lexicon_tags_15 },
    { 0x10ea4d59, SUC_lexicon_tags_47 },
    { 0xa2a4d5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe8a4d5c, SUC_lexicon_tags_25 },
    { 0x62aa4d5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9eca4d5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41aa4d63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xffca4d65, SUC_lexicon_tags_25 },
    { 0xa10a4d65, SUC_lexicon_tags_29 },
    { 0x7cea4d66, SUC_lexicon_tags_12 },
    { 0x518a4d68, SUC_lexicon_tags_37 },
    { 0x124a4d68, SUC_lexicon_tags_17 },
    { 0x8baa4d68, SUC_lexicon_tags_38 },
    { 0x830a4d69, SUC_lexicon_tags_33 },
    { 0x82ea4d66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faa4d70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaea4d72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08a4d78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3eea4d7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7aa4d7c, SUC_lexicon_tags_12 },
    { 0x23ca4d7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8a4d81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806a4d8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd46a4d8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaea4d93, SUC_lexicon_tags_12 },
    { 0x906a4d94, SUC_lexicon_tags_27 },
    { 0x9a8a4d95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4d98, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80a4d9d, SUC_lexicon_tags_33 },
    { 0x2e2a4d9d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aa4da1, SUC_lexicon_tags_25 },
    { 0x2cea4da1, SUC_lexicon_tags_9 },
    { 0xfea4da2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa2ea4da5, SUC_lexicon_tags_15 },
    { 0xba6a4da6, SUC_lexicon_tags_14 },
    { 0x714a4da7, SUC_lexicon_tags_17 },
    { 0x4c8a4da7, SUC_lexicon_tags_39 },
    { 0x1daa4da9, SUC_lexicon_tags_10 },
    { 0xc56a4da7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9e8a4dac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668a4db0, SUC_lexicon_tags_17 },
    { 0x52aa4db0, SUC_lexicon_tags_11 },
    { 0x4aca4db1, SUC_lexicon_tags_37 },
    { 0x3a8a4db3, SUC_lexicon_tags_25 },
    { 0xfaa4db4, SUC_lexicon_tags_37 },
    { 0x984a4db5, SUC_lexicon_tags_60 },
    { 0x40ea4db5, SUC_lexicon_tags_7 },
    { 0x116a4db4, SUC_lexicon_tags_25 },
    { 0x5d4a4db8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x636a4dba, SUC_lexicon_tags_25 },
    { 0xd04a4dba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaaa4dbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930a4dc1, SUC_lexicon_tags_9 },
    { 0x8baa4dc2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ea4dc6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2aa4dc9, SUC_lexicon_tags_65 },
    { 0xa8a4dca, SUC_lexicon_tags_2 },
    { 0x7f2a4dc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecea4dcf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6a4dd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318a4ddd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ea4de6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb90a4de8, SUC_lexicon_tags_19 },
    { 0x2aea4de9, SUC_lexicon_tags_7 },
    { 0x826a4dea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x40ca4dec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9aa4def, SUC_lexicon_tags_37 },
    { 0x4cea4df0, SUC_lexicon_tags_7 },
    { 0x6dca4def, SUC_lexicon_tags_2 },
    { 0x4a2a4df2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32a4df9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4a4dfc, SUC_lexicon_tags_54 },
    { 0xa14a4dfc, SUC_lexicon_tags_37 },
    { 0x2dca4dfe, SUC_lexicon_tags_33 },
    { 0xd82a4dfe, SUC_lexicon_tags_2 },
    { 0xde8a4e00, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcca4e04, SUC_lexicon_tags_27 },
    { 0x598a4e04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81ca4e07, SUC_lexicon_tags_12 },
    { 0xa92a4e08, SUC_lexicon_tags_25 },
    { 0x27ea4e09, SUC_lexicon_tags_49 },
    { 0xcf8a4e07, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaa4e0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508a4e11, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ea4e14, SUC_lexicon_tags_28 },
    { 0x204a4e15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966a4e1b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344a4e20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e2a4e22, SUC_lexicon_tags_14 },
    { 0x520a4e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x940a4e26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4a4e29, SUC_lexicon_tags_0 },
    { 0x40a4e2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78a4e2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12a4e37, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x87ea4e39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ca4e3c, SUC_lexicon_tags_1 },
    { 0x996a4e3c, SUC_lexicon_tags_25 },
    { 0xc7ea4e3d, SUC_lexicon_tags_63 },
    { 0x930a4e3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x868a4e42, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x726a4e45, SUC_lexicon_tags_36 },
    { 0x548a4e45, SUC_lexicon_tags_2 },
    { 0x560a4e46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca4e4f, SUC_lexicon_tags_87 },
    { 0x666a4e50, SUC_lexicon_tags_29 },
    { 0xc0a4e51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c2a4e53, SUC_lexicon_tags_15 },
    { 0x990a4e54, SUC_lexicon_tags_4 },
    { 0x92ca4e55, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ca4e58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ca4e5b, SUC_lexicon_tags_66 },
    { 0x7eaa4e5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe14a4e5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0a4e66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd00a4e68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ca4e6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5f8a4e6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8a4e71, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4a4e75, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5c8a4e77, SUC_lexicon_tags_7 },
    { 0xe36a4e78, SUC_lexicon_tags_17 },
    { 0xb9ea4e78, SUC_lexicon_tags_43 },
    { 0x61aa4e7a, SUC_lexicon_tags_25 },
    { 0x9eea4e78, SUC_lexicon_tags_82 },
    { 0xe74a4e7c, SUC_lexicon_tags_87 },
    { 0xe6ea4e7d, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0xb6aa4e7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2faa4e86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6a4e8d, SUC_lexicon_tags_48 },
    { 0x7c2a4e8e, SUC_lexicon_tags_2 },
    { 0x64aa4e8f, SUC_lexicon_tags_6 },
    { 0xbe0a4e90, SUC_lexicon_tags_2 },
    { 0x2a2a4e8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9ca4e93, SUC_lexicon_tags_565 },
    { 0, NULL },
    { 0xa46a4e95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2c4a4e97, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bca4e9a, SUC_lexicon_tags_23 },
    { 0x9aca4e9b, SUC_lexicon_tags_23 },
    { 0x94aa4e9c, SUC_lexicon_tags_28 },
    { 0x7aca4e9d, SUC_lexicon_tags_19 },
    { 0xe8ca4e9e, SUC_lexicon_tags_33 },
    { 0x782a4e9a, SUC_lexicon_tags_17 },
    { 0x9fca4ea0, SUC_lexicon_tags_7 },
    { 0xad0a4ea1, SUC_lexicon_tags_79 },
    { 0x846a4ea2, SUC_lexicon_tags_7 },
    { 0x78aa4ea2, SUC_lexicon_tags_25 },
    { 0x438a4ea4, SUC_lexicon_tags_7 },
    { 0xcb8a4ea4, SUC_lexicon_tags_32 },
    { 0x29ea4ea6, SUC_lexicon_tags_19 },
    { 0x176a4ea7, SUC_lexicon_tags_40 },
    { 0x8a4a4ea6, SUC_lexicon_tags_25 },
    { 0x34ca4ea9, SUC_lexicon_tags_178 },
    { 0x54aa4ea8, SUC_lexicon_tags_19 },
    { 0xf98a4ea9, SUC_lexicon_tags_12 },
    { 0xaf6a4ea8, SUC_lexicon_tags_2 },
    { 0xd74a4eac, SUC_lexicon_tags_15 },
    { 0xd44a4ea9, SUC_lexicon_tags_10 },
    { 0x50ca4eaf, SUC_lexicon_tags_16 },
    { 0x396a4ead, SUC_lexicon_tags_0 },
    { 0x626a4ea7, SUC_lexicon_tags_62 },
    { 0x2d0a4ea7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe4ca4eb4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4e2a4eb6, SUC_lexicon_tags_19 },
    { 0xf28a4eb6, SUC_lexicon_tags_38 },
    { 0xd3ca4eb8, SUC_lexicon_tags_138 },
    { 0xb8a4eb9, SUC_lexicon_tags_19 },
    { 0xe2ca4eb8, SUC_lexicon_tags_37 },
    { 0xe34a4eba, SUC_lexicon_tags_28 },
    { 0x352a4ebc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ca4ec7, SUC_lexicon_tags_47 },
    { 0x6fea4ec8, SUC_lexicon_tags_47 },
    { 0xe7ea4ec9, SUC_lexicon_tags_38 },
    { 0x700a4eca, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xdbca4ecc, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0x10ea4ece, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392a4ed2, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2a4ed9, SUC_lexicon_tags_5 },
    { 0x3a0a4eda, SUC_lexicon_tags_18 },
    { 0x9fca4edb, SUC_lexicon_tags_47 },
    { 0xb94a4edc, SUC_lexicon_tags_5 },
    { 0xb06a4edd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb02a4edf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4aca4ee1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ea4ee4, SUC_lexicon_tags_17 },
    { 0xe74a4ee5, SUC_lexicon_tags_47 },
    { 0xdaca4ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12ca4ee8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12ca4eea, SUC_lexicon_tags_25 },
    { 0xbe2a4eea, SUC_lexicon_tags_55 },
    { 0x5cca4eec, SUC_lexicon_tags_15 },
    { 0x190a4eea, SUC_lexicon_tags_38 },
    { 0x7c8a4eee, SUC_lexicon_tags_26 },
    { 0xbbea4eeb, SUC_lexicon_tags_35 },
    { 0xa98a4eea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ca4ef6, SUC_lexicon_tags_42 },
    { 0x2e0a4ef6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x606a4ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8ca4efb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baa4f02, SUC_lexicon_tags_49 },
    { 0xce0a4f03, SUC_lexicon_tags_39 },
    { 0xd3ca4f03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ca4f09, SUC_lexicon_tags_15 },
    { 0x800a4f0a, SUC_lexicon_tags_29 },
    { 0xda8a4f0a, SUC_lexicon_tags_0 },
    { 0x39ea4f0c, SUC_lexicon_tags_10 },
    { 0x94aa4f0d, SUC_lexicon_tags_80 },
    { 0x478a4f0e, SUC_lexicon_tags_38 },
    { 0x962a4f0d, SUC_lexicon_tags_37 },
    { 0xab0a4f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f2a4f12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b0a4f14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe3ea4f16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x178a4f18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea0a4f1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0a4f21, SUC_lexicon_tags_11 },
    { 0xceca4f22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xee2a4f24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ca4f27, SUC_lexicon_tags_15 },
    { 0xbe6a4f28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x840a4f2a, SUC_lexicon_tags_25 },
    { 0x746a4f2a, SUC_lexicon_tags_79 },
    { 0x48a4f2c, SUC_lexicon_tags_12 },
    { 0xd5aa4f2c, SUC_lexicon_tags_60 },
    { 0x18ca4f2e, SUC_lexicon_tags_45 },
    { 0xa28a4f2f, SUC_lexicon_tags_25 },
    { 0xd24a4f2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf8a4f32, SUC_lexicon_tags_11 },
    { 0xf0a4f33, SUC_lexicon_tags_12 },
    { 0xd7aa4f34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd56a4f36, SUC_lexicon_tags_28 },
    { 0x52ea4f37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42a4f39, SUC_lexicon_tags_25 },
    { 0xbcaa4f3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x80ca4f3c, SUC_lexicon_tags_65 },
    { 0x230a4f3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f6a4f3f, SUC_lexicon_tags_27 },
    { 0x34ca4f40, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x130a4f42, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474a4f4b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8aa4f55, SUC_lexicon_tags_21 },
    { 0x13ca4f56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62a4f5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x550a4f5f, SUC_lexicon_tags_31 },
    { 0x85ca4f60, SUC_lexicon_tags_7 },
    { 0x240a4f60, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ca4f65, SUC_lexicon_tags_11 },
    { 0x944a4f66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1bca4f68, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4a4f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x396a4f6e, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x21ca4f70, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x644a4f72, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0xa84a4f74, SUC_lexicon_tags_12 },
    { 0x564a4f74, SUC_lexicon_tags_25 },
    { 0x140a4f76, SUC_lexicon_tags_55 },
    { 0xd84a4f77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ea4f7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aa4f82, SUC_lexicon_tags_34 },
    { 0xeaca4f83, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ca4f87, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ca4f8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aa4f8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ea4f91, SUC_lexicon_tags_15 },
    { 0x668a4f91, SUC_lexicon_tags_43 },
    { 0x400a4f93, SUC_lexicon_tags_38 },
    { 0xa80a4f94, SUC_lexicon_tags_7 },
    { 0x8c2a4f95, SUC_lexicon_tags_1 },
    { 0x9b0a4f96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd94a4f98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4a4f9a, SUC_lexicon_tags_40 },
    { 0x57ea4f9b, SUC_lexicon_tags_79 },
    { 0x9eaa4f9c, SUC_lexicon_tags_11 },
    { 0xcb4a4f9d, SUC_lexicon_tags_65 },
    { 0xd50a4f9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aa4fa1, SUC_lexicon_tags_15 },
    { 0xe26a4fa2, SUC_lexicon_tags_30 },
    { 0x7b0a4fa1, SUC_lexicon_tags_14 },
    { 0x124a4fa4, SUC_lexicon_tags_46 },
    { 0x904a4fa5, SUC_lexicon_tags_47 },
    { 0x2faa4fa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf2a4fa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x268a4faa, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xcaa4fac, SUC_lexicon_tags_2 },
    { 0x17ea4fad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b0a4faf, SUC_lexicon_tags_5 },
    { 0xb82a4fb0, SUC_lexicon_tags_56 },
    { 0xdf8a4fb1, SUC_lexicon_tags_14 },
    { 0xfaca4fb1, SUC_lexicon_tags_42 },
    { 0xf00a4fb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352a4fb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154a4fbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ea4fc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd8a4fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3aa4fc8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4a4fd0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea4fd8, SUC_lexicon_tags_11 },
    { 0x8faa4fd8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x170a4fdb, SUC_lexicon_tags_75 },
    { 0x944a4fdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1daa4fde, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x606a4fe0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x606a4fe3, SUC_lexicon_tags_11 },
    { 0xbe0a4fe4, SUC_lexicon_tags_11 },
    { 0x8daa4fe4, SUC_lexicon_tags_25 },
    { 0xaf4a4fe6, SUC_lexicon_tags_45 },
    { 0x226a4fe7, SUC_lexicon_tags_17 },
    { 0x60ea4fe8, SUC_lexicon_tags_37 },
    { 0x3bea4fe9, SUC_lexicon_tags_10 },
    { 0x920a4fe9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd62a4fec, SUC_lexicon_tags_25 },
    { 0x3d6a4fec, SUC_lexicon_tags_37 },
    { 0xd50a4fed, SUC_lexicon_tags_2 },
    { 0x49aa4fef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaa4ff3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6a4ff6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd14a4ff8, SUC_lexicon_tags_30 },
    { 0x6daa4ff9, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8a4ffe, SUC_lexicon_tags_2 },
    { 0x1baa4ffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbca5004, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa96a5006, SUC_lexicon_tags_28 },
    { 0x86ca5007, SUC_lexicon_tags_2 },
    { 0x8cea5008, SUC_lexicon_tags_17 },
    { 0x334a5008, SUC_lexicon_tags_11 },
    { 0x71ea5008, SUC_lexicon_tags_14 },
    { 0x6f4a500b, SUC_lexicon_tags_63 },
    { 0x1eaa5008, SUC_lexicon_tags_45 },
    { 0x8faa500d, SUC_lexicon_tags_60 },
    { 0x206a5008, SUC_lexicon_tags_37 },
    { 0xf84a500f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a5017, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2a501a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51aa5020, SUC_lexicon_tags_15 },
    { 0x2e6a5021, SUC_lexicon_tags_25 },
    { 0x5c0a5020, SUC_lexicon_tags_25 },
    { 0x3b2a5022, SUC_lexicon_tags_27 },
    { 0xe20a5024, SUC_lexicon_tags_68 },
    { 0xdf2a5024, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2a502a, SUC_lexicon_tags_2 },
    { 0x846a502a, SUC_lexicon_tags_47 },
    { 0x2b6a502a, SUC_lexicon_tags_36 },
    { 0xcaaa502d, SUC_lexicon_tags_17 },
    { 0x37aa502a, SUC_lexicon_tags_37 },
    { 0x392a502d, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56a5032, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc38a5034, SUC_lexicon_tags_27 },
    { 0xd3ca5035, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84aa5037, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x936a503a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ca503d, SUC_lexicon_tags_8 },
    { 0x824a503e, SUC_lexicon_tags_17 },
    { 0xed0a503e, SUC_lexicon_tags_25 },
    { 0x688a5040, SUC_lexicon_tags_15 },
    { 0xdbca503e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ea5044, SUC_lexicon_tags_37 },
    { 0xb74a5045, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x866a5048, SUC_lexicon_tags_9 },
    { 0x6c6a5049, SUC_lexicon_tags_14 },
    { 0xf58a5049, SUC_lexicon_tags_14 },
    { 0xe54a504b, SUC_lexicon_tags_11 },
    { 0x77ca504b, SUC_lexicon_tags_61 },
    { 0xc40a5048, SUC_lexicon_tags_124 },
    { 0x162a5048, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaa5053, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ea5057, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6a505f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca5063, SUC_lexicon_tags_17 },
    { 0x20aa5063, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08a506a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8a506e, SUC_lexicon_tags_43 },
    { 0x28ea506e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x998a5072, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8a5076, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6a5079, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcea507d, SUC_lexicon_tags_2 },
    { 0x9d8a507e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ca5086, SUC_lexicon_tags_27 },
    { 0xa78a5087, SUC_lexicon_tags_38 },
    { 0x62aa5088, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd84a508a, SUC_lexicon_tags_45 },
    { 0x878a508b, SUC_lexicon_tags_7 },
    { 0x142a508c, SUC_lexicon_tags_37 },
    { 0x680a508d, SUC_lexicon_tags_63 },
    { 0x1c4a508e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7d8a5090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfeea5092, SUC_lexicon_tags_17 },
    { 0x360a5093, SUC_lexicon_tags_28 },
    { 0xd5aa5094, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8a509c, SUC_lexicon_tags_7 },
    { 0x506a509d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2a50a0, SUC_lexicon_tags_53 },
    { 0x646a50a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bea50a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4a50a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4ea50ab, SUC_lexicon_tags_4 },
    { 0xb54a50ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d4a50ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0a50b1, SUC_lexicon_tags_65 },
    { 0xe44a50b2, SUC_lexicon_tags_28 },
    { 0xc4aa50b1, SUC_lexicon_tags_68 },
    { 0xe0aa50b4, SUC_lexicon_tags_21 },
    { 0xf72a50b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaa50b8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x712a50ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4ea50bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x848a50bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cea50c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x662a50c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ca50ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5faa50d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55aa50db, SUC_lexicon_tags_7 },
    { 0x36ca50dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7aa50de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a50e3, SUC_lexicon_tags_33 },
    { 0xbd6a50e3, SUC_lexicon_tags_1 },
    { 0x4aaa50e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10a50ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448a50ee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2a50f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ca50f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23aa50fc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x584a50fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x326a5100, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaa5104, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880a5109, SUC_lexicon_tags_17 },
    { 0xca4a510a, SUC_lexicon_tags_47 },
    { 0x236a5109, SUC_lexicon_tags_37 },
    { 0x1dea510c, SUC_lexicon_tags_47 },
    { 0x104a510d, SUC_lexicon_tags_6 },
    { 0xcf0a510b, SUC_lexicon_tags_25 },
    { 0x6d6a510f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eca5113, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61ea5115, SUC_lexicon_tags_25 },
    { 0x512a5116, SUC_lexicon_tags_1 },
    { 0x8f0a5117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faa511b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6a511e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9aa5120, SUC_lexicon_tags_92 },
    { 0xbcca5121, SUC_lexicon_tags_29 },
    { 0xc6a5121, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6a5127, SUC_lexicon_tags_17 },
    { 0xc06a5127, SUC_lexicon_tags_17 },
    { 0x710a5127, SUC_lexicon_tags_2 },
    { 0x28ca512a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ca512d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ea5130, SUC_lexicon_tags_47 },
    { 0xb38a5131, SUC_lexicon_tags_32 },
    { 0x194a5131, SUC_lexicon_tags_15 },
    { 0x2eca5133, SUC_lexicon_tags_49 },
    { 0xc84a5133, SUC_lexicon_tags_25 },
    { 0xf2a5133, SUC_lexicon_tags_25 },
    { 0x920a5135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54a5139, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2a513c, SUC_lexicon_tags_14 },
    { 0xd9aa513c, SUC_lexicon_tags_65 },
    { 0x6c6a513e, SUC_lexicon_tags_37 },
    { 0x6a2a513f, SUC_lexicon_tags_12 },
    { 0xc28a513f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44a5143, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67ca5145, SUC_lexicon_tags_16 },
    { 0x508a5145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7ca5148, SUC_lexicon_tags_3 },
    { 0xcb0a5148, SUC_lexicon_tags_62 },
    { 0xe56a5149, SUC_lexicon_tags_1616 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ea514d, SUC_lexicon_tags_37 },
    { 0x122a514d, SUC_lexicon_tags_47 },
    { 0x2eaa514f, SUC_lexicon_tags_17 },
    { 0xcb0a514e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x460a5153, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6aa5156, SUC_lexicon_tags_37 },
    { 0x134a5157, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x68a5159, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aa5160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x236a5162, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2a5165, SUC_lexicon_tags_1 },
    { 0x962a5165, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16a516a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x662a516c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136a5170, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978a5174, SUC_lexicon_tags_2 },
    { 0x3d4a5174, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2a517a, SUC_lexicon_tags_65 },
    { 0x554a517b, SUC_lexicon_tags_12 },
    { 0x4bea517c, SUC_lexicon_tags_11 },
    { 0x258a517d, SUC_lexicon_tags_37 },
    { 0x22aa517e, SUC_lexicon_tags_65 },
    { 0xb7aa517f, SUC_lexicon_tags_2 },
    { 0xddca517e, SUC_lexicon_tags_39 },
    { 0x400a5181, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cea5186, SUC_lexicon_tags_14 },
    { 0xaeea5187, SUC_lexicon_tags_14 },
    { 0xfcea5186, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd50a518a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7ea518c, SUC_lexicon_tags_1 },
    { 0xa90a518d, SUC_lexicon_tags_105 },
    { 0x4e8a518e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x492a5191, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x600a5193, SUC_lexicon_tags_27 },
    { 0x436a5194, SUC_lexicon_tags_29 },
    { 0x2bea5195, SUC_lexicon_tags_16 },
    { 0xa1ea5196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2eea5198, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ca51a0, SUC_lexicon_tags_17 },
    { 0x280a51a0, SUC_lexicon_tags_33 },
    { 0x40ea51a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dea51a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268a51ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6a51b1, SUC_lexicon_tags_17 },
    { 0x8c2a51b2, SUC_lexicon_tags_12 },
    { 0xc5ea51b1, SUC_lexicon_tags_5 },
    { 0x25aa51b2, SUC_lexicon_tags_11 },
    { 0xa4aa51b3, SUC_lexicon_tags_23 },
    { 0x52ea51b1, SUC_lexicon_tags_17 },
    { 0x946a51b4, SUC_lexicon_tags_12 },
    { 0xadca51b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57aa51bc, SUC_lexicon_tags_84 },
    { 0xd3ea51bd, SUC_lexicon_tags_12 },
    { 0x254a51be, SUC_lexicon_tags_10 },
    { 0xf60a51be, SUC_lexicon_tags_6 },
    { 0xe32a51c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x566a51c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74a51c6, SUC_lexicon_tags_4 },
    { 0xbdaa51c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1c0a51c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4a51cd, SUC_lexicon_tags_34 },
    { 0xf30a51ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250a51d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ca51d9, SUC_lexicon_tags_1 },
    { 0x7f8a51da, SUC_lexicon_tags_185 },
    { 0x786a51d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x218a51de, SUC_lexicon_tags_19 },
    { 0x782a51df, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xbb8a51e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x82ea51e3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf8ea51e5, SUC_lexicon_tags_25 },
    { 0xf34a51e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a51ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aa51ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cca51ef, SUC_lexicon_tags_30 },
    { 0xad8a51ef, SUC_lexicon_tags_2 },
    { 0x7caa51ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2a51f5, SUC_lexicon_tags_15 },
    { 0x6f4a51f5, SUC_lexicon_tags_11 },
    { 0xf0a51f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ca51fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e0a51fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b6a5200, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xceea5203, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2aaa5205, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aea520a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x466a520d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ca5213, SUC_lexicon_tags_37 },
    { 0x7ca5213, SUC_lexicon_tags_63 },
    { 0xd9ca5215, SUC_lexicon_tags_17 },
    { 0x672a5216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6a521a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd6a521c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc2a521e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6a5224, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6a5227, SUC_lexicon_tags_25 },
    { 0x524a5227, SUC_lexicon_tags_33 },
    { 0x1baa5229, SUC_lexicon_tags_8 },
    { 0xd40a5227, SUC_lexicon_tags_2 },
    { 0x3b0a5229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666a522f, SUC_lexicon_tags_31 },
    { 0x184a5230, SUC_lexicon_tags_25 },
    { 0xeb2a522f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ea5236, SUC_lexicon_tags_16 },
    { 0xb88a5237, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x446a5239, SUC_lexicon_tags_12 },
    { 0x120a5239, SUC_lexicon_tags_63 },
    { 0x730a523b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6a523e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00a5242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8a524c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188a5252, SUC_lexicon_tags_82 },
    { 0xfe0a5252, SUC_lexicon_tags_114 },
    { 0xc88a5254, SUC_lexicon_tags_25 },
    { 0x46a5255, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce8a5257, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f8a5259, SUC_lexicon_tags_32 },
    { 0xfcca525a, SUC_lexicon_tags_25 },
    { 0x546a525b, SUC_lexicon_tags_11 },
    { 0x856a525b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x936a525e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdea5261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee0a5263, SUC_lexicon_tags_17 },
    { 0x7caa5263, SUC_lexicon_tags_17 },
    { 0x2a0a5264, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xee8a5267, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x648a5269, SUC_lexicon_tags_27 },
    { 0xccaa5269, SUC_lexicon_tags_15 },
    { 0x984a5269, SUC_lexicon_tags_15 },
    { 0x7a0a526b, SUC_lexicon_tags_12 },
    { 0xb5ea526a, SUC_lexicon_tags_37 },
    { 0x946a526e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8a5271, SUC_lexicon_tags_92 },
    { 0xf98a5272, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8a5278, SUC_lexicon_tags_43 },
    { 0x224a5279, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd34a527b, SUC_lexicon_tags_26 },
    { 0x2e4a527c, SUC_lexicon_tags_23 },
    { 0x256a527b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeea5281, SUC_lexicon_tags_31 },
    { 0x60ea5282, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x96aa5284, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x48aa5287, SUC_lexicon_tags_11 },
    { 0x986a5288, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4a528c, SUC_lexicon_tags_11 },
    { 0xe42a528d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baa5291, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd3ea5293, SUC_lexicon_tags_17 },
    { 0x5c4a5293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6a5297, SUC_lexicon_tags_218 },
    { 0x58a5298, SUC_lexicon_tags_87 },
    { 0x78ea5299, SUC_lexicon_tags_7 },
    { 0xeaea529a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaa529f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998a52a7, SUC_lexicon_tags_61 },
    { 0xc98a52a8, SUC_lexicon_tags_63 },
    { 0xf28a52a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaa52ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x454a52af, SUC_lexicon_tags_77 },
    { 0x2f6a52b0, SUC_lexicon_tags_49 },
    { 0x3c4a52b1, SUC_lexicon_tags_27 },
    { 0xc9aa52b2, SUC_lexicon_tags_37 },
    { 0x970a52b3, SUC_lexicon_tags_25 },
    { 0xddaa52b4, SUC_lexicon_tags_17 },
    { 0xe78a52b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cca52b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x646a52ba, SUC_lexicon_tags_26 },
    { 0x9c6a52bb, SUC_lexicon_tags_0 },
    { 0x384a52bc, SUC_lexicon_tags_11 },
    { 0x88aa52bb, SUC_lexicon_tags_47 },
    { 0xa9ea52be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2a52c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ca52c7, SUC_lexicon_tags_12 },
    { 0xf26a52c7, SUC_lexicon_tags_68 },
    { 0xc1aa52c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498a52cd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xb32a52cf, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xccea52d1, SUC_lexicon_tags_43 },
    { 0xfcaa52d2, SUC_lexicon_tags_43 },
    { 0x248a52d1, SUC_lexicon_tags_18 },
    { 0x41ea52d4, SUC_lexicon_tags_11 },
    { 0x74ca52d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978a52dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6aa52e0, SUC_lexicon_tags_47 },
    { 0x66ea52e1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420a52e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2a52ea, SUC_lexicon_tags_45 },
    { 0xe1ca52ea, SUC_lexicon_tags_28 },
    { 0x7cea52ec, SUC_lexicon_tags_7 },
    { 0xdf4a52ec, SUC_lexicon_tags_25 },
    { 0x612a52ee, SUC_lexicon_tags_10 },
    { 0xf40a52ec, SUC_lexicon_tags_6 },
    { 0xbfca52f0, SUC_lexicon_tags_27 },
    { 0x298a52f1, SUC_lexicon_tags_69 },
    { 0xa86a52f2, SUC_lexicon_tags_17 },
    { 0x51ca52f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6a52f7, SUC_lexicon_tags_45 },
    { 0x812a52f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0a52ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x616a5302, SUC_lexicon_tags_37 },
    { 0xb42a5302, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572a5308, SUC_lexicon_tags_7 },
    { 0xc98a5309, SUC_lexicon_tags_49 },
    { 0xd88a530a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5aa530c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f6a530e, SUC_lexicon_tags_49 },
    { 0xeb4a530f, SUC_lexicon_tags_37 },
    { 0x6a5310, SUC_lexicon_tags_25 },
    { 0x618a5310, SUC_lexicon_tags_9 },
    { 0x71aa5311, SUC_lexicon_tags_26 },
    { 0xecca5313, SUC_lexicon_tags_37 },
    { 0x20a5314, SUC_lexicon_tags_79 },
    { 0x3e6a5311, SUC_lexicon_tags_1 },
    { 0x2cca5310, SUC_lexicon_tags_49 },
    { 0x5cca5313, SUC_lexicon_tags_9 },
    { 0x188a5318, SUC_lexicon_tags_33 },
    { 0xe44a5319, SUC_lexicon_tags_48 },
    { 0x6fea5319, SUC_lexicon_tags_40 },
    { 0xd8a5318, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64ca531d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x37ca531f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeca5324, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aa5329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb80a532b, SUC_lexicon_tags_11 },
    { 0x52aa532c, SUC_lexicon_tags_12 },
    { 0x730a532d, SUC_lexicon_tags_48 },
    { 0xf94a532e, SUC_lexicon_tags_0 },
    { 0xc24a532e, SUC_lexicon_tags_15 },
    { 0xd8ca5330, SUC_lexicon_tags_37 },
    { 0xddca5331, SUC_lexicon_tags_23 },
    { 0x882a5332, SUC_lexicon_tags_37 },
    { 0xa72a5332, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6a5339, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0a533e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc7ea5340, SUC_lexicon_tags_11 },
    { 0xc06a5340, SUC_lexicon_tags_11 },
    { 0xac8a5342, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1eca5344, SUC_lexicon_tags_13 },
    { 0x6fea5344, SUC_lexicon_tags_12 },
    { 0x3d6a5344, SUC_lexicon_tags_27 },
    { 0x45ea5344, SUC_lexicon_tags_12 },
    { 0xe24a5348, SUC_lexicon_tags_2 },
    { 0xd0ca5349, SUC_lexicon_tags_47 },
    { 0x17aa5348, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0a534f, SUC_lexicon_tags_6 },
    { 0xf26a534f, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x638a5353, SUC_lexicon_tags_0 },
    { 0x9cea5354, SUC_lexicon_tags_63 },
    { 0xaaa5355, SUC_lexicon_tags_12 },
    { 0xe38a5356, SUC_lexicon_tags_34 },
    { 0x774a5357, SUC_lexicon_tags_12 },
    { 0xc06a5358, SUC_lexicon_tags_11 },
    { 0x390a5358, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54ea535b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x952a535d, SUC_lexicon_tags_14 },
    { 0xbb0a535e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc0a5360, SUC_lexicon_tags_47 },
    { 0xc3aa5361, SUC_lexicon_tags_38 },
    { 0x43ca5362, SUC_lexicon_tags_7 },
    { 0x778a5363, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd1ea5365, SUC_lexicon_tags_37 },
    { 0xfea5365, SUC_lexicon_tags_37 },
    { 0xb50a5366, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faa536d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ca5370, SUC_lexicon_tags_37 },
    { 0xaaaa5371, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6a5375, SUC_lexicon_tags_7 },
    { 0xc6a5376, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a8a5378, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ca537c, SUC_lexicon_tags_49 },
    { 0x8dea537c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x988a5380, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232a5385, SUC_lexicon_tags_27 },
    { 0xcd4a5385, SUC_lexicon_tags_34 },
    { 0x7bea5387, SUC_lexicon_tags_10 },
    { 0xb06a5386, SUC_lexicon_tags_37 },
    { 0x5b4a5386, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x66aa538c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ea5392, SUC_lexicon_tags_29 },
    { 0x550a5392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c2a5395, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aca5398, SUC_lexicon_tags_6 },
    { 0xf3ea5399, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff8a539b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x652a539d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42a53a3, SUC_lexicon_tags_11 },
    { 0x74aa53a4, SUC_lexicon_tags_48 },
    { 0x76a53a5, SUC_lexicon_tags_43 },
    { 0xd08a53a6, SUC_lexicon_tags_28 },
    { 0xebca53a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42a53ab, SUC_lexicon_tags_1 },
    { 0xc80a53ab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dea53b0, SUC_lexicon_tags_3 },
    { 0xeaa53b0, SUC_lexicon_tags_7 },
    { 0xbf2a53b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2a53b6, SUC_lexicon_tags_10 },
    { 0xc6ea53b6, SUC_lexicon_tags_14 },
    { 0x2f0a53b8, SUC_lexicon_tags_0 },
    { 0xec4a53b6, SUC_lexicon_tags_10 },
    { 0xa68a53ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46a53bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58a53c2, SUC_lexicon_tags_26 },
    { 0xd04a53c3, SUC_lexicon_tags_37 },
    { 0xba4a53c4, SUC_lexicon_tags_17 },
    { 0x954a53c4, SUC_lexicon_tags_60 },
    { 0x9f0a53c6, SUC_lexicon_tags_17 },
    { 0x422a53c7, SUC_lexicon_tags_15 },
    { 0xba4a53c3, SUC_lexicon_tags_17 },
    { 0xb6ca53c9, SUC_lexicon_tags_11 },
    { 0x74aa53c9, SUC_lexicon_tags_140 },
    { 0x9d6a53c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaa53cf, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0a53d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6a53d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac4a53da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cca53df, SUC_lexicon_tags_15 },
    { 0x514a53e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8a53e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0a53e8, SUC_lexicon_tags_16 },
    { 0x920a53e9, SUC_lexicon_tags_35 },
    { 0x6fca53ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31aa53ee, SUC_lexicon_tags_7 },
    { 0x286a53ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf4ca53f1, SUC_lexicon_tags_35 },
    { 0x398a53f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8a53f7, SUC_lexicon_tags_45 },
    { 0x8fea53f8, SUC_lexicon_tags_118 },
    { 0x4a2a53f8, SUC_lexicon_tags_7 },
    { 0x62ca53fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ca53fd, SUC_lexicon_tags_38 },
    { 0x540a53fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8a5401, SUC_lexicon_tags_12 },
    { 0x84ea5401, SUC_lexicon_tags_2 },
    { 0x108a5403, SUC_lexicon_tags_1 },
    { 0x3fea5403, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x252a5406, SUC_lexicon_tags_17 },
    { 0xb30a5407, SUC_lexicon_tags_7 },
    { 0x3f2a5406, SUC_lexicon_tags_65 },
    { 0x790a5406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x270a540b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ea5414, SUC_lexicon_tags_43 },
    { 0x1d8a5414, SUC_lexicon_tags_8 },
    { 0x660a5416, SUC_lexicon_tags_37 },
    { 0x1d4a5417, SUC_lexicon_tags_1 },
    { 0x5e0a5418, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6ea541a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc68a541c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2a5421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ea5425, SUC_lexicon_tags_37 },
    { 0xaf6a5425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dca542a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x610a542c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x318a542e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22aa5430, SUC_lexicon_tags_17 },
    { 0xc80a5431, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7faa5433, SUC_lexicon_tags_2 },
    { 0xa88a5434, SUC_lexicon_tags_13 },
    { 0x396a5434, SUC_lexicon_tags_77 },
    { 0x206a5436, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x110a5438, SUC_lexicon_tags_47 },
    { 0xb72a5439, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f4a543b, SUC_lexicon_tags_17 },
    { 0xc6ca543c, SUC_lexicon_tags_32 },
    { 0xbb0a543d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x48a543f, SUC_lexicon_tags_31 },
    { 0x50aa5440, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d4a5442, SUC_lexicon_tags_26 },
    { 0xbcea5443, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb0a5445, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x978a5447, SUC_lexicon_tags_10 },
    { 0xebea5448, SUC_lexicon_tags_25 },
    { 0x770a5449, SUC_lexicon_tags_2 },
    { 0xb52a544a, SUC_lexicon_tags_25 },
    { 0x4d6a544b, SUC_lexicon_tags_17 },
    { 0xc14a5448, SUC_lexicon_tags_11 },
    { 0x5fca544d, SUC_lexicon_tags_25 },
    { 0x446a544e, SUC_lexicon_tags_48 },
    { 0xdd8a5449, SUC_lexicon_tags_0 },
    { 0x1d8a5449, SUC_lexicon_tags_25 },
    { 0x1eea5451, SUC_lexicon_tags_65 },
    { 0x620a5452, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ca545a, SUC_lexicon_tags_12 },
    { 0xe04a545a, SUC_lexicon_tags_26 },
    { 0xcbea545c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a546a, SUC_lexicon_tags_12 },
    { 0x292a546a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8a5471, SUC_lexicon_tags_31 },
    { 0x570a5472, SUC_lexicon_tags_15 },
    { 0xd82a5473, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190a5478, SUC_lexicon_tags_2 },
    { 0x49aa5479, SUC_lexicon_tags_2 },
    { 0x346a547a, SUC_lexicon_tags_2 },
    { 0x7daa547a, SUC_lexicon_tags_37 },
    { 0xd66a5478, SUC_lexicon_tags_66 },
    { 0xaa0a547d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ca5485, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x11ea5487, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2a548a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x320a548d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x682a5490, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe5ea5492, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66a5495, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3ea5497, SUC_lexicon_tags_33 },
    { 0x93aa5497, SUC_lexicon_tags_31 },
    { 0xd8ea5497, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0a549f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952a54a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6ea54a8, SUC_lexicon_tags_17 },
    { 0xda0a54a9, SUC_lexicon_tags_11 },
    { 0x6b6a54aa, SUC_lexicon_tags_13 },
    { 0x87ea54ab, SUC_lexicon_tags_1 },
    { 0xc92a54ac, SUC_lexicon_tags_15 },
    { 0xfd4a54aa, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02a54b1, SUC_lexicon_tags_100 },
    { 0x348a54b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ca54b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15aa54ba, SUC_lexicon_tags_11 },
    { 0xcb4a54bb, SUC_lexicon_tags_22 },
    { 0x7e6a54bb, SUC_lexicon_tags_82 },
    { 0xe6ca54bb, SUC_lexicon_tags_63 },
    { 0xec0a54ba, SUC_lexicon_tags_1 },
    { 0xedea54bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x71aa54c2, SUC_lexicon_tags_6 },
    { 0x5caa54c2, SUC_lexicon_tags_43 },
    { 0x716a54c2, SUC_lexicon_tags_37 },
    { 0xa7ca54c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x570a54c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4a54cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaa54d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164a54d6, SUC_lexicon_tags_29 },
    { 0x44ca54d7, SUC_lexicon_tags_37 },
    { 0x654a54d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d8a54da, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2a54de, SUC_lexicon_tags_37 },
    { 0x1ca54df, SUC_lexicon_tags_33 },
    { 0xfeca54de, SUC_lexicon_tags_8 },
    { 0x2cea54e1, SUC_lexicon_tags_25 },
    { 0x94a54e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ca54eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ca54f1, SUC_lexicon_tags_12 },
    { 0x8aa54f2, SUC_lexicon_tags_14 },
    { 0xeb4a54f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398a54f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc06a54f9, SUC_lexicon_tags_12 },
    { 0x848a54fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14a54fd, SUC_lexicon_tags_3 },
    { 0x6aca54fe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ca5505, SUC_lexicon_tags_25 },
    { 0x3aca5505, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552a550d, SUC_lexicon_tags_11 },
    { 0x142a550e, SUC_lexicon_tags_31 },
    { 0x936a550e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ca5516, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f0a5518, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aa551b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18a5521, SUC_lexicon_tags_37 },
    { 0x492a5521, SUC_lexicon_tags_48 },
    { 0x17ea5523, SUC_lexicon_tags_91 },
    { 0x92ea5524, SUC_lexicon_tags_87 },
    { 0x268a5525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40a552b, SUC_lexicon_tags_7 },
    { 0xc34a552c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854a5533, SUC_lexicon_tags_664 },
    { 0x182a5534, SUC_lexicon_tags_2 },
    { 0xd96a5533, SUC_lexicon_tags_6 },
    { 0x73aa5534, SUC_lexicon_tags_4 },
    { 0x17ea5533, SUC_lexicon_tags_46 },
    { 0x306a5538, SUC_lexicon_tags_17 },
    { 0x28ea5539, SUC_lexicon_tags_22 },
    { 0x3cca5539, SUC_lexicon_tags_2 },
    { 0xc56a553a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x98ea553d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62a5543, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc38a5545, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x156a5547, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfca554a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26a554d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fea5550, SUC_lexicon_tags_65 },
    { 0xaa2a5551, SUC_lexicon_tags_2 },
    { 0x9e4a5551, SUC_lexicon_tags_25 },
    { 0xa00a5553, SUC_lexicon_tags_2 },
    { 0x9e2a5554, SUC_lexicon_tags_11 },
    { 0x4b8a5554, SUC_lexicon_tags_85 },
    { 0x8e2a5554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190a555b, SUC_lexicon_tags_8 },
    { 0x60aa555b, SUC_lexicon_tags_38 },
    { 0x6aa555c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6a5561, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802a5565, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1ca5567, SUC_lexicon_tags_2 },
    { 0x4cca5568, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2a556d, SUC_lexicon_tags_37 },
    { 0x12aa556d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2a6a5570, SUC_lexicon_tags_47 },
    { 0x882a5571, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968a5576, SUC_lexicon_tags_7 },
    { 0x662a5577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0a557f, SUC_lexicon_tags_19 },
    { 0x4cea557f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9ea5582, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20a5585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ea558d, SUC_lexicon_tags_63 },
    { 0xc5ea558d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08a5591, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x468a5593, SUC_lexicon_tags_1 },
    { 0x3a4a5594, SUC_lexicon_tags_60 },
    { 0x296a5594, SUC_lexicon_tags_11 },
    { 0x78ea5596, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96a559b, SUC_lexicon_tags_4 },
    { 0xc6ca559c, SUC_lexicon_tags_31 },
    { 0x4a6a559d, SUC_lexicon_tags_12 },
    { 0xc22a559e, SUC_lexicon_tags_25 },
    { 0x760a559e, SUC_lexicon_tags_12 },
    { 0x266a55a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aa55a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ea55a9, SUC_lexicon_tags_2 },
    { 0x68aa55aa, SUC_lexicon_tags_15 },
    { 0x532a55a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaa55b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aa55b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefca55bf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228a55c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72a55c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a55ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dca55d2, SUC_lexicon_tags_26 },
    { 0x2d0a55d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72ca55d5, SUC_lexicon_tags_30 },
    { 0xff2a55d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1ea55d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4a55da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aa55de, SUC_lexicon_tags_38 },
    { 0x35ea55df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d0a55e1, SUC_lexicon_tags_20 },
    { 0x584a55e2, SUC_lexicon_tags_21 },
    { 0x91aa55e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebea55e5, SUC_lexicon_tags_37 },
    { 0x128a55e6, SUC_lexicon_tags_17 },
    { 0xab0a55e7, SUC_lexicon_tags_28 },
    { 0x13aa55e7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa32a55ea, SUC_lexicon_tags_65 },
    { 0x9dea55ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb00a55ed, SUC_lexicon_tags_37 },
    { 0x6faa55ee, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54a55f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91aa55f4, SUC_lexicon_tags_40 },
    { 0xcd2a55f4, SUC_lexicon_tags_1 },
    { 0x7bea55f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7daa55f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28a55fc, SUC_lexicon_tags_17 },
    { 0x542a55fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9daa55ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28a5602, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x680a5605, SUC_lexicon_tags_15 },
    { 0x47ca5606, SUC_lexicon_tags_2 },
    { 0xc88a5607, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96a560a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbdaa560c, SUC_lexicon_tags_2 },
    { 0xba6a560c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x186a560f, SUC_lexicon_tags_8 },
    { 0x1f8a560f, SUC_lexicon_tags_13 },
    { 0x7b4a5611, SUC_lexicon_tags_46 },
    { 0x552a5610, SUC_lexicon_tags_37 },
    { 0x8bea5613, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x8f0a5615, SUC_lexicon_tags_2 },
    { 0x6eaa5615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda8a5618, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x412a561a, SUC_lexicon_tags_60 },
    { 0xb48a561b, SUC_lexicon_tags_29 },
    { 0x4d4a561c, SUC_lexicon_tags_37 },
    { 0xb1ea561d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xebca561f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0a5622, SUC_lexicon_tags_65 },
    { 0x672a5622, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb2a5625, SUC_lexicon_tags_25 },
    { 0xc0ea5626, SUC_lexicon_tags_17 },
    { 0x384a5627, SUC_lexicon_tags_11 },
    { 0x2a8a5628, SUC_lexicon_tags_29 },
    { 0x6a5625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274a562d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6a5633, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aa5636, SUC_lexicon_tags_43 },
    { 0x3d0a5636, SUC_lexicon_tags_14 },
    { 0x57ca5637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfaa563a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x244a563c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceea5642, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb88a5644, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbbea5646, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ca5649, SUC_lexicon_tags_1 },
    { 0x958a564a, SUC_lexicon_tags_40 },
    { 0x4aca564b, SUC_lexicon_tags_12 },
    { 0x116a564b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x522a564e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0a5651, SUC_lexicon_tags_37 },
    { 0x622a5652, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2a5657, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x552a5659, SUC_lexicon_tags_27 },
    { 0x61aa565a, SUC_lexicon_tags_25 },
    { 0x128a565b, SUC_lexicon_tags_37 },
    { 0xfe6a565b, SUC_lexicon_tags_9 },
    { 0xcf0a565d, SUC_lexicon_tags_2 },
    { 0x89ea565e, SUC_lexicon_tags_88 },
    { 0x62ea565f, SUC_lexicon_tags_2 },
    { 0x74a565a, SUC_lexicon_tags_2 },
    { 0x2faa5659, SUC_lexicon_tags_25 },
    { 0x31ca5662, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cea5666, SUC_lexicon_tags_37 },
    { 0xe4ea5667, SUC_lexicon_tags_63 },
    { 0x816a5668, SUC_lexicon_tags_17 },
    { 0x85aa5667, SUC_lexicon_tags_12 },
    { 0xd32a566a, SUC_lexicon_tags_19 },
    { 0x9baa566b, SUC_lexicon_tags_2 },
    { 0xd8ea5668, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20a5670, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2a5674, SUC_lexicon_tags_38 },
    { 0x274a5675, SUC_lexicon_tags_17 },
    { 0xe9aa5674, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaa5679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06a567e, SUC_lexicon_tags_43 },
    { 0x790a567f, SUC_lexicon_tags_16 },
    { 0x5a0a5680, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5aa5682, SUC_lexicon_tags_17 },
    { 0x9dca5683, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb86a5685, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52a5687, SUC_lexicon_tags_117 },
    { 0x650a5688, SUC_lexicon_tags_35 },
    { 0x178a5689, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x176a568c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x366a568e, SUC_lexicon_tags_12 },
    { 0xd3aa568f, SUC_lexicon_tags_1 },
    { 0xc00a5690, SUC_lexicon_tags_30 },
    { 0xb94a5690, SUC_lexicon_tags_9 },
    { 0x324a5692, SUC_lexicon_tags_17 },
    { 0xfaa5693, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x748a5695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186a569d, SUC_lexicon_tags_103 },
    { 0x124a569e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8a56a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x710a56a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daa56ad, SUC_lexicon_tags_45 },
    { 0x7b6a56ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40a56bf, SUC_lexicon_tags_77 },
    { 0x926a56c0, SUC_lexicon_tags_6 },
    { 0x4d2a56c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36ca56c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaa56c6, SUC_lexicon_tags_17 },
    { 0x512a56c7, SUC_lexicon_tags_1 },
    { 0x6bca56c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdca56ca, SUC_lexicon_tags_19 },
    { 0x790a56cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa6a56cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4a56d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2a56d5, SUC_lexicon_tags_140 },
    { 0x258a56d5, SUC_lexicon_tags_27 },
    { 0xb4a56d7, SUC_lexicon_tags_29 },
    { 0x388a56d5, SUC_lexicon_tags_15 },
    { 0xd14a56d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbea56db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x636a56dd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcca56e0, SUC_lexicon_tags_11 },
    { 0x38ea56e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaea56e4, SUC_lexicon_tags_75 },
    { 0xd78a56e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa0a56e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506a56ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ca56f2, SUC_lexicon_tags_37 },
    { 0x930a56f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcca56f7, SUC_lexicon_tags_47 },
    { 0x814a56f8, SUC_lexicon_tags_4 },
    { 0x5eea56f8, SUC_lexicon_tags_11 },
    { 0xc4aa56fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbca5700, SUC_lexicon_tags_14 },
    { 0x228a5701, SUC_lexicon_tags_12 },
    { 0x64a5701, SUC_lexicon_tags_53 },
    { 0x84a5703, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96a5706, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26aa5708, SUC_lexicon_tags_63 },
    { 0x3faa5709, SUC_lexicon_tags_25 },
    { 0x1aca570a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4a570f, SUC_lexicon_tags_60 },
    { 0x96a5710, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2a5713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ea571b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbca5720, SUC_lexicon_tags_14 },
    { 0x1d2a5720, SUC_lexicon_tags_34 },
    { 0xef4a5721, SUC_lexicon_tags_11 },
    { 0xeb8a5721, SUC_lexicon_tags_17 },
    { 0xa5724, SUC_lexicon_tags_25 },
    { 0xbc6a5720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ea5728, SUC_lexicon_tags_83 },
    { 0x782a5729, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb12a572b, SUC_lexicon_tags_17 },
    { 0x524a572b, SUC_lexicon_tags_45 },
    { 0xee2a572b, SUC_lexicon_tags_37 },
    { 0x680a572b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb14a5730, SUC_lexicon_tags_37 },
    { 0x5f0a5730, SUC_lexicon_tags_13 },
    { 0x988a5732, SUC_lexicon_tags_1 },
    { 0x426a5730, SUC_lexicon_tags_17 },
    { 0xa0a5734, SUC_lexicon_tags_68 },
    { 0x89ca5733, SUC_lexicon_tags_25 },
    { 0x206a5730, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x310a5738, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x30a573a, SUC_lexicon_tags_3 },
    { 0xd16a573b, SUC_lexicon_tags_63 },
    { 0xf42a573a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638a5740, SUC_lexicon_tags_68 },
    { 0x1b2a5741, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a5744, SUC_lexicon_tags_2 },
    { 0x706a5745, SUC_lexicon_tags_17 },
    { 0x7fca5744, SUC_lexicon_tags_12 },
    { 0xc4ca5745, SUC_lexicon_tags_34 },
    { 0x51ca5748, SUC_lexicon_tags_6 },
    { 0xba2a5748, SUC_lexicon_tags_7 },
    { 0x196a5745, SUC_lexicon_tags_17 },
    { 0xac4a5746, SUC_lexicon_tags_79 },
    { 0xd0aa574c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0a5756, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cca575a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a575f, SUC_lexicon_tags_63 },
    { 0xf90a575f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ca5763, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc50a5765, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x43aa5767, SUC_lexicon_tags_14 },
    { 0x76aa5768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8a576f, SUC_lexicon_tags_43 },
    { 0xdbca576f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74a5774, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80a5777, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cca577d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xb36a577f, SUC_lexicon_tags_21 },
    { 0xe1ca577f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0a5784, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa82a5786, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfcaa5788, SUC_lexicon_tags_1 },
    { 0x25aa5789, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe76a578b, SUC_lexicon_tags_9 },
    { 0xc82a578c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2a578e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc5ca5790, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x766a5792, SUC_lexicon_tags_7 },
    { 0x2d0a5793, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aca579b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9aa57a1, SUC_lexicon_tags_25 },
    { 0x2d0a57a2, SUC_lexicon_tags_0 },
    { 0xb54a57a3, SUC_lexicon_tags_57 },
    { 0x55aa57a1, SUC_lexicon_tags_17 },
    { 0x510a57a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x57ca57a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x30a57aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x828a57ac, SUC_lexicon_tags_12 },
    { 0x4c0a57ad, SUC_lexicon_tags_16 },
    { 0x9a8a57ac, SUC_lexicon_tags_19 },
    { 0x96a57af, SUC_lexicon_tags_27 },
    { 0x846a57af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x940a57b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2a57b6, SUC_lexicon_tags_900 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeea57bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8a57c1, SUC_lexicon_tags_11 },
    { 0x694a57c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x69aa57c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280a57ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6a57cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x38aa57d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70a57d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1fea57d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x914a57d8, SUC_lexicon_tags_21 },
    { 0x6fca57d9, SUC_lexicon_tags_33 },
    { 0x152a57d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316a57e0, SUC_lexicon_tags_55 },
    { 0x8fca57e1, SUC_lexicon_tags_23 },
    { 0x27aa57e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfca57e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x230a57ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6a57ed, SUC_lexicon_tags_23 },
    { 0xb8ea57ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86a57f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352a57f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ea57fc, SUC_lexicon_tags_29 },
    { 0x124a57fd, SUC_lexicon_tags_1 },
    { 0x5eca57fd, SUC_lexicon_tags_11 },
    { 0xb4ca57fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ea5803, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x490a5805, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadaa5807, SUC_lexicon_tags_1 },
    { 0xb6ea5807, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ca580b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0a580e, SUC_lexicon_tags_439 },
    { 0xe08a580e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff0a5811, SUC_lexicon_tags_6 },
    { 0xecaa5812, SUC_lexicon_tags_37 },
    { 0x6c0a5813, SUC_lexicon_tags_254 },
    { 0xe82a5814, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4a5817, SUC_lexicon_tags_37 },
    { 0x6e6a5818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x194a581b, SUC_lexicon_tags_68 },
    { 0xbaa581c, SUC_lexicon_tags_25 },
    { 0x21ca581d, SUC_lexicon_tags_25 },
    { 0xa7ca581e, SUC_lexicon_tags_1 },
    { 0xb8ca581d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3cca5821, SUC_lexicon_tags_39 },
    { 0xa2aa5821, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6a582e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x842a5830, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6a5833, SUC_lexicon_tags_28 },
    { 0x400a5834, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92aa5836, SUC_lexicon_tags_27 },
    { 0x5d0a5837, SUC_lexicon_tags_60 },
    { 0x5b6a5838, SUC_lexicon_tags_2 },
    { 0xd5ea5838, SUC_lexicon_tags_17 },
    { 0x754a5839, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aa583e, SUC_lexicon_tags_47 },
    { 0x254a583e, SUC_lexicon_tags_14 },
    { 0x9f8a5840, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ea5845, SUC_lexicon_tags_2 },
    { 0x8c2a5846, SUC_lexicon_tags_25 },
    { 0x45aa5847, SUC_lexicon_tags_13 },
    { 0x9b8a5847, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ea584c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918a5850, SUC_lexicon_tags_43 },
    { 0xbfea5850, SUC_lexicon_tags_19 },
    { 0x7caa5852, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x83aa5855, SUC_lexicon_tags_11 },
    { 0x7c6a5855, SUC_lexicon_tags_25 },
    { 0x15ca5857, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26a585b, SUC_lexicon_tags_37 },
    { 0x396a585c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8a585f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74ca5861, SUC_lexicon_tags_30 },
    { 0x9aca5862, SUC_lexicon_tags_6 },
    { 0xc12a5862, SUC_lexicon_tags_2 },
    { 0x4a5864, SUC_lexicon_tags_7 },
    { 0xa42a5864, SUC_lexicon_tags_16 },
    { 0xd5ea5866, SUC_lexicon_tags_47 },
    { 0xdb2a5866, SUC_lexicon_tags_2 },
    { 0x41ea5862, SUC_lexicon_tags_16 },
    { 0xfdaa5865, SUC_lexicon_tags_28 },
    { 0x902a586a, SUC_lexicon_tags_25 },
    { 0x21ca586b, SUC_lexicon_tags_23 },
    { 0x860a586c, SUC_lexicon_tags_11 },
    { 0xa04a586c, SUC_lexicon_tags_23 },
    { 0xd6ea5862, SUC_lexicon_tags_9 },
    { 0x6f8a586f, SUC_lexicon_tags_14 },
    { 0x544a5866, SUC_lexicon_tags_11 },
    { 0x428a5871, SUC_lexicon_tags_11 },
    { 0x7b0a5862, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64aa5876, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2a587a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610a587e, SUC_lexicon_tags_25 },
    { 0xff4a587e, SUC_lexicon_tags_34 },
    { 0x4f0a5880, SUC_lexicon_tags_10 },
    { 0x9d8a5881, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa4a5883, SUC_lexicon_tags_2 },
    { 0x940a5883, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c2a5886, SUC_lexicon_tags_10 },
    { 0x730a5887, SUC_lexicon_tags_11 },
    { 0x316a5887, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb96a588a, SUC_lexicon_tags_19 },
    { 0xb0ea588a, SUC_lexicon_tags_17 },
    { 0xa6aa588c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xacaa588e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ea5893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128a589b, SUC_lexicon_tags_26 },
    { 0xe64a589c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1aca589e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2aca58a0, SUC_lexicon_tags_34 },
    { 0xf8ea58a0, SUC_lexicon_tags_43 },
    { 0xd5ca58a0, SUC_lexicon_tags_0 },
    { 0x6aca58a3, SUC_lexicon_tags_0 },
    { 0x626a58a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114a58a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbea58ae, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x79ea58b0, SUC_lexicon_tags_60 },
    { 0x89ca58b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8a58b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x144a58b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4a58bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64a58bf, SUC_lexicon_tags_38 },
    { 0x95ca58c0, SUC_lexicon_tags_33 },
    { 0x1c0a58c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aea58c5, SUC_lexicon_tags_38 },
    { 0x24ea58c6, SUC_lexicon_tags_11 },
    { 0x6d2a58c7, SUC_lexicon_tags_68 },
    { 0x374a58c6, SUC_lexicon_tags_30 },
    { 0x78ca58c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f8a58cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84a58d0, SUC_lexicon_tags_63 },
    { 0x66ca58d0, SUC_lexicon_tags_4 },
    { 0x3a0a58d2, SUC_lexicon_tags_49 },
    { 0x9a8a58d3, SUC_lexicon_tags_50 },
    { 0x81ea58d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcf6a58d6, SUC_lexicon_tags_17 },
    { 0x2f2a58d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974a58de, SUC_lexicon_tags_1 },
    { 0x25aa58df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4a58e2, SUC_lexicon_tags_32 },
    { 0x264a58e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eca58ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4caa58ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x120a58ef, SUC_lexicon_tags_1 },
    { 0xb7ca58f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca58f5, SUC_lexicon_tags_31 },
    { 0xbeca58f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0a58f9, SUC_lexicon_tags_19 },
    { 0x4c2a58fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd0a58fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa58fe, SUC_lexicon_tags_1 },
    { 0xc8aa58ff, SUC_lexicon_tags_17 },
    { 0x7a0a5900, SUC_lexicon_tags_31 },
    { 0xa04a5900, SUC_lexicon_tags_37 },
    { 0x828a5902, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a5906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6a5909, SUC_lexicon_tags_15 },
    { 0x932a590a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4eaa590c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0a5910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc46a5912, SUC_lexicon_tags_98 },
    { 0x4aaa5913, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x45aa5916, SUC_lexicon_tags_16 },
    { 0x1d0a5917, SUC_lexicon_tags_17 },
    { 0x572a5918, SUC_lexicon_tags_1 },
    { 0xcd6a5917, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6a6a591b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa591e, SUC_lexicon_tags_37 },
    { 0x58aa591f, SUC_lexicon_tags_13 },
    { 0x92ea5920, SUC_lexicon_tags_2 },
    { 0xbdaa5921, SUC_lexicon_tags_35 },
    { 0xc42a5922, SUC_lexicon_tags_16 },
    { 0x2eca5922, SUC_lexicon_tags_1 },
    { 0xfa0a5923, SUC_lexicon_tags_46 },
    { 0x17ca5924, SUC_lexicon_tags_69 },
    { 0x942a5926, SUC_lexicon_tags_14 },
    { 0xd6ca5927, SUC_lexicon_tags_60 },
    { 0x9bea5927, SUC_lexicon_tags_12 },
    { 0xd7ca5924, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36a592c, SUC_lexicon_tags_17 },
    { 0x670a592c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74ca592f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x386a5931, SUC_lexicon_tags_12 },
    { 0x10ea5932, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cca5937, SUC_lexicon_tags_15 },
    { 0xbaaa5938, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x914a593b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630a5946, SUC_lexicon_tags_12 },
    { 0xc00a5946, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9a4a5949, SUC_lexicon_tags_11 },
    { 0x892a5949, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144a594e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fca5950, SUC_lexicon_tags_33 },
    { 0x952a5951, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dca5958, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe02a595a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x378a595c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2a595f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc88a5961, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a5964, SUC_lexicon_tags_19 },
    { 0xa10a5965, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ea596b, SUC_lexicon_tags_7 },
    { 0x12ea596b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34a596f, SUC_lexicon_tags_15 },
    { 0xb38a5970, SUC_lexicon_tags_1 },
    { 0xbdaa5970, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x650a5973, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ca597b, SUC_lexicon_tags_87 },
    { 0x504a597c, SUC_lexicon_tags_12 },
    { 0x14ea597d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2a5986, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabea5989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8a598d, SUC_lexicon_tags_12 },
    { 0x83ea598e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bca5992, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x532a5994, SUC_lexicon_tags_29 },
    { 0x6daa5995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aca5998, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14a59a0, SUC_lexicon_tags_25 },
    { 0x9eea59a1, SUC_lexicon_tags_27 },
    { 0xbfea59a0, SUC_lexicon_tags_25 },
    { 0x222a59a2, SUC_lexicon_tags_25 },
    { 0xfdca59a1, SUC_lexicon_tags_37 },
    { 0x78aa59a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414a59a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8a59ac, SUC_lexicon_tags_2 },
    { 0x9dca59ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x114a59b0, SUC_lexicon_tags_19 },
    { 0xf7ea59b1, SUC_lexicon_tags_23 },
    { 0x9caa59b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x108a59b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a59b9, SUC_lexicon_tags_17 },
    { 0x646a59ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98a59c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5aa59c5, SUC_lexicon_tags_17 },
    { 0x7baa59c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414a59ca, SUC_lexicon_tags_2 },
    { 0xd94a59cb, SUC_lexicon_tags_11 },
    { 0x2ca59ca, SUC_lexicon_tags_17 },
    { 0x290a59ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4faa59d4, SUC_lexicon_tags_1 },
    { 0x1c0a59d5, SUC_lexicon_tags_16 },
    { 0x582a59d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44a59da, SUC_lexicon_tags_91 },
    { 0x82aa59da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed0a59dd, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80a59e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ca59e5, SUC_lexicon_tags_29 },
    { 0xd66a59e6, SUC_lexicon_tags_104 },
    { 0x270a59e7, SUC_lexicon_tags_5 },
    { 0x3cca59e7, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x7c2a59ea, SUC_lexicon_tags_2 },
    { 0x1e8a59eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x752a59ed, SUC_lexicon_tags_65 },
    { 0xe4aa59ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4a59f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72a59f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf82a59f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdca5a00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ea5a03, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x80a5a06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x16aa5a08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa22a5a0a, SUC_lexicon_tags_6 },
    { 0x5c4a5a0b, SUC_lexicon_tags_49 },
    { 0xe9aa5a0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca5a12, SUC_lexicon_tags_17 },
    { 0x6f2a5a13, SUC_lexicon_tags_25 },
    { 0x4e4a5a14, SUC_lexicon_tags_45 },
    { 0x280a5a12, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aa5a1a, SUC_lexicon_tags_25 },
    { 0xffea5a1b, SUC_lexicon_tags_14 },
    { 0x930a5a1a, SUC_lexicon_tags_2 },
    { 0xa12a5a1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4ea5a1f, SUC_lexicon_tags_1 },
    { 0x6a2a5a20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf0a5a22, SUC_lexicon_tags_14 },
    { 0x19ea5a23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8a5a27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6a5a2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70a5a2f, SUC_lexicon_tags_37 },
    { 0xd96a5a2f, SUC_lexicon_tags_17 },
    { 0x5aa5a31, SUC_lexicon_tags_47 },
    { 0xa9aa5a30, SUC_lexicon_tags_27 },
    { 0x5bca5a30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe42a5a36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22a5a39, SUC_lexicon_tags_47 },
    { 0xa00a5a3a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91aa5a3f, SUC_lexicon_tags_19 },
    { 0xbb2a5a3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6a5a43, SUC_lexicon_tags_38 },
    { 0x2a0a5a44, SUC_lexicon_tags_25 },
    { 0x1a6a5a44, SUC_lexicon_tags_12 },
    { 0xac8a5a45, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb54a5a48, SUC_lexicon_tags_11 },
    { 0x6d4a5a48, SUC_lexicon_tags_11 },
    { 0xc02a5a48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb08a5a4c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x702a5a4e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ca5a54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a5a5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288a5a60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb24a5a62, SUC_lexicon_tags_74 },
    { 0xe0aa5a63, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4f6a5a65, SUC_lexicon_tags_45 },
    { 0xfe8a5a66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x744a5a69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x358a5a6b, SUC_lexicon_tags_27 },
    { 0x23aa5a6b, SUC_lexicon_tags_9 },
    { 0xe2ca5a6d, SUC_lexicon_tags_7 },
    { 0xa9aa5a6b, SUC_lexicon_tags_37 },
    { 0xd74a5a6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4a5a73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938a5a77, SUC_lexicon_tags_7 },
    { 0x484a5a78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97aa5a7a, SUC_lexicon_tags_63 },
    { 0x71ea5a7b, SUC_lexicon_tags_11 },
    { 0xc2ca5a7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6c0a5a7e, SUC_lexicon_tags_12 },
    { 0xbf6a5a7f, SUC_lexicon_tags_7 },
    { 0x590a5a7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a5a83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x692a5a86, SUC_lexicon_tags_2 },
    { 0x69ca5a86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8a5a8b, SUC_lexicon_tags_35 },
    { 0xeeaa5a8c, SUC_lexicon_tags_55 },
    { 0xd88a5a8d, SUC_lexicon_tags_48 },
    { 0x20aa5a8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0a5a92, SUC_lexicon_tags_27 },
    { 0x3b2a5a93, SUC_lexicon_tags_12 },
    { 0x2a5a94, SUC_lexicon_tags_10 },
    { 0xcf2a5a95, SUC_lexicon_tags_64 },
    { 0x748a5a96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ca5a99, SUC_lexicon_tags_69 },
    { 0x5aea5a99, SUC_lexicon_tags_3 },
    { 0x8baa5a9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x340a5a9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3ca5aa0, SUC_lexicon_tags_11 },
    { 0xb1ca5aa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148a5aa5, SUC_lexicon_tags_37 },
    { 0xa5ca5aa6, SUC_lexicon_tags_16 },
    { 0x390a5aa7, SUC_lexicon_tags_178 },
    { 0xe2a5aa8, SUC_lexicon_tags_9 },
    { 0xf88a5aa9, SUC_lexicon_tags_6 },
    { 0xe8ca5aa7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xef8a5aac, SUC_lexicon_tags_23 },
    { 0xf7ea5aad, SUC_lexicon_tags_11 },
    { 0xf72a5aae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x700a5ab1, SUC_lexicon_tags_11 },
    { 0x210a5ab1, SUC_lexicon_tags_15 },
    { 0xfbaa5ab3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184a5ab9, SUC_lexicon_tags_87 },
    { 0x39aa5aba, SUC_lexicon_tags_14 },
    { 0x6ea5ab9, SUC_lexicon_tags_262 },
    { 0x3e6a5abc, SUC_lexicon_tags_43 },
    { 0xd2ea5abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x332a5abf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x568a5ac1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4a5ac5, SUC_lexicon_tags_17 },
    { 0xd20a5ac6, SUC_lexicon_tags_14 },
    { 0x448a5ac6, SUC_lexicon_tags_10 },
    { 0xa64a5ac8, SUC_lexicon_tags_47 },
    { 0x1e6a5ac7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x168a5acb, SUC_lexicon_tags_38 },
    { 0xb52a5acb, SUC_lexicon_tags_33 },
    { 0xdc2a5acd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2a5ad0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ea5ad7, SUC_lexicon_tags_39 },
    { 0xad0a5ad8, SUC_lexicon_tags_12 },
    { 0x5aca5ad8, SUC_lexicon_tags_38 },
    { 0xe5aa5ada, SUC_lexicon_tags_25 },
    { 0x33ea5adb, SUC_lexicon_tags_0 },
    { 0x744a5adb, SUC_lexicon_tags_15 },
    { 0xdd4a5add, SUC_lexicon_tags_15 },
    { 0xa82a5ade, SUC_lexicon_tags_11 },
    { 0xf3aa5ad8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ca5ae2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2aa5ae4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66a5aea, SUC_lexicon_tags_11 },
    { 0x5aaa5aeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x984a5aee, SUC_lexicon_tags_9 },
    { 0xda6a5aef, SUC_lexicon_tags_23 },
    { 0xa28a5af0, SUC_lexicon_tags_9 },
    { 0x2aca5af1, SUC_lexicon_tags_63 },
    { 0xbea5af2, SUC_lexicon_tags_30 },
    { 0x9eca5af3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ca5af9, SUC_lexicon_tags_37 },
    { 0x2c6a5af9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x788a5afd, SUC_lexicon_tags_45 },
    { 0xf6ea5afe, SUC_lexicon_tags_15 },
    { 0x96ca5afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x630a5b01, SUC_lexicon_tags_25 },
    { 0x786a5b02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538a5b06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88ca5b08, SUC_lexicon_tags_2 },
    { 0xd7ea5b09, SUC_lexicon_tags_31 },
    { 0x954a5b08, SUC_lexicon_tags_2 },
    { 0x5e4a5b09, SUC_lexicon_tags_37 },
    { 0x1eca5b0b, SUC_lexicon_tags_12 },
    { 0x728a5b0b, SUC_lexicon_tags_37 },
    { 0x388a5b0a, SUC_lexicon_tags_61 },
    { 0x6c6a5b0b, SUC_lexicon_tags_38 },
    { 0x468a5b0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6a5b14, SUC_lexicon_tags_7 },
    { 0xa2a5b15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572a5b19, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2a5b1d, SUC_lexicon_tags_33 },
    { 0x83ca5b1e, SUC_lexicon_tags_11 },
    { 0xe34a5b1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x632a5b21, SUC_lexicon_tags_49 },
    { 0x176a5b22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x628a5b25, SUC_lexicon_tags_11 },
    { 0xbc8a5b26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aa5b2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc90a5b2d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfea5b34, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ca5b37, SUC_lexicon_tags_7 },
    { 0x306a5b37, SUC_lexicon_tags_3 },
    { 0xcca5b39, SUC_lexicon_tags_1 },
    { 0x34ea5b3a, SUC_lexicon_tags_10 },
    { 0x88ca5b3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x156a5b3e, SUC_lexicon_tags_0 },
    { 0x5aca5b3e, SUC_lexicon_tags_79 },
    { 0xd50a5b40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd68a5b42, SUC_lexicon_tags_10 },
    { 0x1c4a5b43, SUC_lexicon_tags_21 },
    { 0xfaca5b43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12aa5b46, SUC_lexicon_tags_29 },
    { 0x58ea5b47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0a5b4b, SUC_lexicon_tags_17 },
    { 0x73aa5b4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d6a5b4e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0a5b52, SUC_lexicon_tags_7 },
    { 0x684a5b53, SUC_lexicon_tags_15 },
    { 0xebaa5b54, SUC_lexicon_tags_15 },
    { 0xe86a5b55, SUC_lexicon_tags_2 },
    { 0xa66a5b53, SUC_lexicon_tags_7 },
    { 0x3aca5b55, SUC_lexicon_tags_43 },
    { 0xd28a5b58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaa5b5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ca5b5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74a5b62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8a5b65, SUC_lexicon_tags_0 },
    { 0xfc0a5b66, SUC_lexicon_tags_12 },
    { 0x5dea5b67, SUC_lexicon_tags_11 },
    { 0x308a5b67, SUC_lexicon_tags_25 },
    { 0xa5ea5b67, SUC_lexicon_tags_45 },
    { 0xc30a5b6a, SUC_lexicon_tags_18 },
    { 0x822a5b6b, SUC_lexicon_tags_47 },
    { 0xe68a5b6c, SUC_lexicon_tags_37 },
    { 0x84ea5b6d, SUC_lexicon_tags_15 },
    { 0x4aea5b68, SUC_lexicon_tags_12 },
    { 0xe2aa5b6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4a5b74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70a5b77, SUC_lexicon_tags_47 },
    { 0xdcea5b78, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3eca5b7a, SUC_lexicon_tags_17 },
    { 0xefca5b7b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x130a5b7e, SUC_lexicon_tags_17 },
    { 0x6b0a5b7f, SUC_lexicon_tags_11 },
    { 0xdea5b7f, SUC_lexicon_tags_25 },
    { 0x81ca5b81, SUC_lexicon_tags_47 },
    { 0xbb6a5b81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x894a5b84, SUC_lexicon_tags_49 },
    { 0x10aa5b84, SUC_lexicon_tags_38 },
    { 0x5eea5b84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670a5b8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ca5b8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe82a5b8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x942a5b92, SUC_lexicon_tags_29 },
    { 0x6a2a5b92, SUC_lexicon_tags_25 },
    { 0x68a5b94, SUC_lexicon_tags_33 },
    { 0xc14a5b95, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2a5b9d, SUC_lexicon_tags_60 },
    { 0x296a5b9e, SUC_lexicon_tags_31 },
    { 0x18ea5b9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x188a5ba1, SUC_lexicon_tags_22 },
    { 0x170a5ba1, SUC_lexicon_tags_9 },
    { 0x2d4a5ba1, SUC_lexicon_tags_17 },
    { 0xc3aa5ba4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62a5ba7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6c8a5ba9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b2a5bab, SUC_lexicon_tags_7 },
    { 0x86a5bab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa2a5bae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cca5bb1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd94a5bb3, SUC_lexicon_tags_14 },
    { 0xfb4a5bb4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeb6a5bb6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aa5bba, SUC_lexicon_tags_37 },
    { 0xc76a5bbb, SUC_lexicon_tags_0 },
    { 0x5caa5bbc, SUC_lexicon_tags_25 },
    { 0x5dea5bbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eea5bc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc34a5bc2, SUC_lexicon_tags_11 },
    { 0xe1aa5bc3, SUC_lexicon_tags_17 },
    { 0x5bea5bc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bca5bc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0a5bca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafea5bd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daa5bd9, SUC_lexicon_tags_37 },
    { 0x612a5bd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe64a5bdc, SUC_lexicon_tags_65 },
    { 0x6a5bdd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04a5be4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7bca5be6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd8a5be8, SUC_lexicon_tags_29 },
    { 0x698a5be9, SUC_lexicon_tags_11 },
    { 0x330a5bea, SUC_lexicon_tags_14 },
    { 0xcbea5be9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x484a5bee, SUC_lexicon_tags_56 },
    { 0x1aaa5bef, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xffea5bf2, SUC_lexicon_tags_7 },
    { 0x50a5bf2, SUC_lexicon_tags_12 },
    { 0x198a5bf2, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ca5bf9, SUC_lexicon_tags_25 },
    { 0xde2a5bf9, SUC_lexicon_tags_48 },
    { 0x390a5bf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ca5bfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70a5c05, SUC_lexicon_tags_24 },
    { 0xbaea5c05, SUC_lexicon_tags_9 },
    { 0xc72a5c07, SUC_lexicon_tags_30 },
    { 0xb42a5c08, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792a5c0f, SUC_lexicon_tags_25 },
    { 0xaaea5c0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c6a5c12, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628a5c16, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0a5c19, SUC_lexicon_tags_8 },
    { 0xccea5c1a, SUC_lexicon_tags_1 },
    { 0xe2aa5c1b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7caa5c1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x368a5c1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaa5c22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ea5c26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06a5c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x614a5c2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x91aa5c2d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x170a5c2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2a5c34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4a5c37, SUC_lexicon_tags_14 },
    { 0xb6ca5c37, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfea5c3b, SUC_lexicon_tags_27 },
    { 0x6caa5c3c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ea5c3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ca5c43, SUC_lexicon_tags_15 },
    { 0x162a5c44, SUC_lexicon_tags_38 },
    { 0x7bca5c43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x244a5c47, SUC_lexicon_tags_19 },
    { 0xd74a5c47, SUC_lexicon_tags_38 },
    { 0x1e8a5c49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6dea5c4b, SUC_lexicon_tags_18 },
    { 0x252a5c4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ca5c4f, SUC_lexicon_tags_17 },
    { 0x874a5c4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0a5c55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06a5c58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x976a5c5b, SUC_lexicon_tags_47 },
    { 0xf6aa5c5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500a5c60, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ea5c63, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2caa5c65, SUC_lexicon_tags_7 },
    { 0x206a5c65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x422a5c68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaaea5c6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02a5c6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaa5c7a, SUC_lexicon_tags_25 },
    { 0x71ca5c7b, SUC_lexicon_tags_10 },
    { 0x936a5c7c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44a5c7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472a5c84, SUC_lexicon_tags_47 },
    { 0x748a5c84, SUC_lexicon_tags_1 },
    { 0xd48a5c85, SUC_lexicon_tags_238 },
    { 0xdbaa5c87, SUC_lexicon_tags_5 },
    { 0x434a5c88, SUC_lexicon_tags_25 },
    { 0xd6a5c85, SUC_lexicon_tags_62 },
    { 0x712a5c8a, SUC_lexicon_tags_2 },
    { 0x71ea5c89, SUC_lexicon_tags_15 },
    { 0xbe8a5c8c, SUC_lexicon_tags_36 },
    { 0x9a6a5c8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80a5c92, SUC_lexicon_tags_15 },
    { 0xde0a5c92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x86a5c95, SUC_lexicon_tags_15 },
    { 0xfeea5c96, SUC_lexicon_tags_33 },
    { 0xec6a5c97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258a5c9b, SUC_lexicon_tags_37 },
    { 0x7faa5c9b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aca5ca2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04a5ca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffea5cab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aa5cae, SUC_lexicon_tags_54 },
    { 0x64aa5caf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x57aa5cb1, SUC_lexicon_tags_9 },
    { 0xb82a5cb2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe70a5cb4, SUC_lexicon_tags_40 },
    { 0x196a5cb5, SUC_lexicon_tags_28 },
    { 0x202a5cb5, SUC_lexicon_tags_37 },
    { 0x8f8a5cb7, SUC_lexicon_tags_37 },
    { 0x90a5cb7, SUC_lexicon_tags_43 },
    { 0x74ca5cb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4a5cbc, SUC_lexicon_tags_47 },
    { 0x712a5cbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28a5cc3, SUC_lexicon_tags_37 },
    { 0xe48a5cc4, SUC_lexicon_tags_0 },
    { 0xa80a5cc5, SUC_lexicon_tags_11 },
    { 0xc90a5cc4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50a5cc9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7e8a5ccb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca5cd0, SUC_lexicon_tags_4 },
    { 0x138a5cd1, SUC_lexicon_tags_68 },
    { 0x7c4a5cd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcea5cd8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ea5cdd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0a5ce0, SUC_lexicon_tags_23 },
    { 0xfbca5ce0, SUC_lexicon_tags_30 },
    { 0x56a5ce2, SUC_lexicon_tags_19 },
    { 0x4dea5ce1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ea5ce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118a5ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82a5cee, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0a5cf1, SUC_lexicon_tags_37 },
    { 0xbeca5cf1, SUC_lexicon_tags_0 },
    { 0x12ea5cf3, SUC_lexicon_tags_37 },
    { 0xa38a5cf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8a5cfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42a5d04, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0a5d07, SUC_lexicon_tags_0 },
    { 0x23ca5d07, SUC_lexicon_tags_24 },
    { 0x54aa5d09, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa28a5d0b, SUC_lexicon_tags_47 },
    { 0x172a5d0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aea5d11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4a5d14, SUC_lexicon_tags_26 },
    { 0x58ca5d15, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24a5d18, SUC_lexicon_tags_1 },
    { 0xaa8a5d19, SUC_lexicon_tags_25 },
    { 0x2aca5d19, SUC_lexicon_tags_37 },
    { 0x2bca5d1b, SUC_lexicon_tags_4 },
    { 0x62a5d1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8a5d20, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2cea5d22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2baa5d26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6ca5d28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x412a5d2b, SUC_lexicon_tags_19 },
    { 0x314a5d2b, SUC_lexicon_tags_37 },
    { 0xc9aa5d2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08a5d33, SUC_lexicon_tags_37 },
    { 0xc8a5d34, SUC_lexicon_tags_6 },
    { 0xe46a5d35, SUC_lexicon_tags_25 },
    { 0x328a5d35, SUC_lexicon_tags_79 },
    { 0xd34a5d37, SUC_lexicon_tags_0 },
    { 0xc34a5d38, SUC_lexicon_tags_15 },
    { 0x61ca5d39, SUC_lexicon_tags_194 },
    { 0xbb0a5d39, SUC_lexicon_tags_21 },
    { 0xcbea5d3b, SUC_lexicon_tags_38 },
    { 0x412a5d3c, SUC_lexicon_tags_1 },
    { 0x918a5d3d, SUC_lexicon_tags_2 },
    { 0x33ca5d36, SUC_lexicon_tags_17 },
    { 0x960a5d3f, SUC_lexicon_tags_66 },
    { 0x128a5d3c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x248a5d42, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14a5d45, SUC_lexicon_tags_45 },
    { 0x400a5d46, SUC_lexicon_tags_12 },
    { 0x428a5d47, SUC_lexicon_tags_16 },
    { 0x1bca5d46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x350a5d4b, SUC_lexicon_tags_12 },
    { 0x760a5d4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ea5d4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5cea5d51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4a5d56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ca5d5b, SUC_lexicon_tags_43 },
    { 0x95ca5d5c, SUC_lexicon_tags_31 },
    { 0xfe2a5d5c, SUC_lexicon_tags_1 },
    { 0x784a5d5c, SUC_lexicon_tags_11 },
    { 0x286a5d5f, SUC_lexicon_tags_6 },
    { 0x974a5d60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x398a5d63, SUC_lexicon_tags_33 },
    { 0xda4a5d63, SUC_lexicon_tags_9 },
    { 0x314a5d65, SUC_lexicon_tags_1 },
    { 0x248a5d65, SUC_lexicon_tags_17 },
    { 0xdb4a5d67, SUC_lexicon_tags_7 },
    { 0xf2aa5d67, SUC_lexicon_tags_108 },
    { 0x480a5d69, SUC_lexicon_tags_16 },
    { 0xd9ea5d6a, SUC_lexicon_tags_15 },
    { 0x314a5d67, SUC_lexicon_tags_25 },
    { 0xbc8a5d6c, SUC_lexicon_tags_1 },
    { 0x214a5d6c, SUC_lexicon_tags_29 },
    { 0x112a5d66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6a5d73, SUC_lexicon_tags_25 },
    { 0xd76a5d73, SUC_lexicon_tags_7 },
    { 0x366a5d75, SUC_lexicon_tags_33 },
    { 0x8dea5d76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aa5d7a, SUC_lexicon_tags_32 },
    { 0x17ca5d7b, SUC_lexicon_tags_7 },
    { 0xe40a5d7b, SUC_lexicon_tags_234 },
    { 0xaeea5d7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36a5d81, SUC_lexicon_tags_37 },
    { 0x10ea5d82, SUC_lexicon_tags_2 },
    { 0x7c8a5d83, SUC_lexicon_tags_43 },
    { 0xc50a5d81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15aa5d8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0a5d8e, SUC_lexicon_tags_35 },
    { 0x4c8a5d8f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4a5d92, SUC_lexicon_tags_11 },
    { 0x620a5d92, SUC_lexicon_tags_61 },
    { 0x608a5d94, SUC_lexicon_tags_6 },
    { 0x26a5d93, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x840a5d98, SUC_lexicon_tags_2 },
    { 0x980a5d99, SUC_lexicon_tags_46 },
    { 0xeca5d98, SUC_lexicon_tags_37 },
    { 0x4eca5d9b, SUC_lexicon_tags_37 },
    { 0x8aca5d9c, SUC_lexicon_tags_25 },
    { 0x4faa5d9c, SUC_lexicon_tags_60 },
    { 0xb1aa5d9e, SUC_lexicon_tags_17 },
    { 0x138a5d9f, SUC_lexicon_tags_15 },
    { 0xa22a5d9a, SUC_lexicon_tags_26 },
    { 0xc18a5da1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8a5da9, SUC_lexicon_tags_38 },
    { 0x43ca5da9, SUC_lexicon_tags_38 },
    { 0x92ea5dab, SUC_lexicon_tags_42 },
    { 0xb5ea5dab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8a5dae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd6a5db0, SUC_lexicon_tags_1 },
    { 0x88ea5db1, SUC_lexicon_tags_13 },
    { 0xde2a5db1, SUC_lexicon_tags_2 },
    { 0xaa2a5db3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca5dbb, SUC_lexicon_tags_15 },
    { 0x3a6a5dbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf4a5dbe, SUC_lexicon_tags_19 },
    { 0x966a5dbf, SUC_lexicon_tags_14 },
    { 0x90a5dbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ea5dc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1aa5dc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19ca5dc9, SUC_lexicon_tags_15 },
    { 0xa72a5dca, SUC_lexicon_tags_27 },
    { 0x4c0a5dcb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ea5dce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aa5dd2, SUC_lexicon_tags_12 },
    { 0xdbea5dd3, SUC_lexicon_tags_27 },
    { 0x5b8a5dd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76a5dd6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92aa5ddd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9baa5ddf, SUC_lexicon_tags_15 },
    { 0x512a5de0, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x2b0a5de2, SUC_lexicon_tags_1 },
    { 0xf78a5de3, SUC_lexicon_tags_63 },
    { 0x71aa5de4, SUC_lexicon_tags_37 },
    { 0x5bea5de5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874a5dee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x554a5df0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd40a5df2, SUC_lexicon_tags_12 },
    { 0x69ca5df3, SUC_lexicon_tags_11 },
    { 0xc54a5df3, SUC_lexicon_tags_797 },
    { 0x708a5df5, SUC_lexicon_tags_10 },
    { 0x1aa5df2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xacea5df8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaa5dfc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8a5e00, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4a5e04, SUC_lexicon_tags_28 },
    { 0xed4a5e05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1ca5e07, SUC_lexicon_tags_15 },
    { 0x5ea5e08, SUC_lexicon_tags_12 },
    { 0xa70a5e08, SUC_lexicon_tags_124 },
    { 0x73ea5e0a, SUC_lexicon_tags_37 },
    { 0x582a5e07, SUC_lexicon_tags_47 },
    { 0xa56a5e0c, SUC_lexicon_tags_60 },
    { 0x404a5e0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0a5e10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0a5e13, SUC_lexicon_tags_45 },
    { 0x950a5e14, SUC_lexicon_tags_16 },
    { 0x958a5e15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10a5e1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ca5e1f, SUC_lexicon_tags_27 },
    { 0x884a5e20, SUC_lexicon_tags_47 },
    { 0xdfaa5e21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd54a5e23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a5e27, SUC_lexicon_tags_25 },
    { 0xa08a5e28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86a5e2b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6a5e2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeea5e31, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5f8a5e33, SUC_lexicon_tags_63 },
    { 0x1d4a5e34, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeca5e37, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ca5e3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaca5e3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d8a5e40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x224a5e42, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4a5e46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa5e4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222a5e51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50a5e57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca5e5c, SUC_lexicon_tags_33 },
    { 0x28aa5e5d, SUC_lexicon_tags_2 },
    { 0xa8aa5e5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2a5e61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ea5e64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x518a5e67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d8a5e69, SUC_lexicon_tags_3 },
    { 0xa84a5e6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x770a5e6c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x368a5e6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ea5e74, SUC_lexicon_tags_34 },
    { 0x3ca5e75, SUC_lexicon_tags_12 },
    { 0xfcaa5e76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5baa5e78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ea5e7c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250a5e83, SUC_lexicon_tags_12 },
    { 0x3b6a5e84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x622a5e86, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4caa5e89, SUC_lexicon_tags_15 },
    { 0xb2ea5e89, SUC_lexicon_tags_27 },
    { 0x4a5e89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2a5e93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaea5e95, SUC_lexicon_tags_14 },
    { 0xb9ea5e96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74a5e99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57aa5e9e, SUC_lexicon_tags_11 },
    { 0x39ca5e9f, SUC_lexicon_tags_1 },
    { 0x8aca5ea0, SUC_lexicon_tags_129 },
    { 0xb72a5ea1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x370a5ea3, SUC_lexicon_tags_2 },
    { 0x7d0a5ea3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaa5ea7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aea5eab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156a5eb0, SUC_lexicon_tags_7 },
    { 0xed4a5eb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ca5eb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4a5ebc, SUC_lexicon_tags_0 },
    { 0x876a5ebd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x408a5ec0, SUC_lexicon_tags_38 },
    { 0x218a5ec0, SUC_lexicon_tags_25 },
    { 0x23aa5ec2, SUC_lexicon_tags_15 },
    { 0xe82a5ec2, SUC_lexicon_tags_7 },
    { 0x4fca5ec4, SUC_lexicon_tags_37 },
    { 0x318a5ec1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4ea5ec7, SUC_lexicon_tags_12 },
    { 0xe9aa5ec7, SUC_lexicon_tags_37 },
    { 0x5f6a5ec9, SUC_lexicon_tags_1 },
    { 0xf0ea5eca, SUC_lexicon_tags_42 },
    { 0xf1aa5eca, SUC_lexicon_tags_7 },
    { 0x69ca5ecc, SUC_lexicon_tags_37 },
    { 0x49aa5eca, SUC_lexicon_tags_55 },
    { 0x63ea5ece, SUC_lexicon_tags_15 },
    { 0xc60a5ecf, SUC_lexicon_tags_26 },
    { 0x152a5ec9, SUC_lexicon_tags_26 },
    { 0xcc8a5ecb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6a5ed5, SUC_lexicon_tags_12 },
    { 0x888a5ed5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360a5eda, SUC_lexicon_tags_31 },
    { 0xea2a5eda, SUC_lexicon_tags_705 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380a5ee4, SUC_lexicon_tags_37 },
    { 0x6e8a5ee5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528a5ee9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ea5eee, SUC_lexicon_tags_33 },
    { 0x90aa5eee, SUC_lexicon_tags_17 },
    { 0x132a5ef0, SUC_lexicon_tags_26 },
    { 0xe0aa5ef0, SUC_lexicon_tags_32 },
    { 0xaa0a5ef2, SUC_lexicon_tags_2 },
    { 0xedca5ef3, SUC_lexicon_tags_2 },
    { 0xdf0a5ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ca5ef6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0a5efc, SUC_lexicon_tags_39 },
    { 0x1e0a5efc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526a5f03, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xafea5f06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24aa5f09, SUC_lexicon_tags_38 },
    { 0xac2a5f0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2aa5f0c, SUC_lexicon_tags_37 },
    { 0x61aa5f0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ea5f10, SUC_lexicon_tags_31 },
    { 0x746a5f10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72a5f18, SUC_lexicon_tags_2 },
    { 0x650a5f19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf42a5f1b, SUC_lexicon_tags_9 },
    { 0xaa5f1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32a5f29, SUC_lexicon_tags_28 },
    { 0xaeca5f29, SUC_lexicon_tags_37 },
    { 0xa02a5f2b, SUC_lexicon_tags_63 },
    { 0xdb2a5f2b, SUC_lexicon_tags_7 },
    { 0xdfea5f2b, SUC_lexicon_tags_12 },
    { 0x1d0a5f2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea5f32, SUC_lexicon_tags_37 },
    { 0x9fea5f33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6a5f39, SUC_lexicon_tags_9 },
    { 0x1e4a5f3a, SUC_lexicon_tags_9 },
    { 0x9e8a5f39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ca5f3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe16a5f40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34a5f43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ca5f46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x704a5f48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ea5f4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2a5f50, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86a5f56, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8a5f5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcca5f5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0a5f66, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbc8a5f68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x364a5f6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552a5f6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca5f72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2a5f76, SUC_lexicon_tags_29 },
    { 0xd02a5f77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9aaa5f79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a0a5f7b, SUC_lexicon_tags_6 },
    { 0x8f0a5f7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fca5f7e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aa5f85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aa5f89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0a5f8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6a5f91, SUC_lexicon_tags_9 },
    { 0x778a5f92, SUC_lexicon_tags_12 },
    { 0x1caa5f92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274a5f9b, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8a5f9e, SUC_lexicon_tags_25 },
    { 0x604a5f9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3daa5fa1, SUC_lexicon_tags_38 },
    { 0xbdaa5fa2, SUC_lexicon_tags_10 },
    { 0xec4a5fa3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ea5fa6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacea5faa, SUC_lexicon_tags_15 },
    { 0x852a5faa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eca5faf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4cea5fb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdea5fb3, SUC_lexicon_tags_26 },
    { 0x79aa5fb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72a5fb7, SUC_lexicon_tags_14 },
    { 0xe30a5fb8, SUC_lexicon_tags_43 },
    { 0x632a5fb9, SUC_lexicon_tags_47 },
    { 0x190a5fba, SUC_lexicon_tags_1 },
    { 0xd64a5fbb, SUC_lexicon_tags_15 },
    { 0x5d4a5fb9, SUC_lexicon_tags_1 },
    { 0x4f0a5fbd, SUC_lexicon_tags_47 },
    { 0xb12a5fbe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2a5fc3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60a5fcb, SUC_lexicon_tags_77 },
    { 0x778a5fcc, SUC_lexicon_tags_10 },
    { 0x474a5fcd, SUC_lexicon_tags_37 },
    { 0x90aa5fce, SUC_lexicon_tags_91 },
    { 0x5b2a5fce, SUC_lexicon_tags_77 },
    { 0xd3aa5fce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdea5fd7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ca5fda, SUC_lexicon_tags_12 },
    { 0x860a5fda, SUC_lexicon_tags_25 },
    { 0xf94a5fdc, SUC_lexicon_tags_1 },
    { 0xe2aa5fda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0aa5fdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b2a5fe1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aa5fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d8a5fe7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136a5feb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x540a5fee, SUC_lexicon_tags_43 },
    { 0xc2ea5fee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xea8a5ff1, SUC_lexicon_tags_30 },
    { 0x79ca5ff1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9faa5ff6, SUC_lexicon_tags_37 },
    { 0x9fca5ff7, SUC_lexicon_tags_38 },
    { 0xeeca5ff6, SUC_lexicon_tags_57 },
    { 0x13aa5ff6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x566a5ffb, SUC_lexicon_tags_7 },
    { 0x2a4a5ffc, SUC_lexicon_tags_12 },
    { 0xbaca5ffc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49ca5fff, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2a6007, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388a600c, SUC_lexicon_tags_31 },
    { 0xbc4a600d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xddea600f, SUC_lexicon_tags_31 },
    { 0xdeea6010, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5e8a6012, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x19aa6014, SUC_lexicon_tags_18 },
    { 0x450a6014, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x870a6017, SUC_lexicon_tags_17 },
    { 0x9dea6018, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a601c, SUC_lexicon_tags_28 },
    { 0xab2a601c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378a6025, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0a602a, SUC_lexicon_tags_15 },
    { 0x3e2a602b, SUC_lexicon_tags_10 },
    { 0xf6ea602b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6a6030, SUC_lexicon_tags_60 },
    { 0x330a6030, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x480a6033, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4a6037, SUC_lexicon_tags_37 },
    { 0xd90a6037, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2a6040, SUC_lexicon_tags_2 },
    { 0x7d0a6041, SUC_lexicon_tags_37 },
    { 0x9b0a6040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78a6045, SUC_lexicon_tags_19 },
    { 0x60ea6046, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x490a6048, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xdbca604a, SUC_lexicon_tags_37 },
    { 0x4f2a604b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4baa604d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbca6050, SUC_lexicon_tags_25 },
    { 0x21ea6050, SUC_lexicon_tags_31 },
    { 0xba6a6052, SUC_lexicon_tags_12 },
    { 0x362a6053, SUC_lexicon_tags_33 },
    { 0x142a6053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6ea6056, SUC_lexicon_tags_9 },
    { 0xb2aa6056, SUC_lexicon_tags_2 },
    { 0x6ea6058, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x846a605a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2a605d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfbca605f, SUC_lexicon_tags_60 },
    { 0xe68a605f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0a6065, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f0a6067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe64a6069, SUC_lexicon_tags_29 },
    { 0xc6ca6069, SUC_lexicon_tags_25 },
    { 0xb9ea6069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2eca606d, SUC_lexicon_tags_47 },
    { 0xe0aa606e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x940a6071, SUC_lexicon_tags_48 },
    { 0x2eca6072, SUC_lexicon_tags_14 },
    { 0x5c0a6071, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x944a6076, SUC_lexicon_tags_37 },
    { 0x424a6076, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb90a6079, SUC_lexicon_tags_85 },
    { 0xecaa6079, SUC_lexicon_tags_27 },
    { 0x5fea607b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x73ca607d, SUC_lexicon_tags_17 },
    { 0x924a607d, SUC_lexicon_tags_12 },
    { 0x22aa607f, SUC_lexicon_tags_82 },
    { 0xbbea607f, SUC_lexicon_tags_84 },
    { 0x110a6080, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe20a6083, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00a6086, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752a608a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedca608f, SUC_lexicon_tags_35 },
    { 0x704a6090, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128a6096, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x796a6098, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51aa609a, SUC_lexicon_tags_17 },
    { 0x85ca609a, SUC_lexicon_tags_0 },
    { 0x44ea609c, SUC_lexicon_tags_25 },
    { 0xa0a609c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62a60a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ea60a4, SUC_lexicon_tags_47 },
    { 0xd54a60a5, SUC_lexicon_tags_17 },
    { 0xa0ea60a5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x352a60a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42a60b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2a60b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf28a60b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa10a60b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ca60bc, SUC_lexicon_tags_37 },
    { 0xa8aa60bd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddca60c9, SUC_lexicon_tags_37 },
    { 0x28ca60ca, SUC_lexicon_tags_7 },
    { 0x6f8a60cb, SUC_lexicon_tags_2 },
    { 0x816a60cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aa60d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf28a60d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37ea60d4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x22aa60d7, SUC_lexicon_tags_11 },
    { 0xa2ca60d7, SUC_lexicon_tags_25 },
    { 0x5caa60d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x722a60dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x252a60df, SUC_lexicon_tags_32 },
    { 0x548a60e0, SUC_lexicon_tags_23 },
    { 0x570a60e1, SUC_lexicon_tags_17 },
    { 0x89ea60e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf16a60e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6a60e8, SUC_lexicon_tags_70 },
    { 0x1aa60e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320a60fa, SUC_lexicon_tags_49 },
    { 0x21ea60fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2a6101, SUC_lexicon_tags_11 },
    { 0xb74a6102, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ea6108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ea610c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0a610f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8a6113, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0a6117, SUC_lexicon_tags_25 },
    { 0x8aa6117, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xadea611a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde6a611c, SUC_lexicon_tags_61 },
    { 0x594a611c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24ca611f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54a6129, SUC_lexicon_tags_19 },
    { 0xeaea6129, SUC_lexicon_tags_37 },
    { 0x576a612a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aca6132, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722a6137, SUC_lexicon_tags_0 },
    { 0x772a6137, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794a613d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c6a613f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89ea6141, SUC_lexicon_tags_12 },
    { 0x63aa6142, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf40a6144, SUC_lexicon_tags_27 },
    { 0x6a0a6145, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ea6148, SUC_lexicon_tags_8 },
    { 0xdbca6149, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7aa614b, SUC_lexicon_tags_42 },
    { 0xe50a614c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ca614f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fea6153, SUC_lexicon_tags_15 },
    { 0x5d0a6153, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eca615c, SUC_lexicon_tags_15 },
    { 0xa82a615c, SUC_lexicon_tags_38 },
    { 0x26ca615d, SUC_lexicon_tags_11 },
    { 0x38ca615f, SUC_lexicon_tags_11 },
    { 0x4ea6160, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48ca6162, SUC_lexicon_tags_15 },
    { 0xc9ea6163, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ea6166, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eca616c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aca6172, SUC_lexicon_tags_49 },
    { 0x2e8a6172, SUC_lexicon_tags_9 },
    { 0x7fca6174, SUC_lexicon_tags_37 },
    { 0xc0aa6174, SUC_lexicon_tags_201 },
    { 0x702a6174, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca6178, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdea617a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x124a617c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2a617f, SUC_lexicon_tags_7 },
    { 0x524a6180, SUC_lexicon_tags_7 },
    { 0x3dca6180, SUC_lexicon_tags_32 },
    { 0x824a6182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94a6185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f2a6187, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3cca6189, SUC_lexicon_tags_45 },
    { 0x882a6189, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492a618e, SUC_lexicon_tags_26 },
    { 0xae8a618f, SUC_lexicon_tags_70 },
    { 0xafca6190, SUC_lexicon_tags_12 },
    { 0xff8a6191, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaca619a, SUC_lexicon_tags_37 },
    { 0x9e4a619b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70aa61a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ca61a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x524a61a9, SUC_lexicon_tags_49 },
    { 0x276a61aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaa4a61ac, SUC_lexicon_tags_37 },
    { 0xc5aa61ad, SUC_lexicon_tags_60 },
    { 0xcc4a61ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40a61b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bca61b6, SUC_lexicon_tags_275 },
    { 0x4e8a61b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcca61c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9ea61c3, SUC_lexicon_tags_25 },
    { 0xf00a61c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad0a61c6, SUC_lexicon_tags_15 },
    { 0x534a61c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24a61cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536a61cf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894a61d5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfdaa61d7, SUC_lexicon_tags_29 },
    { 0xd0ca61d8, SUC_lexicon_tags_45 },
    { 0xc46a61d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aa61de, SUC_lexicon_tags_25 },
    { 0x130a61df, SUC_lexicon_tags_31 },
    { 0x7d4a61e0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648a61e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea61e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ea61ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44a61f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38a61f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4a61fd, SUC_lexicon_tags_25 },
    { 0xaf8a61fe, SUC_lexicon_tags_230 },
    { 0x294a61fd, SUC_lexicon_tags_45 },
    { 0x7a0a61fe, SUC_lexicon_tags_17 },
    { 0x922a61ff, SUC_lexicon_tags_31 },
    { 0x25ca61fe, SUC_lexicon_tags_19 },
    { 0xe22a61fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xafea6206, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136a620a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8a6216, SUC_lexicon_tags_68 },
    { 0xc22a6217, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4a621a, SUC_lexicon_tags_9 },
    { 0x724a621b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8a6222, SUC_lexicon_tags_38 },
    { 0x8e2a6223, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176a6228, SUC_lexicon_tags_25 },
    { 0xc7ea6228, SUC_lexicon_tags_2 },
    { 0xaf2a6228, SUC_lexicon_tags_19 },
    { 0x5c2a622b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ea622e, SUC_lexicon_tags_11 },
    { 0xed0a622f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92a623e, SUC_lexicon_tags_68 },
    { 0x68ea623f, SUC_lexicon_tags_12 },
    { 0x244a623e, SUC_lexicon_tags_15 },
    { 0xd06a6241, SUC_lexicon_tags_45 },
    { 0xf54a6241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x260a6244, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaa624c, SUC_lexicon_tags_27 },
    { 0x2faa624d, SUC_lexicon_tags_17 },
    { 0xb64a624e, SUC_lexicon_tags_23 },
    { 0xf34a624e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbea6255, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374a625a, SUC_lexicon_tags_47 },
    { 0x72ea625b, SUC_lexicon_tags_25 },
    { 0x5c6a625b, SUC_lexicon_tags_82 },
    { 0x536a625d, SUC_lexicon_tags_43 },
    { 0x8ea625a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfaa6262, SUC_lexicon_tags_37 },
    { 0xc1aa6263, SUC_lexicon_tags_33 },
    { 0x8d8a6262, SUC_lexicon_tags_15 },
    { 0x880a6265, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x53aa6267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70a626a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64ea626c, SUC_lexicon_tags_10 },
    { 0xa38a626d, SUC_lexicon_tags_9 },
    { 0x9aca626e, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6272, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6277, SUC_lexicon_tags_30 },
    { 0xaa8a6277, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a4a627a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ca6286, SUC_lexicon_tags_11 },
    { 0xbc0a6287, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x152a6289, SUC_lexicon_tags_17 },
    { 0x3b6a628a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xebca628c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xea4a628e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc10a6290, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x586a6293, SUC_lexicon_tags_37 },
    { 0x39ca6294, SUC_lexicon_tags_37 },
    { 0x9a0a6295, SUC_lexicon_tags_12 },
    { 0x842a6293, SUC_lexicon_tags_31 },
    { 0xa32a6294, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ea629c, SUC_lexicon_tags_47 },
    { 0xdea629c, SUC_lexicon_tags_15 },
    { 0x10aa629c, SUC_lexicon_tags_8 },
    { 0x14a629f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910a62a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4a62a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ca62ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf12a62ae, SUC_lexicon_tags_4 },
    { 0xad8a62ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73aa62b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x518a62b4, SUC_lexicon_tags_82 },
    { 0xbbea62b5, SUC_lexicon_tags_15 },
    { 0x3bca62b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x596a62b8, SUC_lexicon_tags_34 },
    { 0x6fca62b9, SUC_lexicon_tags_17 },
    { 0xf70a62ba, SUC_lexicon_tags_2 },
    { 0xa48a62bb, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0a62bf, SUC_lexicon_tags_5 },
    { 0x3fca62c0, SUC_lexicon_tags_15 },
    { 0x45ea62bf, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108a62c5, SUC_lexicon_tags_21 },
    { 0xffca62c6, SUC_lexicon_tags_11 },
    { 0xa0ca62c5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xfc2a62c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x400a62cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3c6a62cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacca62d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26a62d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2a62d7, SUC_lexicon_tags_38 },
    { 0xce4a62d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890a62dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94a62e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeea62e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ca62e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4a62f1, SUC_lexicon_tags_45 },
    { 0x7dea62f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2a62f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdfca62fb, SUC_lexicon_tags_55 },
    { 0xdaaa62fb, SUC_lexicon_tags_31 },
    { 0xeb8a62fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfbca62ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0a6302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6a6305, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x830a6308, SUC_lexicon_tags_43 },
    { 0x1d0a6309, SUC_lexicon_tags_15 },
    { 0x768a630a, SUC_lexicon_tags_49 },
    { 0xe72a630a, SUC_lexicon_tags_11 },
    { 0xeaca630c, SUC_lexicon_tags_47 },
    { 0xb2a630d, SUC_lexicon_tags_1 },
    { 0x508a630e, SUC_lexicon_tags_13 },
    { 0x618a630d, SUC_lexicon_tags_15 },
    { 0x52a6310, SUC_lexicon_tags_37 },
    { 0x17ca630f, SUC_lexicon_tags_23 },
    { 0x852a6312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca2a6314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbea6318, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefca631d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d0a631f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xba0a6321, SUC_lexicon_tags_47 },
    { 0x492a6321, SUC_lexicon_tags_25 },
    { 0xb62a6323, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeea632a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8a2a632c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x466a632f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46aa6336, SUC_lexicon_tags_17 },
    { 0x574a6336, SUC_lexicon_tags_25 },
    { 0x66ea6338, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaa633d, SUC_lexicon_tags_63 },
    { 0x8daa633d, SUC_lexicon_tags_11 },
    { 0x132a633e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168a6343, SUC_lexicon_tags_9 },
    { 0x526a6344, SUC_lexicon_tags_60 },
    { 0x746a6344, SUC_lexicon_tags_25 },
    { 0xe98a6346, SUC_lexicon_tags_0 },
    { 0xd62a6347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6baa6349, SUC_lexicon_tags_0 },
    { 0x14ea634a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6a634d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bca634f, SUC_lexicon_tags_12 },
    { 0x278a6350, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x386a6352, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x74aa6354, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x790a6356, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda8a6358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902a6360, SUC_lexicon_tags_37 },
    { 0xe90a6361, SUC_lexicon_tags_7 },
    { 0x808a6362, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa46a6364, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x56a6366, SUC_lexicon_tags_47 },
    { 0xc0ca6367, SUC_lexicon_tags_25 },
    { 0x68a6366, SUC_lexicon_tags_2 },
    { 0x980a6367, SUC_lexicon_tags_7 },
    { 0x9cea636a, SUC_lexicon_tags_27 },
    { 0xa82a636b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daa6370, SUC_lexicon_tags_25 },
    { 0xc96a6370, SUC_lexicon_tags_1 },
    { 0x88a6372, SUC_lexicon_tags_13 },
    { 0xc34a6373, SUC_lexicon_tags_37 },
    { 0x382a6374, SUC_lexicon_tags_14 },
    { 0x418a6373, SUC_lexicon_tags_7 },
    { 0x844a6376, SUC_lexicon_tags_12 },
    { 0xbeaa6377, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2a637b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x342a637e, SUC_lexicon_tags_37 },
    { 0xca8a637e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b6a6381, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x174a6383, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6a638a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x5c6a638c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x958a638f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfeca6391, SUC_lexicon_tags_47 },
    { 0xbe2a6392, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x994a6394, SUC_lexicon_tags_25 },
    { 0xe4ea6395, SUC_lexicon_tags_43 },
    { 0x2bca6396, SUC_lexicon_tags_7 },
    { 0xbc2a6397, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ca639a, SUC_lexicon_tags_47 },
    { 0x9baa639b, SUC_lexicon_tags_8 },
    { 0x9f4a639c, SUC_lexicon_tags_25 },
    { 0xe7aa639d, SUC_lexicon_tags_11 },
    { 0xe80a639e, SUC_lexicon_tags_37 },
    { 0x31aa639f, SUC_lexicon_tags_9 },
    { 0xe40a639e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x464a63a2, SUC_lexicon_tags_45 },
    { 0xa9ca63a2, SUC_lexicon_tags_23 },
    { 0x570a63a4, SUC_lexicon_tags_63 },
    { 0x1aaa63a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342a63ab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d6a63ad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x818a63af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa6a63b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca63b5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ca63b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3f6a63ba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1ca63bc, SUC_lexicon_tags_37 },
    { 0x258a63bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x348a63bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca63c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8a63c7, SUC_lexicon_tags_87 },
    { 0x55ea63c8, SUC_lexicon_tags_25 },
    { 0x796a63c8, SUC_lexicon_tags_1 },
    { 0x9a0a63ca, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7aea63cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0a63cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ca63d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ea63d7, SUC_lexicon_tags_11 },
    { 0x850a63d8, SUC_lexicon_tags_6 },
    { 0x508a63d8, SUC_lexicon_tags_37 },
    { 0xddea63d9, SUC_lexicon_tags_38 },
    { 0x578a63db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa56a63dd, SUC_lexicon_tags_30 },
    { 0xb00a63de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x326a63e1, SUC_lexicon_tags_61 },
    { 0x2ca63e1, SUC_lexicon_tags_17 },
    { 0x5eaa63e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd42a63e5, SUC_lexicon_tags_87 },
    { 0xc74a63e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0a63eb, SUC_lexicon_tags_34 },
    { 0x690a63eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x134a63ef, SUC_lexicon_tags_34 },
    { 0x5fea63ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368a63f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774a63f8, SUC_lexicon_tags_11 },
    { 0xccea63f9, SUC_lexicon_tags_87 },
    { 0x71aa63fa, SUC_lexicon_tags_4 },
    { 0x838a63fa, SUC_lexicon_tags_25 },
    { 0xc06a63fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56a6401, SUC_lexicon_tags_65 },
    { 0x72ea6402, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ea6405, SUC_lexicon_tags_12 },
    { 0x35ca6406, SUC_lexicon_tags_25 },
    { 0xdeea6406, SUC_lexicon_tags_25 },
    { 0x270a6408, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6a640c, SUC_lexicon_tags_27 },
    { 0x542a640c, SUC_lexicon_tags_7 },
    { 0x1d6a640e, SUC_lexicon_tags_60 },
    { 0x93ea640f, SUC_lexicon_tags_14 },
    { 0xf4a640e, SUC_lexicon_tags_63 },
    { 0xb00a6411, SUC_lexicon_tags_2 },
    { 0x2caa6412, SUC_lexicon_tags_15 },
    { 0x5c0a6411, SUC_lexicon_tags_61 },
    { 0xe44a6414, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34ca6416, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaa6421, SUC_lexicon_tags_12 },
    { 0x726a6422, SUC_lexicon_tags_1163 },
    { 0, NULL },
    { 0x528a6424, SUC_lexicon_tags_27 },
    { 0x5daa6425, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x530a6427, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ca6430, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2aa6432, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8a6435, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc98a6437, SUC_lexicon_tags_1 },
    { 0x822a6437, SUC_lexicon_tags_30 },
    { 0x27aa6437, SUC_lexicon_tags_17 },
    { 0xe5aa643a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aca643f, SUC_lexicon_tags_1 },
    { 0x6c4a643f, SUC_lexicon_tags_23 },
    { 0x460a6441, SUC_lexicon_tags_15 },
    { 0x98ca6442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x708a6444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x518a6446, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ca644a, SUC_lexicon_tags_12 },
    { 0xf40a644b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ea6451, SUC_lexicon_tags_37 },
    { 0x684a6452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ea6458, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4a645b, SUC_lexicon_tags_37 },
    { 0xb2aa645c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x572a645e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4a6462, SUC_lexicon_tags_37 },
    { 0x388a6463, SUC_lexicon_tags_7 },
    { 0xbb2a6464, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaa6467, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x540a646a, SUC_lexicon_tags_127 },
    { 0xf10a646b, SUC_lexicon_tags_17 },
    { 0xb22a646c, SUC_lexicon_tags_30 },
    { 0xf96a646c, SUC_lexicon_tags_112 },
    { 0xc8ca646e, SUC_lexicon_tags_11 },
    { 0x58ca646f, SUC_lexicon_tags_19 },
    { 0x748a6470, SUC_lexicon_tags_32 },
    { 0x54aa6471, SUC_lexicon_tags_39 },
    { 0xf70a6470, SUC_lexicon_tags_17 },
    { 0x32a6473, SUC_lexicon_tags_81 },
    { 0x84a6473, SUC_lexicon_tags_31 },
    { 0x634a6475, SUC_lexicon_tags_15 },
    { 0x6dea6476, SUC_lexicon_tags_12 },
    { 0xa6a6477, SUC_lexicon_tags_17 },
    { 0x6b6a6475, SUC_lexicon_tags_47 },
    { 0x3bea6473, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ea647e, SUC_lexicon_tags_28 },
    { 0x782a647e, SUC_lexicon_tags_9 },
    { 0x23aa6480, SUC_lexicon_tags_48 },
    { 0x988a6481, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8a6488, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ea648c, SUC_lexicon_tags_897 },
    { 0x430a648c, SUC_lexicon_tags_2 },
    { 0x1e2a648c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd08a6490, SUC_lexicon_tags_1 },
    { 0x636a6490, SUC_lexicon_tags_31 },
    { 0x53ca6492, SUC_lexicon_tags_8 },
    { 0x5a8a6492, SUC_lexicon_tags_23 },
    { 0x6b6a6494, SUC_lexicon_tags_25 },
    { 0xf52a6495, SUC_lexicon_tags_8 },
    { 0x8d2a6496, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44a649a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498a649f, SUC_lexicon_tags_37 },
    { 0x3bca64a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ca64a7, SUC_lexicon_tags_12 },
    { 0xabaa64a8, SUC_lexicon_tags_55 },
    { 0xa74a64a9, SUC_lexicon_tags_7 },
    { 0x5e2a64a7, SUC_lexicon_tags_0 },
    { 0x8a64a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ea64b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc7aa64b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caa64b8, SUC_lexicon_tags_13 },
    { 0xc90a64b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730a64bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2a64c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfca64c2, SUC_lexicon_tags_15 },
    { 0xe4ea64c3, SUC_lexicon_tags_63 },
    { 0xb52a64c3, SUC_lexicon_tags_45 },
    { 0x772a64c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ca64c8, SUC_lexicon_tags_37 },
    { 0x79aa64c9, SUC_lexicon_tags_7 },
    { 0x94ca64c9, SUC_lexicon_tags_79 },
    { 0xa7aa64c9, SUC_lexicon_tags_33 },
    { 0xebea64cc, SUC_lexicon_tags_11 },
    { 0x890a64cc, SUC_lexicon_tags_19 },
    { 0xac4a64ce, SUC_lexicon_tags_2 },
    { 0xb70a64ca, SUC_lexicon_tags_61 },
    { 0x3ea64c9, SUC_lexicon_tags_39 },
    { 0xa38a64cf, SUC_lexicon_tags_25 },
    { 0xb3ea64d2, SUC_lexicon_tags_24 },
    { 0x1caa64d3, SUC_lexicon_tags_29 },
    { 0x906a64d4, SUC_lexicon_tags_16 },
    { 0xe82a64d3, SUC_lexicon_tags_47 },
    { 0xa4a64d6, SUC_lexicon_tags_12 },
    { 0xad0a64d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x172a64da, SUC_lexicon_tags_47 },
    { 0x4a64da, SUC_lexicon_tags_19 },
    { 0x72aa64dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80a64e5, SUC_lexicon_tags_37 },
    { 0x1f4a64e6, SUC_lexicon_tags_0 },
    { 0xb42a64e7, SUC_lexicon_tags_11 },
    { 0x4e0a64e5, SUC_lexicon_tags_7 },
    { 0xcf4a64e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca64ed, SUC_lexicon_tags_19 },
    { 0x39ea64ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04a64f3, SUC_lexicon_tags_17 },
    { 0xd94a64f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242a64fb, SUC_lexicon_tags_27 },
    { 0xbbca64fc, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eea64ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882a6504, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaa6509, SUC_lexicon_tags_12 },
    { 0x3c4a650a, SUC_lexicon_tags_12 },
    { 0xa06a650a, SUC_lexicon_tags_43 },
    { 0x470a650b, SUC_lexicon_tags_60 },
    { 0xbd8a650a, SUC_lexicon_tags_34 },
    { 0x2f8a650c, SUC_lexicon_tags_55 },
    { 0x60ea650f, SUC_lexicon_tags_15 },
    { 0x27ea650f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8a6515, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3c6a6517, SUC_lexicon_tags_25 },
    { 0x80aa6518, SUC_lexicon_tags_19 },
    { 0x460a6517, SUC_lexicon_tags_23 },
    { 0x782a6518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36a651d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dea6521, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28ea6523, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2a6526, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69ea6528, SUC_lexicon_tags_0 },
    { 0x88a6529, SUC_lexicon_tags_6 },
    { 0x5c0a652a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67ca652c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa92a652e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ca6532, SUC_lexicon_tags_31 },
    { 0x8eca6532, SUC_lexicon_tags_21 },
    { 0xb42a6534, SUC_lexicon_tags_2 },
    { 0x8c6a6535, SUC_lexicon_tags_16 },
    { 0x88a6536, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8a653b, SUC_lexicon_tags_17 },
    { 0x2eea653c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512a6541, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a6a6543, SUC_lexicon_tags_100 },
    { 0xca8a6544, SUC_lexicon_tags_37 },
    { 0xa98a6545, SUC_lexicon_tags_37 },
    { 0x78a6546, SUC_lexicon_tags_8 },
    { 0x2aa6544, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ca6551, SUC_lexicon_tags_17 },
    { 0xbb2a6552, SUC_lexicon_tags_11 },
    { 0xee2a6553, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x82aa6556, SUC_lexicon_tags_37 },
    { 0xcd2a6557, SUC_lexicon_tags_1 },
    { 0x29ca6557, SUC_lexicon_tags_26 },
    { 0xfb0a6556, SUC_lexicon_tags_15 },
    { 0x826a6556, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x606a655d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4ca655f, SUC_lexicon_tags_15 },
    { 0xcbca655f, SUC_lexicon_tags_1 },
    { 0x236a6561, SUC_lexicon_tags_33 },
    { 0xba8a6562, SUC_lexicon_tags_19 },
    { 0xc3ca6560, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8a6567, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46a6570, SUC_lexicon_tags_55 },
    { 0xf8ca6571, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ca6574, SUC_lexicon_tags_17 },
    { 0xe00a6575, SUC_lexicon_tags_11 },
    { 0x166a6576, SUC_lexicon_tags_32 },
    { 0x732a6576, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408a6584, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x808a6587, SUC_lexicon_tags_6 },
    { 0x100a6587, SUC_lexicon_tags_49 },
    { 0xad8a6589, SUC_lexicon_tags_17 },
    { 0x1eaa658a, SUC_lexicon_tags_25 },
    { 0xf5ca658b, SUC_lexicon_tags_124 },
    { 0x5bea658c, SUC_lexicon_tags_82 },
    { 0xf7aa658d, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaca6590, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xda4a6592, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306a659a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfea659e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ea65a2, SUC_lexicon_tags_6 },
    { 0x550a65a3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa2aa65a5, SUC_lexicon_tags_45 },
    { 0xd72a65a5, SUC_lexicon_tags_15 },
    { 0x3c8a65a6, SUC_lexicon_tags_7 },
    { 0x606a65a8, SUC_lexicon_tags_26 },
    { 0x136a65a8, SUC_lexicon_tags_60 },
    { 0xffca65a8, SUC_lexicon_tags_37 },
    { 0x69ca65ab, SUC_lexicon_tags_40 },
    { 0xd0aa65ac, SUC_lexicon_tags_0 },
    { 0x74ea65ad, SUC_lexicon_tags_2 },
    { 0xacea65ac, SUC_lexicon_tags_38 },
    { 0x27aa65af, SUC_lexicon_tags_11 },
    { 0x9e4a65af, SUC_lexicon_tags_17 },
    { 0x14ca65a5, SUC_lexicon_tags_49 },
    { 0x610a65b2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8a65b5, SUC_lexicon_tags_25 },
    { 0x6f0a65b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54a65be, SUC_lexicon_tags_25 },
    { 0x9faa65bf, SUC_lexicon_tags_23 },
    { 0x736a65c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e2a65c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd0a65c4, SUC_lexicon_tags_11 },
    { 0xfa8a65c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7cca65c7, SUC_lexicon_tags_9 },
    { 0xbd8a65c7, SUC_lexicon_tags_14 },
    { 0xc30a65c9, SUC_lexicon_tags_3 },
    { 0xbaca65ca, SUC_lexicon_tags_7 },
    { 0x49ca65cb, SUC_lexicon_tags_2 },
    { 0xf2ca65c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6a65d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e6a65d3, SUC_lexicon_tags_9 },
    { 0xbf8a65d3, SUC_lexicon_tags_15 },
    { 0x86ea65d5, SUC_lexicon_tags_60 },
    { 0xf6a65d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2faa65d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154a65dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52a65e0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a65e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8a65e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bca65f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x712a65f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa62a65f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45aa65f9, SUC_lexicon_tags_44 },
    { 0xc2ea65fa, SUC_lexicon_tags_38 },
    { 0xa66a65f9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232a6601, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x430a6603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a6a6605, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x586a6608, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458a6610, SUC_lexicon_tags_30 },
    { 0x51aa6611, SUC_lexicon_tags_38 },
    { 0x2cea6612, SUC_lexicon_tags_35 },
    { 0xe66a6611, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94a6617, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896a661d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea6621, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6624, SUC_lexicon_tags_1 },
    { 0xd2ea6625, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ea662c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x644a662e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x572a6631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb12a6633, SUC_lexicon_tags_15 },
    { 0x84a6634, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ea6638, SUC_lexicon_tags_47 },
    { 0xd94a6638, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x384a663c, SUC_lexicon_tags_12 },
    { 0x292a663d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbca6641, SUC_lexicon_tags_68 },
    { 0x2fca6642, SUC_lexicon_tags_39 },
    { 0x864a6642, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4a6646, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44a664e, SUC_lexicon_tags_37 },
    { 0x900a664f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb4a6651, SUC_lexicon_tags_47 },
    { 0x38ca6651, SUC_lexicon_tags_46 },
    { 0x796a6653, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x878a6656, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2bea6658, SUC_lexicon_tags_26 },
    { 0x9aa6658, SUC_lexicon_tags_15 },
    { 0x81ca665a, SUC_lexicon_tags_17 },
    { 0xa26a665b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf4ca665d, SUC_lexicon_tags_26 },
    { 0x1ca665e, SUC_lexicon_tags_1 },
    { 0xe46a665d, SUC_lexicon_tags_14 },
    { 0xf32a665e, SUC_lexicon_tags_11 },
    { 0xaaa6661, SUC_lexicon_tags_2 },
    { 0x904a6662, SUC_lexicon_tags_14 },
    { 0x1e0a6662, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x528a6665, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x266a6667, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xed6a6669, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aa666e, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xef8a6670, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x9eea6672, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ea6674, SUC_lexicon_tags_14 },
    { 0x8faa6674, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x270a6677, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ca667c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60a667f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ca6682, SUC_lexicon_tags_15 },
    { 0xa3ca6682, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964a6687, SUC_lexicon_tags_11 },
    { 0xc8ea6688, SUC_lexicon_tags_38 },
    { 0xb94a6689, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16ca668b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20a668f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf28a6691, SUC_lexicon_tags_49 },
    { 0x244a6691, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736a6696, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x666a6698, SUC_lexicon_tags_2 },
    { 0x428a6699, SUC_lexicon_tags_45 },
    { 0x6faa669a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56a669f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ea66a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44a66a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00a66ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x434a66ae, SUC_lexicon_tags_15 },
    { 0xdb0a66af, SUC_lexicon_tags_17 },
    { 0xef6a66ae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26a66b4, SUC_lexicon_tags_38 },
    { 0xb46a66b5, SUC_lexicon_tags_23 },
    { 0xeea66b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6dea66b8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbaaa66ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11ca66bc, SUC_lexicon_tags_9 },
    { 0xc00a66bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4ea66bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04a66c2, SUC_lexicon_tags_29 },
    { 0x788a66c3, SUC_lexicon_tags_23 },
    { 0x284a66c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ca66ca, SUC_lexicon_tags_12 },
    { 0x8aaa66cb, SUC_lexicon_tags_19 },
    { 0xe20a66ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf58a66ce, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3d2a66d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa66a66d2, SUC_lexicon_tags_38 },
    { 0xe6aa66d3, SUC_lexicon_tags_25 },
    { 0x12a66d2, SUC_lexicon_tags_1 },
    { 0x66ca66d5, SUC_lexicon_tags_7 },
    { 0x41ea66d5, SUC_lexicon_tags_10 },
    { 0xdb2a66d7, SUC_lexicon_tags_27 },
    { 0x79aa66d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x866a66da, SUC_lexicon_tags_14 },
    { 0xb8ca66db, SUC_lexicon_tags_7 },
    { 0xad8a66dc, SUC_lexicon_tags_11 },
    { 0x854a66da, SUC_lexicon_tags_31 },
    { 0x60aa66de, SUC_lexicon_tags_1 },
    { 0xc6ea66df, SUC_lexicon_tags_1 },
    { 0xf44a66dd, SUC_lexicon_tags_1 },
    { 0x4a4a66e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ca66e8, SUC_lexicon_tags_1 },
    { 0x93ea66e9, SUC_lexicon_tags_11 },
    { 0xbf2a66ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8a66ed, SUC_lexicon_tags_37 },
    { 0xe56a66ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20a66f3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x314a66f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2a66ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaa6705, SUC_lexicon_tags_5 },
    { 0x9bea6706, SUC_lexicon_tags_12 },
    { 0x1fca6707, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6a6710, SUC_lexicon_tags_244 },
    { 0x374a6711, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x902a6713, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc4ca6715, SUC_lexicon_tags_542 },
    { 0xfc0a6716, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daa671a, SUC_lexicon_tags_47 },
    { 0x1fea671b, SUC_lexicon_tags_8 },
    { 0x220a671c, SUC_lexicon_tags_34 },
    { 0xbd6a671c, SUC_lexicon_tags_60 },
    { 0x706a671e, SUC_lexicon_tags_37 },
    { 0x6f6a671e, SUC_lexicon_tags_25 },
    { 0x5b8a6720, SUC_lexicon_tags_7 },
    { 0xe8ea6720, SUC_lexicon_tags_11 },
    { 0xa04a671e, SUC_lexicon_tags_15 },
    { 0xcfca671f, SUC_lexicon_tags_39 },
    { 0xb18a6724, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256a672a, SUC_lexicon_tags_23 },
    { 0x6b2a672b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x170a672d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46a6731, SUC_lexicon_tags_11 },
    { 0xa94a6731, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfbea6734, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecca6736, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04a673f, SUC_lexicon_tags_12 },
    { 0x1b6a673f, SUC_lexicon_tags_28 },
    { 0x7e8a6741, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x61aa6744, SUC_lexicon_tags_8 },
    { 0xf7ea6744, SUC_lexicon_tags_37 },
    { 0x87aa6746, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x136a6748, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16a674e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x27ea6750, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x608a6753, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x71ca6755, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddea675c, SUC_lexicon_tags_47 },
    { 0x9baa675d, SUC_lexicon_tags_37 },
    { 0x514a675e, SUC_lexicon_tags_7 },
    { 0x43ea675e, SUC_lexicon_tags_47 },
    { 0x478a6760, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434a6766, SUC_lexicon_tags_37 },
    { 0xb26a6767, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ca676d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x170a676f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210a6774, SUC_lexicon_tags_11 },
    { 0x77aa6775, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ea6778, SUC_lexicon_tags_0 },
    { 0x4d6a6779, SUC_lexicon_tags_2 },
    { 0x4cea677a, SUC_lexicon_tags_3 },
    { 0xe82a6778, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35aa6781, SUC_lexicon_tags_11 },
    { 0xe18a6782, SUC_lexicon_tags_1 },
    { 0x34a6783, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476a6787, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd9ea6789, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e6a678b, SUC_lexicon_tags_2 },
    { 0xa18a678c, SUC_lexicon_tags_25 },
    { 0x5b4a678d, SUC_lexicon_tags_17 },
    { 0x39ea678d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca6791, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6a679c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248a67a3, SUC_lexicon_tags_17 },
    { 0x770a67a4, SUC_lexicon_tags_11 },
    { 0x7e2a67a5, SUC_lexicon_tags_12 },
    { 0xc0ca67a3, SUC_lexicon_tags_29 },
    { 0xfe6a67a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38a67ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x550a67ae, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x46aa67b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0a67b4, SUC_lexicon_tags_11 },
    { 0x388a67b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x174a67b8, SUC_lexicon_tags_68 },
    { 0x57ca67b9, SUC_lexicon_tags_37 },
    { 0x2d2a67ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x472a67bc, SUC_lexicon_tags_29 },
    { 0xdeca67bd, SUC_lexicon_tags_25 },
    { 0x784a67bd, SUC_lexicon_tags_19 },
    { 0x232a67bf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x64ea67c1, SUC_lexicon_tags_27 },
    { 0x16ea67c2, SUC_lexicon_tags_7 },
    { 0xbbea67c3, SUC_lexicon_tags_21 },
    { 0x37ea67c4, SUC_lexicon_tags_1 },
    { 0xf2ca67c5, SUC_lexicon_tags_17 },
    { 0x8baa67c6, SUC_lexicon_tags_135 },
    { 0x740a67c5, SUC_lexicon_tags_19 },
    { 0xb7ca67c8, SUC_lexicon_tags_16 },
    { 0xa96a67c9, SUC_lexicon_tags_25 },
    { 0xbdaa67c8, SUC_lexicon_tags_66 },
    { 0x396a67c7, SUC_lexicon_tags_48 },
    { 0xc98a67c8, SUC_lexicon_tags_15 },
    { 0x6a6a67cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4a67d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aea67d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf28a67db, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ca67e2, SUC_lexicon_tags_28 },
    { 0x448a67e3, SUC_lexicon_tags_7 },
    { 0x4f8a67e4, SUC_lexicon_tags_4 },
    { 0xe38a67e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6a67ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x88ca67f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaa67f3, SUC_lexicon_tags_38 },
    { 0x18a67f4, SUC_lexicon_tags_33 },
    { 0xe1aa67f4, SUC_lexicon_tags_30 },
    { 0x5b2a67f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9f8a67f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ca67fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822a6801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814a680b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x216a680d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2a6816, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3cea6818, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aa681f, SUC_lexicon_tags_1 },
    { 0xaa4a681f, SUC_lexicon_tags_15 },
    { 0xdbca6821, SUC_lexicon_tags_49 },
    { 0x206a6822, SUC_lexicon_tags_91 },
    { 0x86ca6821, SUC_lexicon_tags_65 },
    { 0x3fea6824, SUC_lexicon_tags_65 },
    { 0xc40a6820, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54a6829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338a6830, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126a6837, SUC_lexicon_tags_12 },
    { 0xd70a6838, SUC_lexicon_tags_12 },
    { 0x1fea6838, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202a683d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74aa683f, SUC_lexicon_tags_11 },
    { 0x8cea6840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x374a6842, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x972a6845, SUC_lexicon_tags_2 },
    { 0x666a6846, SUC_lexicon_tags_49 },
    { 0xb3ca6846, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6a684a, SUC_lexicon_tags_77 },
    { 0xc10a684a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fea6851, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x980a6853, SUC_lexicon_tags_28 },
    { 0xe2ca6854, SUC_lexicon_tags_2 },
    { 0xd88a6853, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2a685c, SUC_lexicon_tags_2 },
    { 0xbb0a685d, SUC_lexicon_tags_82 },
    { 0x590a685e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6a6862, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90a686a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ca686d, SUC_lexicon_tags_63 },
    { 0x402a686e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ca6872, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaa6876, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52ca6878, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42a687b, SUC_lexicon_tags_12 },
    { 0x442a687c, SUC_lexicon_tags_12 },
    { 0xa28a687d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936a6881, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3cea6883, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaa6886, SUC_lexicon_tags_28 },
    { 0x73ea6887, SUC_lexicon_tags_45 },
    { 0x74a6888, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaa688b, SUC_lexicon_tags_77 },
    { 0x9c0a688c, SUC_lexicon_tags_43 },
    { 0xf1ca688c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a6893, SUC_lexicon_tags_7 },
    { 0x2f2a6893, SUC_lexicon_tags_45 },
    { 0x676a6895, SUC_lexicon_tags_7 },
    { 0x7b6a6896, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4a68a1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82a68a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3aea68a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x842a68aa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20a68ad, SUC_lexicon_tags_1 },
    { 0x350a68ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0a68b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dca68b5, SUC_lexicon_tags_49 },
    { 0x248a68b5, SUC_lexicon_tags_11 },
    { 0x542a68b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x326a68b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe82a68bb, SUC_lexicon_tags_2 },
    { 0x7b4a68bb, SUC_lexicon_tags_21 },
    { 0x2f6a68bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcaa68bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8a68c4, SUC_lexicon_tags_47 },
    { 0xfd6a68c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ca68c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88a68ca, SUC_lexicon_tags_31 },
    { 0xc84a68ca, SUC_lexicon_tags_45 },
    { 0xf1aa68cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34a68d8, SUC_lexicon_tags_62 },
    { 0x208a68d9, SUC_lexicon_tags_37 },
    { 0x730a68da, SUC_lexicon_tags_26 },
    { 0x86ea68db, SUC_lexicon_tags_1 },
    { 0x304a68db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfd0a68de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ca68e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ea68e9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xb50a68eb, SUC_lexicon_tags_6 },
    { 0xbf2a68ec, SUC_lexicon_tags_2 },
    { 0xcb6a68eb, SUC_lexicon_tags_32 },
    { 0xe82a68ee, SUC_lexicon_tags_27 },
    { 0x4b2a68ee, SUC_lexicon_tags_2 },
    { 0x8ea68ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06a68f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8a68f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x278a68fc, SUC_lexicon_tags_17 },
    { 0xe50a68fd, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378a6901, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xae4a6903, SUC_lexicon_tags_11 },
    { 0xe62a6904, SUC_lexicon_tags_60 },
    { 0x980a6903, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72a6909, SUC_lexicon_tags_31 },
    { 0x28aa690a, SUC_lexicon_tags_27 },
    { 0x580a690b, SUC_lexicon_tags_11 },
    { 0x9a4a690b, SUC_lexicon_tags_0 },
    { 0x288a690c, SUC_lexicon_tags_47 },
    { 0xbbaa690e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ca6911, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29ca6913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabca6916, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb70a6918, SUC_lexicon_tags_1 },
    { 0x868a6919, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36a6921, SUC_lexicon_tags_57 },
    { 0x364a6921, SUC_lexicon_tags_5 },
    { 0xd66a6923, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0a6926, SUC_lexicon_tags_15 },
    { 0x29aa6927, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78a6929, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6a6933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8a6937, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728a693d, SUC_lexicon_tags_39 },
    { 0x4a4a693e, SUC_lexicon_tags_17 },
    { 0xc02a693f, SUC_lexicon_tags_19 },
    { 0xc0ea6940, SUC_lexicon_tags_1 },
    { 0x4f0a6940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00a6945, SUC_lexicon_tags_23 },
    { 0xb80a6946, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfb0a6948, SUC_lexicon_tags_15 },
    { 0xadea6949, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800a6953, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2a695b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0aa695e, SUC_lexicon_tags_0 },
    { 0x12aa695e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x482a6961, SUC_lexicon_tags_11 },
    { 0xc60a6961, SUC_lexicon_tags_14 },
    { 0x812a6962, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46a6965, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fca696a, SUC_lexicon_tags_11 },
    { 0xbdea696b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d2a696d, SUC_lexicon_tags_10 },
    { 0xb64a696d, SUC_lexicon_tags_17 },
    { 0x120a696e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6a6977, SUC_lexicon_tags_15 },
    { 0x11ea6977, SUC_lexicon_tags_6 },
    { 0x8aea6979, SUC_lexicon_tags_37 },
    { 0x720a6978, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x210a697c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4a6987, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa698a, SUC_lexicon_tags_24 },
    { 0xfdea698b, SUC_lexicon_tags_44 },
    { 0x588a698c, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0xc38a698e, SUC_lexicon_tags_15 },
    { 0x180a698e, SUC_lexicon_tags_25 },
    { 0xf00a698e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72a6993, SUC_lexicon_tags_37 },
    { 0xf3ca6994, SUC_lexicon_tags_38 },
    { 0x1dea6994, SUC_lexicon_tags_45 },
    { 0x232a6993, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f2a6998, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02a699d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63ea699f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2daa69a2, SUC_lexicon_tags_32 },
    { 0x382a69a3, SUC_lexicon_tags_48 },
    { 0xe5aa69a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70a69a7, SUC_lexicon_tags_15 },
    { 0x28ca69a8, SUC_lexicon_tags_30 },
    { 0x536a69a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x798a69ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb10a69ad, SUC_lexicon_tags_42 },
    { 0xd8aa69ae, SUC_lexicon_tags_62 },
    { 0x1e8a69ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf6a69b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddea69b7, SUC_lexicon_tags_37 },
    { 0x65aa69b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58a69bc, SUC_lexicon_tags_17 },
    { 0xb28a69bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4a69c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfaa69c2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ca69c5, SUC_lexicon_tags_17 },
    { 0xfc8a69c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a69cb, SUC_lexicon_tags_23 },
    { 0x6e6a69cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9ca69ce, SUC_lexicon_tags_1 },
    { 0x440a69cf, SUC_lexicon_tags_17 },
    { 0x5c0a69d0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x46ea69d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaa69d6, SUC_lexicon_tags_7 },
    { 0x54aa69d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbca69dd, SUC_lexicon_tags_68 },
    { 0x71ca69de, SUC_lexicon_tags_14 },
    { 0xe0a69de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63aa69e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6a69e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x762a69e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2ca69e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x842a69eb, SUC_lexicon_tags_28 },
    { 0x420a69ec, SUC_lexicon_tags_49 },
    { 0xfca69ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c0a69ef, SUC_lexicon_tags_32 },
    { 0x27aa69ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93ea69f2, SUC_lexicon_tags_29 },
    { 0x5b2a69f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38a69f6, SUC_lexicon_tags_103 },
    { 0x590a69f7, SUC_lexicon_tags_11 },
    { 0x732a69f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fea69fb, SUC_lexicon_tags_11 },
    { 0x81ea69fb, SUC_lexicon_tags_49 },
    { 0x34ca69fb, SUC_lexicon_tags_84 },
    { 0xb58a69fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x900a6a00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x488a6a03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890a6a09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8a6a0e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd00a6a10, SUC_lexicon_tags_0 },
    { 0xe8a6a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x700a6a13, SUC_lexicon_tags_1 },
    { 0x1a4a6a14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4a6a16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37aa6a19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6f6a6a1b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ca6a24, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08a6a2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x878a6a2f, SUC_lexicon_tags_11 },
    { 0x3b4a6a30, SUC_lexicon_tags_32 },
    { 0x3daa6a31, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338a6a35, SUC_lexicon_tags_53 },
    { 0xa94a6a36, SUC_lexicon_tags_45 },
    { 0x124a6a35, SUC_lexicon_tags_35 },
    { 0xcf2a6a38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980a6a43, SUC_lexicon_tags_23 },
    { 0x736a6a44, SUC_lexicon_tags_17 },
    { 0x858a6a44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4a6a48, SUC_lexicon_tags_37 },
    { 0x154a6a49, SUC_lexicon_tags_28 },
    { 0x52ea6a49, SUC_lexicon_tags_37 },
    { 0x570a6a4a, SUC_lexicon_tags_38 },
    { 0xf6ca6a4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45aa6a52, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4a6a56, SUC_lexicon_tags_31 },
    { 0x4d2a6a57, SUC_lexicon_tags_17 },
    { 0xc62a6a58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6ea6a5a, SUC_lexicon_tags_60 },
    { 0x5d0a6a5a, SUC_lexicon_tags_179 },
    { 0x72ea6a5c, SUC_lexicon_tags_46 },
    { 0x7faa6a5c, SUC_lexicon_tags_4 },
    { 0xa26a6a5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10a6a62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe80a6a64, SUC_lexicon_tags_25 },
    { 0x6aa6a65, SUC_lexicon_tags_9 },
    { 0xf5ca6a65, SUC_lexicon_tags_77 },
    { 0x146a6a67, SUC_lexicon_tags_98 },
    { 0x4dea6a68, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02a6a6d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0a6a77, SUC_lexicon_tags_37 },
    { 0xe22a6a78, SUC_lexicon_tags_15 },
    { 0x9a8a6a79, SUC_lexicon_tags_11 },
    { 0x80aa6a79, SUC_lexicon_tags_37 },
    { 0xdf2a6a7b, SUC_lexicon_tags_15 },
    { 0x4a6a7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x65aa6a7e, SUC_lexicon_tags_26 },
    { 0xa24a6a7f, SUC_lexicon_tags_2 },
    { 0x1d2a6a7f, SUC_lexicon_tags_11 },
    { 0xbc0a6a81, SUC_lexicon_tags_37 },
    { 0xaea6a81, SUC_lexicon_tags_15 },
    { 0xafca6a83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x73ca6a85, SUC_lexicon_tags_30 },
    { 0xe54a6a86, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaa6a8b, SUC_lexicon_tags_17 },
    { 0x956a6a8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f0a6a8e, SUC_lexicon_tags_45 },
    { 0xc68a6a8f, SUC_lexicon_tags_13 },
    { 0x88ca6a90, SUC_lexicon_tags_38 },
    { 0xf62a6a91, SUC_lexicon_tags_2 },
    { 0xdbca6a92, SUC_lexicon_tags_7 },
    { 0x2b2a6a90, SUC_lexicon_tags_191 },
    { 0x438a6a8f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6a6a9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aa6aa6, SUC_lexicon_tags_1 },
    { 0x118a6aa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104a6aac, SUC_lexicon_tags_8 },
    { 0xe36a6aad, SUC_lexicon_tags_37 },
    { 0x15aa6aae, SUC_lexicon_tags_11 },
    { 0xdb4a6aac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb28a6ab1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae6a6ab3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ea6ab8, SUC_lexicon_tags_63 },
    { 0x4fea6ab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74a6ac0, SUC_lexicon_tags_12 },
    { 0xd6a6ac1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x304a6ac3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x328a6ac5, SUC_lexicon_tags_11 },
    { 0x210a6ac5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ea6acd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x83aa6acf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x356a6ad2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71aa6ad4, SUC_lexicon_tags_15 },
    { 0x4f0a6ad5, SUC_lexicon_tags_25 },
    { 0xeaaa6ad6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfaa6ad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x318a6ada, SUC_lexicon_tags_17 },
    { 0x950a6ada, SUC_lexicon_tags_83 },
    { 0x454a6adc, SUC_lexicon_tags_7 },
    { 0xfdaa6add, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ea6ae3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1ea6ae5, SUC_lexicon_tags_11 },
    { 0xba8a6ae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62a6aea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x518a6aed, SUC_lexicon_tags_8 },
    { 0xfb8a6aed, SUC_lexicon_tags_0 },
    { 0x3a0a6aed, SUC_lexicon_tags_32 },
    { 0xae2a6aee, SUC_lexicon_tags_15 },
    { 0x87aa6af1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6a6af4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80aa6af6, SUC_lexicon_tags_43 },
    { 0xc6a6af7, SUC_lexicon_tags_48 },
    { 0x63ea6af8, SUC_lexicon_tags_1 },
    { 0xe5aa6af7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e6a6afb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x124a6afd, SUC_lexicon_tags_11 },
    { 0x7a4a6afe, SUC_lexicon_tags_9 },
    { 0x978a6afd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24a6b04, SUC_lexicon_tags_48 },
    { 0x40a6b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8a6b07, SUC_lexicon_tags_27 },
    { 0x4f8a6b08, SUC_lexicon_tags_23 },
    { 0xf38a6b09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a2a6b0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aa6b0e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ea6b11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16a6b16, SUC_lexicon_tags_25 },
    { 0xc1ca6b16, SUC_lexicon_tags_31 },
    { 0x3e6a6b18, SUC_lexicon_tags_47 },
    { 0x8e8a6b19, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ca6b1f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc2ea6b21, SUC_lexicon_tags_11 },
    { 0x3b8a6b22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x368a6b24, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2a6b29, SUC_lexicon_tags_17 },
    { 0x33ea6b29, SUC_lexicon_tags_14 },
    { 0x924a6b2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x950a6b2d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x316a6b30, SUC_lexicon_tags_19 },
    { 0xe60a6b31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6a6b35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740a6b39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x426a6b3b, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aa6b40, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ea6b46, SUC_lexicon_tags_27 },
    { 0x832a6b47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0a6b4b, SUC_lexicon_tags_59 },
    { 0x85ca6b4c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eea6b56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ea6b59, SUC_lexicon_tags_12 },
    { 0x194a6b5a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x540a6b5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8ea6b5f, SUC_lexicon_tags_47 },
    { 0xd16a6b60, SUC_lexicon_tags_27 },
    { 0x3aaa6b61, SUC_lexicon_tags_11 },
    { 0xc9ca6b5f, SUC_lexicon_tags_27 },
    { 0x392a6b63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x35ca6b65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ca6b6a, SUC_lexicon_tags_2 },
    { 0x8f4a6b6a, SUC_lexicon_tags_19 },
    { 0xf00a6b6c, SUC_lexicon_tags_33 },
    { 0x140a6b6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5eea6b6f, SUC_lexicon_tags_15 },
    { 0xa52a6b6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4a6b74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942a6b78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fca6b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x314a6b7f, SUC_lexicon_tags_5 },
    { 0x62a6b80, SUC_lexicon_tags_15 },
    { 0x2f4a6b81, SUC_lexicon_tags_37 },
    { 0xdc4a6b82, SUC_lexicon_tags_42 },
    { 0x90aa6b83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50a6b88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62ca6b8a, SUC_lexicon_tags_25 },
    { 0xaaaa6b8b, SUC_lexicon_tags_0 },
    { 0x1ea6b8c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x970a6b8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84a6b92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54a6b99, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56a6b9c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x486a6b9e, SUC_lexicon_tags_7 },
    { 0xa22a6b9e, SUC_lexicon_tags_8 },
    { 0x118a6ba0, SUC_lexicon_tags_34 },
    { 0x2a6a6ba0, SUC_lexicon_tags_25 },
    { 0xadea6ba2, SUC_lexicon_tags_2 },
    { 0xb18a6ba2, SUC_lexicon_tags_1 },
    { 0xbd0a6ba4, SUC_lexicon_tags_6 },
    { 0x262a6ba3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56a6ba8, SUC_lexicon_tags_33 },
    { 0xf60a6ba9, SUC_lexicon_tags_25 },
    { 0x1a6a6ba8, SUC_lexicon_tags_2 },
    { 0x64ca6ba9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a6a6bad, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa10a6baf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aca6bb4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ca6bb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9daa6bb9, SUC_lexicon_tags_36 },
    { 0xb56a6bb9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38a6bbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x392a6bc0, SUC_lexicon_tags_11 },
    { 0x406a6bc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f0a6bc3, SUC_lexicon_tags_21 },
    { 0x4aea6bc3, SUC_lexicon_tags_14 },
    { 0x396a6bc5, SUC_lexicon_tags_6 },
    { 0x752a6bc6, SUC_lexicon_tags_15 },
    { 0x5b4a6bc3, SUC_lexicon_tags_25 },
    { 0x77aa6bc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe30a6bca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ca6bcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4a6bd4, SUC_lexicon_tags_6 },
    { 0xa2a6bd5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246a6bdc, SUC_lexicon_tags_25 },
    { 0x8a8a6bdd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ea6be6, SUC_lexicon_tags_37 },
    { 0x70ca6be7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0a6beb, SUC_lexicon_tags_1 },
    { 0xe8a6bec, SUC_lexicon_tags_14 },
    { 0x236a6bec, SUC_lexicon_tags_1 },
    { 0xaa6a6bec, SUC_lexicon_tags_37 },
    { 0xb5ca6bef, SUC_lexicon_tags_0 },
    { 0x9c4a6beb, SUC_lexicon_tags_1 },
    { 0x7d4a6beb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d8a6bf3, SUC_lexicon_tags_19 },
    { 0x6b0a6bf3, SUC_lexicon_tags_82 },
    { 0xde2a6bf5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf02a6bf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8a6c01, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xe4ca6c03, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4a6c09, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x542a6c0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ca6c0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeca6c18, SUC_lexicon_tags_25 },
    { 0x22aa6c19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72a6c1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb4a6c1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580a6c25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794a6c29, SUC_lexicon_tags_46 },
    { 0xcb8a6c29, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x91ea6c2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a6c32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6a6c36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec2a6c38, SUC_lexicon_tags_1 },
    { 0xdc0a6c39, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc78a6c3b, SUC_lexicon_tags_12 },
    { 0xd2ca6c3c, SUC_lexicon_tags_66 },
    { 0x9fea6c3c, SUC_lexicon_tags_11 },
    { 0xd02a6c3e, SUC_lexicon_tags_23 },
    { 0x322a6c3f, SUC_lexicon_tags_25 },
    { 0x906a6c40, SUC_lexicon_tags_1 },
    { 0xa76a6c3e, SUC_lexicon_tags_16 },
    { 0x800a6c42, SUC_lexicon_tags_10 },
    { 0xc4ca6c3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eea6c4e, SUC_lexicon_tags_66 },
    { 0x78a6c4f, SUC_lexicon_tags_46 },
    { 0x94ca6c50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56a6c54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ca6c57, SUC_lexicon_tags_37 },
    { 0x1b0a6c58, SUC_lexicon_tags_25 },
    { 0x20ca6c58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf82a6c5b, SUC_lexicon_tags_10 },
    { 0x832a6c5c, SUC_lexicon_tags_25 },
    { 0x264a6c5d, SUC_lexicon_tags_27 },
    { 0x352a6c5e, SUC_lexicon_tags_27 },
    { 0x89ea6c5f, SUC_lexicon_tags_60 },
    { 0xa6c60, SUC_lexicon_tags_17 },
    { 0x45ea6c5f, SUC_lexicon_tags_22 },
    { 0xbb4a6c60, SUC_lexicon_tags_1 },
    { 0xc6a6c5c, SUC_lexicon_tags_11 },
    { 0x41ca6c60, SUC_lexicon_tags_16 },
    { 0x372a6c65, SUC_lexicon_tags_37 },
    { 0xc96a6c65, SUC_lexicon_tags_1 },
    { 0xfca6c5b, SUC_lexicon_tags_9 },
    { 0x15ea6c68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cca6c6c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfeea6c6e, SUC_lexicon_tags_30 },
    { 0xcd6a6c6f, SUC_lexicon_tags_5 },
    { 0xd6ca6c6f, SUC_lexicon_tags_45 },
    { 0x464a6c71, SUC_lexicon_tags_19 },
    { 0xf20a6c6f, SUC_lexicon_tags_0 },
    { 0x51ea6c72, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x28aa6c75, SUC_lexicon_tags_8 },
    { 0x606a6c76, SUC_lexicon_tags_43 },
    { 0xe7aa6c77, SUC_lexicon_tags_65 },
    { 0xeeaa6c78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daa6c7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aa6c82, SUC_lexicon_tags_17 },
    { 0xefea6c82, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eca6c87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840a6c8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ca6c91, SUC_lexicon_tags_30 },
    { 0x114a6c91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0a6c97, SUC_lexicon_tags_2 },
    { 0x8f4a6c98, SUC_lexicon_tags_46 },
    { 0xd58a6c99, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2aca6c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aea6c9f, SUC_lexicon_tags_48 },
    { 0x706a6ca0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8a6ca3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a6ca7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x702a6caa, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x282a6cad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc6aa6caf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ca6cb5, SUC_lexicon_tags_17 },
    { 0x440a6cb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe4a6cb8, SUC_lexicon_tags_23 },
    { 0xc4ea6cb9, SUC_lexicon_tags_37 },
    { 0x10ca6cba, SUC_lexicon_tags_14 },
    { 0x5fea6cbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8a6cbe, SUC_lexicon_tags_37 },
    { 0xfdea6cbe, SUC_lexicon_tags_15 },
    { 0x3a8a6cbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4a6cc4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ea6ccb, SUC_lexicon_tags_6 },
    { 0x3aaa6ccc, SUC_lexicon_tags_63 },
    { 0xc3ca6ccc, SUC_lexicon_tags_9 },
    { 0x12ca6cce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x726a6cd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dea6cd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222a6ce1, SUC_lexicon_tags_68 },
    { 0xce2a6ce1, SUC_lexicon_tags_56 },
    { 0xe50a6ce3, SUC_lexicon_tags_14 },
    { 0xd5ea6ce4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766a6ce9, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x700a6ceb, SUC_lexicon_tags_26 },
    { 0xd8aa6ceb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcaaa6cee, SUC_lexicon_tags_23 },
    { 0xf0ea6cef, SUC_lexicon_tags_160 },
    { 0x24ea6cf0, SUC_lexicon_tags_48 },
    { 0xa30a6cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12a6cf4, SUC_lexicon_tags_49 },
    { 0xd6ea6cf5, SUC_lexicon_tags_30 },
    { 0x608a6cf5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250a6cfa, SUC_lexicon_tags_66 },
    { 0xcc4a6cfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27aa6d00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeca6d04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404a6d08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa2a6d0a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42a6d0e, SUC_lexicon_tags_4 },
    { 0xfb4a6d0f, SUC_lexicon_tags_7 },
    { 0xeb6a6d10, SUC_lexicon_tags_9 },
    { 0x6b8a6d0e, SUC_lexicon_tags_25 },
    { 0xc4ca6d10, SUC_lexicon_tags_45 },
    { 0x764a6d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44a6d19, SUC_lexicon_tags_55 },
    { 0x25ca6d1a, SUC_lexicon_tags_47 },
    { 0xf8a6d1a, SUC_lexicon_tags_2 },
    { 0x360a6d1c, SUC_lexicon_tags_2 },
    { 0x24ea6d1d, SUC_lexicon_tags_37 },
    { 0x9b6a6d1e, SUC_lexicon_tags_37 },
    { 0x6d6a6d1e, SUC_lexicon_tags_0 },
    { 0x514a6d1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30a6d23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0a6d26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722a6d2b, SUC_lexicon_tags_7 },
    { 0xdbea6d2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2baa6d32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6a6d3b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aa6d3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x804a6d41, SUC_lexicon_tags_17 },
    { 0x432a6d41, SUC_lexicon_tags_4 },
    { 0xb02a6d42, SUC_lexicon_tags_7 },
    { 0x3c0a6d43, SUC_lexicon_tags_0 },
    { 0x2faa6d45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1bea6d47, SUC_lexicon_tags_7 },
    { 0x72aa6d47, SUC_lexicon_tags_32 },
    { 0x7d6a6d47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0a6d4e, SUC_lexicon_tags_7 },
    { 0x570a6d4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x112a6d51, SUC_lexicon_tags_2 },
    { 0x1baa6d52, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bca6d56, SUC_lexicon_tags_43 },
    { 0x494a6d57, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1cca6d59, SUC_lexicon_tags_1 },
    { 0x4aca6d5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x51aa6d5c, SUC_lexicon_tags_65 },
    { 0x612a6d5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb58a6d5f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacea6d63, SUC_lexicon_tags_37 },
    { 0x8a6a6d64, SUC_lexicon_tags_6 },
    { 0xb2ca6d65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x284a6d67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71aa6d6b, SUC_lexicon_tags_15 },
    { 0x196a6d6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ca6d72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbd8a6d74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11aa6d79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6a6d7c, SUC_lexicon_tags_2 },
    { 0x3e6a6d7d, SUC_lexicon_tags_31 },
    { 0xf92a6d7c, SUC_lexicon_tags_13 },
    { 0x496a6d7f, SUC_lexicon_tags_25 },
    { 0x9e8a6d80, SUC_lexicon_tags_31 },
    { 0x9c8a6d80, SUC_lexicon_tags_43 },
    { 0xbf4a6d7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ca6d85, SUC_lexicon_tags_2 },
    { 0xf2ea6d85, SUC_lexicon_tags_4 },
    { 0x8bca6d87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf6a6d89, SUC_lexicon_tags_37 },
    { 0x97aa6d89, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf48a6d8c, SUC_lexicon_tags_15 },
    { 0x7aea6d8d, SUC_lexicon_tags_2 },
    { 0x7c2a6d8e, SUC_lexicon_tags_23 },
    { 0x600a6d8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4baa6d92, SUC_lexicon_tags_1 },
    { 0x14ca6d93, SUC_lexicon_tags_11 },
    { 0xb7ca6d94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x708a6d97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef8a6d99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978a6d9f, SUC_lexicon_tags_12 },
    { 0x346a6d9f, SUC_lexicon_tags_1 },
    { 0xb5aa6da1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x580a6da4, SUC_lexicon_tags_25 },
    { 0xe50a6da4, SUC_lexicon_tags_9 },
    { 0xffea6da6, SUC_lexicon_tags_48 },
    { 0xa62a6da7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaa6dab, SUC_lexicon_tags_37 },
    { 0xdaca6dac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a4a6dae, SUC_lexicon_tags_4 },
    { 0xd9ea6dae, SUC_lexicon_tags_37 },
    { 0x726a6db0, SUC_lexicon_tags_69 },
    { 0xd76a6daf, SUC_lexicon_tags_7 },
    { 0xb08a6db2, SUC_lexicon_tags_37 },
    { 0x690a6db3, SUC_lexicon_tags_12 },
    { 0x380a6db4, SUC_lexicon_tags_16 },
    { 0xb68a6db0, SUC_lexicon_tags_45 },
    { 0xbaca6db6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6a6dbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee8a6dc1, SUC_lexicon_tags_45 },
    { 0xc88a6dc2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfaca6dc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62ea6dc6, SUC_lexicon_tags_9 },
    { 0xa46a6dc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ca6dca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8a6dcf, SUC_lexicon_tags_25 },
    { 0xa06a6dd0, SUC_lexicon_tags_27 },
    { 0x5e2a6dcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dea6dd8, SUC_lexicon_tags_61 },
    { 0xb2aa6dd9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92a6de1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x906a6de4, SUC_lexicon_tags_25 },
    { 0x44ea6de5, SUC_lexicon_tags_138 },
    { 0x9e0a6de4, SUC_lexicon_tags_25 },
    { 0x9f0a6de7, SUC_lexicon_tags_31 },
    { 0xca4a6de8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aa6df5, SUC_lexicon_tags_9 },
    { 0x198a6df5, SUC_lexicon_tags_63 },
    { 0x956a6df7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0a6dfa, SUC_lexicon_tags_11 },
    { 0x226a6dfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54aa6e00, SUC_lexicon_tags_9 },
    { 0x7c6a6e01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4a6e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ea6e08, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244a6e12, SUC_lexicon_tags_34 },
    { 0xdca6e12, SUC_lexicon_tags_1 },
    { 0xff8a6e13, SUC_lexicon_tags_16 },
    { 0xd62a6e15, SUC_lexicon_tags_79 },
    { 0x7a6a6e16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcb0a6e18, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4a6e1b, SUC_lexicon_tags_15 },
    { 0xca0a6e1c, SUC_lexicon_tags_12 },
    { 0xe4ea6e1d, SUC_lexicon_tags_45 },
    { 0x43ea6e1d, SUC_lexicon_tags_15 },
    { 0xba2a6e1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06a6e27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0a6e2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ca6e32, SUC_lexicon_tags_12 },
    { 0xea8a6e32, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaa6e37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0a6e3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70a6e42, SUC_lexicon_tags_14 },
    { 0x270a6e43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0a6e46, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aea6e4a, SUC_lexicon_tags_3 },
    { 0x40ea6e4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9aa6e4e, SUC_lexicon_tags_7 },
    { 0xfea6e4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26a6e56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcea6e5b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4a6e60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e4a6e62, SUC_lexicon_tags_106 },
    { 0x482a6e63, SUC_lexicon_tags_17 },
    { 0xc48a6e64, SUC_lexicon_tags_17 },
    { 0x220a6e64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720a6e6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10a6e70, SUC_lexicon_tags_25 },
    { 0x472a6e71, SUC_lexicon_tags_15 },
    { 0xeeca6e72, SUC_lexicon_tags_2 },
    { 0xed2a6e71, SUC_lexicon_tags_2 },
    { 0xaf8a6e74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6a6e78, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ea6e7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x166a6e81, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdf4a6e83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bca6e87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaa6e90, SUC_lexicon_tags_10 },
    { 0x108a6e91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6a6e94, SUC_lexicon_tags_11 },
    { 0xf26a6e94, SUC_lexicon_tags_11 },
    { 0x91ea6e94, SUC_lexicon_tags_1496 },
    { 0, NULL },
    { 0x446a6e98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60a6e9d, SUC_lexicon_tags_117 },
    { 0xd62a6e9e, SUC_lexicon_tags_8 },
    { 0xc2ea6e9f, SUC_lexicon_tags_30 },
    { 0x25aa6e9f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98a6ea8, SUC_lexicon_tags_106 },
    { 0x46ca6ea8, SUC_lexicon_tags_83 },
    { 0x4e6a6eaa, SUC_lexicon_tags_9 },
    { 0x5d8a6eab, SUC_lexicon_tags_29 },
    { 0x91ca6eaa, SUC_lexicon_tags_19 },
    { 0xd7ca6eab, SUC_lexicon_tags_17 },
    { 0xd54a6ea9, SUC_lexicon_tags_63 },
    { 0x4faa6eaf, SUC_lexicon_tags_12 },
    { 0xa12a6eb0, SUC_lexicon_tags_63 },
    { 0x44a6eb0, SUC_lexicon_tags_29 },
    { 0x158a6eb1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ca6eb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92a6ebb, SUC_lexicon_tags_7 },
    { 0xe6ea6ebc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84ca6ebe, SUC_lexicon_tags_2 },
    { 0x6f2a6ebe, SUC_lexicon_tags_25 },
    { 0xf7ca6ec0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x886a6ec2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32a6ec8, SUC_lexicon_tags_11 },
    { 0x3d8a6ec8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9aa6ecf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x656a6ed1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ea6ed8, SUC_lexicon_tags_37 },
    { 0x446a6ed8, SUC_lexicon_tags_49 },
    { 0x7cca6eda, SUC_lexicon_tags_1 },
    { 0xed6a6edb, SUC_lexicon_tags_37 },
    { 0x316a6edc, SUC_lexicon_tags_15 },
    { 0x286a6ed8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x456a6edf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474a6ee3, SUC_lexicon_tags_10 },
    { 0x748a6ee4, SUC_lexicon_tags_17 },
    { 0x284a6ee5, SUC_lexicon_tags_37 },
    { 0x2fca6ee5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x512a6ee8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcdaa6eea, SUC_lexicon_tags_37 },
    { 0x70ea6eeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a6eee, SUC_lexicon_tags_27 },
    { 0xf4ca6eef, SUC_lexicon_tags_147 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaa6ef2, SUC_lexicon_tags_28 },
    { 0x40aa6ef2, SUC_lexicon_tags_48 },
    { 0xd28a6ef3, SUC_lexicon_tags_2 },
    { 0x63ca6ef5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62aa6efa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d8a6efc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404a6f00, SUC_lexicon_tags_63 },
    { 0x450a6f01, SUC_lexicon_tags_6 },
    { 0x82aa6f02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaea6f04, SUC_lexicon_tags_17 },
    { 0x5d6a6f04, SUC_lexicon_tags_10 },
    { 0xcd4a6f06, SUC_lexicon_tags_0 },
    { 0x1b4a6f07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb86a6f09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760a6f0d, SUC_lexicon_tags_12 },
    { 0x26ca6f0e, SUC_lexicon_tags_32 },
    { 0x6c2a6f0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ca6f12, SUC_lexicon_tags_56 },
    { 0x1d2a6f13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63aa6f19, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52a6f1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba4a6f1f, SUC_lexicon_tags_28 },
    { 0x7b8a6f1f, SUC_lexicon_tags_11 },
    { 0x540a6f21, SUC_lexicon_tags_12 },
    { 0x8fea6f22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46aa6f25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14a6f28, SUC_lexicon_tags_39 },
    { 0x6d0a6f29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ca6f2c, SUC_lexicon_tags_38 },
    { 0xdd2a6f2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x83aa6f30, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ca6f35, SUC_lexicon_tags_26 },
    { 0xa58a6f36, SUC_lexicon_tags_63 },
    { 0x4fea6f35, SUC_lexicon_tags_37 },
    { 0xf5ea6f38, SUC_lexicon_tags_21 },
    { 0x1cca6f37, SUC_lexicon_tags_37 },
    { 0xa92a6f38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x206a6f3d, SUC_lexicon_tags_23 },
    { 0xea6a6f3e, SUC_lexicon_tags_7 },
    { 0x880a6f3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86a6f42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8aa6f45, SUC_lexicon_tags_57 },
    { 0x716a6f46, SUC_lexicon_tags_37 },
    { 0xd7ca6f47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ea6f4b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2eaa6f4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0a6f50, SUC_lexicon_tags_37 },
    { 0xc24a6f51, SUC_lexicon_tags_25 },
    { 0xcb6a6f52, SUC_lexicon_tags_11 },
    { 0x986a6f53, SUC_lexicon_tags_37 },
    { 0xf9aa6f54, SUC_lexicon_tags_0 },
    { 0xedca6f55, SUC_lexicon_tags_26 },
    { 0xb24a6f53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf48a6f58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8a6f60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76ea6f62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa68a6f64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462a6f68, SUC_lexicon_tags_38 },
    { 0x56a6f69, SUC_lexicon_tags_3 },
    { 0xd5aa6f6a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8a6f6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ca6f72, SUC_lexicon_tags_26 },
    { 0x85aa6f73, SUC_lexicon_tags_21 },
    { 0x146a6f74, SUC_lexicon_tags_2 },
    { 0xdbea6f74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa64a6f77, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2e2a6f79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32a6f80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x264a6f83, SUC_lexicon_tags_15 },
    { 0x4a4a6f83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2a6f88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfeca6f8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7faa6f92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82a6f96, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x46aa6f98, SUC_lexicon_tags_27 },
    { 0x59ca6f98, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4a6f9c, SUC_lexicon_tags_23 },
    { 0x306a6f9d, SUC_lexicon_tags_8 },
    { 0xe2ca6f9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdea6fa1, SUC_lexicon_tags_12 },
    { 0x4baa6fa1, SUC_lexicon_tags_31 },
    { 0xda0a6fa3, SUC_lexicon_tags_572 },
    { 0x68ca6fa3, SUC_lexicon_tags_7 },
    { 0xceea6fa5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282a6faa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0a6fae, SUC_lexicon_tags_0 },
    { 0xba4a6faf, SUC_lexicon_tags_70 },
    { 0x8f0a6fb0, SUC_lexicon_tags_14 },
    { 0xe18a6fb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0ca6fb3, SUC_lexicon_tags_60 },
    { 0xb44a6fb4, SUC_lexicon_tags_2 },
    { 0xc00a6fb4, SUC_lexicon_tags_14 },
    { 0x41ea6fb5, SUC_lexicon_tags_25 },
    { 0x800a6fb4, SUC_lexicon_tags_14 },
    { 0x272a6fb6, SUC_lexicon_tags_28 },
    { 0x556a6fb8, SUC_lexicon_tags_39 },
    { 0x534a6fb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538a6fbe, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6a6fc3, SUC_lexicon_tags_1 },
    { 0xfa6a6fc4, SUC_lexicon_tags_13 },
    { 0xeca6fc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ea6fc9, SUC_lexicon_tags_83 },
    { 0xe84a6fca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaa6fd0, SUC_lexicon_tags_11 },
    { 0x8f8a6fd0, SUC_lexicon_tags_1 },
    { 0xdbaa6fd2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa0aa6fd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28a6fd7, SUC_lexicon_tags_12 },
    { 0xd30a6fd7, SUC_lexicon_tags_31 },
    { 0x8aca6fd7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422a6fe3, SUC_lexicon_tags_9 },
    { 0x9b4a6fe4, SUC_lexicon_tags_33 },
    { 0xc0ea6fe3, SUC_lexicon_tags_37 },
    { 0xad4a6fe6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x10a6fe8, SUC_lexicon_tags_37 },
    { 0x660a6fe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484a6fed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a6ff3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ea6ff6, SUC_lexicon_tags_8 },
    { 0x666a6ff7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xacaa6ff9, SUC_lexicon_tags_6 },
    { 0xe3aa6ffa, SUC_lexicon_tags_60 },
    { 0xf12a6ffa, SUC_lexicon_tags_80 },
    { 0xdaca6ffc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906a7001, SUC_lexicon_tags_77 },
    { 0x5aaa7002, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x788a7005, SUC_lexicon_tags_49 },
    { 0xbf2a7006, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ea7009, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4a700c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6a7013, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8a701c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28ca701e, SUC_lexicon_tags_31 },
    { 0x806a701e, SUC_lexicon_tags_30 },
    { 0x3b2a7020, SUC_lexicon_tags_1 },
    { 0xebca7020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdea7024, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x620a7026, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x876a7029, SUC_lexicon_tags_87 },
    { 0xdf8a7029, SUC_lexicon_tags_34 },
    { 0xb6ca702b, SUC_lexicon_tags_1 },
    { 0xf4ea7029, SUC_lexicon_tags_23 },
    { 0x1f0a702d, SUC_lexicon_tags_49 },
    { 0x44aa702a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4a7036, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cea703a, SUC_lexicon_tags_15 },
    { 0x73aa703b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ca703f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ea7045, SUC_lexicon_tags_7 },
    { 0x772a7046, SUC_lexicon_tags_129 },
    { 0x48aa7047, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800a704e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29ca7050, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fea7052, SUC_lexicon_tags_32 },
    { 0x3e4a7053, SUC_lexicon_tags_2 },
    { 0x144a7054, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x656a7057, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x962a7059, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x890a705b, SUC_lexicon_tags_17 },
    { 0x9fea705c, SUC_lexicon_tags_598 },
    { 0x1caa705b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ca7061, SUC_lexicon_tags_0 },
    { 0x6eaa7062, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fea7065, SUC_lexicon_tags_37 },
    { 0x482a7066, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92a7069, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ea706c, SUC_lexicon_tags_43 },
    { 0x36ea706d, SUC_lexicon_tags_0 },
    { 0x3aca706e, SUC_lexicon_tags_37 },
    { 0x38ca706c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ca7072, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0a707e, SUC_lexicon_tags_37 },
    { 0x56a707e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7caa7081, SUC_lexicon_tags_15 },
    { 0x4cca7082, SUC_lexicon_tags_47 },
    { 0x466a7083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ca7086, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x540a7088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ca708c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54a708f, SUC_lexicon_tags_12 },
    { 0x46ea7090, SUC_lexicon_tags_23 },
    { 0xb3ca7090, SUC_lexicon_tags_43 },
    { 0xcd2a7092, SUC_lexicon_tags_37 },
    { 0xfeca7093, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ca7096, SUC_lexicon_tags_7 },
    { 0xd18a7097, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976a709d, SUC_lexicon_tags_32 },
    { 0xe7ea709d, SUC_lexicon_tags_79 },
    { 0xc30a709e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x134a70a2, SUC_lexicon_tags_61 },
    { 0xc0aa70a3, SUC_lexicon_tags_25 },
    { 0x9c8a70a2, SUC_lexicon_tags_28 },
    { 0xf0aa70a3, SUC_lexicon_tags_46 },
    { 0xa9ea70a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd24a70a8, SUC_lexicon_tags_25 },
    { 0xa1aa70a9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48a70ad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdfea70af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcea70b1, SUC_lexicon_tags_2 },
    { 0x6c8a70b2, SUC_lexicon_tags_9 },
    { 0xc1aa70b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x416a70b6, SUC_lexicon_tags_11 },
    { 0x3e0a70b7, SUC_lexicon_tags_9 },
    { 0xc5ca70b8, SUC_lexicon_tags_39 },
    { 0x9fca70b7, SUC_lexicon_tags_27 },
    { 0x48aa70ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6dca70bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8dea70be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2a70c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ca70d1, SUC_lexicon_tags_37 },
    { 0x374a70d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe96a70d4, SUC_lexicon_tags_1 },
    { 0x890a70d5, SUC_lexicon_tags_43 },
    { 0x170a70d6, SUC_lexicon_tags_14 },
    { 0xcaca70d4, SUC_lexicon_tags_0 },
    { 0x3d2a70d8, SUC_lexicon_tags_10 },
    { 0x1a2a70d8, SUC_lexicon_tags_63 },
    { 0xd7ea70d9, SUC_lexicon_tags_14 },
    { 0xabca70d8, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca70e2, SUC_lexicon_tags_37 },
    { 0xef4a70e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50a70e9, SUC_lexicon_tags_60 },
    { 0x3fea70e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xddca70ed, SUC_lexicon_tags_198 },
    { 0x358a70ee, SUC_lexicon_tags_25 },
    { 0xabca70ed, SUC_lexicon_tags_17 },
    { 0x4aca70ee, SUC_lexicon_tags_27 },
    { 0xb4aa70f1, SUC_lexicon_tags_17 },
    { 0x914a70f2, SUC_lexicon_tags_15 },
    { 0x8a4a70ed, SUC_lexicon_tags_30 },
    { 0x56aa70f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2a70f9, SUC_lexicon_tags_17 },
    { 0xac0a70fa, SUC_lexicon_tags_79 },
    { 0x8eea70fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xde8a70fd, SUC_lexicon_tags_15 },
    { 0x4aaa70fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6a7103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf04a7105, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d8a7107, SUC_lexicon_tags_95 },
    { 0xf8ea7107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x902a710a, SUC_lexicon_tags_12 },
    { 0xcf8a710a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26a710f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x462a7112, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fea7116, SUC_lexicon_tags_2 },
    { 0x3a0a7117, SUC_lexicon_tags_11 },
    { 0x190a7118, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8a7122, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaea7126, SUC_lexicon_tags_15 },
    { 0x4a4a7127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x578a7129, SUC_lexicon_tags_12 },
    { 0x9eea712a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ea712d, SUC_lexicon_tags_27 },
    { 0x4eea712e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbca7131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaca7136, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82a7140, SUC_lexicon_tags_17 },
    { 0xd1ea7141, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ea7145, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2a7148, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a714d, SUC_lexicon_tags_55 },
    { 0x4e8a714e, SUC_lexicon_tags_61 },
    { 0xc9ca714f, SUC_lexicon_tags_7 },
    { 0x32a714e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ea7156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb90a7158, SUC_lexicon_tags_0 },
    { 0x138a7158, SUC_lexicon_tags_12 },
    { 0x924a7159, SUC_lexicon_tags_31 },
    { 0xba4a715b, SUC_lexicon_tags_15 },
    { 0x452a715c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x92a715e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276a7168, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a716d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbfca716f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x90ea7171, SUC_lexicon_tags_57 },
    { 0xe7aa7172, SUC_lexicon_tags_38 },
    { 0xf14a7173, SUC_lexicon_tags_29 },
    { 0x7daa7174, SUC_lexicon_tags_7 },
    { 0xd9ea7175, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdb2a7177, SUC_lexicon_tags_17 },
    { 0x26ca7178, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ea717b, SUC_lexicon_tags_84 },
    { 0xc22a717b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa14a717e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xadaa7181, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944a7185, SUC_lexicon_tags_1 },
    { 0xba4a7186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a7189, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ea718c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x78aa718e, SUC_lexicon_tags_13 },
    { 0x1c2a718f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4a7192, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb0a7194, SUC_lexicon_tags_29 },
    { 0xa06a7195, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8a719b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70a719e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ea71a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4a71aa, SUC_lexicon_tags_23 },
    { 0x876a71ab, SUC_lexicon_tags_33 },
    { 0xf38a71ac, SUC_lexicon_tags_27 },
    { 0x9faa71ac, SUC_lexicon_tags_7 },
    { 0xceaa71aa, SUC_lexicon_tags_14 },
    { 0x91ca71af, SUC_lexicon_tags_64 },
    { 0x66ea71b0, SUC_lexicon_tags_775 },
    { 0x21aa71b1, SUC_lexicon_tags_516 },
    { 0, NULL },
    { 0x87ea71b3, SUC_lexicon_tags_0 },
    { 0xc66a71b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4a71b8, SUC_lexicon_tags_12 },
    { 0x67ca71b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceea71be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206a71c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3baa71c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522a71cb, SUC_lexicon_tags_12 },
    { 0x3d2a71cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8a71cf, SUC_lexicon_tags_11 },
    { 0xc84a71d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6fea71d2, SUC_lexicon_tags_2 },
    { 0xe50a71d3, SUC_lexicon_tags_23 },
    { 0x200a71d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf2a71d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcaea71d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x466a71db, SUC_lexicon_tags_6 },
    { 0x9e0a71dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x756a71de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a71e1, SUC_lexicon_tags_25 },
    { 0x132a71e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90a71e6, SUC_lexicon_tags_1 },
    { 0x5f8a71e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf54a71e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aa71ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28a71f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2a71f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a71fd, SUC_lexicon_tags_19 },
    { 0xb2ca71fe, SUC_lexicon_tags_37 },
    { 0xaeaa71fe, SUC_lexicon_tags_12 },
    { 0x7daa71ff, SUC_lexicon_tags_1 },
    { 0xf7ea71fd, SUC_lexicon_tags_11 },
    { 0xd90a71fe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x380a7204, SUC_lexicon_tags_23 },
    { 0x294a7205, SUC_lexicon_tags_48 },
    { 0x628a7206, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa98a7208, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee8a720a, SUC_lexicon_tags_17 },
    { 0xd58a720b, SUC_lexicon_tags_12 },
    { 0x4aaa720c, SUC_lexicon_tags_68 },
    { 0x414a720c, SUC_lexicon_tags_25 },
    { 0x930a720d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ea7212, SUC_lexicon_tags_16 },
    { 0x7f8a7213, SUC_lexicon_tags_7 },
    { 0x2bca7214, SUC_lexicon_tags_65 },
    { 0xf60a7215, SUC_lexicon_tags_9 },
    { 0x272a7215, SUC_lexicon_tags_22 },
    { 0x13aa7217, SUC_lexicon_tags_11 },
    { 0xe02a7218, SUC_lexicon_tags_17 },
    { 0x49ca7219, SUC_lexicon_tags_9 },
    { 0xde6a721a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554a721e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x638a7220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b8a7222, SUC_lexicon_tags_57 },
    { 0x74ca7222, SUC_lexicon_tags_15 },
    { 0x9fea7223, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb70a7226, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x934a7228, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea722d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x25ca722f, SUC_lexicon_tags_8 },
    { 0xd54a722f, SUC_lexicon_tags_1 },
    { 0x324a7230, SUC_lexicon_tags_40 },
    { 0x230a7231, SUC_lexicon_tags_43 },
    { 0x9b2a7233, SUC_lexicon_tags_37 },
    { 0x5d8a7234, SUC_lexicon_tags_1 },
    { 0xdf2a7233, SUC_lexicon_tags_15 },
    { 0xfc6a7234, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x392a7238, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ca723f, SUC_lexicon_tags_49 },
    { 0x542a723f, SUC_lexicon_tags_11 },
    { 0x500a7241, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa44a7243, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946a7247, SUC_lexicon_tags_54 },
    { 0xdcaa7247, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472a724e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff4a7250, SUC_lexicon_tags_23 },
    { 0x806a7251, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4aa7253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4a7256, SUC_lexicon_tags_7 },
    { 0xb28a7257, SUC_lexicon_tags_16 },
    { 0x296a7258, SUC_lexicon_tags_37 },
    { 0xbaaa7256, SUC_lexicon_tags_11 },
    { 0x20ea7259, SUC_lexicon_tags_14 },
    { 0x5caa725b, SUC_lexicon_tags_7 },
    { 0x4b4a7256, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aca7260, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0a7264, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76a7268, SUC_lexicon_tags_17 },
    { 0x75ca7269, SUC_lexicon_tags_7 },
    { 0x656a726a, SUC_lexicon_tags_15 },
    { 0x584a726b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa42a726d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8a7270, SUC_lexicon_tags_0 },
    { 0xca2a7271, SUC_lexicon_tags_10 },
    { 0x35ea7272, SUC_lexicon_tags_14 },
    { 0x32aa7273, SUC_lexicon_tags_30 },
    { 0xe4a7273, SUC_lexicon_tags_121 },
    { 0x542a7275, SUC_lexicon_tags_47 },
    { 0x6a8a7272, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa00a7278, SUC_lexicon_tags_17 },
    { 0xb30a7278, SUC_lexicon_tags_8 },
    { 0x63aa7278, SUC_lexicon_tags_14 },
    { 0x6cea727a, SUC_lexicon_tags_37 },
    { 0x6a0a727c, SUC_lexicon_tags_47 },
    { 0xb98a727d, SUC_lexicon_tags_12 },
    { 0xc54a727d, SUC_lexicon_tags_11 },
    { 0xceea727f, SUC_lexicon_tags_21 },
    { 0x668a727e, SUC_lexicon_tags_7 },
    { 0xf04a7281, SUC_lexicon_tags_17 },
    { 0xbf2a727a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4a7284, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2b0a7286, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ea7289, SUC_lexicon_tags_1 },
    { 0xf60a7289, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefea7292, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa7296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x296a7298, SUC_lexicon_tags_29 },
    { 0xedaa7299, SUC_lexicon_tags_37 },
    { 0xa02a729a, SUC_lexicon_tags_12 },
    { 0xe70a729b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10ea729d, SUC_lexicon_tags_2 },
    { 0xa72a729e, SUC_lexicon_tags_2 },
    { 0x738a729d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914a72a3, SUC_lexicon_tags_0 },
    { 0xd7aa72a3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988a72ab, SUC_lexicon_tags_91 },
    { 0xe84a72ac, SUC_lexicon_tags_60 },
    { 0x856a72ad, SUC_lexicon_tags_17 },
    { 0x2eca72ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedca72b3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbca72b6, SUC_lexicon_tags_0 },
    { 0x6a6a72b7, SUC_lexicon_tags_43 },
    { 0xe0ca72b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2a72ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988a72bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18a72c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bca72c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90a72c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2a72ca, SUC_lexicon_tags_37 },
    { 0x118a72cb, SUC_lexicon_tags_27 },
    { 0xbaea72cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2a72d4, SUC_lexicon_tags_1 },
    { 0xc7aa72d5, SUC_lexicon_tags_7 },
    { 0x718a72d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8a72d9, SUC_lexicon_tags_19 },
    { 0xaf0a72da, SUC_lexicon_tags_37 },
    { 0x4a2a72da, SUC_lexicon_tags_33 },
    { 0xd2ca72dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0a72df, SUC_lexicon_tags_82 },
    { 0xf7ca72e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6a72e3, SUC_lexicon_tags_19 },
    { 0x82aa72e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x900a72e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x634a72e9, SUC_lexicon_tags_28 },
    { 0x5e4a72ea, SUC_lexicon_tags_60 },
    { 0x14aa72eb, SUC_lexicon_tags_33 },
    { 0xc5ca72ec, SUC_lexicon_tags_25 },
    { 0x24aa72ed, SUC_lexicon_tags_1 },
    { 0xcf0a72ee, SUC_lexicon_tags_25 },
    { 0x9d4a72ec, SUC_lexicon_tags_1 },
    { 0x882a72f0, SUC_lexicon_tags_45 },
    { 0x8a0a72f1, SUC_lexicon_tags_14 },
    { 0x97ea72f2, SUC_lexicon_tags_35 },
    { 0x258a72ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec8a72f5, SUC_lexicon_tags_12 },
    { 0x4aa72f6, SUC_lexicon_tags_47 },
    { 0x24a72f7, SUC_lexicon_tags_30 },
    { 0xf1ea72f8, SUC_lexicon_tags_87 },
    { 0x2b6a72f8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bca72fe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7aa7303, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ea730c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2b4a730e, SUC_lexicon_tags_17 },
    { 0xf26a730f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0a7317, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dca731d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7a6a731f, SUC_lexicon_tags_63 },
    { 0x224a7320, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0a7324, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x362a7327, SUC_lexicon_tags_7 },
    { 0x3b4a7328, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x270a732a, SUC_lexicon_tags_16 },
    { 0x468a732a, SUC_lexicon_tags_4 },
    { 0x502a732c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994a7330, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x868a7333, SUC_lexicon_tags_37 },
    { 0xbeea7334, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29aa7338, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350a733c, SUC_lexicon_tags_25 },
    { 0x956a733d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8a7348, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ea734c, SUC_lexicon_tags_37 },
    { 0x4e8a734d, SUC_lexicon_tags_27 },
    { 0xd00a734d, SUC_lexicon_tags_96 },
    { 0xe78a734d, SUC_lexicon_tags_11 },
    { 0x762a7350, SUC_lexicon_tags_2 },
    { 0x746a7351, SUC_lexicon_tags_47 },
    { 0xec0a7350, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784a7357, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe22a7359, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246a736f, SUC_lexicon_tags_43 },
    { 0x4d4a736f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2a7374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd16a7376, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284a737a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x926a737c, SUC_lexicon_tags_17 },
    { 0x5b4a737c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18ca737f, SUC_lexicon_tags_13 },
    { 0xed6a7380, SUC_lexicon_tags_1 },
    { 0xe90a7381, SUC_lexicon_tags_12 },
    { 0xb28a7382, SUC_lexicon_tags_19 },
    { 0x388a7383, SUC_lexicon_tags_15 },
    { 0xffca7384, SUC_lexicon_tags_7 },
    { 0x90a7384, SUC_lexicon_tags_11 },
    { 0xa14a7381, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6a738e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x356a7391, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87aa7393, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412a7397, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a739d, SUC_lexicon_tags_7 },
    { 0x1b2a739e, SUC_lexicon_tags_33 },
    { 0xaca739e, SUC_lexicon_tags_12 },
    { 0x75aa73a0, SUC_lexicon_tags_56 },
    { 0x7c8a73a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a73a4, SUC_lexicon_tags_38 },
    { 0xe3ca73a5, SUC_lexicon_tags_7 },
    { 0x6ea73a6, SUC_lexicon_tags_31 },
    { 0x7b0a73a7, SUC_lexicon_tags_23 },
    { 0xae6a73a7, SUC_lexicon_tags_475 },
    { 0x3e4a73a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34a73ac, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caa73b2, SUC_lexicon_tags_15 },
    { 0x6eea73b3, SUC_lexicon_tags_2 },
    { 0x6cea73b3, SUC_lexicon_tags_37 },
    { 0xe68a73b4, SUC_lexicon_tags_23 },
    { 0xc18a73b4, SUC_lexicon_tags_17 },
    { 0x38a73b2, SUC_lexicon_tags_13 },
    { 0x3a6a73b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x232a73ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa34a73bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x286a73be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55aa73c0, SUC_lexicon_tags_1 },
    { 0x8cca73c1, SUC_lexicon_tags_13 },
    { 0x8b8a73c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdb0a73c4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ea73c8, SUC_lexicon_tags_47 },
    { 0x370a73c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586a73cd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x92aa73cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66a73d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4a73d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eca73db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb1aa73dd, SUC_lexicon_tags_11 },
    { 0xcd2a73de, SUC_lexicon_tags_47 },
    { 0x1ca73df, SUC_lexicon_tags_23 },
    { 0xbc6a73df, SUC_lexicon_tags_37 },
    { 0x610a73e1, SUC_lexicon_tags_1 },
    { 0xff0a73e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02a73e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x594a73e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a73ec, SUC_lexicon_tags_14 },
    { 0x29aa73ed, SUC_lexicon_tags_47 },
    { 0x756a73ed, SUC_lexicon_tags_61 },
    { 0x734a73ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x102a73f2, SUC_lexicon_tags_63 },
    { 0xbb4a73f3, SUC_lexicon_tags_37 },
    { 0x6b4a73f4, SUC_lexicon_tags_32 },
    { 0x164a73f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e4a73f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0a73fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x796a73fc, SUC_lexicon_tags_17 },
    { 0xc58a73fd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700a7404, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x460a7407, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aa740a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ea740d, SUC_lexicon_tags_0 },
    { 0x1bea740e, SUC_lexicon_tags_47 },
    { 0x668a740f, SUC_lexicon_tags_60 },
    { 0x4e4a7410, SUC_lexicon_tags_71 },
    { 0xc9ea740d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52a7416, SUC_lexicon_tags_37 },
    { 0xb04a7416, SUC_lexicon_tags_31 },
    { 0x724a7418, SUC_lexicon_tags_0 },
    { 0xc56a7418, SUC_lexicon_tags_23 },
    { 0x8d0a7416, SUC_lexicon_tags_12 },
    { 0x298a741b, SUC_lexicon_tags_87 },
    { 0x556a741b, SUC_lexicon_tags_15 },
    { 0xf2aa741d, SUC_lexicon_tags_28 },
    { 0x4aca741e, SUC_lexicon_tags_15 },
    { 0xa8aa741d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ca7426, SUC_lexicon_tags_46 },
    { 0x170a7427, SUC_lexicon_tags_1 },
    { 0x70ea7428, SUC_lexicon_tags_17 },
    { 0xcea7429, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0a742c, SUC_lexicon_tags_60 },
    { 0x64ca742d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912a743b, SUC_lexicon_tags_13 },
    { 0x1dea743b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ea7444, SUC_lexicon_tags_37 },
    { 0x748a7445, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x278a7447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x290a744a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2a744e, SUC_lexicon_tags_25 },
    { 0x9c0a744f, SUC_lexicon_tags_49 },
    { 0xac2a744f, SUC_lexicon_tags_61 },
    { 0x91ea744f, SUC_lexicon_tags_17 },
    { 0xdc0a7452, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2a7458, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf0a745a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2a745e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d0a7460, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x388a7462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214a7469, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92a746c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x24a746f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ca7474, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x99ca7476, SUC_lexicon_tags_2 },
    { 0x694a7477, SUC_lexicon_tags_15 },
    { 0x9a6a7478, SUC_lexicon_tags_49 },
    { 0x856a7479, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60a7481, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x938a7483, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8a748d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xeaaa748f, SUC_lexicon_tags_15 },
    { 0x350a7490, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498a749a, SUC_lexicon_tags_9 },
    { 0x7d4a749b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x90a749d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xa74a749f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda0a74a1, SUC_lexicon_tags_14 },
    { 0x336a74a2, SUC_lexicon_tags_17 },
    { 0x460a74a3, SUC_lexicon_tags_15 },
    { 0xd16a74a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692a74a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8a74ae, SUC_lexicon_tags_75 },
    { 0xd4a74af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78a74b3, SUC_lexicon_tags_11 },
    { 0xb96a74b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd00a74b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x162a74b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aea74bc, SUC_lexicon_tags_7 },
    { 0xb50a74bd, SUC_lexicon_tags_21 },
    { 0x736a74be, SUC_lexicon_tags_65 },
    { 0x51aa74bc, SUC_lexicon_tags_7 },
    { 0x9aca74c0, SUC_lexicon_tags_37 },
    { 0xc62a74c1, SUC_lexicon_tags_25 },
    { 0xa6a74c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26a74c5, SUC_lexicon_tags_11 },
    { 0x2b2a74c6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb40a74c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70a74cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ca74cf, SUC_lexicon_tags_28 },
    { 0x668a74d0, SUC_lexicon_tags_8 },
    { 0x2aaa74d0, SUC_lexicon_tags_11 },
    { 0xdaaa74d1, SUC_lexicon_tags_12 },
    { 0x754a74d3, SUC_lexicon_tags_13 },
    { 0xa76a74d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2a74d8, SUC_lexicon_tags_2 },
    { 0x9aaa74d9, SUC_lexicon_tags_15 },
    { 0xeb2a74da, SUC_lexicon_tags_33 },
    { 0x440a74db, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8a74de, SUC_lexicon_tags_41 },
    { 0x358a74df, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6a74e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe90a74eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6a74f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82a74f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6a74fa, SUC_lexicon_tags_37 },
    { 0x98a74fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x358a74fd, SUC_lexicon_tags_25 },
    { 0x4aa74fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xca8a7501, SUC_lexicon_tags_0 },
    { 0xca0a7501, SUC_lexicon_tags_70 },
    { 0x8b6a7503, SUC_lexicon_tags_14 },
    { 0xeeca7504, SUC_lexicon_tags_28 },
    { 0x8d4a7505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cca7508, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeca7510, SUC_lexicon_tags_17 },
    { 0xe2ca7511, SUC_lexicon_tags_17 },
    { 0x416a7511, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2a7515, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a751b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6a751f, SUC_lexicon_tags_4 },
    { 0xf2ea7520, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9eea7522, SUC_lexicon_tags_10 },
    { 0x6f2a7523, SUC_lexicon_tags_15 },
    { 0xcaca7524, SUC_lexicon_tags_11 },
    { 0x76aa7522, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378a752b, SUC_lexicon_tags_23 },
    { 0x9baa752c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ca752f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x19aa7531, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x894a7533, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8aa7535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44a7539, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2a753e, SUC_lexicon_tags_8 },
    { 0xd7ca753e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2a7542, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x974a7545, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d6a7547, SUC_lexicon_tags_14 },
    { 0x1f0a7548, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9aa754a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4baa754d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1aa7559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50a755b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaea7561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06a756b, SUC_lexicon_tags_6 },
    { 0xbf0a756c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0a7570, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdd8a7572, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfaca7574, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3aa7580, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0a7583, SUC_lexicon_tags_15 },
    { 0xa68a7584, SUC_lexicon_tags_45 },
    { 0x662a7585, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614a758b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d8a758d, SUC_lexicon_tags_7 },
    { 0xd4ea758e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x18ca7590, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xb38a7592, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78a7597, SUC_lexicon_tags_39 },
    { 0xeeaa7598, SUC_lexicon_tags_7 },
    { 0x6a8a7598, SUC_lexicon_tags_7 },
    { 0xdcaa759a, SUC_lexicon_tags_26 },
    { 0xe20a7599, SUC_lexicon_tags_23 },
    { 0x1eaa759c, SUC_lexicon_tags_45 },
    { 0x4b4a7597, SUC_lexicon_tags_7 },
    { 0x98a759e, SUC_lexicon_tags_12 },
    { 0x4aa759f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe9ca75a1, SUC_lexicon_tags_1 },
    { 0xe7ea75a2, SUC_lexicon_tags_14 },
    { 0xafaa75a2, SUC_lexicon_tags_19 },
    { 0x248a75a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2eaa75a6, SUC_lexicon_tags_30 },
    { 0x6d0a75a6, SUC_lexicon_tags_17 },
    { 0xb46a75a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x480a75ab, SUC_lexicon_tags_33 },
    { 0xce4a75ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a75b1, SUC_lexicon_tags_68 },
    { 0x720a75b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafaa75b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a75b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8a75be, SUC_lexicon_tags_29 },
    { 0x30ca75bf, SUC_lexicon_tags_12 },
    { 0xf2ca75c0, SUC_lexicon_tags_15 },
    { 0xa68a75bf, SUC_lexicon_tags_37 },
    { 0x30a75be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b2a75c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daa75c8, SUC_lexicon_tags_27 },
    { 0xc10a75c9, SUC_lexicon_tags_12 },
    { 0x6fea75c9, SUC_lexicon_tags_12 },
    { 0xcb2a75c9, SUC_lexicon_tags_37 },
    { 0x4e6a75c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc22a75ce, SUC_lexicon_tags_170 },
    { 0xb6a75cf, SUC_lexicon_tags_12 },
    { 0x4dca75cf, SUC_lexicon_tags_1 },
    { 0x416a75d1, SUC_lexicon_tags_47 },
    { 0xe52a75d1, SUC_lexicon_tags_18 },
    { 0xef8a75ce, SUC_lexicon_tags_194 },
    { 0xf0aa75ce, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8a75dd, SUC_lexicon_tags_19 },
    { 0x850a75de, SUC_lexicon_tags_56 },
    { 0xcd2a75df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0ca75e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd78a75e3, SUC_lexicon_tags_29 },
    { 0x350a75e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffea75e8, SUC_lexicon_tags_33 },
    { 0xf40a75e9, SUC_lexicon_tags_84 },
    { 0x446a75e8, SUC_lexicon_tags_17 },
    { 0xa2aa75eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2a75ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22a75f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4a75f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ea75fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94a7601, SUC_lexicon_tags_0 },
    { 0x746a7602, SUC_lexicon_tags_68 },
    { 0x2d6a7603, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xebca7605, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484a7609, SUC_lexicon_tags_17 },
    { 0xf38a7609, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc90a760c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7aa7613, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ea7617, SUC_lexicon_tags_160 },
    { 0xf5ea7618, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x346a761a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xedaa761c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb0a761e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298a7624, SUC_lexicon_tags_11 },
    { 0x48ca7625, SUC_lexicon_tags_15 },
    { 0x3c2a7626, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe30a7628, SUC_lexicon_tags_47 },
    { 0x92aa7628, SUC_lexicon_tags_21 },
    { 0xd9ea762a, SUC_lexicon_tags_29 },
    { 0xf04a762b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe02a762d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6aa762f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ca7632, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390a7637, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ca763a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2a763d, SUC_lexicon_tags_45 },
    { 0x980a763e, SUC_lexicon_tags_11 },
    { 0x8aca763f, SUC_lexicon_tags_25 },
    { 0xabca763e, SUC_lexicon_tags_47 },
    { 0xcf6a7640, SUC_lexicon_tags_30 },
    { 0xf5ea7642, SUC_lexicon_tags_15 },
    { 0x758a7643, SUC_lexicon_tags_28 },
    { 0xa34a7643, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ca764c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ca764f, SUC_lexicon_tags_11 },
    { 0xf2aa7650, SUC_lexicon_tags_12 },
    { 0x308a7651, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ca7658, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42a765b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x134a765d, SUC_lexicon_tags_17 },
    { 0x83ea765e, SUC_lexicon_tags_16 },
    { 0xca0a765d, SUC_lexicon_tags_0 },
    { 0x48ea7660, SUC_lexicon_tags_38 },
    { 0x2daa7661, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0a7666, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x364a7669, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2a7670, SUC_lexicon_tags_1 },
    { 0x1fea7671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea767d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ea7681, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb00a7683, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ca7688, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb88a768a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaa768f, SUC_lexicon_tags_4 },
    { 0x8e2a768f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92a7694, SUC_lexicon_tags_16 },
    { 0x144a7694, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15ea7697, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca769b, SUC_lexicon_tags_25 },
    { 0x212a769c, SUC_lexicon_tags_12 },
    { 0x372a769d, SUC_lexicon_tags_17 },
    { 0x108a769e, SUC_lexicon_tags_1 },
    { 0xe80a769f, SUC_lexicon_tags_14 },
    { 0x2a769f, SUC_lexicon_tags_2 },
    { 0x3e6a769e, SUC_lexicon_tags_11 },
    { 0x82ca76a2, SUC_lexicon_tags_32 },
    { 0xc34a76a2, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ca76a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33aa76a8, SUC_lexicon_tags_87 },
    { 0xb1aa76a8, SUC_lexicon_tags_17 },
    { 0xd38a76a8, SUC_lexicon_tags_2 },
    { 0x978a76ab, SUC_lexicon_tags_2 },
    { 0xc8ca76ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90a76b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd3ca76b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x456a76b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa1ca76b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2a76b9, SUC_lexicon_tags_11 },
    { 0xfaaa76ba, SUC_lexicon_tags_6 },
    { 0x1bca76ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x67aa76bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370a76c6, SUC_lexicon_tags_45 },
    { 0xacaa76c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x64ea76c9, SUC_lexicon_tags_33 },
    { 0x966a76ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14a76ce, SUC_lexicon_tags_13 },
    { 0xfb8a76ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18ea76d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaa76d4, SUC_lexicon_tags_29 },
    { 0x35ea76d4, SUC_lexicon_tags_31 },
    { 0x490a76d6, SUC_lexicon_tags_27 },
    { 0xa08a76d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36a76da, SUC_lexicon_tags_11 },
    { 0xc8ca76db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6a76e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88a76e6, SUC_lexicon_tags_27 },
    { 0xe96a76e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8a76eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ea76f3, SUC_lexicon_tags_80 },
    { 0x790a76f4, SUC_lexicon_tags_25 },
    { 0x91ea76f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x66a76f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ca76fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ea7701, SUC_lexicon_tags_29 },
    { 0x322a7701, SUC_lexicon_tags_236 },
    { 0x56ca7703, SUC_lexicon_tags_11 },
    { 0x966a7701, SUC_lexicon_tags_4 },
    { 0xc12a7705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a7707, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89aa770c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff2a770e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28a7711, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdea7717, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ea771b, SUC_lexicon_tags_68 },
    { 0xcd0a771b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6ea771e, SUC_lexicon_tags_17 },
    { 0x2a0a771f, SUC_lexicon_tags_60 },
    { 0x926a7720, SUC_lexicon_tags_37 },
    { 0xb98a771f, SUC_lexicon_tags_33 },
    { 0xe70a771e, SUC_lexicon_tags_21 },
    { 0xf3ca7721, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x38aa7725, SUC_lexicon_tags_37 },
    { 0xf70a7726, SUC_lexicon_tags_4 },
    { 0xf98a7726, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x430a7729, SUC_lexicon_tags_17 },
    { 0x22ca772a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf4a772c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf34a772e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ca7731, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x244a7733, SUC_lexicon_tags_261 },
    { 0xaa0a7734, SUC_lexicon_tags_24 },
    { 0xddca7735, SUC_lexicon_tags_1 },
    { 0x74aa7734, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xea0a7738, SUC_lexicon_tags_37 },
    { 0x782a7738, SUC_lexicon_tags_17 },
    { 0x4d8a773a, SUC_lexicon_tags_23 },
    { 0x6f4a773b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698a773f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a7744, SUC_lexicon_tags_57 },
    { 0xbd6a7745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72a7747, SUC_lexicon_tags_43 },
    { 0xd58a7748, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x624a774a, SUC_lexicon_tags_61 },
    { 0xcb2a774b, SUC_lexicon_tags_22 },
    { 0xe6aa774a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa08a774e, SUC_lexicon_tags_2 },
    { 0x7b0a774e, SUC_lexicon_tags_1 },
    { 0xb02a774e, SUC_lexicon_tags_25 },
    { 0x6f8a7751, SUC_lexicon_tags_14 },
    { 0xc18a7751, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x962a7755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc10a7757, SUC_lexicon_tags_26 },
    { 0xe48a7758, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x76ea775a, SUC_lexicon_tags_47 },
    { 0x128a775b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfca775e, SUC_lexicon_tags_63 },
    { 0x526a775e, SUC_lexicon_tags_27 },
    { 0x1fea7760, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0a7767, SUC_lexicon_tags_11 },
    { 0x812a7768, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8a776d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2a7775, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0a7778, SUC_lexicon_tags_38 },
    { 0x1e0a7779, SUC_lexicon_tags_1083 },
    { 0, NULL },
    { 0, NULL },
    { 0x662a777c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf10a777e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x520a7780, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67aa7784, SUC_lexicon_tags_11 },
    { 0x8e2a7784, SUC_lexicon_tags_11 },
    { 0xafea7784, SUC_lexicon_tags_37 },
    { 0xd2a7787, SUC_lexicon_tags_2 },
    { 0x60ea7784, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ea778c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68a778f, SUC_lexicon_tags_0 },
    { 0xdaaa7790, SUC_lexicon_tags_7 },
    { 0xdc8a7791, SUC_lexicon_tags_49 },
    { 0x328a7791, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50a7796, SUC_lexicon_tags_1 },
    { 0x400a7797, SUC_lexicon_tags_43 },
    { 0xb90a7798, SUC_lexicon_tags_11 },
    { 0x9cea7799, SUC_lexicon_tags_14 },
    { 0x350a779a, SUC_lexicon_tags_7 },
    { 0x360a7796, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ea779e, SUC_lexicon_tags_11 },
    { 0x4fea779f, SUC_lexicon_tags_46 },
    { 0xc0a77a0, SUC_lexicon_tags_12 },
    { 0xdca779e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbf0a77a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a6a77a5, SUC_lexicon_tags_43 },
    { 0xbca77a6, SUC_lexicon_tags_17 },
    { 0x720a77a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe90a77a9, SUC_lexicon_tags_14 },
    { 0xea77aa, SUC_lexicon_tags_7 },
    { 0xd94a77ab, SUC_lexicon_tags_25 },
    { 0x960a77ac, SUC_lexicon_tags_15 },
    { 0x564a77ad, SUC_lexicon_tags_1 },
    { 0xaaaa77aa, SUC_lexicon_tags_1 },
    { 0x99ca77ac, SUC_lexicon_tags_38 },
    { 0x900a77a9, SUC_lexicon_tags_2 },
    { 0x55ca77b1, SUC_lexicon_tags_1 },
    { 0x2f8a77b0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aea77b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68ca77b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30a77b9, SUC_lexicon_tags_17 },
    { 0x22aa77b9, SUC_lexicon_tags_7 },
    { 0xc58a77b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52a77be, SUC_lexicon_tags_38 },
    { 0xe02a77be, SUC_lexicon_tags_35 },
    { 0xa14a77bf, SUC_lexicon_tags_7 },
    { 0x1bea77c1, SUC_lexicon_tags_30 },
    { 0xea0a77c2, SUC_lexicon_tags_25 },
    { 0x362a77c3, SUC_lexicon_tags_1 },
    { 0xdb6a77c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x998a77c7, SUC_lexicon_tags_30 },
    { 0x4e0a77c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a77cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x900a77ce, SUC_lexicon_tags_1261 },
    { 0, NULL },
    { 0xa32a77d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcca77d2, SUC_lexicon_tags_43 },
    { 0x1eea77d2, SUC_lexicon_tags_60 },
    { 0xb06a77d3, SUC_lexicon_tags_1 },
    { 0xedea77d5, SUC_lexicon_tags_15 },
    { 0xf0aa77d5, SUC_lexicon_tags_1 },
    { 0x290a77d6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe4ea77d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4a77dd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0a77e1, SUC_lexicon_tags_8 },
    { 0x21aa77e2, SUC_lexicon_tags_49 },
    { 0x86ca77e3, SUC_lexicon_tags_60 },
    { 0x32aa77e2, SUC_lexicon_tags_12 },
    { 0xa00a77e5, SUC_lexicon_tags_48 },
    { 0xdf0a77e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4a77ea, SUC_lexicon_tags_37 },
    { 0x562a77eb, SUC_lexicon_tags_56 },
    { 0x11aa77ea, SUC_lexicon_tags_14 },
    { 0x142a77ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9ca77ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeea77f3, SUC_lexicon_tags_48 },
    { 0xea8a77f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a4a77f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x572a77f9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaa77ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb28a7801, SUC_lexicon_tags_1 },
    { 0x5a0a7801, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2a7805, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14a780a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x60a780c, SUC_lexicon_tags_68 },
    { 0x562a780d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96a780f, SUC_lexicon_tags_11 },
    { 0x4d0a7810, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xafca7812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aa7815, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ca7819, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xffea781b, SUC_lexicon_tags_25 },
    { 0xd8ca781c, SUC_lexicon_tags_7 },
    { 0x366a781d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ca7821, SUC_lexicon_tags_7 },
    { 0xe8ca7822, SUC_lexicon_tags_4 },
    { 0x7c0a7823, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccea7827, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890a782e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca7834, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8a7839, SUC_lexicon_tags_70 },
    { 0xcb0a783a, SUC_lexicon_tags_21 },
    { 0xd78a783b, SUC_lexicon_tags_17 },
    { 0xd48a783c, SUC_lexicon_tags_17 },
    { 0x6ea783b, SUC_lexicon_tags_2 },
    { 0xd2a783e, SUC_lexicon_tags_37 },
    { 0x8e6a783e, SUC_lexicon_tags_2 },
    { 0x75ea783e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464a7844, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dca7848, SUC_lexicon_tags_66 },
    { 0x4d2a7849, SUC_lexicon_tags_82 },
    { 0x6aca784a, SUC_lexicon_tags_45 },
    { 0xfd8a784a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6f0a784d, SUC_lexicon_tags_46 },
    { 0x164a784d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5faa7851, SUC_lexicon_tags_54 },
    { 0x448a7852, SUC_lexicon_tags_46 },
    { 0xc2ca7852, SUC_lexicon_tags_37 },
    { 0x30ca7854, SUC_lexicon_tags_37 },
    { 0xf46a7855, SUC_lexicon_tags_31 },
    { 0x3aaa7851, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6a7867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60a786d, SUC_lexicon_tags_1 },
    { 0xfd0a786d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ea7875, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25aa787a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x118a787d, SUC_lexicon_tags_48 },
    { 0x534a787e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2a7882, SUC_lexicon_tags_3 },
    { 0xee4a7882, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x876a7886, SUC_lexicon_tags_1 },
    { 0x62a7887, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b2a7889, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ea7891, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x394a7893, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ea789a, SUC_lexicon_tags_11 },
    { 0xc76a789a, SUC_lexicon_tags_30 },
    { 0x5fea789b, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xc0a789e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504a78a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76a78a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6a78ab, SUC_lexicon_tags_2 },
    { 0x6aca78ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360a78b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bea78b7, SUC_lexicon_tags_25 },
    { 0x6cca78b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x154a78bb, SUC_lexicon_tags_15 },
    { 0x594a78bb, SUC_lexicon_tags_7 },
    { 0xd82a78bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d2a78bf, SUC_lexicon_tags_6 },
    { 0x364a78bf, SUC_lexicon_tags_1 },
    { 0xaf4a78c1, SUC_lexicon_tags_37 },
    { 0x8daa78c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x920a78c4, SUC_lexicon_tags_7 },
    { 0xb3ca78c5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242a78cc, SUC_lexicon_tags_17 },
    { 0x186a78cd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28a78d1, SUC_lexicon_tags_17 },
    { 0x9eaa78d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd6a78d4, SUC_lexicon_tags_17 },
    { 0xdea78d4, SUC_lexicon_tags_17 },
    { 0xf3ca78d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x294a78d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x454a78db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x288a78dd, SUC_lexicon_tags_46 },
    { 0xa9ea78de, SUC_lexicon_tags_29 },
    { 0xf98a78de, SUC_lexicon_tags_38 },
    { 0x12ca78e0, SUC_lexicon_tags_2 },
    { 0xee8a78df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2a78e4, SUC_lexicon_tags_2 },
    { 0x638a78e5, SUC_lexicon_tags_20 },
    { 0xd82a78e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaa78e9, SUC_lexicon_tags_44 },
    { 0xdbaa78ea, SUC_lexicon_tags_29 },
    { 0xeb8a78eb, SUC_lexicon_tags_15 },
    { 0x11aa78ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4a78ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ea78f2, SUC_lexicon_tags_2 },
    { 0xb22a78f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28a78f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea78f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6a78fb, SUC_lexicon_tags_23 },
    { 0x518a78fb, SUC_lexicon_tags_30 },
    { 0x76ea78fc, SUC_lexicon_tags_12 },
    { 0x63ea78fd, SUC_lexicon_tags_55 },
    { 0x338a78ff, SUC_lexicon_tags_68 },
    { 0x1b2a78ff, SUC_lexicon_tags_19 },
    { 0x250a78ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50a7907, SUC_lexicon_tags_47 },
    { 0x77ca7908, SUC_lexicon_tags_11 },
    { 0xd98a7909, SUC_lexicon_tags_6 },
    { 0xabaa7909, SUC_lexicon_tags_11 },
    { 0x294a790b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6aa790d, SUC_lexicon_tags_27 },
    { 0x2baa790e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc26a7910, SUC_lexicon_tags_15 },
    { 0xce2a7911, SUC_lexicon_tags_27 },
    { 0x5dea7912, SUC_lexicon_tags_6 },
    { 0x1a6a7913, SUC_lexicon_tags_33 },
    { 0x984a7914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd96a7916, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aa7923, SUC_lexicon_tags_37 },
    { 0x9c2a7923, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ca792a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2a792f, SUC_lexicon_tags_2 },
    { 0x6b8a7930, SUC_lexicon_tags_28 },
    { 0x132a7931, SUC_lexicon_tags_25 },
    { 0x762a7931, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88a7936, SUC_lexicon_tags_2 },
    { 0x208a7937, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54aa793a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ea793e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790a7945, SUC_lexicon_tags_37 },
    { 0x396a7945, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636a794a, SUC_lexicon_tags_28 },
    { 0xf92a794b, SUC_lexicon_tags_3 },
    { 0x5a0a794b, SUC_lexicon_tags_14 },
    { 0x994a794c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2a7955, SUC_lexicon_tags_27 },
    { 0xf34a7956, SUC_lexicon_tags_45 },
    { 0x4c0a7957, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x436a795a, SUC_lexicon_tags_38 },
    { 0x66ea795a, SUC_lexicon_tags_3 },
    { 0x706a795c, SUC_lexicon_tags_23 },
    { 0xc42a795c, SUC_lexicon_tags_55 },
    { 0x12ea795c, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0x8dea7960, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61ea7962, SUC_lexicon_tags_43 },
    { 0x2dca7963, SUC_lexicon_tags_11 },
    { 0xa54a7964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ea7969, SUC_lexicon_tags_30 },
    { 0xccea796a, SUC_lexicon_tags_25 },
    { 0xda4a796b, SUC_lexicon_tags_19 },
    { 0x6d4a796b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0a7970, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x844a7972, SUC_lexicon_tags_85 },
    { 0x916a7972, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1d0a7975, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a797a, SUC_lexicon_tags_19 },
    { 0xd8ea797a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb04a797d, SUC_lexicon_tags_23 },
    { 0xa74a797e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56a7982, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x188a7985, SUC_lexicon_tags_17 },
    { 0x150a7985, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18a798c, SUC_lexicon_tags_9 },
    { 0xd06a798d, SUC_lexicon_tags_37 },
    { 0x77aa798d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfea7990, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7997, SUC_lexicon_tags_11 },
    { 0x6eca7997, SUC_lexicon_tags_7 },
    { 0xd32a7998, SUC_lexicon_tags_26 },
    { 0xa90a7998, SUC_lexicon_tags_26 },
    { 0x3bca799b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2a799f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25aa79a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcca79a5, SUC_lexicon_tags_45 },
    { 0xf02a79a6, SUC_lexicon_tags_12 },
    { 0x5b2a79a7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ca79ab, SUC_lexicon_tags_7 },
    { 0x6f0a79ac, SUC_lexicon_tags_17 },
    { 0x95ca79ad, SUC_lexicon_tags_64 },
    { 0x612a79ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ea79b0, SUC_lexicon_tags_2 },
    { 0x1f4a79b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x16a79b3, SUC_lexicon_tags_252 },
    { 0x7c0a79b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dca79b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x532a79b9, SUC_lexicon_tags_23 },
    { 0xdea79ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6a79bd, SUC_lexicon_tags_30 },
    { 0x94a79bd, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770a79ca, SUC_lexicon_tags_1 },
    { 0xa6ca79cb, SUC_lexicon_tags_1 },
    { 0x406a79cc, SUC_lexicon_tags_14 },
    { 0x898a79ca, SUC_lexicon_tags_23 },
    { 0xcb2a79ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x22aa79d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x220a79d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63ca79d4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aa79d8, SUC_lexicon_tags_43 },
    { 0x970a79d9, SUC_lexicon_tags_9 },
    { 0x6dea79da, SUC_lexicon_tags_14 },
    { 0xaea79db, SUC_lexicon_tags_17 },
    { 0xea79dc, SUC_lexicon_tags_7 },
    { 0xc22a79dd, SUC_lexicon_tags_7 },
    { 0xdc6a79de, SUC_lexicon_tags_25 },
    { 0xb6ca79de, SUC_lexicon_tags_588 },
    { 0x64a79df, SUC_lexicon_tags_16 },
    { 0x60ea79df, SUC_lexicon_tags_39 },
    { 0x56ea79de, SUC_lexicon_tags_9 },
    { 0xde8a79e1, SUC_lexicon_tags_56 },
    { 0xd58a79e4, SUC_lexicon_tags_25 },
    { 0x426a79e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492a79eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2a79f1, SUC_lexicon_tags_30 },
    { 0x688a79f2, SUC_lexicon_tags_25 },
    { 0x176a79f2, SUC_lexicon_tags_34 },
    { 0x76aa79f4, SUC_lexicon_tags_160 },
    { 0x882a79f4, SUC_lexicon_tags_61 },
    { 0x64ea79f3, SUC_lexicon_tags_79 },
    { 0x50a79f6, SUC_lexicon_tags_37 },
    { 0x91aa79f8, SUC_lexicon_tags_25 },
    { 0x388a79f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4a79fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720a7a00, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x52ea7a02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10a7a08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x130a7a0b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862a7a10, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77aa7a15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x196a7a18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ea7a1e, SUC_lexicon_tags_38 },
    { 0x86aa7a1e, SUC_lexicon_tags_37 },
    { 0x50aa7a20, SUC_lexicon_tags_17 },
    { 0x48aa7a21, SUC_lexicon_tags_61 },
    { 0xf0aa7a21, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xdf6a7a24, SUC_lexicon_tags_15 },
    { 0xcd0a7a25, SUC_lexicon_tags_12 },
    { 0xf00a7a26, SUC_lexicon_tags_33 },
    { 0x8c0a7a27, SUC_lexicon_tags_11 },
    { 0xed6a7a28, SUC_lexicon_tags_38 },
    { 0x362a7a29, SUC_lexicon_tags_56 },
    { 0x5f4a7a27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe62a7a2c, SUC_lexicon_tags_17 },
    { 0x5faa7a2c, SUC_lexicon_tags_11 },
    { 0x7b0a7a2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140a7a33, SUC_lexicon_tags_47 },
    { 0x420a7a33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13aa7a39, SUC_lexicon_tags_7 },
    { 0x278a7a3a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516a7a3e, SUC_lexicon_tags_16 },
    { 0x836a7a3e, SUC_lexicon_tags_25 },
    { 0xe16a7a40, SUC_lexicon_tags_63 },
    { 0x886a7a3e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aa7a45, SUC_lexicon_tags_33 },
    { 0xff6a7a45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3ca7a48, SUC_lexicon_tags_79 },
    { 0x8b2a7a49, SUC_lexicon_tags_25 },
    { 0x4bea7a4a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5c0a7a4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202a7a55, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8a7a5b, SUC_lexicon_tags_49 },
    { 0xa1ca7a5b, SUC_lexicon_tags_17 },
    { 0xc42a7a5c, SUC_lexicon_tags_37 },
    { 0x314a7a5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ea7a63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4a7a6b, SUC_lexicon_tags_83 },
    { 0xccea7a6c, SUC_lexicon_tags_17 },
    { 0xf16a7a6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6a7a74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4daa7a76, SUC_lexicon_tags_13 },
    { 0x930a7a77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72aa7a81, SUC_lexicon_tags_1 },
    { 0x7aca7a82, SUC_lexicon_tags_1 },
    { 0x96a7a83, SUC_lexicon_tags_11 },
    { 0x6f0a7a84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aa7a8e, SUC_lexicon_tags_7 },
    { 0x7f8a7a8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4a7a95, SUC_lexicon_tags_37 },
    { 0x66aa7a96, SUC_lexicon_tags_12 },
    { 0xb3ea7a96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc60a7a99, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2faa7a9c, SUC_lexicon_tags_47 },
    { 0xa4ea7a9d, SUC_lexicon_tags_11 },
    { 0xf36a7a9e, SUC_lexicon_tags_15 },
    { 0xdbea7a9f, SUC_lexicon_tags_25 },
    { 0x98aa7aa0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5faa7aa5, SUC_lexicon_tags_47 },
    { 0xb24a7aa5, SUC_lexicon_tags_2 },
    { 0x560a7aa5, SUC_lexicon_tags_25 },
    { 0xbaa7aa8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x190a7aaa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x346a7aac, SUC_lexicon_tags_15 },
    { 0x9bca7aad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa4ea7aaf, SUC_lexicon_tags_26 },
    { 0x36ca7ab0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8a7ab5, SUC_lexicon_tags_17 },
    { 0x1d4a7ab6, SUC_lexicon_tags_27 },
    { 0x84aa7ab7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ea7abb, SUC_lexicon_tags_31 },
    { 0x5aea7abc, SUC_lexicon_tags_29 },
    { 0xc1ca7abc, SUC_lexicon_tags_37 },
    { 0xf48a7abe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe52a7ac1, SUC_lexicon_tags_17 },
    { 0xfca7ac2, SUC_lexicon_tags_37 },
    { 0x31aa7ac3, SUC_lexicon_tags_32 },
    { 0xceea7ac4, SUC_lexicon_tags_39 },
    { 0x61ca7ac2, SUC_lexicon_tags_28 },
    { 0xaca7ac6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaa7acc, SUC_lexicon_tags_23 },
    { 0xd8ea7acd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc36a7acf, SUC_lexicon_tags_66 },
    { 0xe0ca7ad0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302a7ad7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x500a7ad9, SUC_lexicon_tags_0 },
    { 0x22ea7ada, SUC_lexicon_tags_37 },
    { 0xea6a7adb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792a7adf, SUC_lexicon_tags_85 },
    { 0xba4a7adf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892a7ae4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6a7ae9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbdea7aeb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0a7af4, SUC_lexicon_tags_14 },
    { 0xe4a7af5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x434a7af7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4a7afc, SUC_lexicon_tags_23 },
    { 0x7aa7afc, SUC_lexicon_tags_11 },
    { 0x15ea7afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfeaa7b00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x648a7b02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aa7b05, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6f4a7b07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadca7b0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7b11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4ea7b13, SUC_lexicon_tags_179 },
    { 0x4d2a7b14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6a7b18, SUC_lexicon_tags_12 },
    { 0xf66a7b18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe4a7b1b, SUC_lexicon_tags_9 },
    { 0x536a7b1b, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xa60a7b1e, SUC_lexicon_tags_37 },
    { 0xb12a7b1f, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xdaca7b21, SUC_lexicon_tags_47 },
    { 0xdc4a7b21, SUC_lexicon_tags_17 },
    { 0x60a7b23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4a7b26, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88a7b2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ca7b31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fea7b36, SUC_lexicon_tags_7 },
    { 0x45ca7b37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc40a7b39, SUC_lexicon_tags_47 },
    { 0x378a7b3a, SUC_lexicon_tags_48 },
    { 0x93ea7b3b, SUC_lexicon_tags_12 },
    { 0x1e4a7b3a, SUC_lexicon_tags_7 },
    { 0xc06a7b3d, SUC_lexicon_tags_7 },
    { 0x712a7b3e, SUC_lexicon_tags_17 },
    { 0xfca7b3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78ca7b41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x576a7b43, SUC_lexicon_tags_45 },
    { 0x162a7b44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4a7b47, SUC_lexicon_tags_19 },
    { 0x4f0a7b48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa08a7b4a, SUC_lexicon_tags_11 },
    { 0x75ea7b4a, SUC_lexicon_tags_37 },
    { 0xa8a7b4a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2c4a7b4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2daa7b50, SUC_lexicon_tags_37 },
    { 0x936a7b50, SUC_lexicon_tags_13 },
    { 0x9a2a7b51, SUC_lexicon_tags_50 },
    { 0xbc0a7b53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x25ea7b55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x40ea7b57, SUC_lexicon_tags_17 },
    { 0x366a7b58, SUC_lexicon_tags_3 },
    { 0x63aa7b58, SUC_lexicon_tags_9 },
    { 0xdf0a7b59, SUC_lexicon_tags_25 },
    { 0xd4ca7b5b, SUC_lexicon_tags_17 },
    { 0xd18a7b5b, SUC_lexicon_tags_12 },
    { 0x95ea7b57, SUC_lexicon_tags_1 },
    { 0x9b4a7b5e, SUC_lexicon_tags_106 },
    { 0x70a7b5e, SUC_lexicon_tags_68 },
    { 0x184a7b60, SUC_lexicon_tags_77 },
    { 0x4ea7b61, SUC_lexicon_tags_7 },
    { 0x94ca7b61, SUC_lexicon_tags_55 },
    { 0x912a7b63, SUC_lexicon_tags_32 },
    { 0x730a7b5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8a7b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3baa7b6a, SUC_lexicon_tags_8 },
    { 0xad6a7b6b, SUC_lexicon_tags_17 },
    { 0xc7ea7b6c, SUC_lexicon_tags_7 },
    { 0x23aa7b6d, SUC_lexicon_tags_37 },
    { 0xb3ea7b6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1aa7b70, SUC_lexicon_tags_25 },
    { 0xa8ca7b71, SUC_lexicon_tags_1 },
    { 0x10aa7b72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4ea7b74, SUC_lexicon_tags_2 },
    { 0x6e6a7b74, SUC_lexicon_tags_12 },
    { 0x184a7b74, SUC_lexicon_tags_25 },
    { 0xb4a7b77, SUC_lexicon_tags_15 },
    { 0x698a7b78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcea7b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6a7b7f, SUC_lexicon_tags_43 },
    { 0xd38a7b80, SUC_lexicon_tags_8 },
    { 0x19ca7b81, SUC_lexicon_tags_27 },
    { 0xd36a7b80, SUC_lexicon_tags_17 },
    { 0xd5aa7b83, SUC_lexicon_tags_25 },
    { 0xa4ea7b84, SUC_lexicon_tags_27 },
    { 0xcc8a7b85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fea7b87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcca7b8b, SUC_lexicon_tags_68 },
    { 0x8fca7b8c, SUC_lexicon_tags_37 },
    { 0xc06a7b8c, SUC_lexicon_tags_11 },
    { 0x5e2a7b8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x472a7b90, SUC_lexicon_tags_17 },
    { 0x8e2a7b91, SUC_lexicon_tags_37 },
    { 0xaa0a7b92, SUC_lexicon_tags_23 },
    { 0x5bea7b92, SUC_lexicon_tags_16 },
    { 0x55aa7b94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xecaa7b96, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2a7b99, SUC_lexicon_tags_25 },
    { 0x7bca7b9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x20a7b9c, SUC_lexicon_tags_8 },
    { 0xba2a7b9d, SUC_lexicon_tags_23 },
    { 0xf04a7b9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a7ba2, SUC_lexicon_tags_14 },
    { 0x6fca7ba3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ca7ba6, SUC_lexicon_tags_7 },
    { 0x62aa7ba7, SUC_lexicon_tags_16 },
    { 0xc18a7ba8, SUC_lexicon_tags_19 },
    { 0x790a7ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd0a7bab, SUC_lexicon_tags_2 },
    { 0x96ca7bab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770a7bb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a6a7bb9, SUC_lexicon_tags_23 },
    { 0x27aa7bba, SUC_lexicon_tags_37 },
    { 0xb46a7bb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ea7bbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324a7bc6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xc88a7bc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4a7bcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114a7bd0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x692a7bd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x694a7bd4, SUC_lexicon_tags_7 },
    { 0x580a7bd5, SUC_lexicon_tags_30 },
    { 0x6aa7bd6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefea7bdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460a7be1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d6a7be3, SUC_lexicon_tags_25 },
    { 0x172a7be4, SUC_lexicon_tags_25 },
    { 0x896a7be3, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956a7bef, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ea7bf2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc12a7bf4, SUC_lexicon_tags_87 },
    { 0xbaea7bf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9aa7bf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996a7bfb, SUC_lexicon_tags_37 },
    { 0x1d4a7bfc, SUC_lexicon_tags_25 },
    { 0x790a7bfd, SUC_lexicon_tags_11 },
    { 0x15ea7bfe, SUC_lexicon_tags_2 },
    { 0xbaaa7bff, SUC_lexicon_tags_17 },
    { 0x7caa7bfc, SUC_lexicon_tags_25 },
    { 0xad6a7c01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x57aa7c04, SUC_lexicon_tags_15 },
    { 0x3cea7c04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21aa7c08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x212a7c0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3dea7c0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aa7c12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4ea7c14, SUC_lexicon_tags_29 },
    { 0x674a7c15, SUC_lexicon_tags_17 },
    { 0xbe2a7c14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b6a7c18, SUC_lexicon_tags_15 },
    { 0x6eea7c18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254a7c1d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fca7c24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814a7c28, SUC_lexicon_tags_25 },
    { 0xeaaa7c28, SUC_lexicon_tags_49 },
    { 0x1f2a7c2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8a7c2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x848a7c30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x464a7c32, SUC_lexicon_tags_25 },
    { 0x7bea7c32, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90a7c36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b2a7c38, SUC_lexicon_tags_12 },
    { 0x138a7c39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8a7c3e, SUC_lexicon_tags_15 },
    { 0x6aaa7c3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc48a7c41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x764a7c43, SUC_lexicon_tags_15 },
    { 0x2aa7c44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fca7c46, SUC_lexicon_tags_8 },
    { 0x868a7c47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8a7c4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dca7c4e, SUC_lexicon_tags_11 },
    { 0x8dca7c4f, SUC_lexicon_tags_15 },
    { 0x6b4a7c50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ca7c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24a7c5e, SUC_lexicon_tags_12 },
    { 0xb56a7c5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78a7c62, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x986a7c64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5aa7c66, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2a7c69, SUC_lexicon_tags_15 },
    { 0x868a7c6a, SUC_lexicon_tags_19 },
    { 0x7b6a7c6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x290a7c6d, SUC_lexicon_tags_168 },
    { 0xf6aa7c6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x900a7c70, SUC_lexicon_tags_37 },
    { 0x72aa7c71, SUC_lexicon_tags_2 },
    { 0xaea7c72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6a7c7a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x168a7c7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaea7c7f, SUC_lexicon_tags_1 },
    { 0x20a7c80, SUC_lexicon_tags_17 },
    { 0xbc2a7c81, SUC_lexicon_tags_16 },
    { 0x6c6a7c82, SUC_lexicon_tags_29 },
    { 0x6a2a7c80, SUC_lexicon_tags_25 },
    { 0x810a7c84, SUC_lexicon_tags_48 },
    { 0x254a7c85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2f6a7c87, SUC_lexicon_tags_15 },
    { 0xad2a7c88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aa7c8d, SUC_lexicon_tags_25 },
    { 0x71ca7c8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x330a7c90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83aa7c92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cea7c94, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x842a7c96, SUC_lexicon_tags_37 },
    { 0x588a7c97, SUC_lexicon_tags_17 },
    { 0x53aa7c97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x218a7c9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70aa7c9d, SUC_lexicon_tags_27 },
    { 0x74ea7c9e, SUC_lexicon_tags_19 },
    { 0x800a7c9f, SUC_lexicon_tags_37 },
    { 0x398a7ca0, SUC_lexicon_tags_17 },
    { 0x63aa7ca1, SUC_lexicon_tags_4 },
    { 0xf32a7ca2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x350a7ca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86a7ca7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50a7cb2, SUC_lexicon_tags_61 },
    { 0x256a7cb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ea7cba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1f0a7cbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcbaa7cbe, SUC_lexicon_tags_37 },
    { 0x592a7cbe, SUC_lexicon_tags_12 },
    { 0x88ea7cbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4a7cc7, SUC_lexicon_tags_17 },
    { 0x510a7cc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d6a7cca, SUC_lexicon_tags_4 },
    { 0x952a7cca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218a7cd1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80a7cd6, SUC_lexicon_tags_17 },
    { 0x42ea7cd7, SUC_lexicon_tags_61 },
    { 0xe0aa7cd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eaa7cda, SUC_lexicon_tags_37 },
    { 0xd0a7cdb, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x842a7cdd, SUC_lexicon_tags_56 },
    { 0xc0aa7cde, SUC_lexicon_tags_17 },
    { 0x3ea7cde, SUC_lexicon_tags_0 },
    { 0x374a7cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a7ce7, SUC_lexicon_tags_0 },
    { 0x5c6a7ce8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430a7cee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x93aa7cf0, SUC_lexicon_tags_37 },
    { 0xa5ea7cf1, SUC_lexicon_tags_87 },
    { 0x846a7cf1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca7cfa, SUC_lexicon_tags_65 },
    { 0xf0aa7cfb, SUC_lexicon_tags_37 },
    { 0x95ca7cfb, SUC_lexicon_tags_39 },
    { 0x88ea7cfa, SUC_lexicon_tags_35 },
    { 0x1f8a7cfe, SUC_lexicon_tags_25 },
    { 0x444a7cfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58a7d03, SUC_lexicon_tags_43 },
    { 0x758a7d04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80ea7d06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2a7d09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d0a7d0b, SUC_lexicon_tags_65 },
    { 0x19aa7d0b, SUC_lexicon_tags_26 },
    { 0xf86a7d0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46a7d19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68a7d1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x95aa7d20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0a7d24, SUC_lexicon_tags_17 },
    { 0xdb8a7d25, SUC_lexicon_tags_68 },
    { 0x716a7d26, SUC_lexicon_tags_27 },
    { 0xbd8a7d27, SUC_lexicon_tags_19 },
    { 0x962a7d27, SUC_lexicon_tags_28 },
    { 0x16ca7d28, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ca7d2e, SUC_lexicon_tags_26 },
    { 0xd92a7d2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2f8a7d31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38aa7d33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aa7d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0a7d39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc0ca7d3b, SUC_lexicon_tags_39 },
    { 0x53ca7d3c, SUC_lexicon_tags_7 },
    { 0x3cea7d3d, SUC_lexicon_tags_2 },
    { 0x912a7d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8cca7d40, SUC_lexicon_tags_37 },
    { 0x390a7d40, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x706a7d43, SUC_lexicon_tags_11 },
    { 0x22a7d43, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8a7d48, SUC_lexicon_tags_27 },
    { 0xc74a7d49, SUC_lexicon_tags_32 },
    { 0x152a7d48, SUC_lexicon_tags_12 },
    { 0x2cea7d4b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a7d4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ca7d57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5f2a7d59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956a7d5e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x380a7d60, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf98a7d62, SUC_lexicon_tags_12 },
    { 0xed8a7d62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2a7d68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7aa7d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x694a7d6d, SUC_lexicon_tags_0 },
    { 0x27aa7d6d, SUC_lexicon_tags_31 },
    { 0x506a7d6f, SUC_lexicon_tags_29 },
    { 0xedca7d6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ea7d76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61ea7d78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b6a7d7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2aa7d7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cea7d7f, SUC_lexicon_tags_11 },
    { 0x576a7d7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfca7d82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4a7d87, SUC_lexicon_tags_605 },
    { 0x31ca7d88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c0a7d8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe66a7d8c, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6a7d90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4aa7d92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56a7d9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50a7d9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ea7da3, SUC_lexicon_tags_15 },
    { 0xed4a7da3, SUC_lexicon_tags_16 },
    { 0x5ca7da5, SUC_lexicon_tags_15 },
    { 0x94ea7da5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdc6a7da8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ea7dac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa7db1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4a7dbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214a7dc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154a7dc7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4a7dd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66a7dde, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x374a7de0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x46aa7de2, SUC_lexicon_tags_38 },
    { 0x92aa7de3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x968a7de6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34a7de9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x700a7deb, SUC_lexicon_tags_17 },
    { 0xeaea7dec, SUC_lexicon_tags_47 },
    { 0x634a7dec, SUC_lexicon_tags_30 },
    { 0x982a7dee, SUC_lexicon_tags_19 },
    { 0x5eca7def, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd88a7df1, SUC_lexicon_tags_38 },
    { 0xe90a7df2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294a7df7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2dea7df9, SUC_lexicon_tags_17 },
    { 0xe84a7dfa, SUC_lexicon_tags_12 },
    { 0xe2aa7dfb, SUC_lexicon_tags_2 },
    { 0xe3ea7dfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7ca7dfe, SUC_lexicon_tags_4 },
    { 0xabea7dff, SUC_lexicon_tags_17 },
    { 0xb0ea7dfe, SUC_lexicon_tags_17 },
    { 0xde4a7e00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0a7e05, SUC_lexicon_tags_12 },
    { 0x6faa7e06, SUC_lexicon_tags_37 },
    { 0x978a7e06, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6a7e0b, SUC_lexicon_tags_3 },
    { 0x2bea7e0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4a7e12, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2c2a7e14, SUC_lexicon_tags_252 },
    { 0xb38a7e15, SUC_lexicon_tags_0 },
    { 0xaaa7e15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6a7e1c, SUC_lexicon_tags_6 },
    { 0xd1aa7e1d, SUC_lexicon_tags_2 },
    { 0x216a7e1e, SUC_lexicon_tags_2 },
    { 0xe02a7e1f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdca7e22, SUC_lexicon_tags_14 },
    { 0x1f8a7e23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2a7e2a, SUC_lexicon_tags_29 },
    { 0x130a7e2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7aa7e2e, SUC_lexicon_tags_43 },
    { 0x564a7e2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aa7e33, SUC_lexicon_tags_25 },
    { 0x63aa7e33, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ca7e37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cca7e3a, SUC_lexicon_tags_12 },
    { 0x12aa7e3b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2a7e3f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb02a7e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ca7e48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aca7e4d, SUC_lexicon_tags_17 },
    { 0x222a7e4e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52a7e52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa28a7e54, SUC_lexicon_tags_68 },
    { 0x68ca7e55, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fca7e58, SUC_lexicon_tags_13 },
    { 0x3fca7e59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ca7e5b, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2a7e5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf2a7e60, SUC_lexicon_tags_7 },
    { 0xc8ca7e60, SUC_lexicon_tags_25 },
    { 0xc3aa7e60, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8b6a7e64, SUC_lexicon_tags_15 },
    { 0x7b4a7e65, SUC_lexicon_tags_12 },
    { 0xbcea7e65, SUC_lexicon_tags_19 },
    { 0xd5ea7e67, SUC_lexicon_tags_63 },
    { 0x428a7e67, SUC_lexicon_tags_15 },
    { 0x50ca7e69, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3faa7e6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762a7e74, SUC_lexicon_tags_28 },
    { 0x9c8a7e75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x512a7e77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ca7e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ea7e84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ea7e88, SUC_lexicon_tags_47 },
    { 0xeaea7e89, SUC_lexicon_tags_1 },
    { 0x9b2a7e8a, SUC_lexicon_tags_25 },
    { 0x4daa7e8b, SUC_lexicon_tags_15 },
    { 0xf04a7e8c, SUC_lexicon_tags_46 },
    { 0x90ca7e8d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0a7e90, SUC_lexicon_tags_28 },
    { 0xf08a7e91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36a7e97, SUC_lexicon_tags_2 },
    { 0x714a7e98, SUC_lexicon_tags_11 },
    { 0x42a7e98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ea7e9c, SUC_lexicon_tags_60 },
    { 0xd28a7e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120a7ea4, SUC_lexicon_tags_27 },
    { 0x36ca7ea5, SUC_lexicon_tags_27 },
    { 0x7aa7ea6, SUC_lexicon_tags_12 },
    { 0x35ea7ea4, SUC_lexicon_tags_141 },
    { 0x4b2a7ea6, SUC_lexicon_tags_17 },
    { 0xbfaa7ea9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9daa7eab, SUC_lexicon_tags_2 },
    { 0xeeca7eab, SUC_lexicon_tags_6 },
    { 0x320a7ead, SUC_lexicon_tags_7 },
    { 0x77ea7ead, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6a7eb3, SUC_lexicon_tags_10 },
    { 0x706a7eb4, SUC_lexicon_tags_17 },
    { 0xe7ca7eb5, SUC_lexicon_tags_47 },
    { 0xcb2a7eb5, SUC_lexicon_tags_1 },
    { 0x662a7eb7, SUC_lexicon_tags_31 },
    { 0xa2a7eb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcaa7eba, SUC_lexicon_tags_47 },
    { 0xc0aa7ebb, SUC_lexicon_tags_7 },
    { 0xfd2a7ebc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x67aa7ebf, SUC_lexicon_tags_7 },
    { 0xa54a7ec0, SUC_lexicon_tags_16 },
    { 0x4fca7ec0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0a7ec5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x102a7ec8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fca7ecb, SUC_lexicon_tags_46 },
    { 0xb0aa7ecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740a7ed1, SUC_lexicon_tags_1 },
    { 0xe3aa7ed2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x864a7ed5, SUC_lexicon_tags_12 },
    { 0x71aa7ed5, SUC_lexicon_tags_0 },
    { 0x194a7ed6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72a7ede, SUC_lexicon_tags_7 },
    { 0x45ca7ede, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaea7ee4, SUC_lexicon_tags_57 },
    { 0x892a7ee4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb40a7ee7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f8a7ee9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x784a7eec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x278a7eee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ea7ef2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44a7ef5, SUC_lexicon_tags_16 },
    { 0x1b2a7ef6, SUC_lexicon_tags_25 },
    { 0x12ea7ef6, SUC_lexicon_tags_37 },
    { 0x8aaa7ef8, SUC_lexicon_tags_46 },
    { 0xb4ca7ef7, SUC_lexicon_tags_11 },
    { 0x5dea7ef9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcbea7efc, SUC_lexicon_tags_25 },
    { 0xfaa7efd, SUC_lexicon_tags_70 },
    { 0xe50a7efc, SUC_lexicon_tags_17 },
    { 0xf8ca7eff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ca7f02, SUC_lexicon_tags_37 },
    { 0x30ea7f02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ea7f07, SUC_lexicon_tags_12 },
    { 0x288a7f08, SUC_lexicon_tags_15 },
    { 0xdeca7f09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x98ea7f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x206a7f0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x880a7f10, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xea4a7f12, SUC_lexicon_tags_1 },
    { 0x8bea7f13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31ea7f15, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936a7f1d, SUC_lexicon_tags_15 },
    { 0x7e4a7f1d, SUC_lexicon_tags_11 },
    { 0x8bea7f1d, SUC_lexicon_tags_26 },
    { 0xbdca7f20, SUC_lexicon_tags_28 },
    { 0x5f4a7f1e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9fca7f23, SUC_lexicon_tags_13 },
    { 0x6daa7f23, SUC_lexicon_tags_10 },
    { 0x774a7f25, SUC_lexicon_tags_2 },
    { 0xfb0a7f25, SUC_lexicon_tags_27 },
    { 0x68ca7f27, SUC_lexicon_tags_25 },
    { 0xe32a7f28, SUC_lexicon_tags_55 },
    { 0x866a7f26, SUC_lexicon_tags_1 },
    { 0x262a7f25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eca7f32, SUC_lexicon_tags_39 },
    { 0x8b6a7f33, SUC_lexicon_tags_15 },
    { 0x598a7f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2a7f37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76aa7f3b, SUC_lexicon_tags_9 },
    { 0x676a7f3c, SUC_lexicon_tags_25 },
    { 0xe9aa7f3d, SUC_lexicon_tags_59 },
    { 0xabaa7f3e, SUC_lexicon_tags_22 },
    { 0xc56a7f3c, SUC_lexicon_tags_1 },
    { 0xe02a7f40, SUC_lexicon_tags_37 },
    { 0xfaca7f3e, SUC_lexicon_tags_2 },
    { 0x7b8a7f3e, SUC_lexicon_tags_40 },
    { 0x17ca7f43, SUC_lexicon_tags_23 },
    { 0xbd0a7f3b, SUC_lexicon_tags_49 },
    { 0xf80a7f45, SUC_lexicon_tags_33 },
    { 0x320a7f46, SUC_lexicon_tags_61 },
    { 0x42ea7f46, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aa7f4b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x68aa7f4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81aa7f50, SUC_lexicon_tags_17 },
    { 0x400a7f51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52a7f57, SUC_lexicon_tags_26 },
    { 0xdbaa7f57, SUC_lexicon_tags_0 },
    { 0xa54a7f57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ca7f5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6a7f60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x730a7f63, SUC_lexicon_tags_60 },
    { 0xf38a7f63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32a7f67, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374a7f6b, SUC_lexicon_tags_15 },
    { 0xfb8a7f6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebea7f6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0a7f72, SUC_lexicon_tags_7 },
    { 0x3f0a7f73, SUC_lexicon_tags_57 },
    { 0xae4a7f74, SUC_lexicon_tags_60 },
    { 0xf28a7f75, SUC_lexicon_tags_1651 },
    { 0, NULL },
    { 0x7d0a7f77, SUC_lexicon_tags_15 },
    { 0x6f0a7f78, SUC_lexicon_tags_4 },
    { 0xa32a7f79, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb0aa7f7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d0a7f7d, SUC_lexicon_tags_30 },
    { 0xf16a7f7d, SUC_lexicon_tags_60 },
    { 0xa6aa7f7f, SUC_lexicon_tags_0 },
    { 0xf7ca7f80, SUC_lexicon_tags_2 },
    { 0x4e2a7f81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ca7f84, SUC_lexicon_tags_28 },
    { 0x7caa7f85, SUC_lexicon_tags_12 },
    { 0xee6a7f85, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4dca7f88, SUC_lexicon_tags_1 },
    { 0x618a7f88, SUC_lexicon_tags_15 },
    { 0x140a7f89, SUC_lexicon_tags_7 },
    { 0xec4a7f8b, SUC_lexicon_tags_16 },
    { 0xb82a7f8c, SUC_lexicon_tags_8 },
    { 0x1bca7f8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e0a7f8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb00a7f91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe80a7f93, SUC_lexicon_tags_15 },
    { 0x6c0a7f94, SUC_lexicon_tags_31 },
    { 0x5a8a7f95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbeca7f97, SUC_lexicon_tags_45 },
    { 0xdeaa7f98, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aa7f9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x180a7f9f, SUC_lexicon_tags_17 },
    { 0xa30a7fa0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3eaa7fa2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710a7fa6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50a7fad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ea7fb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0a7fb3, SUC_lexicon_tags_25 },
    { 0x272a7fb3, SUC_lexicon_tags_15 },
    { 0xa26a7fb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x536a7fb7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbc0a7fb9, SUC_lexicon_tags_25 },
    { 0x7aaa7fb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56ca7fbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1ea7fbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92a7fc3, SUC_lexicon_tags_35 },
    { 0x9a4a7fc3, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116a7fc9, SUC_lexicon_tags_27 },
    { 0xf5ea7fc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eca7fd1, SUC_lexicon_tags_23 },
    { 0xb0ca7fd2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa7fd6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe3ea7fd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ea7fe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2a7fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7aa7fe5, SUC_lexicon_tags_0 },
    { 0xfd0a7fe5, SUC_lexicon_tags_12 },
    { 0xe52a7fe5, SUC_lexicon_tags_0 },
    { 0xc7ca7fe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbca7fed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368a7ff1, SUC_lexicon_tags_60 },
    { 0x326a7ff2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x192a7ff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a4a7ff6, SUC_lexicon_tags_27 },
    { 0x8a7ff7, SUC_lexicon_tags_46 },
    { 0xf38a7ff8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ea7ffc, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0a7fff, SUC_lexicon_tags_15 },
    { 0x570a8000, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faa8005, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc60a8007, SUC_lexicon_tags_3 },
    { 0xa6ea8008, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38a800c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fea8012, SUC_lexicon_tags_275 },
    { 0x7a2a8013, SUC_lexicon_tags_17 },
    { 0x294a8014, SUC_lexicon_tags_38 },
    { 0xba2a8012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa801c, SUC_lexicon_tags_84 },
    { 0xf04a801d, SUC_lexicon_tags_1 },
    { 0x76ea801e, SUC_lexicon_tags_11 },
    { 0xe2a801f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204a8024, SUC_lexicon_tags_37 },
    { 0x50a8025, SUC_lexicon_tags_12 },
    { 0x5faa8026, SUC_lexicon_tags_32 },
    { 0x7daa8024, SUC_lexicon_tags_37 },
    { 0x644a8028, SUC_lexicon_tags_15 },
    { 0x3e8a8029, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ca802c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaa803a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2a8040, SUC_lexicon_tags_5 },
    { 0xd90a8040, SUC_lexicon_tags_49 },
    { 0xc20a8042, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bea8045, SUC_lexicon_tags_25 },
    { 0xbeea8045, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x610a8049, SUC_lexicon_tags_15 },
    { 0xcfea804a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ca804d, SUC_lexicon_tags_2 },
    { 0xb3ea804e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbea8053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4a8057, SUC_lexicon_tags_43 },
    { 0x86aa8058, SUC_lexicon_tags_15 },
    { 0xa00a8058, SUC_lexicon_tags_14 },
    { 0x8cea8059, SUC_lexicon_tags_11 },
    { 0x27ea8058, SUC_lexicon_tags_37 },
    { 0x8b2a8057, SUC_lexicon_tags_37 },
    { 0x356a805d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea8061, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d6a8063, SUC_lexicon_tags_3 },
    { 0x92a8063, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0a8067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0a806a, SUC_lexicon_tags_37 },
    { 0x974a806a, SUC_lexicon_tags_0 },
    { 0x87ca806b, SUC_lexicon_tags_11 },
    { 0xbfaa806d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02a8073, SUC_lexicon_tags_235 },
    { 0xf4ca8074, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5ca8076, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292a8082, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9aca8084, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ca808f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8a8093, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ca8096, SUC_lexicon_tags_30 },
    { 0x29ea8097, SUC_lexicon_tags_62 },
    { 0x452a8098, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaa809c, SUC_lexicon_tags_49 },
    { 0xaea809d, SUC_lexicon_tags_37 },
    { 0x796a809c, SUC_lexicon_tags_11 },
    { 0xaeca809f, SUC_lexicon_tags_25 },
    { 0x5eaa809f, SUC_lexicon_tags_32 },
    { 0xa76a809f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4a80a5, SUC_lexicon_tags_7 },
    { 0xde8a80a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x648a80a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20a80ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x90aa80af, SUC_lexicon_tags_27 },
    { 0x272a80b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x808a80b2, SUC_lexicon_tags_88 },
    { 0xd6a80b3, SUC_lexicon_tags_15 },
    { 0x99ca80b4, SUC_lexicon_tags_29 },
    { 0xb06a80b5, SUC_lexicon_tags_15 },
    { 0xaaea80b6, SUC_lexicon_tags_14 },
    { 0x6cea80b7, SUC_lexicon_tags_12 },
    { 0x1b2a80b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ca80bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ea80c3, SUC_lexicon_tags_21 },
    { 0x142a80c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4a80c7, SUC_lexicon_tags_30 },
    { 0xdd6a80c8, SUC_lexicon_tags_25 },
    { 0x2aca80c9, SUC_lexicon_tags_47 },
    { 0xe6aa80ca, SUC_lexicon_tags_33 },
    { 0x61aa80ca, SUC_lexicon_tags_27 },
    { 0x628a80cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21ea80ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ca80d4, SUC_lexicon_tags_63 },
    { 0x44ca80d5, SUC_lexicon_tags_31 },
    { 0x664a80d6, SUC_lexicon_tags_11 },
    { 0xc6ca80d4, SUC_lexicon_tags_423 },
    { 0xea4a80d8, SUC_lexicon_tags_9 },
    { 0xda8a80d5, SUC_lexicon_tags_37 },
    { 0xca4a80d4, SUC_lexicon_tags_24 },
    { 0x494a80db, SUC_lexicon_tags_17 },
    { 0xd0ca80da, SUC_lexicon_tags_1643 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0a80e5, SUC_lexicon_tags_25 },
    { 0x8d2a80e6, SUC_lexicon_tags_12 },
    { 0xe54a80e5, SUC_lexicon_tags_28 },
    { 0x94ea80e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2a80ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x10ca80ee, SUC_lexicon_tags_23 },
    { 0x96aa80ef, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xfd6a80f1, SUC_lexicon_tags_25 },
    { 0x992a80f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xad8a80f4, SUC_lexicon_tags_6 },
    { 0x1bca80f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0a80fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x594a80fd, SUC_lexicon_tags_26 },
    { 0x708a80fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf7ca8100, SUC_lexicon_tags_37 },
    { 0x854a8100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24a8104, SUC_lexicon_tags_32 },
    { 0x9a4a8105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48a810a, SUC_lexicon_tags_8 },
    { 0x6b6a810a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb76a810d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaa8110, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06a8113, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x412a8116, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a4a8118, SUC_lexicon_tags_15 },
    { 0x47ca8118, SUC_lexicon_tags_25 },
    { 0x47ea811a, SUC_lexicon_tags_1 },
    { 0xf02a811b, SUC_lexicon_tags_37 },
    { 0xc22a811b, SUC_lexicon_tags_7 },
    { 0x1aca811b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x696a811f, SUC_lexicon_tags_11 },
    { 0x3dca8120, SUC_lexicon_tags_26 },
    { 0x1a6a8121, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x430a8124, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x448a8126, SUC_lexicon_tags_37 },
    { 0xc94a8127, SUC_lexicon_tags_32 },
    { 0x642a8127, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x786a812a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x91aa812d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b6a812f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaa8134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x232a8136, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x526a8138, SUC_lexicon_tags_38 },
    { 0x33ca8139, SUC_lexicon_tags_43 },
    { 0xa0ca813a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8a813d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544a8142, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4aea8144, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874a814e, SUC_lexicon_tags_27 },
    { 0x734a814f, SUC_lexicon_tags_0 },
    { 0x98ca814f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0aa8154, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x504a8156, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214a815f, SUC_lexicon_tags_15 },
    { 0x89aa815f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4a8164, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e8a8166, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904a816b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ca8171, SUC_lexicon_tags_17 },
    { 0xda4a8172, SUC_lexicon_tags_1 },
    { 0x7b4a8173, SUC_lexicon_tags_38 },
    { 0x758a8174, SUC_lexicon_tags_32 },
    { 0x5b8a8175, SUC_lexicon_tags_11 },
    { 0x31aa8176, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55aa8178, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x692a817a, SUC_lexicon_tags_12 },
    { 0x336a817b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x676a817d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902a8181, SUC_lexicon_tags_21 },
    { 0xf7ea8182, SUC_lexicon_tags_14 },
    { 0x9a6a8182, SUC_lexicon_tags_15 },
    { 0x9cca8184, SUC_lexicon_tags_32 },
    { 0x822a8182, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ca8188, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x556a818a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2a818f, SUC_lexicon_tags_1 },
    { 0x95ca818f, SUC_lexicon_tags_63 },
    { 0xe74a8191, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0a8194, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eea81a4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x930a81a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812a81ab, SUC_lexicon_tags_3 },
    { 0xa81ac, SUC_lexicon_tags_15 },
    { 0x460a81ad, SUC_lexicon_tags_1 },
    { 0x8cea81ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x810a81b0, SUC_lexicon_tags_25 },
    { 0x998a81b0, SUC_lexicon_tags_7 },
    { 0xbd2a81b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172a81ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8a81bf, SUC_lexicon_tags_7 },
    { 0x16ca81bf, SUC_lexicon_tags_2 },
    { 0xd8ea81c1, SUC_lexicon_tags_15 },
    { 0x442a81bf, SUC_lexicon_tags_55 },
    { 0xa30a81c3, SUC_lexicon_tags_14 },
    { 0x832a81c4, SUC_lexicon_tags_0 },
    { 0xbf4a81c5, SUC_lexicon_tags_37 },
    { 0x3cea81c4, SUC_lexicon_tags_2 },
    { 0x54a81c7, SUC_lexicon_tags_8 },
    { 0x676a81c8, SUC_lexicon_tags_7 },
    { 0xbc4a81c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250a81cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x418a81cf, SUC_lexicon_tags_11 },
    { 0x8b6a81cf, SUC_lexicon_tags_15 },
    { 0x3f2a81d1, SUC_lexicon_tags_2 },
    { 0x57aa81d2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12a81d6, SUC_lexicon_tags_27 },
    { 0x27ea81d7, SUC_lexicon_tags_17 },
    { 0xdcea81d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x48a81da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2a81df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6a81e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf0ca81e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8a81eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ea81ee, SUC_lexicon_tags_37 },
    { 0x25ca81ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78ca81f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ea81f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aa81f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3ea81fb, SUC_lexicon_tags_7 },
    { 0xd44a81fc, SUC_lexicon_tags_12 },
    { 0x258a81fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ea8200, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x580a8202, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc10a8204, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x392a8207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672a820b, SUC_lexicon_tags_35 },
    { 0x8d0a820c, SUC_lexicon_tags_37 },
    { 0x892a820c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x232a820f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf4a8211, SUC_lexicon_tags_7 },
    { 0xcdea8212, SUC_lexicon_tags_37 },
    { 0x35ea8211, SUC_lexicon_tags_56 },
    { 0x17ca8212, SUC_lexicon_tags_49 },
    { 0xb94a8213, SUC_lexicon_tags_6 },
    { 0x270a8216, SUC_lexicon_tags_12 },
    { 0xa66a8217, SUC_lexicon_tags_11 },
    { 0xf62a8218, SUC_lexicon_tags_23 },
    { 0xbf8a8219, SUC_lexicon_tags_0 },
    { 0x350a8217, SUC_lexicon_tags_37 },
    { 0x6aca821b, SUC_lexicon_tags_17 },
    { 0x8aea821b, SUC_lexicon_tags_7 },
    { 0x64aa8214, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9faa8220, SUC_lexicon_tags_28 },
    { 0xadea8221, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6a8224, SUC_lexicon_tags_87 },
    { 0xf68a8225, SUC_lexicon_tags_37 },
    { 0x618a8226, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02a822b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74a822e, SUC_lexicon_tags_8 },
    { 0x102a822f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ea8233, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x394a8236, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80a823c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe96a823e, SUC_lexicon_tags_45 },
    { 0x4c4a823e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ca8250, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b2a8252, SUC_lexicon_tags_19 },
    { 0x3f6a8253, SUC_lexicon_tags_2 },
    { 0xfe4a8252, SUC_lexicon_tags_47 },
    { 0x786a8253, SUC_lexicon_tags_25 },
    { 0xf78a8256, SUC_lexicon_tags_31 },
    { 0xb9ca8256, SUC_lexicon_tags_26 },
    { 0xe22a8257, SUC_lexicon_tags_7 },
    { 0xe1ca8259, SUC_lexicon_tags_136 },
    { 0xfc0a8257, SUC_lexicon_tags_40 },
    { 0xaeca825b, SUC_lexicon_tags_45 },
    { 0x3c4a825a, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xceea825f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ea8263, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x646a8266, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6a826a, SUC_lexicon_tags_7 },
    { 0xa2a826a, SUC_lexicon_tags_15 },
    { 0xff6a826c, SUC_lexicon_tags_65 },
    { 0xa24a826d, SUC_lexicon_tags_37 },
    { 0xe10a826e, SUC_lexicon_tags_0 },
    { 0x37ca826e, SUC_lexicon_tags_15 },
    { 0x874a8270, SUC_lexicon_tags_7 },
    { 0x43ca8271, SUC_lexicon_tags_46 },
    { 0x920a826c, SUC_lexicon_tags_47 },
    { 0xb50a8273, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe78a8275, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4a827f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30a8283, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6a8288, SUC_lexicon_tags_1 },
    { 0x264a8289, SUC_lexicon_tags_10 },
    { 0x93aa828a, SUC_lexicon_tags_45 },
    { 0x3c6a8288, SUC_lexicon_tags_53 },
    { 0xecca828b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb30a828e, SUC_lexicon_tags_43 },
    { 0x6aaa828f, SUC_lexicon_tags_188 },
    { 0xe8a8290, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaa8293, SUC_lexicon_tags_37 },
    { 0xf4aa8294, SUC_lexicon_tags_19 },
    { 0xeaaa8295, SUC_lexicon_tags_0 },
    { 0xa8ea8294, SUC_lexicon_tags_12 },
    { 0xeb4a8297, SUC_lexicon_tags_7 },
    { 0x56a8298, SUC_lexicon_tags_23 },
    { 0xeb4a8293, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4a829c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x274a829f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650a82a3, SUC_lexicon_tags_47 },
    { 0x9eca82a3, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512a82aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6fea82ac, SUC_lexicon_tags_25 },
    { 0x286a82ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6a82b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x306a82ba, SUC_lexicon_tags_1 },
    { 0x8d0a82ba, SUC_lexicon_tags_46 },
    { 0x5f6a82bc, SUC_lexicon_tags_25 },
    { 0x4e2a82bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc3aa82bf, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xde4a82c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3e6a82c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50a82c6, SUC_lexicon_tags_72 },
    { 0x4a2a82c6, SUC_lexicon_tags_11 },
    { 0xd8aa82c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbcaa82ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb30a82cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aa82cf, SUC_lexicon_tags_37 },
    { 0x5c2a82d0, SUC_lexicon_tags_23 },
    { 0x4bea82d0, SUC_lexicon_tags_17 },
    { 0x864a82d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaa82d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26a82da, SUC_lexicon_tags_17 },
    { 0x17ea82db, SUC_lexicon_tags_47 },
    { 0xdeaa82da, SUC_lexicon_tags_4 },
    { 0x5e0a82dd, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x836a82df, SUC_lexicon_tags_27 },
    { 0x390a82e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36a82e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fca82eb, SUC_lexicon_tags_11 },
    { 0x60aa82eb, SUC_lexicon_tags_37 },
    { 0xab0a82ed, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0a82f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74ca82f5, SUC_lexicon_tags_28 },
    { 0xe7ea82f6, SUC_lexicon_tags_2 },
    { 0x7b4a82f5, SUC_lexicon_tags_29 },
    { 0xe1ca82f8, SUC_lexicon_tags_32 },
    { 0x70aa82f8, SUC_lexicon_tags_65 },
    { 0x620a82f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2a82fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb2ca82ff, SUC_lexicon_tags_11 },
    { 0xea8300, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4a8303, SUC_lexicon_tags_25 },
    { 0xeb0a8304, SUC_lexicon_tags_45 },
    { 0xca0a8305, SUC_lexicon_tags_16 },
    { 0xdd0a8305, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1ca8308, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x65ca830a, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x7a6a830c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3aaa830e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x680a8311, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84a8314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ca8317, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4f2a8319, SUC_lexicon_tags_25 },
    { 0x256a831a, SUC_lexicon_tags_26 },
    { 0x8dca831a, SUC_lexicon_tags_4 },
    { 0x5baa831c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ea8321, SUC_lexicon_tags_23 },
    { 0x39ea8322, SUC_lexicon_tags_2 },
    { 0xdb4a8322, SUC_lexicon_tags_151 },
    { 0xae6a8324, SUC_lexicon_tags_31 },
    { 0x76aa8323, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x98ea8327, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46ea8329, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ca832f, SUC_lexicon_tags_14 },
    { 0xb50a8330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc72a8332, SUC_lexicon_tags_15 },
    { 0x7caa8333, SUC_lexicon_tags_1 },
    { 0x4caa8332, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ea8336, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6a833b, SUC_lexicon_tags_1 },
    { 0x3b4a833c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cea8343, SUC_lexicon_tags_15 },
    { 0xd26a8344, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8a834a, SUC_lexicon_tags_45 },
    { 0xa4a834b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4a8350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfca8353, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0a8356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26a8359, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0a8361, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8a8364, SUC_lexicon_tags_15 },
    { 0x626a8364, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb50a8367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aaa836c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x212a836f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x938a8371, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ea8375, SUC_lexicon_tags_2 },
    { 0x7f4a8376, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32a837b, SUC_lexicon_tags_37 },
    { 0x8baa837c, SUC_lexicon_tags_2 },
    { 0xea6a837d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x474a837f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a8383, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2a8386, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2a838a, SUC_lexicon_tags_38 },
    { 0x898a838b, SUC_lexicon_tags_25 },
    { 0x47ea838c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb86a838e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0a8392, SUC_lexicon_tags_65 },
    { 0x7d6a8393, SUC_lexicon_tags_65 },
    { 0xef2a8394, SUC_lexicon_tags_82 },
    { 0xe02a8394, SUC_lexicon_tags_43 },
    { 0x8aaa8394, SUC_lexicon_tags_92 },
    { 0x6c6a8396, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ea839a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0a839d, SUC_lexicon_tags_9 },
    { 0x908a839e, SUC_lexicon_tags_12 },
    { 0x54ca839f, SUC_lexicon_tags_82 },
    { 0xf1ca839e, SUC_lexicon_tags_63 },
    { 0xd08a839e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdea83aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74ca83ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4ea83ae, SUC_lexicon_tags_15 },
    { 0x168a83ae, SUC_lexicon_tags_17 },
    { 0x6e4a83b0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2bca83b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dea83b5, SUC_lexicon_tags_56 },
    { 0x190a83b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a8a83b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4a83bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x246a83c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a83c3, SUC_lexicon_tags_2 },
    { 0x1fca83c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424a83ce, SUC_lexicon_tags_2 },
    { 0x356a83ce, SUC_lexicon_tags_10 },
    { 0x4c8a83d0, SUC_lexicon_tags_79 },
    { 0x1e4a83d1, SUC_lexicon_tags_7 },
    { 0x1f4a83d2, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6a83d7, SUC_lexicon_tags_11 },
    { 0x37ea83d8, SUC_lexicon_tags_31 },
    { 0x44ea83d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x504a83db, SUC_lexicon_tags_7 },
    { 0x1b4a83dc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x28ea83de, SUC_lexicon_tags_31 },
    { 0x406a83df, SUC_lexicon_tags_27 },
    { 0xaa4a83de, SUC_lexicon_tags_46 },
    { 0x1fea83de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26a83e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x320a83e8, SUC_lexicon_tags_2 },
    { 0xdcca83e9, SUC_lexicon_tags_29 },
    { 0xe42a83e9, SUC_lexicon_tags_37 },
    { 0xc5ca83eb, SUC_lexicon_tags_31 },
    { 0x8c0a83ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4a83f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96a83f6, SUC_lexicon_tags_23 },
    { 0x2f0a83f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2a83fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84ca83fc, SUC_lexicon_tags_31 },
    { 0xe22a83fd, SUC_lexicon_tags_53 },
    { 0xbfaa83fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf6ea8400, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910a8406, SUC_lexicon_tags_82 },
    { 0x73ca8406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x438a8409, SUC_lexicon_tags_6 },
    { 0x284a8409, SUC_lexicon_tags_49 },
    { 0xf2ca840b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aa840e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaea8417, SUC_lexicon_tags_37 },
    { 0xba6a8418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8a841b, SUC_lexicon_tags_26 },
    { 0x16ea841c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22a841f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22a8423, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbea8426, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x278a8428, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda8a842a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ca8431, SUC_lexicon_tags_14 },
    { 0xd28a8431, SUC_lexicon_tags_6 },
    { 0x390a8433, SUC_lexicon_tags_2 },
    { 0x408a8433, SUC_lexicon_tags_30 },
    { 0xf4a8435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30a8438, SUC_lexicon_tags_6 },
    { 0x456a8439, SUC_lexicon_tags_25 },
    { 0xdca843a, SUC_lexicon_tags_11 },
    { 0x934a843b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x70a843e, SUC_lexicon_tags_65 },
    { 0x864a843e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dca8441, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734a8447, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55aa844a, SUC_lexicon_tags_25 },
    { 0x6d8a844b, SUC_lexicon_tags_1 },
    { 0xb06a844a, SUC_lexicon_tags_6 },
    { 0x1a8a844d, SUC_lexicon_tags_49 },
    { 0xd9ca844d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x578a8451, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd50a8453, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x712a8455, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3ca8457, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfa0a8459, SUC_lexicon_tags_15 },
    { 0xf98a8459, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40a8460, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x520a8462, SUC_lexicon_tags_359 },
    { 0xefca8463, SUC_lexicon_tags_28 },
    { 0xc6ea8464, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6a8468, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffaa846a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72a846c, SUC_lexicon_tags_802 },
    { 0, NULL },
    { 0x36ea846e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494a8474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x308a8476, SUC_lexicon_tags_17 },
    { 0x3f8a8476, SUC_lexicon_tags_17 },
    { 0xd82a8478, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ea847d, SUC_lexicon_tags_26 },
    { 0x260a847e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ea8484, SUC_lexicon_tags_47 },
    { 0x322a8485, SUC_lexicon_tags_1 },
    { 0x740a8486, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8a848a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348a848e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x40ca8490, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x98ea8492, SUC_lexicon_tags_37 },
    { 0x21ca8493, SUC_lexicon_tags_34 },
    { 0xe30a8494, SUC_lexicon_tags_12 },
    { 0x236a8494, SUC_lexicon_tags_9 },
    { 0x18aa8494, SUC_lexicon_tags_81 },
    { 0xeb2a8497, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x964a8499, SUC_lexicon_tags_11 },
    { 0xe12a8499, SUC_lexicon_tags_19 },
    { 0x7d8a849b, SUC_lexicon_tags_14 },
    { 0x32a849c, SUC_lexicon_tags_30 },
    { 0x586a849d, SUC_lexicon_tags_31 },
    { 0x6d0a849e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0a84a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ea84a4, SUC_lexicon_tags_37 },
    { 0x238a84a5, SUC_lexicon_tags_70 },
    { 0xe1aa84a6, SUC_lexicon_tags_47 },
    { 0x266a84a7, SUC_lexicon_tags_45 },
    { 0x4b2a84a8, SUC_lexicon_tags_15 },
    { 0xa42a84a8, SUC_lexicon_tags_15 },
    { 0xeeaa84a9, SUC_lexicon_tags_6 },
    { 0x990a84ab, SUC_lexicon_tags_37 },
    { 0xe2ca84ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaea84b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1aa84b5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x85aa84b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98aa84bb, SUC_lexicon_tags_13 },
    { 0x17aa84bc, SUC_lexicon_tags_11 },
    { 0xadea84bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2a84c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906a84ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4a84cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286a84d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0a84d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4a84d9, SUC_lexicon_tags_35 },
    { 0x42a84da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322a84de, SUC_lexicon_tags_15 },
    { 0xdd0a84df, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0x76aa84e2, SUC_lexicon_tags_25 },
    { 0x20ca84e3, SUC_lexicon_tags_49 },
    { 0x5caa84e4, SUC_lexicon_tags_87 },
    { 0x81ea84e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6a84e8, SUC_lexicon_tags_12 },
    { 0xbf0a84e9, SUC_lexicon_tags_26 },
    { 0x86ea84ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41aa84ee, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cca84f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ca84f5, SUC_lexicon_tags_37 },
    { 0xf24a84f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcca84fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ca8506, SUC_lexicon_tags_33 },
    { 0x1b8a8507, SUC_lexicon_tags_14 },
    { 0x898a8508, SUC_lexicon_tags_17 },
    { 0x70ca8508, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86a8510, SUC_lexicon_tags_49 },
    { 0x788a8511, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946a8515, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x190a8517, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aa851b, SUC_lexicon_tags_4 },
    { 0x4bca851c, SUC_lexicon_tags_0 },
    { 0xc46a851c, SUC_lexicon_tags_9 },
    { 0xfc8a851d, SUC_lexicon_tags_17 },
    { 0x5c8a851f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ca8522, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ca8526, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee8a8528, SUC_lexicon_tags_37 },
    { 0x10aa8529, SUC_lexicon_tags_11 },
    { 0x860a852a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832a8537, SUC_lexicon_tags_33 },
    { 0xcaaa8537, SUC_lexicon_tags_32 },
    { 0x8aea8538, SUC_lexicon_tags_17 },
    { 0xa4ea8538, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x466a853c, SUC_lexicon_tags_12 },
    { 0x9e2a853d, SUC_lexicon_tags_2 },
    { 0x670a853e, SUC_lexicon_tags_12 },
    { 0x820a853f, SUC_lexicon_tags_23 },
    { 0x56ea853f, SUC_lexicon_tags_37 },
    { 0x20ea8541, SUC_lexicon_tags_47 },
    { 0x76ea8542, SUC_lexicon_tags_10 },
    { 0xfdaa8543, SUC_lexicon_tags_6 },
    { 0x26aa8543, SUC_lexicon_tags_10 },
    { 0x5b6a8545, SUC_lexicon_tags_23 },
    { 0x2bea8546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10a854b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x35ca854d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x474a854f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4a8553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96a8556, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x10ca8558, SUC_lexicon_tags_11 },
    { 0xe42a8558, SUC_lexicon_tags_26 },
    { 0x18aa855a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ea856b, SUC_lexicon_tags_1 },
    { 0xe82a856c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54a8570, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ca857a, SUC_lexicon_tags_12 },
    { 0x788a857b, SUC_lexicon_tags_35 },
    { 0xd7ea857c, SUC_lexicon_tags_9 },
    { 0x38ca857b, SUC_lexicon_tags_151 },
    { 0x1b6a857c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78ea8580, SUC_lexicon_tags_8 },
    { 0x2b6a8581, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd4aa8583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0a8587, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6a858f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbca8592, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baa8597, SUC_lexicon_tags_14 },
    { 0x8b8a8598, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x90a859a, SUC_lexicon_tags_9 },
    { 0x762a859a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dea859f, SUC_lexicon_tags_14 },
    { 0x3a6a859f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x192a85a2, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x642a85a4, SUC_lexicon_tags_43 },
    { 0xbb6a85a5, SUC_lexicon_tags_1 },
    { 0x7d6a85a6, SUC_lexicon_tags_37 },
    { 0x664a85a7, SUC_lexicon_tags_3 },
    { 0x606a85a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x38ca85aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94a85af, SUC_lexicon_tags_65 },
    { 0x3b8a85af, SUC_lexicon_tags_1 },
    { 0x192a85b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2a85b4, SUC_lexicon_tags_27 },
    { 0xb3aa85b4, SUC_lexicon_tags_14 },
    { 0xd78a85b4, SUC_lexicon_tags_15 },
    { 0xaaea85b6, SUC_lexicon_tags_1 },
    { 0xf48a85b8, SUC_lexicon_tags_15 },
    { 0x740a85b9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x512a85bb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aa85be, SUC_lexicon_tags_14 },
    { 0x3c2a85be, SUC_lexicon_tags_7 },
    { 0x650a85bf, SUC_lexicon_tags_158 },
    { 0x7aaa85c1, SUC_lexicon_tags_1240 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aa85ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfd8a85cc, SUC_lexicon_tags_3 },
    { 0xabca85cd, SUC_lexicon_tags_28 },
    { 0x64a85cd, SUC_lexicon_tags_40 },
    { 0xfcaa85cf, SUC_lexicon_tags_26 },
    { 0x172a85d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468a85d6, SUC_lexicon_tags_11 },
    { 0x1a8a85d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe2a85d9, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x420a85db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8a85df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70a85e2, SUC_lexicon_tags_14 },
    { 0x94ca85e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eea85e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfeaa85e8, SUC_lexicon_tags_7 },
    { 0xa06a85e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x524a85eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800a85f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26a85f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x650a85f9, SUC_lexicon_tags_49 },
    { 0x27aa85fa, SUC_lexicon_tags_17 },
    { 0x354a85fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18a85ff, SUC_lexicon_tags_49 },
    { 0xf50a85ff, SUC_lexicon_tags_43 },
    { 0x992a85ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30a8604, SUC_lexicon_tags_25 },
    { 0x288a8605, SUC_lexicon_tags_1 },
    { 0x942a8606, SUC_lexicon_tags_15 },
    { 0x5a8a8607, SUC_lexicon_tags_11 },
    { 0x422a8607, SUC_lexicon_tags_67 },
    { 0xe32a8609, SUC_lexicon_tags_1 },
    { 0x58ca860a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6a8610, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb0a8612, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486a8617, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x79ea8619, SUC_lexicon_tags_1 },
    { 0xc80a861a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288a861e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56a8621, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8a8624, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2a8629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32a862d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6a862f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38a8640, SUC_lexicon_tags_25 },
    { 0x49ea8640, SUC_lexicon_tags_47 },
    { 0x7b2a8641, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54a864c, SUC_lexicon_tags_29 },
    { 0xbb0a864c, SUC_lexicon_tags_17 },
    { 0xe92a864e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf0a8650, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4a8652, SUC_lexicon_tags_1 },
    { 0x694a8653, SUC_lexicon_tags_11 },
    { 0x490a8653, SUC_lexicon_tags_20 },
    { 0x58a8652, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x172a8658, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x556a865b, SUC_lexicon_tags_45 },
    { 0xe2ea865c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb6a865e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2a8660, SUC_lexicon_tags_6 },
    { 0x6a0a8661, SUC_lexicon_tags_25 },
    { 0x568a8662, SUC_lexicon_tags_37 },
    { 0x88a8660, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4a8668, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33aa866d, SUC_lexicon_tags_7 },
    { 0xa7ca866e, SUC_lexicon_tags_63 },
    { 0x278a866f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48a8672, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644a8676, SUC_lexicon_tags_31 },
    { 0x40ca8677, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcf2a8679, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0a867f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68aa868c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x572a868e, SUC_lexicon_tags_60 },
    { 0x7aca868f, SUC_lexicon_tags_0 },
    { 0x528a8690, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3caa8693, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cca8699, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ea869e, SUC_lexicon_tags_49 },
    { 0x432a869f, SUC_lexicon_tags_49 },
    { 0x498a869e, SUC_lexicon_tags_11 },
    { 0x24ca869e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x448a86a3, SUC_lexicon_tags_1 },
    { 0xf3ea86a4, SUC_lexicon_tags_63 },
    { 0xcd0a86a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ca86a9, SUC_lexicon_tags_34 },
    { 0x708a86a9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xca8a86ac, SUC_lexicon_tags_15 },
    { 0x90ca86ad, SUC_lexicon_tags_37 },
    { 0x780a86ad, SUC_lexicon_tags_10 },
    { 0x910a86ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x716a86b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x206a86b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2a86b7, SUC_lexicon_tags_1 },
    { 0x706a86b8, SUC_lexicon_tags_26 },
    { 0x90ea86b8, SUC_lexicon_tags_0 },
    { 0xf62a86ba, SUC_lexicon_tags_12 },
    { 0x632a86ba, SUC_lexicon_tags_47 },
    { 0xd12a86bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2a86c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6dea86c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54a86cc, SUC_lexicon_tags_33 },
    { 0x854a86cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0a86d0, SUC_lexicon_tags_37 },
    { 0xd08a86d1, SUC_lexicon_tags_14 },
    { 0x76ea86d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x97aa86d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ca86de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524a86e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaa86e6, SUC_lexicon_tags_15 },
    { 0x610a86e7, SUC_lexicon_tags_7 },
    { 0xce6a86e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21aa86ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aa86f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ea86f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46a86f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd48a86fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xed4a86fc, SUC_lexicon_tags_37 },
    { 0x1eaa86fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ca8702, SUC_lexicon_tags_45 },
    { 0xb34a8703, SUC_lexicon_tags_23 },
    { 0x872a8703, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ea870a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198a8710, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6a8714, SUC_lexicon_tags_11 },
    { 0x91ea8714, SUC_lexicon_tags_11 },
    { 0x750a8715, SUC_lexicon_tags_25 },
    { 0x6bca8717, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ca871c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4eaa871e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ea8723, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x482a8725, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10a8728, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x810a872a, SUC_lexicon_tags_15 },
    { 0xea872b, SUC_lexicon_tags_11 },
    { 0xe0aa872b, SUC_lexicon_tags_43 },
    { 0x582a872d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7e2a872f, SUC_lexicon_tags_17 },
    { 0x3e8a8730, SUC_lexicon_tags_0 },
    { 0xcaa8730, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8a8737, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eea873c, SUC_lexicon_tags_7 },
    { 0xa1ea873d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0a8743, SUC_lexicon_tags_129 },
    { 0x92a8744, SUC_lexicon_tags_21 },
    { 0xb76a8745, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252a8749, SUC_lexicon_tags_17 },
    { 0x56a8749, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0a874d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa08a874f, SUC_lexicon_tags_12 },
    { 0x39ca874f, SUC_lexicon_tags_14 },
    { 0x3a2a8750, SUC_lexicon_tags_15 },
    { 0xec2a8752, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a875a, SUC_lexicon_tags_19 },
    { 0xdaa875b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b4a875d, SUC_lexicon_tags_2 },
    { 0x8e0a875d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3aa8761, SUC_lexicon_tags_38 },
    { 0xfe0a8762, SUC_lexicon_tags_25 },
    { 0x956a8763, SUC_lexicon_tags_25 },
    { 0x3fca8762, SUC_lexicon_tags_3 },
    { 0xf2ea8765, SUC_lexicon_tags_25 },
    { 0xe88a8765, SUC_lexicon_tags_50 },
    { 0x71ea8763, SUC_lexicon_tags_25 },
    { 0x80a8768, SUC_lexicon_tags_11 },
    { 0xf56a8768, SUC_lexicon_tags_6 },
    { 0x486a876a, SUC_lexicon_tags_30 },
    { 0x25ca876b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ca876e, SUC_lexicon_tags_38 },
    { 0xe78a876f, SUC_lexicon_tags_2 },
    { 0xadaa8770, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768a8775, SUC_lexicon_tags_47 },
    { 0x422a8776, SUC_lexicon_tags_8 },
    { 0x2fca8777, SUC_lexicon_tags_7 },
    { 0x550a8778, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178a877c, SUC_lexicon_tags_11 },
    { 0xdd0a877d, SUC_lexicon_tags_1 },
    { 0x908a877e, SUC_lexicon_tags_39 },
    { 0x3a4a877c, SUC_lexicon_tags_8 },
    { 0xf2aa877f, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4a8783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb16a8785, SUC_lexicon_tags_28 },
    { 0xb96a8786, SUC_lexicon_tags_2 },
    { 0xc1aa8787, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6a878c, SUC_lexicon_tags_25 },
    { 0xec4a878d, SUC_lexicon_tags_11 },
    { 0xcb0a878e, SUC_lexicon_tags_0 },
    { 0x4c2a878f, SUC_lexicon_tags_11 },
    { 0x492a878f, SUC_lexicon_tags_6 },
    { 0xca0a878f, SUC_lexicon_tags_6 },
    { 0x710a878d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ca8796, SUC_lexicon_tags_11 },
    { 0xdf8a8797, SUC_lexicon_tags_37 },
    { 0x36a8798, SUC_lexicon_tags_17 },
    { 0x5f4a8799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222a879e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ea87a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x520a87a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36aa87a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bca87aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80a87ad, SUC_lexicon_tags_25 },
    { 0x3ca87ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dea87b1, SUC_lexicon_tags_17 },
    { 0x79aa87b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a87b5, SUC_lexicon_tags_47 },
    { 0x39ca87b6, SUC_lexicon_tags_59 },
    { 0x91aa87b6, SUC_lexicon_tags_12 },
    { 0x5c4a87b7, SUC_lexicon_tags_17 },
    { 0x1b6a87b9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ea87be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0a87c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xffaa87c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2a87c8, SUC_lexicon_tags_6 },
    { 0x60ea87c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdca87cd, SUC_lexicon_tags_43 },
    { 0xdb0a87cd, SUC_lexicon_tags_8 },
    { 0x1b2a87cd, SUC_lexicon_tags_2 },
    { 0xbcea87d0, SUC_lexicon_tags_212 },
    { 0xd48a87d1, SUC_lexicon_tags_1 },
    { 0x2f8a87d2, SUC_lexicon_tags_17 },
    { 0xb80a87d3, SUC_lexicon_tags_11 },
    { 0x994a87d3, SUC_lexicon_tags_37 },
    { 0x4b0a87d5, SUC_lexicon_tags_37 },
    { 0x8d2a87cd, SUC_lexicon_tags_60 },
    { 0xedea87d0, SUC_lexicon_tags_25 },
    { 0xb0aa87cf, SUC_lexicon_tags_2 },
    { 0xa06a87d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x320a87db, SUC_lexicon_tags_12 },
    { 0x7aa87dc, SUC_lexicon_tags_7 },
    { 0x8cca87dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ea87e1, SUC_lexicon_tags_12 },
    { 0xb60a87e2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x62ca87e4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf72a87e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6a87e9, SUC_lexicon_tags_25 },
    { 0xcbaa87ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512a87f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0ea87f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ea87f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc86a87f7, SUC_lexicon_tags_49 },
    { 0x8dea87f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55ca87fa, SUC_lexicon_tags_14 },
    { 0x96ca87fb, SUC_lexicon_tags_11 },
    { 0xa60a87fb, SUC_lexicon_tags_12 },
    { 0xd60a87fd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46a8809, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704a8810, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x294a8813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6a881a, SUC_lexicon_tags_57 },
    { 0x186a881b, SUC_lexicon_tags_1 },
    { 0x330a881c, SUC_lexicon_tags_37 },
    { 0x6aea881d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x566a881f, SUC_lexicon_tags_57 },
    { 0xd14a8820, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5daa8825, SUC_lexicon_tags_37 },
    { 0x42aa8826, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x292a8829, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60a882e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8b8a8830, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa70a8832, SUC_lexicon_tags_33 },
    { 0x448a8833, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0a8836, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28a8838, SUC_lexicon_tags_47 },
    { 0xcb6a8839, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6a883c, SUC_lexicon_tags_38 },
    { 0xeb2a883c, SUC_lexicon_tags_11 },
    { 0xf26a883c, SUC_lexicon_tags_7 },
    { 0xff2a883f, SUC_lexicon_tags_1 },
    { 0x92ca883e, SUC_lexicon_tags_2 },
    { 0x6c0a883f, SUC_lexicon_tags_2 },
    { 0x8d6a8841, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd14a8844, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeeca8846, SUC_lexicon_tags_91 },
    { 0x602a8847, SUC_lexicon_tags_23 },
    { 0xbb0a8848, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ca884c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe98a884e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606a8859, SUC_lexicon_tags_11 },
    { 0xf7ea8859, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80a8861, SUC_lexicon_tags_12 },
    { 0xdaca8862, SUC_lexicon_tags_4 },
    { 0xe5ca8863, SUC_lexicon_tags_84 },
    { 0x18ea8864, SUC_lexicon_tags_70 },
    { 0xa0ea8864, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeca8868, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ca886e, SUC_lexicon_tags_1512 },
    { 0x2aaa886f, SUC_lexicon_tags_2 },
    { 0xeeaa886f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaa8873, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x46a8875, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x556a8877, SUC_lexicon_tags_1 },
    { 0xd00a8878, SUC_lexicon_tags_31 },
    { 0x486a8879, SUC_lexicon_tags_25 },
    { 0xec4a887a, SUC_lexicon_tags_23 },
    { 0x36ca887b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x976a887e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bea8883, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3caa8885, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x746a8888, SUC_lexicon_tags_7 },
    { 0x97aa8889, SUC_lexicon_tags_12 },
    { 0xeb2a8889, SUC_lexicon_tags_37 },
    { 0xc16a888b, SUC_lexicon_tags_17 },
    { 0x138a888b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe44a888e, SUC_lexicon_tags_19 },
    { 0x884a888f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6a8895, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0a8897, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b0a8899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9baa889b, SUC_lexicon_tags_29 },
    { 0x190a889c, SUC_lexicon_tags_19 },
    { 0xca889c, SUC_lexicon_tags_11 },
    { 0xb2ea889e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4a88a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2a88ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x34aa88af, SUC_lexicon_tags_15 },
    { 0xdd4a88af, SUC_lexicon_tags_74 },
    { 0xb36a88af, SUC_lexicon_tags_31 },
    { 0x64ea88b0, SUC_lexicon_tags_33 },
    { 0xb0a88b3, SUC_lexicon_tags_31 },
    { 0xc8a88b4, SUC_lexicon_tags_0 },
    { 0x19aa88b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52a88bc, SUC_lexicon_tags_15 },
    { 0x666a88bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ca88c2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x260a88c5, SUC_lexicon_tags_14 },
    { 0x3dea88c6, SUC_lexicon_tags_25 },
    { 0xf0ea88c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ea88cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x39ea88cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90a88d3, SUC_lexicon_tags_10 },
    { 0xdaa88d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aa88d7, SUC_lexicon_tags_7 },
    { 0x82ea88d8, SUC_lexicon_tags_8 },
    { 0x9a8a88d9, SUC_lexicon_tags_70 },
    { 0x2bea88d9, SUC_lexicon_tags_1497 },
    { 0x920a88db, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0a88e1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x13aa88e3, SUC_lexicon_tags_45 },
    { 0x168a88e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86ea88e6, SUC_lexicon_tags_31 },
    { 0x9faa88e7, SUC_lexicon_tags_68 },
    { 0xc64a88e8, SUC_lexicon_tags_47 },
    { 0x47ca88e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc38a88eb, SUC_lexicon_tags_10 },
    { 0x652a88ec, SUC_lexicon_tags_20 },
    { 0x8d2a88ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1d4a88ef, SUC_lexicon_tags_37 },
    { 0xec6a88ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daa88fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca88ff, SUC_lexicon_tags_29 },
    { 0x64a88ff, SUC_lexicon_tags_45 },
    { 0xeeea8900, SUC_lexicon_tags_31 },
    { 0x340a8902, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8ea8904, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19aa8907, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1c4a8909, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4a890d, SUC_lexicon_tags_19 },
    { 0x7e4a890e, SUC_lexicon_tags_8 },
    { 0xffea890f, SUC_lexicon_tags_15 },
    { 0x6f0a890f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250a8916, SUC_lexicon_tags_25 },
    { 0xc5ca8917, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aa891e, SUC_lexicon_tags_23 },
    { 0xca2a891f, SUC_lexicon_tags_25 },
    { 0xb66a891e, SUC_lexicon_tags_31 },
    { 0x964a8921, SUC_lexicon_tags_62 },
    { 0x33aa8921, SUC_lexicon_tags_1 },
    { 0x4f6a8923, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ca8927, SUC_lexicon_tags_25 },
    { 0xd56a8928, SUC_lexicon_tags_38 },
    { 0xf36a8927, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27aa892e, SUC_lexicon_tags_11 },
    { 0x446a892e, SUC_lexicon_tags_17 },
    { 0x9c4a8930, SUC_lexicon_tags_7 },
    { 0x6c6a8931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4a8934, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4ca8936, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e2a8938, SUC_lexicon_tags_30 },
    { 0x4b8a8939, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5fea893b, SUC_lexicon_tags_56 },
    { 0xab2a893b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424a8940, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432a8945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e2a8947, SUC_lexicon_tags_35 },
    { 0xdb6a8948, SUC_lexicon_tags_30 },
    { 0xd9ea8949, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6a8955, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x85aa8958, SUC_lexicon_tags_49 },
    { 0xb7ca8958, SUC_lexicon_tags_15 },
    { 0x21ea895a, SUC_lexicon_tags_60 },
    { 0xa8a895b, SUC_lexicon_tags_1 },
    { 0x70ca895b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0a8963, SUC_lexicon_tags_25 },
    { 0xf14a8963, SUC_lexicon_tags_17 },
    { 0xf00a8964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700a896c, SUC_lexicon_tags_7 },
    { 0xc70a896d, SUC_lexicon_tags_33 },
    { 0xca0a896e, SUC_lexicon_tags_31 },
    { 0x63ea896f, SUC_lexicon_tags_18 },
    { 0xa0a8970, SUC_lexicon_tags_172 },
    { 0x4aea8971, SUC_lexicon_tags_45 },
    { 0x178a8972, SUC_lexicon_tags_17 },
    { 0x58a8972, SUC_lexicon_tags_48 },
    { 0xf86a8970, SUC_lexicon_tags_1 },
    { 0xc92a8970, SUC_lexicon_tags_15 },
    { 0xf4ea8976, SUC_lexicon_tags_63 },
    { 0xf42a896d, SUC_lexicon_tags_194 },
    { 0x906a8973, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38a897d, SUC_lexicon_tags_46 },
    { 0x8aa897e, SUC_lexicon_tags_43 },
    { 0x9eea897d, SUC_lexicon_tags_0 },
    { 0x20ea8980, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168a8984, SUC_lexicon_tags_4 },
    { 0x12ca8985, SUC_lexicon_tags_37 },
    { 0x4e0a8985, SUC_lexicon_tags_33 },
    { 0x4d6a8987, SUC_lexicon_tags_29 },
    { 0xfb2a8988, SUC_lexicon_tags_37 },
    { 0x400a8987, SUC_lexicon_tags_1 },
    { 0x24aa898a, SUC_lexicon_tags_7 },
    { 0x418a898a, SUC_lexicon_tags_17 },
    { 0x3e0a898c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348a8993, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2a8996, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x502a8998, SUC_lexicon_tags_12 },
    { 0xbc4a8999, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bea899f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308a89a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ea89aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccea89ae, SUC_lexicon_tags_47 },
    { 0x7d2a89ae, SUC_lexicon_tags_12 },
    { 0x148a89b0, SUC_lexicon_tags_10 },
    { 0x702a89b1, SUC_lexicon_tags_43 },
    { 0xf34a89b1, SUC_lexicon_tags_13 },
    { 0xe6ca89b3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78aa89b7, SUC_lexicon_tags_23 },
    { 0x912a89b8, SUC_lexicon_tags_38 },
    { 0xf02a89b8, SUC_lexicon_tags_65 },
    { 0x182a89b9, SUC_lexicon_tags_49 },
    { 0x45aa89bb, SUC_lexicon_tags_15 },
    { 0xe22a89ba, SUC_lexicon_tags_6 },
    { 0x952a89b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3ca89bf, SUC_lexicon_tags_55 },
    { 0x5faa89c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32a89c4, SUC_lexicon_tags_0 },
    { 0x9eaa89c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252a89cf, SUC_lexicon_tags_62 },
    { 0xbbca89cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x80a89d2, SUC_lexicon_tags_84 },
    { 0xe6a89d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4a89d9, SUC_lexicon_tags_30 },
    { 0xfd8a89da, SUC_lexicon_tags_15 },
    { 0x74a89db, SUC_lexicon_tags_35 },
    { 0xf9ca89db, SUC_lexicon_tags_2 },
    { 0x720a89db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fea89e2, SUC_lexicon_tags_29 },
    { 0x76ea89e3, SUC_lexicon_tags_17 },
    { 0x128a89e4, SUC_lexicon_tags_37 },
    { 0x1bea89e3, SUC_lexicon_tags_25 },
    { 0x488a89e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aca89ea, SUC_lexicon_tags_19 },
    { 0x86aa89eb, SUC_lexicon_tags_7 },
    { 0xf34a89ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xedaa89ee, SUC_lexicon_tags_64 },
    { 0xcfea89ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ea89fa, SUC_lexicon_tags_34 },
    { 0x574a89fa, SUC_lexicon_tags_63 },
    { 0xa26a89fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x178a89fe, SUC_lexicon_tags_13 },
    { 0x432a89fe, SUC_lexicon_tags_2 },
    { 0x14a89ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x63ca8a02, SUC_lexicon_tags_14 },
    { 0xc76a8a02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x98a8a06, SUC_lexicon_tags_42 },
    { 0x6e0a8a07, SUC_lexicon_tags_48 },
    { 0x172a8a08, SUC_lexicon_tags_1 },
    { 0x8aa8a06, SUC_lexicon_tags_21 },
    { 0x12ca8a0a, SUC_lexicon_tags_45 },
    { 0x834a8a0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66a8a10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca8a19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab6a8a1b, SUC_lexicon_tags_63 },
    { 0x4fca8a1c, SUC_lexicon_tags_17 },
    { 0xaf0a8a1b, SUC_lexicon_tags_12 },
    { 0x64aa8a1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160a8a25, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x224a8a28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x772a8a2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eea8a2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ca8a30, SUC_lexicon_tags_4 },
    { 0x764a8a31, SUC_lexicon_tags_12 },
    { 0x8aaa8a32, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6a8a3f, SUC_lexicon_tags_19 },
    { 0xb7aa8a3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ea8a45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x152a8a47, SUC_lexicon_tags_37 },
    { 0x4e6a8a47, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x962a8a4b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ea8a51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6a8a54, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ca8a57, SUC_lexicon_tags_7 },
    { 0xf6ea8a57, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x820a8a5b, SUC_lexicon_tags_28 },
    { 0xd32a8a5c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccca8a61, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8a4a8a63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512a8a70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834a8a76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904a8a7b, SUC_lexicon_tags_2 },
    { 0xcbaa8a7c, SUC_lexicon_tags_48 },
    { 0x7dca8a7d, SUC_lexicon_tags_17 },
    { 0x46aa8a7e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8a8a83, SUC_lexicon_tags_15 },
    { 0x12a8a84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ca8a87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b4a8a89, SUC_lexicon_tags_26 },
    { 0x7b4a8a89, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x822a8a8c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc28a8a8e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdca8a93, SUC_lexicon_tags_7 },
    { 0x68ca8a94, SUC_lexicon_tags_12 },
    { 0x558a8a95, SUC_lexicon_tags_26 },
    { 0xa78a8a95, SUC_lexicon_tags_25 },
    { 0x352a8a96, SUC_lexicon_tags_19 },
    { 0xb44a8a98, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94a8a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd24a8aa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x358a8aa3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fca8aa6, SUC_lexicon_tags_15 },
    { 0x43aa8aa6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9aca8aa9, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2a8aad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6a8ab0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac0a8ab2, SUC_lexicon_tags_57 },
    { 0x1a4a8ab3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272a8ab8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6ca8aba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ea8abd, SUC_lexicon_tags_65 },
    { 0x510a8abe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2a8ac1, SUC_lexicon_tags_0 },
    { 0xc74a8ac2, SUC_lexicon_tags_82 },
    { 0x91aa8ac2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d4a8ac5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ca8ad0, SUC_lexicon_tags_32 },
    { 0x15aa8ad0, SUC_lexicon_tags_37 },
    { 0xd8ea8ad0, SUC_lexicon_tags_30 },
    { 0x988a8ad3, SUC_lexicon_tags_6 },
    { 0xde8a8ad4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9aa8ad7, SUC_lexicon_tags_43 },
    { 0xef2a8ad8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x520a8ada, SUC_lexicon_tags_32 },
    { 0x47ea8adb, SUC_lexicon_tags_60 },
    { 0x13ea8adc, SUC_lexicon_tags_2 },
    { 0xa7ca8add, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68a8ae2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdaea8ae4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ea8ae8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x764a8aea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe24a8aec, SUC_lexicon_tags_29 },
    { 0x39ca8aec, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20a8af0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ca8af3, SUC_lexicon_tags_2 },
    { 0x63ca8af4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eca8af8, SUC_lexicon_tags_1 },
    { 0x9b8a8af9, SUC_lexicon_tags_21 },
    { 0x84ea8af8, SUC_lexicon_tags_16 },
    { 0xd7ca8af9, SUC_lexicon_tags_31 },
    { 0x542a8afb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ca8b00, SUC_lexicon_tags_2 },
    { 0x876a8b01, SUC_lexicon_tags_7 },
    { 0x8b2a8b01, SUC_lexicon_tags_79 },
    { 0x78a8b00, SUC_lexicon_tags_1 },
    { 0x9eca8b04, SUC_lexicon_tags_3 },
    { 0x6d4a8b05, SUC_lexicon_tags_49 },
    { 0x5f2a8b06, SUC_lexicon_tags_15 },
    { 0xd08a8b01, SUC_lexicon_tags_26 },
    { 0x924a8b01, SUC_lexicon_tags_37 },
    { 0x3f8a8b06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dca8b0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdea8b10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcdaa8b12, SUC_lexicon_tags_27 },
    { 0xe42a8b13, SUC_lexicon_tags_33 },
    { 0x8e0a8b14, SUC_lexicon_tags_15 },
    { 0xd6a8b12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0aa8b18, SUC_lexicon_tags_21 },
    { 0x218a8b19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x316a8b1b, SUC_lexicon_tags_8 },
    { 0xa1ea8b1c, SUC_lexicon_tags_16 },
    { 0xaca8b1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aa8b25, SUC_lexicon_tags_56 },
    { 0x93ca8b26, SUC_lexicon_tags_2 },
    { 0x59ca8b27, SUC_lexicon_tags_29 },
    { 0xb14a8b28, SUC_lexicon_tags_37 },
    { 0xb9ea8b29, SUC_lexicon_tags_253 },
    { 0x5d2a8b29, SUC_lexicon_tags_0 },
    { 0xa6ea8b2b, SUC_lexicon_tags_11 },
    { 0x604a8b2c, SUC_lexicon_tags_37 },
    { 0x81ea8b2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32ca8b2f, SUC_lexicon_tags_23 },
    { 0x10a8b2f, SUC_lexicon_tags_11 },
    { 0x922a8b30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ea8b34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf64a8b36, SUC_lexicon_tags_37 },
    { 0xfaa8b37, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6a8b3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63ea8b3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x944a8b40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6a8b43, SUC_lexicon_tags_23 },
    { 0x774a8b44, SUC_lexicon_tags_56 },
    { 0x1cca8b45, SUC_lexicon_tags_113 },
    { 0x5dca8b46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64a8b49, SUC_lexicon_tags_19 },
    { 0x6c6a8b4a, SUC_lexicon_tags_25 },
    { 0x73aa8b4b, SUC_lexicon_tags_11 },
    { 0x2b8a8b4a, SUC_lexicon_tags_23 },
    { 0x66aa8b4c, SUC_lexicon_tags_15 },
    { 0x42ca8b4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ea8b51, SUC_lexicon_tags_68 },
    { 0xb0ca8b52, SUC_lexicon_tags_4 },
    { 0x42a8b52, SUC_lexicon_tags_11 },
    { 0xa54a8b52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176a8b58, SUC_lexicon_tags_10 },
    { 0x440a8b59, SUC_lexicon_tags_11 },
    { 0xb8a8b5a, SUC_lexicon_tags_12 },
    { 0xfe2a8b5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaa8b60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72a8b63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaca8b6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38a8b73, SUC_lexicon_tags_60 },
    { 0xb8ea8b74, SUC_lexicon_tags_42 },
    { 0xeaea8b74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15ca8b77, SUC_lexicon_tags_3 },
    { 0xd10a8b78, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2a8b7c, SUC_lexicon_tags_37 },
    { 0xa98a8b7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8a8b80, SUC_lexicon_tags_1 },
    { 0x378a8b81, SUC_lexicon_tags_11 },
    { 0xcaa8b82, SUC_lexicon_tags_47 },
    { 0x898a8b82, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x39ca8b85, SUC_lexicon_tags_38 },
    { 0xd80a8b86, SUC_lexicon_tags_61 },
    { 0x902a8b87, SUC_lexicon_tags_30 },
    { 0xdb8a8b86, SUC_lexicon_tags_37 },
    { 0xfea8b89, SUC_lexicon_tags_46 },
    { 0xf5ca8b85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfaa8b8c, SUC_lexicon_tags_2 },
    { 0xceea8b8d, SUC_lexicon_tags_27 },
    { 0x90ea8b8e, SUC_lexicon_tags_33 },
    { 0xc2ea8b8f, SUC_lexicon_tags_19 },
    { 0x970a8b8d, SUC_lexicon_tags_40 },
    { 0x9baa8b8e, SUC_lexicon_tags_17 },
    { 0xa1ea8b91, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cca8b98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62a8b9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ca8ba2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21aa8ba6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2a8bab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6eca8bad, SUC_lexicon_tags_11 },
    { 0xabca8bae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2d2a8bb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9baa8bb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ca8bbb, SUC_lexicon_tags_27 },
    { 0x444a8bbb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ca8bbf, SUC_lexicon_tags_138 },
    { 0x1d4a8bc0, SUC_lexicon_tags_15 },
    { 0xc72a8bc0, SUC_lexicon_tags_47 },
    { 0xd84a8bc2, SUC_lexicon_tags_28 },
    { 0x4c6a8bc3, SUC_lexicon_tags_37 },
    { 0x9d0a8bc3, SUC_lexicon_tags_17 },
    { 0x58ea8bc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd4a8bc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaa8bcb, SUC_lexicon_tags_3 },
    { 0xe2aa8bcc, SUC_lexicon_tags_25 },
    { 0xfe6a8bcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ea8bd0, SUC_lexicon_tags_11 },
    { 0x7c6a8bd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cea8bd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ca8bd8, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60a8be1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbaca8be3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8a8be7, SUC_lexicon_tags_14 },
    { 0x670a8be7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2a8beb, SUC_lexicon_tags_37 },
    { 0x8eea8beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x282a8bee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaea8bf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x31ea8bf3, SUC_lexicon_tags_19 },
    { 0xcbca8bf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x764a8bf6, SUC_lexicon_tags_33 },
    { 0xb7aa8bf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d0a8bf9, SUC_lexicon_tags_55 },
    { 0xf5ea8bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x916a8bfc, SUC_lexicon_tags_101 },
    { 0xd58a8bfc, SUC_lexicon_tags_11 },
    { 0x6d8a8bfd, SUC_lexicon_tags_1 },
    { 0x63aa8bff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e0a8c01, SUC_lexicon_tags_17 },
    { 0x97aa8c02, SUC_lexicon_tags_19 },
    { 0xbd8a8c03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598a8c07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0a8c0a, SUC_lexicon_tags_2 },
    { 0x2e6a8c0a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x46aa8c0e, SUC_lexicon_tags_2 },
    { 0xb40a8c0f, SUC_lexicon_tags_49 },
    { 0x906a8c10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24a8c15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb9ca8c17, SUC_lexicon_tags_37 },
    { 0xec6a8c17, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330a8c1e, SUC_lexicon_tags_7 },
    { 0x912a8c1f, SUC_lexicon_tags_65 },
    { 0x688a8c20, SUC_lexicon_tags_22 },
    { 0x4c6a8c21, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690a8c25, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4a8c28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a8c2b, SUC_lexicon_tags_16 },
    { 0x838a8c2b, SUC_lexicon_tags_12 },
    { 0x1e4a8c2d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8a8c30, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32a8c34, SUC_lexicon_tags_25 },
    { 0xb14a8c34, SUC_lexicon_tags_10 },
    { 0xa4ca8c36, SUC_lexicon_tags_14 },
    { 0xe96a8c37, SUC_lexicon_tags_37 },
    { 0xa2a8c38, SUC_lexicon_tags_25 },
    { 0xe38a8c35, SUC_lexicon_tags_38 },
    { 0xf02a8c36, SUC_lexicon_tags_37 },
    { 0x1eaa8c3b, SUC_lexicon_tags_4 },
    { 0xb46a8c34, SUC_lexicon_tags_23 },
    { 0x162a8c3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaa8c46, SUC_lexicon_tags_43 },
    { 0xabaa8c47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ea8c4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x644a8c4f, SUC_lexicon_tags_1 },
    { 0xff0a8c50, SUC_lexicon_tags_44 },
    { 0xf6ea8c50, SUC_lexicon_tags_9 },
    { 0x746a8c52, SUC_lexicon_tags_55 },
    { 0x9d2a8c53, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6caa8c59, SUC_lexicon_tags_23 },
    { 0x200a8c59, SUC_lexicon_tags_0 },
    { 0x24ea8c5a, SUC_lexicon_tags_11 },
    { 0x63aa8c5c, SUC_lexicon_tags_9 },
    { 0xa78a8c5d, SUC_lexicon_tags_31 },
    { 0xf92a8c5c, SUC_lexicon_tags_84 },
    { 0x76ca8c5e, SUC_lexicon_tags_7 },
    { 0x816a8c60, SUC_lexicon_tags_14 },
    { 0x84ca8c60, SUC_lexicon_tags_12 },
    { 0xb70a8c62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd36a8c64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd48a8c66, SUC_lexicon_tags_12 },
    { 0xfaea8c66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740a8c6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x774a8c70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x70ca8c72, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78a8c7b, SUC_lexicon_tags_15 },
    { 0xe94a8c7c, SUC_lexicon_tags_1 },
    { 0x77ea8c7c, SUC_lexicon_tags_14 },
    { 0x748a8c7e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x226a8c80, SUC_lexicon_tags_45 },
    { 0x27ea8c80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x90ea8c83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6a8c86, SUC_lexicon_tags_83 },
    { 0xbcea8c87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970a8c8e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30a8c91, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6a8c98, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6aca8c9a, SUC_lexicon_tags_60 },
    { 0x6e0a8c9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa00a8c9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16a8ca3, SUC_lexicon_tags_12 },
    { 0xeb8a8ca4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x298a8ca7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200a8cac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe58a8cae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fca8cb5, SUC_lexicon_tags_29 },
    { 0xec4a8cb6, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2a8cb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbdaa8cbb, SUC_lexicon_tags_1 },
    { 0x2fea8cbc, SUC_lexicon_tags_12 },
    { 0x73aa8cbc, SUC_lexicon_tags_27 },
    { 0x24ca8cbc, SUC_lexicon_tags_4 },
    { 0x4e2a8cbf, SUC_lexicon_tags_39 },
    { 0x85aa8cc0, SUC_lexicon_tags_43 },
    { 0x87aa8cc1, SUC_lexicon_tags_37 },
    { 0x9eea8cbf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ea8cc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd2a8cc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6aa8cc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadca8ccc, SUC_lexicon_tags_26 },
    { 0x272a8ccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8a8cd2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444a8cd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976a8cdc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eea8ce0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aa8ce3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ca8ce7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf0ea8ce9, SUC_lexicon_tags_66 },
    { 0xd78a8ce9, SUC_lexicon_tags_47 },
    { 0x7baa8ceb, SUC_lexicon_tags_17 },
    { 0xf22a8ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48a8cf5, SUC_lexicon_tags_43 },
    { 0x44ca8cf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa66a8cf8, SUC_lexicon_tags_37 },
    { 0x252a8cf9, SUC_lexicon_tags_37 },
    { 0xed6a8cfa, SUC_lexicon_tags_49 },
    { 0x4aca8cfb, SUC_lexicon_tags_10 },
    { 0xb50a8cfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa8a8cfe, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14a8d02, SUC_lexicon_tags_17 },
    { 0xec6a8d03, SUC_lexicon_tags_0 },
    { 0x100a8d04, SUC_lexicon_tags_15 },
    { 0x4d4a8d05, SUC_lexicon_tags_62 },
    { 0x682a8d06, SUC_lexicon_tags_11 },
    { 0x2b4a8d07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9aa8d0e, SUC_lexicon_tags_31 },
    { 0xdf6a8d0e, SUC_lexicon_tags_17 },
    { 0x458a8d0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bea8d13, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0a8d1a, SUC_lexicon_tags_37 },
    { 0x634a8d1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x968a8d1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x358a8d1f, SUC_lexicon_tags_66 },
    { 0xd4ca8d20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a8d28, SUC_lexicon_tags_45 },
    { 0xf94a8d29, SUC_lexicon_tags_37 },
    { 0xc68a8d2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5faa8d2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8cca8d2f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426a8d34, SUC_lexicon_tags_43 },
    { 0x1cea8d34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c6a8d37, SUC_lexicon_tags_56 },
    { 0xddca8d38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32a8d3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8ca8d3c, SUC_lexicon_tags_15 },
    { 0xea2a8d3d, SUC_lexicon_tags_37 },
    { 0x150a8d3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcfca8d40, SUC_lexicon_tags_10 },
    { 0xe96a8d40, SUC_lexicon_tags_11 },
    { 0xef8a8d40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8a8d45, SUC_lexicon_tags_0 },
    { 0xe64a8d46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbe2a8d48, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbb2a8d4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x994a8d4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaa8d51, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x10a8d53, SUC_lexicon_tags_37 },
    { 0x466a8d54, SUC_lexicon_tags_66 },
    { 0x598a8d54, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x14ca8d57, SUC_lexicon_tags_37 },
    { 0x402a8d57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22ea8d5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54a8d5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x570a8d5f, SUC_lexicon_tags_26 },
    { 0xcc6a8d60, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe02a8d62, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbea8d67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xebca8d69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0a8d6c, SUC_lexicon_tags_19 },
    { 0x85aa8d6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490a8d71, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2a8d78, SUC_lexicon_tags_63 },
    { 0xb86a8d78, SUC_lexicon_tags_27 },
    { 0x6fea8d78, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292a8d7f, SUC_lexicon_tags_16 },
    { 0x1bea8d80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x796a8d83, SUC_lexicon_tags_1 },
    { 0xfca8d84, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x2e6a8d86, SUC_lexicon_tags_7 },
    { 0xf34a8d86, SUC_lexicon_tags_33 },
    { 0x496a8d88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ea8d8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x544a8d8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba4a8d91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2a8d93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f4a8d95, SUC_lexicon_tags_80 },
    { 0xea8a8d95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0a8d9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35ea8d9d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd7ea8d9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x904a8da1, SUC_lexicon_tags_63 },
    { 0xf34a8da1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ea8da7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8a8dac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9aa8daf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4a8db2, SUC_lexicon_tags_45 },
    { 0xdcca8db2, SUC_lexicon_tags_11 },
    { 0x71aa8db4, SUC_lexicon_tags_17 },
    { 0xc9ca8db2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732a8dbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6a8dc4, SUC_lexicon_tags_25 },
    { 0x260a8dc5, SUC_lexicon_tags_46 },
    { 0x63ca8dc6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaa8dce, SUC_lexicon_tags_12 },
    { 0x50aa8dcf, SUC_lexicon_tags_19 },
    { 0xacaa8dd0, SUC_lexicon_tags_34 },
    { 0xf3aa8dcf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36a8dd5, SUC_lexicon_tags_37 },
    { 0x12a8dd6, SUC_lexicon_tags_53 },
    { 0x206a8dd6, SUC_lexicon_tags_12 },
    { 0x64aa8dd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15aa8dda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8a8ddd, SUC_lexicon_tags_25 },
    { 0x530a8dde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58a8de1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9eca8de3, SUC_lexicon_tags_7 },
    { 0xe58a8de4, SUC_lexicon_tags_11 },
    { 0x2aca8de5, SUC_lexicon_tags_23 },
    { 0x2aa8de6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6a8ded, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x200a8df0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84a8df4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ca8df6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88a8dfa, SUC_lexicon_tags_17 },
    { 0xbca8dfb, SUC_lexicon_tags_15 },
    { 0x5e2a8dfa, SUC_lexicon_tags_82 },
    { 0x37ea8dfa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87aa8e04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2a8e09, SUC_lexicon_tags_13 },
    { 0x66ea8e0a, SUC_lexicon_tags_1 },
    { 0x472a8e0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ea8e0e, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x870a8e10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306a8e14, SUC_lexicon_tags_25 },
    { 0x6bea8e15, SUC_lexicon_tags_60 },
    { 0xc6a8e16, SUC_lexicon_tags_15 },
    { 0x3ea8e15, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68a8e21, SUC_lexicon_tags_11 },
    { 0x2a8a8e22, SUC_lexicon_tags_45 },
    { 0x2d0a8e22, SUC_lexicon_tags_1 },
    { 0x6c6a8e23, SUC_lexicon_tags_57 },
    { 0x204a8e22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a6a8e27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf6a8e29, SUC_lexicon_tags_2 },
    { 0x4e2a8e2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78a8e2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfbea8e30, SUC_lexicon_tags_16 },
    { 0xd14a8e31, SUC_lexicon_tags_14 },
    { 0x36a8e32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908a8e3a, SUC_lexicon_tags_7 },
    { 0xc18a8e3b, SUC_lexicon_tags_4 },
    { 0xc26a8e3b, SUC_lexicon_tags_37 },
    { 0x11aa8e3d, SUC_lexicon_tags_12 },
    { 0xc62a8e3e, SUC_lexicon_tags_61 },
    { 0x5faa8e3f, SUC_lexicon_tags_38 },
    { 0x93ea8e3f, SUC_lexicon_tags_46 },
    { 0x9e4a8e41, SUC_lexicon_tags_25 },
    { 0xa2ea8e3a, SUC_lexicon_tags_12 },
    { 0x43ca8e43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dca8e47, SUC_lexicon_tags_15 },
    { 0x524a8e47, SUC_lexicon_tags_2 },
    { 0x4e4a8e49, SUC_lexicon_tags_39 },
    { 0x45ea8e4a, SUC_lexicon_tags_19 },
    { 0xccca8e47, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eca8e4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd2a8e51, SUC_lexicon_tags_23 },
    { 0x5b0a8e52, SUC_lexicon_tags_83 },
    { 0x302a8e52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x820a8e55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92a8e59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4a8e5f, SUC_lexicon_tags_116 },
    { 0xc4ca8e60, SUC_lexicon_tags_37 },
    { 0xf7ca8e61, SUC_lexicon_tags_38 },
    { 0x5e8a8e62, SUC_lexicon_tags_46 },
    { 0xab6a8e61, SUC_lexicon_tags_37 },
    { 0x856a8e64, SUC_lexicon_tags_7 },
    { 0x4e6a8e62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786a8e6a, SUC_lexicon_tags_8 },
    { 0x56ea8e6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ca8e71, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xef6a8e73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4a8e7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06a8e7d, SUC_lexicon_tags_15 },
    { 0x6e4a8e7e, SUC_lexicon_tags_21 },
    { 0x4e6a8e7d, SUC_lexicon_tags_61 },
    { 0xce0a8e80, SUC_lexicon_tags_7 },
    { 0xfa4a8e81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02a8e84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214a8e89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ca8e8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9ca8e91, SUC_lexicon_tags_32 },
    { 0x796a8e91, SUC_lexicon_tags_25 },
    { 0xe9aa8e93, SUC_lexicon_tags_33 },
    { 0xcd2a8e94, SUC_lexicon_tags_17 },
    { 0x122a8e95, SUC_lexicon_tags_47 },
    { 0x1e8a8e93, SUC_lexicon_tags_2 },
    { 0x110a8e97, SUC_lexicon_tags_25 },
    { 0x73aa8e98, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88a8e9c, SUC_lexicon_tags_146 },
    { 0x1aa8e9d, SUC_lexicon_tags_2 },
    { 0x200a8e9e, SUC_lexicon_tags_108 },
    { 0x4aca8e9e, SUC_lexicon_tags_17 },
    { 0x3b0a8e9e, SUC_lexicon_tags_45 },
    { 0xc30a8e9e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0a8ea6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26aa8eaa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3aa8eaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1eaa8eb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ea8eb5, SUC_lexicon_tags_2 },
    { 0x9c0a8eb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x910a8eb8, SUC_lexicon_tags_11 },
    { 0x656a8eb9, SUC_lexicon_tags_0 },
    { 0x564a8eba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0a8ebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb04a8ec0, SUC_lexicon_tags_558 },
    { 0x646a8ec1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9aa8ec3, SUC_lexicon_tags_87 },
    { 0xf8aa8ec4, SUC_lexicon_tags_15 },
    { 0xcf6a8ec5, SUC_lexicon_tags_13 },
    { 0xd52a8ec6, SUC_lexicon_tags_37 },
    { 0xfa6a8ec7, SUC_lexicon_tags_0 },
    { 0x188a8ec3, SUC_lexicon_tags_25 },
    { 0x470a8ec9, SUC_lexicon_tags_25 },
    { 0x86aa8ec5, SUC_lexicon_tags_148 },
    { 0x12ca8ecb, SUC_lexicon_tags_61 },
    { 0x5a0a8ecb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1ca8ece, SUC_lexicon_tags_10 },
    { 0xea8a8ecf, SUC_lexicon_tags_16 },
    { 0xceca8ed0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x920a8ed2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x684a8ed5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bca8edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06a8ee0, SUC_lexicon_tags_62 },
    { 0xf8ca8ee1, SUC_lexicon_tags_26 },
    { 0x6a6a8ee2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x892a8ee4, SUC_lexicon_tags_4 },
    { 0x13aa8ee5, SUC_lexicon_tags_23 },
    { 0x2dea8ee4, SUC_lexicon_tags_7 },
    { 0x516a8ee7, SUC_lexicon_tags_17 },
    { 0xa04a8ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x350a8eeb, SUC_lexicon_tags_17 },
    { 0xd8ea8eec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4a8eef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea6a8ef1, SUC_lexicon_tags_59 },
    { 0x49ca8ef2, SUC_lexicon_tags_29 },
    { 0x526a8ef3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb30a8ef5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0a8ef8, SUC_lexicon_tags_14 },
    { 0x9b6a8ef9, SUC_lexicon_tags_28 },
    { 0x78aa8efa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ca8eff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aea8f06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b0a8f08, SUC_lexicon_tags_15 },
    { 0xc8ea8f09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x780a8f0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa88a8f0d, SUC_lexicon_tags_104 },
    { 0x8bca8f0d, SUC_lexicon_tags_69 },
    { 0x728a8f0f, SUC_lexicon_tags_66 },
    { 0x596a8f10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x632a8f12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48a8f1d, SUC_lexicon_tags_32 },
    { 0x2f2a8f1e, SUC_lexicon_tags_8 },
    { 0x750a8f1f, SUC_lexicon_tags_7 },
    { 0x67aa8f1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98a8f2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ca8f30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x67aa8f33, SUC_lexicon_tags_15 },
    { 0x96aa8f34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd00a8f36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8a8f41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6a8f49, SUC_lexicon_tags_29 },
    { 0x212a8f4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54ca8f4c, SUC_lexicon_tags_2 },
    { 0x4eca8f4c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0a8f51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb84a8f53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5eea8f55, SUC_lexicon_tags_31 },
    { 0x7f0a8f56, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc10a8f58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aa8f5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8eca8f5d, SUC_lexicon_tags_17 },
    { 0x578a8f5d, SUC_lexicon_tags_25 },
    { 0x850a8f5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666a8f66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ea8f6c, SUC_lexicon_tags_14 },
    { 0xd0ea8f6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x524a8f70, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x18a8f73, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ea8f78, SUC_lexicon_tags_87 },
    { 0xdea8f79, SUC_lexicon_tags_112 },
    { 0xdfaa8f7a, SUC_lexicon_tags_14 },
    { 0x70ea8f79, SUC_lexicon_tags_63 },
    { 0xcf0a8f7c, SUC_lexicon_tags_63 },
    { 0x62ca8f7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242a8f83, SUC_lexicon_tags_37 },
    { 0x4caa8f83, SUC_lexicon_tags_12 },
    { 0x75ea8f85, SUC_lexicon_tags_15 },
    { 0xec4a8f86, SUC_lexicon_tags_32 },
    { 0x364a8f87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aca8f8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfaaa8f8d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb74a8f8f, SUC_lexicon_tags_1 },
    { 0x71aa8f90, SUC_lexicon_tags_56 },
    { 0x36a8f8f, SUC_lexicon_tags_17 },
    { 0xab6a8f8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc82a8f94, SUC_lexicon_tags_1 },
    { 0xddca8f95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc22a8f97, SUC_lexicon_tags_36 },
    { 0x1daa8f98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808a8f9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672a8fa2, SUC_lexicon_tags_40 },
    { 0xcd2a8fa2, SUC_lexicon_tags_11 },
    { 0xf04a8fa4, SUC_lexicon_tags_43 },
    { 0x36ca8fa4, SUC_lexicon_tags_43 },
    { 0xbf4a8fa4, SUC_lexicon_tags_6 },
    { 0x960a8fa7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x434a8fa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4a8fac, SUC_lexicon_tags_29 },
    { 0x6aa8fad, SUC_lexicon_tags_15 },
    { 0xaf0a8fac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64aa8fb0, SUC_lexicon_tags_12 },
    { 0x258a8fb0, SUC_lexicon_tags_11 },
    { 0x5c0a8fb2, SUC_lexicon_tags_12 },
    { 0xf9aa8fb2, SUC_lexicon_tags_10 },
    { 0xa90a8fb4, SUC_lexicon_tags_0 },
    { 0x5aea8fb5, SUC_lexicon_tags_11 },
    { 0xec8a8fb5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf0a8fb8, SUC_lexicon_tags_43 },
    { 0x4cca8fb8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x834a8fbc, SUC_lexicon_tags_1519 },
    { 0, NULL },
    { 0xb8ea8fbe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x606a8fc1, SUC_lexicon_tags_15 },
    { 0x542a8fc1, SUC_lexicon_tags_11 },
    { 0x440a8fc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72a8fc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2a8fcd, SUC_lexicon_tags_37 },
    { 0xd9ca8fce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfaa8fd0, SUC_lexicon_tags_47 },
    { 0x30a8fd1, SUC_lexicon_tags_1140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76a8fda, SUC_lexicon_tags_37 },
    { 0xf68a8fdb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8a8fe0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95aa8fe3, SUC_lexicon_tags_3 },
    { 0x766a8fe4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aa8fe8, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x156a8fea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4a8ff1, SUC_lexicon_tags_27 },
    { 0x394a8ff2, SUC_lexicon_tags_15 },
    { 0xb68a8ff3, SUC_lexicon_tags_11 },
    { 0x83ea8ff4, SUC_lexicon_tags_27 },
    { 0xd32a8ff1, SUC_lexicon_tags_11 },
    { 0x3c6a8ff1, SUC_lexicon_tags_87 },
    { 0x3caa8ff7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2a8ffe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680a9003, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x444a9005, SUC_lexicon_tags_46 },
    { 0x562a9006, SUC_lexicon_tags_47 },
    { 0xf0aa9006, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x766a9009, SUC_lexicon_tags_37 },
    { 0x2fca900a, SUC_lexicon_tags_37 },
    { 0xe04a900b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a2a900d, SUC_lexicon_tags_7 },
    { 0x748a900e, SUC_lexicon_tags_15 },
    { 0x640a900d, SUC_lexicon_tags_11 },
    { 0xf84a9010, SUC_lexicon_tags_39 },
    { 0xecca9010, SUC_lexicon_tags_49 },
    { 0x72a9012, SUC_lexicon_tags_23 },
    { 0x31aa9013, SUC_lexicon_tags_49 },
    { 0x552a900e, SUC_lexicon_tags_17 },
    { 0x55aa9015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb68a9017, SUC_lexicon_tags_47 },
    { 0x964a9017, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224a901e, SUC_lexicon_tags_64 },
    { 0x14ea901f, SUC_lexicon_tags_56 },
    { 0xa80a901f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2a9027, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aa902a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xae2a902c, SUC_lexicon_tags_57 },
    { 0xbc6a902d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd7ea902f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x10ea9031, SUC_lexicon_tags_15 },
    { 0x9dea9032, SUC_lexicon_tags_17 },
    { 0xa42a9033, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874a9038, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426a903f, SUC_lexicon_tags_46 },
    { 0x7a6a9040, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ea9048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcea904c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x932a904f, SUC_lexicon_tags_10 },
    { 0x65aa9050, SUC_lexicon_tags_65 },
    { 0x48ca9050, SUC_lexicon_tags_29 },
    { 0xe5ea9052, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32a9056, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8a905a, SUC_lexicon_tags_37 },
    { 0xffca905b, SUC_lexicon_tags_29 },
    { 0x826a905b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88a9066, SUC_lexicon_tags_48 },
    { 0x3d8a9067, SUC_lexicon_tags_38 },
    { 0x2cea9068, SUC_lexicon_tags_12 },
    { 0x1f8a9067, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382a906d, SUC_lexicon_tags_28 },
    { 0x6c8a906d, SUC_lexicon_tags_9 },
    { 0xc1aa906e, SUC_lexicon_tags_66 },
    { 0x802a9070, SUC_lexicon_tags_25 },
    { 0x26aa906d, SUC_lexicon_tags_9 },
    { 0x3f4a9072, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48a9076, SUC_lexicon_tags_49 },
    { 0x488a9077, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa2a9079, SUC_lexicon_tags_47 },
    { 0x300a907a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f2a907c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x544a907e, SUC_lexicon_tags_5 },
    { 0x1aca907f, SUC_lexicon_tags_47 },
    { 0xf76a907e, SUC_lexicon_tags_11 },
    { 0xb82a9080, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe90a9084, SUC_lexicon_tags_2 },
    { 0x33aa9085, SUC_lexicon_tags_17 },
    { 0x1f4a9086, SUC_lexicon_tags_47 },
    { 0xea9085, SUC_lexicon_tags_13 },
    { 0xfdea9086, SUC_lexicon_tags_5 },
    { 0x7c2a9088, SUC_lexicon_tags_29 },
    { 0xac2a9086, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0a9099, SUC_lexicon_tags_7 },
    { 0xa12a909a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x64ea909c, SUC_lexicon_tags_14 },
    { 0x69ea909d, SUC_lexicon_tags_25 },
    { 0x9b8a909e, SUC_lexicon_tags_19 },
    { 0xd42a909f, SUC_lexicon_tags_33 },
    { 0xcf0a90a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x19aa90a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f2a90a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64aa90aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6a90ad, SUC_lexicon_tags_17 },
    { 0x91ea90ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x444a90b0, SUC_lexicon_tags_1 },
    { 0x954a90b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x176a90b4, SUC_lexicon_tags_84 },
    { 0x7b4a90b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d8a90b7, SUC_lexicon_tags_11 },
    { 0x51aa90b8, SUC_lexicon_tags_7 },
    { 0x5f4a90b9, SUC_lexicon_tags_44 },
    { 0x5a2a90b9, SUC_lexicon_tags_25 },
    { 0x66ca90b7, SUC_lexicon_tags_46 },
    { 0xaeea90bc, SUC_lexicon_tags_15 },
    { 0xbf4a90bd, SUC_lexicon_tags_16 },
    { 0xccca90bb, SUC_lexicon_tags_7 },
    { 0x264a90bf, SUC_lexicon_tags_47 },
    { 0x8b2a90bd, SUC_lexicon_tags_49 },
    { 0xb5ea90be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712a90c5, SUC_lexicon_tags_25 },
    { 0xd48a90c6, SUC_lexicon_tags_12 },
    { 0x2fca90c7, SUC_lexicon_tags_37 },
    { 0x2fca90c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14a90cb, SUC_lexicon_tags_47 },
    { 0xc24a90cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980a90d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ca90d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f8a90d7, SUC_lexicon_tags_37 },
    { 0xc64a90d7, SUC_lexicon_tags_12 },
    { 0xefca90d9, SUC_lexicon_tags_39 },
    { 0x686a90d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3daa90dd, SUC_lexicon_tags_63 },
    { 0x442a90dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dca90e1, SUC_lexicon_tags_38 },
    { 0xa64a90e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16a90e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52a90ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd8a90ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa62a90ee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aa90f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaa90f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8a9100, SUC_lexicon_tags_15 },
    { 0x94ea9100, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5a0a9103, SUC_lexicon_tags_84 },
    { 0xb68a9104, SUC_lexicon_tags_26 },
    { 0x316a9105, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ca9108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x848a910a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bea9112, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2aa9114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aa9118, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4a911d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29ca911f, SUC_lexicon_tags_37 },
    { 0xb94a9120, SUC_lexicon_tags_30 },
    { 0x766a9120, SUC_lexicon_tags_1 },
    { 0xfc8a9120, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222a9129, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aa9131, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46a9138, SUC_lexicon_tags_0 },
    { 0x97ca9139, SUC_lexicon_tags_10 },
    { 0xe10a9139, SUC_lexicon_tags_47 },
    { 0x43aa9139, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636a913f, SUC_lexicon_tags_198 },
    { 0x430a9140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19ca9142, SUC_lexicon_tags_27 },
    { 0x6dea9142, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2a9146, SUC_lexicon_tags_12 },
    { 0xaeea9147, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14a914a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8a9157, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2a915c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2a9161, SUC_lexicon_tags_91 },
    { 0xc5aa9162, SUC_lexicon_tags_25 },
    { 0xa18a9162, SUC_lexicon_tags_37 },
    { 0xa06a9162, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aa9169, SUC_lexicon_tags_28 },
    { 0x1c4a916a, SUC_lexicon_tags_8 },
    { 0x6d6a9169, SUC_lexicon_tags_11 },
    { 0x77aa916c, SUC_lexicon_tags_31 },
    { 0xdf0a916a, SUC_lexicon_tags_2 },
    { 0x218a916c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xafea9171, SUC_lexicon_tags_8 },
    { 0xc8ea9171, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa48a9174, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ea9177, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456a917c, SUC_lexicon_tags_1 },
    { 0x970a917d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x402a9180, SUC_lexicon_tags_47 },
    { 0xdcaa9181, SUC_lexicon_tags_9 },
    { 0x7f6a9182, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02a9187, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x972a9189, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ea918e, SUC_lexicon_tags_47 },
    { 0xccca918f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4faa9191, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c2a9193, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aa9196, SUC_lexicon_tags_37 },
    { 0x58ea9197, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x312a919a, SUC_lexicon_tags_29 },
    { 0x2caa919b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6a919f, SUC_lexicon_tags_47 },
    { 0x1a4a919f, SUC_lexicon_tags_13 },
    { 0xf0ea91a1, SUC_lexicon_tags_148 },
    { 0x4daa91a2, SUC_lexicon_tags_27 },
    { 0x884a91a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aa91a7, SUC_lexicon_tags_33 },
    { 0x6baa91a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6ea91aa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268a91b0, SUC_lexicon_tags_31 },
    { 0x8d8a91b0, SUC_lexicon_tags_19 },
    { 0x55ea91b1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x574a91b4, SUC_lexicon_tags_28 },
    { 0x220a91b4, SUC_lexicon_tags_39 },
    { 0xbd4a91b6, SUC_lexicon_tags_17 },
    { 0x3dca91b4, SUC_lexicon_tags_23 },
    { 0x226a91b8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3bea91ba, SUC_lexicon_tags_7 },
    { 0xcaaa91bb, SUC_lexicon_tags_12 },
    { 0x8e2a91bb, SUC_lexicon_tags_11 },
    { 0xdb4a91ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74aa91c0, SUC_lexicon_tags_1 },
    { 0x94a91c1, SUC_lexicon_tags_25 },
    { 0x9bea91c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fea91c6, SUC_lexicon_tags_12 },
    { 0x50a91c7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2a8a91c9, SUC_lexicon_tags_17 },
    { 0x3cea91c9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaa91cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914a91d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa6a91d3, SUC_lexicon_tags_25 },
    { 0x814a91d4, SUC_lexicon_tags_17 },
    { 0x40ea91d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6ca91d7, SUC_lexicon_tags_29 },
    { 0x6daa91d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36a91dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ea91e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x592a91e3, SUC_lexicon_tags_37 },
    { 0xf06a91e4, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x1c0a91e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x462a91e8, SUC_lexicon_tags_44 },
    { 0x78a91e8, SUC_lexicon_tags_2 },
    { 0xde8a91ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9baa91f5, SUC_lexicon_tags_17 },
    { 0x76aa91f6, SUC_lexicon_tags_989 },
    { 0, NULL },
    { 0x708a91f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aa91fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x566a91fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86a9201, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6a9207, SUC_lexicon_tags_25 },
    { 0x330a9207, SUC_lexicon_tags_7 },
    { 0x152a9209, SUC_lexicon_tags_87 },
    { 0xd80a920a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6b0a920c, SUC_lexicon_tags_48 },
    { 0x26ea920d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a9211, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x928a9214, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80a921d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7eaa921f, SUC_lexicon_tags_32 },
    { 0x74a921f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x34a9223, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x28aa9226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ea922b, SUC_lexicon_tags_36 },
    { 0xbdea922c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618a9233, SUC_lexicon_tags_15 },
    { 0xef8a9233, SUC_lexicon_tags_21 },
    { 0x5a0a9235, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x180a9237, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650a923e, SUC_lexicon_tags_66 },
    { 0x418a923e, SUC_lexicon_tags_25 },
    { 0xb42a9240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2a9244, SUC_lexicon_tags_9 },
    { 0x996a9244, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ca924d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9f6a924f, SUC_lexicon_tags_45 },
    { 0x540a9250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x736a9252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x896a9254, SUC_lexicon_tags_15 },
    { 0xcaca9254, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6a9258, SUC_lexicon_tags_22 },
    { 0x70ca9258, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xac4a925b, SUC_lexicon_tags_12 },
    { 0xca2a925c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaa9263, SUC_lexicon_tags_15 },
    { 0x4eca9264, SUC_lexicon_tags_33 },
    { 0x4daa9263, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf12a9267, SUC_lexicon_tags_57 },
    { 0x852a9267, SUC_lexicon_tags_214 },
    { 0x16ca9269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ca926d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6a9270, SUC_lexicon_tags_7 },
    { 0xb82a9271, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4daa9273, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x634a9276, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4a927d, SUC_lexicon_tags_19 },
    { 0x706a927d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0a9281, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xac0a9283, SUC_lexicon_tags_25 },
    { 0xd3ca9284, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd00a9286, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6a928a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c2a928c, SUC_lexicon_tags_12 },
    { 0xbe6a928d, SUC_lexicon_tags_142 },
    { 0, NULL },
    { 0x40a928f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe5aa9291, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ca9296, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xd12a9298, SUC_lexicon_tags_17 },
    { 0xcaea9299, SUC_lexicon_tags_2 },
    { 0x930a929a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212a929e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfca92a2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeca92a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe50a92a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40a92aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2a92af, SUC_lexicon_tags_95 },
    { 0x5dea92af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ca92b6, SUC_lexicon_tags_31 },
    { 0xdcca92b6, SUC_lexicon_tags_17 },
    { 0x3b8a92b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4a92bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612a92c0, SUC_lexicon_tags_47 },
    { 0xc86a92c1, SUC_lexicon_tags_11 },
    { 0xc8ca92c2, SUC_lexicon_tags_37 },
    { 0x75ea92c2, SUC_lexicon_tags_1 },
    { 0xa4a92c4, SUC_lexicon_tags_1 },
    { 0xc9ca92c5, SUC_lexicon_tags_29 },
    { 0x904a92c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ea92ca, SUC_lexicon_tags_12 },
    { 0xe38a92ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6ca92cd, SUC_lexicon_tags_17 },
    { 0xe6a92ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96a92d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810a92d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d8a92d9, SUC_lexicon_tags_12 },
    { 0x6b8a92da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2a92e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8a92eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfaca92ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908a92f3, SUC_lexicon_tags_1 },
    { 0x9d2a92f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0a92f8, SUC_lexicon_tags_11 },
    { 0x29ca92f8, SUC_lexicon_tags_23 },
    { 0x1aaa92f8, SUC_lexicon_tags_11 },
    { 0x958a92fb, SUC_lexicon_tags_17 },
    { 0x32aa92fc, SUC_lexicon_tags_11 },
    { 0x8fca92fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9305, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc3ca9307, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa36a9309, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x558a930c, SUC_lexicon_tags_25 },
    { 0xaa2a930c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfa6a930f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x792a9311, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbaea9313, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaca9316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab8a9318, SUC_lexicon_tags_7 },
    { 0xfbea9319, SUC_lexicon_tags_29 },
    { 0x16ca931a, SUC_lexicon_tags_2 },
    { 0x818a931a, SUC_lexicon_tags_7 },
    { 0x6cea931b, SUC_lexicon_tags_14 },
    { 0x7aa9319, SUC_lexicon_tags_2 },
    { 0x286a931e, SUC_lexicon_tags_37 },
    { 0x9e0a931e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cca9321, SUC_lexicon_tags_17 },
    { 0x4f4a9322, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bea932a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4a932d, SUC_lexicon_tags_25 },
    { 0xf0ea932e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaa9331, SUC_lexicon_tags_3 },
    { 0xb2a9331, SUC_lexicon_tags_37 },
    { 0x450a9333, SUC_lexicon_tags_25 },
    { 0xaa4a9334, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6eca9336, SUC_lexicon_tags_11 },
    { 0x25ca9336, SUC_lexicon_tags_16 },
    { 0x7b6a9338, SUC_lexicon_tags_87 },
    { 0x5ca9339, SUC_lexicon_tags_14 },
    { 0xd96a933a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x252a933c, SUC_lexicon_tags_10 },
    { 0x71ea933d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ca9340, SUC_lexicon_tags_63 },
    { 0x7b2a9341, SUC_lexicon_tags_26 },
    { 0xceca9340, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aea9346, SUC_lexicon_tags_47 },
    { 0xa80a9347, SUC_lexicon_tags_9 },
    { 0xad2a9347, SUC_lexicon_tags_82 },
    { 0x7f4a9349, SUC_lexicon_tags_49 },
    { 0x57ca9346, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74a934e, SUC_lexicon_tags_29 },
    { 0x7ea934f, SUC_lexicon_tags_350 },
    { 0x762a934e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcfea9352, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aa9358, SUC_lexicon_tags_37 },
    { 0xd18a9359, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458a935d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34a9363, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4ea9365, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x10ca9367, SUC_lexicon_tags_12 },
    { 0xa84a9368, SUC_lexicon_tags_9 },
    { 0x42ca9369, SUC_lexicon_tags_25 },
    { 0xf4ea936a, SUC_lexicon_tags_31 },
    { 0x99ca9368, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820a9370, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28a9374, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ca937c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dea937f, SUC_lexicon_tags_63 },
    { 0xc6a9380, SUC_lexicon_tags_19 },
    { 0xd88a9381, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4a9384, SUC_lexicon_tags_10 },
    { 0x6faa9385, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x11aa9387, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x760a938a, SUC_lexicon_tags_60 },
    { 0xa86a938a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab8a938d, SUC_lexicon_tags_43 },
    { 0x37aa938e, SUC_lexicon_tags_27 },
    { 0x988a938f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xceaa9391, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ca9395, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1e4a9397, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x670a939a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3c8a939c, SUC_lexicon_tags_11 },
    { 0x35ea939d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22a93a4, SUC_lexicon_tags_27 },
    { 0xaf6a93a5, SUC_lexicon_tags_22 },
    { 0xdeaa93a6, SUC_lexicon_tags_25 },
    { 0x622a93a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe0ea93a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eea93ac, SUC_lexicon_tags_55 },
    { 0x94a93ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4ca93af, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf80a93b1, SUC_lexicon_tags_2 },
    { 0x772a93b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64a93b6, SUC_lexicon_tags_32 },
    { 0x348a93b7, SUC_lexicon_tags_7 },
    { 0x4baa93b7, SUC_lexicon_tags_0 },
    { 0x742a93b9, SUC_lexicon_tags_17 },
    { 0x2eca93b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b2a93bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952a93c7, SUC_lexicon_tags_16 },
    { 0x102a93c8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf48a93ca, SUC_lexicon_tags_170 },
    { 0x1b0a93cb, SUC_lexicon_tags_25 },
    { 0xf76a93cb, SUC_lexicon_tags_37 },
    { 0x6daa93cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a4a93cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0a93d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614a93df, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8a93e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x964a93e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x290a93e7, SUC_lexicon_tags_11 },
    { 0x108a93e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ca93ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfea93ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1ca93f1, SUC_lexicon_tags_4 },
    { 0xe60a93f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8a93f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac4a93f9, SUC_lexicon_tags_43 },
    { 0x398a93fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f8a93fc, SUC_lexicon_tags_11 },
    { 0xfeaa93fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73aa9403, SUC_lexicon_tags_15 },
    { 0x36a9403, SUC_lexicon_tags_7 },
    { 0x6b6a9404, SUC_lexicon_tags_82 },
    { 0xef4a9406, SUC_lexicon_tags_77 },
    { 0x3ca9405, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2a940b, SUC_lexicon_tags_25 },
    { 0x664a940c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ea940f, SUC_lexicon_tags_13 },
    { 0xde6a9410, SUC_lexicon_tags_11 },
    { 0x25ca9411, SUC_lexicon_tags_13 },
    { 0x564a9412, SUC_lexicon_tags_15 },
    { 0xff4a9411, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x62aa9415, SUC_lexicon_tags_7 },
    { 0xe26a9416, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x296a9418, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2aea941a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ea941d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186a9423, SUC_lexicon_tags_49 },
    { 0x8a9424, SUC_lexicon_tags_12 },
    { 0x6a4a9423, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83aa9429, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8a942c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48a9431, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x796a9433, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774a9437, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xdb2a9439, SUC_lexicon_tags_27 },
    { 0x986a9439, SUC_lexicon_tags_17 },
    { 0x42ea943b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6a943e, SUC_lexicon_tags_15 },
    { 0x222a943f, SUC_lexicon_tags_26 },
    { 0x702a9440, SUC_lexicon_tags_49 },
    { 0x7eaa9440, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12aa9449, SUC_lexicon_tags_29 },
    { 0x9f0a944a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3caa944f, SUC_lexicon_tags_43 },
    { 0xc38a944f, SUC_lexicon_tags_23 },
    { 0xfa0a9451, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17ca9453, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ca9459, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4a945f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a9463, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60aa9466, SUC_lexicon_tags_68 },
    { 0x914a9466, SUC_lexicon_tags_87 },
    { 0x5e8a9468, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcea946c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ca946f, SUC_lexicon_tags_37 },
    { 0x37aa946f, SUC_lexicon_tags_23 },
    { 0x67ca9471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ea9475, SUC_lexicon_tags_35 },
    { 0xc56a9476, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x982a9478, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa8a947a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eca9481, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8ca9483, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x43ca9485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x330a9487, SUC_lexicon_tags_3 },
    { 0xc5ca9488, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08a948b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x710a948e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724a9494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9faa9496, SUC_lexicon_tags_37 },
    { 0x742a9497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x684a949a, SUC_lexicon_tags_37 },
    { 0x2b8a949b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80a949f, SUC_lexicon_tags_23 },
    { 0x6baa94a0, SUC_lexicon_tags_62 },
    { 0xc6aa94a0, SUC_lexicon_tags_60 },
    { 0xe54a94a2, SUC_lexicon_tags_60 },
    { 0xc28a94a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ca94a5, SUC_lexicon_tags_35 },
    { 0x3cca94a6, SUC_lexicon_tags_10 },
    { 0x984a94a7, SUC_lexicon_tags_27 },
    { 0xc76a94a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf92a94aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598a94ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4a94b1, SUC_lexicon_tags_68 },
    { 0x1a8a94b2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ea94b9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaa94bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96aa94c1, SUC_lexicon_tags_33 },
    { 0x170a94c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aa94c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58a94cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4faa94cf, SUC_lexicon_tags_25 },
    { 0x3d2a94d0, SUC_lexicon_tags_47 },
    { 0xad0a94d1, SUC_lexicon_tags_4 },
    { 0x962a94d2, SUC_lexicon_tags_42 },
    { 0xc08a94d3, SUC_lexicon_tags_2 },
    { 0xc3aa94d4, SUC_lexicon_tags_1 },
    { 0x648a94d5, SUC_lexicon_tags_19 },
    { 0xf86a94d5, SUC_lexicon_tags_27 },
    { 0x808a94d2, SUC_lexicon_tags_60 },
    { 0xc44a94d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fca94da, SUC_lexicon_tags_30 },
    { 0x942a94db, SUC_lexicon_tags_352 },
    { 0x3b0a94db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ca94e1, SUC_lexicon_tags_30 },
    { 0x124a94e2, SUC_lexicon_tags_23 },
    { 0xe06a94e3, SUC_lexicon_tags_4 },
    { 0x704a94e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x932a94e6, SUC_lexicon_tags_23 },
    { 0x846a94e6, SUC_lexicon_tags_17 },
    { 0x910a94e8, SUC_lexicon_tags_12 },
    { 0x7daa94e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x392a94eb, SUC_lexicon_tags_38 },
    { 0xae2a94ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1aa94f3, SUC_lexicon_tags_25 },
    { 0xc54a94f4, SUC_lexicon_tags_23 },
    { 0x662a94f4, SUC_lexicon_tags_25 },
    { 0x2d2a94f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf4a94f8, SUC_lexicon_tags_27 },
    { 0xfb2a94f9, SUC_lexicon_tags_37 },
    { 0x77ea94f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8a94fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8a9503, SUC_lexicon_tags_17 },
    { 0x834a9504, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1c6a9506, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aa950b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaf6a950d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x97ea950f, SUC_lexicon_tags_37 },
    { 0xda6a9510, SUC_lexicon_tags_37 },
    { 0xd6ea9511, SUC_lexicon_tags_28 },
    { 0xe8a9512, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daa9516, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592a951b, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78a951f, SUC_lexicon_tags_63 },
    { 0x20ea951f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x590a9522, SUC_lexicon_tags_19 },
    { 0x610a9522, SUC_lexicon_tags_25 },
    { 0xc28a9522, SUC_lexicon_tags_59 },
    { 0x9baa9525, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512a9529, SUC_lexicon_tags_63 },
    { 0xba8a952a, SUC_lexicon_tags_11 },
    { 0xab6a952b, SUC_lexicon_tags_63 },
    { 0xe64a952b, SUC_lexicon_tags_17 },
    { 0xe12a952b, SUC_lexicon_tags_25 },
    { 0xeeaa952d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x748a9530, SUC_lexicon_tags_80 },
    { 0x23ea9531, SUC_lexicon_tags_47 },
    { 0xeeea9532, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934a953c, SUC_lexicon_tags_49 },
    { 0xbca953c, SUC_lexicon_tags_0 },
    { 0x82aa953c, SUC_lexicon_tags_37 },
    { 0x5dea953f, SUC_lexicon_tags_33 },
    { 0x54ea953f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ca9543, SUC_lexicon_tags_37 },
    { 0x7f4a9544, SUC_lexicon_tags_11 },
    { 0xddca9545, SUC_lexicon_tags_23 },
    { 0x7aa9546, SUC_lexicon_tags_16 },
    { 0x98ca9546, SUC_lexicon_tags_38 },
    { 0x25aa9548, SUC_lexicon_tags_49 },
    { 0x36ca9546, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42a9550, SUC_lexicon_tags_20 },
    { 0xab0a9550, SUC_lexicon_tags_1 },
    { 0x3cea9552, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3aa9557, SUC_lexicon_tags_25 },
    { 0x39ea9557, SUC_lexicon_tags_37 },
    { 0xd98a9559, SUC_lexicon_tags_15 },
    { 0x62ea9559, SUC_lexicon_tags_2 },
    { 0x2b6a955b, SUC_lexicon_tags_57 },
    { 0x5bea955c, SUC_lexicon_tags_27 },
    { 0x9e8a9559, SUC_lexicon_tags_12 },
    { 0x348a955c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4a9562, SUC_lexicon_tags_49 },
    { 0xb48a9562, SUC_lexicon_tags_7 },
    { 0xd6a9563, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aa956a, SUC_lexicon_tags_31 },
    { 0x7faa956b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1baa956d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a4a956f, SUC_lexicon_tags_11 },
    { 0xb2aa956f, SUC_lexicon_tags_28 },
    { 0x134a9571, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aea9574, SUC_lexicon_tags_8 },
    { 0xcdea9574, SUC_lexicon_tags_47 },
    { 0xeca9574, SUC_lexicon_tags_14 },
    { 0xfb4a9577, SUC_lexicon_tags_55 },
    { 0x408a9577, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6a957c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x140a957e, SUC_lexicon_tags_47 },
    { 0xf2aa957f, SUC_lexicon_tags_7 },
    { 0xd2aa9580, SUC_lexicon_tags_61 },
    { 0xe72a9580, SUC_lexicon_tags_23 },
    { 0x740a9582, SUC_lexicon_tags_27 },
    { 0xbbca9582, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaa958f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb12a9591, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xebea9593, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88a959a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4aca959c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x200a959e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baa95a2, SUC_lexicon_tags_6 },
    { 0xd76a95a3, SUC_lexicon_tags_17 },
    { 0xd10a95a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0a95a8, SUC_lexicon_tags_11 },
    { 0xaaa95a9, SUC_lexicon_tags_45 },
    { 0x436a95a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9f8a95ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ca95b0, SUC_lexicon_tags_17 },
    { 0x1a6a95b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57ea95b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462a95b8, SUC_lexicon_tags_7 },
    { 0x426a95b8, SUC_lexicon_tags_47 },
    { 0x462a95ba, SUC_lexicon_tags_11 },
    { 0xeb0a95b9, SUC_lexicon_tags_1 },
    { 0x52ea95bc, SUC_lexicon_tags_17 },
    { 0x2b8a95ba, SUC_lexicon_tags_28 },
    { 0x682a95be, SUC_lexicon_tags_57 },
    { 0x728a95ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638a95c4, SUC_lexicon_tags_37 },
    { 0x3ca95c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x976a95c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x430a95cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5aa95cd, SUC_lexicon_tags_11 },
    { 0x7a6a95cd, SUC_lexicon_tags_7 },
    { 0x97ca95cd, SUC_lexicon_tags_17 },
    { 0x5a6a95d0, SUC_lexicon_tags_2 },
    { 0x406a95cf, SUC_lexicon_tags_2 },
    { 0x3f4a95d0, SUC_lexicon_tags_17 },
    { 0x270a95cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256a95dd, SUC_lexicon_tags_79 },
    { 0x992a95dd, SUC_lexicon_tags_17 },
    { 0x80a95de, SUC_lexicon_tags_33 },
    { 0xbbaa95de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb1ea95e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6a95e5, SUC_lexicon_tags_1 },
    { 0xafea95e5, SUC_lexicon_tags_1 },
    { 0xdcaa95e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5aa95ea, SUC_lexicon_tags_15 },
    { 0x996a95ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc4a95ed, SUC_lexicon_tags_38 },
    { 0xdb0a95ee, SUC_lexicon_tags_25 },
    { 0x7e0a95ef, SUC_lexicon_tags_30 },
    { 0x974a95ef, SUC_lexicon_tags_7 },
    { 0xc56a95f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820a95f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876a95fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66aa9600, SUC_lexicon_tags_63 },
    { 0x38aa9601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5aa9603, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xddaa9605, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x632a9608, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284a960d, SUC_lexicon_tags_33 },
    { 0xac4a960e, SUC_lexicon_tags_21 },
    { 0x62a960f, SUC_lexicon_tags_79 },
    { 0x250a9610, SUC_lexicon_tags_37 },
    { 0x98a9610, SUC_lexicon_tags_11 },
    { 0xc02a9612, SUC_lexicon_tags_66 },
    { 0xcfea9613, SUC_lexicon_tags_12 },
    { 0xcc8a9610, SUC_lexicon_tags_62 },
    { 0xeb4a9612, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8a9618, SUC_lexicon_tags_220 },
    { 0xa96a9619, SUC_lexicon_tags_37 },
    { 0x26a961a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aa961d, SUC_lexicon_tags_17 },
    { 0x906a961e, SUC_lexicon_tags_6 },
    { 0x188a961e, SUC_lexicon_tags_19 },
    { 0x936a9620, SUC_lexicon_tags_37 },
    { 0x27ea9621, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc90a9623, SUC_lexicon_tags_61 },
    { 0x3c0a9624, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a9628, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffea962c, SUC_lexicon_tags_2 },
    { 0xe58a962d, SUC_lexicon_tags_7 },
    { 0xd38a962d, SUC_lexicon_tags_8 },
    { 0x654a962f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132a9633, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc8ea9635, SUC_lexicon_tags_19 },
    { 0x764a9636, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0a963a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x230a963c, SUC_lexicon_tags_1 },
    { 0xf3ca963d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x95ea963f, SUC_lexicon_tags_12 },
    { 0xe84a963f, SUC_lexicon_tags_1 },
    { 0xa6aa9640, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x598a9643, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90a9647, SUC_lexicon_tags_2 },
    { 0xfcaa9648, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5aa964c, SUC_lexicon_tags_65 },
    { 0x602a964c, SUC_lexicon_tags_56 },
    { 0xe6ea964c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e8a9650, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ca9654, SUC_lexicon_tags_23 },
    { 0x404a9655, SUC_lexicon_tags_14 },
    { 0xd28a9656, SUC_lexicon_tags_29 },
    { 0x7bca9655, SUC_lexicon_tags_42 },
    { 0x5baa9656, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ea965b, SUC_lexicon_tags_45 },
    { 0xc10a965c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc18a965f, SUC_lexicon_tags_27 },
    { 0x7e6a9660, SUC_lexicon_tags_2 },
    { 0x1f8a9661, SUC_lexicon_tags_7 },
    { 0xc06a9661, SUC_lexicon_tags_37 },
    { 0x158a9661, SUC_lexicon_tags_65 },
    { 0x106a9662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56a9667, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ca966a, SUC_lexicon_tags_23 },
    { 0xec4a966a, SUC_lexicon_tags_63 },
    { 0x946a966c, SUC_lexicon_tags_12 },
    { 0xaa966b, SUC_lexicon_tags_33 },
    { 0x436a966e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a9673, SUC_lexicon_tags_11 },
    { 0x3f8a9674, SUC_lexicon_tags_11 },
    { 0x272a9673, SUC_lexicon_tags_11 },
    { 0xe4ea9676, SUC_lexicon_tags_66 },
    { 0x668a9676, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88a9681, SUC_lexicon_tags_15 },
    { 0xd72a9681, SUC_lexicon_tags_23 },
    { 0x89aa9683, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ca9688, SUC_lexicon_tags_11 },
    { 0x86ea9689, SUC_lexicon_tags_39 },
    { 0xda2a9688, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca968e, SUC_lexicon_tags_21 },
    { 0x768a968e, SUC_lexicon_tags_17 },
    { 0x4e4a968e, SUC_lexicon_tags_14 },
    { 0x332a9691, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x730a9693, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0a969a, SUC_lexicon_tags_11 },
    { 0x38a969b, SUC_lexicon_tags_2 },
    { 0xc8ea969b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04a969f, SUC_lexicon_tags_11 },
    { 0x4a2a96a0, SUC_lexicon_tags_39 },
    { 0x2ca96a1, SUC_lexicon_tags_46 },
    { 0x7b8a96a2, SUC_lexicon_tags_43 },
    { 0x20ca96a3, SUC_lexicon_tags_25 },
    { 0x356a96a3, SUC_lexicon_tags_25 },
    { 0x2aa96a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2a96a8, SUC_lexicon_tags_18 },
    { 0x18ea96a9, SUC_lexicon_tags_77 },
    { 0x1aca96a9, SUC_lexicon_tags_14 },
    { 0x636a96ab, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204a96af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4a6a96b1, SUC_lexicon_tags_2 },
    { 0x590a96b1, SUC_lexicon_tags_17 },
    { 0x182a96b3, SUC_lexicon_tags_12 },
    { 0x40ea96b3, SUC_lexicon_tags_17 },
    { 0x55aa96b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96ca96b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ea96bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ea96c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24a96cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70a96d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8a96d4, SUC_lexicon_tags_23 },
    { 0x622a96d5, SUC_lexicon_tags_45 },
    { 0xca4a96d6, SUC_lexicon_tags_83 },
    { 0x7f2a96d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a96dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xacea96df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd20a96e1, SUC_lexicon_tags_1 },
    { 0xbb0a96e2, SUC_lexicon_tags_45 },
    { 0xd46a96e2, SUC_lexicon_tags_37 },
    { 0xf00a96e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ea96e7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x94a96e9, SUC_lexicon_tags_6 },
    { 0xd44a96e9, SUC_lexicon_tags_8 },
    { 0xceea96e9, SUC_lexicon_tags_33 },
    { 0x78ea96ec, SUC_lexicon_tags_39 },
    { 0x8f8a96eb, SUC_lexicon_tags_33 },
    { 0x58ea96ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d4a96f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x706a96f2, SUC_lexicon_tags_46 },
    { 0x142a96f3, SUC_lexicon_tags_11 },
    { 0x402a96f4, SUC_lexicon_tags_7 },
    { 0x782a96f5, SUC_lexicon_tags_45 },
    { 0xfdaa96f6, SUC_lexicon_tags_10 },
    { 0x76aa96f6, SUC_lexicon_tags_35 },
    { 0x448a96f8, SUC_lexicon_tags_14 },
    { 0x3c0a96f2, SUC_lexicon_tags_48 },
    { 0x4a0a96f6, SUC_lexicon_tags_117 },
    { 0x37ea96fb, SUC_lexicon_tags_7 },
    { 0xdd4a96fc, SUC_lexicon_tags_7 },
    { 0x722a96fd, SUC_lexicon_tags_26 },
    { 0x6fca96fe, SUC_lexicon_tags_2 },
    { 0x92a96ff, SUC_lexicon_tags_37 },
    { 0xfa2a9700, SUC_lexicon_tags_15 },
    { 0xc16a9700, SUC_lexicon_tags_55 },
    { 0x3caa9701, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94aa9704, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbf6a9706, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb02a9708, SUC_lexicon_tags_197 },
    { 0x5b6a9708, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ea970d, SUC_lexicon_tags_0 },
    { 0x25ea970e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd5ca9710, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31aa971c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496a9726, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc3aa9728, SUC_lexicon_tags_37 },
    { 0x160a9728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8a972c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fea9731, SUC_lexicon_tags_50 },
    { 0x718a9731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeea9737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaa973c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4aa973e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ea9742, SUC_lexicon_tags_26 },
    { 0x2aea9743, SUC_lexicon_tags_6 },
    { 0xd92a9743, SUC_lexicon_tags_63 },
    { 0x80ca9745, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31aa9748, SUC_lexicon_tags_45 },
    { 0xef6a9749, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12ea974b, SUC_lexicon_tags_11 },
    { 0x984a974c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718a9752, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbea9755, SUC_lexicon_tags_27 },
    { 0xe1ca9756, SUC_lexicon_tags_1 },
    { 0xd0aa9756, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0a975a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dea975d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca9761, SUC_lexicon_tags_14 },
    { 0xbfea9762, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ea9765, SUC_lexicon_tags_45 },
    { 0x77aa9766, SUC_lexicon_tags_46 },
    { 0x9aca9767, SUC_lexicon_tags_17 },
    { 0x442a9766, SUC_lexicon_tags_7 },
    { 0x42a9769, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe34a976b, SUC_lexicon_tags_25 },
    { 0x672a976b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe64a976e, SUC_lexicon_tags_14 },
    { 0x804a976f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc6a9771, SUC_lexicon_tags_63 },
    { 0xa48a9771, SUC_lexicon_tags_7 },
    { 0xa58a9772, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6a9776, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16a977d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87aa977f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8a9783, SUC_lexicon_tags_77 },
    { 0xa00a9784, SUC_lexicon_tags_43 },
    { 0xad4a9783, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x212a9787, SUC_lexicon_tags_10 },
    { 0x2e0a9787, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xa78a978a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228a978e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb28a9790, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x934a9792, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ea9799, SUC_lexicon_tags_14 },
    { 0x80aa979a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364a979f, SUC_lexicon_tags_174 },
    { 0xe92a979f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702a97a4, SUC_lexicon_tags_7 },
    { 0xa7ca97a4, SUC_lexicon_tags_27 },
    { 0x2a0a97a6, SUC_lexicon_tags_47 },
    { 0xcd8a97a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4a97ab, SUC_lexicon_tags_62 },
    { 0x37ca97ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ca97af, SUC_lexicon_tags_17 },
    { 0x4b4a97af, SUC_lexicon_tags_11 },
    { 0xddca97af, SUC_lexicon_tags_37 },
    { 0xb64a97b2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51aa97b7, SUC_lexicon_tags_1 },
    { 0xdd8a97b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0a97bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf6a97bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2a97c5, SUC_lexicon_tags_60 },
    { 0x134a97c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ca97cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6ea97ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4a97d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x272a97d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x60aa97d7, SUC_lexicon_tags_14 },
    { 0xabaa97d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f8a97da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564a97e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf5ea97e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ea97e7, SUC_lexicon_tags_11 },
    { 0x620a97e8, SUC_lexicon_tags_37 },
    { 0x6ca97e9, SUC_lexicon_tags_23 },
    { 0xf5ca97e9, SUC_lexicon_tags_2 },
    { 0xcaa97eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeca97ee, SUC_lexicon_tags_28 },
    { 0x934a97ef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baa97f4, SUC_lexicon_tags_19 },
    { 0xd76a97f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ca97fa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44a9801, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a980b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44a980d, SUC_lexicon_tags_2 },
    { 0xb68a980d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdea9812, SUC_lexicon_tags_25 },
    { 0xb7ea9813, SUC_lexicon_tags_37 },
    { 0x848a9814, SUC_lexicon_tags_16 },
    { 0xfe2a9813, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc40a9817, SUC_lexicon_tags_44 },
    { 0x1c6a9817, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26a981b, SUC_lexicon_tags_28 },
    { 0xf0a981c, SUC_lexicon_tags_0 },
    { 0x7dca981d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656a9825, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0a9829, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8a982c, SUC_lexicon_tags_12 },
    { 0xe42a982d, SUC_lexicon_tags_10 },
    { 0x32aa982d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aa9834, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0a9837, SUC_lexicon_tags_65 },
    { 0x41aa9837, SUC_lexicon_tags_12 },
    { 0xd2ea9837, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94a983d, SUC_lexicon_tags_2 },
    { 0xda6a983e, SUC_lexicon_tags_25 },
    { 0x8dea983f, SUC_lexicon_tags_2 },
    { 0xe4ea9840, SUC_lexicon_tags_201 },
    { 0x4a983d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596a9845, SUC_lexicon_tags_25 },
    { 0x794a9845, SUC_lexicon_tags_27 },
    { 0xfdea9847, SUC_lexicon_tags_17 },
    { 0x96a9848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x178a984b, SUC_lexicon_tags_63 },
    { 0xb9ea984c, SUC_lexicon_tags_17 },
    { 0x67aa984c, SUC_lexicon_tags_15 },
    { 0xab0a984d, SUC_lexicon_tags_47 },
    { 0x62ea984f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0a9854, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2a9857, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70ea9859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8a985c, SUC_lexicon_tags_1 },
    { 0x5baa985c, SUC_lexicon_tags_1 },
    { 0xe40a985e, SUC_lexicon_tags_2 },
    { 0x27ea985f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81ea9861, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaa9864, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70a9867, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86a986a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe72a986c, SUC_lexicon_tags_1 },
    { 0x21ca986d, SUC_lexicon_tags_27 },
    { 0x9a8a986e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ea9872, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ca9875, SUC_lexicon_tags_33 },
    { 0x134a9875, SUC_lexicon_tags_25 },
    { 0xd2aa9877, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x350a987a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x708a987c, SUC_lexicon_tags_33 },
    { 0x1eaa987d, SUC_lexicon_tags_29 },
    { 0x3a2a987d, SUC_lexicon_tags_15 },
    { 0xdca987d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130a9885, SUC_lexicon_tags_11 },
    { 0xf30a9885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68a988a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3daa988c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754a9891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x802a9893, SUC_lexicon_tags_1 },
    { 0xa2aa9893, SUC_lexicon_tags_17 },
    { 0x5b2a9894, SUC_lexicon_tags_31 },
    { 0x3b0a9893, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aa989b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x824a989d, SUC_lexicon_tags_17 },
    { 0xed6a989e, SUC_lexicon_tags_4 },
    { 0xbb4a989f, SUC_lexicon_tags_38 },
    { 0x438a98a0, SUC_lexicon_tags_11 },
    { 0x730a98a0, SUC_lexicon_tags_2 },
    { 0xd42a98a2, SUC_lexicon_tags_37 },
    { 0x1b0a989e, SUC_lexicon_tags_8 },
    { 0xd8ea98a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8a98a7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x76ca98a9, SUC_lexicon_tags_2 },
    { 0x968a98aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x302a98ac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daa98b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4a98b3, SUC_lexicon_tags_8 },
    { 0xc5ea98b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a98b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ea98bc, SUC_lexicon_tags_8 },
    { 0x3a4a98bd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa0ea98bf, SUC_lexicon_tags_15 },
    { 0x2a98bf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5fea98c2, SUC_lexicon_tags_28 },
    { 0xf16a98c2, SUC_lexicon_tags_25 },
    { 0x6caa98c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76a98c8, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaa98cc, SUC_lexicon_tags_2 },
    { 0xf7ea98cd, SUC_lexicon_tags_43 },
    { 0x946a98ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496a98d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4aa98d5, SUC_lexicon_tags_63 },
    { 0xbeaa98d5, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4a98da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x230a98dc, SUC_lexicon_tags_25 },
    { 0xa84a98dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2a98e1, SUC_lexicon_tags_0 },
    { 0x5d2a98e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa9aa98e4, SUC_lexicon_tags_0 },
    { 0x848a98e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcea98e8, SUC_lexicon_tags_15 },
    { 0x83aa98e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aea98ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2a98ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2caa98f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x908a98f3, SUC_lexicon_tags_25 },
    { 0x7d6a98f4, SUC_lexicon_tags_37 },
    { 0x422a98f3, SUC_lexicon_tags_11 },
    { 0xa4a98f6, SUC_lexicon_tags_25 },
    { 0xe9ea98f7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718a9901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a6a9903, SUC_lexicon_tags_7 },
    { 0x91aa9904, SUC_lexicon_tags_9 },
    { 0xcfca9905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x748a9908, SUC_lexicon_tags_11 },
    { 0x840a9908, SUC_lexicon_tags_8 },
    { 0xb18a990a, SUC_lexicon_tags_15 },
    { 0xa44a990a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6a990e, SUC_lexicon_tags_66 },
    { 0xf0ca990f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8a9913, SUC_lexicon_tags_34 },
    { 0x43aa9914, SUC_lexicon_tags_7 },
    { 0xeaa9915, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbea9918, SUC_lexicon_tags_5 },
    { 0xb48a9918, SUC_lexicon_tags_38 },
    { 0xe90a991a, SUC_lexicon_tags_11 },
    { 0x730a991b, SUC_lexicon_tags_19 },
    { 0x76ea991a, SUC_lexicon_tags_12 },
    { 0x6d0a991a, SUC_lexicon_tags_37 },
    { 0x7cca991c, SUC_lexicon_tags_6 },
    { 0xa70a9919, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x914a9921, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362a9926, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24aa992a, SUC_lexicon_tags_6 },
    { 0x744a992b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44a9931, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aa9936, SUC_lexicon_tags_25 },
    { 0x65ca9937, SUC_lexicon_tags_49 },
    { 0xe46a9938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x914a993a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d4a993c, SUC_lexicon_tags_46 },
    { 0x9a8a993c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4a9941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x732a9944, SUC_lexicon_tags_346 },
    { 0x776a9945, SUC_lexicon_tags_29 },
    { 0x38ea9945, SUC_lexicon_tags_42 },
    { 0xe1ea9947, SUC_lexicon_tags_9 },
    { 0x50ca9948, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4aa994c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cca994f, SUC_lexicon_tags_17 },
    { 0x318a9950, SUC_lexicon_tags_60 },
    { 0x564a9950, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656a9958, SUC_lexicon_tags_15 },
    { 0x8c2a9959, SUC_lexicon_tags_1 },
    { 0x3f0a9959, SUC_lexicon_tags_26 },
    { 0x86ea995b, SUC_lexicon_tags_4 },
    { 0x8b2a9958, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122a9962, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0aa9965, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36a996a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4a996e, SUC_lexicon_tags_18 },
    { 0xa3ca996e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ca9974, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eea9978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dea997f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ea9986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc0a9988, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2a998d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78a9992, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5c8a9994, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x888a9996, SUC_lexicon_tags_7 },
    { 0xa4ca9997, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa9ca9999, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0a999c, SUC_lexicon_tags_17 },
    { 0xea2a999c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaa99a2, SUC_lexicon_tags_11 },
    { 0x99aa99a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556a99aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xadea99ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb80a99ae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354a99b2, SUC_lexicon_tags_15 },
    { 0x688a99b3, SUC_lexicon_tags_23 },
    { 0x9baa99b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8a99b9, SUC_lexicon_tags_37 },
    { 0x2b6a99ba, SUC_lexicon_tags_17 },
    { 0xbaa99ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68a99be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x732a99c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4a99cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaa99cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x920a99d2, SUC_lexicon_tags_26 },
    { 0x4daa99d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a99d7, SUC_lexicon_tags_45 },
    { 0x41ca99d7, SUC_lexicon_tags_45 },
    { 0xffea99d9, SUC_lexicon_tags_15 },
    { 0xd08a99da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x618a99dc, SUC_lexicon_tags_57 },
    { 0x2bea99dc, SUC_lexicon_tags_26 },
    { 0x7b0a99dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2a99e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08a99e5, SUC_lexicon_tags_1 },
    { 0xe68a99e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe14a99e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x224a99ea, SUC_lexicon_tags_47 },
    { 0xc48a99ea, SUC_lexicon_tags_26 },
    { 0x242a99ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6a99f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb28a99f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768a99fa, SUC_lexicon_tags_2 },
    { 0x44aa99fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94a99fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84a9a02, SUC_lexicon_tags_15 },
    { 0x9d2a9a03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8a9a07, SUC_lexicon_tags_0 },
    { 0xba0a9a08, SUC_lexicon_tags_2 },
    { 0x73ca9a09, SUC_lexicon_tags_7 },
    { 0x68ca9a07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402a9a13, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x134a9a15, SUC_lexicon_tags_23 },
    { 0x3c8a9a15, SUC_lexicon_tags_65 },
    { 0xc52a9a17, SUC_lexicon_tags_7 },
    { 0x95aa9a16, SUC_lexicon_tags_48 },
    { 0x254a9a19, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54a9a1d, SUC_lexicon_tags_11 },
    { 0xcd2a9a1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dea9a22, SUC_lexicon_tags_33 },
    { 0xb10a9a23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7ea9a25, SUC_lexicon_tags_1 },
    { 0x204a9a25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x346a9a29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce2a9a2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ea9a2f, SUC_lexicon_tags_7 },
    { 0xffaa9a2f, SUC_lexicon_tags_17 },
    { 0x9a0a9a2f, SUC_lexicon_tags_37 },
    { 0x4ea9a32, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1b2a9a34, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xca6a9a36, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x224a9a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00a9a3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faa9a42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aa9a47, SUC_lexicon_tags_37 },
    { 0x750a9a47, SUC_lexicon_tags_8 },
    { 0x66ea9a49, SUC_lexicon_tags_14 },
    { 0x898a9a49, SUC_lexicon_tags_19 },
    { 0x848a9a4b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8a9a4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6a9a53, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8a9a58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf4aa9a5a, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20a9a5e, SUC_lexicon_tags_16 },
    { 0xc58a9a5e, SUC_lexicon_tags_104 },
    { 0xbfaa9a60, SUC_lexicon_tags_1 },
    { 0x7f0a9a61, SUC_lexicon_tags_8 },
    { 0x38ea9a61, SUC_lexicon_tags_17 },
    { 0x690a9a61, SUC_lexicon_tags_17 },
    { 0x75ca9a5f, SUC_lexicon_tags_28 },
    { 0xe88a9a63, SUC_lexicon_tags_87 },
    { 0x2b2a9a66, SUC_lexicon_tags_37 },
    { 0x1fca9a63, SUC_lexicon_tags_37 },
    { 0xc60a9a67, SUC_lexicon_tags_11 },
    { 0xf5aa9a69, SUC_lexicon_tags_17 },
    { 0x8dea9a65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68a9a71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfaa9a73, SUC_lexicon_tags_3 },
    { 0xfb2a9a74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8a9a77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x668a9a79, SUC_lexicon_tags_47 },
    { 0x864a9a7a, SUC_lexicon_tags_1 },
    { 0xef0a9a7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39aa9a7d, SUC_lexicon_tags_113 },
    { 0xb08a9a7e, SUC_lexicon_tags_27 },
    { 0x226a9a7f, SUC_lexicon_tags_1 },
    { 0x5aa9a7e, SUC_lexicon_tags_12 },
    { 0x72ea9a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb30a9a83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4a9a8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13aa9a95, SUC_lexicon_tags_14 },
    { 0xecaa9a95, SUC_lexicon_tags_61 },
    { 0xb7aa9a95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6a9a9b, SUC_lexicon_tags_39 },
    { 0x524a9a9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaa9aa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8a9aa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06a9aa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe52a9aab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0a9aae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a9ab0, SUC_lexicon_tags_34 },
    { 0x998a9ab0, SUC_lexicon_tags_7 },
    { 0xbaca9ab0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c2a9ab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2a9abd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8a9ac2, SUC_lexicon_tags_29 },
    { 0x154a9ac2, SUC_lexicon_tags_15 },
    { 0x1cca9ac4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60a9ace, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84aa9ad0, SUC_lexicon_tags_17 },
    { 0xff0a9ad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe86a9ad3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4ea9ad5, SUC_lexicon_tags_15 },
    { 0x3fca9ad5, SUC_lexicon_tags_11 },
    { 0x4c6a9ad7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22a9adb, SUC_lexicon_tags_25 },
    { 0xf88a9adc, SUC_lexicon_tags_3 },
    { 0x9eea9adc, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0a9ae0, SUC_lexicon_tags_11 },
    { 0xc52a9ae1, SUC_lexicon_tags_1 },
    { 0x652a9ae1, SUC_lexicon_tags_65 },
    { 0xed8a9ae3, SUC_lexicon_tags_43 },
    { 0x574a9ae4, SUC_lexicon_tags_23 },
    { 0x74ca9ae5, SUC_lexicon_tags_7 },
    { 0x3a0a9ae5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ea9ae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eca9aec, SUC_lexicon_tags_12 },
    { 0xd10a9aec, SUC_lexicon_tags_15 },
    { 0x626a9aee, SUC_lexicon_tags_38 },
    { 0x97ca9aef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726a9af4, SUC_lexicon_tags_7 },
    { 0xcdaa9af4, SUC_lexicon_tags_38 },
    { 0x7bea9af6, SUC_lexicon_tags_15 },
    { 0xb2a9af7, SUC_lexicon_tags_11 },
    { 0xcf6a9af5, SUC_lexicon_tags_25 },
    { 0x15aa9af8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910a9afe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ca9b04, SUC_lexicon_tags_38 },
    { 0x854a9b04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x714a9b07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38a9b0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86a9b0f, SUC_lexicon_tags_19 },
    { 0xab6a9b10, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfaca9b12, SUC_lexicon_tags_17 },
    { 0x968a9b12, SUC_lexicon_tags_85 },
    { 0xde2a9b14, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbca9b19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x286a9b1c, SUC_lexicon_tags_107 },
    { 0x33ea9b1d, SUC_lexicon_tags_14 },
    { 0xc4a9b1d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502a9b29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2a9b2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x522a9b2e, SUC_lexicon_tags_37 },
    { 0x132a9b2f, SUC_lexicon_tags_37 },
    { 0x5e2a9b2f, SUC_lexicon_tags_70 },
    { 0x4ca9b30, SUC_lexicon_tags_17 },
    { 0x5faa9b32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe12a9b34, SUC_lexicon_tags_16 },
    { 0x5bea9b35, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18a9b38, SUC_lexicon_tags_34 },
    { 0x71ca9b39, SUC_lexicon_tags_68 },
    { 0x248a9b39, SUC_lexicon_tags_14 },
    { 0x71ca9b3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46a9b45, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xff8a9b47, SUC_lexicon_tags_28 },
    { 0x83ca9b47, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6a9b4b, SUC_lexicon_tags_26 },
    { 0x8a8a9b4c, SUC_lexicon_tags_55 },
    { 0x748a9b4d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaca9b52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cca9b59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ca9b5c, SUC_lexicon_tags_11 },
    { 0xd82a9b5d, SUC_lexicon_tags_49 },
    { 0xa3aa9b5e, SUC_lexicon_tags_8 },
    { 0x5baa9b5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeea9b64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe36a9b66, SUC_lexicon_tags_37 },
    { 0xa60a9b67, SUC_lexicon_tags_25 },
    { 0x262a9b67, SUC_lexicon_tags_16 },
    { 0xb14a9b69, SUC_lexicon_tags_16 },
    { 0x36ea9b69, SUC_lexicon_tags_17 },
    { 0xf9ca9b6b, SUC_lexicon_tags_6 },
    { 0xe56a9b6c, SUC_lexicon_tags_49 },
    { 0x9b4a9b6d, SUC_lexicon_tags_25 },
    { 0x22a9b6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6faa9b71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeea9b77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x214a9b79, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x700a9b7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x12a9b7e, SUC_lexicon_tags_17 },
    { 0x600a9b7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc20a9b81, SUC_lexicon_tags_12 },
    { 0x5c8a9b82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ca9b87, SUC_lexicon_tags_25 },
    { 0xdd6a9b88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaa9b8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aa9b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34a9b93, SUC_lexicon_tags_55 },
    { 0x728a9b94, SUC_lexicon_tags_265 },
    { 0xa74a9b94, SUC_lexicon_tags_11 },
    { 0xacca9b96, SUC_lexicon_tags_21 },
    { 0xd2ca9b96, SUC_lexicon_tags_1 },
    { 0x3ca9b95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46a9b9c, SUC_lexicon_tags_25 },
    { 0x96aa9b9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30a9ba6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa10a9ba8, SUC_lexicon_tags_25 },
    { 0xc7ea9ba8, SUC_lexicon_tags_17 },
    { 0xbb2a9baa, SUC_lexicon_tags_87 },
    { 0x5cca9bab, SUC_lexicon_tags_43 },
    { 0x7d0a9ba8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ea9bb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73aa9bbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870a9bc1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6d2a9bc3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1e8a9bc5, SUC_lexicon_tags_14 },
    { 0x1b6a9bc6, SUC_lexicon_tags_63 },
    { 0x9dea9bc6, SUC_lexicon_tags_28 },
    { 0xc54a9bc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9aa9bca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4a9bcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386a9bd2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aa9bd5, SUC_lexicon_tags_11 },
    { 0xa28a9bd6, SUC_lexicon_tags_25 },
    { 0xfaca9bd7, SUC_lexicon_tags_39 },
    { 0xe90a9bd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xceca9bdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb2a9bdd, SUC_lexicon_tags_11 },
    { 0x118a9bde, SUC_lexicon_tags_11 },
    { 0x55aa9bdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x268a9be1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2a9be5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4a9bec, SUC_lexicon_tags_15 },
    { 0x844a9bed, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58a9bf3, SUC_lexicon_tags_62 },
    { 0xb84a9bf4, SUC_lexicon_tags_1 },
    { 0xcd4a9bf4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720a9bfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522a9c01, SUC_lexicon_tags_28 },
    { 0x5c4a9c02, SUC_lexicon_tags_17 },
    { 0x8fea9c03, SUC_lexicon_tags_14 },
    { 0x318a9c04, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x858a9c06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ea9c0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaa9c0e, SUC_lexicon_tags_39 },
    { 0xc1aa9c0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0a9c13, SUC_lexicon_tags_49 },
    { 0x34a9c13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92a9c1a, SUC_lexicon_tags_2 },
    { 0xd22a9c1b, SUC_lexicon_tags_27 },
    { 0xf4aa9c1c, SUC_lexicon_tags_28 },
    { 0x904a9c1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed0a9c1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54a9c22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68a9c28, SUC_lexicon_tags_12 },
    { 0x1d8a9c28, SUC_lexicon_tags_1 },
    { 0x41ea9c2a, SUC_lexicon_tags_11 },
    { 0x336a9c2b, SUC_lexicon_tags_63 },
    { 0xd1ea9c2c, SUC_lexicon_tags_11 },
    { 0x130a9c2d, SUC_lexicon_tags_11 },
    { 0xea0a9c2c, SUC_lexicon_tags_15 },
    { 0x98aa9c2f, SUC_lexicon_tags_63 },
    { 0x8faa9c2b, SUC_lexicon_tags_0 },
    { 0x31aa9c30, SUC_lexicon_tags_8 },
    { 0x722a9c2c, SUC_lexicon_tags_7 },
    { 0x36aa9c31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40ea9c35, SUC_lexicon_tags_17 },
    { 0xd74a9c36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ea9c39, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x30a9c3b, SUC_lexicon_tags_37 },
    { 0x7b4a9c3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x55aa9c3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb0a9c41, SUC_lexicon_tags_1 },
    { 0x208a9c42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5aca9c44, SUC_lexicon_tags_0 },
    { 0x2f8a9c45, SUC_lexicon_tags_45 },
    { 0x31ca9c46, SUC_lexicon_tags_37 },
    { 0xb2ea9c47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ca9c4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe58a9c4c, SUC_lexicon_tags_2 },
    { 0x36ca9c4d, SUC_lexicon_tags_1 },
    { 0xaf6a9c4e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944a9c54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ca9c57, SUC_lexicon_tags_2 },
    { 0x6a2a9c58, SUC_lexicon_tags_2 },
    { 0x462a9c58, SUC_lexicon_tags_80 },
    { 0x4d2a9c58, SUC_lexicon_tags_45 },
    { 0x8bca9c5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00a9c60, SUC_lexicon_tags_19 },
    { 0xbeca9c60, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc88a9c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602a9c6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25aa9c70, SUC_lexicon_tags_25 },
    { 0x6aea9c71, SUC_lexicon_tags_55 },
    { 0x44ca9c71, SUC_lexicon_tags_25 },
    { 0x908a9c73, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb0a9c75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8a9c7a, SUC_lexicon_tags_7 },
    { 0x44aa9c7a, SUC_lexicon_tags_65 },
    { 0xf48a9c7a, SUC_lexicon_tags_35 },
    { 0xcc2a9c7c, SUC_lexicon_tags_1 },
    { 0x9a2a9c7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x700a9c81, SUC_lexicon_tags_1 },
    { 0x532a9c82, SUC_lexicon_tags_65 },
    { 0xc92a9c83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8a9c86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccca9c89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2bea9c8b, SUC_lexicon_tags_17 },
    { 0x960a9c8c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6a9c91, SUC_lexicon_tags_25 },
    { 0x24aa9c92, SUC_lexicon_tags_15 },
    { 0xc9aa9c91, SUC_lexicon_tags_26 },
    { 0x492a9c94, SUC_lexicon_tags_2 },
    { 0x1cca9c94, SUC_lexicon_tags_25 },
    { 0xeeea9c94, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28a9c9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63ca9c9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ea9ca1, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0a9cae, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xadaa9cb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02a9cb3, SUC_lexicon_tags_13 },
    { 0x1e6a9cb4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb20a9cb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ea9cbd, SUC_lexicon_tags_11 },
    { 0x440a9cbd, SUC_lexicon_tags_2 },
    { 0xc76a9cbd, SUC_lexicon_tags_29 },
    { 0x182a9cc0, SUC_lexicon_tags_31 },
    { 0x8dea9cc1, SUC_lexicon_tags_7 },
    { 0x2daa9cc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0a9cc7, SUC_lexicon_tags_39 },
    { 0x212a9cc8, SUC_lexicon_tags_43 },
    { 0xaa8a9cc9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fca9ccc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x182a9cce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fea9cd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142a9cd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4a9cdc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x486a9cdf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928a9ce6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x300a9ce8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdea9ceb, SUC_lexicon_tags_17 },
    { 0x63ea9ceb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592a9cf1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16a9cf6, SUC_lexicon_tags_23 },
    { 0xb84a9cf7, SUC_lexicon_tags_65 },
    { 0x12ea9cf7, SUC_lexicon_tags_4 },
    { 0xbf8a9cf9, SUC_lexicon_tags_1 },
    { 0xa8ca9cf8, SUC_lexicon_tags_15 },
    { 0x24ea9cfb, SUC_lexicon_tags_38 },
    { 0x80ea9cfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32a9cff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ca9d06, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x758a9d08, SUC_lexicon_tags_25 },
    { 0xe68a9d09, SUC_lexicon_tags_14 },
    { 0x18a9d09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca8a9d0c, SUC_lexicon_tags_38 },
    { 0xe4ea9d0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4a9d10, SUC_lexicon_tags_11 },
    { 0x2daa9d10, SUC_lexicon_tags_33 },
    { 0xa94a9d10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x382a9d15, SUC_lexicon_tags_47 },
    { 0xa50a9d16, SUC_lexicon_tags_21 },
    { 0xf0a9d17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x41ea9d19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36ea9d1b, SUC_lexicon_tags_60 },
    { 0xaaaa9d1b, SUC_lexicon_tags_14 },
    { 0x87ea9d1c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x204a9d20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2a9d25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc96a9d27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x216a9d29, SUC_lexicon_tags_7 },
    { 0xd34a9d2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5faa9d2c, SUC_lexicon_tags_56 },
    { 0xadca9d2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8a9d30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8a9d32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x582a9d34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b0a9d36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa1ca9d38, SUC_lexicon_tags_17 },
    { 0x44aa9d39, SUC_lexicon_tags_60 },
    { 0xce6a9d39, SUC_lexicon_tags_47 },
    { 0x608a9d3b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a9d3e, SUC_lexicon_tags_19 },
    { 0x7f8a9d3f, SUC_lexicon_tags_25 },
    { 0x1f0a9d40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x168a9d43, SUC_lexicon_tags_25 },
    { 0xf22a9d44, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xde8a9d46, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4a9d4a, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x340a9d4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x240a9d4e, SUC_lexicon_tags_22 },
    { 0xc98a9d4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aa9d55, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x374a9d58, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2a9d5c, SUC_lexicon_tags_55 },
    { 0x164a9d5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa28a9d5f, SUC_lexicon_tags_2 },
    { 0x398a9d60, SUC_lexicon_tags_19 },
    { 0xb48a9d60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ea9d64, SUC_lexicon_tags_10 },
    { 0x2b8a9d65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36a9d68, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2a9d6d, SUC_lexicon_tags_7 },
    { 0xf20a9d6e, SUC_lexicon_tags_29 },
    { 0x230a9d6d, SUC_lexicon_tags_24 },
    { 0x592a9d6f, SUC_lexicon_tags_14 },
    { 0x83ca9d71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9aa9d74, SUC_lexicon_tags_15 },
    { 0xa30a9d74, SUC_lexicon_tags_7 },
    { 0x334a9d74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ea9d7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9ea9d7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd86a9d7e, SUC_lexicon_tags_7 },
    { 0xfd6a9d7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6a9d82, SUC_lexicon_tags_25 },
    { 0xac0a9d83, SUC_lexicon_tags_37 },
    { 0xb7ea9d84, SUC_lexicon_tags_65 },
    { 0xdc0a9d84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbea9d88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x442a9d8b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46a9d8f, SUC_lexicon_tags_2 },
    { 0x986a9d8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbea9d94, SUC_lexicon_tags_65 },
    { 0xc7aa9d95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52a9d99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2daa9d9d, SUC_lexicon_tags_104 },
    { 0x8dca9d9d, SUC_lexicon_tags_45 },
    { 0x9e2a9d9e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x170a9da1, SUC_lexicon_tags_21 },
    { 0x39ea9da1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22a9da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490a9da9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bea9dac, SUC_lexicon_tags_1 },
    { 0xe54a9dad, SUC_lexicon_tags_66 },
    { 0x160a9dac, SUC_lexicon_tags_11 },
    { 0x418a9daf, SUC_lexicon_tags_37 },
    { 0x296a9daf, SUC_lexicon_tags_43 },
    { 0x41ea9dad, SUC_lexicon_tags_23 },
    { 0x8a2a9db2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaa9db5, SUC_lexicon_tags_31 },
    { 0x388a9db6, SUC_lexicon_tags_33 },
    { 0xa16a9db6, SUC_lexicon_tags_1 },
    { 0xd1ea9db7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ea9dbd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x46a9dbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aa9dc1, SUC_lexicon_tags_62 },
    { 0xdf4a9dc1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6a2a9dc4, SUC_lexicon_tags_1 },
    { 0x470a9dc5, SUC_lexicon_tags_43 },
    { 0xbca9dc4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02a9dca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14a9dce, SUC_lexicon_tags_37 },
    { 0x3cca9dce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x228a9dd1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ca9dd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4a9ddb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbca9de3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6a2a9de5, SUC_lexicon_tags_37 },
    { 0x434a9de5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910a9dea, SUC_lexicon_tags_37 },
    { 0xa68a9dea, SUC_lexicon_tags_57 },
    { 0xfbaa9dec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdeea9dee, SUC_lexicon_tags_12 },
    { 0x2e2a9def, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aa9df2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6a9df5, SUC_lexicon_tags_1 },
    { 0x6eea9df6, SUC_lexicon_tags_25 },
    { 0xc34a9df7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x866a9df9, SUC_lexicon_tags_12 },
    { 0x52a9dfa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598a9dff, SUC_lexicon_tags_2 },
    { 0x294a9e00, SUC_lexicon_tags_23 },
    { 0xecca9e01, SUC_lexicon_tags_0 },
    { 0xf74a9e02, SUC_lexicon_tags_21 },
    { 0x576a9e01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x528a9e05, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0a9e08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x614a9e0b, SUC_lexicon_tags_25 },
    { 0xddca9e0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652a9e11, SUC_lexicon_tags_42 },
    { 0x756a9e11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x722a9e14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaa9e1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76a9e24, SUC_lexicon_tags_1 },
    { 0x884a9e25, SUC_lexicon_tags_45 },
    { 0x9baa9e26, SUC_lexicon_tags_47 },
    { 0x6a0a9e26, SUC_lexicon_tags_33 },
    { 0x700a9e25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0a9e2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18aa9e2e, SUC_lexicon_tags_33 },
    { 0x41aa9e2f, SUC_lexicon_tags_32 },
    { 0xb78a9e30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812a9e36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ca9e3b, SUC_lexicon_tags_82 },
    { 0x3a8a9e3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0a9e3f, SUC_lexicon_tags_37 },
    { 0xcd4a9e3f, SUC_lexicon_tags_39 },
    { 0xf42a9e3f, SUC_lexicon_tags_14 },
    { 0x7c0a9e42, SUC_lexicon_tags_27 },
    { 0x2b0a9e40, SUC_lexicon_tags_77 },
    { 0x4e4a9e40, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1ea9e46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x646a9e48, SUC_lexicon_tags_2 },
    { 0xe88a9e48, SUC_lexicon_tags_1 },
    { 0x3a6a9e49, SUC_lexicon_tags_1 },
    { 0xb86a9e4b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x372a9e4d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2a9e51, SUC_lexicon_tags_1 },
    { 0x49aa9e51, SUC_lexicon_tags_49 },
    { 0x61ca9e53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50a9e5c, SUC_lexicon_tags_14 },
    { 0xda6a9e5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aea9e60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x250a9e63, SUC_lexicon_tags_1 },
    { 0xab8a9e64, SUC_lexicon_tags_29 },
    { 0xdaaa9e64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x102a9e68, SUC_lexicon_tags_19 },
    { 0x204a9e69, SUC_lexicon_tags_37 },
    { 0xc6a9e68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6a9e6d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ca9e6f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x956a9e72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bca9e74, SUC_lexicon_tags_1 },
    { 0x604a9e74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x466a9e77, SUC_lexicon_tags_25 },
    { 0x29aa9e77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22a9e7f, SUC_lexicon_tags_17 },
    { 0x398a9e80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baa9e86, SUC_lexicon_tags_91 },
    { 0x1f6a9e87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352a9e8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880a9e93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4a9e99, SUC_lexicon_tags_2 },
    { 0xebca9e9a, SUC_lexicon_tags_17 },
    { 0x3d8a9e9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeea9e9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9a2a9ea1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38a9ea4, SUC_lexicon_tags_37 },
    { 0xd58a9ea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa4a9ea7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2a9eae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82aa9eb3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634a9eba, SUC_lexicon_tags_25 },
    { 0xd3ca9ebb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8a9ec2, SUC_lexicon_tags_11 },
    { 0xcb0a9ec3, SUC_lexicon_tags_0 },
    { 0xdc8a9ec4, SUC_lexicon_tags_12 },
    { 0x360a9ec3, SUC_lexicon_tags_12 },
    { 0xcca9ec6, SUC_lexicon_tags_12 },
    { 0x61ea9ec6, SUC_lexicon_tags_45 },
    { 0x69ea9ec6, SUC_lexicon_tags_1 },
    { 0x7a0a9ec6, SUC_lexicon_tags_15 },
    { 0xb26a9ec6, SUC_lexicon_tags_28 },
    { 0xfeea9ecb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526a9ed0, SUC_lexicon_tags_1 },
    { 0xe38a9ed0, SUC_lexicon_tags_47 },
    { 0x14ca9ed2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x456a9ed5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2a9ed8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0a9edd, SUC_lexicon_tags_29 },
    { 0x34ca9edd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370a9ee2, SUC_lexicon_tags_2 },
    { 0xbc6a9ee3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36a9ee6, SUC_lexicon_tags_47 },
    { 0x31aa9ee6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5a2a9ee9, SUC_lexicon_tags_28 },
    { 0xa72a9eea, SUC_lexicon_tags_3 },
    { 0xff6a9ee9, SUC_lexicon_tags_31 },
    { 0x70a9eec, SUC_lexicon_tags_21 },
    { 0xd76a9eec, SUC_lexicon_tags_14 },
    { 0x22aa9eea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x848a9ef0, SUC_lexicon_tags_14 },
    { 0xa3ea9ef1, SUC_lexicon_tags_33 },
    { 0xd6aa9ef2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dea9ef7, SUC_lexicon_tags_46 },
    { 0xbe8a9ef7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aa9eff, SUC_lexicon_tags_11 },
    { 0x32a9f00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9f8a9f02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eea9f05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x754a9f08, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdcca9f0a, SUC_lexicon_tags_17 },
    { 0xe46a9f0b, SUC_lexicon_tags_12 },
    { 0x4faa9f0c, SUC_lexicon_tags_37 },
    { 0x5fea9f0a, SUC_lexicon_tags_19 },
    { 0x268a9f0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabca9f13, SUC_lexicon_tags_42 },
    { 0x946a9f13, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02a9f17, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x89aa9f19, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62a9f20, SUC_lexicon_tags_43 },
    { 0x222a9f21, SUC_lexicon_tags_0 },
    { 0x12ea9f21, SUC_lexicon_tags_47 },
    { 0x66a9f23, SUC_lexicon_tags_34 },
    { 0x3b8a9f24, SUC_lexicon_tags_17 },
    { 0xb56a9f25, SUC_lexicon_tags_17 },
    { 0xb78a9f26, SUC_lexicon_tags_12 },
    { 0xd0ea9f27, SUC_lexicon_tags_11 },
    { 0x406a9f28, SUC_lexicon_tags_30 },
    { 0x320a9f29, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x78a9f2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa0a9f2d, SUC_lexicon_tags_1 },
    { 0x232a9f2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x94a9f30, SUC_lexicon_tags_146 },
    { 0x40a9f31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd82a9f33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2a9f35, SUC_lexicon_tags_12 },
    { 0xc6ea9f36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ea9f3e, SUC_lexicon_tags_37 },
    { 0x9e0a9f3f, SUC_lexicon_tags_2 },
    { 0x7e2a9f40, SUC_lexicon_tags_38 },
    { 0xddca9f41, SUC_lexicon_tags_66 },
    { 0xb74a9f42, SUC_lexicon_tags_12 },
    { 0x4a6a9f43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x968a9f45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x276a9f47, SUC_lexicon_tags_37 },
    { 0x594a9f47, SUC_lexicon_tags_12 },
    { 0x6fca9f48, SUC_lexicon_tags_28 },
    { 0x594a9f4a, SUC_lexicon_tags_7 },
    { 0x350a9f49, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6a9f4e, SUC_lexicon_tags_37 },
    { 0x46aa9f4e, SUC_lexicon_tags_27 },
    { 0x7f6a9f50, SUC_lexicon_tags_10 },
    { 0x94ca9f51, SUC_lexicon_tags_27 },
    { 0x50a9f52, SUC_lexicon_tags_7 },
    { 0xd80a9f52, SUC_lexicon_tags_1 },
    { 0xd1aa9f54, SUC_lexicon_tags_19 },
    { 0x562a9f52, SUC_lexicon_tags_4 },
    { 0xb50a9f54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16a9f59, SUC_lexicon_tags_67 },
    { 0xb7ca9f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45aa9f5d, SUC_lexicon_tags_55 },
    { 0xb94a9f5e, SUC_lexicon_tags_37 },
    { 0x34aa9f5e, SUC_lexicon_tags_2 },
    { 0x72a9f5d, SUC_lexicon_tags_12 },
    { 0xaa4a9f61, SUC_lexicon_tags_468 },
    { 0x79ca9f62, SUC_lexicon_tags_37 },
    { 0xc54a9f63, SUC_lexicon_tags_2 },
    { 0xf9ea9f64, SUC_lexicon_tags_103 },
    { 0xa08a9f65, SUC_lexicon_tags_12 },
    { 0xf94a9f66, SUC_lexicon_tags_25 },
    { 0xc2aa9f66, SUC_lexicon_tags_45 },
    { 0xaeaa9f65, SUC_lexicon_tags_15 },
    { 0x89ea9f63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34a9f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0a9f6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ca9f75, SUC_lexicon_tags_87 },
    { 0x690a9f75, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ca9f79, SUC_lexicon_tags_11 },
    { 0xf2ea9f7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98a9f7e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x1f6a9f80, SUC_lexicon_tags_79 },
    { 0xe6ea9f81, SUC_lexicon_tags_61 },
    { 0xe9ca9f80, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ca9f87, SUC_lexicon_tags_25 },
    { 0xcdea9f87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ca9f8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x320a9f8e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68a9f96, SUC_lexicon_tags_34 },
    { 0x606a9f97, SUC_lexicon_tags_37 },
    { 0x1c6a9f98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faa9f9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82a9fa0, SUC_lexicon_tags_13 },
    { 0xd4ca9fa1, SUC_lexicon_tags_17 },
    { 0xe18a9fa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe86a9fa4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3b2a9fa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18ea9fa8, SUC_lexicon_tags_47 },
    { 0x590a9fa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84a9fac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4a9fb1, SUC_lexicon_tags_25 },
    { 0xaaaa9fb1, SUC_lexicon_tags_92 },
    { 0x3dea9fb2, SUC_lexicon_tags_58 },
    { 0x27ea9fb4, SUC_lexicon_tags_37 },
    { 0x56ea9fb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf52a9fb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8a9fbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x67ca9fbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x962a9fc2, SUC_lexicon_tags_37 },
    { 0x792a9fc2, SUC_lexicon_tags_25 },
    { 0x3c2a9fc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x87ea9fc6, SUC_lexicon_tags_68 },
    { 0x190a9fc7, SUC_lexicon_tags_17 },
    { 0xab0a9fc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faa9fcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696a9fd1, SUC_lexicon_tags_11 },
    { 0xceaa9fd1, SUC_lexicon_tags_12 },
    { 0x1aa9fd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6a9fdb, SUC_lexicon_tags_60 },
    { 0x196a9fdc, SUC_lexicon_tags_12 },
    { 0x506a9fdd, SUC_lexicon_tags_7 },
    { 0x7b0a9fde, SUC_lexicon_tags_66 },
    { 0x1baa9fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8a9fe2, SUC_lexicon_tags_7 },
    { 0x4caa9fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20a9fe6, SUC_lexicon_tags_27 },
    { 0x60a9fe7, SUC_lexicon_tags_30 },
    { 0xafaa9fe8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xae8a9fea, SUC_lexicon_tags_2 },
    { 0xf22a9fea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34a9ff0, SUC_lexicon_tags_19 },
    { 0x17aa9ff1, SUC_lexicon_tags_37 },
    { 0xc16a9ff1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x784a9ff4, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ca9ff7, SUC_lexicon_tags_25 },
    { 0x564a9ff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eea9ffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0aa006, SUC_lexicon_tags_28 },
    { 0xe52aa007, SUC_lexicon_tags_37 },
    { 0x7b4aa008, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13eaa00b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52eaa00d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x41eaa00f, SUC_lexicon_tags_26 },
    { 0xfdeaa010, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818aa016, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590aa01d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xedeaa020, SUC_lexicon_tags_63 },
    { 0x83eaa021, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d2aa023, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aaa02a, SUC_lexicon_tags_57 },
    { 0xc4aa02b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x604aa02d, SUC_lexicon_tags_7 },
    { 0x84caa02e, SUC_lexicon_tags_36 },
    { 0x2c0aa02f, SUC_lexicon_tags_8 },
    { 0xe0caa030, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x812aa032, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6aa036, SUC_lexicon_tags_42 },
    { 0xe8caa037, SUC_lexicon_tags_45 },
    { 0xa76aa038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50eaa03a, SUC_lexicon_tags_7 },
    { 0x824aa03b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04aa03f, SUC_lexicon_tags_82 },
    { 0x23eaa040, SUC_lexicon_tags_1 },
    { 0xccaa041, SUC_lexicon_tags_68 },
    { 0xd50aa041, SUC_lexicon_tags_1 },
    { 0x66eaa043, SUC_lexicon_tags_37 },
    { 0x570aa042, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4d2aa046, SUC_lexicon_tags_11 },
    { 0x8a0aa047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2aa04a, SUC_lexicon_tags_153 },
    { 0xc64aa04b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbccaa04d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78eaa04f, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80aa059, SUC_lexicon_tags_29 },
    { 0x6a8aa05a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24caa060, SUC_lexicon_tags_25 },
    { 0xb0aaa061, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296aa066, SUC_lexicon_tags_8 },
    { 0x2aaa066, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4baaa069, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aaa06d, SUC_lexicon_tags_63 },
    { 0x382aa06e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4aeaa070, SUC_lexicon_tags_117 },
    { 0x2a2aa070, SUC_lexicon_tags_25 },
    { 0x20aaa070, SUC_lexicon_tags_21 },
    { 0xc2aa072, SUC_lexicon_tags_31 },
    { 0x9f8aa074, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x698aa076, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3aaa078, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe62aa07a, SUC_lexicon_tags_54 },
    { 0xe1caa07b, SUC_lexicon_tags_26 },
    { 0x4caaa07a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43aaa080, SUC_lexicon_tags_17 },
    { 0xd06aa081, SUC_lexicon_tags_17 },
    { 0xa8aa082, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aaa085, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0aa088, SUC_lexicon_tags_19 },
    { 0xbeaa089, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4eaa08f, SUC_lexicon_tags_32 },
    { 0xf96aa090, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x696aa092, SUC_lexicon_tags_7 },
    { 0x6b6aa093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6aa096, SUC_lexicon_tags_11 },
    { 0xa0aa097, SUC_lexicon_tags_0 },
    { 0xa28aa098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72caa09c, SUC_lexicon_tags_27 },
    { 0xf6aa09d, SUC_lexicon_tags_11 },
    { 0x4d2aa09e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4aa0a2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xae4aa0a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aaa0a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2aa0ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aaa0ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x594aa0b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50aa0b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18aa0b6, SUC_lexicon_tags_11 },
    { 0xb4eaa0b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aaa0c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2f8aa0c2, SUC_lexicon_tags_7 },
    { 0xaa4aa0c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x288aa0c5, SUC_lexicon_tags_7 },
    { 0x366aa0c6, SUC_lexicon_tags_23 },
    { 0x7e0aa0c5, SUC_lexicon_tags_11 },
    { 0xfe0aa0c8, SUC_lexicon_tags_12 },
    { 0x652aa0c7, SUC_lexicon_tags_60 },
    { 0x174aa0c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaaa0cd, SUC_lexicon_tags_17 },
    { 0xfb6aa0cd, SUC_lexicon_tags_2 },
    { 0xea6aa0cf, SUC_lexicon_tags_11 },
    { 0x748aa0cd, SUC_lexicon_tags_37 },
    { 0x46aaa0d0, SUC_lexicon_tags_37 },
    { 0x26aaa0d0, SUC_lexicon_tags_11 },
    { 0x56aaa0d3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2dcaa0d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadaaa0d7, SUC_lexicon_tags_37 },
    { 0x7eaaa0d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48aa0dc, SUC_lexicon_tags_11 },
    { 0x85eaa0dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacaa0e6, SUC_lexicon_tags_17 },
    { 0x74aa0e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23aaa0e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3feaa0eb, SUC_lexicon_tags_60 },
    { 0x80aa0ec, SUC_lexicon_tags_1 },
    { 0xabcaa0ed, SUC_lexicon_tags_178 },
    { 0x572aa0ee, SUC_lexicon_tags_7 },
    { 0xa56aa0ef, SUC_lexicon_tags_43 },
    { 0x28caa0f0, SUC_lexicon_tags_38 },
    { 0xf84aa0ec, SUC_lexicon_tags_11 },
    { 0xfceaa0f0, SUC_lexicon_tags_1 },
    { 0x170aa0f3, SUC_lexicon_tags_21 },
    { 0xd22aa0f3, SUC_lexicon_tags_15 },
    { 0x44aaa0f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccaa0f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x248aa0fb, SUC_lexicon_tags_38 },
    { 0xbecaa0fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acaa105, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaa109, SUC_lexicon_tags_0 },
    { 0x2c4aa10a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2aa10d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaeaa117, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x306aa119, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc2aa11b, SUC_lexicon_tags_7 },
    { 0x86caa11c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x100aa11e, SUC_lexicon_tags_11 },
    { 0xda4aa11e, SUC_lexicon_tags_2 },
    { 0x7d4aa120, SUC_lexicon_tags_12 },
    { 0x5f6aa121, SUC_lexicon_tags_8 },
    { 0x3b8aa121, SUC_lexicon_tags_60 },
    { 0x3fcaa123, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0aa127, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54aa130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf0aa132, SUC_lexicon_tags_33 },
    { 0x61eaa133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaaa138, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x88aaa13b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524aa13f, SUC_lexicon_tags_8 },
    { 0xaf2aa140, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x42eaa142, SUC_lexicon_tags_11 },
    { 0x686aa143, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd76aa145, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecaa14a, SUC_lexicon_tags_25 },
    { 0xadeaa14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe98aa14d, SUC_lexicon_tags_16 },
    { 0xc0aaa14e, SUC_lexicon_tags_15 },
    { 0x59eaa14d, SUC_lexicon_tags_11 },
    { 0x35eaa14e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x716aa152, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f4aa154, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x596aa157, SUC_lexicon_tags_25 },
    { 0xcf6aa158, SUC_lexicon_tags_2 },
    { 0xb08aa159, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72aa15d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88aa160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26aa164, SUC_lexicon_tags_282 },
    { 0x74eaa164, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x282aa168, SUC_lexicon_tags_28 },
    { 0x5c4aa169, SUC_lexicon_tags_6 },
    { 0x26caa169, SUC_lexicon_tags_10 },
    { 0xc34aa16b, SUC_lexicon_tags_17 },
    { 0x68caa16c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8aa16f, SUC_lexicon_tags_25 },
    { 0xb2aaa170, SUC_lexicon_tags_17 },
    { 0x7deaa171, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5deaa174, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x696aa177, SUC_lexicon_tags_43 },
    { 0x890aa178, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9caa17b, SUC_lexicon_tags_6 },
    { 0x882aa17c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eaa180, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74eaa183, SUC_lexicon_tags_23 },
    { 0x64aaa184, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496aa189, SUC_lexicon_tags_27 },
    { 0x42aaa18a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548aa18f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70aa193, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92caa195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174aa19d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6aaa1a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26aa1aa, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14aa1b3, SUC_lexicon_tags_25 },
    { 0x1f4aa1b3, SUC_lexicon_tags_60 },
    { 0x3e8aa1b5, SUC_lexicon_tags_0 },
    { 0x5beaa1b6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x28aa1b9, SUC_lexicon_tags_10 },
    { 0x5d0aa1ba, SUC_lexicon_tags_5 },
    { 0x786aa1b9, SUC_lexicon_tags_37 },
    { 0x588aa1b9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaaa1c5, SUC_lexicon_tags_8 },
    { 0x7beaa1c6, SUC_lexicon_tags_27 },
    { 0x116aa1c7, SUC_lexicon_tags_13 },
    { 0x7feaa1c7, SUC_lexicon_tags_25 },
    { 0x4d4aa1c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x476aa1cc, SUC_lexicon_tags_8 },
    { 0x1daaa1cd, SUC_lexicon_tags_11 },
    { 0x126aa1cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0aa1d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1deaa1d7, SUC_lexicon_tags_25 },
    { 0x236aa1d8, SUC_lexicon_tags_39 },
    { 0x954aa1d7, SUC_lexicon_tags_63 },
    { 0x4aa1da, SUC_lexicon_tags_26 },
    { 0x1baaa1d9, SUC_lexicon_tags_23 },
    { 0x864aa1dc, SUC_lexicon_tags_47 },
    { 0xcbcaa1dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280aa1e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740aa1f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6aa1f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34aa1fa, SUC_lexicon_tags_2 },
    { 0xea0aa1fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaaa200, SUC_lexicon_tags_35 },
    { 0xb44aa201, SUC_lexicon_tags_44 },
    { 0xe56aa202, SUC_lexicon_tags_301 },
    { 0xa92aa203, SUC_lexicon_tags_27 },
    { 0x54eaa204, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8aaa206, SUC_lexicon_tags_7 },
    { 0xb8aaa206, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72aa20c, SUC_lexicon_tags_25 },
    { 0xdceaa20d, SUC_lexicon_tags_49 },
    { 0x996aa20e, SUC_lexicon_tags_7 },
    { 0xf46aa20e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92aa215, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x164aa217, SUC_lexicon_tags_14 },
    { 0x6eeaa218, SUC_lexicon_tags_45 },
    { 0x70eaa218, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60aa21c, SUC_lexicon_tags_11 },
    { 0x768aa21d, SUC_lexicon_tags_63 },
    { 0x292aa21c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aa224, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954aa228, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3ccaa22a, SUC_lexicon_tags_49 },
    { 0x51eaa22a, SUC_lexicon_tags_37 },
    { 0x896aa22c, SUC_lexicon_tags_7 },
    { 0x5d0aa22c, SUC_lexicon_tags_25 },
    { 0x86caa22e, SUC_lexicon_tags_37 },
    { 0x942aa22f, SUC_lexicon_tags_1 },
    { 0x7caaa22e, SUC_lexicon_tags_11 },
    { 0x404aa22e, SUC_lexicon_tags_33 },
    { 0x2c2aa231, SUC_lexicon_tags_8 },
    { 0xa94aa22e, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4aa239, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x83caa23b, SUC_lexicon_tags_0 },
    { 0x2b4aa23c, SUC_lexicon_tags_167 },
    { 0x698aa23c, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0xb16aa23f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa8aa241, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94caa243, SUC_lexicon_tags_0 },
    { 0xcc0aa243, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaaa249, SUC_lexicon_tags_60 },
    { 0xe6caa24a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2aa24d, SUC_lexicon_tags_25 },
    { 0x444aa24d, SUC_lexicon_tags_17 },
    { 0x362aa24f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe92aa251, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716aa257, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aa25a, SUC_lexicon_tags_25 },
    { 0x372aa25a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640aa25f, SUC_lexicon_tags_37 },
    { 0xc42aa260, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe4aa262, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6aa265, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf6aa267, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46aa26a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb48aa26c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2aaa26e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa34aa270, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81aaa272, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x782aa274, SUC_lexicon_tags_17 },
    { 0xc04aa275, SUC_lexicon_tags_87 },
    { 0x220aa275, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecaa27e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4aa281, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572aa286, SUC_lexicon_tags_33 },
    { 0xf8aaa286, SUC_lexicon_tags_0 },
    { 0x58aaa288, SUC_lexicon_tags_33 },
    { 0x8e2aa289, SUC_lexicon_tags_1 },
    { 0x72aaa28a, SUC_lexicon_tags_63 },
    { 0x19eaa289, SUC_lexicon_tags_17 },
    { 0x2ceaa28a, SUC_lexicon_tags_37 },
    { 0x9c8aa289, SUC_lexicon_tags_0 },
    { 0x14caa28e, SUC_lexicon_tags_4 },
    { 0xc74aa28f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x59caa291, SUC_lexicon_tags_37 },
    { 0xdf4aa292, SUC_lexicon_tags_12 },
    { 0x4daaa291, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6aa297, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92aa29e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe24aa2a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22aaa2ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe72aa2ad, SUC_lexicon_tags_37 },
    { 0xb9aaa2ad, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xd80aa2b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x474aa2b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eaa2ba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa22aa2bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0aa2bf, SUC_lexicon_tags_11 },
    { 0xa2caa2c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4aa2c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66aaa2c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x426aa2ca, SUC_lexicon_tags_12 },
    { 0x354aa2cb, SUC_lexicon_tags_7 },
    { 0x224aa2cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45eaa2d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcf6aa2d3, SUC_lexicon_tags_25 },
    { 0x870aa2d4, SUC_lexicon_tags_14 },
    { 0x34aa2d3, SUC_lexicon_tags_8 },
    { 0xfa6aa2d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34aa2d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8aa2de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc44aa2e0, SUC_lexicon_tags_66 },
    { 0x79eaa2e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6eaa2e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb38aa2e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd20aa2e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7aaaa2e9, SUC_lexicon_tags_37 },
    { 0x994aa2ea, SUC_lexicon_tags_17 },
    { 0x7feaa2eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc64aa2ed, SUC_lexicon_tags_11 },
    { 0xca2aa2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58aa2f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2aa2f8, SUC_lexicon_tags_37 },
    { 0x614aa2f9, SUC_lexicon_tags_55 },
    { 0x7a2aa2fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72aa2fd, SUC_lexicon_tags_33 },
    { 0xd9caa2fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4f8aa300, SUC_lexicon_tags_15 },
    { 0x81caa301, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68eaa305, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x76aa307, SUC_lexicon_tags_27 },
    { 0xc06aa308, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0aa30c, SUC_lexicon_tags_23 },
    { 0xec2aa30c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0aa310, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x1aaa312, SUC_lexicon_tags_17 },
    { 0x34caa312, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92caa322, SUC_lexicon_tags_25 },
    { 0xc68aa322, SUC_lexicon_tags_1 },
    { 0x3feaa323, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x918aa326, SUC_lexicon_tags_37 },
    { 0xfa0aa326, SUC_lexicon_tags_4 },
    { 0x324aa328, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1caa32e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5aaa330, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aaa33a, SUC_lexicon_tags_0 },
    { 0x294aa33a, SUC_lexicon_tags_15 },
    { 0xaaeaa33c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6aa33e, SUC_lexicon_tags_31 },
    { 0xeccaa33f, SUC_lexicon_tags_23 },
    { 0x99eaa340, SUC_lexicon_tags_15 },
    { 0x9ccaa33e, SUC_lexicon_tags_1 },
    { 0x348aa341, SUC_lexicon_tags_7 },
    { 0xee8aa343, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4aa346, SUC_lexicon_tags_57 },
    { 0xce0aa346, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x748aa34a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56aa351, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410aa355, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92aa358, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76aaa35a, SUC_lexicon_tags_57 },
    { 0x226aa35b, SUC_lexicon_tags_39 },
    { 0x10aa35b, SUC_lexicon_tags_10 },
    { 0x6f8aa35d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x57eaa360, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbcaa362, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50eaa364, SUC_lexicon_tags_48 },
    { 0x9e0aa364, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99aaa367, SUC_lexicon_tags_0 },
    { 0x13aaa368, SUC_lexicon_tags_0 },
    { 0x93aaa369, SUC_lexicon_tags_7 },
    { 0xeb6aa36a, SUC_lexicon_tags_15 },
    { 0x18aa36b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caaa370, SUC_lexicon_tags_14 },
    { 0xd06aa371, SUC_lexicon_tags_37 },
    { 0x7baaa372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eaa375, SUC_lexicon_tags_1 },
    { 0x306aa376, SUC_lexicon_tags_15 },
    { 0xbe4aa377, SUC_lexicon_tags_37 },
    { 0x54caa378, SUC_lexicon_tags_38 },
    { 0x5a0aa379, SUC_lexicon_tags_11 },
    { 0xd4aaa379, SUC_lexicon_tags_15 },
    { 0x730aa379, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75eaa37f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9eaa381, SUC_lexicon_tags_2 },
    { 0x78aa382, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522aa38a, SUC_lexicon_tags_48 },
    { 0xb78aa38a, SUC_lexicon_tags_79 },
    { 0xc70aa38c, SUC_lexicon_tags_15 },
    { 0x5eeaa38c, SUC_lexicon_tags_14 },
    { 0xb84aa38c, SUC_lexicon_tags_25 },
    { 0x278aa38a, SUC_lexicon_tags_69 },
    { 0xb40aa38f, SUC_lexicon_tags_31 },
    { 0xcbeaa391, SUC_lexicon_tags_160 },
    { 0x2c4aa390, SUC_lexicon_tags_38 },
    { 0x34aaa38c, SUC_lexicon_tags_0 },
    { 0x9acaa394, SUC_lexicon_tags_8 },
    { 0xbe0aa395, SUC_lexicon_tags_11 },
    { 0x3fcaa395, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6aa399, SUC_lexicon_tags_43 },
    { 0x72aaa39a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8aa39e, SUC_lexicon_tags_27 },
    { 0x1d4aa39f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6aaa3a1, SUC_lexicon_tags_1 },
    { 0xc86aa3a1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0aa3a9, SUC_lexicon_tags_12 },
    { 0xba8aa3aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x568aa3ac, SUC_lexicon_tags_84 },
    { 0x332aa3ad, SUC_lexicon_tags_9 },
    { 0xcc6aa3ad, SUC_lexicon_tags_25 },
    { 0xff2aa3ad, SUC_lexicon_tags_14 },
    { 0xf9caa3ad, SUC_lexicon_tags_11 },
    { 0x3f2aa3b1, SUC_lexicon_tags_21 },
    { 0x42aa3b1, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xf78aa3b4, SUC_lexicon_tags_26 },
    { 0xbb4aa3b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4aa3ba, SUC_lexicon_tags_37 },
    { 0xe50aa3bb, SUC_lexicon_tags_28 },
    { 0xf5eaa3bb, SUC_lexicon_tags_47 },
    { 0x2caaa3bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70aa3c0, SUC_lexicon_tags_1 },
    { 0xb0eaa3c1, SUC_lexicon_tags_15 },
    { 0x146aa3c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5f6aa3c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0aa3c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe40aa3c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4aa3cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4caa3ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec8aa3d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aa3db, SUC_lexicon_tags_11 },
    { 0xcb4aa3dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaa3e0, SUC_lexicon_tags_7 },
    { 0xa8aa3e1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x756aa3e4, SUC_lexicon_tags_4 },
    { 0x12caa3e5, SUC_lexicon_tags_32 },
    { 0x866aa3e6, SUC_lexicon_tags_46 },
    { 0xad8aa3e6, SUC_lexicon_tags_47 },
    { 0x75eaa3e4, SUC_lexicon_tags_14 },
    { 0x1e0aa3e7, SUC_lexicon_tags_68 },
    { 0xb2caa3e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecaa3ef, SUC_lexicon_tags_7 },
    { 0xdcaa3ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee2aa3f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74aa3f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb46aa3f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41aaa402, SUC_lexicon_tags_8 },
    { 0x900aa402, SUC_lexicon_tags_34 },
    { 0x9b0aa403, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x878aa407, SUC_lexicon_tags_26 },
    { 0x586aa408, SUC_lexicon_tags_47 },
    { 0xec4aa409, SUC_lexicon_tags_25 },
    { 0x53eaa408, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6aa40d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd22aa40f, SUC_lexicon_tags_19 },
    { 0xc5eaa40f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7eaa414, SUC_lexicon_tags_2 },
    { 0xf04aa415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18aa418, SUC_lexicon_tags_7 },
    { 0xc04aa418, SUC_lexicon_tags_15 },
    { 0xb9eaa41a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x498aa41d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2aa421, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x794aa424, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29eaa426, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b6aa428, SUC_lexicon_tags_25 },
    { 0x342aa429, SUC_lexicon_tags_27 },
    { 0x382aa428, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbd4aa42c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeaa430, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eaa434, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64aa438, SUC_lexicon_tags_25 },
    { 0x104aa438, SUC_lexicon_tags_32 },
    { 0xabcaa43a, SUC_lexicon_tags_12 },
    { 0x394aa438, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x64eaa43d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaeaa43f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd2aa441, SUC_lexicon_tags_93 },
    { 0x468aa442, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11aaa449, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862aa44e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276aa452, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe08aa454, SUC_lexicon_tags_1 },
    { 0x3deaa454, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaaa458, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbeaa45b, SUC_lexicon_tags_0 },
    { 0x3eaaa45c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90aa45f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00aa46a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0aa46e, SUC_lexicon_tags_7 },
    { 0x84aaa46f, SUC_lexicon_tags_9 },
    { 0xc50aa470, SUC_lexicon_tags_15 },
    { 0xd48aa471, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc8aa473, SUC_lexicon_tags_11 },
    { 0xd9caa474, SUC_lexicon_tags_10 },
    { 0xbd4aa475, SUC_lexicon_tags_11 },
    { 0x6ccaa475, SUC_lexicon_tags_7 },
    { 0x23caa476, SUC_lexicon_tags_3 },
    { 0xefeaa476, SUC_lexicon_tags_23 },
    { 0x28eaa474, SUC_lexicon_tags_33 },
    { 0x9e4aa47a, SUC_lexicon_tags_34 },
    { 0xb28aa473, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2eaa47e, SUC_lexicon_tags_32 },
    { 0x210aa47f, SUC_lexicon_tags_64 },
    { 0x888aa480, SUC_lexicon_tags_37 },
    { 0xeb6aa480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xceeaa484, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x380aa486, SUC_lexicon_tags_15 },
    { 0xe50aa487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27eaa489, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62aaa48c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12aa496, SUC_lexicon_tags_257 },
    { 0xe04aa497, SUC_lexicon_tags_12 },
    { 0xcd0aa498, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856aa49c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xefcaa49e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5caa4a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d0aa4ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168aa4b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7beaa4b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaa4bd, SUC_lexicon_tags_49 },
    { 0xa3caa4be, SUC_lexicon_tags_8 },
    { 0x46aa4be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0aa4c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30aa4c6, SUC_lexicon_tags_17 },
    { 0x908aa4c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x150aa4c9, SUC_lexicon_tags_47 },
    { 0x5f4aa4ca, SUC_lexicon_tags_11 },
    { 0x1e2aa4cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6aa4d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aa4d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496aa4d7, SUC_lexicon_tags_17 },
    { 0x4b0aa4d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618aa4de, SUC_lexicon_tags_17 },
    { 0xd0aaa4df, SUC_lexicon_tags_87 },
    { 0xe9caa4df, SUC_lexicon_tags_7 },
    { 0x192aa4e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8aa4e4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22aa4e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5caa4f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x732aa4f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8aa4f7, SUC_lexicon_tags_37 },
    { 0x8baaa4f7, SUC_lexicon_tags_24 },
    { 0xae2aa4f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41aaa4fc, SUC_lexicon_tags_37 },
    { 0x65caa4fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206aa502, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd26aa504, SUC_lexicon_tags_19 },
    { 0x14caa504, SUC_lexicon_tags_34 },
    { 0x1e0aa506, SUC_lexicon_tags_14 },
    { 0x4faaa506, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822aa50b, SUC_lexicon_tags_11 },
    { 0x358aa50c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x10aa50e, SUC_lexicon_tags_80 },
    { 0x564aa50f, SUC_lexicon_tags_11 },
    { 0xdcaaa50f, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4caa513, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04aa517, SUC_lexicon_tags_21 },
    { 0xf8eaa517, SUC_lexicon_tags_201 },
    { 0xd84aa519, SUC_lexicon_tags_3 },
    { 0x79caa51a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90aa520, SUC_lexicon_tags_8 },
    { 0xba6aa521, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8aa523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x262aa525, SUC_lexicon_tags_17 },
    { 0xeb0aa525, SUC_lexicon_tags_14 },
    { 0x392aa525, SUC_lexicon_tags_11 },
    { 0xd66aa525, SUC_lexicon_tags_37 },
    { 0x31caa529, SUC_lexicon_tags_1 },
    { 0xd54aa52a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x326aa52c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa40aa52e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86aa532, SUC_lexicon_tags_27 },
    { 0x3c4aa532, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa68aa535, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2aa53e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eaa542, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe0aaa544, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7aaa54c, SUC_lexicon_tags_6 },
    { 0xadaaa54d, SUC_lexicon_tags_28 },
    { 0x82aa54e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300aa552, SUC_lexicon_tags_7 },
    { 0xcacaa552, SUC_lexicon_tags_1 },
    { 0xb06aa554, SUC_lexicon_tags_11 },
    { 0xb44aa554, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134aa55a, SUC_lexicon_tags_49 },
    { 0xd0aaa55b, SUC_lexicon_tags_37 },
    { 0xf72aa55c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7eaa561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x786aa563, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06aa567, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9eaa56d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2aa570, SUC_lexicon_tags_11 },
    { 0x32aaa571, SUC_lexicon_tags_46 },
    { 0x500aa572, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64aa57c, SUC_lexicon_tags_30 },
    { 0x1eaaa57d, SUC_lexicon_tags_60 },
    { 0x510aa57c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eaa582, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148aa58e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8aa595, SUC_lexicon_tags_66 },
    { 0x56eaa596, SUC_lexicon_tags_13 },
    { 0x5aaaa597, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4caa59c, SUC_lexicon_tags_53 },
    { 0xb92aa59d, SUC_lexicon_tags_11 },
    { 0x2a0aa59e, SUC_lexicon_tags_31 },
    { 0x40aa59e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8aa5a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb6eaa5a5, SUC_lexicon_tags_5 },
    { 0xf86aa5a5, SUC_lexicon_tags_84 },
    { 0x756aa5a7, SUC_lexicon_tags_1 },
    { 0xe20aa5a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacaa5ac, SUC_lexicon_tags_1687 },
    { 0, NULL },
    { 0xdb6aa5ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x410aa5b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaf4aa5b2, SUC_lexicon_tags_26 },
    { 0x884aa5b3, SUC_lexicon_tags_7 },
    { 0xc28aa5b4, SUC_lexicon_tags_27 },
    { 0x2faaa5b5, SUC_lexicon_tags_14 },
    { 0xb1aaa5b6, SUC_lexicon_tags_27 },
    { 0x8deaa5b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70aaa5bd, SUC_lexicon_tags_29 },
    { 0x8f0aa5be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44eaa5c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x866aa5c2, SUC_lexicon_tags_14 },
    { 0xdfcaa5c3, SUC_lexicon_tags_60 },
    { 0x4deaa5c2, SUC_lexicon_tags_11 },
    { 0x19aaa5c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19caa5c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3feaa5cb, SUC_lexicon_tags_37 },
    { 0xba2aa5cc, SUC_lexicon_tags_46 },
    { 0x546aa5cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8a6aa5cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8aa5d6, SUC_lexicon_tags_14 },
    { 0x2d8aa5d7, SUC_lexicon_tags_2 },
    { 0x706aa5d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d4aa5da, SUC_lexicon_tags_4 },
    { 0x190aa5db, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594aa5e1, SUC_lexicon_tags_1 },
    { 0x422aa5e1, SUC_lexicon_tags_7 },
    { 0xe8aaa5e1, SUC_lexicon_tags_29 },
    { 0x4a8aa5e4, SUC_lexicon_tags_26 },
    { 0x1ccaa5e4, SUC_lexicon_tags_37 },
    { 0x3dcaa5e3, SUC_lexicon_tags_11 },
    { 0x9a6aa5e7, SUC_lexicon_tags_17 },
    { 0x732aa5e6, SUC_lexicon_tags_11 },
    { 0x238aa5e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b2aa5eb, SUC_lexicon_tags_15 },
    { 0x8f0aa5ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x478aa5ee, SUC_lexicon_tags_1 },
    { 0x58aaa5ef, SUC_lexicon_tags_38 },
    { 0x88caa5f0, SUC_lexicon_tags_15 },
    { 0x754aa5f0, SUC_lexicon_tags_12 },
    { 0x834aa5f1, SUC_lexicon_tags_68 },
    { 0x818aa5f3, SUC_lexicon_tags_17 },
    { 0xe8aaa5f2, SUC_lexicon_tags_32 },
    { 0x162aa5f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aeaa5fb, SUC_lexicon_tags_8 },
    { 0x7ceaa5fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164aa600, SUC_lexicon_tags_7 },
    { 0xcacaa601, SUC_lexicon_tags_47 },
    { 0x2f6aa600, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x162aa605, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8aa608, SUC_lexicon_tags_39 },
    { 0x86caa609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc76aa60b, SUC_lexicon_tags_17 },
    { 0x9acaa60c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04aa60f, SUC_lexicon_tags_7 },
    { 0xfbaaa60f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x468aa613, SUC_lexicon_tags_33 },
    { 0x7aaaa614, SUC_lexicon_tags_14 },
    { 0xd2aa615, SUC_lexicon_tags_17 },
    { 0x568aa616, SUC_lexicon_tags_29 },
    { 0x60aa617, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6eaa619, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86aa61f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632aa627, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2caa62b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82aa62e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x97aaa631, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x476aa634, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc4eaa636, SUC_lexicon_tags_12 },
    { 0xf24aa636, SUC_lexicon_tags_7 },
    { 0xf54aa638, SUC_lexicon_tags_37 },
    { 0xc16aa636, SUC_lexicon_tags_11 },
    { 0x8e8aa63a, SUC_lexicon_tags_15 },
    { 0x9ceaa63b, SUC_lexicon_tags_27 },
    { 0xc26aa63c, SUC_lexicon_tags_19 },
    { 0x49caa639, SUC_lexicon_tags_38 },
    { 0xd38aa639, SUC_lexicon_tags_84 },
    { 0xe44aa63f, SUC_lexicon_tags_9 },
    { 0x754aa63c, SUC_lexicon_tags_63 },
    { 0xeb8aa641, SUC_lexicon_tags_12 },
    { 0x55eaa642, SUC_lexicon_tags_25 },
    { 0xa6caa643, SUC_lexicon_tags_9 },
    { 0x11caa643, SUC_lexicon_tags_25 },
    { 0x598aa641, SUC_lexicon_tags_11 },
    { 0x6ccaa646, SUC_lexicon_tags_16 },
    { 0xb56aa643, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4caa64a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x126aa64d, SUC_lexicon_tags_14 },
    { 0x1b8aa64e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922aa653, SUC_lexicon_tags_82 },
    { 0x456aa654, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x92aaa656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x288aa659, SUC_lexicon_tags_1 },
    { 0xe54aa65a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26aa65e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12aa662, SUC_lexicon_tags_23 },
    { 0xaa4aa663, SUC_lexicon_tags_15 },
    { 0xfb6aa663, SUC_lexicon_tags_11 },
    { 0xdeaa665, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88aa667, SUC_lexicon_tags_12 },
    { 0x9f0aa668, SUC_lexicon_tags_32 },
    { 0x6e2aa669, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08aa66c, SUC_lexicon_tags_21 },
    { 0xd50aa66d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4aa675, SUC_lexicon_tags_25 },
    { 0x9dcaa676, SUC_lexicon_tags_11 },
    { 0xb5caa677, SUC_lexicon_tags_14 },
    { 0x74aa678, SUC_lexicon_tags_19 },
    { 0x148aa677, SUC_lexicon_tags_3 },
    { 0x256aa67a, SUC_lexicon_tags_15 },
    { 0x268aa67b, SUC_lexicon_tags_11 },
    { 0x328aa67b, SUC_lexicon_tags_6 },
    { 0xb76aa67b, SUC_lexicon_tags_11 },
    { 0xc4aa676, SUC_lexicon_tags_2 },
    { 0x2a0aa67f, SUC_lexicon_tags_63 },
    { 0xe00aa67f, SUC_lexicon_tags_63 },
    { 0x10aa681, SUC_lexicon_tags_48 },
    { 0x536aa679, SUC_lexicon_tags_22 },
    { 0x1eaaa67c, SUC_lexicon_tags_11 },
    { 0x4f0aa684, SUC_lexicon_tags_11 },
    { 0x618aa67c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4aa68b, SUC_lexicon_tags_84 },
    { 0xff4aa68b, SUC_lexicon_tags_11 },
    { 0xce4aa68d, SUC_lexicon_tags_19 },
    { 0xa1eaa68c, SUC_lexicon_tags_12 },
    { 0x708aa68d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x61eaa691, SUC_lexicon_tags_0 },
    { 0xf24aa692, SUC_lexicon_tags_0 },
    { 0x542aa692, SUC_lexicon_tags_733 },
    { 0x6f2aa694, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaaaaa696, SUC_lexicon_tags_14 },
    { 0x1caaa696, SUC_lexicon_tags_11 },
    { 0x396aa698, SUC_lexicon_tags_12 },
    { 0x620aa698, SUC_lexicon_tags_2 },
    { 0x400aa69a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecaa6a0, SUC_lexicon_tags_25 },
    { 0xedaaa6a0, SUC_lexicon_tags_66 },
    { 0xa1aaa6a0, SUC_lexicon_tags_39 },
    { 0xe4aaa6a3, SUC_lexicon_tags_27 },
    { 0x8c4aa6a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93eaa6a6, SUC_lexicon_tags_37 },
    { 0xbe2aa6a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a0aa6a9, SUC_lexicon_tags_38 },
    { 0x808aa6a9, SUC_lexicon_tags_49 },
    { 0xc58aa6a9, SUC_lexicon_tags_19 },
    { 0xf28aa6ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644aa6b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbeaa6b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520aa6bb, SUC_lexicon_tags_1 },
    { 0x11eaa6bb, SUC_lexicon_tags_37 },
    { 0x3eaa6bc, SUC_lexicon_tags_35 },
    { 0x142aa6be, SUC_lexicon_tags_37 },
    { 0x4b4aa6bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76aa6c4, SUC_lexicon_tags_4 },
    { 0xebeaa6c5, SUC_lexicon_tags_26 },
    { 0x42eaa6c4, SUC_lexicon_tags_11 },
    { 0x5e4aa6c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8dcaa6c9, SUC_lexicon_tags_1 },
    { 0x6eaaa6ca, SUC_lexicon_tags_9 },
    { 0x34aa6ca, SUC_lexicon_tags_3 },
    { 0x2e8aa6cb, SUC_lexicon_tags_25 },
    { 0x6beaa6cc, SUC_lexicon_tags_14 },
    { 0xee4aa6ce, SUC_lexicon_tags_37 },
    { 0x842aa6cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x852aa6d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42aa6d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83caa6dd, SUC_lexicon_tags_25 },
    { 0x28caa6dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4aa6e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0aa6e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81eaa6e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x158aa6ea, SUC_lexicon_tags_25 },
    { 0x62aa6ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x11caa6ee, SUC_lexicon_tags_8 },
    { 0x84caa6ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5d6aa6f1, SUC_lexicon_tags_0 },
    { 0xbdcaa6f1, SUC_lexicon_tags_15 },
    { 0x816aa6f3, SUC_lexicon_tags_17 },
    { 0x23caa6f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcfaaa6f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08aa6fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7caaa700, SUC_lexicon_tags_12 },
    { 0xfc6aa700, SUC_lexicon_tags_68 },
    { 0xb2aa702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x370aa705, SUC_lexicon_tags_0 },
    { 0x6d6aa706, SUC_lexicon_tags_60 },
    { 0x9f2aa705, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcaa70c, SUC_lexicon_tags_37 },
    { 0x91eaa70c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49eaa711, SUC_lexicon_tags_17 },
    { 0x384aa711, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8aa716, SUC_lexicon_tags_82 },
    { 0x242aa716, SUC_lexicon_tags_13 },
    { 0xba0aa718, SUC_lexicon_tags_11 },
    { 0x368aa719, SUC_lexicon_tags_37 },
    { 0xed2aa718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29caa71c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x882aa71e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40aa724, SUC_lexicon_tags_63 },
    { 0xcf4aa725, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510aa72d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4aa733, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46aa739, SUC_lexicon_tags_92 },
    { 0x77caa73a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86eaa73d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89eaa740, SUC_lexicon_tags_754 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0aa743, SUC_lexicon_tags_15 },
    { 0xd1caa744, SUC_lexicon_tags_37 },
    { 0xd36aa745, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256aa749, SUC_lexicon_tags_25 },
    { 0x9a4aa749, SUC_lexicon_tags_63 },
    { 0xd18aa74b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x100aa74d, SUC_lexicon_tags_38 },
    { 0x29aaa74d, SUC_lexicon_tags_31 },
    { 0x4f8aa74e, SUC_lexicon_tags_23 },
    { 0x9f6aa74e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ceaa752, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccaa758, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284aa760, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570aa766, SUC_lexicon_tags_47 },
    { 0xa9eaa767, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x97eaa76a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbaaaa76c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37aaa770, SUC_lexicon_tags_7 },
    { 0x7a8aa771, SUC_lexicon_tags_28 },
    { 0x9c6aa771, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b0aa774, SUC_lexicon_tags_66 },
    { 0x874aa774, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4aa778, SUC_lexicon_tags_897 },
    { 0, NULL },
    { 0x2acaa77a, SUC_lexicon_tags_22 },
    { 0x45caa77b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7caa77f, SUC_lexicon_tags_25 },
    { 0xae2aa77f, SUC_lexicon_tags_47 },
    { 0xaa0aa77f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36caa787, SUC_lexicon_tags_17 },
    { 0x712aa788, SUC_lexicon_tags_46 },
    { 0x90aa788, SUC_lexicon_tags_25 },
    { 0xbd2aa78a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdcaa78c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68aa795, SUC_lexicon_tags_27 },
    { 0x48eaa795, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2aa79d, SUC_lexicon_tags_25 },
    { 0x5f0aa79d, SUC_lexicon_tags_17 },
    { 0x98aaa79f, SUC_lexicon_tags_17 },
    { 0x858aa7a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8dcaa7a2, SUC_lexicon_tags_14 },
    { 0x4ccaa7a3, SUC_lexicon_tags_11 },
    { 0x104aa7a4, SUC_lexicon_tags_53 },
    { 0x8ceaa7a5, SUC_lexicon_tags_11 },
    { 0x956aa7a3, SUC_lexicon_tags_17 },
    { 0xb3eaa7a6, SUC_lexicon_tags_25 },
    { 0xeaaaa7a8, SUC_lexicon_tags_19 },
    { 0xf88aa7a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864aa7ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2aaa7b1, SUC_lexicon_tags_28 },
    { 0x17eaa7b2, SUC_lexicon_tags_17 },
    { 0x3ecaa7b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0aa7b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752aa7ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x528aa7bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x632aa7be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc18aa7c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49caa7c4, SUC_lexicon_tags_47 },
    { 0xbc4aa7c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe08aa7c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x748aa7c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x534aa7cc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3caa7cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaecaa7d1, SUC_lexicon_tags_87 },
    { 0x3beaa7d2, SUC_lexicon_tags_37 },
    { 0x41caa7d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x602aa7d5, SUC_lexicon_tags_12 },
    { 0xf40aa7d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x346aa7d8, SUC_lexicon_tags_17 },
    { 0xf2aaa7d9, SUC_lexicon_tags_15 },
    { 0x82aa7da, SUC_lexicon_tags_11 },
    { 0x964aa7db, SUC_lexicon_tags_2 },
    { 0xa80aa7da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8aa7e3, SUC_lexicon_tags_17 },
    { 0xe4eaa7e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaaa7e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aa7ee, SUC_lexicon_tags_8 },
    { 0x670aa7ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00aa7f4, SUC_lexicon_tags_351 },
    { 0x216aa7f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89caa7f9, SUC_lexicon_tags_11 },
    { 0x1f4aa7fa, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0x50eaa7fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2caa802, SUC_lexicon_tags_38 },
    { 0xf32aa803, SUC_lexicon_tags_11 },
    { 0x342aa804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabeaa809, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfbaaa80b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaaa80e, SUC_lexicon_tags_17 },
    { 0xefeaa80e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0aa814, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4aa818, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30aa81b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x52caa81d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcaa820, SUC_lexicon_tags_14 },
    { 0x906aa821, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xe94aa823, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65caa827, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x266aa829, SUC_lexicon_tags_47 },
    { 0x334aa82a, SUC_lexicon_tags_37 },
    { 0xae0aa82b, SUC_lexicon_tags_6 },
    { 0xa62aa82c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x664aa82e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xceeaa831, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86eaa833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124aa838, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x26aaa83b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eaa83e, SUC_lexicon_tags_2 },
    { 0x1caaa83f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeaa845, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5caa849, SUC_lexicon_tags_17 },
    { 0xf6caa84a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270aa855, SUC_lexicon_tags_32 },
    { 0x45aaa856, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x724aa858, SUC_lexicon_tags_37 },
    { 0xd4aa859, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9eaa85b, SUC_lexicon_tags_39 },
    { 0xb4eaa85c, SUC_lexicon_tags_7 },
    { 0xe16aa85c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0aa860, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1eaa862, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x754aa864, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf4eaa866, SUC_lexicon_tags_63 },
    { 0x314aa867, SUC_lexicon_tags_19 },
    { 0x540aa868, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x510aa86a, SUC_lexicon_tags_7 },
    { 0xb5caa86b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80aa870, SUC_lexicon_tags_19 },
    { 0xa4aa871, SUC_lexicon_tags_1 },
    { 0xeb4aa871, SUC_lexicon_tags_0 },
    { 0x78caa871, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44eaa87a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8aa87e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e0aa880, SUC_lexicon_tags_39 },
    { 0x510aa881, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x92caa883, SUC_lexicon_tags_30 },
    { 0x832aa884, SUC_lexicon_tags_9 },
    { 0xeaaa883, SUC_lexicon_tags_12 },
    { 0x394aa884, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x138aa888, SUC_lexicon_tags_11 },
    { 0x5aaaa888, SUC_lexicon_tags_34 },
    { 0xd90aa889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d2aa88c, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x630aa88f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6aa892, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcaa898, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22aa89e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4aa8a3, SUC_lexicon_tags_1 },
    { 0x190aa8a4, SUC_lexicon_tags_12 },
    { 0x3a2aa8a3, SUC_lexicon_tags_11 },
    { 0x194aa8a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6eaa8a8, SUC_lexicon_tags_17 },
    { 0xb1aaa8a9, SUC_lexicon_tags_91 },
    { 0xd3caa8aa, SUC_lexicon_tags_40 },
    { 0x98eaa8aa, SUC_lexicon_tags_2 },
    { 0xe14aa8ac, SUC_lexicon_tags_11 },
    { 0xf00aa8aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52caa8b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x708aa8b4, SUC_lexicon_tags_25 },
    { 0x734aa8b4, SUC_lexicon_tags_1 },
    { 0x568aa8b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3caa8ba, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc06aa8bc, SUC_lexicon_tags_15 },
    { 0xc20aa8bd, SUC_lexicon_tags_15 },
    { 0x8d4aa8be, SUC_lexicon_tags_37 },
    { 0xd52aa8bd, SUC_lexicon_tags_113 },
    { 0xe8eaa8be, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2caa8c3, SUC_lexicon_tags_37 },
    { 0xd3caa8c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x590aa8c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470aa8cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44eaa8dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x93eaa8e0, SUC_lexicon_tags_23 },
    { 0xe0eaa8e1, SUC_lexicon_tags_19 },
    { 0x294aa8e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcaa8e7, SUC_lexicon_tags_29 },
    { 0x342aa8e7, SUC_lexicon_tags_45 },
    { 0x78aaa8e9, SUC_lexicon_tags_8 },
    { 0xa94aa8ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6aa8f0, SUC_lexicon_tags_14 },
    { 0xac6aa8f0, SUC_lexicon_tags_7 },
    { 0xf0caa8f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792aa8f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x290aa8f9, SUC_lexicon_tags_65 },
    { 0x44caa8fa, SUC_lexicon_tags_25 },
    { 0x424aa8fa, SUC_lexicon_tags_43 },
    { 0x5e6aa8fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x49aaa8ff, SUC_lexicon_tags_37 },
    { 0xf04aa900, SUC_lexicon_tags_2 },
    { 0xeeaa901, SUC_lexicon_tags_0 },
    { 0xb32aa8ff, SUC_lexicon_tags_1 },
    { 0x5a4aa901, SUC_lexicon_tags_116 },
    { 0x2beaa900, SUC_lexicon_tags_33 },
    { 0x44caa903, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7deaa908, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x780aa90a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccaa911, SUC_lexicon_tags_60 },
    { 0xc36aa911, SUC_lexicon_tags_17 },
    { 0xaeeaa913, SUC_lexicon_tags_63 },
    { 0xde2aa914, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8aa918, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53aaa922, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf26aa924, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa74aa926, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0aa92b, SUC_lexicon_tags_12 },
    { 0xbb2aa92c, SUC_lexicon_tags_11 },
    { 0x924aa92d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc10aa930, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x964aa932, SUC_lexicon_tags_17 },
    { 0x994aa933, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826aa939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x318aa93c, SUC_lexicon_tags_60 },
    { 0xe78aa93c, SUC_lexicon_tags_38 },
    { 0x1f4aa93e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578aa944, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x994aa946, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd76aa948, SUC_lexicon_tags_11 },
    { 0xd10aa948, SUC_lexicon_tags_60 },
    { 0xc8caa94a, SUC_lexicon_tags_39 },
    { 0xeccaa94b, SUC_lexicon_tags_11 },
    { 0x502aa94b, SUC_lexicon_tags_35 },
    { 0x874aa94d, SUC_lexicon_tags_47 },
    { 0x408aa94e, SUC_lexicon_tags_26 },
    { 0xe78aa94f, SUC_lexicon_tags_49 },
    { 0xa7aaa94e, SUC_lexicon_tags_14 },
    { 0x392aa94e, SUC_lexicon_tags_17 },
    { 0xdeaa950, SUC_lexicon_tags_2 },
    { 0x27caa953, SUC_lexicon_tags_17 },
    { 0xf10aa954, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd8caa956, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0aa95a, SUC_lexicon_tags_25 },
    { 0xd12aa95a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccaa961, SUC_lexicon_tags_25 },
    { 0x770aa962, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x76caa965, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x29aaa968, SUC_lexicon_tags_56 },
    { 0x6d8aa968, SUC_lexicon_tags_37 },
    { 0x4a4aa96a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2aa96d, SUC_lexicon_tags_1 },
    { 0xfc2aa96e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50aaa971, SUC_lexicon_tags_16 },
    { 0xcdcaa971, SUC_lexicon_tags_40 },
    { 0x104aa973, SUC_lexicon_tags_4 },
    { 0xa90aa974, SUC_lexicon_tags_1463 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6aa979, SUC_lexicon_tags_49 },
    { 0x604aa979, SUC_lexicon_tags_49 },
    { 0xf0caa97b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4aa97e, SUC_lexicon_tags_17 },
    { 0xd7aaa97f, SUC_lexicon_tags_7 },
    { 0x15aaa97f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2aa984, SUC_lexicon_tags_17 },
    { 0x956aa985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92aa98b, SUC_lexicon_tags_1 },
    { 0x4faaa98b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4daaa98e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x964aa990, SUC_lexicon_tags_8 },
    { 0x196aa991, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x812aa993, SUC_lexicon_tags_1 },
    { 0x934aa994, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54aa99a, SUC_lexicon_tags_5 },
    { 0x134aa99b, SUC_lexicon_tags_12 },
    { 0xe2caa99c, SUC_lexicon_tags_37 },
    { 0x596aa99d, SUC_lexicon_tags_26 },
    { 0x3aaa99e, SUC_lexicon_tags_25 },
    { 0xdd4aa99b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02aa9a2, SUC_lexicon_tags_49 },
    { 0x72aa9a2, SUC_lexicon_tags_32 },
    { 0xf4caa9a3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x38caa9a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x160aa9a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4daaa9ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36aa9b1, SUC_lexicon_tags_7 },
    { 0x9f0aa9b2, SUC_lexicon_tags_14 },
    { 0x664aa9b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3deaa9b9, SUC_lexicon_tags_63 },
    { 0x8f8aa9ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12aa9bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14aa9bf, SUC_lexicon_tags_7 },
    { 0x7d0aa9bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x734aa9c3, SUC_lexicon_tags_29 },
    { 0x650aa9c4, SUC_lexicon_tags_17 },
    { 0x116aa9c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc90aa9c7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4b4aa9c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186aa9ce, SUC_lexicon_tags_11 },
    { 0x7faaa9cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4b8aa9d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8aa9d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eaa9d6, SUC_lexicon_tags_21 },
    { 0x172aa9d7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfa2aa9d9, SUC_lexicon_tags_45 },
    { 0x9fcaa9da, SUC_lexicon_tags_15 },
    { 0xa90aa9da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38aa9e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73aaa9ea, SUC_lexicon_tags_11 },
    { 0xae4aa9eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcaa9ef, SUC_lexicon_tags_11 },
    { 0xb82aa9f0, SUC_lexicon_tags_0 },
    { 0xab4aa9f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58eaa9f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x484aa9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcaa9ff, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x9e0aaa01, SUC_lexicon_tags_12 },
    { 0x162aaa02, SUC_lexicon_tags_8 },
    { 0xaf4aaa02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27aaaa09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f4aaa0b, SUC_lexicon_tags_32 },
    { 0x942aaa0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50aaaa10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec4aaa12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0aaa17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa96aaa19, SUC_lexicon_tags_11 },
    { 0xc62aaa19, SUC_lexicon_tags_33 },
    { 0xe76aaa1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70aaa1f, SUC_lexicon_tags_17 },
    { 0x680aaa20, SUC_lexicon_tags_46 },
    { 0xfd2aaa20, SUC_lexicon_tags_77 },
    { 0xa7caaa21, SUC_lexicon_tags_19 },
    { 0x6a2aaa23, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf6aaaa25, SUC_lexicon_tags_55 },
    { 0xa5aaaa25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2aaa2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30aaaa2e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60aaa31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfa6aaa33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaaaa3a, SUC_lexicon_tags_37 },
    { 0x602aaa3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x266aaa3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5baaaa45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecaaa48, SUC_lexicon_tags_11 },
    { 0xf8aaa49, SUC_lexicon_tags_25 },
    { 0xf66aaa49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x508aaa4c, SUC_lexicon_tags_15 },
    { 0x6deaaa4d, SUC_lexicon_tags_12 },
    { 0xfdcaaa4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44caaa51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8aaa57, SUC_lexicon_tags_37 },
    { 0xfa4aaa58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358aaa5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666aaa65, SUC_lexicon_tags_17 },
    { 0x8e4aaa66, SUC_lexicon_tags_0 },
    { 0xc6eaaa67, SUC_lexicon_tags_38 },
    { 0xcd0aaa68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x14aaaa6b, SUC_lexicon_tags_87 },
    { 0xb14aaa6b, SUC_lexicon_tags_55 },
    { 0x29aaaa6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x360aaa6f, SUC_lexicon_tags_26 },
    { 0x3feaaa6f, SUC_lexicon_tags_11 },
    { 0x98eaaa70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86eaaa73, SUC_lexicon_tags_25 },
    { 0xaacaaa74, SUC_lexicon_tags_64 },
    { 0xf44aaa75, SUC_lexicon_tags_47 },
    { 0x546aaa76, SUC_lexicon_tags_15 },
    { 0xf78aaa75, SUC_lexicon_tags_26 },
    { 0xdd4aaa76, SUC_lexicon_tags_7 },
    { 0xddeaaa79, SUC_lexicon_tags_37 },
    { 0xda4aaa7a, SUC_lexicon_tags_37 },
    { 0x4b4aaa7b, SUC_lexicon_tags_61 },
    { 0xbcaaa77, SUC_lexicon_tags_11 },
    { 0x546aaa7d, SUC_lexicon_tags_1 },
    { 0xe18aaa7d, SUC_lexicon_tags_7 },
    { 0xc8eaaa7e, SUC_lexicon_tags_26 },
    { 0x1feaaa7c, SUC_lexicon_tags_37 },
    { 0x8b2aaa74, SUC_lexicon_tags_11 },
    { 0xd36aaa82, SUC_lexicon_tags_45 },
    { 0x7a6aaa7f, SUC_lexicon_tags_19 },
    { 0x92eaaa79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2caaaa87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aaa8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa54aaa8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768aaa90, SUC_lexicon_tags_32 },
    { 0xb9aaaa90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6aaa96, SUC_lexicon_tags_63 },
    { 0xc92aaa97, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0aaa9a, SUC_lexicon_tags_43 },
    { 0x62aaa9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29caaa9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89aaaaa0, SUC_lexicon_tags_23 },
    { 0xec8aaaa0, SUC_lexicon_tags_15 },
    { 0xf32aaaa1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85caaaac, SUC_lexicon_tags_37 },
    { 0xe80aaaad, SUC_lexicon_tags_17 },
    { 0x248aaaae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x982aaab0, SUC_lexicon_tags_30 },
    { 0xa8aaab0, SUC_lexicon_tags_17 },
    { 0x2f2aaab2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496aaab9, SUC_lexicon_tags_37 },
    { 0x716aaab9, SUC_lexicon_tags_23 },
    { 0x8caaabb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24aaabf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa50aaac1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02aaac8, SUC_lexicon_tags_11 },
    { 0xfd6aaac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0caaacb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb10aaacd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632aaad4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172aaad8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faaaade, SUC_lexicon_tags_37 },
    { 0xc36aaadf, SUC_lexicon_tags_15 },
    { 0x926aaadf, SUC_lexicon_tags_0 },
    { 0x2caaae1, SUC_lexicon_tags_27 },
    { 0x518aaadf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa6caaae4, SUC_lexicon_tags_28 },
    { 0x252aaae5, SUC_lexicon_tags_17 },
    { 0x3f2aaae6, SUC_lexicon_tags_17 },
    { 0x7feaaae6, SUC_lexicon_tags_25 },
    { 0x25caaae8, SUC_lexicon_tags_57 },
    { 0xaaaaae8, SUC_lexicon_tags_11 },
    { 0xb80aaaea, SUC_lexicon_tags_11 },
    { 0xc5caaaeb, SUC_lexicon_tags_11 },
    { 0xd8eaaaec, SUC_lexicon_tags_19 },
    { 0x76eaaaed, SUC_lexicon_tags_27 },
    { 0xee4aaaec, SUC_lexicon_tags_19 },
    { 0x18aaaec, SUC_lexicon_tags_25 },
    { 0x252aaaed, SUC_lexicon_tags_47 },
    { 0x7f6aaaf1, SUC_lexicon_tags_47 },
    { 0xbc2aaae5, SUC_lexicon_tags_43 },
    { 0xbaaaaf3, SUC_lexicon_tags_48 },
    { 0x698aaaf4, SUC_lexicon_tags_2 },
    { 0x4b0aaaf5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72aaaf8, SUC_lexicon_tags_17 },
    { 0x3aaaaf9, SUC_lexicon_tags_25 },
    { 0xaecaaafa, SUC_lexicon_tags_13 },
    { 0xfe4aaafb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2caaafe, SUC_lexicon_tags_30 },
    { 0xa64aaaff, SUC_lexicon_tags_19 },
    { 0x82caaaff, SUC_lexicon_tags_173 },
    { 0xfdeaab01, SUC_lexicon_tags_84 },
    { 0xaf4aab02, SUC_lexicon_tags_6 },
    { 0x396aab03, SUC_lexicon_tags_25 },
    { 0xcd6aab02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x892aab06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50caab09, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8aab0c, SUC_lexicon_tags_87 },
    { 0xbdaaab0d, SUC_lexicon_tags_8 },
    { 0xa36aab0c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6aab16, SUC_lexicon_tags_1 },
    { 0xba8aab17, SUC_lexicon_tags_12 },
    { 0x234aab18, SUC_lexicon_tags_49 },
    { 0x1a0aab16, SUC_lexicon_tags_15 },
    { 0xd0aab1a, SUC_lexicon_tags_37 },
    { 0x39eaab1b, SUC_lexicon_tags_8 },
    { 0xa14aab1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3caaab1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcc0aab20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2aab23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaab28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2eaab2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acaab30, SUC_lexicon_tags_2 },
    { 0x20aaab30, SUC_lexicon_tags_70 },
    { 0x57caab32, SUC_lexicon_tags_107 },
    { 0x706aab33, SUC_lexicon_tags_57 },
    { 0xae6aab33, SUC_lexicon_tags_26 },
    { 0x75aaab35, SUC_lexicon_tags_15 },
    { 0x376aab32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd8aab38, SUC_lexicon_tags_27 },
    { 0xaa6aab38, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6aab3d, SUC_lexicon_tags_23 },
    { 0xa92aab3e, SUC_lexicon_tags_43 },
    { 0xcb4aab3e, SUC_lexicon_tags_17 },
    { 0x4e4aab3e, SUC_lexicon_tags_47 },
    { 0x978aab3d, SUC_lexicon_tags_26 },
    { 0xeb8aab42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7eaab44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06aab49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2aab50, SUC_lexicon_tags_28 },
    { 0xb30aab51, SUC_lexicon_tags_11 },
    { 0x490aab51, SUC_lexicon_tags_1 },
    { 0x276aab51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x844aab55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf28aab57, SUC_lexicon_tags_0 },
    { 0x1b2aab58, SUC_lexicon_tags_14 },
    { 0x7beaab58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeaab5c, SUC_lexicon_tags_63 },
    { 0x476aab5d, SUC_lexicon_tags_46 },
    { 0x62caab5e, SUC_lexicon_tags_0 },
    { 0x63eaab5f, SUC_lexicon_tags_4 },
    { 0x7d4aab60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13caab64, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8aab6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccaab6d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43eaab73, SUC_lexicon_tags_11 },
    { 0x188aab74, SUC_lexicon_tags_68 },
    { 0xf8aaab75, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0aab78, SUC_lexicon_tags_48 },
    { 0x988aab78, SUC_lexicon_tags_65 },
    { 0x1b6aab79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10aaab7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x54caab80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfcaab82, SUC_lexicon_tags_11 },
    { 0xec4aab83, SUC_lexicon_tags_25 },
    { 0xc96aab84, SUC_lexicon_tags_77 },
    { 0x7daaab85, SUC_lexicon_tags_17 },
    { 0xa10aab83, SUC_lexicon_tags_0 },
    { 0x854aab85, SUC_lexicon_tags_16 },
    { 0x5aeaab88, SUC_lexicon_tags_25 },
    { 0x572aab88, SUC_lexicon_tags_12 },
    { 0x104aab88, SUC_lexicon_tags_23 },
    { 0x1aaaab89, SUC_lexicon_tags_17 },
    { 0x430aab8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eaab93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87aaab95, SUC_lexicon_tags_37 },
    { 0x7ceaab96, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9aeaab98, SUC_lexicon_tags_11 },
    { 0xd50aab99, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14aaab9d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecaaba0, SUC_lexicon_tags_3 },
    { 0xfaeaaba0, SUC_lexicon_tags_19 },
    { 0xbc0aaba2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980aaba7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0aabac, SUC_lexicon_tags_1 },
    { 0x210aabad, SUC_lexicon_tags_17 },
    { 0x848aabae, SUC_lexicon_tags_27 },
    { 0xcdaaabaf, SUC_lexicon_tags_12 },
    { 0x18aaabb0, SUC_lexicon_tags_29 },
    { 0xc18aabb1, SUC_lexicon_tags_11 },
    { 0x1f4aabb2, SUC_lexicon_tags_37 },
    { 0xbb6aabad, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8aabb6, SUC_lexicon_tags_27 },
    { 0xa86aabb7, SUC_lexicon_tags_4 },
    { 0xed4aabb7, SUC_lexicon_tags_65 },
    { 0xf32aabb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabbc, SUC_lexicon_tags_5 },
    { 0xfdaaabbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x120aabc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2beaabc7, SUC_lexicon_tags_6 },
    { 0x8a2aabc8, SUC_lexicon_tags_38 },
    { 0x552aabc9, SUC_lexicon_tags_15 },
    { 0xbc0aabc8, SUC_lexicon_tags_1 },
    { 0x76aabcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68aabcf, SUC_lexicon_tags_47 },
    { 0x354aabd0, SUC_lexicon_tags_43 },
    { 0x864aabcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4faaabd3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aaabd9, SUC_lexicon_tags_37 },
    { 0x80aaabda, SUC_lexicon_tags_11 },
    { 0xeeaaabda, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8f8aabdd, SUC_lexicon_tags_16 },
    { 0xc20aabde, SUC_lexicon_tags_27 },
    { 0x218aabdf, SUC_lexicon_tags_29 },
    { 0x662aabe0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aabea, SUC_lexicon_tags_29 },
    { 0x1aeaabeb, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2aabef, SUC_lexicon_tags_43 },
    { 0x5f2aabf0, SUC_lexicon_tags_37 },
    { 0xa56aabf0, SUC_lexicon_tags_19 },
    { 0x626aabf2, SUC_lexicon_tags_2 },
    { 0xfb0aabf3, SUC_lexicon_tags_12 },
    { 0x8d8aabf4, SUC_lexicon_tags_1 },
    { 0xa72aabf5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88aabfa, SUC_lexicon_tags_10 },
    { 0x3aaaabfa, SUC_lexicon_tags_12 },
    { 0xb96aabfa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x91aaabfe, SUC_lexicon_tags_25 },
    { 0x6a2aabff, SUC_lexicon_tags_17 },
    { 0x404aabff, SUC_lexicon_tags_37 },
    { 0x702aac00, SUC_lexicon_tags_6 },
    { 0x934aac02, SUC_lexicon_tags_12 },
    { 0x364aac01, SUC_lexicon_tags_12 },
    { 0x74aaabfe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8aac09, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf92aac0b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9eaac0d, SUC_lexicon_tags_92 },
    { 0xf2caac0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6aac11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9aaac18, SUC_lexicon_tags_12 },
    { 0x510aac18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542aac1f, SUC_lexicon_tags_7 },
    { 0x87eaac20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25caac23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfeaac25, SUC_lexicon_tags_46 },
    { 0x22aaac26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10aac2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb86aac2c, SUC_lexicon_tags_7 },
    { 0xc7caac2c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6aaac30, SUC_lexicon_tags_63 },
    { 0x46eaac31, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13caac38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7caac3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x648aac3e, SUC_lexicon_tags_49 },
    { 0x6eaac3f, SUC_lexicon_tags_37 },
    { 0x8deaac40, SUC_lexicon_tags_29 },
    { 0x420aac41, SUC_lexicon_tags_43 },
    { 0xa20aac42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aac47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x96aac49, SUC_lexicon_tags_151 },
    { 0x13eaac4a, SUC_lexicon_tags_65 },
    { 0x7fcaac4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8aac53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38aac55, SUC_lexicon_tags_12 },
    { 0x7e2aac56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceaac59, SUC_lexicon_tags_15 },
    { 0xcc2aac5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedeaac5d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x200aac5f, SUC_lexicon_tags_37 },
    { 0x14caac60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2aac65, SUC_lexicon_tags_23 },
    { 0xdc4aac65, SUC_lexicon_tags_2 },
    { 0x626aac65, SUC_lexicon_tags_1 },
    { 0xe6caac68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44eaac6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c2aac6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8feaac74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x256aac76, SUC_lexicon_tags_11 },
    { 0xc5caac76, SUC_lexicon_tags_37 },
    { 0x2aac78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19aaac7c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766aac80, SUC_lexicon_tags_27 },
    { 0xea0aac81, SUC_lexicon_tags_28 },
    { 0x20aac82, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154aac86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47aaac88, SUC_lexicon_tags_2 },
    { 0x758aac88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0aac8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aaac91, SUC_lexicon_tags_29 },
    { 0x8daaac91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46aac97, SUC_lexicon_tags_53 },
    { 0x6d0aac98, SUC_lexicon_tags_14 },
    { 0x54aac99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x696aac9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb10aac9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1daaaca0, SUC_lexicon_tags_12 },
    { 0x650aaca1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb52aaca3, SUC_lexicon_tags_12 },
    { 0xf50aaca4, SUC_lexicon_tags_27 },
    { 0x25caaca5, SUC_lexicon_tags_11 },
    { 0x76caaca6, SUC_lexicon_tags_43 },
    { 0x2fcaaca7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58aacaf, SUC_lexicon_tags_37 },
    { 0x7d8aacb0, SUC_lexicon_tags_37 },
    { 0x11caacb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96caacb4, SUC_lexicon_tags_0 },
    { 0x474aacb4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92aacba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894aacbf, SUC_lexicon_tags_7 },
    { 0xd1caacc0, SUC_lexicon_tags_28 },
    { 0x5a4aacc1, SUC_lexicon_tags_27 },
    { 0x8deaacc2, SUC_lexicon_tags_7 },
    { 0x94eaacc3, SUC_lexicon_tags_28 },
    { 0xb3caacc4, SUC_lexicon_tags_3 },
    { 0x882aacc5, SUC_lexicon_tags_47 },
    { 0xd88aacc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2aacc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ceaaccf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498aacd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b6aacd9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb56aacdb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4aace1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x66eaace4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16eaace9, SUC_lexicon_tags_8 },
    { 0x118aace9, SUC_lexicon_tags_12 },
    { 0x7e4aaceb, SUC_lexicon_tags_0 },
    { 0x3aeaaceb, SUC_lexicon_tags_6 },
    { 0xbf8aaced, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3aaacf0, SUC_lexicon_tags_140 },
    { 0x558aacf1, SUC_lexicon_tags_12 },
    { 0xd4aaacf2, SUC_lexicon_tags_28 },
    { 0xf18aacf3, SUC_lexicon_tags_46 },
    { 0x32aaacf3, SUC_lexicon_tags_60 },
    { 0x5e6aacf5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874aacf9, SUC_lexicon_tags_17 },
    { 0x990aacfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb6aacfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34aacff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb06aad01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452aad0a, SUC_lexicon_tags_15 },
    { 0x2a0aad0b, SUC_lexicon_tags_47 },
    { 0x7eaad0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecaad0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8aad13, SUC_lexicon_tags_6 },
    { 0xd3aaad14, SUC_lexicon_tags_39 },
    { 0x582aad13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26aad1a, SUC_lexicon_tags_37 },
    { 0x98aaad1b, SUC_lexicon_tags_16 },
    { 0x24caad1c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eaad26, SUC_lexicon_tags_25 },
    { 0x172aad27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x782aad2a, SUC_lexicon_tags_9 },
    { 0x3deaad2b, SUC_lexicon_tags_7 },
    { 0x44aad2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daaad30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262aad34, SUC_lexicon_tags_15 },
    { 0x9ecaad35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86aad38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4aad3f, SUC_lexicon_tags_55 },
    { 0x2ecaad40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46aad42, SUC_lexicon_tags_7 },
    { 0x410aad42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baaad4b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x134aad4e, SUC_lexicon_tags_40 },
    { 0x888aad4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186aad54, SUC_lexicon_tags_25 },
    { 0xaa4aad55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0aad59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854aad5e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x358aad60, SUC_lexicon_tags_37 },
    { 0x908aad60, SUC_lexicon_tags_31 },
    { 0xdd0aad61, SUC_lexicon_tags_17 },
    { 0x5d8aad60, SUC_lexicon_tags_15 },
    { 0x644aad64, SUC_lexicon_tags_37 },
    { 0x8c0aad65, SUC_lexicon_tags_29 },
    { 0xaa2aad66, SUC_lexicon_tags_2 },
    { 0x56eaad65, SUC_lexicon_tags_33 },
    { 0xc5caad64, SUC_lexicon_tags_31 },
    { 0x83eaad69, SUC_lexicon_tags_1061 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80aad6d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8aad73, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6e0aad75, SUC_lexicon_tags_66 },
    { 0x51eaad76, SUC_lexicon_tags_27 },
    { 0xdaaaad77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x20aad7a, SUC_lexicon_tags_11 },
    { 0x846aad7b, SUC_lexicon_tags_17 },
    { 0x35eaad7c, SUC_lexicon_tags_8 },
    { 0x7eaaad7d, SUC_lexicon_tags_2 },
    { 0x6e4aad7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1aaad80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84aad83, SUC_lexicon_tags_17 },
    { 0xc9aaad84, SUC_lexicon_tags_23 },
    { 0xe20aad83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648aad89, SUC_lexicon_tags_17 },
    { 0x670aad89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b6aad8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x392aad8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe00aad90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68caad92, SUC_lexicon_tags_37 },
    { 0xdceaad93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x220aad95, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34aad9d, SUC_lexicon_tags_63 },
    { 0xd98aad9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9eaada0, SUC_lexicon_tags_12 },
    { 0x770aada1, SUC_lexicon_tags_47 },
    { 0x756aada1, SUC_lexicon_tags_47 },
    { 0xe8eaada3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b6aada5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcaada9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aaadb1, SUC_lexicon_tags_62 },
    { 0x43caadb2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0aadb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6aadbc, SUC_lexicon_tags_23 },
    { 0x46aadbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2dcaadbf, SUC_lexicon_tags_48 },
    { 0x49aaadc0, SUC_lexicon_tags_46 },
    { 0x704aadc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x642aadc3, SUC_lexicon_tags_70 },
    { 0x9aaadc4, SUC_lexicon_tags_17 },
    { 0xbd8aadc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154aadc9, SUC_lexicon_tags_29 },
    { 0xbaaadca, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4aadcd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x47eaadcf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676aadd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x930aadd6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfe0aadd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x262aaddb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8d6aaddd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98aade0, SUC_lexicon_tags_26 },
    { 0x48eaade1, SUC_lexicon_tags_66 },
    { 0x1f6aade2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x838aade4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754aade9, SUC_lexicon_tags_1 },
    { 0x68caadea, SUC_lexicon_tags_2 },
    { 0x70aaadea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98aadf0, SUC_lexicon_tags_47 },
    { 0x9a0aadf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80aadf8, SUC_lexicon_tags_37 },
    { 0x560aadf8, SUC_lexicon_tags_77 },
    { 0xd2caadf9, SUC_lexicon_tags_11 },
    { 0xb5aaadfb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9deaadfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96aae0c, SUC_lexicon_tags_43 },
    { 0xd7eaae0d, SUC_lexicon_tags_63 },
    { 0x5a4aae0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8aae13, SUC_lexicon_tags_26 },
    { 0xa08aae13, SUC_lexicon_tags_11 },
    { 0x4e0aae13, SUC_lexicon_tags_29 },
    { 0xf6caae16, SUC_lexicon_tags_23 },
    { 0x626aae16, SUC_lexicon_tags_30 },
    { 0xf78aae17, SUC_lexicon_tags_1 },
    { 0xddaaae19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd7aaae1b, SUC_lexicon_tags_11 },
    { 0xe88aae1b, SUC_lexicon_tags_60 },
    { 0x5fcaae1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4aae20, SUC_lexicon_tags_37 },
    { 0x236aae21, SUC_lexicon_tags_47 },
    { 0x7feaae22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f4aae24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x596aae26, SUC_lexicon_tags_1 },
    { 0x66aaae26, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eaae2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430aae32, SUC_lexicon_tags_25 },
    { 0xa18aae33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd04aae35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49eaae39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x59eaae3c, SUC_lexicon_tags_29 },
    { 0x304aae3d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccaae44, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2aae48, SUC_lexicon_tags_84 },
    { 0x246aae49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4aae4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x642aae50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcaae54, SUC_lexicon_tags_45 },
    { 0x2bcaae55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24aae58, SUC_lexicon_tags_978 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8aae5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8aaae5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a0aae60, SUC_lexicon_tags_25 },
    { 0x6a0aae60, SUC_lexicon_tags_2 },
    { 0x422aae62, SUC_lexicon_tags_11 },
    { 0x6f6aae63, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6aae68, SUC_lexicon_tags_12 },
    { 0xc5eaae68, SUC_lexicon_tags_29 },
    { 0x466aae6a, SUC_lexicon_tags_23 },
    { 0x8eaae6b, SUC_lexicon_tags_7 },
    { 0xd96aae6a, SUC_lexicon_tags_31 },
    { 0x8b6aae6d, SUC_lexicon_tags_79 },
    { 0x61eaae6e, SUC_lexicon_tags_37 },
    { 0xecaaae6c, SUC_lexicon_tags_57 },
    { 0xa7caae70, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x14aaae72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16aae7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52aae80, SUC_lexicon_tags_12 },
    { 0x21aaae80, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaaae85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6aae89, SUC_lexicon_tags_0 },
    { 0x288aae8a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xaeaae8c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864aae92, SUC_lexicon_tags_37 },
    { 0xa0aaae93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x236aae96, SUC_lexicon_tags_12 },
    { 0x962aae96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28eaae99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24caae9b, SUC_lexicon_tags_7 },
    { 0xd90aae9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5aaae9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0aaea1, SUC_lexicon_tags_4 },
    { 0xf30aaea2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6aaea9, SUC_lexicon_tags_25 },
    { 0x8caaaeaa, SUC_lexicon_tags_11 },
    { 0x9fcaaeab, SUC_lexicon_tags_2 },
    { 0xf26aaeac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc70aaeae, SUC_lexicon_tags_1 },
    { 0xae4aaeaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaaaeb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0aaeb9, SUC_lexicon_tags_33 },
    { 0x2c6aaeba, SUC_lexicon_tags_28 },
    { 0x254aaebb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2eaaebd, SUC_lexicon_tags_1 },
    { 0x9eaaebe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78aaaec0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x864aaec2, SUC_lexicon_tags_10 },
    { 0x69aaaec2, SUC_lexicon_tags_2 },
    { 0xbd0aaec4, SUC_lexicon_tags_29 },
    { 0x728aaec4, SUC_lexicon_tags_23 },
    { 0xccaaec5, SUC_lexicon_tags_43 },
    { 0xad2aaec7, SUC_lexicon_tags_10 },
    { 0xcd6aaec7, SUC_lexicon_tags_25 },
    { 0xe86aaec8, SUC_lexicon_tags_45 },
    { 0xa2aaaec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54aaecc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x998aaece, SUC_lexicon_tags_15 },
    { 0xaf4aaecf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x110aaed1, SUC_lexicon_tags_14 },
    { 0x58caaed2, SUC_lexicon_tags_15 },
    { 0x56aaed2, SUC_lexicon_tags_2 },
    { 0x522aaed2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x598aaed6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x216aaed8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aaedd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0aaaee2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2aaee6, SUC_lexicon_tags_15 },
    { 0xe26aaee7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20aaaeed, SUC_lexicon_tags_1 },
    { 0xedeaaeee, SUC_lexicon_tags_2 },
    { 0xa68aaeef, SUC_lexicon_tags_47 },
    { 0xee2aaef0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x416aaef2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aaaef8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52caaf00, SUC_lexicon_tags_66 },
    { 0xac8aaf00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb12aaf03, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6aaf09, SUC_lexicon_tags_17 },
    { 0x9f6aaf09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x932aaf0c, SUC_lexicon_tags_12 },
    { 0xa88aaf0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaaaf13, SUC_lexicon_tags_37 },
    { 0xc56aaf14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6aaf1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaaf1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaaaf23, SUC_lexicon_tags_66 },
    { 0x6aaaaf23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96aaf26, SUC_lexicon_tags_109 },
    { 0x444aaf27, SUC_lexicon_tags_15 },
    { 0x29caaf28, SUC_lexicon_tags_11 },
    { 0xd9aaaf29, SUC_lexicon_tags_37 },
    { 0x20aaaf28, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x63caaf2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16aaf30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b0aaf32, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe2caaf34, SUC_lexicon_tags_27 },
    { 0x1a8aaf35, SUC_lexicon_tags_17 },
    { 0x818aaf35, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268aaf3d, SUC_lexicon_tags_25 },
    { 0x6e6aaf3e, SUC_lexicon_tags_2 },
    { 0x9aeaaf3d, SUC_lexicon_tags_30 },
    { 0x4aeaaf40, SUC_lexicon_tags_25 },
    { 0x366aaf41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27caaf45, SUC_lexicon_tags_11 },
    { 0x602aaf46, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298aaf4f, SUC_lexicon_tags_84 },
    { 0xbb0aaf50, SUC_lexicon_tags_19 },
    { 0xb70aaf51, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x26aaaf54, SUC_lexicon_tags_37 },
    { 0xb5eaaf55, SUC_lexicon_tags_25 },
    { 0x8feaaf55, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x28aaaf58, SUC_lexicon_tags_49 },
    { 0x962aaf58, SUC_lexicon_tags_37 },
    { 0x16aaf59, SUC_lexicon_tags_7 },
    { 0x6beaaf58, SUC_lexicon_tags_12 },
    { 0x27eaaf5c, SUC_lexicon_tags_17 },
    { 0x570aaf58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22aaf5f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86caaf63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x574aaf65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0aaf6b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x460aaf6d, SUC_lexicon_tags_59 },
    { 0xcd0aaf6e, SUC_lexicon_tags_1 },
    { 0x560aaf6d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620aaf73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06aaf76, SUC_lexicon_tags_15 },
    { 0x182aaf76, SUC_lexicon_tags_7 },
    { 0xcfaaaf77, SUC_lexicon_tags_47 },
    { 0xb02aaf79, SUC_lexicon_tags_17 },
    { 0x476aaf76, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0aaf7f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc5eaaf81, SUC_lexicon_tags_3 },
    { 0x2a8aaf81, SUC_lexicon_tags_37 },
    { 0x348aaf83, SUC_lexicon_tags_7 },
    { 0x3dcaaf82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4aaf87, SUC_lexicon_tags_3 },
    { 0x996aaf88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46aaf8d, SUC_lexicon_tags_705 },
    { 0x918aaf8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x112aaf90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x118aaf92, SUC_lexicon_tags_23 },
    { 0x6b4aaf93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eaaf9b, SUC_lexicon_tags_11 },
    { 0xe2caaf9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762aafa9, SUC_lexicon_tags_5 },
    { 0x600aafaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcaafaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34aafb3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x340aafb6, SUC_lexicon_tags_7 },
    { 0xb80aafb7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7bcaafb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4caaafbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e2aafc1, SUC_lexicon_tags_49 },
    { 0x74caafc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12caafc4, SUC_lexicon_tags_44 },
    { 0x892aafc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x920aafc8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4f8aafca, SUC_lexicon_tags_102 },
    { 0x67caafca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe36aafcd, SUC_lexicon_tags_113 },
    { 0x5f8aafce, SUC_lexicon_tags_14 },
    { 0xb76aafce, SUC_lexicon_tags_11 },
    { 0xdecaafcf, SUC_lexicon_tags_25 },
    { 0x3b6aafd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44aafd5, SUC_lexicon_tags_38 },
    { 0x120aafd6, SUC_lexicon_tags_37 },
    { 0x94eaafd7, SUC_lexicon_tags_37 },
    { 0x232aafd5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8aafde, SUC_lexicon_tags_60 },
    { 0xbecaafde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81caafe7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a8aafe9, SUC_lexicon_tags_20 },
    { 0x3c4aafea, SUC_lexicon_tags_44 },
    { 0x202aafeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8aafee, SUC_lexicon_tags_33 },
    { 0x838aafef, SUC_lexicon_tags_32 },
    { 0x890aafef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54aaff7, SUC_lexicon_tags_0 },
    { 0x412aaff8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaaafff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448ab00c, SUC_lexicon_tags_7 },
    { 0xff2ab00d, SUC_lexicon_tags_15 },
    { 0x5beab00c, SUC_lexicon_tags_15 },
    { 0x82ab00f, SUC_lexicon_tags_30 },
    { 0x2c0ab010, SUC_lexicon_tags_26 },
    { 0xe9eab011, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd7eab013, SUC_lexicon_tags_11 },
    { 0xb04ab014, SUC_lexicon_tags_6 },
    { 0xb8cab015, SUC_lexicon_tags_1 },
    { 0xe46ab016, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6bcab018, SUC_lexicon_tags_45 },
    { 0xaeeab019, SUC_lexicon_tags_46 },
    { 0x8feab01a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48ab01d, SUC_lexicon_tags_31 },
    { 0xcb6ab01e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x706ab020, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3cab022, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ab025, SUC_lexicon_tags_83 },
    { 0x34aab026, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4ab02b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x462ab02d, SUC_lexicon_tags_6 },
    { 0xef0ab02e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xabaab031, SUC_lexicon_tags_15 },
    { 0x47cab032, SUC_lexicon_tags_37 },
    { 0x740ab032, SUC_lexicon_tags_30 },
    { 0x5c0ab031, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe36ab036, SUC_lexicon_tags_17 },
    { 0x1b2ab037, SUC_lexicon_tags_32 },
    { 0x5eaab037, SUC_lexicon_tags_40 },
    { 0xfe0ab037, SUC_lexicon_tags_30 },
    { 0xb98ab037, SUC_lexicon_tags_33 },
    { 0xf36ab039, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcb4ab03d, SUC_lexicon_tags_30 },
    { 0xc64ab03e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x836ab040, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38ab043, SUC_lexicon_tags_38 },
    { 0x150ab044, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ecab046, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7beab048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2aab04a, SUC_lexicon_tags_25 },
    { 0xbb0ab04b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2ab04d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde2ab04f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2ab058, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe94ab05a, SUC_lexicon_tags_15 },
    { 0xcbaab05a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaab05f, SUC_lexicon_tags_23 },
    { 0xdcaab060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x136ab063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x202ab065, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9a8ab067, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22ab06c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5eab06e, SUC_lexicon_tags_1 },
    { 0xb70ab06e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6ab075, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x854ab077, SUC_lexicon_tags_31 },
    { 0x74ab078, SUC_lexicon_tags_11 },
    { 0x9acab077, SUC_lexicon_tags_37 },
    { 0x53cab07a, SUC_lexicon_tags_1 },
    { 0x3eaab07b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaab082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9caab084, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x38eab086, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebaab089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19aab08b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x614ab08d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3b0ab08f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aab092, SUC_lexicon_tags_11 },
    { 0x2a2ab093, SUC_lexicon_tags_33 },
    { 0x932ab094, SUC_lexicon_tags_11 },
    { 0xb66ab093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4ab09d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ab0a0, SUC_lexicon_tags_15 },
    { 0xb76ab0a1, SUC_lexicon_tags_17 },
    { 0x4aeab0a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbf4ab0a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70ab0a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ab0ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34aab0b3, SUC_lexicon_tags_117 },
    { 0xbc6ab0b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7dcab0b6, SUC_lexicon_tags_31 },
    { 0x7d0ab0b7, SUC_lexicon_tags_11 },
    { 0x8cab0b8, SUC_lexicon_tags_17 },
    { 0xffcab0b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86ab0bd, SUC_lexicon_tags_69 },
    { 0x5ecab0bd, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138ab0c3, SUC_lexicon_tags_7 },
    { 0xe36ab0c4, SUC_lexicon_tags_23 },
    { 0x66aab0c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x718ab0c7, SUC_lexicon_tags_25 },
    { 0xdc4ab0c8, SUC_lexicon_tags_0 },
    { 0x286ab0c9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xaeeab0cb, SUC_lexicon_tags_6 },
    { 0x18cab0cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442ab0d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8ab0d7, SUC_lexicon_tags_11 },
    { 0xe04ab0d8, SUC_lexicon_tags_15 },
    { 0xf92ab0d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9aab0db, SUC_lexicon_tags_100 },
    { 0xe30ab0dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcab0e0, SUC_lexicon_tags_60 },
    { 0x7b6ab0e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb7cab0e3, SUC_lexicon_tags_23 },
    { 0x8eaab0e4, SUC_lexicon_tags_7 },
    { 0x59eab0e5, SUC_lexicon_tags_17 },
    { 0xc08ab0e5, SUC_lexicon_tags_66 },
    { 0x2eeab0e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa70ab0e9, SUC_lexicon_tags_7 },
    { 0x704ab0ea, SUC_lexicon_tags_49 },
    { 0xe90ab0ea, SUC_lexicon_tags_46 },
    { 0x832ab0eb, SUC_lexicon_tags_15 },
    { 0xb54ab0ed, SUC_lexicon_tags_2 },
    { 0x9a8ab0ed, SUC_lexicon_tags_15 },
    { 0x55aab0ef, SUC_lexicon_tags_2 },
    { 0xd0aab0ee, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2ab0f7, SUC_lexicon_tags_14 },
    { 0xebcab0f7, SUC_lexicon_tags_12 },
    { 0xd6cab0f9, SUC_lexicon_tags_99 },
    { 0xd2aab0fa, SUC_lexicon_tags_10 },
    { 0x5f4ab0f9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44ab100, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98ab105, SUC_lexicon_tags_12 },
    { 0x8caab106, SUC_lexicon_tags_7 },
    { 0x1e6ab107, SUC_lexicon_tags_69 },
    { 0x78ab105, SUC_lexicon_tags_47 },
    { 0x1eab106, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xbeeab10b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cab110, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaab113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xceeab116, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60ab11f, SUC_lexicon_tags_33 },
    { 0x962ab120, SUC_lexicon_tags_167 },
    { 0x7a8ab121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254ab125, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaab128, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0ab12b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4ab12e, SUC_lexicon_tags_15 },
    { 0x608ab12f, SUC_lexicon_tags_17 },
    { 0xaa2ab12f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44ab133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcab138, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eab13d, SUC_lexicon_tags_342 },
    { 0, NULL },
    { 0x48eab13f, SUC_lexicon_tags_12 },
    { 0xf20ab140, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2ab145, SUC_lexicon_tags_55 },
    { 0x942ab146, SUC_lexicon_tags_37 },
    { 0xedaab146, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x910ab14a, SUC_lexicon_tags_21 },
    { 0x5ccab14b, SUC_lexicon_tags_2 },
    { 0x2eaab14a, SUC_lexicon_tags_26 },
    { 0x26eab14a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0ab153, SUC_lexicon_tags_57 },
    { 0xd70ab154, SUC_lexicon_tags_25 },
    { 0xcaaab154, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x814ab158, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4ab15b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x808ab15e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4ab165, SUC_lexicon_tags_28 },
    { 0x67aab166, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8faab169, SUC_lexicon_tags_11 },
    { 0x3bcab169, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf32ab16c, SUC_lexicon_tags_6 },
    { 0x60cab16d, SUC_lexicon_tags_17 },
    { 0xc42ab16d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x594ab170, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x44aab173, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ab176, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588ab17b, SUC_lexicon_tags_3 },
    { 0xb98ab17c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d2ab17e, SUC_lexicon_tags_68 },
    { 0xb1eab17f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8ab183, SUC_lexicon_tags_37 },
    { 0x4a2ab184, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x6b4ab186, SUC_lexicon_tags_384 },
    { 0xd18ab186, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb18ab189, SUC_lexicon_tags_919 },
    { 0xe5eab18a, SUC_lexicon_tags_57 },
    { 0xb02ab18b, SUC_lexicon_tags_33 },
    { 0xc32ab18c, SUC_lexicon_tags_7 },
    { 0x132ab18d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x12cab18f, SUC_lexicon_tags_12 },
    { 0x3eeab190, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90ab196, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x114ab199, SUC_lexicon_tags_11 },
    { 0x8e0ab19a, SUC_lexicon_tags_1 },
    { 0xf68ab19b, SUC_lexicon_tags_39 },
    { 0x69cab19c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75cab19e, SUC_lexicon_tags_37 },
    { 0xdd4ab19e, SUC_lexicon_tags_11 },
    { 0xd76ab1a0, SUC_lexicon_tags_23 },
    { 0x94ab1a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc8ab1a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb42ab1a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x958ab1a8, SUC_lexicon_tags_12 },
    { 0xddcab1a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8eab1ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952ab1b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x374ab1b2, SUC_lexicon_tags_30 },
    { 0x542ab1b2, SUC_lexicon_tags_39 },
    { 0xcb0ab1b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aab1b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280ab1be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9eab1c1, SUC_lexicon_tags_47 },
    { 0xa90ab1c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfdeab1c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf6ab1c6, SUC_lexicon_tags_37 },
    { 0xc0aab1c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2f6ab1c9, SUC_lexicon_tags_26 },
    { 0xcb2ab1ca, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8ab1cd, SUC_lexicon_tags_45 },
    { 0xebeab1cd, SUC_lexicon_tags_48 },
    { 0x6c8ab1cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x156ab1d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18ab1d4, SUC_lexicon_tags_17 },
    { 0xdaeab1d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x872ab1d7, SUC_lexicon_tags_74 },
    { 0xe8aab1d8, SUC_lexicon_tags_9 },
    { 0x36ab1d8, SUC_lexicon_tags_0 },
    { 0xe7eab1da, SUC_lexicon_tags_12 },
    { 0x70cab1da, SUC_lexicon_tags_26 },
    { 0xe00ab1db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ab1e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dcab1e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x65aab1e4, SUC_lexicon_tags_23 },
    { 0xda8ab1e5, SUC_lexicon_tags_60 },
    { 0x5e0ab1e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99aab1e8, SUC_lexicon_tags_1 },
    { 0x708ab1e9, SUC_lexicon_tags_1 },
    { 0x750ab1ea, SUC_lexicon_tags_37 },
    { 0xc0ab1e9, SUC_lexicon_tags_29 },
    { 0xa3eab1e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xddeab1ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6ab1f3, SUC_lexicon_tags_39 },
    { 0xbf2ab1f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20eab1f6, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0ab1ff, SUC_lexicon_tags_17 },
    { 0xfb6ab1ff, SUC_lexicon_tags_19 },
    { 0x23cab201, SUC_lexicon_tags_25 },
    { 0x1e4ab202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x364ab205, SUC_lexicon_tags_9 },
    { 0x824ab205, SUC_lexicon_tags_19 },
    { 0x392ab205, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f0ab209, SUC_lexicon_tags_12 },
    { 0x3aeab20a, SUC_lexicon_tags_30 },
    { 0x45aab209, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6ab20e, SUC_lexicon_tags_12 },
    { 0x6deab20e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cab213, SUC_lexicon_tags_16 },
    { 0xfcaab214, SUC_lexicon_tags_11 },
    { 0x45eab214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72ab21c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0ab21f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6cab221, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8ab226, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0ab22b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584ab232, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4cab234, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x35eab237, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecab23b, SUC_lexicon_tags_2 },
    { 0x72ab23c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x84eab23f, SUC_lexicon_tags_11 },
    { 0x9ceab240, SUC_lexicon_tags_11 },
    { 0x1aeab240, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29aab245, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcab24a, SUC_lexicon_tags_11 },
    { 0x6b0ab24a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6beab24e, SUC_lexicon_tags_33 },
    { 0x55cab24f, SUC_lexicon_tags_52 },
    { 0x73aab250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x748ab253, SUC_lexicon_tags_42 },
    { 0xe88ab253, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6ab258, SUC_lexicon_tags_11 },
    { 0x7daab258, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x564ab25b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cab25e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374ab262, SUC_lexicon_tags_2 },
    { 0x480ab263, SUC_lexicon_tags_19 },
    { 0x50cab264, SUC_lexicon_tags_63 },
    { 0x16ab265, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6eab267, SUC_lexicon_tags_9 },
    { 0xbcab268, SUC_lexicon_tags_13 },
    { 0x8b4ab269, SUC_lexicon_tags_2 },
    { 0x82cab26a, SUC_lexicon_tags_26 },
    { 0xca4ab26a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338ab272, SUC_lexicon_tags_43 },
    { 0xb9eab273, SUC_lexicon_tags_25 },
    { 0xc60ab274, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5b6ab276, SUC_lexicon_tags_16 },
    { 0xb6ab277, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd38ab279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61eab27b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78eab27e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcab287, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2eab289, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1cab28b, SUC_lexicon_tags_38 },
    { 0x546ab28b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x502ab28e, SUC_lexicon_tags_2 },
    { 0x71eab28e, SUC_lexicon_tags_46 },
    { 0xc2aab290, SUC_lexicon_tags_28 },
    { 0x33eab28e, SUC_lexicon_tags_1 },
    { 0x334ab292, SUC_lexicon_tags_2 },
    { 0xa84ab293, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4ab299, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34ab29d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbeab2a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37aab2a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102ab2ab, SUC_lexicon_tags_30 },
    { 0x830ab2ac, SUC_lexicon_tags_17 },
    { 0xd62ab2ad, SUC_lexicon_tags_42 },
    { 0x2ab2ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd08ab2b0, SUC_lexicon_tags_33 },
    { 0xdb0ab2b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46ab2b3, SUC_lexicon_tags_27 },
    { 0x182ab2b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96ab2b9, SUC_lexicon_tags_25 },
    { 0x12aab2ba, SUC_lexicon_tags_47 },
    { 0xdaab2bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8ab2be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa08ab2c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x65cab2c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa70ab2c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eab2c9, SUC_lexicon_tags_1 },
    { 0xe68ab2ca, SUC_lexicon_tags_37 },
    { 0xaaab2c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe30ab2cd, SUC_lexicon_tags_38 },
    { 0x90aab2ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2ab2d1, SUC_lexicon_tags_16 },
    { 0xfb8ab2d2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74eab2d9, SUC_lexicon_tags_25 },
    { 0x32eab2da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80ab2dd, SUC_lexicon_tags_19 },
    { 0x334ab2de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cab2e2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x216ab2e5, SUC_lexicon_tags_17 },
    { 0xcf6ab2e5, SUC_lexicon_tags_23 },
    { 0x39eab2e5, SUC_lexicon_tags_14 },
    { 0xff2ab2e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x380ab2eb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8ab2f8, SUC_lexicon_tags_33 },
    { 0xf82ab2f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2ab303, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592ab30c, SUC_lexicon_tags_26 },
    { 0xfe6ab30d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x87eab30f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x594ab311, SUC_lexicon_tags_27 },
    { 0x6fcab312, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cab315, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x918ab317, SUC_lexicon_tags_21 },
    { 0x4caab318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa48ab31a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a0ab31c, SUC_lexicon_tags_63 },
    { 0x114ab31c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6cab31f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96ab325, SUC_lexicon_tags_6 },
    { 0x1c4ab326, SUC_lexicon_tags_10 },
    { 0x36aab327, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974ab32c, SUC_lexicon_tags_8 },
    { 0x542ab32d, SUC_lexicon_tags_17 },
    { 0xc26ab32d, SUC_lexicon_tags_15 },
    { 0xaa4ab32f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaab339, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eab342, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250ab348, SUC_lexicon_tags_1 },
    { 0x120ab349, SUC_lexicon_tags_45 },
    { 0xc08ab34a, SUC_lexicon_tags_26 },
    { 0x87cab34b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeab34d, SUC_lexicon_tags_25 },
    { 0x9aeab34e, SUC_lexicon_tags_84 },
    { 0xbd6ab34e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2ab356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556ab35a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348ab360, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x134ab362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cab36a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x840ab36c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x964ab36f, SUC_lexicon_tags_37 },
    { 0xfbcab36f, SUC_lexicon_tags_2 },
    { 0x9aeab36f, SUC_lexicon_tags_7 },
    { 0xcc8ab370, SUC_lexicon_tags_14 },
    { 0xd38ab373, SUC_lexicon_tags_17 },
    { 0x55aab370, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ab377, SUC_lexicon_tags_43 },
    { 0x6caab378, SUC_lexicon_tags_49 },
    { 0xc74ab378, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xecab37c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa20ab37e, SUC_lexicon_tags_43 },
    { 0x1ceab37e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x458ab381, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1deab383, SUC_lexicon_tags_6 },
    { 0x14aab384, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56ab387, SUC_lexicon_tags_6 },
    { 0x114ab387, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18aab38a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdaaab38c, SUC_lexicon_tags_46 },
    { 0x72ab38c, SUC_lexicon_tags_37 },
    { 0x130ab38e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96eab390, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85cab392, SUC_lexicon_tags_3 },
    { 0x12ab392, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18ab399, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cab39c, SUC_lexicon_tags_35 },
    { 0xacaab39d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2ab3a5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x222ab3a7, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ab3ab, SUC_lexicon_tags_17 },
    { 0xd5aab3ac, SUC_lexicon_tags_17 },
    { 0x2daab3ad, SUC_lexicon_tags_29 },
    { 0xc8cab3ab, SUC_lexicon_tags_84 },
    { 0x234ab3ad, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x684ab3b1, SUC_lexicon_tags_8 },
    { 0x8b2ab3b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26ab3b5, SUC_lexicon_tags_37 },
    { 0x6d6ab3b6, SUC_lexicon_tags_40 },
    { 0xa14ab3b6, SUC_lexicon_tags_11 },
    { 0xa12ab3b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x784ab3ba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34ab3c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x44eab3c3, SUC_lexicon_tags_25 },
    { 0x56cab3c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6ab3ca, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x324ab3cc, SUC_lexicon_tags_11 },
    { 0xdf6ab3cc, SUC_lexicon_tags_35 },
    { 0x4ceab3cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466ab3d3, SUC_lexicon_tags_1 },
    { 0xbeab3d3, SUC_lexicon_tags_43 },
    { 0xd2eab3d4, SUC_lexicon_tags_37 },
    { 0x1c4ab3d5, SUC_lexicon_tags_25 },
    { 0x314ab3d7, SUC_lexicon_tags_17 },
    { 0x26eab3d8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2ab3db, SUC_lexicon_tags_63 },
    { 0xc22ab3dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32ab3e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x442ab3e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358ab3eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2aaab3ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aab3f2, SUC_lexicon_tags_17 },
    { 0xae6ab3f2, SUC_lexicon_tags_25 },
    { 0xdacab3f4, SUC_lexicon_tags_17 },
    { 0x60ab3f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8ab3fe, SUC_lexicon_tags_25 },
    { 0x800ab3ff, SUC_lexicon_tags_84 },
    { 0xc4ab400, SUC_lexicon_tags_0 },
    { 0xc56ab3fe, SUC_lexicon_tags_7 },
    { 0x41aab402, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x140ab404, SUC_lexicon_tags_47 },
    { 0xf4eab405, SUC_lexicon_tags_45 },
    { 0x680ab406, SUC_lexicon_tags_15 },
    { 0x924ab407, SUC_lexicon_tags_7 },
    { 0x282ab408, SUC_lexicon_tags_88 },
    { 0xbd8ab409, SUC_lexicon_tags_23 },
    { 0x53aab408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ab40e, SUC_lexicon_tags_11 },
    { 0xf7eab40f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244ab415, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252ab41b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aab41e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862ab426, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b4ab428, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cab42c, SUC_lexicon_tags_11 },
    { 0x690ab42d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaab435, SUC_lexicon_tags_25 },
    { 0x540ab436, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xef8ab438, SUC_lexicon_tags_35 },
    { 0x7caab438, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84ab43d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0aab441, SUC_lexicon_tags_37 },
    { 0xbc6ab442, SUC_lexicon_tags_23 },
    { 0x936ab443, SUC_lexicon_tags_10 },
    { 0x44aab443, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cab448, SUC_lexicon_tags_11 },
    { 0x57aab449, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6ab44d, SUC_lexicon_tags_38 },
    { 0x1c4ab44d, SUC_lexicon_tags_28 },
    { 0xc08ab44f, SUC_lexicon_tags_19 },
    { 0xf16ab450, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5baab453, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x430ab456, SUC_lexicon_tags_62 },
    { 0xac2ab456, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914ab45b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72ab45e, SUC_lexicon_tags_43 },
    { 0xb6aab45f, SUC_lexicon_tags_79 },
    { 0xaeaab45e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa58ab462, SUC_lexicon_tags_43 },
    { 0x2d8ab463, SUC_lexicon_tags_14 },
    { 0xe42ab462, SUC_lexicon_tags_37 },
    { 0x332ab463, SUC_lexicon_tags_62 },
    { 0xc48ab466, SUC_lexicon_tags_25 },
    { 0x2faab467, SUC_lexicon_tags_11 },
    { 0xb62ab467, SUC_lexicon_tags_70 },
    { 0x28eab468, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x416ab46b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x794ab46e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2ab472, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8ab478, SUC_lexicon_tags_31 },
    { 0x1faab479, SUC_lexicon_tags_7 },
    { 0xa64ab47a, SUC_lexicon_tags_2 },
    { 0x486ab47b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb36ab47d, SUC_lexicon_tags_66 },
    { 0xfd6ab47e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x922ab480, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7aab482, SUC_lexicon_tags_7 },
    { 0x8e2ab483, SUC_lexicon_tags_27 },
    { 0x2aab483, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa24ab486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90aab48a, SUC_lexicon_tags_14 },
    { 0xc20ab48a, SUC_lexicon_tags_42 },
    { 0x5faab48c, SUC_lexicon_tags_7 },
    { 0xba6ab48d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38ab493, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe78ab495, SUC_lexicon_tags_23 },
    { 0x35aab495, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfceab499, SUC_lexicon_tags_2 },
    { 0x1ccab499, SUC_lexicon_tags_39 },
    { 0xd44ab49b, SUC_lexicon_tags_15 },
    { 0x84aab49b, SUC_lexicon_tags_37 },
    { 0x502ab49c, SUC_lexicon_tags_7 },
    { 0x8f6ab49e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cab4a5, SUC_lexicon_tags_34 },
    { 0x5acab4a5, SUC_lexicon_tags_37 },
    { 0x5d8ab4a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb2ab4a9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124ab4ae, SUC_lexicon_tags_11 },
    { 0x9f2ab4af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67aab4b4, SUC_lexicon_tags_9 },
    { 0x490ab4b4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89eab4b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26aab4bb, SUC_lexicon_tags_7 },
    { 0x598ab4bb, SUC_lexicon_tags_318 },
    { 0x8eeab4bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaab4c2, SUC_lexicon_tags_42 },
    { 0x264ab4c2, SUC_lexicon_tags_34 },
    { 0xb6cab4c4, SUC_lexicon_tags_46 },
    { 0x7aaab4c5, SUC_lexicon_tags_37 },
    { 0x3f6ab4c6, SUC_lexicon_tags_68 },
    { 0x2b2ab4c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6eab4c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6ab4cd, SUC_lexicon_tags_8 },
    { 0xebaab4ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab4d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xceab4d7, SUC_lexicon_tags_2 },
    { 0x46aab4d8, SUC_lexicon_tags_0 },
    { 0xc4aab4d8, SUC_lexicon_tags_2 },
    { 0xa9aab4d8, SUC_lexicon_tags_15 },
    { 0x196ab4d8, SUC_lexicon_tags_28 },
    { 0xfa6ab4dc, SUC_lexicon_tags_4 },
    { 0x772ab4d9, SUC_lexicon_tags_3 },
    { 0x2a2ab4de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902ab4e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x434ab4e5, SUC_lexicon_tags_26 },
    { 0xeeaab4e6, SUC_lexicon_tags_15 },
    { 0xd94ab4e7, SUC_lexicon_tags_13 },
    { 0xaacab4e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80ab4ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2ab4f7, SUC_lexicon_tags_79 },
    { 0x396ab4f8, SUC_lexicon_tags_12 },
    { 0x4c6ab4f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aab4fd, SUC_lexicon_tags_1 },
    { 0xf24ab4fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeab501, SUC_lexicon_tags_37 },
    { 0xb48ab501, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224ab508, SUC_lexicon_tags_11 },
    { 0x946ab509, SUC_lexicon_tags_60 },
    { 0x148ab50a, SUC_lexicon_tags_70 },
    { 0x1d8ab50b, SUC_lexicon_tags_8 },
    { 0xcc0ab50c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa78ab50e, SUC_lexicon_tags_45 },
    { 0xd36ab50f, SUC_lexicon_tags_17 },
    { 0x230ab510, SUC_lexicon_tags_11 },
    { 0x506ab50f, SUC_lexicon_tags_15 },
    { 0xb1aab512, SUC_lexicon_tags_1 },
    { 0xa9aab513, SUC_lexicon_tags_37 },
    { 0x5caab514, SUC_lexicon_tags_11 },
    { 0xa0cab514, SUC_lexicon_tags_48 },
    { 0x196ab515, SUC_lexicon_tags_25 },
    { 0x198ab516, SUC_lexicon_tags_68 },
    { 0x10eab518, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2eab51c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa0ab51e, SUC_lexicon_tags_1 },
    { 0x806ab51f, SUC_lexicon_tags_14 },
    { 0x5c6ab51f, SUC_lexicon_tags_37 },
    { 0x28eab51f, SUC_lexicon_tags_29 },
    { 0x80ab520, SUC_lexicon_tags_64 },
    { 0xc8ab523, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4ab528, SUC_lexicon_tags_58 },
    { 0x170ab529, SUC_lexicon_tags_15 },
    { 0xc4aab529, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922ab530, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8aab534, SUC_lexicon_tags_10 },
    { 0x2daab535, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa9eab537, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ab53a, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21aab546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x434ab548, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2aab54a, SUC_lexicon_tags_17 },
    { 0xec6ab54b, SUC_lexicon_tags_37 },
    { 0x6b0ab54a, SUC_lexicon_tags_13 },
    { 0x826ab54b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56ab554, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb5aab556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9cab558, SUC_lexicon_tags_768 },
    { 0, NULL },
    { 0xbfcab55a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8ab55d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aab564, SUC_lexicon_tags_1 },
    { 0x5a4ab564, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x322ab567, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecab577, SUC_lexicon_tags_37 },
    { 0xaf8ab578, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2ab57b, SUC_lexicon_tags_1 },
    { 0x988ab57c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85eab57e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x742ab581, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x944ab583, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72aab586, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcaab588, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2ab58b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28ab58f, SUC_lexicon_tags_62 },
    { 0xc8aab590, SUC_lexicon_tags_43 },
    { 0x542ab591, SUC_lexicon_tags_29 },
    { 0xedaab592, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe4ab594, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22ab5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8ab5a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44ab5ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x450ab5af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358ab5b4, SUC_lexicon_tags_230 },
    { 0x366ab5b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40ab5be, SUC_lexicon_tags_33 },
    { 0x1f0ab5bf, SUC_lexicon_tags_26 },
    { 0x124ab5c0, SUC_lexicon_tags_37 },
    { 0xeacab5c1, SUC_lexicon_tags_9 },
    { 0xd6eab5c2, SUC_lexicon_tags_37 },
    { 0xa80ab5c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcab5cb, SUC_lexicon_tags_11 },
    { 0xe2aab5cc, SUC_lexicon_tags_12 },
    { 0x768ab5cc, SUC_lexicon_tags_7 },
    { 0xe4ab5ce, SUC_lexicon_tags_63 },
    { 0x2b0ab5cd, SUC_lexicon_tags_47 },
    { 0x10eab5cc, SUC_lexicon_tags_37 },
    { 0x60aab5d1, SUC_lexicon_tags_11 },
    { 0x392ab5d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcab5d9, SUC_lexicon_tags_37 },
    { 0x74eab5da, SUC_lexicon_tags_27 },
    { 0xf5eab5db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeceab5dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x326ab5e0, SUC_lexicon_tags_25 },
    { 0x8c0ab5e1, SUC_lexicon_tags_23 },
    { 0x548ab5e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x96cab5e4, SUC_lexicon_tags_2 },
    { 0x24cab5e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2ab5e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x378ab5ea, SUC_lexicon_tags_19 },
    { 0x902ab5eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320ab5f1, SUC_lexicon_tags_63 },
    { 0xedeab5f2, SUC_lexicon_tags_28 },
    { 0x310ab5f3, SUC_lexicon_tags_65 },
    { 0xb18ab5f4, SUC_lexicon_tags_29 },
    { 0xc7cab5f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6ab5fd, SUC_lexicon_tags_2 },
    { 0x95cab5fe, SUC_lexicon_tags_7 },
    { 0x932ab5ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8ab602, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178ab607, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1aab60a, SUC_lexicon_tags_0 },
    { 0xfe4ab60b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31aab60f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6ab615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62cab617, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840ab61c, SUC_lexicon_tags_37 },
    { 0x9feab61d, SUC_lexicon_tags_588 },
    { 0xbd0ab61e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc44ab620, SUC_lexicon_tags_7 },
    { 0xa0eab621, SUC_lexicon_tags_17 },
    { 0xd52ab622, SUC_lexicon_tags_1 },
    { 0xd5cab623, SUC_lexicon_tags_37 },
    { 0x310ab622, SUC_lexicon_tags_55 },
    { 0x1deab625, SUC_lexicon_tags_25 },
    { 0x10cab626, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x98cab628, SUC_lexicon_tags_17 },
    { 0x858ab629, SUC_lexicon_tags_23 },
    { 0x86eab629, SUC_lexicon_tags_14 },
    { 0x778ab62a, SUC_lexicon_tags_8 },
    { 0xa86ab62a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x628ab62f, SUC_lexicon_tags_16 },
    { 0xa46ab630, SUC_lexicon_tags_0 },
    { 0x722ab631, SUC_lexicon_tags_9 },
    { 0x84eab62f, SUC_lexicon_tags_45 },
    { 0x506ab633, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676ab637, SUC_lexicon_tags_46 },
    { 0x122ab637, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44ab644, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81cab646, SUC_lexicon_tags_25 },
    { 0x54eab647, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4aab64a, SUC_lexicon_tags_12 },
    { 0x348ab64b, SUC_lexicon_tags_60 },
    { 0x9e4ab64c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29aab651, SUC_lexicon_tags_25 },
    { 0xa48ab652, SUC_lexicon_tags_12 },
    { 0xfeeab653, SUC_lexicon_tags_6 },
    { 0x3a6ab654, SUC_lexicon_tags_9 },
    { 0x232ab655, SUC_lexicon_tags_23 },
    { 0x776ab656, SUC_lexicon_tags_8 },
    { 0xdf6ab656, SUC_lexicon_tags_12 },
    { 0x818ab653, SUC_lexicon_tags_47 },
    { 0xed6ab659, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28ab65e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aab661, SUC_lexicon_tags_17 },
    { 0xb62ab662, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf4ab664, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70ab66b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab671, SUC_lexicon_tags_12 },
    { 0x61cab672, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186ab679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16ab67d, SUC_lexicon_tags_4 },
    { 0x344ab67e, SUC_lexicon_tags_17 },
    { 0xce4ab67e, SUC_lexicon_tags_23 },
    { 0x256ab680, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8ab683, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438ab68c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62ab68f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00ab695, SUC_lexicon_tags_15 },
    { 0x59eab695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6ab69a, SUC_lexicon_tags_27 },
    { 0x390ab69a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeab6a4, SUC_lexicon_tags_2 },
    { 0x810ab6a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a6ab6a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44ab6aa, SUC_lexicon_tags_63 },
    { 0x37aab6aa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4eaab6ad, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4eaab6af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x730ab6b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2ab6b6, SUC_lexicon_tags_2 },
    { 0xa64ab6b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108ab6bc, SUC_lexicon_tags_124 },
    { 0xe8aab6bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9b8ab6bf, SUC_lexicon_tags_15 },
    { 0x808ab6c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x478ab6c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbf2ab6c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4ab6c7, SUC_lexicon_tags_9 },
    { 0xbf6ab6c8, SUC_lexicon_tags_15 },
    { 0x874ab6c8, SUC_lexicon_tags_45 },
    { 0x5eeab6c9, SUC_lexicon_tags_55 },
    { 0x414ab6ca, SUC_lexicon_tags_28 },
    { 0x58ab6cc, SUC_lexicon_tags_185 },
    { 0x22ab6cd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302ab6d1, SUC_lexicon_tags_15 },
    { 0x65cab6d2, SUC_lexicon_tags_17 },
    { 0xfd6ab6d2, SUC_lexicon_tags_19 },
    { 0xea8ab6d1, SUC_lexicon_tags_42 },
    { 0xbf0ab6d5, SUC_lexicon_tags_2 },
    { 0xdc6ab6d6, SUC_lexicon_tags_25 },
    { 0x56ab6d7, SUC_lexicon_tags_11 },
    { 0xf98ab6d7, SUC_lexicon_tags_63 },
    { 0x4a2ab6d9, SUC_lexicon_tags_15 },
    { 0xfa2ab6d5, SUC_lexicon_tags_2 },
    { 0x91cab6d8, SUC_lexicon_tags_25 },
    { 0x2beab6d4, SUC_lexicon_tags_16 },
    { 0x294ab6da, SUC_lexicon_tags_11 },
    { 0x4eaab6de, SUC_lexicon_tags_45 },
    { 0xf4eab6d6, SUC_lexicon_tags_37 },
    { 0x51eab6e0, SUC_lexicon_tags_11 },
    { 0xa54ab6e1, SUC_lexicon_tags_6 },
    { 0x8a2ab6e1, SUC_lexicon_tags_14 },
    { 0xaf4ab6d9, SUC_lexicon_tags_33 },
    { 0x278ab6e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cab6e8, SUC_lexicon_tags_3 },
    { 0x82eab6e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43eab6ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2ab6f4, SUC_lexicon_tags_39 },
    { 0x3d0ab6f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4ab6f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xaccab6fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x332ab6fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa34ab6ff, SUC_lexicon_tags_17 },
    { 0x9beab700, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0ab704, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaab707, SUC_lexicon_tags_7 },
    { 0x8aaab707, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6ab70e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd34ab710, SUC_lexicon_tags_60 },
    { 0xb2ab710, SUC_lexicon_tags_33 },
    { 0x5f8ab712, SUC_lexicon_tags_11 },
    { 0x5c2ab713, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf8ab715, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc4ab717, SUC_lexicon_tags_10 },
    { 0xfaeab718, SUC_lexicon_tags_15 },
    { 0x59cab717, SUC_lexicon_tags_37 },
    { 0x858ab71a, SUC_lexicon_tags_17 },
    { 0x662ab71a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40ab71f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x838ab721, SUC_lexicon_tags_30 },
    { 0xd02ab722, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0ab72e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfa6ab730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b8ab732, SUC_lexicon_tags_7 },
    { 0x194ab732, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5daab737, SUC_lexicon_tags_17 },
    { 0xb62ab738, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24ab73b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xddcab73e, SUC_lexicon_tags_12 },
    { 0xd72ab73f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85cab741, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ab744, SUC_lexicon_tags_17 },
    { 0xbfcab745, SUC_lexicon_tags_25 },
    { 0x3c6ab746, SUC_lexicon_tags_26 },
    { 0x5caab747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29eab749, SUC_lexicon_tags_19 },
    { 0xab2ab74a, SUC_lexicon_tags_30 },
    { 0xfe2ab74b, SUC_lexicon_tags_49 },
    { 0xb16ab74b, SUC_lexicon_tags_15 },
    { 0x2c4ab74d, SUC_lexicon_tags_33 },
    { 0x500ab74c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76ab751, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfbcab753, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2ab758, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98ab75b, SUC_lexicon_tags_2 },
    { 0x6a4ab75b, SUC_lexicon_tags_25 },
    { 0xf04ab75d, SUC_lexicon_tags_8 },
    { 0x292ab75e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23aab762, SUC_lexicon_tags_29 },
    { 0x43aab763, SUC_lexicon_tags_0 },
    { 0x48ab764, SUC_lexicon_tags_7 },
    { 0x6f8ab763, SUC_lexicon_tags_264 },
    { 0x136ab766, SUC_lexicon_tags_25 },
    { 0x820ab767, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x866ab769, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caab76e, SUC_lexicon_tags_11 },
    { 0x298ab76f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728ab777, SUC_lexicon_tags_2 },
    { 0xd4cab778, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cab77b, SUC_lexicon_tags_45 },
    { 0x5aaab77b, SUC_lexicon_tags_25 },
    { 0x108ab77c, SUC_lexicon_tags_11 },
    { 0x98eab77c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110ab783, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaab78d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde6ab78f, SUC_lexicon_tags_60 },
    { 0x19eab790, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8ab79b, SUC_lexicon_tags_7 },
    { 0xa02ab79c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606ab7a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6ab7a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xceaab7a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x54eab7a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x250ab7ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46ab7af, SUC_lexicon_tags_25 },
    { 0x150ab7af, SUC_lexicon_tags_43 },
    { 0x7aab7b1, SUC_lexicon_tags_17 },
    { 0xcaeab7b1, SUC_lexicon_tags_17 },
    { 0xf0aab7b3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116ab7b7, SUC_lexicon_tags_45 },
    { 0xc92ab7b8, SUC_lexicon_tags_0 },
    { 0x90ab7b9, SUC_lexicon_tags_2 },
    { 0x6b0ab7b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6ab7be, SUC_lexicon_tags_7 },
    { 0x174ab7bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0ab7c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798ab7c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e0ab7c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x100ab7cb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x768ab7cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0ab7d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eab7d5, SUC_lexicon_tags_6 },
    { 0xd4aab7d5, SUC_lexicon_tags_1 },
    { 0xdc8ab7d7, SUC_lexicon_tags_15 },
    { 0x24aab7d7, SUC_lexicon_tags_38 },
    { 0x896ab7d9, SUC_lexicon_tags_39 },
    { 0x692ab7da, SUC_lexicon_tags_9 },
    { 0x41aab7d9, SUC_lexicon_tags_15 },
    { 0x382ab7dc, SUC_lexicon_tags_27 },
    { 0x644ab7dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb60ab7df, SUC_lexicon_tags_37 },
    { 0x2a8ab7e0, SUC_lexicon_tags_77 },
    { 0xe54ab7e1, SUC_lexicon_tags_1 },
    { 0x43cab7e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc00ab7e4, SUC_lexicon_tags_1 },
    { 0x818ab7e5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x13eab7e7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xab7e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x96cab7ec, SUC_lexicon_tags_177 },
    { 0x52eab7ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cab7f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdeab7f4, SUC_lexicon_tags_12 },
    { 0xa34ab7f5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4ab7f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xce6ab7fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7aab800, SUC_lexicon_tags_16 },
    { 0x400ab800, SUC_lexicon_tags_37 },
    { 0x67eab802, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16ab805, SUC_lexicon_tags_32 },
    { 0x58cab805, SUC_lexicon_tags_12 },
    { 0xcab805, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56ab80a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecab80f, SUC_lexicon_tags_55 },
    { 0x9a4ab810, SUC_lexicon_tags_11 },
    { 0xb7aab811, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22eab815, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cab819, SUC_lexicon_tags_43 },
    { 0xb78ab81a, SUC_lexicon_tags_63 },
    { 0xd3eab81a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d0ab81d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x13eab81f, SUC_lexicon_tags_23 },
    { 0x1dcab820, SUC_lexicon_tags_12 },
    { 0x630ab820, SUC_lexicon_tags_25 },
    { 0xadeab822, SUC_lexicon_tags_45 },
    { 0x50ab823, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe90ab826, SUC_lexicon_tags_25 },
    { 0x468ab826, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e8ab829, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28ab82f, SUC_lexicon_tags_12 },
    { 0xb5cab830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x816ab833, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x51aab835, SUC_lexicon_tags_12 },
    { 0xf90ab836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ceab838, SUC_lexicon_tags_38 },
    { 0xf06ab839, SUC_lexicon_tags_4 },
    { 0xb92ab838, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2ab83e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87eab840, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4ab848, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0aab84a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecab851, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16ab854, SUC_lexicon_tags_6 },
    { 0xb02ab855, SUC_lexicon_tags_8 },
    { 0xc48ab856, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x56cab858, SUC_lexicon_tags_33 },
    { 0xd7cab859, SUC_lexicon_tags_8 },
    { 0x706ab85a, SUC_lexicon_tags_37 },
    { 0x394ab859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6ab85e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582ab867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6ab86b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x108ab86e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cab872, SUC_lexicon_tags_7 },
    { 0x2d0ab873, SUC_lexicon_tags_8 },
    { 0x7f0ab874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782ab87b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x982ab87e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e0ab880, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74cab882, SUC_lexicon_tags_49 },
    { 0xed0ab883, SUC_lexicon_tags_34 },
    { 0x35aab883, SUC_lexicon_tags_1 },
    { 0x8aaab885, SUC_lexicon_tags_45 },
    { 0xfbeab883, SUC_lexicon_tags_20 },
    { 0x34ab884, SUC_lexicon_tags_16 },
    { 0xdbcab888, SUC_lexicon_tags_23 },
    { 0x79cab889, SUC_lexicon_tags_49 },
    { 0x886ab888, SUC_lexicon_tags_2 },
    { 0x50ab884, SUC_lexicon_tags_26 },
    { 0xefcab88c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5aab88e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0ab892, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176ab897, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aab89a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cab89d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab8a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8eab8a2, SUC_lexicon_tags_1 },
    { 0xf36ab8a3, SUC_lexicon_tags_14 },
    { 0xf0ab8a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c0ab8a6, SUC_lexicon_tags_649 },
    { 0, NULL },
    { 0x2beab8a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786ab8b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4dcab8b2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe3eab8b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aab8b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2ab8bc, SUC_lexicon_tags_7 },
    { 0xcacab8bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd18ab8bf, SUC_lexicon_tags_63 },
    { 0xf36ab8c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22ab8c3, SUC_lexicon_tags_47 },
    { 0xa4cab8c4, SUC_lexicon_tags_25 },
    { 0xf4ab8c5, SUC_lexicon_tags_15 },
    { 0x516ab8c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aab8c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32ab8cd, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaab8d0, SUC_lexicon_tags_7 },
    { 0x266ab8d0, SUC_lexicon_tags_15 },
    { 0x37eab8d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x536ab8d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942ab8d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90aab8db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26eab8dd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf22ab8df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaab8e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa74ab8e3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2ab8eb, SUC_lexicon_tags_15 },
    { 0x3f6ab8ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xde2ab8ee, SUC_lexicon_tags_38 },
    { 0xbdeab8ef, SUC_lexicon_tags_16 },
    { 0x9aaab8ef, SUC_lexicon_tags_11 },
    { 0x7b8ab8f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab8f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44cab8f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8ab8fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcab8fd, SUC_lexicon_tags_17 },
    { 0xd9eab8fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4aaab900, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eeab904, SUC_lexicon_tags_14 },
    { 0x4daab905, SUC_lexicon_tags_14 },
    { 0x1b8ab905, SUC_lexicon_tags_0 },
    { 0xc88ab905, SUC_lexicon_tags_2 },
    { 0x44eab904, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcab90d, SUC_lexicon_tags_40 },
    { 0x4caab90e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82ab914, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x492ab917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x710ab91a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292ab921, SUC_lexicon_tags_12 },
    { 0x12ab922, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xaa8ab924, SUC_lexicon_tags_27 },
    { 0x866ab925, SUC_lexicon_tags_25 },
    { 0xb3cab926, SUC_lexicon_tags_25 },
    { 0x290ab927, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2e8ab929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992ab92d, SUC_lexicon_tags_17 },
    { 0x532ab92d, SUC_lexicon_tags_37 },
    { 0xe32ab92f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda6ab931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eab93a, SUC_lexicon_tags_1086 },
    { 0, NULL },
    { 0x836ab93c, SUC_lexicon_tags_17 },
    { 0xd30ab93c, SUC_lexicon_tags_21 },
    { 0x512ab93c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x206ab941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x586ab944, SUC_lexicon_tags_15 },
    { 0x58ab945, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x288ab947, SUC_lexicon_tags_1 },
    { 0xe0ab948, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3aab94b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cab950, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xccaab953, SUC_lexicon_tags_15 },
    { 0x864ab953, SUC_lexicon_tags_25 },
    { 0xea0ab955, SUC_lexicon_tags_27 },
    { 0xb54ab954, SUC_lexicon_tags_45 },
    { 0x378ab957, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858ab95b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde8ab95d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebeab95f, SUC_lexicon_tags_43 },
    { 0xa0ab960, SUC_lexicon_tags_47 },
    { 0xa84ab960, SUC_lexicon_tags_43 },
    { 0xb7aab95f, SUC_lexicon_tags_11 },
    { 0xe32ab963, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954ab967, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ab96b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2ab96e, SUC_lexicon_tags_55 },
    { 0x97eab96e, SUC_lexicon_tags_27 },
    { 0x200ab970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34ab974, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x84eab976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86ab97a, SUC_lexicon_tags_55 },
    { 0x962ab97a, SUC_lexicon_tags_30 },
    { 0x36aab97c, SUC_lexicon_tags_350 },
    { 0xef2ab97d, SUC_lexicon_tags_17 },
    { 0x8f2ab97c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88ab985, SUC_lexicon_tags_15 },
    { 0x7aeab986, SUC_lexicon_tags_12 },
    { 0xdb0ab986, SUC_lexicon_tags_46 },
    { 0x72eab988, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe2ab98a, SUC_lexicon_tags_25 },
    { 0x974ab98a, SUC_lexicon_tags_11 },
    { 0x9f2ab98b, SUC_lexicon_tags_25 },
    { 0x752ab98d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x10aab98f, SUC_lexicon_tags_37 },
    { 0xfeab990, SUC_lexicon_tags_12 },
    { 0x9a8ab991, SUC_lexicon_tags_60 },
    { 0x542ab991, SUC_lexicon_tags_15 },
    { 0x3eeab993, SUC_lexicon_tags_11 },
    { 0xae6ab993, SUC_lexicon_tags_2 },
    { 0x33cab994, SUC_lexicon_tags_23 },
    { 0xfb4ab994, SUC_lexicon_tags_39 },
    { 0xb2eab997, SUC_lexicon_tags_37 },
    { 0xb06ab998, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x58ab99a, SUC_lexicon_tags_1 },
    { 0xa7aab99a, SUC_lexicon_tags_11 },
    { 0xe94ab99c, SUC_lexicon_tags_7 },
    { 0xff8ab99b, SUC_lexicon_tags_12 },
    { 0x30eab99e, SUC_lexicon_tags_25 },
    { 0x668ab99f, SUC_lexicon_tags_11 },
    { 0x48cab9a0, SUC_lexicon_tags_11 },
    { 0x37eab9a1, SUC_lexicon_tags_47 },
    { 0xde0ab99d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc0ab9a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32ab9a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd2ab9a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956ab9ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6ab9af, SUC_lexicon_tags_11 },
    { 0xc22ab9b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x728ab9b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x628ab9b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8ab9b7, SUC_lexicon_tags_45 },
    { 0x6dcab9b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2ab9be, SUC_lexicon_tags_30 },
    { 0x80aab9bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x742ab9c1, SUC_lexicon_tags_11 },
    { 0x98cab9c2, SUC_lexicon_tags_61 },
    { 0x78aab9c2, SUC_lexicon_tags_26 },
    { 0x736ab9c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd34ab9c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x532ab9c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf18ab9ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x280ab9cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90ab9ce, SUC_lexicon_tags_113 },
    { 0x346ab9cf, SUC_lexicon_tags_43 },
    { 0x6e0ab9cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6ab9d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x642ab9d6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2ab9dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5deab9e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aab9e6, SUC_lexicon_tags_14 },
    { 0xb52ab9e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa36ab9e9, SUC_lexicon_tags_11 },
    { 0x524ab9e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5cab9ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90ab9ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74aba02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112aba08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8aba0b, SUC_lexicon_tags_37 },
    { 0x93aaba0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6aba12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53caba15, SUC_lexicon_tags_11 },
    { 0x400aba15, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18aba1b, SUC_lexicon_tags_14 },
    { 0x948aba1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92aaba23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600aba27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87eaba31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec2aba33, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4aba38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2aba3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x776aba3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08aba42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58aba48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ccaba4a, SUC_lexicon_tags_63 },
    { 0xa34aba4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118aba53, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcaba5d, SUC_lexicon_tags_14 },
    { 0x852aba5d, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28eaba66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x190aba68, SUC_lexicon_tags_45 },
    { 0xdd0aba68, SUC_lexicon_tags_14 },
    { 0xfcaba6a, SUC_lexicon_tags_14 },
    { 0x808aba6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972aba70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x206aba73, SUC_lexicon_tags_30 },
    { 0x6e8aba74, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6aba77, SUC_lexicon_tags_87 },
    { 0xfe0aba78, SUC_lexicon_tags_39 },
    { 0xad2aba78, SUC_lexicon_tags_25 },
    { 0x3f6aba7a, SUC_lexicon_tags_1 },
    { 0x4c6aba7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaba85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeceaba8a, SUC_lexicon_tags_19 },
    { 0xa14aba8b, SUC_lexicon_tags_25 },
    { 0x3caaba8c, SUC_lexicon_tags_14 },
    { 0x1eaba8c, SUC_lexicon_tags_49 },
    { 0xe90aba8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42aaba90, SUC_lexicon_tags_46 },
    { 0x5fcaba91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81caba94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8beaba9a, SUC_lexicon_tags_17 },
    { 0x456aba9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x22caba9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cabaa2, SUC_lexicon_tags_37 },
    { 0x4deabaa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944abaa9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80abaad, SUC_lexicon_tags_526 },
    { 0, NULL },
    { 0xe2aabaaf, SUC_lexicon_tags_33 },
    { 0xf42abab0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29cabab2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdfaabab4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x472abab7, SUC_lexicon_tags_1 },
    { 0xdb0abab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4aababa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x824ababd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996abac3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630abac8, SUC_lexicon_tags_7 },
    { 0x68eabac8, SUC_lexicon_tags_2 },
    { 0xfb2abaca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaabace, SUC_lexicon_tags_138 },
    { 0x2caabace, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x540abad2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cabada, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64abae0, SUC_lexicon_tags_37 },
    { 0x29cabae1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58aabae3, SUC_lexicon_tags_25 },
    { 0x48eabae4, SUC_lexicon_tags_0 },
    { 0x1a6abae5, SUC_lexicon_tags_7 },
    { 0xe7aabae6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cabaec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aabaef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cabafa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736abaff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4abb05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aabb0a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x84aabb0c, SUC_lexicon_tags_9 },
    { 0xbe0abb0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc4abb0f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1b6abb11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0abb18, SUC_lexicon_tags_12 },
    { 0xb14abb18, SUC_lexicon_tags_27 },
    { 0x4d2abb1a, SUC_lexicon_tags_7 },
    { 0xb08abb1b, SUC_lexicon_tags_47 },
    { 0x4e4abb1c, SUC_lexicon_tags_63 },
    { 0x362abb19, SUC_lexicon_tags_23 },
    { 0xa98abb1e, SUC_lexicon_tags_9 },
    { 0x176abb1e, SUC_lexicon_tags_25 },
    { 0x98eabb20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87cabb22, SUC_lexicon_tags_87 },
    { 0xec8abb22, SUC_lexicon_tags_39 },
    { 0xcd8abb24, SUC_lexicon_tags_39 },
    { 0xe82abb25, SUC_lexicon_tags_57 },
    { 0x302abb26, SUC_lexicon_tags_43 },
    { 0xddcabb27, SUC_lexicon_tags_1 },
    { 0xfe2abb27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26abb2a, SUC_lexicon_tags_1 },
    { 0xbf2abb2b, SUC_lexicon_tags_31 },
    { 0x3eaabb2c, SUC_lexicon_tags_25 },
    { 0xe92abb2d, SUC_lexicon_tags_2 },
    { 0xb3cabb2d, SUC_lexicon_tags_40 },
    { 0xd70abb2c, SUC_lexicon_tags_47 },
    { 0x5d2abb30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x690abb32, SUC_lexicon_tags_60 },
    { 0xa6aabb32, SUC_lexicon_tags_1701 },
    { 0, NULL },
    { 0xa60abb35, SUC_lexicon_tags_25 },
    { 0x77cabb35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8cabb38, SUC_lexicon_tags_27 },
    { 0x948abb38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a8abb3b, SUC_lexicon_tags_7 },
    { 0x940abb3b, SUC_lexicon_tags_38 },
    { 0xac4abb3d, SUC_lexicon_tags_11 },
    { 0x9eabb3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0abb40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92abb44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x576abb46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf3cabb48, SUC_lexicon_tags_46 },
    { 0x8f2abb49, SUC_lexicon_tags_17 },
    { 0xf1aabb49, SUC_lexicon_tags_15 },
    { 0x16aabb49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa4abb4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf02abb4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x680abb52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccabb5c, SUC_lexicon_tags_27 },
    { 0x902abb5c, SUC_lexicon_tags_17 },
    { 0x6acabb5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6abb62, SUC_lexicon_tags_18 },
    { 0x3cabb63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136abb69, SUC_lexicon_tags_15 },
    { 0xfceabb69, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66abb6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98abb70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64abb74, SUC_lexicon_tags_25 },
    { 0x436abb74, SUC_lexicon_tags_25 },
    { 0xc76abb75, SUC_lexicon_tags_57 },
    { 0xd86abb77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14abb7b, SUC_lexicon_tags_17 },
    { 0xfaaabb7b, SUC_lexicon_tags_12 },
    { 0xfd2abb7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4abb80, SUC_lexicon_tags_1 },
    { 0x64abb81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2abb88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448abb8c, SUC_lexicon_tags_9 },
    { 0x100abb8d, SUC_lexicon_tags_23 },
    { 0x4b4abb8e, SUC_lexicon_tags_0 },
    { 0x49cabb8f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x878abb92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3b8abb94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cabb99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd22abb9b, SUC_lexicon_tags_17 },
    { 0x4eabb9c, SUC_lexicon_tags_7 },
    { 0xe96abb9c, SUC_lexicon_tags_138 },
    { 0x7b2abb9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x920abba1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8abba7, SUC_lexicon_tags_12 },
    { 0x89cabba8, SUC_lexicon_tags_17 },
    { 0x936abba9, SUC_lexicon_tags_28 },
    { 0xde4abbaa, SUC_lexicon_tags_1 },
    { 0xee4abbaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4eabbb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98abbb3, SUC_lexicon_tags_9 },
    { 0xe0cabbb3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9e4abbb6, SUC_lexicon_tags_15 },
    { 0x7c4abbb7, SUC_lexicon_tags_1 },
    { 0x22eabbb6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00abbbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x928abbbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcabbc0, SUC_lexicon_tags_38 },
    { 0x268abbc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x432abbc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04abbca, SUC_lexicon_tags_25 },
    { 0xc76abbcb, SUC_lexicon_tags_15 },
    { 0xe0abbca, SUC_lexicon_tags_26 },
    { 0xbcaabbcd, SUC_lexicon_tags_25 },
    { 0x2f6abbce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cabbd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6feabbd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf16abbd9, SUC_lexicon_tags_15 },
    { 0xdccabbda, SUC_lexicon_tags_27 },
    { 0x3d4abbdb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aabbde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeabbe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cabbe4, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66abbeb, SUC_lexicon_tags_38 },
    { 0xef4abbec, SUC_lexicon_tags_25 },
    { 0x992abbed, SUC_lexicon_tags_60 },
    { 0xd12abbed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364abbf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cabbf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1eabbfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf36abbfc, SUC_lexicon_tags_63 },
    { 0x4ccabbfd, SUC_lexicon_tags_39 },
    { 0x13cabbfc, SUC_lexicon_tags_43 },
    { 0xc0eabbff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116abc06, SUC_lexicon_tags_70 },
    { 0x3ccabc07, SUC_lexicon_tags_27 },
    { 0x27eabc08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x764abc0a, SUC_lexicon_tags_23 },
    { 0x3eabc0b, SUC_lexicon_tags_15 },
    { 0x864abc0c, SUC_lexicon_tags_12 },
    { 0xdacabc0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0abc11, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646abc15, SUC_lexicon_tags_8 },
    { 0xe2eabc15, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4abc21, SUC_lexicon_tags_23 },
    { 0xe46abc22, SUC_lexicon_tags_38 },
    { 0x744abc23, SUC_lexicon_tags_47 },
    { 0x446abc24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cabc28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaabc2b, SUC_lexicon_tags_7 },
    { 0x21aabc2c, SUC_lexicon_tags_15 },
    { 0x5ecabc2d, SUC_lexicon_tags_25 },
    { 0x4eaabc2c, SUC_lexicon_tags_17 },
    { 0xa90abc2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866abc38, SUC_lexicon_tags_2 },
    { 0x790abc38, SUC_lexicon_tags_82 },
    { 0xa10abc38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf96abc3c, SUC_lexicon_tags_31 },
    { 0x358abc3d, SUC_lexicon_tags_107 },
    { 0xbfeabc3e, SUC_lexicon_tags_10 },
    { 0x374abc3f, SUC_lexicon_tags_12 },
    { 0xbf4abc3f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x4baabc42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb7eabc44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82abc4c, SUC_lexicon_tags_1 },
    { 0x570abc4d, SUC_lexicon_tags_17 },
    { 0x64aabc4e, SUC_lexicon_tags_28 },
    { 0xdccabc4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x13aabc51, SUC_lexicon_tags_55 },
    { 0x262abc52, SUC_lexicon_tags_2 },
    { 0xd2abc53, SUC_lexicon_tags_55 },
    { 0xf88abc53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aabc57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x902abc5a, SUC_lexicon_tags_47 },
    { 0xbfeabc5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eeabc5e, SUC_lexicon_tags_21 },
    { 0x20aabc5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcabc62, SUC_lexicon_tags_198 },
    { 0x260abc63, SUC_lexicon_tags_46 },
    { 0xd80abc64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccabc6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92aabc72, SUC_lexicon_tags_9 },
    { 0xab8abc73, SUC_lexicon_tags_37 },
    { 0x68cabc72, SUC_lexicon_tags_14 },
    { 0xdb6abc75, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490abc80, SUC_lexicon_tags_19 },
    { 0x872abc81, SUC_lexicon_tags_10 },
    { 0x76eabc80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd82abc84, SUC_lexicon_tags_21 },
    { 0x102abc84, SUC_lexicon_tags_1 },
    { 0x66cabc85, SUC_lexicon_tags_37 },
    { 0xf48abc87, SUC_lexicon_tags_33 },
    { 0x7f8abc87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x978abc8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x174abc8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0abc92, SUC_lexicon_tags_1 },
    { 0x980abc92, SUC_lexicon_tags_9 },
    { 0xb4abc92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6abc98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38abc9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12abca0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88abca4, SUC_lexicon_tags_43 },
    { 0x4fcabca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8abcab, SUC_lexicon_tags_261 },
    { 0xb6aabcac, SUC_lexicon_tags_1 },
    { 0x72eabcac, SUC_lexicon_tags_7 },
    { 0xfa6abcae, SUC_lexicon_tags_1 },
    { 0xe04abcaf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600abcb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecabcb6, SUC_lexicon_tags_14 },
    { 0x712abcb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92abcba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x45aabcbc, SUC_lexicon_tags_30 },
    { 0x516abcbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88abcc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x46aabcc7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x542abcc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402abcd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcabcd9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccabce2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0abce6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84abce9, SUC_lexicon_tags_11 },
    { 0x874abce9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766abcf3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61eabcf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd88abcf7, SUC_lexicon_tags_26 },
    { 0xf32abcf8, SUC_lexicon_tags_11 },
    { 0x976abcf9, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xe56abcfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458abcff, SUC_lexicon_tags_2 },
    { 0x970abd00, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580abd06, SUC_lexicon_tags_2 },
    { 0x3f0abd07, SUC_lexicon_tags_61 },
    { 0x4e6abd06, SUC_lexicon_tags_9 },
    { 0x480abd09, SUC_lexicon_tags_39 },
    { 0x6baabd09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cabd0f, SUC_lexicon_tags_37 },
    { 0x43cabd10, SUC_lexicon_tags_2 },
    { 0xb8aabd0f, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0x690abd14, SUC_lexicon_tags_63 },
    { 0xcf2abd15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7beabd1b, SUC_lexicon_tags_7 },
    { 0xd06abd1c, SUC_lexicon_tags_2 },
    { 0xb50abd1d, SUC_lexicon_tags_46 },
    { 0x2f8abd1e, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xe02abd21, SUC_lexicon_tags_27 },
    { 0x4ecabd22, SUC_lexicon_tags_2 },
    { 0xf6aabd22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aabd29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cabd2c, SUC_lexicon_tags_17 },
    { 0x29cabd2d, SUC_lexicon_tags_12 },
    { 0x288abd2d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4abd36, SUC_lexicon_tags_0 },
    { 0x1d0abd36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe26abd39, SUC_lexicon_tags_254 },
    { 0x55aabd39, SUC_lexicon_tags_161 },
    { 0x4aabd3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaabd41, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x654abd43, SUC_lexicon_tags_19 },
    { 0x234abd44, SUC_lexicon_tags_37 },
    { 0xc84abd45, SUC_lexicon_tags_15 },
    { 0x5eabd45, SUC_lexicon_tags_11 },
    { 0x5aeabd45, SUC_lexicon_tags_14 },
    { 0x7c2abd48, SUC_lexicon_tags_11 },
    { 0x224abd49, SUC_lexicon_tags_19 },
    { 0xfdcabd49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d2abd4c, SUC_lexicon_tags_37 },
    { 0xadcabd4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe30abd4f, SUC_lexicon_tags_32 },
    { 0x738abd4f, SUC_lexicon_tags_24 },
    { 0x184abd50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49eabd53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x302abd56, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28abd59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aeabd5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4abd61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22abd67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73aabd6c, SUC_lexicon_tags_33 },
    { 0x744abd6d, SUC_lexicon_tags_15 },
    { 0x1baabd6e, SUC_lexicon_tags_33 },
    { 0x14abd6d, SUC_lexicon_tags_17 },
    { 0x67eabd6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb26abd72, SUC_lexicon_tags_27 },
    { 0x550abd72, SUC_lexicon_tags_15 },
    { 0xd98abd74, SUC_lexicon_tags_37 },
    { 0x7c8abd74, SUC_lexicon_tags_14 },
    { 0x4ecabd76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cabd7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45eabd86, SUC_lexicon_tags_11 },
    { 0xaf2abd87, SUC_lexicon_tags_28 },
    { 0x136abd86, SUC_lexicon_tags_50 },
    { 0xb30abd89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652abd94, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x83eabd96, SUC_lexicon_tags_11 },
    { 0x962abd97, SUC_lexicon_tags_77 },
    { 0xb76abd98, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106abd9c, SUC_lexicon_tags_40 },
    { 0x5b6abd9d, SUC_lexicon_tags_150 },
    { 0xb14abd9e, SUC_lexicon_tags_16 },
    { 0x694abd9f, SUC_lexicon_tags_3 },
    { 0x5e8abda0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x344abda2, SUC_lexicon_tags_6 },
    { 0x384abda2, SUC_lexicon_tags_0 },
    { 0x726abda4, SUC_lexicon_tags_45 },
    { 0x354abda5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x128abda8, SUC_lexicon_tags_21 },
    { 0xe8eabda9, SUC_lexicon_tags_4 },
    { 0xe7aabda9, SUC_lexicon_tags_87 },
    { 0x12eabdab, SUC_lexicon_tags_13 },
    { 0x16abdac, SUC_lexicon_tags_0 },
    { 0x1fcabdad, SUC_lexicon_tags_2 },
    { 0xc22abdae, SUC_lexicon_tags_25 },
    { 0x740abdaa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeabdbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac0abdc0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2abdc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc2abdc8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeecabdca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340abdce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82abdda, SUC_lexicon_tags_26 },
    { 0x6c6abddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x762abddd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138abde4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5aabde9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x942abdec, SUC_lexicon_tags_2 },
    { 0x430abdec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdccabdef, SUC_lexicon_tags_0 },
    { 0x36abdf0, SUC_lexicon_tags_23 },
    { 0xdcaabdf1, SUC_lexicon_tags_1 },
    { 0x4a8abdf2, SUC_lexicon_tags_11 },
    { 0xd08abdf1, SUC_lexicon_tags_11 },
    { 0xde4abdf1, SUC_lexicon_tags_38 },
    { 0x96aabdf5, SUC_lexicon_tags_87 },
    { 0x88abdf6, SUC_lexicon_tags_33 },
    { 0x72abdef, SUC_lexicon_tags_2 },
    { 0xd70abdf8, SUC_lexicon_tags_26 },
    { 0x82aabdf9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cabdfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x152abe00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaabe05, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd6aabe07, SUC_lexicon_tags_37 },
    { 0xa72abe08, SUC_lexicon_tags_0 },
    { 0xbe4abe08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x984abe0b, SUC_lexicon_tags_32 },
    { 0x76abe0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4faabe0e, SUC_lexicon_tags_37 },
    { 0x85aabe0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x182abe12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceabe1d, SUC_lexicon_tags_7 },
    { 0x280abe1e, SUC_lexicon_tags_33 },
    { 0x824abe1e, SUC_lexicon_tags_35 },
    { 0x688abe20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8abe23, SUC_lexicon_tags_11 },
    { 0x4eabe24, SUC_lexicon_tags_37 },
    { 0xecabe23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58abe28, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x788abe2a, SUC_lexicon_tags_50 },
    { 0xd12abe2a, SUC_lexicon_tags_27 },
    { 0x218abe2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8daabe2f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2abe32, SUC_lexicon_tags_12 },
    { 0xe8eabe33, SUC_lexicon_tags_25 },
    { 0x6e2abe32, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x534abe36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696abe3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c2abe3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1dcabe41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2abe46, SUC_lexicon_tags_6 },
    { 0xe60abe46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72eabe4b, SUC_lexicon_tags_2 },
    { 0x228abe4c, SUC_lexicon_tags_12 },
    { 0xcccabe4d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e6abe4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcabe52, SUC_lexicon_tags_31 },
    { 0x748abe53, SUC_lexicon_tags_63 },
    { 0xa2cabe54, SUC_lexicon_tags_43 },
    { 0x90eabe54, SUC_lexicon_tags_62 },
    { 0x2e8abe52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x348abe59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98aabe5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x644abe5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c6abe60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6abe63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2abe66, SUC_lexicon_tags_31 },
    { 0xcd2abe67, SUC_lexicon_tags_11 },
    { 0x6a2abe68, SUC_lexicon_tags_25 },
    { 0x5b0abe69, SUC_lexicon_tags_49 },
    { 0x802abe67, SUC_lexicon_tags_13 },
    { 0x57aabe67, SUC_lexicon_tags_1 },
    { 0x522abe6c, SUC_lexicon_tags_32 },
    { 0x846abe69, SUC_lexicon_tags_2 },
    { 0xef0abe6d, SUC_lexicon_tags_47 },
    { 0x458abe6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414abe73, SUC_lexicon_tags_26 },
    { 0x3e4abe74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cabe7a, SUC_lexicon_tags_1 },
    { 0x74eabe7b, SUC_lexicon_tags_49 },
    { 0x37aabe7b, SUC_lexicon_tags_741 },
    { 0x7e4abe7d, SUC_lexicon_tags_362 },
    { 0x5beabe7d, SUC_lexicon_tags_28 },
    { 0xb6eabe7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cabe84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6abe86, SUC_lexicon_tags_19 },
    { 0x214abe86, SUC_lexicon_tags_46 },
    { 0x23eabe88, SUC_lexicon_tags_6 },
    { 0x4d2abe86, SUC_lexicon_tags_11 },
    { 0x562abe8a, SUC_lexicon_tags_13 },
    { 0x36cabe8b, SUC_lexicon_tags_28 },
    { 0xbd0abe86, SUC_lexicon_tags_2 },
    { 0xb38abe8d, SUC_lexicon_tags_47 },
    { 0x756abe8d, SUC_lexicon_tags_15 },
    { 0x74eabe8f, SUC_lexicon_tags_37 },
    { 0x1caabe8e, SUC_lexicon_tags_37 },
    { 0x6e0abe8f, SUC_lexicon_tags_30 },
    { 0x48aabe92, SUC_lexicon_tags_1 },
    { 0xdb8abe93, SUC_lexicon_tags_11 },
    { 0x97eabe94, SUC_lexicon_tags_63 },
    { 0x55aabe95, SUC_lexicon_tags_2 },
    { 0xc96abe94, SUC_lexicon_tags_23 },
    { 0x990abe97, SUC_lexicon_tags_33 },
    { 0xe64abe98, SUC_lexicon_tags_121 },
    { 0x3e2abe97, SUC_lexicon_tags_32 },
    { 0x3daabe92, SUC_lexicon_tags_19 },
    { 0xa38abe99, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96abea1, SUC_lexicon_tags_17 },
    { 0xcb0abea2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eabea8, SUC_lexicon_tags_31 },
    { 0xfeeabea9, SUC_lexicon_tags_14 },
    { 0xf92abea9, SUC_lexicon_tags_25 },
    { 0x8ccabeab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2abeae, SUC_lexicon_tags_38 },
    { 0x8c0abeae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214abeb4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cabebf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2abec4, SUC_lexicon_tags_28 },
    { 0x32aabec4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x450abec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67aabecc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0cabece, SUC_lexicon_tags_4 },
    { 0xe9aabecf, SUC_lexicon_tags_14 },
    { 0xb84abecf, SUC_lexicon_tags_37 },
    { 0xe4aabece, SUC_lexicon_tags_46 },
    { 0x7eaabed1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26abed8, SUC_lexicon_tags_64 },
    { 0x29eabed8, SUC_lexicon_tags_56 },
    { 0x4f0abeda, SUC_lexicon_tags_12 },
    { 0x3eaabeda, SUC_lexicon_tags_63 },
    { 0x82cabedb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6caabede, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe6abee0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0abee3, SUC_lexicon_tags_47 },
    { 0xb10abee4, SUC_lexicon_tags_57 },
    { 0xa9eabee5, SUC_lexicon_tags_11 },
    { 0xbdaabee3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd1cabee8, SUC_lexicon_tags_37 },
    { 0x524abee9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584abeef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd56abef1, SUC_lexicon_tags_39 },
    { 0xc7cabef2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faabef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x476abef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a8abefb, SUC_lexicon_tags_17 },
    { 0x502abefc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9b6abefe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0abf03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f2abf05, SUC_lexicon_tags_43 },
    { 0xbd2abf06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6abf09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36abf0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90abf12, SUC_lexicon_tags_233 },
    { 0x6d2abf13, SUC_lexicon_tags_63 },
    { 0x482abf12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xefaabf17, SUC_lexicon_tags_11 },
    { 0xc28abf18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136abf1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aabf22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40abf25, SUC_lexicon_tags_37 },
    { 0xe50abf25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaabf2d, SUC_lexicon_tags_19 },
    { 0x316abf2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10abf32, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbe6abf34, SUC_lexicon_tags_1 },
    { 0x202abf35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58eabf38, SUC_lexicon_tags_14 },
    { 0x428abf39, SUC_lexicon_tags_15 },
    { 0x2d2abf3a, SUC_lexicon_tags_23 },
    { 0xdd8abf3b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45eabf43, SUC_lexicon_tags_14 },
    { 0x3f0abf43, SUC_lexicon_tags_25 },
    { 0x536abf44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2abf48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaabf56, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe2aabf58, SUC_lexicon_tags_2 },
    { 0xd8abf59, SUC_lexicon_tags_13 },
    { 0x346abf5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72cabf5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcabf60, SUC_lexicon_tags_37 },
    { 0xbdeabf61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x426abf63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8d4abf65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6abf68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaabf6b, SUC_lexicon_tags_7 },
    { 0xc6aabf6c, SUC_lexicon_tags_8 },
    { 0x8e6abf6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300abf71, SUC_lexicon_tags_60 },
    { 0xeb4abf72, SUC_lexicon_tags_17 },
    { 0xbf4abf73, SUC_lexicon_tags_25 },
    { 0xc40abf71, SUC_lexicon_tags_33 },
    { 0x5f6abf75, SUC_lexicon_tags_45 },
    { 0xf4eabf71, SUC_lexicon_tags_15 },
    { 0xeecabf77, SUC_lexicon_tags_33 },
    { 0xa02abf78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acabf7b, SUC_lexicon_tags_2 },
    { 0x4dcabf7c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xad4abf7e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2abf81, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xfbeabf83, SUC_lexicon_tags_11 },
    { 0x37eabf84, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcabf88, SUC_lexicon_tags_44 },
    { 0xa7eabf89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x35eabf8b, SUC_lexicon_tags_30 },
    { 0x762abf8c, SUC_lexicon_tags_23 },
    { 0x70abf8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfeabf90, SUC_lexicon_tags_63 },
    { 0x93eabf90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8abf94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x65aabf96, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x462abf98, SUC_lexicon_tags_37 },
    { 0x660abf98, SUC_lexicon_tags_17 },
    { 0x62eabf9a, SUC_lexicon_tags_17 },
    { 0x2ccabf9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9baabfa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb44abfa2, SUC_lexicon_tags_0 },
    { 0xeb2abfa3, SUC_lexicon_tags_37 },
    { 0xa28abfa4, SUC_lexicon_tags_87 },
    { 0x12cabfa3, SUC_lexicon_tags_27 },
    { 0x20eabfa6, SUC_lexicon_tags_63 },
    { 0x20abfa3, SUC_lexicon_tags_1 },
    { 0x55cabfa8, SUC_lexicon_tags_15 },
    { 0x336abfa9, SUC_lexicon_tags_12 },
    { 0x8b8abfaa, SUC_lexicon_tags_17 },
    { 0x5c8abfa3, SUC_lexicon_tags_1 },
    { 0x362abfac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcabfaf, SUC_lexicon_tags_25 },
    { 0xd36abfaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eabfb4, SUC_lexicon_tags_15 },
    { 0xcd8abfb4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62eabfb9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2abfbc, SUC_lexicon_tags_27 },
    { 0x32cabfbd, SUC_lexicon_tags_29 },
    { 0x18cabfbd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x326abfc0, SUC_lexicon_tags_14 },
    { 0xc30abfc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b8abfc3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26eabfc8, SUC_lexicon_tags_37 },
    { 0xdbcabfc9, SUC_lexicon_tags_29 },
    { 0xccabfc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eabfd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858abfd9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x16aabfdb, SUC_lexicon_tags_43 },
    { 0x8fcabfdc, SUC_lexicon_tags_17 },
    { 0x696abfdd, SUC_lexicon_tags_19 },
    { 0xd20abfdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x896abfe0, SUC_lexicon_tags_0 },
    { 0xe26abfe0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74abfeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4baabfed, SUC_lexicon_tags_45 },
    { 0xfa8abfed, SUC_lexicon_tags_25 },
    { 0xed0abfee, SUC_lexicon_tags_12 },
    { 0x892abff0, SUC_lexicon_tags_38 },
    { 0x780abff0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3eeabff3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee0abff5, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2abff8, SUC_lexicon_tags_2 },
    { 0x632abff9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x660abffb, SUC_lexicon_tags_44 },
    { 0x458abffc, SUC_lexicon_tags_82 },
    { 0x932abffc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32ac002, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6ac005, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f6ac007, SUC_lexicon_tags_17 },
    { 0xd4cac008, SUC_lexicon_tags_2 },
    { 0xd3cac009, SUC_lexicon_tags_25 },
    { 0xa5eac00a, SUC_lexicon_tags_26 },
    { 0x91cac009, SUC_lexicon_tags_19 },
    { 0xa46ac009, SUC_lexicon_tags_11 },
    { 0x418ac00d, SUC_lexicon_tags_45 },
    { 0x358ac00d, SUC_lexicon_tags_25 },
    { 0x6eaac00f, SUC_lexicon_tags_1 },
    { 0xa94ac00f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28eac014, SUC_lexicon_tags_2 },
    { 0x30ac015, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38cac019, SUC_lexicon_tags_11 },
    { 0x72eac01a, SUC_lexicon_tags_37 },
    { 0xfbeac01a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x16cac01d, SUC_lexicon_tags_19 },
    { 0x77aac01e, SUC_lexicon_tags_1 },
    { 0x61aac01e, SUC_lexicon_tags_25 },
    { 0x408ac01e, SUC_lexicon_tags_47 },
    { 0xfa4ac021, SUC_lexicon_tags_25 },
    { 0x1c6ac01f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8ac025, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x978ac027, SUC_lexicon_tags_25 },
    { 0x104ac028, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x536ac02a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0cac02c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14ac02e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x62ac030, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8f4ac032, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58eac037, SUC_lexicon_tags_0 },
    { 0xaeaac037, SUC_lexicon_tags_38 },
    { 0x596ac039, SUC_lexicon_tags_30 },
    { 0x91cac03a, SUC_lexicon_tags_21 },
    { 0xf9eac03b, SUC_lexicon_tags_2 },
    { 0xc62ac03b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf30ac03e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x240ac040, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6ac046, SUC_lexicon_tags_37 },
    { 0xbbcac046, SUC_lexicon_tags_64 },
    { 0x34aac047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faac04c, SUC_lexicon_tags_37 },
    { 0x9eac04c, SUC_lexicon_tags_25 },
    { 0x1ccac04e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x696ac051, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d2ac053, SUC_lexicon_tags_1 },
    { 0x630ac054, SUC_lexicon_tags_7 },
    { 0x288ac053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ac058, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x580ac05a, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xd3aac05c, SUC_lexicon_tags_11 },
    { 0xecac05c, SUC_lexicon_tags_17 },
    { 0x3baac05e, SUC_lexicon_tags_29 },
    { 0x84cac05f, SUC_lexicon_tags_28 },
    { 0x6a8ac05e, SUC_lexicon_tags_25 },
    { 0xf0ac05d, SUC_lexicon_tags_37 },
    { 0x1f6ac062, SUC_lexicon_tags_43 },
    { 0x6b4ac063, SUC_lexicon_tags_37 },
    { 0xf1cac064, SUC_lexicon_tags_79 },
    { 0x152ac065, SUC_lexicon_tags_7 },
    { 0x3c2ac066, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2aaac068, SUC_lexicon_tags_11 },
    { 0x3a6ac068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x372ac06c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790ac074, SUC_lexicon_tags_17 },
    { 0xe68ac074, SUC_lexicon_tags_45 },
    { 0xaa6ac074, SUC_lexicon_tags_1 },
    { 0x956ac074, SUC_lexicon_tags_12 },
    { 0xb60ac078, SUC_lexicon_tags_7 },
    { 0xc92ac079, SUC_lexicon_tags_11 },
    { 0xe6ac07a, SUC_lexicon_tags_19 },
    { 0xd72ac074, SUC_lexicon_tags_2 },
    { 0x62ac07c, SUC_lexicon_tags_9 },
    { 0xdb6ac07d, SUC_lexicon_tags_37 },
    { 0xaf8ac07e, SUC_lexicon_tags_38 },
    { 0xf1cac074, SUC_lexicon_tags_8 },
    { 0x3bcac080, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x116ac082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cac087, SUC_lexicon_tags_3 },
    { 0x48eac088, SUC_lexicon_tags_37 },
    { 0x822ac087, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x194ac08b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf5cac08d, SUC_lexicon_tags_14 },
    { 0xa04ac08e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246ac092, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecac098, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2ac09b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952ac09f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860ac0a7, SUC_lexicon_tags_15 },
    { 0x180ac0a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x862ac0ab, SUC_lexicon_tags_12 },
    { 0x3fcac0ac, SUC_lexicon_tags_14 },
    { 0x8f6ac0ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x44aac0b0, SUC_lexicon_tags_25 },
    { 0xfe4ac0b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6ac0b6, SUC_lexicon_tags_127 },
    { 0xd58ac0b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8caac0b9, SUC_lexicon_tags_37 },
    { 0x7d0ac0b9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8ac0be, SUC_lexicon_tags_25 },
    { 0x71eac0bf, SUC_lexicon_tags_15 },
    { 0xba6ac0c0, SUC_lexicon_tags_25 },
    { 0xc24ac0c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40eac0c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d2ac0c5, SUC_lexicon_tags_23 },
    { 0xe0cac0c5, SUC_lexicon_tags_35 },
    { 0xbf2ac0c7, SUC_lexicon_tags_37 },
    { 0xb96ac0c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a8ac0ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2b2ac0cc, SUC_lexicon_tags_151 },
    { 0x9aac0cc, SUC_lexicon_tags_44 },
    { 0xd60ac0cd, SUC_lexicon_tags_30 },
    { 0xa84ac0cf, SUC_lexicon_tags_0 },
    { 0xe10ac0d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb6ac0d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7eeac0d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcac0d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22ac0dc, SUC_lexicon_tags_23 },
    { 0xb06ac0dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdaac0df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb68ac0e1, SUC_lexicon_tags_63 },
    { 0x7caac0e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24ac0e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10eac0e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4ac0ef, SUC_lexicon_tags_17 },
    { 0xe26ac0ef, SUC_lexicon_tags_11 },
    { 0x5a6ac0f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1acac0f3, SUC_lexicon_tags_30 },
    { 0x4c4ac0f4, SUC_lexicon_tags_25 },
    { 0x47eac0f5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722ac0f9, SUC_lexicon_tags_47 },
    { 0x5f0ac0f9, SUC_lexicon_tags_12 },
    { 0xdd8ac0fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x156ac0fd, SUC_lexicon_tags_7 },
    { 0x658ac0fe, SUC_lexicon_tags_15 },
    { 0xe9eac0ff, SUC_lexicon_tags_30 },
    { 0x392ac100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10ac102, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x258ac105, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf2ac107, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4ac10b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc8ac10d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2ac110, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x468ac112, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37eac117, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5caac11a, SUC_lexicon_tags_0 },
    { 0x1aaac11a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6ac11e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcaac125, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1aac128, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72ac12b, SUC_lexicon_tags_23 },
    { 0x6ceac12c, SUC_lexicon_tags_29 },
    { 0xe6aac12c, SUC_lexicon_tags_17 },
    { 0xfc6ac12e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cac131, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x288ac133, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc94ac135, SUC_lexicon_tags_43 },
    { 0x9aac136, SUC_lexicon_tags_63 },
    { 0x280ac137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9eac13a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaac13e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45eac149, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2ac14c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8ac14f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a6ac151, SUC_lexicon_tags_37 },
    { 0xa3eac152, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x514ac155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x974ac157, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x434ac159, SUC_lexicon_tags_37 },
    { 0xf1aac159, SUC_lexicon_tags_33 },
    { 0x93cac15b, SUC_lexicon_tags_49 },
    { 0x65cac15c, SUC_lexicon_tags_47 },
    { 0x960ac159, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcacac15f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xad4ac161, SUC_lexicon_tags_45 },
    { 0xda4ac162, SUC_lexicon_tags_7 },
    { 0xcdcac162, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38ac166, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x96ac168, SUC_lexicon_tags_37 },
    { 0x606ac168, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8eac16b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cac171, SUC_lexicon_tags_2 },
    { 0x490ac172, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5bcac174, SUC_lexicon_tags_12 },
    { 0x44cac175, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80ac178, SUC_lexicon_tags_2 },
    { 0xe84ac178, SUC_lexicon_tags_11 },
    { 0x8c8ac179, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x81eac17d, SUC_lexicon_tags_37 },
    { 0xc9eac17e, SUC_lexicon_tags_26 },
    { 0xc38ac17f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b0ac181, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d6ac183, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316ac18b, SUC_lexicon_tags_9 },
    { 0x5caac18c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x962ac18e, SUC_lexicon_tags_56 },
    { 0x14cac18e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2bcac191, SUC_lexicon_tags_28 },
    { 0x7eac192, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48ac196, SUC_lexicon_tags_15 },
    { 0x948ac197, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3d6ac199, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4ac19f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cac1a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578ac1a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6faac1ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd42ac1ad, SUC_lexicon_tags_0 },
    { 0xe16ac1ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aac1b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x36cac1b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceac1c1, SUC_lexicon_tags_37 },
    { 0xf26ac1c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8ac1c7, SUC_lexicon_tags_2 },
    { 0x69cac1c7, SUC_lexicon_tags_39 },
    { 0x72eac1c7, SUC_lexicon_tags_60 },
    { 0x99eac1ca, SUC_lexicon_tags_17 },
    { 0xe6cac1cb, SUC_lexicon_tags_12 },
    { 0xc8aac1c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cac1d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0ac1d8, SUC_lexicon_tags_7 },
    { 0x488ac1d9, SUC_lexicon_tags_45 },
    { 0x34ac1da, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740ac1e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x216ac1e6, SUC_lexicon_tags_15 },
    { 0x4bcac1e7, SUC_lexicon_tags_17 },
    { 0x3e2ac1e7, SUC_lexicon_tags_37 },
    { 0xa64ac1e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420ac1f3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac201, SUC_lexicon_tags_55 },
    { 0xa02ac202, SUC_lexicon_tags_7 },
    { 0x220ac203, SUC_lexicon_tags_29 },
    { 0x972ac201, SUC_lexicon_tags_1 },
    { 0xfd8ac202, SUC_lexicon_tags_2 },
    { 0x220ac206, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x856ac208, SUC_lexicon_tags_37 },
    { 0xe68ac208, SUC_lexicon_tags_1 },
    { 0xc8eac20a, SUC_lexicon_tags_1 },
    { 0x826ac20a, SUC_lexicon_tags_32 },
    { 0xea8ac20a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cac210, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16ac217, SUC_lexicon_tags_17 },
    { 0x7b0ac218, SUC_lexicon_tags_12 },
    { 0x91eac218, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x742ac21b, SUC_lexicon_tags_49 },
    { 0x2ceac21c, SUC_lexicon_tags_8 },
    { 0x1e0ac21d, SUC_lexicon_tags_37 },
    { 0x960ac21b, SUC_lexicon_tags_32 },
    { 0x6d4ac21f, SUC_lexicon_tags_17 },
    { 0x14cac21f, SUC_lexicon_tags_105 },
    { 0xbccac21c, SUC_lexicon_tags_37 },
    { 0x54aac220, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb98ac224, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf96ac226, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906ac22a, SUC_lexicon_tags_7 },
    { 0x330ac22b, SUC_lexicon_tags_27 },
    { 0x4a4ac22c, SUC_lexicon_tags_29 },
    { 0x766ac22d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0ac232, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aeac237, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x276ac23a, SUC_lexicon_tags_7 },
    { 0x254ac23b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcac23e, SUC_lexicon_tags_2 },
    { 0xf7cac23e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf1cac241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf40ac243, SUC_lexicon_tags_49 },
    { 0x442ac243, SUC_lexicon_tags_12 },
    { 0xbfeac243, SUC_lexicon_tags_2 },
    { 0xcf2ac243, SUC_lexicon_tags_124 },
    { 0x976ac247, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cac24d, SUC_lexicon_tags_11 },
    { 0x296ac24e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceac260, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb26ac262, SUC_lexicon_tags_15 },
    { 0x32ac262, SUC_lexicon_tags_31 },
    { 0xb68ac262, SUC_lexicon_tags_15 },
    { 0x2eaac265, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9deac268, SUC_lexicon_tags_17 },
    { 0xde6ac268, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x33cac26c, SUC_lexicon_tags_31 },
    { 0xc12ac26c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x29eac270, SUC_lexicon_tags_9 },
    { 0x7ceac271, SUC_lexicon_tags_12 },
    { 0x60eac272, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68ac276, SUC_lexicon_tags_7 },
    { 0x60cac277, SUC_lexicon_tags_12 },
    { 0x176ac278, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56ac281, SUC_lexicon_tags_30 },
    { 0x250ac281, SUC_lexicon_tags_37 },
    { 0x5d2ac283, SUC_lexicon_tags_47 },
    { 0xd34ac283, SUC_lexicon_tags_43 },
    { 0x2d2ac285, SUC_lexicon_tags_68 },
    { 0x90cac281, SUC_lexicon_tags_27 },
    { 0x934ac287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4ac28d, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x77eac28f, SUC_lexicon_tags_19 },
    { 0xfbeac290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeacac292, SUC_lexicon_tags_37 },
    { 0xff8ac293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcac298, SUC_lexicon_tags_63 },
    { 0x590ac299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x526ac29b, SUC_lexicon_tags_2 },
    { 0x7bcac29c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9aeac29e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb24ac2a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x820ac2a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd62ac2a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6ac2a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x938ac2ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8ac2ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3deac2b0, SUC_lexicon_tags_60 },
    { 0x6f6ac2b1, SUC_lexicon_tags_31 },
    { 0xf58ac2b2, SUC_lexicon_tags_83 },
    { 0x9e4ac2b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82ac2b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaeeac2b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd6cac2bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cac2c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0ac2c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cac2c8, SUC_lexicon_tags_26 },
    { 0x3b4ac2c9, SUC_lexicon_tags_30 },
    { 0x596ac2c8, SUC_lexicon_tags_25 },
    { 0xd7cac2cb, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9aac2cf, SUC_lexicon_tags_11 },
    { 0x8eaac2cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3cac2d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x564ac2d4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x59aac2d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6ac2dc, SUC_lexicon_tags_0 },
    { 0x9f2ac2dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64ac2e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ac2e5, SUC_lexicon_tags_17 },
    { 0x862ac2e6, SUC_lexicon_tags_1 },
    { 0x954ac2e7, SUC_lexicon_tags_1 },
    { 0xab6ac2e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x988ac2eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x356ac2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42eac2f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6caac2f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5aac2f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbb6ac2f8, SUC_lexicon_tags_17 },
    { 0x64cac2f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0aac2fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8beac2fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcac302, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78ac305, SUC_lexicon_tags_16 },
    { 0x2e2ac306, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456ac312, SUC_lexicon_tags_30 },
    { 0x9d8ac313, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcac318, SUC_lexicon_tags_28 },
    { 0x354ac319, SUC_lexicon_tags_11 },
    { 0x626ac31a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc46ac31c, SUC_lexicon_tags_12 },
    { 0x3baac31d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcac323, SUC_lexicon_tags_17 },
    { 0xa96ac324, SUC_lexicon_tags_38 },
    { 0xf22ac325, SUC_lexicon_tags_46 },
    { 0x9f4ac326, SUC_lexicon_tags_17 },
    { 0x7faac327, SUC_lexicon_tags_60 },
    { 0xed6ac328, SUC_lexicon_tags_61 },
    { 0xac2ac327, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfc0ac32b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8daac32d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0ac333, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cac336, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70eac339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88ac33c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4eac341, SUC_lexicon_tags_25 },
    { 0xd8cac341, SUC_lexicon_tags_19 },
    { 0xc20ac343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8ac34a, SUC_lexicon_tags_47 },
    { 0x946ac34b, SUC_lexicon_tags_12 },
    { 0x36cac34c, SUC_lexicon_tags_12 },
    { 0x3a2ac34c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0ac352, SUC_lexicon_tags_30 },
    { 0xb5eac353, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2ac356, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x954ac359, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606ac35e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23eac362, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8ac367, SUC_lexicon_tags_7 },
    { 0x7e8ac368, SUC_lexicon_tags_25 },
    { 0x510ac368, SUC_lexicon_tags_2 },
    { 0x80eac36a, SUC_lexicon_tags_63 },
    { 0x1f0ac369, SUC_lexicon_tags_23 },
    { 0x88ac36c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c4ac36e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x45aac370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe06ac372, SUC_lexicon_tags_27 },
    { 0x552ac372, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18ac377, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8ac37b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3eac37f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552ac383, SUC_lexicon_tags_49 },
    { 0x22ac384, SUC_lexicon_tags_12 },
    { 0x794ac385, SUC_lexicon_tags_12 },
    { 0x7deac386, SUC_lexicon_tags_0 },
    { 0x440ac386, SUC_lexicon_tags_37 },
    { 0x4deac384, SUC_lexicon_tags_26 },
    { 0x11eac389, SUC_lexicon_tags_17 },
    { 0x60cac383, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x85aac38d, SUC_lexicon_tags_45 },
    { 0x42cac38e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cac39a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cac3a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160ac3a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x626ac3a6, SUC_lexicon_tags_18 },
    { 0xf44ac3a7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ceac3ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f0ac3af, SUC_lexicon_tags_65 },
    { 0x40ac3b0, SUC_lexicon_tags_37 },
    { 0x344ac3b0, SUC_lexicon_tags_12 },
    { 0x9fcac3b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4ac3b7, SUC_lexicon_tags_29 },
    { 0x3b4ac3b7, SUC_lexicon_tags_45 },
    { 0xd6ac3b9, SUC_lexicon_tags_37 },
    { 0xce8ac3ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aac3be, SUC_lexicon_tags_15 },
    { 0x278ac3be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1eaac3c1, SUC_lexicon_tags_11 },
    { 0xc1cac3c1, SUC_lexicon_tags_11 },
    { 0x1e0ac3c3, SUC_lexicon_tags_32 },
    { 0x9b4ac3c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95aac3c6, SUC_lexicon_tags_38 },
    { 0x650ac3c6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22ac3ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc34ac3cc, SUC_lexicon_tags_37 },
    { 0x232ac3cd, SUC_lexicon_tags_45 },
    { 0x152ac3cd, SUC_lexicon_tags_25 },
    { 0xbd6ac3cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44aac3d1, SUC_lexicon_tags_54 },
    { 0x254ac3d1, SUC_lexicon_tags_15 },
    { 0xef0ac3d3, SUC_lexicon_tags_12 },
    { 0xa2aac3d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2acac3d6, SUC_lexicon_tags_57 },
    { 0x42aac3d7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xef8ac3d9, SUC_lexicon_tags_47 },
    { 0x4aeac3da, SUC_lexicon_tags_17 },
    { 0x62eac3d9, SUC_lexicon_tags_38 },
    { 0xd16ac3dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ac3e1, SUC_lexicon_tags_19 },
    { 0xe02ac3e2, SUC_lexicon_tags_65 },
    { 0xc4aac3e3, SUC_lexicon_tags_2 },
    { 0x42cac3e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0aac3e8, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xcfcac3ea, SUC_lexicon_tags_42 },
    { 0x9eaac3eb, SUC_lexicon_tags_57 },
    { 0x14ac3ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab8ac3ee, SUC_lexicon_tags_28 },
    { 0x6b2ac3ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ac3f2, SUC_lexicon_tags_43 },
    { 0x546ac3f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd96ac3f5, SUC_lexicon_tags_0 },
    { 0x99cac3f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe70ac3f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1eac3fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17eac3fc, SUC_lexicon_tags_15 },
    { 0x8ccac3fd, SUC_lexicon_tags_1 },
    { 0x5e6ac3fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cac401, SUC_lexicon_tags_45 },
    { 0xe90ac401, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x604ac404, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41eac40e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda8ac410, SUC_lexicon_tags_13 },
    { 0x710ac410, SUC_lexicon_tags_25 },
    { 0x5dcac410, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996ac417, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ac41a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cac420, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcac423, SUC_lexicon_tags_45 },
    { 0xd94ac423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00ac42b, SUC_lexicon_tags_17 },
    { 0x9a4ac42b, SUC_lexicon_tags_23 },
    { 0xe4eac42d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234ac431, SUC_lexicon_tags_12 },
    { 0x216ac431, SUC_lexicon_tags_43 },
    { 0x4b6ac432, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc0ac435, SUC_lexicon_tags_8 },
    { 0x656ac436, SUC_lexicon_tags_11 },
    { 0x52ac436, SUC_lexicon_tags_47 },
    { 0x20cac438, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14ac43b, SUC_lexicon_tags_1 },
    { 0xc02ac43c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x192ac43f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4eac442, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0ac448, SUC_lexicon_tags_23 },
    { 0xc30ac449, SUC_lexicon_tags_15 },
    { 0xd98ac44a, SUC_lexicon_tags_37 },
    { 0x51cac449, SUC_lexicon_tags_43 },
    { 0x28aac44c, SUC_lexicon_tags_37 },
    { 0xd8ac44a, SUC_lexicon_tags_12 },
    { 0xfb4ac44e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37cac450, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2eac452, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0ac455, SUC_lexicon_tags_11 },
    { 0x702ac455, SUC_lexicon_tags_9 },
    { 0x7b0ac457, SUC_lexicon_tags_60 },
    { 0x88eac458, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46ac45a, SUC_lexicon_tags_7 },
    { 0x390ac45b, SUC_lexicon_tags_57 },
    { 0x334ac45b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958ac461, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x908ac464, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64ac467, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd20ac469, SUC_lexicon_tags_2 },
    { 0xa3aac469, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90ac46f, SUC_lexicon_tags_243 },
    { 0xbc2ac470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32ac473, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cac478, SUC_lexicon_tags_19 },
    { 0xea6ac479, SUC_lexicon_tags_27 },
    { 0x288ac479, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8aac47c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8eac481, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79cac483, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecac487, SUC_lexicon_tags_37 },
    { 0x7eac487, SUC_lexicon_tags_0 },
    { 0x982ac487, SUC_lexicon_tags_2 },
    { 0xd2cac487, SUC_lexicon_tags_19 },
    { 0x9aeac489, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeac495, SUC_lexicon_tags_7 },
    { 0xd8cac496, SUC_lexicon_tags_19 },
    { 0x5fcac496, SUC_lexicon_tags_7 },
    { 0xf98ac498, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83aac49a, SUC_lexicon_tags_1 },
    { 0xcd0ac49b, SUC_lexicon_tags_37 },
    { 0x4daac49a, SUC_lexicon_tags_43 },
    { 0x496ac49d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916ac4a4, SUC_lexicon_tags_23 },
    { 0x4e2ac4a4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cac4a9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x30ac4ab, SUC_lexicon_tags_17 },
    { 0x1d4ac4ac, SUC_lexicon_tags_7 },
    { 0x746ac4ab, SUC_lexicon_tags_2 },
    { 0x45aac4ae, SUC_lexicon_tags_27 },
    { 0x62cac4af, SUC_lexicon_tags_1 },
    { 0x8e4ac4af, SUC_lexicon_tags_61 },
    { 0x80aac4af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe06ac4b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbcac4b5, SUC_lexicon_tags_1 },
    { 0x154ac4b6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30ac4b9, SUC_lexicon_tags_17 },
    { 0x30ac4ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2feac4bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68ac4c5, SUC_lexicon_tags_15 },
    { 0x81aac4c6, SUC_lexicon_tags_25 },
    { 0x400ac4c6, SUC_lexicon_tags_37 },
    { 0x2daac4c8, SUC_lexicon_tags_37 },
    { 0xd20ac4c9, SUC_lexicon_tags_55 },
    { 0x5e0ac4c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd44ac4cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x230ac4ce, SUC_lexicon_tags_11 },
    { 0x2acac4ce, SUC_lexicon_tags_14 },
    { 0xe34ac4d0, SUC_lexicon_tags_1 },
    { 0x24ac4d0, SUC_lexicon_tags_77 },
    { 0x8a4ac4ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaeac4d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa2cac4d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27aac4e6, SUC_lexicon_tags_25 },
    { 0x3e2ac4e6, SUC_lexicon_tags_7 },
    { 0x93eac4e8, SUC_lexicon_tags_31 },
    { 0xe88ac4e6, SUC_lexicon_tags_38 },
    { 0xf9aac4ea, SUC_lexicon_tags_45 },
    { 0xe2aac4e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2ac4f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aaac4f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbeac4f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670ac4fc, SUC_lexicon_tags_14 },
    { 0xcecac4fd, SUC_lexicon_tags_14 },
    { 0x682ac4fe, SUC_lexicon_tags_29 },
    { 0x29cac4fc, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678ac503, SUC_lexicon_tags_0 },
    { 0x84ac504, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eac509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1baac50c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x972ac50e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd5cac510, SUC_lexicon_tags_25 },
    { 0x6bcac511, SUC_lexicon_tags_60 },
    { 0x5f0ac512, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x72eac514, SUC_lexicon_tags_27 },
    { 0x4beac515, SUC_lexicon_tags_10 },
    { 0x60eac514, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcac51a, SUC_lexicon_tags_7 },
    { 0x25aac51b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe60ac51d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x34cac51f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcfeac521, SUC_lexicon_tags_65 },
    { 0xf4ac522, SUC_lexicon_tags_61 },
    { 0x32eac523, SUC_lexicon_tags_11 },
    { 0xae4ac521, SUC_lexicon_tags_48 },
    { 0xabaac525, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3deac527, SUC_lexicon_tags_1 },
    { 0x6f2ac528, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ac52b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61eac52d, SUC_lexicon_tags_25 },
    { 0x706ac52e, SUC_lexicon_tags_66 },
    { 0x758ac52e, SUC_lexicon_tags_11 },
    { 0x3d2ac52e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668ac535, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3deac53a, SUC_lexicon_tags_10 },
    { 0x64aac53a, SUC_lexicon_tags_37 },
    { 0xc38ac53c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524ac541, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16ac545, SUC_lexicon_tags_27 },
    { 0x4e6ac546, SUC_lexicon_tags_8 },
    { 0x99cac546, SUC_lexicon_tags_37 },
    { 0x73eac545, SUC_lexicon_tags_7 },
    { 0xa8eac545, SUC_lexicon_tags_17 },
    { 0xfaaac549, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716ac54e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd38ac550, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaac556, SUC_lexicon_tags_61 },
    { 0x846ac556, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd78ac559, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x710ac55b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8ac560, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674ac564, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2ac569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92aac56b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x894ac56e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4ac576, SUC_lexicon_tags_25 },
    { 0x8ccac577, SUC_lexicon_tags_2 },
    { 0xa92ac578, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0ac581, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x850ac583, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xeecac586, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56eac588, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a4ac58a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcac58e, SUC_lexicon_tags_8 },
    { 0x112ac58f, SUC_lexicon_tags_1 },
    { 0x948ac590, SUC_lexicon_tags_25 },
    { 0xbc0ac590, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920ac597, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x436ac599, SUC_lexicon_tags_37 },
    { 0x668ac59a, SUC_lexicon_tags_17 },
    { 0xc4eac59b, SUC_lexicon_tags_17 },
    { 0x734ac59c, SUC_lexicon_tags_158 },
    { 0x560ac59d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x554ac5a0, SUC_lexicon_tags_7 },
    { 0x990ac5a1, SUC_lexicon_tags_25 },
    { 0xc64ac5a1, SUC_lexicon_tags_37 },
    { 0x8d2ac5a2, SUC_lexicon_tags_37 },
    { 0x1e8ac5a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x282ac5a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14eac5ac, SUC_lexicon_tags_41 },
    { 0x70aac5ad, SUC_lexicon_tags_27 },
    { 0x6e6ac5ad, SUC_lexicon_tags_56 },
    { 0x8e8ac5af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceac5b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c8ac5ba, SUC_lexicon_tags_0 },
    { 0x94eac5ba, SUC_lexicon_tags_7 },
    { 0xc2ac5bc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280ac5c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2ac5c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0ac5cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x46cac5d0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54ac5d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2ac5d6, SUC_lexicon_tags_11 },
    { 0x15aac5d7, SUC_lexicon_tags_11 },
    { 0x980ac5d7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac5dd, SUC_lexicon_tags_37 },
    { 0xdfcac5dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59aac5e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x734ac5e6, SUC_lexicon_tags_15 },
    { 0xfa8ac5e7, SUC_lexicon_tags_82 },
    { 0xd52ac5e8, SUC_lexicon_tags_7 },
    { 0x8b8ac5e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cac5ee, SUC_lexicon_tags_1 },
    { 0xce0ac5ee, SUC_lexicon_tags_37 },
    { 0xf4ac5ef, SUC_lexicon_tags_31 },
    { 0x992ac5f1, SUC_lexicon_tags_37 },
    { 0xac2ac5ef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faac5fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x49aac5fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0ac600, SUC_lexicon_tags_7 },
    { 0xc86ac600, SUC_lexicon_tags_38 },
    { 0x7a6ac602, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ac605, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4ac60c, SUC_lexicon_tags_17 },
    { 0xc8aac60d, SUC_lexicon_tags_49 },
    { 0x904ac60d, SUC_lexicon_tags_25 },
    { 0xd0ac60d, SUC_lexicon_tags_7 },
    { 0xc6ac610, SUC_lexicon_tags_1 },
    { 0xc8eac60e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe7cac613, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x318ac615, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aac61a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x138ac61d, SUC_lexicon_tags_65 },
    { 0x5b6ac61e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290ac623, SUC_lexicon_tags_66 },
    { 0xe94ac624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8aac626, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3c6ac628, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x852ac62a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x234ac62d, SUC_lexicon_tags_7 },
    { 0x912ac62e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0ac634, SUC_lexicon_tags_68 },
    { 0x9d4ac635, SUC_lexicon_tags_1 },
    { 0xcccac634, SUC_lexicon_tags_7 },
    { 0xd8cac637, SUC_lexicon_tags_87 },
    { 0x91aac635, SUC_lexicon_tags_46 },
    { 0xebcac635, SUC_lexicon_tags_65 },
    { 0x4beac63a, SUC_lexicon_tags_4 },
    { 0x1deac63b, SUC_lexicon_tags_77 },
    { 0x8b4ac63a, SUC_lexicon_tags_7 },
    { 0x90ac634, SUC_lexicon_tags_19 },
    { 0xcbcac63a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f0ac640, SUC_lexicon_tags_7 },
    { 0x82aac640, SUC_lexicon_tags_17 },
    { 0x42eac642, SUC_lexicon_tags_165 },
    { 0x16ac640, SUC_lexicon_tags_11 },
    { 0xaacac644, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceac649, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0ac64b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xb7eac64d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4ac650, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeac65e, SUC_lexicon_tags_11 },
    { 0xa88ac65f, SUC_lexicon_tags_2 },
    { 0xe08ac660, SUC_lexicon_tags_2 },
    { 0x722ac661, SUC_lexicon_tags_47 },
    { 0x4b8ac662, SUC_lexicon_tags_25 },
    { 0xc7eac662, SUC_lexicon_tags_1 },
    { 0x79eac661, SUC_lexicon_tags_0 },
    { 0x164ac665, SUC_lexicon_tags_10 },
    { 0xd9cac665, SUC_lexicon_tags_11 },
    { 0x510ac660, SUC_lexicon_tags_30 },
    { 0x344ac668, SUC_lexicon_tags_17 },
    { 0x286ac668, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4ac66d, SUC_lexicon_tags_151 },
    { 0x448ac66e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50aac675, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cac67d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89eac681, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x436ac683, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0ac68d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xea6ac68f, SUC_lexicon_tags_116 },
    { 0xf84ac690, SUC_lexicon_tags_60 },
    { 0xec8ac68f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c2ac693, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4ac699, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x542ac69b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x318ac69e, SUC_lexicon_tags_13 },
    { 0xc0eac69e, SUC_lexicon_tags_14 },
    { 0x82eac69e, SUC_lexicon_tags_2 },
    { 0xf22ac6a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486ac6a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cac6aa, SUC_lexicon_tags_60 },
    { 0x7a6ac6ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2a2ac6ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fcac6af, SUC_lexicon_tags_37 },
    { 0xce0ac6b0, SUC_lexicon_tags_49 },
    { 0x27cac6b1, SUC_lexicon_tags_35 },
    { 0x71cac6b1, SUC_lexicon_tags_7 },
    { 0x62aac6b0, SUC_lexicon_tags_77 },
    { 0xa64ac6b4, SUC_lexicon_tags_3 },
    { 0x9bcac6b5, SUC_lexicon_tags_7 },
    { 0x1a2ac6b6, SUC_lexicon_tags_0 },
    { 0x148ac6b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74aac6bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0ac6bf, SUC_lexicon_tags_35 },
    { 0x3caac6c0, SUC_lexicon_tags_28 },
    { 0xa54ac6bf, SUC_lexicon_tags_17 },
    { 0xcc4ac6c2, SUC_lexicon_tags_39 },
    { 0xdd0ac6c1, SUC_lexicon_tags_0 },
    { 0x5deac6c4, SUC_lexicon_tags_43 },
    { 0x41aac6c3, SUC_lexicon_tags_38 },
    { 0x6e6ac6c2, SUC_lexicon_tags_19 },
    { 0x394ac6c7, SUC_lexicon_tags_11 },
    { 0x748ac6c8, SUC_lexicon_tags_14 },
    { 0x6a0ac6c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ceac6cc, SUC_lexicon_tags_1 },
    { 0x278ac6cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cac6d3, SUC_lexicon_tags_8 },
    { 0xc0ac6d4, SUC_lexicon_tags_7 },
    { 0xe22ac6d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d0ac6d7, SUC_lexicon_tags_7 },
    { 0x18ac6d8, SUC_lexicon_tags_4 },
    { 0xe0aac6d9, SUC_lexicon_tags_25 },
    { 0x736ac6da, SUC_lexicon_tags_12 },
    { 0x33cac6db, SUC_lexicon_tags_11 },
    { 0x5bcac6dc, SUC_lexicon_tags_19 },
    { 0xbf0ac6dc, SUC_lexicon_tags_39 },
    { 0x726ac6dc, SUC_lexicon_tags_53 },
    { 0x4c8ac6da, SUC_lexicon_tags_12 },
    { 0xe1eac6d9, SUC_lexicon_tags_33 },
    { 0x76ac6dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20aac6e3, SUC_lexicon_tags_39 },
    { 0x2ac6e4, SUC_lexicon_tags_17 },
    { 0x694ac6e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc8ac6e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdb8ac6e9, SUC_lexicon_tags_15 },
    { 0x9c6ac6ea, SUC_lexicon_tags_49 },
    { 0x74eac6ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6ac6f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4ac6fd, SUC_lexicon_tags_37 },
    { 0x550ac6fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7caac701, SUC_lexicon_tags_19 },
    { 0xeeac702, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xae4ac704, SUC_lexicon_tags_31 },
    { 0xc2cac705, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x172ac708, SUC_lexicon_tags_2 },
    { 0x77aac709, SUC_lexicon_tags_23 },
    { 0xe20ac708, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x506ac70c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cac712, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf9aac714, SUC_lexicon_tags_82 },
    { 0xa9cac714, SUC_lexicon_tags_1 },
    { 0x30eac716, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890ac71c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218ac722, SUC_lexicon_tags_46 },
    { 0xf56ac723, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8deac72b, SUC_lexicon_tags_17 },
    { 0x674ac72c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60eac72f, SUC_lexicon_tags_14 },
    { 0xab0ac730, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0ac733, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862ac739, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72ac73c, SUC_lexicon_tags_11 },
    { 0xe52ac73d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eeac746, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x12aac749, SUC_lexicon_tags_15 },
    { 0xe04ac74a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf2ac74c, SUC_lexicon_tags_30 },
    { 0x23eac74c, SUC_lexicon_tags_80 },
    { 0xceaac74e, SUC_lexicon_tags_689 },
    { 0x7faac74f, SUC_lexicon_tags_10 },
    { 0x886ac74f, SUC_lexicon_tags_2 },
    { 0x4eeac74d, SUC_lexicon_tags_37 },
    { 0xaecac752, SUC_lexicon_tags_1 },
    { 0x264ac750, SUC_lexicon_tags_21 },
    { 0x38aac754, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2ac75b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2ac760, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc6ac762, SUC_lexicon_tags_2 },
    { 0xecac763, SUC_lexicon_tags_7 },
    { 0xc3aac763, SUC_lexicon_tags_37 },
    { 0x13cac762, SUC_lexicon_tags_17 },
    { 0x4a6ac764, SUC_lexicon_tags_37 },
    { 0x884ac765, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ac76a, SUC_lexicon_tags_15 },
    { 0xda6ac76b, SUC_lexicon_tags_17 },
    { 0x5eeac76a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774ac771, SUC_lexicon_tags_15 },
    { 0xf38ac772, SUC_lexicon_tags_12 },
    { 0x262ac773, SUC_lexicon_tags_37 },
    { 0xf9aac773, SUC_lexicon_tags_60 },
    { 0x1cac775, SUC_lexicon_tags_23 },
    { 0xfecac776, SUC_lexicon_tags_18 },
    { 0x146ac776, SUC_lexicon_tags_25 },
    { 0x688ac778, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eac77e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134ac789, SUC_lexicon_tags_4 },
    { 0xc84ac78a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc8ac78c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x486ac78f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84ac792, SUC_lexicon_tags_25 },
    { 0xea8ac793, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb9cac795, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9eac797, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4ac79b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48ac7a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ac7a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeeac7b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cac7b7, SUC_lexicon_tags_1 },
    { 0x12aac7b8, SUC_lexicon_tags_66 },
    { 0x864ac7b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x312ac7bb, SUC_lexicon_tags_11 },
    { 0x5c6ac7bc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1f6ac7be, SUC_lexicon_tags_0 },
    { 0x7c4ac7bf, SUC_lexicon_tags_37 },
    { 0xab0ac7c0, SUC_lexicon_tags_12 },
    { 0x336ac7c1, SUC_lexicon_tags_25 },
    { 0xbe6ac7c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4faac7c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aac7ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754ac7ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x250ac7d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cac7d3, SUC_lexicon_tags_43 },
    { 0x364ac7d4, SUC_lexicon_tags_15 },
    { 0x280ac7d4, SUC_lexicon_tags_58 },
    { 0x342ac7d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c0ac7d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaac7db, SUC_lexicon_tags_2 },
    { 0x71aac7db, SUC_lexicon_tags_13 },
    { 0x616ac7dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xadaac7df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44ac7e2, SUC_lexicon_tags_47 },
    { 0x42cac7e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aac7ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf8aac7ec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300ac7f1, SUC_lexicon_tags_23 },
    { 0x3aeac7f2, SUC_lexicon_tags_27 },
    { 0x68eac7f3, SUC_lexicon_tags_14 },
    { 0x2c8ac7f3, SUC_lexicon_tags_7 },
    { 0x402ac7f4, SUC_lexicon_tags_25 },
    { 0xe7aac7f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcac7fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12ac7ff, SUC_lexicon_tags_30 },
    { 0xcc6ac800, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xc5cac802, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216ac806, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cac80d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06ac810, SUC_lexicon_tags_7 },
    { 0x7c6ac810, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeac814, SUC_lexicon_tags_16 },
    { 0x10cac814, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafeac81a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cac81d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfacac820, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaac825, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cac828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148ac82d, SUC_lexicon_tags_47 },
    { 0x8ceac82d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812ac83b, SUC_lexicon_tags_7 },
    { 0xd6ac83b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeacac83e, SUC_lexicon_tags_2 },
    { 0xe82ac83f, SUC_lexicon_tags_7 },
    { 0x50cac840, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x810ac842, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3aac847, SUC_lexicon_tags_45 },
    { 0xb46ac848, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aac851, SUC_lexicon_tags_25 },
    { 0xf56ac852, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x298ac854, SUC_lexicon_tags_37 },
    { 0x51aac855, SUC_lexicon_tags_7 },
    { 0xa18ac856, SUC_lexicon_tags_17 },
    { 0x872ac857, SUC_lexicon_tags_75 },
    { 0x438ac858, SUC_lexicon_tags_7 },
    { 0x226ac859, SUC_lexicon_tags_7 },
    { 0x7aaac855, SUC_lexicon_tags_9 },
    { 0x22ac859, SUC_lexicon_tags_7 },
    { 0x106ac857, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6feac860, SUC_lexicon_tags_43 },
    { 0xea2ac860, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31aac866, SUC_lexicon_tags_1 },
    { 0xa18ac867, SUC_lexicon_tags_38 },
    { 0xa02ac868, SUC_lexicon_tags_7 },
    { 0x5daac869, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5cac86b, SUC_lexicon_tags_26 },
    { 0x6f0ac86c, SUC_lexicon_tags_17 },
    { 0x9ecac86c, SUC_lexicon_tags_7 },
    { 0x7aac86b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x92aac871, SUC_lexicon_tags_1 },
    { 0xe0cac872, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb34ac874, SUC_lexicon_tags_9 },
    { 0x7c4ac874, SUC_lexicon_tags_39 },
    { 0xa72ac876, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24ac87b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x500ac87e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71eac880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18ac883, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6d6ac885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86ac88f, SUC_lexicon_tags_25 },
    { 0x9deac890, SUC_lexicon_tags_7 },
    { 0x4a8ac890, SUC_lexicon_tags_7 },
    { 0x42eac892, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2ac895, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeac89b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcac89e, SUC_lexicon_tags_47 },
    { 0x162ac89f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78ac8a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x99aac8a4, SUC_lexicon_tags_37 },
    { 0x180ac8a4, SUC_lexicon_tags_14 },
    { 0xa90ac8a6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe98ac8a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748ac8ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22eac8af, SUC_lexicon_tags_9 },
    { 0x50eac8b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554ac8b5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cac8b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x576ac8ba, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8ac8bd, SUC_lexicon_tags_7 },
    { 0xdf8ac8bd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfbaac8c0, SUC_lexicon_tags_12 },
    { 0x9c0ac8c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432ac8c9, SUC_lexicon_tags_35 },
    { 0x83eac8c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2aeac8cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacaac8ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aac8d4, SUC_lexicon_tags_0 },
    { 0xe60ac8d5, SUC_lexicon_tags_9 },
    { 0x528ac8d6, SUC_lexicon_tags_25 },
    { 0x316ac8d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4ac8db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4ac8df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x770ac8e1, SUC_lexicon_tags_37 },
    { 0x1aac8e2, SUC_lexicon_tags_23 },
    { 0x758ac8e3, SUC_lexicon_tags_37 },
    { 0xa0ac8e1, SUC_lexicon_tags_29 },
    { 0xd6cac8e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5beac8e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434ac8eb, SUC_lexicon_tags_23 },
    { 0x58cac8ec, SUC_lexicon_tags_25 },
    { 0x6daac8ed, SUC_lexicon_tags_37 },
    { 0x872ac8ed, SUC_lexicon_tags_46 },
    { 0x5e0ac8ef, SUC_lexicon_tags_0 },
    { 0xcb6ac8ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8ac8f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb06ac8f8, SUC_lexicon_tags_29 },
    { 0x72eac8f9, SUC_lexicon_tags_45 },
    { 0x2caac8fa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aac8ff, SUC_lexicon_tags_17 },
    { 0x1c0ac900, SUC_lexicon_tags_11 },
    { 0x8a0ac900, SUC_lexicon_tags_25 },
    { 0xa82ac902, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x220ac904, SUC_lexicon_tags_15 },
    { 0x24ac905, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x780ac908, SUC_lexicon_tags_44 },
    { 0x7bcac909, SUC_lexicon_tags_27 },
    { 0x9f0ac90a, SUC_lexicon_tags_11 },
    { 0xc54ac90b, SUC_lexicon_tags_4 },
    { 0x7b4ac908, SUC_lexicon_tags_11 },
    { 0x34eac909, SUC_lexicon_tags_18 },
    { 0x38eac90b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4ac913, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcac91d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16ac928, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x44aac92b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xc92ac92d, SUC_lexicon_tags_14 },
    { 0x14cac92e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x656ac930, SUC_lexicon_tags_17 },
    { 0x12cac931, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x832ac933, SUC_lexicon_tags_25 },
    { 0x9cac934, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x384ac936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aac93c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40ac93f, SUC_lexicon_tags_25 },
    { 0xad2ac940, SUC_lexicon_tags_15 },
    { 0x486ac941, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0ac943, SUC_lexicon_tags_15 },
    { 0xb32ac944, SUC_lexicon_tags_25 },
    { 0x876ac945, SUC_lexicon_tags_13 },
    { 0x20cac946, SUC_lexicon_tags_33 },
    { 0xb0ac947, SUC_lexicon_tags_2 },
    { 0x2faac947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x130ac94b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4ac94e, SUC_lexicon_tags_17 },
    { 0xae0ac94f, SUC_lexicon_tags_32 },
    { 0x6e0ac94f, SUC_lexicon_tags_37 },
    { 0xb90ac94e, SUC_lexicon_tags_2 },
    { 0xf3aac94e, SUC_lexicon_tags_29 },
    { 0xd48ac953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42eac955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0ac95a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5f6ac95c, SUC_lexicon_tags_14 },
    { 0x77eac95d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848ac961, SUC_lexicon_tags_25 },
    { 0xa84ac962, SUC_lexicon_tags_11 },
    { 0x466ac963, SUC_lexicon_tags_17 },
    { 0x652ac964, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x724ac966, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x996ac968, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4ac96b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8ac973, SUC_lexicon_tags_47 },
    { 0x52eac974, SUC_lexicon_tags_47 },
    { 0x634ac975, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x516ac978, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x168ac97b, SUC_lexicon_tags_25 },
    { 0x162ac97c, SUC_lexicon_tags_17 },
    { 0x71cac97c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8eac97f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4aac981, SUC_lexicon_tags_25 },
    { 0x89cac981, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd30ac984, SUC_lexicon_tags_17 },
    { 0x8daac985, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aac988, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aac98c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6caac991, SUC_lexicon_tags_30 },
    { 0x636ac992, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ac995, SUC_lexicon_tags_17 },
    { 0x4a0ac996, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ac99a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdaac99c, SUC_lexicon_tags_45 },
    { 0x254ac99d, SUC_lexicon_tags_49 },
    { 0x26eac99e, SUC_lexicon_tags_2 },
    { 0x9b0ac99f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684ac9a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20ac9a7, SUC_lexicon_tags_7 },
    { 0x4caac9a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbeaac9aa, SUC_lexicon_tags_40 },
    { 0xa94ac9ab, SUC_lexicon_tags_1 },
    { 0x966ac9ab, SUC_lexicon_tags_7 },
    { 0x9b2ac9ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11aac9af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaac9bd, SUC_lexicon_tags_25 },
    { 0x3deac9be, SUC_lexicon_tags_30 },
    { 0x4ac9bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaac9c5, SUC_lexicon_tags_48 },
    { 0x38ac9c6, SUC_lexicon_tags_37 },
    { 0x2beac9c7, SUC_lexicon_tags_37 },
    { 0xe74ac9c8, SUC_lexicon_tags_8 },
    { 0x6e4ac9c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbeac9cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968ac9d0, SUC_lexicon_tags_37 },
    { 0x5caac9d1, SUC_lexicon_tags_8 },
    { 0x800ac9d2, SUC_lexicon_tags_15 },
    { 0xf7aac9d3, SUC_lexicon_tags_68 },
    { 0xb00ac9d4, SUC_lexicon_tags_27 },
    { 0x98aac9d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04ac9d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cac9dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800ac9e1, SUC_lexicon_tags_2 },
    { 0x210ac9e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0ac9e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x766ac9ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cac9f1, SUC_lexicon_tags_15 },
    { 0x646ac9f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b4ac9f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36ac9f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32ac9fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x92eac9ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eaca03, SUC_lexicon_tags_38 },
    { 0xbbeaca03, SUC_lexicon_tags_27 },
    { 0x2d0aca05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4caca07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2aca0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6deaca13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386aca17, SUC_lexicon_tags_9 },
    { 0x198aca17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63eaca1c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eaca21, SUC_lexicon_tags_17 },
    { 0x778aca21, SUC_lexicon_tags_64 },
    { 0x780aca21, SUC_lexicon_tags_37 },
    { 0xbf4aca24, SUC_lexicon_tags_15 },
    { 0x362aca24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4aca28, SUC_lexicon_tags_17 },
    { 0xc46aca29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4aca2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aca33, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36eaca39, SUC_lexicon_tags_30 },
    { 0x6d2aca3a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x72eaca3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aca40, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfa4aca42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390aca47, SUC_lexicon_tags_23 },
    { 0xa8aca48, SUC_lexicon_tags_79 },
    { 0xd66aca48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf76aca4b, SUC_lexicon_tags_43 },
    { 0x51eaca4c, SUC_lexicon_tags_87 },
    { 0x396aca4d, SUC_lexicon_tags_25 },
    { 0xcc8aca4e, SUC_lexicon_tags_17 },
    { 0x110aca4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85aaca51, SUC_lexicon_tags_27 },
    { 0xe4eaca52, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3c6aca54, SUC_lexicon_tags_47 },
    { 0x568aca55, SUC_lexicon_tags_87 },
    { 0xd80aca56, SUC_lexicon_tags_17 },
    { 0x9beaca57, SUC_lexicon_tags_15 },
    { 0xaacaca58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12aca60, SUC_lexicon_tags_17 },
    { 0x930aca60, SUC_lexicon_tags_2 },
    { 0x158aca60, SUC_lexicon_tags_0 },
    { 0xddeaca63, SUC_lexicon_tags_63 },
    { 0xcdaaca61, SUC_lexicon_tags_38 },
    { 0x4ceaca61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb6aca67, SUC_lexicon_tags_68 },
    { 0x6a4aca68, SUC_lexicon_tags_49 },
    { 0x706aca69, SUC_lexicon_tags_68 },
    { 0x184aca6a, SUC_lexicon_tags_23 },
    { 0xab2aca6b, SUC_lexicon_tags_15 },
    { 0xbb2aca68, SUC_lexicon_tags_235 },
    { 0x1a6aca6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b8aca6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa4aca71, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4e0aca73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52aca75, SUC_lexicon_tags_43 },
    { 0x794aca76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44aaca7c, SUC_lexicon_tags_6 },
    { 0xfcaca7d, SUC_lexicon_tags_7 },
    { 0xee6aca7c, SUC_lexicon_tags_12 },
    { 0x1a0aca7f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8aca81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82aca85, SUC_lexicon_tags_0 },
    { 0x418aca85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6aca8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4aca96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0aca9e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eacaa1, SUC_lexicon_tags_11 },
    { 0xf7eacaa2, SUC_lexicon_tags_44 },
    { 0x75aacaa2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26acaa7, SUC_lexicon_tags_25 },
    { 0xe84acaa8, SUC_lexicon_tags_2 },
    { 0xb14acaa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24acaad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92acab0, SUC_lexicon_tags_38 },
    { 0x71aacab1, SUC_lexicon_tags_7 },
    { 0x2b4acab2, SUC_lexicon_tags_37 },
    { 0x976acab3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99aacab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x536acab7, SUC_lexicon_tags_85 },
    { 0x13eacab8, SUC_lexicon_tags_17 },
    { 0x26cacab9, SUC_lexicon_tags_19 },
    { 0x298acab8, SUC_lexicon_tags_49 },
    { 0x8feacabb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83aacabf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27eacac3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70acaca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cacad5, SUC_lexicon_tags_2 },
    { 0xfa8acad5, SUC_lexicon_tags_55 },
    { 0x3a4acad7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x440acad9, SUC_lexicon_tags_43 },
    { 0x294acad9, SUC_lexicon_tags_2 },
    { 0xbd4acadb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefeacae1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230acae8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9fcacaea, SUC_lexicon_tags_37 },
    { 0x924acaeb, SUC_lexicon_tags_68 },
    { 0xb6aacaec, SUC_lexicon_tags_7 },
    { 0xdccacaed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2acaf1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56acaf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6acafc, SUC_lexicon_tags_15 },
    { 0x41aacafd, SUC_lexicon_tags_63 },
    { 0x496acafd, SUC_lexicon_tags_2 },
    { 0x3cacaff, SUC_lexicon_tags_10 },
    { 0xeb2acafe, SUC_lexicon_tags_1 },
    { 0x606acb01, SUC_lexicon_tags_37 },
    { 0x646acb02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2acb07, SUC_lexicon_tags_9 },
    { 0x120acb08, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaacb0c, SUC_lexicon_tags_25 },
    { 0xcacacb0c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8aaacb0f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x44acb12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0aacb14, SUC_lexicon_tags_17 },
    { 0xcf8acb15, SUC_lexicon_tags_26 },
    { 0x67cacb15, SUC_lexicon_tags_65 },
    { 0xc4cacb17, SUC_lexicon_tags_1 },
    { 0x83aacb18, SUC_lexicon_tags_7 },
    { 0x2beacb18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60aacb1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaacb21, SUC_lexicon_tags_17 },
    { 0x29eacb21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6acb25, SUC_lexicon_tags_66 },
    { 0x1a8acb26, SUC_lexicon_tags_37 },
    { 0x7c2acb25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80acb2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53eacb31, SUC_lexicon_tags_10 },
    { 0xa1aacb32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85aacb34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa0acb36, SUC_lexicon_tags_92 },
    { 0xa30acb37, SUC_lexicon_tags_8 },
    { 0x11cacb36, SUC_lexicon_tags_43 },
    { 0x6f0acb39, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8aacb3b, SUC_lexicon_tags_7 },
    { 0xa44acb3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b8acb3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cacb47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670acb52, SUC_lexicon_tags_15 },
    { 0xaf2acb52, SUC_lexicon_tags_10 },
    { 0xcb6acb54, SUC_lexicon_tags_63 },
    { 0x3f0acb55, SUC_lexicon_tags_47 },
    { 0x40eacb56, SUC_lexicon_tags_46 },
    { 0x98aacb57, SUC_lexicon_tags_9 },
    { 0x348acb58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaacb5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa70acb5f, SUC_lexicon_tags_63 },
    { 0xd9cacb60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14acb63, SUC_lexicon_tags_31 },
    { 0x3a4acb64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc82acb66, SUC_lexicon_tags_14 },
    { 0xd9eacb67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaaaacb69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6deacb70, SUC_lexicon_tags_64 },
    { 0x8e2acb70, SUC_lexicon_tags_87 },
    { 0x358acb72, SUC_lexicon_tags_14 },
    { 0x146acb70, SUC_lexicon_tags_26 },
    { 0x370acb71, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60acb7c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8acb80, SUC_lexicon_tags_25 },
    { 0xaa4acb80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4acb88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeacb8b, SUC_lexicon_tags_11 },
    { 0xcacacb8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40eacb8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe96acb90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172acb9a, SUC_lexicon_tags_3 },
    { 0x2c2acb9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aacb9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36acba2, SUC_lexicon_tags_17 },
    { 0x2a2acba2, SUC_lexicon_tags_23 },
    { 0x2ccacba2, SUC_lexicon_tags_43 },
    { 0x164acba3, SUC_lexicon_tags_33 },
    { 0xb80acba6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6d8acba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6eacbaa, SUC_lexicon_tags_15 },
    { 0xa32acbab, SUC_lexicon_tags_17 },
    { 0xb4eacbac, SUC_lexicon_tags_1 },
    { 0x3f2acbac, SUC_lexicon_tags_2 },
    { 0x72cacbab, SUC_lexicon_tags_33 },
    { 0x36aacbae, SUC_lexicon_tags_0 },
    { 0xb46acbb0, SUC_lexicon_tags_21 },
    { 0x350acbaf, SUC_lexicon_tags_43 },
    { 0x9aaacbab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100acbb7, SUC_lexicon_tags_25 },
    { 0x6d4acbb8, SUC_lexicon_tags_25 },
    { 0xf06acbb9, SUC_lexicon_tags_27 },
    { 0x60aacbba, SUC_lexicon_tags_1 },
    { 0x37aacbbb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaacbbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aacbc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x918acbc5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x348acbc8, SUC_lexicon_tags_589 },
    { 0x774acbc8, SUC_lexicon_tags_46 },
    { 0x33eacbca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08acbcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7feacbd0, SUC_lexicon_tags_15 },
    { 0x246acbd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2acbd4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x860acbd6, SUC_lexicon_tags_12 },
    { 0x8a6acbd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3aacbdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930acbeb, SUC_lexicon_tags_19 },
    { 0xaaeacbec, SUC_lexicon_tags_47 },
    { 0xf70acbed, SUC_lexicon_tags_30 },
    { 0xf00acbed, SUC_lexicon_tags_15 },
    { 0x456acbef, SUC_lexicon_tags_21 },
    { 0xa2acbf0, SUC_lexicon_tags_151 },
    { 0x4acbf1, SUC_lexicon_tags_17 },
    { 0x76cacbef, SUC_lexicon_tags_141 },
    { 0x2f0acbec, SUC_lexicon_tags_12 },
    { 0x4b2acbf4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaacbf8, SUC_lexicon_tags_60 },
    { 0x28eacbf9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cacbff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62acc03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c4acc05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39eacc07, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2acc0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aacc11, SUC_lexicon_tags_19 },
    { 0x576acc12, SUC_lexicon_tags_43 },
    { 0xcaeacc11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0acc16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e6acc18, SUC_lexicon_tags_0 },
    { 0xbeacc19, SUC_lexicon_tags_32 },
    { 0xf44acc1a, SUC_lexicon_tags_2 },
    { 0x2aacc1a, SUC_lexicon_tags_17 },
    { 0x58aacc1c, SUC_lexicon_tags_12 },
    { 0x154acc1d, SUC_lexicon_tags_37 },
    { 0xf60acc1e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x91cacc20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94acc26, SUC_lexicon_tags_30 },
    { 0x68eacc26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceacc2b, SUC_lexicon_tags_65 },
    { 0x7caacc2c, SUC_lexicon_tags_7 },
    { 0x900acc2c, SUC_lexicon_tags_6 },
    { 0xbaacc2d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcacc34, SUC_lexicon_tags_136 },
    { 0xab8acc35, SUC_lexicon_tags_2 },
    { 0x4f0acc36, SUC_lexicon_tags_37 },
    { 0xca8acc37, SUC_lexicon_tags_7 },
    { 0x996acc38, SUC_lexicon_tags_1 },
    { 0xbd8acc37, SUC_lexicon_tags_39 },
    { 0xb2acc3a, SUC_lexicon_tags_11 },
    { 0x5eeacc3a, SUC_lexicon_tags_32 },
    { 0x284acc3b, SUC_lexicon_tags_15 },
    { 0xe8aacc35, SUC_lexicon_tags_37 },
    { 0x872acc3e, SUC_lexicon_tags_25 },
    { 0xa9eacc37, SUC_lexicon_tags_45 },
    { 0x784acc3c, SUC_lexicon_tags_37 },
    { 0xfdaacc3b, SUC_lexicon_tags_12 },
    { 0x58eacc3a, SUC_lexicon_tags_82 },
    { 0x5aaacc43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaceacc47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36acc4b, SUC_lexicon_tags_11 },
    { 0xbcaacc4b, SUC_lexicon_tags_43 },
    { 0x28aacc4d, SUC_lexicon_tags_7 },
    { 0x8b6acc4b, SUC_lexicon_tags_30 },
    { 0x382acc4f, SUC_lexicon_tags_7 },
    { 0xfa6acc4f, SUC_lexicon_tags_15 },
    { 0xf3cacc51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268acc58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aacc5c, SUC_lexicon_tags_970 },
    { 0, NULL },
    { 0, NULL },
    { 0x28eacc5f, SUC_lexicon_tags_11 },
    { 0x480acc60, SUC_lexicon_tags_0 },
    { 0x22acc61, SUC_lexicon_tags_25 },
    { 0x1d8acc61, SUC_lexicon_tags_47 },
    { 0x33cacc63, SUC_lexicon_tags_25 },
    { 0x744acc64, SUC_lexicon_tags_44 },
    { 0xb34acc60, SUC_lexicon_tags_15 },
    { 0x5ceacc66, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x342acc69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d4acc6b, SUC_lexicon_tags_11 },
    { 0x8eeacc6c, SUC_lexicon_tags_15 },
    { 0xea0acc6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x872acc70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x684acc73, SUC_lexicon_tags_69 },
    { 0x5e4acc73, SUC_lexicon_tags_31 },
    { 0x3daacc75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4acc78, SUC_lexicon_tags_2 },
    { 0x864acc78, SUC_lexicon_tags_25 },
    { 0xe48acc7a, SUC_lexicon_tags_39 },
    { 0x494acc78, SUC_lexicon_tags_38 },
    { 0x2e4acc7a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x488acc7f, SUC_lexicon_tags_5 },
    { 0xaaacc7f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586acc84, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x97aacc86, SUC_lexicon_tags_15 },
    { 0x4e2acc87, SUC_lexicon_tags_43 },
    { 0xccacc88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4beacc8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74acc93, SUC_lexicon_tags_25 },
    { 0x774acc93, SUC_lexicon_tags_37 },
    { 0xfc2acc95, SUC_lexicon_tags_27 },
    { 0x160acc93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80eacc9c, SUC_lexicon_tags_47 },
    { 0x5bcacc9d, SUC_lexicon_tags_12 },
    { 0xa42acc9d, SUC_lexicon_tags_77 },
    { 0x55aacc9f, SUC_lexicon_tags_15 },
    { 0xf2cacca0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77eacca4, SUC_lexicon_tags_47 },
    { 0xeaaacca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cacca8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9d8accaa, SUC_lexicon_tags_12 },
    { 0x61aaccab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1caccae, SUC_lexicon_tags_12 },
    { 0x844accaf, SUC_lexicon_tags_45 },
    { 0x6c8accaf, SUC_lexicon_tags_8 },
    { 0x32eaccb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32caccb4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63eaccb9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaaccbd, SUC_lexicon_tags_13 },
    { 0x58eaccbe, SUC_lexicon_tags_0 },
    { 0xf66accbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2aaccc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0aaccc3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x77eaccc6, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x90caccc8, SUC_lexicon_tags_29 },
    { 0x7deaccc9, SUC_lexicon_tags_65 },
    { 0x8eaccca, SUC_lexicon_tags_25 },
    { 0x70aacccb, SUC_lexicon_tags_15 },
    { 0xf6acccc, SUC_lexicon_tags_11 },
    { 0x97aacccd, SUC_lexicon_tags_2 },
    { 0xcacaccce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaccd1, SUC_lexicon_tags_63 },
    { 0xd3aaccd2, SUC_lexicon_tags_37 },
    { 0x2a2accd3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700accde, SUC_lexicon_tags_0 },
    { 0x730accde, SUC_lexicon_tags_56 },
    { 0x8ceacce0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcacce3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2acce6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe8acce8, SUC_lexicon_tags_107 },
    { 0x888acce9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x296accec, SUC_lexicon_tags_2 },
    { 0xa76acced, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x81eaccf0, SUC_lexicon_tags_12 },
    { 0xf2accf1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49aaccf3, SUC_lexicon_tags_27 },
    { 0x64caccf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6accf9, SUC_lexicon_tags_17 },
    { 0xb48accfa, SUC_lexicon_tags_12 },
    { 0x72eaccfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1deaccfd, SUC_lexicon_tags_28 },
    { 0xfb8accfe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56acd03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90acd05, SUC_lexicon_tags_14 },
    { 0xf32acd06, SUC_lexicon_tags_11 },
    { 0xfa8acd06, SUC_lexicon_tags_33 },
    { 0xb3cacd08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164acd0d, SUC_lexicon_tags_15 },
    { 0xdfcacd0e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0aacd15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66acd18, SUC_lexicon_tags_63 },
    { 0x65cacd19, SUC_lexicon_tags_57 },
    { 0xbe2acd19, SUC_lexicon_tags_12 },
    { 0x350acd19, SUC_lexicon_tags_179 },
    { 0x4f2acd1c, SUC_lexicon_tags_2 },
    { 0xfd8acd19, SUC_lexicon_tags_37 },
    { 0xc38acd18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8acd22, SUC_lexicon_tags_27 },
    { 0x1d6acd23, SUC_lexicon_tags_61 },
    { 0xcfeacd23, SUC_lexicon_tags_9 },
    { 0x4f2acd23, SUC_lexicon_tags_30 },
    { 0x742acd26, SUC_lexicon_tags_7 },
    { 0xc98acd27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d4acd29, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2acd2c, SUC_lexicon_tags_12 },
    { 0x114acd2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x412acd2f, SUC_lexicon_tags_30 },
    { 0x834acd30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6acd34, SUC_lexicon_tags_5 },
    { 0x75eacd35, SUC_lexicon_tags_12 },
    { 0x94aacd36, SUC_lexicon_tags_17 },
    { 0xa0aacd36, SUC_lexicon_tags_38 },
    { 0x84acd36, SUC_lexicon_tags_17 },
    { 0x246acd39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6acd3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14acd44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x814acd47, SUC_lexicon_tags_17 },
    { 0x7f6acd47, SUC_lexicon_tags_11 },
    { 0xa0acd49, SUC_lexicon_tags_25 },
    { 0x266acd4a, SUC_lexicon_tags_7 },
    { 0x3bcacd4b, SUC_lexicon_tags_11 },
    { 0xeb2acd4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa7aacd4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2acd51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2acd54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcaacd56, SUC_lexicon_tags_55 },
    { 0xb3cacd57, SUC_lexicon_tags_9 },
    { 0x480acd58, SUC_lexicon_tags_23 },
    { 0x202acd58, SUC_lexicon_tags_61 },
    { 0xe7cacd58, SUC_lexicon_tags_37 },
    { 0x77eacd5b, SUC_lexicon_tags_14 },
    { 0x1f4acd5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf04acd5e, SUC_lexicon_tags_13 },
    { 0x366acd5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4c8acd61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaacd64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaacd69, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06acd6f, SUC_lexicon_tags_15 },
    { 0x170acd6f, SUC_lexicon_tags_45 },
    { 0x4a4acd6f, SUC_lexicon_tags_6 },
    { 0xf5aacd72, SUC_lexicon_tags_65 },
    { 0xaf6acd73, SUC_lexicon_tags_11 },
    { 0xf94acd70, SUC_lexicon_tags_15 },
    { 0x7ceacd75, SUC_lexicon_tags_27 },
    { 0x84eacd76, SUC_lexicon_tags_8 },
    { 0x182acd77, SUC_lexicon_tags_11 },
    { 0xba8acd72, SUC_lexicon_tags_11 },
    { 0xf28acd77, SUC_lexicon_tags_11 },
    { 0xbe8acd74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592acd80, SUC_lexicon_tags_23 },
    { 0x8baacd80, SUC_lexicon_tags_14 },
    { 0xf8cacd80, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642acd88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x712acd8a, SUC_lexicon_tags_141 },
    { 0x156acd8a, SUC_lexicon_tags_11 },
    { 0xaeeacd8b, SUC_lexicon_tags_15 },
    { 0xbdeacd8a, SUC_lexicon_tags_23 },
    { 0xe26acd8a, SUC_lexicon_tags_19 },
    { 0x46eacd8f, SUC_lexicon_tags_2 },
    { 0x4e8acd90, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x29eacd93, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc28acd95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0acda3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa18acda5, SUC_lexicon_tags_49 },
    { 0x98eacda6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x866acda9, SUC_lexicon_tags_12 },
    { 0x982acdaa, SUC_lexicon_tags_17 },
    { 0x65aacdaa, SUC_lexicon_tags_2 },
    { 0x542acdab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0acdb1, SUC_lexicon_tags_19 },
    { 0xb28acdb2, SUC_lexicon_tags_21 },
    { 0xdbaacdb3, SUC_lexicon_tags_87 },
    { 0xf00acdb4, SUC_lexicon_tags_15 },
    { 0x4dcacdb2, SUC_lexicon_tags_25 },
    { 0x9feacdb4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2acdc0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6acdc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5beacdc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x138acdc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeacdca, SUC_lexicon_tags_27 },
    { 0x534acdca, SUC_lexicon_tags_7 },
    { 0xd38acdcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6acdcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aacdd2, SUC_lexicon_tags_56 },
    { 0x7d8acdd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x840acdd5, SUC_lexicon_tags_19 },
    { 0x524acdd6, SUC_lexicon_tags_25 },
    { 0x954acdd7, SUC_lexicon_tags_15 },
    { 0x828acdd7, SUC_lexicon_tags_11 },
    { 0xdd6acdd8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1bcacddb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3caacddd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0acde0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766acde8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aacdec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300acdf0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cacdf5, SUC_lexicon_tags_8 },
    { 0x370acdf6, SUC_lexicon_tags_9 },
    { 0xedaacdf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2faacdf9, SUC_lexicon_tags_45 },
    { 0xe00acdfa, SUC_lexicon_tags_15 },
    { 0xd16acdfa, SUC_lexicon_tags_2 },
    { 0x666acdfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cace07, SUC_lexicon_tags_14 },
    { 0xfd4ace08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89cace0a, SUC_lexicon_tags_47 },
    { 0xbacace0b, SUC_lexicon_tags_25 },
    { 0xf1cace0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cace10, SUC_lexicon_tags_48 },
    { 0xedeace11, SUC_lexicon_tags_33 },
    { 0x98aace11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614ace16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2aaace18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeace1b, SUC_lexicon_tags_2 },
    { 0x812ace1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8dcace1e, SUC_lexicon_tags_80 },
    { 0xd98ace1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0ace2a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54ace2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92cace31, SUC_lexicon_tags_15 },
    { 0x342ace32, SUC_lexicon_tags_12 },
    { 0x46cace33, SUC_lexicon_tags_6 },
    { 0x72eace34, SUC_lexicon_tags_31 },
    { 0xa6aace33, SUC_lexicon_tags_11 },
    { 0xf08ace33, SUC_lexicon_tags_25 },
    { 0x330ace32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa76ace39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488ace42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cace4b, SUC_lexicon_tags_33 },
    { 0xeecace4c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2baace51, SUC_lexicon_tags_37 },
    { 0xc6aace51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778ace56, SUC_lexicon_tags_25 },
    { 0x420ace56, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xb9eace59, SUC_lexicon_tags_8 },
    { 0x12aace59, SUC_lexicon_tags_1 },
    { 0xf98ace5a, SUC_lexicon_tags_49 },
    { 0x8baace5c, SUC_lexicon_tags_15 },
    { 0x384ace5c, SUC_lexicon_tags_151 },
    { 0xbd4ace5e, SUC_lexicon_tags_28 },
    { 0x102ace5f, SUC_lexicon_tags_65 },
    { 0x246ace5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe5cace62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeace67, SUC_lexicon_tags_6 },
    { 0x774ace68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x35eace6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x666ace6d, SUC_lexicon_tags_21 },
    { 0x502ace6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x432ace71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd62ace73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd22ace75, SUC_lexicon_tags_12 },
    { 0x402ace76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x398ace79, SUC_lexicon_tags_77 },
    { 0xb04ace7a, SUC_lexicon_tags_0 },
    { 0x2f6ace7b, SUC_lexicon_tags_43 },
    { 0x21eace7a, SUC_lexicon_tags_25 },
    { 0xf56ace7d, SUC_lexicon_tags_8 },
    { 0xecaace7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00ace89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcace8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf6ace90, SUC_lexicon_tags_37 },
    { 0x92eace91, SUC_lexicon_tags_26 },
    { 0xde6ace92, SUC_lexicon_tags_1 },
    { 0x7ecace93, SUC_lexicon_tags_17 },
    { 0xe66ace92, SUC_lexicon_tags_11 },
    { 0x92ace92, SUC_lexicon_tags_23 },
    { 0x4feace96, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38ace9b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xba2ace9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdaace9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0acea5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x31eacea7, SUC_lexicon_tags_35 },
    { 0x828acea8, SUC_lexicon_tags_25 },
    { 0xc0cacea9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8acead, SUC_lexicon_tags_37 },
    { 0xe38aceae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59aaceb2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4caceb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x768aceb8, SUC_lexicon_tags_12 },
    { 0xd1caceb8, SUC_lexicon_tags_34 },
    { 0xc1caceba, SUC_lexicon_tags_12 },
    { 0xf7eacebb, SUC_lexicon_tags_37 },
    { 0x3ccacebc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfeeacebe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52acec1, SUC_lexicon_tags_43 },
    { 0x220acec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4acec6, SUC_lexicon_tags_21 },
    { 0x8fcacec7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x336acec9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20caced5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06acee2, SUC_lexicon_tags_29 },
    { 0xdf4acee3, SUC_lexicon_tags_8 },
    { 0xcc6acee4, SUC_lexicon_tags_60 },
    { 0x8f2acee5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cacee8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcaceeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c4aceed, SUC_lexicon_tags_11 },
    { 0x22aceee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x614acef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32aacef2, SUC_lexicon_tags_11 },
    { 0x1b4acef3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366acef7, SUC_lexicon_tags_13 },
    { 0x142acef7, SUC_lexicon_tags_8 },
    { 0x786acef7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186acefd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ceaceff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300acf09, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcacf0e, SUC_lexicon_tags_17 },
    { 0x5c2acf0e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4acf14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78acf19, SUC_lexicon_tags_30 },
    { 0x43aacf19, SUC_lexicon_tags_46 },
    { 0x9aacf1b, SUC_lexicon_tags_31 },
    { 0x6b0acf1c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x460acf1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e4acf21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6aacf23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x34cacf26, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x910acf28, SUC_lexicon_tags_23 },
    { 0x1deacf28, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430acf30, SUC_lexicon_tags_11 },
    { 0xbeacf31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cacf37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xceeacf39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec4acf3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6eacf3d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xce2acf3f, SUC_lexicon_tags_23 },
    { 0x1c0acf40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86aacf42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4acf45, SUC_lexicon_tags_60 },
    { 0x8ceacf46, SUC_lexicon_tags_25 },
    { 0xc62acf47, SUC_lexicon_tags_87 },
    { 0xbbaacf45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x230acf4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a2acf4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4cacf4f, SUC_lexicon_tags_37 },
    { 0xd2aacf50, SUC_lexicon_tags_1 },
    { 0xcaacf51, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x22acf53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cacf59, SUC_lexicon_tags_19 },
    { 0x412acf59, SUC_lexicon_tags_17 },
    { 0x216acf5b, SUC_lexicon_tags_30 },
    { 0x23aacf5b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf5cacf5e, SUC_lexicon_tags_12 },
    { 0x2dcacf5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972acf69, SUC_lexicon_tags_275 },
    { 0x628acf6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaacf6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x928acf72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72acf75, SUC_lexicon_tags_17 },
    { 0xfb4acf76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6c8acf78, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb18acf7a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x21eacf7d, SUC_lexicon_tags_23 },
    { 0x4c6acf7d, SUC_lexicon_tags_15 },
    { 0xa7cacf7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaa2acf81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x27eacf84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606acf93, SUC_lexicon_tags_17 },
    { 0x58cacf94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528acf99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cacf9d, SUC_lexicon_tags_37 },
    { 0x5fcacf9e, SUC_lexicon_tags_25 },
    { 0xdeeacf9e, SUC_lexicon_tags_37 },
    { 0x168acfa0, SUC_lexicon_tags_37 },
    { 0x816acfa0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x38eacfa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x184acfa6, SUC_lexicon_tags_17 },
    { 0x652acfa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41eacfaa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42eacfb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84acfb5, SUC_lexicon_tags_16 },
    { 0x5a4acfb6, SUC_lexicon_tags_23 },
    { 0x8b6acfb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1cacfb9, SUC_lexicon_tags_2 },
    { 0x878acfba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cacfc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40acfc5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376acfc9, SUC_lexicon_tags_17 },
    { 0x28aacfc9, SUC_lexicon_tags_38 },
    { 0x116acfcb, SUC_lexicon_tags_11 },
    { 0x356acfcc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cacfcf, SUC_lexicon_tags_29 },
    { 0x2beacfd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaacfd4, SUC_lexicon_tags_15 },
    { 0x210acfd5, SUC_lexicon_tags_45 },
    { 0xfaeacfd5, SUC_lexicon_tags_47 },
    { 0xf90acfd7, SUC_lexicon_tags_12 },
    { 0xd42acfd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecacfde, SUC_lexicon_tags_33 },
    { 0xd5cacfdf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa92acfe1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52acfe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd0acfe5, SUC_lexicon_tags_46 },
    { 0x1ccacfe6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x540acfe8, SUC_lexicon_tags_108 },
    { 0xe0eacfe8, SUC_lexicon_tags_43 },
    { 0x1eeacfe9, SUC_lexicon_tags_37 },
    { 0x92acfe9, SUC_lexicon_tags_25 },
    { 0xcd6acfe9, SUC_lexicon_tags_12 },
    { 0xb9aacfe8, SUC_lexicon_tags_46 },
    { 0x2f6acfed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb32acff0, SUC_lexicon_tags_1 },
    { 0x59cacff1, SUC_lexicon_tags_22 },
    { 0xdf0acff0, SUC_lexicon_tags_39 },
    { 0xb0acff3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e4acff5, SUC_lexicon_tags_8 },
    { 0x6d4acff6, SUC_lexicon_tags_46 },
    { 0x5e2acff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172acffb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x454acffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0aad000, SUC_lexicon_tags_13 },
    { 0x3d4ad000, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x91cad003, SUC_lexicon_tags_63 },
    { 0xa78ad004, SUC_lexicon_tags_2 },
    { 0x6a2ad005, SUC_lexicon_tags_16 },
    { 0x77aad006, SUC_lexicon_tags_25 },
    { 0x5a4ad006, SUC_lexicon_tags_26 },
    { 0x4aead008, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc58ad00a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x32aad00c, SUC_lexicon_tags_27 },
    { 0x8c0ad00d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xae4ad00f, SUC_lexicon_tags_25 },
    { 0x98ad010, SUC_lexicon_tags_2 },
    { 0x64ad011, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0ad014, SUC_lexicon_tags_6 },
    { 0x908ad015, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ad018, SUC_lexicon_tags_8 },
    { 0xf30ad018, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aad01c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ead021, SUC_lexicon_tags_12 },
    { 0xd68ad022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82ad028, SUC_lexicon_tags_14 },
    { 0x60ead028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cad02b, SUC_lexicon_tags_0 },
    { 0x81aad02c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x486ad02e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562ad038, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2ad03f, SUC_lexicon_tags_26 },
    { 0xf8aad040, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0ad043, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6ad048, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcad04c, SUC_lexicon_tags_1 },
    { 0xd62ad04d, SUC_lexicon_tags_2 },
    { 0xbdead04e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18ad054, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x972ad057, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf16ad059, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ead05c, SUC_lexicon_tags_182 },
    { 0x2a6ad05d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaad062, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fead065, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0aad067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x674ad06a, SUC_lexicon_tags_14 },
    { 0xae6ad06b, SUC_lexicon_tags_37 },
    { 0x78cad06a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcad06f, SUC_lexicon_tags_26 },
    { 0xc6aad070, SUC_lexicon_tags_4 },
    { 0x5a2ad070, SUC_lexicon_tags_34 },
    { 0x866ad072, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x172ad074, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8ad07b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65cad07d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7f8ad07f, SUC_lexicon_tags_25 },
    { 0xa20ad080, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x870ad083, SUC_lexicon_tags_25 },
    { 0xfdcad084, SUC_lexicon_tags_16 },
    { 0x132ad085, SUC_lexicon_tags_19 },
    { 0x85ead086, SUC_lexicon_tags_38 },
    { 0x17cad087, SUC_lexicon_tags_63 },
    { 0xdacad088, SUC_lexicon_tags_12 },
    { 0xabcad088, SUC_lexicon_tags_14 },
    { 0x89ead08a, SUC_lexicon_tags_25 },
    { 0x21cad08a, SUC_lexicon_tags_2 },
    { 0x72ead08c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ad090, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bead09a, SUC_lexicon_tags_29 },
    { 0x57ead09a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7baad09e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x790ad0a1, SUC_lexicon_tags_47 },
    { 0x80ad0a2, SUC_lexicon_tags_6 },
    { 0xb2ad0a2, SUC_lexicon_tags_12 },
    { 0x262ad0a3, SUC_lexicon_tags_7 },
    { 0x7f2ad0a2, SUC_lexicon_tags_7 },
    { 0xf16ad0a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4ad0a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4ad0b0, SUC_lexicon_tags_60 },
    { 0xd94ad0b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd20ad0b3, SUC_lexicon_tags_241 },
    { 0x29ead0b3, SUC_lexicon_tags_25 },
    { 0xc16ad0b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ead0b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b8ad0ba, SUC_lexicon_tags_17 },
    { 0x38cad0ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94ad0c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4ad0c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcad0cb, SUC_lexicon_tags_49 },
    { 0xb96ad0cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaad0d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02ad0d9, SUC_lexicon_tags_47 },
    { 0x268ad0d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc6ad0dc, SUC_lexicon_tags_37 },
    { 0xf9aad0dc, SUC_lexicon_tags_38 },
    { 0x88ead0dc, SUC_lexicon_tags_2 },
    { 0x608ad0df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caad0e8, SUC_lexicon_tags_14 },
    { 0x1dead0e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ad0eb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06ad0ef, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x94ead0f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912ad0f8, SUC_lexicon_tags_26 },
    { 0xcb6ad0f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bead0fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ead102, SUC_lexicon_tags_17 },
    { 0xcacad103, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ead107, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ad10c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaad10f, SUC_lexicon_tags_11 },
    { 0x97ead110, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fead113, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x676ad116, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4ad11c, SUC_lexicon_tags_19 },
    { 0x104ad11d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4ad124, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x806ad127, SUC_lexicon_tags_11 },
    { 0xb4cad128, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688ad12d, SUC_lexicon_tags_13 },
    { 0x4a6ad12e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700ad132, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdd8ad134, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfcad136, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cad140, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cad144, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7eead146, SUC_lexicon_tags_12 },
    { 0xb6cad146, SUC_lexicon_tags_34 },
    { 0x284ad148, SUC_lexicon_tags_2 },
    { 0xca4ad148, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7ead14b, SUC_lexicon_tags_17 },
    { 0x4d4ad14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x892ad14e, SUC_lexicon_tags_12 },
    { 0xe0ead14f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8ad157, SUC_lexicon_tags_8 },
    { 0x5b6ad158, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112ad15d, SUC_lexicon_tags_27 },
    { 0x740ad15e, SUC_lexicon_tags_19 },
    { 0x35cad15f, SUC_lexicon_tags_162 },
    { 0x76ad15f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc00ad162, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2ad165, SUC_lexicon_tags_26 },
    { 0x67aad165, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x82aad169, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xceaad16b, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922ad170, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e2ad172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26aad178, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6ad17b, SUC_lexicon_tags_31 },
    { 0x3eead17c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cad183, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2ad186, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cad18b, SUC_lexicon_tags_30 },
    { 0x700ad18c, SUC_lexicon_tags_0 },
    { 0x524ad18d, SUC_lexicon_tags_31 },
    { 0x5c2ad18e, SUC_lexicon_tags_37 },
    { 0x44ead18f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5baad191, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x156ad194, SUC_lexicon_tags_12 },
    { 0xa6aad195, SUC_lexicon_tags_46 },
    { 0x9aad196, SUC_lexicon_tags_0 },
    { 0x954ad195, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96ad19b, SUC_lexicon_tags_37 },
    { 0x3b4ad19b, SUC_lexicon_tags_7 },
    { 0x776ad19d, SUC_lexicon_tags_15 },
    { 0x3e2ad19e, SUC_lexicon_tags_33 },
    { 0x908ad19f, SUC_lexicon_tags_14 },
    { 0x60cad19f, SUC_lexicon_tags_17 },
    { 0x336ad1a0, SUC_lexicon_tags_28 },
    { 0x118ad1a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x438ad1a5, SUC_lexicon_tags_33 },
    { 0x44ad1a6, SUC_lexicon_tags_17 },
    { 0xcecad1a7, SUC_lexicon_tags_12 },
    { 0xc3cad1a7, SUC_lexicon_tags_6 },
    { 0x504ad1a9, SUC_lexicon_tags_15 },
    { 0x33ead1a6, SUC_lexicon_tags_38 },
    { 0xd80ad1a7, SUC_lexicon_tags_1 },
    { 0x7c0ad1a6, SUC_lexicon_tags_32 },
    { 0xa8ad1a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb88ad1af, SUC_lexicon_tags_23 },
    { 0x8aaad1af, SUC_lexicon_tags_23 },
    { 0x6d8ad1b1, SUC_lexicon_tags_55 },
    { 0x8aad1b2, SUC_lexicon_tags_38 },
    { 0xc0cad1b3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4ad1b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880ad1c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44ad1c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e4ad1c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5cad1c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72ad1cd, SUC_lexicon_tags_2 },
    { 0x8c6ad1ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82ad1d5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212ad1da, SUC_lexicon_tags_77 },
    { 0xf58ad1da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x66cad1dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x498ad1e0, SUC_lexicon_tags_151 },
    { 0xef6ad1e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cad1e4, SUC_lexicon_tags_28 },
    { 0xff8ad1e4, SUC_lexicon_tags_11 },
    { 0xdc6ad1e6, SUC_lexicon_tags_12 },
    { 0xd64ad1e6, SUC_lexicon_tags_9 },
    { 0x5ccad1e7, SUC_lexicon_tags_29 },
    { 0xb6cad1e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cad1ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x228ad1ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bead1f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x740ad1f6, SUC_lexicon_tags_39 },
    { 0x4f2ad1f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2ad1fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caad204, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96aad208, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x56cad20a, SUC_lexicon_tags_11 },
    { 0x914ad20b, SUC_lexicon_tags_140 },
    { 0xc6ead20b, SUC_lexicon_tags_2 },
    { 0x728ad20d, SUC_lexicon_tags_14 },
    { 0x8a2ad20d, SUC_lexicon_tags_7 },
    { 0x966ad20f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cad213, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaead216, SUC_lexicon_tags_30 },
    { 0x11aad217, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ad21a, SUC_lexicon_tags_39 },
    { 0x682ad21a, SUC_lexicon_tags_1 },
    { 0x616ad21b, SUC_lexicon_tags_43 },
    { 0xdead21d, SUC_lexicon_tags_25 },
    { 0x54cad21e, SUC_lexicon_tags_47 },
    { 0x318ad21f, SUC_lexicon_tags_45 },
    { 0xe78ad220, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dead225, SUC_lexicon_tags_23 },
    { 0x250ad226, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ead229, SUC_lexicon_tags_29 },
    { 0x282ad22a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc1cad22c, SUC_lexicon_tags_19 },
    { 0xcb8ad22d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6aad233, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8ad236, SUC_lexicon_tags_81 },
    { 0xf8aad236, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0ad23a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8ad242, SUC_lexicon_tags_66 },
    { 0x452ad243, SUC_lexicon_tags_8 },
    { 0xe54ad244, SUC_lexicon_tags_82 },
    { 0xbc4ad243, SUC_lexicon_tags_23 },
    { 0x56aad245, SUC_lexicon_tags_4 },
    { 0xe20ad242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4ad249, SUC_lexicon_tags_37 },
    { 0xddaad24a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x128ad24c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4ad252, SUC_lexicon_tags_37 },
    { 0x3dead253, SUC_lexicon_tags_15 },
    { 0x2c8ad254, SUC_lexicon_tags_1 },
    { 0x928ad254, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x144ad257, SUC_lexicon_tags_11 },
    { 0xc1cad257, SUC_lexicon_tags_31 },
    { 0x466ad259, SUC_lexicon_tags_45 },
    { 0xd50ad25a, SUC_lexicon_tags_0 },
    { 0x2f0ad25a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54ad25f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x604ad261, SUC_lexicon_tags_25 },
    { 0xcb8ad262, SUC_lexicon_tags_37 },
    { 0x206ad263, SUC_lexicon_tags_14 },
    { 0xca2ad264, SUC_lexicon_tags_11 },
    { 0x59ead264, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1b4ad267, SUC_lexicon_tags_25 },
    { 0x596ad268, SUC_lexicon_tags_27 },
    { 0x942ad269, SUC_lexicon_tags_1 },
    { 0x8d6ad26a, SUC_lexicon_tags_11 },
    { 0x412ad26a, SUC_lexicon_tags_46 },
    { 0x28cad268, SUC_lexicon_tags_5 },
    { 0xb0ad26d, SUC_lexicon_tags_60 },
    { 0x752ad26c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x46cad270, SUC_lexicon_tags_60 },
    { 0x578ad271, SUC_lexicon_tags_49 },
    { 0xaeaad271, SUC_lexicon_tags_60 },
    { 0xa96ad272, SUC_lexicon_tags_47 },
    { 0xbd4ad274, SUC_lexicon_tags_14 },
    { 0xb86ad271, SUC_lexicon_tags_14 },
    { 0x1c8ad276, SUC_lexicon_tags_12 },
    { 0xe9aad277, SUC_lexicon_tags_1 },
    { 0x830ad277, SUC_lexicon_tags_15 },
    { 0x240ad279, SUC_lexicon_tags_36 },
    { 0xc04ad27a, SUC_lexicon_tags_37 },
    { 0x90cad27b, SUC_lexicon_tags_14 },
    { 0x316ad27a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87aad282, SUC_lexicon_tags_1 },
    { 0x490ad282, SUC_lexicon_tags_4 },
    { 0x9f8ad283, SUC_lexicon_tags_55 },
    { 0xea0ad284, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x81cad287, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3aad28c, SUC_lexicon_tags_15 },
    { 0xe3aad28d, SUC_lexicon_tags_6 },
    { 0xdeead28e, SUC_lexicon_tags_7 },
    { 0xee8ad28f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ad296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24aad29a, SUC_lexicon_tags_25 },
    { 0xb68ad29b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4ad2a0, SUC_lexicon_tags_7 },
    { 0xb48ad2a0, SUC_lexicon_tags_11 },
    { 0x644ad2a0, SUC_lexicon_tags_48 },
    { 0x284ad2a3, SUC_lexicon_tags_19 },
    { 0x22aad2a4, SUC_lexicon_tags_12 },
    { 0x178ad2a5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x880ad2a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c0ad2a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfead2ae, SUC_lexicon_tags_13 },
    { 0xc8cad2ae, SUC_lexicon_tags_38 },
    { 0x936ad2ae, SUC_lexicon_tags_82 },
    { 0x8fead2b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x822ad2b4, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2ad2bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cad2c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6ad2c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604ad2c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6aaad2ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cad2d1, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x14ead2d3, SUC_lexicon_tags_7 },
    { 0x546ad2d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552ad2d8, SUC_lexicon_tags_35 },
    { 0xc0cad2d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad8ad2db, SUC_lexicon_tags_2 },
    { 0x4d0ad2dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24ad2df, SUC_lexicon_tags_25 },
    { 0xd78ad2df, SUC_lexicon_tags_30 },
    { 0xc76ad2e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2ad2e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x668ad2ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8aad2f2, SUC_lexicon_tags_37 },
    { 0x900ad2f2, SUC_lexicon_tags_25 },
    { 0xdd4ad2f4, SUC_lexicon_tags_49 },
    { 0x426ad2f4, SUC_lexicon_tags_17 },
    { 0xcaaad2f4, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108ad2fd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8ad301, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x364ad303, SUC_lexicon_tags_8 },
    { 0xd0ad304, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aad308, SUC_lexicon_tags_9 },
    { 0x7a2ad308, SUC_lexicon_tags_39 },
    { 0x3f6ad309, SUC_lexicon_tags_4 },
    { 0xf0cad308, SUC_lexicon_tags_12 },
    { 0xc70ad30c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cad310, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3e6ad312, SUC_lexicon_tags_25 },
    { 0x7cad313, SUC_lexicon_tags_27 },
    { 0x6d2ad312, SUC_lexicon_tags_30 },
    { 0x15aad315, SUC_lexicon_tags_28 },
    { 0xebead315, SUC_lexicon_tags_17 },
    { 0xda2ad315, SUC_lexicon_tags_12 },
    { 0x58ad318, SUC_lexicon_tags_33 },
    { 0x864ad315, SUC_lexicon_tags_27 },
    { 0xba0ad31a, SUC_lexicon_tags_11 },
    { 0xac8ad315, SUC_lexicon_tags_39 },
    { 0x256ad31c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x182ad31f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0ad322, SUC_lexicon_tags_6 },
    { 0xebcad323, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7ead325, SUC_lexicon_tags_17 },
    { 0x632ad326, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cad32a, SUC_lexicon_tags_154 },
    { 0x70ad32b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4ad337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190ad33b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ad341, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae2ad343, SUC_lexicon_tags_59 },
    { 0x7a6ad343, SUC_lexicon_tags_134 },
    { 0xe44ad345, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3caad347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62aad349, SUC_lexicon_tags_47 },
    { 0x648ad34a, SUC_lexicon_tags_7 },
    { 0xfa8ad34a, SUC_lexicon_tags_0 },
    { 0xd38ad34c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514ad351, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x880ad353, SUC_lexicon_tags_12 },
    { 0x14aad354, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4dead356, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24ad35c, SUC_lexicon_tags_15 },
    { 0x5e4ad35c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x426ad35f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe72ad362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x430ad365, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698ad36a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84ad36f, SUC_lexicon_tags_37 },
    { 0xc0ead370, SUC_lexicon_tags_49 },
    { 0x938ad370, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x81aad373, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30ad379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8ad37e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb94ad380, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cad386, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2ad389, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb78ad38b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc68ad38d, SUC_lexicon_tags_7 },
    { 0xbfaad38e, SUC_lexicon_tags_23 },
    { 0x6a6ad38e, SUC_lexicon_tags_6 },
    { 0x1baad390, SUC_lexicon_tags_12 },
    { 0xb2ad38d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1cead393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2ad396, SUC_lexicon_tags_6 },
    { 0x612ad397, SUC_lexicon_tags_25 },
    { 0x5d6ad397, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0ad3a1, SUC_lexicon_tags_14 },
    { 0xe20ad3a1, SUC_lexicon_tags_1 },
    { 0x1aead3a3, SUC_lexicon_tags_26 },
    { 0x82aad3a4, SUC_lexicon_tags_1 },
    { 0xd6cad3a5, SUC_lexicon_tags_14 },
    { 0x238ad3a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xffaad3a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x584ad3ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e6ad3ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf3ead3af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ead3b2, SUC_lexicon_tags_15 },
    { 0x942ad3b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cad3b6, SUC_lexicon_tags_43 },
    { 0x352ad3b7, SUC_lexicon_tags_1 },
    { 0x30ad3b8, SUC_lexicon_tags_35 },
    { 0xbbcad3b7, SUC_lexicon_tags_15 },
    { 0x12ead3ba, SUC_lexicon_tags_33 },
    { 0x916ad3bb, SUC_lexicon_tags_37 },
    { 0xa78ad3b9, SUC_lexicon_tags_15 },
    { 0xa9aad3bd, SUC_lexicon_tags_37 },
    { 0xcead3be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x278ad3c0, SUC_lexicon_tags_27 },
    { 0x5c4ad3c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40ad3c8, SUC_lexicon_tags_23 },
    { 0xbd8ad3c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752ad3ce, SUC_lexicon_tags_2 },
    { 0x806ad3cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcad3d2, SUC_lexicon_tags_2 },
    { 0x82ead3d3, SUC_lexicon_tags_24 },
    { 0xac0ad3d3, SUC_lexicon_tags_73 },
    { 0x32aad3d5, SUC_lexicon_tags_60 },
    { 0x396ad3d5, SUC_lexicon_tags_12 },
    { 0x26aad3d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb66ad3d9, SUC_lexicon_tags_58 },
    { 0x580ad3da, SUC_lexicon_tags_47 },
    { 0x8baad3da, SUC_lexicon_tags_19 },
    { 0x148ad3da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdead3e1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cad3e4, SUC_lexicon_tags_11 },
    { 0x532ad3e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182ad3f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a8ad3f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x448ad3f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x312ad3f8, SUC_lexicon_tags_29 },
    { 0x6fead3f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x316ad3fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cad3ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecad402, SUC_lexicon_tags_29 },
    { 0x674ad402, SUC_lexicon_tags_15 },
    { 0xfe0ad402, SUC_lexicon_tags_15 },
    { 0x58cad405, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6ead407, SUC_lexicon_tags_15 },
    { 0xb02ad408, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3caad40a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380ad40e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dead411, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cad415, SUC_lexicon_tags_17 },
    { 0x34ead416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a6ad418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcad41b, SUC_lexicon_tags_2 },
    { 0x9baad41b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8ad422, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69ead424, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ad427, SUC_lexicon_tags_60 },
    { 0x3cad428, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x324ad42a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d6ad42c, SUC_lexicon_tags_15 },
    { 0x602ad42c, SUC_lexicon_tags_25 },
    { 0x302ad42d, SUC_lexicon_tags_37 },
    { 0x994ad42f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72ad432, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82aad435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaad439, SUC_lexicon_tags_1 },
    { 0x82cad43a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3aaad43c, SUC_lexicon_tags_17 },
    { 0x516ad43c, SUC_lexicon_tags_25 },
    { 0x152ad43e, SUC_lexicon_tags_17 },
    { 0xf4cad43f, SUC_lexicon_tags_14 },
    { 0xd36ad43d, SUC_lexicon_tags_12 },
    { 0x862ad441, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10cad449, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cad44d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x628ad44f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ead452, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x2e4ad454, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654ad458, SUC_lexicon_tags_7 },
    { 0xc20ad458, SUC_lexicon_tags_23 },
    { 0x962ad45a, SUC_lexicon_tags_37 },
    { 0x6e2ad45a, SUC_lexicon_tags_2 },
    { 0xfaead458, SUC_lexicon_tags_2 },
    { 0x28ead45d, SUC_lexicon_tags_36 },
    { 0x5f4ad45e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x302ad460, SUC_lexicon_tags_14 },
    { 0xc62ad461, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28ad464, SUC_lexicon_tags_10 },
    { 0x9baad465, SUC_lexicon_tags_15 },
    { 0x4eaad466, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e4ad468, SUC_lexicon_tags_17 },
    { 0x19ead469, SUC_lexicon_tags_2 },
    { 0x220ad46a, SUC_lexicon_tags_7 },
    { 0x346ad46a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc32ad46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4ad472, SUC_lexicon_tags_8 },
    { 0x848ad472, SUC_lexicon_tags_11 },
    { 0x19ead474, SUC_lexicon_tags_19 },
    { 0x7bcad473, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4ad47a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a8ad47c, SUC_lexicon_tags_74 },
    { 0x2bead47d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522ad485, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2ad489, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6ad494, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ead497, SUC_lexicon_tags_37 },
    { 0xdf2ad498, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cad49c, SUC_lexicon_tags_7 },
    { 0x48aad49d, SUC_lexicon_tags_4 },
    { 0xfe0ad49e, SUC_lexicon_tags_2 },
    { 0xc92ad49f, SUC_lexicon_tags_30 },
    { 0xf42ad49e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424ad4a4, SUC_lexicon_tags_63 },
    { 0x3f2ad4a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174ad4a9, SUC_lexicon_tags_23 },
    { 0x2faad4a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ead4ad, SUC_lexicon_tags_37 },
    { 0xa64ad4ae, SUC_lexicon_tags_179 },
    { 0xa74ad4af, SUC_lexicon_tags_60 },
    { 0xcd6ad4b0, SUC_lexicon_tags_17 },
    { 0x6c6ad4af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18ad4b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90cad4b8, SUC_lexicon_tags_38 },
    { 0x252ad4b9, SUC_lexicon_tags_26 },
    { 0xa0aad4ba, SUC_lexicon_tags_7 },
    { 0xde6ad4bb, SUC_lexicon_tags_43 },
    { 0xff2ad4bc, SUC_lexicon_tags_17 },
    { 0xbe2ad4bd, SUC_lexicon_tags_11 },
    { 0x942ad4bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcad4c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36ad4cb, SUC_lexicon_tags_27 },
    { 0xa30ad4cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6ad4d0, SUC_lexicon_tags_17 },
    { 0x9b2ad4d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x650ad4d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbead4da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ead4dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00ad4e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68ad4e7, SUC_lexicon_tags_17 },
    { 0xc2cad4e8, SUC_lexicon_tags_7 },
    { 0xd2ead4e9, SUC_lexicon_tags_12 },
    { 0x93cad4ea, SUC_lexicon_tags_26 },
    { 0x8c8ad4e8, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62ad4ee, SUC_lexicon_tags_16 },
    { 0x754ad4ef, SUC_lexicon_tags_71 },
    { 0xc94ad4ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aad4f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0ad4f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328ad4fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4ead4ff, SUC_lexicon_tags_15 },
    { 0x828ad500, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfeead502, SUC_lexicon_tags_16 },
    { 0x1eead503, SUC_lexicon_tags_77 },
    { 0xebead503, SUC_lexicon_tags_2 },
    { 0xacad504, SUC_lexicon_tags_11 },
    { 0x8cead506, SUC_lexicon_tags_7 },
    { 0x2bcad507, SUC_lexicon_tags_12 },
    { 0x94cad508, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ead50b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x508ad50d, SUC_lexicon_tags_48 },
    { 0xc8cad50e, SUC_lexicon_tags_7 },
    { 0xb0cad50d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcaad511, SUC_lexicon_tags_172 },
    { 0xeb0ad512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4ad515, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dead51a, SUC_lexicon_tags_17 },
    { 0xe6cad51a, SUC_lexicon_tags_37 },
    { 0x9b2ad51b, SUC_lexicon_tags_37 },
    { 0xaa8ad51d, SUC_lexicon_tags_4 },
    { 0x12ead51e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dead521, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe96ad523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe34ad525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04ad528, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x70ad52a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736ad52e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106ad535, SUC_lexicon_tags_11 },
    { 0x440ad535, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ad53b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0ad541, SUC_lexicon_tags_7 },
    { 0x540ad542, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52aad544, SUC_lexicon_tags_0 },
    { 0xe88ad545, SUC_lexicon_tags_17 },
    { 0xe62ad546, SUC_lexicon_tags_19 },
    { 0x77ead547, SUC_lexicon_tags_17 },
    { 0xcccad548, SUC_lexicon_tags_2 },
    { 0x722ad546, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eead54c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe76ad54f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2faad551, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6ad554, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20ad559, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ead55c, SUC_lexicon_tags_25 },
    { 0xdeaad55d, SUC_lexicon_tags_12 },
    { 0xa7cad55e, SUC_lexicon_tags_1 },
    { 0xd8ad55f, SUC_lexicon_tags_21 },
    { 0x9f2ad55f, SUC_lexicon_tags_37 },
    { 0x4caad55d, SUC_lexicon_tags_1 },
    { 0x406ad562, SUC_lexicon_tags_2 },
    { 0xa9ead563, SUC_lexicon_tags_60 },
    { 0xf34ad563, SUC_lexicon_tags_17 },
    { 0xc36ad562, SUC_lexicon_tags_15 },
    { 0x87ead564, SUC_lexicon_tags_31 },
    { 0xfb8ad563, SUC_lexicon_tags_47 },
    { 0x8a2ad55f, SUC_lexicon_tags_14 },
    { 0x3b2ad561, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecad56f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734ad573, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x46aad575, SUC_lexicon_tags_47 },
    { 0xcecad576, SUC_lexicon_tags_37 },
    { 0xf00ad577, SUC_lexicon_tags_11 },
    { 0x562ad577, SUC_lexicon_tags_103 },
    { 0xa8ad575, SUC_lexicon_tags_37 },
    { 0xd7aad576, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2ad582, SUC_lexicon_tags_0 },
    { 0xfc8ad583, SUC_lexicon_tags_12 },
    { 0xd68ad584, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ead588, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ead58e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x72ad590, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aad595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x134ad597, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720ad59b, SUC_lexicon_tags_17 },
    { 0xe64ad59c, SUC_lexicon_tags_15 },
    { 0x544ad59c, SUC_lexicon_tags_17 },
    { 0xd28ad59b, SUC_lexicon_tags_12 },
    { 0xabaad59f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46ad5a3, SUC_lexicon_tags_25 },
    { 0xaf0ad5a3, SUC_lexicon_tags_15 },
    { 0x6a0ad5a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0aad5a9, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0ad5ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6f4ad5af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb60ad5b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe94ad5b3, SUC_lexicon_tags_48 },
    { 0x1cead5b3, SUC_lexicon_tags_14 },
    { 0x400ad5b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8ad5bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ead5be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aad5c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430ad5ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc48ad5d0, SUC_lexicon_tags_32 },
    { 0xe02ad5d1, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6aad5d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980ad5e0, SUC_lexicon_tags_30 },
    { 0x804ad5e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f6ad5e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6ad5e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe70ad5e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x116ad5eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaad5f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ead5f6, SUC_lexicon_tags_11 },
    { 0x41ead5f6, SUC_lexicon_tags_7 },
    { 0xecaad5f8, SUC_lexicon_tags_38 },
    { 0xa7cad5f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82ad5ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2d2ad601, SUC_lexicon_tags_136 },
    { 0xaa0ad601, SUC_lexicon_tags_38 },
    { 0xc6ad603, SUC_lexicon_tags_46 },
    { 0x70ead602, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d0ad606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x974ad608, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2ad60c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ead610, SUC_lexicon_tags_11 },
    { 0xd1ead610, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf06ad613, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdacad616, SUC_lexicon_tags_25 },
    { 0xa40ad617, SUC_lexicon_tags_43 },
    { 0xe38ad617, SUC_lexicon_tags_47 },
    { 0xfbead617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x186ad61b, SUC_lexicon_tags_14 },
    { 0x252ad61c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0ad61f, SUC_lexicon_tags_33 },
    { 0x87ead61f, SUC_lexicon_tags_28 },
    { 0xab0ad61f, SUC_lexicon_tags_32 },
    { 0xc66ad61f, SUC_lexicon_tags_46 },
    { 0xc62ad622, SUC_lexicon_tags_17 },
    { 0x416ad61f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaad628, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2ad632, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc46ad634, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976ad639, SUC_lexicon_tags_14 },
    { 0xde0ad639, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x246ad63c, SUC_lexicon_tags_47 },
    { 0xe48ad63d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cad640, SUC_lexicon_tags_14 },
    { 0xb5aad641, SUC_lexicon_tags_14 },
    { 0x1fcad641, SUC_lexicon_tags_26 },
    { 0x12cad643, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x194ad646, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aad651, SUC_lexicon_tags_15 },
    { 0x82cad652, SUC_lexicon_tags_15 },
    { 0xb46ad652, SUC_lexicon_tags_37 },
    { 0x352ad654, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb56ad656, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844ad65f, SUC_lexicon_tags_0 },
    { 0x98ad65f, SUC_lexicon_tags_22 },
    { 0x7acad661, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4aad663, SUC_lexicon_tags_25 },
    { 0xe9aad664, SUC_lexicon_tags_7 },
    { 0x7e0ad663, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xdb0ad667, SUC_lexicon_tags_47 },
    { 0x238ad668, SUC_lexicon_tags_17 },
    { 0x5cead669, SUC_lexicon_tags_19 },
    { 0x328ad669, SUC_lexicon_tags_11 },
    { 0x568ad667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89ead66d, SUC_lexicon_tags_12 },
    { 0xa0ead66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14ad670, SUC_lexicon_tags_27 },
    { 0xd22ad671, SUC_lexicon_tags_45 },
    { 0xaa0ad671, SUC_lexicon_tags_32 },
    { 0x218ad671, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb9aad675, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340ad679, SUC_lexicon_tags_2 },
    { 0x60ead67a, SUC_lexicon_tags_15 },
    { 0xce0ad679, SUC_lexicon_tags_15 },
    { 0x972ad67a, SUC_lexicon_tags_32 },
    { 0x914ad67d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe18ad67f, SUC_lexicon_tags_11 },
    { 0x454ad680, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4ad685, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6ead687, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ad68d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x256ad68f, SUC_lexicon_tags_22 },
    { 0xcc2ad690, SUC_lexicon_tags_11 },
    { 0x7faad690, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab4ad693, SUC_lexicon_tags_30 },
    { 0xd0cad693, SUC_lexicon_tags_30 },
    { 0x648ad695, SUC_lexicon_tags_15 },
    { 0x9d6ad695, SUC_lexicon_tags_1 },
    { 0x9caad695, SUC_lexicon_tags_12 },
    { 0x62ead698, SUC_lexicon_tags_7 },
    { 0x8f0ad697, SUC_lexicon_tags_31 },
    { 0x682ad695, SUC_lexicon_tags_10 },
    { 0x432ad699, SUC_lexicon_tags_60 },
    { 0x86cad695, SUC_lexicon_tags_10 },
    { 0x5b6ad69d, SUC_lexicon_tags_18 },
    { 0x26aad69e, SUC_lexicon_tags_25 },
    { 0xf4aad69f, SUC_lexicon_tags_33 },
    { 0x686ad6a0, SUC_lexicon_tags_27 },
    { 0x3a2ad6a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112ad6a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192ad6ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79aad6ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0ad6b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632ad6bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x484ad6be, SUC_lexicon_tags_63 },
    { 0xfe6ad6be, SUC_lexicon_tags_37 },
    { 0xa40ad6c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0ad6c4, SUC_lexicon_tags_15 },
    { 0x81ead6c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x330ad6c7, SUC_lexicon_tags_37 },
    { 0x2aead6c7, SUC_lexicon_tags_7 },
    { 0x52cad6c9, SUC_lexicon_tags_11 },
    { 0xecead6c8, SUC_lexicon_tags_7 },
    { 0x546ad6c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ad6d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56aad6d6, SUC_lexicon_tags_11 },
    { 0x686ad6d7, SUC_lexicon_tags_25 },
    { 0x9baad6d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2b8ad6da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e2ad6dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ead6e3, SUC_lexicon_tags_2 },
    { 0x1ccad6e4, SUC_lexicon_tags_37 },
    { 0xd68ad6e5, SUC_lexicon_tags_21 },
    { 0x52aad6e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x180ad6e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8ad6eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcd8ad6ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8ad6f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecad6f4, SUC_lexicon_tags_2 },
    { 0x136ad6f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646ad6f9, SUC_lexicon_tags_0 },
    { 0xe9cad6fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6caad700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558ad704, SUC_lexicon_tags_1 },
    { 0xf02ad705, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59aad70a, SUC_lexicon_tags_17 },
    { 0xa8ad70b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x350ad70d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2cad70f, SUC_lexicon_tags_1 },
    { 0x14cad70f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x704ad712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x966ad714, SUC_lexicon_tags_32 },
    { 0xbe0ad715, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4ad718, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc62ad71a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaad71e, SUC_lexicon_tags_25 },
    { 0xdccad71f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30ad722, SUC_lexicon_tags_17 },
    { 0xb14ad723, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x478ad725, SUC_lexicon_tags_68 },
    { 0xfd0ad726, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ead72a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0ad72c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6ad72e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec0ad730, SUC_lexicon_tags_37 },
    { 0x396ad731, SUC_lexicon_tags_12 },
    { 0xeecad731, SUC_lexicon_tags_42 },
    { 0xc18ad732, SUC_lexicon_tags_0 },
    { 0xdd6ad733, SUC_lexicon_tags_2 },
    { 0x67aad730, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360ad73f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad6ad741, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x786ad744, SUC_lexicon_tags_43 },
    { 0xec2ad745, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd22ad747, SUC_lexicon_tags_17 },
    { 0x10ead748, SUC_lexicon_tags_37 },
    { 0x852ad749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66ead74b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd7aad74d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaad751, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x406ad754, SUC_lexicon_tags_2 },
    { 0x304ad755, SUC_lexicon_tags_4 },
    { 0x500ad756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aead758, SUC_lexicon_tags_37 },
    { 0xccad758, SUC_lexicon_tags_37 },
    { 0x38ead759, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc6ad75c, SUC_lexicon_tags_0 },
    { 0x498ad75d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f8ad75f, SUC_lexicon_tags_27 },
    { 0xb56ad75f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8ad769, SUC_lexicon_tags_8 },
    { 0xdcad76a, SUC_lexicon_tags_38 },
    { 0x1a4ad769, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24ad76d, SUC_lexicon_tags_41 },
    { 0x99cad76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cad771, SUC_lexicon_tags_14 },
    { 0xcd6ad772, SUC_lexicon_tags_16 },
    { 0x5f8ad773, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x446ad776, SUC_lexicon_tags_14 },
    { 0x63aad777, SUC_lexicon_tags_1 },
    { 0x62ead777, SUC_lexicon_tags_15 },
    { 0x708ad777, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xabead77b, SUC_lexicon_tags_25 },
    { 0xf16ad77c, SUC_lexicon_tags_25 },
    { 0x28ead77c, SUC_lexicon_tags_28 },
    { 0x73cad77e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478ad782, SUC_lexicon_tags_25 },
    { 0x27cad783, SUC_lexicon_tags_66 },
    { 0x97ead784, SUC_lexicon_tags_63 },
    { 0x386ad784, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x832ad787, SUC_lexicon_tags_23 },
    { 0xb9aad788, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4ad78f, SUC_lexicon_tags_61 },
    { 0xf4ad78f, SUC_lexicon_tags_29 },
    { 0xda6ad78f, SUC_lexicon_tags_12 },
    { 0xc10ad792, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x830ad795, SUC_lexicon_tags_31 },
    { 0x9a2ad796, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x894ad798, SUC_lexicon_tags_85 },
    { 0xd00ad799, SUC_lexicon_tags_25 },
    { 0x692ad799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8ad79d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfccad79f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2ad7a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ead7ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e8ad7ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xff2ad7b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x93ead7b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faad7b8, SUC_lexicon_tags_11 },
    { 0xfa8ad7b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516ad7bf, SUC_lexicon_tags_12 },
    { 0x194ad7c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8ad7c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x464ad7c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb48ad7c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8dead7cb, SUC_lexicon_tags_25 },
    { 0x1b4ad7cc, SUC_lexicon_tags_33 },
    { 0xdc0ad7cb, SUC_lexicon_tags_131 },
    { 0xd88ad7ce, SUC_lexicon_tags_1 },
    { 0x9f0ad7cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aead7d2, SUC_lexicon_tags_63 },
    { 0xc78ad7d3, SUC_lexicon_tags_39 },
    { 0x134ad7d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12ad7d8, SUC_lexicon_tags_162 },
    { 0xe92ad7d9, SUC_lexicon_tags_37 },
    { 0x2a0ad7da, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd0aad7dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50ad7e3, SUC_lexicon_tags_35 },
    { 0x60ead7e4, SUC_lexicon_tags_15 },
    { 0xf88ad7e5, SUC_lexicon_tags_12 },
    { 0xa0ad7e6, SUC_lexicon_tags_14 },
    { 0xd6ad7e5, SUC_lexicon_tags_47 },
    { 0x382ad7e5, SUC_lexicon_tags_15 },
    { 0xbf6ad7e9, SUC_lexicon_tags_13 },
    { 0x3a2ad7e6, SUC_lexicon_tags_13 },
    { 0x8b0ad7e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198ad7f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daad7f5, SUC_lexicon_tags_101 },
    { 0xa8ad7f5, SUC_lexicon_tags_38 },
    { 0x686ad7f7, SUC_lexicon_tags_43 },
    { 0xdf8ad7f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bcad7fa, SUC_lexicon_tags_18 },
    { 0xd94ad7fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x974ad7fe, SUC_lexicon_tags_7 },
    { 0x4f6ad7fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66ad802, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaead806, SUC_lexicon_tags_37 },
    { 0x5dcad807, SUC_lexicon_tags_4 },
    { 0x4fead808, SUC_lexicon_tags_30 },
    { 0x2aead809, SUC_lexicon_tags_60 },
    { 0x614ad807, SUC_lexicon_tags_62 },
    { 0xf6aad808, SUC_lexicon_tags_2 },
    { 0x7f4ad80a, SUC_lexicon_tags_37 },
    { 0x6bead80a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aead814, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaad817, SUC_lexicon_tags_25 },
    { 0xa94ad818, SUC_lexicon_tags_37 },
    { 0x420ad819, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x586ad81b, SUC_lexicon_tags_8 },
    { 0xd22ad81c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x472ad81e, SUC_lexicon_tags_14 },
    { 0x37ead81f, SUC_lexicon_tags_38 },
    { 0xe7ead820, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3eaad822, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c8ad824, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x93cad827, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4ad82f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aad835, SUC_lexicon_tags_14 },
    { 0x824ad836, SUC_lexicon_tags_60 },
    { 0x7caad835, SUC_lexicon_tags_37 },
    { 0x3eaad835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600ad840, SUC_lexicon_tags_26 },
    { 0x39aad840, SUC_lexicon_tags_25 },
    { 0xd42ad842, SUC_lexicon_tags_25 },
    { 0xb0ad842, SUC_lexicon_tags_25 },
    { 0xe0ad844, SUC_lexicon_tags_87 },
    { 0x270ad845, SUC_lexicon_tags_12 },
    { 0x3a0ad846, SUC_lexicon_tags_37 },
    { 0xdfead844, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0ad84c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ead855, SUC_lexicon_tags_48 },
    { 0x5bead856, SUC_lexicon_tags_21 },
    { 0x142ad856, SUC_lexicon_tags_14 },
    { 0x58ead858, SUC_lexicon_tags_2 },
    { 0x5fcad859, SUC_lexicon_tags_15 },
    { 0x81cad85a, SUC_lexicon_tags_11 },
    { 0xd3cad85b, SUC_lexicon_tags_29 },
    { 0xb0ead85c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb04ad85e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd02ad860, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a8ad862, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daad866, SUC_lexicon_tags_17 },
    { 0xed4ad866, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf6ad869, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ead86c, SUC_lexicon_tags_66 },
    { 0x186ad86c, SUC_lexicon_tags_17 },
    { 0xccead86e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4ad872, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ad875, SUC_lexicon_tags_17 },
    { 0x124ad876, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9caad878, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb02ad87a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ad87f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45aad886, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878ad88c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cad891, SUC_lexicon_tags_47 },
    { 0xf3ead892, SUC_lexicon_tags_2 },
    { 0x43cad893, SUC_lexicon_tags_60 },
    { 0x6aaad894, SUC_lexicon_tags_1 },
    { 0xb8aad895, SUC_lexicon_tags_56 },
    { 0x826ad896, SUC_lexicon_tags_12 },
    { 0xe88ad896, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cad89e, SUC_lexicon_tags_17 },
    { 0xdacad89e, SUC_lexicon_tags_60 },
    { 0x290ad89e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936ad8a5, SUC_lexicon_tags_53 },
    { 0x2a4ad8a6, SUC_lexicon_tags_8 },
    { 0x142ad8a7, SUC_lexicon_tags_37 },
    { 0xb3ead8a7, SUC_lexicon_tags_16 },
    { 0x240ad8a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa9ead8ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0ad8b0, SUC_lexicon_tags_28 },
    { 0xa66ad8b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cad8b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccad8b9, SUC_lexicon_tags_35 },
    { 0x38cad8ba, SUC_lexicon_tags_66 },
    { 0x722ad8bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d0ad8bd, SUC_lexicon_tags_12 },
    { 0x91aad8bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2ad8c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf98ad8c3, SUC_lexicon_tags_9 },
    { 0x110ad8c3, SUC_lexicon_tags_29 },
    { 0xd16ad8c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06ad8c8, SUC_lexicon_tags_15 },
    { 0x9f6ad8c8, SUC_lexicon_tags_17 },
    { 0xbcaad8c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b4ad8cc, SUC_lexicon_tags_63 },
    { 0x1f8ad8cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf92ad8cf, SUC_lexicon_tags_1 },
    { 0xd32ad8d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eead8d5, SUC_lexicon_tags_15 },
    { 0x64cad8d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cad8da, SUC_lexicon_tags_46 },
    { 0x1eaad8db, SUC_lexicon_tags_23 },
    { 0x27aad8da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b4ad8de, SUC_lexicon_tags_2 },
    { 0x7c2ad8df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe98ad8e1, SUC_lexicon_tags_12 },
    { 0x6cead8e2, SUC_lexicon_tags_15 },
    { 0x570ad8e3, SUC_lexicon_tags_25 },
    { 0x644ad8e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee8ad8e6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaad8eb, SUC_lexicon_tags_37 },
    { 0xde4ad8ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccead8ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6ad8f3, SUC_lexicon_tags_1 },
    { 0xa82ad8f4, SUC_lexicon_tags_37 },
    { 0xb5aad8f4, SUC_lexicon_tags_7 },
    { 0xafaad8f6, SUC_lexicon_tags_11 },
    { 0x6a0ad8f7, SUC_lexicon_tags_12 },
    { 0x4caad8f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabead8fd, SUC_lexicon_tags_9 },
    { 0x6fcad8fe, SUC_lexicon_tags_7 },
    { 0x720ad8ff, SUC_lexicon_tags_23 },
    { 0x1eead8fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aad904, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8ad909, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1aad90c, SUC_lexicon_tags_11 },
    { 0x388ad90d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234ad912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2ad916, SUC_lexicon_tags_1 },
    { 0xde2ad916, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8cad919, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6ad91d, SUC_lexicon_tags_1 },
    { 0xbc4ad91e, SUC_lexicon_tags_11 },
    { 0xa98ad91e, SUC_lexicon_tags_0 },
    { 0xd2ad91f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aad928, SUC_lexicon_tags_8 },
    { 0x344ad928, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524ad935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ad93a, SUC_lexicon_tags_31 },
    { 0x31ead93b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8ad941, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x278ad944, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8ad94d, SUC_lexicon_tags_37 },
    { 0x5e8ad94e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd70ad950, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa58ad952, SUC_lexicon_tags_7 },
    { 0xf9cad953, SUC_lexicon_tags_27 },
    { 0xe64ad954, SUC_lexicon_tags_1 },
    { 0x936ad955, SUC_lexicon_tags_23 },
    { 0xa92ad955, SUC_lexicon_tags_15 },
    { 0x8b6ad956, SUC_lexicon_tags_43 },
    { 0x62ad952, SUC_lexicon_tags_0 },
    { 0xe80ad953, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8cad95b, SUC_lexicon_tags_15 },
    { 0x40ead95b, SUC_lexicon_tags_66 },
    { 0x618ad95d, SUC_lexicon_tags_45 },
    { 0xc02ad95b, SUC_lexicon_tags_15 },
    { 0x7f4ad95c, SUC_lexicon_tags_11 },
    { 0x484ad960, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x228ad963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4baad966, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4ad96d, SUC_lexicon_tags_11 },
    { 0xf42ad96d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5aad970, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a4ad972, SUC_lexicon_tags_7 },
    { 0xa44ad973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x790ad975, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aad97a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66ead97c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78ad984, SUC_lexicon_tags_27 },
    { 0xfd8ad985, SUC_lexicon_tags_25 },
    { 0xe5ead984, SUC_lexicon_tags_773 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748ad98a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14ad98d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476ad993, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aad997, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ead99a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x374ad99c, SUC_lexicon_tags_3 },
    { 0x62aad99d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x954ad99f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ad9a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faad9ab, SUC_lexicon_tags_19 },
    { 0xf0cad9ab, SUC_lexicon_tags_1 },
    { 0xf08ad9ad, SUC_lexicon_tags_12 },
    { 0x5b6ad9ad, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x96cad9b0, SUC_lexicon_tags_21 },
    { 0x5bcad9b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8d8ad9b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ead9b7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf80ad9b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf2ad9bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc40ad9bd, SUC_lexicon_tags_25 },
    { 0x988ad9bd, SUC_lexicon_tags_35 },
    { 0xf68ad9be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1faad9c1, SUC_lexicon_tags_25 },
    { 0xfd2ad9c1, SUC_lexicon_tags_25 },
    { 0xd0cad9c3, SUC_lexicon_tags_37 },
    { 0x2c8ad9c4, SUC_lexicon_tags_0 },
    { 0xf9aad9c5, SUC_lexicon_tags_17 },
    { 0xc0aad9c5, SUC_lexicon_tags_41 },
    { 0x36cad9c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206ad9cc, SUC_lexicon_tags_12 },
    { 0x48aad9cd, SUC_lexicon_tags_27 },
    { 0x384ad9ce, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xd5cad9d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x350ad9d2, SUC_lexicon_tags_17 },
    { 0x1f0ad9d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720ad9d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ead9da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9aad9dd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccad9e3, SUC_lexicon_tags_11 },
    { 0x6ccad9e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89aad9e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cad9eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48ad9ef, SUC_lexicon_tags_32 },
    { 0xe7cad9f0, SUC_lexicon_tags_38 },
    { 0x19cad9f0, SUC_lexicon_tags_17 },
    { 0x264ad9f1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf02ad9f4, SUC_lexicon_tags_15 },
    { 0x90ead9f4, SUC_lexicon_tags_30 },
    { 0x476ad9f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aad9fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85aada01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcada0b, SUC_lexicon_tags_27 },
    { 0x1f0ada0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x434ada0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x248ada10, SUC_lexicon_tags_65 },
    { 0x7e0ada10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36ada19, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x210ada1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8ada21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7eada2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20ada34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8ada38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c0ada3a, SUC_lexicon_tags_37 },
    { 0x252ada3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646ada40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712ada44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698ada48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eeada4b, SUC_lexicon_tags_0 },
    { 0xa56ada4b, SUC_lexicon_tags_7 },
    { 0xb80ada4d, SUC_lexicon_tags_11 },
    { 0xbe0ada4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5aada50, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccada57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6ada5a, SUC_lexicon_tags_43 },
    { 0xd88ada5b, SUC_lexicon_tags_60 },
    { 0x978ada5a, SUC_lexicon_tags_19 },
    { 0xbb2ada5d, SUC_lexicon_tags_0 },
    { 0xc20ada5e, SUC_lexicon_tags_32 },
    { 0xe76ada5f, SUC_lexicon_tags_37 },
    { 0xbc6ada5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fcada62, SUC_lexicon_tags_9 },
    { 0x2d6ada63, SUC_lexicon_tags_0 },
    { 0x46aada64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2ada6c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe7cada6e, SUC_lexicon_tags_25 },
    { 0x898ada6f, SUC_lexicon_tags_16 },
    { 0xaf0ada70, SUC_lexicon_tags_77 },
    { 0x326ada71, SUC_lexicon_tags_31 },
    { 0x93cada70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8ada76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6ada7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63eada7e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94aada82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aada85, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2ada88, SUC_lexicon_tags_12 },
    { 0x99cada89, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xacada8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8ada95, SUC_lexicon_tags_28 },
    { 0x2d4ada96, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92ada9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb56ada9c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5faada9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89aadaa0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x91cadaa2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0adaa6, SUC_lexicon_tags_26 },
    { 0x8caadaa6, SUC_lexicon_tags_32 },
    { 0xc70adaa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cadaab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd56adaad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cadab1, SUC_lexicon_tags_29 },
    { 0x9aaadab2, SUC_lexicon_tags_49 },
    { 0xae0adab3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392adab7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcadabb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x562adabd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa70adabf, SUC_lexicon_tags_49 },
    { 0xf24adac0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32adac5, SUC_lexicon_tags_13 },
    { 0x94aadac5, SUC_lexicon_tags_84 },
    { 0x61eadac7, SUC_lexicon_tags_15 },
    { 0x5baadac8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x20adaca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4baadacc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286adad2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x96cadad4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc52adad6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cadad9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24eadadf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cadae4, SUC_lexicon_tags_74 },
    { 0x524adae5, SUC_lexicon_tags_35 },
    { 0x744adae6, SUC_lexicon_tags_25 },
    { 0x6aeadae6, SUC_lexicon_tags_4 },
    { 0x8f2adae8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf9cadaea, SUC_lexicon_tags_7 },
    { 0xb28adaeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96aadaf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16adaf8, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x28cadafa, SUC_lexicon_tags_92 },
    { 0xd64adafa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274adb00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4deadb03, SUC_lexicon_tags_54 },
    { 0xebaadb03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecadb08, SUC_lexicon_tags_60 },
    { 0xbd4adb08, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0adb0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x252adb0f, SUC_lexicon_tags_43 },
    { 0x8c8adb10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x194adb13, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3eadb15, SUC_lexicon_tags_104 },
    { 0xe3aadb16, SUC_lexicon_tags_1 },
    { 0x56aadb17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6adb1b, SUC_lexicon_tags_15 },
    { 0xcb4adb1c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaadb1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcadb22, SUC_lexicon_tags_2 },
    { 0x3c2adb23, SUC_lexicon_tags_25 },
    { 0x52cadb24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8a0adb26, SUC_lexicon_tags_8 },
    { 0xd98adb26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06adb31, SUC_lexicon_tags_7 },
    { 0x808adb31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8e4adb34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x576adb36, SUC_lexicon_tags_11 },
    { 0x9cadb37, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926adb3c, SUC_lexicon_tags_62 },
    { 0x66adb3c, SUC_lexicon_tags_7 },
    { 0x220adb3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x936adb40, SUC_lexicon_tags_43 },
    { 0x300adb41, SUC_lexicon_tags_2 },
    { 0xda4adb42, SUC_lexicon_tags_12 },
    { 0x656adb42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20adb45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x30adb48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x354adb4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16adb50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x926adb52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984adb56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb5b, SUC_lexicon_tags_2 },
    { 0x5c6adb5c, SUC_lexicon_tags_9 },
    { 0x3a2adb5b, SUC_lexicon_tags_28 },
    { 0xe26adb5e, SUC_lexicon_tags_47 },
    { 0x796adb5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf84adb61, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb4cadb63, SUC_lexicon_tags_12 },
    { 0xaacadb64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdb4adb66, SUC_lexicon_tags_15 },
    { 0xe0aadb66, SUC_lexicon_tags_43 },
    { 0x7eaadb68, SUC_lexicon_tags_37 },
    { 0x4c8adb69, SUC_lexicon_tags_17 },
    { 0x39eadb6a, SUC_lexicon_tags_25 },
    { 0xe56adb6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14eadb6d, SUC_lexicon_tags_1 },
    { 0x9eeadb6e, SUC_lexicon_tags_16 },
    { 0x27eadb6f, SUC_lexicon_tags_26 },
    { 0x45aadb6f, SUC_lexicon_tags_17 },
    { 0x1b6adb71, SUC_lexicon_tags_14 },
    { 0xab0adb72, SUC_lexicon_tags_65 },
    { 0x118adb73, SUC_lexicon_tags_25 },
    { 0x3c2adb6e, SUC_lexicon_tags_37 },
    { 0x238adb73, SUC_lexicon_tags_17 },
    { 0x7e4adb76, SUC_lexicon_tags_7 },
    { 0xa0adb76, SUC_lexicon_tags_32 },
    { 0x77cadb78, SUC_lexicon_tags_16 },
    { 0x1b8adb79, SUC_lexicon_tags_9 },
    { 0xc94adb79, SUC_lexicon_tags_49 },
    { 0x6e8adb7b, SUC_lexicon_tags_25 },
    { 0x576adb7a, SUC_lexicon_tags_15 },
    { 0x67aadb72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6dcadb7f, SUC_lexicon_tags_38 },
    { 0x8c4adb7f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x97eadb82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670adb87, SUC_lexicon_tags_12 },
    { 0x7e4adb88, SUC_lexicon_tags_343 },
    { 0xd3eadb88, SUC_lexicon_tags_0 },
    { 0x9daadb8a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaadb8d, SUC_lexicon_tags_61 },
    { 0xfbeadb8e, SUC_lexicon_tags_1 },
    { 0xeb0adb8d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x66adb92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb4adb94, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44adb98, SUC_lexicon_tags_17 },
    { 0x92cadb99, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x16adb9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0adba1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5ceadba3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59eadba8, SUC_lexicon_tags_45 },
    { 0x676adba9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6adbad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00adbb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6adbb4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54adbb8, SUC_lexicon_tags_37 },
    { 0xa80adbb9, SUC_lexicon_tags_8 },
    { 0xcd6adbba, SUC_lexicon_tags_16 },
    { 0x9a8adbbb, SUC_lexicon_tags_25 },
    { 0xa68adbbc, SUC_lexicon_tags_33 },
    { 0xfc4adbba, SUC_lexicon_tags_21 },
    { 0x7f2adbbe, SUC_lexicon_tags_27 },
    { 0x49eadbbb, SUC_lexicon_tags_113 },
    { 0x69eadbc0, SUC_lexicon_tags_15 },
    { 0xe90adbbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57eadbc4, SUC_lexicon_tags_2 },
    { 0xd44adbc5, SUC_lexicon_tags_37 },
    { 0x282adbc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x668adbc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8adbcd, SUC_lexicon_tags_11 },
    { 0x82aadbcd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbf2adbd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x490adbd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0adbd8, SUC_lexicon_tags_11 },
    { 0x6eadbd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7beadbdc, SUC_lexicon_tags_8 },
    { 0xe40adbdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x500adbdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x87aadbe1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x236adbe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9beadbe7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748adbeb, SUC_lexicon_tags_17 },
    { 0x26aadbec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84adbf0, SUC_lexicon_tags_17 },
    { 0xbdeadbf0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6eadbf4, SUC_lexicon_tags_68 },
    { 0x7fcadbf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadbfb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6adbff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cadc05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccadc09, SUC_lexicon_tags_23 },
    { 0xf60adc0a, SUC_lexicon_tags_12 },
    { 0x8eeadc0a, SUC_lexicon_tags_2 },
    { 0x14adc0c, SUC_lexicon_tags_14 },
    { 0x77cadc0d, SUC_lexicon_tags_19 },
    { 0xd58adc0e, SUC_lexicon_tags_60 },
    { 0x490adc0e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a2adc11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6adc14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x938adc16, SUC_lexicon_tags_17 },
    { 0x122adc17, SUC_lexicon_tags_29 },
    { 0xddaadc18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ecadc1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcadc1d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90adc20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02adc25, SUC_lexicon_tags_255 },
    { 0xd18adc26, SUC_lexicon_tags_92 },
    { 0x95eadc27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b6adc29, SUC_lexicon_tags_14 },
    { 0xbd0adc2a, SUC_lexicon_tags_13 },
    { 0x75aadc2b, SUC_lexicon_tags_82 },
    { 0xe60adc2a, SUC_lexicon_tags_2 },
    { 0x910adc2a, SUC_lexicon_tags_2 },
    { 0x8a4adc2a, SUC_lexicon_tags_15 },
    { 0x5a0adc2f, SUC_lexicon_tags_37 },
    { 0x752adc30, SUC_lexicon_tags_63 },
    { 0x60adc31, SUC_lexicon_tags_45 },
    { 0xe0aadc31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6adc38, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x242adc3a, SUC_lexicon_tags_29 },
    { 0xb62adc3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5aadc3f, SUC_lexicon_tags_14 },
    { 0x780adc40, SUC_lexicon_tags_11 },
    { 0x12aadc41, SUC_lexicon_tags_27 },
    { 0x33aadc41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48adc4a, SUC_lexicon_tags_34 },
    { 0x5eadc4b, SUC_lexicon_tags_33 },
    { 0x1d2adc4c, SUC_lexicon_tags_38 },
    { 0xfcaadc4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78adc50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04adc60, SUC_lexicon_tags_55 },
    { 0xb5eadc60, SUC_lexicon_tags_11 },
    { 0xfc0adc62, SUC_lexicon_tags_127 },
    { 0x512adc62, SUC_lexicon_tags_31 },
    { 0xe54adc63, SUC_lexicon_tags_29 },
    { 0xfbeadc65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0adc69, SUC_lexicon_tags_7 },
    { 0x8deadc6a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x74cadc6d, SUC_lexicon_tags_11 },
    { 0xac4adc6e, SUC_lexicon_tags_25 },
    { 0xb46adc6e, SUC_lexicon_tags_25 },
    { 0xafeadc6f, SUC_lexicon_tags_17 },
    { 0x1aaadc70, SUC_lexicon_tags_61 },
    { 0x2dcadc6d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35aadc79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974adc7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14adc80, SUC_lexicon_tags_37 },
    { 0xfeeadc81, SUC_lexicon_tags_48 },
    { 0xc10adc81, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x20cadc84, SUC_lexicon_tags_17 },
    { 0x346adc84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8cadc87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8adc8c, SUC_lexicon_tags_25 },
    { 0x5c0adc8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ecadc8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2adc91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aadc94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x89aadc96, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aadc99, SUC_lexicon_tags_25 },
    { 0xb38adc9a, SUC_lexicon_tags_15 },
    { 0x384adc9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24eadc9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeadc9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6adca2, SUC_lexicon_tags_25 },
    { 0x58adca2, SUC_lexicon_tags_9 },
    { 0xb4aadca4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10adca7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x49aadcaa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaadcae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb44adcb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa88adcb2, SUC_lexicon_tags_34 },
    { 0xae4adcb3, SUC_lexicon_tags_14 },
    { 0x2daadcb4, SUC_lexicon_tags_33 },
    { 0x622adcb5, SUC_lexicon_tags_63 },
    { 0x4feadcb5, SUC_lexicon_tags_10 },
    { 0xeaadcb7, SUC_lexicon_tags_9 },
    { 0x112adcb7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x466adcba, SUC_lexicon_tags_8 },
    { 0x3d6adcba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x508adcbe, SUC_lexicon_tags_68 },
    { 0xddaadcbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0adcc2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x8aeadcc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4adcca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad2adccc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720adcd1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184adce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24aadce4, SUC_lexicon_tags_19 },
    { 0x1f4adce4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aadceb, SUC_lexicon_tags_17 },
    { 0x414adcec, SUC_lexicon_tags_37 },
    { 0xe16adced, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x622adcef, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xea2adcf1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96adcf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cadcfb, SUC_lexicon_tags_23 },
    { 0x26adcfc, SUC_lexicon_tags_30 },
    { 0x51aadcfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe78adcff, SUC_lexicon_tags_39 },
    { 0xf1eadd00, SUC_lexicon_tags_23 },
    { 0xddaadd01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb16add03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7f8add05, SUC_lexicon_tags_84 },
    { 0xb46add05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99eadd09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x652add0b, SUC_lexicon_tags_7 },
    { 0x462add0b, SUC_lexicon_tags_7 },
    { 0xf92add0c, SUC_lexicon_tags_47 },
    { 0x498add0e, SUC_lexicon_tags_2 },
    { 0x3f8add0f, SUC_lexicon_tags_27 },
    { 0x3beadd0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb72add12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2add16, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94add22, SUC_lexicon_tags_66 },
    { 0xc70add22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896add27, SUC_lexicon_tags_38 },
    { 0x6aaadd28, SUC_lexicon_tags_25 },
    { 0xf18add29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a8add2b, SUC_lexicon_tags_17 },
    { 0x1b6add2c, SUC_lexicon_tags_14 },
    { 0x912add2d, SUC_lexicon_tags_15 },
    { 0x888add2e, SUC_lexicon_tags_85 },
    { 0x544add2e, SUC_lexicon_tags_38 },
    { 0x58cadd30, SUC_lexicon_tags_17 },
    { 0x3a8add31, SUC_lexicon_tags_49 },
    { 0x57cadd31, SUC_lexicon_tags_28 },
    { 0x2e6add32, SUC_lexicon_tags_0 },
    { 0xdc4add34, SUC_lexicon_tags_39 },
    { 0xb1aadd35, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34add39, SUC_lexicon_tags_37 },
    { 0xcd8add39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522add41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bcadd43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6add46, SUC_lexicon_tags_12 },
    { 0x664add46, SUC_lexicon_tags_28 },
    { 0xba0add47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x88aadd4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85eadd4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x640add50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x522add53, SUC_lexicon_tags_84 },
    { 0x572add54, SUC_lexicon_tags_15 },
    { 0x724add53, SUC_lexicon_tags_62 },
    { 0x230add56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x614add58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44aadd5a, SUC_lexicon_tags_11 },
    { 0xc04add5b, SUC_lexicon_tags_28 },
    { 0xf02add5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cadd5f, SUC_lexicon_tags_65 },
    { 0xbf8add5f, SUC_lexicon_tags_91 },
    { 0xce8add61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeadd67, SUC_lexicon_tags_8 },
    { 0xb38add68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4add6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x65eadd70, SUC_lexicon_tags_30 },
    { 0x88add71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51eadd75, SUC_lexicon_tags_21 },
    { 0xb24add76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x620add79, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcadd85, SUC_lexicon_tags_56 },
    { 0x57eadd85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f0add88, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cadd97, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x686add99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef0add9b, SUC_lexicon_tags_1 },
    { 0x1e6add9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e0add9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe0aadda0, SUC_lexicon_tags_47 },
    { 0xdbaadda0, SUC_lexicon_tags_37 },
    { 0xd2eadda2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b0adda4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f0adda6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x5c2adda8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4addac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a6addae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8addb1, SUC_lexicon_tags_33 },
    { 0xf98addb2, SUC_lexicon_tags_47 },
    { 0xbc0addb3, SUC_lexicon_tags_15 },
    { 0x128addb2, SUC_lexicon_tags_12 },
    { 0x7feaddb5, SUC_lexicon_tags_58 },
    { 0x9aaddb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x392addb8, SUC_lexicon_tags_15 },
    { 0x30caddb8, SUC_lexicon_tags_7 },
    { 0xb08addba, SUC_lexicon_tags_11 },
    { 0xddcaddba, SUC_lexicon_tags_6 },
    { 0x9baaddbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956addc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x184addc2, SUC_lexicon_tags_11 },
    { 0x86eaddc3, SUC_lexicon_tags_87 },
    { 0x544addc3, SUC_lexicon_tags_87 },
    { 0xeeeaddc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeaddca, SUC_lexicon_tags_17 },
    { 0x2eaaddcb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00addce, SUC_lexicon_tags_10 },
    { 0xb22addce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaaddd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8caaddda, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb6eadddc, SUC_lexicon_tags_11 },
    { 0xf88adddc, SUC_lexicon_tags_61 },
    { 0x4a8addde, SUC_lexicon_tags_26 },
    { 0x476adddf, SUC_lexicon_tags_46 },
    { 0xa72addde, SUC_lexicon_tags_25 },
    { 0x6feadde1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaadde5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x190adde8, SUC_lexicon_tags_26 },
    { 0x7baadde9, SUC_lexicon_tags_15 },
    { 0xe82addea, SUC_lexicon_tags_1 },
    { 0xa92adde9, SUC_lexicon_tags_37 },
    { 0x70caddec, SUC_lexicon_tags_33 },
    { 0x11eaddeb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x25aaddef, SUC_lexicon_tags_23 },
    { 0xdb4addf0, SUC_lexicon_tags_0 },
    { 0x132addef, SUC_lexicon_tags_21 },
    { 0xa6caddf0, SUC_lexicon_tags_22 },
    { 0x848addef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2addf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aaddfe, SUC_lexicon_tags_25 },
    { 0x450addff, SUC_lexicon_tags_2 },
    { 0x42eaddff, SUC_lexicon_tags_11 },
    { 0x53eade01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5a8ade03, SUC_lexicon_tags_56 },
    { 0xd16ade04, SUC_lexicon_tags_11 },
    { 0x2aeade05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68ade08, SUC_lexicon_tags_5 },
    { 0xd50ade08, SUC_lexicon_tags_7 },
    { 0x296ade0a, SUC_lexicon_tags_37 },
    { 0x8ceade0b, SUC_lexicon_tags_104 },
    { 0x492ade08, SUC_lexicon_tags_37 },
    { 0x8f0ade0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1caade10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa52ade12, SUC_lexicon_tags_10 },
    { 0x9e6ade12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cade1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4ade24, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8ade2c, SUC_lexicon_tags_25 },
    { 0x9aade2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ccade2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf0ade31, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9acade33, SUC_lexicon_tags_48 },
    { 0xf06ade34, SUC_lexicon_tags_37 },
    { 0xd84ade34, SUC_lexicon_tags_144 },
    { 0xe38ade36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf60ade38, SUC_lexicon_tags_60 },
    { 0x4c8ade38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x19eade3b, SUC_lexicon_tags_26 },
    { 0xcb4ade3b, SUC_lexicon_tags_43 },
    { 0xa70ade3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5fcade3f, SUC_lexicon_tags_65 },
    { 0xecaade40, SUC_lexicon_tags_0 },
    { 0xc9eade41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x848ade43, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x6beade45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0ade49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8ade4c, SUC_lexicon_tags_25 },
    { 0xc54ade4d, SUC_lexicon_tags_8 },
    { 0x460ade4e, SUC_lexicon_tags_42 },
    { 0x404ade4e, SUC_lexicon_tags_27 },
    { 0x61aade50, SUC_lexicon_tags_10 },
    { 0xf86ade51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1aade53, SUC_lexicon_tags_11 },
    { 0x5f0ade53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652ade5a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626ade5e, SUC_lexicon_tags_0 },
    { 0x9beade5f, SUC_lexicon_tags_14 },
    { 0x33cade5e, SUC_lexicon_tags_27 },
    { 0xe8cade61, SUC_lexicon_tags_30 },
    { 0xf94ade60, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46aade66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8ade6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94ade6f, SUC_lexicon_tags_11 },
    { 0x6eeade70, SUC_lexicon_tags_7 },
    { 0x778ade70, SUC_lexicon_tags_15 },
    { 0x738ade70, SUC_lexicon_tags_37 },
    { 0x5e8ade6f, SUC_lexicon_tags_14 },
    { 0x53cade74, SUC_lexicon_tags_1 },
    { 0xfdaade75, SUC_lexicon_tags_17 },
    { 0x822ade74, SUC_lexicon_tags_38 },
    { 0x92aade71, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640ade7b, SUC_lexicon_tags_9 },
    { 0xcecade7c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6ade80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302ade87, SUC_lexicon_tags_25 },
    { 0xdb8ade87, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x376ade8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b8ade8c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28ade91, SUC_lexicon_tags_49 },
    { 0x8a8ade92, SUC_lexicon_tags_25 },
    { 0x936ade92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa12ade95, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x332ade98, SUC_lexicon_tags_55 },
    { 0xaaeade99, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x130ade9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x600ade9d, SUC_lexicon_tags_38 },
    { 0x9eade9e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cadea1, SUC_lexicon_tags_43 },
    { 0x96adea2, SUC_lexicon_tags_27 },
    { 0x980adea3, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2adea6, SUC_lexicon_tags_38 },
    { 0xabcadea6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x328adea9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcadeae, SUC_lexicon_tags_1 },
    { 0x38adeaf, SUC_lexicon_tags_8 },
    { 0xc80adeb0, SUC_lexicon_tags_37 },
    { 0x6f4adeb0, SUC_lexicon_tags_37 },
    { 0x60cadeb1, SUC_lexicon_tags_37 },
    { 0x8ecadeb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8adeb8, SUC_lexicon_tags_63 },
    { 0x94aadeb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cadebe, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eadec6, SUC_lexicon_tags_37 },
    { 0xeb2adec6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0adecb, SUC_lexicon_tags_1 },
    { 0xd20adecb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc6adece, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46aded5, SUC_lexicon_tags_11 },
    { 0x3daaded5, SUC_lexicon_tags_34 },
    { 0x5caaded7, SUC_lexicon_tags_12 },
    { 0x7b2aded7, SUC_lexicon_tags_47 },
    { 0x332aded9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8caadedb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x62eadedd, SUC_lexicon_tags_31 },
    { 0x9cadede, SUC_lexicon_tags_19 },
    { 0x462adedf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd04adee1, SUC_lexicon_tags_25 },
    { 0x4b0adee1, SUC_lexicon_tags_17 },
    { 0x2f6adee3, SUC_lexicon_tags_11 },
    { 0x328adee4, SUC_lexicon_tags_12 },
    { 0x3f0adee5, SUC_lexicon_tags_44 },
    { 0x72aadee6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13eadee8, SUC_lexicon_tags_68 },
    { 0x802adee8, SUC_lexicon_tags_1 },
    { 0x724adeea, SUC_lexicon_tags_65 },
    { 0x3aadeea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4adef1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736adef5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b6adef7, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0xf6aadef9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37aadefe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58adf02, SUC_lexicon_tags_1 },
    { 0x8a4adf02, SUC_lexicon_tags_49 },
    { 0x990adf03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87cadf06, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2adf0b, SUC_lexicon_tags_23 },
    { 0x14aadf0b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeadf12, SUC_lexicon_tags_55 },
    { 0x10cadf13, SUC_lexicon_tags_69 },
    { 0x8faadf14, SUC_lexicon_tags_0 },
    { 0xa1cadf12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaadf19, SUC_lexicon_tags_10 },
    { 0x2eaadf1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe6aadf1c, SUC_lexicon_tags_21 },
    { 0x5d4adf1c, SUC_lexicon_tags_2 },
    { 0x55cadf1c, SUC_lexicon_tags_10 },
    { 0xe3aadf1f, SUC_lexicon_tags_7 },
    { 0x9daadf1f, SUC_lexicon_tags_14 },
    { 0x1f0adf1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6adf25, SUC_lexicon_tags_60 },
    { 0xddeadf25, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcadf2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14adf30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x760adf33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178adf39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdaeadf3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x906adf3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7eadf44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x598adf46, SUC_lexicon_tags_2 },
    { 0x972adf46, SUC_lexicon_tags_12 },
    { 0xc92adf48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4adf4e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4cadf50, SUC_lexicon_tags_12 },
    { 0xe5eadf50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0adf5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d8adf5e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x952adf61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868adf66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1beadf6b, SUC_lexicon_tags_82 },
    { 0x604adf6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x746adf6e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x72aadf71, SUC_lexicon_tags_11 },
    { 0xceaadf72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27cadf75, SUC_lexicon_tags_28 },
    { 0x6c8adf75, SUC_lexicon_tags_25 },
    { 0xf4aadf76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacadf7c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf3aadf7e, SUC_lexicon_tags_51 },
    { 0x3b0adf7f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cadf89, SUC_lexicon_tags_37 },
    { 0xda0adf8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x710adf8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e6adf8e, SUC_lexicon_tags_2 },
    { 0x3b2adf8f, SUC_lexicon_tags_11 },
    { 0x224adf90, SUC_lexicon_tags_17 },
    { 0xd24adf91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cadf94, SUC_lexicon_tags_141 },
    { 0xc98adf95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcadf9a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96adfa0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2adfa4, SUC_lexicon_tags_2 },
    { 0x590adfa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8adfa9, SUC_lexicon_tags_27 },
    { 0x73cadfaa, SUC_lexicon_tags_87 },
    { 0xfd0adfa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0adfb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44aadfb6, SUC_lexicon_tags_56 },
    { 0xac8adfb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2adfbb, SUC_lexicon_tags_37 },
    { 0x35eadfbc, SUC_lexicon_tags_16 },
    { 0x380adfbd, SUC_lexicon_tags_27 },
    { 0x4f8adfbe, SUC_lexicon_tags_12 },
    { 0x294adfbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4adfc3, SUC_lexicon_tags_11 },
    { 0x304adfc4, SUC_lexicon_tags_57 },
    { 0x3c4adfc4, SUC_lexicon_tags_160 },
    { 0xf12adfc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f4adfc8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8adfcb, SUC_lexicon_tags_23 },
    { 0x790adfcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2adfd8, SUC_lexicon_tags_25 },
    { 0xffcadfd9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664adfdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0eadfea, SUC_lexicon_tags_42 },
    { 0x9e2adfeb, SUC_lexicon_tags_1 },
    { 0x586adfec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd2eadfee, SUC_lexicon_tags_15 },
    { 0x21eadfee, SUC_lexicon_tags_17 },
    { 0x70eadff0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96cadff2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256adff6, SUC_lexicon_tags_9 },
    { 0xf70adff7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e8adff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84adffe, SUC_lexicon_tags_47 },
    { 0x208adfff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06ae002, SUC_lexicon_tags_3 },
    { 0xb74ae003, SUC_lexicon_tags_49 },
    { 0x1ccae004, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32eae00a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcae00f, SUC_lexicon_tags_95 },
    { 0x430ae00f, SUC_lexicon_tags_87 },
    { 0xe9eae011, SUC_lexicon_tags_11 },
    { 0x13aae011, SUC_lexicon_tags_47 },
    { 0x542ae013, SUC_lexicon_tags_4 },
    { 0xc5aae014, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x67aae016, SUC_lexicon_tags_34 },
    { 0xe1aae017, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd8cae019, SUC_lexicon_tags_55 },
    { 0x70aae019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaae01d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7eae01f, SUC_lexicon_tags_18 },
    { 0xd56ae01f, SUC_lexicon_tags_17 },
    { 0x760ae021, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30ae024, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08ae028, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x59aae02a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd66ae02c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c6ae02e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9d4ae030, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342ae036, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45aae038, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ae03e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x108ae041, SUC_lexicon_tags_37 },
    { 0x882ae041, SUC_lexicon_tags_4 },
    { 0xa00ae043, SUC_lexicon_tags_17 },
    { 0xab0ae044, SUC_lexicon_tags_37 },
    { 0xd38ae043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaae04a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8bcae04c, SUC_lexicon_tags_50 },
    { 0x6cae04c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aae051, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaeae059, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ae05c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec8ae05e, SUC_lexicon_tags_37 },
    { 0x6d8ae05e, SUC_lexicon_tags_17 },
    { 0x7f0ae05f, SUC_lexicon_tags_7 },
    { 0xd4eae05f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18ae064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ae068, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0ae06c, SUC_lexicon_tags_32 },
    { 0x1e8ae06d, SUC_lexicon_tags_38 },
    { 0xc3aae06e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10ae071, SUC_lexicon_tags_17 },
    { 0x25aae071, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x780ae074, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f8ae076, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cae07d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9eae081, SUC_lexicon_tags_11 },
    { 0x19aae082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254ae089, SUC_lexicon_tags_15 },
    { 0x184ae089, SUC_lexicon_tags_11 },
    { 0x4eae08b, SUC_lexicon_tags_7 },
    { 0x1f4ae08c, SUC_lexicon_tags_27 },
    { 0xf40ae08d, SUC_lexicon_tags_45 },
    { 0xb8cae08e, SUC_lexicon_tags_1 },
    { 0x94aae08d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70ae093, SUC_lexicon_tags_29 },
    { 0x166ae094, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaae097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ae09a, SUC_lexicon_tags_2 },
    { 0x900ae09a, SUC_lexicon_tags_1 },
    { 0xb90ae09b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81aae09e, SUC_lexicon_tags_31 },
    { 0x372ae09f, SUC_lexicon_tags_46 },
    { 0xf32ae09f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cae0a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae4ae0a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x76eae0ab, SUC_lexicon_tags_37 },
    { 0xaeeae0ac, SUC_lexicon_tags_1 },
    { 0x9ceae0ab, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x812ae0b0, SUC_lexicon_tags_52 },
    { 0xc1cae0b1, SUC_lexicon_tags_25 },
    { 0xe8ae0b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc36ae0b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310ae0b9, SUC_lexicon_tags_17 },
    { 0x7c6ae0ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfeae0be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x99eae0c0, SUC_lexicon_tags_85 },
    { 0x7bcae0c1, SUC_lexicon_tags_11 },
    { 0x39eae0c1, SUC_lexicon_tags_17 },
    { 0x98ae0c3, SUC_lexicon_tags_85 },
    { 0x78aae0c4, SUC_lexicon_tags_16 },
    { 0x920ae0c4, SUC_lexicon_tags_29 },
    { 0x984ae0c5, SUC_lexicon_tags_45 },
    { 0x16aae0c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06ae0ca, SUC_lexicon_tags_0 },
    { 0xedcae0cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedaae0cf, SUC_lexicon_tags_43 },
    { 0xddcae0d0, SUC_lexicon_tags_25 },
    { 0xf90ae0cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68ae0d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4ae0d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3cae0d7, SUC_lexicon_tags_29 },
    { 0xc0cae0d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aae0dc, SUC_lexicon_tags_132 },
    { 0x31cae0dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10eae0e1, SUC_lexicon_tags_11 },
    { 0xe22ae0e2, SUC_lexicon_tags_17 },
    { 0xacae0e2, SUC_lexicon_tags_66 },
    { 0xef4ae0e2, SUC_lexicon_tags_37 },
    { 0x6aaae0e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4ae0e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eae0ef, SUC_lexicon_tags_12 },
    { 0x518ae0f0, SUC_lexicon_tags_61 },
    { 0xdb6ae0f1, SUC_lexicon_tags_8 },
    { 0x30aae0f2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70ae0f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ae0fa, SUC_lexicon_tags_17 },
    { 0x42cae0fb, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4ae103, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ae107, SUC_lexicon_tags_11 },
    { 0x2fcae108, SUC_lexicon_tags_31 },
    { 0x32eae109, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x116ae10b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cae110, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796ae115, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8aae117, SUC_lexicon_tags_37 },
    { 0xc02ae118, SUC_lexicon_tags_45 },
    { 0xe7aae119, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472ae11e, SUC_lexicon_tags_2 },
    { 0x266ae11f, SUC_lexicon_tags_37 },
    { 0x894ae120, SUC_lexicon_tags_1 },
    { 0xd5cae120, SUC_lexicon_tags_47 },
    { 0xe66ae121, SUC_lexicon_tags_25 },
    { 0xef8ae123, SUC_lexicon_tags_1 },
    { 0xbcaae123, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0ae129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0ae12c, SUC_lexicon_tags_8 },
    { 0x23cae12d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13eae12f, SUC_lexicon_tags_37 },
    { 0x5d4ae12f, SUC_lexicon_tags_4 },
    { 0x8a6ae131, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9ceae133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52ae13e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x898ae140, SUC_lexicon_tags_11 },
    { 0x4b0ae141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde6ae143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4ae147, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ae14a, SUC_lexicon_tags_29 },
    { 0x3d4ae14a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842ae152, SUC_lexicon_tags_11 },
    { 0x338ae153, SUC_lexicon_tags_3 },
    { 0x7aae154, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08ae158, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x286ae15b, SUC_lexicon_tags_21 },
    { 0x408ae15c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcae15f, SUC_lexicon_tags_19 },
    { 0x3faae160, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3eaae162, SUC_lexicon_tags_2 },
    { 0x53aae163, SUC_lexicon_tags_3 },
    { 0x2bcae164, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758ae169, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeae16e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222ae174, SUC_lexicon_tags_32 },
    { 0xfcae175, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2b2ae177, SUC_lexicon_tags_2 },
    { 0xecae178, SUC_lexicon_tags_33 },
    { 0x308ae177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2ae17f, SUC_lexicon_tags_2 },
    { 0xeeae180, SUC_lexicon_tags_4 },
    { 0x802ae181, SUC_lexicon_tags_66 },
    { 0xf50ae181, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ae188, SUC_lexicon_tags_15 },
    { 0xc4eae189, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538ae18d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x424ae190, SUC_lexicon_tags_7 },
    { 0xe20ae191, SUC_lexicon_tags_25 },
    { 0xde4ae190, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8ae196, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108ae19d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcf4ae19f, SUC_lexicon_tags_37 },
    { 0x48eae19f, SUC_lexicon_tags_12 },
    { 0xad6ae1a1, SUC_lexicon_tags_27 },
    { 0x198ae19f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7aae1a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eae1a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cae1ad, SUC_lexicon_tags_15 },
    { 0xfeaae1ae, SUC_lexicon_tags_0 },
    { 0x1a2ae1af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c8ae1b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2cae1b3, SUC_lexicon_tags_30 },
    { 0x358ae1b4, SUC_lexicon_tags_15 },
    { 0xdb4ae1b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2ae1ba, SUC_lexicon_tags_47 },
    { 0x9daae1bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91eae1be, SUC_lexicon_tags_37 },
    { 0x246ae1be, SUC_lexicon_tags_25 },
    { 0x784ae1be, SUC_lexicon_tags_37 },
    { 0x786ae1c0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xcc0ae1c3, SUC_lexicon_tags_11 },
    { 0xf16ae1c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa0ae1c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf8ae1c8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccae1cb, SUC_lexicon_tags_53 },
    { 0x884ae1cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630ae1d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166ae1d9, SUC_lexicon_tags_15 },
    { 0xbeaae1da, SUC_lexicon_tags_2 },
    { 0xd7eae1db, SUC_lexicon_tags_2 },
    { 0xd28ae1dc, SUC_lexicon_tags_27 },
    { 0xa98ae1dd, SUC_lexicon_tags_49 },
    { 0x72aae1de, SUC_lexicon_tags_19 },
    { 0x648ae1dd, SUC_lexicon_tags_87 },
    { 0x6eeae1d9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6ae1e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7aae1e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xee2ae1e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422ae1ed, SUC_lexicon_tags_43 },
    { 0x940ae1ee, SUC_lexicon_tags_4 },
    { 0x6f2ae1ed, SUC_lexicon_tags_27 },
    { 0x2a4ae1f0, SUC_lexicon_tags_81 },
    { 0xeecae1f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x794ae1f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cae1f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0ae1fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1aae1fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a6ae200, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbbeae202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x294ae205, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8ae20c, SUC_lexicon_tags_54 },
    { 0xfd4ae20d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf1cae20f, SUC_lexicon_tags_67 },
    { 0xb04ae20f, SUC_lexicon_tags_17 },
    { 0xc94ae211, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534ae216, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcae21c, SUC_lexicon_tags_14 },
    { 0xb2aae21d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcae220, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x714ae223, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686ae227, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe9aae229, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaae22f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb76ae231, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbc4ae233, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0ae237, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8ae23c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8ae241, SUC_lexicon_tags_11 },
    { 0xec6ae241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c2ae244, SUC_lexicon_tags_14 },
    { 0x91cae244, SUC_lexicon_tags_38 },
    { 0xb4aae246, SUC_lexicon_tags_47 },
    { 0x854ae246, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe68ae24a, SUC_lexicon_tags_11 },
    { 0x270ae24b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16ae24f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596ae256, SUC_lexicon_tags_47 },
    { 0xa2aae256, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20ae25a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650ae25f, SUC_lexicon_tags_15 },
    { 0xae2ae260, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x270ae262, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa4ae264, SUC_lexicon_tags_19 },
    { 0x79eae265, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8ae26a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8ae26d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x346ae270, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6ae274, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0ae278, SUC_lexicon_tags_11 },
    { 0xbeeae279, SUC_lexicon_tags_11 },
    { 0xcaaae27a, SUC_lexicon_tags_37 },
    { 0x1e8ae27b, SUC_lexicon_tags_66 },
    { 0x778ae27c, SUC_lexicon_tags_77 },
    { 0x1bcae27a, SUC_lexicon_tags_15 },
    { 0xe40ae27e, SUC_lexicon_tags_0 },
    { 0x34ae27c, SUC_lexicon_tags_12 },
    { 0xd32ae280, SUC_lexicon_tags_17 },
    { 0xf14ae280, SUC_lexicon_tags_84 },
    { 0x848ae278, SUC_lexicon_tags_91 },
    { 0xbcae283, SUC_lexicon_tags_0 },
    { 0x4fcae281, SUC_lexicon_tags_25 },
    { 0x29aae285, SUC_lexicon_tags_37 },
    { 0x964ae278, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x39cae288, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x624ae28b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0ae290, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4ae294, SUC_lexicon_tags_19 },
    { 0x52ae295, SUC_lexicon_tags_45 },
    { 0x2c6ae295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4ae29a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86eae2a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x180ae2a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66ae2a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f0ae2aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168ae2ae, SUC_lexicon_tags_24 },
    { 0x666ae2af, SUC_lexicon_tags_61 },
    { 0x32cae2b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ae2b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc0ae2b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166ae2bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ae2c0, SUC_lexicon_tags_34 },
    { 0x110ae2c1, SUC_lexicon_tags_0 },
    { 0xeb2ae2c2, SUC_lexicon_tags_29 },
    { 0x2d4ae2c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64ae2c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x496ae2c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x600ae2cb, SUC_lexicon_tags_60 },
    { 0xa50ae2cb, SUC_lexicon_tags_63 },
    { 0x930ae2cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10aae2d0, SUC_lexicon_tags_60 },
    { 0x1fcae2d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ecae2d3, SUC_lexicon_tags_75 },
    { 0xb82ae2d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2ae2da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2ae2de, SUC_lexicon_tags_29 },
    { 0x438ae2df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbd2ae2e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cae2e6, SUC_lexicon_tags_11 },
    { 0x4beae2e7, SUC_lexicon_tags_129 },
    { 0xe1eae2e8, SUC_lexicon_tags_21 },
    { 0x62aae2e6, SUC_lexicon_tags_27 },
    { 0x5d2ae2ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf9cae2ec, SUC_lexicon_tags_37 },
    { 0x14aae2ed, SUC_lexicon_tags_7 },
    { 0x692ae2ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28ae2f4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16ae2f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912ae2fe, SUC_lexicon_tags_26 },
    { 0xe14ae2ff, SUC_lexicon_tags_12 },
    { 0x4d6ae300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22ae305, SUC_lexicon_tags_7 },
    { 0xeacae306, SUC_lexicon_tags_16 },
    { 0xc50ae306, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2ae30a, SUC_lexicon_tags_11 },
    { 0x8beae30a, SUC_lexicon_tags_63 },
    { 0xef2ae30c, SUC_lexicon_tags_33 },
    { 0xa74ae30d, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x212ae310, SUC_lexicon_tags_33 },
    { 0xd7cae310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2ae315, SUC_lexicon_tags_30 },
    { 0x336ae316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2ae31b, SUC_lexicon_tags_82 },
    { 0xeb2ae31c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5cae31e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58ae320, SUC_lexicon_tags_23 },
    { 0xa6aae320, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x968ae323, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21cae326, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeae32b, SUC_lexicon_tags_14 },
    { 0x76ae32c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeae32f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97eae335, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cae33f, SUC_lexicon_tags_11 },
    { 0xd22ae340, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cae344, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6ae347, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeae34d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x94aae350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02ae35d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x174ae35f, SUC_lexicon_tags_37 },
    { 0x204ae360, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ae363, SUC_lexicon_tags_7 },
    { 0x170ae364, SUC_lexicon_tags_13 },
    { 0x242ae365, SUC_lexicon_tags_49 },
    { 0xca2ae364, SUC_lexicon_tags_16 },
    { 0x832ae367, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38ae36c, SUC_lexicon_tags_77 },
    { 0x8f4ae36c, SUC_lexicon_tags_12 },
    { 0x34aae36d, SUC_lexicon_tags_37 },
    { 0x6c6ae36f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc24ae371, SUC_lexicon_tags_47 },
    { 0xac4ae372, SUC_lexicon_tags_26 },
    { 0xfb4ae373, SUC_lexicon_tags_79 },
    { 0xdacae374, SUC_lexicon_tags_37 },
    { 0x72cae374, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8ae37a, SUC_lexicon_tags_26 },
    { 0x878ae37a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82eae37f, SUC_lexicon_tags_49 },
    { 0xcb0ae380, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a2ae382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6ae387, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28ae38b, SUC_lexicon_tags_11 },
    { 0x7eeae38c, SUC_lexicon_tags_9 },
    { 0xb8ae38d, SUC_lexicon_tags_38 },
    { 0xa3eae38e, SUC_lexicon_tags_45 },
    { 0xfceae38e, SUC_lexicon_tags_43 },
    { 0x858ae38d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdbcae392, SUC_lexicon_tags_12 },
    { 0x5cae393, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf2eae395, SUC_lexicon_tags_25 },
    { 0xfeaae396, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0ae39a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x530ae39d, SUC_lexicon_tags_0 },
    { 0xf28ae39e, SUC_lexicon_tags_14 },
    { 0x156ae39e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92ae3a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b0ae3a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0ae3a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf66ae3aa, SUC_lexicon_tags_25 },
    { 0x1c8ae3aa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdd4ae3ad, SUC_lexicon_tags_15 },
    { 0xbc2ae3ad, SUC_lexicon_tags_12 },
    { 0x636ae3af, SUC_lexicon_tags_17 },
    { 0xd94ae3ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96ae3b3, SUC_lexicon_tags_27 },
    { 0x418ae3b4, SUC_lexicon_tags_23 },
    { 0x920ae3b5, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eae3bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52ae3c1, SUC_lexicon_tags_19 },
    { 0x7f4ae3c2, SUC_lexicon_tags_25 },
    { 0x792ae3c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x688ae3c6, SUC_lexicon_tags_6 },
    { 0x94aae3c7, SUC_lexicon_tags_29 },
    { 0x56aae3c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48eae3cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaae3d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ae3d3, SUC_lexicon_tags_25 },
    { 0xca6ae3d3, SUC_lexicon_tags_6 },
    { 0xb20ae3d3, SUC_lexicon_tags_47 },
    { 0x4e4ae3d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0ae3d9, SUC_lexicon_tags_1 },
    { 0x52aae3da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8ae3dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ae3e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50ae3e4, SUC_lexicon_tags_38 },
    { 0xb6eae3e5, SUC_lexicon_tags_10 },
    { 0x76eae3e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cae3f3, SUC_lexicon_tags_25 },
    { 0xdbaae3f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0ae3fd, SUC_lexicon_tags_25 },
    { 0xe1cae3fe, SUC_lexicon_tags_7 },
    { 0x8caae3ff, SUC_lexicon_tags_17 },
    { 0xf66ae400, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84ae404, SUC_lexicon_tags_48 },
    { 0x136ae405, SUC_lexicon_tags_12 },
    { 0x952ae406, SUC_lexicon_tags_23 },
    { 0xe9cae407, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11aae409, SUC_lexicon_tags_47 },
    { 0xbf6ae40a, SUC_lexicon_tags_19 },
    { 0x180ae40b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb8aae40d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52ae414, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30aae418, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37eae41a, SUC_lexicon_tags_35 },
    { 0xfeae41b, SUC_lexicon_tags_0 },
    { 0xcdaae41a, SUC_lexicon_tags_7 },
    { 0xc9eae41b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cae421, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230ae425, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ae42d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x580ae430, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x642ae433, SUC_lexicon_tags_29 },
    { 0x140ae433, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb8cae436, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92ae43a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4ae43d, SUC_lexicon_tags_27 },
    { 0xa80ae43e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x682ae441, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeecae443, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8ae446, SUC_lexicon_tags_27 },
    { 0x1d0ae446, SUC_lexicon_tags_37 },
    { 0xb9cae448, SUC_lexicon_tags_23 },
    { 0x4aaae449, SUC_lexicon_tags_2 },
    { 0x1e8ae448, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xea4ae44c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6ae451, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8ae454, SUC_lexicon_tags_66 },
    { 0x5c4ae454, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb92ae457, SUC_lexicon_tags_25 },
    { 0x7beae458, SUC_lexicon_tags_48 },
    { 0x70eae459, SUC_lexicon_tags_6 },
    { 0x4f8ae45a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392ae45e, SUC_lexicon_tags_49 },
    { 0xcbaae45e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc56ae461, SUC_lexicon_tags_60 },
    { 0x240ae461, SUC_lexicon_tags_63 },
    { 0xd68ae463, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3a6ae465, SUC_lexicon_tags_38 },
    { 0xd9aae466, SUC_lexicon_tags_26 },
    { 0x2f4ae467, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8ae46c, SUC_lexicon_tags_94 },
    { 0x7a8ae46d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cae471, SUC_lexicon_tags_49 },
    { 0xec6ae471, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x192ae474, SUC_lexicon_tags_1 },
    { 0x8eae474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2ae479, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6ae47e, SUC_lexicon_tags_37 },
    { 0xc0aae47f, SUC_lexicon_tags_2 },
    { 0x68ae480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680ae484, SUC_lexicon_tags_54 },
    { 0x3e6ae484, SUC_lexicon_tags_59 },
    { 0x340ae485, SUC_lexicon_tags_15 },
    { 0x198ae487, SUC_lexicon_tags_3 },
    { 0xae8ae488, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cae48f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7caae497, SUC_lexicon_tags_63 },
    { 0xc62ae498, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84ae49f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8ae4a2, SUC_lexicon_tags_6 },
    { 0x1cae4a3, SUC_lexicon_tags_21 },
    { 0xdecae4a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f6ae4a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eae4a9, SUC_lexicon_tags_1 },
    { 0x668ae4aa, SUC_lexicon_tags_70 },
    { 0x52cae4aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8faae4ae, SUC_lexicon_tags_61 },
    { 0x372ae4af, SUC_lexicon_tags_37 },
    { 0xb20ae4b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x168ae4b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eae4b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbaae4b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x53aae4bb, SUC_lexicon_tags_27 },
    { 0x75aae4bb, SUC_lexicon_tags_15 },
    { 0x34eae4bd, SUC_lexicon_tags_21 },
    { 0xb3cae4bd, SUC_lexicon_tags_133 },
    { 0xedeae4bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcae4ca, SUC_lexicon_tags_25 },
    { 0xc84ae4cb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4f6ae4cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64ae4d0, SUC_lexicon_tags_15 },
    { 0x2c8ae4d1, SUC_lexicon_tags_41 },
    { 0xfdeae4d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14aae4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6ae4df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd0ae4e1, SUC_lexicon_tags_37 },
    { 0x1e4ae4e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3aae4e5, SUC_lexicon_tags_16 },
    { 0x7b6ae4e6, SUC_lexicon_tags_15 },
    { 0xd04ae4e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eae4ed, SUC_lexicon_tags_10 },
    { 0x134ae4ed, SUC_lexicon_tags_47 },
    { 0x1f6ae4ed, SUC_lexicon_tags_33 },
    { 0xad0ae4f0, SUC_lexicon_tags_25 },
    { 0x53cae4ee, SUC_lexicon_tags_7 },
    { 0x300ae4ee, SUC_lexicon_tags_12 },
    { 0x2a8ae4f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27aae4f5, SUC_lexicon_tags_11 },
    { 0xf5eae4f6, SUC_lexicon_tags_17 },
    { 0xe9cae4f7, SUC_lexicon_tags_25 },
    { 0x7b4ae4f7, SUC_lexicon_tags_39 },
    { 0xe30ae4f9, SUC_lexicon_tags_1 },
    { 0xe1cae4f7, SUC_lexicon_tags_17 },
    { 0xcbcae4fb, SUC_lexicon_tags_53 },
    { 0x54ae4fc, SUC_lexicon_tags_47 },
    { 0x8e8ae4f7, SUC_lexicon_tags_106 },
    { 0x9deae4fe, SUC_lexicon_tags_17 },
    { 0x3a8ae4ff, SUC_lexicon_tags_37 },
    { 0x5c6ae4ff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66ae503, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aae50a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0ae50d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cae511, SUC_lexicon_tags_6 },
    { 0x498ae511, SUC_lexicon_tags_37 },
    { 0x75cae513, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x550ae515, SUC_lexicon_tags_23 },
    { 0x4f2ae516, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cae519, SUC_lexicon_tags_49 },
    { 0xa60ae519, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eae520, SUC_lexicon_tags_11 },
    { 0x8feae521, SUC_lexicon_tags_26 },
    { 0xd7eae522, SUC_lexicon_tags_17 },
    { 0x470ae521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x622ae525, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884ae52e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5aae530, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688ae536, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17eae538, SUC_lexicon_tags_7 },
    { 0xb04ae539, SUC_lexicon_tags_11 },
    { 0xb3cae539, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x236ae53d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198ae545, SUC_lexicon_tags_49 },
    { 0x496ae546, SUC_lexicon_tags_0 },
    { 0xaf8ae547, SUC_lexicon_tags_1 },
    { 0x9e4ae545, SUC_lexicon_tags_7 },
    { 0x92cae547, SUC_lexicon_tags_47 },
    { 0x530ae54a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418ae54e, SUC_lexicon_tags_5 },
    { 0x91eae54f, SUC_lexicon_tags_2 },
    { 0x962ae550, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4ae554, SUC_lexicon_tags_2 },
    { 0x814ae555, SUC_lexicon_tags_0 },
    { 0x42ae556, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0ae559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946ae55d, SUC_lexicon_tags_1 },
    { 0x598ae55d, SUC_lexicon_tags_66 },
    { 0xdc8ae55f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc54ae561, SUC_lexicon_tags_11 },
    { 0xac4ae561, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616ae569, SUC_lexicon_tags_158 },
    { 0xc4aae56a, SUC_lexicon_tags_43 },
    { 0xd04ae569, SUC_lexicon_tags_2 },
    { 0x8f2ae56c, SUC_lexicon_tags_31 },
    { 0xd12ae56c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x308ae56f, SUC_lexicon_tags_20 },
    { 0xa32ae570, SUC_lexicon_tags_63 },
    { 0xd64ae571, SUC_lexicon_tags_25 },
    { 0x4e4ae571, SUC_lexicon_tags_21 },
    { 0x6b0ae572, SUC_lexicon_tags_11 },
    { 0x5d2ae572, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcae577, SUC_lexicon_tags_15 },
    { 0x420ae577, SUC_lexicon_tags_49 },
    { 0x348ae579, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42ae583, SUC_lexicon_tags_31 },
    { 0xc56ae583, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cae587, SUC_lexicon_tags_1 },
    { 0x5eeae587, SUC_lexicon_tags_17 },
    { 0x220ae589, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x676ae58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecae58f, SUC_lexicon_tags_8 },
    { 0x4a4ae58f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x444ae593, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x776ae596, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cae59c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x596ae59e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcae5a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd88ae5a2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432ae5a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43aae5aa, SUC_lexicon_tags_15 },
    { 0x4a8ae5ab, SUC_lexicon_tags_11 },
    { 0xda8ae5ab, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5aae5ae, SUC_lexicon_tags_25 },
    { 0x93eae5af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8ae5b2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ae5b5, SUC_lexicon_tags_37 },
    { 0xf02ae5b5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6dcae5b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0ae5bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaeae5c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31aae5c5, SUC_lexicon_tags_17 },
    { 0x39aae5c5, SUC_lexicon_tags_11 },
    { 0x168ae5c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d2ae5c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56ae5cd, SUC_lexicon_tags_45 },
    { 0x542ae5ce, SUC_lexicon_tags_37 },
    { 0x352ae5cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792ae5d4, SUC_lexicon_tags_19 },
    { 0x918ae5d5, SUC_lexicon_tags_8 },
    { 0x362ae5d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66ae5db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eae5de, SUC_lexicon_tags_1 },
    { 0x95eae5df, SUC_lexicon_tags_6 },
    { 0x366ae5df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x268ae5e2, SUC_lexicon_tags_11 },
    { 0xdfaae5e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26cae5e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702ae5eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918ae5f0, SUC_lexicon_tags_11 },
    { 0xf28ae5f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928ae5f5, SUC_lexicon_tags_47 },
    { 0xa8eae5f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90ae5f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44eae5fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0ae5fd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4ae600, SUC_lexicon_tags_37 },
    { 0x904ae601, SUC_lexicon_tags_31 },
    { 0x8feae602, SUC_lexicon_tags_23 },
    { 0xa4ae602, SUC_lexicon_tags_19 },
    { 0xab6ae602, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd22ae606, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7f4ae608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aae610, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8ae615, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa98ae617, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe58ae619, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x614ae61b, SUC_lexicon_tags_14 },
    { 0x7b8ae61c, SUC_lexicon_tags_12 },
    { 0xba2ae61c, SUC_lexicon_tags_14 },
    { 0xfdaae61c, SUC_lexicon_tags_27 },
    { 0x8e8ae61c, SUC_lexicon_tags_23 },
    { 0x616ae61c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eae625, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ae628, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x768ae62a, SUC_lexicon_tags_25 },
    { 0xb64ae62b, SUC_lexicon_tags_38 },
    { 0x4a2ae62c, SUC_lexicon_tags_11 },
    { 0xabeae62b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccae634, SUC_lexicon_tags_48 },
    { 0xfe2ae635, SUC_lexicon_tags_9 },
    { 0xc38ae636, SUC_lexicon_tags_38 },
    { 0x1d2ae637, SUC_lexicon_tags_11 },
    { 0xeaae638, SUC_lexicon_tags_81 },
    { 0x590ae638, SUC_lexicon_tags_17 },
    { 0xcfcae639, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cae63d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cae640, SUC_lexicon_tags_16 },
    { 0xd5eae640, SUC_lexicon_tags_23 },
    { 0x3ceae640, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x294ae644, SUC_lexicon_tags_37 },
    { 0xfdeae644, SUC_lexicon_tags_84 },
    { 0x7eaae646, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44ae649, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6ae64c, SUC_lexicon_tags_136 },
    { 0xf82ae64d, SUC_lexicon_tags_32 },
    { 0x8eeae64d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0ae652, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x378ae655, SUC_lexicon_tags_2 },
    { 0xb44ae656, SUC_lexicon_tags_26 },
    { 0x1e4ae655, SUC_lexicon_tags_39 },
    { 0x71cae657, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef8ae65a, SUC_lexicon_tags_15 },
    { 0x606ae65b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6ae664, SUC_lexicon_tags_7 },
    { 0xc5eae665, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbaae667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21aae66e, SUC_lexicon_tags_9 },
    { 0xa4cae66f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860ae673, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcceae676, SUC_lexicon_tags_30 },
    { 0x538ae677, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8ae67f, SUC_lexicon_tags_14 },
    { 0x620ae680, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722ae684, SUC_lexicon_tags_26 },
    { 0x2faae685, SUC_lexicon_tags_12 },
    { 0xbe8ae686, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf38ae688, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40aae68f, SUC_lexicon_tags_12 },
    { 0x3faae690, SUC_lexicon_tags_2 },
    { 0xb5aae68f, SUC_lexicon_tags_38 },
    { 0xf6ae692, SUC_lexicon_tags_25 },
    { 0x658ae693, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8ae699, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9eae69b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc8ae69d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8ae6a1, SUC_lexicon_tags_1 },
    { 0x98cae6a2, SUC_lexicon_tags_14 },
    { 0x9d2ae6a3, SUC_lexicon_tags_44 },
    { 0x4eaae6a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eae6a8, SUC_lexicon_tags_21 },
    { 0x2cae6a9, SUC_lexicon_tags_11 },
    { 0x500ae6aa, SUC_lexicon_tags_45 },
    { 0x6dcae6a9, SUC_lexicon_tags_77 },
    { 0xc30ae6a8, SUC_lexicon_tags_63 },
    { 0x516ae6ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18ae6b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x52cae6b3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cae6b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x29eae6ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54ae6c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46aae6c5, SUC_lexicon_tags_7 },
    { 0x5eaae6c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3f0ae6c8, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876ae6cc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcae6d0, SUC_lexicon_tags_15 },
    { 0x198ae6d1, SUC_lexicon_tags_11 },
    { 0x748ae6d1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63aae6d9, SUC_lexicon_tags_17 },
    { 0xdacae6da, SUC_lexicon_tags_9 },
    { 0x6e8ae6da, SUC_lexicon_tags_25 },
    { 0x586ae6db, SUC_lexicon_tags_65 },
    { 0x804ae6dd, SUC_lexicon_tags_11 },
    { 0xa14ae6dc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ae6e1, SUC_lexicon_tags_33 },
    { 0x972ae6e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4ae6e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4eae6e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aae6ec, SUC_lexicon_tags_37 },
    { 0x8daae6ec, SUC_lexicon_tags_2 },
    { 0x448ae6ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9beae6f4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2ae6fb, SUC_lexicon_tags_13 },
    { 0xc2ae6fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35aae701, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x30cae703, SUC_lexicon_tags_39 },
    { 0x802ae703, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x108ae707, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6eae709, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x99eae70c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32cae70e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcae714, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2ae717, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf12ae719, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa70ae71b, SUC_lexicon_tags_15 },
    { 0xccaae71c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58ae71f, SUC_lexicon_tags_2 },
    { 0xe8ae71f, SUC_lexicon_tags_15 },
    { 0xae4ae721, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x812ae723, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aae728, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baae72c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06ae731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30ae73a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44ae73d, SUC_lexicon_tags_11 },
    { 0x3a6ae73e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28ae741, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc20ae743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cae748, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaae74b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xccae74d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6ae751, SUC_lexicon_tags_30 },
    { 0xd2eae751, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x94ae754, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8ae758, SUC_lexicon_tags_47 },
    { 0xe82ae759, SUC_lexicon_tags_39 },
    { 0xe46ae75a, SUC_lexicon_tags_7 },
    { 0xb30ae75b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8ae761, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ae766, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x82aae768, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb72ae76a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22cae76c, SUC_lexicon_tags_19 },
    { 0xee8ae76c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332ae777, SUC_lexicon_tags_1 },
    { 0x8a4ae777, SUC_lexicon_tags_15 },
    { 0xb98ae779, SUC_lexicon_tags_11 },
    { 0x71cae779, SUC_lexicon_tags_477 },
    { 0x898ae778, SUC_lexicon_tags_7 },
    { 0xdc2ae779, SUC_lexicon_tags_12 },
    { 0xac6ae77b, SUC_lexicon_tags_11 },
    { 0x798ae77d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5aae780, SUC_lexicon_tags_16 },
    { 0x1f4ae780, SUC_lexicon_tags_37 },
    { 0x818ae782, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ceae788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1beae78b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcae78f, SUC_lexicon_tags_11 },
    { 0xf66ae78f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeb4ae792, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x78ae794, SUC_lexicon_tags_83 },
    { 0xa02ae795, SUC_lexicon_tags_25 },
    { 0x35cae796, SUC_lexicon_tags_30 },
    { 0x512ae797, SUC_lexicon_tags_17 },
    { 0x3dcae796, SUC_lexicon_tags_1 },
    { 0xe10ae795, SUC_lexicon_tags_1 },
    { 0xf6ae79a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984ae79f, SUC_lexicon_tags_17 },
    { 0xe5cae79f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4ae7a4, SUC_lexicon_tags_11 },
    { 0xd38ae7a5, SUC_lexicon_tags_15 },
    { 0x28cae7a5, SUC_lexicon_tags_32 },
    { 0xf76ae7a6, SUC_lexicon_tags_704 },
    { 0x170ae7a5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22ae7ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8ae7ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04ae7b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x684ae7b4, SUC_lexicon_tags_23 },
    { 0xb56ae7b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02ae7b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2beae7be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986ae7c7, SUC_lexicon_tags_6 },
    { 0xed8ae7c8, SUC_lexicon_tags_33 },
    { 0xe30ae7c8, SUC_lexicon_tags_23 },
    { 0x65eae7c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x48aae7cd, SUC_lexicon_tags_27 },
    { 0xe2ae7cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x93cae7d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64aae7d2, SUC_lexicon_tags_11 },
    { 0x13aae7d3, SUC_lexicon_tags_14 },
    { 0x368ae7d4, SUC_lexicon_tags_1 },
    { 0x2beae7d5, SUC_lexicon_tags_8 },
    { 0xc5aae7d6, SUC_lexicon_tags_25 },
    { 0x5e4ae7d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2bcae7d9, SUC_lexicon_tags_26 },
    { 0xc7eae7da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8ae7dd, SUC_lexicon_tags_17 },
    { 0x8d6ae7dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xee8ae7e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x472ae7e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300ae7e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe66ae7ea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcae7ee, SUC_lexicon_tags_37 },
    { 0x3d2ae7ee, SUC_lexicon_tags_47 },
    { 0xe8cae7ef, SUC_lexicon_tags_11 },
    { 0xe06ae7f1, SUC_lexicon_tags_146 },
    { 0x886ae7f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6ae801, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3eae803, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8ae807, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x810ae80a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaceae80c, SUC_lexicon_tags_7 },
    { 0xae0ae80d, SUC_lexicon_tags_11 },
    { 0x57cae80e, SUC_lexicon_tags_4 },
    { 0x27cae80f, SUC_lexicon_tags_25 },
    { 0x9d6ae80e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2ae813, SUC_lexicon_tags_17 },
    { 0x87cae814, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4ae817, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x48aae81a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf8ae81c, SUC_lexicon_tags_19 },
    { 0x736ae81d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe6eae81f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0ae822, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97cae824, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840ae82b, SUC_lexicon_tags_0 },
    { 0x716ae82c, SUC_lexicon_tags_38 },
    { 0x36ae82d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x642ae830, SUC_lexicon_tags_25 },
    { 0xdceae830, SUC_lexicon_tags_0 },
    { 0x77aae832, SUC_lexicon_tags_17 },
    { 0x4faae833, SUC_lexicon_tags_1 },
    { 0xc3cae834, SUC_lexicon_tags_17 },
    { 0x970ae835, SUC_lexicon_tags_10 },
    { 0xe0ae836, SUC_lexicon_tags_0 },
    { 0x9b4ae837, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x442ae839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ae840, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecae84a, SUC_lexicon_tags_2 },
    { 0x5f4ae84b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352ae851, SUC_lexicon_tags_12 },
    { 0xe8ae851, SUC_lexicon_tags_39 },
    { 0x25aae853, SUC_lexicon_tags_2 },
    { 0xa14ae853, SUC_lexicon_tags_17 },
    { 0xf92ae851, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aae858, SUC_lexicon_tags_29 },
    { 0x764ae858, SUC_lexicon_tags_27 },
    { 0x1c2ae85a, SUC_lexicon_tags_2 },
    { 0xc38ae85b, SUC_lexicon_tags_47 },
    { 0x860ae85c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x17aae85f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cae865, SUC_lexicon_tags_37 },
    { 0xe8cae866, SUC_lexicon_tags_1 },
    { 0xeb0ae866, SUC_lexicon_tags_15 },
    { 0xc1cae868, SUC_lexicon_tags_21 },
    { 0x5a6ae869, SUC_lexicon_tags_40 },
    { 0xe30ae866, SUC_lexicon_tags_31 },
    { 0xc06ae869, SUC_lexicon_tags_37 },
    { 0x5ceae868, SUC_lexicon_tags_1 },
    { 0x60eae86d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ae871, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x840ae873, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe90ae875, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x880ae878, SUC_lexicon_tags_32 },
    { 0xf14ae879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x496ae87c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeae884, SUC_lexicon_tags_6 },
    { 0x63aae885, SUC_lexicon_tags_7 },
    { 0xcc8ae884, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xfb0ae888, SUC_lexicon_tags_28 },
    { 0x926ae889, SUC_lexicon_tags_1 },
    { 0x806ae88a, SUC_lexicon_tags_60 },
    { 0xcf8ae88b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x938ae88e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910ae893, SUC_lexicon_tags_37 },
    { 0x916ae894, SUC_lexicon_tags_15 },
    { 0x394ae894, SUC_lexicon_tags_2 },
    { 0x49cae896, SUC_lexicon_tags_17 },
    { 0xbe4ae896, SUC_lexicon_tags_17 },
    { 0xbc2ae896, SUC_lexicon_tags_65 },
    { 0xc54ae899, SUC_lexicon_tags_27 },
    { 0x2caae899, SUC_lexicon_tags_2 },
    { 0xbbcae89b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc62ae89d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdf2ae89f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114ae8a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89aae8ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852ae8b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9eaae8b2, SUC_lexicon_tags_2 },
    { 0x7b4ae8b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498ae8b9, SUC_lexicon_tags_33 },
    { 0x192ae8ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43eae8bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceae8c0, SUC_lexicon_tags_19 },
    { 0xaf2ae8c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12ae8c5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xd7aae8c7, SUC_lexicon_tags_32 },
    { 0x570ae8c7, SUC_lexicon_tags_17 },
    { 0x64eae8c9, SUC_lexicon_tags_97 },
    { 0xa04ae8c9, SUC_lexicon_tags_11 },
    { 0x20aae8cb, SUC_lexicon_tags_37 },
    { 0xaae8cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ae8cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4ae8d4, SUC_lexicon_tags_15 },
    { 0x588ae8d4, SUC_lexicon_tags_11 },
    { 0x442ae8d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cae8db, SUC_lexicon_tags_87 },
    { 0xdeae8dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94ae8de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694ae8e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6c0ae8e4, SUC_lexicon_tags_15 },
    { 0xbaae8e5, SUC_lexicon_tags_37 },
    { 0x15aae8e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cae8ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eae8f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb02ae8fb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44ae901, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88cae903, SUC_lexicon_tags_11 },
    { 0xcb0ae904, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50ae907, SUC_lexicon_tags_259 },
    { 0x272ae908, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71aae90b, SUC_lexicon_tags_26 },
    { 0x526ae90c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8eaae90e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80aae916, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4ae91a, SUC_lexicon_tags_43 },
    { 0xb24ae91b, SUC_lexicon_tags_12 },
    { 0x6d2ae91b, SUC_lexicon_tags_7 },
    { 0xfeeae91d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcae920, SUC_lexicon_tags_63 },
    { 0x9baae921, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e8ae923, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x43cae925, SUC_lexicon_tags_10 },
    { 0xbaaae926, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ae929, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6ae931, SUC_lexicon_tags_27 },
    { 0x2f4ae932, SUC_lexicon_tags_57 },
    { 0x1ceae933, SUC_lexicon_tags_32 },
    { 0xf54ae934, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34ae937, SUC_lexicon_tags_10 },
    { 0x3d8ae937, SUC_lexicon_tags_11 },
    { 0xbbcae939, SUC_lexicon_tags_21 },
    { 0x41cae93a, SUC_lexicon_tags_61 },
    { 0x162ae93b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x620ae93d, SUC_lexicon_tags_46 },
    { 0x370ae93e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88ae942, SUC_lexicon_tags_11 },
    { 0x19cae943, SUC_lexicon_tags_2 },
    { 0x9b4ae944, SUC_lexicon_tags_19 },
    { 0x7b0ae943, SUC_lexicon_tags_113 },
    { 0x878ae946, SUC_lexicon_tags_9 },
    { 0x2f6ae947, SUC_lexicon_tags_11 },
    { 0xec4ae948, SUC_lexicon_tags_37 },
    { 0x606ae947, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x420ae94b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22ae94f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc6ae951, SUC_lexicon_tags_12 },
    { 0xb2cae952, SUC_lexicon_tags_43 },
    { 0x350ae953, SUC_lexicon_tags_17 },
    { 0xc0ae953, SUC_lexicon_tags_10 },
    { 0x9faae955, SUC_lexicon_tags_17 },
    { 0xa96ae954, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x390ae958, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6ae95c, SUC_lexicon_tags_12 },
    { 0xe00ae95c, SUC_lexicon_tags_0 },
    { 0x254ae95d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x490ae961, SUC_lexicon_tags_17 },
    { 0x2d0ae962, SUC_lexicon_tags_6 },
    { 0xbe6ae963, SUC_lexicon_tags_7 },
    { 0x618ae962, SUC_lexicon_tags_2 },
    { 0x63eae964, SUC_lexicon_tags_9 },
    { 0x29eae966, SUC_lexicon_tags_1 },
    { 0xa8cae967, SUC_lexicon_tags_15 },
    { 0x932ae967, SUC_lexicon_tags_37 },
    { 0xc18ae963, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6ae970, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02ae973, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd92ae975, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x218ae978, SUC_lexicon_tags_12 },
    { 0x220ae979, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ae980, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x126ae982, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaae989, SUC_lexicon_tags_35 },
    { 0x65eae98a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaae990, SUC_lexicon_tags_55 },
    { 0x1f2ae991, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x630ae993, SUC_lexicon_tags_60 },
    { 0x870ae994, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeccae997, SUC_lexicon_tags_0 },
    { 0xa4cae998, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40ae9a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0ae9aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110ae9b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44ae9b5, SUC_lexicon_tags_47 },
    { 0x87cae9b6, SUC_lexicon_tags_17 },
    { 0xb8ae9b6, SUC_lexicon_tags_25 },
    { 0xdb4ae9b7, SUC_lexicon_tags_0 },
    { 0xbd0ae9b8, SUC_lexicon_tags_11 },
    { 0x854ae9ba, SUC_lexicon_tags_38 },
    { 0xc5aae9b9, SUC_lexicon_tags_62 },
    { 0xfdcae9bc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x5e4ae9be, SUC_lexicon_tags_15 },
    { 0xa48ae9bf, SUC_lexicon_tags_6 },
    { 0xf24ae9be, SUC_lexicon_tags_6 },
    { 0xe36ae9be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878ae9c7, SUC_lexicon_tags_1 },
    { 0xe8cae9c8, SUC_lexicon_tags_3 },
    { 0xa3cae9c9, SUC_lexicon_tags_17 },
    { 0x804ae9c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x52aae9cc, SUC_lexicon_tags_5 },
    { 0x2b2ae9cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd96ae9cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51eae9da, SUC_lexicon_tags_7 },
    { 0x436ae9da, SUC_lexicon_tags_7 },
    { 0xa7eae9dc, SUC_lexicon_tags_7 },
    { 0xb02ae9dd, SUC_lexicon_tags_8 },
    { 0xb3eae9dd, SUC_lexicon_tags_45 },
    { 0xd06ae9da, SUC_lexicon_tags_14 },
    { 0xfccae9e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a4ae9e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeae9ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f6ae9ee, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xcc2ae9f0, SUC_lexicon_tags_12 },
    { 0x1e8ae9f0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x30ae9f3, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8ae9f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf56ae9fa, SUC_lexicon_tags_1 },
    { 0x952ae9fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36cae9fd, SUC_lexicon_tags_27 },
    { 0xe24ae9fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1eaea02, SUC_lexicon_tags_12 },
    { 0xc5caea03, SUC_lexicon_tags_87 },
    { 0x738aea02, SUC_lexicon_tags_28 },
    { 0x5aaaea05, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5faaea08, SUC_lexicon_tags_29 },
    { 0x248aea09, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x90eaea0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x450aea0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0aea11, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xed4aea13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x256aea15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaea18, SUC_lexicon_tags_27 },
    { 0xc70aea19, SUC_lexicon_tags_11 },
    { 0x9c2aea1a, SUC_lexicon_tags_976 },
    { 0x46aea1b, SUC_lexicon_tags_63 },
    { 0x1e0aea1c, SUC_lexicon_tags_31 },
    { 0xeeeaea1d, SUC_lexicon_tags_37 },
    { 0x8deaea1c, SUC_lexicon_tags_14 },
    { 0x394aea1f, SUC_lexicon_tags_63 },
    { 0x242aea20, SUC_lexicon_tags_25 },
    { 0xd3aaea21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaaea31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33aaea36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x64aea38, SUC_lexicon_tags_14 },
    { 0x68aea38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2aea3e, SUC_lexicon_tags_65 },
    { 0x9ecaea3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8aea4d, SUC_lexicon_tags_7 },
    { 0x15eaea4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x360aea50, SUC_lexicon_tags_31 },
    { 0x210aea50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x420aea53, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd46aea55, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6c0aea57, SUC_lexicon_tags_14 },
    { 0xf6caea57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2caea60, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562aea66, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa20aea68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178aea6c, SUC_lexicon_tags_45 },
    { 0xbb2aea6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x524aea6f, SUC_lexicon_tags_15 },
    { 0x816aea70, SUC_lexicon_tags_25 },
    { 0x1d4aea71, SUC_lexicon_tags_0 },
    { 0xcb8aea72, SUC_lexicon_tags_17 },
    { 0x972aea71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x774aea76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcaea7c, SUC_lexicon_tags_12 },
    { 0x6fcaea7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96eaea81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x812aea83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33eaea89, SUC_lexicon_tags_11 },
    { 0x87aaea89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58aea8f, SUC_lexicon_tags_14 },
    { 0x6a2aea90, SUC_lexicon_tags_70 },
    { 0xec8aea8f, SUC_lexicon_tags_43 },
    { 0x15eaea92, SUC_lexicon_tags_77 },
    { 0x63aaea90, SUC_lexicon_tags_179 },
    { 0x628aea94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x294aea97, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf7aaea99, SUC_lexicon_tags_11 },
    { 0x314aea99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x906aea9c, SUC_lexicon_tags_31 },
    { 0xc70aea9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2aeaa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x194aeaa3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x482aeaa5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16aeaa8, SUC_lexicon_tags_15 },
    { 0x3e0aeaa9, SUC_lexicon_tags_0 },
    { 0x438aeaa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44caeab3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0aeab6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764aeabc, SUC_lexicon_tags_30 },
    { 0x5d4aeabd, SUC_lexicon_tags_26 },
    { 0xd9aaeabd, SUC_lexicon_tags_37 },
    { 0xc3caeabf, SUC_lexicon_tags_49 },
    { 0x9dcaeac0, SUC_lexicon_tags_40 },
    { 0x41eaeac1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3aaeac4, SUC_lexicon_tags_15 },
    { 0xe8eaeac4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x510aeac7, SUC_lexicon_tags_26 },
    { 0x804aeac8, SUC_lexicon_tags_25 },
    { 0x26caeac9, SUC_lexicon_tags_11 },
    { 0x49eaeac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8caead2, SUC_lexicon_tags_40 },
    { 0xa9caead3, SUC_lexicon_tags_11 },
    { 0x8d4aead2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aaead7, SUC_lexicon_tags_32 },
    { 0x506aead8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x718aeada, SUC_lexicon_tags_15 },
    { 0x2a4aeadb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x84aeadd, SUC_lexicon_tags_11 },
    { 0xf76aeade, SUC_lexicon_tags_8 },
    { 0x682aeade, SUC_lexicon_tags_26 },
    { 0xec6aeae0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aeaeae5, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6aeaef, SUC_lexicon_tags_49 },
    { 0x1e4aeaf0, SUC_lexicon_tags_37 },
    { 0xd56aeaf1, SUC_lexicon_tags_15 },
    { 0x5ecaeaf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476aeaf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x708aeaf9, SUC_lexicon_tags_8 },
    { 0xc96aeafa, SUC_lexicon_tags_17 },
    { 0xad6aeaf9, SUC_lexicon_tags_66 },
    { 0xfe6aeafc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eaeaff, SUC_lexicon_tags_26 },
    { 0x8ecaeb00, SUC_lexicon_tags_27 },
    { 0xc9aaeb00, SUC_lexicon_tags_23 },
    { 0xdd0aeb02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76aeb05, SUC_lexicon_tags_28 },
    { 0xc1eaeb06, SUC_lexicon_tags_17 },
    { 0x9bcaeb07, SUC_lexicon_tags_4 },
    { 0x4b0aeb05, SUC_lexicon_tags_47 },
    { 0xb16aeb09, SUC_lexicon_tags_11 },
    { 0x338aeb07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4aeb0e, SUC_lexicon_tags_11 },
    { 0xcf4aeb0f, SUC_lexicon_tags_8 },
    { 0x2f4aeb10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6aeb14, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x34aeb16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808aeb1d, SUC_lexicon_tags_12 },
    { 0x610aeb1e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2aeb21, SUC_lexicon_tags_37 },
    { 0x146aeb22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56aeb24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12aeb28, SUC_lexicon_tags_37 },
    { 0x546aeb29, SUC_lexicon_tags_41 },
    { 0x394aeb29, SUC_lexicon_tags_11 },
    { 0x2e2aeb2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0aeb2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144aeb3a, SUC_lexicon_tags_82 },
    { 0x6b2aeb3b, SUC_lexicon_tags_12 },
    { 0xe5eaeb3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4caeb3e, SUC_lexicon_tags_6 },
    { 0x50aaeb3e, SUC_lexicon_tags_64 },
    { 0x886aeb3f, SUC_lexicon_tags_12 },
    { 0x7b0aeb41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4eaeb47, SUC_lexicon_tags_28 },
    { 0x8dcaeb48, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x80aaeb4a, SUC_lexicon_tags_9 },
    { 0x1d8aeb4b, SUC_lexicon_tags_7 },
    { 0xf0eaeb4c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6c2aeb4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x856aeb51, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x594aeb53, SUC_lexicon_tags_45 },
    { 0xbc2aeb54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8eaeb56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x678aeb59, SUC_lexicon_tags_28 },
    { 0xee0aeb5a, SUC_lexicon_tags_25 },
    { 0xe12aeb59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb3aaeb5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702aeb61, SUC_lexicon_tags_12 },
    { 0xda6aeb61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22aeb67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06aeb73, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xedeaeb75, SUC_lexicon_tags_25 },
    { 0x6e8aeb76, SUC_lexicon_tags_226 },
    { 0x810aeb75, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4aeb7a, SUC_lexicon_tags_38 },
    { 0x84aaeb7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26aeb80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcaeb86, SUC_lexicon_tags_0 },
    { 0x52aeb87, SUC_lexicon_tags_56 },
    { 0xb1caeb88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4aeb8b, SUC_lexicon_tags_32 },
    { 0x86eaeb8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52aaeb92, SUC_lexicon_tags_138 },
    { 0x302aeb93, SUC_lexicon_tags_15 },
    { 0xf8eaeb94, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xc6aaeb96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0aeb99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36aeb9e, SUC_lexicon_tags_12 },
    { 0xbb8aeb9f, SUC_lexicon_tags_53 },
    { 0x64aaeba0, SUC_lexicon_tags_33 },
    { 0x9a4aeba1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4aeba4, SUC_lexicon_tags_1 },
    { 0xad0aeba5, SUC_lexicon_tags_14 },
    { 0x690aeba4, SUC_lexicon_tags_69 },
    { 0xf2caeba7, SUC_lexicon_tags_11 },
    { 0x15aaeba8, SUC_lexicon_tags_12 },
    { 0x1b2aeba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5eaebab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x766aebad, SUC_lexicon_tags_17 },
    { 0xe2aebae, SUC_lexicon_tags_15 },
    { 0xdfaaebaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48aebb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52aebb7, SUC_lexicon_tags_38 },
    { 0x146aebb7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb0aaebba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a6aebbc, SUC_lexicon_tags_1 },
    { 0xea4aebbd, SUC_lexicon_tags_2 },
    { 0x2e2aebbc, SUC_lexicon_tags_31 },
    { 0x1e2aebbf, SUC_lexicon_tags_55 },
    { 0x8ceaebc0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbf6aebc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736aebca, SUC_lexicon_tags_33 },
    { 0x288aebca, SUC_lexicon_tags_9 },
    { 0xc0aaebcb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474aebd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64aebd4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x91aaebd6, SUC_lexicon_tags_37 },
    { 0xd0aebd7, SUC_lexicon_tags_12 },
    { 0x9e0aebd8, SUC_lexicon_tags_11 },
    { 0x15caebd7, SUC_lexicon_tags_46 },
    { 0xfd2aebda, SUC_lexicon_tags_12 },
    { 0xe32aebd8, SUC_lexicon_tags_8 },
    { 0x452aebdc, SUC_lexicon_tags_43 },
    { 0x56caebdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceeaebe3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8bcaebe5, SUC_lexicon_tags_25 },
    { 0x2d4aebe6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4aebe9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x57eaebec, SUC_lexicon_tags_63 },
    { 0x1caebed, SUC_lexicon_tags_30 },
    { 0x5c0aebee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852aebf2, SUC_lexicon_tags_11 },
    { 0xa98aebf2, SUC_lexicon_tags_7 },
    { 0x418aebf4, SUC_lexicon_tags_23 },
    { 0xd14aebf4, SUC_lexicon_tags_27 },
    { 0xcd0aebf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc28aebf8, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00aebfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x892aebfe, SUC_lexicon_tags_1 },
    { 0x1b6aebff, SUC_lexicon_tags_17 },
    { 0x33eaec00, SUC_lexicon_tags_14 },
    { 0x416aec01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc26aec03, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc2eaec05, SUC_lexicon_tags_11 },
    { 0x84caec06, SUC_lexicon_tags_16 },
    { 0xd86aec05, SUC_lexicon_tags_47 },
    { 0x46aec08, SUC_lexicon_tags_55 },
    { 0xe28aec08, SUC_lexicon_tags_15 },
    { 0xa16aec08, SUC_lexicon_tags_63 },
    { 0xabcaec05, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xc62aec0d, SUC_lexicon_tags_29 },
    { 0x2c6aec0d, SUC_lexicon_tags_37 },
    { 0x94aec0f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x1a2aec11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccaec15, SUC_lexicon_tags_351 },
    { 0x6daaec16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75caec1c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9eaec1f, SUC_lexicon_tags_65 },
    { 0x8eaec20, SUC_lexicon_tags_37 },
    { 0xf58aec21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56aec28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x324aec2a, SUC_lexicon_tags_8 },
    { 0x404aec2b, SUC_lexicon_tags_17 },
    { 0xe3eaec2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54aec33, SUC_lexicon_tags_17 },
    { 0x7eeaec33, SUC_lexicon_tags_16 },
    { 0xe9caec35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf70aec37, SUC_lexicon_tags_7 },
    { 0x34aec38, SUC_lexicon_tags_1 },
    { 0x8fcaec39, SUC_lexicon_tags_14 },
    { 0x7ecaec39, SUC_lexicon_tags_31 },
    { 0x59eaec39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704aec3f, SUC_lexicon_tags_15 },
    { 0x5daaec40, SUC_lexicon_tags_9 },
    { 0xb5caec40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ccaec43, SUC_lexicon_tags_47 },
    { 0x7a2aec44, SUC_lexicon_tags_1 },
    { 0x1a2aec45, SUC_lexicon_tags_11 },
    { 0xf7caec46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92aaec48, SUC_lexicon_tags_33 },
    { 0x848aec49, SUC_lexicon_tags_0 },
    { 0xdd8aec49, SUC_lexicon_tags_14 },
    { 0x868aec49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd36aec4d, SUC_lexicon_tags_25 },
    { 0x386aec4e, SUC_lexicon_tags_12 },
    { 0x3a8aec4e, SUC_lexicon_tags_37 },
    { 0x3e4aec50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7caec53, SUC_lexicon_tags_11 },
    { 0xce4aec54, SUC_lexicon_tags_0 },
    { 0xd94aec55, SUC_lexicon_tags_415 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60aec59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8aec64, SUC_lexicon_tags_55 },
    { 0xb82aec64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26aec69, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918aec71, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8aec76, SUC_lexicon_tags_25 },
    { 0x6e0aec77, SUC_lexicon_tags_2 },
    { 0x908aec78, SUC_lexicon_tags_37 },
    { 0x7c0aec77, SUC_lexicon_tags_63 },
    { 0x4e8aec78, SUC_lexicon_tags_68 },
    { 0x9caec7b, SUC_lexicon_tags_66 },
    { 0x7a2aec7c, SUC_lexicon_tags_12 },
    { 0xf76aec7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2caec83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3caaec86, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34aec8a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23eaec8f, SUC_lexicon_tags_4 },
    { 0x77eaec90, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x730aec92, SUC_lexicon_tags_30 },
    { 0x6f6aec92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x326aec96, SUC_lexicon_tags_0 },
    { 0xfb6aec97, SUC_lexicon_tags_11 },
    { 0x426aec96, SUC_lexicon_tags_4 },
    { 0x438aec99, SUC_lexicon_tags_69 },
    { 0x456aec9a, SUC_lexicon_tags_38 },
    { 0xafcaec97, SUC_lexicon_tags_37 },
    { 0xc00aec9c, SUC_lexicon_tags_15 },
    { 0xdc0aec9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77caeca1, SUC_lexicon_tags_8 },
    { 0x518aeca2, SUC_lexicon_tags_38 },
    { 0xceaeca3, SUC_lexicon_tags_38 },
    { 0xb46aeca4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acaeca9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92caecb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x288aecb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb4aecb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6aecbe, SUC_lexicon_tags_60 },
    { 0x4baaecbf, SUC_lexicon_tags_343 },
    { 0x88eaecc0, SUC_lexicon_tags_64 },
    { 0x65aaecc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58eaecc4, SUC_lexicon_tags_47 },
    { 0x7c8aecc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4aecc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x224aecca, SUC_lexicon_tags_19 },
    { 0x8aeccb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9caecd5, SUC_lexicon_tags_22 },
    { 0xde6aecd6, SUC_lexicon_tags_27 },
    { 0x28aecd6, SUC_lexicon_tags_7 },
    { 0xe04aecd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17eaecde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecaece2, SUC_lexicon_tags_10 },
    { 0x644aece3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7d8aece5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0aece8, SUC_lexicon_tags_11 },
    { 0xf60aece9, SUC_lexicon_tags_34 },
    { 0xc3eaecea, SUC_lexicon_tags_11 },
    { 0x5f2aece8, SUC_lexicon_tags_15 },
    { 0xc4caecec, SUC_lexicon_tags_7 },
    { 0xa9caecec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aaecf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eaecf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4aecf6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbeaecf8, SUC_lexicon_tags_33 },
    { 0xbe2aecf9, SUC_lexicon_tags_7 },
    { 0xc76aecf9, SUC_lexicon_tags_12 },
    { 0x4f0aecfb, SUC_lexicon_tags_25 },
    { 0xf5caecfc, SUC_lexicon_tags_11 },
    { 0xaeeaecfc, SUC_lexicon_tags_1 },
    { 0x39eaecfe, SUC_lexicon_tags_12 },
    { 0xb0eaecff, SUC_lexicon_tags_2 },
    { 0x9f4aecfe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x624aed02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680aed08, SUC_lexicon_tags_20 },
    { 0xa72aed09, SUC_lexicon_tags_25 },
    { 0xbe2aed0a, SUC_lexicon_tags_19 },
    { 0xf28aed0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8aed0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x15eaed10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeaed16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32aed19, SUC_lexicon_tags_0 },
    { 0xd8eaed1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80aed1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aed20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75caed2b, SUC_lexicon_tags_15 },
    { 0xa42aed2c, SUC_lexicon_tags_11 },
    { 0xf46aed2c, SUC_lexicon_tags_28 },
    { 0x272aed2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8aed31, SUC_lexicon_tags_33 },
    { 0xbf4aed32, SUC_lexicon_tags_21 },
    { 0x3e0aed33, SUC_lexicon_tags_11 },
    { 0xd5eaed34, SUC_lexicon_tags_84 },
    { 0x424aed35, SUC_lexicon_tags_11 },
    { 0xf18aed36, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc02aed38, SUC_lexicon_tags_63 },
    { 0x3a2aed38, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcaed3e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92aed41, SUC_lexicon_tags_45 },
    { 0xc52aed41, SUC_lexicon_tags_11 },
    { 0x194aed43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x476aed45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8aed48, SUC_lexicon_tags_13 },
    { 0xdbaaed49, SUC_lexicon_tags_60 },
    { 0x9d4aed4a, SUC_lexicon_tags_6 },
    { 0xf76aed4b, SUC_lexicon_tags_37 },
    { 0xd7aaed4a, SUC_lexicon_tags_13 },
    { 0xf5aaed4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56caed4f, SUC_lexicon_tags_10 },
    { 0x906aed4f, SUC_lexicon_tags_17 },
    { 0xf3aaed4f, SUC_lexicon_tags_47 },
    { 0x9aeaed52, SUC_lexicon_tags_2 },
    { 0x960aed53, SUC_lexicon_tags_2 },
    { 0xcc0aed53, SUC_lexicon_tags_23 },
    { 0x544aed4f, SUC_lexicon_tags_25 },
    { 0xb22aed50, SUC_lexicon_tags_17 },
    { 0x200aed57, SUC_lexicon_tags_32 },
    { 0x2bcaed52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x256aed5a, SUC_lexicon_tags_37 },
    { 0xc26aed5b, SUC_lexicon_tags_17 },
    { 0x1d2aed5b, SUC_lexicon_tags_37 },
    { 0x5ceaed5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81eaed60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56aed63, SUC_lexicon_tags_14 },
    { 0xe48aed64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc2aed66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x706aed69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eaed73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2aed7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4aed80, SUC_lexicon_tags_30 },
    { 0x2ceaed80, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98aed8d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660aed92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0aed97, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eaeda0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346aeda4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x910aeda6, SUC_lexicon_tags_3 },
    { 0x5b6aeda7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x598aedaa, SUC_lexicon_tags_25 },
    { 0xc34aedab, SUC_lexicon_tags_46 },
    { 0x286aedac, SUC_lexicon_tags_17 },
    { 0xd38aedac, SUC_lexicon_tags_62 },
    { 0xb48aedad, SUC_lexicon_tags_11 },
    { 0x31caedaa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1a8aedb1, SUC_lexicon_tags_4 },
    { 0x9a0aedb1, SUC_lexicon_tags_3 },
    { 0x936aedb3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a0aedb5, SUC_lexicon_tags_11 },
    { 0x1d6aedb5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x22aaedb8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3baaedbc, SUC_lexicon_tags_1 },
    { 0x62eaedbd, SUC_lexicon_tags_37 },
    { 0x196aedbe, SUC_lexicon_tags_377 },
    { 0xb9caedbc, SUC_lexicon_tags_33 },
    { 0xcaedc0, SUC_lexicon_tags_39 },
    { 0xe8caedc0, SUC_lexicon_tags_43 },
    { 0x9eaedbe, SUC_lexicon_tags_45 },
    { 0x1beaedc2, SUC_lexicon_tags_0 },
    { 0xd68aedc4, SUC_lexicon_tags_65 },
    { 0xf84aedc2, SUC_lexicon_tags_91 },
    { 0x460aedc6, SUC_lexicon_tags_70 },
    { 0xa68aedc7, SUC_lexicon_tags_48 },
    { 0x950aedc8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x62eaedca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6aedcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432aedd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafeaeddd, SUC_lexicon_tags_6 },
    { 0x5ecaeddd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61caede2, SUC_lexicon_tags_14 },
    { 0xd2caede3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60caede9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94aedec, SUC_lexicon_tags_4 },
    { 0xb78aedec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x796aedef, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6d8aedf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x478aedf3, SUC_lexicon_tags_23 },
    { 0x6dcaedf3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2aedf9, SUC_lexicon_tags_25 },
    { 0x378aedfa, SUC_lexicon_tags_63 },
    { 0x5aeaedfb, SUC_lexicon_tags_11 },
    { 0x6ceaedfb, SUC_lexicon_tags_29 },
    { 0x10aaedfc, SUC_lexicon_tags_69 },
    { 0x3e0aedfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f2aee00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeaee03, SUC_lexicon_tags_49 },
    { 0xb88aee04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x296aee07, SUC_lexicon_tags_31 },
    { 0xb5caee07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714aee0c, SUC_lexicon_tags_8 },
    { 0x5bcaee0c, SUC_lexicon_tags_2 },
    { 0x20eaee0e, SUC_lexicon_tags_158 },
    { 0xd8eaee0e, SUC_lexicon_tags_49 },
    { 0x704aee10, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8aee18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72aee1f, SUC_lexicon_tags_26 },
    { 0x17aaee1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88caee24, SUC_lexicon_tags_17 },
    { 0xd9caee25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826aee2b, SUC_lexicon_tags_35 },
    { 0x858aee2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdceaee2f, SUC_lexicon_tags_31 },
    { 0x93eaee30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x352aee33, SUC_lexicon_tags_11 },
    { 0xf6caee34, SUC_lexicon_tags_27 },
    { 0x870aee35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdceaee37, SUC_lexicon_tags_29 },
    { 0x428aee37, SUC_lexicon_tags_47 },
    { 0x60aaee38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18aee3c, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x746aee3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aaee43, SUC_lexicon_tags_29 },
    { 0x6f2aee44, SUC_lexicon_tags_37 },
    { 0x8b4aee45, SUC_lexicon_tags_555 },
    { 0x380aee44, SUC_lexicon_tags_124 },
    { 0xfd0aee47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0aee4c, SUC_lexicon_tags_15 },
    { 0x13aaee4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x474aee50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x882aee52, SUC_lexicon_tags_49 },
    { 0xba2aee53, SUC_lexicon_tags_2 },
    { 0x558aee52, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28aee57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x562aee5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4aaee5d, SUC_lexicon_tags_57 },
    { 0xb48aee5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcaee62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eaee68, SUC_lexicon_tags_27 },
    { 0x474aee69, SUC_lexicon_tags_15 },
    { 0x67aaee6a, SUC_lexicon_tags_27 },
    { 0xb5aaee6b, SUC_lexicon_tags_25 },
    { 0x8aaaee68, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaa4aee6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4caee71, SUC_lexicon_tags_104 },
    { 0xadcaee72, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xa68aee74, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10aee7a, SUC_lexicon_tags_8 },
    { 0xcb6aee7a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2aee7e, SUC_lexicon_tags_26 },
    { 0xd44aee7f, SUC_lexicon_tags_27 },
    { 0xeaeaee80, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6aee84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feaee88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6aee8d, SUC_lexicon_tags_1 },
    { 0x328aee8d, SUC_lexicon_tags_25 },
    { 0xa1caee8d, SUC_lexicon_tags_32 },
    { 0xa46aee90, SUC_lexicon_tags_11 },
    { 0xd12aee91, SUC_lexicon_tags_6 },
    { 0x1a4aee90, SUC_lexicon_tags_14 },
    { 0x7a2aee93, SUC_lexicon_tags_63 },
    { 0x630aee93, SUC_lexicon_tags_27 },
    { 0x5aaaee95, SUC_lexicon_tags_26 },
    { 0x3eeaee96, SUC_lexicon_tags_1 },
    { 0xef2aee96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82caee99, SUC_lexicon_tags_28 },
    { 0x748aee9a, SUC_lexicon_tags_7 },
    { 0xed6aee99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0aeea3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38aeea6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72eaeea8, SUC_lexicon_tags_37 },
    { 0x9aeaeea9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8eaeeab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23caeead, SUC_lexicon_tags_56 },
    { 0xdceaeeae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616aeeb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42aeeb7, SUC_lexicon_tags_63 },
    { 0x10aeeb7, SUC_lexicon_tags_12 },
    { 0xc3aaeeb9, SUC_lexicon_tags_17 },
    { 0x97aaeeba, SUC_lexicon_tags_0 },
    { 0xce8aeeb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1aaeebd, SUC_lexicon_tags_25 },
    { 0xd24aeebd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1eaeec2, SUC_lexicon_tags_15 },
    { 0xa40aeec2, SUC_lexicon_tags_15 },
    { 0x32eaeec4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xffaaeec6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x72eaeec9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff8aeecb, SUC_lexicon_tags_81 },
    { 0xd6caeecc, SUC_lexicon_tags_606 },
    { 0x198aeecc, SUC_lexicon_tags_28 },
    { 0xda4aeecd, SUC_lexicon_tags_37 },
    { 0x5bcaeecf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaaeed6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0aeed9, SUC_lexicon_tags_23 },
    { 0x424aeeda, SUC_lexicon_tags_2 },
    { 0xaeedb, SUC_lexicon_tags_7 },
    { 0x24aeeda, SUC_lexicon_tags_47 },
    { 0x1d2aeedd, SUC_lexicon_tags_66 },
    { 0x662aeedd, SUC_lexicon_tags_40 },
    { 0xdd4aeedd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68aaeee1, SUC_lexicon_tags_31 },
    { 0xa8eaeee1, SUC_lexicon_tags_17 },
    { 0xe6caeee2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7eaeee6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x40aeee9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x86caeeeb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4baaeef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caeefd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x494aeeff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x340aef01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92aef03, SUC_lexicon_tags_2 },
    { 0x8caaef04, SUC_lexicon_tags_17 },
    { 0x7baaef04, SUC_lexicon_tags_15 },
    { 0xf94aef06, SUC_lexicon_tags_49 },
    { 0x4b2aef07, SUC_lexicon_tags_12 },
    { 0x766aef08, SUC_lexicon_tags_2 },
    { 0xcccaef09, SUC_lexicon_tags_1 },
    { 0x2faaef0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x640aef0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0aef10, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2aef15, SUC_lexicon_tags_26 },
    { 0xd66aef16, SUC_lexicon_tags_49 },
    { 0x98aef17, SUC_lexicon_tags_31 },
    { 0x4d8aef15, SUC_lexicon_tags_30 },
    { 0x6eeaef19, SUC_lexicon_tags_64 },
    { 0x7aaaef18, SUC_lexicon_tags_1 },
    { 0xe2eaef17, SUC_lexicon_tags_16 },
    { 0x848aef18, SUC_lexicon_tags_30 },
    { 0x5b4aef1d, SUC_lexicon_tags_8 },
    { 0xebcaef1d, SUC_lexicon_tags_49 },
    { 0xd22aef1f, SUC_lexicon_tags_9 },
    { 0xab6aef1f, SUC_lexicon_tags_2 },
    { 0x5d2aef1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe34aef23, SUC_lexicon_tags_68 },
    { 0x9b8aef23, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36aef2a, SUC_lexicon_tags_38 },
    { 0xcb4aef2b, SUC_lexicon_tags_11 },
    { 0x302aef2c, SUC_lexicon_tags_13 },
    { 0xebcaef2d, SUC_lexicon_tags_77 },
    { 0x836aef2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52aef33, SUC_lexicon_tags_2 },
    { 0x70eaef34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f6aef36, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xddeaef38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60caef3a, SUC_lexicon_tags_12 },
    { 0x16caef3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b8aef3d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3beaef3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x25caef41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75eaef43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfaaaef45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4aef48, SUC_lexicon_tags_12 },
    { 0x412aef49, SUC_lexicon_tags_28 },
    { 0x3d2aef4a, SUC_lexicon_tags_48 },
    { 0x51aaef49, SUC_lexicon_tags_1 },
    { 0xaaeaef48, SUC_lexicon_tags_37 },
    { 0xac2aef4d, SUC_lexicon_tags_1 },
    { 0x836aef4a, SUC_lexicon_tags_27 },
    { 0xa6aef49, SUC_lexicon_tags_25 },
    { 0xb90aef4f, SUC_lexicon_tags_45 },
    { 0xdd4aef51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6eaef53, SUC_lexicon_tags_55 },
    { 0xcbeaef54, SUC_lexicon_tags_45 },
    { 0xac2aef55, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596aef59, SUC_lexicon_tags_19 },
    { 0x31aaef59, SUC_lexicon_tags_11 },
    { 0xa0caef5b, SUC_lexicon_tags_37 },
    { 0xf6caef5b, SUC_lexicon_tags_109 },
    { 0xb76aef5c, SUC_lexicon_tags_10 },
    { 0x546aef5a, SUC_lexicon_tags_11 },
    { 0x77aaef5f, SUC_lexicon_tags_15 },
    { 0xce6aef59, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0aef66, SUC_lexicon_tags_17 },
    { 0xca2aef67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7caef69, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x224aef6b, SUC_lexicon_tags_7 },
    { 0xa9eaef6c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x460aef6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acaef73, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x142aef76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3caef78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x668aef7b, SUC_lexicon_tags_32 },
    { 0xcdeaef7b, SUC_lexicon_tags_1 },
    { 0xf48aef7b, SUC_lexicon_tags_32 },
    { 0x27aaef7e, SUC_lexicon_tags_23 },
    { 0xdb0aef7f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6aef83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aaef89, SUC_lexicon_tags_1 },
    { 0x450aef8a, SUC_lexicon_tags_2 },
    { 0x908aef8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8eaef8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14aef94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aef97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38aaef9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10aef9d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x36aef9f, SUC_lexicon_tags_15 },
    { 0xd40aef9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x422aefa3, SUC_lexicon_tags_37 },
    { 0x6faaefa3, SUC_lexicon_tags_28 },
    { 0xcd0aefa5, SUC_lexicon_tags_25 },
    { 0x80aefa6, SUC_lexicon_tags_45 },
    { 0xe5caefa7, SUC_lexicon_tags_16 },
    { 0x65aaefa6, SUC_lexicon_tags_2 },
    { 0xf42aefa7, SUC_lexicon_tags_17 },
    { 0x3ccaefaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0aefad, SUC_lexicon_tags_25 },
    { 0xea6aefae, SUC_lexicon_tags_27 },
    { 0x514aefaf, SUC_lexicon_tags_9 },
    { 0xe48aefb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5feaefb2, SUC_lexicon_tags_9 },
    { 0xa64aefb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0aefb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x476aefb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890aefbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x746aefc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eaefc9, SUC_lexicon_tags_11 },
    { 0x912aefca, SUC_lexicon_tags_25 },
    { 0xe9eaefc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450aefd1, SUC_lexicon_tags_7 },
    { 0xfa6aefd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5feaefd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x566aefd7, SUC_lexicon_tags_4 },
    { 0x956aefd8, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x1a4aefda, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2eaefde, SUC_lexicon_tags_25 },
    { 0xdaeaefde, SUC_lexicon_tags_33 },
    { 0x4ecaefe0, SUC_lexicon_tags_28 },
    { 0xd6eaefe0, SUC_lexicon_tags_8 },
    { 0xb72aefde, SUC_lexicon_tags_66 },
    { 0x644aefe3, SUC_lexicon_tags_19 },
    { 0xc9aaefe3, SUC_lexicon_tags_19 },
    { 0xee8aefe5, SUC_lexicon_tags_23 },
    { 0x3b8aefe6, SUC_lexicon_tags_37 },
    { 0x3b6aefe7, SUC_lexicon_tags_26 },
    { 0x828aefe4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b6aefea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x932aefec, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa7aaefee, SUC_lexicon_tags_14 },
    { 0xe7aaefef, SUC_lexicon_tags_2 },
    { 0xaceaefef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0aeff2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1caeff8, SUC_lexicon_tags_15 },
    { 0x1baaeff9, SUC_lexicon_tags_11 },
    { 0x68eaeffa, SUC_lexicon_tags_1 },
    { 0x950aeffa, SUC_lexicon_tags_25 },
    { 0xb10aeff9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eaefff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72af003, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40caf007, SUC_lexicon_tags_4 },
    { 0x972af008, SUC_lexicon_tags_12 },
    { 0x400af009, SUC_lexicon_tags_63 },
    { 0x456af009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4caf00e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6af012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89caf014, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96af017, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xec4af019, SUC_lexicon_tags_63 },
    { 0x594af019, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f2af01c, SUC_lexicon_tags_17 },
    { 0x358af01c, SUC_lexicon_tags_46 },
    { 0xfd6af01c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42af023, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12af033, SUC_lexicon_tags_7 },
    { 0x5acaf034, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6af037, SUC_lexicon_tags_65 },
    { 0xd92af038, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8eaf03a, SUC_lexicon_tags_13 },
    { 0x93aaf03a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8caaf03d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9e6af03f, SUC_lexicon_tags_33 },
    { 0x41aaf03f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44af044, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eaf047, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63aaf049, SUC_lexicon_tags_14 },
    { 0xaaeaf049, SUC_lexicon_tags_5 },
    { 0x4aaaf04b, SUC_lexicon_tags_11 },
    { 0x80af04b, SUC_lexicon_tags_1 },
    { 0x85caf049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962af052, SUC_lexicon_tags_17 },
    { 0xf4af053, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x142af055, SUC_lexicon_tags_12 },
    { 0x130af056, SUC_lexicon_tags_37 },
    { 0xe62af056, SUC_lexicon_tags_1 },
    { 0x7e0af055, SUC_lexicon_tags_8 },
    { 0x3b0af055, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa00af05b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2caf05d, SUC_lexicon_tags_15 },
    { 0x87aaf05d, SUC_lexicon_tags_12 },
    { 0xed4af05d, SUC_lexicon_tags_31 },
    { 0x48caf060, SUC_lexicon_tags_10 },
    { 0x9f2af061, SUC_lexicon_tags_11 },
    { 0xe38af061, SUC_lexicon_tags_11 },
    { 0x13caf05f, SUC_lexicon_tags_31 },
    { 0x698af064, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83aaf066, SUC_lexicon_tags_30 },
    { 0x3ccaf067, SUC_lexicon_tags_43 },
    { 0x3b8af068, SUC_lexicon_tags_32 },
    { 0xe88af069, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x300af06b, SUC_lexicon_tags_12 },
    { 0x7b4af06c, SUC_lexicon_tags_1 },
    { 0x46af06b, SUC_lexicon_tags_2 },
    { 0x8eeaf06e, SUC_lexicon_tags_9 },
    { 0x6deaf06f, SUC_lexicon_tags_2 },
    { 0x870af06f, SUC_lexicon_tags_57 },
    { 0x908af070, SUC_lexicon_tags_262 },
    { 0x34eaf06f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8af076, SUC_lexicon_tags_26 },
    { 0xa0af077, SUC_lexicon_tags_70 },
    { 0xd06af078, SUC_lexicon_tags_192 },
    { 0x746af077, SUC_lexicon_tags_17 },
    { 0x9f4af078, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eaf07d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2eaf07f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66af086, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccaf089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc6af08b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4af092, SUC_lexicon_tags_25 },
    { 0xdd6af093, SUC_lexicon_tags_28 },
    { 0x64aaf094, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecaf098, SUC_lexicon_tags_37 },
    { 0x3c4af099, SUC_lexicon_tags_28 },
    { 0x258af09a, SUC_lexicon_tags_25 },
    { 0x2b6af09b, SUC_lexicon_tags_11 },
    { 0x7bcaf09b, SUC_lexicon_tags_37 },
    { 0x2eaaf09d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4af0a4, SUC_lexicon_tags_17 },
    { 0x664af0a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60eaf0ab, SUC_lexicon_tags_25 },
    { 0x444af0ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4af0af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2af0b2, SUC_lexicon_tags_57 },
    { 0x312af0b3, SUC_lexicon_tags_37 },
    { 0x2f4af0b4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x366af0b6, SUC_lexicon_tags_49 },
    { 0xe12af0b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778af0c2, SUC_lexicon_tags_4 },
    { 0xdf0af0c3, SUC_lexicon_tags_11 },
    { 0x7feaf0c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x578af0c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa40af0c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd06af0ca, SUC_lexicon_tags_2 },
    { 0xa30af0ca, SUC_lexicon_tags_487 },
    { 0x478af0ca, SUC_lexicon_tags_19 },
    { 0x31caf0cb, SUC_lexicon_tags_1 },
    { 0x56eaf0cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2af0d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x394af0d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b6af0d7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x512af0da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd6af0dc, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684af0e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd7caf0e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x636af0e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eaf0ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130af0f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc2af0f7, SUC_lexicon_tags_45 },
    { 0xee6af0f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb72af0fa, SUC_lexicon_tags_37 },
    { 0xf82af0fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x728af0fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3caaf0ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ceaf101, SUC_lexicon_tags_17 },
    { 0x75aaf101, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd9eaf104, SUC_lexicon_tags_57 },
    { 0x7eeaf104, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a8af107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadcaf10c, SUC_lexicon_tags_37 },
    { 0xadcaf10d, SUC_lexicon_tags_38 },
    { 0xcdeaf10d, SUC_lexicon_tags_2 },
    { 0x4d8af10f, SUC_lexicon_tags_5 },
    { 0x674af110, SUC_lexicon_tags_27 },
    { 0x67aaf111, SUC_lexicon_tags_17 },
    { 0x814af112, SUC_lexicon_tags_41 },
    { 0xf62af113, SUC_lexicon_tags_43 },
    { 0x7eaf114, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x120af117, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x58aaf11a, SUC_lexicon_tags_12 },
    { 0x4acaf11b, SUC_lexicon_tags_1 },
    { 0xb2caf11b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x182af11f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x28caf122, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x722af124, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3fcaf126, SUC_lexicon_tags_9 },
    { 0xfaaaf127, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x5a4af129, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6f4af12b, SUC_lexicon_tags_37 },
    { 0x112af12b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xabaaf12f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x492af132, SUC_lexicon_tags_0 },
    { 0x91eaf133, SUC_lexicon_tags_12 },
    { 0x7d0af133, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656af138, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28eaf13b, SUC_lexicon_tags_25 },
    { 0x854af13c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x824af13e, SUC_lexicon_tags_1 },
    { 0xb28af13f, SUC_lexicon_tags_25 },
    { 0xcd6af140, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586af146, SUC_lexicon_tags_31 },
    { 0x334af147, SUC_lexicon_tags_45 },
    { 0xcbcaf147, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x940af14a, SUC_lexicon_tags_25 },
    { 0x6dcaf14a, SUC_lexicon_tags_17 },
    { 0xce0af14b, SUC_lexicon_tags_149 },
    { 0x398af14d, SUC_lexicon_tags_37 },
    { 0x272af14e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77caf150, SUC_lexicon_tags_34 },
    { 0x42af151, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26af155, SUC_lexicon_tags_27 },
    { 0x520af156, SUC_lexicon_tags_6 },
    { 0x76aaf156, SUC_lexicon_tags_25 },
    { 0xaeeaf155, SUC_lexicon_tags_11 },
    { 0xd6caf159, SUC_lexicon_tags_2 },
    { 0x3f4af15a, SUC_lexicon_tags_11 },
    { 0xa60af15a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb2af15d, SUC_lexicon_tags_367 },
    { 0, NULL },
    { 0x2b0af15f, SUC_lexicon_tags_12 },
    { 0x3d6af160, SUC_lexicon_tags_57 },
    { 0xcc0af161, SUC_lexicon_tags_3 },
    { 0x6b6af160, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aaaf165, SUC_lexicon_tags_15 },
    { 0xe52af166, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6af169, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x262af16b, SUC_lexicon_tags_15 },
    { 0x6a2af16c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aaf16f, SUC_lexicon_tags_11 },
    { 0xd7eaf170, SUC_lexicon_tags_11 },
    { 0x654af170, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x514af173, SUC_lexicon_tags_8 },
    { 0xb26af174, SUC_lexicon_tags_29 },
    { 0x116af175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x556af177, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc16af179, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8af17c, SUC_lexicon_tags_6 },
    { 0x3beaf17d, SUC_lexicon_tags_22 },
    { 0x860af17d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x280af181, SUC_lexicon_tags_11 },
    { 0x5d8af182, SUC_lexicon_tags_7 },
    { 0x7ceaf181, SUC_lexicon_tags_25 },
    { 0x10caf181, SUC_lexicon_tags_42 },
    { 0x88aaf185, SUC_lexicon_tags_45 },
    { 0x43eaf186, SUC_lexicon_tags_37 },
    { 0x830af187, SUC_lexicon_tags_37 },
    { 0xf4eaf188, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x35eaf18a, SUC_lexicon_tags_0 },
    { 0xf12af18b, SUC_lexicon_tags_28 },
    { 0x8f6af18c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x494af18e, SUC_lexicon_tags_27 },
    { 0xa88af18f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76af196, SUC_lexicon_tags_7 },
    { 0x2c0af197, SUC_lexicon_tags_25 },
    { 0x6f2af198, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf2af19a, SUC_lexicon_tags_2 },
    { 0xb80af19b, SUC_lexicon_tags_63 },
    { 0x5a0af19a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6af1a8, SUC_lexicon_tags_12 },
    { 0x3c2af1a8, SUC_lexicon_tags_2 },
    { 0x2d4af1a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8af1bc, SUC_lexicon_tags_33 },
    { 0xbf6af1bd, SUC_lexicon_tags_13 },
    { 0xacaf1be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56af1c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7caaf1c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5b4af1c5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6af1ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938af1ce, SUC_lexicon_tags_280 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9eaf1e0, SUC_lexicon_tags_28 },
    { 0x33eaf1e1, SUC_lexicon_tags_30 },
    { 0xe44af1e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60eaf1e5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x672af1e8, SUC_lexicon_tags_15 },
    { 0x7d0af1e8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112af1f3, SUC_lexicon_tags_156 },
    { 0x57aaf1f3, SUC_lexicon_tags_25 },
    { 0x4daaf1f5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2af205, SUC_lexicon_tags_28 },
    { 0x9a6af206, SUC_lexicon_tags_17 },
    { 0xc9caf207, SUC_lexicon_tags_37 },
    { 0x4d4af207, SUC_lexicon_tags_15 },
    { 0xf16af209, SUC_lexicon_tags_1 },
    { 0xba6af20a, SUC_lexicon_tags_27 },
    { 0xc3aaf20b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32af20e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722af213, SUC_lexicon_tags_0 },
    { 0x3c2af214, SUC_lexicon_tags_125 },
    { 0xe16af213, SUC_lexicon_tags_17 },
    { 0x1c2af214, SUC_lexicon_tags_15 },
    { 0x122af216, SUC_lexicon_tags_25 },
    { 0x714af218, SUC_lexicon_tags_25 },
    { 0xcd6af215, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80aaf222, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaf225, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x414af228, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3beaf22b, SUC_lexicon_tags_12 },
    { 0x9f4af22b, SUC_lexicon_tags_11 },
    { 0x73aaf22d, SUC_lexicon_tags_26 },
    { 0xfa8af22e, SUC_lexicon_tags_46 },
    { 0xa34af22d, SUC_lexicon_tags_1 },
    { 0x9e4af22f, SUC_lexicon_tags_10 },
    { 0x9d2af231, SUC_lexicon_tags_51 },
    { 0x924af231, SUC_lexicon_tags_25 },
    { 0x44caf232, SUC_lexicon_tags_33 },
    { 0xbfcaf234, SUC_lexicon_tags_6 },
    { 0x8c0af22f, SUC_lexicon_tags_11 },
    { 0x2acaf235, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4af23d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5aaf240, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa6caf242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f8af244, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcaf247, SUC_lexicon_tags_63 },
    { 0xa2aaf248, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aaf254, SUC_lexicon_tags_47 },
    { 0x774af255, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9ceaf257, SUC_lexicon_tags_1 },
    { 0x8e2af257, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10af25d, SUC_lexicon_tags_12 },
    { 0xbb8af25e, SUC_lexicon_tags_15 },
    { 0x472af25f, SUC_lexicon_tags_33 },
    { 0xc26af260, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x526af262, SUC_lexicon_tags_1 },
    { 0x83caf263, SUC_lexicon_tags_94 },
    { 0xcdeaf264, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd0af266, SUC_lexicon_tags_17 },
    { 0x26aaf267, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcd2af269, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eaf26d, SUC_lexicon_tags_33 },
    { 0x25eaf26d, SUC_lexicon_tags_15 },
    { 0x42aaf26f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8af273, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18af276, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcaf279, SUC_lexicon_tags_14 },
    { 0x78af27a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3beaf27d, SUC_lexicon_tags_25 },
    { 0xc08af27d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72caf283, SUC_lexicon_tags_6 },
    { 0xc4af284, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x68caf286, SUC_lexicon_tags_1 },
    { 0xb5aaf287, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95eaf289, SUC_lexicon_tags_29 },
    { 0x16aaf289, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x10eaf28d, SUC_lexicon_tags_37 },
    { 0xa22af28d, SUC_lexicon_tags_92 },
    { 0x258af28e, SUC_lexicon_tags_7 },
    { 0x512af28e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19eaf294, SUC_lexicon_tags_29 },
    { 0xd14af295, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36af29a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xde6af29c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6af2a0, SUC_lexicon_tags_15 },
    { 0xc22af2a1, SUC_lexicon_tags_48 },
    { 0x22eaf2a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862af2a6, SUC_lexicon_tags_2 },
    { 0x5a6af2a7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86af2ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9eaf2b0, SUC_lexicon_tags_2 },
    { 0x91eaf2b1, SUC_lexicon_tags_17 },
    { 0x310af2b1, SUC_lexicon_tags_48 },
    { 0x482af2b0, SUC_lexicon_tags_8 },
    { 0x7deaf2b3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x9beaf2b7, SUC_lexicon_tags_62 },
    { 0xdceaf2b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4deaf2bb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422af2c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21caf2c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04af2cb, SUC_lexicon_tags_4 },
    { 0xc1caf2cb, SUC_lexicon_tags_46 },
    { 0xfe8af2cd, SUC_lexicon_tags_79 },
    { 0xb2eaf2cd, SUC_lexicon_tags_6 },
    { 0x226af2cb, SUC_lexicon_tags_11 },
    { 0x724af2cd, SUC_lexicon_tags_11 },
    { 0x356af2cc, SUC_lexicon_tags_60 },
    { 0x456af2ce, SUC_lexicon_tags_26 },
    { 0xcaf2cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af2d7, SUC_lexicon_tags_17 },
    { 0xcacaf2d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4af2dc, SUC_lexicon_tags_66 },
    { 0x6d2af2dd, SUC_lexicon_tags_1 },
    { 0xea6af2dc, SUC_lexicon_tags_93 },
    { 0xfcaaf2df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x348af2e2, SUC_lexicon_tags_2 },
    { 0xb0eaf2e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x924af2e5, SUC_lexicon_tags_69 },
    { 0x644af2e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70af2e9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6baaf2eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8af2ee, SUC_lexicon_tags_2 },
    { 0xbecaf2ef, SUC_lexicon_tags_55 },
    { 0xbacaf2ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2af2f3, SUC_lexicon_tags_1 },
    { 0x280af2f4, SUC_lexicon_tags_37 },
    { 0x7eaf2f3, SUC_lexicon_tags_201 },
    { 0x712af2f5, SUC_lexicon_tags_7 },
    { 0xfe8af2f7, SUC_lexicon_tags_45 },
    { 0x68caf2f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62af300, SUC_lexicon_tags_11 },
    { 0xc2caf301, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48af304, SUC_lexicon_tags_336 },
    { 0xb76af305, SUC_lexicon_tags_14 },
    { 0xd64af306, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114af30a, SUC_lexicon_tags_29 },
    { 0xafcaf30b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c2af30d, SUC_lexicon_tags_2 },
    { 0x9b6af30e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac2af310, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x814af312, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92caf314, SUC_lexicon_tags_15 },
    { 0xe7eaf315, SUC_lexicon_tags_47 },
    { 0x9f2af314, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0af31c, SUC_lexicon_tags_46 },
    { 0x64eaf31c, SUC_lexicon_tags_2 },
    { 0xd3eaf31d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcaf325, SUC_lexicon_tags_33 },
    { 0x7daaf326, SUC_lexicon_tags_69 },
    { 0x1daaf327, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8f6af329, SUC_lexicon_tags_2 },
    { 0x5f2af32a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaf32e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1acaf331, SUC_lexicon_tags_12 },
    { 0xb38af332, SUC_lexicon_tags_11 },
    { 0xca4af333, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x266af335, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaaf338, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99eaf33b, SUC_lexicon_tags_168 },
    { 0, NULL },
    { 0xd36af33d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x28af33f, SUC_lexicon_tags_43 },
    { 0xdeaaf33f, SUC_lexicon_tags_43 },
    { 0x930af341, SUC_lexicon_tags_12 },
    { 0xed6af341, SUC_lexicon_tags_1 },
    { 0x6feaf343, SUC_lexicon_tags_47 },
    { 0x45eaf344, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x540af347, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x60caf349, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68af34b, SUC_lexicon_tags_23 },
    { 0x8caf34b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcaf34f, SUC_lexicon_tags_17 },
    { 0x31eaf34f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccaf359, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa5caf35b, SUC_lexicon_tags_62 },
    { 0x7d2af35b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3deaf35e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914af362, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabeaf367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6af378, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4af37b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daaf380, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0af38c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2af38f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3caf392, SUC_lexicon_tags_10 },
    { 0xde2af393, SUC_lexicon_tags_23 },
    { 0xd80af394, SUC_lexicon_tags_63 },
    { 0x20af394, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6af398, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4af39b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d4af39d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14af3a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaf3ab, SUC_lexicon_tags_30 },
    { 0x7a8af3ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x710af3ae, SUC_lexicon_tags_37 },
    { 0xbdeaf3af, SUC_lexicon_tags_12 },
    { 0x2d2af3b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf20af3b2, SUC_lexicon_tags_12 },
    { 0x93caf3b3, SUC_lexicon_tags_97 },
    { 0xf4caf3b4, SUC_lexicon_tags_6 },
    { 0x5b8af3b4, SUC_lexicon_tags_33 },
    { 0x7beaf3b6, SUC_lexicon_tags_2 },
    { 0x338af3b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac6af3b9, SUC_lexicon_tags_2 },
    { 0x602af3ba, SUC_lexicon_tags_38 },
    { 0xd12af3bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x588af3bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66af3c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x254af3c7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4af3ce, SUC_lexicon_tags_14 },
    { 0x440af3cf, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48af3d2, SUC_lexicon_tags_12 },
    { 0x568af3d2, SUC_lexicon_tags_42 },
    { 0xdfaaf3d2, SUC_lexicon_tags_14 },
    { 0xfc4af3d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0af3db, SUC_lexicon_tags_40 },
    { 0x76eaf3db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe16af3de, SUC_lexicon_tags_4 },
    { 0xe48af3df, SUC_lexicon_tags_27 },
    { 0xbb2af3e0, SUC_lexicon_tags_0 },
    { 0xc50af3e1, SUC_lexicon_tags_19 },
    { 0x52eaf3e2, SUC_lexicon_tags_37 },
    { 0x110af3e1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x36aaf3e5, SUC_lexicon_tags_37 },
    { 0x4e0af3e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x34caf3e8, SUC_lexicon_tags_17 },
    { 0x7c4af3e9, SUC_lexicon_tags_0 },
    { 0xc3caf3ea, SUC_lexicon_tags_45 },
    { 0x134af3e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2f0af3ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6af3f0, SUC_lexicon_tags_82 },
    { 0x83eaf3f1, SUC_lexicon_tags_11 },
    { 0x95eaf3f1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6af3f5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf9aaf3f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x278af3fa, SUC_lexicon_tags_37 },
    { 0x57caf3fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x506af3fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x954af400, SUC_lexicon_tags_37 },
    { 0xaeaaf401, SUC_lexicon_tags_12 },
    { 0x402af402, SUC_lexicon_tags_1 },
    { 0x76aaf403, SUC_lexicon_tags_32 },
    { 0xd44af404, SUC_lexicon_tags_151 },
    { 0x714af404, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf6af407, SUC_lexicon_tags_174 },
    { 0xe26af407, SUC_lexicon_tags_19 },
    { 0x8c4af409, SUC_lexicon_tags_43 },
    { 0x3faaf408, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812af411, SUC_lexicon_tags_12 },
    { 0x3c8af411, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04af415, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00af420, SUC_lexicon_tags_37 },
    { 0xb74af421, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f4af423, SUC_lexicon_tags_44 },
    { 0xc7caf424, SUC_lexicon_tags_124 },
    { 0x4deaf425, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9caf42a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbaeaf42c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x95caf42e, SUC_lexicon_tags_17 },
    { 0x30af42f, SUC_lexicon_tags_11 },
    { 0x7deaf42f, SUC_lexicon_tags_0 },
    { 0x314af431, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64af436, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2caf43c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecaf444, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedaaf446, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa8af448, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4af44d, SUC_lexicon_tags_48 },
    { 0xdb8af44d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78af450, SUC_lexicon_tags_47 },
    { 0x872af451, SUC_lexicon_tags_11 },
    { 0x8acaf451, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaaf456, SUC_lexicon_tags_11 },
    { 0x8b0af457, SUC_lexicon_tags_15 },
    { 0xa30af458, SUC_lexicon_tags_57 },
    { 0xd92af459, SUC_lexicon_tags_14 },
    { 0x17caf456, SUC_lexicon_tags_12 },
    { 0xc90af45b, SUC_lexicon_tags_63 },
    { 0xa6af45c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caf461, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x434af464, SUC_lexicon_tags_7 },
    { 0x73eaf464, SUC_lexicon_tags_27 },
    { 0x93aaf466, SUC_lexicon_tags_12 },
    { 0xe48af466, SUC_lexicon_tags_29 },
    { 0xe08af468, SUC_lexicon_tags_1 },
    { 0x814af464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76af46e, SUC_lexicon_tags_3 },
    { 0x880af46f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50af473, SUC_lexicon_tags_17 },
    { 0x382af473, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4bcaf476, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccaf47a, SUC_lexicon_tags_27 },
    { 0x5eeaf47b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x972af47d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608af485, SUC_lexicon_tags_37 },
    { 0x8eaf486, SUC_lexicon_tags_23 },
    { 0x182af487, SUC_lexicon_tags_38 },
    { 0xc8af487, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aaf48b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x98caf48e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0af491, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8af494, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334af499, SUC_lexicon_tags_19 },
    { 0x6a4af49a, SUC_lexicon_tags_11 },
    { 0x1c4af499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1aaf4a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb5caf4a8, SUC_lexicon_tags_55 },
    { 0xafaaf4a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6af4af, SUC_lexicon_tags_55 },
    { 0x5a6af4af, SUC_lexicon_tags_1 },
    { 0x64af4af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeceaf4b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62af4b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2feaf4ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa10af4bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x52eaf4bf, SUC_lexicon_tags_77 },
    { 0xa28af4c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc2af4c2, SUC_lexicon_tags_12 },
    { 0xc18af4c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaeaf4c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08af4cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28af4cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6af4d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x678af4d6, SUC_lexicon_tags_1 },
    { 0x992af4d7, SUC_lexicon_tags_17 },
    { 0x2faaf4d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x870af4da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4caf4dd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x94aaf4e0, SUC_lexicon_tags_13 },
    { 0x2b0af4e0, SUC_lexicon_tags_8 },
    { 0x704af4e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x348af4e4, SUC_lexicon_tags_37 },
    { 0xf36af4e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaaf4e9, SUC_lexicon_tags_12 },
    { 0x280af4ea, SUC_lexicon_tags_12 },
    { 0xa8eaf4eb, SUC_lexicon_tags_37 },
    { 0x5b8af4ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e4af4ee, SUC_lexicon_tags_45 },
    { 0xa40af4ee, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb8caf4f1, SUC_lexicon_tags_91 },
    { 0xc86af4f1, SUC_lexicon_tags_40 },
    { 0xa96af4f3, SUC_lexicon_tags_17 },
    { 0x936af4f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d4af4f6, SUC_lexicon_tags_37 },
    { 0x758af4f7, SUC_lexicon_tags_45 },
    { 0xed8af4f8, SUC_lexicon_tags_17 },
    { 0x5acaf4f8, SUC_lexicon_tags_0 },
    { 0xc98af4fa, SUC_lexicon_tags_68 },
    { 0xce8af4f7, SUC_lexicon_tags_25 },
    { 0xca6af4fc, SUC_lexicon_tags_49 },
    { 0xde4af4fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55eaf504, SUC_lexicon_tags_542 },
    { 0x624af505, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802af50e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20eaf510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x986af512, SUC_lexicon_tags_27 },
    { 0xdccaf513, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x500af515, SUC_lexicon_tags_26 },
    { 0x1baaf515, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x46caf518, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192af51c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46af524, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcaf527, SUC_lexicon_tags_32 },
    { 0xed8af528, SUC_lexicon_tags_11 },
    { 0x9ccaf529, SUC_lexicon_tags_37 },
    { 0x28aaf52a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x276af52d, SUC_lexicon_tags_11 },
    { 0xb6aaf52d, SUC_lexicon_tags_2 },
    { 0x828af52f, SUC_lexicon_tags_204 },
    { 0xcdcaf52f, SUC_lexicon_tags_27 },
    { 0x34eaf531, SUC_lexicon_tags_37 },
    { 0x3e2af531, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11aaf536, SUC_lexicon_tags_37 },
    { 0xf7caf537, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8aaf539, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x874af53b, SUC_lexicon_tags_66 },
    { 0x8faaf53c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x782af53f, SUC_lexicon_tags_68 },
    { 0xaeaaf540, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe4af542, SUC_lexicon_tags_11 },
    { 0x17aaf543, SUC_lexicon_tags_27 },
    { 0x9fcaf542, SUC_lexicon_tags_79 },
    { 0x242af545, SUC_lexicon_tags_62 },
    { 0x7c4af545, SUC_lexicon_tags_7 },
    { 0xf62af545, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8af54a, SUC_lexicon_tags_4 },
    { 0x3ccaf54b, SUC_lexicon_tags_47 },
    { 0xdfeaf54c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3daaf551, SUC_lexicon_tags_2 },
    { 0xfb8af552, SUC_lexicon_tags_16 },
    { 0x774af553, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6af557, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7deaf55b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x954af55d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c2af55f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2aaaf561, SUC_lexicon_tags_26 },
    { 0x8d6af562, SUC_lexicon_tags_1 },
    { 0xc6af562, SUC_lexicon_tags_47 },
    { 0x674af562, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3caf56a, SUC_lexicon_tags_47 },
    { 0x8baaf56a, SUC_lexicon_tags_198 },
    { 0x71eaf56a, SUC_lexicon_tags_25 },
    { 0xba0af56d, SUC_lexicon_tags_29 },
    { 0x17caf56d, SUC_lexicon_tags_15 },
    { 0x400af56f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaaf572, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2af576, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52af57e, SUC_lexicon_tags_7 },
    { 0xbb0af57f, SUC_lexicon_tags_11 },
    { 0xdc8af57f, SUC_lexicon_tags_11 },
    { 0x97aaf580, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8af58a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33caf58d, SUC_lexicon_tags_7 },
    { 0x54aaf58e, SUC_lexicon_tags_63 },
    { 0xc4eaf58e, SUC_lexicon_tags_10 },
    { 0xbf0af590, SUC_lexicon_tags_39 },
    { 0xa80af591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8af594, SUC_lexicon_tags_2 },
    { 0xc9aaf595, SUC_lexicon_tags_12 },
    { 0xbcaaf596, SUC_lexicon_tags_17 },
    { 0x7f6af596, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x692af59a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e4af59c, SUC_lexicon_tags_19 },
    { 0x50af59d, SUC_lexicon_tags_1 },
    { 0x8c0af59e, SUC_lexicon_tags_10 },
    { 0x462af59f, SUC_lexicon_tags_49 },
    { 0x4faaf5a0, SUC_lexicon_tags_2 },
    { 0x6a4af5a1, SUC_lexicon_tags_68 },
    { 0xe02af5a2, SUC_lexicon_tags_60 },
    { 0x400af5a1, SUC_lexicon_tags_16 },
    { 0xd16af5a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42af5a8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x30caf5aa, SUC_lexicon_tags_38 },
    { 0xb8caf5ab, SUC_lexicon_tags_1 },
    { 0x746af5ac, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47caf5b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62af5b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0af5bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcaf5c3, SUC_lexicon_tags_1 },
    { 0x5d6af5c3, SUC_lexicon_tags_7 },
    { 0x286af5c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852af5cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7eaf5cd, SUC_lexicon_tags_55 },
    { 0xd50af5ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x948af5d0, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240af5d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x41caf5d7, SUC_lexicon_tags_1 },
    { 0x2b0af5d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4eeaf5da, SUC_lexicon_tags_11 },
    { 0x97caf5da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf94af5dd, SUC_lexicon_tags_15 },
    { 0x48eaf5de, SUC_lexicon_tags_39 },
    { 0xdfaaf5df, SUC_lexicon_tags_70 },
    { 0x7e2af5e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccaf5e3, SUC_lexicon_tags_37 },
    { 0x330af5e4, SUC_lexicon_tags_15 },
    { 0x41caf5e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0af5e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaf5ee, SUC_lexicon_tags_12 },
    { 0xb74af5ef, SUC_lexicon_tags_1 },
    { 0xb32af5f0, SUC_lexicon_tags_29 },
    { 0xb2aaf5ee, SUC_lexicon_tags_25 },
    { 0xe68af5f2, SUC_lexicon_tags_10 },
    { 0x90af5ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed0af5f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe22af5f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84caf5fb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x31aaf5fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x596af600, SUC_lexicon_tags_25 },
    { 0x1c8af600, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76af604, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6acaf606, SUC_lexicon_tags_1 },
    { 0xe16af606, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68af60f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aaf614, SUC_lexicon_tags_15 },
    { 0x652af615, SUC_lexicon_tags_47 },
    { 0x6b6af615, SUC_lexicon_tags_7 },
    { 0xbc4af617, SUC_lexicon_tags_39 },
    { 0xe2caf614, SUC_lexicon_tags_39 },
    { 0xf94af615, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74af61c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aaf621, SUC_lexicon_tags_12 },
    { 0xb9eaf622, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6af625, SUC_lexicon_tags_6 },
    { 0x376af625, SUC_lexicon_tags_45 },
    { 0x2b2af627, SUC_lexicon_tags_65 },
    { 0x474af627, SUC_lexicon_tags_17 },
    { 0x516af626, SUC_lexicon_tags_32 },
    { 0xc8eaf62a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6af62d, SUC_lexicon_tags_15 },
    { 0xbd2af62e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38af631, SUC_lexicon_tags_25 },
    { 0x45caf631, SUC_lexicon_tags_5 },
    { 0x29aaf632, SUC_lexicon_tags_6 },
    { 0x5f8af634, SUC_lexicon_tags_63 },
    { 0x9b8af632, SUC_lexicon_tags_25 },
    { 0x40af636, SUC_lexicon_tags_12 },
    { 0xe76af637, SUC_lexicon_tags_8 },
    { 0xd6aaf634, SUC_lexicon_tags_10 },
    { 0x642af631, SUC_lexicon_tags_39 },
    { 0xf80af633, SUC_lexicon_tags_33 },
    { 0x9eaaf63b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59eaf640, SUC_lexicon_tags_36 },
    { 0xfc0af641, SUC_lexicon_tags_1 },
    { 0x9f6af642, SUC_lexicon_tags_25 },
    { 0x5eaaf642, SUC_lexicon_tags_11 },
    { 0x7f2af643, SUC_lexicon_tags_25 },
    { 0x8eaf642, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b4af647, SUC_lexicon_tags_11 },
    { 0xbaaf648, SUC_lexicon_tags_17 },
    { 0x1c6af649, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbdcaf64b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3eaf64e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7caf650, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x870af653, SUC_lexicon_tags_37 },
    { 0xf4aaf654, SUC_lexicon_tags_15 },
    { 0x640af655, SUC_lexicon_tags_15 },
    { 0x2a4af653, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4af658, SUC_lexicon_tags_15 },
    { 0x41eaf659, SUC_lexicon_tags_88 },
    { 0xecaaf65a, SUC_lexicon_tags_17 },
    { 0x5caf65b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc08af65d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90af663, SUC_lexicon_tags_28 },
    { 0x656af663, SUC_lexicon_tags_1 },
    { 0xcf4af665, SUC_lexicon_tags_27 },
    { 0x5feaf666, SUC_lexicon_tags_16 },
    { 0xc0caf665, SUC_lexicon_tags_15 },
    { 0x950af668, SUC_lexicon_tags_26 },
    { 0xda2af669, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6e6af66b, SUC_lexicon_tags_2 },
    { 0x948af66c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2af670, SUC_lexicon_tags_230 },
    { 0x35caf671, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694af676, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x776af678, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aaf67c, SUC_lexicon_tags_25 },
    { 0x2f4af67d, SUC_lexicon_tags_17 },
    { 0x708af67c, SUC_lexicon_tags_70 },
    { 0x7b0af67f, SUC_lexicon_tags_27 },
    { 0xbfcaf680, SUC_lexicon_tags_34 },
    { 0xbd4af67c, SUC_lexicon_tags_4 },
    { 0xb62af682, SUC_lexicon_tags_56 },
    { 0xe2af67d, SUC_lexicon_tags_31 },
    { 0x3aaaf681, SUC_lexicon_tags_1 },
    { 0xa00af683, SUC_lexicon_tags_11 },
    { 0x8f2af680, SUC_lexicon_tags_11 },
    { 0x7b0af687, SUC_lexicon_tags_1 },
    { 0x1a0af67c, SUC_lexicon_tags_15 },
    { 0x9f8af689, SUC_lexicon_tags_42 },
    { 0x8a6af68a, SUC_lexicon_tags_1 },
    { 0xd14af68a, SUC_lexicon_tags_7 },
    { 0x646af68b, SUC_lexicon_tags_15 },
    { 0xe46af68d, SUC_lexicon_tags_11 },
    { 0x886af684, SUC_lexicon_tags_30 },
    { 0xf5caf67d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x314af692, SUC_lexicon_tags_17 },
    { 0xcdcaf692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29caf697, SUC_lexicon_tags_12 },
    { 0x234af697, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63caf6a0, SUC_lexicon_tags_36 },
    { 0x2a2af6a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aaf6a5, SUC_lexicon_tags_65 },
    { 0xf66af6a6, SUC_lexicon_tags_15 },
    { 0xa48af6a7, SUC_lexicon_tags_32 },
    { 0xda6af6a6, SUC_lexicon_tags_6 },
    { 0x25eaf6a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x166af6ac, SUC_lexicon_tags_38 },
    { 0xf50af6ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf82af6af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9feaf6b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772af6be, SUC_lexicon_tags_4 },
    { 0x1caaf6bf, SUC_lexicon_tags_2 },
    { 0xba4af6c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x502af6c3, SUC_lexicon_tags_11 },
    { 0x130af6c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17aaf6ca, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5a6af6cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1caaf6cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70af6d5, SUC_lexicon_tags_17 },
    { 0x728af6d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bcaf6d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x590af6da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec6af6dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8af6e4, SUC_lexicon_tags_17 },
    { 0x97eaf6e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ecaf6e7, SUC_lexicon_tags_4 },
    { 0x9e2af6e7, SUC_lexicon_tags_8 },
    { 0xa2caf6e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x88caf6eb, SUC_lexicon_tags_13 },
    { 0x58eaf6ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4beaf6f8, SUC_lexicon_tags_17 },
    { 0xda6af6f9, SUC_lexicon_tags_32 },
    { 0x44caf6fa, SUC_lexicon_tags_19 },
    { 0x33caf6f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2af6ff, SUC_lexicon_tags_28 },
    { 0xf56af6ff, SUC_lexicon_tags_15 },
    { 0x8e8af701, SUC_lexicon_tags_25 },
    { 0x4acaf702, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764af706, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19eaf70a, SUC_lexicon_tags_28 },
    { 0xa40af70a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22af719, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x990af71b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292af721, SUC_lexicon_tags_33 },
    { 0xa4caf721, SUC_lexicon_tags_30 },
    { 0x8af723, SUC_lexicon_tags_9 },
    { 0xab4af721, SUC_lexicon_tags_37 },
    { 0xaacaf725, SUC_lexicon_tags_11 },
    { 0x8dcaf726, SUC_lexicon_tags_10 },
    { 0xa50af725, SUC_lexicon_tags_37 },
    { 0x54caf728, SUC_lexicon_tags_48 },
    { 0xae0af729, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70eaf72e, SUC_lexicon_tags_17 },
    { 0xb40af72f, SUC_lexicon_tags_47 },
    { 0x660af730, SUC_lexicon_tags_60 },
    { 0xc44af730, SUC_lexicon_tags_15 },
    { 0x420af72e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x224af735, SUC_lexicon_tags_26 },
    { 0x992af736, SUC_lexicon_tags_23 },
    { 0xf5eaf736, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88eaf748, SUC_lexicon_tags_37 },
    { 0x960af749, SUC_lexicon_tags_19 },
    { 0x648af74a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x460af74c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62af74f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa1aaf751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10af757, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7caf75a, SUC_lexicon_tags_37 },
    { 0x8c8af75b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28af75d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50af762, SUC_lexicon_tags_0 },
    { 0x844af762, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364af768, SUC_lexicon_tags_727 },
    { 0x562af769, SUC_lexicon_tags_12 },
    { 0xca6af76a, SUC_lexicon_tags_19 },
    { 0x25eaf768, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f2af76d, SUC_lexicon_tags_38 },
    { 0x51aaf76e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x634af771, SUC_lexicon_tags_16 },
    { 0x1f8af772, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0af778, SUC_lexicon_tags_60 },
    { 0xc0af779, SUC_lexicon_tags_15 },
    { 0xeaaaf77a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x68af77d, SUC_lexicon_tags_16 },
    { 0xd98af77d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4af782, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680af786, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54af789, SUC_lexicon_tags_27 },
    { 0x5ccaf78a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aaf78d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94eaf790, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712af798, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdeaf79b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4af7a1, SUC_lexicon_tags_47 },
    { 0x48caf7a2, SUC_lexicon_tags_65 },
    { 0xc46af7a1, SUC_lexicon_tags_2 },
    { 0x258af7a3, SUC_lexicon_tags_38 },
    { 0x7ccaf7a5, SUC_lexicon_tags_29 },
    { 0x86eaf7a6, SUC_lexicon_tags_2 },
    { 0xc60af7a2, SUC_lexicon_tags_47 },
    { 0x56aaf7a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77eaf7ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9fcaf7b0, SUC_lexicon_tags_55 },
    { 0x6d8af7b0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8c8af7b3, SUC_lexicon_tags_22 },
    { 0x486af7b4, SUC_lexicon_tags_1 },
    { 0xeb8af7b5, SUC_lexicon_tags_25 },
    { 0x1a4af7b4, SUC_lexicon_tags_37 },
    { 0xc9caf7b3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x860af7b9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x82aaf7bb, SUC_lexicon_tags_63 },
    { 0x394af7bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34af7c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x356af7c9, SUC_lexicon_tags_13 },
    { 0x69caf7c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8aaf7cd, SUC_lexicon_tags_17 },
    { 0x2d4af7ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268af7d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5eaf7d4, SUC_lexicon_tags_1 },
    { 0xbfeaf7d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb86af7d7, SUC_lexicon_tags_25 },
    { 0x2e6af7d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68caf7dd, SUC_lexicon_tags_63 },
    { 0x722af7dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162af7e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eaf7e6, SUC_lexicon_tags_1 },
    { 0x8b0af7e6, SUC_lexicon_tags_26 },
    { 0xb9eaf7e6, SUC_lexicon_tags_15 },
    { 0x744af7e6, SUC_lexicon_tags_47 },
    { 0xa7eaf7ea, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xfc6af7ec, SUC_lexicon_tags_82 },
    { 0x78af7ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc18af7ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41eaf7f1, SUC_lexicon_tags_1 },
    { 0xd08af7f2, SUC_lexicon_tags_7 },
    { 0x2ecaf7f1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28af7f7, SUC_lexicon_tags_0 },
    { 0x2aaf7f8, SUC_lexicon_tags_1 },
    { 0x21eaf7f9, SUC_lexicon_tags_55 },
    { 0x7b4af7fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaaf7ff, SUC_lexicon_tags_12 },
    { 0xcd4af800, SUC_lexicon_tags_31 },
    { 0x5f8af801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6faaf804, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0af80d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa34af80f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b4af811, SUC_lexicon_tags_37 },
    { 0xaecaf812, SUC_lexicon_tags_17 },
    { 0x472af811, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd8eaf815, SUC_lexicon_tags_6 },
    { 0x26eaf816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23eaf818, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcaf81d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08af821, SUC_lexicon_tags_45 },
    { 0xf62af821, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5eaf824, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x47caf827, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842af82c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34af82f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80af834, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x86caf837, SUC_lexicon_tags_11 },
    { 0x1b6af838, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aaf83b, SUC_lexicon_tags_17 },
    { 0xb60af83b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceaaf840, SUC_lexicon_tags_25 },
    { 0xf12af841, SUC_lexicon_tags_299 },
    { 0xad4af842, SUC_lexicon_tags_26 },
    { 0x13aaf843, SUC_lexicon_tags_1 },
    { 0xe16af844, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x290af847, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daaf84f, SUC_lexicon_tags_208 },
    { 0x30caf850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9deaf853, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2af85b, SUC_lexicon_tags_0 },
    { 0x5acaf85c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6af861, SUC_lexicon_tags_7 },
    { 0xaaaaf862, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4d0af864, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf7caf866, SUC_lexicon_tags_30 },
    { 0x728af866, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3beaf86a, SUC_lexicon_tags_45 },
    { 0x538af86b, SUC_lexicon_tags_46 },
    { 0xbd4af86c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8af86f, SUC_lexicon_tags_18 },
    { 0xa54af870, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4eaf872, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0af877, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17aaf879, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352af87e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4deaf882, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2af885, SUC_lexicon_tags_40 },
    { 0x76af886, SUC_lexicon_tags_63 },
    { 0x65caf887, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x470af88a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64af890, SUC_lexicon_tags_30 },
    { 0x9d2af890, SUC_lexicon_tags_38 },
    { 0xf4af892, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606af898, SUC_lexicon_tags_11 },
    { 0xadeaf899, SUC_lexicon_tags_47 },
    { 0xee8af89a, SUC_lexicon_tags_37 },
    { 0x77eaf898, SUC_lexicon_tags_17 },
    { 0xea2af899, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828af8a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466af8a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52af8a9, SUC_lexicon_tags_2 },
    { 0x364af8a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2af8b2, SUC_lexicon_tags_12 },
    { 0x606af8b3, SUC_lexicon_tags_25 },
    { 0x788af8b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36af8b7, SUC_lexicon_tags_38 },
    { 0x4aaf8b7, SUC_lexicon_tags_35 },
    { 0xfceaf8b9, SUC_lexicon_tags_45 },
    { 0x20aaf8b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58af8c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4af8c4, SUC_lexicon_tags_38 },
    { 0x42caf8c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36af8c9, SUC_lexicon_tags_87 },
    { 0xd82af8ca, SUC_lexicon_tags_43 },
    { 0xbeaf8cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8eaaf8cd, SUC_lexicon_tags_39 },
    { 0xd1aaf8ce, SUC_lexicon_tags_26 },
    { 0xf06af8cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3eaf8d1, SUC_lexicon_tags_11 },
    { 0xa0aaf8d1, SUC_lexicon_tags_17 },
    { 0x92eaf8d1, SUC_lexicon_tags_29 },
    { 0x764af8d4, SUC_lexicon_tags_38 },
    { 0x3f4af8d5, SUC_lexicon_tags_16 },
    { 0x22af8d4, SUC_lexicon_tags_11 },
    { 0x134af8d2, SUC_lexicon_tags_11 },
    { 0x1eaaf8d8, SUC_lexicon_tags_37 },
    { 0x952af8d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcaf8dc, SUC_lexicon_tags_44 },
    { 0xfccaf8dd, SUC_lexicon_tags_222 },
    { 0xa74af8dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1deaf8e0, SUC_lexicon_tags_25 },
    { 0x426af8e1, SUC_lexicon_tags_2 },
    { 0xfeaaf8e0, SUC_lexicon_tags_33 },
    { 0x228af8e3, SUC_lexicon_tags_15 },
    { 0x8b0af8e4, SUC_lexicon_tags_2 },
    { 0x85aaf8e4, SUC_lexicon_tags_47 },
    { 0xe92af8e6, SUC_lexicon_tags_25 },
    { 0xde8af8e7, SUC_lexicon_tags_28 },
    { 0xd44af8e8, SUC_lexicon_tags_26 },
    { 0xdd8af8e7, SUC_lexicon_tags_65 },
    { 0xb98af8e5, SUC_lexicon_tags_37 },
    { 0xb78af8eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc30af8ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d0af8ef, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xaa2af8f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcaf8f5, SUC_lexicon_tags_1 },
    { 0x78caf8f5, SUC_lexicon_tags_18 },
    { 0x4f6af8f6, SUC_lexicon_tags_7 },
    { 0xb78af8f8, SUC_lexicon_tags_12 },
    { 0x1b8af8f9, SUC_lexicon_tags_43 },
    { 0x776af8f6, SUC_lexicon_tags_12 },
    { 0x3f6af8fb, SUC_lexicon_tags_37 },
    { 0x14eaf8fc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154af902, SUC_lexicon_tags_25 },
    { 0x34aaf902, SUC_lexicon_tags_0 },
    { 0x2a4af904, SUC_lexicon_tags_6 },
    { 0x284af902, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8af909, SUC_lexicon_tags_17 },
    { 0x304af909, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd70af90c, SUC_lexicon_tags_15 },
    { 0xa82af90c, SUC_lexicon_tags_60 },
    { 0x3bcaf90e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62aaf910, SUC_lexicon_tags_38 },
    { 0xabcaf910, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84af914, SUC_lexicon_tags_7 },
    { 0x24af915, SUC_lexicon_tags_7 },
    { 0x332af916, SUC_lexicon_tags_11 },
    { 0x52caf915, SUC_lexicon_tags_9 },
    { 0xc4aaf918, SUC_lexicon_tags_17 },
    { 0xa9eaf919, SUC_lexicon_tags_80 },
    { 0x3d4af91a, SUC_lexicon_tags_19 },
    { 0xd78af91b, SUC_lexicon_tags_11 },
    { 0x540af91c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12eaf923, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4af927, SUC_lexicon_tags_23 },
    { 0xe8aaf928, SUC_lexicon_tags_11 },
    { 0x36caf929, SUC_lexicon_tags_14 },
    { 0x386af92a, SUC_lexicon_tags_2 },
    { 0x9a0af929, SUC_lexicon_tags_7 },
    { 0xf64af929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10af92f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75caf93e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6ccaf940, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4af942, SUC_lexicon_tags_25 },
    { 0x9b0af943, SUC_lexicon_tags_2 },
    { 0xeccaf944, SUC_lexicon_tags_23 },
    { 0x592af944, SUC_lexicon_tags_7 },
    { 0x742af943, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2af949, SUC_lexicon_tags_11 },
    { 0x644af94a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70af94e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe16af950, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x76eaf953, SUC_lexicon_tags_65 },
    { 0x416af954, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8af95d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf00af95f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x960af961, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xadaaf963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x814af965, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eaf96a, SUC_lexicon_tags_17 },
    { 0xe32af96b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf18af96d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2af973, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60caf975, SUC_lexicon_tags_23 },
    { 0x704af976, SUC_lexicon_tags_15 },
    { 0xabeaf977, SUC_lexicon_tags_39 },
    { 0xf72af978, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0af97a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbc8af97c, SUC_lexicon_tags_17 },
    { 0x6faaf97d, SUC_lexicon_tags_25 },
    { 0x88caf97c, SUC_lexicon_tags_10 },
    { 0xe0af97d, SUC_lexicon_tags_6 },
    { 0xac4af980, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x434af983, SUC_lexicon_tags_27 },
    { 0x9c0af984, SUC_lexicon_tags_17 },
    { 0xcf8af984, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x250af988, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6af98c, SUC_lexicon_tags_59 },
    { 0xf86af98d, SUC_lexicon_tags_2 },
    { 0xbd0af98e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbbcaf990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4af993, SUC_lexicon_tags_12 },
    { 0x4a8af993, SUC_lexicon_tags_11 },
    { 0x716af993, SUC_lexicon_tags_60 },
    { 0x6b0af996, SUC_lexicon_tags_48 },
    { 0x4c4af997, SUC_lexicon_tags_45 },
    { 0xdb6af998, SUC_lexicon_tags_25 },
    { 0x110af999, SUC_lexicon_tags_57 },
    { 0x1e2af997, SUC_lexicon_tags_16 },
    { 0x10eaf99b, SUC_lexicon_tags_65 },
    { 0x4feaf997, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x87eaf99e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x232af9a0, SUC_lexicon_tags_47 },
    { 0x23aaf9a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf9a5, SUC_lexicon_tags_2 },
    { 0xb86af9a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53eaf9aa, SUC_lexicon_tags_66 },
    { 0xf1aaf9ab, SUC_lexicon_tags_57 },
    { 0x214af9ac, SUC_lexicon_tags_8 },
    { 0x2a2af9ad, SUC_lexicon_tags_37 },
    { 0x172af9ae, SUC_lexicon_tags_26 },
    { 0xe34af9ad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72af9b2, SUC_lexicon_tags_7 },
    { 0x7b6af9b3, SUC_lexicon_tags_38 },
    { 0xa62af9b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f0af9b6, SUC_lexicon_tags_11 },
    { 0x536af9b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af9bf, SUC_lexicon_tags_15 },
    { 0x3caf9c0, SUC_lexicon_tags_43 },
    { 0x3d4af9c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6af9c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x316af9c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaaf9cb, SUC_lexicon_tags_45 },
    { 0xe72af9cc, SUC_lexicon_tags_12 },
    { 0xfb8af9cb, SUC_lexicon_tags_6 },
    { 0x8eaaf9cd, SUC_lexicon_tags_194 },
    { 0xe3eaf9ce, SUC_lexicon_tags_37 },
    { 0x68aaf9d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x560af9d2, SUC_lexicon_tags_2 },
    { 0x634af9d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0af9d9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44eaf9dd, SUC_lexicon_tags_7 },
    { 0xa32af9de, SUC_lexicon_tags_47 },
    { 0x6beaf9de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1daaf9e2, SUC_lexicon_tags_27 },
    { 0x464af9e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348af9e8, SUC_lexicon_tags_19 },
    { 0xcdcaf9e9, SUC_lexicon_tags_4 },
    { 0x550af9ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350af9f2, SUC_lexicon_tags_19 },
    { 0x73eaf9f3, SUC_lexicon_tags_45 },
    { 0x618af9f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd2af9f6, SUC_lexicon_tags_38 },
    { 0xd3eaf9f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaf9fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x364af9fc, SUC_lexicon_tags_45 },
    { 0x50aaf9fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cafa01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5eafa06, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x102afa09, SUC_lexicon_tags_17 },
    { 0x716afa0a, SUC_lexicon_tags_17 },
    { 0x460afa09, SUC_lexicon_tags_37 },
    { 0xd90afa0a, SUC_lexicon_tags_27 },
    { 0x8a2afa0d, SUC_lexicon_tags_15 },
    { 0x12eafa0e, SUC_lexicon_tags_55 },
    { 0x890afa0f, SUC_lexicon_tags_7 },
    { 0xcb8afa10, SUC_lexicon_tags_65 },
    { 0x658afa0f, SUC_lexicon_tags_27 },
    { 0x7a2afa12, SUC_lexicon_tags_2 },
    { 0x7c8afa12, SUC_lexicon_tags_43 },
    { 0x4faafa14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0cafa16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e0afa18, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x79eafa1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x506afa1c, SUC_lexicon_tags_15 },
    { 0x9c8afa1d, SUC_lexicon_tags_8 },
    { 0x638afa1e, SUC_lexicon_tags_38 },
    { 0xc9eafa1f, SUC_lexicon_tags_39 },
    { 0xf10afa20, SUC_lexicon_tags_0 },
    { 0xd3eafa21, SUC_lexicon_tags_27 },
    { 0xeceafa21, SUC_lexicon_tags_79 },
    { 0x242afa1d, SUC_lexicon_tags_31 },
    { 0x1baafa1c, SUC_lexicon_tags_45 },
    { 0x56eafa25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eafa29, SUC_lexicon_tags_17 },
    { 0x646afa2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4afa30, SUC_lexicon_tags_49 },
    { 0xeaeafa31, SUC_lexicon_tags_46 },
    { 0x482afa32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6afa37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ecafa39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x698afa3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd32afa3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cafa41, SUC_lexicon_tags_25 },
    { 0x3daafa42, SUC_lexicon_tags_14 },
    { 0xe6eafa43, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x272afa46, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cafa4a, SUC_lexicon_tags_15 },
    { 0xdcaafa4b, SUC_lexicon_tags_2 },
    { 0x2b8afa4b, SUC_lexicon_tags_25 },
    { 0xf28afa4a, SUC_lexicon_tags_39 },
    { 0x7e6afa4e, SUC_lexicon_tags_7 },
    { 0xedaafa4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82afa57, SUC_lexicon_tags_12 },
    { 0x3b8afa57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafa5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44afa69, SUC_lexicon_tags_77 },
    { 0x108afa6a, SUC_lexicon_tags_88 },
    { 0x2aeafa6b, SUC_lexicon_tags_61 },
    { 0x938afa6b, SUC_lexicon_tags_12 },
    { 0x514afa6d, SUC_lexicon_tags_47 },
    { 0xb74afa69, SUC_lexicon_tags_6 },
    { 0x68afa6a, SUC_lexicon_tags_12 },
    { 0xbe8afa70, SUC_lexicon_tags_43 },
    { 0xe00afa6b, SUC_lexicon_tags_14 },
    { 0x650afa6f, SUC_lexicon_tags_15 },
    { 0x388afa73, SUC_lexicon_tags_5 },
    { 0x92afa74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678afa7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x452afa7d, SUC_lexicon_tags_11 },
    { 0x846afa7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x378afa80, SUC_lexicon_tags_28 },
    { 0x488afa80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x70eafa84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2eafa86, SUC_lexicon_tags_17 },
    { 0x184afa87, SUC_lexicon_tags_1 },
    { 0xb86afa88, SUC_lexicon_tags_8 },
    { 0xbf2afa89, SUC_lexicon_tags_8 },
    { 0x74aafa89, SUC_lexicon_tags_34 },
    { 0x9b6afa8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acafa8e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x946afa90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabaafa92, SUC_lexicon_tags_1 },
    { 0xa0afa93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x590afa95, SUC_lexicon_tags_21 },
    { 0x706afa96, SUC_lexicon_tags_29 },
    { 0x624afa97, SUC_lexicon_tags_37 },
    { 0x7caafa98, SUC_lexicon_tags_37 },
    { 0x63cafa98, SUC_lexicon_tags_47 },
    { 0x6b6afa9a, SUC_lexicon_tags_46 },
    { 0x9b6afa96, SUC_lexicon_tags_15 },
    { 0xf54afa9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdfcafa9e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabeafaa3, SUC_lexicon_tags_15 },
    { 0xf16afaa4, SUC_lexicon_tags_7 },
    { 0x5aafaa5, SUC_lexicon_tags_60 },
    { 0x9e4afaa6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71aafaa8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8bcafaaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcafab9, SUC_lexicon_tags_1 },
    { 0x20cafaba, SUC_lexicon_tags_39 },
    { 0x8f0afaba, SUC_lexicon_tags_12 },
    { 0xbfcafabc, SUC_lexicon_tags_26 },
    { 0x8dcafab9, SUC_lexicon_tags_23 },
    { 0x8fcafabe, SUC_lexicon_tags_7 },
    { 0xe2eafabe, SUC_lexicon_tags_21 },
    { 0x472afabf, SUC_lexicon_tags_105 },
    { 0x35aafac0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbe2afac3, SUC_lexicon_tags_23 },
    { 0x928afac4, SUC_lexicon_tags_12 },
    { 0x71cafac5, SUC_lexicon_tags_33 },
    { 0x192afac5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2afacb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14aafad1, SUC_lexicon_tags_2 },
    { 0xe92afad2, SUC_lexicon_tags_42 },
    { 0x1e2afad1, SUC_lexicon_tags_37 },
    { 0xc06afad4, SUC_lexicon_tags_12 },
    { 0xa4afad4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa0afad7, SUC_lexicon_tags_17 },
    { 0x942afad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aafadb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6afae3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8afaeb, SUC_lexicon_tags_3 },
    { 0x8cafaec, SUC_lexicon_tags_8 },
    { 0x340afaec, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xab4afaef, SUC_lexicon_tags_2 },
    { 0x25cafaf0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2afaf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53eafafc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626afb00, SUC_lexicon_tags_47 },
    { 0xd10afb01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc00afb03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcafb05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eafb09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20eafb0e, SUC_lexicon_tags_11 },
    { 0xad6afb0e, SUC_lexicon_tags_19 },
    { 0x154afb10, SUC_lexicon_tags_23 },
    { 0xbf2afb0e, SUC_lexicon_tags_37 },
    { 0x44eafb11, SUC_lexicon_tags_27 },
    { 0xc9cafb12, SUC_lexicon_tags_17 },
    { 0xa88afb14, SUC_lexicon_tags_60 },
    { 0x586afb15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc94afb17, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02afb1a, SUC_lexicon_tags_11 },
    { 0xaaeafb1b, SUC_lexicon_tags_45 },
    { 0x1e4afb1c, SUC_lexicon_tags_69 },
    { 0x312afb1d, SUC_lexicon_tags_85 },
    { 0x72afb1a, SUC_lexicon_tags_1 },
    { 0xbe2afb1a, SUC_lexicon_tags_49 },
    { 0x3fcafb1e, SUC_lexicon_tags_63 },
    { 0x406afb21, SUC_lexicon_tags_2 },
    { 0xa7cafb1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5deafb24, SUC_lexicon_tags_15 },
    { 0x5b2afb25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4afb28, SUC_lexicon_tags_2 },
    { 0x7aeafb29, SUC_lexicon_tags_37 },
    { 0x788afb29, SUC_lexicon_tags_79 },
    { 0x4e8afb29, SUC_lexicon_tags_23 },
    { 0x812afb2a, SUC_lexicon_tags_12 },
    { 0x964afb2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdacafb30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xadaafb32, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08afb35, SUC_lexicon_tags_28 },
    { 0xf54afb36, SUC_lexicon_tags_2 },
    { 0x840afb36, SUC_lexicon_tags_3 },
    { 0x928afb37, SUC_lexicon_tags_19 },
    { 0x216afb39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24afb3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cafb43, SUC_lexicon_tags_814 },
    { 0xbe0afb43, SUC_lexicon_tags_65 },
    { 0xb04afb45, SUC_lexicon_tags_48 },
    { 0xd4afb46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb18afb48, SUC_lexicon_tags_37 },
    { 0xa2aafb49, SUC_lexicon_tags_9 },
    { 0xfcafb4a, SUC_lexicon_tags_11 },
    { 0x352afb4b, SUC_lexicon_tags_25 },
    { 0x7dcafb4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2afb51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x438afb53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfaafb55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50eafb5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772afb61, SUC_lexicon_tags_30 },
    { 0x674afb62, SUC_lexicon_tags_1 },
    { 0x798afb63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x438afb66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72afb6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdceafb6f, SUC_lexicon_tags_26 },
    { 0x55eafb6f, SUC_lexicon_tags_0 },
    { 0xaeeafb6f, SUC_lexicon_tags_15 },
    { 0x224afb72, SUC_lexicon_tags_25 },
    { 0xda0afb72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60cafb75, SUC_lexicon_tags_11 },
    { 0x93aafb76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe40afb78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabaafb7b, SUC_lexicon_tags_34 },
    { 0x412afb7b, SUC_lexicon_tags_11 },
    { 0x5eafb7b, SUC_lexicon_tags_11 },
    { 0x4f4afb7c, SUC_lexicon_tags_11 },
    { 0x3faafb7f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x23cafb81, SUC_lexicon_tags_26 },
    { 0x4a4afb82, SUC_lexicon_tags_2 },
    { 0x1aafb81, SUC_lexicon_tags_12 },
    { 0x244afb83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49eafb89, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6bcafb8b, SUC_lexicon_tags_6 },
    { 0xe3eafb8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c4afb8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x786afb91, SUC_lexicon_tags_31 },
    { 0xf2aafb92, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcafb97, SUC_lexicon_tags_95 },
    { 0xc84afb97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0afb9c, SUC_lexicon_tags_55 },
    { 0x5fcafb9d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x968afba0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386afba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662afbaa, SUC_lexicon_tags_28 },
    { 0x686afbab, SUC_lexicon_tags_3 },
    { 0x3b8afbac, SUC_lexicon_tags_21 },
    { 0x5a8afbac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2afbb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeeafbb7, SUC_lexicon_tags_43 },
    { 0x2ccafbb8, SUC_lexicon_tags_37 },
    { 0x132afbb9, SUC_lexicon_tags_12 },
    { 0xc96afbba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceaafbbe, SUC_lexicon_tags_25 },
    { 0x2aaafbbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd0afbc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37aafbc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe6afbc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9aafbd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8afbdb, SUC_lexicon_tags_25 },
    { 0xae0afbdc, SUC_lexicon_tags_47 },
    { 0x86cafbdd, SUC_lexicon_tags_68 },
    { 0xd84afbdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda8afbe0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d0afbe2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaafbe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aafbe9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04afbed, SUC_lexicon_tags_47 },
    { 0x968afbee, SUC_lexicon_tags_47 },
    { 0x590afbef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf12afbf1, SUC_lexicon_tags_47 },
    { 0x5c6afbf1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6aafbf5, SUC_lexicon_tags_35 },
    { 0x446afbf6, SUC_lexicon_tags_47 },
    { 0xa1cafbf7, SUC_lexicon_tags_35 },
    { 0x38afbf8, SUC_lexicon_tags_0 },
    { 0xefcafbf5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcafbfe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3beafc06, SUC_lexicon_tags_16 },
    { 0xa34afc06, SUC_lexicon_tags_23 },
    { 0x526afc08, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86afc10, SUC_lexicon_tags_47 },
    { 0x57aafc11, SUC_lexicon_tags_29 },
    { 0x8d2afc12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8afc16, SUC_lexicon_tags_15 },
    { 0xaa4afc17, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40afc1a, SUC_lexicon_tags_17 },
    { 0x342afc1b, SUC_lexicon_tags_17 },
    { 0x268afc1a, SUC_lexicon_tags_179 },
    { 0x7b4afc1d, SUC_lexicon_tags_234 },
    { 0x1deafc1c, SUC_lexicon_tags_11 },
    { 0x1f4afc1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8afc24, SUC_lexicon_tags_17 },
    { 0xc2afc24, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd58afc28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4baafc2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x45cafc2d, SUC_lexicon_tags_28 },
    { 0x576afc2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3baafc31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcafc34, SUC_lexicon_tags_25 },
    { 0x46cafc35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19aafc39, SUC_lexicon_tags_38 },
    { 0xd2aafc3a, SUC_lexicon_tags_8 },
    { 0xbc4afc3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaafc3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x342afc41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64afc44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56eafc49, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcd6afc4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0afc51, SUC_lexicon_tags_12 },
    { 0xfcafc52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89aafc54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70afc56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0afc59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baafc5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62afc61, SUC_lexicon_tags_7 },
    { 0x4c2afc62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10aafc65, SUC_lexicon_tags_9 },
    { 0xfb4afc66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaafc6c, SUC_lexicon_tags_25 },
    { 0xc7eafc6d, SUC_lexicon_tags_10 },
    { 0xeceafc6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaafc72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9cafc74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x820afc76, SUC_lexicon_tags_32 },
    { 0x310afc77, SUC_lexicon_tags_7 },
    { 0xbdaafc77, SUC_lexicon_tags_63 },
    { 0xf02afc79, SUC_lexicon_tags_23 },
    { 0xea8afc78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3deafc7d, SUC_lexicon_tags_38 },
    { 0xc4afc7d, SUC_lexicon_tags_17 },
    { 0x126afc7f, SUC_lexicon_tags_63 },
    { 0x70aafc7f, SUC_lexicon_tags_47 },
    { 0xe96afc7d, SUC_lexicon_tags_11 },
    { 0x404afc80, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf3eafc84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8afc88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70afc8b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf9cafc8d, SUC_lexicon_tags_10 },
    { 0x3ccafc8e, SUC_lexicon_tags_17 },
    { 0xaa4afc8f, SUC_lexicon_tags_11 },
    { 0x3f8afc90, SUC_lexicon_tags_19 },
    { 0xb26afc91, SUC_lexicon_tags_27 },
    { 0xe4eafc91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b0afc94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1feafc96, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8afc9e, SUC_lexicon_tags_92 },
    { 0x42aafc9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x316afca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb6afca4, SUC_lexicon_tags_7 },
    { 0xa0eafca5, SUC_lexicon_tags_14 },
    { 0x1f0afca6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526afcaa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4afcad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x624afcaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36afcb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4afcb5, SUC_lexicon_tags_17 },
    { 0xad4afcb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78eafcb8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cafcbe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0afcc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cafccc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0afccf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caafcd4, SUC_lexicon_tags_30 },
    { 0x3a4afcd5, SUC_lexicon_tags_19 },
    { 0x6a4afcd5, SUC_lexicon_tags_17 },
    { 0x9c6afcd4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x474afcd9, SUC_lexicon_tags_193 },
    { 0x89eafcda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe24afcdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90afce0, SUC_lexicon_tags_4 },
    { 0xa6eafce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaafce7, SUC_lexicon_tags_11 },
    { 0xd24afce8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364afcef, SUC_lexicon_tags_33 },
    { 0x6aafcf0, SUC_lexicon_tags_14 },
    { 0xe5cafcf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x968afcf3, SUC_lexicon_tags_27 },
    { 0x2f0afcf4, SUC_lexicon_tags_37 },
    { 0xc08afcf5, SUC_lexicon_tags_33 },
    { 0x6f6afcf6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702afcfa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02afd00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faafd04, SUC_lexicon_tags_8 },
    { 0x4e2afd05, SUC_lexicon_tags_87 },
    { 0xc60afd06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26afd0c, SUC_lexicon_tags_346 },
    { 0x400afd0c, SUC_lexicon_tags_15 },
    { 0x544afd0e, SUC_lexicon_tags_33 },
    { 0x93cafd0d, SUC_lexicon_tags_2 },
    { 0xa46afd10, SUC_lexicon_tags_11 },
    { 0x6d2afd11, SUC_lexicon_tags_0 },
    { 0x91eafd0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x548afd14, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x234afd16, SUC_lexicon_tags_25 },
    { 0x184afd17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96aafd19, SUC_lexicon_tags_14 },
    { 0x18eafd1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c6afd1c, SUC_lexicon_tags_14 },
    { 0x96aafd1d, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8afd22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726afd28, SUC_lexicon_tags_55 },
    { 0x224afd29, SUC_lexicon_tags_37 },
    { 0x9b4afd29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7baafd2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4feafd32, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x160afd35, SUC_lexicon_tags_37 },
    { 0x362afd35, SUC_lexicon_tags_47 },
    { 0x28cafd37, SUC_lexicon_tags_25 },
    { 0xe7eafd38, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x578afd3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa94afd3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4aafd3e, SUC_lexicon_tags_12 },
    { 0x246afd3e, SUC_lexicon_tags_11 },
    { 0xc1eafd3f, SUC_lexicon_tags_16 },
    { 0xdf8afd41, SUC_lexicon_tags_19 },
    { 0x7ccafd42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20afd47, SUC_lexicon_tags_37 },
    { 0x3acafd48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd02afd4a, SUC_lexicon_tags_37 },
    { 0x8aeafd4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0afd4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ecafd51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x458afd53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8afd56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b8afd58, SUC_lexicon_tags_46 },
    { 0x2f8afd59, SUC_lexicon_tags_25 },
    { 0x7ccafd59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x29cafd5c, SUC_lexicon_tags_12 },
    { 0xce2afd5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304afd62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcafd66, SUC_lexicon_tags_65 },
    { 0x8ccafd66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986afd6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8afd72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a8afd74, SUC_lexicon_tags_664 },
    { 0x7f6afd74, SUC_lexicon_tags_2 },
    { 0xa9aafd76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68afd7c, SUC_lexicon_tags_7 },
    { 0xb26afd7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x720afd7f, SUC_lexicon_tags_63 },
    { 0x4c4afd7f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2eafd83, SUC_lexicon_tags_3 },
    { 0x99cafd83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aafd87, SUC_lexicon_tags_2 },
    { 0xcceafd88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14afd8d, SUC_lexicon_tags_14 },
    { 0x58afd8e, SUC_lexicon_tags_27 },
    { 0x5e8afd8f, SUC_lexicon_tags_8 },
    { 0x736afd90, SUC_lexicon_tags_11 },
    { 0x8aafd8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x324afd93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2afd97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86afd9a, SUC_lexicon_tags_37 },
    { 0xfdeafd9a, SUC_lexicon_tags_65 },
    { 0xbbcafd9c, SUC_lexicon_tags_2 },
    { 0xef0afd9b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48afda3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfceafda6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaafdac, SUC_lexicon_tags_60 },
    { 0xca0afdac, SUC_lexicon_tags_47 },
    { 0xc5aafdae, SUC_lexicon_tags_45 },
    { 0xd70afdae, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcafdb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8afdb7, SUC_lexicon_tags_63 },
    { 0x46eafdb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6afdbb, SUC_lexicon_tags_37 },
    { 0x3a8afdbc, SUC_lexicon_tags_23 },
    { 0x8b2afdbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6afdc2, SUC_lexicon_tags_17 },
    { 0x342afdc3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aafdc6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0aafdcb, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26afdd2, SUC_lexicon_tags_41 },
    { 0x8c4afdd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeafdd6, SUC_lexicon_tags_47 },
    { 0x2c0afdd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828afddb, SUC_lexicon_tags_7 },
    { 0xc2eafddc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24afde0, SUC_lexicon_tags_33 },
    { 0x7aaafde0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaeafde8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952afdee, SUC_lexicon_tags_11 },
    { 0x92aafdee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758afdf5, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x5cafdf7, SUC_lexicon_tags_60 },
    { 0x9e4afdf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54afdfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x122afdfe, SUC_lexicon_tags_12 },
    { 0x7ecafdff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeb0afe01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cafe03, SUC_lexicon_tags_37 },
    { 0x988afe04, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x264afe06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2afe0c, SUC_lexicon_tags_127 },
    { 0xb52afe0d, SUC_lexicon_tags_48 },
    { 0xb98afe0d, SUC_lexicon_tags_17 },
    { 0xd22afe0e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218afe16, SUC_lexicon_tags_37 },
    { 0x5e2afe17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256afe1f, SUC_lexicon_tags_31 },
    { 0x7a4afe20, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520afe26, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x732afe28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8beafe2a, SUC_lexicon_tags_26 },
    { 0xd54afe2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1eafe2e, SUC_lexicon_tags_30 },
    { 0x4e8afe2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4afe37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x736afe3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x736afe3c, SUC_lexicon_tags_19 },
    { 0x91aafe3c, SUC_lexicon_tags_7 },
    { 0xb20afe3e, SUC_lexicon_tags_23 },
    { 0x21eafe3c, SUC_lexicon_tags_63 },
    { 0xdcaafe40, SUC_lexicon_tags_10 },
    { 0x494afe41, SUC_lexicon_tags_0 },
    { 0x8a6afe42, SUC_lexicon_tags_68 },
    { 0xb9eafe42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x436afe46, SUC_lexicon_tags_30 },
    { 0x5a0afe47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf40afe49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x222afe4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceafe53, SUC_lexicon_tags_33 },
    { 0x618afe54, SUC_lexicon_tags_43 },
    { 0xcd0afe55, SUC_lexicon_tags_2 },
    { 0x326afe53, SUC_lexicon_tags_25 },
    { 0xccafe57, SUC_lexicon_tags_11 },
    { 0x674afe53, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe4eafe5a, SUC_lexicon_tags_9 },
    { 0xbdeafe5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x222afe5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40afe62, SUC_lexicon_tags_37 },
    { 0x65eafe62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234afe6a, SUC_lexicon_tags_32 },
    { 0x202afe6b, SUC_lexicon_tags_50 },
    { 0xc5aafe6c, SUC_lexicon_tags_15 },
    { 0xa60afe6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf2aafe6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0afe76, SUC_lexicon_tags_13 },
    { 0xd5eafe77, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cafe7e, SUC_lexicon_tags_25 },
    { 0xe82afe7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaafe84, SUC_lexicon_tags_9 },
    { 0xa7aafe85, SUC_lexicon_tags_23 },
    { 0xa36afe84, SUC_lexicon_tags_34 },
    { 0x7aafe87, SUC_lexicon_tags_14 },
    { 0x27eafe84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa94afe8a, SUC_lexicon_tags_17 },
    { 0x1ecafe8b, SUC_lexicon_tags_25 },
    { 0x736afe8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa8afe8e, SUC_lexicon_tags_6 },
    { 0x420afe8e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdeafe93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b8afe95, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54afe98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eafe9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaafeaa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc78afeac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490afeb0, SUC_lexicon_tags_15 },
    { 0x40cafeb0, SUC_lexicon_tags_6 },
    { 0x70eafeb2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63aafeb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaa8afeb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690afebe, SUC_lexicon_tags_7 },
    { 0x748afebf, SUC_lexicon_tags_45 },
    { 0xb86afec0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aafecc, SUC_lexicon_tags_37 },
    { 0xbd2afecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8afed2, SUC_lexicon_tags_17 },
    { 0x2b4afed2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44afed7, SUC_lexicon_tags_7 },
    { 0xa36afed7, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x11eafeda, SUC_lexicon_tags_14 },
    { 0x5c6afedb, SUC_lexicon_tags_3 },
    { 0x16eafedb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd46afede, SUC_lexicon_tags_37 },
    { 0xe7aafedf, SUC_lexicon_tags_23 },
    { 0xa8afedf, SUC_lexicon_tags_21 },
    { 0x780afedf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4afee5, SUC_lexicon_tags_26 },
    { 0xc60afee6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4afee9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc92afeeb, SUC_lexicon_tags_11 },
    { 0xff8afeeb, SUC_lexicon_tags_63 },
    { 0x8aafeed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x562afeef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x69eafef1, SUC_lexicon_tags_7 },
    { 0x53eafef2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x288afef4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376afefc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eafeff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2deaff02, SUC_lexicon_tags_29 },
    { 0x178aff03, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aaff0c, SUC_lexicon_tags_37 },
    { 0x324aff0d, SUC_lexicon_tags_7 },
    { 0x520aff0d, SUC_lexicon_tags_15 },
    { 0xdaaaff0f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58aff14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc38aff16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67caff1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25caff1e, SUC_lexicon_tags_31 },
    { 0x690aff1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x310aff21, SUC_lexicon_tags_2 },
    { 0x974aff22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeeaff2b, SUC_lexicon_tags_2 },
    { 0x586aff2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92aaff30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeaff32, SUC_lexicon_tags_8 },
    { 0x576aff33, SUC_lexicon_tags_11 },
    { 0x356aff33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1aaaff36, SUC_lexicon_tags_23 },
    { 0x2baaff37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5eaff39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83aaff3d, SUC_lexicon_tags_29 },
    { 0x334aff3d, SUC_lexicon_tags_46 },
    { 0x7b4aff3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72eaff42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceaaff4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd60aff4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322aff56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d0aff58, SUC_lexicon_tags_11 },
    { 0x660aff59, SUC_lexicon_tags_37 },
    { 0x654aff5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582aff5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6aff62, SUC_lexicon_tags_17 },
    { 0x414aff62, SUC_lexicon_tags_2 },
    { 0x3ecaff64, SUC_lexicon_tags_11 },
    { 0xba2aff65, SUC_lexicon_tags_19 },
    { 0x774aff64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeaff6a, SUC_lexicon_tags_33 },
    { 0xe44aff6b, SUC_lexicon_tags_15 },
    { 0x450aff6a, SUC_lexicon_tags_17 },
    { 0x510aff6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aaff70, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c2aff72, SUC_lexicon_tags_19 },
    { 0x12caff73, SUC_lexicon_tags_113 },
    { 0x648aff74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc66aff76, SUC_lexicon_tags_9 },
    { 0x85aaff77, SUC_lexicon_tags_37 },
    { 0x4caff78, SUC_lexicon_tags_14 },
    { 0x528aff78, SUC_lexicon_tags_25 },
    { 0x296aff7a, SUC_lexicon_tags_12 },
    { 0xbf6aff79, SUC_lexicon_tags_11 },
    { 0x2eaff7c, SUC_lexicon_tags_79 },
    { 0x652aff7d, SUC_lexicon_tags_45 },
    { 0xc64aff7c, SUC_lexicon_tags_17 },
    { 0xadaaff7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310aff84, SUC_lexicon_tags_15 },
    { 0xe6eaff85, SUC_lexicon_tags_2 },
    { 0xd8aaff86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122aff8d, SUC_lexicon_tags_11 },
    { 0xebeaff8e, SUC_lexicon_tags_27 },
    { 0xa1aaff8e, SUC_lexicon_tags_7 },
    { 0x53caff8e, SUC_lexicon_tags_23 },
    { 0xe58aff90, SUC_lexicon_tags_25 },
    { 0x56caff92, SUC_lexicon_tags_23 },
    { 0x49caff93, SUC_lexicon_tags_0 },
    { 0xa68aff94, SUC_lexicon_tags_25 },
    { 0x284aff95, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104aff9a, SUC_lexicon_tags_33 },
    { 0x2daaff9a, SUC_lexicon_tags_46 },
    { 0xa2eaff9c, SUC_lexicon_tags_30 },
    { 0x16aff9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfbeaff9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90affa4, SUC_lexicon_tags_17 },
    { 0xe9eaffa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e8affa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecaffac, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x584affae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30affb2, SUC_lexicon_tags_37 },
    { 0x89eaffb3, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2affb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ceaffb8, SUC_lexicon_tags_47 },
    { 0xa1aaffb9, SUC_lexicon_tags_49 },
    { 0xbe0affba, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0x64caffbc, SUC_lexicon_tags_11 },
    { 0x88affbd, SUC_lexicon_tags_17 },
    { 0xae4affbe, SUC_lexicon_tags_27 },
    { 0xbdaaffbe, SUC_lexicon_tags_14 },
    { 0x54eaffc0, SUC_lexicon_tags_0 },
    { 0x248affc1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4affc4, SUC_lexicon_tags_11 },
    { 0x6ecaffc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eaffca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xea4affcc, SUC_lexicon_tags_29 },
    { 0x596affcc, SUC_lexicon_tags_37 },
    { 0xb88affce, SUC_lexicon_tags_17 },
    { 0x3c2affcd, SUC_lexicon_tags_25 },
    { 0x4c2affd0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2ceaffd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x612affd4, SUC_lexicon_tags_15 },
    { 0xb76affd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74affd8, SUC_lexicon_tags_9 },
    { 0xe34affd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa56affdb, SUC_lexicon_tags_2 },
    { 0x880affdb, SUC_lexicon_tags_49 },
    { 0x128affdd, SUC_lexicon_tags_18 },
    { 0xe16affdd, SUC_lexicon_tags_11 },
    { 0xcfcaffdf, SUC_lexicon_tags_15 },
    { 0xb8eaffe0, SUC_lexicon_tags_29 },
    { 0x738affdd, SUC_lexicon_tags_7 },
    { 0x69eaffdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84affe6, SUC_lexicon_tags_65 },
    { 0x16eaffe7, SUC_lexicon_tags_65 },
    { 0xcbaaffe7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404affec, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x98aaffef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22afff9, SUC_lexicon_tags_37 },
    { 0x1f2afffa, SUC_lexicon_tags_141 },
    { 0xe6cafffb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96b0000, SUC_lexicon_tags_25 },
    { 0xa06b0001, SUC_lexicon_tags_15 },
    { 0x3f8b0001, SUC_lexicon_tags_23 },
    { 0x740b0003, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa54b0005, SUC_lexicon_tags_27 },
    { 0xee4b0006, SUC_lexicon_tags_28 },
    { 0xcc0b0006, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x314b0009, SUC_lexicon_tags_27 },
    { 0x3d4b0009, SUC_lexicon_tags_14 },
    { 0xea6b000b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3b0b000d, SUC_lexicon_tags_19 },
    { 0xc06b000e, SUC_lexicon_tags_25 },
    { 0xfd4b000f, SUC_lexicon_tags_6 },
    { 0xb1cb000e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ab0023, SUC_lexicon_tags_12 },
    { 0xf72b0024, SUC_lexicon_tags_15 },
    { 0x9b4b0024, SUC_lexicon_tags_7 },
    { 0xc34b0025, SUC_lexicon_tags_27 },
    { 0x90eb0023, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5feb0029, SUC_lexicon_tags_7 },
    { 0xbb8b0029, SUC_lexicon_tags_35 },
    { 0xcf6b002b, SUC_lexicon_tags_15 },
    { 0x57cb002b, SUC_lexicon_tags_45 },
    { 0x5b4b002c, SUC_lexicon_tags_25 },
    { 0x7d2b002a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fab0030, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x416b0033, SUC_lexicon_tags_28 },
    { 0xef4b0034, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806b0038, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6b003d, SUC_lexicon_tags_39 },
    { 0xb70b003e, SUC_lexicon_tags_66 },
    { 0x28eb003f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb0041, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92b0047, SUC_lexicon_tags_16 },
    { 0xe06b0047, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcb004c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112b0050, SUC_lexicon_tags_31 },
    { 0xed8b0051, SUC_lexicon_tags_7 },
    { 0x4b2b0050, SUC_lexicon_tags_14 },
    { 0x578b0053, SUC_lexicon_tags_37 },
    { 0x87ab0054, SUC_lexicon_tags_11 },
    { 0x92eb0052, SUC_lexicon_tags_7 },
    { 0xb0eb0056, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x21ab0058, SUC_lexicon_tags_15 },
    { 0x34b0058, SUC_lexicon_tags_148 },
    { 0xa80b005a, SUC_lexicon_tags_37 },
    { 0xeccb005b, SUC_lexicon_tags_29 },
    { 0x9beb005c, SUC_lexicon_tags_8 },
    { 0x394b005d, SUC_lexicon_tags_1 },
    { 0x13ab005c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124b0063, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6b0067, SUC_lexicon_tags_11 },
    { 0x7d2b0067, SUC_lexicon_tags_15 },
    { 0xc14b0067, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fab0071, SUC_lexicon_tags_21 },
    { 0x63eb0072, SUC_lexicon_tags_38 },
    { 0x12ab0073, SUC_lexicon_tags_37 },
    { 0x19ab0074, SUC_lexicon_tags_2 },
    { 0x7a2b0071, SUC_lexicon_tags_29 },
    { 0x9fcb0076, SUC_lexicon_tags_11 },
    { 0xb70b0072, SUC_lexicon_tags_49 },
    { 0x878b0078, SUC_lexicon_tags_12 },
    { 0xd54b0073, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2b007e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8eb0082, SUC_lexicon_tags_1 },
    { 0x20b0083, SUC_lexicon_tags_8 },
    { 0xd98b0083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598b008a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6b0090, SUC_lexicon_tags_38 },
    { 0x2aeb0091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x210b0094, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x22b0096, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ab009b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf28b009d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffab00a4, SUC_lexicon_tags_25 },
    { 0x424b00a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1f8b00a7, SUC_lexicon_tags_25 },
    { 0x1d2b00a7, SUC_lexicon_tags_1 },
    { 0xf56b00a9, SUC_lexicon_tags_37 },
    { 0x8b2b00a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbfeb00ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x716b00ae, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cb00b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18b00b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6b00b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeb00bb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x5e2b00bd, SUC_lexicon_tags_27 },
    { 0x7d4b00be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6eb00c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380b00c5, SUC_lexicon_tags_2 },
    { 0x6aeb00c6, SUC_lexicon_tags_17 },
    { 0x974b00c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1eb00cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198b00d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0b00d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eb00d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ab00de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x738b00e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40b00e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb46b00e5, SUC_lexicon_tags_94 },
    { 0xa54b00e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x510b00e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4eb00ea, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28b00f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6b00f9, SUC_lexicon_tags_7 },
    { 0x5dcb00fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe92b00fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8b00ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ab0106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8ab0108, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa00b010a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cb0110, SUC_lexicon_tags_38 },
    { 0x2deb0110, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b6b0113, SUC_lexicon_tags_12 },
    { 0xe88b0113, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2b0117, SUC_lexicon_tags_17 },
    { 0xbdeb0118, SUC_lexicon_tags_6 },
    { 0x83ab0117, SUC_lexicon_tags_79 },
    { 0xc38b011a, SUC_lexicon_tags_31 },
    { 0x69ab011b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34b011e, SUC_lexicon_tags_17 },
    { 0xceb011f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56b0124, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x406b0127, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb012b, SUC_lexicon_tags_14 },
    { 0x6fcb012b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4b012f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eb0135, SUC_lexicon_tags_11 },
    { 0xca4b0136, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x498b0138, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b013c, SUC_lexicon_tags_39 },
    { 0x930b013c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x292b013f, SUC_lexicon_tags_31 },
    { 0x982b0140, SUC_lexicon_tags_15 },
    { 0xc94b0141, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6b0146, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60b0149, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccb014c, SUC_lexicon_tags_46 },
    { 0x42b014d, SUC_lexicon_tags_46 },
    { 0x748b014d, SUC_lexicon_tags_12 },
    { 0x478b014f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874b0153, SUC_lexicon_tags_31 },
    { 0xaecb0154, SUC_lexicon_tags_16 },
    { 0x73eb0154, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eb015a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x55cb015c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96cb015e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x788b0160, SUC_lexicon_tags_24 },
    { 0xd0eb0161, SUC_lexicon_tags_25 },
    { 0x4ccb0162, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b0166, SUC_lexicon_tags_25 },
    { 0x8d6b0167, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6b016b, SUC_lexicon_tags_6 },
    { 0xc02b016b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d2b016e, SUC_lexicon_tags_45 },
    { 0x56eb016f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8b0176, SUC_lexicon_tags_14 },
    { 0x470b0177, SUC_lexicon_tags_15 },
    { 0xc0b0178, SUC_lexicon_tags_43 },
    { 0x326b0179, SUC_lexicon_tags_17 },
    { 0xf28b0178, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x690b017c, SUC_lexicon_tags_710 },
    { 0, NULL },
    { 0x344b017e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8b0182, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xebab0184, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56cb0186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd26b0188, SUC_lexicon_tags_7 },
    { 0xc6cb0188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8b018c, SUC_lexicon_tags_63 },
    { 0xa64b018d, SUC_lexicon_tags_45 },
    { 0x7c8b018e, SUC_lexicon_tags_1 },
    { 0xddcb018f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12b0198, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39cb019a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf22b019c, SUC_lexicon_tags_14 },
    { 0xf32b019d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4b01a1, SUC_lexicon_tags_77 },
    { 0xd4cb01a2, SUC_lexicon_tags_7 },
    { 0xb0cb01a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6b01a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0cb01a9, SUC_lexicon_tags_15 },
    { 0x85ab01aa, SUC_lexicon_tags_11 },
    { 0x888b01ab, SUC_lexicon_tags_18 },
    { 0x4fcb01ac, SUC_lexicon_tags_9 },
    { 0x750b01ac, SUC_lexicon_tags_27 },
    { 0xb7cb01ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940b01bb, SUC_lexicon_tags_47 },
    { 0xbacb01bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498b01c1, SUC_lexicon_tags_11 },
    { 0x7aeb01c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ab01c6, SUC_lexicon_tags_4 },
    { 0xbe6b01c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44b01cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ab01cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640b01d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0ab01d7, SUC_lexicon_tags_66 },
    { 0xa82b01d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x48cb01db, SUC_lexicon_tags_37 },
    { 0x914b01dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x568b01df, SUC_lexicon_tags_47 },
    { 0x28cb01e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x732b01e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280b01ec, SUC_lexicon_tags_15 },
    { 0xabab01ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa8eb01ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b01f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ab01f8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56b01fc, SUC_lexicon_tags_38 },
    { 0x58ab01fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30eb0203, SUC_lexicon_tags_1 },
    { 0xab8b0203, SUC_lexicon_tags_2 },
    { 0x4fab0205, SUC_lexicon_tags_68 },
    { 0x7c6b0203, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfab020a, SUC_lexicon_tags_12 },
    { 0xec2b020b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x81cb020d, SUC_lexicon_tags_11 },
    { 0x8c0b020d, SUC_lexicon_tags_1 },
    { 0x4dcb020f, SUC_lexicon_tags_17 },
    { 0x556b0210, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ab0215, SUC_lexicon_tags_11 },
    { 0xb16b0216, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0b021a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x272b021d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bcb021f, SUC_lexicon_tags_17 },
    { 0x70eb0220, SUC_lexicon_tags_49 },
    { 0x8d6b021f, SUC_lexicon_tags_1 },
    { 0x828b0222, SUC_lexicon_tags_37 },
    { 0x1eab0220, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaab0231, SUC_lexicon_tags_37 },
    { 0x38b0232, SUC_lexicon_tags_2 },
    { 0x2c0b0233, SUC_lexicon_tags_109 },
    { 0xb86b0234, SUC_lexicon_tags_12 },
    { 0x694b0233, SUC_lexicon_tags_21 },
    { 0xf88b0236, SUC_lexicon_tags_1 },
    { 0x840b0234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eb023b, SUC_lexicon_tags_32 },
    { 0x79eb023c, SUC_lexicon_tags_1 },
    { 0xceb023d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92b0240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b0244, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85eb0247, SUC_lexicon_tags_12 },
    { 0x284b0248, SUC_lexicon_tags_11 },
    { 0xd74b0247, SUC_lexicon_tags_45 },
    { 0x21ab024a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x348b024c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ab024f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aab0255, SUC_lexicon_tags_1 },
    { 0xf90b0256, SUC_lexicon_tags_1 },
    { 0x93ab0256, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0b025d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0b0260, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x19ab0262, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2b026a, SUC_lexicon_tags_1 },
    { 0xc4eb026b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcab026e, SUC_lexicon_tags_27 },
    { 0x6f4b026e, SUC_lexicon_tags_15 },
    { 0xc2eb026e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc8b0272, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x576b0274, SUC_lexicon_tags_43 },
    { 0x910b0275, SUC_lexicon_tags_25 },
    { 0xac6b0276, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcd8b0278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88cb027b, SUC_lexicon_tags_17 },
    { 0x340b027c, SUC_lexicon_tags_1 },
    { 0x44ab027d, SUC_lexicon_tags_65 },
    { 0xdb0b027c, SUC_lexicon_tags_32 },
    { 0x7eab027f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ab0284, SUC_lexicon_tags_29 },
    { 0x536b0285, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ab028c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x684b028e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792b0292, SUC_lexicon_tags_38 },
    { 0x302b0293, SUC_lexicon_tags_15 },
    { 0xa1eb0294, SUC_lexicon_tags_25 },
    { 0xb42b0295, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564b029a, SUC_lexicon_tags_49 },
    { 0x110b029a, SUC_lexicon_tags_28 },
    { 0xd18b029c, SUC_lexicon_tags_15 },
    { 0x52b029d, SUC_lexicon_tags_87 },
    { 0xfe0b029e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992b02a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6b02a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626b02ac, SUC_lexicon_tags_47 },
    { 0xc1ab02ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98b02af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628b02b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6b02b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2b02ba, SUC_lexicon_tags_8 },
    { 0x7dab02bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ab02be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6beb02c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x862b02c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24b02cc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172b02d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cb02d4, SUC_lexicon_tags_25 },
    { 0xeb02d5, SUC_lexicon_tags_7 },
    { 0xa64b02d5, SUC_lexicon_tags_31 },
    { 0x796b02d7, SUC_lexicon_tags_47 },
    { 0x34b02d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8b02e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268b02e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68b02f0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8b02f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cab02f7, SUC_lexicon_tags_30 },
    { 0xc8b02f8, SUC_lexicon_tags_15 },
    { 0x28b02f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae8b02fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3b4b02fd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x970b02ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87eb0302, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x45eb0305, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2b030d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464b0313, SUC_lexicon_tags_1 },
    { 0x128b0314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x582b0317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x782b0319, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370b031e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cb0323, SUC_lexicon_tags_47 },
    { 0x37ab0324, SUC_lexicon_tags_0 },
    { 0xe4eb0325, SUC_lexicon_tags_11 },
    { 0xa00b0326, SUC_lexicon_tags_2 },
    { 0x482b0327, SUC_lexicon_tags_16 },
    { 0x524b0325, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52b032c, SUC_lexicon_tags_47 },
    { 0x802b032d, SUC_lexicon_tags_15 },
    { 0x4e8b032c, SUC_lexicon_tags_17 },
    { 0x5eb032e, SUC_lexicon_tags_15 },
    { 0x8a4b032f, SUC_lexicon_tags_14 },
    { 0xb98b032d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb0335, SUC_lexicon_tags_11 },
    { 0x98eb0335, SUC_lexicon_tags_19 },
    { 0xca6b0335, SUC_lexicon_tags_25 },
    { 0x562b0338, SUC_lexicon_tags_25 },
    { 0x594b0338, SUC_lexicon_tags_2 },
    { 0x7e4b033a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b0344, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cb034c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xab034e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad4b0350, SUC_lexicon_tags_63 },
    { 0xdf0b0351, SUC_lexicon_tags_7 },
    { 0x2beb0352, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xeacb0355, SUC_lexicon_tags_11 },
    { 0xef2b0355, SUC_lexicon_tags_15 },
    { 0x804b0357, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b035e, SUC_lexicon_tags_55 },
    { 0x5ecb035e, SUC_lexicon_tags_38 },
    { 0x1f2b0360, SUC_lexicon_tags_87 },
    { 0xb92b0361, SUC_lexicon_tags_17 },
    { 0x19eb0361, SUC_lexicon_tags_37 },
    { 0xee8b0362, SUC_lexicon_tags_27 },
    { 0x894b0364, SUC_lexicon_tags_30 },
    { 0x866b0361, SUC_lexicon_tags_30 },
    { 0xdc0b0365, SUC_lexicon_tags_14 },
    { 0x8eb0367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6b036d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ceb0371, SUC_lexicon_tags_27 },
    { 0x77ab0371, SUC_lexicon_tags_27 },
    { 0x194b0372, SUC_lexicon_tags_105 },
    { 0x276b0374, SUC_lexicon_tags_1 },
    { 0x624b0375, SUC_lexicon_tags_31 },
    { 0xcb8b0376, SUC_lexicon_tags_37 },
    { 0x798b0376, SUC_lexicon_tags_27 },
    { 0xf58b0377, SUC_lexicon_tags_23 },
    { 0x21ab0377, SUC_lexicon_tags_68 },
    { 0x7d6b0374, SUC_lexicon_tags_15 },
    { 0x860b037b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2b0381, SUC_lexicon_tags_1 },
    { 0x4c8b0382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc62b0384, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2b0387, SUC_lexicon_tags_15 },
    { 0x20eb0387, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa86b038a, SUC_lexicon_tags_12 },
    { 0x412b038a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x934b038e, SUC_lexicon_tags_2 },
    { 0x58eb038f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6b0393, SUC_lexicon_tags_19 },
    { 0x374b0394, SUC_lexicon_tags_65 },
    { 0x82eb0395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0b039a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32b039d, SUC_lexicon_tags_30 },
    { 0xcf8b039e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x446b03a1, SUC_lexicon_tags_17 },
    { 0xb22b03a2, SUC_lexicon_tags_1 },
    { 0x8c8b03a3, SUC_lexicon_tags_21 },
    { 0x324b03a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68b03a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x868b03a9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8b03b2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ab03b5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dab03bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312b03c2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7a0b03c4, SUC_lexicon_tags_26 },
    { 0x868b03c4, SUC_lexicon_tags_15 },
    { 0x10ab03c6, SUC_lexicon_tags_1 },
    { 0x296b03c6, SUC_lexicon_tags_2 },
    { 0x714b03c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb03cb, SUC_lexicon_tags_1 },
    { 0x852b03cc, SUC_lexicon_tags_15 },
    { 0x51eb03cd, SUC_lexicon_tags_42 },
    { 0xfaab03ce, SUC_lexicon_tags_4 },
    { 0x72eb03ce, SUC_lexicon_tags_19 },
    { 0xb3cb03cf, SUC_lexicon_tags_7 },
    { 0x5a8b03cf, SUC_lexicon_tags_11 },
    { 0xbe0b03d2, SUC_lexicon_tags_37 },
    { 0xfc2b03cf, SUC_lexicon_tags_12 },
    { 0x376b03d4, SUC_lexicon_tags_11 },
    { 0x83eb03d5, SUC_lexicon_tags_32 },
    { 0x506b03d6, SUC_lexicon_tags_13 },
    { 0xee2b03cc, SUC_lexicon_tags_7 },
    { 0x570b03d8, SUC_lexicon_tags_38 },
    { 0xd70b03d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeb03dd, SUC_lexicon_tags_12 },
    { 0xc64b03de, SUC_lexicon_tags_19 },
    { 0xd24b03df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0b03e2, SUC_lexicon_tags_2 },
    { 0xe0cb03e3, SUC_lexicon_tags_6 },
    { 0x316b03e2, SUC_lexicon_tags_25 },
    { 0x5b4b03e3, SUC_lexicon_tags_17 },
    { 0x86b03e6, SUC_lexicon_tags_12 },
    { 0x96cb03e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x17ab03e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeb03f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1feb03f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x168b03f9, SUC_lexicon_tags_47 },
    { 0x948b03fa, SUC_lexicon_tags_34 },
    { 0x90eb03fb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcb0402, SUC_lexicon_tags_10 },
    { 0xdab0402, SUC_lexicon_tags_15 },
    { 0xa66b0404, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ab040b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x592b040d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0b0413, SUC_lexicon_tags_12 },
    { 0x76cb0414, SUC_lexicon_tags_33 },
    { 0x3cb0415, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x168b0418, SUC_lexicon_tags_13 },
    { 0xcd2b0419, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eb041f, SUC_lexicon_tags_1 },
    { 0x5deb041f, SUC_lexicon_tags_33 },
    { 0x256b0421, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366b0427, SUC_lexicon_tags_2 },
    { 0x58cb0427, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798b042e, SUC_lexicon_tags_11 },
    { 0x748b042e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728b0434, SUC_lexicon_tags_473 },
    { 0, NULL },
    { 0xd06b0436, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab0439, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x962b0443, SUC_lexicon_tags_25 },
    { 0x7d4b0444, SUC_lexicon_tags_25 },
    { 0xc56b0444, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0b044e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12b0453, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8b0456, SUC_lexicon_tags_1 },
    { 0x23eb0457, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8b045b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x320b045d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90b0461, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556b0465, SUC_lexicon_tags_19 },
    { 0xad4b0466, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2b046d, SUC_lexicon_tags_37 },
    { 0xe50b046d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b0471, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x29eb0473, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x768b0475, SUC_lexicon_tags_12 },
    { 0xff2b0475, SUC_lexicon_tags_105 },
    { 0x886b0477, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78cb0479, SUC_lexicon_tags_9 },
    { 0xeccb047a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7fab047c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cb0480, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798b048c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeb048f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0b0492, SUC_lexicon_tags_11 },
    { 0x794b0493, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256b0497, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x526b049a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea0b049c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80b049f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32b04a6, SUC_lexicon_tags_11 },
    { 0x996b04a6, SUC_lexicon_tags_43 },
    { 0x81cb04a8, SUC_lexicon_tags_47 },
    { 0xcc0b04a8, SUC_lexicon_tags_158 },
    { 0x8b04aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9ab04ac, SUC_lexicon_tags_2 },
    { 0xa04b04ac, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd72b04af, SUC_lexicon_tags_7 },
    { 0x790b04af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6b04b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x494b04b6, SUC_lexicon_tags_25 },
    { 0x704b04b7, SUC_lexicon_tags_15 },
    { 0x17eb04b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x346b04ba, SUC_lexicon_tags_47 },
    { 0xc36b04bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb04c2, SUC_lexicon_tags_62 },
    { 0xe34b04c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2b04c5, SUC_lexicon_tags_4 },
    { 0xc2cb04c6, SUC_lexicon_tags_47 },
    { 0x8eab04c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7cb04c9, SUC_lexicon_tags_24 },
    { 0x7f0b04ca, SUC_lexicon_tags_29 },
    { 0xea4b04cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04b04d4, SUC_lexicon_tags_11 },
    { 0x1beb04d4, SUC_lexicon_tags_11 },
    { 0x79cb04d6, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20b04d9, SUC_lexicon_tags_1 },
    { 0x98b04da, SUC_lexicon_tags_17 },
    { 0x23cb04da, SUC_lexicon_tags_1 },
    { 0x2d2b04db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf74b04de, SUC_lexicon_tags_7 },
    { 0x9feb04de, SUC_lexicon_tags_28 },
    { 0xc22b04df, SUC_lexicon_tags_57 },
    { 0x14ab04e1, SUC_lexicon_tags_8 },
    { 0x70eb04e2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecb04e7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688b04ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eb04ef, SUC_lexicon_tags_15 },
    { 0x4f6b04ef, SUC_lexicon_tags_31 },
    { 0x30cb04f1, SUC_lexicon_tags_1 },
    { 0x73eb04f2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6e2b04f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b04fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386b04ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420b0504, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf04b0506, SUC_lexicon_tags_37 },
    { 0xf64b0507, SUC_lexicon_tags_82 },
    { 0x7d2b0508, SUC_lexicon_tags_12 },
    { 0x138b0509, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9a8b050b, SUC_lexicon_tags_7 },
    { 0x6f2b050b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92b050f, SUC_lexicon_tags_15 },
    { 0x20b0510, SUC_lexicon_tags_25 },
    { 0xb04b0510, SUC_lexicon_tags_37 },
    { 0xee2b0512, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f0b0514, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188b0518, SUC_lexicon_tags_31 },
    { 0xa02b0519, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4b051e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16b0521, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4b0527, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5deb052b, SUC_lexicon_tags_68 },
    { 0xe0b052b, SUC_lexicon_tags_17 },
    { 0x610b052d, SUC_lexicon_tags_1 },
    { 0x8d4b052e, SUC_lexicon_tags_48 },
    { 0x450b052f, SUC_lexicon_tags_47 },
    { 0xafab0530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11eb0533, SUC_lexicon_tags_17 },
    { 0x992b0534, SUC_lexicon_tags_46 },
    { 0x490b0535, SUC_lexicon_tags_16 },
    { 0x954b0534, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x41eb0538, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56b053c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cb053f, SUC_lexicon_tags_17 },
    { 0x590b053f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c0b0542, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cb0545, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x984b0547, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x97cb054a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8b054d, SUC_lexicon_tags_33 },
    { 0x418b054d, SUC_lexicon_tags_37 },
    { 0xb94b054f, SUC_lexicon_tags_25 },
    { 0x61eb0550, SUC_lexicon_tags_25 },
    { 0x270b054e, SUC_lexicon_tags_2 },
    { 0xdecb0552, SUC_lexicon_tags_27 },
    { 0xe4b0552, SUC_lexicon_tags_43 },
    { 0x1a4b0554, SUC_lexicon_tags_21 },
    { 0x16cb0555, SUC_lexicon_tags_17 },
    { 0xa7eb054f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x196b0558, SUC_lexicon_tags_1268 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eeb055b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26b055e, SUC_lexicon_tags_11 },
    { 0x4ccb055e, SUC_lexicon_tags_33 },
    { 0x316b055e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x550b0563, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4b0567, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56b0569, SUC_lexicon_tags_2 },
    { 0xc0b0569, SUC_lexicon_tags_23 },
    { 0x3bab056b, SUC_lexicon_tags_47 },
    { 0x2f4b056c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746b0570, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6b0573, SUC_lexicon_tags_23 },
    { 0x1f6b0574, SUC_lexicon_tags_11 },
    { 0x796b0573, SUC_lexicon_tags_7 },
    { 0x280b0576, SUC_lexicon_tags_23 },
    { 0x4ccb0577, SUC_lexicon_tags_6 },
    { 0x1d6b0577, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488b057c, SUC_lexicon_tags_5 },
    { 0xa90b057c, SUC_lexicon_tags_7 },
    { 0xc7eb057c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b0581, SUC_lexicon_tags_49 },
    { 0x962b0582, SUC_lexicon_tags_25 },
    { 0x3eab0583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b0587, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3b4b0589, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb0cb058b, SUC_lexicon_tags_1 },
    { 0xbdeb058c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc4b058e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82eb0592, SUC_lexicon_tags_87 },
    { 0x66b0592, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x586b0596, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa42b0598, SUC_lexicon_tags_25 },
    { 0x436b0598, SUC_lexicon_tags_1 },
    { 0x698b059a, SUC_lexicon_tags_8 },
    { 0x21cb059a, SUC_lexicon_tags_11 },
    { 0x672b059c, SUC_lexicon_tags_1 },
    { 0x864b059d, SUC_lexicon_tags_17 },
    { 0xfa4b059d, SUC_lexicon_tags_4 },
    { 0x582b059f, SUC_lexicon_tags_2 },
    { 0x514b059d, SUC_lexicon_tags_366 },
    { 0x1f2b059a, SUC_lexicon_tags_17 },
    { 0x534b05a0, SUC_lexicon_tags_11 },
    { 0xfeb059d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0b05a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2b05ac, SUC_lexicon_tags_47 },
    { 0x974b05ad, SUC_lexicon_tags_30 },
    { 0xb66b05ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x386b05b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92eb05b3, SUC_lexicon_tags_8 },
    { 0x588b05b4, SUC_lexicon_tags_57 },
    { 0x932b05b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf16b05b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1deb05c4, SUC_lexicon_tags_33 },
    { 0x112b05c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfab05c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91eb05cd, SUC_lexicon_tags_17 },
    { 0x81eb05ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ab05d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf64b05d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8f6b05d6, SUC_lexicon_tags_37 },
    { 0xfcb05d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x74b05d9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa8b05db, SUC_lexicon_tags_10 },
    { 0x5acb05dc, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x752b05de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9ab05e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bcb05e2, SUC_lexicon_tags_5 },
    { 0x8fab05e3, SUC_lexicon_tags_54 },
    { 0x560b05e4, SUC_lexicon_tags_84 },
    { 0xed6b05e5, SUC_lexicon_tags_19 },
    { 0x92cb05e5, SUC_lexicon_tags_37 },
    { 0xbc6b05e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f8b05e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b05ee, SUC_lexicon_tags_15 },
    { 0x820b05ef, SUC_lexicon_tags_11 },
    { 0x5fab05f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb18b05f2, SUC_lexicon_tags_45 },
    { 0x7eeb05f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcb05f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4eb05fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x69eb05fe, SUC_lexicon_tags_0 },
    { 0x86ab05fe, SUC_lexicon_tags_36 },
    { 0x328b0600, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acb0603, SUC_lexicon_tags_17 },
    { 0xacb0604, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08b0607, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x584b060a, SUC_lexicon_tags_17 },
    { 0x8a2b060b, SUC_lexicon_tags_2 },
    { 0xb0b060c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6feb060f, SUC_lexicon_tags_63 },
    { 0x5a8b060f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeab0619, SUC_lexicon_tags_133 },
    { 0x702b061a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x31cb061c, SUC_lexicon_tags_7 },
    { 0xaeab061c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x190b061f, SUC_lexicon_tags_38 },
    { 0x4bab0620, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2b0623, SUC_lexicon_tags_32 },
    { 0x444b0624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72cb0627, SUC_lexicon_tags_25 },
    { 0x8b0627, SUC_lexicon_tags_33 },
    { 0x99ab0629, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2b062d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d2b062f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x984b0632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x644b0635, SUC_lexicon_tags_11 },
    { 0xd6b0635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d6b0638, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x134b063b, SUC_lexicon_tags_33 },
    { 0xcaab063c, SUC_lexicon_tags_63 },
    { 0x474b063c, SUC_lexicon_tags_12 },
    { 0x636b063d, SUC_lexicon_tags_38 },
    { 0x3ceb063f, SUC_lexicon_tags_30 },
    { 0xc60b0640, SUC_lexicon_tags_29 },
    { 0x23eb0640, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b0644, SUC_lexicon_tags_15 },
    { 0x8aeb0645, SUC_lexicon_tags_17 },
    { 0xeb8b0646, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab064a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab064e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dab0654, SUC_lexicon_tags_8 },
    { 0x572b0655, SUC_lexicon_tags_19 },
    { 0xc62b0656, SUC_lexicon_tags_27 },
    { 0x54ab0657, SUC_lexicon_tags_25 },
    { 0xe28b0658, SUC_lexicon_tags_112 },
    { 0xcdab0656, SUC_lexicon_tags_12 },
    { 0x30eb065a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468b065e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x7c0b0660, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2eb0662, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686b0668, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x482b066a, SUC_lexicon_tags_37 },
    { 0x8aeb066b, SUC_lexicon_tags_2 },
    { 0xbdab066c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfd4b066e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x758b0670, SUC_lexicon_tags_42 },
    { 0x3f2b0671, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73eb0676, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x444b0679, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352b067d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0b0682, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4b0688, SUC_lexicon_tags_11 },
    { 0x95cb0689, SUC_lexicon_tags_87 },
    { 0xcf2b0689, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x930b068c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4b0692, SUC_lexicon_tags_25 },
    { 0x538b0693, SUC_lexicon_tags_27 },
    { 0x912b0694, SUC_lexicon_tags_24 },
    { 0x300b0695, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ab0698, SUC_lexicon_tags_7 },
    { 0xc04b0698, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6b06a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54b06a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2b06a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82b06ad, SUC_lexicon_tags_60 },
    { 0x9c8b06ae, SUC_lexicon_tags_14 },
    { 0xcfcb06ae, SUC_lexicon_tags_25 },
    { 0xee4b06b0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x412b06b3, SUC_lexicon_tags_39 },
    { 0xfe8b06b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cb06b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cb06bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc66b06bf, SUC_lexicon_tags_14 },
    { 0x4aab06bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ab06c4, SUC_lexicon_tags_5 },
    { 0x4bab06c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92b06cc, SUC_lexicon_tags_60 },
    { 0x24ab06cd, SUC_lexicon_tags_17 },
    { 0x3b0b06ce, SUC_lexicon_tags_4 },
    { 0x704b06cd, SUC_lexicon_tags_11 },
    { 0x862b06ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x600b06d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe6b06d4, SUC_lexicon_tags_27 },
    { 0xffeb06d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758b06de, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x844b06e0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x6e0b06e2, SUC_lexicon_tags_6 },
    { 0xa84b06e2, SUC_lexicon_tags_19 },
    { 0x888b06e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7d8b06e6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49eb06ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58b06ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88b06f2, SUC_lexicon_tags_26 },
    { 0x7dcb06f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76b06f5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800b06fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ab06fe, SUC_lexicon_tags_11 },
    { 0x1e0b06fe, SUC_lexicon_tags_66 },
    { 0x824b06ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184b0704, SUC_lexicon_tags_15 },
    { 0xa76b0704, SUC_lexicon_tags_17 },
    { 0xb10b0706, SUC_lexicon_tags_1 },
    { 0xda8b0707, SUC_lexicon_tags_49 },
    { 0x646b0708, SUC_lexicon_tags_60 },
    { 0xff0b0708, SUC_lexicon_tags_27 },
    { 0xf0cb0705, SUC_lexicon_tags_11 },
    { 0xb58b0708, SUC_lexicon_tags_25 },
    { 0x818b070c, SUC_lexicon_tags_15 },
    { 0x6a6b0707, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc12b070f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2b0712, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ab0718, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00b071b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1f2b071d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeab0727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ab072c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4b072f, SUC_lexicon_tags_19 },
    { 0x116b072f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x312b0733, SUC_lexicon_tags_49 },
    { 0xed0b0734, SUC_lexicon_tags_15 },
    { 0xc5ab0735, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd36b0737, SUC_lexicon_tags_8 },
    { 0x2f2b0738, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a6b073a, SUC_lexicon_tags_17 },
    { 0x5eb073a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x768b073e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe90b0740, SUC_lexicon_tags_0 },
    { 0xbfcb0741, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40b0749, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb02b074b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0b074f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32b0754, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x412b0757, SUC_lexicon_tags_27 },
    { 0x9deb0758, SUC_lexicon_tags_0 },
    { 0xe68b0758, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8cb075b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x202b075d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x582b075f, SUC_lexicon_tags_35 },
    { 0xb90b075f, SUC_lexicon_tags_107 },
    { 0x6e4b075f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6b0764, SUC_lexicon_tags_31 },
    { 0xd12b0765, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60b076b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96b076d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356b0778, SUC_lexicon_tags_45 },
    { 0x294b0779, SUC_lexicon_tags_23 },
    { 0x348b077a, SUC_lexicon_tags_17 },
    { 0x6a8b0779, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x94cb077d, SUC_lexicon_tags_25 },
    { 0xd4b077d, SUC_lexicon_tags_37 },
    { 0xe0ab077f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380b0783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ab0787, SUC_lexicon_tags_30 },
    { 0x1d8b0788, SUC_lexicon_tags_84 },
    { 0xdd4b0789, SUC_lexicon_tags_17 },
    { 0x1a0b0789, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4b078f, SUC_lexicon_tags_25 },
    { 0x424b0790, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0xfeb0792, SUC_lexicon_tags_63 },
    { 0x6dcb0792, SUC_lexicon_tags_11 },
    { 0xd1cb0794, SUC_lexicon_tags_25 },
    { 0x8beb0793, SUC_lexicon_tags_15 },
    { 0x496b0793, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x324b0798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x160b079b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eb07a3, SUC_lexicon_tags_17 },
    { 0x13ab07a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b6b07a6, SUC_lexicon_tags_7 },
    { 0x4feb07a7, SUC_lexicon_tags_87 },
    { 0xbc8b07a8, SUC_lexicon_tags_17 },
    { 0x66ab07a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe72b07ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ab07ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94b07b9, SUC_lexicon_tags_60 },
    { 0x8b07b9, SUC_lexicon_tags_47 },
    { 0x774b07ba, SUC_lexicon_tags_60 },
    { 0x6a8b07bc, SUC_lexicon_tags_224 },
    { 0xd9ab07b9, SUC_lexicon_tags_1 },
    { 0xc58b07bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d0b07c0, SUC_lexicon_tags_9 },
    { 0xaa2b07c1, SUC_lexicon_tags_11 },
    { 0x100b07c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918b07ca, SUC_lexicon_tags_11 },
    { 0x8f6b07cb, SUC_lexicon_tags_17 },
    { 0x468b07cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdaeb07ce, SUC_lexicon_tags_60 },
    { 0x3f2b07cf, SUC_lexicon_tags_34 },
    { 0x36b07d0, SUC_lexicon_tags_7 },
    { 0x466b07ce, SUC_lexicon_tags_37 },
    { 0xcaeb07d1, SUC_lexicon_tags_9 },
    { 0xaeb07d3, SUC_lexicon_tags_65 },
    { 0x62cb07d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadeb07d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5cb07d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2cb07da, SUC_lexicon_tags_14 },
    { 0x7f8b07db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd78b07dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2b07e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2b07e7, SUC_lexicon_tags_12 },
    { 0x134b07e8, SUC_lexicon_tags_11 },
    { 0xe7cb07e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e2b07eb, SUC_lexicon_tags_10 },
    { 0xa90b07ec, SUC_lexicon_tags_27 },
    { 0xb7cb07ec, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x84cb07ef, SUC_lexicon_tags_11 },
    { 0x57ab07f0, SUC_lexicon_tags_25 },
    { 0xb0b07f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b07fb, SUC_lexicon_tags_37 },
    { 0xe6b07fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56b0800, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b0805, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x580b0807, SUC_lexicon_tags_8 },
    { 0xc78b0808, SUC_lexicon_tags_25 },
    { 0x22ab0809, SUC_lexicon_tags_43 },
    { 0x882b080a, SUC_lexicon_tags_2 },
    { 0xf18b080b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8b0810, SUC_lexicon_tags_7 },
    { 0x6d0b0811, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdb6b0813, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x866b0816, SUC_lexicon_tags_19 },
    { 0xbb4b0817, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18b0822, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402b0827, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe42b082a, SUC_lexicon_tags_33 },
    { 0x62eb082a, SUC_lexicon_tags_697 },
    { 0x15cb082c, SUC_lexicon_tags_18 },
    { 0xe14b082d, SUC_lexicon_tags_23 },
    { 0x59ab082d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6b0831, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84eb0834, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78b0837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac6b0839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb32b083b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d0b083d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d6b083f, SUC_lexicon_tags_11 },
    { 0x202b0840, SUC_lexicon_tags_70 },
    { 0x45cb083f, SUC_lexicon_tags_151 },
    { 0xb6cb0841, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe24b0844, SUC_lexicon_tags_11 },
    { 0xa10b0845, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98b084a, SUC_lexicon_tags_11 },
    { 0x7d6b084b, SUC_lexicon_tags_47 },
    { 0xcbab084b, SUC_lexicon_tags_25 },
    { 0x106b084d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ab0853, SUC_lexicon_tags_43 },
    { 0xfbeb0853, SUC_lexicon_tags_2 },
    { 0x45ab0855, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb0858, SUC_lexicon_tags_19 },
    { 0x34cb0859, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x72eb085c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24ab085e, SUC_lexicon_tags_1 },
    { 0xc06b085f, SUC_lexicon_tags_14 },
    { 0x9e0b0860, SUC_lexicon_tags_105 },
    { 0xf7ab0860, SUC_lexicon_tags_47 },
    { 0xabcb0862, SUC_lexicon_tags_33 },
    { 0x96b0863, SUC_lexicon_tags_38 },
    { 0x80ab0861, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4b0867, SUC_lexicon_tags_15 },
    { 0x7b8b0868, SUC_lexicon_tags_7 },
    { 0xce8b0869, SUC_lexicon_tags_17 },
    { 0x820b0869, SUC_lexicon_tags_31 },
    { 0xca8b086a, SUC_lexicon_tags_2 },
    { 0xf9eb0869, SUC_lexicon_tags_45 },
    { 0x44ab0868, SUC_lexicon_tags_37 },
    { 0x452b086e, SUC_lexicon_tags_25 },
    { 0x200b086a, SUC_lexicon_tags_9 },
    { 0x9fab0870, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6b0872, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x356b0875, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68b087a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16b087f, SUC_lexicon_tags_30 },
    { 0x136b087f, SUC_lexicon_tags_12 },
    { 0x97eb0881, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2beb0883, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642b0888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b0b088a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x442b088d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdf8b088f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28b0892, SUC_lexicon_tags_55 },
    { 0x6bab0893, SUC_lexicon_tags_45 },
    { 0x18cb0894, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x336b0896, SUC_lexicon_tags_32 },
    { 0xa0ab0896, SUC_lexicon_tags_7 },
    { 0x6fcb0898, SUC_lexicon_tags_11 },
    { 0x84b0899, SUC_lexicon_tags_34 },
    { 0x5b8b0899, SUC_lexicon_tags_6 },
    { 0xf32b0898, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cb089f, SUC_lexicon_tags_37 },
    { 0x14ab08a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x372b08a2, SUC_lexicon_tags_10 },
    { 0xeccb08a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x620b08a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7eeb08a8, SUC_lexicon_tags_17 },
    { 0x6f4b08a9, SUC_lexicon_tags_15 },
    { 0x42ab08aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22b08b5, SUC_lexicon_tags_204 },
    { 0xd68b08b6, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x310b08b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x472b08ba, SUC_lexicon_tags_66 },
    { 0xe26b08bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x830b08bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954b08c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaab08ce, SUC_lexicon_tags_37 },
    { 0x16eb08ce, SUC_lexicon_tags_38 },
    { 0x29cb08d0, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xa2cb08d2, SUC_lexicon_tags_1 },
    { 0xf64b08d2, SUC_lexicon_tags_38 },
    { 0x42cb08d4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86b08d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360b08dd, SUC_lexicon_tags_11 },
    { 0x222b08de, SUC_lexicon_tags_33 },
    { 0x4b2b08df, SUC_lexicon_tags_14 },
    { 0x47ab08de, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ab08e4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe3cb08e6, SUC_lexicon_tags_18 },
    { 0x606b08e7, SUC_lexicon_tags_15 },
    { 0xdb4b08e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x94eb08eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4b08f1, SUC_lexicon_tags_109 },
    { 0x9f6b08f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1cb08f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8b08f7, SUC_lexicon_tags_8 },
    { 0xefab08f8, SUC_lexicon_tags_19 },
    { 0x5b0b08f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbaab08fb, SUC_lexicon_tags_9 },
    { 0x110b08fc, SUC_lexicon_tags_17 },
    { 0x9bcb08fd, SUC_lexicon_tags_17 },
    { 0xe4ab08fe, SUC_lexicon_tags_9 },
    { 0x4ccb08ff, SUC_lexicon_tags_23 },
    { 0x9bab08fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0b0903, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b090b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170b0913, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x418b0915, SUC_lexicon_tags_55 },
    { 0x1d2b0916, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ab0919, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ab091e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40b0925, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48eb092d, SUC_lexicon_tags_23 },
    { 0x5a8b092e, SUC_lexicon_tags_27 },
    { 0x958b092e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b0932, SUC_lexicon_tags_45 },
    { 0x706b0932, SUC_lexicon_tags_27 },
    { 0x62cb0933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70eb0937, SUC_lexicon_tags_104 },
    { 0x8d6b0938, SUC_lexicon_tags_23 },
    { 0x826b0938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4b093d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6b0941, SUC_lexicon_tags_1 },
    { 0x7dcb0941, SUC_lexicon_tags_55 },
    { 0x240b0943, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca0b0945, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x216b0947, SUC_lexicon_tags_0 },
    { 0x9beb0947, SUC_lexicon_tags_9 },
    { 0x44ab0949, SUC_lexicon_tags_30 },
    { 0xdf6b0949, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6beb094c, SUC_lexicon_tags_17 },
    { 0x4f0b094d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cb094f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264b0958, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2b095d, SUC_lexicon_tags_2 },
    { 0x2ceb095d, SUC_lexicon_tags_9 },
    { 0x348b095f, SUC_lexicon_tags_37 },
    { 0x516b0960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8b0963, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb0b0965, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0b0969, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb096f, SUC_lexicon_tags_60 },
    { 0x7eb096f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcb0973, SUC_lexicon_tags_45 },
    { 0x7ccb0974, SUC_lexicon_tags_77 },
    { 0xe7eb0975, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x434b0977, SUC_lexicon_tags_87 },
    { 0xc6b0978, SUC_lexicon_tags_12 },
    { 0xf34b0979, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46b0981, SUC_lexicon_tags_65 },
    { 0x46cb0981, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888b0988, SUC_lexicon_tags_29 },
    { 0x5d8b0988, SUC_lexicon_tags_43 },
    { 0xcc4b0988, SUC_lexicon_tags_30 },
    { 0x54eb0988, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ab098f, SUC_lexicon_tags_11 },
    { 0x556b098f, SUC_lexicon_tags_37 },
    { 0xee0b098f, SUC_lexicon_tags_140 },
    { 0xa6eb098f, SUC_lexicon_tags_15 },
    { 0x5e6b098f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716b0997, SUC_lexicon_tags_43 },
    { 0xf62b0998, SUC_lexicon_tags_38 },
    { 0xb3cb0998, SUC_lexicon_tags_23 },
    { 0xd08b099a, SUC_lexicon_tags_30 },
    { 0x2e2b099a, SUC_lexicon_tags_17 },
    { 0x236b0997, SUC_lexicon_tags_12 },
    { 0x400b099d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6b09a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c8b09a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4b09aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad6b09ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x678b09ae, SUC_lexicon_tags_7 },
    { 0x296b09ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560b09b9, SUC_lexicon_tags_60 },
    { 0x1f0b09ba, SUC_lexicon_tags_37 },
    { 0x35ab09bb, SUC_lexicon_tags_8 },
    { 0x758b09bb, SUC_lexicon_tags_38 },
    { 0xe58b09bb, SUC_lexicon_tags_9 },
    { 0xf12b09bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2eb09c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfa0b09c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316b09c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aab09d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0b09d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1eb09d8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x702b09da, SUC_lexicon_tags_27 },
    { 0x866b09db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26eb09e0, SUC_lexicon_tags_29 },
    { 0x86cb09e1, SUC_lexicon_tags_7 },
    { 0x292b09e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcb09e5, SUC_lexicon_tags_1 },
    { 0x38ab09e5, SUC_lexicon_tags_11 },
    { 0xa00b09e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd32b09e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904b09ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acb09f0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2b6b09f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6a0b09f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe44b09f6, SUC_lexicon_tags_28 },
    { 0xd66b09f7, SUC_lexicon_tags_14 },
    { 0xb18b09f8, SUC_lexicon_tags_38 },
    { 0xf3eb09f9, SUC_lexicon_tags_37 },
    { 0x9a6b09fa, SUC_lexicon_tags_7 },
    { 0xa34b09fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec4b09fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ab0a00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18b0a03, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98b0a06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73eb0a0a, SUC_lexicon_tags_10 },
    { 0x262b0a0b, SUC_lexicon_tags_3 },
    { 0x7eab0a0c, SUC_lexicon_tags_12 },
    { 0x87cb0a0c, SUC_lexicon_tags_12 },
    { 0x9d4b0a0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0b0a12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdeab0a14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef6b0a16, SUC_lexicon_tags_9 },
    { 0x2a6b0a17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6b0a1a, SUC_lexicon_tags_30 },
    { 0xfc8b0a1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b8b0a1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eb0a20, SUC_lexicon_tags_37 },
    { 0x5c8b0a21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22b0a24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x456b0a27, SUC_lexicon_tags_0 },
    { 0x102b0a28, SUC_lexicon_tags_0 },
    { 0xcd8b0a28, SUC_lexicon_tags_23 },
    { 0x85eb0a29, SUC_lexicon_tags_19 },
    { 0x966b0a2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8b0a31, SUC_lexicon_tags_9 },
    { 0xb9ab0a31, SUC_lexicon_tags_17 },
    { 0x1beb0a33, SUC_lexicon_tags_11 },
    { 0x2b0b0a34, SUC_lexicon_tags_14 },
    { 0x330b0a34, SUC_lexicon_tags_30 },
    { 0x83eb0a35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x824b0a38, SUC_lexicon_tags_1 },
    { 0x9dab0a39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8b0a3c, SUC_lexicon_tags_37 },
    { 0x198b0a3d, SUC_lexicon_tags_17 },
    { 0x5c0b0a3e, SUC_lexicon_tags_43 },
    { 0x67eb0a3f, SUC_lexicon_tags_38 },
    { 0xe6cb0a40, SUC_lexicon_tags_19 },
    { 0x954b0a3c, SUC_lexicon_tags_14 },
    { 0xb1cb0a42, SUC_lexicon_tags_15 },
    { 0x5b6b0a43, SUC_lexicon_tags_14 },
    { 0x9fab0a44, SUC_lexicon_tags_23 },
    { 0xc2ab0a43, SUC_lexicon_tags_63 },
    { 0x280b0a43, SUC_lexicon_tags_25 },
    { 0xeeeb0a47, SUC_lexicon_tags_37 },
    { 0x764b0a3e, SUC_lexicon_tags_23 },
    { 0xb0eb0a46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cb0a4d, SUC_lexicon_tags_12 },
    { 0x61cb0a4e, SUC_lexicon_tags_30 },
    { 0x776b0a4f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0b0a52, SUC_lexicon_tags_63 },
    { 0xe62b0a53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x198b0a55, SUC_lexicon_tags_15 },
    { 0x7a8b0a56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd6b0a58, SUC_lexicon_tags_17 },
    { 0x63ab0a59, SUC_lexicon_tags_63 },
    { 0xd2eb0a59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0b0a5f, SUC_lexicon_tags_49 },
    { 0xf0cb0a5f, SUC_lexicon_tags_2 },
    { 0xe1cb0a61, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x99ab0a63, SUC_lexicon_tags_12 },
    { 0xd18b0a63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d0b0a66, SUC_lexicon_tags_15 },
    { 0x94eb0a66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6deb0a69, SUC_lexicon_tags_82 },
    { 0x3b2b0a6a, SUC_lexicon_tags_30 },
    { 0xa18b0a6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b0a6e, SUC_lexicon_tags_12 },
    { 0xdaab0a6f, SUC_lexicon_tags_63 },
    { 0x4ab0a70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9eb0a72, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0b0a77, SUC_lexicon_tags_10 },
    { 0xfd0b0a78, SUC_lexicon_tags_53 },
    { 0x4a4b0a78, SUC_lexicon_tags_28 },
    { 0xeccb0a7a, SUC_lexicon_tags_31 },
    { 0x65eb0a7a, SUC_lexicon_tags_17 },
    { 0xed0b0a79, SUC_lexicon_tags_37 },
    { 0x2cab0a7d, SUC_lexicon_tags_17 },
    { 0xc9eb0a7e, SUC_lexicon_tags_69 },
    { 0x7eeb0a7f, SUC_lexicon_tags_12 },
    { 0xb4eb0a7e, SUC_lexicon_tags_26 },
    { 0x320b0a81, SUC_lexicon_tags_0 },
    { 0x8feb0a7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x472b0a84, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5c6b0a86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52ab0a88, SUC_lexicon_tags_31 },
    { 0x890b0a88, SUC_lexicon_tags_15 },
    { 0x6e8b0a8a, SUC_lexicon_tags_63 },
    { 0x40b0a8a, SUC_lexicon_tags_69 },
    { 0x3beb0a8c, SUC_lexicon_tags_38 },
    { 0x3e8b0a8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0b0a91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0b0a94, SUC_lexicon_tags_207 },
    { 0x82cb0a95, SUC_lexicon_tags_7 },
    { 0xb14b0a96, SUC_lexicon_tags_46 },
    { 0x532b0a97, SUC_lexicon_tags_100 },
    { 0xa78b0a98, SUC_lexicon_tags_25 },
    { 0x464b0a99, SUC_lexicon_tags_17 },
    { 0x79ab0a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12b0a9d, SUC_lexicon_tags_47 },
    { 0x31cb0a9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eb0aa2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8ccb0aa4, SUC_lexicon_tags_19 },
    { 0xd46b0aa5, SUC_lexicon_tags_23 },
    { 0xea6b0aa5, SUC_lexicon_tags_17 },
    { 0x7b4b0aa7, SUC_lexicon_tags_19 },
    { 0xf5ab0aa8, SUC_lexicon_tags_1 },
    { 0xfbab0aa9, SUC_lexicon_tags_25 },
    { 0xa18b0aa8, SUC_lexicon_tags_25 },
    { 0x810b0aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14b0aae, SUC_lexicon_tags_11 },
    { 0xef6b0aae, SUC_lexicon_tags_26 },
    { 0x14b0ab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x878b0ab2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x408b0ab5, SUC_lexicon_tags_63 },
    { 0x82eb0ab6, SUC_lexicon_tags_37 },
    { 0x7eab0ab7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb0abf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18b0ac3, SUC_lexicon_tags_47 },
    { 0xe24b0ac3, SUC_lexicon_tags_7 },
    { 0xb22b0ac5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0b0aca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x662b0acc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284b0ad1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cab0ad4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96b0adc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814b0ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x736b0ae3, SUC_lexicon_tags_12 },
    { 0x48b0ae4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b0ae8, SUC_lexicon_tags_17 },
    { 0x52eb0ae9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe06b0aeb, SUC_lexicon_tags_21 },
    { 0x226b0aec, SUC_lexicon_tags_15 },
    { 0xe0eb0aed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x946b0aef, SUC_lexicon_tags_37 },
    { 0xf64b0af0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784b0af9, SUC_lexicon_tags_9 },
    { 0x5acb0afa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x634b0afc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x832b0afe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd52b0b00, SUC_lexicon_tags_1 },
    { 0x928b0b01, SUC_lexicon_tags_32 },
    { 0xe24b0b02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bab0b06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98b0b0d, SUC_lexicon_tags_7 },
    { 0x9d8b0b0d, SUC_lexicon_tags_87 },
    { 0x166b0b0f, SUC_lexicon_tags_23 },
    { 0xbb8b0b10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78b0b14, SUC_lexicon_tags_37 },
    { 0x590b0b15, SUC_lexicon_tags_56 },
    { 0xc7eb0b14, SUC_lexicon_tags_7 },
    { 0xec6b0b14, SUC_lexicon_tags_48 },
    { 0xde0b0b18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x346b0b1b, SUC_lexicon_tags_17 },
    { 0xe3ab0b1b, SUC_lexicon_tags_13 },
    { 0x398b0b1d, SUC_lexicon_tags_23 },
    { 0x6bcb0b1e, SUC_lexicon_tags_63 },
    { 0x6c2b0b1e, SUC_lexicon_tags_1 },
    { 0xffeb0b1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8deb0b29, SUC_lexicon_tags_60 },
    { 0xcdeb0b2a, SUC_lexicon_tags_12 },
    { 0xa12b0b2b, SUC_lexicon_tags_11 },
    { 0xe28b0b2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774b0b30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470b0b39, SUC_lexicon_tags_1 },
    { 0x80eb0b3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x592b0b3d, SUC_lexicon_tags_11 },
    { 0x3feb0b3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cb0b44, SUC_lexicon_tags_12 },
    { 0x8eab0b45, SUC_lexicon_tags_47 },
    { 0xf0cb0b46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684b0b4c, SUC_lexicon_tags_31 },
    { 0x96eb0b4c, SUC_lexicon_tags_27 },
    { 0xefcb0b4e, SUC_lexicon_tags_31 },
    { 0xb2b0b4e, SUC_lexicon_tags_37 },
    { 0x724b0b4c, SUC_lexicon_tags_38 },
    { 0x580b0b51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4b0b55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x778b0b57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82ab0b59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x428b0b5b, SUC_lexicon_tags_2 },
    { 0x318b0b5c, SUC_lexicon_tags_7 },
    { 0x114b0b5b, SUC_lexicon_tags_2 },
    { 0x8cb0b5c, SUC_lexicon_tags_45 },
    { 0xfc4b0b5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa50b0b61, SUC_lexicon_tags_60 },
    { 0xaf0b0b62, SUC_lexicon_tags_74 },
    { 0x8bcb0b62, SUC_lexicon_tags_30 },
    { 0xec4b0b64, SUC_lexicon_tags_81 },
    { 0x6f4b0b65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276b0b6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x282b0b6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4b0b75, SUC_lexicon_tags_27 },
    { 0xd16b0b75, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b0b7b, SUC_lexicon_tags_25 },
    { 0xd14b0b7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x714b0b7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2cb0b80, SUC_lexicon_tags_31 },
    { 0x35cb0b81, SUC_lexicon_tags_7 },
    { 0x472b0b82, SUC_lexicon_tags_43 },
    { 0x62eb0b81, SUC_lexicon_tags_1 },
    { 0x3e6b0b81, SUC_lexicon_tags_17 },
    { 0xe90b0b80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5beb0b88, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x336b0b8b, SUC_lexicon_tags_25 },
    { 0x904b0b8b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x216b0b8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc86b0b91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ceb0b93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22b0b95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550b0b9a, SUC_lexicon_tags_61 },
    { 0x7d0b0b9b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x110b0b9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2b0ba1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf52b0ba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x954b0ba5, SUC_lexicon_tags_63 },
    { 0x394b0ba6, SUC_lexicon_tags_2 },
    { 0xf82b0ba7, SUC_lexicon_tags_7 },
    { 0x43cb0ba8, SUC_lexicon_tags_9 },
    { 0xd40b0ba7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10b0bb4, SUC_lexicon_tags_33 },
    { 0x41eb0bb5, SUC_lexicon_tags_14 },
    { 0x1f6b0bb4, SUC_lexicon_tags_47 },
    { 0x2e6b0bb6, SUC_lexicon_tags_2 },
    { 0x1c0b0bb8, SUC_lexicon_tags_37 },
    { 0x32cb0bb9, SUC_lexicon_tags_21 },
    { 0xf54b0bb9, SUC_lexicon_tags_37 },
    { 0x346b0bba, SUC_lexicon_tags_38 },
    { 0x96eb0bb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x748b0bbe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ab0bc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2b0bc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x960b0bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63eb0bcd, SUC_lexicon_tags_17 },
    { 0xdab0bce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168b0bd4, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38b0bd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24b0bdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x96eb0be2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b0be6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaeb0be8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4ceb0bea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640b0bf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf88b0bf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4b0bf8, SUC_lexicon_tags_27 },
    { 0x3c0b0bf8, SUC_lexicon_tags_76 },
    { 0x2f2b0bfa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0bfd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4b0c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3aeb0c02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24b0c07, SUC_lexicon_tags_7 },
    { 0x512b0c08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eab0c0e, SUC_lexicon_tags_37 },
    { 0x1e6b0c0f, SUC_lexicon_tags_53 },
    { 0x7b8b0c0f, SUC_lexicon_tags_42 },
    { 0xb68b0c11, SUC_lexicon_tags_14 },
    { 0x994b0c12, SUC_lexicon_tags_17 },
    { 0x78ab0c13, SUC_lexicon_tags_37 },
    { 0x194b0c12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ab0c17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332b0c1d, SUC_lexicon_tags_60 },
    { 0x142b0c1e, SUC_lexicon_tags_28 },
    { 0x8dab0c1f, SUC_lexicon_tags_23 },
    { 0x886b0c20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8eb0c22, SUC_lexicon_tags_2 },
    { 0x3eeb0c23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe98b0c25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b0c28, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cb0c2b, SUC_lexicon_tags_2 },
    { 0x862b0c2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a0b0c2e, SUC_lexicon_tags_2 },
    { 0x1c2b0c2f, SUC_lexicon_tags_11 },
    { 0x2c8b0c30, SUC_lexicon_tags_11 },
    { 0xceb0c31, SUC_lexicon_tags_39 },
    { 0x604b0c32, SUC_lexicon_tags_66 },
    { 0xdfab0c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd02b0c35, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cb0c3b, SUC_lexicon_tags_0 },
    { 0x252b0c3c, SUC_lexicon_tags_1 },
    { 0x3bcb0c3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeb2b0c3f, SUC_lexicon_tags_37 },
    { 0x3f6b0c40, SUC_lexicon_tags_0 },
    { 0x9fab0c40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70b0c44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbacb0c46, SUC_lexicon_tags_1 },
    { 0x512b0c47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2b0c4a, SUC_lexicon_tags_63 },
    { 0x45ab0c4b, SUC_lexicon_tags_82 },
    { 0x990b0c4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6b0c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5deb0c55, SUC_lexicon_tags_26 },
    { 0xf6ab0c56, SUC_lexicon_tags_15 },
    { 0x534b0c57, SUC_lexicon_tags_14 },
    { 0x13cb0c58, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6b0c5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74ab0c5f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236b0c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x868b0c65, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2e6b0c67, SUC_lexicon_tags_33 },
    { 0xfd6b0c68, SUC_lexicon_tags_85 },
    { 0xa7cb0c68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd10b0c6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72b0c6d, SUC_lexicon_tags_61 },
    { 0x94eb0c6d, SUC_lexicon_tags_1 },
    { 0xbdcb0c6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x40ab0c71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecb0c76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x216b0c79, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x846b0c7b, SUC_lexicon_tags_63 },
    { 0x876b0c7b, SUC_lexicon_tags_4 },
    { 0xdb4b0c7d, SUC_lexicon_tags_33 },
    { 0xc3eb0c7c, SUC_lexicon_tags_40 },
    { 0x8b0c7f, SUC_lexicon_tags_1 },
    { 0xd2b0c7d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xddeb0c83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88b0c86, SUC_lexicon_tags_37 },
    { 0xdd8b0c87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17cb0c89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf24b0c8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aeb0c8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2b0ca0, SUC_lexicon_tags_15 },
    { 0x5dab0ca0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf34b0ca3, SUC_lexicon_tags_17 },
    { 0x9a6b0ca4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748b0ca8, SUC_lexicon_tags_26 },
    { 0xbf0b0ca9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876b0cb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd60b0cb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4b0cb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xddcb0cbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b0cbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b0cc1, SUC_lexicon_tags_11 },
    { 0x196b0cc1, SUC_lexicon_tags_39 },
    { 0xcf0b0cc3, SUC_lexicon_tags_16 },
    { 0xf1eb0cc1, SUC_lexicon_tags_37 },
    { 0x610b0cc5, SUC_lexicon_tags_27 },
    { 0x53eb0cc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8b0ccb, SUC_lexicon_tags_4 },
    { 0x736b0ccc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78b0ccf, SUC_lexicon_tags_82 },
    { 0xa0ab0cd0, SUC_lexicon_tags_25 },
    { 0x640b0cd1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf40b0cd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eb0cd9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xa3eb0cdb, SUC_lexicon_tags_37 },
    { 0x222b0cdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccb0cdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cb0ce6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ab0ceb, SUC_lexicon_tags_39 },
    { 0x4f8b0ceb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ab0cef, SUC_lexicon_tags_43 },
    { 0x4b0cf0, SUC_lexicon_tags_6 },
    { 0x24cb0cf1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62b0cf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b0cf8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2deb0cfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cb0cff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b4b0d01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ab0d04, SUC_lexicon_tags_9 },
    { 0x8a8b0d04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cb0d0c, SUC_lexicon_tags_25 },
    { 0x91ab0d0d, SUC_lexicon_tags_10 },
    { 0xf26b0d0e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2b0d11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2cb0d13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x558b0d15, SUC_lexicon_tags_15 },
    { 0xc98b0d16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858b0d1e, SUC_lexicon_tags_27 },
    { 0x992b0d1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfbab0d21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ab0d25, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2b0d29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2b0d2d, SUC_lexicon_tags_38 },
    { 0x238b0d2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b0d34, SUC_lexicon_tags_43 },
    { 0x9a2b0d34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d2b0d37, SUC_lexicon_tags_141 },
    { 0x368b0d38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x398b0d3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x614b0d3c, SUC_lexicon_tags_1 },
    { 0x48b0d3d, SUC_lexicon_tags_37 },
    { 0x12cb0d3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66b0d41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408b0d46, SUC_lexicon_tags_11 },
    { 0x8fab0d46, SUC_lexicon_tags_37 },
    { 0x762b0d48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64ab0d4a, SUC_lexicon_tags_47 },
    { 0xf08b0d4a, SUC_lexicon_tags_2 },
    { 0x290b0d4c, SUC_lexicon_tags_37 },
    { 0xe26b0d4d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8b0d50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36b0d55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa46b0d57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ceb0d5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae0b0d5d, SUC_lexicon_tags_10 },
    { 0xf76b0d5e, SUC_lexicon_tags_27 },
    { 0x29cb0d5d, SUC_lexicon_tags_16 },
    { 0x604b0d60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cb0d64, SUC_lexicon_tags_30 },
    { 0xd8eb0d64, SUC_lexicon_tags_1 },
    { 0x26ab0d65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a4b0d68, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54cb0d6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cb0d76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b0d7b, SUC_lexicon_tags_23 },
    { 0x3c0b0d7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecab0d81, SUC_lexicon_tags_17 },
    { 0x8bcb0d82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x166b0d84, SUC_lexicon_tags_15 },
    { 0xb42b0d85, SUC_lexicon_tags_1 },
    { 0x1bab0d86, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x282b0d88, SUC_lexicon_tags_31 },
    { 0xadeb0d89, SUC_lexicon_tags_60 },
    { 0x696b0d89, SUC_lexicon_tags_25 },
    { 0x784b0d8b, SUC_lexicon_tags_11 },
    { 0x3f4b0d8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x250b0d8f, SUC_lexicon_tags_1 },
    { 0xfcb0d90, SUC_lexicon_tags_37 },
    { 0xc8eb0d91, SUC_lexicon_tags_6 },
    { 0x44b0d8f, SUC_lexicon_tags_7 },
    { 0xf82b0d93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182b0d97, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4b0d9e, SUC_lexicon_tags_7 },
    { 0x58ab0d9f, SUC_lexicon_tags_7 },
    { 0x8f0b0da0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0b0da4, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xcfeb0da6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9eb0da9, SUC_lexicon_tags_38 },
    { 0xe3eb0daa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624b0dae, SUC_lexicon_tags_27 },
    { 0xa7eb0dae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54b0db8, SUC_lexicon_tags_6 },
    { 0x4d6b0db9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf80b0dbb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x478b0dbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb0dc0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb3eb0dc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4b0dc9, SUC_lexicon_tags_15 },
    { 0xb3ab0dca, SUC_lexicon_tags_10 },
    { 0xa6ab0dcb, SUC_lexicon_tags_65 },
    { 0x2f8b0dcc, SUC_lexicon_tags_48 },
    { 0xd7eb0dcc, SUC_lexicon_tags_37 },
    { 0xd8b0dcd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x50cb0dd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39cb0dd4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92b0dd8, SUC_lexicon_tags_11 },
    { 0x396b0dd8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd66b0ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2acb0ddd, SUC_lexicon_tags_3 },
    { 0x850b0dde, SUC_lexicon_tags_16 },
    { 0x882b0ddf, SUC_lexicon_tags_12 },
    { 0xf4cb0ddf, SUC_lexicon_tags_2 },
    { 0xbb4b0de1, SUC_lexicon_tags_63 },
    { 0x99ab0de2, SUC_lexicon_tags_14 },
    { 0xbeab0de1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x402b0de5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa54b0de7, SUC_lexicon_tags_8 },
    { 0xbecb0de8, SUC_lexicon_tags_28 },
    { 0x172b0de9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b0df3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434b0dfb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6b0dff, SUC_lexicon_tags_27 },
    { 0xdcab0dff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x42cb0e02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448b0e07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x874b0e09, SUC_lexicon_tags_17 },
    { 0xb2ab0e0a, SUC_lexicon_tags_49 },
    { 0x8d8b0e09, SUC_lexicon_tags_25 },
    { 0x66eb0e0c, SUC_lexicon_tags_5 },
    { 0x2b4b0e09, SUC_lexicon_tags_31 },
    { 0x9f8b0e0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b0e14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x138b0e16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x926b0e18, SUC_lexicon_tags_7 },
    { 0x930b0e18, SUC_lexicon_tags_2 },
    { 0x3b0b0e1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3aab0e1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x210b0e1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaacb0e21, SUC_lexicon_tags_37 },
    { 0x700b0e22, SUC_lexicon_tags_19 },
    { 0xac2b0e23, SUC_lexicon_tags_1609 },
    { 0, NULL },
    { 0, NULL },
    { 0x198b0e26, SUC_lexicon_tags_33 },
    { 0x90b0e27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x278b0e29, SUC_lexicon_tags_7 },
    { 0x6d0b0e29, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x53eb0e2d, SUC_lexicon_tags_27 },
    { 0x620b0e2d, SUC_lexicon_tags_26 },
    { 0x5bcb0e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8b0e34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe4b0e36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0cb0e38, SUC_lexicon_tags_37 },
    { 0xf24b0e38, SUC_lexicon_tags_11 },
    { 0x68cb0e3a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206b0e46, SUC_lexicon_tags_26 },
    { 0xcb2b0e47, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cb0e4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab0e57, SUC_lexicon_tags_2 },
    { 0x64ab0e58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52b0e5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6b0e60, SUC_lexicon_tags_10 },
    { 0xf3ab0e60, SUC_lexicon_tags_27 },
    { 0xc56b0e62, SUC_lexicon_tags_37 },
    { 0x76ab0e63, SUC_lexicon_tags_15 },
    { 0x79ab0e63, SUC_lexicon_tags_48 },
    { 0x992b0e62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc90b0e67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b0e6e, SUC_lexicon_tags_37 },
    { 0xf36b0e6f, SUC_lexicon_tags_1 },
    { 0x4aab0e6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cab0e74, SUC_lexicon_tags_9 },
    { 0x8bab0e74, SUC_lexicon_tags_2 },
    { 0x524b0e75, SUC_lexicon_tags_17 },
    { 0xea4b0e75, SUC_lexicon_tags_11 },
    { 0x238b0e78, SUC_lexicon_tags_38 },
    { 0xff0b0e79, SUC_lexicon_tags_23 },
    { 0x6c2b0e79, SUC_lexicon_tags_11 },
    { 0x536b0e7a, SUC_lexicon_tags_0 },
    { 0x256b0e7c, SUC_lexicon_tags_43 },
    { 0xd18b0e7d, SUC_lexicon_tags_17 },
    { 0x4feb0e78, SUC_lexicon_tags_104 },
    { 0x76eb0e79, SUC_lexicon_tags_1 },
    { 0x378b0e7a, SUC_lexicon_tags_45 },
    { 0xf0ab0e7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58eb0e83, SUC_lexicon_tags_56 },
    { 0xae6b0e83, SUC_lexicon_tags_3 },
    { 0x4b0b0e83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896b0e89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x858b0e8b, SUC_lexicon_tags_23 },
    { 0xbceb0e8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a4b0e8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724b0e92, SUC_lexicon_tags_40 },
    { 0xf30b0e93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x742b0e95, SUC_lexicon_tags_23 },
    { 0xd7cb0e95, SUC_lexicon_tags_10 },
    { 0xfd0b0e96, SUC_lexicon_tags_30 },
    { 0x63ab0e98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35eb0e9a, SUC_lexicon_tags_84 },
    { 0xc96b0e9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa20b0e9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4b0ea0, SUC_lexicon_tags_53 },
    { 0xda2b0ea1, SUC_lexicon_tags_29 },
    { 0x940b0ea2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x452b0ea4, SUC_lexicon_tags_15 },
    { 0x5e6b0ea5, SUC_lexicon_tags_23 },
    { 0x794b0ea6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeb0ea9, SUC_lexicon_tags_30 },
    { 0xc66b0eaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570b0eae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7ab0eb0, SUC_lexicon_tags_43 },
    { 0xa6b0eb1, SUC_lexicon_tags_17 },
    { 0xf2cb0eb1, SUC_lexicon_tags_7 },
    { 0xedab0eb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3beb0eb6, SUC_lexicon_tags_1 },
    { 0xdfeb0eb6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4b0eba, SUC_lexicon_tags_2 },
    { 0x8d4b0ebb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2b0ec3, SUC_lexicon_tags_1 },
    { 0x8ab0ec3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3bcb0ec6, SUC_lexicon_tags_21 },
    { 0x30ab0ec7, SUC_lexicon_tags_37 },
    { 0x880b0ec8, SUC_lexicon_tags_12 },
    { 0x3b2b0ec7, SUC_lexicon_tags_1 },
    { 0xba0b0eca, SUC_lexicon_tags_1 },
    { 0xcbcb0ecb, SUC_lexicon_tags_25 },
    { 0x5c8b0ecc, SUC_lexicon_tags_43 },
    { 0xd7eb0ecc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeeb0ed0, SUC_lexicon_tags_17 },
    { 0x1b8b0ed1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68b0eda, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcab0edd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6b0ee6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9dab0ee8, SUC_lexicon_tags_25 },
    { 0x900b0ee9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8b0eed, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8b0ef3, SUC_lexicon_tags_25 },
    { 0x998b0ef4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474b0efa, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28b0efe, SUC_lexicon_tags_2 },
    { 0x8e8b0eff, SUC_lexicon_tags_29 },
    { 0xd7cb0f00, SUC_lexicon_tags_23 },
    { 0xbaab0efe, SUC_lexicon_tags_25 },
    { 0xcc4b0f00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x446b0f04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf84b0f06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88b0f0a, SUC_lexicon_tags_29 },
    { 0xf26b0f0a, SUC_lexicon_tags_11 },
    { 0x63cb0f0c, SUC_lexicon_tags_23 },
    { 0x360b0f0c, SUC_lexicon_tags_45 },
    { 0xf50b0f0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84b0f12, SUC_lexicon_tags_6 },
    { 0x944b0f13, SUC_lexicon_tags_30 },
    { 0xe44b0f12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0eb0f16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40b0f19, SUC_lexicon_tags_7 },
    { 0x772b0f1a, SUC_lexicon_tags_2 },
    { 0x38ab0f1b, SUC_lexicon_tags_13 },
    { 0xc76b0f1c, SUC_lexicon_tags_39 },
    { 0x412b0f1d, SUC_lexicon_tags_11 },
    { 0x84b0f1c, SUC_lexicon_tags_37 },
    { 0x234b0f1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66eb0f21, SUC_lexicon_tags_14 },
    { 0x2aab0f22, SUC_lexicon_tags_11 },
    { 0x3feb0f22, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36b0f2d, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8b0f30, SUC_lexicon_tags_17 },
    { 0x884b0f30, SUC_lexicon_tags_14 },
    { 0xce8b0f31, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46b0f38, SUC_lexicon_tags_0 },
    { 0x28eb0f39, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8b0f3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfecb0f41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4eb0f46, SUC_lexicon_tags_15 },
    { 0xceeb0f47, SUC_lexicon_tags_27 },
    { 0x95ab0f48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b0b0f4a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eb0f50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666b0f55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x722b0f57, SUC_lexicon_tags_47 },
    { 0xc3ab0f58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb0b0f5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8b0f5d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22ab0f5f, SUC_lexicon_tags_15 },
    { 0x920b0f5f, SUC_lexicon_tags_39 },
    { 0x4e4b0f5f, SUC_lexicon_tags_46 },
    { 0x136b0f62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340b0f67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf52b0f69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eb0f6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54b0f6f, SUC_lexicon_tags_25 },
    { 0xad0b0f70, SUC_lexicon_tags_13 },
    { 0xbf2b0f70, SUC_lexicon_tags_25 },
    { 0xe9ab0f6f, SUC_lexicon_tags_37 },
    { 0xce0b0f73, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4b0f79, SUC_lexicon_tags_56 },
    { 0x3dab0f7a, SUC_lexicon_tags_25 },
    { 0xfc4b0f7b, SUC_lexicon_tags_33 },
    { 0x63ab0f7b, SUC_lexicon_tags_15 },
    { 0x4ccb0f79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b0f80, SUC_lexicon_tags_5 },
    { 0x264b0f81, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6b0f85, SUC_lexicon_tags_35 },
    { 0xfdcb0f86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x418b0f88, SUC_lexicon_tags_17 },
    { 0x864b0f89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638b0f8e, SUC_lexicon_tags_1 },
    { 0x356b0f8f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0b0f92, SUC_lexicon_tags_11 },
    { 0x3bab0f92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4ab0f95, SUC_lexicon_tags_14 },
    { 0x9b0b0f96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cb0fa2, SUC_lexicon_tags_23 },
    { 0x5d4b0fa2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeb0fad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56ab0faf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0fb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4dab0fb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ccb0fb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eab0fbc, SUC_lexicon_tags_65 },
    { 0x9b0b0fbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aab0fc5, SUC_lexicon_tags_11 },
    { 0xccab0fc6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16b0fca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4b0fd0, SUC_lexicon_tags_87 },
    { 0x9e6b0fd1, SUC_lexicon_tags_27 },
    { 0x4c2b0fd2, SUC_lexicon_tags_0 },
    { 0x75cb0fd1, SUC_lexicon_tags_17 },
    { 0xa56b0fd4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe00b0fd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dab0fdc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8b0fed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4cb0fef, SUC_lexicon_tags_37 },
    { 0x586b0ff0, SUC_lexicon_tags_37 },
    { 0xf88b0fef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6b0ff4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec8b0ff6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecb0ffa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170b0ffe, SUC_lexicon_tags_46 },
    { 0x48eb0fff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96b1002, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb1005, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8b100b, SUC_lexicon_tags_1 },
    { 0x12cb100c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcfeb100e, SUC_lexicon_tags_23 },
    { 0xad2b100f, SUC_lexicon_tags_12 },
    { 0x1dcb1010, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4b1015, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcb1019, SUC_lexicon_tags_38 },
    { 0xfd6b101a, SUC_lexicon_tags_40 },
    { 0xa52b101b, SUC_lexicon_tags_28 },
    { 0xb38b101c, SUC_lexicon_tags_23 },
    { 0xe78b101d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eb1021, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dab1024, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8cb1026, SUC_lexicon_tags_1 },
    { 0x74cb1027, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2b102c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ab1030, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cb1035, SUC_lexicon_tags_29 },
    { 0x464b1035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8eb1038, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecb103e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7eb1043, SUC_lexicon_tags_30 },
    { 0x644b1044, SUC_lexicon_tags_49 },
    { 0x722b1045, SUC_lexicon_tags_47 },
    { 0x352b1046, SUC_lexicon_tags_27 },
    { 0xc80b1044, SUC_lexicon_tags_7 },
    { 0x7ceb1046, SUC_lexicon_tags_2 },
    { 0xaaab1045, SUC_lexicon_tags_55 },
    { 0x5ecb1049, SUC_lexicon_tags_17 },
    { 0x388b1048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250b1053, SUC_lexicon_tags_61 },
    { 0x4e6b1054, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294b1059, SUC_lexicon_tags_7 },
    { 0x340b1059, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66b1060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x222b1062, SUC_lexicon_tags_2 },
    { 0x6ceb1063, SUC_lexicon_tags_26 },
    { 0x412b1064, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642b106c, SUC_lexicon_tags_57 },
    { 0x97eb106d, SUC_lexicon_tags_14 },
    { 0x822b106e, SUC_lexicon_tags_23 },
    { 0x33ab106d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34b1072, SUC_lexicon_tags_60 },
    { 0xd9eb1072, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1cb1075, SUC_lexicon_tags_11 },
    { 0xfaeb1075, SUC_lexicon_tags_0 },
    { 0x9a6b1077, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79ab1079, SUC_lexicon_tags_1469 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8b107d, SUC_lexicon_tags_30 },
    { 0xec0b107d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32b1082, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x16ab1084, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecb1086, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeb8b1088, SUC_lexicon_tags_0 },
    { 0x396b1089, SUC_lexicon_tags_21 },
    { 0x61cb108a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180b108e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6b1091, SUC_lexicon_tags_0 },
    { 0xf9cb1092, SUC_lexicon_tags_47 },
    { 0xdb4b1092, SUC_lexicon_tags_7 },
    { 0x89eb1091, SUC_lexicon_tags_1 },
    { 0xeacb1095, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8eb1097, SUC_lexicon_tags_17 },
    { 0x4bcb1097, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf20b109a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x348b109d, SUC_lexicon_tags_37 },
    { 0x758b109d, SUC_lexicon_tags_55 },
    { 0x1b4b109d, SUC_lexicon_tags_37 },
    { 0x2ccb10a0, SUC_lexicon_tags_7 },
    { 0xa5ab10a1, SUC_lexicon_tags_3 },
    { 0xb3cb10a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5cb10a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e6b10a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb10ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454b10b4, SUC_lexicon_tags_1 },
    { 0x52cb10b4, SUC_lexicon_tags_30 },
    { 0xed6b10b6, SUC_lexicon_tags_49 },
    { 0x4ab10b7, SUC_lexicon_tags_25 },
    { 0x958b10b8, SUC_lexicon_tags_17 },
    { 0x944b10b8, SUC_lexicon_tags_17 },
    { 0xdceb10ba, SUC_lexicon_tags_83 },
    { 0x59eb10bb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6b10be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ab10c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d4b10c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb6b10c5, SUC_lexicon_tags_47 },
    { 0x74ab10c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2dcb10c8, SUC_lexicon_tags_26 },
    { 0x6feb10c9, SUC_lexicon_tags_61 },
    { 0xd50b10c8, SUC_lexicon_tags_12 },
    { 0x550b10cb, SUC_lexicon_tags_61 },
    { 0xdb4b10cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d6b10ce, SUC_lexicon_tags_0 },
    { 0x75eb10cf, SUC_lexicon_tags_15 },
    { 0x7fab10ce, SUC_lexicon_tags_55 },
    { 0xe40b10d1, SUC_lexicon_tags_7 },
    { 0xfe2b10cf, SUC_lexicon_tags_12 },
    { 0xc0eb10d3, SUC_lexicon_tags_1 },
    { 0x666b10d4, SUC_lexicon_tags_7 },
    { 0xe00b10d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42b10db, SUC_lexicon_tags_14 },
    { 0x4aab10db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc8b10de, SUC_lexicon_tags_5 },
    { 0x8aeb10de, SUC_lexicon_tags_38 },
    { 0xeb0b10de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2b10e4, SUC_lexicon_tags_17 },
    { 0x902b10e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24b10e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddeb10ee, SUC_lexicon_tags_33 },
    { 0x8d2b10ee, SUC_lexicon_tags_63 },
    { 0xcfeb10ee, SUC_lexicon_tags_15 },
    { 0xd42b10ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ab10f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x254b10fa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cb1106, SUC_lexicon_tags_17 },
    { 0xc46b1107, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x328b1109, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfe2b110b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92b110e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecb1114, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374b1119, SUC_lexicon_tags_47 },
    { 0xb06b111a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2b111e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6b1123, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x354b1125, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96b1129, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcb1130, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6b1134, SUC_lexicon_tags_7 },
    { 0x63cb1135, SUC_lexicon_tags_43 },
    { 0xdeeb1136, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ab1139, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93eb113d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5dcb113f, SUC_lexicon_tags_29 },
    { 0x820b1140, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7eb114b, SUC_lexicon_tags_29 },
    { 0x754b114c, SUC_lexicon_tags_26 },
    { 0x2b0b114d, SUC_lexicon_tags_37 },
    { 0x8bcb114c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2b1155, SUC_lexicon_tags_7 },
    { 0x53eb1156, SUC_lexicon_tags_25 },
    { 0xa38b1157, SUC_lexicon_tags_66 },
    { 0x994b1156, SUC_lexicon_tags_37 },
    { 0x26ab1157, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e8b115b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcb115e, SUC_lexicon_tags_11 },
    { 0x4aab115f, SUC_lexicon_tags_48 },
    { 0x462b1160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392b1167, SUC_lexicon_tags_7 },
    { 0x248b1168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x330b116a, SUC_lexicon_tags_21 },
    { 0x564b116b, SUC_lexicon_tags_13 },
    { 0x188b116c, SUC_lexicon_tags_17 },
    { 0x4e0b116d, SUC_lexicon_tags_19 },
    { 0x242b116b, SUC_lexicon_tags_17 },
    { 0xd48b116f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850b1173, SUC_lexicon_tags_43 },
    { 0xe88b1173, SUC_lexicon_tags_20 },
    { 0x73eb1174, SUC_lexicon_tags_25 },
    { 0x8a0b1176, SUC_lexicon_tags_25 },
    { 0x370b1175, SUC_lexicon_tags_25 },
    { 0x454b1175, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcfeb117a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7beb117d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4b1180, SUC_lexicon_tags_56 },
    { 0xd5eb1181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8b118a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0b118e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91cb1190, SUC_lexicon_tags_33 },
    { 0x86b1191, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9beb1193, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2b1196, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4ceb1198, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9beb11a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x276b11a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcb11ac, SUC_lexicon_tags_11 },
    { 0xeeeb11ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48b11b0, SUC_lexicon_tags_11 },
    { 0x78eb11b1, SUC_lexicon_tags_48 },
    { 0x2c6b11b1, SUC_lexicon_tags_33 },
    { 0xc8cb11b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934b11b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xda6b11b9, SUC_lexicon_tags_15 },
    { 0xfe4b11ba, SUC_lexicon_tags_7 },
    { 0x420b11bb, SUC_lexicon_tags_47 },
    { 0x818b11bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458b11c1, SUC_lexicon_tags_15 },
    { 0xf2eb11c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca6b11c4, SUC_lexicon_tags_34 },
    { 0xcbab11c5, SUC_lexicon_tags_15 },
    { 0xe18b11c6, SUC_lexicon_tags_11 },
    { 0x6feb11c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ab11ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cb11ce, SUC_lexicon_tags_10 },
    { 0xc5ab11cf, SUC_lexicon_tags_47 },
    { 0x622b11d0, SUC_lexicon_tags_17 },
    { 0xb5cb11d1, SUC_lexicon_tags_17 },
    { 0x220b11d2, SUC_lexicon_tags_42 },
    { 0xa42b11d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808b11d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ab11db, SUC_lexicon_tags_23 },
    { 0xc38b11dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cb11e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710b11ea, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8b11f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10ab11f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cb11f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6b11fa, SUC_lexicon_tags_1 },
    { 0xf20b11fa, SUC_lexicon_tags_45 },
    { 0x9eab11fb, SUC_lexicon_tags_37 },
    { 0x4b8b11fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb1202, SUC_lexicon_tags_15 },
    { 0x2c4b1203, SUC_lexicon_tags_32 },
    { 0xecab1204, SUC_lexicon_tags_47 },
    { 0x1dcb1204, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb8b1207, SUC_lexicon_tags_12 },
    { 0x506b1208, SUC_lexicon_tags_11 },
    { 0x7a0b1207, SUC_lexicon_tags_2 },
    { 0x316b1209, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cb120d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3feb120f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2b1214, SUC_lexicon_tags_25 },
    { 0xb8eb1214, SUC_lexicon_tags_25 },
    { 0x3d6b1215, SUC_lexicon_tags_38 },
    { 0xbb2b1217, SUC_lexicon_tags_7 },
    { 0xf94b1214, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4b121c, SUC_lexicon_tags_25 },
    { 0xa3ab121d, SUC_lexicon_tags_17 },
    { 0x94cb121e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb8b1220, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ab1225, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2b1228, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x3beb122a, SUC_lexicon_tags_23 },
    { 0x744b122b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322b122f, SUC_lexicon_tags_37 },
    { 0x370b1230, SUC_lexicon_tags_25 },
    { 0x364b1231, SUC_lexicon_tags_46 },
    { 0x7e4b1232, SUC_lexicon_tags_19 },
    { 0x1e8b1231, SUC_lexicon_tags_14 },
    { 0xb9cb1230, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x712b1236, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x63cb1238, SUC_lexicon_tags_11 },
    { 0xe44b1239, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32b123f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb90b1241, SUC_lexicon_tags_28 },
    { 0x184b1242, SUC_lexicon_tags_87 },
    { 0xd6eb1242, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b1247, SUC_lexicon_tags_11 },
    { 0x7f2b1248, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb124d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e0b124f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd68b1251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41eb1253, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc74b1255, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x746b1257, SUC_lexicon_tags_0 },
    { 0xc16b1258, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6b125d, SUC_lexicon_tags_2 },
    { 0xb10b125e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2b126b, SUC_lexicon_tags_37 },
    { 0x98cb126b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968b1275, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8b1278, SUC_lexicon_tags_11 },
    { 0x4feb1278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5eb127b, SUC_lexicon_tags_19 },
    { 0x32cb127c, SUC_lexicon_tags_7 },
    { 0xdb4b127d, SUC_lexicon_tags_11 },
    { 0xecab127e, SUC_lexicon_tags_14 },
    { 0x3ecb127f, SUC_lexicon_tags_11 },
    { 0x528b1280, SUC_lexicon_tags_11 },
    { 0xe24b127f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cb1288, SUC_lexicon_tags_37 },
    { 0x46eb1289, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ab128e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81cb1292, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596b129f, SUC_lexicon_tags_50 },
    { 0xdbcb12a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98b12a3, SUC_lexicon_tags_38 },
    { 0xf88b12a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72b12a8, SUC_lexicon_tags_1 },
    { 0x944b12a8, SUC_lexicon_tags_37 },
    { 0xf54b12aa, SUC_lexicon_tags_37 },
    { 0x994b12ab, SUC_lexicon_tags_2 },
    { 0xd7ab12ab, SUC_lexicon_tags_60 },
    { 0x868b12ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2b12b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cb12b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730b12c6, SUC_lexicon_tags_1359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348b12ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x118b12cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x79ab12ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18b12d2, SUC_lexicon_tags_37 },
    { 0x7d8b12d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa22b12d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eb12de, SUC_lexicon_tags_37 },
    { 0x780b12de, SUC_lexicon_tags_29 },
    { 0x9d8b12e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbeb12e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2b12ea, SUC_lexicon_tags_6 },
    { 0xc54b12ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51eb12f0, SUC_lexicon_tags_37 },
    { 0x22b12f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6b12f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb1305, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b130a, SUC_lexicon_tags_37 },
    { 0x448b130b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xee8b130d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfb6b130f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6b1318, SUC_lexicon_tags_17 },
    { 0x614b1319, SUC_lexicon_tags_0 },
    { 0x618b131a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202b131e, SUC_lexicon_tags_1 },
    { 0xc08b131e, SUC_lexicon_tags_29 },
    { 0x85eb131e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2c4b1322, SUC_lexicon_tags_11 },
    { 0xb1eb1323, SUC_lexicon_tags_7 },
    { 0xc04b1324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358b132d, SUC_lexicon_tags_1 },
    { 0x248b132e, SUC_lexicon_tags_25 },
    { 0x7deb132f, SUC_lexicon_tags_1 },
    { 0x202b132d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc0b1332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x790b1334, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcb1338, SUC_lexicon_tags_7 },
    { 0xaeab1339, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xec8b133b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bab133e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680b1342, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2b1348, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8c8b134a, SUC_lexicon_tags_63 },
    { 0xa42b134b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbb4b134d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b1353, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe06b1355, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8b1358, SUC_lexicon_tags_25 },
    { 0x7d4b1358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x452b135c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144b1360, SUC_lexicon_tags_25 },
    { 0x6d6b1361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x428b1363, SUC_lexicon_tags_29 },
    { 0xa9ab1364, SUC_lexicon_tags_25 },
    { 0x5c2b1365, SUC_lexicon_tags_47 },
    { 0xbdcb1365, SUC_lexicon_tags_2 },
    { 0xc70b1367, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7beb1369, SUC_lexicon_tags_37 },
    { 0x21cb1369, SUC_lexicon_tags_32 },
    { 0xbdeb1369, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ab136f, SUC_lexicon_tags_66 },
    { 0x980b1370, SUC_lexicon_tags_49 },
    { 0x30cb1371, SUC_lexicon_tags_92 },
    { 0xf80b1372, SUC_lexicon_tags_80 },
    { 0x77eb1371, SUC_lexicon_tags_13 },
    { 0xb88b1374, SUC_lexicon_tags_25 },
    { 0x238b1375, SUC_lexicon_tags_15 },
    { 0xbc4b1375, SUC_lexicon_tags_25 },
    { 0xe4ab1377, SUC_lexicon_tags_63 },
    { 0x516b1378, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce8b137a, SUC_lexicon_tags_25 },
    { 0xc1cb137a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6e6b137d, SUC_lexicon_tags_43 },
    { 0x978b137e, SUC_lexicon_tags_25 },
    { 0x862b137f, SUC_lexicon_tags_12 },
    { 0x6a4b1380, SUC_lexicon_tags_1 },
    { 0x2bcb1381, SUC_lexicon_tags_27 },
    { 0xb54b1381, SUC_lexicon_tags_17 },
    { 0x612b1380, SUC_lexicon_tags_25 },
    { 0x52cb1381, SUC_lexicon_tags_49 },
    { 0xa8ab1382, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x760b1388, SUC_lexicon_tags_60 },
    { 0xdf6b1389, SUC_lexicon_tags_15 },
    { 0x7fab138a, SUC_lexicon_tags_23 },
    { 0xc2cb1388, SUC_lexicon_tags_28 },
    { 0xbbeb138b, SUC_lexicon_tags_11 },
    { 0x21ab138b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734b1391, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ab1394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4b1398, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2cb139a, SUC_lexicon_tags_15 },
    { 0xc84b139a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ab13a1, SUC_lexicon_tags_1071 },
    { 0, NULL },
    { 0xb76b13a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20b13a6, SUC_lexicon_tags_12 },
    { 0xe88b13a7, SUC_lexicon_tags_9 },
    { 0x17eb13a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470b13ac, SUC_lexicon_tags_16 },
    { 0x464b13ad, SUC_lexicon_tags_43 },
    { 0x392b13ad, SUC_lexicon_tags_79 },
    { 0xb10b13ac, SUC_lexicon_tags_47 },
    { 0x3eab13ac, SUC_lexicon_tags_14 },
    { 0x30ab13ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b8b13b3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6dab13b5, SUC_lexicon_tags_19 },
    { 0x4c8b13b6, SUC_lexicon_tags_0 },
    { 0x79eb13b5, SUC_lexicon_tags_17 },
    { 0xc3ab13b7, SUC_lexicon_tags_65 },
    { 0x684b13b9, SUC_lexicon_tags_25 },
    { 0xdf2b13ba, SUC_lexicon_tags_79 },
    { 0x448b13ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18b13c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfaeb13c2, SUC_lexicon_tags_45 },
    { 0xd96b13c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bab13c5, SUC_lexicon_tags_71 },
    { 0xf2eb13c5, SUC_lexicon_tags_1 },
    { 0x4a4b13c7, SUC_lexicon_tags_37 },
    { 0x62ab13c8, SUC_lexicon_tags_60 },
    { 0x28eb13c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb6b13cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cb13d2, SUC_lexicon_tags_2 },
    { 0x756b13d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260b13d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b13de, SUC_lexicon_tags_151 },
    { 0x3b2b13de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b13e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8b13e6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb40b13e8, SUC_lexicon_tags_0 },
    { 0x8acb13e9, SUC_lexicon_tags_68 },
    { 0xc38b13ea, SUC_lexicon_tags_7 },
    { 0xe7ab13eb, SUC_lexicon_tags_31 },
    { 0x3bcb13eb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792b13f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x806b13f3, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b13f6, SUC_lexicon_tags_23 },
    { 0xff6b13f7, SUC_lexicon_tags_425 },
    { 0xdb2b13f7, SUC_lexicon_tags_87 },
    { 0xcf4b13f8, SUC_lexicon_tags_37 },
    { 0x6cab13f6, SUC_lexicon_tags_15 },
    { 0x290b13fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24b1400, SUC_lexicon_tags_0 },
    { 0x474b1401, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36b1407, SUC_lexicon_tags_47 },
    { 0x158b1407, SUC_lexicon_tags_49 },
    { 0xf24b1409, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f0b140b, SUC_lexicon_tags_45 },
    { 0x5a0b140c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8eb140e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4b1411, SUC_lexicon_tags_13 },
    { 0x498b1412, SUC_lexicon_tags_17 },
    { 0xdd4b1412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ab1415, SUC_lexicon_tags_12 },
    { 0x248b1416, SUC_lexicon_tags_12 },
    { 0x130b1417, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x42ab1419, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc8b141b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cb141e, SUC_lexicon_tags_87 },
    { 0x9bab141f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2b1423, SUC_lexicon_tags_46 },
    { 0x240b1423, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6b1427, SUC_lexicon_tags_15 },
    { 0xc8eb1427, SUC_lexicon_tags_9 },
    { 0x2f6b1429, SUC_lexicon_tags_11 },
    { 0xdf8b1429, SUC_lexicon_tags_25 },
    { 0xcc8b142b, SUC_lexicon_tags_15 },
    { 0x80eb142b, SUC_lexicon_tags_11 },
    { 0xb38b142d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cb1432, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76b1436, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x18eb1439, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58b143f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24b1442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcdeb1444, SUC_lexicon_tags_25 },
    { 0xc06b1444, SUC_lexicon_tags_19 },
    { 0x32eb1446, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cb1449, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0b1452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842b1458, SUC_lexicon_tags_37 },
    { 0xe6cb1459, SUC_lexicon_tags_69 },
    { 0xfd6b145a, SUC_lexicon_tags_17 },
    { 0x540b145a, SUC_lexicon_tags_16 },
    { 0x9acb145b, SUC_lexicon_tags_32 },
    { 0x816b1459, SUC_lexicon_tags_11 },
    { 0xe12b145e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4b1462, SUC_lexicon_tags_30 },
    { 0xe4cb1463, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x532b1466, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd58b1468, SUC_lexicon_tags_26 },
    { 0x620b1468, SUC_lexicon_tags_47 },
    { 0x904b146a, SUC_lexicon_tags_7 },
    { 0xe68b1469, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cb146f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aeb1472, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21ab1474, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36eb1478, SUC_lexicon_tags_25 },
    { 0xf5cb1478, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cb1481, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc0b1483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8b1486, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c6b1488, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa72b148a, SUC_lexicon_tags_6 },
    { 0x198b148b, SUC_lexicon_tags_26 },
    { 0x1aeb148c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3feb1495, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x984b1497, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6a0b1499, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcb149f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcceb14a6, SUC_lexicon_tags_29 },
    { 0x5c4b14a7, SUC_lexicon_tags_14 },
    { 0x5eeb14a7, SUC_lexicon_tags_28 },
    { 0x694b14a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6b14b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeecb14b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacb14b9, SUC_lexicon_tags_1 },
    { 0xb9eb14b9, SUC_lexicon_tags_1 },
    { 0xaa4b14bb, SUC_lexicon_tags_104 },
    { 0x33cb14bc, SUC_lexicon_tags_37 },
    { 0x552b14bc, SUC_lexicon_tags_38 },
    { 0x610b14ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6b14c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ab14c6, SUC_lexicon_tags_37 },
    { 0x242b14c6, SUC_lexicon_tags_12 },
    { 0x92ab14c8, SUC_lexicon_tags_25 },
    { 0x398b14c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3c0b14cb, SUC_lexicon_tags_33 },
    { 0x146b14cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ab14cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x832b14d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2bab14d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cb14d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52b14dc, SUC_lexicon_tags_65 },
    { 0xdf8b14dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dab14ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b14ed, SUC_lexicon_tags_23 },
    { 0x22b14ee, SUC_lexicon_tags_1 },
    { 0xae8b14ee, SUC_lexicon_tags_43 },
    { 0xc80b14f0, SUC_lexicon_tags_12 },
    { 0x3e8b14f1, SUC_lexicon_tags_1 },
    { 0xf00b14f1, SUC_lexicon_tags_118 },
    { 0x82eb14f2, SUC_lexicon_tags_17 },
    { 0xdd2b14f2, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b14f7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb14fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8dcb14fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cb1501, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50ab1503, SUC_lexicon_tags_0 },
    { 0xc40b1504, SUC_lexicon_tags_6 },
    { 0x360b1505, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6b1508, SUC_lexicon_tags_27 },
    { 0xd7cb1509, SUC_lexicon_tags_15 },
    { 0xa92b150a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd70b150c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6b1516, SUC_lexicon_tags_25 },
    { 0xeb8b1516, SUC_lexicon_tags_23 },
    { 0x238b1518, SUC_lexicon_tags_60 },
    { 0x3bcb1517, SUC_lexicon_tags_25 },
    { 0xf04b151a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92cb151c, SUC_lexicon_tags_49 },
    { 0xab0b151c, SUC_lexicon_tags_7 },
    { 0x1c4b151e, SUC_lexicon_tags_1 },
    { 0x636b151f, SUC_lexicon_tags_11 },
    { 0x938b1520, SUC_lexicon_tags_15 },
    { 0xbbeb1521, SUC_lexicon_tags_28 },
    { 0xb92b1522, SUC_lexicon_tags_15 },
    { 0x7e2b1523, SUC_lexicon_tags_1 },
    { 0x288b1524, SUC_lexicon_tags_11 },
    { 0x810b1523, SUC_lexicon_tags_65 },
    { 0xf3ab1526, SUC_lexicon_tags_11 },
    { 0x66cb1527, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cb152a, SUC_lexicon_tags_49 },
    { 0x3bab152b, SUC_lexicon_tags_37 },
    { 0x76b152a, SUC_lexicon_tags_32 },
    { 0x48eb152d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcb1533, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7deb1539, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x704b153c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeb1542, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ab1545, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30b1548, SUC_lexicon_tags_70 },
    { 0x6d4b1549, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa4ab154b, SUC_lexicon_tags_37 },
    { 0x41eb154c, SUC_lexicon_tags_37 },
    { 0xb6ab154b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0b1553, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x824b1556, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x636b1559, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272b1561, SUC_lexicon_tags_2 },
    { 0x7b8b1562, SUC_lexicon_tags_37 },
    { 0x81ab1562, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedab1565, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208b1569, SUC_lexicon_tags_49 },
    { 0xeb8b156a, SUC_lexicon_tags_1 },
    { 0xb50b156b, SUC_lexicon_tags_21 },
    { 0xc8eb156a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eb156f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2beb1572, SUC_lexicon_tags_118 },
    { 0x4beb1572, SUC_lexicon_tags_25 },
    { 0x1feb1573, SUC_lexicon_tags_1 },
    { 0x482b1573, SUC_lexicon_tags_87 },
    { 0x464b1576, SUC_lexicon_tags_1 },
    { 0xdaeb1577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14b157a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b6b157c, SUC_lexicon_tags_37 },
    { 0xd28b157d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95ab157f, SUC_lexicon_tags_32 },
    { 0x6c2b1580, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8b2b1582, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50b1586, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0cb1588, SUC_lexicon_tags_178 },
    { 0x63cb1589, SUC_lexicon_tags_17 },
    { 0xda6b1588, SUC_lexicon_tags_15 },
    { 0x114b158b, SUC_lexicon_tags_38 },
    { 0xb3eb158b, SUC_lexicon_tags_47 },
    { 0x90cb1588, SUC_lexicon_tags_49 },
    { 0x6d6b1589, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x492b1591, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74b159a, SUC_lexicon_tags_1 },
    { 0xaeab159b, SUC_lexicon_tags_18 },
    { 0xdab159c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb159e, SUC_lexicon_tags_43 },
    { 0x338b159f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0b15a2, SUC_lexicon_tags_49 },
    { 0x54cb15a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6d0b15a5, SUC_lexicon_tags_43 },
    { 0x9b8b15a6, SUC_lexicon_tags_63 },
    { 0xcccb15a6, SUC_lexicon_tags_32 },
    { 0x43eb15a8, SUC_lexicon_tags_33 },
    { 0x1eb15a8, SUC_lexicon_tags_37 },
    { 0x5d2b15a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc1ab15ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcecb15ae, SUC_lexicon_tags_47 },
    { 0x328b15ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb48b15b1, SUC_lexicon_tags_28 },
    { 0x70b15b2, SUC_lexicon_tags_15 },
    { 0x2c2b15b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb15bc, SUC_lexicon_tags_11 },
    { 0x390b15bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0eb15c3, SUC_lexicon_tags_11 },
    { 0xcd4b15c3, SUC_lexicon_tags_12 },
    { 0x1a2b15c5, SUC_lexicon_tags_6 },
    { 0x8e4b15c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x84eb15c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20b15cc, SUC_lexicon_tags_1 },
    { 0x962b15cd, SUC_lexicon_tags_25 },
    { 0x8b0b15ce, SUC_lexicon_tags_11 },
    { 0x882b15cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100b15d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x536b15d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x166b15db, SUC_lexicon_tags_12 },
    { 0xb90b15dc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x26cb15de, SUC_lexicon_tags_8 },
    { 0x756b15df, SUC_lexicon_tags_21 },
    { 0xe50b15df, SUC_lexicon_tags_60 },
    { 0xe2b15e1, SUC_lexicon_tags_17 },
    { 0x62b15e1, SUC_lexicon_tags_23 },
    { 0x836b15e3, SUC_lexicon_tags_2 },
    { 0xb00b15e4, SUC_lexicon_tags_37 },
    { 0xdeb15e2, SUC_lexicon_tags_39 },
    { 0x2feb15df, SUC_lexicon_tags_17 },
    { 0xa8ab15e5, SUC_lexicon_tags_23 },
    { 0x3a0b15e4, SUC_lexicon_tags_33 },
    { 0x778b15e6, SUC_lexicon_tags_37 },
    { 0xc6cb15ea, SUC_lexicon_tags_43 },
    { 0x6bab15ea, SUC_lexicon_tags_47 },
    { 0x51ab15ec, SUC_lexicon_tags_11 },
    { 0x44cb15ea, SUC_lexicon_tags_2 },
    { 0x254b15ee, SUC_lexicon_tags_204 },
    { 0x40ab15ef, SUC_lexicon_tags_37 },
    { 0x228b15ef, SUC_lexicon_tags_29 },
    { 0xae4b15ee, SUC_lexicon_tags_60 },
    { 0xdab15f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88eb15f6, SUC_lexicon_tags_49 },
    { 0xc9eb15f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b0b15f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91eb15fb, SUC_lexicon_tags_35 },
    { 0xa3cb15fc, SUC_lexicon_tags_25 },
    { 0x508b15fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdcb15ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b1601, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2b1604, SUC_lexicon_tags_11 },
    { 0x348b1604, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cb160e, SUC_lexicon_tags_14 },
    { 0xb6cb160f, SUC_lexicon_tags_61 },
    { 0x6aeb1610, SUC_lexicon_tags_23 },
    { 0x6ecb1611, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ab1614, SUC_lexicon_tags_49 },
    { 0xb72b1615, SUC_lexicon_tags_82 },
    { 0xee6b1614, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06b1619, SUC_lexicon_tags_1 },
    { 0xb0b1619, SUC_lexicon_tags_29 },
    { 0xafab161a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eb161e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd8b1620, SUC_lexicon_tags_25 },
    { 0xed0b1620, SUC_lexicon_tags_19 },
    { 0x746b1621, SUC_lexicon_tags_25 },
    { 0x94cb1623, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ab1626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab1629, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4b162d, SUC_lexicon_tags_33 },
    { 0xf72b162e, SUC_lexicon_tags_15 },
    { 0x23ab162f, SUC_lexicon_tags_15 },
    { 0xd86b162f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674b1638, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4b163d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eb1648, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd7cb164a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacb164f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64b1656, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9cb1658, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b165d, SUC_lexicon_tags_26 },
    { 0x84eb165e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ab1661, SUC_lexicon_tags_63 },
    { 0x9bab1662, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78b1667, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4eb166b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a0b166d, SUC_lexicon_tags_12 },
    { 0x43ab166e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8b1671, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9beb1674, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69eb167a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6b167d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cb1680, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x674b1683, SUC_lexicon_tags_30 },
    { 0xaacb1684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x224b1686, SUC_lexicon_tags_28 },
    { 0x74ab1687, SUC_lexicon_tags_11 },
    { 0x9dcb1688, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324b1691, SUC_lexicon_tags_45 },
    { 0x1e4b1692, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc94b1694, SUC_lexicon_tags_11 },
    { 0xf2ab1694, SUC_lexicon_tags_17 },
    { 0xb38b1696, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7dab1698, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2b169b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eb16a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ab16a3, SUC_lexicon_tags_3 },
    { 0x766b16a4, SUC_lexicon_tags_14 },
    { 0x828b16a5, SUC_lexicon_tags_19 },
    { 0xe94b16a6, SUC_lexicon_tags_38 },
    { 0x6e2b16a6, SUC_lexicon_tags_15 },
    { 0x524b16a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb9ab16aa, SUC_lexicon_tags_37 },
    { 0xbf2b16aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8b16b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeb16b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacb16be, SUC_lexicon_tags_17 },
    { 0x658b16bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6b16c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77cb16c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b6b16ca, SUC_lexicon_tags_23 },
    { 0x996b16ca, SUC_lexicon_tags_27 },
    { 0xf1ab16cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8b16d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4b16d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36eb16d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b16de, SUC_lexicon_tags_32 },
    { 0x86cb16df, SUC_lexicon_tags_25 },
    { 0xe50b16de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6b16e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cb16eb, SUC_lexicon_tags_31 },
    { 0x23eb16eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfacb16ee, SUC_lexicon_tags_37 },
    { 0xac6b16ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x534b16f1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xffeb16f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836b16f7, SUC_lexicon_tags_11 },
    { 0x19ab16f8, SUC_lexicon_tags_30 },
    { 0x606b16f9, SUC_lexicon_tags_11 },
    { 0x5a0b16fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2b16fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bab16ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532b1703, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xee2b1705, SUC_lexicon_tags_19 },
    { 0xf6ab1705, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3e4b1708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b170b, SUC_lexicon_tags_11 },
    { 0x326b170b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd0b170e, SUC_lexicon_tags_19 },
    { 0x182b170f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cb1714, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244b1718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10ab171a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ab171e, SUC_lexicon_tags_45 },
    { 0xb84b171f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ab1724, SUC_lexicon_tags_14 },
    { 0x7f8b1725, SUC_lexicon_tags_49 },
    { 0xa56b1725, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x508b1729, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7aab172b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb172e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85ab1730, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90b1732, SUC_lexicon_tags_14 },
    { 0x930b1733, SUC_lexicon_tags_15 },
    { 0x30eb1734, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ab1736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x588b1739, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818b173f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00b1743, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0b1747, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcb174a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182b174e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60cb1750, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160b1756, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bab175a, SUC_lexicon_tags_17 },
    { 0x546b175b, SUC_lexicon_tags_19 },
    { 0x330b175b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8cab175e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362b1766, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedeb1769, SUC_lexicon_tags_60 },
    { 0xcacb1769, SUC_lexicon_tags_11 },
    { 0x7ccb176b, SUC_lexicon_tags_38 },
    { 0x108b176b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x484b176f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6b1774, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa60b1776, SUC_lexicon_tags_42 },
    { 0xefeb1776, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38b177a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614b177f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x682b1781, SUC_lexicon_tags_6 },
    { 0xea6b1782, SUC_lexicon_tags_37 },
    { 0x200b1782, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27eb1788, SUC_lexicon_tags_13 },
    { 0x2cb1788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdceb178b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc22b178d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ab1790, SUC_lexicon_tags_27 },
    { 0x34eb1790, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41eb1793, SUC_lexicon_tags_15 },
    { 0x97ab1793, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0b1798, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ab179f, SUC_lexicon_tags_30 },
    { 0x8a0b17a0, SUC_lexicon_tags_68 },
    { 0xff4b17a1, SUC_lexicon_tags_65 },
    { 0x24cb17a1, SUC_lexicon_tags_7 },
    { 0x7fab17a3, SUC_lexicon_tags_12 },
    { 0x5a4b17a3, SUC_lexicon_tags_624 },
    { 0x536b17a5, SUC_lexicon_tags_6 },
    { 0x474b17a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadeb17aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9c0b17ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b17af, SUC_lexicon_tags_0 },
    { 0x8c6b17b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2b17b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeab17bc, SUC_lexicon_tags_30 },
    { 0x474b17bc, SUC_lexicon_tags_43 },
    { 0xf10b17bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cb17c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x596b17c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178b17ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74cb17cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4aab17ce, SUC_lexicon_tags_19 },
    { 0x8eeb17ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x770b17d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2b17d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806b17df, SUC_lexicon_tags_15 },
    { 0x12cb17df, SUC_lexicon_tags_1 },
    { 0x304b17df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eb17e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50b17ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132b17f3, SUC_lexicon_tags_2 },
    { 0xa86b17f4, SUC_lexicon_tags_17 },
    { 0x68eb17f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb17fa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4eb17fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34b1802, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x820b1804, SUC_lexicon_tags_11 },
    { 0xaeeb1804, SUC_lexicon_tags_25 },
    { 0xe32b1804, SUC_lexicon_tags_96 },
    { 0x6f4b1807, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2deb1809, SUC_lexicon_tags_25 },
    { 0x6ecb180a, SUC_lexicon_tags_38 },
    { 0x61eb1809, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8b1813, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae4b1815, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8cb1817, SUC_lexicon_tags_7 },
    { 0x75cb1818, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494b181c, SUC_lexicon_tags_37 },
    { 0xafcb181c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x328b1820, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x3c8b1822, SUC_lexicon_tags_37 },
    { 0x378b1823, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ab1826, SUC_lexicon_tags_43 },
    { 0xa6cb1827, SUC_lexicon_tags_14 },
    { 0x4e4b1828, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952b182c, SUC_lexicon_tags_16 },
    { 0xaaeb182d, SUC_lexicon_tags_49 },
    { 0xd52b182e, SUC_lexicon_tags_136 },
    { 0x50ab182d, SUC_lexicon_tags_43 },
    { 0x296b182f, SUC_lexicon_tags_7 },
    { 0x456b182f, SUC_lexicon_tags_7 },
    { 0x45eb182d, SUC_lexicon_tags_23 },
    { 0x14b1832, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf58b1835, SUC_lexicon_tags_37 },
    { 0x9dcb1836, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ab183b, SUC_lexicon_tags_12 },
    { 0x9f2b183b, SUC_lexicon_tags_27 },
    { 0x99cb183d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0b1840, SUC_lexicon_tags_71 },
    { 0xe92b1841, SUC_lexicon_tags_9 },
    { 0xdab1842, SUC_lexicon_tags_49 },
    { 0xbbab1842, SUC_lexicon_tags_11 },
    { 0x524b1844, SUC_lexicon_tags_1 },
    { 0x440b1845, SUC_lexicon_tags_11 },
    { 0xbd0b1844, SUC_lexicon_tags_1 },
    { 0x346b1847, SUC_lexicon_tags_43 },
    { 0xefeb1848, SUC_lexicon_tags_43 },
    { 0xe74b1843, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24b184f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x680b1852, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf58b1854, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2b185c, SUC_lexicon_tags_7 },
    { 0xa2ab185c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb1860, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eb1864, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ab1869, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238b186d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x356b186f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x800b1872, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cb1876, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3ceb1878, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8b1882, SUC_lexicon_tags_2 },
    { 0x314b1882, SUC_lexicon_tags_47 },
    { 0xb68b1884, SUC_lexicon_tags_9 },
    { 0x222b1885, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eb1888, SUC_lexicon_tags_15 },
    { 0x88cb1889, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab188d, SUC_lexicon_tags_11 },
    { 0xd8cb188e, SUC_lexicon_tags_47 },
    { 0xd96b188e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x544b1892, SUC_lexicon_tags_1528 },
    { 0xc20b1893, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b2b1895, SUC_lexicon_tags_7 },
    { 0x1b8b1896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b2b1898, SUC_lexicon_tags_22 },
    { 0x96b1899, SUC_lexicon_tags_14 },
    { 0xfc6b1899, SUC_lexicon_tags_17 },
    { 0xbbcb189b, SUC_lexicon_tags_47 },
    { 0x706b189b, SUC_lexicon_tags_55 },
    { 0x9b2b189d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768b18a1, SUC_lexicon_tags_95 },
    { 0x84eb18a2, SUC_lexicon_tags_47 },
    { 0x930b18a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0b18a6, SUC_lexicon_tags_2 },
    { 0x332b18a7, SUC_lexicon_tags_9 },
    { 0x3ab18a8, SUC_lexicon_tags_15 },
    { 0x862b18a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96b18b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x404b18b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6b18bb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x756b18bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcb18c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x616b18c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab18c6, SUC_lexicon_tags_47 },
    { 0x2b0b18c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeb18cb, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xd16b18cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x80b18cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x108b18d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96b18d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf80b18d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcb18dc, SUC_lexicon_tags_7 },
    { 0xb38b18dd, SUC_lexicon_tags_14 },
    { 0xbacb18dd, SUC_lexicon_tags_25 },
    { 0xe80b18df, SUC_lexicon_tags_7 },
    { 0x4cb18e0, SUC_lexicon_tags_9 },
    { 0xc82b18df, SUC_lexicon_tags_2 },
    { 0xe9ab18e2, SUC_lexicon_tags_15 },
    { 0x88ab18dd, SUC_lexicon_tags_47 },
    { 0xa38b18e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x12eb18e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224b18ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aab18f2, SUC_lexicon_tags_37 },
    { 0x64b18f3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa84b18f5, SUC_lexicon_tags_45 },
    { 0x4c4b18f6, SUC_lexicon_tags_25 },
    { 0x848b18f7, SUC_lexicon_tags_26 },
    { 0xa4eb18f5, SUC_lexicon_tags_37 },
    { 0xd0eb18f9, SUC_lexicon_tags_17 },
    { 0x3d0b18f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x410b18fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2ab18ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8b1902, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688b1906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cb190a, SUC_lexicon_tags_28 },
    { 0xa40b190a, SUC_lexicon_tags_49 },
    { 0x92cb190c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296b1910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x424b1912, SUC_lexicon_tags_10 },
    { 0xc4eb1913, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee0b1915, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x82cb1917, SUC_lexicon_tags_45 },
    { 0x374b1917, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6eeb191a, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x940b191c, SUC_lexicon_tags_64 },
    { 0x5eeb191d, SUC_lexicon_tags_31 },
    { 0xc46b191e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcb1921, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x952b1924, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ab1927, SUC_lexicon_tags_19 },
    { 0x6ab1928, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b1936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46ab1938, SUC_lexicon_tags_37 },
    { 0xff6b1939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b193c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38cb193e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66b1944, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x152b1947, SUC_lexicon_tags_15 },
    { 0x71eb1947, SUC_lexicon_tags_23 },
    { 0xa10b1949, SUC_lexicon_tags_11 },
    { 0x5f0b194a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64b194d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c2b194f, SUC_lexicon_tags_25 },
    { 0x9a2b194f, SUC_lexicon_tags_27 },
    { 0x7aab1951, SUC_lexicon_tags_30 },
    { 0xb92b1952, SUC_lexicon_tags_10 },
    { 0x30ab1953, SUC_lexicon_tags_69 },
    { 0xcaeb1952, SUC_lexicon_tags_45 },
    { 0x10eb1955, SUC_lexicon_tags_37 },
    { 0x304b1952, SUC_lexicon_tags_12 },
    { 0xfe2b1957, SUC_lexicon_tags_46 },
    { 0xe7cb1957, SUC_lexicon_tags_8 },
    { 0xc56b1959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4b195c, SUC_lexicon_tags_43 },
    { 0xa70b195c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcc4b195f, SUC_lexicon_tags_87 },
    { 0x2a6b1960, SUC_lexicon_tags_2 },
    { 0x344b1961, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b1968, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ab196b, SUC_lexicon_tags_4 },
    { 0xe9eb196c, SUC_lexicon_tags_56 },
    { 0xeceb196c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xadcb196f, SUC_lexicon_tags_0 },
    { 0xcc4b1970, SUC_lexicon_tags_60 },
    { 0xb18b1971, SUC_lexicon_tags_6 },
    { 0x99cb1972, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398b1978, SUC_lexicon_tags_17 },
    { 0xe0b1979, SUC_lexicon_tags_13 },
    { 0xc4ab1978, SUC_lexicon_tags_23 },
    { 0x22eb197b, SUC_lexicon_tags_15 },
    { 0x638b197b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf2b197e, SUC_lexicon_tags_2 },
    { 0xfa2b197f, SUC_lexicon_tags_10 },
    { 0x23cb1980, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cb1986, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51eb198a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x236b198d, SUC_lexicon_tags_15 },
    { 0xbdeb198e, SUC_lexicon_tags_49 },
    { 0xfbeb198f, SUC_lexicon_tags_14 },
    { 0x678b198d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cab1995, SUC_lexicon_tags_13 },
    { 0x590b1996, SUC_lexicon_tags_47 },
    { 0xa22b1997, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cb199c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcb19a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742b19a6, SUC_lexicon_tags_21 },
    { 0xdf4b19a7, SUC_lexicon_tags_56 },
    { 0x1b6b19a7, SUC_lexicon_tags_107 },
    { 0x2feb19a6, SUC_lexicon_tags_4 },
    { 0x286b19a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404b19b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cb19c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd68b19c4, SUC_lexicon_tags_15 },
    { 0x1e2b19c4, SUC_lexicon_tags_34 },
    { 0xd58b19c5, SUC_lexicon_tags_37 },
    { 0x58eb19c7, SUC_lexicon_tags_31 },
    { 0x30cb19c8, SUC_lexicon_tags_17 },
    { 0x29cb19c4, SUC_lexicon_tags_25 },
    { 0x10cb19ca, SUC_lexicon_tags_1 },
    { 0x5deb19cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x388b19cd, SUC_lexicon_tags_2 },
    { 0xd26b19ce, SUC_lexicon_tags_17 },
    { 0x58b19cf, SUC_lexicon_tags_10 },
    { 0x6dcb19d0, SUC_lexicon_tags_60 },
    { 0xa4b19d1, SUC_lexicon_tags_104 },
    { 0xf9cb19d0, SUC_lexicon_tags_7 },
    { 0xf7eb19d3, SUC_lexicon_tags_66 },
    { 0x596b19d4, SUC_lexicon_tags_75 },
    { 0x3b2b19d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16b19d8, SUC_lexicon_tags_91 },
    { 0x6eab19d9, SUC_lexicon_tags_17 },
    { 0x630b19da, SUC_lexicon_tags_63 },
    { 0xdb0b19db, SUC_lexicon_tags_28 },
    { 0xa32b19dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4ab19de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xddab19e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24b19e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0b19e7, SUC_lexicon_tags_47 },
    { 0x1e8b19e7, SUC_lexicon_tags_25 },
    { 0xa98b19e9, SUC_lexicon_tags_60 },
    { 0x842b19ea, SUC_lexicon_tags_46 },
    { 0xe40b19e7, SUC_lexicon_tags_27 },
    { 0x490b19ec, SUC_lexicon_tags_45 },
    { 0xb38b19ed, SUC_lexicon_tags_33 },
    { 0x8e0b19ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342b19f2, SUC_lexicon_tags_37 },
    { 0x424b19f3, SUC_lexicon_tags_11 },
    { 0x292b19f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692b19f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20b19fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4acb19fc, SUC_lexicon_tags_45 },
    { 0xa7cb19fd, SUC_lexicon_tags_761 },
    { 0xc2eb19fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb1a04, SUC_lexicon_tags_60 },
    { 0x376b1a05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd6b1a07, SUC_lexicon_tags_17 },
    { 0x138b1a08, SUC_lexicon_tags_8 },
    { 0x1b6b1a08, SUC_lexicon_tags_65 },
    { 0x3beb1a09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490b1a10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93ab1a12, SUC_lexicon_tags_19 },
    { 0x600b1a13, SUC_lexicon_tags_2 },
    { 0xaaab1a14, SUC_lexicon_tags_15 },
    { 0x1b2b1a14, SUC_lexicon_tags_17 },
    { 0x5dcb1a16, SUC_lexicon_tags_43 },
    { 0xd44b1a16, SUC_lexicon_tags_49 },
    { 0x896b1a17, SUC_lexicon_tags_0 },
    { 0xd02b1a19, SUC_lexicon_tags_25 },
    { 0x982b1a16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x704b1a1c, SUC_lexicon_tags_23 },
    { 0x564b1a1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x77ab1a1f, SUC_lexicon_tags_49 },
    { 0xdc6b1a1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67ab1a22, SUC_lexicon_tags_8 },
    { 0x80eb1a22, SUC_lexicon_tags_84 },
    { 0x91eb1a24, SUC_lexicon_tags_37 },
    { 0xd56b1a23, SUC_lexicon_tags_14 },
    { 0xc0eb1a26, SUC_lexicon_tags_30 },
    { 0x410b1a26, SUC_lexicon_tags_1 },
    { 0xe8eb1a28, SUC_lexicon_tags_6 },
    { 0x75eb1a28, SUC_lexicon_tags_15 },
    { 0xb88b1a2a, SUC_lexicon_tags_37 },
    { 0x13ab1a2b, SUC_lexicon_tags_29 },
    { 0x5d4b1a2c, SUC_lexicon_tags_17 },
    { 0x2f2b1a26, SUC_lexicon_tags_37 },
    { 0xa4ab1a23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688b1a33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcb1a36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x354b1a38, SUC_lexicon_tags_8 },
    { 0x868b1a39, SUC_lexicon_tags_60 },
    { 0xd16b1a39, SUC_lexicon_tags_11 },
    { 0x69eb1a39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4b1a43, SUC_lexicon_tags_39 },
    { 0xa42b1a43, SUC_lexicon_tags_60 },
    { 0x666b1a43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ab1a49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2b1a53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc4b1a55, SUC_lexicon_tags_29 },
    { 0x57cb1a56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6b1a59, SUC_lexicon_tags_11 },
    { 0xc96b1a5a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe18b1a5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2b1a5f, SUC_lexicon_tags_66 },
    { 0xcacb1a5f, SUC_lexicon_tags_65 },
    { 0xa18b1a5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540b1a67, SUC_lexicon_tags_46 },
    { 0xa34b1a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ab1a6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe88b1a6d, SUC_lexicon_tags_46 },
    { 0xfb6b1a6e, SUC_lexicon_tags_9 },
    { 0x83eb1a6d, SUC_lexicon_tags_7 },
    { 0xf92b1a70, SUC_lexicon_tags_68 },
    { 0x73ab1a70, SUC_lexicon_tags_37 },
    { 0x320b1a72, SUC_lexicon_tags_21 },
    { 0x69eb1a6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb1a79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d6b1a7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fcb1a7d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x28cb1a80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x626b1a82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44eb1a84, SUC_lexicon_tags_11 },
    { 0x478b1a84, SUC_lexicon_tags_8 },
    { 0xe40b1a86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x714b1a89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ab1a8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0b1a94, SUC_lexicon_tags_25 },
    { 0x15ab1a94, SUC_lexicon_tags_11 },
    { 0xd8b1a95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4b1a9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acb1aa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0b1aac, SUC_lexicon_tags_49 },
    { 0x8f2b1aad, SUC_lexicon_tags_17 },
    { 0x3f2b1aae, SUC_lexicon_tags_11 },
    { 0xe36b1aae, SUC_lexicon_tags_79 },
    { 0x224b1ab0, SUC_lexicon_tags_49 },
    { 0xb86b1ab1, SUC_lexicon_tags_8 },
    { 0x9e8b1aae, SUC_lexicon_tags_87 },
    { 0xfb4b1ab1, SUC_lexicon_tags_63 },
    { 0xd68b1ab4, SUC_lexicon_tags_33 },
    { 0x40eb1ab1, SUC_lexicon_tags_12 },
    { 0x648b1ab6, SUC_lexicon_tags_39 },
    { 0x728b1ab6, SUC_lexicon_tags_65 },
    { 0x388b1ab8, SUC_lexicon_tags_1 },
    { 0x8c4b1ab2, SUC_lexicon_tags_2 },
    { 0xeacb1aba, SUC_lexicon_tags_7 },
    { 0xe90b1abb, SUC_lexicon_tags_10 },
    { 0x904b1abb, SUC_lexicon_tags_25 },
    { 0xb4eb1ab4, SUC_lexicon_tags_57 },
    { 0x7e6b1abe, SUC_lexicon_tags_28 },
    { 0xaacb1ab3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6b1ac5, SUC_lexicon_tags_9 },
    { 0x216b1ac6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x646b1ac8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x524b1acb, SUC_lexicon_tags_19 },
    { 0xf70b1acc, SUC_lexicon_tags_63 },
    { 0xeceb1acc, SUC_lexicon_tags_11 },
    { 0xd0ab1ace, SUC_lexicon_tags_17 },
    { 0xb0ab1acb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eab1ad7, SUC_lexicon_tags_14 },
    { 0xaa4b1ad8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecb1add, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x838b1adf, SUC_lexicon_tags_28 },
    { 0xea4b1adf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x336b1ae3, SUC_lexicon_tags_0 },
    { 0xd50b1ae4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70cb1ae6, SUC_lexicon_tags_1 },
    { 0x4aab1ae7, SUC_lexicon_tags_0 },
    { 0x6dcb1ae6, SUC_lexicon_tags_14 },
    { 0x3acb1ae7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b1aec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2b1afd, SUC_lexicon_tags_4 },
    { 0xc06b1afd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x69eb1b01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa2b1b03, SUC_lexicon_tags_7 },
    { 0xb5ab1b04, SUC_lexicon_tags_25 },
    { 0xb9ab1b05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x968b1b07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ceb1b09, SUC_lexicon_tags_31 },
    { 0xe0cb1b0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9d8b1b0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x124b1b0f, SUC_lexicon_tags_65 },
    { 0x4e2b1b10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x140b1b12, SUC_lexicon_tags_2 },
    { 0x1e4b1b13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662b1b19, SUC_lexicon_tags_14 },
    { 0x202b1b1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550b1b1f, SUC_lexicon_tags_9 },
    { 0x436b1b1f, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cb1b25, SUC_lexicon_tags_351 },
    { 0x366b1b25, SUC_lexicon_tags_11 },
    { 0x332b1b26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b1b2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cb1b2e, SUC_lexicon_tags_13 },
    { 0x4eeb1b2f, SUC_lexicon_tags_27 },
    { 0x5eeb1b2e, SUC_lexicon_tags_17 },
    { 0x16ab1b31, SUC_lexicon_tags_63 },
    { 0xb14b1b31, SUC_lexicon_tags_31 },
    { 0x1e0b1b32, SUC_lexicon_tags_32 },
    { 0x184b1b31, SUC_lexicon_tags_33 },
    { 0x9ccb1b35, SUC_lexicon_tags_32 },
    { 0x972b1b35, SUC_lexicon_tags_12 },
    { 0x5e0b1b34, SUC_lexicon_tags_36 },
    { 0xd7cb1b34, SUC_lexicon_tags_9 },
    { 0x552b1b33, SUC_lexicon_tags_37 },
    { 0xe26b1b38, SUC_lexicon_tags_7 },
    { 0xbb2b1b3b, SUC_lexicon_tags_11 },
    { 0xd5cb1b3c, SUC_lexicon_tags_2 },
    { 0x390b1b3d, SUC_lexicon_tags_15 },
    { 0xc94b1b3c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9d2b1b40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4b1b43, SUC_lexicon_tags_11 },
    { 0xc62b1b44, SUC_lexicon_tags_35 },
    { 0x34ab1b45, SUC_lexicon_tags_20 },
    { 0x1a6b1b43, SUC_lexicon_tags_60 },
    { 0xe08b1b44, SUC_lexicon_tags_23 },
    { 0x174b1b47, SUC_lexicon_tags_11 },
    { 0x8ceb1b46, SUC_lexicon_tags_2 },
    { 0xc2ab1b4a, SUC_lexicon_tags_25 },
    { 0xb0ab1b4b, SUC_lexicon_tags_12 },
    { 0xb26b1b4a, SUC_lexicon_tags_47 },
    { 0x98cb1b4b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xab4b1b4f, SUC_lexicon_tags_6 },
    { 0xdd2b1b4f, SUC_lexicon_tags_60 },
    { 0xf2ab1b51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164b1b57, SUC_lexicon_tags_12 },
    { 0x102b1b58, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8eab1b5a, SUC_lexicon_tags_49 },
    { 0xbb6b1b5b, SUC_lexicon_tags_12 },
    { 0x200b1b5c, SUC_lexicon_tags_26 },
    { 0x118b1b5c, SUC_lexicon_tags_33 },
    { 0x2ab1b5c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbf4b1b60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x912b1b62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe0cb1b64, SUC_lexicon_tags_17 },
    { 0xce8b1b65, SUC_lexicon_tags_25 },
    { 0x2f0b1b64, SUC_lexicon_tags_1 },
    { 0x700b1b67, SUC_lexicon_tags_47 },
    { 0xde6b1b68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eb1b6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02b1b75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6b1b78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe60b1b7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cb1b7d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x790b1b7f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaceb1b81, SUC_lexicon_tags_12 },
    { 0xf44b1b81, SUC_lexicon_tags_2 },
    { 0x752b1b83, SUC_lexicon_tags_33 },
    { 0xcf6b1b84, SUC_lexicon_tags_8 },
    { 0x1deb1b85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x158b1b87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0b1b8a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cb1b8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cb1b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cb1b94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x888b1b97, SUC_lexicon_tags_37 },
    { 0x7dcb1b97, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4b1b9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x708b1ba0, SUC_lexicon_tags_6 },
    { 0x570b1ba0, SUC_lexicon_tags_2 },
    { 0x48ab1ba2, SUC_lexicon_tags_7 },
    { 0x782b1ba3, SUC_lexicon_tags_66 },
    { 0x3b2b1ba1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x156b1ba6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eb1ba9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b1bad, SUC_lexicon_tags_116 },
    { 0xe5eb1bae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x512b1bb0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcb1bb7, SUC_lexicon_tags_7 },
    { 0x4cb1bb7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cb1bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadcb1bc1, SUC_lexicon_tags_1 },
    { 0x5d6b1bc2, SUC_lexicon_tags_11 },
    { 0x7aab1bc3, SUC_lexicon_tags_25 },
    { 0x946b1bc2, SUC_lexicon_tags_39 },
    { 0x936b1bc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf4b1bc7, SUC_lexicon_tags_28 },
    { 0x278b1bc8, SUC_lexicon_tags_37 },
    { 0x454b1bc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6b1bcc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366b1bd4, SUC_lexicon_tags_5 },
    { 0xe3eb1bd4, SUC_lexicon_tags_37 },
    { 0xc70b1bd6, SUC_lexicon_tags_10 },
    { 0x51ab1bd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544b1be0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8b1be6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cb1be9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cb1bf3, SUC_lexicon_tags_11 },
    { 0x610b1bf3, SUC_lexicon_tags_27 },
    { 0xf8cb1bf5, SUC_lexicon_tags_7 },
    { 0x5b0b1bf4, SUC_lexicon_tags_9 },
    { 0x41ab1bf7, SUC_lexicon_tags_37 },
    { 0xd10b1bf8, SUC_lexicon_tags_9 },
    { 0x2e2b1bf9, SUC_lexicon_tags_1 },
    { 0xa42b1bf9, SUC_lexicon_tags_6 },
    { 0xe9ab1bf7, SUC_lexicon_tags_47 },
    { 0x748b1bfc, SUC_lexicon_tags_39 },
    { 0x772b1bfc, SUC_lexicon_tags_25 },
    { 0x3beb1bfd, SUC_lexicon_tags_7 },
    { 0xe7eb1bff, SUC_lexicon_tags_23 },
    { 0x6ab1bfc, SUC_lexicon_tags_43 },
    { 0x4f0b1c01, SUC_lexicon_tags_0 },
    { 0x9d2b1bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeb1c08, SUC_lexicon_tags_1 },
    { 0x106b1c08, SUC_lexicon_tags_12 },
    { 0x4d6b1c0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ab1c0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa50b1c0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24b1c14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eb1c18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecb1c1d, SUC_lexicon_tags_84 },
    { 0xdfeb1c1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc2b1c20, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb1c25, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2b1c29, SUC_lexicon_tags_17 },
    { 0x516b1c2a, SUC_lexicon_tags_31 },
    { 0x8e4b1c29, SUC_lexicon_tags_107 },
    { 0x7ccb1c2c, SUC_lexicon_tags_28 },
    { 0xbe6b1c2d, SUC_lexicon_tags_43 },
    { 0xe3cb1c29, SUC_lexicon_tags_84 },
    { 0x844b1c2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbf2b1c31, SUC_lexicon_tags_28 },
    { 0xf9cb1c31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66b1c34, SUC_lexicon_tags_6 },
    { 0x882b1c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51cb1c37, SUC_lexicon_tags_23 },
    { 0x19cb1c38, SUC_lexicon_tags_12 },
    { 0xd8b1c39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8b1c3c, SUC_lexicon_tags_2 },
    { 0x524b1c3d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa7cb1c3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x908b1c42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ab1c47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866b1c4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1aab1c4f, SUC_lexicon_tags_25 },
    { 0x76ab1c50, SUC_lexicon_tags_11 },
    { 0x24eb1c50, SUC_lexicon_tags_60 },
    { 0xdd0b1c4f, SUC_lexicon_tags_38 },
    { 0xaeab1c53, SUC_lexicon_tags_11 },
    { 0x650b1c54, SUC_lexicon_tags_2 },
    { 0xc10b1c51, SUC_lexicon_tags_26 },
    { 0x658b1c52, SUC_lexicon_tags_45 },
    { 0x1deb1c57, SUC_lexicon_tags_15 },
    { 0x7fcb1c58, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2b1c5b, SUC_lexicon_tags_11 },
    { 0x9ecb1c5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e2b1c5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0b1c61, SUC_lexicon_tags_21 },
    { 0x8b4b1c62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6b1c66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59ab1c68, SUC_lexicon_tags_43 },
    { 0xabcb1c68, SUC_lexicon_tags_17 },
    { 0x49ab1c68, SUC_lexicon_tags_23 },
    { 0xebab1c6b, SUC_lexicon_tags_25 },
    { 0x288b1c6b, SUC_lexicon_tags_25 },
    { 0x70eb1c69, SUC_lexicon_tags_19 },
    { 0x972b1c6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380b1c76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b1c7a, SUC_lexicon_tags_37 },
    { 0xf20b1c7b, SUC_lexicon_tags_25 },
    { 0x86cb1c7b, SUC_lexicon_tags_8 },
    { 0x2eeb1c7d, SUC_lexicon_tags_19 },
    { 0xfecb1c7e, SUC_lexicon_tags_1 },
    { 0xf16b1c7c, SUC_lexicon_tags_12 },
    { 0x9bcb1c7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2b1c84, SUC_lexicon_tags_19 },
    { 0x828b1c84, SUC_lexicon_tags_0 },
    { 0x654b1c86, SUC_lexicon_tags_1 },
    { 0x818b1c86, SUC_lexicon_tags_15 },
    { 0xecab1c86, SUC_lexicon_tags_4 },
    { 0xbfcb1c89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeecb1c8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd8b1c8d, SUC_lexicon_tags_46 },
    { 0xc8ab1c8d, SUC_lexicon_tags_12 },
    { 0xefcb1c8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x118b1c92, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdfab1c94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62b1c97, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782b1c9d, SUC_lexicon_tags_15 },
    { 0xcfeb1c9e, SUC_lexicon_tags_27 },
    { 0x864b1c9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb1ca4, SUC_lexicon_tags_31 },
    { 0x3acb1ca4, SUC_lexicon_tags_16 },
    { 0x956b1ca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666b1cab, SUC_lexicon_tags_15 },
    { 0xf5cb1cab, SUC_lexicon_tags_64 },
    { 0x38ab1cad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94b1cb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a4b1cb5, SUC_lexicon_tags_17 },
    { 0xb94b1cb6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6b1cb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1eb1cbb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3b0b1cbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x22b1cbf, SUC_lexicon_tags_9 },
    { 0x35eb1cc0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60eb1cc4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2b1cc8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7c0b1cca, SUC_lexicon_tags_26 },
    { 0x64ab1ccb, SUC_lexicon_tags_11 },
    { 0x53cb1ccb, SUC_lexicon_tags_37 },
    { 0xc80b1cca, SUC_lexicon_tags_11 },
    { 0xeb2b1ccc, SUC_lexicon_tags_37 },
    { 0x9eab1ccf, SUC_lexicon_tags_37 },
    { 0x9aeb1cd0, SUC_lexicon_tags_11 },
    { 0xc22b1cd0, SUC_lexicon_tags_14 },
    { 0xe38b1cd2, SUC_lexicon_tags_15 },
    { 0x420b1cd2, SUC_lexicon_tags_27 },
    { 0xc32b1cd3, SUC_lexicon_tags_31 },
    { 0x804b1cd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x816b1cd8, SUC_lexicon_tags_35 },
    { 0x4eeb1cd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecb1cde, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe00b1ce0, SUC_lexicon_tags_60 },
    { 0x6f0b1ce1, SUC_lexicon_tags_17 },
    { 0x7b8b1ce0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16b1ce5, SUC_lexicon_tags_49 },
    { 0xbeb1ce5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x656b1ce8, SUC_lexicon_tags_11 },
    { 0x5fab1ce9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8feb1cec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a0b1cee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcb1cf0, SUC_lexicon_tags_27 },
    { 0xdd6b1cf0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x692b1cf4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x572b1cf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0b1cf9, SUC_lexicon_tags_4 },
    { 0x398b1cfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x90ab1cfc, SUC_lexicon_tags_21 },
    { 0xbd4b1cfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6b1d02, SUC_lexicon_tags_7 },
    { 0xeccb1d02, SUC_lexicon_tags_37 },
    { 0xe74b1d04, SUC_lexicon_tags_64 },
    { 0xa86b1d03, SUC_lexicon_tags_37 },
    { 0x1c0b1d06, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb30b1d08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0b1d0c, SUC_lexicon_tags_7 },
    { 0x9e2b1d0c, SUC_lexicon_tags_7 },
    { 0x552b1d0e, SUC_lexicon_tags_7 },
    { 0xf8b1d0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4b1d14, SUC_lexicon_tags_2 },
    { 0x7b2b1d14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4b1d1a, SUC_lexicon_tags_15 },
    { 0xfeeb1d1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2b1d21, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b1d2a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecb1d2f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98b1d32, SUC_lexicon_tags_12 },
    { 0xd3eb1d33, SUC_lexicon_tags_85 },
    { 0x88ab1d32, SUC_lexicon_tags_31 },
    { 0x2ccb1d35, SUC_lexicon_tags_37 },
    { 0x8f2b1d36, SUC_lexicon_tags_37 },
    { 0xed6b1d35, SUC_lexicon_tags_1 },
    { 0x454b1d37, SUC_lexicon_tags_37 },
    { 0xfe8b1d32, SUC_lexicon_tags_15 },
    { 0x1beb1d35, SUC_lexicon_tags_28 },
    { 0x41eb1d34, SUC_lexicon_tags_17 },
    { 0x67cb1d3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6ab1d3e, SUC_lexicon_tags_27 },
    { 0x8cab1d3f, SUC_lexicon_tags_31 },
    { 0x5dab1d40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570b1d44, SUC_lexicon_tags_25 },
    { 0xfeeb1d45, SUC_lexicon_tags_75 },
    { 0x41cb1d46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ab1d4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x612b1d4d, SUC_lexicon_tags_17 },
    { 0x850b1d4e, SUC_lexicon_tags_26 },
    { 0xb8ab1d4f, SUC_lexicon_tags_6 },
    { 0xeeeb1d50, SUC_lexicon_tags_23 },
    { 0x7c0b1d4f, SUC_lexicon_tags_25 },
    { 0x89ab1d52, SUC_lexicon_tags_14 },
    { 0xbd2b1d4d, SUC_lexicon_tags_160 },
    { 0x7e4b1d54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12b1d5b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50b1d5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe02b1d60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbeab1d62, SUC_lexicon_tags_25 },
    { 0x6aeb1d62, SUC_lexicon_tags_30 },
    { 0xcbeb1d64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x984b1d67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57eb1d70, SUC_lexicon_tags_2 },
    { 0xa4cb1d71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x380b1d74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacb1d79, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfb6b1d7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38b1d7e, SUC_lexicon_tags_27 },
    { 0x216b1d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84b1d84, SUC_lexicon_tags_80 },
    { 0xcd4b1d85, SUC_lexicon_tags_13 },
    { 0x9b6b1d85, SUC_lexicon_tags_73 },
    { 0xb5ab1d87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaa2b1d89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfeb1d8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb24b1d8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe64b1d8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726b1d95, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704b1d9c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3cb1d9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8b1da2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2b1da6, SUC_lexicon_tags_1 },
    { 0x932b1da7, SUC_lexicon_tags_45 },
    { 0xa86b1da8, SUC_lexicon_tags_61 },
    { 0x220b1da9, SUC_lexicon_tags_45 },
    { 0xfceb1daa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6b1db3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2e0b1db5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x902b1db8, SUC_lexicon_tags_37 },
    { 0x978b1db9, SUC_lexicon_tags_106 },
    { 0xf18b1dba, SUC_lexicon_tags_19 },
    { 0xf4eb1dbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x262b1dbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeab1dc0, SUC_lexicon_tags_7 },
    { 0xa0cb1dc1, SUC_lexicon_tags_44 },
    { 0xae0b1dc2, SUC_lexicon_tags_44 },
    { 0x6a4b1dc0, SUC_lexicon_tags_47 },
    { 0xbb6b1dc4, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ab1dc7, SUC_lexicon_tags_2 },
    { 0x5aab1dc8, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x5beb1dcb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358b1dd1, SUC_lexicon_tags_15 },
    { 0x2a4b1dd1, SUC_lexicon_tags_1 },
    { 0xe76b1dd3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cb1dd6, SUC_lexicon_tags_61 },
    { 0x676b1dd7, SUC_lexicon_tags_0 },
    { 0x24ab1dd6, SUC_lexicon_tags_64 },
    { 0xc44b1dd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4b1ddd, SUC_lexicon_tags_1 },
    { 0x8bcb1dde, SUC_lexicon_tags_12 },
    { 0xc1cb1ddf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x866b1de1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cb1de4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x908b1de6, SUC_lexicon_tags_37 },
    { 0xf88b1de6, SUC_lexicon_tags_75 },
    { 0xe9ab1de7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22b1deb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6b1df3, SUC_lexicon_tags_17 },
    { 0xeb8b1df4, SUC_lexicon_tags_15 },
    { 0xcf2b1df5, SUC_lexicon_tags_27 },
    { 0xb40b1df6, SUC_lexicon_tags_7 },
    { 0x4a4b1df5, SUC_lexicon_tags_47 },
    { 0xe28b1df8, SUC_lexicon_tags_61 },
    { 0xc1eb1df5, SUC_lexicon_tags_12 },
    { 0x350b1dfa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x23eb1dfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92b1dff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960b1e04, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ab1e0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ab1e0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1eb1e10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ab1e16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18b1e1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd44b1e1e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb1e27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570b1e2c, SUC_lexicon_tags_37 },
    { 0xce4b1e2d, SUC_lexicon_tags_13 },
    { 0x7e6b1e2e, SUC_lexicon_tags_2 },
    { 0x806b1e2f, SUC_lexicon_tags_37 },
    { 0x66eb1e30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ceb1e32, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88b1e36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x874b1e38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32eb1e3a, SUC_lexicon_tags_37 },
    { 0xa02b1e3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad2b1e3d, SUC_lexicon_tags_25 },
    { 0x63eb1e3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x16cb1e41, SUC_lexicon_tags_37 },
    { 0x968b1e42, SUC_lexicon_tags_65 },
    { 0x9c6b1e43, SUC_lexicon_tags_6 },
    { 0xdf0b1e43, SUC_lexicon_tags_15 },
    { 0xc02b1e43, SUC_lexicon_tags_11 },
    { 0x872b1e46, SUC_lexicon_tags_15 },
    { 0xbe4b1e46, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x548b1e49, SUC_lexicon_tags_40 },
    { 0x64b1e49, SUC_lexicon_tags_55 },
    { 0xeb6b1e4b, SUC_lexicon_tags_63 },
    { 0x268b1e49, SUC_lexicon_tags_12 },
    { 0xa46b1e4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0b1e50, SUC_lexicon_tags_17 },
    { 0xfa2b1e51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd48b1e53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xceb1e55, SUC_lexicon_tags_22 },
    { 0x6d4b1e55, SUC_lexicon_tags_17 },
    { 0x2c8b1e56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98b1e5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x560b1e61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7c6b1e63, SUC_lexicon_tags_27 },
    { 0x552b1e64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0b1e67, SUC_lexicon_tags_2 },
    { 0x3cab1e68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb1e70, SUC_lexicon_tags_2 },
    { 0xb5eb1e70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508b1e78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610b1e80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0b1e83, SUC_lexicon_tags_8 },
    { 0xbecb1e84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4cb1e86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa06b1e88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86b1e90, SUC_lexicon_tags_15 },
    { 0x2ccb1e90, SUC_lexicon_tags_12 },
    { 0xef6b1e92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8b1e96, SUC_lexicon_tags_38 },
    { 0x974b1e96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372b1e9d, SUC_lexicon_tags_25 },
    { 0xf66b1e9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x304b1ea1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x864b1ea4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccb1eab, SUC_lexicon_tags_47 },
    { 0xc96b1eab, SUC_lexicon_tags_12 },
    { 0xa00b1eab, SUC_lexicon_tags_25 },
    { 0x76cb1eac, SUC_lexicon_tags_15 },
    { 0xafab1eaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x312b1eb1, SUC_lexicon_tags_21 },
    { 0x500b1eb1, SUC_lexicon_tags_262 },
    { 0x486b1eb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee0b1eb5, SUC_lexicon_tags_8 },
    { 0xbdab1eb6, SUC_lexicon_tags_11 },
    { 0x7aeb1eb7, SUC_lexicon_tags_0 },
    { 0x490b1eb8, SUC_lexicon_tags_25 },
    { 0x88b1eb8, SUC_lexicon_tags_12 },
    { 0xf8cb1eb7, SUC_lexicon_tags_37 },
    { 0xa8b1eb9, SUC_lexicon_tags_17 },
    { 0xe84b1ebc, SUC_lexicon_tags_37 },
    { 0xea2b1ebd, SUC_lexicon_tags_87 },
    { 0x876b1eb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eb1ec4, SUC_lexicon_tags_61 },
    { 0x6a4b1ec5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x712b1ec7, SUC_lexicon_tags_15 },
    { 0x83eb1ec8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeeb1ecb, SUC_lexicon_tags_27 },
    { 0x870b1ecb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4dcb1ece, SUC_lexicon_tags_15 },
    { 0xa16b1ecf, SUC_lexicon_tags_65 },
    { 0x306b1ecf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44b1ed6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374b1edb, SUC_lexicon_tags_7 },
    { 0x478b1edc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752b1ee5, SUC_lexicon_tags_2 },
    { 0xe40b1ee5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964b1eec, SUC_lexicon_tags_38 },
    { 0xca6b1eec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x534b1eef, SUC_lexicon_tags_6 },
    { 0xb58b1ef0, SUC_lexicon_tags_1 },
    { 0x2fab1ef1, SUC_lexicon_tags_2 },
    { 0xd96b1ef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd00b1ef4, SUC_lexicon_tags_37 },
    { 0x96eb1ef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ab1ef8, SUC_lexicon_tags_23 },
    { 0x7b6b1ef9, SUC_lexicon_tags_12 },
    { 0xdb2b1efa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98b1efd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866b1f01, SUC_lexicon_tags_1 },
    { 0x64b1f02, SUC_lexicon_tags_7 },
    { 0xdbab1f03, SUC_lexicon_tags_28 },
    { 0x644b1f04, SUC_lexicon_tags_2 },
    { 0xc78b1f05, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cb1f08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa16b1f0a, SUC_lexicon_tags_30 },
    { 0xdd8b1f0b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2f8b1f0d, SUC_lexicon_tags_2 },
    { 0xf4b1f0d, SUC_lexicon_tags_26 },
    { 0x67ab1f0f, SUC_lexicon_tags_9 },
    { 0x81eb1f0f, SUC_lexicon_tags_40 },
    { 0x752b1f0f, SUC_lexicon_tags_55 },
    { 0x88cb1f12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56cb1f14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196b1f18, SUC_lexicon_tags_11 },
    { 0x5bab1f19, SUC_lexicon_tags_14 },
    { 0x8c2b1f19, SUC_lexicon_tags_12 },
    { 0x992b1f19, SUC_lexicon_tags_37 },
    { 0x500b1f1b, SUC_lexicon_tags_37 },
    { 0xf26b1f1a, SUC_lexicon_tags_31 },
    { 0xbc0b1f18, SUC_lexicon_tags_37 },
    { 0x2bab1f1c, SUC_lexicon_tags_2 },
    { 0xed6b1f20, SUC_lexicon_tags_17 },
    { 0x712b1f20, SUC_lexicon_tags_30 },
    { 0x90eb1f22, SUC_lexicon_tags_37 },
    { 0xbeab1f23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62eb1f25, SUC_lexicon_tags_61 },
    { 0xd6cb1f26, SUC_lexicon_tags_25 },
    { 0x430b1f27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62eb1f2c, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x734b1f2e, SUC_lexicon_tags_37 },
    { 0x2d0b1f2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7eb1f39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb1f3c, SUC_lexicon_tags_1 },
    { 0x2ceb1f3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2f4b1f3f, SUC_lexicon_tags_21 },
    { 0x51cb1f40, SUC_lexicon_tags_1 },
    { 0x914b1f3f, SUC_lexicon_tags_33 },
    { 0x6a4b1f42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfab1f46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x158b1f48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebab1f4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72b1f50, SUC_lexicon_tags_37 },
    { 0xa22b1f51, SUC_lexicon_tags_26 },
    { 0x4e6b1f52, SUC_lexicon_tags_25 },
    { 0xb18b1f50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2b1f58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa94b1f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b1f5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaeb1f61, SUC_lexicon_tags_1 },
    { 0x8cb1f62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x42b1f64, SUC_lexicon_tags_12 },
    { 0xdf0b1f64, SUC_lexicon_tags_37 },
    { 0xdcab1f65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0b1f69, SUC_lexicon_tags_30 },
    { 0xeeb1f6a, SUC_lexicon_tags_23 },
    { 0x9bab1f6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ab1f76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cb1f7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66b1f7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x48eb1f82, SUC_lexicon_tags_66 },
    { 0xacb1f83, SUC_lexicon_tags_46 },
    { 0x6b8b1f82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356b1f88, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc0cb1f8a, SUC_lexicon_tags_19 },
    { 0x854b1f8a, SUC_lexicon_tags_30 },
    { 0xc0ab1f8c, SUC_lexicon_tags_39 },
    { 0xc16b1f8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ab1f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab1f97, SUC_lexicon_tags_63 },
    { 0x2e8b1f97, SUC_lexicon_tags_15 },
    { 0x74b1f99, SUC_lexicon_tags_2 },
    { 0x334b1f9a, SUC_lexicon_tags_15 },
    { 0xd26b1f9b, SUC_lexicon_tags_12 },
    { 0x4feb1f9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b1f9f, SUC_lexicon_tags_49 },
    { 0x3c0b1fa0, SUC_lexicon_tags_33 },
    { 0x654b1fa0, SUC_lexicon_tags_17 },
    { 0x9feb1fa0, SUC_lexicon_tags_48 },
    { 0x90eb1fa3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282b1fa7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x70b1fa9, SUC_lexicon_tags_38 },
    { 0xc10b1faa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb0eb1fac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ab1fb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc66b1fb3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb1fb7, SUC_lexicon_tags_15 },
    { 0xe64b1fb7, SUC_lexicon_tags_11 },
    { 0x136b1fb9, SUC_lexicon_tags_518 },
    { 0x3d0b1fba, SUC_lexicon_tags_7 },
    { 0x210b1fbb, SUC_lexicon_tags_37 },
    { 0x66eb1fbc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6b1fbf, SUC_lexicon_tags_11 },
    { 0x4eeb1fbf, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984b1fc6, SUC_lexicon_tags_51 },
    { 0xc08b1fc6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x444b1fc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x942b1fcc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40b1fd1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x448b1fd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x124b1fd5, SUC_lexicon_tags_14 },
    { 0x342b1fd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97eb1fd8, SUC_lexicon_tags_37 },
    { 0x2d0b1fd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad2b1fdb, SUC_lexicon_tags_37 },
    { 0x5eab1fdc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xfe8b1fde, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ab1fe2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40b1fe6, SUC_lexicon_tags_2 },
    { 0xb3eb1fe6, SUC_lexicon_tags_11 },
    { 0x2a0b1fe8, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4b1feb, SUC_lexicon_tags_15 },
    { 0xfe0b1feb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd6cb1fee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc2b1ff0, SUC_lexicon_tags_6 },
    { 0x66cb1ff0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922b1ff5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbceb1ff8, SUC_lexicon_tags_27 },
    { 0xc2cb1ff9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b1ffd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xebcb1fff, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8b2002, SUC_lexicon_tags_28 },
    { 0x452b2003, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x470b2006, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x848b2008, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b200a, SUC_lexicon_tags_70 },
    { 0x33cb200b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ab200f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6b2013, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4b2016, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0b201a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x714b201c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcb2023, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4b2026, SUC_lexicon_tags_8 },
    { 0xefeb2026, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b202b, SUC_lexicon_tags_25 },
    { 0x9e4b202c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200b2034, SUC_lexicon_tags_55 },
    { 0xc0b2035, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x898b2037, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb203a, SUC_lexicon_tags_46 },
    { 0xffab203b, SUC_lexicon_tags_37 },
    { 0x1e4b203a, SUC_lexicon_tags_1 },
    { 0x674b203b, SUC_lexicon_tags_15 },
    { 0xb1ab203e, SUC_lexicon_tags_15 },
    { 0xffab203f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2b2047, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb204a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfab204c, SUC_lexicon_tags_3 },
    { 0x1aeb204d, SUC_lexicon_tags_6 },
    { 0xff4b204d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9eb2051, SUC_lexicon_tags_27 },
    { 0xe7cb2052, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86b2056, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7eb205a, SUC_lexicon_tags_26 },
    { 0x2fcb205b, SUC_lexicon_tags_63 },
    { 0xd34b205c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66b2060, SUC_lexicon_tags_31 },
    { 0x29ab2060, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6b2b2063, SUC_lexicon_tags_11 },
    { 0x910b2064, SUC_lexicon_tags_7 },
    { 0xdeab2065, SUC_lexicon_tags_17 },
    { 0xa8ab2066, SUC_lexicon_tags_38 },
    { 0x4d6b2066, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf5eb2069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eb206d, SUC_lexicon_tags_11 },
    { 0xdbcb206d, SUC_lexicon_tags_25 },
    { 0x8e6b206d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ab207d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3cb207f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80b2083, SUC_lexicon_tags_14 },
    { 0x3c0b2083, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda6b2086, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcb2089, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x548b208b, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68b208e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x248b2090, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7a0b2092, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374b209c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16b209f, SUC_lexicon_tags_15 },
    { 0xefeb20a0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56b20a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2b20a7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf2cb20a9, SUC_lexicon_tags_7 },
    { 0xd5ab20aa, SUC_lexicon_tags_46 },
    { 0x80eb20ab, SUC_lexicon_tags_29 },
    { 0x80ab20ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x71ab20ae, SUC_lexicon_tags_38 },
    { 0x914b20af, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40b20b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x160b20b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936b20bc, SUC_lexicon_tags_23 },
    { 0x43ab20bd, SUC_lexicon_tags_15 },
    { 0x166b20be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f0b20c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b20c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4b20c9, SUC_lexicon_tags_47 },
    { 0x864b20ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ab20d0, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x408b20d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb20d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x720b20da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1a8b20dc, SUC_lexicon_tags_9 },
    { 0x4eb20dc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc5ab20df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb20e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8b20ea, SUC_lexicon_tags_47 },
    { 0x5cb20ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8b20ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8b20f2, SUC_lexicon_tags_0 },
    { 0x644b20f3, SUC_lexicon_tags_37 },
    { 0xb34b20f4, SUC_lexicon_tags_37 },
    { 0xc38b20f5, SUC_lexicon_tags_39 },
    { 0x920b20f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeaab20f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ab20fd, SUC_lexicon_tags_38 },
    { 0x832b20fe, SUC_lexicon_tags_68 },
    { 0xc96b20fe, SUC_lexicon_tags_45 },
    { 0xbaeb2100, SUC_lexicon_tags_37 },
    { 0x698b2101, SUC_lexicon_tags_17 },
    { 0x960b20fd, SUC_lexicon_tags_47 },
    { 0xf8b2103, SUC_lexicon_tags_25 },
    { 0xac4b2103, SUC_lexicon_tags_12 },
    { 0xb66b2101, SUC_lexicon_tags_47 },
    { 0x30ab2106, SUC_lexicon_tags_17 },
    { 0x174b2107, SUC_lexicon_tags_37 },
    { 0xb7cb2108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecb210e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532b2114, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2b2117, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0b211b, SUC_lexicon_tags_30 },
    { 0xe18b211b, SUC_lexicon_tags_0 },
    { 0x836b211d, SUC_lexicon_tags_49 },
    { 0xf16b211e, SUC_lexicon_tags_25 },
    { 0x6b6b211f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb8b2121, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc62b2123, SUC_lexicon_tags_19 },
    { 0x24cb2123, SUC_lexicon_tags_23 },
    { 0x578b2124, SUC_lexicon_tags_47 },
    { 0xf60b2126, SUC_lexicon_tags_10 },
    { 0xba2b2127, SUC_lexicon_tags_15 },
    { 0x870b2128, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11eb212c, SUC_lexicon_tags_21 },
    { 0x48b212c, SUC_lexicon_tags_2 },
    { 0x56eb212e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4b2134, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aab2140, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82eb2142, SUC_lexicon_tags_14 },
    { 0x970b2142, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba2b2145, SUC_lexicon_tags_43 },
    { 0x110b2146, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x916b2148, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70b214b, SUC_lexicon_tags_27 },
    { 0x78b214c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564b215f, SUC_lexicon_tags_60 },
    { 0x614b2160, SUC_lexicon_tags_14 },
    { 0x19eb2160, SUC_lexicon_tags_37 },
    { 0xcfeb2162, SUC_lexicon_tags_11 },
    { 0xfbcb2163, SUC_lexicon_tags_37 },
    { 0x6beb2162, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x636b2166, SUC_lexicon_tags_31 },
    { 0x440b2167, SUC_lexicon_tags_36 },
    { 0x560b2168, SUC_lexicon_tags_15 },
    { 0x710b2168, SUC_lexicon_tags_68 },
    { 0x7ab216a, SUC_lexicon_tags_14 },
    { 0x7dcb2166, SUC_lexicon_tags_8 },
    { 0x1bcb216c, SUC_lexicon_tags_11 },
    { 0x786b2169, SUC_lexicon_tags_37 },
    { 0x4a0b216e, SUC_lexicon_tags_25 },
    { 0x116b216e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cb2172, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160b2176, SUC_lexicon_tags_10 },
    { 0x404b2177, SUC_lexicon_tags_1 },
    { 0xfeab2176, SUC_lexicon_tags_7 },
    { 0xcd0b2179, SUC_lexicon_tags_17 },
    { 0x6ecb2177, SUC_lexicon_tags_4 },
    { 0xac2b217b, SUC_lexicon_tags_15 },
    { 0x8a6b217c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4deb217e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dab2185, SUC_lexicon_tags_1137 },
    { 0, NULL },
    { 0, NULL },
    { 0x694b2188, SUC_lexicon_tags_25 },
    { 0x178b2188, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe02b218b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb2191, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaa4b2193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eb2197, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x59eb219a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0b219d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02b21a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6beb21a2, SUC_lexicon_tags_29 },
    { 0xfb2b21a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92b21a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ab21a9, SUC_lexicon_tags_37 },
    { 0x8b0b21aa, SUC_lexicon_tags_30 },
    { 0x8cab21a9, SUC_lexicon_tags_11 },
    { 0x666b21aa, SUC_lexicon_tags_63 },
    { 0x5d4b21ad, SUC_lexicon_tags_17 },
    { 0xec8b21ae, SUC_lexicon_tags_27 },
    { 0x492b21af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab21b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828b21ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7eb21bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7cb21be, SUC_lexicon_tags_27 },
    { 0x2b21bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0b21c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3fab21c5, SUC_lexicon_tags_16 },
    { 0xb00b21c6, SUC_lexicon_tags_103 },
    { 0x8ecb21c5, SUC_lexicon_tags_18 },
    { 0xb8b21c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52b21cd, SUC_lexicon_tags_11 },
    { 0x696b21ce, SUC_lexicon_tags_37 },
    { 0x2eab21cf, SUC_lexicon_tags_27 },
    { 0x1d0b21cf, SUC_lexicon_tags_37 },
    { 0x49ab21d0, SUC_lexicon_tags_63 },
    { 0xceeb21cf, SUC_lexicon_tags_19 },
    { 0x190b21d3, SUC_lexicon_tags_33 },
    { 0x68b21d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf6eb21d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cb21db, SUC_lexicon_tags_1 },
    { 0x7f2b21dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff6b21de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96b21e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x428b21e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6b21eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ab21ef, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x566b21f2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbf4b21f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622b21fb, SUC_lexicon_tags_10 },
    { 0x302b21fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80cb21fe, SUC_lexicon_tags_65 },
    { 0x2d6b21ff, SUC_lexicon_tags_63 },
    { 0x794b2200, SUC_lexicon_tags_38 },
    { 0xcfcb2201, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cb2205, SUC_lexicon_tags_47 },
    { 0x614b2205, SUC_lexicon_tags_85 },
    { 0xe8eb2207, SUC_lexicon_tags_33 },
    { 0x1f0b2208, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36b220b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52b220f, SUC_lexicon_tags_27 },
    { 0x62b220f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472b2214, SUC_lexicon_tags_37 },
    { 0xe50b2215, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4deb2217, SUC_lexicon_tags_3 },
    { 0x1ecb2218, SUC_lexicon_tags_14 },
    { 0xcfeb2218, SUC_lexicon_tags_77 },
    { 0x636b2218, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6b221f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcb2222, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478b222c, SUC_lexicon_tags_7 },
    { 0xeceb222d, SUC_lexicon_tags_27 },
    { 0x7e2b222e, SUC_lexicon_tags_17 },
    { 0xe82b222c, SUC_lexicon_tags_11 },
    { 0x5cb222d, SUC_lexicon_tags_57 },
    { 0x7b2b2231, SUC_lexicon_tags_60 },
    { 0x4f4b2232, SUC_lexicon_tags_11 },
    { 0xd1eb222d, SUC_lexicon_tags_2 },
    { 0xe92b222d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2b2238, SUC_lexicon_tags_9 },
    { 0x6d0b2239, SUC_lexicon_tags_37 },
    { 0xc26b223a, SUC_lexicon_tags_37 },
    { 0x8dcb223a, SUC_lexicon_tags_38 },
    { 0x390b223b, SUC_lexicon_tags_7 },
    { 0x756b2239, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692b2243, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4b2247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8b224a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28b2252, SUC_lexicon_tags_7 },
    { 0xe0ab2253, SUC_lexicon_tags_116 },
    { 0x712b2254, SUC_lexicon_tags_28 },
    { 0x146b2254, SUC_lexicon_tags_17 },
    { 0x414b2256, SUC_lexicon_tags_12 },
    { 0x214b2253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecb225b, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xe36b225d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2beb225f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0b2262, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad4b2264, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512b2268, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x594b226a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd92b226c, SUC_lexicon_tags_25 },
    { 0xc80b226d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50b2270, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750b2274, SUC_lexicon_tags_1 },
    { 0x2ab2275, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ab227f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2cab2281, SUC_lexicon_tags_1 },
    { 0x616b2281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1eb2284, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eb228d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8eb228f, SUC_lexicon_tags_164 },
    { 0x556b2290, SUC_lexicon_tags_26 },
    { 0x28cb228f, SUC_lexicon_tags_39 },
    { 0x4d4b2292, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5cab2294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8eb2297, SUC_lexicon_tags_37 },
    { 0x43cb2297, SUC_lexicon_tags_37 },
    { 0x9f8b2299, SUC_lexicon_tags_16 },
    { 0xb4cb229a, SUC_lexicon_tags_45 },
    { 0xc94b229a, SUC_lexicon_tags_70 },
    { 0x2d8b229c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cb22a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9eb22a9, SUC_lexicon_tags_1 },
    { 0x66cb22aa, SUC_lexicon_tags_25 },
    { 0x89ab22aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x386b22ae, SUC_lexicon_tags_55 },
    { 0xe00b22af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x550b22b1, SUC_lexicon_tags_36 },
    { 0xb9eb22b1, SUC_lexicon_tags_7 },
    { 0x3b6b22b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cb22b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5ab22b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2b22bb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cb22be, SUC_lexicon_tags_107 },
    { 0x756b22bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8b22c4, SUC_lexicon_tags_29 },
    { 0x7a6b22c5, SUC_lexicon_tags_37 },
    { 0x6bcb22c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9aab22c8, SUC_lexicon_tags_30 },
    { 0x888b22c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00b22cd, SUC_lexicon_tags_29 },
    { 0x114b22ce, SUC_lexicon_tags_32 },
    { 0xa36b22ce, SUC_lexicon_tags_17 },
    { 0xecb22cf, SUC_lexicon_tags_1 },
    { 0x13cb22cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cb22d5, SUC_lexicon_tags_38 },
    { 0x270b22d6, SUC_lexicon_tags_17 },
    { 0x832b22d7, SUC_lexicon_tags_146 },
    { 0x690b22d7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeeb22dd, SUC_lexicon_tags_11 },
    { 0x108b22de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cb22e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda4b22e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ab22ea, SUC_lexicon_tags_15 },
    { 0x254b22ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94b22ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x448b22f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18b22f5, SUC_lexicon_tags_11 },
    { 0xc9cb22f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a0b22f8, SUC_lexicon_tags_15 },
    { 0xfab22f8, SUC_lexicon_tags_29 },
    { 0x6b8b22fa, SUC_lexicon_tags_66 },
    { 0x44eb22f9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd9eb22fd, SUC_lexicon_tags_37 },
    { 0xb80b22fe, SUC_lexicon_tags_38 },
    { 0x1f4b22ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552b2303, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x34ab2305, SUC_lexicon_tags_0 },
    { 0x3fcb2306, SUC_lexicon_tags_17 },
    { 0x90eb2307, SUC_lexicon_tags_37 },
    { 0x1fcb2307, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6b230a, SUC_lexicon_tags_15 },
    { 0xce0b230b, SUC_lexicon_tags_30 },
    { 0x620b230c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbecb230f, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21eb2316, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532b231c, SUC_lexicon_tags_37 },
    { 0xb66b231c, SUC_lexicon_tags_32 },
    { 0x8b8b231e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcc4b2320, SUC_lexicon_tags_27 },
    { 0x580b2321, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaecb2323, SUC_lexicon_tags_30 },
    { 0x582b2323, SUC_lexicon_tags_12 },
    { 0xb5ab2325, SUC_lexicon_tags_1559 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4b2329, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacb232e, SUC_lexicon_tags_37 },
    { 0x65cb232f, SUC_lexicon_tags_45 },
    { 0xcb0b2330, SUC_lexicon_tags_201 },
    { 0xc90b2331, SUC_lexicon_tags_30 },
    { 0x5b6b2331, SUC_lexicon_tags_27 },
    { 0xf56b2333, SUC_lexicon_tags_17 },
    { 0xa1eb2333, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x528b2336, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42b233a, SUC_lexicon_tags_37 },
    { 0x48cb233a, SUC_lexicon_tags_17 },
    { 0x9aeb233b, SUC_lexicon_tags_26 },
    { 0x286b233d, SUC_lexicon_tags_37 },
    { 0x786b233e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf56b2340, SUC_lexicon_tags_0 },
    { 0xf82b2340, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2feb2344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6b2348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb6b234a, SUC_lexicon_tags_84 },
    { 0x718b234b, SUC_lexicon_tags_79 },
    { 0xd66b234b, SUC_lexicon_tags_3 },
    { 0x168b234d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x784b234f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ab2352, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab2355, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30b2358, SUC_lexicon_tags_37 },
    { 0xb16b2359, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3f2b235b, SUC_lexicon_tags_30 },
    { 0xe9ab235c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2eb2360, SUC_lexicon_tags_29 },
    { 0x544b2361, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0b2364, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cab2370, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300b2375, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22eb2379, SUC_lexicon_tags_27 },
    { 0x27cb2379, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cb237e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e8b2380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eb2386, SUC_lexicon_tags_277 },
    { 0x2feb2387, SUC_lexicon_tags_11 },
    { 0xda8b2388, SUC_lexicon_tags_23 },
    { 0xb94b2388, SUC_lexicon_tags_31 },
    { 0x7d6b2388, SUC_lexicon_tags_17 },
    { 0x934b2389, SUC_lexicon_tags_87 },
    { 0x454b238b, SUC_lexicon_tags_21 },
    { 0xeb0b238d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x952b2390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ab2393, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94eb239d, SUC_lexicon_tags_14 },
    { 0x888b239e, SUC_lexicon_tags_59 },
    { 0x3f2b239f, SUC_lexicon_tags_1 },
    { 0x722b23a0, SUC_lexicon_tags_47 },
    { 0xaa2b23a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x152b23a3, SUC_lexicon_tags_37 },
    { 0x462b23a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4b23a8, SUC_lexicon_tags_47 },
    { 0x86eb23a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400b23ad, SUC_lexicon_tags_37 },
    { 0x4a0b23ae, SUC_lexicon_tags_65 },
    { 0xa20b23af, SUC_lexicon_tags_21 },
    { 0xbd8b23ae, SUC_lexicon_tags_40 },
    { 0xeb0b23ad, SUC_lexicon_tags_44 },
    { 0xf2cb23b2, SUC_lexicon_tags_2 },
    { 0x53cb23b2, SUC_lexicon_tags_25 },
    { 0x242b23b4, SUC_lexicon_tags_37 },
    { 0xfeeb23b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ab23ba, SUC_lexicon_tags_65 },
    { 0xc8b23bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x546b23bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x710b23bf, SUC_lexicon_tags_6 },
    { 0x1c8b23bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x126b23c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96b23c7, SUC_lexicon_tags_13 },
    { 0x9b8b23c8, SUC_lexicon_tags_17 },
    { 0xb4cb23c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6b23cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330b23d2, SUC_lexicon_tags_37 },
    { 0x27cb23d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6b23d6, SUC_lexicon_tags_1 },
    { 0xb22b23d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfeab23d9, SUC_lexicon_tags_31 },
    { 0x2beb23da, SUC_lexicon_tags_7 },
    { 0x2a6b23da, SUC_lexicon_tags_53 },
    { 0x26ab23dc, SUC_lexicon_tags_12 },
    { 0xc6eb23dd, SUC_lexicon_tags_12 },
    { 0xef6b23db, SUC_lexicon_tags_84 },
    { 0x590b23df, SUC_lexicon_tags_11 },
    { 0xec4b23df, SUC_lexicon_tags_13 },
    { 0x39ab23e0, SUC_lexicon_tags_37 },
    { 0x850b23e2, SUC_lexicon_tags_27 },
    { 0xa74b23e3, SUC_lexicon_tags_33 },
    { 0xd82b23e0, SUC_lexicon_tags_45 },
    { 0x2cab23e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6b23ee, SUC_lexicon_tags_25 },
    { 0x43eb23ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0b23f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb86b23f4, SUC_lexicon_tags_0 },
    { 0xb8cb23f5, SUC_lexicon_tags_31 },
    { 0x380b23f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4b2400, SUC_lexicon_tags_17 },
    { 0xbeab2401, SUC_lexicon_tags_17 },
    { 0xb94b2400, SUC_lexicon_tags_65 },
    { 0x206b2402, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa2eb2405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974b240b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d4b240d, SUC_lexicon_tags_49 },
    { 0xc34b240d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8b2412, SUC_lexicon_tags_15 },
    { 0xbc0b2413, SUC_lexicon_tags_55 },
    { 0xd0cb2414, SUC_lexicon_tags_11 },
    { 0xbf0b2414, SUC_lexicon_tags_138 },
    { 0xa7cb2415, SUC_lexicon_tags_14 },
    { 0xc1ab2414, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6b241f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0b2423, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab2427, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672b242e, SUC_lexicon_tags_12 },
    { 0xd0cb242f, SUC_lexicon_tags_61 },
    { 0x83eb242e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x258b2433, SUC_lexicon_tags_37 },
    { 0x1f2b2433, SUC_lexicon_tags_10 },
    { 0x65cb2434, SUC_lexicon_tags_25 },
    { 0x55cb2436, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x18cb2439, SUC_lexicon_tags_37 },
    { 0xd3eb243a, SUC_lexicon_tags_10 },
    { 0x2a0b2439, SUC_lexicon_tags_1 },
    { 0xb2ab2439, SUC_lexicon_tags_32 },
    { 0x708b243d, SUC_lexicon_tags_7 },
    { 0xbfeb243e, SUC_lexicon_tags_37 },
    { 0xbb8b243f, SUC_lexicon_tags_69 },
    { 0x99eb2440, SUC_lexicon_tags_11 },
    { 0xe0ab2440, SUC_lexicon_tags_11 },
    { 0x1ccb243a, SUC_lexicon_tags_25 },
    { 0x398b2443, SUC_lexicon_tags_25 },
    { 0xb7eb2443, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x384b2447, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x382b2449, SUC_lexicon_tags_6 },
    { 0xa58b2449, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cb2452, SUC_lexicon_tags_25 },
    { 0x24cb2452, SUC_lexicon_tags_64 },
    { 0xe82b2454, SUC_lexicon_tags_22 },
    { 0x692b2454, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8b2458, SUC_lexicon_tags_83 },
    { 0x258b2459, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab8b245b, SUC_lexicon_tags_16 },
    { 0x312b245b, SUC_lexicon_tags_25 },
    { 0xc84b245d, SUC_lexicon_tags_26 },
    { 0x56ab245e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164b2466, SUC_lexicon_tags_65 },
    { 0xb74b2467, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68b246d, SUC_lexicon_tags_7 },
    { 0x19ab246e, SUC_lexicon_tags_12 },
    { 0xa1cb246f, SUC_lexicon_tags_25 },
    { 0x4c2b2470, SUC_lexicon_tags_160 },
    { 0x42b246e, SUC_lexicon_tags_30 },
    { 0xf20b2472, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3acb2474, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x580b2477, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccab247c, SUC_lexicon_tags_11 },
    { 0x146b247d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x538b247f, SUC_lexicon_tags_47 },
    { 0x5b0b2480, SUC_lexicon_tags_46 },
    { 0x3b6b2480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdceb2488, SUC_lexicon_tags_66 },
    { 0x966b2489, SUC_lexicon_tags_7 },
    { 0xfe6b2489, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44eb248c, SUC_lexicon_tags_15 },
    { 0x664b248d, SUC_lexicon_tags_65 },
    { 0x8b4b248d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc54b2490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652b2494, SUC_lexicon_tags_16 },
    { 0xe1cb2494, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948b249a, SUC_lexicon_tags_1 },
    { 0xd6cb249b, SUC_lexicon_tags_14 },
    { 0x940b249a, SUC_lexicon_tags_7 },
    { 0x480b249a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4ab249f, SUC_lexicon_tags_2 },
    { 0x49eb24a0, SUC_lexicon_tags_44 },
    { 0x568b24a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x31cb24a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842b24a9, SUC_lexicon_tags_49 },
    { 0xeb6b24aa, SUC_lexicon_tags_45 },
    { 0xf38b24ab, SUC_lexicon_tags_11 },
    { 0x674b24ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36b24b0, SUC_lexicon_tags_45 },
    { 0xe10b24b0, SUC_lexicon_tags_38 },
    { 0xefeb24b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x880b24b4, SUC_lexicon_tags_17 },
    { 0x5a6b24b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30b24b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eb24bd, SUC_lexicon_tags_9 },
    { 0x790b24be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76b24c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eb24c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedab24c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76b24cb, SUC_lexicon_tags_49 },
    { 0x620b24cc, SUC_lexicon_tags_2 },
    { 0xc42b24cd, SUC_lexicon_tags_32 },
    { 0x442b24cd, SUC_lexicon_tags_12 },
    { 0x2deb24cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbcb24d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754b24db, SUC_lexicon_tags_15 },
    { 0x362b24dc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2b24df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77cb24e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe2ab24e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecb24ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x538b24ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968b24f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ab24f4, SUC_lexicon_tags_2 },
    { 0xf6b24f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4e0b24f7, SUC_lexicon_tags_1 },
    { 0x328b24f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8b24fb, SUC_lexicon_tags_19 },
    { 0xa98b24fc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00b2500, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678b2505, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56b2509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60b250c, SUC_lexicon_tags_37 },
    { 0x8c0b250d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332b2515, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x604b2518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x978b251a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4beb251d, SUC_lexicon_tags_6 },
    { 0x4e0b251d, SUC_lexicon_tags_45 },
    { 0x78eb251f, SUC_lexicon_tags_43 },
    { 0x64b251e, SUC_lexicon_tags_25 },
    { 0xd6eb2521, SUC_lexicon_tags_48 },
    { 0x6ecb2522, SUC_lexicon_tags_6 },
    { 0xd26b2523, SUC_lexicon_tags_21 },
    { 0xd7ab2524, SUC_lexicon_tags_37 },
    { 0x56ab2525, SUC_lexicon_tags_82 },
    { 0x45cb2524, SUC_lexicon_tags_38 },
    { 0xfe6b2527, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65eb2535, SUC_lexicon_tags_11 },
    { 0xf5cb2536, SUC_lexicon_tags_17 },
    { 0x752b2537, SUC_lexicon_tags_30 },
    { 0x95eb2535, SUC_lexicon_tags_38 },
    { 0xcfab2535, SUC_lexicon_tags_17 },
    { 0xfd4b253a, SUC_lexicon_tags_14 },
    { 0x848b253b, SUC_lexicon_tags_1 },
    { 0x2f8b253c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9e8b253e, SUC_lexicon_tags_27 },
    { 0xa76b253e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8b2542, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6b2548, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x482b254b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44ab254d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xf2ab254f, SUC_lexicon_tags_30 },
    { 0xd2ab2550, SUC_lexicon_tags_14 },
    { 0x296b2551, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x950b2553, SUC_lexicon_tags_7 },
    { 0x754b2554, SUC_lexicon_tags_23 },
    { 0x4e0b2555, SUC_lexicon_tags_47 },
    { 0xc5eb2556, SUC_lexicon_tags_12 },
    { 0x2a4b2557, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16b255b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ab255e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8b2563, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838b2567, SUC_lexicon_tags_0 },
    { 0x288b2568, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91cb256b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf42b256d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x49ab256f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x260b2571, SUC_lexicon_tags_12 },
    { 0x3a8b2572, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1e4b2574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70b257b, SUC_lexicon_tags_30 },
    { 0x496b257c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8b2581, SUC_lexicon_tags_14 },
    { 0x9bab2582, SUC_lexicon_tags_9 },
    { 0x5c8b2582, SUC_lexicon_tags_33 },
    { 0xa68b2583, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8beb258e, SUC_lexicon_tags_14 },
    { 0x54cb258e, SUC_lexicon_tags_11 },
    { 0xba0b258e, SUC_lexicon_tags_27 },
    { 0x8ccb2591, SUC_lexicon_tags_38 },
    { 0xc1cb2591, SUC_lexicon_tags_49 },
    { 0x840b2593, SUC_lexicon_tags_23 },
    { 0x2d8b2590, SUC_lexicon_tags_49 },
    { 0xde8b2595, SUC_lexicon_tags_19 },
    { 0x2aeb2594, SUC_lexicon_tags_38 },
    { 0x6cab258e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cb259b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ab259f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8b25a2, SUC_lexicon_tags_17 },
    { 0xdcb25a3, SUC_lexicon_tags_21 },
    { 0x784b25a3, SUC_lexicon_tags_17 },
    { 0x830b25a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34b25a8, SUC_lexicon_tags_49 },
    { 0xfdab25a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcb25ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eab25b9, SUC_lexicon_tags_23 },
    { 0x43eb25b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab25bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcb25c0, SUC_lexicon_tags_0 },
    { 0x46ab25c0, SUC_lexicon_tags_37 },
    { 0x104b25c2, SUC_lexicon_tags_37 },
    { 0xb10b25c3, SUC_lexicon_tags_33 },
    { 0x1f0b25c0, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42b25c7, SUC_lexicon_tags_8 },
    { 0xdc2b25c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41cb25cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4b25d4, SUC_lexicon_tags_0 },
    { 0x318b25d5, SUC_lexicon_tags_25 },
    { 0xae6b25d4, SUC_lexicon_tags_11 },
    { 0x880b25d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32b25dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeb25df, SUC_lexicon_tags_49 },
    { 0xe30b25df, SUC_lexicon_tags_11 },
    { 0x85cb25e1, SUC_lexicon_tags_47 },
    { 0x3d2b25e2, SUC_lexicon_tags_26 },
    { 0x28cb25e3, SUC_lexicon_tags_33 },
    { 0x11eb25e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4b25ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766b25f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8b25f5, SUC_lexicon_tags_56 },
    { 0xaecb25f6, SUC_lexicon_tags_2 },
    { 0x58b25f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00b25fa, SUC_lexicon_tags_37 },
    { 0x834b25fb, SUC_lexicon_tags_15 },
    { 0xbceb25fb, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x66cb25ff, SUC_lexicon_tags_28 },
    { 0xbd6b2600, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8b2605, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6b260e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb96b2610, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4eb2615, SUC_lexicon_tags_11 },
    { 0xe52b2615, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54b2619, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xf3eb261b, SUC_lexicon_tags_19 },
    { 0xab8b261c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166b2620, SUC_lexicon_tags_66 },
    { 0x6c0b2621, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x132b2624, SUC_lexicon_tags_26 },
    { 0x8fcb2624, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xec0b2627, SUC_lexicon_tags_11 },
    { 0xc42b2628, SUC_lexicon_tags_17 },
    { 0xaa0b2628, SUC_lexicon_tags_25 },
    { 0x5aeb2628, SUC_lexicon_tags_25 },
    { 0xaa6b2628, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x682b262d, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8b2633, SUC_lexicon_tags_0 },
    { 0xb8ab2633, SUC_lexicon_tags_14 },
    { 0xf8b2635, SUC_lexicon_tags_46 },
    { 0xfdab2636, SUC_lexicon_tags_1 },
    { 0x26cb2637, SUC_lexicon_tags_23 },
    { 0xcf0b2638, SUC_lexicon_tags_0 },
    { 0x65ab2639, SUC_lexicon_tags_60 },
    { 0xd3eb2639, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x796b263d, SUC_lexicon_tags_10 },
    { 0x682b263d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72b2645, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x392b2647, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7c0b2649, SUC_lexicon_tags_17 },
    { 0x4aeb264a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40b2654, SUC_lexicon_tags_91 },
    { 0xedab2654, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842b265c, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0, NULL },
    { 0x18eb265f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6b2663, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1f6b2665, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0b266b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x410b266e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x13eb2671, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xea4b2673, SUC_lexicon_tags_61 },
    { 0xd7eb2673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270b267b, SUC_lexicon_tags_11 },
    { 0xbcb267c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa98b267e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb2682, SUC_lexicon_tags_14 },
    { 0x4bcb2683, SUC_lexicon_tags_53 },
    { 0x308b2684, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x160b2686, SUC_lexicon_tags_12 },
    { 0x488b2687, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27cb2689, SUC_lexicon_tags_37 },
    { 0xb72b268a, SUC_lexicon_tags_25 },
    { 0x93cb268b, SUC_lexicon_tags_7 },
    { 0x46eb2689, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926b2690, SUC_lexicon_tags_1 },
    { 0x1e8b2691, SUC_lexicon_tags_37 },
    { 0xf0b2692, SUC_lexicon_tags_37 },
    { 0xe6eb2693, SUC_lexicon_tags_7 },
    { 0xe58b2693, SUC_lexicon_tags_47 },
    { 0x9bab2695, SUC_lexicon_tags_30 },
    { 0xd20b2691, SUC_lexicon_tags_1 },
    { 0x6b2695, SUC_lexicon_tags_77 },
    { 0x5a2b2697, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68b269c, SUC_lexicon_tags_60 },
    { 0x438b269d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ab26a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b26a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cab26a9, SUC_lexicon_tags_17 },
    { 0xae8b26aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x980b26ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6bcb26ae, SUC_lexicon_tags_12 },
    { 0x2b8b26af, SUC_lexicon_tags_44 },
    { 0x7f6b26b0, SUC_lexicon_tags_31 },
    { 0xc12b26af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2b26b4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cb26b9, SUC_lexicon_tags_25 },
    { 0x9ecb26ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b26c2, SUC_lexicon_tags_37 },
    { 0xd8cb26c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a4b26c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x438b26c7, SUC_lexicon_tags_6 },
    { 0x9a8b26c8, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412b26cc, SUC_lexicon_tags_37 },
    { 0xc36b26cd, SUC_lexicon_tags_87 },
    { 0xe9cb26ce, SUC_lexicon_tags_17 },
    { 0x6a2b26cd, SUC_lexicon_tags_54 },
    { 0x6b26cc, SUC_lexicon_tags_12 },
    { 0xc00b26d1, SUC_lexicon_tags_46 },
    { 0x10ab26d1, SUC_lexicon_tags_37 },
    { 0x47eb26d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64eb26d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86b26dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250b26e4, SUC_lexicon_tags_37 },
    { 0xb10b26e4, SUC_lexicon_tags_11 },
    { 0xff0b26e6, SUC_lexicon_tags_15 },
    { 0x558b26e6, SUC_lexicon_tags_31 },
    { 0x3c8b26e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x850b26ea, SUC_lexicon_tags_17 },
    { 0x2eeb26eb, SUC_lexicon_tags_1 },
    { 0xd6b26ec, SUC_lexicon_tags_27 },
    { 0x15ab26ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4b26f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0b26f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ab26f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cb26fc, SUC_lexicon_tags_17 },
    { 0xacb26fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf8b26ff, SUC_lexicon_tags_11 },
    { 0x7acb26ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x416b2703, SUC_lexicon_tags_7 },
    { 0x85ab2704, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2b2706, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x646b2708, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x504b270a, SUC_lexicon_tags_37 },
    { 0xb70b270b, SUC_lexicon_tags_37 },
    { 0x7feb270b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ab2710, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4b2717, SUC_lexicon_tags_32 },
    { 0xe0eb2718, SUC_lexicon_tags_1 },
    { 0x9c0b2719, SUC_lexicon_tags_60 },
    { 0xef0b2719, SUC_lexicon_tags_37 },
    { 0x24b271a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x652b271e, SUC_lexicon_tags_17 },
    { 0x5d8b271f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e6b2721, SUC_lexicon_tags_14 },
    { 0x3d2b2722, SUC_lexicon_tags_37 },
    { 0xca0b2723, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6b2726, SUC_lexicon_tags_17 },
    { 0xdb8b2727, SUC_lexicon_tags_16 },
    { 0xc20b2728, SUC_lexicon_tags_31 },
    { 0x3f8b2729, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64eb272b, SUC_lexicon_tags_23 },
    { 0xd04b272c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbfab272e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x364b2730, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0b2737, SUC_lexicon_tags_11 },
    { 0x80eb2738, SUC_lexicon_tags_12 },
    { 0xd06b2739, SUC_lexicon_tags_33 },
    { 0xe88b2739, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374b2740, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab274a, SUC_lexicon_tags_12 },
    { 0x682b274a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512b2750, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x770b2752, SUC_lexicon_tags_2 },
    { 0x6b6b2753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46b2757, SUC_lexicon_tags_11 },
    { 0x556b2758, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x58eb275b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fab2761, SUC_lexicon_tags_38 },
    { 0xd22b2761, SUC_lexicon_tags_27 },
    { 0x186b2762, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288b2768, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8b276d, SUC_lexicon_tags_17 },
    { 0x414b276e, SUC_lexicon_tags_2 },
    { 0xdbeb276e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab2778, SUC_lexicon_tags_17 },
    { 0x198b2779, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6b2780, SUC_lexicon_tags_63 },
    { 0xdd2b2780, SUC_lexicon_tags_11 },
    { 0x20b2780, SUC_lexicon_tags_12 },
    { 0xd58b2783, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ceb2785, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654b2789, SUC_lexicon_tags_27 },
    { 0x816b278a, SUC_lexicon_tags_39 },
    { 0x6ab278b, SUC_lexicon_tags_37 },
    { 0x70b278b, SUC_lexicon_tags_14 },
    { 0x44cb278d, SUC_lexicon_tags_17 },
    { 0x816b278d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x958b2791, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2bab2793, SUC_lexicon_tags_17 },
    { 0x5a4b2794, SUC_lexicon_tags_8 },
    { 0xc38b2793, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54b2799, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd8ab279b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x322b279d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ab27a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x690b27a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab2b27a4, SUC_lexicon_tags_104 },
    { 0x444b27a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x154b27a7, SUC_lexicon_tags_6 },
    { 0x9e6b27a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe64b27aa, SUC_lexicon_tags_38 },
    { 0x93cb27aa, SUC_lexicon_tags_11 },
    { 0x42eb27ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51cb27ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb27b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6b27b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xebeb27b9, SUC_lexicon_tags_17 },
    { 0x9eb27ba, SUC_lexicon_tags_17 },
    { 0xee8b27ba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684b27c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea4b27c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eb27c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb27ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ab27cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccb27d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6b27d7, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x748b27da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x94cb27dd, SUC_lexicon_tags_37 },
    { 0xf12b27dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cb27e2, SUC_lexicon_tags_17 },
    { 0x768b27e3, SUC_lexicon_tags_123 },
    { 0xde2b27e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4b27ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe86b27ec, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2b27f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9eb27f3, SUC_lexicon_tags_11 },
    { 0x1d8b27f3, SUC_lexicon_tags_6 },
    { 0x962b27f5, SUC_lexicon_tags_60 },
    { 0xc5cb27f6, SUC_lexicon_tags_17 },
    { 0xa2cb27f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534b27fe, SUC_lexicon_tags_43 },
    { 0xbb2b27ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0b2802, SUC_lexicon_tags_26 },
    { 0xe00b2803, SUC_lexicon_tags_12 },
    { 0x2cb2804, SUC_lexicon_tags_26 },
    { 0xfb4b2804, SUC_lexicon_tags_8 },
    { 0xa4ab2806, SUC_lexicon_tags_60 },
    { 0x210b2807, SUC_lexicon_tags_11 },
    { 0xcfeb2808, SUC_lexicon_tags_63 },
    { 0x3e8b2809, SUC_lexicon_tags_27 },
    { 0xbe2b2809, SUC_lexicon_tags_19 },
    { 0xe34b280b, SUC_lexicon_tags_12 },
    { 0x724b2809, SUC_lexicon_tags_10 },
    { 0xe48b280d, SUC_lexicon_tags_27 },
    { 0x68cb280d, SUC_lexicon_tags_28 },
    { 0x104b2803, SUC_lexicon_tags_1 },
    { 0xedeb2808, SUC_lexicon_tags_84 },
    { 0x130b2811, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140b281a, SUC_lexicon_tags_37 },
    { 0x388b281b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc48b281d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeb2820, SUC_lexicon_tags_25 },
    { 0xf62b2821, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x876b2823, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75eb282d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcfeb282f, SUC_lexicon_tags_25 },
    { 0x800b282f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ab2833, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa9cb2835, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7deb283a, SUC_lexicon_tags_6 },
    { 0x6eab283b, SUC_lexicon_tags_5 },
    { 0x3b2b283c, SUC_lexicon_tags_64 },
    { 0x694b283b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732b2841, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb0b2843, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82b2848, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x212b284b, SUC_lexicon_tags_56 },
    { 0x7f6b284c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb58b284e, SUC_lexicon_tags_7 },
    { 0xb9eb284f, SUC_lexicon_tags_46 },
    { 0xb0eb2850, SUC_lexicon_tags_15 },
    { 0xbeeb2851, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580b2857, SUC_lexicon_tags_1 },
    { 0x88b2858, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x248b285b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeb285e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81eb2861, SUC_lexicon_tags_37 },
    { 0x576b2862, SUC_lexicon_tags_7 },
    { 0x66b2863, SUC_lexicon_tags_108 },
    { 0x7ceb2864, SUC_lexicon_tags_19 },
    { 0xdc0b2865, SUC_lexicon_tags_64 },
    { 0x6eab2866, SUC_lexicon_tags_27 },
    { 0x6f8b2867, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4b286a, SUC_lexicon_tags_68 },
    { 0x8ccb286b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8b2871, SUC_lexicon_tags_1 },
    { 0x364b2872, SUC_lexicon_tags_27 },
    { 0x946b2872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e8b2875, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xec4b2877, SUC_lexicon_tags_25 },
    { 0x28b2878, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xabcb287a, SUC_lexicon_tags_336 },
    { 0x53cb287b, SUC_lexicon_tags_14 },
    { 0x82ab287c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa22b287e, SUC_lexicon_tags_2 },
    { 0x4ecb287f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cb2882, SUC_lexicon_tags_23 },
    { 0xf8eb2883, SUC_lexicon_tags_14 },
    { 0xb56b2884, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x114b2886, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dab288b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x59ab288d, SUC_lexicon_tags_27 },
    { 0x56eb288e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160b2892, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ab2895, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedcb2897, SUC_lexicon_tags_65 },
    { 0xe00b2898, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2b289f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64b28a3, SUC_lexicon_tags_11 },
    { 0x202b28a4, SUC_lexicon_tags_33 },
    { 0x4bab28a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cb28a8, SUC_lexicon_tags_70 },
    { 0x340b28a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90b28b0, SUC_lexicon_tags_19 },
    { 0xd8b28b0, SUC_lexicon_tags_14 },
    { 0xf84b28b2, SUC_lexicon_tags_43 },
    { 0x1eab28b2, SUC_lexicon_tags_51 },
    { 0xed4b28b2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2b28b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48b28bb, SUC_lexicon_tags_49 },
    { 0x330b28bb, SUC_lexicon_tags_29 },
    { 0xb4eb28bd, SUC_lexicon_tags_43 },
    { 0x86b28bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d0b28c0, SUC_lexicon_tags_15 },
    { 0xda2b28c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cb28ca, SUC_lexicon_tags_49 },
    { 0xfb4b28cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cb28cf, SUC_lexicon_tags_37 },
    { 0x1b8b28d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52b28d6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ab28d9, SUC_lexicon_tags_12 },
    { 0xb1eb28d9, SUC_lexicon_tags_11 },
    { 0xbe0b28db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9eb28dd, SUC_lexicon_tags_60 },
    { 0x93eb28de, SUC_lexicon_tags_35 },
    { 0x526b28df, SUC_lexicon_tags_47 },
    { 0x56eb28e0, SUC_lexicon_tags_2 },
    { 0x522b28e1, SUC_lexicon_tags_45 },
    { 0xa5eb28e1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xac6b28e4, SUC_lexicon_tags_62 },
    { 0x376b28e5, SUC_lexicon_tags_7 },
    { 0xfbcb28e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0b28e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162b28ee, SUC_lexicon_tags_60 },
    { 0x264b28ef, SUC_lexicon_tags_12 },
    { 0xbfab28f0, SUC_lexicon_tags_45 },
    { 0x242b28f1, SUC_lexicon_tags_50 },
    { 0x3ab28f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x522b28f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08b28f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6b28fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76cb28fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360b2902, SUC_lexicon_tags_37 },
    { 0xa6ab2902, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x696b2905, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe0eb2907, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b290a, SUC_lexicon_tags_1 },
    { 0xa96b290b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45eb290e, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26b2911, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbeb2918, SUC_lexicon_tags_82 },
    { 0x200b2919, SUC_lexicon_tags_14 },
    { 0xca0b2918, SUC_lexicon_tags_1 },
    { 0x2f8b2919, SUC_lexicon_tags_25 },
    { 0x954b2919, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e2b291e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x77eb2921, SUC_lexicon_tags_15 },
    { 0x750b2922, SUC_lexicon_tags_11 },
    { 0xafab2922, SUC_lexicon_tags_55 },
    { 0xd7ab2921, SUC_lexicon_tags_60 },
    { 0x9dab2925, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x930b2927, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffab292c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ab2930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8b2933, SUC_lexicon_tags_47 },
    { 0x76ab2933, SUC_lexicon_tags_47 },
    { 0x658b2935, SUC_lexicon_tags_37 },
    { 0xacab2936, SUC_lexicon_tags_2 },
    { 0x6a0b2937, SUC_lexicon_tags_33 },
    { 0xf62b2937, SUC_lexicon_tags_17 },
    { 0x3e6b2937, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0b293c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8b2940, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7dcb2942, SUC_lexicon_tags_49 },
    { 0xfd2b2943, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4beb2946, SUC_lexicon_tags_6 },
    { 0x352b2947, SUC_lexicon_tags_87 },
    { 0x80eb2948, SUC_lexicon_tags_71 },
    { 0xe04b2949, SUC_lexicon_tags_12 },
    { 0x858b294a, SUC_lexicon_tags_26 },
    { 0x424b294a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2b2950, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cab2954, SUC_lexicon_tags_43 },
    { 0xeb0b2955, SUC_lexicon_tags_26 },
    { 0xcaeb2956, SUC_lexicon_tags_15 },
    { 0xbf8b2956, SUC_lexicon_tags_27 },
    { 0xf5ab2958, SUC_lexicon_tags_11 },
    { 0x4e0b2959, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab295d, SUC_lexicon_tags_7 },
    { 0x512b295d, SUC_lexicon_tags_43 },
    { 0x6eeb295d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x110b2962, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b2964, SUC_lexicon_tags_7 },
    { 0x31cb2965, SUC_lexicon_tags_23 },
    { 0xee0b2964, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20b296b, SUC_lexicon_tags_67 },
    { 0x35ab296b, SUC_lexicon_tags_23 },
    { 0x7b6b296c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56b2970, SUC_lexicon_tags_7 },
    { 0x492b2970, SUC_lexicon_tags_15 },
    { 0x6ab2972, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8b2976, SUC_lexicon_tags_31 },
    { 0x8aab2977, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcb297f, SUC_lexicon_tags_17 },
    { 0x508b2980, SUC_lexicon_tags_15 },
    { 0x9d2b2981, SUC_lexicon_tags_17 },
    { 0x396b2982, SUC_lexicon_tags_2 },
    { 0x2d0b2983, SUC_lexicon_tags_11 },
    { 0xf30b2982, SUC_lexicon_tags_25 },
    { 0xe76b2981, SUC_lexicon_tags_37 },
    { 0x2e2b2986, SUC_lexicon_tags_1 },
    { 0x9cab2987, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396b2991, SUC_lexicon_tags_107 },
    { 0x634b2992, SUC_lexicon_tags_70 },
    { 0x5a0b2993, SUC_lexicon_tags_47 },
    { 0xfb6b2993, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x5d4b2996, SUC_lexicon_tags_7 },
    { 0x368b2997, SUC_lexicon_tags_7 },
    { 0xc4eb2998, SUC_lexicon_tags_56 },
    { 0xf4eb2998, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408b299e, SUC_lexicon_tags_1 },
    { 0x73cb299f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc10b29a1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b29a6, SUC_lexicon_tags_2 },
    { 0xed6b29a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x856b29a9, SUC_lexicon_tags_77 },
    { 0x262b29a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x436b29ac, SUC_lexicon_tags_37 },
    { 0xae0b29ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf40b29af, SUC_lexicon_tags_45 },
    { 0x54b29af, SUC_lexicon_tags_43 },
    { 0x5ceb29b1, SUC_lexicon_tags_23 },
    { 0x40eb29b2, SUC_lexicon_tags_17 },
    { 0x538b29b0, SUC_lexicon_tags_25 },
    { 0x8f2b29b4, SUC_lexicon_tags_26 },
    { 0x52cb29b0, SUC_lexicon_tags_414 },
    { 0xc3eb29b6, SUC_lexicon_tags_17 },
    { 0x62cb29b5, SUC_lexicon_tags_1 },
    { 0xd46b29b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4eb29c2, SUC_lexicon_tags_2 },
    { 0xa6ab29c3, SUC_lexicon_tags_63 },
    { 0xe44b29c4, SUC_lexicon_tags_1 },
    { 0xa94b29c3, SUC_lexicon_tags_16 },
    { 0xcf4b29c2, SUC_lexicon_tags_11 },
    { 0x1b0b29c7, SUC_lexicon_tags_49 },
    { 0xcc4b29c8, SUC_lexicon_tags_17 },
    { 0x920b29c9, SUC_lexicon_tags_15 },
    { 0x8b6b29ca, SUC_lexicon_tags_2 },
    { 0xe34b29c9, SUC_lexicon_tags_55 },
    { 0xafab29c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b29d0, SUC_lexicon_tags_29 },
    { 0x3a0b29d0, SUC_lexicon_tags_10 },
    { 0xfd8b29d2, SUC_lexicon_tags_63 },
    { 0x81ab29d3, SUC_lexicon_tags_11 },
    { 0x63eb29d3, SUC_lexicon_tags_87 },
    { 0x810b29d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf66b29d7, SUC_lexicon_tags_37 },
    { 0xdd8b29d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696b29dc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x766b29de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62b29ea, SUC_lexicon_tags_17 },
    { 0x76eb29eb, SUC_lexicon_tags_6 },
    { 0x4aeb29ec, SUC_lexicon_tags_47 },
    { 0xbacb29ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b29f0, SUC_lexicon_tags_1 },
    { 0xe2b29f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2b29f5, SUC_lexicon_tags_7 },
    { 0x6e2b29f6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16b29fa, SUC_lexicon_tags_4 },
    { 0x344b29fb, SUC_lexicon_tags_25 },
    { 0xee0b29fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7ab29fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x892b2a00, SUC_lexicon_tags_27 },
    { 0xd84b2a01, SUC_lexicon_tags_1 },
    { 0xd4eb2a00, SUC_lexicon_tags_43 },
    { 0xe18b2a03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc16b2a05, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64b2a08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x804b2a0a, SUC_lexicon_tags_7 },
    { 0x382b2a0b, SUC_lexicon_tags_17 },
    { 0xba0b2a0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650b2a12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58eb2a14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2cb2a16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce6b2a18, SUC_lexicon_tags_19 },
    { 0x50b2a19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x674b2a1c, SUC_lexicon_tags_31 },
    { 0x3cab2a1d, SUC_lexicon_tags_65 },
    { 0x7b4b2a1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda0b2a20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c4b2a22, SUC_lexicon_tags_43 },
    { 0xe18b2a23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0b2a26, SUC_lexicon_tags_191 },
    { 0x806b2a26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88b2a29, SUC_lexicon_tags_45 },
    { 0x5fab2a2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12b2a2d, SUC_lexicon_tags_19 },
    { 0xe22b2a2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x262b2a31, SUC_lexicon_tags_43 },
    { 0x456b2a32, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbceb2a37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4b2a3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4b2a44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0b2a4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2b2a53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154b2a5a, SUC_lexicon_tags_37 },
    { 0xa2cb2a5a, SUC_lexicon_tags_55 },
    { 0xd7ab2a5c, SUC_lexicon_tags_23 },
    { 0x9cb2a5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa9cb2a5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x926b2a62, SUC_lexicon_tags_63 },
    { 0xb70b2a62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16b2a69, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cb2a6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d8b2a71, SUC_lexicon_tags_140 },
    { 0x766b2a71, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ab2a75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cb2a78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4b2a7b, SUC_lexicon_tags_39 },
    { 0x592b2a7b, SUC_lexicon_tags_1 },
    { 0xcc8b2a7c, SUC_lexicon_tags_14 },
    { 0x232b2a7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2b2a81, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x892b2a83, SUC_lexicon_tags_16 },
    { 0x2a6b2a84, SUC_lexicon_tags_8 },
    { 0xf90b2a85, SUC_lexicon_tags_2 },
    { 0x288b2a85, SUC_lexicon_tags_1 },
    { 0x958b2a87, SUC_lexicon_tags_25 },
    { 0x8a6b2a87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24b2a8c, SUC_lexicon_tags_26 },
    { 0x93cb2a8d, SUC_lexicon_tags_7 },
    { 0xb50b2a8e, SUC_lexicon_tags_69 },
    { 0x79cb2a8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77eb2a93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b2a99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb2a9c, SUC_lexicon_tags_7 },
    { 0x896b2a9d, SUC_lexicon_tags_63 },
    { 0x4d4b2a9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x206b2aa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436b2aa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b2aab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x508b2aae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x312b2ab0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x7fab2ab2, SUC_lexicon_tags_48 },
    { 0x846b2ab3, SUC_lexicon_tags_27 },
    { 0xbb6b2ab4, SUC_lexicon_tags_63 },
    { 0xb26b2ab4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x136b2ab8, SUC_lexicon_tags_11 },
    { 0x10cb2ab8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0ab2abb, SUC_lexicon_tags_12 },
    { 0xf5eb2abb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc4b2abe, SUC_lexicon_tags_45 },
    { 0x9fab2abe, SUC_lexicon_tags_28 },
    { 0xd8b2abe, SUC_lexicon_tags_1 },
    { 0xc08b2ac1, SUC_lexicon_tags_27 },
    { 0xe90b2ac2, SUC_lexicon_tags_7 },
    { 0xd0eb2ac2, SUC_lexicon_tags_0 },
    { 0xdd2b2ac4, SUC_lexicon_tags_27 },
    { 0x192b2ac5, SUC_lexicon_tags_1 },
    { 0x97eb2ac1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4e8b2ac8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6feb2aca, SUC_lexicon_tags_12 },
    { 0xe96b2acb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982b2ad5, SUC_lexicon_tags_25 },
    { 0x806b2ad6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x948b2ad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab2adc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3ab2ade, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe82b2ae0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12b2ae2, SUC_lexicon_tags_25 },
    { 0x1d2b2ae2, SUC_lexicon_tags_10 },
    { 0x568b2ae3, SUC_lexicon_tags_87 },
    { 0x79cb2ae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4beb2ae7, SUC_lexicon_tags_63 },
    { 0x1dab2ae8, SUC_lexicon_tags_47 },
    { 0x3c8b2ae9, SUC_lexicon_tags_15 },
    { 0x7c8b2aea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cb2aed, SUC_lexicon_tags_91 },
    { 0xc84b2aee, SUC_lexicon_tags_77 },
    { 0x748b2aed, SUC_lexicon_tags_7 },
    { 0x306b2aed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6b2af3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994b2af7, SUC_lexicon_tags_30 },
    { 0x1beb2af8, SUC_lexicon_tags_6 },
    { 0x962b2af9, SUC_lexicon_tags_7 },
    { 0xfc0b2af8, SUC_lexicon_tags_43 },
    { 0x4bab2afb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538b2aff, SUC_lexicon_tags_25 },
    { 0xdb6b2b00, SUC_lexicon_tags_15 },
    { 0x46ab2b00, SUC_lexicon_tags_53 },
    { 0xb72b2b01, SUC_lexicon_tags_25 },
    { 0xa82b2b03, SUC_lexicon_tags_2 },
    { 0x432b2b04, SUC_lexicon_tags_31 },
    { 0x8cb2b05, SUC_lexicon_tags_2 },
    { 0x3feb2b04, SUC_lexicon_tags_29 },
    { 0xdfeb2b06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2b09, SUC_lexicon_tags_6 },
    { 0x8b2b2b09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddab2b0e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x49ab2b10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccb2b14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb30b2b16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb2b1a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314b2b1e, SUC_lexicon_tags_7 },
    { 0x76b2b1f, SUC_lexicon_tags_37 },
    { 0xb28b2b1e, SUC_lexicon_tags_32 },
    { 0x9e4b2b21, SUC_lexicon_tags_37 },
    { 0x5c2b2b22, SUC_lexicon_tags_6 },
    { 0xc28b2b22, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb2b2b, SUC_lexicon_tags_8 },
    { 0x7eab2b2c, SUC_lexicon_tags_6 },
    { 0xbbcb2b2d, SUC_lexicon_tags_1 },
    { 0xa6ab2b2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00b2b31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704b2b37, SUC_lexicon_tags_269 },
    { 0x936b2b37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc52b2b3a, SUC_lexicon_tags_11 },
    { 0x20eb2b3b, SUC_lexicon_tags_35 },
    { 0xcccb2b3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa1cb2b3e, SUC_lexicon_tags_48 },
    { 0xc32b2b3f, SUC_lexicon_tags_47 },
    { 0x8cab2b40, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaeb2b45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eb2b48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54b2b52, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcb2b55, SUC_lexicon_tags_17 },
    { 0x716b2b56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56eb2b58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ceb2b5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf4b2b5c, SUC_lexicon_tags_4 },
    { 0x864b2b5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b2b60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0eb2b62, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696b2b68, SUC_lexicon_tags_1 },
    { 0x4d8b2b69, SUC_lexicon_tags_2 },
    { 0xec8b2b6a, SUC_lexicon_tags_41 },
    { 0xaeeb2b6b, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6b2b72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2b2b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2b7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa58b2b80, SUC_lexicon_tags_31 },
    { 0x178b2b81, SUC_lexicon_tags_12 },
    { 0x8d6b2b82, SUC_lexicon_tags_25 },
    { 0xc38b2b83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6b2b86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1dcb2b88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3feb2b91, SUC_lexicon_tags_27 },
    { 0xc48b2b92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcb2b9b, SUC_lexicon_tags_47 },
    { 0xb60b2b9c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcb2ba6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8b2baa, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x3e6b2bac, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840b2bb0, SUC_lexicon_tags_60 },
    { 0x8eb2bb1, SUC_lexicon_tags_56 },
    { 0x35eb2bb1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fab2bbe, SUC_lexicon_tags_1 },
    { 0x8b8b2bbf, SUC_lexicon_tags_37 },
    { 0x3deb2bbe, SUC_lexicon_tags_17 },
    { 0x4ecb2bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b2bc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x92eb2bc8, SUC_lexicon_tags_10 },
    { 0x76b2bc9, SUC_lexicon_tags_113 },
    { 0x3c0b2bca, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eeb2bd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88b2bd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4b2bdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324b2be0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x128b2be3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04b2be8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb46b2bea, SUC_lexicon_tags_2 },
    { 0x84b2beb, SUC_lexicon_tags_10 },
    { 0x266b2bec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x318b2bef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cb2bfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2b2c02, SUC_lexicon_tags_23 },
    { 0x87ab2c03, SUC_lexicon_tags_11 },
    { 0xe6b2c03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46b2c08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6b2c0c, SUC_lexicon_tags_37 },
    { 0x82ab2c0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f2b2c0f, SUC_lexicon_tags_18 },
    { 0x9b8b2c10, SUC_lexicon_tags_39 },
    { 0xa84b2c11, SUC_lexicon_tags_25 },
    { 0x73ab2c12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf38b2c14, SUC_lexicon_tags_12 },
    { 0xc82b2c14, SUC_lexicon_tags_63 },
    { 0xe52b2c14, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b2c19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x666b2c1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaab2c1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e4b2c20, SUC_lexicon_tags_0 },
    { 0x996b2c21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc6b2c23, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2b2c36, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ab2c3e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b2c4a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x76eb2c4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8b2c51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x59ab2c53, SUC_lexicon_tags_1 },
    { 0x934b2c54, SUC_lexicon_tags_15 },
    { 0xbf8b2c53, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2b2c59, SUC_lexicon_tags_17 },
    { 0xc6cb2c59, SUC_lexicon_tags_37 },
    { 0xd5ab2c5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88cb2c5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6b2c61, SUC_lexicon_tags_3 },
    { 0x944b2c61, SUC_lexicon_tags_38 },
    { 0x39eb2c63, SUC_lexicon_tags_29 },
    { 0xfab2c64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x87eb2c66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4eb2c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cb2c6f, SUC_lexicon_tags_7 },
    { 0x71ab2c6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b2c76, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756b2c7b, SUC_lexicon_tags_15 },
    { 0x948b2c7b, SUC_lexicon_tags_29 },
    { 0xca0b2c7d, SUC_lexicon_tags_46 },
    { 0x3bab2c7e, SUC_lexicon_tags_25 },
    { 0x5a8b2c7f, SUC_lexicon_tags_23 },
    { 0x42ab2c7e, SUC_lexicon_tags_15 },
    { 0xa64b2c80, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x52b2c83, SUC_lexicon_tags_60 },
    { 0xfceb2c83, SUC_lexicon_tags_35 },
    { 0xb0ab2c85, SUC_lexicon_tags_33 },
    { 0x682b2c83, SUC_lexicon_tags_25 },
    { 0x58eb2c87, SUC_lexicon_tags_43 },
    { 0xce8b2c84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x962b2c8a, SUC_lexicon_tags_28 },
    { 0x8f4b2c8a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766b2c92, SUC_lexicon_tags_21 },
    { 0xeb2c92, SUC_lexicon_tags_12 },
    { 0x9eab2c93, SUC_lexicon_tags_63 },
    { 0xac8b2c93, SUC_lexicon_tags_7 },
    { 0xf26b2c96, SUC_lexicon_tags_19 },
    { 0xec0b2c97, SUC_lexicon_tags_22 },
    { 0xa90b2c97, SUC_lexicon_tags_46 },
    { 0x690b2c98, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ab2c9d, SUC_lexicon_tags_12 },
    { 0x122b2c9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x742b2ca0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb2ca4, SUC_lexicon_tags_39 },
    { 0x564b2ca4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0b2ca8, SUC_lexicon_tags_25 },
    { 0x6b4b2ca9, SUC_lexicon_tags_298 },
    { 0x2b2b2caa, SUC_lexicon_tags_45 },
    { 0x600b2ca9, SUC_lexicon_tags_25 },
    { 0x69ab2caa, SUC_lexicon_tags_1 },
    { 0xce4b2cad, SUC_lexicon_tags_8 },
    { 0x146b2cae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa24b2cb0, SUC_lexicon_tags_55 },
    { 0x956b2cb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf92b2cb3, SUC_lexicon_tags_12 },
    { 0xd64b2cb4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x700b2cb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6e0b2cb9, SUC_lexicon_tags_23 },
    { 0xe10b2cba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cb2cc1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8b2cc4, SUC_lexicon_tags_44 },
    { 0xb9ab2cc4, SUC_lexicon_tags_42 },
    { 0x434b2cc6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524b2ccc, SUC_lexicon_tags_63 },
    { 0x5aab2ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cb2cd5, SUC_lexicon_tags_11 },
    { 0x96ab2cd5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x75ab2cd8, SUC_lexicon_tags_9 },
    { 0x1fcb2cd9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76b2cdc, SUC_lexicon_tags_11 },
    { 0x71cb2cdd, SUC_lexicon_tags_15 },
    { 0xf88b2cde, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x444b2ce0, SUC_lexicon_tags_47 },
    { 0xb46b2ce1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a8b2ce3, SUC_lexicon_tags_30 },
    { 0xfdab2ce4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5dab2ce6, SUC_lexicon_tags_17 },
    { 0x414b2ce7, SUC_lexicon_tags_27 },
    { 0x476b2ce8, SUC_lexicon_tags_1 },
    { 0xdfeb2ce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecb2cf2, SUC_lexicon_tags_37 },
    { 0xb44b2cf2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388b2cf7, SUC_lexicon_tags_25 },
    { 0x4f8b2cf8, SUC_lexicon_tags_14 },
    { 0x150b2cf9, SUC_lexicon_tags_7 },
    { 0xd8eb2cf9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x892b2cfc, SUC_lexicon_tags_2 },
    { 0x30cb2cfd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc56b2cff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b2d08, SUC_lexicon_tags_46 },
    { 0x9f2b2d09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2b2d0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74b2d11, SUC_lexicon_tags_21 },
    { 0x7fcb2d11, SUC_lexicon_tags_1 },
    { 0xb20b2d12, SUC_lexicon_tags_47 },
    { 0x110b2d14, SUC_lexicon_tags_32 },
    { 0x960b2d12, SUC_lexicon_tags_28 },
    { 0x886b2d13, SUC_lexicon_tags_1 },
    { 0x78b2d17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36eb2d19, SUC_lexicon_tags_25 },
    { 0x512b2d19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdeb2d1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02b2d23, SUC_lexicon_tags_1 },
    { 0x7cab2d24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61eb2d2a, SUC_lexicon_tags_47 },
    { 0x1a6b2d2a, SUC_lexicon_tags_11 },
    { 0xb86b2d2c, SUC_lexicon_tags_39 },
    { 0xb52b2d2d, SUC_lexicon_tags_47 },
    { 0xc52b2d2e, SUC_lexicon_tags_23 },
    { 0x6e0b2d2d, SUC_lexicon_tags_17 },
    { 0x404b2d2d, SUC_lexicon_tags_14 },
    { 0x1d0b2d2c, SUC_lexicon_tags_23 },
    { 0x39cb2d32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x436b2d35, SUC_lexicon_tags_2 },
    { 0x8dab2d36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x118b2d38, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x99ab2d3a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ab2d3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250b2d4b, SUC_lexicon_tags_37 },
    { 0xcecb2d4c, SUC_lexicon_tags_11 },
    { 0x9b2b2d4d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eb2d53, SUC_lexicon_tags_14 },
    { 0xe32b2d53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eb2d59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10b2d5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cb2d63, SUC_lexicon_tags_37 },
    { 0x99ab2d63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x114b2d66, SUC_lexicon_tags_35 },
    { 0x1fab2d67, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x402b2d69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x33eb2d6b, SUC_lexicon_tags_37 },
    { 0x9f0b2d6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b2d6f, SUC_lexicon_tags_28 },
    { 0x810b2d70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92cb2d72, SUC_lexicon_tags_17 },
    { 0xf70b2d73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cb2d77, SUC_lexicon_tags_15 },
    { 0x8cb2d78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eb2d7e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xea4b2d80, SUC_lexicon_tags_15 },
    { 0x34cb2d81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0b2d85, SUC_lexicon_tags_7 },
    { 0x6aeb2d85, SUC_lexicon_tags_37 },
    { 0x5c4b2d87, SUC_lexicon_tags_37 },
    { 0x9aab2d88, SUC_lexicon_tags_25 },
    { 0xc8b2d87, SUC_lexicon_tags_9 },
    { 0xcf8b2d8a, SUC_lexicon_tags_12 },
    { 0xdfeb2d8a, SUC_lexicon_tags_56 },
    { 0x5e6b2d8c, SUC_lexicon_tags_82 },
    { 0xd1eb2d8b, SUC_lexicon_tags_68 },
    { 0x8a8b2d8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ab2d91, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6e4b2d93, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x556b2d96, SUC_lexicon_tags_1133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0b2d9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cb2d9e, SUC_lexicon_tags_37 },
    { 0xb14b2d9e, SUC_lexicon_tags_21 },
    { 0x530b2d9f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x422b2da3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecb2da9, SUC_lexicon_tags_7 },
    { 0x724b2daa, SUC_lexicon_tags_23 },
    { 0x8c4b2dab, SUC_lexicon_tags_30 },
    { 0x1eab2dac, SUC_lexicon_tags_10 },
    { 0x99ab2dad, SUC_lexicon_tags_38 },
    { 0x424b2dae, SUC_lexicon_tags_65 },
    { 0xee0b2daa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0b2db2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956b2db6, SUC_lexicon_tags_92 },
    { 0x27eb2db7, SUC_lexicon_tags_126 },
    { 0x756b2db7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10b2dbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb4b2dbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6b2dc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9eb2dc3, SUC_lexicon_tags_49 },
    { 0x82b2dc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300b2dc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72b2dce, SUC_lexicon_tags_42 },
    { 0x78ab2dce, SUC_lexicon_tags_47 },
    { 0x6cab2dd0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8b2dd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0b2ddc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x84eb2ddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cab2de2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5acb2de4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcb2de8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4b2deb, SUC_lexicon_tags_17 },
    { 0xd78b2deb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa1cb2dee, SUC_lexicon_tags_23 },
    { 0x3e2b2def, SUC_lexicon_tags_65 },
    { 0xbceb2dee, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc5eb2df2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x430b2df5, SUC_lexicon_tags_144 },
    { 0x504b2df6, SUC_lexicon_tags_25 },
    { 0x4eb2df5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6b2dfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cb2dff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49cb2e01, SUC_lexicon_tags_63 },
    { 0xe08b2e01, SUC_lexicon_tags_7 },
    { 0x676b2e01, SUC_lexicon_tags_1368 },
    { 0x1d0b2e04, SUC_lexicon_tags_55 },
    { 0x218b2e04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacb2e09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfecb2e0b, SUC_lexicon_tags_3 },
    { 0xf6eb2e0c, SUC_lexicon_tags_17 },
    { 0xd80b2e0d, SUC_lexicon_tags_16 },
    { 0x666b2e0c, SUC_lexicon_tags_25 },
    { 0xe2eb2e0c, SUC_lexicon_tags_25 },
    { 0x7aab2e10, SUC_lexicon_tags_49 },
    { 0x56cb2e10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2b2e17, SUC_lexicon_tags_66 },
    { 0x742b2e18, SUC_lexicon_tags_17 },
    { 0xbeb2e19, SUC_lexicon_tags_17 },
    { 0x666b2e19, SUC_lexicon_tags_27 },
    { 0x7b0b2e1b, SUC_lexicon_tags_43 },
    { 0x524b2e17, SUC_lexicon_tags_37 },
    { 0xe94b2e1d, SUC_lexicon_tags_69 },
    { 0xbfab2e1e, SUC_lexicon_tags_60 },
    { 0x12b2e1f, SUC_lexicon_tags_2 },
    { 0x372b2e1f, SUC_lexicon_tags_17 },
    { 0xf46b2e20, SUC_lexicon_tags_32 },
    { 0xc4ab2e22, SUC_lexicon_tags_15 },
    { 0x1e2b2e21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ab2e26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eb2e29, SUC_lexicon_tags_37 },
    { 0x182b2e29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778b2e2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18b2e32, SUC_lexicon_tags_60 },
    { 0x4beb2e32, SUC_lexicon_tags_11 },
    { 0x8e2b2e33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634b2e3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19eb2e3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4b2e43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8b2e45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecb2e49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe92b2e4b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700b2e52, SUC_lexicon_tags_4 },
    { 0xf2cb2e53, SUC_lexicon_tags_1 },
    { 0x34cb2e53, SUC_lexicon_tags_25 },
    { 0xb5ab2e55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66eb2e57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4b2e5b, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xc9cb2e5d, SUC_lexicon_tags_6 },
    { 0x252b2e5e, SUC_lexicon_tags_98 },
    { 0x3d6b2e5d, SUC_lexicon_tags_15 },
    { 0xf84b2e5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62eb2e63, SUC_lexicon_tags_8 },
    { 0xac6b2e64, SUC_lexicon_tags_17 },
    { 0x36b2e63, SUC_lexicon_tags_27 },
    { 0xbb2b2e66, SUC_lexicon_tags_9 },
    { 0xdbab2e67, SUC_lexicon_tags_14 },
    { 0xea0b2e68, SUC_lexicon_tags_49 },
    { 0x97ab2e69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60cb2e6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x784b2e6e, SUC_lexicon_tags_11 },
    { 0x502b2e6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392b2e75, SUC_lexicon_tags_27 },
    { 0xcaeb2e76, SUC_lexicon_tags_12 },
    { 0x926b2e76, SUC_lexicon_tags_27 },
    { 0xb44b2e78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4feb2e7d, SUC_lexicon_tags_25 },
    { 0xa50b2e7e, SUC_lexicon_tags_17 },
    { 0xdb4b2e7f, SUC_lexicon_tags_19 },
    { 0x922b2e80, SUC_lexicon_tags_19 },
    { 0x7f4b2e80, SUC_lexicon_tags_83 },
    { 0x52ab2e82, SUC_lexicon_tags_17 },
    { 0xcd6b2e82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f0b2e85, SUC_lexicon_tags_11 },
    { 0xeccb2e86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb06b2e88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cb2e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cb2e98, SUC_lexicon_tags_6 },
    { 0x25eb2e98, SUC_lexicon_tags_31 },
    { 0x312b2e9a, SUC_lexicon_tags_48 },
    { 0x878b2e9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa72b2e9d, SUC_lexicon_tags_37 },
    { 0xfdeb2e9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68eb2ea2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400b2ea8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeb2eaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58ab2eb1, SUC_lexicon_tags_82 },
    { 0xd14b2eb2, SUC_lexicon_tags_61 },
    { 0x81cb2eb2, SUC_lexicon_tags_553 },
    { 0xc06b2eb3, SUC_lexicon_tags_25 },
    { 0x90b2eb5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48ab2eb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2eba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70b2ebd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0b2ec6, SUC_lexicon_tags_57 },
    { 0x2e2b2ec7, SUC_lexicon_tags_45 },
    { 0x904b2ec8, SUC_lexicon_tags_2 },
    { 0x30ab2ec6, SUC_lexicon_tags_2 },
    { 0xf46b2eca, SUC_lexicon_tags_38 },
    { 0xf8b2ecb, SUC_lexicon_tags_39 },
    { 0xc8ab2ecc, SUC_lexicon_tags_14 },
    { 0xe40b2ecd, SUC_lexicon_tags_26 },
    { 0xc6cb2eca, SUC_lexicon_tags_1 },
    { 0x848b2ecb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2b2ed6, SUC_lexicon_tags_37 },
    { 0x5eb2ed7, SUC_lexicon_tags_11 },
    { 0x42cb2ed6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4e0b2eda, SUC_lexicon_tags_38 },
    { 0x234b2eda, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b2ee2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfcb2ee4, SUC_lexicon_tags_17 },
    { 0xdb4b2ee5, SUC_lexicon_tags_37 },
    { 0xff2b2ee4, SUC_lexicon_tags_46 },
    { 0x48b2ee5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4b2eea, SUC_lexicon_tags_37 },
    { 0x44ab2eeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fcb2eed, SUC_lexicon_tags_7 },
    { 0x6aab2eee, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94b2ef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2b2ef6, SUC_lexicon_tags_25 },
    { 0xe94b2ef7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8b2efb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ab2f02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ab2f04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x612b2f06, SUC_lexicon_tags_15 },
    { 0x5e6b2f06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2b2f0a, SUC_lexicon_tags_1 },
    { 0x1b8b2f0b, SUC_lexicon_tags_15 },
    { 0x44b2f0c, SUC_lexicon_tags_17 },
    { 0x5cb2f0d, SUC_lexicon_tags_25 },
    { 0x6feb2f0d, SUC_lexicon_tags_23 },
    { 0xb54b2f0f, SUC_lexicon_tags_14 },
    { 0x284b2f10, SUC_lexicon_tags_43 },
    { 0x2b6b2f0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcceb2f14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x820b2f16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaf2b2f18, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x670b2f1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48b2f1d, SUC_lexicon_tags_48 },
    { 0xc18b2f1d, SUC_lexicon_tags_57 },
    { 0xe62b2f1f, SUC_lexicon_tags_11 },
    { 0xeaab2f1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170b2f24, SUC_lexicon_tags_39 },
    { 0x79ab2f24, SUC_lexicon_tags_11 },
    { 0x42cb2f24, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8b2f2c, SUC_lexicon_tags_7 },
    { 0x15cb2f2d, SUC_lexicon_tags_15 },
    { 0xd26b2f2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cab2f31, SUC_lexicon_tags_38 },
    { 0xc36b2f32, SUC_lexicon_tags_15 },
    { 0x1c4b2f33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6b2f36, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x426b2f39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ab2f3d, SUC_lexicon_tags_30 },
    { 0x82eb2f3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2b2f41, SUC_lexicon_tags_25 },
    { 0x70b2f41, SUC_lexicon_tags_30 },
    { 0xb2ab2f41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cb2f46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eb2f49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x78cb2f4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x66cb2f4d, SUC_lexicon_tags_37 },
    { 0xbe4b2f4e, SUC_lexicon_tags_37 },
    { 0xf4cb2f4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfe0b2f51, SUC_lexicon_tags_2 },
    { 0x3dcb2f52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb8b2f54, SUC_lexicon_tags_1 },
    { 0x8a2b2f55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0b2f58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4b2f5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cb2f64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268b2f6e, SUC_lexicon_tags_0 },
    { 0x608b2f6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa22b2f71, SUC_lexicon_tags_77 },
    { 0xc22b2f72, SUC_lexicon_tags_10 },
    { 0xc86b2f73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x838b2f75, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x97cb2f78, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadeb2f7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb2f80, SUC_lexicon_tags_1 },
    { 0x270b2f81, SUC_lexicon_tags_43 },
    { 0x6c8b2f81, SUC_lexicon_tags_7 },
    { 0x2f2b2f80, SUC_lexicon_tags_57 },
    { 0x5c4b2f84, SUC_lexicon_tags_17 },
    { 0xdbab2f84, SUC_lexicon_tags_23 },
    { 0x488b2f81, SUC_lexicon_tags_12 },
    { 0x818b2f85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688b2f8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188b2f91, SUC_lexicon_tags_17 },
    { 0x4d4b2f91, SUC_lexicon_tags_26 },
    { 0xd8eb2f91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cb2f98, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cb2f9c, SUC_lexicon_tags_37 },
    { 0x7d0b2f9d, SUC_lexicon_tags_43 },
    { 0x9e2b2f9c, SUC_lexicon_tags_46 },
    { 0x696b2f9f, SUC_lexicon_tags_0 },
    { 0xfdab2f9d, SUC_lexicon_tags_25 },
    { 0x240b2fa1, SUC_lexicon_tags_45 },
    { 0x8a0b2fa2, SUC_lexicon_tags_11 },
    { 0xc56b2fa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18b2faa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x986b2fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x358b2faf, SUC_lexicon_tags_7 },
    { 0xf3ab2fb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6b2fb3, SUC_lexicon_tags_1 },
    { 0x376b2fb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa4b2fb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ab2fba, SUC_lexicon_tags_43 },
    { 0xda2b2fbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ab2fc3, SUC_lexicon_tags_53 },
    { 0x8dab2fc3, SUC_lexicon_tags_7 },
    { 0x46eb2fc5, SUC_lexicon_tags_11 },
    { 0x704b2fc6, SUC_lexicon_tags_11 },
    { 0xdd0b2fc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b2fca, SUC_lexicon_tags_63 },
    { 0x2ceb2fcb, SUC_lexicon_tags_5 },
    { 0x934b2fcc, SUC_lexicon_tags_2 },
    { 0xb48b2fcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96eb2fd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6b2fd7, SUC_lexicon_tags_29 },
    { 0x5ab2fd7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc20b2fda, SUC_lexicon_tags_12 },
    { 0x656b2fda, SUC_lexicon_tags_98 },
    { 0xcbab2fdb, SUC_lexicon_tags_25 },
    { 0xab2b2fdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730b2fe2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x832b2fe4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4b2fe9, SUC_lexicon_tags_27 },
    { 0x4c4b2fe9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78b2fee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0b2ff2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2b2ff6, SUC_lexicon_tags_37 },
    { 0xe4ab2ff6, SUC_lexicon_tags_27 },
    { 0x79ab2ff6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49eb2ffa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x40b2ffc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66b3000, SUC_lexicon_tags_33 },
    { 0x8a2b3000, SUC_lexicon_tags_54 },
    { 0xa40b3002, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cab3005, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb300a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4b3013, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e6b3015, SUC_lexicon_tags_1 },
    { 0x110b3015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bab301a, SUC_lexicon_tags_37 },
    { 0xebeb301b, SUC_lexicon_tags_15 },
    { 0xc3eb301c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16eb3021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ab3025, SUC_lexicon_tags_12 },
    { 0x3d2b3025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0b3029, SUC_lexicon_tags_0 },
    { 0xdfcb302a, SUC_lexicon_tags_12 },
    { 0xd5cb302a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b302e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb02b3030, SUC_lexicon_tags_33 },
    { 0x6d4b3031, SUC_lexicon_tags_62 },
    { 0xefeb3032, SUC_lexicon_tags_28 },
    { 0xbab3033, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e8b3035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4b3037, SUC_lexicon_tags_21 },
    { 0x51ab3037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2b303c, SUC_lexicon_tags_16 },
    { 0x930b303d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ab3040, SUC_lexicon_tags_54 },
    { 0x188b3041, SUC_lexicon_tags_15 },
    { 0xceb3041, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2b3047, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb18b3049, SUC_lexicon_tags_59 },
    { 0x666b304a, SUC_lexicon_tags_39 },
    { 0xd72b3049, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ceb304f, SUC_lexicon_tags_33 },
    { 0x154b3050, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0b3054, SUC_lexicon_tags_27 },
    { 0xe1ab3054, SUC_lexicon_tags_47 },
    { 0x5f2b3056, SUC_lexicon_tags_15 },
    { 0x902b3056, SUC_lexicon_tags_47 },
    { 0x90ab3054, SUC_lexicon_tags_25 },
    { 0x68eb3059, SUC_lexicon_tags_116 },
    { 0x894b305a, SUC_lexicon_tags_17 },
    { 0x316b3055, SUC_lexicon_tags_17 },
    { 0x4fcb3057, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34eb3060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154b3065, SUC_lexicon_tags_15 },
    { 0x7b0b3065, SUC_lexicon_tags_17 },
    { 0xc42b3067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x326b306a, SUC_lexicon_tags_16 },
    { 0xea2b306b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb3071, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25eb307a, SUC_lexicon_tags_47 },
    { 0x258b307a, SUC_lexicon_tags_8 },
    { 0xa44b307a, SUC_lexicon_tags_17 },
    { 0x748b307d, SUC_lexicon_tags_17 },
    { 0x5cab307c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x366b3080, SUC_lexicon_tags_12 },
    { 0x618b3081, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eeb3084, SUC_lexicon_tags_12 },
    { 0x266b3085, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd4b3087, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aeb308b, SUC_lexicon_tags_174 },
    { 0x818b308c, SUC_lexicon_tags_13 },
    { 0xcbab308b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0b3090, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544b3094, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x554b3096, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eb3099, SUC_lexicon_tags_17 },
    { 0xd46b309a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b309e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0cb30a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdcb30a2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86b30a7, SUC_lexicon_tags_15 },
    { 0x82b30a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710b30ae, SUC_lexicon_tags_29 },
    { 0xe84b30ae, SUC_lexicon_tags_1 },
    { 0x410b30b0, SUC_lexicon_tags_57 },
    { 0xab8b30b1, SUC_lexicon_tags_37 },
    { 0x666b30b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x262b30b5, SUC_lexicon_tags_114 },
    { 0xec4b30b5, SUC_lexicon_tags_33 },
    { 0xc5eb30b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720b30bb, SUC_lexicon_tags_55 },
    { 0x828b30bc, SUC_lexicon_tags_32 },
    { 0x2ccb30bd, SUC_lexicon_tags_49 },
    { 0xc54b30be, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf1ab30c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22b30c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce8b30c5, SUC_lexicon_tags_1 },
    { 0x218b30c6, SUC_lexicon_tags_11 },
    { 0x69cb30c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14eb30ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x168b30cc, SUC_lexicon_tags_12 },
    { 0xb1eb30cc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7a8b30cf, SUC_lexicon_tags_19 },
    { 0x42eb30cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b30d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acb30d7, SUC_lexicon_tags_16 },
    { 0x77eb30d8, SUC_lexicon_tags_48 },
    { 0xe92b30d8, SUC_lexicon_tags_25 },
    { 0x1f6b30d9, SUC_lexicon_tags_2 },
    { 0xb72b30da, SUC_lexicon_tags_21 },
    { 0xf5eb30dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cb30e4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0b30eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150b30f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2b30f5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x30b30f7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4eb30fa, SUC_lexicon_tags_37 },
    { 0x22cb30fb, SUC_lexicon_tags_40 },
    { 0x790b30fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7ab30fe, SUC_lexicon_tags_1 },
    { 0x12ab30ff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2b8b3101, SUC_lexicon_tags_38 },
    { 0xc8cb3101, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2a0b3104, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x6d2b3106, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b3109, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66b310e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86b3111, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x13ab3113, SUC_lexicon_tags_17 },
    { 0xef2b3114, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37eb311f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e0b3121, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9beb3125, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838b312a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa14b312c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4b312f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4b3132, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x350b3134, SUC_lexicon_tags_6 },
    { 0x522b3135, SUC_lexicon_tags_17 },
    { 0x6a2b3136, SUC_lexicon_tags_33 },
    { 0x72eb3136, SUC_lexicon_tags_17 },
    { 0xc34b3136, SUC_lexicon_tags_28 },
    { 0x938b3139, SUC_lexicon_tags_13 },
    { 0xa8cb3139, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4b3140, SUC_lexicon_tags_17 },
    { 0x60cb3140, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226b3146, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeeab3148, SUC_lexicon_tags_1 },
    { 0x2bcb3149, SUC_lexicon_tags_11 },
    { 0xf4b3148, SUC_lexicon_tags_7 },
    { 0xed2b314b, SUC_lexicon_tags_37 },
    { 0x7ceb314c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2b3158, SUC_lexicon_tags_11 },
    { 0x814b3159, SUC_lexicon_tags_37 },
    { 0xe42b3158, SUC_lexicon_tags_24 },
    { 0x124b315a, SUC_lexicon_tags_23 },
    { 0x2b0b315b, SUC_lexicon_tags_25 },
    { 0x88b3158, SUC_lexicon_tags_17 },
    { 0xd56b3159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0eb3160, SUC_lexicon_tags_60 },
    { 0xbfcb3160, SUC_lexicon_tags_25 },
    { 0x62eb3162, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x926b3164, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3168, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x866b316a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218b316f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x762b3171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadcb3177, SUC_lexicon_tags_3 },
    { 0xceeb3178, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88b317b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2cb317d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x65eb3180, SUC_lexicon_tags_65 },
    { 0xbfcb3180, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x370b3183, SUC_lexicon_tags_45 },
    { 0xb74b3183, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x618b3186, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eeb3190, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b3198, SUC_lexicon_tags_28 },
    { 0x8dcb3198, SUC_lexicon_tags_116 },
    { 0xc9ab3199, SUC_lexicon_tags_2 },
    { 0xb40b3199, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cb319e, SUC_lexicon_tags_7 },
    { 0xb76b319e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4b31a3, SUC_lexicon_tags_63 },
    { 0xce0b31a4, SUC_lexicon_tags_29 },
    { 0x12eb31a4, SUC_lexicon_tags_65 },
    { 0xa52b31a3, SUC_lexicon_tags_27 },
    { 0x870b31a3, SUC_lexicon_tags_12 },
    { 0x546b31a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9eab31aa, SUC_lexicon_tags_31 },
    { 0x6e2b31ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226b31b2, SUC_lexicon_tags_25 },
    { 0xee6b31b3, SUC_lexicon_tags_33 },
    { 0x2c8b31b4, SUC_lexicon_tags_47 },
    { 0xfd6b31b4, SUC_lexicon_tags_37 },
    { 0x196b31b5, SUC_lexicon_tags_17 },
    { 0x91cb31b5, SUC_lexicon_tags_3 },
    { 0x97ab31b4, SUC_lexicon_tags_9 },
    { 0xc9cb31b4, SUC_lexicon_tags_118 },
    { 0xf32b31b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6b31be, SUC_lexicon_tags_65 },
    { 0x2e4b31bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622b31c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x626b31c6, SUC_lexicon_tags_25 },
    { 0xc70b31c7, SUC_lexicon_tags_1 },
    { 0xa0b31c7, SUC_lexicon_tags_9 },
    { 0x7eb31c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf26b31cb, SUC_lexicon_tags_49 },
    { 0xb1eb31cb, SUC_lexicon_tags_48 },
    { 0x294b31cb, SUC_lexicon_tags_15 },
    { 0x432b31cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53ab31d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4b31d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12eb31d7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x10ab31d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbecb31db, SUC_lexicon_tags_12 },
    { 0x3e4b31dc, SUC_lexicon_tags_25 },
    { 0xcb0b31dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeaeb31df, SUC_lexicon_tags_15 },
    { 0x708b31df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x64cb31e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x484b31e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1eeb31e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceb31f1, SUC_lexicon_tags_1 },
    { 0xe4ab31f2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6b31f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40b31fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896b3202, SUC_lexicon_tags_43 },
    { 0x6c4b3203, SUC_lexicon_tags_37 },
    { 0x3e2b3204, SUC_lexicon_tags_25 },
    { 0xd30b3204, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x11cb3207, SUC_lexicon_tags_1 },
    { 0x784b3208, SUC_lexicon_tags_12 },
    { 0x4b8b3208, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2b320d, SUC_lexicon_tags_56 },
    { 0xf46b320e, SUC_lexicon_tags_17 },
    { 0x534b320d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eeb3212, SUC_lexicon_tags_25 },
    { 0x710b3213, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6beb3217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8b321c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x784b321e, SUC_lexicon_tags_25 },
    { 0x1beb321f, SUC_lexicon_tags_1 },
    { 0x162b321e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x172b3222, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ccb3224, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332b322b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416b322f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b3235, SUC_lexicon_tags_17 },
    { 0xa64b3236, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30b323f, SUC_lexicon_tags_17 },
    { 0x63ab3240, SUC_lexicon_tags_21 },
    { 0x5fcb3241, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26b3244, SUC_lexicon_tags_82 },
    { 0xb8ab3244, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdc4b3247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362b3250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bab3256, SUC_lexicon_tags_12 },
    { 0xab8b3257, SUC_lexicon_tags_1 },
    { 0x1eeb3256, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4b325e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb3264, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28b326b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x396b326d, SUC_lexicon_tags_30 },
    { 0xb7ab326d, SUC_lexicon_tags_47 },
    { 0xa3ab326f, SUC_lexicon_tags_8 },
    { 0x490b326f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd94b3272, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6b3275, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ab3279, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6b327d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecb3280, SUC_lexicon_tags_7 },
    { 0xa22b3281, SUC_lexicon_tags_2 },
    { 0xae2b3282, SUC_lexicon_tags_63 },
    { 0x4b2b3282, SUC_lexicon_tags_14 },
    { 0xc06b3283, SUC_lexicon_tags_15 },
    { 0x1c2b3280, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x36b3287, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x696b3289, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ab3295, SUC_lexicon_tags_23 },
    { 0xa2ab3296, SUC_lexicon_tags_5 },
    { 0x2c6b3297, SUC_lexicon_tags_25 },
    { 0x8d2b3297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8b329c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324b32a2, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb32a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6b32aa, SUC_lexicon_tags_25 },
    { 0x6d2b32aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02b32b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x366b32b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54eb32b7, SUC_lexicon_tags_8 },
    { 0x92b32b8, SUC_lexicon_tags_47 },
    { 0x29eb32b7, SUC_lexicon_tags_49 },
    { 0x70ab32b9, SUC_lexicon_tags_28 },
    { 0xff4b32b8, SUC_lexicon_tags_11 },
    { 0x1f6b32b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38b32c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4b32c5, SUC_lexicon_tags_49 },
    { 0x5a6b32c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa2eb32c8, SUC_lexicon_tags_55 },
    { 0x64b32c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe4cb32cb, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cb32d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70cb32d4, SUC_lexicon_tags_11 },
    { 0x452b32d5, SUC_lexicon_tags_2 },
    { 0xcb8b32d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234b32da, SUC_lexicon_tags_48 },
    { 0xf96b32db, SUC_lexicon_tags_37 },
    { 0xce4b32dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14b32e1, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1eb32e4, SUC_lexicon_tags_7 },
    { 0xfdcb32e5, SUC_lexicon_tags_17 },
    { 0xeecb32e6, SUC_lexicon_tags_37 },
    { 0x57ab32e7, SUC_lexicon_tags_43 },
    { 0x71cb32e8, SUC_lexicon_tags_1 },
    { 0x3b2b32e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d0b32eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8b32f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12b32f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2b32f9, SUC_lexicon_tags_1 },
    { 0x70b32f9, SUC_lexicon_tags_17 },
    { 0xd8ab32fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x660b32fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadeb32ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6b3305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36b3309, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30eb330f, SUC_lexicon_tags_15 },
    { 0xe2b3310, SUC_lexicon_tags_1 },
    { 0xf38b3311, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x110b3313, SUC_lexicon_tags_25 },
    { 0x228b3314, SUC_lexicon_tags_37 },
    { 0xab0b3314, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc4b3317, SUC_lexicon_tags_27 },
    { 0x50cb3317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52b331b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8b331e, SUC_lexicon_tags_49 },
    { 0x880b331f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8b3322, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x124b3324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x778b3326, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x342b3328, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe60b332a, SUC_lexicon_tags_8 },
    { 0x176b332a, SUC_lexicon_tags_15 },
    { 0x288b332c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32b3333, SUC_lexicon_tags_12 },
    { 0x97eb3334, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472b333a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956b333e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cb3344, SUC_lexicon_tags_9 },
    { 0xd08b3345, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6b334e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10eb3355, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2b335b, SUC_lexicon_tags_1 },
    { 0x324b335b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cb335f, SUC_lexicon_tags_16 },
    { 0xc96b335f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e4b3362, SUC_lexicon_tags_38 },
    { 0x864b3362, SUC_lexicon_tags_4 },
    { 0x858b3364, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc48b3366, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43cb3368, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x86eb336b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90b336f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41eb3375, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf1cb3377, SUC_lexicon_tags_2 },
    { 0x944b3378, SUC_lexicon_tags_1 },
    { 0xf6b3378, SUC_lexicon_tags_1 },
    { 0xb8cb3377, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb20b337c, SUC_lexicon_tags_7 },
    { 0x5fab337d, SUC_lexicon_tags_45 },
    { 0xe6b337d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccb3382, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17ab3384, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30b3388, SUC_lexicon_tags_25 },
    { 0x70cb3389, SUC_lexicon_tags_46 },
    { 0x416b338a, SUC_lexicon_tags_12 },
    { 0x998b338b, SUC_lexicon_tags_45 },
    { 0xb32b338c, SUC_lexicon_tags_1 },
    { 0x9ab338d, SUC_lexicon_tags_8 },
    { 0x948b3389, SUC_lexicon_tags_11 },
    { 0xfb4b338a, SUC_lexicon_tags_165 },
    { 0x7e2b338a, SUC_lexicon_tags_27 },
    { 0xb0b3391, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7d4b3393, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66eb3397, SUC_lexicon_tags_10 },
    { 0x8a0b3398, SUC_lexicon_tags_40 },
    { 0xacb3399, SUC_lexicon_tags_25 },
    { 0x86eb339a, SUC_lexicon_tags_29 },
    { 0x52ab339b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6b339e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a8b33a0, SUC_lexicon_tags_15 },
    { 0x8a8b33a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ab33a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930b33a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2deb33ae, SUC_lexicon_tags_7 },
    { 0xbd8b33af, SUC_lexicon_tags_12 },
    { 0xf74b33af, SUC_lexicon_tags_17 },
    { 0x96ab33b0, SUC_lexicon_tags_27 },
    { 0x564b33b2, SUC_lexicon_tags_31 },
    { 0xbc0b33b2, SUC_lexicon_tags_23 },
    { 0x98eb33b4, SUC_lexicon_tags_46 },
    { 0xfa0b33b2, SUC_lexicon_tags_43 },
    { 0xdf0b33b6, SUC_lexicon_tags_37 },
    { 0xb68b33b7, SUC_lexicon_tags_65 },
    { 0x12eb33b6, SUC_lexicon_tags_8 },
    { 0x4b8b33b9, SUC_lexicon_tags_0 },
    { 0x834b33ae, SUC_lexicon_tags_1 },
    { 0x350b33bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85eb33bf, SUC_lexicon_tags_11 },
    { 0x5e6b33bf, SUC_lexicon_tags_34 },
    { 0x862b33c0, SUC_lexicon_tags_15 },
    { 0x948b33c2, SUC_lexicon_tags_31 },
    { 0x66ab33c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4deb33c6, SUC_lexicon_tags_25 },
    { 0x910b33c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cab33c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96eb33cb, SUC_lexicon_tags_39 },
    { 0x86ab33cb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x510b33cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460b33d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5acb33d5, SUC_lexicon_tags_28 },
    { 0xd8b33d5, SUC_lexicon_tags_88 },
    { 0x7acb33d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754b33db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f6b33dd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8b33e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90eb33e4, SUC_lexicon_tags_26 },
    { 0x5e8b33e5, SUC_lexicon_tags_19 },
    { 0xe9cb33e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe10b33e8, SUC_lexicon_tags_25 },
    { 0x6b33e9, SUC_lexicon_tags_37 },
    { 0x2b6b33ea, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4b33ef, SUC_lexicon_tags_17 },
    { 0x19ab33f0, SUC_lexicon_tags_6 },
    { 0x83eb33f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24ab33f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dab33f7, SUC_lexicon_tags_63 },
    { 0x8acb33f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960b33ff, SUC_lexicon_tags_12 },
    { 0xa20b3400, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b2b3402, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398b340c, SUC_lexicon_tags_405 },
    { 0x1beb340d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352b3411, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcab3413, SUC_lexicon_tags_38 },
    { 0xe64b3413, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248b3419, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acb341d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x18cb341f, SUC_lexicon_tags_16 },
    { 0x2eeb3420, SUC_lexicon_tags_87 },
    { 0xedeb3421, SUC_lexicon_tags_17 },
    { 0xb76b3422, SUC_lexicon_tags_52 },
    { 0x6f4b3423, SUC_lexicon_tags_25 },
    { 0x76cb3421, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb342e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216b3435, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb76b3437, SUC_lexicon_tags_59 },
    { 0xfc2b3437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ab343c, SUC_lexicon_tags_77 },
    { 0x82b343c, SUC_lexicon_tags_19 },
    { 0xbe2b343c, SUC_lexicon_tags_21 },
    { 0xcd4b343f, SUC_lexicon_tags_7 },
    { 0x936b3440, SUC_lexicon_tags_13 },
    { 0x4a4b3440, SUC_lexicon_tags_7 },
    { 0xc56b3440, SUC_lexicon_tags_49 },
    { 0x758b3442, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322b3447, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x150b344a, SUC_lexicon_tags_11 },
    { 0x5cb344b, SUC_lexicon_tags_1 },
    { 0xdc4b344c, SUC_lexicon_tags_1 },
    { 0x348b344d, SUC_lexicon_tags_19 },
    { 0x2b6b344a, SUC_lexicon_tags_2 },
    { 0x704b344b, SUC_lexicon_tags_15 },
    { 0x65eb3450, SUC_lexicon_tags_17 },
    { 0x24ab3450, SUC_lexicon_tags_27 },
    { 0xd7ab344f, SUC_lexicon_tags_6 },
    { 0x766b344b, SUC_lexicon_tags_29 },
    { 0x4b6b3451, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a0b3456, SUC_lexicon_tags_2 },
    { 0x8eeb3457, SUC_lexicon_tags_38 },
    { 0xd76b3457, SUC_lexicon_tags_12 },
    { 0xb34b3458, SUC_lexicon_tags_37 },
    { 0x994b3457, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4b3461, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35eb3463, SUC_lexicon_tags_17 },
    { 0x828b3464, SUC_lexicon_tags_79 },
    { 0x242b3465, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2b3468, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6b346c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x810b346e, SUC_lexicon_tags_37 },
    { 0xa12b346f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2b3472, SUC_lexicon_tags_15 },
    { 0x786b3472, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50b3479, SUC_lexicon_tags_30 },
    { 0x884b347a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74b347d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2b3481, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8b3488, SUC_lexicon_tags_1 },
    { 0x12eb3488, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcb348c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2b348f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4b3494, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b3497, SUC_lexicon_tags_2 },
    { 0xc72b3498, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x21eb349a, SUC_lexicon_tags_13 },
    { 0xdb4b349a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1cb349d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212b34a1, SUC_lexicon_tags_33 },
    { 0xaa4b34a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852b34a9, SUC_lexicon_tags_10 },
    { 0x7fab34aa, SUC_lexicon_tags_25 },
    { 0x9dab34aa, SUC_lexicon_tags_63 },
    { 0xfd2b34a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200b34b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbab34b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2b34b6, SUC_lexicon_tags_37 },
    { 0x520b34b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0b34bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed6b34bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30b34c1, SUC_lexicon_tags_17 },
    { 0x726b34c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4ab34c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0eb34c6, SUC_lexicon_tags_66 },
    { 0x2b6b34c7, SUC_lexicon_tags_30 },
    { 0xda6b34c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ab34cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x492b34cd, SUC_lexicon_tags_7 },
    { 0xa70b34ce, SUC_lexicon_tags_25 },
    { 0xb1eb34cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7dab34d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a2b34d3, SUC_lexicon_tags_8 },
    { 0xc44b34d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecb34d8, SUC_lexicon_tags_15 },
    { 0x976b34d8, SUC_lexicon_tags_45 },
    { 0x8eeb34da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5e4b34dc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x26eb34de, SUC_lexicon_tags_27 },
    { 0x7d4b34de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x320b34e1, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xaaab34e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80b34e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2cb34e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd14b34eb, SUC_lexicon_tags_19 },
    { 0x566b34ec, SUC_lexicon_tags_14 },
    { 0x1b0b34ed, SUC_lexicon_tags_1 },
    { 0x6ab34ee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcb34f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9eb34f4, SUC_lexicon_tags_23 },
    { 0xcd2b34f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74ab34f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2b3500, SUC_lexicon_tags_12 },
    { 0x914b3501, SUC_lexicon_tags_9 },
    { 0xfe8b3501, SUC_lexicon_tags_15 },
    { 0xdc4b3501, SUC_lexicon_tags_1 },
    { 0x5e8b3504, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92b3508, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf8ab350a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b350e, SUC_lexicon_tags_30 },
    { 0x8d0b350f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32b3512, SUC_lexicon_tags_19 },
    { 0xd1cb3513, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x258b3516, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x73ab3518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x156b351a, SUC_lexicon_tags_2 },
    { 0xe2ab351b, SUC_lexicon_tags_1 },
    { 0xd6ab351c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64eb3521, SUC_lexicon_tags_6 },
    { 0x582b3522, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecb3532, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x374b3534, SUC_lexicon_tags_15 },
    { 0xadab3535, SUC_lexicon_tags_14 },
    { 0x888b3535, SUC_lexicon_tags_25 },
    { 0x63ab3537, SUC_lexicon_tags_47 },
    { 0xfb8b3535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0b353c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cb353f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92eb3541, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2b354c, SUC_lexicon_tags_23 },
    { 0x218b354c, SUC_lexicon_tags_37 },
    { 0x532b354e, SUC_lexicon_tags_81 },
    { 0x674b354f, SUC_lexicon_tags_38 },
    { 0xe00b3550, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd2b3552, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178b3559, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02b355f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaab3564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcb3567, SUC_lexicon_tags_2 },
    { 0x76ab3568, SUC_lexicon_tags_160 },
    { 0xc4eb3569, SUC_lexicon_tags_57 },
    { 0xa50b356a, SUC_lexicon_tags_45 },
    { 0x28ab356a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8b356f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4b3577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80eb357b, SUC_lexicon_tags_25 },
    { 0x6d6b357b, SUC_lexicon_tags_12 },
    { 0x8bcb357d, SUC_lexicon_tags_11 },
    { 0xafeb357e, SUC_lexicon_tags_12 },
    { 0x8d4b357c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0b3583, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4b3586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6b358a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x918b358d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cb3593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf76b3595, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea8b3597, SUC_lexicon_tags_17 },
    { 0xaf2b3598, SUC_lexicon_tags_65 },
    { 0xf4cb3599, SUC_lexicon_tags_34 },
    { 0xefcb359a, SUC_lexicon_tags_45 },
    { 0xf2b3599, SUC_lexicon_tags_2 },
    { 0xc70b359c, SUC_lexicon_tags_12 },
    { 0xd72b359c, SUC_lexicon_tags_17 },
    { 0x1a6b3599, SUC_lexicon_tags_0 },
    { 0xf2cb359d, SUC_lexicon_tags_33 },
    { 0xeeab3598, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04b35a5, SUC_lexicon_tags_3 },
    { 0x792b35a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b35a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90b35b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x458b35b3, SUC_lexicon_tags_12 },
    { 0x6f0b35b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0b35b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2b35bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64b35bf, SUC_lexicon_tags_25 },
    { 0x3aeb35c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcb35c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0b35ca, SUC_lexicon_tags_15 },
    { 0x10b35cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ab35cf, SUC_lexicon_tags_46 },
    { 0x8a6b35d0, SUC_lexicon_tags_7 },
    { 0x4f6b35cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x986b35d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954b35d9, SUC_lexicon_tags_12 },
    { 0x304b35da, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x1e8b35dc, SUC_lexicon_tags_1 },
    { 0x61eb35dc, SUC_lexicon_tags_164 },
    { 0x4a2b35de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb02b35e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x658b35e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x858b35e5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ab35ea, SUC_lexicon_tags_14 },
    { 0xc6b35eb, SUC_lexicon_tags_12 },
    { 0x554b35eb, SUC_lexicon_tags_11 },
    { 0xe8cb35ed, SUC_lexicon_tags_11 },
    { 0xa00b35ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab35f2, SUC_lexicon_tags_14 },
    { 0xd40b35f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc86b35f5, SUC_lexicon_tags_26 },
    { 0xf30b35f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x206b35f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918b35fc, SUC_lexicon_tags_4 },
    { 0x232b35fd, SUC_lexicon_tags_63 },
    { 0x566b35fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0b3604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4acb3606, SUC_lexicon_tags_17 },
    { 0xc60b3606, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77eb3613, SUC_lexicon_tags_37 },
    { 0xf50b3614, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f0b3616, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68b3619, SUC_lexicon_tags_19 },
    { 0x3beb361a, SUC_lexicon_tags_87 },
    { 0x8a8b3619, SUC_lexicon_tags_43 },
    { 0xb0eb361c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ab361f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08b3625, SUC_lexicon_tags_28 },
    { 0xdf4b3625, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfbeb3628, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eb362c, SUC_lexicon_tags_17 },
    { 0x25ab362d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268b3631, SUC_lexicon_tags_26 },
    { 0xbc6b3632, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570b3639, SUC_lexicon_tags_8 },
    { 0xbd4b363a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd2b363c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ab363f, SUC_lexicon_tags_4 },
    { 0x638b363f, SUC_lexicon_tags_7 },
    { 0x2deb363f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a4b3643, SUC_lexicon_tags_47 },
    { 0x3feb3643, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128b3648, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2b364d, SUC_lexicon_tags_2 },
    { 0x164b364d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884b3652, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1eb3659, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd5ab365b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd0b365d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cb3661, SUC_lexicon_tags_2 },
    { 0x848b3662, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x41cb3665, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a2b3667, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6b366a, SUC_lexicon_tags_27 },
    { 0xc5cb366b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19ab366d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cb3670, SUC_lexicon_tags_25 },
    { 0xb40b3671, SUC_lexicon_tags_15 },
    { 0x436b3672, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa50b3674, SUC_lexicon_tags_17 },
    { 0x9f4b3674, SUC_lexicon_tags_17 },
    { 0xe8b3676, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25eb367a, SUC_lexicon_tags_61 },
    { 0x254b367b, SUC_lexicon_tags_14 },
    { 0x98eb367a, SUC_lexicon_tags_8 },
    { 0x8acb367c, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4b3684, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18b3689, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fcb368b, SUC_lexicon_tags_1 },
    { 0xb3eb368c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb0b368e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b0b3690, SUC_lexicon_tags_12 },
    { 0x186b3691, SUC_lexicon_tags_45 },
    { 0xad4b3690, SUC_lexicon_tags_17 },
    { 0x58cb3693, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3eb3697, SUC_lexicon_tags_19 },
    { 0x8d2b3698, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704b369e, SUC_lexicon_tags_17 },
    { 0x660b369f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e8b36a1, SUC_lexicon_tags_9 },
    { 0x4d2b36a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b36a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b36a9, SUC_lexicon_tags_19 },
    { 0xb4ab36aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe6eb36ac, SUC_lexicon_tags_8 },
    { 0x320b36ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcb36b0, SUC_lexicon_tags_15 },
    { 0x656b36b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcb36b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14b36b8, SUC_lexicon_tags_0 },
    { 0xe8cb36b8, SUC_lexicon_tags_26 },
    { 0x5bcb36ba, SUC_lexicon_tags_23 },
    { 0x28eb36ba, SUC_lexicon_tags_19 },
    { 0x40eb36bc, SUC_lexicon_tags_2 },
    { 0x540b36bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab4b36bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30b36c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32b36cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574b36d3, SUC_lexicon_tags_12 },
    { 0xe52b36d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b36d9, SUC_lexicon_tags_17 },
    { 0xd82b36d9, SUC_lexicon_tags_23 },
    { 0xb0ab36db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60b36dd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x78ab36df, SUC_lexicon_tags_21 },
    { 0xda6b36e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x288b36e3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6b6b36e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eab36ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2b36f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562b36f7, SUC_lexicon_tags_68 },
    { 0x8d4b36f8, SUC_lexicon_tags_0 },
    { 0xd00b36f9, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6b36ff, SUC_lexicon_tags_7 },
    { 0x800b3700, SUC_lexicon_tags_35 },
    { 0xb30b3701, SUC_lexicon_tags_27 },
    { 0x1deb3701, SUC_lexicon_tags_12 },
    { 0xeaab3702, SUC_lexicon_tags_103 },
    { 0xc8eb3704, SUC_lexicon_tags_31 },
    { 0x39ab3703, SUC_lexicon_tags_21 },
    { 0xf64b3706, SUC_lexicon_tags_168 },
    { 0, NULL },
    { 0xc4cb3708, SUC_lexicon_tags_38 },
    { 0xa2cb3709, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ab370e, SUC_lexicon_tags_37 },
    { 0xf30b370f, SUC_lexicon_tags_1 },
    { 0x68eb370f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cb3719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaab371b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858b371f, SUC_lexicon_tags_7 },
    { 0xed4b3720, SUC_lexicon_tags_16 },
    { 0x94b3721, SUC_lexicon_tags_0 },
    { 0x542b3720, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf60b3724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadeb372d, SUC_lexicon_tags_51 },
    { 0x8a0b372e, SUC_lexicon_tags_2 },
    { 0x62ab372d, SUC_lexicon_tags_25 },
    { 0xd62b3730, SUC_lexicon_tags_4 },
    { 0xfc0b3731, SUC_lexicon_tags_14 },
    { 0xf82b3730, SUC_lexicon_tags_8 },
    { 0x3e4b3730, SUC_lexicon_tags_17 },
    { 0xa10b3734, SUC_lexicon_tags_12 },
    { 0xc2b3734, SUC_lexicon_tags_37 },
    { 0x994b3736, SUC_lexicon_tags_66 },
    { 0xb3ab3736, SUC_lexicon_tags_2 },
    { 0x5c4b3738, SUC_lexicon_tags_4 },
    { 0x6b0b3738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b373c, SUC_lexicon_tags_7 },
    { 0x816b373d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b4b373f, SUC_lexicon_tags_37 },
    { 0x896b3740, SUC_lexicon_tags_31 },
    { 0x2eb3741, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84b3744, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac0b3746, SUC_lexicon_tags_2 },
    { 0x122b3747, SUC_lexicon_tags_26 },
    { 0x7f4b3748, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0eb3753, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ab3757, SUC_lexicon_tags_14 },
    { 0xdeab3757, SUC_lexicon_tags_43 },
    { 0x3a6b3757, SUC_lexicon_tags_43 },
    { 0xdd2b375a, SUC_lexicon_tags_25 },
    { 0x23cb375a, SUC_lexicon_tags_37 },
    { 0xa64b375c, SUC_lexicon_tags_11 },
    { 0x6d2b375a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x818b375f, SUC_lexicon_tags_5 },
    { 0xc40b3760, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc88b3762, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4b3766, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x98eb3769, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368b3771, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x830b3774, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464b377a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16b377e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28b3784, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50b3788, SUC_lexicon_tags_0 },
    { 0x988b3788, SUC_lexicon_tags_44 },
    { 0x6f4b378a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf04b378c, SUC_lexicon_tags_7 },
    { 0x444b378d, SUC_lexicon_tags_13 },
    { 0xfbeb378d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x872b3791, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0b3797, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8aab3799, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0b379e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x290b37a0, SUC_lexicon_tags_92 },
    { 0x43ab37a0, SUC_lexicon_tags_25 },
    { 0x840b37a2, SUC_lexicon_tags_25 },
    { 0x3ceb37a2, SUC_lexicon_tags_0 },
    { 0x4fab37a3, SUC_lexicon_tags_37 },
    { 0x8dab37a5, SUC_lexicon_tags_17 },
    { 0xd40b37a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aab37ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40eb37b1, SUC_lexicon_tags_30 },
    { 0x4a2b37b1, SUC_lexicon_tags_25 },
    { 0x6c0b37b2, SUC_lexicon_tags_2 },
    { 0xe94b37b3, SUC_lexicon_tags_15 },
    { 0x184b37b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772b37be, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x98eb37c1, SUC_lexicon_tags_7 },
    { 0xe2b37c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5cb37c4, SUC_lexicon_tags_5 },
    { 0x45eb37c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe98b37c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a6b37c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x650b37cb, SUC_lexicon_tags_43 },
    { 0x2b4b37cc, SUC_lexicon_tags_43 },
    { 0xf72b37cd, SUC_lexicon_tags_45 },
    { 0xd58b37ce, SUC_lexicon_tags_48 },
    { 0xa8cb37cf, SUC_lexicon_tags_25 },
    { 0x7c4b37cf, SUC_lexicon_tags_35 },
    { 0x63cb37cd, SUC_lexicon_tags_25 },
    { 0x152b37d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ab37dc, SUC_lexicon_tags_7 },
    { 0xdfcb37dd, SUC_lexicon_tags_12 },
    { 0x31eb37dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeb37e1, SUC_lexicon_tags_27 },
    { 0x268b37e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928b37e6, SUC_lexicon_tags_9 },
    { 0x434b37e7, SUC_lexicon_tags_14 },
    { 0x10ab37e8, SUC_lexicon_tags_29 },
    { 0xbbab37e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeb37eb, SUC_lexicon_tags_27 },
    { 0xb3cb37eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2b37ef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0b37f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9deb37f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eb37f9, SUC_lexicon_tags_11 },
    { 0xacab37f9, SUC_lexicon_tags_1 },
    { 0x220b37fa, SUC_lexicon_tags_427 },
    { 0x416b37fc, SUC_lexicon_tags_35 },
    { 0x9b4b37fc, SUC_lexicon_tags_7 },
    { 0xacb37fb, SUC_lexicon_tags_80 },
    { 0x9b2b37ff, SUC_lexicon_tags_226 },
    { 0x374b37fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x324b3802, SUC_lexicon_tags_23 },
    { 0xa2eb3803, SUC_lexicon_tags_60 },
    { 0x49ab3802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ccb3806, SUC_lexicon_tags_2 },
    { 0x376b3807, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6d8b3809, SUC_lexicon_tags_10 },
    { 0x23ab380a, SUC_lexicon_tags_37 },
    { 0xc52b380b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3a6b380d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x232b380f, SUC_lexicon_tags_11 },
    { 0xb3cb380f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fcb3812, SUC_lexicon_tags_9 },
    { 0x450b3813, SUC_lexicon_tags_60 },
    { 0x1aeb3813, SUC_lexicon_tags_11 },
    { 0x538b3814, SUC_lexicon_tags_31 },
    { 0x4e8b3812, SUC_lexicon_tags_47 },
    { 0xc62b3813, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eb381c, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xc90b381e, SUC_lexicon_tags_17 },
    { 0xb4b381f, SUC_lexicon_tags_25 },
    { 0x974b3820, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ab3823, SUC_lexicon_tags_2 },
    { 0x574b3824, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976b3828, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90eb382e, SUC_lexicon_tags_37 },
    { 0x2aab382e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6b3832, SUC_lexicon_tags_17 },
    { 0x8e2b3832, SUC_lexicon_tags_2 },
    { 0x708b3834, SUC_lexicon_tags_12 },
    { 0xb22b3834, SUC_lexicon_tags_8 },
    { 0x970b3835, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0xca0b3838, SUC_lexicon_tags_17 },
    { 0x52cb3839, SUC_lexicon_tags_37 },
    { 0xf46b383a, SUC_lexicon_tags_37 },
    { 0x1f6b383b, SUC_lexicon_tags_34 },
    { 0x4e0b383c, SUC_lexicon_tags_49 },
    { 0xaccb383b, SUC_lexicon_tags_15 },
    { 0xa00b383e, SUC_lexicon_tags_69 },
    { 0x962b383e, SUC_lexicon_tags_45 },
    { 0xa1eb383a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0b3842, SUC_lexicon_tags_37 },
    { 0xf02b3843, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b3847, SUC_lexicon_tags_8 },
    { 0x8eab3847, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d4b384a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18b384e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x224b3851, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xae8b3853, SUC_lexicon_tags_12 },
    { 0xfacb3854, SUC_lexicon_tags_4 },
    { 0x946b3854, SUC_lexicon_tags_11 },
    { 0xf70b3856, SUC_lexicon_tags_70 },
    { 0x2a2b3856, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0eb385a, SUC_lexicon_tags_61 },
    { 0x9bcb385b, SUC_lexicon_tags_35 },
    { 0x3f8b385c, SUC_lexicon_tags_45 },
    { 0xf64b385d, SUC_lexicon_tags_8 },
    { 0x132b385c, SUC_lexicon_tags_25 },
    { 0x656b385f, SUC_lexicon_tags_38 },
    { 0xc86b385d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182b3864, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cb386c, SUC_lexicon_tags_12 },
    { 0x376b386d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cb3871, SUC_lexicon_tags_1 },
    { 0x39ab3872, SUC_lexicon_tags_19 },
    { 0x8b0b3872, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4b3878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8b387c, SUC_lexicon_tags_37 },
    { 0x120b387d, SUC_lexicon_tags_25 },
    { 0x196b387d, SUC_lexicon_tags_34 },
    { 0x908b387e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x896b3881, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x154b3883, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85cb3885, SUC_lexicon_tags_11 },
    { 0x31ab3886, SUC_lexicon_tags_106 },
    { 0x16b3887, SUC_lexicon_tags_141 },
    { 0x358b3887, SUC_lexicon_tags_12 },
    { 0xd2cb3889, SUC_lexicon_tags_4 },
    { 0x556b388a, SUC_lexicon_tags_43 },
    { 0xa6eb388b, SUC_lexicon_tags_16 },
    { 0x44eb388b, SUC_lexicon_tags_25 },
    { 0x504b3888, SUC_lexicon_tags_11 },
    { 0x20b388e, SUC_lexicon_tags_43 },
    { 0x158b388f, SUC_lexicon_tags_37 },
    { 0xbeab3890, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68b3892, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cb3897, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8b389f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ab38a3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710b38a7, SUC_lexicon_tags_1 },
    { 0xdceb38a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xafab38ab, SUC_lexicon_tags_23 },
    { 0x5d4b38ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ab38b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eb38b4, SUC_lexicon_tags_12 },
    { 0x8c6b38b5, SUC_lexicon_tags_17 },
    { 0x346b38b6, SUC_lexicon_tags_11 },
    { 0x542b38b7, SUC_lexicon_tags_179 },
    { 0xd54b38b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8b38be, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0x958b38c1, SUC_lexicon_tags_75 },
    { 0x12eb38c2, SUC_lexicon_tags_33 },
    { 0x10ab38c3, SUC_lexicon_tags_37 },
    { 0xfcab38c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2b38cf, SUC_lexicon_tags_60 },
    { 0xb4eb38cf, SUC_lexicon_tags_37 },
    { 0x418b38d1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30b38dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256b38e6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ab38ec, SUC_lexicon_tags_23 },
    { 0x7c2b38ed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84b38f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3beb38f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbd4b38f4, SUC_lexicon_tags_37 },
    { 0xda4b38f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38b38f9, SUC_lexicon_tags_23 },
    { 0x916b38f9, SUC_lexicon_tags_25 },
    { 0xd30b38fb, SUC_lexicon_tags_56 },
    { 0xa6ab38fa, SUC_lexicon_tags_15 },
    { 0x9a6b38fd, SUC_lexicon_tags_12 },
    { 0xc38b38fd, SUC_lexicon_tags_17 },
    { 0x6e6b38ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524b3905, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x576b3908, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62ab390a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92eb3913, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12b3919, SUC_lexicon_tags_45 },
    { 0xbe4b391a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bab391d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe2b391f, SUC_lexicon_tags_33 },
    { 0x478b3920, SUC_lexicon_tags_39 },
    { 0xe70b3921, SUC_lexicon_tags_14 },
    { 0x9eeb3922, SUC_lexicon_tags_1 },
    { 0x936b3923, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x790b3925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5ab3927, SUC_lexicon_tags_14 },
    { 0xc82b3928, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b392d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x662b3930, SUC_lexicon_tags_12 },
    { 0xe9eb3931, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2fab3933, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06b3937, SUC_lexicon_tags_161 },
    { 0x8d6b3937, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422b394c, SUC_lexicon_tags_60 },
    { 0x122b394c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36b3951, SUC_lexicon_tags_17 },
    { 0x32ab3952, SUC_lexicon_tags_15 },
    { 0xe4b3952, SUC_lexicon_tags_37 },
    { 0x262b3954, SUC_lexicon_tags_19 },
    { 0x712b3955, SUC_lexicon_tags_1 },
    { 0xf2ab3955, SUC_lexicon_tags_17 },
    { 0xa02b3951, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ab3961, SUC_lexicon_tags_7 },
    { 0xee0b3961, SUC_lexicon_tags_32 },
    { 0x914b3963, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4b3967, SUC_lexicon_tags_15 },
    { 0xf98b3968, SUC_lexicon_tags_12 },
    { 0xa86b3968, SUC_lexicon_tags_17 },
    { 0x1f0b3969, SUC_lexicon_tags_14 },
    { 0xe48b396b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362b3975, SUC_lexicon_tags_154 },
    { 0x5d8b3975, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b3979, SUC_lexicon_tags_28 },
    { 0x760b397a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aab3987, SUC_lexicon_tags_38 },
    { 0x2d4b3988, SUC_lexicon_tags_25 },
    { 0x718b3987, SUC_lexicon_tags_17 },
    { 0xea2b3988, SUC_lexicon_tags_32 },
    { 0xc0cb3987, SUC_lexicon_tags_31 },
    { 0x1c0b398c, SUC_lexicon_tags_65 },
    { 0x7c4b398b, SUC_lexicon_tags_14 },
    { 0xe64b3987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cb3991, SUC_lexicon_tags_33 },
    { 0xd54b3991, SUC_lexicon_tags_46 },
    { 0x780b3993, SUC_lexicon_tags_29 },
    { 0x726b3993, SUC_lexicon_tags_38 },
    { 0x6b2b3995, SUC_lexicon_tags_25 },
    { 0x936b3996, SUC_lexicon_tags_14 },
    { 0xa00b3997, SUC_lexicon_tags_15 },
    { 0x8f8b3998, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcb399b, SUC_lexicon_tags_10 },
    { 0xb2ab399b, SUC_lexicon_tags_478 },
    { 0x440b399d, SUC_lexicon_tags_37 },
    { 0xdb4b399e, SUC_lexicon_tags_41 },
    { 0xcf2b399f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8b39a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cb39a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cb39a9, SUC_lexicon_tags_11 },
    { 0xc26b39a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8deb39ad, SUC_lexicon_tags_17 },
    { 0x29eb39ae, SUC_lexicon_tags_87 },
    { 0x4eeb39af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c6b39b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438b39b7, SUC_lexicon_tags_4 },
    { 0xa5eb39b8, SUC_lexicon_tags_25 },
    { 0xb5eb39b8, SUC_lexicon_tags_26 },
    { 0x2fcb39b8, SUC_lexicon_tags_7 },
    { 0xf2ab39bb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76b39bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8b39c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786b39c9, SUC_lexicon_tags_7 },
    { 0x37eb39c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86b39cf, SUC_lexicon_tags_68 },
    { 0x576b39d0, SUC_lexicon_tags_15 },
    { 0x76eb39d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964b39d6, SUC_lexicon_tags_9 },
    { 0x1eab39d7, SUC_lexicon_tags_610 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632b39db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaeb39e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622b39e4, SUC_lexicon_tags_17 },
    { 0x16eb39e5, SUC_lexicon_tags_7 },
    { 0xec4b39e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x94cb39e9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x396b39eb, SUC_lexicon_tags_65 },
    { 0x114b39ec, SUC_lexicon_tags_10 },
    { 0xc20b39ed, SUC_lexicon_tags_19 },
    { 0xf3cb39ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aab39f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0b39f9, SUC_lexicon_tags_2 },
    { 0xb9ab39fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x176b39fd, SUC_lexicon_tags_9 },
    { 0xbab39fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90b3a02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48eb3a04, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cb3a09, SUC_lexicon_tags_1 },
    { 0xad8b3a09, SUC_lexicon_tags_12 },
    { 0x4f4b3a09, SUC_lexicon_tags_11 },
    { 0xc0cb3a0c, SUC_lexicon_tags_25 },
    { 0xbd4b3a09, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdf2b3a0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1eb3a12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b6b3a14, SUC_lexicon_tags_8 },
    { 0xb2b3a15, SUC_lexicon_tags_37 },
    { 0xfdeb3a16, SUC_lexicon_tags_2 },
    { 0x62ab3a16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2b3a1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf70b3a1e, SUC_lexicon_tags_25 },
    { 0x1c0b3a1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530b3a23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94b3a26, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x3eb3a28, SUC_lexicon_tags_23 },
    { 0x68ab3a29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ab3a30, SUC_lexicon_tags_221 },
    { 0xce6b3a31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xca8b3a34, SUC_lexicon_tags_2 },
    { 0xd7eb3a35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c4b3a37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b3a3a, SUC_lexicon_tags_15 },
    { 0xba4b3a3a, SUC_lexicon_tags_0 },
    { 0x804b3a3c, SUC_lexicon_tags_16 },
    { 0x862b3a3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdab3a43, SUC_lexicon_tags_11 },
    { 0xa96b3a44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848b3a4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4b3a4f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b3a53, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98b3a56, SUC_lexicon_tags_16 },
    { 0x7cab3a57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cb3a5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32b3a5e, SUC_lexicon_tags_27 },
    { 0xf40b3a5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ab3a65, SUC_lexicon_tags_37 },
    { 0x4a6b3a65, SUC_lexicon_tags_12 },
    { 0x2a4b3a67, SUC_lexicon_tags_47 },
    { 0x6e0b3a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2cab3a6a, SUC_lexicon_tags_15 },
    { 0xbb0b3a6b, SUC_lexicon_tags_37 },
    { 0x554b3a6c, SUC_lexicon_tags_15 },
    { 0x3aeb3a6c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4b3a70, SUC_lexicon_tags_12 },
    { 0x71eb3a71, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8b3a75, SUC_lexicon_tags_17 },
    { 0x53cb3a75, SUC_lexicon_tags_15 },
    { 0x7d2b3a76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x416b3a79, SUC_lexicon_tags_10 },
    { 0x93eb3a7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332b3a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b3a83, SUC_lexicon_tags_60 },
    { 0x6e0b3a84, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x43cb3a86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462b3a8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cb3a93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ab3a97, SUC_lexicon_tags_3 },
    { 0x440b3a98, SUC_lexicon_tags_23 },
    { 0x152b3a99, SUC_lexicon_tags_1 },
    { 0x54cb3a97, SUC_lexicon_tags_33 },
    { 0x778b3a99, SUC_lexicon_tags_17 },
    { 0x57cb3a9c, SUC_lexicon_tags_14 },
    { 0x24b3a9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x534b3a9f, SUC_lexicon_tags_37 },
    { 0xf64b3aa0, SUC_lexicon_tags_63 },
    { 0xd4ab3aa0, SUC_lexicon_tags_14 },
    { 0xc76b3aa0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410b3aa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48eb3aa9, SUC_lexicon_tags_47 },
    { 0x754b3aaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x926b3aac, SUC_lexicon_tags_12 },
    { 0x4a2b3aad, SUC_lexicon_tags_0 },
    { 0xd70b3aae, SUC_lexicon_tags_25 },
    { 0x5beb3aaf, SUC_lexicon_tags_1 },
    { 0x1c2b3aaf, SUC_lexicon_tags_25 },
    { 0x6f6b3ab1, SUC_lexicon_tags_9 },
    { 0x60ab3ab2, SUC_lexicon_tags_66 },
    { 0xee4b3aac, SUC_lexicon_tags_2 },
    { 0x496b3ab4, SUC_lexicon_tags_0 },
    { 0x4ecb3aaf, SUC_lexicon_tags_10 },
    { 0x4a2b3ab3, SUC_lexicon_tags_55 },
    { 0xd4eb3ab7, SUC_lexicon_tags_7 },
    { 0xbaab3ab7, SUC_lexicon_tags_31 },
    { 0x83ab3ab9, SUC_lexicon_tags_1 },
    { 0x9e4b3aba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4b3abd, SUC_lexicon_tags_12 },
    { 0xb3ab3abe, SUC_lexicon_tags_11 },
    { 0x740b3abf, SUC_lexicon_tags_37 },
    { 0xa58b3abd, SUC_lexicon_tags_37 },
    { 0x5cb3ac1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eb3ac6, SUC_lexicon_tags_37 },
    { 0x35ab3ac6, SUC_lexicon_tags_25 },
    { 0x82eb3ac6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x226b3acb, SUC_lexicon_tags_47 },
    { 0xc88b3acc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x516b3ace, SUC_lexicon_tags_2 },
    { 0xd98b3acf, SUC_lexicon_tags_1 },
    { 0x1b8b3ad0, SUC_lexicon_tags_30 },
    { 0x4e6b3ad0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8b3ad4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd0b3ad6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc02b3ad8, SUC_lexicon_tags_12 },
    { 0xaceb3ad9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab3adf, SUC_lexicon_tags_11 },
    { 0xd20b3ae0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb3ae3, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4b3ae8, SUC_lexicon_tags_6 },
    { 0x4fab3ae9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcb3af1, SUC_lexicon_tags_43 },
    { 0xf20b3af2, SUC_lexicon_tags_33 },
    { 0x59eb3af3, SUC_lexicon_tags_15 },
    { 0xf34b3af4, SUC_lexicon_tags_1 },
    { 0x36eb3af5, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x906b3af7, SUC_lexicon_tags_15 },
    { 0xe4b3af8, SUC_lexicon_tags_30 },
    { 0xe3eb3af8, SUC_lexicon_tags_63 },
    { 0xfccb3afa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28cb3afc, SUC_lexicon_tags_37 },
    { 0x3b4b3afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b3b00, SUC_lexicon_tags_0 },
    { 0x54eb3b01, SUC_lexicon_tags_32 },
    { 0x860b3b02, SUC_lexicon_tags_32 },
    { 0xa5ab3b02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b3b07, SUC_lexicon_tags_25 },
    { 0x6a8b3b07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46b3b0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x754b3b0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x656b3b10, SUC_lexicon_tags_0 },
    { 0x14ab3b10, SUC_lexicon_tags_12 },
    { 0xb4eb3b12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb3b15, SUC_lexicon_tags_1 },
    { 0x580b3b16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x17eb3b19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70b3b1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290b3b22, SUC_lexicon_tags_25 },
    { 0x654b3b23, SUC_lexicon_tags_37 },
    { 0xa12b3b24, SUC_lexicon_tags_14 },
    { 0xe98b3b24, SUC_lexicon_tags_6 },
    { 0x5b4b3b23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60b3b28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436b3b2c, SUC_lexicon_tags_28 },
    { 0xba8b3b2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecb3b31, SUC_lexicon_tags_644 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cb3b34, SUC_lexicon_tags_23 },
    { 0x14ab3b35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a0b3b37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa50b3b39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x338b3b3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ab3b3f, SUC_lexicon_tags_25 },
    { 0x96cb3b40, SUC_lexicon_tags_6 },
    { 0x996b3b41, SUC_lexicon_tags_13 },
    { 0x67cb3b42, SUC_lexicon_tags_12 },
    { 0xa7ab3b43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbdab3b45, SUC_lexicon_tags_46 },
    { 0x66eb3b45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0b3b4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ab3b4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf96b3b51, SUC_lexicon_tags_37 },
    { 0x5beb3b52, SUC_lexicon_tags_12 },
    { 0x8f2b3b52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eb3b56, SUC_lexicon_tags_47 },
    { 0xaf8b3b57, SUC_lexicon_tags_28 },
    { 0x824b3b58, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceeb3b5c, SUC_lexicon_tags_12 },
    { 0xaa8b3b5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cb3b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88b3b69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cb3b70, SUC_lexicon_tags_1 },
    { 0x72eb3b70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6f4b3b73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x718b3b75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacb3b7c, SUC_lexicon_tags_11 },
    { 0x47ab3b7c, SUC_lexicon_tags_7 },
    { 0x6ccb3b7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ab3b81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x896b3b84, SUC_lexicon_tags_0 },
    { 0xa8b3b84, SUC_lexicon_tags_56 },
    { 0xd5ab3b86, SUC_lexicon_tags_55 },
    { 0xfceb3b87, SUC_lexicon_tags_38 },
    { 0x162b3b86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89cb3b8a, SUC_lexicon_tags_23 },
    { 0x846b3b8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53ab3b8d, SUC_lexicon_tags_47 },
    { 0xcbab3b8e, SUC_lexicon_tags_12 },
    { 0x2fcb3b8e, SUC_lexicon_tags_25 },
    { 0xd74b3b90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76b3b93, SUC_lexicon_tags_26 },
    { 0x560b3b94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456b3ba0, SUC_lexicon_tags_11 },
    { 0x4f8b3ba1, SUC_lexicon_tags_30 },
    { 0xb46b3ba1, SUC_lexicon_tags_0 },
    { 0x4e8b3ba0, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00b3ba9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aeb3bb1, SUC_lexicon_tags_45 },
    { 0x98ab3bb1, SUC_lexicon_tags_37 },
    { 0x758b3bb2, SUC_lexicon_tags_45 },
    { 0xe6cb3bb3, SUC_lexicon_tags_49 },
    { 0x606b3bb5, SUC_lexicon_tags_45 },
    { 0x520b3bb2, SUC_lexicon_tags_25 },
    { 0x8b6b3bb5, SUC_lexicon_tags_17 },
    { 0xf0b3bb3, SUC_lexicon_tags_1 },
    { 0xcaab3bb9, SUC_lexicon_tags_29 },
    { 0x3eb3bba, SUC_lexicon_tags_1 },
    { 0x564b3bb2, SUC_lexicon_tags_9 },
    { 0x3c2b3bbc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36b3bbf, SUC_lexicon_tags_29 },
    { 0xf54b3bbf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x910b3bc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30b3bc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c4b3bc8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x93cb3bca, SUC_lexicon_tags_46 },
    { 0xc04b3bcb, SUC_lexicon_tags_49 },
    { 0xb66b3bcb, SUC_lexicon_tags_15 },
    { 0x596b3bca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cb3bd1, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceab3bd7, SUC_lexicon_tags_13 },
    { 0xa72b3bd7, SUC_lexicon_tags_37 },
    { 0xcc4b3bd9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86b3bdd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb3be1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cb3be7, SUC_lexicon_tags_15 },
    { 0x220b3be8, SUC_lexicon_tags_26 },
    { 0x102b3be8, SUC_lexicon_tags_65 },
    { 0xa96b3be7, SUC_lexicon_tags_47 },
    { 0x73eb3beb, SUC_lexicon_tags_68 },
    { 0x54b3bec, SUC_lexicon_tags_33 },
    { 0x644b3beb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1eb3bef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb3bf3, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cb3bfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab3c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x100b3c0a, SUC_lexicon_tags_40 },
    { 0xdc8b3c0a, SUC_lexicon_tags_11 },
    { 0x9c2b3c0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8b3c0f, SUC_lexicon_tags_47 },
    { 0xa58b3c10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26b3c14, SUC_lexicon_tags_36 },
    { 0xae4b3c15, SUC_lexicon_tags_11 },
    { 0x788b3c16, SUC_lexicon_tags_35 },
    { 0x1a2b3c16, SUC_lexicon_tags_25 },
    { 0xc08b3c15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94eb3c1a, SUC_lexicon_tags_15 },
    { 0xa50b3c1a, SUC_lexicon_tags_12 },
    { 0x334b3c1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x772b3c1e, SUC_lexicon_tags_38 },
    { 0x8c8b3c1f, SUC_lexicon_tags_49 },
    { 0x80b3c1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ab3c28, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71eb3c2c, SUC_lexicon_tags_138 },
    { 0x36eb3c2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2b3c31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe94b3c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccb3c36, SUC_lexicon_tags_7 },
    { 0x6f4b3c36, SUC_lexicon_tags_2 },
    { 0xe6b3c38, SUC_lexicon_tags_11 },
    { 0x3f2b3c36, SUC_lexicon_tags_39 },
    { 0x8a0b3c3a, SUC_lexicon_tags_17 },
    { 0xd2cb3c37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcb3c3e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xda6b3c40, SUC_lexicon_tags_11 },
    { 0xf7ab3c41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff8b3c43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cb3c48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xceb3c4a, SUC_lexicon_tags_5 },
    { 0xc40b3c4b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6b3c4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabab3c51, SUC_lexicon_tags_11 },
    { 0xeab3c51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb10b3c54, SUC_lexicon_tags_27 },
    { 0xba6b3c55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb54b3c57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cb3c5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52b3c68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54b3c6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46b3c77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04b3c80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580b3c85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0b3c88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x286b3c8a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x88b3c8c, SUC_lexicon_tags_37 },
    { 0x986b3c8d, SUC_lexicon_tags_21 },
    { 0x1ccb3c8e, SUC_lexicon_tags_37 },
    { 0x8beb3c8d, SUC_lexicon_tags_30 },
    { 0x822b3c90, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6b3c93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b6b3c95, SUC_lexicon_tags_65 },
    { 0xd4ab3c96, SUC_lexicon_tags_47 },
    { 0x7bab3c96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10eb3c9b, SUC_lexicon_tags_37 },
    { 0xe30b3c9c, SUC_lexicon_tags_17 },
    { 0xed8b3c9b, SUC_lexicon_tags_17 },
    { 0x466b3c9c, SUC_lexicon_tags_47 },
    { 0x8cab3c9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2b3ca4, SUC_lexicon_tags_37 },
    { 0xc8eb3ca5, SUC_lexicon_tags_24 },
    { 0x13cb3ca5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x112b3ca8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab3cac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4cb3cae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70b3cb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726b3cba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4b3cbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72b3cc4, SUC_lexicon_tags_45 },
    { 0x6e0b3cc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24b3cc8, SUC_lexicon_tags_11 },
    { 0xa70b3cc9, SUC_lexicon_tags_17 },
    { 0x570b3cca, SUC_lexicon_tags_47 },
    { 0x14cb3ccb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3feb3ccf, SUC_lexicon_tags_0 },
    { 0x4b8b3ccf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa76b3cd2, SUC_lexicon_tags_0 },
    { 0x150b3cd3, SUC_lexicon_tags_49 },
    { 0x12eb3cd2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xed8b3cd6, SUC_lexicon_tags_5 },
    { 0xe34b3cd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba4b3cd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcb3cdc, SUC_lexicon_tags_16 },
    { 0xd86b3cdd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908b3ce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8b3cec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2b3cf2, SUC_lexicon_tags_11 },
    { 0xf8ab3cf3, SUC_lexicon_tags_2 },
    { 0xc4b3cf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2b3cf8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc22b3cfa, SUC_lexicon_tags_66 },
    { 0xc60b3cfb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc5cb3cfd, SUC_lexicon_tags_77 },
    { 0x36ab3cfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0b3d01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82b3d09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb46b3d0b, SUC_lexicon_tags_26 },
    { 0x2e6b3d0c, SUC_lexicon_tags_8 },
    { 0xa08b3d0c, SUC_lexicon_tags_31 },
    { 0x56ab3d0e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bab3d12, SUC_lexicon_tags_26 },
    { 0xf20b3d13, SUC_lexicon_tags_47 },
    { 0xc62b3d14, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6f8b3d16, SUC_lexicon_tags_28 },
    { 0x440b3d17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x996b3d19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928b3d1d, SUC_lexicon_tags_23 },
    { 0x92ab3d1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x330b3d20, SUC_lexicon_tags_12 },
    { 0x59cb3d21, SUC_lexicon_tags_49 },
    { 0x410b3d22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4b3d25, SUC_lexicon_tags_33 },
    { 0xf64b3d25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884b3d2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acb3d34, SUC_lexicon_tags_27 },
    { 0x66eb3d35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab3d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8b3d3e, SUC_lexicon_tags_8 },
    { 0xad8b3d3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cb3d42, SUC_lexicon_tags_17 },
    { 0x3d2b3d43, SUC_lexicon_tags_25 },
    { 0x5dab3d44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3d4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61eb3d51, SUC_lexicon_tags_11 },
    { 0x5eab3d51, SUC_lexicon_tags_17 },
    { 0x19cb3d53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ceb3d55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd80b3d57, SUC_lexicon_tags_2 },
    { 0xb7cb3d57, SUC_lexicon_tags_37 },
    { 0x5feb3d58, SUC_lexicon_tags_31 },
    { 0xf5cb3d5a, SUC_lexicon_tags_83 },
    { 0xccab3d5a, SUC_lexicon_tags_45 },
    { 0x4bab3d5a, SUC_lexicon_tags_39 },
    { 0x86b3d5d, SUC_lexicon_tags_17 },
    { 0xaaeb3d5b, SUC_lexicon_tags_25 },
    { 0xb38b3d5a, SUC_lexicon_tags_29 },
    { 0x1eb3d59, SUC_lexicon_tags_118 },
    { 0x82ab3d61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ab3d64, SUC_lexicon_tags_2 },
    { 0xc36b3d64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb3d69, SUC_lexicon_tags_121 },
    { 0x728b3d69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b3d6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896b3d75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab3d7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5aab3d7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99cb3d80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ab3d83, SUC_lexicon_tags_39 },
    { 0xda8b3d83, SUC_lexicon_tags_25 },
    { 0x754b3d85, SUC_lexicon_tags_17 },
    { 0xf16b3d86, SUC_lexicon_tags_8 },
    { 0x92cb3d86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xafeb3d89, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9aeb3d8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ab3d8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xabeb3d91, SUC_lexicon_tags_17 },
    { 0x246b3d92, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2b3d95, SUC_lexicon_tags_25 },
    { 0x524b3d96, SUC_lexicon_tags_13 },
    { 0x4c6b3d97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4b3d9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb3da0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x300b3da2, SUC_lexicon_tags_19 },
    { 0x266b3da3, SUC_lexicon_tags_17 },
    { 0xaab3da3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdab3daa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eab3dad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230b3db1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cb3db6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6a0b3db8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1feb3dbb, SUC_lexicon_tags_1 },
    { 0x344b3dbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46b3dbf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cb3dc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b3dc7, SUC_lexicon_tags_48 },
    { 0x686b3dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b3dcb, SUC_lexicon_tags_7 },
    { 0x3b6b3dcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3dd1, SUC_lexicon_tags_37 },
    { 0x1c6b3dd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cb3dd5, SUC_lexicon_tags_53 },
    { 0x9f0b3dd6, SUC_lexicon_tags_12 },
    { 0x6c6b3dd6, SUC_lexicon_tags_17 },
    { 0x246b3dd6, SUC_lexicon_tags_45 },
    { 0xed8b3dd9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8b3ddd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x746b3ddf, SUC_lexicon_tags_36 },
    { 0xdeab3de0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x850b3de3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640b3de9, SUC_lexicon_tags_11 },
    { 0x5f8b3de9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadab3dee, SUC_lexicon_tags_7 },
    { 0x770b3def, SUC_lexicon_tags_2 },
    { 0x946b3df0, SUC_lexicon_tags_15 },
    { 0x7c2b3def, SUC_lexicon_tags_11 },
    { 0xeaab3df2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8bcb3df4, SUC_lexicon_tags_38 },
    { 0x9bab3df4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910b3df9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13eb3dfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42b3dfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcb3e06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8b3e0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x818b3e0f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926b3e1a, SUC_lexicon_tags_15 },
    { 0x25eb3e1b, SUC_lexicon_tags_37 },
    { 0xaa4b3e1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102b3e21, SUC_lexicon_tags_40 },
    { 0x522b3e22, SUC_lexicon_tags_30 },
    { 0x154b3e23, SUC_lexicon_tags_17 },
    { 0xa1eb3e24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50b3e2b, SUC_lexicon_tags_28 },
    { 0xbcab3e2c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x88eb3e2f, SUC_lexicon_tags_68 },
    { 0xcf4b3e30, SUC_lexicon_tags_0 },
    { 0xe0ab3e31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6b3e35, SUC_lexicon_tags_15 },
    { 0x7feb3e35, SUC_lexicon_tags_65 },
    { 0xfd6b3e37, SUC_lexicon_tags_47 },
    { 0xafcb3e38, SUC_lexicon_tags_19 },
    { 0x6b0b3e39, SUC_lexicon_tags_48 },
    { 0x7cb3e3a, SUC_lexicon_tags_1 },
    { 0x43ab3e38, SUC_lexicon_tags_25 },
    { 0xe3ab3e3b, SUC_lexicon_tags_12 },
    { 0x98ab3e35, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0b3e42, SUC_lexicon_tags_1 },
    { 0x696b3e43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0b3e49, SUC_lexicon_tags_2 },
    { 0xf82b3e49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26b3e4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x734b3e4f, SUC_lexicon_tags_11 },
    { 0xeccb3e50, SUC_lexicon_tags_9 },
    { 0xe2cb3e50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb3e54, SUC_lexicon_tags_6 },
    { 0x49eb3e55, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228b3e5a, SUC_lexicon_tags_12 },
    { 0x512b3e5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dab3e6a, SUC_lexicon_tags_7 },
    { 0x310b3e6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x644b3e6d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b3e75, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaab3e78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcb3e7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b3e80, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2b3e83, SUC_lexicon_tags_15 },
    { 0x7a6b3e84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0b3e87, SUC_lexicon_tags_8 },
    { 0xa68b3e88, SUC_lexicon_tags_37 },
    { 0x7cab3e89, SUC_lexicon_tags_47 },
    { 0x7ab3e8a, SUC_lexicon_tags_38 },
    { 0x61ab3e89, SUC_lexicon_tags_14 },
    { 0xc84b3e8c, SUC_lexicon_tags_17 },
    { 0x676b3e8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbd4b3e8f, SUC_lexicon_tags_15 },
    { 0xdab3e8f, SUC_lexicon_tags_15 },
    { 0xe02b3e91, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x17eb3e94, SUC_lexicon_tags_11 },
    { 0xa4eb3e95, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x966b3e97, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x2a2b3e99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2b3e9d, SUC_lexicon_tags_29 },
    { 0xc4b3e9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x280b3ea1, SUC_lexicon_tags_5 },
    { 0xd9eb3ea1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd1ab3ea4, SUC_lexicon_tags_29 },
    { 0x638b3ea4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858b3ea9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x96ab3eab, SUC_lexicon_tags_25 },
    { 0x334b3eac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x706b3eaf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacab3eb7, SUC_lexicon_tags_49 },
    { 0xbd0b3eb7, SUC_lexicon_tags_37 },
    { 0xe88b3eb9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6b3ebe, SUC_lexicon_tags_11 },
    { 0x30cb3ebf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb08b3ec1, SUC_lexicon_tags_37 },
    { 0xbcab3ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2b3ec6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4b3eca, SUC_lexicon_tags_61 },
    { 0xd8b3ecb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc46b3ecd, SUC_lexicon_tags_1 },
    { 0x788b3ece, SUC_lexicon_tags_15 },
    { 0x85cb3ecd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234b3ed5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92b3ed8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd2b3eda, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ceb3edf, SUC_lexicon_tags_38 },
    { 0x6d6b3edf, SUC_lexicon_tags_62 },
    { 0x8a2b3ee1, SUC_lexicon_tags_11 },
    { 0xcf4b3ee0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaf2b3ee4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66b3ee8, SUC_lexicon_tags_10 },
    { 0x460b3ee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7dab3eeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6b3ef0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b2b3ef2, SUC_lexicon_tags_43 },
    { 0x86b3ef3, SUC_lexicon_tags_151 },
    { 0xaf0b3ef3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696b3efb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x882b3efd, SUC_lexicon_tags_15 },
    { 0xa82b3efd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf0b3f00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6b3f05, SUC_lexicon_tags_2 },
    { 0xf3cb3f06, SUC_lexicon_tags_12 },
    { 0xbbeb3f07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84b3f0c, SUC_lexicon_tags_11 },
    { 0xe3eb3f0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x272b3f0f, SUC_lexicon_tags_0 },
    { 0xd7cb3f10, SUC_lexicon_tags_11 },
    { 0xcb2b3f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74b3f14, SUC_lexicon_tags_25 },
    { 0xa8b3f14, SUC_lexicon_tags_117 },
    { 0xf2b3f16, SUC_lexicon_tags_0 },
    { 0xaf2b3f17, SUC_lexicon_tags_17 },
    { 0xb40b3f14, SUC_lexicon_tags_37 },
    { 0x88eb3f19, SUC_lexicon_tags_14 },
    { 0xd40b3f14, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6b3f1d, SUC_lexicon_tags_10 },
    { 0x552b3f1e, SUC_lexicon_tags_17 },
    { 0xefcb3f1f, SUC_lexicon_tags_2 },
    { 0xa7ab3f1f, SUC_lexicon_tags_14 },
    { 0xd5eb3f1d, SUC_lexicon_tags_17 },
    { 0xcf6b3f1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf76b3f24, SUC_lexicon_tags_15 },
    { 0x3f8b3f25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8b3f2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2b3f2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcb3f31, SUC_lexicon_tags_25 },
    { 0xeb0b3f31, SUC_lexicon_tags_11 },
    { 0x5b0b3f33, SUC_lexicon_tags_45 },
    { 0x3f6b3f33, SUC_lexicon_tags_25 },
    { 0xa94b3f33, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cb3f3a, SUC_lexicon_tags_15 },
    { 0x560b3f3a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1eb3f3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f2b3f40, SUC_lexicon_tags_49 },
    { 0x5beb3f41, SUC_lexicon_tags_87 },
    { 0x3c4b3f42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4b3f44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2eb3f46, SUC_lexicon_tags_11 },
    { 0xb6cb3f46, SUC_lexicon_tags_63 },
    { 0x25eb3f48, SUC_lexicon_tags_33 },
    { 0xa50b3f49, SUC_lexicon_tags_11 },
    { 0x7deb3f46, SUC_lexicon_tags_60 },
    { 0xfbeb3f4b, SUC_lexicon_tags_455 },
    { 0x2d8b3f4a, SUC_lexicon_tags_37 },
    { 0xcc8b3f4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f6b3f4f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2b3f52, SUC_lexicon_tags_35 },
    { 0xa0eb3f53, SUC_lexicon_tags_12 },
    { 0x3b6b3f54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e6b3f56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36b3f58, SUC_lexicon_tags_4 },
    { 0xfdeb3f59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc8eb3f5b, SUC_lexicon_tags_11 },
    { 0xb8eb3f5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290b3f60, SUC_lexicon_tags_30 },
    { 0x528b3f60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4ab3f63, SUC_lexicon_tags_40 },
    { 0x1f0b3f63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x108b3f67, SUC_lexicon_tags_14 },
    { 0x11cb3f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cb3f6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37eb3f78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x134b3f7b, SUC_lexicon_tags_7 },
    { 0x6beb3f7c, SUC_lexicon_tags_34 },
    { 0x304b3f7d, SUC_lexicon_tags_15 },
    { 0xc94b3f7e, SUC_lexicon_tags_11 },
    { 0x49eb3f7f, SUC_lexicon_tags_91 },
    { 0xc0cb3f80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176b3f88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc6ab3f8a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ab3f8d, SUC_lexicon_tags_18 },
    { 0x584b3f8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x774b3f90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f0b3f92, SUC_lexicon_tags_15 },
    { 0xa3cb3f93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x242b3f95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2b3f99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86b3fa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4b3fa3, SUC_lexicon_tags_49 },
    { 0xf68b3fa4, SUC_lexicon_tags_27 },
    { 0x90ab3fa3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1e6b3fa7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x89cb3fa9, SUC_lexicon_tags_17 },
    { 0xbecb3fa9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x552b3fad, SUC_lexicon_tags_113 },
    { 0x91ab3fad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2bab3fb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x304b3fb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8b3fbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x598b3fbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4b3fc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x602b3fc2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12eb3fc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6b3fcb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8cb3fcd, SUC_lexicon_tags_8 },
    { 0x63cb3fce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b3fd5, SUC_lexicon_tags_45 },
    { 0x8c4b3fd6, SUC_lexicon_tags_201 },
    { 0x894b3fd6, SUC_lexicon_tags_8 },
    { 0x3a6b3fd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4b3fde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x396b3fe1, SUC_lexicon_tags_1 },
    { 0x5beb3fe2, SUC_lexicon_tags_15 },
    { 0x1ccb3fe3, SUC_lexicon_tags_29 },
    { 0xc7ab3fe4, SUC_lexicon_tags_49 },
    { 0xe74b3fe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcb3fe9, SUC_lexicon_tags_37 },
    { 0x934b3fea, SUC_lexicon_tags_37 },
    { 0x10b3feb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292b3ff0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x108b3ff3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14b3ff6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00b3ff9, SUC_lexicon_tags_30 },
    { 0x8d6b3ffa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2e6b3ffc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaceb4009, SUC_lexicon_tags_85 },
    { 0x85cb4009, SUC_lexicon_tags_23 },
    { 0x43cb400b, SUC_lexicon_tags_21 },
    { 0x5f2b400a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x368b400f, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8b4012, SUC_lexicon_tags_33 },
    { 0x20eb4013, SUC_lexicon_tags_14 },
    { 0x6a8b4013, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x508b4016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b0b4018, SUC_lexicon_tags_8 },
    { 0x7bab4019, SUC_lexicon_tags_37 },
    { 0x856b4019, SUC_lexicon_tags_7 },
    { 0xe4ab401b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x748b401d, SUC_lexicon_tags_37 },
    { 0xc5cb401e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf74b4020, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86b4024, SUC_lexicon_tags_11 },
    { 0x474b4025, SUC_lexicon_tags_0 },
    { 0xfd2b4024, SUC_lexicon_tags_37 },
    { 0x1aeb4027, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16b402b, SUC_lexicon_tags_9 },
    { 0x1f2b402c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55ab402e, SUC_lexicon_tags_6 },
    { 0x9f4b402f, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x382b4031, SUC_lexicon_tags_34 },
    { 0xa6ab4032, SUC_lexicon_tags_35 },
    { 0x8e6b4033, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20b4036, SUC_lexicon_tags_63 },
    { 0x826b4036, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f0b4039, SUC_lexicon_tags_56 },
    { 0xb64b403a, SUC_lexicon_tags_151 },
    { 0xe34b403b, SUC_lexicon_tags_11 },
    { 0xf4cb403c, SUC_lexicon_tags_396 },
    { 0xaaeb403d, SUC_lexicon_tags_46 },
    { 0xe40b403a, SUC_lexicon_tags_7 },
    { 0x93cb403f, SUC_lexicon_tags_26 },
    { 0xd90b4040, SUC_lexicon_tags_0 },
    { 0xcd0b403c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55ab4043, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ab4047, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1f8b4049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684b404e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38b4052, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa60b4055, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672b405f, SUC_lexicon_tags_7 },
    { 0x882b405f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x978b4062, SUC_lexicon_tags_21 },
    { 0x304b4063, SUC_lexicon_tags_12 },
    { 0x63ab4064, SUC_lexicon_tags_48 },
    { 0xc9ab4065, SUC_lexicon_tags_28 },
    { 0x490b4066, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceb4069, SUC_lexicon_tags_116 },
    { 0x5eeb406a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406b4074, SUC_lexicon_tags_6 },
    { 0xf7ab4075, SUC_lexicon_tags_26 },
    { 0x4f6b4076, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb407a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa66b407c, SUC_lexicon_tags_25 },
    { 0xdfcb407c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ab4083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286b4089, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ab408c, SUC_lexicon_tags_12 },
    { 0xbfeb408d, SUC_lexicon_tags_7 },
    { 0x696b408d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10ab4090, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ab4095, SUC_lexicon_tags_49 },
    { 0xb12b4095, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204b409a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ab409d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dab40a0, SUC_lexicon_tags_35 },
    { 0x740b40a1, SUC_lexicon_tags_15 },
    { 0xf04b40a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06b40a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24b40a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x258b40ac, SUC_lexicon_tags_28 },
    { 0x49eb40ad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aeb40b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b40b9, SUC_lexicon_tags_6 },
    { 0xa5eb40b9, SUC_lexicon_tags_8 },
    { 0x3e6b40b9, SUC_lexicon_tags_15 },
    { 0xea8b40b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26b40c2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb22b40c4, SUC_lexicon_tags_65 },
    { 0x828b40c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2b40cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90b40cd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x2b0b40cf, SUC_lexicon_tags_26 },
    { 0x266b40d0, SUC_lexicon_tags_45 },
    { 0x85ab40cf, SUC_lexicon_tags_25 },
    { 0x7cb40d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x232b40d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cb40d8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8b40df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cb40e3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd22b40e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe8b40e7, SUC_lexicon_tags_65 },
    { 0x846b40e8, SUC_lexicon_tags_25 },
    { 0xd4b40e7, SUC_lexicon_tags_15 },
    { 0x5d0b40ea, SUC_lexicon_tags_6 },
    { 0x544b40eb, SUC_lexicon_tags_1 },
    { 0x8eab40ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eb40ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198b40f8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x30b40fa, SUC_lexicon_tags_26 },
    { 0xf08b40fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x158b40fe, SUC_lexicon_tags_21 },
    { 0xc4eb40ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89eb4103, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ab4108, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36eb410a, SUC_lexicon_tags_12 },
    { 0xe9eb410b, SUC_lexicon_tags_45 },
    { 0x8c4b410c, SUC_lexicon_tags_23 },
    { 0x9f0b410d, SUC_lexicon_tags_43 },
    { 0x1a6b410c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7aab4110, SUC_lexicon_tags_14 },
    { 0x340b4111, SUC_lexicon_tags_25 },
    { 0x85ab4111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfeab4114, SUC_lexicon_tags_60 },
    { 0x4a6b4114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960b411b, SUC_lexicon_tags_63 },
    { 0x7fab411c, SUC_lexicon_tags_59 },
    { 0x360b411d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x846b411f, SUC_lexicon_tags_6 },
    { 0x294b4120, SUC_lexicon_tags_43 },
    { 0xea0b4120, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310b4129, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bab412c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64b4131, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b4134, SUC_lexicon_tags_1 },
    { 0x796b4135, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4b4139, SUC_lexicon_tags_4 },
    { 0x2acb4139, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xac0b413c, SUC_lexicon_tags_2 },
    { 0x67cb413d, SUC_lexicon_tags_37 },
    { 0x4a4b413e, SUC_lexicon_tags_17 },
    { 0xebab413f, SUC_lexicon_tags_6 },
    { 0x9ceb4140, SUC_lexicon_tags_14 },
    { 0x3b6b413f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6b4146, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676b414e, SUC_lexicon_tags_12 },
    { 0xf28b414f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecb4156, SUC_lexicon_tags_7 },
    { 0x5aeb4156, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x28cb4159, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3eb415c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88b4160, SUC_lexicon_tags_37 },
    { 0x9d8b4160, SUC_lexicon_tags_25 },
    { 0x1cb4161, SUC_lexicon_tags_9 },
    { 0x24b4160, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88eb416a, SUC_lexicon_tags_25 },
    { 0xbcb416a, SUC_lexicon_tags_87 },
    { 0x340b416c, SUC_lexicon_tags_11 },
    { 0x91eb416d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x400b416f, SUC_lexicon_tags_74 },
    { 0x150b4170, SUC_lexicon_tags_15 },
    { 0xac6b4171, SUC_lexicon_tags_14 },
    { 0xc44b4172, SUC_lexicon_tags_7 },
    { 0xac4b4173, SUC_lexicon_tags_8 },
    { 0x6cab4174, SUC_lexicon_tags_84 },
    { 0x692b4175, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2b4177, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800b417c, SUC_lexicon_tags_14 },
    { 0xf9eb417d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0b4184, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80b4187, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x17ab4189, SUC_lexicon_tags_15 },
    { 0x52ab4189, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb418d, SUC_lexicon_tags_7 },
    { 0xc92b418d, SUC_lexicon_tags_45 },
    { 0x136b418f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6b4196, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf6eb4198, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00b419e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x492b41a1, SUC_lexicon_tags_37 },
    { 0x46b41a2, SUC_lexicon_tags_29 },
    { 0xca0b41a3, SUC_lexicon_tags_7 },
    { 0x114b41a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd68b41a6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xe70b41a8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696b41ad, SUC_lexicon_tags_12 },
    { 0xa1eb41ad, SUC_lexicon_tags_216 },
    { 0x15eb41af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb08b41b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0b41b4, SUC_lexicon_tags_106 },
    { 0x3d6b41b5, SUC_lexicon_tags_15 },
    { 0xbf4b41b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x53cb41b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x596b41ba, SUC_lexicon_tags_11 },
    { 0x432b41bb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4b41be, SUC_lexicon_tags_13 },
    { 0x990b41bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6b41c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8b41c5, SUC_lexicon_tags_63 },
    { 0x520b41c5, SUC_lexicon_tags_37 },
    { 0x3f6b41c6, SUC_lexicon_tags_11 },
    { 0xd5eb41c8, SUC_lexicon_tags_12 },
    { 0x90eb41c8, SUC_lexicon_tags_276 },
    { 0xc48b41ca, SUC_lexicon_tags_57 },
    { 0xe18b41cb, SUC_lexicon_tags_17 },
    { 0xb2eb41cb, SUC_lexicon_tags_14 },
    { 0x23ab41c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5fcb41cf, SUC_lexicon_tags_1 },
    { 0x182b41d0, SUC_lexicon_tags_30 },
    { 0x712b41d1, SUC_lexicon_tags_12 },
    { 0x986b41d1, SUC_lexicon_tags_27 },
    { 0xbb6b41d3, SUC_lexicon_tags_43 },
    { 0xceb41d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabab41da, SUC_lexicon_tags_13 },
    { 0x8eeb41da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x458b41dd, SUC_lexicon_tags_37 },
    { 0xeccb41de, SUC_lexicon_tags_8 },
    { 0x530b41df, SUC_lexicon_tags_15 },
    { 0xf56b41dd, SUC_lexicon_tags_15 },
    { 0x52ab41dd, SUC_lexicon_tags_7 },
    { 0xde0b41de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb41e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb6b41e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4b41ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302b41ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcb41f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4b41f6, SUC_lexicon_tags_25 },
    { 0xc56b41f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x956b41f9, SUC_lexicon_tags_17 },
    { 0xb58b41f9, SUC_lexicon_tags_38 },
    { 0x828b41f9, SUC_lexicon_tags_17 },
    { 0x90b41fc, SUC_lexicon_tags_32 },
    { 0x1c4b41fd, SUC_lexicon_tags_25 },
    { 0x4b41fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x43cb4200, SUC_lexicon_tags_23 },
    { 0x40eb4201, SUC_lexicon_tags_60 },
    { 0xf40b4200, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146b4206, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7eb4208, SUC_lexicon_tags_37 },
    { 0x28b4208, SUC_lexicon_tags_12 },
    { 0xf4eb420a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1eb420f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74eb4213, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4b4216, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36b421a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88b4220, SUC_lexicon_tags_26 },
    { 0xe9ab4220, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf26b4223, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cb4227, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4b422c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44b422f, SUC_lexicon_tags_2 },
    { 0x5c2b4230, SUC_lexicon_tags_37 },
    { 0x1b6b4231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe2b4233, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6b4238, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91cb423a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe74b423c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706b4242, SUC_lexicon_tags_31 },
    { 0xd5ab4242, SUC_lexicon_tags_164 },
    { 0x7eb4244, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cb4249, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17cb424b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42b424f, SUC_lexicon_tags_15 },
    { 0xb18b424f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6b4254, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb7cb4256, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ab4258, SUC_lexicon_tags_23 },
    { 0xa4b4258, SUC_lexicon_tags_61 },
    { 0x298b425a, SUC_lexicon_tags_151 },
    { 0xb48b425a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80b425d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x892b4260, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeb4262, SUC_lexicon_tags_23 },
    { 0x3b4b4263, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60b4268, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf92b426a, SUC_lexicon_tags_14 },
    { 0xa8cb426a, SUC_lexicon_tags_49 },
    { 0xf92b426c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cab4271, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4274, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2b427b, SUC_lexicon_tags_65 },
    { 0xd92b427c, SUC_lexicon_tags_30 },
    { 0x496b427d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49eb427f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x48cb4281, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xad0b4283, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0b4287, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55eb4289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6b428c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x562b428e, SUC_lexicon_tags_25 },
    { 0x5eb428f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa52b4291, SUC_lexicon_tags_33 },
    { 0xfa8b4292, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccb4296, SUC_lexicon_tags_37 },
    { 0x26cb4297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6b429a, SUC_lexicon_tags_85 },
    { 0xe94b429b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcb42a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0b42a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8b42aa, SUC_lexicon_tags_57 },
    { 0x274b42ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916b42b0, SUC_lexicon_tags_17 },
    { 0x658b42b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74b42b7, SUC_lexicon_tags_25 },
    { 0xc2eb42b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2b42bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2b42c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58b42c5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182b42cb, SUC_lexicon_tags_66 },
    { 0xe8cb42cc, SUC_lexicon_tags_33 },
    { 0xd96b42cd, SUC_lexicon_tags_14 },
    { 0xf8cb42ce, SUC_lexicon_tags_91 },
    { 0x728b42cf, SUC_lexicon_tags_39 },
    { 0x1b6b42d0, SUC_lexicon_tags_60 },
    { 0x196b42d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70b42d4, SUC_lexicon_tags_9 },
    { 0x134b42d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb42d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x532b42db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380b42e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c8b42e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5ab42e6, SUC_lexicon_tags_11 },
    { 0x3c6b42e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3eb42ee, SUC_lexicon_tags_7 },
    { 0x61eb42ef, SUC_lexicon_tags_38 },
    { 0xc6eb42ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ab42f4, SUC_lexicon_tags_47 },
    { 0xe7cb42f5, SUC_lexicon_tags_6 },
    { 0xd2b42f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf6b42f8, SUC_lexicon_tags_7 },
    { 0xe0ab42f9, SUC_lexicon_tags_16 },
    { 0x176b42f9, SUC_lexicon_tags_8 },
    { 0xbc6b42fb, SUC_lexicon_tags_11 },
    { 0x736b42fb, SUC_lexicon_tags_38 },
    { 0xac4b42fd, SUC_lexicon_tags_7 },
    { 0xd22b42fe, SUC_lexicon_tags_37 },
    { 0x7ceb42ff, SUC_lexicon_tags_60 },
    { 0x4ccb4300, SUC_lexicon_tags_11 },
    { 0x4c8b4301, SUC_lexicon_tags_9 },
    { 0xe42b4300, SUC_lexicon_tags_21 },
    { 0x588b4301, SUC_lexicon_tags_15 },
    { 0x638b42fe, SUC_lexicon_tags_3 },
    { 0xdeab42fb, SUC_lexicon_tags_8 },
    { 0xdceb4304, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x194b4308, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ab430d, SUC_lexicon_tags_1 },
    { 0x5e4b430e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8b4313, SUC_lexicon_tags_25 },
    { 0x634b4313, SUC_lexicon_tags_13 },
    { 0x226b4315, SUC_lexicon_tags_11 },
    { 0x9dcb4316, SUC_lexicon_tags_33 },
    { 0xa16b4316, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6b431a, SUC_lexicon_tags_7 },
    { 0x596b431b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6b431f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cb4321, SUC_lexicon_tags_7 },
    { 0xa48b4321, SUC_lexicon_tags_39 },
    { 0x33eb4323, SUC_lexicon_tags_21 },
    { 0xc9eb4324, SUC_lexicon_tags_37 },
    { 0xe26b4325, SUC_lexicon_tags_33 },
    { 0xebab4325, SUC_lexicon_tags_92 },
    { 0xeb2b4327, SUC_lexicon_tags_38 },
    { 0x318b4326, SUC_lexicon_tags_8 },
    { 0x9b8b4324, SUC_lexicon_tags_55 },
    { 0xf00b432a, SUC_lexicon_tags_47 },
    { 0x4b432b, SUC_lexicon_tags_31 },
    { 0x10b432c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70b4331, SUC_lexicon_tags_15 },
    { 0xbb4b4331, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x702b4334, SUC_lexicon_tags_7 },
    { 0xc8eb4335, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x168b4337, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850b433f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42b4343, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfcb4345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe40b4347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6b434a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x188b434c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acb4352, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf30b4354, SUC_lexicon_tags_14 },
    { 0xb14b4355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6b4358, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad0b435a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb435f, SUC_lexicon_tags_1 },
    { 0xa34b435f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab4364, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe6eb4366, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac8b4368, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa00b436a, SUC_lexicon_tags_16 },
    { 0x13ab436b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2b436e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cb4373, SUC_lexicon_tags_0 },
    { 0x1c2b4373, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab437f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8b4384, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ccb4386, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2b438a, SUC_lexicon_tags_80 },
    { 0xa2eb438a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xceeb438d, SUC_lexicon_tags_2 },
    { 0xbaab438e, SUC_lexicon_tags_63 },
    { 0xfaab438d, SUC_lexicon_tags_29 },
    { 0x98eb438d, SUC_lexicon_tags_1 },
    { 0x4ecb4391, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ab4395, SUC_lexicon_tags_2 },
    { 0x604b4395, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cb4399, SUC_lexicon_tags_12 },
    { 0x81cb439a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7e6b439c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b43a4, SUC_lexicon_tags_25 },
    { 0xbeb43a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x908b43a7, SUC_lexicon_tags_25 },
    { 0x7a6b43a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x994b43aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x758b43ac, SUC_lexicon_tags_46 },
    { 0x42cb43ad, SUC_lexicon_tags_14 },
    { 0xb40b43ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32b43b3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x680b43b5, SUC_lexicon_tags_12 },
    { 0xd24b43b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166b43be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x342b43c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72b43c5, SUC_lexicon_tags_61 },
    { 0xeaab43c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86eb43ca, SUC_lexicon_tags_11 },
    { 0xd74b43cb, SUC_lexicon_tags_10 },
    { 0xbd0b43cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8d8b43ce, SUC_lexicon_tags_46 },
    { 0xbe6b43ce, SUC_lexicon_tags_37 },
    { 0xd44b43d0, SUC_lexicon_tags_25 },
    { 0x942b43d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4b43d7, SUC_lexicon_tags_11 },
    { 0xd46b43d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2b43dc, SUC_lexicon_tags_8 },
    { 0xd84b43dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4b43e1, SUC_lexicon_tags_12 },
    { 0x6dab43e2, SUC_lexicon_tags_11 },
    { 0xae0b43e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x796b43e6, SUC_lexicon_tags_45 },
    { 0xbaab43e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x698b43e9, SUC_lexicon_tags_62 },
    { 0xa52b43ea, SUC_lexicon_tags_15 },
    { 0x348b43eb, SUC_lexicon_tags_37 },
    { 0xfdab43ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa14b43ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cb43f1, SUC_lexicon_tags_15 },
    { 0x16ab43f2, SUC_lexicon_tags_84 },
    { 0x6a4b43f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02b43fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cb43fe, SUC_lexicon_tags_33 },
    { 0xe1eb43ff, SUC_lexicon_tags_63 },
    { 0xa2cb4400, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0b4403, SUC_lexicon_tags_14 },
    { 0xdeab4404, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x33cb4407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x828b4409, SUC_lexicon_tags_2 },
    { 0x214b4409, SUC_lexicon_tags_15 },
    { 0x180b440a, SUC_lexicon_tags_65 },
    { 0x700b440c, SUC_lexicon_tags_9 },
    { 0xf48b440b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa10b440f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f8b4411, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeeab4413, SUC_lexicon_tags_14 },
    { 0xae2b4414, SUC_lexicon_tags_2 },
    { 0xd4cb4413, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4b4419, SUC_lexicon_tags_43 },
    { 0x7a0b441a, SUC_lexicon_tags_19 },
    { 0x612b441a, SUC_lexicon_tags_25 },
    { 0x972b441c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b442e, SUC_lexicon_tags_37 },
    { 0xc50b442f, SUC_lexicon_tags_25 },
    { 0xc4cb4430, SUC_lexicon_tags_106 },
    { 0x63cb442e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2b4435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68eb4439, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8b4446, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x1a8b4448, SUC_lexicon_tags_38 },
    { 0x532b4449, SUC_lexicon_tags_65 },
    { 0xa02b4449, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cb445a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88b445d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6b4462, SUC_lexicon_tags_17 },
    { 0x250b4463, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cb4466, SUC_lexicon_tags_7 },
    { 0x69eb4467, SUC_lexicon_tags_25 },
    { 0xc2cb4468, SUC_lexicon_tags_0 },
    { 0x75cb4469, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb30b446b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdaeb446d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624b4475, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52b447b, SUC_lexicon_tags_38 },
    { 0x57cb447c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432b4482, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x972b4485, SUC_lexicon_tags_27 },
    { 0x8ab4485, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x998b4488, SUC_lexicon_tags_30 },
    { 0xbaeb4488, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x88cb448c, SUC_lexicon_tags_7 },
    { 0x3f2b448c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb4b448f, SUC_lexicon_tags_14 },
    { 0xe0eb4490, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c6b4492, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2b4495, SUC_lexicon_tags_43 },
    { 0xa4ab4496, SUC_lexicon_tags_7 },
    { 0x3a6b4495, SUC_lexicon_tags_8 },
    { 0x376b4497, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ab449b, SUC_lexicon_tags_33 },
    { 0x8a8b449b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30b449f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69eb44a1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaceb44a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4acb44aa, SUC_lexicon_tags_7 },
    { 0xb0b44aa, SUC_lexicon_tags_11 },
    { 0xf06b44ac, SUC_lexicon_tags_63 },
    { 0xb84b44ad, SUC_lexicon_tags_37 },
    { 0xa6b44ae, SUC_lexicon_tags_39 },
    { 0x98cb44af, SUC_lexicon_tags_11 },
    { 0x752b44b0, SUC_lexicon_tags_17 },
    { 0x19cb44af, SUC_lexicon_tags_46 },
    { 0x18eb44b2, SUC_lexicon_tags_17 },
    { 0x3bcb44af, SUC_lexicon_tags_56 },
    { 0x5f0b44b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2b44b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14b44be, SUC_lexicon_tags_15 },
    { 0x156b44bf, SUC_lexicon_tags_11 },
    { 0xa84b44c0, SUC_lexicon_tags_7 },
    { 0xecab44c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb6b44c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3c6b44c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eb44ca, SUC_lexicon_tags_63 },
    { 0xbf8b44cb, SUC_lexicon_tags_0 },
    { 0x4b4b44cc, SUC_lexicon_tags_126 },
    { 0x530b44cd, SUC_lexicon_tags_77 },
    { 0x3e4b44cd, SUC_lexicon_tags_27 },
    { 0xa56b44ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40b44d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2b44d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb44dc, SUC_lexicon_tags_52 },
    { 0x2a0b44dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f0b44df, SUC_lexicon_tags_45 },
    { 0xf28b44df, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428b44ed, SUC_lexicon_tags_79 },
    { 0x82eb44ee, SUC_lexicon_tags_37 },
    { 0x35ab44ef, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6b44f8, SUC_lexicon_tags_15 },
    { 0x4ccb44f9, SUC_lexicon_tags_14 },
    { 0x406b44fa, SUC_lexicon_tags_235 },
    { 0xf04b44fb, SUC_lexicon_tags_37 },
    { 0x3e2b44fc, SUC_lexicon_tags_15 },
    { 0x924b44fb, SUC_lexicon_tags_6 },
    { 0x3e8b44fe, SUC_lexicon_tags_57 },
    { 0xe18b44fa, SUC_lexicon_tags_44 },
    { 0xceeb4500, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdeb4504, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa16b4506, SUC_lexicon_tags_29 },
    { 0x454b4507, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccb450b, SUC_lexicon_tags_11 },
    { 0xdc6b450c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd2b450e, SUC_lexicon_tags_21 },
    { 0xfeab450e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54b4512, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796b451b, SUC_lexicon_tags_7 },
    { 0x416b451c, SUC_lexicon_tags_7 },
    { 0xb6eb451d, SUC_lexicon_tags_29 },
    { 0x488b451d, SUC_lexicon_tags_31 },
    { 0x6ab451f, SUC_lexicon_tags_67 },
    { 0xfbcb4520, SUC_lexicon_tags_7 },
    { 0x4a2b451b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eb4525, SUC_lexicon_tags_11 },
    { 0x268b4525, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x208b4529, SUC_lexicon_tags_25 },
    { 0x6f0b4529, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2b452f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab4535, SUC_lexicon_tags_32 },
    { 0x7bcb4535, SUC_lexicon_tags_80 },
    { 0x6b2b4537, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a4b4539, SUC_lexicon_tags_1 },
    { 0x938b453a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x250b453c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eeb4545, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dab454a, SUC_lexicon_tags_28 },
    { 0x50cb454b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500b4553, SUC_lexicon_tags_2 },
    { 0x9cab4554, SUC_lexicon_tags_2 },
    { 0x150b4554, SUC_lexicon_tags_140 },
    { 0xe90b4555, SUC_lexicon_tags_33 },
    { 0x592b4553, SUC_lexicon_tags_12 },
    { 0xb12b4558, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6b455b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x988b455e, SUC_lexicon_tags_1 },
    { 0x9d6b455f, SUC_lexicon_tags_33 },
    { 0x10cb455e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb22b4562, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26b456a, SUC_lexicon_tags_34 },
    { 0x322b456b, SUC_lexicon_tags_47 },
    { 0xb6b456c, SUC_lexicon_tags_37 },
    { 0xf62b456d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x510b456f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cb4576, SUC_lexicon_tags_7 },
    { 0xae8b4576, SUC_lexicon_tags_25 },
    { 0xc32b4577, SUC_lexicon_tags_6 },
    { 0xa60b4579, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794b457e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd66b4580, SUC_lexicon_tags_56 },
    { 0xf18b4580, SUC_lexicon_tags_1 },
    { 0x29cb4580, SUC_lexicon_tags_31 },
    { 0xd76b4583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcb4586, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccb4590, SUC_lexicon_tags_15 },
    { 0xdceb4591, SUC_lexicon_tags_12 },
    { 0xe9ab4591, SUC_lexicon_tags_79 },
    { 0x5e2b4593, SUC_lexicon_tags_23 },
    { 0x3cb4594, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6b459b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eb459f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cb45a5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9eab45a7, SUC_lexicon_tags_1 },
    { 0x184b45a8, SUC_lexicon_tags_12 },
    { 0x664b45a9, SUC_lexicon_tags_6 },
    { 0x3dcb45aa, SUC_lexicon_tags_12 },
    { 0x7c2b45a9, SUC_lexicon_tags_2 },
    { 0x966b45ab, SUC_lexicon_tags_7 },
    { 0x1d2b45ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5ab45af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6d4b45b1, SUC_lexicon_tags_152 },
    { 0x7d0b45b2, SUC_lexicon_tags_1 },
    { 0x4ceb45b3, SUC_lexicon_tags_696 },
    { 0xb14b45b4, SUC_lexicon_tags_5 },
    { 0x4eeb45b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eb45ba, SUC_lexicon_tags_7 },
    { 0x54eb45bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224b45c2, SUC_lexicon_tags_19 },
    { 0xa64b45c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x668b45c5, SUC_lexicon_tags_14 },
    { 0xc40b45c6, SUC_lexicon_tags_17 },
    { 0x21eb45c7, SUC_lexicon_tags_25 },
    { 0x92cb45c8, SUC_lexicon_tags_46 },
    { 0x45eb45c7, SUC_lexicon_tags_17 },
    { 0xc28b45ca, SUC_lexicon_tags_2 },
    { 0x880b45cb, SUC_lexicon_tags_60 },
    { 0x24b45cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6ab45ce, SUC_lexicon_tags_25 },
    { 0xebeb45cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cb45d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0b45d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430b45dd, SUC_lexicon_tags_14 },
    { 0x71eb45de, SUC_lexicon_tags_15 },
    { 0x988b45df, SUC_lexicon_tags_12 },
    { 0xc94b45e0, SUC_lexicon_tags_47 },
    { 0x1aab45e1, SUC_lexicon_tags_6 },
    { 0x886b45e1, SUC_lexicon_tags_15 },
    { 0x9c4b45e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94eb45e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52ab45e9, SUC_lexicon_tags_25 },
    { 0xb8b45e9, SUC_lexicon_tags_17 },
    { 0xcc2b45e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x848b45ee, SUC_lexicon_tags_61 },
    { 0x80b45ef, SUC_lexicon_tags_30 },
    { 0x210b45f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x448b45f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80eb45fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cb4600, SUC_lexicon_tags_0 },
    { 0xab0b4601, SUC_lexicon_tags_6 },
    { 0xff4b4602, SUC_lexicon_tags_39 },
    { 0x19cb4602, SUC_lexicon_tags_39 },
    { 0x502b4604, SUC_lexicon_tags_28 },
    { 0x992b4601, SUC_lexicon_tags_26 },
    { 0x6e4b4606, SUC_lexicon_tags_30 },
    { 0xa00b4607, SUC_lexicon_tags_37 },
    { 0x782b4606, SUC_lexicon_tags_55 },
    { 0x6c2b4607, SUC_lexicon_tags_12 },
    { 0x422b460a, SUC_lexicon_tags_38 },
    { 0x6aab460a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0ab460d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb4610, SUC_lexicon_tags_38 },
    { 0x952b4611, SUC_lexicon_tags_2 },
    { 0x816b4612, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6b4617, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cb461b, SUC_lexicon_tags_0 },
    { 0x494b461c, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572b4626, SUC_lexicon_tags_17 },
    { 0xa8ab4626, SUC_lexicon_tags_33 },
    { 0x808b4627, SUC_lexicon_tags_11 },
    { 0xad2b4629, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38b462c, SUC_lexicon_tags_11 },
    { 0xf8eb462d, SUC_lexicon_tags_55 },
    { 0x33eb462e, SUC_lexicon_tags_25 },
    { 0xa62b462d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x590b4631, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b4637, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0cb4639, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcb463c, SUC_lexicon_tags_37 },
    { 0x656b463d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x540b463f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00b4642, SUC_lexicon_tags_11 },
    { 0xe9cb4643, SUC_lexicon_tags_11 },
    { 0xb50b4643, SUC_lexicon_tags_30 },
    { 0x7beb4643, SUC_lexicon_tags_43 },
    { 0xa74b4646, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50b464b, SUC_lexicon_tags_28 },
    { 0xb12b464c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x406b464e, SUC_lexicon_tags_11 },
    { 0x4b8b464f, SUC_lexicon_tags_30 },
    { 0xc2ab4650, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeccb4653, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50ab4655, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4b4658, SUC_lexicon_tags_11 },
    { 0x1f6b4659, SUC_lexicon_tags_25 },
    { 0x1d2b465a, SUC_lexicon_tags_88 },
    { 0xfd6b465a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcb465d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x162b465f, SUC_lexicon_tags_26 },
    { 0x4ecb465f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400b4669, SUC_lexicon_tags_29 },
    { 0x2a8b466a, SUC_lexicon_tags_4 },
    { 0x840b466b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ab466f, SUC_lexicon_tags_38 },
    { 0xacb4670, SUC_lexicon_tags_15 },
    { 0x38cb466f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x910b4674, SUC_lexicon_tags_37 },
    { 0x484b4674, SUC_lexicon_tags_11 },
    { 0x2d4b4676, SUC_lexicon_tags_45 },
    { 0x92eb4674, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ab467c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30b467f, SUC_lexicon_tags_92 },
    { 0xf9ab4680, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6b4683, SUC_lexicon_tags_28 },
    { 0x93cb4684, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbf2b4686, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x674b4688, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71ab468a, SUC_lexicon_tags_11 },
    { 0x284b468b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75cb468d, SUC_lexicon_tags_17 },
    { 0x686b468e, SUC_lexicon_tags_38 },
    { 0xe98b468f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8b4692, SUC_lexicon_tags_12 },
    { 0xacab4692, SUC_lexicon_tags_26 },
    { 0xb0b4692, SUC_lexicon_tags_56 },
    { 0xe9ab4695, SUC_lexicon_tags_61 },
    { 0x94eb4696, SUC_lexicon_tags_11 },
    { 0x456b4697, SUC_lexicon_tags_25 },
    { 0x600b4697, SUC_lexicon_tags_17 },
    { 0x772b4699, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bab469b, SUC_lexicon_tags_85 },
    { 0x9aeb469b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2b46a6, SUC_lexicon_tags_16 },
    { 0x3e0b46a7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe3eb46a9, SUC_lexicon_tags_1 },
    { 0x838b46aa, SUC_lexicon_tags_45 },
    { 0xe6b46ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d6b46ad, SUC_lexicon_tags_12 },
    { 0xc6b46ae, SUC_lexicon_tags_37 },
    { 0x154b46ae, SUC_lexicon_tags_15 },
    { 0xbeeb46b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66b46b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x562b46ba, SUC_lexicon_tags_11 },
    { 0xa4eb46ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd2b46bd, SUC_lexicon_tags_6 },
    { 0xec8b46bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8b46c2, SUC_lexicon_tags_19 },
    { 0x4aeb46c3, SUC_lexicon_tags_18 },
    { 0xbd4b46c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x724b46c7, SUC_lexicon_tags_77 },
    { 0xdab46c7, SUC_lexicon_tags_40 },
    { 0xb84b46c9, SUC_lexicon_tags_28 },
    { 0xa9ab46c9, SUC_lexicon_tags_30 },
    { 0xedeb46c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x100b46ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeb46d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98b46d7, SUC_lexicon_tags_15 },
    { 0x5b0b46d8, SUC_lexicon_tags_45 },
    { 0xeeab46d7, SUC_lexicon_tags_11 },
    { 0x780b46da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffeb46de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ab46e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00b46e6, SUC_lexicon_tags_16 },
    { 0x106b46e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4b46ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b4b46f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6b46fa, SUC_lexicon_tags_9 },
    { 0x3c4b46fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cb46fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8b4707, SUC_lexicon_tags_23 },
    { 0xbd6b4707, SUC_lexicon_tags_17 },
    { 0x482b4709, SUC_lexicon_tags_60 },
    { 0x5c4b470a, SUC_lexicon_tags_61 },
    { 0xce2b470b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x79cb470d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cb4710, SUC_lexicon_tags_15 },
    { 0xd52b4711, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ab4718, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858b471f, SUC_lexicon_tags_15 },
    { 0x2d2b4720, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d0b4722, SUC_lexicon_tags_15 },
    { 0x620b4722, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2b4728, SUC_lexicon_tags_19 },
    { 0xa52b4729, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x626b472b, SUC_lexicon_tags_15 },
    { 0x376b472c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74b4734, SUC_lexicon_tags_2 },
    { 0x874b4735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6eeb4737, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ab473b, SUC_lexicon_tags_21 },
    { 0x34eb473b, SUC_lexicon_tags_60 },
    { 0x736b473d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b4743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80cb4745, SUC_lexicon_tags_37 },
    { 0xb32b4746, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ab474a, SUC_lexicon_tags_9 },
    { 0xdf2b474b, SUC_lexicon_tags_37 },
    { 0xe96b474b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2cb474e, SUC_lexicon_tags_1 },
    { 0x102b474e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd22b4751, SUC_lexicon_tags_7 },
    { 0x2d8b4752, SUC_lexicon_tags_11 },
    { 0x8dab4753, SUC_lexicon_tags_15 },
    { 0x6e0b4752, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eb4757, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc7eb4759, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a8b475b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7deb4768, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb476b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70b4771, SUC_lexicon_tags_25 },
    { 0x1eb4772, SUC_lexicon_tags_4 },
    { 0x6ceb4773, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6b4777, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x13cb477a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844b477e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2b4783, SUC_lexicon_tags_2 },
    { 0x9c6b4784, SUC_lexicon_tags_45 },
    { 0xc22b4785, SUC_lexicon_tags_37 },
    { 0xfbab4785, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2b4788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876b478d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe42b478f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a8b4791, SUC_lexicon_tags_7 },
    { 0x3d0b4792, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb4799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8b479c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc4b479e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24b47a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bab47a7, SUC_lexicon_tags_7 },
    { 0xb0eb47a7, SUC_lexicon_tags_2 },
    { 0xef4b47a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12b47ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa0b47b0, SUC_lexicon_tags_7 },
    { 0x2aab47b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eb47b6, SUC_lexicon_tags_27 },
    { 0x59ab47b7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9ab47b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa70b47bb, SUC_lexicon_tags_12 },
    { 0x6a6b47bc, SUC_lexicon_tags_5 },
    { 0x890b47bc, SUC_lexicon_tags_15 },
    { 0x894b47bd, SUC_lexicon_tags_6 },
    { 0x632b47bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ceb47c7, SUC_lexicon_tags_28 },
    { 0x7b8b47c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfeb47ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b47d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24b47d5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x7c0b47d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2b47db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726b47e2, SUC_lexicon_tags_56 },
    { 0x4c2b47e3, SUC_lexicon_tags_25 },
    { 0x3cab47e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3fcb47e6, SUC_lexicon_tags_63 },
    { 0x6e2b47e7, SUC_lexicon_tags_45 },
    { 0xa3eb47e7, SUC_lexicon_tags_65 },
    { 0x594b47e6, SUC_lexicon_tags_37 },
    { 0x5bcb47e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998b47ee, SUC_lexicon_tags_13 },
    { 0x228b47ee, SUC_lexicon_tags_17 },
    { 0x40ab47ef, SUC_lexicon_tags_7 },
    { 0xfccb47f1, SUC_lexicon_tags_19 },
    { 0x324b47f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x284b47f4, SUC_lexicon_tags_11 },
    { 0x37cb47f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10eb47fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ab4802, SUC_lexicon_tags_17 },
    { 0xad0b4803, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d4b4805, SUC_lexicon_tags_47 },
    { 0x2b8b4806, SUC_lexicon_tags_17 },
    { 0xcaeb4807, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaaab4809, SUC_lexicon_tags_8 },
    { 0x84cb4809, SUC_lexicon_tags_15 },
    { 0x6ccb480b, SUC_lexicon_tags_12 },
    { 0x2beb480c, SUC_lexicon_tags_1 },
    { 0x87cb480d, SUC_lexicon_tags_14 },
    { 0xf82b480e, SUC_lexicon_tags_7 },
    { 0xb94b480f, SUC_lexicon_tags_17 },
    { 0x7deb4809, SUC_lexicon_tags_27 },
    { 0x8b2b4811, SUC_lexicon_tags_11 },
    { 0x7d4b4812, SUC_lexicon_tags_63 },
    { 0x964b4812, SUC_lexicon_tags_8 },
    { 0x3f4b480a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890b4818, SUC_lexicon_tags_17 },
    { 0xbccb4819, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a8b481b, SUC_lexicon_tags_35 },
    { 0xaccb481b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2b481f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2b4822, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4b4826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4b4829, SUC_lexicon_tags_37 },
    { 0x700b482a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ab482d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2b4833, SUC_lexicon_tags_37 },
    { 0x99cb4834, SUC_lexicon_tags_37 },
    { 0x3b8b4834, SUC_lexicon_tags_2 },
    { 0xcbcb4836, SUC_lexicon_tags_25 },
    { 0x864b4834, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10cb4839, SUC_lexicon_tags_12 },
    { 0xa4eb4839, SUC_lexicon_tags_32 },
    { 0xcbab483b, SUC_lexicon_tags_17 },
    { 0x1b0b483b, SUC_lexicon_tags_14 },
    { 0x5a6b483d, SUC_lexicon_tags_25 },
    { 0x5a6b483e, SUC_lexicon_tags_55 },
    { 0xb12b4839, SUC_lexicon_tags_29 },
    { 0xd54b483a, SUC_lexicon_tags_1 },
    { 0x16ab4841, SUC_lexicon_tags_12 },
    { 0xc62b483f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7a8b4844, SUC_lexicon_tags_19 },
    { 0x1d8b4845, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84b484d, SUC_lexicon_tags_2 },
    { 0xcfab484e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ab4852, SUC_lexicon_tags_37 },
    { 0x2ab4853, SUC_lexicon_tags_15 },
    { 0x2f0b4853, SUC_lexicon_tags_33 },
    { 0x826b4855, SUC_lexicon_tags_37 },
    { 0xc7ab4853, SUC_lexicon_tags_26 },
    { 0xfbeb4857, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb485a, SUC_lexicon_tags_11 },
    { 0x40eb485a, SUC_lexicon_tags_46 },
    { 0x142b485a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706b4860, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb4865, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x372b4867, SUC_lexicon_tags_33 },
    { 0x806b4868, SUC_lexicon_tags_27 },
    { 0xc9cb4869, SUC_lexicon_tags_25 },
    { 0xc54b486a, SUC_lexicon_tags_68 },
    { 0x988b486b, SUC_lexicon_tags_7 },
    { 0xe32b486b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8b4873, SUC_lexicon_tags_33 },
    { 0x5feb4873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ab4877, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cb487d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208b4882, SUC_lexicon_tags_19 },
    { 0x5b6b4882, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceeb4888, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbacb488a, SUC_lexicon_tags_17 },
    { 0x1aeb488b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6b488f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ab4893, SUC_lexicon_tags_9 },
    { 0x79ab4893, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ab4898, SUC_lexicon_tags_15 },
    { 0xbccb4898, SUC_lexicon_tags_37 },
    { 0x752b489a, SUC_lexicon_tags_15 },
    { 0xee2b489a, SUC_lexicon_tags_19 },
    { 0xe9cb4899, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xab0b489e, SUC_lexicon_tags_28 },
    { 0xa1cb489f, SUC_lexicon_tags_15 },
    { 0x8bab489f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xae2b48a2, SUC_lexicon_tags_37 },
    { 0x182b48a3, SUC_lexicon_tags_0 },
    { 0x25ab48a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6b48a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x176b48aa, SUC_lexicon_tags_31 },
    { 0x3b4b48ab, SUC_lexicon_tags_17 },
    { 0x48cb48ac, SUC_lexicon_tags_1 },
    { 0x870b48ad, SUC_lexicon_tags_15 },
    { 0x774b48ad, SUC_lexicon_tags_17 },
    { 0x516b48ad, SUC_lexicon_tags_0 },
    { 0x36ab48ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ab48b3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328b48bb, SUC_lexicon_tags_45 },
    { 0x990b48bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68b48c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862b48c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc2b48c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20b48cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80b48d0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x184b48d2, SUC_lexicon_tags_129 },
    { 0x5a6b48d2, SUC_lexicon_tags_17 },
    { 0xcaeb48d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x918b48d7, SUC_lexicon_tags_2 },
    { 0xaaeb48d8, SUC_lexicon_tags_17 },
    { 0xc9ab48d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdb0b48db, SUC_lexicon_tags_25 },
    { 0xa1ab48dc, SUC_lexicon_tags_23 },
    { 0xe3cb48dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7a8b48df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2b48e3, SUC_lexicon_tags_37 },
    { 0xe60b48e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38eb48e9, SUC_lexicon_tags_17 },
    { 0x10eb48ea, SUC_lexicon_tags_29 },
    { 0x6c6b48eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x328b48ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18b48f5, SUC_lexicon_tags_1 },
    { 0x6a0b48f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6aeb48f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab4900, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ab4905, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ab4908, SUC_lexicon_tags_6 },
    { 0x606b4909, SUC_lexicon_tags_38 },
    { 0x7ccb4909, SUC_lexicon_tags_15 },
    { 0xec6b490a, SUC_lexicon_tags_25 },
    { 0xc20b490c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x49ab490e, SUC_lexicon_tags_12 },
    { 0xaccb490e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2ab4911, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60b4914, SUC_lexicon_tags_85 },
    { 0x2a2b4914, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2b491a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb491f, SUC_lexicon_tags_69 },
    { 0x652b491f, SUC_lexicon_tags_30 },
    { 0x76cb4921, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefab492a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2b492d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x790b492f, SUC_lexicon_tags_63 },
    { 0x5f8b4930, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb4933, SUC_lexicon_tags_59 },
    { 0x52eb4934, SUC_lexicon_tags_7 },
    { 0x512b4935, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x142b4938, SUC_lexicon_tags_27 },
    { 0x320b4939, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x218b493b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18b4940, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30b4945, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd00b4947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb494a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2b494d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742b4953, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cb495c, SUC_lexicon_tags_1 },
    { 0x4aeb495d, SUC_lexicon_tags_15 },
    { 0x39cb495c, SUC_lexicon_tags_26 },
    { 0x628b495f, SUC_lexicon_tags_37 },
    { 0x958b4960, SUC_lexicon_tags_49 },
    { 0x84ab4961, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe80b4963, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34b4967, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46b496a, SUC_lexicon_tags_25 },
    { 0x58eb496b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafab4972, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4b4977, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ab497a, SUC_lexicon_tags_23 },
    { 0x72cb497b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584b497f, SUC_lexicon_tags_37 },
    { 0x824b4980, SUC_lexicon_tags_14 },
    { 0xe2b4980, SUC_lexicon_tags_37 },
    { 0x2cb4982, SUC_lexicon_tags_37 },
    { 0xb62b4982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x862b4985, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x792b4988, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbab498a, SUC_lexicon_tags_1 },
    { 0x426b498b, SUC_lexicon_tags_49 },
    { 0xf7ab498c, SUC_lexicon_tags_7 },
    { 0xd9eb498d, SUC_lexicon_tags_49 },
    { 0x184b498e, SUC_lexicon_tags_88 },
    { 0x2e6b498f, SUC_lexicon_tags_11 },
    { 0x602b498b, SUC_lexicon_tags_38 },
    { 0x75ab4991, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x578b4994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x566b4997, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcb499a, SUC_lexicon_tags_17 },
    { 0x51ab499b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x588b499d, SUC_lexicon_tags_82 },
    { 0x962b499e, SUC_lexicon_tags_17 },
    { 0xa20b499e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cb49a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328b49aa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x10eb49ac, SUC_lexicon_tags_60 },
    { 0x19cb49ac, SUC_lexicon_tags_26 },
    { 0xe5ab49ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ab49b3, SUC_lexicon_tags_17 },
    { 0xb7eb49b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x492b49b6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x61eb49b9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x42ab49bb, SUC_lexicon_tags_29 },
    { 0xdc6b49bc, SUC_lexicon_tags_126 },
    { 0xf28b49bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608b49c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb49c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1ccb49c7, SUC_lexicon_tags_21 },
    { 0xe4ab49c8, SUC_lexicon_tags_17 },
    { 0x640b49c8, SUC_lexicon_tags_45 },
    { 0x6d8b49ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x360b49cd, SUC_lexicon_tags_28 },
    { 0x22eb49cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0b49d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96b49de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2b49e3, SUC_lexicon_tags_27 },
    { 0x688b49e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x42eb49e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeeb49e9, SUC_lexicon_tags_38 },
    { 0xd68b49e9, SUC_lexicon_tags_37 },
    { 0x1e6b49eb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xfe6b49ed, SUC_lexicon_tags_0 },
    { 0x3fab49ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0b49f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb49fa, SUC_lexicon_tags_37 },
    { 0x5a8b49fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6b49ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04b4a03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0b4a08, SUC_lexicon_tags_27 },
    { 0x222b4a09, SUC_lexicon_tags_27 },
    { 0x8ccb4a09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4b4a0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78eb4a0f, SUC_lexicon_tags_17 },
    { 0x4bcb4a0f, SUC_lexicon_tags_85 },
    { 0x8a4b4a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2b4a15, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x640b4a17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10b4a1a, SUC_lexicon_tags_15 },
    { 0x178b4a1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8eb4a1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1aab4a1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc8b4a21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980b4a25, SUC_lexicon_tags_1 },
    { 0x1f6b4a26, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa46b4a28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa64b4a2a, SUC_lexicon_tags_56 },
    { 0xcc4b4a2b, SUC_lexicon_tags_7 },
    { 0xafeb4a2c, SUC_lexicon_tags_63 },
    { 0x18b4a2a, SUC_lexicon_tags_17 },
    { 0xffcb4a2e, SUC_lexicon_tags_37 },
    { 0xbb6b4a2f, SUC_lexicon_tags_65 },
    { 0xf96b4a2e, SUC_lexicon_tags_37 },
    { 0x836b4a31, SUC_lexicon_tags_25 },
    { 0x11eb4a32, SUC_lexicon_tags_37 },
    { 0x6f2b4a32, SUC_lexicon_tags_14 },
    { 0xfcab4a34, SUC_lexicon_tags_1 },
    { 0x5dab4a35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabab4a37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb4a3d, SUC_lexicon_tags_27 },
    { 0x64ab4a3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x246b4a41, SUC_lexicon_tags_41 },
    { 0xad6b4a42, SUC_lexicon_tags_29 },
    { 0x69eb4a42, SUC_lexicon_tags_47 },
    { 0x270b4a41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cb4a47, SUC_lexicon_tags_17 },
    { 0x292b4a47, SUC_lexicon_tags_82 },
    { 0x710b4a47, SUC_lexicon_tags_3 },
    { 0xf94b4a4a, SUC_lexicon_tags_11 },
    { 0x8bab4a4a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eb4a4f, SUC_lexicon_tags_7 },
    { 0xf92b4a50, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab4a59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710b4a61, SUC_lexicon_tags_38 },
    { 0x35cb4a62, SUC_lexicon_tags_47 },
    { 0x3b0b4a61, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6b4a66, SUC_lexicon_tags_25 },
    { 0x562b4a67, SUC_lexicon_tags_46 },
    { 0x4fab4a68, SUC_lexicon_tags_14 },
    { 0xba0b4a67, SUC_lexicon_tags_37 },
    { 0x3cb4a66, SUC_lexicon_tags_25 },
    { 0xcfab4a6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fab4a6e, SUC_lexicon_tags_25 },
    { 0x262b4a6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x418b4a71, SUC_lexicon_tags_9 },
    { 0x1aeb4a71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4b4a79, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x732b4a7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58b4a7d, SUC_lexicon_tags_21 },
    { 0xdfcb4a7e, SUC_lexicon_tags_15 },
    { 0xa52b4a7d, SUC_lexicon_tags_37 },
    { 0xc14b4a7e, SUC_lexicon_tags_9 },
    { 0x5d4b4a7f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeccb4a83, SUC_lexicon_tags_12 },
    { 0xbe2b4a84, SUC_lexicon_tags_82 },
    { 0xce0b4a85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ab4a8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86b4a8f, SUC_lexicon_tags_28 },
    { 0x7b0b4a8f, SUC_lexicon_tags_37 },
    { 0xec0b4a91, SUC_lexicon_tags_15 },
    { 0xf26b4a92, SUC_lexicon_tags_13 },
    { 0xc8eb4a92, SUC_lexicon_tags_35 },
    { 0x14b4a93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680b4a99, SUC_lexicon_tags_14 },
    { 0x9e2b4a99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46b4aa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x846b4aa7, SUC_lexicon_tags_2 },
    { 0xfe6b4aa8, SUC_lexicon_tags_55 },
    { 0xadcb4aa8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2b4aad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2b4ab8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30b4abf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd7ab4ac1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b4ac7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cb4aca, SUC_lexicon_tags_25 },
    { 0x3acb4acb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaacb4acd, SUC_lexicon_tags_25 },
    { 0xe8cb4ace, SUC_lexicon_tags_88 },
    { 0xa4eb4acf, SUC_lexicon_tags_151 },
    { 0xeeeb4ad0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x236b4ad2, SUC_lexicon_tags_65 },
    { 0x160b4ad3, SUC_lexicon_tags_11 },
    { 0xe50b4ad4, SUC_lexicon_tags_11 },
    { 0xd36b4ad5, SUC_lexicon_tags_45 },
    { 0x36b4ad6, SUC_lexicon_tags_2 },
    { 0xbfab4ad4, SUC_lexicon_tags_8 },
    { 0xd00b4ad3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ecb4ada, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37eb4adc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cb4ae2, SUC_lexicon_tags_1 },
    { 0xeb2b4ae2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e2b4ae5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabeb4aee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf2b4af0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50b4af3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0b4af7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x444b4af9, SUC_lexicon_tags_27 },
    { 0x1ceb4afa, SUC_lexicon_tags_23 },
    { 0xaab4afa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x96eb4afd, SUC_lexicon_tags_25 },
    { 0x22b4afd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38b4b07, SUC_lexicon_tags_7 },
    { 0xc0cb4b08, SUC_lexicon_tags_106 },
    { 0x676b4b08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x558b4b0b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ab4b0f, SUC_lexicon_tags_0 },
    { 0xd34b4b10, SUC_lexicon_tags_15 },
    { 0xe8b4b11, SUC_lexicon_tags_2 },
    { 0x464b4b12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4beb4b16, SUC_lexicon_tags_15 },
    { 0x5b0b4b16, SUC_lexicon_tags_17 },
    { 0xc7eb4b16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0b4b20, SUC_lexicon_tags_14 },
    { 0xd70b4b21, SUC_lexicon_tags_17 },
    { 0xe4b4b21, SUC_lexicon_tags_46 },
    { 0xed4b4b23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb12b4b25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e6b4b27, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x20cb4b29, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbeeb4b2b, SUC_lexicon_tags_28 },
    { 0x4beb4b2c, SUC_lexicon_tags_57 },
    { 0x324b4b2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8eb4b2f, SUC_lexicon_tags_12 },
    { 0xba8b4b30, SUC_lexicon_tags_37 },
    { 0xf12b4b30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48b4b34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eb4b38, SUC_lexicon_tags_37 },
    { 0xca0b4b39, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb4b3e, SUC_lexicon_tags_19 },
    { 0x34cb4b3e, SUC_lexicon_tags_25 },
    { 0xfdeb4b40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x922b4b43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cb4b4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddeb4b4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2b4b52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6b4b59, SUC_lexicon_tags_63 },
    { 0x306b4b5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4b4b63, SUC_lexicon_tags_0 },
    { 0xd20b4b63, SUC_lexicon_tags_17 },
    { 0xd04b4b63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x716b4b67, SUC_lexicon_tags_12 },
    { 0x318b4b67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x522b4b6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4b4b6f, SUC_lexicon_tags_37 },
    { 0xe16b4b70, SUC_lexicon_tags_47 },
    { 0x116b4b6f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x71eb4b73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6b4b76, SUC_lexicon_tags_37 },
    { 0x1c4b4b76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f2b4b79, SUC_lexicon_tags_79 },
    { 0xd20b4b7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x122b4b7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928b4b82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17cb4b85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x454b4b88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabeb4b8c, SUC_lexicon_tags_7 },
    { 0x9a4b4b8d, SUC_lexicon_tags_37 },
    { 0x698b4b8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd04b4b90, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2b4b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ab4b97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628b4b9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd92b4ba0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x544b4ba2, SUC_lexicon_tags_2 },
    { 0xcab4ba2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9eb4ba5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x788b4ba8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08b4bae, SUC_lexicon_tags_33 },
    { 0xdb8b4baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54b4bb5, SUC_lexicon_tags_46 },
    { 0x93cb4bb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b4bb9, SUC_lexicon_tags_25 },
    { 0xc4eb4bb9, SUC_lexicon_tags_0 },
    { 0x76eb4bba, SUC_lexicon_tags_37 },
    { 0x908b4bb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb4bc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75ab4bc2, SUC_lexicon_tags_45 },
    { 0xb2b4bc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b4b4bc5, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0b4bca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30b4bd0, SUC_lexicon_tags_15 },
    { 0xec4b4bd0, SUC_lexicon_tags_11 },
    { 0x83eb4bd2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8b4bdb, SUC_lexicon_tags_3 },
    { 0xa70b4bdb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9eb4beb, SUC_lexicon_tags_27 },
    { 0xe62b4beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8b4bf4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x466b4bf6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xee8b4bf8, SUC_lexicon_tags_2 },
    { 0x48b4bf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b0b4bfb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48b4bfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcb4c04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64b4c06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4b4c09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250b4c0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa36b4c10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x660b4c12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x94ab4c14, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916b4c1a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252b4c1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b4c21, SUC_lexicon_tags_14 },
    { 0xb38b4c21, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c0b4c24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6b4c27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60cb4c29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02b4c33, SUC_lexicon_tags_2 },
    { 0x364b4c34, SUC_lexicon_tags_25 },
    { 0x9ceb4c34, SUC_lexicon_tags_8 },
    { 0x57eb4c36, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944b4c3b, SUC_lexicon_tags_7 },
    { 0x3d2b4c3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9feb4c3e, SUC_lexicon_tags_2 },
    { 0x602b4c3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13cb4c41, SUC_lexicon_tags_11 },
    { 0x57ab4c42, SUC_lexicon_tags_15 },
    { 0x1b0b4c43, SUC_lexicon_tags_2 },
    { 0x71ab4c43, SUC_lexicon_tags_66 },
    { 0x934b4c45, SUC_lexicon_tags_1 },
    { 0xee2b4c46, SUC_lexicon_tags_84 },
    { 0xafcb4c47, SUC_lexicon_tags_38 },
    { 0x924b4c45, SUC_lexicon_tags_17 },
    { 0xb70b4c47, SUC_lexicon_tags_170 },
    { 0xa64b4c49, SUC_lexicon_tags_12 },
    { 0xe66b4c4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4ab4c4d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58eb4c51, SUC_lexicon_tags_14 },
    { 0xc56b4c51, SUC_lexicon_tags_8 },
    { 0x25cb4c51, SUC_lexicon_tags_21 },
    { 0xa12b4c51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3e0b4c56, SUC_lexicon_tags_12 },
    { 0x6f8b4c57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b4c5e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b4c62, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4b4c65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dab4c68, SUC_lexicon_tags_11 },
    { 0x28ab4c69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbab4c6d, SUC_lexicon_tags_1 },
    { 0x2c2b4c6d, SUC_lexicon_tags_485 },
    { 0x694b4c6e, SUC_lexicon_tags_37 },
    { 0x606b4c6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab4c74, SUC_lexicon_tags_33 },
    { 0x2a6b4c74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b4c79, SUC_lexicon_tags_47 },
    { 0x458b4c7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932b4c80, SUC_lexicon_tags_29 },
    { 0xdceb4c81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x320b4c83, SUC_lexicon_tags_17 },
    { 0x458b4c84, SUC_lexicon_tags_12 },
    { 0x8a6b4c85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cb4c89, SUC_lexicon_tags_35 },
    { 0x630b4c8a, SUC_lexicon_tags_3 },
    { 0x946b4c89, SUC_lexicon_tags_6 },
    { 0xf9ab4c8c, SUC_lexicon_tags_28 },
    { 0x654b4c8c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdab4c91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbacb4c93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd84b4c95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b4c9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6beb4c9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x420b4ca1, SUC_lexicon_tags_19 },
    { 0x8b6b4ca2, SUC_lexicon_tags_26 },
    { 0x94eb4ca3, SUC_lexicon_tags_14 },
    { 0x8bcb4ca4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x278b4ca7, SUC_lexicon_tags_2 },
    { 0x388b4ca8, SUC_lexicon_tags_17 },
    { 0x878b4ca9, SUC_lexicon_tags_21 },
    { 0x3a2b4caa, SUC_lexicon_tags_113 },
    { 0xa98b4caa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4b4cb1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3c2b4cb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ab4cb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ab4cb9, SUC_lexicon_tags_37 },
    { 0x82cb4cba, SUC_lexicon_tags_30 },
    { 0xfc4b4cbb, SUC_lexicon_tags_46 },
    { 0x2aeb4cb9, SUC_lexicon_tags_12 },
    { 0x510b4cbd, SUC_lexicon_tags_17 },
    { 0x27ab4cb9, SUC_lexicon_tags_30 },
    { 0x536b4cbf, SUC_lexicon_tags_2 },
    { 0xb06b4cba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464b4cc4, SUC_lexicon_tags_37 },
    { 0x1bcb4cc5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab4cca, SUC_lexicon_tags_57 },
    { 0x488b4cca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x896b4cce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x584b4cd0, SUC_lexicon_tags_68 },
    { 0x40ab4cd1, SUC_lexicon_tags_11 },
    { 0xbceb4cd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x202b4cd5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb84b4cd7, SUC_lexicon_tags_1 },
    { 0x694b4cd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x908b4cda, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab4cdd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25eb4ce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86b4ce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6deb4ce8, SUC_lexicon_tags_6 },
    { 0x70cb4ce9, SUC_lexicon_tags_11 },
    { 0x43cb4ce8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f6b4cec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0b4cef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ab4cf3, SUC_lexicon_tags_12 },
    { 0xd0eb4cf3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb4cf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424b4cfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadcb4cff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd64b4d01, SUC_lexicon_tags_12 },
    { 0xbb0b4d01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13eb4d06, SUC_lexicon_tags_14 },
    { 0x910b4d06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2b4d12, SUC_lexicon_tags_43 },
    { 0xe9ab4d12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b4d16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ab4d19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92b4d1c, SUC_lexicon_tags_30 },
    { 0xe42b4d1c, SUC_lexicon_tags_13 },
    { 0xdc8b4d1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556b4d22, SUC_lexicon_tags_1 },
    { 0x9e8b4d22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b4d26, SUC_lexicon_tags_2 },
    { 0x60cb4d27, SUC_lexicon_tags_5 },
    { 0x426b4d28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32b4d2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ab4d32, SUC_lexicon_tags_32 },
    { 0xdeb4d32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342b4d43, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44b4d48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x306b4d4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34b4d4f, SUC_lexicon_tags_11 },
    { 0x12b4d50, SUC_lexicon_tags_38 },
    { 0x558b4d51, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41eb4d57, SUC_lexicon_tags_47 },
    { 0x2b6b4d57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cb4d5b, SUC_lexicon_tags_2 },
    { 0x588b4d5b, SUC_lexicon_tags_21 },
    { 0x8b4b4d5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446b4d64, SUC_lexicon_tags_152 },
    { 0xf6ab4d65, SUC_lexicon_tags_47 },
    { 0x712b4d66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84b4d6a, SUC_lexicon_tags_42 },
    { 0x48eb4d6a, SUC_lexicon_tags_28 },
    { 0x23ab4d6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50b4d6f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6b4d72, SUC_lexicon_tags_40 },
    { 0x922b4d72, SUC_lexicon_tags_10 },
    { 0xcdcb4d73, SUC_lexicon_tags_29 },
    { 0x48eb4d75, SUC_lexicon_tags_6 },
    { 0x3f8b4d75, SUC_lexicon_tags_11 },
    { 0x7feb4d76, SUC_lexicon_tags_11 },
    { 0xad4b4d73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe42b4d7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bab4d7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5bab4d7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ab4d82, SUC_lexicon_tags_11 },
    { 0xb6b4d83, SUC_lexicon_tags_1 },
    { 0xc36b4d83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4b4d88, SUC_lexicon_tags_56 },
    { 0xd98b4d89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x26cb4d8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x374b4d8e, SUC_lexicon_tags_21 },
    { 0x1dcb4d8f, SUC_lexicon_tags_25 },
    { 0x958b4d90, SUC_lexicon_tags_1 },
    { 0xa70b4d91, SUC_lexicon_tags_25 },
    { 0x81cb4d91, SUC_lexicon_tags_30 },
    { 0x3b8b4d8e, SUC_lexicon_tags_61 },
    { 0x8aab4d93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2b4d9a, SUC_lexicon_tags_17 },
    { 0x608b4d9b, SUC_lexicon_tags_0 },
    { 0x1ceb4d9b, SUC_lexicon_tags_27 },
    { 0x852b4d9a, SUC_lexicon_tags_17 },
    { 0x542b4d9d, SUC_lexicon_tags_37 },
    { 0xdb2b4d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14b4da2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x760b4da4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b4da7, SUC_lexicon_tags_38 },
    { 0xa7eb4da8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b6b4daa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xffeb4dad, SUC_lexicon_tags_12 },
    { 0x704b4dad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4db5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4eb4dbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x35cb4dc0, SUC_lexicon_tags_38 },
    { 0x894b4dc1, SUC_lexicon_tags_37 },
    { 0x38cb4dc2, SUC_lexicon_tags_49 },
    { 0xdaeb4dc1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6b4dc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2b4dc9, SUC_lexicon_tags_33 },
    { 0x588b4dc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x300b4dcc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e4b4dce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8beb4dd6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6b4dd9, SUC_lexicon_tags_17 },
    { 0x2a6b4dd9, SUC_lexicon_tags_9 },
    { 0x36cb4ddb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554b4de4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16b4de9, SUC_lexicon_tags_11 },
    { 0x97eb4de9, SUC_lexicon_tags_55 },
    { 0xc30b4deb, SUC_lexicon_tags_49 },
    { 0x9b6b4dec, SUC_lexicon_tags_13 },
    { 0x36b4ded, SUC_lexicon_tags_17 },
    { 0xb0ab4dee, SUC_lexicon_tags_15 },
    { 0x9dab4def, SUC_lexicon_tags_17 },
    { 0x516b4df0, SUC_lexicon_tags_11 },
    { 0xea2b4df0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6b4df5, SUC_lexicon_tags_65 },
    { 0x8ab4df6, SUC_lexicon_tags_10 },
    { 0x7b8b4df6, SUC_lexicon_tags_12 },
    { 0x640b4df6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eab4dfb, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x20cb4dfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e8b4dff, SUC_lexicon_tags_46 },
    { 0x5a6b4e00, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x32cb4e02, SUC_lexicon_tags_25 },
    { 0x9a2b4e03, SUC_lexicon_tags_23 },
    { 0xb70b4e04, SUC_lexicon_tags_7 },
    { 0xe9eb4e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58b4e09, SUC_lexicon_tags_1 },
    { 0xa7cb4e0a, SUC_lexicon_tags_43 },
    { 0x80b4e0a, SUC_lexicon_tags_2 },
    { 0xdc8b4e0c, SUC_lexicon_tags_45 },
    { 0x456b4e0a, SUC_lexicon_tags_15 },
    { 0xae2b4e0e, SUC_lexicon_tags_1 },
    { 0x584b4e0b, SUC_lexicon_tags_49 },
    { 0xaacb4e10, SUC_lexicon_tags_54 },
    { 0xf50b4e11, SUC_lexicon_tags_28 },
    { 0xc8cb4e11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20b4e19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x220b4e1b, SUC_lexicon_tags_7 },
    { 0x7aeb4e1b, SUC_lexicon_tags_17 },
    { 0xcb2b4e1d, SUC_lexicon_tags_63 },
    { 0x300b4e1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe4b4e20, SUC_lexicon_tags_15 },
    { 0x66b4e21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd18b4e23, SUC_lexicon_tags_34 },
    { 0xae8b4e23, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab4e28, SUC_lexicon_tags_37 },
    { 0xb44b4e29, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cab4e2c, SUC_lexicon_tags_47 },
    { 0x58cb4e2d, SUC_lexicon_tags_19 },
    { 0x5a0b4e2e, SUC_lexicon_tags_17 },
    { 0xc86b4e2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bab4e32, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2b4e37, SUC_lexicon_tags_25 },
    { 0x610b4e38, SUC_lexicon_tags_17 },
    { 0x21ab4e39, SUC_lexicon_tags_8 },
    { 0x5f8b4e3a, SUC_lexicon_tags_12 },
    { 0x898b4e3a, SUC_lexicon_tags_38 },
    { 0xe94b4e3c, SUC_lexicon_tags_47 },
    { 0xc4eb4e3b, SUC_lexicon_tags_11 },
    { 0xc6cb4e3d, SUC_lexicon_tags_0 },
    { 0x65eb4e39, SUC_lexicon_tags_45 },
    { 0x624b4e39, SUC_lexicon_tags_127 },
    { 0x630b4e3e, SUC_lexicon_tags_37 },
    { 0x97eb4e39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54ab4e44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668b4e4b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8b4e4e, SUC_lexicon_tags_11 },
    { 0xc7ab4e4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14b4e54, SUC_lexicon_tags_12 },
    { 0x1a8b4e55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4e58, SUC_lexicon_tags_2 },
    { 0x2f6b4e59, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18b4e5d, SUC_lexicon_tags_84 },
    { 0xc56b4e5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944b4e64, SUC_lexicon_tags_21 },
    { 0xd86b4e65, SUC_lexicon_tags_10 },
    { 0xe02b4e66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e4b4e68, SUC_lexicon_tags_12 },
    { 0x76b4e68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fab4e6e, SUC_lexicon_tags_17 },
    { 0xdcab4e6e, SUC_lexicon_tags_6 },
    { 0x4cab4e6e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x4a6b4e72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2b4e7d, SUC_lexicon_tags_38 },
    { 0xb0cb4e7d, SUC_lexicon_tags_11 },
    { 0x70b4e7e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498b4e87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4b4e8c, SUC_lexicon_tags_15 },
    { 0x3f8b4e8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52b4e95, SUC_lexicon_tags_47 },
    { 0xfab4e96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaeb4e9d, SUC_lexicon_tags_1 },
    { 0xba2b4e9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8b4ea2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1eb4ea4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x42eb4ea6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600b4eaa, SUC_lexicon_tags_25 },
    { 0xbdcb4eaa, SUC_lexicon_tags_15 },
    { 0x522b4eab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x800b4eae, SUC_lexicon_tags_17 },
    { 0x2acb4eaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e4b4eb1, SUC_lexicon_tags_37 },
    { 0xec6b4eb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4eb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x604b4eb8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd32b4eba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eb4ec2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x74b4ec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x394b4ec7, SUC_lexicon_tags_2 },
    { 0xb8b4ec7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114b4ecd, SUC_lexicon_tags_0 },
    { 0x6f8b4ece, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ab4ed2, SUC_lexicon_tags_37 },
    { 0xa12b4ed3, SUC_lexicon_tags_47 },
    { 0xe8b4ed4, SUC_lexicon_tags_26 },
    { 0xca2b4ed4, SUC_lexicon_tags_15 },
    { 0xf66b4ed6, SUC_lexicon_tags_25 },
    { 0x110b4ed6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668b4edd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730b4ee2, SUC_lexicon_tags_47 },
    { 0x724b4ee3, SUC_lexicon_tags_27 },
    { 0x684b4ee3, SUC_lexicon_tags_33 },
    { 0x3b2b4ee5, SUC_lexicon_tags_15 },
    { 0xc0cb4ee6, SUC_lexicon_tags_7 },
    { 0x11ab4ee7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80b4eea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6b4ef0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8b4ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x706b4ef7, SUC_lexicon_tags_1 },
    { 0xf2ab4ef8, SUC_lexicon_tags_65 },
    { 0x9deb4ef8, SUC_lexicon_tags_37 },
    { 0xaa0b4ef7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x198b4efc, SUC_lexicon_tags_49 },
    { 0xa9ab4efd, SUC_lexicon_tags_25 },
    { 0x5eab4efd, SUC_lexicon_tags_28 },
    { 0xd00b4efd, SUC_lexicon_tags_48 },
    { 0xe6eb4efe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414b4f04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaab4f07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46b4f0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6b4f0e, SUC_lexicon_tags_25 },
    { 0x3f6b4f0e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d8b4f11, SUC_lexicon_tags_10 },
    { 0xc2b4f12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2b4f16, SUC_lexicon_tags_23 },
    { 0xc8cb4f17, SUC_lexicon_tags_25 },
    { 0x604b4f18, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x50eb4f1a, SUC_lexicon_tags_37 },
    { 0xed8b4f1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8b4f20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5cab4f22, SUC_lexicon_tags_0 },
    { 0x846b4f23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x104b4f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x988b4f29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cb4f2c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ab4f2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80b4f37, SUC_lexicon_tags_49 },
    { 0xbfcb4f38, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xaa6b4f3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x362b4f3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a0b4f3f, SUC_lexicon_tags_17 },
    { 0x40eb4f40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71cb4f42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322b4f46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25eb4f4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4e0b4f4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe4eb4f4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x276b4f50, SUC_lexicon_tags_2 },
    { 0xb3cb4f51, SUC_lexicon_tags_17 },
    { 0x18b4f51, SUC_lexicon_tags_23 },
    { 0xa3cb4f53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd6b4f55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33ab4f57, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x786b4f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2b4f61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218b4f6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8b4f73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x626b4f76, SUC_lexicon_tags_11 },
    { 0xbbcb4f76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x442b4f79, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0b4f87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790b4f8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48b4f92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24eb4f95, SUC_lexicon_tags_37 },
    { 0xe22b4f96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cb4f99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ab4fa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb4fa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbceb4fa6, SUC_lexicon_tags_15 },
    { 0xefab4fa6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb4fb2, SUC_lexicon_tags_19 },
    { 0x702b4fb3, SUC_lexicon_tags_17 },
    { 0xc6cb4fb3, SUC_lexicon_tags_0 },
    { 0x42cb4fb3, SUC_lexicon_tags_832 },
    { 0, NULL },
    { 0x568b4fb7, SUC_lexicon_tags_68 },
    { 0x1a8b4fb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0b4fbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69cb4fbe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258b4fc7, SUC_lexicon_tags_15 },
    { 0x9a0b4fc7, SUC_lexicon_tags_28 },
    { 0x850b4fc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ab4fcc, SUC_lexicon_tags_37 },
    { 0xda6b4fcd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x662b4fd0, SUC_lexicon_tags_15 },
    { 0x974b4fd1, SUC_lexicon_tags_19 },
    { 0x4eb4fd0, SUC_lexicon_tags_31 },
    { 0xa4ab4fd0, SUC_lexicon_tags_14 },
    { 0xd04b4fd4, SUC_lexicon_tags_25 },
    { 0x9e2b4fd4, SUC_lexicon_tags_17 },
    { 0xdeeb4fd6, SUC_lexicon_tags_11 },
    { 0xc5eb4fd7, SUC_lexicon_tags_49 },
    { 0xb4cb4fd3, SUC_lexicon_tags_2 },
    { 0x736b4fd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16b4fde, SUC_lexicon_tags_80 },
    { 0xd7ab4fdf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2b4fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x480b4fe6, SUC_lexicon_tags_7 },
    { 0x9a2b4fe7, SUC_lexicon_tags_43 },
    { 0xb3eb4fe8, SUC_lexicon_tags_7 },
    { 0x3eab4fe9, SUC_lexicon_tags_26 },
    { 0xeb8b4fea, SUC_lexicon_tags_9 },
    { 0x516b4fe9, SUC_lexicon_tags_49 },
    { 0xae0b4fe7, SUC_lexicon_tags_47 },
    { 0x2beb4fe8, SUC_lexicon_tags_25 },
    { 0x80b4fee, SUC_lexicon_tags_6 },
    { 0xf32b4fee, SUC_lexicon_tags_0 },
    { 0x9cb4fe7, SUC_lexicon_tags_15 },
    { 0x9f4b4ff1, SUC_lexicon_tags_26 },
    { 0xd62b4ff1, SUC_lexicon_tags_12 },
    { 0x8e8b4ff3, SUC_lexicon_tags_1 },
    { 0xb2b4ff2, SUC_lexicon_tags_43 },
    { 0xaf0b4ff4, SUC_lexicon_tags_47 },
    { 0x1acb4ff6, SUC_lexicon_tags_23 },
    { 0x6a2b4ff2, SUC_lexicon_tags_6 },
    { 0xec0b4feb, SUC_lexicon_tags_20 },
    { 0x77eb4ff9, SUC_lexicon_tags_2 },
    { 0xb04b4ffa, SUC_lexicon_tags_12 },
    { 0xb44b4ffb, SUC_lexicon_tags_47 },
    { 0xfbab4ffc, SUC_lexicon_tags_48 },
    { 0xc6b4ffa, SUC_lexicon_tags_23 },
    { 0xe4eb4ffe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab5004, SUC_lexicon_tags_42 },
    { 0x6eeb5005, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x66eb5008, SUC_lexicon_tags_12 },
    { 0x950b5009, SUC_lexicon_tags_57 },
    { 0xa22b500a, SUC_lexicon_tags_37 },
    { 0xd34b500b, SUC_lexicon_tags_49 },
    { 0x32eb500c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4b500f, SUC_lexicon_tags_1 },
    { 0xd78b5010, SUC_lexicon_tags_2 },
    { 0xe58b500f, SUC_lexicon_tags_23 },
    { 0x948b5012, SUC_lexicon_tags_63 },
    { 0xbbcb5013, SUC_lexicon_tags_32 },
    { 0x136b5014, SUC_lexicon_tags_2 },
    { 0x2d8b5015, SUC_lexicon_tags_28 },
    { 0x1ccb5016, SUC_lexicon_tags_26 },
    { 0x80cb5015, SUC_lexicon_tags_2 },
    { 0x2ecb5017, SUC_lexicon_tags_15 },
    { 0x302b5016, SUC_lexicon_tags_31 },
    { 0xf14b501a, SUC_lexicon_tags_28 },
    { 0x3ccb501b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x398b501e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116b5026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x638b5028, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4eb502a, SUC_lexicon_tags_1 },
    { 0xde6b502a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdcb502d, SUC_lexicon_tags_25 },
    { 0xfbeb502e, SUC_lexicon_tags_0 },
    { 0x672b502f, SUC_lexicon_tags_529 },
    { 0xc06b502e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed6b5032, SUC_lexicon_tags_15 },
    { 0x7c4b5033, SUC_lexicon_tags_63 },
    { 0xf48b5034, SUC_lexicon_tags_37 },
    { 0x12cb5034, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x278b5037, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x544b503a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98b503e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ab5041, SUC_lexicon_tags_17 },
    { 0x6b6b5041, SUC_lexicon_tags_15 },
    { 0xf32b5043, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaab5046, SUC_lexicon_tags_15 },
    { 0x36cb5047, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xaa6b5049, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99eb504b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396b5050, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb5053, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa60b5055, SUC_lexicon_tags_17 },
    { 0xe16b5055, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14b505a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ab5060, SUC_lexicon_tags_29 },
    { 0x804b5061, SUC_lexicon_tags_2 },
    { 0x25eb5062, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x68b5064, SUC_lexicon_tags_1 },
    { 0x880b5065, SUC_lexicon_tags_37 },
    { 0xec2b5066, SUC_lexicon_tags_11 },
    { 0x524b5067, SUC_lexicon_tags_1406 },
    { 0, NULL },
    { 0x402b5069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacb5071, SUC_lexicon_tags_7 },
    { 0x912b5072, SUC_lexicon_tags_17 },
    { 0x8a6b5073, SUC_lexicon_tags_11 },
    { 0xb78b5074, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316b5079, SUC_lexicon_tags_45 },
    { 0xba2b5079, SUC_lexicon_tags_31 },
    { 0xe5ab507b, SUC_lexicon_tags_37 },
    { 0x130b507c, SUC_lexicon_tags_2 },
    { 0xe2ab507d, SUC_lexicon_tags_38 },
    { 0x3b4b507b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x964b5080, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96b5088, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ab508b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85eb508d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2b5090, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x98cb5093, SUC_lexicon_tags_6 },
    { 0xbaab5094, SUC_lexicon_tags_17 },
    { 0xd0eb5095, SUC_lexicon_tags_48 },
    { 0xe94b5096, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796b50a0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8b50a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12b50aa, SUC_lexicon_tags_27 },
    { 0x902b50ab, SUC_lexicon_tags_15 },
    { 0x4f4b50ab, SUC_lexicon_tags_37 },
    { 0x37eb50ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eb50b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe0b50b7, SUC_lexicon_tags_0 },
    { 0xd2eb50b8, SUC_lexicon_tags_87 },
    { 0xa26b50b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ab50c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeb4b50c2, SUC_lexicon_tags_9 },
    { 0xddab50c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x236b50c6, SUC_lexicon_tags_423 },
    { 0xee0b50c7, SUC_lexicon_tags_46 },
    { 0x9e4b50c8, SUC_lexicon_tags_15 },
    { 0xa1cb50c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0b50ce, SUC_lexicon_tags_151 },
    { 0x36cb50cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2b50d3, SUC_lexicon_tags_48 },
    { 0x42b50d4, SUC_lexicon_tags_39 },
    { 0xc84b50d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8b50d8, SUC_lexicon_tags_55 },
    { 0x67eb50d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6cb50db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336b50df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f6b50e1, SUC_lexicon_tags_37 },
    { 0x70b50e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8b50e7, SUC_lexicon_tags_57 },
    { 0x79eb50e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25eb50ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc0b50ec, SUC_lexicon_tags_49 },
    { 0xd38b50ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896b50f2, SUC_lexicon_tags_12 },
    { 0x9bcb50f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf4b50f5, SUC_lexicon_tags_26 },
    { 0x3beb50f6, SUC_lexicon_tags_17 },
    { 0x724b50f6, SUC_lexicon_tags_17 },
    { 0x100b50f8, SUC_lexicon_tags_15 },
    { 0x928b50f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b50fd, SUC_lexicon_tags_17 },
    { 0xe94b50fe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34b5103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6b5108, SUC_lexicon_tags_17 },
    { 0x678b5109, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x61ab510b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6b510f, SUC_lexicon_tags_17 },
    { 0xc2eb5110, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcb5115, SUC_lexicon_tags_15 },
    { 0xa50b5116, SUC_lexicon_tags_33 },
    { 0xd2eb5117, SUC_lexicon_tags_94 },
    { 0xcfeb5118, SUC_lexicon_tags_35 },
    { 0x6eeb5119, SUC_lexicon_tags_25 },
    { 0xa9ab5119, SUC_lexicon_tags_11 },
    { 0x378b511b, SUC_lexicon_tags_103 },
    { 0x31cb511b, SUC_lexicon_tags_4 },
    { 0x7feb511d, SUC_lexicon_tags_17 },
    { 0x4cab511c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b5126, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x688b5129, SUC_lexicon_tags_14 },
    { 0x2e8b512a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78b512d, SUC_lexicon_tags_45 },
    { 0xa38b512d, SUC_lexicon_tags_3 },
    { 0x46cb512f, SUC_lexicon_tags_63 },
    { 0x5d0b5130, SUC_lexicon_tags_25 },
    { 0x32ab512e, SUC_lexicon_tags_67 },
    { 0xc48b512d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x368b5135, SUC_lexicon_tags_11 },
    { 0xe02b5136, SUC_lexicon_tags_56 },
    { 0xa4b5135, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16b513a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x870b513c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac0b513e, SUC_lexicon_tags_40 },
    { 0xbf0b513f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2b5143, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318b5149, SUC_lexicon_tags_2 },
    { 0xf5ab514a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3aeb514c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28cb514f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08b5152, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x906b5154, SUC_lexicon_tags_23 },
    { 0xabeb5154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26b5158, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a6b515a, SUC_lexicon_tags_1 },
    { 0x842b515a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dab515f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62b5164, SUC_lexicon_tags_1 },
    { 0x27eb5164, SUC_lexicon_tags_33 },
    { 0xf0eb5166, SUC_lexicon_tags_11 },
    { 0xf30b5166, SUC_lexicon_tags_29 },
    { 0x4c6b5165, SUC_lexicon_tags_12 },
    { 0xb46b5166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffab5170, SUC_lexicon_tags_43 },
    { 0x6b2b5170, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a2b5173, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x268b5175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9eb5177, SUC_lexicon_tags_27 },
    { 0x430b5177, SUC_lexicon_tags_32 },
    { 0xa86b5178, SUC_lexicon_tags_49 },
    { 0x7fab517a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898b517e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb5182, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4b5188, SUC_lexicon_tags_10 },
    { 0x6b0b5188, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfbcb518b, SUC_lexicon_tags_68 },
    { 0x41eb518c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf16b518e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x816b5190, SUC_lexicon_tags_2 },
    { 0xdeeb5190, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2b5198, SUC_lexicon_tags_1 },
    { 0xcd2b5198, SUC_lexicon_tags_33 },
    { 0x5b2b519a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1dcb519c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ab51a2, SUC_lexicon_tags_81 },
    { 0xc46b51a2, SUC_lexicon_tags_68 },
    { 0x20eb51a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x266b51a7, SUC_lexicon_tags_24 },
    { 0xe14b51a8, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x20ab51aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa28b51ac, SUC_lexicon_tags_11 },
    { 0x2dcb51ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56b51b2, SUC_lexicon_tags_33 },
    { 0x8a2b51b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33eb51ba, SUC_lexicon_tags_47 },
    { 0xd04b51ba, SUC_lexicon_tags_4 },
    { 0x76cb51bc, SUC_lexicon_tags_1 },
    { 0x594b51bc, SUC_lexicon_tags_16 },
    { 0xf52b51bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbf4b51c0, SUC_lexicon_tags_16 },
    { 0xdd6b51c0, SUC_lexicon_tags_30 },
    { 0x7bab51c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x95cb51c4, SUC_lexicon_tags_2 },
    { 0xbc2b51c5, SUC_lexicon_tags_37 },
    { 0xdaeb51c6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8b51c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78b51cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190b51d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x800b51d4, SUC_lexicon_tags_39 },
    { 0xa46b51d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd06b51d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x81cb51d9, SUC_lexicon_tags_13 },
    { 0xfeab51d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x230b51dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964b51e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b51ec, SUC_lexicon_tags_0 },
    { 0xc88b51ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b51f0, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0x5c8b51f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0b51f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x700b51fc, SUC_lexicon_tags_14 },
    { 0x69ab51fc, SUC_lexicon_tags_47 },
    { 0x3d8b51fe, SUC_lexicon_tags_9 },
    { 0x5b6b51ff, SUC_lexicon_tags_11 },
    { 0xe4cb5200, SUC_lexicon_tags_34 },
    { 0xdacb51fe, SUC_lexicon_tags_26 },
    { 0xe20b5201, SUC_lexicon_tags_19 },
    { 0xe0eb5203, SUC_lexicon_tags_31 },
    { 0xf36b5204, SUC_lexicon_tags_1 },
    { 0x566b51ff, SUC_lexicon_tags_33 },
    { 0x4ccb5206, SUC_lexicon_tags_0 },
    { 0x21ab51ff, SUC_lexicon_tags_47 },
    { 0xbfcb5208, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x852b520a, SUC_lexicon_tags_14 },
    { 0x5c4b520b, SUC_lexicon_tags_27 },
    { 0x892b520c, SUC_lexicon_tags_37 },
    { 0x618b520d, SUC_lexicon_tags_11 },
    { 0x3acb520d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23eb5210, SUC_lexicon_tags_17 },
    { 0x91eb5211, SUC_lexicon_tags_9 },
    { 0x524b5212, SUC_lexicon_tags_7 },
    { 0x14cb5213, SUC_lexicon_tags_15 },
    { 0x9d8b5214, SUC_lexicon_tags_38 },
    { 0x802b5212, SUC_lexicon_tags_11 },
    { 0xbeeb5210, SUC_lexicon_tags_65 },
    { 0xbb8b5217, SUC_lexicon_tags_25 },
    { 0xea4b5213, SUC_lexicon_tags_9 },
    { 0x946b5214, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x402b521c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa62b521e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa08b5220, SUC_lexicon_tags_7 },
    { 0x508b5221, SUC_lexicon_tags_12 },
    { 0x18b5221, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x41cb5225, SUC_lexicon_tags_11 },
    { 0xe4ab5226, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6b5229, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2b522e, SUC_lexicon_tags_30 },
    { 0x4feb522f, SUC_lexicon_tags_77 },
    { 0xe80b522e, SUC_lexicon_tags_26 },
    { 0x7ceb5231, SUC_lexicon_tags_47 },
    { 0xb56b5231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb5236, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb1ab5238, SUC_lexicon_tags_26 },
    { 0x2b5238, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa16b523b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30b523e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb8b5240, SUC_lexicon_tags_65 },
    { 0x9d8b5241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb5247, SUC_lexicon_tags_9 },
    { 0xda0b5248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdaab524a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4b524c, SUC_lexicon_tags_37 },
    { 0xa42b524d, SUC_lexicon_tags_63 },
    { 0xe9ab524e, SUC_lexicon_tags_11 },
    { 0xb7cb524e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06b5255, SUC_lexicon_tags_26 },
    { 0xb9eb5255, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ab5261, SUC_lexicon_tags_0 },
    { 0xaab5261, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28b5264, SUC_lexicon_tags_15 },
    { 0x41cb5265, SUC_lexicon_tags_37 },
    { 0xdd2b5266, SUC_lexicon_tags_23 },
    { 0x496b5267, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b526c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cb5274, SUC_lexicon_tags_17 },
    { 0xe2eb5275, SUC_lexicon_tags_9 },
    { 0xbb2b5276, SUC_lexicon_tags_37 },
    { 0xf44b5277, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x922b5279, SUC_lexicon_tags_17 },
    { 0xb12b527a, SUC_lexicon_tags_43 },
    { 0xdf0b527b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2b5280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680b5287, SUC_lexicon_tags_43 },
    { 0xfa4b5288, SUC_lexicon_tags_33 },
    { 0xdfcb5289, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6b528c, SUC_lexicon_tags_14 },
    { 0x14ab528d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd50b528f, SUC_lexicon_tags_38 },
    { 0x56ab5290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb68b5292, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab5294, SUC_lexicon_tags_1 },
    { 0x398b5294, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x924b5298, SUC_lexicon_tags_17 },
    { 0x120b5299, SUC_lexicon_tags_11 },
    { 0xcd4b5299, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292b529e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466b52a6, SUC_lexicon_tags_63 },
    { 0xd46b52a7, SUC_lexicon_tags_1 },
    { 0xa6b52a8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x202b52aa, SUC_lexicon_tags_13 },
    { 0x3b8b52ab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x426b52ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x548b52b1, SUC_lexicon_tags_33 },
    { 0x6d8b52b1, SUC_lexicon_tags_7 },
    { 0x79cb52b3, SUC_lexicon_tags_1 },
    { 0xad0b52b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42eb52b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x588b52b8, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x900b52ba, SUC_lexicon_tags_129 },
    { 0xc24b52bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4eb52bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x23eb52bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x362b52c1, SUC_lexicon_tags_11 },
    { 0x5d6b52c2, SUC_lexicon_tags_12 },
    { 0x4ecb52c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aab52c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x492b52cc, SUC_lexicon_tags_0 },
    { 0x22b52cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b0b52cf, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x844b52d1, SUC_lexicon_tags_42 },
    { 0x2eab52d1, SUC_lexicon_tags_1 },
    { 0x150b52d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x666b52d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c0b52d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x210b52db, SUC_lexicon_tags_38 },
    { 0xc9eb52db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53eb52e2, SUC_lexicon_tags_25 },
    { 0x7c4b52e3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652b52e7, SUC_lexicon_tags_31 },
    { 0x2f4b52e8, SUC_lexicon_tags_17 },
    { 0x62cb52e9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860b52f1, SUC_lexicon_tags_1 },
    { 0x9fcb52f1, SUC_lexicon_tags_2 },
    { 0x442b52f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cb52fa, SUC_lexicon_tags_61 },
    { 0x154b52fb, SUC_lexicon_tags_17 },
    { 0x288b52fc, SUC_lexicon_tags_43 },
    { 0x2c8b52fb, SUC_lexicon_tags_15 },
    { 0xebeb52fe, SUC_lexicon_tags_77 },
    { 0x934b52fb, SUC_lexicon_tags_60 },
    { 0x932b5300, SUC_lexicon_tags_54 },
    { 0x828b5300, SUC_lexicon_tags_19 },
    { 0xc7eb5300, SUC_lexicon_tags_81 },
    { 0x41cb5303, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cb5306, SUC_lexicon_tags_47 },
    { 0x12cb5307, SUC_lexicon_tags_15 },
    { 0xf68b5308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524b530c, SUC_lexicon_tags_2 },
    { 0x6b0b530d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x612b530f, SUC_lexicon_tags_23 },
    { 0x7dab5310, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8eab5312, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee4b5314, SUC_lexicon_tags_14 },
    { 0x764b5315, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x696b5317, SUC_lexicon_tags_7 },
    { 0x19ab5318, SUC_lexicon_tags_17 },
    { 0xf4eb5319, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51cb531b, SUC_lexicon_tags_1 },
    { 0x6bcb531c, SUC_lexicon_tags_47 },
    { 0x816b531d, SUC_lexicon_tags_25 },
    { 0x922b531b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956b5323, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cb5329, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cb532c, SUC_lexicon_tags_25 },
    { 0xb2eb532d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2b5330, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xff0b5332, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68b5334, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466b533b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd76b533d, SUC_lexicon_tags_25 },
    { 0xba4b533d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf6eb5340, SUC_lexicon_tags_12 },
    { 0x330b5340, SUC_lexicon_tags_15 },
    { 0x402b5342, SUC_lexicon_tags_0 },
    { 0xe54b5342, SUC_lexicon_tags_17 },
    { 0xd78b5344, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2b5349, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8b534c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0b5352, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4b5354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0b5358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b535b, SUC_lexicon_tags_1 },
    { 0xdb8b535c, SUC_lexicon_tags_26 },
    { 0xeb2b535c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6eb5360, SUC_lexicon_tags_14 },
    { 0xe88b5360, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbb6b5363, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f8b5365, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b5369, SUC_lexicon_tags_17 },
    { 0xd68b536a, SUC_lexicon_tags_33 },
    { 0xcc6b536b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x944b536d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8b5373, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8b5376, SUC_lexicon_tags_8 },
    { 0xfd0b5377, SUC_lexicon_tags_1 },
    { 0x8d2b5377, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cb537d, SUC_lexicon_tags_46 },
    { 0xe74b537e, SUC_lexicon_tags_57 },
    { 0xb3ab537e, SUC_lexicon_tags_11 },
    { 0x548b537d, SUC_lexicon_tags_56 },
    { 0x17cb5381, SUC_lexicon_tags_40 },
    { 0x16b5382, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceb5387, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eb538a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282b5395, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccb539c, SUC_lexicon_tags_4 },
    { 0x5a2b539d, SUC_lexicon_tags_25 },
    { 0xf4b539c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50b53a6, SUC_lexicon_tags_12 },
    { 0xebcb53a7, SUC_lexicon_tags_79 },
    { 0x7acb53a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384b53b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2b53b8, SUC_lexicon_tags_17 },
    { 0xb3cb53b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5ab53bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b53be, SUC_lexicon_tags_60 },
    { 0x1feb53bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x802b53c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b53c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ab53cc, SUC_lexicon_tags_5 },
    { 0x1a0b53cd, SUC_lexicon_tags_45 },
    { 0xba6b53ce, SUC_lexicon_tags_39 },
    { 0x9a4b53cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f2b53d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50b53d5, SUC_lexicon_tags_17 },
    { 0x698b53d6, SUC_lexicon_tags_10 },
    { 0xbd4b53d7, SUC_lexicon_tags_25 },
    { 0xc36b53d7, SUC_lexicon_tags_2 },
    { 0x2f8b53d8, SUC_lexicon_tags_15 },
    { 0x28b53d8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x69ab53dc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ab53df, SUC_lexicon_tags_33 },
    { 0x218b53e0, SUC_lexicon_tags_0 },
    { 0xccab53df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeb53e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeb53e9, SUC_lexicon_tags_2 },
    { 0x3d8b53ea, SUC_lexicon_tags_14 },
    { 0x4b53eb, SUC_lexicon_tags_23 },
    { 0x83cb53ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b53ef, SUC_lexicon_tags_30 },
    { 0x6aeb53ef, SUC_lexicon_tags_37 },
    { 0x204b53f1, SUC_lexicon_tags_56 },
    { 0xbbeb53f2, SUC_lexicon_tags_17 },
    { 0xa6eb53f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24eb53f5, SUC_lexicon_tags_1 },
    { 0x30b53f6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfa2b53f8, SUC_lexicon_tags_47 },
    { 0xcf0b53f9, SUC_lexicon_tags_39 },
    { 0x5c0b53f8, SUC_lexicon_tags_11 },
    { 0x1a8b53fb, SUC_lexicon_tags_1 },
    { 0xadab53fc, SUC_lexicon_tags_87 },
    { 0xa56b53fc, SUC_lexicon_tags_17 },
    { 0x56ab53fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x548b5400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x74eb5403, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbab5406, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842b540b, SUC_lexicon_tags_43 },
    { 0x562b540c, SUC_lexicon_tags_11 },
    { 0xa88b540c, SUC_lexicon_tags_72 },
    { 0xcbcb540e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260b5417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6feb541b, SUC_lexicon_tags_12 },
    { 0xc10b541b, SUC_lexicon_tags_65 },
    { 0x9b6b541d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x75ab541f, SUC_lexicon_tags_8 },
    { 0xdaab541f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204b5424, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdaeb5426, SUC_lexicon_tags_14 },
    { 0xc24b5427, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7fcb5429, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64b542f, SUC_lexicon_tags_138 },
    { 0x1d8b542f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x610b5432, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ab5435, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x542b5437, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xeb2b5439, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43eb543c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ab5444, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790b5448, SUC_lexicon_tags_11 },
    { 0x990b5448, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ab5450, SUC_lexicon_tags_55 },
    { 0x16ab5450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168b5456, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40b5468, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8b546b, SUC_lexicon_tags_36 },
    { 0x312b546c, SUC_lexicon_tags_17 },
    { 0x77cb546b, SUC_lexicon_tags_35 },
    { 0x9feb546e, SUC_lexicon_tags_37 },
    { 0x350b546f, SUC_lexicon_tags_19 },
    { 0x4aeb5470, SUC_lexicon_tags_6 },
    { 0x374b546f, SUC_lexicon_tags_2 },
    { 0xab4b546c, SUC_lexicon_tags_351 },
    { 0x54eb5473, SUC_lexicon_tags_12 },
    { 0xb9ab546d, SUC_lexicon_tags_49 },
    { 0xe94b5472, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b5477, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed2b5479, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12b547d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2b5480, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8b5486, SUC_lexicon_tags_17 },
    { 0xd98b5487, SUC_lexicon_tags_1 },
    { 0x2f6b5488, SUC_lexicon_tags_87 },
    { 0xc76b5489, SUC_lexicon_tags_4 },
    { 0xebcb5489, SUC_lexicon_tags_17 },
    { 0x49cb5488, SUC_lexicon_tags_15 },
    { 0x6eab5486, SUC_lexicon_tags_14 },
    { 0xf3ab548d, SUC_lexicon_tags_47 },
    { 0x2e4b548e, SUC_lexicon_tags_17 },
    { 0xc90b548e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8b5495, SUC_lexicon_tags_124 },
    { 0x25cb5495, SUC_lexicon_tags_15 },
    { 0x686b5497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88b549a, SUC_lexicon_tags_15 },
    { 0x818b549b, SUC_lexicon_tags_10 },
    { 0x4f2b549c, SUC_lexicon_tags_82 },
    { 0xed0b549c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34b54a1, SUC_lexicon_tags_19 },
    { 0x5cb54a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x828b54a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348b54a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x992b54ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0eb54ad, SUC_lexicon_tags_2 },
    { 0x562b54ad, SUC_lexicon_tags_25 },
    { 0xc9ab54ad, SUC_lexicon_tags_17 },
    { 0xbfab54b0, SUC_lexicon_tags_37 },
    { 0xe9ab54ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xceb54b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6d6b54b5, SUC_lexicon_tags_2 },
    { 0x1dab54b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb10b54b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8b54be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ceb54c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2b54c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab54c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb54cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fab54cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x16ab54d1, SUC_lexicon_tags_60 },
    { 0x6feb54d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b54d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35eb54d9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xee2b54db, SUC_lexicon_tags_11 },
    { 0xd8eb54dc, SUC_lexicon_tags_17 },
    { 0xf7cb54dd, SUC_lexicon_tags_23 },
    { 0xaccb54de, SUC_lexicon_tags_4 },
    { 0xaaab54df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5a2b54e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0b54e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134b54f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x278b54f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcb54f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92b54fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffab54ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ab5505, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ab5509, SUC_lexicon_tags_10 },
    { 0xf24b550a, SUC_lexicon_tags_25 },
    { 0x33eb550b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x250b550d, SUC_lexicon_tags_0 },
    { 0xdf2b550d, SUC_lexicon_tags_17 },
    { 0x892b550f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76ab5511, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x32cb5513, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x788b5516, SUC_lexicon_tags_11 },
    { 0x806b5516, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4b551c, SUC_lexicon_tags_17 },
    { 0x30b551c, SUC_lexicon_tags_0 },
    { 0x56ab551d, SUC_lexicon_tags_17 },
    { 0x6d6b551f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb40b5521, SUC_lexicon_tags_75 },
    { 0x67cb5521, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0b5528, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8eb5531, SUC_lexicon_tags_17 },
    { 0xabcb5532, SUC_lexicon_tags_55 },
    { 0x8ccb5533, SUC_lexicon_tags_45 },
    { 0x74b5531, SUC_lexicon_tags_17 },
    { 0xd06b5533, SUC_lexicon_tags_38 },
    { 0x7acb5536, SUC_lexicon_tags_14 },
    { 0x67ab5537, SUC_lexicon_tags_13 },
    { 0x9ecb5536, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64b553f, SUC_lexicon_tags_49 },
    { 0xc4ab553f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ab5545, SUC_lexicon_tags_14 },
    { 0xc8b5545, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11ab5548, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb554c, SUC_lexicon_tags_43 },
    { 0x82eb554c, SUC_lexicon_tags_4 },
    { 0x256b554c, SUC_lexicon_tags_65 },
    { 0x12eb554f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eb5552, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3cb5554, SUC_lexicon_tags_2 },
    { 0x4eb5555, SUC_lexicon_tags_33 },
    { 0xcb6b5556, SUC_lexicon_tags_7 },
    { 0x22b5557, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a6b5559, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x344b555b, SUC_lexicon_tags_11 },
    { 0x6c6b555b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb5563, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdab5566, SUC_lexicon_tags_23 },
    { 0x7aab5566, SUC_lexicon_tags_37 },
    { 0x2b0b5566, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aeb556d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd96b556f, SUC_lexicon_tags_1 },
    { 0x22b5570, SUC_lexicon_tags_43 },
    { 0x7acb5571, SUC_lexicon_tags_46 },
    { 0x980b5571, SUC_lexicon_tags_55 },
    { 0x2b0b5572, SUC_lexicon_tags_29 },
    { 0xa58b5574, SUC_lexicon_tags_11 },
    { 0xb92b5573, SUC_lexicon_tags_45 },
    { 0xa9eb5576, SUC_lexicon_tags_15 },
    { 0xb88b5577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccb557a, SUC_lexicon_tags_11 },
    { 0xac0b557b, SUC_lexicon_tags_11 },
    { 0x4deb557b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6b5583, SUC_lexicon_tags_25 },
    { 0xd52b5584, SUC_lexicon_tags_0 },
    { 0x862b5585, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8b5588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f6b558a, SUC_lexicon_tags_63 },
    { 0xc3ab558b, SUC_lexicon_tags_47 },
    { 0x600b558b, SUC_lexicon_tags_1 },
    { 0x96eb558d, SUC_lexicon_tags_19 },
    { 0xc80b558e, SUC_lexicon_tags_37 },
    { 0x66ab558e, SUC_lexicon_tags_63 },
    { 0x654b558e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x842b5592, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd8eb5594, SUC_lexicon_tags_63 },
    { 0xf0b5595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14b5597, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986b559d, SUC_lexicon_tags_2 },
    { 0x190b559e, SUC_lexicon_tags_19 },
    { 0xa74b559e, SUC_lexicon_tags_26 },
    { 0x11cb559e, SUC_lexicon_tags_68 },
    { 0x822b559e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4b55a7, SUC_lexicon_tags_17 },
    { 0x2eab55a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446b55ac, SUC_lexicon_tags_43 },
    { 0x5b6b55ad, SUC_lexicon_tags_43 },
    { 0x7feb55ae, SUC_lexicon_tags_29 },
    { 0x15eb55af, SUC_lexicon_tags_37 },
    { 0x72b55b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9eb55b2, SUC_lexicon_tags_25 },
    { 0xa50b55b3, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x30ab55b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f6b55b7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12b55ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908b55c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2b55c3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50eb55c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab55cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27eb55d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88b55da, SUC_lexicon_tags_21 },
    { 0xe88b55da, SUC_lexicon_tags_12 },
    { 0x41cb55dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcb55df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5ab55e1, SUC_lexicon_tags_26 },
    { 0x552b55e2, SUC_lexicon_tags_12 },
    { 0xca0b55e3, SUC_lexicon_tags_63 },
    { 0xdacb55e2, SUC_lexicon_tags_28 },
    { 0xa9cb55e5, SUC_lexicon_tags_47 },
    { 0x2a6b55e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1cb55e8, SUC_lexicon_tags_25 },
    { 0x8bab55e8, SUC_lexicon_tags_6 },
    { 0x104b55ea, SUC_lexicon_tags_154 },
    { 0x3eb55ea, SUC_lexicon_tags_17 },
    { 0xe44b55ea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3bcb55ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472b55f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf2b55f4, SUC_lexicon_tags_25 },
    { 0x8f6b55f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcb55f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2b55fd, SUC_lexicon_tags_11 },
    { 0xcfcb55fd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cb5603, SUC_lexicon_tags_25 },
    { 0xa90b5604, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5bcb5606, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x55cb5608, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67cb560a, SUC_lexicon_tags_63 },
    { 0x4f6b560b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76b560f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe76b5611, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b5614, SUC_lexicon_tags_129 },
    { 0xf66b5614, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb5619, SUC_lexicon_tags_6 },
    { 0x6c0b5619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76b561d, SUC_lexicon_tags_15 },
    { 0x68eb561e, SUC_lexicon_tags_28 },
    { 0xdf8b561f, SUC_lexicon_tags_45 },
    { 0xfe6b5620, SUC_lexicon_tags_7 },
    { 0xf12b5621, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834b5628, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4b562d, SUC_lexicon_tags_25 },
    { 0xe4b562e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x792b5631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eb5638, SUC_lexicon_tags_19 },
    { 0x7eab5639, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe0b563b, SUC_lexicon_tags_27 },
    { 0xeb563b, SUC_lexicon_tags_26 },
    { 0xe52b563d, SUC_lexicon_tags_26 },
    { 0x118b563d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ab5641, SUC_lexicon_tags_60 },
    { 0x1aab5642, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaab5645, SUC_lexicon_tags_11 },
    { 0xe7eb5646, SUC_lexicon_tags_1 },
    { 0xdb2b5647, SUC_lexicon_tags_31 },
    { 0xcacb5648, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x910b564b, SUC_lexicon_tags_37 },
    { 0x628b564c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1b4b564e, SUC_lexicon_tags_27 },
    { 0xdc2b564f, SUC_lexicon_tags_4 },
    { 0x4e2b564f, SUC_lexicon_tags_11 },
    { 0xc66b5651, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9c8b5653, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2b5656, SUC_lexicon_tags_12 },
    { 0x894b5656, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b565b, SUC_lexicon_tags_39 },
    { 0xbc0b565c, SUC_lexicon_tags_27 },
    { 0x9f4b565d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24b5662, SUC_lexicon_tags_37 },
    { 0x8b2b5663, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30b5666, SUC_lexicon_tags_17 },
    { 0x610b5667, SUC_lexicon_tags_11 },
    { 0x86b5667, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x738b566a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaccb566c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x316b566e, SUC_lexicon_tags_31 },
    { 0x754b566f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cab5678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x176b567b, SUC_lexicon_tags_11 },
    { 0x782b567b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x762b567e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cb5682, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262b5686, SUC_lexicon_tags_51 },
    { 0xf6b5687, SUC_lexicon_tags_19 },
    { 0x74cb5688, SUC_lexicon_tags_7 },
    { 0x3eab5686, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefcb568b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x32cb568e, SUC_lexicon_tags_1 },
    { 0xa28b568e, SUC_lexicon_tags_25 },
    { 0x190b5690, SUC_lexicon_tags_23 },
    { 0x25eb5691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6b5697, SUC_lexicon_tags_7 },
    { 0xcc4b5697, SUC_lexicon_tags_14 },
    { 0xcbeb5698, SUC_lexicon_tags_17 },
    { 0xb94b5699, SUC_lexicon_tags_12 },
    { 0xca8b5697, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x298b569e, SUC_lexicon_tags_23 },
    { 0x88b569f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x912b56a2, SUC_lexicon_tags_27 },
    { 0x4a6b56a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcb56a7, SUC_lexicon_tags_27 },
    { 0xbdeb56a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ab56b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x638b56b5, SUC_lexicon_tags_6 },
    { 0x6dab56b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92eb56b9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92b56be, SUC_lexicon_tags_7 },
    { 0x9eeb56bf, SUC_lexicon_tags_37 },
    { 0x798b56c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4b56c3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200b56cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb56cf, SUC_lexicon_tags_7 },
    { 0x27ab56d0, SUC_lexicon_tags_106 },
    { 0xff2b56cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2b56d4, SUC_lexicon_tags_1 },
    { 0x7d4b56d5, SUC_lexicon_tags_77 },
    { 0x2b8b56d6, SUC_lexicon_tags_114 },
    { 0xceeb56d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3eb56d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2eb56db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92b56e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x560b56e2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x462b56e4, SUC_lexicon_tags_1 },
    { 0xb90b56e5, SUC_lexicon_tags_9 },
    { 0x856b56e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x460b56e9, SUC_lexicon_tags_12 },
    { 0x6aeb56ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cb56ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206b56f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x698b56f6, SUC_lexicon_tags_28 },
    { 0x7fab56f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894b56fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8b56ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b4b5701, SUC_lexicon_tags_66 },
    { 0xcbeb5702, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8ab5704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4b5708, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2b570c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ab5713, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996b571a, SUC_lexicon_tags_8 },
    { 0x846b571a, SUC_lexicon_tags_26 },
    { 0xa0ab571c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42b571f, SUC_lexicon_tags_27 },
    { 0x950b5720, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cab5723, SUC_lexicon_tags_1 },
    { 0xecab5724, SUC_lexicon_tags_19 },
    { 0x2b5724, SUC_lexicon_tags_26 },
    { 0xc12b5725, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccb5731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ab5736, SUC_lexicon_tags_2 },
    { 0x72b5737, SUC_lexicon_tags_15 },
    { 0xa98b5738, SUC_lexicon_tags_27 },
    { 0x390b5737, SUC_lexicon_tags_2 },
    { 0xb8b573a, SUC_lexicon_tags_7 },
    { 0x978b573a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0b5740, SUC_lexicon_tags_83 },
    { 0x340b5740, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd84b5743, SUC_lexicon_tags_33 },
    { 0xb68b5743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x740b5746, SUC_lexicon_tags_1004 },
    { 0, NULL },
    { 0x486b5748, SUC_lexicon_tags_8 },
    { 0x580b5749, SUC_lexicon_tags_17 },
    { 0x658b574a, SUC_lexicon_tags_25 },
    { 0x5eeb5748, SUC_lexicon_tags_12 },
    { 0x748b574c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4b574f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcb5754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb5756, SUC_lexicon_tags_25 },
    { 0x7ccb5757, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc66b5759, SUC_lexicon_tags_26 },
    { 0x4e8b575a, SUC_lexicon_tags_32 },
    { 0x6d4b575a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aab575e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc10b5760, SUC_lexicon_tags_48 },
    { 0x746b5761, SUC_lexicon_tags_39 },
    { 0x13cb5762, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12b5769, SUC_lexicon_tags_66 },
    { 0xb0b576a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52eb576c, SUC_lexicon_tags_23 },
    { 0xae6b576d, SUC_lexicon_tags_14 },
    { 0x862b576d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c2b5770, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3c4b5772, SUC_lexicon_tags_27 },
    { 0xb52b5772, SUC_lexicon_tags_27 },
    { 0xe08b5774, SUC_lexicon_tags_60 },
    { 0xa0b5773, SUC_lexicon_tags_21 },
    { 0xb86b5776, SUC_lexicon_tags_14 },
    { 0xc70b5776, SUC_lexicon_tags_1 },
    { 0xc80b5772, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab577d, SUC_lexicon_tags_2 },
    { 0x18b577e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8b5785, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x54b5787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11eb578a, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x62ab578c, SUC_lexicon_tags_49 },
    { 0x890b578d, SUC_lexicon_tags_28 },
    { 0xaceb578d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cb5793, SUC_lexicon_tags_37 },
    { 0x1e8b5794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204b579d, SUC_lexicon_tags_37 },
    { 0x95ab579d, SUC_lexicon_tags_12 },
    { 0x98ab579f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46b57a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990b57a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x416b57aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8b57ae, SUC_lexicon_tags_37 },
    { 0x8f6b57af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8acb57b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ab57b5, SUC_lexicon_tags_77 },
    { 0xee8b57b6, SUC_lexicon_tags_49 },
    { 0xbfcb57b5, SUC_lexicon_tags_12 },
    { 0x478b57b8, SUC_lexicon_tags_17 },
    { 0x8c4b57b6, SUC_lexicon_tags_37 },
    { 0x630b57ba, SUC_lexicon_tags_23 },
    { 0xa5eb57b6, SUC_lexicon_tags_14 },
    { 0x486b57b9, SUC_lexicon_tags_32 },
    { 0x9a0b57bd, SUC_lexicon_tags_11 },
    { 0xf9eb57bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ab57cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2c0b57cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf0b57d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x58eb57d4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x10b57d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cb57dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6b57e2, SUC_lexicon_tags_25 },
    { 0x966b57e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x608b57e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8b57e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfab57eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2b57f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ab57f9, SUC_lexicon_tags_0 },
    { 0xf34b57fa, SUC_lexicon_tags_7 },
    { 0x18b57fb, SUC_lexicon_tags_27 },
    { 0xef0b57fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cb5800, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd92b5802, SUC_lexicon_tags_25 },
    { 0xdb4b5803, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bab5808, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eab5814, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7b0b5816, SUC_lexicon_tags_12 },
    { 0xc3cb5817, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84b581e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf6b5820, SUC_lexicon_tags_867 },
    { 0x81cb5821, SUC_lexicon_tags_60 },
    { 0x820b5822, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8b5827, SUC_lexicon_tags_45 },
    { 0x756b5828, SUC_lexicon_tags_1 },
    { 0x8b8b5829, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cb582d, SUC_lexicon_tags_1 },
    { 0x2dcb582e, SUC_lexicon_tags_0 },
    { 0xd14b582f, SUC_lexicon_tags_15 },
    { 0x226b582f, SUC_lexicon_tags_1 },
    { 0x6fcb5831, SUC_lexicon_tags_47 },
    { 0x550b5832, SUC_lexicon_tags_19 },
    { 0x620b5833, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578b5839, SUC_lexicon_tags_11 },
    { 0xe70b583a, SUC_lexicon_tags_49 },
    { 0x212b583b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa18b583d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcb5842, SUC_lexicon_tags_47 },
    { 0xf14b5843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab5847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c8b5849, SUC_lexicon_tags_63 },
    { 0x97eb584a, SUC_lexicon_tags_7 },
    { 0xb54b584b, SUC_lexicon_tags_17 },
    { 0xb2cb584b, SUC_lexicon_tags_19 },
    { 0x6c8b584d, SUC_lexicon_tags_17 },
    { 0xd2cb584e, SUC_lexicon_tags_2 },
    { 0x4c2b584c, SUC_lexicon_tags_37 },
    { 0x490b584a, SUC_lexicon_tags_39 },
    { 0xd72b5851, SUC_lexicon_tags_25 },
    { 0x736b584b, SUC_lexicon_tags_33 },
    { 0x500b584f, SUC_lexicon_tags_30 },
    { 0xbbab5854, SUC_lexicon_tags_1 },
    { 0xf84b5855, SUC_lexicon_tags_15 },
    { 0x6d0b5856, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x812b5858, SUC_lexicon_tags_15 },
    { 0x1c6b5859, SUC_lexicon_tags_43 },
    { 0xe80b5858, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x33ab585c, SUC_lexicon_tags_61 },
    { 0x9b4b585d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ab5860, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb70b5862, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8b5866, SUC_lexicon_tags_17 },
    { 0x8bcb5867, SUC_lexicon_tags_33 },
    { 0x310b5867, SUC_lexicon_tags_68 },
    { 0x732b5868, SUC_lexicon_tags_19 },
    { 0x906b5869, SUC_lexicon_tags_2 },
    { 0xe66b5869, SUC_lexicon_tags_65 },
    { 0x390b586a, SUC_lexicon_tags_17 },
    { 0x100b586d, SUC_lexicon_tags_1 },
    { 0x356b5868, SUC_lexicon_tags_7 },
    { 0x16eb586f, SUC_lexicon_tags_47 },
    { 0xfe4b586f, SUC_lexicon_tags_27 },
    { 0x448b5870, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe86b5873, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a8b5875, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x208b5878, SUC_lexicon_tags_1 },
    { 0x294b5878, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x17eb587b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd8eb587d, SUC_lexicon_tags_11 },
    { 0x78eb587e, SUC_lexicon_tags_79 },
    { 0xcd6b587f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4b5883, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x594b5886, SUC_lexicon_tags_12 },
    { 0xc26b5886, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ab588a, SUC_lexicon_tags_48 },
    { 0xfb8b588b, SUC_lexicon_tags_15 },
    { 0x818b588b, SUC_lexicon_tags_15 },
    { 0x882b588d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1cb588f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eb5893, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ab5898, SUC_lexicon_tags_11 },
    { 0x64eb5898, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0b589d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cb58a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdc0b58a2, SUC_lexicon_tags_12 },
    { 0xc34b58a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x402b58a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad0b58a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6b58ac, SUC_lexicon_tags_37 },
    { 0xa56b58ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe1eb58af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcb58b4, SUC_lexicon_tags_25 },
    { 0x134b58b5, SUC_lexicon_tags_37 },
    { 0xbe0b58b6, SUC_lexicon_tags_34 },
    { 0xbc8b58b7, SUC_lexicon_tags_27 },
    { 0x9c6b58b7, SUC_lexicon_tags_11 },
    { 0xd10b58b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a6b58bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2b58be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b58c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd90b58c4, SUC_lexicon_tags_37 },
    { 0xa14b58c5, SUC_lexicon_tags_21 },
    { 0xafeb58c6, SUC_lexicon_tags_46 },
    { 0xdc6b58c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93eb58c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x224b58cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44b58d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa20b58d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32b58d7, SUC_lexicon_tags_104 },
    { 0x580b58d8, SUC_lexicon_tags_7 },
    { 0xa24b58d9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa7ab58db, SUC_lexicon_tags_1 },
    { 0xad6b58dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40b58df, SUC_lexicon_tags_34 },
    { 0x318b58e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd7eb58e2, SUC_lexicon_tags_47 },
    { 0xceab58e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eb58e7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2b58ec, SUC_lexicon_tags_46 },
    { 0x850b58ed, SUC_lexicon_tags_60 },
    { 0x5d2b58ed, SUC_lexicon_tags_15 },
    { 0x4cab58ef, SUC_lexicon_tags_15 },
    { 0x9a6b58ef, SUC_lexicon_tags_31 },
    { 0x75eb58ec, SUC_lexicon_tags_15 },
    { 0x254b58f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74b58f5, SUC_lexicon_tags_7 },
    { 0x742b58f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x54b58f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76b5908, SUC_lexicon_tags_7 },
    { 0x4deb5909, SUC_lexicon_tags_17 },
    { 0x20cb590a, SUC_lexicon_tags_2 },
    { 0x536b590a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95cb590d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ab5910, SUC_lexicon_tags_9 },
    { 0xd8cb5911, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceb5916, SUC_lexicon_tags_17 },
    { 0x234b5917, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770b5922, SUC_lexicon_tags_37 },
    { 0xe60b5923, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc5eb5925, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7cb5927, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408b592d, SUC_lexicon_tags_37 },
    { 0xadeb592e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76b5931, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcb5938, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1beb593c, SUC_lexicon_tags_15 },
    { 0xcd8b593c, SUC_lexicon_tags_19 },
    { 0x466b593c, SUC_lexicon_tags_38 },
    { 0x6cb593f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950b5943, SUC_lexicon_tags_14 },
    { 0x8e4b5944, SUC_lexicon_tags_47 },
    { 0x964b5945, SUC_lexicon_tags_60 },
    { 0x9d8b5945, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x682b5948, SUC_lexicon_tags_11 },
    { 0x1dab5949, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0b5950, SUC_lexicon_tags_12 },
    { 0xd2cb5951, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55cb5953, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eab5958, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x20ab595a, SUC_lexicon_tags_151 },
    { 0xf60b595b, SUC_lexicon_tags_47 },
    { 0x668b595b, SUC_lexicon_tags_17 },
    { 0x112b595c, SUC_lexicon_tags_1 },
    { 0xffeb595e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734b5963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0cb5965, SUC_lexicon_tags_1 },
    { 0xb70b5966, SUC_lexicon_tags_17 },
    { 0x8b5967, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe32b5969, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94b5977, SUC_lexicon_tags_37 },
    { 0xe46b5978, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaeb597c, SUC_lexicon_tags_15 },
    { 0x458b597d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5982, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89eb5986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x618b5989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4b5990, SUC_lexicon_tags_23 },
    { 0xaa4b5991, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x262b5994, SUC_lexicon_tags_14 },
    { 0xddeb5995, SUC_lexicon_tags_88 },
    { 0xfaab5996, SUC_lexicon_tags_15 },
    { 0x3e2b5997, SUC_lexicon_tags_11 },
    { 0xdeab5998, SUC_lexicon_tags_31 },
    { 0x48ab5999, SUC_lexicon_tags_38 },
    { 0x59cb5999, SUC_lexicon_tags_26 },
    { 0xf08b599a, SUC_lexicon_tags_23 },
    { 0x8deb5995, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x296b599f, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92b59a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884b59ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1acb59b0, SUC_lexicon_tags_1 },
    { 0x8e2b59b0, SUC_lexicon_tags_6 },
    { 0xca0b59b0, SUC_lexicon_tags_10 },
    { 0x9d4b59b3, SUC_lexicon_tags_0 },
    { 0xd8eb59b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b59bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4b59c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e8b59c9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x538b59cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28b59cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xceeb59d1, SUC_lexicon_tags_15 },
    { 0x7aeb59d2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b59d6, SUC_lexicon_tags_10 },
    { 0x81cb59d7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416b59dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8e6b59df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc08b59e1, SUC_lexicon_tags_11 },
    { 0xf30b59e2, SUC_lexicon_tags_43 },
    { 0x50eb59e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x874b59e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd30b59e8, SUC_lexicon_tags_16 },
    { 0x42cb59e9, SUC_lexicon_tags_151 },
    { 0xc26b59ea, SUC_lexicon_tags_17 },
    { 0x1b2b59eb, SUC_lexicon_tags_17 },
    { 0x78b59e9, SUC_lexicon_tags_16 },
    { 0xb58b59ed, SUC_lexicon_tags_17 },
    { 0x68b59ee, SUC_lexicon_tags_33 },
    { 0x17eb59ec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5c0b59f1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x606b59f3, SUC_lexicon_tags_47 },
    { 0xa32b59f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b59f9, SUC_lexicon_tags_27 },
    { 0x952b59f9, SUC_lexicon_tags_28 },
    { 0xea4b59fb, SUC_lexicon_tags_14 },
    { 0xdf2b59fb, SUC_lexicon_tags_11 },
    { 0xbc0b59fb, SUC_lexicon_tags_2 },
    { 0xaeeb59fd, SUC_lexicon_tags_47 },
    { 0xd2ab59f9, SUC_lexicon_tags_9 },
    { 0x38b59fc, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0xb62b5a02, SUC_lexicon_tags_25 },
    { 0x192b5a02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44b5a0a, SUC_lexicon_tags_11 },
    { 0xbbeb5a0b, SUC_lexicon_tags_2 },
    { 0x1cab5a0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126b5a11, SUC_lexicon_tags_15 },
    { 0xd16b5a12, SUC_lexicon_tags_37 },
    { 0x2c4b5a12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91eb5a15, SUC_lexicon_tags_25 },
    { 0xe54b5a16, SUC_lexicon_tags_11 },
    { 0xba6b5a15, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b5a1c, SUC_lexicon_tags_2 },
    { 0xb96b5a1d, SUC_lexicon_tags_15 },
    { 0x79ab5a1e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdb2b5a20, SUC_lexicon_tags_66 },
    { 0x278b5a20, SUC_lexicon_tags_10 },
    { 0x4e0b5a22, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x890b5a25, SUC_lexicon_tags_86 },
    { 0x58cb5a26, SUC_lexicon_tags_12 },
    { 0x8eeb5a25, SUC_lexicon_tags_63 },
    { 0xeaeb5a27, SUC_lexicon_tags_11 },
    { 0xd14b5a29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920b5a37, SUC_lexicon_tags_37 },
    { 0x894b5a38, SUC_lexicon_tags_14 },
    { 0xfab5a39, SUC_lexicon_tags_1 },
    { 0x2acb5a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802b5a3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0b5a46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa82b5a48, SUC_lexicon_tags_47 },
    { 0xca6b5a49, SUC_lexicon_tags_15 },
    { 0xa74b5a4a, SUC_lexicon_tags_6 },
    { 0xc14b5a4b, SUC_lexicon_tags_29 },
    { 0xbf0b5a4a, SUC_lexicon_tags_45 },
    { 0xf58b5a4a, SUC_lexicon_tags_25 },
    { 0x498b5a4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd8b5a50, SUC_lexicon_tags_15 },
    { 0xabeb5a50, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4b5a56, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aeb5a59, SUC_lexicon_tags_43 },
    { 0x97eb5a59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48eb5a5d, SUC_lexicon_tags_37 },
    { 0x130b5a5e, SUC_lexicon_tags_0 },
    { 0x4aeb5a5f, SUC_lexicon_tags_9 },
    { 0xbe0b5a60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ab5a67, SUC_lexicon_tags_21 },
    { 0x12cb5a68, SUC_lexicon_tags_7 },
    { 0x60cb5a69, SUC_lexicon_tags_55 },
    { 0x534b5a68, SUC_lexicon_tags_46 },
    { 0xb2ab5a6b, SUC_lexicon_tags_33 },
    { 0x836b5a69, SUC_lexicon_tags_27 },
    { 0xd36b5a6d, SUC_lexicon_tags_23 },
    { 0x8d0b5a6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10b5a76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14b5a78, SUC_lexicon_tags_15 },
    { 0x424b5a79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ab5a80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372b5a85, SUC_lexicon_tags_14 },
    { 0xb0ab5a85, SUC_lexicon_tags_12 },
    { 0xb74b5a86, SUC_lexicon_tags_43 },
    { 0x4aab5a85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ab5a8f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ab5a94, SUC_lexicon_tags_12 },
    { 0x4fab5a95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc2b5a97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe20b5a99, SUC_lexicon_tags_13 },
    { 0xc4eb5a9a, SUC_lexicon_tags_7 },
    { 0x306b5a9b, SUC_lexicon_tags_32 },
    { 0xfd0b5a9c, SUC_lexicon_tags_25 },
    { 0x79eb5a9c, SUC_lexicon_tags_45 },
    { 0x604b5a9e, SUC_lexicon_tags_27 },
    { 0x98cb5a9f, SUC_lexicon_tags_35 },
    { 0x224b5aa0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6b5aa3, SUC_lexicon_tags_12 },
    { 0xda4b5aa3, SUC_lexicon_tags_31 },
    { 0x90eb5aa4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ab5aaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb5ab1, SUC_lexicon_tags_13 },
    { 0x3f6b5ab2, SUC_lexicon_tags_80 },
    { 0xf0eb5ab1, SUC_lexicon_tags_0 },
    { 0xc94b5ab4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cb5ab7, SUC_lexicon_tags_37 },
    { 0x882b5ab7, SUC_lexicon_tags_45 },
    { 0x4cab5ab9, SUC_lexicon_tags_31 },
    { 0xfb4b5ab8, SUC_lexicon_tags_2 },
    { 0x72b5abb, SUC_lexicon_tags_23 },
    { 0x2b2b5abb, SUC_lexicon_tags_10 },
    { 0x3beb5abb, SUC_lexicon_tags_15 },
    { 0x528b5abe, SUC_lexicon_tags_2 },
    { 0x284b5abf, SUC_lexicon_tags_17 },
    { 0x536b5ac0, SUC_lexicon_tags_21 },
    { 0xd3cb5ac1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc18b5ac3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x12eb5ac6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbacb5ac9, SUC_lexicon_tags_7 },
    { 0x944b5ac9, SUC_lexicon_tags_2 },
    { 0x64ab5aca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x686b5acd, SUC_lexicon_tags_7 },
    { 0xb30b5ace, SUC_lexicon_tags_37 },
    { 0x454b5acf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0b5ad3, SUC_lexicon_tags_3 },
    { 0x612b5ad4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b5ad9, SUC_lexicon_tags_45 },
    { 0xe5ab5ad9, SUC_lexicon_tags_398 },
    { 0, NULL },
    { 0, NULL },
    { 0x102b5add, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cb5ae0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ab5ae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcb5ae9, SUC_lexicon_tags_66 },
    { 0xecb5aea, SUC_lexicon_tags_25 },
    { 0x10b5aeb, SUC_lexicon_tags_33 },
    { 0xcf0b5ae9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28b5af3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96b5af6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98b5afb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb5aff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06b5b04, SUC_lexicon_tags_1 },
    { 0x26cb5b04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0b5b09, SUC_lexicon_tags_31 },
    { 0x9ceb5b09, SUC_lexicon_tags_37 },
    { 0x7e2b5b09, SUC_lexicon_tags_17 },
    { 0xbb8b5b0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294b5b12, SUC_lexicon_tags_12 },
    { 0x14b5b13, SUC_lexicon_tags_7 },
    { 0x828b5b14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ccb5b16, SUC_lexicon_tags_23 },
    { 0x544b5b16, SUC_lexicon_tags_43 },
    { 0x27cb5b16, SUC_lexicon_tags_37 },
    { 0x1c6b5b19, SUC_lexicon_tags_19 },
    { 0xffeb5b1a, SUC_lexicon_tags_37 },
    { 0x380b5b1b, SUC_lexicon_tags_8 },
    { 0x7ccb5b1a, SUC_lexicon_tags_8 },
    { 0xc02b5b16, SUC_lexicon_tags_10 },
    { 0x3cab5b1b, SUC_lexicon_tags_37 },
    { 0x574b5b1f, SUC_lexicon_tags_0 },
    { 0x688b5b20, SUC_lexicon_tags_12 },
    { 0xd8cb5b21, SUC_lexicon_tags_11 },
    { 0x74b5b22, SUC_lexicon_tags_1 },
    { 0xf4eb5b20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788b5b27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b2b5b29, SUC_lexicon_tags_12 },
    { 0x34b5b29, SUC_lexicon_tags_37 },
    { 0x66cb5b2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6b5b2e, SUC_lexicon_tags_0 },
    { 0x56ab5b2e, SUC_lexicon_tags_19 },
    { 0x784b5b30, SUC_lexicon_tags_15 },
    { 0x272b5b2e, SUC_lexicon_tags_779 },
    { 0x9a2b5b2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916b5b3d, SUC_lexicon_tags_17 },
    { 0x93eb5b3d, SUC_lexicon_tags_1 },
    { 0x29ab5b3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810b5b43, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388b5b47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1eb5b4e, SUC_lexicon_tags_27 },
    { 0xf06b5b4f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54b5b54, SUC_lexicon_tags_25 },
    { 0xdccb5b54, SUC_lexicon_tags_17 },
    { 0xf32b5b56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x414b5b58, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0x9ccb5b5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b5b5d, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816b5b62, SUC_lexicon_tags_25 },
    { 0x594b5b62, SUC_lexicon_tags_38 },
    { 0x766b5b64, SUC_lexicon_tags_16 },
    { 0x57eb5b64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6b5b69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ab5b6d, SUC_lexicon_tags_31 },
    { 0x64eb5b6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacb5b73, SUC_lexicon_tags_11 },
    { 0x616b5b73, SUC_lexicon_tags_66 },
    { 0xd72b5b74, SUC_lexicon_tags_30 },
    { 0x982b5b76, SUC_lexicon_tags_2 },
    { 0x6cb5b77, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b5b7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3ab5b7f, SUC_lexicon_tags_29 },
    { 0xeceb5b80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa24b5b82, SUC_lexicon_tags_37 },
    { 0x5f6b5b82, SUC_lexicon_tags_37 },
    { 0x364b5b84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab5b87, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcacb5b89, SUC_lexicon_tags_68 },
    { 0xb34b5b8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb5b91, SUC_lexicon_tags_31 },
    { 0x252b5b92, SUC_lexicon_tags_56 },
    { 0x842b5b93, SUC_lexicon_tags_29 },
    { 0x96eb5b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb5b96, SUC_lexicon_tags_37 },
    { 0xdeeb5b97, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eeb5b9a, SUC_lexicon_tags_37 },
    { 0xa90b5b9b, SUC_lexicon_tags_25 },
    { 0xb22b5b9c, SUC_lexicon_tags_2 },
    { 0xd58b5b9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdd0b5b9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4b5ba2, SUC_lexicon_tags_25 },
    { 0xdcb5ba3, SUC_lexicon_tags_12 },
    { 0x6eeb5ba3, SUC_lexicon_tags_2 },
    { 0xeeab5ba2, SUC_lexicon_tags_11 },
    { 0xbf4b5ba2, SUC_lexicon_tags_2 },
    { 0x92ab5ba5, SUC_lexicon_tags_33 },
    { 0x3c4b5ba8, SUC_lexicon_tags_1 },
    { 0xf8ab5ba9, SUC_lexicon_tags_15 },
    { 0x9e6b5baa, SUC_lexicon_tags_14 },
    { 0xe76b5ba9, SUC_lexicon_tags_19 },
    { 0x814b5bac, SUC_lexicon_tags_134 },
    { 0x9f2b5bad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb5bb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88b5bb4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x168b5bb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b5bb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4b5bbd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734b5bc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x448b5bc5, SUC_lexicon_tags_17 },
    { 0x440b5bc6, SUC_lexicon_tags_4 },
    { 0xe92b5bc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd82b5bc9, SUC_lexicon_tags_0 },
    { 0x72b5bca, SUC_lexicon_tags_32 },
    { 0x360b5bcb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x382b5bce, SUC_lexicon_tags_38 },
    { 0x860b5bcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774b5bd3, SUC_lexicon_tags_14 },
    { 0x9a2b5bd4, SUC_lexicon_tags_1 },
    { 0xf2cb5bd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b5bd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6eb5bda, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2b5bdd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfceb5be1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68b5beb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a0b5bed, SUC_lexicon_tags_11 },
    { 0x58b5bee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cb5bf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d0b5bf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x63ab5bf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2b5bf9, SUC_lexicon_tags_27 },
    { 0xb3ab5bfa, SUC_lexicon_tags_21 },
    { 0xee0b5bfb, SUC_lexicon_tags_1 },
    { 0x230b5bf9, SUC_lexicon_tags_1 },
    { 0x15ab5bfd, SUC_lexicon_tags_37 },
    { 0x516b5bfe, SUC_lexicon_tags_68 },
    { 0x22cb5bfa, SUC_lexicon_tags_77 },
    { 0x1ecb5c00, SUC_lexicon_tags_47 },
    { 0xa58b5bff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4b5c06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccb5c0a, SUC_lexicon_tags_4 },
    { 0x798b5c0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x88eb5c0e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb4b5c10, SUC_lexicon_tags_49 },
    { 0xd38b5c11, SUC_lexicon_tags_37 },
    { 0xae6b5c12, SUC_lexicon_tags_1 },
    { 0x2b6b5c12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9deb5c16, SUC_lexicon_tags_17 },
    { 0x3e4b5c17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580b5c1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cb5c21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4b5c24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784b5c29, SUC_lexicon_tags_97 },
    { 0x81cb5c29, SUC_lexicon_tags_7 },
    { 0x4e4b5c2b, SUC_lexicon_tags_43 },
    { 0xd64b5c2c, SUC_lexicon_tags_13 },
    { 0x630b5c2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cb5c31, SUC_lexicon_tags_17 },
    { 0xa40b5c32, SUC_lexicon_tags_15 },
    { 0x712b5c32, SUC_lexicon_tags_1 },
    { 0xd5cb5c34, SUC_lexicon_tags_2 },
    { 0x4ab5c34, SUC_lexicon_tags_68 },
    { 0x558b5c36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb94b5c38, SUC_lexicon_tags_45 },
    { 0xc04b5c38, SUC_lexicon_tags_85 },
    { 0x798b5c38, SUC_lexicon_tags_17 },
    { 0xe76b5c3b, SUC_lexicon_tags_25 },
    { 0x77eb5c39, SUC_lexicon_tags_82 },
    { 0x7cab5c3d, SUC_lexicon_tags_32 },
    { 0x2a8b5c3a, SUC_lexicon_tags_68 },
    { 0x340b5c3f, SUC_lexicon_tags_16 },
    { 0x960b5c40, SUC_lexicon_tags_0 },
    { 0x534b5c40, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x376b5c44, SUC_lexicon_tags_11 },
    { 0xf44b5c45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfaeb5c47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92b5c4a, SUC_lexicon_tags_26 },
    { 0xad4b5c4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcb5c4f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6b5c56, SUC_lexicon_tags_34 },
    { 0x750b5c56, SUC_lexicon_tags_13 },
    { 0x5c6b5c58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x424b5c5a, SUC_lexicon_tags_43 },
    { 0x84cb5c5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x754b5c5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdccb5c60, SUC_lexicon_tags_25 },
    { 0x48eb5c60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa16b5c63, SUC_lexicon_tags_35 },
    { 0x9ecb5c64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2b5c6a, SUC_lexicon_tags_26 },
    { 0xf94b5c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6b5c71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4b5c74, SUC_lexicon_tags_9 },
    { 0xdbab5c75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984b5c7a, SUC_lexicon_tags_48 },
    { 0x9a4b5c7b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x948b5c7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc3ab5c7f, SUC_lexicon_tags_2 },
    { 0x1a8b5c7f, SUC_lexicon_tags_23 },
    { 0x2f6b5c81, SUC_lexicon_tags_586 },
    { 0x7eab5c80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b5c96, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x780b5c98, SUC_lexicon_tags_26 },
    { 0x742b5c99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aab5c9c, SUC_lexicon_tags_14 },
    { 0x2f0b5c9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3aeb5c9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968b5ca4, SUC_lexicon_tags_27 },
    { 0xb8b5ca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2b5caa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cb5cad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7d0b5caf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6b5cbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6b5cc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6eb5cc7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380b5ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bab5cd0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x586b5cd3, SUC_lexicon_tags_0 },
    { 0xcdeb5cd4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa6b5cd6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd54b5cd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6b5cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8eb5cde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x98cb5ce1, SUC_lexicon_tags_29 },
    { 0xff8b5ce1, SUC_lexicon_tags_11 },
    { 0x166b5ce1, SUC_lexicon_tags_15 },
    { 0xd2cb5ce4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4b5ce7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe14b5ce9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0b5cec, SUC_lexicon_tags_37 },
    { 0x5eeb5cec, SUC_lexicon_tags_44 },
    { 0xd74b5cee, SUC_lexicon_tags_2 },
    { 0x85ab5cec, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x53ab5cf1, SUC_lexicon_tags_2 },
    { 0x15ab5cf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0b5cf5, SUC_lexicon_tags_10 },
    { 0x538b5cf5, SUC_lexicon_tags_63 },
    { 0xb70b5cf7, SUC_lexicon_tags_31 },
    { 0x86b5cf7, SUC_lexicon_tags_7 },
    { 0xe6cb5cf7, SUC_lexicon_tags_17 },
    { 0x618b5cf7, SUC_lexicon_tags_0 },
    { 0x8beb5cf5, SUC_lexicon_tags_47 },
    { 0x1cab5cfc, SUC_lexicon_tags_37 },
    { 0x3c8b5cfd, SUC_lexicon_tags_2 },
    { 0xe8cb5cfe, SUC_lexicon_tags_19 },
    { 0xb42b5cfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ab5d02, SUC_lexicon_tags_0 },
    { 0xed8b5d03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa72b5d05, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cb5d0c, SUC_lexicon_tags_7 },
    { 0xa9ab5d0d, SUC_lexicon_tags_2 },
    { 0x3b2b5d0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374b5d13, SUC_lexicon_tags_25 },
    { 0x1d8b5d13, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170b5d1e, SUC_lexicon_tags_43 },
    { 0xe3ab5d1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d0b5d21, SUC_lexicon_tags_21 },
    { 0x762b5d22, SUC_lexicon_tags_17 },
    { 0xa3ab5d23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84b5d25, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x588b5d27, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x79cb5d29, SUC_lexicon_tags_28 },
    { 0x90cb5d29, SUC_lexicon_tags_40 },
    { 0x6aeb5d2b, SUC_lexicon_tags_11 },
    { 0x13cb5d2c, SUC_lexicon_tags_1 },
    { 0xdab5d2d, SUC_lexicon_tags_15 },
    { 0x3ecb5d2d, SUC_lexicon_tags_6 },
    { 0xef0b5d2c, SUC_lexicon_tags_37 },
    { 0x80eb5d30, SUC_lexicon_tags_2 },
    { 0x7ceb5d2e, SUC_lexicon_tags_15 },
    { 0x786b5d32, SUC_lexicon_tags_17 },
    { 0xf7cb5d2d, SUC_lexicon_tags_72 },
    { 0x30eb5d34, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984b5d3d, SUC_lexicon_tags_11 },
    { 0xefcb5d3e, SUC_lexicon_tags_79 },
    { 0x742b5d3f, SUC_lexicon_tags_17 },
    { 0xcb0b5d40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb5d49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ab5d4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90b5d50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f4b5d52, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccb5d55, SUC_lexicon_tags_7 },
    { 0xc28b5d56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ab5d5c, SUC_lexicon_tags_47 },
    { 0xc4ab5d5c, SUC_lexicon_tags_11 },
    { 0xfe6b5d5d, SUC_lexicon_tags_7 },
    { 0x836b5d5c, SUC_lexicon_tags_15 },
    { 0x50b5d5e, SUC_lexicon_tags_11 },
    { 0xe14b5d5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x38cb5d63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d4b5d65, SUC_lexicon_tags_61 },
    { 0x306b5d66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5eb5d6c, SUC_lexicon_tags_63 },
    { 0x378b5d6d, SUC_lexicon_tags_343 },
    { 0xdcab5d6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04b5d73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x636b5d75, SUC_lexicon_tags_12 },
    { 0x498b5d75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8b5d79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbb0b5d7b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7cb5d7d, SUC_lexicon_tags_0 },
    { 0x26b5d7e, SUC_lexicon_tags_63 },
    { 0x3cab5d7e, SUC_lexicon_tags_9 },
    { 0xab0b5d80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72eb5d83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ab5d88, SUC_lexicon_tags_21 },
    { 0x406b5d89, SUC_lexicon_tags_15 },
    { 0x83eb5d8a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d8f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86b5d92, SUC_lexicon_tags_8 },
    { 0xd86b5d92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90b5d9b, SUC_lexicon_tags_14 },
    { 0x2ab5d9b, SUC_lexicon_tags_26 },
    { 0xf1ab5d9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acb5da2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556b5da6, SUC_lexicon_tags_37 },
    { 0xd8ab5da7, SUC_lexicon_tags_31 },
    { 0x1c0b5da8, SUC_lexicon_tags_140 },
    { 0x822b5da9, SUC_lexicon_tags_37 },
    { 0x4b2b5daa, SUC_lexicon_tags_32 },
    { 0x348b5dab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04b5db5, SUC_lexicon_tags_31 },
    { 0xf40b5db6, SUC_lexicon_tags_8 },
    { 0xa54b5db7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd0b5db9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2b5dbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35eb5dc0, SUC_lexicon_tags_17 },
    { 0x824b5dc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x494b5dc4, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x946b5dc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2deb5dc8, SUC_lexicon_tags_30 },
    { 0x596b5dc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x374b5dcc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2b5dd0, SUC_lexicon_tags_19 },
    { 0xcfab5dd1, SUC_lexicon_tags_87 },
    { 0x186b5dd2, SUC_lexicon_tags_10 },
    { 0x9eeb5dd0, SUC_lexicon_tags_11 },
    { 0xc8b5dd3, SUC_lexicon_tags_23 },
    { 0xbe4b5dd5, SUC_lexicon_tags_39 },
    { 0xbccb5dd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22b5dd9, SUC_lexicon_tags_11 },
    { 0x2a4b5dda, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x3d4b5ddc, SUC_lexicon_tags_8 },
    { 0x66b5ddc, SUC_lexicon_tags_0 },
    { 0x5dcb5dde, SUC_lexicon_tags_11 },
    { 0x244b5ddf, SUC_lexicon_tags_11 },
    { 0x658b5ddf, SUC_lexicon_tags_25 },
    { 0x516b5ddf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6b5de4, SUC_lexicon_tags_23 },
    { 0x8b4b5de5, SUC_lexicon_tags_47 },
    { 0x74cb5de6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18b5dea, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x7a6b5dec, SUC_lexicon_tags_19 },
    { 0x478b5dec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6b5df0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x520b5df2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecb5dfa, SUC_lexicon_tags_0 },
    { 0xf64b5dfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2b5dfe, SUC_lexicon_tags_27 },
    { 0xde6b5dfe, SUC_lexicon_tags_7 },
    { 0x41ab5dff, SUC_lexicon_tags_7 },
    { 0x2f8b5dff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x958b5e04, SUC_lexicon_tags_84 },
    { 0x886b5e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cb5e09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cb5e0d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf68b5e0f, SUC_lexicon_tags_3 },
    { 0xe4b5e0f, SUC_lexicon_tags_15 },
    { 0xe74b5e10, SUC_lexicon_tags_15 },
    { 0x746b5e11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc0b5e14, SUC_lexicon_tags_26 },
    { 0xd6eb5e15, SUC_lexicon_tags_27 },
    { 0x70cb5e16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cab5e18, SUC_lexicon_tags_32 },
    { 0x14eb5e19, SUC_lexicon_tags_43 },
    { 0x1ccb5e1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccb5e1d, SUC_lexicon_tags_11 },
    { 0xe6b5e1d, SUC_lexicon_tags_29 },
    { 0x326b5e1e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28b5e23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62b5e27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7deb5e29, SUC_lexicon_tags_1 },
    { 0xf7cb5e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91cb5e2c, SUC_lexicon_tags_12 },
    { 0xf2cb5e2d, SUC_lexicon_tags_12 },
    { 0x296b5e2e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eb5e33, SUC_lexicon_tags_11 },
    { 0xb16b5e33, SUC_lexicon_tags_47 },
    { 0xa44b5e34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6b5e3a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf12b5e3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb5e3f, SUC_lexicon_tags_25 },
    { 0x82b5e3f, SUC_lexicon_tags_107 },
    { 0x12ab5e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b5e48, SUC_lexicon_tags_8 },
    { 0x17eb5e49, SUC_lexicon_tags_3 },
    { 0x75ab5e4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eb5e52, SUC_lexicon_tags_47 },
    { 0xccb5e53, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cb5e58, SUC_lexicon_tags_25 },
    { 0x9a4b5e59, SUC_lexicon_tags_37 },
    { 0xd7ab5e59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eb5e6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cb5e75, SUC_lexicon_tags_65 },
    { 0x59ab5e76, SUC_lexicon_tags_38 },
    { 0xf4cb5e75, SUC_lexicon_tags_15 },
    { 0x8cab5e78, SUC_lexicon_tags_17 },
    { 0xcb8b5e78, SUC_lexicon_tags_7 },
    { 0xdc6b5e7a, SUC_lexicon_tags_14 },
    { 0xe6ab5e78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1fab5e7d, SUC_lexicon_tags_4 },
    { 0xa5eb5e7e, SUC_lexicon_tags_27 },
    { 0x390b5e7f, SUC_lexicon_tags_29 },
    { 0xee0b5e7e, SUC_lexicon_tags_38 },
    { 0xe16b5e81, SUC_lexicon_tags_87 },
    { 0x76ab5e7e, SUC_lexicon_tags_12 },
    { 0xc68b5e83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6b5e88, SUC_lexicon_tags_17 },
    { 0xb08b5e88, SUC_lexicon_tags_11 },
    { 0x2ecb5e8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4b5e8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc76b5e91, SUC_lexicon_tags_6 },
    { 0x550b5e92, SUC_lexicon_tags_45 },
    { 0x41ab5e91, SUC_lexicon_tags_47 },
    { 0x570b5e94, SUC_lexicon_tags_12 },
    { 0xd64b5e92, SUC_lexicon_tags_12 },
    { 0x578b5e95, SUC_lexicon_tags_12 },
    { 0x9d6b5e91, SUC_lexicon_tags_29 },
    { 0xf88b5e98, SUC_lexicon_tags_82 },
    { 0x156b5e99, SUC_lexicon_tags_7 },
    { 0x12b5e99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc22b5e9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3feb5e9f, SUC_lexicon_tags_1 },
    { 0x6f8b5ea0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3b8b5ea2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cb5ea7, SUC_lexicon_tags_9 },
    { 0x1aeb5ea8, SUC_lexicon_tags_11 },
    { 0x15ab5ea8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x640b5eab, SUC_lexicon_tags_23 },
    { 0x700b5eac, SUC_lexicon_tags_15 },
    { 0xc2eb5eac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cb5eb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x900b5eb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28b5eb6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2ab5eb8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ceb5ebb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75eb5ec1, SUC_lexicon_tags_0 },
    { 0xdb2b5ec2, SUC_lexicon_tags_15 },
    { 0xb90b5ec3, SUC_lexicon_tags_108 },
    { 0xfcb5ec1, SUC_lexicon_tags_25 },
    { 0xfc8b5ec2, SUC_lexicon_tags_1 },
    { 0x9a0b5ec6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebab5eca, SUC_lexicon_tags_2 },
    { 0xd82b5eca, SUC_lexicon_tags_49 },
    { 0x844b5ecc, SUC_lexicon_tags_0 },
    { 0xabeb5ecd, SUC_lexicon_tags_14 },
    { 0xcdab5ece, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0b5ed2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ab5ed8, SUC_lexicon_tags_87 },
    { 0xcd8b5ed8, SUC_lexicon_tags_77 },
    { 0x844b5ed9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8b5edd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52b5ee0, SUC_lexicon_tags_0 },
    { 0xa74b5ee1, SUC_lexicon_tags_79 },
    { 0xeceb5ee2, SUC_lexicon_tags_29 },
    { 0xfb8b5ee1, SUC_lexicon_tags_2 },
    { 0xd10b5ee4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ab5ee7, SUC_lexicon_tags_2 },
    { 0xbd4b5ee7, SUC_lexicon_tags_26 },
    { 0x872b5ee8, SUC_lexicon_tags_184 },
    { 0x816b5eea, SUC_lexicon_tags_1 },
    { 0xac2b5eeb, SUC_lexicon_tags_12 },
    { 0xdf6b5eec, SUC_lexicon_tags_1 },
    { 0xed8b5eed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cb5ef6, SUC_lexicon_tags_17 },
    { 0x4acb5ef6, SUC_lexicon_tags_6 },
    { 0xfeab5ef6, SUC_lexicon_tags_37 },
    { 0xeeeb5ef9, SUC_lexicon_tags_37 },
    { 0xf8cb5ef7, SUC_lexicon_tags_25 },
    { 0x5ab5efb, SUC_lexicon_tags_63 },
    { 0xef8b5ef6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30b5f00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb5f08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x902b5f0a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2b5f0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x344b5f10, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xd5eb5f12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13ab5f14, SUC_lexicon_tags_28 },
    { 0x200b5f15, SUC_lexicon_tags_23 },
    { 0xfb2b5f14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcb5f1c, SUC_lexicon_tags_15 },
    { 0x526b5f1d, SUC_lexicon_tags_0 },
    { 0x2e0b5f1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x904b5f20, SUC_lexicon_tags_6 },
    { 0x9aab5f21, SUC_lexicon_tags_25 },
    { 0x274b5f22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd7ab5f24, SUC_lexicon_tags_2 },
    { 0xac8b5f24, SUC_lexicon_tags_3 },
    { 0x500b5f26, SUC_lexicon_tags_23 },
    { 0xfceb5f25, SUC_lexicon_tags_15 },
    { 0x476b5f28, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664b5f2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0b5f30, SUC_lexicon_tags_12 },
    { 0xafeb5f31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50eb5f33, SUC_lexicon_tags_29 },
    { 0xa5cb5f34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53eb5f36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cb5f39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x568b5f3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a4b5f3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0b5f40, SUC_lexicon_tags_37 },
    { 0x92b5f41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48b5f46, SUC_lexicon_tags_87 },
    { 0xd5eb5f47, SUC_lexicon_tags_47 },
    { 0x7dcb5f47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe90b5f4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5ab5f4c, SUC_lexicon_tags_37 },
    { 0x424b5f4d, SUC_lexicon_tags_25 },
    { 0x2eeb5f4e, SUC_lexicon_tags_60 },
    { 0x106b5f4f, SUC_lexicon_tags_4 },
    { 0xb0cb5f4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77cb5f52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726b5f58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x80ab5f5a, SUC_lexicon_tags_68 },
    { 0x8c0b5f5a, SUC_lexicon_tags_17 },
    { 0x248b5f5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48eb5f5f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514b5f68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8b5f6c, SUC_lexicon_tags_49 },
    { 0x812b5f6d, SUC_lexicon_tags_38 },
    { 0xa36b5f6c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2b5f73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50b5f76, SUC_lexicon_tags_34 },
    { 0xf90b5f77, SUC_lexicon_tags_10 },
    { 0x5ccb5f78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc82b5f7a, SUC_lexicon_tags_0 },
    { 0xadeb5f7b, SUC_lexicon_tags_37 },
    { 0x122b5f7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb5f7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab5f82, SUC_lexicon_tags_6 },
    { 0x1f0b5f82, SUC_lexicon_tags_27 },
    { 0xdd8b5f83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742b5f8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ab5f8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8b5f93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40b5f96, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882b5f9e, SUC_lexicon_tags_3 },
    { 0xfb6b5f9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x120b5fa1, SUC_lexicon_tags_40 },
    { 0x88ab5fa1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa5eb5fa4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab5fa7, SUC_lexicon_tags_57 },
    { 0xd68b5fa8, SUC_lexicon_tags_32 },
    { 0x2ecb5fa9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe70b5fab, SUC_lexicon_tags_37 },
    { 0x6b5fab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4b5faf, SUC_lexicon_tags_7 },
    { 0x6d2b5fb0, SUC_lexicon_tags_61 },
    { 0x3b2b5fb1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38b5fb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cb5fbb, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xfc0b5fbd, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b5fc0, SUC_lexicon_tags_15 },
    { 0x566b5fc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeb5fc6, SUC_lexicon_tags_53 },
    { 0xf16b5fc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0b5fcc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe44b5fce, SUC_lexicon_tags_33 },
    { 0xd56b5fce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58eb5fd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x43eb5fd6, SUC_lexicon_tags_37 },
    { 0xadcb5fd7, SUC_lexicon_tags_47 },
    { 0xeacb5fd8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78b5fdb, SUC_lexicon_tags_37 },
    { 0xb30b5fdc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8b5fe0, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75eb5fe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cb5fea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x392b5fec, SUC_lexicon_tags_1 },
    { 0xc26b5fec, SUC_lexicon_tags_55 },
    { 0xf66b5fee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b5ff4, SUC_lexicon_tags_17 },
    { 0x3b4b5ff4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf8b5ff7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x110b5ffa, SUC_lexicon_tags_1 },
    { 0x954b5ffb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808b5fff, SUC_lexicon_tags_82 },
    { 0x4a6b6000, SUC_lexicon_tags_30 },
    { 0xfeb6001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ab6004, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcb6008, SUC_lexicon_tags_4 },
    { 0x736b6009, SUC_lexicon_tags_5 },
    { 0x654b600a, SUC_lexicon_tags_45 },
    { 0x49ab600b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae0b600d, SUC_lexicon_tags_45 },
    { 0x86cb600e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13eb601a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32b601d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeb2b601f, SUC_lexicon_tags_13 },
    { 0xfeb6020, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3eb6025, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x714b6027, SUC_lexicon_tags_23 },
    { 0x2d4b6028, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40b602f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728b6035, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cb6039, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aab603f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ccb6041, SUC_lexicon_tags_45 },
    { 0xc6cb6042, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd90b6044, SUC_lexicon_tags_1 },
    { 0x476b6045, SUC_lexicon_tags_11 },
    { 0x76ab6045, SUC_lexicon_tags_37 },
    { 0x330b6044, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0b604b, SUC_lexicon_tags_17 },
    { 0xcbeb604c, SUC_lexicon_tags_9 },
    { 0x6aeb604d, SUC_lexicon_tags_30 },
    { 0x250b604e, SUC_lexicon_tags_17 },
    { 0xa80b604d, SUC_lexicon_tags_49 },
    { 0xef2b6050, SUC_lexicon_tags_15 },
    { 0x1f0b6050, SUC_lexicon_tags_26 },
    { 0x3fcb6052, SUC_lexicon_tags_15 },
    { 0xb40b6052, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x368b6055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eb6062, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ccb6064, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eb6067, SUC_lexicon_tags_11 },
    { 0x816b6067, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0b606b, SUC_lexicon_tags_14 },
    { 0x976b606c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3eb606f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448b6073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac4b6075, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68eb6077, SUC_lexicon_tags_66 },
    { 0x2aeb6078, SUC_lexicon_tags_7 },
    { 0xa70b6079, SUC_lexicon_tags_18 },
    { 0x63cb6079, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31cb607d, SUC_lexicon_tags_4 },
    { 0xe04b607d, SUC_lexicon_tags_26 },
    { 0x60eb607d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22b6082, SUC_lexicon_tags_15 },
    { 0xb3eb6082, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cb6086, SUC_lexicon_tags_25 },
    { 0x88cb6087, SUC_lexicon_tags_27 },
    { 0xd3cb6086, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6b608d, SUC_lexicon_tags_19 },
    { 0x246b608e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x344b6090, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72b6094, SUC_lexicon_tags_43 },
    { 0xde6b6095, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2b6099, SUC_lexicon_tags_19 },
    { 0x112b609a, SUC_lexicon_tags_6 },
    { 0xe9cb6099, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x686b609d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcb60a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b6b60a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabeb60ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190b60b2, SUC_lexicon_tags_7 },
    { 0x28ab60b3, SUC_lexicon_tags_17 },
    { 0x328b60b2, SUC_lexicon_tags_55 },
    { 0xb5ab60b5, SUC_lexicon_tags_15 },
    { 0xeab60b6, SUC_lexicon_tags_61 },
    { 0xe50b60b7, SUC_lexicon_tags_38 },
    { 0x552b60b8, SUC_lexicon_tags_23 },
    { 0x186b60b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08b60be, SUC_lexicon_tags_37 },
    { 0xb3cb60bf, SUC_lexicon_tags_8 },
    { 0xe38b60c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cb60c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x602b60cb, SUC_lexicon_tags_26 },
    { 0xf6cb60cc, SUC_lexicon_tags_17 },
    { 0xdb4b60cd, SUC_lexicon_tags_187 },
    { 0x22b60cc, SUC_lexicon_tags_15 },
    { 0xb1ab60cb, SUC_lexicon_tags_15 },
    { 0x70cb60d0, SUC_lexicon_tags_12 },
    { 0x17cb60d1, SUC_lexicon_tags_25 },
    { 0x5eb60d0, SUC_lexicon_tags_25 },
    { 0x3ccb60d3, SUC_lexicon_tags_1 },
    { 0x576b60d3, SUC_lexicon_tags_11 },
    { 0x8aeb60d3, SUC_lexicon_tags_79 },
    { 0xea4b60d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf62b60d8, SUC_lexicon_tags_43 },
    { 0x18cb60d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ab60db, SUC_lexicon_tags_14 },
    { 0xd14b60dc, SUC_lexicon_tags_0 },
    { 0x2d2b60db, SUC_lexicon_tags_17 },
    { 0x89eb60db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8b60e1, SUC_lexicon_tags_55 },
    { 0x19ab60e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x352b60e4, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x146b60e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d6b60e8, SUC_lexicon_tags_38 },
    { 0x268b60e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90b60ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40b60f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdab60f4, SUC_lexicon_tags_1 },
    { 0x284b60f5, SUC_lexicon_tags_11 },
    { 0xe90b60f5, SUC_lexicon_tags_7 },
    { 0x488b60f7, SUC_lexicon_tags_0 },
    { 0xb5ab60f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x812b60fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcceb60fc, SUC_lexicon_tags_16 },
    { 0x53eb60fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24b6101, SUC_lexicon_tags_23 },
    { 0xcb8b6102, SUC_lexicon_tags_23 },
    { 0xd66b6103, SUC_lexicon_tags_0 },
    { 0xc86b6103, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2b610a, SUC_lexicon_tags_23 },
    { 0x6feb610b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cb610e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30b6111, SUC_lexicon_tags_25 },
    { 0x64cb6112, SUC_lexicon_tags_25 },
    { 0x39ab6113, SUC_lexicon_tags_8 },
    { 0x3c0b6112, SUC_lexicon_tags_68 },
    { 0xe76b6115, SUC_lexicon_tags_60 },
    { 0x106b6113, SUC_lexicon_tags_8 },
    { 0x9aeb6115, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb6119, SUC_lexicon_tags_2 },
    { 0x148b6119, SUC_lexicon_tags_9 },
    { 0xd8b611b, SUC_lexicon_tags_737 },
    { 0xd22b611c, SUC_lexicon_tags_1 },
    { 0x2eb611c, SUC_lexicon_tags_89 },
    { 0x9acb611c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8b6123, SUC_lexicon_tags_7 },
    { 0x5c2b6124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260b6128, SUC_lexicon_tags_15 },
    { 0xc90b6129, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecb612d, SUC_lexicon_tags_47 },
    { 0x224b612d, SUC_lexicon_tags_6 },
    { 0xafeb612f, SUC_lexicon_tags_30 },
    { 0xef4b612f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4b6138, SUC_lexicon_tags_55 },
    { 0x716b6138, SUC_lexicon_tags_30 },
    { 0x706b6138, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cb6141, SUC_lexicon_tags_28 },
    { 0x162b6142, SUC_lexicon_tags_8 },
    { 0x1c8b6142, SUC_lexicon_tags_25 },
    { 0xc72b6144, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4b614f, SUC_lexicon_tags_47 },
    { 0xe8cb6150, SUC_lexicon_tags_17 },
    { 0xbb2b6151, SUC_lexicon_tags_87 },
    { 0x7eb6151, SUC_lexicon_tags_14 },
    { 0x406b6150, SUC_lexicon_tags_63 },
    { 0xb04b614f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa3eb6156, SUC_lexicon_tags_12 },
    { 0xdeb6157, SUC_lexicon_tags_27 },
    { 0xfc4b6156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4ab615a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c4b615c, SUC_lexicon_tags_60 },
    { 0xb0b615c, SUC_lexicon_tags_38 },
    { 0x726b615d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab6163, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3deb6165, SUC_lexicon_tags_25 },
    { 0x35ab6166, SUC_lexicon_tags_611 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94b616b, SUC_lexicon_tags_8 },
    { 0x290b616c, SUC_lexicon_tags_17 },
    { 0xacb616d, SUC_lexicon_tags_38 },
    { 0xa8cb616e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14b6174, SUC_lexicon_tags_13 },
    { 0x25cb6175, SUC_lexicon_tags_15 },
    { 0x158b6174, SUC_lexicon_tags_63 },
    { 0x152b6174, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398b617b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb6180, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eb6183, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab6186, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f0b6188, SUC_lexicon_tags_29 },
    { 0xf86b6188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfcb618b, SUC_lexicon_tags_8 },
    { 0x9cab618c, SUC_lexicon_tags_0 },
    { 0x852b618d, SUC_lexicon_tags_39 },
    { 0x63eb618c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6b6193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6b6196, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d4b6198, SUC_lexicon_tags_510 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cb619c, SUC_lexicon_tags_6 },
    { 0xd26b619c, SUC_lexicon_tags_49 },
    { 0x302b619d, SUC_lexicon_tags_2 },
    { 0xc5eb619d, SUC_lexicon_tags_27 },
    { 0xe44b619e, SUC_lexicon_tags_55 },
    { 0x422b61a1, SUC_lexicon_tags_15 },
    { 0x2dcb61a1, SUC_lexicon_tags_104 },
    { 0x3c2b61a3, SUC_lexicon_tags_77 },
    { 0x212b619d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2eb61a9, SUC_lexicon_tags_43 },
    { 0x54b61aa, SUC_lexicon_tags_6 },
    { 0x928b61ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x318b61ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x136b61b0, SUC_lexicon_tags_60 },
    { 0x932b61b1, SUC_lexicon_tags_15 },
    { 0x800b61b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb61b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3cb61b8, SUC_lexicon_tags_30 },
    { 0x196b61b8, SUC_lexicon_tags_1 },
    { 0x4d2b61b8, SUC_lexicon_tags_7 },
    { 0x88b61bb, SUC_lexicon_tags_27 },
    { 0x694b61bc, SUC_lexicon_tags_54 },
    { 0x480b61bb, SUC_lexicon_tags_7 },
    { 0x1bcb61bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366b61c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb61c6, SUC_lexicon_tags_14 },
    { 0x194b61c7, SUC_lexicon_tags_25 },
    { 0xb02b61c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232b61ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xbfcb61d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0b61d3, SUC_lexicon_tags_7 },
    { 0x622b61d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9deb61da, SUC_lexicon_tags_27 },
    { 0xb2cb61da, SUC_lexicon_tags_6 },
    { 0x582b61dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2b61df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94b61e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x850b61e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x47eb61e8, SUC_lexicon_tags_11 },
    { 0x888b61e8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500b61ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x108b61f1, SUC_lexicon_tags_49 },
    { 0xa9ab61f2, SUC_lexicon_tags_25 },
    { 0x4a2b61f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6b61f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4b61fb, SUC_lexicon_tags_17 },
    { 0xcf0b61fc, SUC_lexicon_tags_4 },
    { 0x978b61fc, SUC_lexicon_tags_49 },
    { 0xeb2b61fe, SUC_lexicon_tags_19 },
    { 0x194b61ff, SUC_lexicon_tags_47 },
    { 0xac0b6200, SUC_lexicon_tags_14 },
    { 0xae4b6201, SUC_lexicon_tags_12 },
    { 0x60eb6201, SUC_lexicon_tags_37 },
    { 0xcb2b6201, SUC_lexicon_tags_2 },
    { 0x16eb6204, SUC_lexicon_tags_2 },
    { 0xaeeb61fe, SUC_lexicon_tags_39 },
    { 0xfc0b61fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4b6211, SUC_lexicon_tags_21 },
    { 0x50ab6211, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x312b6214, SUC_lexicon_tags_43 },
    { 0x27ab6215, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x294b6218, SUC_lexicon_tags_38 },
    { 0xe4eb6219, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cb621d, SUC_lexicon_tags_2 },
    { 0xdb6b621e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cb6223, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x344b6225, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0b622c, SUC_lexicon_tags_25 },
    { 0xc34b622d, SUC_lexicon_tags_37 },
    { 0x9b2b622d, SUC_lexicon_tags_7 },
    { 0x1cab622f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cb6235, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8b623c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x202b623e, SUC_lexicon_tags_11 },
    { 0x604b623e, SUC_lexicon_tags_37 },
    { 0x5a8b6240, SUC_lexicon_tags_43 },
    { 0xc02b6241, SUC_lexicon_tags_12 },
    { 0x1cb6240, SUC_lexicon_tags_14 },
    { 0x5ab6241, SUC_lexicon_tags_11 },
    { 0x7f2b6242, SUC_lexicon_tags_2 },
    { 0xf98b6241, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9eb6247, SUC_lexicon_tags_49 },
    { 0x468b6248, SUC_lexicon_tags_10 },
    { 0x682b6248, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8cb624b, SUC_lexicon_tags_43 },
    { 0x646b624c, SUC_lexicon_tags_1 },
    { 0xab4b624c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32b6252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcceb6256, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6b6259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cb625e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8ab6260, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6b6264, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ab626a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ceb6272, SUC_lexicon_tags_37 },
    { 0x154b6272, SUC_lexicon_tags_31 },
    { 0xa5eb6273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500b627a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cb627e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b0b6280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ab6283, SUC_lexicon_tags_83 },
    { 0x2e8b6284, SUC_lexicon_tags_17 },
    { 0x41eb6285, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x906b6287, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87ab6289, SUC_lexicon_tags_27 },
    { 0x610b6289, SUC_lexicon_tags_31 },
    { 0x38b6289, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x970b628d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d0b628f, SUC_lexicon_tags_37 },
    { 0x542b6290, SUC_lexicon_tags_17 },
    { 0x732b6291, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacb6296, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb629b, SUC_lexicon_tags_7 },
    { 0xb04b629c, SUC_lexicon_tags_2 },
    { 0x1a6b629d, SUC_lexicon_tags_37 },
    { 0x2c4b629c, SUC_lexicon_tags_15 },
    { 0xa4ab629f, SUC_lexicon_tags_44 },
    { 0x518b629f, SUC_lexicon_tags_2 },
    { 0x678b62a1, SUC_lexicon_tags_1 },
    { 0x322b62a2, SUC_lexicon_tags_0 },
    { 0x384b629f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x446b62a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefab62a8, SUC_lexicon_tags_34 },
    { 0x684b62a9, SUC_lexicon_tags_37 },
    { 0xe08b62a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5eb62ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d4b62ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4b62b2, SUC_lexicon_tags_25 },
    { 0xc0eb62b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6b62b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdab62be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40b62c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d4b62c5, SUC_lexicon_tags_37 },
    { 0x498b62c5, SUC_lexicon_tags_21 },
    { 0x5b0b62c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeeab62c9, SUC_lexicon_tags_49 },
    { 0xff4b62ca, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876b62da, SUC_lexicon_tags_0 },
    { 0x170b62db, SUC_lexicon_tags_11 },
    { 0xd8eb62dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1f6b62de, SUC_lexicon_tags_2 },
    { 0x5dcb62df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67eb62e3, SUC_lexicon_tags_4 },
    { 0x236b62e4, SUC_lexicon_tags_7 },
    { 0x6bab62e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x522b62e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6b62f0, SUC_lexicon_tags_30 },
    { 0xf18b62f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4b62f8, SUC_lexicon_tags_2 },
    { 0x266b62f9, SUC_lexicon_tags_11 },
    { 0xda2b62fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c8b62fc, SUC_lexicon_tags_27 },
    { 0x22ab62fd, SUC_lexicon_tags_17 },
    { 0xbb8b62fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbd2b6300, SUC_lexicon_tags_8 },
    { 0xfbcb6301, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x600b6303, SUC_lexicon_tags_14 },
    { 0x6aeb6303, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x454b6307, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4b630b, SUC_lexicon_tags_14 },
    { 0x946b630c, SUC_lexicon_tags_0 },
    { 0xe64b630d, SUC_lexicon_tags_37 },
    { 0xdfcb630b, SUC_lexicon_tags_45 },
    { 0xd92b630c, SUC_lexicon_tags_45 },
    { 0xa60b630b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd0b6312, SUC_lexicon_tags_70 },
    { 0x944b6313, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4b6317, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb631b, SUC_lexicon_tags_27 },
    { 0xc18b631b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33cb631e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58b6325, SUC_lexicon_tags_53 },
    { 0x1f8b6326, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaeb6329, SUC_lexicon_tags_86 },
    { 0xb4eb632a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb04b632c, SUC_lexicon_tags_12 },
    { 0xe7eb632d, SUC_lexicon_tags_7 },
    { 0x8a4b632c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30ab6330, SUC_lexicon_tags_55 },
    { 0x778b6331, SUC_lexicon_tags_17 },
    { 0x2d8b6330, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeeb6339, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b633e, SUC_lexicon_tags_28 },
    { 0xc3eb633f, SUC_lexicon_tags_25 },
    { 0x71cb6340, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4eb6342, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ab6346, SUC_lexicon_tags_1 },
    { 0x56ab6347, SUC_lexicon_tags_26 },
    { 0x79ab6348, SUC_lexicon_tags_289 },
    { 0xf66b6349, SUC_lexicon_tags_15 },
    { 0xaaab634a, SUC_lexicon_tags_39 },
    { 0xec6b6349, SUC_lexicon_tags_17 },
    { 0x78b634a, SUC_lexicon_tags_37 },
    { 0x70eb634b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeb6351, SUC_lexicon_tags_15 },
    { 0xa56b6351, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fab6355, SUC_lexicon_tags_12 },
    { 0x786b6355, SUC_lexicon_tags_30 },
    { 0xa4cb6357, SUC_lexicon_tags_0 },
    { 0x610b6358, SUC_lexicon_tags_148 },
    { 0x966b6358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57eb635e, SUC_lexicon_tags_49 },
    { 0xd36b635f, SUC_lexicon_tags_11 },
    { 0xe40b635e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18b6366, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ab636a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9feb636d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x28ab636f, SUC_lexicon_tags_25 },
    { 0x740b636f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab6373, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cb6377, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12b6379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602b637e, SUC_lexicon_tags_7 },
    { 0xaaeb637f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x694b6382, SUC_lexicon_tags_87 },
    { 0x2f8b6383, SUC_lexicon_tags_13 },
    { 0xde6b6383, SUC_lexicon_tags_60 },
    { 0x17ab6384, SUC_lexicon_tags_41 },
    { 0xb3eb6382, SUC_lexicon_tags_17 },
    { 0x5c0b6382, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x456b6389, SUC_lexicon_tags_45 },
    { 0xe62b638a, SUC_lexicon_tags_27 },
    { 0xabcb638b, SUC_lexicon_tags_7 },
    { 0x522b638c, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x452b638e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36b6393, SUC_lexicon_tags_14 },
    { 0x80ab6394, SUC_lexicon_tags_37 },
    { 0xe4cb6395, SUC_lexicon_tags_9 },
    { 0x9b6b6395, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x68cb6398, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x426b639b, SUC_lexicon_tags_1180 },
    { 0x786b639c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6b639f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eb63ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda6b63b0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4b63be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0b63c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514b63c7, SUC_lexicon_tags_12 },
    { 0x7aab63c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4b63d1, SUC_lexicon_tags_43 },
    { 0xbeeb63d2, SUC_lexicon_tags_26 },
    { 0xf9cb63d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdeeb63d5, SUC_lexicon_tags_2 },
    { 0x2f2b63d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb63d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd44b63db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8b63df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eb63ec, SUC_lexicon_tags_34 },
    { 0x234b63ed, SUC_lexicon_tags_37 },
    { 0xff4b63ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152b63f4, SUC_lexicon_tags_39 },
    { 0xefab63f5, SUC_lexicon_tags_0 },
    { 0xecb63f6, SUC_lexicon_tags_11 },
    { 0x9e4b63f6, SUC_lexicon_tags_2 },
    { 0x57cb63f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cb63fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e2b63fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedab6402, SUC_lexicon_tags_12 },
    { 0xf72b6403, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x930b6406, SUC_lexicon_tags_64 },
    { 0xfaab6407, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298b640c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ab6412, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508b6416, SUC_lexicon_tags_1 },
    { 0xadab6417, SUC_lexicon_tags_25 },
    { 0x6deb6418, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc32b641a, SUC_lexicon_tags_13 },
    { 0xd7eb641b, SUC_lexicon_tags_129 },
    { 0x17cb641b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8cab641e, SUC_lexicon_tags_1 },
    { 0x4eeb641e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0b6424, SUC_lexicon_tags_12 },
    { 0x120b6425, SUC_lexicon_tags_15 },
    { 0x1c4b6426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24b642d, SUC_lexicon_tags_14 },
    { 0x694b642d, SUC_lexicon_tags_2 },
    { 0xc7ab642e, SUC_lexicon_tags_37 },
    { 0x984b6430, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x148b6433, SUC_lexicon_tags_10 },
    { 0xe78b6433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcb6438, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x582b643a, SUC_lexicon_tags_38 },
    { 0x6d0b643b, SUC_lexicon_tags_25 },
    { 0x558b643c, SUC_lexicon_tags_17 },
    { 0x6fcb643d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8b6440, SUC_lexicon_tags_38 },
    { 0x33ab6441, SUC_lexicon_tags_11 },
    { 0xf16b6442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eb6447, SUC_lexicon_tags_2 },
    { 0xc74b6448, SUC_lexicon_tags_12 },
    { 0xd0ab6448, SUC_lexicon_tags_12 },
    { 0x9f6b6448, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eb644e, SUC_lexicon_tags_14 },
    { 0xd2cb644f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4b6457, SUC_lexicon_tags_2 },
    { 0xb38b6458, SUC_lexicon_tags_37 },
    { 0xad6b6457, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5aab645b, SUC_lexicon_tags_25 },
    { 0x66b645c, SUC_lexicon_tags_2 },
    { 0xfbeb645c, SUC_lexicon_tags_17 },
    { 0xeeab645b, SUC_lexicon_tags_7 },
    { 0x2cab645f, SUC_lexicon_tags_0 },
    { 0x3e8b645c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x176b6462, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2beb6464, SUC_lexicon_tags_37 },
    { 0xecb6465, SUC_lexicon_tags_240 },
    { 0x6e8b6464, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cb646d, SUC_lexicon_tags_47 },
    { 0x48eb646d, SUC_lexicon_tags_46 },
    { 0xf62b646d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x346b6472, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x88eb6475, SUC_lexicon_tags_21 },
    { 0x9d6b6475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40b647c, SUC_lexicon_tags_39 },
    { 0xeaeb647d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24eb6480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8b6486, SUC_lexicon_tags_12 },
    { 0xf42b6486, SUC_lexicon_tags_63 },
    { 0x40cb6487, SUC_lexicon_tags_27 },
    { 0x9bcb6486, SUC_lexicon_tags_2 },
    { 0x12ab648a, SUC_lexicon_tags_37 },
    { 0x1e0b648b, SUC_lexicon_tags_60 },
    { 0x80eb648c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x126b648e, SUC_lexicon_tags_2 },
    { 0x9c0b648f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb6494, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b6497, SUC_lexicon_tags_27 },
    { 0x73ab6498, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442b649d, SUC_lexicon_tags_2 },
    { 0xdbcb649e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78b64a1, SUC_lexicon_tags_3 },
    { 0x3f4b64a1, SUC_lexicon_tags_27 },
    { 0xc90b64a3, SUC_lexicon_tags_26 },
    { 0x8c2b64a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9eb64a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x648b64a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ab64ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceab64b0, SUC_lexicon_tags_14 },
    { 0xa76b64b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x688b64b3, SUC_lexicon_tags_37 },
    { 0x3ecb64b3, SUC_lexicon_tags_25 },
    { 0xac6b64b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88b64b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x556b64bc, SUC_lexicon_tags_31 },
    { 0x4d4b64bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70b64c0, SUC_lexicon_tags_23 },
    { 0x7f0b64c1, SUC_lexicon_tags_81 },
    { 0x6f2b64c2, SUC_lexicon_tags_37 },
    { 0xe1ab64c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48b64ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02b64cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cb64d2, SUC_lexicon_tags_12 },
    { 0xb12b64d3, SUC_lexicon_tags_63 },
    { 0x79ab64d4, SUC_lexicon_tags_15 },
    { 0xf02b64d5, SUC_lexicon_tags_21 },
    { 0x862b64d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122b64da, SUC_lexicon_tags_17 },
    { 0x63ab64da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26eb64dd, SUC_lexicon_tags_49 },
    { 0x2aab64de, SUC_lexicon_tags_12 },
    { 0xb96b64df, SUC_lexicon_tags_0 },
    { 0x1e6b64de, SUC_lexicon_tags_17 },
    { 0xaccb64de, SUC_lexicon_tags_13 },
    { 0xd4ab64e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd2b64e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ab64e7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720b64ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x534b64ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f4b64f0, SUC_lexicon_tags_0 },
    { 0xa50b64f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cb64f7, SUC_lexicon_tags_9 },
    { 0x384b64f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbe2b64fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x166b64fd, SUC_lexicon_tags_87 },
    { 0xe8eb64fe, SUC_lexicon_tags_16 },
    { 0x6eb64fd, SUC_lexicon_tags_37 },
    { 0x6d8b6500, SUC_lexicon_tags_1 },
    { 0x7b8b6501, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8b6505, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfab650b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8b650e, SUC_lexicon_tags_2 },
    { 0x5dcb650f, SUC_lexicon_tags_15 },
    { 0xf40b650e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20eb6512, SUC_lexicon_tags_0 },
    { 0x27cb6513, SUC_lexicon_tags_7 },
    { 0x6beb6514, SUC_lexicon_tags_37 },
    { 0x862b6512, SUC_lexicon_tags_12 },
    { 0x3c0b6516, SUC_lexicon_tags_15 },
    { 0x8fcb6513, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ab651a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x61eb651d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28b6521, SUC_lexicon_tags_11 },
    { 0xc9eb6522, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fab652c, SUC_lexicon_tags_37 },
    { 0xd50b652d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x876b652f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x80eb6532, SUC_lexicon_tags_25 },
    { 0xdb0b6533, SUC_lexicon_tags_23 },
    { 0x9eb6534, SUC_lexicon_tags_7 },
    { 0x816b6534, SUC_lexicon_tags_15 },
    { 0xdf2b6534, SUC_lexicon_tags_25 },
    { 0x1eab6536, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1eab6539, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2b653e, SUC_lexicon_tags_10 },
    { 0xca8b653e, SUC_lexicon_tags_1 },
    { 0xb94b6540, SUC_lexicon_tags_37 },
    { 0xcceb653e, SUC_lexicon_tags_7 },
    { 0x50cb6542, SUC_lexicon_tags_14 },
    { 0x800b653e, SUC_lexicon_tags_2 },
    { 0xe54b6542, SUC_lexicon_tags_37 },
    { 0x33cb6545, SUC_lexicon_tags_15 },
    { 0xb26b6543, SUC_lexicon_tags_14 },
    { 0xb7ab6547, SUC_lexicon_tags_31 },
    { 0xf78b6547, SUC_lexicon_tags_19 },
    { 0x6a6b6547, SUC_lexicon_tags_26 },
    { 0x474b654a, SUC_lexicon_tags_37 },
    { 0x2d0b6548, SUC_lexicon_tags_17 },
    { 0xe14b6547, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x84eb654e, SUC_lexicon_tags_17 },
    { 0x1bcb654e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eb6553, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x810b6556, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466b655a, SUC_lexicon_tags_27 },
    { 0x988b655b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x706b655e, SUC_lexicon_tags_11 },
    { 0x566b655f, SUC_lexicon_tags_23 },
    { 0xc4cb655f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcb6569, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eab656f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4deb6571, SUC_lexicon_tags_32 },
    { 0x282b6571, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x51cb6575, SUC_lexicon_tags_21 },
    { 0xcceb6576, SUC_lexicon_tags_17 },
    { 0x798b6576, SUC_lexicon_tags_7 },
    { 0xc42b6577, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eb657c, SUC_lexicon_tags_25 },
    { 0x8a8b657d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574b6581, SUC_lexicon_tags_45 },
    { 0x560b6582, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4b658c, SUC_lexicon_tags_63 },
    { 0x3dcb658d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xf8eb658f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x45eb6591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfbeb6593, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x606b6596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa44b6598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8b659b, SUC_lexicon_tags_47 },
    { 0xad0b659c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68eb659e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91eb65a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74b65a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd88b65a9, SUC_lexicon_tags_37 },
    { 0xddeb65aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2b65ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b65b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x426b65b6, SUC_lexicon_tags_29 },
    { 0x83cb65b7, SUC_lexicon_tags_37 },
    { 0xeaeb65b7, SUC_lexicon_tags_2 },
    { 0x2feb65b9, SUC_lexicon_tags_8 },
    { 0xe56b65ba, SUC_lexicon_tags_15 },
    { 0x498b65b8, SUC_lexicon_tags_45 },
    { 0x886b65b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626b65c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecb65c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8b65cb, SUC_lexicon_tags_2 },
    { 0x894b65cb, SUC_lexicon_tags_15 },
    { 0x77cb65cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc92b65cf, SUC_lexicon_tags_15 },
    { 0xf24b65cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6b65d3, SUC_lexicon_tags_32 },
    { 0x11ab65d4, SUC_lexicon_tags_33 },
    { 0x4b65d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18ab65d7, SUC_lexicon_tags_45 },
    { 0xc88b65d8, SUC_lexicon_tags_12 },
    { 0xa98b65d8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x816b65db, SUC_lexicon_tags_0 },
    { 0x63ab65dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x286b65de, SUC_lexicon_tags_17 },
    { 0x874b65de, SUC_lexicon_tags_33 },
    { 0x89eb65e0, SUC_lexicon_tags_47 },
    { 0xa92b65e1, SUC_lexicon_tags_1 },
    { 0xc7ab65df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9eb65e4, SUC_lexicon_tags_0 },
    { 0xc4b65e5, SUC_lexicon_tags_47 },
    { 0xfeb65e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cb65ed, SUC_lexicon_tags_12 },
    { 0xd7ab65ed, SUC_lexicon_tags_31 },
    { 0xecb65ed, SUC_lexicon_tags_118 },
    { 0x892b65f0, SUC_lexicon_tags_11 },
    { 0xddeb65f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1eb65f4, SUC_lexicon_tags_45 },
    { 0x740b65f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x574b65f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecab65fa, SUC_lexicon_tags_62 },
    { 0x53ab65fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0b6601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18b6605, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcab660a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeb660e, SUC_lexicon_tags_70 },
    { 0x836b660f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd06b6611, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x466b6613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96b6616, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x648b6618, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x704b661b, SUC_lexicon_tags_65 },
    { 0x9b4b661c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x356b661e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x640b6620, SUC_lexicon_tags_900 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674b6627, SUC_lexicon_tags_19 },
    { 0xb9eb6628, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cb662b, SUC_lexicon_tags_15 },
    { 0x800b662b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfaab662e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x110b6630, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1cb6632, SUC_lexicon_tags_28 },
    { 0xdaab6633, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14b663c, SUC_lexicon_tags_7 },
    { 0x166b663d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x404b663f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49cb6642, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x950b6644, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cb6648, SUC_lexicon_tags_11 },
    { 0xe12b6649, SUC_lexicon_tags_43 },
    { 0x428b664a, SUC_lexicon_tags_37 },
    { 0xfcab664a, SUC_lexicon_tags_43 },
    { 0x974b6649, SUC_lexicon_tags_25 },
    { 0xae6b664a, SUC_lexicon_tags_37 },
    { 0xd1cb664e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ab6651, SUC_lexicon_tags_11 },
    { 0xa08b6652, SUC_lexicon_tags_14 },
    { 0xf2eb6653, SUC_lexicon_tags_17 },
    { 0x35eb6653, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908b665a, SUC_lexicon_tags_7 },
    { 0x54ab665a, SUC_lexicon_tags_25 },
    { 0xe50b665b, SUC_lexicon_tags_33 },
    { 0xde4b665d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80eb6661, SUC_lexicon_tags_25 },
    { 0xf8b6662, SUC_lexicon_tags_3 },
    { 0xa12b6663, SUC_lexicon_tags_2 },
    { 0x99ab6663, SUC_lexicon_tags_19 },
    { 0x8a4b6665, SUC_lexicon_tags_37 },
    { 0x4f2b6666, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x400b6669, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2b666c, SUC_lexicon_tags_55 },
    { 0xac2b666d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x256b6670, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00b6673, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418b667a, SUC_lexicon_tags_11 },
    { 0x3eeb667b, SUC_lexicon_tags_87 },
    { 0x17eb667a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x910b667e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56b6683, SUC_lexicon_tags_7 },
    { 0x454b6684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b0b6686, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834b6691, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36b6696, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc92b6698, SUC_lexicon_tags_25 },
    { 0xe3cb6699, SUC_lexicon_tags_17 },
    { 0xd66b6698, SUC_lexicon_tags_37 },
    { 0x40ab669a, SUC_lexicon_tags_39 },
    { 0x92b6699, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x572b669f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x53ab66a1, SUC_lexicon_tags_12 },
    { 0xa06b66a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cb66ad, SUC_lexicon_tags_15 },
    { 0x5dcb66ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4b66b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eb66b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceb66b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeb66bc, SUC_lexicon_tags_48 },
    { 0x9c6b66bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cb66c0, SUC_lexicon_tags_23 },
    { 0x524b66c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10b66c4, SUC_lexicon_tags_4 },
    { 0xbbab66c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4b66c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x70cb66ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78b66cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb92b66ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00b66d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe4b66d3, SUC_lexicon_tags_11 },
    { 0x7bab66d4, SUC_lexicon_tags_17 },
    { 0x246b66d3, SUC_lexicon_tags_2 },
    { 0x70cb66d3, SUC_lexicon_tags_17 },
    { 0xe0ab66d7, SUC_lexicon_tags_62 },
    { 0x836b66d8, SUC_lexicon_tags_14 },
    { 0xec8b66d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcb66dc, SUC_lexicon_tags_1 },
    { 0x140b66dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4b66e1, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cb66e6, SUC_lexicon_tags_11 },
    { 0x6e2b66e6, SUC_lexicon_tags_0 },
    { 0xfecb66e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764b66f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62b66f4, SUC_lexicon_tags_19 },
    { 0x7ecb66f4, SUC_lexicon_tags_12 },
    { 0x88b66f6, SUC_lexicon_tags_12 },
    { 0x5beb66f7, SUC_lexicon_tags_27 },
    { 0xc36b66f6, SUC_lexicon_tags_43 },
    { 0x520b66f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35cb66fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6b6700, SUC_lexicon_tags_17 },
    { 0xf38b6701, SUC_lexicon_tags_65 },
    { 0xa60b6702, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22b6707, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cb670b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cb6712, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa38b6714, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cb6719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0b671f, SUC_lexicon_tags_17 },
    { 0x9ccb671f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50b6724, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x926b6726, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cb672b, SUC_lexicon_tags_11 },
    { 0x2f6b672c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d8b672e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300b6734, SUC_lexicon_tags_11 },
    { 0xc60b6735, SUC_lexicon_tags_2 },
    { 0xd96b6736, SUC_lexicon_tags_21 },
    { 0x93ab6737, SUC_lexicon_tags_13 },
    { 0x650b6738, SUC_lexicon_tags_65 },
    { 0x694b6737, SUC_lexicon_tags_109 },
    { 0xebcb673a, SUC_lexicon_tags_88 },
    { 0xe92b673a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2b6742, SUC_lexicon_tags_37 },
    { 0x582b6743, SUC_lexicon_tags_17 },
    { 0x8e6b6744, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x848b6746, SUC_lexicon_tags_2 },
    { 0x894b6747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4eeb6749, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x156b674b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4b674e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x318b6750, SUC_lexicon_tags_17 },
    { 0x9e6b6750, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0b6754, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x856b6756, SUC_lexicon_tags_87 },
    { 0xb80b6756, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x724b675a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x714b675c, SUC_lexicon_tags_68 },
    { 0x38ab675d, SUC_lexicon_tags_1 },
    { 0x7b4b675e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bab6764, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xae0b6767, SUC_lexicon_tags_11 },
    { 0x9c0b6768, SUC_lexicon_tags_12 },
    { 0x16b6769, SUC_lexicon_tags_74 },
    { 0xf6b6767, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacb6772, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cb677a, SUC_lexicon_tags_37 },
    { 0x73eb677a, SUC_lexicon_tags_10 },
    { 0x83eb677a, SUC_lexicon_tags_15 },
    { 0x16eb677b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8feb677f, SUC_lexicon_tags_0 },
    { 0x62eb6780, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eb6784, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ab678a, SUC_lexicon_tags_4 },
    { 0x4d4b678a, SUC_lexicon_tags_10 },
    { 0x69eb678c, SUC_lexicon_tags_29 },
    { 0x6e4b678d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadeb6795, SUC_lexicon_tags_17 },
    { 0x50b6796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6b679b, SUC_lexicon_tags_77 },
    { 0xf3cb679c, SUC_lexicon_tags_1 },
    { 0xc4b679b, SUC_lexicon_tags_60 },
    { 0xfb2b679e, SUC_lexicon_tags_11 },
    { 0xcd8b679e, SUC_lexicon_tags_15 },
    { 0x9dab679c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xde6b67a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66b67ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf90b67af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8b67b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64b67b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x282b67b7, SUC_lexicon_tags_25 },
    { 0xd94b67b8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x194b67ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb62b67bc, SUC_lexicon_tags_14 },
    { 0x6f2b67bd, SUC_lexicon_tags_15 },
    { 0xceeb67be, SUC_lexicon_tags_3 },
    { 0x9beb67bf, SUC_lexicon_tags_27 },
    { 0xdd4b67c0, SUC_lexicon_tags_14 },
    { 0xe18b67bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18b67ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x954b67cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98b67d3, SUC_lexicon_tags_16 },
    { 0xe50b67d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32b67d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb67dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2b67e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3eb67e7, SUC_lexicon_tags_11 },
    { 0x4fcb67e8, SUC_lexicon_tags_7 },
    { 0xb4ab67e8, SUC_lexicon_tags_23 },
    { 0x1d6b67e9, SUC_lexicon_tags_21 },
    { 0x5eb67eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310b67ef, SUC_lexicon_tags_4 },
    { 0x2e6b67f0, SUC_lexicon_tags_17 },
    { 0x7ecb67f0, SUC_lexicon_tags_46 },
    { 0xca2b67f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b67fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370b6802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82eb6804, SUC_lexicon_tags_11 },
    { 0x4f4b6805, SUC_lexicon_tags_26 },
    { 0x55eb6805, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2cb6808, SUC_lexicon_tags_28 },
    { 0x320b6809, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ab680e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cb6816, SUC_lexicon_tags_27 },
    { 0xa42b6817, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ab6819, SUC_lexicon_tags_60 },
    { 0xcc0b6819, SUC_lexicon_tags_0 },
    { 0x778b681b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab681f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcdab6821, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20b6824, SUC_lexicon_tags_2 },
    { 0x2d6b6825, SUC_lexicon_tags_12 },
    { 0x458b6826, SUC_lexicon_tags_11 },
    { 0x2aeb6825, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2b682a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44eb682c, SUC_lexicon_tags_27 },
    { 0xa6b682d, SUC_lexicon_tags_17 },
    { 0x74cb682c, SUC_lexicon_tags_17 },
    { 0x220b682f, SUC_lexicon_tags_37 },
    { 0x9fcb6830, SUC_lexicon_tags_47 },
    { 0x3a2b6831, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68b6836, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeb6839, SUC_lexicon_tags_23 },
    { 0xdcb683a, SUC_lexicon_tags_2 },
    { 0x2d2b6839, SUC_lexicon_tags_1 },
    { 0x3c8b683c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8acb683e, SUC_lexicon_tags_84 },
    { 0x7ecb683e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x356b6841, SUC_lexicon_tags_27 },
    { 0xe44b6841, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92b6846, SUC_lexicon_tags_15 },
    { 0xa22b6847, SUC_lexicon_tags_66 },
    { 0x9ceb6847, SUC_lexicon_tags_30 },
    { 0xc0b6849, SUC_lexicon_tags_2 },
    { 0x75eb684a, SUC_lexicon_tags_26 },
    { 0x6e4b684b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86b684e, SUC_lexicon_tags_252 },
    { 0xd22b684e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x28b6851, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb8b6853, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426b6857, SUC_lexicon_tags_46 },
    { 0x712b6858, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf3eb685a, SUC_lexicon_tags_2 },
    { 0xbbcb685b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc02b685d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1eb6863, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98cb6865, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9cb6867, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b686f, SUC_lexicon_tags_11 },
    { 0xcc8b6870, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74b6873, SUC_lexicon_tags_25 },
    { 0xbeab6874, SUC_lexicon_tags_30 },
    { 0x74b6875, SUC_lexicon_tags_16 },
    { 0xd78b6875, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744b687e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x594b6881, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474b6886, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7cb6888, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b6891, SUC_lexicon_tags_25 },
    { 0xd8b6892, SUC_lexicon_tags_27 },
    { 0xf56b6892, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x65ab6895, SUC_lexicon_tags_49 },
    { 0x15cb6896, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8b689b, SUC_lexicon_tags_12 },
    { 0xc34b689c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cb68a4, SUC_lexicon_tags_15 },
    { 0xd90b68a4, SUC_lexicon_tags_2 },
    { 0xbf2b68a6, SUC_lexicon_tags_17 },
    { 0x24eb68a7, SUC_lexicon_tags_7 },
    { 0xf14b68a8, SUC_lexicon_tags_10 },
    { 0x5aeb68a9, SUC_lexicon_tags_17 },
    { 0x608b68a7, SUC_lexicon_tags_11 },
    { 0xc48b68a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7deb68ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4b68b3, SUC_lexicon_tags_15 },
    { 0x3aeb68b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x410b68b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3cb68b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ab68bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ab68bf, SUC_lexicon_tags_37 },
    { 0x274b68c0, SUC_lexicon_tags_11 },
    { 0xfc8b68c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cb68c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736b68ca, SUC_lexicon_tags_57 },
    { 0xdf0b68cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902b68cf, SUC_lexicon_tags_30 },
    { 0xd3eb68cf, SUC_lexicon_tags_37 },
    { 0x342b68d1, SUC_lexicon_tags_11 },
    { 0xe54b68d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2b68dd, SUC_lexicon_tags_57 },
    { 0x66b68de, SUC_lexicon_tags_11 },
    { 0x682b68df, SUC_lexicon_tags_10 },
    { 0x7d6b68df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cb68e4, SUC_lexicon_tags_83 },
    { 0x19eb68e5, SUC_lexicon_tags_28 },
    { 0x86b68e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26b68f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1cb68f7, SUC_lexicon_tags_12 },
    { 0x2cab68f7, SUC_lexicon_tags_37 },
    { 0xd32b68f8, SUC_lexicon_tags_2 },
    { 0xfcab68fa, SUC_lexicon_tags_92 },
    { 0xe94b68fb, SUC_lexicon_tags_27 },
    { 0x7f0b68fb, SUC_lexicon_tags_25 },
    { 0xe3cb68fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x746b68ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c4b6901, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4b690b, SUC_lexicon_tags_27 },
    { 0xe04b690c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x38eb690f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ab6913, SUC_lexicon_tags_17 },
    { 0xd24b6914, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85eb6916, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6b691b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfd6b691d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0eb691f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa96b6921, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xef8b6923, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62b6926, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b0b6928, SUC_lexicon_tags_15 },
    { 0xe30b6929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6cb692b, SUC_lexicon_tags_11 },
    { 0x962b692c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81cb692e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f6b6930, SUC_lexicon_tags_25 },
    { 0x650b6930, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44b6934, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x200b6937, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73eb693e, SUC_lexicon_tags_27 },
    { 0xd7cb693e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbacb6941, SUC_lexicon_tags_145 },
    { 0xe24b6942, SUC_lexicon_tags_2 },
    { 0x8e6b6942, SUC_lexicon_tags_46 },
    { 0x3e4b6944, SUC_lexicon_tags_50 },
    { 0x7acb6944, SUC_lexicon_tags_15 },
    { 0x3e6b6946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b694e, SUC_lexicon_tags_49 },
    { 0xe48b694f, SUC_lexicon_tags_16 },
    { 0x82b694f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x894b6953, SUC_lexicon_tags_9 },
    { 0xb22b6954, SUC_lexicon_tags_30 },
    { 0x900b6954, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x958b6957, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb8b6959, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2b695c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ab6961, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe64b6963, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa84b6965, SUC_lexicon_tags_46 },
    { 0xc68b6965, SUC_lexicon_tags_8 },
    { 0xcf6b6966, SUC_lexicon_tags_6 },
    { 0x60cb6968, SUC_lexicon_tags_11 },
    { 0xfc2b6969, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed4b696b, SUC_lexicon_tags_13 },
    { 0x924b696c, SUC_lexicon_tags_0 },
    { 0x206b696d, SUC_lexicon_tags_38 },
    { 0x9aab696e, SUC_lexicon_tags_25 },
    { 0xa6b696f, SUC_lexicon_tags_27 },
    { 0xe02b6970, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6b6973, SUC_lexicon_tags_38 },
    { 0xec6b6974, SUC_lexicon_tags_25 },
    { 0x4a0b6974, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x50eb6977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910b697f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x278b6981, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbdcb6983, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecb6986, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0b6989, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9b4b698b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x940b698e, SUC_lexicon_tags_25 },
    { 0xc1eb698f, SUC_lexicon_tags_17 },
    { 0x22ab6990, SUC_lexicon_tags_17 },
    { 0x4e4b698e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58b6994, SUC_lexicon_tags_30 },
    { 0x8b4b6995, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeb0b6997, SUC_lexicon_tags_79 },
    { 0xa32b6998, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x138b699b, SUC_lexicon_tags_25 },
    { 0xcb0b699c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69eb69a3, SUC_lexicon_tags_44 },
    { 0x5b6b69a4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74eb69a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4b69ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cb69af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52eb69b3, SUC_lexicon_tags_19 },
    { 0x56eb69b3, SUC_lexicon_tags_37 },
    { 0x57ab69b5, SUC_lexicon_tags_12 },
    { 0x946b69b5, SUC_lexicon_tags_65 },
    { 0x952b69b7, SUC_lexicon_tags_21 },
    { 0x686b69b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76b69bb, SUC_lexicon_tags_63 },
    { 0x3a8b69bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632b69c0, SUC_lexicon_tags_21 },
    { 0x6fcb69c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf08b69c3, SUC_lexicon_tags_77 },
    { 0x5cab69c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510b69c8, SUC_lexicon_tags_65 },
    { 0x168b69c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2b69d1, SUC_lexicon_tags_15 },
    { 0xab0b69d2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xdd6b69d4, SUC_lexicon_tags_19 },
    { 0x290b69d5, SUC_lexicon_tags_34 },
    { 0x518b69d5, SUC_lexicon_tags_0 },
    { 0xacb69d6, SUC_lexicon_tags_12 },
    { 0x86cb69d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e0b69da, SUC_lexicon_tags_48 },
    { 0xcd4b69da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xffab69dd, SUC_lexicon_tags_37 },
    { 0x78eb69dd, SUC_lexicon_tags_2 },
    { 0xa4eb69dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b69e2, SUC_lexicon_tags_28 },
    { 0x88b69e2, SUC_lexicon_tags_37 },
    { 0x95eb69e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eab69e8, SUC_lexicon_tags_17 },
    { 0x984b69e9, SUC_lexicon_tags_714 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cb69ec, SUC_lexicon_tags_25 },
    { 0xd2eb69ed, SUC_lexicon_tags_2 },
    { 0xc66b69ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa64b69f0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x668b69f2, SUC_lexicon_tags_1 },
    { 0x242b69f2, SUC_lexicon_tags_17 },
    { 0x148b69f2, SUC_lexicon_tags_8 },
    { 0x98ab69f5, SUC_lexicon_tags_103 },
    { 0x4d2b69f6, SUC_lexicon_tags_14 },
    { 0x1ab69f7, SUC_lexicon_tags_14 },
    { 0x6a6b69f7, SUC_lexicon_tags_35 },
    { 0xc04b69f7, SUC_lexicon_tags_25 },
    { 0x2d0b69f9, SUC_lexicon_tags_29 },
    { 0xdd0b69fb, SUC_lexicon_tags_25 },
    { 0x5f0b69f7, SUC_lexicon_tags_9 },
    { 0x30b69f9, SUC_lexicon_tags_37 },
    { 0x340b69fe, SUC_lexicon_tags_37 },
    { 0x68cb69fe, SUC_lexicon_tags_33 },
    { 0x3cb6a00, SUC_lexicon_tags_37 },
    { 0x91eb6a00, SUC_lexicon_tags_43 },
    { 0x320b69ff, SUC_lexicon_tags_24 },
    { 0xdeb6a03, SUC_lexicon_tags_25 },
    { 0x402b6a04, SUC_lexicon_tags_25 },
    { 0x704b6a00, SUC_lexicon_tags_37 },
    { 0x39eb69f2, SUC_lexicon_tags_7 },
    { 0x428b6a07, SUC_lexicon_tags_17 },
    { 0x50ab6a08, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x530b6a0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742b6a11, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2a6b6a13, SUC_lexicon_tags_37 },
    { 0xba4b6a14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x10eb6a16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84b6a1a, SUC_lexicon_tags_9 },
    { 0x10ab6a1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516b6a22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458b6a27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4eb6a29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2b6a2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x946b6a31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b0b6a33, SUC_lexicon_tags_254 },
    { 0xed8b6a33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56ab6a36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6b6a3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8eab6a3e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6beb6a42, SUC_lexicon_tags_14 },
    { 0x92eb6a42, SUC_lexicon_tags_37 },
    { 0x4e4b6a42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8b6a47, SUC_lexicon_tags_25 },
    { 0x5d4b6a48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6b6a4b, SUC_lexicon_tags_25 },
    { 0xc88b6a4c, SUC_lexicon_tags_7 },
    { 0xe5cb6a4d, SUC_lexicon_tags_7 },
    { 0xb14b6a4d, SUC_lexicon_tags_20 },
    { 0x448b6a4d, SUC_lexicon_tags_12 },
    { 0x4f8b6a50, SUC_lexicon_tags_38 },
    { 0x9c4b6a51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e2b6a53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab6a58, SUC_lexicon_tags_37 },
    { 0x6f4b6a59, SUC_lexicon_tags_17 },
    { 0xdd0b6a5a, SUC_lexicon_tags_11 },
    { 0x8deb6a5b, SUC_lexicon_tags_37 },
    { 0xceab6a5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2b6a60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868b6a6d, SUC_lexicon_tags_0 },
    { 0x568b6a6d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x192b6a70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720b6a74, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ab6a7f, SUC_lexicon_tags_49 },
    { 0xbd6b6a80, SUC_lexicon_tags_46 },
    { 0x4fab6a81, SUC_lexicon_tags_37 },
    { 0x388b6a82, SUC_lexicon_tags_37 },
    { 0xeacb6a83, SUC_lexicon_tags_15 },
    { 0xa98b6a83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab2b6a86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xecb6a88, SUC_lexicon_tags_28 },
    { 0x4b4b6a88, SUC_lexicon_tags_10 },
    { 0x918b6a8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40b6a8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98b6a90, SUC_lexicon_tags_1 },
    { 0xc96b6a91, SUC_lexicon_tags_11 },
    { 0x2ceb6a92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6b6a95, SUC_lexicon_tags_9 },
    { 0x7e4b6a96, SUC_lexicon_tags_15 },
    { 0x31eb6a95, SUC_lexicon_tags_75 },
    { 0x9cab6a98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65cb6a9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x122b6a9c, SUC_lexicon_tags_7 },
    { 0x1c4b6a9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84b6aa5, SUC_lexicon_tags_85 },
    { 0x590b6aa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x690b6aa9, SUC_lexicon_tags_0 },
    { 0x5a8b6aa9, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b6aae, SUC_lexicon_tags_33 },
    { 0xf7eb6aae, SUC_lexicon_tags_15 },
    { 0x294b6ab0, SUC_lexicon_tags_14 },
    { 0x22eb6ab1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40eb6ab4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd54b6ab6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbc4b6ab8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccb6abc, SUC_lexicon_tags_17 },
    { 0xd58b6abc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b8b6abf, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x20cb6ac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1eb6ac6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbd8b6ac8, SUC_lexicon_tags_23 },
    { 0x10eb6ac9, SUC_lexicon_tags_25 },
    { 0xeab6aca, SUC_lexicon_tags_2 },
    { 0xdf8b6acb, SUC_lexicon_tags_37 },
    { 0x8f6b6acc, SUC_lexicon_tags_25 },
    { 0xed8b6acc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26eb6ad0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4b6ad6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x654b6ad8, SUC_lexicon_tags_38 },
    { 0xb0eb6ad9, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xd98b6adb, SUC_lexicon_tags_37 },
    { 0xb1ab6adc, SUC_lexicon_tags_12 },
    { 0xf52b6adc, SUC_lexicon_tags_49 },
    { 0xa2cb6adb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aeb6ae1, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b6ae5, SUC_lexicon_tags_47 },
    { 0x58b6ae6, SUC_lexicon_tags_1 },
    { 0x846b6ae7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eb6aeb, SUC_lexicon_tags_37 },
    { 0xd4eb6aec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8fcb6aee, SUC_lexicon_tags_10 },
    { 0x878b6aef, SUC_lexicon_tags_14 },
    { 0xac4b6af0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ab6af4, SUC_lexicon_tags_17 },
    { 0xb6eb6af5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bab6af8, SUC_lexicon_tags_35 },
    { 0x64eb6af8, SUC_lexicon_tags_7 },
    { 0xb16b6af8, SUC_lexicon_tags_27 },
    { 0x1e8b6afb, SUC_lexicon_tags_19 },
    { 0x5cab6afc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacb6b02, SUC_lexicon_tags_34 },
    { 0xea0b6b02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cb6b06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb6b09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172b6b13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6feb6b15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a6b6b17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcb6b1e, SUC_lexicon_tags_38 },
    { 0x32cb6b1e, SUC_lexicon_tags_23 },
    { 0x47eb6b1e, SUC_lexicon_tags_77 },
    { 0x966b6b21, SUC_lexicon_tags_25 },
    { 0x66eb6b1e, SUC_lexicon_tags_38 },
    { 0x9d4b6b1f, SUC_lexicon_tags_31 },
    { 0x3c4b6b24, SUC_lexicon_tags_2 },
    { 0x32ab6b24, SUC_lexicon_tags_60 },
    { 0xfccb6b24, SUC_lexicon_tags_70 },
    { 0x2a8b6b27, SUC_lexicon_tags_17 },
    { 0xfccb6b25, SUC_lexicon_tags_31 },
    { 0x22b6b24, SUC_lexicon_tags_8 },
    { 0x6b0b6b24, SUC_lexicon_tags_3 },
    { 0x49eb6b26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3f0b6b2d, SUC_lexicon_tags_25 },
    { 0x4eab6b2e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cab6b32, SUC_lexicon_tags_11 },
    { 0xc48b6b33, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b6b37, SUC_lexicon_tags_29 },
    { 0x1e0b6b38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x238b6b3b, SUC_lexicon_tags_1 },
    { 0x178b6b3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdd6b6b3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf74b6b40, SUC_lexicon_tags_37 },
    { 0xb80b6b41, SUC_lexicon_tags_21 },
    { 0x464b6b41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b6b45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4b6b49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6b6b4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x472b6b4e, SUC_lexicon_tags_15 },
    { 0xdd2b6b4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfeb6b51, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcccb6b53, SUC_lexicon_tags_2 },
    { 0x990b6b54, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1c8b6b56, SUC_lexicon_tags_63 },
    { 0x514b6b57, SUC_lexicon_tags_37 },
    { 0xe04b6b58, SUC_lexicon_tags_77 },
    { 0x1c8b6b59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5f6b6b5b, SUC_lexicon_tags_6 },
    { 0xc4ab6b5b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x332b6b5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72b6b62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cb6b73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x488b6b76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44b6b7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x424b6b7c, SUC_lexicon_tags_55 },
    { 0x5b4b6b7d, SUC_lexicon_tags_6 },
    { 0x172b6b7e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x426b6b81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x576b6b83, SUC_lexicon_tags_0 },
    { 0xe6ab6b83, SUC_lexicon_tags_2 },
    { 0xbf0b6b83, SUC_lexicon_tags_31 },
    { 0xf82b6b86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec2b6b88, SUC_lexicon_tags_0 },
    { 0xcd4b6b89, SUC_lexicon_tags_23 },
    { 0x9ceb6b89, SUC_lexicon_tags_37 },
    { 0x5deb6b8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b6b8e, SUC_lexicon_tags_7 },
    { 0x9f8b6b8f, SUC_lexicon_tags_45 },
    { 0x1b4b6b8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x300b6b93, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6b6b98, SUC_lexicon_tags_154 },
    { 0x210b6b99, SUC_lexicon_tags_17 },
    { 0x19ab6b9a, SUC_lexicon_tags_65 },
    { 0xf04b6b9b, SUC_lexicon_tags_15 },
    { 0x6b0b6b9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x566b6b9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0b6ba2, SUC_lexicon_tags_49 },
    { 0x980b6ba2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf42b6ba5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93eb6ba7, SUC_lexicon_tags_22 },
    { 0xa48b6ba8, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0x71ab6baa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14cb6bad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30ab6baf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812b6bb4, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0x63cb6bb6, SUC_lexicon_tags_55 },
    { 0x766b6bb7, SUC_lexicon_tags_11 },
    { 0x8ccb6bb7, SUC_lexicon_tags_2 },
    { 0x950b6bb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xab6b6bbb, SUC_lexicon_tags_17 },
    { 0x6acb6bbb, SUC_lexicon_tags_1 },
    { 0xcdab6bbd, SUC_lexicon_tags_37 },
    { 0x98ab6bbe, SUC_lexicon_tags_63 },
    { 0x52ab6bbe, SUC_lexicon_tags_55 },
    { 0x66cb6bc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ab6bcb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b6bce, SUC_lexicon_tags_7 },
    { 0x14eb6bcf, SUC_lexicon_tags_14 },
    { 0xe2ab6bd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b6bd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974b6bdd, SUC_lexicon_tags_0 },
    { 0x97cb6bde, SUC_lexicon_tags_31 },
    { 0x6f6b6bde, SUC_lexicon_tags_48 },
    { 0x904b6be0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2b6be4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x250b6be7, SUC_lexicon_tags_15 },
    { 0x9e4b6be8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4cb6bea, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366b6bf3, SUC_lexicon_tags_12 },
    { 0x3e2b6bf3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716b6bf9, SUC_lexicon_tags_27 },
    { 0x8a2b6bf9, SUC_lexicon_tags_46 },
    { 0xb28b6bf9, SUC_lexicon_tags_0 },
    { 0x240b6bfc, SUC_lexicon_tags_17 },
    { 0x77cb6bfd, SUC_lexicon_tags_1 },
    { 0xda2b6bfd, SUC_lexicon_tags_11 },
    { 0x3acb6bff, SUC_lexicon_tags_37 },
    { 0xedcb6bff, SUC_lexicon_tags_49 },
    { 0x942b6c01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840b6c05, SUC_lexicon_tags_25 },
    { 0xe5cb6c05, SUC_lexicon_tags_23 },
    { 0x13ab6c05, SUC_lexicon_tags_29 },
    { 0xc86b6c06, SUC_lexicon_tags_4 },
    { 0xfdab6c05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f6b6c0b, SUC_lexicon_tags_37 },
    { 0x8cab6c0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4b6c15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2b6c19, SUC_lexicon_tags_17 },
    { 0xd04b6c1a, SUC_lexicon_tags_4 },
    { 0x3bcb6c1a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdfeb6c1d, SUC_lexicon_tags_231 },
    { 0xdf4b6c1e, SUC_lexicon_tags_47 },
    { 0x434b6c1e, SUC_lexicon_tags_18 },
    { 0xeecb6c1e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96b6c28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd04b6c2a, SUC_lexicon_tags_45 },
    { 0x9d2b6c2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x484b6c2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccb6c37, SUC_lexicon_tags_31 },
    { 0xab2b6c38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0b6c3c, SUC_lexicon_tags_14 },
    { 0x8f6b6c3c, SUC_lexicon_tags_30 },
    { 0xb34b6c3e, SUC_lexicon_tags_33 },
    { 0x7ab6c3e, SUC_lexicon_tags_15 },
    { 0x944b6c40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcaeb6c42, SUC_lexicon_tags_40 },
    { 0x820b6c42, SUC_lexicon_tags_38 },
    { 0x482b6c44, SUC_lexicon_tags_25 },
    { 0xd9ab6c45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736b6c49, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcb6c4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x954b6c50, SUC_lexicon_tags_0 },
    { 0xb22b6c51, SUC_lexicon_tags_15 },
    { 0x230b6c52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7ab6c54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x258b6c56, SUC_lexicon_tags_23 },
    { 0x11eb6c57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24cb6c59, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x322b6c5c, SUC_lexicon_tags_37 },
    { 0xd72b6c5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ab6c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb6c66, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0xec6b6c69, SUC_lexicon_tags_12 },
    { 0x2beb6c6a, SUC_lexicon_tags_25 },
    { 0xc4eb6c69, SUC_lexicon_tags_12 },
    { 0x97eb6c6c, SUC_lexicon_tags_37 },
    { 0xf46b6c6d, SUC_lexicon_tags_55 },
    { 0xabab6c6c, SUC_lexicon_tags_13 },
    { 0x322b6c6f, SUC_lexicon_tags_8 },
    { 0xf9eb6c70, SUC_lexicon_tags_66 },
    { 0xc28b6c70, SUC_lexicon_tags_2 },
    { 0x318b6c70, SUC_lexicon_tags_60 },
    { 0xc8eb6c73, SUC_lexicon_tags_29 },
    { 0xcd8b6c74, SUC_lexicon_tags_14 },
    { 0x18cb6c70, SUC_lexicon_tags_27 },
    { 0x470b6c76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x214b6c78, SUC_lexicon_tags_25 },
    { 0x58cb6c78, SUC_lexicon_tags_53 },
    { 0x61eb6c79, SUC_lexicon_tags_15 },
    { 0x764b6c78, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfdab6c7d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8b2b6c7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338b6c84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1eb6c86, SUC_lexicon_tags_1 },
    { 0x9ab6c87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4b6c8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x374b6c8c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xed2b6c8e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eb6c93, SUC_lexicon_tags_25 },
    { 0x85eb6c94, SUC_lexicon_tags_19 },
    { 0x322b6c94, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcdab6c97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab6c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb6ca1, SUC_lexicon_tags_7 },
    { 0x486b6ca2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cb6ca7, SUC_lexicon_tags_15 },
    { 0xdeb6ca8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3feb6caa, SUC_lexicon_tags_31 },
    { 0xd50b6cab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916b6cb3, SUC_lexicon_tags_2 },
    { 0xea8b6cb4, SUC_lexicon_tags_15 },
    { 0xbfcb6cb4, SUC_lexicon_tags_1 },
    { 0xc02b6cb3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ab6cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca8b6cbf, SUC_lexicon_tags_84 },
    { 0x842b6cc0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b6cc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bab6cc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6aab6cca, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ab6cd0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330b6cd5, SUC_lexicon_tags_17 },
    { 0x39eb6cd6, SUC_lexicon_tags_10 },
    { 0xe90b6cd7, SUC_lexicon_tags_17 },
    { 0x180b6cd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2b6cdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16b6ce3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21eb6ce5, SUC_lexicon_tags_145 },
    { 0xa50b6ce5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0b6ce9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eb6cf0, SUC_lexicon_tags_28 },
    { 0xf54b6cf0, SUC_lexicon_tags_16 },
    { 0xf12b6cf0, SUC_lexicon_tags_48 },
    { 0x414b6cf3, SUC_lexicon_tags_1 },
    { 0xe4cb6cf4, SUC_lexicon_tags_60 },
    { 0xc10b6cf5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ab6cf9, SUC_lexicon_tags_25 },
    { 0x4eab6cfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2b6cfd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33eb6d09, SUC_lexicon_tags_25 },
    { 0x464b6d09, SUC_lexicon_tags_45 },
    { 0x22ab6d09, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa8eb6d0d, SUC_lexicon_tags_0 },
    { 0x3a4b6d0d, SUC_lexicon_tags_12 },
    { 0x4cab6d0f, SUC_lexicon_tags_48 },
    { 0x8aab6d0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2b6d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cb6d17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcb6d1b, SUC_lexicon_tags_15 },
    { 0x69eb6d1c, SUC_lexicon_tags_37 },
    { 0x78b6d1b, SUC_lexicon_tags_37 },
    { 0x2cb6d1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x316b6d21, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x49ab6d23, SUC_lexicon_tags_11 },
    { 0x7c0b6d24, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5feb6d26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38b6d28, SUC_lexicon_tags_29 },
    { 0x2f2b6d29, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00b6d2c, SUC_lexicon_tags_77 },
    { 0xc6eb6d2d, SUC_lexicon_tags_554 },
    { 0xa12b6d2e, SUC_lexicon_tags_1 },
    { 0x8cb6d2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4b6d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40b6d41, SUC_lexicon_tags_12 },
    { 0x7d4b6d42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4b6d48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8b6d50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9deb6d52, SUC_lexicon_tags_14 },
    { 0xbfcb6d53, SUC_lexicon_tags_1 },
    { 0x15cb6d54, SUC_lexicon_tags_224 },
    { 0xf9ab6d55, SUC_lexicon_tags_11 },
    { 0x5fab6d56, SUC_lexicon_tags_25 },
    { 0xa16b6d57, SUC_lexicon_tags_27 },
    { 0xadcb6d57, SUC_lexicon_tags_25 },
    { 0xbf8b6d59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x586b6d5b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0b6d62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc74b6d64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6b6d67, SUC_lexicon_tags_15 },
    { 0xa88b6d68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7ab6d6a, SUC_lexicon_tags_0 },
    { 0x6eb6d6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76b6d70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ab6d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf8b6d75, SUC_lexicon_tags_8 },
    { 0xdaab6d75, SUC_lexicon_tags_11 },
    { 0x996b6d75, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x726b6d7a, SUC_lexicon_tags_7 },
    { 0x594b6d7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73eb6d7d, SUC_lexicon_tags_37 },
    { 0xb14b6d7e, SUC_lexicon_tags_29 },
    { 0xaccb6d7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926b6d87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cb6d8a, SUC_lexicon_tags_1 },
    { 0xc1ab6d8b, SUC_lexicon_tags_17 },
    { 0xb66b6d8b, SUC_lexicon_tags_10 },
    { 0xa58b6d8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2b6d94, SUC_lexicon_tags_45 },
    { 0xef6b6d95, SUC_lexicon_tags_7 },
    { 0x1c8b6d95, SUC_lexicon_tags_14 },
    { 0xbf6b6d97, SUC_lexicon_tags_14 },
    { 0x946b6d98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100b6da2, SUC_lexicon_tags_27 },
    { 0x328b6da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a2b6da5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30b6da8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86b6dab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bab6dae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4b6db1, SUC_lexicon_tags_39 },
    { 0x578b6db2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccb6db5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdeab6db7, SUC_lexicon_tags_32 },
    { 0x790b6db8, SUC_lexicon_tags_60 },
    { 0xc96b6db9, SUC_lexicon_tags_7 },
    { 0xa92b6dba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b6dc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110b6dc9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd86b6dcb, SUC_lexicon_tags_47 },
    { 0xc2b6dcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cb6dd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3dab6dd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x586b6dd5, SUC_lexicon_tags_1 },
    { 0x2dcb6dd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x906b6dd8, SUC_lexicon_tags_12 },
    { 0x942b6dd9, SUC_lexicon_tags_38 },
    { 0xe64b6dd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2b6ddd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68b6de9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x404b6dec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64b6df4, SUC_lexicon_tags_11 },
    { 0x4b6df5, SUC_lexicon_tags_1 },
    { 0x99cb6df6, SUC_lexicon_tags_2 },
    { 0x1b8b6df7, SUC_lexicon_tags_65 },
    { 0x2f0b6df8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b6dfb, SUC_lexicon_tags_30 },
    { 0x254b6dfb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x16eb6dfe, SUC_lexicon_tags_38 },
    { 0x97cb6dff, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbe6b6e01, SUC_lexicon_tags_35 },
    { 0xfccb6e02, SUC_lexicon_tags_37 },
    { 0x31cb6e02, SUC_lexicon_tags_8 },
    { 0xf88b6e03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cb6e0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6b6e0e, SUC_lexicon_tags_47 },
    { 0x718b6e0e, SUC_lexicon_tags_47 },
    { 0xe9eb6e0e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ab6e19, SUC_lexicon_tags_41 },
    { 0x376b6e1a, SUC_lexicon_tags_25 },
    { 0xb30b6e1a, SUC_lexicon_tags_1 },
    { 0xba0b6e1a, SUC_lexicon_tags_34 },
    { 0x2b4b6e19, SUC_lexicon_tags_47 },
    { 0xfaeb6e1e, SUC_lexicon_tags_1 },
    { 0xf12b6e19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4b6e23, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc86b6e25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08b6e32, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8b6e36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8b6e3c, SUC_lexicon_tags_15 },
    { 0x124b6e3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfacb6e3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cb6e42, SUC_lexicon_tags_49 },
    { 0x1e2b6e43, SUC_lexicon_tags_43 },
    { 0x9f2b6e44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cb6e47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532b6e4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04b6e4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x954b6e50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeeb6e55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaeb6e5a, SUC_lexicon_tags_10 },
    { 0x884b6e5a, SUC_lexicon_tags_12 },
    { 0xd8b6e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82b6e5e, SUC_lexicon_tags_30 },
    { 0xd2ab6e5f, SUC_lexicon_tags_11 },
    { 0x5eab6e5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x454b6e63, SUC_lexicon_tags_2 },
    { 0x89eb6e64, SUC_lexicon_tags_45 },
    { 0x3beb6e64, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2b6e6a, SUC_lexicon_tags_178 },
    { 0x494b6e6b, SUC_lexicon_tags_17 },
    { 0xa4cb6e6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436b6e71, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x296b6e73, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe9cb6e75, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2ecb6e77, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4b6e82, SUC_lexicon_tags_19 },
    { 0x844b6e83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb80b6e85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7deb6e8c, SUC_lexicon_tags_7 },
    { 0xa30b6e8d, SUC_lexicon_tags_33 },
    { 0x9a8b6e8e, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222b6e92, SUC_lexicon_tags_17 },
    { 0xbd4b6e93, SUC_lexicon_tags_28 },
    { 0x338b6e93, SUC_lexicon_tags_37 },
    { 0x2a6b6e95, SUC_lexicon_tags_2 },
    { 0xbd8b6e93, SUC_lexicon_tags_63 },
    { 0xc7eb6e96, SUC_lexicon_tags_26 },
    { 0xeb4b6e98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8b6e9d, SUC_lexicon_tags_25 },
    { 0xff0b6e9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21eb6ea0, SUC_lexicon_tags_0 },
    { 0xc8b6ea1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30b6ea4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x108b6ea7, SUC_lexicon_tags_19 },
    { 0x444b6ea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4b6eac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc42b6eae, SUC_lexicon_tags_15 },
    { 0x902b6eae, SUC_lexicon_tags_118 },
    { 0x4dab6eb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8b6eb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe72b6eb5, SUC_lexicon_tags_7 },
    { 0x30ab6eb5, SUC_lexicon_tags_11 },
    { 0x458b6eb5, SUC_lexicon_tags_55 },
    { 0xcb2b6eb7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x504b6eba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x26eb6ebd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e8b6ebf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xda2b6ec1, SUC_lexicon_tags_23 },
    { 0x7f8b6ec2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa24b6ec4, SUC_lexicon_tags_27 },
    { 0xf7cb6ec5, SUC_lexicon_tags_15 },
    { 0x2f0b6ec4, SUC_lexicon_tags_9 },
    { 0x250b6ec7, SUC_lexicon_tags_23 },
    { 0x564b6ec6, SUC_lexicon_tags_25 },
    { 0xb9eb6ec9, SUC_lexicon_tags_25 },
    { 0xec2b6eca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4b6ece, SUC_lexicon_tags_4 },
    { 0xeab6ecf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ab6ed6, SUC_lexicon_tags_23 },
    { 0x7b6b6ed7, SUC_lexicon_tags_21 },
    { 0xeb6b6ed8, SUC_lexicon_tags_27 },
    { 0xb16b6ed9, SUC_lexicon_tags_26 },
    { 0x7d2b6ed9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432b6edf, SUC_lexicon_tags_15 },
    { 0x874b6ee0, SUC_lexicon_tags_33 },
    { 0xdf8b6ee1, SUC_lexicon_tags_35 },
    { 0xe28b6ee1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280b6eeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe92b6eed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69cb6ef0, SUC_lexicon_tags_27 },
    { 0x15eb6ef0, SUC_lexicon_tags_33 },
    { 0x5f2b6ef2, SUC_lexicon_tags_14 },
    { 0xfdeb6ef2, SUC_lexicon_tags_15 },
    { 0xb58b6ef4, SUC_lexicon_tags_45 },
    { 0x8a6b6ef4, SUC_lexicon_tags_8 },
    { 0x398b6ef5, SUC_lexicon_tags_6 },
    { 0xdb0b6ef7, SUC_lexicon_tags_17 },
    { 0xde6b6ef4, SUC_lexicon_tags_4 },
    { 0x3c6b6ef0, SUC_lexicon_tags_0 },
    { 0xd30b6ef4, SUC_lexicon_tags_1 },
    { 0xcecb6efb, SUC_lexicon_tags_33 },
    { 0x170b6efc, SUC_lexicon_tags_35 },
    { 0x190b6efc, SUC_lexicon_tags_1 },
    { 0xf70b6ef1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32b6f08, SUC_lexicon_tags_27 },
    { 0xe02b6f09, SUC_lexicon_tags_17 },
    { 0x36ab6f0a, SUC_lexicon_tags_46 },
    { 0x24cb6f09, SUC_lexicon_tags_27 },
    { 0x72ab6f0c, SUC_lexicon_tags_15 },
    { 0xb74b6f0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b6f11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8eb6f1d, SUC_lexicon_tags_7 },
    { 0xac0b6f1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99cb6f20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x160b6f22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49eb6f25, SUC_lexicon_tags_17 },
    { 0x1e0b6f25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfdcb6f28, SUC_lexicon_tags_25 },
    { 0x8dab6f29, SUC_lexicon_tags_25 },
    { 0x486b6f2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x906b6f2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb6f2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe72b6f32, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfcb6f34, SUC_lexicon_tags_25 },
    { 0x2ecb6f35, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x272b6f38, SUC_lexicon_tags_0 },
    { 0xb58b6f39, SUC_lexicon_tags_63 },
    { 0x6d2b6f39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x322b6f3d, SUC_lexicon_tags_11 },
    { 0xa96b6f3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcb6f4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcb6f5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0b6f5d, SUC_lexicon_tags_14 },
    { 0xde0b6f5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x222b6f60, SUC_lexicon_tags_25 },
    { 0x346b6f60, SUC_lexicon_tags_15 },
    { 0x796b6f60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c0b6f64, SUC_lexicon_tags_37 },
    { 0xa2eb6f64, SUC_lexicon_tags_14 },
    { 0x89ab6f65, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcb6f6d, SUC_lexicon_tags_23 },
    { 0xc4b6f6d, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0xb92b6f70, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82b6f74, SUC_lexicon_tags_13 },
    { 0xa5ab6f74, SUC_lexicon_tags_25 },
    { 0x2c4b6f75, SUC_lexicon_tags_1 },
    { 0x6a8b6f77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8b6f7a, SUC_lexicon_tags_7 },
    { 0xfdab6f7b, SUC_lexicon_tags_37 },
    { 0xfd6b6f7c, SUC_lexicon_tags_6 },
    { 0xf10b6f7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcb6f8e, SUC_lexicon_tags_21 },
    { 0xad0b6f8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68b6f93, SUC_lexicon_tags_28 },
    { 0x28ab6f94, SUC_lexicon_tags_17 },
    { 0x536b6f94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedeb6f9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2cb6f9e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8b6fa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54ab6fa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56b6fa7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eb6fab, SUC_lexicon_tags_37 },
    { 0xa32b6fab, SUC_lexicon_tags_25 },
    { 0xdc8b6fad, SUC_lexicon_tags_37 },
    { 0xaceb6fac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8b6fb1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80b6fb3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0b6fc0, SUC_lexicon_tags_12 },
    { 0xaeb6fc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422b6fcd, SUC_lexicon_tags_1 },
    { 0x38ab6fce, SUC_lexicon_tags_14 },
    { 0x690b6fcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ab6fd2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8eb6fd5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9eb6fdb, SUC_lexicon_tags_47 },
    { 0x708b6fdc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x29cb6fde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2b6fe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcb6fe4, SUC_lexicon_tags_7 },
    { 0x714b6fe5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9ab6fe7, SUC_lexicon_tags_15 },
    { 0x456b6fe8, SUC_lexicon_tags_6 },
    { 0x31cb6fe9, SUC_lexicon_tags_57 },
    { 0xd40b6fea, SUC_lexicon_tags_27 },
    { 0xf1eb6feb, SUC_lexicon_tags_38 },
    { 0xa88b6fe8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ab6fef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b6ff4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4b6ff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0b6ffe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb7003, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4b7007, SUC_lexicon_tags_27 },
    { 0x186b7007, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x192b700a, SUC_lexicon_tags_33 },
    { 0x632b700b, SUC_lexicon_tags_1 },
    { 0xa3eb700a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74b700f, SUC_lexicon_tags_8 },
    { 0x1e6b7010, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662b7014, SUC_lexicon_tags_60 },
    { 0x786b7015, SUC_lexicon_tags_37 },
    { 0x6a0b7015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80b7019, SUC_lexicon_tags_37 },
    { 0x7dab7019, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954b7021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb4b7023, SUC_lexicon_tags_11 },
    { 0x868b7023, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366b7029, SUC_lexicon_tags_0 },
    { 0x6ecb702a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fab702e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6b7033, SUC_lexicon_tags_7 },
    { 0x49ab7034, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1f4b7036, SUC_lexicon_tags_37 },
    { 0xf50b7037, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe52b7039, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988b703d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbab7040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7eb7044, SUC_lexicon_tags_2 },
    { 0x3f6b7044, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ab704a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6b704d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88b7051, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e0b7053, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x230b7056, SUC_lexicon_tags_0 },
    { 0xa20b7056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36b705d, SUC_lexicon_tags_19 },
    { 0x13ab705e, SUC_lexicon_tags_15 },
    { 0x3d4b705f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ab7062, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaf2b7064, SUC_lexicon_tags_26 },
    { 0x8b8b7065, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab7069, SUC_lexicon_tags_37 },
    { 0xf82b7069, SUC_lexicon_tags_7 },
    { 0xb706a, SUC_lexicon_tags_25 },
    { 0xe40b706c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcaab706e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x45cb7070, SUC_lexicon_tags_69 },
    { 0x728b7070, SUC_lexicon_tags_27 },
    { 0x48cb7071, SUC_lexicon_tags_54 },
    { 0x746b7073, SUC_lexicon_tags_6 },
    { 0x35eb7070, SUC_lexicon_tags_19 },
    { 0x622b7074, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6b707e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4b7082, SUC_lexicon_tags_79 },
    { 0xb8ab7083, SUC_lexicon_tags_63 },
    { 0x7f0b7082, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x10eb7086, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420b708b, SUC_lexicon_tags_30 },
    { 0xc10b708c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6b708f, SUC_lexicon_tags_32 },
    { 0xe7ab708f, SUC_lexicon_tags_29 },
    { 0xf74b7090, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ab7097, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaab709e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf6ab70a0, SUC_lexicon_tags_37 },
    { 0x776b70a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b70a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cb70ab, SUC_lexicon_tags_11 },
    { 0x408b70ab, SUC_lexicon_tags_37 },
    { 0xc40b70ac, SUC_lexicon_tags_45 },
    { 0x774b70ae, SUC_lexicon_tags_11 },
    { 0x946b70ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86eb70b5, SUC_lexicon_tags_2 },
    { 0x5b6b70b6, SUC_lexicon_tags_12 },
    { 0x116b70b7, SUC_lexicon_tags_37 },
    { 0x28ab70b8, SUC_lexicon_tags_37 },
    { 0x88b70b8, SUC_lexicon_tags_30 },
    { 0xbb8b70ba, SUC_lexicon_tags_26 },
    { 0x9fcb70b9, SUC_lexicon_tags_29 },
    { 0xaa8b70b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x564b70be, SUC_lexicon_tags_26 },
    { 0x8aeb70bf, SUC_lexicon_tags_25 },
    { 0xb14b70c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70cb70c2, SUC_lexicon_tags_25 },
    { 0xa3cb70c2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x92cb70c5, SUC_lexicon_tags_4 },
    { 0x64eb70c6, SUC_lexicon_tags_32 },
    { 0x3a0b70c6, SUC_lexicon_tags_9 },
    { 0x17eb70c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eb70cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cb70d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186b70de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cb70e4, SUC_lexicon_tags_1 },
    { 0xe66b70e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214b70ec, SUC_lexicon_tags_17 },
    { 0x688b70ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb6b70ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eb70f7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6b70fa, SUC_lexicon_tags_17 },
    { 0xd70b70fb, SUC_lexicon_tags_60 },
    { 0x4ceb70fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986b7102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ab7106, SUC_lexicon_tags_12 },
    { 0x5e2b7106, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0b710b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ab710f, SUC_lexicon_tags_158 },
    { 0x310b710f, SUC_lexicon_tags_0 },
    { 0x9d8b7110, SUC_lexicon_tags_6 },
    { 0x55cb7112, SUC_lexicon_tags_24 },
    { 0x4b0b7112, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x91ab7115, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x890b7118, SUC_lexicon_tags_55 },
    { 0x2beb7118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc74b711b, SUC_lexicon_tags_43 },
    { 0xb02b711c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04b711f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38b7123, SUC_lexicon_tags_30 },
    { 0xba6b7124, SUC_lexicon_tags_87 },
    { 0x5d4b7124, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc34b7127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35cb712a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44b712d, SUC_lexicon_tags_37 },
    { 0xda8b712e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x364b7130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2eb7132, SUC_lexicon_tags_236 },
    { 0x32ab7132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1ab7135, SUC_lexicon_tags_43 },
    { 0x3fcb7136, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x622b7138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8b713e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2b7141, SUC_lexicon_tags_37 },
    { 0x7b4b7142, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732b7146, SUC_lexicon_tags_17 },
    { 0xb46b7146, SUC_lexicon_tags_27 },
    { 0x7d0b7148, SUC_lexicon_tags_2 },
    { 0xc94b7149, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8b714e, SUC_lexicon_tags_11 },
    { 0xd1eb714e, SUC_lexicon_tags_33 },
    { 0xcbeb7150, SUC_lexicon_tags_2 },
    { 0x62eb7150, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ab715a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59cb715c, SUC_lexicon_tags_25 },
    { 0xf68b715c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6e6b715f, SUC_lexicon_tags_33 },
    { 0x138b715f, SUC_lexicon_tags_23 },
    { 0xaf2b715f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566b7165, SUC_lexicon_tags_11 },
    { 0x910b7166, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02b716d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc18b716f, SUC_lexicon_tags_11 },
    { 0xdaab7170, SUC_lexicon_tags_1 },
    { 0xbbcb7170, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14b7176, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa48b7178, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x290b717a, SUC_lexicon_tags_31 },
    { 0xfbab717b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x470b717e, SUC_lexicon_tags_1 },
    { 0x524b717f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eb7182, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x33cb7185, SUC_lexicon_tags_29 },
    { 0x22ab7186, SUC_lexicon_tags_25 },
    { 0x5a4b7186, SUC_lexicon_tags_49 },
    { 0x924b7188, SUC_lexicon_tags_17 },
    { 0x96eb7186, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fab718c, SUC_lexicon_tags_11 },
    { 0x896b718d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2b7190, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9eb7192, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cb7196, SUC_lexicon_tags_0 },
    { 0x8b2b7196, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe56b7199, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2ab719b, SUC_lexicon_tags_12 },
    { 0xba4b719c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916b71a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648b71a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298b71aa, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2b71b0, SUC_lexicon_tags_30 },
    { 0xa42b71b1, SUC_lexicon_tags_224 },
    { 0xa82b71b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14b71ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x91cb71bd, SUC_lexicon_tags_55 },
    { 0xc5cb71be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4b71c1, SUC_lexicon_tags_9 },
    { 0xd62b71c2, SUC_lexicon_tags_37 },
    { 0x2b71c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe9ab71c5, SUC_lexicon_tags_11 },
    { 0xc6eb71c6, SUC_lexicon_tags_26 },
    { 0x776b71c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0eb71cf, SUC_lexicon_tags_19 },
    { 0xf82b71cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6d4b71d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50eb71d4, SUC_lexicon_tags_17 },
    { 0x2f2b71d4, SUC_lexicon_tags_60 },
    { 0x486b71d6, SUC_lexicon_tags_37 },
    { 0xfcb71d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab71da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2eb71dd, SUC_lexicon_tags_1 },
    { 0x8c4b71de, SUC_lexicon_tags_28 },
    { 0x996b71dd, SUC_lexicon_tags_19 },
    { 0x1dcb71de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12cb71e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf74b71e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0b71e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ab71f1, SUC_lexicon_tags_31 },
    { 0xebab71f2, SUC_lexicon_tags_63 },
    { 0xfdab71f1, SUC_lexicon_tags_2 },
    { 0x7cb71f4, SUC_lexicon_tags_26 },
    { 0x7e6b71f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x196b71f7, SUC_lexicon_tags_17 },
    { 0xbcb71f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce8b71fa, SUC_lexicon_tags_12 },
    { 0x716b71fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eb7200, SUC_lexicon_tags_6 },
    { 0x51eb7200, SUC_lexicon_tags_15 },
    { 0x7b8b7201, SUC_lexicon_tags_30 },
    { 0x276b7203, SUC_lexicon_tags_12 },
    { 0x528b7204, SUC_lexicon_tags_45 },
    { 0xe0cb7204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x894b7208, SUC_lexicon_tags_6 },
    { 0xdbab7209, SUC_lexicon_tags_7 },
    { 0xa7cb7209, SUC_lexicon_tags_19 },
    { 0x8c8b7208, SUC_lexicon_tags_25 },
    { 0x6f0b720b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe12b720e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x132b7210, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396b7215, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4b7218, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28b721d, SUC_lexicon_tags_21 },
    { 0x702b721e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8b7221, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36b7225, SUC_lexicon_tags_11 },
    { 0x134b7226, SUC_lexicon_tags_38 },
    { 0x318b7225, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42b722a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab722d, SUC_lexicon_tags_25 },
    { 0xcbeb722d, SUC_lexicon_tags_1 },
    { 0x16b722f, SUC_lexicon_tags_66 },
    { 0xd10b7230, SUC_lexicon_tags_15 },
    { 0x768b722e, SUC_lexicon_tags_31 },
    { 0xf4b7231, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ab7235, SUC_lexicon_tags_14 },
    { 0x1d6b7236, SUC_lexicon_tags_37 },
    { 0x23ab7235, SUC_lexicon_tags_17 },
    { 0xef4b7238, SUC_lexicon_tags_11 },
    { 0x2b0b7239, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xabcb723b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84b7240, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218b7246, SUC_lexicon_tags_7 },
    { 0x962b7247, SUC_lexicon_tags_8 },
    { 0xe18b7248, SUC_lexicon_tags_43 },
    { 0xed0b7249, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e6b724b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x812b724e, SUC_lexicon_tags_17 },
    { 0x562b724f, SUC_lexicon_tags_15 },
    { 0x1b2b7250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0b7254, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44b7257, SUC_lexicon_tags_1 },
    { 0x578b7258, SUC_lexicon_tags_14 },
    { 0x87cb7259, SUC_lexicon_tags_11 },
    { 0xa6cb725a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eb7261, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb2b7263, SUC_lexicon_tags_17 },
    { 0xaeb7263, SUC_lexicon_tags_15 },
    { 0x6f6b7264, SUC_lexicon_tags_1 },
    { 0x2beb7266, SUC_lexicon_tags_11 },
    { 0xe9eb7266, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe96b7269, SUC_lexicon_tags_12 },
    { 0x17cb7269, SUC_lexicon_tags_2 },
    { 0x590b726b, SUC_lexicon_tags_1 },
    { 0xc60b726c, SUC_lexicon_tags_60 },
    { 0x3e6b726d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6b7273, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2b7282, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ab7285, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x312b7287, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104b728b, SUC_lexicon_tags_0 },
    { 0x696b728b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b4b728e, SUC_lexicon_tags_17 },
    { 0xc88b728e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe6b7291, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d0b7293, SUC_lexicon_tags_114 },
    { 0x4d8b7294, SUC_lexicon_tags_49 },
    { 0x24eb7293, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2b7297, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc52b7299, SUC_lexicon_tags_15 },
    { 0xe3ab729a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6b72a1, SUC_lexicon_tags_33 },
    { 0x762b72a2, SUC_lexicon_tags_28 },
    { 0xf5eb72a2, SUC_lexicon_tags_25 },
    { 0xc00b72a2, SUC_lexicon_tags_31 },
    { 0xa00b72a1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceb72ab, SUC_lexicon_tags_7 },
    { 0xa28b72ab, SUC_lexicon_tags_37 },
    { 0xc40b72ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510b72b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeb72b6, SUC_lexicon_tags_29 },
    { 0x3f8b72b6, SUC_lexicon_tags_7 },
    { 0x530b72b8, SUC_lexicon_tags_17 },
    { 0x40cb72b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2cab72bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfab72c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa4b72c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf48b72c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104b72ce, SUC_lexicon_tags_60 },
    { 0x22b72cf, SUC_lexicon_tags_11 },
    { 0x7b0b72cf, SUC_lexicon_tags_7 },
    { 0x11cb72d1, SUC_lexicon_tags_23 },
    { 0xfe2b72d2, SUC_lexicon_tags_2 },
    { 0x4aab72d3, SUC_lexicon_tags_1 },
    { 0x83eb72d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x35cb72d6, SUC_lexicon_tags_129 },
    { 0x794b72d7, SUC_lexicon_tags_65 },
    { 0xa04b72d8, SUC_lexicon_tags_14 },
    { 0xe5cb72d7, SUC_lexicon_tags_224 },
    { 0x830b72da, SUC_lexicon_tags_14 },
    { 0xad6b72db, SUC_lexicon_tags_37 },
    { 0x74eb72dc, SUC_lexicon_tags_9 },
    { 0xda4b72db, SUC_lexicon_tags_65 },
    { 0x920b72dd, SUC_lexicon_tags_4 },
    { 0x8cab72d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590b72e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x968b72e6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd26b72e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72b72f1, SUC_lexicon_tags_56 },
    { 0xba0b72f2, SUC_lexicon_tags_25 },
    { 0x56cb72f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xed6b72f5, SUC_lexicon_tags_9 },
    { 0xa0b72f6, SUC_lexicon_tags_0 },
    { 0x36cb72f6, SUC_lexicon_tags_33 },
    { 0x270b72f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x696b72fb, SUC_lexicon_tags_18 },
    { 0xbc6b72fc, SUC_lexicon_tags_0 },
    { 0x730b72fb, SUC_lexicon_tags_1 },
    { 0x8c4b72fe, SUC_lexicon_tags_12 },
    { 0x536b72ff, SUC_lexicon_tags_17 },
    { 0x86cb7300, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea4b7302, SUC_lexicon_tags_45 },
    { 0xe78b7303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144b7308, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5fcb730a, SUC_lexicon_tags_14 },
    { 0x32cb730b, SUC_lexicon_tags_2 },
    { 0x332b730c, SUC_lexicon_tags_38 },
    { 0x1d0b730d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadab730f, SUC_lexicon_tags_37 },
    { 0xc36b7310, SUC_lexicon_tags_15 },
    { 0xc4cb7311, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x738b7313, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feb7318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcb731b, SUC_lexicon_tags_12 },
    { 0xd08b731b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x91ab731e, SUC_lexicon_tags_1 },
    { 0xf60b731f, SUC_lexicon_tags_15 },
    { 0x910b731f, SUC_lexicon_tags_48 },
    { 0x170b7321, SUC_lexicon_tags_25 },
    { 0x848b7322, SUC_lexicon_tags_10 },
    { 0x69eb7321, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x268b7325, SUC_lexicon_tags_37 },
    { 0xb9eb7325, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526b732e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccb7331, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722b7337, SUC_lexicon_tags_28 },
    { 0xadcb7338, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc34b733a, SUC_lexicon_tags_48 },
    { 0xe50b733a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd40b733d, SUC_lexicon_tags_37 },
    { 0xc6cb733d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d8b7340, SUC_lexicon_tags_11 },
    { 0x882b7341, SUC_lexicon_tags_13 },
    { 0x690b7342, SUC_lexicon_tags_2 },
    { 0x4aab7343, SUC_lexicon_tags_12 },
    { 0x322b7344, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900b734b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cb734f, SUC_lexicon_tags_25 },
    { 0x334b7350, SUC_lexicon_tags_19 },
    { 0xbbeb7351, SUC_lexicon_tags_63 },
    { 0x750b7351, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4eb7355, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6c4b7357, SUC_lexicon_tags_17 },
    { 0x556b7358, SUC_lexicon_tags_7 },
    { 0xdacb7359, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7a8b735b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18b735e, SUC_lexicon_tags_16 },
    { 0xf04b735f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x450b7361, SUC_lexicon_tags_2 },
    { 0xbbab7361, SUC_lexicon_tags_15 },
    { 0x120b7363, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66b7367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd18b7369, SUC_lexicon_tags_1 },
    { 0xbfab7369, SUC_lexicon_tags_25 },
    { 0xb2cb736b, SUC_lexicon_tags_31 },
    { 0xa7ab736c, SUC_lexicon_tags_17 },
    { 0xacab736b, SUC_lexicon_tags_17 },
    { 0xaccb736d, SUC_lexicon_tags_9 },
    { 0xbdab736f, SUC_lexicon_tags_46 },
    { 0x23ab7369, SUC_lexicon_tags_0 },
    { 0x852b736c, SUC_lexicon_tags_0 },
    { 0x4c0b736a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522b737a, SUC_lexicon_tags_0 },
    { 0xbf2b737a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b7380, SUC_lexicon_tags_32 },
    { 0x138b7381, SUC_lexicon_tags_13 },
    { 0xd5ab7380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18b7387, SUC_lexicon_tags_14 },
    { 0xbfeb7388, SUC_lexicon_tags_67 },
    { 0xa72b7388, SUC_lexicon_tags_17 },
    { 0x58b738a, SUC_lexicon_tags_17 },
    { 0xe6ab7389, SUC_lexicon_tags_11 },
    { 0xf72b738c, SUC_lexicon_tags_37 },
    { 0x5a2b738d, SUC_lexicon_tags_7 },
    { 0xaccb738e, SUC_lexicon_tags_17 },
    { 0x3bab738e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c4b7391, SUC_lexicon_tags_11 },
    { 0x70ab7392, SUC_lexicon_tags_25 },
    { 0x7e2b7392, SUC_lexicon_tags_10 },
    { 0xf6b7391, SUC_lexicon_tags_37 },
    { 0x84ab7395, SUC_lexicon_tags_12 },
    { 0xed0b7395, SUC_lexicon_tags_48 },
    { 0xaccb7393, SUC_lexicon_tags_25 },
    { 0xab2b7397, SUC_lexicon_tags_68 },
    { 0x670b7399, SUC_lexicon_tags_3 },
    { 0x5ab739a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa78b739c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xab739f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x672b73a1, SUC_lexicon_tags_45 },
    { 0xd6cb73a1, SUC_lexicon_tags_38 },
    { 0x764b73a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eab73a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2b73ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8b73ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4b73b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4b73b4, SUC_lexicon_tags_1 },
    { 0x894b73b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8b73b8, SUC_lexicon_tags_25 },
    { 0x3feb73b9, SUC_lexicon_tags_2 },
    { 0x4d0b73ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x352b73bc, SUC_lexicon_tags_11 },
    { 0x88cb73bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x130b73bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6b73c2, SUC_lexicon_tags_45 },
    { 0x9a8b73c2, SUC_lexicon_tags_8 },
    { 0xe34b73c2, SUC_lexicon_tags_77 },
    { 0xa0eb73c4, SUC_lexicon_tags_14 },
    { 0xa00b73c6, SUC_lexicon_tags_49 },
    { 0xb7cb73c7, SUC_lexicon_tags_26 },
    { 0x150b73c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x816b73cb, SUC_lexicon_tags_2 },
    { 0x264b73cb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56b73cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86b73d5, SUC_lexicon_tags_4 },
    { 0xb9cb73d5, SUC_lexicon_tags_25 },
    { 0x35ab73d7, SUC_lexicon_tags_63 },
    { 0xf3ab73d6, SUC_lexicon_tags_23 },
    { 0xf62b73d9, SUC_lexicon_tags_15 },
    { 0x276b73da, SUC_lexicon_tags_28 },
    { 0xab4b73db, SUC_lexicon_tags_15 },
    { 0x660b73dc, SUC_lexicon_tags_38 },
    { 0x7a2b73db, SUC_lexicon_tags_0 },
    { 0x9ccb73dd, SUC_lexicon_tags_7 },
    { 0x42eb73df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xabab73e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2b73e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ab73f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30b73f9, SUC_lexicon_tags_84 },
    { 0x740b73fa, SUC_lexicon_tags_28 },
    { 0x882b73fa, SUC_lexicon_tags_14 },
    { 0xf50b73fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab73ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ab7402, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfab7404, SUC_lexicon_tags_10 },
    { 0xa30b7405, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf16b7407, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2b740c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaeeb740e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x882b7410, SUC_lexicon_tags_30 },
    { 0xde2b7411, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54b7414, SUC_lexicon_tags_37 },
    { 0x9ab7415, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b7418, SUC_lexicon_tags_1 },
    { 0x91ab7419, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ab741d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x742b741f, SUC_lexicon_tags_46 },
    { 0xec4b741f, SUC_lexicon_tags_9 },
    { 0xb58b7420, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x904b7423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x288b7426, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b742a, SUC_lexicon_tags_37 },
    { 0xfbab742a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x956b742d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4b7431, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0b7435, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30b7439, SUC_lexicon_tags_12 },
    { 0x154b7439, SUC_lexicon_tags_25 },
    { 0x276b743b, SUC_lexicon_tags_38 },
    { 0x7ab743a, SUC_lexicon_tags_17 },
    { 0x78b743d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4b7442, SUC_lexicon_tags_28 },
    { 0xe02b7443, SUC_lexicon_tags_1 },
    { 0x37ab7443, SUC_lexicon_tags_2 },
    { 0x690b7445, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932b7449, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34b744f, SUC_lexicon_tags_25 },
    { 0x93ab7450, SUC_lexicon_tags_33 },
    { 0xdacb744f, SUC_lexicon_tags_63 },
    { 0x11eb744f, SUC_lexicon_tags_26 },
    { 0xeb8b7453, SUC_lexicon_tags_37 },
    { 0x940b7453, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x98b7456, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70ab7458, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x836b745a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646b7461, SUC_lexicon_tags_15 },
    { 0xccab7462, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd3eb7464, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2b7467, SUC_lexicon_tags_17 },
    { 0x2a6b7467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcb746c, SUC_lexicon_tags_31 },
    { 0x880b746c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170b7474, SUC_lexicon_tags_11 },
    { 0xd2eb7475, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x736b7478, SUC_lexicon_tags_7 },
    { 0xe7ab7479, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b747e, SUC_lexicon_tags_12 },
    { 0xd54b747f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xf98b7481, SUC_lexicon_tags_27 },
    { 0x360b7482, SUC_lexicon_tags_2 },
    { 0x4aeb7482, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6b7487, SUC_lexicon_tags_27 },
    { 0xe02b7488, SUC_lexicon_tags_11 },
    { 0x55ab7487, SUC_lexicon_tags_15 },
    { 0xf30b7488, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cb748f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb7497, SUC_lexicon_tags_37 },
    { 0x24eb7497, SUC_lexicon_tags_55 },
    { 0x346b7498, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbeb749d, SUC_lexicon_tags_11 },
    { 0x4dcb749e, SUC_lexicon_tags_6 },
    { 0x32cb749f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x234b74a1, SUC_lexicon_tags_7 },
    { 0x26eb74a2, SUC_lexicon_tags_25 },
    { 0x27ab74a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x202b74a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x158b74a8, SUC_lexicon_tags_61 },
    { 0xa34b74a9, SUC_lexicon_tags_15 },
    { 0x686b74a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x340b74ac, SUC_lexicon_tags_37 },
    { 0x5bab74ad, SUC_lexicon_tags_1 },
    { 0x32ab74ad, SUC_lexicon_tags_10 },
    { 0xd64b74ae, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94b74b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcb74b7, SUC_lexicon_tags_12 },
    { 0x78b74b8, SUC_lexicon_tags_38 },
    { 0x704b74b8, SUC_lexicon_tags_29 },
    { 0xfdeb74ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceb74bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748b74c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5aeb74c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x170b74c6, SUC_lexicon_tags_15 },
    { 0x8e6b74c7, SUC_lexicon_tags_6 },
    { 0x682b74c6, SUC_lexicon_tags_25 },
    { 0xf68b74c9, SUC_lexicon_tags_0 },
    { 0xfdeb74ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eb74ce, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecb74d3, SUC_lexicon_tags_12 },
    { 0x938b74d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640b74d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61ab74da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x558b74dd, SUC_lexicon_tags_15 },
    { 0x76b74de, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaa4b74e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c8b74e2, SUC_lexicon_tags_25 },
    { 0x7c4b74e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70eb74e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x508b74e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39cb74ea, SUC_lexicon_tags_11 },
    { 0x8f2b74eb, SUC_lexicon_tags_11 },
    { 0x35eb74ec, SUC_lexicon_tags_11 },
    { 0xb04b74ed, SUC_lexicon_tags_11 },
    { 0xa96b74ee, SUC_lexicon_tags_27 },
    { 0x5cb74ef, SUC_lexicon_tags_0 },
    { 0x186b74f0, SUC_lexicon_tags_49 },
    { 0x8a0b74f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x732b74f3, SUC_lexicon_tags_17 },
    { 0x36cb74f4, SUC_lexicon_tags_151 },
    { 0x604b74f5, SUC_lexicon_tags_19 },
    { 0x93ab74f3, SUC_lexicon_tags_9 },
    { 0xeccb74f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2b8b74f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c4b74fb, SUC_lexicon_tags_87 },
    { 0x4ceb74fb, SUC_lexicon_tags_0 },
    { 0x92b74fb, SUC_lexicon_tags_17 },
    { 0x73cb74fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x148b7500, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb7506, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe9eb7508, SUC_lexicon_tags_11 },
    { 0x280b7509, SUC_lexicon_tags_12 },
    { 0x3f8b7509, SUC_lexicon_tags_15 },
    { 0x23eb750b, SUC_lexicon_tags_1 },
    { 0x9b8b7509, SUC_lexicon_tags_19 },
    { 0x9b2b7509, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34b7517, SUC_lexicon_tags_1 },
    { 0x1c2b7517, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2b751b, SUC_lexicon_tags_7 },
    { 0x220b751c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ab7520, SUC_lexicon_tags_14 },
    { 0x5beb7521, SUC_lexicon_tags_37 },
    { 0x686b7522, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x97cb7525, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182b752a, SUC_lexicon_tags_23 },
    { 0xf5cb752b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd4b752d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x64cb7530, SUC_lexicon_tags_28 },
    { 0x916b7531, SUC_lexicon_tags_87 },
    { 0xb76b7531, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00b7536, SUC_lexicon_tags_48 },
    { 0x468b7537, SUC_lexicon_tags_17 },
    { 0x496b7536, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b753b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ab7544, SUC_lexicon_tags_25 },
    { 0xec8b7545, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69eb754b, SUC_lexicon_tags_62 },
    { 0x5f6b754b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80b754e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b7551, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3cb7553, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6cb7555, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0b7558, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0b755b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98b755f, SUC_lexicon_tags_4 },
    { 0x22cb7560, SUC_lexicon_tags_28 },
    { 0x7b8b7560, SUC_lexicon_tags_23 },
    { 0x19eb7562, SUC_lexicon_tags_1 },
    { 0x7c2b7563, SUC_lexicon_tags_84 },
    { 0x5b6b755f, SUC_lexicon_tags_19 },
    { 0xd8b7565, SUC_lexicon_tags_35 },
    { 0x6cab7565, SUC_lexicon_tags_65 },
    { 0x12eb7566, SUC_lexicon_tags_234 },
    { 0xe9ab7567, SUC_lexicon_tags_7 },
    { 0x57ab7569, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb8eb756b, SUC_lexicon_tags_30 },
    { 0xc2ab756c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x6bab756e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1eb7570, SUC_lexicon_tags_15 },
    { 0x5ecb7571, SUC_lexicon_tags_15 },
    { 0x502b7572, SUC_lexicon_tags_35 },
    { 0x4d4b7573, SUC_lexicon_tags_1 },
    { 0xa66b7571, SUC_lexicon_tags_33 },
    { 0x93ab7575, SUC_lexicon_tags_60 },
    { 0x2f8b7575, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e4b7578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e0b757a, SUC_lexicon_tags_7 },
    { 0xc1eb757b, SUC_lexicon_tags_3 },
    { 0x134b757c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868b7580, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4b7585, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x30eb7588, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce4b758a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x914b758d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2b7594, SUC_lexicon_tags_25 },
    { 0x22b7595, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8b7598, SUC_lexicon_tags_28 },
    { 0x526b7599, SUC_lexicon_tags_43 },
    { 0xc06b759a, SUC_lexicon_tags_15 },
    { 0x7e4b7599, SUC_lexicon_tags_2 },
    { 0x572b759c, SUC_lexicon_tags_37 },
    { 0xe34b759d, SUC_lexicon_tags_19 },
    { 0x906b759b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ab75a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb68b75a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e2b75a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecb75ad, SUC_lexicon_tags_37 },
    { 0xbeb75ae, SUC_lexicon_tags_65 },
    { 0xa70b75ae, SUC_lexicon_tags_1 },
    { 0xc94b75b0, SUC_lexicon_tags_19 },
    { 0x372b75b1, SUC_lexicon_tags_10 },
    { 0x2ccb75b2, SUC_lexicon_tags_38 },
    { 0xf0b75b2, SUC_lexicon_tags_60 },
    { 0xeceb75b3, SUC_lexicon_tags_0 },
    { 0x77cb75b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x282b75b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6eb75b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe62b75bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ab75be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb75c2, SUC_lexicon_tags_30 },
    { 0xcdeb75c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf68b75c5, SUC_lexicon_tags_30 },
    { 0x194b75c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b75cc, SUC_lexicon_tags_30 },
    { 0x326b75cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdeeb75cf, SUC_lexicon_tags_21 },
    { 0x5a4b75d0, SUC_lexicon_tags_22 },
    { 0x39cb75d0, SUC_lexicon_tags_33 },
    { 0xc2b75d2, SUC_lexicon_tags_2 },
    { 0x8aab75d2, SUC_lexicon_tags_17 },
    { 0x952b75d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcb75d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd3cb75d9, SUC_lexicon_tags_1 },
    { 0xa24b75da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x428b75dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb75e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0b75e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4ab75e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb0b75eb, SUC_lexicon_tags_14 },
    { 0xc50b75ec, SUC_lexicon_tags_79 },
    { 0x6c4b75ec, SUC_lexicon_tags_38 },
    { 0x91ab75ec, SUC_lexicon_tags_37 },
    { 0xe0b75ee, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x40ab75f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb0b75f3, SUC_lexicon_tags_7 },
    { 0x9a0b75f3, SUC_lexicon_tags_27 },
    { 0xeb6b75f5, SUC_lexicon_tags_45 },
    { 0xfdeb75f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0b75fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb24b75fc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0b75ff, SUC_lexicon_tags_10 },
    { 0x440b75ff, SUC_lexicon_tags_30 },
    { 0x238b7600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x416b7603, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8eeb7605, SUC_lexicon_tags_37 },
    { 0x2bcb7606, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcb760a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cb760f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x648b7611, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6b7618, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x96ab761a, SUC_lexicon_tags_1 },
    { 0xaa2b761b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504b7620, SUC_lexicon_tags_11 },
    { 0x6c6b7621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97eb7625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaeb7627, SUC_lexicon_tags_12 },
    { 0x3f8b7628, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaacb762a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fab762f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8b7635, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0b763a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xffeb763d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeb7640, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48b7645, SUC_lexicon_tags_17 },
    { 0xcd2b7646, SUC_lexicon_tags_151 },
    { 0x612b7645, SUC_lexicon_tags_7 },
    { 0xd50b7645, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd66b764a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2feb764c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562b7651, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9eeb7653, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4b765a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x57eb765c, SUC_lexicon_tags_29 },
    { 0xed6b765d, SUC_lexicon_tags_37 },
    { 0x312b765d, SUC_lexicon_tags_47 },
    { 0xe40b765f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x932b7661, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92b7663, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ecb7665, SUC_lexicon_tags_46 },
    { 0xb1ab7665, SUC_lexicon_tags_30 },
    { 0x74ab7667, SUC_lexicon_tags_28 },
    { 0x842b7665, SUC_lexicon_tags_26 },
    { 0xd64b7665, SUC_lexicon_tags_23 },
    { 0xfd4b7667, SUC_lexicon_tags_107 },
    { 0x994b7669, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe3eb766d, SUC_lexicon_tags_47 },
    { 0x148b766e, SUC_lexicon_tags_43 },
    { 0xc4ab766e, SUC_lexicon_tags_11 },
    { 0x3eab7670, SUC_lexicon_tags_17 },
    { 0xf4eb7671, SUC_lexicon_tags_14 },
    { 0x93ab7672, SUC_lexicon_tags_37 },
    { 0xcf2b7670, SUC_lexicon_tags_25 },
    { 0xf80b7674, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf8ab7676, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x74cb7678, SUC_lexicon_tags_30 },
    { 0x63eb7678, SUC_lexicon_tags_11 },
    { 0x88b7679, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fab767d, SUC_lexicon_tags_17 },
    { 0xe24b767e, SUC_lexicon_tags_1 },
    { 0xa96b767e, SUC_lexicon_tags_7 },
    { 0xc9cb767e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x544b7682, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430b7688, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0b768b, SUC_lexicon_tags_11 },
    { 0x8b768c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682b7690, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x168b7692, SUC_lexicon_tags_17 },
    { 0x6a0b7693, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb7698, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb42b769a, SUC_lexicon_tags_27 },
    { 0xffab769b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc6b769d, SUC_lexicon_tags_31 },
    { 0xe80b769e, SUC_lexicon_tags_26 },
    { 0xfdcb769d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08b76a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb76ae, SUC_lexicon_tags_2 },
    { 0xd64b76af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95ab76b1, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x74eb76b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb0cb76b5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcf4b76b7, SUC_lexicon_tags_15 },
    { 0xb10b76b8, SUC_lexicon_tags_14 },
    { 0x708b76b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f2b76bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ab76c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ab76c3, SUC_lexicon_tags_37 },
    { 0xed0b76c4, SUC_lexicon_tags_11 },
    { 0xe5eb76c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfdeb76c7, SUC_lexicon_tags_0 },
    { 0x4beb76c7, SUC_lexicon_tags_12 },
    { 0xaecb76c9, SUC_lexicon_tags_17 },
    { 0x5e2b76c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436b76d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acb76d8, SUC_lexicon_tags_25 },
    { 0x22cb76d9, SUC_lexicon_tags_5 },
    { 0x514b76da, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98b76dd, SUC_lexicon_tags_21 },
    { 0x180b76de, SUC_lexicon_tags_25 },
    { 0x5eab76df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cb76e3, SUC_lexicon_tags_278 },
    { 0x1cb76e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12ab76e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cb76ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660b76f4, SUC_lexicon_tags_37 },
    { 0x1eb76f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbacb76f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x732b76f9, SUC_lexicon_tags_15 },
    { 0xd00b76fa, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x2beb76fd, SUC_lexicon_tags_7 },
    { 0xfe6b76fe, SUC_lexicon_tags_43 },
    { 0x5a2b76fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0b7702, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2b770e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aeb7711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e6b7713, SUC_lexicon_tags_17 },
    { 0xd76b7714, SUC_lexicon_tags_43 },
    { 0x92ab7715, SUC_lexicon_tags_31 },
    { 0x390b7714, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x812b7718, SUC_lexicon_tags_63 },
    { 0xc5eb7719, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08b771c, SUC_lexicon_tags_70 },
    { 0x64cb771d, SUC_lexicon_tags_0 },
    { 0xd6b771e, SUC_lexicon_tags_12 },
    { 0xd66b771f, SUC_lexicon_tags_11 },
    { 0xa2eb771d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4eeb7722, SUC_lexicon_tags_1 },
    { 0x54ab7723, SUC_lexicon_tags_25 },
    { 0xa0b7722, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x204b7727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0b772b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc90b772d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44eb7731, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9f0b7733, SUC_lexicon_tags_229 },
    { 0x980b7734, SUC_lexicon_tags_6 },
    { 0x5bab7735, SUC_lexicon_tags_8 },
    { 0x1f8b7735, SUC_lexicon_tags_10 },
    { 0xbe2b7736, SUC_lexicon_tags_60 },
    { 0x996b7738, SUC_lexicon_tags_1 },
    { 0xd4b7736, SUC_lexicon_tags_15 },
    { 0x106b773a, SUC_lexicon_tags_15 },
    { 0x892b773b, SUC_lexicon_tags_7 },
    { 0xec8b773b, SUC_lexicon_tags_61 },
    { 0x5aab773d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2eb7740, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa88b7742, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08b7746, SUC_lexicon_tags_47 },
    { 0x50ab7747, SUC_lexicon_tags_57 },
    { 0x4e2b7748, SUC_lexicon_tags_63 },
    { 0x332b7749, SUC_lexicon_tags_65 },
    { 0xd90b774a, SUC_lexicon_tags_14 },
    { 0xb30b774b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5c4b774d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44b7753, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x946b7755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0b7759, SUC_lexicon_tags_37 },
    { 0xe98b775a, SUC_lexicon_tags_151 },
    { 0xb4cb775b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382b775f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb72b7761, SUC_lexicon_tags_25 },
    { 0x8eab7761, SUC_lexicon_tags_45 },
    { 0x3fab7763, SUC_lexicon_tags_57 },
    { 0xa12b7764, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f8b7766, SUC_lexicon_tags_31 },
    { 0x1cab7767, SUC_lexicon_tags_19 },
    { 0x6deb7767, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x95ab776a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b776f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe36b7771, SUC_lexicon_tags_49 },
    { 0xe26b7772, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6b7775, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752b777b, SUC_lexicon_tags_12 },
    { 0xfceb777b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242b7780, SUC_lexicon_tags_7 },
    { 0xc16b7781, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xefcb7783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58cb7785, SUC_lexicon_tags_77 },
    { 0xd6cb7786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97ab7788, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b778b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4eb778d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4b7792, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4cb7794, SUC_lexicon_tags_1 },
    { 0x52ab7795, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaf2b7797, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4b779c, SUC_lexicon_tags_77 },
    { 0x8b2b779d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x144b779f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaeb77a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf8b77a4, SUC_lexicon_tags_37 },
    { 0xb4eb77a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82b77a8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6b77ab, SUC_lexicon_tags_15 },
    { 0xef6b77ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8cb77ae, SUC_lexicon_tags_15 },
    { 0x34eb77af, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6b77b5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x382b77b8, SUC_lexicon_tags_19 },
    { 0xe20b77b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1bab77bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb30b77bd, SUC_lexicon_tags_41 },
    { 0xab8b77be, SUC_lexicon_tags_37 },
    { 0x9f6b77bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ab77c3, SUC_lexicon_tags_116 },
    { 0x5eeb77c4, SUC_lexicon_tags_2 },
    { 0xc70b77c5, SUC_lexicon_tags_23 },
    { 0x9e0b77c5, SUC_lexicon_tags_11 },
    { 0x23cb77c7, SUC_lexicon_tags_63 },
    { 0x67eb77c8, SUC_lexicon_tags_1 },
    { 0x93eb77c8, SUC_lexicon_tags_30 },
    { 0x372b77ca, SUC_lexicon_tags_28 },
    { 0x158b77ca, SUC_lexicon_tags_7 },
    { 0xebeb77cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780b77d0, SUC_lexicon_tags_6 },
    { 0xa0eb77d0, SUC_lexicon_tags_26 },
    { 0xab2b77d2, SUC_lexicon_tags_37 },
    { 0xc60b77d0, SUC_lexicon_tags_12 },
    { 0x3ccb77d3, SUC_lexicon_tags_11 },
    { 0x86b77d2, SUC_lexicon_tags_6 },
    { 0xa38b77d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70b77d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450b77dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0b77e2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cb77e7, SUC_lexicon_tags_9 },
    { 0x2c8b77e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2b77f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x296b77fa, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x10cb77fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4eb77fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x834b7800, SUC_lexicon_tags_15 },
    { 0xce2b7800, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b8b7803, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960b7809, SUC_lexicon_tags_17 },
    { 0xd66b780a, SUC_lexicon_tags_37 },
    { 0xdf4b780a, SUC_lexicon_tags_7 },
    { 0x210b780a, SUC_lexicon_tags_25 },
    { 0x584b780d, SUC_lexicon_tags_17 },
    { 0xc62b780e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd42b7810, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x110b7812, SUC_lexicon_tags_12 },
    { 0xb8eb7812, SUC_lexicon_tags_91 },
    { 0x8beb7813, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ab7817, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eb781d, SUC_lexicon_tags_12 },
    { 0xd2eb781d, SUC_lexicon_tags_1 },
    { 0xda8b781d, SUC_lexicon_tags_23 },
    { 0x94cb781f, SUC_lexicon_tags_11 },
    { 0x158b7821, SUC_lexicon_tags_60 },
    { 0x458b781d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b7825, SUC_lexicon_tags_1 },
    { 0xa64b7826, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cb7829, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeb782f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ab7835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388b7839, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x50cb783c, SUC_lexicon_tags_23 },
    { 0x68eb783d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb10b783f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6d8b7841, SUC_lexicon_tags_46 },
    { 0xf08b7841, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0b7845, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd56b7847, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc54b7849, SUC_lexicon_tags_15 },
    { 0xb98b784a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda0b784c, SUC_lexicon_tags_45 },
    { 0xb86b784c, SUC_lexicon_tags_63 },
    { 0xea4b784e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x670b7850, SUC_lexicon_tags_63 },
    { 0x998b7851, SUC_lexicon_tags_4 },
    { 0x7c4b7850, SUC_lexicon_tags_11 },
    { 0xf78b7851, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94b7859, SUC_lexicon_tags_49 },
    { 0x954b7859, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb785d, SUC_lexicon_tags_25 },
    { 0x570b785e, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x89eb7861, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bab7864, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x632b7866, SUC_lexicon_tags_11 },
    { 0x1f8b7866, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112b786e, SUC_lexicon_tags_1 },
    { 0xc2ab786f, SUC_lexicon_tags_12 },
    { 0xf4eb7870, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256b7874, SUC_lexicon_tags_38 },
    { 0xbb6b7875, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2ab7877, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb70b7879, SUC_lexicon_tags_21 },
    { 0xaeeb787a, SUC_lexicon_tags_25 },
    { 0xf90b787b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x444b787e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4b7881, SUC_lexicon_tags_60 },
    { 0x29eb7882, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ecb7884, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4b788a, SUC_lexicon_tags_25 },
    { 0xa46b788b, SUC_lexicon_tags_27 },
    { 0xd98b788c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936b7891, SUC_lexicon_tags_37 },
    { 0x7e2b7892, SUC_lexicon_tags_37 },
    { 0x50ab7892, SUC_lexicon_tags_7 },
    { 0xe46b7891, SUC_lexicon_tags_47 },
    { 0xdbcb7891, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ab789a, SUC_lexicon_tags_56 },
    { 0x4a6b789b, SUC_lexicon_tags_16 },
    { 0x200b789a, SUC_lexicon_tags_17 },
    { 0xf16b789b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aab789f, SUC_lexicon_tags_37 },
    { 0xd8eb78a0, SUC_lexicon_tags_4 },
    { 0xa64b78a0, SUC_lexicon_tags_16 },
    { 0x238b78a2, SUC_lexicon_tags_0 },
    { 0x93cb78a3, SUC_lexicon_tags_21 },
    { 0xc86b78a4, SUC_lexicon_tags_107 },
    { 0x5aeb78a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffeb78a9, SUC_lexicon_tags_27 },
    { 0x27ab78aa, SUC_lexicon_tags_38 },
    { 0xc98b78ab, SUC_lexicon_tags_26 },
    { 0xd08b78ac, SUC_lexicon_tags_2 },
    { 0x4a2b78ad, SUC_lexicon_tags_31 },
    { 0xbc2b78ae, SUC_lexicon_tags_2 },
    { 0x748b78ae, SUC_lexicon_tags_16 },
    { 0x88eb78af, SUC_lexicon_tags_12 },
    { 0x9e4b78b1, SUC_lexicon_tags_11 },
    { 0xd30b78b2, SUC_lexicon_tags_11 },
    { 0x432b78ac, SUC_lexicon_tags_6 },
    { 0x63cb78b4, SUC_lexicon_tags_1 },
    { 0x90cb78b3, SUC_lexicon_tags_49 },
    { 0xf0eb78b3, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4b78b9, SUC_lexicon_tags_37 },
    { 0x678b78ba, SUC_lexicon_tags_49 },
    { 0x9f6b78bb, SUC_lexicon_tags_0 },
    { 0x692b78bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60cb78be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccb78c1, SUC_lexicon_tags_37 },
    { 0x1bab78c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81eb78cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x520b78cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910b78d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8b78dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cb78e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x19eb78e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8b4b78e9, SUC_lexicon_tags_7 },
    { 0x6d8b78ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3eb78ef, SUC_lexicon_tags_15 },
    { 0x4ceb78ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22b78f5, SUC_lexicon_tags_45 },
    { 0x45cb78f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8b78f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x614b78fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372b7900, SUC_lexicon_tags_33 },
    { 0x388b7900, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x134b7903, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6b7906, SUC_lexicon_tags_2 },
    { 0x648b7906, SUC_lexicon_tags_2 },
    { 0x24cb7908, SUC_lexicon_tags_28 },
    { 0xd4eb7906, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6b790c, SUC_lexicon_tags_29 },
    { 0x876b790d, SUC_lexicon_tags_2 },
    { 0xb38b790d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8b7912, SUC_lexicon_tags_27 },
    { 0x126b7913, SUC_lexicon_tags_23 },
    { 0x6e2b7914, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb7919, SUC_lexicon_tags_17 },
    { 0xea2b7919, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8b791f, SUC_lexicon_tags_37 },
    { 0xcfab7920, SUC_lexicon_tags_10 },
    { 0x2c8b7921, SUC_lexicon_tags_12 },
    { 0xd86b7922, SUC_lexicon_tags_6 },
    { 0x464b7921, SUC_lexicon_tags_2 },
    { 0xa0cb7924, SUC_lexicon_tags_19 },
    { 0x95ab7924, SUC_lexicon_tags_17 },
    { 0x2f4b7926, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcb792f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb7938, SUC_lexicon_tags_43 },
    { 0x3b2b7938, SUC_lexicon_tags_15 },
    { 0x5feb793a, SUC_lexicon_tags_33 },
    { 0x894b793b, SUC_lexicon_tags_17 },
    { 0xc2b793c, SUC_lexicon_tags_30 },
    { 0x98eb793a, SUC_lexicon_tags_7 },
    { 0x1dcb793e, SUC_lexicon_tags_1 },
    { 0x2f2b793f, SUC_lexicon_tags_55 },
    { 0xde6b7940, SUC_lexicon_tags_7 },
    { 0xb2b7941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1eb7943, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b6b7945, SUC_lexicon_tags_25 },
    { 0x70b7946, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2b794e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b4b7950, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb7955, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8b7961, SUC_lexicon_tags_2 },
    { 0x25eb7961, SUC_lexicon_tags_11 },
    { 0xd66b7962, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab7967, SUC_lexicon_tags_15 },
    { 0x1feb7968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14eb796a, SUC_lexicon_tags_0 },
    { 0xa40b796b, SUC_lexicon_tags_33 },
    { 0x614b796c, SUC_lexicon_tags_12 },
    { 0xc2ab796c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x222b7970, SUC_lexicon_tags_37 },
    { 0xf0ab7971, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aab797b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0b797e, SUC_lexicon_tags_151 },
    { 0x192b797f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510b7983, SUC_lexicon_tags_11 },
    { 0x33ab7984, SUC_lexicon_tags_11 },
    { 0xabeb7984, SUC_lexicon_tags_12 },
    { 0x420b7984, SUC_lexicon_tags_43 },
    { 0x92eb7987, SUC_lexicon_tags_88 },
    { 0x484b7988, SUC_lexicon_tags_38 },
    { 0xdaab7987, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0eb798b, SUC_lexicon_tags_55 },
    { 0xaf8b798c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b7991, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b0b7993, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8b7997, SUC_lexicon_tags_17 },
    { 0xe5ab7997, SUC_lexicon_tags_49 },
    { 0xdb6b7998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa70b799b, SUC_lexicon_tags_8 },
    { 0x758b799b, SUC_lexicon_tags_25 },
    { 0x4b0b799c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4aab799f, SUC_lexicon_tags_23 },
    { 0xb90b79a0, SUC_lexicon_tags_1 },
    { 0xaab79a1, SUC_lexicon_tags_15 },
    { 0xeb799f, SUC_lexicon_tags_138 },
    { 0xe86b79a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb79a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x364b79ab, SUC_lexicon_tags_25 },
    { 0x2f2b79ac, SUC_lexicon_tags_6 },
    { 0xe76b79ad, SUC_lexicon_tags_27 },
    { 0xe9cb79ac, SUC_lexicon_tags_45 },
    { 0x886b79af, SUC_lexicon_tags_27 },
    { 0x8f2b79af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb79b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8dab79b6, SUC_lexicon_tags_37 },
    { 0xdf4b79b6, SUC_lexicon_tags_7 },
    { 0x5ecb79b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x254b79ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cb79bd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872b79c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40cb79c3, SUC_lexicon_tags_6 },
    { 0xad6b79c4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4b79c7, SUC_lexicon_tags_1 },
    { 0x9f8b79c8, SUC_lexicon_tags_148 },
    { 0x142b79c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b79ce, SUC_lexicon_tags_11 },
    { 0xf4ab79ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0b79d2, SUC_lexicon_tags_19 },
    { 0xa58b79d3, SUC_lexicon_tags_37 },
    { 0xe6cb79d2, SUC_lexicon_tags_43 },
    { 0xb12b79d5, SUC_lexicon_tags_26 },
    { 0xf70b79d6, SUC_lexicon_tags_35 },
    { 0x78b79d2, SUC_lexicon_tags_37 },
    { 0x18eb79d8, SUC_lexicon_tags_36 },
    { 0x742b79d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42b79e6, SUC_lexicon_tags_1 },
    { 0xb12b79e6, SUC_lexicon_tags_83 },
    { 0x85cb79e8, SUC_lexicon_tags_201 },
    { 0xbfeb79e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50b79ef, SUC_lexicon_tags_8 },
    { 0x6c4b79f0, SUC_lexicon_tags_29 },
    { 0x92eb79f1, SUC_lexicon_tags_17 },
    { 0xb6ab79f2, SUC_lexicon_tags_7 },
    { 0x3e2b79f2, SUC_lexicon_tags_7 },
    { 0xa60b79f4, SUC_lexicon_tags_25 },
    { 0x8c6b79f5, SUC_lexicon_tags_14 },
    { 0xcfeb79f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90b79f8, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2b79ff, SUC_lexicon_tags_45 },
    { 0x74eb7a00, SUC_lexicon_tags_0 },
    { 0xb68b7a01, SUC_lexicon_tags_12 },
    { 0x38cb7a02, SUC_lexicon_tags_23 },
    { 0xc7eb7a00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xceeb7a06, SUC_lexicon_tags_106 },
    { 0xf2cb7a06, SUC_lexicon_tags_15 },
    { 0x728b7a08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecb7a0a, SUC_lexicon_tags_9 },
    { 0x1c0b7a0b, SUC_lexicon_tags_12 },
    { 0xa20b7a0b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x960b7a0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x650b7a12, SUC_lexicon_tags_60 },
    { 0xafeb7a13, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18b7a18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x106b7a1a, SUC_lexicon_tags_2 },
    { 0x576b7a1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6b7a21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x78b7a23, SUC_lexicon_tags_38 },
    { 0x3f2b7a24, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x230b7a27, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aab7a2f, SUC_lexicon_tags_12 },
    { 0x436b7a30, SUC_lexicon_tags_32 },
    { 0x8cab7a30, SUC_lexicon_tags_106 },
    { 0x18ab7a31, SUC_lexicon_tags_8 },
    { 0xccab7a33, SUC_lexicon_tags_30 },
    { 0x672b7a34, SUC_lexicon_tags_14 },
    { 0xe64b7a32, SUC_lexicon_tags_68 },
    { 0xc32b7a35, SUC_lexicon_tags_27 },
    { 0xd6ab7a37, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2e6b7a39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97cb7a3b, SUC_lexicon_tags_45 },
    { 0x1aeb7a3c, SUC_lexicon_tags_8 },
    { 0x38cb7a3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96b7a40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3cb7a42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53eb7a44, SUC_lexicon_tags_77 },
    { 0x17ab7a45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaeab7a47, SUC_lexicon_tags_28 },
    { 0xe02b7a48, SUC_lexicon_tags_60 },
    { 0xfc6b7a49, SUC_lexicon_tags_31 },
    { 0x99eb7a49, SUC_lexicon_tags_65 },
    { 0xc0ab7a4b, SUC_lexicon_tags_12 },
    { 0xaf0b7a4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x784b7a4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab7a54, SUC_lexicon_tags_27 },
    { 0xd72b7a55, SUC_lexicon_tags_6 },
    { 0xaaab7a55, SUC_lexicon_tags_35 },
    { 0x718b7a57, SUC_lexicon_tags_12 },
    { 0x2b6b7a58, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872b7a61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa0b7a63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534b7a68, SUC_lexicon_tags_9 },
    { 0x9c0b7a68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x840b7a6c, SUC_lexicon_tags_12 },
    { 0xcb8b7a6c, SUC_lexicon_tags_0 },
    { 0x432b7a6d, SUC_lexicon_tags_25 },
    { 0xdaab7a6f, SUC_lexicon_tags_14 },
    { 0x6b4b7a6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74eb7a73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88cb7a75, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b7a78, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8c4b7a7a, SUC_lexicon_tags_3 },
    { 0xd68b7a7b, SUC_lexicon_tags_25 },
    { 0x2a0b7a7c, SUC_lexicon_tags_49 },
    { 0xb8b7a7d, SUC_lexicon_tags_25 },
    { 0x4b4b7a7d, SUC_lexicon_tags_0 },
    { 0x578b7a7b, SUC_lexicon_tags_6 },
    { 0x8bab7a7c, SUC_lexicon_tags_11 },
    { 0x7cab7a7a, SUC_lexicon_tags_15 },
    { 0xb52b7a82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x678b7a84, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f6b7a86, SUC_lexicon_tags_11 },
    { 0xef4b7a87, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xcc2b7a89, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xab0b7a8b, SUC_lexicon_tags_1505 },
    { 0, NULL },
    { 0, NULL },
    { 0x966b7a8e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366b7a92, SUC_lexicon_tags_37 },
    { 0xf3eb7a92, SUC_lexicon_tags_29 },
    { 0x5a8b7a92, SUC_lexicon_tags_26 },
    { 0x978b7a95, SUC_lexicon_tags_37 },
    { 0x96cb7a96, SUC_lexicon_tags_26 },
    { 0x250b7a97, SUC_lexicon_tags_6 },
    { 0xbaeb7a98, SUC_lexicon_tags_1 },
    { 0x696b7a92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8b7a9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fab7a9e, SUC_lexicon_tags_45 },
    { 0xd44b7a9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43eb7aa1, SUC_lexicon_tags_1 },
    { 0xebeb7aa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658b7aa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2b7aac, SUC_lexicon_tags_1 },
    { 0xf04b7aad, SUC_lexicon_tags_1 },
    { 0x1a4b7aad, SUC_lexicon_tags_47 },
    { 0x82eb7aaf, SUC_lexicon_tags_84 },
    { 0xbf8b7ab0, SUC_lexicon_tags_88 },
    { 0xc2eb7ab1, SUC_lexicon_tags_33 },
    { 0xd46b7ab1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2b7ab5, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x39eb7ab8, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b7abc, SUC_lexicon_tags_23 },
    { 0xa6eb7abd, SUC_lexicon_tags_37 },
    { 0xfaab7abe, SUC_lexicon_tags_32 },
    { 0xe6b7abd, SUC_lexicon_tags_13 },
    { 0x50eb7abc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3eb7ac2, SUC_lexicon_tags_30 },
    { 0x9c8b7ac3, SUC_lexicon_tags_12 },
    { 0xfdeb7ac2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cb7ac8, SUC_lexicon_tags_62 },
    { 0xbaeb7ac9, SUC_lexicon_tags_84 },
    { 0xbbeb7ac8, SUC_lexicon_tags_2 },
    { 0x688b7aca, SUC_lexicon_tags_17 },
    { 0x2d0b7aca, SUC_lexicon_tags_9 },
    { 0xd0ab7acb, SUC_lexicon_tags_0 },
    { 0xe40b7ac9, SUC_lexicon_tags_49 },
    { 0x62ab7acf, SUC_lexicon_tags_19 },
    { 0xae8b7ad0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d8b7ad2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x466b7ad5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa92b7ad7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb32b7ad9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5beb7add, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0b7ae2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf0b7ae4, SUC_lexicon_tags_17 },
    { 0xa6eb7ae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcb7ae9, SUC_lexicon_tags_61 },
    { 0x728b7aea, SUC_lexicon_tags_26 },
    { 0x84b7aea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b7aef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa14b7af1, SUC_lexicon_tags_17 },
    { 0x53cb7af2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x660b7af5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb94b7af7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x372b7af9, SUC_lexicon_tags_11 },
    { 0x42b7af9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884b7aff, SUC_lexicon_tags_11 },
    { 0xe3eb7b00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2b7b03, SUC_lexicon_tags_52 },
    { 0x59cb7b04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ab7b07, SUC_lexicon_tags_37 },
    { 0xa04b7b08, SUC_lexicon_tags_17 },
    { 0xfd6b7b08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ab7b0c, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0x276b7b0f, SUC_lexicon_tags_1 },
    { 0x422b7b10, SUC_lexicon_tags_1 },
    { 0x674b7b11, SUC_lexicon_tags_49 },
    { 0xa62b7b11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44b7b16, SUC_lexicon_tags_29 },
    { 0x2c0b7b17, SUC_lexicon_tags_19 },
    { 0x82ab7b17, SUC_lexicon_tags_32 },
    { 0xbceb7b16, SUC_lexicon_tags_37 },
    { 0xc2eb7b19, SUC_lexicon_tags_25 },
    { 0x434b7b17, SUC_lexicon_tags_88 },
    { 0xdc4b7b19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x526b7b1e, SUC_lexicon_tags_39 },
    { 0xfbcb7b1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6b7b26, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480b7b2c, SUC_lexicon_tags_15 },
    { 0xa10b7b2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x376b7b30, SUC_lexicon_tags_27 },
    { 0xe1ab7b30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eeb7b35, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x822b7b37, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb9cb7b39, SUC_lexicon_tags_7 },
    { 0x5c8b7b3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccb7b3e, SUC_lexicon_tags_60 },
    { 0xd34b7b3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99cb7b41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14b7b45, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14b7b49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6b7b50, SUC_lexicon_tags_14 },
    { 0x34cb7b50, SUC_lexicon_tags_17 },
    { 0xd96b7b51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0b7b55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcb7b58, SUC_lexicon_tags_8 },
    { 0xe96b7b59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93ab7b5b, SUC_lexicon_tags_38 },
    { 0x658b7b5c, SUC_lexicon_tags_20 },
    { 0xe72b7b5b, SUC_lexicon_tags_14 },
    { 0x184b7b5e, SUC_lexicon_tags_9 },
    { 0x3a0b7b5c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf6ab7b61, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68b7b65, SUC_lexicon_tags_8 },
    { 0xecab7b66, SUC_lexicon_tags_1 },
    { 0xe54b7b67, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb7b6a, SUC_lexicon_tags_12 },
    { 0x158b7b6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0b7b75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49eb7b79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdeb7b7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa24b7b7d, SUC_lexicon_tags_22 },
    { 0x394b7b7e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68b7b83, SUC_lexicon_tags_17 },
    { 0x112b7b83, SUC_lexicon_tags_1 },
    { 0x762b7b85, SUC_lexicon_tags_2 },
    { 0xa56b7b86, SUC_lexicon_tags_10 },
    { 0xea4b7b87, SUC_lexicon_tags_11 },
    { 0x510b7b83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ccb7b8a, SUC_lexicon_tags_1 },
    { 0xbc8b7b8b, SUC_lexicon_tags_12 },
    { 0xdc0b7b8b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0b7b8f, SUC_lexicon_tags_14 },
    { 0xbeb7b90, SUC_lexicon_tags_12 },
    { 0x6f8b7b91, SUC_lexicon_tags_37 },
    { 0x548b7b90, SUC_lexicon_tags_29 },
    { 0x856b7b92, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700b7b97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ab7b9a, SUC_lexicon_tags_27 },
    { 0x674b7b9b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb3eb7b9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774b7ba1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cb7ba4, SUC_lexicon_tags_66 },
    { 0x6b4b7ba5, SUC_lexicon_tags_33 },
    { 0x13eb7ba6, SUC_lexicon_tags_2 },
    { 0x99cb7ba7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8b7bac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ab7baf, SUC_lexicon_tags_7 },
    { 0x74eb7baf, SUC_lexicon_tags_98 },
    { 0xb48b7baf, SUC_lexicon_tags_17 },
    { 0xee6b7bb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x712b7bb4, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540b7bb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1beb7bc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x782b7bc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4feb7bc5, SUC_lexicon_tags_11 },
    { 0x7c4b7bc6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x37ab7bc8, SUC_lexicon_tags_1 },
    { 0xe8eb7bc9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb0eb7bcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30b7bd0, SUC_lexicon_tags_14 },
    { 0x2aeb7bd1, SUC_lexicon_tags_1 },
    { 0x22cb7bd2, SUC_lexicon_tags_38 },
    { 0xc40b7bd1, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xa7eb7bd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6cb7bd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf56b7bd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eb7bdd, SUC_lexicon_tags_12 },
    { 0xe60b7bde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a4b7be0, SUC_lexicon_tags_23 },
    { 0x1c6b7be1, SUC_lexicon_tags_1 },
    { 0x542b7be2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1eb7be5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6b7be8, SUC_lexicon_tags_79 },
    { 0x4a6b7be9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcb7bed, SUC_lexicon_tags_1 },
    { 0xde0b7bee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cb7bf2, SUC_lexicon_tags_44 },
    { 0xb06b7bf2, SUC_lexicon_tags_17 },
    { 0x13eb7bf4, SUC_lexicon_tags_17 },
    { 0xcd2b7bf5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370b7bfb, SUC_lexicon_tags_6 },
    { 0x852b7bfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc2b7bfe, SUC_lexicon_tags_11 },
    { 0xa64b7bff, SUC_lexicon_tags_17 },
    { 0x620b7bff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcb7c06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6dcb7c08, SUC_lexicon_tags_17 },
    { 0xbab7c09, SUC_lexicon_tags_23 },
    { 0xcc6b7c09, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174b7c19, SUC_lexicon_tags_15 },
    { 0xb58b7c1a, SUC_lexicon_tags_17 },
    { 0x61cb7c1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x760b7c1e, SUC_lexicon_tags_25 },
    { 0x2f0b7c1f, SUC_lexicon_tags_84 },
    { 0xcd4b7c1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8b7c23, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb7c2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x95ab7c2d, SUC_lexicon_tags_34 },
    { 0xe9ab7c2e, SUC_lexicon_tags_25 },
    { 0x9a6b7c2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cb7c31, SUC_lexicon_tags_1 },
    { 0x2e4b7c31, SUC_lexicon_tags_47 },
    { 0xdd2b7c33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b7c36, SUC_lexicon_tags_25 },
    { 0x87eb7c37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9deb7c3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b7c41, SUC_lexicon_tags_5 },
    { 0x3b0b7c42, SUC_lexicon_tags_30 },
    { 0xde4b7c42, SUC_lexicon_tags_14 },
    { 0x124b7c43, SUC_lexicon_tags_17 },
    { 0x7feb7c45, SUC_lexicon_tags_2 },
    { 0x624b7c46, SUC_lexicon_tags_12 },
    { 0xe26b7c47, SUC_lexicon_tags_16 },
    { 0x16cb7c46, SUC_lexicon_tags_17 },
    { 0x5a8b7c45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x890b7c4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762b7c52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecb7c56, SUC_lexicon_tags_17 },
    { 0xff2b7c57, SUC_lexicon_tags_79 },
    { 0xcf6b7c58, SUC_lexicon_tags_7 },
    { 0xd3cb7c59, SUC_lexicon_tags_63 },
    { 0x1e4b7c5a, SUC_lexicon_tags_2 },
    { 0x510b7c58, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ab7c5e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x334b7c60, SUC_lexicon_tags_25 },
    { 0x9beb7c61, SUC_lexicon_tags_27 },
    { 0xd24b7c62, SUC_lexicon_tags_12 },
    { 0x60eb7c63, SUC_lexicon_tags_35 },
    { 0xa14b7c64, SUC_lexicon_tags_63 },
    { 0x616b7c65, SUC_lexicon_tags_45 },
    { 0xebeb7c62, SUC_lexicon_tags_29 },
    { 0xf8b7c64, SUC_lexicon_tags_28 },
    { 0xc6eb7c60, SUC_lexicon_tags_12 },
    { 0x2eeb7c69, SUC_lexicon_tags_1 },
    { 0xb9ab7c68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x520b7c6d, SUC_lexicon_tags_25 },
    { 0x138b7c6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x384b7c70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x446b7c72, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x604b7c74, SUC_lexicon_tags_0 },
    { 0xa8cb7c75, SUC_lexicon_tags_37 },
    { 0xc0cb7c76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x678b7c78, SUC_lexicon_tags_6 },
    { 0xae0b7c79, SUC_lexicon_tags_1 },
    { 0xf0cb7c78, SUC_lexicon_tags_2 },
    { 0x4c0b7c79, SUC_lexicon_tags_11 },
    { 0x610b7c7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e6b7c7e, SUC_lexicon_tags_15 },
    { 0x974b7c7f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4b7c86, SUC_lexicon_tags_3 },
    { 0x212b7c86, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4b7c8f, SUC_lexicon_tags_0 },
    { 0x380b7c90, SUC_lexicon_tags_49 },
    { 0x4feb7c91, SUC_lexicon_tags_1 },
    { 0xa2b7c90, SUC_lexicon_tags_17 },
    { 0xbe8b7c93, SUC_lexicon_tags_17 },
    { 0x9ab7c90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eb7c98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb2b7c9a, SUC_lexicon_tags_17 },
    { 0x594b7c9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6b7ca0, SUC_lexicon_tags_25 },
    { 0xa0ab7ca1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x446b7ca3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecab7ca6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd42b7ca8, SUC_lexicon_tags_60 },
    { 0xa5ab7ca9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8eb7cac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf98b7cae, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xacab7cb1, SUC_lexicon_tags_32 },
    { 0x6b8b7cb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30b7cb5, SUC_lexicon_tags_46 },
    { 0x9f0b7cb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8b7cba, SUC_lexicon_tags_1 },
    { 0xa9ab7cba, SUC_lexicon_tags_33 },
    { 0x5e8b7cba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceeb7cc7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60b7ccb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7eb7cce, SUC_lexicon_tags_27 },
    { 0xfe2b7ccf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x112b7cd1, SUC_lexicon_tags_11 },
    { 0xe3cb7cd1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ab7cd8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4b7cde, SUC_lexicon_tags_11 },
    { 0x55cb7cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8eb7ce1, SUC_lexicon_tags_60 },
    { 0xf9eb7ce2, SUC_lexicon_tags_37 },
    { 0x7fab7ce3, SUC_lexicon_tags_35 },
    { 0xafcb7ce4, SUC_lexicon_tags_28 },
    { 0xd2cb7ce5, SUC_lexicon_tags_12 },
    { 0x374b7ce5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf60b7ce8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886b7cec, SUC_lexicon_tags_37 },
    { 0xa2cb7cec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cb7cef, SUC_lexicon_tags_12 },
    { 0x150b7cef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bab7cf2, SUC_lexicon_tags_4 },
    { 0x5e4b7cf2, SUC_lexicon_tags_7 },
    { 0x272b7cf3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b7d00, SUC_lexicon_tags_6 },
    { 0x436b7d01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfab7d08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x372b7d0b, SUC_lexicon_tags_10 },
    { 0xdc2b7d0c, SUC_lexicon_tags_37 },
    { 0x73ab7d0d, SUC_lexicon_tags_2 },
    { 0xc04b7d0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8eb7d11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa3cb7d13, SUC_lexicon_tags_37 },
    { 0xe5cb7d14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x710b7d16, SUC_lexicon_tags_45 },
    { 0xbc4b7d16, SUC_lexicon_tags_37 },
    { 0x7b8b7d18, SUC_lexicon_tags_62 },
    { 0x8b8b7d18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x508b7d1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2b7d1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaeb7d26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9ab7d28, SUC_lexicon_tags_46 },
    { 0x568b7d29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70b7d2d, SUC_lexicon_tags_55 },
    { 0x21ab7d2d, SUC_lexicon_tags_25 },
    { 0x918b7d2d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xadcb7d31, SUC_lexicon_tags_49 },
    { 0x514b7d32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cb7d36, SUC_lexicon_tags_33 },
    { 0x53cb7d37, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb8cb7d39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4eb7d3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8feb7d41, SUC_lexicon_tags_37 },
    { 0x1f0b7d41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ceb7d4c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ab7d52, SUC_lexicon_tags_25 },
    { 0x530b7d52, SUC_lexicon_tags_68 },
    { 0x2fab7d52, SUC_lexicon_tags_14 },
    { 0x534b7d55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b7d5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x73eb7d5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5ab7d5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x52b7d60, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1eb7d65, SUC_lexicon_tags_4 },
    { 0x3e2b7d66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6b7d6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x49eb7d6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b7d71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcb7d76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x132b7d79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa6b7d7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cb7d7f, SUC_lexicon_tags_17 },
    { 0x974b7d80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4b7d84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3dcb7d86, SUC_lexicon_tags_37 },
    { 0x2e2b7d87, SUC_lexicon_tags_10 },
    { 0xcbab7d87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2b7d8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x920b7d8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50b7d8f, SUC_lexicon_tags_37 },
    { 0xe8cb7d90, SUC_lexicon_tags_29 },
    { 0x1f6b7d8f, SUC_lexicon_tags_37 },
    { 0x1b8b7d91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafcb7d94, SUC_lexicon_tags_16 },
    { 0x6ecb7d95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cb7d99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ab7d9e, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4b7da3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664b7daa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21cb7dac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x954b7dae, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xde8b7db0, SUC_lexicon_tags_15 },
    { 0xbf2b7db1, SUC_lexicon_tags_6 },
    { 0xd2cb7db2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x812b7db5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510b7db9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x514b7dbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0b7dc0, SUC_lexicon_tags_17 },
    { 0xa66b7dc1, SUC_lexicon_tags_0 },
    { 0x9e6b7dc1, SUC_lexicon_tags_25 },
    { 0xfecb7dc2, SUC_lexicon_tags_15 },
    { 0x38ab7dc4, SUC_lexicon_tags_14 },
    { 0x85ab7dc4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x844b7dc7, SUC_lexicon_tags_19 },
    { 0xc46b7dc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b7dcc, SUC_lexicon_tags_19 },
    { 0xaaab7dcc, SUC_lexicon_tags_25 },
    { 0xffcb7dce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96b7dd5, SUC_lexicon_tags_18 },
    { 0x524b7dd6, SUC_lexicon_tags_11 },
    { 0xe00b7dd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57eb7dd9, SUC_lexicon_tags_30 },
    { 0x564b7dda, SUC_lexicon_tags_79 },
    { 0xc1cb7ddb, SUC_lexicon_tags_7 },
    { 0xad8b7ddc, SUC_lexicon_tags_15 },
    { 0xd62b7ddd, SUC_lexicon_tags_19 },
    { 0x7f8b7ddd, SUC_lexicon_tags_10 },
    { 0xf88b7ddc, SUC_lexicon_tags_29 },
    { 0xbc0b7de0, SUC_lexicon_tags_43 },
    { 0x370b7de0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724b7de5, SUC_lexicon_tags_25 },
    { 0x570b7de6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46b7dea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ab7df9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0b7dfb, SUC_lexicon_tags_37 },
    { 0xa12b7dfc, SUC_lexicon_tags_37 },
    { 0x474b7dfd, SUC_lexicon_tags_17 },
    { 0x9d6b7dfd, SUC_lexicon_tags_48 },
    { 0x5dab7dff, SUC_lexicon_tags_47 },
    { 0x184b7dfd, SUC_lexicon_tags_26 },
    { 0x88b7e01, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x556b7e04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252b7e0e, SUC_lexicon_tags_37 },
    { 0x36cb7e0f, SUC_lexicon_tags_15 },
    { 0xb18b7e0e, SUC_lexicon_tags_48 },
    { 0x210b7e11, SUC_lexicon_tags_15 },
    { 0xd2eb7e12, SUC_lexicon_tags_11 },
    { 0x7ceb7e13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdecb7e16, SUC_lexicon_tags_15 },
    { 0xc28b7e17, SUC_lexicon_tags_37 },
    { 0x5dcb7e18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecb7e1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8eb7e1d, SUC_lexicon_tags_11 },
    { 0xe08b7e1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6b7e22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54eb7e24, SUC_lexicon_tags_55 },
    { 0x98eb7e24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78b7e2a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8b7e38, SUC_lexicon_tags_0 },
    { 0xe98b7e39, SUC_lexicon_tags_15 },
    { 0x648b7e38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28b7e3d, SUC_lexicon_tags_8 },
    { 0x9c8b7e3e, SUC_lexicon_tags_1 },
    { 0xd00b7e3f, SUC_lexicon_tags_1 },
    { 0x6fcb7e40, SUC_lexicon_tags_45 },
    { 0x60ab7e40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55cb7e44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58b7e48, SUC_lexicon_tags_2 },
    { 0xbdcb7e48, SUC_lexicon_tags_27 },
    { 0x72eb7e4a, SUC_lexicon_tags_47 },
    { 0xeb7e4b, SUC_lexicon_tags_55 },
    { 0xcb6b7e4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6b7e52, SUC_lexicon_tags_60 },
    { 0x64ab7e53, SUC_lexicon_tags_11 },
    { 0x11cb7e54, SUC_lexicon_tags_25 },
    { 0xf50b7e54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x582b7e5e, SUC_lexicon_tags_7 },
    { 0xbc8b7e5f, SUC_lexicon_tags_37 },
    { 0x17eb7e60, SUC_lexicon_tags_7 },
    { 0xca0b7e61, SUC_lexicon_tags_12 },
    { 0xf04b7e61, SUC_lexicon_tags_15 },
    { 0xf8cb7e63, SUC_lexicon_tags_38 },
    { 0x504b7e64, SUC_lexicon_tags_72 },
    { 0xc2cb7e65, SUC_lexicon_tags_2 },
    { 0x254b7e64, SUC_lexicon_tags_37 },
    { 0x3e0b7e64, SUC_lexicon_tags_57 },
    { 0xbdcb7e5f, SUC_lexicon_tags_28 },
    { 0x16cb7e69, SUC_lexicon_tags_16 },
    { 0x56cb7e60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0b7e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8cb7e6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4b7e71, SUC_lexicon_tags_47 },
    { 0x1bcb7e72, SUC_lexicon_tags_48 },
    { 0x45cb7e71, SUC_lexicon_tags_1 },
    { 0x12eb7e71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc34b7e76, SUC_lexicon_tags_43 },
    { 0x698b7e76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb90b7e79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ab7e83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b7e8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34cb7e8d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b7e91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44cb7e94, SUC_lexicon_tags_69 },
    { 0x638b7e95, SUC_lexicon_tags_1 },
    { 0x1fab7e96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bab7e99, SUC_lexicon_tags_14 },
    { 0x8c8b7e9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472b7ea0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8b7ea4, SUC_lexicon_tags_31 },
    { 0xcc0b7ea4, SUC_lexicon_tags_33 },
    { 0x18cb7ea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b7ea9, SUC_lexicon_tags_7 },
    { 0x7c6b7eaa, SUC_lexicon_tags_116 },
    { 0x996b7eab, SUC_lexicon_tags_31 },
    { 0x4d2b7eab, SUC_lexicon_tags_14 },
    { 0xf34b7eac, SUC_lexicon_tags_30 },
    { 0xcab7eab, SUC_lexicon_tags_14 },
    { 0x744b7ea9, SUC_lexicon_tags_7 },
    { 0x800b7eae, SUC_lexicon_tags_61 },
    { 0x77cb7eaa, SUC_lexicon_tags_17 },
    { 0x452b7eb2, SUC_lexicon_tags_1 },
    { 0x800b7eb2, SUC_lexicon_tags_2 },
    { 0x308b7eb2, SUC_lexicon_tags_23 },
    { 0xa44b7eb5, SUC_lexicon_tags_11 },
    { 0xef0b7eaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68cb7eb8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x808b7eba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cb7ec1, SUC_lexicon_tags_1 },
    { 0x7a4b7ec2, SUC_lexicon_tags_1 },
    { 0x2deb7ec3, SUC_lexicon_tags_10 },
    { 0x5cb7ec4, SUC_lexicon_tags_2 },
    { 0x4deb7ec5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x286b7ec8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6b7ecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdeeb7ecd, SUC_lexicon_tags_8 },
    { 0x52b7ecd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600b7ed2, SUC_lexicon_tags_369 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ab7ed5, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x26eb7ed7, SUC_lexicon_tags_25 },
    { 0x2f6b7ed7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff6b7eda, SUC_lexicon_tags_36 },
    { 0xa0eb7edb, SUC_lexicon_tags_28 },
    { 0x66cb7edc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x964b7edf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd32b7ee1, SUC_lexicon_tags_17 },
    { 0x970b7ee1, SUC_lexicon_tags_17 },
    { 0xb8b7ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572b7ee9, SUC_lexicon_tags_17 },
    { 0xf2eb7eea, SUC_lexicon_tags_33 },
    { 0x2fcb7ee9, SUC_lexicon_tags_27 },
    { 0x1d0b7eeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0b7ef1, SUC_lexicon_tags_0 },
    { 0x7d2b7ef1, SUC_lexicon_tags_1 },
    { 0xd8b7ef3, SUC_lexicon_tags_9 },
    { 0x16ab7ef1, SUC_lexicon_tags_45 },
    { 0x6d2b7ef5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2b7efb, SUC_lexicon_tags_49 },
    { 0x190b7efb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x376b7eff, SUC_lexicon_tags_55 },
    { 0x21eb7eff, SUC_lexicon_tags_7 },
    { 0xbfeb7f01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b7f06, SUC_lexicon_tags_45 },
    { 0x530b7f07, SUC_lexicon_tags_49 },
    { 0x7a0b7f06, SUC_lexicon_tags_17 },
    { 0xd06b7f06, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20b7f0c, SUC_lexicon_tags_9 },
    { 0xea0b7f0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6b7f12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b2b7f14, SUC_lexicon_tags_43 },
    { 0xd4eb7f15, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4b7f18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x622b7f1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b7f1d, SUC_lexicon_tags_7 },
    { 0x25eb7f1d, SUC_lexicon_tags_2 },
    { 0x4ceb7f1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eb7f24, SUC_lexicon_tags_26 },
    { 0xf8eb7f25, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xa4b7f27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54b7f2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x196b7f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd6b7f31, SUC_lexicon_tags_1 },
    { 0xffab7f32, SUC_lexicon_tags_25 },
    { 0x356b7f32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d4b7f35, SUC_lexicon_tags_14 },
    { 0x3ccb7f36, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcb7f3b, SUC_lexicon_tags_8 },
    { 0xe66b7f3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dab7f3f, SUC_lexicon_tags_15 },
    { 0x52b7f40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ab7f43, SUC_lexicon_tags_11 },
    { 0x86cb7f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8b7f48, SUC_lexicon_tags_1 },
    { 0x1beb7f49, SUC_lexicon_tags_45 },
    { 0xb8ab7f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab7f50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb7f53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90b7f5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb46b7f5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x110b7f5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33eb7f62, SUC_lexicon_tags_11 },
    { 0xcc6b7f62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x97ab7f65, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x630b7f68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e0b7f6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21eb7f6e, SUC_lexicon_tags_19 },
    { 0xc10b7f6f, SUC_lexicon_tags_21 },
    { 0x3f6b7f70, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x342b7f72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccb7f77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cb7f7a, SUC_lexicon_tags_7 },
    { 0x464b7f7a, SUC_lexicon_tags_56 },
    { 0x46cb7f7c, SUC_lexicon_tags_63 },
    { 0x284b7f7d, SUC_lexicon_tags_66 },
    { 0x420b7f7e, SUC_lexicon_tags_0 },
    { 0x60ab7f7e, SUC_lexicon_tags_2 },
    { 0x7dab7f80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36b7f84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe8eb7f86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52b7f8a, SUC_lexicon_tags_0 },
    { 0x876b7f8b, SUC_lexicon_tags_45 },
    { 0x69cb7f8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5cb7f8e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cab7f93, SUC_lexicon_tags_43 },
    { 0x4a6b7f94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x20cb7f97, SUC_lexicon_tags_26 },
    { 0x60b7f98, SUC_lexicon_tags_37 },
    { 0xfcab7f99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174b7f9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2b7fa0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x938b7fa3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8b7fa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180b7fac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb7faf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726b7fb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca8b7fb5, SUC_lexicon_tags_14 },
    { 0xdeeb7fb6, SUC_lexicon_tags_33 },
    { 0xad8b7fb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700b7fbe, SUC_lexicon_tags_47 },
    { 0x7f8b7fbf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x520b7fc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4b7fc6, SUC_lexicon_tags_12 },
    { 0x532b7fc6, SUC_lexicon_tags_19 },
    { 0xf54b7fc8, SUC_lexicon_tags_14 },
    { 0xa56b7fc6, SUC_lexicon_tags_8 },
    { 0x7d0b7fc8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x334b7fcd, SUC_lexicon_tags_27 },
    { 0xed2b7fcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bcb7fd0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x22eb7fd3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68b7fda, SUC_lexicon_tags_37 },
    { 0x5e2b7fdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2b7fe1, SUC_lexicon_tags_7 },
    { 0x7f4b7fe1, SUC_lexicon_tags_15 },
    { 0xf3eb7fe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6b7fe6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb7fe9, SUC_lexicon_tags_46 },
    { 0x5c0b7fe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eab7fed, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546b7ff2, SUC_lexicon_tags_6 },
    { 0x264b7ff3, SUC_lexicon_tags_29 },
    { 0xf52b7ff2, SUC_lexicon_tags_43 },
    { 0xa40b7ff5, SUC_lexicon_tags_2 },
    { 0xcceb7ff5, SUC_lexicon_tags_16 },
    { 0xbe6b7ff6, SUC_lexicon_tags_37 },
    { 0xcb0b7ff8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7beb7ffb, SUC_lexicon_tags_45 },
    { 0x8ceb7ffc, SUC_lexicon_tags_33 },
    { 0x27cb7ffb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214b8001, SUC_lexicon_tags_25 },
    { 0x4eeb8002, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74b8006, SUC_lexicon_tags_16 },
    { 0x9ab8007, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134b800e, SUC_lexicon_tags_82 },
    { 0x9fab800f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24eb8013, SUC_lexicon_tags_74 },
    { 0xba4b8013, SUC_lexicon_tags_17 },
    { 0xf20b8015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eb801a, SUC_lexicon_tags_19 },
    { 0x9beb801b, SUC_lexicon_tags_17 },
    { 0x4a0b801a, SUC_lexicon_tags_19 },
    { 0x82b801a, SUC_lexicon_tags_62 },
    { 0xb52b801e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56b8021, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4b8027, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cb802a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5beb802d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeb803e, SUC_lexicon_tags_15 },
    { 0x202b803f, SUC_lexicon_tags_7 },
    { 0x44ab8040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72eb8046, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ab8050, SUC_lexicon_tags_12 },
    { 0xa8ab8051, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314b8055, SUC_lexicon_tags_25 },
    { 0x8c6b8055, SUC_lexicon_tags_37 },
    { 0xe3ab8055, SUC_lexicon_tags_1379 },
    { 0, NULL },
    { 0, NULL },
    { 0x228b805a, SUC_lexicon_tags_40 },
    { 0xf0ab805a, SUC_lexicon_tags_66 },
    { 0xf0b805c, SUC_lexicon_tags_17 },
    { 0xfb4b805d, SUC_lexicon_tags_25 },
    { 0x632b805e, SUC_lexicon_tags_46 },
    { 0x196b805f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x422b8062, SUC_lexicon_tags_15 },
    { 0x81eb8062, SUC_lexicon_tags_47 },
    { 0x110b8064, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266b806c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918b8070, SUC_lexicon_tags_45 },
    { 0xe40b8071, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372b8075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ab807d, SUC_lexicon_tags_70 },
    { 0x5cab807e, SUC_lexicon_tags_11 },
    { 0x566b807d, SUC_lexicon_tags_21 },
    { 0x7b6b8080, SUC_lexicon_tags_15 },
    { 0x748b807d, SUC_lexicon_tags_10 },
    { 0x6dab8080, SUC_lexicon_tags_3 },
    { 0x706b8083, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4a8b8085, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6b808c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4b8091, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0b8097, SUC_lexicon_tags_49 },
    { 0x292b8097, SUC_lexicon_tags_30 },
    { 0x558b8099, SUC_lexicon_tags_37 },
    { 0xfcb8097, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x138b809c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4b80a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe82b80a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecb80ad, SUC_lexicon_tags_14 },
    { 0x268b80ae, SUC_lexicon_tags_63 },
    { 0x8d2b80af, SUC_lexicon_tags_2 },
    { 0xbd2b80ae, SUC_lexicon_tags_39 },
    { 0xce4b80b1, SUC_lexicon_tags_33 },
    { 0x3e6b80b2, SUC_lexicon_tags_55 },
    { 0x6cb80ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8b80b5, SUC_lexicon_tags_11 },
    { 0x364b80b5, SUC_lexicon_tags_11 },
    { 0x164b80b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96b80ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cb80bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6c2b80bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614b80c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2b80c6, SUC_lexicon_tags_40 },
    { 0xcd8b80c7, SUC_lexicon_tags_25 },
    { 0x19eb80c8, SUC_lexicon_tags_37 },
    { 0x600b80c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4b80cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b2b80ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebeb80d0, SUC_lexicon_tags_2 },
    { 0x52ab80d1, SUC_lexicon_tags_10 },
    { 0x6b4b80d1, SUC_lexicon_tags_45 },
    { 0x146b80d1, SUC_lexicon_tags_47 },
    { 0x6b80d4, SUC_lexicon_tags_57 },
    { 0xda6b80d2, SUC_lexicon_tags_14 },
    { 0xc42b80d0, SUC_lexicon_tags_9 },
    { 0x1d0b80d7, SUC_lexicon_tags_12 },
    { 0x44eb80d7, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0x5f8b80da, SUC_lexicon_tags_37 },
    { 0xcb8b80da, SUC_lexicon_tags_0 },
    { 0xea0b80da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20b80e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1beb80e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8b80e7, SUC_lexicon_tags_37 },
    { 0x400b80e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100b80ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5a8b80f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376b80f5, SUC_lexicon_tags_17 },
    { 0x2cb80f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ab80fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae4b80ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54b8103, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eb8108, SUC_lexicon_tags_37 },
    { 0x99ab8108, SUC_lexicon_tags_19 },
    { 0xae0b810a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb28b810c, SUC_lexicon_tags_34 },
    { 0x5c0b810c, SUC_lexicon_tags_10 },
    { 0x470b810e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330b811a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6b811d, SUC_lexicon_tags_30 },
    { 0x488b811d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2b6b8120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe32b8122, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x618b8124, SUC_lexicon_tags_65 },
    { 0x618b8125, SUC_lexicon_tags_25 },
    { 0x43eb8125, SUC_lexicon_tags_29 },
    { 0xe4ab8127, SUC_lexicon_tags_17 },
    { 0xd84b8128, SUC_lexicon_tags_2 },
    { 0x1deb8127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbecb812c, SUC_lexicon_tags_0 },
    { 0x952b812d, SUC_lexicon_tags_87 },
    { 0x908b812d, SUC_lexicon_tags_2 },
    { 0x288b812f, SUC_lexicon_tags_21 },
    { 0x2f8b812f, SUC_lexicon_tags_12 },
    { 0x106b8131, SUC_lexicon_tags_1 },
    { 0xad8b8132, SUC_lexicon_tags_613 },
    { 0x48b8133, SUC_lexicon_tags_38 },
    { 0xfd6b8134, SUC_lexicon_tags_38 },
    { 0x6c0b8135, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96cb8137, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b813f, SUC_lexicon_tags_17 },
    { 0x8bab8140, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56b8146, SUC_lexicon_tags_47 },
    { 0x872b8147, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5eb8149, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66eb814b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x780b814d, SUC_lexicon_tags_2 },
    { 0x6a4b814e, SUC_lexicon_tags_15 },
    { 0x7f6b814f, SUC_lexicon_tags_4 },
    { 0xdceb814f, SUC_lexicon_tags_87 },
    { 0x900b814f, SUC_lexicon_tags_13 },
    { 0xf4b8152, SUC_lexicon_tags_65 },
    { 0x20ab8152, SUC_lexicon_tags_7 },
    { 0x56cb8154, SUC_lexicon_tags_2 },
    { 0xde2b8155, SUC_lexicon_tags_21 },
    { 0xd50b8156, SUC_lexicon_tags_37 },
    { 0x86ab8150, SUC_lexicon_tags_15 },
    { 0xc54b8151, SUC_lexicon_tags_37 },
    { 0xf24b8157, SUC_lexicon_tags_45 },
    { 0x6a0b815a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538b8161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8b8164, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ab8167, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41eb8169, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cb816e, SUC_lexicon_tags_14 },
    { 0x6e0b816f, SUC_lexicon_tags_26 },
    { 0xd98b816f, SUC_lexicon_tags_17 },
    { 0x5d2b8171, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b4b8173, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2b8176, SUC_lexicon_tags_17 },
    { 0xa22b8176, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eb817b, SUC_lexicon_tags_23 },
    { 0xd0ab817c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x178b817f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4b8183, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbf0b8185, SUC_lexicon_tags_45 },
    { 0x1ecb8186, SUC_lexicon_tags_15 },
    { 0x7d8b8187, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8b818b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ab818f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52b8197, SUC_lexicon_tags_2 },
    { 0x6e0b8198, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dab819f, SUC_lexicon_tags_91 },
    { 0x3b8b81a0, SUC_lexicon_tags_49 },
    { 0x27cb81a1, SUC_lexicon_tags_12 },
    { 0x89cb819f, SUC_lexicon_tags_85 },
    { 0xdceb81a3, SUC_lexicon_tags_527 },
    { 0, NULL },
    { 0x12ab81a5, SUC_lexicon_tags_19 },
    { 0x12cb81a6, SUC_lexicon_tags_27 },
    { 0x30cb81a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0b81ab, SUC_lexicon_tags_15 },
    { 0xbdab81ab, SUC_lexicon_tags_46 },
    { 0xa90b81ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x908b81b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348b81b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86b81bb, SUC_lexicon_tags_37 },
    { 0xf36b81bc, SUC_lexicon_tags_17 },
    { 0x1e8b81bd, SUC_lexicon_tags_25 },
    { 0x8c6b81be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x50b81c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc94b81c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58b81c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1beb81c7, SUC_lexicon_tags_30 },
    { 0x2b81c7, SUC_lexicon_tags_14 },
    { 0xd68b81c7, SUC_lexicon_tags_9 },
    { 0x52cb81c9, SUC_lexicon_tags_15 },
    { 0xa9eb81c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14b81d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x142b81dc, SUC_lexicon_tags_68 },
    { 0xc20b81dd, SUC_lexicon_tags_11 },
    { 0x37ab81dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d2b81e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4eb81e2, SUC_lexicon_tags_9 },
    { 0xd3eb81e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cb81e6, SUC_lexicon_tags_46 },
    { 0x242b81e7, SUC_lexicon_tags_8 },
    { 0x72eb81e7, SUC_lexicon_tags_2 },
    { 0xa8eb81e6, SUC_lexicon_tags_28 },
    { 0x236b81ea, SUC_lexicon_tags_45 },
    { 0x3c8b81eb, SUC_lexicon_tags_11 },
    { 0x6d6b81ec, SUC_lexicon_tags_7 },
    { 0xd6ab81eb, SUC_lexicon_tags_37 },
    { 0x69ab81ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfdab81f0, SUC_lexicon_tags_31 },
    { 0x496b81f0, SUC_lexicon_tags_39 },
    { 0xb92b81f0, SUC_lexicon_tags_27 },
    { 0x2a6b81f3, SUC_lexicon_tags_25 },
    { 0xf94b81f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc28b81f6, SUC_lexicon_tags_38 },
    { 0x1d0b81f7, SUC_lexicon_tags_1 },
    { 0xca8b81f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa10b81fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576b8201, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550b8207, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12b820f, SUC_lexicon_tags_49 },
    { 0x1d4b820f, SUC_lexicon_tags_11 },
    { 0xf3ab8211, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x462b8213, SUC_lexicon_tags_38 },
    { 0xeaeb8213, SUC_lexicon_tags_619 },
    { 0, NULL },
    { 0x79ab8216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cb821b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914b8221, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4b8224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef6b8226, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152b822b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2b822e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ab8242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0b8245, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x692b8247, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b8255, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e2b8257, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefeb8267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938b826b, SUC_lexicon_tags_66 },
    { 0x93cb826c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20b8273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8b827b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62b827e, SUC_lexicon_tags_15 },
    { 0x37cb827e, SUC_lexicon_tags_3 },
    { 0x13eb8280, SUC_lexicon_tags_11 },
    { 0xcbab8280, SUC_lexicon_tags_11 },
    { 0xb54b8281, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5e2b8284, SUC_lexicon_tags_910 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50b8289, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x11eb828b, SUC_lexicon_tags_1 },
    { 0x9b4b828c, SUC_lexicon_tags_23 },
    { 0x54eb828c, SUC_lexicon_tags_8 },
    { 0x4a2b828e, SUC_lexicon_tags_23 },
    { 0x914b828f, SUC_lexicon_tags_85 },
    { 0xa5cb828b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfaab8292, SUC_lexicon_tags_13 },
    { 0x830b8293, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa0ab8295, SUC_lexicon_tags_82 },
    { 0x8e2b8296, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb829a, SUC_lexicon_tags_17 },
    { 0xc32b829b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x146b829d, SUC_lexicon_tags_63 },
    { 0x94b829d, SUC_lexicon_tags_11 },
    { 0xf04b829f, SUC_lexicon_tags_65 },
    { 0xa50b82a0, SUC_lexicon_tags_11 },
    { 0x1c8b829f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76b82a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70b82ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808b82b2, SUC_lexicon_tags_17 },
    { 0x20ab82b3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ab82b7, SUC_lexicon_tags_37 },
    { 0x748b82b8, SUC_lexicon_tags_34 },
    { 0xb8cb82b9, SUC_lexicon_tags_14 },
    { 0x766b82b9, SUC_lexicon_tags_32 },
    { 0x570b82bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x578b82bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104b82c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd0b82c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cb82c9, SUC_lexicon_tags_7 },
    { 0x4b8b82ca, SUC_lexicon_tags_9 },
    { 0x246b82ca, SUC_lexicon_tags_29 },
    { 0x422b82cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4b82d7, SUC_lexicon_tags_37 },
    { 0xca4b82d8, SUC_lexicon_tags_1 },
    { 0xd10b82d9, SUC_lexicon_tags_65 },
    { 0x640b82da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98b82dd, SUC_lexicon_tags_1 },
    { 0xcbab82de, SUC_lexicon_tags_17 },
    { 0x7b0b82de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x750b82e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xce6b82e3, SUC_lexicon_tags_11 },
    { 0x6d0b82e4, SUC_lexicon_tags_7 },
    { 0x864b82e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab82e8, SUC_lexicon_tags_8 },
    { 0x8ceb82e9, SUC_lexicon_tags_14 },
    { 0x3feb82e9, SUC_lexicon_tags_19 },
    { 0xcb82eb, SUC_lexicon_tags_25 },
    { 0xb62b82ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8ab82ee, SUC_lexicon_tags_23 },
    { 0x678b82ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0b82f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x848b82f9, SUC_lexicon_tags_17 },
    { 0xe58b82f9, SUC_lexicon_tags_37 },
    { 0x14b82fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8b8300, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6b8303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ab8306, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xb5ab8308, SUC_lexicon_tags_7 },
    { 0x7a4b8309, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712b830e, SUC_lexicon_tags_38 },
    { 0xe46b830f, SUC_lexicon_tags_37 },
    { 0xc20b8310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbaab8312, SUC_lexicon_tags_0 },
    { 0x108b8313, SUC_lexicon_tags_11 },
    { 0xa8eb8314, SUC_lexicon_tags_45 },
    { 0x76cb8314, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cb8319, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x450b831c, SUC_lexicon_tags_14 },
    { 0x5b0b831c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x716b8320, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622b8324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbab8326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4b8329, SUC_lexicon_tags_12 },
    { 0xef6b832a, SUC_lexicon_tags_9 },
    { 0x4cb8329, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8b832f, SUC_lexicon_tags_15 },
    { 0x296b832f, SUC_lexicon_tags_2 },
    { 0x63ab8331, SUC_lexicon_tags_16 },
    { 0xd1ab8332, SUC_lexicon_tags_21 },
    { 0xfdab8333, SUC_lexicon_tags_57 },
    { 0x24ab8334, SUC_lexicon_tags_27 },
    { 0xc78b8331, SUC_lexicon_tags_15 },
    { 0x268b8336, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b833a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaab833e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bcb8340, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x638b8342, SUC_lexicon_tags_25 },
    { 0x100b8342, SUC_lexicon_tags_12 },
    { 0xd2eb8344, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b8348, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0b8352, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2b8355, SUC_lexicon_tags_27 },
    { 0x6f8b8355, SUC_lexicon_tags_10 },
    { 0x198b8357, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xec8b8359, SUC_lexicon_tags_17 },
    { 0x17cb8359, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82b8361, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92eb8363, SUC_lexicon_tags_27 },
    { 0xe30b8363, SUC_lexicon_tags_6 },
    { 0xfd8b8365, SUC_lexicon_tags_15 },
    { 0x570b8366, SUC_lexicon_tags_15 },
    { 0xe22b8366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100b836b, SUC_lexicon_tags_8 },
    { 0x280b836c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8b8370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782b837b, SUC_lexicon_tags_92 },
    { 0x28ab837c, SUC_lexicon_tags_13 },
    { 0x7c2b837b, SUC_lexicon_tags_32 },
    { 0xd16b837e, SUC_lexicon_tags_15 },
    { 0xfc4b837e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5eb8388, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2eb838b, SUC_lexicon_tags_70 },
    { 0x80eb838c, SUC_lexicon_tags_37 },
    { 0xd5ab838d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacb8392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafcb8394, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cb8397, SUC_lexicon_tags_19 },
    { 0x10eb8397, SUC_lexicon_tags_79 },
    { 0xff0b8399, SUC_lexicon_tags_7 },
    { 0x16eb8399, SUC_lexicon_tags_32 },
    { 0x93eb839b, SUC_lexicon_tags_11 },
    { 0x860b839c, SUC_lexicon_tags_66 },
    { 0xc12b839d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd86b839f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd42b83a1, SUC_lexicon_tags_8 },
    { 0xc98b83a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8b83a7, SUC_lexicon_tags_106 },
    { 0x63ab83a7, SUC_lexicon_tags_19 },
    { 0xca0b83a9, SUC_lexicon_tags_14 },
    { 0x174b83a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2b83b0, SUC_lexicon_tags_9 },
    { 0xfbeb83b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8fab83b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eb83b6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x20eb83b9, SUC_lexicon_tags_27 },
    { 0xcd4b83ba, SUC_lexicon_tags_2 },
    { 0x494b83bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2b83be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aeb83c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x524b83ca, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0b83cd, SUC_lexicon_tags_7 },
    { 0x85ab83ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x962b83d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fab83d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x464b83d9, SUC_lexicon_tags_49 },
    { 0x25ab83da, SUC_lexicon_tags_9 },
    { 0x7a6b83d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x186b83de, SUC_lexicon_tags_1 },
    { 0x26eb83de, SUC_lexicon_tags_41 },
    { 0xc7eb83e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718b83e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b83ed, SUC_lexicon_tags_17 },
    { 0x3aab83ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x26b83f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cb83f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aab83fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b8401, SUC_lexicon_tags_45 },
    { 0xa8b8402, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0b8407, SUC_lexicon_tags_19 },
    { 0x2d2b8408, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206b840e, SUC_lexicon_tags_27 },
    { 0x1f4b840f, SUC_lexicon_tags_7 },
    { 0xe6b8410, SUC_lexicon_tags_7 },
    { 0xdb4b840e, SUC_lexicon_tags_92 },
    { 0xd0eb840e, SUC_lexicon_tags_60 },
    { 0xf5eb8413, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xddeb8415, SUC_lexicon_tags_11 },
    { 0xe0b8416, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96eb841c, SUC_lexicon_tags_66 },
    { 0x120b841d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6b8420, SUC_lexicon_tags_10 },
    { 0x930b8420, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9cb8423, SUC_lexicon_tags_38 },
    { 0x718b8423, SUC_lexicon_tags_14 },
    { 0xce6b8425, SUC_lexicon_tags_37 },
    { 0xa58b8425, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914b842c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15eb842f, SUC_lexicon_tags_19 },
    { 0x642b8430, SUC_lexicon_tags_37 },
    { 0xbe8b8431, SUC_lexicon_tags_37 },
    { 0xe50b8432, SUC_lexicon_tags_17 },
    { 0x8f2b8431, SUC_lexicon_tags_6 },
    { 0xbcab8433, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b8439, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae2b843b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4b8444, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x862b8446, SUC_lexicon_tags_37 },
    { 0xccab8446, SUC_lexicon_tags_8 },
    { 0xf1cb8448, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916b844c, SUC_lexicon_tags_34 },
    { 0xf2cb844d, SUC_lexicon_tags_17 },
    { 0x8fcb844e, SUC_lexicon_tags_37 },
    { 0xed0b844d, SUC_lexicon_tags_17 },
    { 0x398b844d, SUC_lexicon_tags_17 },
    { 0xee0b8451, SUC_lexicon_tags_19 },
    { 0x98eb844d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3deb8455, SUC_lexicon_tags_11 },
    { 0xcf0b8455, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x79eb8458, SUC_lexicon_tags_47 },
    { 0x40ab8459, SUC_lexicon_tags_39 },
    { 0x18ab8458, SUC_lexicon_tags_0 },
    { 0x554b845b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aab845e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x402b8460, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8eb8462, SUC_lexicon_tags_29 },
    { 0xfdab8463, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x590b8465, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe0b8467, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52b8472, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x632b8475, SUC_lexicon_tags_31 },
    { 0x298b8476, SUC_lexicon_tags_37 },
    { 0xf0ab8475, SUC_lexicon_tags_23 },
    { 0xdceb8475, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20eb847c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eb847f, SUC_lexicon_tags_17 },
    { 0x12b8480, SUC_lexicon_tags_25 },
    { 0xca4b8481, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x19eb8483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8b8486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eb848a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cb848f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58b8493, SUC_lexicon_tags_29 },
    { 0x90eb8494, SUC_lexicon_tags_14 },
    { 0xb40b8495, SUC_lexicon_tags_2 },
    { 0x7feb8493, SUC_lexicon_tags_15 },
    { 0xc0ab8497, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x862b8499, SUC_lexicon_tags_12 },
    { 0x3c4b8499, SUC_lexicon_tags_19 },
    { 0xcc8b849b, SUC_lexicon_tags_49 },
    { 0xb18b849c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb84aa, SUC_lexicon_tags_7 },
    { 0xbd8b84aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4b84ae, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64b84b2, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xcab84b4, SUC_lexicon_tags_2 },
    { 0x8b4b84b5, SUC_lexicon_tags_63 },
    { 0xf82b84b6, SUC_lexicon_tags_27 },
    { 0x51cb84b6, SUC_lexicon_tags_12 },
    { 0xe98b84b8, SUC_lexicon_tags_51 },
    { 0x2a8b84b9, SUC_lexicon_tags_229 },
    { 0xd60b84b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418b84be, SUC_lexicon_tags_30 },
    { 0x640b84be, SUC_lexicon_tags_47 },
    { 0x6ab84c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94b84c3, SUC_lexicon_tags_2 },
    { 0xbd4b84c4, SUC_lexicon_tags_35 },
    { 0x4e2b84c5, SUC_lexicon_tags_2 },
    { 0x66b84c5, SUC_lexicon_tags_17 },
    { 0xe32b84c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86b84d1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceb84d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0b84db, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9feb84e6, SUC_lexicon_tags_8 },
    { 0x266b84e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4dab84e9, SUC_lexicon_tags_12 },
    { 0xd08b84ea, SUC_lexicon_tags_2 },
    { 0x80cb84eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x696b84ed, SUC_lexicon_tags_131 },
    { 0xb5cb84ee, SUC_lexicon_tags_2 },
    { 0xc82b84ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd76b84f1, SUC_lexicon_tags_60 },
    { 0x294b84f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cb84f7, SUC_lexicon_tags_27 },
    { 0x7c4b84f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75eb8502, SUC_lexicon_tags_82 },
    { 0xea8b8503, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdab8506, SUC_lexicon_tags_11 },
    { 0xbf8b8507, SUC_lexicon_tags_198 },
    { 0xa7eb8507, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x35eb850a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cb850d, SUC_lexicon_tags_6 },
    { 0xf0eb850e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cb8511, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68b8518, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d4b851a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30b8521, SUC_lexicon_tags_1 },
    { 0xfbab8522, SUC_lexicon_tags_2 },
    { 0x60b8523, SUC_lexicon_tags_7 },
    { 0x924b8523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x900b8526, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x500b8528, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x600b852b, SUC_lexicon_tags_25 },
    { 0xb8ab852b, SUC_lexicon_tags_49 },
    { 0x2bcb852d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66b8534, SUC_lexicon_tags_49 },
    { 0xba2b8535, SUC_lexicon_tags_6 },
    { 0x7fab8536, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8b8541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30b8544, SUC_lexicon_tags_37 },
    { 0x9c4b8545, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b8548, SUC_lexicon_tags_28 },
    { 0xaa0b8549, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x562b854b, SUC_lexicon_tags_60 },
    { 0x7f6b854b, SUC_lexicon_tags_54 },
    { 0xd10b854d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x348b8550, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x834b8553, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcccb8555, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2a0b8557, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaceb855a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7deb855e, SUC_lexicon_tags_17 },
    { 0xc1cb855f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x624b8562, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x992b8564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26ab8566, SUC_lexicon_tags_27 },
    { 0x1a8b8567, SUC_lexicon_tags_42 },
    { 0x3a2b8568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5ab856a, SUC_lexicon_tags_12 },
    { 0xf98b856a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb856e, SUC_lexicon_tags_7 },
    { 0xceeb856f, SUC_lexicon_tags_39 },
    { 0xaf2b856f, SUC_lexicon_tags_26 },
    { 0x5b8b8571, SUC_lexicon_tags_11 },
    { 0x758b856f, SUC_lexicon_tags_6 },
    { 0x606b8573, SUC_lexicon_tags_17 },
    { 0xc24b8573, SUC_lexicon_tags_29 },
    { 0x8b856f, SUC_lexicon_tags_4 },
    { 0x762b856f, SUC_lexicon_tags_25 },
    { 0x24cb8571, SUC_lexicon_tags_17 },
    { 0xb2b8578, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0b857f, SUC_lexicon_tags_17 },
    { 0x35ab8580, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x974b8582, SUC_lexicon_tags_11 },
    { 0x146b8583, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x968b8585, SUC_lexicon_tags_7 },
    { 0xed8b8585, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902b858a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xef6b858c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4b8593, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e8b8595, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x300b8597, SUC_lexicon_tags_2 },
    { 0xd3cb8598, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52b859b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4b85a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x770b85a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252b85ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0b85b1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456b85b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecb85bb, SUC_lexicon_tags_4 },
    { 0x7ecb85bc, SUC_lexicon_tags_25 },
    { 0xe40b85bc, SUC_lexicon_tags_11 },
    { 0xe16b85bc, SUC_lexicon_tags_27 },
    { 0x8a6b85bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5cb85c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678b85c5, SUC_lexicon_tags_33 },
    { 0x57ab85c6, SUC_lexicon_tags_37 },
    { 0xc50b85c7, SUC_lexicon_tags_49 },
    { 0x35cb85c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4b85ce, SUC_lexicon_tags_21 },
    { 0xda6b85ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x616b85d1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42b85d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x986b85db, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cb85df, SUC_lexicon_tags_8 },
    { 0x472b85e0, SUC_lexicon_tags_47 },
    { 0x3d2b85e0, SUC_lexicon_tags_27 },
    { 0xee6b85e2, SUC_lexicon_tags_23 },
    { 0x4d2b85e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5dcb85e5, SUC_lexicon_tags_8 },
    { 0xc10b85e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7cb85e8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cb85eb, SUC_lexicon_tags_23 },
    { 0x8b85ec, SUC_lexicon_tags_26 },
    { 0x340b85ed, SUC_lexicon_tags_7 },
    { 0x8b85ee, SUC_lexicon_tags_11 },
    { 0x9b0b85ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6c4b85f1, SUC_lexicon_tags_1 },
    { 0xd8eb85f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x41cb85f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50b85fb, SUC_lexicon_tags_19 },
    { 0x7acb85fb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x87cb85fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce4b8600, SUC_lexicon_tags_23 },
    { 0xc64b8600, SUC_lexicon_tags_1 },
    { 0x614b8600, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8b8605, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26eb8609, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0b860e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71eb8613, SUC_lexicon_tags_37 },
    { 0xee2b8613, SUC_lexicon_tags_0 },
    { 0xa58b8615, SUC_lexicon_tags_60 },
    { 0x450b8615, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcb0b8618, SUC_lexicon_tags_33 },
    { 0xc24b8619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250b861f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x880b8622, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cb8628, SUC_lexicon_tags_16 },
    { 0x2c4b8628, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25ab862b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234b8631, SUC_lexicon_tags_17 },
    { 0x68b8631, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeab8634, SUC_lexicon_tags_25 },
    { 0x38cb8635, SUC_lexicon_tags_2 },
    { 0xd70b8636, SUC_lexicon_tags_19 },
    { 0x270b8637, SUC_lexicon_tags_23 },
    { 0x88eb8638, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6b8640, SUC_lexicon_tags_15 },
    { 0xc86b8640, SUC_lexicon_tags_7 },
    { 0x48ab8642, SUC_lexicon_tags_1 },
    { 0x480b8640, SUC_lexicon_tags_17 },
    { 0xe2b8643, SUC_lexicon_tags_9 },
    { 0x4c4b8643, SUC_lexicon_tags_39 },
    { 0x48b8640, SUC_lexicon_tags_12 },
    { 0x86ab8647, SUC_lexicon_tags_30 },
    { 0xd78b8648, SUC_lexicon_tags_60 },
    { 0x3a0b8649, SUC_lexicon_tags_15 },
    { 0xb00b8642, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208b864f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32b8652, SUC_lexicon_tags_15 },
    { 0xa38b8653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f4b8655, SUC_lexicon_tags_42 },
    { 0xae6b8656, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72b8659, SUC_lexicon_tags_14 },
    { 0x266b865a, SUC_lexicon_tags_27 },
    { 0x3d2b865b, SUC_lexicon_tags_37 },
    { 0x35cb865b, SUC_lexicon_tags_74 },
    { 0xaf0b865d, SUC_lexicon_tags_72 },
    { 0x204b865e, SUC_lexicon_tags_47 },
    { 0x34cb865f, SUC_lexicon_tags_37 },
    { 0xd8eb865c, SUC_lexicon_tags_4 },
    { 0xe6cb8661, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdacb8664, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8b8668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33ab866a, SUC_lexicon_tags_1 },
    { 0x1a4b866b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8b866e, SUC_lexicon_tags_14 },
    { 0x8fab866f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340b8674, SUC_lexicon_tags_1 },
    { 0x8ab8675, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8b8679, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x202b867b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cb867e, SUC_lexicon_tags_15 },
    { 0xfdcb867e, SUC_lexicon_tags_1 },
    { 0xfa4b8680, SUC_lexicon_tags_30 },
    { 0xae8b8681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6b8684, SUC_lexicon_tags_7 },
    { 0x134b8685, SUC_lexicon_tags_26 },
    { 0xf1eb8686, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2b868b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b8693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6b869b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x332b869d, SUC_lexicon_tags_12 },
    { 0x512b869e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa04b86a0, SUC_lexicon_tags_29 },
    { 0xd7ab86a0, SUC_lexicon_tags_7 },
    { 0x436b86a1, SUC_lexicon_tags_14 },
    { 0x9a8b86a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0b86a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b86aa, SUC_lexicon_tags_16 },
    { 0xb5cb86ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eb86ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43ab86b0, SUC_lexicon_tags_9 },
    { 0xe16b86b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x206b86b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf4b86b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698b86ba, SUC_lexicon_tags_23 },
    { 0x59cb86bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab86be, SUC_lexicon_tags_7 },
    { 0x94cb86bf, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112b86c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x45eb86c7, SUC_lexicon_tags_14 },
    { 0xd90b86c8, SUC_lexicon_tags_7 },
    { 0x2f4b86c8, SUC_lexicon_tags_28 },
    { 0xd60b86c8, SUC_lexicon_tags_37 },
    { 0x220b86cb, SUC_lexicon_tags_2 },
    { 0x332b86cb, SUC_lexicon_tags_37 },
    { 0x694b86ca, SUC_lexicon_tags_62 },
    { 0xb4ab86ce, SUC_lexicon_tags_63 },
    { 0x586b86cf, SUC_lexicon_tags_15 },
    { 0x1eeb86c8, SUC_lexicon_tags_37 },
    { 0xfbeb86d1, SUC_lexicon_tags_4 },
    { 0x194b86d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4b86d5, SUC_lexicon_tags_17 },
    { 0x9e2b86d5, SUC_lexicon_tags_11 },
    { 0x132b86d7, SUC_lexicon_tags_47 },
    { 0xab6b86d6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x18cb86da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcb86df, SUC_lexicon_tags_18 },
    { 0xdb2b86e0, SUC_lexicon_tags_15 },
    { 0xad4b86e0, SUC_lexicon_tags_37 },
    { 0xcf2b86e2, SUC_lexicon_tags_15 },
    { 0x4acb86e3, SUC_lexicon_tags_16 },
    { 0xf38b86e3, SUC_lexicon_tags_4 },
    { 0x964b86e3, SUC_lexicon_tags_2 },
    { 0xc84b86e6, SUC_lexicon_tags_15 },
    { 0x11ab86e6, SUC_lexicon_tags_33 },
    { 0xd76b86e8, SUC_lexicon_tags_15 },
    { 0x91cb86e2, SUC_lexicon_tags_23 },
    { 0x578b86e4, SUC_lexicon_tags_17 },
    { 0x7cab86eb, SUC_lexicon_tags_1 },
    { 0x918b86e6, SUC_lexicon_tags_25 },
    { 0xa48b86ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x252b86ef, SUC_lexicon_tags_23 },
    { 0xeb4b86f0, SUC_lexicon_tags_136 },
    { 0xb9eb86f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68b86f4, SUC_lexicon_tags_25 },
    { 0x6cb86f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcab86fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90b86fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0b8704, SUC_lexicon_tags_11 },
    { 0xaeab8705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d8b8707, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ab870b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b8710, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48b8713, SUC_lexicon_tags_30 },
    { 0xfb0b8714, SUC_lexicon_tags_8 },
    { 0x89eb8715, SUC_lexicon_tags_1 },
    { 0x66ab8715, SUC_lexicon_tags_20 },
    { 0x1a0b8717, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1fcb8719, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b871c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf54b871e, SUC_lexicon_tags_47 },
    { 0xb0b871f, SUC_lexicon_tags_7 },
    { 0xe60b871f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x228b8723, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe80b8725, SUC_lexicon_tags_27 },
    { 0x958b8726, SUC_lexicon_tags_8 },
    { 0x8ecb8726, SUC_lexicon_tags_15 },
    { 0xde6b8727, SUC_lexicon_tags_33 },
    { 0x584b8725, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0b872d, SUC_lexicon_tags_15 },
    { 0x208b872e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2b8731, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x90eb8734, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20b873c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe18b873e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2b8744, SUC_lexicon_tags_43 },
    { 0xe10b8745, SUC_lexicon_tags_14 },
    { 0xc42b8746, SUC_lexicon_tags_47 },
    { 0x598b8746, SUC_lexicon_tags_62 },
    { 0x9ecb8748, SUC_lexicon_tags_11 },
    { 0x18ab8749, SUC_lexicon_tags_9 },
    { 0x3f6b8746, SUC_lexicon_tags_68 },
    { 0x6b2b874b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x566b874e, SUC_lexicon_tags_37 },
    { 0xc2ab874f, SUC_lexicon_tags_68 },
    { 0x5eeb8750, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7ab8752, SUC_lexicon_tags_7 },
    { 0x4f2b8753, SUC_lexicon_tags_65 },
    { 0x92ab8753, SUC_lexicon_tags_23 },
    { 0x50ab8754, SUC_lexicon_tags_23 },
    { 0x460b8756, SUC_lexicon_tags_25 },
    { 0x752b8757, SUC_lexicon_tags_60 },
    { 0x31cb8752, SUC_lexicon_tags_7 },
    { 0xd6eb8759, SUC_lexicon_tags_31 },
    { 0x1c0b8755, SUC_lexicon_tags_5 },
    { 0x6eeb875b, SUC_lexicon_tags_25 },
    { 0xc48b8754, SUC_lexicon_tags_23 },
    { 0x588b8757, SUC_lexicon_tags_17 },
    { 0x19ab875e, SUC_lexicon_tags_15 },
    { 0x2d6b875c, SUC_lexicon_tags_7 },
    { 0xb62b8760, SUC_lexicon_tags_23 },
    { 0xaeab8761, SUC_lexicon_tags_26 },
    { 0x90ab8761, SUC_lexicon_tags_33 },
    { 0x67eb8762, SUC_lexicon_tags_15 },
    { 0x564b8762, SUC_lexicon_tags_17 },
    { 0x2f4b8764, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4c4b8767, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ab876a, SUC_lexicon_tags_11 },
    { 0x2ab876b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3dab876d, SUC_lexicon_tags_15 },
    { 0x836b876e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826b8772, SUC_lexicon_tags_2 },
    { 0x4f2b8773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7eb8775, SUC_lexicon_tags_15 },
    { 0x1b2b8775, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb2b8778, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbeb877c, SUC_lexicon_tags_25 },
    { 0xe22b877c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24eb8781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x606b8783, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2b8786, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa30b8788, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa76b878a, SUC_lexicon_tags_12 },
    { 0xf62b878b, SUC_lexicon_tags_55 },
    { 0x1aab878c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ab8790, SUC_lexicon_tags_29 },
    { 0xc0eb8791, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6b8794, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840b8799, SUC_lexicon_tags_11 },
    { 0x418b8799, SUC_lexicon_tags_38 },
    { 0xd1ab879a, SUC_lexicon_tags_30 },
    { 0x1a6b879c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94b87a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2b87a4, SUC_lexicon_tags_7 },
    { 0x8b6b87a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98eb87a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ab87ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e2b87ad, SUC_lexicon_tags_17 },
    { 0x852b87ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ab87b1, SUC_lexicon_tags_12 },
    { 0x80b87b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x314b87b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eab87b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf8b87bb, SUC_lexicon_tags_14 },
    { 0xcd4b87bc, SUC_lexicon_tags_19 },
    { 0x990b87bc, SUC_lexicon_tags_30 },
    { 0x4ceb87bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20b87c3, SUC_lexicon_tags_7 },
    { 0xfc4b87c4, SUC_lexicon_tags_46 },
    { 0x3a0b87c4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xb2eb87c7, SUC_lexicon_tags_235 },
    { 0x616b87c8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x342b87ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0b87ce, SUC_lexicon_tags_15 },
    { 0xe56b87cf, SUC_lexicon_tags_380 },
    { 0x8b87cf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x410b87d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eab87da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x44eb87dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4b87e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe50b87e2, SUC_lexicon_tags_84 },
    { 0xb5cb87e3, SUC_lexicon_tags_66 },
    { 0x75eb87e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x290b87e6, SUC_lexicon_tags_43 },
    { 0x3ceb87e7, SUC_lexicon_tags_31 },
    { 0xfc4b87e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e8b87ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cb87ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8b87f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eb87f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x294b87f9, SUC_lexicon_tags_47 },
    { 0x186b87f9, SUC_lexicon_tags_29 },
    { 0xfd0b87fb, SUC_lexicon_tags_55 },
    { 0x666b87f9, SUC_lexicon_tags_1 },
    { 0x978b87fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ab8800, SUC_lexicon_tags_7 },
    { 0xa3eb8801, SUC_lexicon_tags_79 },
    { 0xadcb8800, SUC_lexicon_tags_17 },
    { 0x24b8803, SUC_lexicon_tags_1 },
    { 0x63ab8803, SUC_lexicon_tags_26 },
    { 0x5f2b8805, SUC_lexicon_tags_39 },
    { 0x74b8803, SUC_lexicon_tags_23 },
    { 0x8eab8807, SUC_lexicon_tags_13 },
    { 0xfeb8807, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2b8817, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x366b881a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cb881d, SUC_lexicon_tags_32 },
    { 0x7b2b881d, SUC_lexicon_tags_37 },
    { 0xf76b881f, SUC_lexicon_tags_11 },
    { 0xeccb8820, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb8826, SUC_lexicon_tags_33 },
    { 0x642b8827, SUC_lexicon_tags_12 },
    { 0xb0ab8826, SUC_lexicon_tags_31 },
    { 0x98eb8829, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x36b882b, SUC_lexicon_tags_37 },
    { 0xebab882c, SUC_lexicon_tags_17 },
    { 0x57ab882d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8b8833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe34b8835, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfeeb8837, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b883a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852b8842, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2b8847, SUC_lexicon_tags_2 },
    { 0xd82b8847, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x458b884a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb884d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b2b884f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06b8852, SUC_lexicon_tags_27 },
    { 0x422b8853, SUC_lexicon_tags_12 },
    { 0xc4b8853, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54b8857, SUC_lexicon_tags_8 },
    { 0x86ab8858, SUC_lexicon_tags_15 },
    { 0x880b8859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2b885d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502b8862, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0b8865, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x270b8868, SUC_lexicon_tags_10 },
    { 0xc08b8869, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40b886d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50b8870, SUC_lexicon_tags_29 },
    { 0x81cb8871, SUC_lexicon_tags_1 },
    { 0x1a6b8871, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24ab8874, SUC_lexicon_tags_4 },
    { 0xdd6b8875, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb887d, SUC_lexicon_tags_65 },
    { 0xc92b887d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ab8881, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ab8884, SUC_lexicon_tags_12 },
    { 0x3f6b8884, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbacb8887, SUC_lexicon_tags_15 },
    { 0x270b8888, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436b888c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ab8891, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd0cb8893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46b8898, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250b889d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0b88a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeccb88a2, SUC_lexicon_tags_8 },
    { 0x4dcb88a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ab88a8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x63cb88aa, SUC_lexicon_tags_17 },
    { 0xc9eb88ab, SUC_lexicon_tags_12 },
    { 0x57eb88ac, SUC_lexicon_tags_2 },
    { 0xc94b88ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44b88b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cb88b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4c2b88b6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xef4b88b8, SUC_lexicon_tags_27 },
    { 0x82b88b9, SUC_lexicon_tags_15 },
    { 0x15eb88b8, SUC_lexicon_tags_1 },
    { 0x86b88b9, SUC_lexicon_tags_11 },
    { 0x82eb88bc, SUC_lexicon_tags_47 },
    { 0xaa0b88bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbfeb88bf, SUC_lexicon_tags_12 },
    { 0x2dab88c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136b88d3, SUC_lexicon_tags_23 },
    { 0x18eb88d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578b88da, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198b88df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5f2b88e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf28b88e3, SUC_lexicon_tags_14 },
    { 0xccab88e3, SUC_lexicon_tags_7 },
    { 0x9ab88e5, SUC_lexicon_tags_11 },
    { 0xebcb88e6, SUC_lexicon_tags_37 },
    { 0xb10b88e7, SUC_lexicon_tags_2 },
    { 0x822b88e7, SUC_lexicon_tags_56 },
    { 0xfdcb88e7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2b88f0, SUC_lexicon_tags_6 },
    { 0x820b88f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eab88f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0b88fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0b8900, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bab8906, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46b8908, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cb890b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922b8910, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e8b8912, SUC_lexicon_tags_9 },
    { 0x496b8913, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88cb8915, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x64cb8917, SUC_lexicon_tags_39 },
    { 0x640b8918, SUC_lexicon_tags_11 },
    { 0x8b4b8919, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b891c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e8b891e, SUC_lexicon_tags_17 },
    { 0xe28b891f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacb8926, SUC_lexicon_tags_2 },
    { 0x59cb8926, SUC_lexicon_tags_25 },
    { 0x3ccb8928, SUC_lexicon_tags_1 },
    { 0x716b8926, SUC_lexicon_tags_27 },
    { 0xb8eb8927, SUC_lexicon_tags_0 },
    { 0x5ceb892b, SUC_lexicon_tags_69 },
    { 0x654b892b, SUC_lexicon_tags_15 },
    { 0x250b892d, SUC_lexicon_tags_38 },
    { 0x650b892b, SUC_lexicon_tags_32 },
    { 0x532b892d, SUC_lexicon_tags_7 },
    { 0x40ab8930, SUC_lexicon_tags_27 },
    { 0x588b8930, SUC_lexicon_tags_27 },
    { 0xf78b8930, SUC_lexicon_tags_11 },
    { 0xa40b892a, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x44eb8935, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4ab8937, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x40eb8939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x492b893c, SUC_lexicon_tags_25 },
    { 0xb3ab893c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94eb8943, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9cb8945, SUC_lexicon_tags_11 },
    { 0xe32b8945, SUC_lexicon_tags_17 },
    { 0x922b8946, SUC_lexicon_tags_15 },
    { 0xef2b8948, SUC_lexicon_tags_45 },
    { 0xcf0b8949, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb894e, SUC_lexicon_tags_11 },
    { 0xc5eb894f, SUC_lexicon_tags_1 },
    { 0x47cb8950, SUC_lexicon_tags_33 },
    { 0x5a8b8950, SUC_lexicon_tags_12 },
    { 0x154b8951, SUC_lexicon_tags_11 },
    { 0x226b894f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1c2b8955, SUC_lexicon_tags_46 },
    { 0xbf0b8956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76ab8958, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceb8960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32ab8962, SUC_lexicon_tags_60 },
    { 0x6e0b8963, SUC_lexicon_tags_23 },
    { 0xa64b8964, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf52b8966, SUC_lexicon_tags_16 },
    { 0x502b8967, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aeb896a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c6b896c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76cb896e, SUC_lexicon_tags_7 },
    { 0xa96b896f, SUC_lexicon_tags_37 },
    { 0xbaeb896e, SUC_lexicon_tags_7 },
    { 0xc6b8971, SUC_lexicon_tags_63 },
    { 0xae8b8972, SUC_lexicon_tags_56 },
    { 0x24cb8973, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8dcb8975, SUC_lexicon_tags_17 },
    { 0x916b8976, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cb897b, SUC_lexicon_tags_45 },
    { 0xa40b897c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8b8980, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8d8b8982, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622b8986, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ab8989, SUC_lexicon_tags_14 },
    { 0x2eab8989, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2ab898c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccab898f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acb8993, SUC_lexicon_tags_55 },
    { 0x99eb8993, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ab899e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb89a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2b89a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2b89ab, SUC_lexicon_tags_79 },
    { 0x2d4b89ab, SUC_lexicon_tags_0 },
    { 0x49eb89ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccb89b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x64cb89b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ccb89b9, SUC_lexicon_tags_8 },
    { 0xd42b89ba, SUC_lexicon_tags_7 },
    { 0xac2b89bb, SUC_lexicon_tags_2 },
    { 0xdf0b89bc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8ccb89be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x172b89c1, SUC_lexicon_tags_33 },
    { 0x41cb89c1, SUC_lexicon_tags_25 },
    { 0x10cb89c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeb89c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfa0b89c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x206b89cb, SUC_lexicon_tags_45 },
    { 0x9ab89cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc28b89ce, SUC_lexicon_tags_25 },
    { 0xbc4b89ce, SUC_lexicon_tags_45 },
    { 0x93ab89d0, SUC_lexicon_tags_29 },
    { 0x4ab89ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb89d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36ab89d6, SUC_lexicon_tags_14 },
    { 0xe74b89d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2b89dd, SUC_lexicon_tags_2 },
    { 0xc52b89de, SUC_lexicon_tags_20 },
    { 0x506b89dd, SUC_lexicon_tags_2 },
    { 0x65cb89e0, SUC_lexicon_tags_0 },
    { 0x87eb89e0, SUC_lexicon_tags_17 },
    { 0xe7cb89e2, SUC_lexicon_tags_542 },
    { 0xcf8b89e3, SUC_lexicon_tags_12 },
    { 0x19cb89e4, SUC_lexicon_tags_27 },
    { 0x21eb89e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x51cb89e7, SUC_lexicon_tags_9 },
    { 0xaf6b89e7, SUC_lexicon_tags_1 },
    { 0xb42b89e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4b89eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x77cb89ed, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646b89f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x112b89f6, SUC_lexicon_tags_66 },
    { 0x370b89f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0b8a03, SUC_lexicon_tags_1 },
    { 0xb66b8a04, SUC_lexicon_tags_31 },
    { 0xd10b8a05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea8b8a07, SUC_lexicon_tags_0 },
    { 0xfcab8a08, SUC_lexicon_tags_2 },
    { 0x8acb8a07, SUC_lexicon_tags_129 },
    { 0xb68b8a07, SUC_lexicon_tags_37 },
    { 0x78eb8a0b, SUC_lexicon_tags_17 },
    { 0x2bcb8a0c, SUC_lexicon_tags_15 },
    { 0xb12b8a0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2cb8a0f, SUC_lexicon_tags_87 },
    { 0xc6b8a10, SUC_lexicon_tags_14 },
    { 0x87cb8a0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348b8a16, SUC_lexicon_tags_60 },
    { 0xc42b8a17, SUC_lexicon_tags_61 },
    { 0xa9cb8a18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd0cb8a1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b4b8a1c, SUC_lexicon_tags_0 },
    { 0xe80b8a1d, SUC_lexicon_tags_3 },
    { 0xf2ab8a1e, SUC_lexicon_tags_11 },
    { 0x92cb8a1c, SUC_lexicon_tags_4 },
    { 0x52ab8a1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80b8a23, SUC_lexicon_tags_33 },
    { 0x626b8a23, SUC_lexicon_tags_25 },
    { 0xcaeb8a25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46b8a28, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x85ab8a2a, SUC_lexicon_tags_14 },
    { 0x110b8a2b, SUC_lexicon_tags_15 },
    { 0xd72b8a2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9eb8a2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x640b8a30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab8a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700b8a3a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cb8a3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x332b8a40, SUC_lexicon_tags_63 },
    { 0x8c0b8a41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab8a45, SUC_lexicon_tags_12 },
    { 0xa78b8a46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a6b8a48, SUC_lexicon_tags_37 },
    { 0xf3cb8a49, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08b8a4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4b8a51, SUC_lexicon_tags_26 },
    { 0xd02b8a52, SUC_lexicon_tags_31 },
    { 0x4f8b8a51, SUC_lexicon_tags_15 },
    { 0x1dcb8a53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ab8a57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3cab8a59, SUC_lexicon_tags_37 },
    { 0xa38b8a59, SUC_lexicon_tags_30 },
    { 0xd66b8a5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xedcb8a5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x442b8a5f, SUC_lexicon_tags_45 },
    { 0x9beb8a5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72b8a69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528b8a6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938b8a72, SUC_lexicon_tags_37 },
    { 0xce4b8a73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x682b8a75, SUC_lexicon_tags_1 },
    { 0x2e0b8a76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ab8a7a, SUC_lexicon_tags_25 },
    { 0x5acb8a7b, SUC_lexicon_tags_37 },
    { 0xdbeb8a7c, SUC_lexicon_tags_0 },
    { 0xa58b8a7d, SUC_lexicon_tags_38 },
    { 0x7ceb8a7c, SUC_lexicon_tags_29 },
    { 0x7cab8a7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eb8a8a, SUC_lexicon_tags_83 },
    { 0xf36b8a8a, SUC_lexicon_tags_37 },
    { 0x70ab8a8c, SUC_lexicon_tags_160 },
    { 0xeceb8a8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x550b8a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd34b8a92, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x32b8a94, SUC_lexicon_tags_14 },
    { 0x60eb8a95, SUC_lexicon_tags_27 },
    { 0xe28b8a96, SUC_lexicon_tags_27 },
    { 0x7aeb8a94, SUC_lexicon_tags_27 },
    { 0x814b8a95, SUC_lexicon_tags_25 },
    { 0xb46b8a99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x726b8a9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ab8a9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f0b8aa1, SUC_lexicon_tags_7 },
    { 0xf2b8aa2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cb8aa5, SUC_lexicon_tags_15 },
    { 0xb6eb8aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdaab8aa8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdb0b8aaa, SUC_lexicon_tags_27 },
    { 0x912b8aab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcdeb8aad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2b0b8aaf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24eb8ab1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992b8ab9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ab8abd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cb8ac1, SUC_lexicon_tags_538 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62b8ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22b8acb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x272b8ace, SUC_lexicon_tags_29 },
    { 0x6aab8acf, SUC_lexicon_tags_36 },
    { 0xa04b8ad0, SUC_lexicon_tags_47 },
    { 0x778b8ace, SUC_lexicon_tags_7 },
    { 0xebcb8ad2, SUC_lexicon_tags_0 },
    { 0x9e6b8ad3, SUC_lexicon_tags_11 },
    { 0x71cb8ad4, SUC_lexicon_tags_23 },
    { 0xfc2b8ad3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274b8ad9, SUC_lexicon_tags_7 },
    { 0xcdcb8ad9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ab8add, SUC_lexicon_tags_8 },
    { 0x7c6b8ade, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba4b8ae0, SUC_lexicon_tags_15 },
    { 0xf60b8ae1, SUC_lexicon_tags_7 },
    { 0x80cb8ae2, SUC_lexicon_tags_37 },
    { 0x726b8ae3, SUC_lexicon_tags_45 },
    { 0x400b8ae4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x10eb8ae6, SUC_lexicon_tags_17 },
    { 0x8e4b8ae7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x772b8aea, SUC_lexicon_tags_65 },
    { 0xdeab8aea, SUC_lexicon_tags_2 },
    { 0x7f4b8aec, SUC_lexicon_tags_31 },
    { 0x384b8aec, SUC_lexicon_tags_13 },
    { 0xf8b8aee, SUC_lexicon_tags_25 },
    { 0x994b8aec, SUC_lexicon_tags_37 },
    { 0xd7eb8af0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ab8af4, SUC_lexicon_tags_11 },
    { 0x1f0b8af5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6eb8af7, SUC_lexicon_tags_63 },
    { 0x7ceb8af8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5eb8afa, SUC_lexicon_tags_0 },
    { 0xfaab8afb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0b8afe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeab8b02, SUC_lexicon_tags_37 },
    { 0x326b8b03, SUC_lexicon_tags_43 },
    { 0x2f0b8b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4b8b07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240b8b11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8ab8b13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2b8b17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7f6b8b19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedab8b1b, SUC_lexicon_tags_39 },
    { 0x66b8b1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8eb8b20, SUC_lexicon_tags_7 },
    { 0xa40b8b21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x800b8b23, SUC_lexicon_tags_17 },
    { 0x18eb8b24, SUC_lexicon_tags_1 },
    { 0xf38b8b25, SUC_lexicon_tags_44 },
    { 0xf72b8b26, SUC_lexicon_tags_0 },
    { 0x772b8b27, SUC_lexicon_tags_1 },
    { 0xf8ab8b27, SUC_lexicon_tags_19 },
    { 0xbc0b8b29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44eb8b2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x334b8b2e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12b8b32, SUC_lexicon_tags_25 },
    { 0xfd4b8b33, SUC_lexicon_tags_15 },
    { 0xa28b8b34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87eb8b45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcb8b49, SUC_lexicon_tags_29 },
    { 0x97eb8b49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2b8b4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcb8b52, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x716b8b54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4b8b57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0b8b5c, SUC_lexicon_tags_39 },
    { 0xb52b8b5c, SUC_lexicon_tags_37 },
    { 0x4deb8b5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6b8b61, SUC_lexicon_tags_15 },
    { 0x1d4b8b61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8b8b64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2b8b67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91eb8b6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1aeb8b6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb8ab8b6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc6b8b71, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53eb8b76, SUC_lexicon_tags_63 },
    { 0x1cab8b76, SUC_lexicon_tags_49 },
    { 0x354b8b78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x648b8b7b, SUC_lexicon_tags_8 },
    { 0x6ecb8b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2beb8b7f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x704b8b81, SUC_lexicon_tags_27 },
    { 0x42cb8b82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90cb8b84, SUC_lexicon_tags_1 },
    { 0xf60b8b85, SUC_lexicon_tags_15 },
    { 0xebeb8b86, SUC_lexicon_tags_14 },
    { 0x71eb8b87, SUC_lexicon_tags_9 },
    { 0xc84b8b88, SUC_lexicon_tags_2 },
    { 0x394b8b89, SUC_lexicon_tags_15 },
    { 0xb7cb8b8a, SUC_lexicon_tags_15 },
    { 0x936b8b88, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x864b8b8e, SUC_lexicon_tags_45 },
    { 0x368b8b8f, SUC_lexicon_tags_1 },
    { 0x2e6b8b90, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eab8b97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326b8b9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0b8ba2, SUC_lexicon_tags_87 },
    { 0xef0b8ba3, SUC_lexicon_tags_12 },
    { 0xc44b8ba4, SUC_lexicon_tags_4 },
    { 0x9c2b8ba2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb7eb8ba7, SUC_lexicon_tags_37 },
    { 0x560b8ba7, SUC_lexicon_tags_2 },
    { 0x7beb8ba9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x490b8bac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeb8bb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49eb8bb8, SUC_lexicon_tags_31 },
    { 0x488b8bb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4b8bbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ab8bbf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x11cb8bc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb72b8bc3, SUC_lexicon_tags_37 },
    { 0xfa4b8bc4, SUC_lexicon_tags_23 },
    { 0xec2b8bc3, SUC_lexicon_tags_12 },
    { 0x39eb8bc5, SUC_lexicon_tags_31 },
    { 0x654b8bc7, SUC_lexicon_tags_27 },
    { 0x504b8bc7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x976b8bca, SUC_lexicon_tags_28 },
    { 0xd4ab8bcb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x954b8bce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eb8bd2, SUC_lexicon_tags_48 },
    { 0x28cb8bd3, SUC_lexicon_tags_11 },
    { 0x38cb8bd3, SUC_lexicon_tags_17 },
    { 0x4a4b8bd5, SUC_lexicon_tags_7 },
    { 0xaeb8bd6, SUC_lexicon_tags_6 },
    { 0xc2cb8bd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6feb8bdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968b8bdf, SUC_lexicon_tags_2 },
    { 0xe58b8be0, SUC_lexicon_tags_11 },
    { 0x6f8b8be0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4beb8be3, SUC_lexicon_tags_15 },
    { 0xbeab8be4, SUC_lexicon_tags_35 },
    { 0xffcb8be4, SUC_lexicon_tags_0 },
    { 0x1f0b8be6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec4b8be8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x120b8beb, SUC_lexicon_tags_17 },
    { 0x11cb8bec, SUC_lexicon_tags_25 },
    { 0xddeb8bed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9ab8bef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eab8bf3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818b8bfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ab8c00, SUC_lexicon_tags_7 },
    { 0x540b8c00, SUC_lexicon_tags_36 },
    { 0x57ab8c02, SUC_lexicon_tags_103 },
    { 0xc78b8c03, SUC_lexicon_tags_3 },
    { 0xc5ab8c03, SUC_lexicon_tags_17 },
    { 0x5c0b8c04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fab8c08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x38eb8c0b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcaeb8c0d, SUC_lexicon_tags_2 },
    { 0xfacb8c0e, SUC_lexicon_tags_14 },
    { 0xc84b8c0f, SUC_lexicon_tags_275 },
    { 0xeeb8c10, SUC_lexicon_tags_37 },
    { 0xc38b8c11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4b8c15, SUC_lexicon_tags_26 },
    { 0xe10b8c16, SUC_lexicon_tags_61 },
    { 0x2b8b8c17, SUC_lexicon_tags_12 },
    { 0x1fab8c17, SUC_lexicon_tags_17 },
    { 0xf24b8c19, SUC_lexicon_tags_45 },
    { 0x5ecb8c19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4cab8c1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab8c20, SUC_lexicon_tags_3 },
    { 0x6fab8c21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38b8c26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50b8c2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccab8c31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16b8c35, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4b8c3b, SUC_lexicon_tags_7 },
    { 0x62ab8c3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdaab8c3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916b8c42, SUC_lexicon_tags_2 },
    { 0xd76b8c43, SUC_lexicon_tags_25 },
    { 0x182b8c44, SUC_lexicon_tags_28 },
    { 0xc10b8c43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4b8c48, SUC_lexicon_tags_27 },
    { 0x7c6b8c48, SUC_lexicon_tags_2 },
    { 0xea0b8c49, SUC_lexicon_tags_16 },
    { 0xd6ab8c4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78eb8c5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4b8c5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbab8c63, SUC_lexicon_tags_14 },
    { 0x234b8c64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86b8c67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x902b8c6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dab8c71, SUC_lexicon_tags_43 },
    { 0x37eb8c72, SUC_lexicon_tags_55 },
    { 0xcb4b8c73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ab8c77, SUC_lexicon_tags_11 },
    { 0x82ab8c78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614b8c7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ab8c81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2b8c84, SUC_lexicon_tags_1 },
    { 0x82b8c85, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcb8c89, SUC_lexicon_tags_60 },
    { 0x6d2b8c89, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe3ab8c8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cb8c96, SUC_lexicon_tags_18 },
    { 0xfeab8c97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4b8c9b, SUC_lexicon_tags_2 },
    { 0x5d0b8c9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a0b8c9e, SUC_lexicon_tags_978 },
    { 0xd76b8c9f, SUC_lexicon_tags_8 },
    { 0x530b8ca0, SUC_lexicon_tags_46 },
    { 0x4ccb8ca1, SUC_lexicon_tags_0 },
    { 0xc2cb8ca2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31eb8ca4, SUC_lexicon_tags_1 },
    { 0xc22b8ca5, SUC_lexicon_tags_127 },
    { 0x526b8ca4, SUC_lexicon_tags_37 },
    { 0x410b8ca7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x944b8caa, SUC_lexicon_tags_61 },
    { 0xbeb8caa, SUC_lexicon_tags_30 },
    { 0x5f2b8caa, SUC_lexicon_tags_6 },
    { 0x9beb8cad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b8cb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef6b8cbb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcb8cbe, SUC_lexicon_tags_14 },
    { 0xf5eb8cbf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x434b8cc2, SUC_lexicon_tags_23 },
    { 0xd80b8cc2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0b8cc8, SUC_lexicon_tags_26 },
    { 0x78cb8cc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x256b8ccb, SUC_lexicon_tags_7 },
    { 0xdf8b8ccc, SUC_lexicon_tags_8 },
    { 0x4bab8ccd, SUC_lexicon_tags_17 },
    { 0x4e8b8ccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8cd2, SUC_lexicon_tags_74 },
    { 0xe46b8cd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb46b8cd5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x546b8cd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x672b8cda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4eb8cdc, SUC_lexicon_tags_79 },
    { 0x264b8cdd, SUC_lexicon_tags_6 },
    { 0x7c6b8cdc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ab8ce4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978b8cef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x546b8cf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcb8cf5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4f0b8cf7, SUC_lexicon_tags_586 },
    { 0x6eeb8cf8, SUC_lexicon_tags_17 },
    { 0xbeb8cf9, SUC_lexicon_tags_11 },
    { 0xec4b8cfa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf0b8cfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f4b8cfe, SUC_lexicon_tags_14 },
    { 0xe42b8cfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcab8d03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46eb8d05, SUC_lexicon_tags_65 },
    { 0xfc8b8d06, SUC_lexicon_tags_8 },
    { 0xfb6b8d07, SUC_lexicon_tags_37 },
    { 0x3b8b8d07, SUC_lexicon_tags_23 },
    { 0xddab8d09, SUC_lexicon_tags_37 },
    { 0x7bab8d0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe16b8d0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeb8b8d0e, SUC_lexicon_tags_17 },
    { 0xb4cb8d0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828b8d13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf94b8d15, SUC_lexicon_tags_85 },
    { 0xff2b8d16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61eb8d18, SUC_lexicon_tags_12 },
    { 0xc7ab8d19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584b8d20, SUC_lexicon_tags_21 },
    { 0x3deb8d21, SUC_lexicon_tags_11 },
    { 0xe9ab8d21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22b8d26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c2b8d28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc00b8d2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722b8d2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2b8d32, SUC_lexicon_tags_17 },
    { 0x4b4b8d33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf0eb8d35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb22b8d37, SUC_lexicon_tags_27 },
    { 0x59eb8d37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80b8d3b, SUC_lexicon_tags_0 },
    { 0x7fcb8d3b, SUC_lexicon_tags_14 },
    { 0xd40b8d3d, SUC_lexicon_tags_48 },
    { 0xee8b8d3e, SUC_lexicon_tags_15 },
    { 0xbfcb8d3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54b8d43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ab8d4e, SUC_lexicon_tags_45 },
    { 0x5a4b8d4f, SUC_lexicon_tags_7 },
    { 0x910b8d4f, SUC_lexicon_tags_66 },
    { 0x8b6b8d51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0b8d59, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44b8d5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86b8d61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91eb8d66, SUC_lexicon_tags_60 },
    { 0x5ecb8d66, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cab8d73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0eb8d78, SUC_lexicon_tags_10 },
    { 0xd76b8d78, SUC_lexicon_tags_62 },
    { 0x8d4b8d7a, SUC_lexicon_tags_15 },
    { 0x8a4b8d7b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2b8d7d, SUC_lexicon_tags_0 },
    { 0x3e8b8d7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd24b8d80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb8d83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04b8d87, SUC_lexicon_tags_68 },
    { 0x720b8d88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aab8d8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x394b8d92, SUC_lexicon_tags_16 },
    { 0xd58b8d93, SUC_lexicon_tags_8 },
    { 0xa0eb8d94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2b8d9a, SUC_lexicon_tags_27 },
    { 0x7f2b8d9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b8d9e, SUC_lexicon_tags_2 },
    { 0xf70b8d9f, SUC_lexicon_tags_2 },
    { 0x3eab8da0, SUC_lexicon_tags_1 },
    { 0xc28b8da1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa84b8da3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06b8daa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x730b8dad, SUC_lexicon_tags_37 },
    { 0xe24b8dae, SUC_lexicon_tags_48 },
    { 0xdd2b8daf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202b8db4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcb8db8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c0b8dba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4b8dbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x372b8dc1, SUC_lexicon_tags_37 },
    { 0x89eb8dc2, SUC_lexicon_tags_31 },
    { 0x676b8dc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaab8dc8, SUC_lexicon_tags_12 },
    { 0x56ab8dc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04b8dcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x854b8dd1, SUC_lexicon_tags_2 },
    { 0x818b8dd2, SUC_lexicon_tags_14 },
    { 0x4b0b8dd2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2b8dd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b8dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdab8de0, SUC_lexicon_tags_12 },
    { 0xb56b8de1, SUC_lexicon_tags_37 },
    { 0x836b8de0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ab8de5, SUC_lexicon_tags_70 },
    { 0xfa0b8de5, SUC_lexicon_tags_12 },
    { 0x8bcb8de7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x404b8dea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cb8dee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x506b8df0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x758b8df2, SUC_lexicon_tags_21 },
    { 0xceeb8df3, SUC_lexicon_tags_15 },
    { 0x5e8b8df4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b4b8df6, SUC_lexicon_tags_10 },
    { 0x462b8df7, SUC_lexicon_tags_179 },
    { 0x812b8df8, SUC_lexicon_tags_37 },
    { 0x582b8df6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0b8dfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce6b8dfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee0b8e00, SUC_lexicon_tags_47 },
    { 0xa16b8e00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96eb8e09, SUC_lexicon_tags_17 },
    { 0x9aab8e09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794b8e11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x156b8e13, SUC_lexicon_tags_10 },
    { 0xc32b8e13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cb8e19, SUC_lexicon_tags_60 },
    { 0xba2b8e19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74b8e1d, SUC_lexicon_tags_85 },
    { 0x4c2b8e1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab8e22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95eb8e24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ab8e31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0b8e35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ab8e3a, SUC_lexicon_tags_23 },
    { 0x93cb8e3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x124b8e3d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cb8e42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2eb8e44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888b8e48, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666b8e4c, SUC_lexicon_tags_65 },
    { 0xec4b8e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fab8e4f, SUC_lexicon_tags_9 },
    { 0x14eb8e50, SUC_lexicon_tags_15 },
    { 0xc46b8e51, SUC_lexicon_tags_3 },
    { 0x7acb8e52, SUC_lexicon_tags_27 },
    { 0xc68b8e53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8b8e57, SUC_lexicon_tags_17 },
    { 0xe32b8e58, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acb8e5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeceb8e5f, SUC_lexicon_tags_87 },
    { 0x1cab8e60, SUC_lexicon_tags_17 },
    { 0xaccb8e61, SUC_lexicon_tags_39 },
    { 0x2feb8e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb8e67, SUC_lexicon_tags_113 },
    { 0x4b2b8e68, SUC_lexicon_tags_2 },
    { 0x40eb8e68, SUC_lexicon_tags_15 },
    { 0xb9cb8e6a, SUC_lexicon_tags_17 },
    { 0xfd8b8e69, SUC_lexicon_tags_45 },
    { 0xb88b8e6c, SUC_lexicon_tags_37 },
    { 0x1d6b8e6c, SUC_lexicon_tags_28 },
    { 0xc1eb8e6e, SUC_lexicon_tags_15 },
    { 0x4f2b8e6c, SUC_lexicon_tags_47 },
    { 0x8cb8e70, SUC_lexicon_tags_7 },
    { 0x7e4b8e69, SUC_lexicon_tags_11 },
    { 0x6d6b8e6c, SUC_lexicon_tags_151 },
    { 0x3c2b8e73, SUC_lexicon_tags_17 },
    { 0x23eb8e6c, SUC_lexicon_tags_37 },
    { 0xc02b8e75, SUC_lexicon_tags_10 },
    { 0x1a8b8e75, SUC_lexicon_tags_11 },
    { 0xbd4b8e76, SUC_lexicon_tags_13 },
    { 0xb2eb8e76, SUC_lexicon_tags_7 },
    { 0x93ab8e77, SUC_lexicon_tags_11 },
    { 0x67eb8e6b, SUC_lexicon_tags_10 },
    { 0x76cb8e6a, SUC_lexicon_tags_26 },
    { 0xa74b8e7c, SUC_lexicon_tags_7 },
    { 0x86ab8e7d, SUC_lexicon_tags_17 },
    { 0xe60b8e7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x85cb8e80, SUC_lexicon_tags_12 },
    { 0x462b8e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa62b8e83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4b8e87, SUC_lexicon_tags_31 },
    { 0xa8eb8e88, SUC_lexicon_tags_33 },
    { 0x6feb8e89, SUC_lexicon_tags_27 },
    { 0x880b8e8a, SUC_lexicon_tags_21 },
    { 0xbb6b8e8a, SUC_lexicon_tags_87 },
    { 0x6c4b8e8c, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94eb8e92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0eb8e94, SUC_lexicon_tags_37 },
    { 0xc64b8e95, SUC_lexicon_tags_45 },
    { 0x64ab8e95, SUC_lexicon_tags_84 },
    { 0x1c8b8e97, SUC_lexicon_tags_113 },
    { 0xf92b8e97, SUC_lexicon_tags_1 },
    { 0x5d6b8e97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6b8e9c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcb8e9f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cb8ea2, SUC_lexicon_tags_16 },
    { 0xb28b8ea3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x616b8ea6, SUC_lexicon_tags_136 },
    { 0x814b8ea7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53ab8ea9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x602b8eac, SUC_lexicon_tags_0 },
    { 0xb60b8ead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732b8eb1, SUC_lexicon_tags_7 },
    { 0xe76b8eb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62b8eb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260b8ebe, SUC_lexicon_tags_11 },
    { 0x1c4b8ebf, SUC_lexicon_tags_11 },
    { 0x280b8ec0, SUC_lexicon_tags_45 },
    { 0x514b8ec1, SUC_lexicon_tags_37 },
    { 0xac2b8ebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19cb8ec5, SUC_lexicon_tags_18 },
    { 0x2b8b8ec6, SUC_lexicon_tags_17 },
    { 0x1d8b8ec6, SUC_lexicon_tags_7 },
    { 0xabab8ec6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x998b8eca, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0b8ecf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x166b8ed1, SUC_lexicon_tags_46 },
    { 0x47cb8ed2, SUC_lexicon_tags_38 },
    { 0x3f8b8ed3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d0b8ed5, SUC_lexicon_tags_16 },
    { 0x37ab8ed6, SUC_lexicon_tags_17 },
    { 0x71cb8ed7, SUC_lexicon_tags_11 },
    { 0x7e2b8ed8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cb8edd, SUC_lexicon_tags_7 },
    { 0xb9ab8edd, SUC_lexicon_tags_26 },
    { 0xf26b8edf, SUC_lexicon_tags_66 },
    { 0xd24b8ee0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd72b8ee2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbab8ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd54b8ee7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89cb8eea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb8eec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x382b8eee, SUC_lexicon_tags_47 },
    { 0xaccb8eef, SUC_lexicon_tags_87 },
    { 0x7c8b8eef, SUC_lexicon_tags_11 },
    { 0xf7cb8ef1, SUC_lexicon_tags_31 },
    { 0x6beb8eef, SUC_lexicon_tags_17 },
    { 0xbaab8ef3, SUC_lexicon_tags_38 },
    { 0x4e2b8ef3, SUC_lexicon_tags_17 },
    { 0x296b8eee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ab8efb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeccb8efd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x682b8eff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeeb8f02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8b8f06, SUC_lexicon_tags_2 },
    { 0x7c8b8f06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2b8f0a, SUC_lexicon_tags_7 },
    { 0x83cb8f0b, SUC_lexicon_tags_27 },
    { 0x8cb8f0b, SUC_lexicon_tags_37 },
    { 0x1d6b8f0a, SUC_lexicon_tags_25 },
    { 0x56eb8f0e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x96cb8f10, SUC_lexicon_tags_37 },
    { 0xe1eb8f11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcb8f1a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686b8f1e, SUC_lexicon_tags_224 },
    { 0x8a6b8f1f, SUC_lexicon_tags_8 },
    { 0x20cb8f20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cb8f23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8f6b8f25, SUC_lexicon_tags_1 },
    { 0x9a6b8f25, SUC_lexicon_tags_15 },
    { 0x772b8f26, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254b8f2d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x53ab8f2f, SUC_lexicon_tags_26 },
    { 0x864b8f30, SUC_lexicon_tags_23 },
    { 0xef4b8f31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402b8f37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832b8f3b, SUC_lexicon_tags_39 },
    { 0x876b8f3c, SUC_lexicon_tags_25 },
    { 0x34eb8f3d, SUC_lexicon_tags_25 },
    { 0xbc2b8f3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa28b8f40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846b8f46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x856b8f48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14b8f56, SUC_lexicon_tags_25 },
    { 0xbb8b8f57, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c2b8f59, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30b8f64, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2b8f67, SUC_lexicon_tags_12 },
    { 0x112b8f68, SUC_lexicon_tags_10 },
    { 0xea0b8f69, SUC_lexicon_tags_37 },
    { 0xbfcb8f69, SUC_lexicon_tags_11 },
    { 0x316b8f67, SUC_lexicon_tags_12 },
    { 0x946b8f69, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ab8f70, SUC_lexicon_tags_14 },
    { 0x6c0b8f71, SUC_lexicon_tags_39 },
    { 0x3ccb8f72, SUC_lexicon_tags_14 },
    { 0x20cb8f73, SUC_lexicon_tags_25 },
    { 0x6c0b8f74, SUC_lexicon_tags_37 },
    { 0x50ab8f74, SUC_lexicon_tags_29 },
    { 0xb60b8f73, SUC_lexicon_tags_9 },
    { 0xb7cb8f77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cb8f7c, SUC_lexicon_tags_54 },
    { 0x7bcb8f7d, SUC_lexicon_tags_7 },
    { 0xb8b8f7e, SUC_lexicon_tags_25 },
    { 0x546b8f7e, SUC_lexicon_tags_3 },
    { 0x52ab8f7e, SUC_lexicon_tags_63 },
    { 0xd22b8f80, SUC_lexicon_tags_293 },
    { 0xebcb8f81, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aab8f86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680b8f91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6b8f95, SUC_lexicon_tags_43 },
    { 0x118b8f96, SUC_lexicon_tags_17 },
    { 0x946b8f96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94b8f9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808b8fa9, SUC_lexicon_tags_23 },
    { 0x75cb8faa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430b8fae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcaeb8fb0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd80b8fb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8b8fb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8aeb8fb9, SUC_lexicon_tags_42 },
    { 0x7a4b8fba, SUC_lexicon_tags_63 },
    { 0xc10b8fba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4b8fbf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x248b8fc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac8b8fc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcb6b8fc6, SUC_lexicon_tags_66 },
    { 0x486b8fc6, SUC_lexicon_tags_49 },
    { 0x2b6b8fc7, SUC_lexicon_tags_49 },
    { 0x960b8fc9, SUC_lexicon_tags_37 },
    { 0xe52b8fc8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cb8fce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4beb8fd1, SUC_lexicon_tags_19 },
    { 0x3fab8fd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8b8fd6, SUC_lexicon_tags_13 },
    { 0x100b8fd7, SUC_lexicon_tags_2 },
    { 0xbe4b8fd8, SUC_lexicon_tags_33 },
    { 0x69ab8fd9, SUC_lexicon_tags_11 },
    { 0xdc8b8fd6, SUC_lexicon_tags_37 },
    { 0x1cb8fdb, SUC_lexicon_tags_1 },
    { 0xde4b8fda, SUC_lexicon_tags_47 },
    { 0x716b8fdd, SUC_lexicon_tags_60 },
    { 0xf32b8fde, SUC_lexicon_tags_39 },
    { 0xceeb8fdf, SUC_lexicon_tags_91 },
    { 0x1a0b8fe0, SUC_lexicon_tags_4 },
    { 0x1d0b8fe0, SUC_lexicon_tags_19 },
    { 0x8ab8fdd, SUC_lexicon_tags_80 },
    { 0xa26b8fe3, SUC_lexicon_tags_26 },
    { 0xb9eb8fe1, SUC_lexicon_tags_19 },
    { 0x564b8fe5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06b8fe8, SUC_lexicon_tags_106 },
    { 0x80eb8fe9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcb8fec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4b8ff2, SUC_lexicon_tags_37 },
    { 0x2c8b8ff3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ab8ff9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dab9000, SUC_lexicon_tags_14 },
    { 0xf5ab9000, SUC_lexicon_tags_2 },
    { 0xf00b9001, SUC_lexicon_tags_45 },
    { 0xaa6b9003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56b9007, SUC_lexicon_tags_0 },
    { 0xe0ab9008, SUC_lexicon_tags_33 },
    { 0xf5eb9008, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b900d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16b9010, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x56ab9012, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2b9015, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9eb901f, SUC_lexicon_tags_2 },
    { 0x10ab9020, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf18b9022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb9025, SUC_lexicon_tags_148 },
    { 0x2f4b9026, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658b902b, SUC_lexicon_tags_11 },
    { 0x7aeb902b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x396b902e, SUC_lexicon_tags_14 },
    { 0x280b902f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52ab9031, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ab9035, SUC_lexicon_tags_27 },
    { 0x726b9036, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990b903e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x108b9041, SUC_lexicon_tags_25 },
    { 0x62eb9042, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96b9049, SUC_lexicon_tags_28 },
    { 0x132b904a, SUC_lexicon_tags_204 },
    { 0xe2ab904b, SUC_lexicon_tags_7 },
    { 0x3eeb904c, SUC_lexicon_tags_11 },
    { 0xa22b904c, SUC_lexicon_tags_61 },
    { 0x142b904e, SUC_lexicon_tags_26 },
    { 0xf90b904f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eab9052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa32b9054, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcb9057, SUC_lexicon_tags_46 },
    { 0x680b9058, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca2b905a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cb905f, SUC_lexicon_tags_39 },
    { 0x5ecb905f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb9067, SUC_lexicon_tags_23 },
    { 0x51cb9068, SUC_lexicon_tags_17 },
    { 0xd2b9067, SUC_lexicon_tags_30 },
    { 0x86ab906a, SUC_lexicon_tags_1 },
    { 0xe08b906b, SUC_lexicon_tags_11 },
    { 0xf00b906c, SUC_lexicon_tags_84 },
    { 0x4a8b906c, SUC_lexicon_tags_7 },
    { 0x862b906e, SUC_lexicon_tags_23 },
    { 0x4e6b906f, SUC_lexicon_tags_25 },
    { 0x57eb906f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4b907b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cb907f, SUC_lexicon_tags_38 },
    { 0x526b9080, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40b9084, SUC_lexicon_tags_27 },
    { 0xcdcb9084, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7beb908b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x342b908d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0b9096, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b9099, SUC_lexicon_tags_60 },
    { 0xd7ab909a, SUC_lexicon_tags_23 },
    { 0xee2b909b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728b90a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b90a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72b90aa, SUC_lexicon_tags_14 },
    { 0x83eb90ab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cb90b1, SUC_lexicon_tags_19 },
    { 0xa0cb90b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcb90b8, SUC_lexicon_tags_25 },
    { 0x5b0b90b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97cb90bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x69eb90c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94eb90c5, SUC_lexicon_tags_15 },
    { 0x67cb90c6, SUC_lexicon_tags_49 },
    { 0xea0b90c7, SUC_lexicon_tags_38 },
    { 0xab90c6, SUC_lexicon_tags_0 },
    { 0xdfab90c7, SUC_lexicon_tags_8 },
    { 0x108b90ca, SUC_lexicon_tags_11 },
    { 0xdb2b90c9, SUC_lexicon_tags_15 },
    { 0xd1ab90c8, SUC_lexicon_tags_30 },
    { 0x104b90cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4b90d8, SUC_lexicon_tags_37 },
    { 0xd0cb90d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb70b90db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x746b90de, SUC_lexicon_tags_7 },
    { 0xee0b90df, SUC_lexicon_tags_21 },
    { 0xe7ab90e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0b90e3, SUC_lexicon_tags_43 },
    { 0x27eb90e4, SUC_lexicon_tags_17 },
    { 0x2f8b90e4, SUC_lexicon_tags_1 },
    { 0x29cb90e6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30ab90e8, SUC_lexicon_tags_11 },
    { 0x76eb90e9, SUC_lexicon_tags_25 },
    { 0xcb2b90ea, SUC_lexicon_tags_30 },
    { 0x4a2b90eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2aeb90ed, SUC_lexicon_tags_43 },
    { 0x484b90ed, SUC_lexicon_tags_31 },
    { 0x50cb90ef, SUC_lexicon_tags_25 },
    { 0x2c0b90ef, SUC_lexicon_tags_14 },
    { 0x152b90f1, SUC_lexicon_tags_47 },
    { 0x4e0b90f1, SUC_lexicon_tags_25 },
    { 0x82cb90f3, SUC_lexicon_tags_25 },
    { 0x84ab90ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8b90f7, SUC_lexicon_tags_23 },
    { 0xcd0b90f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9beb90fc, SUC_lexicon_tags_25 },
    { 0xf9cb90fd, SUC_lexicon_tags_17 },
    { 0x906b90fd, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x456b9101, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaab9105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeab9108, SUC_lexicon_tags_68 },
    { 0xec8b9109, SUC_lexicon_tags_17 },
    { 0xc4ab910a, SUC_lexicon_tags_13 },
    { 0x988b910a, SUC_lexicon_tags_31 },
    { 0xcaab910b, SUC_lexicon_tags_30 },
    { 0x482b910d, SUC_lexicon_tags_27 },
    { 0x59eb910e, SUC_lexicon_tags_92 },
    { 0xec4b910e, SUC_lexicon_tags_34 },
    { 0x3b0b9109, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x890b9112, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabeb9114, SUC_lexicon_tags_10 },
    { 0xc6cb9114, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c2b9117, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944b911b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4b9123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeb9127, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xde4b912a, SUC_lexicon_tags_7 },
    { 0xf96b912b, SUC_lexicon_tags_11 },
    { 0x5fab912c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6b912f, SUC_lexicon_tags_14 },
    { 0x28b912f, SUC_lexicon_tags_25 },
    { 0x44ab9131, SUC_lexicon_tags_15 },
    { 0xedcb912f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d6b9134, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884b9139, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66b913c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x586b913f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0b9143, SUC_lexicon_tags_62 },
    { 0xf50b9144, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaceb9146, SUC_lexicon_tags_5 },
    { 0xbf8b9147, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128b914c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0b9151, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10b9154, SUC_lexicon_tags_11 },
    { 0x61ab9155, SUC_lexicon_tags_17 },
    { 0x748b9156, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330b915a, SUC_lexicon_tags_37 },
    { 0x3a4b915b, SUC_lexicon_tags_4 },
    { 0x86ab915c, SUC_lexicon_tags_11 },
    { 0x148b915a, SUC_lexicon_tags_7 },
    { 0xf56b915a, SUC_lexicon_tags_9 },
    { 0xa06b915f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa22b9161, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270b9165, SUC_lexicon_tags_11 },
    { 0xee4b9166, SUC_lexicon_tags_11 },
    { 0x8deb9165, SUC_lexicon_tags_25 },
    { 0x5cab9168, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8b9170, SUC_lexicon_tags_68 },
    { 0x6b0b9171, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866b9175, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8b9178, SUC_lexicon_tags_11 },
    { 0x9ceb9178, SUC_lexicon_tags_27 },
    { 0x50b917a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2b917d, SUC_lexicon_tags_21 },
    { 0x5f2b917e, SUC_lexicon_tags_17 },
    { 0x7f4b917e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16b9184, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9beb9187, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6b918e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cb9191, SUC_lexicon_tags_7 },
    { 0x2beb9192, SUC_lexicon_tags_7 },
    { 0x604b9192, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6b9196, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8b919a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x100b919d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ab91a9, SUC_lexicon_tags_17 },
    { 0x650b91aa, SUC_lexicon_tags_3 },
    { 0x904b91a9, SUC_lexicon_tags_37 },
    { 0x28eb91ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x958b91af, SUC_lexicon_tags_1 },
    { 0x526b91b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x63eb91b3, SUC_lexicon_tags_7 },
    { 0xd94b91b4, SUC_lexicon_tags_14 },
    { 0x6c2b91b3, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x388b91b7, SUC_lexicon_tags_25 },
    { 0xb96b91b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x328b91bb, SUC_lexicon_tags_2 },
    { 0x9b8b91bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2b91c5, SUC_lexicon_tags_27 },
    { 0xc5cb91c6, SUC_lexicon_tags_12 },
    { 0x4ecb91c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28b91ca, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xdb4b91cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfcab91ce, SUC_lexicon_tags_17 },
    { 0xe2b91cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530b91d7, SUC_lexicon_tags_7 },
    { 0x898b91d7, SUC_lexicon_tags_82 },
    { 0xcab91d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e0b91db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954b91df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cb91e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc6ab91ea, SUC_lexicon_tags_37 },
    { 0x672b91ea, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00b91ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56b91f4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158b91f8, SUC_lexicon_tags_37 },
    { 0x988b91f8, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xd7eb91fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cb91fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cb9207, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x166b9209, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabeb920b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c2b920d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ab9213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ab9217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x406b921a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x812b921c, SUC_lexicon_tags_25 },
    { 0x812b921d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cb9220, SUC_lexicon_tags_8 },
    { 0x298b9221, SUC_lexicon_tags_7 },
    { 0x814b9222, SUC_lexicon_tags_63 },
    { 0x5a2b9221, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b8b9225, SUC_lexicon_tags_17 },
    { 0x632b9225, SUC_lexicon_tags_15 },
    { 0x812b9227, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8b922c, SUC_lexicon_tags_37 },
    { 0x894b922d, SUC_lexicon_tags_84 },
    { 0x416b922e, SUC_lexicon_tags_2 },
    { 0x3eeb922f, SUC_lexicon_tags_17 },
    { 0xe2b922d, SUC_lexicon_tags_7 },
    { 0x114b9230, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0xdcb9233, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x63ab9235, SUC_lexicon_tags_25 },
    { 0x674b9236, SUC_lexicon_tags_151 },
    { 0xc36b9237, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91cb9239, SUC_lexicon_tags_37 },
    { 0x252b9239, SUC_lexicon_tags_56 },
    { 0x22b923a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eb923f, SUC_lexicon_tags_14 },
    { 0xe98b9240, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30b9246, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d4b9248, SUC_lexicon_tags_47 },
    { 0x766b9249, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79ab924b, SUC_lexicon_tags_0 },
    { 0x9c4b924c, SUC_lexicon_tags_15 },
    { 0x840b924d, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208b9251, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x322b9253, SUC_lexicon_tags_55 },
    { 0xb90b9254, SUC_lexicon_tags_37 },
    { 0x52b9255, SUC_lexicon_tags_1 },
    { 0x518b9256, SUC_lexicon_tags_29 },
    { 0x472b9257, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54ab9259, SUC_lexicon_tags_9 },
    { 0xfaab925a, SUC_lexicon_tags_4 },
    { 0x346b925b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68b9265, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ab926a, SUC_lexicon_tags_17 },
    { 0x136b926a, SUC_lexicon_tags_17 },
    { 0x5eeb926a, SUC_lexicon_tags_8 },
    { 0x39ab926b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ab9272, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x96cb9275, SUC_lexicon_tags_11 },
    { 0xe36b9276, SUC_lexicon_tags_1 },
    { 0x72eb9277, SUC_lexicon_tags_7 },
    { 0x358b9278, SUC_lexicon_tags_17 },
    { 0xb0cb9279, SUC_lexicon_tags_43 },
    { 0x246b9277, SUC_lexicon_tags_9 },
    { 0xa6b9276, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb927f, SUC_lexicon_tags_25 },
    { 0xe30b927f, SUC_lexicon_tags_38 },
    { 0xe82b9281, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8b9289, SUC_lexicon_tags_188 },
    { 0xfa4b9289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8b928e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89cb9290, SUC_lexicon_tags_26 },
    { 0xa8eb9291, SUC_lexicon_tags_0 },
    { 0x410b9292, SUC_lexicon_tags_7 },
    { 0x260b9293, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9feb9296, SUC_lexicon_tags_14 },
    { 0x762b9297, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b929a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8b929d, SUC_lexicon_tags_103 },
    { 0x9d4b929d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98b92a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8b92a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b92aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84b92af, SUC_lexicon_tags_0 },
    { 0x47ab92b0, SUC_lexicon_tags_37 },
    { 0x696b92b0, SUC_lexicon_tags_12 },
    { 0x4acb92b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10eb92b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eeb92b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc16b92b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65eb92c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92c6, SUC_lexicon_tags_7 },
    { 0x5e6b92c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ab92ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b0b92d0, SUC_lexicon_tags_29 },
    { 0xca6b92d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb26b92d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x138b92d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x898b92d7, SUC_lexicon_tags_17 },
    { 0x456b92d7, SUC_lexicon_tags_17 },
    { 0x35ab92d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cb92df, SUC_lexicon_tags_32 },
    { 0x870b92e0, SUC_lexicon_tags_12 },
    { 0x8c2b92e0, SUC_lexicon_tags_27 },
    { 0xbdeb92e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b92e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f0b92e9, SUC_lexicon_tags_11 },
    { 0x3acb92ea, SUC_lexicon_tags_45 },
    { 0xd4ab92e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcb92f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842b92f5, SUC_lexicon_tags_17 },
    { 0xaa2b92f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x296b92f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860b92fc, SUC_lexicon_tags_1 },
    { 0x8b0b92fc, SUC_lexicon_tags_37 },
    { 0x2b0b92fd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8b9301, SUC_lexicon_tags_12 },
    { 0xb0ab9301, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x202b9305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eab9308, SUC_lexicon_tags_11 },
    { 0x804b9309, SUC_lexicon_tags_293 },
    { 0x9b2b9309, SUC_lexicon_tags_80 },
    { 0xeb0b9309, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccb9311, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eb9318, SUC_lexicon_tags_4 },
    { 0xb00b9319, SUC_lexicon_tags_17 },
    { 0xc14b931a, SUC_lexicon_tags_12 },
    { 0x362b9318, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45eb931f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab9324, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5eb9328, SUC_lexicon_tags_33 },
    { 0x31cb9329, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7dab932b, SUC_lexicon_tags_38 },
    { 0x9c4b932c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbcb932e, SUC_lexicon_tags_1 },
    { 0x932b932f, SUC_lexicon_tags_38 },
    { 0xb22b9330, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcb9333, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef6b9335, SUC_lexicon_tags_11 },
    { 0x824b9336, SUC_lexicon_tags_45 },
    { 0xe98b9336, SUC_lexicon_tags_17 },
    { 0x796b9338, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92cb933a, SUC_lexicon_tags_25 },
    { 0xd66b933b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdeb933d, SUC_lexicon_tags_1 },
    { 0x9b4b933e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x76ab9340, SUC_lexicon_tags_1 },
    { 0x266b9340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8b9345, SUC_lexicon_tags_15 },
    { 0xb28b9345, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x602b9349, SUC_lexicon_tags_19 },
    { 0x7e6b934a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4b934d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b9355, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bab9359, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x65cb935b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc20b935d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa32b935f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ab9362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6b9365, SUC_lexicon_tags_23 },
    { 0x8a8b9366, SUC_lexicon_tags_37 },
    { 0xd4cb9367, SUC_lexicon_tags_46 },
    { 0x7bab9367, SUC_lexicon_tags_38 },
    { 0xcf2b9368, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18ab936b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4b936f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x116b9371, SUC_lexicon_tags_70 },
    { 0x230b9371, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80b9375, SUC_lexicon_tags_15 },
    { 0x31eb9376, SUC_lexicon_tags_1 },
    { 0x2dab9376, SUC_lexicon_tags_24 },
    { 0xefeb9378, SUC_lexicon_tags_7 },
    { 0xc92b9378, SUC_lexicon_tags_17 },
    { 0x8b2b937a, SUC_lexicon_tags_12 },
    { 0x5c2b9379, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa26b937d, SUC_lexicon_tags_0 },
    { 0x85cb937d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6b9383, SUC_lexicon_tags_15 },
    { 0xd06b9384, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46b9387, SUC_lexicon_tags_37 },
    { 0x96ab9388, SUC_lexicon_tags_56 },
    { 0xd2ab9389, SUC_lexicon_tags_0 },
    { 0x1eab9387, SUC_lexicon_tags_60 },
    { 0x4eeb938b, SUC_lexicon_tags_14 },
    { 0x63cb938c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10b9390, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8b9394, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2ab9396, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x302b9399, SUC_lexicon_tags_25 },
    { 0x1e8b939a, SUC_lexicon_tags_10 },
    { 0x13eb939a, SUC_lexicon_tags_1 },
    { 0xcf4b9399, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x704b939e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ab93a2, SUC_lexicon_tags_7 },
    { 0x77cb93a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4b93a7, SUC_lexicon_tags_39 },
    { 0xcd4b93a7, SUC_lexicon_tags_29 },
    { 0x4dcb93a7, SUC_lexicon_tags_42 },
    { 0xb6b93aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a8b93ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eeb93b0, SUC_lexicon_tags_33 },
    { 0x64ab93b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54b93b4, SUC_lexicon_tags_45 },
    { 0xdbab93b5, SUC_lexicon_tags_7 },
    { 0x208b93b6, SUC_lexicon_tags_160 },
    { 0x3a0b93b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b2b93b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aeb93bd, SUC_lexicon_tags_25 },
    { 0x80eb93be, SUC_lexicon_tags_38 },
    { 0x176b93bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f6b93c1, SUC_lexicon_tags_38 },
    { 0x688b93c2, SUC_lexicon_tags_30 },
    { 0x632b93c1, SUC_lexicon_tags_27 },
    { 0xd8eb93c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36b93c7, SUC_lexicon_tags_14 },
    { 0x9d8b93c8, SUC_lexicon_tags_15 },
    { 0x3ab93c9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x494b93cc, SUC_lexicon_tags_63 },
    { 0xf6cb93cd, SUC_lexicon_tags_5 },
    { 0x22eb93cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae2b93d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952b93d5, SUC_lexicon_tags_59 },
    { 0x67cb93d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00b93da, SUC_lexicon_tags_25 },
    { 0x36eb93da, SUC_lexicon_tags_7 },
    { 0x7a0b93dc, SUC_lexicon_tags_55 },
    { 0x150b93dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cb93e1, SUC_lexicon_tags_11 },
    { 0x22ab93e2, SUC_lexicon_tags_11 },
    { 0x872b93e3, SUC_lexicon_tags_8 },
    { 0x516b93e3, SUC_lexicon_tags_70 },
    { 0x14ab93e5, SUC_lexicon_tags_25 },
    { 0xd66b93e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b93e9, SUC_lexicon_tags_39 },
    { 0x1d6b93e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeb93ed, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7cab93ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394b93f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x858b93f7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcb93fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154b9404, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5c2b9406, SUC_lexicon_tags_2 },
    { 0x52eb9407, SUC_lexicon_tags_10 },
    { 0xb8eb9408, SUC_lexicon_tags_7 },
    { 0x3c0b9408, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b940d, SUC_lexicon_tags_17 },
    { 0x124b940e, SUC_lexicon_tags_94 },
    { 0x13cb940f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd86b9411, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cb9415, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2a8b9417, SUC_lexicon_tags_11 },
    { 0x3f4b9418, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74eb941b, SUC_lexicon_tags_11 },
    { 0x426b941c, SUC_lexicon_tags_17 },
    { 0x2aeb941c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8cb941f, SUC_lexicon_tags_21 },
    { 0xf6eb9420, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x324b9422, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccb942c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0b942f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6b9432, SUC_lexicon_tags_1 },
    { 0xecab9433, SUC_lexicon_tags_15 },
    { 0x520b9433, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0b943b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a4b943d, SUC_lexicon_tags_7 },
    { 0xebab943e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836b9448, SUC_lexicon_tags_7 },
    { 0x65ab9449, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0b944d, SUC_lexicon_tags_48 },
    { 0x4ccb944d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b9456, SUC_lexicon_tags_6 },
    { 0x4acb9457, SUC_lexicon_tags_65 },
    { 0x9ab9457, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x970b945a, SUC_lexicon_tags_1 },
    { 0xea4b945b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6b945f, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0x8b6b9461, SUC_lexicon_tags_2 },
    { 0xeb8b9461, SUC_lexicon_tags_2 },
    { 0x21eb9461, SUC_lexicon_tags_44 },
    { 0xc0b9464, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28b9467, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326b946b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x658b946e, SUC_lexicon_tags_17 },
    { 0xc16b946f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x688b9471, SUC_lexicon_tags_29 },
    { 0xae0b9472, SUC_lexicon_tags_11 },
    { 0xd90b9472, SUC_lexicon_tags_33 },
    { 0xafeb9472, SUC_lexicon_tags_1 },
    { 0xc58b9475, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73ab9477, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556b947c, SUC_lexicon_tags_1 },
    { 0xf68b947d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454b9487, SUC_lexicon_tags_14 },
    { 0x7ecb9488, SUC_lexicon_tags_95 },
    { 0xc0ab9488, SUC_lexicon_tags_37 },
    { 0xb10b948a, SUC_lexicon_tags_61 },
    { 0xd7eb948b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x920b948d, SUC_lexicon_tags_17 },
    { 0x660b948e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccb9497, SUC_lexicon_tags_66 },
    { 0x22b9497, SUC_lexicon_tags_15 },
    { 0x52ab9498, SUC_lexicon_tags_60 },
    { 0xaeab949a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672b94a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99cb94a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cb94a6, SUC_lexicon_tags_2 },
    { 0xf8b94a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8b94aa, SUC_lexicon_tags_4 },
    { 0xf2b94aa, SUC_lexicon_tags_3 },
    { 0x894b94ab, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcb94b0, SUC_lexicon_tags_53 },
    { 0x38ab94b0, SUC_lexicon_tags_21 },
    { 0x4beb94b2, SUC_lexicon_tags_10 },
    { 0x7cb94b3, SUC_lexicon_tags_2 },
    { 0xef0b94b3, SUC_lexicon_tags_4 },
    { 0x606b94b4, SUC_lexicon_tags_65 },
    { 0xc2cb94b3, SUC_lexicon_tags_26 },
    { 0x6c2b94b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ab94be, SUC_lexicon_tags_19 },
    { 0xc42b94bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc98b94c1, SUC_lexicon_tags_0 },
    { 0x200b94c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ab94c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880b94cd, SUC_lexicon_tags_11 },
    { 0xa5eb94cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa34b94d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b6b94d2, SUC_lexicon_tags_11 },
    { 0xd06b94d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x788b94d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xad4b94d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8ab94d9, SUC_lexicon_tags_17 },
    { 0xea8b94da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x500b94dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x650b94de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92b94e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x900b94e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72b94e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02b94ec, SUC_lexicon_tags_28 },
    { 0xd06b94ec, SUC_lexicon_tags_61 },
    { 0x518b94ec, SUC_lexicon_tags_47 },
    { 0xb88b94ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x930b94f2, SUC_lexicon_tags_47 },
    { 0x920b94f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cb94f6, SUC_lexicon_tags_25 },
    { 0x656b94f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbdcb94f9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xca0b94fb, SUC_lexicon_tags_112 },
    { 0xfaab94fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa9ab94fe, SUC_lexicon_tags_485 },
    { 0xe90b94ff, SUC_lexicon_tags_9 },
    { 0x5d0b94ff, SUC_lexicon_tags_19 },
    { 0x59ab9501, SUC_lexicon_tags_15 },
    { 0x7d4b9500, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fab9505, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb48b9508, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x86b950a, SUC_lexicon_tags_37 },
    { 0x6aeb950a, SUC_lexicon_tags_66 },
    { 0x300b950c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7d6b950e, SUC_lexicon_tags_2 },
    { 0x9ccb950e, SUC_lexicon_tags_77 },
    { 0x802b9510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08b9513, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x704b9515, SUC_lexicon_tags_29 },
    { 0x700b9516, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ab951a, SUC_lexicon_tags_17 },
    { 0xa88b951b, SUC_lexicon_tags_30 },
    { 0x7c4b951c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b9524, SUC_lexicon_tags_12 },
    { 0x3d2b9525, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cb952b, SUC_lexicon_tags_27 },
    { 0x98ab952b, SUC_lexicon_tags_11 },
    { 0xf88b952d, SUC_lexicon_tags_37 },
    { 0x422b952e, SUC_lexicon_tags_10 },
    { 0x4c2b952f, SUC_lexicon_tags_14 },
    { 0x518b952d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52eb9533, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62b9536, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59ab9538, SUC_lexicon_tags_37 },
    { 0x7f8b9539, SUC_lexicon_tags_1 },
    { 0xc6ab9539, SUC_lexicon_tags_37 },
    { 0x8bcb953b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x308b953d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ab9540, SUC_lexicon_tags_29 },
    { 0xbb2b9540, SUC_lexicon_tags_17 },
    { 0xd18b9542, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8b9549, SUC_lexicon_tags_25 },
    { 0x398b9549, SUC_lexicon_tags_17 },
    { 0x7eab9549, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0b954e, SUC_lexicon_tags_25 },
    { 0xef4b954e, SUC_lexicon_tags_8 },
    { 0xa24b9550, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eb9555, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7feb9557, SUC_lexicon_tags_26 },
    { 0x9e6b9558, SUC_lexicon_tags_60 },
    { 0x1beb9559, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ab955b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8b955e, SUC_lexicon_tags_1 },
    { 0x80eb955e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb9567, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982b956c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24b956f, SUC_lexicon_tags_25 },
    { 0xacab956f, SUC_lexicon_tags_8 },
    { 0xfa4b9571, SUC_lexicon_tags_11 },
    { 0x9c6b9572, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acb9577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccb957a, SUC_lexicon_tags_4 },
    { 0xab8b957b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x466b957e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640b9585, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2b958a, SUC_lexicon_tags_39 },
    { 0xaf2b958b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5feb958e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7f2b9590, SUC_lexicon_tags_15 },
    { 0x4fcb9591, SUC_lexicon_tags_29 },
    { 0x144b9591, SUC_lexicon_tags_12 },
    { 0xdd2b9593, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x28b9595, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb44b9597, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84b959a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cb959e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd18b95a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb95a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x666b95a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0b95ad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xef8b95af, SUC_lexicon_tags_74 },
    { 0xd60b95b0, SUC_lexicon_tags_30 },
    { 0x294b95b1, SUC_lexicon_tags_68 },
    { 0xd48b95b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aeb95b6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b2b95b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x226b95bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf78b95bd, SUC_lexicon_tags_99 },
    { 0xd9eb95bd, SUC_lexicon_tags_1 },
    { 0x3feb95bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274b95c3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88b95c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38b95d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x19ab95d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0b95d6, SUC_lexicon_tags_2 },
    { 0xc14b95d7, SUC_lexicon_tags_0 },
    { 0xaf2b95d6, SUC_lexicon_tags_1 },
    { 0x170b95d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eab95dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86b95e1, SUC_lexicon_tags_25 },
    { 0x794b95e1, SUC_lexicon_tags_39 },
    { 0xdb6b95e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0eb95e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6dab95e8, SUC_lexicon_tags_14 },
    { 0x630b95e8, SUC_lexicon_tags_47 },
    { 0x4ab95e8, SUC_lexicon_tags_2 },
    { 0xdeeb95eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ab95f2, SUC_lexicon_tags_17 },
    { 0xaa8b95f3, SUC_lexicon_tags_19 },
    { 0x65eb95f4, SUC_lexicon_tags_11 },
    { 0x76cb95f5, SUC_lexicon_tags_37 },
    { 0x8c8b95f6, SUC_lexicon_tags_11 },
    { 0x63ab95f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b95fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x482b95fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6deb9601, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6b9604, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8b9608, SUC_lexicon_tags_7 },
    { 0xb64b9608, SUC_lexicon_tags_11 },
    { 0x68cb960a, SUC_lexicon_tags_11 },
    { 0x3b6b960b, SUC_lexicon_tags_11 },
    { 0xafab960c, SUC_lexicon_tags_124 },
    { 0x158b960a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ab9610, SUC_lexicon_tags_11 },
    { 0x2b9611, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xefeb9613, SUC_lexicon_tags_37 },
    { 0x730b9614, SUC_lexicon_tags_31 },
    { 0x1b6b9614, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcb9619, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776b961d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40b9624, SUC_lexicon_tags_37 },
    { 0xf0ab9624, SUC_lexicon_tags_14 },
    { 0xf90b9626, SUC_lexicon_tags_2 },
    { 0x5ceb9626, SUC_lexicon_tags_37 },
    { 0xdcab9628, SUC_lexicon_tags_33 },
    { 0x80eb9629, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b8b962b, SUC_lexicon_tags_29 },
    { 0xac4b962b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2b9639, SUC_lexicon_tags_27 },
    { 0x6d4b963a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf38b963c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x490b963e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8b9641, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b9645, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12b9649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6b964f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124b9653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24cb9655, SUC_lexicon_tags_19 },
    { 0x99eb9656, SUC_lexicon_tags_23 },
    { 0x1f6b9657, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ab965a, SUC_lexicon_tags_37 },
    { 0xa60b965a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe90b965d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2e8b965f, SUC_lexicon_tags_68 },
    { 0xff6b965f, SUC_lexicon_tags_3 },
    { 0x266b9661, SUC_lexicon_tags_35 },
    { 0xca0b9661, SUC_lexicon_tags_17 },
    { 0xdeb9663, SUC_lexicon_tags_14 },
    { 0xc8ab9661, SUC_lexicon_tags_68 },
    { 0xfb0b9665, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8b966a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cb966e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf64b9670, SUC_lexicon_tags_27 },
    { 0xaeeb9670, SUC_lexicon_tags_33 },
    { 0xa1eb9672, SUC_lexicon_tags_37 },
    { 0xaa2b9673, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x230b9675, SUC_lexicon_tags_4 },
    { 0x990b9676, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeb9679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6deb967b, SUC_lexicon_tags_0 },
    { 0x97cb967c, SUC_lexicon_tags_10 },
    { 0xeaeb967d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8b9681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36b9684, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cb968c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x330b968e, SUC_lexicon_tags_77 },
    { 0xda6b968f, SUC_lexicon_tags_27 },
    { 0x892b9690, SUC_lexicon_tags_33 },
    { 0x7e6b968e, SUC_lexicon_tags_107 },
    { 0xdf2b9692, SUC_lexicon_tags_17 },
    { 0x8d0b9693, SUC_lexicon_tags_18 },
    { 0x854b9693, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb14b9696, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcc4b9698, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998b969c, SUC_lexicon_tags_15 },
    { 0xf6b969d, SUC_lexicon_tags_49 },
    { 0xdecb969d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552b96a4, SUC_lexicon_tags_23 },
    { 0xe2cb96a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0b96a8, SUC_lexicon_tags_26 },
    { 0x7e6b96a8, SUC_lexicon_tags_47 },
    { 0xb72b96a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b96ae, SUC_lexicon_tags_47 },
    { 0xc56b96af, SUC_lexicon_tags_45 },
    { 0x49cb96b0, SUC_lexicon_tags_17 },
    { 0xb8ab96b0, SUC_lexicon_tags_87 },
    { 0x290b96b2, SUC_lexicon_tags_8 },
    { 0x842b96ae, SUC_lexicon_tags_60 },
    { 0xebcb96b4, SUC_lexicon_tags_37 },
    { 0xdb2b96b5, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x87cb96b8, SUC_lexicon_tags_45 },
    { 0x810b96b8, SUC_lexicon_tags_11 },
    { 0x66ab96ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aeb96bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x918b96bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x862b96c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a4b96c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98b96d3, SUC_lexicon_tags_31 },
    { 0x730b96d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2b96d7, SUC_lexicon_tags_37 },
    { 0xdacb96d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x89cb96da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b96de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8b96e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6b96e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4b96e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2b96e9, SUC_lexicon_tags_15 },
    { 0xa82b96ea, SUC_lexicon_tags_60 },
    { 0xf90b96eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874b96f1, SUC_lexicon_tags_37 },
    { 0x580b96f2, SUC_lexicon_tags_9 },
    { 0xac6b96f3, SUC_lexicon_tags_9 },
    { 0x4f6b96f2, SUC_lexicon_tags_15 },
    { 0xc5cb96f5, SUC_lexicon_tags_15 },
    { 0xc3eb96f5, SUC_lexicon_tags_17 },
    { 0x230b96f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x992b96f9, SUC_lexicon_tags_25 },
    { 0xf22b96fa, SUC_lexicon_tags_15 },
    { 0xb6cb96f9, SUC_lexicon_tags_25 },
    { 0xc0eb96fa, SUC_lexicon_tags_0 },
    { 0xe20b96fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0b9700, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ab9705, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb9708, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71eb970a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x798b970d, SUC_lexicon_tags_49 },
    { 0x4deb970d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50b9713, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1aeb9715, SUC_lexicon_tags_15 },
    { 0xdbcb9716, SUC_lexicon_tags_7 },
    { 0xfe0b9716, SUC_lexicon_tags_237 },
    { 0x80ab9716, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3cab971a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64b971e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe70b9720, SUC_lexicon_tags_12 },
    { 0x2f2b9721, SUC_lexicon_tags_28 },
    { 0xec8b9722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56ab9724, SUC_lexicon_tags_31 },
    { 0xfe4b9725, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x204b9727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64cb9729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38eb972f, SUC_lexicon_tags_21 },
    { 0x5f2b9730, SUC_lexicon_tags_16 },
    { 0x6b6b9731, SUC_lexicon_tags_8 },
    { 0x212b9732, SUC_lexicon_tags_60 },
    { 0xd66b9732, SUC_lexicon_tags_37 },
    { 0x828b9734, SUC_lexicon_tags_70 },
    { 0x1bcb972f, SUC_lexicon_tags_13 },
    { 0xad4b9736, SUC_lexicon_tags_38 },
    { 0x268b9732, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x440b9739, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6c2b973b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51cb973e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc28b9740, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f0b9742, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x812b9745, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa2cb9747, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4b974f, SUC_lexicon_tags_17 },
    { 0x19cb974f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x428b9752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe66b9754, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89cb9756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27eb9758, SUC_lexicon_tags_0 },
    { 0xc28b9759, SUC_lexicon_tags_2 },
    { 0xafeb9759, SUC_lexicon_tags_31 },
    { 0x1beb975b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xae0b975d, SUC_lexicon_tags_37 },
    { 0x340b975e, SUC_lexicon_tags_19 },
    { 0x7ecb975f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcb9768, SUC_lexicon_tags_25 },
    { 0x4fab9768, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x790b976b, SUC_lexicon_tags_1 },
    { 0xbe2b976c, SUC_lexicon_tags_1 },
    { 0x34eb976d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6b9772, SUC_lexicon_tags_25 },
    { 0x630b9773, SUC_lexicon_tags_38 },
    { 0xfc2b9774, SUC_lexicon_tags_33 },
    { 0xd32b9774, SUC_lexicon_tags_60 },
    { 0x7c0b9774, SUC_lexicon_tags_46 },
    { 0x936b9775, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8b9781, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf00b9783, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x538b9786, SUC_lexicon_tags_92 },
    { 0x68ab9786, SUC_lexicon_tags_16 },
    { 0x7b2b9788, SUC_lexicon_tags_15 },
    { 0xa30b9789, SUC_lexicon_tags_7 },
    { 0xf32b9788, SUC_lexicon_tags_47 },
    { 0xe16b9788, SUC_lexicon_tags_25 },
    { 0xdf0b978a, SUC_lexicon_tags_61 },
    { 0x526b978d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x484b9790, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90ab9792, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16b9796, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cb979b, SUC_lexicon_tags_37 },
    { 0xcbab979c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed6b979e, SUC_lexicon_tags_11 },
    { 0xbecb979f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302b97a5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cab97aa, SUC_lexicon_tags_35 },
    { 0xe9ab97ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeb97ae, SUC_lexicon_tags_17 },
    { 0x9ccb97ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dab97b2, SUC_lexicon_tags_12 },
    { 0xdbeb97b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57eb97b7, SUC_lexicon_tags_48 },
    { 0x5e4b97b8, SUC_lexicon_tags_14 },
    { 0xde0b97b9, SUC_lexicon_tags_15 },
    { 0x27cb97b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2eb97bd, SUC_lexicon_tags_11 },
    { 0xc12b97bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152b97c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6b97c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb97cd, SUC_lexicon_tags_37 },
    { 0xdfcb97ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48b97d2, SUC_lexicon_tags_37 },
    { 0x768b97d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778b97d9, SUC_lexicon_tags_45 },
    { 0x92b97da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0b97de, SUC_lexicon_tags_32 },
    { 0x19eb97de, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28eb97e4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4eab97e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bab97e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6b97f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafab97f6, SUC_lexicon_tags_33 },
    { 0x832b97f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ab97fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9ab97fc, SUC_lexicon_tags_37 },
    { 0x67ab97fd, SUC_lexicon_tags_21 },
    { 0xaf6b97fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0b9806, SUC_lexicon_tags_2 },
    { 0x7e6b9807, SUC_lexicon_tags_49 },
    { 0x386b9807, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ab980f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x378b9811, SUC_lexicon_tags_13 },
    { 0x1dab9811, SUC_lexicon_tags_17 },
    { 0xc24b9813, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86b9817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x384b9819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca4b981b, SUC_lexicon_tags_60 },
    { 0xacab981c, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8b9820, SUC_lexicon_tags_37 },
    { 0xf32b9821, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4b9824, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0b9827, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x94cb9829, SUC_lexicon_tags_2 },
    { 0x13eb982a, SUC_lexicon_tags_5 },
    { 0x6deb982a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ab982e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc88b9830, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cb9833, SUC_lexicon_tags_23 },
    { 0xf68b9834, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb86b9836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0b983c, SUC_lexicon_tags_25 },
    { 0xe4ab983d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8aab983f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74b9844, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8b984b, SUC_lexicon_tags_15 },
    { 0x34b984c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92b9850, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeb9858, SUC_lexicon_tags_0 },
    { 0x600b9859, SUC_lexicon_tags_106 },
    { 0xfb8b985a, SUC_lexicon_tags_7 },
    { 0xcdab9859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b9860, SUC_lexicon_tags_49 },
    { 0xf16b9861, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76b9864, SUC_lexicon_tags_37 },
    { 0x67eb9865, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x910b9868, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b8b986a, SUC_lexicon_tags_77 },
    { 0xd86b986a, SUC_lexicon_tags_92 },
    { 0xdd6b986b, SUC_lexicon_tags_27 },
    { 0xa36b986d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b9872, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ab9875, SUC_lexicon_tags_1 },
    { 0x3acb9876, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810b987e, SUC_lexicon_tags_5 },
    { 0x4e2b987f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36b9881, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4b9884, SUC_lexicon_tags_23 },
    { 0x1c2b9885, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2fab9887, SUC_lexicon_tags_38 },
    { 0xf02b9888, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2b988c, SUC_lexicon_tags_37 },
    { 0x9feb988c, SUC_lexicon_tags_11 },
    { 0xd1eb988e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x502b9891, SUC_lexicon_tags_43 },
    { 0x4deb9892, SUC_lexicon_tags_36 },
    { 0x4a2b9891, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd0b9895, SUC_lexicon_tags_8 },
    { 0x17eb9896, SUC_lexicon_tags_0 },
    { 0xfd8b9895, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dab989a, SUC_lexicon_tags_10 },
    { 0x81eb989b, SUC_lexicon_tags_25 },
    { 0xf9cb989c, SUC_lexicon_tags_5 },
    { 0x2dcb989a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52eb98a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e8b98a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42b98a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aab98aa, SUC_lexicon_tags_65 },
    { 0x52b98aa, SUC_lexicon_tags_14 },
    { 0x9e4b98ab, SUC_lexicon_tags_17 },
    { 0xd82b98ad, SUC_lexicon_tags_33 },
    { 0x858b98ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x930b98b0, SUC_lexicon_tags_15 },
    { 0x78eb98b1, SUC_lexicon_tags_87 },
    { 0x250b98b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcacb98b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf02b98b6, SUC_lexicon_tags_16 },
    { 0x774b98b6, SUC_lexicon_tags_17 },
    { 0x1e6b98b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cb98be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22b98c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6b98c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8b98cb, SUC_lexicon_tags_25 },
    { 0x71eb98cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb98cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938b98d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6cb98d5, SUC_lexicon_tags_6 },
    { 0xa00b98d5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ab98d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58ab98db, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44b98df, SUC_lexicon_tags_37 },
    { 0x8f2b98e0, SUC_lexicon_tags_19 },
    { 0xfe2b98e1, SUC_lexicon_tags_47 },
    { 0x2eeb98df, SUC_lexicon_tags_2 },
    { 0x66ab98df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eb98e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfceb98ed, SUC_lexicon_tags_11 },
    { 0x97ab98ee, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6b98f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcb98f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794b98fc, SUC_lexicon_tags_25 },
    { 0xfd2b98fd, SUC_lexicon_tags_7 },
    { 0x834b98fe, SUC_lexicon_tags_31 },
    { 0xaa4b98ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dab9904, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0b9908, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4b990d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfaab990f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ceb9912, SUC_lexicon_tags_65 },
    { 0xf28b9913, SUC_lexicon_tags_207 },
    { 0x3cab9913, SUC_lexicon_tags_56 },
    { 0x9f2b9914, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x380b9918, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284b991c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6b991f, SUC_lexicon_tags_26 },
    { 0x606b991f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x896b9922, SUC_lexicon_tags_65 },
    { 0x28ab9922, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x75cb9925, SUC_lexicon_tags_4 },
    { 0xddab9926, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34cb9928, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ecb992a, SUC_lexicon_tags_27 },
    { 0xc54b992a, SUC_lexicon_tags_38 },
    { 0x334b992a, SUC_lexicon_tags_15 },
    { 0x940b992d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab9933, SUC_lexicon_tags_61 },
    { 0x524b9934, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7beb9936, SUC_lexicon_tags_39 },
    { 0x850b9936, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766b993c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ab9941, SUC_lexicon_tags_1 },
    { 0x9d8b9941, SUC_lexicon_tags_27 },
    { 0x5e8b9942, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00b994b, SUC_lexicon_tags_25 },
    { 0x4deb994c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0b9950, SUC_lexicon_tags_26 },
    { 0x706b9951, SUC_lexicon_tags_17 },
    { 0x424b9951, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168b9956, SUC_lexicon_tags_47 },
    { 0xffeb9957, SUC_lexicon_tags_26 },
    { 0x220b9958, SUC_lexicon_tags_66 },
    { 0x54b9959, SUC_lexicon_tags_16 },
    { 0x91eb995a, SUC_lexicon_tags_25 },
    { 0x100b9959, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf5ab995d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638b9962, SUC_lexicon_tags_45 },
    { 0xd10b9963, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cb9967, SUC_lexicon_tags_31 },
    { 0xd6ab9968, SUC_lexicon_tags_7 },
    { 0xe36b9969, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4b996c, SUC_lexicon_tags_7 },
    { 0x94ab996d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2b9970, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4b9976, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56b997e, SUC_lexicon_tags_28 },
    { 0x1d8b997f, SUC_lexicon_tags_7 },
    { 0x49cb997f, SUC_lexicon_tags_4 },
    { 0xa30b9981, SUC_lexicon_tags_26 },
    { 0x9d4b9982, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650b9987, SUC_lexicon_tags_17 },
    { 0x512b9987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b998f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fab9994, SUC_lexicon_tags_27 },
    { 0xa80b9995, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554b999f, SUC_lexicon_tags_14 },
    { 0xea8b99a0, SUC_lexicon_tags_29 },
    { 0x18b99a1, SUC_lexicon_tags_66 },
    { 0x978b99a2, SUC_lexicon_tags_16 },
    { 0x1fcb99a2, SUC_lexicon_tags_7 },
    { 0x51cb99a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16b99a8, SUC_lexicon_tags_12 },
    { 0x5ccb99a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x66cb99ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2b99af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bab99b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8b99b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45eb99bc, SUC_lexicon_tags_9 },
    { 0x5c0b99bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x874b99bf, SUC_lexicon_tags_28 },
    { 0xd90b99c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814b99c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2b99d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2b99d5, SUC_lexicon_tags_11 },
    { 0xf6cb99d6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8b99e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6b99e3, SUC_lexicon_tags_15 },
    { 0x7feb99e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8cb99e6, SUC_lexicon_tags_12 },
    { 0xe7ab99e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x76eb99ea, SUC_lexicon_tags_25 },
    { 0xaceb99eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x498b99ed, SUC_lexicon_tags_68 },
    { 0x940b99ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2b99f1, SUC_lexicon_tags_1 },
    { 0x98cb99f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f6b99f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x612b99f6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceb99f9, SUC_lexicon_tags_11 },
    { 0xdc4b99fa, SUC_lexicon_tags_27 },
    { 0x806b99fa, SUC_lexicon_tags_15 },
    { 0xd30b99fb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbab99fe, SUC_lexicon_tags_79 },
    { 0x7a2b99fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x296b9a01, SUC_lexicon_tags_15 },
    { 0x292b9a02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c8b9a04, SUC_lexicon_tags_129 },
    { 0x1a2b9a04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ab9a0a, SUC_lexicon_tags_60 },
    { 0xfe4b9a0b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63eb9a10, SUC_lexicon_tags_25 },
    { 0x69eb9a10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28b9a14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2eb9a16, SUC_lexicon_tags_38 },
    { 0xf82b9a17, SUC_lexicon_tags_1 },
    { 0x58eb9a18, SUC_lexicon_tags_37 },
    { 0xcceb9a19, SUC_lexicon_tags_12 },
    { 0xcd8b9a19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x458b9a1c, SUC_lexicon_tags_15 },
    { 0xa7eb9a1c, SUC_lexicon_tags_33 },
    { 0x906b9a1e, SUC_lexicon_tags_12 },
    { 0xf32b9a1f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x4b0b9a21, SUC_lexicon_tags_194 },
    { 0x110b9a22, SUC_lexicon_tags_12 },
    { 0x780b9a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eb9a28, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ab9a2d, SUC_lexicon_tags_7 },
    { 0xedab9a2e, SUC_lexicon_tags_12 },
    { 0xe0cb9a2f, SUC_lexicon_tags_1 },
    { 0x316b9a30, SUC_lexicon_tags_2 },
    { 0x218b9a31, SUC_lexicon_tags_37 },
    { 0xb8ab9a2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfaab9a34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf74b9a36, SUC_lexicon_tags_47 },
    { 0xf70b9a37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8b9a3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3beb9a3d, SUC_lexicon_tags_17 },
    { 0xe02b9a3e, SUC_lexicon_tags_7 },
    { 0x9aeb9a3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6b9a42, SUC_lexicon_tags_140 },
    { 0xd2eb9a42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x226b9a45, SUC_lexicon_tags_31 },
    { 0x9b0b9a46, SUC_lexicon_tags_65 },
    { 0x1bcb9a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80b9a4c, SUC_lexicon_tags_1 },
    { 0x6ecb9a4d, SUC_lexicon_tags_11 },
    { 0x28eb9a4c, SUC_lexicon_tags_14 },
    { 0xe46b9a4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88b9a53, SUC_lexicon_tags_7 },
    { 0xa36b9a54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cb9a59, SUC_lexicon_tags_11 },
    { 0xa3eb9a5a, SUC_lexicon_tags_11 },
    { 0x3f0b9a5a, SUC_lexicon_tags_21 },
    { 0x94eb9a5c, SUC_lexicon_tags_26 },
    { 0x726b9a59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72cb9a60, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40b9a65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2a4b9a67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x678b9a69, SUC_lexicon_tags_11 },
    { 0xea8b9a6a, SUC_lexicon_tags_87 },
    { 0xf30b9a69, SUC_lexicon_tags_1 },
    { 0xd2eb9a69, SUC_lexicon_tags_25 },
    { 0xd0eb9a6d, SUC_lexicon_tags_28 },
    { 0xebcb9a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8b9a71, SUC_lexicon_tags_26 },
    { 0xaab9a72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee8b9a74, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cab9a77, SUC_lexicon_tags_12 },
    { 0x63cb9a78, SUC_lexicon_tags_7 },
    { 0xaacb9a77, SUC_lexicon_tags_25 },
    { 0x91cb9a7a, SUC_lexicon_tags_63 },
    { 0xa72b9a7b, SUC_lexicon_tags_48 },
    { 0x3deb9a7c, SUC_lexicon_tags_47 },
    { 0xb32b9a7b, SUC_lexicon_tags_33 },
    { 0xf5ab9a7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aeb9a84, SUC_lexicon_tags_7 },
    { 0x1b8b9a84, SUC_lexicon_tags_138 },
    { 0x596b9a86, SUC_lexicon_tags_11 },
    { 0x9d0b9a86, SUC_lexicon_tags_7 },
    { 0x8f4b9a86, SUC_lexicon_tags_32 },
    { 0xf5cb9a89, SUC_lexicon_tags_85 },
    { 0x9f4b9a8a, SUC_lexicon_tags_37 },
    { 0xf9cb9a8b, SUC_lexicon_tags_23 },
    { 0x1e8b9a86, SUC_lexicon_tags_31 },
    { 0xb5cb9a8d, SUC_lexicon_tags_12 },
    { 0x352b9a8b, SUC_lexicon_tags_8 },
    { 0x352b9a8f, SUC_lexicon_tags_27 },
    { 0x84b9a8f, SUC_lexicon_tags_1 },
    { 0x50ab9a90, SUC_lexicon_tags_37 },
    { 0xcf4b9a8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6ab9a94, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cb9a97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcb9a9b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cb9a9f, SUC_lexicon_tags_12 },
    { 0x380b9aa0, SUC_lexicon_tags_45 },
    { 0xec2b9aa1, SUC_lexicon_tags_63 },
    { 0x168b9aa2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x232b9aa5, SUC_lexicon_tags_30 },
    { 0xecb9aa6, SUC_lexicon_tags_47 },
    { 0x48eb9aa7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x71eb9aa9, SUC_lexicon_tags_25 },
    { 0xd24b9aaa, SUC_lexicon_tags_5 },
    { 0x8fcb9aab, SUC_lexicon_tags_1 },
    { 0xdc8b9aac, SUC_lexicon_tags_38 },
    { 0xb48b9aac, SUC_lexicon_tags_12 },
    { 0x440b9aad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426b9ab3, SUC_lexicon_tags_46 },
    { 0x23cb9ab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dab9ab7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbbcb9ab9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb9eb9abb, SUC_lexicon_tags_25 },
    { 0x876b9abc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf42b9abe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88b9ac6, SUC_lexicon_tags_31 },
    { 0xf7eb9ac7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4b9acc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44b9ad0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcb9ad2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbab9ad5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52b9ada, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x548b9adc, SUC_lexicon_tags_25 },
    { 0xfe4b9add, SUC_lexicon_tags_31 },
    { 0x200b9ade, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232b9ae4, SUC_lexicon_tags_15 },
    { 0x1c0b9ae5, SUC_lexicon_tags_82 },
    { 0x670b9ae4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65eb9aec, SUC_lexicon_tags_25 },
    { 0x2a8b9aed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x872b9aef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfab9af7, SUC_lexicon_tags_37 },
    { 0xea8b9af8, SUC_lexicon_tags_25 },
    { 0xdaab9af8, SUC_lexicon_tags_1 },
    { 0xb6cb9afa, SUC_lexicon_tags_23 },
    { 0x62eb9af8, SUC_lexicon_tags_37 },
    { 0xbdcb9afc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0b9aff, SUC_lexicon_tags_1 },
    { 0x1dab9b00, SUC_lexicon_tags_60 },
    { 0x6a8b9aff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x988b9b03, SUC_lexicon_tags_27 },
    { 0x132b9b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x100b9b06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bab9b08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d0b9b0a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa58b9b0c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc00b9b0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4ab9b10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb9b15, SUC_lexicon_tags_14 },
    { 0x416b9b16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1dcb9b18, SUC_lexicon_tags_23 },
    { 0x43eb9b19, SUC_lexicon_tags_47 },
    { 0xc10b9b1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6ab9b1c, SUC_lexicon_tags_45 },
    { 0xa44b9b1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0b9b25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa7eb9b27, SUC_lexicon_tags_1 },
    { 0xafeb9b28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526b9b2d, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x740b9b2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd68b9b31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd22b9b33, SUC_lexicon_tags_2 },
    { 0x72ab9b33, SUC_lexicon_tags_7 },
    { 0xee4b9b35, SUC_lexicon_tags_65 },
    { 0x58eb9b36, SUC_lexicon_tags_37 },
    { 0xb7eb9b37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74b9b3a, SUC_lexicon_tags_55 },
    { 0x578b9b3a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acb9b3f, SUC_lexicon_tags_23 },
    { 0x29ab9b40, SUC_lexicon_tags_96 },
    { 0x760b9b40, SUC_lexicon_tags_25 },
    { 0x732b9b42, SUC_lexicon_tags_9 },
    { 0xd4cb9b43, SUC_lexicon_tags_29 },
    { 0x64ab9b40, SUC_lexicon_tags_11 },
    { 0x9eb9b45, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340b9b49, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x59eb9b4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710b9b52, SUC_lexicon_tags_37 },
    { 0x372b9b53, SUC_lexicon_tags_27 },
    { 0xb1eb9b54, SUC_lexicon_tags_7 },
    { 0x8e4b9b55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x156b9b57, SUC_lexicon_tags_33 },
    { 0xdd2b9b58, SUC_lexicon_tags_31 },
    { 0x1eeb9b58, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00b9b5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4b9b62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4b9b6b, SUC_lexicon_tags_46 },
    { 0x616b9b6c, SUC_lexicon_tags_1 },
    { 0xaf4b9b6c, SUC_lexicon_tags_29 },
    { 0x9dab9b6c, SUC_lexicon_tags_38 },
    { 0x95ab9b6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2b9b72, SUC_lexicon_tags_49 },
    { 0x34ab9b72, SUC_lexicon_tags_16 },
    { 0xfc4b9b74, SUC_lexicon_tags_17 },
    { 0xd80b9b72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8b9b78, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x5deb9b7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb9b7e, SUC_lexicon_tags_37 },
    { 0x918b9b7f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16b9b86, SUC_lexicon_tags_25 },
    { 0xd8eb9b87, SUC_lexicon_tags_37 },
    { 0xbfeb9b87, SUC_lexicon_tags_19 },
    { 0x3e6b9b89, SUC_lexicon_tags_37 },
    { 0xb20b9b89, SUC_lexicon_tags_0 },
    { 0xe78b9b89, SUC_lexicon_tags_49 },
    { 0x946b9b86, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6b9b8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a6b9b91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x322b9b93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2b9b96, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cb9b99, SUC_lexicon_tags_11 },
    { 0x80b9b99, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc28b9b9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x656b9b9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x980b9ba1, SUC_lexicon_tags_12 },
    { 0x7fcb9ba2, SUC_lexicon_tags_91 },
    { 0x5d4b9ba2, SUC_lexicon_tags_65 },
    { 0xaab9ba4, SUC_lexicon_tags_0 },
    { 0x72b9ba4, SUC_lexicon_tags_1 },
    { 0xe98b9ba6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882b9bab, SUC_lexicon_tags_35 },
    { 0x368b9bac, SUC_lexicon_tags_47 },
    { 0x2c2b9bad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a2b9baf, SUC_lexicon_tags_12 },
    { 0x5cab9bb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eb9bb8, SUC_lexicon_tags_6 },
    { 0x97ab9bb9, SUC_lexicon_tags_37 },
    { 0xe4ab9bba, SUC_lexicon_tags_1 },
    { 0x8e4b9bba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd2eb9bbd, SUC_lexicon_tags_23 },
    { 0x6a0b9bbe, SUC_lexicon_tags_17 },
    { 0x4b9bbe, SUC_lexicon_tags_7 },
    { 0x3b8b9bbe, SUC_lexicon_tags_11 },
    { 0x8eab9bc1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x724b9bc3, SUC_lexicon_tags_37 },
    { 0x5d0b9bc3, SUC_lexicon_tags_31 },
    { 0x7fcb9bc5, SUC_lexicon_tags_37 },
    { 0xa68b9bc5, SUC_lexicon_tags_7 },
    { 0xe8b9bc7, SUC_lexicon_tags_2 },
    { 0xecab9bc7, SUC_lexicon_tags_17 },
    { 0xd62b9bc4, SUC_lexicon_tags_21 },
    { 0x434b9bca, SUC_lexicon_tags_65 },
    { 0x66eb9bc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2deb9bce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xadab9bd1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8b9bd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eeb9bdc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894b9be4, SUC_lexicon_tags_31 },
    { 0x9cab9be5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4b9beb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1eb9bee, SUC_lexicon_tags_12 },
    { 0xd2eb9bef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436b9bf3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54b9bfb, SUC_lexicon_tags_77 },
    { 0x67ab9bfc, SUC_lexicon_tags_7 },
    { 0x25cb9bfb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9eb9c01, SUC_lexicon_tags_79 },
    { 0x8f2b9c01, SUC_lexicon_tags_55 },
    { 0x3a8b9c01, SUC_lexicon_tags_40 },
    { 0xe2b9c02, SUC_lexicon_tags_8 },
    { 0x4cab9c05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd00b9c07, SUC_lexicon_tags_1 },
    { 0x47eb9c07, SUC_lexicon_tags_35 },
    { 0x312b9c08, SUC_lexicon_tags_15 },
    { 0xc60b9c09, SUC_lexicon_tags_19 },
    { 0x586b9c0b, SUC_lexicon_tags_37 },
    { 0x44cb9c0b, SUC_lexicon_tags_11 },
    { 0x354b9c0d, SUC_lexicon_tags_7 },
    { 0xbb8b9c0e, SUC_lexicon_tags_12 },
    { 0x344b9c0f, SUC_lexicon_tags_2 },
    { 0x35eb9c10, SUC_lexicon_tags_296 },
    { 0xc26b9c11, SUC_lexicon_tags_29 },
    { 0xee2b9c10, SUC_lexicon_tags_4 },
    { 0x532b9c13, SUC_lexicon_tags_1 },
    { 0x89cb9c13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacb9c19, SUC_lexicon_tags_33 },
    { 0xab4b9c1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8b9c22, SUC_lexicon_tags_60 },
    { 0x7b6b9c23, SUC_lexicon_tags_15 },
    { 0x66b9c24, SUC_lexicon_tags_25 },
    { 0xaecb9c25, SUC_lexicon_tags_28 },
    { 0x946b9c22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06b9c2b, SUC_lexicon_tags_47 },
    { 0x3bcb9c2b, SUC_lexicon_tags_17 },
    { 0x5ceb9c2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ab9c30, SUC_lexicon_tags_12 },
    { 0x6a0b9c30, SUC_lexicon_tags_13 },
    { 0x5c2b9c32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140b9c36, SUC_lexicon_tags_2 },
    { 0x15ab9c37, SUC_lexicon_tags_17 },
    { 0x2cab9c36, SUC_lexicon_tags_25 },
    { 0x8ceb9c39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fab9c45, SUC_lexicon_tags_17 },
    { 0xf16b9c46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x644b9c48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedeb9c4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x136b9c4f, SUC_lexicon_tags_37 },
    { 0x76cb9c50, SUC_lexicon_tags_23 },
    { 0x498b9c50, SUC_lexicon_tags_19 },
    { 0x2b8b9c4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcb9c65, SUC_lexicon_tags_27 },
    { 0x30eb9c66, SUC_lexicon_tags_1 },
    { 0xa6eb9c67, SUC_lexicon_tags_65 },
    { 0x39cb9c68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1cab9c6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb52b9c6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaab9c71, SUC_lexicon_tags_14 },
    { 0xcd0b9c71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6b9c77, SUC_lexicon_tags_38 },
    { 0xf26b9c78, SUC_lexicon_tags_2 },
    { 0x532b9c79, SUC_lexicon_tags_133 },
    { 0xbc0b9c77, SUC_lexicon_tags_22 },
    { 0xf26b9c7b, SUC_lexicon_tags_34 },
    { 0xeceb9c7c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x3e2b9c7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecb9c82, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x942b9c84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x674b9c87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68eb9c8b, SUC_lexicon_tags_17 },
    { 0x66eb9c8c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1d2b9c8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd46b9c90, SUC_lexicon_tags_49 },
    { 0x402b9c91, SUC_lexicon_tags_47 },
    { 0x64b9c91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2b9c97, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cb9c9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x656b9c9e, SUC_lexicon_tags_63 },
    { 0x146b9c9f, SUC_lexicon_tags_8 },
    { 0x8fcb9ca0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2deb9ca7, SUC_lexicon_tags_11 },
    { 0xef6b9ca7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74b9cab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4b9cb3, SUC_lexicon_tags_481 },
    { 0x9f8b9cb4, SUC_lexicon_tags_30 },
    { 0x55eb9cb5, SUC_lexicon_tags_25 },
    { 0xe9ab9cb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88b9cb9, SUC_lexicon_tags_27 },
    { 0xea4b9cba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa78b9cbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x814b9cbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cb9cc6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbfcb9cc8, SUC_lexicon_tags_31 },
    { 0xda0b9cc9, SUC_lexicon_tags_2 },
    { 0xf8ab9cca, SUC_lexicon_tags_107 },
    { 0xa78b9ccb, SUC_lexicon_tags_30 },
    { 0xd3cb9ccc, SUC_lexicon_tags_15 },
    { 0x464b9ccd, SUC_lexicon_tags_2 },
    { 0x964b9ccd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836b9cd5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b9cd8, SUC_lexicon_tags_41 },
    { 0xb5eb9cd9, SUC_lexicon_tags_1 },
    { 0xfa6b9cd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfacb9cdd, SUC_lexicon_tags_11 },
    { 0xdeab9cde, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ab9ce3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6b9ce9, SUC_lexicon_tags_1 },
    { 0xfb6b9cea, SUC_lexicon_tags_6 },
    { 0x110b9ceb, SUC_lexicon_tags_47 },
    { 0xeeab9cec, SUC_lexicon_tags_7 },
    { 0x79ab9ced, SUC_lexicon_tags_92 },
    { 0xe2cb9cec, SUC_lexicon_tags_2 },
    { 0x35cb9cef, SUC_lexicon_tags_27 },
    { 0xd42b9cec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfab9cf2, SUC_lexicon_tags_114 },
    { 0xf06b9cf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cb9cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcb9cfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa06b9cfc, SUC_lexicon_tags_9 },
    { 0xc62b9cfc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ab9d00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888b9d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x428b9d07, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986b9d0c, SUC_lexicon_tags_1 },
    { 0x20cb9d0d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94b9d10, SUC_lexicon_tags_11 },
    { 0xde4b9d10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x998b9d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4b9d17, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40b9d1b, SUC_lexicon_tags_45 },
    { 0xa9eb9d1c, SUC_lexicon_tags_39 },
    { 0xff6b9d1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6b9d20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cb9d23, SUC_lexicon_tags_1 },
    { 0x8f8b9d23, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cb9d28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c6b9d2a, SUC_lexicon_tags_15 },
    { 0xb84b9d2b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcb9d2e, SUC_lexicon_tags_37 },
    { 0x9fab9d2f, SUC_lexicon_tags_6 },
    { 0x618b9d30, SUC_lexicon_tags_15 },
    { 0xf48b9d2e, SUC_lexicon_tags_28 },
    { 0x12cb9d32, SUC_lexicon_tags_33 },
    { 0x2e6b9d32, SUC_lexicon_tags_25 },
    { 0x2acb9d34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebcb9d36, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2b9d3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acb9d40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfab9d43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cb9d4b, SUC_lexicon_tags_25 },
    { 0x990b9d4c, SUC_lexicon_tags_45 },
    { 0x836b9d4d, SUC_lexicon_tags_63 },
    { 0x896b9d4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe16b9d50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0b9d57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194b9d5c, SUC_lexicon_tags_3 },
    { 0x974b9d5d, SUC_lexicon_tags_43 },
    { 0xe78b9d5d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbbeb9d60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x16ab9d62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17eb9d64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x402b9d67, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd4b9d69, SUC_lexicon_tags_11 },
    { 0x21ab9d69, SUC_lexicon_tags_95 },
    { 0x91ab9d6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x804b9d6e, SUC_lexicon_tags_11 },
    { 0x6c4b9d6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3cb9d71, SUC_lexicon_tags_37 },
    { 0x630b9d72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2b9d75, SUC_lexicon_tags_12 },
    { 0x9fab9d76, SUC_lexicon_tags_13 },
    { 0xb5cb9d76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfa2b9d79, SUC_lexicon_tags_6 },
    { 0x422b9d79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386b9d7f, SUC_lexicon_tags_26 },
    { 0x8a4b9d80, SUC_lexicon_tags_1 },
    { 0xeaeb9d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecb9d86, SUC_lexicon_tags_25 },
    { 0xd28b9d87, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ab9d8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x652b9d8e, SUC_lexicon_tags_70 },
    { 0xd2ab9d8f, SUC_lexicon_tags_21 },
    { 0xf14b9d90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ab9d93, SUC_lexicon_tags_47 },
    { 0x466b9d94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x53ab9d96, SUC_lexicon_tags_63 },
    { 0x674b9d96, SUC_lexicon_tags_17 },
    { 0xd22b9d98, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eb9d9b, SUC_lexicon_tags_6 },
    { 0xe48b9d9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84b9d9f, SUC_lexicon_tags_30 },
    { 0x954b9d9f, SUC_lexicon_tags_45 },
    { 0xa22b9da1, SUC_lexicon_tags_37 },
    { 0xbbab9da1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8b9da5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcb9da9, SUC_lexicon_tags_17 },
    { 0xafcb9da9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cb9db0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x352b9db3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ab9dbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x100b9dc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc5ab9dc2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfdab9dc4, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0x63ab9dc6, SUC_lexicon_tags_15 },
    { 0x9deb9dc7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00b9dca, SUC_lexicon_tags_12 },
    { 0xe00b9dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeb9dce, SUC_lexicon_tags_84 },
    { 0x44eb9dcf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf12b9dd1, SUC_lexicon_tags_12 },
    { 0x6e4b9dd2, SUC_lexicon_tags_12 },
    { 0xd00b9dd1, SUC_lexicon_tags_1 },
    { 0xe82b9dd4, SUC_lexicon_tags_55 },
    { 0x70cb9dd5, SUC_lexicon_tags_6 },
    { 0x782b9dd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eb9dd9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x678b9ddb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216b9ddf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cb9de3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cab9de7, SUC_lexicon_tags_14 },
    { 0xae8b9de7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466b9ded, SUC_lexicon_tags_8 },
    { 0x698b9ded, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x91eb9df0, SUC_lexicon_tags_47 },
    { 0x720b9df1, SUC_lexicon_tags_17 },
    { 0x7b6b9df1, SUC_lexicon_tags_37 },
    { 0xfd6b9df1, SUC_lexicon_tags_38 },
    { 0x2e4b9df3, SUC_lexicon_tags_29 },
    { 0x24eb9df0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bab9df8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938b9e01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90b9e06, SUC_lexicon_tags_9 },
    { 0x6e4b9e06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28b9e0c, SUC_lexicon_tags_30 },
    { 0x2bcb9e0d, SUC_lexicon_tags_17 },
    { 0xc4cb9e0e, SUC_lexicon_tags_37 },
    { 0xa00b9e0f, SUC_lexicon_tags_2 },
    { 0xd20b9e0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02b9e13, SUC_lexicon_tags_39 },
    { 0x74cb9e13, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x78cb9e16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cb9e1a, SUC_lexicon_tags_60 },
    { 0x83ab9e1b, SUC_lexicon_tags_7 },
    { 0xf74b9e1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x576b9e1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x618b9e21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acb9e24, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84b9e28, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6b9e2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aeb9e35, SUC_lexicon_tags_7 },
    { 0x3ecb9e36, SUC_lexicon_tags_1 },
    { 0xbd0b9e37, SUC_lexicon_tags_46 },
    { 0xa6eb9e38, SUC_lexicon_tags_1 },
    { 0xa0cb9e38, SUC_lexicon_tags_19 },
    { 0x556b9e36, SUC_lexicon_tags_25 },
    { 0x31eb9e3a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cb9e3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a0b9e41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x556b9e44, SUC_lexicon_tags_25 },
    { 0x822b9e45, SUC_lexicon_tags_0 },
    { 0xd6eb9e44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eb9e4c, SUC_lexicon_tags_220 },
    { 0x2dcb9e4d, SUC_lexicon_tags_25 },
    { 0xadab9e4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb6b9e50, SUC_lexicon_tags_15 },
    { 0x4cab9e51, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24b9e58, SUC_lexicon_tags_12 },
    { 0xdf8b9e59, SUC_lexicon_tags_92 },
    { 0x62cb9e5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcceb9e5d, SUC_lexicon_tags_12 },
    { 0xe8ab9e5d, SUC_lexicon_tags_12 },
    { 0xbfcb9e5f, SUC_lexicon_tags_23 },
    { 0x626b9e60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90eb9e62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x548b9e64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb82b9e66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12b9e6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9e6e, SUC_lexicon_tags_12 },
    { 0x78ab9e6f, SUC_lexicon_tags_60 },
    { 0xfb4b9e70, SUC_lexicon_tags_17 },
    { 0x52cb9e70, SUC_lexicon_tags_38 },
    { 0xc38b9e70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaaab9e74, SUC_lexicon_tags_25 },
    { 0x462b9e75, SUC_lexicon_tags_28 },
    { 0x480b9e74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf12b9e79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30b9e7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302b9e82, SUC_lexicon_tags_60 },
    { 0xef4b9e83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x142b9e85, SUC_lexicon_tags_6 },
    { 0xcb9e86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10b9e8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b8b9e8d, SUC_lexicon_tags_77 },
    { 0x2aeb9e8e, SUC_lexicon_tags_7 },
    { 0xce4b9e8f, SUC_lexicon_tags_48 },
    { 0xc5ab9e90, SUC_lexicon_tags_47 },
    { 0xfccb9e91, SUC_lexicon_tags_26 },
    { 0x9dcb9e8f, SUC_lexicon_tags_13 },
    { 0x45cb9e93, SUC_lexicon_tags_7 },
    { 0xc9eb9e94, SUC_lexicon_tags_15 },
    { 0xb32b9e95, SUC_lexicon_tags_23 },
    { 0x7cb9e92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384b9e9a, SUC_lexicon_tags_63 },
    { 0x362b9e9a, SUC_lexicon_tags_15 },
    { 0x5beb9e9c, SUC_lexicon_tags_2 },
    { 0xcbcb9e9d, SUC_lexicon_tags_7 },
    { 0xd72b9e9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ab9ea3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc5ab9ea5, SUC_lexicon_tags_63 },
    { 0x974b9ea6, SUC_lexicon_tags_31 },
    { 0xd56b9ea6, SUC_lexicon_tags_0 },
    { 0x1b0b9ea8, SUC_lexicon_tags_2 },
    { 0x388b9ea9, SUC_lexicon_tags_25 },
    { 0x4c4b9eaa, SUC_lexicon_tags_15 },
    { 0xdf2b9eab, SUC_lexicon_tags_37 },
    { 0x74b9ea9, SUC_lexicon_tags_7 },
    { 0x46cb9ea9, SUC_lexicon_tags_11 },
    { 0x9a0b9eae, SUC_lexicon_tags_27 },
    { 0x488b9eae, SUC_lexicon_tags_2 },
    { 0x6cb9ea8, SUC_lexicon_tags_9 },
    { 0xb88b9ea6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x756b9eb3, SUC_lexicon_tags_69 },
    { 0xb08b9eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20b9eba, SUC_lexicon_tags_2 },
    { 0xf7ab9ebb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecb9ebe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acb9ec2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaeb9ec9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb30b9ecb, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0x328b9ecd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b9ed0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6b9ed3, SUC_lexicon_tags_30 },
    { 0xe28b9ed4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876b9ed8, SUC_lexicon_tags_28 },
    { 0x5e8b9ed9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x926b9edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x674b9edd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda6b9edf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaaab9ee1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x14cb9ee3, SUC_lexicon_tags_45 },
    { 0x3b6b9ee4, SUC_lexicon_tags_14 },
    { 0x420b9ee5, SUC_lexicon_tags_29 },
    { 0x790b9ee5, SUC_lexicon_tags_21 },
    { 0x4e0b9ee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ab9eec, SUC_lexicon_tags_15 },
    { 0xcfeb9eed, SUC_lexicon_tags_21 },
    { 0x8c6b9eee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486b9ef6, SUC_lexicon_tags_12 },
    { 0xf06b9ef7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda4b9ef9, SUC_lexicon_tags_2 },
    { 0x7c2b9efa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc64b9efc, SUC_lexicon_tags_12 },
    { 0x296b9efd, SUC_lexicon_tags_17 },
    { 0xbb4b9efc, SUC_lexicon_tags_11 },
    { 0x8cab9eff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x14cb9f01, SUC_lexicon_tags_15 },
    { 0x63eb9f01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b9f05, SUC_lexicon_tags_25 },
    { 0x3a8b9f05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ab9f0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc3ab9f0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58b9f12, SUC_lexicon_tags_74 },
    { 0x18b9f13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67eb9f18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34b9f1e, SUC_lexicon_tags_37 },
    { 0x8fab9f1f, SUC_lexicon_tags_14 },
    { 0xc3cb9f20, SUC_lexicon_tags_54 },
    { 0xbb4b9f21, SUC_lexicon_tags_38 },
    { 0x558b9f21, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40ab9f24, SUC_lexicon_tags_1 },
    { 0x4b8b9f24, SUC_lexicon_tags_11 },
    { 0xaf0b9f26, SUC_lexicon_tags_1 },
    { 0xbaeb9f24, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7bcb9f29, SUC_lexicon_tags_11 },
    { 0x4f2b9f29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ceb9f2d, SUC_lexicon_tags_17 },
    { 0x46ab9f2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638b9f32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eb9f3e, SUC_lexicon_tags_10 },
    { 0x60b9f3f, SUC_lexicon_tags_0 },
    { 0xb50b9f40, SUC_lexicon_tags_14 },
    { 0xc48b9f3f, SUC_lexicon_tags_12 },
    { 0x450b9f41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x76eb9f44, SUC_lexicon_tags_25 },
    { 0x7aeb9f45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd2eb9f47, SUC_lexicon_tags_15 },
    { 0x472b9f47, SUC_lexicon_tags_23 },
    { 0xc12b9f49, SUC_lexicon_tags_23 },
    { 0x544b9f4a, SUC_lexicon_tags_12 },
    { 0xa42b9f49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ab9f51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4cb9f53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7eb9f55, SUC_lexicon_tags_107 },
    { 0x950b9f56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416b9f5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10b9f61, SUC_lexicon_tags_17 },
    { 0x272b9f61, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60b9f6b, SUC_lexicon_tags_65 },
    { 0x7f0b9f6b, SUC_lexicon_tags_25 },
    { 0x4b6b9f6b, SUC_lexicon_tags_65 },
    { 0x1c8b9f6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x720b9f70, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc00b9f72, SUC_lexicon_tags_34 },
    { 0x89cb9f73, SUC_lexicon_tags_68 },
    { 0x88ab9f73, SUC_lexicon_tags_65 },
    { 0xae0b9f75, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x36eb9f78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606b9f7e, SUC_lexicon_tags_37 },
    { 0xac8b9f7f, SUC_lexicon_tags_53 },
    { 0x612b9f7f, SUC_lexicon_tags_92 },
    { 0xb86b9f81, SUC_lexicon_tags_63 },
    { 0xf64b9f82, SUC_lexicon_tags_7 },
    { 0x706b9f83, SUC_lexicon_tags_38 },
    { 0x2dcb9f84, SUC_lexicon_tags_70 },
    { 0x650b9f85, SUC_lexicon_tags_11 },
    { 0x1aab9f85, SUC_lexicon_tags_45 },
    { 0x1e2b9f7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e4b9f89, SUC_lexicon_tags_27 },
    { 0x224b9f8a, SUC_lexicon_tags_11 },
    { 0x4e2b9f89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ab9f8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cb9f92, SUC_lexicon_tags_26 },
    { 0xc34b9f92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eab9f97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cb9f9c, SUC_lexicon_tags_37 },
    { 0x4b2b9f9d, SUC_lexicon_tags_21 },
    { 0xb38b9f9e, SUC_lexicon_tags_19 },
    { 0x1deb9f9d, SUC_lexicon_tags_70 },
    { 0x46eb9f9f, SUC_lexicon_tags_19 },
    { 0x67eb9f9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47eb9fa6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f8b9fa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38b9fb1, SUC_lexicon_tags_0 },
    { 0x936b9fb1, SUC_lexicon_tags_25 },
    { 0x456b9fb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5eb9fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x602b9fb7, SUC_lexicon_tags_24 },
    { 0x264b9fb7, SUC_lexicon_tags_3 },
    { 0xac0b9fb9, SUC_lexicon_tags_91 },
    { 0xe78b9fba, SUC_lexicon_tags_201 },
    { 0xdceb9fbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64b9fbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670b9fcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12b9fd1, SUC_lexicon_tags_27 },
    { 0xff4b9fd2, SUC_lexicon_tags_25 },
    { 0x8e8b9fd3, SUC_lexicon_tags_11 },
    { 0xbcab9fd3, SUC_lexicon_tags_9 },
    { 0x5aeb9fd2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x73ab9fd7, SUC_lexicon_tags_37 },
    { 0x39cb9fd8, SUC_lexicon_tags_15 },
    { 0x574b9fd9, SUC_lexicon_tags_11 },
    { 0x174b9fd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdab9fe1, SUC_lexicon_tags_16 },
    { 0xab2b9fe1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x546b9fe5, SUC_lexicon_tags_17 },
    { 0x7e8b9fe6, SUC_lexicon_tags_25 },
    { 0x154b9fe7, SUC_lexicon_tags_11 },
    { 0xc0eb9fe6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x108b9fea, SUC_lexicon_tags_60 },
    { 0x12ab9feb, SUC_lexicon_tags_14 },
    { 0xc90b9feb, SUC_lexicon_tags_1 },
    { 0x672b9feb, SUC_lexicon_tags_7 },
    { 0x4d8b9feb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x21ab9ff0, SUC_lexicon_tags_6 },
    { 0x6fcb9ff1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb68b9ff3, SUC_lexicon_tags_62 },
    { 0x126b9ff4, SUC_lexicon_tags_37 },
    { 0xa02b9ff5, SUC_lexicon_tags_39 },
    { 0xddeb9ff6, SUC_lexicon_tags_1 },
    { 0x522b9ff6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x9beb9ffa, SUC_lexicon_tags_82 },
    { 0x938b9ffa, SUC_lexicon_tags_33 },
    { 0xf26b9ffc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2b9fff, SUC_lexicon_tags_68 },
    { 0xce4ba000, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8ba004, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ba008, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x404ba00a, SUC_lexicon_tags_63 },
    { 0xb5aba00a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eba013, SUC_lexicon_tags_1 },
    { 0xf42ba014, SUC_lexicon_tags_14 },
    { 0x6d8ba015, SUC_lexicon_tags_40 },
    { 0xcb4ba014, SUC_lexicon_tags_2 },
    { 0x4feba017, SUC_lexicon_tags_7 },
    { 0x790ba018, SUC_lexicon_tags_37 },
    { 0x144ba019, SUC_lexicon_tags_35 },
    { 0x5b2ba01a, SUC_lexicon_tags_27 },
    { 0xc76ba018, SUC_lexicon_tags_12 },
    { 0x8e2ba01c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96cba01e, SUC_lexicon_tags_15 },
    { 0xffeba01f, SUC_lexicon_tags_27 },
    { 0x36eba01e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x342ba022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aba027, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x110ba029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82ba02d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59aba030, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xefaba032, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6ba039, SUC_lexicon_tags_30 },
    { 0xd0cba039, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x302ba03c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcba03f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4ba042, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd64ba044, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58ba04f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecba054, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaeba057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8ba05a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460ba05e, SUC_lexicon_tags_7 },
    { 0xe16ba05f, SUC_lexicon_tags_17 },
    { 0x8acba05e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a0ba062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb06ba064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2ba069, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8ba06e, SUC_lexicon_tags_0 },
    { 0xa7eba06e, SUC_lexicon_tags_12 },
    { 0x18aba070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64ba073, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39aba075, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd56ba077, SUC_lexicon_tags_25 },
    { 0x50aba078, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf54ba07a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x712ba07d, SUC_lexicon_tags_151 },
    { 0xce4ba07e, SUC_lexicon_tags_13 },
    { 0xd5eba07f, SUC_lexicon_tags_2 },
    { 0x55cba080, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x808ba083, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0ba086, SUC_lexicon_tags_55 },
    { 0x9f0ba087, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x12ba089, SUC_lexicon_tags_25 },
    { 0x8a4ba08a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aba091, SUC_lexicon_tags_26 },
    { 0x97eba092, SUC_lexicon_tags_14 },
    { 0x1b8ba091, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x650ba096, SUC_lexicon_tags_47 },
    { 0x792ba097, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xecaba099, SUC_lexicon_tags_8 },
    { 0x1d0ba09a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x426ba09d, SUC_lexicon_tags_25 },
    { 0x2f0ba09d, SUC_lexicon_tags_24 },
    { 0x86ba09d, SUC_lexicon_tags_15 },
    { 0x6b6ba09d, SUC_lexicon_tags_15 },
    { 0xa2ba09f, SUC_lexicon_tags_47 },
    { 0x964ba0a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62ba0aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36eba0af, SUC_lexicon_tags_11 },
    { 0x4beba0af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd58ba0b2, SUC_lexicon_tags_11 },
    { 0x50ba0b3, SUC_lexicon_tags_14 },
    { 0x22eba0b4, SUC_lexicon_tags_26 },
    { 0x646ba0b2, SUC_lexicon_tags_6 },
    { 0xd5cba0b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682ba0ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cba0be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aba0c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x97aba0c3, SUC_lexicon_tags_7 },
    { 0xd9aba0c4, SUC_lexicon_tags_14 },
    { 0x30cba0c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724ba0c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x99eba0cb, SUC_lexicon_tags_25 },
    { 0xa10ba0cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x216ba0ce, SUC_lexicon_tags_1 },
    { 0x6e4ba0cf, SUC_lexicon_tags_12 },
    { 0xc24ba0d0, SUC_lexicon_tags_63 },
    { 0xc54ba0d1, SUC_lexicon_tags_11 },
    { 0xf46ba0d0, SUC_lexicon_tags_25 },
    { 0xaecba0d3, SUC_lexicon_tags_45 },
    { 0x572ba0d0, SUC_lexicon_tags_91 },
    { 0x1ccba0d5, SUC_lexicon_tags_27 },
    { 0xf3cba0d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x79eba0d8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x152ba0da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf94ba0dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x408ba0df, SUC_lexicon_tags_4 },
    { 0xad0ba0e0, SUC_lexicon_tags_37 },
    { 0x872ba0e1, SUC_lexicon_tags_11 },
    { 0x3ceba0e1, SUC_lexicon_tags_17 },
    { 0x40ba0e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4ba0e6, SUC_lexicon_tags_0 },
    { 0xe8cba0e7, SUC_lexicon_tags_26 },
    { 0x648ba0e8, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xf44ba0ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaba0ef, SUC_lexicon_tags_33 },
    { 0xee6ba0ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76ba0f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x716ba0f4, SUC_lexicon_tags_82 },
    { 0xfdcba0f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c6ba0f7, SUC_lexicon_tags_32 },
    { 0x1a8ba0f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcba0fb, SUC_lexicon_tags_138 },
    { 0x346ba0fb, SUC_lexicon_tags_39 },
    { 0xe9aba0fd, SUC_lexicon_tags_17 },
    { 0x2f4ba0fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x92eba100, SUC_lexicon_tags_28 },
    { 0x2b4ba101, SUC_lexicon_tags_6 },
    { 0x9ccba101, SUC_lexicon_tags_25 },
    { 0xca2ba103, SUC_lexicon_tags_55 },
    { 0x962ba104, SUC_lexicon_tags_25 },
    { 0x262ba101, SUC_lexicon_tags_31 },
    { 0x5c4ba104, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfaba108, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4ba10e, SUC_lexicon_tags_11 },
    { 0x896ba10e, SUC_lexicon_tags_60 },
    { 0x856ba110, SUC_lexicon_tags_7 },
    { 0xc70ba10f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23aba117, SUC_lexicon_tags_17 },
    { 0x244ba117, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2ba11b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8ba11d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa98ba11f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x222ba122, SUC_lexicon_tags_2 },
    { 0x44ba123, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14ba126, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00ba12f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54ba138, SUC_lexicon_tags_37 },
    { 0x36eba139, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32ba147, SUC_lexicon_tags_30 },
    { 0x4aaba147, SUC_lexicon_tags_31 },
    { 0x4c6ba149, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4d0ba14b, SUC_lexicon_tags_37 },
    { 0x2e0ba14b, SUC_lexicon_tags_12 },
    { 0xa40ba14c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe7eba14f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692ba153, SUC_lexicon_tags_12 },
    { 0xfa8ba154, SUC_lexicon_tags_25 },
    { 0x710ba153, SUC_lexicon_tags_7 },
    { 0x172ba156, SUC_lexicon_tags_47 },
    { 0x208ba157, SUC_lexicon_tags_30 },
    { 0x420ba157, SUC_lexicon_tags_37 },
    { 0x5d6ba159, SUC_lexicon_tags_0 },
    { 0x958ba15a, SUC_lexicon_tags_69 },
    { 0x244ba159, SUC_lexicon_tags_33 },
    { 0xa46ba157, SUC_lexicon_tags_2 },
    { 0xf18ba155, SUC_lexicon_tags_48 },
    { 0xb56ba15e, SUC_lexicon_tags_37 },
    { 0x870ba15f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824ba164, SUC_lexicon_tags_61 },
    { 0x722ba164, SUC_lexicon_tags_37 },
    { 0x8e0ba165, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x5feba169, SUC_lexicon_tags_38 },
    { 0x994ba169, SUC_lexicon_tags_47 },
    { 0x92ba16a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ba16f, SUC_lexicon_tags_84 },
    { 0xfe0ba170, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8ba176, SUC_lexicon_tags_49 },
    { 0x55aba176, SUC_lexicon_tags_2 },
    { 0xba8ba178, SUC_lexicon_tags_113 },
    { 0x69eba179, SUC_lexicon_tags_27 },
    { 0xe6cba17a, SUC_lexicon_tags_1 },
    { 0xbb0ba17b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x35cba17d, SUC_lexicon_tags_15 },
    { 0x12cba17e, SUC_lexicon_tags_17 },
    { 0x6c0ba17e, SUC_lexicon_tags_14 },
    { 0xd5cba17f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8e0ba182, SUC_lexicon_tags_61 },
    { 0xf4eba183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cba18c, SUC_lexicon_tags_37 },
    { 0x57cba18d, SUC_lexicon_tags_11 },
    { 0x600ba18d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168ba195, SUC_lexicon_tags_19 },
    { 0x532ba196, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170ba19a, SUC_lexicon_tags_1 },
    { 0xf10ba19a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cba1a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498ba1a4, SUC_lexicon_tags_51 },
    { 0xa2eba1a4, SUC_lexicon_tags_48 },
    { 0x970ba1a5, SUC_lexicon_tags_11 },
    { 0x25cba1a6, SUC_lexicon_tags_410 },
    { 0x208ba1a5, SUC_lexicon_tags_12 },
    { 0x5e6ba1a9, SUC_lexicon_tags_30 },
    { 0xe54ba1a9, SUC_lexicon_tags_47 },
    { 0xb32ba1ab, SUC_lexicon_tags_28 },
    { 0xa86ba1ac, SUC_lexicon_tags_57 },
    { 0xd98ba1ad, SUC_lexicon_tags_9 },
    { 0xc7cba1ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6ba1b2, SUC_lexicon_tags_29 },
    { 0xc4aba1b3, SUC_lexicon_tags_11 },
    { 0xc7eba1b4, SUC_lexicon_tags_2 },
    { 0xe78ba1b4, SUC_lexicon_tags_116 },
    { 0x5eba1b6, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4ba1b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a2ba1bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70aba1be, SUC_lexicon_tags_19 },
    { 0x360ba1be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350ba1c3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8ba1ca, SUC_lexicon_tags_32 },
    { 0x30cba1ca, SUC_lexicon_tags_49 },
    { 0x628ba1cc, SUC_lexicon_tags_23 },
    { 0x8deba1ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x288ba1cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38ba1d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22aba1da, SUC_lexicon_tags_9 },
    { 0xde8ba1db, SUC_lexicon_tags_95 },
    { 0x902ba1dc, SUC_lexicon_tags_4 },
    { 0x6f0ba1dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49aba1df, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2ba1e3, SUC_lexicon_tags_30 },
    { 0xc2cba1e4, SUC_lexicon_tags_25 },
    { 0xc1cba1e3, SUC_lexicon_tags_17 },
    { 0xd58ba1e4, SUC_lexicon_tags_2 },
    { 0x968ba1e7, SUC_lexicon_tags_47 },
    { 0x2b4ba1e8, SUC_lexicon_tags_28 },
    { 0x9c4ba1e9, SUC_lexicon_tags_15 },
    { 0x842ba1e7, SUC_lexicon_tags_31 },
    { 0xf84ba1eb, SUC_lexicon_tags_38 },
    { 0x720ba1e7, SUC_lexicon_tags_62 },
    { 0x53eba1e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122ba1f3, SUC_lexicon_tags_4 },
    { 0xb2eba1f4, SUC_lexicon_tags_25 },
    { 0xd54ba1f5, SUC_lexicon_tags_25 },
    { 0x5c6ba1f6, SUC_lexicon_tags_7 },
    { 0x19cba1f7, SUC_lexicon_tags_15 },
    { 0x712ba1f7, SUC_lexicon_tags_17 },
    { 0x224ba1f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x194ba1fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x814ba1fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216ba204, SUC_lexicon_tags_23 },
    { 0x6faba204, SUC_lexicon_tags_37 },
    { 0xb06ba206, SUC_lexicon_tags_38 },
    { 0x95aba207, SUC_lexicon_tags_4 },
    { 0x4ba206, SUC_lexicon_tags_27 },
    { 0xa8eba209, SUC_lexicon_tags_12 },
    { 0x154ba204, SUC_lexicon_tags_1 },
    { 0x89aba20b, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0x340ba20e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecba213, SUC_lexicon_tags_49 },
    { 0xa46ba214, SUC_lexicon_tags_0 },
    { 0x588ba214, SUC_lexicon_tags_25 },
    { 0xb10ba216, SUC_lexicon_tags_29 },
    { 0x24aba215, SUC_lexicon_tags_8 },
    { 0xb26ba217, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaba21c, SUC_lexicon_tags_37 },
    { 0xbacba21c, SUC_lexicon_tags_0 },
    { 0xcc6ba21e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6ba221, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x88eba224, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd86ba226, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542ba22b, SUC_lexicon_tags_33 },
    { 0x3caba22b, SUC_lexicon_tags_14 },
    { 0x544ba22d, SUC_lexicon_tags_17 },
    { 0x364ba22b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ba232, SUC_lexicon_tags_29 },
    { 0x322ba233, SUC_lexicon_tags_11 },
    { 0xd8cba233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18cba236, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0xedaba238, SUC_lexicon_tags_12 },
    { 0x560ba239, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aba23c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32ba23e, SUC_lexicon_tags_32 },
    { 0xe5aba23e, SUC_lexicon_tags_12 },
    { 0x92aba240, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8ba243, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fcba245, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284ba24a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaba24f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246ba254, SUC_lexicon_tags_0 },
    { 0x44aba254, SUC_lexicon_tags_1 },
    { 0x2a2ba256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eba259, SUC_lexicon_tags_4 },
    { 0x8eba25a, SUC_lexicon_tags_11 },
    { 0xc54ba25a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x648ba25e, SUC_lexicon_tags_12 },
    { 0xed6ba25f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4eba261, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86ba265, SUC_lexicon_tags_25 },
    { 0x398ba266, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cba269, SUC_lexicon_tags_37 },
    { 0xebaba26a, SUC_lexicon_tags_47 },
    { 0xc60ba26b, SUC_lexicon_tags_318 },
    { 0xc14ba26c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaba272, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29eba274, SUC_lexicon_tags_7 },
    { 0x500ba275, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb8eba277, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640ba27c, SUC_lexicon_tags_26 },
    { 0xbd4ba27c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06ba281, SUC_lexicon_tags_16 },
    { 0x1e2ba282, SUC_lexicon_tags_2 },
    { 0xec4ba283, SUC_lexicon_tags_10 },
    { 0x7d0ba283, SUC_lexicon_tags_40 },
    { 0x9c6ba285, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eba288, SUC_lexicon_tags_32 },
    { 0x61cba288, SUC_lexicon_tags_7 },
    { 0x6feba28a, SUC_lexicon_tags_28 },
    { 0x45cba289, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1beba28f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4cba291, SUC_lexicon_tags_49 },
    { 0xfe8ba292, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8ba295, SUC_lexicon_tags_16 },
    { 0xd4cba296, SUC_lexicon_tags_17 },
    { 0x6e4ba296, SUC_lexicon_tags_33 },
    { 0xe3cba296, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16ba29c, SUC_lexicon_tags_1 },
    { 0x23aba29d, SUC_lexicon_tags_11 },
    { 0x710ba29d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5aba2a1, SUC_lexicon_tags_14 },
    { 0xff6ba2a1, SUC_lexicon_tags_35 },
    { 0x94eba2a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10ba2ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4ba2af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2aaba2b1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cba2b5, SUC_lexicon_tags_16 },
    { 0xb04ba2b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2deba2bd, SUC_lexicon_tags_11 },
    { 0xf32ba2be, SUC_lexicon_tags_37 },
    { 0xe60ba2be, SUC_lexicon_tags_9 },
    { 0x7deba2c0, SUC_lexicon_tags_25 },
    { 0x420ba2c0, SUC_lexicon_tags_25 },
    { 0xf62ba2c1, SUC_lexicon_tags_103 },
    { 0x382ba2c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2c7, SUC_lexicon_tags_15 },
    { 0xdbeba2c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecba2cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x908ba2ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74ba2d1, SUC_lexicon_tags_12 },
    { 0xfeeba2d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976ba2d9, SUC_lexicon_tags_11 },
    { 0xf7eba2da, SUC_lexicon_tags_12 },
    { 0xcd2ba2db, SUC_lexicon_tags_48 },
    { 0x166ba2dc, SUC_lexicon_tags_11 },
    { 0x748ba2d9, SUC_lexicon_tags_1 },
    { 0x18ba2d9, SUC_lexicon_tags_26 },
    { 0xbcaba2df, SUC_lexicon_tags_82 },
    { 0x9eba2e0, SUC_lexicon_tags_29 },
    { 0x252ba2e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222ba2e8, SUC_lexicon_tags_11 },
    { 0x152ba2e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ba2ef, SUC_lexicon_tags_27 },
    { 0x2c2ba2f0, SUC_lexicon_tags_2 },
    { 0xfbaba2f0, SUC_lexicon_tags_15 },
    { 0x75aba2f2, SUC_lexicon_tags_27 },
    { 0xd5eba2f0, SUC_lexicon_tags_12 },
    { 0x92aba2f1, SUC_lexicon_tags_4 },
    { 0xed6ba2f5, SUC_lexicon_tags_84 },
    { 0xfb8ba2f6, SUC_lexicon_tags_7 },
    { 0xd36ba2f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97eba2ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe8ba301, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec4ba303, SUC_lexicon_tags_7 },
    { 0xa48ba303, SUC_lexicon_tags_2 },
    { 0x4e0ba303, SUC_lexicon_tags_49 },
    { 0x988ba306, SUC_lexicon_tags_114 },
    { 0x29aba307, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad4ba309, SUC_lexicon_tags_7 },
    { 0x396ba309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36aba30d, SUC_lexicon_tags_23 },
    { 0x2ba30e, SUC_lexicon_tags_49 },
    { 0xaf4ba30d, SUC_lexicon_tags_49 },
    { 0x552ba310, SUC_lexicon_tags_38 },
    { 0x860ba311, SUC_lexicon_tags_23 },
    { 0xe28ba312, SUC_lexicon_tags_8 },
    { 0xcb8ba312, SUC_lexicon_tags_2 },
    { 0xff8ba311, SUC_lexicon_tags_25 },
    { 0x79cba314, SUC_lexicon_tags_46 },
    { 0x9faba314, SUC_lexicon_tags_16 },
    { 0xa34ba316, SUC_lexicon_tags_17 },
    { 0x2f6ba318, SUC_lexicon_tags_7 },
    { 0x72ba319, SUC_lexicon_tags_11 },
    { 0xe0cba31a, SUC_lexicon_tags_2 },
    { 0x710ba319, SUC_lexicon_tags_25 },
    { 0x784ba31a, SUC_lexicon_tags_38 },
    { 0x68eba31c, SUC_lexicon_tags_43 },
    { 0xcb2ba319, SUC_lexicon_tags_9 },
    { 0x92cba31f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cba328, SUC_lexicon_tags_15 },
    { 0xde8ba328, SUC_lexicon_tags_68 },
    { 0x550ba32a, SUC_lexicon_tags_10 },
    { 0xcdaba32b, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xf0ba32d, SUC_lexicon_tags_37 },
    { 0x4c8ba32e, SUC_lexicon_tags_29 },
    { 0x856ba32e, SUC_lexicon_tags_36 },
    { 0x5b8ba32e, SUC_lexicon_tags_25 },
    { 0xc48ba330, SUC_lexicon_tags_31 },
    { 0x570ba32e, SUC_lexicon_tags_9 },
    { 0xa44ba333, SUC_lexicon_tags_17 },
    { 0x2aaba32d, SUC_lexicon_tags_10 },
    { 0xe48ba334, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdacba337, SUC_lexicon_tags_11 },
    { 0x2b4ba337, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6deba341, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d2ba343, SUC_lexicon_tags_12 },
    { 0xa4aba343, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf32ba346, SUC_lexicon_tags_2 },
    { 0x5f8ba346, SUC_lexicon_tags_19 },
    { 0xcecba348, SUC_lexicon_tags_15 },
    { 0xb82ba348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04ba34c, SUC_lexicon_tags_37 },
    { 0x886ba34d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12ba352, SUC_lexicon_tags_12 },
    { 0xb10ba352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x572ba356, SUC_lexicon_tags_23 },
    { 0xac4ba357, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c2ba359, SUC_lexicon_tags_10 },
    { 0x794ba35a, SUC_lexicon_tags_60 },
    { 0x148ba35a, SUC_lexicon_tags_9 },
    { 0xbd4ba35c, SUC_lexicon_tags_32 },
    { 0x1f6ba35c, SUC_lexicon_tags_25 },
    { 0x2acba35e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6ba364, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8ba366, SUC_lexicon_tags_218 },
    { 0x90eba367, SUC_lexicon_tags_11 },
    { 0xfcaba368, SUC_lexicon_tags_12 },
    { 0xc8aba369, SUC_lexicon_tags_17 },
    { 0x766ba369, SUC_lexicon_tags_25 },
    { 0x364ba36b, SUC_lexicon_tags_26 },
    { 0x7a6ba36c, SUC_lexicon_tags_25 },
    { 0x804ba366, SUC_lexicon_tags_33 },
    { 0xeeeba369, SUC_lexicon_tags_28 },
    { 0x744ba36f, SUC_lexicon_tags_22 },
    { 0x736ba36f, SUC_lexicon_tags_17 },
    { 0xa10ba36e, SUC_lexicon_tags_23 },
    { 0xb62ba372, SUC_lexicon_tags_15 },
    { 0x60eba36d, SUC_lexicon_tags_17 },
    { 0xa12ba36e, SUC_lexicon_tags_17 },
    { 0x6caba375, SUC_lexicon_tags_37 },
    { 0xbe2ba376, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d6ba378, SUC_lexicon_tags_160 },
    { 0xc46ba379, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbeeba37b, SUC_lexicon_tags_37 },
    { 0x982ba37c, SUC_lexicon_tags_28 },
    { 0x652ba37c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76ba380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca8ba382, SUC_lexicon_tags_46 },
    { 0xc22ba383, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x336ba385, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0ba38a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe8ba38c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8ba38f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cba392, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e4ba394, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfdaba396, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x37aba398, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x806ba39a, SUC_lexicon_tags_5 },
    { 0xf08ba39a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8eba39f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7eba3a1, SUC_lexicon_tags_87 },
    { 0xa7eba3a1, SUC_lexicon_tags_12 },
    { 0x58eba3a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85eba3a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c6ba3a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268ba3af, SUC_lexicon_tags_34 },
    { 0xa24ba3af, SUC_lexicon_tags_45 },
    { 0xbeba3af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d0ba3b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a4ba3b5, SUC_lexicon_tags_172 },
    { 0xa9cba3b6, SUC_lexicon_tags_1 },
    { 0xd4eba3b7, SUC_lexicon_tags_11 },
    { 0x7b6ba3b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254ba3bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf6ba3c1, SUC_lexicon_tags_26 },
    { 0x47aba3c2, SUC_lexicon_tags_0 },
    { 0x9e4ba3c3, SUC_lexicon_tags_11 },
    { 0x15cba3c3, SUC_lexicon_tags_0 },
    { 0x576ba3c3, SUC_lexicon_tags_17 },
    { 0x42aba3c4, SUC_lexicon_tags_30 },
    { 0x77cba3c7, SUC_lexicon_tags_103 },
    { 0x3e2ba3c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe04ba3ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x480ba3cc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xee8ba3ce, SUC_lexicon_tags_0 },
    { 0x382ba3ce, SUC_lexicon_tags_36 },
    { 0xefcba3d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x320ba3d2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6ba3d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2ba3d9, SUC_lexicon_tags_37 },
    { 0xd36ba3da, SUC_lexicon_tags_63 },
    { 0x618ba3db, SUC_lexicon_tags_37 },
    { 0xcceba3db, SUC_lexicon_tags_15 },
    { 0x35eba3dd, SUC_lexicon_tags_28 },
    { 0xcc6ba3db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8ba3e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74ba3e3, SUC_lexicon_tags_15 },
    { 0x9beba3e4, SUC_lexicon_tags_11 },
    { 0x9e2ba3e5, SUC_lexicon_tags_9 },
    { 0xc8aba3e6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272ba3ea, SUC_lexicon_tags_37 },
    { 0xdc2ba3ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402ba3ef, SUC_lexicon_tags_17 },
    { 0x202ba3ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cba3f3, SUC_lexicon_tags_37 },
    { 0xb02ba3f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aba3fb, SUC_lexicon_tags_49 },
    { 0xe76ba3fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e4ba3fe, SUC_lexicon_tags_5 },
    { 0xdd6ba3ff, SUC_lexicon_tags_25 },
    { 0x336ba3fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7aba403, SUC_lexicon_tags_66 },
    { 0xad6ba403, SUC_lexicon_tags_25 },
    { 0x5d0ba404, SUC_lexicon_tags_25 },
    { 0xfc0ba406, SUC_lexicon_tags_63 },
    { 0x868ba407, SUC_lexicon_tags_0 },
    { 0x7e4ba407, SUC_lexicon_tags_14 },
    { 0x95eba409, SUC_lexicon_tags_50 },
    { 0x99eba409, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4ba413, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aba418, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6ba41d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x252ba420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4ba423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28cba426, SUC_lexicon_tags_14 },
    { 0xe58ba427, SUC_lexicon_tags_11 },
    { 0x10aba427, SUC_lexicon_tags_32 },
    { 0x27cba427, SUC_lexicon_tags_43 },
    { 0xd62ba429, SUC_lexicon_tags_37 },
    { 0xc8ba42b, SUC_lexicon_tags_45 },
    { 0x8aeba42c, SUC_lexicon_tags_9 },
    { 0x988ba42d, SUC_lexicon_tags_6 },
    { 0x150ba428, SUC_lexicon_tags_25 },
    { 0xb8cba426, SUC_lexicon_tags_10 },
    { 0x46aba430, SUC_lexicon_tags_37 },
    { 0x8d4ba426, SUC_lexicon_tags_25 },
    { 0x820ba432, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95cba434, SUC_lexicon_tags_14 },
    { 0xe10ba434, SUC_lexicon_tags_2 },
    { 0x2a0ba435, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x942ba438, SUC_lexicon_tags_49 },
    { 0x908ba439, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d6ba43b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eba43f, SUC_lexicon_tags_33 },
    { 0xbbaba440, SUC_lexicon_tags_27 },
    { 0x9aba441, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0ba444, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98ba449, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb2eba44b, SUC_lexicon_tags_37 },
    { 0x71cba44c, SUC_lexicon_tags_47 },
    { 0x60cba44c, SUC_lexicon_tags_47 },
    { 0x37aba44e, SUC_lexicon_tags_48 },
    { 0x88eba44e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3aba454, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80ba457, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aba45d, SUC_lexicon_tags_30 },
    { 0x722ba45e, SUC_lexicon_tags_37 },
    { 0x72aba45e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aba46c, SUC_lexicon_tags_11 },
    { 0x46eba46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64ba470, SUC_lexicon_tags_60 },
    { 0xc04ba470, SUC_lexicon_tags_10 },
    { 0xecba472, SUC_lexicon_tags_1 },
    { 0x194ba472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422ba477, SUC_lexicon_tags_24 },
    { 0xd26ba478, SUC_lexicon_tags_1 },
    { 0x7d8ba477, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x868ba47c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd78ba47e, SUC_lexicon_tags_37 },
    { 0x846ba47f, SUC_lexicon_tags_37 },
    { 0x582ba480, SUC_lexicon_tags_2 },
    { 0x73eba481, SUC_lexicon_tags_43 },
    { 0x74eba481, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458ba486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0ba48c, SUC_lexicon_tags_35 },
    { 0xd86ba48c, SUC_lexicon_tags_15 },
    { 0xbdaba48d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66aba490, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cba493, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x300ba495, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344ba49a, SUC_lexicon_tags_44 },
    { 0x766ba49b, SUC_lexicon_tags_15 },
    { 0x698ba49b, SUC_lexicon_tags_8 },
    { 0xce0ba49c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2eba49f, SUC_lexicon_tags_108 },
    { 0x466ba4a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceba4b2, SUC_lexicon_tags_25 },
    { 0xc94ba4b3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccba4bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27eba4c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb8cba4c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2cba4c9, SUC_lexicon_tags_7 },
    { 0x194ba4c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x942ba4cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23cba4cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2ba4d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4ba4d7, SUC_lexicon_tags_6 },
    { 0xdceba4d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26ba4de, SUC_lexicon_tags_17 },
    { 0x1f2ba4df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x81aba4e1, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x8feba4e3, SUC_lexicon_tags_23 },
    { 0x10aba4e3, SUC_lexicon_tags_1 },
    { 0xea2ba4e3, SUC_lexicon_tags_17 },
    { 0xedeba4e3, SUC_lexicon_tags_38 },
    { 0x416ba4e4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x29eba4e9, SUC_lexicon_tags_1 },
    { 0xf58ba4ea, SUC_lexicon_tags_7 },
    { 0xf08ba4ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa06ba4ed, SUC_lexicon_tags_30 },
    { 0x658ba4ee, SUC_lexicon_tags_25 },
    { 0xc74ba4ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cba4f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d4ba4f4, SUC_lexicon_tags_33 },
    { 0xb40ba4f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeba4f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94eba4fb, SUC_lexicon_tags_57 },
    { 0xc00ba4fc, SUC_lexicon_tags_47 },
    { 0x912ba4fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3eeba4ff, SUC_lexicon_tags_0 },
    { 0x142ba500, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756ba504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154ba50a, SUC_lexicon_tags_1 },
    { 0xa78ba50b, SUC_lexicon_tags_12 },
    { 0x25aba50c, SUC_lexicon_tags_16 },
    { 0xfeba50a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x250ba510, SUC_lexicon_tags_34 },
    { 0x276ba511, SUC_lexicon_tags_61 },
    { 0x84aba512, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aba515, SUC_lexicon_tags_49 },
    { 0xf8aba515, SUC_lexicon_tags_12 },
    { 0xa1eba517, SUC_lexicon_tags_15 },
    { 0xbe2ba518, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x464ba51a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36ba51d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5acba520, SUC_lexicon_tags_84 },
    { 0x542ba521, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x570ba523, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848ba52b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x946ba52d, SUC_lexicon_tags_37 },
    { 0x260ba52e, SUC_lexicon_tags_23 },
    { 0x1a6ba52f, SUC_lexicon_tags_25 },
    { 0xb8eba52e, SUC_lexicon_tags_11 },
    { 0x1daba52d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166ba536, SUC_lexicon_tags_68 },
    { 0x2a4ba537, SUC_lexicon_tags_25 },
    { 0x4a6ba538, SUC_lexicon_tags_17 },
    { 0xf6cba536, SUC_lexicon_tags_55 },
    { 0xa64ba539, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6daba53d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x856ba53f, SUC_lexicon_tags_25 },
    { 0xeaba53f, SUC_lexicon_tags_30 },
    { 0x8e0ba540, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eba545, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe68ba548, SUC_lexicon_tags_14 },
    { 0xc02ba548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa48ba54b, SUC_lexicon_tags_2 },
    { 0x450ba54c, SUC_lexicon_tags_6 },
    { 0x650ba54c, SUC_lexicon_tags_22 },
    { 0xa6eba54e, SUC_lexicon_tags_4 },
    { 0x866ba54f, SUC_lexicon_tags_19 },
    { 0xc26ba54e, SUC_lexicon_tags_2 },
    { 0xe24ba54e, SUC_lexicon_tags_8 },
    { 0xf2aba552, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3eba556, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x606ba559, SUC_lexicon_tags_75 },
    { 0x7c4ba559, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eba55f, SUC_lexicon_tags_21 },
    { 0x97eba560, SUC_lexicon_tags_23 },
    { 0xde6ba55f, SUC_lexicon_tags_23 },
    { 0x566ba562, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9eba574, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3deba577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76ba579, SUC_lexicon_tags_28 },
    { 0xad0ba57a, SUC_lexicon_tags_28 },
    { 0xea6ba57b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55aba581, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55eba583, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaba58c, SUC_lexicon_tags_47 },
    { 0x568ba58d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe04ba58f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134ba594, SUC_lexicon_tags_25 },
    { 0xfcaba595, SUC_lexicon_tags_37 },
    { 0xcdaba596, SUC_lexicon_tags_49 },
    { 0x15aba596, SUC_lexicon_tags_7 },
    { 0x2fcba598, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cba59b, SUC_lexicon_tags_60 },
    { 0xb0aba59c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbeaba59e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x48aba5a0, SUC_lexicon_tags_29 },
    { 0x93cba5a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x41eba5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0ba5a8, SUC_lexicon_tags_11 },
    { 0x436ba5a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x756ba5ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96ba5ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cba5b1, SUC_lexicon_tags_61 },
    { 0x242ba5b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c8ba5b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82eba5b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00ba5ba, SUC_lexicon_tags_68 },
    { 0xb16ba5bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff2ba5bd, SUC_lexicon_tags_17 },
    { 0x6baba5be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x270ba5c1, SUC_lexicon_tags_26 },
    { 0xa3eba5c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8ba5c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x732ba5c7, SUC_lexicon_tags_30 },
    { 0xad6ba5c8, SUC_lexicon_tags_7 },
    { 0xde2ba5c8, SUC_lexicon_tags_11 },
    { 0xd28ba5ca, SUC_lexicon_tags_26 },
    { 0xf26ba5ca, SUC_lexicon_tags_3 },
    { 0x19eba5c7, SUC_lexicon_tags_65 },
    { 0x4eaba5c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cba5d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcba5d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972ba5de, SUC_lexicon_tags_7 },
    { 0xe0aba5df, SUC_lexicon_tags_53 },
    { 0x9eeba5e0, SUC_lexicon_tags_17 },
    { 0x492ba5df, SUC_lexicon_tags_12 },
    { 0xa2cba5e2, SUC_lexicon_tags_60 },
    { 0xde4ba5e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x896ba5e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774ba5e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464ba5ed, SUC_lexicon_tags_14 },
    { 0xf5cba5ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xad4ba5f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb86ba5f2, SUC_lexicon_tags_31 },
    { 0xf70ba5f2, SUC_lexicon_tags_25 },
    { 0x9eeba5f4, SUC_lexicon_tags_49 },
    { 0xbd0ba5f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25eba5f7, SUC_lexicon_tags_2 },
    { 0x762ba5f8, SUC_lexicon_tags_29 },
    { 0x55aba5f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaeba5fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60ba5ff, SUC_lexicon_tags_6 },
    { 0xff2ba600, SUC_lexicon_tags_11 },
    { 0xbaaba5ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacba605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358ba609, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816ba60d, SUC_lexicon_tags_25 },
    { 0xcacba60d, SUC_lexicon_tags_269 },
    { 0x26ba60d, SUC_lexicon_tags_14 },
    { 0x552ba610, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x66ba612, SUC_lexicon_tags_23 },
    { 0x356ba612, SUC_lexicon_tags_28 },
    { 0x16ba614, SUC_lexicon_tags_17 },
    { 0xfa6ba614, SUC_lexicon_tags_30 },
    { 0x42aba616, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcba618, SUC_lexicon_tags_7 },
    { 0x30eba619, SUC_lexicon_tags_12 },
    { 0x280ba618, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54ba61d, SUC_lexicon_tags_38 },
    { 0xc20ba61e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6aba620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6aba625, SUC_lexicon_tags_8 },
    { 0xf3eba625, SUC_lexicon_tags_48 },
    { 0xa14ba627, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ba62f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4beba633, SUC_lexicon_tags_17 },
    { 0x7faba634, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faba63c, SUC_lexicon_tags_25 },
    { 0xfb0ba63c, SUC_lexicon_tags_45 },
    { 0x724ba63e, SUC_lexicon_tags_45 },
    { 0x192ba63f, SUC_lexicon_tags_45 },
    { 0x2f4ba63f, SUC_lexicon_tags_81 },
    { 0x90ba641, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682ba647, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x5d6ba649, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f6ba64b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34ba656, SUC_lexicon_tags_29 },
    { 0x2ba657, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79eba660, SUC_lexicon_tags_60 },
    { 0xe9eba661, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4ba664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ba667, SUC_lexicon_tags_10 },
    { 0x520ba667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8deba66d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538ba678, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfaba67e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x696ba681, SUC_lexicon_tags_14 },
    { 0xc60ba681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3aba687, SUC_lexicon_tags_21 },
    { 0x64cba687, SUC_lexicon_tags_11 },
    { 0x896ba689, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feba68e, SUC_lexicon_tags_26 },
    { 0x3d4ba68f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488ba695, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0ba699, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x404ba69b, SUC_lexicon_tags_33 },
    { 0x720ba69c, SUC_lexicon_tags_27 },
    { 0x788ba69b, SUC_lexicon_tags_11 },
    { 0x3dcba69e, SUC_lexicon_tags_26 },
    { 0xdccba69f, SUC_lexicon_tags_12 },
    { 0x358ba69f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x63aba6a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x330ba6a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2d0ba6a7, SUC_lexicon_tags_2 },
    { 0x5a4ba6a8, SUC_lexicon_tags_80 },
    { 0x190ba6a8, SUC_lexicon_tags_38 },
    { 0x7f6ba6aa, SUC_lexicon_tags_46 },
    { 0x1c4ba6ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x748ba6ad, SUC_lexicon_tags_224 },
    { 0x2feba6ad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa12ba6b0, SUC_lexicon_tags_17 },
    { 0xcb6ba6b0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5e8ba6b3, SUC_lexicon_tags_2 },
    { 0xa02ba6b4, SUC_lexicon_tags_17 },
    { 0xec6ba6b5, SUC_lexicon_tags_8 },
    { 0xb84ba6b6, SUC_lexicon_tags_273 },
    { 0x2aba6b6, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56ba6bd, SUC_lexicon_tags_15 },
    { 0x68ba6bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe3cba6c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daba6c7, SUC_lexicon_tags_48 },
    { 0xea4ba6c8, SUC_lexicon_tags_0 },
    { 0x66eba6c9, SUC_lexicon_tags_25 },
    { 0xe3eba6c7, SUC_lexicon_tags_13 },
    { 0x7beba6c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b6ba6cd, SUC_lexicon_tags_19 },
    { 0xc48ba6ce, SUC_lexicon_tags_47 },
    { 0x4dcba6ce, SUC_lexicon_tags_114 },
    { 0xb62ba6ce, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4ba6d5, SUC_lexicon_tags_17 },
    { 0xbc4ba6d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80ba6e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddeba6e3, SUC_lexicon_tags_32 },
    { 0xc74ba6e3, SUC_lexicon_tags_14 },
    { 0x7dcba6e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cba6ea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x59aba6ec, SUC_lexicon_tags_216 },
    { 0x886ba6ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c2ba6ef, SUC_lexicon_tags_37 },
    { 0x752ba6f0, SUC_lexicon_tags_33 },
    { 0xa68ba6f0, SUC_lexicon_tags_65 },
    { 0x1a8ba6f0, SUC_lexicon_tags_37 },
    { 0x524ba6f3, SUC_lexicon_tags_4 },
    { 0x836ba6f4, SUC_lexicon_tags_33 },
    { 0x5daba6f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f4ba6f7, SUC_lexicon_tags_9 },
    { 0x752ba6f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x258ba6fb, SUC_lexicon_tags_11 },
    { 0x2eaba6fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2ba704, SUC_lexicon_tags_11 },
    { 0xcceba705, SUC_lexicon_tags_17 },
    { 0xb04ba705, SUC_lexicon_tags_14 },
    { 0x88ba707, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71aba70a, SUC_lexicon_tags_1 },
    { 0x8f8ba70b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cba70e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd2ba710, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80ba714, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aaba717, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe84ba719, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba71c, SUC_lexicon_tags_12 },
    { 0x764ba71c, SUC_lexicon_tags_177 },
    { 0x5d6ba71e, SUC_lexicon_tags_46 },
    { 0xbc4ba71f, SUC_lexicon_tags_31 },
    { 0xebeba71f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8ba723, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23eba725, SUC_lexicon_tags_17 },
    { 0xa8aba726, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaba72a, SUC_lexicon_tags_7 },
    { 0x4f4ba72a, SUC_lexicon_tags_1 },
    { 0x2a8ba72a, SUC_lexicon_tags_47 },
    { 0xa1cba72a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x794ba730, SUC_lexicon_tags_23 },
    { 0x724ba731, SUC_lexicon_tags_15 },
    { 0xbbaba731, SUC_lexicon_tags_19 },
    { 0xd7aba733, SUC_lexicon_tags_1 },
    { 0x5e8ba731, SUC_lexicon_tags_46 },
    { 0xff8ba735, SUC_lexicon_tags_25 },
    { 0xc0cba736, SUC_lexicon_tags_17 },
    { 0xeb8ba737, SUC_lexicon_tags_34 },
    { 0xbeeba738, SUC_lexicon_tags_13 },
    { 0x752ba739, SUC_lexicon_tags_37 },
    { 0x2f2ba73a, SUC_lexicon_tags_47 },
    { 0xb4cba737, SUC_lexicon_tags_47 },
    { 0xbc4ba735, SUC_lexicon_tags_14 },
    { 0x30cba73d, SUC_lexicon_tags_19 },
    { 0xe98ba73d, SUC_lexicon_tags_31 },
    { 0xda0ba73e, SUC_lexicon_tags_17 },
    { 0xb64ba740, SUC_lexicon_tags_37 },
    { 0x1f4ba739, SUC_lexicon_tags_95 },
    { 0x744ba73d, SUC_lexicon_tags_29 },
    { 0xeaeba738, SUC_lexicon_tags_43 },
    { 0x6d4ba737, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89eba749, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x224ba74c, SUC_lexicon_tags_59 },
    { 0xcb8ba74c, SUC_lexicon_tags_17 },
    { 0xd98ba74e, SUC_lexicon_tags_15 },
    { 0x82aba74f, SUC_lexicon_tags_79 },
    { 0x7b2ba750, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2ba754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe24ba756, SUC_lexicon_tags_11 },
    { 0xab6ba757, SUC_lexicon_tags_12 },
    { 0xe1aba758, SUC_lexicon_tags_11 },
    { 0x768ba759, SUC_lexicon_tags_17 },
    { 0xbbaba756, SUC_lexicon_tags_37 },
    { 0xbceba75b, SUC_lexicon_tags_11 },
    { 0xf48ba75c, SUC_lexicon_tags_27 },
    { 0xdb0ba75c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112ba761, SUC_lexicon_tags_60 },
    { 0xe0eba762, SUC_lexicon_tags_15 },
    { 0xd5eba762, SUC_lexicon_tags_4 },
    { 0x838ba763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdceba766, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464ba76a, SUC_lexicon_tags_1 },
    { 0xe5eba76a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa58ba76d, SUC_lexicon_tags_9 },
    { 0xac2ba76d, SUC_lexicon_tags_7 },
    { 0x72aba76f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daba774, SUC_lexicon_tags_17 },
    { 0xc8eba774, SUC_lexicon_tags_47 },
    { 0x44aba776, SUC_lexicon_tags_19 },
    { 0xa98ba776, SUC_lexicon_tags_1 },
    { 0xac6ba778, SUC_lexicon_tags_31 },
    { 0x9eba776, SUC_lexicon_tags_91 },
    { 0xeeeba77a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbaeba77c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xafcba77f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeba782, SUC_lexicon_tags_137 },
    { 0x2deba783, SUC_lexicon_tags_21 },
    { 0xa7aba784, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446ba788, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0ba790, SUC_lexicon_tags_191 },
    { 0xb1aba791, SUC_lexicon_tags_49 },
    { 0x288ba790, SUC_lexicon_tags_49 },
    { 0xf08ba793, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8ba797, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x192ba799, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x758ba79c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6ba79f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcba7a2, SUC_lexicon_tags_45 },
    { 0x7d4ba7a3, SUC_lexicon_tags_37 },
    { 0x8fcba7a3, SUC_lexicon_tags_1 },
    { 0x18ba7a3, SUC_lexicon_tags_26 },
    { 0xefaba7a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaba7aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2ba7ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534ba7b3, SUC_lexicon_tags_19 },
    { 0xc20ba7b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe6eba7b6, SUC_lexicon_tags_27 },
    { 0x46aba7b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cba7be, SUC_lexicon_tags_14 },
    { 0x9daba7bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cba7c3, SUC_lexicon_tags_25 },
    { 0x69eba7c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0ba7cc, SUC_lexicon_tags_37 },
    { 0xaaaba7cc, SUC_lexicon_tags_37 },
    { 0x6daba7ce, SUC_lexicon_tags_104 },
    { 0x718ba7ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850ba7d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcba7d8, SUC_lexicon_tags_55 },
    { 0x812ba7d9, SUC_lexicon_tags_46 },
    { 0xb48ba7d8, SUC_lexicon_tags_23 },
    { 0x9c0ba7db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62cba7dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0aba7e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96aba7e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2acba7e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4ba7ed, SUC_lexicon_tags_2 },
    { 0xaeaba7ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4ba7f3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x732ba7f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccba7fa, SUC_lexicon_tags_37 },
    { 0x25aba7fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684ba805, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x168ba808, SUC_lexicon_tags_61 },
    { 0x5eba809, SUC_lexicon_tags_4 },
    { 0x614ba809, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6ba811, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70eba819, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0ba81c, SUC_lexicon_tags_17 },
    { 0x65aba81c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x6aaba81f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb9eba821, SUC_lexicon_tags_25 },
    { 0x958ba822, SUC_lexicon_tags_26 },
    { 0xf12ba822, SUC_lexicon_tags_25 },
    { 0x498ba824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46ba828, SUC_lexicon_tags_21 },
    { 0xe1eba828, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cba82d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x478ba82f, SUC_lexicon_tags_29 },
    { 0xcfcba82f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23aba838, SUC_lexicon_tags_15 },
    { 0x8cba838, SUC_lexicon_tags_1 },
    { 0x2e0ba83a, SUC_lexicon_tags_2 },
    { 0xe6ba83b, SUC_lexicon_tags_16 },
    { 0x50cba839, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x118ba83e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xeccba840, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0ba843, SUC_lexicon_tags_37 },
    { 0x748ba844, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x38eba846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcdcba848, SUC_lexicon_tags_64 },
    { 0x774ba849, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4ba84c, SUC_lexicon_tags_1 },
    { 0x6daba84d, SUC_lexicon_tags_6 },
    { 0x380ba84c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ceba850, SUC_lexicon_tags_11 },
    { 0x1eeba850, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x482ba853, SUC_lexicon_tags_32 },
    { 0xc0aba853, SUC_lexicon_tags_20 },
    { 0xd28ba855, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412ba85a, SUC_lexicon_tags_49 },
    { 0x44cba85b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24ba862, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafaba864, SUC_lexicon_tags_37 },
    { 0xde8ba865, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x776ba867, SUC_lexicon_tags_37 },
    { 0xb82ba868, SUC_lexicon_tags_15 },
    { 0xe54ba869, SUC_lexicon_tags_8 },
    { 0x85eba86a, SUC_lexicon_tags_214 },
    { 0x458ba86a, SUC_lexicon_tags_0 },
    { 0x69aba86b, SUC_lexicon_tags_11 },
    { 0xb8eba86d, SUC_lexicon_tags_26 },
    { 0xee2ba86e, SUC_lexicon_tags_32 },
    { 0x596ba86d, SUC_lexicon_tags_16 },
    { 0xaf4ba870, SUC_lexicon_tags_25 },
    { 0x3d0ba871, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56ba874, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246ba879, SUC_lexicon_tags_12 },
    { 0x3f6ba87a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8ba87d, SUC_lexicon_tags_7 },
    { 0xe16ba87e, SUC_lexicon_tags_11 },
    { 0x524ba87e, SUC_lexicon_tags_63 },
    { 0x20cba880, SUC_lexicon_tags_14 },
    { 0xa22ba87e, SUC_lexicon_tags_28 },
    { 0xef4ba880, SUC_lexicon_tags_46 },
    { 0x944ba87d, SUC_lexicon_tags_14 },
    { 0x47aba884, SUC_lexicon_tags_4 },
    { 0x9feba885, SUC_lexicon_tags_15 },
    { 0x892ba885, SUC_lexicon_tags_82 },
    { 0x458ba887, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4baba889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448ba88f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82ba893, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x614ba895, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5cba897, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8ba89f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0ba8aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aeba8ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdeeba8af, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aba8b2, SUC_lexicon_tags_27 },
    { 0x824ba8b2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72ba8b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1aba8bb, SUC_lexicon_tags_25 },
    { 0x9faba8bc, SUC_lexicon_tags_25 },
    { 0x9eeba8bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4ba8c2, SUC_lexicon_tags_47 },
    { 0x94cba8c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1aba8c5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8ba8c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa46ba8ca, SUC_lexicon_tags_10 },
    { 0xc8eba8cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2ba8cf, SUC_lexicon_tags_25 },
    { 0x4d2ba8d0, SUC_lexicon_tags_15 },
    { 0x7acba8cf, SUC_lexicon_tags_1 },
    { 0x1aba8d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6eba8d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38ba8d7, SUC_lexicon_tags_29 },
    { 0xa8eba8d7, SUC_lexicon_tags_32 },
    { 0x8a0ba8d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xafeba8db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9c8ba8dd, SUC_lexicon_tags_25 },
    { 0xc5eba8dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ba8e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2ba8e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4ba8ec, SUC_lexicon_tags_37 },
    { 0x6aeba8ec, SUC_lexicon_tags_70 },
    { 0xba0ba8ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfaba8f1, SUC_lexicon_tags_1 },
    { 0xd02ba8f2, SUC_lexicon_tags_37 },
    { 0xeaaba8f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x616ba8f6, SUC_lexicon_tags_43 },
    { 0x412ba8f6, SUC_lexicon_tags_1 },
    { 0x554ba8f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3b0ba8fa, SUC_lexicon_tags_47 },
    { 0x660ba8fb, SUC_lexicon_tags_2 },
    { 0x56cba8fc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834ba903, SUC_lexicon_tags_35 },
    { 0x69cba904, SUC_lexicon_tags_16 },
    { 0xd38ba905, SUC_lexicon_tags_14 },
    { 0xf3cba904, SUC_lexicon_tags_27 },
    { 0x5f4ba903, SUC_lexicon_tags_4 },
    { 0xb44ba908, SUC_lexicon_tags_63 },
    { 0x510ba908, SUC_lexicon_tags_38 },
    { 0x9f0ba90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50ba90c, SUC_lexicon_tags_48 },
    { 0xe78ba90d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x380ba910, SUC_lexicon_tags_1 },
    { 0x616ba910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366ba916, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506ba91d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252ba925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190ba929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe86ba92b, SUC_lexicon_tags_9 },
    { 0xf08ba92c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acba931, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x400ba933, SUC_lexicon_tags_17 },
    { 0x94eba933, SUC_lexicon_tags_7 },
    { 0xf3cba935, SUC_lexicon_tags_1 },
    { 0x43cba933, SUC_lexicon_tags_2 },
    { 0xc22ba935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e8ba939, SUC_lexicon_tags_14 },
    { 0x1eba93a, SUC_lexicon_tags_9 },
    { 0xaecba93a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x848ba93d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acba943, SUC_lexicon_tags_2 },
    { 0x39aba944, SUC_lexicon_tags_68 },
    { 0x25eba945, SUC_lexicon_tags_12 },
    { 0x2e4ba943, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe64ba948, SUC_lexicon_tags_54 },
    { 0x640ba949, SUC_lexicon_tags_11 },
    { 0xd1aba94a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cba94e, SUC_lexicon_tags_11 },
    { 0x742ba94f, SUC_lexicon_tags_11 },
    { 0x3e4ba94f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a6ba952, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x542ba955, SUC_lexicon_tags_7 },
    { 0xbe0ba956, SUC_lexicon_tags_8 },
    { 0xba2ba957, SUC_lexicon_tags_25 },
    { 0x460ba955, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2ba95f, SUC_lexicon_tags_747 },
    { 0xb78ba960, SUC_lexicon_tags_25 },
    { 0x768ba961, SUC_lexicon_tags_9 },
    { 0x816ba962, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7faba965, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4ba96f, SUC_lexicon_tags_49 },
    { 0x54cba970, SUC_lexicon_tags_2 },
    { 0x7e8ba971, SUC_lexicon_tags_1 },
    { 0xc82ba972, SUC_lexicon_tags_0 },
    { 0x2f4ba970, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc72ba975, SUC_lexicon_tags_14 },
    { 0xee4ba976, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8ba979, SUC_lexicon_tags_104 },
    { 0xd98ba97a, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150ba97f, SUC_lexicon_tags_21 },
    { 0x786ba980, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1daba982, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1b4ba984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ba987, SUC_lexicon_tags_12 },
    { 0x9caba988, SUC_lexicon_tags_77 },
    { 0xa6aba989, SUC_lexicon_tags_14 },
    { 0x464ba98a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91eba98d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92aba990, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50ba993, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eba996, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4ba999, SUC_lexicon_tags_29 },
    { 0x3eba99a, SUC_lexicon_tags_11 },
    { 0x202ba99a, SUC_lexicon_tags_36 },
    { 0x94ba99b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x410ba99f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50ba9a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x622ba9a4, SUC_lexicon_tags_1 },
    { 0x1daba9a5, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308ba9ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22ba9ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3d8ba9b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccba9b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0ba9b8, SUC_lexicon_tags_66 },
    { 0x22eba9b9, SUC_lexicon_tags_47 },
    { 0x926ba9ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24ba9be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4ba9c5, SUC_lexicon_tags_15 },
    { 0xf30ba9c6, SUC_lexicon_tags_27 },
    { 0x636ba9c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4b6ba9c9, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22ba9cd, SUC_lexicon_tags_4 },
    { 0x1deba9cd, SUC_lexicon_tags_37 },
    { 0xc26ba9ce, SUC_lexicon_tags_25 },
    { 0x92ba9ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59eba9d6, SUC_lexicon_tags_23 },
    { 0x198ba9d7, SUC_lexicon_tags_35 },
    { 0xe04ba9d8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aba9dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffcba9de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecba9e4, SUC_lexicon_tags_60 },
    { 0xcbaba9e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed2ba9e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x240ba9e9, SUC_lexicon_tags_1 },
    { 0xfd2ba9e9, SUC_lexicon_tags_25 },
    { 0x600ba9e9, SUC_lexicon_tags_0 },
    { 0xeeeba9e9, SUC_lexicon_tags_63 },
    { 0x480ba9ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52aba9f1, SUC_lexicon_tags_77 },
    { 0xa58ba9f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60ba9f8, SUC_lexicon_tags_11 },
    { 0xe38ba9f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x964ba9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x688ba9fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94baa00, SUC_lexicon_tags_1 },
    { 0xcb6baa01, SUC_lexicon_tags_1 },
    { 0xe7cbaa02, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x310baa05, SUC_lexicon_tags_14 },
    { 0xffcbaa06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x566baa08, SUC_lexicon_tags_9 },
    { 0xe02baa08, SUC_lexicon_tags_334 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40baa0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7abaa13, SUC_lexicon_tags_60 },
    { 0x7f4baa14, SUC_lexicon_tags_27 },
    { 0xad8baa15, SUC_lexicon_tags_11 },
    { 0xb00baa16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x796baa19, SUC_lexicon_tags_1 },
    { 0x2aebaa1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9abaa1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c0baa1e, SUC_lexicon_tags_17 },
    { 0x76ebaa1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ebaa22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x746baa24, SUC_lexicon_tags_25 },
    { 0x30abaa24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36baa2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4debaa30, SUC_lexicon_tags_37 },
    { 0x75ebaa31, SUC_lexicon_tags_7 },
    { 0x782baa31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacbaa38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284baa3d, SUC_lexicon_tags_77 },
    { 0x80abaa3d, SUC_lexicon_tags_37 },
    { 0x2d4baa3e, SUC_lexicon_tags_47 },
    { 0x1ecbaa3d, SUC_lexicon_tags_60 },
    { 0xeeabaa41, SUC_lexicon_tags_11 },
    { 0xc6ebaa3f, SUC_lexicon_tags_11 },
    { 0x802baa43, SUC_lexicon_tags_47 },
    { 0x510baa44, SUC_lexicon_tags_12 },
    { 0x7c2baa45, SUC_lexicon_tags_14 },
    { 0x4abaa41, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbaa4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc10baa4d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8baa56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebcbaa58, SUC_lexicon_tags_29 },
    { 0x67cbaa59, SUC_lexicon_tags_37 },
    { 0x1f0baa5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806baa5e, SUC_lexicon_tags_19 },
    { 0x8babaa5f, SUC_lexicon_tags_0 },
    { 0xc98baa5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x544baa63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2baa6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60baa71, SUC_lexicon_tags_45 },
    { 0x9eebaa72, SUC_lexicon_tags_2 },
    { 0xe5abaa73, SUC_lexicon_tags_7 },
    { 0x688baa74, SUC_lexicon_tags_1 },
    { 0x39abaa71, SUC_lexicon_tags_1 },
    { 0xd0cbaa71, SUC_lexicon_tags_25 },
    { 0x698baa77, SUC_lexicon_tags_37 },
    { 0xc66baa75, SUC_lexicon_tags_30 },
    { 0x394baa79, SUC_lexicon_tags_49 },
    { 0x3aabaa79, SUC_lexicon_tags_15 },
    { 0xd0ebaa7b, SUC_lexicon_tags_24 },
    { 0xdc2baa7b, SUC_lexicon_tags_84 },
    { 0xd8ebaa7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf80baa7f, SUC_lexicon_tags_37 },
    { 0x5a8baa80, SUC_lexicon_tags_27 },
    { 0xf62baa80, SUC_lexicon_tags_37 },
    { 0x500baa82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1b8baa84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0cbaa86, SUC_lexicon_tags_23 },
    { 0xd3ebaa86, SUC_lexicon_tags_26 },
    { 0xe1ebaa87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x388baa8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960baa90, SUC_lexicon_tags_27 },
    { 0x80abaa91, SUC_lexicon_tags_23 },
    { 0x9fcbaa92, SUC_lexicon_tags_59 },
    { 0xb8abaa93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x390baa95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0baa99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f4baa9b, SUC_lexicon_tags_8 },
    { 0x858baa9c, SUC_lexicon_tags_26 },
    { 0xfe0baa9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc90baa9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6debaaa1, SUC_lexicon_tags_27 },
    { 0xed6baaa2, SUC_lexicon_tags_23 },
    { 0x8bcbaaa2, SUC_lexicon_tags_23 },
    { 0x2b8baaa3, SUC_lexicon_tags_25 },
    { 0x8cbaaa2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe78baaa7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x94baaaa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6abaab4, SUC_lexicon_tags_7 },
    { 0x110baab5, SUC_lexicon_tags_6 },
    { 0x140baab5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa58baab8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcbaabb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6babaabe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124baac2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260baac7, SUC_lexicon_tags_11 },
    { 0x5a6baac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf34baaca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2ebaacc, SUC_lexicon_tags_21 },
    { 0xd20baacc, SUC_lexicon_tags_11 },
    { 0xab2baace, SUC_lexicon_tags_25 },
    { 0x70ebaacf, SUC_lexicon_tags_14 },
    { 0xecebaace, SUC_lexicon_tags_24 },
    { 0xb9cbaad0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ebaad7, SUC_lexicon_tags_7 },
    { 0xf0ebaad8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2cbaada, SUC_lexicon_tags_37 },
    { 0x672baada, SUC_lexicon_tags_25 },
    { 0x8c4baadc, SUC_lexicon_tags_10 },
    { 0xba4baadc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc08baadf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42baae7, SUC_lexicon_tags_91 },
    { 0x980baae8, SUC_lexicon_tags_17 },
    { 0xc70baae8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aebaaed, SUC_lexicon_tags_83 },
    { 0xc6abaaee, SUC_lexicon_tags_17 },
    { 0x316baaef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cbaaf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654baaf7, SUC_lexicon_tags_48 },
    { 0xbd6baaf8, SUC_lexicon_tags_15 },
    { 0x90ebaaf8, SUC_lexicon_tags_37 },
    { 0xc62baafa, SUC_lexicon_tags_21 },
    { 0x60ebaafb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990bab02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbab08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1abab0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2bab11, SUC_lexicon_tags_6 },
    { 0x8d4bab12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4ebab14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0bab1b, SUC_lexicon_tags_7 },
    { 0x52bab1b, SUC_lexicon_tags_17 },
    { 0x62abab1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8bab24, SUC_lexicon_tags_7 },
    { 0xf14bab25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x46cbab28, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x550bab2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334bab34, SUC_lexicon_tags_38 },
    { 0x132bab35, SUC_lexicon_tags_23 },
    { 0xa84bab36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876bab3c, SUC_lexicon_tags_43 },
    { 0x41abab3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x324bab40, SUC_lexicon_tags_30 },
    { 0xd5ebab41, SUC_lexicon_tags_14 },
    { 0x7c0bab41, SUC_lexicon_tags_4 },
    { 0x9d0bab43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a4bab45, SUC_lexicon_tags_9 },
    { 0x5d8bab46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6bab4b, SUC_lexicon_tags_45 },
    { 0xa9cbab4c, SUC_lexicon_tags_26 },
    { 0x314bab4d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2abab52, SUC_lexicon_tags_25 },
    { 0xedabab53, SUC_lexicon_tags_7 },
    { 0xd90bab53, SUC_lexicon_tags_25 },
    { 0x46bab53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x54abab57, SUC_lexicon_tags_11 },
    { 0xeecbab58, SUC_lexicon_tags_49 },
    { 0x5cbab59, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ebab5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6eebab60, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4bab6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4bab73, SUC_lexicon_tags_37 },
    { 0x59cbab74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x914bab76, SUC_lexicon_tags_31 },
    { 0xf0ebab77, SUC_lexicon_tags_32 },
    { 0x476bab77, SUC_lexicon_tags_1 },
    { 0x53abab79, SUC_lexicon_tags_22 },
    { 0x360bab77, SUC_lexicon_tags_87 },
    { 0xa7cbab79, SUC_lexicon_tags_49 },
    { 0xa9ebab7c, SUC_lexicon_tags_0 },
    { 0x302bab7c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54bab85, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dabab8a, SUC_lexicon_tags_63 },
    { 0x87abab8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfabab8d, SUC_lexicon_tags_37 },
    { 0x9d8bab8d, SUC_lexicon_tags_32 },
    { 0xfe2bab8f, SUC_lexicon_tags_27 },
    { 0xb06bab90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78bab92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4babab94, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0xbf4bab96, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x998bab98, SUC_lexicon_tags_0 },
    { 0x2cebab98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcbab9d, SUC_lexicon_tags_27 },
    { 0x708bab9e, SUC_lexicon_tags_12 },
    { 0x95ebab9f, SUC_lexicon_tags_17 },
    { 0x86baba0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x526baba3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa50baba5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcbaba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ababad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x798babaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50babb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ababb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cbabb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cbabbd, SUC_lexicon_tags_11 },
    { 0x402babbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x114babc0, SUC_lexicon_tags_9 },
    { 0x5cbabc1, SUC_lexicon_tags_23 },
    { 0xdc6babc2, SUC_lexicon_tags_12 },
    { 0x712babc2, SUC_lexicon_tags_17 },
    { 0x4eababc3, SUC_lexicon_tags_33 },
    { 0xcacbabc3, SUC_lexicon_tags_7 },
    { 0x1aebabc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694babca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x532babcd, SUC_lexicon_tags_77 },
    { 0x8c2babcd, SUC_lexicon_tags_27 },
    { 0x6fcbabcf, SUC_lexicon_tags_7 },
    { 0xd66babd0, SUC_lexicon_tags_63 },
    { 0x704babcf, SUC_lexicon_tags_12 },
    { 0xcb8babd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ababd6, SUC_lexicon_tags_37 },
    { 0x78ababd7, SUC_lexicon_tags_2 },
    { 0x834babd7, SUC_lexicon_tags_7 },
    { 0xd52babd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x58babdb, SUC_lexicon_tags_37 },
    { 0xd14babdb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacbabe2, SUC_lexicon_tags_11 },
    { 0xfa8babe3, SUC_lexicon_tags_56 },
    { 0x826babe4, SUC_lexicon_tags_0 },
    { 0x7acbabe5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4babe7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d4babe9, SUC_lexicon_tags_17 },
    { 0x758babe9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56babf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ebabfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4bac01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68bac04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefabac07, SUC_lexicon_tags_12 },
    { 0xecbac07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x17cbac0b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4bac11, SUC_lexicon_tags_1 },
    { 0x1d8bac12, SUC_lexicon_tags_0 },
    { 0xf74bac13, SUC_lexicon_tags_8 },
    { 0x468bac14, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebac1a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306bac20, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf18bac22, SUC_lexicon_tags_7 },
    { 0xdf0bac23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d2bac25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c8bac27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ebac2a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xe1cbac2c, SUC_lexicon_tags_38 },
    { 0x8e4bac2d, SUC_lexicon_tags_7 },
    { 0x7fcbac2c, SUC_lexicon_tags_39 },
    { 0x4f8bac2f, SUC_lexicon_tags_60 },
    { 0x1e4bac30, SUC_lexicon_tags_26 },
    { 0xff2bac30, SUC_lexicon_tags_57 },
    { 0xc20bac2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15abac36, SUC_lexicon_tags_28 },
    { 0x6dabac37, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656bac42, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x454bac44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x772bac47, SUC_lexicon_tags_15 },
    { 0x9bcbac48, SUC_lexicon_tags_45 },
    { 0x6f0bac49, SUC_lexicon_tags_151 },
    { 0x222bac49, SUC_lexicon_tags_17 },
    { 0x166bac4b, SUC_lexicon_tags_19 },
    { 0x4aabac4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6bac4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706bac53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6aebac55, SUC_lexicon_tags_46 },
    { 0xf5ebac55, SUC_lexicon_tags_23 },
    { 0xa16bac57, SUC_lexicon_tags_0 },
    { 0x4f4bac58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x806bac5a, SUC_lexicon_tags_9 },
    { 0x7e2bac5a, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xe98bac5d, SUC_lexicon_tags_1 },
    { 0xe1cbac5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82cbac61, SUC_lexicon_tags_6 },
    { 0x436bac62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bac68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ebac6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x326bac6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8bac76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x186bac78, SUC_lexicon_tags_7 },
    { 0x74bac78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x272bac7b, SUC_lexicon_tags_23 },
    { 0x404bac7c, SUC_lexicon_tags_37 },
    { 0xfacbac7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ebac81, SUC_lexicon_tags_235 },
    { 0xe40bac82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816bac89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108bac8e, SUC_lexicon_tags_14 },
    { 0x4a8bac8f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84bac92, SUC_lexicon_tags_47 },
    { 0xe08bac93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ebac97, SUC_lexicon_tags_17 },
    { 0xf0abac98, SUC_lexicon_tags_38 },
    { 0x7fcbac99, SUC_lexicon_tags_49 },
    { 0xb86bac9a, SUC_lexicon_tags_17 },
    { 0xc0bac9b, SUC_lexicon_tags_69 },
    { 0x414bac9c, SUC_lexicon_tags_17 },
    { 0x6b6bac9c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1e2bac9f, SUC_lexicon_tags_47 },
    { 0xad8baca0, SUC_lexicon_tags_27 },
    { 0x276baca1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bcbaca3, SUC_lexicon_tags_12 },
    { 0xc96baca3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa32baca6, SUC_lexicon_tags_1 },
    { 0xda8baca7, SUC_lexicon_tags_49 },
    { 0xbc4baca7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84bacb1, SUC_lexicon_tags_26 },
    { 0x1f2bacb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84bacb6, SUC_lexicon_tags_15 },
    { 0x1b8bacb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60bacbc, SUC_lexicon_tags_15 },
    { 0x61abacbc, SUC_lexicon_tags_46 },
    { 0xc90bacbe, SUC_lexicon_tags_22 },
    { 0xb54bacbc, SUC_lexicon_tags_0 },
    { 0x762bacc0, SUC_lexicon_tags_14 },
    { 0x1e0bacbd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6bacc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8abacc8, SUC_lexicon_tags_25 },
    { 0xb08bacc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5cbaccb, SUC_lexicon_tags_2 },
    { 0x1ccbaccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29abacd2, SUC_lexicon_tags_56 },
    { 0xa9ebacd2, SUC_lexicon_tags_11 },
    { 0x41cbacd4, SUC_lexicon_tags_14 },
    { 0xbf2bacd5, SUC_lexicon_tags_3 },
    { 0x6aebacd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cabacda, SUC_lexicon_tags_25 },
    { 0x5dabacdb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1abacde, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cbace1, SUC_lexicon_tags_45 },
    { 0x100bace2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288bace7, SUC_lexicon_tags_37 },
    { 0x63abace8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cbaceb, SUC_lexicon_tags_60 },
    { 0x9c8bacec, SUC_lexicon_tags_19 },
    { 0xfb0baced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0bacf1, SUC_lexicon_tags_43 },
    { 0x9d8bacf1, SUC_lexicon_tags_11 },
    { 0xabebacf3, SUC_lexicon_tags_46 },
    { 0xe50bacf4, SUC_lexicon_tags_7 },
    { 0xe28bacf3, SUC_lexicon_tags_18 },
    { 0x2c8bacf6, SUC_lexicon_tags_7 },
    { 0x3f2bacf5, SUC_lexicon_tags_15 },
    { 0x128bacf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abad03, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0bad06, SUC_lexicon_tags_109 },
    { 0x8f8bad07, SUC_lexicon_tags_27 },
    { 0x1c8bad08, SUC_lexicon_tags_12 },
    { 0xe06bad09, SUC_lexicon_tags_15 },
    { 0xe58bad0a, SUC_lexicon_tags_23 },
    { 0x740bad0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad0bad0d, SUC_lexicon_tags_11 },
    { 0x3fcbad0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8abad12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02bad15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x868bad17, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcbad1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58bad20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x722bad22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834bad27, SUC_lexicon_tags_45 },
    { 0x8ecbad28, SUC_lexicon_tags_59 },
    { 0x8aabad28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66bad2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2cbad30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcbad33, SUC_lexicon_tags_12 },
    { 0x2bcbad34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4bad37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7abad3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x474bad3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2bad42, SUC_lexicon_tags_47 },
    { 0x7c2bad43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eabad46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602bad4a, SUC_lexicon_tags_26 },
    { 0xacebad4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebad4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x220bad51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x694bad54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4bad58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7abad5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcbad5e, SUC_lexicon_tags_2 },
    { 0xc4cbad5f, SUC_lexicon_tags_42 },
    { 0x83ebad60, SUC_lexicon_tags_7 },
    { 0xbb2bad61, SUC_lexicon_tags_11 },
    { 0x56cbad5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4bad67, SUC_lexicon_tags_23 },
    { 0x9f0bad68, SUC_lexicon_tags_37 },
    { 0x2c8bad69, SUC_lexicon_tags_8 },
    { 0x1b2bad68, SUC_lexicon_tags_1 },
    { 0x9f6bad68, SUC_lexicon_tags_28 },
    { 0xfdcbad6c, SUC_lexicon_tags_38 },
    { 0x15abad6d, SUC_lexicon_tags_1 },
    { 0xe18bad69, SUC_lexicon_tags_12 },
    { 0x296bad68, SUC_lexicon_tags_8 },
    { 0x804bad70, SUC_lexicon_tags_25 },
    { 0xb64bad71, SUC_lexicon_tags_28 },
    { 0xce8bad71, SUC_lexicon_tags_2 },
    { 0x30cbad73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cbad78, SUC_lexicon_tags_63 },
    { 0x260bad79, SUC_lexicon_tags_25 },
    { 0x458bad7a, SUC_lexicon_tags_25 },
    { 0x3dcbad7a, SUC_lexicon_tags_2 },
    { 0x776bad79, SUC_lexicon_tags_47 },
    { 0xe4bad7d, SUC_lexicon_tags_2 },
    { 0xdcabad7a, SUC_lexicon_tags_11 },
    { 0x12bad7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92bad84, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452bad88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf6cbad8a, SUC_lexicon_tags_1 },
    { 0xd0abad8b, SUC_lexicon_tags_25 },
    { 0xdc6bad8c, SUC_lexicon_tags_600 },
    { 0xfb0bad8d, SUC_lexicon_tags_79 },
    { 0x7d2bad8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5d2bad90, SUC_lexicon_tags_29 },
    { 0xb96bad91, SUC_lexicon_tags_49 },
    { 0x656bad92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46bad97, SUC_lexicon_tags_12 },
    { 0xc38bad98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ebad9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62bada3, SUC_lexicon_tags_26 },
    { 0x222bada4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbebada7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ebadaa, SUC_lexicon_tags_47 },
    { 0xfb2badab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82badad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x288badb0, SUC_lexicon_tags_25 },
    { 0x838badb1, SUC_lexicon_tags_227 },
    { 0xfebadb1, SUC_lexicon_tags_37 },
    { 0xdd0badb2, SUC_lexicon_tags_16 },
    { 0x5e2badb4, SUC_lexicon_tags_17 },
    { 0x5fabadb2, SUC_lexicon_tags_27 },
    { 0xe40badb3, SUC_lexicon_tags_32 },
    { 0x558badb7, SUC_lexicon_tags_60 },
    { 0x6cbadb7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4badbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89abadc3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x37abadc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674badd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1abadd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50badd8, SUC_lexicon_tags_15 },
    { 0xf12badd9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4baddb, SUC_lexicon_tags_17 },
    { 0xbf6baddb, SUC_lexicon_tags_32 },
    { 0xe8cbaddd, SUC_lexicon_tags_8 },
    { 0xcecbadde, SUC_lexicon_tags_11 },
    { 0xca6baddf, SUC_lexicon_tags_91 },
    { 0xd30bade0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54bade4, SUC_lexicon_tags_1 },
    { 0x20bade5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bade8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114badec, SUC_lexicon_tags_2 },
    { 0xa78baded, SUC_lexicon_tags_17 },
    { 0xaf6baded, SUC_lexicon_tags_46 },
    { 0x3c8baded, SUC_lexicon_tags_2 },
    { 0x3ecbadf0, SUC_lexicon_tags_17 },
    { 0x5aabaded, SUC_lexicon_tags_15 },
    { 0x2d8badf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dabadf6, SUC_lexicon_tags_28 },
    { 0xd62badf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ebadfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dabadfe, SUC_lexicon_tags_39 },
    { 0x2d0badff, SUC_lexicon_tags_17 },
    { 0x64badfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc2bae02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeabae0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92abae12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4bae18, SUC_lexicon_tags_7 },
    { 0x6e4bae18, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80bae1c, SUC_lexicon_tags_15 },
    { 0x7c4bae1d, SUC_lexicon_tags_31 },
    { 0xb86bae1e, SUC_lexicon_tags_4 },
    { 0x59abae1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182bae25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880bae2f, SUC_lexicon_tags_158 },
    { 0x47ebae30, SUC_lexicon_tags_6 },
    { 0xd7ebae30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x752bae33, SUC_lexicon_tags_16 },
    { 0xb4ebae33, SUC_lexicon_tags_59 },
    { 0xaf8bae34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66bae3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3babae40, SUC_lexicon_tags_0 },
    { 0x830bae41, SUC_lexicon_tags_11 },
    { 0xf82bae40, SUC_lexicon_tags_8 },
    { 0x9bcbae42, SUC_lexicon_tags_37 },
    { 0x606bae41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2bae47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa82bae49, SUC_lexicon_tags_66 },
    { 0xe78bae4a, SUC_lexicon_tags_27 },
    { 0xa12bae4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x620bae4e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8bae53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x484bae56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde4bae59, SUC_lexicon_tags_19 },
    { 0x2bebae5a, SUC_lexicon_tags_31 },
    { 0x3f0bae5a, SUC_lexicon_tags_37 },
    { 0xa72bae5c, SUC_lexicon_tags_27 },
    { 0x266bae5c, SUC_lexicon_tags_17 },
    { 0x4eabae59, SUC_lexicon_tags_62 },
    { 0xfa2bae59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafebae64, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x70ebae66, SUC_lexicon_tags_23 },
    { 0xffebae67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02bae6b, SUC_lexicon_tags_25 },
    { 0x956bae6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4bae6f, SUC_lexicon_tags_7 },
    { 0xb4ebae70, SUC_lexicon_tags_47 },
    { 0x3babae6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6cbae73, SUC_lexicon_tags_23 },
    { 0x592bae74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd22bae76, SUC_lexicon_tags_25 },
    { 0xe26bae77, SUC_lexicon_tags_14 },
    { 0x57abae77, SUC_lexicon_tags_26 },
    { 0x9aebae79, SUC_lexicon_tags_28 },
    { 0xdd0bae7a, SUC_lexicon_tags_224 },
    { 0xe78bae7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f8bae7d, SUC_lexicon_tags_25 },
    { 0x40abae7e, SUC_lexicon_tags_25 },
    { 0xa1abae7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16bae82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x132bae84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f6bae86, SUC_lexicon_tags_14 },
    { 0xccebae86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x844bae8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2bae8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91ebae8e, SUC_lexicon_tags_47 },
    { 0xa8bae8f, SUC_lexicon_tags_26 },
    { 0x9dabae90, SUC_lexicon_tags_37 },
    { 0x1c0bae90, SUC_lexicon_tags_47 },
    { 0x95ebae90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4bae98, SUC_lexicon_tags_1 },
    { 0xe68bae99, SUC_lexicon_tags_9 },
    { 0xe3ebae98, SUC_lexicon_tags_37 },
    { 0x8d0bae99, SUC_lexicon_tags_43 },
    { 0x8c2bae9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bebaea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52baea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6baeab, SUC_lexicon_tags_5 },
    { 0xf1ebaeac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbaeb1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa30baeb3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08baeb6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10baeba, SUC_lexicon_tags_644 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cebaebd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ebaec7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546baecc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6abaecf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ebaed5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac4baed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8baedb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0baee7, SUC_lexicon_tags_22 },
    { 0xaa8baee7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18abaeea, SUC_lexicon_tags_5 },
    { 0x1fabaeeb, SUC_lexicon_tags_8 },
    { 0xe6baeec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2febaeee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4baef2, SUC_lexicon_tags_7 },
    { 0xbecbaef3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2baef6, SUC_lexicon_tags_63 },
    { 0xe24baef6, SUC_lexicon_tags_1 },
    { 0x9aebaef8, SUC_lexicon_tags_2 },
    { 0x89cbaef9, SUC_lexicon_tags_8 },
    { 0x76cbaef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b8baefc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf2baefe, SUC_lexicon_tags_17 },
    { 0xd48baeff, SUC_lexicon_tags_25 },
    { 0x316baeff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ebaf06, SUC_lexicon_tags_11 },
    { 0x728baf07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x640baf0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8baf13, SUC_lexicon_tags_11 },
    { 0xe80baf14, SUC_lexicon_tags_2 },
    { 0x762baf14, SUC_lexicon_tags_37 },
    { 0x6d8baf16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cbaf1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb24baf1f, SUC_lexicon_tags_47 },
    { 0x5b4baf20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2baf23, SUC_lexicon_tags_32 },
    { 0x18ebaf23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42baf28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8baf31, SUC_lexicon_tags_7 },
    { 0xeb0baf31, SUC_lexicon_tags_2 },
    { 0x2aebaf33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff6baf35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82baf39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2ebaf3b, SUC_lexicon_tags_11 },
    { 0xe22baf3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xceebaf3f, SUC_lexicon_tags_15 },
    { 0x7b6baf3f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2e6baf42, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496baf46, SUC_lexicon_tags_9 },
    { 0xc3cbaf47, SUC_lexicon_tags_30 },
    { 0x392baf47, SUC_lexicon_tags_7 },
    { 0x35cbaf49, SUC_lexicon_tags_17 },
    { 0x694baf49, SUC_lexicon_tags_11 },
    { 0x52baf4b, SUC_lexicon_tags_7 },
    { 0x8fcbaf4b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x666baf4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b6baf51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6baf5a, SUC_lexicon_tags_25 },
    { 0x9babaf5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6baf5f, SUC_lexicon_tags_7 },
    { 0x8a2baf60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0baf63, SUC_lexicon_tags_60 },
    { 0x38cbaf64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4cabaf66, SUC_lexicon_tags_1128 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abaf69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26baf73, SUC_lexicon_tags_38 },
    { 0x48abaf74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc4abaf76, SUC_lexicon_tags_0 },
    { 0x90cbaf77, SUC_lexicon_tags_14 },
    { 0x51ebaf78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750baf7d, SUC_lexicon_tags_1 },
    { 0x350baf7e, SUC_lexicon_tags_37 },
    { 0x3a8baf7f, SUC_lexicon_tags_17 },
    { 0x76abaf7d, SUC_lexicon_tags_11 },
    { 0x414baf7e, SUC_lexicon_tags_37 },
    { 0x45abaf80, SUC_lexicon_tags_33 },
    { 0x78ebaf83, SUC_lexicon_tags_32 },
    { 0x856baf84, SUC_lexicon_tags_33 },
    { 0xec8baf85, SUC_lexicon_tags_11 },
    { 0x6aabaf86, SUC_lexicon_tags_46 },
    { 0x5f0baf87, SUC_lexicon_tags_33 },
    { 0x4a4baf86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f0baf8a, SUC_lexicon_tags_11 },
    { 0x8c6baf8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cabaf90, SUC_lexicon_tags_17 },
    { 0x75abaf91, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe02baf94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x13ebaf96, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8baf9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacbaf9f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x436bafa1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8bafa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91abafaf, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60bafb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x998bafb4, SUC_lexicon_tags_33 },
    { 0xb9cbafb5, SUC_lexicon_tags_7 },
    { 0x63ebafb6, SUC_lexicon_tags_43 },
    { 0xf36bafb6, SUC_lexicon_tags_12 },
    { 0x56ebafb8, SUC_lexicon_tags_17 },
    { 0x7eebafb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97ebafbb, SUC_lexicon_tags_11 },
    { 0x3c8bafbc, SUC_lexicon_tags_2 },
    { 0xcaebafbc, SUC_lexicon_tags_15 },
    { 0x86ebafbd, SUC_lexicon_tags_17 },
    { 0x690bafbf, SUC_lexicon_tags_39 },
    { 0x7dabafbf, SUC_lexicon_tags_2 },
    { 0x64abafbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dcbafc3, SUC_lexicon_tags_37 },
    { 0x190bafc4, SUC_lexicon_tags_42 },
    { 0xea6bafc4, SUC_lexicon_tags_15 },
    { 0x942bafc5, SUC_lexicon_tags_11 },
    { 0x38cbafc5, SUC_lexicon_tags_65 },
    { 0xfe2bafc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0bafcd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0bafd3, SUC_lexicon_tags_47 },
    { 0x348bafd4, SUC_lexicon_tags_1 },
    { 0xe92bafd4, SUC_lexicon_tags_1 },
    { 0xf5abafd5, SUC_lexicon_tags_49 },
    { 0xafebafd4, SUC_lexicon_tags_6 },
    { 0x5dcbafd4, SUC_lexicon_tags_25 },
    { 0x4fcbafd3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x25cbafdb, SUC_lexicon_tags_17 },
    { 0xb28bafdc, SUC_lexicon_tags_39 },
    { 0xfa2bafdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62bafe2, SUC_lexicon_tags_11 },
    { 0x9dabafe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ebafed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x864baff0, SUC_lexicon_tags_29 },
    { 0xeacbaff0, SUC_lexicon_tags_15 },
    { 0x6d0baff0, SUC_lexicon_tags_7 },
    { 0x4c6baff3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24abaff7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccbaffd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaabb000, SUC_lexicon_tags_15 },
    { 0xe08bb001, SUC_lexicon_tags_12 },
    { 0xe44bb001, SUC_lexicon_tags_60 },
    { 0x616bb003, SUC_lexicon_tags_15 },
    { 0x556bb003, SUC_lexicon_tags_11 },
    { 0xab8bb005, SUC_lexicon_tags_37 },
    { 0xd74bb006, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x13cbb008, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64bb011, SUC_lexicon_tags_11 },
    { 0x9d2bb012, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x606bb014, SUC_lexicon_tags_12 },
    { 0xeaabb015, SUC_lexicon_tags_9 },
    { 0x8b6bb016, SUC_lexicon_tags_77 },
    { 0x9ccbb017, SUC_lexicon_tags_19 },
    { 0x20ebb018, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80abb01e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2bb021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bb025, SUC_lexicon_tags_17 },
    { 0x94cbb026, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0bb02e, SUC_lexicon_tags_19 },
    { 0x86bb02f, SUC_lexicon_tags_37 },
    { 0xc92bb030, SUC_lexicon_tags_45 },
    { 0x12bb030, SUC_lexicon_tags_62 },
    { 0xd7abb032, SUC_lexicon_tags_25 },
    { 0xa84bb030, SUC_lexicon_tags_4 },
    { 0xdd8bb034, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bebb037, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe52bb039, SUC_lexicon_tags_15 },
    { 0xa8cbb039, SUC_lexicon_tags_47 },
    { 0x276bb03b, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xebb03d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x346bb040, SUC_lexicon_tags_22 },
    { 0xffebb041, SUC_lexicon_tags_25 },
    { 0x16bb041, SUC_lexicon_tags_1 },
    { 0xdb6bb043, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x346bb045, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe00bb047, SUC_lexicon_tags_31 },
    { 0x62abb047, SUC_lexicon_tags_1 },
    { 0x5b2bb049, SUC_lexicon_tags_2 },
    { 0x836bb049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe8bb04c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ebb051, SUC_lexicon_tags_23 },
    { 0x490bb052, SUC_lexicon_tags_25 },
    { 0x986bb053, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8bb056, SUC_lexicon_tags_11 },
    { 0xbf6bb057, SUC_lexicon_tags_11 },
    { 0xa9cbb058, SUC_lexicon_tags_15 },
    { 0xd6abb059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcbb05c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xef4bb05e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebb062, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0bb066, SUC_lexicon_tags_17 },
    { 0xc30bb066, SUC_lexicon_tags_3 },
    { 0xdabb068, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacbb06f, SUC_lexicon_tags_49 },
    { 0x5d4bb06f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdabb07b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674bb081, SUC_lexicon_tags_19 },
    { 0x78bb082, SUC_lexicon_tags_11 },
    { 0x778bb083, SUC_lexicon_tags_43 },
    { 0xd7abb083, SUC_lexicon_tags_63 },
    { 0x9dabb082, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6bb08a, SUC_lexicon_tags_2 },
    { 0xb48bb08b, SUC_lexicon_tags_11 },
    { 0x9cabb08c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06bb090, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x292bb092, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36bb095, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xa9abb097, SUC_lexicon_tags_19 },
    { 0xdc8bb098, SUC_lexicon_tags_25 },
    { 0xdb6bb099, SUC_lexicon_tags_65 },
    { 0x29cbb09a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fabb09f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dabb0a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4bb0a5, SUC_lexicon_tags_34 },
    { 0x272bb0a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa88bb0a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cbb0b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ebb0b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcbb0bd, SUC_lexicon_tags_47 },
    { 0xe26bb0be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ebb0c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08bb0c7, SUC_lexicon_tags_30 },
    { 0xec4bb0c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aabb0cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcbb0cf, SUC_lexicon_tags_17 },
    { 0x544bb0d0, SUC_lexicon_tags_2 },
    { 0xd26bb0d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548bb0d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ebb0e1, SUC_lexicon_tags_15 },
    { 0x7f0bb0e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15ebb0e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ebb0e8, SUC_lexicon_tags_37 },
    { 0xb6abb0e8, SUC_lexicon_tags_55 },
    { 0xb36bb0e9, SUC_lexicon_tags_12 },
    { 0x3cbb0e8, SUC_lexicon_tags_8 },
    { 0xdb8bb0ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712bb0f0, SUC_lexicon_tags_7 },
    { 0xf80bb0f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc82bb0f3, SUC_lexicon_tags_33 },
    { 0xbcabb0f4, SUC_lexicon_tags_38 },
    { 0xd84bb0f4, SUC_lexicon_tags_43 },
    { 0x4aabb0f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146bb0fc, SUC_lexicon_tags_2 },
    { 0xbb8bb0fd, SUC_lexicon_tags_32 },
    { 0xbabb0fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90bb101, SUC_lexicon_tags_17 },
    { 0xad4bb102, SUC_lexicon_tags_17 },
    { 0xababb101, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ebb106, SUC_lexicon_tags_7 },
    { 0x93ebb106, SUC_lexicon_tags_37 },
    { 0x9febb108, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9abb10d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716bb113, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd4ebb115, SUC_lexicon_tags_82 },
    { 0xd84bb116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7abb11b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320bb122, SUC_lexicon_tags_1 },
    { 0x8debb123, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4bb126, SUC_lexicon_tags_133 },
    { 0x6b0bb126, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad8bb129, SUC_lexicon_tags_26 },
    { 0xbe0bb129, SUC_lexicon_tags_77 },
    { 0x686bb12a, SUC_lexicon_tags_7 },
    { 0x9febb12c, SUC_lexicon_tags_7 },
    { 0x530bb12a, SUC_lexicon_tags_16 },
    { 0x492bb12e, SUC_lexicon_tags_14 },
    { 0x738bb129, SUC_lexicon_tags_15 },
    { 0xd0bb130, SUC_lexicon_tags_8 },
    { 0xc58bb131, SUC_lexicon_tags_11 },
    { 0x102bb132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2bb135, SUC_lexicon_tags_49 },
    { 0xe86bb136, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x21cbb139, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94bb13e, SUC_lexicon_tags_7 },
    { 0x3ebb13e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ebb143, SUC_lexicon_tags_15 },
    { 0x398bb144, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ebb14c, SUC_lexicon_tags_77 },
    { 0x76ebb14d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x328bb150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3cbb152, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6ebb154, SUC_lexicon_tags_12 },
    { 0x2a0bb155, SUC_lexicon_tags_17 },
    { 0x1dabb156, SUC_lexicon_tags_11 },
    { 0x66cbb157, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbabb15a, SUC_lexicon_tags_12 },
    { 0x420bb15b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1ebb15d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba6bb15f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x586bb161, SUC_lexicon_tags_45 },
    { 0xab0bb161, SUC_lexicon_tags_8 },
    { 0xecbb162, SUC_lexicon_tags_17 },
    { 0xa3ebb163, SUC_lexicon_tags_25 },
    { 0xa96bb165, SUC_lexicon_tags_60 },
    { 0x502bb166, SUC_lexicon_tags_11 },
    { 0x56ebb161, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cbb16b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x886bb16d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f4bb16f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76cbb171, SUC_lexicon_tags_37 },
    { 0xef0bb172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b6bb174, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x94abb176, SUC_lexicon_tags_47 },
    { 0xb50bb177, SUC_lexicon_tags_29 },
    { 0x3e8bb176, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbebb17b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebb17f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3b4bb181, SUC_lexicon_tags_55 },
    { 0x19abb182, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x25abb184, SUC_lexicon_tags_3 },
    { 0xb84bb185, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1c0bb187, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4bb193, SUC_lexicon_tags_1 },
    { 0x5f0bb194, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdabb1a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d0bb1a5, SUC_lexicon_tags_6 },
    { 0xd12bb1a6, SUC_lexicon_tags_15 },
    { 0xce6bb1a7, SUC_lexicon_tags_28 },
    { 0xc76bb1a7, SUC_lexicon_tags_47 },
    { 0xc04bb1a9, SUC_lexicon_tags_55 },
    { 0x80ebb1a6, SUC_lexicon_tags_0 },
    { 0xb80bb1a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aabb1ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50abb1b0, SUC_lexicon_tags_15 },
    { 0x862bb1b1, SUC_lexicon_tags_26 },
    { 0xbb8bb1b2, SUC_lexicon_tags_167 },
    { 0xb50bb1b1, SUC_lexicon_tags_600 },
    { 0xafcbb1b3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bb1bc, SUC_lexicon_tags_387 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccbb1c4, SUC_lexicon_tags_0 },
    { 0xd1abb1c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6bb1c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22ebb1ca, SUC_lexicon_tags_45 },
    { 0x156bb1ca, SUC_lexicon_tags_47 },
    { 0x10cbb1cc, SUC_lexicon_tags_49 },
    { 0x9f0bb1cc, SUC_lexicon_tags_151 },
    { 0x500bb1ce, SUC_lexicon_tags_30 },
    { 0x65ebb1cf, SUC_lexicon_tags_15 },
    { 0xcdebb1d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf46bb1d2, SUC_lexicon_tags_4 },
    { 0xf80bb1d3, SUC_lexicon_tags_26 },
    { 0x26ebb1d3, SUC_lexicon_tags_2 },
    { 0x6d6bb1d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ccbb1d7, SUC_lexicon_tags_38 },
    { 0x10bb1d7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddabb1dc, SUC_lexicon_tags_170 },
    { 0x224bb1dd, SUC_lexicon_tags_1 },
    { 0x524bb1dd, SUC_lexicon_tags_23 },
    { 0xb22bb1df, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2bb1e3, SUC_lexicon_tags_25 },
    { 0xacbb1e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x802bb1e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccbb1ed, SUC_lexicon_tags_11 },
    { 0xafebb1ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb88bb1f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x600bb1f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x614bb1f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1c0bb1f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736bb1fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328bb202, SUC_lexicon_tags_7 },
    { 0x512bb203, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70bb207, SUC_lexicon_tags_15 },
    { 0xde0bb208, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706bb20e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cbb211, SUC_lexicon_tags_11 },
    { 0x326bb211, SUC_lexicon_tags_19 },
    { 0xa58bb212, SUC_lexicon_tags_17 },
    { 0x4e6bb214, SUC_lexicon_tags_25 },
    { 0xfcebb215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ebb218, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8eebb21a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1abb21e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cbb224, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b8bb226, SUC_lexicon_tags_61 },
    { 0x48cbb227, SUC_lexicon_tags_61 },
    { 0xbfcbb228, SUC_lexicon_tags_6 },
    { 0x1c0bb229, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51abb22b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876bb232, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654bb236, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ebb239, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x836bb23b, SUC_lexicon_tags_15 },
    { 0xdf0bb23c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb94bb23e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14bb242, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8abb24d, SUC_lexicon_tags_8 },
    { 0x1a4bb24d, SUC_lexicon_tags_37 },
    { 0x55ebb24d, SUC_lexicon_tags_14 },
    { 0x164bb24f, SUC_lexicon_tags_26 },
    { 0xf90bb24e, SUC_lexicon_tags_15 },
    { 0xe06bb252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48bb255, SUC_lexicon_tags_7 },
    { 0xf6abb256, SUC_lexicon_tags_37 },
    { 0xf9cbb257, SUC_lexicon_tags_12 },
    { 0xcdabb258, SUC_lexicon_tags_70 },
    { 0xa3cbb259, SUC_lexicon_tags_23 },
    { 0xad6bb25a, SUC_lexicon_tags_25 },
    { 0xb7ebb25b, SUC_lexicon_tags_47 },
    { 0xd02bb25a, SUC_lexicon_tags_1 },
    { 0x598bb25d, SUC_lexicon_tags_2 },
    { 0xf84bb25e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x516bb261, SUC_lexicon_tags_8 },
    { 0xb2cbb262, SUC_lexicon_tags_82 },
    { 0xe92bb262, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610bb26d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ebb270, SUC_lexicon_tags_66 },
    { 0x9c4bb271, SUC_lexicon_tags_0 },
    { 0x69abb272, SUC_lexicon_tags_14 },
    { 0x8acbb271, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x262bb275, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x840bb277, SUC_lexicon_tags_15 },
    { 0x69abb277, SUC_lexicon_tags_31 },
    { 0x80abb279, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x880bb27b, SUC_lexicon_tags_45 },
    { 0xd94bb27b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x460bb27e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecbb281, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c2bb283, SUC_lexicon_tags_37 },
    { 0x840bb284, SUC_lexicon_tags_15 },
    { 0x50abb285, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x950bb287, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa36bb289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80bb28c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ebb292, SUC_lexicon_tags_31 },
    { 0x602bb293, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x95abb295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658bb299, SUC_lexicon_tags_44 },
    { 0xd34bb299, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x386bb29d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33abb2a5, SUC_lexicon_tags_64 },
    { 0x59ebb2a6, SUC_lexicon_tags_25 },
    { 0x230bb2a5, SUC_lexicon_tags_38 },
    { 0xa36bb2a8, SUC_lexicon_tags_17 },
    { 0xffabb2a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70bb2af, SUC_lexicon_tags_174 },
    { 0x496bb2b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acbb2b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ebb2b9, SUC_lexicon_tags_12 },
    { 0xa2abb2ba, SUC_lexicon_tags_0 },
    { 0xb1cbb2ba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86bb2bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642bb2c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfdebb2c6, SUC_lexicon_tags_27 },
    { 0xfecbb2c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdb4bb2c9, SUC_lexicon_tags_1 },
    { 0xbe6bb2ca, SUC_lexicon_tags_19 },
    { 0x560bb2cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ebb2ce, SUC_lexicon_tags_47 },
    { 0xd90bb2cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218bb2d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6bb2d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4bb2dd, SUC_lexicon_tags_7 },
    { 0x7b6bb2de, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x28ebb2e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ebb2e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x188bb2e5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb12bb2e7, SUC_lexicon_tags_7 },
    { 0xa92bb2e7, SUC_lexicon_tags_57 },
    { 0x174bb2e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x532bb2ec, SUC_lexicon_tags_34 },
    { 0x478bb2ec, SUC_lexicon_tags_10 },
    { 0xbecbb2ed, SUC_lexicon_tags_47 },
    { 0xa56bb2ef, SUC_lexicon_tags_151 },
    { 0xda8bb2ef, SUC_lexicon_tags_12 },
    { 0x858bb2f0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbebb2f3, SUC_lexicon_tags_25 },
    { 0xb3ebb2f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14abb2f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd54bb2f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf82bb2fa, SUC_lexicon_tags_12 },
    { 0x39ebb2fa, SUC_lexicon_tags_690 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abb2ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2cabb301, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cbb308, SUC_lexicon_tags_133 },
    { 0x954bb309, SUC_lexicon_tags_41 },
    { 0x4a2bb308, SUC_lexicon_tags_4 },
    { 0xa76bb30b, SUC_lexicon_tags_2 },
    { 0x478bb308, SUC_lexicon_tags_5 },
    { 0x874bb30d, SUC_lexicon_tags_10 },
    { 0xe4bb30d, SUC_lexicon_tags_45 },
    { 0x2debb30f, SUC_lexicon_tags_17 },
    { 0xb50bb310, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ebb312, SUC_lexicon_tags_21 },
    { 0xcc6bb313, SUC_lexicon_tags_12 },
    { 0x556bb313, SUC_lexicon_tags_25 },
    { 0x132bb312, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cebb318, SUC_lexicon_tags_4 },
    { 0x87ebb319, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22bb31d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36bb325, SUC_lexicon_tags_12 },
    { 0x972bb326, SUC_lexicon_tags_15 },
    { 0x668bb327, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4bb329, SUC_lexicon_tags_11 },
    { 0xc28bb32a, SUC_lexicon_tags_3 },
    { 0xffcbb32a, SUC_lexicon_tags_421 },
    { 0, NULL },
    { 0x4e8bb32d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aebb332, SUC_lexicon_tags_45 },
    { 0x430bb333, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x452bb335, SUC_lexicon_tags_13 },
    { 0xd7abb335, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ebb33a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ebb33d, SUC_lexicon_tags_2 },
    { 0xa44bb33d, SUC_lexicon_tags_47 },
    { 0xef0bb33f, SUC_lexicon_tags_49 },
    { 0x3eebb33e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x172bb343, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71abb34e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bb355, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4bb358, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8bb35c, SUC_lexicon_tags_9 },
    { 0x902bb35c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86bb360, SUC_lexicon_tags_12 },
    { 0x674bb361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7abb366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430bb36d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2f8bb36f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ebb373, SUC_lexicon_tags_37 },
    { 0x6e8bb373, SUC_lexicon_tags_38 },
    { 0xc62bb375, SUC_lexicon_tags_60 },
    { 0xfd2bb373, SUC_lexicon_tags_17 },
    { 0x22abb377, SUC_lexicon_tags_37 },
    { 0xce8bb377, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x162bb37a, SUC_lexicon_tags_23 },
    { 0x7cebb37a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb56bb37d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ebb382, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bb38c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6bb38f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcebb391, SUC_lexicon_tags_2 },
    { 0x18bb391, SUC_lexicon_tags_14 },
    { 0xd86bb393, SUC_lexicon_tags_25 },
    { 0xcc4bb391, SUC_lexicon_tags_0 },
    { 0x2b8bb394, SUC_lexicon_tags_2 },
    { 0x3f8bb396, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476bb39d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38cbb3a2, SUC_lexicon_tags_25 },
    { 0xdecbb3a2, SUC_lexicon_tags_61 },
    { 0xb40bb3a2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x246bb3a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726bb3ab, SUC_lexicon_tags_0 },
    { 0xd00bb3ac, SUC_lexicon_tags_37 },
    { 0x85abb3ab, SUC_lexicon_tags_4 },
    { 0x4babb3ab, SUC_lexicon_tags_7 },
    { 0x968bb3ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x574bb3b1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x34abb3b3, SUC_lexicon_tags_15 },
    { 0xa16bb3b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5a2bb3b6, SUC_lexicon_tags_17 },
    { 0xfd2bb3b7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636bb3bc, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x70ebb3be, SUC_lexicon_tags_16 },
    { 0x6dcbb3be, SUC_lexicon_tags_12 },
    { 0xf54bb3c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbeebb3c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x76cbb3c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66cbb3c7, SUC_lexicon_tags_15 },
    { 0xb2abb3c7, SUC_lexicon_tags_26 },
    { 0xbecbb3c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x970bb3cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x992bb3cd, SUC_lexicon_tags_1 },
    { 0x1cabb3ce, SUC_lexicon_tags_97 },
    { 0x3cabb3ce, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa2cbb3d1, SUC_lexicon_tags_26 },
    { 0x3a2bb3d2, SUC_lexicon_tags_38 },
    { 0x4d8bb3d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6babb3d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d4bb3d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12abb3dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcbb3df, SUC_lexicon_tags_47 },
    { 0xed0bb3e0, SUC_lexicon_tags_61 },
    { 0x7dcbb3e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228bb3ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6abb3f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fabb3f7, SUC_lexicon_tags_11 },
    { 0xd14bb3f7, SUC_lexicon_tags_27 },
    { 0xe26bb3f9, SUC_lexicon_tags_7 },
    { 0x7b4bb3fa, SUC_lexicon_tags_4 },
    { 0xdbcbb3f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2bb405, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6b4bb407, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14bb40e, SUC_lexicon_tags_7 },
    { 0xe16bb40f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb86bb411, SUC_lexicon_tags_87 },
    { 0x99cbb412, SUC_lexicon_tags_2 },
    { 0xa0ebb413, SUC_lexicon_tags_38 },
    { 0x93abb414, SUC_lexicon_tags_34 },
    { 0xbccbb413, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x902bb417, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6b4bb419, SUC_lexicon_tags_17 },
    { 0x6eabb41a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bb41e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x12abb420, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b2bb422, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230bb427, SUC_lexicon_tags_11 },
    { 0x982bb427, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6bb42f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cbb434, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bb437, SUC_lexicon_tags_37 },
    { 0x3d4bb437, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e4bb43a, SUC_lexicon_tags_63 },
    { 0x26abb43b, SUC_lexicon_tags_38 },
    { 0x59ebb43c, SUC_lexicon_tags_37 },
    { 0xdb4bb43b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x968bb43f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0bb444, SUC_lexicon_tags_37 },
    { 0xccbb445, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494bb44c, SUC_lexicon_tags_37 },
    { 0x976bb44d, SUC_lexicon_tags_25 },
    { 0xb96bb44e, SUC_lexicon_tags_7 },
    { 0xbdebb44e, SUC_lexicon_tags_47 },
    { 0x32bb450, SUC_lexicon_tags_4 },
    { 0x2eabb451, SUC_lexicon_tags_23 },
    { 0x6b4bb452, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbb455, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61abb459, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cbb462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x388bb465, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20bb467, SUC_lexicon_tags_7 },
    { 0xb98bb468, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cbb46c, SUC_lexicon_tags_16 },
    { 0xbf2bb46c, SUC_lexicon_tags_197 },
    { 0xa9cbb46d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x616bb471, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496bb479, SUC_lexicon_tags_33 },
    { 0xea8bb47a, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0x324bb47d, SUC_lexicon_tags_7 },
    { 0x188bb47d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac4bb480, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1febb483, SUC_lexicon_tags_38 },
    { 0x86bb484, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0ebb486, SUC_lexicon_tags_17 },
    { 0x4dcbb487, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x386bb489, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x208bb48c, SUC_lexicon_tags_38 },
    { 0xb74bb48d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6bb490, SUC_lexicon_tags_57 },
    { 0xe40bb491, SUC_lexicon_tags_17 },
    { 0x162bb492, SUC_lexicon_tags_30 },
    { 0xf26bb493, SUC_lexicon_tags_49 },
    { 0xe60bb494, SUC_lexicon_tags_82 },
    { 0x546bb495, SUC_lexicon_tags_29 },
    { 0x7debb496, SUC_lexicon_tags_65 },
    { 0xf66bb497, SUC_lexicon_tags_38 },
    { 0x7eabb498, SUC_lexicon_tags_17 },
    { 0x5b8bb499, SUC_lexicon_tags_45 },
    { 0x73ebb499, SUC_lexicon_tags_17 },
    { 0x3febb498, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80bb4a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182bb4a9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfd6bb4ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906bb4af, SUC_lexicon_tags_15 },
    { 0x3b8bb4af, SUC_lexicon_tags_9 },
    { 0xf1cbb4af, SUC_lexicon_tags_1 },
    { 0xaf0bb4b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6bb4be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52bb4c1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4bb4c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf98bb4c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ebb4cc, SUC_lexicon_tags_37 },
    { 0x6a8bb4cd, SUC_lexicon_tags_25 },
    { 0x31ebb4ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188bb4d3, SUC_lexicon_tags_1 },
    { 0x8e4bb4d4, SUC_lexicon_tags_15 },
    { 0x9cbb4d5, SUC_lexicon_tags_37 },
    { 0xae4bb4d4, SUC_lexicon_tags_47 },
    { 0x686bb4d7, SUC_lexicon_tags_14 },
    { 0x2e0bb4d8, SUC_lexicon_tags_15 },
    { 0x6e6bb4d7, SUC_lexicon_tags_15 },
    { 0x1abb4d8, SUC_lexicon_tags_12 },
    { 0xe0abb4db, SUC_lexicon_tags_7 },
    { 0xbdcbb4dc, SUC_lexicon_tags_0 },
    { 0x170bb4d9, SUC_lexicon_tags_165 },
    { 0x8f0bb4d3, SUC_lexicon_tags_30 },
    { 0x64bb4df, SUC_lexicon_tags_2 },
    { 0x134bb4da, SUC_lexicon_tags_13 },
    { 0xb10bb4e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdcbb4e3, SUC_lexicon_tags_45 },
    { 0x868bb4e4, SUC_lexicon_tags_1 },
    { 0x430bb4e4, SUC_lexicon_tags_10 },
    { 0x8bebb4e6, SUC_lexicon_tags_82 },
    { 0x3a6bb4e7, SUC_lexicon_tags_79 },
    { 0xa5abb4e8, SUC_lexicon_tags_63 },
    { 0x96abb4e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc38bb4eb, SUC_lexicon_tags_12 },
    { 0xa4ebb4eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744bb4f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaebb4f4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4bb503, SUC_lexicon_tags_30 },
    { 0x5c4bb504, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcbb507, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa8bb509, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0abb51a, SUC_lexicon_tags_23 },
    { 0x972bb51a, SUC_lexicon_tags_37 },
    { 0xd8ebb51c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5eabb51e, SUC_lexicon_tags_47 },
    { 0x2eebb51e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cbb524, SUC_lexicon_tags_1 },
    { 0x1d8bb525, SUC_lexicon_tags_66 },
    { 0x358bb526, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x76bb529, SUC_lexicon_tags_33 },
    { 0x24ebb52a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8bb52e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe7cbb530, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8bb532, SUC_lexicon_tags_45 },
    { 0x1f0bb532, SUC_lexicon_tags_23 },
    { 0x4cbb534, SUC_lexicon_tags_2 },
    { 0xfe2bb535, SUC_lexicon_tags_1 },
    { 0x7eebb533, SUC_lexicon_tags_1 },
    { 0x9debb537, SUC_lexicon_tags_2 },
    { 0xb42bb536, SUC_lexicon_tags_25 },
    { 0xe3cbb535, SUC_lexicon_tags_40 },
    { 0x8dcbb53a, SUC_lexicon_tags_2 },
    { 0x800bb53b, SUC_lexicon_tags_37 },
    { 0x6a6bb53a, SUC_lexicon_tags_15 },
    { 0xb44bb53d, SUC_lexicon_tags_37 },
    { 0xeb6bb539, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa5abb540, SUC_lexicon_tags_14 },
    { 0x316bb540, SUC_lexicon_tags_19 },
    { 0xf64bb542, SUC_lexicon_tags_15 },
    { 0xfc2bb543, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6bb54a, SUC_lexicon_tags_8 },
    { 0x5c8bb54b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0bb54f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778bb555, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28bb558, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4febb55d, SUC_lexicon_tags_37 },
    { 0x208bb55d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680bb562, SUC_lexicon_tags_25 },
    { 0xceabb562, SUC_lexicon_tags_47 },
    { 0x6eebb564, SUC_lexicon_tags_15 },
    { 0x8abb564, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0bb56e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x94abb571, SUC_lexicon_tags_17 },
    { 0x6e6bb572, SUC_lexicon_tags_11 },
    { 0x2d6bb571, SUC_lexicon_tags_38 },
    { 0x3dabb574, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x610bb576, SUC_lexicon_tags_17 },
    { 0xc2cbb577, SUC_lexicon_tags_31 },
    { 0xbc2bb578, SUC_lexicon_tags_37 },
    { 0x324bb578, SUC_lexicon_tags_31 },
    { 0xf48bb57a, SUC_lexicon_tags_43 },
    { 0x504bb57b, SUC_lexicon_tags_37 },
    { 0x854bb57c, SUC_lexicon_tags_47 },
    { 0xed2bb57d, SUC_lexicon_tags_14 },
    { 0x9c6bb57e, SUC_lexicon_tags_9 },
    { 0x6fabb57e, SUC_lexicon_tags_37 },
    { 0x498bb57e, SUC_lexicon_tags_14 },
    { 0x532bb576, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ebb584, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd10bb586, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ebb589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2bb58c, SUC_lexicon_tags_87 },
    { 0x8c4bb58d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3cbb58f, SUC_lexicon_tags_0 },
    { 0x1a6bb590, SUC_lexicon_tags_12 },
    { 0x8d2bb591, SUC_lexicon_tags_25 },
    { 0x462bb592, SUC_lexicon_tags_15 },
    { 0x48abb592, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xceebb595, SUC_lexicon_tags_1 },
    { 0x1e6bb595, SUC_lexicon_tags_15 },
    { 0x49ebb595, SUC_lexicon_tags_13 },
    { 0x126bb598, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x55cbb59a, SUC_lexicon_tags_15 },
    { 0x8d0bb59a, SUC_lexicon_tags_12 },
    { 0x8eabb59c, SUC_lexicon_tags_12 },
    { 0x7d0bb59c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8bb5a3, SUC_lexicon_tags_1 },
    { 0x5cbb5a3, SUC_lexicon_tags_37 },
    { 0xae6bb5a5, SUC_lexicon_tags_1 },
    { 0xd3abb5a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bb5aa, SUC_lexicon_tags_11 },
    { 0x988bb5ab, SUC_lexicon_tags_7 },
    { 0x4a2bb5ac, SUC_lexicon_tags_11 },
    { 0x578bb5ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b2bb5af, SUC_lexicon_tags_10 },
    { 0x4f2bb5af, SUC_lexicon_tags_1 },
    { 0x51cbb5b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa12bb5b3, SUC_lexicon_tags_27 },
    { 0x648bb5b3, SUC_lexicon_tags_29 },
    { 0x822bb5b5, SUC_lexicon_tags_17 },
    { 0x2ebb5b6, SUC_lexicon_tags_65 },
    { 0x8bcbb5b7, SUC_lexicon_tags_33 },
    { 0xc62bb5b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638bb5be, SUC_lexicon_tags_15 },
    { 0x43abb5bf, SUC_lexicon_tags_15 },
    { 0x4bb5c0, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x1dcbb5c2, SUC_lexicon_tags_25 },
    { 0xfb6bb5c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x53ebb5c5, SUC_lexicon_tags_11 },
    { 0x842bb5c5, SUC_lexicon_tags_41 },
    { 0xbb0bb5c7, SUC_lexicon_tags_25 },
    { 0xea8bb5c8, SUC_lexicon_tags_37 },
    { 0x5c8bb5c9, SUC_lexicon_tags_9 },
    { 0xfaabb5ca, SUC_lexicon_tags_1 },
    { 0x74bb5cb, SUC_lexicon_tags_63 },
    { 0x964bb5c9, SUC_lexicon_tags_7 },
    { 0xda2bb5cd, SUC_lexicon_tags_19 },
    { 0x242bb5cd, SUC_lexicon_tags_7 },
    { 0x340bb5cf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x478bb5d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x512bb5d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34bb5d8, SUC_lexicon_tags_17 },
    { 0x1b8bb5d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbebb5db, SUC_lexicon_tags_14 },
    { 0x290bb5dc, SUC_lexicon_tags_7 },
    { 0xdaebb5dd, SUC_lexicon_tags_82 },
    { 0x9ebb5de, SUC_lexicon_tags_27 },
    { 0x41cbb5dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc42bb5e1, SUC_lexicon_tags_19 },
    { 0x450bb5e1, SUC_lexicon_tags_1 },
    { 0x260bb5e3, SUC_lexicon_tags_12 },
    { 0xd1cbb5e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8bb5e8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548bb5ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2bb5f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x500bb5f5, SUC_lexicon_tags_25 },
    { 0x4b2bb5f6, SUC_lexicon_tags_0 },
    { 0xe62bb5f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ebb5fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ebb5fe, SUC_lexicon_tags_2 },
    { 0xa3abb5fe, SUC_lexicon_tags_25 },
    { 0xba4bb600, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eebb603, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4debb609, SUC_lexicon_tags_12 },
    { 0xd6abb609, SUC_lexicon_tags_12 },
    { 0xcfabb60b, SUC_lexicon_tags_26 },
    { 0x6b0bb60b, SUC_lexicon_tags_15 },
    { 0xc70bb60c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd20bb60f, SUC_lexicon_tags_8 },
    { 0x25ebb610, SUC_lexicon_tags_37 },
    { 0xd8ebb610, SUC_lexicon_tags_2 },
    { 0x880bb612, SUC_lexicon_tags_37 },
    { 0xf12bb611, SUC_lexicon_tags_37 },
    { 0x942bb614, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4bb617, SUC_lexicon_tags_68 },
    { 0x3f0bb618, SUC_lexicon_tags_47 },
    { 0x26ebb619, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cbb61e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cbb623, SUC_lexicon_tags_7 },
    { 0x606bb624, SUC_lexicon_tags_37 },
    { 0x50cbb624, SUC_lexicon_tags_36 },
    { 0xaaabb624, SUC_lexicon_tags_9 },
    { 0xda0bb627, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2aebb629, SUC_lexicon_tags_11 },
    { 0x598bb62a, SUC_lexicon_tags_11 },
    { 0x220bb62a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8bb62e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476bb632, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454bb637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0bb639, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13abb63d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d6bb63f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88abb643, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6bb647, SUC_lexicon_tags_26 },
    { 0xababb648, SUC_lexicon_tags_39 },
    { 0xd46bb649, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bebb64c, SUC_lexicon_tags_17 },
    { 0xd7cbb64d, SUC_lexicon_tags_6 },
    { 0xddebb64e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcbb653, SUC_lexicon_tags_11 },
    { 0x5abb654, SUC_lexicon_tags_27 },
    { 0x3a0bb655, SUC_lexicon_tags_45 },
    { 0x2ecbb655, SUC_lexicon_tags_66 },
    { 0x63ebb657, SUC_lexicon_tags_34 },
    { 0x468bb657, SUC_lexicon_tags_23 },
    { 0x51abb656, SUC_lexicon_tags_364 },
    { 0x4fabb65a, SUC_lexicon_tags_33 },
    { 0x4e4bb654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ebb663, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10bb667, SUC_lexicon_tags_68 },
    { 0x88ebb667, SUC_lexicon_tags_7 },
    { 0xdacbb669, SUC_lexicon_tags_17 },
    { 0x210bb66a, SUC_lexicon_tags_29 },
    { 0x228bb66a, SUC_lexicon_tags_37 },
    { 0xf38bb66c, SUC_lexicon_tags_54 },
    { 0xc0bb66c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43cbb66f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x932bb671, SUC_lexicon_tags_47 },
    { 0xb44bb671, SUC_lexicon_tags_29 },
    { 0x90abb673, SUC_lexicon_tags_15 },
    { 0xd6abb672, SUC_lexicon_tags_10 },
    { 0xd34bb675, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4bb678, SUC_lexicon_tags_6 },
    { 0x5b6bb678, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8bb67d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x806bb67f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x412bb681, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4abb688, SUC_lexicon_tags_11 },
    { 0xd1cbb688, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9bebb68b, SUC_lexicon_tags_2 },
    { 0xe70bb68b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2bb68f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa58bb691, SUC_lexicon_tags_31 },
    { 0x7e2bb692, SUC_lexicon_tags_45 },
    { 0xa30bb693, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248bb697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8cbb699, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ebb6a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x73abb6a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48bb6b0, SUC_lexicon_tags_37 },
    { 0xf2bb6b1, SUC_lexicon_tags_0 },
    { 0x152bb6b2, SUC_lexicon_tags_85 },
    { 0x766bb6b3, SUC_lexicon_tags_15 },
    { 0x1fabb6b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ebb6b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x142bb6ba, SUC_lexicon_tags_45 },
    { 0x26cbb6bb, SUC_lexicon_tags_37 },
    { 0xeb6bb6bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346bb6c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb6c5, SUC_lexicon_tags_34 },
    { 0xbf8bb6c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x510bb6c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40bb6cd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626bb6d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x300bb6d8, SUC_lexicon_tags_68 },
    { 0x5febb6d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cebb6e0, SUC_lexicon_tags_32 },
    { 0x9a4bb6e1, SUC_lexicon_tags_0 },
    { 0x9f4bb6e2, SUC_lexicon_tags_12 },
    { 0x766bb6e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8bb6f2, SUC_lexicon_tags_1 },
    { 0x6a8bb6f3, SUC_lexicon_tags_49 },
    { 0x1e4bb6f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x358bb6f6, SUC_lexicon_tags_29 },
    { 0xb02bb6f7, SUC_lexicon_tags_14 },
    { 0xfcebb6f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7abb705, SUC_lexicon_tags_63 },
    { 0x8ebb706, SUC_lexicon_tags_48 },
    { 0x2f2bb707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x264bb709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61cbb70c, SUC_lexicon_tags_11 },
    { 0xe9abb70d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4bb714, SUC_lexicon_tags_4 },
    { 0xee8bb714, SUC_lexicon_tags_17 },
    { 0x25ebb715, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed0bb718, SUC_lexicon_tags_11 },
    { 0x5c2bb719, SUC_lexicon_tags_1 },
    { 0xb74bb71a, SUC_lexicon_tags_37 },
    { 0xb40bb71b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6debb722, SUC_lexicon_tags_83 },
    { 0xc08bb722, SUC_lexicon_tags_7 },
    { 0xe5cbb724, SUC_lexicon_tags_25 },
    { 0x9e6bb725, SUC_lexicon_tags_4 },
    { 0xd80bb724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8abb72b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe20bb72d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438bb733, SUC_lexicon_tags_23 },
    { 0xfa2bb733, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cbb737, SUC_lexicon_tags_77 },
    { 0xa46bb737, SUC_lexicon_tags_32 },
    { 0x294bb739, SUC_lexicon_tags_44 },
    { 0x496bb73a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6bb73e, SUC_lexicon_tags_10 },
    { 0x1c0bb73e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836bb746, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ebb74c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2bb74f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x47abb752, SUC_lexicon_tags_39 },
    { 0xcf4bb752, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bb757, SUC_lexicon_tags_33 },
    { 0x9a6bb758, SUC_lexicon_tags_31 },
    { 0xc40bb758, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7debb761, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc6ebb763, SUC_lexicon_tags_8 },
    { 0xf40bb763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2bb769, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x740bb76b, SUC_lexicon_tags_39 },
    { 0xf2bb76c, SUC_lexicon_tags_1 },
    { 0x162bb76d, SUC_lexicon_tags_67 },
    { 0x5d6bb76c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x324bb771, SUC_lexicon_tags_6 },
    { 0xe44bb772, SUC_lexicon_tags_8 },
    { 0xea0bb773, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x586bb776, SUC_lexicon_tags_15 },
    { 0xf4ebb777, SUC_lexicon_tags_27 },
    { 0xc90bb778, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48bb77b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6abb781, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80bb786, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x280bb789, SUC_lexicon_tags_28 },
    { 0x1babb78a, SUC_lexicon_tags_25 },
    { 0xaf6bb789, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x51abb78d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7fcbb78f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x936bb792, SUC_lexicon_tags_43 },
    { 0x992bb793, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x48cbb795, SUC_lexicon_tags_43 },
    { 0xea8bb795, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414bb79a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f8bb79c, SUC_lexicon_tags_7 },
    { 0xcf2bb79d, SUC_lexicon_tags_14 },
    { 0xbb8bb79d, SUC_lexicon_tags_37 },
    { 0x5eabb79c, SUC_lexicon_tags_7 },
    { 0x4febb79c, SUC_lexicon_tags_43 },
    { 0xe92bb7a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454bb7a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8bb7ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5a4bb7ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb7b4, SUC_lexicon_tags_3 },
    { 0xaeabb7b5, SUC_lexicon_tags_11 },
    { 0xca4bb7b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x570bb7b9, SUC_lexicon_tags_63 },
    { 0xcb8bb7ba, SUC_lexicon_tags_1 },
    { 0x254bb7ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786bb7c0, SUC_lexicon_tags_37 },
    { 0x1bebb7c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4bb7c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ebb7c9, SUC_lexicon_tags_7 },
    { 0xe32bb7ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x960bb7cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bb7d4, SUC_lexicon_tags_2 },
    { 0x820bb7d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42bb7df, SUC_lexicon_tags_23 },
    { 0x8fabb7e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2bb7e5, SUC_lexicon_tags_7 },
    { 0xaeabb7e6, SUC_lexicon_tags_25 },
    { 0xddabb7e7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x95ebb7e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ebb7ee, SUC_lexicon_tags_4 },
    { 0x812bb7ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0bb7f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccbb7f7, SUC_lexicon_tags_33 },
    { 0x6d8bb7f8, SUC_lexicon_tags_23 },
    { 0xc6cbb7f9, SUC_lexicon_tags_12 },
    { 0xf40bb7f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78bb801, SUC_lexicon_tags_107 },
    { 0xc70bb802, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16bb805, SUC_lexicon_tags_33 },
    { 0x5f4bb806, SUC_lexicon_tags_27 },
    { 0x572bb806, SUC_lexicon_tags_63 },
    { 0x1aabb805, SUC_lexicon_tags_63 },
    { 0x1c6bb806, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe94bb80b, SUC_lexicon_tags_1 },
    { 0xc00bb80c, SUC_lexicon_tags_25 },
    { 0x236bb80c, SUC_lexicon_tags_38 },
    { 0xba0bb80d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3c6bb810, SUC_lexicon_tags_25 },
    { 0x612bb811, SUC_lexicon_tags_21 },
    { 0xc00bb810, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966bb816, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8bb81c, SUC_lexicon_tags_7 },
    { 0x670bb81d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x19cbb820, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcbb827, SUC_lexicon_tags_25 },
    { 0x6febb827, SUC_lexicon_tags_7 },
    { 0xf78bb829, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ebb82d, SUC_lexicon_tags_25 },
    { 0xa36bb82e, SUC_lexicon_tags_29 },
    { 0x488bb82e, SUC_lexicon_tags_37 },
    { 0x2bebb82f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc20bb832, SUC_lexicon_tags_12 },
    { 0x15cbb833, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x908bb836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0bb839, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x462bb83b, SUC_lexicon_tags_60 },
    { 0x7d8bb83c, SUC_lexicon_tags_27 },
    { 0x92cbb83d, SUC_lexicon_tags_37 },
    { 0x7acbb83e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1c8bb840, SUC_lexicon_tags_7 },
    { 0x592bb841, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bb84a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f0bb84c, SUC_lexicon_tags_15 },
    { 0x938bb84c, SUC_lexicon_tags_0 },
    { 0x3a4bb84e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d4bb850, SUC_lexicon_tags_69 },
    { 0xb24bb851, SUC_lexicon_tags_33 },
    { 0xfaabb852, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cbb855, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a4bb857, SUC_lexicon_tags_15 },
    { 0xdc4bb858, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6bb85b, SUC_lexicon_tags_68 },
    { 0x5febb85c, SUC_lexicon_tags_17 },
    { 0x170bb85d, SUC_lexicon_tags_46 },
    { 0xe16bb85b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73abb860, SUC_lexicon_tags_23 },
    { 0xe7cbb860, SUC_lexicon_tags_11 },
    { 0x5a0bb860, SUC_lexicon_tags_27 },
    { 0xe24bb860, SUC_lexicon_tags_11 },
    { 0x474bb862, SUC_lexicon_tags_9 },
    { 0xb44bb865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78cbb868, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xacabb86a, SUC_lexicon_tags_49 },
    { 0xd60bb86b, SUC_lexicon_tags_28 },
    { 0xce4bb86b, SUC_lexicon_tags_25 },
    { 0x438bb86d, SUC_lexicon_tags_11 },
    { 0x9abb86d, SUC_lexicon_tags_77 },
    { 0x152bb86f, SUC_lexicon_tags_8 },
    { 0xa86bb86a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2ebb872, SUC_lexicon_tags_56 },
    { 0x4cabb872, SUC_lexicon_tags_11 },
    { 0xe4cbb874, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646bb87b, SUC_lexicon_tags_11 },
    { 0xa78bb87c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x726bb87e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0abb885, SUC_lexicon_tags_43 },
    { 0xc9abb886, SUC_lexicon_tags_17 },
    { 0xb5cbb885, SUC_lexicon_tags_33 },
    { 0xae2bb887, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4bb88c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112bb895, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2bb898, SUC_lexicon_tags_19 },
    { 0x55abb899, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x40abb89c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf46bb89e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50bb8a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4bb8a5, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cbb8ac, SUC_lexicon_tags_37 },
    { 0xfb4bb8ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470bb8b2, SUC_lexicon_tags_25 },
    { 0x708bb8b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97cbb8b5, SUC_lexicon_tags_79 },
    { 0xe7ebb8b5, SUC_lexicon_tags_15 },
    { 0xc1ebb8b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8bb8bc, SUC_lexicon_tags_19 },
    { 0x4f8bb8bc, SUC_lexicon_tags_57 },
    { 0xaacbb8be, SUC_lexicon_tags_47 },
    { 0xc76bb8bf, SUC_lexicon_tags_4 },
    { 0x1b6bb8c0, SUC_lexicon_tags_37 },
    { 0xd32bb8c1, SUC_lexicon_tags_47 },
    { 0xb2ebb8c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6bb8c8, SUC_lexicon_tags_11 },
    { 0xbd4bb8c9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8b6bb8cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592bb8d0, SUC_lexicon_tags_23 },
    { 0xc8abb8d0, SUC_lexicon_tags_19 },
    { 0x7b8bb8d1, SUC_lexicon_tags_84 },
    { 0xae2bb8d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6bb8e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x610bb8e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x152bb8e5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x32bb8e7, SUC_lexicon_tags_23 },
    { 0x31cbb8e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbb8ed, SUC_lexicon_tags_25 },
    { 0xaf0bb8ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc2cbb8f0, SUC_lexicon_tags_26 },
    { 0x864bb8f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cabb8f4, SUC_lexicon_tags_38 },
    { 0xf34bb8f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcbb8f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcbb8fb, SUC_lexicon_tags_25 },
    { 0x87cbb8fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb28bb8fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeccbb900, SUC_lexicon_tags_37 },
    { 0x628bb901, SUC_lexicon_tags_151 },
    { 0x6a8bb900, SUC_lexicon_tags_11 },
    { 0xfc6bb903, SUC_lexicon_tags_739 },
    { 0x7ccbb904, SUC_lexicon_tags_25 },
    { 0x84bb900, SUC_lexicon_tags_11 },
    { 0xeeabb900, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808bb90d, SUC_lexicon_tags_17 },
    { 0x99ebb90d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbfabb910, SUC_lexicon_tags_37 },
    { 0x68cbb910, SUC_lexicon_tags_1715 },
    { 0xa50bb912, SUC_lexicon_tags_37 },
    { 0x954bb912, SUC_lexicon_tags_7 },
    { 0xe5ebb914, SUC_lexicon_tags_15 },
    { 0xc5cbb915, SUC_lexicon_tags_8 },
    { 0xc6abb915, SUC_lexicon_tags_2 },
    { 0x856bb917, SUC_lexicon_tags_12 },
    { 0x6b2bb915, SUC_lexicon_tags_7 },
    { 0xdf0bb916, SUC_lexicon_tags_26 },
    { 0xa3ebb91a, SUC_lexicon_tags_33 },
    { 0x32abb919, SUC_lexicon_tags_12 },
    { 0xb56bb914, SUC_lexicon_tags_33 },
    { 0x772bb91d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434bb923, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe02bb925, SUC_lexicon_tags_49 },
    { 0x828bb925, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9dcbb928, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x844bb92a, SUC_lexicon_tags_3 },
    { 0x1f0bb92a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8bb92e, SUC_lexicon_tags_47 },
    { 0xae6bb92f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8c6bb931, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ebb938, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22bb93c, SUC_lexicon_tags_25 },
    { 0x15ebb93d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x224bb93f, SUC_lexicon_tags_82 },
    { 0x134bb940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10bb942, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x126bb945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30bb948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ebb94b, SUC_lexicon_tags_7 },
    { 0x764bb94c, SUC_lexicon_tags_19 },
    { 0xa6abb94c, SUC_lexicon_tags_2 },
    { 0x9b8bb94e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cbb956, SUC_lexicon_tags_37 },
    { 0x104bb956, SUC_lexicon_tags_60 },
    { 0x2d8bb958, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadcbb95c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46bb960, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x60abb963, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96bb967, SUC_lexicon_tags_33 },
    { 0xb5abb968, SUC_lexicon_tags_14 },
    { 0x248bb968, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x852bb96c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7febb970, SUC_lexicon_tags_12 },
    { 0x738bb971, SUC_lexicon_tags_40 },
    { 0x966bb972, SUC_lexicon_tags_48 },
    { 0xb8bb973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x694bb976, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebb979, SUC_lexicon_tags_79 },
    { 0x722bb97a, SUC_lexicon_tags_14 },
    { 0x66cbb97b, SUC_lexicon_tags_17 },
    { 0x372bb97c, SUC_lexicon_tags_45 },
    { 0x240bb97b, SUC_lexicon_tags_6 },
    { 0x2c8bb97e, SUC_lexicon_tags_14 },
    { 0xa50bb97f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbebb982, SUC_lexicon_tags_17 },
    { 0x3babb983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x802bb985, SUC_lexicon_tags_25 },
    { 0x6ccbb985, SUC_lexicon_tags_33 },
    { 0xb1ebb985, SUC_lexicon_tags_60 },
    { 0x88ebb987, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bb98d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164bb991, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbb994, SUC_lexicon_tags_25 },
    { 0xc6ebb995, SUC_lexicon_tags_17 },
    { 0x94abb995, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x248bb998, SUC_lexicon_tags_2 },
    { 0x952bb998, SUC_lexicon_tags_33 },
    { 0x78abb998, SUC_lexicon_tags_17 },
    { 0x8b6bb99b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2bb99f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840bb9a8, SUC_lexicon_tags_17 },
    { 0x902bb9a8, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xf60bb9ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbabb9b0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2f2bb9b2, SUC_lexicon_tags_2 },
    { 0x58bb9b3, SUC_lexicon_tags_14 },
    { 0xbdabb9b4, SUC_lexicon_tags_31 },
    { 0x4c0bb9b5, SUC_lexicon_tags_34 },
    { 0x58abb9b6, SUC_lexicon_tags_10 },
    { 0x90abb9b7, SUC_lexicon_tags_33 },
    { 0x8a0bb9b8, SUC_lexicon_tags_17 },
    { 0x192bb9b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4ebb9bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x158bb9be, SUC_lexicon_tags_45 },
    { 0xb7abb9bf, SUC_lexicon_tags_15 },
    { 0x9aebb9bf, SUC_lexicon_tags_17 },
    { 0x66abb9c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x930bb9c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x18cbb9c6, SUC_lexicon_tags_7 },
    { 0xf5abb9c6, SUC_lexicon_tags_21 },
    { 0x8b4bb9c6, SUC_lexicon_tags_7 },
    { 0xb6bb9c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922bb9cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x944bb9cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x510bb9d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb36bb9d3, SUC_lexicon_tags_25 },
    { 0xe62bb9d4, SUC_lexicon_tags_14 },
    { 0x400bb9d5, SUC_lexicon_tags_13 },
    { 0x628bb9d6, SUC_lexicon_tags_15 },
    { 0x1d6bb9d4, SUC_lexicon_tags_1338 },
    { 0xaebb9d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2bb9de, SUC_lexicon_tags_2 },
    { 0x71abb9df, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6bb9e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f8bb9e4, SUC_lexicon_tags_37 },
    { 0x846bb9e4, SUC_lexicon_tags_11 },
    { 0x900bb9e4, SUC_lexicon_tags_62 },
    { 0xd0abb9e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c2bb9e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6ebb9eb, SUC_lexicon_tags_1 },
    { 0x654bb9ec, SUC_lexicon_tags_37 },
    { 0xbd2bb9ed, SUC_lexicon_tags_31 },
    { 0x9debb9ed, SUC_lexicon_tags_17 },
    { 0xf86bb9ed, SUC_lexicon_tags_7 },
    { 0xab0bb9ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa10bb9f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf8bb9f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaabb9f7, SUC_lexicon_tags_66 },
    { 0x7e2bb9f8, SUC_lexicon_tags_15 },
    { 0x202bb9f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50bb9fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4abba03, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bba07, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe5ebba09, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80bba0c, SUC_lexicon_tags_107 },
    { 0x1a4bba0c, SUC_lexicon_tags_66 },
    { 0xcfebba0e, SUC_lexicon_tags_0 },
    { 0xaa2bba0e, SUC_lexicon_tags_32 },
    { 0xbe0bba0d, SUC_lexicon_tags_48 },
    { 0xaa4bba11, SUC_lexicon_tags_33 },
    { 0xa96bba12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cbba19, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84bba1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32bba20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9abba22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ebba25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfcebba27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8bba30, SUC_lexicon_tags_25 },
    { 0x36abba31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x67ebba33, SUC_lexicon_tags_63 },
    { 0xe42bba34, SUC_lexicon_tags_45 },
    { 0x38abba33, SUC_lexicon_tags_11 },
    { 0x9bcbba35, SUC_lexicon_tags_56 },
    { 0xb54bba37, SUC_lexicon_tags_33 },
    { 0xd32bba37, SUC_lexicon_tags_11 },
    { 0x6a4bba37, SUC_lexicon_tags_37 },
    { 0x106bba3a, SUC_lexicon_tags_7 },
    { 0xb52bba3b, SUC_lexicon_tags_39 },
    { 0x73ebba3c, SUC_lexicon_tags_39 },
    { 0xf3abba3d, SUC_lexicon_tags_33 },
    { 0x78ebba3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd0bba40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6bba44, SUC_lexicon_tags_21 },
    { 0xafebba44, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896bba52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38ebba54, SUC_lexicon_tags_7 },
    { 0xc54bba55, SUC_lexicon_tags_19 },
    { 0x27ebba56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x996bba59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae0bba5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2bba61, SUC_lexicon_tags_7 },
    { 0x8f2bba62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92ebba64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x682bba66, SUC_lexicon_tags_2 },
    { 0xc6bba66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x772bba69, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cabba6e, SUC_lexicon_tags_17 },
    { 0x5aabba6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c0bba71, SUC_lexicon_tags_31 },
    { 0xc9ebba72, SUC_lexicon_tags_61 },
    { 0xbe0bba73, SUC_lexicon_tags_17 },
    { 0x292bba74, SUC_lexicon_tags_17 },
    { 0xa44bba74, SUC_lexicon_tags_17 },
    { 0x3b6bba76, SUC_lexicon_tags_2 },
    { 0x228bba77, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eabba80, SUC_lexicon_tags_27 },
    { 0x534bba80, SUC_lexicon_tags_37 },
    { 0x1e6bba82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbebba84, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8abba92, SUC_lexicon_tags_17 },
    { 0xe10bba92, SUC_lexicon_tags_8 },
    { 0x8abba93, SUC_lexicon_tags_5 },
    { 0x32abba93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91cbba98, SUC_lexicon_tags_26 },
    { 0x84abba98, SUC_lexicon_tags_17 },
    { 0xa64bba9a, SUC_lexicon_tags_12 },
    { 0x45cbba98, SUC_lexicon_tags_8 },
    { 0xf38bba99, SUC_lexicon_tags_8 },
    { 0x878bba9d, SUC_lexicon_tags_17 },
    { 0xdccbba98, SUC_lexicon_tags_12 },
    { 0x96abba9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcbbaa2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bbaa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb40bbaa9, SUC_lexicon_tags_77 },
    { 0xe62bbaaa, SUC_lexicon_tags_15 },
    { 0xec4bbaa9, SUC_lexicon_tags_38 },
    { 0xbdabbaac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeabbaaf, SUC_lexicon_tags_10 },
    { 0xfecbbaaf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa94bbab2, SUC_lexicon_tags_37 },
    { 0xdabbab3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960bbabd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfebbabf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72bbac3, SUC_lexicon_tags_32 },
    { 0xc64bbac4, SUC_lexicon_tags_39 },
    { 0x24abbac4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7abbac7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ebbacd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbecbbad0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x38ebbad2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88bbad5, SUC_lexicon_tags_7 },
    { 0xe74bbad6, SUC_lexicon_tags_27 },
    { 0xa9cbbad6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb56bbad9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0bbade, SUC_lexicon_tags_37 },
    { 0xf10bbade, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bbae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe22bbae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8bbaea, SUC_lexicon_tags_37 },
    { 0xae2bbaea, SUC_lexicon_tags_11 },
    { 0xf5cbbaec, SUC_lexicon_tags_25 },
    { 0xc0ebbaec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1debbaf1, SUC_lexicon_tags_17 },
    { 0xee2bbaf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecbbaf7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100bbafb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcbbb02, SUC_lexicon_tags_25 },
    { 0x26ebbb03, SUC_lexicon_tags_49 },
    { 0x98ebbb04, SUC_lexicon_tags_39 },
    { 0x78ebbb05, SUC_lexicon_tags_23 },
    { 0x224bbb05, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4bbb0c, SUC_lexicon_tags_14 },
    { 0x476bbb0d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaf0bbb0f, SUC_lexicon_tags_6 },
    { 0x4c8bbb10, SUC_lexicon_tags_17 },
    { 0xcfabbb10, SUC_lexicon_tags_15 },
    { 0xb62bbb12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4bbb15, SUC_lexicon_tags_7 },
    { 0x432bbb15, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x500bbb19, SUC_lexicon_tags_15 },
    { 0xbd4bbb19, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x428bbb1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64bbb25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8bbb28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb50bbb2a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4bbb36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2debbb39, SUC_lexicon_tags_0 },
    { 0xcecbbb3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddebbb3e, SUC_lexicon_tags_17 },
    { 0x74bbb3f, SUC_lexicon_tags_17 },
    { 0x792bbb40, SUC_lexicon_tags_7 },
    { 0x2a0bbb3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbbb44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8bbb48, SUC_lexicon_tags_1 },
    { 0x872bbb49, SUC_lexicon_tags_12 },
    { 0xfeebbb48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdabbb4c, SUC_lexicon_tags_104 },
    { 0xf80bbb4d, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xea2bbb4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96abbb51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x106bbb54, SUC_lexicon_tags_17 },
    { 0xa90bbb55, SUC_lexicon_tags_17 },
    { 0x824bbb56, SUC_lexicon_tags_7 },
    { 0xc96bbb55, SUC_lexicon_tags_47 },
    { 0x936bbb55, SUC_lexicon_tags_29 },
    { 0x2bcbbb59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x270bbb5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ecbbb5d, SUC_lexicon_tags_32 },
    { 0x3c4bbb5e, SUC_lexicon_tags_1 },
    { 0xe12bbb5e, SUC_lexicon_tags_12 },
    { 0xee6bbb5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57ebbb62, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32bbb65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeacbbb67, SUC_lexicon_tags_0 },
    { 0x730bbb68, SUC_lexicon_tags_47 },
    { 0x2fcbbb67, SUC_lexicon_tags_28 },
    { 0xa68bbb6a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98bbb6d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464bbb71, SUC_lexicon_tags_52 },
    { 0x896bbb72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786bbb76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cbbb79, SUC_lexicon_tags_45 },
    { 0xb0cbbb7a, SUC_lexicon_tags_68 },
    { 0x63abbb7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2eabbb7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x636bbb7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85abbb82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eabbb87, SUC_lexicon_tags_27 },
    { 0xd0cbbb88, SUC_lexicon_tags_37 },
    { 0xd8abbb88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebbb8d, SUC_lexicon_tags_0 },
    { 0xd6abbb8e, SUC_lexicon_tags_2 },
    { 0x3d4bbb8f, SUC_lexicon_tags_0 },
    { 0xc4bbb90, SUC_lexicon_tags_7 },
    { 0xd50bbb8f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ebbb94, SUC_lexicon_tags_12 },
    { 0x1d0bbb95, SUC_lexicon_tags_15 },
    { 0xdecbbb96, SUC_lexicon_tags_15 },
    { 0x2babbb97, SUC_lexicon_tags_37 },
    { 0x9eabbb98, SUC_lexicon_tags_0 },
    { 0xd86bbb95, SUC_lexicon_tags_56 },
    { 0x3febbb9a, SUC_lexicon_tags_2 },
    { 0xfa8bbb9b, SUC_lexicon_tags_23 },
    { 0x2f2bbb9c, SUC_lexicon_tags_11 },
    { 0x4ebbb9b, SUC_lexicon_tags_19 },
    { 0xf42bbb9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0bbba2, SUC_lexicon_tags_60 },
    { 0xccbbba3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea2bbba5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd58bbba7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x230bbba9, SUC_lexicon_tags_60 },
    { 0x838bbbaa, SUC_lexicon_tags_43 },
    { 0x7f0bbbab, SUC_lexicon_tags_87 },
    { 0x852bbbaa, SUC_lexicon_tags_12 },
    { 0x350bbba9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bbbb1, SUC_lexicon_tags_13 },
    { 0x1cebbbb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6bbbb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x114bbbbb, SUC_lexicon_tags_2 },
    { 0x23abbbbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52cbbbbe, SUC_lexicon_tags_66 },
    { 0x878bbbbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90abbbc6, SUC_lexicon_tags_1 },
    { 0x4c4bbbc7, SUC_lexicon_tags_51 },
    { 0x466bbbc6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0bbbcc, SUC_lexicon_tags_9 },
    { 0x8c4bbbcd, SUC_lexicon_tags_2 },
    { 0x164bbbce, SUC_lexicon_tags_11 },
    { 0x95abbbcf, SUC_lexicon_tags_47 },
    { 0x4e2bbbd0, SUC_lexicon_tags_11 },
    { 0x800bbbd0, SUC_lexicon_tags_25 },
    { 0xd26bbbd2, SUC_lexicon_tags_39 },
    { 0x39abbbd3, SUC_lexicon_tags_37 },
    { 0xfa6bbbd4, SUC_lexicon_tags_10 },
    { 0x5f6bbbd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf34bbbd7, SUC_lexicon_tags_60 },
    { 0xde2bbbd8, SUC_lexicon_tags_12 },
    { 0x270bbbd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x38abbbdc, SUC_lexicon_tags_11 },
    { 0x4aabbbdd, SUC_lexicon_tags_31 },
    { 0x6bbbde, SUC_lexicon_tags_17 },
    { 0xe64bbbdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x39abbbe2, SUC_lexicon_tags_25 },
    { 0x47ebbbe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd12bbbe5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbbbe8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52bbbed, SUC_lexicon_tags_1 },
    { 0x6b0bbbee, SUC_lexicon_tags_2 },
    { 0x514bbbee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd26bbbf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaabbbf9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bbbfd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x420bbc00, SUC_lexicon_tags_63 },
    { 0x382bbc00, SUC_lexicon_tags_2 },
    { 0xab6bbc00, SUC_lexicon_tags_12 },
    { 0x6b8bbc03, SUC_lexicon_tags_47 },
    { 0x37abbc00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0bbc0c, SUC_lexicon_tags_14 },
    { 0x6ccbbc0d, SUC_lexicon_tags_26 },
    { 0xd5cbbc0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fcbbc10, SUC_lexicon_tags_14 },
    { 0xc7ebbc11, SUC_lexicon_tags_14 },
    { 0x616bbc12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6bbc15, SUC_lexicon_tags_49 },
    { 0xdb8bbc16, SUC_lexicon_tags_37 },
    { 0xf60bbc16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x828bbc1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x230bbc1d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36abbc21, SUC_lexicon_tags_108 },
    { 0x94abbc21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886bbc2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70bbc34, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x848bbc37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cbbc3a, SUC_lexicon_tags_80 },
    { 0x10cbbc3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4bbc3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cbbc45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x940bbc47, SUC_lexicon_tags_1 },
    { 0x94cbbc48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe86bbc4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0bbc4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7babbc53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434bbc58, SUC_lexicon_tags_16 },
    { 0x27cbbc58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00bbc5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa18bbc60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x442bbc63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e2bbc65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdabbc6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0bbc6e, SUC_lexicon_tags_2 },
    { 0x5e6bbc6f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x642bbc72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646bbc77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06bbc7b, SUC_lexicon_tags_27 },
    { 0x400bbc7c, SUC_lexicon_tags_60 },
    { 0xf9cbbc7d, SUC_lexicon_tags_63 },
    { 0xd66bbc7e, SUC_lexicon_tags_37 },
    { 0x1c2bbc7f, SUC_lexicon_tags_29 },
    { 0xe6ebbc7c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cbbc84, SUC_lexicon_tags_68 },
    { 0x54bbc85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x336bbc88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90cbbc8a, SUC_lexicon_tags_54 },
    { 0xd3ebbc8b, SUC_lexicon_tags_37 },
    { 0x3eebbc8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x19cbbc8e, SUC_lexicon_tags_25 },
    { 0xf22bbc8e, SUC_lexicon_tags_63 },
    { 0xe38bbc8e, SUC_lexicon_tags_11 },
    { 0xf50bbc90, SUC_lexicon_tags_49 },
    { 0xed4bbc8f, SUC_lexicon_tags_79 },
    { 0x24abbc93, SUC_lexicon_tags_39 },
    { 0x658bbc91, SUC_lexicon_tags_7 },
    { 0x17cbbc93, SUC_lexicon_tags_11 },
    { 0xc40bbc90, SUC_lexicon_tags_87 },
    { 0xe62bbc92, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb94bbc99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2bbc9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74bbca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ebbcab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cbbcb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ebbcb9, SUC_lexicon_tags_7 },
    { 0xbc6bbcba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xff6bbcbc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbcc0, SUC_lexicon_tags_17 },
    { 0xca8bbcc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb2bbcc3, SUC_lexicon_tags_17 },
    { 0x76cbbcc3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138bbcc9, SUC_lexicon_tags_15 },
    { 0x21abbcc9, SUC_lexicon_tags_49 },
    { 0x6a0bbcca, SUC_lexicon_tags_17 },
    { 0xa20bbccc, SUC_lexicon_tags_65 },
    { 0xc00bbccd, SUC_lexicon_tags_172 },
    { 0xecabbcce, SUC_lexicon_tags_1 },
    { 0xc54bbcce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ebbcd2, SUC_lexicon_tags_1 },
    { 0x53ebbcd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8bbcd6, SUC_lexicon_tags_14 },
    { 0x8eebbcd7, SUC_lexicon_tags_38 },
    { 0x64ebbcd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63abbce5, SUC_lexicon_tags_83 },
    { 0x434bbce5, SUC_lexicon_tags_25 },
    { 0x60abbce6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97abbcee, SUC_lexicon_tags_26 },
    { 0x6f4bbcee, SUC_lexicon_tags_21 },
    { 0x494bbcf0, SUC_lexicon_tags_12 },
    { 0x41ebbcf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f4bbcf3, SUC_lexicon_tags_8 },
    { 0x6c8bbcf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf2bbcf6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2bbcfe, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x862bbd00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22bbd06, SUC_lexicon_tags_49 },
    { 0x3b8bbd07, SUC_lexicon_tags_9 },
    { 0x78ebbd08, SUC_lexicon_tags_15 },
    { 0x1eabbd07, SUC_lexicon_tags_11 },
    { 0x4a2bbd0a, SUC_lexicon_tags_70 },
    { 0x424bbd0b, SUC_lexicon_tags_7 },
    { 0x65abbd0a, SUC_lexicon_tags_12 },
    { 0xc6cbbd0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc2bbd0f, SUC_lexicon_tags_11 },
    { 0xcbbd10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ebbd17, SUC_lexicon_tags_33 },
    { 0x1d0bbd18, SUC_lexicon_tags_27 },
    { 0xe08bbd19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cbbd1f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xa92bbd21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x834bbd24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f4bbd26, SUC_lexicon_tags_17 },
    { 0x992bbd26, SUC_lexicon_tags_70 },
    { 0xf78bbd28, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aebbd2f, SUC_lexicon_tags_15 },
    { 0xed4bbd2f, SUC_lexicon_tags_1 },
    { 0x47ebbd31, SUC_lexicon_tags_1 },
    { 0x2b8bbd30, SUC_lexicon_tags_43 },
    { 0xe28bbd2f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8bbd3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220bbd3f, SUC_lexicon_tags_13 },
    { 0xed2bbd40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x654bbd42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x296bbd45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bebbd4c, SUC_lexicon_tags_60 },
    { 0x896bbd4d, SUC_lexicon_tags_28 },
    { 0xa88bbd4e, SUC_lexicon_tags_7 },
    { 0xbb4bbd4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6bbd55, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730bbd59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cbbd5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x576bbd5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1dabbd61, SUC_lexicon_tags_5 },
    { 0x518bbd61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9abbd65, SUC_lexicon_tags_25 },
    { 0xf34bbd65, SUC_lexicon_tags_7 },
    { 0xd4ebbd67, SUC_lexicon_tags_11 },
    { 0x86bbd66, SUC_lexicon_tags_43 },
    { 0x3d4bbd66, SUC_lexicon_tags_1 },
    { 0x3ccbbd68, SUC_lexicon_tags_11 },
    { 0x87abbd6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd40bbd6d, SUC_lexicon_tags_23 },
    { 0xfabbd6e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x218bbd70, SUC_lexicon_tags_6 },
    { 0xaf2bbd71, SUC_lexicon_tags_65 },
    { 0xefcbbd72, SUC_lexicon_tags_25 },
    { 0x4cebbd73, SUC_lexicon_tags_17 },
    { 0xff8bbd74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1babbd78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180bbd7c, SUC_lexicon_tags_23 },
    { 0xcb6bbd7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956bbd85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ebbd8d, SUC_lexicon_tags_6 },
    { 0x40ebbd8e, SUC_lexicon_tags_27 },
    { 0x3d2bbd8f, SUC_lexicon_tags_11 },
    { 0x2d4bbd90, SUC_lexicon_tags_37 },
    { 0x668bbd90, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748bbd99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91abbda0, SUC_lexicon_tags_7 },
    { 0x9febbda1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e0bbda3, SUC_lexicon_tags_0 },
    { 0x42ebbda4, SUC_lexicon_tags_33 },
    { 0x8a6bbda3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb86bbda7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8bbdac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cbbdaf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaabbdb2, SUC_lexicon_tags_30 },
    { 0x2c2bbdb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x546bbdb5, SUC_lexicon_tags_3 },
    { 0x4d0bbdb6, SUC_lexicon_tags_11 },
    { 0x724bbdb7, SUC_lexicon_tags_14 },
    { 0x668bbdb7, SUC_lexicon_tags_25 },
    { 0x342bbdb6, SUC_lexicon_tags_38 },
    { 0x2c6bbdba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f2bbdbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4bbdc5, SUC_lexicon_tags_26 },
    { 0xbccbbdc6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582bbdca, SUC_lexicon_tags_12 },
    { 0xfc8bbdcb, SUC_lexicon_tags_27 },
    { 0x7b6bbdcb, SUC_lexicon_tags_61 },
    { 0xfa8bbdcb, SUC_lexicon_tags_17 },
    { 0x778bbdce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8bbdd3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f6bbdd5, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebbdd9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950bbde3, SUC_lexicon_tags_27 },
    { 0x1a4bbde4, SUC_lexicon_tags_62 },
    { 0xd72bbde5, SUC_lexicon_tags_25 },
    { 0x284bbde6, SUC_lexicon_tags_26 },
    { 0x84abbde5, SUC_lexicon_tags_60 },
    { 0x848bbde8, SUC_lexicon_tags_25 },
    { 0x506bbde8, SUC_lexicon_tags_60 },
    { 0xa6abbde9, SUC_lexicon_tags_60 },
    { 0x468bbde9, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0bbdf4, SUC_lexicon_tags_37 },
    { 0x6e2bbdf5, SUC_lexicon_tags_37 },
    { 0x846bbdf6, SUC_lexicon_tags_53 },
    { 0xc3abbdf6, SUC_lexicon_tags_37 },
    { 0x3f8bbdf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b8bbdfa, SUC_lexicon_tags_17 },
    { 0xd56bbdfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8bbe00, SUC_lexicon_tags_27 },
    { 0xdcebbe01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcbbe09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970bbe0f, SUC_lexicon_tags_38 },
    { 0xa54bbe10, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832bbe17, SUC_lexicon_tags_19 },
    { 0xbbabbe18, SUC_lexicon_tags_68 },
    { 0xe40bbe19, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb04bbe1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4bbe1e, SUC_lexicon_tags_23 },
    { 0xa4abbe1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1bcbbe21, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04bbe24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bcbbe26, SUC_lexicon_tags_2 },
    { 0x25cbbe27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cbbe2c, SUC_lexicon_tags_6 },
    { 0xf96bbe2d, SUC_lexicon_tags_43 },
    { 0xb32bbe2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2bbe33, SUC_lexicon_tags_11 },
    { 0xabcbbe34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dabbe38, SUC_lexicon_tags_177 },
    { 0x8d6bbe39, SUC_lexicon_tags_11 },
    { 0x898bbe38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4bbe3d, SUC_lexicon_tags_23 },
    { 0x42abbe3e, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660bbe42, SUC_lexicon_tags_37 },
    { 0x404bbe43, SUC_lexicon_tags_37 },
    { 0x366bbe44, SUC_lexicon_tags_19 },
    { 0x93abbe44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x174bbe48, SUC_lexicon_tags_74 },
    { 0xe2bbe49, SUC_lexicon_tags_0 },
    { 0x9c6bbe48, SUC_lexicon_tags_70 },
    { 0x6f2bbe48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcbbe4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ebbe54, SUC_lexicon_tags_25 },
    { 0x616bbe55, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd12bbe57, SUC_lexicon_tags_7 },
    { 0x300bbe57, SUC_lexicon_tags_7 },
    { 0xff6bbe57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cebbe5b, SUC_lexicon_tags_2 },
    { 0xbbabbe5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcecbbe5e, SUC_lexicon_tags_2 },
    { 0x180bbe5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe16bbe61, SUC_lexicon_tags_11 },
    { 0x4c4bbe62, SUC_lexicon_tags_25 },
    { 0xa08bbe63, SUC_lexicon_tags_25 },
    { 0x568bbe63, SUC_lexicon_tags_43 },
    { 0x830bbe64, SUC_lexicon_tags_25 },
    { 0x608bbe64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4bbe6e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6bbe71, SUC_lexicon_tags_47 },
    { 0xa4bbe72, SUC_lexicon_tags_25 },
    { 0x6f8bbe73, SUC_lexicon_tags_33 },
    { 0x750bbe74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592bbe79, SUC_lexicon_tags_19 },
    { 0x794bbe7a, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474bbe7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x702bbe82, SUC_lexicon_tags_15 },
    { 0x33abbe83, SUC_lexicon_tags_45 },
    { 0xbd2bbe84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ebbe8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c2bbe8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f4bbe8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa94bbe90, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cbbe98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68abbe9c, SUC_lexicon_tags_37 },
    { 0x940bbe9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cbbea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4bbea6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x532bbea8, SUC_lexicon_tags_43 },
    { 0x47abbea9, SUC_lexicon_tags_11 },
    { 0xfb8bbeaa, SUC_lexicon_tags_8 },
    { 0x3e6bbeab, SUC_lexicon_tags_4 },
    { 0x7a6bbea9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52bbeaf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7abbeb3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1b2bbeb5, SUC_lexicon_tags_19 },
    { 0x8fcbbeb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccbbebc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x38abbebf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4bbec7, SUC_lexicon_tags_63 },
    { 0xbe4bbec8, SUC_lexicon_tags_11 },
    { 0xe46bbec8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1acbbecc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270bbed6, SUC_lexicon_tags_12 },
    { 0xba6bbed7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xefcbbed9, SUC_lexicon_tags_48 },
    { 0xf2cbbeda, SUC_lexicon_tags_5 },
    { 0x246bbedb, SUC_lexicon_tags_17 },
    { 0xca4bbedc, SUC_lexicon_tags_2 },
    { 0x6acbbedc, SUC_lexicon_tags_25 },
    { 0x1ecbbedd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x842bbee0, SUC_lexicon_tags_13 },
    { 0x31abbee1, SUC_lexicon_tags_16 },
    { 0x8debbee0, SUC_lexicon_tags_17 },
    { 0xc3cbbee3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaabbee8, SUC_lexicon_tags_6 },
    { 0xf9ebbee9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccbbeec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2bbeef, SUC_lexicon_tags_7 },
    { 0x36ebbef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x148bbef2, SUC_lexicon_tags_1 },
    { 0xde6bbef2, SUC_lexicon_tags_13 },
    { 0x292bbef4, SUC_lexicon_tags_7 },
    { 0x82ebbef5, SUC_lexicon_tags_26 },
    { 0xb58bbef5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x882bbef9, SUC_lexicon_tags_47 },
    { 0x4bcbbefa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424bbf01, SUC_lexicon_tags_48 },
    { 0x140bbf02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2bbf05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd92bbf07, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0bbf0b, SUC_lexicon_tags_61 },
    { 0x55abbf0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cbbf11, SUC_lexicon_tags_1 },
    { 0x5cabbf12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7abbf14, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16bbf17, SUC_lexicon_tags_2 },
    { 0xf52bbf17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf0bbf1a, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcbbf1e, SUC_lexicon_tags_15 },
    { 0xb94bbf1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe06bbf21, SUC_lexicon_tags_0 },
    { 0x12abbf21, SUC_lexicon_tags_11 },
    { 0x8f8bbf21, SUC_lexicon_tags_35 },
    { 0x620bbf24, SUC_lexicon_tags_4 },
    { 0xd0bbf22, SUC_lexicon_tags_1 },
    { 0xeb4bbf25, SUC_lexicon_tags_27 },
    { 0x6cbbf21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbb4bbf29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0bbf2e, SUC_lexicon_tags_49 },
    { 0xac0bbf2f, SUC_lexicon_tags_28 },
    { 0xeaabbf2e, SUC_lexicon_tags_37 },
    { 0x4f2bbf31, SUC_lexicon_tags_11 },
    { 0x558bbf30, SUC_lexicon_tags_38 },
    { 0x76bbf33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42cbbf35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6bbf3a, SUC_lexicon_tags_17 },
    { 0xef4bbf3b, SUC_lexicon_tags_49 },
    { 0xda6bbf3b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6bbf48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x588bbf4b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x18ebbf4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bbf52, SUC_lexicon_tags_38 },
    { 0x892bbf53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ebbf58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aebbf5a, SUC_lexicon_tags_31 },
    { 0xbe2bbf5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6bbf66, SUC_lexicon_tags_11 },
    { 0xcb2bbf67, SUC_lexicon_tags_30 },
    { 0xad2bbf68, SUC_lexicon_tags_16 },
    { 0xffabbf69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeebbf6d, SUC_lexicon_tags_14 },
    { 0xf92bbf6d, SUC_lexicon_tags_30 },
    { 0x230bbf6f, SUC_lexicon_tags_16 },
    { 0xf70bbf6d, SUC_lexicon_tags_19 },
    { 0x9e6bbf71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ebbf77, SUC_lexicon_tags_12 },
    { 0xf32bbf77, SUC_lexicon_tags_47 },
    { 0x77abbf79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486bbf7d, SUC_lexicon_tags_87 },
    { 0x530bbf7e, SUC_lexicon_tags_23 },
    { 0x6e0bbf7d, SUC_lexicon_tags_37 },
    { 0xc0cbbf80, SUC_lexicon_tags_13 },
    { 0xc7cbbf81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x184bbf84, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x648bbf86, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xef2bbf88, SUC_lexicon_tags_79 },
    { 0xa36bbf88, SUC_lexicon_tags_79 },
    { 0xf3ebbf8a, SUC_lexicon_tags_27 },
    { 0x312bbf8b, SUC_lexicon_tags_9 },
    { 0x5d2bbf88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x562bbf8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812bbf92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4bbf97, SUC_lexicon_tags_30 },
    { 0x99cbbf97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccbbf9d, SUC_lexicon_tags_17 },
    { 0xaa6bbf9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x588bbfa1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0bbfab, SUC_lexicon_tags_11 },
    { 0x956bbfab, SUC_lexicon_tags_14 },
    { 0xed0bbfac, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x888bbfb0, SUC_lexicon_tags_559 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccbbfb3, SUC_lexicon_tags_25 },
    { 0xb70bbfb3, SUC_lexicon_tags_37 },
    { 0x81abbfb5, SUC_lexicon_tags_1 },
    { 0x1abbfb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x514bbfb9, SUC_lexicon_tags_25 },
    { 0xfc8bbfb9, SUC_lexicon_tags_55 },
    { 0xa20bbfbb, SUC_lexicon_tags_43 },
    { 0x74cbbfbc, SUC_lexicon_tags_18 },
    { 0xc58bbfbd, SUC_lexicon_tags_68 },
    { 0xaaabbfbb, SUC_lexicon_tags_17 },
    { 0x554bbfbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0bbfc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacbbfc9, SUC_lexicon_tags_45 },
    { 0x4dcbbfc9, SUC_lexicon_tags_37 },
    { 0xb30bbfcb, SUC_lexicon_tags_56 },
    { 0x208bbfcc, SUC_lexicon_tags_30 },
    { 0x4d4bbfcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02bbfd3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98bbfd6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ebbfd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x382bbfdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8bbfdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x770bbfe1, SUC_lexicon_tags_70 },
    { 0xb02bbfe1, SUC_lexicon_tags_27 },
    { 0x664bbfe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x322bbfe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2bbfef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572bbff3, SUC_lexicon_tags_1 },
    { 0x9a2bbff4, SUC_lexicon_tags_17 },
    { 0x3d6bbff5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd9abbff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b2bbff9, SUC_lexicon_tags_47 },
    { 0xf0ebbff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56bc003, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xec8bc005, SUC_lexicon_tags_2 },
    { 0x54cbc006, SUC_lexicon_tags_37 },
    { 0xab0bc007, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abc00b, SUC_lexicon_tags_60 },
    { 0xac6bc00c, SUC_lexicon_tags_37 },
    { 0xe44bc00b, SUC_lexicon_tags_17 },
    { 0xfb6bc00c, SUC_lexicon_tags_2 },
    { 0xf40bc00b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x778bc011, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x690bc014, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb2cbc016, SUC_lexicon_tags_454 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66bc019, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x644bc01c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976bc020, SUC_lexicon_tags_0 },
    { 0xd76bc021, SUC_lexicon_tags_31 },
    { 0xdf2bc022, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acbc027, SUC_lexicon_tags_37 },
    { 0x4f0bc028, SUC_lexicon_tags_25 },
    { 0xb60bc029, SUC_lexicon_tags_8 },
    { 0xca8bc02a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8bc02f, SUC_lexicon_tags_14 },
    { 0xfa2bc030, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dabc036, SUC_lexicon_tags_70 },
    { 0x934bc036, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cbc03e, SUC_lexicon_tags_12 },
    { 0xca4bc03f, SUC_lexicon_tags_32 },
    { 0xf40bc040, SUC_lexicon_tags_6 },
    { 0x59ebc03f, SUC_lexicon_tags_7 },
    { 0x34ebc042, SUC_lexicon_tags_56 },
    { 0xfbcbc043, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854bc048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0bc04d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x390bc050, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bc053, SUC_lexicon_tags_0 },
    { 0x1ebc053, SUC_lexicon_tags_0 },
    { 0x7ccbc055, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bebc05a, SUC_lexicon_tags_170 },
    { 0x772bc05a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x67abc05d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0bc060, SUC_lexicon_tags_37 },
    { 0xcc0bc061, SUC_lexicon_tags_11 },
    { 0x24ebc061, SUC_lexicon_tags_31 },
    { 0x408bc061, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70bc065, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26ebc067, SUC_lexicon_tags_28 },
    { 0x9b0bc068, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704bc06d, SUC_lexicon_tags_25 },
    { 0x122bc06e, SUC_lexicon_tags_7 },
    { 0x368bc06f, SUC_lexicon_tags_55 },
    { 0x816bc06f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb4bc072, SUC_lexicon_tags_7 },
    { 0xf74bc073, SUC_lexicon_tags_29 },
    { 0x900bc074, SUC_lexicon_tags_37 },
    { 0x620bc075, SUC_lexicon_tags_31 },
    { 0x75cbc073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abc07b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1abc082, SUC_lexicon_tags_11 },
    { 0x16abc083, SUC_lexicon_tags_70 },
    { 0x9dcbc084, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1febc089, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x29ebc08b, SUC_lexicon_tags_87 },
    { 0x2c2bc08b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfebc08e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x39cbc090, SUC_lexicon_tags_65 },
    { 0x52abc090, SUC_lexicon_tags_12 },
    { 0x4e2bc092, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4bc095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabc099, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8bc09e, SUC_lexicon_tags_49 },
    { 0xee2bc09e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2bc0a3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2bc0a6, SUC_lexicon_tags_8 },
    { 0x9cabc0a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5febc0a9, SUC_lexicon_tags_11 },
    { 0x91abc0a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc0b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466bc0bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefcbc0bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6bc0c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748bc0c4, SUC_lexicon_tags_19 },
    { 0xadabc0c5, SUC_lexicon_tags_33 },
    { 0xffabc0c6, SUC_lexicon_tags_27 },
    { 0xa94bc0c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb60bc0c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00bc0ce, SUC_lexicon_tags_33 },
    { 0x490bc0cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66bc0d3, SUC_lexicon_tags_9 },
    { 0x400bc0d3, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x972bc0d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2bc0e1, SUC_lexicon_tags_12 },
    { 0x65abc0e2, SUC_lexicon_tags_29 },
    { 0x8d2bc0e3, SUC_lexicon_tags_12 },
    { 0x88cbc0e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdebc0e7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x228bc0e9, SUC_lexicon_tags_1 },
    { 0xb8bc0ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480bc0f2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1a8bc0f4, SUC_lexicon_tags_11 },
    { 0x6fabc0f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccbc0f9, SUC_lexicon_tags_25 },
    { 0x48bc0fa, SUC_lexicon_tags_65 },
    { 0x6e4bc0fb, SUC_lexicon_tags_32 },
    { 0x202bc0fc, SUC_lexicon_tags_7 },
    { 0x514bc0fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x790bc100, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf70bc102, SUC_lexicon_tags_17 },
    { 0xa56bc103, SUC_lexicon_tags_1 },
    { 0x36abc103, SUC_lexicon_tags_170 },
    { 0x9dcbc105, SUC_lexicon_tags_0 },
    { 0xa1cbc106, SUC_lexicon_tags_17 },
    { 0xc44bc103, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2bc10a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66bc111, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7acbc113, SUC_lexicon_tags_3 },
    { 0xa0bc114, SUC_lexicon_tags_87 },
    { 0x9e4bc115, SUC_lexicon_tags_14 },
    { 0x738bc116, SUC_lexicon_tags_1 },
    { 0xebc115, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x90abc11a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebc11f, SUC_lexicon_tags_37 },
    { 0x43ebc120, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8bc127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc74bc129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xadabc12c, SUC_lexicon_tags_49 },
    { 0xdf4bc12d, SUC_lexicon_tags_5 },
    { 0xe58bc12e, SUC_lexicon_tags_564 },
    { 0x40bc12c, SUC_lexicon_tags_37 },
    { 0xb90bc130, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ebc133, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8bc136, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1abc139, SUC_lexicon_tags_33 },
    { 0x738bc13a, SUC_lexicon_tags_68 },
    { 0xe54bc13b, SUC_lexicon_tags_7 },
    { 0x332bc13b, SUC_lexicon_tags_29 },
    { 0xa34bc13d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18bc140, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xed2bc142, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4bc148, SUC_lexicon_tags_12 },
    { 0x780bc149, SUC_lexicon_tags_14 },
    { 0xb3ebc149, SUC_lexicon_tags_15 },
    { 0xb00bc14b, SUC_lexicon_tags_25 },
    { 0x684bc14c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8bc156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6abc159, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecbc15d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90bc161, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0x78bc163, SUC_lexicon_tags_37 },
    { 0x2e8bc164, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x760bc166, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82bc16d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc2bc16f, SUC_lexicon_tags_33 },
    { 0x6abc170, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816bc178, SUC_lexicon_tags_35 },
    { 0xb1ebc179, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994bc17d, SUC_lexicon_tags_8 },
    { 0x28ebc17e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5febc180, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x766bc183, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x590bc186, SUC_lexicon_tags_577 },
    { 0xf98bc186, SUC_lexicon_tags_33 },
    { 0x544bc186, SUC_lexicon_tags_11 },
    { 0xdf8bc189, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc60bc18b, SUC_lexicon_tags_47 },
    { 0x5dcbc18c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x998bc18e, SUC_lexicon_tags_63 },
    { 0x268bc18f, SUC_lexicon_tags_17 },
    { 0x470bc18e, SUC_lexicon_tags_25 },
    { 0x4cabc191, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf60bc193, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76bc199, SUC_lexicon_tags_7 },
    { 0x910bc19a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x838bc19d, SUC_lexicon_tags_68 },
    { 0xaecbc19e, SUC_lexicon_tags_27 },
    { 0x39cbc19f, SUC_lexicon_tags_7 },
    { 0x26ebc1a0, SUC_lexicon_tags_26 },
    { 0xa62bc1a1, SUC_lexicon_tags_79 },
    { 0xd08bc19f, SUC_lexicon_tags_47 },
    { 0xeb8bc1a3, SUC_lexicon_tags_40 },
    { 0x4b2bc1a3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2b8bc1a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe80bc1a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cbc1ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2bc1af, SUC_lexicon_tags_1 },
    { 0x480bc1b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd8bc1b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56bc1b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x750bc1b9, SUC_lexicon_tags_15 },
    { 0xfecbc1ba, SUC_lexicon_tags_11 },
    { 0x5b2bc1ba, SUC_lexicon_tags_31 },
    { 0xe8cbc1ba, SUC_lexicon_tags_29 },
    { 0x4a0bc1bd, SUC_lexicon_tags_12 },
    { 0xb04bc1be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe08bc1c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7abc1c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ebc1c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5e4bc1ca, SUC_lexicon_tags_7 },
    { 0x30ebc1ca, SUC_lexicon_tags_55 },
    { 0xd34bc1cc, SUC_lexicon_tags_2 },
    { 0xffebc1cd, SUC_lexicon_tags_32 },
    { 0x46abc1cc, SUC_lexicon_tags_17 },
    { 0xbf4bc1cf, SUC_lexicon_tags_8 },
    { 0xb4abc1d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49abc1d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632bc1de, SUC_lexicon_tags_25 },
    { 0xb96bc1de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0ebc1e1, SUC_lexicon_tags_15 },
    { 0x6bc1e2, SUC_lexicon_tags_10 },
    { 0xc82bc1e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x29cbc1e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57abc1e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708bc1ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa76bc1ee, SUC_lexicon_tags_12 },
    { 0x386bc1ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82cbc1f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x710bc1f3, SUC_lexicon_tags_37 },
    { 0x394bc1f4, SUC_lexicon_tags_0 },
    { 0x9febc1f5, SUC_lexicon_tags_37 },
    { 0xeb0bc1f6, SUC_lexicon_tags_31 },
    { 0x71ebc1f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x948bc1f9, SUC_lexicon_tags_31 },
    { 0x140bc1fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2bc201, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06bc204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1abc206, SUC_lexicon_tags_38 },
    { 0x6bc207, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4bc20a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470bc219, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcbc21c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6bc222, SUC_lexicon_tags_27 },
    { 0x3debc222, SUC_lexicon_tags_1 },
    { 0xb4cbc223, SUC_lexicon_tags_39 },
    { 0x72bc225, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0bc22e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e2bc230, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x334bc233, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb0bc235, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6bc23b, SUC_lexicon_tags_7 },
    { 0xf04bc23c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ebc24a, SUC_lexicon_tags_37 },
    { 0x26cbc24b, SUC_lexicon_tags_2 },
    { 0xc46bc24b, SUC_lexicon_tags_7 },
    { 0x364bc24c, SUC_lexicon_tags_10 },
    { 0x666bc24c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26bc252, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ebc255, SUC_lexicon_tags_49 },
    { 0x1b2bc256, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20bc25a, SUC_lexicon_tags_55 },
    { 0x136bc25b, SUC_lexicon_tags_85 },
    { 0x9e6bc25b, SUC_lexicon_tags_15 },
    { 0xda8bc25a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0abc260, SUC_lexicon_tags_1 },
    { 0x722bc261, SUC_lexicon_tags_56 },
    { 0xaacbc260, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04bc265, SUC_lexicon_tags_7 },
    { 0xd16bc266, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0bc26f, SUC_lexicon_tags_37 },
    { 0xb52bc270, SUC_lexicon_tags_7 },
    { 0x942bc271, SUC_lexicon_tags_142 },
    { 0x77abc272, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94bc275, SUC_lexicon_tags_25 },
    { 0xa44bc275, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc96bc278, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5abc27b, SUC_lexicon_tags_95 },
    { 0x168bc27c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cbc284, SUC_lexicon_tags_37 },
    { 0xab6bc285, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27abc28b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc9abc28d, SUC_lexicon_tags_17 },
    { 0x356bc28e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac2bc290, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ebc297, SUC_lexicon_tags_31 },
    { 0x3eebc297, SUC_lexicon_tags_15 },
    { 0xd56bc299, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcebc29c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfabc2a2, SUC_lexicon_tags_25 },
    { 0xd6bc2a3, SUC_lexicon_tags_17 },
    { 0x306bc2a4, SUC_lexicon_tags_14 },
    { 0x148bc2a5, SUC_lexicon_tags_37 },
    { 0xc2abc2a3, SUC_lexicon_tags_37 },
    { 0x4b4bc2a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90abc2aa, SUC_lexicon_tags_1 },
    { 0xb24bc2ab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcbc2ae, SUC_lexicon_tags_37 },
    { 0x9e0bc2af, SUC_lexicon_tags_11 },
    { 0x3b0bc2ae, SUC_lexicon_tags_2 },
    { 0xf52bc2b1, SUC_lexicon_tags_60 },
    { 0x2d0bc2af, SUC_lexicon_tags_47 },
    { 0xb02bc2b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58abc2b7, SUC_lexicon_tags_77 },
    { 0xeeebc2b8, SUC_lexicon_tags_0 },
    { 0xb4cbc2b9, SUC_lexicon_tags_0 },
    { 0xbe8bc2b9, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbebc2c1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0bc2c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6abc2cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ebc2d2, SUC_lexicon_tags_25 },
    { 0xa44bc2d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ebc2db, SUC_lexicon_tags_82 },
    { 0x958bc2db, SUC_lexicon_tags_46 },
    { 0x9d8bc2db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588bc2e3, SUC_lexicon_tags_27 },
    { 0x16abc2e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0bc2ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcbc2f0, SUC_lexicon_tags_49 },
    { 0xab8bc2f0, SUC_lexicon_tags_25 },
    { 0x3b6bc2f0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xfa6bc2f4, SUC_lexicon_tags_17 },
    { 0xc36bc2f5, SUC_lexicon_tags_29 },
    { 0x794bc2f5, SUC_lexicon_tags_45 },
    { 0x438bc2f6, SUC_lexicon_tags_7 },
    { 0xef2bc2f8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8bc2ff, SUC_lexicon_tags_6 },
    { 0x64cbc300, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x41ebc302, SUC_lexicon_tags_11 },
    { 0x698bc302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6bc30a, SUC_lexicon_tags_15 },
    { 0xbdabc30b, SUC_lexicon_tags_15 },
    { 0x404bc30c, SUC_lexicon_tags_29 },
    { 0xda6bc30a, SUC_lexicon_tags_323 },
    { 0xea2bc30e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94bc315, SUC_lexicon_tags_19 },
    { 0x7b0bc316, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8bc31b, SUC_lexicon_tags_2 },
    { 0xdc6bc31c, SUC_lexicon_tags_235 },
    { 0xb66bc31d, SUC_lexicon_tags_1 },
    { 0x664bc31d, SUC_lexicon_tags_25 },
    { 0x8d6bc31d, SUC_lexicon_tags_63 },
    { 0x876bc31e, SUC_lexicon_tags_25 },
    { 0x4babc31b, SUC_lexicon_tags_17 },
    { 0x7febc322, SUC_lexicon_tags_38 },
    { 0x178bc320, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde6bc325, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf2bc327, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4bc32c, SUC_lexicon_tags_11 },
    { 0x5cbc32d, SUC_lexicon_tags_32 },
    { 0xe2ebc32e, SUC_lexicon_tags_49 },
    { 0x1fabc32f, SUC_lexicon_tags_11 },
    { 0xdc2bc330, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22bc335, SUC_lexicon_tags_47 },
    { 0xa54bc336, SUC_lexicon_tags_1 },
    { 0x5c6bc337, SUC_lexicon_tags_29 },
    { 0xe1ebc337, SUC_lexicon_tags_8 },
    { 0x934bc335, SUC_lexicon_tags_8 },
    { 0xc7abc33a, SUC_lexicon_tags_11 },
    { 0x708bc336, SUC_lexicon_tags_25 },
    { 0xddcbc338, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58bc33f, SUC_lexicon_tags_35 },
    { 0x538bc340, SUC_lexicon_tags_25 },
    { 0xcd6bc341, SUC_lexicon_tags_1 },
    { 0x208bc342, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85cbc344, SUC_lexicon_tags_25 },
    { 0x8eabc345, SUC_lexicon_tags_37 },
    { 0x6f4bc344, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc52bc348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a8bc34a, SUC_lexicon_tags_19 },
    { 0xac0bc34a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdabc34e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x99abc350, SUC_lexicon_tags_12 },
    { 0x218bc350, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcbc354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc88bc356, SUC_lexicon_tags_23 },
    { 0x70abc356, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0bc35b, SUC_lexicon_tags_17 },
    { 0xacbc35c, SUC_lexicon_tags_8 },
    { 0xf40bc35d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212bc361, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b6bc363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374bc36b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bc36f, SUC_lexicon_tags_17 },
    { 0xed0bc370, SUC_lexicon_tags_33 },
    { 0xcd8bc371, SUC_lexicon_tags_45 },
    { 0x32cbc371, SUC_lexicon_tags_9 },
    { 0x2c0bc371, SUC_lexicon_tags_30 },
    { 0x362bc374, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aabc377, SUC_lexicon_tags_1 },
    { 0x8a6bc377, SUC_lexicon_tags_32 },
    { 0x3dabc377, SUC_lexicon_tags_63 },
    { 0xd6bc37a, SUC_lexicon_tags_47 },
    { 0x8cbc37b, SUC_lexicon_tags_1 },
    { 0xf0abc377, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0bc380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccbc387, SUC_lexicon_tags_56 },
    { 0xd80bc388, SUC_lexicon_tags_1 },
    { 0xee2bc387, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0bc38b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad8bc38d, SUC_lexicon_tags_1 },
    { 0x8e8bc38d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe76bc390, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd92bc392, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824bc396, SUC_lexicon_tags_37 },
    { 0xb66bc396, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204bc39b, SUC_lexicon_tags_37 },
    { 0xf90bc39c, SUC_lexicon_tags_49 },
    { 0xf80bc39d, SUC_lexicon_tags_11 },
    { 0xe1cbc39e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xf10bc3a0, SUC_lexicon_tags_7 },
    { 0x89abc3a0, SUC_lexicon_tags_33 },
    { 0xb48bc3a2, SUC_lexicon_tags_21 },
    { 0xa58bc3a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ebc3a6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6bc3a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ebc3ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66cbc3ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0bc3b7, SUC_lexicon_tags_23 },
    { 0x700bc3b8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ebc3bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8bc3c0, SUC_lexicon_tags_47 },
    { 0xa76bc3c1, SUC_lexicon_tags_2 },
    { 0x7eabc3c2, SUC_lexicon_tags_26 },
    { 0x84bc3c3, SUC_lexicon_tags_7 },
    { 0xb98bc3c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12bc3c9, SUC_lexicon_tags_19 },
    { 0x206bc3ca, SUC_lexicon_tags_27 },
    { 0xfd8bc3c9, SUC_lexicon_tags_17 },
    { 0xd3ebc3cc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x754bc3ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ebc3d1, SUC_lexicon_tags_23 },
    { 0x31ebc3d1, SUC_lexicon_tags_82 },
    { 0x762bc3d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7abc3d8, SUC_lexicon_tags_39 },
    { 0xc32bc3d9, SUC_lexicon_tags_37 },
    { 0x70cbc3d9, SUC_lexicon_tags_47 },
    { 0x48ebc3db, SUC_lexicon_tags_55 },
    { 0xcdabc3dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcbc3e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abc3e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x602bc3e6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x81cbc3e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x51ebc3ea, SUC_lexicon_tags_47 },
    { 0x88ebc3eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3eabc3ed, SUC_lexicon_tags_37 },
    { 0x37ebc3ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf4bc3f0, SUC_lexicon_tags_19 },
    { 0x726bc3f1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884bc3f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cbc3fa, SUC_lexicon_tags_7 },
    { 0x516bc3fb, SUC_lexicon_tags_11 },
    { 0xcabc3fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42abc3fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60bc406, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cbc40c, SUC_lexicon_tags_30 },
    { 0x5e4bc40d, SUC_lexicon_tags_12 },
    { 0xfd4bc40e, SUC_lexicon_tags_14 },
    { 0x632bc40c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbc41f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddabc424, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cbc428, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34bc42c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50bc42f, SUC_lexicon_tags_0 },
    { 0x2eabc42f, SUC_lexicon_tags_68 },
    { 0x186bc430, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450bc435, SUC_lexicon_tags_3 },
    { 0xbabc436, SUC_lexicon_tags_11 },
    { 0xfccbc437, SUC_lexicon_tags_15 },
    { 0x60bc436, SUC_lexicon_tags_11 },
    { 0x7a6bc436, SUC_lexicon_tags_11 },
    { 0x68abc43a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78abc43e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0bc442, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4bc44e, SUC_lexicon_tags_25 },
    { 0x3b4bc44e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0bc453, SUC_lexicon_tags_1024 },
    { 0xb44bc453, SUC_lexicon_tags_14 },
    { 0xed8bc455, SUC_lexicon_tags_63 },
    { 0xd9cbc453, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x314bc458, SUC_lexicon_tags_43 },
    { 0xf66bc459, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6bc45c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5bebc45e, SUC_lexicon_tags_17 },
    { 0x606bc45e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaacbc461, SUC_lexicon_tags_26 },
    { 0xcbcbc462, SUC_lexicon_tags_55 },
    { 0x478bc463, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68bc466, SUC_lexicon_tags_1 },
    { 0x6aabc466, SUC_lexicon_tags_2 },
    { 0x2acbc468, SUC_lexicon_tags_2 },
    { 0x402bc466, SUC_lexicon_tags_10 },
    { 0x75cbc469, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaacbc46c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe44bc46e, SUC_lexicon_tags_7 },
    { 0x4bebc46e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xc3ebc471, SUC_lexicon_tags_2 },
    { 0x814bc471, SUC_lexicon_tags_2 },
    { 0x79abc472, SUC_lexicon_tags_87 },
    { 0x1aebc474, SUC_lexicon_tags_43 },
    { 0x92ebc475, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3ebc477, SUC_lexicon_tags_19 },
    { 0xfb2bc477, SUC_lexicon_tags_30 },
    { 0xab0bc477, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb64bc47b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0bc480, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5e0bc482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692bc48b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x560bc48d, SUC_lexicon_tags_17 },
    { 0x15abc48d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6bc493, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x112bc495, SUC_lexicon_tags_37 },
    { 0xb8bc496, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26bc4a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350bc4a4, SUC_lexicon_tags_45 },
    { 0xb5cbc4a4, SUC_lexicon_tags_138 },
    { 0x1f8bc4a6, SUC_lexicon_tags_17 },
    { 0xf80bc4a7, SUC_lexicon_tags_25 },
    { 0xb68bc4a6, SUC_lexicon_tags_26 },
    { 0x680bc4a9, SUC_lexicon_tags_25 },
    { 0xfd8bc4aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50bc4ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x710bc4b0, SUC_lexicon_tags_12 },
    { 0x4ebc4b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08bc4b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85abc4be, SUC_lexicon_tags_28 },
    { 0xd68bc4be, SUC_lexicon_tags_414 },
    { 0x34cbc4c0, SUC_lexicon_tags_17 },
    { 0xf6abc4c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26abc4c3, SUC_lexicon_tags_25 },
    { 0xae4bc4c4, SUC_lexicon_tags_0 },
    { 0x2febc4c4, SUC_lexicon_tags_12 },
    { 0xf7ebc4c5, SUC_lexicon_tags_14 },
    { 0x7fcbc4c7, SUC_lexicon_tags_14 },
    { 0x4f6bc4c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bc4cb, SUC_lexicon_tags_31 },
    { 0xb30bc4cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb1abc4ce, SUC_lexicon_tags_11 },
    { 0xc7cbc4ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x378bc4d1, SUC_lexicon_tags_25 },
    { 0x7b8bc4d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134bc4d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf42bc4d9, SUC_lexicon_tags_11 },
    { 0x926bc4da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdc2bc4dc, SUC_lexicon_tags_14 },
    { 0x6b6bc4dd, SUC_lexicon_tags_69 },
    { 0xa3cbc4dc, SUC_lexicon_tags_30 },
    { 0x3acbc4dd, SUC_lexicon_tags_15 },
    { 0x158bc4dc, SUC_lexicon_tags_8 },
    { 0x7dabc4e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x112bc4e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94bc4ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcebc4f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dabc4f8, SUC_lexicon_tags_40 },
    { 0x8cbc4f8, SUC_lexicon_tags_25 },
    { 0xd80bc4f9, SUC_lexicon_tags_25 },
    { 0x4bcbc4fb, SUC_lexicon_tags_55 },
    { 0xad0bc4fc, SUC_lexicon_tags_12 },
    { 0x132bc4fd, SUC_lexicon_tags_17 },
    { 0x4f8bc4fe, SUC_lexicon_tags_19 },
    { 0x82bc4fa, SUC_lexicon_tags_6 },
    { 0x956bc500, SUC_lexicon_tags_84 },
    { 0x5d6bc4fc, SUC_lexicon_tags_11 },
    { 0xa96bc502, SUC_lexicon_tags_38 },
    { 0xd98bc502, SUC_lexicon_tags_12 },
    { 0xd32bc4fc, SUC_lexicon_tags_19 },
    { 0xe28bc503, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6bc509, SUC_lexicon_tags_45 },
    { 0xdeebc50a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3abc50d, SUC_lexicon_tags_60 },
    { 0xfccbc50e, SUC_lexicon_tags_37 },
    { 0x53ebc50d, SUC_lexicon_tags_11 },
    { 0xcdcbc50f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccbc515, SUC_lexicon_tags_33 },
    { 0x7aabc516, SUC_lexicon_tags_18 },
    { 0x87abc516, SUC_lexicon_tags_1 },
    { 0x712bc518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x322bc51b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bc524, SUC_lexicon_tags_36 },
    { 0x9eebc524, SUC_lexicon_tags_43 },
    { 0xb74bc525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bc52d, SUC_lexicon_tags_37 },
    { 0x5f8bc52d, SUC_lexicon_tags_14 },
    { 0x382bc52d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6bc533, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe92bc535, SUC_lexicon_tags_11 },
    { 0xf1cbc536, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8bc53c, SUC_lexicon_tags_198 },
    { 0xd52bc53c, SUC_lexicon_tags_7 },
    { 0xc02bc53e, SUC_lexicon_tags_47 },
    { 0xe78bc53f, SUC_lexicon_tags_37 },
    { 0x728bc53f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908bc549, SUC_lexicon_tags_37 },
    { 0x8f6bc549, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5ebc54c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70bc550, SUC_lexicon_tags_27 },
    { 0x7b8bc550, SUC_lexicon_tags_7 },
    { 0x5debc552, SUC_lexicon_tags_15 },
    { 0x9f0bc553, SUC_lexicon_tags_50 },
    { 0x4dcbc553, SUC_lexicon_tags_123 },
    { 0x448bc555, SUC_lexicon_tags_8 },
    { 0xe3cbc555, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd1cbc558, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4eabc55a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd58bc55c, SUC_lexicon_tags_69 },
    { 0x52ebc55d, SUC_lexicon_tags_1 },
    { 0x76cbc55e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cbc561, SUC_lexicon_tags_3 },
    { 0x8dcbc562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34bc56c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x554bc56f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310bc577, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cbc57d, SUC_lexicon_tags_29 },
    { 0x5b8bc57e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc76bc580, SUC_lexicon_tags_15 },
    { 0x2a2bc581, SUC_lexicon_tags_49 },
    { 0x9a0bc580, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x402bc584, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x780bc586, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4bc589, SUC_lexicon_tags_60 },
    { 0xceebc589, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2bc58d, SUC_lexicon_tags_77 },
    { 0x65abc58e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ebc592, SUC_lexicon_tags_2 },
    { 0xc6abc592, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1f0bc595, SUC_lexicon_tags_7 },
    { 0xcc2bc596, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x948bc598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4bc59e, SUC_lexicon_tags_2 },
    { 0xbbcbc59f, SUC_lexicon_tags_12 },
    { 0x106bc5a0, SUC_lexicon_tags_49 },
    { 0x4e4bc5a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a6bc5a3, SUC_lexicon_tags_17 },
    { 0xd0abc5a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x62cbc5a6, SUC_lexicon_tags_37 },
    { 0x69abc5a7, SUC_lexicon_tags_17 },
    { 0x8a2bc5a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44bc5ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b8bc5ad, SUC_lexicon_tags_418 },
    { 0x3dabc5ae, SUC_lexicon_tags_12 },
    { 0x52bc5ad, SUC_lexicon_tags_7 },
    { 0xbe2bc5ae, SUC_lexicon_tags_14 },
    { 0xea2bc5b1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x954bc5b3, SUC_lexicon_tags_7 },
    { 0x8c4bc5b3, SUC_lexicon_tags_24 },
    { 0xa0abc5b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400bc5bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86abc5c1, SUC_lexicon_tags_37 },
    { 0xd48bc5c2, SUC_lexicon_tags_9 },
    { 0xb66bc5c2, SUC_lexicon_tags_2 },
    { 0xa66bc5c2, SUC_lexicon_tags_10 },
    { 0xfa6bc5c4, SUC_lexicon_tags_27 },
    { 0x790bc5c5, SUC_lexicon_tags_37 },
    { 0x5e0bc5c6, SUC_lexicon_tags_2 },
    { 0x2dabc5c5, SUC_lexicon_tags_16 },
    { 0x5c0bc5c9, SUC_lexicon_tags_65 },
    { 0x9f4bc5c9, SUC_lexicon_tags_4 },
    { 0x4abc5cb, SUC_lexicon_tags_6 },
    { 0xafcbc5cc, SUC_lexicon_tags_15 },
    { 0xed0bc5cd, SUC_lexicon_tags_45 },
    { 0x9fcbc5cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10bc5d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3abc5e1, SUC_lexicon_tags_37 },
    { 0x1aebc5e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2bc5e9, SUC_lexicon_tags_17 },
    { 0xfc8bc5ea, SUC_lexicon_tags_14 },
    { 0x522bc5ea, SUC_lexicon_tags_11 },
    { 0xb6cbc5ea, SUC_lexicon_tags_8 },
    { 0x60cbc5ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaabc5ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeebc5f4, SUC_lexicon_tags_47 },
    { 0xd18bc5f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6cbc5f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04bc5fb, SUC_lexicon_tags_11 },
    { 0x642bc5fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb9cbc5fe, SUC_lexicon_tags_1 },
    { 0x8fabc5ff, SUC_lexicon_tags_12 },
    { 0x530bc5ff, SUC_lexicon_tags_15 },
    { 0x712bc601, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0bc604, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x812bc606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8bc609, SUC_lexicon_tags_15 },
    { 0xb30bc60a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad8bc60c, SUC_lexicon_tags_79 },
    { 0xc02bc60d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56abc60f, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8bc613, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bc61b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4bcbc61d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78bc61f, SUC_lexicon_tags_21 },
    { 0x370bc61f, SUC_lexicon_tags_33 },
    { 0x782bc621, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdabc625, SUC_lexicon_tags_32 },
    { 0xc44bc626, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986bc62a, SUC_lexicon_tags_27 },
    { 0x372bc62b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecbc62e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d2bc630, SUC_lexicon_tags_1 },
    { 0xbb8bc630, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd6cbc633, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844bc638, SUC_lexicon_tags_25 },
    { 0x1b4bc639, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8bc63d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130bc646, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xbfabc648, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0bc64c, SUC_lexicon_tags_30 },
    { 0xc48bc64d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x33abc64f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2bc655, SUC_lexicon_tags_21 },
    { 0x5e4bc655, SUC_lexicon_tags_7 },
    { 0x4a8bc657, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876bc65c, SUC_lexicon_tags_88 },
    { 0x436bc65c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722bc661, SUC_lexicon_tags_25 },
    { 0xe74bc662, SUC_lexicon_tags_37 },
    { 0xf52bc661, SUC_lexicon_tags_28 },
    { 0x52abc663, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4bc667, SUC_lexicon_tags_68 },
    { 0x900bc668, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bc66d, SUC_lexicon_tags_15 },
    { 0x8acbc66e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0bc671, SUC_lexicon_tags_32 },
    { 0x2c8bc671, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ccbc674, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2bc677, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc67a, SUC_lexicon_tags_27 },
    { 0x3a8bc67a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83abc684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2ebc686, SUC_lexicon_tags_2 },
    { 0x72abc687, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cbc68b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x38abc68d, SUC_lexicon_tags_46 },
    { 0x6d4bc68e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x286bc691, SUC_lexicon_tags_12 },
    { 0x77abc691, SUC_lexicon_tags_49 },
    { 0x8a2bc693, SUC_lexicon_tags_37 },
    { 0x4cabc691, SUC_lexicon_tags_33 },
    { 0xf0cbc695, SUC_lexicon_tags_15 },
    { 0x50cbc696, SUC_lexicon_tags_7 },
    { 0x856bc692, SUC_lexicon_tags_17 },
    { 0xa68bc698, SUC_lexicon_tags_12 },
    { 0xf64bc696, SUC_lexicon_tags_65 },
    { 0x3dabc691, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340bc69e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d2bc6a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf6bc6a2, SUC_lexicon_tags_25 },
    { 0x2f4bc6a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6cbc6a5, SUC_lexicon_tags_1 },
    { 0x120bc6a6, SUC_lexicon_tags_16 },
    { 0xb8abc6a7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94bc6aa, SUC_lexicon_tags_15 },
    { 0x518bc6ab, SUC_lexicon_tags_2 },
    { 0x42cbc6ac, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd30bc6ae, SUC_lexicon_tags_26 },
    { 0xe76bc6af, SUC_lexicon_tags_12 },
    { 0x512bc6b0, SUC_lexicon_tags_37 },
    { 0x6fabc6b1, SUC_lexicon_tags_17 },
    { 0x2a4bc6ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x718bc6b4, SUC_lexicon_tags_15 },
    { 0x20bc6b5, SUC_lexicon_tags_31 },
    { 0xa06bc6b6, SUC_lexicon_tags_2 },
    { 0x942bc6b4, SUC_lexicon_tags_47 },
    { 0x6f6bc6b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0bc6bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf12bc6bd, SUC_lexicon_tags_25 },
    { 0x30cbc6be, SUC_lexicon_tags_52 },
    { 0x1cebc6bf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0bc6c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978bc6c7, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ebc6ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22bc6cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a2bc6cf, SUC_lexicon_tags_17 },
    { 0xa18bc6d0, SUC_lexicon_tags_899 },
    { 0, NULL },
    { 0, NULL },
    { 0xafebc6d3, SUC_lexicon_tags_0 },
    { 0xbdabc6d4, SUC_lexicon_tags_33 },
    { 0xd9cbc6d5, SUC_lexicon_tags_15 },
    { 0x6abc6d4, SUC_lexicon_tags_87 },
    { 0x832bc6d5, SUC_lexicon_tags_37 },
    { 0x37cbc6d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x128bc6da, SUC_lexicon_tags_84 },
    { 0xda8bc6db, SUC_lexicon_tags_12 },
    { 0x8b6bc6db, SUC_lexicon_tags_37 },
    { 0x604bc6dd, SUC_lexicon_tags_17 },
    { 0x5fcbc6dd, SUC_lexicon_tags_37 },
    { 0x512bc6de, SUC_lexicon_tags_7 },
    { 0x578bc6e0, SUC_lexicon_tags_17 },
    { 0x70cbc6e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744bc6e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4bc6e8, SUC_lexicon_tags_63 },
    { 0xc1ebc6e9, SUC_lexicon_tags_11 },
    { 0xce6bc6ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ebc6ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618bc6f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccbc6fa, SUC_lexicon_tags_0 },
    { 0xebcbc6fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76bc6fe, SUC_lexicon_tags_57 },
    { 0x45ebc6ff, SUC_lexicon_tags_17 },
    { 0x5eabc700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54abc702, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa42bc704, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd2bc706, SUC_lexicon_tags_14 },
    { 0xa82bc707, SUC_lexicon_tags_25 },
    { 0xe0ebc708, SUC_lexicon_tags_48 },
    { 0xae8bc709, SUC_lexicon_tags_12 },
    { 0x1f4bc70a, SUC_lexicon_tags_38 },
    { 0x14abc70b, SUC_lexicon_tags_33 },
    { 0x568bc70c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x500bc70f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcabc712, SUC_lexicon_tags_17 },
    { 0xb3ebc713, SUC_lexicon_tags_14 },
    { 0xa5abc714, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ccbc716, SUC_lexicon_tags_14 },
    { 0x3cabc717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12bc71a, SUC_lexicon_tags_7 },
    { 0xb58bc71b, SUC_lexicon_tags_8 },
    { 0xdccbc71c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8bc71f, SUC_lexicon_tags_45 },
    { 0x16cbc720, SUC_lexicon_tags_16 },
    { 0x6d0bc720, SUC_lexicon_tags_37 },
    { 0x6b4bc722, SUC_lexicon_tags_1 },
    { 0x33cbc723, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55abc726, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf90bc728, SUC_lexicon_tags_2 },
    { 0xeb0bc729, SUC_lexicon_tags_30 },
    { 0xcc4bc72a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514bc72e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca6bc730, SUC_lexicon_tags_11 },
    { 0x35ebc731, SUC_lexicon_tags_11 },
    { 0xaa0bc731, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x114bc734, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x160bc736, SUC_lexicon_tags_37 },
    { 0xfaebc737, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x754bc73a, SUC_lexicon_tags_84 },
    { 0xb9ebc73a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x466bc73d, SUC_lexicon_tags_1 },
    { 0x6b4bc73e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388bc742, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96bc745, SUC_lexicon_tags_26 },
    { 0x590bc746, SUC_lexicon_tags_11 },
    { 0x9e4bc747, SUC_lexicon_tags_17 },
    { 0x536bc748, SUC_lexicon_tags_28 },
    { 0x97abc749, SUC_lexicon_tags_49 },
    { 0xd3ebc746, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20abc74e, SUC_lexicon_tags_43 },
    { 0x83abc74f, SUC_lexicon_tags_3 },
    { 0xe66bc750, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572bc754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbcbc756, SUC_lexicon_tags_63 },
    { 0x900bc757, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76bc75d, SUC_lexicon_tags_17 },
    { 0x450bc75e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x544bc761, SUC_lexicon_tags_88 },
    { 0x42bc761, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170bc769, SUC_lexicon_tags_1 },
    { 0xfe8bc769, SUC_lexicon_tags_15 },
    { 0x4b6bc769, SUC_lexicon_tags_0 },
    { 0xfbcbc76c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14bc770, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8bc777, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6a4bc779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbaabc77b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b0bc77d, SUC_lexicon_tags_15 },
    { 0x860bc77e, SUC_lexicon_tags_33 },
    { 0x9eabc77f, SUC_lexicon_tags_43 },
    { 0x332bc780, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8bc785, SUC_lexicon_tags_15 },
    { 0x632bc785, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe8bc788, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10cbc794, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0bc797, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x23cbc799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142bc79d, SUC_lexicon_tags_47 },
    { 0xa78bc79e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x26bc7a0, SUC_lexicon_tags_43 },
    { 0xceebc7a0, SUC_lexicon_tags_2 },
    { 0x100bc7a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ebc7a8, SUC_lexicon_tags_12 },
    { 0x8b8bc7a8, SUC_lexicon_tags_23 },
    { 0x80cbc7aa, SUC_lexicon_tags_29 },
    { 0xb72bc7ab, SUC_lexicon_tags_43 },
    { 0x9f4bc7ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ebc7b4, SUC_lexicon_tags_25 },
    { 0xc68bc7b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750bc7bb, SUC_lexicon_tags_60 },
    { 0xa88bc7bc, SUC_lexicon_tags_38 },
    { 0xe12bc7bd, SUC_lexicon_tags_7 },
    { 0x526bc7be, SUC_lexicon_tags_25 },
    { 0x612bc7bd, SUC_lexicon_tags_15 },
    { 0x436bc7bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x156bc7c2, SUC_lexicon_tags_12 },
    { 0x680bc7c3, SUC_lexicon_tags_4 },
    { 0xae0bc7c4, SUC_lexicon_tags_30 },
    { 0xc90bc7c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe98bc7c7, SUC_lexicon_tags_77 },
    { 0x164bc7c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704bc7cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c4bc7ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398bc7d8, SUC_lexicon_tags_15 },
    { 0xafabc7d8, SUC_lexicon_tags_25 },
    { 0xea0bc7da, SUC_lexicon_tags_7 },
    { 0x40bc7db, SUC_lexicon_tags_26 },
    { 0x8ccbc7d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x614bc7df, SUC_lexicon_tags_68 },
    { 0xfc6bc7e0, SUC_lexicon_tags_14 },
    { 0xa5ebc7e1, SUC_lexicon_tags_17 },
    { 0xc68bc7e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc62bc7e4, SUC_lexicon_tags_93 },
    { 0xb9ebc7e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18bc7f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x590bc7f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b2bc7f7, SUC_lexicon_tags_82 },
    { 0x7a8bc7f8, SUC_lexicon_tags_27 },
    { 0xc38bc7f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x95abc7fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a8bc7fd, SUC_lexicon_tags_35 },
    { 0x836bc7fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x79cbc800, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x68ebc802, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36bc805, SUC_lexicon_tags_32 },
    { 0x182bc805, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bc809, SUC_lexicon_tags_29 },
    { 0x310bc80a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c2bc80c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x61cbc80f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccebc816, SUC_lexicon_tags_1 },
    { 0x40abc817, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x598bc81a, SUC_lexicon_tags_37 },
    { 0x78bc81a, SUC_lexicon_tags_43 },
    { 0x126bc81c, SUC_lexicon_tags_23 },
    { 0x2cbc81b, SUC_lexicon_tags_12 },
    { 0x5fabc81d, SUC_lexicon_tags_14 },
    { 0x920bc81f, SUC_lexicon_tags_6 },
    { 0x3febc820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cabc827, SUC_lexicon_tags_27 },
    { 0x304bc828, SUC_lexicon_tags_37 },
    { 0xb9cbc828, SUC_lexicon_tags_81 },
    { 0xb2ebc82a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2bc82d, SUC_lexicon_tags_1 },
    { 0x482bc82e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44bc832, SUC_lexicon_tags_47 },
    { 0xd9abc832, SUC_lexicon_tags_1 },
    { 0x4febc832, SUC_lexicon_tags_37 },
    { 0x92cbc835, SUC_lexicon_tags_35 },
    { 0x318bc836, SUC_lexicon_tags_37 },
    { 0xa38bc837, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74bc83b, SUC_lexicon_tags_2 },
    { 0x550bc83c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958bc840, SUC_lexicon_tags_1 },
    { 0xa6cbc840, SUC_lexicon_tags_47 },
    { 0x508bc842, SUC_lexicon_tags_17 },
    { 0x47cbc842, SUC_lexicon_tags_17 },
    { 0x8f6bc843, SUC_lexicon_tags_37 },
    { 0x5cbc845, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1abc848, SUC_lexicon_tags_56 },
    { 0xe4bc848, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadabc84b, SUC_lexicon_tags_87 },
    { 0x3a6bc84b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfebc84e, SUC_lexicon_tags_12 },
    { 0xc02bc84f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38cbc851, SUC_lexicon_tags_17 },
    { 0xde2bc852, SUC_lexicon_tags_28 },
    { 0xab0bc851, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4bc859, SUC_lexicon_tags_39 },
    { 0x42cbc859, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x79abc85c, SUC_lexicon_tags_25 },
    { 0x5c0bc85d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0bc862, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f0bc864, SUC_lexicon_tags_7 },
    { 0x5bcbc864, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b2bc867, SUC_lexicon_tags_63 },
    { 0xc9cbc868, SUC_lexicon_tags_12 },
    { 0x548bc868, SUC_lexicon_tags_8 },
    { 0x66cbc86a, SUC_lexicon_tags_14 },
    { 0x7aabc86b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x74cbc86e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622bc874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0bc878, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842bc87e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b0bc880, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x574bc882, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x638bc884, SUC_lexicon_tags_0 },
    { 0xb5ebc885, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xad8bc887, SUC_lexicon_tags_17 },
    { 0x920bc888, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90bc88b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58bc892, SUC_lexicon_tags_17 },
    { 0x488bc893, SUC_lexicon_tags_2 },
    { 0xf14bc894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x840bc896, SUC_lexicon_tags_11 },
    { 0x87abc896, SUC_lexicon_tags_26 },
    { 0xe4bc898, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd32bc89a, SUC_lexicon_tags_17 },
    { 0xc44bc89b, SUC_lexicon_tags_29 },
    { 0xc80bc89a, SUC_lexicon_tags_25 },
    { 0xfacbc89d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcbc8a5, SUC_lexicon_tags_47 },
    { 0xef0bc8a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54bc8aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0bc8b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ebc8b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66abc8bb, SUC_lexicon_tags_14 },
    { 0x14cbc8bc, SUC_lexicon_tags_288 },
    { 0x9d0bc8bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6babc8bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bc8c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8bc8cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x594bc8ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cbc8d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf0bc8d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ebc8d6, SUC_lexicon_tags_43 },
    { 0x128bc8d6, SUC_lexicon_tags_1 },
    { 0x788bc8d7, SUC_lexicon_tags_2 },
    { 0xa6ebc8d6, SUC_lexicon_tags_48 },
    { 0x13cbc8da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bc8de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6bc8e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94bc8e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x986bc8e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeacbc8e9, SUC_lexicon_tags_33 },
    { 0xb88bc8e9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56bc8f1, SUC_lexicon_tags_21 },
    { 0xafcbc8f1, SUC_lexicon_tags_27 },
    { 0xe6cbc8f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bc8f6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb16bc8f8, SUC_lexicon_tags_51 },
    { 0x124bc8f9, SUC_lexicon_tags_62 },
    { 0xdf0bc8f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9f2bc8fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6bc903, SUC_lexicon_tags_1 },
    { 0x1f2bc904, SUC_lexicon_tags_49 },
    { 0x59cbc905, SUC_lexicon_tags_6 },
    { 0xd1cbc905, SUC_lexicon_tags_30 },
    { 0xecebc907, SUC_lexicon_tags_25 },
    { 0xe0cbc906, SUC_lexicon_tags_4 },
    { 0xb4cbc906, SUC_lexicon_tags_38 },
    { 0xc24bc90a, SUC_lexicon_tags_26 },
    { 0xfa6bc90b, SUC_lexicon_tags_15 },
    { 0x3febc90b, SUC_lexicon_tags_19 },
    { 0x1c0bc90d, SUC_lexicon_tags_1 },
    { 0x38ebc90c, SUC_lexicon_tags_23 },
    { 0x9b8bc90f, SUC_lexicon_tags_31 },
    { 0xd06bc90b, SUC_lexicon_tags_45 },
    { 0xef0bc911, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73abc916, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3acbc919, SUC_lexicon_tags_0 },
    { 0xb9cbc919, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x352bc91c, SUC_lexicon_tags_7 },
    { 0x524bc91d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x59abc91f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c6bc921, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56bc927, SUC_lexicon_tags_27 },
    { 0x698bc927, SUC_lexicon_tags_77 },
    { 0xdcabc929, SUC_lexicon_tags_15 },
    { 0x70bc929, SUC_lexicon_tags_15 },
    { 0xbb4bc92a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782bc931, SUC_lexicon_tags_34 },
    { 0x388bc932, SUC_lexicon_tags_11 },
    { 0x486bc931, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ebc937, SUC_lexicon_tags_25 },
    { 0xd96bc938, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x160bc93b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926bc941, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2bc947, SUC_lexicon_tags_2 },
    { 0xb62bc947, SUC_lexicon_tags_12 },
    { 0x176bc949, SUC_lexicon_tags_31 },
    { 0xfd6bc94a, SUC_lexicon_tags_87 },
    { 0x5dabc94b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8bc953, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc00bc955, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d0bc957, SUC_lexicon_tags_12 },
    { 0x5c6bc958, SUC_lexicon_tags_47 },
    { 0x660bc959, SUC_lexicon_tags_37 },
    { 0x62bc95a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4bc95e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32ebc960, SUC_lexicon_tags_61 },
    { 0x540bc961, SUC_lexicon_tags_25 },
    { 0xbecbc961, SUC_lexicon_tags_15 },
    { 0x716bc960, SUC_lexicon_tags_12 },
    { 0xff6bc961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcebc975, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x626bc977, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x682bc97a, SUC_lexicon_tags_20 },
    { 0xa40bc97b, SUC_lexicon_tags_7 },
    { 0xf22bc97c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x148bc97f, SUC_lexicon_tags_6 },
    { 0xa26bc980, SUC_lexicon_tags_25 },
    { 0x81cbc981, SUC_lexicon_tags_12 },
    { 0xc4ebc981, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcabc989, SUC_lexicon_tags_4 },
    { 0xfaabc98a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56bc98e, SUC_lexicon_tags_16 },
    { 0x8babc98f, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4bc992, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x882bc994, SUC_lexicon_tags_15 },
    { 0xe66bc995, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29abc99c, SUC_lexicon_tags_11 },
    { 0xe06bc99d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cbc9a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2bc9a7, SUC_lexicon_tags_7 },
    { 0x590bc9a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb70bc9aa, SUC_lexicon_tags_11 },
    { 0xa34bc9ab, SUC_lexicon_tags_39 },
    { 0x420bc9ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2bc9b2, SUC_lexicon_tags_7 },
    { 0x31abc9b2, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682bc9b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4bc9bc, SUC_lexicon_tags_1 },
    { 0x32ebc9bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1ebc9bf, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0bc9c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ebc9c8, SUC_lexicon_tags_37 },
    { 0xca0bc9c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b2bc9cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bc9d2, SUC_lexicon_tags_13 },
    { 0x4ebc9d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x634bc9d6, SUC_lexicon_tags_59 },
    { 0xe20bc9d7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464bc9db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf8bc9dd, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x632bc9df, SUC_lexicon_tags_25 },
    { 0x7cebc9df, SUC_lexicon_tags_37 },
    { 0xde8bc9e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f0bc9e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ebc9e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4bc9ee, SUC_lexicon_tags_65 },
    { 0x7d4bc9ee, SUC_lexicon_tags_25 },
    { 0xf84bc9ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x736bc9f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x546bc9f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54bc9f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x286bc9fb, SUC_lexicon_tags_45 },
    { 0x554bc9fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4c6bc9fe, SUC_lexicon_tags_37 },
    { 0xdf4bc9fe, SUC_lexicon_tags_38 },
    { 0x358bc9ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2bca03, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2bca07, SUC_lexicon_tags_64 },
    { 0x7d2bca08, SUC_lexicon_tags_14 },
    { 0x5eebca09, SUC_lexicon_tags_45 },
    { 0xc52bca07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcaebca0c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x626bca0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7ebca11, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x558bca13, SUC_lexicon_tags_49 },
    { 0x36ebca14, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2bca1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1cebca1e, SUC_lexicon_tags_14 },
    { 0x50bca1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84bca22, SUC_lexicon_tags_2 },
    { 0x354bca23, SUC_lexicon_tags_235 },
    { 0xb1ebca23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x562bca27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acbca2d, SUC_lexicon_tags_49 },
    { 0x342bca2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8bca31, SUC_lexicon_tags_31 },
    { 0xfc8bca32, SUC_lexicon_tags_45 },
    { 0xa46bca33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ebca36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaacbca38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cbca3b, SUC_lexicon_tags_33 },
    { 0x33cbca3c, SUC_lexicon_tags_12 },
    { 0xbabca3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cbca41, SUC_lexicon_tags_17 },
    { 0xccebca42, SUC_lexicon_tags_12 },
    { 0xa24bca43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x676bca46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92bca4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x75cbca4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f8bca51, SUC_lexicon_tags_14 },
    { 0x880bca52, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ebca5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x470bca61, SUC_lexicon_tags_253 },
    { 0xb1cbca61, SUC_lexicon_tags_12 },
    { 0xf78bca61, SUC_lexicon_tags_15 },
    { 0x5b8bca64, SUC_lexicon_tags_29 },
    { 0x178bca64, SUC_lexicon_tags_11 },
    { 0xe08bca64, SUC_lexicon_tags_30 },
    { 0x54bca67, SUC_lexicon_tags_19 },
    { 0xe92bca68, SUC_lexicon_tags_11 },
    { 0xa5cbca67, SUC_lexicon_tags_11 },
    { 0xcbebca6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ebca6d, SUC_lexicon_tags_14 },
    { 0x6e4bca6e, SUC_lexicon_tags_27 },
    { 0xa8ebca6f, SUC_lexicon_tags_25 },
    { 0x3e8bca70, SUC_lexicon_tags_45 },
    { 0x116bca70, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2f6bca73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x666bca75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0bca78, SUC_lexicon_tags_12 },
    { 0x62abca79, SUC_lexicon_tags_68 },
    { 0xb28bca7a, SUC_lexicon_tags_87 },
    { 0xbd4bca7b, SUC_lexicon_tags_56 },
    { 0x38cbca7c, SUC_lexicon_tags_17 },
    { 0x1eabca7d, SUC_lexicon_tags_30 },
    { 0x896bca7d, SUC_lexicon_tags_25 },
    { 0x6b6bca7f, SUC_lexicon_tags_37 },
    { 0xf2abca7d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x500bca83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc64bca85, SUC_lexicon_tags_8 },
    { 0x6c6bca86, SUC_lexicon_tags_15 },
    { 0x7ebca86, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298bca8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4bca92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6bca96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x888bca99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa30bca9b, SUC_lexicon_tags_14 },
    { 0xa8cbca9c, SUC_lexicon_tags_17 },
    { 0x798bca9b, SUC_lexicon_tags_16 },
    { 0xaa6bca9d, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x968bcaa0, SUC_lexicon_tags_34 },
    { 0x150bcaa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeabcaa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292bcaad, SUC_lexicon_tags_11 },
    { 0x43cbcaae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x988bcab0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6bcab4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7a0bcab6, SUC_lexicon_tags_32 },
    { 0xa4abcab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798bcabb, SUC_lexicon_tags_60 },
    { 0x322bcabc, SUC_lexicon_tags_19 },
    { 0x2fcbcabd, SUC_lexicon_tags_25 },
    { 0x82bcabe, SUC_lexicon_tags_25 },
    { 0x21cbcabc, SUC_lexicon_tags_16 },
    { 0x69ebcac0, SUC_lexicon_tags_2 },
    { 0x6b6bcac1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xafebcac3, SUC_lexicon_tags_9 },
    { 0xbd2bcac3, SUC_lexicon_tags_37 },
    { 0x2fabcac5, SUC_lexicon_tags_27 },
    { 0xb60bcac5, SUC_lexicon_tags_85 },
    { 0xf34bcac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21cbcac9, SUC_lexicon_tags_534 },
    { 0x650bcac9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40bcad0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa18bcad2, SUC_lexicon_tags_45 },
    { 0x816bcad2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe66bcad5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x248bcad8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcbcadb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76abcadf, SUC_lexicon_tags_79 },
    { 0x404bcae0, SUC_lexicon_tags_0 },
    { 0x402bcae1, SUC_lexicon_tags_29 },
    { 0xcf2bcae0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0bcae6, SUC_lexicon_tags_37 },
    { 0x7d4bcae7, SUC_lexicon_tags_0 },
    { 0x82cbcae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eabcaeb, SUC_lexicon_tags_17 },
    { 0xc7abcaeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbcaee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadebcaf8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acbcafb, SUC_lexicon_tags_1 },
    { 0x162bcafc, SUC_lexicon_tags_32 },
    { 0x9f2bcafd, SUC_lexicon_tags_25 },
    { 0x41ebcafd, SUC_lexicon_tags_108 },
    { 0x1d4bcaff, SUC_lexicon_tags_0 },
    { 0x980bcafc, SUC_lexicon_tags_11 },
    { 0x802bcaff, SUC_lexicon_tags_11 },
    { 0x282bcb02, SUC_lexicon_tags_522 },
    { 0xd24bcaff, SUC_lexicon_tags_12 },
    { 0xf88bcb01, SUC_lexicon_tags_45 },
    { 0x144bcafe, SUC_lexicon_tags_35 },
    { 0xe92bcb06, SUC_lexicon_tags_43 },
    { 0x892bcb06, SUC_lexicon_tags_15 },
    { 0x38bcafb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcbcb10, SUC_lexicon_tags_7 },
    { 0xcc0bcb11, SUC_lexicon_tags_6 },
    { 0x93ebcb12, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6bcb16, SUC_lexicon_tags_27 },
    { 0x93cbcb17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52bcb1a, SUC_lexicon_tags_7 },
    { 0x656bcb1b, SUC_lexicon_tags_17 },
    { 0x53abcb1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150bcb22, SUC_lexicon_tags_25 },
    { 0xceabcb23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x662bcb25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x362bcb27, SUC_lexicon_tags_12 },
    { 0x31cbcb28, SUC_lexicon_tags_19 },
    { 0x736bcb28, SUC_lexicon_tags_56 },
    { 0x72ebcb2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496bcb2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6bcb32, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc84bcb34, SUC_lexicon_tags_31 },
    { 0xa54bcb34, SUC_lexicon_tags_17 },
    { 0xfdcbcb36, SUC_lexicon_tags_11 },
    { 0xe6ebcb35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb94bcb39, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64bcb3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438bcb44, SUC_lexicon_tags_15 },
    { 0x904bcb44, SUC_lexicon_tags_5 },
    { 0xa20bcb45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x40bcb48, SUC_lexicon_tags_107 },
    { 0x3c6bcb49, SUC_lexicon_tags_12 },
    { 0x3bcbcb4a, SUC_lexicon_tags_1 },
    { 0x2aebcb4a, SUC_lexicon_tags_0 },
    { 0x842bcb4b, SUC_lexicon_tags_87 },
    { 0x632bcb4d, SUC_lexicon_tags_25 },
    { 0xe32bcb4d, SUC_lexicon_tags_106 },
    { 0x734bcb4d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xae0bcb51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfeebcb53, SUC_lexicon_tags_17 },
    { 0x92bcb54, SUC_lexicon_tags_29 },
    { 0x4c2bcb54, SUC_lexicon_tags_2 },
    { 0x490bcb56, SUC_lexicon_tags_1 },
    { 0xcccbcb57, SUC_lexicon_tags_25 },
    { 0x5fcbcb58, SUC_lexicon_tags_23 },
    { 0x2debcb59, SUC_lexicon_tags_17 },
    { 0x542bcb59, SUC_lexicon_tags_82 },
    { 0x9a2bcb5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfebcb5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0bcb63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bebcb66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x768bcb68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5abcb6d, SUC_lexicon_tags_23 },
    { 0x308bcb6e, SUC_lexicon_tags_39 },
    { 0x870bcb6f, SUC_lexicon_tags_46 },
    { 0x822bcb6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa28bcb72, SUC_lexicon_tags_15 },
    { 0xe9ebcb73, SUC_lexicon_tags_11 },
    { 0xd04bcb73, SUC_lexicon_tags_37 },
    { 0x8b0bcb75, SUC_lexicon_tags_21 },
    { 0xfacbcb76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf06bcb78, SUC_lexicon_tags_1 },
    { 0xe86bcb79, SUC_lexicon_tags_2 },
    { 0xed0bcb7a, SUC_lexicon_tags_38 },
    { 0x9d2bcb79, SUC_lexicon_tags_68 },
    { 0x770bcb78, SUC_lexicon_tags_25 },
    { 0x5f2bcb7d, SUC_lexicon_tags_23 },
    { 0x4abcb7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4debcb81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ebcb86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14bcb89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ebcb8c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xbaebcb8e, SUC_lexicon_tags_12 },
    { 0x898bcb8e, SUC_lexicon_tags_11 },
    { 0x348bcb90, SUC_lexicon_tags_37 },
    { 0xc12bcb91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ebcb94, SUC_lexicon_tags_38 },
    { 0xbabcb95, SUC_lexicon_tags_8 },
    { 0x14abcb96, SUC_lexicon_tags_2 },
    { 0x4fcbcb97, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xff2bcb99, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3cbcb9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x940bcb9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ebcba1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ebcba4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x530bcba7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96bcbac, SUC_lexicon_tags_63 },
    { 0xd3cbcbad, SUC_lexicon_tags_2 },
    { 0x730bcbac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57ebcbb0, SUC_lexicon_tags_25 },
    { 0x672bcbb1, SUC_lexicon_tags_14 },
    { 0xb0cbcbb2, SUC_lexicon_tags_45 },
    { 0x9cebcbb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x154bcbb6, SUC_lexicon_tags_4 },
    { 0xedcbcbb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f8bcbb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3cabcbbb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8fabcbbd, SUC_lexicon_tags_27 },
    { 0x216bcbbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2bcbc1, SUC_lexicon_tags_11 },
    { 0xae0bcbc2, SUC_lexicon_tags_19 },
    { 0xbf6bcbc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x10abcbc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620bcbc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ebcbce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66cbcbd0, SUC_lexicon_tags_49 },
    { 0x6c4bcbd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf18bcbd3, SUC_lexicon_tags_17 },
    { 0xed2bcbd4, SUC_lexicon_tags_17 },
    { 0xb4cbcbd5, SUC_lexicon_tags_25 },
    { 0x37abcbd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8bcbd8, SUC_lexicon_tags_17 },
    { 0x502bcbd9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6e4bcbdb, SUC_lexicon_tags_11 },
    { 0xce8bcbdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x532bcbdf, SUC_lexicon_tags_37 },
    { 0xce0bcbe0, SUC_lexicon_tags_40 },
    { 0xcfcbcbe1, SUC_lexicon_tags_25 },
    { 0xba8bcbe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cebcbe8, SUC_lexicon_tags_37 },
    { 0x270bcbe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf20bcbeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8bcbf0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4bcbf7, SUC_lexicon_tags_0 },
    { 0x53abcbf8, SUC_lexicon_tags_6 },
    { 0x39abcbf9, SUC_lexicon_tags_29 },
    { 0xcabcbf9, SUC_lexicon_tags_11 },
    { 0x890bcbfb, SUC_lexicon_tags_9 },
    { 0x1ecbcbfb, SUC_lexicon_tags_37 },
    { 0xaeabcbfc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8bcc01, SUC_lexicon_tags_27 },
    { 0x260bcc02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x75ebcc04, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa70bcc06, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebcc0c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x546bcc0e, SUC_lexicon_tags_1 },
    { 0x17abcc0e, SUC_lexicon_tags_11 },
    { 0x6d6bcc10, SUC_lexicon_tags_37 },
    { 0xf9abcc11, SUC_lexicon_tags_7 },
    { 0x850bcc12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fabcc14, SUC_lexicon_tags_45 },
    { 0x962bcc15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5debcc1c, SUC_lexicon_tags_1 },
    { 0x1c0bcc1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x494bcc1f, SUC_lexicon_tags_31 },
    { 0xb06bcc1f, SUC_lexicon_tags_17 },
    { 0xffabcc21, SUC_lexicon_tags_23 },
    { 0x308bcc1f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06bcc26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cbcc2a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4bcc2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38bcc2e, SUC_lexicon_tags_12 },
    { 0x356bcc2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6bcc42, SUC_lexicon_tags_15 },
    { 0xc5ebcc42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938bcc47, SUC_lexicon_tags_47 },
    { 0x19abcc48, SUC_lexicon_tags_29 },
    { 0x4fcbcc49, SUC_lexicon_tags_1 },
    { 0x2bcc49, SUC_lexicon_tags_45 },
    { 0xb9abcc4b, SUC_lexicon_tags_68 },
    { 0x204bcc4c, SUC_lexicon_tags_15 },
    { 0x238bcc4b, SUC_lexicon_tags_55 },
    { 0x8a0bcc4e, SUC_lexicon_tags_15 },
    { 0xa46bcc4c, SUC_lexicon_tags_1 },
    { 0xa6bcc50, SUC_lexicon_tags_15 },
    { 0xfc6bcc50, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2bcc55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ebcc58, SUC_lexicon_tags_12 },
    { 0x7f4bcc59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x52ebcc5b, SUC_lexicon_tags_15 },
    { 0xfdabcc5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81ebcc5e, SUC_lexicon_tags_29 },
    { 0xb4bcc5f, SUC_lexicon_tags_27 },
    { 0x9dcbcc5f, SUC_lexicon_tags_11 },
    { 0xb6bcc5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bcc64, SUC_lexicon_tags_47 },
    { 0x80cbcc65, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x992bcc68, SUC_lexicon_tags_38 },
    { 0x7b0bcc69, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cbcc6e, SUC_lexicon_tags_12 },
    { 0x9f2bcc6e, SUC_lexicon_tags_34 },
    { 0xc44bcc70, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780bcc74, SUC_lexicon_tags_49 },
    { 0xca4bcc75, SUC_lexicon_tags_19 },
    { 0x916bcc76, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bcc7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bcc7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cbcc84, SUC_lexicon_tags_8 },
    { 0x946bcc85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8bcc88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366bcc92, SUC_lexicon_tags_60 },
    { 0xcc6bcc92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x538bcc96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6bcc99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x784bcc9c, SUC_lexicon_tags_25 },
    { 0x30ebcc9d, SUC_lexicon_tags_27 },
    { 0x3d6bcc9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ebcca1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cbcca4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968bcca9, SUC_lexicon_tags_38 },
    { 0x3c4bcca9, SUC_lexicon_tags_26 },
    { 0xc06bccab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x22abccae, SUC_lexicon_tags_26 },
    { 0x9bebccaf, SUC_lexicon_tags_7 },
    { 0x256bccb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ebccb4, SUC_lexicon_tags_87 },
    { 0xfeebccb5, SUC_lexicon_tags_47 },
    { 0x8b4bccb6, SUC_lexicon_tags_17 },
    { 0x192bccb6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x744bccba, SUC_lexicon_tags_37 },
    { 0xba0bccba, SUC_lexicon_tags_21 },
    { 0x308bccbc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x714bccbf, SUC_lexicon_tags_17 },
    { 0x928bccc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bebccc6, SUC_lexicon_tags_23 },
    { 0x8c6bccc7, SUC_lexicon_tags_63 },
    { 0x6dabccc8, SUC_lexicon_tags_15 },
    { 0xdd4bccc8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4bcccc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bcccf, SUC_lexicon_tags_11 },
    { 0x412bcccf, SUC_lexicon_tags_8 },
    { 0xfd0bccd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eebccd9, SUC_lexicon_tags_23 },
    { 0x220bccd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18bccde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8abcce0, SUC_lexicon_tags_21 },
    { 0x8c0bcce1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe76bcce3, SUC_lexicon_tags_15 },
    { 0x102bcce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cbcce7, SUC_lexicon_tags_39 },
    { 0xb1cbcce8, SUC_lexicon_tags_15 },
    { 0x6e4bcce9, SUC_lexicon_tags_12 },
    { 0x476bccea, SUC_lexicon_tags_27 },
    { 0x69cbcce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x934bcced, SUC_lexicon_tags_15 },
    { 0x2d2bcced, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x312bccf0, SUC_lexicon_tags_31 },
    { 0x270bccf1, SUC_lexicon_tags_15 },
    { 0x194bccf2, SUC_lexicon_tags_11 },
    { 0xd6ebccf3, SUC_lexicon_tags_49 },
    { 0xd94bccf4, SUC_lexicon_tags_0 },
    { 0x6dcbccf1, SUC_lexicon_tags_7 },
    { 0x272bccf6, SUC_lexicon_tags_43 },
    { 0x23abccf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94bccff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71ebcd01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5abcd05, SUC_lexicon_tags_37 },
    { 0x1aebcd06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x188bcd09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1abcd0b, SUC_lexicon_tags_15 },
    { 0xf1ebcd0c, SUC_lexicon_tags_60 },
    { 0x72abcd0c, SUC_lexicon_tags_25 },
    { 0xaf8bcd0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe00bcd10, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xccbcd12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcbcd15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d4bcd17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ebcd1a, SUC_lexicon_tags_38 },
    { 0x684bcd1b, SUC_lexicon_tags_1 },
    { 0xbccbcd1b, SUC_lexicon_tags_7 },
    { 0x3c0bcd1d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x176bcd1f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950bcd25, SUC_lexicon_tags_15 },
    { 0x156bcd26, SUC_lexicon_tags_14 },
    { 0xa2ebcd26, SUC_lexicon_tags_33 },
    { 0xa1abcd27, SUC_lexicon_tags_25 },
    { 0xa44bcd28, SUC_lexicon_tags_7 },
    { 0x5e6bcd2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762bcd33, SUC_lexicon_tags_32 },
    { 0xf2ebcd34, SUC_lexicon_tags_31 },
    { 0x9e0bcd34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccbcd3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1dabcd40, SUC_lexicon_tags_7 },
    { 0xdbcbcd41, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x612bcd44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16bcd4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bcd50, SUC_lexicon_tags_37 },
    { 0x40ebcd51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8bcd57, SUC_lexicon_tags_46 },
    { 0xddcbcd57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ebcd5c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32abcd62, SUC_lexicon_tags_28 },
    { 0x718bcd63, SUC_lexicon_tags_12 },
    { 0x9eebcd64, SUC_lexicon_tags_77 },
    { 0xec4bcd64, SUC_lexicon_tags_27 },
    { 0x406bcd64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4d2bcd68, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xefabcd6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e0bcd6d, SUC_lexicon_tags_17 },
    { 0xffcbcd6e, SUC_lexicon_tags_1036 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74bcd72, SUC_lexicon_tags_2 },
    { 0x64bcd73, SUC_lexicon_tags_49 },
    { 0x5f6bcd72, SUC_lexicon_tags_15 },
    { 0xd68bcd75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702bcd7a, SUC_lexicon_tags_886 },
    { 0x360bcd7a, SUC_lexicon_tags_62 },
    { 0x6bcbcd7c, SUC_lexicon_tags_46 },
    { 0xf52bcd7d, SUC_lexicon_tags_39 },
    { 0xc78bcd7e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6b2bcd80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0bcd84, SUC_lexicon_tags_1 },
    { 0x14bcd85, SUC_lexicon_tags_19 },
    { 0x518bcd84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8bcd89, SUC_lexicon_tags_19 },
    { 0xf8abcd8a, SUC_lexicon_tags_79 },
    { 0x2d8bcd89, SUC_lexicon_tags_45 },
    { 0x444bcd8c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcbcd91, SUC_lexicon_tags_12 },
    { 0xedabcd92, SUC_lexicon_tags_25 },
    { 0xb74bcd92, SUC_lexicon_tags_25 },
    { 0x340bcd91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbcbcd96, SUC_lexicon_tags_25 },
    { 0x4dabcd96, SUC_lexicon_tags_129 },
    { 0xadabcd98, SUC_lexicon_tags_60 },
    { 0xa6cbcd99, SUC_lexicon_tags_1 },
    { 0xd9cbcd99, SUC_lexicon_tags_85 },
    { 0x4d6bcd9b, SUC_lexicon_tags_0 },
    { 0x36abcd9c, SUC_lexicon_tags_11 },
    { 0x65abcd9d, SUC_lexicon_tags_1 },
    { 0x836bcd9e, SUC_lexicon_tags_7 },
    { 0xaa6bcd9a, SUC_lexicon_tags_12 },
    { 0xa32bcda0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x280bcda3, SUC_lexicon_tags_39 },
    { 0x4b4bcda3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274bcdab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578bcdb0, SUC_lexicon_tags_17 },
    { 0xd3cbcdb0, SUC_lexicon_tags_29 },
    { 0xd32bcdb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98bcdb5, SUC_lexicon_tags_4 },
    { 0x9e0bcdb6, SUC_lexicon_tags_11 },
    { 0xab4bcdb6, SUC_lexicon_tags_1 },
    { 0x75cbcdb8, SUC_lexicon_tags_11 },
    { 0xa32bcdb6, SUC_lexicon_tags_31 },
    { 0x46abcdb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4bcdbd, SUC_lexicon_tags_0 },
    { 0x468bcdbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4abcdc0, SUC_lexicon_tags_11 },
    { 0x31abcdc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618bcdc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b2bcdc9, SUC_lexicon_tags_21 },
    { 0x83abcdc9, SUC_lexicon_tags_12 },
    { 0x3eabcdca, SUC_lexicon_tags_23 },
    { 0x1d8bcdcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2bcdd3, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0x2c4bcdd5, SUC_lexicon_tags_7 },
    { 0xbf2bcdd5, SUC_lexicon_tags_10 },
    { 0xc58bcdd7, SUC_lexicon_tags_43 },
    { 0xd6bcdd8, SUC_lexicon_tags_49 },
    { 0x5debcdd9, SUC_lexicon_tags_11 },
    { 0xa32bcdd9, SUC_lexicon_tags_7 },
    { 0xf90bcdda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d8bcddd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb92bcddf, SUC_lexicon_tags_14 },
    { 0x862bcde0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8bcde7, SUC_lexicon_tags_0 },
    { 0xfdebcde7, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x67cbcdea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe32bcdec, SUC_lexicon_tags_1 },
    { 0x90ebcded, SUC_lexicon_tags_3 },
    { 0x436bcdec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47abcdf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4debcdfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50abce00, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0xa68bce02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5ebce04, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8bce07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82bce0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cbce0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x626bce10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bce16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfebce19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdabce1b, SUC_lexicon_tags_6 },
    { 0x92abce1c, SUC_lexicon_tags_47 },
    { 0xf6abce1c, SUC_lexicon_tags_7 },
    { 0xd18bce1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x706bce21, SUC_lexicon_tags_45 },
    { 0xa96bce22, SUC_lexicon_tags_26 },
    { 0xeb6bce22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16ebce25, SUC_lexicon_tags_31 },
    { 0xe44bce25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2debce2d, SUC_lexicon_tags_8 },
    { 0x618bce2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacbce34, SUC_lexicon_tags_11 },
    { 0x130bce34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0abce37, SUC_lexicon_tags_37 },
    { 0x8f0bce38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x41cbce3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602bce40, SUC_lexicon_tags_1 },
    { 0x2fcbce40, SUC_lexicon_tags_38 },
    { 0x5fabce42, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7cebce44, SUC_lexicon_tags_12 },
    { 0x2bebce45, SUC_lexicon_tags_1 },
    { 0x276bce44, SUC_lexicon_tags_2 },
    { 0x97ebce47, SUC_lexicon_tags_10 },
    { 0xfaebce48, SUC_lexicon_tags_37 },
    { 0x66cbce49, SUC_lexicon_tags_17 },
    { 0x836bce44, SUC_lexicon_tags_25 },
    { 0x3acbce45, SUC_lexicon_tags_47 },
    { 0xf9cbce4c, SUC_lexicon_tags_12 },
    { 0x93abce48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8dcbce4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8bce54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa02bce56, SUC_lexicon_tags_10 },
    { 0x9d2bce56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0bce5b, SUC_lexicon_tags_37 },
    { 0x58cbce5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cbce62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760bce66, SUC_lexicon_tags_55 },
    { 0x91abce67, SUC_lexicon_tags_63 },
    { 0x424bce68, SUC_lexicon_tags_5 },
    { 0xd2bce68, SUC_lexicon_tags_29 },
    { 0x41cbce6a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bce6d, SUC_lexicon_tags_13 },
    { 0xb96bce6e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x798bce70, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56bce74, SUC_lexicon_tags_1 },
    { 0x9f0bce75, SUC_lexicon_tags_7 },
    { 0xb3ebce76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe0cbce78, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cbce7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10bce80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x734bce82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf2bce84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64ebce86, SUC_lexicon_tags_37 },
    { 0x8a4bce86, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x154bce8a, SUC_lexicon_tags_25 },
    { 0xd3abce8b, SUC_lexicon_tags_11 },
    { 0x78cbce8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e2bce8e, SUC_lexicon_tags_9 },
    { 0xccbce8f, SUC_lexicon_tags_45 },
    { 0xc26bce90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x108bce92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2abce94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0bce97, SUC_lexicon_tags_234 },
    { 0xffcbce98, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8bce9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6bcea0, SUC_lexicon_tags_38 },
    { 0x62cbcea1, SUC_lexicon_tags_43 },
    { 0xe82bcea0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x970bcea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6bcea8, SUC_lexicon_tags_37 },
    { 0x43cbcea8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a0bceab, SUC_lexicon_tags_25 },
    { 0x55ebceac, SUC_lexicon_tags_16 },
    { 0xe50bcead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934bceb1, SUC_lexicon_tags_7 },
    { 0xc42bceb1, SUC_lexicon_tags_1 },
    { 0x9bcbceb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420bceb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4bcebb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6bcec0, SUC_lexicon_tags_13 },
    { 0x126bcec0, SUC_lexicon_tags_30 },
    { 0x7a4bcec2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764bceca, SUC_lexicon_tags_27 },
    { 0x124bcecb, SUC_lexicon_tags_11 },
    { 0x44cbceca, SUC_lexicon_tags_37 },
    { 0x842bcecb, SUC_lexicon_tags_37 },
    { 0xc0bcece, SUC_lexicon_tags_29 },
    { 0xd50bcecf, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ebced3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaabced6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0bcedb, SUC_lexicon_tags_26 },
    { 0xfc0bcedc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bcee6, SUC_lexicon_tags_15 },
    { 0xc4ebcee6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafabceea, SUC_lexicon_tags_45 },
    { 0xf58bceeb, SUC_lexicon_tags_336 },
    { 0x734bceea, SUC_lexicon_tags_43 },
    { 0xc5cbceec, SUC_lexicon_tags_37 },
    { 0xc76bceec, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4ecbcef0, SUC_lexicon_tags_61 },
    { 0x5d8bcef1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42bcef4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54abcf01, SUC_lexicon_tags_17 },
    { 0xea2bcf02, SUC_lexicon_tags_23 },
    { 0x9f8bcf03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cbcf07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74bcf0d, SUC_lexicon_tags_27 },
    { 0x8aabcf0d, SUC_lexicon_tags_39 },
    { 0xe66bcf0f, SUC_lexicon_tags_15 },
    { 0xaf0bcf10, SUC_lexicon_tags_11 },
    { 0x9bcbcf11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcbcf19, SUC_lexicon_tags_172 },
    { 0xe6abcf19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278bcf1f, SUC_lexicon_tags_1 },
    { 0x5cbcf1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x478bcf22, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3abcf25, SUC_lexicon_tags_19 },
    { 0xabcbcf26, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x85cbcf29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8abcf2b, SUC_lexicon_tags_47 },
    { 0x8d4bcf2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x596bcf2f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1abcf34, SUC_lexicon_tags_11 },
    { 0xf3abcf34, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcd6bcf37, SUC_lexicon_tags_2 },
    { 0x8babcf37, SUC_lexicon_tags_0 },
    { 0x7d4bcf38, SUC_lexicon_tags_49 },
    { 0x6b0bcf3a, SUC_lexicon_tags_15 },
    { 0xaf2bcf3a, SUC_lexicon_tags_16 },
    { 0xb6bcf3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10abcf3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x810bcf40, SUC_lexicon_tags_37 },
    { 0xd60bcf40, SUC_lexicon_tags_49 },
    { 0x1aebcf42, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeacbcf44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778bcf48, SUC_lexicon_tags_19 },
    { 0xbfabcf49, SUC_lexicon_tags_38 },
    { 0xe6bcf49, SUC_lexicon_tags_37 },
    { 0x91ebcf4b, SUC_lexicon_tags_1 },
    { 0xf00bcf4b, SUC_lexicon_tags_15 },
    { 0x19ebcf4d, SUC_lexicon_tags_23 },
    { 0xedabcf4c, SUC_lexicon_tags_29 },
    { 0x58abcf4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0bcf52, SUC_lexicon_tags_104 },
    { 0x7fcbcf52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454bcf57, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cbcf5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ebcf5e, SUC_lexicon_tags_19 },
    { 0x33ebcf5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe0bcf61, SUC_lexicon_tags_87 },
    { 0x8d4bcf62, SUC_lexicon_tags_57 },
    { 0x676bcf62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26bcf69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6bebcf6b, SUC_lexicon_tags_7 },
    { 0xc2ebcf6b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4bcf6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xed4bcf71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9abcf73, SUC_lexicon_tags_12 },
    { 0x57ebcf73, SUC_lexicon_tags_2 },
    { 0xe80bcf75, SUC_lexicon_tags_15 },
    { 0xf96bcf75, SUC_lexicon_tags_15 },
    { 0xec8bcf77, SUC_lexicon_tags_70 },
    { 0x528bcf75, SUC_lexicon_tags_37 },
    { 0x542bcf74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x678bcf7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0bcf82, SUC_lexicon_tags_7 },
    { 0xfacbcf83, SUC_lexicon_tags_0 },
    { 0xad8bcf82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50bcf89, SUC_lexicon_tags_0 },
    { 0xccabcf8a, SUC_lexicon_tags_0 },
    { 0xd3ebcf8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad0bcf8d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cbcf92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8a4bcf94, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeabcf9b, SUC_lexicon_tags_16 },
    { 0xcecbcf9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x162bcf9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140bcfa3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xba6bcfa5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cbcfa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82bcfad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6bcfb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x256bcfb3, SUC_lexicon_tags_25 },
    { 0x49cbcfb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cbcfbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ebcfc0, SUC_lexicon_tags_1 },
    { 0x80abcfc1, SUC_lexicon_tags_37 },
    { 0x6e8bcfc1, SUC_lexicon_tags_12 },
    { 0xd28bcfc1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1febcfc5, SUC_lexicon_tags_51 },
    { 0x9ebcfc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ebcfcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ebcfce, SUC_lexicon_tags_12 },
    { 0xdacbcfce, SUC_lexicon_tags_30 },
    { 0xeabcfcf, SUC_lexicon_tags_19 },
    { 0x47cbcfcf, SUC_lexicon_tags_17 },
    { 0xa7ebcfd1, SUC_lexicon_tags_2 },
    { 0x96abcfd3, SUC_lexicon_tags_7 },
    { 0xb60bcfd4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2e4bcfd6, SUC_lexicon_tags_11 },
    { 0x9a2bcfd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62bcfdb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2bcfdf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04bcfe2, SUC_lexicon_tags_39 },
    { 0xa0ebcfe2, SUC_lexicon_tags_0 },
    { 0xa4ebcfe4, SUC_lexicon_tags_17 },
    { 0xb20bcfe5, SUC_lexicon_tags_14 },
    { 0xc32bcfe6, SUC_lexicon_tags_7 },
    { 0x5aebcfe7, SUC_lexicon_tags_11 },
    { 0xef2bcfe5, SUC_lexicon_tags_27 },
    { 0xab0bcfe9, SUC_lexicon_tags_25 },
    { 0x322bcfea, SUC_lexicon_tags_47 },
    { 0x5e4bcfea, SUC_lexicon_tags_15 },
    { 0x366bcfe7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52bcfee, SUC_lexicon_tags_0 },
    { 0x85ebcfef, SUC_lexicon_tags_60 },
    { 0xb7cbcfee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae4bcff2, SUC_lexicon_tags_49 },
    { 0x5f8bcff3, SUC_lexicon_tags_7 },
    { 0x6c0bcff4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccbcff8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22cbcffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x606bcffc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18abd001, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54bd004, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd008, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2abd00b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7abd00f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fabd011, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72bd014, SUC_lexicon_tags_63 },
    { 0xa24bd014, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cbd018, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x144bd01a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x356bd01d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x824bd020, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe42bd022, SUC_lexicon_tags_2 },
    { 0x3eebd023, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95cbd025, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180bd02b, SUC_lexicon_tags_68 },
    { 0x66abd02b, SUC_lexicon_tags_7 },
    { 0x50ebd02d, SUC_lexicon_tags_37 },
    { 0x4ecbd02e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750bd033, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8bd037, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92ebd039, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2bd040, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2bd049, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x184bd04b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x724bd04d, SUC_lexicon_tags_14 },
    { 0x474bd04e, SUC_lexicon_tags_7 },
    { 0x83abd04d, SUC_lexicon_tags_3 },
    { 0x182bd050, SUC_lexicon_tags_9 },
    { 0xa6abd050, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66ebd053, SUC_lexicon_tags_39 },
    { 0x582bd054, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eabd057, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348bd05c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cbd060, SUC_lexicon_tags_31 },
    { 0xfcebd061, SUC_lexicon_tags_68 },
    { 0xb14bd062, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadabd065, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ebd06b, SUC_lexicon_tags_17 },
    { 0xd78bd06b, SUC_lexicon_tags_37 },
    { 0x684bd06c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x606bd06f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2bd072, SUC_lexicon_tags_43 },
    { 0xa2bd072, SUC_lexicon_tags_37 },
    { 0xcacbd074, SUC_lexicon_tags_8 },
    { 0xe86bd075, SUC_lexicon_tags_8 },
    { 0xf44bd075, SUC_lexicon_tags_1 },
    { 0x9b0bd077, SUC_lexicon_tags_87 },
    { 0xe30bd078, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6bd082, SUC_lexicon_tags_32 },
    { 0x3d8bd083, SUC_lexicon_tags_47 },
    { 0xad2bd084, SUC_lexicon_tags_15 },
    { 0xcb2bd085, SUC_lexicon_tags_2 },
    { 0xe08bd085, SUC_lexicon_tags_70 },
    { 0x17ebd082, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebd08c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aebd096, SUC_lexicon_tags_21 },
    { 0xed6bd097, SUC_lexicon_tags_27 },
    { 0x7cbd098, SUC_lexicon_tags_25 },
    { 0x1fcbd098, SUC_lexicon_tags_28 },
    { 0x78abd096, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x162bd09d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f8bd09f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6bd0a2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x8b8bd0a4, SUC_lexicon_tags_15 },
    { 0xed6bd0a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ebd0ab, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82abd0b0, SUC_lexicon_tags_17 },
    { 0xc12bd0b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeabd0b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x422bd0b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0bd0be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x708bd0c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a8bd0c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cbd0c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x274bd0c7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630bd0cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x118bd0ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8abd0d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xeb0bd0da, SUC_lexicon_tags_15 },
    { 0xb0abd0db, SUC_lexicon_tags_1 },
    { 0x2eebd0db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd78bd0de, SUC_lexicon_tags_4 },
    { 0x906bd0de, SUC_lexicon_tags_12 },
    { 0xf40bd0e0, SUC_lexicon_tags_7 },
    { 0xc7abd0e0, SUC_lexicon_tags_25 },
    { 0x942bd0e1, SUC_lexicon_tags_37 },
    { 0x24abd0e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6abd0e6, SUC_lexicon_tags_37 },
    { 0x550bd0e7, SUC_lexicon_tags_1 },
    { 0xa76bd0e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd4bd0ea, SUC_lexicon_tags_82 },
    { 0x4d4bd0eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382bd0f0, SUC_lexicon_tags_19 },
    { 0x98cbd0f0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eabd104, SUC_lexicon_tags_65 },
    { 0xbe8bd105, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x11abd107, SUC_lexicon_tags_45 },
    { 0xe5ebd107, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310bd10c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5abd10e, SUC_lexicon_tags_29 },
    { 0x9cebd10e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a4bd111, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0bd114, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdecbd116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4bd11a, SUC_lexicon_tags_23 },
    { 0x6e4bd11b, SUC_lexicon_tags_17 },
    { 0x538bd11c, SUC_lexicon_tags_12 },
    { 0x18abd11c, SUC_lexicon_tags_7 },
    { 0x7d2bd11c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cbd124, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x380bd126, SUC_lexicon_tags_14 },
    { 0xa54bd126, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcbd12d, SUC_lexicon_tags_23 },
    { 0xe36bd12e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144bd132, SUC_lexicon_tags_15 },
    { 0xa16bd132, SUC_lexicon_tags_1 },
    { 0x824bd134, SUC_lexicon_tags_30 },
    { 0xfcebd134, SUC_lexicon_tags_87 },
    { 0xaccbd132, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1abd139, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76bd13f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cbd142, SUC_lexicon_tags_31 },
    { 0x466bd142, SUC_lexicon_tags_63 },
    { 0x492bd144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638bd148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ebd14b, SUC_lexicon_tags_31 },
    { 0x29abd14c, SUC_lexicon_tags_37 },
    { 0x83ebd14d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe2bd14f, SUC_lexicon_tags_16 },
    { 0xf04bd14f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976bd157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x286bd15a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96bd15f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebd163, SUC_lexicon_tags_74 },
    { 0x7eebd163, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x308bd166, SUC_lexicon_tags_17 },
    { 0xcaabd167, SUC_lexicon_tags_22 },
    { 0xeaabd168, SUC_lexicon_tags_33 },
    { 0x140bd169, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x936bd16c, SUC_lexicon_tags_60 },
    { 0x6b2bd16d, SUC_lexicon_tags_7 },
    { 0x594bd16e, SUC_lexicon_tags_37 },
    { 0xb46bd16e, SUC_lexicon_tags_60 },
    { 0xcb6bd16c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbebd176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf30bd178, SUC_lexicon_tags_11 },
    { 0x360bd179, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd8ebd17b, SUC_lexicon_tags_7 },
    { 0xc3abd17b, SUC_lexicon_tags_23 },
    { 0x392bd17d, SUC_lexicon_tags_36 },
    { 0x91cbd17e, SUC_lexicon_tags_38 },
    { 0xccebd17f, SUC_lexicon_tags_25 },
    { 0x92cbd17b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aebd186, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x348bd188, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c8bd18a, SUC_lexicon_tags_11 },
    { 0x554bd18b, SUC_lexicon_tags_84 },
    { 0xf36bd18b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8abd18e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0bd193, SUC_lexicon_tags_25 },
    { 0x62bd193, SUC_lexicon_tags_61 },
    { 0x858bd193, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94bd198, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2bd19b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd86bd19d, SUC_lexicon_tags_58 },
    { 0x97ebd19e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd94bd1a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8febd1a4, SUC_lexicon_tags_1 },
    { 0x408bd1a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eabd1a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbaebd1aa, SUC_lexicon_tags_48 },
    { 0x8a4bd1ab, SUC_lexicon_tags_85 },
    { 0xbe4bd1ab, SUC_lexicon_tags_25 },
    { 0xfaabd1ab, SUC_lexicon_tags_33 },
    { 0xd44bd1ab, SUC_lexicon_tags_1 },
    { 0x554bd1ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x998bd1b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbd1bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42bd1c2, SUC_lexicon_tags_11 },
    { 0xe32bd1c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930bd1c7, SUC_lexicon_tags_14 },
    { 0x35cbd1c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfb4bd1ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776bd1cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80ebd1d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbabd1d5, SUC_lexicon_tags_37 },
    { 0x92ebd1d5, SUC_lexicon_tags_11 },
    { 0x534bd1d7, SUC_lexicon_tags_27 },
    { 0x326bd1d6, SUC_lexicon_tags_37 },
    { 0x95abd1d7, SUC_lexicon_tags_37 },
    { 0x55ebd1d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc0bd1dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49abd1df, SUC_lexicon_tags_23 },
    { 0xf50bd1df, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccbd1e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bd1eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdacbd1ee, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x56bd1f1, SUC_lexicon_tags_1 },
    { 0x6f8bd1f1, SUC_lexicon_tags_2 },
    { 0x8b8bd1f3, SUC_lexicon_tags_29 },
    { 0xac2bd1f1, SUC_lexicon_tags_63 },
    { 0xefebd1f5, SUC_lexicon_tags_30 },
    { 0x5c8bd1f5, SUC_lexicon_tags_11 },
    { 0x506bd1f6, SUC_lexicon_tags_27 },
    { 0x844bd1f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dabd205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccbd20e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x226bd210, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ebd217, SUC_lexicon_tags_70 },
    { 0x818bd218, SUC_lexicon_tags_59 },
    { 0x3c8bd219, SUC_lexicon_tags_35 },
    { 0xdccbd218, SUC_lexicon_tags_47 },
    { 0x14ebd21b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x536bd21d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdabd21f, SUC_lexicon_tags_25 },
    { 0xd88bd220, SUC_lexicon_tags_45 },
    { 0x276bd221, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2bd224, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d6bd226, SUC_lexicon_tags_17 },
    { 0x948bd227, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70ebd229, SUC_lexicon_tags_27 },
    { 0x29ebd229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x104bd22d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbbcbd22f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940bd233, SUC_lexicon_tags_7 },
    { 0xc30bd233, SUC_lexicon_tags_85 },
    { 0xef2bd235, SUC_lexicon_tags_2 },
    { 0x998bd236, SUC_lexicon_tags_7 },
    { 0xcd0bd233, SUC_lexicon_tags_27 },
    { 0x6c4bd236, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46bd23d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eabd240, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cbd244, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522bd24a, SUC_lexicon_tags_11 },
    { 0x9e8bd24b, SUC_lexicon_tags_48 },
    { 0x1b8bd24c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18cbd24e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x488bd250, SUC_lexicon_tags_1 },
    { 0xecebd250, SUC_lexicon_tags_34 },
    { 0xba0bd252, SUC_lexicon_tags_68 },
    { 0x48ebd253, SUC_lexicon_tags_8 },
    { 0x444bd253, SUC_lexicon_tags_45 },
    { 0xc1ebd254, SUC_lexicon_tags_11 },
    { 0x3b0bd252, SUC_lexicon_tags_14 },
    { 0x670bd252, SUC_lexicon_tags_11 },
    { 0x4f2bd253, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40bd25c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb4bd25e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef0bd260, SUC_lexicon_tags_28 },
    { 0x4ecbd260, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752bd265, SUC_lexicon_tags_2 },
    { 0x618bd266, SUC_lexicon_tags_14 },
    { 0x49cbd267, SUC_lexicon_tags_12 },
    { 0xc0cbd266, SUC_lexicon_tags_66 },
    { 0xd6ebd269, SUC_lexicon_tags_27 },
    { 0xaa2bd26a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3abd272, SUC_lexicon_tags_107 },
    { 0x27cbd273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0bd276, SUC_lexicon_tags_8 },
    { 0x1ecbd277, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaabd27e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4bd284, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22bd289, SUC_lexicon_tags_28 },
    { 0x7acbd289, SUC_lexicon_tags_39 },
    { 0x4b8bd289, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x796bd28e, SUC_lexicon_tags_63 },
    { 0x584bd28e, SUC_lexicon_tags_11 },
    { 0x180bd290, SUC_lexicon_tags_27 },
    { 0xfd8bd28e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362bd297, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522bd29b, SUC_lexicon_tags_26 },
    { 0xd3ebd29b, SUC_lexicon_tags_218 },
    { 0xd3ebd29d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20bd2a0, SUC_lexicon_tags_49 },
    { 0xdccbd2a0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacbd2a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cbd2ab, SUC_lexicon_tags_17 },
    { 0x46abd2ac, SUC_lexicon_tags_27 },
    { 0x45cbd2ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8bd2b1, SUC_lexicon_tags_21 },
    { 0x8d6bd2b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4bd2b8, SUC_lexicon_tags_6 },
    { 0x668bd2b9, SUC_lexicon_tags_17 },
    { 0x67ebd2ba, SUC_lexicon_tags_2 },
    { 0xd1abd2bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7debd2bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cbd2c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398bd2cc, SUC_lexicon_tags_7 },
    { 0x764bd2cd, SUC_lexicon_tags_7 },
    { 0xa44bd2cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40bd2d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7abd2da, SUC_lexicon_tags_23 },
    { 0xd2ebd2da, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cbd2e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcacbd2e2, SUC_lexicon_tags_4 },
    { 0xe08bd2e3, SUC_lexicon_tags_2 },
    { 0x3ecbd2e4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4eebd2e6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfebd2eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72cbd2ed, SUC_lexicon_tags_49 },
    { 0x4acbd2ee, SUC_lexicon_tags_15 },
    { 0x49ebd2ed, SUC_lexicon_tags_107 },
    { 0x474bd2ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8bd2f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cbd308, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x37cbd30a, SUC_lexicon_tags_92 },
    { 0xa44bd30b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610bd310, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x43ebd312, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x756bd314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x312bd317, SUC_lexicon_tags_30 },
    { 0x588bd318, SUC_lexicon_tags_45 },
    { 0xaa8bd318, SUC_lexicon_tags_12 },
    { 0x96ebd31a, SUC_lexicon_tags_7 },
    { 0xb8abd318, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcbd323, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eabd326, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc0bd328, SUC_lexicon_tags_120 },
    { 0x438bd329, SUC_lexicon_tags_25 },
    { 0xc92bd32a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf12bd32d, SUC_lexicon_tags_15 },
    { 0x1eabd32e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70abd330, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28bd333, SUC_lexicon_tags_25 },
    { 0xababd334, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x286bd337, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f4bd339, SUC_lexicon_tags_37 },
    { 0x5b8bd339, SUC_lexicon_tags_55 },
    { 0x336bd33b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0abd340, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7f4bd342, SUC_lexicon_tags_35 },
    { 0x302bd343, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97cbd347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcbd34e, SUC_lexicon_tags_23 },
    { 0xd04bd34e, SUC_lexicon_tags_10 },
    { 0x73ebd350, SUC_lexicon_tags_0 },
    { 0x426bd351, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x854bd354, SUC_lexicon_tags_9 },
    { 0xa0cbd355, SUC_lexicon_tags_31 },
    { 0x74ebd355, SUC_lexicon_tags_19 },
    { 0x2d6bd357, SUC_lexicon_tags_77 },
    { 0x40cbd358, SUC_lexicon_tags_26 },
    { 0xb7cbd358, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6a2bd35b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5babd35d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ebd363, SUC_lexicon_tags_23 },
    { 0x504bd363, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40bd368, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4bd370, SUC_lexicon_tags_27 },
    { 0xbc6bd371, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a2bd373, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31cbd375, SUC_lexicon_tags_59 },
    { 0xf5abd375, SUC_lexicon_tags_1 },
    { 0x738bd377, SUC_lexicon_tags_15 },
    { 0x1d4bd377, SUC_lexicon_tags_47 },
    { 0x554bd379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4abd37b, SUC_lexicon_tags_7 },
    { 0xa7abd37c, SUC_lexicon_tags_25 },
    { 0x356bd37d, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bd381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0bd385, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588bd38c, SUC_lexicon_tags_37 },
    { 0xfd2bd38c, SUC_lexicon_tags_11 },
    { 0x3babd38c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582bd394, SUC_lexicon_tags_16 },
    { 0xd12bd394, SUC_lexicon_tags_87 },
    { 0xa68bd396, SUC_lexicon_tags_17 },
    { 0xc1abd397, SUC_lexicon_tags_55 },
    { 0x370bd397, SUC_lexicon_tags_37 },
    { 0x43ebd399, SUC_lexicon_tags_25 },
    { 0x1b2bd399, SUC_lexicon_tags_31 },
    { 0x9abd39b, SUC_lexicon_tags_1 },
    { 0xb42bd39a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x674bd39e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x68cbd3a0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x656bd3a3, SUC_lexicon_tags_82 },
    { 0x100bd3a3, SUC_lexicon_tags_31 },
    { 0x68abd3a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x864bd3a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8bd3b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x528bd3b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4bd3b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84abd3bb, SUC_lexicon_tags_17 },
    { 0x4ebd3bc, SUC_lexicon_tags_19 },
    { 0xa2abd3bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x52cbd3bf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70bd3c2, SUC_lexicon_tags_31 },
    { 0xfc6bd3c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206bd3c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1abd3cb, SUC_lexicon_tags_2 },
    { 0x702bd3cc, SUC_lexicon_tags_26 },
    { 0x362bd3cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xac6bd3cf, SUC_lexicon_tags_2 },
    { 0xef6bd3d0, SUC_lexicon_tags_14 },
    { 0x338bd3d0, SUC_lexicon_tags_57 },
    { 0x648bd3d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2bd3d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcbd3d9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10bd3dc, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x35cbd3de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06bd3e1, SUC_lexicon_tags_4 },
    { 0x78ebd3e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8babd3e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ebd3ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f0bd3ec, SUC_lexicon_tags_15 },
    { 0x756bd3ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cbd3f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dabd3f8, SUC_lexicon_tags_11 },
    { 0x24bd3f8, SUC_lexicon_tags_30 },
    { 0x9b6bd3fa, SUC_lexicon_tags_29 },
    { 0x912bd3fb, SUC_lexicon_tags_28 },
    { 0xf28bd3fc, SUC_lexicon_tags_77 },
    { 0xcdebd3fd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eabd401, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cbd409, SUC_lexicon_tags_1 },
    { 0xf54bd409, SUC_lexicon_tags_37 },
    { 0xc4bd40b, SUC_lexicon_tags_37 },
    { 0xd8bd40c, SUC_lexicon_tags_2 },
    { 0x874bd40c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462bd411, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584bd41a, SUC_lexicon_tags_17 },
    { 0x54cbd41b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x554bd41d, SUC_lexicon_tags_15 },
    { 0x3b6bd41e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa26bd420, SUC_lexicon_tags_31 },
    { 0x118bd421, SUC_lexicon_tags_11 },
    { 0x8eebd420, SUC_lexicon_tags_25 },
    { 0xe2abd423, SUC_lexicon_tags_17 },
    { 0xc5ebd424, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592bd42a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x416bd42c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x600bd42f, SUC_lexicon_tags_1 },
    { 0xe3abd42f, SUC_lexicon_tags_8 },
    { 0x99ebd430, SUC_lexicon_tags_10 },
    { 0x82abd432, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfcebd434, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5cbd436, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332bd43c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7cbd43e, SUC_lexicon_tags_15 },
    { 0x1d2bd43f, SUC_lexicon_tags_17 },
    { 0x1fabd43e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1cbd442, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x720bd445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ebd449, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74bd44e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a0bd450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbabd458, SUC_lexicon_tags_37 },
    { 0x812bd459, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02bd45c, SUC_lexicon_tags_43 },
    { 0x1e4bd45c, SUC_lexicon_tags_9 },
    { 0x386bd45c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1f2bd460, SUC_lexicon_tags_15 },
    { 0x6f2bd460, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2bd466, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cbd46d, SUC_lexicon_tags_6 },
    { 0xe40bd46d, SUC_lexicon_tags_8 },
    { 0xdc8bd46f, SUC_lexicon_tags_2 },
    { 0x45ebd46e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecebd472, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e4bd474, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0bd477, SUC_lexicon_tags_37 },
    { 0xd90bd478, SUC_lexicon_tags_11 },
    { 0xbd479, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd38bd47b, SUC_lexicon_tags_26 },
    { 0xd72bd47c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bd487, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6bd48e, SUC_lexicon_tags_27 },
    { 0xfc0bd48e, SUC_lexicon_tags_37 },
    { 0x182bd48e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x216bd492, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86bd495, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776bd49c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdabd4a0, SUC_lexicon_tags_17 },
    { 0xdb2bd4a1, SUC_lexicon_tags_16 },
    { 0x3d8bd4a2, SUC_lexicon_tags_12 },
    { 0xbfabd4a1, SUC_lexicon_tags_129 },
    { 0x8f4bd4a3, SUC_lexicon_tags_37 },
    { 0x15ebd4a5, SUC_lexicon_tags_19 },
    { 0xad2bd4a1, SUC_lexicon_tags_27 },
    { 0x422bd4a0, SUC_lexicon_tags_49 },
    { 0x866bd4a5, SUC_lexicon_tags_3 },
    { 0xb02bd4a5, SUC_lexicon_tags_17 },
    { 0x778bd4aa, SUC_lexicon_tags_47 },
    { 0xabebd4ab, SUC_lexicon_tags_45 },
    { 0xbbcbd4ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93abd4ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bd4b2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66bd4ba, SUC_lexicon_tags_0 },
    { 0xb20bd4bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5e2bd4bd, SUC_lexicon_tags_25 },
    { 0x1acbd4be, SUC_lexicon_tags_45 },
    { 0x5bcbd4be, SUC_lexicon_tags_10 },
    { 0x7e8bd4c0, SUC_lexicon_tags_37 },
    { 0xd18bd4c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8bd4c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe18bd4c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcebd4cb, SUC_lexicon_tags_1014 },
    { 0, NULL },
    { 0xde8bd4cd, SUC_lexicon_tags_12 },
    { 0x594bd4ce, SUC_lexicon_tags_32 },
    { 0x6a4bd4cf, SUC_lexicon_tags_37 },
    { 0x95cbd4d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8bd4d5, SUC_lexicon_tags_26 },
    { 0xb32bd4d6, SUC_lexicon_tags_11 },
    { 0x5b4bd4d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378bd4db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4bd4de, SUC_lexicon_tags_37 },
    { 0xf58bd4df, SUC_lexicon_tags_1 },
    { 0x7bcbd4df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5abd4e3, SUC_lexicon_tags_55 },
    { 0x3d8bd4e4, SUC_lexicon_tags_15 },
    { 0xd46bd4e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebd4ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x42abd4f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00bd4f6, SUC_lexicon_tags_159 },
    { 0x90ebd4f6, SUC_lexicon_tags_31 },
    { 0x6b0bd4f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30abd4fa, SUC_lexicon_tags_37 },
    { 0x77ebd4fa, SUC_lexicon_tags_70 },
    { 0xad2bd4fb, SUC_lexicon_tags_2 },
    { 0xb76bd4fd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6bd500, SUC_lexicon_tags_756 },
    { 0x83abd501, SUC_lexicon_tags_30 },
    { 0x366bd502, SUC_lexicon_tags_0 },
    { 0xdcbd503, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e8bd505, SUC_lexicon_tags_20 },
    { 0x5b6bd505, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc88bd508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x804bd50a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86bd50d, SUC_lexicon_tags_9 },
    { 0x906bd50e, SUC_lexicon_tags_11 },
    { 0x296bd50e, SUC_lexicon_tags_25 },
    { 0x93ebd510, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ebd513, SUC_lexicon_tags_33 },
    { 0x74bd514, SUC_lexicon_tags_38 },
    { 0x714bd514, SUC_lexicon_tags_49 },
    { 0x7e0bd515, SUC_lexicon_tags_37 },
    { 0x568bd517, SUC_lexicon_tags_37 },
    { 0xedebd518, SUC_lexicon_tags_1 },
    { 0x272bd518, SUC_lexicon_tags_17 },
    { 0xcccbd51a, SUC_lexicon_tags_23 },
    { 0xe84bd51b, SUC_lexicon_tags_37 },
    { 0x732bd513, SUC_lexicon_tags_25 },
    { 0x336bd51d, SUC_lexicon_tags_2 },
    { 0x57abd51b, SUC_lexicon_tags_37 },
    { 0x3f4bd51f, SUC_lexicon_tags_37 },
    { 0x41cbd517, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4bd523, SUC_lexicon_tags_7 },
    { 0x5a2bd524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcbd529, SUC_lexicon_tags_46 },
    { 0xa38bd529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x482bd52c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x436bd52e, SUC_lexicon_tags_11 },
    { 0x5fabd52f, SUC_lexicon_tags_39 },
    { 0xfcabd52f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7babd532, SUC_lexicon_tags_47 },
    { 0xd82bd533, SUC_lexicon_tags_15 },
    { 0x49ebd532, SUC_lexicon_tags_33 },
    { 0x776bd532, SUC_lexicon_tags_38 },
    { 0x982bd532, SUC_lexicon_tags_37 },
    { 0x4b0bd537, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10bd53b, SUC_lexicon_tags_12 },
    { 0xbc6bd53b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x990bd53f, SUC_lexicon_tags_17 },
    { 0x28ebd540, SUC_lexicon_tags_45 },
    { 0xba4bd53f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62bd544, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bd548, SUC_lexicon_tags_37 },
    { 0x2d0bd549, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6e6bd54b, SUC_lexicon_tags_11 },
    { 0x422bd54c, SUC_lexicon_tags_104 },
    { 0xd02bd54d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ebd550, SUC_lexicon_tags_3 },
    { 0x682bd551, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28bd554, SUC_lexicon_tags_62 },
    { 0x3e2bd555, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4bd55b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5cebd55d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14bd562, SUC_lexicon_tags_45 },
    { 0xa36bd563, SUC_lexicon_tags_17 },
    { 0x1aabd563, SUC_lexicon_tags_1 },
    { 0x626bd562, SUC_lexicon_tags_17 },
    { 0xe32bd562, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bd56a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x486bd56c, SUC_lexicon_tags_35 },
    { 0x74ebd56d, SUC_lexicon_tags_8 },
    { 0xf0ebd56e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x828bd571, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858bd575, SUC_lexicon_tags_7 },
    { 0x68abd576, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56bd57b, SUC_lexicon_tags_17 },
    { 0xa12bd57c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x144bd57f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x760bd582, SUC_lexicon_tags_11 },
    { 0x82abd583, SUC_lexicon_tags_6 },
    { 0x6f6bd583, SUC_lexicon_tags_35 },
    { 0xc58bd584, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286bd58c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2abd592, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ebd59c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb88bd59e, SUC_lexicon_tags_15 },
    { 0x586bd59f, SUC_lexicon_tags_2 },
    { 0xab0bd5a0, SUC_lexicon_tags_22 },
    { 0x760bd59e, SUC_lexicon_tags_70 },
    { 0x75ebd5a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0bd5a5, SUC_lexicon_tags_2 },
    { 0xc32bd5a6, SUC_lexicon_tags_45 },
    { 0xf5ebd5a7, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8bd5aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb4bd5ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abd5b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeebd5b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4abd5bc, SUC_lexicon_tags_1 },
    { 0x870bd5bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8bd5c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffebd5c7, SUC_lexicon_tags_7 },
    { 0x97ebd5c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe40bd5ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaebd5d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x344bd5d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x326bd5d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cbd5dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3cbd5de, SUC_lexicon_tags_2 },
    { 0xddebd5df, SUC_lexicon_tags_17 },
    { 0xf52bd5df, SUC_lexicon_tags_4 },
    { 0x908bd5e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x198bd5e3, SUC_lexicon_tags_38 },
    { 0xd9cbd5e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394bd5e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42ebd5ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xacabd5ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x892bd5ee, SUC_lexicon_tags_187 },
    { 0x904bd5ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf64bd5f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x27ebd5f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc12bd5f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914bd5fa, SUC_lexicon_tags_9 },
    { 0x18abd5fa, SUC_lexicon_tags_1 },
    { 0xe42bd5fc, SUC_lexicon_tags_82 },
    { 0x938bd5fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932bd601, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48bd605, SUC_lexicon_tags_32 },
    { 0x3f0bd605, SUC_lexicon_tags_8 },
    { 0xce4bd606, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4bd60a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bd60e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68bd613, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc18bd616, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cbd61a, SUC_lexicon_tags_15 },
    { 0xbbabd61a, SUC_lexicon_tags_61 },
    { 0x63ebd61a, SUC_lexicon_tags_14 },
    { 0xe96bd61a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8bd621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadebd623, SUC_lexicon_tags_5 },
    { 0x36abd624, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6ebd626, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38bd62c, SUC_lexicon_tags_28 },
    { 0xe6bd62c, SUC_lexicon_tags_2 },
    { 0xc7abd62e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2bd632, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x534bd634, SUC_lexicon_tags_37 },
    { 0x2b6bd635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2bd638, SUC_lexicon_tags_12 },
    { 0xaabd639, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x606bd63c, SUC_lexicon_tags_17 },
    { 0xad8bd63d, SUC_lexicon_tags_12 },
    { 0x718bd63c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e4bd640, SUC_lexicon_tags_12 },
    { 0x25abd641, SUC_lexicon_tags_15 },
    { 0x8eebd642, SUC_lexicon_tags_77 },
    { 0x45cbd642, SUC_lexicon_tags_55 },
    { 0xb04bd644, SUC_lexicon_tags_14 },
    { 0x4d6bd645, SUC_lexicon_tags_1 },
    { 0x288bd646, SUC_lexicon_tags_8 },
    { 0x27ebd647, SUC_lexicon_tags_2 },
    { 0xd4abd641, SUC_lexicon_tags_19 },
    { 0xb28bd648, SUC_lexicon_tags_85 },
    { 0x738bd64a, SUC_lexicon_tags_26 },
    { 0x840bd64b, SUC_lexicon_tags_11 },
    { 0x52abd64c, SUC_lexicon_tags_27 },
    { 0x89ebd64d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d6bd64f, SUC_lexicon_tags_44 },
    { 0x5bebd64f, SUC_lexicon_tags_7 },
    { 0xfa4bd651, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0bd65a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcbd65e, SUC_lexicon_tags_1 },
    { 0xa20bd65e, SUC_lexicon_tags_7 },
    { 0xa8abd660, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6bd664, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd667, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6bd66a, SUC_lexicon_tags_9 },
    { 0x846bd66a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562bd671, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768bd675, SUC_lexicon_tags_33 },
    { 0x3d8bd675, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8ebd678, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ebd67b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ebd67e, SUC_lexicon_tags_11 },
    { 0xe00bd67f, SUC_lexicon_tags_43 },
    { 0x50abd67e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8bd688, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cbd68d, SUC_lexicon_tags_12 },
    { 0xe9abd68d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabebd692, SUC_lexicon_tags_33 },
    { 0x730bd693, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcf8bd695, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcbebd697, SUC_lexicon_tags_25 },
    { 0x1bcbd697, SUC_lexicon_tags_12 },
    { 0x562bd697, SUC_lexicon_tags_12 },
    { 0xce0bd699, SUC_lexicon_tags_11 },
    { 0xecabd69b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0bd69e, SUC_lexicon_tags_7 },
    { 0x3b4bd69e, SUC_lexicon_tags_25 },
    { 0xdeabd69e, SUC_lexicon_tags_26 },
    { 0x2cbd6a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x57cbd6a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444bd6aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc38bd6ac, SUC_lexicon_tags_27 },
    { 0x14ebd6ad, SUC_lexicon_tags_74 },
    { 0xcc0bd6ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8bd6b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a6bd6b6, SUC_lexicon_tags_351 },
    { 0x9c8bd6b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef2bd6b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2bd6c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794bd6c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32ebd6c9, SUC_lexicon_tags_15 },
    { 0xf18bd6c9, SUC_lexicon_tags_4 },
    { 0xa46bd6c9, SUC_lexicon_tags_28 },
    { 0x500bd6c9, SUC_lexicon_tags_11 },
    { 0xa1ebd6cd, SUC_lexicon_tags_23 },
    { 0xb5abd6ce, SUC_lexicon_tags_12 },
    { 0x32cbd6cf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x390bd6d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xebabd6d3, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xf66bd6d5, SUC_lexicon_tags_37 },
    { 0xfb6bd6d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550bd6db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8bd6df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7e8bd6e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x762bd6e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80bd6e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ebd6ed, SUC_lexicon_tags_17 },
    { 0x154bd6ed, SUC_lexicon_tags_37 },
    { 0xdb2bd6ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4bd6f4, SUC_lexicon_tags_15 },
    { 0x23abd6f4, SUC_lexicon_tags_11 },
    { 0xf9cbd6f6, SUC_lexicon_tags_37 },
    { 0xbf6bd6f7, SUC_lexicon_tags_84 },
    { 0x63ebd6f8, SUC_lexicon_tags_33 },
    { 0xd4ebd6f9, SUC_lexicon_tags_10 },
    { 0x7abd6f7, SUC_lexicon_tags_45 },
    { 0x578bd6fb, SUC_lexicon_tags_37 },
    { 0x38bd6fb, SUC_lexicon_tags_38 },
    { 0x82bd6fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4bd704, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bd707, SUC_lexicon_tags_2 },
    { 0x126bd707, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xec0bd70a, SUC_lexicon_tags_1 },
    { 0xc46bd70b, SUC_lexicon_tags_6 },
    { 0xffebd70b, SUC_lexicon_tags_25 },
    { 0x5bcbd70d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4bd710, SUC_lexicon_tags_47 },
    { 0xe74bd711, SUC_lexicon_tags_1 },
    { 0x85abd712, SUC_lexicon_tags_11 },
    { 0x7cbd711, SUC_lexicon_tags_2 },
    { 0xf46bd714, SUC_lexicon_tags_16 },
    { 0xbe6bd715, SUC_lexicon_tags_11 },
    { 0xd1cbd716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700bd71c, SUC_lexicon_tags_11 },
    { 0x44ebd71d, SUC_lexicon_tags_11 },
    { 0x8fabd71d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x490bd720, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x22abd723, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf3cbd725, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8bd728, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8bd72d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0bd732, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6bd735, SUC_lexicon_tags_21 },
    { 0x30bd736, SUC_lexicon_tags_24 },
    { 0x70bd735, SUC_lexicon_tags_56 },
    { 0x912bd736, SUC_lexicon_tags_37 },
    { 0x1ccbd739, SUC_lexicon_tags_7 },
    { 0x3c8bd73a, SUC_lexicon_tags_11 },
    { 0x198bd73a, SUC_lexicon_tags_8 },
    { 0x98bd73a, SUC_lexicon_tags_7 },
    { 0xf84bd73d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478bd742, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520bd748, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xeccbd74a, SUC_lexicon_tags_7 },
    { 0xd68bd74a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82abd750, SUC_lexicon_tags_35 },
    { 0x768bd750, SUC_lexicon_tags_1 },
    { 0xe98bd752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcbd756, SUC_lexicon_tags_12 },
    { 0xf78bd756, SUC_lexicon_tags_12 },
    { 0xe44bd758, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb68bd75a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf12bd75c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72bd763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2bd768, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2bd76e, SUC_lexicon_tags_9 },
    { 0x52ebd76f, SUC_lexicon_tags_37 },
    { 0xc74bd76e, SUC_lexicon_tags_37 },
    { 0x3debd771, SUC_lexicon_tags_63 },
    { 0xb1cbd772, SUC_lexicon_tags_40 },
    { 0x3a0bd773, SUC_lexicon_tags_25 },
    { 0x8bcbd774, SUC_lexicon_tags_11 },
    { 0xc5ebd76e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ebd77b, SUC_lexicon_tags_44 },
    { 0xd4ebd77b, SUC_lexicon_tags_536 },
    { 0x96cbd77d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630bd781, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6cbd783, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cbd786, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77abd78c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ebd791, SUC_lexicon_tags_84 },
    { 0x57abd791, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4bd796, SUC_lexicon_tags_25 },
    { 0x396bd797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c0bd799, SUC_lexicon_tags_11 },
    { 0xf56bd79a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0bd79d, SUC_lexicon_tags_30 },
    { 0x206bd79e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980bd7a2, SUC_lexicon_tags_16 },
    { 0x37ebd7a3, SUC_lexicon_tags_63 },
    { 0xe56bd7a3, SUC_lexicon_tags_23 },
    { 0x512bd7a5, SUC_lexicon_tags_7 },
    { 0xe58bd7a6, SUC_lexicon_tags_17 },
    { 0xbabd7a7, SUC_lexicon_tags_12 },
    { 0x900bd7a8, SUC_lexicon_tags_15 },
    { 0xdb6bd7a7, SUC_lexicon_tags_12 },
    { 0x7fcbd7a7, SUC_lexicon_tags_112 },
    { 0x144bd7a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200bd7b0, SUC_lexicon_tags_63 },
    { 0x824bd7b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecbd7b5, SUC_lexicon_tags_17 },
    { 0xe9ebd7b6, SUC_lexicon_tags_0 },
    { 0x5babd7b7, SUC_lexicon_tags_17 },
    { 0x80bd7b5, SUC_lexicon_tags_5 },
    { 0x2e8bd7b9, SUC_lexicon_tags_63 },
    { 0x564bd7ba, SUC_lexicon_tags_17 },
    { 0x7cebd7b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38bd7bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xad8bd7c1, SUC_lexicon_tags_1 },
    { 0xc8abd7c2, SUC_lexicon_tags_12 },
    { 0xf7abd7c3, SUC_lexicon_tags_28 },
    { 0xa52bd7c3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abd7c7, SUC_lexicon_tags_2 },
    { 0x3b6bd7c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ebd7cc, SUC_lexicon_tags_14 },
    { 0x4febd7cd, SUC_lexicon_tags_27 },
    { 0xd24bd7cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26cbd7d0, SUC_lexicon_tags_11 },
    { 0x50bd7d0, SUC_lexicon_tags_9 },
    { 0xec6bd7d2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932bd7d6, SUC_lexicon_tags_11 },
    { 0x6c4bd7d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cbd7dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x84cbd7df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56bd7e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00bd7e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4bd7e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770bd7ed, SUC_lexicon_tags_32 },
    { 0x21abd7ee, SUC_lexicon_tags_27 },
    { 0x18ebd7ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136bd7f3, SUC_lexicon_tags_30 },
    { 0x838bd7f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ebd7f7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x96abd7fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbabd7fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0bd801, SUC_lexicon_tags_37 },
    { 0x554bd801, SUC_lexicon_tags_63 },
    { 0x86bd802, SUC_lexicon_tags_7 },
    { 0x95ebd804, SUC_lexicon_tags_1 },
    { 0xc50bd805, SUC_lexicon_tags_38 },
    { 0x1ccbd806, SUC_lexicon_tags_11 },
    { 0xe2bd806, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff0bd809, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ebd80d, SUC_lexicon_tags_12 },
    { 0x7dabd80e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46abd814, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x914bd816, SUC_lexicon_tags_68 },
    { 0x692bd817, SUC_lexicon_tags_100 },
    { 0x46ebd818, SUC_lexicon_tags_11 },
    { 0x34ebd816, SUC_lexicon_tags_12 },
    { 0xb72bd819, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2abd820, SUC_lexicon_tags_31 },
    { 0xbfebd821, SUC_lexicon_tags_29 },
    { 0xd4cbd822, SUC_lexicon_tags_9 },
    { 0x644bd820, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92abd827, SUC_lexicon_tags_1 },
    { 0x62bd828, SUC_lexicon_tags_25 },
    { 0x79cbd828, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecbd82f, SUC_lexicon_tags_37 },
    { 0x2e8bd830, SUC_lexicon_tags_1 },
    { 0xfdabd830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d4bd833, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20bd836, SUC_lexicon_tags_15 },
    { 0x148bd837, SUC_lexicon_tags_26 },
    { 0x6eabd838, SUC_lexicon_tags_47 },
    { 0xb34bd839, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbebd83d, SUC_lexicon_tags_1 },
    { 0xb8bd83e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ebd842, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ebd846, SUC_lexicon_tags_15 },
    { 0xdfebd847, SUC_lexicon_tags_7 },
    { 0xd5ebd848, SUC_lexicon_tags_63 },
    { 0x85ebd849, SUC_lexicon_tags_0 },
    { 0xa56bd84a, SUC_lexicon_tags_25 },
    { 0x534bd84b, SUC_lexicon_tags_17 },
    { 0x3f8bd84c, SUC_lexicon_tags_37 },
    { 0x40abd846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd92bd84f, SUC_lexicon_tags_11 },
    { 0x5aebd850, SUC_lexicon_tags_37 },
    { 0x4bcbd851, SUC_lexicon_tags_1 },
    { 0xa8cbd852, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x372bd854, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ebd857, SUC_lexicon_tags_12 },
    { 0xebcbd858, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x936bd85a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8bd85f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512bd863, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aabd866, SUC_lexicon_tags_17 },
    { 0x646bd866, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710bd86c, SUC_lexicon_tags_25 },
    { 0xe36bd86d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90bd870, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74bd873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0bd87a, SUC_lexicon_tags_17 },
    { 0x9debd87b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02bd87f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebebd888, SUC_lexicon_tags_0 },
    { 0xf14bd889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0bd88d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80cbd88f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x550bd891, SUC_lexicon_tags_351 },
    { 0xadcbd892, SUC_lexicon_tags_17 },
    { 0x86cbd893, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd44bd895, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6eabd897, SUC_lexicon_tags_23 },
    { 0xf3cbd897, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fabd89b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0bd89e, SUC_lexicon_tags_29 },
    { 0xb14bd89f, SUC_lexicon_tags_12 },
    { 0x90cbd89e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2cabd8a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8bd8a7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x352bd8aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfabd8ad, SUC_lexicon_tags_60 },
    { 0xad0bd8ad, SUC_lexicon_tags_15 },
    { 0x5d4bd8af, SUC_lexicon_tags_63 },
    { 0xc2abd8b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90bd8b2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6bd8ba, SUC_lexicon_tags_3 },
    { 0xc42bd8ba, SUC_lexicon_tags_7 },
    { 0xee8bd8bc, SUC_lexicon_tags_37 },
    { 0x98cbd8bb, SUC_lexicon_tags_1 },
    { 0xda4bd8be, SUC_lexicon_tags_43 },
    { 0xd04bd8be, SUC_lexicon_tags_23 },
    { 0xe5abd8be, SUC_lexicon_tags_47 },
    { 0x664bd8c1, SUC_lexicon_tags_25 },
    { 0x2cebd8be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x500bd8c4, SUC_lexicon_tags_15 },
    { 0x854bd8c5, SUC_lexicon_tags_48 },
    { 0xf70bd8c5, SUC_lexicon_tags_0 },
    { 0xc7abd8c6, SUC_lexicon_tags_39 },
    { 0x144bd8c8, SUC_lexicon_tags_17 },
    { 0x666bd8c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0bd8d1, SUC_lexicon_tags_17 },
    { 0x310bd8d2, SUC_lexicon_tags_12 },
    { 0x4febd8d1, SUC_lexicon_tags_47 },
    { 0x3d8bd8d4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bd8d8, SUC_lexicon_tags_136 },
    { 0xb06bd8d8, SUC_lexicon_tags_46 },
    { 0x772bd8da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x598bd8dc, SUC_lexicon_tags_26 },
    { 0x6a0bd8dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0bd8e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92bd8ef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe6cbd8f1, SUC_lexicon_tags_49 },
    { 0xf94bd8f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51ebd8f4, SUC_lexicon_tags_65 },
    { 0xf02bd8f5, SUC_lexicon_tags_31 },
    { 0x102bd8f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4febd8f8, SUC_lexicon_tags_17 },
    { 0xf2ebd8f9, SUC_lexicon_tags_65 },
    { 0x238bd8f9, SUC_lexicon_tags_11 },
    { 0xe0abd8fa, SUC_lexicon_tags_27 },
    { 0xf1ebd8fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37abd8fe, SUC_lexicon_tags_26 },
    { 0x5ccbd8fe, SUC_lexicon_tags_15 },
    { 0xba0bd900, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6abd90a, SUC_lexicon_tags_61 },
    { 0xd62bd90a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eebd910, SUC_lexicon_tags_56 },
    { 0x1cabd911, SUC_lexicon_tags_48 },
    { 0xa04bd912, SUC_lexicon_tags_15 },
    { 0xc58bd913, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x836bd916, SUC_lexicon_tags_32 },
    { 0x486bd917, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecebd919, SUC_lexicon_tags_8 },
    { 0xd5ebd91a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecbd91d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20bd921, SUC_lexicon_tags_66 },
    { 0x996bd921, SUC_lexicon_tags_23 },
    { 0x10abd923, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610bd92c, SUC_lexicon_tags_32 },
    { 0x5f4bd92d, SUC_lexicon_tags_43 },
    { 0x962bd92e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fcbd930, SUC_lexicon_tags_13 },
    { 0xb5abd931, SUC_lexicon_tags_6 },
    { 0x47ebd931, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d2bd934, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x466bd937, SUC_lexicon_tags_25 },
    { 0x3c4bd938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb64bd93a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x120bd93c, SUC_lexicon_tags_2 },
    { 0xf7cbd93d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784bd944, SUC_lexicon_tags_16 },
    { 0xff2bd945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x472bd947, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32bd94c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf9ebd94e, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6bd952, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340bd961, SUC_lexicon_tags_9 },
    { 0x97abd962, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6bd966, SUC_lexicon_tags_56 },
    { 0xf5abd967, SUC_lexicon_tags_17 },
    { 0x634bd968, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6bd96f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ebd974, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcdebd976, SUC_lexicon_tags_14 },
    { 0x114bd977, SUC_lexicon_tags_23 },
    { 0x232bd978, SUC_lexicon_tags_23 },
    { 0xfc4bd978, SUC_lexicon_tags_15 },
    { 0x7f4bd97a, SUC_lexicon_tags_3 },
    { 0x9f2bd978, SUC_lexicon_tags_63 },
    { 0x9aebd97c, SUC_lexicon_tags_4 },
    { 0x1aebd97c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ebd981, SUC_lexicon_tags_32 },
    { 0x8c2bd982, SUC_lexicon_tags_16 },
    { 0xeb8bd983, SUC_lexicon_tags_63 },
    { 0x7c4bd984, SUC_lexicon_tags_68 },
    { 0xafabd984, SUC_lexicon_tags_25 },
    { 0xd7abd984, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972bd98e, SUC_lexicon_tags_2 },
    { 0xba2bd98e, SUC_lexicon_tags_37 },
    { 0x8abd990, SUC_lexicon_tags_100 },
    { 0x232bd990, SUC_lexicon_tags_45 },
    { 0x57ebd991, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x340bd995, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6bd999, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ebd99c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34bd9a3, SUC_lexicon_tags_47 },
    { 0xcf4bd9a4, SUC_lexicon_tags_15 },
    { 0x906bd9a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46bd9a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ebd9ae, SUC_lexicon_tags_11 },
    { 0x7a4bd9ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44bd9b8, SUC_lexicon_tags_112 },
    { 0xc82bd9b9, SUC_lexicon_tags_33 },
    { 0xe30bd9ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccabd9bc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xccabd9bf, SUC_lexicon_tags_6 },
    { 0xcf8bd9bf, SUC_lexicon_tags_9 },
    { 0xfe6bd9c1, SUC_lexicon_tags_11 },
    { 0xa68bd9c1, SUC_lexicon_tags_3 },
    { 0x17cbd9c3, SUC_lexicon_tags_68 },
    { 0x7bcbd9bf, SUC_lexicon_tags_7 },
    { 0x6a6bd9c5, SUC_lexicon_tags_61 },
    { 0x902bd9c6, SUC_lexicon_tags_38 },
    { 0x830bd9c0, SUC_lexicon_tags_0 },
    { 0x98cbd9c8, SUC_lexicon_tags_17 },
    { 0xf88bd9c9, SUC_lexicon_tags_79 },
    { 0xcdabd9ca, SUC_lexicon_tags_17 },
    { 0x9f0bd9cb, SUC_lexicon_tags_38 },
    { 0x45ebd9cc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xddabd9ce, SUC_lexicon_tags_23 },
    { 0x708bd9cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360bd9d4, SUC_lexicon_tags_57 },
    { 0x216bd9d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678bd9da, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0abd9df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572bd9e3, SUC_lexicon_tags_65 },
    { 0x4a8bd9e4, SUC_lexicon_tags_37 },
    { 0xc8bd9e5, SUC_lexicon_tags_37 },
    { 0x1ebd9e5, SUC_lexicon_tags_15 },
    { 0xfe8bd9e6, SUC_lexicon_tags_0 },
    { 0x398bd9e4, SUC_lexicon_tags_9 },
    { 0x9b2bd9e9, SUC_lexicon_tags_25 },
    { 0xc88bd9ea, SUC_lexicon_tags_31 },
    { 0xaeebd9eb, SUC_lexicon_tags_7 },
    { 0x906bd9ec, SUC_lexicon_tags_0 },
    { 0x1f8bd9e8, SUC_lexicon_tags_65 },
    { 0xe8bd9ee, SUC_lexicon_tags_12 },
    { 0x16abd9e5, SUC_lexicon_tags_19 },
    { 0x3d8bd9f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960bd9f7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x35cbd9f9, SUC_lexicon_tags_47 },
    { 0x5d2bd9fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd24bd9fc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x304bd9ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebebda06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x990bda09, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ebda0e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x73abda11, SUC_lexicon_tags_15 },
    { 0x60ebda12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68bda19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbda1c, SUC_lexicon_tags_15 },
    { 0xd78bda1c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xabcbda1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e0bda21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26bda25, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17abda2f, SUC_lexicon_tags_47 },
    { 0x216bda2f, SUC_lexicon_tags_61 },
    { 0x4aabda31, SUC_lexicon_tags_43 },
    { 0x930bda31, SUC_lexicon_tags_37 },
    { 0xc14bda33, SUC_lexicon_tags_9 },
    { 0x720bda34, SUC_lexicon_tags_35 },
    { 0x65abda35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcbda38, SUC_lexicon_tags_37 },
    { 0x86cbda39, SUC_lexicon_tags_1 },
    { 0x38ebda38, SUC_lexicon_tags_15 },
    { 0xb2bda39, SUC_lexicon_tags_60 },
    { 0xef8bda3c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xcdebda3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8babda41, SUC_lexicon_tags_33 },
    { 0x93abda42, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x3babda45, SUC_lexicon_tags_3 },
    { 0x73abda46, SUC_lexicon_tags_6 },
    { 0xacbda47, SUC_lexicon_tags_14 },
    { 0x822bda46, SUC_lexicon_tags_8 },
    { 0x4c0bda49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x82cbda4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeeabda4e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8bda52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x512bda54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb98bda56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x342bda58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41cbda5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ebda5f, SUC_lexicon_tags_46 },
    { 0x282bda5f, SUC_lexicon_tags_16 },
    { 0x6e2bda61, SUC_lexicon_tags_38 },
    { 0xc92bda62, SUC_lexicon_tags_12 },
    { 0xdacbda62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x772bda66, SUC_lexicon_tags_12 },
    { 0x44ebda67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x974bda69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aebda6c, SUC_lexicon_tags_30 },
    { 0x192bda6d, SUC_lexicon_tags_31 },
    { 0x434bda6c, SUC_lexicon_tags_7 },
    { 0x26ebda6d, SUC_lexicon_tags_17 },
    { 0xd72bda70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecbda78, SUC_lexicon_tags_17 },
    { 0x78cbda79, SUC_lexicon_tags_17 },
    { 0xd94bda7a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fabda7d, SUC_lexicon_tags_46 },
    { 0xde8bda7e, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x40abda81, SUC_lexicon_tags_28 },
    { 0xef0bda82, SUC_lexicon_tags_13 },
    { 0xcf4bda82, SUC_lexicon_tags_0 },
    { 0x44bda83, SUC_lexicon_tags_15 },
    { 0xf48bda85, SUC_lexicon_tags_0 },
    { 0x3b8bda86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44bda8c, SUC_lexicon_tags_27 },
    { 0x880bda8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x10abda90, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8bda9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0bda9e, SUC_lexicon_tags_14 },
    { 0xedebda9e, SUC_lexicon_tags_25 },
    { 0x6bdaa0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626bdaa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134bdaae, SUC_lexicon_tags_23 },
    { 0x228bdaaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1abdab1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0bdab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116bdaba, SUC_lexicon_tags_8 },
    { 0x914bdabb, SUC_lexicon_tags_49 },
    { 0xb8abdabc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe64bdabe, SUC_lexicon_tags_7 },
    { 0xeaebdabf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5a6bdac1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fabdac4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4eabdac6, SUC_lexicon_tags_80 },
    { 0xddabdac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66bdaca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86bdace, SUC_lexicon_tags_12 },
    { 0xbc2bdace, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4bdad7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46bdadb, SUC_lexicon_tags_30 },
    { 0xc62bdadc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0bdae2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x918bdae4, SUC_lexicon_tags_39 },
    { 0xb48bdae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacbdae9, SUC_lexicon_tags_37 },
    { 0x990bdae9, SUC_lexicon_tags_33 },
    { 0xbdcbdaeb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cbdaee, SUC_lexicon_tags_6 },
    { 0x402bdaef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x116bdaf1, SUC_lexicon_tags_1 },
    { 0xd02bdaf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cbdafc, SUC_lexicon_tags_25 },
    { 0xc16bdafd, SUC_lexicon_tags_25 },
    { 0x91cbdafd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5f6bdb00, SUC_lexicon_tags_2 },
    { 0xe8abdb01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8bdb08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0bdb0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27cbdb0e, SUC_lexicon_tags_63 },
    { 0x298bdb0f, SUC_lexicon_tags_12 },
    { 0x216bdb10, SUC_lexicon_tags_7 },
    { 0x118bdb10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cbdb16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62bdb19, SUC_lexicon_tags_12 },
    { 0x318bdb1a, SUC_lexicon_tags_15 },
    { 0xe18bdb1a, SUC_lexicon_tags_47 },
    { 0x13cbdb1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cbdb21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x222bdb24, SUC_lexicon_tags_30 },
    { 0xb26bdb24, SUC_lexicon_tags_7 },
    { 0x704bdb26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302bdb2a, SUC_lexicon_tags_32 },
    { 0xa30bdb2b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cbdb2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ebdb35, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6bdb39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328bdb3e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcbdb41, SUC_lexicon_tags_12 },
    { 0x2f2bdb42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248bdb4b, SUC_lexicon_tags_25 },
    { 0x32cbdb4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5c0bdb4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2bdb52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeabdb55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabdb5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc40bdb5d, SUC_lexicon_tags_22 },
    { 0x92bdb5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb6bdb60, SUC_lexicon_tags_17 },
    { 0xf86bdb61, SUC_lexicon_tags_29 },
    { 0x794bdb61, SUC_lexicon_tags_33 },
    { 0x654bdb61, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0bdb66, SUC_lexicon_tags_48 },
    { 0x812bdb67, SUC_lexicon_tags_11 },
    { 0x984bdb67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x312bdb6a, SUC_lexicon_tags_68 },
    { 0xd44bdb6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cbdb6e, SUC_lexicon_tags_14 },
    { 0xe6ebdb6f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x66abdb71, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x400bdb73, SUC_lexicon_tags_17 },
    { 0x6e0bdb74, SUC_lexicon_tags_58 },
    { 0x564bdb74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf40bdb77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5cbdb79, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56bdb7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72bdb81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4bdb86, SUC_lexicon_tags_17 },
    { 0x448bdb87, SUC_lexicon_tags_7 },
    { 0x24abdb86, SUC_lexicon_tags_45 },
    { 0x928bdb89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586bdb93, SUC_lexicon_tags_7 },
    { 0x82ebdb93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8bdb9c, SUC_lexicon_tags_11 },
    { 0x8fcbdb9c, SUC_lexicon_tags_49 },
    { 0x406bdb9e, SUC_lexicon_tags_37 },
    { 0x112bdb9e, SUC_lexicon_tags_30 },
    { 0xa2abdba0, SUC_lexicon_tags_2 },
    { 0x6aabdba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454bdba6, SUC_lexicon_tags_47 },
    { 0x926bdba6, SUC_lexicon_tags_7 },
    { 0xd38bdba7, SUC_lexicon_tags_29 },
    { 0x54bdba9, SUC_lexicon_tags_45 },
    { 0x266bdbaa, SUC_lexicon_tags_33 },
    { 0x5c2bdbaa, SUC_lexicon_tags_10 },
    { 0xc10bdbab, SUC_lexicon_tags_17 },
    { 0x2d4bdbab, SUC_lexicon_tags_14 },
    { 0xfcabdbae, SUC_lexicon_tags_63 },
    { 0xfdcbdbaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ebdbb2, SUC_lexicon_tags_7 },
    { 0x28cbdbb3, SUC_lexicon_tags_39 },
    { 0x8c2bdbb4, SUC_lexicon_tags_49 },
    { 0xecebdbb4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e6bdbb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59cbdbb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97abdbbe, SUC_lexicon_tags_4 },
    { 0x1dabdbbe, SUC_lexicon_tags_35 },
    { 0xca6bdbbe, SUC_lexicon_tags_47 },
    { 0x4b6bdbc1, SUC_lexicon_tags_218 },
    { 0xc4cbdbbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54cbdbc4, SUC_lexicon_tags_48 },
    { 0x80bdbc4, SUC_lexicon_tags_2 },
    { 0x24abdbc6, SUC_lexicon_tags_15 },
    { 0x38bdbc7, SUC_lexicon_tags_11 },
    { 0x416bdbc8, SUC_lexicon_tags_84 },
    { 0xd6bdbc9, SUC_lexicon_tags_25 },
    { 0xb1ebdbc9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80bdbcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6bdbd1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0bdbd4, SUC_lexicon_tags_27 },
    { 0x4aabdbd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32bdbd8, SUC_lexicon_tags_33 },
    { 0x616bdbd9, SUC_lexicon_tags_17 },
    { 0x930bdbda, SUC_lexicon_tags_14 },
    { 0x41abdbd9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x230bdbde, SUC_lexicon_tags_37 },
    { 0x7f2bdbdf, SUC_lexicon_tags_37 },
    { 0x692bdbde, SUC_lexicon_tags_12 },
    { 0x4acbdbe1, SUC_lexicon_tags_25 },
    { 0x556bdbe2, SUC_lexicon_tags_17 },
    { 0x1febdbe1, SUC_lexicon_tags_10 },
    { 0x342bdbde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8bdbe8, SUC_lexicon_tags_55 },
    { 0x59cbdbe9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6bdbef, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44bdbf2, SUC_lexicon_tags_19 },
    { 0x152bdbf3, SUC_lexicon_tags_55 },
    { 0xd3cbdbf4, SUC_lexicon_tags_7 },
    { 0x962bdbf2, SUC_lexicon_tags_63 },
    { 0x88cbdbf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf4bdbf8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8bdbfb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x654bdbfd, SUC_lexicon_tags_25 },
    { 0x922bdbfe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x318bdc01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4aabdc03, SUC_lexicon_tags_6 },
    { 0x74bdc04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dabdc07, SUC_lexicon_tags_11 },
    { 0xda0bdc07, SUC_lexicon_tags_149 },
    { 0x40ebdc09, SUC_lexicon_tags_56 },
    { 0xdc6bdc0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x458bdc0d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa2ebdc0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e0bdc11, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ebdc14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x638bdc16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdecbdc18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe52bdc1b, SUC_lexicon_tags_38 },
    { 0xde4bdc1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x93abdc1f, SUC_lexicon_tags_13 },
    { 0x9cabdc20, SUC_lexicon_tags_3 },
    { 0xe3cbdc21, SUC_lexicon_tags_151 },
    { 0xd38bdc22, SUC_lexicon_tags_17 },
    { 0xad0bdc21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfebdc25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16bdc29, SUC_lexicon_tags_37 },
    { 0x8c0bdc2a, SUC_lexicon_tags_25 },
    { 0xe5abdc2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecebdc32, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4bdc37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610bdc3c, SUC_lexicon_tags_43 },
    { 0xf92bdc3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcbdc40, SUC_lexicon_tags_17 },
    { 0x146bdc41, SUC_lexicon_tags_20 },
    { 0xca8bdc42, SUC_lexicon_tags_23 },
    { 0x97abdc42, SUC_lexicon_tags_10 },
    { 0x46ebdc44, SUC_lexicon_tags_15 },
    { 0xcd6bdc42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1c2bdc47, SUC_lexicon_tags_46 },
    { 0xd0cbdc48, SUC_lexicon_tags_37 },
    { 0x834bdc49, SUC_lexicon_tags_31 },
    { 0x370bdc49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb4cbdc4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd20bdc4e, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0x54abdc51, SUC_lexicon_tags_27 },
    { 0xc78bdc51, SUC_lexicon_tags_7 },
    { 0x5c0bdc51, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2abdc57, SUC_lexicon_tags_32 },
    { 0xb3cbdc58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x822bdc5a, SUC_lexicon_tags_30 },
    { 0x114bdc5a, SUC_lexicon_tags_4 },
    { 0xc1ebdc5c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xdb4bdc5e, SUC_lexicon_tags_1 },
    { 0xcd8bdc5e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50abdc63, SUC_lexicon_tags_5 },
    { 0xe76bdc64, SUC_lexicon_tags_11 },
    { 0xcf0bdc64, SUC_lexicon_tags_14 },
    { 0x6e8bdc66, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccbdc6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ebdc70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcbdc79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd32bdc7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6bdc7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370bdc83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352bdc89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cbdc8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x73cbdc90, SUC_lexicon_tags_1 },
    { 0x830bdc90, SUC_lexicon_tags_7 },
    { 0x774bdc90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ebdc96, SUC_lexicon_tags_6 },
    { 0xdd8bdc97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ebdc9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfbebdc9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4bdca0, SUC_lexicon_tags_17 },
    { 0x294bdca1, SUC_lexicon_tags_2 },
    { 0x37abdca0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaabdca6, SUC_lexicon_tags_11 },
    { 0x54cbdca7, SUC_lexicon_tags_17 },
    { 0xc08bdca8, SUC_lexicon_tags_8 },
    { 0x778bdca9, SUC_lexicon_tags_60 },
    { 0xdacbdcaa, SUC_lexicon_tags_11 },
    { 0x47ebdcab, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44bdcae, SUC_lexicon_tags_69 },
    { 0x688bdcaf, SUC_lexicon_tags_26 },
    { 0x488bdcb0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc32bdcb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb6bdcb4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x7ccbdcb6, SUC_lexicon_tags_15 },
    { 0xd1cbdcb6, SUC_lexicon_tags_43 },
    { 0x25ebdcb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0bdcbc, SUC_lexicon_tags_29 },
    { 0x442bdcbd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bdcc3, SUC_lexicon_tags_7 },
    { 0x8d2bdcc3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ebdccd, SUC_lexicon_tags_172 },
    { 0xf3ebdcce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42bdcd3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x86bdcd5, SUC_lexicon_tags_55 },
    { 0x922bdcd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ebdcdc, SUC_lexicon_tags_2 },
    { 0xd18bdcdd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x17cbdce0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dcbdce2, SUC_lexicon_tags_68 },
    { 0x402bdce2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x17abdce5, SUC_lexicon_tags_16 },
    { 0xdb6bdce6, SUC_lexicon_tags_25 },
    { 0xf14bdce7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4febdcea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2bdced, SUC_lexicon_tags_25 },
    { 0x528bdcee, SUC_lexicon_tags_27 },
    { 0xc62bdcef, SUC_lexicon_tags_48 },
    { 0x8ecbdcf0, SUC_lexicon_tags_36 },
    { 0xc14bdcee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x684bdcf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dcbdcf6, SUC_lexicon_tags_14 },
    { 0xeb4bdcf7, SUC_lexicon_tags_65 },
    { 0x6f8bdcf7, SUC_lexicon_tags_8 },
    { 0x420bdcf9, SUC_lexicon_tags_25 },
    { 0xe60bdcfa, SUC_lexicon_tags_25 },
    { 0xa36bdcfb, SUC_lexicon_tags_26 },
    { 0x3dabdcfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bdcff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95abdd04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cbdd09, SUC_lexicon_tags_37 },
    { 0xaeabdd0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38bdd0f, SUC_lexicon_tags_12 },
    { 0x5fabdd10, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x40abdd12, SUC_lexicon_tags_43 },
    { 0xbebdd13, SUC_lexicon_tags_15 },
    { 0x35abdd14, SUC_lexicon_tags_47 },
    { 0x602bdd14, SUC_lexicon_tags_25 },
    { 0xcaabdd14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46bdd1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60abdd1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8ccbdd20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacebdd27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cbdd2c, SUC_lexicon_tags_17 },
    { 0x2a0bdd2d, SUC_lexicon_tags_25 },
    { 0xacebdd2e, SUC_lexicon_tags_38 },
    { 0x14bdd2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf6bdd31, SUC_lexicon_tags_1 },
    { 0x4bcbdd32, SUC_lexicon_tags_30 },
    { 0xc9abdd32, SUC_lexicon_tags_28 },
    { 0x81abdd34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942bdd3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x94cbdd3f, SUC_lexicon_tags_48 },
    { 0xd18bdd40, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cbdd44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf4bdd46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0bdd49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3abdd4b, SUC_lexicon_tags_27 },
    { 0x906bdd4c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82bdd4f, SUC_lexicon_tags_31 },
    { 0xdf8bdd50, SUC_lexicon_tags_12 },
    { 0xf06bdd51, SUC_lexicon_tags_60 },
    { 0xaa0bdd51, SUC_lexicon_tags_37 },
    { 0x6dabdd53, SUC_lexicon_tags_37 },
    { 0x3c6bdd52, SUC_lexicon_tags_1 },
    { 0xd3cbdd54, SUC_lexicon_tags_16 },
    { 0x884bdd56, SUC_lexicon_tags_49 },
    { 0x4cebdd57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ccbdd59, SUC_lexicon_tags_69 },
    { 0x3b0bdd5a, SUC_lexicon_tags_13 },
    { 0x5ebdd5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff0bdd5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdd61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe76bdd63, SUC_lexicon_tags_9 },
    { 0xf20bdd63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f4bdd66, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x162bdd68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6febdd6b, SUC_lexicon_tags_7 },
    { 0xa5ebdd6c, SUC_lexicon_tags_3 },
    { 0x1abdd6d, SUC_lexicon_tags_35 },
    { 0xc8ebdd6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x592bdd70, SUC_lexicon_tags_25 },
    { 0x1d4bdd70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccbdd78, SUC_lexicon_tags_7 },
    { 0xabcbdd79, SUC_lexicon_tags_15 },
    { 0x694bdd78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d8bdd7c, SUC_lexicon_tags_11 },
    { 0x994bdd7c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8bdd81, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x52ebdd83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cbdd87, SUC_lexicon_tags_49 },
    { 0x38cbdd87, SUC_lexicon_tags_28 },
    { 0xa64bdd89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55abdd8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ebdd94, SUC_lexicon_tags_38 },
    { 0xba2bdd95, SUC_lexicon_tags_37 },
    { 0x566bdd94, SUC_lexicon_tags_37 },
    { 0x58abdd97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10bdd9a, SUC_lexicon_tags_17 },
    { 0x2b8bdd9a, SUC_lexicon_tags_27 },
    { 0x192bdd9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2bdda0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978bdda8, SUC_lexicon_tags_38 },
    { 0x432bdda8, SUC_lexicon_tags_12 },
    { 0x68cbdda8, SUC_lexicon_tags_15 },
    { 0x44ebddab, SUC_lexicon_tags_12 },
    { 0x27abddac, SUC_lexicon_tags_54 },
    { 0xcf6bddac, SUC_lexicon_tags_16 },
    { 0xa36bddae, SUC_lexicon_tags_87 },
    { 0xc9abddad, SUC_lexicon_tags_25 },
    { 0xebcbdda9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8bddb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62bddb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc8ebddb8, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdabddbc, SUC_lexicon_tags_55 },
    { 0x2cbddbd, SUC_lexicon_tags_5 },
    { 0x478bddbd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10cbddc5, SUC_lexicon_tags_25 },
    { 0x772bddc6, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x8acbddc9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd10bddcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358bddd1, SUC_lexicon_tags_7 },
    { 0x750bddd2, SUC_lexicon_tags_14 },
    { 0x83cbddd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4abddd8, SUC_lexicon_tags_26 },
    { 0x598bddd9, SUC_lexicon_tags_12 },
    { 0x88ebddd8, SUC_lexicon_tags_17 },
    { 0x526bddd8, SUC_lexicon_tags_12 },
    { 0xde0bdddc, SUC_lexicon_tags_45 },
    { 0x714bdddb, SUC_lexicon_tags_9 },
    { 0xa60bddda, SUC_lexicon_tags_26 },
    { 0xc02bdddf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902bdde4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4bdde8, SUC_lexicon_tags_7 },
    { 0xdf8bdde8, SUC_lexicon_tags_17 },
    { 0x65abdde9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x110bddec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fabddf4, SUC_lexicon_tags_25 },
    { 0xdf6bddf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbcbddf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814bddfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc42bddfd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04bde00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72ebde02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ebde05, SUC_lexicon_tags_37 },
    { 0x90cbde06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48bde0a, SUC_lexicon_tags_6 },
    { 0xb26bde0b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc22bde0d, SUC_lexicon_tags_23 },
    { 0x596bde0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x504bde10, SUC_lexicon_tags_15 },
    { 0x288bde11, SUC_lexicon_tags_38 },
    { 0x6dcbde11, SUC_lexicon_tags_6 },
    { 0x736bde13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x526bde15, SUC_lexicon_tags_23 },
    { 0x6ebde16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ebde1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x942bde1e, SUC_lexicon_tags_70 },
    { 0x792bde1f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5f8bde21, SUC_lexicon_tags_10 },
    { 0x49ebde22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aebde24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2bde29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed8bde2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x544bde2e, SUC_lexicon_tags_82 },
    { 0xdc4bde2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77abde36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aabde3a, SUC_lexicon_tags_2 },
    { 0xea8bde3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82bde3f, SUC_lexicon_tags_37 },
    { 0x2c4bde40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16bde46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0bde4d, SUC_lexicon_tags_1 },
    { 0xf84bde4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ebde52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8bde56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4abde58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876bde5d, SUC_lexicon_tags_27 },
    { 0x24cbde5e, SUC_lexicon_tags_9 },
    { 0x3e2bde5d, SUC_lexicon_tags_13 },
    { 0xf04bde5d, SUC_lexicon_tags_8 },
    { 0xeeebde61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ebde64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2bde67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcaabde69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76bde6d, SUC_lexicon_tags_25 },
    { 0xdeabde6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf08bde70, SUC_lexicon_tags_154 },
    { 0x1b8bde71, SUC_lexicon_tags_12 },
    { 0x848bde72, SUC_lexicon_tags_25 },
    { 0xb50bde73, SUC_lexicon_tags_31 },
    { 0x864bde70, SUC_lexicon_tags_8 },
    { 0x74cbde75, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x316bde78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6febde7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72abde7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2bde80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6bde86, SUC_lexicon_tags_29 },
    { 0x34cbde87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x24cbde89, SUC_lexicon_tags_29 },
    { 0xf82bde8a, SUC_lexicon_tags_15 },
    { 0x11ebde8b, SUC_lexicon_tags_1 },
    { 0xed0bde8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9c8bde8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9c0bde90, SUC_lexicon_tags_49 },
    { 0xd46bde90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08bde94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecbde9d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6d8bde9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb78bdea1, SUC_lexicon_tags_19 },
    { 0xe9abdea2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbabdea6, SUC_lexicon_tags_31 },
    { 0x79abdea7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x30abdea9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcbdead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8bdeb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f4bdeb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96bdebb, SUC_lexicon_tags_19 },
    { 0xe48bdebc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x760bdebe, SUC_lexicon_tags_25 },
    { 0x3bcbdebf, SUC_lexicon_tags_37 },
    { 0xad0bdebf, SUC_lexicon_tags_4 },
    { 0x3babdec0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89ebdec3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x140bdec5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cbdec8, SUC_lexicon_tags_1 },
    { 0x332bdec9, SUC_lexicon_tags_31 },
    { 0xd3ebdeca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbebdecc, SUC_lexicon_tags_19 },
    { 0xc9cbdecd, SUC_lexicon_tags_63 },
    { 0xb6ebdece, SUC_lexicon_tags_15 },
    { 0x8bebdecd, SUC_lexicon_tags_17 },
    { 0xa0bded0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30bded5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x296bded7, SUC_lexicon_tags_40 },
    { 0x47ebded8, SUC_lexicon_tags_33 },
    { 0x3c0bded9, SUC_lexicon_tags_23 },
    { 0xd14bded8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ebdedd, SUC_lexicon_tags_1 },
    { 0x29abdedd, SUC_lexicon_tags_3 },
    { 0xc10bdedf, SUC_lexicon_tags_27 },
    { 0x86bdede, SUC_lexicon_tags_12 },
    { 0xb4cbdee1, SUC_lexicon_tags_37 },
    { 0x322bdee0, SUC_lexicon_tags_39 },
    { 0xef8bdee3, SUC_lexicon_tags_77 },
    { 0xa18bdee4, SUC_lexicon_tags_252 },
    { 0x97cbdee3, SUC_lexicon_tags_11 },
    { 0xe36bdee6, SUC_lexicon_tags_37 },
    { 0x8a0bdee4, SUC_lexicon_tags_11 },
    { 0xacebdee8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x236bdeeb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aebdef0, SUC_lexicon_tags_14 },
    { 0x96ebdef0, SUC_lexicon_tags_15 },
    { 0xf84bdef2, SUC_lexicon_tags_41 },
    { 0xa30bdef1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x648bdef5, SUC_lexicon_tags_34 },
    { 0x7c4bdef6, SUC_lexicon_tags_8 },
    { 0x432bdef7, SUC_lexicon_tags_14 },
    { 0x612bdef8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x484bdefa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6bdeff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9a6bdf01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c8bdf03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9c4bdf05, SUC_lexicon_tags_31 },
    { 0xfe6bdf05, SUC_lexicon_tags_29 },
    { 0xf1cbdf07, SUC_lexicon_tags_45 },
    { 0xf02bdf07, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6b4bdf0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x410bdf0c, SUC_lexicon_tags_15 },
    { 0xc60bdf0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708bdf16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x218bdf19, SUC_lexicon_tags_7 },
    { 0x984bdf1a, SUC_lexicon_tags_2 },
    { 0xa96bdf1b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ebdf1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1acbdf21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9abdf25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e4bdf27, SUC_lexicon_tags_37 },
    { 0xafcbdf28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cbdf2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcabdf30, SUC_lexicon_tags_48 },
    { 0x42cbdf31, SUC_lexicon_tags_3 },
    { 0xcb4bdf31, SUC_lexicon_tags_25 },
    { 0x47abdf30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bdf37, SUC_lexicon_tags_32 },
    { 0x9dabdf37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3acbdf3a, SUC_lexicon_tags_4 },
    { 0x5eebdf3b, SUC_lexicon_tags_19 },
    { 0xf00bdf3b, SUC_lexicon_tags_15 },
    { 0xee0bdf3c, SUC_lexicon_tags_49 },
    { 0xf8cbdf3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12bdf43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8bdf47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6cbdf49, SUC_lexicon_tags_6 },
    { 0x9e0bdf4a, SUC_lexicon_tags_60 },
    { 0x310bdf4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x370bdf4e, SUC_lexicon_tags_25 },
    { 0x2debdf4f, SUC_lexicon_tags_17 },
    { 0x8babdf4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584bdf54, SUC_lexicon_tags_79 },
    { 0x936bdf54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x22cbdf57, SUC_lexicon_tags_34 },
    { 0x40cbdf58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x452bdf5b, SUC_lexicon_tags_60 },
    { 0xd04bdf5c, SUC_lexicon_tags_25 },
    { 0xd18bdf5d, SUC_lexicon_tags_17 },
    { 0x6d0bdf5d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2bdf62, SUC_lexicon_tags_44 },
    { 0x7fabdf63, SUC_lexicon_tags_381 },
    { 0, NULL },
    { 0, NULL },
    { 0x646bdf66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60bdf6b, SUC_lexicon_tags_12 },
    { 0xdfabdf6b, SUC_lexicon_tags_15 },
    { 0x314bdf6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf80bdf6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebdf7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x596bdf7d, SUC_lexicon_tags_9 },
    { 0x8e0bdf7e, SUC_lexicon_tags_14 },
    { 0x648bdf7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cbdf82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294bdf88, SUC_lexicon_tags_13 },
    { 0x1a2bdf88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4bdf8c, SUC_lexicon_tags_37 },
    { 0x498bdf8d, SUC_lexicon_tags_15 },
    { 0x3e8bdf8e, SUC_lexicon_tags_43 },
    { 0xa4bdf8f, SUC_lexicon_tags_7 },
    { 0x6aabdf90, SUC_lexicon_tags_12 },
    { 0x40abdf91, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12bdf95, SUC_lexicon_tags_11 },
    { 0xb18bdf96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x882bdf98, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebdf9c, SUC_lexicon_tags_12 },
    { 0xdf4bdf9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ebdf9f, SUC_lexicon_tags_31 },
    { 0x7d2bdfa0, SUC_lexicon_tags_9 },
    { 0x2ccbdfa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64bdfa4, SUC_lexicon_tags_45 },
    { 0xe80bdfa5, SUC_lexicon_tags_14 },
    { 0xe62bdfa5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cbdfac, SUC_lexicon_tags_14 },
    { 0x41cbdfad, SUC_lexicon_tags_17 },
    { 0x858bdfae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x596bdfb1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bdfb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904bdfba, SUC_lexicon_tags_14 },
    { 0xf18bdfbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00bdfbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x814bdfc1, SUC_lexicon_tags_45 },
    { 0xa78bdfc2, SUC_lexicon_tags_3 },
    { 0x9d4bdfc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0bdfc8, SUC_lexicon_tags_17 },
    { 0xb5ebdfc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdcbdfcb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9fabdfcd, SUC_lexicon_tags_48 },
    { 0x818bdfce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112bdfd5, SUC_lexicon_tags_63 },
    { 0xc28bdfd6, SUC_lexicon_tags_25 },
    { 0x4e8bdfd7, SUC_lexicon_tags_1 },
    { 0xab6bdfd7, SUC_lexicon_tags_4 },
    { 0xaebdfd9, SUC_lexicon_tags_14 },
    { 0xf38bdfd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14bdfdd, SUC_lexicon_tags_37 },
    { 0x614bdfde, SUC_lexicon_tags_7 },
    { 0x8c6bdfdf, SUC_lexicon_tags_7 },
    { 0x840bdfdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb82bdfe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa02bdfe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd50bdfe6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6bdfe9, SUC_lexicon_tags_37 },
    { 0xba6bdfe9, SUC_lexicon_tags_1 },
    { 0xe22bdfe9, SUC_lexicon_tags_37 },
    { 0xa3abdfea, SUC_lexicon_tags_17 },
    { 0x4e4bdfe9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cbdff4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11ebdff6, SUC_lexicon_tags_37 },
    { 0x156bdff7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62bdffa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02be000, SUC_lexicon_tags_68 },
    { 0x862be001, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3b6be003, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xf5abe005, SUC_lexicon_tags_17 },
    { 0xb62be006, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3febe009, SUC_lexicon_tags_37 },
    { 0x6a6be00a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2be011, SUC_lexicon_tags_7 },
    { 0x4a8be012, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abe016, SUC_lexicon_tags_2 },
    { 0xc22be017, SUC_lexicon_tags_45 },
    { 0xbd0be018, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dabe01e, SUC_lexicon_tags_17 },
    { 0xf5abe01e, SUC_lexicon_tags_11 },
    { 0xbb2be020, SUC_lexicon_tags_15 },
    { 0x768be021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ebe025, SUC_lexicon_tags_32 },
    { 0x4b2be026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb8be028, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5febe02a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x200be02c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccabe02e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42be036, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264be03d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a2be03f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaeebe041, SUC_lexicon_tags_9 },
    { 0xa1cbe042, SUC_lexicon_tags_17 },
    { 0x50be043, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fabe045, SUC_lexicon_tags_33 },
    { 0xbf2be046, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3cbe048, SUC_lexicon_tags_7 },
    { 0x908be049, SUC_lexicon_tags_60 },
    { 0xdfcbe04a, SUC_lexicon_tags_77 },
    { 0xb90be048, SUC_lexicon_tags_7 },
    { 0xdbebe04c, SUC_lexicon_tags_37 },
    { 0xed4be04d, SUC_lexicon_tags_116 },
    { 0x52abe04e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448be053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0ebe055, SUC_lexicon_tags_15 },
    { 0x37cbe055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x348be058, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acbe05d, SUC_lexicon_tags_7 },
    { 0xf02be05d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x366be061, SUC_lexicon_tags_25 },
    { 0xe6ebe062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316be066, SUC_lexicon_tags_11 },
    { 0xa84be067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246be072, SUC_lexicon_tags_49 },
    { 0x8eabe073, SUC_lexicon_tags_6 },
    { 0xd56be072, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa72be076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xebebe079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8be07d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2be083, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cbe087, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284be08c, SUC_lexicon_tags_12 },
    { 0xcc2be08d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10be091, SUC_lexicon_tags_27 },
    { 0x46be092, SUC_lexicon_tags_7 },
    { 0x144be092, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88be09a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd94be09c, SUC_lexicon_tags_25 },
    { 0xe10be09d, SUC_lexicon_tags_15 },
    { 0xdbabe09e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xde2be0a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x140be0a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88be0a6, SUC_lexicon_tags_17 },
    { 0x752be0a7, SUC_lexicon_tags_11 },
    { 0xc9abe0a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ebe0ad, SUC_lexicon_tags_4 },
    { 0x902be0ad, SUC_lexicon_tags_14 },
    { 0xf12be0af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8be0b3, SUC_lexicon_tags_38 },
    { 0xc7abe0b3, SUC_lexicon_tags_11 },
    { 0x3bebe0b5, SUC_lexicon_tags_43 },
    { 0x628be0b3, SUC_lexicon_tags_17 },
    { 0x9bcbe0b4, SUC_lexicon_tags_29 },
    { 0xa70be0b8, SUC_lexicon_tags_0 },
    { 0x960be0b6, SUC_lexicon_tags_11 },
    { 0x9c6be0ba, SUC_lexicon_tags_17 },
    { 0x916be0b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36cbe0bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12be0c2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xfe0be0c4, SUC_lexicon_tags_15 },
    { 0xa30be0c5, SUC_lexicon_tags_14 },
    { 0xadcbe0c6, SUC_lexicon_tags_82 },
    { 0xa24be0c4, SUC_lexicon_tags_5 },
    { 0xfeabe0c8, SUC_lexicon_tags_87 },
    { 0xe98be0c9, SUC_lexicon_tags_37 },
    { 0x45ebe0c9, SUC_lexicon_tags_28 },
    { 0x338be0c9, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8be0ce, SUC_lexicon_tags_37 },
    { 0x18cbe0cf, SUC_lexicon_tags_43 },
    { 0xf74be0d0, SUC_lexicon_tags_15 },
    { 0xeeabe0cf, SUC_lexicon_tags_45 },
    { 0x6b4be0d2, SUC_lexicon_tags_23 },
    { 0x474be0d2, SUC_lexicon_tags_7 },
    { 0x8bebe0cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x96cbe0d6, SUC_lexicon_tags_47 },
    { 0x474be0d7, SUC_lexicon_tags_26 },
    { 0xf0abe0d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cbe0de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4be0e4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eebe0eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2be0ef, SUC_lexicon_tags_56 },
    { 0xf18be0f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4be0f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf8be0f6, SUC_lexicon_tags_1 },
    { 0x71ebe0f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceebe0fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4be101, SUC_lexicon_tags_11 },
    { 0xc3cbe102, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x738be104, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202be10e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aebe114, SUC_lexicon_tags_37 },
    { 0x9d8be114, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2be118, SUC_lexicon_tags_35 },
    { 0xd30be118, SUC_lexicon_tags_25 },
    { 0x7babe11a, SUC_lexicon_tags_27 },
    { 0x8bebe11a, SUC_lexicon_tags_784 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ebe11f, SUC_lexicon_tags_15 },
    { 0xaeabe120, SUC_lexicon_tags_15 },
    { 0x4f8be121, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddebe125, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0be128, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2be134, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16be137, SUC_lexicon_tags_12 },
    { 0x7aebe138, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cbe13e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x67abe141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x430be143, SUC_lexicon_tags_17 },
    { 0xbbcbe144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218be14b, SUC_lexicon_tags_45 },
    { 0xfdabe14c, SUC_lexicon_tags_10 },
    { 0x1b4be14d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10be156, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cbe159, SUC_lexicon_tags_65 },
    { 0x1ccbe15a, SUC_lexicon_tags_37 },
    { 0xcf4be15a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38be15e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acbe163, SUC_lexicon_tags_37 },
    { 0x706be164, SUC_lexicon_tags_14 },
    { 0x82cbe163, SUC_lexicon_tags_145 },
    { 0x3f0be164, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ebe169, SUC_lexicon_tags_27 },
    { 0x4c6be16a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x978be16d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x18be16f, SUC_lexicon_tags_15 },
    { 0x3b4be170, SUC_lexicon_tags_37 },
    { 0xedabe171, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60be176, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cbe17c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8be181, SUC_lexicon_tags_82 },
    { 0x46be182, SUC_lexicon_tags_146 },
    { 0x1fabe182, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8be188, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4be18b, SUC_lexicon_tags_30 },
    { 0xa3ebe18c, SUC_lexicon_tags_21 },
    { 0xfa0be18d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ebe192, SUC_lexicon_tags_4 },
    { 0xb50be193, SUC_lexicon_tags_23 },
    { 0x4d6be194, SUC_lexicon_tags_1 },
    { 0xfc6be195, SUC_lexicon_tags_49 },
    { 0x1a0be196, SUC_lexicon_tags_19 },
    { 0x3d4be196, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x106be19a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x194be19d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ebe1a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdcbe1a4, SUC_lexicon_tags_39 },
    { 0xf46be1a5, SUC_lexicon_tags_178 },
    { 0x1c6be1a6, SUC_lexicon_tags_57 },
    { 0x56cbe1a5, SUC_lexicon_tags_27 },
    { 0x7b2be1a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16ebe1aa, SUC_lexicon_tags_17 },
    { 0xebebe1ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cbe1af, SUC_lexicon_tags_1 },
    { 0x2bebe1af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa98be1b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcbe1b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf2be1b7, SUC_lexicon_tags_27 },
    { 0x31ebe1b8, SUC_lexicon_tags_75 },
    { 0xf1ebe1b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8be1bd, SUC_lexicon_tags_13 },
    { 0xfb4be1bd, SUC_lexicon_tags_25 },
    { 0xcd4be1bf, SUC_lexicon_tags_53 },
    { 0x80abe1c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf2be1c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbd6be1c4, SUC_lexicon_tags_25 },
    { 0xd40be1c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5c0be1c7, SUC_lexicon_tags_15 },
    { 0x56abe1c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3eebe1ca, SUC_lexicon_tags_37 },
    { 0x306be1cb, SUC_lexicon_tags_80 },
    { 0x540be1cc, SUC_lexicon_tags_47 },
    { 0xf1abe1cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4be1d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886be1d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9ebe1d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18be1dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebe1e2, SUC_lexicon_tags_27 },
    { 0xb54be1e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8be1eb, SUC_lexicon_tags_30 },
    { 0xdecbe1eb, SUC_lexicon_tags_23 },
    { 0xef0be1ed, SUC_lexicon_tags_39 },
    { 0xfd6be1eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aabe1f8, SUC_lexicon_tags_19 },
    { 0x5febe1f9, SUC_lexicon_tags_38 },
    { 0x432be1f9, SUC_lexicon_tags_8 },
    { 0xd2abe1fb, SUC_lexicon_tags_30 },
    { 0xd06be1fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8aabe1fe, SUC_lexicon_tags_2 },
    { 0xa5abe1ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe203, SUC_lexicon_tags_584 },
    { 0, NULL },
    { 0, NULL },
    { 0xdebe206, SUC_lexicon_tags_27 },
    { 0x70ebe207, SUC_lexicon_tags_11 },
    { 0x390be208, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc90be20a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918be213, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4a0be215, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76be218, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eebe21f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb48be221, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xcebe223, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76abe228, SUC_lexicon_tags_2 },
    { 0xf94be229, SUC_lexicon_tags_129 },
    { 0xa4abe22a, SUC_lexicon_tags_1 },
    { 0xc86be22b, SUC_lexicon_tags_6 },
    { 0xcfabe22b, SUC_lexicon_tags_23 },
    { 0x1c8be22d, SUC_lexicon_tags_37 },
    { 0x5fabe22e, SUC_lexicon_tags_13 },
    { 0xfcebe22e, SUC_lexicon_tags_32 },
    { 0x4acbe230, SUC_lexicon_tags_3 },
    { 0xa8cbe231, SUC_lexicon_tags_47 },
    { 0x8b4be232, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x370be234, SUC_lexicon_tags_16 },
    { 0xb96be234, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x418be237, SUC_lexicon_tags_63 },
    { 0x7e4be238, SUC_lexicon_tags_12 },
    { 0x526be237, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x474be23c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6cbe23e, SUC_lexicon_tags_17 },
    { 0x292be23f, SUC_lexicon_tags_26 },
    { 0x6febe240, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6be246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8be249, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0be24e, SUC_lexicon_tags_25 },
    { 0xb58be24e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x278be252, SUC_lexicon_tags_9 },
    { 0x102be253, SUC_lexicon_tags_7 },
    { 0x450be252, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcbe259, SUC_lexicon_tags_2 },
    { 0x6cabe259, SUC_lexicon_tags_7 },
    { 0x338be25b, SUC_lexicon_tags_71 },
    { 0x56cbe25a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5abe25f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ecbe261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4be26d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88be270, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30be272, SUC_lexicon_tags_2 },
    { 0xc0ebe273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4cbe275, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804be27c, SUC_lexicon_tags_63 },
    { 0xac2be27d, SUC_lexicon_tags_1 },
    { 0x6a4be27e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c6be280, SUC_lexicon_tags_37 },
    { 0x920be281, SUC_lexicon_tags_37 },
    { 0x4abe280, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11abe285, SUC_lexicon_tags_15 },
    { 0xd0ebe285, SUC_lexicon_tags_14 },
    { 0x4bcbe285, SUC_lexicon_tags_17 },
    { 0xa10be288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04be28c, SUC_lexicon_tags_9 },
    { 0x78ebe28c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x180be28f, SUC_lexicon_tags_10 },
    { 0xbb2be28f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368be294, SUC_lexicon_tags_37 },
    { 0x316be295, SUC_lexicon_tags_10 },
    { 0xc9abe295, SUC_lexicon_tags_7 },
    { 0xabcbe297, SUC_lexicon_tags_91 },
    { 0xe96be298, SUC_lexicon_tags_2 },
    { 0xe40be297, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x204be29b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebabe2a0, SUC_lexicon_tags_45 },
    { 0x57abe2a0, SUC_lexicon_tags_37 },
    { 0x554be2a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccbe2a9, SUC_lexicon_tags_17 },
    { 0xd4be2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8be2ad, SUC_lexicon_tags_35 },
    { 0x430be2ae, SUC_lexicon_tags_15 },
    { 0x2d2be2af, SUC_lexicon_tags_17 },
    { 0x1f6be2ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd42be2b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50be2b9, SUC_lexicon_tags_11 },
    { 0xe00be2ba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818be2c5, SUC_lexicon_tags_27 },
    { 0x68ebe2c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd4cbe2c8, SUC_lexicon_tags_8 },
    { 0x3b6be2c9, SUC_lexicon_tags_56 },
    { 0xde6be2c8, SUC_lexicon_tags_20 },
    { 0xefcbe2cb, SUC_lexicon_tags_45 },
    { 0x214be2cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2be2cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccbe2d3, SUC_lexicon_tags_16 },
    { 0x33cbe2d3, SUC_lexicon_tags_28 },
    { 0x7e0be2d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30be2da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fabe2df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8abe2e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44be2e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64be2eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97abe2f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2be2f8, SUC_lexicon_tags_25 },
    { 0x644be2f9, SUC_lexicon_tags_15 },
    { 0x848be2fa, SUC_lexicon_tags_25 },
    { 0x146be2fa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918be2ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecbe30c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa62be30e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x988be310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0be313, SUC_lexicon_tags_56 },
    { 0x8b0be314, SUC_lexicon_tags_6 },
    { 0x56cbe313, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cbe319, SUC_lexicon_tags_2 },
    { 0x930be31a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbe321, SUC_lexicon_tags_27 },
    { 0x2e6be322, SUC_lexicon_tags_11 },
    { 0xdd4be322, SUC_lexicon_tags_0 },
    { 0xcd4be324, SUC_lexicon_tags_27 },
    { 0xb12be325, SUC_lexicon_tags_46 },
    { 0x438be326, SUC_lexicon_tags_14 },
    { 0x432be327, SUC_lexicon_tags_11 },
    { 0xeaabe322, SUC_lexicon_tags_32 },
    { 0xe2abe329, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x280be32c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x422be32e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x266be330, SUC_lexicon_tags_9 },
    { 0x756be331, SUC_lexicon_tags_29 },
    { 0x702be330, SUC_lexicon_tags_62 },
    { 0x532be330, SUC_lexicon_tags_2 },
    { 0x10abe334, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cbe337, SUC_lexicon_tags_1 },
    { 0xc9abe338, SUC_lexicon_tags_33 },
    { 0x2f8be337, SUC_lexicon_tags_60 },
    { 0xa4abe33a, SUC_lexicon_tags_11 },
    { 0xaf4be339, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc6be33d, SUC_lexicon_tags_17 },
    { 0xd3ebe33e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eabe344, SUC_lexicon_tags_35 },
    { 0xca0be344, SUC_lexicon_tags_56 },
    { 0x23cbe345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cbe34a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe08be34c, SUC_lexicon_tags_23 },
    { 0x8babe34d, SUC_lexicon_tags_0 },
    { 0x65cbe34e, SUC_lexicon_tags_25 },
    { 0x64be34e, SUC_lexicon_tags_37 },
    { 0xdaebe34f, SUC_lexicon_tags_6 },
    { 0x16ebe350, SUC_lexicon_tags_11 },
    { 0x1bcbe34d, SUC_lexicon_tags_45 },
    { 0x69cbe34e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56be356, SUC_lexicon_tags_112 },
    { 0x80abe357, SUC_lexicon_tags_31 },
    { 0xec4be356, SUC_lexicon_tags_41 },
    { 0x2aebe359, SUC_lexicon_tags_1 },
    { 0x86cbe358, SUC_lexicon_tags_33 },
    { 0xcd0be35b, SUC_lexicon_tags_79 },
    { 0x718be35b, SUC_lexicon_tags_19 },
    { 0xf14be35c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10be360, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97cbe362, SUC_lexicon_tags_1 },
    { 0x3f4be363, SUC_lexicon_tags_3 },
    { 0xd7cbe364, SUC_lexicon_tags_7 },
    { 0x524be364, SUC_lexicon_tags_23 },
    { 0xeb8be366, SUC_lexicon_tags_13 },
    { 0xda4be366, SUC_lexicon_tags_46 },
    { 0x27ebe368, SUC_lexicon_tags_55 },
    { 0x4d8be365, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3ebe36b, SUC_lexicon_tags_27 },
    { 0xacabe36c, SUC_lexicon_tags_14 },
    { 0xd04be36d, SUC_lexicon_tags_11 },
    { 0x598be36e, SUC_lexicon_tags_17 },
    { 0x9b8be36f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0be372, SUC_lexicon_tags_108 },
    { 0x756be373, SUC_lexicon_tags_62 },
    { 0x754be372, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6be377, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12be37a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ebe37d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2be383, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618be388, SUC_lexicon_tags_5 },
    { 0x4f2be389, SUC_lexicon_tags_12 },
    { 0x3bebe38a, SUC_lexicon_tags_35 },
    { 0x3e4be389, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0be390, SUC_lexicon_tags_15 },
    { 0xcd6be391, SUC_lexicon_tags_17 },
    { 0x6d2be392, SUC_lexicon_tags_84 },
    { 0x4c8be393, SUC_lexicon_tags_12 },
    { 0xdb2be394, SUC_lexicon_tags_28 },
    { 0x192be395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2be399, SUC_lexicon_tags_2 },
    { 0xa9abe39a, SUC_lexicon_tags_30 },
    { 0x256be399, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb8be39d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14be3a2, SUC_lexicon_tags_11 },
    { 0x18ebe3a3, SUC_lexicon_tags_38 },
    { 0x2e4be3a4, SUC_lexicon_tags_15 },
    { 0x8e2be3a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4be3ad, SUC_lexicon_tags_9 },
    { 0x538be3ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x860be3b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb2be3b2, SUC_lexicon_tags_152 },
    { 0x89cbe3b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x760be3b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x708be3b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8be3bb, SUC_lexicon_tags_49 },
    { 0x484be3bb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe3abe3be, SUC_lexicon_tags_15 },
    { 0xf30be3bf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536be3c3, SUC_lexicon_tags_25 },
    { 0x2ccbe3c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622be3ca, SUC_lexicon_tags_25 },
    { 0x86be3ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d2be3cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3cbe3cf, SUC_lexicon_tags_0 },
    { 0x4babe3cf, SUC_lexicon_tags_32 },
    { 0xa74be3d0, SUC_lexicon_tags_13 },
    { 0x180be3d2, SUC_lexicon_tags_61 },
    { 0xb3ebe3d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcbe3d6, SUC_lexicon_tags_37 },
    { 0xffabe3d7, SUC_lexicon_tags_7 },
    { 0x530be3d8, SUC_lexicon_tags_28 },
    { 0xa58be3d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228be3de, SUC_lexicon_tags_17 },
    { 0xcb4be3de, SUC_lexicon_tags_2 },
    { 0xf82be3e0, SUC_lexicon_tags_1 },
    { 0xdacbe3de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x592be3e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x622be3e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4cbe3e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcc6be3e9, SUC_lexicon_tags_29 },
    { 0x88be3ea, SUC_lexicon_tags_32 },
    { 0x760be3ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb2be3ed, SUC_lexicon_tags_8 },
    { 0xb48be3ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa24be3f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4be3f4, SUC_lexicon_tags_4 },
    { 0xd4ebe3f5, SUC_lexicon_tags_63 },
    { 0x716be3f6, SUC_lexicon_tags_25 },
    { 0xa76be3f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff0be3f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1b4be3fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cbe3ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc28be401, SUC_lexicon_tags_49 },
    { 0x82cbe402, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbe407, SUC_lexicon_tags_25 },
    { 0xc20be408, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa42be40a, SUC_lexicon_tags_63 },
    { 0x724be40b, SUC_lexicon_tags_8 },
    { 0x648be40b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94be413, SUC_lexicon_tags_32 },
    { 0xdccbe414, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa12be416, SUC_lexicon_tags_7 },
    { 0x382be417, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ebe41b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ebe41e, SUC_lexicon_tags_19 },
    { 0x1d4be41f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80be423, SUC_lexicon_tags_2 },
    { 0x40cbe424, SUC_lexicon_tags_24 },
    { 0xb68be424, SUC_lexicon_tags_7 },
    { 0x460be426, SUC_lexicon_tags_27 },
    { 0x8ebe426, SUC_lexicon_tags_1 },
    { 0xe0ebe427, SUC_lexicon_tags_29 },
    { 0xc6ebe428, SUC_lexicon_tags_19 },
    { 0x92be425, SUC_lexicon_tags_26 },
    { 0x54be429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850be42f, SUC_lexicon_tags_19 },
    { 0xcc6be430, SUC_lexicon_tags_1 },
    { 0xa96be431, SUC_lexicon_tags_12 },
    { 0x5b6be42f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0be43e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x10abe440, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90be446, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618be44d, SUC_lexicon_tags_0 },
    { 0xc1abe44e, SUC_lexicon_tags_45 },
    { 0xd76be44e, SUC_lexicon_tags_26 },
    { 0x15abe44d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12be45a, SUC_lexicon_tags_11 },
    { 0xd4cbe45b, SUC_lexicon_tags_14 },
    { 0x146be45a, SUC_lexicon_tags_14 },
    { 0xfbabe45b, SUC_lexicon_tags_17 },
    { 0x442be45e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cabe461, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x19ebe463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aebe467, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0be46a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5abe46c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb08be46e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x652be471, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e2be473, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152be47f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcbe486, SUC_lexicon_tags_12 },
    { 0x3e6be487, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86be48b, SUC_lexicon_tags_7 },
    { 0xcebe48c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x90be48e, SUC_lexicon_tags_10 },
    { 0x16be48f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x470be491, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaebe494, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf70be496, SUC_lexicon_tags_82 },
    { 0x386be496, SUC_lexicon_tags_37 },
    { 0x6cbe498, SUC_lexicon_tags_28 },
    { 0x204be499, SUC_lexicon_tags_2 },
    { 0xe5abe49a, SUC_lexicon_tags_17 },
    { 0xbbabe49a, SUC_lexicon_tags_27 },
    { 0x9b0be49c, SUC_lexicon_tags_2 },
    { 0x648be49c, SUC_lexicon_tags_0 },
    { 0x61ebe49e, SUC_lexicon_tags_15 },
    { 0x1a2be49e, SUC_lexicon_tags_32 },
    { 0x894be4a0, SUC_lexicon_tags_15 },
    { 0x234be4a1, SUC_lexicon_tags_6 },
    { 0xbccbe49a, SUC_lexicon_tags_11 },
    { 0x304be4a2, SUC_lexicon_tags_29 },
    { 0x51abe4a2, SUC_lexicon_tags_25 },
    { 0x5a4be496, SUC_lexicon_tags_15 },
    { 0x314be497, SUC_lexicon_tags_19 },
    { 0x84be4a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69ebe4a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ebe4ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfeabe4ae, SUC_lexicon_tags_43 },
    { 0x42cbe4af, SUC_lexicon_tags_1 },
    { 0x19abe4af, SUC_lexicon_tags_11 },
    { 0x6c0be4b1, SUC_lexicon_tags_17 },
    { 0x4b6be4b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72be4b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x89abe4b8, SUC_lexicon_tags_11 },
    { 0xec8be4b9, SUC_lexicon_tags_108 },
    { 0xba0be4ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76be4c1, SUC_lexicon_tags_15 },
    { 0x9b6be4c1, SUC_lexicon_tags_15 },
    { 0x5d4be4c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa2be4c5, SUC_lexicon_tags_25 },
    { 0x8e2be4c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cbe4c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148be4ce, SUC_lexicon_tags_3 },
    { 0xc0be4cf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952be4d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ebe4d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6abe4dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68be4df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4be4e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1babe4eb, SUC_lexicon_tags_27 },
    { 0x948be4ec, SUC_lexicon_tags_11 },
    { 0xef0be4eb, SUC_lexicon_tags_15 },
    { 0xfe6be4ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12be4f2, SUC_lexicon_tags_25 },
    { 0x12ebe4f3, SUC_lexicon_tags_31 },
    { 0x984be4f4, SUC_lexicon_tags_21 },
    { 0xc9ebe4f3, SUC_lexicon_tags_12 },
    { 0x4c2be4f3, SUC_lexicon_tags_11 },
    { 0x55ebe4f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658be4fb, SUC_lexicon_tags_34 },
    { 0x372be4fb, SUC_lexicon_tags_2 },
    { 0x88cbe4fd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xbd2be4ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4be502, SUC_lexicon_tags_1 },
    { 0x40abe502, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6be50a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0be50d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88be512, SUC_lexicon_tags_17 },
    { 0x668be513, SUC_lexicon_tags_82 },
    { 0x294be514, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cbe518, SUC_lexicon_tags_8 },
    { 0xdd2be519, SUC_lexicon_tags_17 },
    { 0x282be51a, SUC_lexicon_tags_32 },
    { 0xfa4be51b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70abe526, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2be52a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x260be52c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58abe531, SUC_lexicon_tags_33 },
    { 0xc0cbe531, SUC_lexicon_tags_49 },
    { 0xd50be533, SUC_lexicon_tags_91 },
    { 0xd2cbe534, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x128be537, SUC_lexicon_tags_1 },
    { 0xaa0be538, SUC_lexicon_tags_15 },
    { 0x78be539, SUC_lexicon_tags_37 },
    { 0x516be539, SUC_lexicon_tags_45 },
    { 0x1d4be53b, SUC_lexicon_tags_17 },
    { 0x696be53c, SUC_lexicon_tags_17 },
    { 0x11cbe539, SUC_lexicon_tags_29 },
    { 0x1a6be538, SUC_lexicon_tags_15 },
    { 0x8a0be53f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x552be541, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xfecbe544, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ebe547, SUC_lexicon_tags_27 },
    { 0x4d0be548, SUC_lexicon_tags_25 },
    { 0xf64be547, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cbe555, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ebe560, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa4be562, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80abe564, SUC_lexicon_tags_17 },
    { 0xbb6be565, SUC_lexicon_tags_25 },
    { 0x920be566, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcbe569, SUC_lexicon_tags_9 },
    { 0xa96be56a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x648be56d, SUC_lexicon_tags_63 },
    { 0xa26be56d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70be571, SUC_lexicon_tags_30 },
    { 0x6d4be572, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380be578, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acbe57c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x630be57f, SUC_lexicon_tags_31 },
    { 0x516be57f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cbe583, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd78be585, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72be58f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286be594, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956be598, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5eabe59a, SUC_lexicon_tags_25 },
    { 0x28abe59a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166be5a4, SUC_lexicon_tags_7 },
    { 0xc62be5a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0be5a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60be5ad, SUC_lexicon_tags_17 },
    { 0x996be5ae, SUC_lexicon_tags_11 },
    { 0xe26be5ae, SUC_lexicon_tags_25 },
    { 0x22abe5b0, SUC_lexicon_tags_29 },
    { 0x1e2be5b1, SUC_lexicon_tags_79 },
    { 0x80be5af, SUC_lexicon_tags_37 },
    { 0x582be5b3, SUC_lexicon_tags_22 },
    { 0xfc2be5b2, SUC_lexicon_tags_31 },
    { 0xe40be5b3, SUC_lexicon_tags_38 },
    { 0xe4cbe5b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcbe5b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a6be5ba, SUC_lexicon_tags_32 },
    { 0x732be5bb, SUC_lexicon_tags_30 },
    { 0xb5abe5bb, SUC_lexicon_tags_47 },
    { 0xb44be5bb, SUC_lexicon_tags_2 },
    { 0x9dabe5be, SUC_lexicon_tags_19 },
    { 0xa54be5bf, SUC_lexicon_tags_2 },
    { 0x234be5c0, SUC_lexicon_tags_32 },
    { 0x152be5c1, SUC_lexicon_tags_37 },
    { 0xd9abe5c2, SUC_lexicon_tags_23 },
    { 0x366be5c3, SUC_lexicon_tags_15 },
    { 0xd34be5c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2aebe5c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x126be5c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x272be5cc, SUC_lexicon_tags_12 },
    { 0x13abe5cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0be5d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ebe5db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416be5df, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8be5e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x15ebe5ea, SUC_lexicon_tags_49 },
    { 0x93abe5eb, SUC_lexicon_tags_37 },
    { 0x710be5eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacbe5f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7b8be5f2, SUC_lexicon_tags_23 },
    { 0x64cbe5f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aebe5f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16be5ff, SUC_lexicon_tags_7 },
    { 0xa04be600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6be604, SUC_lexicon_tags_25 },
    { 0x63abe605, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a0be607, SUC_lexicon_tags_8 },
    { 0xbf8be608, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682be60f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fabe612, SUC_lexicon_tags_5 },
    { 0x98cbe613, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44be616, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x896be619, SUC_lexicon_tags_2 },
    { 0x874be61a, SUC_lexicon_tags_29 },
    { 0xee4be619, SUC_lexicon_tags_12 },
    { 0xd4ebe61b, SUC_lexicon_tags_11 },
    { 0x6c6be61b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x47cbe61f, SUC_lexicon_tags_17 },
    { 0xd52be620, SUC_lexicon_tags_81 },
    { 0x6c8be620, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcc6be623, SUC_lexicon_tags_14 },
    { 0x5d0be624, SUC_lexicon_tags_12 },
    { 0x5c4be624, SUC_lexicon_tags_0 },
    { 0x8e6be626, SUC_lexicon_tags_0 },
    { 0x7d8be626, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xab8be629, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24be62e, SUC_lexicon_tags_11 },
    { 0xe5abe62f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e0be631, SUC_lexicon_tags_23 },
    { 0x852be632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac8be634, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46be63a, SUC_lexicon_tags_10 },
    { 0xc08be63a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7abe63d, SUC_lexicon_tags_70 },
    { 0x4c8be63e, SUC_lexicon_tags_87 },
    { 0x582be63f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x950be642, SUC_lexicon_tags_23 },
    { 0x648be642, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4be649, SUC_lexicon_tags_15 },
    { 0xdbabe64a, SUC_lexicon_tags_49 },
    { 0x320be64b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecbe650, SUC_lexicon_tags_0 },
    { 0x91ebe651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0be65c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4be661, SUC_lexicon_tags_11 },
    { 0x1c2be662, SUC_lexicon_tags_5 },
    { 0xde2be662, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4abe669, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cbe66c, SUC_lexicon_tags_9 },
    { 0x16be66d, SUC_lexicon_tags_38 },
    { 0x104be66c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d8be670, SUC_lexicon_tags_27 },
    { 0xa8be671, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x2e2be673, SUC_lexicon_tags_37 },
    { 0x5d8be674, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636be67a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x800be67d, SUC_lexicon_tags_14 },
    { 0x6b2be67d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1debe680, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x18cbe682, SUC_lexicon_tags_37 },
    { 0xf1ebe683, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x782be685, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c0be687, SUC_lexicon_tags_29 },
    { 0x56cbe687, SUC_lexicon_tags_11 },
    { 0xc84be687, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6be68e, SUC_lexicon_tags_8 },
    { 0xd48be68f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x690be692, SUC_lexicon_tags_27 },
    { 0xaecbe693, SUC_lexicon_tags_0 },
    { 0x5f2be694, SUC_lexicon_tags_66 },
    { 0x54cbe694, SUC_lexicon_tags_15 },
    { 0x3bebe696, SUC_lexicon_tags_15 },
    { 0xfbcbe692, SUC_lexicon_tags_12 },
    { 0x60abe698, SUC_lexicon_tags_30 },
    { 0x30ebe698, SUC_lexicon_tags_1161 },
    { 0xb72be693, SUC_lexicon_tags_60 },
    { 0x304be69b, SUC_lexicon_tags_28 },
    { 0xc96be692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x108be69f, SUC_lexicon_tags_12 },
    { 0x250be69f, SUC_lexicon_tags_45 },
    { 0xde4be6a1, SUC_lexicon_tags_1 },
    { 0x344be6a2, SUC_lexicon_tags_17 },
    { 0x3bcbe6a3, SUC_lexicon_tags_12 },
    { 0x3dcbe69f, SUC_lexicon_tags_63 },
    { 0x8e6be6a5, SUC_lexicon_tags_15 },
    { 0x234be6a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2be6ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x75abe6ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8be6b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310be6b6, SUC_lexicon_tags_2 },
    { 0x3ebe6b7, SUC_lexicon_tags_12 },
    { 0xe66be6b8, SUC_lexicon_tags_8 },
    { 0x462be6b7, SUC_lexicon_tags_17 },
    { 0xfabe6ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x30cbe6bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa66be6bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x186be6c2, SUC_lexicon_tags_33 },
    { 0xe58be6c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe2be6c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8be6c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x930be6ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4be6ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66abe6d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95abe6d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x68cbe6d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ebe6da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcbe6dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2be6e0, SUC_lexicon_tags_1 },
    { 0x356be6e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28be6e3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc32be6e5, SUC_lexicon_tags_29 },
    { 0xdfcbe6e6, SUC_lexicon_tags_17 },
    { 0x36abe6e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa42be6e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x856be6eb, SUC_lexicon_tags_7 },
    { 0x632be6ec, SUC_lexicon_tags_37 },
    { 0x6d2be6ed, SUC_lexicon_tags_11 },
    { 0x5babe6ee, SUC_lexicon_tags_2 },
    { 0xb82be6ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72abe6f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2debe6f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc6be6fa, SUC_lexicon_tags_14 },
    { 0xf8be6fa, SUC_lexicon_tags_49 },
    { 0x95cbe6fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc6be6fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172be704, SUC_lexicon_tags_17 },
    { 0x4d4be705, SUC_lexicon_tags_10 },
    { 0xe46be705, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x200be709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ebe70f, SUC_lexicon_tags_19 },
    { 0xbfabe710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cbe715, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8be719, SUC_lexicon_tags_77 },
    { 0x820be71a, SUC_lexicon_tags_7 },
    { 0xc84be71b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276be721, SUC_lexicon_tags_98 },
    { 0xa78be722, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46be727, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x518be72a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x186be72c, SUC_lexicon_tags_49 },
    { 0x61cbe72c, SUC_lexicon_tags_1 },
    { 0x6c2be72c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d4be730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x372be732, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xd4abe734, SUC_lexicon_tags_26 },
    { 0x110be734, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x690be738, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ebe73b, SUC_lexicon_tags_77 },
    { 0x76ebe73c, SUC_lexicon_tags_11 },
    { 0x276be73c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x172be73f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64be744, SUC_lexicon_tags_14 },
    { 0xad4be744, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f8be747, SUC_lexicon_tags_31 },
    { 0xfb2be748, SUC_lexicon_tags_25 },
    { 0x18cbe749, SUC_lexicon_tags_25 },
    { 0x242be749, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc66be74c, SUC_lexicon_tags_32 },
    { 0x27cbe74d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40be751, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4be755, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x37ebe757, SUC_lexicon_tags_14 },
    { 0x7c4be758, SUC_lexicon_tags_63 },
    { 0x738be759, SUC_lexicon_tags_12 },
    { 0xfcabe757, SUC_lexicon_tags_4 },
    { 0x40be75a, SUC_lexicon_tags_56 },
    { 0x7f8be75a, SUC_lexicon_tags_44 },
    { 0xc30be75d, SUC_lexicon_tags_11 },
    { 0xe4cbe75e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420be763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbe766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abe76c, SUC_lexicon_tags_14 },
    { 0x87abe76c, SUC_lexicon_tags_17 },
    { 0x718be76c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x39abe770, SUC_lexicon_tags_19 },
    { 0x60cbe771, SUC_lexicon_tags_31 },
    { 0xf3cbe772, SUC_lexicon_tags_15 },
    { 0xa34be772, SUC_lexicon_tags_37 },
    { 0xa80be772, SUC_lexicon_tags_113 },
    { 0x62cbe775, SUC_lexicon_tags_38 },
    { 0x320be776, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8be778, SUC_lexicon_tags_48 },
    { 0xefebe778, SUC_lexicon_tags_30 },
    { 0x10be77a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6be782, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6be787, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x542be78a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0be78f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8be79a, SUC_lexicon_tags_21 },
    { 0x268be79b, SUC_lexicon_tags_25 },
    { 0x41ebe79c, SUC_lexicon_tags_11 },
    { 0xad0be79b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf96be79f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66be7a3, SUC_lexicon_tags_16 },
    { 0x14abe7a3, SUC_lexicon_tags_55 },
    { 0xd6cbe7a5, SUC_lexicon_tags_26 },
    { 0x238be7a5, SUC_lexicon_tags_25 },
    { 0xfb6be7a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1e4be7a9, SUC_lexicon_tags_43 },
    { 0x6c6be7a9, SUC_lexicon_tags_23 },
    { 0xe84be7a9, SUC_lexicon_tags_28 },
    { 0x94abe7ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9babe7ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x114be7b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x85ebe7b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08be7b8, SUC_lexicon_tags_14 },
    { 0x6c0be7b9, SUC_lexicon_tags_23 },
    { 0x670be7b8, SUC_lexicon_tags_49 },
    { 0x5aebe7bb, SUC_lexicon_tags_11 },
    { 0xb72be7bb, SUC_lexicon_tags_15 },
    { 0xc72be7b9, SUC_lexicon_tags_158 },
    { 0xba6be7bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53abe7c2, SUC_lexicon_tags_37 },
    { 0x560be7c3, SUC_lexicon_tags_63 },
    { 0x90cbe7c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382be7c8, SUC_lexicon_tags_32 },
    { 0x73cbe7c9, SUC_lexicon_tags_25 },
    { 0xde0be7c9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4eebe7cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad6be7ce, SUC_lexicon_tags_14 },
    { 0x1b2be7cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfecbe7d1, SUC_lexicon_tags_45 },
    { 0x8bebe7d1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x430be7d5, SUC_lexicon_tags_33 },
    { 0x190be7d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecbe7de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0be7e5, SUC_lexicon_tags_15 },
    { 0xf36be7e6, SUC_lexicon_tags_77 },
    { 0x346be7e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396be7eb, SUC_lexicon_tags_14 },
    { 0x3fabe7ec, SUC_lexicon_tags_47 },
    { 0xacebe7ed, SUC_lexicon_tags_25 },
    { 0xe72be7ed, SUC_lexicon_tags_45 },
    { 0x4c0be7ec, SUC_lexicon_tags_32 },
    { 0x8e6be7ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4be7f7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x32abe7f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x936be7fc, SUC_lexicon_tags_11 },
    { 0xf10be7fd, SUC_lexicon_tags_15 },
    { 0x356be7fe, SUC_lexicon_tags_48 },
    { 0x964be7ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x82abe802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d6be804, SUC_lexicon_tags_16 },
    { 0xb68be804, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cabe808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ebe80c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5cbe80e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cbe812, SUC_lexicon_tags_8 },
    { 0x55abe813, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2abe815, SUC_lexicon_tags_0 },
    { 0x7e6be816, SUC_lexicon_tags_15 },
    { 0xa96be816, SUC_lexicon_tags_47 },
    { 0xae4be818, SUC_lexicon_tags_47 },
    { 0x99cbe819, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x146be81c, SUC_lexicon_tags_57 },
    { 0x794be81d, SUC_lexicon_tags_25 },
    { 0x21abe81e, SUC_lexicon_tags_14 },
    { 0x254be81e, SUC_lexicon_tags_11 },
    { 0xf06be820, SUC_lexicon_tags_33 },
    { 0x478be821, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8be826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16abe828, SUC_lexicon_tags_33 },
    { 0xa7ebe828, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ebe832, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaaebe834, SUC_lexicon_tags_16 },
    { 0xe2be835, SUC_lexicon_tags_87 },
    { 0x848be834, SUC_lexicon_tags_48 },
    { 0x68cbe836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcbe83d, SUC_lexicon_tags_15 },
    { 0x9b8be83e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f8be840, SUC_lexicon_tags_77 },
    { 0xfdcbe841, SUC_lexicon_tags_17 },
    { 0x50ebe842, SUC_lexicon_tags_0 },
    { 0x4d4be843, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8abe848, SUC_lexicon_tags_16 },
    { 0x686be849, SUC_lexicon_tags_55 },
    { 0x6d0be84a, SUC_lexicon_tags_11 },
    { 0xac6be84b, SUC_lexicon_tags_23 },
    { 0xfeebe84c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6be850, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aabe853, SUC_lexicon_tags_17 },
    { 0x5e4be854, SUC_lexicon_tags_14 },
    { 0x59ebe855, SUC_lexicon_tags_15 },
    { 0xe30be854, SUC_lexicon_tags_19 },
    { 0xeeabe857, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6dcbe859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700be85e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x38cbe861, SUC_lexicon_tags_11 },
    { 0x3b2be861, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fabe865, SUC_lexicon_tags_59 },
    { 0x114be865, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42abe868, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b0be86a, SUC_lexicon_tags_0 },
    { 0x11ebe86b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64be87c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6cbe87e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ebe883, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30be88b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cbe890, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6be895, SUC_lexicon_tags_45 },
    { 0x63abe895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59abe898, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ebe89d, SUC_lexicon_tags_19 },
    { 0x2c2be89e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x584be8a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e6be8a3, SUC_lexicon_tags_27 },
    { 0x278be8a4, SUC_lexicon_tags_7 },
    { 0x6c2be8a5, SUC_lexicon_tags_32 },
    { 0xc2ebe8a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x576be8a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16be8ac, SUC_lexicon_tags_15 },
    { 0xecabe8ad, SUC_lexicon_tags_11 },
    { 0x1b4be8ac, SUC_lexicon_tags_49 },
    { 0x546be8af, SUC_lexicon_tags_17 },
    { 0xd7abe8af, SUC_lexicon_tags_25 },
    { 0xbcabe8af, SUC_lexicon_tags_7 },
    { 0xc84be8b2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebe8b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abe8ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49cbe8bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc82be8be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202be8c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18be8c5, SUC_lexicon_tags_1 },
    { 0xccbe8c6, SUC_lexicon_tags_6 },
    { 0xc50be8c7, SUC_lexicon_tags_26 },
    { 0x7f8be8c8, SUC_lexicon_tags_2 },
    { 0x428be8c9, SUC_lexicon_tags_27 },
    { 0x8b0be8ca, SUC_lexicon_tags_23 },
    { 0x41cbe8c9, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22be8cf, SUC_lexicon_tags_2 },
    { 0x872be8cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cbe8d5, SUC_lexicon_tags_2 },
    { 0x49cbe8d6, SUC_lexicon_tags_11 },
    { 0xb32be8d7, SUC_lexicon_tags_33 },
    { 0xfc6be8d8, SUC_lexicon_tags_6 },
    { 0x494be8d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcabe8dc, SUC_lexicon_tags_36 },
    { 0x400be8dd, SUC_lexicon_tags_47 },
    { 0x3c6be8dd, SUC_lexicon_tags_32 },
    { 0xf64be8dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x144be8e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x152be8e5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x650be8e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcbe8f0, SUC_lexicon_tags_17 },
    { 0x166be8f0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbe8f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x570be8f7, SUC_lexicon_tags_85 },
    { 0xbacbe8f8, SUC_lexicon_tags_6 },
    { 0x59ebe8f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5abe8fc, SUC_lexicon_tags_10 },
    { 0x36ebe8fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6be901, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4be904, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780be908, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xecbe90a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd6cbe90c, SUC_lexicon_tags_15 },
    { 0x36ebe90d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0ebe90f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80be912, SUC_lexicon_tags_46 },
    { 0x97ebe912, SUC_lexicon_tags_17 },
    { 0x60abe914, SUC_lexicon_tags_9 },
    { 0x9a4be915, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb92be917, SUC_lexicon_tags_12 },
    { 0x9d6be918, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecbe91d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428be925, SUC_lexicon_tags_27 },
    { 0x3a0be925, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cebe929, SUC_lexicon_tags_11 },
    { 0x45abe92a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104be930, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8be934, SUC_lexicon_tags_15 },
    { 0xfb4be935, SUC_lexicon_tags_19 },
    { 0x434be934, SUC_lexicon_tags_12 },
    { 0x4c4be937, SUC_lexicon_tags_113 },
    { 0xfe0be937, SUC_lexicon_tags_25 },
    { 0x596be936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x786be93c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x63abe93e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00be941, SUC_lexicon_tags_0 },
    { 0x600be941, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x73ebe944, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6be947, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86be94a, SUC_lexicon_tags_37 },
    { 0xf10be94a, SUC_lexicon_tags_48 },
    { 0x278be94c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd46be94e, SUC_lexicon_tags_8 },
    { 0x4bebe94f, SUC_lexicon_tags_64 },
    { 0x296be94f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cbe953, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176be958, SUC_lexicon_tags_4 },
    { 0x642be959, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40be95b, SUC_lexicon_tags_31 },
    { 0x64abe95b, SUC_lexicon_tags_37 },
    { 0xef4be95d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2be960, SUC_lexicon_tags_37 },
    { 0xf46be960, SUC_lexicon_tags_25 },
    { 0x3fabe960, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8be966, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4be96b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x706be96d, SUC_lexicon_tags_1 },
    { 0xde0be96d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60be971, SUC_lexicon_tags_11 },
    { 0x6eebe972, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578be979, SUC_lexicon_tags_6 },
    { 0xeabe97a, SUC_lexicon_tags_33 },
    { 0x112be97a, SUC_lexicon_tags_1 },
    { 0x17abe97c, SUC_lexicon_tags_17 },
    { 0xfdcbe97d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0be980, SUC_lexicon_tags_3 },
    { 0x184be981, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394be986, SUC_lexicon_tags_15 },
    { 0x5d4be987, SUC_lexicon_tags_13 },
    { 0x95abe988, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0be98f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eabe992, SUC_lexicon_tags_2 },
    { 0x50ebe993, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6abe998, SUC_lexicon_tags_53 },
    { 0x538be999, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1babe99b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4be9a0, SUC_lexicon_tags_25 },
    { 0xaf6be9a1, SUC_lexicon_tags_37 },
    { 0xeb8be9a2, SUC_lexicon_tags_14 },
    { 0xe10be9a3, SUC_lexicon_tags_1 },
    { 0x77abe9a4, SUC_lexicon_tags_28 },
    { 0x10be9a3, SUC_lexicon_tags_945 },
    { 0x5d4be9a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x558be9a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64be9ac, SUC_lexicon_tags_1 },
    { 0xcacbe9ad, SUC_lexicon_tags_7 },
    { 0x472be9ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2be9b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cbe9b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a4be9b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67abe9ba, SUC_lexicon_tags_60 },
    { 0x588be9bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118be9c0, SUC_lexicon_tags_37 },
    { 0xb6be9c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282be9c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8abe9c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56be9cd, SUC_lexicon_tags_65 },
    { 0xc26be9cd, SUC_lexicon_tags_23 },
    { 0xd74be9cf, SUC_lexicon_tags_33 },
    { 0xdb2be9cd, SUC_lexicon_tags_25 },
    { 0x80cbe9ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8be9d6, SUC_lexicon_tags_16 },
    { 0xf4be9d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3e8be9d9, SUC_lexicon_tags_37 },
    { 0xf1abe9da, SUC_lexicon_tags_29 },
    { 0xe34be9db, SUC_lexicon_tags_4 },
    { 0xb3abe9dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x358be9df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6be9e8, SUC_lexicon_tags_2 },
    { 0x820be9e9, SUC_lexicon_tags_15 },
    { 0xc36be9ea, SUC_lexicon_tags_37 },
    { 0x5aebe9ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aebe9ef, SUC_lexicon_tags_37 },
    { 0x94abe9f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ebe9f5, SUC_lexicon_tags_25 },
    { 0xc2abe9f6, SUC_lexicon_tags_12 },
    { 0x894be9f7, SUC_lexicon_tags_37 },
    { 0x7d2be9f7, SUC_lexicon_tags_33 },
    { 0x9a4be9f8, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22be9fc, SUC_lexicon_tags_23 },
    { 0x7a0be9fc, SUC_lexicon_tags_15 },
    { 0x2d8be9fe, SUC_lexicon_tags_31 },
    { 0x85ebe9fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0ebea01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x118bea04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44bea07, SUC_lexicon_tags_29 },
    { 0xcb4bea08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x652bea0a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x564bea0d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7c6bea0f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71abea1a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x120bea1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6bea21, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9babea28, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cbea31, SUC_lexicon_tags_1 },
    { 0xdfabea32, SUC_lexicon_tags_60 },
    { 0x5f4bea33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf14bea35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5dabea37, SUC_lexicon_tags_2 },
    { 0x9c6bea38, SUC_lexicon_tags_7 },
    { 0xb9ebea38, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xb60bea3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44abea40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x344bea42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cbea46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188bea4b, SUC_lexicon_tags_38 },
    { 0xc5abea4c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164bea54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabea59, SUC_lexicon_tags_181 },
    { 0x30cbea59, SUC_lexicon_tags_1 },
    { 0x612bea5a, SUC_lexicon_tags_12 },
    { 0xbbabea5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x702bea5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13abea62, SUC_lexicon_tags_37 },
    { 0xbaabea63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebea67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9abea6b, SUC_lexicon_tags_82 },
    { 0xc3abea6b, SUC_lexicon_tags_11 },
    { 0xd7abea6d, SUC_lexicon_tags_32 },
    { 0x7cabea6e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240bea73, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950bea7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ebea7e, SUC_lexicon_tags_62 },
    { 0xdbabea7f, SUC_lexicon_tags_1 },
    { 0x248bea7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32bea91, SUC_lexicon_tags_15 },
    { 0x5e2bea92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcbea9c, SUC_lexicon_tags_10 },
    { 0x6c2bea9d, SUC_lexicon_tags_25 },
    { 0x7cebea9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ebeaa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe92beaa7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22beaaa, SUC_lexicon_tags_60 },
    { 0x262beaaa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe42beaae, SUC_lexicon_tags_11 },
    { 0x5a6beaaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58beab8, SUC_lexicon_tags_25 },
    { 0x5c6beab8, SUC_lexicon_tags_2 },
    { 0x74ebeaba, SUC_lexicon_tags_4 },
    { 0x1e4beaba, SUC_lexicon_tags_48 },
    { 0xca4beabb, SUC_lexicon_tags_11 },
    { 0x954beab9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138beac2, SUC_lexicon_tags_49 },
    { 0xa68beac3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ebeac6, SUC_lexicon_tags_43 },
    { 0xf10beac6, SUC_lexicon_tags_45 },
    { 0x61cbeac8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcbeace, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acbead1, SUC_lexicon_tags_14 },
    { 0xd98bead2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x124bead4, SUC_lexicon_tags_14 },
    { 0x538bead4, SUC_lexicon_tags_108 },
    { 0x790bead6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca0bead8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908beadc, SUC_lexicon_tags_8 },
    { 0xcf0beadc, SUC_lexicon_tags_15 },
    { 0xd5cbeadc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x330beae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x690beae4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaebeae8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74beaee, SUC_lexicon_tags_27 },
    { 0x590beaef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ebeaf2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114beafc, SUC_lexicon_tags_7 },
    { 0xc90beafd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xceabeb00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcbeb06, SUC_lexicon_tags_25 },
    { 0xb8beb07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb24beb09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ebeb0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x368beb10, SUC_lexicon_tags_11 },
    { 0x904beb10, SUC_lexicon_tags_9 },
    { 0x39cbeb12, SUC_lexicon_tags_1 },
    { 0xedcbeb12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4beb17, SUC_lexicon_tags_33 },
    { 0xbaabeb18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ebeb1e, SUC_lexicon_tags_19 },
    { 0x9f6beb1f, SUC_lexicon_tags_85 },
    { 0x78cbeb1e, SUC_lexicon_tags_1 },
    { 0x7f4beb1e, SUC_lexicon_tags_4 },
    { 0xdb6beb1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc92beb24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76beb28, SUC_lexicon_tags_77 },
    { 0xb7ebeb29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbebeb2c, SUC_lexicon_tags_77 },
    { 0x566beb2d, SUC_lexicon_tags_16 },
    { 0x992beb2e, SUC_lexicon_tags_7 },
    { 0xb24beb2f, SUC_lexicon_tags_80 },
    { 0x15ebeb2f, SUC_lexicon_tags_17 },
    { 0x628beb31, SUC_lexicon_tags_22 },
    { 0xe48beb2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0beb37, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194beb3b, SUC_lexicon_tags_1 },
    { 0xf5cbeb3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acbeb3f, SUC_lexicon_tags_689 },
    { 0xdacbeb40, SUC_lexicon_tags_17 },
    { 0xea4beb40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x520beb44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38beb48, SUC_lexicon_tags_33 },
    { 0x426beb48, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aebeb4c, SUC_lexicon_tags_11 },
    { 0x4a6beb4d, SUC_lexicon_tags_17 },
    { 0x78beb4e, SUC_lexicon_tags_43 },
    { 0x560beb4e, SUC_lexicon_tags_8 },
    { 0xf4beb50, SUC_lexicon_tags_17 },
    { 0x9e6beb51, SUC_lexicon_tags_91 },
    { 0x4cebeb4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x48ebeb54, SUC_lexicon_tags_19 },
    { 0x984beb55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8beb5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ebeb5e, SUC_lexicon_tags_7 },
    { 0xd1cbeb5e, SUC_lexicon_tags_11 },
    { 0xb1ebeb5f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ebeb63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42abeb65, SUC_lexicon_tags_13 },
    { 0x9c4beb66, SUC_lexicon_tags_11 },
    { 0x6c6beb67, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748beb6b, SUC_lexicon_tags_3 },
    { 0x120beb6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x664beb6f, SUC_lexicon_tags_21 },
    { 0xa3cbeb70, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd9cbeb72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa28beb74, SUC_lexicon_tags_37 },
    { 0x69ebeb75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57cbeb77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1abeb7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92beb80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x51abeb83, SUC_lexicon_tags_15 },
    { 0xfa6beb84, SUC_lexicon_tags_29 },
    { 0xe28beb85, SUC_lexicon_tags_15 },
    { 0xbe4beb85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90beb89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x912beb8c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0beb8f, SUC_lexicon_tags_12 },
    { 0xcdebeb8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2beb93, SUC_lexicon_tags_1 },
    { 0x22beb94, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x55abeb96, SUC_lexicon_tags_31 },
    { 0xdc6beb97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2abeb9e, SUC_lexicon_tags_33 },
    { 0xb8cbeb9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8beba2, SUC_lexicon_tags_37 },
    { 0x4c8beba2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68beba8, SUC_lexicon_tags_7 },
    { 0x378beba9, SUC_lexicon_tags_8 },
    { 0xd18beba8, SUC_lexicon_tags_17 },
    { 0xd22bebab, SUC_lexicon_tags_17 },
    { 0xbb8bebac, SUC_lexicon_tags_47 },
    { 0x7ecbebab, SUC_lexicon_tags_14 },
    { 0x204bebae, SUC_lexicon_tags_64 },
    { 0x782bebaf, SUC_lexicon_tags_12 },
    { 0xd00bebaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x514bebb3, SUC_lexicon_tags_27 },
    { 0x392bebb4, SUC_lexicon_tags_37 },
    { 0x2aebebb5, SUC_lexicon_tags_47 },
    { 0xd8cbebb3, SUC_lexicon_tags_6 },
    { 0xcf0bebb4, SUC_lexicon_tags_17 },
    { 0x198bebb8, SUC_lexicon_tags_49 },
    { 0x1b2bebb3, SUC_lexicon_tags_12 },
    { 0x98abebba, SUC_lexicon_tags_17 },
    { 0x6dabebba, SUC_lexicon_tags_37 },
    { 0xd94bebb7, SUC_lexicon_tags_15 },
    { 0xbb4bebbc, SUC_lexicon_tags_17 },
    { 0x41ebebba, SUC_lexicon_tags_65 },
    { 0xd80bebbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15cbebc1, SUC_lexicon_tags_126 },
    { 0x280bebc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd68bebc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2bebc7, SUC_lexicon_tags_0 },
    { 0xc72bebc8, SUC_lexicon_tags_12 },
    { 0x57ebebc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00bebcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x418bebce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40bebd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272bebd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x860bebd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98bebde, SUC_lexicon_tags_9 },
    { 0xe90bebdf, SUC_lexicon_tags_29 },
    { 0xe00bebde, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ebebe3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x45cbebe5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcbebe9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ebebed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cbebf7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eabebfd, SUC_lexicon_tags_37 },
    { 0x9cabebfd, SUC_lexicon_tags_31 },
    { 0xbb6bebff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x984bec02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21cbec04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad8bec06, SUC_lexicon_tags_7 },
    { 0x3cbec06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa28bec09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb3cbec0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4bec15, SUC_lexicon_tags_55 },
    { 0x264bec16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb0bec18, SUC_lexicon_tags_37 },
    { 0x72cbec19, SUC_lexicon_tags_38 },
    { 0x588bec1a, SUC_lexicon_tags_37 },
    { 0x734bec1b, SUC_lexicon_tags_12 },
    { 0xea4bec1c, SUC_lexicon_tags_17 },
    { 0xb74bec1d, SUC_lexicon_tags_2 },
    { 0x4b8bec1e, SUC_lexicon_tags_9 },
    { 0x350bec1b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x346bec21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0abec23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242bec27, SUC_lexicon_tags_11 },
    { 0xdf2bec27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6bec2c, SUC_lexicon_tags_37 },
    { 0x48ebec2d, SUC_lexicon_tags_8 },
    { 0x38cbec2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6bec32, SUC_lexicon_tags_37 },
    { 0xfaabec32, SUC_lexicon_tags_37 },
    { 0x10cbec34, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcbec39, SUC_lexicon_tags_17 },
    { 0x8f4bec3a, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4bec3e, SUC_lexicon_tags_26 },
    { 0xf4cbec3f, SUC_lexicon_tags_25 },
    { 0x658bec40, SUC_lexicon_tags_88 },
    { 0x9c2bec3f, SUC_lexicon_tags_39 },
    { 0x876bec42, SUC_lexicon_tags_17 },
    { 0x948bec3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4e2bec45, SUC_lexicon_tags_68 },
    { 0xc56bec45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72bec49, SUC_lexicon_tags_7 },
    { 0x24abec4a, SUC_lexicon_tags_10 },
    { 0x63ebec4b, SUC_lexicon_tags_21 },
    { 0x12ebec4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca4bec4e, SUC_lexicon_tags_11 },
    { 0xeeebec4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ebec52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x254bec55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd7cbec57, SUC_lexicon_tags_68 },
    { 0xc90bec58, SUC_lexicon_tags_0 },
    { 0xbe8bec59, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd8ebec5b, SUC_lexicon_tags_12 },
    { 0xb84bec5c, SUC_lexicon_tags_43 },
    { 0x4eebec5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d0bec5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x914bec61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ebec65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeacbec68, SUC_lexicon_tags_1 },
    { 0xb88bec69, SUC_lexicon_tags_82 },
    { 0x3f4bec6a, SUC_lexicon_tags_55 },
    { 0xb34bec6b, SUC_lexicon_tags_37 },
    { 0xda4bec6b, SUC_lexicon_tags_27 },
    { 0xc18bec6d, SUC_lexicon_tags_60 },
    { 0x608bec68, SUC_lexicon_tags_27 },
    { 0x784bec6f, SUC_lexicon_tags_63 },
    { 0x7a8bec70, SUC_lexicon_tags_37 },
    { 0xd46bec71, SUC_lexicon_tags_47 },
    { 0x8c0bec72, SUC_lexicon_tags_47 },
    { 0xda4bec6f, SUC_lexicon_tags_46 },
    { 0x2bec70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cbec7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32bec80, SUC_lexicon_tags_8 },
    { 0xd50bec81, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x56cbec83, SUC_lexicon_tags_1 },
    { 0x96cbec84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb00bec86, SUC_lexicon_tags_11 },
    { 0x414bec86, SUC_lexicon_tags_2 },
    { 0x9d4bec86, SUC_lexicon_tags_84 },
    { 0xc0cbec89, SUC_lexicon_tags_34 },
    { 0x4e6bec8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2abec8c, SUC_lexicon_tags_13 },
    { 0x90ebec8c, SUC_lexicon_tags_19 },
    { 0x432bec8e, SUC_lexicon_tags_8 },
    { 0x6a2bec8f, SUC_lexicon_tags_25 },
    { 0xef6bec8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90bec95, SUC_lexicon_tags_30 },
    { 0xc98bec95, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbeca0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cbeca3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbecad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x338becb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3abecb4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b2becb6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124becba, SUC_lexicon_tags_37 },
    { 0x6debecbb, SUC_lexicon_tags_18 },
    { 0x96becbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x198becbe, SUC_lexicon_tags_37 },
    { 0xb3abecbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc18becc1, SUC_lexicon_tags_2 },
    { 0xfcebecc1, SUC_lexicon_tags_271 },
    { 0xa5cbecc2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x84becc6, SUC_lexicon_tags_10 },
    { 0x75abecc7, SUC_lexicon_tags_14 },
    { 0x536becc7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8becd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56becd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf6becd8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4becdd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322bece1, SUC_lexicon_tags_37 },
    { 0xa4ebece1, SUC_lexicon_tags_23 },
    { 0x570bece3, SUC_lexicon_tags_12 },
    { 0xe9cbece4, SUC_lexicon_tags_17 },
    { 0xb40bece5, SUC_lexicon_tags_35 },
    { 0x5c4bece4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464becf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e0becf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x250becf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2becf8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x90cbecfa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc84becfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53ebecfe, SUC_lexicon_tags_1 },
    { 0xa7ebecff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cbed02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f8bed04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x394bed07, SUC_lexicon_tags_25 },
    { 0x89abed07, SUC_lexicon_tags_2 },
    { 0x390bed08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844bed0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbabed14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4bed16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cbed1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe22bed1d, SUC_lexicon_tags_43 },
    { 0x51abed1e, SUC_lexicon_tags_4 },
    { 0x28bed1e, SUC_lexicon_tags_17 },
    { 0xd6bed20, SUC_lexicon_tags_781 },
    { 0, NULL },
    { 0, NULL },
    { 0x914bed23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebed26, SUC_lexicon_tags_0 },
    { 0x930bed27, SUC_lexicon_tags_45 },
    { 0x7f0bed27, SUC_lexicon_tags_33 },
    { 0xedcbed29, SUC_lexicon_tags_43 },
    { 0x338bed2a, SUC_lexicon_tags_31 },
    { 0xdeebed28, SUC_lexicon_tags_17 },
    { 0x318bed27, SUC_lexicon_tags_66 },
    { 0x666bed2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x342bed2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4bed32, SUC_lexicon_tags_17 },
    { 0xeb2bed33, SUC_lexicon_tags_28 },
    { 0xadabed34, SUC_lexicon_tags_5 },
    { 0x518bed34, SUC_lexicon_tags_14 },
    { 0x65abed34, SUC_lexicon_tags_151 },
    { 0xd16bed37, SUC_lexicon_tags_33 },
    { 0xd10bed35, SUC_lexicon_tags_37 },
    { 0xdb0bed39, SUC_lexicon_tags_47 },
    { 0x8d6bed32, SUC_lexicon_tags_82 },
    { 0x9a4bed39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89abed3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696bed44, SUC_lexicon_tags_35 },
    { 0x2c2bed45, SUC_lexicon_tags_9 },
    { 0x73ebed44, SUC_lexicon_tags_17 },
    { 0x6fabed47, SUC_lexicon_tags_17 },
    { 0x286bed48, SUC_lexicon_tags_7 },
    { 0xc4ebed49, SUC_lexicon_tags_4 },
    { 0x1dcbed47, SUC_lexicon_tags_6 },
    { 0xb8ebed46, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4bed4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0bed52, SUC_lexicon_tags_62 },
    { 0xaa4bed53, SUC_lexicon_tags_27 },
    { 0xfabed54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x95cbed56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaebed5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbabed5d, SUC_lexicon_tags_2 },
    { 0xe84bed5e, SUC_lexicon_tags_17 },
    { 0x8e4bed5f, SUC_lexicon_tags_56 },
    { 0x2c8bed5e, SUC_lexicon_tags_63 },
    { 0x92abed5e, SUC_lexicon_tags_33 },
    { 0x326bed62, SUC_lexicon_tags_47 },
    { 0xbfebed63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcbed69, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7c0bed6b, SUC_lexicon_tags_12 },
    { 0x920bed6b, SUC_lexicon_tags_3 },
    { 0xa9ebed6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106bed76, SUC_lexicon_tags_57 },
    { 0x65ebed77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x786bed79, SUC_lexicon_tags_12 },
    { 0xbc2bed79, SUC_lexicon_tags_15 },
    { 0xec8bed7a, SUC_lexicon_tags_14 },
    { 0x4f4bed7b, SUC_lexicon_tags_65 },
    { 0xebebed7d, SUC_lexicon_tags_6 },
    { 0xf7abed7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6bed82, SUC_lexicon_tags_25 },
    { 0x37cbed83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x464bed85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8bed88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdd6bed8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a6bed8c, SUC_lexicon_tags_31 },
    { 0xbabed8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x920bed8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x372bed91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924bed95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966bed99, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2beda1, SUC_lexicon_tags_0 },
    { 0x1a6beda2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29ebeda4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcebedab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d2bedad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52bedb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ebedb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4eabedbb, SUC_lexicon_tags_2 },
    { 0x9acbedbc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8bedc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ebedc4, SUC_lexicon_tags_19 },
    { 0x528bedc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6bedcb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2bedce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x738bedd0, SUC_lexicon_tags_366 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ebedd6, SUC_lexicon_tags_38 },
    { 0xe50bedd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6bedd9, SUC_lexicon_tags_29 },
    { 0x32cbedda, SUC_lexicon_tags_15 },
    { 0x5dcbeddb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964bede2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534bede6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62bedeb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cbedee, SUC_lexicon_tags_2 },
    { 0x6debedef, SUC_lexicon_tags_11 },
    { 0x654bedf0, SUC_lexicon_tags_15 },
    { 0xf80bedf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cbedf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bedfb, SUC_lexicon_tags_21 },
    { 0xc62bedfb, SUC_lexicon_tags_7 },
    { 0x618bedfd, SUC_lexicon_tags_25 },
    { 0x8d4bedfe, SUC_lexicon_tags_11 },
    { 0xd4ebedff, SUC_lexicon_tags_0 },
    { 0x6dabee00, SUC_lexicon_tags_37 },
    { 0x95ebee01, SUC_lexicon_tags_60 },
    { 0x966bee01, SUC_lexicon_tags_12 },
    { 0x598bee03, SUC_lexicon_tags_11 },
    { 0x2aabee04, SUC_lexicon_tags_25 },
    { 0xf32bee05, SUC_lexicon_tags_26 },
    { 0xe94bee06, SUC_lexicon_tags_12 },
    { 0x3dabee07, SUC_lexicon_tags_39 },
    { 0x578bee07, SUC_lexicon_tags_26 },
    { 0x310bee09, SUC_lexicon_tags_15 },
    { 0xfaabee03, SUC_lexicon_tags_63 },
    { 0xbf2bedfe, SUC_lexicon_tags_1 },
    { 0xc0abedfb, SUC_lexicon_tags_15 },
    { 0xfe4bee0d, SUC_lexicon_tags_39 },
    { 0x8b4bee0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd80bee10, SUC_lexicon_tags_7 },
    { 0xab4bee11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0bee18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea8bee1a, SUC_lexicon_tags_47 },
    { 0x980bee1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2bee1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94bee23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x760bee25, SUC_lexicon_tags_0 },
    { 0xf5ebee26, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574bee2c, SUC_lexicon_tags_11 },
    { 0xafabee2d, SUC_lexicon_tags_2 },
    { 0x3e8bee2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8bee31, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88abee36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x786bee39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6bee3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf0bee3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392bee44, SUC_lexicon_tags_12 },
    { 0x2c6bee45, SUC_lexicon_tags_49 },
    { 0xbebee45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cebee49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898bee4d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ebee51, SUC_lexicon_tags_14 },
    { 0xaa0bee52, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x292bee54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x296bee57, SUC_lexicon_tags_1 },
    { 0x348bee58, SUC_lexicon_tags_7 },
    { 0xf0abee58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04bee5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4bee5f, SUC_lexicon_tags_0 },
    { 0x7b0bee60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x668bee63, SUC_lexicon_tags_17 },
    { 0x60cbee64, SUC_lexicon_tags_6 },
    { 0x4ecbee64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2bee6a, SUC_lexicon_tags_11 },
    { 0xe40bee6b, SUC_lexicon_tags_7 },
    { 0x74abee6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebee70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x284bee72, SUC_lexicon_tags_37 },
    { 0xf6abee73, SUC_lexicon_tags_2 },
    { 0xed0bee74, SUC_lexicon_tags_1 },
    { 0xbecbee75, SUC_lexicon_tags_37 },
    { 0x328bee76, SUC_lexicon_tags_63 },
    { 0x5a2bee72, SUC_lexicon_tags_12 },
    { 0x74abee78, SUC_lexicon_tags_53 },
    { 0x60bee78, SUC_lexicon_tags_25 },
    { 0xfa0bee7a, SUC_lexicon_tags_13 },
    { 0x216bee78, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5febee87, SUC_lexicon_tags_37 },
    { 0x1b6bee88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2eebee8a, SUC_lexicon_tags_30 },
    { 0x3eabee8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cbee98, SUC_lexicon_tags_43 },
    { 0xebcbee99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6beea3, SUC_lexicon_tags_17 },
    { 0x8aebeea4, SUC_lexicon_tags_17 },
    { 0xec4beea5, SUC_lexicon_tags_17 },
    { 0x948beea4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa88beea8, SUC_lexicon_tags_63 },
    { 0xd10beea9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x452beeab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4f8beead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cbeeb0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5f6beeb2, SUC_lexicon_tags_37 },
    { 0x2c6beeb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8beeb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630beebc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x324beebe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcbeec1, SUC_lexicon_tags_56 },
    { 0x646beec2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fabeec5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa12beec7, SUC_lexicon_tags_1 },
    { 0x12ebeec7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f0beeca, SUC_lexicon_tags_28 },
    { 0x38beecb, SUC_lexicon_tags_25 },
    { 0xb7cbeecb, SUC_lexicon_tags_87 },
    { 0x560beecb, SUC_lexicon_tags_7 },
    { 0x954beecb, SUC_lexicon_tags_62 },
    { 0xd1abeecf, SUC_lexicon_tags_17 },
    { 0x11abeeca, SUC_lexicon_tags_34 },
    { 0x284beed1, SUC_lexicon_tags_11 },
    { 0x3debeed2, SUC_lexicon_tags_41 },
    { 0x1f0beed3, SUC_lexicon_tags_26 },
    { 0x9cebeed1, SUC_lexicon_tags_45 },
    { 0x7c2beecd, SUC_lexicon_tags_30 },
    { 0x83cbeece, SUC_lexicon_tags_16 },
    { 0xe6abeed7, SUC_lexicon_tags_25 },
    { 0x7fcbeed7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x404beeda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34beee0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacebeee8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x122beeea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2beeee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x654beef1, SUC_lexicon_tags_140 },
    { 0xf68beef1, SUC_lexicon_tags_63 },
    { 0x8eebeef1, SUC_lexicon_tags_31 },
    { 0x5ccbeef1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x216beef7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2cabeef9, SUC_lexicon_tags_47 },
    { 0xd56beefa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xed4beefc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a2beefe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8bef01, SUC_lexicon_tags_25 },
    { 0x4ccbef02, SUC_lexicon_tags_34 },
    { 0x184bef03, SUC_lexicon_tags_25 },
    { 0x41abef04, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x896bef07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582bef0f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2abef12, SUC_lexicon_tags_56 },
    { 0xf24bef13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450bef1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bef1e, SUC_lexicon_tags_31 },
    { 0x1e0bef1f, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeabef24, SUC_lexicon_tags_56 },
    { 0x9acbef25, SUC_lexicon_tags_37 },
    { 0xf10bef26, SUC_lexicon_tags_3 },
    { 0x894bef26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26bef2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63abef31, SUC_lexicon_tags_77 },
    { 0x16ebef32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40ebef34, SUC_lexicon_tags_27 },
    { 0xedebef35, SUC_lexicon_tags_64 },
    { 0x46bef36, SUC_lexicon_tags_15 },
    { 0x582bef35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bef3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63cbef3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x44bef3e, SUC_lexicon_tags_6 },
    { 0xe42bef3e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4bef42, SUC_lexicon_tags_39 },
    { 0x75abef42, SUC_lexicon_tags_2 },
    { 0x274bef44, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8bef47, SUC_lexicon_tags_37 },
    { 0xb12bef48, SUC_lexicon_tags_27 },
    { 0xba8bef48, SUC_lexicon_tags_9 },
    { 0x14cbef4a, SUC_lexicon_tags_9 },
    { 0x558bef47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a0bef4d, SUC_lexicon_tags_60 },
    { 0xab4bef4e, SUC_lexicon_tags_37 },
    { 0x472bef4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e4bef51, SUC_lexicon_tags_7 },
    { 0xc2bef51, SUC_lexicon_tags_34 },
    { 0x80cbef53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6bef57, SUC_lexicon_tags_15 },
    { 0x844bef58, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680bef62, SUC_lexicon_tags_2 },
    { 0x65abef63, SUC_lexicon_tags_55 },
    { 0xee4bef64, SUC_lexicon_tags_9 },
    { 0x138bef62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bef69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecbef6c, SUC_lexicon_tags_37 },
    { 0x1d6bef6c, SUC_lexicon_tags_15 },
    { 0x2f0bef6e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bef73, SUC_lexicon_tags_55 },
    { 0x720bef74, SUC_lexicon_tags_17 },
    { 0x88ebef75, SUC_lexicon_tags_23 },
    { 0xf16bef75, SUC_lexicon_tags_8 },
    { 0xe76bef77, SUC_lexicon_tags_87 },
    { 0x698bef78, SUC_lexicon_tags_43 },
    { 0x7b8bef74, SUC_lexicon_tags_34 },
    { 0xd38bef77, SUC_lexicon_tags_11 },
    { 0x3bcbef75, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe88bef7d, SUC_lexicon_tags_17 },
    { 0xee4bef7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x750bef81, SUC_lexicon_tags_31 },
    { 0x5fcbef82, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aebef85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4bef88, SUC_lexicon_tags_25 },
    { 0x82abef88, SUC_lexicon_tags_12 },
    { 0x7cebef89, SUC_lexicon_tags_8 },
    { 0x22cbef8b, SUC_lexicon_tags_12 },
    { 0x5c2bef8c, SUC_lexicon_tags_11 },
    { 0xcc0bef8c, SUC_lexicon_tags_37 },
    { 0x9e8bef89, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x224bef90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44cbef93, SUC_lexicon_tags_0 },
    { 0x3f8bef93, SUC_lexicon_tags_0 },
    { 0xcacbef95, SUC_lexicon_tags_25 },
    { 0xb6bef93, SUC_lexicon_tags_25 },
    { 0x404bef96, SUC_lexicon_tags_47 },
    { 0x724bef94, SUC_lexicon_tags_28 },
    { 0x3ccbef99, SUC_lexicon_tags_9 },
    { 0x912bef93, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0bef9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812befa4, SUC_lexicon_tags_23 },
    { 0x300befa5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26befb3, SUC_lexicon_tags_26 },
    { 0xfa6befb4, SUC_lexicon_tags_6 },
    { 0x826befb5, SUC_lexicon_tags_17 },
    { 0xa8ebefb6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfb4befb8, SUC_lexicon_tags_37 },
    { 0x656befb8, SUC_lexicon_tags_19 },
    { 0x1c4befba, SUC_lexicon_tags_124 },
    { 0xb7cbefbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6befc2, SUC_lexicon_tags_7 },
    { 0xa16befc2, SUC_lexicon_tags_26 },
    { 0xe8befc4, SUC_lexicon_tags_13 },
    { 0xacbefc2, SUC_lexicon_tags_15 },
    { 0x3d6befc6, SUC_lexicon_tags_27 },
    { 0xeb0befc6, SUC_lexicon_tags_6 },
    { 0xd50befc3, SUC_lexicon_tags_4 },
    { 0xc6befc8, SUC_lexicon_tags_30 },
    { 0xc60befca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5abefcd, SUC_lexicon_tags_30 },
    { 0x874befce, SUC_lexicon_tags_15 },
    { 0x2b4befcf, SUC_lexicon_tags_12 },
    { 0xccebefd0, SUC_lexicon_tags_17 },
    { 0xfe2befd1, SUC_lexicon_tags_9 },
    { 0xc6befd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcbefda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2c6befdc, SUC_lexicon_tags_30 },
    { 0x6acbefdd, SUC_lexicon_tags_33 },
    { 0x408befde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8befe3, SUC_lexicon_tags_0 },
    { 0xe00befe4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94befe7, SUC_lexicon_tags_37 },
    { 0xc2ebefe7, SUC_lexicon_tags_38 },
    { 0x8eebefe7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4beff7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aabf002, SUC_lexicon_tags_17 },
    { 0x79abf003, SUC_lexicon_tags_7 },
    { 0x300bf002, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38bf007, SUC_lexicon_tags_25 },
    { 0xeeebf007, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x12cbf00a, SUC_lexicon_tags_33 },
    { 0x68abf00a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70cbf00d, SUC_lexicon_tags_46 },
    { 0xe24bf00e, SUC_lexicon_tags_2 },
    { 0x748bf00e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5a6bf011, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8debf015, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98bf017, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584bf01b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x616bf01d, SUC_lexicon_tags_46 },
    { 0x3cbf01e, SUC_lexicon_tags_12 },
    { 0x568bf01d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ebf026, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41abf02c, SUC_lexicon_tags_39 },
    { 0x1eabf02d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0bf030, SUC_lexicon_tags_46 },
    { 0x474bf031, SUC_lexicon_tags_31 },
    { 0xadebf032, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cbf035, SUC_lexicon_tags_14 },
    { 0x90cbf035, SUC_lexicon_tags_43 },
    { 0x826bf037, SUC_lexicon_tags_87 },
    { 0xb00bf037, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0abf03b, SUC_lexicon_tags_12 },
    { 0xdd4bf03c, SUC_lexicon_tags_7 },
    { 0x6b4bf03d, SUC_lexicon_tags_23 },
    { 0x64cbf03d, SUC_lexicon_tags_63 },
    { 0xf0cbf03e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x572bf041, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20bf045, SUC_lexicon_tags_15 },
    { 0x8bf046, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46bf04b, SUC_lexicon_tags_12 },
    { 0xd4cbf04c, SUC_lexicon_tags_8 },
    { 0xf56bf04c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2bf051, SUC_lexicon_tags_11 },
    { 0xab2bf052, SUC_lexicon_tags_56 },
    { 0x6ecbf053, SUC_lexicon_tags_23 },
    { 0xbe2bf054, SUC_lexicon_tags_19 },
    { 0xbaebf055, SUC_lexicon_tags_37 },
    { 0x9e2bf056, SUC_lexicon_tags_45 },
    { 0xc8ebf052, SUC_lexicon_tags_57 },
    { 0x6abf057, SUC_lexicon_tags_17 },
    { 0x794bf059, SUC_lexicon_tags_12 },
    { 0x3a6bf057, SUC_lexicon_tags_12 },
    { 0x71abf05b, SUC_lexicon_tags_29 },
    { 0x5c0bf05c, SUC_lexicon_tags_31 },
    { 0x8d8bf05d, SUC_lexicon_tags_17 },
    { 0x8c0bf055, SUC_lexicon_tags_2 },
    { 0x2c0bf053, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6bf062, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762bf066, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x280bf068, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69abf06e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ebf071, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eabf075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f2bf077, SUC_lexicon_tags_2 },
    { 0xe68bf078, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbdabf07a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84bf07d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc10bf07f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x54bf081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168bf086, SUC_lexicon_tags_37 },
    { 0x9d0bf087, SUC_lexicon_tags_25 },
    { 0x9debf087, SUC_lexicon_tags_138 },
    { 0x250bf089, SUC_lexicon_tags_11 },
    { 0xf16bf087, SUC_lexicon_tags_1 },
    { 0xbe8bf087, SUC_lexicon_tags_77 },
    { 0xd56bf089, SUC_lexicon_tags_25 },
    { 0x804bf08d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570bf092, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bf095, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdbcbf097, SUC_lexicon_tags_17 },
    { 0xccbf098, SUC_lexicon_tags_25 },
    { 0x9e0bf099, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x358bf09b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2bf0a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x202bf0a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x594bf0a8, SUC_lexicon_tags_66 },
    { 0x2f2bf0a8, SUC_lexicon_tags_6 },
    { 0x3cebf0aa, SUC_lexicon_tags_63 },
    { 0x790bf0a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4bf0b0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460bf0b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60ebf0b6, SUC_lexicon_tags_49 },
    { 0xc0ebf0b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ebf0bf, SUC_lexicon_tags_7 },
    { 0xcf4bf0bf, SUC_lexicon_tags_49 },
    { 0x1f6bf0c0, SUC_lexicon_tags_16 },
    { 0xac2bf0c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78cbf0c5, SUC_lexicon_tags_37 },
    { 0x806bf0c6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ebf0c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bcbf0cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x206bf0ce, SUC_lexicon_tags_29 },
    { 0xd7cbf0ce, SUC_lexicon_tags_15 },
    { 0x484bf0d0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x274bf0d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x172bf0d5, SUC_lexicon_tags_10 },
    { 0xb64bf0d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0bf0dd, SUC_lexicon_tags_45 },
    { 0x49abf0de, SUC_lexicon_tags_136 },
    { 0xe16bf0de, SUC_lexicon_tags_85 },
    { 0xd6abf0dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334bf0e4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16bf0e7, SUC_lexicon_tags_11 },
    { 0x32ebf0e8, SUC_lexicon_tags_80 },
    { 0x10cbf0e9, SUC_lexicon_tags_66 },
    { 0xc90bf0e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x542bf0ec, SUC_lexicon_tags_16 },
    { 0xb8abf0ed, SUC_lexicon_tags_306 },
    { 0x214bf0ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4bf0f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecbf0f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42bf0f9, SUC_lexicon_tags_7 },
    { 0xb60bf0fa, SUC_lexicon_tags_19 },
    { 0x956bf0f9, SUC_lexicon_tags_26 },
    { 0x87ebf0fa, SUC_lexicon_tags_37 },
    { 0x602bf0fd, SUC_lexicon_tags_185 },
    { 0x2ebf0fb, SUC_lexicon_tags_7 },
    { 0x468bf0fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x610bf101, SUC_lexicon_tags_25 },
    { 0x870bf102, SUC_lexicon_tags_26 },
    { 0x48bf103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca6bf105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2cbf107, SUC_lexicon_tags_7 },
    { 0x79abf108, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf10d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cbf10f, SUC_lexicon_tags_27 },
    { 0x826bf110, SUC_lexicon_tags_14 },
    { 0x4c2bf111, SUC_lexicon_tags_1 },
    { 0x660bf111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xceabf115, SUC_lexicon_tags_61 },
    { 0x91cbf115, SUC_lexicon_tags_12 },
    { 0xc1ebf117, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53abf11b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x510bf11e, SUC_lexicon_tags_45 },
    { 0x37cbf11e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecbf128, SUC_lexicon_tags_17 },
    { 0xb98bf128, SUC_lexicon_tags_12 },
    { 0x9b4bf12a, SUC_lexicon_tags_17 },
    { 0x54cbf12b, SUC_lexicon_tags_31 },
    { 0x748bf12a, SUC_lexicon_tags_17 },
    { 0xcb2bf12a, SUC_lexicon_tags_8 },
    { 0x72ebf128, SUC_lexicon_tags_60 },
    { 0xaf2bf12f, SUC_lexicon_tags_21 },
    { 0xc44bf128, SUC_lexicon_tags_12 },
    { 0xd32bf131, SUC_lexicon_tags_37 },
    { 0x89abf132, SUC_lexicon_tags_1 },
    { 0x984bf133, SUC_lexicon_tags_23 },
    { 0xd50bf133, SUC_lexicon_tags_25 },
    { 0x644bf135, SUC_lexicon_tags_12 },
    { 0x5dabf133, SUC_lexicon_tags_11 },
    { 0x4b8bf137, SUC_lexicon_tags_15 },
    { 0xf30bf137, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x210bf13a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832bf13f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecbf142, SUC_lexicon_tags_33 },
    { 0x55cbf142, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820bf149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4abf14d, SUC_lexicon_tags_1 },
    { 0xbc8bf14e, SUC_lexicon_tags_17 },
    { 0x5a8bf14e, SUC_lexicon_tags_7 },
    { 0x582bf14f, SUC_lexicon_tags_26 },
    { 0x49ebf150, SUC_lexicon_tags_25 },
    { 0x4eebf152, SUC_lexicon_tags_6 },
    { 0x9fabf152, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41cbf156, SUC_lexicon_tags_66 },
    { 0xe90bf157, SUC_lexicon_tags_43 },
    { 0x7f2bf158, SUC_lexicon_tags_32 },
    { 0x41abf159, SUC_lexicon_tags_30 },
    { 0xc4bf156, SUC_lexicon_tags_7 },
    { 0xf04bf15b, SUC_lexicon_tags_37 },
    { 0xee6bf15c, SUC_lexicon_tags_8 },
    { 0x452bf15b, SUC_lexicon_tags_25 },
    { 0xf6ebf15b, SUC_lexicon_tags_26 },
    { 0x8eebf158, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5abf162, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48bf165, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabf167, SUC_lexicon_tags_11 },
    { 0xe9ebf167, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x514bf16b, SUC_lexicon_tags_12 },
    { 0x14abf16b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ebf174, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x676bf176, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xae4bf178, SUC_lexicon_tags_28 },
    { 0xe78bf179, SUC_lexicon_tags_1 },
    { 0x77ebf17a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338bf17f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x492bf181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4bf184, SUC_lexicon_tags_63 },
    { 0xb46bf185, SUC_lexicon_tags_31 },
    { 0xdd8bf185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbebf188, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b0bf18a, SUC_lexicon_tags_37 },
    { 0x522bf18b, SUC_lexicon_tags_11 },
    { 0x43ebf18a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x438bf18e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb8bf190, SUC_lexicon_tags_11 },
    { 0x1e4bf190, SUC_lexicon_tags_12 },
    { 0xbd8bf192, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2bf196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18bf199, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x39ebf19b, SUC_lexicon_tags_15 },
    { 0x392bf19c, SUC_lexicon_tags_10 },
    { 0xde6bf19d, SUC_lexicon_tags_7 },
    { 0x7cebf19c, SUC_lexicon_tags_201 },
    { 0x3dabf19e, SUC_lexicon_tags_14 },
    { 0xa9cbf19c, SUC_lexicon_tags_45 },
    { 0x61cbf1a0, SUC_lexicon_tags_79 },
    { 0xdb6bf19e, SUC_lexicon_tags_2 },
    { 0x502bf19c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x584bf1a6, SUC_lexicon_tags_23 },
    { 0xf02bf1a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1bcbf1a9, SUC_lexicon_tags_45 },
    { 0x9d6bf1a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2bf1ae, SUC_lexicon_tags_38 },
    { 0x588bf1af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14bf1ba, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xedebf1bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aebf1c1, SUC_lexicon_tags_92 },
    { 0xd6cbf1c2, SUC_lexicon_tags_30 },
    { 0x152bf1c3, SUC_lexicon_tags_21 },
    { 0x49cbf1c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfabf1c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf14bf1c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8bf1ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cbf1d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ebf1d5, SUC_lexicon_tags_43 },
    { 0xc5abf1d5, SUC_lexicon_tags_12 },
    { 0xaabf1d6, SUC_lexicon_tags_55 },
    { 0xc1ebf1d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85abf1de, SUC_lexicon_tags_16 },
    { 0xc56bf1df, SUC_lexicon_tags_299 },
    { 0x8d0bf1e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0bf1eb, SUC_lexicon_tags_60 },
    { 0x894bf1ec, SUC_lexicon_tags_6 },
    { 0x520bf1ed, SUC_lexicon_tags_12 },
    { 0xbbebf1ed, SUC_lexicon_tags_7 },
    { 0x560bf1ef, SUC_lexicon_tags_39 },
    { 0xadabf1f0, SUC_lexicon_tags_1 },
    { 0x4d6bf1eb, SUC_lexicon_tags_11 },
    { 0x58cbf1f2, SUC_lexicon_tags_2 },
    { 0xc2abf1f2, SUC_lexicon_tags_71 },
    { 0x8cbf1ec, SUC_lexicon_tags_2 },
    { 0x75abf1f5, SUC_lexicon_tags_11 },
    { 0xf36bf1f6, SUC_lexicon_tags_32 },
    { 0xab4bf1f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x382bf1f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972bf201, SUC_lexicon_tags_138 },
    { 0xb18bf201, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x182bf205, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x574bf207, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302bf20d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5abf210, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6bf215, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94bf218, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebf220, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acbf228, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x508bf22b, SUC_lexicon_tags_33 },
    { 0x560bf22c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cbf231, SUC_lexicon_tags_60 },
    { 0x188bf232, SUC_lexicon_tags_49 },
    { 0x580bf233, SUC_lexicon_tags_49 },
    { 0x8a2bf234, SUC_lexicon_tags_25 },
    { 0xd6bf232, SUC_lexicon_tags_17 },
    { 0x556bf236, SUC_lexicon_tags_47 },
    { 0x384bf237, SUC_lexicon_tags_37 },
    { 0x834bf232, SUC_lexicon_tags_25 },
    { 0xbeabf239, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9e2bf23b, SUC_lexicon_tags_1471 },
    { 0xd1abf23b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86abf243, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa14bf245, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x362bf248, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2bf24b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ebf24f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42bf256, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48abf259, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3ebf25b, SUC_lexicon_tags_2 },
    { 0x276bf25c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8bf25f, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4abf262, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72bf266, SUC_lexicon_tags_4 },
    { 0x828bf267, SUC_lexicon_tags_0 },
    { 0x59abf267, SUC_lexicon_tags_27 },
    { 0x152bf268, SUC_lexicon_tags_11 },
    { 0x33ebf26a, SUC_lexicon_tags_2 },
    { 0x994bf26b, SUC_lexicon_tags_17 },
    { 0x6b0bf26a, SUC_lexicon_tags_45 },
    { 0xadebf269, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcd6bf26f, SUC_lexicon_tags_25 },
    { 0xc6abf270, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2bebf272, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bf274, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa24bf276, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08bf27a, SUC_lexicon_tags_26 },
    { 0xb44bf27b, SUC_lexicon_tags_49 },
    { 0x9e4bf27c, SUC_lexicon_tags_188 },
    { 0xc00bf27c, SUC_lexicon_tags_12 },
    { 0xa0bf27e, SUC_lexicon_tags_62 },
    { 0x586bf27e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb0bf281, SUC_lexicon_tags_37 },
    { 0xaf2bf282, SUC_lexicon_tags_6 },
    { 0x3dcbf283, SUC_lexicon_tags_35 },
    { 0x5d2bf284, SUC_lexicon_tags_19 },
    { 0x456bf285, SUC_lexicon_tags_35 },
    { 0xe3ebf284, SUC_lexicon_tags_11 },
    { 0xb7cbf287, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ebf28a, SUC_lexicon_tags_17 },
    { 0xd86bf28a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34bf290, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4abf294, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf298, SUC_lexicon_tags_27 },
    { 0x8d6bf299, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8bf29d, SUC_lexicon_tags_11 },
    { 0xf7cbf29d, SUC_lexicon_tags_12 },
    { 0x19cbf29f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ebf2a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2bf2a9, SUC_lexicon_tags_27 },
    { 0x286bf2aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560bf2ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4bf2b5, SUC_lexicon_tags_25 },
    { 0x854bf2b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9febf2ba, SUC_lexicon_tags_25 },
    { 0x48cbf2bb, SUC_lexicon_tags_63 },
    { 0xf66bf2bc, SUC_lexicon_tags_38 },
    { 0xb4ebf2bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x436bf2bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe10bf2c1, SUC_lexicon_tags_25 },
    { 0xabebf2c1, SUC_lexicon_tags_33 },
    { 0x3debf2c3, SUC_lexicon_tags_37 },
    { 0x36bf2c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93abf2c6, SUC_lexicon_tags_30 },
    { 0xdccbf2c6, SUC_lexicon_tags_29 },
    { 0x966bf2c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda4bf2ca, SUC_lexicon_tags_1 },
    { 0x16bf2ca, SUC_lexicon_tags_21 },
    { 0xf46bf2cc, SUC_lexicon_tags_47 },
    { 0xf88bf2cd, SUC_lexicon_tags_30 },
    { 0x25cbf2cc, SUC_lexicon_tags_14 },
    { 0x51cbf2cf, SUC_lexicon_tags_33 },
    { 0xa4cbf2cf, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x62abf2d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b4bf2d5, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x2b6bf2d7, SUC_lexicon_tags_1 },
    { 0xdd0bf2d8, SUC_lexicon_tags_2 },
    { 0x610bf2d8, SUC_lexicon_tags_33 },
    { 0x308bf2d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ebf2e5, SUC_lexicon_tags_0 },
    { 0x676bf2e5, SUC_lexicon_tags_39 },
    { 0x9ecbf2e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ccbf2e9, SUC_lexicon_tags_0 },
    { 0xd54bf2ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cbf2fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20bf2ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cbf303, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32bf308, SUC_lexicon_tags_45 },
    { 0xba8bf308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4bf30d, SUC_lexicon_tags_25 },
    { 0xa22bf30d, SUC_lexicon_tags_25 },
    { 0xe22bf30f, SUC_lexicon_tags_37 },
    { 0x706bf310, SUC_lexicon_tags_15 },
    { 0x40bf311, SUC_lexicon_tags_30 },
    { 0x116bf310, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecbf316, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ebf31d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e8bf31f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x202bf321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x732bf324, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2bf329, SUC_lexicon_tags_15 },
    { 0xc38bf329, SUC_lexicon_tags_43 },
    { 0x48ebf32b, SUC_lexicon_tags_68 },
    { 0xeebf32c, SUC_lexicon_tags_63 },
    { 0x662bf32d, SUC_lexicon_tags_47 },
    { 0x644bf32e, SUC_lexicon_tags_11 },
    { 0xf24bf32e, SUC_lexicon_tags_29 },
    { 0x406bf32e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654bf335, SUC_lexicon_tags_74 },
    { 0x66abf335, SUC_lexicon_tags_2 },
    { 0xf26bf337, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6bf33b, SUC_lexicon_tags_9 },
    { 0x76abf33c, SUC_lexicon_tags_48 },
    { 0x54abf33d, SUC_lexicon_tags_87 },
    { 0x50cbf33e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3cabf340, SUC_lexicon_tags_17 },
    { 0x60abf341, SUC_lexicon_tags_47 },
    { 0xd92bf342, SUC_lexicon_tags_38 },
    { 0x9cebf343, SUC_lexicon_tags_37 },
    { 0xc9ebf344, SUC_lexicon_tags_39 },
    { 0xdf4bf344, SUC_lexicon_tags_136 },
    { 0xbf0bf344, SUC_lexicon_tags_11 },
    { 0xd46bf346, SUC_lexicon_tags_63 },
    { 0x8a6bf347, SUC_lexicon_tags_39 },
    { 0x8a6bf349, SUC_lexicon_tags_11 },
    { 0x77abf34a, SUC_lexicon_tags_60 },
    { 0xbfebf344, SUC_lexicon_tags_9 },
    { 0x3abf34c, SUC_lexicon_tags_31 },
    { 0x5babf348, SUC_lexicon_tags_0 },
    { 0x128bf340, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ebf354, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704bf35f, SUC_lexicon_tags_27 },
    { 0x2d6bf35f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4bebf362, SUC_lexicon_tags_37 },
    { 0x57ebf362, SUC_lexicon_tags_25 },
    { 0xe3ebf364, SUC_lexicon_tags_37 },
    { 0x9fabf365, SUC_lexicon_tags_11 },
    { 0x554bf364, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222bf36a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6abf370, SUC_lexicon_tags_37 },
    { 0x32ebf370, SUC_lexicon_tags_11 },
    { 0xe9abf372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32bf375, SUC_lexicon_tags_38 },
    { 0x53cbf375, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2bf37a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xca2bf37c, SUC_lexicon_tags_12 },
    { 0xeeabf37d, SUC_lexicon_tags_12 },
    { 0xe66bf37e, SUC_lexicon_tags_9 },
    { 0x7e4bf37e, SUC_lexicon_tags_95 },
    { 0xd1ebf380, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ebf384, SUC_lexicon_tags_2 },
    { 0xd04bf384, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbabf389, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00bf38c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb98bf38e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4bf391, SUC_lexicon_tags_7 },
    { 0xadebf391, SUC_lexicon_tags_0 },
    { 0xf6bf391, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794bf399, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bf39c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350bf3a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6abf3a6, SUC_lexicon_tags_17 },
    { 0x150bf3a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x678bf3a9, SUC_lexicon_tags_27 },
    { 0xdcbf3aa, SUC_lexicon_tags_11 },
    { 0x59cbf3ab, SUC_lexicon_tags_2 },
    { 0x470bf3ab, SUC_lexicon_tags_2 },
    { 0xd40bf3ab, SUC_lexicon_tags_25 },
    { 0x73ebf3ad, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8f6bf3b0, SUC_lexicon_tags_47 },
    { 0x878bf3b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cabf3be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacebf3c0, SUC_lexicon_tags_37 },
    { 0x3b2bf3c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x95abf3c3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x358bf3c6, SUC_lexicon_tags_57 },
    { 0x1ccbf3c7, SUC_lexicon_tags_15 },
    { 0xa84bf3c8, SUC_lexicon_tags_37 },
    { 0xcbabf3c8, SUC_lexicon_tags_57 },
    { 0x28bf3ca, SUC_lexicon_tags_7 },
    { 0xe4abf3cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7abf3cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c0bf3cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ebf3d5, SUC_lexicon_tags_29 },
    { 0x61cbf3d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ebf3e0, SUC_lexicon_tags_13 },
    { 0xd3ebf3e1, SUC_lexicon_tags_55 },
    { 0x290bf3e2, SUC_lexicon_tags_43 },
    { 0x3a6bf3e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4bf3ed, SUC_lexicon_tags_7 },
    { 0xe72bf3ed, SUC_lexicon_tags_35 },
    { 0x6c2bf3ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x978bf3f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a8bf3f4, SUC_lexicon_tags_11 },
    { 0x550bf3f5, SUC_lexicon_tags_12 },
    { 0xb36bf3f5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecbf3fc, SUC_lexicon_tags_1 },
    { 0xe0bf3fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cbf404, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2bf407, SUC_lexicon_tags_14 },
    { 0xea6bf408, SUC_lexicon_tags_29 },
    { 0xe56bf409, SUC_lexicon_tags_2 },
    { 0xabebf40a, SUC_lexicon_tags_12 },
    { 0xea6bf40b, SUC_lexicon_tags_17 },
    { 0xbc2bf40a, SUC_lexicon_tags_27 },
    { 0x75cbf40d, SUC_lexicon_tags_15 },
    { 0x730bf40e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x426bf410, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccbf413, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacbf417, SUC_lexicon_tags_11 },
    { 0x2d4bf417, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaabf41b, SUC_lexicon_tags_45 },
    { 0x6b8bf41c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x202bf41e, SUC_lexicon_tags_77 },
    { 0xdccbf41f, SUC_lexicon_tags_7 },
    { 0x3debf41f, SUC_lexicon_tags_37 },
    { 0xa80bf421, SUC_lexicon_tags_46 },
    { 0x88abf422, SUC_lexicon_tags_43 },
    { 0xd06bf423, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fabf426, SUC_lexicon_tags_0 },
    { 0xcfcbf426, SUC_lexicon_tags_25 },
    { 0x32abf428, SUC_lexicon_tags_175 },
    { 0, NULL },
    { 0x2b8bf42a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30bf42e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2bf432, SUC_lexicon_tags_23 },
    { 0xe1ebf432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0bf436, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118bf43a, SUC_lexicon_tags_7 },
    { 0x7c6bf43b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50bf43d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2abf43f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138bf444, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x42ebf446, SUC_lexicon_tags_8 },
    { 0x45ebf447, SUC_lexicon_tags_1190 },
    { 0, NULL },
    { 0x3a4bf449, SUC_lexicon_tags_7 },
    { 0x4e6bf449, SUC_lexicon_tags_85 },
    { 0xab4bf44b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x422bf44d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4c4bf44f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74cbf452, SUC_lexicon_tags_2 },
    { 0xa06bf452, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ecbf455, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9ebf457, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbe8bf459, SUC_lexicon_tags_28 },
    { 0xd00bf45a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x604bf45c, SUC_lexicon_tags_47 },
    { 0xdaebf45d, SUC_lexicon_tags_27 },
    { 0x442bf45e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25abf468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510bf46e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278bf476, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x40ebf478, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x560bf47b, SUC_lexicon_tags_25 },
    { 0xb90bf47c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdebf480, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe56bf482, SUC_lexicon_tags_49 },
    { 0x86bf482, SUC_lexicon_tags_15 },
    { 0x49cbf483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eabf48e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28bf492, SUC_lexicon_tags_92 },
    { 0x382bf492, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8bcbf495, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x940bf497, SUC_lexicon_tags_37 },
    { 0xac2bf498, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ebf49d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cbf4a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0bf4a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd34bf4a8, SUC_lexicon_tags_8 },
    { 0xeb8bf4a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0bf4ad, SUC_lexicon_tags_37 },
    { 0x7f6bf4ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74bf4b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0bf4b5, SUC_lexicon_tags_37 },
    { 0x278bf4b5, SUC_lexicon_tags_49 },
    { 0x3eebf4b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ebf4bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4bf4c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x122bf4c8, SUC_lexicon_tags_47 },
    { 0xde2bf4c9, SUC_lexicon_tags_47 },
    { 0x7febf4ca, SUC_lexicon_tags_8 },
    { 0x3f6bf4cb, SUC_lexicon_tags_8 },
    { 0x23abf4c8, SUC_lexicon_tags_12 },
    { 0x15cbf4cb, SUC_lexicon_tags_22 },
    { 0x892bf4ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba0bf4d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6bf4d3, SUC_lexicon_tags_14 },
    { 0xe5cbf4d3, SUC_lexicon_tags_7 },
    { 0xf58bf4d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360bf4dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7abf4df, SUC_lexicon_tags_37 },
    { 0x96abf4e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eabf4e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdcebf4e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7eabf4e8, SUC_lexicon_tags_15 },
    { 0x472bf4e9, SUC_lexicon_tags_15 },
    { 0x54abf4ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90bf4f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36cbf4f9, SUC_lexicon_tags_9 },
    { 0x358bf4fa, SUC_lexicon_tags_28 },
    { 0x3f0bf4fb, SUC_lexicon_tags_17 },
    { 0x26bf4fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534bf505, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434bf50d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4bf515, SUC_lexicon_tags_7 },
    { 0x5aebf515, SUC_lexicon_tags_0 },
    { 0x7c4bf517, SUC_lexicon_tags_7 },
    { 0xf9cbf518, SUC_lexicon_tags_30 },
    { 0xf2ebf519, SUC_lexicon_tags_37 },
    { 0x450bf51a, SUC_lexicon_tags_60 },
    { 0x32abf51a, SUC_lexicon_tags_20 },
    { 0xfd2bf51a, SUC_lexicon_tags_23 },
    { 0x996bf51d, SUC_lexicon_tags_15 },
    { 0x210bf51e, SUC_lexicon_tags_47 },
    { 0x672bf515, SUC_lexicon_tags_11 },
    { 0x826bf520, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8bf527, SUC_lexicon_tags_12 },
    { 0x6ebf527, SUC_lexicon_tags_0 },
    { 0xbf0bf527, SUC_lexicon_tags_29 },
    { 0xddebf52a, SUC_lexicon_tags_10 },
    { 0xca0bf52a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd6bf52d, SUC_lexicon_tags_25 },
    { 0x62bf52e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acbf531, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa54bf533, SUC_lexicon_tags_65 },
    { 0xe2bf533, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ebf53a, SUC_lexicon_tags_23 },
    { 0x9b6bf53b, SUC_lexicon_tags_6 },
    { 0xa7ebf53c, SUC_lexicon_tags_35 },
    { 0x908bf53d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf80bf53f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaeabf541, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x344bf543, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2bf548, SUC_lexicon_tags_2 },
    { 0x5c8bf549, SUC_lexicon_tags_54 },
    { 0x630bf548, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x822bf54d, SUC_lexicon_tags_2 },
    { 0xa4cbf54e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccbf553, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27abf55a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cbf561, SUC_lexicon_tags_43 },
    { 0x846bf562, SUC_lexicon_tags_12 },
    { 0xe7ebf562, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccbf566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x538bf568, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a6bf56a, SUC_lexicon_tags_23 },
    { 0xc28bf56b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4bf56e, SUC_lexicon_tags_27 },
    { 0xc7cbf56f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabf571, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0bf574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542bf57b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfebf57e, SUC_lexicon_tags_37 },
    { 0xddcbf57f, SUC_lexicon_tags_37 },
    { 0xecbf580, SUC_lexicon_tags_11 },
    { 0xe3ebf581, SUC_lexicon_tags_1107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426bf586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccbf589, SUC_lexicon_tags_65 },
    { 0xde8bf58a, SUC_lexicon_tags_26 },
    { 0xc1abf58b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54ebf58d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9babf590, SUC_lexicon_tags_21 },
    { 0x2ecbf591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94bf595, SUC_lexicon_tags_11 },
    { 0x77abf596, SUC_lexicon_tags_12 },
    { 0x10ebf597, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fabf599, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2abf59c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ebf59f, SUC_lexicon_tags_17 },
    { 0xca6bf5a0, SUC_lexicon_tags_7 },
    { 0xb54bf59f, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x50abf5a3, SUC_lexicon_tags_8 },
    { 0xcf4bf5a3, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18abf5a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2bf5ac, SUC_lexicon_tags_37 },
    { 0xa48bf5ad, SUC_lexicon_tags_25 },
    { 0x58ebf5ae, SUC_lexicon_tags_53 },
    { 0x332bf5af, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x35ebf5b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656bf5b7, SUC_lexicon_tags_31 },
    { 0x648bf5b8, SUC_lexicon_tags_12 },
    { 0xb80bf5b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6bf5be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x822bf5c0, SUC_lexicon_tags_30 },
    { 0x9cebf5c0, SUC_lexicon_tags_49 },
    { 0x7c0bf5c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6bf5ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54bf5cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29abf5cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb40bf5d1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abf5d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x76cbf5da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ebf5dd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x53ebf5df, SUC_lexicon_tags_12 },
    { 0x736bf5df, SUC_lexicon_tags_14 },
    { 0xeccbf5df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ebf5e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x82bf5e8, SUC_lexicon_tags_30 },
    { 0x91ebf5e8, SUC_lexicon_tags_15 },
    { 0x642bf5e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc34bf5ec, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x206bf5ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4abf5f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ebf5f5, SUC_lexicon_tags_87 },
    { 0x726bf5f5, SUC_lexicon_tags_17 },
    { 0x36ebf5f6, SUC_lexicon_tags_65 },
    { 0x532bf5f8, SUC_lexicon_tags_1 },
    { 0x5c0bf5f9, SUC_lexicon_tags_11 },
    { 0xa90bf5f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb7ebf5fc, SUC_lexicon_tags_49 },
    { 0x58abf5fd, SUC_lexicon_tags_25 },
    { 0xec8bf5fe, SUC_lexicon_tags_47 },
    { 0xe26bf5ff, SUC_lexicon_tags_47 },
    { 0x3bcbf5fd, SUC_lexicon_tags_35 },
    { 0x640bf601, SUC_lexicon_tags_46 },
    { 0xa72bf601, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4bf606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfebf60a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x574bf60c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdabf60e, SUC_lexicon_tags_11 },
    { 0x1f2bf60e, SUC_lexicon_tags_45 },
    { 0xc00bf610, SUC_lexicon_tags_49 },
    { 0xa48bf60e, SUC_lexicon_tags_37 },
    { 0xd84bf612, SUC_lexicon_tags_0 },
    { 0x1babf613, SUC_lexicon_tags_15 },
    { 0xebcbf613, SUC_lexicon_tags_6 },
    { 0xdd6bf615, SUC_lexicon_tags_25 },
    { 0x23cbf614, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0cbf618, SUC_lexicon_tags_0 },
    { 0x5e8bf619, SUC_lexicon_tags_11 },
    { 0xed4bf618, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb84bf61c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe48bf61e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5abf622, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12bf625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccbf629, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bf632, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb12bf634, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ebf638, SUC_lexicon_tags_15 },
    { 0x272bf638, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806bf640, SUC_lexicon_tags_140 },
    { 0x222bf641, SUC_lexicon_tags_25 },
    { 0x7abf642, SUC_lexicon_tags_15 },
    { 0x666bf642, SUC_lexicon_tags_48 },
    { 0xff8bf644, SUC_lexicon_tags_10 },
    { 0x9aebf645, SUC_lexicon_tags_45 },
    { 0xc06bf646, SUC_lexicon_tags_14 },
    { 0x500bf645, SUC_lexicon_tags_11 },
    { 0xa74bf646, SUC_lexicon_tags_2 },
    { 0x79ebf649, SUC_lexicon_tags_17 },
    { 0x496bf64a, SUC_lexicon_tags_45 },
    { 0xdc0bf64b, SUC_lexicon_tags_2 },
    { 0x7bebf649, SUC_lexicon_tags_31 },
    { 0x6e8bf648, SUC_lexicon_tags_12 },
    { 0x342bf641, SUC_lexicon_tags_15 },
    { 0x1aabf64f, SUC_lexicon_tags_60 },
    { 0x726bf64a, SUC_lexicon_tags_12 },
    { 0xb06bf649, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x554bf653, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x478bf656, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34bf659, SUC_lexicon_tags_34 },
    { 0x698bf65a, SUC_lexicon_tags_37 },
    { 0x8eebf65b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x16ebf65d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336bf661, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f4bf663, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb6bf665, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xec8bf667, SUC_lexicon_tags_33 },
    { 0xc7ebf668, SUC_lexicon_tags_3 },
    { 0xbfcbf668, SUC_lexicon_tags_37 },
    { 0x97abf66a, SUC_lexicon_tags_21 },
    { 0xa78bf66a, SUC_lexicon_tags_47 },
    { 0x6b6bf66c, SUC_lexicon_tags_55 },
    { 0xdb0bf66d, SUC_lexicon_tags_25 },
    { 0xa06bf66e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x832bf670, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74bf674, SUC_lexicon_tags_11 },
    { 0xcd0bf675, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf14bf677, SUC_lexicon_tags_28 },
    { 0xf64bf678, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36bf67c, SUC_lexicon_tags_22 },
    { 0x27ebf67c, SUC_lexicon_tags_25 },
    { 0x70bf67d, SUC_lexicon_tags_37 },
    { 0xe2ebf67f, SUC_lexicon_tags_11 },
    { 0x2b6bf680, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606bf685, SUC_lexicon_tags_17 },
    { 0xd1cbf686, SUC_lexicon_tags_37 },
    { 0x6a8bf686, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4bf68d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeebf691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86bf693, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x380bf696, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30bf69a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cbf6a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecbf6a8, SUC_lexicon_tags_7 },
    { 0xfeabf6a8, SUC_lexicon_tags_32 },
    { 0x5eabf6a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdebf6af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ebf6b2, SUC_lexicon_tags_23 },
    { 0x496bf6b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcb4bf6b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9abf6b7, SUC_lexicon_tags_1 },
    { 0xa2cbf6b7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2bf6bc, SUC_lexicon_tags_10 },
    { 0xacebf6bc, SUC_lexicon_tags_54 },
    { 0x402bf6bc, SUC_lexicon_tags_25 },
    { 0x456bf6bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bcbf6c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x860bf6c4, SUC_lexicon_tags_2 },
    { 0x48ebf6c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x168bf6c7, SUC_lexicon_tags_12 },
    { 0x7c0bf6c8, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x6babf6cb, SUC_lexicon_tags_38 },
    { 0x954bf6cc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x522bf6cf, SUC_lexicon_tags_47 },
    { 0x810bf6d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x314bf6d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcabf6dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a6bf6de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98bf6e1, SUC_lexicon_tags_7 },
    { 0xf00bf6e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbdebf6e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8bf6e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20bf6ea, SUC_lexicon_tags_30 },
    { 0x46bf6eb, SUC_lexicon_tags_17 },
    { 0xe7cbf6ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72bf6f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcbf6fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x956bf6fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52bf700, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8bf703, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cbf706, SUC_lexicon_tags_17 },
    { 0x43abf707, SUC_lexicon_tags_15 },
    { 0xd62bf706, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ebf70d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaebf710, SUC_lexicon_tags_7 },
    { 0xafabf711, SUC_lexicon_tags_32 },
    { 0x12abf711, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4bf718, SUC_lexicon_tags_15 },
    { 0xa48bf719, SUC_lexicon_tags_37 },
    { 0x5eabf719, SUC_lexicon_tags_33 },
    { 0x876bf71b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4bf723, SUC_lexicon_tags_118 },
    { 0x5bcbf724, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcebf727, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ebf72b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd2bf72d, SUC_lexicon_tags_23 },
    { 0xff4bf72d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60bf734, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546bf738, SUC_lexicon_tags_17 },
    { 0x33ebf738, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeebf73f, SUC_lexicon_tags_15 },
    { 0xa24bf73f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0bf746, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccbf74d, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26bf753, SUC_lexicon_tags_25 },
    { 0xeecbf754, SUC_lexicon_tags_25 },
    { 0x4c8bf755, SUC_lexicon_tags_47 },
    { 0xe0bf755, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552bf75f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4bf761, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cbf76a, SUC_lexicon_tags_2 },
    { 0xcaebf76b, SUC_lexicon_tags_14 },
    { 0x84abf76c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602bf774, SUC_lexicon_tags_42 },
    { 0xf7ebf774, SUC_lexicon_tags_7 },
    { 0x930bf776, SUC_lexicon_tags_37 },
    { 0xdf8bf776, SUC_lexicon_tags_25 },
    { 0x108bf775, SUC_lexicon_tags_61 },
    { 0xe96bf778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe4bf77b, SUC_lexicon_tags_17 },
    { 0xf74bf77c, SUC_lexicon_tags_11 },
    { 0xd8cbf77d, SUC_lexicon_tags_7 },
    { 0x6b4bf77e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cbf781, SUC_lexicon_tags_47 },
    { 0xe7abf782, SUC_lexicon_tags_7 },
    { 0x756bf783, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2bf787, SUC_lexicon_tags_335 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44bf790, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72bf794, SUC_lexicon_tags_34 },
    { 0x612bf795, SUC_lexicon_tags_37 },
    { 0x5c8bf795, SUC_lexicon_tags_7 },
    { 0x664bf797, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ebf79c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ebf79f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb04bf7a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836bf7a5, SUC_lexicon_tags_87 },
    { 0xcecbf7a6, SUC_lexicon_tags_2 },
    { 0x77abf7a7, SUC_lexicon_tags_26 },
    { 0x4f0bf7a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4bf7ab, SUC_lexicon_tags_17 },
    { 0x6d2bf7ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ebf7af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd60bf7b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x840bf7b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94bf7b7, SUC_lexicon_tags_23 },
    { 0xe6cbf7b8, SUC_lexicon_tags_7 },
    { 0x3aabf7b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x890bf7bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cbf7bf, SUC_lexicon_tags_2 },
    { 0x7d0bf7bf, SUC_lexicon_tags_37 },
    { 0x786bf7bf, SUC_lexicon_tags_33 },
    { 0x3aebf7bf, SUC_lexicon_tags_25 },
    { 0xd9cbf7c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0bf7c8, SUC_lexicon_tags_46 },
    { 0x46abf7c8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6bf7cb, SUC_lexicon_tags_55 },
    { 0x56abf7cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34bf7d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476bf7d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbf7de, SUC_lexicon_tags_12 },
    { 0x25cbf7de, SUC_lexicon_tags_82 },
    { 0x93abf7de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cbf7e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2dabf7e8, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xf0abf7ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548bf7ef, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdc2bf7f1, SUC_lexicon_tags_11 },
    { 0x4d6bf7f2, SUC_lexicon_tags_12 },
    { 0x744bf7f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1babf7f6, SUC_lexicon_tags_63 },
    { 0xf94bf7f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc7abf7f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9ebf7fb, SUC_lexicon_tags_17 },
    { 0xeacbf7fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8cabf7fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28abf800, SUC_lexicon_tags_14 },
    { 0xbe2bf801, SUC_lexicon_tags_17 },
    { 0xeccbf800, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eabf805, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x588bf807, SUC_lexicon_tags_35 },
    { 0xb8ebf808, SUC_lexicon_tags_27 },
    { 0xeb4bf807, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x950bf80c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbabf810, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaebf814, SUC_lexicon_tags_44 },
    { 0xb00bf815, SUC_lexicon_tags_11 },
    { 0x778bf814, SUC_lexicon_tags_17 },
    { 0x7d4bf817, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdebf81f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0bf822, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ebf827, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96bf82b, SUC_lexicon_tags_23 },
    { 0xc6abf82c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf6bf82e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ccbf830, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116bf835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8abf83e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5babf842, SUC_lexicon_tags_47 },
    { 0xd72bf843, SUC_lexicon_tags_7 },
    { 0x2fcbf844, SUC_lexicon_tags_4 },
    { 0x5ecbf843, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x874bf847, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x770bf849, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bebf84b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28bf850, SUC_lexicon_tags_25 },
    { 0x7a2bf850, SUC_lexicon_tags_25 },
    { 0xcdcbf852, SUC_lexicon_tags_37 },
    { 0x838bf852, SUC_lexicon_tags_38 },
    { 0x9dcbf850, SUC_lexicon_tags_2 },
    { 0xf42bf853, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160bf85a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a8bf85c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34bf85f, SUC_lexicon_tags_62 },
    { 0xa94bf860, SUC_lexicon_tags_37 },
    { 0x3d8bf861, SUC_lexicon_tags_7 },
    { 0x2f0bf862, SUC_lexicon_tags_60 },
    { 0x64bf863, SUC_lexicon_tags_38 },
    { 0xac2bf864, SUC_lexicon_tags_19 },
    { 0xcb8bf864, SUC_lexicon_tags_19 },
    { 0x2c2bf866, SUC_lexicon_tags_29 },
    { 0x882bf866, SUC_lexicon_tags_11 },
    { 0x984bf866, SUC_lexicon_tags_63 },
    { 0xdeebf869, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ccbf86b, SUC_lexicon_tags_37 },
    { 0xf7abf86c, SUC_lexicon_tags_51 },
    { 0xba8bf86d, SUC_lexicon_tags_49 },
    { 0x4b0bf86e, SUC_lexicon_tags_27 },
    { 0xf80bf86e, SUC_lexicon_tags_45 },
    { 0xcd0bf870, SUC_lexicon_tags_46 },
    { 0xc96bf871, SUC_lexicon_tags_15 },
    { 0xe6abf871, SUC_lexicon_tags_15 },
    { 0x288bf86f, SUC_lexicon_tags_2 },
    { 0xc72bf874, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x258bf877, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4bf87b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3babf87f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0abf881, SUC_lexicon_tags_17 },
    { 0xb78bf882, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x558bf885, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6bf889, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3ccbf88b, SUC_lexicon_tags_8 },
    { 0xb8ebf88b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccbf891, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8bf896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50bf89a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408bf89e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cbf8a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf82bf8a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8bf8a7, SUC_lexicon_tags_15 },
    { 0xeb0bf8a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ebf8ab, SUC_lexicon_tags_23 },
    { 0xea0bf8ac, SUC_lexicon_tags_10 },
    { 0x918bf8ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x342bf8af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cbf8b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccbf8b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x104bf8b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcbf8bf, SUC_lexicon_tags_25 },
    { 0x778bf8c0, SUC_lexicon_tags_9 },
    { 0x8f2bf8bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19cbf8c3, SUC_lexicon_tags_37 },
    { 0x598bf8c4, SUC_lexicon_tags_12 },
    { 0x47abf8c5, SUC_lexicon_tags_37 },
    { 0x31ebf8c5, SUC_lexicon_tags_201 },
    { 0x4ccbf8c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x712bf8c9, SUC_lexicon_tags_56 },
    { 0x46bf8ca, SUC_lexicon_tags_1 },
    { 0x2ccbf8cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0bf8d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6bf8d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x306bf8d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf26bf8d9, SUC_lexicon_tags_166 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2bf8e0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf14bf8e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228bf8e7, SUC_lexicon_tags_165 },
    { 0xd94bf8e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x944bf8ea, SUC_lexicon_tags_87 },
    { 0xce8bf8eb, SUC_lexicon_tags_12 },
    { 0x908bf8ec, SUC_lexicon_tags_10 },
    { 0x8bf8eb, SUC_lexicon_tags_1 },
    { 0x872bf8eb, SUC_lexicon_tags_43 },
    { 0x7f8bf8ef, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x558bf8f2, SUC_lexicon_tags_37 },
    { 0x1fabf8f3, SUC_lexicon_tags_57 },
    { 0x616bf8f4, SUC_lexicon_tags_0 },
    { 0x106bf8f5, SUC_lexicon_tags_17 },
    { 0xe6bf8f6, SUC_lexicon_tags_12 },
    { 0x812bf8f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeabf8fd, SUC_lexicon_tags_43 },
    { 0x370bf8fd, SUC_lexicon_tags_0 },
    { 0xfdcbf8ff, SUC_lexicon_tags_68 },
    { 0xbdebf900, SUC_lexicon_tags_255 },
    { 0x6aebf901, SUC_lexicon_tags_29 },
    { 0x208bf900, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2bf906, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ebf909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd4bf90b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1abf90d, SUC_lexicon_tags_34 },
    { 0x556bf90e, SUC_lexicon_tags_29 },
    { 0x3acbf90f, SUC_lexicon_tags_17 },
    { 0x588bf90f, SUC_lexicon_tags_33 },
    { 0x416bf90f, SUC_lexicon_tags_63 },
    { 0xc4abf912, SUC_lexicon_tags_2 },
    { 0xe32bf913, SUC_lexicon_tags_15 },
    { 0xebebf90f, SUC_lexicon_tags_25 },
    { 0x508bf915, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa86bf917, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4b4bf919, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f2bf91b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad4bf91d, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34bf922, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf2abf924, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cbf927, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x988bf929, SUC_lexicon_tags_118 },
    { 0xe58bf929, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcbf933, SUC_lexicon_tags_2 },
    { 0xf08bf934, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34bf937, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6bf93c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x858bf93e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8abf941, SUC_lexicon_tags_60 },
    { 0x66cbf942, SUC_lexicon_tags_421 },
    { 0, NULL },
    { 0xecebf944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb08bf946, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54bf948, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x382bf94a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bf94d, SUC_lexicon_tags_19 },
    { 0x718bf94e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea4bf950, SUC_lexicon_tags_43 },
    { 0x26bf951, SUC_lexicon_tags_60 },
    { 0xd16bf952, SUC_lexicon_tags_42 },
    { 0x148bf953, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xea6bf955, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2cbf957, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x88bf959, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95abf95c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6cbf95e, SUC_lexicon_tags_15 },
    { 0x6d8bf95f, SUC_lexicon_tags_11 },
    { 0xd0bf960, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9babf965, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20abf96b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366bf972, SUC_lexicon_tags_45 },
    { 0xffabf973, SUC_lexicon_tags_43 },
    { 0x602bf974, SUC_lexicon_tags_23 },
    { 0x16cbf975, SUC_lexicon_tags_17 },
    { 0x580bf974, SUC_lexicon_tags_38 },
    { 0x974bf977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66bf97e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0bf981, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x878bf983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x592bf986, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dabf98b, SUC_lexicon_tags_4 },
    { 0x274bf98c, SUC_lexicon_tags_49 },
    { 0x99abf98d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95cbf98f, SUC_lexicon_tags_60 },
    { 0x778bf990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314bf995, SUC_lexicon_tags_7 },
    { 0x8ccbf996, SUC_lexicon_tags_47 },
    { 0xffcbf997, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2abf99a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd62bf99c, SUC_lexicon_tags_37 },
    { 0x2f6bf99d, SUC_lexicon_tags_12 },
    { 0x47ebf99e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ebf9a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0bf9a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ebf9a9, SUC_lexicon_tags_19 },
    { 0xb9cbf9a9, SUC_lexicon_tags_27 },
    { 0xfdcbf9ab, SUC_lexicon_tags_15 },
    { 0x54abf9aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54bf9af, SUC_lexicon_tags_29 },
    { 0xf5ebf9af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0bf9b4, SUC_lexicon_tags_1 },
    { 0x94abf9b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d4bf9b7, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2bf9ba, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4babf9bd, SUC_lexicon_tags_39 },
    { 0x8c4bf9be, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe4bf9c0, SUC_lexicon_tags_37 },
    { 0x31ebf9c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bf9c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cbf9cb, SUC_lexicon_tags_19 },
    { 0x752bf9cb, SUC_lexicon_tags_17 },
    { 0x618bf9cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78bf9d0, SUC_lexicon_tags_17 },
    { 0x4febf9d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378bf9d6, SUC_lexicon_tags_37 },
    { 0x63cbf9d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x224bf9d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x600bf9db, SUC_lexicon_tags_12 },
    { 0x4b8bf9dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa0bf9de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336bf9e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8bf9ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cbf9f0, SUC_lexicon_tags_8 },
    { 0x68abf9f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd76bf9f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d2bf9f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a6bf9f7, SUC_lexicon_tags_12 },
    { 0x692bf9f8, SUC_lexicon_tags_0 },
    { 0x348bf9f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28bf9fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2abf9fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x474bfa01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6bfa05, SUC_lexicon_tags_1 },
    { 0xf50bfa05, SUC_lexicon_tags_12 },
    { 0xd5abfa07, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9e0bfa09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acbfa0e, SUC_lexicon_tags_25 },
    { 0x5b8bfa0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eabfa16, SUC_lexicon_tags_23 },
    { 0x5b8bfa17, SUC_lexicon_tags_9 },
    { 0x65cbfa17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x838bfa1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9e2bfa1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe92bfa1e, SUC_lexicon_tags_13 },
    { 0xe62bfa1e, SUC_lexicon_tags_28 },
    { 0xed0bfa20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cebfa23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xecabfa25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e2bfa27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ebfa2c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0bfa30, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ebfa33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bebfa39, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x824bfa3b, SUC_lexicon_tags_17 },
    { 0xaacbfa3c, SUC_lexicon_tags_3 },
    { 0x73ebfa3d, SUC_lexicon_tags_37 },
    { 0x232bfa3e, SUC_lexicon_tags_87 },
    { 0xb70bfa3f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x898bfa41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6bfa45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xedabfa47, SUC_lexicon_tags_66 },
    { 0x308bfa47, SUC_lexicon_tags_87 },
    { 0x1a8bfa48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x800bfa4c, SUC_lexicon_tags_7 },
    { 0x3d6bfa4c, SUC_lexicon_tags_63 },
    { 0x748bfa4d, SUC_lexicon_tags_928 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08bfa54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55abfa59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe10bfa5b, SUC_lexicon_tags_0 },
    { 0x768bfa5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d6bfa5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bebfa61, SUC_lexicon_tags_11 },
    { 0x778bfa62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6bfa66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720bfa6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67abfa74, SUC_lexicon_tags_9 },
    { 0x166bfa74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2bfa78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x836bfa7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ebfa7c, SUC_lexicon_tags_14 },
    { 0xa22bfa7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4bfa83, SUC_lexicon_tags_32 },
    { 0x51ebfa84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90abfa88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76bfa90, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x454bfa93, SUC_lexicon_tags_55 },
    { 0xf0abfa94, SUC_lexicon_tags_12 },
    { 0x514bfa94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cbfa98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606bfa9c, SUC_lexicon_tags_14 },
    { 0xcaebfa9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x488bfa9f, SUC_lexicon_tags_0 },
    { 0x962bfaa0, SUC_lexicon_tags_31 },
    { 0x2c0bfa9f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5d8bfaa3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5debfaa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ebfaaa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6abfaad, SUC_lexicon_tags_1 },
    { 0xf1abfaae, SUC_lexicon_tags_11 },
    { 0x4f8bfaae, SUC_lexicon_tags_25 },
    { 0x6dcbfab0, SUC_lexicon_tags_0 },
    { 0x14ebfab0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6bfab8, SUC_lexicon_tags_28 },
    { 0x672bfab9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cbfabd, SUC_lexicon_tags_37 },
    { 0xc8cbfabe, SUC_lexicon_tags_7 },
    { 0xb6cbfabf, SUC_lexicon_tags_37 },
    { 0xe3ebfabe, SUC_lexicon_tags_38 },
    { 0x9cebfac0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cbfac6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c8bfac8, SUC_lexicon_tags_55 },
    { 0x8bcbfac9, SUC_lexicon_tags_1 },
    { 0xda8bfac9, SUC_lexicon_tags_28 },
    { 0xfd4bfacb, SUC_lexicon_tags_79 },
    { 0x2a6bfac8, SUC_lexicon_tags_57 },
    { 0xda0bfac9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcbfad0, SUC_lexicon_tags_88 },
    { 0x41cbfad1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2bfad6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416bfae3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdebfae6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bfae9, SUC_lexicon_tags_37 },
    { 0x1fabfae9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90bfaf5, SUC_lexicon_tags_6 },
    { 0xe84bfaf6, SUC_lexicon_tags_63 },
    { 0xf18bfaf7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56bfafc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bfb00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a4bfb02, SUC_lexicon_tags_2 },
    { 0x47ebfb03, SUC_lexicon_tags_25 },
    { 0x7dabfb02, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x64cbfb06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18abfb08, SUC_lexicon_tags_1 },
    { 0x93ebfb08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaabfb0e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x9b0bfb10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0bfb13, SUC_lexicon_tags_7 },
    { 0x8cbfb14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x492bfb16, SUC_lexicon_tags_15 },
    { 0x478bfb17, SUC_lexicon_tags_7 },
    { 0x2eabfb18, SUC_lexicon_tags_31 },
    { 0x40abfb19, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xe26bfb1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ecbfb1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ebfb23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x948bfb25, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8bfb2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576bfb2e, SUC_lexicon_tags_12 },
    { 0xebebfb2e, SUC_lexicon_tags_25 },
    { 0x886bfb30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538bfb36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdecbfb39, SUC_lexicon_tags_60 },
    { 0x7b2bfb3a, SUC_lexicon_tags_2 },
    { 0x844bfb3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cbfb3f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ebfb42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bfb45, SUC_lexicon_tags_33 },
    { 0x250bfb46, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bfb49, SUC_lexicon_tags_21 },
    { 0xb4abfb49, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb4e, SUC_lexicon_tags_5 },
    { 0xf26bfb4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ebfb52, SUC_lexicon_tags_35 },
    { 0x344bfb52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26abfb59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8bfb5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfb62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ebfb65, SUC_lexicon_tags_25 },
    { 0x43abfb65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xccabfb69, SUC_lexicon_tags_12 },
    { 0xfaebfb6a, SUC_lexicon_tags_12 },
    { 0x150bfb6a, SUC_lexicon_tags_25 },
    { 0x4cbfb6b, SUC_lexicon_tags_61 },
    { 0x77ebfb6d, SUC_lexicon_tags_65 },
    { 0x1b6bfb6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ebfb72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58bfb76, SUC_lexicon_tags_17 },
    { 0xb32bfb77, SUC_lexicon_tags_33 },
    { 0x156bfb78, SUC_lexicon_tags_25 },
    { 0x8cbfb79, SUC_lexicon_tags_12 },
    { 0x970bfb7a, SUC_lexicon_tags_15 },
    { 0x38abfb7a, SUC_lexicon_tags_7 },
    { 0x9a4bfb79, SUC_lexicon_tags_0 },
    { 0x200bfb7d, SUC_lexicon_tags_32 },
    { 0x4c0bfb7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20bfb8a, SUC_lexicon_tags_16 },
    { 0x26cbfb8b, SUC_lexicon_tags_178 },
    { 0x9b2bfb8a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd78bfb8e, SUC_lexicon_tags_37 },
    { 0x1a0bfb8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfcbfb91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29ebfb93, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bfb97, SUC_lexicon_tags_65 },
    { 0x63abfb97, SUC_lexicon_tags_8 },
    { 0xfe2bfb98, SUC_lexicon_tags_47 },
    { 0x9c6bfb97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe76bfb9c, SUC_lexicon_tags_17 },
    { 0x75cbfb9c, SUC_lexicon_tags_1 },
    { 0xa36bfb9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba8bfba0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffabfba7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd94bfba9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4a2bfbab, SUC_lexicon_tags_40 },
    { 0x5b6bfbab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafabfbb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0bfbb6, SUC_lexicon_tags_7 },
    { 0x144bfbb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66bfbba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d6bfbbc, SUC_lexicon_tags_44 },
    { 0xc76bfbbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbfbc5, SUC_lexicon_tags_17 },
    { 0xed6bfbc6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8abfbcd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x452bfbd0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ebfbd7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312bfbdc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x304bfbde, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x972bfbe1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4bfbe4, SUC_lexicon_tags_43 },
    { 0xafebfbe5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4bfbed, SUC_lexicon_tags_37 },
    { 0x6c4bfbed, SUC_lexicon_tags_37 },
    { 0xaa4bfbef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacabfbf2, SUC_lexicon_tags_85 },
    { 0x936bfbf3, SUC_lexicon_tags_7 },
    { 0x4d6bfbf4, SUC_lexicon_tags_2 },
    { 0xfcabfbf5, SUC_lexicon_tags_15 },
    { 0xac6bfbf6, SUC_lexicon_tags_10 },
    { 0xbc8bfbf6, SUC_lexicon_tags_14 },
    { 0xafabfbf5, SUC_lexicon_tags_17 },
    { 0xed2bfbf9, SUC_lexicon_tags_14 },
    { 0x2a4bfbf9, SUC_lexicon_tags_56 },
    { 0x91ebfbfa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xedabfbfd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeaabfbff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0bfc04, SUC_lexicon_tags_4 },
    { 0x712bfc05, SUC_lexicon_tags_38 },
    { 0xfccbfc04, SUC_lexicon_tags_27 },
    { 0x478bfc07, SUC_lexicon_tags_37 },
    { 0x3e2bfc05, SUC_lexicon_tags_59 },
    { 0x1d4bfc04, SUC_lexicon_tags_38 },
    { 0xbe8bfc0a, SUC_lexicon_tags_7 },
    { 0x91abfc0b, SUC_lexicon_tags_11 },
    { 0xba4bfc0a, SUC_lexicon_tags_28 },
    { 0xa4cbfc0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56cbfc0f, SUC_lexicon_tags_49 },
    { 0x91cbfc0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb54bfc12, SUC_lexicon_tags_30 },
    { 0x80ebfc12, SUC_lexicon_tags_49 },
    { 0x5fcbfc12, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774bfc18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x21ebfc1a, SUC_lexicon_tags_25 },
    { 0x3b4bfc1b, SUC_lexicon_tags_37 },
    { 0x2debfc1c, SUC_lexicon_tags_108 },
    { 0xb66bfc1b, SUC_lexicon_tags_33 },
    { 0x88cbfc1a, SUC_lexicon_tags_19 },
    { 0x10cbfc1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x318bfc22, SUC_lexicon_tags_11 },
    { 0xbebfc22, SUC_lexicon_tags_25 },
    { 0xc72bfc22, SUC_lexicon_tags_37 },
    { 0x802bfc25, SUC_lexicon_tags_47 },
    { 0xc3cbfc26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1ebfc28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x402bfc2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04bfc30, SUC_lexicon_tags_23 },
    { 0xcf8bfc31, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04bfc35, SUC_lexicon_tags_2 },
    { 0xa1cbfc36, SUC_lexicon_tags_17 },
    { 0xdbabfc37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4abfc39, SUC_lexicon_tags_55 },
    { 0xac4bfc39, SUC_lexicon_tags_5 },
    { 0xfaabfc3a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x186bfc3d, SUC_lexicon_tags_8 },
    { 0x722bfc3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x22abfc41, SUC_lexicon_tags_63 },
    { 0x932bfc41, SUC_lexicon_tags_4 },
    { 0x70bfc43, SUC_lexicon_tags_28 },
    { 0x764bfc41, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0bfc4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd90bfc4f, SUC_lexicon_tags_19 },
    { 0xbe4bfc50, SUC_lexicon_tags_15 },
    { 0x16abfc4f, SUC_lexicon_tags_8 },
    { 0x3bcbfc52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ebfc56, SUC_lexicon_tags_17 },
    { 0x4d2bfc57, SUC_lexicon_tags_7 },
    { 0xc24bfc58, SUC_lexicon_tags_2 },
    { 0x9f8bfc59, SUC_lexicon_tags_29 },
    { 0x83cbfc59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166bfc5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68abfc67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0bfc6c, SUC_lexicon_tags_37 },
    { 0xabfc6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc6ebfc6f, SUC_lexicon_tags_7 },
    { 0x852bfc70, SUC_lexicon_tags_25 },
    { 0xc50bfc71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8bfc74, SUC_lexicon_tags_9 },
    { 0xb82bfc75, SUC_lexicon_tags_65 },
    { 0x7eabfc76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeabfc79, SUC_lexicon_tags_28 },
    { 0x88abfc7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ebfc7f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3abfc83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x318bfc85, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x26ebfc87, SUC_lexicon_tags_17 },
    { 0x4a0bfc87, SUC_lexicon_tags_65 },
    { 0x49abfc89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aebfc8c, SUC_lexicon_tags_25 },
    { 0xd6bfc8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66bfc91, SUC_lexicon_tags_16 },
    { 0x464bfc92, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cbfc98, SUC_lexicon_tags_15 },
    { 0xf00bfc99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cbfc9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec4bfc9f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x390bfca1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aebfca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x706bfca7, SUC_lexicon_tags_6 },
    { 0x390bfca7, SUC_lexicon_tags_2 },
    { 0x34cbfca7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cabfcac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24bfcaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeabfcb6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ebfcb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4d6bfcbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4bfcc0, SUC_lexicon_tags_19 },
    { 0xb30bfcc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0abfcca, SUC_lexicon_tags_17 },
    { 0x376bfcca, SUC_lexicon_tags_45 },
    { 0x3a0bfcca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69abfcd1, SUC_lexicon_tags_27 },
    { 0x9d4bfcd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156bfcd8, SUC_lexicon_tags_45 },
    { 0xbb4bfcd9, SUC_lexicon_tags_11 },
    { 0x3b4bfcda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42bfcdd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf62bfcdf, SUC_lexicon_tags_1 },
    { 0x2b6bfce0, SUC_lexicon_tags_2 },
    { 0x9a2bfcdf, SUC_lexicon_tags_38 },
    { 0xb5ebfce2, SUC_lexicon_tags_8 },
    { 0x938bfce3, SUC_lexicon_tags_79 },
    { 0xacebfcdf, SUC_lexicon_tags_15 },
    { 0xafabfce4, SUC_lexicon_tags_47 },
    { 0x25cbfce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeebfce9, SUC_lexicon_tags_49 },
    { 0x656bfcea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfaabfcec, SUC_lexicon_tags_0 },
    { 0x16cbfced, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978bfcf7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ebfcfe, SUC_lexicon_tags_16 },
    { 0x558bfcff, SUC_lexicon_tags_49 },
    { 0x52ebfd00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3debfd03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe86bfd05, SUC_lexicon_tags_11 },
    { 0xc1cbfd06, SUC_lexicon_tags_31 },
    { 0xc6bfd05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2bfd0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ebfd16, SUC_lexicon_tags_15 },
    { 0xf2cbfd16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4debfd1b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa22bfd1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ebfd23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6bfd28, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678bfd2c, SUC_lexicon_tags_14 },
    { 0x322bfd2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeabfd2f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812bfd3b, SUC_lexicon_tags_47 },
    { 0xa42bfd3c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2bfd40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc0bfd42, SUC_lexicon_tags_7 },
    { 0xe88bfd43, SUC_lexicon_tags_38 },
    { 0xb04bfd42, SUC_lexicon_tags_11 },
    { 0xdd2bfd45, SUC_lexicon_tags_23 },
    { 0x8debfd46, SUC_lexicon_tags_43 },
    { 0x814bfd47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85cbfd49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fabfd4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc14bfd4f, SUC_lexicon_tags_48 },
    { 0x47abfd50, SUC_lexicon_tags_33 },
    { 0x9e4bfd50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0bfd5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x522bfd5c, SUC_lexicon_tags_15 },
    { 0x7a4bfd5d, SUC_lexicon_tags_11 },
    { 0xa3cbfd5c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ebfd65, SUC_lexicon_tags_12 },
    { 0x268bfd65, SUC_lexicon_tags_15 },
    { 0x6f4bfd67, SUC_lexicon_tags_15 },
    { 0x892bfd68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x974bfd6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa6bfd6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eebfd72, SUC_lexicon_tags_21 },
    { 0x168bfd73, SUC_lexicon_tags_87 },
    { 0x2acbfd73, SUC_lexicon_tags_2 },
    { 0x558bfd72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16bfd79, SUC_lexicon_tags_8 },
    { 0x90bfd7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a2bfd7c, SUC_lexicon_tags_53 },
    { 0x572bfd7d, SUC_lexicon_tags_7 },
    { 0x398bfd7e, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x74bfd80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ebfd85, SUC_lexicon_tags_39 },
    { 0x12bfd86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138bfd8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe90bfd8c, SUC_lexicon_tags_17 },
    { 0x256bfd8d, SUC_lexicon_tags_17 },
    { 0xdbcbfd8e, SUC_lexicon_tags_9 },
    { 0x16ebfd8c, SUC_lexicon_tags_5 },
    { 0x630bfd90, SUC_lexicon_tags_1 },
    { 0x684bfd8d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x432bfd93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ccbfd95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802bfd99, SUC_lexicon_tags_10 },
    { 0x99ebfd9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2bfda5, SUC_lexicon_tags_1 },
    { 0xc64bfda5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cbfdaa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x732bfdad, SUC_lexicon_tags_816 },
    { 0xd72bfdae, SUC_lexicon_tags_20 },
    { 0x6d2bfdaf, SUC_lexicon_tags_31 },
    { 0xf5abfdb0, SUC_lexicon_tags_17 },
    { 0x586bfdb0, SUC_lexicon_tags_37 },
    { 0x51abfdad, SUC_lexicon_tags_25 },
    { 0x868bfdb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cbfdb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x506bfdbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cbfdc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6debfdc9, SUC_lexicon_tags_8 },
    { 0xfe4bfdca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcbfdd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4bfdd6, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x43abfdd8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4eebfdda, SUC_lexicon_tags_17 },
    { 0xe2cbfdda, SUC_lexicon_tags_2 },
    { 0xd58bfdda, SUC_lexicon_tags_39 },
    { 0x398bfddd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x974bfddf, SUC_lexicon_tags_37 },
    { 0xf6cbfddf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5febfde2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ebfde5, SUC_lexicon_tags_15 },
    { 0x304bfde5, SUC_lexicon_tags_17 },
    { 0x56cbfde7, SUC_lexicon_tags_7 },
    { 0xfeabfde8, SUC_lexicon_tags_37 },
    { 0x5e6bfde6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe00bfdeb, SUC_lexicon_tags_15 },
    { 0xcc2bfdec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x992bfdee, SUC_lexicon_tags_15 },
    { 0xff4bfdef, SUC_lexicon_tags_1 },
    { 0xef8bfdf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94ebfdf2, SUC_lexicon_tags_12 },
    { 0xe26bfdf3, SUC_lexicon_tags_47 },
    { 0xa26bfdf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644bfdf8, SUC_lexicon_tags_40 },
    { 0xb82bfdf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8bfdfc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bfe01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5abfe03, SUC_lexicon_tags_2 },
    { 0xdeebfe03, SUC_lexicon_tags_92 },
    { 0x1a2bfe04, SUC_lexicon_tags_35 },
    { 0xd88bfe06, SUC_lexicon_tags_16 },
    { 0xdb0bfe07, SUC_lexicon_tags_17 },
    { 0xe08bfe05, SUC_lexicon_tags_63 },
    { 0xf00bfe09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ebfe0b, SUC_lexicon_tags_63 },
    { 0x496bfe0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810bfe11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ebfe18, SUC_lexicon_tags_29 },
    { 0x56bfe19, SUC_lexicon_tags_16 },
    { 0xde0bfe18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64bfe1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ebfe24, SUC_lexicon_tags_0 },
    { 0xcb6bfe25, SUC_lexicon_tags_11 },
    { 0xe04bfe25, SUC_lexicon_tags_45 },
    { 0xc62bfe24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x520bfe2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf54bfe2c, SUC_lexicon_tags_27 },
    { 0xb18bfe2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2bfe32, SUC_lexicon_tags_65 },
    { 0xf5cbfe33, SUC_lexicon_tags_12 },
    { 0xa9abfe34, SUC_lexicon_tags_25 },
    { 0xba0bfe34, SUC_lexicon_tags_11 },
    { 0xc76bfe36, SUC_lexicon_tags_12 },
    { 0x3d4bfe37, SUC_lexicon_tags_2 },
    { 0xe24bfe37, SUC_lexicon_tags_61 },
    { 0x23ebfe33, SUC_lexicon_tags_19 },
    { 0x7c2bfe33, SUC_lexicon_tags_37 },
    { 0xa5ebfe3b, SUC_lexicon_tags_12 },
    { 0x3eabfe3c, SUC_lexicon_tags_0 },
    { 0xe0abfe3d, SUC_lexicon_tags_1 },
    { 0x1e2bfe3e, SUC_lexicon_tags_37 },
    { 0x528bfe3c, SUC_lexicon_tags_37 },
    { 0xe10bfe3f, SUC_lexicon_tags_38 },
    { 0xec8bfe3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0bfe47, SUC_lexicon_tags_47 },
    { 0x38ebfe48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebfe4e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bfe54, SUC_lexicon_tags_15 },
    { 0x886bfe55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4bfe5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788bfe5f, SUC_lexicon_tags_542 },
    { 0x4a8bfe60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19cbfe62, SUC_lexicon_tags_60 },
    { 0x138bfe63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8bfe66, SUC_lexicon_tags_11 },
    { 0x762bfe66, SUC_lexicon_tags_16 },
    { 0xdf0bfe67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x512bfe6a, SUC_lexicon_tags_25 },
    { 0xbabfe6a, SUC_lexicon_tags_37 },
    { 0xff0bfe6b, SUC_lexicon_tags_60 },
    { 0x730bfe6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc46bfe6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd36bfe71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bfe75, SUC_lexicon_tags_15 },
    { 0xdfabfe75, SUC_lexicon_tags_37 },
    { 0xbdabfe77, SUC_lexicon_tags_17 },
    { 0x866bfe78, SUC_lexicon_tags_108 },
    { 0x5f8bfe79, SUC_lexicon_tags_34 },
    { 0x74abfe7a, SUC_lexicon_tags_49 },
    { 0x98abfe7a, SUC_lexicon_tags_47 },
    { 0xc88bfe7c, SUC_lexicon_tags_60 },
    { 0x262bfe75, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40bfe80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x628bfe83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50bfe89, SUC_lexicon_tags_37 },
    { 0x842bfe89, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x534bfe8c, SUC_lexicon_tags_87 },
    { 0x728bfe8c, SUC_lexicon_tags_10 },
    { 0x94ebfe8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88bfe91, SUC_lexicon_tags_17 },
    { 0x14abfe91, SUC_lexicon_tags_2 },
    { 0xe26bfe91, SUC_lexicon_tags_2 },
    { 0xa32bfe94, SUC_lexicon_tags_19 },
    { 0x4abfe95, SUC_lexicon_tags_17 },
    { 0xccabfe95, SUC_lexicon_tags_15 },
    { 0x82ebfe97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6abfe99, SUC_lexicon_tags_13 },
    { 0xcd8bfe9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538bfe9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7bcbfea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf64bfea2, SUC_lexicon_tags_74 },
    { 0x6e8bfea2, SUC_lexicon_tags_301 },
    { 0xe64bfea2, SUC_lexicon_tags_58 },
    { 0xd86bfea5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8bfea9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556bfeae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98bfeb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0bfeb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bfeb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccbfebd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448bfec1, SUC_lexicon_tags_65 },
    { 0x654bfec1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x330bfec4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc78bfec6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6bfec8, SUC_lexicon_tags_12 },
    { 0x702bfec8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x386bfecb, SUC_lexicon_tags_27 },
    { 0x8b2bfecc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6bfecf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70cbfed1, SUC_lexicon_tags_49 },
    { 0x60bfed2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26bfed6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7abfed9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0bfede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7cbfee0, SUC_lexicon_tags_1 },
    { 0x7a4bfee1, SUC_lexicon_tags_7 },
    { 0xd62bfee1, SUC_lexicon_tags_12 },
    { 0x9c4bfee3, SUC_lexicon_tags_37 },
    { 0xb58bfee4, SUC_lexicon_tags_26 },
    { 0xe42bfee5, SUC_lexicon_tags_37 },
    { 0x676bfee5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1b0bfee8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4bfef0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84bfef3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8bfef7, SUC_lexicon_tags_2 },
    { 0xf82bfef7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x744bfefa, SUC_lexicon_tags_21 },
    { 0xc42bfefa, SUC_lexicon_tags_2 },
    { 0x5c8bfefc, SUC_lexicon_tags_43 },
    { 0x52cbfefd, SUC_lexicon_tags_11 },
    { 0x3c0bfefe, SUC_lexicon_tags_7 },
    { 0x9e6bfeff, SUC_lexicon_tags_25 },
    { 0xe3ebfeff, SUC_lexicon_tags_25 },
    { 0x8d4bfeff, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2f4bff03, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x254bff06, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x468bff09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872bff0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6bff13, SUC_lexicon_tags_7 },
    { 0x8cebff14, SUC_lexicon_tags_12 },
    { 0x80abff15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf6bff17, SUC_lexicon_tags_11 },
    { 0x222bff17, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe5cbff1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ebff1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3abff20, SUC_lexicon_tags_63 },
    { 0xf5abff20, SUC_lexicon_tags_37 },
    { 0xe4abff22, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x826bff25, SUC_lexicon_tags_45 },
    { 0x340bff25, SUC_lexicon_tags_47 },
    { 0xd32bff25, SUC_lexicon_tags_17 },
    { 0x594bff27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ebff2a, SUC_lexicon_tags_11 },
    { 0x32bff2b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x5f8bff2d, SUC_lexicon_tags_14 },
    { 0x780bff2e, SUC_lexicon_tags_63 },
    { 0x9f2bff2f, SUC_lexicon_tags_28 },
    { 0x568bff30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8bff33, SUC_lexicon_tags_19 },
    { 0xd56bff33, SUC_lexicon_tags_15 },
    { 0x240bff35, SUC_lexicon_tags_4 },
    { 0xf9cbff35, SUC_lexicon_tags_4 },
    { 0x3e6bff34, SUC_lexicon_tags_16 },
    { 0x31ebff38, SUC_lexicon_tags_15 },
    { 0x1cabff39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x976bff3b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd82bff3d, SUC_lexicon_tags_37 },
    { 0xec0bff3e, SUC_lexicon_tags_11 },
    { 0xef0bff3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94bff43, SUC_lexicon_tags_37 },
    { 0x80ebff43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe22bff46, SUC_lexicon_tags_37 },
    { 0x958bff47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbebff4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ebff55, SUC_lexicon_tags_14 },
    { 0x690bff55, SUC_lexicon_tags_15 },
    { 0x386bff57, SUC_lexicon_tags_25 },
    { 0xb68bff58, SUC_lexicon_tags_0 },
    { 0x9febff57, SUC_lexicon_tags_19 },
    { 0xe76bff5a, SUC_lexicon_tags_17 },
    { 0x2ecbff5a, SUC_lexicon_tags_2 },
    { 0x2cebff5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x922bff5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1cabff61, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x5acbff63, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x760bff65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08bff69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcbff6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x618bff70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8bff73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6bff77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1babff79, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1cabff7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bff80, SUC_lexicon_tags_2 },
    { 0xba2bff81, SUC_lexicon_tags_302 },
    { 0xa98bff81, SUC_lexicon_tags_12 },
    { 0x39ebff82, SUC_lexicon_tags_15 },
    { 0xcc0bff80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0bff87, SUC_lexicon_tags_38 },
    { 0x6d6bff87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6bff8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70bff8e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78bff91, SUC_lexicon_tags_37 },
    { 0x6eabff92, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7eabff94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x692bff96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4abff98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60bff9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35cbff9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b4bff9f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ebffa3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc6ebffa5, SUC_lexicon_tags_40 },
    { 0x4bcbffa6, SUC_lexicon_tags_3 },
    { 0x8abffa7, SUC_lexicon_tags_15 },
    { 0xcc4bffa7, SUC_lexicon_tags_45 },
    { 0x528bffa9, SUC_lexicon_tags_1 },
    { 0x9d0bffaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138bffae, SUC_lexicon_tags_25 },
    { 0xfccbffaf, SUC_lexicon_tags_1 },
    { 0x7dcbffb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf88bffb3, SUC_lexicon_tags_35 },
    { 0xc24bffb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x344bffb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4bffba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cbffbe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf08bffc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x448bffc2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926bffc6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798bffcc, SUC_lexicon_tags_63 },
    { 0x14cbffcc, SUC_lexicon_tags_25 },
    { 0x1ccbffce, SUC_lexicon_tags_38 },
    { 0xf78bffcf, SUC_lexicon_tags_7 },
    { 0xfdabffd0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30abffd5, SUC_lexicon_tags_11 },
    { 0xd5cbffd5, SUC_lexicon_tags_17 },
    { 0xca8bffd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0bffd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32abffe3, SUC_lexicon_tags_1 },
    { 0xe90bffe4, SUC_lexicon_tags_15 },
    { 0xcc2bffe4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe70bffe7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x870bffe9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97abffeb, SUC_lexicon_tags_35 },
    { 0x816bffeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf08bffee, SUC_lexicon_tags_25 },
    { 0x98ebffee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7abfff1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24bfff5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22bfff8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x680bfffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x620bfffe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8c0002, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x91ac0004, SUC_lexicon_tags_55 },
    { 0x648c0005, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcc000a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa84c000c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42c0012, SUC_lexicon_tags_40 },
    { 0x512c0013, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d6c0015, SUC_lexicon_tags_1 },
    { 0x112c0015, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x810c0019, SUC_lexicon_tags_17 },
    { 0xfdec001a, SUC_lexicon_tags_63 },
    { 0x39cc001b, SUC_lexicon_tags_15 },
    { 0x202c001c, SUC_lexicon_tags_43 },
    { 0x356c001b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73cc001f, SUC_lexicon_tags_11 },
    { 0x7a2c001f, SUC_lexicon_tags_16 },
    { 0x224c0021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bec0027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cc002d, SUC_lexicon_tags_14 },
    { 0xfecc002d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79cc0030, SUC_lexicon_tags_6 },
    { 0x39ac0030, SUC_lexicon_tags_10 },
    { 0x1eac0032, SUC_lexicon_tags_158 },
    { 0x762c0032, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0c0039, SUC_lexicon_tags_37 },
    { 0xa36c0039, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcaac003c, SUC_lexicon_tags_37 },
    { 0x3bcc003c, SUC_lexicon_tags_25 },
    { 0xc82c003e, SUC_lexicon_tags_15 },
    { 0x5a6c003d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cc0042, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2c0047, SUC_lexicon_tags_13 },
    { 0xd62c0048, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40c004c, SUC_lexicon_tags_47 },
    { 0xa96c004d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x830c0050, SUC_lexicon_tags_2 },
    { 0x318c0050, SUC_lexicon_tags_85 },
    { 0xdc8c0051, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x808c0054, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c0059, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aac005d, SUC_lexicon_tags_4 },
    { 0x268c005e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc0061, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ac0064, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4c006e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652c0072, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904c0077, SUC_lexicon_tags_17 },
    { 0x5e4c0078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddec007a, SUC_lexicon_tags_37 },
    { 0x440c007b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cc0084, SUC_lexicon_tags_25 },
    { 0x5c0c0085, SUC_lexicon_tags_21 },
    { 0x188c0085, SUC_lexicon_tags_12 },
    { 0x306c0087, SUC_lexicon_tags_46 },
    { 0x7fac0088, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436c008d, SUC_lexicon_tags_47 },
    { 0x60ac008d, SUC_lexicon_tags_25 },
    { 0x1d8c008e, SUC_lexicon_tags_49 },
    { 0xf04c0090, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x660c0092, SUC_lexicon_tags_29 },
    { 0x85ec0093, SUC_lexicon_tags_10 },
    { 0x1ecc0092, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536c009b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144c00a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fac00a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbcec00a8, SUC_lexicon_tags_12 },
    { 0x9d2c00a9, SUC_lexicon_tags_11 },
    { 0xec6c00a9, SUC_lexicon_tags_25 },
    { 0xbcc00a8, SUC_lexicon_tags_37 },
    { 0x866c00ac, SUC_lexicon_tags_37 },
    { 0xca6c00ad, SUC_lexicon_tags_60 },
    { 0xeec00ae, SUC_lexicon_tags_12 },
    { 0xc04c00ae, SUC_lexicon_tags_33 },
    { 0xd94c00b0, SUC_lexicon_tags_1 },
    { 0xffcc00af, SUC_lexicon_tags_5 },
    { 0xd78c00af, SUC_lexicon_tags_6 },
    { 0xc3ec00b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5f0c00b5, SUC_lexicon_tags_23 },
    { 0xb86c00b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0cc00b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x504c00bb, SUC_lexicon_tags_12 },
    { 0xbf2c00bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414c00c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde8c00c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d8c00c5, SUC_lexicon_tags_25 },
    { 0x9e4c00c6, SUC_lexicon_tags_12 },
    { 0xbb2c00c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56c00ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ec00d0, SUC_lexicon_tags_15 },
    { 0x7f4c00d1, SUC_lexicon_tags_31 },
    { 0x20c00d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe70c00d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecc00d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352c00dd, SUC_lexicon_tags_1 },
    { 0x4eec00dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c00e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eec00e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aac00eb, SUC_lexicon_tags_70 },
    { 0x318c00eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb0c00ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0c00f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728c00f5, SUC_lexicon_tags_151 },
    { 0x2b4c00f6, SUC_lexicon_tags_37 },
    { 0x67ec00f6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x718c00f9, SUC_lexicon_tags_8 },
    { 0x9b6c00fa, SUC_lexicon_tags_16 },
    { 0x6dcc00fb, SUC_lexicon_tags_11 },
    { 0x792c00fc, SUC_lexicon_tags_14 },
    { 0x566c00fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8c0101, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb4ac0103, SUC_lexicon_tags_1 },
    { 0x3d2c0103, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182c0108, SUC_lexicon_tags_15 },
    { 0x594c0109, SUC_lexicon_tags_43 },
    { 0x860c0109, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434c0110, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4c0114, SUC_lexicon_tags_26 },
    { 0xeccc0114, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6c0119, SUC_lexicon_tags_11 },
    { 0x7c4c011a, SUC_lexicon_tags_11 },
    { 0x832c011b, SUC_lexicon_tags_1 },
    { 0x796c011b, SUC_lexicon_tags_44 },
    { 0xb9ec0119, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6c0121, SUC_lexicon_tags_17 },
    { 0xcf6c0122, SUC_lexicon_tags_37 },
    { 0x938c0123, SUC_lexicon_tags_45 },
    { 0xe82c0123, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cc0127, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ac012d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc44c012f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfd8c0131, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xafec0133, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804c0139, SUC_lexicon_tags_19 },
    { 0xa5ec013a, SUC_lexicon_tags_7 },
    { 0x644c013a, SUC_lexicon_tags_25 },
    { 0x978c013c, SUC_lexicon_tags_11 },
    { 0xaa6c013c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8c0141, SUC_lexicon_tags_25 },
    { 0x9e4c0142, SUC_lexicon_tags_38 },
    { 0x1b6c0143, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740c0150, SUC_lexicon_tags_17 },
    { 0x2bcc0151, SUC_lexicon_tags_7 },
    { 0x4e8c0150, SUC_lexicon_tags_42 },
    { 0x6b6c0153, SUC_lexicon_tags_20 },
    { 0x9fac0153, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6c015b, SUC_lexicon_tags_1 },
    { 0xbb6c015c, SUC_lexicon_tags_12 },
    { 0x24ac015d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85cc015f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996c0163, SUC_lexicon_tags_12 },
    { 0x974c0164, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc0169, SUC_lexicon_tags_47 },
    { 0x5b6c016a, SUC_lexicon_tags_29 },
    { 0x958c016b, SUC_lexicon_tags_17 },
    { 0x458c016c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68c0171, SUC_lexicon_tags_11 },
    { 0xeb4c0171, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x322c0175, SUC_lexicon_tags_10 },
    { 0x68c0175, SUC_lexicon_tags_37 },
    { 0xf38c0175, SUC_lexicon_tags_63 },
    { 0xb2cc0176, SUC_lexicon_tags_29 },
    { 0x542c0179, SUC_lexicon_tags_1 },
    { 0x396c0175, SUC_lexicon_tags_3 },
    { 0x4eec017b, SUC_lexicon_tags_17 },
    { 0x3ec017b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc0185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c0189, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114c018d, SUC_lexicon_tags_14 },
    { 0xcc4c018e, SUC_lexicon_tags_37 },
    { 0x2a8c018e, SUC_lexicon_tags_28 },
    { 0x890c018d, SUC_lexicon_tags_17 },
    { 0x312c0191, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x734c0193, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634c0197, SUC_lexicon_tags_19 },
    { 0x9bcc0197, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4c019b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aac019f, SUC_lexicon_tags_55 },
    { 0xfecc01a0, SUC_lexicon_tags_60 },
    { 0x15ec01a1, SUC_lexicon_tags_7 },
    { 0x716c01a1, SUC_lexicon_tags_34 },
    { 0x734c01a3, SUC_lexicon_tags_12 },
    { 0x142c01a4, SUC_lexicon_tags_45 },
    { 0xd0c01a1, SUC_lexicon_tags_27 },
    { 0xa3ec01a1, SUC_lexicon_tags_19 },
    { 0x788c01a7, SUC_lexicon_tags_23 },
    { 0x29ec01a3, SUC_lexicon_tags_7 },
    { 0x126c01a9, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2c01af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x61cc01b2, SUC_lexicon_tags_47 },
    { 0xa9ec01b3, SUC_lexicon_tags_49 },
    { 0xbb8c01b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06c01b7, SUC_lexicon_tags_37 },
    { 0xfcec01b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6c01ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e4c01bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf84c01be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x790c01c0, SUC_lexicon_tags_37 },
    { 0x2e0c01c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x72ec01c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53ac01c5, SUC_lexicon_tags_15 },
    { 0x876c01c6, SUC_lexicon_tags_58 },
    { 0x73ec01c7, SUC_lexicon_tags_18 },
    { 0x2e2c01c5, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c01d0, SUC_lexicon_tags_87 },
    { 0xe46c01d0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x972c01d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8c01d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dec01db, SUC_lexicon_tags_25 },
    { 0xb18c01dc, SUC_lexicon_tags_11 },
    { 0xaa8c01dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd72c01df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c6c01e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2c01e5, SUC_lexicon_tags_32 },
    { 0xc5cc01e5, SUC_lexicon_tags_25 },
    { 0x87cc01e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60c01ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854c01ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a6c01f0, SUC_lexicon_tags_61 },
    { 0x32c01f0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ec01f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x364c01f9, SUC_lexicon_tags_7 },
    { 0x15cc01fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4ac01fc, SUC_lexicon_tags_17 },
    { 0x19ac01fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542c0201, SUC_lexicon_tags_37 },
    { 0x86c0201, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8c0205, SUC_lexicon_tags_37 },
    { 0x5bcc0205, SUC_lexicon_tags_0 },
    { 0x7d2c0206, SUC_lexicon_tags_48 },
    { 0xdec0205, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270c0210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70c0213, SUC_lexicon_tags_65 },
    { 0xb2ec0214, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xffac0216, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x290c0218, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x864c021b, SUC_lexicon_tags_21 },
    { 0x87ec021c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6c0220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x714c0223, SUC_lexicon_tags_193 },
    { 0xdb0c0223, SUC_lexicon_tags_25 },
    { 0x910c0224, SUC_lexicon_tags_25 },
    { 0x4bac0226, SUC_lexicon_tags_11 },
    { 0x6dec0227, SUC_lexicon_tags_27 },
    { 0x4f4c0227, SUC_lexicon_tags_54 },
    { 0xf84c0229, SUC_lexicon_tags_49 },
    { 0xd4c022a, SUC_lexicon_tags_2 },
    { 0x95ac022b, SUC_lexicon_tags_23 },
    { 0x398c022b, SUC_lexicon_tags_38 },
    { 0xd6ac022d, SUC_lexicon_tags_1 },
    { 0xc8ac022d, SUC_lexicon_tags_113 },
    { 0x3fec0228, SUC_lexicon_tags_25 },
    { 0x338c022f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0c0233, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xaf8c0235, SUC_lexicon_tags_12 },
    { 0x12cc0236, SUC_lexicon_tags_17 },
    { 0x738c0236, SUC_lexicon_tags_4 },
    { 0xb4ec0236, SUC_lexicon_tags_39 },
    { 0x830c0239, SUC_lexicon_tags_11 },
    { 0x626c0237, SUC_lexicon_tags_14 },
    { 0x3b8c023b, SUC_lexicon_tags_2 },
    { 0x14cc023c, SUC_lexicon_tags_75 },
    { 0xd00c023d, SUC_lexicon_tags_47 },
    { 0xf58c023c, SUC_lexicon_tags_27 },
    { 0xbaec023d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ac0243, SUC_lexicon_tags_9 },
    { 0xf42c0244, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120c024a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4c0253, SUC_lexicon_tags_1 },
    { 0x100c0254, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x324c0256, SUC_lexicon_tags_17 },
    { 0x540c0257, SUC_lexicon_tags_46 },
    { 0xccac0256, SUC_lexicon_tags_97 },
    { 0x9f8c0259, SUC_lexicon_tags_7 },
    { 0x28cc0257, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x852c025d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cc0262, SUC_lexicon_tags_12 },
    { 0x26ac0262, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572c0267, SUC_lexicon_tags_11 },
    { 0x6ac0268, SUC_lexicon_tags_10 },
    { 0xf14c0269, SUC_lexicon_tags_37 },
    { 0x3fcc0267, SUC_lexicon_tags_17 },
    { 0x9acc026b, SUC_lexicon_tags_30 },
    { 0x18ac026b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cec026f, SUC_lexicon_tags_1 },
    { 0x650c0270, SUC_lexicon_tags_1 },
    { 0x4a0c0270, SUC_lexicon_tags_46 },
    { 0x1bac0272, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc2cc0274, SUC_lexicon_tags_182 },
    { 0xe50c0275, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6c0278, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x656c027b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ec027f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72c0283, SUC_lexicon_tags_43 },
    { 0x682c0283, SUC_lexicon_tags_43 },
    { 0x6e4c0283, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00c028e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x964c0291, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2c0294, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0c0297, SUC_lexicon_tags_17 },
    { 0x332c0298, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808c029c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cc029f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ac02a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2c02a6, SUC_lexicon_tags_4 },
    { 0x58ec02a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ac02aa, SUC_lexicon_tags_74 },
    { 0x6b2c02aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8c02ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cc02b4, SUC_lexicon_tags_60 },
    { 0xc8cc02b5, SUC_lexicon_tags_11 },
    { 0x5b2c02b6, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6c02b9, SUC_lexicon_tags_46 },
    { 0xe7ac02ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf50c02bc, SUC_lexicon_tags_7 },
    { 0x83ac02bd, SUC_lexicon_tags_17 },
    { 0x6eac02be, SUC_lexicon_tags_1 },
    { 0x18ac02be, SUC_lexicon_tags_6 },
    { 0xa0ec02c0, SUC_lexicon_tags_27 },
    { 0xc4c02c1, SUC_lexicon_tags_45 },
    { 0x162c02c1, SUC_lexicon_tags_37 },
    { 0x286c02c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc74c02c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eac02d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8c02d3, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x2fcc02d5, SUC_lexicon_tags_11 },
    { 0x72c02d6, SUC_lexicon_tags_11 },
    { 0xb4ac02d7, SUC_lexicon_tags_68 },
    { 0x28c02d7, SUC_lexicon_tags_11 },
    { 0xd80c02d9, SUC_lexicon_tags_23 },
    { 0x878c02d9, SUC_lexicon_tags_27 },
    { 0x492c02d9, SUC_lexicon_tags_37 },
    { 0x8b2c02dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14ac02de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0c02e1, SUC_lexicon_tags_4 },
    { 0xf36c02e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7b6c02e4, SUC_lexicon_tags_37 },
    { 0x49cc02e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x432c02e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8c02f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38c02fc, SUC_lexicon_tags_82 },
    { 0xb38c02fc, SUC_lexicon_tags_2 },
    { 0xd5ac02fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc0303, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25ac0305, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8ac0307, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446c030f, SUC_lexicon_tags_1 },
    { 0x7a0c0310, SUC_lexicon_tags_37 },
    { 0x90ac0311, SUC_lexicon_tags_2 },
    { 0x184c030f, SUC_lexicon_tags_27 },
    { 0xc7cc0311, SUC_lexicon_tags_25 },
    { 0x8cec0314, SUC_lexicon_tags_47 },
    { 0xb1ec0315, SUC_lexicon_tags_25 },
    { 0x872c0316, SUC_lexicon_tags_25 },
    { 0x470c0315, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x820c031a, SUC_lexicon_tags_37 },
    { 0xfec031a, SUC_lexicon_tags_11 },
    { 0x73cc031c, SUC_lexicon_tags_25 },
    { 0x9f6c031d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dec0324, SUC_lexicon_tags_60 },
    { 0xaf0c0324, SUC_lexicon_tags_2 },
    { 0xb08c0325, SUC_lexicon_tags_6 },
    { 0xe6ac0327, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa48c0329, SUC_lexicon_tags_6 },
    { 0x73ac032a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98c032e, SUC_lexicon_tags_16 },
    { 0x8bac032f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c0333, SUC_lexicon_tags_11 },
    { 0x4a4c0333, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x598c0337, SUC_lexicon_tags_0 },
    { 0xca6c0338, SUC_lexicon_tags_68 },
    { 0x38c0339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30c033f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238c0344, SUC_lexicon_tags_15 },
    { 0xec4c0345, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54c034b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2c0350, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x594c0352, SUC_lexicon_tags_80 },
    { 0xb52c0353, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0c0357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168c035b, SUC_lexicon_tags_38 },
    { 0xb6ec035c, SUC_lexicon_tags_37 },
    { 0xfa2c035c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ac0368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946c036c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x408c036f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaa4c0371, SUC_lexicon_tags_2 },
    { 0xa2cc0371, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54c0377, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9e4c0379, SUC_lexicon_tags_17 },
    { 0x904c037a, SUC_lexicon_tags_37 },
    { 0x908c0379, SUC_lexicon_tags_38 },
    { 0x16ac037c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74c0381, SUC_lexicon_tags_7 },
    { 0xf04c0381, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234c0387, SUC_lexicon_tags_39 },
    { 0xd2ac0388, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7b0c038a, SUC_lexicon_tags_55 },
    { 0xf3ac038b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1ec038d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f2c038f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cc0394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872c0398, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6c039b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc88c039d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ec03a1, SUC_lexicon_tags_49 },
    { 0x43ec03a2, SUC_lexicon_tags_8 },
    { 0x66ac03a3, SUC_lexicon_tags_32 },
    { 0x7c0c03a4, SUC_lexicon_tags_13 },
    { 0xd9cc03a5, SUC_lexicon_tags_1 },
    { 0x87ac03a1, SUC_lexicon_tags_32 },
    { 0xe32c03a7, SUC_lexicon_tags_16 },
    { 0x62c03a7, SUC_lexicon_tags_60 },
    { 0xfaec03a9, SUC_lexicon_tags_15 },
    { 0x5d2c03aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4dec03ac, SUC_lexicon_tags_23 },
    { 0xc62c03ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ac03b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x424c03b3, SUC_lexicon_tags_21 },
    { 0xc48c03b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676c03b8, SUC_lexicon_tags_46 },
    { 0x328c03b9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xba0c03bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8cc03bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2cec03bf, SUC_lexicon_tags_2 },
    { 0xce2c03c0, SUC_lexicon_tags_0 },
    { 0x798c03c0, SUC_lexicon_tags_11 },
    { 0x808c03c2, SUC_lexicon_tags_12 },
    { 0x46c03c2, SUC_lexicon_tags_68 },
    { 0x7b8c03c4, SUC_lexicon_tags_17 },
    { 0x39ac03c3, SUC_lexicon_tags_38 },
    { 0x516c03c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4c03c9, SUC_lexicon_tags_45 },
    { 0x6e8c03ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8c03ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f2c03d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c03d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cac03d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x360c03dc, SUC_lexicon_tags_61 },
    { 0xd00c03dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x25cc03df, SUC_lexicon_tags_14 },
    { 0x4b4c03e0, SUC_lexicon_tags_11 },
    { 0x37ac03e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2ac03e3, SUC_lexicon_tags_21 },
    { 0x73ac03e3, SUC_lexicon_tags_16 },
    { 0x1e8c03e5, SUC_lexicon_tags_9 },
    { 0x5c4c03e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4c03e9, SUC_lexicon_tags_23 },
    { 0x2c4c03e9, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xa8ac03ec, SUC_lexicon_tags_315 },
    { 0, NULL },
    { 0xa52c03ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc03f2, SUC_lexicon_tags_39 },
    { 0x602c03f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cc03f6, SUC_lexicon_tags_32 },
    { 0x6b0c03f6, SUC_lexicon_tags_7 },
    { 0xd32c03f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cc03ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x422c0401, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaacc0403, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4c0407, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0c040e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90ec0410, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x612c0413, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc8c0415, SUC_lexicon_tags_26 },
    { 0x186c0416, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x49ac0418, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6c041b, SUC_lexicon_tags_46 },
    { 0xb78c041b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8c041f, SUC_lexicon_tags_60 },
    { 0xdd0c0420, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x658c0422, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66c0425, SUC_lexicon_tags_1 },
    { 0x7c2c0426, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9c6c0428, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c8c042a, SUC_lexicon_tags_118 },
    { 0xe94c042b, SUC_lexicon_tags_23 },
    { 0xaf0c042c, SUC_lexicon_tags_82 },
    { 0x2d4c042d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafec0433, SUC_lexicon_tags_12 },
    { 0x4ecc0434, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cc0437, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4c043a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fec043e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dec0440, SUC_lexicon_tags_15 },
    { 0x434c0441, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6c0444, SUC_lexicon_tags_47 },
    { 0xc6ec0445, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe4c0447, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x860c0449, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf64c044b, SUC_lexicon_tags_25 },
    { 0x3a4c044c, SUC_lexicon_tags_26 },
    { 0x9b4c044c, SUC_lexicon_tags_7 },
    { 0xb96c044e, SUC_lexicon_tags_33 },
    { 0x8eac044b, SUC_lexicon_tags_15 },
    { 0xb7ac044c, SUC_lexicon_tags_25 },
    { 0x13cc044c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6c0454, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cc0461, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ec0464, SUC_lexicon_tags_21 },
    { 0xf8ac0465, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eac0468, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec046e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60c0472, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcc0478, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c047d, SUC_lexicon_tags_23 },
    { 0xa60c047e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x96ec0480, SUC_lexicon_tags_151 },
    { 0xdfec0481, SUC_lexicon_tags_65 },
    { 0x232c0480, SUC_lexicon_tags_55 },
    { 0x502c0483, SUC_lexicon_tags_11 },
    { 0x6c8c0483, SUC_lexicon_tags_29 },
    { 0x83ec0485, SUC_lexicon_tags_2 },
    { 0x852c0485, SUC_lexicon_tags_15 },
    { 0xc84c0480, SUC_lexicon_tags_46 },
    { 0xdfcc0480, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0c048b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eac0494, SUC_lexicon_tags_10 },
    { 0x730c0495, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0c0498, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96c049c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c049f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44c04a2, SUC_lexicon_tags_12 },
    { 0x6e6c04a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aec04a7, SUC_lexicon_tags_60 },
    { 0x8dec04a8, SUC_lexicon_tags_37 },
    { 0x21ac04a7, SUC_lexicon_tags_11 },
    { 0xa9cc04aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc04b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9ec04b2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42c04ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cc04be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaac04c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb48c04c4, SUC_lexicon_tags_188 },
    { 0xbb8c04c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x944c04c7, SUC_lexicon_tags_17 },
    { 0x286c04c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c04cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac04d1, SUC_lexicon_tags_1 },
    { 0x58ac04d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb96c04d4, SUC_lexicon_tags_19 },
    { 0xbb0c04d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x90c04d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac04db, SUC_lexicon_tags_45 },
    { 0x552c04db, SUC_lexicon_tags_37 },
    { 0x2e8c04dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb38c04df, SUC_lexicon_tags_12 },
    { 0x25ec04e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2c04e4, SUC_lexicon_tags_61 },
    { 0x2c6c04e5, SUC_lexicon_tags_29 },
    { 0x502c04e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3b4c04e8, SUC_lexicon_tags_32 },
    { 0xabcc04e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x17ec04eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868c04f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0c04f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf52c04f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6ac04fa, SUC_lexicon_tags_26 },
    { 0x128c04fb, SUC_lexicon_tags_27 },
    { 0x512c04fb, SUC_lexicon_tags_7 },
    { 0x556c04fd, SUC_lexicon_tags_15 },
    { 0x548c04fe, SUC_lexicon_tags_17 },
    { 0xf72c04fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4c0505, SUC_lexicon_tags_16 },
    { 0xa1ec0506, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cc050a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x530c050c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4c0510, SUC_lexicon_tags_29 },
    { 0xe6ac0511, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xebcc0513, SUC_lexicon_tags_17 },
    { 0xc6cc0514, SUC_lexicon_tags_6 },
    { 0xc60c0514, SUC_lexicon_tags_23 },
    { 0x30ec0513, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0c051b, SUC_lexicon_tags_29 },
    { 0x2a8c051c, SUC_lexicon_tags_38 },
    { 0xaa2c051c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6c0523, SUC_lexicon_tags_6 },
    { 0xc5ac0524, SUC_lexicon_tags_4 },
    { 0xe38c0525, SUC_lexicon_tags_27 },
    { 0xe8c0526, SUC_lexicon_tags_49 },
    { 0x4e4c0526, SUC_lexicon_tags_2 },
    { 0x300c0528, SUC_lexicon_tags_2 },
    { 0xb8ac0523, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4c052e, SUC_lexicon_tags_11 },
    { 0xd0ac052f, SUC_lexicon_tags_17 },
    { 0xfd4c0530, SUC_lexicon_tags_37 },
    { 0x95ec0530, SUC_lexicon_tags_26 },
    { 0xeccc0532, SUC_lexicon_tags_37 },
    { 0xccc0530, SUC_lexicon_tags_8 },
    { 0xcc8c0533, SUC_lexicon_tags_45 },
    { 0x524c0530, SUC_lexicon_tags_25 },
    { 0x4c0c0536, SUC_lexicon_tags_1 },
    { 0xb52c0537, SUC_lexicon_tags_46 },
    { 0x6dec0538, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7ac053a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0c0541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52cc0543, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ac0546, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cc0549, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2c054d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54c0551, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cc0556, SUC_lexicon_tags_19 },
    { 0x7f4c0557, SUC_lexicon_tags_14 },
    { 0x3e2c0556, SUC_lexicon_tags_17 },
    { 0xe2ec0559, SUC_lexicon_tags_31 },
    { 0x6c4c0559, SUC_lexicon_tags_17 },
    { 0x1b2c055a, SUC_lexicon_tags_14 },
    { 0x9d8c055b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6c055f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffac0561, SUC_lexicon_tags_47 },
    { 0x1e8c0562, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fcc0564, SUC_lexicon_tags_15 },
    { 0xb32c0565, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122c056d, SUC_lexicon_tags_32 },
    { 0xd04c056e, SUC_lexicon_tags_21 },
    { 0x6acc056e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2c0571, SUC_lexicon_tags_12 },
    { 0xf76c0572, SUC_lexicon_tags_17 },
    { 0xfacc0572, SUC_lexicon_tags_32 },
    { 0xa84c0572, SUC_lexicon_tags_60 },
    { 0xfacc0575, SUC_lexicon_tags_17 },
    { 0x3fcc0576, SUC_lexicon_tags_12 },
    { 0xdcec0576, SUC_lexicon_tags_10 },
    { 0x54ec0577, SUC_lexicon_tags_37 },
    { 0x5c6c0574, SUC_lexicon_tags_81 },
    { 0x720c057a, SUC_lexicon_tags_2 },
    { 0x5f2c057a, SUC_lexicon_tags_33 },
    { 0x10cc0579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x652c057e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cc058a, SUC_lexicon_tags_17 },
    { 0xdac058b, SUC_lexicon_tags_43 },
    { 0xb68c058a, SUC_lexicon_tags_12 },
    { 0x7a4c058d, SUC_lexicon_tags_11 },
    { 0xaeec058e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0c0592, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf68c0594, SUC_lexicon_tags_14 },
    { 0x7cc0595, SUC_lexicon_tags_25 },
    { 0x59cc0594, SUC_lexicon_tags_12 },
    { 0x112c0597, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ec059a, SUC_lexicon_tags_14 },
    { 0x400c059b, SUC_lexicon_tags_46 },
    { 0x758c059c, SUC_lexicon_tags_47 },
    { 0x9aac059d, SUC_lexicon_tags_25 },
    { 0x382c059d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eac05a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6c05a4, SUC_lexicon_tags_25 },
    { 0x174c05a5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xe2ac05a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48c05ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08c05b0, SUC_lexicon_tags_37 },
    { 0x974c05b1, SUC_lexicon_tags_37 },
    { 0x8cc05b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cc05b8, SUC_lexicon_tags_33 },
    { 0x5a4c05b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ec05bc, SUC_lexicon_tags_17 },
    { 0xc42c05bc, SUC_lexicon_tags_49 },
    { 0xb92c05be, SUC_lexicon_tags_37 },
    { 0x62c05bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0c05c3, SUC_lexicon_tags_29 },
    { 0xd76c05c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd10c05c6, SUC_lexicon_tags_38 },
    { 0xa5ac05c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd8cc05c9, SUC_lexicon_tags_17 },
    { 0x55cc05c9, SUC_lexicon_tags_28 },
    { 0xa60c05cb, SUC_lexicon_tags_14 },
    { 0xe90c05cc, SUC_lexicon_tags_17 },
    { 0xc74c05cc, SUC_lexicon_tags_3 },
    { 0x642c05ce, SUC_lexicon_tags_25 },
    { 0x636c05cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceac05d5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8c05d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6c05dc, SUC_lexicon_tags_23 },
    { 0x4a8c05dc, SUC_lexicon_tags_95 },
    { 0xffcc05dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4c05e1, SUC_lexicon_tags_9 },
    { 0x14ac05e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x710c05e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36c05e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbeac05eb, SUC_lexicon_tags_6 },
    { 0x4dec05ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408c05f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ec05f6, SUC_lexicon_tags_19 },
    { 0xfc6c05f7, SUC_lexicon_tags_174 },
    { 0xca6c05f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x326c05fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8c05ff, SUC_lexicon_tags_11 },
    { 0x3dec0600, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ac0604, SUC_lexicon_tags_12 },
    { 0xb6c0605, SUC_lexicon_tags_48 },
    { 0x132c0604, SUC_lexicon_tags_47 },
    { 0x91ec0607, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x852c060a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6c060e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ac0611, SUC_lexicon_tags_2 },
    { 0xeec0612, SUC_lexicon_tags_27 },
    { 0x58ac0613, SUC_lexicon_tags_15 },
    { 0xd7ec0613, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4c0617, SUC_lexicon_tags_30 },
    { 0x3bec0618, SUC_lexicon_tags_66 },
    { 0x42cc0619, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa74c061b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0c0623, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dac0626, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2c0629, SUC_lexicon_tags_98 },
    { 0xca4c062a, SUC_lexicon_tags_26 },
    { 0x47cc062a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x236c062e, SUC_lexicon_tags_11 },
    { 0xbdec062f, SUC_lexicon_tags_27 },
    { 0x3ccc0630, SUC_lexicon_tags_17 },
    { 0x5cec0631, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ac0636, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ac0639, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8c0641, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x416c0643, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ec0648, SUC_lexicon_tags_7 },
    { 0x48cc0649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x354c064b, SUC_lexicon_tags_37 },
    { 0xe10c064c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x824c064e, SUC_lexicon_tags_12 },
    { 0xa34c064e, SUC_lexicon_tags_1 },
    { 0x1f2c0650, SUC_lexicon_tags_27 },
    { 0x4a0c064e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2f2c0653, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0c0656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ec065a, SUC_lexicon_tags_23 },
    { 0x240c065b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8c065e, SUC_lexicon_tags_14 },
    { 0x83ec065f, SUC_lexicon_tags_25 },
    { 0xd32c0660, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa74c0662, SUC_lexicon_tags_37 },
    { 0xfe4c0662, SUC_lexicon_tags_30 },
    { 0xd7cc0664, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4c066b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0c066e, SUC_lexicon_tags_61 },
    { 0x36ac066f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ac0675, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6b0c0677, SUC_lexicon_tags_16 },
    { 0x438c0678, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486c067c, SUC_lexicon_tags_37 },
    { 0x4f4c067d, SUC_lexicon_tags_45 },
    { 0x8c8c067d, SUC_lexicon_tags_15 },
    { 0xfb4c067d, SUC_lexicon_tags_12 },
    { 0x4dcc067f, SUC_lexicon_tags_25 },
    { 0xb4cc0681, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cc0685, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaeac0687, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332c068f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcc0693, SUC_lexicon_tags_11 },
    { 0x2b6c0693, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddac0698, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a2c069a, SUC_lexicon_tags_47 },
    { 0xdc6c069b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eec06a1, SUC_lexicon_tags_62 },
    { 0xee8c06a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x452c06a5, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xca8c06a7, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cc06ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0c06b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc06ba, SUC_lexicon_tags_17 },
    { 0x558c06ba, SUC_lexicon_tags_14 },
    { 0x448c06ba, SUC_lexicon_tags_201 },
    { 0xd4ac06bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cc06c1, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xbcec06c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4c06cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238c06d5, SUC_lexicon_tags_17 },
    { 0x736c06d6, SUC_lexicon_tags_23 },
    { 0x170c06d7, SUC_lexicon_tags_17 },
    { 0x51ac06d6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ec06db, SUC_lexicon_tags_13 },
    { 0x658c06db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ac06df, SUC_lexicon_tags_35 },
    { 0x88c06e0, SUC_lexicon_tags_12 },
    { 0x9ac06e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282c06e8, SUC_lexicon_tags_9 },
    { 0x92ac06e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cc06ec, SUC_lexicon_tags_7 },
    { 0xc1ac06ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x236c06ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66c06f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddec06f4, SUC_lexicon_tags_12 },
    { 0x57ac06f5, SUC_lexicon_tags_115 },
    { 0xfdec06f4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x754c06f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccc06fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06c0705, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86c0708, SUC_lexicon_tags_23 },
    { 0x560c0708, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0c070d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4c0710, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaac0714, SUC_lexicon_tags_27 },
    { 0xa6cc0715, SUC_lexicon_tags_43 },
    { 0xe98c0714, SUC_lexicon_tags_7 },
    { 0xd6c0714, SUC_lexicon_tags_31 },
    { 0x136c0714, SUC_lexicon_tags_37 },
    { 0x9bac0719, SUC_lexicon_tags_47 },
    { 0x154c071a, SUC_lexicon_tags_1 },
    { 0x4d8c071a, SUC_lexicon_tags_17 },
    { 0xc24c071c, SUC_lexicon_tags_49 },
    { 0xe2cc071c, SUC_lexicon_tags_1 },
    { 0x2e8c071c, SUC_lexicon_tags_136 },
    { 0x9f6c071f, SUC_lexicon_tags_17 },
    { 0xaccc071f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c0722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x142c0725, SUC_lexicon_tags_38 },
    { 0x508c0725, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34c072b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcc0731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac0737, SUC_lexicon_tags_55 },
    { 0x14c0738, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x33ec073a, SUC_lexicon_tags_15 },
    { 0x202c073b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ac0743, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ac0748, SUC_lexicon_tags_37 },
    { 0x2e6c0748, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82c074c, SUC_lexicon_tags_0 },
    { 0x366c074d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628c0751, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00c0756, SUC_lexicon_tags_37 },
    { 0x5f2c0757, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x876c0759, SUC_lexicon_tags_1 },
    { 0xef4c075a, SUC_lexicon_tags_37 },
    { 0xc40c075b, SUC_lexicon_tags_12 },
    { 0xaac075c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cc0763, SUC_lexicon_tags_63 },
    { 0x692c0764, SUC_lexicon_tags_25 },
    { 0x56cc0765, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec0768, SUC_lexicon_tags_43 },
    { 0xa8ec0769, SUC_lexicon_tags_68 },
    { 0xf54c0768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c076f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cc0772, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x406c0774, SUC_lexicon_tags_21 },
    { 0x478c0775, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xc54c0777, SUC_lexicon_tags_1 },
    { 0x8f0c0778, SUC_lexicon_tags_37 },
    { 0x9ec0778, SUC_lexicon_tags_12 },
    { 0x2d8c077a, SUC_lexicon_tags_15 },
    { 0xfe6c077b, SUC_lexicon_tags_17 },
    { 0x208c077a, SUC_lexicon_tags_55 },
    { 0x37cc077a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ac0780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4c0783, SUC_lexicon_tags_38 },
    { 0x500c0784, SUC_lexicon_tags_38 },
    { 0xfc8c0785, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x890c0788, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6c0790, SUC_lexicon_tags_37 },
    { 0xa32c0790, SUC_lexicon_tags_40 },
    { 0xc18c0790, SUC_lexicon_tags_11 },
    { 0x87ec0790, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd6c0795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c079b, SUC_lexicon_tags_43 },
    { 0xa36c079b, SUC_lexicon_tags_7 },
    { 0x578c079d, SUC_lexicon_tags_31 },
    { 0x4c8c079b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2c07a1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x284c07a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9ec07a5, SUC_lexicon_tags_25 },
    { 0x154c07a6, SUC_lexicon_tags_37 },
    { 0x5f8c07a6, SUC_lexicon_tags_19 },
    { 0x7e0c07a7, SUC_lexicon_tags_25 },
    { 0xb96c07a9, SUC_lexicon_tags_46 },
    { 0x12ac07aa, SUC_lexicon_tags_31 },
    { 0x406c07ab, SUC_lexicon_tags_9 },
    { 0xf62c07a8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8c07af, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150c07b5, SUC_lexicon_tags_49 },
    { 0xb92c07b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x192c07b9, SUC_lexicon_tags_17 },
    { 0x720c07ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2c07c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfacc07c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac07c9, SUC_lexicon_tags_1 },
    { 0x176c07ca, SUC_lexicon_tags_15 },
    { 0x638c07c9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6c07d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4c07d3, SUC_lexicon_tags_15 },
    { 0x192c07d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x84cc07d7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8c07db, SUC_lexicon_tags_60 },
    { 0x76c07db, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cc07e1, SUC_lexicon_tags_1 },
    { 0x98c07e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x614c07e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba8c07e6, SUC_lexicon_tags_21 },
    { 0x9fec07e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x290c07e9, SUC_lexicon_tags_28 },
    { 0xaa4c07ea, SUC_lexicon_tags_29 },
    { 0x7a4c07ea, SUC_lexicon_tags_25 },
    { 0x486c07ec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c07f1, SUC_lexicon_tags_37 },
    { 0xd8ec07f1, SUC_lexicon_tags_37 },
    { 0xab0c07f2, SUC_lexicon_tags_25 },
    { 0x64ac07f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44c07f6, SUC_lexicon_tags_21 },
    { 0xbaec07f7, SUC_lexicon_tags_0 },
    { 0xf7ac07f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ec0802, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x280c0805, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2c080c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5cc080e, SUC_lexicon_tags_55 },
    { 0x170c080f, SUC_lexicon_tags_16 },
    { 0xb6ec0810, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688c0814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60c0818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8c081d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54cc081f, SUC_lexicon_tags_83 },
    { 0xc9ec0820, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x92cc0823, SUC_lexicon_tags_11 },
    { 0xb0c0824, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x254c0827, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc32c0829, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2c082f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fec0832, SUC_lexicon_tags_15 },
    { 0x31cc0833, SUC_lexicon_tags_7 },
    { 0x61ec0834, SUC_lexicon_tags_7 },
    { 0xa4c0834, SUC_lexicon_tags_7 },
    { 0x36cc0835, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08c083b, SUC_lexicon_tags_1 },
    { 0x2ecc083b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x662c083e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2c0844, SUC_lexicon_tags_61 },
    { 0x2f8c0845, SUC_lexicon_tags_0 },
    { 0xdf0c0846, SUC_lexicon_tags_30 },
    { 0x2aec0847, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56c084c, SUC_lexicon_tags_25 },
    { 0xab8c084d, SUC_lexicon_tags_37 },
    { 0xe76c084c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x548c0851, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548c0857, SUC_lexicon_tags_0 },
    { 0x9c4c0858, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4c085b, SUC_lexicon_tags_82 },
    { 0x3a8c085c, SUC_lexicon_tags_15 },
    { 0xba4c085d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffec0862, SUC_lexicon_tags_17 },
    { 0x7f2c0863, SUC_lexicon_tags_64 },
    { 0x2a2c0864, SUC_lexicon_tags_43 },
    { 0xa3cc0865, SUC_lexicon_tags_1 },
    { 0xe64c0866, SUC_lexicon_tags_45 },
    { 0xca2c0865, SUC_lexicon_tags_37 },
    { 0xabec0868, SUC_lexicon_tags_37 },
    { 0xa22c0869, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ac0870, SUC_lexicon_tags_4 },
    { 0x69ac0871, SUC_lexicon_tags_45 },
    { 0xd58c0871, SUC_lexicon_tags_11 },
    { 0x728c0871, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x88ec0875, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf6c0877, SUC_lexicon_tags_17 },
    { 0xa14c0878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ec087c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x194c087e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0c0881, SUC_lexicon_tags_49 },
    { 0x122c0882, SUC_lexicon_tags_37 },
    { 0xc4ec0881, SUC_lexicon_tags_29 },
    { 0xe82c0884, SUC_lexicon_tags_31 },
    { 0xa14c0882, SUC_lexicon_tags_60 },
    { 0x96c0885, SUC_lexicon_tags_9 },
    { 0x66ac0881, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8c088b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cc088f, SUC_lexicon_tags_63 },
    { 0xab6c0890, SUC_lexicon_tags_25 },
    { 0x7b2c0890, SUC_lexicon_tags_1 },
    { 0x638c0890, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cc089d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x834c089f, SUC_lexicon_tags_15 },
    { 0x42c08a0, SUC_lexicon_tags_7 },
    { 0x2d2c08a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2dec08a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06c08a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0ec08a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8c08ac, SUC_lexicon_tags_17 },
    { 0x250c08ac, SUC_lexicon_tags_15 },
    { 0x78ac08ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618c08b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x868c08b4, SUC_lexicon_tags_37 },
    { 0x8d0c08b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccc08c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1ec08c4, SUC_lexicon_tags_15 },
    { 0x40ec08c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcc08c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa30c08ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x470c08cc, SUC_lexicon_tags_7 },
    { 0x6c8c08cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bac08d0, SUC_lexicon_tags_68 },
    { 0x82c08d0, SUC_lexicon_tags_0 },
    { 0xa2cc08d2, SUC_lexicon_tags_17 },
    { 0x6eec08d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x45ec08d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b0c08d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x836c08da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xceac08dc, SUC_lexicon_tags_87 },
    { 0x1dec08dc, SUC_lexicon_tags_60 },
    { 0x882c08dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x678c08e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x392c08e2, SUC_lexicon_tags_11 },
    { 0x4a8c08e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x718c08e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x876c08e8, SUC_lexicon_tags_47 },
    { 0x91ec08e9, SUC_lexicon_tags_17 },
    { 0xf88c08ea, SUC_lexicon_tags_1 },
    { 0x4aec08eb, SUC_lexicon_tags_15 },
    { 0x52cc08eb, SUC_lexicon_tags_28 },
    { 0xbb4c08ed, SUC_lexicon_tags_1 },
    { 0x82ac08ee, SUC_lexicon_tags_11 },
    { 0x71ec08ec, SUC_lexicon_tags_63 },
    { 0x5ecc08ea, SUC_lexicon_tags_7 },
    { 0x114c08f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbec08f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4c08f6, SUC_lexicon_tags_26 },
    { 0x192c08f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0c08fb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f0c08fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0c0903, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f2c0905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20c090a, SUC_lexicon_tags_9 },
    { 0xcdcc090b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74c090e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98c0912, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cc0915, SUC_lexicon_tags_12 },
    { 0xb04c0916, SUC_lexicon_tags_27 },
    { 0xc40c0917, SUC_lexicon_tags_37 },
    { 0x598c0918, SUC_lexicon_tags_11 },
    { 0xa48c0918, SUC_lexicon_tags_0 },
    { 0xd08c091a, SUC_lexicon_tags_178 },
    { 0xc2c091b, SUC_lexicon_tags_25 },
    { 0x5ac091a, SUC_lexicon_tags_17 },
    { 0x894c091d, SUC_lexicon_tags_48 },
    { 0xe4ac091d, SUC_lexicon_tags_12 },
    { 0x11ec0918, SUC_lexicon_tags_3 },
    { 0xd2cc0920, SUC_lexicon_tags_12 },
    { 0x4e4c0920, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x102c0924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79cc0927, SUC_lexicon_tags_11 },
    { 0xb56c0928, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8c092d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12c0930, SUC_lexicon_tags_30 },
    { 0x856c0930, SUC_lexicon_tags_60 },
    { 0x4e6c0932, SUC_lexicon_tags_25 },
    { 0x64ec0933, SUC_lexicon_tags_25 },
    { 0x800c0932, SUC_lexicon_tags_15 },
    { 0x78ac0935, SUC_lexicon_tags_25 },
    { 0x52cc0931, SUC_lexicon_tags_15 },
    { 0x216c0937, SUC_lexicon_tags_37 },
    { 0xd02c0938, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cec093b, SUC_lexicon_tags_33 },
    { 0x24ac093b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x460c093e, SUC_lexicon_tags_23 },
    { 0x7e0c093f, SUC_lexicon_tags_12 },
    { 0x76ec0940, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88c0949, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x918c094b, SUC_lexicon_tags_25 },
    { 0xc8c094c, SUC_lexicon_tags_17 },
    { 0xf4ac094d, SUC_lexicon_tags_7 },
    { 0x98c094d, SUC_lexicon_tags_12 },
    { 0xd76c094f, SUC_lexicon_tags_38 },
    { 0x59ec094c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x476c0953, SUC_lexicon_tags_37 },
    { 0x5fac0954, SUC_lexicon_tags_8 },
    { 0xdaec0955, SUC_lexicon_tags_26 },
    { 0x264c0955, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c095b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fec095d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6c0960, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552c096c, SUC_lexicon_tags_30 },
    { 0x32ac096d, SUC_lexicon_tags_87 },
    { 0xb5ac096d, SUC_lexicon_tags_7 },
    { 0xf6ac096d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0c0973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aec0979, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a0c097b, SUC_lexicon_tags_27 },
    { 0x6cec097c, SUC_lexicon_tags_17 },
    { 0x524c097d, SUC_lexicon_tags_47 },
    { 0x7a2c097d, SUC_lexicon_tags_37 },
    { 0x628c097b, SUC_lexicon_tags_19 },
    { 0xfc2c0980, SUC_lexicon_tags_506 },
    { 0xb04c097b, SUC_lexicon_tags_29 },
    { 0x3e2c097f, SUC_lexicon_tags_12 },
    { 0xb08c0983, SUC_lexicon_tags_31 },
    { 0x914c0983, SUC_lexicon_tags_26 },
    { 0x138c0982, SUC_lexicon_tags_25 },
    { 0xd4cc0981, SUC_lexicon_tags_25 },
    { 0x2c4c0987, SUC_lexicon_tags_5 },
    { 0x14ac0988, SUC_lexicon_tags_21 },
    { 0x76cc0982, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66c098f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x360c0992, SUC_lexicon_tags_8 },
    { 0xa90c0992, SUC_lexicon_tags_27 },
    { 0xee2c0994, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90c0997, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x500c099a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdec099c, SUC_lexicon_tags_17 },
    { 0xfa2c099d, SUC_lexicon_tags_0 },
    { 0xeeac099c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90c09a2, SUC_lexicon_tags_17 },
    { 0xa1ec09a2, SUC_lexicon_tags_60 },
    { 0xcf4c09a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7a6c09a6, SUC_lexicon_tags_12 },
    { 0xc3ac09a7, SUC_lexicon_tags_19 },
    { 0xc6c09a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156c09ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44cc09ae, SUC_lexicon_tags_38 },
    { 0x42ac09af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356c09b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924c09bb, SUC_lexicon_tags_2 },
    { 0x310c09bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6c09bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508c09c3, SUC_lexicon_tags_14 },
    { 0x620c09c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950c09c9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x16cc09cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36c09d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cc09de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3dac09e0, SUC_lexicon_tags_17 },
    { 0x6b4c09e1, SUC_lexicon_tags_27 },
    { 0x8a0c09e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc2c09e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454c09ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbac09ec, SUC_lexicon_tags_2 },
    { 0x83ec09ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ec09f1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08c09f5, SUC_lexicon_tags_12 },
    { 0x9a2c09f5, SUC_lexicon_tags_1 },
    { 0xbb4c09f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0c09fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1aec09fd, SUC_lexicon_tags_0 },
    { 0xd2ec09fd, SUC_lexicon_tags_7 },
    { 0xcf6c09ff, SUC_lexicon_tags_19 },
    { 0x610c0a00, SUC_lexicon_tags_47 },
    { 0x3f0c0a00, SUC_lexicon_tags_14 },
    { 0xb94c0a02, SUC_lexicon_tags_38 },
    { 0x3dcc0a01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x106c0a05, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92c0a09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf8c0a0b, SUC_lexicon_tags_7 },
    { 0x2ecc0a0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ec0a0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecc0a12, SUC_lexicon_tags_4 },
    { 0xae4c0a12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dec0a15, SUC_lexicon_tags_1 },
    { 0x386c0a16, SUC_lexicon_tags_25 },
    { 0x1dcc0a15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2c0a1b, SUC_lexicon_tags_7 },
    { 0x750c0a1c, SUC_lexicon_tags_2 },
    { 0x30ec0a1d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4e0c0a1f, SUC_lexicon_tags_14 },
    { 0x6aec0a20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c6c0a22, SUC_lexicon_tags_46 },
    { 0x45ec0a23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe8c0a25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4c0a2a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x602c0a2d, SUC_lexicon_tags_16 },
    { 0xa20c0a2d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf2c0a30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b2c0a32, SUC_lexicon_tags_15 },
    { 0x144c0a33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeec0a3a, SUC_lexicon_tags_121 },
    { 0xda4c0a3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8cc0a3d, SUC_lexicon_tags_6 },
    { 0x89cc0a3d, SUC_lexicon_tags_17 },
    { 0x36ec0a3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x18cc0a41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x84cc0a44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x680c0a46, SUC_lexicon_tags_25 },
    { 0x9ac0a47, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccc0a4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470c0a51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x698c0a54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb8ec0a56, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x284c0a59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf86c0a5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27ec0a5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ac0a66, SUC_lexicon_tags_61 },
    { 0x314c0a67, SUC_lexicon_tags_23 },
    { 0x536c0a68, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08c0a6d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228c0a7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xafcc0a7c, SUC_lexicon_tags_15 },
    { 0x5d2c0a7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6c0a80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aac0a85, SUC_lexicon_tags_26 },
    { 0x318c0a86, SUC_lexicon_tags_33 },
    { 0x802c0a87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeec0a8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x810c0a8e, SUC_lexicon_tags_23 },
    { 0x18cc0a8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd36c0a91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5cc0a93, SUC_lexicon_tags_10 },
    { 0x96cc0a93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba6c0a96, SUC_lexicon_tags_1 },
    { 0xc18c0a97, SUC_lexicon_tags_65 },
    { 0xbbac0a98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ac0a9f, SUC_lexicon_tags_28 },
    { 0x506c0aa0, SUC_lexicon_tags_7 },
    { 0x618c0aa0, SUC_lexicon_tags_47 },
    { 0xee4c0a9f, SUC_lexicon_tags_37 },
    { 0xc5ac0aa0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64c0aa7, SUC_lexicon_tags_0 },
    { 0x38ec0aa8, SUC_lexicon_tags_15 },
    { 0xc54c0aa9, SUC_lexicon_tags_33 },
    { 0x5cec0aaa, SUC_lexicon_tags_12 },
    { 0x7ccc0aab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb26c0aad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac0ab2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb0c0ab4, SUC_lexicon_tags_43 },
    { 0xea2c0ab5, SUC_lexicon_tags_19 },
    { 0xaccc0ab6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbaec0ab8, SUC_lexicon_tags_25 },
    { 0x648c0ab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102c0ac4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc64c0ac6, SUC_lexicon_tags_49 },
    { 0x5d6c0ac6, SUC_lexicon_tags_103 },
    { 0xcecc0ac8, SUC_lexicon_tags_11 },
    { 0x832c0ac6, SUC_lexicon_tags_12 },
    { 0x3bcc0ac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcc0acd, SUC_lexicon_tags_17 },
    { 0x444c0acd, SUC_lexicon_tags_25 },
    { 0xd00c0acf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390c0ad3, SUC_lexicon_tags_48 },
    { 0x920c0ad3, SUC_lexicon_tags_16 },
    { 0x59ec0ad4, SUC_lexicon_tags_33 },
    { 0x7ac0ad4, SUC_lexicon_tags_2 },
    { 0xe7ec0ad7, SUC_lexicon_tags_14 },
    { 0x4bac0ad8, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x338c0adb, SUC_lexicon_tags_7 },
    { 0xe70c0adc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe00c0ade, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72c0ae5, SUC_lexicon_tags_293 },
    { 0xab8c0ae6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bec0ae8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76c0aed, SUC_lexicon_tags_1 },
    { 0x3b0c0aee, SUC_lexicon_tags_43 },
    { 0xda0c0aef, SUC_lexicon_tags_29 },
    { 0x774c0aed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8c0af6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14c0b03, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2c0b0b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ac0b0e, SUC_lexicon_tags_12 },
    { 0x7a0c0b0f, SUC_lexicon_tags_21 },
    { 0x68ec0b10, SUC_lexicon_tags_0 },
    { 0xe32c0b0e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acc0b14, SUC_lexicon_tags_11 },
    { 0xc4ac0b15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc20c0b17, SUC_lexicon_tags_11 },
    { 0x844c0b18, SUC_lexicon_tags_46 },
    { 0x908c0b19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168c0b1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f4c0b1f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x146c0b22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x626c0b25, SUC_lexicon_tags_40 },
    { 0x82c0b26, SUC_lexicon_tags_49 },
    { 0xa52c0b26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfe8c0b29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328c0b37, SUC_lexicon_tags_17 },
    { 0x262c0b38, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x466c0b3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9fec0b3c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794c0b41, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x33cc0b44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eec0b47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddcc0b49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x656c0b4b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x524c0b4d, SUC_lexicon_tags_50 },
    { 0x844c0b4e, SUC_lexicon_tags_38 },
    { 0x8c0c0b4e, SUC_lexicon_tags_2 },
    { 0x2e4c0b50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0b54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ec0b59, SUC_lexicon_tags_15 },
    { 0x8f4c0b5a, SUC_lexicon_tags_25 },
    { 0x55ec0b5a, SUC_lexicon_tags_14 },
    { 0x962c0b5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdec0b5e, SUC_lexicon_tags_15 },
    { 0xaaec0b5e, SUC_lexicon_tags_8 },
    { 0x9d8c0b5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0c0b67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa24c0b6a, SUC_lexicon_tags_2 },
    { 0xe82c0b6b, SUC_lexicon_tags_31 },
    { 0x790c0b6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586c0b71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e2c0b73, SUC_lexicon_tags_1 },
    { 0xa2ac0b74, SUC_lexicon_tags_28 },
    { 0x566c0b75, SUC_lexicon_tags_64 },
    { 0xd76c0b73, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dac0b7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c0b7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x828c0b80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x94ac0b82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40ac0b84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4c0b87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ac0b8c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x104c0b8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d4c0b90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0c0b94, SUC_lexicon_tags_37 },
    { 0x1d6c0b95, SUC_lexicon_tags_48 },
    { 0x90ec0b95, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x218c0b98, SUC_lexicon_tags_9 },
    { 0xfaac0b99, SUC_lexicon_tags_13 },
    { 0x5e2c0b98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x358c0b9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2c0ba0, SUC_lexicon_tags_7 },
    { 0xa2ac0ba1, SUC_lexicon_tags_32 },
    { 0x3d4c0ba0, SUC_lexicon_tags_21 },
    { 0x67ec0ba0, SUC_lexicon_tags_69 },
    { 0xa76c0ba0, SUC_lexicon_tags_43 },
    { 0x53ac0ba1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfec0ba9, SUC_lexicon_tags_7 },
    { 0x93ec0baa, SUC_lexicon_tags_45 },
    { 0x7aec0baa, SUC_lexicon_tags_25 },
    { 0x8c4c0bab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc0cc0bae, SUC_lexicon_tags_19 },
    { 0x68ec0baf, SUC_lexicon_tags_13 },
    { 0xbd0c0bb0, SUC_lexicon_tags_25 },
    { 0x36c0bb0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xeecc0bb3, SUC_lexicon_tags_60 },
    { 0xc0ac0bb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4c0bb7, SUC_lexicon_tags_28 },
    { 0xe9cc0bb8, SUC_lexicon_tags_25 },
    { 0x6ccc0bb8, SUC_lexicon_tags_39 },
    { 0xcd4c0bb7, SUC_lexicon_tags_26 },
    { 0xdbcc0bb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcc0bc0, SUC_lexicon_tags_75 },
    { 0xa6ac0bc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x930c0bc4, SUC_lexicon_tags_31 },
    { 0x6aec0bc4, SUC_lexicon_tags_8 },
    { 0xf4cc0bc6, SUC_lexicon_tags_19 },
    { 0xce2c0bc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ac0bcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc70c0bcd, SUC_lexicon_tags_28 },
    { 0x396c0bce, SUC_lexicon_tags_6 },
    { 0x698c0bcf, SUC_lexicon_tags_9 },
    { 0x848c0bcd, SUC_lexicon_tags_68 },
    { 0x40c0bd1, SUC_lexicon_tags_65 },
    { 0xea2c0bd2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ac0bd5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4c0bd8, SUC_lexicon_tags_17 },
    { 0xe18c0bd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28c0bdc, SUC_lexicon_tags_17 },
    { 0x844c0bdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10c0be3, SUC_lexicon_tags_201 },
    { 0x814c0be3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb5cc0be6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92cc0be8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eec0beb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa08c0bed, SUC_lexicon_tags_0 },
    { 0xfe4c0bee, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7f0c0bf0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ac0bf6, SUC_lexicon_tags_15 },
    { 0xcb4c0bf6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8c0bfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864c0bfe, SUC_lexicon_tags_0 },
    { 0xf68c0bfe, SUC_lexicon_tags_15 },
    { 0x72ec0c00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb90c0c02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6c0c06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cc0c09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc24c0c0b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x5dcc0c0d, SUC_lexicon_tags_15 },
    { 0xa80c0c0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x576c0c10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x632c0c12, SUC_lexicon_tags_43 },
    { 0xdcac0c12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c0c16, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2bcc0c18, SUC_lexicon_tags_30 },
    { 0x34ac0c18, SUC_lexicon_tags_0 },
    { 0x4b6c0c1a, SUC_lexicon_tags_26 },
    { 0x70ec0c1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe6ec0c1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x402c0c20, SUC_lexicon_tags_14 },
    { 0xf6ac0c21, SUC_lexicon_tags_6 },
    { 0x6d8c0c22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2c0c27, SUC_lexicon_tags_2 },
    { 0x3e2c0c28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x784c0c2b, SUC_lexicon_tags_10 },
    { 0xebec0c2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62c0c30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc0c34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ac0c37, SUC_lexicon_tags_1 },
    { 0x240c0c38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4c0c3c, SUC_lexicon_tags_33 },
    { 0xabcc0c3c, SUC_lexicon_tags_265 },
    { 0x966c0c3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb36c0c40, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x208c0c42, SUC_lexicon_tags_7 },
    { 0x432c0c42, SUC_lexicon_tags_45 },
    { 0xbc8c0c44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcc0c49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20ac0c4b, SUC_lexicon_tags_12 },
    { 0xd8c0c4c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6c0c4f, SUC_lexicon_tags_32 },
    { 0x62cc0c50, SUC_lexicon_tags_49 },
    { 0xa48c0c4f, SUC_lexicon_tags_1 },
    { 0x9cac0c52, SUC_lexicon_tags_60 },
    { 0xcecc0c53, SUC_lexicon_tags_3 },
    { 0xc90c0c54, SUC_lexicon_tags_17 },
    { 0xbc8c0c54, SUC_lexicon_tags_87 },
    { 0x838c0c56, SUC_lexicon_tags_38 },
    { 0xbf4c0c55, SUC_lexicon_tags_1 },
    { 0x46ec0c55, SUC_lexicon_tags_43 },
    { 0xf04c0c54, SUC_lexicon_tags_15 },
    { 0x756c0c5a, SUC_lexicon_tags_33 },
    { 0xf10c0c4f, SUC_lexicon_tags_6 },
    { 0xebac0c5c, SUC_lexicon_tags_15 },
    { 0x52c0c58, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eec0c61, SUC_lexicon_tags_25 },
    { 0x7acc0c62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10c0c67, SUC_lexicon_tags_79 },
    { 0xa96c0c68, SUC_lexicon_tags_60 },
    { 0xb96c0c69, SUC_lexicon_tags_12 },
    { 0xdf0c0c69, SUC_lexicon_tags_11 },
    { 0x29cc0c68, SUC_lexicon_tags_17 },
    { 0xe12c0c6a, SUC_lexicon_tags_263 },
    { 0x92cc0c6d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x114c0c70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x632c0c73, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x166c0c75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebac0c79, SUC_lexicon_tags_7 },
    { 0x1cec0c7a, SUC_lexicon_tags_1 },
    { 0x7fac0c7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a4c0c7d, SUC_lexicon_tags_2 },
    { 0x3f0c0c7d, SUC_lexicon_tags_8 },
    { 0xf22c0c7f, SUC_lexicon_tags_47 },
    { 0x10ac0c80, SUC_lexicon_tags_44 },
    { 0xd38c0c80, SUC_lexicon_tags_68 },
    { 0x3c2c0c82, SUC_lexicon_tags_2 },
    { 0xdb0c0c83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd34c0c85, SUC_lexicon_tags_11 },
    { 0x82cc0c85, SUC_lexicon_tags_1 },
    { 0x6e8c0c86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcc0c8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ec0c8e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888c0c93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66c0c96, SUC_lexicon_tags_7 },
    { 0x2b2c0c96, SUC_lexicon_tags_7 },
    { 0x17ac0c96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe20c0c9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfec0ca1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0c0ca7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x274c0ca9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51cc0cac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cc0cb4, SUC_lexicon_tags_29 },
    { 0x214c0cb5, SUC_lexicon_tags_8 },
    { 0xc76c0cb6, SUC_lexicon_tags_2 },
    { 0x318c0cb7, SUC_lexicon_tags_23 },
    { 0x64cc0cb6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc0cbf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cc0cc4, SUC_lexicon_tags_12 },
    { 0x538c0cc4, SUC_lexicon_tags_17 },
    { 0x160c0cc6, SUC_lexicon_tags_34 },
    { 0x29cc0cc6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8c0ccd, SUC_lexicon_tags_15 },
    { 0x5acc0cce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ec0cd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784c0cd7, SUC_lexicon_tags_7 },
    { 0xffcc0cd8, SUC_lexicon_tags_14 },
    { 0x906c0cd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4c0cde, SUC_lexicon_tags_43 },
    { 0x87ac0cde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x778c0ce1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592c0ce5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2c0cea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ec0cf0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7dac0cf2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10c0cf5, SUC_lexicon_tags_14 },
    { 0xe26c0cf6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecc0cfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x816c0cff, SUC_lexicon_tags_79 },
    { 0xac0cff, SUC_lexicon_tags_7 },
    { 0x25ac0d01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb0c0d03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dac0d06, SUC_lexicon_tags_5 },
    { 0xaa0c0d07, SUC_lexicon_tags_129 },
    { 0x3f4c0d06, SUC_lexicon_tags_7 },
    { 0x266c0d06, SUC_lexicon_tags_37 },
    { 0x372c0d08, SUC_lexicon_tags_13 },
    { 0x55ac0d0b, SUC_lexicon_tags_14 },
    { 0x43cc0d0c, SUC_lexicon_tags_23 },
    { 0x9ec0d0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd36c0d0f, SUC_lexicon_tags_25 },
    { 0x4b6c0d10, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x12c0d12, SUC_lexicon_tags_29 },
    { 0x450c0d13, SUC_lexicon_tags_7 },
    { 0x314c0d14, SUC_lexicon_tags_16 },
    { 0x3b2c0d14, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0c0d19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578c0d1f, SUC_lexicon_tags_21 },
    { 0xce6c0d20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7cc0d22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x124c0d24, SUC_lexicon_tags_11 },
    { 0x584c0d24, SUC_lexicon_tags_55 },
    { 0xdaac0d26, SUC_lexicon_tags_192 },
    { 0x9ccc0d26, SUC_lexicon_tags_17 },
    { 0xe9ec0d27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a8c0d2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcac0d33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecc0d37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x120c0d39, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cc0d40, SUC_lexicon_tags_27 },
    { 0x84ac0d41, SUC_lexicon_tags_26 },
    { 0xc4cc0d42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda8c0d44, SUC_lexicon_tags_33 },
    { 0xcaec0d45, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2c0d49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760c0d4e, SUC_lexicon_tags_37 },
    { 0xc10c0d4f, SUC_lexicon_tags_9 },
    { 0x390c0d50, SUC_lexicon_tags_19 },
    { 0x8d6c0d50, SUC_lexicon_tags_45 },
    { 0x6d4c0d51, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd80c0d54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc08c0d56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2c0d5a, SUC_lexicon_tags_15 },
    { 0x46ac0d5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x220c0d5e, SUC_lexicon_tags_17 },
    { 0x2b6c0d5e, SUC_lexicon_tags_1 },
    { 0xf82c0d5e, SUC_lexicon_tags_19 },
    { 0x1a0c0d60, SUC_lexicon_tags_7 },
    { 0x300c0d62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec4c0d64, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x146c0d66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0c0d6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04c0d70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ac0d73, SUC_lexicon_tags_43 },
    { 0x69ac0d74, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x262c0d76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x806c0d78, SUC_lexicon_tags_2 },
    { 0x176c0d79, SUC_lexicon_tags_63 },
    { 0x73ec0d79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x126c0d7c, SUC_lexicon_tags_37 },
    { 0xe90c0d7d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1e6c0d7f, SUC_lexicon_tags_17 },
    { 0xa5cc0d80, SUC_lexicon_tags_15 },
    { 0x8d4c0d80, SUC_lexicon_tags_26 },
    { 0x3c0c0d81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4c0d86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93ac0d88, SUC_lexicon_tags_29 },
    { 0xb1ec0d89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74c0d8c, SUC_lexicon_tags_11 },
    { 0xf26c0d8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2c0d90, SUC_lexicon_tags_19 },
    { 0x80ac0d91, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966c0d97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcc0d9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6c0d9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cc0da1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x756c0da3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bec0da8, SUC_lexicon_tags_37 },
    { 0x90ac0da9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef0c0dab, SUC_lexicon_tags_7 },
    { 0x802c0dac, SUC_lexicon_tags_28 },
    { 0x8f0c0dad, SUC_lexicon_tags_45 },
    { 0xaccc0dad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0c0db1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x50cc0db3, SUC_lexicon_tags_19 },
    { 0xef0c0db3, SUC_lexicon_tags_38 },
    { 0x2cc0db5, SUC_lexicon_tags_1 },
    { 0x56c0db5, SUC_lexicon_tags_42 },
    { 0x436c0db7, SUC_lexicon_tags_17 },
    { 0x1d8c0db6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x470c0dba, SUC_lexicon_tags_1 },
    { 0xfa2c0dba, SUC_lexicon_tags_27 },
    { 0x4c0dbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88c0dc1, SUC_lexicon_tags_47 },
    { 0x85cc0dc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c0dc6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aac0dc9, SUC_lexicon_tags_38 },
    { 0x6c2c0dc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x41ec0dcc, SUC_lexicon_tags_27 },
    { 0x31ac0dcd, SUC_lexicon_tags_79 },
    { 0x9f4c0dcc, SUC_lexicon_tags_252 },
    { 0xffec0dcf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8c0dd3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x242c0dd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70c0dd9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dec0ddc, SUC_lexicon_tags_16 },
    { 0x30ac0ddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42c0de8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b8c0dea, SUC_lexicon_tags_12 },
    { 0x9d2c0dea, SUC_lexicon_tags_14 },
    { 0xf78c0dec, SUC_lexicon_tags_1 },
    { 0xe72c0ded, SUC_lexicon_tags_25 },
    { 0x464c0dea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x54cc0df0, SUC_lexicon_tags_15 },
    { 0x25ec0df0, SUC_lexicon_tags_26 },
    { 0x95ec0df1, SUC_lexicon_tags_15 },
    { 0x4d6c0df3, SUC_lexicon_tags_82 },
    { 0x708c0df4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ec0df7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cc0dfb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c0e00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eac0e05, SUC_lexicon_tags_11 },
    { 0xeeac0e05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786c0e0a, SUC_lexicon_tags_11 },
    { 0x47cc0e0b, SUC_lexicon_tags_63 },
    { 0x1cac0e0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x502c0e0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f8c0e10, SUC_lexicon_tags_47 },
    { 0x6b2c0e10, SUC_lexicon_tags_37 },
    { 0x55cc0e12, SUC_lexicon_tags_27 },
    { 0xaa2c0e13, SUC_lexicon_tags_9 },
    { 0x814c0e14, SUC_lexicon_tags_29 },
    { 0xaa8c0e15, SUC_lexicon_tags_49 },
    { 0x76cc0e16, SUC_lexicon_tags_15 },
    { 0xfaac0e17, SUC_lexicon_tags_12 },
    { 0xf48c0e18, SUC_lexicon_tags_1 },
    { 0xa76c0e19, SUC_lexicon_tags_37 },
    { 0x9acc0e1a, SUC_lexicon_tags_2 },
    { 0xf62c0e1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1d8c0e1d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ec0e20, SUC_lexicon_tags_47 },
    { 0x9bec0e20, SUC_lexicon_tags_11 },
    { 0x4fcc0e20, SUC_lexicon_tags_7 },
    { 0x570c0e23, SUC_lexicon_tags_28 },
    { 0xe18c0e24, SUC_lexicon_tags_15 },
    { 0x54cc0e25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dac0e27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c0e2b, SUC_lexicon_tags_25 },
    { 0xb04c0e2b, SUC_lexicon_tags_23 },
    { 0xb0ac0e2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x564c0e30, SUC_lexicon_tags_1 },
    { 0xc92c0e31, SUC_lexicon_tags_100 },
    { 0x79cc0e32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8c0e39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x708c0e3b, SUC_lexicon_tags_37 },
    { 0x2d4c0e3b, SUC_lexicon_tags_85 },
    { 0xa26c0e3d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aec0e4b, SUC_lexicon_tags_0 },
    { 0xc5cc0e4b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfe2c0e4e, SUC_lexicon_tags_12 },
    { 0x1f4c0e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0c0e54, SUC_lexicon_tags_26 },
    { 0xdacc0e55, SUC_lexicon_tags_17 },
    { 0x1f2c0e56, SUC_lexicon_tags_7 },
    { 0xd98c0e56, SUC_lexicon_tags_47 },
    { 0x3fcc0e58, SUC_lexicon_tags_68 },
    { 0x5bec0e57, SUC_lexicon_tags_1 },
    { 0xed6c0e5a, SUC_lexicon_tags_15 },
    { 0xb76c0e59, SUC_lexicon_tags_27 },
    { 0xfec0e5c, SUC_lexicon_tags_11 },
    { 0xef8c0e58, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa8cc0e5f, SUC_lexicon_tags_156 },
    { 0x3aec0e60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c0e67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cc0e6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac8c0e6d, SUC_lexicon_tags_2 },
    { 0x5d4c0e6d, SUC_lexicon_tags_37 },
    { 0x394c0e6f, SUC_lexicon_tags_12 },
    { 0xac4c0e70, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb34c0e72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128c0e76, SUC_lexicon_tags_60 },
    { 0xdf8c0e77, SUC_lexicon_tags_68 },
    { 0x3fec0e78, SUC_lexicon_tags_1 },
    { 0x270c0e79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74c0e7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0c0e81, SUC_lexicon_tags_6 },
    { 0xe18c0e81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcccc0e84, SUC_lexicon_tags_489 },
    { 0xc82c0e85, SUC_lexicon_tags_68 },
    { 0x156c0e85, SUC_lexicon_tags_60 },
    { 0xb76c0e87, SUC_lexicon_tags_65 },
    { 0x170c0e88, SUC_lexicon_tags_25 },
    { 0xa8ec0e88, SUC_lexicon_tags_12 },
    { 0x54ac0e88, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c0e8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x426c0e91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc0e95, SUC_lexicon_tags_2 },
    { 0xb3ac0e95, SUC_lexicon_tags_6 },
    { 0x422c0e97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4c0e9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb4c0e9c, SUC_lexicon_tags_6 },
    { 0x9b2c0e9d, SUC_lexicon_tags_7 },
    { 0x6a8c0e9e, SUC_lexicon_tags_43 },
    { 0x48c0e9e, SUC_lexicon_tags_12 },
    { 0xbdec0ea0, SUC_lexicon_tags_7 },
    { 0xa08c0ea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ec0ea5, SUC_lexicon_tags_37 },
    { 0x3a4c0ea6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa0c0ea8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ac0ead, SUC_lexicon_tags_68 },
    { 0xab0c0eae, SUC_lexicon_tags_0 },
    { 0x256c0eaf, SUC_lexicon_tags_2 },
    { 0x166c0eb0, SUC_lexicon_tags_17 },
    { 0x274c0eaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4cc0eb3, SUC_lexicon_tags_7 },
    { 0xeb6c0eb4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23ac0eb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19ec0eb8, SUC_lexicon_tags_0 },
    { 0xa70c0eb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x558c0ebc, SUC_lexicon_tags_15 },
    { 0x1a2c0ebd, SUC_lexicon_tags_127 },
    { 0x83ec0ebe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64c0eca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x32c0ecd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04c0ed0, SUC_lexicon_tags_60 },
    { 0x3e4c0ed1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ec0ed4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330c0ed8, SUC_lexicon_tags_158 },
    { 0x5c8c0ed9, SUC_lexicon_tags_28 },
    { 0x45ec0eda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42c0ee0, SUC_lexicon_tags_25 },
    { 0x91ec0ee0, SUC_lexicon_tags_32 },
    { 0xd64c0ee2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ac0ee8, SUC_lexicon_tags_37 },
    { 0xdd4c0ee9, SUC_lexicon_tags_6 },
    { 0x48c0eea, SUC_lexicon_tags_40 },
    { 0x9f4c0eeb, SUC_lexicon_tags_27 },
    { 0x5f2c0eec, SUC_lexicon_tags_15 },
    { 0x660c0eeb, SUC_lexicon_tags_60 },
    { 0x878c0eee, SUC_lexicon_tags_11 },
    { 0x66ac0eef, SUC_lexicon_tags_83 },
    { 0xd6cc0ef0, SUC_lexicon_tags_11 },
    { 0xbc4c0ef1, SUC_lexicon_tags_34 },
    { 0x3fac0ef1, SUC_lexicon_tags_68 },
    { 0x92ec0ef0, SUC_lexicon_tags_1 },
    { 0x89cc0ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6c0ef6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacc0efb, SUC_lexicon_tags_0 },
    { 0xb42c0efb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cac0f01, SUC_lexicon_tags_1 },
    { 0x944c0f02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aec0f08, SUC_lexicon_tags_15 },
    { 0xc1ac0f09, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xad0c0f0b, SUC_lexicon_tags_8 },
    { 0xaaec0f0c, SUC_lexicon_tags_7 },
    { 0x83ac0f0d, SUC_lexicon_tags_7 },
    { 0x758c0f0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98c0f11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cc0f14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x896c0f17, SUC_lexicon_tags_37 },
    { 0x294c0f17, SUC_lexicon_tags_17 },
    { 0xe4ec0f18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2c0f1d, SUC_lexicon_tags_7 },
    { 0x28cc0f1e, SUC_lexicon_tags_87 },
    { 0xd2ac0f1d, SUC_lexicon_tags_17 },
    { 0x8c4c0f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcc0f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68ac0f28, SUC_lexicon_tags_27 },
    { 0xa30c0f29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68c0f31, SUC_lexicon_tags_25 },
    { 0xb6ac0f32, SUC_lexicon_tags_1 },
    { 0xac0f32, SUC_lexicon_tags_14 },
    { 0xcdec0f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcacc0f37, SUC_lexicon_tags_9 },
    { 0xe88c0f38, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724c0f3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aac0f3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2c0f42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc46c0f44, SUC_lexicon_tags_17 },
    { 0xa62c0f45, SUC_lexicon_tags_23 },
    { 0xd6ac0f46, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x998c0f49, SUC_lexicon_tags_9 },
    { 0x4d4c0f4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60c0f4d, SUC_lexicon_tags_27 },
    { 0x828c0f4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaac0f50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x46cc0f52, SUC_lexicon_tags_25 },
    { 0xdccc0f53, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4c0f59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538c0f5d, SUC_lexicon_tags_2 },
    { 0xc0cc0f5e, SUC_lexicon_tags_5 },
    { 0x4a2c0f5f, SUC_lexicon_tags_12 },
    { 0x3e6c0f60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52ec0f62, SUC_lexicon_tags_34 },
    { 0xa4ec0f63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x428c0f65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x728c0f68, SUC_lexicon_tags_13 },
    { 0x8aac0f69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18c0f6e, SUC_lexicon_tags_12 },
    { 0x3b6c0f6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4c0f72, SUC_lexicon_tags_37 },
    { 0x452c0f73, SUC_lexicon_tags_7 },
    { 0xa66c0f74, SUC_lexicon_tags_43 },
    { 0xde2c0f75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72c0f78, SUC_lexicon_tags_7 },
    { 0xfd0c0f79, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0c0f7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936c0f82, SUC_lexicon_tags_17 },
    { 0x522c0f83, SUC_lexicon_tags_17 },
    { 0x646c0f84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cc0f89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaeac0f8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c6c0f8d, SUC_lexicon_tags_116 },
    { 0x4c6c0f8e, SUC_lexicon_tags_37 },
    { 0xe66c0f8d, SUC_lexicon_tags_47 },
    { 0xdcac0f90, SUC_lexicon_tags_6 },
    { 0x40ec0f8e, SUC_lexicon_tags_48 },
    { 0x9bec0f92, SUC_lexicon_tags_17 },
    { 0x16c0f92, SUC_lexicon_tags_63 },
    { 0xe50c0f91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb2ac0f96, SUC_lexicon_tags_26 },
    { 0xd54c0f96, SUC_lexicon_tags_2 },
    { 0x4d2c0f98, SUC_lexicon_tags_12 },
    { 0xab4c0f98, SUC_lexicon_tags_21 },
    { 0x3fac0f9a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6c0f9d, SUC_lexicon_tags_17 },
    { 0xe62c0f9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ec0fa5, SUC_lexicon_tags_31 },
    { 0x68cc0fa6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacc0fae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58c0fb3, SUC_lexicon_tags_37 },
    { 0x470c0fb4, SUC_lexicon_tags_29 },
    { 0x6d4c0fb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x832c0fb8, SUC_lexicon_tags_1 },
    { 0x5ec0fb9, SUC_lexicon_tags_26 },
    { 0x5a4c0fba, SUC_lexicon_tags_14 },
    { 0x74ec0fb9, SUC_lexicon_tags_17 },
    { 0xa80c0fb8, SUC_lexicon_tags_12 },
    { 0x252c0fbd, SUC_lexicon_tags_1 },
    { 0xf68c0fbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74c0fc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x826c0fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2c0fc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf4c0fc8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x324c0fca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x608c0fcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2fec0fce, SUC_lexicon_tags_26 },
    { 0x152c0fcf, SUC_lexicon_tags_37 },
    { 0xf12c0fd0, SUC_lexicon_tags_15 },
    { 0xdc4c0fd1, SUC_lexicon_tags_12 },
    { 0xec8c0fd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a8c0fd4, SUC_lexicon_tags_88 },
    { 0x32c0fd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8c0fd8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xccac0fdb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18cc0fdd, SUC_lexicon_tags_7 },
    { 0x2cec0fdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c0fe2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x88cc0fe4, SUC_lexicon_tags_14 },
    { 0xe84c0fe4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x514c0fe7, SUC_lexicon_tags_27 },
    { 0x9bcc0fe8, SUC_lexicon_tags_34 },
    { 0xde4c0fe9, SUC_lexicon_tags_33 },
    { 0xeb2c0fe7, SUC_lexicon_tags_65 },
    { 0x226c0feb, SUC_lexicon_tags_11 },
    { 0x132c0fec, SUC_lexicon_tags_17 },
    { 0xb58c0fea, SUC_lexicon_tags_27 },
    { 0x2a6c0fe7, SUC_lexicon_tags_7 },
    { 0xa70c0fef, SUC_lexicon_tags_43 },
    { 0x768c0fe7, SUC_lexicon_tags_2 },
    { 0x966c0fe7, SUC_lexicon_tags_7 },
    { 0x1a8c0fec, SUC_lexicon_tags_12 },
    { 0x318c0ff1, SUC_lexicon_tags_29 },
    { 0x636c0ff4, SUC_lexicon_tags_38 },
    { 0xb60c0fec, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22c0ff9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x178c0ffb, SUC_lexicon_tags_25 },
    { 0x262c0ffb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2c1008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898c100c, SUC_lexicon_tags_320 },
    { 0x7b4c100c, SUC_lexicon_tags_64 },
    { 0x8b0c100e, SUC_lexicon_tags_12 },
    { 0xb12c100f, SUC_lexicon_tags_17 },
    { 0x5f0c100f, SUC_lexicon_tags_65 },
    { 0x314c1011, SUC_lexicon_tags_0 },
    { 0xa92c1012, SUC_lexicon_tags_25 },
    { 0xc5ac1011, SUC_lexicon_tags_17 },
    { 0x342c1014, SUC_lexicon_tags_2 },
    { 0xb0cc1012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcac101a, SUC_lexicon_tags_25 },
    { 0x222c101a, SUC_lexicon_tags_45 },
    { 0xf0cc101c, SUC_lexicon_tags_12 },
    { 0x7a8c101d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aec1021, SUC_lexicon_tags_62 },
    { 0x926c1022, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64c1025, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0c1029, SUC_lexicon_tags_37 },
    { 0x658c102a, SUC_lexicon_tags_45 },
    { 0x718c102b, SUC_lexicon_tags_44 },
    { 0xc8ec102c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468c1030, SUC_lexicon_tags_37 },
    { 0xc1031, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732c1037, SUC_lexicon_tags_2 },
    { 0xd8cc1037, SUC_lexicon_tags_1 },
    { 0x70cc1037, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4c103b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea4c103d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16c103f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94c1046, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x996c1048, SUC_lexicon_tags_15 },
    { 0xb3ec1049, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x90c104b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cc104f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96c1052, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308c1058, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fec105b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52c105f, SUC_lexicon_tags_63 },
    { 0xd3cc1060, SUC_lexicon_tags_14 },
    { 0xc2ec1061, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x550c1063, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52cc1065, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6c1068, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8c106d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbaec106f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6c1072, SUC_lexicon_tags_28 },
    { 0xca4c1072, SUC_lexicon_tags_11 },
    { 0x7b4c1074, SUC_lexicon_tags_43 },
    { 0x55ac1075, SUC_lexicon_tags_40 },
    { 0x836c1076, SUC_lexicon_tags_28 },
    { 0xb9cc1077, SUC_lexicon_tags_42 },
    { 0x96ac1077, SUC_lexicon_tags_21 },
    { 0x5f0c1076, SUC_lexicon_tags_38 },
    { 0x270c1079, SUC_lexicon_tags_61 },
    { 0x972c107b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ac107e, SUC_lexicon_tags_25 },
    { 0xae2c107e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6c1082, SUC_lexicon_tags_15 },
    { 0x12ec1083, SUC_lexicon_tags_7 },
    { 0x164c1082, SUC_lexicon_tags_2 },
    { 0x56c1085, SUC_lexicon_tags_11 },
    { 0x37cc1086, SUC_lexicon_tags_12 },
    { 0xcacc1085, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x29ac1089, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740c108d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd66c108f, SUC_lexicon_tags_33 },
    { 0x57ac1090, SUC_lexicon_tags_38 },
    { 0xc2c1090, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72c1095, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6c109d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ec10a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x424c10a6, SUC_lexicon_tags_18 },
    { 0x720c10a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x730c10aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x166c10ad, SUC_lexicon_tags_23 },
    { 0x9f6c10ae, SUC_lexicon_tags_19 },
    { 0xf32c10af, SUC_lexicon_tags_182 },
    { 0xa2ac10b0, SUC_lexicon_tags_35 },
    { 0xf04c10b0, SUC_lexicon_tags_31 },
    { 0x88ec10b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82c10b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0c10ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfac10be, SUC_lexicon_tags_152 },
    { 0xf68c10bf, SUC_lexicon_tags_29 },
    { 0x92ac10bf, SUC_lexicon_tags_11 },
    { 0x112c10bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x368c10c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d0c10c5, SUC_lexicon_tags_15 },
    { 0x20c10c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x828c10c9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2c10ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x288c10d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6c10d3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6bac10d5, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0c10d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x628c10db, SUC_lexicon_tags_1 },
    { 0xb04c10db, SUC_lexicon_tags_7 },
    { 0xc74c10dc, SUC_lexicon_tags_25 },
    { 0xd80c10dc, SUC_lexicon_tags_4 },
    { 0x1c0c10df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682c10e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe46c10ea, SUC_lexicon_tags_17 },
    { 0x50cc10eb, SUC_lexicon_tags_49 },
    { 0xc94c10ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5ac10ee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870c10f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec10f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d8c10f8, SUC_lexicon_tags_49 },
    { 0x63ec10f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf9ec10fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x124c10fd, SUC_lexicon_tags_18 },
    { 0xf28c10fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa2c1100, SUC_lexicon_tags_4 },
    { 0x34c1100, SUC_lexicon_tags_55 },
    { 0x644c1102, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86c1105, SUC_lexicon_tags_49 },
    { 0x26cc1105, SUC_lexicon_tags_12 },
    { 0x2a6c1105, SUC_lexicon_tags_32 },
    { 0x446c1107, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b8c110a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcc110f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xae6c1111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0c1115, SUC_lexicon_tags_25 },
    { 0x578c1116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ec1119, SUC_lexicon_tags_83 },
    { 0x874c111a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684c111e, SUC_lexicon_tags_7 },
    { 0xbd2c111f, SUC_lexicon_tags_171 },
    { 0x330c1120, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34c1124, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ac1127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ac112c, SUC_lexicon_tags_49 },
    { 0x67cc112d, SUC_lexicon_tags_13 },
    { 0xfa2c112e, SUC_lexicon_tags_63 },
    { 0xf48c112e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafac1131, SUC_lexicon_tags_333 },
    { 0xf2cc1131, SUC_lexicon_tags_29 },
    { 0x746c1131, SUC_lexicon_tags_185 },
    { 0x112c1134, SUC_lexicon_tags_12 },
    { 0x766c1134, SUC_lexicon_tags_15 },
    { 0x322c1136, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0c1138, SUC_lexicon_tags_15 },
    { 0x94ac1139, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb4c113b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cc1140, SUC_lexicon_tags_17 },
    { 0x3f4c1141, SUC_lexicon_tags_38 },
    { 0x590c1142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b6c1144, SUC_lexicon_tags_17 },
    { 0xbb4c1145, SUC_lexicon_tags_12 },
    { 0xabec1146, SUC_lexicon_tags_30 },
    { 0xccc1146, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x462c114a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecec1150, SUC_lexicon_tags_15 },
    { 0xdd6c1150, SUC_lexicon_tags_12 },
    { 0x83cc1152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1ec1154, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4c1157, SUC_lexicon_tags_6 },
    { 0xa5ec1157, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36c115c, SUC_lexicon_tags_26 },
    { 0x5cc115c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe00c115f, SUC_lexicon_tags_49 },
    { 0xe7ac115f, SUC_lexicon_tags_28 },
    { 0xb94c1161, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2c1164, SUC_lexicon_tags_37 },
    { 0x24ac1165, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560c116b, SUC_lexicon_tags_4 },
    { 0x856c116b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2c116f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4c1173, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd58c1175, SUC_lexicon_tags_26 },
    { 0xd18c1175, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6c117a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd94c117c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6c1180, SUC_lexicon_tags_37 },
    { 0xb98c1181, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20c1185, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940c118b, SUC_lexicon_tags_66 },
    { 0x4fcc118c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04c1190, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac1195, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c119e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6f6c11a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22c11a6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x728c11a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd1ec11ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1ac11ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ec11af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76c11b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0c11b9, SUC_lexicon_tags_17 },
    { 0xb36c11ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932c11be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa66c11c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aac11c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x88cc11c6, SUC_lexicon_tags_28 },
    { 0xb6ec11c7, SUC_lexicon_tags_8 },
    { 0x1ac11c7, SUC_lexicon_tags_7 },
    { 0xd60c11c9, SUC_lexicon_tags_178 },
    { 0x24ac11ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x552c11cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546c11d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8c11d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0c11e5, SUC_lexicon_tags_1 },
    { 0xec2c11e6, SUC_lexicon_tags_14 },
    { 0x8a0c11e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4c11ea, SUC_lexicon_tags_7 },
    { 0x682c11ea, SUC_lexicon_tags_11 },
    { 0xa72c11ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4ac11ee, SUC_lexicon_tags_43 },
    { 0xae4c11ef, SUC_lexicon_tags_12 },
    { 0xac2c11ef, SUC_lexicon_tags_12 },
    { 0x8b2c11ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa8c11f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc28c11f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc11f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844c11ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x446c1201, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66c1207, SUC_lexicon_tags_34 },
    { 0x6cec1207, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534c120c, SUC_lexicon_tags_12 },
    { 0x400c120c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x684c1210, SUC_lexicon_tags_8 },
    { 0x98ac1211, SUC_lexicon_tags_25 },
    { 0x5e6c1211, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe30c1214, SUC_lexicon_tags_97 },
    { 0xa80c1215, SUC_lexicon_tags_15 },
    { 0xf84c1216, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcc121a, SUC_lexicon_tags_2 },
    { 0xae6c121b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x514c121d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52c1221, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bec1223, SUC_lexicon_tags_17 },
    { 0x208c1224, SUC_lexicon_tags_38 },
    { 0xb4cc1224, SUC_lexicon_tags_14 },
    { 0xb54c1225, SUC_lexicon_tags_43 },
    { 0x3b2c1224, SUC_lexicon_tags_29 },
    { 0x4f4c1228, SUC_lexicon_tags_60 },
    { 0x35cc1228, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2c122c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac1234, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6c1237, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabec1239, SUC_lexicon_tags_17 },
    { 0xe90c123a, SUC_lexicon_tags_38 },
    { 0x7a8c123b, SUC_lexicon_tags_21 },
    { 0xb04c123c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ac1240, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6e2c1242, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaac1244, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63cc1246, SUC_lexicon_tags_812 },
    { 0xefec1247, SUC_lexicon_tags_12 },
    { 0x28c1247, SUC_lexicon_tags_7 },
    { 0x280c1249, SUC_lexicon_tags_15 },
    { 0x588c124a, SUC_lexicon_tags_15 },
    { 0x616c124a, SUC_lexicon_tags_34 },
    { 0xe54c124c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x474c124f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1ac1251, SUC_lexicon_tags_25 },
    { 0xa74c1251, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cc1259, SUC_lexicon_tags_15 },
    { 0x54cc125a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c1262, SUC_lexicon_tags_2 },
    { 0xc04c1262, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x592c1266, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaec1269, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304c126e, SUC_lexicon_tags_68 },
    { 0x6b8c126e, SUC_lexicon_tags_11 },
    { 0x600c126f, SUC_lexicon_tags_1 },
    { 0x9e0c1271, SUC_lexicon_tags_11 },
    { 0x7a6c1271, SUC_lexicon_tags_30 },
    { 0xafac1273, SUC_lexicon_tags_37 },
    { 0x79cc1274, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x754c1277, SUC_lexicon_tags_17 },
    { 0x352c1278, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676c127c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34c1280, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cc1285, SUC_lexicon_tags_68 },
    { 0x8dcc1285, SUC_lexicon_tags_17 },
    { 0x548c1285, SUC_lexicon_tags_23 },
    { 0xd9ac1288, SUC_lexicon_tags_19 },
    { 0xfcac1289, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ec1296, SUC_lexicon_tags_28 },
    { 0x95cc1297, SUC_lexicon_tags_79 },
    { 0xe02c1298, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcecc129b, SUC_lexicon_tags_33 },
    { 0x152c129c, SUC_lexicon_tags_47 },
    { 0xffac129c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0c12a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ac12a6, SUC_lexicon_tags_17 },
    { 0x9e0c12a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3dcc12a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16c12ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81cc12af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444c12b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a4c12b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x312c12b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2c12bb, SUC_lexicon_tags_21 },
    { 0xb00c12bc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168c12c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac12cc, SUC_lexicon_tags_17 },
    { 0x5c4c12cd, SUC_lexicon_tags_49 },
    { 0xfa4c12ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78c12d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ac12d8, SUC_lexicon_tags_61 },
    { 0xe2ac12d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeac12db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde4c12dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeaac12df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50c12e6, SUC_lexicon_tags_37 },
    { 0x7bac12e7, SUC_lexicon_tags_33 },
    { 0x45ec12e8, SUC_lexicon_tags_1 },
    { 0x1c4c12e6, SUC_lexicon_tags_20 },
    { 0xf16c12e7, SUC_lexicon_tags_8 },
    { 0x338c12ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c2c12ed, SUC_lexicon_tags_7 },
    { 0xb9ec12ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x552c12f0, SUC_lexicon_tags_34 },
    { 0xae8c12f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x242c12f3, SUC_lexicon_tags_45 },
    { 0x5fcc12f4, SUC_lexicon_tags_1 },
    { 0xe54c12f4, SUC_lexicon_tags_25 },
    { 0x342c12f3, SUC_lexicon_tags_14 },
    { 0x204c12f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88c12fc, SUC_lexicon_tags_12 },
    { 0xe4ec12fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd32c12ff, SUC_lexicon_tags_11 },
    { 0x388c12ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ec1304, SUC_lexicon_tags_84 },
    { 0xf6cc1304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792c130b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28c1310, SUC_lexicon_tags_14 },
    { 0xf06c1311, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4c1317, SUC_lexicon_tags_17 },
    { 0xde8c1318, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8fac131a, SUC_lexicon_tags_25 },
    { 0xe52c131b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c131e, SUC_lexicon_tags_13 },
    { 0xe3ec131e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cac1322, SUC_lexicon_tags_4 },
    { 0x5ccc1322, SUC_lexicon_tags_37 },
    { 0x3aec1324, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x596c1326, SUC_lexicon_tags_23 },
    { 0x41ec1326, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174c132c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x194c132f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2c1333, SUC_lexicon_tags_12 },
    { 0x358c1333, SUC_lexicon_tags_17 },
    { 0x35cc1333, SUC_lexicon_tags_129 },
    { 0x6b4c1335, SUC_lexicon_tags_12 },
    { 0x68c1335, SUC_lexicon_tags_39 },
    { 0x626c1338, SUC_lexicon_tags_63 },
    { 0x77ac1337, SUC_lexicon_tags_37 },
    { 0x768c133a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccc133d, SUC_lexicon_tags_17 },
    { 0x42c133e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76c1341, SUC_lexicon_tags_26 },
    { 0x63ac1342, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x668c1344, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcc1347, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4bcc1349, SUC_lexicon_tags_30 },
    { 0xcf0c134a, SUC_lexicon_tags_7 },
    { 0x666c134b, SUC_lexicon_tags_46 },
    { 0x664c134c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92c1351, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ac1354, SUC_lexicon_tags_15 },
    { 0x954c1354, SUC_lexicon_tags_25 },
    { 0xf30c1354, SUC_lexicon_tags_49 },
    { 0x900c1357, SUC_lexicon_tags_27 },
    { 0x946c1358, SUC_lexicon_tags_17 },
    { 0xc58c1359, SUC_lexicon_tags_15 },
    { 0x1eac1357, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x992c135c, SUC_lexicon_tags_39 },
    { 0xa1ec135d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bac1362, SUC_lexicon_tags_12 },
    { 0x622c1363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106c1367, SUC_lexicon_tags_6 },
    { 0x9eac1368, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5a4c136a, SUC_lexicon_tags_1 },
    { 0xa2cc136b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x126c136d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4c1373, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x778c1375, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ec1378, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf42c137a, SUC_lexicon_tags_84 },
    { 0xacec137b, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2c1385, SUC_lexicon_tags_37 },
    { 0xf30c1385, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88c1389, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94ec138b, SUC_lexicon_tags_12 },
    { 0xc98c138b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x500c138e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe42c1391, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ac1395, SUC_lexicon_tags_43 },
    { 0xdd0c1396, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x72ac1398, SUC_lexicon_tags_6 },
    { 0x3a4c1398, SUC_lexicon_tags_26 },
    { 0x1a0c139a, SUC_lexicon_tags_37 },
    { 0x730c139b, SUC_lexicon_tags_87 },
    { 0x1e6c139b, SUC_lexicon_tags_38 },
    { 0xf4cc139d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb76c139f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786c13a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b0c13aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x356c13ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554c13b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4c13b7, SUC_lexicon_tags_17 },
    { 0xaacc13b8, SUC_lexicon_tags_17 },
    { 0x712c13b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8fcc13bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x318c13bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ac13c1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8fcc13c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x776c13c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4c13cb, SUC_lexicon_tags_0 },
    { 0x4ac13cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c13d3, SUC_lexicon_tags_15 },
    { 0x26ac13d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc2c13d6, SUC_lexicon_tags_7 },
    { 0xe24c13d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64cc13da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x262c13dc, SUC_lexicon_tags_31 },
    { 0x9d8c13dd, SUC_lexicon_tags_25 },
    { 0x14ec13de, SUC_lexicon_tags_0 },
    { 0x4a6c13df, SUC_lexicon_tags_31 },
    { 0xf5ac13de, SUC_lexicon_tags_26 },
    { 0x9cc13e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6c13e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x386c13e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e6c13e9, SUC_lexicon_tags_88 },
    { 0xe8cc13e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538c13f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74c13f5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfac13f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23cc13fa, SUC_lexicon_tags_15 },
    { 0x3dec13fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d0c13fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec1401, SUC_lexicon_tags_21 },
    { 0x65ec1402, SUC_lexicon_tags_21 },
    { 0xe78c1402, SUC_lexicon_tags_15 },
    { 0x41cc1402, SUC_lexicon_tags_108 },
    { 0x8ec1401, SUC_lexicon_tags_55 },
    { 0xa70c1403, SUC_lexicon_tags_4 },
    { 0x63ac1407, SUC_lexicon_tags_45 },
    { 0xa20c1408, SUC_lexicon_tags_37 },
    { 0x718c1408, SUC_lexicon_tags_21 },
    { 0x1d8c140a, SUC_lexicon_tags_17 },
    { 0x220c140b, SUC_lexicon_tags_11 },
    { 0x18c140b, SUC_lexicon_tags_27 },
    { 0x486c140a, SUC_lexicon_tags_30 },
    { 0xd58c1409, SUC_lexicon_tags_2 },
    { 0xa9cc140b, SUC_lexicon_tags_37 },
    { 0x290c1408, SUC_lexicon_tags_7 },
    { 0xa0ac1411, SUC_lexicon_tags_14 },
    { 0x502c1412, SUC_lexicon_tags_27 },
    { 0x728c1413, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cc1419, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ec141c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3bcc141e, SUC_lexicon_tags_65 },
    { 0x58ac141f, SUC_lexicon_tags_6 },
    { 0x492c1420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe74c1422, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ec142b, SUC_lexicon_tags_4 },
    { 0x5e6c142c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68c1432, SUC_lexicon_tags_29 },
    { 0x89cc1433, SUC_lexicon_tags_87 },
    { 0x3f0c1433, SUC_lexicon_tags_33 },
    { 0x668c1432, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcc143a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2c143e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cc1441, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e8c1443, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8c1449, SUC_lexicon_tags_25 },
    { 0x350c144a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ec144f, SUC_lexicon_tags_37 },
    { 0xb28c1450, SUC_lexicon_tags_49 },
    { 0x696c1451, SUC_lexicon_tags_45 },
    { 0xe9cc1452, SUC_lexicon_tags_1 },
    { 0x9aac1451, SUC_lexicon_tags_151 },
    { 0xf8c1454, SUC_lexicon_tags_6 },
    { 0x3dec1454, SUC_lexicon_tags_37 },
    { 0xdccc1456, SUC_lexicon_tags_2 },
    { 0xd10c1456, SUC_lexicon_tags_37 },
    { 0x484c1458, SUC_lexicon_tags_10 },
    { 0x8bec1454, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x794c145b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228c1461, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb06c1463, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdec1466, SUC_lexicon_tags_16 },
    { 0x3f8c1466, SUC_lexicon_tags_17 },
    { 0x1a6c1466, SUC_lexicon_tags_33 },
    { 0x24ac1467, SUC_lexicon_tags_45 },
    { 0x7cec146a, SUC_lexicon_tags_43 },
    { 0xd1ec146b, SUC_lexicon_tags_27 },
    { 0xdcc146c, SUC_lexicon_tags_2 },
    { 0x85ac146c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc1470, SUC_lexicon_tags_87 },
    { 0x6e0c1471, SUC_lexicon_tags_61 },
    { 0x74cc1471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c1475, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cc1479, SUC_lexicon_tags_12 },
    { 0x6bec1479, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9d6c147c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4c147f, SUC_lexicon_tags_92 },
    { 0x46ac1480, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x496c1483, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554c1487, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x220c1489, SUC_lexicon_tags_49 },
    { 0x13ec148a, SUC_lexicon_tags_12 },
    { 0xee2c148b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x494c148e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86c1492, SUC_lexicon_tags_17 },
    { 0x762c1492, SUC_lexicon_tags_33 },
    { 0x2c4c1492, SUC_lexicon_tags_33 },
    { 0xea8c1495, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c149a, SUC_lexicon_tags_28 },
    { 0xe34c149b, SUC_lexicon_tags_38 },
    { 0x47ec149b, SUC_lexicon_tags_83 },
    { 0x598c149b, SUC_lexicon_tags_7 },
    { 0xff0c149e, SUC_lexicon_tags_28 },
    { 0x364c149e, SUC_lexicon_tags_22 },
    { 0x9a4c149e, SUC_lexicon_tags_3 },
    { 0xbccc14a1, SUC_lexicon_tags_37 },
    { 0x83ec14a1, SUC_lexicon_tags_55 },
    { 0xf1ac14a3, SUC_lexicon_tags_14 },
    { 0x47ac149f, SUC_lexicon_tags_18 },
    { 0xd38c14a5, SUC_lexicon_tags_14 },
    { 0xdf6c14a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeaec14a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf80c14aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcc14ad, SUC_lexicon_tags_30 },
    { 0x68cc14ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852c14b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912c14b7, SUC_lexicon_tags_1 },
    { 0xcb0c14b7, SUC_lexicon_tags_26 },
    { 0xbcc14b7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac14be, SUC_lexicon_tags_12 },
    { 0x714c14bf, SUC_lexicon_tags_25 },
    { 0x256c14c0, SUC_lexicon_tags_25 },
    { 0x88ec14c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba8c14c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x482c14c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8c14c8, SUC_lexicon_tags_37 },
    { 0xaaac14c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ac14ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10c14d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xac6c14d4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6ec14d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadac14dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ac14e7, SUC_lexicon_tags_2 },
    { 0x21ec14e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89cc14ea, SUC_lexicon_tags_17 },
    { 0xcdcc14ea, SUC_lexicon_tags_0 },
    { 0xddac14ec, SUC_lexicon_tags_30 },
    { 0xecc14ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcdac14ef, SUC_lexicon_tags_29 },
    { 0x338c14f0, SUC_lexicon_tags_31 },
    { 0xcd4c14ef, SUC_lexicon_tags_14 },
    { 0x8d2c14f2, SUC_lexicon_tags_27 },
    { 0xc80c14f0, SUC_lexicon_tags_37 },
    { 0x2c2c14f4, SUC_lexicon_tags_38 },
    { 0x1fcc14f1, SUC_lexicon_tags_37 },
    { 0x5ccc14f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee6c14f8, SUC_lexicon_tags_19 },
    { 0xf82c14f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ec14fc, SUC_lexicon_tags_91 },
    { 0xe12c14fd, SUC_lexicon_tags_87 },
    { 0x672c14fc, SUC_lexicon_tags_0 },
    { 0x71cc14ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140c1506, SUC_lexicon_tags_7 },
    { 0xa5cc1506, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dec150a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568c1513, SUC_lexicon_tags_25 },
    { 0x640c1514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48c151c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6c1520, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ac1523, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306c1527, SUC_lexicon_tags_25 },
    { 0xb1ec1528, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x586c152b, SUC_lexicon_tags_23 },
    { 0x56c152c, SUC_lexicon_tags_14 },
    { 0x4e4c152c, SUC_lexicon_tags_30 },
    { 0x672c152e, SUC_lexicon_tags_27 },
    { 0x804c152f, SUC_lexicon_tags_17 },
    { 0x742c152e, SUC_lexicon_tags_25 },
    { 0xb3ec152e, SUC_lexicon_tags_46 },
    { 0xed0c152b, SUC_lexicon_tags_2 },
    { 0xcb4c152f, SUC_lexicon_tags_28 },
    { 0x698c1534, SUC_lexicon_tags_17 },
    { 0xc68c1534, SUC_lexicon_tags_30 },
    { 0xed4c1536, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eac153b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac1548, SUC_lexicon_tags_34 },
    { 0xc1cc1548, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ec154c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98c1550, SUC_lexicon_tags_1 },
    { 0x68cc1551, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58cc1554, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x324c1556, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00c1559, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf30c155b, SUC_lexicon_tags_1 },
    { 0x8c0c155c, SUC_lexicon_tags_987 },
    { 0x3c4c155c, SUC_lexicon_tags_25 },
    { 0x490c155d, SUC_lexicon_tags_48 },
    { 0x3b6c155f, SUC_lexicon_tags_30 },
    { 0x2e0c155f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ac156d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x482c1570, SUC_lexicon_tags_2 },
    { 0x6bcc1571, SUC_lexicon_tags_7 },
    { 0x742c1572, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574c1576, SUC_lexicon_tags_27 },
    { 0x578c1577, SUC_lexicon_tags_28 },
    { 0x836c1577, SUC_lexicon_tags_25 },
    { 0xcac1579, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcec157b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xac157e, SUC_lexicon_tags_6 },
    { 0xa08c157f, SUC_lexicon_tags_25 },
    { 0x4f4c1580, SUC_lexicon_tags_14 },
    { 0x2ecc157e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10c1584, SUC_lexicon_tags_39 },
    { 0x26ac1585, SUC_lexicon_tags_25 },
    { 0xde6c1586, SUC_lexicon_tags_25 },
    { 0x2eec1587, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928c158e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x78c1590, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb68c1592, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x190c1595, SUC_lexicon_tags_28 },
    { 0x9d2c1596, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516c159a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xff2c159c, SUC_lexicon_tags_9 },
    { 0x82ec159d, SUC_lexicon_tags_14 },
    { 0x98ec159e, SUC_lexicon_tags_15 },
    { 0x130c159f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x930c15a2, SUC_lexicon_tags_12 },
    { 0xfb8c15a3, SUC_lexicon_tags_2 },
    { 0xc38c15a4, SUC_lexicon_tags_2 },
    { 0xd44c15a5, SUC_lexicon_tags_17 },
    { 0xf9cc15a6, SUC_lexicon_tags_0 },
    { 0x85ec15a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6a2c15a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16c15ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ac15b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798c15b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ec15b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x648c15bb, SUC_lexicon_tags_17 },
    { 0xe2c15bc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb22c15be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602c15c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c15cd, SUC_lexicon_tags_37 },
    { 0xc2ec15ce, SUC_lexicon_tags_25 },
    { 0x564c15cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf8c15d1, SUC_lexicon_tags_45 },
    { 0x44c15d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cc15d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd0ec15d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4c15de, SUC_lexicon_tags_11 },
    { 0x52cc15df, SUC_lexicon_tags_29 },
    { 0x59ec15df, SUC_lexicon_tags_4 },
    { 0x8b4c15e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad8c15e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06c15e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc76c15eb, SUC_lexicon_tags_11 },
    { 0xf16c15eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29cc15ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876c15f4, SUC_lexicon_tags_1 },
    { 0x24ac15f5, SUC_lexicon_tags_35 },
    { 0xe22c15f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0c15f9, SUC_lexicon_tags_1 },
    { 0xa40c15fa, SUC_lexicon_tags_47 },
    { 0x632c15fa, SUC_lexicon_tags_28 },
    { 0xaf2c15f9, SUC_lexicon_tags_7 },
    { 0x2fec15fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12c1607, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8c160c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2c160f, SUC_lexicon_tags_14 },
    { 0x3d6c1610, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ec1613, SUC_lexicon_tags_25 },
    { 0x26ac1614, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ac1618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cc1622, SUC_lexicon_tags_27 },
    { 0xe5ec1623, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0c1626, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c1629, SUC_lexicon_tags_2 },
    { 0xb30c162a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c1634, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06c163a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6fac163c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x892c163e, SUC_lexicon_tags_11 },
    { 0x1a0c163f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x356c1642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x218c1645, SUC_lexicon_tags_42 },
    { 0xe0ec1645, SUC_lexicon_tags_7 },
    { 0x12c1646, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x272c164a, SUC_lexicon_tags_25 },
    { 0xda2c164b, SUC_lexicon_tags_25 },
    { 0x224c164c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470c1650, SUC_lexicon_tags_17 },
    { 0xc92c1651, SUC_lexicon_tags_55 },
    { 0x620c1652, SUC_lexicon_tags_23 },
    { 0x7d6c1651, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0cc1655, SUC_lexicon_tags_9 },
    { 0xec1656, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68c165e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8c1666, SUC_lexicon_tags_63 },
    { 0x412c1667, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcac1669, SUC_lexicon_tags_12 },
    { 0xcd8c166a, SUC_lexicon_tags_584 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4c166f, SUC_lexicon_tags_17 },
    { 0xdcec166f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc16c1672, SUC_lexicon_tags_15 },
    { 0xa10c1672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124c1678, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeaec167a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91ec167c, SUC_lexicon_tags_6 },
    { 0x7a0c167d, SUC_lexicon_tags_17 },
    { 0x772c167e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8c1684, SUC_lexicon_tags_16 },
    { 0x99cc1685, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa40c1687, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x252c168a, SUC_lexicon_tags_45 },
    { 0x500c168b, SUC_lexicon_tags_49 },
    { 0x9b0c168b, SUC_lexicon_tags_23 },
    { 0xd86c168b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd90c1690, SUC_lexicon_tags_14 },
    { 0xbaac1691, SUC_lexicon_tags_0 },
    { 0xf06c1691, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c1698, SUC_lexicon_tags_1 },
    { 0xb48c1699, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ec169c, SUC_lexicon_tags_23 },
    { 0x8ecc169d, SUC_lexicon_tags_25 },
    { 0x956c169c, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x94c16a0, SUC_lexicon_tags_30 },
    { 0x426c16a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4c16a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e4c16a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x996c16a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x566c16aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c16ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ac16b3, SUC_lexicon_tags_12 },
    { 0xaeec16b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa60c16b6, SUC_lexicon_tags_68 },
    { 0x95ec16b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe44c16b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x646c16bc, SUC_lexicon_tags_10 },
    { 0xc90c16bd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x56cc16bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadec16c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11cc16c5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x902c16c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f4c16c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a4c16cb, SUC_lexicon_tags_25 },
    { 0xc14c16cb, SUC_lexicon_tags_60 },
    { 0xc1ec16cb, SUC_lexicon_tags_13 },
    { 0x3cac16ce, SUC_lexicon_tags_43 },
    { 0x85ec16cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630c16d3, SUC_lexicon_tags_13 },
    { 0x4e0c16d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x824c16d7, SUC_lexicon_tags_46 },
    { 0xfc8c16d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cec16dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eec16e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9cc16e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2c16e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8c16f1, SUC_lexicon_tags_49 },
    { 0x656c16f2, SUC_lexicon_tags_9 },
    { 0xd4c16f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16c16f9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6c16ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ac1703, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x55cc1706, SUC_lexicon_tags_1 },
    { 0x7c4c1706, SUC_lexicon_tags_37 },
    { 0x7bac1706, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604c170e, SUC_lexicon_tags_25 },
    { 0x426c170f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ac1714, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cc1719, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0c171c, SUC_lexicon_tags_37 },
    { 0x136c171c, SUC_lexicon_tags_28 },
    { 0x4e6c171e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614c1723, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634c1728, SUC_lexicon_tags_49 },
    { 0x450c1729, SUC_lexicon_tags_6 },
    { 0xe98c172a, SUC_lexicon_tags_37 },
    { 0xcf2c172b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc172f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968c1735, SUC_lexicon_tags_12 },
    { 0xeb0c1736, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c173d, SUC_lexicon_tags_27 },
    { 0x9c0c173d, SUC_lexicon_tags_37 },
    { 0x2d4c173f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda0c1741, SUC_lexicon_tags_11 },
    { 0xdeac1742, SUC_lexicon_tags_3 },
    { 0xb1ec1743, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ac1747, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cec174d, SUC_lexicon_tags_25 },
    { 0xbe6c174e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914c1752, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6c1756, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06c175b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19cc175d, SUC_lexicon_tags_25 },
    { 0xd4cc175e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb86c1760, SUC_lexicon_tags_38 },
    { 0x24cc1761, SUC_lexicon_tags_32 },
    { 0xff2c1761, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4c176a, SUC_lexicon_tags_27 },
    { 0x7eac176b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4ec176d, SUC_lexicon_tags_32 },
    { 0xfb2c176e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ac1771, SUC_lexicon_tags_68 },
    { 0x3fac1771, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7dac1774, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6c1778, SUC_lexicon_tags_43 },
    { 0x5b8c1779, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4c1780, SUC_lexicon_tags_11 },
    { 0x346c1781, SUC_lexicon_tags_38 },
    { 0xa08c1781, SUC_lexicon_tags_9 },
    { 0xb3ec1783, SUC_lexicon_tags_1 },
    { 0x2f2c1781, SUC_lexicon_tags_19 },
    { 0x1c6c1785, SUC_lexicon_tags_25 },
    { 0xa5ac1781, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c1789, SUC_lexicon_tags_17 },
    { 0x6a2c178a, SUC_lexicon_tags_101 },
    { 0xdcc178a, SUC_lexicon_tags_63 },
    { 0xf70c1789, SUC_lexicon_tags_33 },
    { 0x48ec178a, SUC_lexicon_tags_33 },
    { 0xd92c1789, SUC_lexicon_tags_6 },
    { 0xb6ac178f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ecc1791, SUC_lexicon_tags_33 },
    { 0xb0cc1792, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf04c1794, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8c1799, SUC_lexicon_tags_1 },
    { 0x5e4c1799, SUC_lexicon_tags_14 },
    { 0xe3cc179b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2c17a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa6c17a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8c17ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6c17b2, SUC_lexicon_tags_17 },
    { 0x402c17b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf24c17b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d6c17b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96ac17b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61cc17bb, SUC_lexicon_tags_30 },
    { 0x10ec17bc, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998c17c5, SUC_lexicon_tags_60 },
    { 0x610c17c6, SUC_lexicon_tags_9 },
    { 0x3c6c17c5, SUC_lexicon_tags_37 },
    { 0xe88c17c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0c17cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192c17d6, SUC_lexicon_tags_69 },
    { 0x618c17d7, SUC_lexicon_tags_87 },
    { 0xcd4c17d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966c17de, SUC_lexicon_tags_44 },
    { 0x630c17de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6c17e3, SUC_lexicon_tags_6 },
    { 0xabac17e3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06c17f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942c17f7, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x9ac17f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f0c17fb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254c17ff, SUC_lexicon_tags_47 },
    { 0x47cc1800, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8c1803, SUC_lexicon_tags_46 },
    { 0x938c1803, SUC_lexicon_tags_63 },
    { 0xd52c1805, SUC_lexicon_tags_27 },
    { 0x5bcc1806, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cc1809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ec180d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabec1811, SUC_lexicon_tags_11 },
    { 0x4a2c1812, SUC_lexicon_tags_0 },
    { 0xb9cc1813, SUC_lexicon_tags_37 },
    { 0x3ecc1814, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6c181a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb22c181c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336c1820, SUC_lexicon_tags_16 },
    { 0x9bac1820, SUC_lexicon_tags_38 },
    { 0xaf8c1822, SUC_lexicon_tags_14 },
    { 0x700c1821, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780c1829, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x97cc182c, SUC_lexicon_tags_43 },
    { 0x822c182d, SUC_lexicon_tags_47 },
    { 0x770c182e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c1835, SUC_lexicon_tags_63 },
    { 0x312c1836, SUC_lexicon_tags_47 },
    { 0x208c1835, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cc183c, SUC_lexicon_tags_23 },
    { 0xa0cc183c, SUC_lexicon_tags_11 },
    { 0xb1cc183e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0c1843, SUC_lexicon_tags_29 },
    { 0x41cc1844, SUC_lexicon_tags_415 },
    { 0xb28c1843, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5fec1847, SUC_lexicon_tags_11 },
    { 0x99cc1847, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c184b, SUC_lexicon_tags_1 },
    { 0x69cc184c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b6c184e, SUC_lexicon_tags_33 },
    { 0xbacc184e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x342c1851, SUC_lexicon_tags_37 },
    { 0x62ac1852, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5bcc1854, SUC_lexicon_tags_15 },
    { 0x482c1854, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cc1858, SUC_lexicon_tags_49 },
    { 0xd5ac1859, SUC_lexicon_tags_11 },
    { 0x588c185a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdfac185c, SUC_lexicon_tags_10 },
    { 0x56cc185d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc18c185f, SUC_lexicon_tags_48 },
    { 0x5cec1860, SUC_lexicon_tags_89 },
    { 0xaa8c1861, SUC_lexicon_tags_7 },
    { 0x96ac1860, SUC_lexicon_tags_87 },
    { 0xc0cc1860, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100c1867, SUC_lexicon_tags_7 },
    { 0x84ec1868, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04c1871, SUC_lexicon_tags_15 },
    { 0x9d4c1872, SUC_lexicon_tags_15 },
    { 0xebec1872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x156c1875, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0c1878, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8c187b, SUC_lexicon_tags_27 },
    { 0x16c187c, SUC_lexicon_tags_4 },
    { 0xad2c187d, SUC_lexicon_tags_7 },
    { 0xfc2c187e, SUC_lexicon_tags_8 },
    { 0x5b8c187d, SUC_lexicon_tags_43 },
    { 0x8e8c1880, SUC_lexicon_tags_27 },
    { 0x3e8c1881, SUC_lexicon_tags_7 },
    { 0x2a2c1882, SUC_lexicon_tags_23 },
    { 0xe90c1880, SUC_lexicon_tags_12 },
    { 0xe9cc1881, SUC_lexicon_tags_29 },
    { 0x1b6c1885, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430c1889, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884c188d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c1891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312c1896, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436c189a, SUC_lexicon_tags_25 },
    { 0x6ccc189b, SUC_lexicon_tags_23 },
    { 0xd20c189a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20c18a2, SUC_lexicon_tags_25 },
    { 0xb28c18a3, SUC_lexicon_tags_17 },
    { 0xe88c18a4, SUC_lexicon_tags_26 },
    { 0xf52c18a5, SUC_lexicon_tags_77 },
    { 0x3bcc18a5, SUC_lexicon_tags_25 },
    { 0x712c18a5, SUC_lexicon_tags_11 },
    { 0xc66c18a8, SUC_lexicon_tags_1 },
    { 0xd86c18a9, SUC_lexicon_tags_12 },
    { 0xd08c18aa, SUC_lexicon_tags_7 },
    { 0x958c18ab, SUC_lexicon_tags_17 },
    { 0xcb8c18aa, SUC_lexicon_tags_2 },
    { 0x4e2c18ac, SUC_lexicon_tags_17 },
    { 0xd0cc18a8, SUC_lexicon_tags_7 },
    { 0x308c18a8, SUC_lexicon_tags_8 },
    { 0x2eec18ac, SUC_lexicon_tags_23 },
    { 0xa84c18a3, SUC_lexicon_tags_65 },
    { 0xb58c18b2, SUC_lexicon_tags_11 },
    { 0xd74c18ab, SUC_lexicon_tags_37 },
    { 0xe4ac18b4, SUC_lexicon_tags_11 },
    { 0xbf4c18b5, SUC_lexicon_tags_77 },
    { 0xd2ac18b5, SUC_lexicon_tags_4 },
    { 0x474c18b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1fcc18b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecc18be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x38ac18c0, SUC_lexicon_tags_84 },
    { 0x970c18c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48c18c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74c18c8, SUC_lexicon_tags_63 },
    { 0x8b0c18c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cc18cc, SUC_lexicon_tags_45 },
    { 0xac4c18cd, SUC_lexicon_tags_17 },
    { 0x6a2c18cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x232c18d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c8c18d2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcc18d6, SUC_lexicon_tags_71 },
    { 0xbac18d7, SUC_lexicon_tags_7 },
    { 0xba4c18d6, SUC_lexicon_tags_27 },
    { 0x7fac18d7, SUC_lexicon_tags_15 },
    { 0xe5ac18d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x202c18dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ec18e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ec18eb, SUC_lexicon_tags_37 },
    { 0xd2c18eb, SUC_lexicon_tags_45 },
    { 0xa0ac18ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3ac18ef, SUC_lexicon_tags_81 },
    { 0x13ec18f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ec18f5, SUC_lexicon_tags_14 },
    { 0x6e2c18f5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02c18f9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd0ec18fb, SUC_lexicon_tags_60 },
    { 0x23ec18fc, SUC_lexicon_tags_45 },
    { 0x9f4c18fd, SUC_lexicon_tags_67 },
    { 0xafac18fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23ac1900, SUC_lexicon_tags_28 },
    { 0x78ec1901, SUC_lexicon_tags_7 },
    { 0x1f6c1902, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ac1909, SUC_lexicon_tags_61 },
    { 0x802c1909, SUC_lexicon_tags_1 },
    { 0xaacc190b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x874c190e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x470c1911, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90ec1913, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x598c1916, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c1919, SUC_lexicon_tags_4 },
    { 0xb02c191a, SUC_lexicon_tags_19 },
    { 0x83ec191b, SUC_lexicon_tags_12 },
    { 0x4bcc191b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x772c191f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234c1925, SUC_lexicon_tags_48 },
    { 0x2e8c1926, SUC_lexicon_tags_43 },
    { 0x85cc1927, SUC_lexicon_tags_12 },
    { 0x31ac1928, SUC_lexicon_tags_17 },
    { 0x924c1929, SUC_lexicon_tags_17 },
    { 0x474c192a, SUC_lexicon_tags_37 },
    { 0xc14c192b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8c192e, SUC_lexicon_tags_15 },
    { 0xd7ac192e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x538c1931, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278c193a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92ec193c, SUC_lexicon_tags_17 },
    { 0x998c193d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc42c193f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34c1944, SUC_lexicon_tags_25 },
    { 0xb0cc1944, SUC_lexicon_tags_12 },
    { 0x37ec1946, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90c1949, SUC_lexicon_tags_61 },
    { 0x348c1949, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66c194f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x632c1952, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccc1955, SUC_lexicon_tags_25 },
    { 0x1c2c1956, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cac1960, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718c1965, SUC_lexicon_tags_1 },
    { 0xaa4c1966, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde8c1968, SUC_lexicon_tags_7 },
    { 0x2f8c1969, SUC_lexicon_tags_6 },
    { 0x62c1968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288c1972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518c197a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510c1980, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ec1983, SUC_lexicon_tags_2 },
    { 0xefcc1983, SUC_lexicon_tags_26 },
    { 0x8e4c1985, SUC_lexicon_tags_15 },
    { 0xcf4c1983, SUC_lexicon_tags_2 },
    { 0x292c1985, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8c198a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x926c198c, SUC_lexicon_tags_1 },
    { 0xd7cc198d, SUC_lexicon_tags_63 },
    { 0xdf0c198c, SUC_lexicon_tags_12 },
    { 0xefcc198f, SUC_lexicon_tags_14 },
    { 0xff0c1990, SUC_lexicon_tags_15 },
    { 0xbc8c1991, SUC_lexicon_tags_2 },
    { 0x4bcc198f, SUC_lexicon_tags_87 },
    { 0xb0ac1993, SUC_lexicon_tags_49 },
    { 0xb9ac1993, SUC_lexicon_tags_25 },
    { 0xae8c1994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cac1999, SUC_lexicon_tags_15 },
    { 0xb58c1999, SUC_lexicon_tags_31 },
    { 0x7aec199a, SUC_lexicon_tags_23 },
    { 0xfc8c199b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e2c199e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x916c19a0, SUC_lexicon_tags_47 },
    { 0x492c19a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564c19a6, SUC_lexicon_tags_23 },
    { 0x71ec19a6, SUC_lexicon_tags_32 },
    { 0x1f0c19a6, SUC_lexicon_tags_16 },
    { 0xb68c19a9, SUC_lexicon_tags_2 },
    { 0x222c19aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2c19af, SUC_lexicon_tags_43 },
    { 0x884c19b0, SUC_lexicon_tags_12 },
    { 0xa72c19b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cc19b4, SUC_lexicon_tags_48 },
    { 0x2cc19b4, SUC_lexicon_tags_25 },
    { 0x4b8c19b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdec19b9, SUC_lexicon_tags_17 },
    { 0xbb6c19b9, SUC_lexicon_tags_17 },
    { 0xee8c19bb, SUC_lexicon_tags_61 },
    { 0xb02c19bc, SUC_lexicon_tags_17 },
    { 0x208c19bd, SUC_lexicon_tags_49 },
    { 0xa10c19be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ac19c2, SUC_lexicon_tags_17 },
    { 0x4bec19c3, SUC_lexicon_tags_45 },
    { 0x632c19c4, SUC_lexicon_tags_2 },
    { 0xec6c19c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x588c19c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd24c19c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x502c19cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa34c19cd, SUC_lexicon_tags_118 },
    { 0xbdcc19ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7acc19d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5eec19d2, SUC_lexicon_tags_16 },
    { 0x9cec19d2, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6c19d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ac19df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8ec19e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ac19e5, SUC_lexicon_tags_27 },
    { 0x316c19e6, SUC_lexicon_tags_23 },
    { 0x2f4c19e5, SUC_lexicon_tags_10 },
    { 0xb2c19e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4c19eb, SUC_lexicon_tags_23 },
    { 0xe16c19eb, SUC_lexicon_tags_1 },
    { 0x110c19ed, SUC_lexicon_tags_2 },
    { 0xf58c19ee, SUC_lexicon_tags_23 },
    { 0xfa8c19ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ec19f3, SUC_lexicon_tags_17 },
    { 0x8eec19f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148c19f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcc19fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6c1a00, SUC_lexicon_tags_65 },
    { 0xcf0c1a00, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ac1a04, SUC_lexicon_tags_2 },
    { 0xc4ec1a05, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6c1a09, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08c1a0e, SUC_lexicon_tags_25 },
    { 0xb06c1a0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fcc1a11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ec1a14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcc1a17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac1a1c, SUC_lexicon_tags_91 },
    { 0xe8c1a1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe16c1a1f, SUC_lexicon_tags_11 },
    { 0x760c1a20, SUC_lexicon_tags_2 },
    { 0xff2c1a21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecc1a24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaec1a27, SUC_lexicon_tags_57 },
    { 0x570c1a28, SUC_lexicon_tags_17 },
    { 0x930c1a27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6c1a2e, SUC_lexicon_tags_26 },
    { 0xe80c1a2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2c1a32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208c1a3c, SUC_lexicon_tags_15 },
    { 0x872c1a3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ac1a43, SUC_lexicon_tags_23 },
    { 0x5e0c1a44, SUC_lexicon_tags_25 },
    { 0x892c1a45, SUC_lexicon_tags_38 },
    { 0x5b8c1a46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ac1a49, SUC_lexicon_tags_12 },
    { 0x5d0c1a4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x538c1a4c, SUC_lexicon_tags_37 },
    { 0x89ec1a4d, SUC_lexicon_tags_29 },
    { 0xacac1a4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf38c1a50, SUC_lexicon_tags_31 },
    { 0x5bac1a50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44c1a54, SUC_lexicon_tags_11 },
    { 0x97ac1a55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6c1a58, SUC_lexicon_tags_82 },
    { 0x958c1a59, SUC_lexicon_tags_12 },
    { 0x776c1a5a, SUC_lexicon_tags_12 },
    { 0x4f2c1a5b, SUC_lexicon_tags_17 },
    { 0xaa8c1a5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd9cc1a5e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ac1a65, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ec1a68, SUC_lexicon_tags_19 },
    { 0x8b4c1a69, SUC_lexicon_tags_9 },
    { 0xcf4c1a6a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62c1a6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2eec1a70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cc1a74, SUC_lexicon_tags_29 },
    { 0xa78c1a74, SUC_lexicon_tags_45 },
    { 0x38ec1a76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42c1a79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f8c1a7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2c1a7f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xba4c1a81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc08c1a83, SUC_lexicon_tags_19 },
    { 0xa58c1a83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484c1a8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ec1a90, SUC_lexicon_tags_28 },
    { 0xeaec1a91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c1a95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2c1a98, SUC_lexicon_tags_13 },
    { 0xd28c1a99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd72c1a9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272c1aa1, SUC_lexicon_tags_20 },
    { 0xa92c1aa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408c1aa9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8ac1aab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10ac1aad, SUC_lexicon_tags_47 },
    { 0x2ccc1aae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262c1ab5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8c0c1ab7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2c1abc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x872c1abe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856c1ac3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcc1ac9, SUC_lexicon_tags_15 },
    { 0x386c1aca, SUC_lexicon_tags_30 },
    { 0x3dec1aca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb6c1acd, SUC_lexicon_tags_37 },
    { 0x2b6c1acd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x292c1ad1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa44c1ad3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5ec1ad5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3aac1ad7, SUC_lexicon_tags_1 },
    { 0x41cc1ad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1ade, SUC_lexicon_tags_17 },
    { 0xe1ac1adf, SUC_lexicon_tags_17 },
    { 0x158c1ae0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908c1ae5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc1aee, SUC_lexicon_tags_19 },
    { 0x6eac1aef, SUC_lexicon_tags_6 },
    { 0x4a8c1af0, SUC_lexicon_tags_29 },
    { 0xb68c1af0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa28c1af4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff2c1af6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x380c1af9, SUC_lexicon_tags_33 },
    { 0x786c1af9, SUC_lexicon_tags_12 },
    { 0x148c1afb, SUC_lexicon_tags_23 },
    { 0xfd2c1afc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500c1b00, SUC_lexicon_tags_6 },
    { 0xe1cc1b01, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4c1b05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafac1b09, SUC_lexicon_tags_25 },
    { 0x404c1b09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20ec1b0c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4e4c1b0e, SUC_lexicon_tags_21 },
    { 0x9b8c1b0f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68c1b12, SUC_lexicon_tags_11 },
    { 0x77ec1b12, SUC_lexicon_tags_2 },
    { 0x4f2c1b14, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36c1b19, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06c1b1e, SUC_lexicon_tags_8 },
    { 0x6bec1b1f, SUC_lexicon_tags_16 },
    { 0xc44c1b1e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbac1b26, SUC_lexicon_tags_84 },
    { 0xf8c1b27, SUC_lexicon_tags_25 },
    { 0xfdac1b28, SUC_lexicon_tags_60 },
    { 0xbd2c1b29, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20c1b2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162c1b33, SUC_lexicon_tags_6 },
    { 0x744c1b34, SUC_lexicon_tags_47 },
    { 0xb48c1b35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cc1b3d, SUC_lexicon_tags_0 },
    { 0x7c4c1b3e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748c1b43, SUC_lexicon_tags_146 },
    { 0x6f6c1b43, SUC_lexicon_tags_29 },
    { 0x908c1b45, SUC_lexicon_tags_14 },
    { 0x8cac1b45, SUC_lexicon_tags_37 },
    { 0xea6c1b47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802c1b4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960c1b52, SUC_lexicon_tags_33 },
    { 0xedac1b52, SUC_lexicon_tags_17 },
    { 0x764c1b53, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cc1b58, SUC_lexicon_tags_15 },
    { 0xdbcc1b59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0c1b5c, SUC_lexicon_tags_14 },
    { 0xccac1b5d, SUC_lexicon_tags_29 },
    { 0x4cac1b5e, SUC_lexicon_tags_58 },
    { 0x68c1b5d, SUC_lexicon_tags_988 },
    { 0, NULL },
    { 0x208c1b61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ecc1b63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0c1b6a, SUC_lexicon_tags_7 },
    { 0xd88c1b6a, SUC_lexicon_tags_17 },
    { 0x78ac1b6a, SUC_lexicon_tags_25 },
    { 0x206c1b6d, SUC_lexicon_tags_49 },
    { 0x37cc1b6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cc1b72, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2c1b75, SUC_lexicon_tags_11 },
    { 0x586c1b76, SUC_lexicon_tags_27 },
    { 0xafac1b77, SUC_lexicon_tags_17 },
    { 0x8cec1b78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecc1b7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8c1b7d, SUC_lexicon_tags_1 },
    { 0xd0c1b7e, SUC_lexicon_tags_39 },
    { 0x70ec1b7e, SUC_lexicon_tags_65 },
    { 0x340c1b80, SUC_lexicon_tags_66 },
    { 0xdb4c1b80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24c1b86, SUC_lexicon_tags_46 },
    { 0x2bec1b86, SUC_lexicon_tags_1 },
    { 0x1a8c1b87, SUC_lexicon_tags_60 },
    { 0x2c2c1b89, SUC_lexicon_tags_85 },
    { 0xf88c1b8a, SUC_lexicon_tags_17 },
    { 0xe8c1b8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eec1b90, SUC_lexicon_tags_83 },
    { 0x746c1b91, SUC_lexicon_tags_1 },
    { 0xa76c1b91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc1b9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc1b9f, SUC_lexicon_tags_14 },
    { 0xf64c1ba0, SUC_lexicon_tags_92 },
    { 0x3dac1ba1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4c1ba5, SUC_lexicon_tags_17 },
    { 0xfdec1ba6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82c1baa, SUC_lexicon_tags_9 },
    { 0x50c1bab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x458c1bad, SUC_lexicon_tags_2 },
    { 0x8aec1bad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d2c1bb0, SUC_lexicon_tags_39 },
    { 0xb06c1bb1, SUC_lexicon_tags_59 },
    { 0x15ac1bb2, SUC_lexicon_tags_37 },
    { 0x6bcc1bb3, SUC_lexicon_tags_37 },
    { 0xf7ec1bb3, SUC_lexicon_tags_31 },
    { 0x632c1bb1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cc1bb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4ec1bba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49cc1bbc, SUC_lexicon_tags_15 },
    { 0x4b6c1bbd, SUC_lexicon_tags_63 },
    { 0x9acc1bbe, SUC_lexicon_tags_25 },
    { 0x6fec1bbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee2c1bc1, SUC_lexicon_tags_14 },
    { 0xf6ac1bc1, SUC_lexicon_tags_63 },
    { 0x5f8c1bc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14c1bc8, SUC_lexicon_tags_6 },
    { 0x7b2c1bc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ec1bcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8c1bd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cc1bdc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdac1bde, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8c1be1, SUC_lexicon_tags_69 },
    { 0xf02c1be2, SUC_lexicon_tags_310 },
    { 0x24cc1be3, SUC_lexicon_tags_16 },
    { 0x84c1be4, SUC_lexicon_tags_49 },
    { 0x48ec1be4, SUC_lexicon_tags_25 },
    { 0x6c0c1be6, SUC_lexicon_tags_8 },
    { 0xf92c1be7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cc1bf2, SUC_lexicon_tags_1 },
    { 0xbaec1bf3, SUC_lexicon_tags_15 },
    { 0x3e0c1bf3, SUC_lexicon_tags_14 },
    { 0x342c1bf3, SUC_lexicon_tags_18 },
    { 0x346c1bf3, SUC_lexicon_tags_12 },
    { 0xe92c1bf7, SUC_lexicon_tags_37 },
    { 0x608c1bf8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x928c1bfa, SUC_lexicon_tags_12 },
    { 0x718c1bfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec1bff, SUC_lexicon_tags_2 },
    { 0xe98c1c00, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0xc9ac1c02, SUC_lexicon_tags_11 },
    { 0x3b6c1c03, SUC_lexicon_tags_2 },
    { 0xb2ac1c04, SUC_lexicon_tags_43 },
    { 0x300c1c04, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdbcc1c07, SUC_lexicon_tags_18 },
    { 0xf82c1c07, SUC_lexicon_tags_2 },
    { 0x8e4c1c07, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2c1c0c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb5ec1c0e, SUC_lexicon_tags_15 },
    { 0xf7cc1c0f, SUC_lexicon_tags_59 },
    { 0x6dac1c0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ec1c13, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00c1c18, SUC_lexicon_tags_16 },
    { 0x3d6c1c19, SUC_lexicon_tags_9 },
    { 0xb3ac1c1a, SUC_lexicon_tags_47 },
    { 0x9cec1c19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e2c1c1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fac1c23, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198c1c2b, SUC_lexicon_tags_2 },
    { 0x14c1c2b, SUC_lexicon_tags_2 },
    { 0x7a0c1c2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ec1c35, SUC_lexicon_tags_25 },
    { 0x15ec1c35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4c1c3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c1c47, SUC_lexicon_tags_25 },
    { 0xa0ac1c48, SUC_lexicon_tags_12 },
    { 0x9fcc1c49, SUC_lexicon_tags_12 },
    { 0x7ecc1c48, SUC_lexicon_tags_2 },
    { 0x388c1c4b, SUC_lexicon_tags_77 },
    { 0xaf2c1c4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4c1c50, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdecc1c53, SUC_lexicon_tags_1 },
    { 0xbe6c1c54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e0c1c56, SUC_lexicon_tags_35 },
    { 0x98c1c57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf84c1c59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278c1c5e, SUC_lexicon_tags_2 },
    { 0x860c1c5f, SUC_lexicon_tags_37 },
    { 0x242c1c60, SUC_lexicon_tags_47 },
    { 0x456c1c60, SUC_lexicon_tags_38 },
    { 0x3e0c1c60, SUC_lexicon_tags_419 },
    { 0xbf2c1c5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf96c1c65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90c1c6b, SUC_lexicon_tags_1 },
    { 0x272c1c6b, SUC_lexicon_tags_31 },
    { 0xcbec1c6d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c1c70, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2c1c77, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ac1c7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xffcc1c7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91ac1c80, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ac1c84, SUC_lexicon_tags_64 },
    { 0xcecc1c85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16c1c89, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cc1c8c, SUC_lexicon_tags_19 },
    { 0x64c1c8d, SUC_lexicon_tags_10 },
    { 0xb82c1c8d, SUC_lexicon_tags_28 },
    { 0xdd6c1c8c, SUC_lexicon_tags_65 },
    { 0x132c1c8c, SUC_lexicon_tags_2 },
    { 0xf2cc1c91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16cc1c93, SUC_lexicon_tags_109 },
    { 0x59ac1c94, SUC_lexicon_tags_2 },
    { 0x808c1c95, SUC_lexicon_tags_26 },
    { 0x2f2c1c96, SUC_lexicon_tags_158 },
    { 0x960c1c96, SUC_lexicon_tags_17 },
    { 0x212c1c98, SUC_lexicon_tags_10 },
    { 0x2bac1c95, SUC_lexicon_tags_235 },
    { 0x4cac1c9a, SUC_lexicon_tags_25 },
    { 0xea2c1c9b, SUC_lexicon_tags_2 },
    { 0x2cec1c9b, SUC_lexicon_tags_91 },
    { 0x9f6c1c9c, SUC_lexicon_tags_25 },
    { 0xd98c1c9c, SUC_lexicon_tags_10 },
    { 0x1dac1c9f, SUC_lexicon_tags_154 },
    { 0x464c1c9f, SUC_lexicon_tags_0 },
    { 0x534c1ca1, SUC_lexicon_tags_14 },
    { 0x76c1c9c, SUC_lexicon_tags_48 },
    { 0xb80c1c9a, SUC_lexicon_tags_12 },
    { 0xeccc1c98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4c1ca9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xda8c1cab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd78c1cad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ac1cb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26c1cb5, SUC_lexicon_tags_17 },
    { 0xd00c1cb6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x556c1cb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68c1cbb, SUC_lexicon_tags_15 },
    { 0x978c1cbb, SUC_lexicon_tags_42 },
    { 0xcf4c1cbc, SUC_lexicon_tags_45 },
    { 0x854c1cbe, SUC_lexicon_tags_15 },
    { 0x192c1cbf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48c1cc5, SUC_lexicon_tags_55 },
    { 0x37ec1cc6, SUC_lexicon_tags_17 },
    { 0x950c1cc5, SUC_lexicon_tags_1 },
    { 0xd3ac1cc5, SUC_lexicon_tags_11 },
    { 0xb7ac1cc8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x250c1ccb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x656c1cce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4c1cd1, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0c1cd7, SUC_lexicon_tags_25 },
    { 0x2b8c1cd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2c1cdc, SUC_lexicon_tags_6 },
    { 0x87ec1cdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87ec1cdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2c1ce2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbcec1ce4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248c1ce9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe84c1ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ec1cef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeb8c1cf1, SUC_lexicon_tags_60 },
    { 0x9aec1cf2, SUC_lexicon_tags_37 },
    { 0x31cc1cf2, SUC_lexicon_tags_15 },
    { 0x698c1cf3, SUC_lexicon_tags_17 },
    { 0x8fac1cf5, SUC_lexicon_tags_33 },
    { 0xaacc1cf6, SUC_lexicon_tags_31 },
    { 0x836c1cf7, SUC_lexicon_tags_4 },
    { 0x176c1cf7, SUC_lexicon_tags_23 },
    { 0x792c1cf9, SUC_lexicon_tags_47 },
    { 0xacec1cf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x944c1cfd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8c1d01, SUC_lexicon_tags_64 },
    { 0x7e4c1d01, SUC_lexicon_tags_10 },
    { 0x1e2c1d03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ac1d07, SUC_lexicon_tags_60 },
    { 0xf78c1d07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x238c1d0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcbcc1d0d, SUC_lexicon_tags_25 },
    { 0x39ac1d0e, SUC_lexicon_tags_19 },
    { 0x11ec1d0f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcc1d15, SUC_lexicon_tags_11 },
    { 0xf3cc1d16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41cc1d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c1d1b, SUC_lexicon_tags_1 },
    { 0xc98c1d1c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x60ec1d1e, SUC_lexicon_tags_27 },
    { 0xc24c1d1f, SUC_lexicon_tags_39 },
    { 0x53ec1d1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x514c1d23, SUC_lexicon_tags_47 },
    { 0xf58c1d24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81ec1d26, SUC_lexicon_tags_48 },
    { 0xff4c1d27, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294c1d2d, SUC_lexicon_tags_7 },
    { 0x83ec1d2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96c1d32, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec1d35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8c1d38, SUC_lexicon_tags_27 },
    { 0x586c1d38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890c1d3d, SUC_lexicon_tags_61 },
    { 0xf3ec1d3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cc1d42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7d0c1d44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefcc1d46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dac1d4a, SUC_lexicon_tags_11 },
    { 0x8f6c1d4b, SUC_lexicon_tags_32 },
    { 0x8fec1d4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x728c1d4f, SUC_lexicon_tags_27 },
    { 0xd8ac1d50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bac1d53, SUC_lexicon_tags_25 },
    { 0xf3ac1d54, SUC_lexicon_tags_45 },
    { 0x73ac1d55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23ec1d57, SUC_lexicon_tags_35 },
    { 0xa56c1d58, SUC_lexicon_tags_17 },
    { 0xeec1d57, SUC_lexicon_tags_30 },
    { 0xf38c1d5a, SUC_lexicon_tags_79 },
    { 0x4ec1d58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c1d61, SUC_lexicon_tags_28 },
    { 0x188c1d62, SUC_lexicon_tags_47 },
    { 0xd0ac1d62, SUC_lexicon_tags_7 },
    { 0xa6cc1d62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf50c1d66, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x612c1d68, SUC_lexicon_tags_46 },
    { 0x30ec1d68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c1d6d, SUC_lexicon_tags_63 },
    { 0x5e8c1d6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fac1d71, SUC_lexicon_tags_11 },
    { 0xd0cc1d71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x120c1d74, SUC_lexicon_tags_17 },
    { 0x958c1d75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c1d7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10c1d80, SUC_lexicon_tags_49 },
    { 0x3b0c1d81, SUC_lexicon_tags_7 },
    { 0xfa8c1d81, SUC_lexicon_tags_28 },
    { 0xfcac1d83, SUC_lexicon_tags_2 },
    { 0x2cc1d83, SUC_lexicon_tags_17 },
    { 0x338c1d81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4c1d88, SUC_lexicon_tags_17 },
    { 0xc6ec1d88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x396c1d8b, SUC_lexicon_tags_25 },
    { 0xda6c1d8c, SUC_lexicon_tags_63 },
    { 0x114c1d8d, SUC_lexicon_tags_17 },
    { 0x494c1d8e, SUC_lexicon_tags_2 },
    { 0x374c1d8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4c1d96, SUC_lexicon_tags_30 },
    { 0x79ec1d97, SUC_lexicon_tags_38 },
    { 0x1eec1d96, SUC_lexicon_tags_43 },
    { 0xf02c1d96, SUC_lexicon_tags_7 },
    { 0x5a8c1d98, SUC_lexicon_tags_38 },
    { 0x8aec1d96, SUC_lexicon_tags_31 },
    { 0x54ec1d9c, SUC_lexicon_tags_37 },
    { 0x8c4c1d97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8dac1d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3cc1da1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x16cc1da3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x876c1da5, SUC_lexicon_tags_27 },
    { 0x57ac1da6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8c1dab, SUC_lexicon_tags_12 },
    { 0x55ac1dab, SUC_lexicon_tags_2 },
    { 0x24cc1dad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690c1db2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ac1db5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x566c1db8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6c1dbc, SUC_lexicon_tags_17 },
    { 0xb18c1dbd, SUC_lexicon_tags_47 },
    { 0x5d2c1dbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecec1dc0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674c1dc7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x822c1dc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272c1dd1, SUC_lexicon_tags_37 },
    { 0x51ac1dd1, SUC_lexicon_tags_8 },
    { 0x60ec1dd3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x362c1dd5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa5cc1dd7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0c1ddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ac1de0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4c1de5, SUC_lexicon_tags_28 },
    { 0x818c1de6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24c1dea, SUC_lexicon_tags_37 },
    { 0x912c1dea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8c1def, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdec1df7, SUC_lexicon_tags_1 },
    { 0xe6c1df8, SUC_lexicon_tags_29 },
    { 0x712c1df9, SUC_lexicon_tags_77 },
    { 0xfdec1dfa, SUC_lexicon_tags_11 },
    { 0xf72c1dfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272c1e00, SUC_lexicon_tags_17 },
    { 0xab4c1e01, SUC_lexicon_tags_0 },
    { 0x4eac1e02, SUC_lexicon_tags_17 },
    { 0xf8ec1e02, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x618c1e05, SUC_lexicon_tags_13 },
    { 0xa5cc1e05, SUC_lexicon_tags_66 },
    { 0x242c1e07, SUC_lexicon_tags_49 },
    { 0xe68c1e06, SUC_lexicon_tags_37 },
    { 0x89ec1e07, SUC_lexicon_tags_25 },
    { 0x710c1e09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4c1e0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788c1e15, SUC_lexicon_tags_37 },
    { 0xe4ac1e16, SUC_lexicon_tags_7 },
    { 0x86c1e17, SUC_lexicon_tags_7 },
    { 0x36cc1e15, SUC_lexicon_tags_49 },
    { 0xaf2c1e19, SUC_lexicon_tags_11 },
    { 0xe20c1e16, SUC_lexicon_tags_7 },
    { 0x8a0c1e1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe86c1e1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ec1e23, SUC_lexicon_tags_15 },
    { 0xad0c1e23, SUC_lexicon_tags_37 },
    { 0x8fac1e25, SUC_lexicon_tags_25 },
    { 0x20ac1e26, SUC_lexicon_tags_11 },
    { 0x1d4c1e27, SUC_lexicon_tags_15 },
    { 0x882c1e26, SUC_lexicon_tags_17 },
    { 0xe9cc1e29, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcc1e2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cc1e30, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6c1e37, SUC_lexicon_tags_7 },
    { 0x726c1e37, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x63cc1e3a, SUC_lexicon_tags_25 },
    { 0x7b6c1e3a, SUC_lexicon_tags_13 },
    { 0x982c1e3c, SUC_lexicon_tags_12 },
    { 0xd60c1e3d, SUC_lexicon_tags_37 },
    { 0xe74c1e3e, SUC_lexicon_tags_7 },
    { 0x33cc1e3f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98c1e43, SUC_lexicon_tags_7 },
    { 0xce0c1e44, SUC_lexicon_tags_25 },
    { 0xb74c1e44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x536c1e47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6c1e4a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596c1e50, SUC_lexicon_tags_0 },
    { 0x9a8c1e51, SUC_lexicon_tags_2 },
    { 0x872c1e50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6d2c1e54, SUC_lexicon_tags_30 },
    { 0x52ec1e55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5d8c1e57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x608c1e59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xadec1e5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x410c1e5d, SUC_lexicon_tags_14 },
    { 0x87cc1e5e, SUC_lexicon_tags_12 },
    { 0x6b6c1e5f, SUC_lexicon_tags_25 },
    { 0x7e0c1e5e, SUC_lexicon_tags_26 },
    { 0xd46c1e5d, SUC_lexicon_tags_45 },
    { 0x5eac1e62, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164c1e66, SUC_lexicon_tags_28 },
    { 0xcccc1e67, SUC_lexicon_tags_37 },
    { 0x3eec1e66, SUC_lexicon_tags_35 },
    { 0x634c1e69, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cc1e71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb8c1e73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8c1e76, SUC_lexicon_tags_23 },
    { 0x330c1e77, SUC_lexicon_tags_15 },
    { 0xbc6c1e78, SUC_lexicon_tags_26 },
    { 0x898c1e79, SUC_lexicon_tags_17 },
    { 0x7d2c1e78, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568c1e7f, SUC_lexicon_tags_1 },
    { 0xc1e80, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34c1e86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1ec1e88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1c2c1e8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c1e8f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2c1e92, SUC_lexicon_tags_136 },
    { 0x68c1e93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ec1e95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972c1e99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ac1e9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942c1ea2, SUC_lexicon_tags_1 },
    { 0xd2ec1ea2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ac1ea8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ec1eb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eac1ebc, SUC_lexicon_tags_11 },
    { 0xa0cc1ebd, SUC_lexicon_tags_15 },
    { 0x6a0c1ebe, SUC_lexicon_tags_24 },
    { 0xafcc1ebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10c1ec3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa00c1ec5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cc1ecc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738c1ed2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6c1edf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x786c1ee1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ec1ee4, SUC_lexicon_tags_37 },
    { 0xd72c1ee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194c1eec, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x634c1eee, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04c1ef2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc5ac1ef4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f6c1ef6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64c1ef9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa90c1efb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x336c1efe, SUC_lexicon_tags_38 },
    { 0x416c1eff, SUC_lexicon_tags_37 },
    { 0xae4c1f00, SUC_lexicon_tags_16 },
    { 0x9dac1f01, SUC_lexicon_tags_39 },
    { 0x81ec1f02, SUC_lexicon_tags_43 },
    { 0x866c1f02, SUC_lexicon_tags_15 },
    { 0x482c1f02, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x832c1f06, SUC_lexicon_tags_28 },
    { 0x5acc1f07, SUC_lexicon_tags_87 },
    { 0x764c1f08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10c1f0e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdac1f14, SUC_lexicon_tags_14 },
    { 0xaf4c1f15, SUC_lexicon_tags_0 },
    { 0xab8c1f14, SUC_lexicon_tags_43 },
    { 0xe66c1f16, SUC_lexicon_tags_14 },
    { 0x8d0c1f15, SUC_lexicon_tags_13 },
    { 0xcd8c1f14, SUC_lexicon_tags_14 },
    { 0x308c1f1a, SUC_lexicon_tags_19 },
    { 0xdd0c1f1a, SUC_lexicon_tags_17 },
    { 0xe84c1f15, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x420c1f1f, SUC_lexicon_tags_23 },
    { 0x688c1f20, SUC_lexicon_tags_0 },
    { 0x7ec1f1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eec1f28, SUC_lexicon_tags_38 },
    { 0x8a8c1f29, SUC_lexicon_tags_34 },
    { 0xd50c1f2a, SUC_lexicon_tags_3 },
    { 0xefec1f28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15cc1f2d, SUC_lexicon_tags_7 },
    { 0xda4c1f2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96ec1f30, SUC_lexicon_tags_25 },
    { 0x860c1f30, SUC_lexicon_tags_11 },
    { 0xe9ec1f31, SUC_lexicon_tags_1 },
    { 0x92cc1f33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400c1f37, SUC_lexicon_tags_25 },
    { 0x93ac1f37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8c1f3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x426c1f3f, SUC_lexicon_tags_15 },
    { 0xc1ec1f40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ac1f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8c1f49, SUC_lexicon_tags_17 },
    { 0xdfec1f4a, SUC_lexicon_tags_25 },
    { 0x296c1f4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa92c1f4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb8c1f4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4c1f51, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584c1f56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ec1f5b, SUC_lexicon_tags_31 },
    { 0xd36c1f5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2c1f5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10c1f62, SUC_lexicon_tags_68 },
    { 0x9aec1f62, SUC_lexicon_tags_19 },
    { 0x546c1f62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798c1f6a, SUC_lexicon_tags_254 },
    { 0xf14c1f6b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x152c1f6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206c1f78, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1ccc1f7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf2c1f7c, SUC_lexicon_tags_17 },
    { 0xf54c1f7d, SUC_lexicon_tags_63 },
    { 0x11ec1f7c, SUC_lexicon_tags_12 },
    { 0xcfac1f7e, SUC_lexicon_tags_23 },
    { 0xb88c1f80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x830c1f83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c1f89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506c1f8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x904c1f91, SUC_lexicon_tags_2 },
    { 0x834c1f91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0c1f96, SUC_lexicon_tags_62 },
    { 0x726c1f96, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x310c1f99, SUC_lexicon_tags_26 },
    { 0xfbcc1f99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe26c1f9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbec1fa0, SUC_lexicon_tags_33 },
    { 0xc18c1fa0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x94c1fa4, SUC_lexicon_tags_38 },
    { 0x9fac1fa5, SUC_lexicon_tags_1 },
    { 0xdecc1fa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8c1fa9, SUC_lexicon_tags_1 },
    { 0x8e4c1faa, SUC_lexicon_tags_31 },
    { 0x1e2c1fa9, SUC_lexicon_tags_60 },
    { 0xeccc1fac, SUC_lexicon_tags_68 },
    { 0x9d2c1fad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcc1fb0, SUC_lexicon_tags_11 },
    { 0xe92c1fb1, SUC_lexicon_tags_7 },
    { 0xc44c1fb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfac1fb8, SUC_lexicon_tags_34 },
    { 0x35ec1fb9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60c1fc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576c1fc9, SUC_lexicon_tags_12 },
    { 0x532c1fca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52c1fd2, SUC_lexicon_tags_82 },
    { 0x2b6c1fd3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ac1fd9, SUC_lexicon_tags_2 },
    { 0x168c1fd9, SUC_lexicon_tags_11 },
    { 0x860c1fda, SUC_lexicon_tags_63 },
    { 0x888c1fdc, SUC_lexicon_tags_11 },
    { 0xb28c1fdd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x154c1fdf, SUC_lexicon_tags_12 },
    { 0xb84c1fe0, SUC_lexicon_tags_2 },
    { 0x6a4c1fdf, SUC_lexicon_tags_37 },
    { 0x672c1fe2, SUC_lexicon_tags_25 },
    { 0x10ec1fe3, SUC_lexicon_tags_127 },
    { 0x52ec1fe3, SUC_lexicon_tags_12 },
    { 0xf56c1fe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc6c1fe7, SUC_lexicon_tags_25 },
    { 0x488c1fe8, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870c1fee, SUC_lexicon_tags_37 },
    { 0x1e2c1fee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ac1ff2, SUC_lexicon_tags_49 },
    { 0x5f4c1ff3, SUC_lexicon_tags_55 },
    { 0x18ac1ff4, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2c1ff7, SUC_lexicon_tags_3 },
    { 0xab6c1ff7, SUC_lexicon_tags_37 },
    { 0xb66c1ff9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x69cc1ffc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6c1fff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02c2009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74c2015, SUC_lexicon_tags_54 },
    { 0xfcec2016, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x602c2018, SUC_lexicon_tags_2 },
    { 0x44cc2019, SUC_lexicon_tags_25 },
    { 0x182c201a, SUC_lexicon_tags_11 },
    { 0x26ac2019, SUC_lexicon_tags_10 },
    { 0xf26c201c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x154c201f, SUC_lexicon_tags_11 },
    { 0xdd2c2020, SUC_lexicon_tags_233 },
    { 0xeeec2020, SUC_lexicon_tags_4 },
    { 0x18c2022, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78c202a, SUC_lexicon_tags_0 },
    { 0xe14c202b, SUC_lexicon_tags_11 },
    { 0x604c202c, SUC_lexicon_tags_60 },
    { 0x9c4c202d, SUC_lexicon_tags_37 },
    { 0xdf2c202d, SUC_lexicon_tags_17 },
    { 0x114c202d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bac2032, SUC_lexicon_tags_15 },
    { 0xabcc2033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd04c2035, SUC_lexicon_tags_31 },
    { 0xc08c2035, SUC_lexicon_tags_37 },
    { 0xf8ec2037, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226c2040, SUC_lexicon_tags_15 },
    { 0xc6cc2040, SUC_lexicon_tags_423 },
    { 0x62cc2042, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ec2048, SUC_lexicon_tags_23 },
    { 0x916c2049, SUC_lexicon_tags_15 },
    { 0x8bec2049, SUC_lexicon_tags_17 },
    { 0x3ccc204b, SUC_lexicon_tags_25 },
    { 0xd0c2049, SUC_lexicon_tags_26 },
    { 0x752c204c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x600c204f, SUC_lexicon_tags_28 },
    { 0xddec2050, SUC_lexicon_tags_28 },
    { 0x56ec2051, SUC_lexicon_tags_17 },
    { 0x7bec204f, SUC_lexicon_tags_65 },
    { 0x172c2053, SUC_lexicon_tags_48 },
    { 0xe14c2054, SUC_lexicon_tags_25 },
    { 0xee8c2055, SUC_lexicon_tags_0 },
    { 0x984c2056, SUC_lexicon_tags_45 },
    { 0x820c2057, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0c205a, SUC_lexicon_tags_3 },
    { 0x15ec205b, SUC_lexicon_tags_37 },
    { 0xe9cc205c, SUC_lexicon_tags_16 },
    { 0x494c205c, SUC_lexicon_tags_6 },
    { 0xa7ec205d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcc2061, SUC_lexicon_tags_14 },
    { 0x8d8c2062, SUC_lexicon_tags_87 },
    { 0x96ec2061, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30c206a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8ec206c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2c2070, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcc2078, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aec207b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ac207f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900c2083, SUC_lexicon_tags_29 },
    { 0xa06c2083, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5ec2086, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3ec2088, SUC_lexicon_tags_15 },
    { 0xbdac2089, SUC_lexicon_tags_8 },
    { 0x15ac208a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ec208e, SUC_lexicon_tags_26 },
    { 0x852c208f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6ccc2091, SUC_lexicon_tags_37 },
    { 0x61ec2092, SUC_lexicon_tags_11 },
    { 0x2fcc2093, SUC_lexicon_tags_402 },
    { 0x558c2093, SUC_lexicon_tags_37 },
    { 0x320c2093, SUC_lexicon_tags_12 },
    { 0xc56c2096, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438c209c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86c209e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66c20a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828c20a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0c20ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe42c20af, SUC_lexicon_tags_25 },
    { 0xee2c20af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10c20b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafec20bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6c20be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd64c20c0, SUC_lexicon_tags_17 },
    { 0xcec20c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x162c20c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2f4c20c5, SUC_lexicon_tags_2 },
    { 0xaa2c20c6, SUC_lexicon_tags_9 },
    { 0x544c20c6, SUC_lexicon_tags_60 },
    { 0x1c6c20c7, SUC_lexicon_tags_25 },
    { 0x414c20c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cc20cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0c20d2, SUC_lexicon_tags_13 },
    { 0xe24c20d2, SUC_lexicon_tags_23 },
    { 0xbac20d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f6c20d6, SUC_lexicon_tags_25 },
    { 0xefac20d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc00c20d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346c20de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87ec20e0, SUC_lexicon_tags_2 },
    { 0x86cc20e0, SUC_lexicon_tags_2 },
    { 0x174c20e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x458c20e5, SUC_lexicon_tags_15 },
    { 0x2e6c20e5, SUC_lexicon_tags_49 },
    { 0x5ecc20e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc42c20e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x928c20ec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82c20f1, SUC_lexicon_tags_48 },
    { 0x6e2c20f2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac2100, SUC_lexicon_tags_7 },
    { 0x7eec2101, SUC_lexicon_tags_84 },
    { 0x1acc2102, SUC_lexicon_tags_21 },
    { 0x742c2103, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe9ec2105, SUC_lexicon_tags_77 },
    { 0xfc8c2106, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48c210b, SUC_lexicon_tags_60 },
    { 0x356c210c, SUC_lexicon_tags_14 },
    { 0xff2c210d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xde2c210f, SUC_lexicon_tags_15 },
    { 0x1aec2110, SUC_lexicon_tags_17 },
    { 0x8e6c2111, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c2115, SUC_lexicon_tags_15 },
    { 0xbb0c2115, SUC_lexicon_tags_39 },
    { 0x27cc2116, SUC_lexicon_tags_1 },
    { 0x3a8c2118, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ac211c, SUC_lexicon_tags_38 },
    { 0xa42c211d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8c2120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x344c2123, SUC_lexicon_tags_17 },
    { 0xb7cc2123, SUC_lexicon_tags_1 },
    { 0x110c2124, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6c212a, SUC_lexicon_tags_12 },
    { 0x3dec212a, SUC_lexicon_tags_65 },
    { 0x46ec212b, SUC_lexicon_tags_45 },
    { 0xaa0c212d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9cac212f, SUC_lexicon_tags_37 },
    { 0x79cc2130, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x120c2133, SUC_lexicon_tags_7 },
    { 0xfdec2134, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c213a, SUC_lexicon_tags_34 },
    { 0xaeec213b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fac213e, SUC_lexicon_tags_1 },
    { 0xf4c213f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdec2141, SUC_lexicon_tags_17 },
    { 0xf40c2142, SUC_lexicon_tags_37 },
    { 0x1eac2142, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cc214f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x622c2152, SUC_lexicon_tags_6 },
    { 0x712c2152, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4d6c2155, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86c215b, SUC_lexicon_tags_37 },
    { 0xf1cc215b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2ac215e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1dec2160, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x3eac2162, SUC_lexicon_tags_29 },
    { 0xf2c2163, SUC_lexicon_tags_7 },
    { 0x47ac2163, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512c2168, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456c216e, SUC_lexicon_tags_28 },
    { 0x8ac216f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cc2175, SUC_lexicon_tags_0 },
    { 0x22cc2175, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712c217b, SUC_lexicon_tags_15 },
    { 0xef8c217c, SUC_lexicon_tags_26 },
    { 0xe7cc217b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4c2181, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0c2184, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x36ec2186, SUC_lexicon_tags_151 },
    { 0xb0cc2187, SUC_lexicon_tags_37 },
    { 0xc52c2186, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aec218c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeac2190, SUC_lexicon_tags_33 },
    { 0x164c2190, SUC_lexicon_tags_28 },
    { 0x558c2192, SUC_lexicon_tags_48 },
    { 0x3c8c2193, SUC_lexicon_tags_7 },
    { 0xc1ac2194, SUC_lexicon_tags_11 },
    { 0xb4c2192, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x586c2197, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12c21a4, SUC_lexicon_tags_15 },
    { 0x1e8c21a5, SUC_lexicon_tags_17 },
    { 0x41ac21a4, SUC_lexicon_tags_66 },
    { 0x314c21a7, SUC_lexicon_tags_63 },
    { 0xad8c21a5, SUC_lexicon_tags_42 },
    { 0x9d8c21a8, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x12ec21ab, SUC_lexicon_tags_26 },
    { 0x254c21ac, SUC_lexicon_tags_33 },
    { 0xc1ac21ac, SUC_lexicon_tags_55 },
    { 0x1a8c21ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c21b1, SUC_lexicon_tags_47 },
    { 0x388c21b2, SUC_lexicon_tags_17 },
    { 0xc36c21b3, SUC_lexicon_tags_11 },
    { 0xd1ec21b3, SUC_lexicon_tags_37 },
    { 0xac2c21b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x794c21b7, SUC_lexicon_tags_2 },
    { 0x1fac21b8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186c21bf, SUC_lexicon_tags_15 },
    { 0x754c21bf, SUC_lexicon_tags_37 },
    { 0xb2ec21c1, SUC_lexicon_tags_31 },
    { 0xb9ac21c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510c21cc, SUC_lexicon_tags_45 },
    { 0xf3ec21cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4ac21cf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x188c21d1, SUC_lexicon_tags_7 },
    { 0xa40c21d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aec21d7, SUC_lexicon_tags_37 },
    { 0xa44c21d8, SUC_lexicon_tags_12 },
    { 0x4b4c21d9, SUC_lexicon_tags_10 },
    { 0x718c21da, SUC_lexicon_tags_7 },
    { 0xb98c21da, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2c21e4, SUC_lexicon_tags_118 },
    { 0x74cc21e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb6c21e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4c21f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4c21f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe30c21f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eac21fb, SUC_lexicon_tags_1 },
    { 0x9c6c21fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaac2200, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ac2203, SUC_lexicon_tags_33 },
    { 0xc2204, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ac2207, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6c220a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cec220d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cc2211, SUC_lexicon_tags_1 },
    { 0xfc6c2211, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2f0c2214, SUC_lexicon_tags_14 },
    { 0xd5ec2214, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaaac2217, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cc221a, SUC_lexicon_tags_1 },
    { 0xc2c221b, SUC_lexicon_tags_15 },
    { 0xadec221c, SUC_lexicon_tags_31 },
    { 0xf48c221c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94c2220, SUC_lexicon_tags_46 },
    { 0x9dec2220, SUC_lexicon_tags_551 },
    { 0xc38c2220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8c2226, SUC_lexicon_tags_17 },
    { 0x856c2227, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a2c2229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38c222f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ec2233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ec2236, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50ac2238, SUC_lexicon_tags_8 },
    { 0x346c2238, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacc223e, SUC_lexicon_tags_19 },
    { 0x532c223f, SUC_lexicon_tags_8 },
    { 0xc2c223f, SUC_lexicon_tags_33 },
    { 0x47cc2241, SUC_lexicon_tags_23 },
    { 0x48c223f, SUC_lexicon_tags_25 },
    { 0x3d0c2243, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52c2245, SUC_lexicon_tags_15 },
    { 0xbe2c2246, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2c2249, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcc224e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ac2256, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xddec2258, SUC_lexicon_tags_12 },
    { 0xc4ec2259, SUC_lexicon_tags_63 },
    { 0x940c225a, SUC_lexicon_tags_8 },
    { 0xdb0c2258, SUC_lexicon_tags_30 },
    { 0x266c225c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ecc225e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaacc2260, SUC_lexicon_tags_17 },
    { 0xf8cc2261, SUC_lexicon_tags_25 },
    { 0x94ac2262, SUC_lexicon_tags_0 },
    { 0xc20c2262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4c2268, SUC_lexicon_tags_1 },
    { 0xb94c2269, SUC_lexicon_tags_12 },
    { 0x784c226a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac226e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x204c2270, SUC_lexicon_tags_15 },
    { 0x7bcc2270, SUC_lexicon_tags_56 },
    { 0xdaec2271, SUC_lexicon_tags_33 },
    { 0x8b2c2272, SUC_lexicon_tags_26 },
    { 0xaaec2274, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x752c2276, SUC_lexicon_tags_9 },
    { 0x432c2276, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xde0c2279, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6c227d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7ec227f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2acc2282, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf0c2284, SUC_lexicon_tags_46 },
    { 0xb00c2285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58c2289, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1cc228b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc6ac228d, SUC_lexicon_tags_37 },
    { 0xec4c228e, SUC_lexicon_tags_25 },
    { 0xe0ec228d, SUC_lexicon_tags_14 },
    { 0xcb2c228f, SUC_lexicon_tags_12 },
    { 0xc8c2291, SUC_lexicon_tags_25 },
    { 0xb78c2292, SUC_lexicon_tags_32 },
    { 0x3d4c2293, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6c2298, SUC_lexicon_tags_2 },
    { 0xdc8c2299, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6bcc229b, SUC_lexicon_tags_63 },
    { 0xb10c229c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa00c229e, SUC_lexicon_tags_2 },
    { 0x12c229f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ec22a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x230c22aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6c22af, SUC_lexicon_tags_37 },
    { 0x9a4c22b0, SUC_lexicon_tags_65 },
    { 0xd34c22b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4cc22b3, SUC_lexicon_tags_31 },
    { 0x1c8c22b4, SUC_lexicon_tags_11 },
    { 0x4fac22b5, SUC_lexicon_tags_23 },
    { 0x2b8c22b6, SUC_lexicon_tags_87 },
    { 0xd88c22b6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ec22bb, SUC_lexicon_tags_77 },
    { 0x77ec22bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c22c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x50c22c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70c22cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42ac22cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ec22d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8ec22d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422c22d6, SUC_lexicon_tags_17 },
    { 0x82ac22d7, SUC_lexicon_tags_37 },
    { 0xddcc22d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x424c22da, SUC_lexicon_tags_27 },
    { 0x79cc22db, SUC_lexicon_tags_25 },
    { 0x8eac22dc, SUC_lexicon_tags_27 },
    { 0xbac22db, SUC_lexicon_tags_14 },
    { 0x1d8c22de, SUC_lexicon_tags_43 },
    { 0x78ec22df, SUC_lexicon_tags_21 },
    { 0x59ec22e0, SUC_lexicon_tags_1 },
    { 0xd0c22df, SUC_lexicon_tags_1 },
    { 0xc4ec22e2, SUC_lexicon_tags_60 },
    { 0x12ac22e3, SUC_lexicon_tags_7 },
    { 0x2c0c22e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630c22e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c22f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2ec22f6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4c22f9, SUC_lexicon_tags_25 },
    { 0x646c22f9, SUC_lexicon_tags_25 },
    { 0xb5ec22fb, SUC_lexicon_tags_17 },
    { 0x320c22fc, SUC_lexicon_tags_0 },
    { 0x752c22fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cc2300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a0c2302, SUC_lexicon_tags_0 },
    { 0xdb2c2303, SUC_lexicon_tags_15 },
    { 0xadac2304, SUC_lexicon_tags_26 },
    { 0xefcc2305, SUC_lexicon_tags_37 },
    { 0xd92c2306, SUC_lexicon_tags_11 },
    { 0xac8c2303, SUC_lexicon_tags_37 },
    { 0x690c2308, SUC_lexicon_tags_39 },
    { 0x8b8c2309, SUC_lexicon_tags_27 },
    { 0x88ac230a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdec230d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e4c230f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x614c2311, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66c2313, SUC_lexicon_tags_65 },
    { 0xbf4c2314, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81cc2319, SUC_lexicon_tags_16 },
    { 0xe36c231a, SUC_lexicon_tags_3 },
    { 0x42ec231b, SUC_lexicon_tags_14 },
    { 0xb9cc231c, SUC_lexicon_tags_39 },
    { 0x4f6c231d, SUC_lexicon_tags_7 },
    { 0xe9ec231d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6c2322, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4f2c2324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948c2328, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdec232b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884c232f, SUC_lexicon_tags_17 },
    { 0xb0cc2330, SUC_lexicon_tags_15 },
    { 0x978c2330, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce2c2333, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x424c2335, SUC_lexicon_tags_7 },
    { 0x5c2c2335, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x83cc2338, SUC_lexicon_tags_7 },
    { 0x788c2339, SUC_lexicon_tags_8 },
    { 0x5bcc2339, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x75ec233c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ec233f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccec2343, SUC_lexicon_tags_14 },
    { 0xd22c2344, SUC_lexicon_tags_37 },
    { 0x92ac2345, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2c2349, SUC_lexicon_tags_15 },
    { 0x606c234a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x460c234c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ac2350, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cc235b, SUC_lexicon_tags_37 },
    { 0x700c235b, SUC_lexicon_tags_11 },
    { 0x932c235c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4c2364, SUC_lexicon_tags_19 },
    { 0xd18c2364, SUC_lexicon_tags_17 },
    { 0x634c2364, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74c236a, SUC_lexicon_tags_11 },
    { 0x604c236a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92c236e, SUC_lexicon_tags_30 },
    { 0xd3ec236f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0c2372, SUC_lexicon_tags_47 },
    { 0x9c4c2372, SUC_lexicon_tags_47 },
    { 0x258c2373, SUC_lexicon_tags_31 },
    { 0x628c2375, SUC_lexicon_tags_15 },
    { 0x46ac2376, SUC_lexicon_tags_2 },
    { 0xcb0c2372, SUC_lexicon_tags_30 },
    { 0x2dcc2378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a2c237a, SUC_lexicon_tags_1 },
    { 0x6d2c237a, SUC_lexicon_tags_7 },
    { 0x954c237a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4c237f, SUC_lexicon_tags_72 },
    { 0xbc0c2380, SUC_lexicon_tags_7 },
    { 0xe6ac237f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x328c2383, SUC_lexicon_tags_23 },
    { 0x51cc2384, SUC_lexicon_tags_27 },
    { 0x1d0c2385, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x466c2387, SUC_lexicon_tags_49 },
    { 0x43cc2387, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4c238c, SUC_lexicon_tags_15 },
    { 0xf12c238c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8c2391, SUC_lexicon_tags_23 },
    { 0xcac2392, SUC_lexicon_tags_11 },
    { 0xcfac2391, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7ac2395, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x608c2398, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x354c239b, SUC_lexicon_tags_37 },
    { 0x9d4c239b, SUC_lexicon_tags_14 },
    { 0x9a6c239d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8c23a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cc23a8, SUC_lexicon_tags_47 },
    { 0xf62c23a9, SUC_lexicon_tags_45 },
    { 0xa6cc23a9, SUC_lexicon_tags_40 },
    { 0x75ec23ab, SUC_lexicon_tags_7 },
    { 0xaecc23ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4c23af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xab8c23b1, SUC_lexicon_tags_27 },
    { 0x806c23b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecc23b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8c23b7, SUC_lexicon_tags_1 },
    { 0x6b2c23b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184c23bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8c23bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4e6c23c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcc23c8, SUC_lexicon_tags_25 },
    { 0x79ec23c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eec23cd, SUC_lexicon_tags_30 },
    { 0xb3cc23ce, SUC_lexicon_tags_1 },
    { 0x46c23cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdcc23d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4c23d4, SUC_lexicon_tags_11 },
    { 0x3f0c23d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cc23d9, SUC_lexicon_tags_29 },
    { 0x338c23da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0c23dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe6c23df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cc23e3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x658c23e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44c23e7, SUC_lexicon_tags_9 },
    { 0x890c23e7, SUC_lexicon_tags_65 },
    { 0x3bcc23e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa30c23eb, SUC_lexicon_tags_7 },
    { 0x7d4c23ec, SUC_lexicon_tags_17 },
    { 0x3a6c23ec, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec23fa, SUC_lexicon_tags_49 },
    { 0x196c23fb, SUC_lexicon_tags_23 },
    { 0x7e2c23fa, SUC_lexicon_tags_12 },
    { 0x6f0c23fd, SUC_lexicon_tags_25 },
    { 0x754c23fa, SUC_lexicon_tags_57 },
    { 0xf30c23ff, SUC_lexicon_tags_17 },
    { 0x10cc2400, SUC_lexicon_tags_49 },
    { 0xb44c23ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f4c2403, SUC_lexicon_tags_14 },
    { 0xcd2c2404, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x658c2407, SUC_lexicon_tags_15 },
    { 0x4b6c2408, SUC_lexicon_tags_37 },
    { 0x746c2407, SUC_lexicon_tags_32 },
    { 0xf14c240a, SUC_lexicon_tags_63 },
    { 0x2c6c240a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc2411, SUC_lexicon_tags_37 },
    { 0xf34c2412, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8c2415, SUC_lexicon_tags_33 },
    { 0x5d4c2416, SUC_lexicon_tags_17 },
    { 0x5f6c2417, SUC_lexicon_tags_82 },
    { 0x6c4c2418, SUC_lexicon_tags_23 },
    { 0x78cc2419, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0c241d, SUC_lexicon_tags_32 },
    { 0x7b4c241e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x444c2420, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x192c2422, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ac2429, SUC_lexicon_tags_18 },
    { 0xa0ec242a, SUC_lexicon_tags_25 },
    { 0x6cac242b, SUC_lexicon_tags_31 },
    { 0x380c242a, SUC_lexicon_tags_27 },
    { 0x9ecc242d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc26c242f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dac2436, SUC_lexicon_tags_17 },
    { 0x276c2437, SUC_lexicon_tags_33 },
    { 0x474c2438, SUC_lexicon_tags_17 },
    { 0x984c2438, SUC_lexicon_tags_77 },
    { 0xbdac2438, SUC_lexicon_tags_45 },
    { 0xeac243b, SUC_lexicon_tags_15 },
    { 0xbeec243a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56c2440, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ac2443, SUC_lexicon_tags_282 },
    { 0x5ccc2443, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2c244b, SUC_lexicon_tags_1 },
    { 0xd20c244c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166c2450, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fac2453, SUC_lexicon_tags_25 },
    { 0xc2454, SUC_lexicon_tags_62 },
    { 0x6b4c2454, SUC_lexicon_tags_19 },
    { 0x738c2456, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83cc2458, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfaac245a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6c245e, SUC_lexicon_tags_21 },
    { 0xd84c245f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d2c2461, SUC_lexicon_tags_7 },
    { 0x4d4c2462, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x80c2465, SUC_lexicon_tags_27 },
    { 0x2d6c2466, SUC_lexicon_tags_21 },
    { 0x6f6c2467, SUC_lexicon_tags_11 },
    { 0x4e8c2467, SUC_lexicon_tags_11 },
    { 0x682c2466, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72c246e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c2474, SUC_lexicon_tags_12 },
    { 0xcd6c2474, SUC_lexicon_tags_129 },
    { 0xf14c2474, SUC_lexicon_tags_12 },
    { 0xca4c2475, SUC_lexicon_tags_25 },
    { 0x14ec2478, SUC_lexicon_tags_12 },
    { 0x308c2478, SUC_lexicon_tags_11 },
    { 0xc96c247a, SUC_lexicon_tags_21 },
    { 0xc54c247a, SUC_lexicon_tags_31 },
    { 0x832c247a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacc2480, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cc2485, SUC_lexicon_tags_25 },
    { 0x456c2486, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8c2489, SUC_lexicon_tags_23 },
    { 0xb20c248a, SUC_lexicon_tags_25 },
    { 0x2c8c248b, SUC_lexicon_tags_27 },
    { 0x678c248a, SUC_lexicon_tags_7 },
    { 0x30cc248d, SUC_lexicon_tags_8 },
    { 0x8f2c248e, SUC_lexicon_tags_62 },
    { 0x8b6c248f, SUC_lexicon_tags_39 },
    { 0x5aac2490, SUC_lexicon_tags_7 },
    { 0x53cc2491, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0c2494, SUC_lexicon_tags_28 },
    { 0xd4ac2495, SUC_lexicon_tags_0 },
    { 0xa80c2496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ac249a, SUC_lexicon_tags_2 },
    { 0x9a8c249b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x438c249d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cc24a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cc24a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcc24ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7cc24ad, SUC_lexicon_tags_17 },
    { 0x228c24ae, SUC_lexicon_tags_33 },
    { 0xaacc24af, SUC_lexicon_tags_26 },
    { 0x90cc24b0, SUC_lexicon_tags_21 },
    { 0xda6c24af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80c24b5, SUC_lexicon_tags_43 },
    { 0x6b8c24b6, SUC_lexicon_tags_38 },
    { 0x680c24b7, SUC_lexicon_tags_39 },
    { 0xeac24b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36c24bb, SUC_lexicon_tags_3 },
    { 0xa8c24bc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cc24bf, SUC_lexicon_tags_17 },
    { 0xab2c24c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0c24c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3ac24c5, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730c24c9, SUC_lexicon_tags_16 },
    { 0xc6c24ca, SUC_lexicon_tags_19 },
    { 0x680c24cb, SUC_lexicon_tags_17 },
    { 0xcc4c24cc, SUC_lexicon_tags_32 },
    { 0xe5cc24cd, SUC_lexicon_tags_8 },
    { 0x248c24c9, SUC_lexicon_tags_15 },
    { 0xe80c24ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0c24d3, SUC_lexicon_tags_28 },
    { 0x6e4c24d4, SUC_lexicon_tags_15 },
    { 0x9f0c24d5, SUC_lexicon_tags_11 },
    { 0x34ec24d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260c24e2, SUC_lexicon_tags_15 },
    { 0x5f2c24e3, SUC_lexicon_tags_4 },
    { 0x42cc24e3, SUC_lexicon_tags_0 },
    { 0xe34c24e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x284c24e7, SUC_lexicon_tags_28 },
    { 0x812c24e7, SUC_lexicon_tags_151 },
    { 0x13ac24e8, SUC_lexicon_tags_37 },
    { 0xbbcc24ea, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2f4c24ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0c24f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cc24f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x36ec24f5, SUC_lexicon_tags_11 },
    { 0xe5ec24f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x984c24f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb64c24fa, SUC_lexicon_tags_17 },
    { 0x54c24fa, SUC_lexicon_tags_31 },
    { 0x364c24fc, SUC_lexicon_tags_43 },
    { 0x1acc24fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ac2501, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdac2506, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cc250b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x960c250d, SUC_lexicon_tags_8 },
    { 0x43ec250d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4ec2510, SUC_lexicon_tags_1 },
    { 0xbeec2510, SUC_lexicon_tags_2 },
    { 0x76ec2512, SUC_lexicon_tags_37 },
    { 0x57ec2511, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c0c2515, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x836c2518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944c251c, SUC_lexicon_tags_12 },
    { 0x89ec251d, SUC_lexicon_tags_8 },
    { 0x84ac251e, SUC_lexicon_tags_33 },
    { 0xb2cc251f, SUC_lexicon_tags_17 },
    { 0xf2ac251d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x662c2523, SUC_lexicon_tags_38 },
    { 0x2b0c2524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524c2528, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36c252b, SUC_lexicon_tags_60 },
    { 0x8d6c252c, SUC_lexicon_tags_28 },
    { 0x142c252c, SUC_lexicon_tags_15 },
    { 0x402c252c, SUC_lexicon_tags_11 },
    { 0x9c0c252f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbec2531, SUC_lexicon_tags_83 },
    { 0xfc6c2532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x896c2535, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50c253c, SUC_lexicon_tags_29 },
    { 0x3b8c253d, SUC_lexicon_tags_17 },
    { 0x9a2c253d, SUC_lexicon_tags_12 },
    { 0xa52c253f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22ec2541, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ec2545, SUC_lexicon_tags_56 },
    { 0xf80c2546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53ec2548, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ac254d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2c2551, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x16ac2553, SUC_lexicon_tags_2 },
    { 0xb98c2554, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc16c2556, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ec2559, SUC_lexicon_tags_25 },
    { 0xf98c255a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb12c255d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf84c255f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x32ec2561, SUC_lexicon_tags_1 },
    { 0x2bec2562, SUC_lexicon_tags_2 },
    { 0xfb2c2563, SUC_lexicon_tags_104 },
    { 0x11ac2562, SUC_lexicon_tags_29 },
    { 0x9aec2565, SUC_lexicon_tags_9 },
    { 0x380c2563, SUC_lexicon_tags_37 },
    { 0x57cc2567, SUC_lexicon_tags_2 },
    { 0x3aec2568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3cc256a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf30c256c, SUC_lexicon_tags_7 },
    { 0xc90c256d, SUC_lexicon_tags_12 },
    { 0x8a8c256e, SUC_lexicon_tags_23 },
    { 0xe20c256d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0c2572, SUC_lexicon_tags_33 },
    { 0x8d2c2572, SUC_lexicon_tags_31 },
    { 0x34c2574, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9cac2576, SUC_lexicon_tags_12 },
    { 0x7c8c2576, SUC_lexicon_tags_235 },
    { 0x866c2578, SUC_lexicon_tags_84 },
    { 0x336c2576, SUC_lexicon_tags_12 },
    { 0x890c2576, SUC_lexicon_tags_31 },
    { 0x15ac257b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x628c257e, SUC_lexicon_tags_38 },
    { 0x482c257f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x700c2581, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2c2585, SUC_lexicon_tags_1 },
    { 0x7d4c2585, SUC_lexicon_tags_2 },
    { 0x792c2587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdac2589, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618c2590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaec2593, SUC_lexicon_tags_17 },
    { 0xdd0c2593, SUC_lexicon_tags_12 },
    { 0x46ec2593, SUC_lexicon_tags_11 },
    { 0x3d0c2596, SUC_lexicon_tags_6 },
    { 0xb34c2597, SUC_lexicon_tags_37 },
    { 0xe26c2593, SUC_lexicon_tags_17 },
    { 0x16cc2599, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc25a0, SUC_lexicon_tags_33 },
    { 0xe1ec25a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42c25a8, SUC_lexicon_tags_30 },
    { 0xf38c25a9, SUC_lexicon_tags_27 },
    { 0x27ac25a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x210c25ac, SUC_lexicon_tags_2 },
    { 0xb16c25ad, SUC_lexicon_tags_1287 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10c25b4, SUC_lexicon_tags_11 },
    { 0xba2c25b5, SUC_lexicon_tags_15 },
    { 0xb20c25b6, SUC_lexicon_tags_23 },
    { 0x478c25b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4d2c25b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cc25bd, SUC_lexicon_tags_15 },
    { 0x8ccc25be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3ac25c0, SUC_lexicon_tags_17 },
    { 0x37ec25c1, SUC_lexicon_tags_31 },
    { 0x72ac25c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a6c25c4, SUC_lexicon_tags_1 },
    { 0xd0c25c4, SUC_lexicon_tags_10 },
    { 0x13ac25c6, SUC_lexicon_tags_10 },
    { 0x8bec25c6, SUC_lexicon_tags_33 },
    { 0x314c25c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cc25cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6c25d0, SUC_lexicon_tags_30 },
    { 0xf20c25d1, SUC_lexicon_tags_6 },
    { 0xec8c25d0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eac25d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c8c25d9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf28c25db, SUC_lexicon_tags_2 },
    { 0x744c25dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54c25e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78c25e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbac25e9, SUC_lexicon_tags_63 },
    { 0x2e6c25ea, SUC_lexicon_tags_11 },
    { 0x976c25eb, SUC_lexicon_tags_12 },
    { 0x8b2c25ec, SUC_lexicon_tags_19 },
    { 0x82c25ed, SUC_lexicon_tags_17 },
    { 0x1eec25eb, SUC_lexicon_tags_15 },
    { 0xb10c25ee, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8c25fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56c25ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274c2604, SUC_lexicon_tags_15 },
    { 0xd76c2605, SUC_lexicon_tags_47 },
    { 0x944c2605, SUC_lexicon_tags_37 },
    { 0x7f0c2604, SUC_lexicon_tags_12 },
    { 0x45ac2605, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x232c260a, SUC_lexicon_tags_1 },
    { 0x396c260b, SUC_lexicon_tags_11 },
    { 0xae0c260a, SUC_lexicon_tags_37 },
    { 0x1eac260d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x68c2610, SUC_lexicon_tags_45 },
    { 0x5f6c2611, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0c261a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28c261e, SUC_lexicon_tags_25 },
    { 0x638c261f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6c2624, SUC_lexicon_tags_0 },
    { 0xcec2625, SUC_lexicon_tags_43 },
    { 0x390c2625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8c2629, SUC_lexicon_tags_12 },
    { 0x62ec262a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694c2633, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccc263f, SUC_lexicon_tags_11 },
    { 0x584c2640, SUC_lexicon_tags_10 },
    { 0xc2ec2641, SUC_lexicon_tags_2 },
    { 0xef8c2641, SUC_lexicon_tags_32 },
    { 0x4e8c2643, SUC_lexicon_tags_39 },
    { 0x2eac2644, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa76c2646, SUC_lexicon_tags_7 },
    { 0x3ac2647, SUC_lexicon_tags_14 },
    { 0xd84c2648, SUC_lexicon_tags_8 },
    { 0xc06c2648, SUC_lexicon_tags_15 },
    { 0x4d6c2649, SUC_lexicon_tags_26 },
    { 0xabec2649, SUC_lexicon_tags_63 },
    { 0xed8c264c, SUC_lexicon_tags_7 },
    { 0xfa6c264b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4c2657, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12c265a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88c265d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448c2661, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bec2668, SUC_lexicon_tags_17 },
    { 0xd7ec2669, SUC_lexicon_tags_37 },
    { 0x66c266a, SUC_lexicon_tags_25 },
    { 0x6fec266b, SUC_lexicon_tags_63 },
    { 0x9fcc266b, SUC_lexicon_tags_30 },
    { 0xb0ac266d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2c2670, SUC_lexicon_tags_16 },
    { 0x71cc2671, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecc2679, SUC_lexicon_tags_1 },
    { 0xb9cc267a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x70cc267c, SUC_lexicon_tags_6 },
    { 0x38c267d, SUC_lexicon_tags_7 },
    { 0xc92c267d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ec2683, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ac268b, SUC_lexicon_tags_37 },
    { 0x6b2c268c, SUC_lexicon_tags_45 },
    { 0x8bcc268b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ec269a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb8c269c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6c0c269e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ec26a5, SUC_lexicon_tags_839 },
    { 0x1d2c26a5, SUC_lexicon_tags_29 },
    { 0xf62c26a6, SUC_lexicon_tags_15 },
    { 0x4d2c26a8, SUC_lexicon_tags_160 },
    { 0xc74c26a8, SUC_lexicon_tags_11 },
    { 0xf0cc26aa, SUC_lexicon_tags_7 },
    { 0xf08c26a8, SUC_lexicon_tags_13 },
    { 0x938c26aa, SUC_lexicon_tags_11 },
    { 0x7a6c26a7, SUC_lexicon_tags_63 },
    { 0x86ec26a9, SUC_lexicon_tags_26 },
    { 0xb92c26af, SUC_lexicon_tags_51 },
    { 0x256c26b0, SUC_lexicon_tags_9 },
    { 0x3d8c26b1, SUC_lexicon_tags_64 },
    { 0x78cc26b0, SUC_lexicon_tags_66 },
    { 0xbb2c26ad, SUC_lexicon_tags_27 },
    { 0x89cc26b4, SUC_lexicon_tags_65 },
    { 0xea6c26b5, SUC_lexicon_tags_19 },
    { 0xcd6c26b6, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x240c26b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ac26bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472c26c5, SUC_lexicon_tags_48 },
    { 0xabac26c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0c26c9, SUC_lexicon_tags_55 },
    { 0x5b8c26ca, SUC_lexicon_tags_47 },
    { 0x10c26cb, SUC_lexicon_tags_1 },
    { 0x6f0c26cc, SUC_lexicon_tags_15 },
    { 0xa80c26cc, SUC_lexicon_tags_47 },
    { 0x686c26ce, SUC_lexicon_tags_402 },
    { 0x9b0c26cc, SUC_lexicon_tags_15 },
    { 0x7d8c26d0, SUC_lexicon_tags_37 },
    { 0x2cac26d1, SUC_lexicon_tags_4 },
    { 0x49ac26d2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x926c26d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cc26d9, SUC_lexicon_tags_33 },
    { 0xdc0c26da, SUC_lexicon_tags_87 },
    { 0xbfcc26db, SUC_lexicon_tags_33 },
    { 0x34cc26dc, SUC_lexicon_tags_14 },
    { 0x402c26dc, SUC_lexicon_tags_15 },
    { 0x26c26da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96c26e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6c26e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb0c26e6, SUC_lexicon_tags_19 },
    { 0xd92c26e7, SUC_lexicon_tags_57 },
    { 0x8a6c26e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cec26ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x922c26ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x840c26ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc26f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x26c26f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78c26fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ec26fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd44c2700, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12c2707, SUC_lexicon_tags_11 },
    { 0x590c2708, SUC_lexicon_tags_17 },
    { 0xb5cc2709, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12c270f, SUC_lexicon_tags_17 },
    { 0x42ac2710, SUC_lexicon_tags_0 },
    { 0x532c270f, SUC_lexicon_tags_12 },
    { 0x9f2c2712, SUC_lexicon_tags_6 },
    { 0xba2c2713, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92c2718, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ac271d, SUC_lexicon_tags_29 },
    { 0x70ac271e, SUC_lexicon_tags_37 },
    { 0x484c271f, SUC_lexicon_tags_332 },
    { 0x9dac2720, SUC_lexicon_tags_11 },
    { 0x6eac2721, SUC_lexicon_tags_49 },
    { 0x32ec271f, SUC_lexicon_tags_17 },
    { 0x7f8c2722, SUC_lexicon_tags_114 },
    { 0xdb2c2721, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ec2729, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552c272e, SUC_lexicon_tags_63 },
    { 0x576c272f, SUC_lexicon_tags_37 },
    { 0xcc6c2730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64c2733, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46c2735, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cc2739, SUC_lexicon_tags_17 },
    { 0x214c273a, SUC_lexicon_tags_69 },
    { 0x966c273a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64c2742, SUC_lexicon_tags_43 },
    { 0x226c2743, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118c2748, SUC_lexicon_tags_48 },
    { 0x4f6c2748, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a6c274b, SUC_lexicon_tags_37 },
    { 0x6ecc274b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf30c274e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8c2751, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac2755, SUC_lexicon_tags_1 },
    { 0xf3cc2755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36c275d, SUC_lexicon_tags_30 },
    { 0x29ac275e, SUC_lexicon_tags_25 },
    { 0xaeac275f, SUC_lexicon_tags_1 },
    { 0xf54c2760, SUC_lexicon_tags_16 },
    { 0xf9cc275f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc84c2763, SUC_lexicon_tags_65 },
    { 0x1b0c2763, SUC_lexicon_tags_59 },
    { 0x948c2763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefac2768, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0c276c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2c276f, SUC_lexicon_tags_12 },
    { 0x850c276f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdec2772, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2c2779, SUC_lexicon_tags_33 },
    { 0xeaec2779, SUC_lexicon_tags_37 },
    { 0x276c2779, SUC_lexicon_tags_11 },
    { 0x254c277c, SUC_lexicon_tags_31 },
    { 0xd8ac277c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd8c277f, SUC_lexicon_tags_25 },
    { 0x784c277f, SUC_lexicon_tags_47 },
    { 0xc84c2781, SUC_lexicon_tags_17 },
    { 0xd92c2782, SUC_lexicon_tags_7 },
    { 0xc68c2782, SUC_lexicon_tags_37 },
    { 0xc70c2781, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2a6c2786, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6c278a, SUC_lexicon_tags_56 },
    { 0x634c278a, SUC_lexicon_tags_1 },
    { 0xfcc278c, SUC_lexicon_tags_60 },
    { 0x53ec278b, SUC_lexicon_tags_16 },
    { 0x4e6c278e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c2791, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x584c2794, SUC_lexicon_tags_26 },
    { 0xdd0c2795, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadcc279b, SUC_lexicon_tags_15 },
    { 0xc68c279c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ac27a2, SUC_lexicon_tags_14 },
    { 0x68ec27a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xecc27a5, SUC_lexicon_tags_47 },
    { 0x144c27a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6c27ac, SUC_lexicon_tags_6 },
    { 0x6eac27ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892c27b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dac27b5, SUC_lexicon_tags_9 },
    { 0x520c27b6, SUC_lexicon_tags_3 },
    { 0x44ac27b6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890c27bd, SUC_lexicon_tags_49 },
    { 0xa50c27be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782c27c3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x708c27c5, SUC_lexicon_tags_7 },
    { 0xe98c27c6, SUC_lexicon_tags_32 },
    { 0x208c27c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ec27cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9bcc27cd, SUC_lexicon_tags_38 },
    { 0xe7cc27ce, SUC_lexicon_tags_33 },
    { 0xbf0c27cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670c27d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c27d6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c27da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b6c27dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cc27e1, SUC_lexicon_tags_15 },
    { 0x82ec27e1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x350c27e4, SUC_lexicon_tags_63 },
    { 0x4c6c27e5, SUC_lexicon_tags_12 },
    { 0xe16c27e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44ec27e8, SUC_lexicon_tags_25 },
    { 0xd26c27e9, SUC_lexicon_tags_1 },
    { 0xaa4c27e9, SUC_lexicon_tags_47 },
    { 0x91ec27eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaac27f0, SUC_lexicon_tags_25 },
    { 0x3dac27f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cc27f5, SUC_lexicon_tags_11 },
    { 0xbd0c27f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfac27f9, SUC_lexicon_tags_47 },
    { 0xce0c27fa, SUC_lexicon_tags_1 },
    { 0xf76c27fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ac27fe, SUC_lexicon_tags_11 },
    { 0xbc6c27ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcc2805, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbacc2807, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5b0c2809, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x106c280b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164c280f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcc2813, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d2c2815, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cc2818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bcc281a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x564c281d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212c2826, SUC_lexicon_tags_4 },
    { 0xa40c2827, SUC_lexicon_tags_17 },
    { 0xdc8c2827, SUC_lexicon_tags_17 },
    { 0xd94c2826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34c282b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4c2832, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0x19cc2835, SUC_lexicon_tags_25 },
    { 0xb96c2836, SUC_lexicon_tags_11 },
    { 0xfd4c2836, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142c283d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x118c283f, SUC_lexicon_tags_4 },
    { 0xe0ec2840, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dec2846, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40c284e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fec2854, SUC_lexicon_tags_49 },
    { 0x8c8c2854, SUC_lexicon_tags_1 },
    { 0x74c2856, SUC_lexicon_tags_32 },
    { 0xff6c2857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6c285a, SUC_lexicon_tags_79 },
    { 0xdc4c285b, SUC_lexicon_tags_17 },
    { 0xf08c285c, SUC_lexicon_tags_37 },
    { 0xbd2c285a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2c2860, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc2863, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5b2c2865, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fec2867, SUC_lexicon_tags_47 },
    { 0xfe2c2868, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ac286b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xebec286d, SUC_lexicon_tags_0 },
    { 0x128c286e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41cc2870, SUC_lexicon_tags_60 },
    { 0x2b6c2871, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8c2875, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc6c2877, SUC_lexicon_tags_13 },
    { 0x9a0c2878, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18c287b, SUC_lexicon_tags_38 },
    { 0xe40c287c, SUC_lexicon_tags_25 },
    { 0x41ec287d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x816c2880, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ac2883, SUC_lexicon_tags_7 },
    { 0x94c2884, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5acc2886, SUC_lexicon_tags_49 },
    { 0x438c2886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcacc2889, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394c288d, SUC_lexicon_tags_25 },
    { 0xad6c288e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x466c2891, SUC_lexicon_tags_396 },
    { 0xd88c2891, SUC_lexicon_tags_25 },
    { 0x490c2893, SUC_lexicon_tags_38 },
    { 0x39ac2894, SUC_lexicon_tags_2 },
    { 0xc9ac2895, SUC_lexicon_tags_15 },
    { 0xfcec2896, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c2899, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cc289c, SUC_lexicon_tags_12 },
    { 0x1fcc289d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ec28a1, SUC_lexicon_tags_45 },
    { 0xe3ac28a2, SUC_lexicon_tags_17 },
    { 0x114c28a2, SUC_lexicon_tags_11 },
    { 0x9e2c28a2, SUC_lexicon_tags_1 },
    { 0x6e4c28a3, SUC_lexicon_tags_19 },
    { 0xf88c28a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2c28ab, SUC_lexicon_tags_7 },
    { 0x9e4c28ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276c28b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ec28b8, SUC_lexicon_tags_16 },
    { 0xe5ec28b9, SUC_lexicon_tags_15 },
    { 0xfe8c28ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1ec28bc, SUC_lexicon_tags_64 },
    { 0x90ac28bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aac28c0, SUC_lexicon_tags_68 },
    { 0x69cc28c1, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x8b6c28c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcac28c8, SUC_lexicon_tags_38 },
    { 0x7e4c28c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec28cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0c28d4, SUC_lexicon_tags_14 },
    { 0x746c28d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x168c28d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfd4c28da, SUC_lexicon_tags_11 },
    { 0xc4c28db, SUC_lexicon_tags_17 },
    { 0xf44c28dc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8c28df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80c28e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6c28e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8c28ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeac28f3, SUC_lexicon_tags_27 },
    { 0xf64c28f3, SUC_lexicon_tags_6 },
    { 0x178c28f3, SUC_lexicon_tags_27 },
    { 0xca4c28f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cc28fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e0c28fc, SUC_lexicon_tags_45 },
    { 0x9a0c28fd, SUC_lexicon_tags_46 },
    { 0x828c28fe, SUC_lexicon_tags_11 },
    { 0x60cc28fc, SUC_lexicon_tags_15 },
    { 0x456c2900, SUC_lexicon_tags_17 },
    { 0xf6ac2901, SUC_lexicon_tags_47 },
    { 0x696c2902, SUC_lexicon_tags_11 },
    { 0x988c2903, SUC_lexicon_tags_38 },
    { 0xb54c2902, SUC_lexicon_tags_23 },
    { 0x1a8c28fe, SUC_lexicon_tags_1 },
    { 0x80cc2900, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96c2908, SUC_lexicon_tags_2 },
    { 0xf2ec2909, SUC_lexicon_tags_32 },
    { 0xec0c2909, SUC_lexicon_tags_15 },
    { 0x5dcc290a, SUC_lexicon_tags_30 },
    { 0x708c290c, SUC_lexicon_tags_13 },
    { 0xcbec290c, SUC_lexicon_tags_43 },
    { 0x2e8c2908, SUC_lexicon_tags_21 },
    { 0x574c290f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9ec2911, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10cc2915, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7c2c2917, SUC_lexicon_tags_25 },
    { 0x7d8c2918, SUC_lexicon_tags_35 },
    { 0x276c2917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70c291d, SUC_lexicon_tags_2 },
    { 0x5cc291e, SUC_lexicon_tags_56 },
    { 0xcd8c291e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0c2929, SUC_lexicon_tags_31 },
    { 0xd22c292a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x918c292d, SUC_lexicon_tags_37 },
    { 0xf2ac292e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x376c2930, SUC_lexicon_tags_7 },
    { 0xe5ac2930, SUC_lexicon_tags_15 },
    { 0x62ec2932, SUC_lexicon_tags_17 },
    { 0x890c2933, SUC_lexicon_tags_61 },
    { 0xe14c2931, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbe6c2936, SUC_lexicon_tags_57 },
    { 0x44cc2937, SUC_lexicon_tags_32 },
    { 0x3d0c2938, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5cc293a, SUC_lexicon_tags_27 },
    { 0x9a0c293b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5ec293d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68c2943, SUC_lexicon_tags_43 },
    { 0xcec2944, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x37cc2946, SUC_lexicon_tags_2 },
    { 0x14ac2946, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c294f, SUC_lexicon_tags_14 },
    { 0x434c2950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604c295a, SUC_lexicon_tags_48 },
    { 0x91ac295a, SUC_lexicon_tags_25 },
    { 0xcc295c, SUC_lexicon_tags_31 },
    { 0x628c295d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc34c295f, SUC_lexicon_tags_12 },
    { 0x7b8c295f, SUC_lexicon_tags_45 },
    { 0xf70c2961, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280c2967, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefac2969, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bec2970, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dac2979, SUC_lexicon_tags_14 },
    { 0xe3cc297a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9ec297c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x354c297f, SUC_lexicon_tags_11 },
    { 0xd76c2980, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb58c2982, SUC_lexicon_tags_7 },
    { 0x702c2982, SUC_lexicon_tags_13 },
    { 0xb8ec2984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x922c2987, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc7ec2989, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4c298f, SUC_lexicon_tags_37 },
    { 0x678c2990, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacc2995, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32c2997, SUC_lexicon_tags_7 },
    { 0x35ac2997, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ac299f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x194c29a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97ec29a3, SUC_lexicon_tags_60 },
    { 0x31ec29a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890c29aa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x794c29ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfac29b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae8c29b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92c29bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564c29c0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x26ac29c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xacac29c5, SUC_lexicon_tags_63 },
    { 0x33ac29c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x582c29c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xde2c29ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfaec29cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x526c29cf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbaec29d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x862c29d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6c29db, SUC_lexicon_tags_45 },
    { 0x2b4c29db, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88c29e1, SUC_lexicon_tags_16 },
    { 0xabac29e2, SUC_lexicon_tags_7 },
    { 0xb5ec29e2, SUC_lexicon_tags_37 },
    { 0xa5ac29e4, SUC_lexicon_tags_2 },
    { 0x928c29e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ac29ed, SUC_lexicon_tags_0 },
    { 0x45ac29ee, SUC_lexicon_tags_11 },
    { 0x176c29ef, SUC_lexicon_tags_15 },
    { 0x4c29ed, SUC_lexicon_tags_30 },
    { 0x8e6c29f1, SUC_lexicon_tags_68 },
    { 0xcd4c29f2, SUC_lexicon_tags_17 },
    { 0xe60c29f1, SUC_lexicon_tags_17 },
    { 0xdcec29f4, SUC_lexicon_tags_13 },
    { 0x1aac29f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc29f8, SUC_lexicon_tags_30 },
    { 0xf1ac29f9, SUC_lexicon_tags_3 },
    { 0x662c29f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x184c29fc, SUC_lexicon_tags_38 },
    { 0xcfcc29fd, SUC_lexicon_tags_25 },
    { 0x2c2c29fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770c2a03, SUC_lexicon_tags_37 },
    { 0x5b0c2a04, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbb2c2a06, SUC_lexicon_tags_47 },
    { 0x3a4c2a07, SUC_lexicon_tags_23 },
    { 0x9f2c2a08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8c2a13, SUC_lexicon_tags_17 },
    { 0xb96c2a13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcc2a18, SUC_lexicon_tags_28 },
    { 0xf9ac2a18, SUC_lexicon_tags_14 },
    { 0xad0c2a1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6c2a22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x410c2a24, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2cc2a26, SUC_lexicon_tags_8 },
    { 0x8c8c2a26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252c2a2b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x418c2a2e, SUC_lexicon_tags_7 },
    { 0xbbac2a2f, SUC_lexicon_tags_11 },
    { 0xf7ec2a2f, SUC_lexicon_tags_37 },
    { 0x852c2a2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x71cc2a34, SUC_lexicon_tags_31 },
    { 0x54ac2a35, SUC_lexicon_tags_30 },
    { 0xfd6c2a34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf60c2a38, SUC_lexicon_tags_23 },
    { 0xb98c2a39, SUC_lexicon_tags_8 },
    { 0x906c2a39, SUC_lexicon_tags_0 },
    { 0x79ec2a3b, SUC_lexicon_tags_7 },
    { 0xd14c2a38, SUC_lexicon_tags_19 },
    { 0x7e8c2a3a, SUC_lexicon_tags_12 },
    { 0x104c2a3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ecc2a40, SUC_lexicon_tags_32 },
    { 0xb90c2a40, SUC_lexicon_tags_23 },
    { 0xe4cc2a42, SUC_lexicon_tags_0 },
    { 0xb0c2a42, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ac2a47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x510c2a4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94c2a4e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec2a51, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ac2a55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x300c2a58, SUC_lexicon_tags_12 },
    { 0x2aec2a58, SUC_lexicon_tags_70 },
    { 0xa5ec2a5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x258c2a5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20c2a5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a0c2a60, SUC_lexicon_tags_63 },
    { 0x254c2a60, SUC_lexicon_tags_23 },
    { 0xe28c2a62, SUC_lexicon_tags_28 },
    { 0x7cc2a61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8c2a67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4c2a6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x370c2a6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136c2a71, SUC_lexicon_tags_7 },
    { 0xdb8c2a72, SUC_lexicon_tags_15 },
    { 0x398c2a72, SUC_lexicon_tags_21 },
    { 0xcf4c2a74, SUC_lexicon_tags_38 },
    { 0xc08c2a74, SUC_lexicon_tags_15 },
    { 0x51ec2a76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe64c2a78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360c2a7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cc2a80, SUC_lexicon_tags_37 },
    { 0x28c2a81, SUC_lexicon_tags_91 },
    { 0x92c2a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c2a85, SUC_lexicon_tags_12 },
    { 0xe68c2a85, SUC_lexicon_tags_14 },
    { 0x53ec2a87, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464c2a8b, SUC_lexicon_tags_9 },
    { 0x868c2a8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460c2a91, SUC_lexicon_tags_35 },
    { 0x2c0c2a92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc86c2a94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x488c2a97, SUC_lexicon_tags_15 },
    { 0x788c2a98, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac2a9b, SUC_lexicon_tags_11 },
    { 0x34ec2a9b, SUC_lexicon_tags_61 },
    { 0x16c2a9d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x6bcc2a9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cc2aa7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x782c2aaa, SUC_lexicon_tags_25 },
    { 0x7fcc2aab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c2ab6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x272c2ab9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0c2abe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0c2ac2, SUC_lexicon_tags_17 },
    { 0x5d6c2ac3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcac2acb, SUC_lexicon_tags_17 },
    { 0xcd2c2acc, SUC_lexicon_tags_68 },
    { 0xca4c2acd, SUC_lexicon_tags_11 },
    { 0x5bcc2ace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x784c2ad0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xecc2ad3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4c2ad9, SUC_lexicon_tags_65 },
    { 0x928c2ad9, SUC_lexicon_tags_54 },
    { 0xde2c2ad9, SUC_lexicon_tags_2 },
    { 0x1bac2adc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xecac2adf, SUC_lexicon_tags_25 },
    { 0x630c2ae0, SUC_lexicon_tags_7 },
    { 0x8eec2adf, SUC_lexicon_tags_79 },
    { 0x292c2ae2, SUC_lexicon_tags_11 },
    { 0xda8c2ae3, SUC_lexicon_tags_129 },
    { 0x1c0c2ae4, SUC_lexicon_tags_0 },
    { 0x8aac2ae3, SUC_lexicon_tags_37 },
    { 0xdc2c2ae5, SUC_lexicon_tags_25 },
    { 0xd60c2ae2, SUC_lexicon_tags_10 },
    { 0xbeac2ae1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x338c2aea, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x890c2aec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740c2af2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x420c2af5, SUC_lexicon_tags_1290 },
    { 0, NULL },
    { 0x9cc2af7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ec2afb, SUC_lexicon_tags_45 },
    { 0xa92c2afc, SUC_lexicon_tags_12 },
    { 0xfb2c2afd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x54ac2aff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e4c2b01, SUC_lexicon_tags_49 },
    { 0xc62c2b02, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x304c2b05, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474c2b09, SUC_lexicon_tags_63 },
    { 0x16ec2b0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4c2b10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x648c2b12, SUC_lexicon_tags_2 },
    { 0xc38c2b13, SUC_lexicon_tags_32 },
    { 0xd72c2b13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d0c2b16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x604c2b18, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf76c2b1a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cc2b1d, SUC_lexicon_tags_25 },
    { 0x19ec2b1e, SUC_lexicon_tags_32 },
    { 0xf84c2b1e, SUC_lexicon_tags_38 },
    { 0xfd0c2b20, SUC_lexicon_tags_19 },
    { 0x916c2b1f, SUC_lexicon_tags_27 },
    { 0xeec2b21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fac2b25, SUC_lexicon_tags_38 },
    { 0x978c2b25, SUC_lexicon_tags_29 },
    { 0x8f8c2b27, SUC_lexicon_tags_17 },
    { 0x64cc2b26, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xe0ac2b2a, SUC_lexicon_tags_63 },
    { 0xa2cc2b2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2c2b2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cc2b31, SUC_lexicon_tags_17 },
    { 0x9d4c2b32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2c2b35, SUC_lexicon_tags_12 },
    { 0x890c2b36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5cc2b38, SUC_lexicon_tags_65 },
    { 0x63cc2b39, SUC_lexicon_tags_7 },
    { 0x634c2b3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe4c2b3c, SUC_lexicon_tags_32 },
    { 0xa02c2b3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x478c2b3f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6c2b42, SUC_lexicon_tags_8 },
    { 0xd84c2b42, SUC_lexicon_tags_11 },
    { 0xb42c2b44, SUC_lexicon_tags_10 },
    { 0xe94c2b45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22c2b4a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6c2b4e, SUC_lexicon_tags_2 },
    { 0x8fcc2b4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c2b52, SUC_lexicon_tags_37 },
    { 0x3e4c2b53, SUC_lexicon_tags_7 },
    { 0x444c2b54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ec2b57, SUC_lexicon_tags_63 },
    { 0x318c2b58, SUC_lexicon_tags_6 },
    { 0x556c2b59, SUC_lexicon_tags_9 },
    { 0x20cc2b57, SUC_lexicon_tags_10 },
    { 0xa28c2b5b, SUC_lexicon_tags_61 },
    { 0x910c2b5c, SUC_lexicon_tags_77 },
    { 0x26c2b5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2c2b60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ec2b65, SUC_lexicon_tags_7 },
    { 0x622c2b66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ec2b69, SUC_lexicon_tags_1 },
    { 0xe44c2b6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d4c2b6c, SUC_lexicon_tags_7 },
    { 0xf36c2b6d, SUC_lexicon_tags_15 },
    { 0x6dec2b6d, SUC_lexicon_tags_56 },
    { 0x892c2b6c, SUC_lexicon_tags_31 },
    { 0xe34c2b6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8c2b74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4c2b78, SUC_lexicon_tags_26 },
    { 0x1b0c2b78, SUC_lexicon_tags_17 },
    { 0x8e8c2b79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394c2b7e, SUC_lexicon_tags_43 },
    { 0x830c2b7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b6c2b81, SUC_lexicon_tags_36 },
    { 0x31cc2b81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744c2b87, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa08c2b89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6c2b8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90c2b96, SUC_lexicon_tags_57 },
    { 0xe96c2b96, SUC_lexicon_tags_37 },
    { 0xbfec2b98, SUC_lexicon_tags_14 },
    { 0xe66c2b99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0c2ba1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8e8c2ba3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbac2baa, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82c2bb0, SUC_lexicon_tags_25 },
    { 0x1ecc2bb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ac2bb4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38c2bb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06c2bba, SUC_lexicon_tags_60 },
    { 0x2e0c2bbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6c2bbf, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeac2bc2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170c2bc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x960c2bcb, SUC_lexicon_tags_9 },
    { 0x30c2bcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x642c2bcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366c2bd9, SUC_lexicon_tags_37 },
    { 0xedac2bda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ac2bdd, SUC_lexicon_tags_17 },
    { 0xbd4c2bde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde6c2be0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde2c2be2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0cc2be4, SUC_lexicon_tags_27 },
    { 0x1f0c2be5, SUC_lexicon_tags_27 },
    { 0x2ecc2be6, SUC_lexicon_tags_0 },
    { 0xf10c2be7, SUC_lexicon_tags_32 },
    { 0xfdac2be8, SUC_lexicon_tags_15 },
    { 0x5f4c2be6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34c2bf1, SUC_lexicon_tags_63 },
    { 0x776c2bf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cc2bf8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c2bfc, SUC_lexicon_tags_1 },
    { 0x618c2bfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dec2bff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8c2c02, SUC_lexicon_tags_7 },
    { 0xb0ac2c03, SUC_lexicon_tags_995 },
    { 0x114c2c04, SUC_lexicon_tags_12 },
    { 0x7f8c2c04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2c2c0c, SUC_lexicon_tags_0 },
    { 0x36cc2c0d, SUC_lexicon_tags_1 },
    { 0x870c2c0e, SUC_lexicon_tags_56 },
    { 0x6c2c0e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936c2c13, SUC_lexicon_tags_11 },
    { 0x95ec2c13, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccc2c18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00c2c1b, SUC_lexicon_tags_37 },
    { 0x792c2c1b, SUC_lexicon_tags_32 },
    { 0x372c2c1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44c2c24, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62c2c28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xecc2c2a, SUC_lexicon_tags_12 },
    { 0xf04c2c2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ec2c2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c4c2c30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdccc2c33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x290c2c36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c2c39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0c2c3e, SUC_lexicon_tags_16 },
    { 0x658c2c3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cc2c44, SUC_lexicon_tags_0 },
    { 0x1ecc2c44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6c2c4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa92c2c4e, SUC_lexicon_tags_37 },
    { 0xb30c2c4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4fcc2c51, SUC_lexicon_tags_17 },
    { 0x5a2c2c52, SUC_lexicon_tags_37 },
    { 0xfc0c2c53, SUC_lexicon_tags_33 },
    { 0xd9cc2c52, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0c2c57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf8ec2c59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36c2c5d, SUC_lexicon_tags_11 },
    { 0x320c2c5e, SUC_lexicon_tags_79 },
    { 0x4d2c2c5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cec2c63, SUC_lexicon_tags_43 },
    { 0x60ec2c64, SUC_lexicon_tags_11 },
    { 0xd2cc2c64, SUC_lexicon_tags_25 },
    { 0x1fac2c66, SUC_lexicon_tags_17 },
    { 0x1a0c2c66, SUC_lexicon_tags_7 },
    { 0xfd0c2c64, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x380c2c6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0c2c71, SUC_lexicon_tags_1 },
    { 0x252c2c71, SUC_lexicon_tags_12 },
    { 0xc52c2c73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ec2c7a, SUC_lexicon_tags_12 },
    { 0x96cc2c7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb6cc2c7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4c2c81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11ec2c83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194c2c88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8eec2c8a, SUC_lexicon_tags_0 },
    { 0xfc8c2c8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96c2c98, SUC_lexicon_tags_6 },
    { 0xae2c2c99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaec2c9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ac2ca0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75ac2ca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc2cac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6c2cb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ec2cb4, SUC_lexicon_tags_25 },
    { 0x614c2cb5, SUC_lexicon_tags_26 },
    { 0x29cc2cb6, SUC_lexicon_tags_26 },
    { 0x24ec2cb6, SUC_lexicon_tags_30 },
    { 0x730c2cb7, SUC_lexicon_tags_34 },
    { 0xd92c2cb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a8c2cbb, SUC_lexicon_tags_15 },
    { 0x6b8c2cbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x492c2cbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1cc2cc1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588c2cc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2c2cc8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cc2ccd, SUC_lexicon_tags_10 },
    { 0x682c2cce, SUC_lexicon_tags_1 },
    { 0x152c2ccf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x380c2cd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6c2cd7, SUC_lexicon_tags_23 },
    { 0xa30c2cd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4acc2cdb, SUC_lexicon_tags_11 },
    { 0x2dcc2cdc, SUC_lexicon_tags_1 },
    { 0x7d4c2cdd, SUC_lexicon_tags_0 },
    { 0x908c2cde, SUC_lexicon_tags_11 },
    { 0xd58c2cdc, SUC_lexicon_tags_12 },
    { 0x4acc2cdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fec2ce2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x704c2ce4, SUC_lexicon_tags_37 },
    { 0x16ac2ce5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf88c2ce7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2c2ced, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x550c2cef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ac2cf3, SUC_lexicon_tags_15 },
    { 0x38cc2cf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02c2cf9, SUC_lexicon_tags_31 },
    { 0x4c0c2cf9, SUC_lexicon_tags_15 },
    { 0xf36c2cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4ac2cfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cc2d00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x234c2d03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ac2d07, SUC_lexicon_tags_33 },
    { 0x6c8c2d08, SUC_lexicon_tags_7 },
    { 0x18c2d09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa68c2d0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cc2d0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12c2d18, SUC_lexicon_tags_11 },
    { 0x940c2d18, SUC_lexicon_tags_65 },
    { 0x1acc2d19, SUC_lexicon_tags_31 },
    { 0x8e8c2d1b, SUC_lexicon_tags_37 },
    { 0x7eec2d1c, SUC_lexicon_tags_25 },
    { 0xb30c2d1c, SUC_lexicon_tags_95 },
    { 0x1acc2d1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cc2d26, SUC_lexicon_tags_6 },
    { 0x89ac2d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x754c2d2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ec2d2d, SUC_lexicon_tags_85 },
    { 0xdcc2d2e, SUC_lexicon_tags_7 },
    { 0x9a6c2d2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628c2d38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4c2d3b, SUC_lexicon_tags_1 },
    { 0x73ec2d3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ac2d3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf16c2d41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306c2d47, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4c2d4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600c2d53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf98c2d55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x608c2d58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec2d5c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbfac2d5e, SUC_lexicon_tags_23 },
    { 0xb04c2d5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7fcc2d61, SUC_lexicon_tags_21 },
    { 0x39ac2d62, SUC_lexicon_tags_17 },
    { 0xfe6c2d63, SUC_lexicon_tags_23 },
    { 0x884c2d64, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc8ec2d66, SUC_lexicon_tags_569 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978c2d6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ec2d6f, SUC_lexicon_tags_33 },
    { 0x346c2d70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674c2d76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24c2d7a, SUC_lexicon_tags_61 },
    { 0x4acc2d7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6aac2d7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eac2d84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7d2c2d86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20c2d8a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8c2d8e, SUC_lexicon_tags_31 },
    { 0x8c0c2d8f, SUC_lexicon_tags_37 },
    { 0x120c2d8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x822c2d92, SUC_lexicon_tags_15 },
    { 0x5f6c2d93, SUC_lexicon_tags_30 },
    { 0x80ec2d92, SUC_lexicon_tags_84 },
    { 0x6dcc2d95, SUC_lexicon_tags_11 },
    { 0xbcac2d96, SUC_lexicon_tags_87 },
    { 0x566c2d97, SUC_lexicon_tags_61 },
    { 0x344c2d92, SUC_lexicon_tags_1 },
    { 0xfa2c2d95, SUC_lexicon_tags_33 },
    { 0xd0ac2d9a, SUC_lexicon_tags_12 },
    { 0x9c0c2d9b, SUC_lexicon_tags_0 },
    { 0xca2c2d9b, SUC_lexicon_tags_63 },
    { 0x612c2d98, SUC_lexicon_tags_1 },
    { 0x926c2d9e, SUC_lexicon_tags_37 },
    { 0x328c2d9e, SUC_lexicon_tags_6 },
    { 0x52cc2d9f, SUC_lexicon_tags_25 },
    { 0x9a2c2da1, SUC_lexicon_tags_17 },
    { 0xb10c2d9f, SUC_lexicon_tags_38 },
    { 0xbf6c2da3, SUC_lexicon_tags_44 },
    { 0x158c2da4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x122c2da6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc4c2da8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6c2dab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ec2daf, SUC_lexicon_tags_17 },
    { 0x732c2db0, SUC_lexicon_tags_23 },
    { 0x618c2db1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x760c2db4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cc2db7, SUC_lexicon_tags_63 },
    { 0x1fec2db7, SUC_lexicon_tags_46 },
    { 0xee8c2db7, SUC_lexicon_tags_37 },
    { 0x7c6c2dba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168c2dbe, SUC_lexicon_tags_25 },
    { 0x8aac2dbf, SUC_lexicon_tags_9 },
    { 0x62cc2dc0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x7bcc2dc2, SUC_lexicon_tags_26 },
    { 0x96c2dc3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1ac2dc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccc2dc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60ac2dc9, SUC_lexicon_tags_65 },
    { 0xf08c2dca, SUC_lexicon_tags_1 },
    { 0x5eac2dc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8c2dd1, SUC_lexicon_tags_19 },
    { 0x3cac2dd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f0c2dd4, SUC_lexicon_tags_12 },
    { 0x738c2dd5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ec2dd8, SUC_lexicon_tags_38 },
    { 0xaeec2dd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x332c2ddb, SUC_lexicon_tags_33 },
    { 0xa22c2ddc, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaec2de0, SUC_lexicon_tags_43 },
    { 0x312c2de1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92c2de3, SUC_lexicon_tags_55 },
    { 0xe44c2de3, SUC_lexicon_tags_202 },
    { 0xb30c2de3, SUC_lexicon_tags_0 },
    { 0xb3ec2de3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x200c2de8, SUC_lexicon_tags_15 },
    { 0x9ccc2de9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8c2ded, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962c2df1, SUC_lexicon_tags_25 },
    { 0x47cc2df2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc2df7, SUC_lexicon_tags_11 },
    { 0xe52c2df8, SUC_lexicon_tags_1 },
    { 0x328c2df8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x106c2dfc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844c2e01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc08c2e03, SUC_lexicon_tags_7 },
    { 0x20ac2e04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4c2e07, SUC_lexicon_tags_13 },
    { 0x1dec2e07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dac2e0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bac2e11, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2dcc2e13, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6aac2e15, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98c2e19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e2c2e1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xadcc2e1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90c2e20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e0c2e22, SUC_lexicon_tags_17 },
    { 0xc1ec2e23, SUC_lexicon_tags_7 },
    { 0xb8cc2e24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8c2e33, SUC_lexicon_tags_69 },
    { 0x48c2e34, SUC_lexicon_tags_92 },
    { 0x122c2e35, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24c2e39, SUC_lexicon_tags_25 },
    { 0xcb8c2e3a, SUC_lexicon_tags_63 },
    { 0xc54c2e3a, SUC_lexicon_tags_7 },
    { 0xfc4c2e3c, SUC_lexicon_tags_6 },
    { 0x24ac2e3c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1dec2e3f, SUC_lexicon_tags_38 },
    { 0x2e8c2e40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x926c2e42, SUC_lexicon_tags_64 },
    { 0x10ac2e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8c2e46, SUC_lexicon_tags_68 },
    { 0x79ec2e47, SUC_lexicon_tags_25 },
    { 0xb6c2e48, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26c2e51, SUC_lexicon_tags_0 },
    { 0xc30c2e52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122c2e56, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800c2e5b, SUC_lexicon_tags_15 },
    { 0x606c2e5b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aec2e65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x106c2e67, SUC_lexicon_tags_45 },
    { 0xf60c2e67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2ac2e6a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84c2e6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8c2e72, SUC_lexicon_tags_15 },
    { 0x62c2e72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88c2e76, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcc2e79, SUC_lexicon_tags_23 },
    { 0x1aac2e7a, SUC_lexicon_tags_120 },
    { 0x7a4c2e7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x918c2e7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ec2e80, SUC_lexicon_tags_60 },
    { 0x26cc2e81, SUC_lexicon_tags_7 },
    { 0x334c2e81, SUC_lexicon_tags_6 },
    { 0xaeec2e82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76c2e89, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960c2e8e, SUC_lexicon_tags_37 },
    { 0xb02c2e8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x152c2e91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x55ec2e93, SUC_lexicon_tags_220 },
    { 0xc00c2e94, SUC_lexicon_tags_11 },
    { 0x9d8c2e95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23ac2e97, SUC_lexicon_tags_1666 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8c2e9e, SUC_lexicon_tags_27 },
    { 0xeb8c2e9f, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x368c2ea1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf66c2ea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dac2ea9, SUC_lexicon_tags_12 },
    { 0x1cc2eaa, SUC_lexicon_tags_17 },
    { 0x3fac2eab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xceac2ead, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68c2eb1, SUC_lexicon_tags_19 },
    { 0x1bac2eb2, SUC_lexicon_tags_38 },
    { 0x61ac2eb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8c2eba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a8c2ebc, SUC_lexicon_tags_47 },
    { 0x64ac2ebc, SUC_lexicon_tags_1 },
    { 0xb24c2ebe, SUC_lexicon_tags_9 },
    { 0x558c2ebe, SUC_lexicon_tags_19 },
    { 0xc9ac2ebf, SUC_lexicon_tags_17 },
    { 0xd9ac2ebe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2c2ec5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acc2ec8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4c2ecb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec8c2ecd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x58c2ecf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cc2ed2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154c2ed7, SUC_lexicon_tags_42 },
    { 0x3c6c2ed8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ac2edb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c2c2edd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384c2ee1, SUC_lexicon_tags_29 },
    { 0x988c2ee2, SUC_lexicon_tags_11 },
    { 0x9c6c2ee2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2c2ee7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x964c2eea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecac2ef1, SUC_lexicon_tags_1 },
    { 0x8f4c2ef2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ac2ef5, SUC_lexicon_tags_11 },
    { 0x6f2c2ef6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x460c2ef8, SUC_lexicon_tags_35 },
    { 0xe54c2ef8, SUC_lexicon_tags_7 },
    { 0x310c2ef9, SUC_lexicon_tags_19 },
    { 0x93ec2efb, SUC_lexicon_tags_15 },
    { 0x418c2efc, SUC_lexicon_tags_15 },
    { 0xe5ac2ef8, SUC_lexicon_tags_25 },
    { 0x2d6c2efe, SUC_lexicon_tags_23 },
    { 0xb7ec2ef8, SUC_lexicon_tags_2 },
    { 0xfacc2efd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112c2f07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2aac2f09, SUC_lexicon_tags_1147 },
    { 0x8e4c2f0a, SUC_lexicon_tags_17 },
    { 0xac2f09, SUC_lexicon_tags_63 },
    { 0x7bcc2f0c, SUC_lexicon_tags_11 },
    { 0xaeec2f0a, SUC_lexicon_tags_8 },
    { 0xd18c2f0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccc2f12, SUC_lexicon_tags_12 },
    { 0xe96c2f13, SUC_lexicon_tags_65 },
    { 0x80ec2f13, SUC_lexicon_tags_17 },
    { 0x4b8c2f15, SUC_lexicon_tags_23 },
    { 0x30c2f16, SUC_lexicon_tags_45 },
    { 0x6ac2f17, SUC_lexicon_tags_11 },
    { 0xef2c2f12, SUC_lexicon_tags_0 },
    { 0xd78c2f19, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x724c2f1c, SUC_lexicon_tags_46 },
    { 0x596c2f1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8c2f22, SUC_lexicon_tags_11 },
    { 0x3bcc2f22, SUC_lexicon_tags_15 },
    { 0xe48c2f24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62c2f27, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x14cc2f2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xecc2f2c, SUC_lexicon_tags_62 },
    { 0x2a2c2f2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x506c2f2f, SUC_lexicon_tags_11 },
    { 0x6bcc2f2f, SUC_lexicon_tags_91 },
    { 0xba2c2f31, SUC_lexicon_tags_39 },
    { 0xc22c2f31, SUC_lexicon_tags_47 },
    { 0xfc8c2f31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80cc2f35, SUC_lexicon_tags_29 },
    { 0x7fcc2f36, SUC_lexicon_tags_65 },
    { 0xa1ac2f36, SUC_lexicon_tags_15 },
    { 0x38ec2f37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x206c2f3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802c2f40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde0c2f42, SUC_lexicon_tags_11 },
    { 0x3f4c2f43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ac2f47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa80c2f49, SUC_lexicon_tags_100 },
    { 0x440c2f4a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678c2f4e, SUC_lexicon_tags_60 },
    { 0x1dac2f4f, SUC_lexicon_tags_12 },
    { 0x5dac2f4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894c2f55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x380c2f57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe28c2f59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acc2f5e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x830c2f61, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfacc2f64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4dec2f66, SUC_lexicon_tags_45 },
    { 0x4fec2f67, SUC_lexicon_tags_1 },
    { 0x3e0c2f67, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34c2f6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7fac2f71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c2f74, SUC_lexicon_tags_8 },
    { 0xd82c2f75, SUC_lexicon_tags_2 },
    { 0x916c2f76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6ac2f78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62c2f7f, SUC_lexicon_tags_6 },
    { 0x3e8c2f7f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdccc2f82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242c2f87, SUC_lexicon_tags_112 },
    { 0xf94c2f88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2c2f93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc14c2f95, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x922c2f97, SUC_lexicon_tags_17 },
    { 0xacac2f97, SUC_lexicon_tags_37 },
    { 0xe46c2f99, SUC_lexicon_tags_31 },
    { 0xc0ec2f9a, SUC_lexicon_tags_12 },
    { 0x4e2c2f98, SUC_lexicon_tags_11 },
    { 0x614c2f97, SUC_lexicon_tags_0 },
    { 0x62cc2f9d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xda0c2f9f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcc2fa4, SUC_lexicon_tags_21 },
    { 0x7eac2fa5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508c2faa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6c2c2fac, SUC_lexicon_tags_11 },
    { 0x3fac2fad, SUC_lexicon_tags_7 },
    { 0xe3ec2fae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738c2fb2, SUC_lexicon_tags_31 },
    { 0xd6cc2fb3, SUC_lexicon_tags_8 },
    { 0x460c2fb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x732c2fb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90ac2fb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x124c2fbb, SUC_lexicon_tags_17 },
    { 0xdf2c2fbc, SUC_lexicon_tags_11 },
    { 0xa82c2fbc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376c2fc1, SUC_lexicon_tags_55 },
    { 0x7a4c2fc2, SUC_lexicon_tags_30 },
    { 0x364c2fc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2c2fca, SUC_lexicon_tags_9 },
    { 0x246c2fcb, SUC_lexicon_tags_31 },
    { 0xf64c2fcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ac2fce, SUC_lexicon_tags_25 },
    { 0x2cc2fce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ec2fd2, SUC_lexicon_tags_17 },
    { 0x7aac2fd2, SUC_lexicon_tags_0 },
    { 0x260c2fd4, SUC_lexicon_tags_65 },
    { 0x6ec2fd4, SUC_lexicon_tags_27 },
    { 0x584c2fd3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x824c2fd8, SUC_lexicon_tags_74 },
    { 0xa52c2fd9, SUC_lexicon_tags_12 },
    { 0x7f4c2fda, SUC_lexicon_tags_6 },
    { 0x762c2fd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x406c2fdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cc2fe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8c2fe6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f0c2fe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaec2feb, SUC_lexicon_tags_56 },
    { 0x718c2feb, SUC_lexicon_tags_42 },
    { 0x8b2c2fed, SUC_lexicon_tags_1 },
    { 0xdd2c2fed, SUC_lexicon_tags_55 },
    { 0x896c2fef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x900c2ff2, SUC_lexicon_tags_60 },
    { 0x362c2ff3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0c2ffd, SUC_lexicon_tags_12 },
    { 0x488c2ffe, SUC_lexicon_tags_9 },
    { 0x3c4c2ffe, SUC_lexicon_tags_37 },
    { 0x454c2fff, SUC_lexicon_tags_7 },
    { 0x56cc2ffe, SUC_lexicon_tags_82 },
    { 0x2e0c3001, SUC_lexicon_tags_45 },
    { 0xf8c2ffd, SUC_lexicon_tags_16 },
    { 0x498c3004, SUC_lexicon_tags_12 },
    { 0x624c3005, SUC_lexicon_tags_38 },
    { 0x864c3005, SUC_lexicon_tags_5 },
    { 0x4d4c3005, SUC_lexicon_tags_7 },
    { 0xee6c3006, SUC_lexicon_tags_25 },
    { 0x1c8c3005, SUC_lexicon_tags_37 },
    { 0x172c300a, SUC_lexicon_tags_25 },
    { 0x348c300b, SUC_lexicon_tags_14 },
    { 0x1fcc300c, SUC_lexicon_tags_1 },
    { 0x174c300c, SUC_lexicon_tags_38 },
    { 0x9f4c300e, SUC_lexicon_tags_16 },
    { 0x662c300a, SUC_lexicon_tags_11 },
    { 0xfeec300c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5bcc3012, SUC_lexicon_tags_129 },
    { 0x1f6c3013, SUC_lexicon_tags_12 },
    { 0xf7ec3013, SUC_lexicon_tags_37 },
    { 0x33cc3014, SUC_lexicon_tags_55 },
    { 0xeec3015, SUC_lexicon_tags_15 },
    { 0xc2ec3017, SUC_lexicon_tags_37 },
    { 0x5eac3017, SUC_lexicon_tags_29 },
    { 0x426c3014, SUC_lexicon_tags_1 },
    { 0xef2c3014, SUC_lexicon_tags_17 },
    { 0x72ac3014, SUC_lexicon_tags_26 },
    { 0xd62c301c, SUC_lexicon_tags_39 },
    { 0x472c301a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xacac301f, SUC_lexicon_tags_7 },
    { 0xf64c3020, SUC_lexicon_tags_23 },
    { 0x93cc301f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ec3025, SUC_lexicon_tags_15 },
    { 0xb48c3026, SUC_lexicon_tags_87 },
    { 0x714c3025, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6fac3029, SUC_lexicon_tags_2 },
    { 0x8c4c302a, SUC_lexicon_tags_1 },
    { 0xe4cc302b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf58c302d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8c3031, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aec3038, SUC_lexicon_tags_31 },
    { 0xcc0c3039, SUC_lexicon_tags_2 },
    { 0xec4c3038, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x618c303d, SUC_lexicon_tags_0 },
    { 0x9b6c303e, SUC_lexicon_tags_29 },
    { 0x696c303f, SUC_lexicon_tags_46 },
    { 0xe54c3040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bcc3042, SUC_lexicon_tags_11 },
    { 0x906c3042, SUC_lexicon_tags_47 },
    { 0x1b2c3043, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc4c3046, SUC_lexicon_tags_43 },
    { 0x8d6c3047, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x378c304a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaacc304c, SUC_lexicon_tags_11 },
    { 0xcac304c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58c3052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ec3056, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x222c3059, SUC_lexicon_tags_87 },
    { 0x256c305a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4c3061, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x302c3063, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ac3066, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4c306a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686c3070, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520c3077, SUC_lexicon_tags_33 },
    { 0xa14c3077, SUC_lexicon_tags_18 },
    { 0x672c3079, SUC_lexicon_tags_11 },
    { 0x420c3078, SUC_lexicon_tags_8 },
    { 0x7c8c307b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490c307f, SUC_lexicon_tags_27 },
    { 0xfe8c307f, SUC_lexicon_tags_7 },
    { 0xb84c3081, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ec3086, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30c308b, SUC_lexicon_tags_12 },
    { 0x6d2c308b, SUC_lexicon_tags_25 },
    { 0x580c308d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ec3090, SUC_lexicon_tags_2 },
    { 0x4e6c3091, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962c3096, SUC_lexicon_tags_17 },
    { 0x4ac3097, SUC_lexicon_tags_66 },
    { 0xfb0c3096, SUC_lexicon_tags_37 },
    { 0x58ec3099, SUC_lexicon_tags_11 },
    { 0x36ac309a, SUC_lexicon_tags_25 },
    { 0xc1ac309b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0c309d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6c30a3, SUC_lexicon_tags_33 },
    { 0x2c0c30a4, SUC_lexicon_tags_15 },
    { 0x70ec30a5, SUC_lexicon_tags_17 },
    { 0x758c30a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x694c30a8, SUC_lexicon_tags_14 },
    { 0x7b8c30a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dec30ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaac30ae, SUC_lexicon_tags_2 },
    { 0x494c30ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x46ac30b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ec30b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6c30be, SUC_lexicon_tags_28 },
    { 0xa82c30bf, SUC_lexicon_tags_27 },
    { 0x63ec30be, SUC_lexicon_tags_25 },
    { 0xebac30be, SUC_lexicon_tags_6 },
    { 0xecc30c2, SUC_lexicon_tags_37 },
    { 0x6b0c30be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ac30c6, SUC_lexicon_tags_14 },
    { 0x636c30c7, SUC_lexicon_tags_2 },
    { 0xa58c30c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2c30d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x954c30d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5c0c30d4, SUC_lexicon_tags_17 },
    { 0xcf6c30d5, SUC_lexicon_tags_28 },
    { 0x950c30d5, SUC_lexicon_tags_14 },
    { 0x16cc30d7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7c6c30d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452c30e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6c30e7, SUC_lexicon_tags_16 },
    { 0xeeac30e7, SUC_lexicon_tags_47 },
    { 0x448c30e9, SUC_lexicon_tags_17 },
    { 0x26cc30e7, SUC_lexicon_tags_11 },
    { 0x180c30eb, SUC_lexicon_tags_7 },
    { 0x7bcc30ec, SUC_lexicon_tags_28 },
    { 0xfe0c30ed, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8c30f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff0c30f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ac30f7, SUC_lexicon_tags_48 },
    { 0xd3cc30f8, SUC_lexicon_tags_47 },
    { 0xefec30f9, SUC_lexicon_tags_12 },
    { 0x166c30fa, SUC_lexicon_tags_2 },
    { 0x4e2c30fb, SUC_lexicon_tags_11 },
    { 0xdc2c30f7, SUC_lexicon_tags_17 },
    { 0x704c30fa, SUC_lexicon_tags_7 },
    { 0xde0c30f8, SUC_lexicon_tags_25 },
    { 0x7cec30ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73ac3101, SUC_lexicon_tags_84 },
    { 0xf6ac3102, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x850c3105, SUC_lexicon_tags_15 },
    { 0x97ac3106, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c3109, SUC_lexicon_tags_11 },
    { 0x7c6c310a, SUC_lexicon_tags_25 },
    { 0x5c4c310a, SUC_lexicon_tags_27 },
    { 0xab6c310c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534c3110, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816c3114, SUC_lexicon_tags_33 },
    { 0x860c3115, SUC_lexicon_tags_63 },
    { 0xd08c3116, SUC_lexicon_tags_11 },
    { 0xc28c3115, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6bec3119, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefcc311b, SUC_lexicon_tags_3 },
    { 0x2f6c311b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cc3121, SUC_lexicon_tags_17 },
    { 0xc24c3121, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420c3126, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0c3129, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeec312f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ac3133, SUC_lexicon_tags_11 },
    { 0xb02c3134, SUC_lexicon_tags_54 },
    { 0xf38c3134, SUC_lexicon_tags_45 },
    { 0x6b4c3136, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffac313f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac3147, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68c314c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x14cc314e, SUC_lexicon_tags_1 },
    { 0x15cc314e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568c3154, SUC_lexicon_tags_49 },
    { 0xb18c3155, SUC_lexicon_tags_12 },
    { 0x7fcc3154, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0c3159, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fec315d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3163, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158c3170, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x304c3172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8c3175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ec317d, SUC_lexicon_tags_0 },
    { 0x580c317e, SUC_lexicon_tags_19 },
    { 0x778c317e, SUC_lexicon_tags_37 },
    { 0xb72c317d, SUC_lexicon_tags_15 },
    { 0x286c3181, SUC_lexicon_tags_43 },
    { 0xa6ec3181, SUC_lexicon_tags_14 },
    { 0x840c3183, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x222c3186, SUC_lexicon_tags_38 },
    { 0x2c8c3187, SUC_lexicon_tags_37 },
    { 0x52cc3186, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29ec318a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100c318e, SUC_lexicon_tags_11 },
    { 0x1bec318f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x748c3192, SUC_lexicon_tags_14 },
    { 0x74c3193, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ac3196, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x192c3199, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6c319e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dac31a0, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164c31a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb56c31a9, SUC_lexicon_tags_7 },
    { 0x5f4c31a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198c31ae, SUC_lexicon_tags_15 },
    { 0x4bec31af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac31b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340c31ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2c31be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94ac31c0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bac31cc, SUC_lexicon_tags_7 },
    { 0xedcc31cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4c31d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76c31d7, SUC_lexicon_tags_15 },
    { 0x6b8c31d8, SUC_lexicon_tags_37 },
    { 0xe4c31d9, SUC_lexicon_tags_25 },
    { 0x330c31d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810c31df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x490c31e1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0c31e9, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6c31f0, SUC_lexicon_tags_14 },
    { 0xfcac31f1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x542c31f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd4c31f5, SUC_lexicon_tags_19 },
    { 0xae0c31f5, SUC_lexicon_tags_7 },
    { 0xe14c31f7, SUC_lexicon_tags_29 },
    { 0x5ec31f7, SUC_lexicon_tags_37 },
    { 0x2cec31f7, SUC_lexicon_tags_2 },
    { 0x8eec31f9, SUC_lexicon_tags_37 },
    { 0x2f8c31fb, SUC_lexicon_tags_11 },
    { 0x76cc31fc, SUC_lexicon_tags_38 },
    { 0x2a0c31fd, SUC_lexicon_tags_15 },
    { 0x1eec31fe, SUC_lexicon_tags_2 },
    { 0xd0ac31ff, SUC_lexicon_tags_46 },
    { 0x174c31ff, SUC_lexicon_tags_37 },
    { 0x748c3200, SUC_lexicon_tags_43 },
    { 0x702c31ff, SUC_lexicon_tags_12 },
    { 0xdcc3203, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ec320d, SUC_lexicon_tags_12 },
    { 0x3d0c320e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe3ac3210, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x678c3212, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf5cc3214, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612c321a, SUC_lexicon_tags_49 },
    { 0x452c321b, SUC_lexicon_tags_83 },
    { 0x44ec321a, SUC_lexicon_tags_86 },
    { 0x314c321d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecc3220, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acc3225, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0c3229, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0c322f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35ec3231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe12c3233, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cc3236, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708c3240, SUC_lexicon_tags_37 },
    { 0xd5cc3241, SUC_lexicon_tags_107 },
    { 0x8aec3242, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ac3247, SUC_lexicon_tags_45 },
    { 0xfbac3248, SUC_lexicon_tags_35 },
    { 0x49cc3247, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x708c324b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cc3251, SUC_lexicon_tags_65 },
    { 0x8acc3251, SUC_lexicon_tags_62 },
    { 0x460c3252, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38c325a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x67cc325c, SUC_lexicon_tags_18 },
    { 0xc9ac325c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x628c325f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7dac3261, SUC_lexicon_tags_37 },
    { 0xe02c3262, SUC_lexicon_tags_45 },
    { 0xa1ec3261, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc326b, SUC_lexicon_tags_21 },
    { 0x428c326c, SUC_lexicon_tags_1 },
    { 0xbfcc326d, SUC_lexicon_tags_7 },
    { 0xd6ac326d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3274, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd7ec3276, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcc327d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dec3280, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0xe80c3282, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590c3288, SUC_lexicon_tags_25 },
    { 0x2a0c3289, SUC_lexicon_tags_45 },
    { 0xa2ac328a, SUC_lexicon_tags_35 },
    { 0xdaec328a, SUC_lexicon_tags_6 },
    { 0x1f8c328c, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66c3295, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6b2c3297, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcc329c, SUC_lexicon_tags_26 },
    { 0x302c329d, SUC_lexicon_tags_8 },
    { 0xdd4c329e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e8c32a0, SUC_lexicon_tags_14 },
    { 0xbc4c32a1, SUC_lexicon_tags_10 },
    { 0x8a8c32a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ec32a5, SUC_lexicon_tags_17 },
    { 0xcec32a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ac32aa, SUC_lexicon_tags_0 },
    { 0x6eec32ab, SUC_lexicon_tags_43 },
    { 0xcdac32ab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6c32b0, SUC_lexicon_tags_6 },
    { 0xb4c32b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x19cc32b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdcec32b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4c32bc, SUC_lexicon_tags_0 },
    { 0xf9ec32bc, SUC_lexicon_tags_45 },
    { 0x8c6c32be, SUC_lexicon_tags_7 },
    { 0x838c32bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x90c32c2, SUC_lexicon_tags_55 },
    { 0x2dcc32c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0ec32c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e2c32c7, SUC_lexicon_tags_2 },
    { 0xb20c32c8, SUC_lexicon_tags_25 },
    { 0xb7ec32c7, SUC_lexicon_tags_207 },
    { 0xc60c32ca, SUC_lexicon_tags_12 },
    { 0xb28c32ca, SUC_lexicon_tags_12 },
    { 0xdb2c32cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280c32d0, SUC_lexicon_tags_32 },
    { 0xbccc32d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cc32d3, SUC_lexicon_tags_2 },
    { 0x40c32d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde8c32d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdac32d8, SUC_lexicon_tags_12 },
    { 0x846c32d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x984c32db, SUC_lexicon_tags_25 },
    { 0x354c32db, SUC_lexicon_tags_2 },
    { 0xcbac32dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x906c32df, SUC_lexicon_tags_49 },
    { 0x3cec32e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb60c32e2, SUC_lexicon_tags_8 },
    { 0xd30c32e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4c32e9, SUC_lexicon_tags_17 },
    { 0xa12c32e9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x39cc32ec, SUC_lexicon_tags_28 },
    { 0x1c6c32ed, SUC_lexicon_tags_1 },
    { 0x526c32ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0c32f1, SUC_lexicon_tags_29 },
    { 0x50cc32f2, SUC_lexicon_tags_56 },
    { 0xbbac32f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ac32f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110c32fb, SUC_lexicon_tags_11 },
    { 0xc56c32fc, SUC_lexicon_tags_56 },
    { 0x740c32fd, SUC_lexicon_tags_1 },
    { 0xb20c32fd, SUC_lexicon_tags_12 },
    { 0x984c32fe, SUC_lexicon_tags_7 },
    { 0xc58c32fb, SUC_lexicon_tags_15 },
    { 0xbec3300, SUC_lexicon_tags_25 },
    { 0xb32c3302, SUC_lexicon_tags_11 },
    { 0x3cec32ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac330c, SUC_lexicon_tags_37 },
    { 0x1f4c330d, SUC_lexicon_tags_7 },
    { 0xf2c330e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44c3312, SUC_lexicon_tags_79 },
    { 0x718c3312, SUC_lexicon_tags_113 },
    { 0x182c3314, SUC_lexicon_tags_14 },
    { 0x486c3314, SUC_lexicon_tags_15 },
    { 0x76c3316, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ec331b, SUC_lexicon_tags_6 },
    { 0x60ec331c, SUC_lexicon_tags_49 },
    { 0xbe4c331d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x136c331f, SUC_lexicon_tags_55 },
    { 0xb04c3320, SUC_lexicon_tags_44 },
    { 0xdf2c3320, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaec3326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ac332b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926c332f, SUC_lexicon_tags_2 },
    { 0xb0c332f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08c3336, SUC_lexicon_tags_29 },
    { 0x978c3337, SUC_lexicon_tags_18 },
    { 0x962c3336, SUC_lexicon_tags_56 },
    { 0x560c3339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac333e, SUC_lexicon_tags_17 },
    { 0xd0ec333e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3ac3341, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6c3346, SUC_lexicon_tags_522 },
    { 0x45cc3347, SUC_lexicon_tags_48 },
    { 0xcaec3348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eac334b, SUC_lexicon_tags_12 },
    { 0x20cc334c, SUC_lexicon_tags_14 },
    { 0xf8ac334d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7acc334f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9d2c3351, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x596c3353, SUC_lexicon_tags_26 },
    { 0x268c3354, SUC_lexicon_tags_11 },
    { 0x78ac3355, SUC_lexicon_tags_38 },
    { 0xc12c3356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x778c3359, SUC_lexicon_tags_27 },
    { 0xf28c335a, SUC_lexicon_tags_65 },
    { 0xa8cc335b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x696c335e, SUC_lexicon_tags_15 },
    { 0xdf6c335f, SUC_lexicon_tags_11 },
    { 0x3ecc3360, SUC_lexicon_tags_23 },
    { 0xbfcc3360, SUC_lexicon_tags_33 },
    { 0xc9ec3362, SUC_lexicon_tags_15 },
    { 0xf8ac3363, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822c3367, SUC_lexicon_tags_11 },
    { 0x120c3367, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcc336b, SUC_lexicon_tags_43 },
    { 0xddec336c, SUC_lexicon_tags_1032 },
    { 0x2e0c336c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ec3370, SUC_lexicon_tags_33 },
    { 0xcccc3370, SUC_lexicon_tags_15 },
    { 0x950c3372, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa88c3374, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88c3377, SUC_lexicon_tags_194 },
    { 0xa2cc3378, SUC_lexicon_tags_25 },
    { 0xa3cc3379, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e2c337b, SUC_lexicon_tags_29 },
    { 0x75cc337c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2c3382, SUC_lexicon_tags_2 },
    { 0x46ac3383, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528c338f, SUC_lexicon_tags_7 },
    { 0x30ac3390, SUC_lexicon_tags_26 },
    { 0xe6ac3391, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ac3397, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c339c, SUC_lexicon_tags_11 },
    { 0x3a0c339d, SUC_lexicon_tags_11 },
    { 0x878c339e, SUC_lexicon_tags_55 },
    { 0x4f8c339e, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ac33a4, SUC_lexicon_tags_12 },
    { 0x346c33a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c33aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c33bd, SUC_lexicon_tags_17 },
    { 0xdacc33be, SUC_lexicon_tags_0 },
    { 0x7b2c33bd, SUC_lexicon_tags_29 },
    { 0xcf4c33bd, SUC_lexicon_tags_3 },
    { 0x4e2c33bd, SUC_lexicon_tags_7 },
    { 0x440c33bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718c33c7, SUC_lexicon_tags_28 },
    { 0x72ec33c8, SUC_lexicon_tags_2 },
    { 0x4c2c33c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4bcc33cb, SUC_lexicon_tags_2 },
    { 0x97cc33cc, SUC_lexicon_tags_69 },
    { 0x246c33cc, SUC_lexicon_tags_38 },
    { 0x8b2c33ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24c33d1, SUC_lexicon_tags_57 },
    { 0x260c33d2, SUC_lexicon_tags_37 },
    { 0xb84c33d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5a4c33d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0c33d8, SUC_lexicon_tags_17 },
    { 0x4dac33d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84c33de, SUC_lexicon_tags_45 },
    { 0x4f2c33de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x592c33e2, SUC_lexicon_tags_27 },
    { 0xdeec33e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26c33e7, SUC_lexicon_tags_7 },
    { 0xb7cc33e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0c33ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x218c33ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c33f5, SUC_lexicon_tags_37 },
    { 0x558c33f6, SUC_lexicon_tags_48 },
    { 0x1f8c33f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aac33fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04c3400, SUC_lexicon_tags_12 },
    { 0xed4c3400, SUC_lexicon_tags_6 },
    { 0x372c3402, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x908c3405, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cc3408, SUC_lexicon_tags_1 },
    { 0x290c3409, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5dec340b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70c3410, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772c3415, SUC_lexicon_tags_30 },
    { 0x3c6c3416, SUC_lexicon_tags_25 },
    { 0x192c3417, SUC_lexicon_tags_30 },
    { 0x6c4c3415, SUC_lexicon_tags_65 },
    { 0x63cc3419, SUC_lexicon_tags_23 },
    { 0x94ac341a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfeec341c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25cc341e, SUC_lexicon_tags_60 },
    { 0x7dcc341f, SUC_lexicon_tags_63 },
    { 0xcccc3420, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ec3429, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe46c342b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6c342f, SUC_lexicon_tags_37 },
    { 0xefcc342f, SUC_lexicon_tags_11 },
    { 0xec6c3431, SUC_lexicon_tags_0 },
    { 0x8bac3432, SUC_lexicon_tags_1 },
    { 0xd64c3433, SUC_lexicon_tags_12 },
    { 0x234c3433, SUC_lexicon_tags_11 },
    { 0x4acc3431, SUC_lexicon_tags_9 },
    { 0x710c3436, SUC_lexicon_tags_35 },
    { 0xbc2c3437, SUC_lexicon_tags_1 },
    { 0xdb8c3438, SUC_lexicon_tags_33 },
    { 0xa76c3438, SUC_lexicon_tags_0 },
    { 0x4ac343a, SUC_lexicon_tags_25 },
    { 0xedac343b, SUC_lexicon_tags_33 },
    { 0x41ac343b, SUC_lexicon_tags_7 },
    { 0x17ac3437, SUC_lexicon_tags_11 },
    { 0xfa6c3436, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cc3441, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c3444, SUC_lexicon_tags_17 },
    { 0x88ec3445, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cc3449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98c3451, SUC_lexicon_tags_6 },
    { 0xef8c3452, SUC_lexicon_tags_11 },
    { 0x902c3453, SUC_lexicon_tags_22 },
    { 0xab6c3453, SUC_lexicon_tags_27 },
    { 0xf82c3455, SUC_lexicon_tags_39 },
    { 0xe12c3456, SUC_lexicon_tags_7 },
    { 0x4e8c3457, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4c345c, SUC_lexicon_tags_6 },
    { 0x70c345d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf48c345f, SUC_lexicon_tags_2 },
    { 0x4a8c3460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdec3465, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c3468, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86c346e, SUC_lexicon_tags_7 },
    { 0x66ac346f, SUC_lexicon_tags_7 },
    { 0xefcc346f, SUC_lexicon_tags_17 },
    { 0xe1cc3471, SUC_lexicon_tags_2 },
    { 0x744c3471, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa14c3474, SUC_lexicon_tags_25 },
    { 0xfd6c3474, SUC_lexicon_tags_48 },
    { 0x90ac3475, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac347d, SUC_lexicon_tags_25 },
    { 0xa36c347d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bac3486, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908c348c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe12c348e, SUC_lexicon_tags_37 },
    { 0x8e2c348e, SUC_lexicon_tags_12 },
    { 0x834c3490, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714c3494, SUC_lexicon_tags_28 },
    { 0xbe8c3494, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0c3498, SUC_lexicon_tags_33 },
    { 0xff4c3499, SUC_lexicon_tags_3 },
    { 0xe62c3499, SUC_lexicon_tags_151 },
    { 0xf02c349b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92c349f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ac34a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2dac34a4, SUC_lexicon_tags_33 },
    { 0xafac34a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ec34b0, SUC_lexicon_tags_47 },
    { 0x70ac34b0, SUC_lexicon_tags_14 },
    { 0x1b6c34b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654c34b7, SUC_lexicon_tags_23 },
    { 0x950c34b8, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4c34bb, SUC_lexicon_tags_33 },
    { 0x394c34bc, SUC_lexicon_tags_16 },
    { 0xd30c34bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf70c34bf, SUC_lexicon_tags_47 },
    { 0x1cac34c0, SUC_lexicon_tags_47 },
    { 0x366c34bf, SUC_lexicon_tags_27 },
    { 0xad2c34c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd44c34c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x86c34c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8c34ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x812c34cd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x542c34cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470c34d4, SUC_lexicon_tags_11 },
    { 0x1c4c34d5, SUC_lexicon_tags_37 },
    { 0x438c34d6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0c34de, SUC_lexicon_tags_49 },
    { 0xf00c34df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x422c34e2, SUC_lexicon_tags_65 },
    { 0x9aec34e3, SUC_lexicon_tags_25 },
    { 0x510c34e4, SUC_lexicon_tags_2 },
    { 0x7fcc34e4, SUC_lexicon_tags_2 },
    { 0x34ec34e6, SUC_lexicon_tags_1 },
    { 0x7f6c34e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6c34f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x772c34f5, SUC_lexicon_tags_26 },
    { 0xb9ec34f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8cc34f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c34fb, SUC_lexicon_tags_12 },
    { 0x5b8c34fb, SUC_lexicon_tags_3 },
    { 0x148c34fd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4c3506, SUC_lexicon_tags_35 },
    { 0xc6ac3507, SUC_lexicon_tags_11 },
    { 0xdf4c3507, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8c350b, SUC_lexicon_tags_11 },
    { 0x35ac350b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x824c350f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e0c3511, SUC_lexicon_tags_14 },
    { 0xb56c3512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ac3515, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06c351a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x104c351c, SUC_lexicon_tags_2 },
    { 0x648c351d, SUC_lexicon_tags_2 },
    { 0x1a6c351d, SUC_lexicon_tags_11 },
    { 0x682c351d, SUC_lexicon_tags_17 },
    { 0xfa0c3520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41cc3522, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x390c3525, SUC_lexicon_tags_19 },
    { 0x1e6c3525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x936c3528, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bac352b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c6c352d, SUC_lexicon_tags_6 },
    { 0x48ec352e, SUC_lexicon_tags_23 },
    { 0x280c352f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bac3531, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324c3539, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x362c353c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x36c353e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ec3541, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf9ac3543, SUC_lexicon_tags_1 },
    { 0x2d2c3544, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0c354b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4c3550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x508c3553, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa96c3555, SUC_lexicon_tags_47 },
    { 0xd8c3556, SUC_lexicon_tags_53 },
    { 0xe38c3557, SUC_lexicon_tags_25 },
    { 0x39cc3557, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb80c355a, SUC_lexicon_tags_17 },
    { 0x42c355b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa8cc355d, SUC_lexicon_tags_12 },
    { 0xd7cc355d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50ec3560, SUC_lexicon_tags_37 },
    { 0x992c3560, SUC_lexicon_tags_10 },
    { 0xe14c3560, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a2c3564, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ec3569, SUC_lexicon_tags_49 },
    { 0x5e8c356a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x360c356c, SUC_lexicon_tags_27 },
    { 0x92cc356d, SUC_lexicon_tags_28 },
    { 0x85ac356e, SUC_lexicon_tags_37 },
    { 0xc8ec356d, SUC_lexicon_tags_17 },
    { 0xa90c3570, SUC_lexicon_tags_6 },
    { 0x11ec356e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc10c3573, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22c3576, SUC_lexicon_tags_17 },
    { 0xaac3576, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6c357c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2c3580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x690c3582, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca0c3584, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ec3589, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x448c358c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd46c358e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9eac3590, SUC_lexicon_tags_45 },
    { 0xcd2c3591, SUC_lexicon_tags_37 },
    { 0xfeac3592, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242c3596, SUC_lexicon_tags_7 },
    { 0xc84c3597, SUC_lexicon_tags_11 },
    { 0xd8cc3598, SUC_lexicon_tags_11 },
    { 0x5aec3598, SUC_lexicon_tags_17 },
    { 0xe2cc3599, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb12c359d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42c35a2, SUC_lexicon_tags_57 },
    { 0xa70c35a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0c35a9, SUC_lexicon_tags_68 },
    { 0x6bac35aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb6c35ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40c35af, SUC_lexicon_tags_6 },
    { 0x992c35b0, SUC_lexicon_tags_12 },
    { 0x132c35b1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x840c35b3, SUC_lexicon_tags_17 },
    { 0xaeac35b4, SUC_lexicon_tags_3 },
    { 0x36ac35b5, SUC_lexicon_tags_32 },
    { 0x256c35b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cc35b9, SUC_lexicon_tags_11 },
    { 0x78ac35ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbec35bd, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cc35c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ac35c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x674c35ca, SUC_lexicon_tags_84 },
    { 0xbf4c35cb, SUC_lexicon_tags_15 },
    { 0x402c35cc, SUC_lexicon_tags_47 },
    { 0x4a8c35cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcc35d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6c35d3, SUC_lexicon_tags_17 },
    { 0xa7ac35d4, SUC_lexicon_tags_7 },
    { 0x836c35d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8c35d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ac35dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32c35e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d2c35e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ec35e8, SUC_lexicon_tags_0 },
    { 0x3dec35e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec35ec, SUC_lexicon_tags_15 },
    { 0xb08c35ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ac35f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0c35f4, SUC_lexicon_tags_57 },
    { 0xf68c35f4, SUC_lexicon_tags_13 },
    { 0x9eac35f5, SUC_lexicon_tags_214 },
    { 0xec8c35f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1ac35f9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x48ec35fb, SUC_lexicon_tags_60 },
    { 0x682c35fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c4c35fe, SUC_lexicon_tags_2 },
    { 0x3acc35fe, SUC_lexicon_tags_17 },
    { 0x1a0c3600, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ec3603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x108c3606, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250c360c, SUC_lexicon_tags_45 },
    { 0x286c360d, SUC_lexicon_tags_4 },
    { 0xf4ec360d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacc3612, SUC_lexicon_tags_37 },
    { 0x9e2c3613, SUC_lexicon_tags_30 },
    { 0x3c8c3613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x898c3616, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00c361b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda6c361d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26c3620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82c3623, SUC_lexicon_tags_63 },
    { 0xed4c3624, SUC_lexicon_tags_37 },
    { 0x93cc3623, SUC_lexicon_tags_2 },
    { 0x1cec3626, SUC_lexicon_tags_25 },
    { 0xebec3626, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100c3634, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6c3638, SUC_lexicon_tags_30 },
    { 0x8fcc3638, SUC_lexicon_tags_17 },
    { 0xa58c363a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82c363d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc3643, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768c3647, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x256c364a, SUC_lexicon_tags_32 },
    { 0x518c364b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbb8c364d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ac3650, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x762c3653, SUC_lexicon_tags_11 },
    { 0x30ac3654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ec365a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ac365d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x88c365f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcc3663, SUC_lexicon_tags_46 },
    { 0x7b0c3664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf6c3666, SUC_lexicon_tags_25 },
    { 0xd44c3666, SUC_lexicon_tags_7 },
    { 0x404c3667, SUC_lexicon_tags_11 },
    { 0x86cc3669, SUC_lexicon_tags_12 },
    { 0x716c366a, SUC_lexicon_tags_63 },
    { 0xcccc3669, SUC_lexicon_tags_1 },
    { 0x9bac366a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x12ac366e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dec3676, SUC_lexicon_tags_1 },
    { 0xb3cc3677, SUC_lexicon_tags_1 },
    { 0xb22c3677, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ec3681, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8c3685, SUC_lexicon_tags_104 },
    { 0xf0ec3686, SUC_lexicon_tags_33 },
    { 0x366c3687, SUC_lexicon_tags_1 },
    { 0xe8ac3688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85ec368a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x114c368c, SUC_lexicon_tags_77 },
    { 0xcc4c368d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ec3690, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4c3694, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x4d8c3696, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816c369a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2ac369c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaec36a0, SUC_lexicon_tags_47 },
    { 0x55ac36a1, SUC_lexicon_tags_37 },
    { 0x86ac36a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ac36a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d2c36a7, SUC_lexicon_tags_9 },
    { 0xd9ec36a8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690c36ac, SUC_lexicon_tags_37 },
    { 0x578c36ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8c36b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824c36b9, SUC_lexicon_tags_47 },
    { 0x4ec36ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1ec36bc, SUC_lexicon_tags_25 },
    { 0x780c36bd, SUC_lexicon_tags_36 },
    { 0xb02c36bc, SUC_lexicon_tags_27 },
    { 0xc00c36be, SUC_lexicon_tags_28 },
    { 0x7bac36c0, SUC_lexicon_tags_31 },
    { 0x2aec36c1, SUC_lexicon_tags_97 },
    { 0x25ec36c1, SUC_lexicon_tags_10 },
    { 0xa72c36c3, SUC_lexicon_tags_66 },
    { 0x326c36c3, SUC_lexicon_tags_46 },
    { 0x7fec36c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980c36ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2c36cd, SUC_lexicon_tags_25 },
    { 0x348c36cd, SUC_lexicon_tags_798 },
    { 0x2eec36ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x386c36d1, SUC_lexicon_tags_19 },
    { 0x160c36d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ec36d5, SUC_lexicon_tags_25 },
    { 0xc0cc36d5, SUC_lexicon_tags_37 },
    { 0xa3ac36d7, SUC_lexicon_tags_2 },
    { 0xa34c36d8, SUC_lexicon_tags_28 },
    { 0x86ec36d7, SUC_lexicon_tags_38 },
    { 0xe3ec36da, SUC_lexicon_tags_17 },
    { 0x7e0c36db, SUC_lexicon_tags_45 },
    { 0x9a4c36dc, SUC_lexicon_tags_32 },
    { 0xef8c36dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d2c36df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x392c36e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706c36ea, SUC_lexicon_tags_2 },
    { 0xeacc36eb, SUC_lexicon_tags_46 },
    { 0x538c36ec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x73ac36ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4c36f2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cc36fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fec36fd, SUC_lexicon_tags_98 },
    { 0xb2c36fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ec3704, SUC_lexicon_tags_48 },
    { 0x6bac3705, SUC_lexicon_tags_55 },
    { 0xe02c3706, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1cec3708, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ec370b, SUC_lexicon_tags_37 },
    { 0x4ac370c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cc3717, SUC_lexicon_tags_15 },
    { 0x226c3718, SUC_lexicon_tags_45 },
    { 0x986c3718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x928c371c, SUC_lexicon_tags_35 },
    { 0xde6c371d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2ac371f, SUC_lexicon_tags_38 },
    { 0x7bac3720, SUC_lexicon_tags_7 },
    { 0x836c3721, SUC_lexicon_tags_47 },
    { 0x550c3722, SUC_lexicon_tags_49 },
    { 0x970c3723, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72ac3725, SUC_lexicon_tags_31 },
    { 0xe48c3726, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2c3729, SUC_lexicon_tags_11 },
    { 0x34c372a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x436c372c, SUC_lexicon_tags_7 },
    { 0x90cc372d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758c3737, SUC_lexicon_tags_11 },
    { 0x376c3737, SUC_lexicon_tags_17 },
    { 0x5ac3739, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x304c373b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x808c373d, SUC_lexicon_tags_12 },
    { 0x69cc373e, SUC_lexicon_tags_33 },
    { 0x3a2c373d, SUC_lexicon_tags_23 },
    { 0xa34c3740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6cc3742, SUC_lexicon_tags_49 },
    { 0x4a8c3742, SUC_lexicon_tags_19 },
    { 0x44c3744, SUC_lexicon_tags_12 },
    { 0x44ec3742, SUC_lexicon_tags_25 },
    { 0x58c3746, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c374b, SUC_lexicon_tags_15 },
    { 0xc40c374b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88c3750, SUC_lexicon_tags_10 },
    { 0xc02c3750, SUC_lexicon_tags_2 },
    { 0x35cc3752, SUC_lexicon_tags_37 },
    { 0xf9ec3753, SUC_lexicon_tags_27 },
    { 0x51ec3754, SUC_lexicon_tags_1 },
    { 0xe96c3753, SUC_lexicon_tags_7 },
    { 0x10ac3756, SUC_lexicon_tags_17 },
    { 0x6f2c3756, SUC_lexicon_tags_25 },
    { 0x588c3754, SUC_lexicon_tags_1 },
    { 0xc10c3755, SUC_lexicon_tags_11 },
    { 0xd22c375a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac3762, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74c3768, SUC_lexicon_tags_11 },
    { 0x60c3768, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x342c376b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502c3771, SUC_lexicon_tags_370 },
    { 0, NULL },
    { 0xbc4c3773, SUC_lexicon_tags_15 },
    { 0xf48c3773, SUC_lexicon_tags_59 },
    { 0xcc3774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab6c3777, SUC_lexicon_tags_13 },
    { 0xa42c3777, SUC_lexicon_tags_33 },
    { 0x42ac3778, SUC_lexicon_tags_46 },
    { 0x110c377a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6c377e, SUC_lexicon_tags_38 },
    { 0x66cc377f, SUC_lexicon_tags_31 },
    { 0x36cc377e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ec3783, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bac378e, SUC_lexicon_tags_631 },
    { 0, NULL },
    { 0, NULL },
    { 0x828c3791, SUC_lexicon_tags_23 },
    { 0xfb2c3792, SUC_lexicon_tags_29 },
    { 0x8bec3793, SUC_lexicon_tags_60 },
    { 0xc40c3794, SUC_lexicon_tags_15 },
    { 0x960c3795, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cc379e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf08c37a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x448c37a3, SUC_lexicon_tags_19 },
    { 0x356c37a4, SUC_lexicon_tags_14 },
    { 0xb04c37a5, SUC_lexicon_tags_68 },
    { 0x360c37a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94ec37a8, SUC_lexicon_tags_7 },
    { 0xabec37a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112c37ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f2c37b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46c37b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54c37ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x32cc37bd, SUC_lexicon_tags_82 },
    { 0x4eac37bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x882c37c0, SUC_lexicon_tags_25 },
    { 0x67ec37c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6c37c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00c37ca, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296c37cf, SUC_lexicon_tags_11 },
    { 0x67ac37d0, SUC_lexicon_tags_27 },
    { 0x416c37cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4c37d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742c37e2, SUC_lexicon_tags_25 },
    { 0xef4c37e3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x408c37e6, SUC_lexicon_tags_6 },
    { 0xda0c37e6, SUC_lexicon_tags_37 },
    { 0xea4c37e8, SUC_lexicon_tags_39 },
    { 0x204c37e6, SUC_lexicon_tags_65 },
    { 0xd2cc37ea, SUC_lexicon_tags_1 },
    { 0xc62c37eb, SUC_lexicon_tags_69 },
    { 0x550c37eb, SUC_lexicon_tags_16 },
    { 0xf14c37ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcc37f1, SUC_lexicon_tags_7 },
    { 0xd84c37f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ec37f5, SUC_lexicon_tags_45 },
    { 0x330c37f5, SUC_lexicon_tags_2 },
    { 0x96ac37f7, SUC_lexicon_tags_32 },
    { 0x174c37f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8cec37fa, SUC_lexicon_tags_32 },
    { 0xa98c37fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedac37fe, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xbacc3801, SUC_lexicon_tags_12 },
    { 0xdaec3802, SUC_lexicon_tags_30 },
    { 0x83ac3801, SUC_lexicon_tags_430 },
    { 0xe8c3801, SUC_lexicon_tags_17 },
    { 0x94c3804, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840c380d, SUC_lexicon_tags_47 },
    { 0x256c380e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626c3813, SUC_lexicon_tags_17 },
    { 0x878c3814, SUC_lexicon_tags_59 },
    { 0x912c3813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758c3819, SUC_lexicon_tags_136 },
    { 0x6a4c381a, SUC_lexicon_tags_29 },
    { 0xeecc381b, SUC_lexicon_tags_25 },
    { 0xc54c381c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x484c381e, SUC_lexicon_tags_6 },
    { 0x67ec381e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c3823, SUC_lexicon_tags_12 },
    { 0x362c3824, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6c3828, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4cc382a, SUC_lexicon_tags_7 },
    { 0x8cec382b, SUC_lexicon_tags_82 },
    { 0x4a0c382c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcc3830, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1bac3832, SUC_lexicon_tags_1 },
    { 0x1b4c3833, SUC_lexicon_tags_17 },
    { 0xbb4c3834, SUC_lexicon_tags_25 },
    { 0x9e8c3835, SUC_lexicon_tags_23 },
    { 0x888c3836, SUC_lexicon_tags_37 },
    { 0x496c3836, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782c383c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa14c383e, SUC_lexicon_tags_77 },
    { 0x3c4c383f, SUC_lexicon_tags_79 },
    { 0x504c3840, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x498c3842, SUC_lexicon_tags_15 },
    { 0x6a2c3842, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ac3849, SUC_lexicon_tags_29 },
    { 0xc00c3849, SUC_lexicon_tags_31 },
    { 0x580c384b, SUC_lexicon_tags_17 },
    { 0xeeec384c, SUC_lexicon_tags_21 },
    { 0x470c384b, SUC_lexicon_tags_37 },
    { 0x9acc384e, SUC_lexicon_tags_65 },
    { 0x9c6c384a, SUC_lexicon_tags_7 },
    { 0x492c384f, SUC_lexicon_tags_27 },
    { 0xc84c384e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x204c3853, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bac385a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeac385f, SUC_lexicon_tags_25 },
    { 0x96cc3860, SUC_lexicon_tags_15 },
    { 0x2d2c3860, SUC_lexicon_tags_23 },
    { 0x7f6c3861, SUC_lexicon_tags_31 },
    { 0x49ac3861, SUC_lexicon_tags_60 },
    { 0x792c3864, SUC_lexicon_tags_6 },
    { 0x20ec3861, SUC_lexicon_tags_12 },
    { 0x2ac3866, SUC_lexicon_tags_46 },
    { 0xc12c3867, SUC_lexicon_tags_4 },
    { 0xcaac3862, SUC_lexicon_tags_37 },
    { 0x592c3869, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6c386b, SUC_lexicon_tags_11 },
    { 0xcc8c386b, SUC_lexicon_tags_37 },
    { 0x66ec386c, SUC_lexicon_tags_45 },
    { 0x752c386d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8d0c3870, SUC_lexicon_tags_63 },
    { 0xbc0c3870, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d8c3873, SUC_lexicon_tags_47 },
    { 0x188c3874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4c387b, SUC_lexicon_tags_11 },
    { 0xa6ac387b, SUC_lexicon_tags_40 },
    { 0xf3ac387b, SUC_lexicon_tags_77 },
    { 0xccc387e, SUC_lexicon_tags_60 },
    { 0x68ac387e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xf44c3881, SUC_lexicon_tags_558 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0c3885, SUC_lexicon_tags_11 },
    { 0xe82c3886, SUC_lexicon_tags_0 },
    { 0x79cc3887, SUC_lexicon_tags_7 },
    { 0xb7ec3888, SUC_lexicon_tags_15 },
    { 0xaeec3889, SUC_lexicon_tags_23 },
    { 0xbecc388a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf2c388c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8c3890, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90c3894, SUC_lexicon_tags_37 },
    { 0xfd2c3895, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x65ec3897, SUC_lexicon_tags_12 },
    { 0xe5ec3898, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x662c389a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4c389d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecc38a1, SUC_lexicon_tags_287 },
    { 0, NULL },
    { 0x42ec38a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ac38a7, SUC_lexicon_tags_126 },
    { 0x9e8c38a8, SUC_lexicon_tags_11 },
    { 0x4f6c38a7, SUC_lexicon_tags_37 },
    { 0xe4ec38a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0c38ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fec38b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508c38ba, SUC_lexicon_tags_27 },
    { 0x904c38bb, SUC_lexicon_tags_83 },
    { 0x95cc38bc, SUC_lexicon_tags_10 },
    { 0x83cc38bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608c38c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x884c38c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6c38c7, SUC_lexicon_tags_66 },
    { 0x594c38c8, SUC_lexicon_tags_63 },
    { 0x9fac38c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ec38cc, SUC_lexicon_tags_7 },
    { 0xfac38cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592c38d1, SUC_lexicon_tags_12 },
    { 0xd42c38d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x678c38d4, SUC_lexicon_tags_32 },
    { 0x250c38d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf6ac38d7, SUC_lexicon_tags_160 },
    { 0x266c38d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93ec38da, SUC_lexicon_tags_46 },
    { 0xc18c38db, SUC_lexicon_tags_15 },
    { 0x5bec38da, SUC_lexicon_tags_41 },
    { 0xab0c38dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x280c38e0, SUC_lexicon_tags_21 },
    { 0x306c38e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48c38e8, SUC_lexicon_tags_46 },
    { 0x600c38e8, SUC_lexicon_tags_25 },
    { 0x676c38ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcc38f1, SUC_lexicon_tags_14 },
    { 0x43cc38f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x600c38f4, SUC_lexicon_tags_8 },
    { 0x92c38f4, SUC_lexicon_tags_37 },
    { 0xb36c38f4, SUC_lexicon_tags_17 },
    { 0xbd4c38f7, SUC_lexicon_tags_10 },
    { 0x514c38f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50ec38fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ac38fd, SUC_lexicon_tags_30 },
    { 0xf5ec38fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2c3906, SUC_lexicon_tags_23 },
    { 0x14ec3906, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cc390b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cc390f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a6c3911, SUC_lexicon_tags_25 },
    { 0xa7cc3912, SUC_lexicon_tags_2 },
    { 0x924c3913, SUC_lexicon_tags_0 },
    { 0xc78c3911, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6c3917, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0c3923, SUC_lexicon_tags_8 },
    { 0x122c3923, SUC_lexicon_tags_17 },
    { 0x2dac3925, SUC_lexicon_tags_25 },
    { 0xc7cc3926, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2c3932, SUC_lexicon_tags_2 },
    { 0xd56c3933, SUC_lexicon_tags_15 },
    { 0x22ac3934, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acc3939, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866c393e, SUC_lexicon_tags_33 },
    { 0x872c393e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950c3949, SUC_lexicon_tags_45 },
    { 0x86cc394a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6c394d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986c3959, SUC_lexicon_tags_43 },
    { 0xaaac3959, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4c395e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcc3962, SUC_lexicon_tags_14 },
    { 0x26cc3963, SUC_lexicon_tags_48 },
    { 0xdecc3963, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4dcc3966, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cc3969, SUC_lexicon_tags_21 },
    { 0xcdec3969, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fec396d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x89cc396f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0c3973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33cc3976, SUC_lexicon_tags_48 },
    { 0xc12c3977, SUC_lexicon_tags_0 },
    { 0xbe8c3978, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x500c397b, SUC_lexicon_tags_15 },
    { 0xe94c397c, SUC_lexicon_tags_1 },
    { 0xd4ec397b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6c3980, SUC_lexicon_tags_17 },
    { 0x9d2c3981, SUC_lexicon_tags_27 },
    { 0x6c4c3981, SUC_lexicon_tags_1646 },
    { 0x40c3983, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cac3985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb66c3987, SUC_lexicon_tags_31 },
    { 0xfdcc3988, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ec398b, SUC_lexicon_tags_7 },
    { 0x8cc398b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cc398e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd48c3990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ac3993, SUC_lexicon_tags_17 },
    { 0xe2ec3994, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958c3998, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc42c399a, SUC_lexicon_tags_15 },
    { 0x9a6c399b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6c399f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec39a2, SUC_lexicon_tags_82 },
    { 0xf4c39a3, SUC_lexicon_tags_46 },
    { 0xc06c39a4, SUC_lexicon_tags_12 },
    { 0x6c0c39a4, SUC_lexicon_tags_6 },
    { 0x918c39a2, SUC_lexicon_tags_30 },
    { 0x822c39a6, SUC_lexicon_tags_49 },
    { 0x66c39a8, SUC_lexicon_tags_23 },
    { 0xf6ac39a8, SUC_lexicon_tags_37 },
    { 0x8b2c39aa, SUC_lexicon_tags_17 },
    { 0x65ec39a9, SUC_lexicon_tags_108 },
    { 0x9c0c39ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22ac39ae, SUC_lexicon_tags_45 },
    { 0x348c39af, SUC_lexicon_tags_83 },
    { 0xc5cc39b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aac39b5, SUC_lexicon_tags_2 },
    { 0xb62c39b6, SUC_lexicon_tags_15 },
    { 0x1cac39b6, SUC_lexicon_tags_25 },
    { 0xe82c39b8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ec39bb, SUC_lexicon_tags_2 },
    { 0x108c39bc, SUC_lexicon_tags_47 },
    { 0xb50c39bd, SUC_lexicon_tags_14 },
    { 0x164c39be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30c39c1, SUC_lexicon_tags_11 },
    { 0x916c39c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ec39c5, SUC_lexicon_tags_23 },
    { 0xb9ec39c6, SUC_lexicon_tags_17 },
    { 0xc2c39c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452c39ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37ec39d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ac39d5, SUC_lexicon_tags_47 },
    { 0xed2c39d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc6c39d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8c39df, SUC_lexicon_tags_17 },
    { 0x614c39df, SUC_lexicon_tags_25 },
    { 0x2cc39e1, SUC_lexicon_tags_25 },
    { 0x346c39e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x416c39e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x70cc39e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cc39eb, SUC_lexicon_tags_27 },
    { 0xccac39ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ac39ef, SUC_lexicon_tags_84 },
    { 0xf7ac39ef, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08c39f5, SUC_lexicon_tags_25 },
    { 0xc9ac39f6, SUC_lexicon_tags_16 },
    { 0xcd6c39f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbccc39f9, SUC_lexicon_tags_57 },
    { 0xf50c39fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb40c39fc, SUC_lexicon_tags_37 },
    { 0xf8ec39fc, SUC_lexicon_tags_25 },
    { 0xc48c39fe, SUC_lexicon_tags_7 },
    { 0x520c39ff, SUC_lexicon_tags_12 },
    { 0x78ec39ff, SUC_lexicon_tags_30 },
    { 0x522c3a00, SUC_lexicon_tags_15 },
    { 0x8dcc3a02, SUC_lexicon_tags_349 },
    { 0xe68c3a03, SUC_lexicon_tags_25 },
    { 0x99ac39fc, SUC_lexicon_tags_61 },
    { 0x74cc3a05, SUC_lexicon_tags_8 },
    { 0xfeec3a06, SUC_lexicon_tags_48 },
    { 0xfaac39fe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x674c3a09, SUC_lexicon_tags_10 },
    { 0x2a8c3a0a, SUC_lexicon_tags_37 },
    { 0x150c3a0a, SUC_lexicon_tags_39 },
    { 0x926c3a0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494c3a12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac3a17, SUC_lexicon_tags_7 },
    { 0xf9cc3a17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc0ac3a1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x650c3a1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x370c3a20, SUC_lexicon_tags_12 },
    { 0x568c3a21, SUC_lexicon_tags_11 },
    { 0x102c3a22, SUC_lexicon_tags_19 },
    { 0x3dcc3a23, SUC_lexicon_tags_65 },
    { 0xc82c3a24, SUC_lexicon_tags_37 },
    { 0x6fec3a23, SUC_lexicon_tags_25 },
    { 0xe7ac3a23, SUC_lexicon_tags_17 },
    { 0x878c3a24, SUC_lexicon_tags_27 },
    { 0xcd4c3a24, SUC_lexicon_tags_19 },
    { 0xdcec3a27, SUC_lexicon_tags_7 },
    { 0x350c3a2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacc3a2e, SUC_lexicon_tags_15 },
    { 0xc1ac3a2f, SUC_lexicon_tags_16 },
    { 0x2ec3a30, SUC_lexicon_tags_55 },
    { 0x982c3a31, SUC_lexicon_tags_31 },
    { 0xf72c3a32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf28c3a34, SUC_lexicon_tags_60 },
    { 0xeb6c3a35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c3a38, SUC_lexicon_tags_1 },
    { 0x384c3a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8c3a3d, SUC_lexicon_tags_17 },
    { 0xd54c3a3e, SUC_lexicon_tags_26 },
    { 0x7cc3a3e, SUC_lexicon_tags_47 },
    { 0x5c0c3a40, SUC_lexicon_tags_17 },
    { 0xe74c3a40, SUC_lexicon_tags_17 },
    { 0x988c3a42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcc3a4b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x69cc3a4d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8c3a53, SUC_lexicon_tags_7 },
    { 0x43ac3a54, SUC_lexicon_tags_19 },
    { 0xbf4c3a55, SUC_lexicon_tags_2 },
    { 0x3bec3a56, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cc3a59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512c3a5d, SUC_lexicon_tags_43 },
    { 0x5dcc3a5e, SUC_lexicon_tags_57 },
    { 0xffec3a5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33cc3a61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x246c3a64, SUC_lexicon_tags_21 },
    { 0xe32c3a65, SUC_lexicon_tags_12 },
    { 0xbfec3a66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f8c3a68, SUC_lexicon_tags_15 },
    { 0x7ccc3a69, SUC_lexicon_tags_45 },
    { 0xf62c3a6a, SUC_lexicon_tags_31 },
    { 0xd5ac3a6a, SUC_lexicon_tags_7 },
    { 0x576c3a6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x750c3a6f, SUC_lexicon_tags_29 },
    { 0xc10c3a70, SUC_lexicon_tags_66 },
    { 0xb08c3a71, SUC_lexicon_tags_37 },
    { 0x6fcc3a72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36c3a79, SUC_lexicon_tags_14 },
    { 0xf74c3a79, SUC_lexicon_tags_77 },
    { 0x356c3a79, SUC_lexicon_tags_19 },
    { 0x6b4c3a7c, SUC_lexicon_tags_17 },
    { 0x380c3a7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40c3a80, SUC_lexicon_tags_7 },
    { 0xd82c3a81, SUC_lexicon_tags_11 },
    { 0x1fec3a81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422c3a86, SUC_lexicon_tags_16 },
    { 0x878c3a86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cc3a8e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366c3a94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec8c3a96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe0cc3a98, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6c3aa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18c3aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cc3aaa, SUC_lexicon_tags_38 },
    { 0x940c3aab, SUC_lexicon_tags_23 },
    { 0xd7cc3aab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cc3aaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784c3ab3, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ac3ab6, SUC_lexicon_tags_10 },
    { 0x4d4c3ab6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ec3abf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdf2c3ac1, SUC_lexicon_tags_25 },
    { 0x838c3ac2, SUC_lexicon_tags_37 },
    { 0x9fac3ac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2ec3ac5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4f4c3ac7, SUC_lexicon_tags_21 },
    { 0x718c3ac8, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86c3ace, SUC_lexicon_tags_2 },
    { 0x44ec3acf, SUC_lexicon_tags_8 },
    { 0xec6c3ace, SUC_lexicon_tags_2 },
    { 0x874c3acf, SUC_lexicon_tags_12 },
    { 0x682c3ad2, SUC_lexicon_tags_0 },
    { 0x214c3ad3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ac3ad6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cc3ad9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cc3ae0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5fac3ae2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x394c3ae5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ec3ae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42c3af5, SUC_lexicon_tags_7 },
    { 0xbdcc3af6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x860c3af8, SUC_lexicon_tags_32 },
    { 0x9e4c3af9, SUC_lexicon_tags_23 },
    { 0x560c3af9, SUC_lexicon_tags_14 },
    { 0x2c6c3afb, SUC_lexicon_tags_60 },
    { 0x22c3afc, SUC_lexicon_tags_26 },
    { 0x8dec3afa, SUC_lexicon_tags_33 },
    { 0xafac3afb, SUC_lexicon_tags_15 },
    { 0x304c3aff, SUC_lexicon_tags_29 },
    { 0x9cc3afb, SUC_lexicon_tags_2 },
    { 0x9fac3aff, SUC_lexicon_tags_6 },
    { 0x926c3af8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ec3b05, SUC_lexicon_tags_65 },
    { 0x416c3b05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ac3b0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b4c3b0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc78c3b0e, SUC_lexicon_tags_11 },
    { 0xee4c3b0f, SUC_lexicon_tags_25 },
    { 0xea6c3b0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8c3b13, SUC_lexicon_tags_25 },
    { 0x134c3b14, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ec3b1a, SUC_lexicon_tags_6 },
    { 0x548c3b1a, SUC_lexicon_tags_1 },
    { 0xdd2c3b1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ac3b22, SUC_lexicon_tags_26 },
    { 0x128c3b23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc5ac3b25, SUC_lexicon_tags_12 },
    { 0x330c3b26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ac3b29, SUC_lexicon_tags_49 },
    { 0x6ecc3b29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0c3b31, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c3b36, SUC_lexicon_tags_1 },
    { 0x858c3b37, SUC_lexicon_tags_1 },
    { 0xb40c3b37, SUC_lexicon_tags_15 },
    { 0x286c3b39, SUC_lexicon_tags_15 },
    { 0x986c3b3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x700c3b3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ac3b40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccc3b47, SUC_lexicon_tags_55 },
    { 0xe9ec3b48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dac3b4b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6c3b4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75ec3b51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a0c3b53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x212c3b56, SUC_lexicon_tags_25 },
    { 0x2bcc3b57, SUC_lexicon_tags_25 },
    { 0x388c3b58, SUC_lexicon_tags_8 },
    { 0x5d2c3b57, SUC_lexicon_tags_48 },
    { 0xe36c3b5a, SUC_lexicon_tags_31 },
    { 0xe3cc3b5b, SUC_lexicon_tags_66 },
    { 0xfcac3b5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cac3b60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8c3b68, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x866c3b6a, SUC_lexicon_tags_11 },
    { 0x600c3b6a, SUC_lexicon_tags_17 },
    { 0xa6c3b6c, SUC_lexicon_tags_47 },
    { 0x75ec3b6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90c3b75, SUC_lexicon_tags_37 },
    { 0x9d8c3b76, SUC_lexicon_tags_25 },
    { 0xf48c3b77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02c3b7b, SUC_lexicon_tags_11 },
    { 0x3cec3b7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e0c3b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ac3b80, SUC_lexicon_tags_10 },
    { 0xbe0c3b80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ac3b86, SUC_lexicon_tags_15 },
    { 0xd8ec3b87, SUC_lexicon_tags_65 },
    { 0x536c3b87, SUC_lexicon_tags_11 },
    { 0x8a0c3b88, SUC_lexicon_tags_15 },
    { 0x5f8c3b86, SUC_lexicon_tags_27 },
    { 0xe48c3b8b, SUC_lexicon_tags_14 },
    { 0x42ac3b8c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ec3b92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ec3b97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5a2c3b99, SUC_lexicon_tags_1 },
    { 0x62c3b9a, SUC_lexicon_tags_63 },
    { 0x6e2c3b9b, SUC_lexicon_tags_7 },
    { 0x328c3b9a, SUC_lexicon_tags_30 },
    { 0xb18c3b99, SUC_lexicon_tags_1 },
    { 0xb60c3b9c, SUC_lexicon_tags_23 },
    { 0x1d2c3b9f, SUC_lexicon_tags_25 },
    { 0x208c3b99, SUC_lexicon_tags_27 },
    { 0x2f4c3b9f, SUC_lexicon_tags_30 },
    { 0x5cec3ba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ecc3ba4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ec3ba9, SUC_lexicon_tags_25 },
    { 0x45cc3baa, SUC_lexicon_tags_4 },
    { 0xee4c3baa, SUC_lexicon_tags_23 },
    { 0x70ec3bac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9ac3bae, SUC_lexicon_tags_181 },
    { 0x1b2c3bae, SUC_lexicon_tags_104 },
    { 0x806c3bb0, SUC_lexicon_tags_9 },
    { 0x85cc3bb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0c3bb6, SUC_lexicon_tags_0 },
    { 0xa84c3bb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0c3bba, SUC_lexicon_tags_25 },
    { 0xadec3bbb, SUC_lexicon_tags_15 },
    { 0x60cc3bbb, SUC_lexicon_tags_12 },
    { 0x6a4c3bbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e8c3bbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356c3bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60ec3bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8c3bc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcec3bcc, SUC_lexicon_tags_33 },
    { 0x17cc3bcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x834c3bd0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc1ac3bd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170c3bd7, SUC_lexicon_tags_14 },
    { 0x496c3bd8, SUC_lexicon_tags_7 },
    { 0xff4c3bd8, SUC_lexicon_tags_1 },
    { 0xff0c3bda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x148c3bdd, SUC_lexicon_tags_1 },
    { 0xce6c3bde, SUC_lexicon_tags_11 },
    { 0x432c3bdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2c3be2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc0c3be4, SUC_lexicon_tags_11 },
    { 0xa52c3be4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x98ec3be7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fcc3be9, SUC_lexicon_tags_12 },
    { 0xd0cc3be9, SUC_lexicon_tags_6 },
    { 0xf58c3be9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ac3bf0, SUC_lexicon_tags_15 },
    { 0x862c3bf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0c3bf4, SUC_lexicon_tags_1 },
    { 0x4cec3bf5, SUC_lexicon_tags_32 },
    { 0xedec3bf6, SUC_lexicon_tags_19 },
    { 0xd06c3bf7, SUC_lexicon_tags_7 },
    { 0xb5ac3bf8, SUC_lexicon_tags_12 },
    { 0x3fcc3bf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8c3bfc, SUC_lexicon_tags_25 },
    { 0xfe0c3bfd, SUC_lexicon_tags_11 },
    { 0x50ac3bfc, SUC_lexicon_tags_47 },
    { 0x934c3bff, SUC_lexicon_tags_25 },
    { 0x802c3bfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ac3c05, SUC_lexicon_tags_15 },
    { 0x6dcc3c06, SUC_lexicon_tags_16 },
    { 0x76ac3c06, SUC_lexicon_tags_201 },
    { 0x28c3c08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x358c3c0a, SUC_lexicon_tags_25 },
    { 0x1bcc3c0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392c3c14, SUC_lexicon_tags_25 },
    { 0x14ac3c14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x424c3c17, SUC_lexicon_tags_0 },
    { 0x4c2c3c18, SUC_lexicon_tags_37 },
    { 0xa08c3c18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6c3c1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f2c3c1e, SUC_lexicon_tags_65 },
    { 0x2bec3c1f, SUC_lexicon_tags_74 },
    { 0xfc8c3c1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ec3c23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5ac3c25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114c3c33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x732c3c36, SUC_lexicon_tags_45 },
    { 0x94cc3c36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30c3c3b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xef6c3c3d, SUC_lexicon_tags_15 },
    { 0x7cac3c3d, SUC_lexicon_tags_23 },
    { 0xb18c3c3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdeac3c41, SUC_lexicon_tags_17 },
    { 0xa66c3c41, SUC_lexicon_tags_29 },
    { 0x3f4c3c42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ec3c46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ac3c4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x548c3c4e, SUC_lexicon_tags_7 },
    { 0x126c3c4f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670c3c54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40ac3c56, SUC_lexicon_tags_17 },
    { 0x832c3c57, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ec3c5a, SUC_lexicon_tags_65 },
    { 0xfcac3c5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83cc3c5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cc3c60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcc3c63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ac3c67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6c3c6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xafac3c72, SUC_lexicon_tags_124 },
    { 0xf96c3c73, SUC_lexicon_tags_127 },
    { 0xfc0c3c74, SUC_lexicon_tags_39 },
    { 0x128c3c73, SUC_lexicon_tags_56 },
    { 0x628c3c76, SUC_lexicon_tags_30 },
    { 0xf14c3c74, SUC_lexicon_tags_118 },
    { 0x754c3c74, SUC_lexicon_tags_35 },
    { 0x60ec3c74, SUC_lexicon_tags_37 },
    { 0x59cc3c7a, SUC_lexicon_tags_29 },
    { 0x686c3c7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ac3c7e, SUC_lexicon_tags_12 },
    { 0x394c3c7e, SUC_lexicon_tags_27 },
    { 0x6acc3c80, SUC_lexicon_tags_43 },
    { 0x4bcc3c80, SUC_lexicon_tags_9 },
    { 0xc4ec3c80, SUC_lexicon_tags_26 },
    { 0xcc4c3c82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ec3c87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60ac3c89, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x782c3c8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c3c8d, SUC_lexicon_tags_25 },
    { 0xac8c3c8e, SUC_lexicon_tags_4 },
    { 0x9e0c3c8e, SUC_lexicon_tags_25 },
    { 0xc02c3c8e, SUC_lexicon_tags_7 },
    { 0x518c3c91, SUC_lexicon_tags_7 },
    { 0x9e8c3c92, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x312c3c94, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e6c3c96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6c3c99, SUC_lexicon_tags_19 },
    { 0xf36c3c9a, SUC_lexicon_tags_83 },
    { 0x750c3c99, SUC_lexicon_tags_17 },
    { 0xfb2c3c9c, SUC_lexicon_tags_171 },
    { 0x658c3c9a, SUC_lexicon_tags_37 },
    { 0x5eac3c9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54c3ca5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ec3ca9, SUC_lexicon_tags_7 },
    { 0x950c3caa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22c3cb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e0c3cb2, SUC_lexicon_tags_17 },
    { 0x2d4c3cb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e2c3cb5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4fac3cb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498c3cbb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40c3cc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54ec3cc2, SUC_lexicon_tags_14 },
    { 0x10ac3cc3, SUC_lexicon_tags_31 },
    { 0xd2ac3cc4, SUC_lexicon_tags_30 },
    { 0x8c6c3cc5, SUC_lexicon_tags_17 },
    { 0x2c6c3cc4, SUC_lexicon_tags_2 },
    { 0xc70c3cc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6c3ccb, SUC_lexicon_tags_11 },
    { 0xd0cc3ccc, SUC_lexicon_tags_37 },
    { 0x4dac3ccb, SUC_lexicon_tags_23 },
    { 0x3f2c3ccc, SUC_lexicon_tags_11 },
    { 0x92c3ccf, SUC_lexicon_tags_37 },
    { 0x56ac3ccd, SUC_lexicon_tags_2 },
    { 0xe4cc3ccf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744c3cdb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x794c3cde, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928c3ce8, SUC_lexicon_tags_19 },
    { 0xa54c3ce9, SUC_lexicon_tags_47 },
    { 0x4fcc3cea, SUC_lexicon_tags_7 },
    { 0xdbcc3cea, SUC_lexicon_tags_45 },
    { 0x9e6c3cea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ec3cf6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd46c3cf8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac3cfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd64c3cfd, SUC_lexicon_tags_25 },
    { 0xa7cc3cfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x648c3d00, SUC_lexicon_tags_37 },
    { 0xd9ac3d01, SUC_lexicon_tags_14 },
    { 0x928c3d01, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dac3d0b, SUC_lexicon_tags_47 },
    { 0xf12c3d0c, SUC_lexicon_tags_11 },
    { 0x8c0c3d0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4c3d11, SUC_lexicon_tags_26 },
    { 0x496c3d12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3d18, SUC_lexicon_tags_0 },
    { 0xb9cc3d19, SUC_lexicon_tags_17 },
    { 0x79ec3d1a, SUC_lexicon_tags_27 },
    { 0x7c6c3d1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x100c3d1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x956c3d20, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1b6c3d22, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ec3d25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ac3d2f, SUC_lexicon_tags_11 },
    { 0x6b0c3d2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc4c3d32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954c3d36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cec3d3a, SUC_lexicon_tags_31 },
    { 0xc62c3d3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8c3d3e, SUC_lexicon_tags_26 },
    { 0xb02c3d3e, SUC_lexicon_tags_15 },
    { 0x230c3d3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf0c3d42, SUC_lexicon_tags_136 },
    { 0x620c3d43, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe86c3d45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacc3d49, SUC_lexicon_tags_31 },
    { 0x7b4c3d49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb92c3d4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16ec3d4e, SUC_lexicon_tags_1 },
    { 0xdb8c3d4e, SUC_lexicon_tags_17 },
    { 0xd94c3d50, SUC_lexicon_tags_79 },
    { 0x82cc3d51, SUC_lexicon_tags_63 },
    { 0x8d6c3d51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfec3d54, SUC_lexicon_tags_6 },
    { 0xe20c3d54, SUC_lexicon_tags_60 },
    { 0x3d8c3d54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eac3d59, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xac3d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa26c3d5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582c3d61, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c3d64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x912c3d67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cc3d71, SUC_lexicon_tags_65 },
    { 0x4e0c3d72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbac3d74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dac3d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ec3d89, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c3d8c, SUC_lexicon_tags_12 },
    { 0xc5cc3d8c, SUC_lexicon_tags_87 },
    { 0x742c3d8e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc3d93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cc3d9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cc3d9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66c3da3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9ac3da5, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeec3da8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ec3dab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc36c3dad, SUC_lexicon_tags_30 },
    { 0x792c3dad, SUC_lexicon_tags_33 },
    { 0xf60c3dad, SUC_lexicon_tags_0 },
    { 0xadec3db0, SUC_lexicon_tags_8 },
    { 0x420c3db1, SUC_lexicon_tags_49 },
    { 0xdd4c3daf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x182c3db5, SUC_lexicon_tags_7 },
    { 0xb62c3db6, SUC_lexicon_tags_33 },
    { 0x788c3db7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cc3dbc, SUC_lexicon_tags_109 },
    { 0xabcc3dbc, SUC_lexicon_tags_15 },
    { 0xa70c3dbe, SUC_lexicon_tags_26 },
    { 0xb9ec3dbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x184c3dc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x986c3dc5, SUC_lexicon_tags_5 },
    { 0xa7ec3dc6, SUC_lexicon_tags_25 },
    { 0x60ec3dc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456c3dcb, SUC_lexicon_tags_60 },
    { 0x944c3dcc, SUC_lexicon_tags_154 },
    { 0x92cc3dcc, SUC_lexicon_tags_27 },
    { 0x940c3dce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81cc3dd0, SUC_lexicon_tags_7 },
    { 0xdc2c3dd0, SUC_lexicon_tags_16 },
    { 0x3cec3dd0, SUC_lexicon_tags_32 },
    { 0x43ec3dd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64cc3dd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x960c3dd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68c3dda, SUC_lexicon_tags_2 },
    { 0x51ec3ddb, SUC_lexicon_tags_17 },
    { 0xd2ac3ddc, SUC_lexicon_tags_23 },
    { 0x22c3ddc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d4c3ddf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c8c3de1, SUC_lexicon_tags_17 },
    { 0xf1cc3de2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1aac3de4, SUC_lexicon_tags_0 },
    { 0xa34c3de4, SUC_lexicon_tags_27 },
    { 0x4fec3de4, SUC_lexicon_tags_2 },
    { 0x466c3de7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc5cc3de9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cc3dec, SUC_lexicon_tags_30 },
    { 0x10c3ded, SUC_lexicon_tags_2 },
    { 0x6a4c3dec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0c3df0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc24c3df2, SUC_lexicon_tags_26 },
    { 0xc6cc3df2, SUC_lexicon_tags_27 },
    { 0x5e0c3df2, SUC_lexicon_tags_14 },
    { 0x6cac3df4, SUC_lexicon_tags_47 },
    { 0x1e4c3df6, SUC_lexicon_tags_47 },
    { 0x308c3df7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cc3dfb, SUC_lexicon_tags_46 },
    { 0x874c3dfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8c3e02, SUC_lexicon_tags_111 },
    { 0xb1ec3e02, SUC_lexicon_tags_47 },
    { 0xe2c3e04, SUC_lexicon_tags_1 },
    { 0x59ec3e05, SUC_lexicon_tags_262 },
    { 0x87ac3e04, SUC_lexicon_tags_56 },
    { 0x7eec3e06, SUC_lexicon_tags_12 },
    { 0xf3ac3e06, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x83cc3e0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32c3e0d, SUC_lexicon_tags_7 },
    { 0x676c3e0e, SUC_lexicon_tags_65 },
    { 0x162c3e0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10c3e15, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4bec3e17, SUC_lexicon_tags_1 },
    { 0x2f4c3e18, SUC_lexicon_tags_8 },
    { 0x520c3e18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabac3e1e, SUC_lexicon_tags_23 },
    { 0x4dec3e1f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304c3e23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x44cc3e25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78c3e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806c3e2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21cc3e30, SUC_lexicon_tags_1 },
    { 0xbd6c3e31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dec3e37, SUC_lexicon_tags_11 },
    { 0x472c3e38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cc3e3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ec3e40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86c3e45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02c3e54, SUC_lexicon_tags_15 },
    { 0xc40c3e55, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362c3e59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6c3e61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x390c3e64, SUC_lexicon_tags_55 },
    { 0x1dac3e65, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8c3e68, SUC_lexicon_tags_27 },
    { 0x332c3e69, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6c3e70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0c3e75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac3e78, SUC_lexicon_tags_38 },
    { 0xee4c3e78, SUC_lexicon_tags_7 },
    { 0x530c3e7a, SUC_lexicon_tags_17 },
    { 0x254c3e7a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xe7ac3e7d, SUC_lexicon_tags_30 },
    { 0x516c3e7e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xae6c3e80, SUC_lexicon_tags_9 },
    { 0x5c6c3e80, SUC_lexicon_tags_23 },
    { 0x472c3e82, SUC_lexicon_tags_16 },
    { 0xf24c3e83, SUC_lexicon_tags_23 },
    { 0xa50c3e80, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa6ec3e86, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640c3e8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2c3e8f, SUC_lexicon_tags_6 },
    { 0xbd4c3e8f, SUC_lexicon_tags_12 },
    { 0xaacc3e90, SUC_lexicon_tags_26 },
    { 0xd10c3e92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb22c3e94, SUC_lexicon_tags_17 },
    { 0xe66c3e94, SUC_lexicon_tags_32 },
    { 0xda8c3e96, SUC_lexicon_tags_19 },
    { 0x2d2c3e97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94c3e9b, SUC_lexicon_tags_12 },
    { 0x3e8c3e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x292c3e9e, SUC_lexicon_tags_38 },
    { 0x524c3e9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b6c3ea1, SUC_lexicon_tags_68 },
    { 0x6c4c3ea2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ac3ea5, SUC_lexicon_tags_14 },
    { 0x89cc3ea5, SUC_lexicon_tags_17 },
    { 0xb28c3ea5, SUC_lexicon_tags_2 },
    { 0xba6c3ea7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa24c3eaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ac3eb1, SUC_lexicon_tags_7 },
    { 0x994c3eb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x872c3eb4, SUC_lexicon_tags_29 },
    { 0x6f4c3eb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccc3eb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf5cc3eba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3cc3ebc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ac3ebf, SUC_lexicon_tags_4 },
    { 0xee8c3ec0, SUC_lexicon_tags_30 },
    { 0x694c3ec1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xadec3ec4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x910c3ec6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1e2c3ec8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x128c3ecb, SUC_lexicon_tags_17 },
    { 0xbfac3ecc, SUC_lexicon_tags_4 },
    { 0x86c3ecd, SUC_lexicon_tags_65 },
    { 0x7e6c3ecb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4c3ed3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2c3ed7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c3edb, SUC_lexicon_tags_33 },
    { 0x87ac3edb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880c3ee6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x666c3ee8, SUC_lexicon_tags_14 },
    { 0x602c3ee9, SUC_lexicon_tags_37 },
    { 0x7f0c3ee9, SUC_lexicon_tags_30 },
    { 0xe6ec3eeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x574c3eee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58c3ef1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9e0c3ef3, SUC_lexicon_tags_12 },
    { 0xd0c3ef4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7dec3ef6, SUC_lexicon_tags_37 },
    { 0xedac3ef7, SUC_lexicon_tags_1 },
    { 0xd1ec3ef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc2c3efa, SUC_lexicon_tags_25 },
    { 0xa8ec3efb, SUC_lexicon_tags_9 },
    { 0x392c3efb, SUC_lexicon_tags_15 },
    { 0x2f2c3efd, SUC_lexicon_tags_11 },
    { 0x6b0c3efe, SUC_lexicon_tags_47 },
    { 0x3a0c3eff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c3f06, SUC_lexicon_tags_32 },
    { 0xa52c3f07, SUC_lexicon_tags_38 },
    { 0x7eec3f07, SUC_lexicon_tags_2 },
    { 0x11ac3f08, SUC_lexicon_tags_64 },
    { 0x32ec3f09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebac3f0d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6c3f11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ac3f17, SUC_lexicon_tags_23 },
    { 0xd6ec3f18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ec3f1c, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0c3f25, SUC_lexicon_tags_63 },
    { 0xe3cc3f26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cc3f2a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x556c3f2d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec3f30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecc3f34, SUC_lexicon_tags_15 },
    { 0x6ac3f34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37ac3f37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124c3f3b, SUC_lexicon_tags_84 },
    { 0xc64c3f3c, SUC_lexicon_tags_25 },
    { 0xac2c3f3d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2c3f44, SUC_lexicon_tags_61 },
    { 0x7d6c3f45, SUC_lexicon_tags_27 },
    { 0x402c3f46, SUC_lexicon_tags_14 },
    { 0x7dac3f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cc3f4b, SUC_lexicon_tags_15 },
    { 0x3e8c3f4c, SUC_lexicon_tags_7 },
    { 0xe2cc3f4b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58c3f56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854c3f5b, SUC_lexicon_tags_25 },
    { 0xec4c3f5b, SUC_lexicon_tags_14 },
    { 0x808c3f5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x802c3f5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ac3f62, SUC_lexicon_tags_12 },
    { 0xc34c3f63, SUC_lexicon_tags_26 },
    { 0x48ac3f63, SUC_lexicon_tags_43 },
    { 0xc76c3f63, SUC_lexicon_tags_11 },
    { 0x65cc3f66, SUC_lexicon_tags_15 },
    { 0x734c3f67, SUC_lexicon_tags_11 },
    { 0xa2cc3f68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aec3f6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2cc3f6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f8c3f6f, SUC_lexicon_tags_10 },
    { 0x67cc3f70, SUC_lexicon_tags_2 },
    { 0xd1ec3f71, SUC_lexicon_tags_60 },
    { 0x65ac3f70, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x80c3f74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cc3f79, SUC_lexicon_tags_1 },
    { 0x32ac3f79, SUC_lexicon_tags_37 },
    { 0x3bec3f7b, SUC_lexicon_tags_14 },
    { 0xdfcc3f7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ac3f82, SUC_lexicon_tags_37 },
    { 0x53ec3f83, SUC_lexicon_tags_14 },
    { 0x98ec3f84, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4ecc3f86, SUC_lexicon_tags_11 },
    { 0x2b4c3f87, SUC_lexicon_tags_37 },
    { 0x610c3f87, SUC_lexicon_tags_45 },
    { 0xdf4c3f88, SUC_lexicon_tags_15 },
    { 0xf7cc3f8a, SUC_lexicon_tags_15 },
    { 0x250c3f8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2a4c3f8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0c3f90, SUC_lexicon_tags_43 },
    { 0xc8c3f91, SUC_lexicon_tags_151 },
    { 0x742c3f91, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3bcc3f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ac3f9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac3fa1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe68c3fa3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40c3fa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2c3fab, SUC_lexicon_tags_12 },
    { 0x7f8c3fac, SUC_lexicon_tags_44 },
    { 0xa6c3fab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccc3faf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9dec3fb1, SUC_lexicon_tags_11 },
    { 0x530c3fb2, SUC_lexicon_tags_13 },
    { 0x482c3fb3, SUC_lexicon_tags_27 },
    { 0x188c3fb2, SUC_lexicon_tags_46 },
    { 0xd0ec3fb5, SUC_lexicon_tags_45 },
    { 0x9c4c3fb6, SUC_lexicon_tags_33 },
    { 0x5c2c3fb3, SUC_lexicon_tags_37 },
    { 0x6ecc3fb8, SUC_lexicon_tags_15 },
    { 0x2dac3fb9, SUC_lexicon_tags_16 },
    { 0xa48c3fb9, SUC_lexicon_tags_26 },
    { 0xc00c3fba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4c3fbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x606c3fc1, SUC_lexicon_tags_23 },
    { 0xa46c3fc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646c3fcd, SUC_lexicon_tags_45 },
    { 0xd96c3fce, SUC_lexicon_tags_42 },
    { 0xc1cc3fcf, SUC_lexicon_tags_17 },
    { 0x1ec3fd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cc3fd3, SUC_lexicon_tags_60 },
    { 0x4b4c3fd4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc2ac3fd6, SUC_lexicon_tags_7 },
    { 0x4f8c3fd6, SUC_lexicon_tags_54 },
    { 0xea2c3fd8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1aac3fda, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cac3fe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300c3feb, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x958c3fed, SUC_lexicon_tags_10 },
    { 0x37cc3fee, SUC_lexicon_tags_11 },
    { 0x748c3fed, SUC_lexicon_tags_25 },
    { 0x8ec3ff0, SUC_lexicon_tags_17 },
    { 0x3dac3ff0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x606c3ff4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4cc3ff6, SUC_lexicon_tags_30 },
    { 0x4aac3ff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x832c3ffa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0c4001, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ac4004, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3dec4006, SUC_lexicon_tags_15 },
    { 0xd96c4007, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ac400a, SUC_lexicon_tags_38 },
    { 0x48ec400a, SUC_lexicon_tags_16 },
    { 0xeac400b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702c4010, SUC_lexicon_tags_37 },
    { 0x12ec4011, SUC_lexicon_tags_46 },
    { 0xdd2c4011, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecac4018, SUC_lexicon_tags_30 },
    { 0x136c4019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe70c401b, SUC_lexicon_tags_0 },
    { 0xc08c401b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6c401f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0c4022, SUC_lexicon_tags_15 },
    { 0xdf8c4022, SUC_lexicon_tags_25 },
    { 0xc90c4023, SUC_lexicon_tags_49 },
    { 0xce2c4025, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4f6c4027, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x28cc4029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x266c402b, SUC_lexicon_tags_14 },
    { 0x97ec402c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38c402f, SUC_lexicon_tags_1 },
    { 0xf06c402f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd28c4032, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2c4035, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ec4038, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770c403d, SUC_lexicon_tags_46 },
    { 0x482c403e, SUC_lexicon_tags_1 },
    { 0x432c403f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4c4042, SUC_lexicon_tags_14 },
    { 0x39ac4042, SUC_lexicon_tags_31 },
    { 0x688c4044, SUC_lexicon_tags_46 },
    { 0xffac4044, SUC_lexicon_tags_45 },
    { 0x4e0c4045, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcc404a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acc404e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8c4057, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cc405d, SUC_lexicon_tags_17 },
    { 0x978c405e, SUC_lexicon_tags_1 },
    { 0x9fcc405d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8c4066, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9ec4068, SUC_lexicon_tags_11 },
    { 0x822c4068, SUC_lexicon_tags_47 },
    { 0x32c406a, SUC_lexicon_tags_43 },
    { 0xe38c406a, SUC_lexicon_tags_17 },
    { 0xa6c406a, SUC_lexicon_tags_6 },
    { 0x7bcc4069, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412c4074, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654c4079, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x930c407b, SUC_lexicon_tags_11 },
    { 0x75ec407c, SUC_lexicon_tags_0 },
    { 0x612c407c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24c4082, SUC_lexicon_tags_2 },
    { 0x8fec4083, SUC_lexicon_tags_1531 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ac4087, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10c408e, SUC_lexicon_tags_27 },
    { 0xe6cc408f, SUC_lexicon_tags_24 },
    { 0x960c408f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcc4095, SUC_lexicon_tags_37 },
    { 0xddac4096, SUC_lexicon_tags_11 },
    { 0x9b4c4096, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x936c4099, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ac40a0, SUC_lexicon_tags_39 },
    { 0x120c40a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6c40a4, SUC_lexicon_tags_9 },
    { 0x98ec40a5, SUC_lexicon_tags_37 },
    { 0x958c40a6, SUC_lexicon_tags_17 },
    { 0x92cc40a7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36c40ac, SUC_lexicon_tags_10 },
    { 0x5b2c40ad, SUC_lexicon_tags_8 },
    { 0xe34c40ad, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c40b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ac40b6, SUC_lexicon_tags_37 },
    { 0x6a0c40b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178c40be, SUC_lexicon_tags_33 },
    { 0x8a2c40be, SUC_lexicon_tags_1 },
    { 0x43cc40bf, SUC_lexicon_tags_7 },
    { 0x61ec40c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600c40c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x152c40ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba4c40cc, SUC_lexicon_tags_11 },
    { 0x40ac40cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30c40d8, SUC_lexicon_tags_47 },
    { 0xb84c40d9, SUC_lexicon_tags_60 },
    { 0xafcc40d9, SUC_lexicon_tags_12 },
    { 0xa64c40da, SUC_lexicon_tags_82 },
    { 0x8fcc40dc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x9b8c40de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28cc40e0, SUC_lexicon_tags_10 },
    { 0x284c40e1, SUC_lexicon_tags_2 },
    { 0xd26c40e0, SUC_lexicon_tags_1 },
    { 0x9b4c40e3, SUC_lexicon_tags_65 },
    { 0xd7ac40e4, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28c40e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x184c40e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eac40f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50c40f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcc40fe, SUC_lexicon_tags_38 },
    { 0xefac40fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb92c4101, SUC_lexicon_tags_37 },
    { 0xd4cc4102, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ec4105, SUC_lexicon_tags_39 },
    { 0x25ac4105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6c410a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x524c410c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x792c410f, SUC_lexicon_tags_6 },
    { 0xb3ec4110, SUC_lexicon_tags_11 },
    { 0x3c2c4111, SUC_lexicon_tags_15 },
    { 0x26ec4110, SUC_lexicon_tags_11 },
    { 0x712c4113, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa7ec4115, SUC_lexicon_tags_45 },
    { 0x6a6c4116, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8c411c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcc4120, SUC_lexicon_tags_3 },
    { 0xf88c4120, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcc4124, SUC_lexicon_tags_12 },
    { 0xf2c4125, SUC_lexicon_tags_0 },
    { 0x8dcc4126, SUC_lexicon_tags_25 },
    { 0xb3ac4125, SUC_lexicon_tags_17 },
    { 0xaeac4126, SUC_lexicon_tags_38 },
    { 0xd9ac4129, SUC_lexicon_tags_15 },
    { 0xa76c4126, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x514c412c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefec4133, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74c4138, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26cc413b, SUC_lexicon_tags_7 },
    { 0x7f0c413c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ac4141, SUC_lexicon_tags_6 },
    { 0x76cc4141, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x878c4145, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaac414a, SUC_lexicon_tags_28 },
    { 0x6e2c414b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6eec414d, SUC_lexicon_tags_11 },
    { 0x2fac414e, SUC_lexicon_tags_14 },
    { 0xaaac414f, SUC_lexicon_tags_15 },
    { 0xb38c414f, SUC_lexicon_tags_37 },
    { 0x55ec4151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750c4155, SUC_lexicon_tags_74 },
    { 0x8e6c4156, SUC_lexicon_tags_12 },
    { 0x354c4156, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cc415a, SUC_lexicon_tags_63 },
    { 0xca6c415b, SUC_lexicon_tags_48 },
    { 0x8c0c415b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ec4162, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cc4165, SUC_lexicon_tags_7 },
    { 0x4e0c4166, SUC_lexicon_tags_63 },
    { 0x954c4167, SUC_lexicon_tags_63 },
    { 0x9f2c4168, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97ec416a, SUC_lexicon_tags_25 },
    { 0xbe8c416a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04c416e, SUC_lexicon_tags_14 },
    { 0x548c416e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ac4174, SUC_lexicon_tags_2 },
    { 0x324c4175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6c417a, SUC_lexicon_tags_12 },
    { 0x6cac417b, SUC_lexicon_tags_26 },
    { 0x13ac417b, SUC_lexicon_tags_19 },
    { 0x6fac417a, SUC_lexicon_tags_14 },
    { 0x27ec417e, SUC_lexicon_tags_1 },
    { 0x354c417a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6c4183, SUC_lexicon_tags_11 },
    { 0xacc4184, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ec4188, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ec418d, SUC_lexicon_tags_25 },
    { 0x2b2c418d, SUC_lexicon_tags_63 },
    { 0xb12c418f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332c4193, SUC_lexicon_tags_11 },
    { 0x1a8c4194, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00c4197, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43ac4199, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4c419b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x82c419d, SUC_lexicon_tags_14 },
    { 0x8aec419d, SUC_lexicon_tags_9 },
    { 0xfbcc419d, SUC_lexicon_tags_29 },
    { 0x484c419f, SUC_lexicon_tags_49 },
    { 0x198c41a1, SUC_lexicon_tags_4 },
    { 0xc5cc419d, SUC_lexicon_tags_47 },
    { 0xd6cc41a0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64c41a7, SUC_lexicon_tags_17 },
    { 0x39ac41a8, SUC_lexicon_tags_12 },
    { 0x912c41a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31ec41ab, SUC_lexicon_tags_37 },
    { 0x9ccc41ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4c41b0, SUC_lexicon_tags_757 },
    { 0x980c41b1, SUC_lexicon_tags_17 },
    { 0x87ac41b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfac41b6, SUC_lexicon_tags_1 },
    { 0x292c41b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338c41bb, SUC_lexicon_tags_17 },
    { 0x8c0c41bb, SUC_lexicon_tags_31 },
    { 0xd0ac41bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8c41c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ac41c8, SUC_lexicon_tags_11 },
    { 0xf1ec41c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2a2c41cb, SUC_lexicon_tags_47 },
    { 0x592c41cc, SUC_lexicon_tags_254 },
    { 0xdc6c41cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87cc41cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2c41d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448c41d7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aec41dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272c41e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2c41e5, SUC_lexicon_tags_11 },
    { 0x8a6c41e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd58c41e8, SUC_lexicon_tags_35 },
    { 0x9d4c41e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x404c41eb, SUC_lexicon_tags_68 },
    { 0x646c41ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992c41f0, SUC_lexicon_tags_27 },
    { 0x58ac41f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ec41f5, SUC_lexicon_tags_77 },
    { 0xd6ec41f5, SUC_lexicon_tags_6 },
    { 0x6ecc41f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8c41fb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc1ac41fd, SUC_lexicon_tags_306 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94c4203, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420c4208, SUC_lexicon_tags_66 },
    { 0xdeac4208, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xedec420c, SUC_lexicon_tags_38 },
    { 0xcccc420d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2c4211, SUC_lexicon_tags_19 },
    { 0x17ac4212, SUC_lexicon_tags_62 },
    { 0x890c4213, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78c421a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2c421d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23cc421f, SUC_lexicon_tags_25 },
    { 0x1eac4220, SUC_lexicon_tags_47 },
    { 0x51ec4221, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3ac4223, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d6c4225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ec4227, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x89cc4229, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4c422d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x328c4230, SUC_lexicon_tags_48 },
    { 0xdb2c4231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cc4235, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x348c4238, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ec423b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf14c423d, SUC_lexicon_tags_33 },
    { 0xff4c423e, SUC_lexicon_tags_43 },
    { 0x868c423f, SUC_lexicon_tags_17 },
    { 0x956c423f, SUC_lexicon_tags_12 },
    { 0x382c423e, SUC_lexicon_tags_30 },
    { 0x28c423e, SUC_lexicon_tags_62 },
    { 0xad8c4243, SUC_lexicon_tags_37 },
    { 0x602c4240, SUC_lexicon_tags_15 },
    { 0xa8ec4245, SUC_lexicon_tags_37 },
    { 0x564c4241, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248c424b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436c4252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dec4256, SUC_lexicon_tags_21 },
    { 0x904c4257, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x39cc4259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecc4261, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa64c4263, SUC_lexicon_tags_60 },
    { 0x84cc4264, SUC_lexicon_tags_15 },
    { 0x5d0c4265, SUC_lexicon_tags_11 },
    { 0xf2c4266, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3d8c4268, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ac426d, SUC_lexicon_tags_25 },
    { 0x24ec426e, SUC_lexicon_tags_25 },
    { 0x436c426d, SUC_lexicon_tags_12 },
    { 0xa56c4270, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170c4274, SUC_lexicon_tags_114 },
    { 0x63ac4275, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66c4277, SUC_lexicon_tags_43 },
    { 0x794c4277, SUC_lexicon_tags_23 },
    { 0xdecc4277, SUC_lexicon_tags_27 },
    { 0xbfec4278, SUC_lexicon_tags_17 },
    { 0xe4cc427a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacc4282, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cec4285, SUC_lexicon_tags_38 },
    { 0x358c4285, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52ac4288, SUC_lexicon_tags_16 },
    { 0x3c4c4289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcc428c, SUC_lexicon_tags_765 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ac4291, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf8ec4293, SUC_lexicon_tags_12 },
    { 0xcf0c4294, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d6c4296, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c429b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x404c429e, SUC_lexicon_tags_9 },
    { 0x3a2c429f, SUC_lexicon_tags_27 },
    { 0x360c429e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c42a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acc42a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30c42a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8c42ab, SUC_lexicon_tags_16 },
    { 0x510c42ac, SUC_lexicon_tags_7 },
    { 0x1ecc42ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754c42b2, SUC_lexicon_tags_31 },
    { 0xc2c42b2, SUC_lexicon_tags_17 },
    { 0xa54c42b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43ec42b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4c42bb, SUC_lexicon_tags_45 },
    { 0x5dec42bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x868c42be, SUC_lexicon_tags_274 },
    { 0x94c42be, SUC_lexicon_tags_108 },
    { 0x94ac42c0, SUC_lexicon_tags_17 },
    { 0xdccc42bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6cc42c3, SUC_lexicon_tags_26 },
    { 0x674c42c4, SUC_lexicon_tags_37 },
    { 0xca0c42c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ec42c9, SUC_lexicon_tags_25 },
    { 0x238c42ca, SUC_lexicon_tags_1 },
    { 0x7bec42ca, SUC_lexicon_tags_11 },
    { 0x948c42cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe86c42ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x238c42d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa54c42d3, SUC_lexicon_tags_25 },
    { 0xe68c42d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cc42d7, SUC_lexicon_tags_31 },
    { 0x53cc42d8, SUC_lexicon_tags_25 },
    { 0x232c42d9, SUC_lexicon_tags_15 },
    { 0xc3ec42da, SUC_lexicon_tags_82 },
    { 0x510c42d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ec42de, SUC_lexicon_tags_1 },
    { 0x808c42de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x78ac42e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x606c42e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c42e8, SUC_lexicon_tags_15 },
    { 0xb9ac42e9, SUC_lexicon_tags_14 },
    { 0x81ac42ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6c42ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64c42f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x96ec42f3, SUC_lexicon_tags_23 },
    { 0xd5ec42f3, SUC_lexicon_tags_10 },
    { 0x290c42f5, SUC_lexicon_tags_17 },
    { 0x3a6c42f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bac42f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfac42fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2ec42fe, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1e6c4300, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4305, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17ac4307, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408c4314, SUC_lexicon_tags_37 },
    { 0x968c4314, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cc4318, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4c431d, SUC_lexicon_tags_15 },
    { 0xee0c431e, SUC_lexicon_tags_12 },
    { 0x2cec431e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366c4325, SUC_lexicon_tags_19 },
    { 0xf16c4326, SUC_lexicon_tags_1 },
    { 0xafec4327, SUC_lexicon_tags_37 },
    { 0xbaac4328, SUC_lexicon_tags_68 },
    { 0xef2c4329, SUC_lexicon_tags_2 },
    { 0xecac432a, SUC_lexicon_tags_15 },
    { 0xa70c4326, SUC_lexicon_tags_31 },
    { 0xef8c432c, SUC_lexicon_tags_6 },
    { 0xc72c432b, SUC_lexicon_tags_2 },
    { 0x348c4327, SUC_lexicon_tags_39 },
    { 0x72c432d, SUC_lexicon_tags_235 },
    { 0x43ac432f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96c4335, SUC_lexicon_tags_33 },
    { 0x4f4c4336, SUC_lexicon_tags_45 },
    { 0xd7cc4337, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0c433f, SUC_lexicon_tags_60 },
    { 0xd26c433f, SUC_lexicon_tags_7 },
    { 0xfacc4341, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340c4345, SUC_lexicon_tags_37 },
    { 0xeac4345, SUC_lexicon_tags_33 },
    { 0xa70c4347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0c4352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dcc4354, SUC_lexicon_tags_34 },
    { 0x90ac4354, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98c4358, SUC_lexicon_tags_33 },
    { 0x2bac4359, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656c435d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c4361, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4c4367, SUC_lexicon_tags_1 },
    { 0xaecc4367, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ec436b, SUC_lexicon_tags_12 },
    { 0xfaec436b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe94c436e, SUC_lexicon_tags_11 },
    { 0xe1ac436f, SUC_lexicon_tags_1 },
    { 0x1d8c436f, SUC_lexicon_tags_11 },
    { 0xb42c4371, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x112c4374, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcd8c4376, SUC_lexicon_tags_17 },
    { 0x7dac4376, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5dec4379, SUC_lexicon_tags_57 },
    { 0xb42c437a, SUC_lexicon_tags_46 },
    { 0xbdac437b, SUC_lexicon_tags_14 },
    { 0xa04c4379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x144c437e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x108c4380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd0c4382, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cc4385, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x87cc4388, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfac438b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x682c438d, SUC_lexicon_tags_9 },
    { 0x586c438e, SUC_lexicon_tags_82 },
    { 0x1f8c438f, SUC_lexicon_tags_27 },
    { 0x7dcc4390, SUC_lexicon_tags_37 },
    { 0xdb8c438e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5ec4393, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844c4399, SUC_lexicon_tags_7 },
    { 0xe26c439a, SUC_lexicon_tags_2 },
    { 0x816c4399, SUC_lexicon_tags_23 },
    { 0x248c439a, SUC_lexicon_tags_25 },
    { 0xa48c439a, SUC_lexicon_tags_2 },
    { 0x932c439d, SUC_lexicon_tags_100 },
    { 0x220c439f, SUC_lexicon_tags_12 },
    { 0x36c4399, SUC_lexicon_tags_112 },
    { 0xfb8c4399, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2c43a5, SUC_lexicon_tags_37 },
    { 0x9d2c43a5, SUC_lexicon_tags_7 },
    { 0x842c43a7, SUC_lexicon_tags_37 },
    { 0x758c43a8, SUC_lexicon_tags_2 },
    { 0xfcac43a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90c43ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x170c43ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74c43b1, SUC_lexicon_tags_38 },
    { 0x2a0c43b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x678c43b5, SUC_lexicon_tags_10 },
    { 0x31ac43b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c43b9, SUC_lexicon_tags_11 },
    { 0xf60c43ba, SUC_lexicon_tags_2 },
    { 0xc5cc43bb, SUC_lexicon_tags_25 },
    { 0x1e2c43bc, SUC_lexicon_tags_27 },
    { 0x8a6c43ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac43c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ac43c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ec43c6, SUC_lexicon_tags_47 },
    { 0x918c43c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06c43d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x134c43d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x486c43d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98ec43db, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ac43e8, SUC_lexicon_tags_0 },
    { 0xb6ec43e9, SUC_lexicon_tags_12 },
    { 0x1c4c43ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc52c43ec, SUC_lexicon_tags_27 },
    { 0x62cc43ed, SUC_lexicon_tags_25 },
    { 0x7eec43ee, SUC_lexicon_tags_27 },
    { 0xde0c43ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490c43f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x70ac43f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x374c43f9, SUC_lexicon_tags_37 },
    { 0xcaec43fa, SUC_lexicon_tags_84 },
    { 0xfaac43f9, SUC_lexicon_tags_2 },
    { 0x9acc43fb, SUC_lexicon_tags_60 },
    { 0xc76c43fc, SUC_lexicon_tags_7 },
    { 0xcc43fe, SUC_lexicon_tags_49 },
    { 0xdc8c43fe, SUC_lexicon_tags_656 },
    { 0xb2ec43fb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8c4403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x776c4405, SUC_lexicon_tags_6 },
    { 0xa74c4405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ec440e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x840c4411, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x392c4413, SUC_lexicon_tags_437 },
    { 0xf14c4413, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x79ac4416, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10c441a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ec441d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30c4420, SUC_lexicon_tags_30 },
    { 0xcf6c4421, SUC_lexicon_tags_29 },
    { 0xcb0c4422, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72c4426, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xd22c4428, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc9cc442a, SUC_lexicon_tags_28 },
    { 0x628c442b, SUC_lexicon_tags_15 },
    { 0xe78c442c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x458c442e, SUC_lexicon_tags_49 },
    { 0xb22c442f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c4431, SUC_lexicon_tags_16 },
    { 0xdeec4431, SUC_lexicon_tags_29 },
    { 0x81ec4433, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6c4436, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xccac4438, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x54c443a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214c4440, SUC_lexicon_tags_26 },
    { 0xfc6c4441, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ac4444, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x268c4447, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c444c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x662c444e, SUC_lexicon_tags_15 },
    { 0x1fec444f, SUC_lexicon_tags_2 },
    { 0xba6c4450, SUC_lexicon_tags_28 },
    { 0xeecc4451, SUC_lexicon_tags_87 },
    { 0x29ac4451, SUC_lexicon_tags_2 },
    { 0x5fec4453, SUC_lexicon_tags_0 },
    { 0x866c4452, SUC_lexicon_tags_47 },
    { 0x5d8c4455, SUC_lexicon_tags_17 },
    { 0xec4456, SUC_lexicon_tags_25 },
    { 0xa3ec4451, SUC_lexicon_tags_1 },
    { 0x3e8c444e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffac445b, SUC_lexicon_tags_17 },
    { 0x4b8c445b, SUC_lexicon_tags_38 },
    { 0xd0ec445c, SUC_lexicon_tags_63 },
    { 0x2bac445e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xad6c4460, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ac4463, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5acc4466, SUC_lexicon_tags_2 },
    { 0x6e4c4466, SUC_lexicon_tags_1 },
    { 0x7d8c4466, SUC_lexicon_tags_45 },
    { 0x6f4c4469, SUC_lexicon_tags_34 },
    { 0x6aec446a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78c446e, SUC_lexicon_tags_13 },
    { 0x294c446f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76c4472, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x496c4475, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320c4479, SUC_lexicon_tags_15 },
    { 0xe64c4479, SUC_lexicon_tags_16 },
    { 0xbb2c447a, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0x536c447e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968c4483, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2c448a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfcc448c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0c4491, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfac4493, SUC_lexicon_tags_15 },
    { 0x73ac4494, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28c4496, SUC_lexicon_tags_28 },
    { 0x842c4497, SUC_lexicon_tags_65 },
    { 0x32ec4498, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66c449b, SUC_lexicon_tags_1 },
    { 0x64c449b, SUC_lexicon_tags_43 },
    { 0xc38c449d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ac449f, SUC_lexicon_tags_11 },
    { 0x298c44a0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xadec44a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecc44a6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc44a9, SUC_lexicon_tags_15 },
    { 0xeec44aa, SUC_lexicon_tags_17 },
    { 0xbe8c44aa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x182c44ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202c44b3, SUC_lexicon_tags_42 },
    { 0xb80c44b4, SUC_lexicon_tags_26 },
    { 0xa3cc44b5, SUC_lexicon_tags_1 },
    { 0xb0ac44b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6c44b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ec44bf, SUC_lexicon_tags_2 },
    { 0xcfcc44bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2c44c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90c44ca, SUC_lexicon_tags_8 },
    { 0xb8c44cb, SUC_lexicon_tags_23 },
    { 0xac6c44cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aac44cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x618c44d1, SUC_lexicon_tags_62 },
    { 0x42ec44d1, SUC_lexicon_tags_85 },
    { 0xa98c44d3, SUC_lexicon_tags_19 },
    { 0x9b8c44d4, SUC_lexicon_tags_12 },
    { 0x8ecc44d5, SUC_lexicon_tags_15 },
    { 0xc30c44d6, SUC_lexicon_tags_79 },
    { 0x796c44d7, SUC_lexicon_tags_12 },
    { 0x72c44d8, SUC_lexicon_tags_32 },
    { 0xcb8c44d6, SUC_lexicon_tags_46 },
    { 0x88cc44da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22c44dd, SUC_lexicon_tags_3 },
    { 0xdc0c44de, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6c44e1, SUC_lexicon_tags_7 },
    { 0xf52c44e2, SUC_lexicon_tags_25 },
    { 0xd94c44e3, SUC_lexicon_tags_7 },
    { 0x538c44e2, SUC_lexicon_tags_27 },
    { 0x336c44e1, SUC_lexicon_tags_45 },
    { 0x932c44e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964c44ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf2c44ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4c44f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x22ec44f5, SUC_lexicon_tags_37 },
    { 0xeaec44f6, SUC_lexicon_tags_58 },
    { 0xda6c44f5, SUC_lexicon_tags_55 },
    { 0xee0c44f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd28c44fa, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddac4509, SUC_lexicon_tags_12 },
    { 0xba0c450a, SUC_lexicon_tags_48 },
    { 0x1f4c450a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ac450e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58c4516, SUC_lexicon_tags_37 },
    { 0xb6c4517, SUC_lexicon_tags_17 },
    { 0x8acc4518, SUC_lexicon_tags_65 },
    { 0xd4ec4519, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ec451c, SUC_lexicon_tags_65 },
    { 0xa7ac451d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ec4528, SUC_lexicon_tags_2 },
    { 0x732c4528, SUC_lexicon_tags_29 },
    { 0x2a8c452a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74cc452d, SUC_lexicon_tags_12 },
    { 0x8d2c452e, SUC_lexicon_tags_49 },
    { 0x6bcc452f, SUC_lexicon_tags_29 },
    { 0xd0cc4530, SUC_lexicon_tags_61 },
    { 0x6a6c4530, SUC_lexicon_tags_12 },
    { 0xc2cc4532, SUC_lexicon_tags_15 },
    { 0xf7ac4532, SUC_lexicon_tags_27 },
    { 0x664c4534, SUC_lexicon_tags_12 },
    { 0xcc8c4530, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6c4538, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8c453d, SUC_lexicon_tags_79 },
    { 0xabcc453e, SUC_lexicon_tags_45 },
    { 0xe96c453f, SUC_lexicon_tags_30 },
    { 0x2a6c4540, SUC_lexicon_tags_25 },
    { 0x298c4541, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576c4547, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcc4549, SUC_lexicon_tags_39 },
    { 0xa18c4549, SUC_lexicon_tags_47 },
    { 0x76c454b, SUC_lexicon_tags_29 },
    { 0xa12c454b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12c454f, SUC_lexicon_tags_12 },
    { 0x1dac4550, SUC_lexicon_tags_49 },
    { 0x50ac4551, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486c455a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd30c455c, SUC_lexicon_tags_28 },
    { 0xe6cc455d, SUC_lexicon_tags_47 },
    { 0x886c455e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54c4560, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8c4568, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5eac456a, SUC_lexicon_tags_38 },
    { 0xe60c456b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70ac456d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x130c4570, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996c4578, SUC_lexicon_tags_2 },
    { 0xa18c4579, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580c457d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x35ec457f, SUC_lexicon_tags_15 },
    { 0xae8c4580, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ec4583, SUC_lexicon_tags_70 },
    { 0x7e0c4584, SUC_lexicon_tags_19 },
    { 0xcd6c4583, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4c4588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dec458b, SUC_lexicon_tags_31 },
    { 0xb3ec458c, SUC_lexicon_tags_37 },
    { 0x86ec458d, SUC_lexicon_tags_37 },
    { 0x402c458e, SUC_lexicon_tags_60 },
    { 0x9d4c458f, SUC_lexicon_tags_33 },
    { 0x1bcc4590, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec4593, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x582c4595, SUC_lexicon_tags_17 },
    { 0xee8c4595, SUC_lexicon_tags_1 },
    { 0x2fcc4597, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x914c4599, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542c45a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x60ec45a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40c45a8, SUC_lexicon_tags_435 },
    { 0xc10c45a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x466c45ab, SUC_lexicon_tags_7 },
    { 0xa0ac45ac, SUC_lexicon_tags_47 },
    { 0xb1cc45ab, SUC_lexicon_tags_80 },
    { 0xfe8c45ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56c45b1, SUC_lexicon_tags_25 },
    { 0x366c45b1, SUC_lexicon_tags_11 },
    { 0x83ac45b3, SUC_lexicon_tags_37 },
    { 0x2e6c45b4, SUC_lexicon_tags_45 },
    { 0xe4ac45b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cc45b8, SUC_lexicon_tags_9 },
    { 0xe60c45b9, SUC_lexicon_tags_27 },
    { 0x29ac45ba, SUC_lexicon_tags_7 },
    { 0xa54c45b9, SUC_lexicon_tags_66 },
    { 0x724c45ba, SUC_lexicon_tags_48 },
    { 0x1dec45bd, SUC_lexicon_tags_12 },
    { 0x324c45bc, SUC_lexicon_tags_43 },
    { 0xadcc45bf, SUC_lexicon_tags_23 },
    { 0x1f8c45c0, SUC_lexicon_tags_1 },
    { 0xf4ac45c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ec45c4, SUC_lexicon_tags_40 },
    { 0xcaec45c5, SUC_lexicon_tags_25 },
    { 0xb08c45c6, SUC_lexicon_tags_37 },
    { 0x964c45c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2fcc45c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd8c45cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52c45d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b8c45d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dac45d8, SUC_lexicon_tags_25 },
    { 0x29ac45d8, SUC_lexicon_tags_29 },
    { 0x56cc45da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8c45e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30c45ea, SUC_lexicon_tags_29 },
    { 0x91ac45eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962c45f0, SUC_lexicon_tags_7 },
    { 0x556c45f1, SUC_lexicon_tags_17 },
    { 0x9d8c45f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cc45f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x110c45f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cc45fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90c4604, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8ec4606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544c460c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac4610, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x792c4612, SUC_lexicon_tags_2 },
    { 0x9fac4613, SUC_lexicon_tags_37 },
    { 0x70cc4614, SUC_lexicon_tags_7 },
    { 0x646c4615, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86c4619, SUC_lexicon_tags_14 },
    { 0x99ec461a, SUC_lexicon_tags_65 },
    { 0x3b6c461b, SUC_lexicon_tags_7 },
    { 0x862c461b, SUC_lexicon_tags_47 },
    { 0x516c461d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x630c461f, SUC_lexicon_tags_17 },
    { 0x92c4620, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xea0c4622, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dac4626, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8c462a, SUC_lexicon_tags_8 },
    { 0x712c462a, SUC_lexicon_tags_16 },
    { 0x22c462c, SUC_lexicon_tags_68 },
    { 0xf5ec462b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x920c4630, SUC_lexicon_tags_12 },
    { 0x918c4631, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cc4635, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bec4638, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c463c, SUC_lexicon_tags_9 },
    { 0xe2ac463d, SUC_lexicon_tags_38 },
    { 0x4fec463e, SUC_lexicon_tags_12 },
    { 0xf96c463f, SUC_lexicon_tags_38 },
    { 0xc6ac463e, SUC_lexicon_tags_77 },
    { 0xebac4640, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ac4647, SUC_lexicon_tags_1556 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8c464e, SUC_lexicon_tags_25 },
    { 0x3a6c464f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc4653, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4c4656, SUC_lexicon_tags_56 },
    { 0xebac4657, SUC_lexicon_tags_172 },
    { 0x678c4658, SUC_lexicon_tags_55 },
    { 0x56ac4659, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ac4663, SUC_lexicon_tags_11 },
    { 0xf0ec4664, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x534c4666, SUC_lexicon_tags_14 },
    { 0x50c4666, SUC_lexicon_tags_42 },
    { 0x7a8c4668, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aec466e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6dac4670, SUC_lexicon_tags_15 },
    { 0xba4c4671, SUC_lexicon_tags_61 },
    { 0xda2c4670, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf1ac4674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98c4677, SUC_lexicon_tags_15 },
    { 0x90cc4678, SUC_lexicon_tags_38 },
    { 0xcd8c4679, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ec467e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc44c4680, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282c4688, SUC_lexicon_tags_48 },
    { 0x340c4689, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6c468d, SUC_lexicon_tags_27 },
    { 0x342c468e, SUC_lexicon_tags_17 },
    { 0x48c468f, SUC_lexicon_tags_2 },
    { 0xd2ac468f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cc4694, SUC_lexicon_tags_46 },
    { 0x850c4694, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0c469d, SUC_lexicon_tags_31 },
    { 0x906c469e, SUC_lexicon_tags_37 },
    { 0x8a0c469d, SUC_lexicon_tags_31 },
    { 0x72ac469d, SUC_lexicon_tags_17 },
    { 0xaf8c46a1, SUC_lexicon_tags_16 },
    { 0xd10c46a2, SUC_lexicon_tags_118 },
    { 0x6a6c46a3, SUC_lexicon_tags_15 },
    { 0xb2ec46a3, SUC_lexicon_tags_33 },
    { 0x872c46a4, SUC_lexicon_tags_9 },
    { 0x5aac46a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x258c46a8, SUC_lexicon_tags_0 },
    { 0x578c46a9, SUC_lexicon_tags_7 },
    { 0x79ac46aa, SUC_lexicon_tags_7 },
    { 0x2aec46ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02c46b4, SUC_lexicon_tags_12 },
    { 0xdb8c46b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fac46ba, SUC_lexicon_tags_11 },
    { 0x5c8c46bb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xac4c46bd, SUC_lexicon_tags_32 },
    { 0x860c46bd, SUC_lexicon_tags_16 },
    { 0x996c46be, SUC_lexicon_tags_38 },
    { 0x93ac46c0, SUC_lexicon_tags_43 },
    { 0x40cc46c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8eec46c3, SUC_lexicon_tags_47 },
    { 0x51ac46c4, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0x374c46c6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c46cc, SUC_lexicon_tags_37 },
    { 0xbe0c46cd, SUC_lexicon_tags_25 },
    { 0x4f0c46cd, SUC_lexicon_tags_25 },
    { 0x46ec46cf, SUC_lexicon_tags_46 },
    { 0x94ec46cc, SUC_lexicon_tags_47 },
    { 0xd78c46cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548c46d7, SUC_lexicon_tags_15 },
    { 0x348c46d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94c46db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe86c46dd, SUC_lexicon_tags_63 },
    { 0xae4c46de, SUC_lexicon_tags_37 },
    { 0xcdec46de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dac46e1, SUC_lexicon_tags_11 },
    { 0x3a4c46e2, SUC_lexicon_tags_37 },
    { 0x6d6c46e3, SUC_lexicon_tags_45 },
    { 0x3d0c46e4, SUC_lexicon_tags_28 },
    { 0xaeac46e5, SUC_lexicon_tags_11 },
    { 0xa18c46e4, SUC_lexicon_tags_31 },
    { 0xca6c46e4, SUC_lexicon_tags_8 },
    { 0x6a4c46e8, SUC_lexicon_tags_1 },
    { 0x7aec46e9, SUC_lexicon_tags_33 },
    { 0x2a8c46ea, SUC_lexicon_tags_37 },
    { 0x394c46ea, SUC_lexicon_tags_35 },
    { 0x978c46eb, SUC_lexicon_tags_60 },
    { 0xe0ec46e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x442c46f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf68c46f2, SUC_lexicon_tags_79 },
    { 0xaf6c46f3, SUC_lexicon_tags_17 },
    { 0x6e4c46f4, SUC_lexicon_tags_14 },
    { 0x8dac46f3, SUC_lexicon_tags_45 },
    { 0xd6ac46f2, SUC_lexicon_tags_61 },
    { 0x8a6c46f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dec46f9, SUC_lexicon_tags_48 },
    { 0xaeac46fa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288c4700, SUC_lexicon_tags_16 },
    { 0x274c4701, SUC_lexicon_tags_23 },
    { 0x5c8c4701, SUC_lexicon_tags_12 },
    { 0x1a0c4703, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cc4709, SUC_lexicon_tags_37 },
    { 0x590c470a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48c4712, SUC_lexicon_tags_106 },
    { 0xa64c4712, SUC_lexicon_tags_31 },
    { 0x766c4714, SUC_lexicon_tags_19 },
    { 0x6d0c4714, SUC_lexicon_tags_17 },
    { 0x2f2c4715, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6c8c4718, SUC_lexicon_tags_43 },
    { 0x7d8c4719, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cc471e, SUC_lexicon_tags_38 },
    { 0x394c471f, SUC_lexicon_tags_15 },
    { 0x332c4720, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84c4723, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dac4727, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ec472b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422c472f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4c4736, SUC_lexicon_tags_46 },
    { 0x67ac4737, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fcc4739, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02c473f, SUC_lexicon_tags_7 },
    { 0x6c2c473f, SUC_lexicon_tags_66 },
    { 0x1e6c4741, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932c4745, SUC_lexicon_tags_9 },
    { 0x11cc4746, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2c4749, SUC_lexicon_tags_33 },
    { 0xd34c474a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd22c474c, SUC_lexicon_tags_46 },
    { 0xe06c474d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4751, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cec4754, SUC_lexicon_tags_45 },
    { 0xe4ac4755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62cc4757, SUC_lexicon_tags_23 },
    { 0xcaec4758, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6c475b, SUC_lexicon_tags_1 },
    { 0xf86c475c, SUC_lexicon_tags_46 },
    { 0x9ccc475c, SUC_lexicon_tags_2 },
    { 0x372c475c, SUC_lexicon_tags_16 },
    { 0x98c475e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6c4763, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e2c4765, SUC_lexicon_tags_27 },
    { 0x75ac4765, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10c4769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64cc476c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa18c476e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77cc4772, SUC_lexicon_tags_25 },
    { 0x35cc4773, SUC_lexicon_tags_37 },
    { 0x724c4772, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeec4777, SUC_lexicon_tags_49 },
    { 0xd34c4777, SUC_lexicon_tags_82 },
    { 0x884c4777, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0c477d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x368c477f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4c4791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d2c4793, SUC_lexicon_tags_12 },
    { 0x24ac4793, SUC_lexicon_tags_11 },
    { 0x498c4795, SUC_lexicon_tags_2 },
    { 0x67ec4796, SUC_lexicon_tags_2 },
    { 0x2a4c4797, SUC_lexicon_tags_30 },
    { 0x880c4795, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ec479c, SUC_lexicon_tags_37 },
    { 0x92cc479d, SUC_lexicon_tags_23 },
    { 0xdfec479e, SUC_lexicon_tags_23 },
    { 0x8f0c479e, SUC_lexicon_tags_15 },
    { 0x8acc479f, SUC_lexicon_tags_13 },
    { 0x8f0c479e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59ec47a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6cac47a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x826c47a7, SUC_lexicon_tags_30 },
    { 0xe92c47a7, SUC_lexicon_tags_33 },
    { 0xd9cc47a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x232c47ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafec47b0, SUC_lexicon_tags_63 },
    { 0x782c47b1, SUC_lexicon_tags_14 },
    { 0x33cc47b1, SUC_lexicon_tags_15 },
    { 0xca2c47b3, SUC_lexicon_tags_45 },
    { 0xd6ac47b4, SUC_lexicon_tags_28 },
    { 0x22cc47b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954c47b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444c47bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130c47c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae8c47c5, SUC_lexicon_tags_17 },
    { 0x136c47c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128c47cb, SUC_lexicon_tags_47 },
    { 0xd62c47cb, SUC_lexicon_tags_12 },
    { 0x534c47cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c47d0, SUC_lexicon_tags_12 },
    { 0x554c47d1, SUC_lexicon_tags_47 },
    { 0xfb0c47d2, SUC_lexicon_tags_7 },
    { 0x1dec47d3, SUC_lexicon_tags_11 },
    { 0x526c47d2, SUC_lexicon_tags_37 },
    { 0x526c47d5, SUC_lexicon_tags_25 },
    { 0x60ac47d2, SUC_lexicon_tags_7 },
    { 0x64ac47d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf88c47d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcc47dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c47e0, SUC_lexicon_tags_1 },
    { 0x1c6c47e1, SUC_lexicon_tags_68 },
    { 0xfb4c47e1, SUC_lexicon_tags_22 },
    { 0x3f6c47e3, SUC_lexicon_tags_15 },
    { 0x6e2c47e4, SUC_lexicon_tags_11 },
    { 0x454c47e1, SUC_lexicon_tags_17 },
    { 0x3ecc47e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218c47ea, SUC_lexicon_tags_12 },
    { 0xd88c47ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10ec47ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8c47f0, SUC_lexicon_tags_37 },
    { 0x9f0c47f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcc47f6, SUC_lexicon_tags_12 },
    { 0x82ac47f6, SUC_lexicon_tags_6 },
    { 0xa42c47f6, SUC_lexicon_tags_11 },
    { 0xf6ac47f9, SUC_lexicon_tags_15 },
    { 0xae4c47f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112c4802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fec480b, SUC_lexicon_tags_11 },
    { 0xd06c480c, SUC_lexicon_tags_216 },
    { 0x5f4c480d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678c4813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562c481b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd5ac481d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30c4823, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aac4827, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x24cc4829, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ec482c, SUC_lexicon_tags_25 },
    { 0x222c482d, SUC_lexicon_tags_27 },
    { 0xaa6c482e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cc4836, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x848c4838, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x278c483a, SUC_lexicon_tags_46 },
    { 0xbc6c483a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeac483e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa06c4840, SUC_lexicon_tags_14 },
    { 0x714c4841, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eec4844, SUC_lexicon_tags_17 },
    { 0x152c4845, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6c4848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4c484b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ec484e, SUC_lexicon_tags_12 },
    { 0xffec484e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8c4854, SUC_lexicon_tags_2 },
    { 0x40cc4855, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c4858, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c485b, SUC_lexicon_tags_91 },
    { 0xb30c485b, SUC_lexicon_tags_6 },
    { 0x43cc485d, SUC_lexicon_tags_6 },
    { 0xc80c485e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cec4861, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4c4865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8c486d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf0c486f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86c4875, SUC_lexicon_tags_0 },
    { 0x3c0c4875, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340c487b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22c487e, SUC_lexicon_tags_80 },
    { 0x86cc487f, SUC_lexicon_tags_25 },
    { 0x264c4880, SUC_lexicon_tags_25 },
    { 0x6cec4881, SUC_lexicon_tags_17 },
    { 0x990c4882, SUC_lexicon_tags_4 },
    { 0x30cc4882, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ac4888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488c488c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42c4890, SUC_lexicon_tags_12 },
    { 0xa26c4891, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaaec4893, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7cec4895, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x922c4898, SUC_lexicon_tags_11 },
    { 0x9aac4899, SUC_lexicon_tags_37 },
    { 0xd40c4898, SUC_lexicon_tags_12 },
    { 0xefac489a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ac489e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ec48a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8c48a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0c48ae, SUC_lexicon_tags_47 },
    { 0x77ec48af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73ac48b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcc48b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26c48b8, SUC_lexicon_tags_41 },
    { 0x478c48b9, SUC_lexicon_tags_15 },
    { 0xa72c48b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb8c48bc, SUC_lexicon_tags_12 },
    { 0x40cc48bd, SUC_lexicon_tags_37 },
    { 0x6d8c48bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd44c48c0, SUC_lexicon_tags_15 },
    { 0xfd0c48c1, SUC_lexicon_tags_47 },
    { 0x4d4c48c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe86c48c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe42c48c6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ec48cd, SUC_lexicon_tags_11 },
    { 0x726c48ce, SUC_lexicon_tags_24 },
    { 0x166c48cf, SUC_lexicon_tags_11 },
    { 0xf52c48d0, SUC_lexicon_tags_1 },
    { 0xd10c48d1, SUC_lexicon_tags_19 },
    { 0x4f8c48d1, SUC_lexicon_tags_49 },
    { 0xd38c48cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326c48d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c48de, SUC_lexicon_tags_25 },
    { 0xc68c48df, SUC_lexicon_tags_49 },
    { 0x3c4c48df, SUC_lexicon_tags_7 },
    { 0x8aec48de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6c48e6, SUC_lexicon_tags_30 },
    { 0xcbcc48e7, SUC_lexicon_tags_38 },
    { 0x54ec48e8, SUC_lexicon_tags_42 },
    { 0xb0cc48e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ec48ee, SUC_lexicon_tags_43 },
    { 0xb56c48ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0c48f6, SUC_lexicon_tags_15 },
    { 0x430c48f7, SUC_lexicon_tags_14 },
    { 0x386c48f8, SUC_lexicon_tags_17 },
    { 0x31ac48f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918c4900, SUC_lexicon_tags_28 },
    { 0xc42c4900, SUC_lexicon_tags_2 },
    { 0xd6ac4900, SUC_lexicon_tags_16 },
    { 0x4e0c4901, SUC_lexicon_tags_17 },
    { 0xdf6c4904, SUC_lexicon_tags_15 },
    { 0xb2cc4904, SUC_lexicon_tags_182 },
    { 0x580c4901, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa46c4908, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6c490c, SUC_lexicon_tags_7 },
    { 0x52ec490c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ac4912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156c4918, SUC_lexicon_tags_68 },
    { 0x370c4918, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4c491e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4c4921, SUC_lexicon_tags_17 },
    { 0xbf8c4922, SUC_lexicon_tags_10 },
    { 0x266c4922, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x244c4925, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2c4928, SUC_lexicon_tags_1 },
    { 0x954c4928, SUC_lexicon_tags_9 },
    { 0x606c492a, SUC_lexicon_tags_23 },
    { 0x876c492a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9eac492d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe4c492f, SUC_lexicon_tags_1 },
    { 0x402c492f, SUC_lexicon_tags_10 },
    { 0x3b4c4931, SUC_lexicon_tags_82 },
    { 0x2b4c4932, SUC_lexicon_tags_15 },
    { 0xd38c4931, SUC_lexicon_tags_14 },
    { 0x4f0c4932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bac4939, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x382c493c, SUC_lexicon_tags_29 },
    { 0xb4c493c, SUC_lexicon_tags_27 },
    { 0xeb0c493d, SUC_lexicon_tags_17 },
    { 0x304c493c, SUC_lexicon_tags_38 },
    { 0x500c493e, SUC_lexicon_tags_23 },
    { 0x9ccc4941, SUC_lexicon_tags_23 },
    { 0x1f6c4942, SUC_lexicon_tags_51 },
    { 0x932c4942, SUC_lexicon_tags_33 },
    { 0x7e8c493e, SUC_lexicon_tags_141 },
    { 0x2c4c4945, SUC_lexicon_tags_13 },
    { 0x6fec4946, SUC_lexicon_tags_0 },
    { 0x186c4947, SUC_lexicon_tags_7 },
    { 0x1eac4945, SUC_lexicon_tags_1625 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac4950, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc88c4952, SUC_lexicon_tags_31 },
    { 0x7b4c4952, SUC_lexicon_tags_29 },
    { 0x82cc4954, SUC_lexicon_tags_1 },
    { 0xc5ac4954, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1c8c4957, SUC_lexicon_tags_47 },
    { 0x234c4958, SUC_lexicon_tags_14 },
    { 0xc2cc4959, SUC_lexicon_tags_43 },
    { 0xf10c495a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x95cc495c, SUC_lexicon_tags_25 },
    { 0x74ec495c, SUC_lexicon_tags_12 },
    { 0xc3cc495c, SUC_lexicon_tags_60 },
    { 0x6a8c495e, SUC_lexicon_tags_15 },
    { 0x8eac495f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe32c4962, SUC_lexicon_tags_1 },
    { 0xc50c4963, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcc4966, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecc4969, SUC_lexicon_tags_12 },
    { 0x884c496a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57ec496c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124c4976, SUC_lexicon_tags_26 },
    { 0x6f4c4977, SUC_lexicon_tags_2 },
    { 0x952c4978, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x840c497a, SUC_lexicon_tags_13 },
    { 0x740c497b, SUC_lexicon_tags_2 },
    { 0x182c497c, SUC_lexicon_tags_17 },
    { 0xfecc497d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x614c497f, SUC_lexicon_tags_21 },
    { 0x8b0c4980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4c4983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b8c4985, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa8ec4987, SUC_lexicon_tags_48 },
    { 0x116c4988, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd84c498a, SUC_lexicon_tags_38 },
    { 0xca0c498a, SUC_lexicon_tags_15 },
    { 0x30c498b, SUC_lexicon_tags_49 },
    { 0xd0ec498a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55ec498f, SUC_lexicon_tags_17 },
    { 0xd2ec4990, SUC_lexicon_tags_1 },
    { 0xf7cc4990, SUC_lexicon_tags_13 },
    { 0x9c8c4992, SUC_lexicon_tags_1 },
    { 0xbcec4992, SUC_lexicon_tags_8 },
    { 0x11ac4991, SUC_lexicon_tags_29 },
    { 0x4dcc4991, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2c499b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46c499d, SUC_lexicon_tags_63 },
    { 0xe30c499e, SUC_lexicon_tags_14 },
    { 0xaccc499d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852c49a4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x236c49a7, SUC_lexicon_tags_68 },
    { 0xde4c49a8, SUC_lexicon_tags_35 },
    { 0x8bac49a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa7ac49ab, SUC_lexicon_tags_103 },
    { 0xf36c49ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122c49b6, SUC_lexicon_tags_25 },
    { 0x7bcc49b7, SUC_lexicon_tags_1 },
    { 0x7c0c49b8, SUC_lexicon_tags_23 },
    { 0x716c49b9, SUC_lexicon_tags_49 },
    { 0x29ac49b9, SUC_lexicon_tags_10 },
    { 0xf8ec49b9, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252c49c1, SUC_lexicon_tags_15 },
    { 0x7d2c49c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28cc49c4, SUC_lexicon_tags_27 },
    { 0xb2c49c4, SUC_lexicon_tags_21 },
    { 0x1c8c49c6, SUC_lexicon_tags_19 },
    { 0x9ac49c7, SUC_lexicon_tags_1 },
    { 0x1f8c49c8, SUC_lexicon_tags_77 },
    { 0xe54c49c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c4c49cb, SUC_lexicon_tags_37 },
    { 0x4a6c49cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452c49d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eec49d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8c49d8, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bac49db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c49df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cc49e3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86c49e6, SUC_lexicon_tags_11 },
    { 0xc60c49e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0c49ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16c49f0, SUC_lexicon_tags_9 },
    { 0x4c49f1, SUC_lexicon_tags_1 },
    { 0xfbec49f2, SUC_lexicon_tags_11 },
    { 0x9cc49f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f0c49f5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4c49fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ac49ff, SUC_lexicon_tags_2 },
    { 0x6dac4a00, SUC_lexicon_tags_2 },
    { 0xf3ac4a01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92c4a04, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x96ac4a06, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408c4a0a, SUC_lexicon_tags_15 },
    { 0x7eac4a0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886c4a0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8c4a14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc88c4a16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a4c4a18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1b6c4a1a, SUC_lexicon_tags_19 },
    { 0xe18c4a1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ac4a1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b6c4a21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x10c4a23, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd6cc4a25, SUC_lexicon_tags_16 },
    { 0xb76c4a26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bac4a2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6c4a32, SUC_lexicon_tags_12 },
    { 0x5bec4a33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aec4a36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60c4a39, SUC_lexicon_tags_37 },
    { 0xdaec4a3a, SUC_lexicon_tags_25 },
    { 0xb4cc4a3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2c4a3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x878c4a41, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadac4a48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac4a4a, SUC_lexicon_tags_11 },
    { 0xa00c4a4b, SUC_lexicon_tags_28 },
    { 0x6c8c4a4c, SUC_lexicon_tags_25 },
    { 0x9f2c4a4d, SUC_lexicon_tags_29 },
    { 0xed2c4a4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8c4a53, SUC_lexicon_tags_14 },
    { 0xa50c4a54, SUC_lexicon_tags_63 },
    { 0x678c4a55, SUC_lexicon_tags_17 },
    { 0xafcc4a56, SUC_lexicon_tags_17 },
    { 0x2d0c4a54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x384c4a59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42c4a5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ac4a5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192c4a62, SUC_lexicon_tags_65 },
    { 0x46ac4a62, SUC_lexicon_tags_15 },
    { 0x244c4a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922c4a72, SUC_lexicon_tags_37 },
    { 0x81cc4a72, SUC_lexicon_tags_47 },
    { 0x23ac4a72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c4a77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x126c4a7a, SUC_lexicon_tags_47 },
    { 0x5f0c4a7b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x872c4a7e, SUC_lexicon_tags_19 },
    { 0x432c4a7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2c4a83, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cec4a87, SUC_lexicon_tags_9 },
    { 0x34ac4a88, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678c4a8c, SUC_lexicon_tags_107 },
    { 0xa0ec4a8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x608c4a8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c4c4a91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bac4a94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb60c4a96, SUC_lexicon_tags_47 },
    { 0xa74c4a96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x378c4a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18c4a9d, SUC_lexicon_tags_21 },
    { 0x63cc4a9e, SUC_lexicon_tags_65 },
    { 0x3dac4a9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x434c4aa1, SUC_lexicon_tags_7 },
    { 0x9c0c4aa2, SUC_lexicon_tags_37 },
    { 0xd50c4aa3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4c4aa7, SUC_lexicon_tags_12 },
    { 0xc3ec4aa8, SUC_lexicon_tags_15 },
    { 0x7d0c4aa8, SUC_lexicon_tags_31 },
    { 0x392c4aa9, SUC_lexicon_tags_39 },
    { 0x4ec4aa8, SUC_lexicon_tags_82 },
    { 0xb04c4aac, SUC_lexicon_tags_37 },
    { 0xddec4aa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fac4aaf, SUC_lexicon_tags_2 },
    { 0x26c4ab0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18cc4ab2, SUC_lexicon_tags_15 },
    { 0x6f4c4ab3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcc4ab6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13ec4ab8, SUC_lexicon_tags_56 },
    { 0xa60c4ab9, SUC_lexicon_tags_17 },
    { 0x264c4ab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358c4abe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ac4ac6, SUC_lexicon_tags_12 },
    { 0x61ac4ac7, SUC_lexicon_tags_33 },
    { 0xc4ac8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46c4acb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38c4ad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcc4ad4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76c4ada, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2c4add, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146c4ae3, SUC_lexicon_tags_47 },
    { 0xc8ec4ae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600c4aea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58c4aee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ec4af1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe20c4af3, SUC_lexicon_tags_12 },
    { 0xc86c4af3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288c4af8, SUC_lexicon_tags_12 },
    { 0x27ec4af9, SUC_lexicon_tags_2 },
    { 0x96ac4af9, SUC_lexicon_tags_37 },
    { 0x8bcc4afb, SUC_lexicon_tags_71 },
    { 0x332c4afc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4c4b00, SUC_lexicon_tags_11 },
    { 0x886c4b01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x458c4b03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28c4b06, SUC_lexicon_tags_15 },
    { 0x62ec4b07, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccc4b0b, SUC_lexicon_tags_1 },
    { 0xe00c4b0c, SUC_lexicon_tags_12 },
    { 0xbccc4b0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ac4b12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2ccc4b14, SUC_lexicon_tags_60 },
    { 0x5bec4b15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706c4b1a, SUC_lexicon_tags_8 },
    { 0xa34c4b1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10ac4b1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6fec4b1f, SUC_lexicon_tags_8 },
    { 0xe0ac4b20, SUC_lexicon_tags_11 },
    { 0x7fcc4b1f, SUC_lexicon_tags_15 },
    { 0x85ec4b22, SUC_lexicon_tags_27 },
    { 0xd2ec4b23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cc4b26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x814c4b28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cc4b2b, SUC_lexicon_tags_11 },
    { 0xedec4b2c, SUC_lexicon_tags_15 },
    { 0xc32c4b2c, SUC_lexicon_tags_82 },
    { 0x634c4b2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4c4b31, SUC_lexicon_tags_30 },
    { 0x3fcc4b31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ec4b3a, SUC_lexicon_tags_0 },
    { 0x3b4c4b3b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94c4b3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a0c4b41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2e6c4b43, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x70ac4b45, SUC_lexicon_tags_60 },
    { 0xb3ac4b46, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ec4b49, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ac4b4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ec4b50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x774c4b52, SUC_lexicon_tags_17 },
    { 0xa46c4b53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ac4b56, SUC_lexicon_tags_7 },
    { 0x3f0c4b57, SUC_lexicon_tags_29 },
    { 0x710c4b58, SUC_lexicon_tags_46 },
    { 0x7d6c4b58, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcc4b60, SUC_lexicon_tags_12 },
    { 0x8a6c4b61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872c4b66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2c4b69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ec4b6d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cc4b70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca4c4b72, SUC_lexicon_tags_1 },
    { 0xfe8c4b73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c4b76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c4b79, SUC_lexicon_tags_47 },
    { 0x59ac4b7a, SUC_lexicon_tags_8 },
    { 0x4a0c4b7a, SUC_lexicon_tags_49 },
    { 0x78ec4b7c, SUC_lexicon_tags_47 },
    { 0xe60c4b7c, SUC_lexicon_tags_11 },
    { 0x642c4b7d, SUC_lexicon_tags_10 },
    { 0x610c4b7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4b87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcc4b8f, SUC_lexicon_tags_53 },
    { 0x444c4b90, SUC_lexicon_tags_17 },
    { 0x798c4b91, SUC_lexicon_tags_69 },
    { 0x886c4b91, SUC_lexicon_tags_39 },
    { 0x8f8c4b92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa78c4b95, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3e2c4b97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318c4b9f, SUC_lexicon_tags_1 },
    { 0x2cec4ba0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x312c4ba2, SUC_lexicon_tags_0 },
    { 0x9e6c4ba3, SUC_lexicon_tags_37 },
    { 0xffac4ba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ac4baa, SUC_lexicon_tags_11 },
    { 0xf84c4bab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46c4bb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x230c4bb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d0c4bb4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74c4bb7, SUC_lexicon_tags_38 },
    { 0x3c6c4bb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84c4bbb, SUC_lexicon_tags_2 },
    { 0x83ec4bbc, SUC_lexicon_tags_11 },
    { 0xfe6c4bbd, SUC_lexicon_tags_38 },
    { 0x90ec4bbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x344c4bc0, SUC_lexicon_tags_63 },
    { 0xf46c4bc1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9cac4bc3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ec4bc6, SUC_lexicon_tags_56 },
    { 0xca6c4bc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97ac4bc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaac4bcd, SUC_lexicon_tags_8 },
    { 0xfdec4bcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2cc4bd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffec4bd8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8c4bde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8c4be5, SUC_lexicon_tags_97 },
    { 0x8bec4be6, SUC_lexicon_tags_25 },
    { 0xfd8c4be7, SUC_lexicon_tags_33 },
    { 0x7ecc4be6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcc4bec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518c4bf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866c4bf4, SUC_lexicon_tags_2 },
    { 0x388c4bf5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58c4bf8, SUC_lexicon_tags_15 },
    { 0xc80c4bf8, SUC_lexicon_tags_37 },
    { 0x8a6c4bfa, SUC_lexicon_tags_7 },
    { 0x9e8c4bfb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68c4bff, SUC_lexicon_tags_37 },
    { 0xff8c4bff, SUC_lexicon_tags_45 },
    { 0x500c4bff, SUC_lexicon_tags_45 },
    { 0xb6cc4bff, SUC_lexicon_tags_2 },
    { 0xb1cc4bff, SUC_lexicon_tags_27 },
    { 0x13cc4c00, SUC_lexicon_tags_6 },
    { 0x75ec4c05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8c4c0d, SUC_lexicon_tags_0 },
    { 0x3bcc4c0e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0c4c15, SUC_lexicon_tags_9 },
    { 0x46ec4c16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6c4c1b, SUC_lexicon_tags_15 },
    { 0x832c4c1c, SUC_lexicon_tags_10 },
    { 0x3d6c4c1d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0c4c20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88c4c23, SUC_lexicon_tags_16 },
    { 0xa6cc4c23, SUC_lexicon_tags_12 },
    { 0xa22c4c25, SUC_lexicon_tags_1 },
    { 0xa26c4c26, SUC_lexicon_tags_19 },
    { 0xdec4c23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x752c4c2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48c4c2e, SUC_lexicon_tags_8 },
    { 0xb94c4c2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572c4c33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80c4c39, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x42c4c3b, SUC_lexicon_tags_6 },
    { 0x8a8c4c3c, SUC_lexicon_tags_27 },
    { 0x668c4c3d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf3ec4c3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74c4c42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6c4c4a, SUC_lexicon_tags_43 },
    { 0x6a8c4c4b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x894c4c4d, SUC_lexicon_tags_2 },
    { 0x286c4c4e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8c4c55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fac4c58, SUC_lexicon_tags_25 },
    { 0xb26c4c58, SUC_lexicon_tags_39 },
    { 0x75ec4c5a, SUC_lexicon_tags_31 },
    { 0x25ac4c5a, SUC_lexicon_tags_19 },
    { 0xf8ac4c5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf96c4c5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2ac4c60, SUC_lexicon_tags_2 },
    { 0x54cc4c60, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe4c4c63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71ac4c65, SUC_lexicon_tags_3 },
    { 0xf4cc4c66, SUC_lexicon_tags_15 },
    { 0x290c4c67, SUC_lexicon_tags_265 },
    { 0xc02c4c68, SUC_lexicon_tags_30 },
    { 0xa04c4c66, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd34c4c6b, SUC_lexicon_tags_17 },
    { 0xe44c4c6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86c4c71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6fcc4c73, SUC_lexicon_tags_31 },
    { 0xef6c4c73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d8c4c76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6c4c7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc76c4c7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38c4c82, SUC_lexicon_tags_25 },
    { 0xae8c4c83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68ec4c85, SUC_lexicon_tags_9 },
    { 0x826c4c86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ac4c89, SUC_lexicon_tags_37 },
    { 0x5ac4c89, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd50c4c8c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4c4c8f, SUC_lexicon_tags_26 },
    { 0xc9ac4c90, SUC_lexicon_tags_184 },
    { 0x254c4c8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb76c4c93, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc0c4c95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x792c4c98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36c4c9f, SUC_lexicon_tags_37 },
    { 0x270c4ca0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0c4cab, SUC_lexicon_tags_17 },
    { 0x6ecc4cac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa4c4cae, SUC_lexicon_tags_12 },
    { 0x7d2c4caf, SUC_lexicon_tags_0 },
    { 0xdecc4cb0, SUC_lexicon_tags_33 },
    { 0x2dac4cb1, SUC_lexicon_tags_38 },
    { 0xf70c4cb2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf4c4cb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0c4cba, SUC_lexicon_tags_46 },
    { 0x1f0c4cbb, SUC_lexicon_tags_17 },
    { 0x264c4cba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764c4cc0, SUC_lexicon_tags_0 },
    { 0x878c4cc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20ec4cc3, SUC_lexicon_tags_15 },
    { 0x100c4cc3, SUC_lexicon_tags_11 },
    { 0x7a2c4cc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0c4ccd, SUC_lexicon_tags_68 },
    { 0x1f2c4cce, SUC_lexicon_tags_27 },
    { 0xc94c4ccf, SUC_lexicon_tags_14 },
    { 0x396c4ccf, SUC_lexicon_tags_29 },
    { 0xde0c4cd1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d2c4cd3, SUC_lexicon_tags_37 },
    { 0x76ec4cd4, SUC_lexicon_tags_12 },
    { 0x9d8c4cd5, SUC_lexicon_tags_31 },
    { 0x932c4cd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12c4cd9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x110c4cdb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2c4cdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec4ce6, SUC_lexicon_tags_43 },
    { 0xad2c4ce6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x62ac4ce9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cc4cec, SUC_lexicon_tags_10 },
    { 0x2aec4ced, SUC_lexicon_tags_11 },
    { 0x9dec4cee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ac4cf2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cc4cf5, SUC_lexicon_tags_1 },
    { 0xd06c4cf5, SUC_lexicon_tags_1 },
    { 0x604c4cf7, SUC_lexicon_tags_2 },
    { 0x5b8c4cf6, SUC_lexicon_tags_12 },
    { 0x53cc4cf9, SUC_lexicon_tags_70 },
    { 0x5c4c4cfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ac4cff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3bec4d01, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ac4d04, SUC_lexicon_tags_17 },
    { 0x392c4d05, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76c4d0a, SUC_lexicon_tags_39 },
    { 0xdd2c4d0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c4d0f, SUC_lexicon_tags_60 },
    { 0x3d0c4d10, SUC_lexicon_tags_23 },
    { 0xf08c4d10, SUC_lexicon_tags_29 },
    { 0x6ecc4d11, SUC_lexicon_tags_15 },
    { 0x418c4d0f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb0ac4d15, SUC_lexicon_tags_19 },
    { 0x378c4d16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7cc4d18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0c4d1c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x400c4d1f, SUC_lexicon_tags_31 },
    { 0x986c4d1f, SUC_lexicon_tags_23 },
    { 0xc58c4d21, SUC_lexicon_tags_390 },
    { 0x474c4d22, SUC_lexicon_tags_11 },
    { 0x8e2c4d21, SUC_lexicon_tags_31 },
    { 0xf48c4d24, SUC_lexicon_tags_27 },
    { 0x97cc4d25, SUC_lexicon_tags_17 },
    { 0x93ec4d25, SUC_lexicon_tags_11 },
    { 0xa70c4d1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab0c4d29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x676c4d2c, SUC_lexicon_tags_49 },
    { 0xc94c4d2c, SUC_lexicon_tags_47 },
    { 0x5f2c4d2d, SUC_lexicon_tags_37 },
    { 0x9c6c4d2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8c4d33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcc4d37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeac4d3a, SUC_lexicon_tags_6 },
    { 0x500c4d3a, SUC_lexicon_tags_16 },
    { 0xc2ac4d3b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886c4d40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194c4d46, SUC_lexicon_tags_23 },
    { 0x490c4d47, SUC_lexicon_tags_37 },
    { 0x506c4d46, SUC_lexicon_tags_11 },
    { 0xa32c4d49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96c4d50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa14c4d52, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2c4d56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9ec4d58, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ac4d62, SUC_lexicon_tags_37 },
    { 0x86ec4d63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeac4d66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x100c4d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b2c4d6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b2c4d6c, SUC_lexicon_tags_19 },
    { 0xb56c4d6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79ec4d6f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ac4d76, SUC_lexicon_tags_28 },
    { 0x8e6c4d76, SUC_lexicon_tags_16 },
    { 0x69ec4d76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ac4d7b, SUC_lexicon_tags_2 },
    { 0x65ec4d7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5fcc4d7e, SUC_lexicon_tags_15 },
    { 0x82c4d7f, SUC_lexicon_tags_25 },
    { 0x23ac4d80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2ac4d82, SUC_lexicon_tags_21 },
    { 0xc2ec4d83, SUC_lexicon_tags_301 },
    { 0xca6c4d83, SUC_lexicon_tags_12 },
    { 0x384c4d84, SUC_lexicon_tags_26 },
    { 0xdc4c4d86, SUC_lexicon_tags_17 },
    { 0x666c4d84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x184c4d89, SUC_lexicon_tags_37 },
    { 0xea6c4d8a, SUC_lexicon_tags_25 },
    { 0x402c4d8a, SUC_lexicon_tags_30 },
    { 0x252c4d8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c4c4d8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ec4d99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44c4d9c, SUC_lexicon_tags_0 },
    { 0x844c4d9d, SUC_lexicon_tags_11 },
    { 0x614c4d9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd60c4da0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ac4da2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf5ac4da4, SUC_lexicon_tags_14 },
    { 0xa98c4da4, SUC_lexicon_tags_19 },
    { 0xea0c4da6, SUC_lexicon_tags_9 },
    { 0xdb2c4da7, SUC_lexicon_tags_17 },
    { 0xcfec4da8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15ac4daa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ec4daf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38ec4db1, SUC_lexicon_tags_1 },
    { 0x300c4db2, SUC_lexicon_tags_60 },
    { 0xf76c4db1, SUC_lexicon_tags_15 },
    { 0xa14c4db4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb9ac4db6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f4c4db8, SUC_lexicon_tags_35 },
    { 0x6a4c4db9, SUC_lexicon_tags_7 },
    { 0x1d4c4dba, SUC_lexicon_tags_33 },
    { 0xc90c4dbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8c4dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1cc4dc3, SUC_lexicon_tags_31 },
    { 0xc9cc4dc4, SUC_lexicon_tags_7 },
    { 0x71ec4dc5, SUC_lexicon_tags_2 },
    { 0x856c4dc6, SUC_lexicon_tags_7 },
    { 0xbb2c4dc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72ac4dc9, SUC_lexicon_tags_29 },
    { 0x2bec4dca, SUC_lexicon_tags_114 },
    { 0x8f2c4dcb, SUC_lexicon_tags_11 },
    { 0x28c4dca, SUC_lexicon_tags_5 },
    { 0x2bac4dcc, SUC_lexicon_tags_7 },
    { 0xadec4dce, SUC_lexicon_tags_6 },
    { 0x9d8c4dcb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x800c4dd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ac4dd7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2c4de3, SUC_lexicon_tags_45 },
    { 0x276c4de4, SUC_lexicon_tags_57 },
    { 0xd1ec4de4, SUC_lexicon_tags_1010 },
    { 0xf4ac4de4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x284c4de8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230c4dee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94c4df1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x432c4df4, SUC_lexicon_tags_387 },
    { 0, NULL },
    { 0, NULL },
    { 0xcacc4df7, SUC_lexicon_tags_17 },
    { 0xa16c4df7, SUC_lexicon_tags_14 },
    { 0x49ac4df9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2c4dfd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248c4e02, SUC_lexicon_tags_27 },
    { 0x694c4e02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886c4e07, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44c4e0d, SUC_lexicon_tags_38 },
    { 0x9e2c4e0e, SUC_lexicon_tags_28 },
    { 0x8cac4e0d, SUC_lexicon_tags_68 },
    { 0xc0c4e10, SUC_lexicon_tags_25 },
    { 0x4ac4e10, SUC_lexicon_tags_12 },
    { 0x8b8c4e12, SUC_lexicon_tags_63 },
    { 0xb66c4e13, SUC_lexicon_tags_8 },
    { 0xb1ec4e10, SUC_lexicon_tags_49 },
    { 0x86ac4e10, SUC_lexicon_tags_11 },
    { 0x986c4e16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1fec4e18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94c4e25, SUC_lexicon_tags_12 },
    { 0xaa8c4e26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ec4e30, SUC_lexicon_tags_26 },
    { 0xff8c4e31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c4e36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce6c4e38, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cc4e3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeec4e40, SUC_lexicon_tags_19 },
    { 0xb2ac4e41, SUC_lexicon_tags_45 },
    { 0x44c4e40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x498c4e44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec4e49, SUC_lexicon_tags_47 },
    { 0x462c4e49, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c4e4e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ac4e52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab2c4e54, SUC_lexicon_tags_30 },
    { 0x5ecc4e55, SUC_lexicon_tags_27 },
    { 0x8d4c4e55, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x21cc4e59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb6ac4e5b, SUC_lexicon_tags_12 },
    { 0x1dac4e5b, SUC_lexicon_tags_14 },
    { 0x272c4e5b, SUC_lexicon_tags_60 },
    { 0x5eec4e5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cc4e65, SUC_lexicon_tags_12 },
    { 0xc22c4e66, SUC_lexicon_tags_15 },
    { 0x8d0c4e66, SUC_lexicon_tags_1 },
    { 0x2e6c4e67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4c4e6b, SUC_lexicon_tags_17 },
    { 0x1a2c4e6c, SUC_lexicon_tags_11 },
    { 0xd76c4e6b, SUC_lexicon_tags_25 },
    { 0x72c4e6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc4e74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cc4e7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x858c4e7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686c4e82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a4c4e84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1ec4e86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4c4e89, SUC_lexicon_tags_69 },
    { 0xf80c4e8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddec4e8e, SUC_lexicon_tags_25 },
    { 0xcec4e8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4c4e93, SUC_lexicon_tags_11 },
    { 0xb3ec4e94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aac4e99, SUC_lexicon_tags_33 },
    { 0xf3cc4e9a, SUC_lexicon_tags_8 },
    { 0xd66c4e9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x108c4e9d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x576c4e9f, SUC_lexicon_tags_17 },
    { 0xdbcc4ea0, SUC_lexicon_tags_28 },
    { 0xd36c4ea1, SUC_lexicon_tags_63 },
    { 0x82cc4ea0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fac4eaf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828c4eb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f0c4eb6, SUC_lexicon_tags_47 },
    { 0xdacc4eb6, SUC_lexicon_tags_8 },
    { 0xf2c4eb8, SUC_lexicon_tags_0 },
    { 0xda2c4eb9, SUC_lexicon_tags_4 },
    { 0xac4c4eb6, SUC_lexicon_tags_14 },
    { 0x838c4eba, SUC_lexicon_tags_7 },
    { 0xd42c4ebc, SUC_lexicon_tags_29 },
    { 0xd8c4ebd, SUC_lexicon_tags_12 },
    { 0x20c4ebd, SUC_lexicon_tags_27 },
    { 0xb5cc4ebe, SUC_lexicon_tags_43 },
    { 0xe70c4ebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x786c4ec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dac4ec7, SUC_lexicon_tags_16 },
    { 0x92cc4ec8, SUC_lexicon_tags_60 },
    { 0x420c4ec9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12c4ecc, SUC_lexicon_tags_1 },
    { 0x1fcc4ecc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4c4ed1, SUC_lexicon_tags_25 },
    { 0xe1ec4ed2, SUC_lexicon_tags_11 },
    { 0xec8c4ed1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ac4ed7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2c4ede, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04c4ee1, SUC_lexicon_tags_1 },
    { 0x3a6c4ee2, SUC_lexicon_tags_37 },
    { 0x48c4ee2, SUC_lexicon_tags_12 },
    { 0x146c4ee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96ec4ee6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8c4eef, SUC_lexicon_tags_2 },
    { 0x436c4eef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dec4ef5, SUC_lexicon_tags_21 },
    { 0xabec4ef6, SUC_lexicon_tags_15 },
    { 0x86c4ef5, SUC_lexicon_tags_11 },
    { 0x304c4ef8, SUC_lexicon_tags_37 },
    { 0x106c4ef9, SUC_lexicon_tags_16 },
    { 0x58c4ef8, SUC_lexicon_tags_33 },
    { 0x55ac4ef8, SUC_lexicon_tags_10 },
    { 0xaacc4ef9, SUC_lexicon_tags_11 },
    { 0x48cc4efd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dec4eff, SUC_lexicon_tags_1 },
    { 0xdc6c4eff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x704c4f03, SUC_lexicon_tags_60 },
    { 0xd8ec4f03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82c4f07, SUC_lexicon_tags_37 },
    { 0x292c4f08, SUC_lexicon_tags_27 },
    { 0x754c4f09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cc4f0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x758c4f0f, SUC_lexicon_tags_7 },
    { 0x252c4f10, SUC_lexicon_tags_10 },
    { 0xe0ac4f11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cc4f17, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ec4f1b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236c4f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedac4f27, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4c4f2a, SUC_lexicon_tags_17 },
    { 0x148c4f2b, SUC_lexicon_tags_11 },
    { 0x68ec4f2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2dac4f2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccc4f36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe70c4f38, SUC_lexicon_tags_61 },
    { 0xcfcc4f39, SUC_lexicon_tags_11 },
    { 0xa30c4f39, SUC_lexicon_tags_33 },
    { 0xc8ac4f3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cac4f42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ac4f46, SUC_lexicon_tags_31 },
    { 0xe30c4f47, SUC_lexicon_tags_178 },
    { 0xe90c4f48, SUC_lexicon_tags_12 },
    { 0x72c4f49, SUC_lexicon_tags_29 },
    { 0x8f8c4f4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x142c4f4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x24ac4f4e, SUC_lexicon_tags_1 },
    { 0x52ac4f4f, SUC_lexicon_tags_37 },
    { 0x23ac4f4f, SUC_lexicon_tags_29 },
    { 0xf00c4f50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b0c4f53, SUC_lexicon_tags_47 },
    { 0xeccc4f53, SUC_lexicon_tags_27 },
    { 0xb12c4f53, SUC_lexicon_tags_65 },
    { 0x38cc4f53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cc4f59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14cc4f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cc4f5f, SUC_lexicon_tags_7 },
    { 0x438c4f60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x472c4f63, SUC_lexicon_tags_28 },
    { 0xfb0c4f64, SUC_lexicon_tags_63 },
    { 0xfa0c4f65, SUC_lexicon_tags_1 },
    { 0x914c4f65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cc4f6c, SUC_lexicon_tags_48 },
    { 0xd06c4f6d, SUC_lexicon_tags_43 },
    { 0xd44c4f6e, SUC_lexicon_tags_1 },
    { 0x970c4f6f, SUC_lexicon_tags_23 },
    { 0xa14c4f6f, SUC_lexicon_tags_11 },
    { 0x6d6c4f71, SUC_lexicon_tags_28 },
    { 0x8ec4f72, SUC_lexicon_tags_37 },
    { 0xd64c4f73, SUC_lexicon_tags_63 },
    { 0x2acc4f74, SUC_lexicon_tags_63 },
    { 0x842c4f74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf6cc4f77, SUC_lexicon_tags_49 },
    { 0xb50c4f77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb10c4f7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28c4f7d, SUC_lexicon_tags_17 },
    { 0xd5cc4f7e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x148c4f81, SUC_lexicon_tags_60 },
    { 0xea4c4f82, SUC_lexicon_tags_23 },
    { 0x1b4c4f83, SUC_lexicon_tags_8 },
    { 0x7bec4f83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18c4f8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40c4f90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116c4f98, SUC_lexicon_tags_17 },
    { 0x578c4f98, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x902c4f9b, SUC_lexicon_tags_2 },
    { 0xa78c4f9c, SUC_lexicon_tags_38 },
    { 0x14c4f9d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe22c4f9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108c4fa4, SUC_lexicon_tags_11 },
    { 0x634c4fa4, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aac4fa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72c4faf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4c4fbb, SUC_lexicon_tags_33 },
    { 0x944c4fbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e4c4fbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x612c4fc0, SUC_lexicon_tags_37 },
    { 0x282c4fc0, SUC_lexicon_tags_32 },
    { 0x676c4fc1, SUC_lexicon_tags_15 },
    { 0x75ec4fc3, SUC_lexicon_tags_84 },
    { 0x88ec4fc4, SUC_lexicon_tags_38 },
    { 0xd4cc4fc3, SUC_lexicon_tags_6 },
    { 0x1ccc4fc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe02c4fc8, SUC_lexicon_tags_47 },
    { 0x154c4fc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd88c4fcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444c4fd0, SUC_lexicon_tags_19 },
    { 0xfeec4fd0, SUC_lexicon_tags_25 },
    { 0xedcc4fd2, SUC_lexicon_tags_28 },
    { 0x15cc4fd0, SUC_lexicon_tags_18 },
    { 0x66c4fd4, SUC_lexicon_tags_3 },
    { 0xaac4fd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52c4fdc, SUC_lexicon_tags_1 },
    { 0x82ec4fdd, SUC_lexicon_tags_4 },
    { 0x140c4fdc, SUC_lexicon_tags_43 },
    { 0xbf8c4fdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4c4fe2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8c4fe8, SUC_lexicon_tags_43 },
    { 0x618c4fe9, SUC_lexicon_tags_56 },
    { 0x36ec4fea, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806c4fef, SUC_lexicon_tags_27 },
    { 0x96ec4fef, SUC_lexicon_tags_17 },
    { 0xdf6c4ff1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xc62c4ff3, SUC_lexicon_tags_19 },
    { 0x256c4ff3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0c4ffb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8c5002, SUC_lexicon_tags_38 },
    { 0x8bac5003, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c5006, SUC_lexicon_tags_25 },
    { 0x494c5006, SUC_lexicon_tags_23 },
    { 0xc62c5008, SUC_lexicon_tags_23 },
    { 0xc0c5009, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fac500b, SUC_lexicon_tags_2 },
    { 0xedec500c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4c500f, SUC_lexicon_tags_25 },
    { 0xb28c500f, SUC_lexicon_tags_38 },
    { 0x258c500f, SUC_lexicon_tags_6 },
    { 0xd62c5012, SUC_lexicon_tags_9 },
    { 0x752c500f, SUC_lexicon_tags_17 },
    { 0xdc2c5014, SUC_lexicon_tags_48 },
    { 0xfd2c5014, SUC_lexicon_tags_24 },
    { 0x3d4c5015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cc5018, SUC_lexicon_tags_44 },
    { 0xcaec5019, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56c501d, SUC_lexicon_tags_47 },
    { 0x418c501d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4c5021, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94c5024, SUC_lexicon_tags_14 },
    { 0xcfec5025, SUC_lexicon_tags_15 },
    { 0xe9ec5026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6c502a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66c502e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4c5031, SUC_lexicon_tags_21 },
    { 0x1f6c5032, SUC_lexicon_tags_15 },
    { 0xc1cc5031, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfdec5035, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x366c5037, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6c503d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcc5041, SUC_lexicon_tags_57 },
    { 0x414c5042, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20c504c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24c5053, SUC_lexicon_tags_43 },
    { 0xeb2c5053, SUC_lexicon_tags_43 },
    { 0x390c5054, SUC_lexicon_tags_17 },
    { 0x822c5056, SUC_lexicon_tags_37 },
    { 0x6a2c5055, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0c505b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x512c505e, SUC_lexicon_tags_18 },
    { 0x974c505f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xefec5061, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acc5068, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x414c506a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe4ac506c, SUC_lexicon_tags_39 },
    { 0x802c506c, SUC_lexicon_tags_69 },
    { 0xc6c506c, SUC_lexicon_tags_47 },
    { 0xbb0c506c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x656c5071, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0c5074, SUC_lexicon_tags_46 },
    { 0x240c5075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574c507c, SUC_lexicon_tags_0 },
    { 0x7c6c507c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6c5080, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912c5086, SUC_lexicon_tags_17 },
    { 0xc1ac5087, SUC_lexicon_tags_7 },
    { 0x814c5086, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcbac508a, SUC_lexicon_tags_12 },
    { 0xf34c508a, SUC_lexicon_tags_37 },
    { 0x96cc508c, SUC_lexicon_tags_1 },
    { 0xf22c508d, SUC_lexicon_tags_113 },
    { 0x27ec508e, SUC_lexicon_tags_11 },
    { 0xd10c508f, SUC_lexicon_tags_5 },
    { 0x1e4c508d, SUC_lexicon_tags_3 },
    { 0x24c508e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb60c5094, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cc5097, SUC_lexicon_tags_45 },
    { 0xe22c5098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2c509b, SUC_lexicon_tags_7 },
    { 0xd36c509c, SUC_lexicon_tags_12 },
    { 0x9a8c509d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdac50a0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xbf0c50a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904c50a7, SUC_lexicon_tags_66 },
    { 0xecec50a8, SUC_lexicon_tags_15 },
    { 0x9ecc50a8, SUC_lexicon_tags_11 },
    { 0x116c50a7, SUC_lexicon_tags_17 },
    { 0xfacc50a9, SUC_lexicon_tags_25 },
    { 0x510c50ac, SUC_lexicon_tags_17 },
    { 0xd80c50ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0c50b3, SUC_lexicon_tags_7 },
    { 0xdb2c50b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd02c50b6, SUC_lexicon_tags_34 },
    { 0x662c50b7, SUC_lexicon_tags_26 },
    { 0xe1ac50b8, SUC_lexicon_tags_25 },
    { 0x60cc50b6, SUC_lexicon_tags_27 },
    { 0xaa0c50ba, SUC_lexicon_tags_1 },
    { 0xecac50b8, SUC_lexicon_tags_48 },
    { 0xcbac50b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1b2c50be, SUC_lexicon_tags_45 },
    { 0x488c50be, SUC_lexicon_tags_25 },
    { 0xcf2c50bf, SUC_lexicon_tags_2 },
    { 0x760c50c0, SUC_lexicon_tags_2 },
    { 0x3c4c50c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c8c50c4, SUC_lexicon_tags_12 },
    { 0x650c50c5, SUC_lexicon_tags_12 },
    { 0xb1ec50c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccc50ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16c50cc, SUC_lexicon_tags_65 },
    { 0x578c50cd, SUC_lexicon_tags_23 },
    { 0x992c50cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x44ec50d0, SUC_lexicon_tags_37 },
    { 0x5d6c50d1, SUC_lexicon_tags_29 },
    { 0xc60c50d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x34c50d4, SUC_lexicon_tags_33 },
    { 0x37ec50d5, SUC_lexicon_tags_4 },
    { 0x4a6c50d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eec50da, SUC_lexicon_tags_23 },
    { 0x84ac50db, SUC_lexicon_tags_48 },
    { 0x330c50dc, SUC_lexicon_tags_45 },
    { 0x6aac50dd, SUC_lexicon_tags_37 },
    { 0x960c50de, SUC_lexicon_tags_27 },
    { 0x650c50de, SUC_lexicon_tags_29 },
    { 0x656c50e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2c50e3, SUC_lexicon_tags_37 },
    { 0x270c50e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7ac50e6, SUC_lexicon_tags_14 },
    { 0x71ac50e7, SUC_lexicon_tags_38 },
    { 0x9b2c50e6, SUC_lexicon_tags_37 },
    { 0x5eec50e9, SUC_lexicon_tags_26 },
    { 0xb06c50e9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x712c50ed, SUC_lexicon_tags_11 },
    { 0xf1ec50ee, SUC_lexicon_tags_1 },
    { 0xc38c50ed, SUC_lexicon_tags_15 },
    { 0x1f6c50ed, SUC_lexicon_tags_1 },
    { 0x816c50f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2c50fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1dec50fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382c510d, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0xaf4c510f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x41cc5111, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2c5114, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6f6c5116, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0c511a, SUC_lexicon_tags_37 },
    { 0x660c511a, SUC_lexicon_tags_45 },
    { 0x676c511c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcc511f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xadec5121, SUC_lexicon_tags_15 },
    { 0xb52c5121, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1ac5124, SUC_lexicon_tags_21 },
    { 0xcb2c5125, SUC_lexicon_tags_25 },
    { 0x8cac5124, SUC_lexicon_tags_37 },
    { 0x42ec5127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24ac5129, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ac512c, SUC_lexicon_tags_3 },
    { 0xd3ac512c, SUC_lexicon_tags_3 },
    { 0x7ac512e, SUC_lexicon_tags_12 },
    { 0x8b4c512e, SUC_lexicon_tags_25 },
    { 0x4e0c5130, SUC_lexicon_tags_30 },
    { 0x44ec5131, SUC_lexicon_tags_14 },
    { 0x87ac5131, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x708c5135, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ec5138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7ac513a, SUC_lexicon_tags_60 },
    { 0xc18c513b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2c513e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x820c5140, SUC_lexicon_tags_65 },
    { 0xc9ec5141, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08c5146, SUC_lexicon_tags_15 },
    { 0x55cc5147, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc30c5149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6c514e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac5151, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cc5154, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x116c5156, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36c515c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c515f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8c5162, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ac5165, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecc5168, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c516b, SUC_lexicon_tags_17 },
    { 0xbb4c516b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38c516f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c5172, SUC_lexicon_tags_38 },
    { 0x18ec5173, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x132c5176, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd42c5178, SUC_lexicon_tags_7 },
    { 0xc32c5179, SUC_lexicon_tags_1 },
    { 0x7b4c5179, SUC_lexicon_tags_25 },
    { 0xfc4c517b, SUC_lexicon_tags_12 },
    { 0xe06c517c, SUC_lexicon_tags_28 },
    { 0xf58c517d, SUC_lexicon_tags_1 },
    { 0x3a4c517c, SUC_lexicon_tags_10 },
    { 0x22ac5178, SUC_lexicon_tags_39 },
    { 0xd5ec5180, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x950c5182, SUC_lexicon_tags_15 },
    { 0xd04c5183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256c5188, SUC_lexicon_tags_62 },
    { 0x530c5188, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb16c518b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6c518f, SUC_lexicon_tags_30 },
    { 0x26ac5190, SUC_lexicon_tags_43 },
    { 0x106c5191, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaec5194, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x244c5196, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x6c0c5198, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0c519e, SUC_lexicon_tags_7 },
    { 0x442c519e, SUC_lexicon_tags_43 },
    { 0x572c519f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ac51a9, SUC_lexicon_tags_25 },
    { 0xd3ac51a9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf58c51ac, SUC_lexicon_tags_47 },
    { 0x966c51ad, SUC_lexicon_tags_2 },
    { 0xb66c51ad, SUC_lexicon_tags_30 },
    { 0x61ac51af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94c51b3, SUC_lexicon_tags_65 },
    { 0x99ac51b3, SUC_lexicon_tags_7 },
    { 0x2aec51b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff6c51b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x954c51ba, SUC_lexicon_tags_7 },
    { 0xda2c51bb, SUC_lexicon_tags_9 },
    { 0x1a6c51bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cc51c1, SUC_lexicon_tags_47 },
    { 0x678c51c2, SUC_lexicon_tags_7 },
    { 0xabcc51c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcc51c7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4c51cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486c51d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4c51d6, SUC_lexicon_tags_12 },
    { 0x4b0c51d6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x780c51d9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd5ec51db, SUC_lexicon_tags_32 },
    { 0xda6c51db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ac51e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeac51e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490c51e9, SUC_lexicon_tags_37 },
    { 0xc16c51ea, SUC_lexicon_tags_15 },
    { 0xfe4c51eb, SUC_lexicon_tags_22 },
    { 0x4d2c51ec, SUC_lexicon_tags_17 },
    { 0x840c51ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30cc51ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ac51f5, SUC_lexicon_tags_64 },
    { 0x4f4c51f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x310c51f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330c51fe, SUC_lexicon_tags_38 },
    { 0x9c0c51fe, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7dec5201, SUC_lexicon_tags_17 },
    { 0xd5ac5202, SUC_lexicon_tags_37 },
    { 0x9fec5203, SUC_lexicon_tags_19 },
    { 0xc5204, SUC_lexicon_tags_7 },
    { 0xdd6c5204, SUC_lexicon_tags_47 },
    { 0x6f0c5206, SUC_lexicon_tags_63 },
    { 0x216c5202, SUC_lexicon_tags_38 },
    { 0x58ac5208, SUC_lexicon_tags_23 },
    { 0xebec5206, SUC_lexicon_tags_77 },
    { 0xe52c520a, SUC_lexicon_tags_32 },
    { 0xa54c520b, SUC_lexicon_tags_11 },
    { 0x41cc520b, SUC_lexicon_tags_1 },
    { 0xe3ec520d, SUC_lexicon_tags_15 },
    { 0x1c6c520e, SUC_lexicon_tags_11 },
    { 0x622c520e, SUC_lexicon_tags_58 },
    { 0xde4c5202, SUC_lexicon_tags_28 },
    { 0x5dac5206, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cec5218, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac521c, SUC_lexicon_tags_63 },
    { 0xa14c521d, SUC_lexicon_tags_37 },
    { 0x3cc521e, SUC_lexicon_tags_25 },
    { 0x42c521e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ac5226, SUC_lexicon_tags_25 },
    { 0xe54c5227, SUC_lexicon_tags_25 },
    { 0xb58c5228, SUC_lexicon_tags_68 },
    { 0x2eec5229, SUC_lexicon_tags_25 },
    { 0xca8c5228, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ac522d, SUC_lexicon_tags_194 },
    { 0xffac522e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700c5237, SUC_lexicon_tags_14 },
    { 0xaacc5238, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cc523e, SUC_lexicon_tags_47 },
    { 0x506c523e, SUC_lexicon_tags_11 },
    { 0x61ec5240, SUC_lexicon_tags_7 },
    { 0x6cac5241, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0c5246, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ac524b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b0c524d, SUC_lexicon_tags_11 },
    { 0x1b0c524d, SUC_lexicon_tags_0 },
    { 0xc42c524e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadec5256, SUC_lexicon_tags_17 },
    { 0xc8c5256, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390c525e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x230c5260, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6c5267, SUC_lexicon_tags_17 },
    { 0x160c5267, SUC_lexicon_tags_25 },
    { 0xd84c5269, SUC_lexicon_tags_17 },
    { 0x416c5268, SUC_lexicon_tags_11 },
    { 0xe9ac526b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dac526f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298c5274, SUC_lexicon_tags_86 },
    { 0x6c8c5275, SUC_lexicon_tags_0 },
    { 0x486c5276, SUC_lexicon_tags_12 },
    { 0x37ec5276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x848c527a, SUC_lexicon_tags_2 },
    { 0xa28c527a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940c5281, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaec5288, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ec528f, SUC_lexicon_tags_27 },
    { 0x9c2c5290, SUC_lexicon_tags_1 },
    { 0xfc8c5291, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18c5298, SUC_lexicon_tags_14 },
    { 0xecac5299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd06c529b, SUC_lexicon_tags_62 },
    { 0x56ac529c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c52a6, SUC_lexicon_tags_38 },
    { 0x4bac52a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acc52ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630c52b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcec52b6, SUC_lexicon_tags_2 },
    { 0xa12c52b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc68c52b9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7b6c52bb, SUC_lexicon_tags_17 },
    { 0x6dcc52bb, SUC_lexicon_tags_4 },
    { 0x960c52bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cc52c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548c52c6, SUC_lexicon_tags_8 },
    { 0xbfec52c6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0c52cb, SUC_lexicon_tags_39 },
    { 0x238c52cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0c52d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434c52dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x398c52df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecc52e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eec52e7, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfec52ec, SUC_lexicon_tags_12 },
    { 0x824c52ed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188c52f2, SUC_lexicon_tags_12 },
    { 0xe26c52f3, SUC_lexicon_tags_27 },
    { 0xc96c52f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd52c52f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff4c52f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa60c52fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ac52fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x330c5300, SUC_lexicon_tags_66 },
    { 0xa92c5300, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacc5307, SUC_lexicon_tags_72 },
    { 0x148c5308, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x358c530a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38c530d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x13cc5310, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x352c5312, SUC_lexicon_tags_14 },
    { 0x962c5313, SUC_lexicon_tags_27 },
    { 0x37cc5313, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04c5318, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50c531b, SUC_lexicon_tags_37 },
    { 0xb9cc531b, SUC_lexicon_tags_33 },
    { 0x30ec531c, SUC_lexicon_tags_9 },
    { 0x394c531e, SUC_lexicon_tags_11 },
    { 0x2e2c531d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x236c5322, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x190c5325, SUC_lexicon_tags_45 },
    { 0x54cc5326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91ec5328, SUC_lexicon_tags_114 },
    { 0x4c4c5329, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ec532e, SUC_lexicon_tags_6 },
    { 0x5acc532f, SUC_lexicon_tags_23 },
    { 0xb36c5330, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8d0c5332, SUC_lexicon_tags_15 },
    { 0x6f0c5332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6c5337, SUC_lexicon_tags_11 },
    { 0xf40c5338, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8c533b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafec533f, SUC_lexicon_tags_45 },
    { 0x920c5340, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe74c5342, SUC_lexicon_tags_1 },
    { 0xb74c5343, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8c5347, SUC_lexicon_tags_17 },
    { 0xbaac5347, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8c534d, SUC_lexicon_tags_31 },
    { 0xb54c534d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aec5352, SUC_lexicon_tags_569 },
    { 0, NULL },
    { 0x8dcc5354, SUC_lexicon_tags_11 },
    { 0xf8cc5355, SUC_lexicon_tags_37 },
    { 0xb1ec5356, SUC_lexicon_tags_37 },
    { 0xd80c5355, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x782c5359, SUC_lexicon_tags_11 },
    { 0xcc8c535a, SUC_lexicon_tags_7 },
    { 0xb0cc535b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2c5362, SUC_lexicon_tags_15 },
    { 0xcac5363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06c5367, SUC_lexicon_tags_66 },
    { 0x596c5368, SUC_lexicon_tags_17 },
    { 0xe8ec5368, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32c536e, SUC_lexicon_tags_17 },
    { 0x4dac536f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x26c5371, SUC_lexicon_tags_14 },
    { 0x2bac5372, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ac5377, SUC_lexicon_tags_42 },
    { 0xb78c5378, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x118c537b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eac537f, SUC_lexicon_tags_19 },
    { 0xc06c5380, SUC_lexicon_tags_12 },
    { 0xb0ac537f, SUC_lexicon_tags_11 },
    { 0x224c5382, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38c5385, SUC_lexicon_tags_852 },
    { 0, NULL },
    { 0, NULL },
    { 0x406c5388, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x702c538b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf18c538d, SUC_lexicon_tags_12 },
    { 0x2e0c538d, SUC_lexicon_tags_25 },
    { 0xeb8c538f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ac5394, SUC_lexicon_tags_45 },
    { 0xc76c5394, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cc5399, SUC_lexicon_tags_37 },
    { 0x89ac5399, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d2c539c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6c539f, SUC_lexicon_tags_87 },
    { 0xc58c539f, SUC_lexicon_tags_1 },
    { 0xef4c53a1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cc53a4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ac53a9, SUC_lexicon_tags_25 },
    { 0x3acc53aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36c53ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1cc53af, SUC_lexicon_tags_7 },
    { 0x23ec53b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a6c53b2, SUC_lexicon_tags_37 },
    { 0xdd0c53b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32c53b7, SUC_lexicon_tags_1 },
    { 0x99ec53b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8c53bb, SUC_lexicon_tags_47 },
    { 0xc00c53bb, SUC_lexicon_tags_2 },
    { 0xf64c53bd, SUC_lexicon_tags_17 },
    { 0xed8c53bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x600c53c0, SUC_lexicon_tags_11 },
    { 0x10c53c1, SUC_lexicon_tags_2 },
    { 0x3b8c53c0, SUC_lexicon_tags_7 },
    { 0xb54c53c3, SUC_lexicon_tags_7 },
    { 0x282c53c4, SUC_lexicon_tags_55 },
    { 0xb70c53c3, SUC_lexicon_tags_6 },
    { 0x9d8c53c5, SUC_lexicon_tags_66 },
    { 0x72ec53c7, SUC_lexicon_tags_7 },
    { 0xeecc53c8, SUC_lexicon_tags_7 },
    { 0xf78c53c9, SUC_lexicon_tags_30 },
    { 0xc9ec53ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82c53cd, SUC_lexicon_tags_30 },
    { 0x27ec53ce, SUC_lexicon_tags_28 },
    { 0x7dec53cf, SUC_lexicon_tags_17 },
    { 0xfac53d0, SUC_lexicon_tags_65 },
    { 0x408c53cf, SUC_lexicon_tags_47 },
    { 0xa8cc53cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8c53d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2c53da, SUC_lexicon_tags_28 },
    { 0x428c53db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c53e0, SUC_lexicon_tags_37 },
    { 0xa90c53e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2c53e4, SUC_lexicon_tags_45 },
    { 0x4fcc53e5, SUC_lexicon_tags_6 },
    { 0x82ec53e6, SUC_lexicon_tags_10 },
    { 0x29ac53e6, SUC_lexicon_tags_2 },
    { 0xdecc53e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cc53eb, SUC_lexicon_tags_25 },
    { 0x48c53ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13ec53ee, SUC_lexicon_tags_33 },
    { 0x63ac53ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cc53f2, SUC_lexicon_tags_11 },
    { 0xe76c53f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cc53f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92c53fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x996c53fe, SUC_lexicon_tags_11 },
    { 0xf0ec53fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd54c5401, SUC_lexicon_tags_11 },
    { 0x6eac5402, SUC_lexicon_tags_23 },
    { 0x76ec5402, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe64c5405, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6c5409, SUC_lexicon_tags_37 },
    { 0xb90c540a, SUC_lexicon_tags_43 },
    { 0xad8c540b, SUC_lexicon_tags_31 },
    { 0xa3ac540b, SUC_lexicon_tags_37 },
    { 0xc72c540d, SUC_lexicon_tags_7 },
    { 0x8d6c540e, SUC_lexicon_tags_7 },
    { 0x1b0c540f, SUC_lexicon_tags_47 },
    { 0x8b0c5410, SUC_lexicon_tags_68 },
    { 0x16c540f, SUC_lexicon_tags_46 },
    { 0x9c2c5412, SUC_lexicon_tags_12 },
    { 0xde2c5411, SUC_lexicon_tags_27 },
    { 0x8eac5410, SUC_lexicon_tags_66 },
    { 0xb6ec5415, SUC_lexicon_tags_37 },
    { 0x1b6c540a, SUC_lexicon_tags_7 },
    { 0xd1ac5413, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16c541b, SUC_lexicon_tags_156 },
    { 0x80ac541b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52c5421, SUC_lexicon_tags_27 },
    { 0x4aec5421, SUC_lexicon_tags_33 },
    { 0xf6c5423, SUC_lexicon_tags_2 },
    { 0x7b8c5423, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4c5427, SUC_lexicon_tags_7 },
    { 0xc62c5427, SUC_lexicon_tags_31 },
    { 0x6ec5428, SUC_lexicon_tags_1 },
    { 0x800c542a, SUC_lexicon_tags_7 },
    { 0xdd6c542b, SUC_lexicon_tags_33 },
    { 0xad4c542c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40c5430, SUC_lexicon_tags_1 },
    { 0xf92c5431, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0c5434, SUC_lexicon_tags_45 },
    { 0xfbac5435, SUC_lexicon_tags_15 },
    { 0x6ac5435, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc4ec5438, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8c543d, SUC_lexicon_tags_43 },
    { 0x7a8c543e, SUC_lexicon_tags_84 },
    { 0x6a2c543f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cc5442, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86c544b, SUC_lexicon_tags_63 },
    { 0x134c544c, SUC_lexicon_tags_11 },
    { 0xeecc544c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64c5450, SUC_lexicon_tags_7 },
    { 0x4d0c5451, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc30c5453, SUC_lexicon_tags_14 },
    { 0x97cc5453, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xafac5456, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac545a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144c545e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x210c5461, SUC_lexicon_tags_17 },
    { 0x61ac5462, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x53ac5464, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cc5467, SUC_lexicon_tags_17 },
    { 0x24ec5468, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2c546c, SUC_lexicon_tags_17 },
    { 0x4e6c546d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x568c5470, SUC_lexicon_tags_6 },
    { 0x89ac5471, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x124c5474, SUC_lexicon_tags_8 },
    { 0xfa4c5475, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa8c5477, SUC_lexicon_tags_1362 },
    { 0, NULL },
    { 0x468c5479, SUC_lexicon_tags_14 },
    { 0x986c5479, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30c547e, SUC_lexicon_tags_6 },
    { 0xa2cc547f, SUC_lexicon_tags_65 },
    { 0x81ec5480, SUC_lexicon_tags_8 },
    { 0x366c5481, SUC_lexicon_tags_2 },
    { 0xeaac547f, SUC_lexicon_tags_35 },
    { 0x18ac5483, SUC_lexicon_tags_11 },
    { 0xbf8c5484, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf86c5486, SUC_lexicon_tags_19 },
    { 0x412c5487, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ac548d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc5490, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2ac5492, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x710c5495, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0c5498, SUC_lexicon_tags_63 },
    { 0x48ec5499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12c54a4, SUC_lexicon_tags_11 },
    { 0x684c54a5, SUC_lexicon_tags_15 },
    { 0xad6c54a6, SUC_lexicon_tags_18 },
    { 0x6b2c54a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfec54a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x996c54ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x708c54ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156c54b2, SUC_lexicon_tags_12 },
    { 0x53cc54b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8c54b6, SUC_lexicon_tags_42 },
    { 0x704c54b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4c54bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x69cc54be, SUC_lexicon_tags_2 },
    { 0x150c54bf, SUC_lexicon_tags_14 },
    { 0x2b8c54c0, SUC_lexicon_tags_15 },
    { 0x9eec54c1, SUC_lexicon_tags_11 },
    { 0xbc8c54c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516c54c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250c54cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf4c54cf, SUC_lexicon_tags_15 },
    { 0xabac54d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ac54d4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x958c54d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40c54da, SUC_lexicon_tags_28 },
    { 0x2eec54da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc84c54dd, SUC_lexicon_tags_35 },
    { 0xb1ec54dd, SUC_lexicon_tags_49 },
    { 0x330c54de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aec54e6, SUC_lexicon_tags_12 },
    { 0x314c54e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ac54ea, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa7ac54ec, SUC_lexicon_tags_11 },
    { 0x5f8c54ec, SUC_lexicon_tags_10 },
    { 0xbfac54ee, SUC_lexicon_tags_47 },
    { 0x536c54ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc2c54f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0ac54f3, SUC_lexicon_tags_11 },
    { 0x25ac54f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22c54f7, SUC_lexicon_tags_17 },
    { 0xc7ac54f8, SUC_lexicon_tags_95 },
    { 0xc58c54f9, SUC_lexicon_tags_70 },
    { 0x992c54f7, SUC_lexicon_tags_25 },
    { 0x916c54fa, SUC_lexicon_tags_29 },
    { 0x520c54fa, SUC_lexicon_tags_54 },
    { 0x700c54fd, SUC_lexicon_tags_25 },
    { 0x3eec54fb, SUC_lexicon_tags_47 },
    { 0x7cec54fd, SUC_lexicon_tags_27 },
    { 0x204c5500, SUC_lexicon_tags_7 },
    { 0x6a4c5500, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x58c5503, SUC_lexicon_tags_14 },
    { 0x7f0c5503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46cc5507, SUC_lexicon_tags_60 },
    { 0x474c5508, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302c550c, SUC_lexicon_tags_17 },
    { 0xb1ec550c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38c5510, SUC_lexicon_tags_43 },
    { 0x1a0c5511, SUC_lexicon_tags_17 },
    { 0xe8c5510, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfec5515, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfec5517, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x964c5519, SUC_lexicon_tags_45 },
    { 0x91ec551a, SUC_lexicon_tags_15 },
    { 0x5b2c551b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e6c551d, SUC_lexicon_tags_63 },
    { 0xbb4c551e, SUC_lexicon_tags_12 },
    { 0xfa0c551f, SUC_lexicon_tags_1 },
    { 0x61cc5520, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dec5524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6c5529, SUC_lexicon_tags_27 },
    { 0x9a0c5529, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0c552e, SUC_lexicon_tags_23 },
    { 0x8c0c552f, SUC_lexicon_tags_17 },
    { 0xa2c552e, SUC_lexicon_tags_17 },
    { 0xf0cc5530, SUC_lexicon_tags_138 },
    { 0x7d8c5532, SUC_lexicon_tags_27 },
    { 0x41ec552f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec553a, SUC_lexicon_tags_27 },
    { 0x928c553a, SUC_lexicon_tags_10 },
    { 0x56ec553c, SUC_lexicon_tags_8 },
    { 0xd1cc553c, SUC_lexicon_tags_10 },
    { 0xc40c553b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66c5541, SUC_lexicon_tags_41 },
    { 0x49cc5541, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ec5545, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaac5547, SUC_lexicon_tags_15 },
    { 0x496c5548, SUC_lexicon_tags_37 },
    { 0xb9cc5549, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a0c554b, SUC_lexicon_tags_6 },
    { 0xa9ec554c, SUC_lexicon_tags_22 },
    { 0x4c554d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc554f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbc4c5551, SUC_lexicon_tags_27 },
    { 0x5eac5551, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0c5556, SUC_lexicon_tags_72 },
    { 0xe16c5556, SUC_lexicon_tags_26 },
    { 0x80ec5558, SUC_lexicon_tags_37 },
    { 0x8c6c5559, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998c555d, SUC_lexicon_tags_17 },
    { 0x3d4c555d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9ec5560, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cec5564, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30c5569, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x864c556b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ec5571, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2c5578, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdecc557a, SUC_lexicon_tags_11 },
    { 0xae6c557a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x628c557d, SUC_lexicon_tags_11 },
    { 0x96c557e, SUC_lexicon_tags_1 },
    { 0x692c557e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2c5583, SUC_lexicon_tags_39 },
    { 0x144c5583, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dac558a, SUC_lexicon_tags_14 },
    { 0x8cc558b, SUC_lexicon_tags_12 },
    { 0x70cc558a, SUC_lexicon_tags_2 },
    { 0xbbec558a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd82c558f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x704c5591, SUC_lexicon_tags_17 },
    { 0x4fec5592, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ccc5594, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98c5599, SUC_lexicon_tags_9 },
    { 0xb9cc5599, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70c559d, SUC_lexicon_tags_17 },
    { 0xee4c559e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86c55a1, SUC_lexicon_tags_9 },
    { 0x870c55a2, SUC_lexicon_tags_33 },
    { 0x832c55a3, SUC_lexicon_tags_17 },
    { 0xc6ec55a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ec55a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cec55ac, SUC_lexicon_tags_1 },
    { 0x7a6c55ac, SUC_lexicon_tags_17 },
    { 0x934c55ae, SUC_lexicon_tags_25 },
    { 0x5aec55af, SUC_lexicon_tags_15 },
    { 0x6fac55ac, SUC_lexicon_tags_25 },
    { 0xf50c55b1, SUC_lexicon_tags_7 },
    { 0x6e6c55b2, SUC_lexicon_tags_12 },
    { 0x192c55ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4c55b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c0c55ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x496c55bc, SUC_lexicon_tags_47 },
    { 0x4d2c55bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedac55c1, SUC_lexicon_tags_7 },
    { 0xf8ec55c2, SUC_lexicon_tags_38 },
    { 0xb30c55c3, SUC_lexicon_tags_10 },
    { 0x8a0c55c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaeac55c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ac55cb, SUC_lexicon_tags_26 },
    { 0x26ac55cc, SUC_lexicon_tags_12 },
    { 0xa52c55cd, SUC_lexicon_tags_59 },
    { 0xe38c55cd, SUC_lexicon_tags_17 },
    { 0xcc4c55cf, SUC_lexicon_tags_9 },
    { 0x57cc55d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc55d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdec55d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xefac55db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x312c55dd, SUC_lexicon_tags_60 },
    { 0x9f8c55dd, SUC_lexicon_tags_25 },
    { 0xdaac55df, SUC_lexicon_tags_66 },
    { 0x21ac55df, SUC_lexicon_tags_55 },
    { 0xaac55dd, SUC_lexicon_tags_25 },
    { 0xdcc55e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc24c55e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76c55e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2c55eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92c55f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580c55f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88c55f9, SUC_lexicon_tags_7 },
    { 0x3c2c55f9, SUC_lexicon_tags_25 },
    { 0x6e4c55f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x914c55fd, SUC_lexicon_tags_37 },
    { 0x5f8c55fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0c5605, SUC_lexicon_tags_11 },
    { 0x5e6c5605, SUC_lexicon_tags_43 },
    { 0x1f8c5607, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x518c5609, SUC_lexicon_tags_14 },
    { 0x742c5609, SUC_lexicon_tags_0 },
    { 0x362c560b, SUC_lexicon_tags_49 },
    { 0xd6c560c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4c5610, SUC_lexicon_tags_47 },
    { 0x60cc5611, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7ac5613, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x360c5615, SUC_lexicon_tags_25 },
    { 0x8cec5615, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b0c5618, SUC_lexicon_tags_30 },
    { 0x2e4c5618, SUC_lexicon_tags_2 },
    { 0xca2c561a, SUC_lexicon_tags_45 },
    { 0x8f0c561b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6c561e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2c5621, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb56c5623, SUC_lexicon_tags_14 },
    { 0xf44c5624, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cc562a, SUC_lexicon_tags_60 },
    { 0x324c562a, SUC_lexicon_tags_37 },
    { 0x8d4c562c, SUC_lexicon_tags_2 },
    { 0x374c562d, SUC_lexicon_tags_6 },
    { 0x946c562c, SUC_lexicon_tags_7 },
    { 0x25ec562f, SUC_lexicon_tags_37 },
    { 0x460c562f, SUC_lexicon_tags_12 },
    { 0x106c5631, SUC_lexicon_tags_4 },
    { 0x7c8c562d, SUC_lexicon_tags_17 },
    { 0x744c5633, SUC_lexicon_tags_47 },
    { 0x39ac5634, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8cc5636, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a0c5638, SUC_lexicon_tags_125 },
    { 0xa1cc5639, SUC_lexicon_tags_7 },
    { 0xb72c563a, SUC_lexicon_tags_12 },
    { 0x602c563b, SUC_lexicon_tags_7 },
    { 0xecec563a, SUC_lexicon_tags_4 },
    { 0x422c563d, SUC_lexicon_tags_43 },
    { 0xb34c5639, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe06c5640, SUC_lexicon_tags_56 },
    { 0xb4ec5641, SUC_lexicon_tags_23 },
    { 0xb72c5641, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cec5646, SUC_lexicon_tags_38 },
    { 0x150c5647, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x640c5649, SUC_lexicon_tags_60 },
    { 0x114c564a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x93ec564c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eac5657, SUC_lexicon_tags_65 },
    { 0x6d8c5657, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x74ac565a, SUC_lexicon_tags_39 },
    { 0x262c565b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd72c565d, SUC_lexicon_tags_14 },
    { 0x4acc565d, SUC_lexicon_tags_25 },
    { 0xb24c565f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ccc5661, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860c5665, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cc566c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x222c566f, SUC_lexicon_tags_2 },
    { 0xd8ec5670, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eec5677, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52c567d, SUC_lexicon_tags_46 },
    { 0xbfec567d, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0x3cc5680, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33ac5682, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe42c5684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9ac5686, SUC_lexicon_tags_9 },
    { 0x2f4c5686, SUC_lexicon_tags_37 },
    { 0xaccc5687, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cc568b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96c5694, SUC_lexicon_tags_33 },
    { 0x44c5695, SUC_lexicon_tags_23 },
    { 0x9f0c5696, SUC_lexicon_tags_2 },
    { 0xe86c5697, SUC_lexicon_tags_6 },
    { 0xf8ac5696, SUC_lexicon_tags_53 },
    { 0x79ac5697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acc569c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x522c569e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c56a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41ec56a4, SUC_lexicon_tags_77 },
    { 0x860c56a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x218c56a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb54c56aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cc56ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8ec56af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6c56b3, SUC_lexicon_tags_37 },
    { 0xd14c56b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59ac56b6, SUC_lexicon_tags_2 },
    { 0x9ecc56b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8c56bf, SUC_lexicon_tags_15 },
    { 0x8c0c56c0, SUC_lexicon_tags_17 },
    { 0x196c56c1, SUC_lexicon_tags_26 },
    { 0xb70c56c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76c56c6, SUC_lexicon_tags_2 },
    { 0x992c56c7, SUC_lexicon_tags_8 },
    { 0xca6c56c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dec56ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd2c56cc, SUC_lexicon_tags_19 },
    { 0x4cc56cd, SUC_lexicon_tags_48 },
    { 0xa16c56ce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38c56d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c56d9, SUC_lexicon_tags_11 },
    { 0x3ecc56d9, SUC_lexicon_tags_69 },
    { 0x3e4c56da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81cc56dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4c56e7, SUC_lexicon_tags_33 },
    { 0x936c56e8, SUC_lexicon_tags_15 },
    { 0x1a8c56e9, SUC_lexicon_tags_87 },
    { 0xd68c56e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ec56ed, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60c56f2, SUC_lexicon_tags_21 },
    { 0x970c56f2, SUC_lexicon_tags_17 },
    { 0xddec56f3, SUC_lexicon_tags_45 },
    { 0xbe4c56f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cc56fd, SUC_lexicon_tags_11 },
    { 0xbccc56fd, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x40cc5700, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6ac5702, SUC_lexicon_tags_27 },
    { 0xca6c5702, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428c5707, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xabec5709, SUC_lexicon_tags_27 },
    { 0xdb0c570a, SUC_lexicon_tags_63 },
    { 0x9b6c570b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa3ac570d, SUC_lexicon_tags_181 },
    { 0x9e0c570e, SUC_lexicon_tags_14 },
    { 0x898c570d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4c5719, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x108c571c, SUC_lexicon_tags_7 },
    { 0x294c571d, SUC_lexicon_tags_25 },
    { 0xd4ac571e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4c572c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9fac572e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46c5732, SUC_lexicon_tags_65 },
    { 0x1ec5733, SUC_lexicon_tags_49 },
    { 0xa48c5733, SUC_lexicon_tags_29 },
    { 0xc88c5735, SUC_lexicon_tags_25 },
    { 0xbc6c5736, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x698c5739, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916c573d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0c5740, SUC_lexicon_tags_38 },
    { 0x8e2c5741, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ac574b, SUC_lexicon_tags_1 },
    { 0x89cc574b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00c5750, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ac5753, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x82cc5755, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bec5758, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x990c575a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96c575d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7ac575f, SUC_lexicon_tags_37 },
    { 0xc4c5760, SUC_lexicon_tags_12 },
    { 0xe6ac5760, SUC_lexicon_tags_27 },
    { 0xb1ac5761, SUC_lexicon_tags_45 },
    { 0xb26c5763, SUC_lexicon_tags_38 },
    { 0x756c5764, SUC_lexicon_tags_37 },
    { 0x786c5761, SUC_lexicon_tags_15 },
    { 0x480c5766, SUC_lexicon_tags_25 },
    { 0xe2ec5761, SUC_lexicon_tags_0 },
    { 0xd26c5765, SUC_lexicon_tags_1 },
    { 0xa20c5769, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd54c576b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf00c576d, SUC_lexicon_tags_9 },
    { 0x906c576d, SUC_lexicon_tags_27 },
    { 0x6c0c576f, SUC_lexicon_tags_34 },
    { 0x15ac576e, SUC_lexicon_tags_49 },
    { 0xe14c5771, SUC_lexicon_tags_7 },
    { 0x3fec5771, SUC_lexicon_tags_5 },
    { 0xefac5773, SUC_lexicon_tags_29 },
    { 0xabac5774, SUC_lexicon_tags_70 },
    { 0xacac5774, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11cc5777, SUC_lexicon_tags_79 },
    { 0x18ac5778, SUC_lexicon_tags_37 },
    { 0x176c5779, SUC_lexicon_tags_27 },
    { 0x48cc5779, SUC_lexicon_tags_7 },
    { 0x170c577b, SUC_lexicon_tags_39 },
    { 0xcdcc577b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc577f, SUC_lexicon_tags_4 },
    { 0xca4c577f, SUC_lexicon_tags_11 },
    { 0xbe2c5780, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10c578a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefec5792, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58c579e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45ec57a0, SUC_lexicon_tags_31 },
    { 0x53ac57a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310c57a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf6c57a7, SUC_lexicon_tags_29 },
    { 0x9cac57a8, SUC_lexicon_tags_33 },
    { 0x48ac57a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x83ec57ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4c57af, SUC_lexicon_tags_1 },
    { 0x358c57b0, SUC_lexicon_tags_12 },
    { 0x5e2c57af, SUC_lexicon_tags_27 },
    { 0x316c57b2, SUC_lexicon_tags_56 },
    { 0x1a6c57b3, SUC_lexicon_tags_24 },
    { 0xb16c57b4, SUC_lexicon_tags_11 },
    { 0x51ec57b5, SUC_lexicon_tags_15 },
    { 0xf24c57b4, SUC_lexicon_tags_30 },
    { 0xd5ec57b7, SUC_lexicon_tags_47 },
    { 0x280c57b7, SUC_lexicon_tags_1 },
    { 0x466c57b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6c57bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cc57c1, SUC_lexicon_tags_21 },
    { 0x42ac57c2, SUC_lexicon_tags_48 },
    { 0x80cc57c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ec57c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606c57cd, SUC_lexicon_tags_19 },
    { 0xee8c57ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4c57d1, SUC_lexicon_tags_77 },
    { 0x4cec57d2, SUC_lexicon_tags_30 },
    { 0xd02c57d3, SUC_lexicon_tags_15 },
    { 0x4dec57d4, SUC_lexicon_tags_46 },
    { 0x450c57d5, SUC_lexicon_tags_1 },
    { 0x832c57d4, SUC_lexicon_tags_79 },
    { 0x92ec57d5, SUC_lexicon_tags_10 },
    { 0xadec57d8, SUC_lexicon_tags_1 },
    { 0xaa0c57d1, SUC_lexicon_tags_27 },
    { 0x7f6c57da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4ec57dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08c57e2, SUC_lexicon_tags_15 },
    { 0x1d0c57e3, SUC_lexicon_tags_12 },
    { 0x4e2c57e3, SUC_lexicon_tags_11 },
    { 0x67ac57e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec57e8, SUC_lexicon_tags_63 },
    { 0xc52c57e9, SUC_lexicon_tags_17 },
    { 0x58ec57e8, SUC_lexicon_tags_418 },
    { 0x9c0c57eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x232c57ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb8c57ef, SUC_lexicon_tags_15 },
    { 0x468c57ef, SUC_lexicon_tags_23 },
    { 0xffec57f0, SUC_lexicon_tags_77 },
    { 0xb8c57f2, SUC_lexicon_tags_17 },
    { 0xb6cc57f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ac57f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13cc57f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e2c57fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63ac57fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20c5801, SUC_lexicon_tags_6 },
    { 0x884c5802, SUC_lexicon_tags_7 },
    { 0x3c2c5803, SUC_lexicon_tags_32 },
    { 0xa00c5804, SUC_lexicon_tags_0 },
    { 0xf66c5805, SUC_lexicon_tags_14 },
    { 0x7dcc5806, SUC_lexicon_tags_27 },
    { 0x660c5806, SUC_lexicon_tags_9 },
    { 0xdc4c5806, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9dec580a, SUC_lexicon_tags_27 },
    { 0xb26c580a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64c5811, SUC_lexicon_tags_1 },
    { 0xd3cc5811, SUC_lexicon_tags_26 },
    { 0x636c5811, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecec5817, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14c581b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c5822, SUC_lexicon_tags_55 },
    { 0xbf2c5823, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780c5829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264c5830, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238c5834, SUC_lexicon_tags_16 },
    { 0x8e6c5835, SUC_lexicon_tags_12 },
    { 0xb8ec5835, SUC_lexicon_tags_2 },
    { 0xa8ac5837, SUC_lexicon_tags_28 },
    { 0xb12c5838, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x460c583b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x850c583d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e2c583f, SUC_lexicon_tags_7 },
    { 0xfa4c583f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9cc5842, SUC_lexicon_tags_21 },
    { 0xe76c5843, SUC_lexicon_tags_37 },
    { 0x4a2c5844, SUC_lexicon_tags_12 },
    { 0xceec5845, SUC_lexicon_tags_23 },
    { 0xda2c5846, SUC_lexicon_tags_19 },
    { 0x9d2c5842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626c584e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe72c5851, SUC_lexicon_tags_25 },
    { 0xc3ac5852, SUC_lexicon_tags_87 },
    { 0x668c5853, SUC_lexicon_tags_7 },
    { 0x8e6c5854, SUC_lexicon_tags_82 },
    { 0xc44c5852, SUC_lexicon_tags_79 },
    { 0x31ac5851, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xecec5858, SUC_lexicon_tags_0 },
    { 0x10cc5859, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8c585c, SUC_lexicon_tags_27 },
    { 0x49ac585d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x364c5860, SUC_lexicon_tags_0 },
    { 0xa90c5860, SUC_lexicon_tags_9 },
    { 0xe9cc5860, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6c5867, SUC_lexicon_tags_19 },
    { 0x866c5867, SUC_lexicon_tags_17 },
    { 0x39ec5869, SUC_lexicon_tags_45 },
    { 0x6aec586a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecc5870, SUC_lexicon_tags_15 },
    { 0xe72c5871, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdeac5873, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4c5877, SUC_lexicon_tags_47 },
    { 0x1f4c5877, SUC_lexicon_tags_9 },
    { 0xc58c5879, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82c5880, SUC_lexicon_tags_17 },
    { 0xb2c5880, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x61cc5883, SUC_lexicon_tags_11 },
    { 0x1ac5884, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3e2c5886, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cc588d, SUC_lexicon_tags_11 },
    { 0xe66c588e, SUC_lexicon_tags_1 },
    { 0x18ac588d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2bac5891, SUC_lexicon_tags_11 },
    { 0xc9ec5892, SUC_lexicon_tags_25 },
    { 0xa4cc5891, SUC_lexicon_tags_15 },
    { 0xceac5894, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3ac5896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dec589a, SUC_lexicon_tags_8 },
    { 0xad8c589a, SUC_lexicon_tags_12 },
    { 0x804c589a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1ec589e, SUC_lexicon_tags_60 },
    { 0x102c589e, SUC_lexicon_tags_30 },
    { 0xcfec589e, SUC_lexicon_tags_1 },
    { 0x1cac58a1, SUC_lexicon_tags_47 },
    { 0x7dac58a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bec58a6, SUC_lexicon_tags_61 },
    { 0x298c58a7, SUC_lexicon_tags_15 },
    { 0x8bac58a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa02c58aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1ccc58ac, SUC_lexicon_tags_49 },
    { 0x68cc58ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6fec58af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x670c58b1, SUC_lexicon_tags_25 },
    { 0xcac58b2, SUC_lexicon_tags_25 },
    { 0x844c58b3, SUC_lexicon_tags_17 },
    { 0x78ac58b4, SUC_lexicon_tags_402 },
    { 0x360c58b4, SUC_lexicon_tags_33 },
    { 0x1d4c58b3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476c58bd, SUC_lexicon_tags_6 },
    { 0xbe6c58bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ac58c3, SUC_lexicon_tags_15 },
    { 0xe6ac58c4, SUC_lexicon_tags_108 },
    { 0x38cc58c3, SUC_lexicon_tags_55 },
    { 0x4b6c58c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68c58cd, SUC_lexicon_tags_30 },
    { 0xb0c58cd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ec58d1, SUC_lexicon_tags_15 },
    { 0x70ec58d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00c58d6, SUC_lexicon_tags_37 },
    { 0x110c58d7, SUC_lexicon_tags_26 },
    { 0x8ac58d8, SUC_lexicon_tags_10 },
    { 0x664c58d8, SUC_lexicon_tags_27 },
    { 0x202c58d9, SUC_lexicon_tags_79 },
    { 0xfc4c58db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10c58de, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20c58e2, SUC_lexicon_tags_6 },
    { 0x658c58e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x992c58e5, SUC_lexicon_tags_15 },
    { 0xe4ec58e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814c58ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa1cc58ec, SUC_lexicon_tags_37 },
    { 0xe68c58ed, SUC_lexicon_tags_17 },
    { 0x1a8c58ee, SUC_lexicon_tags_14 },
    { 0xbdcc58ee, SUC_lexicon_tags_158 },
    { 0xc7ec58f0, SUC_lexicon_tags_174 },
    { 0xd7ac58ef, SUC_lexicon_tags_8 },
    { 0xec2c58f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x74ec58f4, SUC_lexicon_tags_46 },
    { 0xaccc58f4, SUC_lexicon_tags_28 },
    { 0x41ec58f4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x96cc58f8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6a2c58fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe34c58fc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x534c58ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x94ac5901, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc4c5903, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26c5908, SUC_lexicon_tags_25 },
    { 0xcf2c5909, SUC_lexicon_tags_14 },
    { 0x35cc590a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd7ec590c, SUC_lexicon_tags_19 },
    { 0x600c590d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x598c5910, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840c5915, SUC_lexicon_tags_15 },
    { 0x912c5916, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cc5919, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x482c591b, SUC_lexicon_tags_30 },
    { 0x792c591b, SUC_lexicon_tags_47 },
    { 0xca4c591d, SUC_lexicon_tags_19 },
    { 0x6bcc591e, SUC_lexicon_tags_8 },
    { 0xcfac591f, SUC_lexicon_tags_17 },
    { 0x74ec591f, SUC_lexicon_tags_25 },
    { 0x46cc5921, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6c5924, SUC_lexicon_tags_17 },
    { 0x3f2c5925, SUC_lexicon_tags_17 },
    { 0x462c5925, SUC_lexicon_tags_25 },
    { 0xa06c5927, SUC_lexicon_tags_37 },
    { 0x850c5924, SUC_lexicon_tags_82 },
    { 0xdb6c5926, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x192c592c, SUC_lexicon_tags_37 },
    { 0x212c592d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ec5930, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9ac5932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eac5937, SUC_lexicon_tags_2 },
    { 0x35ec5937, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ac593b, SUC_lexicon_tags_31 },
    { 0x4bcc593c, SUC_lexicon_tags_37 },
    { 0x3dcc593d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954c5946, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856c5950, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x128c5952, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x748c5955, SUC_lexicon_tags_45 },
    { 0x8e4c5956, SUC_lexicon_tags_15 },
    { 0x684c5955, SUC_lexicon_tags_8 },
    { 0x2f4c5958, SUC_lexicon_tags_4 },
    { 0xe6ac5959, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0c595e, SUC_lexicon_tags_8 },
    { 0xe00c595e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eec5967, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x404c5969, SUC_lexicon_tags_33 },
    { 0x4f6c596a, SUC_lexicon_tags_33 },
    { 0x7a0c596a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4c596e, SUC_lexicon_tags_30 },
    { 0xfcec596e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872c5973, SUC_lexicon_tags_2 },
    { 0x354c5973, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad6c5976, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x356c5978, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44c597b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ac597f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa4ec5981, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64c5984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4c5989, SUC_lexicon_tags_11 },
    { 0x822c598a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3bac598c, SUC_lexicon_tags_1 },
    { 0x4d8c598d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfd2c598f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x606c5991, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10ac5993, SUC_lexicon_tags_0 },
    { 0xab0c5994, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44c5998, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebec599a, SUC_lexicon_tags_17 },
    { 0x1a2c599b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ec599f, SUC_lexicon_tags_37 },
    { 0x9f4c59a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eec59a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432c59a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cc59ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c59b1, SUC_lexicon_tags_12 },
    { 0x2acc59b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48c59b5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2c59b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2c59c0, SUC_lexicon_tags_6 },
    { 0xce4c59c0, SUC_lexicon_tags_17 },
    { 0x5d6c59c1, SUC_lexicon_tags_17 },
    { 0xa5cc59c2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cc59ce, SUC_lexicon_tags_60 },
    { 0x802c59ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40c59d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x290c59d6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x602c59d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06c59e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x250c59e6, SUC_lexicon_tags_23 },
    { 0x6e8c59e7, SUC_lexicon_tags_7 },
    { 0x8aec59e8, SUC_lexicon_tags_42 },
    { 0x426c59e9, SUC_lexicon_tags_38 },
    { 0xde6c59ea, SUC_lexicon_tags_12 },
    { 0x9d4c59e9, SUC_lexicon_tags_47 },
    { 0xb66c59ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x72cc59ef, SUC_lexicon_tags_63 },
    { 0x77ec59f0, SUC_lexicon_tags_101 },
    { 0x94c59f0, SUC_lexicon_tags_66 },
    { 0xe32c59f2, SUC_lexicon_tags_19 },
    { 0xf84c59f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b6c59f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcc59f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ac59fb, SUC_lexicon_tags_27 },
    { 0x3ec59fb, SUC_lexicon_tags_138 },
    { 0x72c59fc, SUC_lexicon_tags_10 },
    { 0x892c59fe, SUC_lexicon_tags_26 },
    { 0x98ec59fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cc5a03, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ac5a09, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ac5a0e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ac5a11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6c5a15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa08c5a17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6c5a1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6c5a1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524c5a23, SUC_lexicon_tags_7 },
    { 0x2aec5a24, SUC_lexicon_tags_26 },
    { 0xf92c5a25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8c5a29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908c5a2d, SUC_lexicon_tags_17 },
    { 0xa92c5a2e, SUC_lexicon_tags_38 },
    { 0xb7cc5a2d, SUC_lexicon_tags_12 },
    { 0x7d0c5a30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53ac5a32, SUC_lexicon_tags_23 },
    { 0x98c5a32, SUC_lexicon_tags_11 },
    { 0xacc5a34, SUC_lexicon_tags_47 },
    { 0x1e2c5a34, SUC_lexicon_tags_87 },
    { 0x396c5a34, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0c5a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6c5a3e, SUC_lexicon_tags_47 },
    { 0x2a4c5a3f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ac5a44, SUC_lexicon_tags_17 },
    { 0xae4c5a45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x804c5a48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aec5a4b, SUC_lexicon_tags_38 },
    { 0x3acc5a4c, SUC_lexicon_tags_47 },
    { 0xe90c5a4b, SUC_lexicon_tags_15 },
    { 0x48ac5a4e, SUC_lexicon_tags_64 },
    { 0x15ec5a4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50c5a54, SUC_lexicon_tags_49 },
    { 0xdc8c5a55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x850c5a57, SUC_lexicon_tags_30 },
    { 0x1e0c5a58, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4c5a5c, SUC_lexicon_tags_19 },
    { 0xef6c5a5c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x682c5a5f, SUC_lexicon_tags_61 },
    { 0x48c5a60, SUC_lexicon_tags_114 },
    { 0x88ac5a60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46ac5a63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350c5a69, SUC_lexicon_tags_47 },
    { 0xd4ac5a6a, SUC_lexicon_tags_37 },
    { 0xbecc5a6a, SUC_lexicon_tags_1 },
    { 0xeec5a6c, SUC_lexicon_tags_49 },
    { 0x8dec5a6d, SUC_lexicon_tags_10 },
    { 0x530c5a6e, SUC_lexicon_tags_2 },
    { 0x4d0c5a6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18c5a72, SUC_lexicon_tags_77 },
    { 0xd1ac5a72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6c5a75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c5a7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfeec5a7e, SUC_lexicon_tags_30 },
    { 0x6b6c5a7f, SUC_lexicon_tags_82 },
    { 0xdeac5a80, SUC_lexicon_tags_17 },
    { 0x744c5a80, SUC_lexicon_tags_47 },
    { 0x7d6c5a82, SUC_lexicon_tags_33 },
    { 0xa3ec5a83, SUC_lexicon_tags_82 },
    { 0xad0c5a84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x994c5a86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbd6c5a88, SUC_lexicon_tags_39 },
    { 0xb7ec5a88, SUC_lexicon_tags_48 },
    { 0x282c5a8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb52c5a8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ac5a8f, SUC_lexicon_tags_11 },
    { 0xe20c5a8f, SUC_lexicon_tags_17 },
    { 0x790c5a91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf30c5a93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x376c5a96, SUC_lexicon_tags_7 },
    { 0xbfec5a97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8c5a9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684c5a9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5acc5aa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e2c5aa4, SUC_lexicon_tags_2 },
    { 0x44c5aa5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94c5aab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeec5ab0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0c5ab3, SUC_lexicon_tags_19 },
    { 0x152c5ab4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4c5ab8, SUC_lexicon_tags_2 },
    { 0xddec5ab8, SUC_lexicon_tags_25 },
    { 0xdec5aba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x108c5abc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0c5ac0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ec5ac5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec5ac8, SUC_lexicon_tags_15 },
    { 0x15cc5ac9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe40c5acb, SUC_lexicon_tags_113 },
    { 0x3f8c5acc, SUC_lexicon_tags_0 },
    { 0xa5cc5acc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9b8c5acf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x63ec5ad1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x888c5ad4, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bac5ada, SUC_lexicon_tags_9 },
    { 0x8c8c5ada, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x242c5add, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbd8c5adf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122c5ae4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x846c5ae6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcc5ae9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ac5af0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87ec5af2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d4c5af4, SUC_lexicon_tags_45 },
    { 0x154c5af5, SUC_lexicon_tags_2 },
    { 0x4c6c5af6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x120c5af8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcf2c5afa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324c5b02, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x682c5b05, SUC_lexicon_tags_85 },
    { 0x4d8c5b06, SUC_lexicon_tags_8 },
    { 0xb60c5b05, SUC_lexicon_tags_37 },
    { 0x7dcc5b08, SUC_lexicon_tags_11 },
    { 0x930c5b09, SUC_lexicon_tags_17 },
    { 0xf30c5b07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ac5b11, SUC_lexicon_tags_31 },
    { 0x414c5b12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb7ec5b14, SUC_lexicon_tags_43 },
    { 0x846c5b14, SUC_lexicon_tags_7 },
    { 0xb42c5b16, SUC_lexicon_tags_12 },
    { 0xc18c5b15, SUC_lexicon_tags_6 },
    { 0x9f2c5b14, SUC_lexicon_tags_7 },
    { 0x668c5b19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cc5b1e, SUC_lexicon_tags_14 },
    { 0xab6c5b1f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c5b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5eec5b25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x17ec5b27, SUC_lexicon_tags_37 },
    { 0x3aec5b28, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fec5b2b, SUC_lexicon_tags_287 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30c5b2f, SUC_lexicon_tags_61 },
    { 0x590c5b30, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xb2cc5b32, SUC_lexicon_tags_15 },
    { 0x20ec5b32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x632c5b35, SUC_lexicon_tags_84 },
    { 0x39cc5b35, SUC_lexicon_tags_16 },
    { 0xa08c5b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x872c5b39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8c5b3d, SUC_lexicon_tags_2 },
    { 0x8ecc5b3e, SUC_lexicon_tags_19 },
    { 0xcec5b3e, SUC_lexicon_tags_16 },
    { 0x63ec5b3d, SUC_lexicon_tags_45 },
    { 0x364c5b41, SUC_lexicon_tags_25 },
    { 0xdaec5b42, SUC_lexicon_tags_33 },
    { 0x502c5b41, SUC_lexicon_tags_17 },
    { 0xbb2c5b41, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x238c5b47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900c5b4b, SUC_lexicon_tags_63 },
    { 0x4acc5b4c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd4ec5b4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x734c5b51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x328c5b53, SUC_lexicon_tags_64 },
    { 0x67ac5b54, SUC_lexicon_tags_2 },
    { 0x624c5b55, SUC_lexicon_tags_9 },
    { 0x8a6c5b56, SUC_lexicon_tags_8 },
    { 0x354c5b57, SUC_lexicon_tags_21 },
    { 0xa78c5b58, SUC_lexicon_tags_49 },
    { 0x8aac5b59, SUC_lexicon_tags_37 },
    { 0xf38c5b5a, SUC_lexicon_tags_12 },
    { 0x234c5b5b, SUC_lexicon_tags_61 },
    { 0xcc2c5b5c, SUC_lexicon_tags_43 },
    { 0x88ac5b5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ec5b60, SUC_lexicon_tags_7 },
    { 0xedcc5b61, SUC_lexicon_tags_32 },
    { 0x202c5b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6c5b67, SUC_lexicon_tags_44 },
    { 0x5d4c5b68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8c5b6d, SUC_lexicon_tags_38 },
    { 0x316c5b6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6ec5b70, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7f0c5b72, SUC_lexicon_tags_37 },
    { 0xcbac5b73, SUC_lexicon_tags_25 },
    { 0xe1ac5b74, SUC_lexicon_tags_418 },
    { 0xfa6c5b75, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd68c5b77, SUC_lexicon_tags_12 },
    { 0xcbac5b78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3ec5b7a, SUC_lexicon_tags_55 },
    { 0x7a4c5b7b, SUC_lexicon_tags_46 },
    { 0x61ec5b7c, SUC_lexicon_tags_1 },
    { 0x3d0c5b7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c5b84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176c5b88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28c5b92, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x11cc5b94, SUC_lexicon_tags_15 },
    { 0x2cc5b95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02c5b99, SUC_lexicon_tags_23 },
    { 0xeb2c5b99, SUC_lexicon_tags_107 },
    { 0x3b8c5b9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36c5b9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cc5ba3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c5ba6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5eac5ba8, SUC_lexicon_tags_17 },
    { 0xfa6c5ba9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394c5bae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x334c5bb0, SUC_lexicon_tags_43 },
    { 0xce0c5bb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8c5bb6, SUC_lexicon_tags_18 },
    { 0xbc4c5bb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x350c5bb9, SUC_lexicon_tags_33 },
    { 0x4e6c5bba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x856c5bbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0c5bc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c6c5bc9, SUC_lexicon_tags_49 },
    { 0xf9cc5bc9, SUC_lexicon_tags_12 },
    { 0x98ec5bcb, SUC_lexicon_tags_68 },
    { 0x4dac5bc9, SUC_lexicon_tags_7 },
    { 0xb60c5bcc, SUC_lexicon_tags_34 },
    { 0xd9ac5bcd, SUC_lexicon_tags_2 },
    { 0x318c5bc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ec5bd2, SUC_lexicon_tags_19 },
    { 0x13ec5bd3, SUC_lexicon_tags_21 },
    { 0xda0c5bd4, SUC_lexicon_tags_12 },
    { 0xedec5bd4, SUC_lexicon_tags_37 },
    { 0x420c5bd6, SUC_lexicon_tags_2 },
    { 0x24c5bd3, SUC_lexicon_tags_29 },
    { 0x112c5bd7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x106c5bda, SUC_lexicon_tags_39 },
    { 0x150c5bdb, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0x104c5bdd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cc5be1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x910c5be3, SUC_lexicon_tags_60 },
    { 0xb56c5be3, SUC_lexicon_tags_63 },
    { 0x97ac5be4, SUC_lexicon_tags_44 },
    { 0x410c5be3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956c5bea, SUC_lexicon_tags_4 },
    { 0x36ec5bea, SUC_lexicon_tags_45 },
    { 0xc66c5bea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbec5bf0, SUC_lexicon_tags_12 },
    { 0xb42c5bf1, SUC_lexicon_tags_11 },
    { 0xb6ac5bf1, SUC_lexicon_tags_17 },
    { 0x158c5bf3, SUC_lexicon_tags_197 },
    { 0x54c5bf4, SUC_lexicon_tags_2 },
    { 0x126c5bf4, SUC_lexicon_tags_23 },
    { 0x28ac5bf6, SUC_lexicon_tags_21 },
    { 0x36ec5bf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cc5bfb, SUC_lexicon_tags_11 },
    { 0x8aac5bfc, SUC_lexicon_tags_63 },
    { 0xadcc5bfc, SUC_lexicon_tags_23 },
    { 0xa52c5bfc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192c5c02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeecc5c05, SUC_lexicon_tags_0 },
    { 0x590c5c06, SUC_lexicon_tags_129 },
    { 0xd9ac5c06, SUC_lexicon_tags_28 },
    { 0xed2c5c08, SUC_lexicon_tags_26 },
    { 0xfccc5c09, SUC_lexicon_tags_7 },
    { 0x244c5c07, SUC_lexicon_tags_31 },
    { 0x45ac5c0a, SUC_lexicon_tags_15 },
    { 0xcc5c0c, SUC_lexicon_tags_12 },
    { 0x4eec5c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccac5c0f, SUC_lexicon_tags_12 },
    { 0x898c5c0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x526c5c12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ec5c17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x782c5c1a, SUC_lexicon_tags_12 },
    { 0xfe6c5c1a, SUC_lexicon_tags_37 },
    { 0xaa0c5c1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ec5c20, SUC_lexicon_tags_14 },
    { 0x3a0c5c21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214c5c29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4c5c2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ec5c32, SUC_lexicon_tags_30 },
    { 0x528c5c33, SUC_lexicon_tags_14 },
    { 0xfc0c5c34, SUC_lexicon_tags_28 },
    { 0xeeec5c32, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x874c5c38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ec5c44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ac5c47, SUC_lexicon_tags_11 },
    { 0x156c5c47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc06c5c4a, SUC_lexicon_tags_68 },
    { 0x922c5c4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd32c5c4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790c5c56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196c5c5d, SUC_lexicon_tags_29 },
    { 0xfbac5c5d, SUC_lexicon_tags_58 },
    { 0xdf6c5c5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6c5c64, SUC_lexicon_tags_172 },
    { 0x918c5c65, SUC_lexicon_tags_44 },
    { 0xe9ac5c66, SUC_lexicon_tags_60 },
    { 0x102c5c64, SUC_lexicon_tags_7 },
    { 0x296c5c65, SUC_lexicon_tags_37 },
    { 0xcc2c5c69, SUC_lexicon_tags_1 },
    { 0x5a6c5c68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ec5c6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8c5c74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854c5c7c, SUC_lexicon_tags_7 },
    { 0x4c2c5c7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2ac5c7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6c5c82, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ac5c8a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa80c5c8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6c5c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bac5c94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294c5c99, SUC_lexicon_tags_15 },
    { 0xb78c5c9a, SUC_lexicon_tags_23 },
    { 0x90cc5c9a, SUC_lexicon_tags_103 },
    { 0x84cc5c9c, SUC_lexicon_tags_9 },
    { 0x6a8c5c9c, SUC_lexicon_tags_17 },
    { 0xd8ec5c9c, SUC_lexicon_tags_43 },
    { 0xb66c5c9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4cc5ca1, SUC_lexicon_tags_17 },
    { 0xd32c5ca1, SUC_lexicon_tags_0 },
    { 0x8c8c5ca3, SUC_lexicon_tags_25 },
    { 0xe10c5ca4, SUC_lexicon_tags_11 },
    { 0x604c5ca5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc72c5ca7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ac5cae, SUC_lexicon_tags_37 },
    { 0xe28c5cae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ec5cb2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2c5cb5, SUC_lexicon_tags_14 },
    { 0xe02c5cb5, SUC_lexicon_tags_15 },
    { 0x676c5cb6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446c5cbb, SUC_lexicon_tags_17 },
    { 0x18cc5cbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8b4c5cbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04c5cc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6c5cc6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aec5ccd, SUC_lexicon_tags_49 },
    { 0x908c5cce, SUC_lexicon_tags_49 },
    { 0xe22c5ccf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ac5cd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x990c5cd4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd62c5cd6, SUC_lexicon_tags_23 },
    { 0xf60c5cd6, SUC_lexicon_tags_17 },
    { 0x55ec5cd8, SUC_lexicon_tags_1 },
    { 0x930c5cd9, SUC_lexicon_tags_10 },
    { 0xacec5cda, SUC_lexicon_tags_65 },
    { 0x71ac5cdb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7c2c5cdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda2c5cdf, SUC_lexicon_tags_2 },
    { 0x790c5cdf, SUC_lexicon_tags_33 },
    { 0x274c5cdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6c5ce6, SUC_lexicon_tags_12 },
    { 0xd76c5ce7, SUC_lexicon_tags_30 },
    { 0x12cc5ce8, SUC_lexicon_tags_57 },
    { 0x61cc5ce9, SUC_lexicon_tags_42 },
    { 0x1dac5ce6, SUC_lexicon_tags_48 },
    { 0xe80c5ce7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x750c5ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac5cf2, SUC_lexicon_tags_37 },
    { 0xf32c5cf3, SUC_lexicon_tags_0 },
    { 0xadac5cf4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0c5cf9, SUC_lexicon_tags_17 },
    { 0x278c5cfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cc5cff, SUC_lexicon_tags_49 },
    { 0x892c5cff, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x1f6c5d02, SUC_lexicon_tags_63 },
    { 0x182c5d02, SUC_lexicon_tags_0 },
    { 0x4ccc5d04, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cc5d07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x55cc5d09, SUC_lexicon_tags_11 },
    { 0x28c5d0a, SUC_lexicon_tags_9 },
    { 0xa40c5d0a, SUC_lexicon_tags_87 },
    { 0xbbec5d0c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ec5d13, SUC_lexicon_tags_11 },
    { 0x27cc5d14, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540c5d1a, SUC_lexicon_tags_46 },
    { 0x9c8c5d1b, SUC_lexicon_tags_39 },
    { 0x2eac5d1a, SUC_lexicon_tags_23 },
    { 0x200c5d1d, SUC_lexicon_tags_17 },
    { 0x402c5d1e, SUC_lexicon_tags_30 },
    { 0x634c5d1e, SUC_lexicon_tags_133 },
    { 0xea4c5d20, SUC_lexicon_tags_25 },
    { 0x4ccc5d21, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccc5d2e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00c5d35, SUC_lexicon_tags_60 },
    { 0x3eac5d35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x680c5d39, SUC_lexicon_tags_33 },
    { 0x1cec5d3a, SUC_lexicon_tags_62 },
    { 0x914c5d3b, SUC_lexicon_tags_7 },
    { 0x5d4c5d3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb2c5d3e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9b4c5d40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec5d43, SUC_lexicon_tags_151 },
    { 0xff0c5d44, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcb4c5d46, SUC_lexicon_tags_10 },
    { 0x56cc5d47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406c5d4c, SUC_lexicon_tags_63 },
    { 0xc22c5d4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5d6c5d4f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x270c5d51, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ec5d59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9dec5d5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6c5d60, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eec5d63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe52c5d65, SUC_lexicon_tags_17 },
    { 0x654c5d66, SUC_lexicon_tags_44 },
    { 0xd32c5d67, SUC_lexicon_tags_2 },
    { 0xe82c5d65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccc5d6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeacc5d6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4c5d72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa0c5d74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc5d7a, SUC_lexicon_tags_30 },
    { 0x82ac5d7b, SUC_lexicon_tags_12 },
    { 0x780c5d7a, SUC_lexicon_tags_7 },
    { 0xc1ec5d7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ac5d8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2c5d8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8c5d92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196c5d96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92c5d9d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x42cc5d9f, SUC_lexicon_tags_29 },
    { 0x128c5da0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x938c5da2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ec5da6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42ac5da8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd20c5daa, SUC_lexicon_tags_9 },
    { 0x796c5dab, SUC_lexicon_tags_23 },
    { 0x96ec5dac, SUC_lexicon_tags_49 },
    { 0xd02c5dab, SUC_lexicon_tags_22 },
    { 0xf5ac5dae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cc5db3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa10c5db5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x736c5db8, SUC_lexicon_tags_7 },
    { 0xdc8c5db8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x152c5dbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cc5dbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bac5dc1, SUC_lexicon_tags_45 },
    { 0x144c5dc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ec5dcb, SUC_lexicon_tags_13 },
    { 0x70c5dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3ac5dce, SUC_lexicon_tags_65 },
    { 0x51ac5dcf, SUC_lexicon_tags_0 },
    { 0xe74c5dcf, SUC_lexicon_tags_19 },
    { 0xb4ec5dd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54c5dd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cac5dd7, SUC_lexicon_tags_11 },
    { 0x2c8c5dd7, SUC_lexicon_tags_66 },
    { 0x36cc5dd9, SUC_lexicon_tags_18 },
    { 0xc2c5dda, SUC_lexicon_tags_17 },
    { 0xd9cc5dd9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6c5de1, SUC_lexicon_tags_1 },
    { 0x9ac5de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00c5de8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a0c5dea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa64c5dec, SUC_lexicon_tags_14 },
    { 0x1ecc5dec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52c5df7, SUC_lexicon_tags_0 },
    { 0x19ac5df7, SUC_lexicon_tags_37 },
    { 0xf64c5df9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x524c5dfc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2c5dff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ec5e03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ac5e07, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x930c5e09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4c5e0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976c5e12, SUC_lexicon_tags_16 },
    { 0x484c5e13, SUC_lexicon_tags_2 },
    { 0xd64c5e13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880c5e18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ac5e1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6c5e21, SUC_lexicon_tags_37 },
    { 0xf2ec5e22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe8ec5e24, SUC_lexicon_tags_27 },
    { 0x810c5e25, SUC_lexicon_tags_17 },
    { 0xe46c5e26, SUC_lexicon_tags_10 },
    { 0xe2cc5e25, SUC_lexicon_tags_17 },
    { 0x100c5e28, SUC_lexicon_tags_31 },
    { 0x58cc5e25, SUC_lexicon_tags_37 },
    { 0xdf4c5e27, SUC_lexicon_tags_84 },
    { 0x664c5e2b, SUC_lexicon_tags_25 },
    { 0x9f6c5e2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e4c5e2e, SUC_lexicon_tags_25 },
    { 0x23cc5e2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb44c5e31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xccec5e34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b0c5e36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb24c5e38, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776c5e43, SUC_lexicon_tags_1 },
    { 0x6dac5e43, SUC_lexicon_tags_43 },
    { 0x770c5e44, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xee2c5e47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c5e4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384c5e4f, SUC_lexicon_tags_2 },
    { 0x1bec5e4f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7ac5e52, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3c8c5e54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d8c5e56, SUC_lexicon_tags_23 },
    { 0x516c5e56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbec5e59, SUC_lexicon_tags_6 },
    { 0xb6c5e5a, SUC_lexicon_tags_27 },
    { 0x678c5e5a, SUC_lexicon_tags_16 },
    { 0xe06c5e5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eec5e5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ac5e62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cc5e66, SUC_lexicon_tags_0 },
    { 0x62c5e67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acc5e6b, SUC_lexicon_tags_1 },
    { 0xc1ec5e6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ec5e70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ec5e74, SUC_lexicon_tags_29 },
    { 0x5ecc5e75, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x572c5e77, SUC_lexicon_tags_19 },
    { 0x528c5e77, SUC_lexicon_tags_17 },
    { 0x6a4c5e79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecc5e80, SUC_lexicon_tags_9 },
    { 0x790c5e81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1cc5e83, SUC_lexicon_tags_6 },
    { 0xffcc5e84, SUC_lexicon_tags_7 },
    { 0x302c5e85, SUC_lexicon_tags_29 },
    { 0x608c5e86, SUC_lexicon_tags_37 },
    { 0xcfcc5e84, SUC_lexicon_tags_4 },
    { 0xfc4c5e88, SUC_lexicon_tags_39 },
    { 0x2a4c5e88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40c5e8c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ec5e92, SUC_lexicon_tags_14 },
    { 0x6ecc5e93, SUC_lexicon_tags_3 },
    { 0x582c5e93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ac5e98, SUC_lexicon_tags_37 },
    { 0x52ac5e99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cc5e9e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xfbec5ea0, SUC_lexicon_tags_37 },
    { 0xa46c5ea1, SUC_lexicon_tags_38 },
    { 0x1a0c5ea1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acc5ea9, SUC_lexicon_tags_12 },
    { 0xea8c5eaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd2c5eac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3cc5eae, SUC_lexicon_tags_4 },
    { 0xf78c5eaf, SUC_lexicon_tags_25 },
    { 0x358c5eae, SUC_lexicon_tags_17 },
    { 0xb7cc5eb1, SUC_lexicon_tags_7 },
    { 0x6c8c5eb1, SUC_lexicon_tags_25 },
    { 0x102c5eb3, SUC_lexicon_tags_15 },
    { 0xd8c5eb3, SUC_lexicon_tags_100 },
    { 0xea2c5eb2, SUC_lexicon_tags_31 },
    { 0x264c5eb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c5ebf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0c5ec7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x960c5eca, SUC_lexicon_tags_13 },
    { 0xd0c5ecb, SUC_lexicon_tags_43 },
    { 0x24c5ecb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ec5ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320c5eda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720c5ee4, SUC_lexicon_tags_37 },
    { 0xd4c5ee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6c5ee9, SUC_lexicon_tags_0 },
    { 0xcccc5eea, SUC_lexicon_tags_17 },
    { 0x6a0c5eeb, SUC_lexicon_tags_37 },
    { 0x120c5eec, SUC_lexicon_tags_49 },
    { 0x8a6c5eea, SUC_lexicon_tags_63 },
    { 0xae2c5ee9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcc5ef1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4ac5ef3, SUC_lexicon_tags_11 },
    { 0x906c5ef4, SUC_lexicon_tags_17 },
    { 0x138c5ef5, SUC_lexicon_tags_37 },
    { 0xaa6c5ef4, SUC_lexicon_tags_37 },
    { 0x7c4c5ef6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142c5efd, SUC_lexicon_tags_14 },
    { 0xb00c5efd, SUC_lexicon_tags_7 },
    { 0x886c5eff, SUC_lexicon_tags_15 },
    { 0xa2cc5efe, SUC_lexicon_tags_33 },
    { 0x30ec5efd, SUC_lexicon_tags_25 },
    { 0xcb0c5f02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c5f07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c8c5f09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2ac5f0b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ec5f0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ac5f12, SUC_lexicon_tags_9 },
    { 0x8c6c5f13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8c5f17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cc5f1b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefec5f1f, SUC_lexicon_tags_27 },
    { 0x41ac5f20, SUC_lexicon_tags_49 },
    { 0xe3ac5f21, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8c5f29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeecc5f2c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2c5f30, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecc5f3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x35ac5f3c, SUC_lexicon_tags_1601 },
    { 0x6eec5f3d, SUC_lexicon_tags_6 },
    { 0x2a4c5f3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcc5f41, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x414c5f44, SUC_lexicon_tags_65 },
    { 0x590c5f45, SUC_lexicon_tags_14 },
    { 0x23ec5f46, SUC_lexicon_tags_31 },
    { 0x110c5f45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79cc5f4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4fec5f4d, SUC_lexicon_tags_3 },
    { 0x8a4c5f4d, SUC_lexicon_tags_17 },
    { 0xfbac5f4d, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x1aac5f51, SUC_lexicon_tags_66 },
    { 0x3a0c5f52, SUC_lexicon_tags_17 },
    { 0x90cc5f53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcc5f56, SUC_lexicon_tags_19 },
    { 0x8dec5f57, SUC_lexicon_tags_215 },
    { 0x70ec5f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84c5f5c, SUC_lexicon_tags_25 },
    { 0x75ec5f5d, SUC_lexicon_tags_53 },
    { 0x780c5f5c, SUC_lexicon_tags_47 },
    { 0xa00c5f5e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec5f63, SUC_lexicon_tags_23 },
    { 0xe66c5f63, SUC_lexicon_tags_38 },
    { 0xd7ac5f63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6c5f6a, SUC_lexicon_tags_15 },
    { 0xc12c5f6b, SUC_lexicon_tags_2 },
    { 0x440c5f6c, SUC_lexicon_tags_45 },
    { 0x660c5f6d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8c5f73, SUC_lexicon_tags_7 },
    { 0xfdec5f74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8c5f77, SUC_lexicon_tags_1 },
    { 0xca0c5f78, SUC_lexicon_tags_24 },
    { 0xee4c5f79, SUC_lexicon_tags_19 },
    { 0x270c5f7a, SUC_lexicon_tags_45 },
    { 0x602c5f7b, SUC_lexicon_tags_1 },
    { 0x59ac5f7a, SUC_lexicon_tags_46 },
    { 0xa60c5f7a, SUC_lexicon_tags_12 },
    { 0x94ec5f77, SUC_lexicon_tags_27 },
    { 0xb26c5f7f, SUC_lexicon_tags_25 },
    { 0xa8ec5f80, SUC_lexicon_tags_0 },
    { 0x694c5f81, SUC_lexicon_tags_48 },
    { 0x51ec5f79, SUC_lexicon_tags_6 },
    { 0x4f0c5f83, SUC_lexicon_tags_37 },
    { 0x43ec5f83, SUC_lexicon_tags_11 },
    { 0xab2c5f85, SUC_lexicon_tags_26 },
    { 0xc88c5f85, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf82c5f88, SUC_lexicon_tags_7 },
    { 0x910c5f89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf6c5f8b, SUC_lexicon_tags_1 },
    { 0xb08c5f8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8c5f90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942c5f94, SUC_lexicon_tags_21 },
    { 0xf52c5f94, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaec5f97, SUC_lexicon_tags_7 },
    { 0x954c5f97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700c5f9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8c5f9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8c5fa7, SUC_lexicon_tags_19 },
    { 0x716c5fa8, SUC_lexicon_tags_87 },
    { 0x968c5fa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac5fae, SUC_lexicon_tags_37 },
    { 0xd50c5faf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aec5fb5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x97cc5fb8, SUC_lexicon_tags_15 },
    { 0x6fec5fb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x16ac5fbb, SUC_lexicon_tags_49 },
    { 0x764c5fbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86c5fc1, SUC_lexicon_tags_3 },
    { 0x578c5fc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ec5fc5, SUC_lexicon_tags_37 },
    { 0x900c5fc6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2c5fcd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5fd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45ec5fd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8c5fd6, SUC_lexicon_tags_17 },
    { 0x172c5fd7, SUC_lexicon_tags_2 },
    { 0xdeec5fd8, SUC_lexicon_tags_1 },
    { 0x152c5fd7, SUC_lexicon_tags_114 },
    { 0xbacc5fd8, SUC_lexicon_tags_37 },
    { 0x6bcc5fd6, SUC_lexicon_tags_7 },
    { 0x5dec5fdc, SUC_lexicon_tags_60 },
    { 0x5b0c5fdc, SUC_lexicon_tags_48 },
    { 0xda6c5fd8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe8c5fe0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ac5fe6, SUC_lexicon_tags_11 },
    { 0xef8c5fe7, SUC_lexicon_tags_7 },
    { 0x98cc5fe8, SUC_lexicon_tags_10 },
    { 0xa9ac5fe8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ac5fee, SUC_lexicon_tags_14 },
    { 0x818c5fee, SUC_lexicon_tags_53 },
    { 0xa2cc5fef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ec5ff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfecc5ff6, SUC_lexicon_tags_25 },
    { 0xd38c5ff6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x12ec5ff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dac5ffc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x372c5ffe, SUC_lexicon_tags_25 },
    { 0x970c5fff, SUC_lexicon_tags_11 },
    { 0xbeec5fff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ac6003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccc6006, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eec6009, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2a2c600b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280c6010, SUC_lexicon_tags_0 },
    { 0x1c4c6010, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8c6018, SUC_lexicon_tags_38 },
    { 0xecc6019, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774c601e, SUC_lexicon_tags_0 },
    { 0x650c601f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260c6025, SUC_lexicon_tags_63 },
    { 0x19ac6026, SUC_lexicon_tags_37 },
    { 0x4c4c6027, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedec6029, SUC_lexicon_tags_47 },
    { 0x382c6029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ec602e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58c6035, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x130c6037, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccc603e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8c6041, SUC_lexicon_tags_43 },
    { 0xec4c6041, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cc6045, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2c604c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x486c604f, SUC_lexicon_tags_6 },
    { 0xe24c604f, SUC_lexicon_tags_2 },
    { 0xd12c6051, SUC_lexicon_tags_7 },
    { 0xc1ac6052, SUC_lexicon_tags_13 },
    { 0x276c6053, SUC_lexicon_tags_37 },
    { 0x998c6054, SUC_lexicon_tags_37 },
    { 0x6d6c6053, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522c6059, SUC_lexicon_tags_10 },
    { 0x2fcc6059, SUC_lexicon_tags_63 },
    { 0xa6c605b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4f6c605d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4c6060, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecac6065, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6c606e, SUC_lexicon_tags_55 },
    { 0x91ac606e, SUC_lexicon_tags_45 },
    { 0xb28c6070, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ec6073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdec6077, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0c607c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70c6083, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x606c6085, SUC_lexicon_tags_12 },
    { 0x99cc6085, SUC_lexicon_tags_25 },
    { 0xcf6c6086, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2c608a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ec608e, SUC_lexicon_tags_48 },
    { 0x7aac608f, SUC_lexicon_tags_2 },
    { 0xfa0c608f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcac6092, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe54c6094, SUC_lexicon_tags_15 },
    { 0xd64c6095, SUC_lexicon_tags_66 },
    { 0xed6c6095, SUC_lexicon_tags_1 },
    { 0xa90c6094, SUC_lexicon_tags_34 },
    { 0x286c6098, SUC_lexicon_tags_15 },
    { 0x82ac6099, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2c609c, SUC_lexicon_tags_23 },
    { 0x344c609c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eec60a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1ec60a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7ec60a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe48c60ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ac60b0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704c60b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8c60b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x568c60ba, SUC_lexicon_tags_1 },
    { 0x64c60bb, SUC_lexicon_tags_37 },
    { 0xf08c60bc, SUC_lexicon_tags_62 },
    { 0x2d6c60ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90cc60bf, SUC_lexicon_tags_28 },
    { 0xff4c60c0, SUC_lexicon_tags_12 },
    { 0x45cc60c1, SUC_lexicon_tags_43 },
    { 0xd98c60c2, SUC_lexicon_tags_38 },
    { 0x6ecc60c2, SUC_lexicon_tags_13 },
    { 0x48ac60c4, SUC_lexicon_tags_31 },
    { 0x16c60c3, SUC_lexicon_tags_30 },
    { 0x1e4c60bf, SUC_lexicon_tags_15 },
    { 0x43cc60c4, SUC_lexicon_tags_15 },
    { 0x27ec60c8, SUC_lexicon_tags_23 },
    { 0x6dec60c9, SUC_lexicon_tags_57 },
    { 0x64c60c7, SUC_lexicon_tags_31 },
    { 0x242c60cb, SUC_lexicon_tags_29 },
    { 0x736c60cb, SUC_lexicon_tags_7 },
    { 0xed0c60c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20c60d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a6c60d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x198c60d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0c60e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8c60e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380c60f0, SUC_lexicon_tags_43 },
    { 0xe82c60f0, SUC_lexicon_tags_33 },
    { 0xb72c60f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94c60f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x976c60f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734c60ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8c6103, SUC_lexicon_tags_7 },
    { 0x7bcc6104, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3cac6106, SUC_lexicon_tags_29 },
    { 0x78ac6106, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cec610a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ac610d, SUC_lexicon_tags_40 },
    { 0xdb2c610d, SUC_lexicon_tags_37 },
    { 0x168c610f, SUC_lexicon_tags_2 },
    { 0xf98c6110, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6c6114, SUC_lexicon_tags_0 },
    { 0x2d0c6115, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2a6c6117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8c611b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbac611f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ac6125, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4c612b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xddac612e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cec6131, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x768c6134, SUC_lexicon_tags_1 },
    { 0xda8c6134, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6c6138, SUC_lexicon_tags_15 },
    { 0xec2c6139, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x14ac613b, SUC_lexicon_tags_37 },
    { 0x194c613c, SUC_lexicon_tags_7 },
    { 0x4d0c613d, SUC_lexicon_tags_38 },
    { 0x304c613d, SUC_lexicon_tags_27 },
    { 0xf98c613f, SUC_lexicon_tags_17 },
    { 0x4f6c6140, SUC_lexicon_tags_12 },
    { 0x546c6140, SUC_lexicon_tags_11 },
    { 0xdec6142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72c6144, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6c2c6146, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbec614c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe12c614e, SUC_lexicon_tags_45 },
    { 0x16c614e, SUC_lexicon_tags_17 },
    { 0x450c614e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3acc6152, SUC_lexicon_tags_7 },
    { 0xcf0c6153, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8c6156, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fec6159, SUC_lexicon_tags_38 },
    { 0xfa0c6159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fac615f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ac6162, SUC_lexicon_tags_21 },
    { 0xa52c6163, SUC_lexicon_tags_14 },
    { 0x61ec6163, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ac6169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20c616c, SUC_lexicon_tags_77 },
    { 0x9b4c616d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bcc616f, SUC_lexicon_tags_25 },
    { 0x408c6170, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484c6176, SUC_lexicon_tags_16 },
    { 0x630c6177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48c617a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bec617d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ac6180, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x684c6182, SUC_lexicon_tags_6 },
    { 0x6cc6182, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9dec6185, SUC_lexicon_tags_17 },
    { 0xca0c6185, SUC_lexicon_tags_13 },
    { 0x8c0c6187, SUC_lexicon_tags_17 },
    { 0x55ac6188, SUC_lexicon_tags_14 },
    { 0x3f4c6187, SUC_lexicon_tags_11 },
    { 0x752c618a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24c618f, SUC_lexicon_tags_11 },
    { 0xc0ec6190, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0c6195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754c619f, SUC_lexicon_tags_25 },
    { 0x36ec619f, SUC_lexicon_tags_38 },
    { 0x9f0c61a1, SUC_lexicon_tags_1 },
    { 0x404c61a1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2c2c61a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x808c61a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x874c61aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cac61ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2c61b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4c61b6, SUC_lexicon_tags_45 },
    { 0x5a4c61b7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350c61c1, SUC_lexicon_tags_0 },
    { 0x972c61c2, SUC_lexicon_tags_38 },
    { 0x856c61c3, SUC_lexicon_tags_33 },
    { 0x9b2c61c4, SUC_lexicon_tags_25 },
    { 0x338c61c5, SUC_lexicon_tags_55 },
    { 0x20cc61c6, SUC_lexicon_tags_27 },
    { 0xc4cc61c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x528c61c9, SUC_lexicon_tags_11 },
    { 0x62ac61c9, SUC_lexicon_tags_37 },
    { 0xfd8c61cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ac61d1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x48c61d3, SUC_lexicon_tags_45 },
    { 0x3bec61d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac61de, SUC_lexicon_tags_10 },
    { 0xf06c61df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x42c61e1, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c61e4, SUC_lexicon_tags_12 },
    { 0x548c61e5, SUC_lexicon_tags_25 },
    { 0x70cc61e6, SUC_lexicon_tags_351 },
    { 0x4f2c61e7, SUC_lexicon_tags_27 },
    { 0x412c61e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e2c61ea, SUC_lexicon_tags_84 },
    { 0x67ac61eb, SUC_lexicon_tags_4 },
    { 0xf3cc61ec, SUC_lexicon_tags_37 },
    { 0x4d2c61ed, SUC_lexicon_tags_8 },
    { 0x5bec61ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ec61f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x996c61f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0c61fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf4c61fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd3cc61ff, SUC_lexicon_tags_27 },
    { 0x83ec61ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ec6203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330c6207, SUC_lexicon_tags_1 },
    { 0xa88c6207, SUC_lexicon_tags_79 },
    { 0xe88c6207, SUC_lexicon_tags_0 },
    { 0x9eec620a, SUC_lexicon_tags_23 },
    { 0x8e2c620a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0c620f, SUC_lexicon_tags_8 },
    { 0xe12c6210, SUC_lexicon_tags_1 },
    { 0x7e6c6210, SUC_lexicon_tags_68 },
    { 0x440c6212, SUC_lexicon_tags_11 },
    { 0x2dec6213, SUC_lexicon_tags_23 },
    { 0x34c6214, SUC_lexicon_tags_74 },
    { 0xd90c6213, SUC_lexicon_tags_23 },
    { 0xad0c6216, SUC_lexicon_tags_33 },
    { 0x846c6212, SUC_lexicon_tags_37 },
    { 0x182c6218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64c621e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dec6223, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84c6228, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7e8c622a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66c622e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb8cc6230, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x802c6232, SUC_lexicon_tags_151 },
    { 0x5fcc6232, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ec6237, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb88c6239, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ac623c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cc623f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ec6246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102c624e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6254, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x760c6256, SUC_lexicon_tags_15 },
    { 0x3cec6257, SUC_lexicon_tags_48 },
    { 0x152c6258, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88c625b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x328c625d, SUC_lexicon_tags_7 },
    { 0x38ec625e, SUC_lexicon_tags_105 },
    { 0xde0c625f, SUC_lexicon_tags_6 },
    { 0xbc2c625f, SUC_lexicon_tags_11 },
    { 0xdb2c625f, SUC_lexicon_tags_45 },
    { 0x1cac6262, SUC_lexicon_tags_19 },
    { 0xe84c6263, SUC_lexicon_tags_23 },
    { 0x952c6264, SUC_lexicon_tags_35 },
    { 0xf5cc6264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5bec6267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c626a, SUC_lexicon_tags_11 },
    { 0x884c626b, SUC_lexicon_tags_1 },
    { 0x644c626b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490c6273, SUC_lexicon_tags_7 },
    { 0xcc6274, SUC_lexicon_tags_37 },
    { 0x80ac6273, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4c627a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcaec627c, SUC_lexicon_tags_14 },
    { 0x970c627d, SUC_lexicon_tags_15 },
    { 0x1f8c627d, SUC_lexicon_tags_63 },
    { 0x184c627f, SUC_lexicon_tags_17 },
    { 0x8cac6280, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a0c6282, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cc6288, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c628b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x584c628d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cc6290, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acc6294, SUC_lexicon_tags_15 },
    { 0x8d6c6295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e4c6297, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76c629b, SUC_lexicon_tags_14 },
    { 0xb34c629c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ac62a0, SUC_lexicon_tags_7 },
    { 0x79ec62a0, SUC_lexicon_tags_37 },
    { 0x2b8c62a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112c62aa, SUC_lexicon_tags_17 },
    { 0x84cc62ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6ac62ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38c62b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238c62b8, SUC_lexicon_tags_8 },
    { 0x846c62b8, SUC_lexicon_tags_62 },
    { 0x13ac62ba, SUC_lexicon_tags_12 },
    { 0x826c62ba, SUC_lexicon_tags_26 },
    { 0xcdec62bb, SUC_lexicon_tags_17 },
    { 0xb9ac62bb, SUC_lexicon_tags_0 },
    { 0x95ac62ba, SUC_lexicon_tags_25 },
    { 0xfac62bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694c62c5, SUC_lexicon_tags_57 },
    { 0x57cc62c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0c62ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaa6c62cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb80c62ce, SUC_lexicon_tags_27 },
    { 0x66ec62cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cc62d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a2c62d7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x938c62da, SUC_lexicon_tags_7 },
    { 0xa40c62db, SUC_lexicon_tags_1 },
    { 0x814c62dc, SUC_lexicon_tags_25 },
    { 0xb2c62dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x68ec62df, SUC_lexicon_tags_207 },
    { 0xfe2c62e0, SUC_lexicon_tags_32 },
    { 0xa6c62e0, SUC_lexicon_tags_25 },
    { 0x22ec62e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x754c62e4, SUC_lexicon_tags_14 },
    { 0x3aec62e5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x620c62e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960c62eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750c62ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x140c62f1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646c62f7, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0x35ac62f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6c62fd, SUC_lexicon_tags_12 },
    { 0xbfec62fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9ac6300, SUC_lexicon_tags_11 },
    { 0x784c6301, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c6304, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xec0c6306, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2c630a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc38c630c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c6311, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x998c6314, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ec6317, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10c631a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b2c631c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ac6321, SUC_lexicon_tags_14 },
    { 0xaf8c6322, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400c6330, SUC_lexicon_tags_12 },
    { 0xaecc6331, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dec6336, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf84c6338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ac633a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7f2c633c, SUC_lexicon_tags_15 },
    { 0xfd6c633c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eec6340, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfac6349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ac634e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2c6351, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83cc6353, SUC_lexicon_tags_11 },
    { 0x93ec6354, SUC_lexicon_tags_15 },
    { 0x6b4c6354, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x816c6358, SUC_lexicon_tags_37 },
    { 0x180c6359, SUC_lexicon_tags_37 },
    { 0x754c6359, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ec635f, SUC_lexicon_tags_12 },
    { 0x63cc6360, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xabac6362, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x148c6365, SUC_lexicon_tags_37 },
    { 0xc8ac6365, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c6369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9ec636b, SUC_lexicon_tags_124 },
    { 0x696c636b, SUC_lexicon_tags_17 },
    { 0xa98c636d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ec6370, SUC_lexicon_tags_0 },
    { 0x474c6371, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c6375, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xffac6377, SUC_lexicon_tags_48 },
    { 0xa80c6378, SUC_lexicon_tags_60 },
    { 0x182c6377, SUC_lexicon_tags_957 },
    { 0x6cec6377, SUC_lexicon_tags_1 },
    { 0xb44c6378, SUC_lexicon_tags_23 },
    { 0x88cc637c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc5cc637e, SUC_lexicon_tags_63 },
    { 0xddcc637f, SUC_lexicon_tags_60 },
    { 0xc16c6380, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x826c6383, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fec6385, SUC_lexicon_tags_48 },
    { 0x88ac6386, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2c6389, SUC_lexicon_tags_47 },
    { 0x36ec638a, SUC_lexicon_tags_19 },
    { 0xea6c6389, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x818c638d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaec6390, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52c6393, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacac6399, SUC_lexicon_tags_15 },
    { 0x4c2c639a, SUC_lexicon_tags_11 },
    { 0xc6ec6399, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cc639f, SUC_lexicon_tags_7 },
    { 0xd92c639f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72c63a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438c63aa, SUC_lexicon_tags_23 },
    { 0x1e4c63aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88c63b9, SUC_lexicon_tags_12 },
    { 0xb38c63ba, SUC_lexicon_tags_6 },
    { 0xf76c63b9, SUC_lexicon_tags_25 },
    { 0xea8c63b9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cc63c0, SUC_lexicon_tags_25 },
    { 0x570c63c0, SUC_lexicon_tags_11 },
    { 0x772c63c2, SUC_lexicon_tags_11 },
    { 0x364c63c3, SUC_lexicon_tags_32 },
    { 0x1c2c63c3, SUC_lexicon_tags_37 },
    { 0xbf0c63c5, SUC_lexicon_tags_61 },
    { 0x278c63c2, SUC_lexicon_tags_17 },
    { 0x5e0c63c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04c63cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ac63ce, SUC_lexicon_tags_7 },
    { 0x244c63cf, SUC_lexicon_tags_37 },
    { 0xe6ec63d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44c63d9, SUC_lexicon_tags_6 },
    { 0xbdac63da, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cc63dd, SUC_lexicon_tags_28 },
    { 0xc42c63dd, SUC_lexicon_tags_37 },
    { 0x286c63dd, SUC_lexicon_tags_1 },
    { 0xe9ec63e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5ec63e2, SUC_lexicon_tags_0 },
    { 0x730c63e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0c63eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x406c63ed, SUC_lexicon_tags_37 },
    { 0xa60c63ee, SUC_lexicon_tags_23 },
    { 0x9aec63ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd30c63f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24cc63f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998c63f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac63fc, SUC_lexicon_tags_60 },
    { 0xff4c63fc, SUC_lexicon_tags_29 },
    { 0xbaac63fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b0c6400, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2c6405, SUC_lexicon_tags_46 },
    { 0xd58c6406, SUC_lexicon_tags_7 },
    { 0xdcac6407, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cc640a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cc640d, SUC_lexicon_tags_25 },
    { 0xe46c640e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeac6413, SUC_lexicon_tags_14 },
    { 0x1e2c6414, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296c6419, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c4c641b, SUC_lexicon_tags_47 },
    { 0xd3cc641b, SUC_lexicon_tags_0 },
    { 0xdf6c641d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cc6420, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85ec6422, SUC_lexicon_tags_15 },
    { 0x212c6422, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ac6426, SUC_lexicon_tags_15 },
    { 0xdd6c6427, SUC_lexicon_tags_25 },
    { 0xda2c6427, SUC_lexicon_tags_2 },
    { 0x244c6426, SUC_lexicon_tags_11 },
    { 0x25cc642a, SUC_lexicon_tags_19 },
    { 0x6e6c6429, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbec6430, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeeac6432, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296c6436, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x794c6438, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x24cc643a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2c643e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69cc6440, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4c6444, SUC_lexicon_tags_13 },
    { 0x90cc6444, SUC_lexicon_tags_1 },
    { 0x672c6446, SUC_lexicon_tags_1 },
    { 0x158c6447, SUC_lexicon_tags_37 },
    { 0xa38c6448, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ec644b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98c644e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362c6454, SUC_lexicon_tags_26 },
    { 0xa34c6454, SUC_lexicon_tags_1 },
    { 0xea0c6456, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0c6459, SUC_lexicon_tags_9 },
    { 0x636c6459, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4c645d, SUC_lexicon_tags_11 },
    { 0x864c645e, SUC_lexicon_tags_29 },
    { 0x13ec645e, SUC_lexicon_tags_4 },
    { 0x54ac6460, SUC_lexicon_tags_17 },
    { 0x6b4c6461, SUC_lexicon_tags_6 },
    { 0x628c6461, SUC_lexicon_tags_39 },
    { 0xc6cc645e, SUC_lexicon_tags_17 },
    { 0x124c6462, SUC_lexicon_tags_47 },
    { 0xe18c645d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x772c6468, SUC_lexicon_tags_26 },
    { 0xed2c6469, SUC_lexicon_tags_10 },
    { 0x2eac646a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ac646e, SUC_lexicon_tags_1 },
    { 0x7e2c646f, SUC_lexicon_tags_7 },
    { 0x5bac6470, SUC_lexicon_tags_15 },
    { 0x8a2c6470, SUC_lexicon_tags_17 },
    { 0x82ec6470, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa36c6474, SUC_lexicon_tags_82 },
    { 0xaacc6474, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x982c6478, SUC_lexicon_tags_2 },
    { 0x2a4c6479, SUC_lexicon_tags_33 },
    { 0x57ec647a, SUC_lexicon_tags_1 },
    { 0x3e2c647a, SUC_lexicon_tags_25 },
    { 0xdcc6479, SUC_lexicon_tags_8 },
    { 0x44cc647d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74c6485, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ac6488, SUC_lexicon_tags_12 },
    { 0x52c6488, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718c648f, SUC_lexicon_tags_31 },
    { 0xa00c6490, SUC_lexicon_tags_79 },
    { 0x138c6491, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1ec6493, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8b6c6495, SUC_lexicon_tags_11 },
    { 0x9c6c6496, SUC_lexicon_tags_12 },
    { 0xfbac6497, SUC_lexicon_tags_49 },
    { 0xb22c6498, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2c649c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ac64a1, SUC_lexicon_tags_25 },
    { 0x1aac64a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x354c64a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cc64a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x400c64ab, SUC_lexicon_tags_45 },
    { 0xadec64ac, SUC_lexicon_tags_56 },
    { 0x80ac64ad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x95ac64af, SUC_lexicon_tags_15 },
    { 0xbaec64af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ac64b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcc64bc, SUC_lexicon_tags_38 },
    { 0x6aec64bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cc64c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862c64d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15ec64d2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42c64d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aec64d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cc64dd, SUC_lexicon_tags_17 },
    { 0x144c64dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64c64e2, SUC_lexicon_tags_2 },
    { 0x7ecc64e3, SUC_lexicon_tags_6 },
    { 0x1acc64e4, SUC_lexicon_tags_60 },
    { 0x862c64e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdaec64e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cc64ed, SUC_lexicon_tags_7 },
    { 0x8fac64ed, SUC_lexicon_tags_63 },
    { 0x280c64ee, SUC_lexicon_tags_63 },
    { 0x2d4c64f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c64f4, SUC_lexicon_tags_11 },
    { 0x870c64f4, SUC_lexicon_tags_12 },
    { 0x528c64f4, SUC_lexicon_tags_23 },
    { 0xc52c64f7, SUC_lexicon_tags_1 },
    { 0xe24c64f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x126c64fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f8c64fd, SUC_lexicon_tags_43 },
    { 0xa8ac64fe, SUC_lexicon_tags_47 },
    { 0x54c64fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x484c6502, SUC_lexicon_tags_26 },
    { 0x24ac6503, SUC_lexicon_tags_62 },
    { 0x32ec6502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0c650a, SUC_lexicon_tags_43 },
    { 0x45ec650b, SUC_lexicon_tags_37 },
    { 0x834c650c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x928c650f, SUC_lexicon_tags_7 },
    { 0xf9ac6510, SUC_lexicon_tags_31 },
    { 0x486c650f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76c6516, SUC_lexicon_tags_40 },
    { 0x5ac6517, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa82c6519, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78c651c, SUC_lexicon_tags_29 },
    { 0x3a6c651c, SUC_lexicon_tags_43 },
    { 0xb5cc651c, SUC_lexicon_tags_26 },
    { 0x76c651f, SUC_lexicon_tags_44 },
    { 0xf32c651f, SUC_lexicon_tags_7 },
    { 0x86c651d, SUC_lexicon_tags_8 },
    { 0xc6ac6520, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74c6525, SUC_lexicon_tags_34 },
    { 0x624c6526, SUC_lexicon_tags_15 },
    { 0x7e0c6527, SUC_lexicon_tags_30 },
    { 0xcfac6527, SUC_lexicon_tags_65 },
    { 0xc00c6529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bac652c, SUC_lexicon_tags_7 },
    { 0xe3ec652c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370c6531, SUC_lexicon_tags_7 },
    { 0x706c6531, SUC_lexicon_tags_2 },
    { 0xcc0c6533, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7acc6535, SUC_lexicon_tags_15 },
    { 0x414c6536, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8c6538, SUC_lexicon_tags_12 },
    { 0xe2ec6539, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc7cc653b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa6c653d, SUC_lexicon_tags_0 },
    { 0x13ec653d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42ec6540, SUC_lexicon_tags_15 },
    { 0xd0c6541, SUC_lexicon_tags_30 },
    { 0xe46c6542, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c6546, SUC_lexicon_tags_49 },
    { 0x736c6547, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ec654d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6fac654f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6c6552, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ac6556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0ec6558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcc655d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2c6562, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86c6565, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfec6568, SUC_lexicon_tags_7 },
    { 0x668c6569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ac656d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ac6573, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd60c6575, SUC_lexicon_tags_29 },
    { 0x87ec6576, SUC_lexicon_tags_38 },
    { 0xa4ac6577, SUC_lexicon_tags_23 },
    { 0x144c6578, SUC_lexicon_tags_56 },
    { 0x5d6c6579, SUC_lexicon_tags_17 },
    { 0xb6ec6578, SUC_lexicon_tags_37 },
    { 0x5e4c657b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6c6582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cc6586, SUC_lexicon_tags_1 },
    { 0x900c6587, SUC_lexicon_tags_13 },
    { 0xb7ec6587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8c658c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x818c658e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x94c6590, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ec6593, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ac6596, SUC_lexicon_tags_37 },
    { 0xc4ec6596, SUC_lexicon_tags_17 },
    { 0x7a0c6598, SUC_lexicon_tags_7 },
    { 0x7aac6598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x382c659b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x730c659e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8c65a1, SUC_lexicon_tags_25 },
    { 0x15cc65a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x420c65a5, SUC_lexicon_tags_11 },
    { 0x4d6c65a5, SUC_lexicon_tags_25 },
    { 0xa46c65a6, SUC_lexicon_tags_14 },
    { 0x82c65a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964c65af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acc65b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0c65b6, SUC_lexicon_tags_37 },
    { 0xa6c65b7, SUC_lexicon_tags_37 },
    { 0xd60c65b8, SUC_lexicon_tags_11 },
    { 0x37cc65b8, SUC_lexicon_tags_25 },
    { 0xb54c65b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398c65c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c65d0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6c65d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99ec65d8, SUC_lexicon_tags_26 },
    { 0x5f2c65d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c4c65db, SUC_lexicon_tags_17 },
    { 0xf6c65dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d0c65de, SUC_lexicon_tags_1 },
    { 0xbdec65df, SUC_lexicon_tags_33 },
    { 0x744c65e0, SUC_lexicon_tags_11 },
    { 0x6dcc65df, SUC_lexicon_tags_33 },
    { 0x374c65e0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cc65e8, SUC_lexicon_tags_32 },
    { 0xacec65e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcc65ec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574c65f0, SUC_lexicon_tags_1 },
    { 0xfd2c65f1, SUC_lexicon_tags_15 },
    { 0x242c65f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884c65f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736c65fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0c65ff, SUC_lexicon_tags_47 },
    { 0x57ec6600, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ec6603, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ec6608, SUC_lexicon_tags_25 },
    { 0xfe2c6608, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4c660d, SUC_lexicon_tags_19 },
    { 0x9f0c660d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0c6612, SUC_lexicon_tags_31 },
    { 0x106c6612, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ac6616, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50c661b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2c6623, SUC_lexicon_tags_25 },
    { 0xf60c6623, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57cc6626, SUC_lexicon_tags_15 },
    { 0x7aac6627, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5eec6629, SUC_lexicon_tags_27 },
    { 0xe44c662a, SUC_lexicon_tags_70 },
    { 0xfa0c662b, SUC_lexicon_tags_2 },
    { 0xb96c662c, SUC_lexicon_tags_37 },
    { 0x740c662b, SUC_lexicon_tags_17 },
    { 0xb34c662e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc8ec6630, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536c6634, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x166c6636, SUC_lexicon_tags_82 },
    { 0xf12c6636, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6c663a, SUC_lexicon_tags_47 },
    { 0xc30c663b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6c663e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2c6640, SUC_lexicon_tags_82 },
    { 0x304c6641, SUC_lexicon_tags_26 },
    { 0x5fcc6641, SUC_lexicon_tags_31 },
    { 0xe36c6641, SUC_lexicon_tags_43 },
    { 0x314c6640, SUC_lexicon_tags_29 },
    { 0x4ecc6645, SUC_lexicon_tags_15 },
    { 0xbb2c6646, SUC_lexicon_tags_66 },
    { 0x958c6647, SUC_lexicon_tags_27 },
    { 0x962c6645, SUC_lexicon_tags_28 },
    { 0x876c6649, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78c664d, SUC_lexicon_tags_136 },
    { 0x51cc664e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae0c6651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x342c6654, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa0c6656, SUC_lexicon_tags_15 },
    { 0x7cac6657, SUC_lexicon_tags_48 },
    { 0x3e6c6658, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cc6661, SUC_lexicon_tags_1578 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64c666a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x346c666c, SUC_lexicon_tags_29 },
    { 0x676c666c, SUC_lexicon_tags_81 },
    { 0xd1ac666d, SUC_lexicon_tags_4 },
    { 0xf1ec666f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bac6675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e2c6677, SUC_lexicon_tags_61 },
    { 0xec8c6677, SUC_lexicon_tags_47 },
    { 0x4d4c6679, SUC_lexicon_tags_47 },
    { 0x4b8c6679, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cc6685, SUC_lexicon_tags_1 },
    { 0x390c6686, SUC_lexicon_tags_17 },
    { 0xe2ec6687, SUC_lexicon_tags_12 },
    { 0x7d0c6688, SUC_lexicon_tags_25 },
    { 0xf18c6688, SUC_lexicon_tags_30 },
    { 0xf44c6688, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3b6c668c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c668f, SUC_lexicon_tags_61 },
    { 0xa20c6690, SUC_lexicon_tags_40 },
    { 0xc36c6691, SUC_lexicon_tags_30 },
    { 0xbcac668f, SUC_lexicon_tags_77 },
    { 0xa02c6691, SUC_lexicon_tags_23 },
    { 0xd0cc6694, SUC_lexicon_tags_19 },
    { 0x4ec6695, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ac6699, SUC_lexicon_tags_75 },
    { 0xd08c669a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ec669f, SUC_lexicon_tags_47 },
    { 0x8c8c66a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fec66a4, SUC_lexicon_tags_25 },
    { 0x8a0c66a5, SUC_lexicon_tags_21 },
    { 0xd8ec66a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccc66aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ec66ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x988c66ae, SUC_lexicon_tags_29 },
    { 0xd42c66af, SUC_lexicon_tags_25 },
    { 0x53ec66af, SUC_lexicon_tags_1 },
    { 0x682c66b1, SUC_lexicon_tags_19 },
    { 0x9f4c66af, SUC_lexicon_tags_9 },
    { 0x850c66b3, SUC_lexicon_tags_14 },
    { 0x76ec66b4, SUC_lexicon_tags_37 },
    { 0x6f4c66b2, SUC_lexicon_tags_65 },
    { 0x43ec66b0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64c66b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538c66bf, SUC_lexicon_tags_2 },
    { 0x872c66c0, SUC_lexicon_tags_2 },
    { 0xde4c66c0, SUC_lexicon_tags_8 },
    { 0x948c66c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac66c9, SUC_lexicon_tags_15 },
    { 0x754c66ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838c66d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ec66d7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4c66df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2c66e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ac66e7, SUC_lexicon_tags_31 },
    { 0xa56c66e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54c66f0, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4c66f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x614c66f7, SUC_lexicon_tags_12 },
    { 0xd1ec66f8, SUC_lexicon_tags_15 },
    { 0xfe6c66f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8c66ff, SUC_lexicon_tags_14 },
    { 0x2f4c6700, SUC_lexicon_tags_17 },
    { 0xe1ac6700, SUC_lexicon_tags_0 },
    { 0xd14c6702, SUC_lexicon_tags_77 },
    { 0xc0ac6700, SUC_lexicon_tags_19 },
    { 0x5aac6703, SUC_lexicon_tags_25 },
    { 0x3dcc6705, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x716c6707, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x39ec6709, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0c670f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fec6711, SUC_lexicon_tags_2 },
    { 0x416c6712, SUC_lexicon_tags_8 },
    { 0x7cac6712, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc6716, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x43ec6718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec671d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cc6723, SUC_lexicon_tags_32 },
    { 0x294c6724, SUC_lexicon_tags_0 },
    { 0x9f2c6725, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2c6728, SUC_lexicon_tags_87 },
    { 0x8aec6728, SUC_lexicon_tags_37 },
    { 0x66ac672a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x656c672c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8dcc672e, SUC_lexicon_tags_40 },
    { 0xbe0c672f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6c6732, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa24c6734, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4ec6736, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ac6739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x796c673b, SUC_lexicon_tags_27 },
    { 0xdc6c673c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02c6741, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc6744, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c6747, SUC_lexicon_tags_48 },
    { 0xf5ac6748, SUC_lexicon_tags_47 },
    { 0xadac6749, SUC_lexicon_tags_15 },
    { 0xee8c674a, SUC_lexicon_tags_719 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42c674f, SUC_lexicon_tags_158 },
    { 0x48cc6750, SUC_lexicon_tags_1 },
    { 0xfccc6751, SUC_lexicon_tags_0 },
    { 0x23ac6752, SUC_lexicon_tags_7 },
    { 0x534c6752, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xcd0c6755, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ac675a, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac675e, SUC_lexicon_tags_79 },
    { 0xafcc675e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aec6762, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xacac6764, SUC_lexicon_tags_30 },
    { 0x290c6765, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68c676f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dac6773, SUC_lexicon_tags_17 },
    { 0x86ac6773, SUC_lexicon_tags_17 },
    { 0xe1cc6775, SUC_lexicon_tags_19 },
    { 0x108c6776, SUC_lexicon_tags_118 },
    { 0x5e2c6776, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9b4c6779, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x13ac677b, SUC_lexicon_tags_42 },
    { 0x3bec677c, SUC_lexicon_tags_66 },
    { 0x1cac677d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cc6782, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f4c6784, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x544c6786, SUC_lexicon_tags_25 },
    { 0xd6ec6786, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cac678e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6c6795, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x112c6797, SUC_lexicon_tags_37 },
    { 0x28ac6797, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x766c679a, SUC_lexicon_tags_15 },
    { 0xeac679b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x628c679d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ac67a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1dec67a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcc67a5, SUC_lexicon_tags_29 },
    { 0xc36c67a6, SUC_lexicon_tags_64 },
    { 0xa0ec67a5, SUC_lexicon_tags_12 },
    { 0x82c67a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ac67b2, SUC_lexicon_tags_10 },
    { 0xbdac67b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x134c67b6, SUC_lexicon_tags_21 },
    { 0x5d4c67b7, SUC_lexicon_tags_2 },
    { 0xed0c67b8, SUC_lexicon_tags_23 },
    { 0xd46c67b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260c67c1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xea2c67c3, SUC_lexicon_tags_15 },
    { 0x8aac67c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8c67c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24c67d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x714c67d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceac67d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8c67de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cc67e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2c67e8, SUC_lexicon_tags_84 },
    { 0xa2cc67e9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ec67ec, SUC_lexicon_tags_141 },
    { 0x516c67ed, SUC_lexicon_tags_63 },
    { 0x944c67ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc67f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc8c67f5, SUC_lexicon_tags_8 },
    { 0xea0c67f6, SUC_lexicon_tags_2 },
    { 0x474c67f7, SUC_lexicon_tags_27 },
    { 0x4f4c67f6, SUC_lexicon_tags_34 },
    { 0xf4ac67f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaec67fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2c6801, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4c6804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04c6807, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0c680a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4c680d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x156c6810, SUC_lexicon_tags_57 },
    { 0x8b0c6810, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x380c6813, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ec6816, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cc6819, SUC_lexicon_tags_17 },
    { 0xe2ec681a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ac681d, SUC_lexicon_tags_1 },
    { 0x8f4c681d, SUC_lexicon_tags_25 },
    { 0x1aac681d, SUC_lexicon_tags_38 },
    { 0x74ec6820, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81cc6822, SUC_lexicon_tags_23 },
    { 0x98c6823, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x352c6825, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1c0c6827, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b6c6829, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24c682b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x772c682d, SUC_lexicon_tags_12 },
    { 0x9cac682d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64c6830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8c6838, SUC_lexicon_tags_77 },
    { 0x16cc6838, SUC_lexicon_tags_17 },
    { 0x282c683a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc6cc683c, SUC_lexicon_tags_17 },
    { 0x9ccc683d, SUC_lexicon_tags_12 },
    { 0x676c683e, SUC_lexicon_tags_33 },
    { 0xc0cc683f, SUC_lexicon_tags_25 },
    { 0x500c6840, SUC_lexicon_tags_25 },
    { 0xb00c6841, SUC_lexicon_tags_7 },
    { 0x2d6c6842, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80c6845, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ec6848, SUC_lexicon_tags_9 },
    { 0xbf0c6849, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ec684e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x978c6850, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452c6855, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ac6860, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82cc6862, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeacc6864, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4d0c6866, SUC_lexicon_tags_25 },
    { 0x68c6867, SUC_lexicon_tags_49 },
    { 0x88ec6868, SUC_lexicon_tags_32 },
    { 0x61cc6867, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70c686f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0c6872, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34ac6874, SUC_lexicon_tags_25 },
    { 0x11ac6875, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24c687a, SUC_lexicon_tags_10 },
    { 0xe02c687a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14c687e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93cc6880, SUC_lexicon_tags_33 },
    { 0x492c6881, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bac688a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428c6890, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fcc6892, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514c689a, SUC_lexicon_tags_19 },
    { 0x892c689b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ec689e, SUC_lexicon_tags_29 },
    { 0x9d4c689e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48c68a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32c68a8, SUC_lexicon_tags_15 },
    { 0xd44c68a9, SUC_lexicon_tags_14 },
    { 0x7f2c68aa, SUC_lexicon_tags_2 },
    { 0x118c68ab, SUC_lexicon_tags_12 },
    { 0x312c68a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x184c68ae, SUC_lexicon_tags_18 },
    { 0x97ac68af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bac68b1, SUC_lexicon_tags_48 },
    { 0x13ac68b2, SUC_lexicon_tags_11 },
    { 0x9b8c68b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c68b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cc68bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x578c68bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2c68c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75ac68c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9cec68c9, SUC_lexicon_tags_160 },
    { 0x18cc68ca, SUC_lexicon_tags_71 },
    { 0x72ac68ca, SUC_lexicon_tags_4 },
    { 0x332c68cc, SUC_lexicon_tags_7 },
    { 0xe60c68cd, SUC_lexicon_tags_43 },
    { 0xf90c68cd, SUC_lexicon_tags_25 },
    { 0x58ac68c9, SUC_lexicon_tags_15 },
    { 0x492c68d0, SUC_lexicon_tags_37 },
    { 0x578c68d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0c68d6, SUC_lexicon_tags_23 },
    { 0xd64c68d6, SUC_lexicon_tags_39 },
    { 0xd28c68d8, SUC_lexicon_tags_11 },
    { 0x6dcc68d9, SUC_lexicon_tags_17 },
    { 0x4f0c68d9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8c68dd, SUC_lexicon_tags_37 },
    { 0xc82c68dd, SUC_lexicon_tags_37 },
    { 0x3f6c68df, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ec68e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5d4c68e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ac68eb, SUC_lexicon_tags_37 },
    { 0x65cc68ec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf50c68ee, SUC_lexicon_tags_14 },
    { 0xda6c68ef, SUC_lexicon_tags_15 },
    { 0xb0c68f0, SUC_lexicon_tags_49 },
    { 0xe24c68f0, SUC_lexicon_tags_12 },
    { 0xbfcc68f1, SUC_lexicon_tags_15 },
    { 0xdd6c68f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ac68f7, SUC_lexicon_tags_116 },
    { 0x3cc68f8, SUC_lexicon_tags_1 },
    { 0x992c68f9, SUC_lexicon_tags_37 },
    { 0xeeec68fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ac68fd, SUC_lexicon_tags_12 },
    { 0x2e8c68fe, SUC_lexicon_tags_0 },
    { 0x738c68ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa40c6901, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0c6905, SUC_lexicon_tags_15 },
    { 0xadac6906, SUC_lexicon_tags_12 },
    { 0x4aac6907, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccc690b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x470c690d, SUC_lexicon_tags_14 },
    { 0x468c690d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbec6911, SUC_lexicon_tags_43 },
    { 0x68cc6912, SUC_lexicon_tags_77 },
    { 0x282c6913, SUC_lexicon_tags_27 },
    { 0xe3cc6913, SUC_lexicon_tags_32 },
    { 0x2fac6915, SUC_lexicon_tags_4 },
    { 0x8aec6916, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94c691b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660c6920, SUC_lexicon_tags_22 },
    { 0xcbac6921, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcc6925, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08c6929, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6c692c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe84c692e, SUC_lexicon_tags_37 },
    { 0xdccc692f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0c6932, SUC_lexicon_tags_26 },
    { 0xed6c6933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48c6936, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acc6939, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4c6943, SUC_lexicon_tags_33 },
    { 0xe36c6943, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccc6947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x640c6949, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40c694c, SUC_lexicon_tags_49 },
    { 0x23ac694d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x332c694f, SUC_lexicon_tags_18 },
    { 0x2fac694f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aec6955, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31cc6957, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x98cc6959, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ec695d, SUC_lexicon_tags_61 },
    { 0xfaac695e, SUC_lexicon_tags_49 },
    { 0xb20c695f, SUC_lexicon_tags_29 },
    { 0x9a0c695f, SUC_lexicon_tags_14 },
    { 0x8a2c695e, SUC_lexicon_tags_25 },
    { 0x286c6962, SUC_lexicon_tags_15 },
    { 0xf80c6962, SUC_lexicon_tags_63 },
    { 0xc76c6960, SUC_lexicon_tags_32 },
    { 0xf6c6965, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1fcc6967, SUC_lexicon_tags_25 },
    { 0x160c6968, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6973, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06c6977, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cc697a, SUC_lexicon_tags_30 },
    { 0x448c697b, SUC_lexicon_tags_49 },
    { 0x84cc697a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cc6985, SUC_lexicon_tags_15 },
    { 0x8fec6985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cc698c, SUC_lexicon_tags_25 },
    { 0x520c698c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2aac698f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdc8c6991, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x538c6994, SUC_lexicon_tags_31 },
    { 0x402c6995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2c699a, SUC_lexicon_tags_27 },
    { 0x68cc699a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44c699e, SUC_lexicon_tags_37 },
    { 0x3bac699f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ec69a2, SUC_lexicon_tags_1 },
    { 0xb3ac69a3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78c69ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48c69ad, SUC_lexicon_tags_63 },
    { 0xfb6c69ae, SUC_lexicon_tags_38 },
    { 0xa52c69af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fec69b2, SUC_lexicon_tags_65 },
    { 0x860c69b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x358c69b6, SUC_lexicon_tags_35 },
    { 0x49ec69b7, SUC_lexicon_tags_38 },
    { 0x888c69b6, SUC_lexicon_tags_27 },
    { 0xd2c69b9, SUC_lexicon_tags_15 },
    { 0xda6c69ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8c69be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x458c69c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94c69c8, SUC_lexicon_tags_25 },
    { 0x93cc69c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508c69d4, SUC_lexicon_tags_46 },
    { 0x536c69d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb14c69d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cc69dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ec69e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cc69e6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7d4c69e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dac69eb, SUC_lexicon_tags_11 },
    { 0xaccc69eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ec69f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd62c69f5, SUC_lexicon_tags_60 },
    { 0xf06c69f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10c69f9, SUC_lexicon_tags_63 },
    { 0x6dec69fa, SUC_lexicon_tags_2 },
    { 0x554c69fb, SUC_lexicon_tags_68 },
    { 0xec6c69fc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe56c69fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cc6a01, SUC_lexicon_tags_45 },
    { 0x11ec6a02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56c6a05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb66c6a07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x864c6a0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cec6a0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18c6a14, SUC_lexicon_tags_11 },
    { 0x1c6c6a14, SUC_lexicon_tags_2 },
    { 0x946c6a16, SUC_lexicon_tags_11 },
    { 0xb20c6a17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd26c6a19, SUC_lexicon_tags_87 },
    { 0xec4c6a19, SUC_lexicon_tags_25 },
    { 0x8cac6a1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb62c6a1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1ac6a1f, SUC_lexicon_tags_17 },
    { 0xedec6a20, SUC_lexicon_tags_6 },
    { 0xcbac6a1f, SUC_lexicon_tags_4 },
    { 0xa58c6a22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x14cc6a24, SUC_lexicon_tags_2 },
    { 0x2c6c6a25, SUC_lexicon_tags_6 },
    { 0x20ac6a26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe6c6a28, SUC_lexicon_tags_7 },
    { 0x78ac6a29, SUC_lexicon_tags_27 },
    { 0xaa2c6a29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5d0c6a2c, SUC_lexicon_tags_15 },
    { 0xb44c6a2d, SUC_lexicon_tags_17 },
    { 0x4d8c6a2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d2c6a30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136c6a35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa50c6a37, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ac6a3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x706c6a41, SUC_lexicon_tags_15 },
    { 0xd6ac6a42, SUC_lexicon_tags_14 },
    { 0x322c6a42, SUC_lexicon_tags_12 },
    { 0x79ac6a44, SUC_lexicon_tags_12 },
    { 0x8c4c6a43, SUC_lexicon_tags_1 },
    { 0xbe8c6a46, SUC_lexicon_tags_37 },
    { 0xefac6a44, SUC_lexicon_tags_56 },
    { 0x132c6a45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138c6a4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xecec6a4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4c6a53, SUC_lexicon_tags_7 },
    { 0xe78c6a54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4ac6a56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98c6a5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58c6a5e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc62c6a60, SUC_lexicon_tags_14 },
    { 0x2ccc6a61, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x94c6a63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x74cc6a66, SUC_lexicon_tags_17 },
    { 0x290c6a66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x550c6a69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96c6a6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f2c6a71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2cc6a73, SUC_lexicon_tags_6 },
    { 0xf0c6a74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83ec6a76, SUC_lexicon_tags_47 },
    { 0xcdac6a77, SUC_lexicon_tags_27 },
    { 0xe6ac6a77, SUC_lexicon_tags_47 },
    { 0xb70c6a79, SUC_lexicon_tags_7 },
    { 0x526c6a7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2c6a80, SUC_lexicon_tags_61 },
    { 0xa86c6a81, SUC_lexicon_tags_87 },
    { 0xefac6a82, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78c6a85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8c6a88, SUC_lexicon_tags_25 },
    { 0xe9ac6a89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fac6a8b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cc6a92, SUC_lexicon_tags_26 },
    { 0x7c8c6a93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16c6a9b, SUC_lexicon_tags_55 },
    { 0x94ec6a9c, SUC_lexicon_tags_31 },
    { 0x91cc6a9d, SUC_lexicon_tags_38 },
    { 0x76ac6a9e, SUC_lexicon_tags_1 },
    { 0xf2c6a9e, SUC_lexicon_tags_11 },
    { 0x4b0c6aa0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cc6aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48cc6aa8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x994c6aaa, SUC_lexicon_tags_14 },
    { 0x198c6aab, SUC_lexicon_tags_15 },
    { 0x5e8c6aab, SUC_lexicon_tags_37 },
    { 0xff2c6aac, SUC_lexicon_tags_10 },
    { 0x790c6aac, SUC_lexicon_tags_2 },
    { 0x9c8c6aaf, SUC_lexicon_tags_19 },
    { 0x5eac6aaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dec6ab2, SUC_lexicon_tags_29 },
    { 0xe1cc6ab3, SUC_lexicon_tags_30 },
    { 0x98c6ab4, SUC_lexicon_tags_6 },
    { 0xbe4c6ab5, SUC_lexicon_tags_2 },
    { 0x458c6ab6, SUC_lexicon_tags_6 },
    { 0x57ac6ab5, SUC_lexicon_tags_14 },
    { 0x34ec6ab6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab0c6aba, SUC_lexicon_tags_7 },
    { 0x666c6abb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ac6ac2, SUC_lexicon_tags_37 },
    { 0xc3cc6ac3, SUC_lexicon_tags_37 },
    { 0x330c6ac3, SUC_lexicon_tags_25 },
    { 0x826c6ac5, SUC_lexicon_tags_32 },
    { 0x63ec6ac6, SUC_lexicon_tags_25 },
    { 0x960c6ac7, SUC_lexicon_tags_19 },
    { 0xbfcc6ac2, SUC_lexicon_tags_11 },
    { 0x75cc6ac9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x632c6acc, SUC_lexicon_tags_2 },
    { 0x684c6acc, SUC_lexicon_tags_17 },
    { 0xafec6acd, SUC_lexicon_tags_47 },
    { 0x776c6acd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6ada, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x950c6adc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928c6ae0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ec6ae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ec6ae9, SUC_lexicon_tags_26 },
    { 0x8d0c6aea, SUC_lexicon_tags_25 },
    { 0x85ec6aeb, SUC_lexicon_tags_11 },
    { 0xd94c6aec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60c6aef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c6af2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ec6af8, SUC_lexicon_tags_37 },
    { 0xe9cc6af9, SUC_lexicon_tags_2 },
    { 0xb8cc6afa, SUC_lexicon_tags_657 },
    { 0x86cc6afb, SUC_lexicon_tags_15 },
    { 0xfb4c6afa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x546c6aff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ec6b09, SUC_lexicon_tags_28 },
    { 0x384c6b0a, SUC_lexicon_tags_17 },
    { 0xb92c6b0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x43ac6b0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f4c6b0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cc6b13, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac6b1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ac6b20, SUC_lexicon_tags_1 },
    { 0x76ac6b20, SUC_lexicon_tags_16 },
    { 0xb5ac6b20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x360c6b25, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1d2c6b27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc26c6b29, SUC_lexicon_tags_17 },
    { 0x782c6b2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc44c6b2c, SUC_lexicon_tags_25 },
    { 0xd32c6b2d, SUC_lexicon_tags_7 },
    { 0xde8c6b2d, SUC_lexicon_tags_25 },
    { 0xfbac6b2c, SUC_lexicon_tags_55 },
    { 0x250c6b30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272c6b36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88c6b3e, SUC_lexicon_tags_9 },
    { 0x9fcc6b3f, SUC_lexicon_tags_37 },
    { 0xfaec6b3f, SUC_lexicon_tags_45 },
    { 0x3c8c6b41, SUC_lexicon_tags_64 },
    { 0x2d4c6b42, SUC_lexicon_tags_31 },
    { 0x850c6b43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ac6b46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x96ac6b48, SUC_lexicon_tags_25 },
    { 0x650c6b48, SUC_lexicon_tags_2 },
    { 0xb48c6b4a, SUC_lexicon_tags_131 },
    { 0x47ec6b4b, SUC_lexicon_tags_62 },
    { 0x7a8c6b4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x278c6b4e, SUC_lexicon_tags_25 },
    { 0x75cc6b4f, SUC_lexicon_tags_12 },
    { 0x1b2c6b50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x26ac6b52, SUC_lexicon_tags_7 },
    { 0xd4cc6b52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x918c6b55, SUC_lexicon_tags_47 },
    { 0x8d2c6b55, SUC_lexicon_tags_15 },
    { 0xe94c6b57, SUC_lexicon_tags_2 },
    { 0x3c8c6b58, SUC_lexicon_tags_42 },
    { 0x5ccc6b59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7cc6b5b, SUC_lexicon_tags_1 },
    { 0xbf2c6b5c, SUC_lexicon_tags_30 },
    { 0xd58c6b5d, SUC_lexicon_tags_25 },
    { 0x760c6b5d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8e2c6b60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692c6b64, SUC_lexicon_tags_60 },
    { 0x2d4c6b64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd44c6b67, SUC_lexicon_tags_83 },
    { 0x9aec6b68, SUC_lexicon_tags_1 },
    { 0xbdac6b68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeec6b6c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2c6b70, SUC_lexicon_tags_17 },
    { 0xa0cc6b70, SUC_lexicon_tags_0 },
    { 0x60cc6b72, SUC_lexicon_tags_17 },
    { 0x376c6b73, SUC_lexicon_tags_134 },
    { 0x5f4c6b74, SUC_lexicon_tags_17 },
    { 0xaa0c6b75, SUC_lexicon_tags_27 },
    { 0xcaec6b75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cc6b7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52c6b7f, SUC_lexicon_tags_168 },
    { 0, NULL },
    { 0x2f0c6b81, SUC_lexicon_tags_45 },
    { 0x6f8c6b82, SUC_lexicon_tags_12 },
    { 0x14ac6b82, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ec6b87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64c6b8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ac6b8d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x42c6b8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6c6b92, SUC_lexicon_tags_11 },
    { 0xf08c6b93, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x842c6b95, SUC_lexicon_tags_27 },
    { 0x966c6b96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558c6b9a, SUC_lexicon_tags_11 },
    { 0x992c6b9a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426c6b9f, SUC_lexicon_tags_14 },
    { 0xa20c6ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bec6ba2, SUC_lexicon_tags_30 },
    { 0xd14c6ba2, SUC_lexicon_tags_1 },
    { 0x9fac6ba2, SUC_lexicon_tags_71 },
    { 0x610c6ba5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefac6ba9, SUC_lexicon_tags_23 },
    { 0x6f8c6ba9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc54c6bac, SUC_lexicon_tags_15 },
    { 0x64c6bad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96c6bb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cc6bba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ec6bc0, SUC_lexicon_tags_63 },
    { 0x6cec6bc1, SUC_lexicon_tags_38 },
    { 0xdf6c6bc0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ec6bc7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xebec6bca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc2cc6bcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cec6bce, SUC_lexicon_tags_11 },
    { 0x1bcc6bcf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x808c6bd1, SUC_lexicon_tags_9 },
    { 0xa98c6bd2, SUC_lexicon_tags_47 },
    { 0x59ac6bd3, SUC_lexicon_tags_15 },
    { 0x4dac6bd3, SUC_lexicon_tags_1 },
    { 0x5fec6bd1, SUC_lexicon_tags_1 },
    { 0x782c6bd1, SUC_lexicon_tags_37 },
    { 0xdfcc6bd1, SUC_lexicon_tags_17 },
    { 0xa5ac6bd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cc6bdc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x44ec6bde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x408c6be1, SUC_lexicon_tags_2 },
    { 0xcbec6be1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcc6be5, SUC_lexicon_tags_11 },
    { 0xc0ec6be6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x840c6be8, SUC_lexicon_tags_282 },
    { 0x692c6be9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ec6bf1, SUC_lexicon_tags_38 },
    { 0xf44c6bf2, SUC_lexicon_tags_12 },
    { 0x7aec6bf2, SUC_lexicon_tags_7 },
    { 0xb82c6bf4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac6bfb, SUC_lexicon_tags_49 },
    { 0xd10c6bfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cc6c01, SUC_lexicon_tags_32 },
    { 0xa32c6c02, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x84c6c04, SUC_lexicon_tags_7 },
    { 0xbb6c6c04, SUC_lexicon_tags_4 },
    { 0x55ac6c06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd98c6c08, SUC_lexicon_tags_36 },
    { 0x428c6c09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90ec6c0b, SUC_lexicon_tags_17 },
    { 0xe1cc6c0c, SUC_lexicon_tags_33 },
    { 0xb36c6c0d, SUC_lexicon_tags_11 },
    { 0x30ec6c0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x234c6c10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720c6c20, SUC_lexicon_tags_33 },
    { 0x532c6c20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x592c6c23, SUC_lexicon_tags_37 },
    { 0xa80c6c24, SUC_lexicon_tags_15 },
    { 0x8ccc6c25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ecc6c27, SUC_lexicon_tags_31 },
    { 0xf00c6c28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0c6c2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ac6c31, SUC_lexicon_tags_53 },
    { 0xbb0c6c31, SUC_lexicon_tags_14 },
    { 0x194c6c33, SUC_lexicon_tags_33 },
    { 0xfcec6c34, SUC_lexicon_tags_10 },
    { 0xfac6c32, SUC_lexicon_tags_49 },
    { 0x93ac6c32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ac6c39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ec6c3c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x876c6c3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec6c42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc66c6c44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x638c6c47, SUC_lexicon_tags_29 },
    { 0x54ac6c48, SUC_lexicon_tags_87 },
    { 0x6cc6c49, SUC_lexicon_tags_11 },
    { 0xf0ac6c4a, SUC_lexicon_tags_26 },
    { 0xacc6c49, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8bec6c4d, SUC_lexicon_tags_2 },
    { 0x7a2c6c4e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6c6c6c50, SUC_lexicon_tags_10 },
    { 0x5c8c6c51, SUC_lexicon_tags_6 },
    { 0x402c6c51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2c6c5c, SUC_lexicon_tags_15 },
    { 0xd8c6c5c, SUC_lexicon_tags_69 },
    { 0xc9ec6c5e, SUC_lexicon_tags_38 },
    { 0x2f8c6c5c, SUC_lexicon_tags_17 },
    { 0x196c6c5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x494c6c62, SUC_lexicon_tags_38 },
    { 0x780c6c63, SUC_lexicon_tags_8 },
    { 0x98c6c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9dec6c66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa10c6c68, SUC_lexicon_tags_26 },
    { 0xec6c6c69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x766c6c6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0ac6c6d, SUC_lexicon_tags_25 },
    { 0x720c6c6d, SUC_lexicon_tags_1 },
    { 0x89ac6c6d, SUC_lexicon_tags_15 },
    { 0x914c6c70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c6c74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x320c6c77, SUC_lexicon_tags_33 },
    { 0x3cec6c78, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x428c6c7b, SUC_lexicon_tags_19 },
    { 0x892c6c7c, SUC_lexicon_tags_27 },
    { 0x5f8c6c7d, SUC_lexicon_tags_1 },
    { 0xd10c6c7b, SUC_lexicon_tags_77 },
    { 0x76ac6c7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe8c6c81, SUC_lexicon_tags_48 },
    { 0x44cc6c82, SUC_lexicon_tags_12 },
    { 0xbc4c6c83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe5ec6c85, SUC_lexicon_tags_15 },
    { 0x9f8c6c86, SUC_lexicon_tags_37 },
    { 0x89ec6c87, SUC_lexicon_tags_35 },
    { 0x2c8c6c87, SUC_lexicon_tags_14 },
    { 0x788c6c89, SUC_lexicon_tags_37 },
    { 0x724c6c8a, SUC_lexicon_tags_37 },
    { 0x2f0c6c89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x864c6c8d, SUC_lexicon_tags_27 },
    { 0x2a6c6c8e, SUC_lexicon_tags_27 },
    { 0xc50c6c8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66c6c92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74cc6c94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcc6c96, SUC_lexicon_tags_84 },
    { 0x48ec6c97, SUC_lexicon_tags_31 },
    { 0xbf0c6c98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aec6c9e, SUC_lexicon_tags_1 },
    { 0x1bcc6c9f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6c6ca2, SUC_lexicon_tags_11 },
    { 0x93cc6ca2, SUC_lexicon_tags_37 },
    { 0xca2c6ca4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb00c6ca6, SUC_lexicon_tags_47 },
    { 0x148c6ca7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0c6caa, SUC_lexicon_tags_33 },
    { 0xbbac6cab, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8c6cb0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58c6cb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c6cba, SUC_lexicon_tags_13 },
    { 0x706c6cbb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x992c6cbd, SUC_lexicon_tags_12 },
    { 0x8a8c6cbd, SUC_lexicon_tags_49 },
    { 0x650c6cbf, SUC_lexicon_tags_63 },
    { 0xc36c6cbe, SUC_lexicon_tags_2 },
    { 0xe8ec6cbd, SUC_lexicon_tags_23 },
    { 0x2d2c6cc2, SUC_lexicon_tags_9 },
    { 0x988c6cc2, SUC_lexicon_tags_38 },
    { 0x702c6cc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948c6ccb, SUC_lexicon_tags_25 },
    { 0xa0c6ccb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbfac6cce, SUC_lexicon_tags_25 },
    { 0x3f8c6ccf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4c6cd3, SUC_lexicon_tags_5 },
    { 0xec0c6cd4, SUC_lexicon_tags_27 },
    { 0xe48c6cd5, SUC_lexicon_tags_87 },
    { 0x890c6cd6, SUC_lexicon_tags_37 },
    { 0xd84c6cd3, SUC_lexicon_tags_12 },
    { 0x2bac6cd8, SUC_lexicon_tags_53 },
    { 0xb68c6cd9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ec6cdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf60c6cdf, SUC_lexicon_tags_2 },
    { 0x3b4c6ce0, SUC_lexicon_tags_77 },
    { 0xec0c6ce1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ac6ce4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ac6ce8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0c6ceb, SUC_lexicon_tags_25 },
    { 0x10cc6cec, SUC_lexicon_tags_545 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cc6cf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4c6cfc, SUC_lexicon_tags_1 },
    { 0xaaac6cfc, SUC_lexicon_tags_11 },
    { 0xb0ec6cfe, SUC_lexicon_tags_1 },
    { 0x42c6cfe, SUC_lexicon_tags_12 },
    { 0x4d2c6cff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406c6d04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x68ec6d06, SUC_lexicon_tags_37 },
    { 0x612c6d07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ac6d0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c8c6d0d, SUC_lexicon_tags_17 },
    { 0x214c6d0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4c6d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ec6d17, SUC_lexicon_tags_12 },
    { 0x9c2c6d18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x518c6d1b, SUC_lexicon_tags_13 },
    { 0xbc0c6d1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ac6d21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74c6d24, SUC_lexicon_tags_37 },
    { 0x72cc6d25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102c6d2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28c6d32, SUC_lexicon_tags_10 },
    { 0x3f8c6d33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f2c6d35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72c6d38, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c6d3b, SUC_lexicon_tags_16 },
    { 0x742c6d3c, SUC_lexicon_tags_15 },
    { 0xc6ec6d3d, SUC_lexicon_tags_55 },
    { 0xe30c6d3e, SUC_lexicon_tags_63 },
    { 0x768c6d3f, SUC_lexicon_tags_15 },
    { 0x6c2c6d3d, SUC_lexicon_tags_15 },
    { 0x882c6d41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe3cc6d43, SUC_lexicon_tags_43 },
    { 0x3c2c6d44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51ec6d46, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x10ec6d48, SUC_lexicon_tags_37 },
    { 0x20c6d49, SUC_lexicon_tags_0 },
    { 0xb78c6d4a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c6d4e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cc6d51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe18c6d53, SUC_lexicon_tags_49 },
    { 0x818c6d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x232c6d57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38c6d5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x496c6d60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16c6d63, SUC_lexicon_tags_25 },
    { 0xbaec6d64, SUC_lexicon_tags_11 },
    { 0x53cc6d65, SUC_lexicon_tags_14 },
    { 0x744c6d65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88c6d6c, SUC_lexicon_tags_27 },
    { 0xa94c6d6d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x602c6d6f, SUC_lexicon_tags_0 },
    { 0x8d0c6d70, SUC_lexicon_tags_79 },
    { 0x6ec6d71, SUC_lexicon_tags_17 },
    { 0x58c6d6f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc6cc6d74, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2c6d77, SUC_lexicon_tags_15 },
    { 0x714c6d77, SUC_lexicon_tags_79 },
    { 0x468c6d77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeac6d7b, SUC_lexicon_tags_0 },
    { 0xbccc6d7b, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xa4cc6d7e, SUC_lexicon_tags_15 },
    { 0x4cc6d7e, SUC_lexicon_tags_14 },
    { 0x1e2c6d7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36c6d83, SUC_lexicon_tags_10 },
    { 0x176c6d83, SUC_lexicon_tags_21 },
    { 0xe22c6d85, SUC_lexicon_tags_49 },
    { 0xb0ac6d85, SUC_lexicon_tags_38 },
    { 0x6ac6d87, SUC_lexicon_tags_12 },
    { 0x95ac6d85, SUC_lexicon_tags_17 },
    { 0x17ec6d89, SUC_lexicon_tags_37 },
    { 0xac4c6d85, SUC_lexicon_tags_1 },
    { 0xb2ac6d8b, SUC_lexicon_tags_63 },
    { 0x122c6d8c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2c6d91, SUC_lexicon_tags_14 },
    { 0xae6c6d92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70c6d95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56c6d98, SUC_lexicon_tags_45 },
    { 0xf38c6d99, SUC_lexicon_tags_2 },
    { 0x64ec6d9a, SUC_lexicon_tags_38 },
    { 0x336c6d9b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cc6d9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x292c6da0, SUC_lexicon_tags_13 },
    { 0xcfcc6da1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd68c6da3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf10c6da5, SUC_lexicon_tags_49 },
    { 0xeb8c6da6, SUC_lexicon_tags_10 },
    { 0xf90c6da7, SUC_lexicon_tags_10 },
    { 0x34ec6da6, SUC_lexicon_tags_82 },
    { 0x682c6da8, SUC_lexicon_tags_2 },
    { 0xafac6daa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x144c6dac, SUC_lexicon_tags_47 },
    { 0x9a8c6dac, SUC_lexicon_tags_47 },
    { 0xc26c6dae, SUC_lexicon_tags_29 },
    { 0xf0ec6daf, SUC_lexicon_tags_30 },
    { 0x5aec6daf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0c6db7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40c6dbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c6dc2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cc6dc6, SUC_lexicon_tags_8 },
    { 0x1bac6dc7, SUC_lexicon_tags_25 },
    { 0x116c6dc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaec6dcb, SUC_lexicon_tags_7 },
    { 0x3bec6dcc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ec6dcf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762c6dd4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8c6dd7, SUC_lexicon_tags_48 },
    { 0xa26c6dd7, SUC_lexicon_tags_2 },
    { 0xf44c6dd9, SUC_lexicon_tags_37 },
    { 0x82c6dda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dec6ddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0c6de0, SUC_lexicon_tags_12 },
    { 0x45ec6de1, SUC_lexicon_tags_33 },
    { 0x738c6de2, SUC_lexicon_tags_25 },
    { 0x558c6de0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b8c6de5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0c6de9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836c6ded, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeac6df2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x84ac6df4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81ac6df6, SUC_lexicon_tags_26 },
    { 0xad0c6df6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdfcc6df9, SUC_lexicon_tags_11 },
    { 0x320c6dfa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56c6dfe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5fcc6e00, SUC_lexicon_tags_26 },
    { 0x71cc6e00, SUC_lexicon_tags_25 },
    { 0xf88c6e02, SUC_lexicon_tags_87 },
    { 0xeec6e02, SUC_lexicon_tags_17 },
    { 0x652c6e00, SUC_lexicon_tags_33 },
    { 0xb9ac6e02, SUC_lexicon_tags_0 },
    { 0xe38c6e04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10cc6e08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0c6e0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6c6e0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c4c6e11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a2c6e13, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248c6e19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cec6e1d, SUC_lexicon_tags_14 },
    { 0x60ac6e1e, SUC_lexicon_tags_7 },
    { 0x5dcc6e1f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776c6e23, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x178c6e26, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcc6e29, SUC_lexicon_tags_5 },
    { 0x392c6e29, SUC_lexicon_tags_38 },
    { 0xceac6e2b, SUC_lexicon_tags_38 },
    { 0x732c6e2b, SUC_lexicon_tags_12 },
    { 0x45cc6e2d, SUC_lexicon_tags_25 },
    { 0x8e6c6e2e, SUC_lexicon_tags_25 },
    { 0xdb8c6e2c, SUC_lexicon_tags_39 },
    { 0x596c6e2b, SUC_lexicon_tags_25 },
    { 0xad2c6e31, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb08c6e33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x83cc6e36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80c6e3a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eac6e44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82c6e47, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850c6e4c, SUC_lexicon_tags_17 },
    { 0xd06c6e4c, SUC_lexicon_tags_28 },
    { 0x42c6e4e, SUC_lexicon_tags_25 },
    { 0x28cc6e4e, SUC_lexicon_tags_10 },
    { 0x81ec6e4e, SUC_lexicon_tags_19 },
    { 0xab2c6e4d, SUC_lexicon_tags_7 },
    { 0xdc6c6e52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c6e55, SUC_lexicon_tags_25 },
    { 0x4eec6e56, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xb98c6e58, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4c6e60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1cc6e62, SUC_lexicon_tags_91 },
    { 0x644c6e63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10c6e67, SUC_lexicon_tags_14 },
    { 0x2bcc6e68, SUC_lexicon_tags_17 },
    { 0x82cc6e68, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cc6e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ac6e6f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x282c6e71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488c6e75, SUC_lexicon_tags_25 },
    { 0x7f8c6e75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4c6e7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0c6e7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ec6e84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ec6e88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdec6e91, SUC_lexicon_tags_32 },
    { 0x94ec6e92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4c6e98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x846c6e9b, SUC_lexicon_tags_33 },
    { 0x8bcc6e9b, SUC_lexicon_tags_9 },
    { 0xae2c6e9d, SUC_lexicon_tags_60 },
    { 0xf26c6e9e, SUC_lexicon_tags_12 },
    { 0x75cc6e9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72c6ea2, SUC_lexicon_tags_2 },
    { 0x702c6ea3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3acc6ea5, SUC_lexicon_tags_27 },
    { 0x44c6ea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaec6ea8, SUC_lexicon_tags_7 },
    { 0xc00c6ea8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2c6eae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82c6eb2, SUC_lexicon_tags_23 },
    { 0x1f8c6eb3, SUC_lexicon_tags_47 },
    { 0xf36c6eb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614c6eba, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecc6ec0, SUC_lexicon_tags_29 },
    { 0x4e2c6ec1, SUC_lexicon_tags_11 },
    { 0xf3cc6ec2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e8c6ec4, SUC_lexicon_tags_30 },
    { 0x35ac6ec5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770c6ec9, SUC_lexicon_tags_48 },
    { 0xcc6eca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8c6ece, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8c6ed2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x87ec6ed4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ac6ed7, SUC_lexicon_tags_7 },
    { 0xb2cc6ed8, SUC_lexicon_tags_23 },
    { 0x306c6ed8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6c6edc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6c6ee0, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16c6ee3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2c6ee6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa6c6ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42c6eec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x850c6eee, SUC_lexicon_tags_27 },
    { 0x4c6eef, SUC_lexicon_tags_15 },
    { 0xb42c6ef0, SUC_lexicon_tags_12 },
    { 0x604c6ef1, SUC_lexicon_tags_11 },
    { 0xe1ac6ef2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252c6ef6, SUC_lexicon_tags_21 },
    { 0x99ac6ef7, SUC_lexicon_tags_17 },
    { 0x40ec6ef7, SUC_lexicon_tags_39 },
    { 0xec0c6ef8, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x5e2c6efb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4c6efe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78c6f01, SUC_lexicon_tags_27 },
    { 0x9a0c6f02, SUC_lexicon_tags_15 },
    { 0xb6ac6f03, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cc6f07, SUC_lexicon_tags_82 },
    { 0x560c6f08, SUC_lexicon_tags_49 },
    { 0x2a6c6f09, SUC_lexicon_tags_16 },
    { 0xe66c6f07, SUC_lexicon_tags_26 },
    { 0xee8c6f0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8c6f0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x194c6f11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2ec6f13, SUC_lexicon_tags_37 },
    { 0x72ac6f14, SUC_lexicon_tags_12 },
    { 0xc6c6f15, SUC_lexicon_tags_21 },
    { 0x87ac6f16, SUC_lexicon_tags_25 },
    { 0x324c6f16, SUC_lexicon_tags_28 },
    { 0x84ac6f18, SUC_lexicon_tags_7 },
    { 0xf74c6f19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4c6f1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4ac6f1e, SUC_lexicon_tags_10 },
    { 0x18cc6f1f, SUC_lexicon_tags_1 },
    { 0xc98c6f1f, SUC_lexicon_tags_1 },
    { 0xcb8c6f21, SUC_lexicon_tags_6 },
    { 0x43ec6f22, SUC_lexicon_tags_68 },
    { 0x23ec6f22, SUC_lexicon_tags_28 },
    { 0x6e0c6f24, SUC_lexicon_tags_30 },
    { 0x406c6f22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac6f2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7ac6f2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76c6f31, SUC_lexicon_tags_1 },
    { 0x322c6f32, SUC_lexicon_tags_23 },
    { 0xb42c6f33, SUC_lexicon_tags_7 },
    { 0xb44c6f31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x860c6f36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2c6f3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0c6f40, SUC_lexicon_tags_11 },
    { 0x3d2c6f41, SUC_lexicon_tags_77 },
    { 0xeb4c6f42, SUC_lexicon_tags_23 },
    { 0x9c0c6f42, SUC_lexicon_tags_11 },
    { 0xd1cc6f43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0c6f47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfec6f49, SUC_lexicon_tags_25 },
    { 0xf64c6f4a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76c6f4d, SUC_lexicon_tags_29 },
    { 0x246c6f4e, SUC_lexicon_tags_6 },
    { 0x6f4c6f4f, SUC_lexicon_tags_15 },
    { 0xad2c6f4f, SUC_lexicon_tags_26 },
    { 0xb94c6f51, SUC_lexicon_tags_17 },
    { 0xa44c6f51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x684c6f54, SUC_lexicon_tags_37 },
    { 0x9eec6f55, SUC_lexicon_tags_14 },
    { 0x5b4c6f54, SUC_lexicon_tags_25 },
    { 0x386c6f57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cac6f5c, SUC_lexicon_tags_84 },
    { 0xe10c6f5d, SUC_lexicon_tags_68 },
    { 0xe6ac6f5d, SUC_lexicon_tags_2 },
    { 0x3f4c6f5f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaec6f66, SUC_lexicon_tags_35 },
    { 0x51ac6f67, SUC_lexicon_tags_12 },
    { 0xb74c6f68, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ac6f6b, SUC_lexicon_tags_12 },
    { 0x4b8c6f6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf96c6f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd8c6f70, SUC_lexicon_tags_201 },
    { 0xff0c6f71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8c6f77, SUC_lexicon_tags_12 },
    { 0x2c2c6f78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7ec6f7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc6f7e, SUC_lexicon_tags_82 },
    { 0x218c6f7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0c6f82, SUC_lexicon_tags_3 },
    { 0x3a2c6f83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc4c6f85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x514c6f88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x75cc6f8b, SUC_lexicon_tags_14 },
    { 0xc2ac6f8c, SUC_lexicon_tags_37 },
    { 0x7bec6f8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80c6f8f, SUC_lexicon_tags_151 },
    { 0x800c6f8f, SUC_lexicon_tags_60 },
    { 0x922c6f91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabac6f94, SUC_lexicon_tags_14 },
    { 0x112c6f94, SUC_lexicon_tags_17 },
    { 0xfd6c6f96, SUC_lexicon_tags_9 },
    { 0x454c6f97, SUC_lexicon_tags_26 },
    { 0xbfec6f98, SUC_lexicon_tags_38 },
    { 0x5aac6f97, SUC_lexicon_tags_21 },
    { 0xb0ac6f98, SUC_lexicon_tags_29 },
    { 0x8f2c6f9b, SUC_lexicon_tags_43 },
    { 0xb16c6f9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xefcc6f9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4c6fa5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26c6fa8, SUC_lexicon_tags_60 },
    { 0xbbec6fa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x638c6fac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2c6faf, SUC_lexicon_tags_2 },
    { 0xcb0c6fb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452c6fb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb14c6fb6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb00c6fb8, SUC_lexicon_tags_29 },
    { 0x456c6fb8, SUC_lexicon_tags_17 },
    { 0x3fcc6fb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98c6fbc, SUC_lexicon_tags_67 },
    { 0x87cc6fbd, SUC_lexicon_tags_28 },
    { 0x5d0c6fbd, SUC_lexicon_tags_37 },
    { 0xaa0c6fbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x666c6fc1, SUC_lexicon_tags_28 },
    { 0xa3cc6fc2, SUC_lexicon_tags_45 },
    { 0x882c6fc2, SUC_lexicon_tags_14 },
    { 0x2bec6fc2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfa6c6fc6, SUC_lexicon_tags_7 },
    { 0x82cc6fc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6c6fce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dac6fd3, SUC_lexicon_tags_26 },
    { 0x650c6fd4, SUC_lexicon_tags_45 },
    { 0x75ac6fd4, SUC_lexicon_tags_25 },
    { 0x8cec6fd6, SUC_lexicon_tags_23 },
    { 0xc32c6fd7, SUC_lexicon_tags_27 },
    { 0x48c6fd5, SUC_lexicon_tags_12 },
    { 0x1e0c6fd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a0c6fdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ec6fdf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18c6fe4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbec6fe6, SUC_lexicon_tags_104 },
    { 0x986c6fe7, SUC_lexicon_tags_7 },
    { 0xad6c6fe8, SUC_lexicon_tags_11 },
    { 0x88ec6fe9, SUC_lexicon_tags_25 },
    { 0x864c6fea, SUC_lexicon_tags_37 },
    { 0x942c6fea, SUC_lexicon_tags_7 },
    { 0xa2c6fe8, SUC_lexicon_tags_30 },
    { 0x722c6fed, SUC_lexicon_tags_29 },
    { 0x7cac6fee, SUC_lexicon_tags_71 },
    { 0x3eec6fef, SUC_lexicon_tags_26 },
    { 0xcacc6ff0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aac6ff4, SUC_lexicon_tags_20 },
    { 0x62cc6ff5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x850c6ff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb16c6ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x918c6ffb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6cc6ffd, SUC_lexicon_tags_33 },
    { 0xd20c6ffe, SUC_lexicon_tags_9 },
    { 0x14ec6ffe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532c7004, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6c700f, SUC_lexicon_tags_48 },
    { 0xddcc7010, SUC_lexicon_tags_2 },
    { 0x6f8c7011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108c7015, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x89ec7017, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cc701a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ec7020, SUC_lexicon_tags_87 },
    { 0x91cc7021, SUC_lexicon_tags_47 },
    { 0x838c7022, SUC_lexicon_tags_60 },
    { 0x36ac7023, SUC_lexicon_tags_1 },
    { 0x74c7023, SUC_lexicon_tags_2 },
    { 0xd1cc7025, SUC_lexicon_tags_27 },
    { 0xeb2c7026, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c702d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6c7033, SUC_lexicon_tags_1 },
    { 0xde0c7034, SUC_lexicon_tags_32 },
    { 0x294c7034, SUC_lexicon_tags_33 },
    { 0x670c7036, SUC_lexicon_tags_11 },
    { 0x26ac7035, SUC_lexicon_tags_11 },
    { 0x384c7035, SUC_lexicon_tags_66 },
    { 0x2f2c7037, SUC_lexicon_tags_33 },
    { 0xdc0c703a, SUC_lexicon_tags_47 },
    { 0x7d2c703b, SUC_lexicon_tags_29 },
    { 0xeec703c, SUC_lexicon_tags_12 },
    { 0x9f8c7034, SUC_lexicon_tags_379 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0c7043, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538c704a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540c704e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x528c7050, SUC_lexicon_tags_27 },
    { 0x8e0c7050, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x422c7053, SUC_lexicon_tags_15 },
    { 0x83cc7054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04c7057, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb92c7059, SUC_lexicon_tags_9 },
    { 0xafac705a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4c705d, SUC_lexicon_tags_12 },
    { 0x91ec705d, SUC_lexicon_tags_15 },
    { 0xd0ac705d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaec7064, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf7cc7066, SUC_lexicon_tags_49 },
    { 0xbbac7067, SUC_lexicon_tags_383 },
    { 0xa66c7068, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8c706b, SUC_lexicon_tags_37 },
    { 0xb8ac706c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60c7070, SUC_lexicon_tags_12 },
    { 0xc7ac7071, SUC_lexicon_tags_16 },
    { 0x8a2c7072, SUC_lexicon_tags_15 },
    { 0xa1ec7073, SUC_lexicon_tags_11 },
    { 0x4e8c7073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x806c7076, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66c707c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8c7080, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8c7086, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ac7089, SUC_lexicon_tags_17 },
    { 0xe72c708a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf6c708c, SUC_lexicon_tags_34 },
    { 0xd4ec708c, SUC_lexicon_tags_17 },
    { 0x6d8c708d, SUC_lexicon_tags_2 },
    { 0xd46c708f, SUC_lexicon_tags_7 },
    { 0xb20c708f, SUC_lexicon_tags_1 },
    { 0x478c7091, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6c7094, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874c7098, SUC_lexicon_tags_17 },
    { 0x834c7099, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88ac709b, SUC_lexicon_tags_87 },
    { 0x622c709b, SUC_lexicon_tags_0 },
    { 0xd24c709d, SUC_lexicon_tags_25 },
    { 0xea2c709e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec70a2, SUC_lexicon_tags_7 },
    { 0x728c70a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f0c70a5, SUC_lexicon_tags_60 },
    { 0xc06c70a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77cc70aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x466c70ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a4c70af, SUC_lexicon_tags_1 },
    { 0x3b0c70af, SUC_lexicon_tags_21 },
    { 0xca2c70b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fec70b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19ec70b8, SUC_lexicon_tags_37 },
    { 0x982c70b9, SUC_lexicon_tags_15 },
    { 0x17cc70ba, SUC_lexicon_tags_29 },
    { 0xbf6c70b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ac70be, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510c70c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cc70c8, SUC_lexicon_tags_15 },
    { 0x2c2c70c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cec70cd, SUC_lexicon_tags_16 },
    { 0x890c70ce, SUC_lexicon_tags_61 },
    { 0x7bec70ce, SUC_lexicon_tags_37 },
    { 0xd20c70d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434c70d4, SUC_lexicon_tags_14 },
    { 0x400c70d4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x47cc70d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60ec70d9, SUC_lexicon_tags_12 },
    { 0xe56c70da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffac70de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcc70e2, SUC_lexicon_tags_17 },
    { 0x3a2c70e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66c70e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4c70eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360c70f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa66c70f2, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x154c70f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496c70f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7a2c70fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294c7101, SUC_lexicon_tags_37 },
    { 0x4fac7102, SUC_lexicon_tags_31 },
    { 0x5eec7101, SUC_lexicon_tags_0 },
    { 0x658c7104, SUC_lexicon_tags_3 },
    { 0xfcc7105, SUC_lexicon_tags_17 },
    { 0x1dac7102, SUC_lexicon_tags_37 },
    { 0x42c7107, SUC_lexicon_tags_27 },
    { 0xc6c7108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x432c710a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9aac710c, SUC_lexicon_tags_38 },
    { 0x4e6c710c, SUC_lexicon_tags_37 },
    { 0xeeec710e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa0cc7110, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bec7117, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416c7120, SUC_lexicon_tags_45 },
    { 0x6b6c7121, SUC_lexicon_tags_37 },
    { 0xb48c7122, SUC_lexicon_tags_25 },
    { 0x8ecc7123, SUC_lexicon_tags_485 },
    { 0xa62c7120, SUC_lexicon_tags_6 },
    { 0x58ec7125, SUC_lexicon_tags_2 },
    { 0x536c7122, SUC_lexicon_tags_27 },
    { 0xcc4c7122, SUC_lexicon_tags_12 },
    { 0x5fcc7124, SUC_lexicon_tags_61 },
    { 0x7cc7129, SUC_lexicon_tags_83 },
    { 0xe6cc7129, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0c7132, SUC_lexicon_tags_1 },
    { 0x670c7133, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaec7138, SUC_lexicon_tags_15 },
    { 0x28ac7139, SUC_lexicon_tags_68 },
    { 0xcd6c713a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760c713e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x248c7141, SUC_lexicon_tags_30 },
    { 0xb96c7142, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ec7147, SUC_lexicon_tags_60 },
    { 0x99ec7147, SUC_lexicon_tags_27 },
    { 0x3eac7149, SUC_lexicon_tags_25 },
    { 0xdf8c7149, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8c8c714c, SUC_lexicon_tags_63 },
    { 0x274c714d, SUC_lexicon_tags_45 },
    { 0x1ec714d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8c7151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8c7155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2c7159, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ac715d, SUC_lexicon_tags_29 },
    { 0x5f8c715e, SUC_lexicon_tags_77 },
    { 0x90ec715e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26c7162, SUC_lexicon_tags_11 },
    { 0x262c7163, SUC_lexicon_tags_17 },
    { 0x208c7163, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450c7168, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d6c716a, SUC_lexicon_tags_37 },
    { 0x56cc716b, SUC_lexicon_tags_2 },
    { 0x140c716c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740c7170, SUC_lexicon_tags_182 },
    { 0x49ac7171, SUC_lexicon_tags_43 },
    { 0x930c7172, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0c7175, SUC_lexicon_tags_28 },
    { 0x4ecc7176, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9acc7178, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeec717b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x110c717e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4c7184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26ac7186, SUC_lexicon_tags_17 },
    { 0x250c7187, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cc718b, SUC_lexicon_tags_26 },
    { 0xdbec718b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172c7192, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dac7196, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2c7199, SUC_lexicon_tags_15 },
    { 0x99cc719a, SUC_lexicon_tags_2 },
    { 0x408c719b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71ec719d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbc8c719f, SUC_lexicon_tags_39 },
    { 0xf12c71a0, SUC_lexicon_tags_37 },
    { 0x392c71a1, SUC_lexicon_tags_15 },
    { 0x8aac71a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xddac71a5, SUC_lexicon_tags_49 },
    { 0xeaac71a6, SUC_lexicon_tags_17 },
    { 0xd9ac71a6, SUC_lexicon_tags_25 },
    { 0x1c0c71a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecc71ac, SUC_lexicon_tags_9 },
    { 0x270c71ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2c71b8, SUC_lexicon_tags_1 },
    { 0x642c71b9, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0x25ac71bb, SUC_lexicon_tags_25 },
    { 0xe86c71bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ec71c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x594c71c3, SUC_lexicon_tags_64 },
    { 0x1ecc71c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cc71c8, SUC_lexicon_tags_23 },
    { 0xc74c71c8, SUC_lexicon_tags_6 },
    { 0xbe2c71ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426c71d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7a6c71d2, SUC_lexicon_tags_2 },
    { 0x10c71d3, SUC_lexicon_tags_49 },
    { 0x8bec71d4, SUC_lexicon_tags_12 },
    { 0xda6c71d5, SUC_lexicon_tags_37 },
    { 0xdd8c71d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3ac71d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4c71dd, SUC_lexicon_tags_37 },
    { 0xeecc71dd, SUC_lexicon_tags_38 },
    { 0x556c71df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cc71e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60ec71e4, SUC_lexicon_tags_10 },
    { 0xa5ec71e5, SUC_lexicon_tags_77 },
    { 0x1b6c71e6, SUC_lexicon_tags_25 },
    { 0x962c71e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08c71ea, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2c71ee, SUC_lexicon_tags_29 },
    { 0x7b6c71ef, SUC_lexicon_tags_11 },
    { 0x824c71ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442c71f6, SUC_lexicon_tags_12 },
    { 0x5d8c71f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc2c71f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcc71fc, SUC_lexicon_tags_26 },
    { 0xb36c71fc, SUC_lexicon_tags_49 },
    { 0x9bac71fd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826c7202, SUC_lexicon_tags_1 },
    { 0xb50c7202, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834c7207, SUC_lexicon_tags_23 },
    { 0x650c7208, SUC_lexicon_tags_2 },
    { 0x8b6c7207, SUC_lexicon_tags_45 },
    { 0xee4c7208, SUC_lexicon_tags_12 },
    { 0x1c8c720b, SUC_lexicon_tags_1 },
    { 0xa74c720c, SUC_lexicon_tags_25 },
    { 0x2d6c720d, SUC_lexicon_tags_37 },
    { 0x320c720c, SUC_lexicon_tags_26 },
    { 0x80c720f, SUC_lexicon_tags_9 },
    { 0x734c720b, SUC_lexicon_tags_11 },
    { 0x9d8c7211, SUC_lexicon_tags_7 },
    { 0x72ec7207, SUC_lexicon_tags_14 },
    { 0x7f0c7213, SUC_lexicon_tags_11 },
    { 0x384c720c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7f2c7216, SUC_lexicon_tags_14 },
    { 0xb1ec7217, SUC_lexicon_tags_19 },
    { 0x7a2c7218, SUC_lexicon_tags_21 },
    { 0x86ec7219, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b2c721b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4c721f, SUC_lexicon_tags_692 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaec7225, SUC_lexicon_tags_12 },
    { 0xd7ec7226, SUC_lexicon_tags_25 },
    { 0x196c7227, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62c7230, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4c7233, SUC_lexicon_tags_32 },
    { 0xfbcc7233, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90c7237, SUC_lexicon_tags_49 },
    { 0xe40c7237, SUC_lexicon_tags_7 },
    { 0xa32c7237, SUC_lexicon_tags_12 },
    { 0x276c723a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0c723f, SUC_lexicon_tags_1 },
    { 0x46cc7240, SUC_lexicon_tags_4 },
    { 0xe90c7241, SUC_lexicon_tags_66 },
    { 0x63ac7240, SUC_lexicon_tags_0 },
    { 0x710c7240, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cec7249, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772c724f, SUC_lexicon_tags_43 },
    { 0x452c724f, SUC_lexicon_tags_15 },
    { 0xd0cc7251, SUC_lexicon_tags_37 },
    { 0xb70c7252, SUC_lexicon_tags_25 },
    { 0xaf8c7251, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738c725d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x836c725f, SUC_lexicon_tags_17 },
    { 0xa30c7260, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x948c7262, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x208c7265, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x376c7267, SUC_lexicon_tags_37 },
    { 0x792c7268, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x612c726a, SUC_lexicon_tags_25 },
    { 0x126c726b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6c7274, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68c7278, SUC_lexicon_tags_17 },
    { 0x9eac7279, SUC_lexicon_tags_17 },
    { 0x9d8c7279, SUC_lexicon_tags_17 },
    { 0x86ec727b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cc7280, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508c7284, SUC_lexicon_tags_10 },
    { 0x85cc7285, SUC_lexicon_tags_63 },
    { 0xdd0c7285, SUC_lexicon_tags_7 },
    { 0xf4cc7287, SUC_lexicon_tags_10 },
    { 0x7e0c7287, SUC_lexicon_tags_23 },
    { 0x96ec7289, SUC_lexicon_tags_2 },
    { 0xed2c7287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8c728d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xafcc728f, SUC_lexicon_tags_1 },
    { 0x99ec7290, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb00c7292, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44c7295, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848c729b, SUC_lexicon_tags_0 },
    { 0xff8c729b, SUC_lexicon_tags_17 },
    { 0xee2c729b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8c72a0, SUC_lexicon_tags_26 },
    { 0xddcc72a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aec72a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890c72a9, SUC_lexicon_tags_17 },
    { 0x5b0c72a9, SUC_lexicon_tags_12 },
    { 0x96c72ab, SUC_lexicon_tags_0 },
    { 0x936c72ac, SUC_lexicon_tags_2 },
    { 0xa8ac72ad, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0c72b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb2c72b3, SUC_lexicon_tags_21 },
    { 0xac4c72b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64c72b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4b2c72ba, SUC_lexicon_tags_34 },
    { 0x4ec72ba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdac72bd, SUC_lexicon_tags_5 },
    { 0x830c72bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac72c9, SUC_lexicon_tags_6 },
    { 0x14cc72ca, SUC_lexicon_tags_87 },
    { 0x9dcc72cb, SUC_lexicon_tags_12 },
    { 0x5aec72cc, SUC_lexicon_tags_37 },
    { 0x4dec72cd, SUC_lexicon_tags_15 },
    { 0xa9ac72ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4c72d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ac72d6, SUC_lexicon_tags_3 },
    { 0x312c72d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x296c72d9, SUC_lexicon_tags_30 },
    { 0xc08c72d9, SUC_lexicon_tags_12 },
    { 0x44ec72db, SUC_lexicon_tags_37 },
    { 0x324c72dc, SUC_lexicon_tags_7 },
    { 0x8d2c72da, SUC_lexicon_tags_11 },
    { 0x802c72dd, SUC_lexicon_tags_25 },
    { 0xdfec72df, SUC_lexicon_tags_1 },
    { 0x95cc72e0, SUC_lexicon_tags_31 },
    { 0xec2c72e1, SUC_lexicon_tags_45 },
    { 0x8fcc72e0, SUC_lexicon_tags_45 },
    { 0x6e4c72df, SUC_lexicon_tags_12 },
    { 0xd00c72e4, SUC_lexicon_tags_19 },
    { 0xc72c72e4, SUC_lexicon_tags_15 },
    { 0x894c72e6, SUC_lexicon_tags_15 },
    { 0x6aac72e7, SUC_lexicon_tags_11 },
    { 0x48c72d9, SUC_lexicon_tags_37 },
    { 0xd7ac72e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x430c72eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbaac72ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2c72f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0c72fd, SUC_lexicon_tags_25 },
    { 0xa28c72fe, SUC_lexicon_tags_30 },
    { 0xcc0c72fe, SUC_lexicon_tags_12 },
    { 0xdeac7300, SUC_lexicon_tags_43 },
    { 0xd38c72ff, SUC_lexicon_tags_33 },
    { 0x372c7302, SUC_lexicon_tags_37 },
    { 0x3fec7303, SUC_lexicon_tags_9 },
    { 0x53ac7304, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cc7308, SUC_lexicon_tags_46 },
    { 0xcf0c7309, SUC_lexicon_tags_17 },
    { 0x2acc7309, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x856c730d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac7310, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d0c7312, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96c7315, SUC_lexicon_tags_1 },
    { 0xf08c7316, SUC_lexicon_tags_6 },
    { 0x550c7317, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56c731a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ac731d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8c7323, SUC_lexicon_tags_17 },
    { 0xfc8c7323, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ac7327, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928c732b, SUC_lexicon_tags_23 },
    { 0x52ec732c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d2c732e, SUC_lexicon_tags_12 },
    { 0x842c732f, SUC_lexicon_tags_17 },
    { 0x41ac7330, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf50c7332, SUC_lexicon_tags_11 },
    { 0x63ac7332, SUC_lexicon_tags_37 },
    { 0x9d2c7334, SUC_lexicon_tags_2 },
    { 0xca0c7335, SUC_lexicon_tags_7 },
    { 0xdc0c7336, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0c733b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec733f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x418c7342, SUC_lexicon_tags_16 },
    { 0x86c7342, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x28ec7345, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc18c7348, SUC_lexicon_tags_17 },
    { 0x504c7349, SUC_lexicon_tags_30 },
    { 0x7bcc734a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22c734d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88cc734f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86c7352, SUC_lexicon_tags_12 },
    { 0xde0c7353, SUC_lexicon_tags_11 },
    { 0xc26c7353, SUC_lexicon_tags_2 },
    { 0xde8c7355, SUC_lexicon_tags_11 },
    { 0xfbac7354, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20c735a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaac7361, SUC_lexicon_tags_37 },
    { 0xdd8c7361, SUC_lexicon_tags_12 },
    { 0xeb4c7363, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99ec7365, SUC_lexicon_tags_0 },
    { 0xba8c7366, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60c736d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd94c736f, SUC_lexicon_tags_23 },
    { 0xbb6c7370, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294c7377, SUC_lexicon_tags_12 },
    { 0xb30c7378, SUC_lexicon_tags_37 },
    { 0xfcac7378, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8c737e, SUC_lexicon_tags_7 },
    { 0x188c737f, SUC_lexicon_tags_7 },
    { 0xa50c737f, SUC_lexicon_tags_46 },
    { 0xc84c7381, SUC_lexicon_tags_2 },
    { 0x5c2c7382, SUC_lexicon_tags_45 },
    { 0x7c8c7383, SUC_lexicon_tags_42 },
    { 0x62ac7384, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x518c7386, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80c7396, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaac739c, SUC_lexicon_tags_12 },
    { 0xac0c739d, SUC_lexicon_tags_25 },
    { 0xb2c739e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938c73a4, SUC_lexicon_tags_61 },
    { 0xca4c73a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74ac73a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cc73ab, SUC_lexicon_tags_129 },
    { 0x50cc73ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04c73b0, SUC_lexicon_tags_1 },
    { 0xa8ac73b1, SUC_lexicon_tags_43 },
    { 0xea6c73b1, SUC_lexicon_tags_23 },
    { 0xa8ec73b3, SUC_lexicon_tags_19 },
    { 0xa74c73b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52cc73b6, SUC_lexicon_tags_16 },
    { 0xbcec73b7, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x8e2c73b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6c73bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa4c73be, SUC_lexicon_tags_37 },
    { 0xa18c73be, SUC_lexicon_tags_62 },
    { 0xcc0c73c0, SUC_lexicon_tags_6 },
    { 0x73ec73bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x928c73c3, SUC_lexicon_tags_31 },
    { 0x632c73c3, SUC_lexicon_tags_2 },
    { 0xa28c73c5, SUC_lexicon_tags_23 },
    { 0x8c6c73c5, SUC_lexicon_tags_37 },
    { 0xdeec73c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x562c73c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9c4c73cb, SUC_lexicon_tags_60 },
    { 0x704c73cc, SUC_lexicon_tags_37 },
    { 0xb58c73cb, SUC_lexicon_tags_43 },
    { 0xda4c73ce, SUC_lexicon_tags_1 },
    { 0xceec73cc, SUC_lexicon_tags_7 },
    { 0x1f0c73d0, SUC_lexicon_tags_1 },
    { 0x2f2c73d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770c73d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xffcc73d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8c73db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf2c73dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd42c73df, SUC_lexicon_tags_4 },
    { 0x858c73df, SUC_lexicon_tags_61 },
    { 0x92cc73e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110c73e5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8c73ea, SUC_lexicon_tags_11 },
    { 0x7a4c73eb, SUC_lexicon_tags_7 },
    { 0xcb6c73ec, SUC_lexicon_tags_47 },
    { 0x44cc73ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c73f4, SUC_lexicon_tags_48 },
    { 0x9bec73f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984c73fa, SUC_lexicon_tags_31 },
    { 0xab2c73fa, SUC_lexicon_tags_15 },
    { 0xdf8c73fc, SUC_lexicon_tags_12 },
    { 0x1acc73fd, SUC_lexicon_tags_29 },
    { 0x4f8c73fe, SUC_lexicon_tags_63 },
    { 0x364c73fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776c7404, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26c7408, SUC_lexicon_tags_15 },
    { 0xf20c7409, SUC_lexicon_tags_63 },
    { 0x984c7409, SUC_lexicon_tags_17 },
    { 0xfd8c740a, SUC_lexicon_tags_19 },
    { 0xf5ec7408, SUC_lexicon_tags_65 },
    { 0x4bcc740d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x880c740f, SUC_lexicon_tags_3 },
    { 0xce8c7410, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498c7419, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44c741d, SUC_lexicon_tags_12 },
    { 0x3c4c741d, SUC_lexicon_tags_2 },
    { 0xdec741e, SUC_lexicon_tags_33 },
    { 0xfb8c7420, SUC_lexicon_tags_7 },
    { 0x2a2c741f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740c7429, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ac742c, SUC_lexicon_tags_1 },
    { 0xed0c742d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0c7431, SUC_lexicon_tags_15 },
    { 0xd80c7432, SUC_lexicon_tags_35 },
    { 0x856c7433, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x314c7436, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0c7439, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe2c743b, SUC_lexicon_tags_8 },
    { 0x752c743c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbac7442, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x11ac7444, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02c7449, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x692c744b, SUC_lexicon_tags_32 },
    { 0xcb2c744c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x648c744e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26c7450, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ac7453, SUC_lexicon_tags_11 },
    { 0x378c7453, SUC_lexicon_tags_60 },
    { 0x312c7455, SUC_lexicon_tags_14 },
    { 0x7ecc7456, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656c745a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb70c745c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0c745f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c7462, SUC_lexicon_tags_33 },
    { 0x752c7463, SUC_lexicon_tags_87 },
    { 0xd7cc7464, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x170c7466, SUC_lexicon_tags_17 },
    { 0xab4c7467, SUC_lexicon_tags_19 },
    { 0x2e8c7467, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ec7472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4c7475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ac7478, SUC_lexicon_tags_33 },
    { 0xcc8c7479, SUC_lexicon_tags_11 },
    { 0x414c747a, SUC_lexicon_tags_60 },
    { 0xe30c7478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80cc747d, SUC_lexicon_tags_37 },
    { 0xff8c747e, SUC_lexicon_tags_105 },
    { 0x688c747f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654c7484, SUC_lexicon_tags_74 },
    { 0xab4c7485, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa12c7487, SUC_lexicon_tags_14 },
    { 0xbfcc7488, SUC_lexicon_tags_12 },
    { 0x5dec7489, SUC_lexicon_tags_26 },
    { 0xd0ec748a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4c748d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c7491, SUC_lexicon_tags_7 },
    { 0x19cc7492, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaecc7494, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0c749b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c74a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd54c74a4, SUC_lexicon_tags_15 },
    { 0xdc6c74a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246c74a9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd8c74ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa48c74ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ec74b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1dec74b2, SUC_lexicon_tags_14 },
    { 0xd46c74b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a4c74b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8c74b9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x2f6c74bb, SUC_lexicon_tags_25 },
    { 0xbaec74bc, SUC_lexicon_tags_25 },
    { 0x7e6c74bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6c74c0, SUC_lexicon_tags_2 },
    { 0x77ac74c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76c74c4, SUC_lexicon_tags_17 },
    { 0x5acc74c5, SUC_lexicon_tags_27 },
    { 0xeac74c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x994c74c8, SUC_lexicon_tags_27 },
    { 0x8b8c74c9, SUC_lexicon_tags_25 },
    { 0xf8ec74ca, SUC_lexicon_tags_56 },
    { 0xaeec74cb, SUC_lexicon_tags_25 },
    { 0xf06c74cb, SUC_lexicon_tags_0 },
    { 0x250c74cd, SUC_lexicon_tags_12 },
    { 0xdf6c74ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x600c74d1, SUC_lexicon_tags_2 },
    { 0x3ccc74d2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06c74db, SUC_lexicon_tags_17 },
    { 0xf32c74db, SUC_lexicon_tags_37 },
    { 0xa42c74db, SUC_lexicon_tags_31 },
    { 0x6f0c74db, SUC_lexicon_tags_47 },
    { 0x198c74df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fec74e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6c74ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c74f0, SUC_lexicon_tags_14 },
    { 0x328c74f1, SUC_lexicon_tags_17 },
    { 0x1a4c74f0, SUC_lexicon_tags_16 },
    { 0xedec74f3, SUC_lexicon_tags_37 },
    { 0x722c74f1, SUC_lexicon_tags_11 },
    { 0xb3ac74f0, SUC_lexicon_tags_1 },
    { 0x3c8c74f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86c74f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd4cc74fb, SUC_lexicon_tags_35 },
    { 0x2f4c74fb, SUC_lexicon_tags_11 },
    { 0x264c74fc, SUC_lexicon_tags_14 },
    { 0xb0ec74fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x99cc7500, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6c7504, SUC_lexicon_tags_11 },
    { 0x930c7505, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4eac7507, SUC_lexicon_tags_30 },
    { 0x3ec7507, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ec750e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146c7515, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3ec7517, SUC_lexicon_tags_49 },
    { 0x70c7517, SUC_lexicon_tags_17 },
    { 0x162c7519, SUC_lexicon_tags_29 },
    { 0x4a0c751a, SUC_lexicon_tags_14 },
    { 0x2e8c7519, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6ec751d, SUC_lexicon_tags_47 },
    { 0x3d0c751d, SUC_lexicon_tags_12 },
    { 0x9fac751d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ec7521, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x348c7523, SUC_lexicon_tags_40 },
    { 0x25ac7523, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2c7528, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ec752b, SUC_lexicon_tags_15 },
    { 0x44cc752c, SUC_lexicon_tags_30 },
    { 0xd9ac752d, SUC_lexicon_tags_7 },
    { 0x588c752e, SUC_lexicon_tags_77 },
    { 0x452c752f, SUC_lexicon_tags_1 },
    { 0x80ec752f, SUC_lexicon_tags_60 },
    { 0x7dcc7531, SUC_lexicon_tags_63 },
    { 0x67ec7530, SUC_lexicon_tags_1 },
    { 0x116c7533, SUC_lexicon_tags_12 },
    { 0x4c7534, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0c7537, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658c7541, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ac7544, SUC_lexicon_tags_6 },
    { 0x45cc7545, SUC_lexicon_tags_21 },
    { 0xe8c7546, SUC_lexicon_tags_11 },
    { 0xf06c7547, SUC_lexicon_tags_37 },
    { 0xfc0c7544, SUC_lexicon_tags_14 },
    { 0xd04c7545, SUC_lexicon_tags_25 },
    { 0x3dac754a, SUC_lexicon_tags_38 },
    { 0x3fec754a, SUC_lexicon_tags_14 },
    { 0x7f6c754b, SUC_lexicon_tags_21 },
    { 0x5ec754d, SUC_lexicon_tags_87 },
    { 0x2bec754b, SUC_lexicon_tags_12 },
    { 0x738c7544, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x612c7551, SUC_lexicon_tags_9 },
    { 0x7fcc7552, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14ac7554, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ec7559, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4c755c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cc755f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6c7564, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38c7567, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2c756a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4c756e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896c7573, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c7576, SUC_lexicon_tags_11 },
    { 0x508c7577, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x754c7579, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0c757c, SUC_lexicon_tags_36 },
    { 0xe90c757d, SUC_lexicon_tags_17 },
    { 0xfb0c757e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7581, SUC_lexicon_tags_45 },
    { 0x910c7581, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ec7586, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfe8c7588, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8cc758a, SUC_lexicon_tags_28 },
    { 0x8b6c758b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8c758d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8c7593, SUC_lexicon_tags_45 },
    { 0x116c7593, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x210c7597, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa90c7599, SUC_lexicon_tags_1 },
    { 0x71ac759a, SUC_lexicon_tags_11 },
    { 0xe82c759a, SUC_lexicon_tags_14 },
    { 0x826c759a, SUC_lexicon_tags_17 },
    { 0x59cc759d, SUC_lexicon_tags_21 },
    { 0xe68c759c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb68c75a0, SUC_lexicon_tags_47 },
    { 0xe84c75a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c75a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ec75a8, SUC_lexicon_tags_28 },
    { 0xc78c75a8, SUC_lexicon_tags_17 },
    { 0x64cc75aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4c75b0, SUC_lexicon_tags_30 },
    { 0xcd8c75b1, SUC_lexicon_tags_1 },
    { 0xf42c75b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc6c75b4, SUC_lexicon_tags_27 },
    { 0x7e0c75b5, SUC_lexicon_tags_15 },
    { 0x90ac75b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ac75bd, SUC_lexicon_tags_29 },
    { 0x7ccc75bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x376c75c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac75c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9b8c75c5, SUC_lexicon_tags_63 },
    { 0x3b8c75c6, SUC_lexicon_tags_2 },
    { 0x824c75c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x618c75c9, SUC_lexicon_tags_0 },
    { 0x9b0c75ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cc75df, SUC_lexicon_tags_0 },
    { 0x738c75e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ec75e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06c75ea, SUC_lexicon_tags_15 },
    { 0x1eec75ea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x588c75ee, SUC_lexicon_tags_16 },
    { 0xf58c75ef, SUC_lexicon_tags_25 },
    { 0xf48c75ef, SUC_lexicon_tags_30 },
    { 0x56ac75f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc75f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6c75f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54ec75fa, SUC_lexicon_tags_15 },
    { 0x95ac75fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4aec75fd, SUC_lexicon_tags_47 },
    { 0xe0ec75fd, SUC_lexicon_tags_46 },
    { 0xb92c75ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ac7604, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7c8c7606, SUC_lexicon_tags_3 },
    { 0xf4ac7607, SUC_lexicon_tags_12 },
    { 0x9f2c7607, SUC_lexicon_tags_15 },
    { 0xe22c7609, SUC_lexicon_tags_60 },
    { 0xc98c760a, SUC_lexicon_tags_25 },
    { 0xd4ec7609, SUC_lexicon_tags_19 },
    { 0x450c7606, SUC_lexicon_tags_1 },
    { 0x992c760d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aac7610, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12c7615, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x438c7617, SUC_lexicon_tags_17 },
    { 0x310c7617, SUC_lexicon_tags_49 },
    { 0x40ec7617, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ec761c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dec7620, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x428c7623, SUC_lexicon_tags_2 },
    { 0x99ac7624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcc7627, SUC_lexicon_tags_11 },
    { 0x8a8c7627, SUC_lexicon_tags_38 },
    { 0xf64c7627, SUC_lexicon_tags_17 },
    { 0xb02c762a, SUC_lexicon_tags_1 },
    { 0x246c762a, SUC_lexicon_tags_55 },
    { 0x196c762c, SUC_lexicon_tags_49 },
    { 0xfb8c762c, SUC_lexicon_tags_26 },
    { 0xeec762e, SUC_lexicon_tags_12 },
    { 0x370c762f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228c763b, SUC_lexicon_tags_96 },
    { 0x54c763c, SUC_lexicon_tags_15 },
    { 0x38cc763d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1f2c763f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58c7649, SUC_lexicon_tags_17 },
    { 0x35ec7649, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0c764d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acc7654, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2cc7656, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4c7659, SUC_lexicon_tags_87 },
    { 0x88c765a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ec765d, SUC_lexicon_tags_30 },
    { 0xcac765d, SUC_lexicon_tags_11 },
    { 0xfcec765d, SUC_lexicon_tags_25 },
    { 0xe80c765e, SUC_lexicon_tags_11 },
    { 0x43ac7661, SUC_lexicon_tags_49 },
    { 0xc7ec765f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ac766b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12c7670, SUC_lexicon_tags_66 },
    { 0xe22c7671, SUC_lexicon_tags_14 },
    { 0x6a2c7672, SUC_lexicon_tags_16 },
    { 0x46ec7672, SUC_lexicon_tags_17 },
    { 0x43ac7673, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fec7676, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x230c7679, SUC_lexicon_tags_23 },
    { 0xf04c767a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cc767e, SUC_lexicon_tags_37 },
    { 0x51ec767f, SUC_lexicon_tags_45 },
    { 0x860c767e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2c7689, SUC_lexicon_tags_82 },
    { 0xd90c768a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1ecc768c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0c768f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ac7699, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22c769d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42c76a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x100c76a2, SUC_lexicon_tags_23 },
    { 0x200c76a3, SUC_lexicon_tags_37 },
    { 0x3a2c76a4, SUC_lexicon_tags_10 },
    { 0xe06c76a4, SUC_lexicon_tags_21 },
    { 0x646c76a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd56c76a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x198c76ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x322c76ae, SUC_lexicon_tags_37 },
    { 0x6f4c76af, SUC_lexicon_tags_5 },
    { 0x38c76ae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x964c76b2, SUC_lexicon_tags_45 },
    { 0x820c76b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eec76b7, SUC_lexicon_tags_15 },
    { 0x6b8c76b7, SUC_lexicon_tags_27 },
    { 0x8b2c76b8, SUC_lexicon_tags_56 },
    { 0x384c76b7, SUC_lexicon_tags_57 },
    { 0x1dec76bb, SUC_lexicon_tags_27 },
    { 0x8e0c76bc, SUC_lexicon_tags_38 },
    { 0xac6c76bd, SUC_lexicon_tags_14 },
    { 0xf42c76be, SUC_lexicon_tags_15 },
    { 0x428c76bf, SUC_lexicon_tags_27 },
    { 0x196c76bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84ac76c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516c76cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224c76d5, SUC_lexicon_tags_46 },
    { 0xb16c76d5, SUC_lexicon_tags_37 },
    { 0x3a0c76d7, SUC_lexicon_tags_21 },
    { 0x572c76d7, SUC_lexicon_tags_32 },
    { 0xf38c76d9, SUC_lexicon_tags_8 },
    { 0xaec76da, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x666c76dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ac76e3, SUC_lexicon_tags_21 },
    { 0xb6ec76e4, SUC_lexicon_tags_64 },
    { 0x954c76e4, SUC_lexicon_tags_63 },
    { 0x906c76e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0c76ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf0c76ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eac76f1, SUC_lexicon_tags_17 },
    { 0x6d2c76f2, SUC_lexicon_tags_27 },
    { 0x47cc76f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaec76f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ec76fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xadec76fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5cc76ff, SUC_lexicon_tags_11 },
    { 0x57cc7700, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x348c7702, SUC_lexicon_tags_46 },
    { 0x86cc7702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68c7707, SUC_lexicon_tags_61 },
    { 0x7c4c7707, SUC_lexicon_tags_17 },
    { 0x112c7709, SUC_lexicon_tags_43 },
    { 0x862c770a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x146c770d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcc7712, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaeac7714, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd4cc7716, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4c771c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62cc771e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42cc7720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ec7724, SUC_lexicon_tags_11 },
    { 0x6cec7725, SUC_lexicon_tags_27 },
    { 0x6ecc7724, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106c772c, SUC_lexicon_tags_60 },
    { 0x8e8c772d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x856c772f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62c7732, SUC_lexicon_tags_26 },
    { 0x33ac7733, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdf0c7735, SUC_lexicon_tags_14 },
    { 0xd10c7736, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdbcc7738, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6dcc773a, SUC_lexicon_tags_19 },
    { 0x24cc773b, SUC_lexicon_tags_80 },
    { 0x11ec773c, SUC_lexicon_tags_17 },
    { 0x396c773d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4c7740, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520c7745, SUC_lexicon_tags_17 },
    { 0x55ec7746, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ec7749, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe46c774b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9e8c774d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34c774f, SUC_lexicon_tags_4 },
    { 0xf0ac7750, SUC_lexicon_tags_87 },
    { 0xdfec774f, SUC_lexicon_tags_17 },
    { 0x54ec7751, SUC_lexicon_tags_61 },
    { 0xc86c7753, SUC_lexicon_tags_6 },
    { 0xaa4c7753, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ec7757, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ec775c, SUC_lexicon_tags_45 },
    { 0x61ac775d, SUC_lexicon_tags_1 },
    { 0x224c775d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ec7762, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cac7766, SUC_lexicon_tags_15 },
    { 0x2c4c7767, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb4c7769, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x510c776c, SUC_lexicon_tags_11 },
    { 0xd8ac776d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76c7778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12ac777a, SUC_lexicon_tags_12 },
    { 0x184c777a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x956c777e, SUC_lexicon_tags_37 },
    { 0x356c777f, SUC_lexicon_tags_24 },
    { 0xa0c777f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c7786, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2c7793, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc70c7795, SUC_lexicon_tags_2 },
    { 0xf2ac7796, SUC_lexicon_tags_38 },
    { 0x124c7797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88c779a, SUC_lexicon_tags_21 },
    { 0xd26c779b, SUC_lexicon_tags_7 },
    { 0xdf6c779b, SUC_lexicon_tags_2 },
    { 0x650c779d, SUC_lexicon_tags_1 },
    { 0x1ccc779e, SUC_lexicon_tags_23 },
    { 0x21ec779d, SUC_lexicon_tags_16 },
    { 0xf50c779f, SUC_lexicon_tags_2 },
    { 0x6cc77a1, SUC_lexicon_tags_11 },
    { 0x944c77a2, SUC_lexicon_tags_2 },
    { 0xc30c77a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18c77a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x592c77a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcc77ad, SUC_lexicon_tags_15 },
    { 0x7bec77ae, SUC_lexicon_tags_87 },
    { 0xfd4c77ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c77b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22c77b6, SUC_lexicon_tags_38 },
    { 0x29ac77b6, SUC_lexicon_tags_37 },
    { 0xc50c77b7, SUC_lexicon_tags_35 },
    { 0xfbac77b9, SUC_lexicon_tags_17 },
    { 0xf32c77ba, SUC_lexicon_tags_48 },
    { 0x53ac77bb, SUC_lexicon_tags_34 },
    { 0x3d0c77ba, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462c77c0, SUC_lexicon_tags_7 },
    { 0x166c77c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf22c77c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36cc77c6, SUC_lexicon_tags_29 },
    { 0xf20c77c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ac77ca, SUC_lexicon_tags_5 },
    { 0x586c77ca, SUC_lexicon_tags_48 },
    { 0xd6ac77cc, SUC_lexicon_tags_27 },
    { 0xce4c77cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77cc77d1, SUC_lexicon_tags_60 },
    { 0x314c77d1, SUC_lexicon_tags_1 },
    { 0xccac77d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a0c77d5, SUC_lexicon_tags_25 },
    { 0xfecc77d6, SUC_lexicon_tags_49 },
    { 0x504c77d7, SUC_lexicon_tags_38 },
    { 0x11ec77d6, SUC_lexicon_tags_25 },
    { 0x256c77d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x358c77db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ac77e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdec77e9, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4c77ed, SUC_lexicon_tags_11 },
    { 0x706c77ee, SUC_lexicon_tags_55 },
    { 0x308c77ed, SUC_lexicon_tags_0 },
    { 0x58ec77ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924c77fa, SUC_lexicon_tags_11 },
    { 0x956c77fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa76c77fd, SUC_lexicon_tags_49 },
    { 0x75cc77fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6c7802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x596c7805, SUC_lexicon_tags_13 },
    { 0xbd4c7806, SUC_lexicon_tags_17 },
    { 0x1dac7807, SUC_lexicon_tags_17 },
    { 0x5b2c7805, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcc780e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ac7811, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xac2c7813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ac7818, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0ac781a, SUC_lexicon_tags_7 },
    { 0x956c781b, SUC_lexicon_tags_15 },
    { 0xea2c781b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8c7824, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x630c7827, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78c782b, SUC_lexicon_tags_12 },
    { 0x786c782b, SUC_lexicon_tags_15 },
    { 0x41ac782b, SUC_lexicon_tags_15 },
    { 0xffcc782d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4c7831, SUC_lexicon_tags_32 },
    { 0xac2c7832, SUC_lexicon_tags_49 },
    { 0xdb4c7833, SUC_lexicon_tags_27 },
    { 0x15cc7834, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dec7837, SUC_lexicon_tags_4 },
    { 0x6aec7838, SUC_lexicon_tags_1 },
    { 0x546c7839, SUC_lexicon_tags_17 },
    { 0xf90c783a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80c783c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766c7842, SUC_lexicon_tags_1 },
    { 0xeb6c7842, SUC_lexicon_tags_2 },
    { 0x85ac7844, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x276c7846, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8c784f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x528c7851, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbc8c7853, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8c785a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa88c785c, SUC_lexicon_tags_15 },
    { 0xac0c785c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11ac785f, SUC_lexicon_tags_106 },
    { 0x31cc7860, SUC_lexicon_tags_15 },
    { 0xa5ac7861, SUC_lexicon_tags_45 },
    { 0x496c7860, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6c7868, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ec786d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60ac786f, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8c7878, SUC_lexicon_tags_85 },
    { 0x24ac7878, SUC_lexicon_tags_12 },
    { 0xc28c787a, SUC_lexicon_tags_63 },
    { 0x1e2c7879, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x160c787d, SUC_lexicon_tags_38 },
    { 0x536c787e, SUC_lexicon_tags_600 },
    { 0xde8c787f, SUC_lexicon_tags_11 },
    { 0x1e8c7880, SUC_lexicon_tags_11 },
    { 0xb66c7881, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc8cc7883, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ecc7885, SUC_lexicon_tags_48 },
    { 0xcecc7886, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0c7889, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438c7892, SUC_lexicon_tags_14 },
    { 0x2b0c7893, SUC_lexicon_tags_11 },
    { 0x314c7893, SUC_lexicon_tags_25 },
    { 0xc98c7895, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262c789a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64c789e, SUC_lexicon_tags_0 },
    { 0x84ac789f, SUC_lexicon_tags_14 },
    { 0x860c78a0, SUC_lexicon_tags_25 },
    { 0x68ac78a1, SUC_lexicon_tags_17 },
    { 0x260c78a2, SUC_lexicon_tags_11 },
    { 0x5ccc78a1, SUC_lexicon_tags_57 },
    { 0xca8c789e, SUC_lexicon_tags_37 },
    { 0xcc0c78a5, SUC_lexicon_tags_27 },
    { 0x500c78a0, SUC_lexicon_tags_9 },
    { 0x5b0c78a7, SUC_lexicon_tags_11 },
    { 0xab4c78a2, SUC_lexicon_tags_37 },
    { 0x932c789f, SUC_lexicon_tags_17 },
    { 0xc58c78aa, SUC_lexicon_tags_12 },
    { 0xf44c78ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x810c78ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ec78b1, SUC_lexicon_tags_25 },
    { 0x15cc78b2, SUC_lexicon_tags_7 },
    { 0x20ac78b2, SUC_lexicon_tags_13 },
    { 0xa80c78b2, SUC_lexicon_tags_11 },
    { 0xfb8c78b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x262c78b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2c78bd, SUC_lexicon_tags_473 },
    { 0x10c78be, SUC_lexicon_tags_11 },
    { 0xf0cc78bf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64c78c3, SUC_lexicon_tags_30 },
    { 0xf48c78c3, SUC_lexicon_tags_12 },
    { 0xc10c78c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x272c78c8, SUC_lexicon_tags_15 },
    { 0x22c78c8, SUC_lexicon_tags_15 },
    { 0xd32c78c8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fac78d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x81ac78d6, SUC_lexicon_tags_7 },
    { 0xaeac78d7, SUC_lexicon_tags_47 },
    { 0x70ec78d7, SUC_lexicon_tags_37 },
    { 0x9b2c78d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6c78dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2c78df, SUC_lexicon_tags_9 },
    { 0xbeec78e0, SUC_lexicon_tags_32 },
    { 0x572c78e0, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02c78e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca8c78e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfec78e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4c78ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe34c78ee, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0c78f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x198c78f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d4c78f5, SUC_lexicon_tags_6 },
    { 0xbcac78f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8c78f9, SUC_lexicon_tags_3 },
    { 0xbd2c78f9, SUC_lexicon_tags_39 },
    { 0xa12c78fb, SUC_lexicon_tags_10 },
    { 0x2b0c78f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34c7900, SUC_lexicon_tags_17 },
    { 0x2bcc7900, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2c7905, SUC_lexicon_tags_25 },
    { 0x152c7906, SUC_lexicon_tags_17 },
    { 0xf22c7907, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240c790c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166c7913, SUC_lexicon_tags_43 },
    { 0x43cc7913, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x142c7916, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf72c7918, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ac791b, SUC_lexicon_tags_16 },
    { 0x196c791c, SUC_lexicon_tags_37 },
    { 0xe7ec791c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40c7925, SUC_lexicon_tags_87 },
    { 0x698c7926, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdaec7928, SUC_lexicon_tags_47 },
    { 0xdc8c7929, SUC_lexicon_tags_43 },
    { 0x6d8c792a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf2c792c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcc7931, SUC_lexicon_tags_68 },
    { 0x2a4c7932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x622c7934, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ec7937, SUC_lexicon_tags_60 },
    { 0xc92c7938, SUC_lexicon_tags_47 },
    { 0x262c7938, SUC_lexicon_tags_37 },
    { 0x826c793a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0c793d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8c7942, SUC_lexicon_tags_63 },
    { 0x9b6c7943, SUC_lexicon_tags_43 },
    { 0x82cc7944, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe02c7946, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8b0c7948, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16c794b, SUC_lexicon_tags_6 },
    { 0x6c0c794c, SUC_lexicon_tags_12 },
    { 0x5a6c794b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4c7952, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd34c7954, SUC_lexicon_tags_8 },
    { 0x52ec7954, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb0c7957, SUC_lexicon_tags_11 },
    { 0x9d4c7958, SUC_lexicon_tags_61 },
    { 0xae2c7958, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cc795e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14c7964, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c8c7966, SUC_lexicon_tags_37 },
    { 0x3cec7966, SUC_lexicon_tags_25 },
    { 0xcf4c7968, SUC_lexicon_tags_9 },
    { 0x528c7967, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4c796e, SUC_lexicon_tags_31 },
    { 0x1bac796e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecc7975, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cec7977, SUC_lexicon_tags_25 },
    { 0x13cc7978, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c7985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7987, SUC_lexicon_tags_0 },
    { 0xaf4c7988, SUC_lexicon_tags_25 },
    { 0x912c7989, SUC_lexicon_tags_23 },
    { 0x81ac7989, SUC_lexicon_tags_29 },
    { 0x420c798b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2c798f, SUC_lexicon_tags_47 },
    { 0x3e0c7990, SUC_lexicon_tags_39 },
    { 0x266c7990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x672c7993, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac7997, SUC_lexicon_tags_27 },
    { 0x474c7998, SUC_lexicon_tags_25 },
    { 0x57ac7999, SUC_lexicon_tags_15 },
    { 0xec6c799a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d0c799c, SUC_lexicon_tags_15 },
    { 0x6bec799d, SUC_lexicon_tags_29 },
    { 0xec2c799d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ec79a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x51ec79a5, SUC_lexicon_tags_7 },
    { 0x148c79a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x690c79a8, SUC_lexicon_tags_1 },
    { 0x232c79a9, SUC_lexicon_tags_37 },
    { 0x856c79aa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5d2c79ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcec79ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ec79b1, SUC_lexicon_tags_27 },
    { 0x6bec79b1, SUC_lexicon_tags_27 },
    { 0x8a6c79b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac79b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746c79bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf02c79be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd70c79c0, SUC_lexicon_tags_2 },
    { 0xfacc79c1, SUC_lexicon_tags_6 },
    { 0xdc2c79c2, SUC_lexicon_tags_7 },
    { 0x7ac79c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x930c79c5, SUC_lexicon_tags_17 },
    { 0x514c79c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfac79c9, SUC_lexicon_tags_11 },
    { 0x9cac79c9, SUC_lexicon_tags_1 },
    { 0x6b0c79cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ac79cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x364c79d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52c79d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88c79db, SUC_lexicon_tags_18 },
    { 0x822c79db, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x83ac79de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62c79e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x828c79e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4c79e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x59ec79e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f8c79eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4c79f3, SUC_lexicon_tags_11 },
    { 0x4a0c79f4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfacc79f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6cc79f8, SUC_lexicon_tags_17 },
    { 0x68cc79f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x33cc79fb, SUC_lexicon_tags_2 },
    { 0xf8c79fc, SUC_lexicon_tags_25 },
    { 0x3e6c79fd, SUC_lexicon_tags_28 },
    { 0xcb4c79fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe88c7a00, SUC_lexicon_tags_7 },
    { 0x4dac7a01, SUC_lexicon_tags_2 },
    { 0x90ec7a02, SUC_lexicon_tags_26 },
    { 0xf6c7a01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd48c7a05, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852c7a0a, SUC_lexicon_tags_29 },
    { 0xbfcc7a0a, SUC_lexicon_tags_7 },
    { 0x82ac7a0c, SUC_lexicon_tags_14 },
    { 0x77cc7a0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x778c7a0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36c7a14, SUC_lexicon_tags_29 },
    { 0xadac7a14, SUC_lexicon_tags_12 },
    { 0x654c7a16, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cc7a1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7dec7a1d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcc7a20, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x930c7a23, SUC_lexicon_tags_17 },
    { 0xcdcc7a23, SUC_lexicon_tags_17 },
    { 0xe2c7a24, SUC_lexicon_tags_7 },
    { 0xdb0c7a26, SUC_lexicon_tags_198 },
    { 0x804c7a27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d6c7a29, SUC_lexicon_tags_28 },
    { 0x106c7a2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29ec7a2c, SUC_lexicon_tags_63 },
    { 0x7f2c7a2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58c7a32, SUC_lexicon_tags_25 },
    { 0x10ac7a32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ec7a39, SUC_lexicon_tags_37 },
    { 0xaa8c7a3a, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecac7a41, SUC_lexicon_tags_82 },
    { 0x74c7a42, SUC_lexicon_tags_17 },
    { 0xfccc7a43, SUC_lexicon_tags_2 },
    { 0x1f0c7a44, SUC_lexicon_tags_7 },
    { 0xb20c7a45, SUC_lexicon_tags_0 },
    { 0x9c0c7a42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4c7a49, SUC_lexicon_tags_47 },
    { 0x6b8c7a4a, SUC_lexicon_tags_79 },
    { 0xd1ec7a4b, SUC_lexicon_tags_4 },
    { 0x122c7a4c, SUC_lexicon_tags_37 },
    { 0xabec7a4c, SUC_lexicon_tags_37 },
    { 0x63ac7a4e, SUC_lexicon_tags_92 },
    { 0x296c7a4b, SUC_lexicon_tags_37 },
    { 0xafcc7a50, SUC_lexicon_tags_14 },
    { 0xfe6c7a4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6c7a54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cc7a59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cc7a5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2c7a60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfd2c7a62, SUC_lexicon_tags_0 },
    { 0xed0c7a63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2eac7a65, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98c7a68, SUC_lexicon_tags_15 },
    { 0x8b6c7a69, SUC_lexicon_tags_28 },
    { 0x7d8c7a69, SUC_lexicon_tags_11 },
    { 0x782c7a6b, SUC_lexicon_tags_25 },
    { 0x746c7a68, SUC_lexicon_tags_38 },
    { 0x5dcc7a6d, SUC_lexicon_tags_7 },
    { 0x90cc7a6e, SUC_lexicon_tags_25 },
    { 0x46c7a6e, SUC_lexicon_tags_12 },
    { 0xdf6c7a70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xddec7a73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ec7a77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x772c7a79, SUC_lexicon_tags_62 },
    { 0x510c7a7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa84c7a7c, SUC_lexicon_tags_37 },
    { 0x286c7a7c, SUC_lexicon_tags_56 },
    { 0x2dec7a7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x74c7a81, SUC_lexicon_tags_17 },
    { 0x992c7a82, SUC_lexicon_tags_77 },
    { 0x21ac7a83, SUC_lexicon_tags_29 },
    { 0x252c7a84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112c7a89, SUC_lexicon_tags_6 },
    { 0x4dec7a8a, SUC_lexicon_tags_17 },
    { 0x298c7a8b, SUC_lexicon_tags_11 },
    { 0x878c7a8a, SUC_lexicon_tags_12 },
    { 0x63ec7a89, SUC_lexicon_tags_31 },
    { 0x578c7a8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ac7a91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadac7a96, SUC_lexicon_tags_37 },
    { 0xa4c7a97, SUC_lexicon_tags_2 },
    { 0x7cec7a96, SUC_lexicon_tags_45 },
    { 0xcd4c7a99, SUC_lexicon_tags_14 },
    { 0xeb8c7a99, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x696c7a9c, SUC_lexicon_tags_83 },
    { 0x608c7a9c, SUC_lexicon_tags_14 },
    { 0xfa0c7a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0c7aa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560c7aa8, SUC_lexicon_tags_17 },
    { 0x310c7aa9, SUC_lexicon_tags_25 },
    { 0xbd2c7aaa, SUC_lexicon_tags_37 },
    { 0xef6c7aab, SUC_lexicon_tags_11 },
    { 0x76ec7aaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8c7aaf, SUC_lexicon_tags_63 },
    { 0x9e8c7ab0, SUC_lexicon_tags_17 },
    { 0xdfcc7ab1, SUC_lexicon_tags_17 },
    { 0xf26c7ab1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98c7ab4, SUC_lexicon_tags_448 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdac7abd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924c7ac4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ac7ac7, SUC_lexicon_tags_7 },
    { 0x874c7ac8, SUC_lexicon_tags_37 },
    { 0x714c7ac8, SUC_lexicon_tags_14 },
    { 0x5f8c7ac7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x194c7acc, SUC_lexicon_tags_27 },
    { 0xc42c7acc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44cc7acf, SUC_lexicon_tags_11 },
    { 0xf56c7acf, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cc7ada, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aac7adc, SUC_lexicon_tags_25 },
    { 0x9f8c7adc, SUC_lexicon_tags_47 },
    { 0x1c8c7ade, SUC_lexicon_tags_8 },
    { 0xcdec7adf, SUC_lexicon_tags_23 },
    { 0x29cc7ae0, SUC_lexicon_tags_17 },
    { 0xef2c7ade, SUC_lexicon_tags_37 },
    { 0xa0cc7adf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cec7ae4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aec7ae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46c7aef, SUC_lexicon_tags_124 },
    { 0x88c7af0, SUC_lexicon_tags_158 },
    { 0xf0c7af0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbac7af8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4c7b00, SUC_lexicon_tags_4 },
    { 0x84ac7b00, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefac7b05, SUC_lexicon_tags_8 },
    { 0x93ec7b06, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40c7b0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c7b10, SUC_lexicon_tags_2 },
    { 0x482c7b10, SUC_lexicon_tags_50 },
    { 0x176c7b12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd02c7b14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa8ec7b16, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecac7b1b, SUC_lexicon_tags_2 },
    { 0xeb6c7b1c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cc7b20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498c7b25, SUC_lexicon_tags_39 },
    { 0x498c7b26, SUC_lexicon_tags_63 },
    { 0x512c7b27, SUC_lexicon_tags_45 },
    { 0x5aac7b28, SUC_lexicon_tags_33 },
    { 0xf24c7b29, SUC_lexicon_tags_19 },
    { 0xab4c7b28, SUC_lexicon_tags_2 },
    { 0xe56c7b2a, SUC_lexicon_tags_57 },
    { 0xc0ac7b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cc7b32, SUC_lexicon_tags_1 },
    { 0x31ec7b32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4c7b36, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x97ec7b38, SUC_lexicon_tags_33 },
    { 0xe38c7b39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfac7b42, SUC_lexicon_tags_25 },
    { 0x570c7b42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x608c7b45, SUC_lexicon_tags_12 },
    { 0x6e2c7b46, SUC_lexicon_tags_28 },
    { 0xef0c7b47, SUC_lexicon_tags_45 },
    { 0xcc0c7b46, SUC_lexicon_tags_55 },
    { 0x140c7b47, SUC_lexicon_tags_14 },
    { 0x8dcc7b46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa02c7b4c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7b4f, SUC_lexicon_tags_48 },
    { 0x65cc7b50, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcc7b56, SUC_lexicon_tags_48 },
    { 0xd48c7b56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acc7b5a, SUC_lexicon_tags_43 },
    { 0xc1ac7b5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19cc7b5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2c7b63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28c7b67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ac7b6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cc7b6e, SUC_lexicon_tags_25 },
    { 0x1a0c7b6e, SUC_lexicon_tags_8 },
    { 0x342c7b70, SUC_lexicon_tags_23 },
    { 0x3dac7b71, SUC_lexicon_tags_12 },
    { 0x858c7b71, SUC_lexicon_tags_12 },
    { 0xaa4c7b70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288c7b77, SUC_lexicon_tags_5 },
    { 0x976c7b78, SUC_lexicon_tags_25 },
    { 0xf8cc7b78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16c7b7c, SUC_lexicon_tags_37 },
    { 0x398c7b7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c7b80, SUC_lexicon_tags_31 },
    { 0xa2cc7b81, SUC_lexicon_tags_7 },
    { 0x8eac7b81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecc7b88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x300c7b8a, SUC_lexicon_tags_55 },
    { 0x82ec7b8b, SUC_lexicon_tags_11 },
    { 0x498c7b8b, SUC_lexicon_tags_43 },
    { 0x4f2c7b8d, SUC_lexicon_tags_33 },
    { 0x474c7b8b, SUC_lexicon_tags_45 },
    { 0x68cc7b8f, SUC_lexicon_tags_7 },
    { 0xca6c7b90, SUC_lexicon_tags_15 },
    { 0x64ec7b91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f4c7b93, SUC_lexicon_tags_19 },
    { 0x8a6c7b94, SUC_lexicon_tags_49 },
    { 0x82ac7b95, SUC_lexicon_tags_57 },
    { 0x734c7b94, SUC_lexicon_tags_14 },
    { 0x8ac7b97, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cc7b9b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xea6c7b9d, SUC_lexicon_tags_37 },
    { 0xa2c7b9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x29ac7ba0, SUC_lexicon_tags_16 },
    { 0x9f2c7ba1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe74c7ba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928c7bae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x720c7bb0, SUC_lexicon_tags_29 },
    { 0x798c7bb1, SUC_lexicon_tags_15 },
    { 0xefac7bb2, SUC_lexicon_tags_1 },
    { 0xcaac7bb3, SUC_lexicon_tags_17 },
    { 0x612c7bb3, SUC_lexicon_tags_15 },
    { 0x584c7bb5, SUC_lexicon_tags_12 },
    { 0xa40c7bb6, SUC_lexicon_tags_7 },
    { 0xb7ec7bb7, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x8bac7bb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878c7bbe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc38c7bc0, SUC_lexicon_tags_26 },
    { 0x658c7bc1, SUC_lexicon_tags_64 },
    { 0x62cc7bc1, SUC_lexicon_tags_66 },
    { 0xcd2c7bc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96c7bc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55cc7bca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2cc7bcc, SUC_lexicon_tags_31 },
    { 0x20ac7bcd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6c7bd2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b6c7bd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc06c7bd6, SUC_lexicon_tags_22 },
    { 0x45cc7bd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cc7bdb, SUC_lexicon_tags_57 },
    { 0x8c4c7bdb, SUC_lexicon_tags_37 },
    { 0xd26c7bdd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4c7be3, SUC_lexicon_tags_14 },
    { 0x73cc7be4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0c7bf0, SUC_lexicon_tags_32 },
    { 0xb6cc7bf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe16c7bf3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf26c7bf5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6c7bf8, SUC_lexicon_tags_17 },
    { 0x982c7bf9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x274c7bfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eec7bff, SUC_lexicon_tags_8 },
    { 0x8cc7c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbec7c02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x304c7c04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcc7c08, SUC_lexicon_tags_49 },
    { 0x512c7c08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb30c7c0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ec7c10, SUC_lexicon_tags_47 },
    { 0xe0ec7c10, SUC_lexicon_tags_5 },
    { 0xa6ec7c11, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x82cc7c15, SUC_lexicon_tags_7 },
    { 0x904c7c16, SUC_lexicon_tags_15 },
    { 0x320c7c17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf16c7c19, SUC_lexicon_tags_46 },
    { 0x2fac7c19, SUC_lexicon_tags_37 },
    { 0x316c7c1b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6c7c21, SUC_lexicon_tags_27 },
    { 0xb64c7c22, SUC_lexicon_tags_12 },
    { 0xf6ec7c23, SUC_lexicon_tags_47 },
    { 0xdd0c7c23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52c7c27, SUC_lexicon_tags_47 },
    { 0x348c7c28, SUC_lexicon_tags_7 },
    { 0x8b0c7c29, SUC_lexicon_tags_15 },
    { 0xc14c7c29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56c7c2d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf64c7c2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36c7c32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6c7c37, SUC_lexicon_tags_12 },
    { 0x8eac7c37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ec7c41, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6c7c44, SUC_lexicon_tags_61 },
    { 0xed0c7c44, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bec7c48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0c7c53, SUC_lexicon_tags_7 },
    { 0xcccc7c54, SUC_lexicon_tags_37 },
    { 0x4ec7c54, SUC_lexicon_tags_57 },
    { 0x28ec7c55, SUC_lexicon_tags_15 },
    { 0x22cc7c56, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2dec7c59, SUC_lexicon_tags_35 },
    { 0x566c7c5a, SUC_lexicon_tags_15 },
    { 0x930c7c5b, SUC_lexicon_tags_11 },
    { 0xb34c7c5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868c7c65, SUC_lexicon_tags_27 },
    { 0xd64c7c66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6c7c6b, SUC_lexicon_tags_15 },
    { 0x54cc7c6c, SUC_lexicon_tags_2 },
    { 0x5fcc7c6c, SUC_lexicon_tags_117 },
    { 0xeb6c7c6e, SUC_lexicon_tags_27 },
    { 0x248c7c6f, SUC_lexicon_tags_37 },
    { 0x33cc7c6e, SUC_lexicon_tags_2 },
    { 0x98cc7c6b, SUC_lexicon_tags_485 },
    { 0x564c7c6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ec7c75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0c7c79, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x56c7c7b, SUC_lexicon_tags_64 },
    { 0xe56c7c7b, SUC_lexicon_tags_11 },
    { 0xe84c7c7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ec7c80, SUC_lexicon_tags_49 },
    { 0x9ec7c81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992c7c91, SUC_lexicon_tags_26 },
    { 0x300c7c91, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860c7c96, SUC_lexicon_tags_15 },
    { 0x3cac7c96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x28c7c9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86ec7c9c, SUC_lexicon_tags_1 },
    { 0xa16c7c9c, SUC_lexicon_tags_25 },
    { 0x3cac7c9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc7ca0, SUC_lexicon_tags_25 },
    { 0x248c7ca1, SUC_lexicon_tags_100 },
    { 0x3c4c7ca2, SUC_lexicon_tags_68 },
    { 0xc2ec7ca3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cab, SUC_lexicon_tags_35 },
    { 0x496c7cac, SUC_lexicon_tags_27 },
    { 0x6e4c7cac, SUC_lexicon_tags_15 },
    { 0x6d0c7cae, SUC_lexicon_tags_30 },
    { 0x1f6c7caf, SUC_lexicon_tags_15 },
    { 0x32c7cae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2c7cb4, SUC_lexicon_tags_55 },
    { 0xcb0c7cb4, SUC_lexicon_tags_25 },
    { 0xe40c7cb6, SUC_lexicon_tags_17 },
    { 0x340c7cb7, SUC_lexicon_tags_65 },
    { 0x6bec7cb8, SUC_lexicon_tags_80 },
    { 0xc98c7cb7, SUC_lexicon_tags_25 },
    { 0xa94c7cba, SUC_lexicon_tags_26 },
    { 0xc4cc7cbb, SUC_lexicon_tags_32 },
    { 0xd18c7cbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ac7cc3, SUC_lexicon_tags_8 },
    { 0x754c7cc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x674c7cc6, SUC_lexicon_tags_37 },
    { 0x5eec7cc6, SUC_lexicon_tags_60 },
    { 0x400c7cc7, SUC_lexicon_tags_17 },
    { 0x3bcc7cc8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902c7cce, SUC_lexicon_tags_43 },
    { 0xc22c7ccf, SUC_lexicon_tags_2 },
    { 0x2ac7ccf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd6c7cd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2c7cdf, SUC_lexicon_tags_11 },
    { 0x9dac7ce0, SUC_lexicon_tags_2 },
    { 0x96ec7ce1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74c7ce7, SUC_lexicon_tags_134 },
    { 0x992c7ce8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x594c7cea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0c7ced, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346c7cf1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac7cf6, SUC_lexicon_tags_45 },
    { 0x542c7cf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ec7cfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0c7d00, SUC_lexicon_tags_11 },
    { 0x8dec7d00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ec7d05, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5d8c7d07, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0c7d0c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbac7d0f, SUC_lexicon_tags_25 },
    { 0xda6c7d10, SUC_lexicon_tags_17 },
    { 0xe4ec7d10, SUC_lexicon_tags_2 },
    { 0xd6ec7d12, SUC_lexicon_tags_27 },
    { 0xeeec7d13, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8c7d16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ac7d1c, SUC_lexicon_tags_30 },
    { 0x80ac7d1d, SUC_lexicon_tags_15 },
    { 0x99ac7d1d, SUC_lexicon_tags_2 },
    { 0xa1ac7d1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x132c7d22, SUC_lexicon_tags_60 },
    { 0xc12c7d23, SUC_lexicon_tags_2 },
    { 0xfeec7d24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4c7d28, SUC_lexicon_tags_1 },
    { 0xa74c7d28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24c7d2d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x656c7d2f, SUC_lexicon_tags_23 },
    { 0xa5ec7d30, SUC_lexicon_tags_31 },
    { 0x49ec7d31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ac7d36, SUC_lexicon_tags_11 },
    { 0x410c7d37, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcc7d3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832c7d3f, SUC_lexicon_tags_70 },
    { 0xe22c7d40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x560c7d43, SUC_lexicon_tags_9 },
    { 0x236c7d44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb56c7d46, SUC_lexicon_tags_62 },
    { 0xfd4c7d46, SUC_lexicon_tags_33 },
    { 0x914c7d48, SUC_lexicon_tags_12 },
    { 0x856c7d49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e4c7d4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6c7d4f, SUC_lexicon_tags_7 },
    { 0x916c7d4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6c7d54, SUC_lexicon_tags_12 },
    { 0x148c7d55, SUC_lexicon_tags_29 },
    { 0xa9ac7d56, SUC_lexicon_tags_19 },
    { 0xca8c7d56, SUC_lexicon_tags_34 },
    { 0xf6cc7d56, SUC_lexicon_tags_12 },
    { 0xee4c7d59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eac7d5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbac7d5f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0c7d64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34c7d67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc7d6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeac7d75, SUC_lexicon_tags_7 },
    { 0xde2c7d76, SUC_lexicon_tags_19 },
    { 0x52cc7d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ec7d7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc62c7d7c, SUC_lexicon_tags_9 },
    { 0xdd6c7d7c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808c7d81, SUC_lexicon_tags_33 },
    { 0xffac7d81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6c7d87, SUC_lexicon_tags_323 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930c7d8e, SUC_lexicon_tags_42 },
    { 0x120c7d8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04c7d93, SUC_lexicon_tags_12 },
    { 0x8bcc7d93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7ec7d96, SUC_lexicon_tags_30 },
    { 0xedcc7d97, SUC_lexicon_tags_3 },
    { 0x3c6c7d97, SUC_lexicon_tags_17 },
    { 0x35ec7d99, SUC_lexicon_tags_28 },
    { 0xd28c7d97, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364c7d9e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330c7da3, SUC_lexicon_tags_6 },
    { 0x9a0c7da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54c7dab, SUC_lexicon_tags_11 },
    { 0xc30c7dac, SUC_lexicon_tags_15 },
    { 0xc90c7dad, SUC_lexicon_tags_11 },
    { 0x906c7dac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50c7db0, SUC_lexicon_tags_15 },
    { 0x9bcc7db1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcb4c7db3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff4c7db5, SUC_lexicon_tags_49 },
    { 0x24c7db6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccc7db8, SUC_lexicon_tags_1 },
    { 0xe78c7db8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77ec7dbb, SUC_lexicon_tags_15 },
    { 0x9cec7dbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd80c7dbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4dec7dc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10c7dc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x664c7dc5, SUC_lexicon_tags_27 },
    { 0x4a4c7dc5, SUC_lexicon_tags_27 },
    { 0x758c7dc7, SUC_lexicon_tags_59 },
    { 0xd6ac7dc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ec7dcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2c7dcf, SUC_lexicon_tags_43 },
    { 0xc04c7dd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x332c7dd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ac7dd7, SUC_lexicon_tags_7 },
    { 0x16c7dd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x744c7dda, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ec7de2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa22c7de4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2c6c7de6, SUC_lexicon_tags_0 },
    { 0xacc7de7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x924c7de9, SUC_lexicon_tags_33 },
    { 0x20ac7dea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xacc7ded, SUC_lexicon_tags_30 },
    { 0xe4cc7dee, SUC_lexicon_tags_22 },
    { 0x6c0c7def, SUC_lexicon_tags_14 },
    { 0x5f0c7df0, SUC_lexicon_tags_4 },
    { 0x4d2c7df1, SUC_lexicon_tags_30 },
    { 0x358c7df0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfeac7df4, SUC_lexicon_tags_1 },
    { 0xee4c7df4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ec7dfc, SUC_lexicon_tags_17 },
    { 0x9c0c7dfd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x2a4c7dff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x602c7e02, SUC_lexicon_tags_47 },
    { 0x678c7e03, SUC_lexicon_tags_12 },
    { 0xa18c7e02, SUC_lexicon_tags_40 },
    { 0x464c7e05, SUC_lexicon_tags_33 },
    { 0xf90c7e03, SUC_lexicon_tags_56 },
    { 0x844c7e07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aec7e0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97cc7e13, SUC_lexicon_tags_28 },
    { 0x7f6c7e13, SUC_lexicon_tags_23 },
    { 0x5e2c7e14, SUC_lexicon_tags_31 },
    { 0xd30c7e16, SUC_lexicon_tags_37 },
    { 0xc0cc7e17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4c7e1a, SUC_lexicon_tags_0 },
    { 0x9cc7e1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafac7e23, SUC_lexicon_tags_4 },
    { 0x110c7e24, SUC_lexicon_tags_12 },
    { 0x660c7e24, SUC_lexicon_tags_11 },
    { 0x42ec7e26, SUC_lexicon_tags_11 },
    { 0xa7ac7e25, SUC_lexicon_tags_27 },
    { 0x8e0c7e27, SUC_lexicon_tags_37 },
    { 0x95ec7e24, SUC_lexicon_tags_46 },
    { 0xf34c7e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752c7e2e, SUC_lexicon_tags_33 },
    { 0x43ac7e2f, SUC_lexicon_tags_12 },
    { 0x38ec7e2f, SUC_lexicon_tags_15 },
    { 0x906c7e31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa6c7e33, SUC_lexicon_tags_33 },
    { 0xf92c7e34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdec7e37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb6c7e39, SUC_lexicon_tags_12 },
    { 0xaeac7e3a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x214c7e3c, SUC_lexicon_tags_1 },
    { 0xd48c7e3d, SUC_lexicon_tags_37 },
    { 0x606c7e3e, SUC_lexicon_tags_15 },
    { 0xc20c7e3f, SUC_lexicon_tags_12 },
    { 0x2fac7e40, SUC_lexicon_tags_7 },
    { 0x27ec7e3e, SUC_lexicon_tags_17 },
    { 0x4c4c7e3d, SUC_lexicon_tags_27 },
    { 0x848c7e43, SUC_lexicon_tags_11 },
    { 0xedac7e42, SUC_lexicon_tags_7 },
    { 0xfc4c7e3e, SUC_lexicon_tags_10 },
    { 0xe6ec7e43, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf4ac7e48, SUC_lexicon_tags_23 },
    { 0xf16c7e49, SUC_lexicon_tags_7 },
    { 0x548c7e4a, SUC_lexicon_tags_47 },
    { 0xb16c7e4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaac7e4e, SUC_lexicon_tags_37 },
    { 0x938c7e4f, SUC_lexicon_tags_39 },
    { 0xdcc7e50, SUC_lexicon_tags_47 },
    { 0xd8c7e51, SUC_lexicon_tags_63 },
    { 0xcb6c7e51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640c7e57, SUC_lexicon_tags_9 },
    { 0xff2c7e57, SUC_lexicon_tags_47 },
    { 0x924c7e59, SUC_lexicon_tags_889 },
    { 0xe54c7e5a, SUC_lexicon_tags_106 },
    { 0x27ac7e5b, SUC_lexicon_tags_29 },
    { 0xbdac7e5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2c7e5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0c7e61, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4c7e65, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x620c7e67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd8c7e69, SUC_lexicon_tags_77 },
    { 0xb0cc7e69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c8c7e6c, SUC_lexicon_tags_1 },
    { 0x372c7e6c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4c7e71, SUC_lexicon_tags_26 },
    { 0xb54c7e72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00c7e79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0c7e7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe8ec7e80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cc7e85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x572c7e87, SUC_lexicon_tags_13 },
    { 0x6aec7e88, SUC_lexicon_tags_27 },
    { 0x5b8c7e89, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x1eec7e8b, SUC_lexicon_tags_10 },
    { 0x3d0c7e8c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ac7e8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ec7e93, SUC_lexicon_tags_68 },
    { 0x6b8c7e94, SUC_lexicon_tags_134 },
    { 0x8e2c7e95, SUC_lexicon_tags_47 },
    { 0x538c7e96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x50ac7e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eac7e9c, SUC_lexicon_tags_56 },
    { 0x400c7e9d, SUC_lexicon_tags_2 },
    { 0x80c7e9e, SUC_lexicon_tags_1 },
    { 0x83ec7e9d, SUC_lexicon_tags_19 },
    { 0xc82c7e9f, SUC_lexicon_tags_19 },
    { 0xd08c7e9e, SUC_lexicon_tags_1 },
    { 0xa38c7ea2, SUC_lexicon_tags_29 },
    { 0x47ec7e9e, SUC_lexicon_tags_3 },
    { 0xb18c7ea3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fac7ea9, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118c7eaf, SUC_lexicon_tags_25 },
    { 0x3dcc7eb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cac7eb4, SUC_lexicon_tags_49 },
    { 0xd1cc7eb4, SUC_lexicon_tags_46 },
    { 0x20cc7eb6, SUC_lexicon_tags_38 },
    { 0xec8c7eb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c7eba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x452c7ebc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce8c7ebe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4c7ec1, SUC_lexicon_tags_8 },
    { 0x170c7ec1, SUC_lexicon_tags_63 },
    { 0x3cec7ec3, SUC_lexicon_tags_25 },
    { 0x652c7ec4, SUC_lexicon_tags_15 },
    { 0x26ac7ec5, SUC_lexicon_tags_37 },
    { 0x9a4c7ec4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96c7ec9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8c7ece, SUC_lexicon_tags_19 },
    { 0xd32c7ece, SUC_lexicon_tags_17 },
    { 0x79ec7ecf, SUC_lexicon_tags_27 },
    { 0xb7cc7ed1, SUC_lexicon_tags_83 },
    { 0x510c7ed2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f4c7ed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c4c7ed6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd52c7ed8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x770c7eda, SUC_lexicon_tags_16 },
    { 0xba6c7edb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eec7edf, SUC_lexicon_tags_0 },
    { 0xc9ac7ee0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530c7ee6, SUC_lexicon_tags_27 },
    { 0x2ac7ee7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecc7eec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4c7eef, SUC_lexicon_tags_26 },
    { 0x534c7ef0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x606c7ef2, SUC_lexicon_tags_38 },
    { 0xe9ac7ef2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc7ef8, SUC_lexicon_tags_48 },
    { 0xc32c7ef9, SUC_lexicon_tags_0 },
    { 0xe2c7efa, SUC_lexicon_tags_37 },
    { 0x79cc7ef9, SUC_lexicon_tags_1 },
    { 0xc94c7efc, SUC_lexicon_tags_87 },
    { 0xea2c7efb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef6c7eff, SUC_lexicon_tags_33 },
    { 0x210c7f00, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dec7f03, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4c7f06, SUC_lexicon_tags_28 },
    { 0xac4c7f06, SUC_lexicon_tags_60 },
    { 0x49cc7f06, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x26c7f0a, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x9eec7f0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71ec7f0e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cac7f12, SUC_lexicon_tags_31 },
    { 0x28cc7f12, SUC_lexicon_tags_6 },
    { 0x194c7f14, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x7a4c7f16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0c7f1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2c7f1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64c7f20, SUC_lexicon_tags_40 },
    { 0xd52c7f20, SUC_lexicon_tags_15 },
    { 0xbc4c7f22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886c7f2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ac7f33, SUC_lexicon_tags_40 },
    { 0x2ec7f34, SUC_lexicon_tags_11 },
    { 0xef0c7f35, SUC_lexicon_tags_146 },
    { 0x99cc7f35, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbec7f3d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18c7f41, SUC_lexicon_tags_16 },
    { 0xf70c7f41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4c7f46, SUC_lexicon_tags_17 },
    { 0x8c7f46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x964c7f49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4ac7f4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12c7f4f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61ec7f51, SUC_lexicon_tags_17 },
    { 0x5dcc7f51, SUC_lexicon_tags_5 },
    { 0x8c4c7f53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x742c7f55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62c7f59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x980c7f5b, SUC_lexicon_tags_27 },
    { 0x730c7f5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6ac7f5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36c7f61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc08c7f63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694c7f6f, SUC_lexicon_tags_29 },
    { 0xb02c7f70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ac7f73, SUC_lexicon_tags_1 },
    { 0x8fcc7f74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae6c7f76, SUC_lexicon_tags_19 },
    { 0x30ac7f76, SUC_lexicon_tags_12 },
    { 0xd78c7f78, SUC_lexicon_tags_17 },
    { 0x810c7f79, SUC_lexicon_tags_27 },
    { 0x24ec7f7a, SUC_lexicon_tags_15 },
    { 0x510c7f7b, SUC_lexicon_tags_25 },
    { 0xbbcc7f7c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ac7f80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ac7f83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d8c7f85, SUC_lexicon_tags_17 },
    { 0xedcc7f86, SUC_lexicon_tags_10 },
    { 0xe16c7f87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f2c7f89, SUC_lexicon_tags_11 },
    { 0xcaec7f8a, SUC_lexicon_tags_15 },
    { 0xe0ac7f8b, SUC_lexicon_tags_26 },
    { 0x596c7f8b, SUC_lexicon_tags_48 },
    { 0x684c7f8c, SUC_lexicon_tags_17 },
    { 0x818c7f8d, SUC_lexicon_tags_15 },
    { 0x464c7f8f, SUC_lexicon_tags_25 },
    { 0x124c7f8e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd30c7f92, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cc7f99, SUC_lexicon_tags_1 },
    { 0x542c7f99, SUC_lexicon_tags_68 },
    { 0x4a4c7f9b, SUC_lexicon_tags_45 },
    { 0xec6c7f9c, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4c7fa0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8c7fa3, SUC_lexicon_tags_37 },
    { 0x27ac7fa4, SUC_lexicon_tags_38 },
    { 0x2c2c7fa3, SUC_lexicon_tags_55 },
    { 0x6bcc7fa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ac7fab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614c7faf, SUC_lexicon_tags_46 },
    { 0xd20c7fb0, SUC_lexicon_tags_1 },
    { 0x338c7fb0, SUC_lexicon_tags_26 },
    { 0x146c7fb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x394c7fb5, SUC_lexicon_tags_65 },
    { 0x6c7fb6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1a0c7fb8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x30cc7fba, SUC_lexicon_tags_16 },
    { 0x12c7fba, SUC_lexicon_tags_23 },
    { 0x234c7fbc, SUC_lexicon_tags_48 },
    { 0x832c7fbd, SUC_lexicon_tags_23 },
    { 0x7a4c7fbd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4c7fc4, SUC_lexicon_tags_82 },
    { 0x2e0c7fc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cac7fc9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf8c7fcb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0cc7fcd, SUC_lexicon_tags_48 },
    { 0xb74c7fce, SUC_lexicon_tags_45 },
    { 0x4b4c7fce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58c7fd3, SUC_lexicon_tags_39 },
    { 0x59cc7fd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4c7fdb, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xb4c7fdd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe68c7fdf, SUC_lexicon_tags_17 },
    { 0x13ac7fe0, SUC_lexicon_tags_1 },
    { 0xadac7fe1, SUC_lexicon_tags_27 },
    { 0x1aac7fe2, SUC_lexicon_tags_28 },
    { 0x134c7fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2c7fe7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcec7fec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e4c7fee, SUC_lexicon_tags_11 },
    { 0x1e2c7fef, SUC_lexicon_tags_19 },
    { 0x222c7ff0, SUC_lexicon_tags_0 },
    { 0xa46c7ff1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2c7ff7, SUC_lexicon_tags_19 },
    { 0xffac7ff8, SUC_lexicon_tags_15 },
    { 0x73ec7ff8, SUC_lexicon_tags_15 },
    { 0x40ec7ff9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd6c7ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8c8002, SUC_lexicon_tags_25 },
    { 0x832c8003, SUC_lexicon_tags_11 },
    { 0x7f6c8002, SUC_lexicon_tags_27 },
    { 0xa24c8005, SUC_lexicon_tags_45 },
    { 0x422c8006, SUC_lexicon_tags_9 },
    { 0x396c8007, SUC_lexicon_tags_30 },
    { 0x44c8005, SUC_lexicon_tags_25 },
    { 0x4cac8006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76ac800b, SUC_lexicon_tags_15 },
    { 0x30ec800c, SUC_lexicon_tags_60 },
    { 0xdf2c800d, SUC_lexicon_tags_17 },
    { 0xa18c800c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712c8012, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8c8015, SUC_lexicon_tags_17 },
    { 0x124c8016, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x816c8019, SUC_lexicon_tags_87 },
    { 0xcc6c801a, SUC_lexicon_tags_74 },
    { 0x262c801b, SUC_lexicon_tags_17 },
    { 0xf1ac801c, SUC_lexicon_tags_7 },
    { 0x820c801d, SUC_lexicon_tags_31 },
    { 0x746c801e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cc8023, SUC_lexicon_tags_1 },
    { 0x4b4c8024, SUC_lexicon_tags_25 },
    { 0xd60c8024, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28c802a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74c802f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ac8032, SUC_lexicon_tags_25 },
    { 0x312c8033, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120c8039, SUC_lexicon_tags_25 },
    { 0x380c8039, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08c8042, SUC_lexicon_tags_82 },
    { 0x694c8042, SUC_lexicon_tags_19 },
    { 0x2ec8044, SUC_lexicon_tags_47 },
    { 0xc06c8045, SUC_lexicon_tags_1 },
    { 0x988c8043, SUC_lexicon_tags_17 },
    { 0x5aac8047, SUC_lexicon_tags_12 },
    { 0x9c8c8048, SUC_lexicon_tags_17 },
    { 0xd52c8047, SUC_lexicon_tags_7 },
    { 0xdcec8042, SUC_lexicon_tags_23 },
    { 0x8ec8042, SUC_lexicon_tags_30 },
    { 0x108c8045, SUC_lexicon_tags_0 },
    { 0xbeac8048, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4c8050, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5cec8052, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ac8056, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ac8059, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bec805f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfec8062, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ac8066, SUC_lexicon_tags_17 },
    { 0x518c8067, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ac806a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588c806e, SUC_lexicon_tags_8 },
    { 0x7dcc806e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf0c8071, SUC_lexicon_tags_351 },
    { 0xf6ec8071, SUC_lexicon_tags_45 },
    { 0x6cec8073, SUC_lexicon_tags_12 },
    { 0x200c8073, SUC_lexicon_tags_23 },
    { 0x2b2c8075, SUC_lexicon_tags_11 },
    { 0xc26c8075, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fac807d, SUC_lexicon_tags_1 },
    { 0x328c807e, SUC_lexicon_tags_47 },
    { 0x5ecc807e, SUC_lexicon_tags_1 },
    { 0x3dac8080, SUC_lexicon_tags_2 },
    { 0xf40c8080, SUC_lexicon_tags_25 },
    { 0x100c8082, SUC_lexicon_tags_25 },
    { 0xa96c8082, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ac8086, SUC_lexicon_tags_4 },
    { 0x86cc8086, SUC_lexicon_tags_11 },
    { 0x80ec8088, SUC_lexicon_tags_8 },
    { 0xa8cc8089, SUC_lexicon_tags_11 },
    { 0x93cc8088, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x29ec808c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c808f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eec8094, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7cec8096, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80c809c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ac80a0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86c80a6, SUC_lexicon_tags_7 },
    { 0xbaac80a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66c80ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2c80b0, SUC_lexicon_tags_38 },
    { 0x324c80b0, SUC_lexicon_tags_77 },
    { 0x8ecc80b2, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x46cc80b4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x186c80b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722c80bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x86ec80bd, SUC_lexicon_tags_38 },
    { 0x69cc80be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0c80c1, SUC_lexicon_tags_17 },
    { 0xc0cc80c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aac80c9, SUC_lexicon_tags_10 },
    { 0x414c80c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6aec80cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2c80d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc80d3, SUC_lexicon_tags_17 },
    { 0x8d6c80d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cec80d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa0c80da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40c80dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf32c80df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cc80e7, SUC_lexicon_tags_39 },
    { 0xc00c80e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852c80ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac80f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eac80f8, SUC_lexicon_tags_15 },
    { 0x880c80f9, SUC_lexicon_tags_87 },
    { 0xad2c80fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6c80fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cec8103, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x830c8105, SUC_lexicon_tags_37 },
    { 0x5cac8106, SUC_lexicon_tags_25 },
    { 0xbdac8106, SUC_lexicon_tags_7 },
    { 0x1e0c8105, SUC_lexicon_tags_15 },
    { 0xad6c8107, SUC_lexicon_tags_27 },
    { 0xf5ec8105, SUC_lexicon_tags_19 },
    { 0x6cec8108, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccc810e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x744c8110, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950c8114, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506c811a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6c811f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x614c8121, SUC_lexicon_tags_0 },
    { 0x8f2c8122, SUC_lexicon_tags_17 },
    { 0x854c8121, SUC_lexicon_tags_77 },
    { 0x24cc8124, SUC_lexicon_tags_37 },
    { 0x3b2c8125, SUC_lexicon_tags_85 },
    { 0x2c8126, SUC_lexicon_tags_45 },
    { 0x606c8127, SUC_lexicon_tags_11 },
    { 0x6f4c8128, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8c812c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2acc812e, SUC_lexicon_tags_25 },
    { 0xfb0c812f, SUC_lexicon_tags_6 },
    { 0x4bec812e, SUC_lexicon_tags_77 },
    { 0x946c8131, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x524c8134, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ec8141, SUC_lexicon_tags_28 },
    { 0xed2c8142, SUC_lexicon_tags_17 },
    { 0xfb4c8142, SUC_lexicon_tags_11 },
    { 0x890c8144, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x852c8146, SUC_lexicon_tags_23 },
    { 0x524c8147, SUC_lexicon_tags_65 },
    { 0x5eec8147, SUC_lexicon_tags_20 },
    { 0x5fec8148, SUC_lexicon_tags_50 },
    { 0x55ec814a, SUC_lexicon_tags_15 },
    { 0x518c8149, SUC_lexicon_tags_63 },
    { 0x882c814c, SUC_lexicon_tags_6 },
    { 0xfe0c814d, SUC_lexicon_tags_14 },
    { 0xff8c8148, SUC_lexicon_tags_25 },
    { 0x462c814f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6c8151, SUC_lexicon_tags_25 },
    { 0xba4c8152, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x974c8155, SUC_lexicon_tags_0 },
    { 0x4b6c8155, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8c8159, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fec815e, SUC_lexicon_tags_16 },
    { 0xaaec815e, SUC_lexicon_tags_5 },
    { 0xa0c815e, SUC_lexicon_tags_19 },
    { 0xb1cc8161, SUC_lexicon_tags_17 },
    { 0xc06c8162, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfd2c8164, SUC_lexicon_tags_87 },
    { 0x5d0c8165, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8c8168, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4d6c816a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746c8170, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4c8176, SUC_lexicon_tags_11 },
    { 0x1e6c8176, SUC_lexicon_tags_12 },
    { 0xdcac8178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24c8180, SUC_lexicon_tags_12 },
    { 0x43cc8180, SUC_lexicon_tags_17 },
    { 0xfdac8182, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8186, SUC_lexicon_tags_2 },
    { 0xd38c8187, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ac818b, SUC_lexicon_tags_65 },
    { 0xfcc818c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0c8191, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fec8194, SUC_lexicon_tags_37 },
    { 0x6c2c8195, SUC_lexicon_tags_11 },
    { 0x4e0c8195, SUC_lexicon_tags_12 },
    { 0xa76c8194, SUC_lexicon_tags_33 },
    { 0xf38c8198, SUC_lexicon_tags_25 },
    { 0x5e6c8199, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82c819d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6c81a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554c81b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3ec81b6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6c81bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4c81c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2c81c3, SUC_lexicon_tags_198 },
    { 0x262c81c4, SUC_lexicon_tags_11 },
    { 0xd6ec81c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x126c81c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2c81d5, SUC_lexicon_tags_37 },
    { 0x69ac81d6, SUC_lexicon_tags_17 },
    { 0xdf2c81d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0c81dd, SUC_lexicon_tags_1 },
    { 0xeeec81de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bac81e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ac81e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9eec81e6, SUC_lexicon_tags_16 },
    { 0x766c81e7, SUC_lexicon_tags_45 },
    { 0xe6ec81e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6c81f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52c81f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486c81f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eac81ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec8203, SUC_lexicon_tags_48 },
    { 0x874c8204, SUC_lexicon_tags_69 },
    { 0xf64c8205, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac8211, SUC_lexicon_tags_15 },
    { 0x7bac8211, SUC_lexicon_tags_12 },
    { 0xc9ec8213, SUC_lexicon_tags_25 },
    { 0x3dcc8214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x47cc8216, SUC_lexicon_tags_11 },
    { 0xf3ec8217, SUC_lexicon_tags_68 },
    { 0x85cc8218, SUC_lexicon_tags_19 },
    { 0xdfcc8218, SUC_lexicon_tags_2 },
    { 0xbd8c821a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766c8220, SUC_lexicon_tags_11 },
    { 0x720c8221, SUC_lexicon_tags_8 },
    { 0xa50c8220, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cc8226, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x476c8229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b4c822b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ac8231, SUC_lexicon_tags_17 },
    { 0xd08c8232, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x696c8234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6ac8236, SUC_lexicon_tags_6 },
    { 0x380c8237, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x402c823a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6c823e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300c8242, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1cac8244, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fac8246, SUC_lexicon_tags_6 },
    { 0xdc0c8247, SUC_lexicon_tags_37 },
    { 0x17cc8247, SUC_lexicon_tags_7 },
    { 0x612c8249, SUC_lexicon_tags_23 },
    { 0x66cc8248, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x922c824c, SUC_lexicon_tags_0 },
    { 0x74c824d, SUC_lexicon_tags_17 },
    { 0xe28c824e, SUC_lexicon_tags_49 },
    { 0x72cc824c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78c8252, SUC_lexicon_tags_38 },
    { 0x40ec8252, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x152c8255, SUC_lexicon_tags_37 },
    { 0x66ec8256, SUC_lexicon_tags_57 },
    { 0xea8c8257, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510c825b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cc825f, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44c8265, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x290c8268, SUC_lexicon_tags_7 },
    { 0x202c8269, SUC_lexicon_tags_7 },
    { 0xd7ec826a, SUC_lexicon_tags_68 },
    { 0xd72c826a, SUC_lexicon_tags_9 },
    { 0x41ac826c, SUC_lexicon_tags_7 },
    { 0x334c826b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48c8270, SUC_lexicon_tags_12 },
    { 0xbb4c8270, SUC_lexicon_tags_7 },
    { 0xf8cc8272, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ec8276, SUC_lexicon_tags_16 },
    { 0x2d2c8276, SUC_lexicon_tags_7 },
    { 0xbe0c8278, SUC_lexicon_tags_12 },
    { 0x930c8279, SUC_lexicon_tags_27 },
    { 0x44ac8278, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac8280, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fac8285, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2c0c8287, SUC_lexicon_tags_11 },
    { 0xceac8287, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0c828b, SUC_lexicon_tags_27 },
    { 0x2eec828b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cc828e, SUC_lexicon_tags_0 },
    { 0xc2cc828e, SUC_lexicon_tags_12 },
    { 0xb26c8290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aac8294, SUC_lexicon_tags_8 },
    { 0x45ec8294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364c829a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2c82a1, SUC_lexicon_tags_545 },
    { 0x57ac82a2, SUC_lexicon_tags_33 },
    { 0x9c6c82a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa76c82a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d2c82a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27ec82a9, SUC_lexicon_tags_17 },
    { 0xb42c82aa, SUC_lexicon_tags_17 },
    { 0xf88c82ab, SUC_lexicon_tags_15 },
    { 0xd46c82a9, SUC_lexicon_tags_11 },
    { 0xdc0c82ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x524c82af, SUC_lexicon_tags_25 },
    { 0xb0cc82b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x946c82b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62c82b7, SUC_lexicon_tags_65 },
    { 0x380c82b8, SUC_lexicon_tags_260 },
    { 0xe2ac82b9, SUC_lexicon_tags_28 },
    { 0x78ec82ba, SUC_lexicon_tags_34 },
    { 0xc82b9, SUC_lexicon_tags_7 },
    { 0x220c82bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x710c82be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dec82c4, SUC_lexicon_tags_20 },
    { 0xed4c82c5, SUC_lexicon_tags_39 },
    { 0xd2c82c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06c82ca, SUC_lexicon_tags_29 },
    { 0xe62c82cb, SUC_lexicon_tags_2 },
    { 0xaa6c82cc, SUC_lexicon_tags_17 },
    { 0x3fec82cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712c82d1, SUC_lexicon_tags_25 },
    { 0xf54c82d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ec82d6, SUC_lexicon_tags_43 },
    { 0xed4c82d6, SUC_lexicon_tags_37 },
    { 0xb10c82d6, SUC_lexicon_tags_9 },
    { 0xa74c82d9, SUC_lexicon_tags_13 },
    { 0x486c82da, SUC_lexicon_tags_29 },
    { 0xddcc82db, SUC_lexicon_tags_37 },
    { 0x624c82db, SUC_lexicon_tags_14 },
    { 0x3f8c82dc, SUC_lexicon_tags_49 },
    { 0x9c4c82de, SUC_lexicon_tags_29 },
    { 0xd62c82df, SUC_lexicon_tags_63 },
    { 0x4a4c82df, SUC_lexicon_tags_16 },
    { 0x310c82da, SUC_lexicon_tags_37 },
    { 0x956c82e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x292c82e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116c82ea, SUC_lexicon_tags_6 },
    { 0x932c82eb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8cac82ed, SUC_lexicon_tags_17 },
    { 0x784c82ed, SUC_lexicon_tags_7 },
    { 0xe60c82ee, SUC_lexicon_tags_7 },
    { 0x8d2c82f0, SUC_lexicon_tags_0 },
    { 0x728c82f1, SUC_lexicon_tags_14 },
    { 0xdd6c82f2, SUC_lexicon_tags_23 },
    { 0xe84c82f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x402c82f5, SUC_lexicon_tags_23 },
    { 0x3e6c82f6, SUC_lexicon_tags_12 },
    { 0xfcc82f7, SUC_lexicon_tags_23 },
    { 0x3ecc82f8, SUC_lexicon_tags_25 },
    { 0x392c82f6, SUC_lexicon_tags_37 },
    { 0x754c82fa, SUC_lexicon_tags_11 },
    { 0xbc4c82fb, SUC_lexicon_tags_87 },
    { 0xb3cc82fb, SUC_lexicon_tags_25 },
    { 0xccac82f5, SUC_lexicon_tags_1 },
    { 0x668c82fe, SUC_lexicon_tags_37 },
    { 0x362c82ff, SUC_lexicon_tags_4 },
    { 0x978c82fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06c8304, SUC_lexicon_tags_17 },
    { 0x260c8305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8ac8307, SUC_lexicon_tags_2 },
    { 0xbdac8308, SUC_lexicon_tags_10 },
    { 0x442c8309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae4c830b, SUC_lexicon_tags_26 },
    { 0x790c830b, SUC_lexicon_tags_11 },
    { 0x47cc830c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930c8311, SUC_lexicon_tags_47 },
    { 0xabec8311, SUC_lexicon_tags_11 },
    { 0xd54c8313, SUC_lexicon_tags_2 },
    { 0x682c8314, SUC_lexicon_tags_8 },
    { 0xc7ec8313, SUC_lexicon_tags_2 },
    { 0x87cc8315, SUC_lexicon_tags_11 },
    { 0x332c8313, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x100c8319, SUC_lexicon_tags_17 },
    { 0x76c831a, SUC_lexicon_tags_30 },
    { 0xf28c831a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6dcc831d, SUC_lexicon_tags_14 },
    { 0x382c831d, SUC_lexicon_tags_112 },
    { 0xda4c831f, SUC_lexicon_tags_27 },
    { 0x2ec831f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x236c8323, SUC_lexicon_tags_8 },
    { 0x69cc8323, SUC_lexicon_tags_25 },
    { 0x7a8c8325, SUC_lexicon_tags_37 },
    { 0xe9ec8326, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62c832c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0c8330, SUC_lexicon_tags_2 },
    { 0x604c8330, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6c8334, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7fac8336, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ec833d, SUC_lexicon_tags_12 },
    { 0x8f0c833d, SUC_lexicon_tags_17 },
    { 0xbc6c833d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7e2c8341, SUC_lexicon_tags_11 },
    { 0x55cc8342, SUC_lexicon_tags_19 },
    { 0x300c8341, SUC_lexicon_tags_11 },
    { 0x75ac8343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84cc8346, SUC_lexicon_tags_63 },
    { 0x6cec8347, SUC_lexicon_tags_12 },
    { 0xd50c8347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe52c834a, SUC_lexicon_tags_118 },
    { 0x4eec834a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598c8350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ec8354, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa04c8356, SUC_lexicon_tags_33 },
    { 0x238c8357, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2c835a, SUC_lexicon_tags_49 },
    { 0xb6cc835b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8dec835d, SUC_lexicon_tags_82 },
    { 0x34c835e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64c8365, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x388c8367, SUC_lexicon_tags_1 },
    { 0x2c8368, SUC_lexicon_tags_14 },
    { 0x122c8369, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45ec836b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73ec836d, SUC_lexicon_tags_47 },
    { 0x6b4c836d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14c8371, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc78c8373, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x588c8375, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ec8378, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0c837c, SUC_lexicon_tags_17 },
    { 0x414c837d, SUC_lexicon_tags_17 },
    { 0xe18c837d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ec8381, SUC_lexicon_tags_11 },
    { 0xe08c8381, SUC_lexicon_tags_7 },
    { 0xdecc8383, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d4c8385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68c8389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fac838e, SUC_lexicon_tags_7 },
    { 0xac838e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x23ac8391, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x50cc8393, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x444c8396, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bec8399, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x486c839b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6c839f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cc83a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dac83a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec83a8, SUC_lexicon_tags_12 },
    { 0x81cc83a8, SUC_lexicon_tags_1 },
    { 0x72ac83aa, SUC_lexicon_tags_34 },
    { 0xea6c83a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46c83b1, SUC_lexicon_tags_45 },
    { 0x7b0c83b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cc83c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1ac83c2, SUC_lexicon_tags_27 },
    { 0xc66c83c3, SUC_lexicon_tags_25 },
    { 0xb7ec83c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbac83c9, SUC_lexicon_tags_7 },
    { 0x368c83ca, SUC_lexicon_tags_28 },
    { 0xa2cc83c9, SUC_lexicon_tags_25 },
    { 0xfcec83ca, SUC_lexicon_tags_25 },
    { 0x30c83cc, SUC_lexicon_tags_2 },
    { 0xfd8c83ce, SUC_lexicon_tags_27 },
    { 0x5bac83ce, SUC_lexicon_tags_37 },
    { 0xaeec83d0, SUC_lexicon_tags_92 },
    { 0x35ac83d1, SUC_lexicon_tags_30 },
    { 0x1d8c83d2, SUC_lexicon_tags_79 },
    { 0x42ac83d3, SUC_lexicon_tags_0 },
    { 0x228c83ce, SUC_lexicon_tags_37 },
    { 0x19ec83d5, SUC_lexicon_tags_17 },
    { 0x4bec83cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8c83dc, SUC_lexicon_tags_11 },
    { 0x86cc83dd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cc83e1, SUC_lexicon_tags_60 },
    { 0x1b0c83e2, SUC_lexicon_tags_46 },
    { 0x77ec83e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x110c83e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf30c83e7, SUC_lexicon_tags_43 },
    { 0x96ac83e8, SUC_lexicon_tags_37 },
    { 0x1e2c83e9, SUC_lexicon_tags_14 },
    { 0x840c83ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd10c83ec, SUC_lexicon_tags_0 },
    { 0x70ac83ed, SUC_lexicon_tags_7 },
    { 0xe12c83ee, SUC_lexicon_tags_11 },
    { 0xe70c83ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cc83f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x840c83f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd36c83f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51cc83f8, SUC_lexicon_tags_49 },
    { 0x342c83f8, SUC_lexicon_tags_26 },
    { 0x15ec83f9, SUC_lexicon_tags_17 },
    { 0xad2c83fb, SUC_lexicon_tags_28 },
    { 0x1d6c83fc, SUC_lexicon_tags_7 },
    { 0xbeac83fb, SUC_lexicon_tags_11 },
    { 0xcc2c83fe, SUC_lexicon_tags_56 },
    { 0xf24c83fd, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xed8c8401, SUC_lexicon_tags_2 },
    { 0xe0ac8402, SUC_lexicon_tags_11 },
    { 0xfcac8401, SUC_lexicon_tags_45 },
    { 0xf82c8404, SUC_lexicon_tags_65 },
    { 0xe4cc8405, SUC_lexicon_tags_1 },
    { 0xe24c8406, SUC_lexicon_tags_15 },
    { 0x7cac8404, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x100c8409, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x390c840b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ec8412, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x416c8414, SUC_lexicon_tags_42 },
    { 0x14ec8415, SUC_lexicon_tags_57 },
    { 0x52ec8415, SUC_lexicon_tags_6 },
    { 0x712c8417, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316c841b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x266c841d, SUC_lexicon_tags_8 },
    { 0x31ac841e, SUC_lexicon_tags_29 },
    { 0x1acc841f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6c8425, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0c842c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c8430, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6c8435, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24c843b, SUC_lexicon_tags_17 },
    { 0x4b4c843c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x166c843e, SUC_lexicon_tags_14 },
    { 0x986c843f, SUC_lexicon_tags_14 },
    { 0x15cc8440, SUC_lexicon_tags_108 },
    { 0xd6ac8441, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x35ac8443, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x276c8445, SUC_lexicon_tags_12 },
    { 0x32c8445, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94c8449, SUC_lexicon_tags_14 },
    { 0xa44c8449, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9cec844c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766c8450, SUC_lexicon_tags_60 },
    { 0x122c8451, SUC_lexicon_tags_47 },
    { 0xc5ac8452, SUC_lexicon_tags_45 },
    { 0x608c8450, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5acc8455, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cc8459, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42c845e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438c8462, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x176c8465, SUC_lexicon_tags_33 },
    { 0x418c8466, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x954c8469, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56c846c, SUC_lexicon_tags_60 },
    { 0xb7cc846c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966c8472, SUC_lexicon_tags_17 },
    { 0x8c2c8473, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0c8478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b6c847a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x912c847d, SUC_lexicon_tags_7 },
    { 0x3a8c847d, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xc38c8480, SUC_lexicon_tags_49 },
    { 0x3d6c8480, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cc8487, SUC_lexicon_tags_23 },
    { 0x80ec8487, SUC_lexicon_tags_6 },
    { 0x918c8488, SUC_lexicon_tags_62 },
    { 0xf1ec848a, SUC_lexicon_tags_2 },
    { 0x440c848b, SUC_lexicon_tags_1 },
    { 0x8cc848a, SUC_lexicon_tags_35 },
    { 0x1b8c848b, SUC_lexicon_tags_15 },
    { 0xccec8488, SUC_lexicon_tags_9 },
    { 0x24ac848d, SUC_lexicon_tags_10 },
    { 0x80c8490, SUC_lexicon_tags_47 },
    { 0x756c848e, SUC_lexicon_tags_11 },
    { 0x356c848e, SUC_lexicon_tags_37 },
    { 0x9f0c8493, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cc8499, SUC_lexicon_tags_10 },
    { 0x70ec849a, SUC_lexicon_tags_29 },
    { 0xe96c849b, SUC_lexicon_tags_24 },
    { 0x76ac849c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb52c849e, SUC_lexicon_tags_8 },
    { 0x680c849e, SUC_lexicon_tags_11 },
    { 0x290c84a0, SUC_lexicon_tags_16 },
    { 0xc0c84a1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x974c84a3, SUC_lexicon_tags_17 },
    { 0xaa6c84a4, SUC_lexicon_tags_1 },
    { 0x8d4c84a5, SUC_lexicon_tags_27 },
    { 0x62ac84a5, SUC_lexicon_tags_23 },
    { 0x18cc84a7, SUC_lexicon_tags_69 },
    { 0x39ac84a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cc84aa, SUC_lexicon_tags_49 },
    { 0xf1cc84ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d2c84ad, SUC_lexicon_tags_12 },
    { 0x24cc84ae, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xea6c84b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2c84b5, SUC_lexicon_tags_17 },
    { 0x3d6c84b6, SUC_lexicon_tags_7 },
    { 0xfd4c84b7, SUC_lexicon_tags_32 },
    { 0x70ac84b8, SUC_lexicon_tags_9 },
    { 0x738c84b8, SUC_lexicon_tags_37 },
    { 0x7eec84b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6c84bf, SUC_lexicon_tags_7 },
    { 0x32cc84c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672c84c4, SUC_lexicon_tags_11 },
    { 0xd0cc84c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c84ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958c84ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06c84d2, SUC_lexicon_tags_1 },
    { 0x1dcc84d2, SUC_lexicon_tags_31 },
    { 0x2b6c84d4, SUC_lexicon_tags_26 },
    { 0x5dac84d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8c84d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190c84dd, SUC_lexicon_tags_87 },
    { 0xdeec84de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ac84e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38cc84e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996c84ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeec84ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912c84f4, SUC_lexicon_tags_2 },
    { 0x49cc84f5, SUC_lexicon_tags_12 },
    { 0x1bcc84f6, SUC_lexicon_tags_56 },
    { 0x2f2c84f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a8c84f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x370c84fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaac8501, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbec8504, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x15cc8506, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78c850a, SUC_lexicon_tags_49 },
    { 0x44cc850b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x602c850d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadac850f, SUC_lexicon_tags_26 },
    { 0xdd6c850f, SUC_lexicon_tags_8 },
    { 0x2c6c8511, SUC_lexicon_tags_15 },
    { 0xb36c8512, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x18c8514, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8518, SUC_lexicon_tags_39 },
    { 0x1bac8519, SUC_lexicon_tags_11 },
    { 0x6d0c8519, SUC_lexicon_tags_53 },
    { 0xdb8c851b, SUC_lexicon_tags_27 },
    { 0x502c851c, SUC_lexicon_tags_6 },
    { 0x15ac851d, SUC_lexicon_tags_2 },
    { 0x34cc851e, SUC_lexicon_tags_1 },
    { 0xbbac851f, SUC_lexicon_tags_46 },
    { 0xe4ec8520, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x89cc8522, SUC_lexicon_tags_15 },
    { 0x7a2c8522, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6c8527, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72c852a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92c8531, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dec8535, SUC_lexicon_tags_1 },
    { 0x1d0c8535, SUC_lexicon_tags_60 },
    { 0xc78c8537, SUC_lexicon_tags_12 },
    { 0x942c8537, SUC_lexicon_tags_14 },
    { 0xceac8537, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6c853d, SUC_lexicon_tags_47 },
    { 0xb18c853e, SUC_lexicon_tags_31 },
    { 0x5b6c853f, SUC_lexicon_tags_38 },
    { 0x752c853f, SUC_lexicon_tags_39 },
    { 0x194c853f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x648c8544, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ec854b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580c854f, SUC_lexicon_tags_11 },
    { 0x6e8c8550, SUC_lexicon_tags_33 },
    { 0x61cc8551, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x676c8553, SUC_lexicon_tags_12 },
    { 0x6fac8553, SUC_lexicon_tags_32 },
    { 0x750c8554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634c8559, SUC_lexicon_tags_23 },
    { 0xd32c8559, SUC_lexicon_tags_6 },
    { 0x5acc855b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4c8560, SUC_lexicon_tags_17 },
    { 0x96ac8560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0c8564, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x614c8567, SUC_lexicon_tags_12 },
    { 0xfacc8567, SUC_lexicon_tags_37 },
    { 0x54ac8569, SUC_lexicon_tags_25 },
    { 0x5bac8568, SUC_lexicon_tags_2 },
    { 0xfdac856b, SUC_lexicon_tags_14 },
    { 0x1d0c856a, SUC_lexicon_tags_81 },
    { 0x7d4c8568, SUC_lexicon_tags_2 },
    { 0x3a6c856a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x624c8570, SUC_lexicon_tags_61 },
    { 0xa64c8571, SUC_lexicon_tags_12 },
    { 0xccac8572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa32c8574, SUC_lexicon_tags_1 },
    { 0x7e6c8574, SUC_lexicon_tags_27 },
    { 0xe6c8575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18c857c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0c857f, SUC_lexicon_tags_33 },
    { 0x7dac857f, SUC_lexicon_tags_54 },
    { 0x84ec8581, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2c8584, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x32ec8586, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6fcc8588, SUC_lexicon_tags_15 },
    { 0x6f8c8589, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdfec858b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68c8592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4c859a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4c859f, SUC_lexicon_tags_12 },
    { 0x38cc859f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ec85a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ec85a7, SUC_lexicon_tags_31 },
    { 0x66cc85a8, SUC_lexicon_tags_37 },
    { 0x992c85a7, SUC_lexicon_tags_11 },
    { 0x1a8c85a8, SUC_lexicon_tags_0 },
    { 0x488c85ab, SUC_lexicon_tags_25 },
    { 0x996c85ab, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x136c85af, SUC_lexicon_tags_30 },
    { 0x5e8c85af, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6c85b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e6c85b7, SUC_lexicon_tags_7 },
    { 0xeac85b8, SUC_lexicon_tags_33 },
    { 0x818c85b7, SUC_lexicon_tags_12 },
    { 0x4f0c85ba, SUC_lexicon_tags_2 },
    { 0x8d4c85b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672c85bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf30c85c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dec85c6, SUC_lexicon_tags_9 },
    { 0xb28c85c7, SUC_lexicon_tags_37 },
    { 0xa5ec85c8, SUC_lexicon_tags_8 },
    { 0x93ec85c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430c85ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xccac85d1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x43ec85d3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8c85d6, SUC_lexicon_tags_1564 },
    { 0xe66c85d7, SUC_lexicon_tags_49 },
    { 0x75ac85d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86c85dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2acc85de, SUC_lexicon_tags_31 },
    { 0xa5ec85df, SUC_lexicon_tags_9 },
    { 0xb08c85e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aec85e8, SUC_lexicon_tags_28 },
    { 0xfb8c85e9, SUC_lexicon_tags_49 },
    { 0x180c85ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48cc85ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756c85f3, SUC_lexicon_tags_12 },
    { 0xba2c85f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0c85f7, SUC_lexicon_tags_0 },
    { 0x5a4c85f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x678c85fb, SUC_lexicon_tags_17 },
    { 0x430c85fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x232c85fe, SUC_lexicon_tags_49 },
    { 0xe1cc85ff, SUC_lexicon_tags_43 },
    { 0x5b8c8600, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76ac8602, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354c8606, SUC_lexicon_tags_64 },
    { 0x58ec8607, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ec8611, SUC_lexicon_tags_9 },
    { 0x2a2c8612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63ac8614, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0c8617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x462c8619, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ac861c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd2c861e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52c8621, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0c862b, SUC_lexicon_tags_4 },
    { 0x658c862c, SUC_lexicon_tags_25 },
    { 0x144c862d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x368c862f, SUC_lexicon_tags_1 },
    { 0xcbec8630, SUC_lexicon_tags_23 },
    { 0x4fcc8630, SUC_lexicon_tags_21 },
    { 0x98cc8631, SUC_lexicon_tags_21 },
    { 0x7c2c8630, SUC_lexicon_tags_25 },
    { 0x310c8634, SUC_lexicon_tags_15 },
    { 0x1ac8634, SUC_lexicon_tags_12 },
    { 0x2bcc8636, SUC_lexicon_tags_7 },
    { 0x598c8630, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ac8640, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0c8645, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672c8649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcc864d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc4cc864f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8cec8651, SUC_lexicon_tags_14 },
    { 0x636c8652, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0c8656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cec8659, SUC_lexicon_tags_37 },
    { 0x7d8c865a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x100c865d, SUC_lexicon_tags_7 },
    { 0x250c865e, SUC_lexicon_tags_17 },
    { 0xe76c865f, SUC_lexicon_tags_17 },
    { 0xea6c8660, SUC_lexicon_tags_28 },
    { 0xb30c8660, SUC_lexicon_tags_37 },
    { 0x9fac8662, SUC_lexicon_tags_0 },
    { 0x1e6c865f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ac8669, SUC_lexicon_tags_37 },
    { 0xc38c8669, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9a6c866c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x37cc866e, SUC_lexicon_tags_17 },
    { 0x802c866e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94ec8671, SUC_lexicon_tags_25 },
    { 0x854c8672, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cec8675, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x932c8677, SUC_lexicon_tags_14 },
    { 0xaaac8678, SUC_lexicon_tags_17 },
    { 0x100c8679, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58ec867b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd04c867d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ec8680, SUC_lexicon_tags_59 },
    { 0x71cc8681, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x864c8684, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66c8692, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1a4c8694, SUC_lexicon_tags_45 },
    { 0xc60c8695, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2c8699, SUC_lexicon_tags_48 },
    { 0x74ac8699, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x678c869d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc86a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6bec86a5, SUC_lexicon_tags_16 },
    { 0xfecc86a6, SUC_lexicon_tags_1 },
    { 0xa96c86a5, SUC_lexicon_tags_1 },
    { 0xcc4c86a8, SUC_lexicon_tags_7 },
    { 0xc8cc86a9, SUC_lexicon_tags_12 },
    { 0xfa2c86a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900c86b0, SUC_lexicon_tags_17 },
    { 0x1ac86b1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x622c86b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776c86c0, SUC_lexicon_tags_1 },
    { 0x4b6c86c0, SUC_lexicon_tags_17 },
    { 0x38ec86c0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12c86c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b8c86c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf38c86ca, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x58c86cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ec86d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde4c86d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc28c86d5, SUC_lexicon_tags_35 },
    { 0x9ecc86d6, SUC_lexicon_tags_37 },
    { 0xb1cc86d7, SUC_lexicon_tags_37 },
    { 0x1dec86d8, SUC_lexicon_tags_37 },
    { 0xb14c86d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bec86dc, SUC_lexicon_tags_45 },
    { 0x90c86dd, SUC_lexicon_tags_48 },
    { 0xf06c86de, SUC_lexicon_tags_11 },
    { 0x258c86dd, SUC_lexicon_tags_37 },
    { 0x3d2c86e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x110c86e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c86ea, SUC_lexicon_tags_25 },
    { 0x88ac86eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6c86f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6c86f4, SUC_lexicon_tags_7 },
    { 0x6e8c86f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508c86fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4c86fe, SUC_lexicon_tags_43 },
    { 0xdf6c86ff, SUC_lexicon_tags_19 },
    { 0x2eec86fe, SUC_lexicon_tags_37 },
    { 0x2c6c8701, SUC_lexicon_tags_19 },
    { 0xc96c8702, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6c8705, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12c8708, SUC_lexicon_tags_46 },
    { 0x116c8709, SUC_lexicon_tags_37 },
    { 0x890c870a, SUC_lexicon_tags_14 },
    { 0xa12c8709, SUC_lexicon_tags_14 },
    { 0xb6cc870c, SUC_lexicon_tags_63 },
    { 0xcccc870d, SUC_lexicon_tags_45 },
    { 0xdbec870e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6c8714, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x940c8717, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xde6c8719, SUC_lexicon_tags_47 },
    { 0x1fec871a, SUC_lexicon_tags_33 },
    { 0xe02c871b, SUC_lexicon_tags_11 },
    { 0x482c8719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85ec871e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ac8726, SUC_lexicon_tags_28 },
    { 0x6c6c8727, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36cc8729, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80c872e, SUC_lexicon_tags_37 },
    { 0x1ac872f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x940c8731, SUC_lexicon_tags_15 },
    { 0x306c8732, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f8c8734, SUC_lexicon_tags_15 },
    { 0xb06c8735, SUC_lexicon_tags_23 },
    { 0xd06c8735, SUC_lexicon_tags_2 },
    { 0xad2c8734, SUC_lexicon_tags_19 },
    { 0x480c8738, SUC_lexicon_tags_32 },
    { 0xdcc8736, SUC_lexicon_tags_43 },
    { 0x642c8737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xffcc873c, SUC_lexicon_tags_7 },
    { 0xdfcc873d, SUC_lexicon_tags_77 },
    { 0xd8c873d, SUC_lexicon_tags_2 },
    { 0x812c873f, SUC_lexicon_tags_57 },
    { 0x758c873f, SUC_lexicon_tags_11 },
    { 0xc56c873d, SUC_lexicon_tags_49 },
    { 0xb08c8742, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f6c8744, SUC_lexicon_tags_17 },
    { 0x18ac8745, SUC_lexicon_tags_7 },
    { 0x4eec8746, SUC_lexicon_tags_12 },
    { 0x176c8747, SUC_lexicon_tags_27 },
    { 0xfccc8748, SUC_lexicon_tags_1 },
    { 0x16cc8748, SUC_lexicon_tags_2 },
    { 0xe0ec8749, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566c874f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8c875b, SUC_lexicon_tags_27 },
    { 0x3b0c875b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112c8761, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc56c8763, SUC_lexicon_tags_25 },
    { 0x19cc8764, SUC_lexicon_tags_66 },
    { 0x9d2c8765, SUC_lexicon_tags_15 },
    { 0xe08c8765, SUC_lexicon_tags_29 },
    { 0xc94c8765, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dec8773, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aec8778, SUC_lexicon_tags_7 },
    { 0x70ec8778, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4ac877b, SUC_lexicon_tags_1 },
    { 0x2dcc877c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888c8786, SUC_lexicon_tags_12 },
    { 0x25ec8786, SUC_lexicon_tags_2 },
    { 0x994c8787, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x170c878b, SUC_lexicon_tags_13 },
    { 0x5bac878c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x542c878e, SUC_lexicon_tags_37 },
    { 0xe60c878f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534c8795, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe02c8797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ac879a, SUC_lexicon_tags_14 },
    { 0x4eec879a, SUC_lexicon_tags_26 },
    { 0xa70c879a, SUC_lexicon_tags_38 },
    { 0x790c879d, SUC_lexicon_tags_9 },
    { 0x952c879e, SUC_lexicon_tags_23 },
    { 0x9eac879f, SUC_lexicon_tags_2 },
    { 0x88ac87a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8c87a3, SUC_lexicon_tags_55 },
    { 0x512c87a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878c87ad, SUC_lexicon_tags_2 },
    { 0xc40c87ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x142c87b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e2c87b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24c87ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ac87bd, SUC_lexicon_tags_7 },
    { 0xf0cc87bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44c87c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0c87c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4c87cc, SUC_lexicon_tags_37 },
    { 0xba6c87cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ac87d1, SUC_lexicon_tags_31 },
    { 0xc46c87d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe6cc87d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6c87d6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x182c87d8, SUC_lexicon_tags_0 },
    { 0x5dec87d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ec87df, SUC_lexicon_tags_12 },
    { 0xbaec87e0, SUC_lexicon_tags_11 },
    { 0x92c87e1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeec87e4, SUC_lexicon_tags_7 },
    { 0x50ec87e5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78c87e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2c2c87ea, SUC_lexicon_tags_54 },
    { 0x8e8c87eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2c87f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa6cc87f2, SUC_lexicon_tags_10 },
    { 0x7fec87f3, SUC_lexicon_tags_15 },
    { 0xec2c87f3, SUC_lexicon_tags_27 },
    { 0x5e2c87f5, SUC_lexicon_tags_25 },
    { 0x7ec87f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x37cc87f8, SUC_lexicon_tags_3 },
    { 0x886c87f9, SUC_lexicon_tags_37 },
    { 0x312c87fa, SUC_lexicon_tags_0 },
    { 0xb96c87fb, SUC_lexicon_tags_7 },
    { 0x976c87fb, SUC_lexicon_tags_11 },
    { 0x9e4c87fd, SUC_lexicon_tags_23 },
    { 0x2d4c87f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x470c8801, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x286c8803, SUC_lexicon_tags_30 },
    { 0xf44c8804, SUC_lexicon_tags_41 },
    { 0xe9ec8804, SUC_lexicon_tags_8 },
    { 0xbd2c8805, SUC_lexicon_tags_45 },
    { 0xcdcc8807, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x656c880a, SUC_lexicon_tags_11 },
    { 0xc46c880b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x738c880d, SUC_lexicon_tags_2 },
    { 0xc8ec880e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x992c8811, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866c8816, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328c881a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0c881f, SUC_lexicon_tags_40 },
    { 0xd30c8820, SUC_lexicon_tags_16 },
    { 0xbe8c8821, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2c8825, SUC_lexicon_tags_2 },
    { 0x99ec8826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ec882b, SUC_lexicon_tags_12 },
    { 0x4f6c882b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe96c882e, SUC_lexicon_tags_25 },
    { 0xbcec882e, SUC_lexicon_tags_32 },
    { 0xadcc8830, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ec8833, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208c8838, SUC_lexicon_tags_37 },
    { 0xedcc8838, SUC_lexicon_tags_13 },
    { 0xedcc883a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236c8842, SUC_lexicon_tags_87 },
    { 0x8d6c8843, SUC_lexicon_tags_11 },
    { 0x8b0c8843, SUC_lexicon_tags_10 },
    { 0xf56c8845, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52c884c, SUC_lexicon_tags_19 },
    { 0x700c884d, SUC_lexicon_tags_9 },
    { 0x296c884c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ac8857, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x582c885a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4c885e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9ac8860, SUC_lexicon_tags_0 },
    { 0xcaac8861, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838c8867, SUC_lexicon_tags_49 },
    { 0xab8c8868, SUC_lexicon_tags_11 },
    { 0x904c8867, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4c886c, SUC_lexicon_tags_1 },
    { 0xbf4c886c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78c8870, SUC_lexicon_tags_74 },
    { 0x928c8870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfec8875, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x980c8877, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4c887c, SUC_lexicon_tags_25 },
    { 0x4c6c887d, SUC_lexicon_tags_14 },
    { 0x80ec887c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56c8881, SUC_lexicon_tags_4 },
    { 0xab8c8882, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2c8888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ac888c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98c888f, SUC_lexicon_tags_7 },
    { 0x9a0c888f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x804c8892, SUC_lexicon_tags_15 },
    { 0xb4ac8893, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9c6c8895, SUC_lexicon_tags_32 },
    { 0x970c8896, SUC_lexicon_tags_40 },
    { 0xacc8895, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe52c889a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610c88a0, SUC_lexicon_tags_15 },
    { 0x184c88a1, SUC_lexicon_tags_23 },
    { 0xd30c88a2, SUC_lexicon_tags_11 },
    { 0xcfec88a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bac88a5, SUC_lexicon_tags_17 },
    { 0x76c88a6, SUC_lexicon_tags_32 },
    { 0xed4c88a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32c88aa, SUC_lexicon_tags_15 },
    { 0x3b8c88ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcc88ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6c88b1, SUC_lexicon_tags_15 },
    { 0x578c88b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x926c88b5, SUC_lexicon_tags_34 },
    { 0x5dac88b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5acc88b8, SUC_lexicon_tags_9 },
    { 0x22ec88b9, SUC_lexicon_tags_5 },
    { 0x2c0c88ba, SUC_lexicon_tags_15 },
    { 0x216c88bb, SUC_lexicon_tags_27 },
    { 0x852c88bc, SUC_lexicon_tags_12 },
    { 0x6acc88bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc70c88bf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x262c88c2, SUC_lexicon_tags_19 },
    { 0xf28c88c2, SUC_lexicon_tags_26 },
    { 0xebec88c3, SUC_lexicon_tags_7 },
    { 0x802c88c4, SUC_lexicon_tags_23 },
    { 0xcacc88c6, SUC_lexicon_tags_7 },
    { 0xfa8c88c6, SUC_lexicon_tags_15 },
    { 0x3b4c88c8, SUC_lexicon_tags_29 },
    { 0x756c88c2, SUC_lexicon_tags_2 },
    { 0xb50c88ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ec88cf, SUC_lexicon_tags_17 },
    { 0x916c88cf, SUC_lexicon_tags_23 },
    { 0xfaac88d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdd4c88d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904c88d7, SUC_lexicon_tags_47 },
    { 0xc72c88d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x606c88db, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec88df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x376c88e1, SUC_lexicon_tags_0 },
    { 0xab2c88e2, SUC_lexicon_tags_28 },
    { 0xa66c88e2, SUC_lexicon_tags_6 },
    { 0x262c88e4, SUC_lexicon_tags_25 },
    { 0x60cc88e4, SUC_lexicon_tags_1 },
    { 0x84ac88e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf42c88e8, SUC_lexicon_tags_19 },
    { 0xa4c88e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980c88f0, SUC_lexicon_tags_29 },
    { 0x6c8c88f1, SUC_lexicon_tags_47 },
    { 0x40c88f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e6c88f4, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x490c88f6, SUC_lexicon_tags_17 },
    { 0xbdec88f7, SUC_lexicon_tags_17 },
    { 0xd8cc88f8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccc88fc, SUC_lexicon_tags_1 },
    { 0xc9cc88fd, SUC_lexicon_tags_17 },
    { 0x8ec88fe, SUC_lexicon_tags_17 },
    { 0xebec88fe, SUC_lexicon_tags_12 },
    { 0x6dcc8900, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa78c8902, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8c8906, SUC_lexicon_tags_55 },
    { 0x628c8907, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x146c890a, SUC_lexicon_tags_2 },
    { 0x78cc890a, SUC_lexicon_tags_37 },
    { 0x774c890b, SUC_lexicon_tags_17 },
    { 0xf02c890d, SUC_lexicon_tags_37 },
    { 0xa94c890e, SUC_lexicon_tags_17 },
    { 0x256c890d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac8913, SUC_lexicon_tags_49 },
    { 0xb0cc8914, SUC_lexicon_tags_1 },
    { 0x6fac8915, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c8918, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232c891c, SUC_lexicon_tags_72 },
    { 0xfa2c891c, SUC_lexicon_tags_27 },
    { 0x156c891e, SUC_lexicon_tags_49 },
    { 0x730c891e, SUC_lexicon_tags_12 },
    { 0x5cac891e, SUC_lexicon_tags_7 },
    { 0xbaec8921, SUC_lexicon_tags_17 },
    { 0xc9ec8922, SUC_lexicon_tags_11 },
    { 0x496c8923, SUC_lexicon_tags_12 },
    { 0xdc0c8922, SUC_lexicon_tags_28 },
    { 0x70cc8923, SUC_lexicon_tags_15 },
    { 0x1a8c8926, SUC_lexicon_tags_1 },
    { 0x340c8927, SUC_lexicon_tags_11 },
    { 0xb20c8928, SUC_lexicon_tags_23 },
    { 0xa16c8926, SUC_lexicon_tags_11 },
    { 0xa70c8929, SUC_lexicon_tags_11 },
    { 0x550c892a, SUC_lexicon_tags_2 },
    { 0xaa2c891e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x790c892e, SUC_lexicon_tags_47 },
    { 0x542c892f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490c8936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1ec8938, SUC_lexicon_tags_14 },
    { 0xb56c8938, SUC_lexicon_tags_65 },
    { 0x75ec893a, SUC_lexicon_tags_15 },
    { 0x6b4c8938, SUC_lexicon_tags_12 },
    { 0x186c8938, SUC_lexicon_tags_96 },
    { 0x9b8c893b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848c8944, SUC_lexicon_tags_27 },
    { 0x682c8945, SUC_lexicon_tags_7 },
    { 0x7bac8945, SUC_lexicon_tags_47 },
    { 0x8fcc8947, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe70c8949, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56c894c, SUC_lexicon_tags_68 },
    { 0x838c894c, SUC_lexicon_tags_17 },
    { 0x7f0c894e, SUC_lexicon_tags_25 },
    { 0x190c894f, SUC_lexicon_tags_17 },
    { 0xb82c8950, SUC_lexicon_tags_30 },
    { 0x5fec8951, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aec8955, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38cc895d, SUC_lexicon_tags_7 },
    { 0x7a0c895d, SUC_lexicon_tags_37 },
    { 0xa36c895f, SUC_lexicon_tags_108 },
    { 0xfc8c8960, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4c8963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c896a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x978c896d, SUC_lexicon_tags_5 },
    { 0xb98c896e, SUC_lexicon_tags_49 },
    { 0xba8c896e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ec8978, SUC_lexicon_tags_16 },
    { 0x804c8979, SUC_lexicon_tags_1 },
    { 0xb26c897a, SUC_lexicon_tags_77 },
    { 0x7bec897a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x116c897e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc8987, SUC_lexicon_tags_12 },
    { 0xe60c8987, SUC_lexicon_tags_25 },
    { 0x21ac8989, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x838c898b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40c898f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8c8996, SUC_lexicon_tags_65 },
    { 0x4aec8997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cc899b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x640c899e, SUC_lexicon_tags_17 },
    { 0xaa2c899f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacec89a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292c89af, SUC_lexicon_tags_87 },
    { 0x6fac89af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf38c89b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ecc89b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a0c89b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100c89bc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cac89bf, SUC_lexicon_tags_40 },
    { 0xb82c89c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ec89c5, SUC_lexicon_tags_7 },
    { 0xe02c89c6, SUC_lexicon_tags_64 },
    { 0x6eec89c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xda2c89c9, SUC_lexicon_tags_33 },
    { 0x768c89ca, SUC_lexicon_tags_87 },
    { 0x296c89ca, SUC_lexicon_tags_7 },
    { 0xa18c89ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6c89cf, SUC_lexicon_tags_40 },
    { 0xb6ac89d0, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x6acc89d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10ac89d4, SUC_lexicon_tags_92 },
    { 0x750c89d5, SUC_lexicon_tags_33 },
    { 0xa58c89d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2c89d9, SUC_lexicon_tags_37 },
    { 0xe6ec89d9, SUC_lexicon_tags_17 },
    { 0x988c89db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ac89df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba6c89e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2c89e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc89e9, SUC_lexicon_tags_26 },
    { 0xa4c89ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ac89f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa6c89f2, SUC_lexicon_tags_46 },
    { 0xa68c89f2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cc89f7, SUC_lexicon_tags_31 },
    { 0x7d2c89f7, SUC_lexicon_tags_17 },
    { 0x20ac89f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492c89fd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34c8a03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x468c8a05, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ec8a0b, SUC_lexicon_tags_55 },
    { 0x1aac8a0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cec8a0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86c8a14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd22c8a16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0c8a19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcc8a1d, SUC_lexicon_tags_7 },
    { 0xbeec8a1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98c8a25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf54c8a27, SUC_lexicon_tags_66 },
    { 0xcfac8a27, SUC_lexicon_tags_43 },
    { 0x8cc8a28, SUC_lexicon_tags_25 },
    { 0x1b4c8a2a, SUC_lexicon_tags_70 },
    { 0x4bac8a2b, SUC_lexicon_tags_47 },
    { 0x5acc8a2b, SUC_lexicon_tags_91 },
    { 0x74ac8a2d, SUC_lexicon_tags_10 },
    { 0xf78c8a2b, SUC_lexicon_tags_11 },
    { 0xa92c8a2c, SUC_lexicon_tags_37 },
    { 0xffec8a2b, SUC_lexicon_tags_11 },
    { 0xa54c8a31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x262c8a33, SUC_lexicon_tags_82 },
    { 0xc64c8a34, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd66c8a36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x354c8a38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x818c8a3a, SUC_lexicon_tags_37 },
    { 0xf86c8a3b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x698c8a3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ec8a41, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6c8a44, SUC_lexicon_tags_30 },
    { 0xf04c8a44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ac8a49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x132c8a4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814c8a56, SUC_lexicon_tags_82 },
    { 0x7f2c8a57, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8c8a68, SUC_lexicon_tags_33 },
    { 0xb32c8a68, SUC_lexicon_tags_56 },
    { 0xbd6c8a6a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90c8a6e, SUC_lexicon_tags_0 },
    { 0x1b2c8a6f, SUC_lexicon_tags_1 },
    { 0x752c8a70, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeb6c8a72, SUC_lexicon_tags_15 },
    { 0x136c8a73, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2c8a78, SUC_lexicon_tags_28 },
    { 0x958c8a79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x114c8a7b, SUC_lexicon_tags_10 },
    { 0x4f8c8a7c, SUC_lexicon_tags_39 },
    { 0xb22c8a7d, SUC_lexicon_tags_1 },
    { 0x3a4c8a7e, SUC_lexicon_tags_23 },
    { 0xbb2c8a7f, SUC_lexicon_tags_163 },
    { 0xcf0c8a7f, SUC_lexicon_tags_0 },
    { 0x26c8a81, SUC_lexicon_tags_38 },
    { 0xcacc8a7d, SUC_lexicon_tags_15 },
    { 0x544c8a7b, SUC_lexicon_tags_70 },
    { 0xf32c8a84, SUC_lexicon_tags_14 },
    { 0x810c8a84, SUC_lexicon_tags_25 },
    { 0x93ac8a86, SUC_lexicon_tags_0 },
    { 0x906c8a86, SUC_lexicon_tags_16 },
    { 0x40cc8a7e, SUC_lexicon_tags_29 },
    { 0x710c8a85, SUC_lexicon_tags_2 },
    { 0xe2ac8a85, SUC_lexicon_tags_7 },
    { 0xe10c8a8b, SUC_lexicon_tags_16 },
    { 0x8a8c8a8c, SUC_lexicon_tags_11 },
    { 0x68ec8a8d, SUC_lexicon_tags_37 },
    { 0x486c8a8e, SUC_lexicon_tags_19 },
    { 0xc9ec8a8e, SUC_lexicon_tags_16 },
    { 0xd8cc8a8b, SUC_lexicon_tags_21 },
    { 0xe5ac8a91, SUC_lexicon_tags_11 },
    { 0x6ac8a8c, SUC_lexicon_tags_12 },
    { 0xa38c8a91, SUC_lexicon_tags_12 },
    { 0x6eac8a94, SUC_lexicon_tags_7 },
    { 0xb48c8a95, SUC_lexicon_tags_25 },
    { 0x114c8a7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0c8a99, SUC_lexicon_tags_23 },
    { 0x270c8a99, SUC_lexicon_tags_37 },
    { 0x624c8a99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ec8aa4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc92c8aa6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x544c8aa9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ec8ab0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78c8ab5, SUC_lexicon_tags_63 },
    { 0xe74c8ab6, SUC_lexicon_tags_26 },
    { 0xc70c8ab5, SUC_lexicon_tags_40 },
    { 0x16cc8ab8, SUC_lexicon_tags_27 },
    { 0xa12c8ab9, SUC_lexicon_tags_71 },
    { 0x57ec8ab8, SUC_lexicon_tags_27 },
    { 0x4c6c8abb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x574c8abd, SUC_lexicon_tags_37 },
    { 0x7fec8abd, SUC_lexicon_tags_26 },
    { 0x6ec8abe, SUC_lexicon_tags_15 },
    { 0xc2ac8ac0, SUC_lexicon_tags_37 },
    { 0xea8c8ac1, SUC_lexicon_tags_47 },
    { 0x66cc8ac2, SUC_lexicon_tags_7 },
    { 0xc9ac8ac2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4c8ac8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ac8acb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22c8ace, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764c8ad2, SUC_lexicon_tags_31 },
    { 0xcfec8ad3, SUC_lexicon_tags_7 },
    { 0xd94c8ad4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x552c8ad6, SUC_lexicon_tags_15 },
    { 0x2a4c8ad6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594c8ade, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7cec8ae0, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xf7cc8ae2, SUC_lexicon_tags_15 },
    { 0x27ac8ae2, SUC_lexicon_tags_11 },
    { 0x664c8ae4, SUC_lexicon_tags_85 },
    { 0xc5cc8ae5, SUC_lexicon_tags_11 },
    { 0xa62c8ae5, SUC_lexicon_tags_1 },
    { 0x5b2c8ae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64c8aea, SUC_lexicon_tags_1 },
    { 0x962c8aeb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608c8af2, SUC_lexicon_tags_12 },
    { 0x9f4c8af3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x778c8af5, SUC_lexicon_tags_17 },
    { 0xbc4c8af5, SUC_lexicon_tags_1 },
    { 0xc0ac8af5, SUC_lexicon_tags_27 },
    { 0xa4cc8af7, SUC_lexicon_tags_6 },
    { 0x46c8af9, SUC_lexicon_tags_66 },
    { 0xd46c8afa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf6ac8afc, SUC_lexicon_tags_30 },
    { 0xa44c8afd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x496c8aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ac8b04, SUC_lexicon_tags_25 },
    { 0xedec8b05, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aec8b09, SUC_lexicon_tags_27 },
    { 0x5fcc8b0a, SUC_lexicon_tags_49 },
    { 0xfcac8b0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ec8b0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2c8b13, SUC_lexicon_tags_27 },
    { 0x1cac8b14, SUC_lexicon_tags_11 },
    { 0x350c8b15, SUC_lexicon_tags_10 },
    { 0xda6c8b13, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84c8b1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8c8b24, SUC_lexicon_tags_15 },
    { 0x960c8b25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ac8b2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2c8b2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13cc8b30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x30ec8b32, SUC_lexicon_tags_15 },
    { 0x4e0c8b32, SUC_lexicon_tags_17 },
    { 0x220c8b34, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcc8b38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52c8b3c, SUC_lexicon_tags_7 },
    { 0x844c8b3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d8c8b3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ec8b42, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf6cc8b44, SUC_lexicon_tags_10 },
    { 0xc10c8b45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e8c8b47, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x65ac8b49, SUC_lexicon_tags_27 },
    { 0xe64c8b49, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b2c8b4c, SUC_lexicon_tags_25 },
    { 0xdaac8b4d, SUC_lexicon_tags_341 },
    { 0x398c8b4e, SUC_lexicon_tags_60 },
    { 0x352c8b4e, SUC_lexicon_tags_10 },
    { 0x5aac8b4e, SUC_lexicon_tags_45 },
    { 0x16cc8b4e, SUC_lexicon_tags_136 },
    { 0x6c2c8b50, SUC_lexicon_tags_265 },
    { 0x66ec8b4e, SUC_lexicon_tags_48 },
    { 0x2f6c8b4e, SUC_lexicon_tags_33 },
    { 0xce8c8b55, SUC_lexicon_tags_0 },
    { 0xa6ac8b53, SUC_lexicon_tags_7 },
    { 0x304c8b55, SUC_lexicon_tags_54 },
    { 0xb6ac8b58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa66c8b5a, SUC_lexicon_tags_0 },
    { 0x138c8b5b, SUC_lexicon_tags_152 },
    { 0x994c8b5c, SUC_lexicon_tags_38 },
    { 0xc40c8b5d, SUC_lexicon_tags_15 },
    { 0x150c8b5c, SUC_lexicon_tags_5 },
    { 0x29ec8b5e, SUC_lexicon_tags_17 },
    { 0xd76c8b5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ec8b63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cc8b69, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ac8b6c, SUC_lexicon_tags_16 },
    { 0xb6ac8b6c, SUC_lexicon_tags_38 },
    { 0x22c8b6e, SUC_lexicon_tags_4 },
    { 0xb2c8b6f, SUC_lexicon_tags_29 },
    { 0x14cc8b6f, SUC_lexicon_tags_19 },
    { 0xa6ac8b71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232c8b76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaaac8b78, SUC_lexicon_tags_33 },
    { 0x5e6c8b79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe76c8b7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4c8b7e, SUC_lexicon_tags_25 },
    { 0xcbcc8b7e, SUC_lexicon_tags_55 },
    { 0xf8ac8b80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb2c8b82, SUC_lexicon_tags_15 },
    { 0xad0c8b83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26c8b85, SUC_lexicon_tags_85 },
    { 0x3c2c8b86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ac8b8c, SUC_lexicon_tags_8 },
    { 0xdc2c8b8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8c8b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26c8b94, SUC_lexicon_tags_23 },
    { 0x76ec8b95, SUC_lexicon_tags_15 },
    { 0x1a0c8b96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x508c8b99, SUC_lexicon_tags_28 },
    { 0xed0c8b9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98c8ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6c8ba5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95ac8ba7, SUC_lexicon_tags_19 },
    { 0x874c8ba8, SUC_lexicon_tags_19 },
    { 0x712c8ba8, SUC_lexicon_tags_18 },
    { 0xf46c8baa, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0x684c8bad, SUC_lexicon_tags_15 },
    { 0xf0c8bae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecc8bb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b6c8bb4, SUC_lexicon_tags_2 },
    { 0xd12c8bb5, SUC_lexicon_tags_7 },
    { 0x5acc8bb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236c8bba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32cc8bbd, SUC_lexicon_tags_65 },
    { 0xb6ec8bbe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ec8bc5, SUC_lexicon_tags_12 },
    { 0x31ec8bc6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x706c8bc8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c8bcd, SUC_lexicon_tags_25 },
    { 0x868c8bce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4c8bd2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8c8bd7, SUC_lexicon_tags_12 },
    { 0x9f6c8bd7, SUC_lexicon_tags_2 },
    { 0xca8c8bd8, SUC_lexicon_tags_7 },
    { 0x264c8bd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58c8be1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x356c8be3, SUC_lexicon_tags_14 },
    { 0xd0ec8be4, SUC_lexicon_tags_15 },
    { 0x640c8be5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e8c8be7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x708c8be9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84c8bf2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550c8bfa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ec8bfe, SUC_lexicon_tags_10 },
    { 0x738c8bff, SUC_lexicon_tags_43 },
    { 0xbdac8c00, SUC_lexicon_tags_12 },
    { 0xb5ec8c01, SUC_lexicon_tags_19 },
    { 0x928c8c02, SUC_lexicon_tags_17 },
    { 0x7dec8c01, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6c8c0b, SUC_lexicon_tags_37 },
    { 0xbe6c8c0c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4c8c12, SUC_lexicon_tags_17 },
    { 0x962c8c13, SUC_lexicon_tags_47 },
    { 0xcf8c8c13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180c8c19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd4c8c1b, SUC_lexicon_tags_17 },
    { 0x294c8c1c, SUC_lexicon_tags_43 },
    { 0xf2c8c1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaac8c21, SUC_lexicon_tags_8 },
    { 0x588c8c21, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cc8c28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cc8c2d, SUC_lexicon_tags_2 },
    { 0x40cc8c2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b0c8c30, SUC_lexicon_tags_25 },
    { 0xb3ac8c31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4c8c37, SUC_lexicon_tags_37 },
    { 0x8acc8c38, SUC_lexicon_tags_15 },
    { 0xd66c8c39, SUC_lexicon_tags_11 },
    { 0xd36c8c3a, SUC_lexicon_tags_127 },
    { 0x250c8c3b, SUC_lexicon_tags_2 },
    { 0x5e0c8c3c, SUC_lexicon_tags_11 },
    { 0xcf2c8c3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32c8c45, SUC_lexicon_tags_28 },
    { 0x4d2c8c46, SUC_lexicon_tags_39 },
    { 0xc92c8c47, SUC_lexicon_tags_87 },
    { 0x36ec8c46, SUC_lexicon_tags_43 },
    { 0x83ac8c49, SUC_lexicon_tags_11 },
    { 0xfdac8c49, SUC_lexicon_tags_14 },
    { 0x90ac8c4b, SUC_lexicon_tags_27 },
    { 0x7ecc8c4b, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86c8c54, SUC_lexicon_tags_19 },
    { 0xc34c8c55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ac8c58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20c8c5b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9cec8c5d, SUC_lexicon_tags_37 },
    { 0x21ac8c5e, SUC_lexicon_tags_37 },
    { 0xa14c8c5f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x92ac8c61, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x3f4c8c63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x482c8c66, SUC_lexicon_tags_63 },
    { 0xa08c8c66, SUC_lexicon_tags_82 },
    { 0x42cc8c67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6c8c6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0c8c71, SUC_lexicon_tags_30 },
    { 0x86ec8c71, SUC_lexicon_tags_1 },
    { 0x1b8c8c72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40c8c76, SUC_lexicon_tags_34 },
    { 0x20cc8c77, SUC_lexicon_tags_25 },
    { 0xc1ec8c78, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0c8c7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98c8c7f, SUC_lexicon_tags_37 },
    { 0xaa0c8c7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x324c8c82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecc8c8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56c8c8e, SUC_lexicon_tags_47 },
    { 0x4ec8c8e, SUC_lexicon_tags_23 },
    { 0xa0ec8c8e, SUC_lexicon_tags_0 },
    { 0x330c8c91, SUC_lexicon_tags_47 },
    { 0x712c8c91, SUC_lexicon_tags_77 },
    { 0x15cc8c93, SUC_lexicon_tags_4 },
    { 0x61ac8c93, SUC_lexicon_tags_15 },
    { 0x3a2c8c95, SUC_lexicon_tags_7 },
    { 0xbfec8c96, SUC_lexicon_tags_33 },
    { 0x56ac8c91, SUC_lexicon_tags_45 },
    { 0x2bcc8c98, SUC_lexicon_tags_25 },
    { 0x80cc8c99, SUC_lexicon_tags_68 },
    { 0xe6cc8c96, SUC_lexicon_tags_15 },
    { 0x520c8c9b, SUC_lexicon_tags_5 },
    { 0x466c8c9c, SUC_lexicon_tags_47 },
    { 0x8c6c8c95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaac8ca0, SUC_lexicon_tags_25 },
    { 0xe3ac8ca1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bac8ca7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9ccc8ca9, SUC_lexicon_tags_15 },
    { 0x7eac8caa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cc8cb0, SUC_lexicon_tags_23 },
    { 0x51cc8cb1, SUC_lexicon_tags_45 },
    { 0xdd8c8cb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x718c8cb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58c8cb9, SUC_lexicon_tags_17 },
    { 0xc66c8cba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9b8c8cbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cc8cc1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb60c8cc3, SUC_lexicon_tags_2 },
    { 0x960c8cc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28c8cc7, SUC_lexicon_tags_15 },
    { 0xf7cc8cc7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x72c8cca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x752c8ccd, SUC_lexicon_tags_10 },
    { 0x874c8cce, SUC_lexicon_tags_37 },
    { 0x64c8ccf, SUC_lexicon_tags_23 },
    { 0xf9ec8cce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804c8cd5, SUC_lexicon_tags_47 },
    { 0xe84c8cd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06c8cdd, SUC_lexicon_tags_11 },
    { 0x6b8c8cdd, SUC_lexicon_tags_38 },
    { 0x6dac8cdd, SUC_lexicon_tags_25 },
    { 0xe0c8ce0, SUC_lexicon_tags_11 },
    { 0x5dec8ce1, SUC_lexicon_tags_5 },
    { 0x9f0c8ce2, SUC_lexicon_tags_1 },
    { 0x1e4c8ce3, SUC_lexicon_tags_15 },
    { 0x286c8ce1, SUC_lexicon_tags_14 },
    { 0x598c8ce0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15ac8ce7, SUC_lexicon_tags_37 },
    { 0x934c8ce7, SUC_lexicon_tags_34 },
    { 0xe94c8ce9, SUC_lexicon_tags_1 },
    { 0x84ac8cea, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd62c8cec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd66c8cee, SUC_lexicon_tags_10 },
    { 0xc26c8cee, SUC_lexicon_tags_60 },
    { 0x6f2c8cee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0c8cf3, SUC_lexicon_tags_1 },
    { 0x390c8cf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68c8cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd12c8cf8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0c8cfb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x27cc8cfd, SUC_lexicon_tags_46 },
    { 0x34c8cfe, SUC_lexicon_tags_14 },
    { 0x4fec8cfd, SUC_lexicon_tags_31 },
    { 0xbdec8cfe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc8ac8d02, SUC_lexicon_tags_55 },
    { 0xda6c8d03, SUC_lexicon_tags_11 },
    { 0x4bec8d04, SUC_lexicon_tags_31 },
    { 0xca0c8d05, SUC_lexicon_tags_7 },
    { 0x1ccc8d04, SUC_lexicon_tags_7 },
    { 0xc38c8d07, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78c8d0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e0c8d0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbd4c8d10, SUC_lexicon_tags_47 },
    { 0x60c8d11, SUC_lexicon_tags_26 },
    { 0x704c8d11, SUC_lexicon_tags_45 },
    { 0xd40c8d13, SUC_lexicon_tags_57 },
    { 0xefac8d13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2c8d17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcacc8d19, SUC_lexicon_tags_56 },
    { 0x410c8d19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x472c8d1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ac8d22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618c8d26, SUC_lexicon_tags_45 },
    { 0x2d0c8d27, SUC_lexicon_tags_2 },
    { 0x224c8d28, SUC_lexicon_tags_12 },
    { 0xed8c8d28, SUC_lexicon_tags_2 },
    { 0xb84c8d29, SUC_lexicon_tags_37 },
    { 0x192c8d2b, SUC_lexicon_tags_33 },
    { 0x19ec8d2c, SUC_lexicon_tags_15 },
    { 0xf98c8d2c, SUC_lexicon_tags_29 },
    { 0x624c8d2c, SUC_lexicon_tags_27 },
    { 0x1d6c8d28, SUC_lexicon_tags_25 },
    { 0xef4c8d30, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x90cc8d32, SUC_lexicon_tags_3 },
    { 0xddcc8d33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5cc8d35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aac8d38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4c8d3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86c8d42, SUC_lexicon_tags_30 },
    { 0x77cc8d43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x288c8d45, SUC_lexicon_tags_11 },
    { 0x1c4c8d46, SUC_lexicon_tags_12 },
    { 0x612c8d47, SUC_lexicon_tags_47 },
    { 0xb42c8d46, SUC_lexicon_tags_0 },
    { 0x6e0c8d49, SUC_lexicon_tags_53 },
    { 0x1e2c8d45, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ac8d51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x620c8d54, SUC_lexicon_tags_7 },
    { 0xbc2c8d55, SUC_lexicon_tags_12 },
    { 0x7e2c8d56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326c8d5a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2c8d61, SUC_lexicon_tags_47 },
    { 0x31ac8d61, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0xba4c8d64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b2c8d66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ac8d6b, SUC_lexicon_tags_45 },
    { 0x778c8d6b, SUC_lexicon_tags_17 },
    { 0x614c8d6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828c8d72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ec8d78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4c8d7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ac8d81, SUC_lexicon_tags_9 },
    { 0x2ccc8d81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16c8d85, SUC_lexicon_tags_17 },
    { 0x6d8c8d85, SUC_lexicon_tags_30 },
    { 0x81cc8d87, SUC_lexicon_tags_11 },
    { 0xcd8c8d88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44c8d8e, SUC_lexicon_tags_2 },
    { 0x37ec8d8f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x542c8d91, SUC_lexicon_tags_37 },
    { 0x5c6c8d92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4c8d98, SUC_lexicon_tags_30 },
    { 0x394c8d99, SUC_lexicon_tags_11 },
    { 0x23ac8d9a, SUC_lexicon_tags_28 },
    { 0xbeac8d9a, SUC_lexicon_tags_52 },
    { 0x690c8d98, SUC_lexicon_tags_1 },
    { 0x84cc8d9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa66c8d9f, SUC_lexicon_tags_2 },
    { 0xb2ec8d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974c8da5, SUC_lexicon_tags_25 },
    { 0xfc6c8da6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40c8daa, SUC_lexicon_tags_31 },
    { 0x694c8dab, SUC_lexicon_tags_25 },
    { 0x688c8dac, SUC_lexicon_tags_30 },
    { 0xfa4c8dad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220c8db1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x878c8db3, SUC_lexicon_tags_44 },
    { 0x65ec8db3, SUC_lexicon_tags_45 },
    { 0x53ac8db5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8a2c8db7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cc8dbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x568c8dc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764c8dc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854c8dc9, SUC_lexicon_tags_17 },
    { 0x3d6c8dc9, SUC_lexicon_tags_4 },
    { 0xc5ac8dc9, SUC_lexicon_tags_17 },
    { 0xf80c8dcc, SUC_lexicon_tags_12 },
    { 0x1e2c8dcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446c8dd1, SUC_lexicon_tags_14 },
    { 0x3eac8dd2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106c8ddb, SUC_lexicon_tags_103 },
    { 0x720c8ddb, SUC_lexicon_tags_1 },
    { 0x862c8ddd, SUC_lexicon_tags_65 },
    { 0x284c8dde, SUC_lexicon_tags_37 },
    { 0xbbec8ddc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78c8de3, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x8c8c8de5, SUC_lexicon_tags_7 },
    { 0x508c8de6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0c8dea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ec8ded, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0c8df5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bec8dfb, SUC_lexicon_tags_7 },
    { 0x12ac8dfb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8eec8dfe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x336c8e01, SUC_lexicon_tags_27 },
    { 0x4cac8e02, SUC_lexicon_tags_17 },
    { 0x660c8e03, SUC_lexicon_tags_2 },
    { 0xb86c8e02, SUC_lexicon_tags_44 },
    { 0xfacc8e05, SUC_lexicon_tags_16 },
    { 0xb8ac8e06, SUC_lexicon_tags_26 },
    { 0x488c8e05, SUC_lexicon_tags_15 },
    { 0xbcac8e03, SUC_lexicon_tags_31 },
    { 0x412c8e04, SUC_lexicon_tags_656 },
    { 0xd6c8e05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ec8e11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf4c8e13, SUC_lexicon_tags_68 },
    { 0xe74c8e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4c8e1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8c8e20, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcec8e22, SUC_lexicon_tags_37 },
    { 0x4a2c8e23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80c8e2b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4c8e2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0ac8e31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02c8e3b, SUC_lexicon_tags_21 },
    { 0xaa2c8e3c, SUC_lexicon_tags_55 },
    { 0x81cc8e3d, SUC_lexicon_tags_7 },
    { 0xb56c8e3e, SUC_lexicon_tags_19 },
    { 0xd66c8e3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162c8e44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92c8e49, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ac8e4c, SUC_lexicon_tags_30 },
    { 0xafcc8e4c, SUC_lexicon_tags_29 },
    { 0xa04c8e4c, SUC_lexicon_tags_8 },
    { 0x2d6c8e4e, SUC_lexicon_tags_39 },
    { 0xae6c8e4d, SUC_lexicon_tags_1 },
    { 0xd8c8e51, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ac8e56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4aac8e58, SUC_lexicon_tags_17 },
    { 0x62cc8e58, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300c8e60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x818c8e62, SUC_lexicon_tags_15 },
    { 0x348c8e63, SUC_lexicon_tags_25 },
    { 0xa6c8e63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac8e67, SUC_lexicon_tags_15 },
    { 0x744c8e67, SUC_lexicon_tags_15 },
    { 0xf78c8e68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x608c8e6b, SUC_lexicon_tags_12 },
    { 0xedcc8e6b, SUC_lexicon_tags_9 },
    { 0x640c8e6b, SUC_lexicon_tags_31 },
    { 0x6b6c8e6e, SUC_lexicon_tags_37 },
    { 0x584c8e6c, SUC_lexicon_tags_79 },
    { 0x3d4c8e70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x864c8e72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x966c8e74, SUC_lexicon_tags_1 },
    { 0x9acc8e75, SUC_lexicon_tags_12 },
    { 0xdccc8e76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x964c8e78, SUC_lexicon_tags_17 },
    { 0x5b0c8e79, SUC_lexicon_tags_15 },
    { 0x19ac8e78, SUC_lexicon_tags_27 },
    { 0x61ec8e78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbac8e7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68c8e84, SUC_lexicon_tags_15 },
    { 0x272c8e84, SUC_lexicon_tags_17 },
    { 0x838c8e86, SUC_lexicon_tags_37 },
    { 0xad0c8e87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ec8e8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18c8e8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea0c8e91, SUC_lexicon_tags_15 },
    { 0x8b2c8e91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aec8e95, SUC_lexicon_tags_37 },
    { 0x216c8e95, SUC_lexicon_tags_13 },
    { 0x406c8e97, SUC_lexicon_tags_11 },
    { 0xceac8e97, SUC_lexicon_tags_2 },
    { 0x850c8e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82c8e9e, SUC_lexicon_tags_28 },
    { 0x618c8e9f, SUC_lexicon_tags_17 },
    { 0xf4ac8ea0, SUC_lexicon_tags_30 },
    { 0xc36c8e9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcc8ea6, SUC_lexicon_tags_17 },
    { 0x848c8ea6, SUC_lexicon_tags_6 },
    { 0xa60c8ea8, SUC_lexicon_tags_25 },
    { 0x7d8c8ea9, SUC_lexicon_tags_87 },
    { 0xc2ac8eaa, SUC_lexicon_tags_14 },
    { 0x41cc8ea9, SUC_lexicon_tags_37 },
    { 0x6f0c8eac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x332c8eaf, SUC_lexicon_tags_14 },
    { 0x3ecc8eb0, SUC_lexicon_tags_28 },
    { 0xc28c8eb1, SUC_lexicon_tags_60 },
    { 0xd80c8eb2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbdec8eb4, SUC_lexicon_tags_25 },
    { 0xb56c8eb5, SUC_lexicon_tags_31 },
    { 0x4c8c8eb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaec8eb9, SUC_lexicon_tags_13 },
    { 0xa4cc8eba, SUC_lexicon_tags_85 },
    { 0x96cc8ebb, SUC_lexicon_tags_30 },
    { 0x160c8ebc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ec8ebf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ac8ec2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x896c8ec5, SUC_lexicon_tags_2 },
    { 0x2dac8ec5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x308c8ec8, SUC_lexicon_tags_12 },
    { 0x778c8ec9, SUC_lexicon_tags_1 },
    { 0x552c8eca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76c8ecf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ec8ed2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840c8ed7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8c8eda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcac8ee1, SUC_lexicon_tags_8 },
    { 0x8f6c8ee1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x704c8ee5, SUC_lexicon_tags_1 },
    { 0xecc8ee6, SUC_lexicon_tags_8 },
    { 0x1dac8ee6, SUC_lexicon_tags_27 },
    { 0x1ecc8ee7, SUC_lexicon_tags_10 },
    { 0x9cc8ee6, SUC_lexicon_tags_23 },
    { 0x94ec8ee9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ec8eee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746c8ef4, SUC_lexicon_tags_10 },
    { 0xcac8ef5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5c2c8ef7, SUC_lexicon_tags_15 },
    { 0xad4c8ef8, SUC_lexicon_tags_49 },
    { 0x470c8ef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36c8efe, SUC_lexicon_tags_7 },
    { 0xfa0c8eff, SUC_lexicon_tags_17 },
    { 0x1b4c8f00, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x438c8f02, SUC_lexicon_tags_37 },
    { 0x574c8f03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1acc8f05, SUC_lexicon_tags_11 },
    { 0xc98c8f06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96c8f0b, SUC_lexicon_tags_12 },
    { 0xce8c8f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda8c8f0e, SUC_lexicon_tags_15 },
    { 0xe50c8f0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x96c8f12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd04c8f14, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfd4c8f16, SUC_lexicon_tags_20 },
    { 0x3bac8f17, SUC_lexicon_tags_2 },
    { 0x162c8f17, SUC_lexicon_tags_57 },
    { 0x9ccc8f19, SUC_lexicon_tags_7 },
    { 0xe9ac8f1a, SUC_lexicon_tags_15 },
    { 0x214c8f1b, SUC_lexicon_tags_31 },
    { 0x7dac8f1a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccac8f20, SUC_lexicon_tags_68 },
    { 0x378c8f20, SUC_lexicon_tags_25 },
    { 0x79ec8f21, SUC_lexicon_tags_37 },
    { 0xa28c8f21, SUC_lexicon_tags_34 },
    { 0x5eec8f20, SUC_lexicon_tags_15 },
    { 0xc98c8f25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4c8f29, SUC_lexicon_tags_30 },
    { 0x47cc8f2a, SUC_lexicon_tags_46 },
    { 0x21ec8f2a, SUC_lexicon_tags_6 },
    { 0x83ec8f2c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2b2c8f2e, SUC_lexicon_tags_27 },
    { 0x384c8f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32c8f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6c8f37, SUC_lexicon_tags_24 },
    { 0xc7cc8f37, SUC_lexicon_tags_26 },
    { 0xa00c8f39, SUC_lexicon_tags_201 },
    { 0x794c8f3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xabac8f3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb0c8f3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x108c8f40, SUC_lexicon_tags_46 },
    { 0x866c8f41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4c8f45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacc8f49, SUC_lexicon_tags_2 },
    { 0x30ec8f49, SUC_lexicon_tags_43 },
    { 0xb58c8f4b, SUC_lexicon_tags_37 },
    { 0xac6c8f4c, SUC_lexicon_tags_65 },
    { 0xfb2c8f4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50c8f53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0c8f57, SUC_lexicon_tags_6 },
    { 0x8aec8f57, SUC_lexicon_tags_26 },
    { 0x9b4c8f59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec8f5c, SUC_lexicon_tags_12 },
    { 0xb48c8f5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5fec8f5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x492c8f62, SUC_lexicon_tags_2 },
    { 0x1c0c8f63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2c8f67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ec8f6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f0c8f6c, SUC_lexicon_tags_48 },
    { 0xcf8c8f6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x31cc8f70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2c8f76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ec8f79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x276c8f7b, SUC_lexicon_tags_11 },
    { 0xb38c8f7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ec8f7f, SUC_lexicon_tags_2 },
    { 0xa04c8f7f, SUC_lexicon_tags_7 },
    { 0x36cc8f81, SUC_lexicon_tags_26 },
    { 0x486c8f82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4c8f85, SUC_lexicon_tags_25 },
    { 0x84c8f85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x682c8f88, SUC_lexicon_tags_60 },
    { 0x310c8f89, SUC_lexicon_tags_17 },
    { 0xa22c8f8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74c8f8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678c8f93, SUC_lexicon_tags_11 },
    { 0xbb8c8f94, SUC_lexicon_tags_25 },
    { 0xb3ec8f94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ac8f9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4c8f9f, SUC_lexicon_tags_7 },
    { 0x116c8fa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10c8fa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x142c8fa7, SUC_lexicon_tags_12 },
    { 0xacc8fa8, SUC_lexicon_tags_12 },
    { 0xbacc8fa8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x942c8fac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ec8fb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4c8fb5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc9ac8fb7, SUC_lexicon_tags_19 },
    { 0x92c8fb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x418c8fbb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x730c8fbd, SUC_lexicon_tags_28 },
    { 0xa74c8fbd, SUC_lexicon_tags_14 },
    { 0x7e4c8fbf, SUC_lexicon_tags_69 },
    { 0x4a4c8fbf, SUC_lexicon_tags_14 },
    { 0x62ec8fc1, SUC_lexicon_tags_23 },
    { 0x71ac8fc2, SUC_lexicon_tags_15 },
    { 0xa86c8fc2, SUC_lexicon_tags_48 },
    { 0xa1ac8fc0, SUC_lexicon_tags_87 },
    { 0x588c8fc5, SUC_lexicon_tags_49 },
    { 0x414c8fc6, SUC_lexicon_tags_14 },
    { 0x938c8fc7, SUC_lexicon_tags_40 },
    { 0xfbec8fc7, SUC_lexicon_tags_32 },
    { 0xacac8fc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ec8fcb, SUC_lexicon_tags_17 },
    { 0x210c8fcb, SUC_lexicon_tags_47 },
    { 0xaacc8fcd, SUC_lexicon_tags_37 },
    { 0x356c8fce, SUC_lexicon_tags_63 },
    { 0xa48c8fcf, SUC_lexicon_tags_25 },
    { 0xf2c8fcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6c8fd5, SUC_lexicon_tags_37 },
    { 0x75ac8fd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec8fda, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540c8fde, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x304c8fe0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6c8fe6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8c8fea, SUC_lexicon_tags_37 },
    { 0x1e0c8fea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ac8fee, SUC_lexicon_tags_11 },
    { 0x26cc8fef, SUC_lexicon_tags_35 },
    { 0xf78c8ff0, SUC_lexicon_tags_45 },
    { 0x512c8ff1, SUC_lexicon_tags_17 },
    { 0x374c8ff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660c8ff6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46ac8ff8, SUC_lexicon_tags_2 },
    { 0xf6ac8ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x222c8ffb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x476c8ffd, SUC_lexicon_tags_16 },
    { 0x47ac8ffe, SUC_lexicon_tags_49 },
    { 0x40cc8fff, SUC_lexicon_tags_28 },
    { 0xcf2c8ffe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cac900a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cc900d, SUC_lexicon_tags_63 },
    { 0x96cc900e, SUC_lexicon_tags_30 },
    { 0xedcc900f, SUC_lexicon_tags_0 },
    { 0xd14c9010, SUC_lexicon_tags_14 },
    { 0xe04c9010, SUC_lexicon_tags_1518 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcc9015, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x224c9017, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ac901b, SUC_lexicon_tags_17 },
    { 0xe7cc901c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4c9022, SUC_lexicon_tags_23 },
    { 0x4b4c9023, SUC_lexicon_tags_2 },
    { 0x956c9024, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430c902f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x10c9031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c8c9033, SUC_lexicon_tags_77 },
    { 0xa44c9033, SUC_lexicon_tags_11 },
    { 0x226c9035, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb0ec9037, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8c903a, SUC_lexicon_tags_7 },
    { 0x6c6c903b, SUC_lexicon_tags_43 },
    { 0x904c903c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ac903f, SUC_lexicon_tags_12 },
    { 0x612c9040, SUC_lexicon_tags_7 },
    { 0x2d2c9041, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798c9046, SUC_lexicon_tags_15 },
    { 0x882c9047, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x1a8c9049, SUC_lexicon_tags_15 },
    { 0xb80c904a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xeacc904c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9fec904e, SUC_lexicon_tags_136 },
    { 0xc6cc904e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb8c9051, SUC_lexicon_tags_43 },
    { 0x10ac9051, SUC_lexicon_tags_32 },
    { 0x688c9053, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1c6c9055, SUC_lexicon_tags_51 },
    { 0x1c2c9055, SUC_lexicon_tags_33 },
    { 0xc5ac9057, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x77cc9059, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x504c905c, SUC_lexicon_tags_2 },
    { 0x440c905d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52ec905f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86c9063, SUC_lexicon_tags_47 },
    { 0x1a0c9064, SUC_lexicon_tags_15 },
    { 0xc84c9064, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0c906a, SUC_lexicon_tags_55 },
    { 0x760c906b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c906e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x466c9071, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2ec9073, SUC_lexicon_tags_10 },
    { 0x9bcc9073, SUC_lexicon_tags_13 },
    { 0x928c9074, SUC_lexicon_tags_11 },
    { 0xf3ac9076, SUC_lexicon_tags_39 },
    { 0x9c2c9075, SUC_lexicon_tags_9 },
    { 0x6b0c9078, SUC_lexicon_tags_15 },
    { 0x1a8c9076, SUC_lexicon_tags_15 },
    { 0x512c9074, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4c907d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4c9083, SUC_lexicon_tags_25 },
    { 0x852c9084, SUC_lexicon_tags_15 },
    { 0xd96c9083, SUC_lexicon_tags_28 },
    { 0x3ecc9083, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70c908b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872c9095, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94c909c, SUC_lexicon_tags_2 },
    { 0xa9cc909d, SUC_lexicon_tags_33 },
    { 0xc7cc909d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38c90a2, SUC_lexicon_tags_1 },
    { 0x4f4c90a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4c90a6, SUC_lexicon_tags_17 },
    { 0x816c90a7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4c90aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x446c90ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaec90b4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9a0c90b6, SUC_lexicon_tags_16 },
    { 0x93ac90b6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ec90bc, SUC_lexicon_tags_28 },
    { 0xfb6c90bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112c90c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x516c90cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x656c90cd, SUC_lexicon_tags_47 },
    { 0x8ac90cd, SUC_lexicon_tags_2 },
    { 0x6d6c90ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30c90d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ac90d7, SUC_lexicon_tags_7 },
    { 0x188c90d8, SUC_lexicon_tags_40 },
    { 0xe5ec90d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x768c90db, SUC_lexicon_tags_1 },
    { 0xf64c90dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a4c90de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82c90e2, SUC_lexicon_tags_9 },
    { 0xec0c90e2, SUC_lexicon_tags_25 },
    { 0x97ec90e4, SUC_lexicon_tags_17 },
    { 0x480c90e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dec90e8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa04c90ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0c90ed, SUC_lexicon_tags_3 },
    { 0x49cc90ee, SUC_lexicon_tags_57 },
    { 0x9fec90ef, SUC_lexicon_tags_25 },
    { 0x842c90ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cc90f4, SUC_lexicon_tags_68 },
    { 0x6d6c90f5, SUC_lexicon_tags_17 },
    { 0xa42c90f6, SUC_lexicon_tags_1 },
    { 0x21ec90f6, SUC_lexicon_tags_37 },
    { 0x134c90f4, SUC_lexicon_tags_12 },
    { 0x7dec90f5, SUC_lexicon_tags_63 },
    { 0x226c90fa, SUC_lexicon_tags_17 },
    { 0x46ac90fb, SUC_lexicon_tags_174 },
    { 0xb18c90f5, SUC_lexicon_tags_37 },
    { 0x8c0c90fd, SUC_lexicon_tags_45 },
    { 0xd1ec90fd, SUC_lexicon_tags_45 },
    { 0xa3ec90f8, SUC_lexicon_tags_14 },
    { 0x170c90fc, SUC_lexicon_tags_25 },
    { 0xe5ac90fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104c9105, SUC_lexicon_tags_17 },
    { 0x4dec9105, SUC_lexicon_tags_2 },
    { 0xf9ec9107, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986c910d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed6c910f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd60c9111, SUC_lexicon_tags_9 },
    { 0x584c9111, SUC_lexicon_tags_60 },
    { 0xfcec9112, SUC_lexicon_tags_37 },
    { 0xd52c9114, SUC_lexicon_tags_43 },
    { 0xa0ac9114, SUC_lexicon_tags_37 },
    { 0xb32c9115, SUC_lexicon_tags_10 },
    { 0x88cc9115, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94c911b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1aec911d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf6ac911f, SUC_lexicon_tags_2 },
    { 0x42c911f, SUC_lexicon_tags_27 },
    { 0xb8ec9121, SUC_lexicon_tags_30 },
    { 0x8c911f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6c9125, SUC_lexicon_tags_30 },
    { 0x84cc9126, SUC_lexicon_tags_14 },
    { 0x956c9127, SUC_lexicon_tags_4 },
    { 0x52cc9126, SUC_lexicon_tags_66 },
    { 0xf9cc9127, SUC_lexicon_tags_38 },
    { 0x6f2c912a, SUC_lexicon_tags_19 },
    { 0xbbcc912b, SUC_lexicon_tags_15 },
    { 0x15ec912a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2c9134, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfac9137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0c913b, SUC_lexicon_tags_11 },
    { 0x610c913c, SUC_lexicon_tags_33 },
    { 0x69cc913d, SUC_lexicon_tags_64 },
    { 0xfbcc913e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6cc9140, SUC_lexicon_tags_11 },
    { 0x538c9141, SUC_lexicon_tags_39 },
    { 0xd3ec9141, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74c9145, SUC_lexicon_tags_17 },
    { 0xea8c9146, SUC_lexicon_tags_3 },
    { 0xa56c9147, SUC_lexicon_tags_37 },
    { 0x85ec9148, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x70ec914a, SUC_lexicon_tags_77 },
    { 0x85cc914b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x828c914d, SUC_lexicon_tags_37 },
    { 0x4a4c914e, SUC_lexicon_tags_0 },
    { 0xd3ac914f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12ac9151, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x94ac9153, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe50c9155, SUC_lexicon_tags_19 },
    { 0x332c9155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26c915b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcc9163, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc38c9165, SUC_lexicon_tags_7 },
    { 0x944c9166, SUC_lexicon_tags_14 },
    { 0x764c9166, SUC_lexicon_tags_47 },
    { 0xa18c9166, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8c9177, SUC_lexicon_tags_0 },
    { 0x79cc9177, SUC_lexicon_tags_27 },
    { 0x632c9177, SUC_lexicon_tags_17 },
    { 0x2cec917a, SUC_lexicon_tags_15 },
    { 0x586c917a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x650c917e, SUC_lexicon_tags_789 },
    { 0xf00c917e, SUC_lexicon_tags_11 },
    { 0xa8c9180, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cec9185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796c918a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6c9190, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cc919b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2c919e, SUC_lexicon_tags_17 },
    { 0x866c919f, SUC_lexicon_tags_25 },
    { 0xc08c919e, SUC_lexicon_tags_17 },
    { 0xac0c91a0, SUC_lexicon_tags_27 },
    { 0x7f8c919f, SUC_lexicon_tags_11 },
    { 0x7a2c91a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2c91a7, SUC_lexicon_tags_47 },
    { 0x7ec91a8, SUC_lexicon_tags_11 },
    { 0x570c91a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18c91af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x832c91b1, SUC_lexicon_tags_37 },
    { 0xec91b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8c91b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x822c91b9, SUC_lexicon_tags_37 },
    { 0x7e4c91ba, SUC_lexicon_tags_7 },
    { 0x346c91ba, SUC_lexicon_tags_37 },
    { 0xcacc91ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee4c91be, SUC_lexicon_tags_62 },
    { 0x37ec91bf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dec91c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0c91c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0c91cb, SUC_lexicon_tags_14 },
    { 0xcdec91cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2c91d1, SUC_lexicon_tags_14 },
    { 0x6eac91d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x654c91d5, SUC_lexicon_tags_11 },
    { 0x5f2c91d5, SUC_lexicon_tags_39 },
    { 0xf3cc91d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb60c91d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ec91dc, SUC_lexicon_tags_45 },
    { 0x780c91dd, SUC_lexicon_tags_27 },
    { 0x5c4c91dd, SUC_lexicon_tags_27 },
    { 0x876c91df, SUC_lexicon_tags_60 },
    { 0x178c91df, SUC_lexicon_tags_7 },
    { 0x8ac91e1, SUC_lexicon_tags_31 },
    { 0x7a8c91e2, SUC_lexicon_tags_65 },
    { 0x4e4c91e3, SUC_lexicon_tags_17 },
    { 0xe34c91e3, SUC_lexicon_tags_10 },
    { 0xf84c91e5, SUC_lexicon_tags_7 },
    { 0x81cc91e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974c91ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846c91f1, SUC_lexicon_tags_11 },
    { 0xd9cc91f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8cc91f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x184c91f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf14c91f8, SUC_lexicon_tags_14 },
    { 0xb86c91f9, SUC_lexicon_tags_48 },
    { 0xe9ec91fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8e6c91fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216c9203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02c9206, SUC_lexicon_tags_19 },
    { 0xad6c9206, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89ec9209, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x384c920b, SUC_lexicon_tags_12 },
    { 0xbcac920c, SUC_lexicon_tags_55 },
    { 0xcec920c, SUC_lexicon_tags_29 },
    { 0x144c920b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686c9214, SUC_lexicon_tags_14 },
    { 0xb92c9215, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898c9219, SUC_lexicon_tags_15 },
    { 0x176c9219, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cc921d, SUC_lexicon_tags_15 },
    { 0x28c921e, SUC_lexicon_tags_68 },
    { 0x26c921e, SUC_lexicon_tags_17 },
    { 0x440c921f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4bec9222, SUC_lexicon_tags_27 },
    { 0xcc0c9223, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x716c9225, SUC_lexicon_tags_12 },
    { 0x1eec9226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cc9229, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x262c922b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50c922d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x228c922f, SUC_lexicon_tags_8 },
    { 0xe0cc9230, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2c9234, SUC_lexicon_tags_25 },
    { 0xbd0c9235, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854c923a, SUC_lexicon_tags_39 },
    { 0x556c923b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44c9242, SUC_lexicon_tags_29 },
    { 0xdb2c9243, SUC_lexicon_tags_17 },
    { 0x1a6c9244, SUC_lexicon_tags_31 },
    { 0xb26c9243, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cec9248, SUC_lexicon_tags_15 },
    { 0x470c9248, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ac924d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170c9253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0cc9255, SUC_lexicon_tags_13 },
    { 0x8ecc9256, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcc925b, SUC_lexicon_tags_10 },
    { 0x53ac925c, SUC_lexicon_tags_2 },
    { 0xc82c925d, SUC_lexicon_tags_37 },
    { 0x172c925c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268c9264, SUC_lexicon_tags_46 },
    { 0xa9ec9265, SUC_lexicon_tags_2 },
    { 0x512c9265, SUC_lexicon_tags_12 },
    { 0x954c9267, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d0c9269, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550c926d, SUC_lexicon_tags_25 },
    { 0x662c926e, SUC_lexicon_tags_47 },
    { 0x86c926e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x474c9271, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1aec9273, SUC_lexicon_tags_9 },
    { 0x8b6c9274, SUC_lexicon_tags_7 },
    { 0x794c9273, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcc9278, SUC_lexicon_tags_13 },
    { 0x8d0c9278, SUC_lexicon_tags_4 },
    { 0xeec927a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6c9280, SUC_lexicon_tags_1 },
    { 0x9ac9281, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x132c9283, SUC_lexicon_tags_16 },
    { 0x51ec9284, SUC_lexicon_tags_15 },
    { 0x55ec9285, SUC_lexicon_tags_21 },
    { 0xacc9285, SUC_lexicon_tags_37 },
    { 0x936c9284, SUC_lexicon_tags_38 },
    { 0xda8c9288, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ec928b, SUC_lexicon_tags_17 },
    { 0xd40c928c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6cc928e, SUC_lexicon_tags_11 },
    { 0x5eac928e, SUC_lexicon_tags_39 },
    { 0x3e2c928e, SUC_lexicon_tags_55 },
    { 0xabac9291, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa00c9293, SUC_lexicon_tags_38 },
    { 0x4dec9293, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x702c9296, SUC_lexicon_tags_14 },
    { 0xe68c9296, SUC_lexicon_tags_32 },
    { 0x960c9298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cc929d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd0c929f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e4c92a1, SUC_lexicon_tags_27 },
    { 0xb6ac92a2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14c92a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf76c92a8, SUC_lexicon_tags_33 },
    { 0xdcac92a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890c92b0, SUC_lexicon_tags_35 },
    { 0x916c92b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4c92b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4c92b7, SUC_lexicon_tags_45 },
    { 0x3dac92b7, SUC_lexicon_tags_12 },
    { 0xf02c92b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ac92bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cc92c1, SUC_lexicon_tags_0 },
    { 0xd6cc92c2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bec92c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26c92d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b8c92d2, SUC_lexicon_tags_14 },
    { 0x25ac92d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x970c92d5, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x484c92d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2c92db, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x59ec92dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228c92e1, SUC_lexicon_tags_37 },
    { 0xad4c92e1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x684c92e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ec92e9, SUC_lexicon_tags_27 },
    { 0x628c92e9, SUC_lexicon_tags_66 },
    { 0x8c92e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x572c92ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218c92f5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2c92f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04c92fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cc9304, SUC_lexicon_tags_27 },
    { 0xe2c9305, SUC_lexicon_tags_7 },
    { 0x95ac9305, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ac9311, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc3ec9313, SUC_lexicon_tags_25 },
    { 0xae0c9314, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20c931d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4c9323, SUC_lexicon_tags_11 },
    { 0x6cc9324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34cc9326, SUC_lexicon_tags_11 },
    { 0xa08c9327, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x98ac9329, SUC_lexicon_tags_19 },
    { 0x584c932a, SUC_lexicon_tags_70 },
    { 0x362c9329, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74c9330, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf70c9332, SUC_lexicon_tags_2 },
    { 0x77cc9332, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ac9336, SUC_lexicon_tags_35 },
    { 0x98ec9337, SUC_lexicon_tags_11 },
    { 0x8c2c9336, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0c9343, SUC_lexicon_tags_15 },
    { 0xa94c9344, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96ec9346, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6c9349, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcc934c, SUC_lexicon_tags_172 },
    { 0x294c934c, SUC_lexicon_tags_92 },
    { 0x134c934e, SUC_lexicon_tags_37 },
    { 0x67ec934f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cc9353, SUC_lexicon_tags_23 },
    { 0x6b2c9354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd36c9356, SUC_lexicon_tags_141 },
    { 0xd22c9357, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ec935e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46c9362, SUC_lexicon_tags_7 },
    { 0xef4c9362, SUC_lexicon_tags_15 },
    { 0x6b4c9363, SUC_lexicon_tags_1 },
    { 0xa7ec9364, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c9367, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02c936c, SUC_lexicon_tags_15 },
    { 0x13ac936c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfe8c936f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4aac9371, SUC_lexicon_tags_31 },
    { 0xb56c9372, SUC_lexicon_tags_17 },
    { 0xfec9372, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfaac9375, SUC_lexicon_tags_7 },
    { 0xad2c9375, SUC_lexicon_tags_39 },
    { 0x56c9377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9ec9379, SUC_lexicon_tags_56 },
    { 0x584c937a, SUC_lexicon_tags_66 },
    { 0xfb4c937b, SUC_lexicon_tags_1 },
    { 0xfcec937b, SUC_lexicon_tags_26 },
    { 0x2f4c937d, SUC_lexicon_tags_11 },
    { 0x750c937d, SUC_lexicon_tags_25 },
    { 0xd90c937f, SUC_lexicon_tags_47 },
    { 0x632c9380, SUC_lexicon_tags_29 },
    { 0x66cc9380, SUC_lexicon_tags_2 },
    { 0x28c9380, SUC_lexicon_tags_25 },
    { 0xfe4c937b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf10c9385, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ac938e, SUC_lexicon_tags_2 },
    { 0xc0ec938f, SUC_lexicon_tags_11 },
    { 0x936c9390, SUC_lexicon_tags_48 },
    { 0x766c938f, SUC_lexicon_tags_55 },
    { 0xf96c938e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa92c9394, SUC_lexicon_tags_15 },
    { 0xed8c9395, SUC_lexicon_tags_23 },
    { 0x624c9396, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cc939a, SUC_lexicon_tags_43 },
    { 0x9c8c939b, SUC_lexicon_tags_25 },
    { 0x6d2c939c, SUC_lexicon_tags_57 },
    { 0xe50c939c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7e0c939f, SUC_lexicon_tags_1 },
    { 0xaa2c93a0, SUC_lexicon_tags_21 },
    { 0xd8cc93a0, SUC_lexicon_tags_2 },
    { 0x75ec93a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ec93a6, SUC_lexicon_tags_15 },
    { 0xadec93a7, SUC_lexicon_tags_63 },
    { 0x974c93a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ac93ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x594c93ad, SUC_lexicon_tags_19 },
    { 0xc20c93ad, SUC_lexicon_tags_2 },
    { 0xa8cc93ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12c93b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5acc93b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d4c93b9, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2c93bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cc93c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b6c93c3, SUC_lexicon_tags_2 },
    { 0x51ac93c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28c93c6, SUC_lexicon_tags_11 },
    { 0x488c93c7, SUC_lexicon_tags_14 },
    { 0xf60c93c7, SUC_lexicon_tags_61 },
    { 0xa76c93c9, SUC_lexicon_tags_11 },
    { 0x552c93c6, SUC_lexicon_tags_31 },
    { 0x306c93cb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9cac93cd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2c93d1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x242c93d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcc93d6, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x6fcc93d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ac93e0, SUC_lexicon_tags_64 },
    { 0x888c93e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ec93e8, SUC_lexicon_tags_63 },
    { 0x90ac93e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6c93ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bac93ef, SUC_lexicon_tags_23 },
    { 0xc5ac93ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf66c93f2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344c93f7, SUC_lexicon_tags_47 },
    { 0xa76c93f7, SUC_lexicon_tags_275 },
    { 0xfacc93f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa70c93fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762c93ff, SUC_lexicon_tags_12 },
    { 0x2b4c9400, SUC_lexicon_tags_25 },
    { 0xf4cc9400, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x424c9404, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148c9409, SUC_lexicon_tags_57 },
    { 0xd30c940a, SUC_lexicon_tags_25 },
    { 0xe6ec940b, SUC_lexicon_tags_11 },
    { 0x6f4c940c, SUC_lexicon_tags_80 },
    { 0x55cc940d, SUC_lexicon_tags_1 },
    { 0xc4cc940d, SUC_lexicon_tags_46 },
    { 0xc1ec940f, SUC_lexicon_tags_28 },
    { 0x612c940f, SUC_lexicon_tags_15 },
    { 0xdf4c9411, SUC_lexicon_tags_30 },
    { 0xb04c9412, SUC_lexicon_tags_85 },
    { 0x222c9413, SUC_lexicon_tags_17 },
    { 0x41ec9413, SUC_lexicon_tags_108 },
    { 0xbc2c9414, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x238c9418, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x186c941a, SUC_lexicon_tags_30 },
    { 0xa08c941a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcc941f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x718c9421, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4c9429, SUC_lexicon_tags_1 },
    { 0x312c942a, SUC_lexicon_tags_11 },
    { 0x4a4c942a, SUC_lexicon_tags_37 },
    { 0xccc942b, SUC_lexicon_tags_25 },
    { 0x4fec942d, SUC_lexicon_tags_49 },
    { 0x742c942d, SUC_lexicon_tags_37 },
    { 0x1d2c942a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8c9434, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356c943a, SUC_lexicon_tags_27 },
    { 0x172c943b, SUC_lexicon_tags_47 },
    { 0x5d4c943a, SUC_lexicon_tags_17 },
    { 0xcd0c943a, SUC_lexicon_tags_8 },
    { 0xf08c943e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ac9441, SUC_lexicon_tags_45 },
    { 0x9b4c9442, SUC_lexicon_tags_5 },
    { 0x46ec9443, SUC_lexicon_tags_65 },
    { 0xec0c9443, SUC_lexicon_tags_19 },
    { 0xb76c9444, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x590c9448, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x776c944a, SUC_lexicon_tags_15 },
    { 0x94ac944a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dec9451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2ac9453, SUC_lexicon_tags_19 },
    { 0x66c9454, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0c945a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83ac945c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eac945f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cc9462, SUC_lexicon_tags_1 },
    { 0x810c9463, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bcc9465, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x546c9467, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcc946c, SUC_lexicon_tags_71 },
    { 0x5a0c946c, SUC_lexicon_tags_12 },
    { 0x682c946e, SUC_lexicon_tags_31 },
    { 0x4c8c946e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04c9473, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cc9479, SUC_lexicon_tags_43 },
    { 0x8c2c947a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcc9482, SUC_lexicon_tags_25 },
    { 0x74c9483, SUC_lexicon_tags_236 },
    { 0xcb6c9484, SUC_lexicon_tags_15 },
    { 0xf8ec9484, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafac948b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cc948f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626c9493, SUC_lexicon_tags_29 },
    { 0x926c9494, SUC_lexicon_tags_37 },
    { 0x40c9493, SUC_lexicon_tags_12 },
    { 0xd1cc9496, SUC_lexicon_tags_27 },
    { 0x586c9497, SUC_lexicon_tags_17 },
    { 0xfa0c9498, SUC_lexicon_tags_19 },
    { 0xecec9493, SUC_lexicon_tags_63 },
    { 0x5cc949a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x90ec949c, SUC_lexicon_tags_37 },
    { 0xccac949c, SUC_lexicon_tags_25 },
    { 0x2a0c949c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd06c94a0, SUC_lexicon_tags_15 },
    { 0xdac94a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206c94a9, SUC_lexicon_tags_2 },
    { 0xf0c94aa, SUC_lexicon_tags_38 },
    { 0xae0c94ab, SUC_lexicon_tags_35 },
    { 0x6dcc94ab, SUC_lexicon_tags_29 },
    { 0x74c94ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54c94b0, SUC_lexicon_tags_46 },
    { 0xe84c94b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e4c94b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8c94b6, SUC_lexicon_tags_15 },
    { 0xd30c94b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8cec94b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8c94c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x508c94c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x320c94c8, SUC_lexicon_tags_17 },
    { 0xf06c94c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x16cc94cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46c94ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf6c94d0, SUC_lexicon_tags_25 },
    { 0x2e4c94d1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68c94da, SUC_lexicon_tags_21 },
    { 0xfdac94db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4c94de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8c94e1, SUC_lexicon_tags_34 },
    { 0x270c94e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6c94e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2b2c94e7, SUC_lexicon_tags_138 },
    { 0xe20c94e7, SUC_lexicon_tags_12 },
    { 0x974c94e7, SUC_lexicon_tags_1 },
    { 0xe1ac94ea, SUC_lexicon_tags_1 },
    { 0x23cc94eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468c94f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272c94f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x922c94f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26c94fe, SUC_lexicon_tags_2 },
    { 0x702c94ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172c9504, SUC_lexicon_tags_25 },
    { 0xbf0c9505, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ac9508, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xad6c950a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf22c950c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x188c950e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80c9513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ac9516, SUC_lexicon_tags_46 },
    { 0xe1ec9517, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7a8c9519, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ecc951b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2c951f, SUC_lexicon_tags_27 },
    { 0x79ec9520, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5ac9522, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8c9526, SUC_lexicon_tags_17 },
    { 0x280c9526, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676c9535, SUC_lexicon_tags_10 },
    { 0xe6c9535, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb1ec9538, SUC_lexicon_tags_30 },
    { 0x3a2c9539, SUC_lexicon_tags_27 },
    { 0x5a4c9538, SUC_lexicon_tags_25 },
    { 0x9c8c953b, SUC_lexicon_tags_17 },
    { 0x14ec953a, SUC_lexicon_tags_28 },
    { 0xc8c953d, SUC_lexicon_tags_15 },
    { 0x7bec953d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42c9541, SUC_lexicon_tags_37 },
    { 0x22ac9542, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960c954b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96c954f, SUC_lexicon_tags_45 },
    { 0xf3ac9550, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x978c9553, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560c9559, SUC_lexicon_tags_49 },
    { 0x588c955a, SUC_lexicon_tags_2 },
    { 0x4c4c955b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998c9561, SUC_lexicon_tags_26 },
    { 0xd34c9562, SUC_lexicon_tags_37 },
    { 0x9a6c9563, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38c956b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd2c956d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x792c956f, SUC_lexicon_tags_25 },
    { 0xbdec9570, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ec9574, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb80c9576, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x686c9579, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6ac957b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcac957f, SUC_lexicon_tags_9 },
    { 0x3e0c957f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26c9586, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eac958b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ac958e, SUC_lexicon_tags_43 },
    { 0x788c958e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d4c9591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c4c9593, SUC_lexicon_tags_2 },
    { 0xfdec9594, SUC_lexicon_tags_14 },
    { 0xf30c9595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cc9599, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ec959c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454c95a0, SUC_lexicon_tags_12 },
    { 0x314c95a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6c95a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b0c95a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ac95ac, SUC_lexicon_tags_25 },
    { 0xc1ec95ad, SUC_lexicon_tags_65 },
    { 0xebcc95ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x36ac95b0, SUC_lexicon_tags_25 },
    { 0x92ec95b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202c95b6, SUC_lexicon_tags_12 },
    { 0x4c95b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774c95bc, SUC_lexicon_tags_31 },
    { 0x636c95bd, SUC_lexicon_tags_14 },
    { 0x644c95be, SUC_lexicon_tags_60 },
    { 0x3bcc95bf, SUC_lexicon_tags_9 },
    { 0x79ec95bf, SUC_lexicon_tags_39 },
    { 0xb34c95c1, SUC_lexicon_tags_31 },
    { 0xc64c95c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaac95c8, SUC_lexicon_tags_30 },
    { 0x132c95c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a6c95cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcc95d4, SUC_lexicon_tags_15 },
    { 0x578c95d4, SUC_lexicon_tags_43 },
    { 0x3d8c95d6, SUC_lexicon_tags_87 },
    { 0x694c95d7, SUC_lexicon_tags_25 },
    { 0xedec95d6, SUC_lexicon_tags_15 },
    { 0x3b4c95d7, SUC_lexicon_tags_7 },
    { 0x6fcc95d6, SUC_lexicon_tags_25 },
    { 0x1ccc95db, SUC_lexicon_tags_77 },
    { 0xe2ec95d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576c95e1, SUC_lexicon_tags_37 },
    { 0x89cc95e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x822c95e4, SUC_lexicon_tags_14 },
    { 0xc3ac95e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefec95e9, SUC_lexicon_tags_6 },
    { 0xb28c95ea, SUC_lexicon_tags_49 },
    { 0x226c95eb, SUC_lexicon_tags_25 },
    { 0xbe2c95eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3cc95ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x166c95f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x154c95f3, SUC_lexicon_tags_6 },
    { 0x724c95f4, SUC_lexicon_tags_33 },
    { 0x828c95f4, SUC_lexicon_tags_17 },
    { 0x98c95f6, SUC_lexicon_tags_6 },
    { 0x86ec95f6, SUC_lexicon_tags_37 },
    { 0x566c95f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4c95fd, SUC_lexicon_tags_47 },
    { 0xdc8c95fe, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ac9601, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xabac9603, SUC_lexicon_tags_2 },
    { 0xd22c9603, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0c9609, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc76c960b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40c960e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0c9612, SUC_lexicon_tags_87 },
    { 0x1f4c9613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16c9618, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cc961c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6c961f, SUC_lexicon_tags_25 },
    { 0x76ec961f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x840c9622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cac962a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2c962d, SUC_lexicon_tags_57 },
    { 0x482c962e, SUC_lexicon_tags_9 },
    { 0xb16c962f, SUC_lexicon_tags_28 },
    { 0x246c962d, SUC_lexicon_tags_34 },
    { 0xcacc962d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae0c9633, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ec9639, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8c9640, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80c9646, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcc964a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68c964e, SUC_lexicon_tags_10 },
    { 0x7d8c964f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbbac9651, SUC_lexicon_tags_63 },
    { 0x63ac9652, SUC_lexicon_tags_1 },
    { 0xc2ec9651, SUC_lexicon_tags_27 },
    { 0xe30c9654, SUC_lexicon_tags_11 },
    { 0xfecc9654, SUC_lexicon_tags_87 },
    { 0x946c9656, SUC_lexicon_tags_17 },
    { 0x10ec9654, SUC_lexicon_tags_25 },
    { 0xfeac9658, SUC_lexicon_tags_49 },
    { 0x986c9659, SUC_lexicon_tags_15 },
    { 0x74cc9659, SUC_lexicon_tags_85 },
    { 0x27ac9657, SUC_lexicon_tags_30 },
    { 0x5fcc965b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2c965f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4ec9661, SUC_lexicon_tags_32 },
    { 0x1aac9662, SUC_lexicon_tags_8 },
    { 0x682c9663, SUC_lexicon_tags_19 },
    { 0xe1cc9664, SUC_lexicon_tags_7 },
    { 0x4e0c9665, SUC_lexicon_tags_10 },
    { 0xb8c9665, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6acc9668, SUC_lexicon_tags_17 },
    { 0x33cc9668, SUC_lexicon_tags_15 },
    { 0x99ec9668, SUC_lexicon_tags_29 },
    { 0xf30c9668, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf74c966d, SUC_lexicon_tags_48 },
    { 0x572c966d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306c9674, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4c9678, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eec967b, SUC_lexicon_tags_7 },
    { 0x180c967c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0c9681, SUC_lexicon_tags_9 },
    { 0xc48c9682, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0c9686, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338c968f, SUC_lexicon_tags_15 },
    { 0xde0c968f, SUC_lexicon_tags_1 },
    { 0x4b8c9691, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc6c9693, SUC_lexicon_tags_15 },
    { 0xdb4c9694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34c9697, SUC_lexicon_tags_41 },
    { 0xa50c9697, SUC_lexicon_tags_6 },
    { 0x4c8c9698, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x46c969c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8c96a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeec96a9, SUC_lexicon_tags_7 },
    { 0xee2c96a9, SUC_lexicon_tags_9 },
    { 0x6eec96ab, SUC_lexicon_tags_12 },
    { 0x900c96ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880c96b1, SUC_lexicon_tags_16 },
    { 0x72c96b2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x132c96b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8c96c0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cac96c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ec96c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8c96cb, SUC_lexicon_tags_8 },
    { 0xc0c96cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x484c96cf, SUC_lexicon_tags_14 },
    { 0x2b0c96d0, SUC_lexicon_tags_47 },
    { 0x5dac96d1, SUC_lexicon_tags_77 },
    { 0xf22c96d1, SUC_lexicon_tags_14 },
    { 0x414c96d3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x708c96d6, SUC_lexicon_tags_1 },
    { 0x9f8c96d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8c96dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698c96e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcc96e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64c96e7, SUC_lexicon_tags_11 },
    { 0x700c96e8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecc96eb, SUC_lexicon_tags_27 },
    { 0x54ec96ec, SUC_lexicon_tags_1 },
    { 0xe72c96ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8c96f3, SUC_lexicon_tags_23 },
    { 0x480c96f4, SUC_lexicon_tags_25 },
    { 0x12ac96f4, SUC_lexicon_tags_2 },
    { 0xdc4c96f6, SUC_lexicon_tags_104 },
    { 0x818c96f6, SUC_lexicon_tags_11 },
    { 0xac2c96f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8c96fb, SUC_lexicon_tags_46 },
    { 0x6fac96fc, SUC_lexicon_tags_23 },
    { 0xa94c96fd, SUC_lexicon_tags_32 },
    { 0x19ec96fe, SUC_lexicon_tags_17 },
    { 0x7b0c96fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x408c9701, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x704c9704, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c4c9706, SUC_lexicon_tags_7 },
    { 0x5acc9706, SUC_lexicon_tags_37 },
    { 0xb60c9707, SUC_lexicon_tags_7 },
    { 0x94ec9709, SUC_lexicon_tags_30 },
    { 0x7d4c9709, SUC_lexicon_tags_7 },
    { 0xa08c9709, SUC_lexicon_tags_81 },
    { 0x2bcc970b, SUC_lexicon_tags_1 },
    { 0xee8c970a, SUC_lexicon_tags_12 },
    { 0x90ec970e, SUC_lexicon_tags_37 },
    { 0xc14c970f, SUC_lexicon_tags_16 },
    { 0xfc8c9710, SUC_lexicon_tags_25 },
    { 0x62ac9711, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148c971a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864c971e, SUC_lexicon_tags_26 },
    { 0x90ac971f, SUC_lexicon_tags_11 },
    { 0x1e4c971e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x330c9723, SUC_lexicon_tags_52 },
    { 0x92ac9724, SUC_lexicon_tags_14 },
    { 0x1cec9725, SUC_lexicon_tags_29 },
    { 0xd7ec9726, SUC_lexicon_tags_25 },
    { 0x5bcc9727, SUC_lexicon_tags_40 },
    { 0x900c9728, SUC_lexicon_tags_66 },
    { 0x53ac9726, SUC_lexicon_tags_11 },
    { 0x66cc9725, SUC_lexicon_tags_13 },
    { 0x326c972b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22c972e, SUC_lexicon_tags_33 },
    { 0x38ec972e, SUC_lexicon_tags_27 },
    { 0xc80c9730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26c9735, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882c9743, SUC_lexicon_tags_11 },
    { 0xc46c9743, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2c9748, SUC_lexicon_tags_37 },
    { 0x9e0c9749, SUC_lexicon_tags_37 },
    { 0x372c974a, SUC_lexicon_tags_60 },
    { 0x78c9748, SUC_lexicon_tags_37 },
    { 0x656c974c, SUC_lexicon_tags_11 },
    { 0xcc4c974c, SUC_lexicon_tags_17 },
    { 0xaecc974d, SUC_lexicon_tags_0 },
    { 0x98c974f, SUC_lexicon_tags_30 },
    { 0xd98c9750, SUC_lexicon_tags_0 },
    { 0x88ec974c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x326c9754, SUC_lexicon_tags_6 },
    { 0x4ecc9754, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874c9761, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46c9766, SUC_lexicon_tags_16 },
    { 0x684c9767, SUC_lexicon_tags_49 },
    { 0x3fcc9768, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0xf36c976a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0ac976c, SUC_lexicon_tags_28 },
    { 0x21ec976d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26c9772, SUC_lexicon_tags_60 },
    { 0x128c9773, SUC_lexicon_tags_12 },
    { 0x7d0c9774, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9a0c9776, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd10c9778, SUC_lexicon_tags_25 },
    { 0xdfec9779, SUC_lexicon_tags_17 },
    { 0xe0ac977a, SUC_lexicon_tags_37 },
    { 0x5bcc977b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ac9781, SUC_lexicon_tags_30 },
    { 0xfbac9782, SUC_lexicon_tags_14 },
    { 0xdd4c9781, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bec978a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x364c978c, SUC_lexicon_tags_10 },
    { 0x2d6c978c, SUC_lexicon_tags_33 },
    { 0x41ec978d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0c9792, SUC_lexicon_tags_21 },
    { 0x862c9793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842c979e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x938c97a0, SUC_lexicon_tags_15 },
    { 0x136c97a1, SUC_lexicon_tags_23 },
    { 0xfb2c97a1, SUC_lexicon_tags_17 },
    { 0x416c97a1, SUC_lexicon_tags_12 },
    { 0xc54c97a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x974c97a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x530c97a8, SUC_lexicon_tags_29 },
    { 0x6e4c97a9, SUC_lexicon_tags_65 },
    { 0x256c97aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf9ec97ac, SUC_lexicon_tags_57 },
    { 0x9a6c97ac, SUC_lexicon_tags_27 },
    { 0x452c97ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63ac97b0, SUC_lexicon_tags_65 },
    { 0x75ec97b1, SUC_lexicon_tags_1 },
    { 0xf12c97b2, SUC_lexicon_tags_25 },
    { 0x254c97b3, SUC_lexicon_tags_34 },
    { 0xf7ec97b0, SUC_lexicon_tags_15 },
    { 0xd7ec97b5, SUC_lexicon_tags_39 },
    { 0xc66c97b6, SUC_lexicon_tags_25 },
    { 0xd56c97b6, SUC_lexicon_tags_301 },
    { 0x5a6c97b5, SUC_lexicon_tags_54 },
    { 0x4aec97b0, SUC_lexicon_tags_27 },
    { 0x348c97b0, SUC_lexicon_tags_25 },
    { 0xba8c97b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8c97be, SUC_lexicon_tags_56 },
    { 0xdb0c97be, SUC_lexicon_tags_1 },
    { 0xaaec97bf, SUC_lexicon_tags_2 },
    { 0xab6c97c1, SUC_lexicon_tags_151 },
    { 0x4dec97c2, SUC_lexicon_tags_8 },
    { 0x82ec97c3, SUC_lexicon_tags_11 },
    { 0xe40c97c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x93cc97c6, SUC_lexicon_tags_80 },
    { 0xf04c97c6, SUC_lexicon_tags_29 },
    { 0x31cc97c8, SUC_lexicon_tags_7 },
    { 0xcaac97c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52c97ce, SUC_lexicon_tags_7 },
    { 0xd44c97cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ec97d2, SUC_lexicon_tags_11 },
    { 0x69ac97d3, SUC_lexicon_tags_2 },
    { 0xcf0c97d4, SUC_lexicon_tags_14 },
    { 0xd26c97d5, SUC_lexicon_tags_25 },
    { 0x152c97d4, SUC_lexicon_tags_37 },
    { 0xa5ac97d7, SUC_lexicon_tags_34 },
    { 0xac6c97d7, SUC_lexicon_tags_7 },
    { 0x614c97d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x580c97db, SUC_lexicon_tags_49 },
    { 0x9e8c97db, SUC_lexicon_tags_28 },
    { 0x76ac97db, SUC_lexicon_tags_37 },
    { 0x7d0c97de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cc97e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac97e5, SUC_lexicon_tags_38 },
    { 0xe36c97e5, SUC_lexicon_tags_33 },
    { 0x644c97e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x826c97e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2c97ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xde6c97ee, SUC_lexicon_tags_21 },
    { 0x3f0c97ef, SUC_lexicon_tags_7 },
    { 0x84cc97f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4c97f8, SUC_lexicon_tags_65 },
    { 0xfcec97f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4ac97fb, SUC_lexicon_tags_17 },
    { 0xb52c97fc, SUC_lexicon_tags_32 },
    { 0x7ecc97fc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x238c97ff, SUC_lexicon_tags_14 },
    { 0xdcc97ff, SUC_lexicon_tags_7 },
    { 0xd74c9801, SUC_lexicon_tags_37 },
    { 0x3acc9802, SUC_lexicon_tags_27 },
    { 0x380c9803, SUC_lexicon_tags_15 },
    { 0x4e0c9804, SUC_lexicon_tags_12 },
    { 0x49ac9805, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfe6c9807, SUC_lexicon_tags_17 },
    { 0x4cac9808, SUC_lexicon_tags_68 },
    { 0xa8c9809, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cc980e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x628c9810, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5dcc9812, SUC_lexicon_tags_37 },
    { 0x3a8c9813, SUC_lexicon_tags_27 },
    { 0x10c9814, SUC_lexicon_tags_45 },
    { 0xb5ac9815, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0c981a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ec981f, SUC_lexicon_tags_30 },
    { 0x95cc981f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80c9823, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6cac9825, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6c982c, SUC_lexicon_tags_2 },
    { 0x9a2c982c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73ec982f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23cc9831, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2c9836, SUC_lexicon_tags_46 },
    { 0x280c9837, SUC_lexicon_tags_12 },
    { 0x2eac9838, SUC_lexicon_tags_23 },
    { 0xdccc9838, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x524c983b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cc983e, SUC_lexicon_tags_25 },
    { 0xd1ec983f, SUC_lexicon_tags_25 },
    { 0x554c9840, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x242c9842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ac9846, SUC_lexicon_tags_25 },
    { 0x900c9847, SUC_lexicon_tags_23 },
    { 0x554c9848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78c9850, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x824c9852, SUC_lexicon_tags_4 },
    { 0x5d6c9853, SUC_lexicon_tags_37 },
    { 0x3c0c9852, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd2c9856, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbd0c9858, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eec985b, SUC_lexicon_tags_15 },
    { 0x52ec985c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfc4c985e, SUC_lexicon_tags_114 },
    { 0x88c985f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eac9864, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4ec9866, SUC_lexicon_tags_47 },
    { 0x498c9866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cc9870, SUC_lexicon_tags_47 },
    { 0xdecc9871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c9874, SUC_lexicon_tags_12 },
    { 0xc66c9875, SUC_lexicon_tags_7 },
    { 0x572c9874, SUC_lexicon_tags_15 },
    { 0x692c9877, SUC_lexicon_tags_87 },
    { 0xdecc9878, SUC_lexicon_tags_9 },
    { 0xcd0c9879, SUC_lexicon_tags_1 },
    { 0xd30c9878, SUC_lexicon_tags_66 },
    { 0xe62c987b, SUC_lexicon_tags_15 },
    { 0x334c987b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94c9880, SUC_lexicon_tags_46 },
    { 0xb78c9880, SUC_lexicon_tags_37 },
    { 0x738c9880, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482c9886, SUC_lexicon_tags_32 },
    { 0xdfac9886, SUC_lexicon_tags_7 },
    { 0xb04c9887, SUC_lexicon_tags_55 },
    { 0x8acc9889, SUC_lexicon_tags_37 },
    { 0x544c988a, SUC_lexicon_tags_11 },
    { 0xbaec988b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x404c988d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06c9891, SUC_lexicon_tags_15 },
    { 0x918c9891, SUC_lexicon_tags_7 },
    { 0x5d2c9892, SUC_lexicon_tags_1 },
    { 0x7c6c9891, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28c9899, SUC_lexicon_tags_7 },
    { 0x4b4c989a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ac989d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ec98a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8c98a9, SUC_lexicon_tags_11 },
    { 0xdbac98aa, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52c98b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bac98b6, SUC_lexicon_tags_28 },
    { 0xb58c98b7, SUC_lexicon_tags_6 },
    { 0x466c98b8, SUC_lexicon_tags_62 },
    { 0x31ec98b9, SUC_lexicon_tags_33 },
    { 0x908c98ba, SUC_lexicon_tags_11 },
    { 0xce2c98ba, SUC_lexicon_tags_11 },
    { 0x3d6c98bc, SUC_lexicon_tags_1 },
    { 0xd6c98bd, SUC_lexicon_tags_4 },
    { 0xbaec98b6, SUC_lexicon_tags_8 },
    { 0x724c98be, SUC_lexicon_tags_19 },
    { 0x424c98c0, SUC_lexicon_tags_11 },
    { 0xc18c98ba, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eac98c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4c98c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76c98ca, SUC_lexicon_tags_62 },
    { 0x222c98cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x684c98ce, SUC_lexicon_tags_11 },
    { 0x352c98cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x162c98d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4c98dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ac98e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30c98e7, SUC_lexicon_tags_68 },
    { 0x3e0c98e7, SUC_lexicon_tags_68 },
    { 0xde4c98e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8ac98eb, SUC_lexicon_tags_31 },
    { 0x706c98ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2c98f7, SUC_lexicon_tags_7 },
    { 0xfeec98f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748c98fd, SUC_lexicon_tags_15 },
    { 0x874c98fe, SUC_lexicon_tags_25 },
    { 0xad4c98fe, SUC_lexicon_tags_2 },
    { 0x6fcc98ff, SUC_lexicon_tags_0 },
    { 0x430c9901, SUC_lexicon_tags_37 },
    { 0x18c98ff, SUC_lexicon_tags_27 },
    { 0x3bac98ff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0c9906, SUC_lexicon_tags_7 },
    { 0x68c9907, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20c990d, SUC_lexicon_tags_2 },
    { 0x282c990d, SUC_lexicon_tags_74 },
    { 0x602c990f, SUC_lexicon_tags_40 },
    { 0x1e0c9910, SUC_lexicon_tags_82 },
    { 0xb4ec990f, SUC_lexicon_tags_12 },
    { 0x6aac9912, SUC_lexicon_tags_37 },
    { 0x26ec990f, SUC_lexicon_tags_65 },
    { 0xce8c9914, SUC_lexicon_tags_1 },
    { 0xe02c9915, SUC_lexicon_tags_47 },
    { 0xf2cc990d, SUC_lexicon_tags_23 },
    { 0xcc6c9913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ec991a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac991e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0c9920, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6c9927, SUC_lexicon_tags_1 },
    { 0x452c9928, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x800c992a, SUC_lexicon_tags_402 },
    { 0x922c992b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24c992e, SUC_lexicon_tags_11 },
    { 0x11ec992f, SUC_lexicon_tags_2 },
    { 0x722c9930, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaec9938, SUC_lexicon_tags_12 },
    { 0xf9ac9939, SUC_lexicon_tags_15 },
    { 0xc0cc993a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ac993d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7a4c993f, SUC_lexicon_tags_27 },
    { 0xd96c9940, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cc9944, SUC_lexicon_tags_17 },
    { 0x5c8c9945, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcfcc9947, SUC_lexicon_tags_1 },
    { 0x846c9948, SUC_lexicon_tags_37 },
    { 0xb6ec9949, SUC_lexicon_tags_9 },
    { 0xb84c994a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56c994e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa9ac9950, SUC_lexicon_tags_37 },
    { 0xb0cc9950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dac9958, SUC_lexicon_tags_37 },
    { 0xe4ac9959, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bec9960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ec996b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ec9972, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb08c9974, SUC_lexicon_tags_37 },
    { 0xb7cc9975, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x116c9977, SUC_lexicon_tags_23 },
    { 0x5dec9978, SUC_lexicon_tags_1 },
    { 0x3c6c9978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422c997e, SUC_lexicon_tags_7 },
    { 0xcd2c997e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2c9984, SUC_lexicon_tags_17 },
    { 0x656c9985, SUC_lexicon_tags_25 },
    { 0x3b8c9986, SUC_lexicon_tags_85 },
    { 0x580c9986, SUC_lexicon_tags_11 },
    { 0xa02c9986, SUC_lexicon_tags_12 },
    { 0x214c9987, SUC_lexicon_tags_1 },
    { 0x678c998a, SUC_lexicon_tags_2 },
    { 0x2f2c998b, SUC_lexicon_tags_4 },
    { 0x368c9989, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c998f, SUC_lexicon_tags_17 },
    { 0xbfcc9990, SUC_lexicon_tags_31 },
    { 0x342c9991, SUC_lexicon_tags_17 },
    { 0x3bec9992, SUC_lexicon_tags_42 },
    { 0xec8c9993, SUC_lexicon_tags_45 },
    { 0xe7ac9992, SUC_lexicon_tags_11 },
    { 0xcf4c9994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88c999b, SUC_lexicon_tags_12 },
    { 0x2a8c999b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaac99a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccc99a8, SUC_lexicon_tags_33 },
    { 0xee2c99a8, SUC_lexicon_tags_27 },
    { 0xdf4c99aa, SUC_lexicon_tags_43 },
    { 0x79cc99aa, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xef6c99ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762c99b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20ac99b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f8c99b6, SUC_lexicon_tags_37 },
    { 0x828c99b7, SUC_lexicon_tags_7 },
    { 0xb06c99b8, SUC_lexicon_tags_17 },
    { 0xaa8c99b9, SUC_lexicon_tags_2 },
    { 0xfa0c99ba, SUC_lexicon_tags_15 },
    { 0x394c99bb, SUC_lexicon_tags_17 },
    { 0xb7ac99bb, SUC_lexicon_tags_17 },
    { 0x112c99b9, SUC_lexicon_tags_17 },
    { 0xa3ec99be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6dcc99c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52c99c4, SUC_lexicon_tags_12 },
    { 0xf5ac99c4, SUC_lexicon_tags_11 },
    { 0xf4c99c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x438c99c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x620c99cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf82c99cd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78c99d0, SUC_lexicon_tags_47 },
    { 0x360c99d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c8c99d3, SUC_lexicon_tags_2 },
    { 0xf24c99d3, SUC_lexicon_tags_7 },
    { 0xe8cc99d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa30c99d7, SUC_lexicon_tags_29 },
    { 0x582c99d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x786c99da, SUC_lexicon_tags_12 },
    { 0xfd8c99db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ec99df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34cc99e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24c99e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5b6c99e6, SUC_lexicon_tags_117 },
    { 0x4bac99e7, SUC_lexicon_tags_34 },
    { 0xc58c99e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x516c99ea, SUC_lexicon_tags_23 },
    { 0xe64c99ea, SUC_lexicon_tags_25 },
    { 0x652c99ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0c99f9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ec99fd, SUC_lexicon_tags_2 },
    { 0x2c6c99fe, SUC_lexicon_tags_91 },
    { 0x2c6c99fe, SUC_lexicon_tags_2 },
    { 0x5eac99ff, SUC_lexicon_tags_12 },
    { 0x170c99fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x38cc9a03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c9a0a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x324c9a0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0c9a0f, SUC_lexicon_tags_12 },
    { 0x304c9a0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6c9a16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9f2c9a18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x156c9a1a, SUC_lexicon_tags_6 },
    { 0x686c9a1a, SUC_lexicon_tags_4 },
    { 0x3f0c9a1c, SUC_lexicon_tags_1 },
    { 0x90ac9a1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24c9a1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ac9a23, SUC_lexicon_tags_7 },
    { 0x60ac9a24, SUC_lexicon_tags_4 },
    { 0xd3ec9a24, SUC_lexicon_tags_57 },
    { 0xedec9a26, SUC_lexicon_tags_17 },
    { 0xed0c9a27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x400c9a2a, SUC_lexicon_tags_32 },
    { 0xac2c9a2b, SUC_lexicon_tags_14 },
    { 0x5f0c9a2a, SUC_lexicon_tags_3 },
    { 0xad4c9a2d, SUC_lexicon_tags_27 },
    { 0x810c9a2e, SUC_lexicon_tags_3 },
    { 0xc16c9a2f, SUC_lexicon_tags_7 },
    { 0xf4ec9a2f, SUC_lexicon_tags_11 },
    { 0xb02c9a31, SUC_lexicon_tags_10 },
    { 0x340c9a2c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x754c9a35, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb92c9a37, SUC_lexicon_tags_37 },
    { 0x78ac9a38, SUC_lexicon_tags_2 },
    { 0x572c9a37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70c9a3d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeb6c9a3f, SUC_lexicon_tags_1 },
    { 0x282c9a3f, SUC_lexicon_tags_35 },
    { 0xff0c9a41, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ec9a45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432c9a4a, SUC_lexicon_tags_11 },
    { 0x72cc9a4b, SUC_lexicon_tags_15 },
    { 0xf6ec9a4c, SUC_lexicon_tags_37 },
    { 0xc46c9a4a, SUC_lexicon_tags_9 },
    { 0x770c9a4e, SUC_lexicon_tags_15 },
    { 0x614c9a4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad4c9a51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714c9a56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x768c9a58, SUC_lexicon_tags_1 },
    { 0x13ac9a59, SUC_lexicon_tags_25 },
    { 0x4d2c9a5a, SUC_lexicon_tags_11 },
    { 0x23ec9a58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aac9a5e, SUC_lexicon_tags_7 },
    { 0xef6c9a5f, SUC_lexicon_tags_7 },
    { 0x728c9a5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38c9a66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x672c9a69, SUC_lexicon_tags_38 },
    { 0x1d0c9a6a, SUC_lexicon_tags_37 },
    { 0xe5ec9a6b, SUC_lexicon_tags_45 },
    { 0x64ac9a69, SUC_lexicon_tags_49 },
    { 0x780c9a6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x102c9a70, SUC_lexicon_tags_17 },
    { 0xb58c9a71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ac9a74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4c9a7c, SUC_lexicon_tags_25 },
    { 0x886c9a7d, SUC_lexicon_tags_2 },
    { 0x9c2c9a7d, SUC_lexicon_tags_46 },
    { 0xcec9a7f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x700c9a81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838c9a86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4c9a8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6c9a91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852c9a95, SUC_lexicon_tags_1 },
    { 0x680c9a96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ec9a9d, SUC_lexicon_tags_61 },
    { 0xebec9a9e, SUC_lexicon_tags_27 },
    { 0x7b4c9a9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ac9aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4c9aad, SUC_lexicon_tags_7 },
    { 0xa9cc9aad, SUC_lexicon_tags_1 },
    { 0x1d0c9aaf, SUC_lexicon_tags_11 },
    { 0xc2cc9aae, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948c9ab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2c9abb, SUC_lexicon_tags_38 },
    { 0x142c9abb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ac9ac7, SUC_lexicon_tags_37 },
    { 0x24cc9ac7, SUC_lexicon_tags_47 },
    { 0xea2c9ac9, SUC_lexicon_tags_15 },
    { 0xbf0c9ac9, SUC_lexicon_tags_47 },
    { 0x2ac9acb, SUC_lexicon_tags_12 },
    { 0xeeac9aca, SUC_lexicon_tags_9 },
    { 0x428c9acd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572c9ad1, SUC_lexicon_tags_23 },
    { 0x8fcc9ad2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26c9ad6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8c9ae1, SUC_lexicon_tags_37 },
    { 0xe28c9ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84c9ae7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ec9aea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x992c9aec, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ac9af1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x988c9af4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a2c9af6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23cc9af8, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dac9afb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ec9afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dcc9b00, SUC_lexicon_tags_16 },
    { 0xe94c9b01, SUC_lexicon_tags_11 },
    { 0xc78c9b02, SUC_lexicon_tags_4 },
    { 0xee6c9b02, SUC_lexicon_tags_29 },
    { 0xfa4c9b00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x670c9b06, SUC_lexicon_tags_37 },
    { 0x55ec9b07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90c9b0b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x91ec9b0d, SUC_lexicon_tags_15 },
    { 0xb64c9b0e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x512c9b11, SUC_lexicon_tags_0 },
    { 0x3bec9b11, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cc9b1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x378c9b1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20c9b1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6c9b22, SUC_lexicon_tags_45 },
    { 0xe4ac9b23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ac9b26, SUC_lexicon_tags_19 },
    { 0x1d0c9b27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4cc9b29, SUC_lexicon_tags_37 },
    { 0xc16c9b2a, SUC_lexicon_tags_30 },
    { 0xc7ac9b2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x96ac9b2d, SUC_lexicon_tags_23 },
    { 0xd12c9b2e, SUC_lexicon_tags_87 },
    { 0xe54c9b2f, SUC_lexicon_tags_14 },
    { 0x7a4c9b30, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0c9b34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c8c9b36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x124c9b38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cc9b3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ec9b3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdbcc9b41, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ac9b46, SUC_lexicon_tags_47 },
    { 0xf28c9b47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966c9b4b, SUC_lexicon_tags_11 },
    { 0xc1ec9b4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c2c9b4e, SUC_lexicon_tags_19 },
    { 0x8cec9b4f, SUC_lexicon_tags_47 },
    { 0x694c9b50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7ac9b52, SUC_lexicon_tags_15 },
    { 0x1aec9b53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a8c9b55, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ec9b59, SUC_lexicon_tags_77 },
    { 0x5ccc9b5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x162c9b5c, SUC_lexicon_tags_0 },
    { 0x352c9b5d, SUC_lexicon_tags_19 },
    { 0xc56c9b5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8c9b64, SUC_lexicon_tags_37 },
    { 0xd8ec9b64, SUC_lexicon_tags_60 },
    { 0x8b8c9b66, SUC_lexicon_tags_1 },
    { 0x514c9b67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340c9b6c, SUC_lexicon_tags_45 },
    { 0x386c9b6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50c9b70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ac9b79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc34c9b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86cc9b7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cac9b85, SUC_lexicon_tags_11 },
    { 0xfd2c9b86, SUC_lexicon_tags_33 },
    { 0xdd0c9b87, SUC_lexicon_tags_49 },
    { 0x122c9b87, SUC_lexicon_tags_2 },
    { 0x6f8c9b89, SUC_lexicon_tags_11 },
    { 0xe3ec9b8a, SUC_lexicon_tags_527 },
    { 0xd3ec9b8b, SUC_lexicon_tags_7 },
    { 0x470c9b8c, SUC_lexicon_tags_63 },
    { 0xe64c9b8b, SUC_lexicon_tags_65 },
    { 0x792c9b8e, SUC_lexicon_tags_63 },
    { 0xaeac9b8f, SUC_lexicon_tags_68 },
    { 0x57cc9b89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202c9b97, SUC_lexicon_tags_10 },
    { 0x830c9b98, SUC_lexicon_tags_2 },
    { 0x42cc9b99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48c9b9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x368c9b9e, SUC_lexicon_tags_2 },
    { 0xfa6c9b9f, SUC_lexicon_tags_4 },
    { 0xf76c9ba0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4c9ba6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00c9bb0, SUC_lexicon_tags_23 },
    { 0x8a0c9bb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ac9bb5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8c9bb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ac9bbb, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ac9bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6c9bd0, SUC_lexicon_tags_23 },
    { 0xb08c9bd1, SUC_lexicon_tags_25 },
    { 0x7cec9bd2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32c9bd6, SUC_lexicon_tags_29 },
    { 0x25ec9bd7, SUC_lexicon_tags_16 },
    { 0x51cc9bd7, SUC_lexicon_tags_77 },
    { 0x3ec9bd9, SUC_lexicon_tags_43 },
    { 0x386c9bda, SUC_lexicon_tags_25 },
    { 0x34ec9bda, SUC_lexicon_tags_38 },
    { 0xcb0c9bdb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cc9be4, SUC_lexicon_tags_23 },
    { 0x5e6c9be5, SUC_lexicon_tags_31 },
    { 0xe06c9be4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e0c9be8, SUC_lexicon_tags_7 },
    { 0x76cc9be8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeb2c9beb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78c9bf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0c9bf5, SUC_lexicon_tags_60 },
    { 0x592c9bf5, SUC_lexicon_tags_33 },
    { 0x5dac9bf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ec9bfd, SUC_lexicon_tags_75 },
    { 0x78ac9bfe, SUC_lexicon_tags_38 },
    { 0x9a4c9bfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72c9c03, SUC_lexicon_tags_0 },
    { 0x4fac9c04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccc9c07, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04c9c0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878c9c11, SUC_lexicon_tags_23 },
    { 0x3fec9c12, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ac9c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cc9c1a, SUC_lexicon_tags_1 },
    { 0xf1cc9c1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x452c9c1d, SUC_lexicon_tags_2 },
    { 0x7c4c9c1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x836c9c20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x270c9c23, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10c9c28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800c9c2c, SUC_lexicon_tags_10 },
    { 0xbbac9c2d, SUC_lexicon_tags_32 },
    { 0x40ac9c2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242c9c32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ec9c41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46c9c45, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eac9c48, SUC_lexicon_tags_17 },
    { 0x65cc9c48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258c9c4d, SUC_lexicon_tags_55 },
    { 0xf56c9c4e, SUC_lexicon_tags_32 },
    { 0xcd4c9c4f, SUC_lexicon_tags_74 },
    { 0x924c9c50, SUC_lexicon_tags_7 },
    { 0xc4ec9c4e, SUC_lexicon_tags_17 },
    { 0xa28c9c4f, SUC_lexicon_tags_11 },
    { 0xe62c9c53, SUC_lexicon_tags_17 },
    { 0x16cc9c53, SUC_lexicon_tags_1 },
    { 0xf58c9c53, SUC_lexicon_tags_15 },
    { 0xea8c9c56, SUC_lexicon_tags_37 },
    { 0x88c9c56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44c9c5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd70c9c5c, SUC_lexicon_tags_2 },
    { 0x844c9c5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ec9c63, SUC_lexicon_tags_9 },
    { 0x5d0c9c64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776c9c6c, SUC_lexicon_tags_8 },
    { 0x20cc9c6c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcc9c70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73ac9c72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00c9c79, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0c9c80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a0c9c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96c9c85, SUC_lexicon_tags_17 },
    { 0xe0cc9c86, SUC_lexicon_tags_2 },
    { 0xf30c9c85, SUC_lexicon_tags_9 },
    { 0x27cc9c88, SUC_lexicon_tags_2 },
    { 0x49ac9c88, SUC_lexicon_tags_15 },
    { 0x186c9c86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4c9c8f, SUC_lexicon_tags_1 },
    { 0xc04c9c8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914c9c94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338c9c99, SUC_lexicon_tags_11 },
    { 0x25ac9c9a, SUC_lexicon_tags_72 },
    { 0x236c9c9a, SUC_lexicon_tags_62 },
    { 0x980c9c9a, SUC_lexicon_tags_68 },
    { 0x3e0c9c9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd06c9c9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ec9ca3, SUC_lexicon_tags_13 },
    { 0x898c9ca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88c9ca7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36c9cac, SUC_lexicon_tags_23 },
    { 0xd3ec9cac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccc9cb0, SUC_lexicon_tags_28 },
    { 0xe88c9cb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4c9cb5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccec9cbb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cc9cc0, SUC_lexicon_tags_250 },
    { 0xeeec9cc1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34c9cc5, SUC_lexicon_tags_15 },
    { 0x67ec9cc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586c9ccc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb7cc9cce, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xfa4c9cd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ec9cd4, SUC_lexicon_tags_126 },
    { 0x4d2c9cd5, SUC_lexicon_tags_30 },
    { 0xa12c9cd6, SUC_lexicon_tags_29 },
    { 0x700c9cd7, SUC_lexicon_tags_17 },
    { 0x654c9cd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x768c9cda, SUC_lexicon_tags_37 },
    { 0x37ec9cdb, SUC_lexicon_tags_47 },
    { 0x182c9cdb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x258c9cdf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82c9ce4, SUC_lexicon_tags_12 },
    { 0x7e8c9ce4, SUC_lexicon_tags_96 },
    { 0xa10c9ce4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6c9cec, SUC_lexicon_tags_95 },
    { 0x930c9cec, SUC_lexicon_tags_226 },
    { 0xd54c9ced, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc9cf3, SUC_lexicon_tags_0 },
    { 0x46ac9cf3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x22ec9cf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cc9cfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8c9d0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x84c9d0d, SUC_lexicon_tags_37 },
    { 0x5a4c9d0e, SUC_lexicon_tags_11 },
    { 0xfacc9d0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54c9d12, SUC_lexicon_tags_25 },
    { 0x8fac9d13, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376c9d1b, SUC_lexicon_tags_74 },
    { 0xdecc9d1b, SUC_lexicon_tags_35 },
    { 0x536c9d1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecc9d23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ac9d27, SUC_lexicon_tags_17 },
    { 0x498c9d27, SUC_lexicon_tags_17 },
    { 0xe4ac9d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeaac9d2b, SUC_lexicon_tags_11 },
    { 0x7d2c9d2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5cc9d2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc9d30, SUC_lexicon_tags_25 },
    { 0x218c9d31, SUC_lexicon_tags_37 },
    { 0x32c9d30, SUC_lexicon_tags_15 },
    { 0xb68c9d33, SUC_lexicon_tags_12 },
    { 0x77ac9d34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb2c9d36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bec9d39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa6c9d3b, SUC_lexicon_tags_57 },
    { 0xabec9d3b, SUC_lexicon_tags_25 },
    { 0xd62c9d3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c6c9d3f, SUC_lexicon_tags_3 },
    { 0x182c9d40, SUC_lexicon_tags_55 },
    { 0xe0c9d41, SUC_lexicon_tags_1 },
    { 0x3c2c9d41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc72c9d44, SUC_lexicon_tags_61 },
    { 0xc2cc9d45, SUC_lexicon_tags_9 },
    { 0xc78c9d44, SUC_lexicon_tags_7 },
    { 0x5b0c9d47, SUC_lexicon_tags_15 },
    { 0x85ec9d45, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0c9d4d, SUC_lexicon_tags_7 },
    { 0xc9ec9d4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x532c9d50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0cc9d52, SUC_lexicon_tags_13 },
    { 0x2a2c9d53, SUC_lexicon_tags_3 },
    { 0x98ac9d54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ac9d58, SUC_lexicon_tags_33 },
    { 0x61ac9d58, SUC_lexicon_tags_17 },
    { 0xbacc9d5a, SUC_lexicon_tags_2 },
    { 0xcdec9d59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36c9d61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e0c9d63, SUC_lexicon_tags_23 },
    { 0xe3ec9d64, SUC_lexicon_tags_19 },
    { 0x5ecc9d65, SUC_lexicon_tags_10 },
    { 0x3e2c9d63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcc9d69, SUC_lexicon_tags_48 },
    { 0xbc0c9d6a, SUC_lexicon_tags_1 },
    { 0xe0ac9d6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62c9d6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ec9d75, SUC_lexicon_tags_6 },
    { 0xfd2c9d75, SUC_lexicon_tags_17 },
    { 0x784c9d77, SUC_lexicon_tags_12 },
    { 0x702c9d76, SUC_lexicon_tags_38 },
    { 0x3f2c9d79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecc9d7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ac9d81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8c9d85, SUC_lexicon_tags_45 },
    { 0x5bec9d86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cc9d8a, SUC_lexicon_tags_38 },
    { 0x234c9d8a, SUC_lexicon_tags_1 },
    { 0x736c9d8c, SUC_lexicon_tags_1 },
    { 0x302c9d8b, SUC_lexicon_tags_106 },
    { 0xceec9d8e, SUC_lexicon_tags_37 },
    { 0xffcc9d8f, SUC_lexicon_tags_28 },
    { 0x5cc9d8e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74c9d93, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5c4c9d95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecac9d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeccc9d9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932c9da1, SUC_lexicon_tags_17 },
    { 0xf32c9da2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36c9da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa82c9da7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb22c9da9, SUC_lexicon_tags_46 },
    { 0x392c9daa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbe2c9dac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf28c9dae, SUC_lexicon_tags_17 },
    { 0x3c0c9dae, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50c9db6, SUC_lexicon_tags_16 },
    { 0x70cc9db6, SUC_lexicon_tags_29 },
    { 0x330c9db8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44c9dbd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96c9dc6, SUC_lexicon_tags_41 },
    { 0xc04c9dc7, SUC_lexicon_tags_65 },
    { 0x61ac9dc8, SUC_lexicon_tags_17 },
    { 0x286c9dc6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308c9dce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18c9dd1, SUC_lexicon_tags_25 },
    { 0xb00c9dd2, SUC_lexicon_tags_433 },
    { 0x512c9dd1, SUC_lexicon_tags_1 },
    { 0xf90c9dd1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x494c9dd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6ec9dd8, SUC_lexicon_tags_2 },
    { 0xceac9dd9, SUC_lexicon_tags_1 },
    { 0xd7ec9dd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6c9dde, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa82c9de0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16c9de3, SUC_lexicon_tags_23 },
    { 0x64c9de4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcc9de9, SUC_lexicon_tags_11 },
    { 0x1cac9dea, SUC_lexicon_tags_12 },
    { 0xd64c9dea, SUC_lexicon_tags_12 },
    { 0xe42c9de9, SUC_lexicon_tags_45 },
    { 0xef6c9dea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8c9df4, SUC_lexicon_tags_12 },
    { 0xe08c9df5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52c9dfa, SUC_lexicon_tags_7 },
    { 0xd96c9dfb, SUC_lexicon_tags_11 },
    { 0x6eac9dfc, SUC_lexicon_tags_1 },
    { 0xfdac9dfc, SUC_lexicon_tags_63 },
    { 0xf68c9dfe, SUC_lexicon_tags_43 },
    { 0xbe8c9dff, SUC_lexicon_tags_49 },
    { 0x22c9e00, SUC_lexicon_tags_59 },
    { 0x882c9dff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d6c9e03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0c9e0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c9e10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x38c9e13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacec9e19, SUC_lexicon_tags_12 },
    { 0x834c9e1a, SUC_lexicon_tags_1 },
    { 0x856c9e19, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2c9e1e, SUC_lexicon_tags_12 },
    { 0xfbec9e1f, SUC_lexicon_tags_25 },
    { 0x3a6c9e1e, SUC_lexicon_tags_9 },
    { 0x276c9e21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00c9e27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cc9e2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ac9e33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ac9e3b, SUC_lexicon_tags_23 },
    { 0x9fac9e3c, SUC_lexicon_tags_12 },
    { 0x9c0c9e3c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfb8c9e3f, SUC_lexicon_tags_8 },
    { 0x360c9e3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e0c9e42, SUC_lexicon_tags_11 },
    { 0x35ac9e42, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x586c9e46, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x26ec9e48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x394c9e4b, SUC_lexicon_tags_39 },
    { 0xcc9e4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4c2c9e4e, SUC_lexicon_tags_1 },
    { 0xdd4c9e4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x658c9e51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe16c9e53, SUC_lexicon_tags_33 },
    { 0x510c9e54, SUC_lexicon_tags_7 },
    { 0x244c9e54, SUC_lexicon_tags_19 },
    { 0x2ac9e56, SUC_lexicon_tags_15 },
    { 0xcdec9e57, SUC_lexicon_tags_777 },
    { 0, NULL },
    { 0xb40c9e59, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x158c9e5c, SUC_lexicon_tags_8 },
    { 0xc08c9e5d, SUC_lexicon_tags_27 },
    { 0x518c9e5e, SUC_lexicon_tags_15 },
    { 0x966c9e5c, SUC_lexicon_tags_12 },
    { 0xb92c9e5d, SUC_lexicon_tags_7 },
    { 0x9fac9e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9ec9e63, SUC_lexicon_tags_63 },
    { 0x662c9e63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dac9e67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800c9e6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28c9e73, SUC_lexicon_tags_14 },
    { 0x420c9e74, SUC_lexicon_tags_15 },
    { 0xb9ec9e75, SUC_lexicon_tags_43 },
    { 0x172c9e76, SUC_lexicon_tags_1 },
    { 0x500c9e76, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340c9e7b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06c9e7e, SUC_lexicon_tags_29 },
    { 0x696c9e7f, SUC_lexicon_tags_1 },
    { 0x492c9e80, SUC_lexicon_tags_9 },
    { 0xa70c9e81, SUC_lexicon_tags_4 },
    { 0x358c9e82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ec9e86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b8c9e88, SUC_lexicon_tags_23 },
    { 0x584c9e88, SUC_lexicon_tags_8 },
    { 0xbc8c9e89, SUC_lexicon_tags_17 },
    { 0xae8c9e8b, SUC_lexicon_tags_7 },
    { 0x55ec9e8c, SUC_lexicon_tags_23 },
    { 0x4b6c9e8d, SUC_lexicon_tags_15 },
    { 0xb8ec9e8e, SUC_lexicon_tags_9 },
    { 0x6eac9e8f, SUC_lexicon_tags_7 },
    { 0xf34c9e90, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ec9e96, SUC_lexicon_tags_33 },
    { 0x386c9e96, SUC_lexicon_tags_12 },
    { 0x5e4c9e98, SUC_lexicon_tags_0 },
    { 0xf0c9e98, SUC_lexicon_tags_25 },
    { 0xcfcc9e9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aec9e9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aec9ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fac9eab, SUC_lexicon_tags_42 },
    { 0x8e8c9eac, SUC_lexicon_tags_0 },
    { 0x6d2c9eab, SUC_lexicon_tags_55 },
    { 0x618c9eac, SUC_lexicon_tags_1 },
    { 0x55ac9eab, SUC_lexicon_tags_11 },
    { 0x91cc9eb0, SUC_lexicon_tags_1 },
    { 0xf92c9eb1, SUC_lexicon_tags_41 },
    { 0xc6ec9eb2, SUC_lexicon_tags_9 },
    { 0x40c9eb3, SUC_lexicon_tags_12 },
    { 0x228c9eb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8ec9eb6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x956c9eb8, SUC_lexicon_tags_14 },
    { 0x4f6c9eb8, SUC_lexicon_tags_35 },
    { 0xf7ac9eb9, SUC_lexicon_tags_15 },
    { 0x16c9eb8, SUC_lexicon_tags_6 },
    { 0x80cc9eb9, SUC_lexicon_tags_60 },
    { 0xee0c9eb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ac9ec0, SUC_lexicon_tags_14 },
    { 0x8a0c9ec0, SUC_lexicon_tags_27 },
    { 0x34c9ec2, SUC_lexicon_tags_2 },
    { 0x95ec9ec3, SUC_lexicon_tags_15 },
    { 0x174c9ec4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16c9ec8, SUC_lexicon_tags_6 },
    { 0xdf6c9ec8, SUC_lexicon_tags_60 },
    { 0x7d8c9eca, SUC_lexicon_tags_17 },
    { 0x51ec9ecb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaec9ed0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x422c9ed3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ac9ed6, SUC_lexicon_tags_27 },
    { 0xb42c9ed7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x38ec9ed9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288c9edf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x384c9ee1, SUC_lexicon_tags_19 },
    { 0xc66c9ee2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02c9ee6, SUC_lexicon_tags_18 },
    { 0xb16c9ee7, SUC_lexicon_tags_48 },
    { 0xeb2c9ee7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x70c9eea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90c9ef4, SUC_lexicon_tags_6 },
    { 0x548c9ef5, SUC_lexicon_tags_30 },
    { 0xde4c9ef6, SUC_lexicon_tags_38 },
    { 0x2e6c9ef7, SUC_lexicon_tags_15 },
    { 0xba8c9ef8, SUC_lexicon_tags_31 },
    { 0xf7ec9ef9, SUC_lexicon_tags_25 },
    { 0xb36c9ef4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0c9efd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74c9eff, SUC_lexicon_tags_26 },
    { 0x122c9eff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556c9f04, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0c9f08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6cc9f0a, SUC_lexicon_tags_37 },
    { 0xd12c9f0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x516c9f0e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6c9f13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x396c9f15, SUC_lexicon_tags_43 },
    { 0x970c9f15, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54c9f1a, SUC_lexicon_tags_40 },
    { 0x15cc9f1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320c9f21, SUC_lexicon_tags_17 },
    { 0x36cc9f21, SUC_lexicon_tags_15 },
    { 0x16cc9f22, SUC_lexicon_tags_11 },
    { 0x6cc9f24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ac9f27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18cc9f2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42c9f2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0c9f33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcbcc9f35, SUC_lexicon_tags_44 },
    { 0x794c9f36, SUC_lexicon_tags_17 },
    { 0x91cc9f35, SUC_lexicon_tags_48 },
    { 0x21ac9f36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafec9f3a, SUC_lexicon_tags_11 },
    { 0xf36c9f3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90c9f40, SUC_lexicon_tags_37 },
    { 0xa2ac9f41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x784c9f44, SUC_lexicon_tags_26 },
    { 0x110c9f45, SUC_lexicon_tags_2 },
    { 0xeccc9f45, SUC_lexicon_tags_43 },
    { 0x6fcc9f45, SUC_lexicon_tags_1 },
    { 0x172c9f44, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa7ec9f4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596c9f4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeec9f51, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02c9f55, SUC_lexicon_tags_9 },
    { 0x73cc9f56, SUC_lexicon_tags_37 },
    { 0xc42c9f55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94c9f5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30c9f5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4c9f61, SUC_lexicon_tags_37 },
    { 0xf7cc9f62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdecc9f64, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ec9f69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x80ec9f6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2c9f71, SUC_lexicon_tags_42 },
    { 0xd9ec9f72, SUC_lexicon_tags_14 },
    { 0x3eac9f72, SUC_lexicon_tags_38 },
    { 0xaacc9f74, SUC_lexicon_tags_32 },
    { 0x7fcc9f75, SUC_lexicon_tags_1 },
    { 0xb32c9f76, SUC_lexicon_tags_17 },
    { 0xc90c9f75, SUC_lexicon_tags_17 },
    { 0xed4c9f75, SUC_lexicon_tags_2 },
    { 0xc18c9f77, SUC_lexicon_tags_7 },
    { 0x476c9f7a, SUC_lexicon_tags_26 },
    { 0x8ac9f7b, SUC_lexicon_tags_23 },
    { 0x2f0c9f7c, SUC_lexicon_tags_4 },
    { 0xa48c9f78, SUC_lexicon_tags_9 },
    { 0x712c9f7e, SUC_lexicon_tags_23 },
    { 0xfe2c9f7e, SUC_lexicon_tags_17 },
    { 0x5dac9f77, SUC_lexicon_tags_47 },
    { 0x700c9f74, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222c9f86, SUC_lexicon_tags_80 },
    { 0x560c9f87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf28c9f89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1bac9f8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194c9f8f, SUC_lexicon_tags_8 },
    { 0xd8c9f8f, SUC_lexicon_tags_63 },
    { 0x1fac9f8f, SUC_lexicon_tags_65 },
    { 0xb2c9f8f, SUC_lexicon_tags_65 },
    { 0x3dcc9f93, SUC_lexicon_tags_25 },
    { 0xab6c9f93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4c9f97, SUC_lexicon_tags_12 },
    { 0xfe6c9f98, SUC_lexicon_tags_25 },
    { 0x84cc9f98, SUC_lexicon_tags_2 },
    { 0x87ec9f98, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xdac9f9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51cc9f9f, SUC_lexicon_tags_17 },
    { 0x5ac9f9f, SUC_lexicon_tags_7 },
    { 0x430c9fa1, SUC_lexicon_tags_1 },
    { 0xec9fa2, SUC_lexicon_tags_49 },
    { 0x806c9fa3, SUC_lexicon_tags_35 },
    { 0x256c9f9f, SUC_lexicon_tags_14 },
    { 0x27ec9fa5, SUC_lexicon_tags_79 },
    { 0x324c9fa5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f2c9fa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc28c9faa, SUC_lexicon_tags_12 },
    { 0xe46c9fab, SUC_lexicon_tags_11 },
    { 0xd70c9fac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4c9fb1, SUC_lexicon_tags_19 },
    { 0xae8c9fb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadac9fb8, SUC_lexicon_tags_2 },
    { 0xdf6c9fb8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe44c9fbb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x71cc9fbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe10c9fbf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x638c9fc1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eac9fc5, SUC_lexicon_tags_37 },
    { 0x2c4c9fc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c9fce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ec9fd2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa66c9fd4, SUC_lexicon_tags_26 },
    { 0x148c9fd4, SUC_lexicon_tags_63 },
    { 0x1a4c9fd6, SUC_lexicon_tags_7 },
    { 0xef8c9fd7, SUC_lexicon_tags_19 },
    { 0x41cc9fd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580c9fdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688c9fe1, SUC_lexicon_tags_2 },
    { 0x790c9fe2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70c9fe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82c9feb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfbcc9fed, SUC_lexicon_tags_19 },
    { 0x84c9fee, SUC_lexicon_tags_57 },
    { 0x3e0c9fef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbcac9ff1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x212c9ff4, SUC_lexicon_tags_17 },
    { 0xd0ec9ff5, SUC_lexicon_tags_69 },
    { 0xc1ec9ff5, SUC_lexicon_tags_6 },
    { 0x1c0c9ff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f6c9ff9, SUC_lexicon_tags_25 },
    { 0x836c9ff9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe2c9ffc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24ca004, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc26ca006, SUC_lexicon_tags_17 },
    { 0xdaaca007, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172ca015, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2eca01a, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9eca01e, SUC_lexicon_tags_14 },
    { 0x8b6ca01f, SUC_lexicon_tags_47 },
    { 0x30aca020, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcf0ca022, SUC_lexicon_tags_11 },
    { 0x704ca023, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aca02a, SUC_lexicon_tags_12 },
    { 0x34cca02a, SUC_lexicon_tags_11 },
    { 0x1cca02c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x504ca02e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1eca037, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe40ca039, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526ca03d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ca040, SUC_lexicon_tags_1 },
    { 0x2bcca041, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb64ca043, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63eca048, SUC_lexicon_tags_57 },
    { 0xd56ca049, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cca04f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x576ca052, SUC_lexicon_tags_92 },
    { 0x8ca052, SUC_lexicon_tags_12 },
    { 0xa54ca054, SUC_lexicon_tags_12 },
    { 0x86aca055, SUC_lexicon_tags_55 },
    { 0x66aca056, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x544ca058, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55aca05d, SUC_lexicon_tags_14 },
    { 0xfb0ca05d, SUC_lexicon_tags_11 },
    { 0xb42ca05f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eca065, SUC_lexicon_tags_12 },
    { 0x7f8ca065, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6baca069, SUC_lexicon_tags_43 },
    { 0x7e2ca06a, SUC_lexicon_tags_2 },
    { 0x38aca06b, SUC_lexicon_tags_19 },
    { 0xe76ca06a, SUC_lexicon_tags_23 },
    { 0xf2eca06a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcc8ca06f, SUC_lexicon_tags_8 },
    { 0x776ca070, SUC_lexicon_tags_1 },
    { 0x96aca070, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2eca073, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08ca078, SUC_lexicon_tags_2 },
    { 0x986ca079, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0ca086, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cca08a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xecaca08c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12aca091, SUC_lexicon_tags_26 },
    { 0xdb6ca091, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x524ca094, SUC_lexicon_tags_7 },
    { 0x34cca095, SUC_lexicon_tags_29 },
    { 0xf54ca095, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x44eca099, SUC_lexicon_tags_1 },
    { 0x440ca09a, SUC_lexicon_tags_63 },
    { 0x8dcca09a, SUC_lexicon_tags_37 },
    { 0xa76ca09c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420ca0a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10ca0a6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbc2ca0a8, SUC_lexicon_tags_47 },
    { 0x992ca0a8, SUC_lexicon_tags_31 },
    { 0xf88ca0aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4ca0ad, SUC_lexicon_tags_17 },
    { 0x43cca0ae, SUC_lexicon_tags_7 },
    { 0x326ca0ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8ca0b2, SUC_lexicon_tags_14 },
    { 0xd8aca0b3, SUC_lexicon_tags_38 },
    { 0x352ca0b2, SUC_lexicon_tags_31 },
    { 0xdbcca0b5, SUC_lexicon_tags_37 },
    { 0xa76ca0b4, SUC_lexicon_tags_7 },
    { 0xd7aca0b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa60ca0b9, SUC_lexicon_tags_23 },
    { 0x6d6ca0ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eca0be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baca0c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceca0ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1eca0d1, SUC_lexicon_tags_26 },
    { 0x8feca0d2, SUC_lexicon_tags_12 },
    { 0xe16ca0d3, SUC_lexicon_tags_38 },
    { 0x828ca0d1, SUC_lexicon_tags_26 },
    { 0x47eca0d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40eca0d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4ca0dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14ca0e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe44ca0e4, SUC_lexicon_tags_2 },
    { 0xfd2ca0e5, SUC_lexicon_tags_1 },
    { 0x148ca0e6, SUC_lexicon_tags_7 },
    { 0xc42ca0e7, SUC_lexicon_tags_15 },
    { 0xb52ca0e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30ca0ed, SUC_lexicon_tags_25 },
    { 0x6aca0ee, SUC_lexicon_tags_1 },
    { 0x226ca0ef, SUC_lexicon_tags_45 },
    { 0x38ca0f0, SUC_lexicon_tags_11 },
    { 0x7f0ca0f0, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cca0f9, SUC_lexicon_tags_4 },
    { 0x416ca0f9, SUC_lexicon_tags_17 },
    { 0x5e2ca0fb, SUC_lexicon_tags_1 },
    { 0x240ca0fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570ca107, SUC_lexicon_tags_2 },
    { 0x234ca108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeca10c, SUC_lexicon_tags_25 },
    { 0xc5aca10c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4eca112, SUC_lexicon_tags_56 },
    { 0xb98ca113, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3eca116, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646ca11b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938ca11f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x556ca121, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6ca128, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdbcca12a, SUC_lexicon_tags_49 },
    { 0x976ca12b, SUC_lexicon_tags_9 },
    { 0x40aca12a, SUC_lexicon_tags_1367 },
    { 0xf6cca12d, SUC_lexicon_tags_45 },
    { 0xffaca12a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8ca134, SUC_lexicon_tags_60 },
    { 0x596ca135, SUC_lexicon_tags_64 },
    { 0x7c8ca135, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ca13d, SUC_lexicon_tags_12 },
    { 0xdc4ca13e, SUC_lexicon_tags_60 },
    { 0x27aca13e, SUC_lexicon_tags_0 },
    { 0xe60ca140, SUC_lexicon_tags_25 },
    { 0x7baca141, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ceca151, SUC_lexicon_tags_14 },
    { 0xb6eca152, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9ecca154, SUC_lexicon_tags_12 },
    { 0x142ca155, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97aca158, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0ca15d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146ca164, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682ca16a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366ca16e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x676ca170, SUC_lexicon_tags_26 },
    { 0xffaca171, SUC_lexicon_tags_25 },
    { 0x49aca172, SUC_lexicon_tags_14 },
    { 0x1d8ca171, SUC_lexicon_tags_49 },
    { 0xc44ca173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4daca179, SUC_lexicon_tags_37 },
    { 0x96aca17a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x562ca17c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb3aca17e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52aca183, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128ca187, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa2eca189, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aca18b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182ca18f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39aca191, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1eca198, SUC_lexicon_tags_7 },
    { 0x1a0ca198, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x528ca19b, SUC_lexicon_tags_17 },
    { 0xcd2ca19b, SUC_lexicon_tags_32 },
    { 0x49eca19c, SUC_lexicon_tags_13 },
    { 0xd92ca19e, SUC_lexicon_tags_17 },
    { 0xb9aca19e, SUC_lexicon_tags_45 },
    { 0x6b2ca19c, SUC_lexicon_tags_25 },
    { 0x31eca1a1, SUC_lexicon_tags_9 },
    { 0xbc0ca19b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb92ca1a4, SUC_lexicon_tags_49 },
    { 0xf48ca1a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a8ca1a7, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734ca1ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbceca1b2, SUC_lexicon_tags_27 },
    { 0xcc2ca1b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77aca1bc, SUC_lexicon_tags_4 },
    { 0x4feca1bd, SUC_lexicon_tags_13 },
    { 0x2ccca1bd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71eca1c4, SUC_lexicon_tags_11 },
    { 0x500ca1c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ca1c9, SUC_lexicon_tags_55 },
    { 0xcb8ca1c9, SUC_lexicon_tags_37 },
    { 0xf5cca1ca, SUC_lexicon_tags_47 },
    { 0x63eca1ca, SUC_lexicon_tags_15 },
    { 0x310ca1c9, SUC_lexicon_tags_39 },
    { 0x162ca1ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94ca1d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x982ca1d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22ca1dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576ca1e2, SUC_lexicon_tags_17 },
    { 0x250ca1e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6ca1e8, SUC_lexicon_tags_17 },
    { 0xf4ca1e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8ca1f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x920ca1f4, SUC_lexicon_tags_34 },
    { 0xd98ca1f5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88ca1f9, SUC_lexicon_tags_0 },
    { 0xd70ca1fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ca1fd, SUC_lexicon_tags_32 },
    { 0x608ca1fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb78ca200, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeca204, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cca207, SUC_lexicon_tags_15 },
    { 0x924ca208, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ca20e, SUC_lexicon_tags_27 },
    { 0x1baca20f, SUC_lexicon_tags_17 },
    { 0xc1cca20f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28ca213, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8ca218, SUC_lexicon_tags_47 },
    { 0xff6ca219, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eeca21e, SUC_lexicon_tags_15 },
    { 0xf16ca21f, SUC_lexicon_tags_12 },
    { 0x8b0ca220, SUC_lexicon_tags_17 },
    { 0xf5cca220, SUC_lexicon_tags_15 },
    { 0x3d0ca222, SUC_lexicon_tags_8 },
    { 0xa52ca222, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x144ca226, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6ca228, SUC_lexicon_tags_2 },
    { 0xc1aca228, SUC_lexicon_tags_17 },
    { 0xe40ca22a, SUC_lexicon_tags_55 },
    { 0x512ca22b, SUC_lexicon_tags_148 },
    { 0x614ca22a, SUC_lexicon_tags_1 },
    { 0x44ca22d, SUC_lexicon_tags_17 },
    { 0x9bcca22b, SUC_lexicon_tags_11 },
    { 0x61cca22f, SUC_lexicon_tags_26 },
    { 0xaaca230, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eca234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44ca23a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442ca23e, SUC_lexicon_tags_63 },
    { 0xc82ca23e, SUC_lexicon_tags_68 },
    { 0xe16ca240, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9eca242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98aca247, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cca24c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceca250, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cca256, SUC_lexicon_tags_31 },
    { 0x30cca256, SUC_lexicon_tags_3 },
    { 0x23cca257, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3bcca25a, SUC_lexicon_tags_49 },
    { 0x480ca25b, SUC_lexicon_tags_43 },
    { 0x502ca25c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe30ca25e, SUC_lexicon_tags_17 },
    { 0x42cca25f, SUC_lexicon_tags_37 },
    { 0xae4ca260, SUC_lexicon_tags_37 },
    { 0x73eca25e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66ca264, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69aca267, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x29eca269, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720ca26d, SUC_lexicon_tags_6 },
    { 0x614ca26d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50aca271, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x328ca274, SUC_lexicon_tags_19 },
    { 0x83aca275, SUC_lexicon_tags_14 },
    { 0x1ceca276, SUC_lexicon_tags_32 },
    { 0x5e4ca277, SUC_lexicon_tags_62 },
    { 0xa6ca275, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x534ca27b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cca27e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cca282, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x260ca284, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24eca286, SUC_lexicon_tags_11 },
    { 0x514ca286, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8ca28b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x33eca28e, SUC_lexicon_tags_11 },
    { 0xe4cca28f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2ca293, SUC_lexicon_tags_12 },
    { 0x790ca293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x366ca296, SUC_lexicon_tags_113 },
    { 0x852ca297, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57aca29c, SUC_lexicon_tags_31 },
    { 0x4aaca29d, SUC_lexicon_tags_1 },
    { 0xeb4ca29e, SUC_lexicon_tags_17 },
    { 0xa5aca29d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846ca2a4, SUC_lexicon_tags_79 },
    { 0x920ca2a4, SUC_lexicon_tags_17 },
    { 0x5d6ca2a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0ca2aa, SUC_lexicon_tags_1 },
    { 0xe92ca2aa, SUC_lexicon_tags_25 },
    { 0xbaca2ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x38cca2af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58aca2b2, SUC_lexicon_tags_631 },
    { 0, NULL },
    { 0xec8ca2b4, SUC_lexicon_tags_12 },
    { 0x734ca2b4, SUC_lexicon_tags_17 },
    { 0x76aca2b5, SUC_lexicon_tags_23 },
    { 0xf84ca2b5, SUC_lexicon_tags_17 },
    { 0xcca2b8, SUC_lexicon_tags_12 },
    { 0x56cca2b9, SUC_lexicon_tags_27 },
    { 0x62cca2ba, SUC_lexicon_tags_17 },
    { 0xc76ca2b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeca2c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6ca2c9, SUC_lexicon_tags_63 },
    { 0xa54ca2ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cca2cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1eca2d2, SUC_lexicon_tags_26 },
    { 0x7a0ca2d3, SUC_lexicon_tags_14 },
    { 0xc3aca2d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x676ca2d7, SUC_lexicon_tags_27 },
    { 0x778ca2d8, SUC_lexicon_tags_15 },
    { 0x884ca2d8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x53eca2dc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecca2e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710ca2ea, SUC_lexicon_tags_6 },
    { 0x5faca2eb, SUC_lexicon_tags_129 },
    { 0xa28ca2ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184ca2f3, SUC_lexicon_tags_2 },
    { 0x3d0ca2f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2ca2f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2eaca2fa, SUC_lexicon_tags_33 },
    { 0x454ca2fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3aca2ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52ca306, SUC_lexicon_tags_2 },
    { 0xedeca307, SUC_lexicon_tags_10 },
    { 0x9e6ca308, SUC_lexicon_tags_68 },
    { 0x1a4ca306, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x44ca30b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x450ca30d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40ca311, SUC_lexicon_tags_6 },
    { 0x248ca312, SUC_lexicon_tags_14 },
    { 0x628ca311, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4eca316, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8ca319, SUC_lexicon_tags_66 },
    { 0x4b2ca31a, SUC_lexicon_tags_23 },
    { 0x3aaca31b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8e4ca31d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8ca320, SUC_lexicon_tags_2 },
    { 0xbf6ca321, SUC_lexicon_tags_4 },
    { 0xc9eca322, SUC_lexicon_tags_7 },
    { 0x168ca323, SUC_lexicon_tags_47 },
    { 0xe34ca323, SUC_lexicon_tags_47 },
    { 0xce4ca324, SUC_lexicon_tags_0 },
    { 0x44aca321, SUC_lexicon_tags_17 },
    { 0xacaca326, SUC_lexicon_tags_1 },
    { 0x494ca327, SUC_lexicon_tags_10 },
    { 0x8a6ca329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaca32c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cca330, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910ca335, SUC_lexicon_tags_17 },
    { 0x66ca336, SUC_lexicon_tags_15 },
    { 0x38eca337, SUC_lexicon_tags_1 },
    { 0x6beca338, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ca33c, SUC_lexicon_tags_29 },
    { 0x64cca33c, SUC_lexicon_tags_11 },
    { 0x86eca33e, SUC_lexicon_tags_12 },
    { 0x946ca33f, SUC_lexicon_tags_38 },
    { 0x22cca33e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x194ca342, SUC_lexicon_tags_23 },
    { 0x580ca343, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90ca349, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6ca34c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eca352, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6ca356, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62aca35b, SUC_lexicon_tags_14 },
    { 0x26cca35b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720ca365, SUC_lexicon_tags_29 },
    { 0x210ca365, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eca369, SUC_lexicon_tags_26 },
    { 0x370ca36a, SUC_lexicon_tags_2 },
    { 0xf5cca369, SUC_lexicon_tags_8 },
    { 0xf6ca36c, SUC_lexicon_tags_65 },
    { 0xc1cca36d, SUC_lexicon_tags_25 },
    { 0xdbaca36e, SUC_lexicon_tags_12 },
    { 0x45eca36d, SUC_lexicon_tags_25 },
    { 0x6b0ca370, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x332ca372, SUC_lexicon_tags_26 },
    { 0xb9eca373, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866ca378, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcca37f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x834ca381, SUC_lexicon_tags_43 },
    { 0x7d2ca382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0ca386, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe8ca388, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecca392, SUC_lexicon_tags_27 },
    { 0x31cca393, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06ca396, SUC_lexicon_tags_25 },
    { 0x96eca397, SUC_lexicon_tags_15 },
    { 0xc88ca397, SUC_lexicon_tags_37 },
    { 0x514ca399, SUC_lexicon_tags_25 },
    { 0xf60ca39a, SUC_lexicon_tags_17 },
    { 0xe4aca399, SUC_lexicon_tags_6 },
    { 0x8a2ca39b, SUC_lexicon_tags_23 },
    { 0x41eca39d, SUC_lexicon_tags_7 },
    { 0xd1aca39e, SUC_lexicon_tags_27 },
    { 0x5aca39e, SUC_lexicon_tags_0 },
    { 0xb8aca3a0, SUC_lexicon_tags_0 },
    { 0xaf8ca39e, SUC_lexicon_tags_11 },
    { 0x48ca39c, SUC_lexicon_tags_0 },
    { 0xb02ca39c, SUC_lexicon_tags_17 },
    { 0xaca3a4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x36eca3a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeca3b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34eca3b4, SUC_lexicon_tags_28 },
    { 0x422ca3b5, SUC_lexicon_tags_1 },
    { 0xf30ca3b6, SUC_lexicon_tags_37 },
    { 0x78ca3b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x716ca3b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca3bc, SUC_lexicon_tags_11 },
    { 0x1a4ca3bd, SUC_lexicon_tags_11 },
    { 0x50aca3be, SUC_lexicon_tags_45 },
    { 0xa4aca3bf, SUC_lexicon_tags_47 },
    { 0x694ca3c0, SUC_lexicon_tags_4 },
    { 0x854ca3c0, SUC_lexicon_tags_4 },
    { 0x2daca3bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94ca3c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42aca3c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6ca3cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baca3d1, SUC_lexicon_tags_6 },
    { 0x50aca3d1, SUC_lexicon_tags_15 },
    { 0x156ca3d3, SUC_lexicon_tags_25 },
    { 0x41cca3d1, SUC_lexicon_tags_16 },
    { 0x36eca3d1, SUC_lexicon_tags_17 },
    { 0x40eca3d6, SUC_lexicon_tags_7 },
    { 0xdfeca3d7, SUC_lexicon_tags_30 },
    { 0x5deca3d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaca3dd, SUC_lexicon_tags_15 },
    { 0x964ca3dd, SUC_lexicon_tags_7 },
    { 0x276ca3df, SUC_lexicon_tags_25 },
    { 0x3c6ca3e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4ca3e4, SUC_lexicon_tags_2 },
    { 0x810ca3e5, SUC_lexicon_tags_23 },
    { 0x796ca3e6, SUC_lexicon_tags_13 },
    { 0xf4cca3e5, SUC_lexicon_tags_2 },
    { 0xb88ca3e6, SUC_lexicon_tags_15 },
    { 0x652ca3e9, SUC_lexicon_tags_10 },
    { 0xd4cca3ea, SUC_lexicon_tags_15 },
    { 0xf24ca3e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd44ca3ed, SUC_lexicon_tags_38 },
    { 0x328ca3ee, SUC_lexicon_tags_63 },
    { 0x104ca3ef, SUC_lexicon_tags_11 },
    { 0x7d6ca3f0, SUC_lexicon_tags_40 },
    { 0xfeaca3ed, SUC_lexicon_tags_34 },
    { 0x7c2ca3f2, SUC_lexicon_tags_6 },
    { 0xc74ca3f3, SUC_lexicon_tags_60 },
    { 0xaa8ca3ed, SUC_lexicon_tags_47 },
    { 0x492ca3ee, SUC_lexicon_tags_1 },
    { 0x53aca3f6, SUC_lexicon_tags_7 },
    { 0x52eca3f7, SUC_lexicon_tags_13 },
    { 0xf52ca3f6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xc9eca3fa, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ca3ff, SUC_lexicon_tags_25 },
    { 0xf74ca400, SUC_lexicon_tags_12 },
    { 0xe64ca400, SUC_lexicon_tags_38 },
    { 0x6f0ca401, SUC_lexicon_tags_33 },
    { 0x17eca403, SUC_lexicon_tags_37 },
    { 0xa96ca402, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x50cca406, SUC_lexicon_tags_318 },
    { 0x166ca407, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2ca40e, SUC_lexicon_tags_2 },
    { 0x988ca40f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a0ca411, SUC_lexicon_tags_10 },
    { 0x268ca412, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd64ca414, SUC_lexicon_tags_2 },
    { 0xceaca415, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcca41a, SUC_lexicon_tags_60 },
    { 0x6c2ca41b, SUC_lexicon_tags_6 },
    { 0x3fcca41c, SUC_lexicon_tags_37 },
    { 0x5acca41c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7e0ca41f, SUC_lexicon_tags_17 },
    { 0x33aca420, SUC_lexicon_tags_12 },
    { 0xdb2ca420, SUC_lexicon_tags_37 },
    { 0xe5eca422, SUC_lexicon_tags_46 },
    { 0x8ca41f, SUC_lexicon_tags_9 },
    { 0x8e8ca424, SUC_lexicon_tags_37 },
    { 0xd62ca420, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf6cca427, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1a2ca429, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8ca42f, SUC_lexicon_tags_7 },
    { 0x5ceca42f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68ca433, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x694ca435, SUC_lexicon_tags_7 },
    { 0xff6ca435, SUC_lexicon_tags_15 },
    { 0x51aca437, SUC_lexicon_tags_27 },
    { 0x64cca437, SUC_lexicon_tags_68 },
    { 0xea4ca439, SUC_lexicon_tags_29 },
    { 0xf4eca437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0ca43d, SUC_lexicon_tags_10 },
    { 0xd6cca43d, SUC_lexicon_tags_17 },
    { 0x8ceca43f, SUC_lexicon_tags_63 },
    { 0x50cca43f, SUC_lexicon_tags_15 },
    { 0x8a6ca441, SUC_lexicon_tags_12 },
    { 0x830ca441, SUC_lexicon_tags_1 },
    { 0x218ca43e, SUC_lexicon_tags_10 },
    { 0x250ca444, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2ca447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0ca44e, SUC_lexicon_tags_12 },
    { 0x40cca44f, SUC_lexicon_tags_19 },
    { 0xbceca450, SUC_lexicon_tags_95 },
    { 0x2f2ca450, SUC_lexicon_tags_24 },
    { 0x6cca44f, SUC_lexicon_tags_32 },
    { 0x544ca453, SUC_lexicon_tags_11 },
    { 0x56cca454, SUC_lexicon_tags_11 },
    { 0x408ca455, SUC_lexicon_tags_49 },
    { 0x208ca455, SUC_lexicon_tags_37 },
    { 0x2ccca455, SUC_lexicon_tags_25 },
    { 0xdb2ca458, SUC_lexicon_tags_17 },
    { 0x89cca459, SUC_lexicon_tags_1 },
    { 0xb8aca458, SUC_lexicon_tags_8 },
    { 0x18aca453, SUC_lexicon_tags_11 },
    { 0xa7cca45c, SUC_lexicon_tags_60 },
    { 0xa30ca458, SUC_lexicon_tags_82 },
    { 0x132ca45d, SUC_lexicon_tags_1 },
    { 0x5b0ca45f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3baca468, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83aca46a, SUC_lexicon_tags_33 },
    { 0xf5aca46a, SUC_lexicon_tags_47 },
    { 0xf1aca46c, SUC_lexicon_tags_25 },
    { 0x100ca46d, SUC_lexicon_tags_16 },
    { 0x94eca46e, SUC_lexicon_tags_12 },
    { 0xd88ca46f, SUC_lexicon_tags_66 },
    { 0xaccca46e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcca475, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6ca47c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452ca480, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c6ca482, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99eca486, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x27cca488, SUC_lexicon_tags_1 },
    { 0x986ca489, SUC_lexicon_tags_9 },
    { 0x818ca48a, SUC_lexicon_tags_1 },
    { 0x50cca48b, SUC_lexicon_tags_66 },
    { 0x7b4ca48a, SUC_lexicon_tags_17 },
    { 0xeaaca489, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692ca492, SUC_lexicon_tags_174 },
    { 0x852ca493, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826ca497, SUC_lexicon_tags_11 },
    { 0xca498, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5aca49a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ca49d, SUC_lexicon_tags_60 },
    { 0xdb2ca49e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66eca4a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33cca4a3, SUC_lexicon_tags_54 },
    { 0x9aaca4a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2feca4a6, SUC_lexicon_tags_0 },
    { 0xd06ca4a7, SUC_lexicon_tags_1 },
    { 0x68aca4a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792ca4ac, SUC_lexicon_tags_21 },
    { 0x8b6ca4ad, SUC_lexicon_tags_38 },
    { 0x720ca4ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212ca4b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6c0ca4b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c8ca4bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaca4bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108ca4c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46ca4cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8ca4d4, SUC_lexicon_tags_17 },
    { 0xac2ca4d4, SUC_lexicon_tags_37 },
    { 0xedaca4d6, SUC_lexicon_tags_17 },
    { 0x2a6ca4d7, SUC_lexicon_tags_33 },
    { 0x832ca4d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x294ca4db, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622ca4e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaca4e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x780ca4ea, SUC_lexicon_tags_39 },
    { 0x822ca4eb, SUC_lexicon_tags_8 },
    { 0x1baca4eb, SUC_lexicon_tags_13 },
    { 0xec0ca4ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x804ca4ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536ca4f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb14ca4f7, SUC_lexicon_tags_17 },
    { 0xc1aca4f8, SUC_lexicon_tags_15 },
    { 0x3c8ca4f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aca4ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaa0ca501, SUC_lexicon_tags_27 },
    { 0x972ca501, SUC_lexicon_tags_60 },
    { 0xeecca503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686ca509, SUC_lexicon_tags_25 },
    { 0xb7eca50a, SUC_lexicon_tags_0 },
    { 0x416ca509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x616ca50d, SUC_lexicon_tags_6 },
    { 0x650ca50e, SUC_lexicon_tags_27 },
    { 0xe9aca50f, SUC_lexicon_tags_14 },
    { 0xe98ca50f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcca513, SUC_lexicon_tags_54 },
    { 0xabaca514, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x316ca516, SUC_lexicon_tags_27 },
    { 0x85eca517, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x774ca51a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6ca520, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30ca523, SUC_lexicon_tags_64 },
    { 0x59eca524, SUC_lexicon_tags_15 },
    { 0xddaca524, SUC_lexicon_tags_33 },
    { 0xcaca526, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x64eca528, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b4ca52a, SUC_lexicon_tags_2 },
    { 0x2fcca52b, SUC_lexicon_tags_2 },
    { 0xa8aca52b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x982ca52e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x30cca531, SUC_lexicon_tags_52 },
    { 0xd40ca532, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9fcca534, SUC_lexicon_tags_9 },
    { 0xff0ca534, SUC_lexicon_tags_23 },
    { 0x4f0ca536, SUC_lexicon_tags_7 },
    { 0x69eca536, SUC_lexicon_tags_17 },
    { 0x7caca538, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x412ca53a, SUC_lexicon_tags_17 },
    { 0x814ca53b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2ca53f, SUC_lexicon_tags_15 },
    { 0x872ca540, SUC_lexicon_tags_25 },
    { 0x882ca541, SUC_lexicon_tags_40 },
    { 0x62aca542, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbeca548, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcca54f, SUC_lexicon_tags_12 },
    { 0x3faca550, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32ca55f, SUC_lexicon_tags_1 },
    { 0x3cca560, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd0aca562, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82ca567, SUC_lexicon_tags_15 },
    { 0xecca567, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cca56d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f4ca56f, SUC_lexicon_tags_0 },
    { 0x4eca56f, SUC_lexicon_tags_170 },
    { 0x38ca571, SUC_lexicon_tags_17 },
    { 0x6aaca56f, SUC_lexicon_tags_14 },
    { 0x73eca56f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd8aca575, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x274ca577, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0ca57c, SUC_lexicon_tags_3 },
    { 0x3c8ca57d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6fcca57f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceca582, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcca58c, SUC_lexicon_tags_12 },
    { 0x4f0ca58c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930ca593, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eca598, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaca59c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9beca5a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18ca5aa, SUC_lexicon_tags_14 },
    { 0x654ca5ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e8ca5ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9ceca5af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x548ca5b2, SUC_lexicon_tags_30 },
    { 0x9eca5b3, SUC_lexicon_tags_69 },
    { 0x204ca5b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9aca5b7, SUC_lexicon_tags_1 },
    { 0x5a8ca5b7, SUC_lexicon_tags_47 },
    { 0xfceca5b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cca5c1, SUC_lexicon_tags_38 },
    { 0x94cca5c2, SUC_lexicon_tags_2 },
    { 0x850ca5c1, SUC_lexicon_tags_41 },
    { 0xf9cca5c4, SUC_lexicon_tags_37 },
    { 0x77aca5c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c8ca5c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2ca5ca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8ca5ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f8ca5d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8ca5dd, SUC_lexicon_tags_25 },
    { 0xae6ca5de, SUC_lexicon_tags_32 },
    { 0xbacca5df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104ca5e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa38ca5e6, SUC_lexicon_tags_7 },
    { 0xd8aca5e7, SUC_lexicon_tags_39 },
    { 0x9eeca5e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4ca5eb, SUC_lexicon_tags_45 },
    { 0xc8eca5eb, SUC_lexicon_tags_26 },
    { 0x18aca5ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3baca5ef, SUC_lexicon_tags_39 },
    { 0x334ca5ef, SUC_lexicon_tags_17 },
    { 0xdeca5f1, SUC_lexicon_tags_1 },
    { 0xf28ca5f2, SUC_lexicon_tags_81 },
    { 0xe78ca5f2, SUC_lexicon_tags_65 },
    { 0xcb4ca5f3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3caca5f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2deca5fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34ca5ff, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaca603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6ca608, SUC_lexicon_tags_45 },
    { 0x24aca609, SUC_lexicon_tags_2 },
    { 0x26aca60a, SUC_lexicon_tags_37 },
    { 0xe22ca60b, SUC_lexicon_tags_25 },
    { 0x18eca60c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586ca610, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa56ca612, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcb2ca614, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4ca61e, SUC_lexicon_tags_32 },
    { 0x1f8ca61f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80ca624, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9feca626, SUC_lexicon_tags_19 },
    { 0x584ca626, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4ca62a, SUC_lexicon_tags_12 },
    { 0x17eca62b, SUC_lexicon_tags_15 },
    { 0x562ca62b, SUC_lexicon_tags_19 },
    { 0xbaca62d, SUC_lexicon_tags_17 },
    { 0x85cca62c, SUC_lexicon_tags_26 },
    { 0x492ca62e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950ca635, SUC_lexicon_tags_1 },
    { 0xb50ca635, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3eca638, SUC_lexicon_tags_60 },
    { 0xf14ca639, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbe8ca63b, SUC_lexicon_tags_57 },
    { 0x1f6ca63b, SUC_lexicon_tags_664 },
    { 0x99cca63d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6ca63f, SUC_lexicon_tags_38 },
    { 0x34ca640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf6ca642, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ca647, SUC_lexicon_tags_8 },
    { 0x65cca648, SUC_lexicon_tags_37 },
    { 0xd8ca649, SUC_lexicon_tags_11 },
    { 0x960ca648, SUC_lexicon_tags_12 },
    { 0x5dcca64b, SUC_lexicon_tags_37 },
    { 0xcaaca647, SUC_lexicon_tags_25 },
    { 0xa56ca647, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaca658, SUC_lexicon_tags_17 },
    { 0xfb4ca659, SUC_lexicon_tags_2 },
    { 0x4faca658, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa0cca65c, SUC_lexicon_tags_33 },
    { 0x826ca65d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x860ca660, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4cca662, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734ca66f, SUC_lexicon_tags_11 },
    { 0x9feca66f, SUC_lexicon_tags_31 },
    { 0x636ca671, SUC_lexicon_tags_15 },
    { 0xecca671, SUC_lexicon_tags_17 },
    { 0x7aaca672, SUC_lexicon_tags_15 },
    { 0x772ca674, SUC_lexicon_tags_63 },
    { 0xe2aca674, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eca67b, SUC_lexicon_tags_34 },
    { 0x92cca67c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x600ca67e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecca685, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eca68f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81cca695, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2caca697, SUC_lexicon_tags_49 },
    { 0xfc2ca698, SUC_lexicon_tags_7 },
    { 0xa8eca699, SUC_lexicon_tags_19 },
    { 0x3beca69a, SUC_lexicon_tags_17 },
    { 0xf4cca69b, SUC_lexicon_tags_9 },
    { 0xbcca69c, SUC_lexicon_tags_15 },
    { 0x2d2ca697, SUC_lexicon_tags_12 },
    { 0xaf4ca69e, SUC_lexicon_tags_25 },
    { 0xb1cca69f, SUC_lexicon_tags_25 },
    { 0xcbaca699, SUC_lexicon_tags_9 },
    { 0x7b6ca6a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabeca6a4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5e2ca6a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaeaca6a8, SUC_lexicon_tags_16 },
    { 0xa36ca6a9, SUC_lexicon_tags_30 },
    { 0x988ca6a9, SUC_lexicon_tags_7 },
    { 0xed4ca6ab, SUC_lexicon_tags_7 },
    { 0x596ca6ac, SUC_lexicon_tags_25 },
    { 0x352ca6ad, SUC_lexicon_tags_7 },
    { 0xe16ca6ab, SUC_lexicon_tags_17 },
    { 0x1c4ca6aa, SUC_lexicon_tags_47 },
    { 0x45eca6b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5daca6b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52eca6b4, SUC_lexicon_tags_21 },
    { 0x3a0ca6b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c8ca6b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572ca6c3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6ca6c6, SUC_lexicon_tags_31 },
    { 0x4eca6c6, SUC_lexicon_tags_17 },
    { 0x64aca6c8, SUC_lexicon_tags_69 },
    { 0x11cca6c9, SUC_lexicon_tags_2 },
    { 0x9c4ca6ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ccca6cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632ca6d7, SUC_lexicon_tags_31 },
    { 0x3eca6d8, SUC_lexicon_tags_49 },
    { 0x806ca6d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaca6dc, SUC_lexicon_tags_61 },
    { 0xff8ca6dd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbc4ca6df, SUC_lexicon_tags_6 },
    { 0xe30ca6e0, SUC_lexicon_tags_44 },
    { 0xb6eca6df, SUC_lexicon_tags_37 },
    { 0xbcca6e2, SUC_lexicon_tags_10 },
    { 0xc14ca6e2, SUC_lexicon_tags_2 },
    { 0x2c2ca6e2, SUC_lexicon_tags_84 },
    { 0xb24ca6e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcaca6e7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x428ca6e9, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0x738ca6ec, SUC_lexicon_tags_12 },
    { 0x8f0ca6ed, SUC_lexicon_tags_17 },
    { 0xd70ca6ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x528ca6f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20eca6f2, SUC_lexicon_tags_17 },
    { 0xcbeca6f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aeca6f8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cca700, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30cca702, SUC_lexicon_tags_11 },
    { 0x7ccca703, SUC_lexicon_tags_15 },
    { 0x1aeca704, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260ca708, SUC_lexicon_tags_21 },
    { 0xe98ca709, SUC_lexicon_tags_15 },
    { 0x3eaca709, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x272ca70d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x474ca70f, SUC_lexicon_tags_60 },
    { 0x1a0ca710, SUC_lexicon_tags_45 },
    { 0x9aaca710, SUC_lexicon_tags_4 },
    { 0x20eca710, SUC_lexicon_tags_55 },
    { 0x2ccca713, SUC_lexicon_tags_158 },
    { 0x3faca714, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eca718, SUC_lexicon_tags_1 },
    { 0x854ca719, SUC_lexicon_tags_37 },
    { 0x11eca718, SUC_lexicon_tags_26 },
    { 0x866ca719, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6cca71d, SUC_lexicon_tags_18 },
    { 0x10cca71e, SUC_lexicon_tags_28 },
    { 0x5acca71f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e4ca721, SUC_lexicon_tags_29 },
    { 0xa60ca722, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feca72c, SUC_lexicon_tags_15 },
    { 0xe28ca72d, SUC_lexicon_tags_9 },
    { 0xdf2ca72e, SUC_lexicon_tags_7 },
    { 0x94aca72d, SUC_lexicon_tags_38 },
    { 0x9f0ca72d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x488ca733, SUC_lexicon_tags_14 },
    { 0x8d8ca734, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1deca73b, SUC_lexicon_tags_15 },
    { 0xaca73b, SUC_lexicon_tags_14 },
    { 0x998ca73d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86eca73f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63eca741, SUC_lexicon_tags_25 },
    { 0x35cca742, SUC_lexicon_tags_1 },
    { 0x61eca741, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aca746, SUC_lexicon_tags_61 },
    { 0xa50ca746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aca74c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf98ca74e, SUC_lexicon_tags_37 },
    { 0x8c2ca74f, SUC_lexicon_tags_2 },
    { 0x36cca74f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x788ca753, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x69eca756, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaca75a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132ca75f, SUC_lexicon_tags_13 },
    { 0x5ccca75f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb34ca762, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2ca765, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aca768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4ca76b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaca76e, SUC_lexicon_tags_2 },
    { 0xd10ca76e, SUC_lexicon_tags_32 },
    { 0x440ca770, SUC_lexicon_tags_25 },
    { 0xe4aca771, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe8ca773, SUC_lexicon_tags_8 },
    { 0x64cca774, SUC_lexicon_tags_25 },
    { 0x18cca773, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71aca77a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cca77f, SUC_lexicon_tags_1 },
    { 0xfc6ca780, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1cca782, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x610ca785, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x812ca787, SUC_lexicon_tags_10 },
    { 0x74ca787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4ca78c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69eca78e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28ca791, SUC_lexicon_tags_25 },
    { 0xfc2ca792, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe80ca794, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cca79a, SUC_lexicon_tags_7 },
    { 0x186ca79b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baca79f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aca7a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272ca7a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cca7aa, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x3d2ca7ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf2cca7ae, SUC_lexicon_tags_37 },
    { 0x5baca7ae, SUC_lexicon_tags_11 },
    { 0x93cca7b0, SUC_lexicon_tags_30 },
    { 0x6aca7ae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40ca7b5, SUC_lexicon_tags_43 },
    { 0xebaca7b6, SUC_lexicon_tags_25 },
    { 0x46ca7b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8ca7be, SUC_lexicon_tags_37 },
    { 0x70ca7bf, SUC_lexicon_tags_49 },
    { 0xf98ca7be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4ca7c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5eca7c6, SUC_lexicon_tags_25 },
    { 0xb86ca7c7, SUC_lexicon_tags_37 },
    { 0xb0aca7c8, SUC_lexicon_tags_37 },
    { 0x504ca7c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1d2ca7cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162ca7cf, SUC_lexicon_tags_25 },
    { 0x8e2ca7d0, SUC_lexicon_tags_1 },
    { 0xc58ca7d0, SUC_lexicon_tags_28 },
    { 0x59eca7d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xec4ca7d4, SUC_lexicon_tags_15 },
    { 0x1d0ca7d5, SUC_lexicon_tags_16 },
    { 0x346ca7d6, SUC_lexicon_tags_32 },
    { 0xfb6ca7d6, SUC_lexicon_tags_113 },
    { 0xb42ca7d8, SUC_lexicon_tags_5 },
    { 0x7f0ca7d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36ca7df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcca7e3, SUC_lexicon_tags_4 },
    { 0xf62ca7e4, SUC_lexicon_tags_12 },
    { 0x216ca7e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26eca7ee, SUC_lexicon_tags_2 },
    { 0x6d2ca7ef, SUC_lexicon_tags_47 },
    { 0xf6cca7f0, SUC_lexicon_tags_16 },
    { 0x6eeca7ee, SUC_lexicon_tags_29 },
    { 0xccca7ee, SUC_lexicon_tags_11 },
    { 0xc0cca7f3, SUC_lexicon_tags_25 },
    { 0xeaaca7f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0ca7fa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaca801, SUC_lexicon_tags_45 },
    { 0xab6ca802, SUC_lexicon_tags_61 },
    { 0x768ca803, SUC_lexicon_tags_109 },
    { 0xfa4ca804, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaca813, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4ca816, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45aca81b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x926ca81e, SUC_lexicon_tags_43 },
    { 0x66aca81f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9aca821, SUC_lexicon_tags_11 },
    { 0x60eca821, SUC_lexicon_tags_29 },
    { 0xe70ca823, SUC_lexicon_tags_15 },
    { 0x9feca823, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0ca827, SUC_lexicon_tags_8 },
    { 0x5deca828, SUC_lexicon_tags_63 },
    { 0xc12ca827, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2ca82e, SUC_lexicon_tags_12 },
    { 0xd12ca82e, SUC_lexicon_tags_45 },
    { 0x8b4ca82f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16cca832, SUC_lexicon_tags_10 },
    { 0xd9aca833, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b8ca835, SUC_lexicon_tags_49 },
    { 0x2a8ca835, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06ca847, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x790ca849, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0ca84f, SUC_lexicon_tags_15 },
    { 0xa7aca84f, SUC_lexicon_tags_33 },
    { 0xf32ca84f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18ca85a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9eca85e, SUC_lexicon_tags_37 },
    { 0x7c0ca85e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8cca861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb6ca863, SUC_lexicon_tags_12 },
    { 0x358ca864, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeca86a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feca873, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x890ca875, SUC_lexicon_tags_17 },
    { 0x392ca876, SUC_lexicon_tags_84 },
    { 0x602ca876, SUC_lexicon_tags_37 },
    { 0xb5eca878, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x514ca87a, SUC_lexicon_tags_27 },
    { 0xbccca87b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd54ca87d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6ca880, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132ca888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe2ca88a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ca88d, SUC_lexicon_tags_46 },
    { 0x912ca88e, SUC_lexicon_tags_31 },
    { 0x9fcca88f, SUC_lexicon_tags_31 },
    { 0xf10ca88f, SUC_lexicon_tags_37 },
    { 0x5e8ca891, SUC_lexicon_tags_4 },
    { 0xee0ca892, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd0ca894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfceca89c, SUC_lexicon_tags_45 },
    { 0x91eca89c, SUC_lexicon_tags_7 },
    { 0x5c2ca89c, SUC_lexicon_tags_65 },
    { 0x900ca89f, SUC_lexicon_tags_19 },
    { 0xca6ca8a0, SUC_lexicon_tags_29 },
    { 0xcf2ca8a0, SUC_lexicon_tags_48 },
    { 0x2aca89e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cca8a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72ca8a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ca8ad, SUC_lexicon_tags_515 },
    { 0xfaaca8ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeb8ca8b0, SUC_lexicon_tags_16 },
    { 0x642ca8b0, SUC_lexicon_tags_15 },
    { 0x10cca8b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8eca8b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ca8b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8ca8ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598ca8be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8ca8c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x836ca8c4, SUC_lexicon_tags_4 },
    { 0x244ca8c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4ca8c9, SUC_lexicon_tags_43 },
    { 0xe48ca8c9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cca8ce, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7e0ca8d0, SUC_lexicon_tags_0 },
    { 0x712ca8d1, SUC_lexicon_tags_1 },
    { 0x2feca8d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4ca8d5, SUC_lexicon_tags_63 },
    { 0x6eca8d6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2ca8db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc32ca8dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6ca8e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ca8e7, SUC_lexicon_tags_60 },
    { 0xd26ca8e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x998ca8eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aca8f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2ca8f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x892ca8f8, SUC_lexicon_tags_1 },
    { 0xadeca8f9, SUC_lexicon_tags_8 },
    { 0x542ca8fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdaca8fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x950ca8ff, SUC_lexicon_tags_25 },
    { 0x75eca900, SUC_lexicon_tags_25 },
    { 0xe58ca901, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74ca906, SUC_lexicon_tags_29 },
    { 0x4dcca907, SUC_lexicon_tags_11 },
    { 0x2c8ca908, SUC_lexicon_tags_7 },
    { 0x68aca909, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57eca90b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5aca90d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfceca915, SUC_lexicon_tags_7 },
    { 0xbbeca916, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288ca91e, SUC_lexicon_tags_37 },
    { 0x18aca91f, SUC_lexicon_tags_6 },
    { 0x3d6ca920, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286ca924, SUC_lexicon_tags_17 },
    { 0x6f0ca925, SUC_lexicon_tags_6 },
    { 0x2eaca926, SUC_lexicon_tags_33 },
    { 0xea8ca926, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8ca92b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cca930, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2aca932, SUC_lexicon_tags_27 },
    { 0xc3aca933, SUC_lexicon_tags_37 },
    { 0x7e0ca932, SUC_lexicon_tags_38 },
    { 0x41aca934, SUC_lexicon_tags_9 },
    { 0xb48ca934, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6ca93b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90eca93e, SUC_lexicon_tags_30 },
    { 0x64ca93f, SUC_lexicon_tags_65 },
    { 0xdc2ca940, SUC_lexicon_tags_18 },
    { 0xdb8ca941, SUC_lexicon_tags_2 },
    { 0x13aca942, SUC_lexicon_tags_21 },
    { 0x4beca93f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10ca948, SUC_lexicon_tags_1 },
    { 0xa22ca949, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29cca94b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfd4ca94d, SUC_lexicon_tags_25 },
    { 0xfb4ca94e, SUC_lexicon_tags_32 },
    { 0xf2aca94f, SUC_lexicon_tags_136 },
    { 0xa28ca950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576ca955, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa4ca957, SUC_lexicon_tags_12 },
    { 0x18cca957, SUC_lexicon_tags_12 },
    { 0x866ca959, SUC_lexicon_tags_37 },
    { 0x3b8ca95a, SUC_lexicon_tags_4 },
    { 0x9eeca958, SUC_lexicon_tags_57 },
    { 0xebeca959, SUC_lexicon_tags_10 },
    { 0xcb4ca95d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x540ca95f, SUC_lexicon_tags_11 },
    { 0xfb4ca960, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x888ca962, SUC_lexicon_tags_63 },
    { 0x12ca962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcca969, SUC_lexicon_tags_12 },
    { 0xce8ca96a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38ca970, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x602ca973, SUC_lexicon_tags_37 },
    { 0x6b4ca974, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eeca977, SUC_lexicon_tags_38 },
    { 0x4e2ca978, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6aca97f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc38ca981, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x972ca983, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6ca98b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91aca98d, SUC_lexicon_tags_12 },
    { 0xb5aca98e, SUC_lexicon_tags_44 },
    { 0x750ca98f, SUC_lexicon_tags_27 },
    { 0x87aca98d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xea4ca992, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x992ca994, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72ca99a, SUC_lexicon_tags_37 },
    { 0x228ca99b, SUC_lexicon_tags_13 },
    { 0x1c6ca99c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaca9a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8ca9a6, SUC_lexicon_tags_17 },
    { 0xb86ca9a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86ca9ab, SUC_lexicon_tags_25 },
    { 0x200ca9ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedeca9ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1aca9b1, SUC_lexicon_tags_29 },
    { 0xd3eca9b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8ca9b5, SUC_lexicon_tags_7 },
    { 0xae8ca9b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1eca9bf, SUC_lexicon_tags_17 },
    { 0x7e4ca9bf, SUC_lexicon_tags_0 },
    { 0xda2ca9c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298ca9c9, SUC_lexicon_tags_25 },
    { 0x140ca9c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0ca9cd, SUC_lexicon_tags_25 },
    { 0xf8cca9ce, SUC_lexicon_tags_11 },
    { 0xcccca9cf, SUC_lexicon_tags_26 },
    { 0x31aca9d0, SUC_lexicon_tags_12 },
    { 0x48ca9cf, SUC_lexicon_tags_47 },
    { 0x86ca9d1, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x2acca9d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a8ca9d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cca9d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6aca9db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6ca9e3, SUC_lexicon_tags_49 },
    { 0x842ca9e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfccca9e6, SUC_lexicon_tags_30 },
    { 0xa4aca9e7, SUC_lexicon_tags_33 },
    { 0x3a4ca9e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaca9eb, SUC_lexicon_tags_47 },
    { 0x12eca9eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d6ca9ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xceaca9f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560ca9f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76caa01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34caa05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1cacaa07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe84caa09, SUC_lexicon_tags_17 },
    { 0x5d0caa0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbacaa0c, SUC_lexicon_tags_8 },
    { 0xf1acaa0c, SUC_lexicon_tags_1 },
    { 0x386caa0d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168caa13, SUC_lexicon_tags_60 },
    { 0xf4ecaa13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586caa18, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x256caa1a, SUC_lexicon_tags_20 },
    { 0x6a0caa1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacecaa20, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ecaa29, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8caa30, SUC_lexicon_tags_35 },
    { 0xba6caa30, SUC_lexicon_tags_7 },
    { 0x126caa31, SUC_lexicon_tags_19 },
    { 0xf02caa31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa72caa35, SUC_lexicon_tags_26 },
    { 0xdaccaa36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x604caa38, SUC_lexicon_tags_84 },
    { 0x1d0caa38, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ecaa41, SUC_lexicon_tags_29 },
    { 0xc06caa41, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xececaa46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x572caa49, SUC_lexicon_tags_9 },
    { 0xb8caa4a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfa4caa4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fccaa4f, SUC_lexicon_tags_38 },
    { 0x6eacaa50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x424caa53, SUC_lexicon_tags_23 },
    { 0x1aacaa53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dccaa59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4caa5d, SUC_lexicon_tags_32 },
    { 0xa88caa5e, SUC_lexicon_tags_45 },
    { 0x8cccaa5f, SUC_lexicon_tags_11 },
    { 0xb3ecaa5f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebacaa65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572caa69, SUC_lexicon_tags_60 },
    { 0x1b0caa6a, SUC_lexicon_tags_12 },
    { 0xaf2caa69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30caa71, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2caa78, SUC_lexicon_tags_63 },
    { 0x866caa79, SUC_lexicon_tags_15 },
    { 0xa94caa78, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x67ccaa7c, SUC_lexicon_tags_1 },
    { 0x294caa7d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8caa81, SUC_lexicon_tags_45 },
    { 0x7dacaa82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aecaa8a, SUC_lexicon_tags_17 },
    { 0x706caa8b, SUC_lexicon_tags_11 },
    { 0x6accaa8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8caa92, SUC_lexicon_tags_25 },
    { 0x174caa93, SUC_lexicon_tags_31 },
    { 0xf34caa94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97ecaa96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbecaa9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8caa9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xffccaaa1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48caaa5, SUC_lexicon_tags_65 },
    { 0xe18caaa6, SUC_lexicon_tags_2 },
    { 0x396caaa6, SUC_lexicon_tags_17 },
    { 0x352caaa6, SUC_lexicon_tags_0 },
    { 0x94ecaaa8, SUC_lexicon_tags_7 },
    { 0xdb2caaa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6caab0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4caab6, SUC_lexicon_tags_7 },
    { 0x2bacaab7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810caabd, SUC_lexicon_tags_12 },
    { 0xdfacaabe, SUC_lexicon_tags_46 },
    { 0xb98caabe, SUC_lexicon_tags_7 },
    { 0x744caabe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x620caac2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eacaac7, SUC_lexicon_tags_75 },
    { 0x1d8caac8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d6caaca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfaecaacc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22caad0, SUC_lexicon_tags_46 },
    { 0xaa6caad0, SUC_lexicon_tags_4 },
    { 0xe8ecaad2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a0caad4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x926caad6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x288caad9, SUC_lexicon_tags_15 },
    { 0xd42caad9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926caadf, SUC_lexicon_tags_19 },
    { 0xc12caadf, SUC_lexicon_tags_11 },
    { 0xf9ecaae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x100caae3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380caae7, SUC_lexicon_tags_1 },
    { 0xff4caae7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb6caaea, SUC_lexicon_tags_37 },
    { 0xf1ccaaeb, SUC_lexicon_tags_12 },
    { 0x226caaeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x558caaef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1c8caaf1, SUC_lexicon_tags_28 },
    { 0xd30caaf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ecaaf7, SUC_lexicon_tags_12 },
    { 0x3a4caaf8, SUC_lexicon_tags_15 },
    { 0x7c6caaf8, SUC_lexicon_tags_34 },
    { 0x5f2caafa, SUC_lexicon_tags_37 },
    { 0xbb6caafb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894cab03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a2cab05, SUC_lexicon_tags_27 },
    { 0xd00cab06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4cab0d, SUC_lexicon_tags_38 },
    { 0xc48cab0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cab11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06cab18, SUC_lexicon_tags_83 },
    { 0x376cab19, SUC_lexicon_tags_1 },
    { 0x73ecab1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0cab1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08cab20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ccab25, SUC_lexicon_tags_12 },
    { 0xf22cab26, SUC_lexicon_tags_27 },
    { 0xc62cab27, SUC_lexicon_tags_14 },
    { 0xf4ccab28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b2cab2a, SUC_lexicon_tags_37 },
    { 0x70cab2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ccab34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750cab3d, SUC_lexicon_tags_8 },
    { 0xc60cab3e, SUC_lexicon_tags_25 },
    { 0x77ecab3f, SUC_lexicon_tags_37 },
    { 0x40acab40, SUC_lexicon_tags_7 },
    { 0x30acab3e, SUC_lexicon_tags_46 },
    { 0xad0cab41, SUC_lexicon_tags_12 },
    { 0x68acab43, SUC_lexicon_tags_2 },
    { 0xea2cab3f, SUC_lexicon_tags_11 },
    { 0xdcccab45, SUC_lexicon_tags_37 },
    { 0x64ecab45, SUC_lexicon_tags_17 },
    { 0x5aacab44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67acab4a, SUC_lexicon_tags_29 },
    { 0x6b8cab4b, SUC_lexicon_tags_33 },
    { 0x1b2cab4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4cab51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cab54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9aecab56, SUC_lexicon_tags_1 },
    { 0x1bacab56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ccab59, SUC_lexicon_tags_14 },
    { 0x104cab5a, SUC_lexicon_tags_12 },
    { 0x8fecab5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8ccab5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7acab60, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xa32cab62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eacab66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x442cab68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4decab6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc76cab6f, SUC_lexicon_tags_17 },
    { 0x8d2cab70, SUC_lexicon_tags_17 },
    { 0x6c0cab71, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dacab74, SUC_lexicon_tags_10 },
    { 0x85acab75, SUC_lexicon_tags_69 },
    { 0x68ccab76, SUC_lexicon_tags_3 },
    { 0xb3acab76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728cab85, SUC_lexicon_tags_7 },
    { 0x70acab86, SUC_lexicon_tags_8 },
    { 0x3c6cab87, SUC_lexicon_tags_27 },
    { 0x71ccab87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x296cab8b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3a0cab8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4cab92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d4cab94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e4cab96, SUC_lexicon_tags_7 },
    { 0x3cecab97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aecab9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x756caba0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182caba5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3ccaba7, SUC_lexicon_tags_11 },
    { 0x55ecaba7, SUC_lexicon_tags_37 },
    { 0xc54caba9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20acabad, SUC_lexicon_tags_27 },
    { 0xca6cabad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf30cabb0, SUC_lexicon_tags_37 },
    { 0x278cabb1, SUC_lexicon_tags_10 },
    { 0x800cabb2, SUC_lexicon_tags_29 },
    { 0x8a6cabb3, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ecabb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x900cabba, SUC_lexicon_tags_77 },
    { 0x9fecabba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x844cabbd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ecabc5, SUC_lexicon_tags_11 },
    { 0x28ccabc5, SUC_lexicon_tags_38 },
    { 0xc58cabc7, SUC_lexicon_tags_11 },
    { 0xf40cabc8, SUC_lexicon_tags_31 },
    { 0x3d8cabc6, SUC_lexicon_tags_47 },
    { 0x876cabc5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x11acabcc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xae0cabce, SUC_lexicon_tags_47 },
    { 0x68ecabcf, SUC_lexicon_tags_45 },
    { 0x4a8cabd0, SUC_lexicon_tags_68 },
    { 0x53ecabd1, SUC_lexicon_tags_11 },
    { 0x258cabd1, SUC_lexicon_tags_37 },
    { 0x352cabd0, SUC_lexicon_tags_14 },
    { 0xe52cabd4, SUC_lexicon_tags_47 },
    { 0x534cabd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16cabd8, SUC_lexicon_tags_14 },
    { 0xae2cabd9, SUC_lexicon_tags_30 },
    { 0xaa0cabd9, SUC_lexicon_tags_47 },
    { 0xa0acabd9, SUC_lexicon_tags_1 },
    { 0x922cabdc, SUC_lexicon_tags_11 },
    { 0x9cecabd9, SUC_lexicon_tags_47 },
    { 0xe58cabd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118cabe6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1ecabe8, SUC_lexicon_tags_95 },
    { 0x45acabe9, SUC_lexicon_tags_83 },
    { 0x2c2cabea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204cabef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bccabf3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cecabf6, SUC_lexicon_tags_17 },
    { 0x142cabf7, SUC_lexicon_tags_15 },
    { 0x7aacabf7, SUC_lexicon_tags_43 },
    { 0xc8ecabf9, SUC_lexicon_tags_43 },
    { 0xba8cabfa, SUC_lexicon_tags_0 },
    { 0xf54cabfb, SUC_lexicon_tags_56 },
    { 0xdf2cabfc, SUC_lexicon_tags_11 },
    { 0x2cccabf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0cabff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xffacac02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddacac11, SUC_lexicon_tags_15 },
    { 0x468cac12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734cac17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4cac1d, SUC_lexicon_tags_48 },
    { 0xee0cac1e, SUC_lexicon_tags_49 },
    { 0x8b6cac1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x440cac22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcccac25, SUC_lexicon_tags_11 },
    { 0xec0cac26, SUC_lexicon_tags_37 },
    { 0x126cac25, SUC_lexicon_tags_63 },
    { 0x7b2cac28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd28cac2a, SUC_lexicon_tags_43 },
    { 0xc3ccac2a, SUC_lexicon_tags_32 },
    { 0x984cac2c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x604cac2e, SUC_lexicon_tags_66 },
    { 0x3cecac2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeacac32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aacac3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4cac44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc58cac46, SUC_lexicon_tags_19 },
    { 0x7a4cac46, SUC_lexicon_tags_17 },
    { 0x648cac48, SUC_lexicon_tags_12 },
    { 0x504cac49, SUC_lexicon_tags_16 },
    { 0x970cac48, SUC_lexicon_tags_1 },
    { 0x59acac47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ccac50, SUC_lexicon_tags_17 },
    { 0x35ccac51, SUC_lexicon_tags_37 },
    { 0x66ccac50, SUC_lexicon_tags_1 },
    { 0xbd4cac50, SUC_lexicon_tags_15 },
    { 0x802cac51, SUC_lexicon_tags_61 },
    { 0xfe6cac55, SUC_lexicon_tags_43 },
    { 0x432cac53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ccac5d, SUC_lexicon_tags_37 },
    { 0xf06cac5d, SUC_lexicon_tags_2 },
    { 0xa08cac5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cecac64, SUC_lexicon_tags_60 },
    { 0xc76cac65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366cac6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28acac6c, SUC_lexicon_tags_37 },
    { 0x426cac6d, SUC_lexicon_tags_7 },
    { 0xfb6cac6e, SUC_lexicon_tags_4 },
    { 0x4a4cac6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28cac73, SUC_lexicon_tags_0 },
    { 0x6c4cac74, SUC_lexicon_tags_12 },
    { 0x3becac74, SUC_lexicon_tags_11 },
    { 0x114cac74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8cac7b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xb30cac7d, SUC_lexicon_tags_33 },
    { 0x2d0cac7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf0cac80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04cac85, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb2ecac87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268cac90, SUC_lexicon_tags_37 },
    { 0x4b6cac91, SUC_lexicon_tags_27 },
    { 0xc44cac91, SUC_lexicon_tags_39 },
    { 0xc18cac93, SUC_lexicon_tags_17 },
    { 0x4becac94, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26acac99, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbecac9d, SUC_lexicon_tags_19 },
    { 0xe38cac9e, SUC_lexicon_tags_14 },
    { 0xd62cac9f, SUC_lexicon_tags_32 },
    { 0xa46caca0, SUC_lexicon_tags_15 },
    { 0xe68cac9d, SUC_lexicon_tags_38 },
    { 0x532caca2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96caca6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5ecaca8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x30acacab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6acacb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbecacb8, SUC_lexicon_tags_15 },
    { 0xd62cacb9, SUC_lexicon_tags_7 },
    { 0xcfecacb8, SUC_lexicon_tags_34 },
    { 0xc0acacb9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19acacc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52cacd0, SUC_lexicon_tags_1 },
    { 0x16cacd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3decacd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11acacd8, SUC_lexicon_tags_8 },
    { 0xacecacd9, SUC_lexicon_tags_2 },
    { 0x4f6cacd8, SUC_lexicon_tags_11 },
    { 0x7aacacdb, SUC_lexicon_tags_11 },
    { 0x3d4cacdc, SUC_lexicon_tags_28 },
    { 0x4cccacdc, SUC_lexicon_tags_26 },
    { 0x8e8cacd9, SUC_lexicon_tags_965 },
    { 0xf38cacdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f2cace1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520cace6, SUC_lexicon_tags_15 },
    { 0x3cccace7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf1ccace9, SUC_lexicon_tags_43 },
    { 0xc96cace9, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7acacee, SUC_lexicon_tags_33 },
    { 0xc38cacee, SUC_lexicon_tags_65 },
    { 0x982cacee, SUC_lexicon_tags_38 },
    { 0x53ecacef, SUC_lexicon_tags_1 },
    { 0x734cacf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398cacf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd42cacf8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68cacff, SUC_lexicon_tags_37 },
    { 0xffacacff, SUC_lexicon_tags_638 },
    { 0, NULL },
    { 0xea2cad02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeecad05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6acad08, SUC_lexicon_tags_7 },
    { 0xeeecad08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222cad0f, SUC_lexicon_tags_7 },
    { 0x718cad10, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5f8cad12, SUC_lexicon_tags_31 },
    { 0xf18cad13, SUC_lexicon_tags_25 },
    { 0x12acad14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x488cad17, SUC_lexicon_tags_33 },
    { 0x9d4cad17, SUC_lexicon_tags_87 },
    { 0xfcccad18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84cad1c, SUC_lexicon_tags_43 },
    { 0x44ccad1c, SUC_lexicon_tags_2 },
    { 0x440cad1d, SUC_lexicon_tags_25 },
    { 0xb52cad1f, SUC_lexicon_tags_11 },
    { 0xa1acad1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00cad2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ecad2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632cad32, SUC_lexicon_tags_7 },
    { 0xe4cad33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ecad37, SUC_lexicon_tags_29 },
    { 0x2c8cad37, SUC_lexicon_tags_35 },
    { 0xa90cad39, SUC_lexicon_tags_11 },
    { 0xfc8cad3a, SUC_lexicon_tags_8 },
    { 0xf74cad3b, SUC_lexicon_tags_137 },
    { 0x912cad3c, SUC_lexicon_tags_43 },
    { 0xabccad3c, SUC_lexicon_tags_39 },
    { 0xc28cad3e, SUC_lexicon_tags_37 },
    { 0x496cad3f, SUC_lexicon_tags_17 },
    { 0x766cad40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ccad45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0cad48, SUC_lexicon_tags_7 },
    { 0xf48cad49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4cad4f, SUC_lexicon_tags_28 },
    { 0x65acad50, SUC_lexicon_tags_1 },
    { 0xfdecad51, SUC_lexicon_tags_48 },
    { 0x7fccad52, SUC_lexicon_tags_31 },
    { 0x482cad50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa08cad55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x702cad57, SUC_lexicon_tags_9 },
    { 0x950cad57, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x884cad5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dccad5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cad60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24cad64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x382cad66, SUC_lexicon_tags_37 },
    { 0x348cad67, SUC_lexicon_tags_77 },
    { 0x6a0cad66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520cad6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cad72, SUC_lexicon_tags_15 },
    { 0x6bacad73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cad7c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x262cad7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x714cad80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6cad89, SUC_lexicon_tags_13 },
    { 0x3cacad8a, SUC_lexicon_tags_6 },
    { 0x170cad8b, SUC_lexicon_tags_35 },
    { 0x854cad89, SUC_lexicon_tags_25 },
    { 0x872cad8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60cad92, SUC_lexicon_tags_11 },
    { 0x75ccad92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752cad97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88cad9b, SUC_lexicon_tags_30 },
    { 0x8c4cad9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4cada0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66ccada2, SUC_lexicon_tags_17 },
    { 0xd30cada3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542cadad, SUC_lexicon_tags_39 },
    { 0xa0ccadad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cadb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0cadb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102cadb8, SUC_lexicon_tags_829 },
    { 0, NULL },
    { 0xf4ccadba, SUC_lexicon_tags_15 },
    { 0x43ecadbb, SUC_lexicon_tags_15 },
    { 0xf1acadbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x602cadbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0cadc5, SUC_lexicon_tags_14 },
    { 0xcbccadc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd2cadc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24ccadca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68cadd0, SUC_lexicon_tags_65 },
    { 0x44acadd1, SUC_lexicon_tags_1 },
    { 0x920cadd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d4cadd4, SUC_lexicon_tags_1 },
    { 0x6c2cadd4, SUC_lexicon_tags_25 },
    { 0x62ecadd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaccadd8, SUC_lexicon_tags_1 },
    { 0xe7ecadd9, SUC_lexicon_tags_30 },
    { 0x410cadd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934caddf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceccade1, SUC_lexicon_tags_25 },
    { 0xa06cade1, SUC_lexicon_tags_1 },
    { 0xaf0cade2, SUC_lexicon_tags_1 },
    { 0x7ccade4, SUC_lexicon_tags_39 },
    { 0xc02cade5, SUC_lexicon_tags_37 },
    { 0xc74cade3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2facadef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836cadf7, SUC_lexicon_tags_31 },
    { 0x216cadf7, SUC_lexicon_tags_60 },
    { 0x4d0cadf9, SUC_lexicon_tags_25 },
    { 0x648cadfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3ecadfc, SUC_lexicon_tags_2 },
    { 0xe08cadfd, SUC_lexicon_tags_11 },
    { 0x2c0cadfe, SUC_lexicon_tags_15 },
    { 0x30cadff, SUC_lexicon_tags_31 },
    { 0x8fccadfd, SUC_lexicon_tags_29 },
    { 0x9facae01, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0x74cae03, SUC_lexicon_tags_31 },
    { 0x63acae04, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706cae08, SUC_lexicon_tags_27 },
    { 0xf7ccae09, SUC_lexicon_tags_87 },
    { 0x1a0cae0a, SUC_lexicon_tags_10 },
    { 0x704cae0b, SUC_lexicon_tags_28 },
    { 0x8eccae0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x26acae0f, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4cae14, SUC_lexicon_tags_4 },
    { 0xb96cae15, SUC_lexicon_tags_23 },
    { 0x4acae16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8cae1a, SUC_lexicon_tags_2 },
    { 0x83acae1b, SUC_lexicon_tags_12 },
    { 0xf2ecae1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00cae1f, SUC_lexicon_tags_28 },
    { 0x6accae20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9accae25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ecae28, SUC_lexicon_tags_48 },
    { 0x94ecae28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x660cae2c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfd2cae2e, SUC_lexicon_tags_15 },
    { 0x778cae2e, SUC_lexicon_tags_25 },
    { 0xa2ccae30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x996cae32, SUC_lexicon_tags_51 },
    { 0x380cae33, SUC_lexicon_tags_229 },
    { 0xc6ecae34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8cae36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ecae3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80cae40, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x824cae43, SUC_lexicon_tags_17 },
    { 0x906cae43, SUC_lexicon_tags_26 },
    { 0x88acae43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fecae48, SUC_lexicon_tags_46 },
    { 0xacacae49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x866cae4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8cae4f, SUC_lexicon_tags_14 },
    { 0x99ccae4f, SUC_lexicon_tags_2 },
    { 0xdecae51, SUC_lexicon_tags_31 },
    { 0xd40cae52, SUC_lexicon_tags_63 },
    { 0xad8cae52, SUC_lexicon_tags_19 },
    { 0x184cae53, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888cae58, SUC_lexicon_tags_1 },
    { 0x350cae59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8cae5e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0cae6f, SUC_lexicon_tags_25 },
    { 0xe5ccae70, SUC_lexicon_tags_30 },
    { 0xaf8cae6f, SUC_lexicon_tags_11 },
    { 0xfeecae6f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x19ccae74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6cae81, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdecae85, SUC_lexicon_tags_15 },
    { 0xdf0cae86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x108cae88, SUC_lexicon_tags_17 },
    { 0xf54cae89, SUC_lexicon_tags_1 },
    { 0xdf6cae8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8cae90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffecae94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98cae97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x640cae9a, SUC_lexicon_tags_29 },
    { 0x5f8cae9b, SUC_lexicon_tags_37 },
    { 0x866cae9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122caea1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2caea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67ecaea8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4ecaeaa, SUC_lexicon_tags_25 },
    { 0xd30caeab, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xef6caead, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ecaeb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x592caeb6, SUC_lexicon_tags_37 },
    { 0xe14caeb7, SUC_lexicon_tags_1 },
    { 0xc18caeb8, SUC_lexicon_tags_63 },
    { 0xd22caeb8, SUC_lexicon_tags_15 },
    { 0x7acaeb9, SUC_lexicon_tags_17 },
    { 0x324caeba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x706caebe, SUC_lexicon_tags_37 },
    { 0xb6caebf, SUC_lexicon_tags_25 },
    { 0x3bccaebe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x670caec3, SUC_lexicon_tags_11 },
    { 0xc3ecaec4, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ccaecc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd34caece, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e4caed0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ccaed4, SUC_lexicon_tags_28 },
    { 0x3eecaed5, SUC_lexicon_tags_14 },
    { 0x9d4caed4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x32ccaed8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccaedd, SUC_lexicon_tags_14 },
    { 0xf2caedd, SUC_lexicon_tags_63 },
    { 0x2d2caedf, SUC_lexicon_tags_0 },
    { 0x844caedf, SUC_lexicon_tags_7 },
    { 0x390caee1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8acaee3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefacaeea, SUC_lexicon_tags_38 },
    { 0xdd4caeeb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274caef5, SUC_lexicon_tags_11 },
    { 0xd7ccaef5, SUC_lexicon_tags_31 },
    { 0xe00caef6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a2caef9, SUC_lexicon_tags_55 },
    { 0xb88caef9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x378caefd, SUC_lexicon_tags_6 },
    { 0x3b2caefe, SUC_lexicon_tags_37 },
    { 0xc44caeff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448caf03, SUC_lexicon_tags_30 },
    { 0xd78caf04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeeccaf06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ecaf08, SUC_lexicon_tags_11 },
    { 0x76ccaf09, SUC_lexicon_tags_17 },
    { 0xe9ecaf0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x224caf0c, SUC_lexicon_tags_40 },
    { 0x31acaf0d, SUC_lexicon_tags_23 },
    { 0x4cacaf0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc22caf10, SUC_lexicon_tags_17 },
    { 0xc2ccaf11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36caf19, SUC_lexicon_tags_12 },
    { 0xc1acaf1a, SUC_lexicon_tags_6 },
    { 0xa22caf1b, SUC_lexicon_tags_27 },
    { 0x438caf1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x906caf1e, SUC_lexicon_tags_1 },
    { 0x234caf1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0ccaf21, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440caf25, SUC_lexicon_tags_920 },
    { 0x928caf26, SUC_lexicon_tags_87 },
    { 0x18caf27, SUC_lexicon_tags_11 },
    { 0x474caf26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354caf2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89ecaf30, SUC_lexicon_tags_26 },
    { 0x83ecaf31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa08caf33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95ecaf35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44caf38, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6caf3b, SUC_lexicon_tags_48 },
    { 0x3eecaf3c, SUC_lexicon_tags_1 },
    { 0x7dccaf3d, SUC_lexicon_tags_37 },
    { 0x758caf3e, SUC_lexicon_tags_31 },
    { 0xae0caf3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d8caf41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4acaf47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232caf4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12caf4f, SUC_lexicon_tags_71 },
    { 0x98ecaf4f, SUC_lexicon_tags_23 },
    { 0xc4caf51, SUC_lexicon_tags_31 },
    { 0x36caf52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118caf57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2caf5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4a2caf61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470caf67, SUC_lexicon_tags_65 },
    { 0x6c2caf67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x368caf6b, SUC_lexicon_tags_181 },
    { 0xdacaf6b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xafacaf6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b0caf71, SUC_lexicon_tags_7 },
    { 0xa52caf71, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0caf75, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dacaf80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fecaf85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28caf89, SUC_lexicon_tags_45 },
    { 0x3becaf8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2caf8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabacaf91, SUC_lexicon_tags_34 },
    { 0x13acaf92, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x698caf94, SUC_lexicon_tags_1 },
    { 0xdeccaf95, SUC_lexicon_tags_19 },
    { 0xa2ecaf96, SUC_lexicon_tags_65 },
    { 0x8d0caf95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x368caf99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ecafa8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ccafab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36cafae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd74cafb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bacafb2, SUC_lexicon_tags_14 },
    { 0x446cafb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ccafb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeacafbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x824cafc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaccafc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fccafc8, SUC_lexicon_tags_63 },
    { 0x386cafc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ecafcc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2aecafce, SUC_lexicon_tags_26 },
    { 0x5decafcf, SUC_lexicon_tags_10 },
    { 0x94ccafd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434cafd6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8cafda, SUC_lexicon_tags_37 },
    { 0x268cafdb, SUC_lexicon_tags_25 },
    { 0xe7ccafdb, SUC_lexicon_tags_32 },
    { 0x9b6cafdb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf76cafdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44ccafe1, SUC_lexicon_tags_12 },
    { 0x182cafe2, SUC_lexicon_tags_15 },
    { 0x284cafe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6acafe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x138cafe9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6cafec, SUC_lexicon_tags_30 },
    { 0x412cafed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea0cafef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ccaff2, SUC_lexicon_tags_27 },
    { 0x4cccaff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08caff6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334caffd, SUC_lexicon_tags_0 },
    { 0xcf2caffe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02cb001, SUC_lexicon_tags_60 },
    { 0x4accb002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed6cb004, SUC_lexicon_tags_79 },
    { 0x48cb004, SUC_lexicon_tags_1 },
    { 0xb54cb006, SUC_lexicon_tags_60 },
    { 0xa04cb007, SUC_lexicon_tags_29 },
    { 0xd8cb008, SUC_lexicon_tags_17 },
    { 0x588cb009, SUC_lexicon_tags_23 },
    { 0x8b2cb009, SUC_lexicon_tags_17 },
    { 0x888cb00a, SUC_lexicon_tags_85 },
    { 0x1bccb00b, SUC_lexicon_tags_4 },
    { 0x85acb00b, SUC_lexicon_tags_83 },
    { 0x4e0cb00e, SUC_lexicon_tags_37 },
    { 0xf5ecb00b, SUC_lexicon_tags_46 },
    { 0x72ccb008, SUC_lexicon_tags_43 },
    { 0x178cb011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39acb013, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54cb01b, SUC_lexicon_tags_15 },
    { 0xa40cb01c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ecb021, SUC_lexicon_tags_2 },
    { 0x126cb022, SUC_lexicon_tags_34 },
    { 0xde6cb023, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xabacb026, SUC_lexicon_tags_37 },
    { 0xdbacb026, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaacb02f, SUC_lexicon_tags_62 },
    { 0x1cecb02f, SUC_lexicon_tags_37 },
    { 0x678cb031, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x33acb033, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5ecb035, SUC_lexicon_tags_45 },
    { 0xde0cb036, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeccb039, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x810cb03b, SUC_lexicon_tags_29 },
    { 0xdb0cb03b, SUC_lexicon_tags_33 },
    { 0x5eccb03d, SUC_lexicon_tags_7 },
    { 0x324cb03e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaacb041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf80cb043, SUC_lexicon_tags_63 },
    { 0x21ecb044, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xefccb046, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20cb04a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830cb04e, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ecb051, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xb3acb053, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4cb057, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ccb05e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92acb063, SUC_lexicon_tags_7 },
    { 0xdccb064, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fecb068, SUC_lexicon_tags_15 },
    { 0x9e0cb068, SUC_lexicon_tags_17 },
    { 0x900cb06a, SUC_lexicon_tags_26 },
    { 0x24ccb069, SUC_lexicon_tags_15 },
    { 0xe6ecb069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60cb06f, SUC_lexicon_tags_151 },
    { 0x2f4cb070, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4cb072, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5accb075, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02cb079, SUC_lexicon_tags_17 },
    { 0xbfccb079, SUC_lexicon_tags_37 },
    { 0xacccb079, SUC_lexicon_tags_55 },
    { 0xb0acb07a, SUC_lexicon_tags_65 },
    { 0xdcacb07b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33acb081, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ecb086, SUC_lexicon_tags_17 },
    { 0xfc2cb087, SUC_lexicon_tags_17 },
    { 0xa68cb088, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1f4cb08a, SUC_lexicon_tags_25 },
    { 0x480cb08a, SUC_lexicon_tags_14 },
    { 0xe88cb08c, SUC_lexicon_tags_87 },
    { 0x970cb08c, SUC_lexicon_tags_37 },
    { 0xb1acb08e, SUC_lexicon_tags_37 },
    { 0x50cb08f, SUC_lexicon_tags_9 },
    { 0xdbccb08f, SUC_lexicon_tags_37 },
    { 0x992cb090, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9acb094, SUC_lexicon_tags_63 },
    { 0xe6acb094, SUC_lexicon_tags_2 },
    { 0xe80cb096, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x214cb099, SUC_lexicon_tags_26 },
    { 0x12cb09a, SUC_lexicon_tags_30 },
    { 0x9b4cb09b, SUC_lexicon_tags_25 },
    { 0x98acb09c, SUC_lexicon_tags_10 },
    { 0x618cb09b, SUC_lexicon_tags_26 },
    { 0xf8acb09d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8cb0a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x422cb0a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ecb0ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x840cb0ae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x514cb0b0, SUC_lexicon_tags_82 },
    { 0xad0cb0b0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cb0b5, SUC_lexicon_tags_7 },
    { 0xfcccb0b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd96cb0b8, SUC_lexicon_tags_37 },
    { 0x1f2cb0b9, SUC_lexicon_tags_14 },
    { 0x32acb0b9, SUC_lexicon_tags_1 },
    { 0x750cb0ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ecb0c1, SUC_lexicon_tags_2 },
    { 0x98cb0c2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53acb0c8, SUC_lexicon_tags_27 },
    { 0xf8ccb0c9, SUC_lexicon_tags_60 },
    { 0xba8cb0ca, SUC_lexicon_tags_17 },
    { 0x2eecb0cb, SUC_lexicon_tags_21 },
    { 0x49acb0c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706cb0d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0cb0da, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ecb0de, SUC_lexicon_tags_59 },
    { 0x926cb0df, SUC_lexicon_tags_6 },
    { 0x900cb0de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4cb0e3, SUC_lexicon_tags_23 },
    { 0xf28cb0e4, SUC_lexicon_tags_31 },
    { 0x5d6cb0e5, SUC_lexicon_tags_17 },
    { 0xdd8cb0e6, SUC_lexicon_tags_15 },
    { 0x7bacb0e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb2cb0e9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x58acb0ec, SUC_lexicon_tags_68 },
    { 0x3d4cb0ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x272cb0f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xef8cb0f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x760cb0f4, SUC_lexicon_tags_43 },
    { 0xe24cb0f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb80cb0f7, SUC_lexicon_tags_48 },
    { 0x984cb0f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8cb0fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ccb100, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360cb104, SUC_lexicon_tags_14 },
    { 0xd0ecb105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2cb107, SUC_lexicon_tags_17 },
    { 0xee4cb108, SUC_lexicon_tags_55 },
    { 0x344cb109, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980cb110, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6cb113, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x774cb116, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800cb11b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2cb11f, SUC_lexicon_tags_17 },
    { 0x272cb11f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6cb126, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x858cb129, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ecb12f, SUC_lexicon_tags_14 },
    { 0x2a2cb130, SUC_lexicon_tags_49 },
    { 0xed2cb131, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4cb138, SUC_lexicon_tags_12 },
    { 0x5ccb139, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x688cb13b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4cb13e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd02cb140, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ccb144, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80ecb146, SUC_lexicon_tags_19 },
    { 0xef8cb146, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe7ccb149, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2facb14c, SUC_lexicon_tags_17 },
    { 0x1aecb14d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x64acb14f, SUC_lexicon_tags_29 },
    { 0x8c8cb150, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1acb154, SUC_lexicon_tags_17 },
    { 0x8cccb155, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774cb15a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ecb15d, SUC_lexicon_tags_65 },
    { 0x9aacb15d, SUC_lexicon_tags_11 },
    { 0x678cb15f, SUC_lexicon_tags_45 },
    { 0x44ccb15e, SUC_lexicon_tags_60 },
    { 0x392cb161, SUC_lexicon_tags_79 },
    { 0x8a4cb161, SUC_lexicon_tags_17 },
    { 0x8becb163, SUC_lexicon_tags_2 },
    { 0x61ecb163, SUC_lexicon_tags_43 },
    { 0xa32cb15d, SUC_lexicon_tags_43 },
    { 0x9bacb15f, SUC_lexicon_tags_37 },
    { 0x3ccb167, SUC_lexicon_tags_17 },
    { 0x59ccb167, SUC_lexicon_tags_45 },
    { 0x324cb168, SUC_lexicon_tags_1 },
    { 0x350cb16a, SUC_lexicon_tags_49 },
    { 0x5d0cb16a, SUC_lexicon_tags_15 },
    { 0x93ccb16c, SUC_lexicon_tags_37 },
    { 0x992cb16d, SUC_lexicon_tags_10 },
    { 0xdccb16e, SUC_lexicon_tags_15 },
    { 0x568cb16b, SUC_lexicon_tags_34 },
    { 0xcccb168, SUC_lexicon_tags_7 },
    { 0x6bacb171, SUC_lexicon_tags_14 },
    { 0xd26cb172, SUC_lexicon_tags_32 },
    { 0xd78cb15f, SUC_lexicon_tags_31 },
    { 0x2c2cb174, SUC_lexicon_tags_0 },
    { 0xda2cb175, SUC_lexicon_tags_25 },
    { 0xfb0cb165, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c8cb178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee0cb17a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92ccb17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe02cb17e, SUC_lexicon_tags_25 },
    { 0x274cb17e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472cb184, SUC_lexicon_tags_43 },
    { 0xf0ccb185, SUC_lexicon_tags_12 },
    { 0x8ecb185, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa7ccb188, SUC_lexicon_tags_2 },
    { 0xa40cb189, SUC_lexicon_tags_72 },
    { 0x36ecb18a, SUC_lexicon_tags_47 },
    { 0x65ccb18a, SUC_lexicon_tags_7 },
    { 0xcd8cb18b, SUC_lexicon_tags_26 },
    { 0x8d2cb18d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2cb193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82cb196, SUC_lexicon_tags_11 },
    { 0x884cb196, SUC_lexicon_tags_33 },
    { 0x10ecb198, SUC_lexicon_tags_37 },
    { 0xf14cb199, SUC_lexicon_tags_31 },
    { 0xf20cb19a, SUC_lexicon_tags_12 },
    { 0x544cb199, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x398cb19e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x564cb1a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33acb1a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbecb1a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2ecb1a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x606cb1ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x224cb1ae, SUC_lexicon_tags_201 },
    { 0xe6acb1af, SUC_lexicon_tags_45 },
    { 0xeaccb1b0, SUC_lexicon_tags_26 },
    { 0x6c8cb1b1, SUC_lexicon_tags_10 },
    { 0xb14cb1af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c8cb1b4, SUC_lexicon_tags_45 },
    { 0xfe6cb1b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ecb1bb, SUC_lexicon_tags_60 },
    { 0x9d4cb1bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcf2cb1be, SUC_lexicon_tags_37 },
    { 0x664cb1bf, SUC_lexicon_tags_33 },
    { 0xc50cb1be, SUC_lexicon_tags_37 },
    { 0x102cb1c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21ecb1c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x614cb1c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec4cb1c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58ccb1ca, SUC_lexicon_tags_31 },
    { 0x54ccb1ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42ecb1cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12acb1cf, SUC_lexicon_tags_50 },
    { 0x1c6cb1cf, SUC_lexicon_tags_17 },
    { 0x5ccb1d1, SUC_lexicon_tags_12 },
    { 0xb26cb1d2, SUC_lexicon_tags_48 },
    { 0xc46cb1d3, SUC_lexicon_tags_6 },
    { 0x840cb1d3, SUC_lexicon_tags_68 },
    { 0x8b2cb1d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ecb1da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724cb1e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36cb1e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cb1eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7acb1ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88cb1f1, SUC_lexicon_tags_0 },
    { 0x7dacb1f1, SUC_lexicon_tags_25 },
    { 0x952cb1f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60acb1fd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32cb202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326cb206, SUC_lexicon_tags_28 },
    { 0xa58cb207, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ccb20d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84cb210, SUC_lexicon_tags_2 },
    { 0x77ecb211, SUC_lexicon_tags_15 },
    { 0x64cb212, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ccb215, SUC_lexicon_tags_7 },
    { 0xdc6cb216, SUC_lexicon_tags_82 },
    { 0x2dacb217, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ecb220, SUC_lexicon_tags_7 },
    { 0xf32cb220, SUC_lexicon_tags_17 },
    { 0xc16cb220, SUC_lexicon_tags_25 },
    { 0x900cb221, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3eccb225, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428cb22d, SUC_lexicon_tags_27 },
    { 0xc0ecb22e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeecb230, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748cb235, SUC_lexicon_tags_85 },
    { 0x422cb235, SUC_lexicon_tags_27 },
    { 0x76ccb236, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86cb23e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14cb252, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924cb259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x330cb25c, SUC_lexicon_tags_8 },
    { 0x518cb25d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x936cb25f, SUC_lexicon_tags_11 },
    { 0xebacb260, SUC_lexicon_tags_12 },
    { 0xdc6cb260, SUC_lexicon_tags_15 },
    { 0x724cb25f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6cb265, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4cb26c, SUC_lexicon_tags_58 },
    { 0x190cb26d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa48cb26f, SUC_lexicon_tags_47 },
    { 0xadecb270, SUC_lexicon_tags_37 },
    { 0x782cb26f, SUC_lexicon_tags_2 },
    { 0x9acb270, SUC_lexicon_tags_77 },
    { 0x762cb273, SUC_lexicon_tags_47 },
    { 0xaccb273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x680cb276, SUC_lexicon_tags_38 },
    { 0xfc4cb277, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xffecb27a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4e0cb27c, SUC_lexicon_tags_28 },
    { 0xf28cb27c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752cb281, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x770cb283, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x18ccb285, SUC_lexicon_tags_32 },
    { 0xc2cb286, SUC_lexicon_tags_66 },
    { 0xb50cb285, SUC_lexicon_tags_16 },
    { 0xb00cb287, SUC_lexicon_tags_21 },
    { 0xf1acb289, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644cb291, SUC_lexicon_tags_2 },
    { 0xc8acb291, SUC_lexicon_tags_45 },
    { 0xf8cb293, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xabacb296, SUC_lexicon_tags_38 },
    { 0x316cb296, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b8cb299, SUC_lexicon_tags_47 },
    { 0xe0cb299, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ecb29d, SUC_lexicon_tags_162 },
    { 0x86acb29e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc72cb2a0, SUC_lexicon_tags_15 },
    { 0xd30cb2a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x642cb2a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8acb2a7, SUC_lexicon_tags_32 },
    { 0x82acb2a8, SUC_lexicon_tags_46 },
    { 0x8eecb2a8, SUC_lexicon_tags_79 },
    { 0x81ccb2aa, SUC_lexicon_tags_65 },
    { 0x4cecb2a9, SUC_lexicon_tags_19 },
    { 0x57ecb2ac, SUC_lexicon_tags_14 },
    { 0xc7ccb2ad, SUC_lexicon_tags_0 },
    { 0x624cb2a7, SUC_lexicon_tags_25 },
    { 0x30cb2ae, SUC_lexicon_tags_15 },
    { 0xbc2cb2b0, SUC_lexicon_tags_12 },
    { 0xfc0cb2ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ecb2b7, SUC_lexicon_tags_23 },
    { 0x67ecb2b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f0cb2ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426cb2c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2cb2c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38cb2cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf86cb2cf, SUC_lexicon_tags_12 },
    { 0xa38cb2d0, SUC_lexicon_tags_23 },
    { 0x96cb2d1, SUC_lexicon_tags_3 },
    { 0x46ecb2d1, SUC_lexicon_tags_21 },
    { 0xe42cb2d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cecb2d7, SUC_lexicon_tags_23 },
    { 0xd60cb2d8, SUC_lexicon_tags_18 },
    { 0xbf2cb2d9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462cb2df, SUC_lexicon_tags_17 },
    { 0x298cb2e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0cb2e3, SUC_lexicon_tags_396 },
    { 0xcc4cb2e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7acb2e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4cb2ec, SUC_lexicon_tags_47 },
    { 0xbb0cb2ed, SUC_lexicon_tags_37 },
    { 0x14cb2ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ccb2f4, SUC_lexicon_tags_66 },
    { 0xd4ecb2f4, SUC_lexicon_tags_12 },
    { 0xcd4cb2f6, SUC_lexicon_tags_14 },
    { 0xc98cb2f7, SUC_lexicon_tags_17 },
    { 0xe82cb2f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4cb2fb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7a4cb2fd, SUC_lexicon_tags_11 },
    { 0x6acb2fe, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb60cb300, SUC_lexicon_tags_2 },
    { 0xf80cb300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c0cb303, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x574cb305, SUC_lexicon_tags_0 },
    { 0x91acb306, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4cb309, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8cb30c, SUC_lexicon_tags_29 },
    { 0x9f6cb30c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e4cb30f, SUC_lexicon_tags_30 },
    { 0xb52cb310, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x290cb312, SUC_lexicon_tags_37 },
    { 0xe06cb313, SUC_lexicon_tags_11 },
    { 0x10acb312, SUC_lexicon_tags_2 },
    { 0x5bccb314, SUC_lexicon_tags_37 },
    { 0xbeacb315, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2cb319, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ccb31b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654cb323, SUC_lexicon_tags_63 },
    { 0x630cb324, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x326cb326, SUC_lexicon_tags_12 },
    { 0xefecb327, SUC_lexicon_tags_21 },
    { 0x400cb327, SUC_lexicon_tags_23 },
    { 0x418cb329, SUC_lexicon_tags_48 },
    { 0xc4acb32a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x122cb32c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x518cb32f, SUC_lexicon_tags_37 },
    { 0xc10cb32f, SUC_lexicon_tags_12 },
    { 0x890cb32f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d2cb333, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6cb339, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254cb340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bacb342, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54cb346, SUC_lexicon_tags_7 },
    { 0x96cb347, SUC_lexicon_tags_17 },
    { 0xdacb347, SUC_lexicon_tags_11 },
    { 0xa8cb348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6cb34c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82cb352, SUC_lexicon_tags_11 },
    { 0x508cb352, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2cb359, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82cb360, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fecb364, SUC_lexicon_tags_25 },
    { 0xd0cb365, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5c0cb367, SUC_lexicon_tags_68 },
    { 0x74ccb368, SUC_lexicon_tags_55 },
    { 0x794cb368, SUC_lexicon_tags_47 },
    { 0x252cb36a, SUC_lexicon_tags_47 },
    { 0x45acb369, SUC_lexicon_tags_7 },
    { 0xf40cb36c, SUC_lexicon_tags_29 },
    { 0xe76cb36d, SUC_lexicon_tags_63 },
    { 0xc92cb36c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1ccb370, SUC_lexicon_tags_10 },
    { 0xf38cb370, SUC_lexicon_tags_8 },
    { 0x1eccb372, SUC_lexicon_tags_7 },
    { 0x8acb373, SUC_lexicon_tags_30 },
    { 0x996cb370, SUC_lexicon_tags_91 },
    { 0x5facb375, SUC_lexicon_tags_14 },
    { 0xce0cb374, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x474cb379, SUC_lexicon_tags_2 },
    { 0xe74cb37a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ecb37f, SUC_lexicon_tags_30 },
    { 0xb00cb380, SUC_lexicon_tags_0 },
    { 0x3bccb381, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc46cb383, SUC_lexicon_tags_17 },
    { 0xeb0cb384, SUC_lexicon_tags_6 },
    { 0x1e0cb384, SUC_lexicon_tags_6 },
    { 0x8b2cb386, SUC_lexicon_tags_15 },
    { 0x1c6cb387, SUC_lexicon_tags_38 },
    { 0x580cb387, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26cb38b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43acb390, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2cb393, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428cb397, SUC_lexicon_tags_7 },
    { 0x788cb398, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaecb3a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbc8cb3a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaacb3aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42cb3b2, SUC_lexicon_tags_17 },
    { 0x6becb3b3, SUC_lexicon_tags_65 },
    { 0x4c0cb3b4, SUC_lexicon_tags_49 },
    { 0x162cb3b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aacb3b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3acb3bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdecb3c0, SUC_lexicon_tags_7 },
    { 0xacb3c1, SUC_lexicon_tags_17 },
    { 0x1cacb3c2, SUC_lexicon_tags_37 },
    { 0x658cb3c2, SUC_lexicon_tags_45 },
    { 0xec4cb3c1, SUC_lexicon_tags_15 },
    { 0x6cb3c2, SUC_lexicon_tags_21 },
    { 0xaa0cb3c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2acb3c9, SUC_lexicon_tags_12 },
    { 0xce8cb3ca, SUC_lexicon_tags_26 },
    { 0xc72cb3ca, SUC_lexicon_tags_7 },
    { 0x644cb3cc, SUC_lexicon_tags_15 },
    { 0x10acb3cd, SUC_lexicon_tags_66 },
    { 0x170cb3ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58cb3d0, SUC_lexicon_tags_77 },
    { 0x42ccb3d0, SUC_lexicon_tags_15 },
    { 0x7b8cb3d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd34cb3d4, SUC_lexicon_tags_1 },
    { 0x80acb3d5, SUC_lexicon_tags_12 },
    { 0x4b8cb3d6, SUC_lexicon_tags_2 },
    { 0xed4cb3d7, SUC_lexicon_tags_19 },
    { 0xa90cb3d8, SUC_lexicon_tags_140 },
    { 0xd6ecb3d7, SUC_lexicon_tags_17 },
    { 0xcecb3da, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe70cb3dc, SUC_lexicon_tags_14 },
    { 0xa2ecb3dd, SUC_lexicon_tags_17 },
    { 0xbaecb3dd, SUC_lexicon_tags_37 },
    { 0x998cb3dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8cb3e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ecb3e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0cb3eb, SUC_lexicon_tags_37 },
    { 0x506cb3ec, SUC_lexicon_tags_2 },
    { 0x338cb3ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccacb3f1, SUC_lexicon_tags_17 },
    { 0x9c0cb3f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x514cb3f5, SUC_lexicon_tags_27 },
    { 0x524cb3f6, SUC_lexicon_tags_0 },
    { 0x924cb3f5, SUC_lexicon_tags_17 },
    { 0x436cb3f8, SUC_lexicon_tags_17 },
    { 0x36cb3f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fecb3fe, SUC_lexicon_tags_21 },
    { 0xcf4cb3ff, SUC_lexicon_tags_9 },
    { 0x716cb400, SUC_lexicon_tags_12 },
    { 0x286cb400, SUC_lexicon_tags_29 },
    { 0x7a0cb401, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x10ccb404, SUC_lexicon_tags_33 },
    { 0x2cecb405, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52cb408, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8cb40b, SUC_lexicon_tags_91 },
    { 0x6dacb40c, SUC_lexicon_tags_12 },
    { 0xc04cb40d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5becb40f, SUC_lexicon_tags_8 },
    { 0xf6ccb40f, SUC_lexicon_tags_38 },
    { 0x896cb40f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884cb41b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ecb420, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0cb424, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388cb428, SUC_lexicon_tags_7 },
    { 0x640cb429, SUC_lexicon_tags_60 },
    { 0x4f2cb428, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cb42e, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x46ccb430, SUC_lexicon_tags_301 },
    { 0x34acb431, SUC_lexicon_tags_25 },
    { 0xdc4cb432, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x126cb435, SUC_lexicon_tags_25 },
    { 0x8dacb436, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ecb43a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24cb443, SUC_lexicon_tags_46 },
    { 0x2a4cb444, SUC_lexicon_tags_23 },
    { 0xd80cb444, SUC_lexicon_tags_31 },
    { 0xcaccb446, SUC_lexicon_tags_8 },
    { 0x778cb446, SUC_lexicon_tags_31 },
    { 0x4c0cb446, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1aecb44a, SUC_lexicon_tags_9 },
    { 0x5a2cb44a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2cb451, SUC_lexicon_tags_12 },
    { 0xc3ecb452, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68cb455, SUC_lexicon_tags_11 },
    { 0x4aecb455, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x388cb458, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb45b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x48acb45d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8cb461, SUC_lexicon_tags_17 },
    { 0x9d0cb462, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x282cb464, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ecb46e, SUC_lexicon_tags_12 },
    { 0x346cb46e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aecb473, SUC_lexicon_tags_25 },
    { 0x7a0cb474, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70cb478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83acb47a, SUC_lexicon_tags_8 },
    { 0x818cb47b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5d8cb47d, SUC_lexicon_tags_1 },
    { 0x960cb47e, SUC_lexicon_tags_37 },
    { 0xb9acb47f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6cb482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ccb486, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ccb48a, SUC_lexicon_tags_42 },
    { 0x9f6cb48a, SUC_lexicon_tags_65 },
    { 0xd9acb48a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ccb491, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x930cb493, SUC_lexicon_tags_37 },
    { 0xa10cb494, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa8cb496, SUC_lexicon_tags_27 },
    { 0xd98cb496, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2cb49a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6cb49e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13acb4a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x198cb4a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2cb4a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cccb4ac, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4cb4af, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa6ecb4b1, SUC_lexicon_tags_60 },
    { 0x37acb4b2, SUC_lexicon_tags_2 },
    { 0x1c6cb4b3, SUC_lexicon_tags_11 },
    { 0x43acb4b4, SUC_lexicon_tags_9 },
    { 0x878cb4b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x882cb4b7, SUC_lexicon_tags_1 },
    { 0xe62cb4b7, SUC_lexicon_tags_80 },
    { 0x20cb4b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910cb4bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4cb4c1, SUC_lexicon_tags_65 },
    { 0xf7ccb4c1, SUC_lexicon_tags_7 },
    { 0x8aacb4c2, SUC_lexicon_tags_10 },
    { 0xc1ecb4c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5dccb4c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2cb4ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8cb4cf, SUC_lexicon_tags_30 },
    { 0x4f2cb4d0, SUC_lexicon_tags_48 },
    { 0xef2cb4cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x318cb4d4, SUC_lexicon_tags_63 },
    { 0x1b8cb4d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0cb4da, SUC_lexicon_tags_1 },
    { 0x930cb4db, SUC_lexicon_tags_63 },
    { 0x102cb4db, SUC_lexicon_tags_7 },
    { 0xc62cb4dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x352cb4e0, SUC_lexicon_tags_25 },
    { 0x176cb4e1, SUC_lexicon_tags_11 },
    { 0x658cb4e1, SUC_lexicon_tags_15 },
    { 0x3cccb4e1, SUC_lexicon_tags_2 },
    { 0x288cb4e4, SUC_lexicon_tags_7 },
    { 0xfbccb4e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecb4e9, SUC_lexicon_tags_61 },
    { 0x12acb4ea, SUC_lexicon_tags_66 },
    { 0xb3ccb4eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74acb4f1, SUC_lexicon_tags_17 },
    { 0xbb2cb4f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67acb4f5, SUC_lexicon_tags_29 },
    { 0x5a6cb4f6, SUC_lexicon_tags_10 },
    { 0xb6cb4f6, SUC_lexicon_tags_37 },
    { 0x224cb4f8, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92cb4fb, SUC_lexicon_tags_1 },
    { 0x62ccb4fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ccb4ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8cb503, SUC_lexicon_tags_15 },
    { 0x562cb503, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370cb509, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4cb50d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4cb510, SUC_lexicon_tags_31 },
    { 0x842cb510, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20ecb513, SUC_lexicon_tags_1381 },
    { 0, NULL },
    { 0x49acb515, SUC_lexicon_tags_103 },
    { 0x170cb515, SUC_lexicon_tags_47 },
    { 0x69ecb516, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42cb51a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ecb51d, SUC_lexicon_tags_65 },
    { 0xca2cb51e, SUC_lexicon_tags_11 },
    { 0xe56cb51d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518cb528, SUC_lexicon_tags_0 },
    { 0xcf4cb529, SUC_lexicon_tags_33 },
    { 0x724cb52a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x29acb52d, SUC_lexicon_tags_2 },
    { 0x45ccb52e, SUC_lexicon_tags_1 },
    { 0xc56cb52f, SUC_lexicon_tags_14 },
    { 0x19ccb530, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadecb536, SUC_lexicon_tags_15 },
    { 0x49acb537, SUC_lexicon_tags_37 },
    { 0xd72cb537, SUC_lexicon_tags_15 },
    { 0xaaacb539, SUC_lexicon_tags_47 },
    { 0xda8cb53a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68ecb53c, SUC_lexicon_tags_37 },
    { 0x476cb53d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x834cb53f, SUC_lexicon_tags_46 },
    { 0x68acb53f, SUC_lexicon_tags_12 },
    { 0x8a2cb540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcecb543, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4cb546, SUC_lexicon_tags_15 },
    { 0xf62cb547, SUC_lexicon_tags_14 },
    { 0xb6ccb546, SUC_lexicon_tags_14 },
    { 0x1ccb546, SUC_lexicon_tags_2 },
    { 0xe20cb54a, SUC_lexicon_tags_55 },
    { 0xfcccb546, SUC_lexicon_tags_30 },
    { 0x42ccb54c, SUC_lexicon_tags_12 },
    { 0x12acb54d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932cb552, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe30cb554, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2e0cb556, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecb55b, SUC_lexicon_tags_32 },
    { 0xfcecb55c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8acb561, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ccb564, SUC_lexicon_tags_11 },
    { 0x5a0cb565, SUC_lexicon_tags_29 },
    { 0x40ccb566, SUC_lexicon_tags_25 },
    { 0xfb4cb567, SUC_lexicon_tags_1677 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110cb56b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0cb571, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x294cb573, SUC_lexicon_tags_37 },
    { 0xd00cb574, SUC_lexicon_tags_16 },
    { 0xb88cb573, SUC_lexicon_tags_7 },
    { 0x57acb576, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140cb57b, SUC_lexicon_tags_23 },
    { 0x9e8cb57c, SUC_lexicon_tags_1 },
    { 0x68ccb57d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cecb580, SUC_lexicon_tags_46 },
    { 0x26acb581, SUC_lexicon_tags_25 },
    { 0x16ccb582, SUC_lexicon_tags_61 },
    { 0xf84cb580, SUC_lexicon_tags_39 },
    { 0xb8acb584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3acb587, SUC_lexicon_tags_69 },
    { 0x94acb588, SUC_lexicon_tags_0 },
    { 0x728cb588, SUC_lexicon_tags_12 },
    { 0xe86cb58a, SUC_lexicon_tags_11 },
    { 0xf4ccb588, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf64cb58d, SUC_lexicon_tags_43 },
    { 0x4decb58e, SUC_lexicon_tags_57 },
    { 0xf14cb58f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0cb596, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ccb59a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968cb5a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x232cb5a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xce0cb5a8, SUC_lexicon_tags_26 },
    { 0x402cb5a9, SUC_lexicon_tags_55 },
    { 0x6facb5a8, SUC_lexicon_tags_6 },
    { 0x13ccb5a9, SUC_lexicon_tags_7 },
    { 0xad6cb5ab, SUC_lexicon_tags_2 },
    { 0xc6ecb5ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x696cb5af, SUC_lexicon_tags_16 },
    { 0x92ecb5af, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcecb5b4, SUC_lexicon_tags_8 },
    { 0x10ccb5b5, SUC_lexicon_tags_1 },
    { 0xf4ecb5b4, SUC_lexicon_tags_37 },
    { 0xdd2cb5b4, SUC_lexicon_tags_12 },
    { 0x8a6cb5b8, SUC_lexicon_tags_0 },
    { 0xe0cb5b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2cb5bf, SUC_lexicon_tags_138 },
    { 0xf7ccb5c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ccb5c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xabacb5c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4cb5c9, SUC_lexicon_tags_8 },
    { 0xc36cb5c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba2cb5cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4cb5cf, SUC_lexicon_tags_60 },
    { 0x526cb5d0, SUC_lexicon_tags_13 },
    { 0x81ecb5cf, SUC_lexicon_tags_70 },
    { 0xb94cb5d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ccb5d5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x152cb5d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd80cb5d9, SUC_lexicon_tags_15 },
    { 0x2e6cb5da, SUC_lexicon_tags_15 },
    { 0xce4cb5db, SUC_lexicon_tags_7 },
    { 0xce8cb5db, SUC_lexicon_tags_12 },
    { 0xcfacb5dc, SUC_lexicon_tags_63 },
    { 0x468cb5de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe46cb5e0, SUC_lexicon_tags_8 },
    { 0x882cb5e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xecccb5e3, SUC_lexicon_tags_12 },
    { 0x520cb5e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aacb5e9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x92cb5eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20cb5ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbecb5f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x644cb5f3, SUC_lexicon_tags_25 },
    { 0x876cb5f4, SUC_lexicon_tags_37 },
    { 0x546cb5f3, SUC_lexicon_tags_17 },
    { 0xa02cb5f6, SUC_lexicon_tags_17 },
    { 0xb10cb5f6, SUC_lexicon_tags_1 },
    { 0x1eccb5f8, SUC_lexicon_tags_47 },
    { 0x452cb5f8, SUC_lexicon_tags_57 },
    { 0x2b8cb5f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2cb5fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f8cb5ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66acb604, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x632cb607, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x136cb609, SUC_lexicon_tags_45 },
    { 0x5aecb609, SUC_lexicon_tags_37 },
    { 0x4e4cb60b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22ccb60d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4b8cb60f, SUC_lexicon_tags_1 },
    { 0x9d6cb60f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x380cb612, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19ccb614, SUC_lexicon_tags_2 },
    { 0xf3ccb614, SUC_lexicon_tags_17 },
    { 0x95acb615, SUC_lexicon_tags_27 },
    { 0xcb6cb617, SUC_lexicon_tags_49 },
    { 0x9fecb618, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28cb61c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2cb61f, SUC_lexicon_tags_60 },
    { 0x95ccb620, SUC_lexicon_tags_7 },
    { 0xf48cb61f, SUC_lexicon_tags_82 },
    { 0x884cb622, SUC_lexicon_tags_61 },
    { 0x49ccb622, SUC_lexicon_tags_6 },
    { 0x254cb624, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624cb628, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ecb62c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x982cb62f, SUC_lexicon_tags_35 },
    { 0x5e8cb630, SUC_lexicon_tags_170 },
    { 0x68acb630, SUC_lexicon_tags_23 },
    { 0x41ecb632, SUC_lexicon_tags_43 },
    { 0x8fccb631, SUC_lexicon_tags_2 },
    { 0xfb6cb634, SUC_lexicon_tags_38 },
    { 0x2e6cb635, SUC_lexicon_tags_27 },
    { 0x192cb636, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64cb63e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa84cb640, SUC_lexicon_tags_11 },
    { 0xb38cb641, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212cb645, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4cb649, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eccb64d, SUC_lexicon_tags_29 },
    { 0xcc0cb64d, SUC_lexicon_tags_84 },
    { 0x294cb64e, SUC_lexicon_tags_17 },
    { 0x332cb650, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230cb655, SUC_lexicon_tags_2 },
    { 0x86ccb656, SUC_lexicon_tags_63 },
    { 0xaaccb656, SUC_lexicon_tags_17 },
    { 0x970cb657, SUC_lexicon_tags_32 },
    { 0x402cb659, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0cb65d, SUC_lexicon_tags_46 },
    { 0xaf0cb65e, SUC_lexicon_tags_6 },
    { 0x4f6cb65f, SUC_lexicon_tags_26 },
    { 0x108cb65d, SUC_lexicon_tags_30 },
    { 0x854cb661, SUC_lexicon_tags_15 },
    { 0x30acb662, SUC_lexicon_tags_33 },
    { 0x3dacb661, SUC_lexicon_tags_116 },
    { 0xefacb661, SUC_lexicon_tags_34 },
    { 0x616cb65f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf46cb667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ecb66c, SUC_lexicon_tags_37 },
    { 0xa4ecb66d, SUC_lexicon_tags_25 },
    { 0x3bacb66e, SUC_lexicon_tags_14 },
    { 0x9f8cb66f, SUC_lexicon_tags_0 },
    { 0x2aecb66f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xda6cb672, SUC_lexicon_tags_82 },
    { 0x432cb672, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4cb676, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x59ecb678, SUC_lexicon_tags_25 },
    { 0x858cb679, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60cb67d, SUC_lexicon_tags_25 },
    { 0xc48cb67d, SUC_lexicon_tags_25 },
    { 0x628cb67f, SUC_lexicon_tags_19 },
    { 0x222cb67f, SUC_lexicon_tags_1 },
    { 0x9d0cb681, SUC_lexicon_tags_23 },
    { 0xce8cb680, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x814cb684, SUC_lexicon_tags_47 },
    { 0xf3ccb685, SUC_lexicon_tags_15 },
    { 0x8a0cb685, SUC_lexicon_tags_37 },
    { 0x2c4cb687, SUC_lexicon_tags_7 },
    { 0x88acb688, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384cb68c, SUC_lexicon_tags_7 },
    { 0x57acb68d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828cb692, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cb696, SUC_lexicon_tags_40 },
    { 0x862cb697, SUC_lexicon_tags_15 },
    { 0x724cb698, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55acb69a, SUC_lexicon_tags_38 },
    { 0xf88cb69a, SUC_lexicon_tags_68 },
    { 0x388cb69c, SUC_lexicon_tags_43 },
    { 0xb2acb69d, SUC_lexicon_tags_0 },
    { 0x3b6cb69e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eecb6a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42cb6a5, SUC_lexicon_tags_19 },
    { 0x824cb6a6, SUC_lexicon_tags_45 },
    { 0x66acb6a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28cb6ab, SUC_lexicon_tags_0 },
    { 0x576cb6ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78cb6b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82cb6b5, SUC_lexicon_tags_11 },
    { 0x518cb6b6, SUC_lexicon_tags_14 },
    { 0xecacb6b5, SUC_lexicon_tags_37 },
    { 0x93acb6b6, SUC_lexicon_tags_47 },
    { 0xf1ccb6b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x184cb6bc, SUC_lexicon_tags_1 },
    { 0xf2ccb6bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf46cb6bf, SUC_lexicon_tags_11 },
    { 0x60cb6c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x696cb6c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778cb6cb, SUC_lexicon_tags_27 },
    { 0xf2cb6cb, SUC_lexicon_tags_33 },
    { 0x50ecb6cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6acb6d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96cb6d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1decb6d9, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x3becb6dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2cb6e2, SUC_lexicon_tags_12 },
    { 0xe9ccb6e3, SUC_lexicon_tags_48 },
    { 0xf30cb6e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2cb6e9, SUC_lexicon_tags_38 },
    { 0xa42cb6e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584cb6f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44cb6f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29acb6fb, SUC_lexicon_tags_1 },
    { 0xdc8cb6fc, SUC_lexicon_tags_11 },
    { 0x388cb6fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4cb700, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248cb705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150cb709, SUC_lexicon_tags_46 },
    { 0x72ecb70a, SUC_lexicon_tags_27 },
    { 0x314cb70a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36acb70d, SUC_lexicon_tags_17 },
    { 0x49ccb70e, SUC_lexicon_tags_19 },
    { 0x4eccb70e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x260cb711, SUC_lexicon_tags_15 },
    { 0x43acb712, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6cb716, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe74cb718, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6cb71e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x8f2cb720, SUC_lexicon_tags_2 },
    { 0x82ecb720, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe2ccb723, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c8cb725, SUC_lexicon_tags_285 },
    { 0x6e4cb726, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4cb729, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0cb72d, SUC_lexicon_tags_252 },
    { 0x92acb72e, SUC_lexicon_tags_40 },
    { 0x88cb72f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x15ccb731, SUC_lexicon_tags_33 },
    { 0xc2ecb732, SUC_lexicon_tags_17 },
    { 0x380cb733, SUC_lexicon_tags_37 },
    { 0xfaecb734, SUC_lexicon_tags_27 },
    { 0x58cb732, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaacb737, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2cb73b, SUC_lexicon_tags_29 },
    { 0xf82cb73b, SUC_lexicon_tags_25 },
    { 0xa9ccb73d, SUC_lexicon_tags_23 },
    { 0xdbacb73e, SUC_lexicon_tags_17 },
    { 0xdecb73b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcaccb741, SUC_lexicon_tags_25 },
    { 0xdb8cb741, SUC_lexicon_tags_60 },
    { 0x96ecb741, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c2cb745, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa68cb747, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ccb74f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e8cb751, SUC_lexicon_tags_13 },
    { 0xf6acb752, SUC_lexicon_tags_33 },
    { 0x522cb753, SUC_lexicon_tags_84 },
    { 0x9e2cb754, SUC_lexicon_tags_37 },
    { 0xd1acb753, SUC_lexicon_tags_15 },
    { 0xf14cb752, SUC_lexicon_tags_1 },
    { 0xf84cb757, SUC_lexicon_tags_12 },
    { 0x24ecb755, SUC_lexicon_tags_37 },
    { 0xa76cb755, SUC_lexicon_tags_7 },
    { 0xf46cb75a, SUC_lexicon_tags_55 },
    { 0x374cb755, SUC_lexicon_tags_27 },
    { 0x3b8cb759, SUC_lexicon_tags_25 },
    { 0x8e8cb75d, SUC_lexicon_tags_11 },
    { 0x78acb75d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x726cb760, SUC_lexicon_tags_60 },
    { 0xd8ccb761, SUC_lexicon_tags_2 },
    { 0xa7ecb761, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ccb766, SUC_lexicon_tags_37 },
    { 0xce8cb767, SUC_lexicon_tags_55 },
    { 0xd1ccb768, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694cb76d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x24acb76f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732cb774, SUC_lexicon_tags_37 },
    { 0x678cb774, SUC_lexicon_tags_2 },
    { 0xa48cb776, SUC_lexicon_tags_45 },
    { 0xde0cb774, SUC_lexicon_tags_45 },
    { 0xd4cb777, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb40cb77a, SUC_lexicon_tags_23 },
    { 0xc2cb77b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ecb77f, SUC_lexicon_tags_37 },
    { 0xa30cb780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cb784, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x542cb786, SUC_lexicon_tags_0 },
    { 0xfecb787, SUC_lexicon_tags_11 },
    { 0xdccb788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214cb78f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27ecb791, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8acb796, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x712cb798, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36acb79d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908cb7a5, SUC_lexicon_tags_87 },
    { 0xc8cb7a5, SUC_lexicon_tags_4 },
    { 0x7e0cb7a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e6cb7a9, SUC_lexicon_tags_7 },
    { 0x96acb7aa, SUC_lexicon_tags_14 },
    { 0xd44cb7aa, SUC_lexicon_tags_7 },
    { 0xba6cb7a9, SUC_lexicon_tags_33 },
    { 0x900cb7ad, SUC_lexicon_tags_37 },
    { 0xb7ecb7ae, SUC_lexicon_tags_55 },
    { 0x470cb7ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8acb7b7, SUC_lexicon_tags_60 },
    { 0x7bacb7b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x308cb7bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1f6cb7bd, SUC_lexicon_tags_27 },
    { 0xeaccb7be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd98cb7c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44cb7c4, SUC_lexicon_tags_39 },
    { 0xd92cb7c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf6cb7c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x310cb7ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92cb7cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94cb7d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152cb7d8, SUC_lexicon_tags_12 },
    { 0x62acb7d9, SUC_lexicon_tags_39 },
    { 0x986cb7da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe02cb7dd, SUC_lexicon_tags_6 },
    { 0xae4cb7dd, SUC_lexicon_tags_49 },
    { 0x794cb7df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x49ecb7e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8cb7e4, SUC_lexicon_tags_15 },
    { 0x126cb7e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862cb7e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee6cb7eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2cb7ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eccb7f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64cb7f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504cb7fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaecb806, SUC_lexicon_tags_37 },
    { 0x768cb806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40acb80b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2cb80d, SUC_lexicon_tags_7 },
    { 0xa8cb80e, SUC_lexicon_tags_66 },
    { 0x538cb80d, SUC_lexicon_tags_37 },
    { 0x4c8cb80f, SUC_lexicon_tags_1 },
    { 0x600cb811, SUC_lexicon_tags_38 },
    { 0xd6cb80f, SUC_lexicon_tags_12 },
    { 0x406cb811, SUC_lexicon_tags_17 },
    { 0x392cb814, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe8cb816, SUC_lexicon_tags_1 },
    { 0x3b6cb816, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ecb81f, SUC_lexicon_tags_129 },
    { 0x346cb820, SUC_lexicon_tags_43 },
    { 0xececb820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2cb824, SUC_lexicon_tags_4 },
    { 0x7cacb824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75acb829, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cb82c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x760cb82f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62acb835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x574cb838, SUC_lexicon_tags_0 },
    { 0xd50cb839, SUC_lexicon_tags_11 },
    { 0x16ecb839, SUC_lexicon_tags_6 },
    { 0xa1ecb838, SUC_lexicon_tags_7 },
    { 0x87ccb83c, SUC_lexicon_tags_23 },
    { 0x1c0cb83d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa48cb83f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x37ccb841, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x260cb844, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8cb84b, SUC_lexicon_tags_12 },
    { 0xaccb84c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbccb84e, SUC_lexicon_tags_11 },
    { 0xb48cb84f, SUC_lexicon_tags_12 },
    { 0x3c8cb84e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340cb854, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4cb858, SUC_lexicon_tags_15 },
    { 0x360cb859, SUC_lexicon_tags_32 },
    { 0x834cb859, SUC_lexicon_tags_112 },
    { 0x6f0cb85b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccccb85d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ccb862, SUC_lexicon_tags_15 },
    { 0x814cb863, SUC_lexicon_tags_37 },
    { 0x32acb864, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6cb867, SUC_lexicon_tags_52 },
    { 0xfaecb868, SUC_lexicon_tags_0 },
    { 0x64acb869, SUC_lexicon_tags_37 },
    { 0xda8cb867, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ccb86d, SUC_lexicon_tags_68 },
    { 0xbd4cb86e, SUC_lexicon_tags_30 },
    { 0x33acb86f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16acb878, SUC_lexicon_tags_25 },
    { 0x14ccb879, SUC_lexicon_tags_38 },
    { 0x618cb87a, SUC_lexicon_tags_14 },
    { 0x43ecb87b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28cb87e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c4cb880, SUC_lexicon_tags_61 },
    { 0x818cb881, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52cb887, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0cb891, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2becb896, SUC_lexicon_tags_25 },
    { 0x3fecb896, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bccb89a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4cb89e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb40cb8a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccccb8a5, SUC_lexicon_tags_1 },
    { 0x4a2cb8a5, SUC_lexicon_tags_65 },
    { 0x28ccb8a7, SUC_lexicon_tags_37 },
    { 0x2f6cb8a6, SUC_lexicon_tags_17 },
    { 0x2becb8a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48acb8ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecb8b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610cb8b9, SUC_lexicon_tags_11 },
    { 0xf84cb8ba, SUC_lexicon_tags_21 },
    { 0x3b8cb8ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc8cb8bd, SUC_lexicon_tags_19 },
    { 0x45ecb8be, SUC_lexicon_tags_17 },
    { 0xdd2cb8bd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x50ecb8c1, SUC_lexicon_tags_7 },
    { 0xfaacb8c2, SUC_lexicon_tags_11 },
    { 0x264cb8c2, SUC_lexicon_tags_25 },
    { 0xef4cb8c4, SUC_lexicon_tags_43 },
    { 0xb4ccb8c1, SUC_lexicon_tags_40 },
    { 0xfa4cb8c3, SUC_lexicon_tags_17 },
    { 0xc28cb8c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd04cb8c9, SUC_lexicon_tags_1 },
    { 0xe00cb8c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4cb8ce, SUC_lexicon_tags_11 },
    { 0xc4acb8ce, SUC_lexicon_tags_15 },
    { 0x9dccb8d0, SUC_lexicon_tags_15 },
    { 0x114cb8d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47ccb8d3, SUC_lexicon_tags_17 },
    { 0xf1ccb8d4, SUC_lexicon_tags_11 },
    { 0xf8ccb8d3, SUC_lexicon_tags_43 },
    { 0x472cb8d6, SUC_lexicon_tags_28 },
    { 0x186cb8d5, SUC_lexicon_tags_15 },
    { 0x2f8cb8d6, SUC_lexicon_tags_26 },
    { 0xe76cb8d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x770cb8dc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x584cb8df, SUC_lexicon_tags_55 },
    { 0x848cb8e0, SUC_lexicon_tags_31 },
    { 0x5cacb8e1, SUC_lexicon_tags_29 },
    { 0x544cb8e2, SUC_lexicon_tags_37 },
    { 0x10cb8e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ccb8e7, SUC_lexicon_tags_40 },
    { 0x582cb8e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ccb8ec, SUC_lexicon_tags_28 },
    { 0x4cecb8ed, SUC_lexicon_tags_11 },
    { 0x53ecb8ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0cb8f3, SUC_lexicon_tags_40 },
    { 0x1aecb8f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x430cb8f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794cb8fb, SUC_lexicon_tags_37 },
    { 0x5a6cb8fb, SUC_lexicon_tags_33 },
    { 0xa3ecb8fd, SUC_lexicon_tags_15 },
    { 0x6d4cb8fe, SUC_lexicon_tags_15 },
    { 0x6cecb8ff, SUC_lexicon_tags_14 },
    { 0x544cb8fe, SUC_lexicon_tags_14 },
    { 0x688cb901, SUC_lexicon_tags_32 },
    { 0x934cb902, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780cb908, SUC_lexicon_tags_28 },
    { 0x790cb909, SUC_lexicon_tags_13 },
    { 0x342cb909, SUC_lexicon_tags_16 },
    { 0x352cb90b, SUC_lexicon_tags_12 },
    { 0x7aacb90c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76cb912, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422cb916, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ccb91c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70cb920, SUC_lexicon_tags_12 },
    { 0x98cb921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cb924, SUC_lexicon_tags_23 },
    { 0xa88cb924, SUC_lexicon_tags_11 },
    { 0x59ecb925, SUC_lexicon_tags_84 },
    { 0xdc4cb925, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb16cb929, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9cacb92b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88cb930, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46cb937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8cb93c, SUC_lexicon_tags_311 },
    { 0xd16cb93c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x562cb940, SUC_lexicon_tags_31 },
    { 0x3c8cb941, SUC_lexicon_tags_84 },
    { 0x72acb941, SUC_lexicon_tags_15 },
    { 0xa8acb943, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb6cb945, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x13ecb947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0cb94b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0cb94e, SUC_lexicon_tags_7 },
    { 0xbbecb94f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46cb953, SUC_lexicon_tags_15 },
    { 0x71acb954, SUC_lexicon_tags_33 },
    { 0xa2ecb955, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cb95a, SUC_lexicon_tags_0 },
    { 0x586cb95a, SUC_lexicon_tags_11 },
    { 0x112cb95c, SUC_lexicon_tags_15 },
    { 0x9bccb95c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384cb961, SUC_lexicon_tags_0 },
    { 0x4b2cb961, SUC_lexicon_tags_12 },
    { 0xa86cb963, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x714cb965, SUC_lexicon_tags_2 },
    { 0xd00cb965, SUC_lexicon_tags_37 },
    { 0xee4cb966, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332cb96b, SUC_lexicon_tags_6 },
    { 0xd7acb96c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32cb974, SUC_lexicon_tags_15 },
    { 0xa0cb975, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf70cb977, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4cb97a, SUC_lexicon_tags_15 },
    { 0x6c8cb97b, SUC_lexicon_tags_37 },
    { 0xf52cb97c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6cb980, SUC_lexicon_tags_62 },
    { 0x8dacb980, SUC_lexicon_tags_167 },
    { 0x2a0cb980, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6bacb984, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2cb987, SUC_lexicon_tags_37 },
    { 0xad6cb988, SUC_lexicon_tags_27 },
    { 0x7cecb988, SUC_lexicon_tags_30 },
    { 0x518cb98a, SUC_lexicon_tags_19 },
    { 0x714cb98a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x21ccb98d, SUC_lexicon_tags_10 },
    { 0x108cb98e, SUC_lexicon_tags_12 },
    { 0xbcecb98e, SUC_lexicon_tags_47 },
    { 0xf5ccb98f, SUC_lexicon_tags_2 },
    { 0x1e2cb991, SUC_lexicon_tags_23 },
    { 0xd9acb991, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc86cb994, SUC_lexicon_tags_16 },
    { 0x814cb994, SUC_lexicon_tags_141 },
    { 0x740cb996, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb6cb998, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84cb99e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x496cb9a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22acb9a3, SUC_lexicon_tags_15 },
    { 0xedccb9a3, SUC_lexicon_tags_235 },
    { 0x302cb9a3, SUC_lexicon_tags_9 },
    { 0xa8ccb9a6, SUC_lexicon_tags_17 },
    { 0x518cb9a7, SUC_lexicon_tags_65 },
    { 0x11acb9a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x286cb9aa, SUC_lexicon_tags_15 },
    { 0xbe6cb9ab, SUC_lexicon_tags_16 },
    { 0x472cb9ac, SUC_lexicon_tags_37 },
    { 0x84cb9ac, SUC_lexicon_tags_79 },
    { 0x6acb9ae, SUC_lexicon_tags_2 },
    { 0x36cb9af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8cb9b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc86cb9b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa8cb9b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x132cb9bb, SUC_lexicon_tags_32 },
    { 0x39acb9bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bacb9bf, SUC_lexicon_tags_17 },
    { 0x190cb9c0, SUC_lexicon_tags_25 },
    { 0xf14cb9c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x38cb9c3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x794cb9c5, SUC_lexicon_tags_23 },
    { 0x3e0cb9c6, SUC_lexicon_tags_37 },
    { 0xc72cb9c5, SUC_lexicon_tags_25 },
    { 0x2eccb9c8, SUC_lexicon_tags_12 },
    { 0x25ccb9c8, SUC_lexicon_tags_29 },
    { 0xbf2cb9c7, SUC_lexicon_tags_53 },
    { 0x4dacb9cb, SUC_lexicon_tags_30 },
    { 0x2d2cb9cb, SUC_lexicon_tags_60 },
    { 0xef8cb9c7, SUC_lexicon_tags_25 },
    { 0x610cb9cb, SUC_lexicon_tags_14 },
    { 0x94cb9cf, SUC_lexicon_tags_15 },
    { 0xb12cb9ce, SUC_lexicon_tags_12 },
    { 0xfecb9c7, SUC_lexicon_tags_28 },
    { 0x9cecb9d2, SUC_lexicon_tags_25 },
    { 0xa6ecb9d1, SUC_lexicon_tags_60 },
    { 0xa80cb9d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564cb9da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8cb9df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d6cb9e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80cb9e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724cb9ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4cecb9ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42cb9f6, SUC_lexicon_tags_25 },
    { 0x8bacb9f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe9ccb9f9, SUC_lexicon_tags_47 },
    { 0x698cb9fa, SUC_lexicon_tags_77 },
    { 0x81ccb9fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666cba01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86cba04, SUC_lexicon_tags_8 },
    { 0x710cba05, SUC_lexicon_tags_33 },
    { 0x2a2cba06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2cba09, SUC_lexicon_tags_29 },
    { 0x7cecba0a, SUC_lexicon_tags_23 },
    { 0xcc6cba0a, SUC_lexicon_tags_37 },
    { 0x7f0cba0c, SUC_lexicon_tags_33 },
    { 0x83ecba0d, SUC_lexicon_tags_17 },
    { 0xc76cba0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36cba12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf48cba14, SUC_lexicon_tags_33 },
    { 0x2eecba14, SUC_lexicon_tags_37 },
    { 0xa12cba15, SUC_lexicon_tags_92 },
    { 0x8a0cba15, SUC_lexicon_tags_25 },
    { 0xe0acba15, SUC_lexicon_tags_27 },
    { 0xa4ccba17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8cba1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14cba23, SUC_lexicon_tags_39 },
    { 0xe86cba24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ccba2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a6cba2c, SUC_lexicon_tags_11 },
    { 0x30cba2c, SUC_lexicon_tags_65 },
    { 0xfcccba2e, SUC_lexicon_tags_11 },
    { 0x560cba2f, SUC_lexicon_tags_35 },
    { 0x67acba30, SUC_lexicon_tags_15 },
    { 0x9a6cba31, SUC_lexicon_tags_25 },
    { 0xdeacba31, SUC_lexicon_tags_11 },
    { 0xc46cba32, SUC_lexicon_tags_37 },
    { 0x4decba34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82cba37, SUC_lexicon_tags_27 },
    { 0x800cba38, SUC_lexicon_tags_17 },
    { 0xb0ecba39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2ecba3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2cba3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88cba42, SUC_lexicon_tags_1 },
    { 0xee2cba43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4cba46, SUC_lexicon_tags_37 },
    { 0x146cba46, SUC_lexicon_tags_25 },
    { 0x1f2cba47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6cba4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdecba50, SUC_lexicon_tags_17 },
    { 0x2c0cba50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6cba53, SUC_lexicon_tags_3 },
    { 0xbbccba54, SUC_lexicon_tags_63 },
    { 0x20ecba55, SUC_lexicon_tags_46 },
    { 0x4d2cba55, SUC_lexicon_tags_15 },
    { 0xb72cba57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x644cba59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8dacba5b, SUC_lexicon_tags_14 },
    { 0xc2ccba5c, SUC_lexicon_tags_1 },
    { 0x9f2cba5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7acba61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb36cba63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196cba67, SUC_lexicon_tags_17 },
    { 0x39ecba67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656cba6f, SUC_lexicon_tags_1 },
    { 0x630cba70, SUC_lexicon_tags_7 },
    { 0xfb0cba71, SUC_lexicon_tags_32 },
    { 0xbbccba72, SUC_lexicon_tags_11 },
    { 0x1e4cba72, SUC_lexicon_tags_26 },
    { 0x2b0cba70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3ccba76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274cba7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf2cba7c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x86ccba7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5acba82, SUC_lexicon_tags_33 },
    { 0xb10cba82, SUC_lexicon_tags_46 },
    { 0x660cba84, SUC_lexicon_tags_2 },
    { 0x390cba84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ecba8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6cba8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35acba92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x542cba94, SUC_lexicon_tags_37 },
    { 0xfe8cba95, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x176cba98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x436cba9b, SUC_lexicon_tags_19 },
    { 0x31ecba9c, SUC_lexicon_tags_23 },
    { 0x90acba9c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5accbaa0, SUC_lexicon_tags_30 },
    { 0x452cbaa1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2cbaa9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x25acbaab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314cbab1, SUC_lexicon_tags_7 },
    { 0x906cbab2, SUC_lexicon_tags_37 },
    { 0x9fccbab2, SUC_lexicon_tags_8 },
    { 0xc0ccbab4, SUC_lexicon_tags_47 },
    { 0x248cbab5, SUC_lexicon_tags_29 },
    { 0xe7ecbab6, SUC_lexicon_tags_15 },
    { 0x444cbab7, SUC_lexicon_tags_37 },
    { 0xd62cbab7, SUC_lexicon_tags_14 },
    { 0xdd2cbab9, SUC_lexicon_tags_32 },
    { 0x4b6cbab4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988cbabe, SUC_lexicon_tags_27 },
    { 0xde4cbabe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868cbac4, SUC_lexicon_tags_37 },
    { 0xe7ecbac5, SUC_lexicon_tags_11 },
    { 0x73ccbac6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16acbac8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac2cbaca, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8acbad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04cbad5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0cbad9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c0cbadb, SUC_lexicon_tags_0 },
    { 0x7eecbadb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddccbade, SUC_lexicon_tags_336 },
    { 0x58cbadf, SUC_lexicon_tags_37 },
    { 0x41ccbade, SUC_lexicon_tags_1 },
    { 0x198cbae1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6accbae3, SUC_lexicon_tags_0 },
    { 0x15ccbae4, SUC_lexicon_tags_37 },
    { 0x6d8cbae5, SUC_lexicon_tags_69 },
    { 0xb32cbae3, SUC_lexicon_tags_7 },
    { 0x58cbae7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6cbaec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacecbaf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426cbaf4, SUC_lexicon_tags_60 },
    { 0x64ccbaf5, SUC_lexicon_tags_25 },
    { 0xd2ecbaf6, SUC_lexicon_tags_25 },
    { 0xef2cbaf7, SUC_lexicon_tags_28 },
    { 0x62acbaf8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3accbafa, SUC_lexicon_tags_17 },
    { 0x70cbafb, SUC_lexicon_tags_15 },
    { 0x5ccbafb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4cbaff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x386cbb02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ccbb05, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacecbb09, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa6ccbb0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca2cbb0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x646cbb10, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0cbb13, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dacbb20, SUC_lexicon_tags_45 },
    { 0xc0ecbb21, SUC_lexicon_tags_17 },
    { 0xab4cbb21, SUC_lexicon_tags_45 },
    { 0xd0acbb21, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78cbb29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ccbb30, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x80ecbb32, SUC_lexicon_tags_63 },
    { 0xbcccbb33, SUC_lexicon_tags_14 },
    { 0x7f2cbb32, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2cbb39, SUC_lexicon_tags_2 },
    { 0xd8ccbb3a, SUC_lexicon_tags_55 },
    { 0x37ecbb3a, SUC_lexicon_tags_43 },
    { 0xe34cbb3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f8cbb3e, SUC_lexicon_tags_21 },
    { 0x368cbb3f, SUC_lexicon_tags_17 },
    { 0x60acbb3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44cbb45, SUC_lexicon_tags_14 },
    { 0xa36cbb46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aacbb49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648cbb4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ecbb51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa1acbb53, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x732cbb56, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc74cbb58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcccbb5a, SUC_lexicon_tags_12 },
    { 0xb6acbb5b, SUC_lexicon_tags_15 },
    { 0xc6ccbb5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4cbb5f, SUC_lexicon_tags_82 },
    { 0x98cbb60, SUC_lexicon_tags_63 },
    { 0x60ccbb5f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aecbb64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8ccbb66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894cbb6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0cbb72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ccbb75, SUC_lexicon_tags_33 },
    { 0xdd8cbb76, SUC_lexicon_tags_19 },
    { 0xb1acbb76, SUC_lexicon_tags_37 },
    { 0x1f8cbb78, SUC_lexicon_tags_19 },
    { 0x65ecbb79, SUC_lexicon_tags_38 },
    { 0x514cbb79, SUC_lexicon_tags_33 },
    { 0x4bacbb7b, SUC_lexicon_tags_33 },
    { 0x2eacbb7a, SUC_lexicon_tags_2 },
    { 0xbf6cbb76, SUC_lexicon_tags_23 },
    { 0x736cbb7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fecbb89, SUC_lexicon_tags_37 },
    { 0x864cbb89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8eecbb8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb32cbb8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276cbb94, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x936cbb96, SUC_lexicon_tags_25 },
    { 0x822cbb97, SUC_lexicon_tags_5 },
    { 0x758cbb98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x36acbb9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346cbba1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44cbba4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cecbbab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecbbb6, SUC_lexicon_tags_351 },
    { 0x1accbbb7, SUC_lexicon_tags_11 },
    { 0x746cbbb6, SUC_lexicon_tags_10 },
    { 0x488cbbb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484cbbc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ecbbc3, SUC_lexicon_tags_28 },
    { 0xd04cbbc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ccbbc8, SUC_lexicon_tags_49 },
    { 0xca8cbbc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854cbbce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98cbbd1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fecbbd5, SUC_lexicon_tags_15 },
    { 0xc46cbbd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ccbbd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f8cbbdb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x59acbbdd, SUC_lexicon_tags_12 },
    { 0xb3acbbde, SUC_lexicon_tags_3 },
    { 0x956cbbdd, SUC_lexicon_tags_8 },
    { 0x956cbbde, SUC_lexicon_tags_14 },
    { 0x66ecbbe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x496cbbe3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136cbbe8, SUC_lexicon_tags_11 },
    { 0x8b2cbbe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46cbbeb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ccbbf2, SUC_lexicon_tags_63 },
    { 0x174cbbf3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff8cbbf5, SUC_lexicon_tags_14 },
    { 0x5d2cbbf6, SUC_lexicon_tags_15 },
    { 0x798cbbf5, SUC_lexicon_tags_63 },
    { 0x3aecbbf8, SUC_lexicon_tags_18 },
    { 0xeeacbbf7, SUC_lexicon_tags_47 },
    { 0xe1ccbbfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92ecbbfc, SUC_lexicon_tags_8 },
    { 0x974cbbfd, SUC_lexicon_tags_29 },
    { 0x88acbbfe, SUC_lexicon_tags_35 },
    { 0x89ecbbff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc58cbc01, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458cbc06, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80cbc09, SUC_lexicon_tags_184 },
    { 0x742cbc09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34cbc0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ecbc14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2cbc17, SUC_lexicon_tags_46 },
    { 0x31acbc18, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfb8cbc1a, SUC_lexicon_tags_48 },
    { 0x8decbc1b, SUC_lexicon_tags_11 },
    { 0xe92cbc1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd68cbc1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x130cbc21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4cbc26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x474cbc29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x966cbc2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4cbc32, SUC_lexicon_tags_49 },
    { 0x9f6cbc33, SUC_lexicon_tags_33 },
    { 0xd74cbc34, SUC_lexicon_tags_55 },
    { 0xd26cbc34, SUC_lexicon_tags_11 },
    { 0x968cbc36, SUC_lexicon_tags_47 },
    { 0xa62cbc37, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8cacbc39, SUC_lexicon_tags_29 },
    { 0x580cbc3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368cbc3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806cbc43, SUC_lexicon_tags_9 },
    { 0x448cbc44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42acbc47, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x230cbc4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dacbc50, SUC_lexicon_tags_33 },
    { 0xc16cbc50, SUC_lexicon_tags_1 },
    { 0x368cbc51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dccbc55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5acbc57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4aecbc59, SUC_lexicon_tags_17 },
    { 0xdd6cbc59, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6cbc5f, SUC_lexicon_tags_43 },
    { 0x7facbc60, SUC_lexicon_tags_38 },
    { 0xd26cbc61, SUC_lexicon_tags_6 },
    { 0x3becbc60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776cbc67, SUC_lexicon_tags_1 },
    { 0x61ecbc67, SUC_lexicon_tags_37 },
    { 0x266cbc69, SUC_lexicon_tags_23 },
    { 0x1fccbc6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ccbc6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cbc74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4acbc7a, SUC_lexicon_tags_21 },
    { 0xe7acbc7a, SUC_lexicon_tags_34 },
    { 0xb70cbc7a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2cbc7f, SUC_lexicon_tags_11 },
    { 0x164cbc80, SUC_lexicon_tags_46 },
    { 0xc36cbc81, SUC_lexicon_tags_6 },
    { 0x892cbc82, SUC_lexicon_tags_12 },
    { 0xb56cbc82, SUC_lexicon_tags_23 },
    { 0x820cbc82, SUC_lexicon_tags_17 },
    { 0x234cbc85, SUC_lexicon_tags_26 },
    { 0x6c4cbc86, SUC_lexicon_tags_26 },
    { 0x492cbc87, SUC_lexicon_tags_25 },
    { 0x12ccbc84, SUC_lexicon_tags_1 },
    { 0xe30cbc84, SUC_lexicon_tags_30 },
    { 0x676cbc89, SUC_lexicon_tags_11 },
    { 0x5f8cbc84, SUC_lexicon_tags_46 },
    { 0x9a6cbc8c, SUC_lexicon_tags_1 },
    { 0xbe4cbc8d, SUC_lexicon_tags_66 },
    { 0x1facbc8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cacbc91, SUC_lexicon_tags_65 },
    { 0xa12cbc92, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ecbc97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4cbc9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8cbca2, SUC_lexicon_tags_25 },
    { 0x89ccbca2, SUC_lexicon_tags_12 },
    { 0x94ccbca4, SUC_lexicon_tags_14 },
    { 0x69ccbca2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cbca8, SUC_lexicon_tags_45 },
    { 0xc3acbca9, SUC_lexicon_tags_12 },
    { 0x842cbcaa, SUC_lexicon_tags_7 },
    { 0xa14cbcab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2cbcaf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ccbcb4, SUC_lexicon_tags_25 },
    { 0xa98cbcb5, SUC_lexicon_tags_7 },
    { 0x2ecbcb6, SUC_lexicon_tags_29 },
    { 0x630cbcb7, SUC_lexicon_tags_31 },
    { 0x5ccbcb5, SUC_lexicon_tags_31 },
    { 0x264cbcb6, SUC_lexicon_tags_45 },
    { 0x2aacbcb6, SUC_lexicon_tags_7 },
    { 0xfd0cbcbb, SUC_lexicon_tags_1 },
    { 0xc10cbcbc, SUC_lexicon_tags_14 },
    { 0x704cbcbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89acbcc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdecbcc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76acbcc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344cbccb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcc0cbccd, SUC_lexicon_tags_33 },
    { 0x264cbcce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf8cbcd0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x218cbcd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eacbcda, SUC_lexicon_tags_6 },
    { 0x948cbcda, SUC_lexicon_tags_17 },
    { 0x948cbcdb, SUC_lexicon_tags_1 },
    { 0x7a4cbcdc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fccbce0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50cbce5, SUC_lexicon_tags_45 },
    { 0x814cbce6, SUC_lexicon_tags_1 },
    { 0xf58cbce7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa06cbce9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ccbcee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6cbcf1, SUC_lexicon_tags_37 },
    { 0x5f8cbcf2, SUC_lexicon_tags_25 },
    { 0xb7acbcf3, SUC_lexicon_tags_56 },
    { 0xb80cbcf2, SUC_lexicon_tags_11 },
    { 0xc6acbcf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8cbcf9, SUC_lexicon_tags_14 },
    { 0x284cbcfa, SUC_lexicon_tags_38 },
    { 0x6a4cbcf9, SUC_lexicon_tags_11 },
    { 0xabccbcfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3cecbcfe, SUC_lexicon_tags_494 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ccbd02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x86cbd04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbccbd0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9eacbd0d, SUC_lexicon_tags_17 },
    { 0x8d2cbd0e, SUC_lexicon_tags_25 },
    { 0x730cbd0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86acbd13, SUC_lexicon_tags_45 },
    { 0x5eecbd14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80cbd17, SUC_lexicon_tags_16 },
    { 0x3c4cbd17, SUC_lexicon_tags_27 },
    { 0x15ecbd19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x712cbd1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ccbd1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cbd24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2cbd27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92acbd2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32acbd33, SUC_lexicon_tags_37 },
    { 0x85ccbd34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34ccbd36, SUC_lexicon_tags_7 },
    { 0xf2cbd37, SUC_lexicon_tags_26 },
    { 0xf94cbd38, SUC_lexicon_tags_37 },
    { 0x414cbd37, SUC_lexicon_tags_15 },
    { 0xbf0cbd36, SUC_lexicon_tags_17 },
    { 0x432cbd3b, SUC_lexicon_tags_31 },
    { 0x7ecbd3c, SUC_lexicon_tags_25 },
    { 0x874cbd3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6cbd48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe70cbd4a, SUC_lexicon_tags_4 },
    { 0x8accbd4b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa44cbd4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacccbd4f, SUC_lexicon_tags_26 },
    { 0x370cbd50, SUC_lexicon_tags_188 },
    { 0xcdecbd50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2cbd54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x614cbd56, SUC_lexicon_tags_63 },
    { 0x1ecbd57, SUC_lexicon_tags_45 },
    { 0xad0cbd57, SUC_lexicon_tags_25 },
    { 0xb60cbd59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968cbd5e, SUC_lexicon_tags_9 },
    { 0x43ecbd5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7facbd67, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x22ccbd69, SUC_lexicon_tags_2 },
    { 0x1e6cbd69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4cbd70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d0cbd72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10ecbd74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0ccbd76, SUC_lexicon_tags_60 },
    { 0xa84cbd77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x918cbd79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2cbd81, SUC_lexicon_tags_1 },
    { 0x708cbd82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2ccbd84, SUC_lexicon_tags_11 },
    { 0x3f4cbd84, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32acbd89, SUC_lexicon_tags_2 },
    { 0xcd8cbd8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x502cbd8d, SUC_lexicon_tags_11 },
    { 0x40acbd8e, SUC_lexicon_tags_25 },
    { 0x560cbd8f, SUC_lexicon_tags_39 },
    { 0x8f4cbd8f, SUC_lexicon_tags_27 },
    { 0xdccbd8f, SUC_lexicon_tags_14 },
    { 0x91acbd91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22cbd97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12cbd9d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x53ccbd9f, SUC_lexicon_tags_37 },
    { 0x320cbda0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xceacbda2, SUC_lexicon_tags_7 },
    { 0xaf8cbda3, SUC_lexicon_tags_27 },
    { 0x578cbda2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1becbda6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488cbdaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x580cbdad, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc52cbdaf, SUC_lexicon_tags_33 },
    { 0xaeccbdaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770cbdb8, SUC_lexicon_tags_38 },
    { 0x902cbdb9, SUC_lexicon_tags_2 },
    { 0x8d0cbdb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa14cbdbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3facbdbe, SUC_lexicon_tags_11 },
    { 0xbe8cbdbf, SUC_lexicon_tags_14 },
    { 0xf20cbdbe, SUC_lexicon_tags_19 },
    { 0x624cbdbf, SUC_lexicon_tags_32 },
    { 0xd84cbdc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98cbdc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2cbdcb, SUC_lexicon_tags_33 },
    { 0x144cbdcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4cbdcf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4cbdd2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732cbdd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970cbdda, SUC_lexicon_tags_7 },
    { 0xd36cbddb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x58ccbddd, SUC_lexicon_tags_7 },
    { 0xbc0cbdde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x566cbde0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2acbde2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89ccbde4, SUC_lexicon_tags_117 },
    { 0x4e6cbde5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cbdeb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x864cbded, SUC_lexicon_tags_15 },
    { 0xcb2cbded, SUC_lexicon_tags_47 },
    { 0x44ecbdee, SUC_lexicon_tags_38 },
    { 0x76ccbdee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2a6cbdf2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256cbdf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2becbdfd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf4acbdff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x130cbe01, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74cbe07, SUC_lexicon_tags_42 },
    { 0xe8cbe07, SUC_lexicon_tags_15 },
    { 0x80acbe09, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5d0cbe0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896cbe12, SUC_lexicon_tags_26 },
    { 0x990cbe13, SUC_lexicon_tags_7 },
    { 0xf70cbe12, SUC_lexicon_tags_37 },
    { 0x848cbe15, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0cbe18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eecbe1c, SUC_lexicon_tags_2 },
    { 0xf7ccbe1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ccbe21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4cbe27, SUC_lexicon_tags_30 },
    { 0x5b6cbe28, SUC_lexicon_tags_48 },
    { 0xf48cbe27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2cbe2c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xdb0cbe2e, SUC_lexicon_tags_1 },
    { 0x17acbe2e, SUC_lexicon_tags_11 },
    { 0x2c2cbe30, SUC_lexicon_tags_11 },
    { 0x884cbe30, SUC_lexicon_tags_1 },
    { 0xf92cbe32, SUC_lexicon_tags_25 },
    { 0x6c0cbe32, SUC_lexicon_tags_15 },
    { 0xbcccbe34, SUC_lexicon_tags_6 },
    { 0xabecbe35, SUC_lexicon_tags_57 },
    { 0xea4cbe36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2acbe39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794cbe4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24cbe4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dccbe51, SUC_lexicon_tags_17 },
    { 0xec6cbe52, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cbe55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ccbe5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400cbe5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4cbe62, SUC_lexicon_tags_47 },
    { 0x3f2cbe62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ecbe6a, SUC_lexicon_tags_12 },
    { 0x1eccbe6a, SUC_lexicon_tags_12 },
    { 0x138cbe6c, SUC_lexicon_tags_31 },
    { 0xe02cbe6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884cbe71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2cbe75, SUC_lexicon_tags_2 },
    { 0x48acbe76, SUC_lexicon_tags_25 },
    { 0xa1acbe77, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4cbe7a, SUC_lexicon_tags_5 },
    { 0x4e8cbe7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a2cbe7d, SUC_lexicon_tags_68 },
    { 0x286cbe7e, SUC_lexicon_tags_44 },
    { 0xad6cbe7f, SUC_lexicon_tags_38 },
    { 0x3f6cbe80, SUC_lexicon_tags_2 },
    { 0xdaecbe81, SUC_lexicon_tags_13 },
    { 0x98acbe80, SUC_lexicon_tags_37 },
    { 0xa74cbe83, SUC_lexicon_tags_17 },
    { 0xbb8cbe83, SUC_lexicon_tags_27 },
    { 0xa32cbe85, SUC_lexicon_tags_0 },
    { 0xb0acbe7d, SUC_lexicon_tags_7 },
    { 0x890cbe85, SUC_lexicon_tags_34 },
    { 0xeacbe7f, SUC_lexicon_tags_19 },
    { 0xd8acbe89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x868cbe8b, SUC_lexicon_tags_25 },
    { 0xa8cbe8b, SUC_lexicon_tags_8 },
    { 0xbcccbe8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eacbe90, SUC_lexicon_tags_47 },
    { 0xe9ccbe90, SUC_lexicon_tags_70 },
    { 0xf0ccbe90, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3c8cbe94, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3facbe97, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x622cbe9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x992cbe9d, SUC_lexicon_tags_45 },
    { 0x112cbe9e, SUC_lexicon_tags_3 },
    { 0xea6cbe9d, SUC_lexicon_tags_60 },
    { 0xa54cbe9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ecbea2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aecbea6, SUC_lexicon_tags_17 },
    { 0x15ccbea7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde4cbead, SUC_lexicon_tags_17 },
    { 0x4a2cbeae, SUC_lexicon_tags_0 },
    { 0x31acbeaf, SUC_lexicon_tags_11 },
    { 0xaeacbeb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988cbeb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02cbebb, SUC_lexicon_tags_45 },
    { 0xd72cbebc, SUC_lexicon_tags_80 },
    { 0x2becbebc, SUC_lexicon_tags_1 },
    { 0x312cbebd, SUC_lexicon_tags_6 },
    { 0x354cbebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710cbec3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966cbec9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70cbecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ecbed2, SUC_lexicon_tags_14 },
    { 0x97ccbed3, SUC_lexicon_tags_28 },
    { 0xb02cbed2, SUC_lexicon_tags_31 },
    { 0x3d4cbed2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x682cbed8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2cbedc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x378cbedf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8cbee4, SUC_lexicon_tags_9 },
    { 0xd0cbee5, SUC_lexicon_tags_49 },
    { 0xd02cbee6, SUC_lexicon_tags_2 },
    { 0x86ccbee7, SUC_lexicon_tags_1 },
    { 0x198cbee7, SUC_lexicon_tags_57 },
    { 0x54cbee9, SUC_lexicon_tags_15 },
    { 0xaf4cbeea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eccbeee, SUC_lexicon_tags_45 },
    { 0x79ccbeef, SUC_lexicon_tags_25 },
    { 0xf6acbeef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x24acbef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52cbef5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82cbef9, SUC_lexicon_tags_84 },
    { 0xdaecbefa, SUC_lexicon_tags_25 },
    { 0x324cbefb, SUC_lexicon_tags_34 },
    { 0xac6cbefb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaccbf00, SUC_lexicon_tags_37 },
    { 0xb3ccbf01, SUC_lexicon_tags_21 },
    { 0xe18cbf00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ccbf06, SUC_lexicon_tags_17 },
    { 0xd14cbf07, SUC_lexicon_tags_47 },
    { 0x412cbf08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0acbf0b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0cbf0e, SUC_lexicon_tags_33 },
    { 0x2ecbf0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x738cbf11, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4cbf14, SUC_lexicon_tags_11 },
    { 0xf1ecbf15, SUC_lexicon_tags_17 },
    { 0x1ecbf16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910cbf1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936cbf26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x446cbf28, SUC_lexicon_tags_8 },
    { 0xb88cbf29, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6cbf2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1acbf2f, SUC_lexicon_tags_38 },
    { 0xba8cbf2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56ccbf32, SUC_lexicon_tags_2 },
    { 0xbe0cbf33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e0cbf35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40acbf3a, SUC_lexicon_tags_7 },
    { 0x47acbf3b, SUC_lexicon_tags_14 },
    { 0x7bccbf3c, SUC_lexicon_tags_26 },
    { 0xa82cbf3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x674cbf40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9dccbf42, SUC_lexicon_tags_2 },
    { 0xee4cbf42, SUC_lexicon_tags_32 },
    { 0x8e2cbf44, SUC_lexicon_tags_45 },
    { 0xe8ecbf42, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x9decbf47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32acbf49, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xeb4cbf4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa8cbf4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3acbf50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x792cbf52, SUC_lexicon_tags_65 },
    { 0xaeccbf53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x600cbf55, SUC_lexicon_tags_45 },
    { 0x66acbf56, SUC_lexicon_tags_38 },
    { 0x344cbf57, SUC_lexicon_tags_29 },
    { 0xe6ccbf57, SUC_lexicon_tags_624 },
    { 0x524cbf58, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce4cbf5b, SUC_lexicon_tags_25 },
    { 0xbc0cbf5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x394cbf5e, SUC_lexicon_tags_7 },
    { 0x552cbf5f, SUC_lexicon_tags_63 },
    { 0xcaecbf5f, SUC_lexicon_tags_25 },
    { 0x870cbf61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834cbf6a, SUC_lexicon_tags_11 },
    { 0x2e0cbf6b, SUC_lexicon_tags_15 },
    { 0xb0cbf6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ccbf70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ccbf72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x438cbf74, SUC_lexicon_tags_47 },
    { 0xd20cbf75, SUC_lexicon_tags_14 },
    { 0xa2ccbf74, SUC_lexicon_tags_45 },
    { 0x4aacbf77, SUC_lexicon_tags_42 },
    { 0x262cbf78, SUC_lexicon_tags_31 },
    { 0xf96cbf79, SUC_lexicon_tags_7 },
    { 0xfa6cbf7a, SUC_lexicon_tags_37 },
    { 0xf58cbf79, SUC_lexicon_tags_12 },
    { 0x2eecbf76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x34ecbf7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4cbf81, SUC_lexicon_tags_37 },
    { 0x122cbf82, SUC_lexicon_tags_39 },
    { 0xc48cbf81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dacbf86, SUC_lexicon_tags_57 },
    { 0xfaacbf87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x184cbf89, SUC_lexicon_tags_15 },
    { 0xd5acbf8a, SUC_lexicon_tags_83 },
    { 0x9facbf8b, SUC_lexicon_tags_0 },
    { 0x436cbf8b, SUC_lexicon_tags_17 },
    { 0x60ecbf89, SUC_lexicon_tags_1 },
    { 0x512cbf8a, SUC_lexicon_tags_19 },
    { 0x3a6cbf8f, SUC_lexicon_tags_7 },
    { 0xd2ccbf8b, SUC_lexicon_tags_8 },
    { 0x4a6cbf91, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2cbf99, SUC_lexicon_tags_17 },
    { 0x18acbf9a, SUC_lexicon_tags_1 },
    { 0xca4cbf9b, SUC_lexicon_tags_37 },
    { 0x378cbf9c, SUC_lexicon_tags_25 },
    { 0x518cbf9a, SUC_lexicon_tags_12 },
    { 0x37acbf9a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x33acbfa1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x886cbfa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x296cbfa6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc60cbfa8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4cbfae, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ccbfb2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf68cbfb4, SUC_lexicon_tags_11 },
    { 0xf64cbfb5, SUC_lexicon_tags_11 },
    { 0xfeacbfb6, SUC_lexicon_tags_30 },
    { 0x57acbfb7, SUC_lexicon_tags_15 },
    { 0x3eccbfb4, SUC_lexicon_tags_46 },
    { 0x6becbfb7, SUC_lexicon_tags_17 },
    { 0x944cbfb9, SUC_lexicon_tags_10 },
    { 0xc4ecbfb9, SUC_lexicon_tags_0 },
    { 0x2cbfbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd38cbfbe, SUC_lexicon_tags_2 },
    { 0x8f0cbfbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760cbfc3, SUC_lexicon_tags_45 },
    { 0x416cbfc4, SUC_lexicon_tags_2 },
    { 0xd70cbfc4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x910cbfc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12cbfcb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d4cbfcd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x138cbfd0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbcacbfd2, SUC_lexicon_tags_11 },
    { 0x6a6cbfd2, SUC_lexicon_tags_33 },
    { 0xff4cbfd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x324cbfd7, SUC_lexicon_tags_27 },
    { 0xb5acbfd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8cbfdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f6cbfde, SUC_lexicon_tags_56 },
    { 0xf4cbfde, SUC_lexicon_tags_35 },
    { 0x946cbfde, SUC_lexicon_tags_17 },
    { 0x5aacbfe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16acbfe3, SUC_lexicon_tags_1 },
    { 0xba6cbfe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404cbfe9, SUC_lexicon_tags_3 },
    { 0x80ecbfe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25acbff2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x200cbff5, SUC_lexicon_tags_7 },
    { 0x880cbff5, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x7f2cbff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ccbffe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dacc003, SUC_lexicon_tags_37 },
    { 0x1a8cc004, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85acc00a, SUC_lexicon_tags_7 },
    { 0x6c6cc00b, SUC_lexicon_tags_25 },
    { 0xda8cc00c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7acc010, SUC_lexicon_tags_11 },
    { 0x260cc010, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x466cc014, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894cc01c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5acc023, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a8cc025, SUC_lexicon_tags_19 },
    { 0xf08cc026, SUC_lexicon_tags_31 },
    { 0x220cc027, SUC_lexicon_tags_12 },
    { 0xb32cc027, SUC_lexicon_tags_30 },
    { 0xdc6cc028, SUC_lexicon_tags_37 },
    { 0xae6cc02a, SUC_lexicon_tags_37 },
    { 0x7c8cc028, SUC_lexicon_tags_4 },
    { 0xcd2cc02c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cc035, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xcf8cc037, SUC_lexicon_tags_37 },
    { 0x64ccc038, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4fecc03a, SUC_lexicon_tags_74 },
    { 0xcf8cc03b, SUC_lexicon_tags_63 },
    { 0xbfacc03c, SUC_lexicon_tags_37 },
    { 0xdaccc03c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3acc041, SUC_lexicon_tags_57 },
    { 0xf94cc042, SUC_lexicon_tags_3 },
    { 0x6a2cc041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89ecc045, SUC_lexicon_tags_42 },
    { 0x62cc045, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99acc048, SUC_lexicon_tags_11 },
    { 0xb54cc049, SUC_lexicon_tags_19 },
    { 0x8f4cc049, SUC_lexicon_tags_19 },
    { 0xcd4cc04b, SUC_lexicon_tags_2 },
    { 0x8a8cc04a, SUC_lexicon_tags_2 },
    { 0x37ccc04d, SUC_lexicon_tags_74 },
    { 0xd2ecc04d, SUC_lexicon_tags_33 },
    { 0x758cc04f, SUC_lexicon_tags_7 },
    { 0x8becc050, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa78cc052, SUC_lexicon_tags_15 },
    { 0x3b2cc053, SUC_lexicon_tags_17 },
    { 0x5bacc054, SUC_lexicon_tags_11 },
    { 0x51ccc055, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x508cc057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478cc05b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972cc05f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb0cc061, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe2acc063, SUC_lexicon_tags_46 },
    { 0x58ccc063, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ecc067, SUC_lexicon_tags_9 },
    { 0x38cc068, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9f0cc06a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x41ccc06c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeccc070, SUC_lexicon_tags_17 },
    { 0x59ecc071, SUC_lexicon_tags_1 },
    { 0x2cc072, SUC_lexicon_tags_15 },
    { 0x2c2cc073, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc16cc075, SUC_lexicon_tags_5 },
    { 0x10cc076, SUC_lexicon_tags_38 },
    { 0xd10cc076, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x466cc079, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x330cc07b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26cc082, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8cc084, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6cc08d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984cc098, SUC_lexicon_tags_17 },
    { 0x770cc099, SUC_lexicon_tags_15 },
    { 0xaaacc09a, SUC_lexicon_tags_35 },
    { 0xe6ecc09b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4accc0a2, SUC_lexicon_tags_2 },
    { 0x5ccc0a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2cc0ad, SUC_lexicon_tags_12 },
    { 0x38ccc0ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x280cc0b0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf58cc0b2, SUC_lexicon_tags_17 },
    { 0xf34cc0b2, SUC_lexicon_tags_19 },
    { 0x6e8cc0b4, SUC_lexicon_tags_0 },
    { 0x648cc0b5, SUC_lexicon_tags_7 },
    { 0x90ecc0b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8cc0bf, SUC_lexicon_tags_12 },
    { 0xec2cc0bf, SUC_lexicon_tags_87 },
    { 0x268cc0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acc0c6, SUC_lexicon_tags_37 },
    { 0xbf0cc0c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x994cc0ca, SUC_lexicon_tags_27 },
    { 0x894cc0cb, SUC_lexicon_tags_2 },
    { 0x59ecc0cc, SUC_lexicon_tags_28 },
    { 0x58ccc0cc, SUC_lexicon_tags_0 },
    { 0xecc0ce, SUC_lexicon_tags_25 },
    { 0x416cc0cf, SUC_lexicon_tags_54 },
    { 0x8ecc0cf, SUC_lexicon_tags_30 },
    { 0x24acc0cf, SUC_lexicon_tags_49 },
    { 0xe12cc0cf, SUC_lexicon_tags_2 },
    { 0x55ccc0d3, SUC_lexicon_tags_6 },
    { 0x6d8cc0cf, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x26ccc0d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab0cc0d8, SUC_lexicon_tags_37 },
    { 0xca2cc0d9, SUC_lexicon_tags_37 },
    { 0xe92cc0da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14acc0de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0cc0e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660cc0ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6cc0f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150cc0f6, SUC_lexicon_tags_45 },
    { 0x8a0cc0f7, SUC_lexicon_tags_47 },
    { 0x590cc0f8, SUC_lexicon_tags_49 },
    { 0xd4acc0f7, SUC_lexicon_tags_19 },
    { 0x176cc0f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77ccc0fc, SUC_lexicon_tags_29 },
    { 0x172cc0fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc100, SUC_lexicon_tags_378 },
    { 0x6c6cc100, SUC_lexicon_tags_49 },
    { 0x944cc102, SUC_lexicon_tags_1 },
    { 0xafccc100, SUC_lexicon_tags_65 },
    { 0xd4acc104, SUC_lexicon_tags_0 },
    { 0x608cc103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5acc108, SUC_lexicon_tags_15 },
    { 0x522cc108, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2d6cc10b, SUC_lexicon_tags_4 },
    { 0x4eecc10b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x34cc10e, SUC_lexicon_tags_10 },
    { 0x22ecc10f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26ecc111, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbc2cc113, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eccc116, SUC_lexicon_tags_56 },
    { 0xc5acc116, SUC_lexicon_tags_17 },
    { 0x73acc118, SUC_lexicon_tags_39 },
    { 0x9eecc119, SUC_lexicon_tags_1 },
    { 0x5d6cc118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaecc120, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8cc123, SUC_lexicon_tags_27 },
    { 0xd42cc124, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2cc12a, SUC_lexicon_tags_2 },
    { 0x3b2cc12b, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xa34cc12d, SUC_lexicon_tags_12 },
    { 0xa88cc12d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x824cc130, SUC_lexicon_tags_40 },
    { 0xf6cc130, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ccc13a, SUC_lexicon_tags_62 },
    { 0xf08cc13a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cc13f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8cc142, SUC_lexicon_tags_4 },
    { 0xdeecc143, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ecc14a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bccc14d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x800cc14f, SUC_lexicon_tags_11 },
    { 0x966cc14f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadacc156, SUC_lexicon_tags_30 },
    { 0xcdccc157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ccc15b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2cc160, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe68cc162, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4cc165, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50cc168, SUC_lexicon_tags_47 },
    { 0xc66cc168, SUC_lexicon_tags_2 },
    { 0x5f4cc16a, SUC_lexicon_tags_103 },
    { 0xd98cc169, SUC_lexicon_tags_21 },
    { 0xa72cc16c, SUC_lexicon_tags_31 },
    { 0x354cc16b, SUC_lexicon_tags_13 },
    { 0xe5ecc16e, SUC_lexicon_tags_15 },
    { 0x6fecc16e, SUC_lexicon_tags_0 },
    { 0xa7ccc170, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x354cc172, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04cc175, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe32cc177, SUC_lexicon_tags_37 },
    { 0x550cc178, SUC_lexicon_tags_12 },
    { 0x63ecc177, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34cc17e, SUC_lexicon_tags_11 },
    { 0xda0cc17f, SUC_lexicon_tags_15 },
    { 0xc9acc180, SUC_lexicon_tags_12 },
    { 0x96ccc181, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x466cc183, SUC_lexicon_tags_26 },
    { 0x42acc184, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0cc187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cc18f, SUC_lexicon_tags_43 },
    { 0x91ecc190, SUC_lexicon_tags_63 },
    { 0xecccc191, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88ecc193, SUC_lexicon_tags_12 },
    { 0x36acc193, SUC_lexicon_tags_91 },
    { 0xb18cc195, SUC_lexicon_tags_37 },
    { 0xb4acc196, SUC_lexicon_tags_7 },
    { 0xf1ecc197, SUC_lexicon_tags_118 },
    { 0xa68cc198, SUC_lexicon_tags_63 },
    { 0xd54cc196, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504cc19d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4cc19f, SUC_lexicon_tags_29 },
    { 0x13acc1a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270cc1a4, SUC_lexicon_tags_1 },
    { 0x468cc1a5, SUC_lexicon_tags_49 },
    { 0x3c4cc1a5, SUC_lexicon_tags_39 },
    { 0xe6ccc1a7, SUC_lexicon_tags_29 },
    { 0x5f8cc1a5, SUC_lexicon_tags_7 },
    { 0xf3acc1a4, SUC_lexicon_tags_11 },
    { 0x88ccc1aa, SUC_lexicon_tags_44 },
    { 0x602cc1ab, SUC_lexicon_tags_15 },
    { 0x2f0cc1aa, SUC_lexicon_tags_14 },
    { 0xe92cc1a4, SUC_lexicon_tags_11 },
    { 0x6d8cc1ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98cc1b3, SUC_lexicon_tags_47 },
    { 0xfbacc1b4, SUC_lexicon_tags_8 },
    { 0x834cc1b3, SUC_lexicon_tags_37 },
    { 0xdcecc1b5, SUC_lexicon_tags_38 },
    { 0x970cc1b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdcacc1b9, SUC_lexicon_tags_33 },
    { 0x864cc1b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ecc1be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ccc1c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ccc1c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9f8cc1c7, SUC_lexicon_tags_49 },
    { 0xaa2cc1c8, SUC_lexicon_tags_37 },
    { 0xb8ecc1c8, SUC_lexicon_tags_37 },
    { 0xe4ccc1ca, SUC_lexicon_tags_37 },
    { 0x8ecc1ca, SUC_lexicon_tags_12 },
    { 0x4bccc1cc, SUC_lexicon_tags_6 },
    { 0xe26cc1cd, SUC_lexicon_tags_11 },
    { 0xb44cc1ce, SUC_lexicon_tags_30 },
    { 0xe4ecc1cf, SUC_lexicon_tags_10 },
    { 0xce6cc1d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68acc1d4, SUC_lexicon_tags_12 },
    { 0x56ccc1d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed8cc1d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x794cc1d9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384cc1de, SUC_lexicon_tags_28 },
    { 0x15ecc1de, SUC_lexicon_tags_25 },
    { 0x686cc1e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204cc1e4, SUC_lexicon_tags_6 },
    { 0x57ccc1e5, SUC_lexicon_tags_26 },
    { 0xb1ccc1e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f0cc1e8, SUC_lexicon_tags_4 },
    { 0xa1ecc1e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x806cc1ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x736cc1ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc62cc1f0, SUC_lexicon_tags_43 },
    { 0x77acc1f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ccc1fa, SUC_lexicon_tags_9 },
    { 0xbf0cc1fb, SUC_lexicon_tags_12 },
    { 0x178cc1fc, SUC_lexicon_tags_11 },
    { 0xd48cc1fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6cc204, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510cc20a, SUC_lexicon_tags_23 },
    { 0xf1ecc20b, SUC_lexicon_tags_7 },
    { 0xceccc20c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa88cc20e, SUC_lexicon_tags_15 },
    { 0x60ecc20f, SUC_lexicon_tags_27 },
    { 0x96ecc20e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecc212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48cc215, SUC_lexicon_tags_40 },
    { 0x592cc216, SUC_lexicon_tags_42 },
    { 0x296cc215, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878cc21c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x528cc21f, SUC_lexicon_tags_4 },
    { 0x5becc21f, SUC_lexicon_tags_45 },
    { 0x1bccc221, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8cc224, SUC_lexicon_tags_0 },
    { 0xad6cc225, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ecc228, SUC_lexicon_tags_25 },
    { 0xfd6cc229, SUC_lexicon_tags_37 },
    { 0x5d2cc22a, SUC_lexicon_tags_1 },
    { 0x342cc22b, SUC_lexicon_tags_12 },
    { 0x6e4cc22a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110cc230, SUC_lexicon_tags_14 },
    { 0xb52cc231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0cc236, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cc23d, SUC_lexicon_tags_11 },
    { 0x7f2cc23e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x98ccc240, SUC_lexicon_tags_1 },
    { 0x3c8cc241, SUC_lexicon_tags_11 },
    { 0x544cc242, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdccc246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626cc24f, SUC_lexicon_tags_2 },
    { 0x9becc24f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482cc255, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38cc259, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9cacc25b, SUC_lexicon_tags_37 },
    { 0x3a4cc25b, SUC_lexicon_tags_366 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500cc261, SUC_lexicon_tags_7 },
    { 0xcccc261, SUC_lexicon_tags_17 },
    { 0xdeccc263, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x608cc266, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4decc268, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x852cc26a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2cc274, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3acc276, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08cc279, SUC_lexicon_tags_13 },
    { 0x738cc27a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x752cc27c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ecc282, SUC_lexicon_tags_30 },
    { 0xfcecc283, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x964cc286, SUC_lexicon_tags_23 },
    { 0xa40cc287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee2cc289, SUC_lexicon_tags_27 },
    { 0x58ccc289, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x418cc28c, SUC_lexicon_tags_47 },
    { 0x450cc28d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x928cc28f, SUC_lexicon_tags_11 },
    { 0x700cc290, SUC_lexicon_tags_32 },
    { 0x6fccc291, SUC_lexicon_tags_38 },
    { 0xd86cc292, SUC_lexicon_tags_17 },
    { 0x364cc293, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x380cc296, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78cc2a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd14cc2a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340cc2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x458cc2ad, SUC_lexicon_tags_30 },
    { 0xd52cc2ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ecc2b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80cc2b7, SUC_lexicon_tags_17 },
    { 0x9fecc2b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10cc2bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd76cc2bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cc2c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x438cc2c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x744cc2c6, SUC_lexicon_tags_17 },
    { 0x79acc2c7, SUC_lexicon_tags_2 },
    { 0xafccc2c8, SUC_lexicon_tags_25 },
    { 0xd3ecc2c8, SUC_lexicon_tags_60 },
    { 0x66ecc2ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfccc2d3, SUC_lexicon_tags_30 },
    { 0xb52cc2d4, SUC_lexicon_tags_49 },
    { 0xb3ccc2d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8acc2db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b0cc2dd, SUC_lexicon_tags_23 },
    { 0x2bacc2de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0cc2e2, SUC_lexicon_tags_12 },
    { 0x90ecc2e3, SUC_lexicon_tags_23 },
    { 0x386cc2e4, SUC_lexicon_tags_15 },
    { 0x3decc2e5, SUC_lexicon_tags_7 },
    { 0x566cc2e6, SUC_lexicon_tags_19 },
    { 0x86cc2e7, SUC_lexicon_tags_37 },
    { 0x182cc2e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0ccc2ea, SUC_lexicon_tags_1 },
    { 0xa16cc2eb, SUC_lexicon_tags_45 },
    { 0x378cc2ec, SUC_lexicon_tags_37 },
    { 0xc2acc2ea, SUC_lexicon_tags_47 },
    { 0x172cc2eb, SUC_lexicon_tags_11 },
    { 0x81ccc2ef, SUC_lexicon_tags_8 },
    { 0xbbecc2f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96cc2f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf82cc2f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cc2fc, SUC_lexicon_tags_0 },
    { 0xa1ecc2fd, SUC_lexicon_tags_45 },
    { 0x358cc2fe, SUC_lexicon_tags_55 },
    { 0x8bccc2fc, SUC_lexicon_tags_12 },
    { 0xd18cc300, SUC_lexicon_tags_37 },
    { 0xe1ccc301, SUC_lexicon_tags_21 },
    { 0x354cc302, SUC_lexicon_tags_7 },
    { 0xc32cc303, SUC_lexicon_tags_1 },
    { 0xd84cc302, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8cc308, SUC_lexicon_tags_11 },
    { 0xe98cc309, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8cc30c, SUC_lexicon_tags_25 },
    { 0x504cc30d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842cc312, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ccc315, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ecc31b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x360cc31d, SUC_lexicon_tags_25 },
    { 0xb0ccc31e, SUC_lexicon_tags_37 },
    { 0x236cc31f, SUC_lexicon_tags_70 },
    { 0xbcecc320, SUC_lexicon_tags_17 },
    { 0x45ccc320, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ecc324, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96cc32d, SUC_lexicon_tags_1 },
    { 0x5f2cc32e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972cc332, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72cc334, SUC_lexicon_tags_54 },
    { 0xa48cc334, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96cc338, SUC_lexicon_tags_25 },
    { 0xc8ccc339, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44cc33c, SUC_lexicon_tags_1 },
    { 0x86ccc33d, SUC_lexicon_tags_17 },
    { 0x5d2cc33e, SUC_lexicon_tags_26 },
    { 0xe78cc33f, SUC_lexicon_tags_66 },
    { 0x6d8cc340, SUC_lexicon_tags_25 },
    { 0x294cc341, SUC_lexicon_tags_17 },
    { 0x712cc341, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2cc347, SUC_lexicon_tags_14 },
    { 0x9bacc348, SUC_lexicon_tags_10 },
    { 0x290cc349, SUC_lexicon_tags_38 },
    { 0xcdacc34a, SUC_lexicon_tags_45 },
    { 0x25ecc34a, SUC_lexicon_tags_68 },
    { 0x13ecc34c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xffccc34f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdccc352, SUC_lexicon_tags_63 },
    { 0xe6ecc352, SUC_lexicon_tags_27 },
    { 0xccecc353, SUC_lexicon_tags_9 },
    { 0x864cc353, SUC_lexicon_tags_1 },
    { 0xc0acc356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696cc35b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadccc361, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534cc365, SUC_lexicon_tags_30 },
    { 0xbb4cc366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x12ecc368, SUC_lexicon_tags_11 },
    { 0x712cc369, SUC_lexicon_tags_14 },
    { 0x44ccc36a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812cc376, SUC_lexicon_tags_1 },
    { 0x3cecc376, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606cc37c, SUC_lexicon_tags_15 },
    { 0xda4cc37c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4cc380, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2cc383, SUC_lexicon_tags_1 },
    { 0xe28cc384, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecc387, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xea6cc389, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f0cc38b, SUC_lexicon_tags_29 },
    { 0xf96cc38c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x846cc38f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ccc393, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8cc396, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6cacc398, SUC_lexicon_tags_28 },
    { 0xc68cc399, SUC_lexicon_tags_27 },
    { 0x620cc39a, SUC_lexicon_tags_25 },
    { 0x5cacc39b, SUC_lexicon_tags_1 },
    { 0x284cc39b, SUC_lexicon_tags_49 },
    { 0x418cc39d, SUC_lexicon_tags_17 },
    { 0x3f4cc39e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x592cc3a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590cc3aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff0cc3ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5eacc3ae, SUC_lexicon_tags_15 },
    { 0x4accc3ae, SUC_lexicon_tags_0 },
    { 0x858cc3b0, SUC_lexicon_tags_25 },
    { 0xe52cc3b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x512cc3b3, SUC_lexicon_tags_37 },
    { 0xd24cc3b4, SUC_lexicon_tags_30 },
    { 0xa06cc3b5, SUC_lexicon_tags_11 },
    { 0xd8ecc3b6, SUC_lexicon_tags_12 },
    { 0x204cc3b7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5b0cc3b9, SUC_lexicon_tags_11 },
    { 0xf54cc3ba, SUC_lexicon_tags_2 },
    { 0xc26cc3b9, SUC_lexicon_tags_1 },
    { 0xbbacc3ba, SUC_lexicon_tags_85 },
    { 0x20acc3bd, SUC_lexicon_tags_7 },
    { 0x4ccc3be, SUC_lexicon_tags_30 },
    { 0x1cacc3ba, SUC_lexicon_tags_29 },
    { 0xf30cc3c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3decc3c5, SUC_lexicon_tags_27 },
    { 0x23acc3c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9eecc3c8, SUC_lexicon_tags_19 },
    { 0xdcacc3c9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x29acc3cb, SUC_lexicon_tags_37 },
    { 0x9d6cc3cc, SUC_lexicon_tags_27 },
    { 0x90cc3cc, SUC_lexicon_tags_25 },
    { 0x2c8cc3ce, SUC_lexicon_tags_25 },
    { 0xe0cc3cf, SUC_lexicon_tags_25 },
    { 0x42ccc3cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83ccc3d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88acc3d4, SUC_lexicon_tags_34 },
    { 0x4c0cc3d5, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cecc3d8, SUC_lexicon_tags_79 },
    { 0x6f6cc3d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbccc3db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20cc3de, SUC_lexicon_tags_492 },
    { 0xda6cc3df, SUC_lexicon_tags_64 },
    { 0x27ecc3e0, SUC_lexicon_tags_12 },
    { 0x1a0cc3e1, SUC_lexicon_tags_17 },
    { 0xca2cc3e0, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4cc3eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x98ecc3ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9acc3f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866cc402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7a8cc404, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x608cc406, SUC_lexicon_tags_33 },
    { 0x6fecc407, SUC_lexicon_tags_14 },
    { 0x686cc408, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xef8cc40a, SUC_lexicon_tags_37 },
    { 0xb40cc40b, SUC_lexicon_tags_1 },
    { 0xaa0cc40a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ecc410, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ecc416, SUC_lexicon_tags_70 },
    { 0x1c6cc416, SUC_lexicon_tags_13 },
    { 0x226cc416, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd6cc41a, SUC_lexicon_tags_25 },
    { 0x6eacc41a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8cc41e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ccc423, SUC_lexicon_tags_12 },
    { 0x6cecc423, SUC_lexicon_tags_47 },
    { 0x9cacc424, SUC_lexicon_tags_23 },
    { 0xa2ccc426, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x408cc429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86cc42d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ccc430, SUC_lexicon_tags_37 },
    { 0x7b8cc431, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe00cc433, SUC_lexicon_tags_60 },
    { 0x726cc433, SUC_lexicon_tags_57 },
    { 0x91acc435, SUC_lexicon_tags_2 },
    { 0x200cc435, SUC_lexicon_tags_17 },
    { 0xb8acc437, SUC_lexicon_tags_1 },
    { 0x5a6cc437, SUC_lexicon_tags_30 },
    { 0xc12cc435, SUC_lexicon_tags_91 },
    { 0xa54cc433, SUC_lexicon_tags_1 },
    { 0x54ccc438, SUC_lexicon_tags_47 },
    { 0x882cc43c, SUC_lexicon_tags_15 },
    { 0x936cc43d, SUC_lexicon_tags_15 },
    { 0xc8ccc43e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cc444, SUC_lexicon_tags_70 },
    { 0x448cc444, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xceecc447, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9decc449, SUC_lexicon_tags_47 },
    { 0x6decc44a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x888cc44d, SUC_lexicon_tags_11 },
    { 0x12cc44e, SUC_lexicon_tags_7 },
    { 0x996cc44f, SUC_lexicon_tags_7 },
    { 0x256cc450, SUC_lexicon_tags_7 },
    { 0x4a2cc44f, SUC_lexicon_tags_69 },
    { 0x84ccc451, SUC_lexicon_tags_63 },
    { 0x120cc453, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2cc458, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2cc45b, SUC_lexicon_tags_55 },
    { 0x118cc45b, SUC_lexicon_tags_11 },
    { 0x7a6cc45d, SUC_lexicon_tags_23 },
    { 0xf14cc45d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9ccc460, SUC_lexicon_tags_15 },
    { 0xea0cc461, SUC_lexicon_tags_25 },
    { 0x528cc462, SUC_lexicon_tags_32 },
    { 0x2dccc462, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930cc468, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb70cc46a, SUC_lexicon_tags_17 },
    { 0x6d2cc46b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792cc46f, SUC_lexicon_tags_45 },
    { 0xdb6cc470, SUC_lexicon_tags_47 },
    { 0xd66cc471, SUC_lexicon_tags_60 },
    { 0x35ccc472, SUC_lexicon_tags_1 },
    { 0xd2acc473, SUC_lexicon_tags_234 },
    { 0x8ccc472, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898cc478, SUC_lexicon_tags_17 },
    { 0xec6cc478, SUC_lexicon_tags_39 },
    { 0x3d6cc47a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe72cc47c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6cc47f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f8cc481, SUC_lexicon_tags_43 },
    { 0x2d4cc481, SUC_lexicon_tags_65 },
    { 0x634cc483, SUC_lexicon_tags_116 },
    { 0xb7ecc484, SUC_lexicon_tags_38 },
    { 0x90cc485, SUC_lexicon_tags_2 },
    { 0xc22cc484, SUC_lexicon_tags_47 },
    { 0xc16cc487, SUC_lexicon_tags_0 },
    { 0x2acc487, SUC_lexicon_tags_12 },
    { 0xfeacc488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cecc48e, SUC_lexicon_tags_11 },
    { 0xa3acc48e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eecc493, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdacc495, SUC_lexicon_tags_15 },
    { 0x99ecc496, SUC_lexicon_tags_34 },
    { 0xa20cc496, SUC_lexicon_tags_15 },
    { 0x912cc495, SUC_lexicon_tags_12 },
    { 0xe26cc496, SUC_lexicon_tags_47 },
    { 0x38acc49a, SUC_lexicon_tags_7 },
    { 0x104cc49a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7c4cc49d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428cc4a3, SUC_lexicon_tags_15 },
    { 0x684cc4a4, SUC_lexicon_tags_63 },
    { 0x76acc4a5, SUC_lexicon_tags_17 },
    { 0xc62cc4a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4becc4a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc8cc4aa, SUC_lexicon_tags_65 },
    { 0x25acc4ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22cc4b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ecc4b6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x414cc4b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc20cc4ba, SUC_lexicon_tags_6 },
    { 0xc3ccc4ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa98cc4bd, SUC_lexicon_tags_33 },
    { 0x8c6cc4be, SUC_lexicon_tags_37 },
    { 0xb3ecc4bf, SUC_lexicon_tags_37 },
    { 0xb46cc4c0, SUC_lexicon_tags_8 },
    { 0xc56cc4be, SUC_lexicon_tags_21 },
    { 0x58acc4c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x844cc4c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cc4c7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe3ecc4c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2cc4d2, SUC_lexicon_tags_37 },
    { 0xb6ecc4d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aacc4d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdccc4d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x77ecc4da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ccc4de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbecc4e1, SUC_lexicon_tags_37 },
    { 0x3aecc4e2, SUC_lexicon_tags_0 },
    { 0xd72cc4e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c6cc4e5, SUC_lexicon_tags_104 },
    { 0x154cc4e6, SUC_lexicon_tags_11 },
    { 0x5ccc4e5, SUC_lexicon_tags_44 },
    { 0x56acc4e5, SUC_lexicon_tags_0 },
    { 0xc20cc4e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa2cc4eb, SUC_lexicon_tags_17 },
    { 0xa0acc4ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ccc4ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbeecc4f1, SUC_lexicon_tags_7 },
    { 0x6cc4f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x204cc4f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cc4f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32acc4fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86cc4fd, SUC_lexicon_tags_6 },
    { 0x59ecc4fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe06cc500, SUC_lexicon_tags_33 },
    { 0x156cc501, SUC_lexicon_tags_9 },
    { 0xa7ecc502, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2acc504, SUC_lexicon_tags_10 },
    { 0xb12cc505, SUC_lexicon_tags_55 },
    { 0x966cc504, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58cc509, SUC_lexicon_tags_25 },
    { 0x86ecc50a, SUC_lexicon_tags_47 },
    { 0x882cc50b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8cc50f, SUC_lexicon_tags_67 },
    { 0x346cc510, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7e8cc512, SUC_lexicon_tags_43 },
    { 0x5c4cc513, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6cc518, SUC_lexicon_tags_27 },
    { 0x846cc519, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccc51d, SUC_lexicon_tags_77 },
    { 0x2cacc51d, SUC_lexicon_tags_37 },
    { 0x62cc51f, SUC_lexicon_tags_275 },
    { 0xec0cc520, SUC_lexicon_tags_17 },
    { 0x6eccc521, SUC_lexicon_tags_45 },
    { 0x152cc51f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54cc525, SUC_lexicon_tags_11 },
    { 0xc46cc525, SUC_lexicon_tags_27 },
    { 0x4dacc526, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ecc52c, SUC_lexicon_tags_33 },
    { 0xc52cc52d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad6cc52f, SUC_lexicon_tags_25 },
    { 0x156cc52f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcacc532, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x50ecc534, SUC_lexicon_tags_65 },
    { 0xd6ecc535, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2cc539, SUC_lexicon_tags_60 },
    { 0xfd6cc53a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65acc542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63acc545, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc70cc547, SUC_lexicon_tags_55 },
    { 0xbaacc548, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ccc54e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6cc551, SUC_lexicon_tags_11 },
    { 0x4fecc551, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2cc55c, SUC_lexicon_tags_15 },
    { 0xdaacc55d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4eacc55f, SUC_lexicon_tags_7 },
    { 0x4f0cc55f, SUC_lexicon_tags_12 },
    { 0x518cc560, SUC_lexicon_tags_37 },
    { 0xed0cc562, SUC_lexicon_tags_6 },
    { 0x7eccc560, SUC_lexicon_tags_12 },
    { 0x240cc564, SUC_lexicon_tags_24 },
    { 0xe1acc564, SUC_lexicon_tags_6 },
    { 0xeaccc566, SUC_lexicon_tags_17 },
    { 0xe3acc567, SUC_lexicon_tags_112 },
    { 0x5f4cc564, SUC_lexicon_tags_38 },
    { 0xd6cc560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6cc56f, SUC_lexicon_tags_57 },
    { 0xf8acc570, SUC_lexicon_tags_11 },
    { 0xccecc56f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd46cc573, SUC_lexicon_tags_43 },
    { 0x634cc574, SUC_lexicon_tags_1 },
    { 0x23acc575, SUC_lexicon_tags_25 },
    { 0x660cc574, SUC_lexicon_tags_12 },
    { 0xa3ecc575, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0acc57b, SUC_lexicon_tags_29 },
    { 0x682cc57c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62acc57e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0cc583, SUC_lexicon_tags_15 },
    { 0x3fccc584, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x540cc587, SUC_lexicon_tags_27 },
    { 0xc0acc588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x608cc58a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb64cc58c, SUC_lexicon_tags_4 },
    { 0x476cc58d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762cc596, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fccc59e, SUC_lexicon_tags_12 },
    { 0x2b6cc59f, SUC_lexicon_tags_7 },
    { 0xe86cc5a0, SUC_lexicon_tags_12 },
    { 0x19acc5a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60cc5a5, SUC_lexicon_tags_82 },
    { 0xa42cc5a5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ecc5ad, SUC_lexicon_tags_37 },
    { 0xf88cc5ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ccc5b3, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ecc5b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2cc5bb, SUC_lexicon_tags_25 },
    { 0xc6cc5bc, SUC_lexicon_tags_17 },
    { 0x32ccc5bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xceacc5bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2cc5c2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ccc5c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ecc5c9, SUC_lexicon_tags_26 },
    { 0x1b4cc5ca, SUC_lexicon_tags_25 },
    { 0x9becc5ca, SUC_lexicon_tags_57 },
    { 0xd2acc5cb, SUC_lexicon_tags_29 },
    { 0x1bacc5cc, SUC_lexicon_tags_15 },
    { 0x650cc5ce, SUC_lexicon_tags_6 },
    { 0xdd4cc5cf, SUC_lexicon_tags_14 },
    { 0xc32cc5cf, SUC_lexicon_tags_49 },
    { 0x45ecc5d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffccc5d3, SUC_lexicon_tags_19 },
    { 0x614cc5d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ccc5d7, SUC_lexicon_tags_7 },
    { 0x306cc5d8, SUC_lexicon_tags_17 },
    { 0xa40cc5d8, SUC_lexicon_tags_402 },
    { 0x9c8cc5da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60cc5e2, SUC_lexicon_tags_29 },
    { 0xef4cc5e2, SUC_lexicon_tags_33 },
    { 0x82ecc5e2, SUC_lexicon_tags_3 },
    { 0x8e2cc5e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ecc5e8, SUC_lexicon_tags_37 },
    { 0x87ecc5e9, SUC_lexicon_tags_12 },
    { 0xf2ecc5ea, SUC_lexicon_tags_2 },
    { 0x490cc5ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918cc5f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660cc5f9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0cc5fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf08cc5fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3c4cc600, SUC_lexicon_tags_68 },
    { 0x4becc601, SUC_lexicon_tags_47 },
    { 0x28ccc601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ccc608, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2cc60b, SUC_lexicon_tags_27 },
    { 0xc56cc60b, SUC_lexicon_tags_37 },
    { 0x3fccc60d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xc80cc60f, SUC_lexicon_tags_55 },
    { 0xc9ccc60f, SUC_lexicon_tags_37 },
    { 0xf88cc611, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eacc617, SUC_lexicon_tags_11 },
    { 0x3f0cc618, SUC_lexicon_tags_11 },
    { 0x1d8cc619, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4c6cc61b, SUC_lexicon_tags_11 },
    { 0x5a6cc61b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86acc620, SUC_lexicon_tags_17 },
    { 0x89ecc620, SUC_lexicon_tags_15 },
    { 0xfc6cc622, SUC_lexicon_tags_46 },
    { 0xe0ccc623, SUC_lexicon_tags_37 },
    { 0x3accc623, SUC_lexicon_tags_41 },
    { 0x550cc623, SUC_lexicon_tags_12 },
    { 0x98cc625, SUC_lexicon_tags_47 },
    { 0x96cc626, SUC_lexicon_tags_23 },
    { 0x4f6cc628, SUC_lexicon_tags_87 },
    { 0x2c4cc629, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6cc62c, SUC_lexicon_tags_12 },
    { 0x436cc62d, SUC_lexicon_tags_2 },
    { 0x588cc62e, SUC_lexicon_tags_46 },
    { 0x1b4cc62f, SUC_lexicon_tags_31 },
    { 0x32acc630, SUC_lexicon_tags_11 },
    { 0x610cc62e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ccc634, SUC_lexicon_tags_2 },
    { 0xa7ccc635, SUC_lexicon_tags_2 },
    { 0xa08cc636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ecc63a, SUC_lexicon_tags_43 },
    { 0x846cc63b, SUC_lexicon_tags_66 },
    { 0xd3acc63a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4cc63f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3ecc641, SUC_lexicon_tags_15 },
    { 0x3fecc642, SUC_lexicon_tags_47 },
    { 0xe58cc643, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94cc648, SUC_lexicon_tags_29 },
    { 0x19ecc649, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812cc64e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82cc655, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x73ecc657, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x588cc65a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cc660, SUC_lexicon_tags_6 },
    { 0x738cc660, SUC_lexicon_tags_12 },
    { 0x6b0cc662, SUC_lexicon_tags_14 },
    { 0x93ecc663, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aecc666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776cc66a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc68cc66c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdecc66e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cc679, SUC_lexicon_tags_47 },
    { 0x8e8cc67a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecc67d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98cc682, SUC_lexicon_tags_29 },
    { 0xb38cc683, SUC_lexicon_tags_37 },
    { 0x682cc683, SUC_lexicon_tags_14 },
    { 0x334cc685, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf5ccc687, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ecc68c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ccc68f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328cc696, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4cc69a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230cc6a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ecc6a3, SUC_lexicon_tags_31 },
    { 0xd60cc6a4, SUC_lexicon_tags_32 },
    { 0x126cc6a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b6cc6a7, SUC_lexicon_tags_2 },
    { 0x7a4cc6a8, SUC_lexicon_tags_34 },
    { 0x9ccc6a9, SUC_lexicon_tags_9 },
    { 0xd50cc6a9, SUC_lexicon_tags_45 },
    { 0xc42cc6aa, SUC_lexicon_tags_7 },
    { 0x4f2cc6ac, SUC_lexicon_tags_15 },
    { 0x55ecc6ab, SUC_lexicon_tags_63 },
    { 0xdfacc6ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2acc6b0, SUC_lexicon_tags_12 },
    { 0x7fccc6b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4ecc6b3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78cc6b8, SUC_lexicon_tags_6 },
    { 0xfa4cc6b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde0cc6bb, SUC_lexicon_tags_2 },
    { 0xa24cc6bc, SUC_lexicon_tags_16 },
    { 0x4bacc6bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x106cc6bf, SUC_lexicon_tags_37 },
    { 0x17ecc6bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x872cc6c2, SUC_lexicon_tags_15 },
    { 0x7eacc6c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974cc6c8, SUC_lexicon_tags_27 },
    { 0xe8ecc6c8, SUC_lexicon_tags_37 },
    { 0x80acc6c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ccc6cc, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aacc6d0, SUC_lexicon_tags_6 },
    { 0xb1ecc6d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8c8cc6d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d8cc6d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a0cc6d7, SUC_lexicon_tags_47 },
    { 0xf5ccc6d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaacc6de, SUC_lexicon_tags_28 },
    { 0x9fecc6de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f2cc6e1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40cc6e5, SUC_lexicon_tags_0 },
    { 0x4acc6e6, SUC_lexicon_tags_1 },
    { 0x8c8cc6e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x50cc6e9, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x5aacc6eb, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xe3ccc6ed, SUC_lexicon_tags_1 },
    { 0x80ccc6ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14cc6f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156cc6f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x508cc6f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96cc6ff, SUC_lexicon_tags_17 },
    { 0x904cc6ff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64cc704, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2cc708, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aecc70c, SUC_lexicon_tags_25 },
    { 0x1becc70d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d6cc70f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6d6cc711, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992cc724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d8cc726, SUC_lexicon_tags_43 },
    { 0x34ecc727, SUC_lexicon_tags_70 },
    { 0xe52cc728, SUC_lexicon_tags_10 },
    { 0x6b4cc727, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x132cc72c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x28acc72f, SUC_lexicon_tags_15 },
    { 0x200cc730, SUC_lexicon_tags_47 },
    { 0x188cc731, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8acc734, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x804cc736, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82acc738, SUC_lexicon_tags_17 },
    { 0x442cc738, SUC_lexicon_tags_2 },
    { 0x8eccc739, SUC_lexicon_tags_25 },
    { 0xc4acc73a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12cc73f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfaecc741, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48cc744, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37acc749, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x350cc74b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ccc750, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8cc753, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74cc756, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf8cc758, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180cc75c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ecc75f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aacc764, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9e0cc766, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8cc769, SUC_lexicon_tags_37 },
    { 0x8c8cc769, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xacccc76d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24cc771, SUC_lexicon_tags_84 },
    { 0xdf8cc772, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x976cc774, SUC_lexicon_tags_32 },
    { 0xb0cc775, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e6cc777, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8cc77b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ecc77f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8cc785, SUC_lexicon_tags_17 },
    { 0x266cc786, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98cc78a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xccccc78d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cc794, SUC_lexicon_tags_27 },
    { 0x506cc795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16acc798, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250cc79d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40cc7a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecc7a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838cc7a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ecc7ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52cc7b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acc7b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x48acc7b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2cc7bb, SUC_lexicon_tags_70 },
    { 0xbf4cc7bc, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x544cc7be, SUC_lexicon_tags_2 },
    { 0xd96cc7bf, SUC_lexicon_tags_17 },
    { 0xd6ecc7c0, SUC_lexicon_tags_7 },
    { 0xb48cc7c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ccc7c4, SUC_lexicon_tags_14 },
    { 0x256cc7c5, SUC_lexicon_tags_27 },
    { 0xf24cc7c6, SUC_lexicon_tags_49 },
    { 0x30cc7c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896cc7cd, SUC_lexicon_tags_2 },
    { 0x370cc7ce, SUC_lexicon_tags_291 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bccc7d2, SUC_lexicon_tags_40 },
    { 0xd26cc7d3, SUC_lexicon_tags_1 },
    { 0x928cc7d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98acc7da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8cc7e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228cc7e9, SUC_lexicon_tags_9 },
    { 0x828cc7ea, SUC_lexicon_tags_60 },
    { 0x82cc7eb, SUC_lexicon_tags_30 },
    { 0x1fccc7ea, SUC_lexicon_tags_26 },
    { 0xf14cc7eb, SUC_lexicon_tags_27 },
    { 0x536cc7eb, SUC_lexicon_tags_47 },
    { 0x37acc7ef, SUC_lexicon_tags_11 },
    { 0x222cc7ed, SUC_lexicon_tags_19 },
    { 0xc2ccc7ed, SUC_lexicon_tags_12 },
    { 0xe48cc7f2, SUC_lexicon_tags_17 },
    { 0x302cc7f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33acc7fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x812cc7fd, SUC_lexicon_tags_14 },
    { 0x9b8cc7fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeacc801, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06cc804, SUC_lexicon_tags_1 },
    { 0x23ccc805, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74acc807, SUC_lexicon_tags_19 },
    { 0x48ecc808, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00cc80c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6cc811, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0cc814, SUC_lexicon_tags_77 },
    { 0xe2ccc815, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x882cc817, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84cc81a, SUC_lexicon_tags_47 },
    { 0xc70cc81b, SUC_lexicon_tags_0 },
    { 0xc2cc81c, SUC_lexicon_tags_25 },
    { 0x48acc81a, SUC_lexicon_tags_33 },
    { 0x698cc81e, SUC_lexicon_tags_148 },
    { 0x46cc81f, SUC_lexicon_tags_38 },
    { 0x530cc820, SUC_lexicon_tags_34 },
    { 0xf74cc821, SUC_lexicon_tags_33 },
    { 0x5c0cc81e, SUC_lexicon_tags_2 },
    { 0x43ccc823, SUC_lexicon_tags_11 },
    { 0x49ccc823, SUC_lexicon_tags_25 },
    { 0x7e6cc820, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0cc828, SUC_lexicon_tags_19 },
    { 0xf80cc829, SUC_lexicon_tags_11 },
    { 0xfd6cc829, SUC_lexicon_tags_17 },
    { 0x8accc82b, SUC_lexicon_tags_32 },
    { 0x114cc82c, SUC_lexicon_tags_25 },
    { 0xb2ccc82d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdfacc82f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2cc835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946cc839, SUC_lexicon_tags_26 },
    { 0xf30cc839, SUC_lexicon_tags_49 },
    { 0x4b0cc83b, SUC_lexicon_tags_27 },
    { 0x8f2cc839, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cacc844, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2cc84b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8cc84f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82cc855, SUC_lexicon_tags_28 },
    { 0xef8cc855, SUC_lexicon_tags_12 },
    { 0xc3acc856, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67acc85a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492cc85f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54cc864, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52acc86e, SUC_lexicon_tags_12 },
    { 0xbe8cc86e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x250cc871, SUC_lexicon_tags_22 },
    { 0x91ecc872, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ccc87a, SUC_lexicon_tags_17 },
    { 0xc8acc87a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32cc87d, SUC_lexicon_tags_38 },
    { 0x518cc87e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884cc883, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ccc886, SUC_lexicon_tags_12 },
    { 0x15ecc887, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52cc88a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xea4cc88c, SUC_lexicon_tags_198 },
    { 0x264cc88d, SUC_lexicon_tags_16 },
    { 0xb30cc88d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xec6cc891, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x484cc893, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178cc897, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58ecc899, SUC_lexicon_tags_37 },
    { 0x926cc899, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2cc89e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934cc8a3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fccc8a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cc8ad, SUC_lexicon_tags_1 },
    { 0x4ccc8ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36cc8b1, SUC_lexicon_tags_1 },
    { 0xcc2cc8b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x158cc8b5, SUC_lexicon_tags_25 },
    { 0x7fecc8b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x136cc8b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852cc8bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb48cc8bf, SUC_lexicon_tags_6 },
    { 0xb88cc8bf, SUC_lexicon_tags_25 },
    { 0x5eccc8c1, SUC_lexicon_tags_27 },
    { 0x546cc8c1, SUC_lexicon_tags_1 },
    { 0x3dccc8c0, SUC_lexicon_tags_7 },
    { 0x79ccc8c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b0cc8c6, SUC_lexicon_tags_37 },
    { 0xd58cc8c7, SUC_lexicon_tags_15 },
    { 0xd34cc8c8, SUC_lexicon_tags_14 },
    { 0xd28cc8c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4cc8d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fecc8d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62ecc8d6, SUC_lexicon_tags_2 },
    { 0x180cc8d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ecc8de, SUC_lexicon_tags_32 },
    { 0x6c8cc8de, SUC_lexicon_tags_25 },
    { 0x530cc8e0, SUC_lexicon_tags_35 },
    { 0xec8cc8de, SUC_lexicon_tags_37 },
    { 0x8accc8e2, SUC_lexicon_tags_47 },
    { 0x2a2cc8e3, SUC_lexicon_tags_12 },
    { 0x57acc8e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8cc8e7, SUC_lexicon_tags_37 },
    { 0x1d4cc8e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa10cc8ea, SUC_lexicon_tags_47 },
    { 0xc5ecc8eb, SUC_lexicon_tags_42 },
    { 0xda6cc8ec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0cc8f2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x962cc8f4, SUC_lexicon_tags_1 },
    { 0x2cacc8f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x322cc8f7, SUC_lexicon_tags_63 },
    { 0xad4cc8f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e4cc8fa, SUC_lexicon_tags_13 },
    { 0x5b6cc8fa, SUC_lexicon_tags_11 },
    { 0x42ecc8fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e8cc8fe, SUC_lexicon_tags_15 },
    { 0x426cc8ff, SUC_lexicon_tags_85 },
    { 0xd1ecc900, SUC_lexicon_tags_29 },
    { 0x4acc901, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72cc907, SUC_lexicon_tags_9 },
    { 0x320cc907, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00cc90d, SUC_lexicon_tags_12 },
    { 0xd46cc90e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb6cc910, SUC_lexicon_tags_63 },
    { 0xe9acc910, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb12cc914, SUC_lexicon_tags_11 },
    { 0x2eecc915, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x290cc918, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb18cc91a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc3acc91c, SUC_lexicon_tags_15 },
    { 0x68acc91d, SUC_lexicon_tags_11 },
    { 0xa32cc91d, SUC_lexicon_tags_2 },
    { 0xf12cc91c, SUC_lexicon_tags_17 },
    { 0xf34cc920, SUC_lexicon_tags_30 },
    { 0x51ecc920, SUC_lexicon_tags_31 },
    { 0x4b6cc921, SUC_lexicon_tags_37 },
    { 0x7bccc920, SUC_lexicon_tags_11 },
    { 0xd4acc91d, SUC_lexicon_tags_14 },
    { 0x646cc923, SUC_lexicon_tags_25 },
    { 0xfc0cc91d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb44cc928, SUC_lexicon_tags_2 },
    { 0x12ecc929, SUC_lexicon_tags_75 },
    { 0x2b4cc929, SUC_lexicon_tags_17 },
    { 0xbfecc92b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cc92e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x718cc930, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ccc934, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4cc938, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0cc93b, SUC_lexicon_tags_19 },
    { 0x814cc93c, SUC_lexicon_tags_29 },
    { 0x5c4cc93b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45acc942, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78cc947, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1bacc949, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdeecc94b, SUC_lexicon_tags_31 },
    { 0xeeecc94b, SUC_lexicon_tags_2 },
    { 0xfecc94c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58cc952, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdeacc954, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e6cc956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10cc959, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682cc95d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2cc962, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50cc965, SUC_lexicon_tags_62 },
    { 0xde4cc965, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66cc969, SUC_lexicon_tags_37 },
    { 0xdeccc96a, SUC_lexicon_tags_33 },
    { 0x4f2cc96b, SUC_lexicon_tags_7 },
    { 0x90ccc96c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558cc971, SUC_lexicon_tags_146 },
    { 0x4b2cc972, SUC_lexicon_tags_26 },
    { 0x7e8cc972, SUC_lexicon_tags_63 },
    { 0x6becc971, SUC_lexicon_tags_11 },
    { 0xedccc975, SUC_lexicon_tags_7 },
    { 0x5b8cc972, SUC_lexicon_tags_15 },
    { 0xbcccc977, SUC_lexicon_tags_45 },
    { 0x39acc975, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ecc97c, SUC_lexicon_tags_25 },
    { 0x3e6cc97d, SUC_lexicon_tags_37 },
    { 0xa8ecc97e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fccc982, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ccc985, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb8cc987, SUC_lexicon_tags_53 },
    { 0xb10cc988, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x582cc98a, SUC_lexicon_tags_16 },
    { 0xc72cc98b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x446cc98d, SUC_lexicon_tags_29 },
    { 0x514cc98e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8cc993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ecc997, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cc99c, SUC_lexicon_tags_29 },
    { 0xa5ccc99c, SUC_lexicon_tags_37 },
    { 0xab6cc99d, SUC_lexicon_tags_23 },
    { 0xcb4cc99f, SUC_lexicon_tags_11 },
    { 0x1a2cc9a0, SUC_lexicon_tags_37 },
    { 0xe8acc9a1, SUC_lexicon_tags_8 },
    { 0xb00cc99d, SUC_lexicon_tags_17 },
    { 0x53ecc9a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4cc9a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302cc9aa, SUC_lexicon_tags_166 },
    { 0x528cc9ab, SUC_lexicon_tags_27 },
    { 0xdc6cc9ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944cc9b2, SUC_lexicon_tags_1 },
    { 0x3dccc9b2, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x5a8cc9b5, SUC_lexicon_tags_19 },
    { 0x658cc9b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844cc9ba, SUC_lexicon_tags_32 },
    { 0xdd0cc9bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82cc9c0, SUC_lexicon_tags_11 },
    { 0xfdccc9c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x94ecc9c3, SUC_lexicon_tags_39 },
    { 0x1a4cc9c3, SUC_lexicon_tags_60 },
    { 0xcfacc9c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ccc9c9, SUC_lexicon_tags_17 },
    { 0x1cecc9ca, SUC_lexicon_tags_2 },
    { 0x6d8cc9cb, SUC_lexicon_tags_19 },
    { 0xaeccc9cc, SUC_lexicon_tags_5 },
    { 0xcfecc9cd, SUC_lexicon_tags_15 },
    { 0x814cc9ce, SUC_lexicon_tags_55 },
    { 0xb66cc9cf, SUC_lexicon_tags_65 },
    { 0x29ecc9d0, SUC_lexicon_tags_94 },
    { 0x19ccc9ca, SUC_lexicon_tags_12 },
    { 0x438cc9ca, SUC_lexicon_tags_2 },
    { 0x706cc9c9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xfcacc9d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadacc9da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c4cc9dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cc9e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12cc9e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71ecc9ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ccc9f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06cc9f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58acc9f9, SUC_lexicon_tags_17 },
    { 0x946cc9f9, SUC_lexicon_tags_1 },
    { 0xa86cc9fb, SUC_lexicon_tags_25 },
    { 0x5fecc9fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2ccc9fe, SUC_lexicon_tags_1 },
    { 0xab6cc9ff, SUC_lexicon_tags_37 },
    { 0x23ccc9fe, SUC_lexicon_tags_201 },
    { 0x9e4cca01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf6cca03, SUC_lexicon_tags_15 },
    { 0x5d4cca04, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xad8cca06, SUC_lexicon_tags_14 },
    { 0xb5ccca07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8cca0a, SUC_lexicon_tags_11 },
    { 0x690cca0b, SUC_lexicon_tags_37 },
    { 0x2accca0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cca10, SUC_lexicon_tags_25 },
    { 0x7cecca11, SUC_lexicon_tags_47 },
    { 0x942cca10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fccca15, SUC_lexicon_tags_15 },
    { 0x1e8cca15, SUC_lexicon_tags_82 },
    { 0xe8cca17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x16ecca19, SUC_lexicon_tags_15 },
    { 0x10ccca1a, SUC_lexicon_tags_37 },
    { 0x5cecca1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39ccca1d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ecca22, SUC_lexicon_tags_1315 },
    { 0x94ecca23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25acca29, SUC_lexicon_tags_37 },
    { 0xed6cca2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6c8cca2c, SUC_lexicon_tags_8 },
    { 0xe1ecca2d, SUC_lexicon_tags_5 },
    { 0xbaccca2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12cca32, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf60cca34, SUC_lexicon_tags_14 },
    { 0xe1acca35, SUC_lexicon_tags_14 },
    { 0xf12cca36, SUC_lexicon_tags_1 },
    { 0x2ecca35, SUC_lexicon_tags_26 },
    { 0xdc8cca37, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xccecca3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42ecca3c, SUC_lexicon_tags_12 },
    { 0xe54cca3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4cca40, SUC_lexicon_tags_19 },
    { 0xc50cca40, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x358cca43, SUC_lexicon_tags_17 },
    { 0x210cca44, SUC_lexicon_tags_37 },
    { 0x67ecca45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2cca48, SUC_lexicon_tags_17 },
    { 0x6bccca48, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x60acca4c, SUC_lexicon_tags_12 },
    { 0x4d6cca4d, SUC_lexicon_tags_47 },
    { 0xa76cca4e, SUC_lexicon_tags_11 },
    { 0xbeccca4e, SUC_lexicon_tags_29 },
    { 0x62ecca4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf10cca52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadecca54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6cca57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b6cca59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x114cca5c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb10cca5e, SUC_lexicon_tags_8 },
    { 0x8bccca5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4acca62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0cca67, SUC_lexicon_tags_27 },
    { 0x71acca68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x486cca6a, SUC_lexicon_tags_2 },
    { 0xbb4cca6b, SUC_lexicon_tags_1 },
    { 0x404cca6b, SUC_lexicon_tags_11 },
    { 0x548cca6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29ccca6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25ccca71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810cca77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f6cca79, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2cca7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x590cca80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4cca85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2cccca87, SUC_lexicon_tags_7 },
    { 0x73ecca88, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6cca8c, SUC_lexicon_tags_19 },
    { 0x5fccca8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94cca95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4cca98, SUC_lexicon_tags_47 },
    { 0xf66cca99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2cca9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef2cca9e, SUC_lexicon_tags_25 },
    { 0xb2acca9f, SUC_lexicon_tags_23 },
    { 0x21accaa0, SUC_lexicon_tags_0 },
    { 0x25accaa1, SUC_lexicon_tags_37 },
    { 0xc4eccaa2, SUC_lexicon_tags_3 },
    { 0x888ccaa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8cccaa5, SUC_lexicon_tags_15 },
    { 0xa70ccaa6, SUC_lexicon_tags_2 },
    { 0x520ccaa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44ccaaa, SUC_lexicon_tags_17 },
    { 0x332ccaab, SUC_lexicon_tags_7 },
    { 0x234ccaab, SUC_lexicon_tags_1 },
    { 0x41accaad, SUC_lexicon_tags_61 },
    { 0x82accaae, SUC_lexicon_tags_7 },
    { 0x986ccaaf, SUC_lexicon_tags_43 },
    { 0xe28ccab0, SUC_lexicon_tags_17 },
    { 0x8e0ccab0, SUC_lexicon_tags_7 },
    { 0xd5cccab2, SUC_lexicon_tags_77 },
    { 0x3d0ccaaf, SUC_lexicon_tags_29 },
    { 0x420ccab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcecccab6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976ccaba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eccac0, SUC_lexicon_tags_59 },
    { 0xfa8ccac1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x778ccac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97eccac5, SUC_lexicon_tags_11 },
    { 0xeeeccac5, SUC_lexicon_tags_4 },
    { 0xefeccac7, SUC_lexicon_tags_12 },
    { 0x7d8ccac5, SUC_lexicon_tags_19 },
    { 0x1accac9, SUC_lexicon_tags_2 },
    { 0x6d8ccaca, SUC_lexicon_tags_33 },
    { 0x132ccacb, SUC_lexicon_tags_16 },
    { 0x864ccacb, SUC_lexicon_tags_7 },
    { 0xd22ccaca, SUC_lexicon_tags_2 },
    { 0xb86ccac5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaf0ccad0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacccadc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838ccae0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886ccae6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8ccaea, SUC_lexicon_tags_29 },
    { 0xc9eccaea, SUC_lexicon_tags_68 },
    { 0x902ccaea, SUC_lexicon_tags_16 },
    { 0x9feccaec, SUC_lexicon_tags_37 },
    { 0xb48ccaee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0ccaf3, SUC_lexicon_tags_31 },
    { 0x19eccaf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02ccaf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd32ccafa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6cccafc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16ccaff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10accb05, SUC_lexicon_tags_10 },
    { 0x888ccb05, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02ccb0c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54ccb0f, SUC_lexicon_tags_37 },
    { 0x8acccb10, SUC_lexicon_tags_37 },
    { 0xe6ccb11, SUC_lexicon_tags_43 },
    { 0x31eccb11, SUC_lexicon_tags_113 },
    { 0x684ccb13, SUC_lexicon_tags_17 },
    { 0xd72ccb11, SUC_lexicon_tags_42 },
    { 0x6a6ccb15, SUC_lexicon_tags_37 },
    { 0x7a8ccb16, SUC_lexicon_tags_46 },
    { 0x81cccb15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a8ccb19, SUC_lexicon_tags_4 },
    { 0xa12ccb1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830ccb1e, SUC_lexicon_tags_14 },
    { 0xb0accb1f, SUC_lexicon_tags_13 },
    { 0xe6eccb1f, SUC_lexicon_tags_30 },
    { 0xea2ccb21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fcccb23, SUC_lexicon_tags_17 },
    { 0x882ccb24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaccb28, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4ccb2b, SUC_lexicon_tags_11 },
    { 0x57accb2c, SUC_lexicon_tags_2 },
    { 0x1d4ccb2c, SUC_lexicon_tags_37 },
    { 0x360ccb2b, SUC_lexicon_tags_15 },
    { 0xe22ccb2f, SUC_lexicon_tags_25 },
    { 0x89cccb30, SUC_lexicon_tags_43 },
    { 0x22cccb2f, SUC_lexicon_tags_43 },
    { 0x8c2ccb2b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ccb35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12ccb3a, SUC_lexicon_tags_38 },
    { 0xd50ccb3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48ccb3e, SUC_lexicon_tags_1 },
    { 0xd7cccb3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67accb43, SUC_lexicon_tags_7 },
    { 0x6beccb44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eccb47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccb4a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720ccb50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x482ccb52, SUC_lexicon_tags_27 },
    { 0x74cccb53, SUC_lexicon_tags_17 },
    { 0x74eccb54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8ccb5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32ccb60, SUC_lexicon_tags_84 },
    { 0x9e6ccb60, SUC_lexicon_tags_11 },
    { 0xd6ccb61, SUC_lexicon_tags_14 },
    { 0xc1eccb63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698ccb67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82ccb6d, SUC_lexicon_tags_25 },
    { 0xbdeccb6e, SUC_lexicon_tags_27 },
    { 0x5a0ccb6f, SUC_lexicon_tags_63 },
    { 0xce2ccb6d, SUC_lexicon_tags_0 },
    { 0xfb4ccb71, SUC_lexicon_tags_38 },
    { 0xf16ccb6d, SUC_lexicon_tags_49 },
    { 0x56accb73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4f4ccb75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x520ccb78, SUC_lexicon_tags_32 },
    { 0x12eccb78, SUC_lexicon_tags_27 },
    { 0xf06ccb78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaccb80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4feccb82, SUC_lexicon_tags_27 },
    { 0xf0eccb82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eccb89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8ccb90, SUC_lexicon_tags_26 },
    { 0x886ccb91, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0ccb98, SUC_lexicon_tags_33 },
    { 0xca4ccb99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2ccb9d, SUC_lexicon_tags_2 },
    { 0x2eeccb9e, SUC_lexicon_tags_17 },
    { 0x80ccb9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaccba2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28accba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3acccba8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfe8ccbaa, SUC_lexicon_tags_43 },
    { 0x9a2ccbaa, SUC_lexicon_tags_4 },
    { 0x5daccbaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89eccbae, SUC_lexicon_tags_23 },
    { 0x2deccbaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf10ccbb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0ccbb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cccbb8, SUC_lexicon_tags_1 },
    { 0x6e4ccbb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0ccbbc, SUC_lexicon_tags_37 },
    { 0x802ccbbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78ccbc1, SUC_lexicon_tags_7 },
    { 0xc4ccbc2, SUC_lexicon_tags_31 },
    { 0xfeaccbc2, SUC_lexicon_tags_17 },
    { 0x1acccbc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02ccbc8, SUC_lexicon_tags_6 },
    { 0x3caccbc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83accbd3, SUC_lexicon_tags_11 },
    { 0x2bcccbd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdbeccbd6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd32ccbd8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0ccbdd, SUC_lexicon_tags_43 },
    { 0xb28ccbdd, SUC_lexicon_tags_4 },
    { 0x69eccbde, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9f8ccbe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770ccbe9, SUC_lexicon_tags_17 },
    { 0x698ccbea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eccbef, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0ccbf4, SUC_lexicon_tags_2 },
    { 0xd0ccbf5, SUC_lexicon_tags_151 },
    { 0x1b8ccbf6, SUC_lexicon_tags_15 },
    { 0x2a6ccbf7, SUC_lexicon_tags_7 },
    { 0x7c4ccbf7, SUC_lexicon_tags_26 },
    { 0x1cccbf8, SUC_lexicon_tags_23 },
    { 0x778ccbf7, SUC_lexicon_tags_63 },
    { 0x462ccbfb, SUC_lexicon_tags_23 },
    { 0x3b6ccbf7, SUC_lexicon_tags_63 },
    { 0x82eccbf7, SUC_lexicon_tags_29 },
    { 0xa40ccbfe, SUC_lexicon_tags_14 },
    { 0xfc0ccbf7, SUC_lexicon_tags_19 },
    { 0x2a8ccbfc, SUC_lexicon_tags_7 },
    { 0xd8eccc01, SUC_lexicon_tags_49 },
    { 0x2d4ccc01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb26ccc04, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceccc08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5feccc0a, SUC_lexicon_tags_0 },
    { 0x7b8ccc0b, SUC_lexicon_tags_37 },
    { 0xa6ccc0a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70ccc10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40ccc13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2ccc17, SUC_lexicon_tags_37 },
    { 0xfdaccc18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x306ccc1b, SUC_lexicon_tags_38 },
    { 0xc9eccc1c, SUC_lexicon_tags_65 },
    { 0x5acccc1b, SUC_lexicon_tags_55 },
    { 0x4c6ccc1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6ccc21, SUC_lexicon_tags_63 },
    { 0x9eaccc22, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x42cccc24, SUC_lexicon_tags_32 },
    { 0x172ccc25, SUC_lexicon_tags_37 },
    { 0x4daccc26, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cccc2f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3accc35, SUC_lexicon_tags_17 },
    { 0x576ccc36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ceccc38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9eccc3b, SUC_lexicon_tags_35 },
    { 0x28cccc3c, SUC_lexicon_tags_11 },
    { 0x3d8ccc3b, SUC_lexicon_tags_107 },
    { 0xb2eccc3e, SUC_lexicon_tags_37 },
    { 0xb2ccc3d, SUC_lexicon_tags_37 },
    { 0x3beccc40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08ccc46, SUC_lexicon_tags_7 },
    { 0x19cccc46, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfe0ccc49, SUC_lexicon_tags_55 },
    { 0xf4eccc4a, SUC_lexicon_tags_39 },
    { 0xc6eccc4b, SUC_lexicon_tags_37 },
    { 0x57eccc4b, SUC_lexicon_tags_45 },
    { 0xc44ccc4d, SUC_lexicon_tags_23 },
    { 0xf82ccc4b, SUC_lexicon_tags_82 },
    { 0x712ccc4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x774ccc52, SUC_lexicon_tags_65 },
    { 0x734ccc52, SUC_lexicon_tags_2 },
    { 0xe46ccc54, SUC_lexicon_tags_7 },
    { 0xe6cccc54, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x272ccc57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8acccc5a, SUC_lexicon_tags_17 },
    { 0x6e6ccc5b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfb2ccc5d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa68ccc5f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cccc65, SUC_lexicon_tags_37 },
    { 0x464ccc65, SUC_lexicon_tags_49 },
    { 0xf28ccc66, SUC_lexicon_tags_1 },
    { 0xbbeccc66, SUC_lexicon_tags_45 },
    { 0x9b8ccc65, SUC_lexicon_tags_14 },
    { 0x49accc6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22cccc6c, SUC_lexicon_tags_2 },
    { 0x1eeccc6c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8d4ccc6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x540ccc72, SUC_lexicon_tags_11 },
    { 0x882ccc73, SUC_lexicon_tags_477 },
    { 0xef2ccc72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8ccc77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538ccc7b, SUC_lexicon_tags_31 },
    { 0x9acccc7b, SUC_lexicon_tags_18 },
    { 0xe1eccc7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1fcccc7f, SUC_lexicon_tags_0 },
    { 0x1c4ccc80, SUC_lexicon_tags_27 },
    { 0xd00ccc81, SUC_lexicon_tags_33 },
    { 0x78accc82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96ccc85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa46ccc87, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeccc8a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcccc8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324ccc91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecaccc94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bcccc96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeccc9c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x312ccc9f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x4a8ccca1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50ccca7, SUC_lexicon_tags_37 },
    { 0x776ccca8, SUC_lexicon_tags_43 },
    { 0xea4ccca9, SUC_lexicon_tags_15 },
    { 0xa24ccca8, SUC_lexicon_tags_49 },
    { 0x534cccab, SUC_lexicon_tags_60 },
    { 0xcb4cccac, SUC_lexicon_tags_15 },
    { 0xee8cccaa, SUC_lexicon_tags_2 },
    { 0x56ecccae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeeccccb0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe42cccb2, SUC_lexicon_tags_14 },
    { 0x55ecccb3, SUC_lexicon_tags_66 },
    { 0xcecccb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x996cccb7, SUC_lexicon_tags_19 },
    { 0x136cccb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0cccbd, SUC_lexicon_tags_14 },
    { 0x380cccbe, SUC_lexicon_tags_0 },
    { 0xfe2cccbf, SUC_lexicon_tags_14 },
    { 0x24ccccbf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x516cccc3, SUC_lexicon_tags_56 },
    { 0xa52cccc4, SUC_lexicon_tags_337 },
    { 0x5f2cccc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0cccc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2cccce, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xd96cccd0, SUC_lexicon_tags_17 },
    { 0xd32cccd1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x73ccccd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752cccd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eecccdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac6cccdd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0ccce1, SUC_lexicon_tags_2 },
    { 0xa40ccce2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80ccce9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70cccec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x572cccee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d0cccf0, SUC_lexicon_tags_11 },
    { 0x9d6cccf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4cccf8, SUC_lexicon_tags_2 },
    { 0x700cccf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x882cccfc, SUC_lexicon_tags_62 },
    { 0x16ccccfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd12cccff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372ccd06, SUC_lexicon_tags_12 },
    { 0xc2cccd07, SUC_lexicon_tags_37 },
    { 0x12ccd08, SUC_lexicon_tags_289 },
    { 0x9b0ccd08, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0x664ccd0b, SUC_lexicon_tags_48 },
    { 0xc30ccd0c, SUC_lexicon_tags_40 },
    { 0x7ecccd0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f6ccd0f, SUC_lexicon_tags_6 },
    { 0x790ccd10, SUC_lexicon_tags_2 },
    { 0x886ccd11, SUC_lexicon_tags_0 },
    { 0x1c8ccd12, SUC_lexicon_tags_37 },
    { 0xc18ccd13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe1eccd15, SUC_lexicon_tags_11 },
    { 0x674ccd16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9acccd18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8faccd1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2ccd1d, SUC_lexicon_tags_32 },
    { 0x9a0ccd1d, SUC_lexicon_tags_23 },
    { 0x6c8ccd1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cccd22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdeccd24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x412ccd27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10ccd2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54accd32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc80ccd34, SUC_lexicon_tags_17 },
    { 0xbf2ccd35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce2ccd37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x310ccd39, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4ccd3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492ccd40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd12ccd42, SUC_lexicon_tags_29 },
    { 0x730ccd43, SUC_lexicon_tags_3 },
    { 0xce2ccd43, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490ccd49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2baccd4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20accd51, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316ccd55, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbbeccd57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x694ccd5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54ccd5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80ccd62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58ccd65, SUC_lexicon_tags_15 },
    { 0x916ccd65, SUC_lexicon_tags_47 },
    { 0x5a4ccd66, SUC_lexicon_tags_12 },
    { 0xbc6ccd68, SUC_lexicon_tags_37 },
    { 0x9acccd69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa66ccd6b, SUC_lexicon_tags_11 },
    { 0x628ccd6c, SUC_lexicon_tags_6 },
    { 0xd54ccd6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daccd71, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8caccd73, SUC_lexicon_tags_15 },
    { 0x85cccd74, SUC_lexicon_tags_15 },
    { 0x856ccd75, SUC_lexicon_tags_17 },
    { 0x28ccd75, SUC_lexicon_tags_13 },
    { 0x3eeccd75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97accd7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x756ccd7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaccd81, SUC_lexicon_tags_23 },
    { 0xf00ccd81, SUC_lexicon_tags_2 },
    { 0x880ccd83, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902ccd89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xacaccd8b, SUC_lexicon_tags_2 },
    { 0xfdeccd8c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x272ccd8e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x83cccd91, SUC_lexicon_tags_3 },
    { 0x9a4ccd91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x966ccd94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31accd9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3aeccd9d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdc2ccd9f, SUC_lexicon_tags_14 },
    { 0x3f4ccda0, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56ccda4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692ccdab, SUC_lexicon_tags_31 },
    { 0xf12ccdab, SUC_lexicon_tags_29 },
    { 0x2ccdab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x762ccdb0, SUC_lexicon_tags_30 },
    { 0xd18ccdb1, SUC_lexicon_tags_23 },
    { 0x458ccdb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x704ccdb5, SUC_lexicon_tags_30 },
    { 0x222ccdb6, SUC_lexicon_tags_2 },
    { 0xf4accdb6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8ccdc0, SUC_lexicon_tags_63 },
    { 0xce4ccdc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaccdc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x452ccdc6, SUC_lexicon_tags_25 },
    { 0xb74ccdc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a0ccdc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faccdcf, SUC_lexicon_tags_16 },
    { 0x3f8ccdd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48eccdd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98ccddb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6ccde2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94ccdeb, SUC_lexicon_tags_12 },
    { 0x244ccdeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x96cccdef, SUC_lexicon_tags_7 },
    { 0xcb4ccdf0, SUC_lexicon_tags_2 },
    { 0x868ccdf0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22ccdf7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x24accdfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77cccdfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8cce06, SUC_lexicon_tags_85 },
    { 0xd94cce07, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb5acce09, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782cce0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768cce12, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2d2cce14, SUC_lexicon_tags_198 },
    { 0x9f2cce14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9ccce17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x918cce19, SUC_lexicon_tags_17 },
    { 0x922cce1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36acce1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbecce1f, SUC_lexicon_tags_14 },
    { 0x94acce20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x686cce23, SUC_lexicon_tags_15 },
    { 0x13acce24, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aecce2b, SUC_lexicon_tags_68 },
    { 0xa00cce2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6cce31, SUC_lexicon_tags_0 },
    { 0x6facce32, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74cce39, SUC_lexicon_tags_11 },
    { 0x258cce3a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7eacce3c, SUC_lexicon_tags_0 },
    { 0xcb0cce3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa2cce3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ecce42, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x752cce45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0acce48, SUC_lexicon_tags_2 },
    { 0xf2acce49, SUC_lexicon_tags_17 },
    { 0x528cce4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ccce50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6cce56, SUC_lexicon_tags_12 },
    { 0x1dccce57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ccce59, SUC_lexicon_tags_47 },
    { 0xd3ecce5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692cce5f, SUC_lexicon_tags_26 },
    { 0xe28cce60, SUC_lexicon_tags_8 },
    { 0x1f2cce5f, SUC_lexicon_tags_2 },
    { 0xba0cce62, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe54cce64, SUC_lexicon_tags_49 },
    { 0xd8ecce64, SUC_lexicon_tags_10 },
    { 0xb4cce66, SUC_lexicon_tags_14 },
    { 0x2b2cce67, SUC_lexicon_tags_27 },
    { 0x536cce68, SUC_lexicon_tags_25 },
    { 0x754cce68, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf6cce6b, SUC_lexicon_tags_3 },
    { 0x134cce6c, SUC_lexicon_tags_11 },
    { 0x602cce6c, SUC_lexicon_tags_933 },
    { 0xb22cce6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c0cce70, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xf56cce72, SUC_lexicon_tags_25 },
    { 0xacecce73, SUC_lexicon_tags_15 },
    { 0x456cce74, SUC_lexicon_tags_1 },
    { 0x7b8cce75, SUC_lexicon_tags_197 },
    { 0xd3ccce73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08cce7b, SUC_lexicon_tags_7 },
    { 0x790cce7c, SUC_lexicon_tags_12 },
    { 0x578cce7d, SUC_lexicon_tags_26 },
    { 0x2d0cce7e, SUC_lexicon_tags_46 },
    { 0x358cce7f, SUC_lexicon_tags_12 },
    { 0x576cce80, SUC_lexicon_tags_28 },
    { 0x316cce7d, SUC_lexicon_tags_57 },
    { 0x56ecce82, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb4ecce84, SUC_lexicon_tags_61 },
    { 0x3e6cce84, SUC_lexicon_tags_25 },
    { 0x928cce86, SUC_lexicon_tags_12 },
    { 0x39ccce87, SUC_lexicon_tags_33 },
    { 0x60acce86, SUC_lexicon_tags_23 },
    { 0x9b2cce85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00cce8f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0cce92, SUC_lexicon_tags_2 },
    { 0xb04cce93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718cce9b, SUC_lexicon_tags_55 },
    { 0x9c4cce9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x124cce9e, SUC_lexicon_tags_21 },
    { 0x63acce9f, SUC_lexicon_tags_17 },
    { 0xbc4cce9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccaccea4, SUC_lexicon_tags_23 },
    { 0x292ccea5, SUC_lexicon_tags_49 },
    { 0x354ccea6, SUC_lexicon_tags_38 },
    { 0xcb4ccea5, SUC_lexicon_tags_25 },
    { 0x20cccea7, SUC_lexicon_tags_0 },
    { 0x3d4ccea9, SUC_lexicon_tags_68 },
    { 0xec4cceaa, SUC_lexicon_tags_25 },
    { 0xb14cceaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2cceae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e0cceb0, SUC_lexicon_tags_37 },
    { 0x430cceb0, SUC_lexicon_tags_56 },
    { 0x72cceb2, SUC_lexicon_tags_18 },
    { 0x32ccceb3, SUC_lexicon_tags_11 },
    { 0xda0cceb4, SUC_lexicon_tags_15 },
    { 0x44acceb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x572cceb7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbccceb9, SUC_lexicon_tags_21 },
    { 0x346cceba, SUC_lexicon_tags_15 },
    { 0x430cceb9, SUC_lexicon_tags_23 },
    { 0x82cccebb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x904ccebf, SUC_lexicon_tags_45 },
    { 0xa2cccec0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x678ccec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84ccecc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x812ccece, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40cced4, SUC_lexicon_tags_37 },
    { 0x1a2cced5, SUC_lexicon_tags_21 },
    { 0x47acced5, SUC_lexicon_tags_63 },
    { 0xc5acced7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82eccedc, SUC_lexicon_tags_37 },
    { 0xb8cccedd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccaccee2, SUC_lexicon_tags_77 },
    { 0x4a2ccee3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0ccee8, SUC_lexicon_tags_55 },
    { 0xd28ccee9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf66cceeb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eccef1, SUC_lexicon_tags_1 },
    { 0x58accef2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe32ccef4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96ccef7, SUC_lexicon_tags_7 },
    { 0xd70ccef8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x110ccefa, SUC_lexicon_tags_7 },
    { 0x456ccefb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6cceff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202ccf06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd6ccf08, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cccf0c, SUC_lexicon_tags_37 },
    { 0x3accf0d, SUC_lexicon_tags_2 },
    { 0x812ccf0e, SUC_lexicon_tags_25 },
    { 0xebeccf0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39cccf11, SUC_lexicon_tags_14 },
    { 0x64ccf12, SUC_lexicon_tags_60 },
    { 0x41accf13, SUC_lexicon_tags_27 },
    { 0x9f2ccf14, SUC_lexicon_tags_2 },
    { 0x506ccf12, SUC_lexicon_tags_37 },
    { 0x98accf13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x444ccf18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2ccf1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x454ccf1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6ccf24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592ccf28, SUC_lexicon_tags_262 },
    { 0xd5cccf29, SUC_lexicon_tags_15 },
    { 0x6faccf28, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xe8eccf2c, SUC_lexicon_tags_15 },
    { 0x4ccccf2d, SUC_lexicon_tags_38 },
    { 0x182ccf2e, SUC_lexicon_tags_15 },
    { 0xc50ccf2e, SUC_lexicon_tags_9 },
    { 0x5daccf30, SUC_lexicon_tags_35 },
    { 0xd88ccf30, SUC_lexicon_tags_37 },
    { 0xa86ccf30, SUC_lexicon_tags_8 },
    { 0xfaeccf33, SUC_lexicon_tags_15 },
    { 0x53eccf34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x398ccf37, SUC_lexicon_tags_60 },
    { 0x760ccf38, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faccf3d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06ccf45, SUC_lexicon_tags_118 },
    { 0xde8ccf46, SUC_lexicon_tags_2 },
    { 0x7acccf47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59cccf4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84ccf4c, SUC_lexicon_tags_31 },
    { 0x84cccf4d, SUC_lexicon_tags_26 },
    { 0x1d2ccf4e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x24eccf50, SUC_lexicon_tags_15 },
    { 0xf32ccf50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cccf56, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1baccf58, SUC_lexicon_tags_26 },
    { 0xc4accf59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3faccf5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x762ccf5d, SUC_lexicon_tags_11 },
    { 0x848ccf5e, SUC_lexicon_tags_359 },
    { 0xc10ccf5d, SUC_lexicon_tags_10 },
    { 0xd36ccf60, SUC_lexicon_tags_88 },
    { 0x9a6ccf60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7d6ccf63, SUC_lexicon_tags_19 },
    { 0x25accf64, SUC_lexicon_tags_7 },
    { 0xfacccf65, SUC_lexicon_tags_9 },
    { 0xcb4ccf63, SUC_lexicon_tags_16 },
    { 0x1f2ccf67, SUC_lexicon_tags_87 },
    { 0xdb6ccf68, SUC_lexicon_tags_17 },
    { 0x12ccf69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x718ccf6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458ccf70, SUC_lexicon_tags_31 },
    { 0x4ecccf70, SUC_lexicon_tags_25 },
    { 0x538ccf71, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cccf75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe84ccf77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5e6ccf79, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9deccf7c, SUC_lexicon_tags_14 },
    { 0x8f8ccf7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfd8ccf7f, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xe44ccf81, SUC_lexicon_tags_8 },
    { 0xc46ccf82, SUC_lexicon_tags_60 },
    { 0xedeccf82, SUC_lexicon_tags_15 },
    { 0x448ccf84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa36ccf86, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160ccf8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386ccf91, SUC_lexicon_tags_11 },
    { 0x12eccf92, SUC_lexicon_tags_104 },
    { 0xbacccf92, SUC_lexicon_tags_40 },
    { 0x438ccf93, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x174ccf96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x840ccf98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d4ccf9a, SUC_lexicon_tags_3 },
    { 0x150ccf9b, SUC_lexicon_tags_25 },
    { 0xe6ccf9c, SUC_lexicon_tags_35 },
    { 0x4f0ccf9b, SUC_lexicon_tags_25 },
    { 0xd6cccf9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cccfa2, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66accfa9, SUC_lexicon_tags_31 },
    { 0x100ccfaa, SUC_lexicon_tags_19 },
    { 0xa9accfab, SUC_lexicon_tags_19 },
    { 0xfd2ccfac, SUC_lexicon_tags_47 },
    { 0xb9accfab, SUC_lexicon_tags_25 },
    { 0x424ccfae, SUC_lexicon_tags_25 },
    { 0x186ccfaf, SUC_lexicon_tags_37 },
    { 0x5d2ccfae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248ccfb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccccfb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x648ccfba, SUC_lexicon_tags_14 },
    { 0x1ecccfba, SUC_lexicon_tags_43 },
    { 0x4c8ccfba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83accfc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x816ccfc3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668ccfc7, SUC_lexicon_tags_77 },
    { 0x4faccfc8, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x736ccfca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c4ccfcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998ccfd5, SUC_lexicon_tags_45 },
    { 0xe7cccfd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804ccfda, SUC_lexicon_tags_30 },
    { 0xb14ccfda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc70ccfdd, SUC_lexicon_tags_26 },
    { 0xa2accfde, SUC_lexicon_tags_7 },
    { 0xa80ccfdf, SUC_lexicon_tags_14 },
    { 0xec4ccfdd, SUC_lexicon_tags_19 },
    { 0xb68ccfe1, SUC_lexicon_tags_60 },
    { 0x380ccfe2, SUC_lexicon_tags_0 },
    { 0x570ccfe3, SUC_lexicon_tags_14 },
    { 0x88ccfdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xedcccfe7, SUC_lexicon_tags_287 },
    { 0xce2ccfe8, SUC_lexicon_tags_6 },
    { 0x96accfe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1accff3, SUC_lexicon_tags_11 },
    { 0x504ccff4, SUC_lexicon_tags_19 },
    { 0x844ccff5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6ccff9, SUC_lexicon_tags_16 },
    { 0x146ccffa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34eccffe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126cd002, SUC_lexicon_tags_17 },
    { 0x5d0cd002, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ccd008, SUC_lexicon_tags_30 },
    { 0x7dccd008, SUC_lexicon_tags_7 },
    { 0x504cd00a, SUC_lexicon_tags_30 },
    { 0x1a0cd009, SUC_lexicon_tags_15 },
    { 0x8a0cd00b, SUC_lexicon_tags_39 },
    { 0xc74cd008, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9acd00f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa18cd011, SUC_lexicon_tags_11 },
    { 0x828cd012, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa4cd014, SUC_lexicon_tags_63 },
    { 0x30ecd014, SUC_lexicon_tags_129 },
    { 0x4a0cd015, SUC_lexicon_tags_45 },
    { 0x3b8cd017, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x438cd019, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1ccd01b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4facd021, SUC_lexicon_tags_11 },
    { 0xd7acd022, SUC_lexicon_tags_11 },
    { 0x6a4cd021, SUC_lexicon_tags_37 },
    { 0x58ccd022, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52cd02a, SUC_lexicon_tags_43 },
    { 0xffecd02a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244cd030, SUC_lexicon_tags_63 },
    { 0x16acd031, SUC_lexicon_tags_74 },
    { 0x724cd030, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584cd036, SUC_lexicon_tags_17 },
    { 0xc8ecd036, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4cd03b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4cd03e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ccd042, SUC_lexicon_tags_1 },
    { 0xb84cd043, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54cd04b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa22cd04d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8cd053, SUC_lexicon_tags_21 },
    { 0xd9ecd053, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x802cd057, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeccd06e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42cd074, SUC_lexicon_tags_28 },
    { 0x518cd075, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcbccd077, SUC_lexicon_tags_12 },
    { 0xc52cd077, SUC_lexicon_tags_12 },
    { 0x86acd079, SUC_lexicon_tags_43 },
    { 0x898cd07a, SUC_lexicon_tags_14 },
    { 0xdf6cd07b, SUC_lexicon_tags_15 },
    { 0xeacd07b, SUC_lexicon_tags_27 },
    { 0xa0acd07b, SUC_lexicon_tags_4 },
    { 0xde2cd07e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430cd082, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa08cd084, SUC_lexicon_tags_9 },
    { 0x28acd085, SUC_lexicon_tags_8 },
    { 0x212cd086, SUC_lexicon_tags_27 },
    { 0x58cd087, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1acd089, SUC_lexicon_tags_140 },
    { 0xc5acd08a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd92cd08c, SUC_lexicon_tags_5 },
    { 0xff6cd08d, SUC_lexicon_tags_9 },
    { 0x16ccd08e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x896cd090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eccd093, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2cd098, SUC_lexicon_tags_5 },
    { 0x26acd098, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4cd09c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e0cd09e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x872cd0a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cd0a3, SUC_lexicon_tags_15 },
    { 0x24ecd0a3, SUC_lexicon_tags_29 },
    { 0x262cd0a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb4cd0a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf60cd0a9, SUC_lexicon_tags_194 },
    { 0x66ecd0aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x568cd0ad, SUC_lexicon_tags_55 },
    { 0xa54cd0ae, SUC_lexicon_tags_21 },
    { 0x92ccd0af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2cd0b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0cd0c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcc6cd0c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ccd0c8, SUC_lexicon_tags_0 },
    { 0x61ecd0c9, SUC_lexicon_tags_5 },
    { 0xe88cd0ca, SUC_lexicon_tags_10 },
    { 0xfe0cd0cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x462cd0cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acd0e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f2cd0e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3decd0eb, SUC_lexicon_tags_33 },
    { 0x452cd0ec, SUC_lexicon_tags_11 },
    { 0xa44cd0eb, SUC_lexicon_tags_23 },
    { 0xe16cd0ee, SUC_lexicon_tags_37 },
    { 0xf9ecd0ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x798cd0f2, SUC_lexicon_tags_5 },
    { 0xa44cd0f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf0cd0f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4cd0fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc9acd0fc, SUC_lexicon_tags_13 },
    { 0x916cd0fc, SUC_lexicon_tags_25 },
    { 0x228cd0fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe00cd100, SUC_lexicon_tags_7 },
    { 0x806cd101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbccd105, SUC_lexicon_tags_11 },
    { 0xcfccd106, SUC_lexicon_tags_68 },
    { 0xb40cd107, SUC_lexicon_tags_15 },
    { 0x876cd108, SUC_lexicon_tags_9 },
    { 0xf82cd109, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3becd10d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4acd110, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2accd113, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2d8cd115, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ecd11b, SUC_lexicon_tags_32 },
    { 0xe14cd11c, SUC_lexicon_tags_1 },
    { 0xe96cd11d, SUC_lexicon_tags_37 },
    { 0xbc4cd11c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5acd121, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4cd124, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ecd129, SUC_lexicon_tags_15 },
    { 0x6cacd129, SUC_lexicon_tags_19 },
    { 0x824cd12a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ccd12f, SUC_lexicon_tags_11 },
    { 0x788cd130, SUC_lexicon_tags_12 },
    { 0xa9ecd12f, SUC_lexicon_tags_17 },
    { 0xb3ccd12f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d0cd134, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaccd136, SUC_lexicon_tags_37 },
    { 0x34acd137, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08cd13c, SUC_lexicon_tags_43 },
    { 0xc40cd13c, SUC_lexicon_tags_28 },
    { 0x96acd13e, SUC_lexicon_tags_60 },
    { 0xc80cd13e, SUC_lexicon_tags_7 },
    { 0x1dacd13c, SUC_lexicon_tags_25 },
    { 0xf88cd141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbacd145, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718cd14b, SUC_lexicon_tags_12 },
    { 0x9c6cd14c, SUC_lexicon_tags_15 },
    { 0xf44cd14d, SUC_lexicon_tags_12 },
    { 0x97ccd14d, SUC_lexicon_tags_37 },
    { 0x34acd14e, SUC_lexicon_tags_61 },
    { 0x68acd14d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84acd155, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912cd15b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xceccd15d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x122cd160, SUC_lexicon_tags_160 },
    { 0xabecd161, SUC_lexicon_tags_0 },
    { 0xc70cd162, SUC_lexicon_tags_25 },
    { 0x9aecd163, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ecd167, SUC_lexicon_tags_33 },
    { 0xb12cd168, SUC_lexicon_tags_17 },
    { 0x27ecd169, SUC_lexicon_tags_7 },
    { 0x964cd167, SUC_lexicon_tags_6 },
    { 0x11acd16a, SUC_lexicon_tags_9 },
    { 0x90ccd16c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae6cd16e, SUC_lexicon_tags_3 },
    { 0x7fecd16e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0cd173, SUC_lexicon_tags_25 },
    { 0xe82cd174, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cd177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f2cd179, SUC_lexicon_tags_0 },
    { 0x3e0cd17a, SUC_lexicon_tags_17 },
    { 0xa5acd17b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fecd180, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4cd183, SUC_lexicon_tags_17 },
    { 0x68ccd183, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x214cd187, SUC_lexicon_tags_21 },
    { 0x39acd188, SUC_lexicon_tags_15 },
    { 0xe24cd189, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x860cd18b, SUC_lexicon_tags_23 },
    { 0xccecd18c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa98cd18e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0cd191, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1cacd193, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962cd199, SUC_lexicon_tags_74 },
    { 0xc6cd19a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x78ccd19c, SUC_lexicon_tags_82 },
    { 0xcecd19d, SUC_lexicon_tags_39 },
    { 0xadecd19e, SUC_lexicon_tags_15 },
    { 0xe00cd19f, SUC_lexicon_tags_25 },
    { 0xad0cd19f, SUC_lexicon_tags_0 },
    { 0xd98cd1a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95acd1a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2acd1a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2cd1a8, SUC_lexicon_tags_65 },
    { 0x8dacd1a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90cd1ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bccd1b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd8ccd1b2, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0xeacd1b5, SUC_lexicon_tags_33 },
    { 0xfcecd1b6, SUC_lexicon_tags_11 },
    { 0xbe8cd1b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ecd1bc, SUC_lexicon_tags_17 },
    { 0x7e2cd1bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x666cd1bf, SUC_lexicon_tags_27 },
    { 0xfccd1c0, SUC_lexicon_tags_27 },
    { 0xe96cd1c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36cd1cc, SUC_lexicon_tags_49 },
    { 0xf18cd1cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6cd1d0, SUC_lexicon_tags_7 },
    { 0x6eacd1d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40cd1db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62ccd1dd, SUC_lexicon_tags_17 },
    { 0x7f8cd1dd, SUC_lexicon_tags_68 },
    { 0x51ecd1de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3ccd1e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cd1e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ccd1ea, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42cd1f1, SUC_lexicon_tags_19 },
    { 0xb94cd1f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fecd1f7, SUC_lexicon_tags_12 },
    { 0x198cd1f7, SUC_lexicon_tags_17 },
    { 0xc06cd1f7, SUC_lexicon_tags_7 },
    { 0x6acd1fa, SUC_lexicon_tags_65 },
    { 0xdd0cd1fb, SUC_lexicon_tags_15 },
    { 0xb56cd1f9, SUC_lexicon_tags_14 },
    { 0x542cd1fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ecd203, SUC_lexicon_tags_12 },
    { 0xb5ecd204, SUC_lexicon_tags_31 },
    { 0x136cd205, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecd20c, SUC_lexicon_tags_8 },
    { 0xe88cd20c, SUC_lexicon_tags_9 },
    { 0x74acd20e, SUC_lexicon_tags_7 },
    { 0xa5ecd20f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bacd213, SUC_lexicon_tags_60 },
    { 0xa3ccd214, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ccd217, SUC_lexicon_tags_13 },
    { 0x5e0cd217, SUC_lexicon_tags_1 },
    { 0x19ccd219, SUC_lexicon_tags_34 },
    { 0xb58cd21a, SUC_lexicon_tags_7 },
    { 0x11ccd21a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4cd21f, SUC_lexicon_tags_45 },
    { 0x59acd220, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06cd223, SUC_lexicon_tags_17 },
    { 0xa18cd223, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x486cd227, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x806cd229, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x402cd22c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714cd233, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91acd235, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3decd237, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcacd23a, SUC_lexicon_tags_7 },
    { 0xe7acd23b, SUC_lexicon_tags_11 },
    { 0x2decd23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdacd245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc96cd247, SUC_lexicon_tags_25 },
    { 0x484cd248, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x722cd24a, SUC_lexicon_tags_10 },
    { 0x6a0cd24b, SUC_lexicon_tags_11 },
    { 0x3d6cd24c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6cd250, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0cd253, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x112cd255, SUC_lexicon_tags_66 },
    { 0x84ecd256, SUC_lexicon_tags_65 },
    { 0x5e8cd255, SUC_lexicon_tags_32 },
    { 0xb7ccd258, SUC_lexicon_tags_29 },
    { 0x36acd258, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x652cd25b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfecd25e, SUC_lexicon_tags_6 },
    { 0xd3acd25f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4cecd261, SUC_lexicon_tags_11 },
    { 0x348cd261, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd268, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x446cd26a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x182cd26c, SUC_lexicon_tags_4 },
    { 0xc28cd26c, SUC_lexicon_tags_10 },
    { 0x356cd26c, SUC_lexicon_tags_0 },
    { 0x5aacd26c, SUC_lexicon_tags_17 },
    { 0x38ccd270, SUC_lexicon_tags_17 },
    { 0x3e8cd26d, SUC_lexicon_tags_1 },
    { 0x5a8cd272, SUC_lexicon_tags_14 },
    { 0x57ccd26c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542cd278, SUC_lexicon_tags_17 },
    { 0x17ccd279, SUC_lexicon_tags_26 },
    { 0x41ecd27a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc84cd27c, SUC_lexicon_tags_47 },
    { 0x156cd27c, SUC_lexicon_tags_7 },
    { 0xc22cd27e, SUC_lexicon_tags_77 },
    { 0xe2ccd27e, SUC_lexicon_tags_25 },
    { 0xdfecd27e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2acd282, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cecd287, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ecd28a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6cd28d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8cd29d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cd2a1, SUC_lexicon_tags_15 },
    { 0x452cd2a2, SUC_lexicon_tags_1 },
    { 0xbf8cd2a1, SUC_lexicon_tags_32 },
    { 0xd6acd2a4, SUC_lexicon_tags_37 },
    { 0x328cd2a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35acd2a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb76cd2a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ecd2ad, SUC_lexicon_tags_23 },
    { 0x88acd2ae, SUC_lexicon_tags_12 },
    { 0xc6cd2af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfacd2b3, SUC_lexicon_tags_25 },
    { 0xe66cd2b4, SUC_lexicon_tags_56 },
    { 0x35ecd2b5, SUC_lexicon_tags_17 },
    { 0x2cd2b6, SUC_lexicon_tags_7 },
    { 0x870cd2b6, SUC_lexicon_tags_11 },
    { 0xc18cd2b8, SUC_lexicon_tags_13 },
    { 0x3a2cd2b9, SUC_lexicon_tags_9 },
    { 0x2b2cd2b5, SUC_lexicon_tags_40 },
    { 0x296cd2bb, SUC_lexicon_tags_60 },
    { 0xed0cd2bc, SUC_lexicon_tags_45 },
    { 0x436cd2bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28cd2c1, SUC_lexicon_tags_8 },
    { 0x1dacd2c2, SUC_lexicon_tags_42 },
    { 0x380cd2c1, SUC_lexicon_tags_12 },
    { 0x692cd2c4, SUC_lexicon_tags_1 },
    { 0x936cd2c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x354cd2c8, SUC_lexicon_tags_17 },
    { 0x2e6cd2c9, SUC_lexicon_tags_77 },
    { 0x940cd2ca, SUC_lexicon_tags_1 },
    { 0xea6cd2cb, SUC_lexicon_tags_17 },
    { 0xa1acd2ca, SUC_lexicon_tags_23 },
    { 0xcb4cd2c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x244cd2d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc80cd2d2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ccd2d6, SUC_lexicon_tags_25 },
    { 0x270cd2d6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18cd2db, SUC_lexicon_tags_0 },
    { 0x9ccd2db, SUC_lexicon_tags_19 },
    { 0xa7ecd2db, SUC_lexicon_tags_15 },
    { 0x56cd2dd, SUC_lexicon_tags_21 },
    { 0xe72cd2dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x982cd2e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73acd2e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6cd2e8, SUC_lexicon_tags_27 },
    { 0xa96cd2e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20cd2eb, SUC_lexicon_tags_124 },
    { 0x8b0cd2ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x804cd2ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x982cd2f1, SUC_lexicon_tags_107 },
    { 0xefccd2f1, SUC_lexicon_tags_11 },
    { 0xbfecd2f1, SUC_lexicon_tags_2 },
    { 0x2eccd2f1, SUC_lexicon_tags_11 },
    { 0x458cd2f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa02cd2f7, SUC_lexicon_tags_7 },
    { 0x894cd2f8, SUC_lexicon_tags_25 },
    { 0x56cd2f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf42cd2fb, SUC_lexicon_tags_12 },
    { 0xfa4cd2fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x702cd2fe, SUC_lexicon_tags_65 },
    { 0x5becd2fe, SUC_lexicon_tags_6 },
    { 0x23acd300, SUC_lexicon_tags_37 },
    { 0x720cd301, SUC_lexicon_tags_60 },
    { 0x626cd2ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x18acd304, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56cd30b, SUC_lexicon_tags_797 },
    { 0, NULL },
    { 0x308cd30d, SUC_lexicon_tags_15 },
    { 0x31ecd30d, SUC_lexicon_tags_31 },
    { 0xddccd30f, SUC_lexicon_tags_46 },
    { 0xf90cd310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cccd317, SUC_lexicon_tags_40 },
    { 0x562cd317, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x32ccd31a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618cd31e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x18acd320, SUC_lexicon_tags_7 },
    { 0x550cd321, SUC_lexicon_tags_7 },
    { 0x144cd320, SUC_lexicon_tags_16 },
    { 0x588cd323, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cacd32b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x24ecd32d, SUC_lexicon_tags_2 },
    { 0x7f0cd32e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd50cd330, SUC_lexicon_tags_1 },
    { 0x9dacd330, SUC_lexicon_tags_11 },
    { 0x800cd332, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82cd335, SUC_lexicon_tags_12 },
    { 0x91ccd336, SUC_lexicon_tags_0 },
    { 0x21acd336, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56cd33e, SUC_lexicon_tags_12 },
    { 0x972cd33e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ccd342, SUC_lexicon_tags_77 },
    { 0x39acd343, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xdd2cd345, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6acd348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x198cd34a, SUC_lexicon_tags_12 },
    { 0x6eecd34a, SUC_lexicon_tags_56 },
    { 0x558cd34c, SUC_lexicon_tags_87 },
    { 0xd68cd34d, SUC_lexicon_tags_23 },
    { 0x118cd34e, SUC_lexicon_tags_12 },
    { 0x390cd34f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccccd351, SUC_lexicon_tags_2 },
    { 0xf2ccd352, SUC_lexicon_tags_12 },
    { 0x896cd353, SUC_lexicon_tags_37 },
    { 0x8facd354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba0cd356, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bacd358, SUC_lexicon_tags_92 },
    { 0x6aecd359, SUC_lexicon_tags_28 },
    { 0x646cd359, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47ecd35c, SUC_lexicon_tags_1 },
    { 0xf2acd35d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacccd362, SUC_lexicon_tags_63 },
    { 0x7e0cd363, SUC_lexicon_tags_54 },
    { 0x468cd364, SUC_lexicon_tags_11 },
    { 0xe12cd362, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x782cd367, SUC_lexicon_tags_16 },
    { 0x2f6cd368, SUC_lexicon_tags_17 },
    { 0xf38cd369, SUC_lexicon_tags_19 },
    { 0x90ccd36a, SUC_lexicon_tags_66 },
    { 0xe34cd36b, SUC_lexicon_tags_17 },
    { 0xc8cd36b, SUC_lexicon_tags_126 },
    { 0xa4ecd369, SUC_lexicon_tags_37 },
    { 0x9c0cd36c, SUC_lexicon_tags_12 },
    { 0x2eccd36f, SUC_lexicon_tags_158 },
    { 0x672cd36a, SUC_lexicon_tags_12 },
    { 0xe1ccd371, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x706cd374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6acd379, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6cd37f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ecd385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4becd388, SUC_lexicon_tags_27 },
    { 0x3d4cd389, SUC_lexicon_tags_14 },
    { 0x3bacd38a, SUC_lexicon_tags_31 },
    { 0x5d2cd389, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fccd391, SUC_lexicon_tags_46 },
    { 0x48cd391, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37ecd394, SUC_lexicon_tags_25 },
    { 0xd88cd395, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe3ccd397, SUC_lexicon_tags_37 },
    { 0x8fccd397, SUC_lexicon_tags_2 },
    { 0x1c0cd397, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x832cd39b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe40cd39d, SUC_lexicon_tags_31 },
    { 0x8c0cd39e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcaacd3a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ecd3a8, SUC_lexicon_tags_17 },
    { 0x344cd3a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22ccd3ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06cd3b0, SUC_lexicon_tags_39 },
    { 0x472cd3b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbecd3b5, SUC_lexicon_tags_7 },
    { 0xfd2cd3b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd02cd3b8, SUC_lexicon_tags_15 },
    { 0x474cd3b8, SUC_lexicon_tags_25 },
    { 0xed2cd3ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b6cd3bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72cd3bf, SUC_lexicon_tags_59 },
    { 0x300cd3c0, SUC_lexicon_tags_11 },
    { 0x2fccd3c1, SUC_lexicon_tags_25 },
    { 0xe0acd3c2, SUC_lexicon_tags_31 },
    { 0x506cd3c2, SUC_lexicon_tags_79 },
    { 0xc0acd3c1, SUC_lexicon_tags_15 },
    { 0x410cd3c3, SUC_lexicon_tags_3 },
    { 0x58cd3c6, SUC_lexicon_tags_17 },
    { 0xc58cd3c6, SUC_lexicon_tags_26 },
    { 0x424cd3c8, SUC_lexicon_tags_48 },
    { 0xda6cd3c7, SUC_lexicon_tags_3 },
    { 0x266cd3ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ccd3cf, SUC_lexicon_tags_29 },
    { 0x622cd3d0, SUC_lexicon_tags_44 },
    { 0xf10cd3d0, SUC_lexicon_tags_49 },
    { 0x1cecd3d1, SUC_lexicon_tags_15 },
    { 0xab0cd3d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536cd3d8, SUC_lexicon_tags_9 },
    { 0x49acd3d8, SUC_lexicon_tags_48 },
    { 0x290cd3da, SUC_lexicon_tags_1 },
    { 0x408cd3db, SUC_lexicon_tags_28 },
    { 0x5d0cd3dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x168cd3df, SUC_lexicon_tags_30 },
    { 0x4c0cd3e0, SUC_lexicon_tags_1 },
    { 0xc10cd3df, SUC_lexicon_tags_49 },
    { 0xaeccd3e2, SUC_lexicon_tags_77 },
    { 0x5bacd3e2, SUC_lexicon_tags_2 },
    { 0xb4ccd3e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7cccd3e6, SUC_lexicon_tags_60 },
    { 0x426cd3e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x276cd3e9, SUC_lexicon_tags_27 },
    { 0xb8cd3ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c2cd3ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0cd3ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838cd3f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cd3f7, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acd3fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaecd3fe, SUC_lexicon_tags_8 },
    { 0x7decd3ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2cd404, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962cd40b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x738cd40d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4cd410, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768cd414, SUC_lexicon_tags_15 },
    { 0x644cd415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108cd419, SUC_lexicon_tags_2 },
    { 0xdd2cd419, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248cd425, SUC_lexicon_tags_25 },
    { 0xbfccd426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4cd429, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2cd42c, SUC_lexicon_tags_79 },
    { 0x9ccd42d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8cd433, SUC_lexicon_tags_7 },
    { 0x12ccd434, SUC_lexicon_tags_0 },
    { 0xe24cd434, SUC_lexicon_tags_17 },
    { 0x818cd434, SUC_lexicon_tags_65 },
    { 0x456cd437, SUC_lexicon_tags_29 },
    { 0x34ecd436, SUC_lexicon_tags_19 },
    { 0xd50cd439, SUC_lexicon_tags_14 },
    { 0x594cd43a, SUC_lexicon_tags_11 },
    { 0x912cd43b, SUC_lexicon_tags_27 },
    { 0x440cd43b, SUC_lexicon_tags_17 },
    { 0xc36cd43d, SUC_lexicon_tags_1 },
    { 0x528cd43d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4cd441, SUC_lexicon_tags_17 },
    { 0xe80cd441, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdacd447, SUC_lexicon_tags_45 },
    { 0xd06cd448, SUC_lexicon_tags_28 },
    { 0xaa0cd449, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0cd44d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x602cd44f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04cd455, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83ecd457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ecd45b, SUC_lexicon_tags_25 },
    { 0xfcecd45c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ccd460, SUC_lexicon_tags_37 },
    { 0x11ecd460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1ecd463, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794cd469, SUC_lexicon_tags_37 },
    { 0xc3ccd46a, SUC_lexicon_tags_33 },
    { 0xd9ecd46a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5decd46e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce6cd470, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeacd472, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2cd475, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x944cd478, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08cd47d, SUC_lexicon_tags_17 },
    { 0xe26cd47e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x34acd481, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x22acd484, SUC_lexicon_tags_46 },
    { 0x3a6cd485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaacd48a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34acd48e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936cd497, SUC_lexicon_tags_11 },
    { 0x1a0cd498, SUC_lexicon_tags_7 },
    { 0x27ccd499, SUC_lexicon_tags_8 },
    { 0x81acd49a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558cd49f, SUC_lexicon_tags_75 },
    { 0x5a0cd49f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd8cd4a2, SUC_lexicon_tags_23 },
    { 0x118cd4a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10cd4a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ccd4a9, SUC_lexicon_tags_117 },
    { 0xb28cd4a9, SUC_lexicon_tags_37 },
    { 0x2eacd4aa, SUC_lexicon_tags_38 },
    { 0x89acd4ac, SUC_lexicon_tags_2 },
    { 0x6bacd4ad, SUC_lexicon_tags_46 },
    { 0xefccd4ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60acd4b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2cd4b5, SUC_lexicon_tags_2 },
    { 0x286cd4b6, SUC_lexicon_tags_14 },
    { 0x456cd4b6, SUC_lexicon_tags_16 },
    { 0x97ecd4b5, SUC_lexicon_tags_15 },
    { 0x7a2cd4b9, SUC_lexicon_tags_11 },
    { 0x416cd4ba, SUC_lexicon_tags_30 },
    { 0x3fccd4ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x932cd4bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xceacd4c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa9ccd4c2, SUC_lexicon_tags_37 },
    { 0xfc2cd4c2, SUC_lexicon_tags_38 },
    { 0xaaecd4c4, SUC_lexicon_tags_56 },
    { 0xe4ccd4c3, SUC_lexicon_tags_47 },
    { 0xfb8cd4c5, SUC_lexicon_tags_37 },
    { 0x2eacd4c4, SUC_lexicon_tags_49 },
    { 0x43acd4c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7becd4ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46acd4cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccd4d1, SUC_lexicon_tags_11 },
    { 0x170cd4d2, SUC_lexicon_tags_45 },
    { 0xcd4cd4d3, SUC_lexicon_tags_42 },
    { 0x858cd4d4, SUC_lexicon_tags_23 },
    { 0xc72cd4d5, SUC_lexicon_tags_49 },
    { 0x78ccd4d6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc3ecd4d8, SUC_lexicon_tags_12 },
    { 0x124cd4d8, SUC_lexicon_tags_7 },
    { 0x960cd4da, SUC_lexicon_tags_19 },
    { 0xd6ccd4db, SUC_lexicon_tags_2 },
    { 0xf78cd4da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcecd4df, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4cd4e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cd4e6, SUC_lexicon_tags_1 },
    { 0x9bacd4e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cd4eb, SUC_lexicon_tags_7 },
    { 0x54ccd4ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37acd4ee, SUC_lexicon_tags_37 },
    { 0xe2ccd4ef, SUC_lexicon_tags_1 },
    { 0xc04cd4f0, SUC_lexicon_tags_35 },
    { 0x5c2cd4f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1fccd4f3, SUC_lexicon_tags_7 },
    { 0xb06cd4f3, SUC_lexicon_tags_12 },
    { 0xb1ccd4f5, SUC_lexicon_tags_16 },
    { 0xa54cd4f4, SUC_lexicon_tags_37 },
    { 0x43acd4f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x490cd4f9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6cd500, SUC_lexicon_tags_224 },
    { 0xd0ccd501, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ccd504, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f8cd506, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828cd50a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66cd50f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfe6cd511, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3acd515, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6cd518, SUC_lexicon_tags_19 },
    { 0x3eecd519, SUC_lexicon_tags_17 },
    { 0xb44cd51a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ccd51d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262cd521, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4cd526, SUC_lexicon_tags_37 },
    { 0xb54cd527, SUC_lexicon_tags_17 },
    { 0x9becd528, SUC_lexicon_tags_63 },
    { 0x84cd528, SUC_lexicon_tags_33 },
    { 0x51ecd527, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08cd532, SUC_lexicon_tags_33 },
    { 0x6fecd533, SUC_lexicon_tags_37 },
    { 0xf4ccd533, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450cd539, SUC_lexicon_tags_17 },
    { 0xc1ccd53a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1d4cd53c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888cd546, SUC_lexicon_tags_12 },
    { 0x86ecd547, SUC_lexicon_tags_60 },
    { 0xdf6cd547, SUC_lexicon_tags_10 },
    { 0xc24cd548, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8decd54e, SUC_lexicon_tags_60 },
    { 0x768cd54f, SUC_lexicon_tags_17 },
    { 0x700cd54f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x510cd552, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc8cd554, SUC_lexicon_tags_32 },
    { 0x5c4cd554, SUC_lexicon_tags_40 },
    { 0xf04cd554, SUC_lexicon_tags_71 },
    { 0x3b6cd557, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220cd55b, SUC_lexicon_tags_45 },
    { 0x79ecd55c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ccd560, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc50cd562, SUC_lexicon_tags_1 },
    { 0x99ecd563, SUC_lexicon_tags_17 },
    { 0x73ccd563, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc06cd566, SUC_lexicon_tags_1 },
    { 0x280cd566, SUC_lexicon_tags_45 },
    { 0xe3ccd568, SUC_lexicon_tags_38 },
    { 0x70ccd568, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cd56d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70ecd56f, SUC_lexicon_tags_26 },
    { 0x234cd570, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716cd578, SUC_lexicon_tags_1 },
    { 0xba8cd579, SUC_lexicon_tags_17 },
    { 0xc02cd57a, SUC_lexicon_tags_15 },
    { 0xbbacd57a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116cd57f, SUC_lexicon_tags_37 },
    { 0xf4ccd580, SUC_lexicon_tags_16 },
    { 0x66cd581, SUC_lexicon_tags_33 },
    { 0x980cd581, SUC_lexicon_tags_3 },
    { 0x162cd583, SUC_lexicon_tags_1 },
    { 0x3f4cd583, SUC_lexicon_tags_11 },
    { 0xd4acd585, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4acd58b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bccd590, SUC_lexicon_tags_70 },
    { 0x7bccd590, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3cacd593, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x39ecd595, SUC_lexicon_tags_31 },
    { 0x5eccd596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ecd599, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb80cd59b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6cd5a1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88cd5a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6cd5b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ccd5b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114cd5b7, SUC_lexicon_tags_21 },
    { 0x2e8cd5b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a6cd5ba, SUC_lexicon_tags_12 },
    { 0xa7ecd5ba, SUC_lexicon_tags_12 },
    { 0x14acd5ba, SUC_lexicon_tags_37 },
    { 0x2e6cd5bd, SUC_lexicon_tags_11 },
    { 0xf98cd5be, SUC_lexicon_tags_17 },
    { 0xbf0cd5bf, SUC_lexicon_tags_15 },
    { 0x9c2cd5ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ecd5c3, SUC_lexicon_tags_0 },
    { 0xdc0cd5c4, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bccd5c9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x13ecd5cb, SUC_lexicon_tags_17 },
    { 0x7a8cd5cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ccd5cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ccd5d3, SUC_lexicon_tags_1 },
    { 0xe64cd5d4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718cd5da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83ccd5dc, SUC_lexicon_tags_2 },
    { 0x854cd5dd, SUC_lexicon_tags_11 },
    { 0x5accd5de, SUC_lexicon_tags_33 },
    { 0xb52cd5df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2cd5e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfacd5e8, SUC_lexicon_tags_13 },
    { 0x686cd5e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558cd5ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x89acd5f0, SUC_lexicon_tags_14 },
    { 0x7c8cd5f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732cd5f6, SUC_lexicon_tags_7 },
    { 0x47ccd5f6, SUC_lexicon_tags_2 },
    { 0xe38cd5f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64cd5fe, SUC_lexicon_tags_38 },
    { 0x3b2cd5ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bacd605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4ccd607, SUC_lexicon_tags_25 },
    { 0x346cd608, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ccd60b, SUC_lexicon_tags_1 },
    { 0xbd2cd60b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2cd612, SUC_lexicon_tags_2 },
    { 0xacacd613, SUC_lexicon_tags_2 },
    { 0x902cd614, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740cd61b, SUC_lexicon_tags_2 },
    { 0xca8cd61b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x292cd61e, SUC_lexicon_tags_33 },
    { 0x97acd61e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x230cd621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10cd624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62cd628, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16cd62b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14cd633, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a8cd635, SUC_lexicon_tags_0 },
    { 0xabccd635, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618cd63b, SUC_lexicon_tags_7 },
    { 0x258cd63c, SUC_lexicon_tags_7 },
    { 0xc4cd63d, SUC_lexicon_tags_60 },
    { 0x894cd63e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdeecd640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8cd648, SUC_lexicon_tags_33 },
    { 0xbf6cd649, SUC_lexicon_tags_1304 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ccd64f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bacd652, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604cd656, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcbecd658, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x660cd65a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cd65f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadecd661, SUC_lexicon_tags_11 },
    { 0x210cd661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe42cd664, SUC_lexicon_tags_25 },
    { 0x7becd665, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x218cd668, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24acd66e, SUC_lexicon_tags_46 },
    { 0x344cd66f, SUC_lexicon_tags_17 },
    { 0xc6cd670, SUC_lexicon_tags_17 },
    { 0x80acd66f, SUC_lexicon_tags_2 },
    { 0xab2cd66e, SUC_lexicon_tags_37 },
    { 0x606cd66e, SUC_lexicon_tags_1 },
    { 0xc4ccd66f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaaacd676, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x506cd679, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc18cd67b, SUC_lexicon_tags_26 },
    { 0x658cd67c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8ecd67e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86cd681, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x5d2cd683, SUC_lexicon_tags_14 },
    { 0x9aacd683, SUC_lexicon_tags_1 },
    { 0xee2cd684, SUC_lexicon_tags_30 },
    { 0x404cd686, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e6cd688, SUC_lexicon_tags_47 },
    { 0x516cd689, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21acd68f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2cd695, SUC_lexicon_tags_14 },
    { 0xccccd696, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34cd6a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc6ecd6a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86cd6aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf12cd6ac, SUC_lexicon_tags_17 },
    { 0x4becd6ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aacd6b1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592cd6b7, SUC_lexicon_tags_17 },
    { 0x886cd6b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ecd6c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x336cd6c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x608cd6c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x994cd6c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338cd6cd, SUC_lexicon_tags_25 },
    { 0x37ccd6ce, SUC_lexicon_tags_48 },
    { 0xcdacd6cf, SUC_lexicon_tags_1 },
    { 0xdcecd6ce, SUC_lexicon_tags_27 },
    { 0xbb0cd6d1, SUC_lexicon_tags_27 },
    { 0x6f2cd6d2, SUC_lexicon_tags_65 },
    { 0x2cd6d3, SUC_lexicon_tags_25 },
    { 0xc22cd6cf, SUC_lexicon_tags_103 },
    { 0x15acd6d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ecd6da, SUC_lexicon_tags_45 },
    { 0xd0cd6db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ccd6de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x526cd6e0, SUC_lexicon_tags_27 },
    { 0x2bacd6e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cd6e5, SUC_lexicon_tags_28 },
    { 0x41acd6e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8cd6f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ccd6f8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x47ecd6fa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecd702, SUC_lexicon_tags_65 },
    { 0xdd6cd703, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540cd708, SUC_lexicon_tags_12 },
    { 0xc24cd708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28acd70b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b0cd70d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6cd712, SUC_lexicon_tags_91 },
    { 0x3cccd713, SUC_lexicon_tags_25 },
    { 0xd7ccd714, SUC_lexicon_tags_13 },
    { 0xcc4cd715, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x270cd717, SUC_lexicon_tags_12 },
    { 0x6b6cd717, SUC_lexicon_tags_15 },
    { 0x528cd719, SUC_lexicon_tags_17 },
    { 0x8eacd71a, SUC_lexicon_tags_27 },
    { 0xee4cd71b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88cd720, SUC_lexicon_tags_23 },
    { 0x332cd721, SUC_lexicon_tags_17 },
    { 0x4a0cd722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14cd726, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafacd72b, SUC_lexicon_tags_7 },
    { 0x3c8cd72c, SUC_lexicon_tags_25 },
    { 0x3b6cd72b, SUC_lexicon_tags_1 },
    { 0x252cd72c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb62cd730, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3becd733, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc4cd735, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564cd739, SUC_lexicon_tags_15 },
    { 0x612cd739, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4cd73d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96acd740, SUC_lexicon_tags_38 },
    { 0x4facd740, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54ccd743, SUC_lexicon_tags_25 },
    { 0x9b2cd743, SUC_lexicon_tags_17 },
    { 0x7c2cd745, SUC_lexicon_tags_1 },
    { 0x70cd744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e8cd748, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83acd74b, SUC_lexicon_tags_17 },
    { 0x45ecd74c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2cd751, SUC_lexicon_tags_16 },
    { 0x5eecd752, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84cd75b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x906cd75e, SUC_lexicon_tags_35 },
    { 0x8d6cd75f, SUC_lexicon_tags_25 },
    { 0xf8cd75e, SUC_lexicon_tags_47 },
    { 0x17acd761, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d2cd763, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x150cd765, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ccd769, SUC_lexicon_tags_47 },
    { 0x13acd76a, SUC_lexicon_tags_28 },
    { 0x13ccd76b, SUC_lexicon_tags_3 },
    { 0x8fccd76a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6cd771, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfe8cd773, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2cd776, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11acd77e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eecd781, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x69acd784, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x420cd787, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2cd78c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaacd78f, SUC_lexicon_tags_17 },
    { 0x9fccd790, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1f6cd792, SUC_lexicon_tags_15 },
    { 0xb0ecd792, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ecd797, SUC_lexicon_tags_62 },
    { 0x6cecd797, SUC_lexicon_tags_17 },
    { 0x818cd799, SUC_lexicon_tags_11 },
    { 0xc68cd79a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x738cd79d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb62cd79f, SUC_lexicon_tags_16 },
    { 0x18ecd79f, SUC_lexicon_tags_7 },
    { 0xc9ccd7a1, SUC_lexicon_tags_4 },
    { 0xedecd7a2, SUC_lexicon_tags_31 },
    { 0x2accd7a2, SUC_lexicon_tags_124 },
    { 0xf0ccd7a2, SUC_lexicon_tags_38 },
    { 0x6becd7a5, SUC_lexicon_tags_37 },
    { 0x21ccd7a6, SUC_lexicon_tags_17 },
    { 0xebacd7a0, SUC_lexicon_tags_10 },
    { 0x854cd7a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x458cd7aa, SUC_lexicon_tags_10 },
    { 0x43ccd7ab, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x386cd7ad, SUC_lexicon_tags_56 },
    { 0x308cd7ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x238cd7b0, SUC_lexicon_tags_49 },
    { 0x3cacd7b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566cd7b5, SUC_lexicon_tags_17 },
    { 0x214cd7b5, SUC_lexicon_tags_23 },
    { 0xd3acd7b7, SUC_lexicon_tags_28 },
    { 0x290cd7b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf6cd7ba, SUC_lexicon_tags_11 },
    { 0x558cd7ba, SUC_lexicon_tags_34 },
    { 0x5becd7bc, SUC_lexicon_tags_64 },
    { 0x440cd7bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7c2cd7bf, SUC_lexicon_tags_66 },
    { 0x4f6cd7c0, SUC_lexicon_tags_26 },
    { 0x836cd7bf, SUC_lexicon_tags_2 },
    { 0x862cd7c2, SUC_lexicon_tags_14 },
    { 0x8c4cd7c2, SUC_lexicon_tags_1570 },
    { 0, NULL },
    { 0xaeccd7c5, SUC_lexicon_tags_61 },
    { 0x528cd7c5, SUC_lexicon_tags_2 },
    { 0x4bccd7c7, SUC_lexicon_tags_47 },
    { 0x3b2cd7c7, SUC_lexicon_tags_29 },
    { 0x236cd7c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfa2cd7cb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x402cd7cd, SUC_lexicon_tags_9 },
    { 0x79ecd7ce, SUC_lexicon_tags_37 },
    { 0x482cd7cf, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0xb80cd7d1, SUC_lexicon_tags_37 },
    { 0xeecd7d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98cd7d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x608cd7d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bccd7da, SUC_lexicon_tags_45 },
    { 0x5fccd7da, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812cd7e1, SUC_lexicon_tags_42 },
    { 0x728cd7e2, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0cd7e5, SUC_lexicon_tags_11 },
    { 0xa5ccd7e5, SUC_lexicon_tags_37 },
    { 0x6a6cd7e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x382cd7e9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x29ccd7eb, SUC_lexicon_tags_12 },
    { 0x438cd7eb, SUC_lexicon_tags_12 },
    { 0xe84cd7ed, SUC_lexicon_tags_2 },
    { 0x67ecd7ec, SUC_lexicon_tags_25 },
    { 0xb80cd7ef, SUC_lexicon_tags_70 },
    { 0xcc6cd7ec, SUC_lexicon_tags_65 },
    { 0xcbacd7f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x504cd7f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb94cd7f5, SUC_lexicon_tags_9 },
    { 0x52cd7f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae8cd7f8, SUC_lexicon_tags_13 },
    { 0x71ecd7f9, SUC_lexicon_tags_348 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4cd801, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192cd808, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xab4cd80a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16cd80f, SUC_lexicon_tags_1 },
    { 0xf72cd810, SUC_lexicon_tags_4 },
    { 0xe4ecd811, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8cd81b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0cd821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cd824, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf48cd826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8cd82c, SUC_lexicon_tags_50 },
    { 0x29ccd82c, SUC_lexicon_tags_4 },
    { 0x66ccd82e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2cd832, SUC_lexicon_tags_6 },
    { 0xf90cd832, SUC_lexicon_tags_7 },
    { 0xe5acd834, SUC_lexicon_tags_17 },
    { 0x9eecd834, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4cd83d, SUC_lexicon_tags_11 },
    { 0x504cd83d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0cd842, SUC_lexicon_tags_85 },
    { 0x7aecd842, SUC_lexicon_tags_39 },
    { 0xf3ccd843, SUC_lexicon_tags_45 },
    { 0xe3ccd845, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6cd84b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906cd855, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec4cd857, SUC_lexicon_tags_9 },
    { 0x4eecd857, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ccd860, SUC_lexicon_tags_26 },
    { 0x7d6cd861, SUC_lexicon_tags_33 },
    { 0x266cd861, SUC_lexicon_tags_9 },
    { 0xc5ccd863, SUC_lexicon_tags_10 },
    { 0x6dccd864, SUC_lexicon_tags_7 },
    { 0x8f0cd863, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0xd82cd867, SUC_lexicon_tags_71 },
    { 0x260cd867, SUC_lexicon_tags_0 },
    { 0xc28cd869, SUC_lexicon_tags_11 },
    { 0x5b0cd86a, SUC_lexicon_tags_12 },
    { 0x366cd86b, SUC_lexicon_tags_56 },
    { 0x2a0cd867, SUC_lexicon_tags_87 },
    { 0x306cd867, SUC_lexicon_tags_68 },
    { 0xbdecd86e, SUC_lexicon_tags_16 },
    { 0x470cd868, SUC_lexicon_tags_55 },
    { 0x688cd870, SUC_lexicon_tags_14 },
    { 0xc9acd869, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x842cd874, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cd877, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x748cd87a, SUC_lexicon_tags_3 },
    { 0xa38cd87a, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x588cd87d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeaacd87f, SUC_lexicon_tags_19 },
    { 0xe2acd87f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d8cd882, SUC_lexicon_tags_60 },
    { 0x55ccd882, SUC_lexicon_tags_33 },
    { 0xfbacd884, SUC_lexicon_tags_37 },
    { 0xdd8cd884, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x202cd887, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23acd88a, SUC_lexicon_tags_11 },
    { 0x5f2cd88b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x430cd88e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2cd892, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ecd895, SUC_lexicon_tags_200 },
    { 0x9bccd896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b8cd898, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772cd89c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500cd8a3, SUC_lexicon_tags_85 },
    { 0x26ccd8a4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48cd8a8, SUC_lexicon_tags_16 },
    { 0x1d8cd8a9, SUC_lexicon_tags_23 },
    { 0x2d6cd8a8, SUC_lexicon_tags_17 },
    { 0xb36cd8ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ccd8b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb2ecd8b3, SUC_lexicon_tags_31 },
    { 0xf18cd8b4, SUC_lexicon_tags_2 },
    { 0x346cd8b5, SUC_lexicon_tags_17 },
    { 0xe90cd8b6, SUC_lexicon_tags_45 },
    { 0xccccd8b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef4cd8b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4c2cd8bb, SUC_lexicon_tags_29 },
    { 0xdb0cd8bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4acd8c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x626cd8c3, SUC_lexicon_tags_17 },
    { 0x392cd8c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2cd8c8, SUC_lexicon_tags_15 },
    { 0xdd0cd8c8, SUC_lexicon_tags_47 },
    { 0x4e0cd8c8, SUC_lexicon_tags_25 },
    { 0x8c2cd8ca, SUC_lexicon_tags_28 },
    { 0x7e0cd8ca, SUC_lexicon_tags_1 },
    { 0x66ccd8cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd7ecd8cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x404cd8d1, SUC_lexicon_tags_15 },
    { 0x432cd8d1, SUC_lexicon_tags_45 },
    { 0x3d8cd8d1, SUC_lexicon_tags_49 },
    { 0xedacd8d4, SUC_lexicon_tags_62 },
    { 0xbc8cd8d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728cd8dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136cd8e0, SUC_lexicon_tags_2 },
    { 0x958cd8e0, SUC_lexicon_tags_17 },
    { 0xcaecd8e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x29ecd8e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x290cd8e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acd8eb, SUC_lexicon_tags_2 },
    { 0x786cd8ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6cd8ef, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xc7ccd8f1, SUC_lexicon_tags_7 },
    { 0xcc6cd8f2, SUC_lexicon_tags_7 },
    { 0xceecd8f3, SUC_lexicon_tags_48 },
    { 0x796cd8f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54cd8f7, SUC_lexicon_tags_25 },
    { 0x6e8cd8f8, SUC_lexicon_tags_7 },
    { 0xfaecd8f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58cd8fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbecd900, SUC_lexicon_tags_45 },
    { 0xe14cd901, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb6ccd903, SUC_lexicon_tags_17 },
    { 0xf86cd903, SUC_lexicon_tags_15 },
    { 0xda2cd904, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ccd90a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbacd90f, SUC_lexicon_tags_17 },
    { 0x5cacd910, SUC_lexicon_tags_12 },
    { 0xdeacd911, SUC_lexicon_tags_15 },
    { 0xcb6cd912, SUC_lexicon_tags_4 },
    { 0xff6cd910, SUC_lexicon_tags_7 },
    { 0xddecd90f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2aecd916, SUC_lexicon_tags_37 },
    { 0x12cd916, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa34cd919, SUC_lexicon_tags_37 },
    { 0xea4cd919, SUC_lexicon_tags_12 },
    { 0x7c2cd91a, SUC_lexicon_tags_6 },
    { 0x66acd91b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaacd91e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0cd921, SUC_lexicon_tags_12 },
    { 0xf5ecd921, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456cd927, SUC_lexicon_tags_25 },
    { 0xb8ccd928, SUC_lexicon_tags_25 },
    { 0x856cd929, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ccd92e, SUC_lexicon_tags_11 },
    { 0xf70cd92e, SUC_lexicon_tags_37 },
    { 0x9c0cd930, SUC_lexicon_tags_65 },
    { 0xce4cd931, SUC_lexicon_tags_17 },
    { 0xf0ecd932, SUC_lexicon_tags_16 },
    { 0xabecd933, SUC_lexicon_tags_27 },
    { 0x484cd934, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7eacd936, SUC_lexicon_tags_17 },
    { 0x582cd937, SUC_lexicon_tags_11 },
    { 0xf34cd937, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bccd93f, SUC_lexicon_tags_1 },
    { 0xfdccd940, SUC_lexicon_tags_11 },
    { 0xd7ccd941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ecd946, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74acd948, SUC_lexicon_tags_27 },
    { 0x622cd949, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9acd94c, SUC_lexicon_tags_37 },
    { 0x5b2cd94d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eacd952, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x522cd954, SUC_lexicon_tags_0 },
    { 0x5fccd955, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x352cd958, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cecd95a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x554cd95d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb10cd95f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784cd963, SUC_lexicon_tags_295 },
    { 0x7acd964, SUC_lexicon_tags_7 },
    { 0x6cacd964, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2cd969, SUC_lexicon_tags_2 },
    { 0xc90cd96a, SUC_lexicon_tags_14 },
    { 0xef8cd96b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ecd96e, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x4acd971, SUC_lexicon_tags_63 },
    { 0x7f2cd972, SUC_lexicon_tags_23 },
    { 0x826cd972, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2cd975, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ccd97c, SUC_lexicon_tags_10 },
    { 0x1d4cd97d, SUC_lexicon_tags_17 },
    { 0x4cecd97e, SUC_lexicon_tags_31 },
    { 0xdcccd97e, SUC_lexicon_tags_65 },
    { 0x64acd980, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0cd988, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53ecd98a, SUC_lexicon_tags_33 },
    { 0x994cd98b, SUC_lexicon_tags_46 },
    { 0x236cd98c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ccd992, SUC_lexicon_tags_12 },
    { 0x480cd992, SUC_lexicon_tags_60 },
    { 0x9bccd994, SUC_lexicon_tags_45 },
    { 0xf20cd994, SUC_lexicon_tags_185 },
    { 0x54ecd996, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ccd99f, SUC_lexicon_tags_1 },
    { 0xe50cd9a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x500cd9a3, SUC_lexicon_tags_30 },
    { 0x988cd9a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ecd9a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ccd9ac, SUC_lexicon_tags_14 },
    { 0x7e8cd9ad, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x27ccd9af, SUC_lexicon_tags_2 },
    { 0xed2cd9af, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822cd9b8, SUC_lexicon_tags_45 },
    { 0x4e6cd9b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ecd9c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa42cd9c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8ecd9c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84cd9cb, SUC_lexicon_tags_13 },
    { 0xb32cd9cc, SUC_lexicon_tags_31 },
    { 0xc20cd9cc, SUC_lexicon_tags_27 },
    { 0x2d8cd9cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cd9d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32acd9d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x244cd9d7, SUC_lexicon_tags_21 },
    { 0xc08cd9d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x370cd9db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdaccd9dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccd9e7, SUC_lexicon_tags_26 },
    { 0xf5acd9e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72cd9ef, SUC_lexicon_tags_45 },
    { 0xf9acd9f0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360cd9f5, SUC_lexicon_tags_37 },
    { 0xb66cd9f6, SUC_lexicon_tags_1 },
    { 0x54cd9f5, SUC_lexicon_tags_33 },
    { 0x8accd9f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0cd9fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0ccd9fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aecda01, SUC_lexicon_tags_25 },
    { 0x854cda01, SUC_lexicon_tags_16 },
    { 0x3d4cda01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecda0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee0cda0d, SUC_lexicon_tags_797 },
    { 0x32cda0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b8cda10, SUC_lexicon_tags_38 },
    { 0xb48cda11, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x906cda13, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x370cda15, SUC_lexicon_tags_55 },
    { 0xc3acda15, SUC_lexicon_tags_57 },
    { 0xf14cda17, SUC_lexicon_tags_17 },
    { 0x86cda18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaacda1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x584cda20, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34acda25, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x734cda27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2cda2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad4cda2c, SUC_lexicon_tags_46 },
    { 0x9e4cda2c, SUC_lexicon_tags_4 },
    { 0xdf0cda2e, SUC_lexicon_tags_79 },
    { 0xe2ecda2e, SUC_lexicon_tags_37 },
    { 0xacccda30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ccda34, SUC_lexicon_tags_2 },
    { 0x5accda34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ccda38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x196cda3a, SUC_lexicon_tags_7 },
    { 0xb0acda3b, SUC_lexicon_tags_49 },
    { 0x146cda3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ecda46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2cda4a, SUC_lexicon_tags_15 },
    { 0xaf0cda4a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ecda50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x770cda52, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3eecda54, SUC_lexicon_tags_47 },
    { 0x8b8cda55, SUC_lexicon_tags_88 },
    { 0x370cda56, SUC_lexicon_tags_37 },
    { 0x47ccda57, SUC_lexicon_tags_24 },
    { 0x22cda56, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x6dccda5a, SUC_lexicon_tags_49 },
    { 0x64ecda5b, SUC_lexicon_tags_37 },
    { 0xc2cda5c, SUC_lexicon_tags_49 },
    { 0x84ccda5c, SUC_lexicon_tags_12 },
    { 0x59acda5a, SUC_lexicon_tags_63 },
    { 0x2a4cda5f, SUC_lexicon_tags_2 },
    { 0x6becda60, SUC_lexicon_tags_8 },
    { 0x994cda61, SUC_lexicon_tags_8 },
    { 0xf00cda61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96cda67, SUC_lexicon_tags_12 },
    { 0x134cda68, SUC_lexicon_tags_2 },
    { 0xcb4cda67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeecda6c, SUC_lexicon_tags_19 },
    { 0x8a8cda6c, SUC_lexicon_tags_36 },
    { 0x5c0cda6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x972cda71, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x148cda74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4cda7d, SUC_lexicon_tags_43 },
    { 0x83acda7e, SUC_lexicon_tags_68 },
    { 0x65acda7e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82cda84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0cda88, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bccda8d, SUC_lexicon_tags_11 },
    { 0xbc2cda8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccda92, SUC_lexicon_tags_11 },
    { 0x69ecda92, SUC_lexicon_tags_837 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4cda98, SUC_lexicon_tags_25 },
    { 0x5c4cda99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8cda9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62cdaa0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x862cdaa3, SUC_lexicon_tags_12 },
    { 0x968cdaa4, SUC_lexicon_tags_23 },
    { 0x6e0cdaa5, SUC_lexicon_tags_2 },
    { 0x8fccdaa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0cdaa8, SUC_lexicon_tags_57 },
    { 0xf8ecdaa9, SUC_lexicon_tags_14 },
    { 0xaa4cdaaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2ccdaac, SUC_lexicon_tags_39 },
    { 0x52acdaad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x216cdaaf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x224cdab1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40ccdab3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x946cdab5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x350cdab7, SUC_lexicon_tags_18 },
    { 0x602cdab8, SUC_lexicon_tags_1 },
    { 0x72ccdab9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe5ecdabb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8cdac9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e8cdacb, SUC_lexicon_tags_37 },
    { 0xd5ccdacb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9acdace, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2decdad1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eccdad4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4acdad9, SUC_lexicon_tags_2 },
    { 0x3c2cdad9, SUC_lexicon_tags_25 },
    { 0x2b2cdadb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed8cdadd, SUC_lexicon_tags_2 },
    { 0xd7acdadd, SUC_lexicon_tags_47 },
    { 0x904cdadf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42cdae5, SUC_lexicon_tags_7 },
    { 0x6a6cdae5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x49acdae8, SUC_lexicon_tags_72 },
    { 0xd56cdae9, SUC_lexicon_tags_30 },
    { 0x426cdaea, SUC_lexicon_tags_17 },
    { 0xe9ecdaeb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccecdaf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3ccdaf4, SUC_lexicon_tags_45 },
    { 0xca6cdaf4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e2cdaf7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbacdafc, SUC_lexicon_tags_79 },
    { 0x65acdafd, SUC_lexicon_tags_23 },
    { 0x3b6cdafe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9acdb07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf12cdb09, SUC_lexicon_tags_25 },
    { 0xd32cdb0a, SUC_lexicon_tags_10 },
    { 0x5cacdb0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4cdb0f, SUC_lexicon_tags_71 },
    { 0x1eacdb10, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42cdb14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x468cdb17, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34cdb1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986cdb21, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22cdb27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc0ecdb29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dccdb2c, SUC_lexicon_tags_6 },
    { 0xe62cdb2c, SUC_lexicon_tags_17 },
    { 0x120cdb2e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1c6cdb30, SUC_lexicon_tags_47 },
    { 0x364cdb31, SUC_lexicon_tags_151 },
    { 0xa58cdb31, SUC_lexicon_tags_7 },
    { 0x22ecdb33, SUC_lexicon_tags_63 },
    { 0x38acdb34, SUC_lexicon_tags_26 },
    { 0x894cdb35, SUC_lexicon_tags_11 },
    { 0x402cdb34, SUC_lexicon_tags_12 },
    { 0x6cecdb37, SUC_lexicon_tags_38 },
    { 0x748cdb38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x958cdb3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20cdb41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeacdb43, SUC_lexicon_tags_45 },
    { 0x91ecdb44, SUC_lexicon_tags_43 },
    { 0xc4ccdb44, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcacdb4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf46cdb4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x156cdb52, SUC_lexicon_tags_2 },
    { 0xd96cdb53, SUC_lexicon_tags_37 },
    { 0x556cdb52, SUC_lexicon_tags_61 },
    { 0x600cdb54, SUC_lexicon_tags_65 },
    { 0x794cdb54, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aecdb59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x118cdb5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4becdb5f, SUC_lexicon_tags_15 },
    { 0xf54cdb60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ccdb63, SUC_lexicon_tags_14 },
    { 0x5b6cdb64, SUC_lexicon_tags_55 },
    { 0x260cdb64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192cdb6a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150cdb70, SUC_lexicon_tags_17 },
    { 0xb16cdb71, SUC_lexicon_tags_15 },
    { 0x976cdb70, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ccdb77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fccdb7c, SUC_lexicon_tags_4 },
    { 0xfccdb7d, SUC_lexicon_tags_37 },
    { 0x8d2cdb7c, SUC_lexicon_tags_15 },
    { 0x894cdb7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654cdb87, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0cdb8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ccdb8e, SUC_lexicon_tags_32 },
    { 0x256cdb8f, SUC_lexicon_tags_17 },
    { 0x7b0cdb8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b6cdb92, SUC_lexicon_tags_37 },
    { 0x8c8cdb92, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x294cdb95, SUC_lexicon_tags_38 },
    { 0xdc2cdb95, SUC_lexicon_tags_14 },
    { 0xf94cdb95, SUC_lexicon_tags_63 },
    { 0x918cdb95, SUC_lexicon_tags_160 },
    { 0x228cdb99, SUC_lexicon_tags_27 },
    { 0xe8ccdb95, SUC_lexicon_tags_11 },
    { 0xf0ecdb9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda2cdb9d, SUC_lexicon_tags_23 },
    { 0xf8ecdb9e, SUC_lexicon_tags_6 },
    { 0x67ecdb9d, SUC_lexicon_tags_23 },
    { 0xe74cdb9d, SUC_lexicon_tags_11 },
    { 0xf8ecdba1, SUC_lexicon_tags_15 },
    { 0x476cdba2, SUC_lexicon_tags_7 },
    { 0x376cdba2, SUC_lexicon_tags_17 },
    { 0x316cdb9f, SUC_lexicon_tags_17 },
    { 0xe74cdba1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446cdbad, SUC_lexicon_tags_15 },
    { 0x86ecdbae, SUC_lexicon_tags_77 },
    { 0x20ccdbaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aecdbb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6cdbb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ecdbbb, SUC_lexicon_tags_18 },
    { 0xcfecdbbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe04cdbbe, SUC_lexicon_tags_7 },
    { 0xa78cdbbe, SUC_lexicon_tags_15 },
    { 0x1e8cdbbf, SUC_lexicon_tags_15 },
    { 0xc84cdbc1, SUC_lexicon_tags_13 },
    { 0x448cdbbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ecdbc8, SUC_lexicon_tags_7 },
    { 0xe9acdbc9, SUC_lexicon_tags_8 },
    { 0xbe4cdbca, SUC_lexicon_tags_57 },
    { 0x97ecdbcb, SUC_lexicon_tags_31 },
    { 0x2aacdbcb, SUC_lexicon_tags_6 },
    { 0x852cdbca, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x156cdbd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c2cdbd2, SUC_lexicon_tags_12 },
    { 0xe6ccdbd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36cdbd6, SUC_lexicon_tags_26 },
    { 0x556cdbd7, SUC_lexicon_tags_19 },
    { 0x442cdbd8, SUC_lexicon_tags_37 },
    { 0x264cdbd9, SUC_lexicon_tags_28 },
    { 0x65ccdbda, SUC_lexicon_tags_14 },
    { 0x110cdbd8, SUC_lexicon_tags_65 },
    { 0x764cdbd8, SUC_lexicon_tags_7 },
    { 0x13ccdbdb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x130cdbdf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62cdbe2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x554cdbe4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5c2cdbe6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ccdbe9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352cdbee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9becdbf0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06cdbf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x53acdbf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x352cdbfa, SUC_lexicon_tags_47 },
    { 0x27ecdbfa, SUC_lexicon_tags_23 },
    { 0x78cdbfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15ccdbfe, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4cdc01, SUC_lexicon_tags_12 },
    { 0x322cdc01, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x378cdc05, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6cdc09, SUC_lexicon_tags_11 },
    { 0x446cdc0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcacdc0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ecdc11, SUC_lexicon_tags_38 },
    { 0x9facdc11, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0cdc16, SUC_lexicon_tags_11 },
    { 0x12ecdc17, SUC_lexicon_tags_54 },
    { 0x75acdc18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64cdc1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x754cdc1e, SUC_lexicon_tags_8 },
    { 0x376cdc1f, SUC_lexicon_tags_25 },
    { 0x266cdc20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ecdc27, SUC_lexicon_tags_11 },
    { 0x244cdc28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8facdc2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ccdc31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284cdc37, SUC_lexicon_tags_37 },
    { 0x2d2cdc37, SUC_lexicon_tags_37 },
    { 0xd6ccdc39, SUC_lexicon_tags_68 },
    { 0x5f4cdc38, SUC_lexicon_tags_12 },
    { 0xcc2cdc3b, SUC_lexicon_tags_47 },
    { 0xcaccdc39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02cdc42, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ccdc4b, SUC_lexicon_tags_11 },
    { 0xd6cdc4b, SUC_lexicon_tags_26 },
    { 0x986cdc4d, SUC_lexicon_tags_12 },
    { 0x3d4cdc4e, SUC_lexicon_tags_21 },
    { 0xa88cdc4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ecdc55, SUC_lexicon_tags_20 },
    { 0x18ecdc56, SUC_lexicon_tags_65 },
    { 0x524cdc56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x70cdc59, SUC_lexicon_tags_15 },
    { 0x8cacdc5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6cdc5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4cdc61, SUC_lexicon_tags_60 },
    { 0x2eecdc62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x74cdc64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118cdc6a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x762cdc6c, SUC_lexicon_tags_26 },
    { 0x1bacdc6d, SUC_lexicon_tags_9 },
    { 0x302cdc6e, SUC_lexicon_tags_7 },
    { 0x3dccdc6c, SUC_lexicon_tags_37 },
    { 0x62cdc70, SUC_lexicon_tags_1 },
    { 0xceccdc71, SUC_lexicon_tags_27 },
    { 0x10ccdc70, SUC_lexicon_tags_25 },
    { 0x766cdc72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1acdc76, SUC_lexicon_tags_37 },
    { 0xd38cdc77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x84ccdc79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x532cdc7c, SUC_lexicon_tags_65 },
    { 0xbf0cdc7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4aecdc7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250cdc87, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52cdc8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ccdc91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48cdc94, SUC_lexicon_tags_31 },
    { 0x1acdc95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0acdc99, SUC_lexicon_tags_11 },
    { 0xef2cdc9a, SUC_lexicon_tags_17 },
    { 0x75ecdc9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58ecdc9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8cdcaf, SUC_lexicon_tags_37 },
    { 0xb2ecdcb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2cdcb6, SUC_lexicon_tags_37 },
    { 0x93ccdcb7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf0cdcb9, SUC_lexicon_tags_53 },
    { 0x20acdcba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bccdcbc, SUC_lexicon_tags_29 },
    { 0x57ecdcbd, SUC_lexicon_tags_25 },
    { 0xb62cdcbe, SUC_lexicon_tags_25 },
    { 0x4c6cdcbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96cdccd, SUC_lexicon_tags_29 },
    { 0x39ccdccd, SUC_lexicon_tags_23 },
    { 0x4bccdccf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2cdcd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ccdcde, SUC_lexicon_tags_48 },
    { 0xfa8cdcdf, SUC_lexicon_tags_17 },
    { 0x48acdcde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x248cdce3, SUC_lexicon_tags_43 },
    { 0x252cdce4, SUC_lexicon_tags_57 },
    { 0x654cdce5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8cdcf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ccdcf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xddecdcf9, SUC_lexicon_tags_43 },
    { 0x2accdcfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa52cdcfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccecdd01, SUC_lexicon_tags_4 },
    { 0x1d6cdd02, SUC_lexicon_tags_25 },
    { 0xc10cdd03, SUC_lexicon_tags_14 },
    { 0x6a0cdd04, SUC_lexicon_tags_45 },
    { 0x86ccdd02, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x810cdd08, SUC_lexicon_tags_12 },
    { 0xce6cdd09, SUC_lexicon_tags_7 },
    { 0x97acdd09, SUC_lexicon_tags_14 },
    { 0xb6ccdd0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ecdd10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6cdd1d, SUC_lexicon_tags_1 },
    { 0x8e8cdd1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x614cdd20, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x832cdd22, SUC_lexicon_tags_1 },
    { 0x8c2cdd23, SUC_lexicon_tags_10 },
    { 0xad8cdd24, SUC_lexicon_tags_25 },
    { 0x12cdd24, SUC_lexicon_tags_23 },
    { 0xa12cdd26, SUC_lexicon_tags_37 },
    { 0x11ccdd25, SUC_lexicon_tags_61 },
    { 0xf32cdd23, SUC_lexicon_tags_47 },
    { 0x708cdd25, SUC_lexicon_tags_63 },
    { 0x46acdd2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f4cdd2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208cdd30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa26cdd32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bccdd3b, SUC_lexicon_tags_15 },
    { 0xbd4cdd3c, SUC_lexicon_tags_14 },
    { 0x86acdd3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872cdd42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57acdd44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ecdd48, SUC_lexicon_tags_15 },
    { 0x66cdd49, SUC_lexicon_tags_38 },
    { 0xdbacdd48, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x640cdd4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8acdd4e, SUC_lexicon_tags_37 },
    { 0x80ccdd4f, SUC_lexicon_tags_7 },
    { 0x57ccdd4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x344cdd52, SUC_lexicon_tags_37 },
    { 0x23ecdd53, SUC_lexicon_tags_10 },
    { 0x3e8cdd52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ccdd57, SUC_lexicon_tags_15 },
    { 0x358cdd57, SUC_lexicon_tags_63 },
    { 0xd74cdd59, SUC_lexicon_tags_17 },
    { 0x4a2cdd5a, SUC_lexicon_tags_31 },
    { 0x210cdd5a, SUC_lexicon_tags_2 },
    { 0x4accdd5a, SUC_lexicon_tags_11 },
    { 0xd4acdd5b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9e2cdd5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ecdd63, SUC_lexicon_tags_14 },
    { 0x1eacdd64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9acdd66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7acdd68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0acdd6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aacdd6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd58cdd6f, SUC_lexicon_tags_15 },
    { 0xd8ecdd70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9ecdd72, SUC_lexicon_tags_47 },
    { 0x3e6cdd73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cdd77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda8cdd79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe66cdd7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0cdd81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ecdd84, SUC_lexicon_tags_62 },
    { 0xd8cdd84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80cdd87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742cdd8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19ecdd8e, SUC_lexicon_tags_60 },
    { 0xd4ecdd8f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcacdd95, SUC_lexicon_tags_38 },
    { 0x106cdd95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6cdd99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ccdda2, SUC_lexicon_tags_47 },
    { 0xf8acdda3, SUC_lexicon_tags_55 },
    { 0x1bacdda4, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ccddaa, SUC_lexicon_tags_25 },
    { 0x432cddab, SUC_lexicon_tags_25 },
    { 0x350cddac, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942cddb1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6cddb5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccddba, SUC_lexicon_tags_9 },
    { 0x92ccddba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc42cddbd, SUC_lexicon_tags_15 },
    { 0x4c8cddbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9d4cddc0, SUC_lexicon_tags_2 },
    { 0x138cddc1, SUC_lexicon_tags_45 },
    { 0xec8cddc2, SUC_lexicon_tags_26 },
    { 0x208cddc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96cddc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124cddd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cddd6, SUC_lexicon_tags_14 },
    { 0xa46cddd7, SUC_lexicon_tags_27 },
    { 0x844cddd8, SUC_lexicon_tags_47 },
    { 0xeb0cddd8, SUC_lexicon_tags_26 },
    { 0x89ecddd8, SUC_lexicon_tags_33 },
    { 0xc04cddd9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0cddde, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fccdde1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacacdde7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bccddea, SUC_lexicon_tags_46 },
    { 0x864cddeb, SUC_lexicon_tags_49 },
    { 0x44ccddec, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x31acddee, SUC_lexicon_tags_19 },
    { 0x88ecddef, SUC_lexicon_tags_29 },
    { 0xb46cddef, SUC_lexicon_tags_17 },
    { 0x2decddf1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8acddf3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x328cddf6, SUC_lexicon_tags_13 },
    { 0xf0ccddf7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7decde01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600cde06, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x258cde08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6cde0d, SUC_lexicon_tags_17 },
    { 0x75acde0d, SUC_lexicon_tags_77 },
    { 0x562cde0f, SUC_lexicon_tags_43 },
    { 0x78acde10, SUC_lexicon_tags_15 },
    { 0xb76cde0d, SUC_lexicon_tags_11 },
    { 0xdc0cde12, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xba2cde14, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf58cde16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ccde1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2cde1f, SUC_lexicon_tags_27 },
    { 0xf9acde1f, SUC_lexicon_tags_0 },
    { 0xc08cde21, SUC_lexicon_tags_2 },
    { 0xceecde20, SUC_lexicon_tags_47 },
    { 0xeaecde23, SUC_lexicon_tags_2 },
    { 0xc68cde1f, SUC_lexicon_tags_65 },
    { 0x826cde25, SUC_lexicon_tags_17 },
    { 0xd2ccde26, SUC_lexicon_tags_42 },
    { 0xd08cde20, SUC_lexicon_tags_8 },
    { 0x94ecde24, SUC_lexicon_tags_17 },
    { 0x51ccde29, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x564cde2c, SUC_lexicon_tags_68 },
    { 0xe4acde2d, SUC_lexicon_tags_436 },
    { 0x524cde2e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x79acde30, SUC_lexicon_tags_7 },
    { 0xc6ccde30, SUC_lexicon_tags_31 },
    { 0xaf2cde32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfacde34, SUC_lexicon_tags_19 },
    { 0x5c2cde34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ecde38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294cde3c, SUC_lexicon_tags_14 },
    { 0x432cde3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x408cde3f, SUC_lexicon_tags_7 },
    { 0xe76cde40, SUC_lexicon_tags_7 },
    { 0x22ecde40, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8b8cde43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994cde49, SUC_lexicon_tags_25 },
    { 0x8f6cde4a, SUC_lexicon_tags_12 },
    { 0xdb4cde4b, SUC_lexicon_tags_60 },
    { 0x53acde49, SUC_lexicon_tags_0 },
    { 0xeb0cde4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30cde56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84cde5a, SUC_lexicon_tags_21 },
    { 0x870cde5b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf40cde5d, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3acde61, SUC_lexicon_tags_7 },
    { 0xe0acde62, SUC_lexicon_tags_35 },
    { 0x50acde62, SUC_lexicon_tags_85 },
    { 0xcf0cde64, SUC_lexicon_tags_11 },
    { 0xceccde65, SUC_lexicon_tags_2 },
    { 0xa8ccde63, SUC_lexicon_tags_7 },
    { 0xd8cde64, SUC_lexicon_tags_25 },
    { 0x3eacde68, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11acde6f, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28acde77, SUC_lexicon_tags_33 },
    { 0xc5acde78, SUC_lexicon_tags_29 },
    { 0x4d2cde77, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ccde7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaaacde7e, SUC_lexicon_tags_47 },
    { 0x436cde7f, SUC_lexicon_tags_29 },
    { 0x62acde80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50cde85, SUC_lexicon_tags_7 },
    { 0x23acde86, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xed6cde88, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfe8cde8a, SUC_lexicon_tags_37 },
    { 0xdc0cde8b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920cde94, SUC_lexicon_tags_12 },
    { 0x546cde95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4cde98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb9ecde9a, SUC_lexicon_tags_113 },
    { 0x21ccde9a, SUC_lexicon_tags_37 },
    { 0xe96cde9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x972cde9f, SUC_lexicon_tags_27 },
    { 0xde8cde9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54cdea3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ecdeaa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ccdeae, SUC_lexicon_tags_1 },
    { 0xd3ecdeaf, SUC_lexicon_tags_55 },
    { 0x130cdeaf, SUC_lexicon_tags_45 },
    { 0xab8cdeb1, SUC_lexicon_tags_38 },
    { 0xbe4cdeb1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0cdeb5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2dccdeb7, SUC_lexicon_tags_17 },
    { 0xb9ccdeb8, SUC_lexicon_tags_49 },
    { 0x31ccdeb7, SUC_lexicon_tags_49 },
    { 0x77acdeba, SUC_lexicon_tags_15 },
    { 0x5e2cdeb7, SUC_lexicon_tags_55 },
    { 0x9c4cdeba, SUC_lexicon_tags_1 },
    { 0x30ccdebd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x830cdebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8cdec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56ecdec3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654cdec9, SUC_lexicon_tags_0 },
    { 0xd1ecdec9, SUC_lexicon_tags_25 },
    { 0x6d8cdecb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8cded6, SUC_lexicon_tags_15 },
    { 0x862cded6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x36acdeda, SUC_lexicon_tags_25 },
    { 0x72ecdeda, SUC_lexicon_tags_10 },
    { 0x354cdedc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4cdee1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acdee5, SUC_lexicon_tags_25 },
    { 0x714cdee6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82cdeea, SUC_lexicon_tags_7 },
    { 0x774cdeeb, SUC_lexicon_tags_25 },
    { 0x41acdeec, SUC_lexicon_tags_65 },
    { 0xf70cdeed, SUC_lexicon_tags_7 },
    { 0xcf8cdeed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500cdef4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32acdef8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cdefe, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x928cdf00, SUC_lexicon_tags_25 },
    { 0x862cdf00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaacdf04, SUC_lexicon_tags_10 },
    { 0xce4cdf04, SUC_lexicon_tags_26 },
    { 0xf22cdf06, SUC_lexicon_tags_23 },
    { 0x708cdf07, SUC_lexicon_tags_12 },
    { 0x2eccdf08, SUC_lexicon_tags_25 },
    { 0xf7ccdf08, SUC_lexicon_tags_17 },
    { 0xa36cdf08, SUC_lexicon_tags_60 },
    { 0x80cdf0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58cdf10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6cdf14, SUC_lexicon_tags_11 },
    { 0x4bccdf14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2cdf18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xffecdf1b, SUC_lexicon_tags_29 },
    { 0xe04cdf1c, SUC_lexicon_tags_12 },
    { 0xe70cdf1b, SUC_lexicon_tags_44 },
    { 0x1a8cdf1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32cdf24, SUC_lexicon_tags_37 },
    { 0x970cdf24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6cdf28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe8cdf2a, SUC_lexicon_tags_0 },
    { 0x7fccdf2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326cdf2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd7ecdf31, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58cdf35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50cdf38, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddacdf3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0cdf3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ecdf44, SUC_lexicon_tags_3 },
    { 0xd4ccdf45, SUC_lexicon_tags_47 },
    { 0x95ecdf44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0cdf49, SUC_lexicon_tags_12 },
    { 0x9d8cdf49, SUC_lexicon_tags_17 },
    { 0x80ccdf4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaccdf4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ecdf51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2acdf57, SUC_lexicon_tags_31 },
    { 0x102cdf58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfecdf66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0cdf71, SUC_lexicon_tags_37 },
    { 0x7c2cdf72, SUC_lexicon_tags_66 },
    { 0x9f8cdf73, SUC_lexicon_tags_17 },
    { 0xac8cdf74, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4aacdf76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26cdf7d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x996cdf7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca8cdf82, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8cdf85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdc0cdf87, SUC_lexicon_tags_43 },
    { 0xdcecdf87, SUC_lexicon_tags_37 },
    { 0xd8cdf89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x960cdf8c, SUC_lexicon_tags_47 },
    { 0x640cdf8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ccdf92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cacdf97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e8cdf99, SUC_lexicon_tags_31 },
    { 0x54cdf9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2cdfa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x634cdfa5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50acdfa9, SUC_lexicon_tags_60 },
    { 0xe4cdfaa, SUC_lexicon_tags_36 },
    { 0x736cdfab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4cdfae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b8cdfb0, SUC_lexicon_tags_26 },
    { 0xc20cdfb1, SUC_lexicon_tags_10 },
    { 0x880cdfb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4cdfb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x420cdfb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7ccdfba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe44cdfbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x746cdfbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafecdfc5, SUC_lexicon_tags_15 },
    { 0x5eccdfc6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8facdfc8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ecdfcb, SUC_lexicon_tags_80 },
    { 0x22ccdfcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8cdfd4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ccdfda, SUC_lexicon_tags_37 },
    { 0xdd2cdfda, SUC_lexicon_tags_28 },
    { 0x4facdfdb, SUC_lexicon_tags_12 },
    { 0x2f8cdfdd, SUC_lexicon_tags_60 },
    { 0xfb0cdfde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6cdfe5, SUC_lexicon_tags_23 },
    { 0x974cdfe6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x612cdfe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350cdfec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4cdff0, SUC_lexicon_tags_12 },
    { 0x55acdff1, SUC_lexicon_tags_7 },
    { 0xc96cdff2, SUC_lexicon_tags_65 },
    { 0xfc8cdff3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdb2cdff5, SUC_lexicon_tags_17 },
    { 0x7e4cdff6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4cdff9, SUC_lexicon_tags_12 },
    { 0xc4ecdffa, SUC_lexicon_tags_1 },
    { 0xae4cdffa, SUC_lexicon_tags_25 },
    { 0xa2cdffb, SUC_lexicon_tags_14 },
    { 0x814cdffa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480ce006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14ce00e, SUC_lexicon_tags_19 },
    { 0x85ace00f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdc6ce011, SUC_lexicon_tags_7 },
    { 0x81ace012, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0ce015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89cce017, SUC_lexicon_tags_25 },
    { 0x15cce017, SUC_lexicon_tags_25 },
    { 0x414ce018, SUC_lexicon_tags_15 },
    { 0xdf4ce01a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x766ce01d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f4ce01f, SUC_lexicon_tags_37 },
    { 0xa1cce020, SUC_lexicon_tags_37 },
    { 0x652ce020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2ce029, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0ce02b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cce02f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0ece031, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd7ece033, SUC_lexicon_tags_7 },
    { 0xa98ce034, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cce037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x668ce03a, SUC_lexicon_tags_17 },
    { 0xd8cce03b, SUC_lexicon_tags_11 },
    { 0xaa8ce03c, SUC_lexicon_tags_17 },
    { 0xa04ce03d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x616ce03f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad2ce041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd8ce043, SUC_lexicon_tags_43 },
    { 0xa94ce044, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ece048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08ce04b, SUC_lexicon_tags_15 },
    { 0xc0ace04c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5ece04e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8bace050, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184ce056, SUC_lexicon_tags_42 },
    { 0xa4ece057, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd20ce059, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992ce05d, SUC_lexicon_tags_60 },
    { 0x24cce05d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6ce066, SUC_lexicon_tags_17 },
    { 0xe7ace066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x216ce06a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0ce06c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabece073, SUC_lexicon_tags_25 },
    { 0x696ce074, SUC_lexicon_tags_43 },
    { 0x5aace073, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0xaeece077, SUC_lexicon_tags_26 },
    { 0x6ce078, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cce07b, SUC_lexicon_tags_12 },
    { 0xccace07c, SUC_lexicon_tags_11 },
    { 0x6c6ce07d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bece07f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70ce086, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x404ce088, SUC_lexicon_tags_16 },
    { 0x78cce089, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacce08f, SUC_lexicon_tags_17 },
    { 0x36ace08f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ece097, SUC_lexicon_tags_17 },
    { 0xc00ce097, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcce0a4, SUC_lexicon_tags_35 },
    { 0x794ce0a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdaace0a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0ace0a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eace0ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ece0b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x376ce0b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ace0ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ace0bd, SUC_lexicon_tags_87 },
    { 0x8cace0bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa82ce0c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x952ce0c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2ce0c5, SUC_lexicon_tags_7 },
    { 0x9a0ce0c6, SUC_lexicon_tags_1 },
    { 0x8a2ce0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3ace0c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd4ace0cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8cce0cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cce0d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cce0d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafece0dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cce0e1, SUC_lexicon_tags_136 },
    { 0x71ece0e2, SUC_lexicon_tags_11 },
    { 0x3d8ce0e2, SUC_lexicon_tags_1 },
    { 0x1dece0e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6ce0e9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x980ce0eb, SUC_lexicon_tags_7 },
    { 0x6f6ce0eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x800ce0ee, SUC_lexicon_tags_35 },
    { 0x73cce0ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cce0f4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474ce0f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb2ce0fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80ece0fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b6ce0ff, SUC_lexicon_tags_151 },
    { 0xfe6ce100, SUC_lexicon_tags_33 },
    { 0xd9ace101, SUC_lexicon_tags_2 },
    { 0xe30ce102, SUC_lexicon_tags_2 },
    { 0xe96ce100, SUC_lexicon_tags_33 },
    { 0x6ccce101, SUC_lexicon_tags_45 },
    { 0xfecce101, SUC_lexicon_tags_17 },
    { 0x67ace101, SUC_lexicon_tags_47 },
    { 0x634ce106, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65ece109, SUC_lexicon_tags_15 },
    { 0x76cce10a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7acce10c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6ce10f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ce117, SUC_lexicon_tags_47 },
    { 0x9f8ce118, SUC_lexicon_tags_37 },
    { 0x1fece118, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2a2ce11b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cce123, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4ce126, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6ce129, SUC_lexicon_tags_7 },
    { 0x33cce12a, SUC_lexicon_tags_25 },
    { 0x1c6ce129, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ce12f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638ce13b, SUC_lexicon_tags_25 },
    { 0x598ce13b, SUC_lexicon_tags_27 },
    { 0x79ece13d, SUC_lexicon_tags_4 },
    { 0x802ce13e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a0ce140, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x58cce143, SUC_lexicon_tags_17 },
    { 0x7a2ce144, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31cce146, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418ce14b, SUC_lexicon_tags_8 },
    { 0xacce14c, SUC_lexicon_tags_30 },
    { 0xab6ce14b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8ce150, SUC_lexicon_tags_17 },
    { 0xb94ce151, SUC_lexicon_tags_29 },
    { 0x610ce150, SUC_lexicon_tags_32 },
    { 0x3a4ce153, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeface156, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ece15d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cce166, SUC_lexicon_tags_38 },
    { 0xfd4ce167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccce16f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x420ce171, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff8ce173, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ce178, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cce17d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8ce180, SUC_lexicon_tags_27 },
    { 0xdeace180, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf94ce183, SUC_lexicon_tags_15 },
    { 0xe56ce184, SUC_lexicon_tags_7 },
    { 0x712ce185, SUC_lexicon_tags_12 },
    { 0x2f8ce184, SUC_lexicon_tags_25 },
    { 0x4ccce187, SUC_lexicon_tags_25 },
    { 0xb9ece184, SUC_lexicon_tags_25 },
    { 0x600ce189, SUC_lexicon_tags_9 },
    { 0x88cce18a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48ce18e, SUC_lexicon_tags_38 },
    { 0x446ce18f, SUC_lexicon_tags_37 },
    { 0x950ce190, SUC_lexicon_tags_32 },
    { 0x976ce191, SUC_lexicon_tags_60 },
    { 0x71ace192, SUC_lexicon_tags_55 },
    { 0x57ece193, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24ce197, SUC_lexicon_tags_31 },
    { 0xc82ce198, SUC_lexicon_tags_25 },
    { 0xc8cce198, SUC_lexicon_tags_12 },
    { 0xbface198, SUC_lexicon_tags_15 },
    { 0xbcce19b, SUC_lexicon_tags_2 },
    { 0x212ce19b, SUC_lexicon_tags_14 },
    { 0xb94ce19b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672ce1a1, SUC_lexicon_tags_49 },
    { 0x222ce1a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x76cce1a4, SUC_lexicon_tags_2 },
    { 0xd78ce1a5, SUC_lexicon_tags_15 },
    { 0x5acce1a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x47cce1a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cce1ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcce1af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cce1b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ace1b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dace1bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bece1c2, SUC_lexicon_tags_15 },
    { 0xb3ace1c3, SUC_lexicon_tags_15 },
    { 0x596ce1c4, SUC_lexicon_tags_10 },
    { 0x4f2ce1c5, SUC_lexicon_tags_31 },
    { 0x41cce1c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1ece1c8, SUC_lexicon_tags_12 },
    { 0x8d2ce1c9, SUC_lexicon_tags_11 },
    { 0xfaace1c9, SUC_lexicon_tags_2 },
    { 0x66cce1cb, SUC_lexicon_tags_11 },
    { 0xc68ce1cb, SUC_lexicon_tags_108 },
    { 0x21ace1cd, SUC_lexicon_tags_7 },
    { 0xba0ce1cc, SUC_lexicon_tags_38 },
    { 0x930ce1cf, SUC_lexicon_tags_19 },
    { 0xaece1cd, SUC_lexicon_tags_68 },
    { 0x150ce1d1, SUC_lexicon_tags_25 },
    { 0xa8ce1c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cce1d5, SUC_lexicon_tags_0 },
    { 0x5f2ce1d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ece1d9, SUC_lexicon_tags_7 },
    { 0x4eace1d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0ace1dc, SUC_lexicon_tags_15 },
    { 0xf64ce1dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ce1e1, SUC_lexicon_tags_61 },
    { 0xa3ace1e1, SUC_lexicon_tags_27 },
    { 0x272ce1e2, SUC_lexicon_tags_38 },
    { 0xa8ece1e4, SUC_lexicon_tags_25 },
    { 0x5e4ce1e5, SUC_lexicon_tags_11 },
    { 0x98ace1e6, SUC_lexicon_tags_12 },
    { 0x10ece1e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0ce1ec, SUC_lexicon_tags_7 },
    { 0x1e6ce1ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6ce1f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4ce1f5, SUC_lexicon_tags_26 },
    { 0x478ce1f6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8ce1fb, SUC_lexicon_tags_45 },
    { 0x41cce1fb, SUC_lexicon_tags_14 },
    { 0x7cce1fd, SUC_lexicon_tags_1 },
    { 0x996ce1fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474ce205, SUC_lexicon_tags_25 },
    { 0xcd2ce206, SUC_lexicon_tags_16 },
    { 0x606ce207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x460ce209, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530ce20f, SUC_lexicon_tags_45 },
    { 0x476ce20f, SUC_lexicon_tags_12 },
    { 0x82cce20f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958ce215, SUC_lexicon_tags_45 },
    { 0xab2ce216, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbece21b, SUC_lexicon_tags_12 },
    { 0x442ce21b, SUC_lexicon_tags_15 },
    { 0xb7cce21d, SUC_lexicon_tags_7 },
    { 0x9e0ce21e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa96ce220, SUC_lexicon_tags_46 },
    { 0x664ce221, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722ce225, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe46ce227, SUC_lexicon_tags_8 },
    { 0x1f4ce228, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcece22c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4ce231, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0ce239, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xc6ce23b, SUC_lexicon_tags_2 },
    { 0x7d4ce23c, SUC_lexicon_tags_10 },
    { 0x60ace23d, SUC_lexicon_tags_74 },
    { 0x488ce23d, SUC_lexicon_tags_2 },
    { 0xe46ce23f, SUC_lexicon_tags_11 },
    { 0x9eace23f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc68ce242, SUC_lexicon_tags_17 },
    { 0x8b8ce243, SUC_lexicon_tags_27 },
    { 0xc2cce242, SUC_lexicon_tags_7 },
    { 0x764ce243, SUC_lexicon_tags_14 },
    { 0xa6ce246, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8ce249, SUC_lexicon_tags_12 },
    { 0x42cce249, SUC_lexicon_tags_15 },
    { 0xe14ce24a, SUC_lexicon_tags_21 },
    { 0xd10ce24c, SUC_lexicon_tags_23 },
    { 0x302ce24d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaece250, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cce253, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40cce255, SUC_lexicon_tags_11 },
    { 0xc7cce256, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecce25b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cce262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2ce266, SUC_lexicon_tags_57 },
    { 0xfe2ce267, SUC_lexicon_tags_26 },
    { 0xf12ce268, SUC_lexicon_tags_25 },
    { 0xb06ce269, SUC_lexicon_tags_66 },
    { 0xf62ce269, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ace26f, SUC_lexicon_tags_2 },
    { 0x2c4ce270, SUC_lexicon_tags_42 },
    { 0x692ce271, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ace276, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0x9f4ce278, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4ce27c, SUC_lexicon_tags_37 },
    { 0x72ce27c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2ce281, SUC_lexicon_tags_45 },
    { 0xa56ce282, SUC_lexicon_tags_37 },
    { 0x95cce283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x152ce286, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ece28e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x650ce290, SUC_lexicon_tags_14 },
    { 0x4c0ce290, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfd4ce293, SUC_lexicon_tags_25 },
    { 0x6a2ce293, SUC_lexicon_tags_37 },
    { 0xd76ce295, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942ce29b, SUC_lexicon_tags_17 },
    { 0xf3ece29c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cce29f, SUC_lexicon_tags_27 },
    { 0x7fece2a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7f6ce2a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78ce2a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136ce2a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522ce2ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0ace2af, SUC_lexicon_tags_28 },
    { 0x7cece2b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14ce2b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2ce2be, SUC_lexicon_tags_10 },
    { 0xc50ce2bf, SUC_lexicon_tags_9 },
    { 0xb34ce2c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf60ce2c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3ece2c4, SUC_lexicon_tags_63 },
    { 0xaeece2c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x842ce2c7, SUC_lexicon_tags_0 },
    { 0x284ce2c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4ce2cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x98ce2cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d8ce2cf, SUC_lexicon_tags_23 },
    { 0x858ce2cf, SUC_lexicon_tags_15 },
    { 0x646ce2d1, SUC_lexicon_tags_265 },
    { 0x424ce2cf, SUC_lexicon_tags_124 },
    { 0x156ce2cf, SUC_lexicon_tags_25 },
    { 0x5c6ce2d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c0ce2d6, SUC_lexicon_tags_25 },
    { 0xa4cce2d7, SUC_lexicon_tags_16 },
    { 0xd66ce2d8, SUC_lexicon_tags_38 },
    { 0xea8ce2d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90ce2de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cace2e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4ce2e7, SUC_lexicon_tags_15 },
    { 0x9b0ce2e8, SUC_lexicon_tags_30 },
    { 0x630ce2e7, SUC_lexicon_tags_11 },
    { 0x2f6ce2ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa86ce2ec, SUC_lexicon_tags_31 },
    { 0xd0ece2ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8ce2f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ace2f3, SUC_lexicon_tags_13 },
    { 0x1f6ce2f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ce2f7, SUC_lexicon_tags_2 },
    { 0xb38ce2f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dace305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6ce308, SUC_lexicon_tags_68 },
    { 0x674ce308, SUC_lexicon_tags_36 },
    { 0xb82ce30a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ace30d, SUC_lexicon_tags_2 },
    { 0x8a2ce30e, SUC_lexicon_tags_17 },
    { 0x7d6ce30e, SUC_lexicon_tags_37 },
    { 0xcc2ce310, SUC_lexicon_tags_25 },
    { 0x24ce311, SUC_lexicon_tags_30 },
    { 0xae6ce311, SUC_lexicon_tags_61 },
    { 0xad8ce312, SUC_lexicon_tags_65 },
    { 0x3aece314, SUC_lexicon_tags_37 },
    { 0x114ce314, SUC_lexicon_tags_7 },
    { 0xe8ce316, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3f4ce318, SUC_lexicon_tags_8 },
    { 0x4e2ce318, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ace31e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x27ece320, SUC_lexicon_tags_1 },
    { 0x704ce320, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a8ce323, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x802ce325, SUC_lexicon_tags_37 },
    { 0x2b8ce326, SUC_lexicon_tags_29 },
    { 0xa10ce327, SUC_lexicon_tags_2 },
    { 0x69ece328, SUC_lexicon_tags_15 },
    { 0x236ce328, SUC_lexicon_tags_38 },
    { 0x876ce32a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ace32e, SUC_lexicon_tags_12 },
    { 0x152ce32e, SUC_lexicon_tags_37 },
    { 0xda6ce32e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6ce334, SUC_lexicon_tags_179 },
    { 0xe78ce335, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2ce33b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ace341, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecce344, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82ce347, SUC_lexicon_tags_17 },
    { 0xe52ce348, SUC_lexicon_tags_62 },
    { 0x96ece348, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00ce353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312ce358, SUC_lexicon_tags_15 },
    { 0x27ece358, SUC_lexicon_tags_68 },
    { 0x8d8ce35a, SUC_lexicon_tags_25 },
    { 0xe00ce35b, SUC_lexicon_tags_37 },
    { 0x94ace35c, SUC_lexicon_tags_15 },
    { 0x6a6ce35d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982ce364, SUC_lexicon_tags_33 },
    { 0x2f2ce365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a0ce367, SUC_lexicon_tags_37 },
    { 0x4e6ce368, SUC_lexicon_tags_154 },
    { 0xccace369, SUC_lexicon_tags_15 },
    { 0xab6ce368, SUC_lexicon_tags_16 },
    { 0xa14ce36b, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dace36e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dece373, SUC_lexicon_tags_17 },
    { 0x7a8ce373, SUC_lexicon_tags_3 },
    { 0xc5ece375, SUC_lexicon_tags_1 },
    { 0x424ce376, SUC_lexicon_tags_37 },
    { 0xf4ace377, SUC_lexicon_tags_7 },
    { 0x98ace378, SUC_lexicon_tags_17 },
    { 0xa4ace379, SUC_lexicon_tags_27 },
    { 0xe92ce37a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aace37d, SUC_lexicon_tags_37 },
    { 0xe0ace37e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2ecce380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcce383, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0ce386, SUC_lexicon_tags_12 },
    { 0xae0ce387, SUC_lexicon_tags_37 },
    { 0x226ce386, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8ace38a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9ace38c, SUC_lexicon_tags_19 },
    { 0x20cce38c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x656ce390, SUC_lexicon_tags_1 },
    { 0xc2ece390, SUC_lexicon_tags_15 },
    { 0x958ce391, SUC_lexicon_tags_6 },
    { 0x838ce390, SUC_lexicon_tags_43 },
    { 0x7bece394, SUC_lexicon_tags_38 },
    { 0xb8cce392, SUC_lexicon_tags_49 },
    { 0xe80ce394, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46ace398, SUC_lexicon_tags_6 },
    { 0x4f6ce398, SUC_lexicon_tags_81 },
    { 0xc9cce39a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82ce39d, SUC_lexicon_tags_220 },
    { 0xd36ce39e, SUC_lexicon_tags_34 },
    { 0x3b8ce39f, SUC_lexicon_tags_2 },
    { 0x294ce39f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb4ce3a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ece3a9, SUC_lexicon_tags_42 },
    { 0x578ce3a9, SUC_lexicon_tags_47 },
    { 0x12ce3ab, SUC_lexicon_tags_37 },
    { 0xf80ce3ac, SUC_lexicon_tags_0 },
    { 0x796ce3ab, SUC_lexicon_tags_8 },
    { 0x5f6ce3ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba4ce3b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ece3bb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd16ce3bd, SUC_lexicon_tags_27 },
    { 0x8f8ce3be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d2ce3c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82ce3c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd4ce3c5, SUC_lexicon_tags_2 },
    { 0xc1cce3c6, SUC_lexicon_tags_23 },
    { 0xda8ce3c7, SUC_lexicon_tags_17 },
    { 0xe1ace3c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bace3cb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x810ce3ce, SUC_lexicon_tags_0 },
    { 0x278ce3cf, SUC_lexicon_tags_7 },
    { 0x3e6ce3cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48ece3d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8acce3d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcce3d8, SUC_lexicon_tags_17 },
    { 0x1ccce3d9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0ce3dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1b4ce3de, SUC_lexicon_tags_60 },
    { 0x99ace3df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982ce3e4, SUC_lexicon_tags_33 },
    { 0x6dece3e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8ce3e9, SUC_lexicon_tags_17 },
    { 0xa6cce3ea, SUC_lexicon_tags_17 },
    { 0x9cece3e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654ce3f0, SUC_lexicon_tags_37 },
    { 0xdb2ce3f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56ce3f5, SUC_lexicon_tags_16 },
    { 0xeb4ce3f5, SUC_lexicon_tags_77 },
    { 0x416ce3f5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4ce3fe, SUC_lexicon_tags_12 },
    { 0x6b4ce3fe, SUC_lexicon_tags_45 },
    { 0x4e2ce400, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cce407, SUC_lexicon_tags_19 },
    { 0xf26ce408, SUC_lexicon_tags_23 },
    { 0x504ce409, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8ce40c, SUC_lexicon_tags_2 },
    { 0xd2ece40d, SUC_lexicon_tags_30 },
    { 0xc20ce40d, SUC_lexicon_tags_28 },
    { 0x62ace40f, SUC_lexicon_tags_17 },
    { 0x908ce410, SUC_lexicon_tags_25 },
    { 0x468ce411, SUC_lexicon_tags_12 },
    { 0xa76ce410, SUC_lexicon_tags_26 },
    { 0xef2ce412, SUC_lexicon_tags_61 },
    { 0x10ace414, SUC_lexicon_tags_49 },
    { 0x772ce415, SUC_lexicon_tags_27 },
    { 0xcb6ce416, SUC_lexicon_tags_151 },
    { 0x984ce417, SUC_lexicon_tags_140 },
    { 0x438ce417, SUC_lexicon_tags_55 },
    { 0x94ce414, SUC_lexicon_tags_2 },
    { 0x55ece41a, SUC_lexicon_tags_65 },
    { 0x78ace41b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8e0ce41d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xedace420, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe12ce422, SUC_lexicon_tags_30 },
    { 0x568ce422, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x506ce426, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6ce42b, SUC_lexicon_tags_39 },
    { 0x78ace42b, SUC_lexicon_tags_1 },
    { 0x39ece42d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb98ce42f, SUC_lexicon_tags_7 },
    { 0x126ce430, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0ce438, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0ce43c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x212ce43e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb84ce440, SUC_lexicon_tags_27 },
    { 0x452ce441, SUC_lexicon_tags_1 },
    { 0xabace442, SUC_lexicon_tags_15 },
    { 0x456ce441, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x462ce446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x426ce449, SUC_lexicon_tags_7 },
    { 0xd04ce44a, SUC_lexicon_tags_11 },
    { 0x7fece44b, SUC_lexicon_tags_11 },
    { 0x72ace44c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecce451, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe32ce453, SUC_lexicon_tags_43 },
    { 0x92cce454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x910ce457, SUC_lexicon_tags_37 },
    { 0x3bace458, SUC_lexicon_tags_38 },
    { 0xf28ce459, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x168ce45c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772ce460, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce469, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04ce46c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x274ce46e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4ce473, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244ce479, SUC_lexicon_tags_34 },
    { 0x394ce479, SUC_lexicon_tags_15 },
    { 0x634ce47b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982ce487, SUC_lexicon_tags_33 },
    { 0x696ce487, SUC_lexicon_tags_70 },
    { 0x2f8ce489, SUC_lexicon_tags_2 },
    { 0xb3ece489, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ece48f, SUC_lexicon_tags_1 },
    { 0xf60ce490, SUC_lexicon_tags_15 },
    { 0xa32ce491, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90ce496, SUC_lexicon_tags_31 },
    { 0xfacce497, SUC_lexicon_tags_12 },
    { 0x6fcce496, SUC_lexicon_tags_14 },
    { 0xaf6ce499, SUC_lexicon_tags_48 },
    { 0x9e8ce49a, SUC_lexicon_tags_7 },
    { 0xce8ce497, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8cce49d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6ce49f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3a8ce4a1, SUC_lexicon_tags_39 },
    { 0x3ccce4a1, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656ce4a6, SUC_lexicon_tags_1 },
    { 0x82ece4a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17ace4a9, SUC_lexicon_tags_7 },
    { 0x314ce4aa, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xcace4ac, SUC_lexicon_tags_44 },
    { 0x860ce4ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd60ce4af, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1c2ce4b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdd8ce4b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298ce4b8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84ce4bb, SUC_lexicon_tags_1 },
    { 0xf8cce4bb, SUC_lexicon_tags_7 },
    { 0x33cce4bd, SUC_lexicon_tags_6 },
    { 0x326ce4be, SUC_lexicon_tags_14 },
    { 0x966ce4be, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x238ce4c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0ce4c7, SUC_lexicon_tags_55 },
    { 0xc52ce4c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd28ce4ca, SUC_lexicon_tags_15 },
    { 0x766ce4cb, SUC_lexicon_tags_10 },
    { 0x4dece4cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cce4d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378ce4d8, SUC_lexicon_tags_4 },
    { 0x4ace4d8, SUC_lexicon_tags_33 },
    { 0x598ce4da, SUC_lexicon_tags_29 },
    { 0x9e8ce4da, SUC_lexicon_tags_17 },
    { 0xd92ce4d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef2ce4de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x454ce4e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x536ce4e3, SUC_lexicon_tags_38 },
    { 0x76ace4e4, SUC_lexicon_tags_6 },
    { 0x17ece4e3, SUC_lexicon_tags_1 },
    { 0x526ce4e6, SUC_lexicon_tags_69 },
    { 0x2ccce4e7, SUC_lexicon_tags_49 },
    { 0xdc4ce4e7, SUC_lexicon_tags_35 },
    { 0x88ce4e9, SUC_lexicon_tags_7 },
    { 0xb86ce4e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08ce4ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ace4f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404ce4f7, SUC_lexicon_tags_43 },
    { 0xb34ce4f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x270ce4fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0ce500, SUC_lexicon_tags_14 },
    { 0x16ce501, SUC_lexicon_tags_129 },
    { 0x706ce501, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa74ce504, SUC_lexicon_tags_11 },
    { 0xf2cce504, SUC_lexicon_tags_11 },
    { 0xe38ce505, SUC_lexicon_tags_35 },
    { 0x40ace505, SUC_lexicon_tags_2 },
    { 0xd62ce508, SUC_lexicon_tags_39 },
    { 0x766ce509, SUC_lexicon_tags_37 },
    { 0xcc2ce509, SUC_lexicon_tags_31 },
    { 0x692ce50b, SUC_lexicon_tags_29 },
    { 0xd50ce50a, SUC_lexicon_tags_26 },
    { 0x348ce50d, SUC_lexicon_tags_15 },
    { 0xccce50a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2ce519, SUC_lexicon_tags_25 },
    { 0x2aece51a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x918ce51c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ace520, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x578ce522, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278ce529, SUC_lexicon_tags_11 },
    { 0x6d0ce52a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x136ce52c, SUC_lexicon_tags_15 },
    { 0x40ece52c, SUC_lexicon_tags_38 },
    { 0x64cce52d, SUC_lexicon_tags_6 },
    { 0x704ce52f, SUC_lexicon_tags_17 },
    { 0xcaece52e, SUC_lexicon_tags_15 },
    { 0x384ce531, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68ce535, SUC_lexicon_tags_25 },
    { 0x6f0ce536, SUC_lexicon_tags_8 },
    { 0x532ce536, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd6ce539, SUC_lexicon_tags_49 },
    { 0x40cce53a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ace540, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ece546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3aece548, SUC_lexicon_tags_83 },
    { 0x6a6ce549, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fece54b, SUC_lexicon_tags_27 },
    { 0xca2ce54c, SUC_lexicon_tags_17 },
    { 0x972ce54d, SUC_lexicon_tags_2 },
    { 0xfbece54e, SUC_lexicon_tags_12 },
    { 0x9ccce54c, SUC_lexicon_tags_33 },
    { 0xa7ece550, SUC_lexicon_tags_61 },
    { 0x5acce551, SUC_lexicon_tags_24 },
    { 0x7f8ce551, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc76ce554, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cece55d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50ce562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0ce567, SUC_lexicon_tags_21 },
    { 0x362ce567, SUC_lexicon_tags_55 },
    { 0x7a0ce569, SUC_lexicon_tags_55 },
    { 0x994ce56a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49cce56c, SUC_lexicon_tags_12 },
    { 0xa6ce56c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6d0ce56f, SUC_lexicon_tags_28 },
    { 0x8ce56f, SUC_lexicon_tags_25 },
    { 0x1d4ce571, SUC_lexicon_tags_66 },
    { 0xf90ce572, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eace578, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4ce57b, SUC_lexicon_tags_19 },
    { 0x8c6ce57b, SUC_lexicon_tags_37 },
    { 0x506ce57d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6ce582, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8ce585, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5e4ce587, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0ce58b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e0ce58d, SUC_lexicon_tags_43 },
    { 0x2c6ce58e, SUC_lexicon_tags_11 },
    { 0x80ace58d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x604ce591, SUC_lexicon_tags_12 },
    { 0xabcce592, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cce595, SUC_lexicon_tags_82 },
    { 0x5cace596, SUC_lexicon_tags_52 },
    { 0x5f8ce597, SUC_lexicon_tags_14 },
    { 0xb1ace597, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91ace59a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccce59d, SUC_lexicon_tags_45 },
    { 0x840ce59e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568ce5a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cce5a8, SUC_lexicon_tags_47 },
    { 0xf5cce5a8, SUC_lexicon_tags_15 },
    { 0x71ace5a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccce5ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8ce5b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0ce5b8, SUC_lexicon_tags_23 },
    { 0x96ace5b9, SUC_lexicon_tags_45 },
    { 0x890ce5ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ce5bf, SUC_lexicon_tags_49 },
    { 0x4acce5c0, SUC_lexicon_tags_63 },
    { 0x780ce5c1, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6ce5c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x266ce5c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x376ce5ca, SUC_lexicon_tags_31 },
    { 0x90ace5cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x73cce5ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc6ce5d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34ce5d4, SUC_lexicon_tags_2 },
    { 0xe20ce5d4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6ce5d9, SUC_lexicon_tags_46 },
    { 0x166ce5da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ace5de, SUC_lexicon_tags_25 },
    { 0x7b8ce5df, SUC_lexicon_tags_108 },
    { 0xd1cce5e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ece5e3, SUC_lexicon_tags_63 },
    { 0x2c0ce5e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb90ce5e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdace5e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4ce5ee, SUC_lexicon_tags_8 },
    { 0x89ece5ef, SUC_lexicon_tags_12 },
    { 0x8d6ce5ef, SUC_lexicon_tags_11 },
    { 0xd12ce5f1, SUC_lexicon_tags_1 },
    { 0x418ce5f0, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x14cce5f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8ce5f7, SUC_lexicon_tags_45 },
    { 0x488ce5f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cce5fd, SUC_lexicon_tags_9 },
    { 0x448ce5fe, SUC_lexicon_tags_37 },
    { 0x85cce5fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cce602, SUC_lexicon_tags_19 },
    { 0x450ce602, SUC_lexicon_tags_14 },
    { 0xe6cce604, SUC_lexicon_tags_7 },
    { 0x9d8ce605, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7eace607, SUC_lexicon_tags_29 },
    { 0xedcce608, SUC_lexicon_tags_0 },
    { 0xffece608, SUC_lexicon_tags_27 },
    { 0x5face60a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ce60e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a0ce610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cce618, SUC_lexicon_tags_22 },
    { 0x10ace619, SUC_lexicon_tags_17 },
    { 0x564ce61a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ece61d, SUC_lexicon_tags_6 },
    { 0x7e6ce61d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ece622, SUC_lexicon_tags_12 },
    { 0xaace622, SUC_lexicon_tags_23 },
    { 0xed0ce624, SUC_lexicon_tags_57 },
    { 0xdf0ce625, SUC_lexicon_tags_17 },
    { 0xa0ace625, SUC_lexicon_tags_25 },
    { 0x8e2ce627, SUC_lexicon_tags_21 },
    { 0xce4ce627, SUC_lexicon_tags_1 },
    { 0xc66ce627, SUC_lexicon_tags_15 },
    { 0x9e8ce62a, SUC_lexicon_tags_47 },
    { 0xb76ce62b, SUC_lexicon_tags_25 },
    { 0xbb0ce62c, SUC_lexicon_tags_17 },
    { 0x760ce62d, SUC_lexicon_tags_48 },
    { 0xf5ece62e, SUC_lexicon_tags_54 },
    { 0x172ce62e, SUC_lexicon_tags_31 },
    { 0x76ece628, SUC_lexicon_tags_37 },
    { 0xb46ce62d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ce635, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88ce639, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cace63c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x550ce63f, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28ce644, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cce649, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248ce64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe30ce64f, SUC_lexicon_tags_46 },
    { 0x798ce650, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6ce653, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x196ce655, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92ce658, SUC_lexicon_tags_8 },
    { 0xeaace658, SUC_lexicon_tags_15 },
    { 0xa98ce65a, SUC_lexicon_tags_48 },
    { 0x5acce65b, SUC_lexicon_tags_12 },
    { 0x3a4ce659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x596ce65f, SUC_lexicon_tags_17 },
    { 0x68ece65f, SUC_lexicon_tags_37 },
    { 0x358ce661, SUC_lexicon_tags_57 },
    { 0xc6ece661, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4ce665, SUC_lexicon_tags_11 },
    { 0x9b0ce666, SUC_lexicon_tags_6 },
    { 0xacce665, SUC_lexicon_tags_11 },
    { 0x3face667, SUC_lexicon_tags_6 },
    { 0xe04ce666, SUC_lexicon_tags_1 },
    { 0x54ce66a, SUC_lexicon_tags_25 },
    { 0x39ece66b, SUC_lexicon_tags_15 },
    { 0xfeace66c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688ce671, SUC_lexicon_tags_29 },
    { 0x31ace672, SUC_lexicon_tags_6 },
    { 0xc40ce673, SUC_lexicon_tags_45 },
    { 0xcfece672, SUC_lexicon_tags_38 },
    { 0x296ce675, SUC_lexicon_tags_25 },
    { 0x116ce676, SUC_lexicon_tags_2 },
    { 0x9acce672, SUC_lexicon_tags_4 },
    { 0x48ce671, SUC_lexicon_tags_3 },
    { 0x28ce679, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcce67c, SUC_lexicon_tags_25 },
    { 0xb0ece67c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6ce684, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248ce68f, SUC_lexicon_tags_25 },
    { 0x6c6ce690, SUC_lexicon_tags_47 },
    { 0xc2cce691, SUC_lexicon_tags_38 },
    { 0x3aece692, SUC_lexicon_tags_4 },
    { 0x48ace692, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecce697, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfe0ce699, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x538ce69b, SUC_lexicon_tags_30 },
    { 0xd92ce69c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce69f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccce6a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96ce6ab, SUC_lexicon_tags_8 },
    { 0x188ce6ac, SUC_lexicon_tags_30 },
    { 0x754ce6ab, SUC_lexicon_tags_12 },
    { 0x38ce6ad, SUC_lexicon_tags_33 },
    { 0xa0ce6af, SUC_lexicon_tags_12 },
    { 0x10ace6af, SUC_lexicon_tags_104 },
    { 0x80ace6b1, SUC_lexicon_tags_12 },
    { 0x882ce6ae, SUC_lexicon_tags_33 },
    { 0x184ce6b3, SUC_lexicon_tags_60 },
    { 0x48ce6b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ace6b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6ce6bd, SUC_lexicon_tags_12 },
    { 0xb48ce6be, SUC_lexicon_tags_54 },
    { 0xe12ce6bf, SUC_lexicon_tags_37 },
    { 0x4bcce6c0, SUC_lexicon_tags_43 },
    { 0x25ace6c1, SUC_lexicon_tags_19 },
    { 0x2e8ce6c0, SUC_lexicon_tags_49 },
    { 0x764ce6bd, SUC_lexicon_tags_14 },
    { 0x2dace6c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f2ce6c6, SUC_lexicon_tags_14 },
    { 0xa30ce6c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4ace6c9, SUC_lexicon_tags_60 },
    { 0x60ace6c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fece6d1, SUC_lexicon_tags_45 },
    { 0x544ce6d1, SUC_lexicon_tags_13 },
    { 0xf36ce6d1, SUC_lexicon_tags_1 },
    { 0xf68ce6d4, SUC_lexicon_tags_25 },
    { 0xd9cce6d3, SUC_lexicon_tags_67 },
    { 0xb38ce6d6, SUC_lexicon_tags_15 },
    { 0x320ce6d7, SUC_lexicon_tags_2 },
    { 0x8ace6d8, SUC_lexicon_tags_33 },
    { 0xc4cce6d8, SUC_lexicon_tags_31 },
    { 0x59ece6d7, SUC_lexicon_tags_27 },
    { 0xdf6ce6db, SUC_lexicon_tags_9 },
    { 0x94ece6dc, SUC_lexicon_tags_25 },
    { 0x23ece6dd, SUC_lexicon_tags_15 },
    { 0x61ece6dd, SUC_lexicon_tags_25 },
    { 0x612ce6df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x77ace6e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f4ce6e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cce6ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x886ce6ec, SUC_lexicon_tags_1 },
    { 0x366ce6ed, SUC_lexicon_tags_25 },
    { 0x48ece6ee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4ce6f1, SUC_lexicon_tags_25 },
    { 0x500ce6f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f8ce6f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8ce6fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cce702, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88ce706, SUC_lexicon_tags_12 },
    { 0xf16ce706, SUC_lexicon_tags_11 },
    { 0x634ce708, SUC_lexicon_tags_47 },
    { 0x1aece708, SUC_lexicon_tags_12 },
    { 0x33ece70a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6ce70e, SUC_lexicon_tags_47 },
    { 0xcd0ce70e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd90ce711, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcce714, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dece717, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x568ce719, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ace720, SUC_lexicon_tags_25 },
    { 0x4f6ce720, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486ce727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780ce72c, SUC_lexicon_tags_1 },
    { 0x5a2ce72d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bace730, SUC_lexicon_tags_2 },
    { 0x122ce731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cce734, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0ce73a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378ce741, SUC_lexicon_tags_2 },
    { 0x1f8ce742, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4ce748, SUC_lexicon_tags_9 },
    { 0xb96ce749, SUC_lexicon_tags_261 },
    { 0xc22ce749, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1c8ce74c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ace755, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0ce759, SUC_lexicon_tags_30 },
    { 0x8c0ce759, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52ce75f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa96ce761, SUC_lexicon_tags_38 },
    { 0x584ce762, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2ce76b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x676ce76e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cce773, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d4ce775, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x91cce778, SUC_lexicon_tags_37 },
    { 0xe52ce779, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eece77c, SUC_lexicon_tags_14 },
    { 0x43cce77d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904ce781, SUC_lexicon_tags_11 },
    { 0xc16ce782, SUC_lexicon_tags_7 },
    { 0xc5ece783, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x722ce786, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ece78a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88ce790, SUC_lexicon_tags_8 },
    { 0xe90ce791, SUC_lexicon_tags_13 },
    { 0xfdace792, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dece794, SUC_lexicon_tags_19 },
    { 0x6b8ce794, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x946ce797, SUC_lexicon_tags_47 },
    { 0xf52ce797, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf74ce79a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ace79e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2ce7a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4ce7a6, SUC_lexicon_tags_31 },
    { 0x6e6ce7a7, SUC_lexicon_tags_2 },
    { 0x834ce7a8, SUC_lexicon_tags_7 },
    { 0xa6ce7a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x43ece7ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x862ce7ad, SUC_lexicon_tags_11 },
    { 0xe28ce7ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242ce7b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xab4ce7bb, SUC_lexicon_tags_37 },
    { 0xc2ace7bc, SUC_lexicon_tags_39 },
    { 0x664ce7bd, SUC_lexicon_tags_17 },
    { 0x406ce7be, SUC_lexicon_tags_2 },
    { 0x8c6ce7be, SUC_lexicon_tags_8 },
    { 0xdeece7c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b2ce7c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc34ce7c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50ce7c7, SUC_lexicon_tags_40 },
    { 0x84ace7c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ce7cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ece7ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9d4ce7d0, SUC_lexicon_tags_27 },
    { 0x6bcce7d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcace7d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ace7d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24ce7dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b8ce7de, SUC_lexicon_tags_43 },
    { 0xd4ce7df, SUC_lexicon_tags_7 },
    { 0xb3ece7de, SUC_lexicon_tags_17 },
    { 0xd86ce7de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x750ce7e3, SUC_lexicon_tags_37 },
    { 0xa3ece7e4, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ece7ea, SUC_lexicon_tags_12 },
    { 0xeeece7eb, SUC_lexicon_tags_1 },
    { 0x886ce7ec, SUC_lexicon_tags_19 },
    { 0xff0ce7ed, SUC_lexicon_tags_33 },
    { 0x3a8ce7ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcece7f0, SUC_lexicon_tags_234 },
    { 0x17ace7f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8cce7f3, SUC_lexicon_tags_7 },
    { 0xac6ce7f4, SUC_lexicon_tags_31 },
    { 0xe2ce7f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bace7f8, SUC_lexicon_tags_31 },
    { 0x970ce7f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47cce7fb, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x352ce7fd, SUC_lexicon_tags_17 },
    { 0xe20ce7fe, SUC_lexicon_tags_16 },
    { 0x5dace7ff, SUC_lexicon_tags_1 },
    { 0x798ce7fd, SUC_lexicon_tags_17 },
    { 0x98cce7ff, SUC_lexicon_tags_56 },
    { 0x6e0ce7ff, SUC_lexicon_tags_11 },
    { 0x51ece803, SUC_lexicon_tags_14 },
    { 0xe8cce803, SUC_lexicon_tags_32 },
    { 0x2b4ce805, SUC_lexicon_tags_32 },
    { 0x358ce804, SUC_lexicon_tags_1 },
    { 0x81cce807, SUC_lexicon_tags_202 },
    { 0, NULL },
    { 0xb5cce809, SUC_lexicon_tags_21 },
    { 0xb5ece80a, SUC_lexicon_tags_19 },
    { 0x644ce80a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cce80e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0ce810, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ece815, SUC_lexicon_tags_12 },
    { 0xce6ce816, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62ce81c, SUC_lexicon_tags_7 },
    { 0x78cce81d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x900ce81f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x232ce821, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x578ce824, SUC_lexicon_tags_29 },
    { 0x664ce825, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ece829, SUC_lexicon_tags_7 },
    { 0xcb4ce82a, SUC_lexicon_tags_47 },
    { 0xc2cce82a, SUC_lexicon_tags_47 },
    { 0xfc2ce82c, SUC_lexicon_tags_59 },
    { 0xa84ce82d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd06ce82f, SUC_lexicon_tags_12 },
    { 0xa48ce82f, SUC_lexicon_tags_1 },
    { 0xb10ce830, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf60ce833, SUC_lexicon_tags_17 },
    { 0xe6cce833, SUC_lexicon_tags_17 },
    { 0x802ce835, SUC_lexicon_tags_11 },
    { 0x86ace836, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396ce83a, SUC_lexicon_tags_29 },
    { 0xa28ce83b, SUC_lexicon_tags_1 },
    { 0x97ace83c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x886ce83e, SUC_lexicon_tags_30 },
    { 0xdcece83e, SUC_lexicon_tags_124 },
    { 0x6b2ce840, SUC_lexicon_tags_2 },
    { 0x868ce841, SUC_lexicon_tags_11 },
    { 0xf0ace840, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x936ce844, SUC_lexicon_tags_37 },
    { 0xd4cce844, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cce848, SUC_lexicon_tags_201 },
    { 0xc32ce849, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ce84c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd0ce84e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0ce852, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4c8ce854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc40ce856, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ce859, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x498ce85c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x97ace85e, SUC_lexicon_tags_1 },
    { 0xf0ce85e, SUC_lexicon_tags_34 },
    { 0xee0ce860, SUC_lexicon_tags_15 },
    { 0xdb2ce85e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdeece863, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ce86a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce86d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x872ce870, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x262ce873, SUC_lexicon_tags_23 },
    { 0x57ace873, SUC_lexicon_tags_7 },
    { 0xfaace875, SUC_lexicon_tags_37 },
    { 0xf86ce876, SUC_lexicon_tags_12 },
    { 0x404ce873, SUC_lexicon_tags_47 },
    { 0xf82ce878, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaecce87a, SUC_lexicon_tags_19 },
    { 0xcd0ce87a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756ce883, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc10ce885, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aace888, SUC_lexicon_tags_46 },
    { 0xac8ce889, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ece88f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c2ce891, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x80cce893, SUC_lexicon_tags_38 },
    { 0xa0ce894, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaece897, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ece89a, SUC_lexicon_tags_11 },
    { 0xb78ce89b, SUC_lexicon_tags_6 },
    { 0xdface89c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d4ce89e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34ace8a0, SUC_lexicon_tags_9 },
    { 0xcdace8a0, SUC_lexicon_tags_25 },
    { 0x868ce8a2, SUC_lexicon_tags_21 },
    { 0x47ece8a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e8ce8a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x80ece8a7, SUC_lexicon_tags_17 },
    { 0x8cace8a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2ce8ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206ce8b1, SUC_lexicon_tags_56 },
    { 0xc58ce8b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0ce8b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x822ce8bb, SUC_lexicon_tags_27 },
    { 0xcfcce8bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318ce8c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd26ce8c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6ce8c6, SUC_lexicon_tags_38 },
    { 0xd6ace8c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0ce8cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2ce8ce, SUC_lexicon_tags_28 },
    { 0x504ce8cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36ece8d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ece8d5, SUC_lexicon_tags_0 },
    { 0x6c4ce8d6, SUC_lexicon_tags_9 },
    { 0x8cece8d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0ce8da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5ace8dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x142ce8de, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x612ce8e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dece8e4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552ce8e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4ce8ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x612ce8f0, SUC_lexicon_tags_26 },
    { 0xe0ce8f1, SUC_lexicon_tags_25 },
    { 0xfface8f2, SUC_lexicon_tags_47 },
    { 0xa1ece8f0, SUC_lexicon_tags_18 },
    { 0x8ece8f4, SUC_lexicon_tags_37 },
    { 0xb04ce8f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x314ce8f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x320ce8f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb26ce8fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ace8ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ce909, SUC_lexicon_tags_9 },
    { 0x8a2ce90a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddace90f, SUC_lexicon_tags_11 },
    { 0xb18ce910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ccce912, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cce915, SUC_lexicon_tags_75 },
    { 0xfbace915, SUC_lexicon_tags_40 },
    { 0x52cce917, SUC_lexicon_tags_37 },
    { 0x646ce917, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2ce91b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578ce921, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x782ce923, SUC_lexicon_tags_23 },
    { 0x49ece924, SUC_lexicon_tags_25 },
    { 0xf8ace925, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bece933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8ce937, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aece93b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x710ce93e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa34ce940, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dece944, SUC_lexicon_tags_63 },
    { 0xfd6ce945, SUC_lexicon_tags_30 },
    { 0x428ce946, SUC_lexicon_tags_37 },
    { 0x69ece947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x704ce949, SUC_lexicon_tags_23 },
    { 0xf14ce94a, SUC_lexicon_tags_11 },
    { 0xb10ce949, SUC_lexicon_tags_23 },
    { 0xd5cce94c, SUC_lexicon_tags_87 },
    { 0x6ace94c, SUC_lexicon_tags_7 },
    { 0x9f2ce949, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dace954, SUC_lexicon_tags_37 },
    { 0x16ce955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d2ce957, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe2ce959, SUC_lexicon_tags_15 },
    { 0xa0ace95a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ce95e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd04ce960, SUC_lexicon_tags_187 },
    { 0xe24ce961, SUC_lexicon_tags_38 },
    { 0x70ce962, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93ace964, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5aace966, SUC_lexicon_tags_60 },
    { 0x77ace967, SUC_lexicon_tags_14 },
    { 0x9a0ce968, SUC_lexicon_tags_7 },
    { 0xb40ce967, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6ce96d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x112ce970, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954ce977, SUC_lexicon_tags_7 },
    { 0xf0ce978, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2ce97b, SUC_lexicon_tags_47 },
    { 0x95cce97c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cce97f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3ece981, SUC_lexicon_tags_15 },
    { 0xccace982, SUC_lexicon_tags_77 },
    { 0xd94ce983, SUC_lexicon_tags_17 },
    { 0x72ece983, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x266ce986, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x968ce989, SUC_lexicon_tags_1 },
    { 0xecce989, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd8cce98c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3ace98e, SUC_lexicon_tags_37 },
    { 0xe12ce98e, SUC_lexicon_tags_18 },
    { 0x100ce98e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6eece992, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6ce996, SUC_lexicon_tags_25 },
    { 0x82ece997, SUC_lexicon_tags_60 },
    { 0x6f2ce998, SUC_lexicon_tags_8 },
    { 0x1d6ce996, SUC_lexicon_tags_15 },
    { 0xf26ce997, SUC_lexicon_tags_25 },
    { 0x676ce996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ace99f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x648ce9a2, SUC_lexicon_tags_0 },
    { 0x16ace9a3, SUC_lexicon_tags_25 },
    { 0x57ace9a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd66ce9a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5eace9a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6ce9ab, SUC_lexicon_tags_25 },
    { 0xe62ce9ac, SUC_lexicon_tags_25 },
    { 0x386ce9ad, SUC_lexicon_tags_17 },
    { 0xaa4ce9ac, SUC_lexicon_tags_32 },
    { 0xe10ce9af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2ce9b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ece9b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666ce9c1, SUC_lexicon_tags_19 },
    { 0xfc0ce9c2, SUC_lexicon_tags_3 },
    { 0x62ece9c1, SUC_lexicon_tags_84 },
    { 0x8c8ce9c4, SUC_lexicon_tags_23 },
    { 0x84ece9c5, SUC_lexicon_tags_60 },
    { 0x600ce9c5, SUC_lexicon_tags_2 },
    { 0xe7cce9c4, SUC_lexicon_tags_37 },
    { 0xcd4ce9c1, SUC_lexicon_tags_87 },
    { 0x334ce9c7, SUC_lexicon_tags_7 },
    { 0xd58ce9c4, SUC_lexicon_tags_25 },
    { 0x97cce9cb, SUC_lexicon_tags_15 },
    { 0x2dace9cc, SUC_lexicon_tags_1 },
    { 0xa50ce9cc, SUC_lexicon_tags_68 },
    { 0x57cce9ce, SUC_lexicon_tags_27 },
    { 0x256ce9cc, SUC_lexicon_tags_8 },
    { 0xcaece9cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2ce9d3, SUC_lexicon_tags_45 },
    { 0x890ce9d3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdface9d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36ce9db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ece9de, SUC_lexicon_tags_99 },
    { 0xde2ce9de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x436ce9e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ece9e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec0ce9e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeace9eb, SUC_lexicon_tags_15 },
    { 0xfb0ce9ec, SUC_lexicon_tags_37 },
    { 0x864ce9ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cce9f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe80ce9f7, SUC_lexicon_tags_60 },
    { 0xe0ce9f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902ce9fc, SUC_lexicon_tags_38 },
    { 0xe4cce9fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac8ce9ff, SUC_lexicon_tags_37 },
    { 0x1bacea00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x908cea03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdccea05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x182cea07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38cea0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x364cea0c, SUC_lexicon_tags_23 },
    { 0x6eccea0d, SUC_lexicon_tags_17 },
    { 0x4cccea0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ccea11, SUC_lexicon_tags_25 },
    { 0x402cea12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8eccea14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeccea16, SUC_lexicon_tags_23 },
    { 0x756cea17, SUC_lexicon_tags_15 },
    { 0xb5acea17, SUC_lexicon_tags_37 },
    { 0xe9ccea19, SUC_lexicon_tags_65 },
    { 0xfdccea19, SUC_lexicon_tags_65 },
    { 0xd40cea1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe0cea1d, SUC_lexicon_tags_12 },
    { 0xdfecea1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bccea22, SUC_lexicon_tags_63 },
    { 0xb50cea23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60ccea25, SUC_lexicon_tags_80 },
    { 0x1e0cea26, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x648cea28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fecea2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe76cea2e, SUC_lexicon_tags_27 },
    { 0x74cea2f, SUC_lexicon_tags_27 },
    { 0xcecea30, SUC_lexicon_tags_16 },
    { 0x6cccea31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe0cea33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdacea36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738cea3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x65acea3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ecea3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02cea46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4cea4c, SUC_lexicon_tags_38 },
    { 0x700cea4d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64cea50, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x112cea52, SUC_lexicon_tags_37 },
    { 0x154cea53, SUC_lexicon_tags_31 },
    { 0x45ccea54, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596cea5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926cea64, SUC_lexicon_tags_37 },
    { 0x4b0cea65, SUC_lexicon_tags_2 },
    { 0x4becea65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e8cea68, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cccea6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10cea73, SUC_lexicon_tags_25 },
    { 0x3a0cea73, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52cea7e, SUC_lexicon_tags_15 },
    { 0x33ccea7f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cea84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8cea87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ccea8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4cea91, SUC_lexicon_tags_17 },
    { 0xf8ecea91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208cea96, SUC_lexicon_tags_37 },
    { 0x5c2cea96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7bacea99, SUC_lexicon_tags_11 },
    { 0xb04cea9a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x278cea9d, SUC_lexicon_tags_25 },
    { 0xf10cea9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86ceaa1, SUC_lexicon_tags_120 },
    { 0xc68ceaa2, SUC_lexicon_tags_10 },
    { 0x796ceaa3, SUC_lexicon_tags_8 },
    { 0x190ceaa3, SUC_lexicon_tags_45 },
    { 0xf70ceaa5, SUC_lexicon_tags_55 },
    { 0xe42ceaa5, SUC_lexicon_tags_15 },
    { 0x808ceaa3, SUC_lexicon_tags_5 },
    { 0xfe2ceaa7, SUC_lexicon_tags_12 },
    { 0xecceaa6, SUC_lexicon_tags_2 },
    { 0x904ceaaa, SUC_lexicon_tags_15 },
    { 0x558ceaa4, SUC_lexicon_tags_15 },
    { 0xfe0ceaac, SUC_lexicon_tags_25 },
    { 0x522ceaa9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1acceaaf, SUC_lexicon_tags_8 },
    { 0x33eceaaf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4ceab6, SUC_lexicon_tags_11 },
    { 0x5a0ceab7, SUC_lexicon_tags_14 },
    { 0xa6ceab7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x678ceabb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3beceacc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aceacf, SUC_lexicon_tags_63 },
    { 0x2e8ceacf, SUC_lexicon_tags_30 },
    { 0xbe0ceacf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0acead4, SUC_lexicon_tags_127 },
    { 0x18acead5, SUC_lexicon_tags_10 },
    { 0x86acead6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178ceae3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08ceae8, SUC_lexicon_tags_2 },
    { 0x43cceae8, SUC_lexicon_tags_2 },
    { 0x6caceae8, SUC_lexicon_tags_68 },
    { 0xaf8ceae8, SUC_lexicon_tags_17 },
    { 0x366ceaec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40ceaf2, SUC_lexicon_tags_11 },
    { 0x50cceaf2, SUC_lexicon_tags_15 },
    { 0x10ceaf3, SUC_lexicon_tags_4 },
    { 0x62ceaf5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaceafc, SUC_lexicon_tags_60 },
    { 0x4aceafc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbcaceaff, SUC_lexicon_tags_49 },
    { 0x86eceaff, SUC_lexicon_tags_7 },
    { 0x1a8ceb01, SUC_lexicon_tags_17 },
    { 0x364ceb01, SUC_lexicon_tags_37 },
    { 0xb70ceb03, SUC_lexicon_tags_17 },
    { 0xaaeceb01, SUC_lexicon_tags_43 },
    { 0x590ceb05, SUC_lexicon_tags_23 },
    { 0x294ceb05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6ceb0a, SUC_lexicon_tags_37 },
    { 0xa2eceb0b, SUC_lexicon_tags_23 },
    { 0x24eceb0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbaeceb0e, SUC_lexicon_tags_87 },
    { 0x89aceb0f, SUC_lexicon_tags_50 },
    { 0xc96ceb0f, SUC_lexicon_tags_27 },
    { 0xe2cceb0f, SUC_lexicon_tags_12 },
    { 0xf8cceb11, SUC_lexicon_tags_63 },
    { 0xbecceb13, SUC_lexicon_tags_46 },
    { 0x20ceb14, SUC_lexicon_tags_23 },
    { 0x608ceb15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22ceb18, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884ceb20, SUC_lexicon_tags_10 },
    { 0x754ceb21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1feceb26, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x870ceb28, SUC_lexicon_tags_19 },
    { 0x448ceb29, SUC_lexicon_tags_49 },
    { 0x18cceb28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2ceb30, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x450ceb33, SUC_lexicon_tags_27 },
    { 0x360ceb34, SUC_lexicon_tags_25 },
    { 0x896ceb35, SUC_lexicon_tags_12 },
    { 0x578ceb36, SUC_lexicon_tags_15 },
    { 0xb28ceb37, SUC_lexicon_tags_14 },
    { 0xa70ceb37, SUC_lexicon_tags_37 },
    { 0xb8cceb39, SUC_lexicon_tags_11 },
    { 0x790ceb39, SUC_lexicon_tags_0 },
    { 0x960ceb38, SUC_lexicon_tags_23 },
    { 0x378ceb37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4ceb3f, SUC_lexicon_tags_2 },
    { 0xa1eceb3f, SUC_lexicon_tags_17 },
    { 0xd64ceb40, SUC_lexicon_tags_2 },
    { 0x69eceb42, SUC_lexicon_tags_32 },
    { 0x464ceb43, SUC_lexicon_tags_65 },
    { 0x2a8ceb41, SUC_lexicon_tags_19 },
    { 0xa68ceb45, SUC_lexicon_tags_17 },
    { 0x796ceb46, SUC_lexicon_tags_87 },
    { 0xa80ceb46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf98ceb49, SUC_lexicon_tags_27 },
    { 0xc2eceb4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50ceb4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8ceb4f, SUC_lexicon_tags_28 },
    { 0xa90ceb50, SUC_lexicon_tags_11 },
    { 0xd2ceb51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecceb54, SUC_lexicon_tags_27 },
    { 0xc12ceb55, SUC_lexicon_tags_23 },
    { 0xb40ceb56, SUC_lexicon_tags_47 },
    { 0x9beceb57, SUC_lexicon_tags_37 },
    { 0xa04ceb58, SUC_lexicon_tags_7 },
    { 0x958ceb59, SUC_lexicon_tags_6 },
    { 0x366ceb5a, SUC_lexicon_tags_38 },
    { 0x288ceb5b, SUC_lexicon_tags_26 },
    { 0xe20ceb5c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x318ceb5e, SUC_lexicon_tags_31 },
    { 0xf08ceb5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cceb62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6ceb68, SUC_lexicon_tags_33 },
    { 0xd3cceb69, SUC_lexicon_tags_38 },
    { 0x14cceb68, SUC_lexicon_tags_7 },
    { 0x7b8ceb6b, SUC_lexicon_tags_9 },
    { 0xa0ceb6a, SUC_lexicon_tags_6 },
    { 0x40cceb6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7caceb71, SUC_lexicon_tags_64 },
    { 0x92cceb72, SUC_lexicon_tags_45 },
    { 0x6cceb71, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52ceb77, SUC_lexicon_tags_19 },
    { 0x416ceb78, SUC_lexicon_tags_87 },
    { 0x1aceb79, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x430ceb7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddaceb7e, SUC_lexicon_tags_82 },
    { 0xccceb7f, SUC_lexicon_tags_25 },
    { 0xa36ceb7e, SUC_lexicon_tags_2 },
    { 0xc86ceb7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52ceb84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x51cceb86, SUC_lexicon_tags_12 },
    { 0xac0ceb87, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ceceb8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa74ceb8e, SUC_lexicon_tags_56 },
    { 0x478ceb8f, SUC_lexicon_tags_56 },
    { 0xad2ceb8f, SUC_lexicon_tags_37 },
    { 0x40aceb8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0ceb97, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88ceba0, SUC_lexicon_tags_37 },
    { 0x1c8ceba1, SUC_lexicon_tags_27 },
    { 0xb06ceba2, SUC_lexicon_tags_70 },
    { 0xf6aceba1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2ceba6, SUC_lexicon_tags_11 },
    { 0x3ccceba7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x910ceba9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292cebb1, SUC_lexicon_tags_15 },
    { 0x824cebb1, SUC_lexicon_tags_7 },
    { 0x40acebb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x164cebb5, SUC_lexicon_tags_2 },
    { 0x59ecebb6, SUC_lexicon_tags_2 },
    { 0x4d8cebb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1becebbc, SUC_lexicon_tags_7 },
    { 0x978cebbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606cebc1, SUC_lexicon_tags_12 },
    { 0x172cebc2, SUC_lexicon_tags_6 },
    { 0xb26cebc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbacebc7, SUC_lexicon_tags_45 },
    { 0xd4ccebc8, SUC_lexicon_tags_38 },
    { 0x37ecebc9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0cebcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1acebcf, SUC_lexicon_tags_28 },
    { 0x310cebd0, SUC_lexicon_tags_15 },
    { 0x3c4cebd1, SUC_lexicon_tags_37 },
    { 0xf44cebd2, SUC_lexicon_tags_21 },
    { 0x826cebd3, SUC_lexicon_tags_16 },
    { 0xddecebd4, SUC_lexicon_tags_10 },
    { 0xa62cebd4, SUC_lexicon_tags_15 },
    { 0x61acebd1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60cebdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf44cebdd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb2ecebdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5acebe8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ccebed, SUC_lexicon_tags_12 },
    { 0xb2ecebee, SUC_lexicon_tags_12 },
    { 0x69ccebee, SUC_lexicon_tags_37 },
    { 0x5b6cebef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0cebf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfecebfc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40ecebfe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe9ccec00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fecec03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2cec09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34cec0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08cec15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2ccec17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2cec1c, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45acec21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11acec23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5acec2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ecec31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x662cec33, SUC_lexicon_tags_56 },
    { 0x74acec33, SUC_lexicon_tags_87 },
    { 0xcccec33, SUC_lexicon_tags_65 },
    { 0xd34cec36, SUC_lexicon_tags_63 },
    { 0x1facec37, SUC_lexicon_tags_91 },
    { 0xf6cec37, SUC_lexicon_tags_8 },
    { 0x45ccec38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdecec3b, SUC_lexicon_tags_11 },
    { 0xfe8cec3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2cec41, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeccec45, SUC_lexicon_tags_15 },
    { 0x65ccec46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0cec4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3acec4c, SUC_lexicon_tags_30 },
    { 0x61ecec4c, SUC_lexicon_tags_79 },
    { 0x3fccec4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2cec52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846cec56, SUC_lexicon_tags_17 },
    { 0xd92cec57, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6cec5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82cec60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8acec64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aecec66, SUC_lexicon_tags_37 },
    { 0x258cec67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44cec72, SUC_lexicon_tags_1 },
    { 0xed6cec73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596cec77, SUC_lexicon_tags_0 },
    { 0x41ecec78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ecec84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x778cec87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x176cec89, SUC_lexicon_tags_37 },
    { 0x54acec89, SUC_lexicon_tags_33 },
    { 0x1dacec8b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe20cec8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ecec91, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xed2cec93, SUC_lexicon_tags_17 },
    { 0xedacec94, SUC_lexicon_tags_0 },
    { 0x7f0cec94, SUC_lexicon_tags_43 },
    { 0x218cec96, SUC_lexicon_tags_64 },
    { 0x1b8cec97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29acec99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd26cec9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5fecec9d, SUC_lexicon_tags_0 },
    { 0xf82cec9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998ceca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ceca9, SUC_lexicon_tags_15 },
    { 0x9e6cecaa, SUC_lexicon_tags_49 },
    { 0xe52cecab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x112cecae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2cecb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x474cecb3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa66cecb5, SUC_lexicon_tags_1 },
    { 0x26acecb6, SUC_lexicon_tags_65 },
    { 0x4a8cecb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2cecbb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ececbf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270cecc3, SUC_lexicon_tags_27 },
    { 0xe32cecc4, SUC_lexicon_tags_15 },
    { 0x93ccecc3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0cecce, SUC_lexicon_tags_10 },
    { 0x9a2ceccf, SUC_lexicon_tags_7 },
    { 0x69acecd0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecccecd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf8cecda, SUC_lexicon_tags_17 },
    { 0xe1ececda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540cecdf, SUC_lexicon_tags_25 },
    { 0x8a4cecdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8cece3, SUC_lexicon_tags_14 },
    { 0x82cece3, SUC_lexicon_tags_17 },
    { 0x6a4cece3, SUC_lexicon_tags_49 },
    { 0x70ecece5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ececea, SUC_lexicon_tags_2 },
    { 0x1b2ceceb, SUC_lexicon_tags_23 },
    { 0x3accecec, SUC_lexicon_tags_1 },
    { 0xb5acecea, SUC_lexicon_tags_27 },
    { 0xb46cecee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc4cecf0, SUC_lexicon_tags_107 },
    { 0x380cecf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8cecf4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x886cecf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf16cecf8, SUC_lexicon_tags_17 },
    { 0x628cecf8, SUC_lexicon_tags_60 },
    { 0x380cecfa, SUC_lexicon_tags_11 },
    { 0xb8ccecfb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50ced01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4daced04, SUC_lexicon_tags_6 },
    { 0xb6ced05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6ced0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cced0e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710ced12, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2ced18, SUC_lexicon_tags_0 },
    { 0xf54ced18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2ced20, SUC_lexicon_tags_25 },
    { 0xc20ced21, SUC_lexicon_tags_19 },
    { 0x6eced22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498ced27, SUC_lexicon_tags_31 },
    { 0xbecced28, SUC_lexicon_tags_43 },
    { 0x4eaced29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0ced2c, SUC_lexicon_tags_21 },
    { 0x774ced2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6dcced2f, SUC_lexicon_tags_25 },
    { 0x1dcced30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc60ced32, SUC_lexicon_tags_37 },
    { 0x28ced32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eced3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888ced43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x150ced46, SUC_lexicon_tags_2 },
    { 0xd30ced47, SUC_lexicon_tags_12 },
    { 0x6c8ced48, SUC_lexicon_tags_17 },
    { 0x1f2ced48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b4ced4b, SUC_lexicon_tags_68 },
    { 0x536ced4c, SUC_lexicon_tags_11 },
    { 0xe3aced4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cced53, SUC_lexicon_tags_9 },
    { 0xb90ced54, SUC_lexicon_tags_37 },
    { 0xfecced55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eeced5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa20ced5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cced63, SUC_lexicon_tags_15 },
    { 0x9acced64, SUC_lexicon_tags_12 },
    { 0x198ced65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x190ced67, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x216ced69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2ced6c, SUC_lexicon_tags_2 },
    { 0x3d6ced6d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd6eced6f, SUC_lexicon_tags_7 },
    { 0x3aaced70, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdeaced72, SUC_lexicon_tags_25 },
    { 0x2e4ced73, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8ced76, SUC_lexicon_tags_1 },
    { 0xddaced76, SUC_lexicon_tags_26 },
    { 0x1fcced77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b2ced7a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ced80, SUC_lexicon_tags_0 },
    { 0x4aeced80, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cced85, SUC_lexicon_tags_0 },
    { 0x150ced86, SUC_lexicon_tags_9 },
    { 0x8feced85, SUC_lexicon_tags_34 },
    { 0xe6aced88, SUC_lexicon_tags_1 },
    { 0xf9cced87, SUC_lexicon_tags_55 },
    { 0x9feced8a, SUC_lexicon_tags_35 },
    { 0x8caced8b, SUC_lexicon_tags_60 },
    { 0x24aced8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8ced91, SUC_lexicon_tags_17 },
    { 0x350ced92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7baced94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73eced96, SUC_lexicon_tags_12 },
    { 0x14aced97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x838ced99, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdd8ced9b, SUC_lexicon_tags_7 },
    { 0x86ced9c, SUC_lexicon_tags_33 },
    { 0x94eced9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6ceda0, SUC_lexicon_tags_19 },
    { 0x9cceda1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90ceda5, SUC_lexicon_tags_14 },
    { 0xb48ceda6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf26ceda8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10cedac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bccedb1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570cedb6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4becedba, SUC_lexicon_tags_15 },
    { 0xd1acedbb, SUC_lexicon_tags_47 },
    { 0x56ccedba, SUC_lexicon_tags_37 },
    { 0xcb4cedbd, SUC_lexicon_tags_15 },
    { 0x1d6cedbe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x574cedc0, SUC_lexicon_tags_38 },
    { 0x738cedc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36acedc6, SUC_lexicon_tags_49 },
    { 0xf90cedc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2cedcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeccedce, SUC_lexicon_tags_15 },
    { 0x912cedcf, SUC_lexicon_tags_33 },
    { 0xaeacedd0, SUC_lexicon_tags_15 },
    { 0x3f0cedd0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2cedd4, SUC_lexicon_tags_38 },
    { 0xd64cedd4, SUC_lexicon_tags_1 },
    { 0xf56cedd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x830cedd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9facede0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd02cede2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4cede5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ccede8, SUC_lexicon_tags_25 },
    { 0xbe0cede8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dccedec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58cedef, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66cedf2, SUC_lexicon_tags_6 },
    { 0xedacedf3, SUC_lexicon_tags_7 },
    { 0xceccedf3, SUC_lexicon_tags_60 },
    { 0x840cedf2, SUC_lexicon_tags_68 },
    { 0x1f4cedf2, SUC_lexicon_tags_38 },
    { 0x962cedf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ecedfb, SUC_lexicon_tags_7 },
    { 0x4cedfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56cee00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x76ecee02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62cee05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe96cee07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0cee0d, SUC_lexicon_tags_25 },
    { 0xacecee0e, SUC_lexicon_tags_17 },
    { 0x524cee0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7acee12, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xef4cee14, SUC_lexicon_tags_16 },
    { 0xebccee15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x114cee17, SUC_lexicon_tags_16 },
    { 0x5d6cee18, SUC_lexicon_tags_57 },
    { 0x44ecee17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x248cee1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcacee1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ccee21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ecee25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6cee2d, SUC_lexicon_tags_15 },
    { 0x7aacee2d, SUC_lexicon_tags_38 },
    { 0x27ecee2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69acee33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388cee39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x630cee3b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb1ecee3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x630cee40, SUC_lexicon_tags_4 },
    { 0x57ccee41, SUC_lexicon_tags_47 },
    { 0x95ecee40, SUC_lexicon_tags_17 },
    { 0x8c0cee43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x590cee46, SUC_lexicon_tags_84 },
    { 0x7eecee46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x436cee49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890cee4d, SUC_lexicon_tags_33 },
    { 0xf74cee4e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd4acee50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x862cee52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xceccee54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820cee5b, SUC_lexicon_tags_79 },
    { 0x3e2cee5b, SUC_lexicon_tags_2 },
    { 0x784cee5d, SUC_lexicon_tags_66 },
    { 0xa8acee5e, SUC_lexicon_tags_0 },
    { 0x7e8cee5f, SUC_lexicon_tags_4 },
    { 0xaa4cee60, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0x956cee62, SUC_lexicon_tags_2 },
    { 0xaaecee63, SUC_lexicon_tags_27 },
    { 0x628cee64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x304cee66, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94acee6d, SUC_lexicon_tags_37 },
    { 0xbe8cee6e, SUC_lexicon_tags_38 },
    { 0x4d8cee6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x782cee72, SUC_lexicon_tags_11 },
    { 0x182cee73, SUC_lexicon_tags_32 },
    { 0xfa8cee72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2cee7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc0acee7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x650cee81, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794cee85, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4cee89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x720cee8b, SUC_lexicon_tags_5 },
    { 0x69ccee8c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cee90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee8cee92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282cee9e, SUC_lexicon_tags_29 },
    { 0xce2cee9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0ceea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9eceea8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x290ceeab, SUC_lexicon_tags_13 },
    { 0x1b4ceeac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6ceeb3, SUC_lexicon_tags_25 },
    { 0x27aceeb4, SUC_lexicon_tags_1 },
    { 0x4beceeb5, SUC_lexicon_tags_9 },
    { 0x5caceeb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268ceebf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40aceec8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa52ceeca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x41cceecd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x92eceecf, SUC_lexicon_tags_17 },
    { 0xf78ceed0, SUC_lexicon_tags_9 },
    { 0x43cceecf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6ceed9, SUC_lexicon_tags_25 },
    { 0xeaaceeda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaceee0, SUC_lexicon_tags_17 },
    { 0xe34ceee0, SUC_lexicon_tags_1555 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16eceee5, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xcaaceee7, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x5faceeea, SUC_lexicon_tags_33 },
    { 0x862ceeeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844ceef3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438ceef8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994ceefe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fccef01, SUC_lexicon_tags_16 },
    { 0x1b8cef02, SUC_lexicon_tags_25 },
    { 0x200cef03, SUC_lexicon_tags_15 },
    { 0x320cef03, SUC_lexicon_tags_47 },
    { 0xa72cef02, SUC_lexicon_tags_47 },
    { 0xc28cef06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8cef08, SUC_lexicon_tags_898 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2cef0b, SUC_lexicon_tags_48 },
    { 0x95ecef0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a2cef0e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xdbacef10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc94cef12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7acef14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x954cef17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622cef1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb74cef1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10acef1f, SUC_lexicon_tags_2 },
    { 0x2acef20, SUC_lexicon_tags_53 },
    { 0xe74cef21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cef29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa76cef2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02cef31, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2cef37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda6cef39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4cef3c, SUC_lexicon_tags_15 },
    { 0x5e6cef3d, SUC_lexicon_tags_47 },
    { 0xf48cef3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc60cef40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488cef47, SUC_lexicon_tags_1 },
    { 0x77acef48, SUC_lexicon_tags_39 },
    { 0xcd8cef48, SUC_lexicon_tags_17 },
    { 0x2ecef4a, SUC_lexicon_tags_11 },
    { 0xb58cef4b, SUC_lexicon_tags_38 },
    { 0xc4cef48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe10cef4e, SUC_lexicon_tags_2 },
    { 0x4d4cef4e, SUC_lexicon_tags_7 },
    { 0xffccef50, SUC_lexicon_tags_15 },
    { 0x882cef4f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806cef5e, SUC_lexicon_tags_29 },
    { 0x67ccef5f, SUC_lexicon_tags_12 },
    { 0x49ccef60, SUC_lexicon_tags_12 },
    { 0x590cef61, SUC_lexicon_tags_37 },
    { 0x158cef60, SUC_lexicon_tags_63 },
    { 0xf24cef5f, SUC_lexicon_tags_14 },
    { 0x38ecef5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6cef66, SUC_lexicon_tags_68 },
    { 0xc92cef67, SUC_lexicon_tags_37 },
    { 0x7f6cef66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f2cef6a, SUC_lexicon_tags_13 },
    { 0x828cef6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44cef6d, SUC_lexicon_tags_1521 },
    { 0x814cef6e, SUC_lexicon_tags_38 },
    { 0x250cef6e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51ccef71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd06cef73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8cef81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ecef85, SUC_lexicon_tags_56 },
    { 0x884cef85, SUC_lexicon_tags_146 },
    { 0xfbecef87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x852cef89, SUC_lexicon_tags_28 },
    { 0x780cef8a, SUC_lexicon_tags_17 },
    { 0x5d0cef8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x540cef8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4acef92, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccef96, SUC_lexicon_tags_17 },
    { 0xb80cef96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b2cef99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccef9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfeecef9f, SUC_lexicon_tags_9 },
    { 0x116cefa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8cefa3, SUC_lexicon_tags_7 },
    { 0xcd8cefa4, SUC_lexicon_tags_0 },
    { 0x11ecefa4, SUC_lexicon_tags_17 },
    { 0xa94cefa6, SUC_lexicon_tags_29 },
    { 0xaacefa5, SUC_lexicon_tags_26 },
    { 0xa36cefa8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x15acefaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafacefac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ecefb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ccefb7, SUC_lexicon_tags_33 },
    { 0x20ccefb8, SUC_lexicon_tags_25 },
    { 0x9accefb7, SUC_lexicon_tags_39 },
    { 0x70ccefb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94cefbd, SUC_lexicon_tags_19 },
    { 0xe74cefbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526cefc4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd98cefc6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8a0cefc8, SUC_lexicon_tags_28 },
    { 0x1e8cefc9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40cefcd, SUC_lexicon_tags_17 },
    { 0x3e0cefce, SUC_lexicon_tags_0 },
    { 0x36ccefcd, SUC_lexicon_tags_37 },
    { 0x1ecefcd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42cefd3, SUC_lexicon_tags_17 },
    { 0xc3acefd3, SUC_lexicon_tags_23 },
    { 0x302cefd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29acefd7, SUC_lexicon_tags_11 },
    { 0x5b0cefd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6cefde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc98cefe0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x38ccefe2, SUC_lexicon_tags_25 },
    { 0xd58cefe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90acefe6, SUC_lexicon_tags_19 },
    { 0x908cefe7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626cefed, SUC_lexicon_tags_14 },
    { 0xf80cefee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8ceff5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6aceff7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2ceffb, SUC_lexicon_tags_11 },
    { 0xdbaceffb, SUC_lexicon_tags_37 },
    { 0xe8aceffb, SUC_lexicon_tags_37 },
    { 0xb76ceffc, SUC_lexicon_tags_9 },
    { 0x2dcceffb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96cf003, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79ecf005, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x28ccf007, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6cf00b, SUC_lexicon_tags_13 },
    { 0x400cf00c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdecf011, SUC_lexicon_tags_82 },
    { 0x214cf012, SUC_lexicon_tags_43 },
    { 0x3e0cf013, SUC_lexicon_tags_82 },
    { 0xe94cf013, SUC_lexicon_tags_545 },
    { 0xc6ccf013, SUC_lexicon_tags_7 },
    { 0x786cf016, SUC_lexicon_tags_30 },
    { 0xd06cf017, SUC_lexicon_tags_45 },
    { 0x924cf014, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aecf01d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x642cf01f, SUC_lexicon_tags_15 },
    { 0xd84cf020, SUC_lexicon_tags_37 },
    { 0xa58cf021, SUC_lexicon_tags_15 },
    { 0x990cf022, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x126cf024, SUC_lexicon_tags_11 },
    { 0xb9ecf025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x446cf028, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ecf02c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bacf030, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26cf033, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ecf038, SUC_lexicon_tags_43 },
    { 0xee6cf039, SUC_lexicon_tags_1 },
    { 0xb16cf03a, SUC_lexicon_tags_17 },
    { 0xb44cf03a, SUC_lexicon_tags_17 },
    { 0x808cf03b, SUC_lexicon_tags_7 },
    { 0x7aacf03d, SUC_lexicon_tags_55 },
    { 0xcacf03e, SUC_lexicon_tags_11 },
    { 0xeaecf03e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ecf043, SUC_lexicon_tags_10 },
    { 0x366cf044, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120cf04d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x184cf04f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3decf054, SUC_lexicon_tags_25 },
    { 0x658cf055, SUC_lexicon_tags_27 },
    { 0xbc2cf056, SUC_lexicon_tags_47 },
    { 0x8b4cf057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc82cf059, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x420cf05b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90cf05f, SUC_lexicon_tags_6 },
    { 0x2f8cf060, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328cf066, SUC_lexicon_tags_8 },
    { 0xb4ecf066, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ecf06d, SUC_lexicon_tags_68 },
    { 0xb00cf06e, SUC_lexicon_tags_27 },
    { 0xf28cf06e, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x306cf072, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71acf074, SUC_lexicon_tags_11 },
    { 0x986cf074, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x868cf078, SUC_lexicon_tags_26 },
    { 0xa60cf079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecacf07b, SUC_lexicon_tags_2 },
    { 0x424cf07b, SUC_lexicon_tags_63 },
    { 0xf2cf07d, SUC_lexicon_tags_12 },
    { 0x35ccf07d, SUC_lexicon_tags_9 },
    { 0x88acf07d, SUC_lexicon_tags_38 },
    { 0x654cf07f, SUC_lexicon_tags_49 },
    { 0xf4ccf07b, SUC_lexicon_tags_47 },
    { 0x2a6cf082, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42cf087, SUC_lexicon_tags_17 },
    { 0x68cf088, SUC_lexicon_tags_27 },
    { 0xab8cf089, SUC_lexicon_tags_11 },
    { 0x44ccf08a, SUC_lexicon_tags_108 },
    { 0xa5acf08b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd8cf08d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2acf090, SUC_lexicon_tags_17 },
    { 0x86ecf090, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eecf096, SUC_lexicon_tags_37 },
    { 0xeaacf097, SUC_lexicon_tags_17 },
    { 0x870cf096, SUC_lexicon_tags_17 },
    { 0x22acf099, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8cf09f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6cf0a4, SUC_lexicon_tags_9 },
    { 0x904cf0a4, SUC_lexicon_tags_33 },
    { 0xdfacf0a4, SUC_lexicon_tags_19 },
    { 0xb00cf0a7, SUC_lexicon_tags_17 },
    { 0xac8cf0a7, SUC_lexicon_tags_11 },
    { 0xa4ecf0a9, SUC_lexicon_tags_65 },
    { 0x9c4cf0a5, SUC_lexicon_tags_23 },
    { 0x998cf0a5, SUC_lexicon_tags_47 },
    { 0x23acf0ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50acf0ae, SUC_lexicon_tags_10 },
    { 0x672cf0ae, SUC_lexicon_tags_25 },
    { 0x39ecf0b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8cf0b4, SUC_lexicon_tags_37 },
    { 0x81ccf0b5, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cf0c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850cf0c5, SUC_lexicon_tags_7 },
    { 0x9eecf0c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x412cf0c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64cf0cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2cf0d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x194cf0d4, SUC_lexicon_tags_18 },
    { 0x666cf0d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6cf0dc, SUC_lexicon_tags_16 },
    { 0xc06cf0dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xffecf0df, SUC_lexicon_tags_1 },
    { 0x4c6cf0e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe62cf0e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eecf0e4, SUC_lexicon_tags_25 },
    { 0x43ecf0e4, SUC_lexicon_tags_37 },
    { 0x1d8cf0e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6cf0e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24cf0ec, SUC_lexicon_tags_12 },
    { 0xf9ccf0ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4acf0ef, SUC_lexicon_tags_14 },
    { 0x530cf0f0, SUC_lexicon_tags_27 },
    { 0x45acf0f1, SUC_lexicon_tags_113 },
    { 0x89ccf0f1, SUC_lexicon_tags_2 },
    { 0x872cf0f3, SUC_lexicon_tags_25 },
    { 0xeb4cf0f4, SUC_lexicon_tags_116 },
    { 0xdcecf0f4, SUC_lexicon_tags_15 },
    { 0x1b0cf0f3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ecf0fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e6cf0fc, SUC_lexicon_tags_15 },
    { 0x91ccf0fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcacf0ff, SUC_lexicon_tags_35 },
    { 0xb2cf100, SUC_lexicon_tags_21 },
    { 0xe6ecf0ff, SUC_lexicon_tags_47 },
    { 0x3dccf102, SUC_lexicon_tags_36 },
    { 0x678cf102, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x666cf106, SUC_lexicon_tags_43 },
    { 0xe96cf106, SUC_lexicon_tags_15 },
    { 0x95acf108, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d6cf10a, SUC_lexicon_tags_15 },
    { 0x2c2cf10b, SUC_lexicon_tags_11 },
    { 0x56ecf10b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0cf10f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1aecf111, SUC_lexicon_tags_26 },
    { 0x19ecf112, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x636cf115, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a0cf117, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ecf11a, SUC_lexicon_tags_47 },
    { 0x82cf11b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddecf121, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe10cf123, SUC_lexicon_tags_54 },
    { 0x798cf124, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe22cf126, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x994cf128, SUC_lexicon_tags_12 },
    { 0x978cf128, SUC_lexicon_tags_7 },
    { 0x302cf12a, SUC_lexicon_tags_16 },
    { 0x1a8cf12b, SUC_lexicon_tags_61 },
    { 0x6a6cf129, SUC_lexicon_tags_146 },
    { 0x1d4cf12d, SUC_lexicon_tags_6 },
    { 0x9acf12c, SUC_lexicon_tags_15 },
    { 0xe64cf12f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ccf134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x382cf136, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ccf13f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c8cf141, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22ecf143, SUC_lexicon_tags_38 },
    { 0x5d0cf144, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbccf146, SUC_lexicon_tags_46 },
    { 0xa86cf146, SUC_lexicon_tags_17 },
    { 0x80ecf148, SUC_lexicon_tags_82 },
    { 0x6f4cf148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccecf14f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970cf153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3acf155, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842cf159, SUC_lexicon_tags_12 },
    { 0x410cf159, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ccf15e, SUC_lexicon_tags_25 },
    { 0x9f6cf15f, SUC_lexicon_tags_19 },
    { 0xdccf160, SUC_lexicon_tags_37 },
    { 0xd12cf161, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578cf165, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4cf16c, SUC_lexicon_tags_11 },
    { 0xe5ecf16d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10acf171, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xefccf173, SUC_lexicon_tags_2 },
    { 0x990cf174, SUC_lexicon_tags_0 },
    { 0xe76cf175, SUC_lexicon_tags_66 },
    { 0x75ccf175, SUC_lexicon_tags_26 },
    { 0xa36cf176, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5ccf179, SUC_lexicon_tags_43 },
    { 0x7becf179, SUC_lexicon_tags_1 },
    { 0x978cf17b, SUC_lexicon_tags_15 },
    { 0x8f4cf17c, SUC_lexicon_tags_1 },
    { 0xcaacf179, SUC_lexicon_tags_12 },
    { 0xb9acf17d, SUC_lexicon_tags_25 },
    { 0x3d2cf17c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf4cf181, SUC_lexicon_tags_8 },
    { 0x356cf182, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940cf18b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84cf18f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212cf194, SUC_lexicon_tags_74 },
    { 0x932cf195, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3facf197, SUC_lexicon_tags_17 },
    { 0x54acf198, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398cf19c, SUC_lexicon_tags_29 },
    { 0xbaccf19d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x358cf1a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938cf1a5, SUC_lexicon_tags_11 },
    { 0x278cf1a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x656cf1a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x744cf1ac, SUC_lexicon_tags_7 },
    { 0xf80cf1ac, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1d6cf1af, SUC_lexicon_tags_47 },
    { 0xb1ccf1b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaacf1b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ecf1ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6cf1bd, SUC_lexicon_tags_1 },
    { 0x4d4cf1be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3acf1c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38cf1c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78cf1cb, SUC_lexicon_tags_15 },
    { 0xae2cf1cc, SUC_lexicon_tags_2 },
    { 0xc9acf1cd, SUC_lexicon_tags_17 },
    { 0xef6cf1ce, SUC_lexicon_tags_0 },
    { 0x2ccf1cf, SUC_lexicon_tags_1 },
    { 0xf7ccf1cc, SUC_lexicon_tags_16 },
    { 0x41ccf1cf, SUC_lexicon_tags_2 },
    { 0xca8cf1d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20cf1d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1dccf1d9, SUC_lexicon_tags_9 },
    { 0x344cf1da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed6cf1dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4cf1df, SUC_lexicon_tags_8 },
    { 0x37ecf1e0, SUC_lexicon_tags_0 },
    { 0x52cf1e1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0cf1ea, SUC_lexicon_tags_28 },
    { 0x64ecf1eb, SUC_lexicon_tags_12 },
    { 0xdb4cf1eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd70cf1ee, SUC_lexicon_tags_29 },
    { 0x948cf1ef, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cf1f3, SUC_lexicon_tags_45 },
    { 0x1b8cf1f3, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x414cf1f6, SUC_lexicon_tags_0 },
    { 0xe7acf1f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x432cf1f9, SUC_lexicon_tags_28 },
    { 0x58acf1f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff6cf1fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6cf1ff, SUC_lexicon_tags_9 },
    { 0x50acf1ff, SUC_lexicon_tags_14 },
    { 0x10ccf200, SUC_lexicon_tags_16 },
    { 0xd96cf1ff, SUC_lexicon_tags_32 },
    { 0x772cf200, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0acf20d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4cf214, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x864cf216, SUC_lexicon_tags_17 },
    { 0xb6cf216, SUC_lexicon_tags_63 },
    { 0x6cacf218, SUC_lexicon_tags_25 },
    { 0xb20cf218, SUC_lexicon_tags_4 },
    { 0x6eecf219, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4cf21d, SUC_lexicon_tags_12 },
    { 0x640cf21d, SUC_lexicon_tags_47 },
    { 0xa3ecf21f, SUC_lexicon_tags_11 },
    { 0x636cf220, SUC_lexicon_tags_7 },
    { 0xb92cf220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x766cf223, SUC_lexicon_tags_45 },
    { 0x814cf224, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2acf228, SUC_lexicon_tags_9 },
    { 0xc96cf228, SUC_lexicon_tags_1 },
    { 0xcf22a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522cf234, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaccf237, SUC_lexicon_tags_6 },
    { 0xe04cf237, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf82cf23a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2cf23d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x568cf23f, SUC_lexicon_tags_47 },
    { 0xe92cf240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x984cf243, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x350cf245, SUC_lexicon_tags_12 },
    { 0x81acf246, SUC_lexicon_tags_12 },
    { 0x8aecf245, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x97acf249, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1aecf24b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68cf250, SUC_lexicon_tags_7 },
    { 0x120cf251, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad2cf253, SUC_lexicon_tags_12 },
    { 0xd0ccf254, SUC_lexicon_tags_37 },
    { 0xc38cf255, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82ecf257, SUC_lexicon_tags_17 },
    { 0x372cf257, SUC_lexicon_tags_25 },
    { 0x5eacf259, SUC_lexicon_tags_11 },
    { 0x81ecf25a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4cf25d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ccf262, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd30cf264, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0cf267, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676cf26d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42cf270, SUC_lexicon_tags_1 },
    { 0x75ccf271, SUC_lexicon_tags_33 },
    { 0xc3acf272, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2cf277, SUC_lexicon_tags_17 },
    { 0x932cf278, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4cf27c, SUC_lexicon_tags_1 },
    { 0x742cf27d, SUC_lexicon_tags_2 },
    { 0x56cf27e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ccf282, SUC_lexicon_tags_79 },
    { 0x90acf283, SUC_lexicon_tags_56 },
    { 0x8cf284, SUC_lexicon_tags_25 },
    { 0x9eacf285, SUC_lexicon_tags_11 },
    { 0x694cf286, SUC_lexicon_tags_1 },
    { 0x5bccf284, SUC_lexicon_tags_37 },
    { 0x63ecf282, SUC_lexicon_tags_37 },
    { 0x36cf285, SUC_lexicon_tags_63 },
    { 0x19acf282, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4cf290, SUC_lexicon_tags_60 },
    { 0xe9ccf290, SUC_lexicon_tags_7 },
    { 0xbe0cf290, SUC_lexicon_tags_14 },
    { 0xa0ecf290, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x860cf296, SUC_lexicon_tags_1 },
    { 0x9a2cf297, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecf29d, SUC_lexicon_tags_32 },
    { 0x478cf29d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3a6cf2a0, SUC_lexicon_tags_17 },
    { 0xa6ecf2a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafccf2a7, SUC_lexicon_tags_1 },
    { 0x3accf2a8, SUC_lexicon_tags_12 },
    { 0x9bccf2a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ccf2ac, SUC_lexicon_tags_37 },
    { 0xe7acf2ad, SUC_lexicon_tags_85 },
    { 0x754cf2ae, SUC_lexicon_tags_47 },
    { 0x8b8cf2ad, SUC_lexicon_tags_38 },
    { 0x8c2cf2b0, SUC_lexicon_tags_23 },
    { 0x2a0cf2ac, SUC_lexicon_tags_1 },
    { 0x9a4cf2ac, SUC_lexicon_tags_19 },
    { 0xec2cf2ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ecf2ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x344cf2bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xec6cf2be, SUC_lexicon_tags_11 },
    { 0xe6ecf2bf, SUC_lexicon_tags_13 },
    { 0x8fecf2be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f8cf2c2, SUC_lexicon_tags_35 },
    { 0x60ecf2c2, SUC_lexicon_tags_49 },
    { 0xb40cf2c4, SUC_lexicon_tags_17 },
    { 0x238cf2c3, SUC_lexicon_tags_4 },
    { 0xe2acf2c3, SUC_lexicon_tags_8 },
    { 0xb42cf2c7, SUC_lexicon_tags_23 },
    { 0x1a0cf2c8, SUC_lexicon_tags_63 },
    { 0xc6cf2c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0cf2cd, SUC_lexicon_tags_68 },
    { 0x8a0cf2ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338cf2d4, SUC_lexicon_tags_13 },
    { 0xa1acf2d4, SUC_lexicon_tags_28 },
    { 0x294cf2d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442cf2dd, SUC_lexicon_tags_37 },
    { 0xe70cf2de, SUC_lexicon_tags_60 },
    { 0xacecf2de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c6cf2e1, SUC_lexicon_tags_29 },
    { 0x7c8cf2e1, SUC_lexicon_tags_33 },
    { 0xb8ccf2e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652cf2e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x128cf2ea, SUC_lexicon_tags_17 },
    { 0xba0cf2eb, SUC_lexicon_tags_12 },
    { 0xf70cf2ec, SUC_lexicon_tags_45 },
    { 0x754cf2eb, SUC_lexicon_tags_6 },
    { 0x142cf2ee, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc92cf2f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3ecf2f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x26cf2f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3d8cf2f6, SUC_lexicon_tags_0 },
    { 0xda2cf2f7, SUC_lexicon_tags_1 },
    { 0x2ccf2f8, SUC_lexicon_tags_10 },
    { 0xc18cf2f9, SUC_lexicon_tags_11 },
    { 0xe40cf2fa, SUC_lexicon_tags_87 },
    { 0xf2ecf2fb, SUC_lexicon_tags_1 },
    { 0x4a8cf2fc, SUC_lexicon_tags_0 },
    { 0x17acf2fd, SUC_lexicon_tags_33 },
    { 0x1bacf2fd, SUC_lexicon_tags_17 },
    { 0x23acf2fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23acf302, SUC_lexicon_tags_11 },
    { 0x5c0cf303, SUC_lexicon_tags_37 },
    { 0xffacf303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25ecf306, SUC_lexicon_tags_25 },
    { 0xbeccf306, SUC_lexicon_tags_47 },
    { 0xe1ecf307, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6ccf30a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8cf30e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x692cf311, SUC_lexicon_tags_33 },
    { 0xbbacf312, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef4cf314, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360cf31e, SUC_lexicon_tags_0 },
    { 0x9ccf31e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc7ecf321, SUC_lexicon_tags_34 },
    { 0x676cf322, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a0cf324, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x810cf326, SUC_lexicon_tags_9 },
    { 0xcccf327, SUC_lexicon_tags_38 },
    { 0xd44cf326, SUC_lexicon_tags_15 },
    { 0x386cf328, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6cf32c, SUC_lexicon_tags_0 },
    { 0x81acf32d, SUC_lexicon_tags_11 },
    { 0xe68cf32e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232cf332, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8cf338, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4cf33d, SUC_lexicon_tags_29 },
    { 0xb10cf33d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4cf341, SUC_lexicon_tags_37 },
    { 0x7b6cf342, SUC_lexicon_tags_7 },
    { 0x3a0cf342, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x386cf346, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dacf349, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90cf352, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefccf357, SUC_lexicon_tags_26 },
    { 0x608cf357, SUC_lexicon_tags_68 },
    { 0x92cf358, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa9ecf35b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x512cf35e, SUC_lexicon_tags_46 },
    { 0x8accf35f, SUC_lexicon_tags_7 },
    { 0xfccf360, SUC_lexicon_tags_10 },
    { 0xecccf360, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4cf368, SUC_lexicon_tags_7 },
    { 0xa10cf369, SUC_lexicon_tags_15 },
    { 0xbb8cf36a, SUC_lexicon_tags_3 },
    { 0x2fccf36b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230cf36f, SUC_lexicon_tags_11 },
    { 0x2c6cf36f, SUC_lexicon_tags_2 },
    { 0x9ecf371, SUC_lexicon_tags_1 },
    { 0x868cf371, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ccf375, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x47ccf377, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cf37a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ccf37d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cecf384, SUC_lexicon_tags_14 },
    { 0x8bacf385, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cccf38a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1ccf38c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2acf390, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986cf394, SUC_lexicon_tags_11 },
    { 0x858cf394, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfb8cf397, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440cf39b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0acf3a2, SUC_lexicon_tags_45 },
    { 0x312cf3a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cacf3a5, SUC_lexicon_tags_34 },
    { 0xf9ecf3a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6cf3ac, SUC_lexicon_tags_12 },
    { 0xbe0cf3ac, SUC_lexicon_tags_17 },
    { 0xea6cf3ad, SUC_lexicon_tags_47 },
    { 0x1fccf3af, SUC_lexicon_tags_2 },
    { 0x49ccf3b0, SUC_lexicon_tags_27 },
    { 0xd18cf3b1, SUC_lexicon_tags_17 },
    { 0x8d4cf3b2, SUC_lexicon_tags_27 },
    { 0x4e8cf3b3, SUC_lexicon_tags_45 },
    { 0xbe0cf3ae, SUC_lexicon_tags_43 },
    { 0xe54cf3b0, SUC_lexicon_tags_60 },
    { 0xe54cf3ac, SUC_lexicon_tags_12 },
    { 0xb10cf3b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe66cf3b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0cf3bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6b2cf3bf, SUC_lexicon_tags_15 },
    { 0x426cf3c0, SUC_lexicon_tags_27 },
    { 0x50cf3c1, SUC_lexicon_tags_2 },
    { 0x60ccf3c2, SUC_lexicon_tags_17 },
    { 0x540cf3c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ccf3c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x122cf3c8, SUC_lexicon_tags_63 },
    { 0x98ccf3c8, SUC_lexicon_tags_63 },
    { 0xc9acf3ca, SUC_lexicon_tags_11 },
    { 0x97ecf3ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2cf3ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41acf3d7, SUC_lexicon_tags_68 },
    { 0x44ecf3d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffecf3da, SUC_lexicon_tags_12 },
    { 0xa84cf3db, SUC_lexicon_tags_17 },
    { 0x12acf3dc, SUC_lexicon_tags_7 },
    { 0xe88cf3dc, SUC_lexicon_tags_17 },
    { 0x4ecf3dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba4cf3e0, SUC_lexicon_tags_17 },
    { 0x114cf3e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880cf3e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cf3e8, SUC_lexicon_tags_37 },
    { 0x63ecf3e9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9d6cf3eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x370cf3ed, SUC_lexicon_tags_37 },
    { 0x836cf3ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfaacf3f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cacf3f7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cf3fa, SUC_lexicon_tags_19 },
    { 0x748cf3fa, SUC_lexicon_tags_0 },
    { 0x560cf3fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676cf402, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa9ccf404, SUC_lexicon_tags_27 },
    { 0x46ecf405, SUC_lexicon_tags_38 },
    { 0x24ccf406, SUC_lexicon_tags_66 },
    { 0xff8cf407, SUC_lexicon_tags_38 },
    { 0xd6ecf408, SUC_lexicon_tags_1 },
    { 0x90cf409, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ccf40c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630cf414, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x908cf416, SUC_lexicon_tags_1 },
    { 0x80ecf417, SUC_lexicon_tags_2 },
    { 0x608cf418, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ccf41b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4cf41e, SUC_lexicon_tags_11 },
    { 0xdaacf41e, SUC_lexicon_tags_1 },
    { 0x18cf420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ecf424, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb2cf426, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x104cf428, SUC_lexicon_tags_37 },
    { 0x5eacf429, SUC_lexicon_tags_48 },
    { 0xa78cf42a, SUC_lexicon_tags_71 },
    { 0x9f8cf42a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90cf42f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd6cf431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdecf438, SUC_lexicon_tags_15 },
    { 0xf0cf439, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64acf43d, SUC_lexicon_tags_25 },
    { 0x716cf43d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58acf443, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2cf446, SUC_lexicon_tags_12 },
    { 0xf6acf447, SUC_lexicon_tags_47 },
    { 0xff2cf446, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206cf44c, SUC_lexicon_tags_4 },
    { 0x344cf44d, SUC_lexicon_tags_12 },
    { 0xa14cf44e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ccf45a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccf45d, SUC_lexicon_tags_28 },
    { 0x172cf45e, SUC_lexicon_tags_23 },
    { 0x7c2cf45f, SUC_lexicon_tags_29 },
    { 0xa08cf460, SUC_lexicon_tags_15 },
    { 0x104cf461, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ecf46a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26acf46c, SUC_lexicon_tags_66 },
    { 0xe58cf46d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xea0cf46f, SUC_lexicon_tags_12 },
    { 0x3aecf46f, SUC_lexicon_tags_15 },
    { 0x882cf471, SUC_lexicon_tags_26 },
    { 0x736cf472, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c0cf474, SUC_lexicon_tags_83 },
    { 0x580cf475, SUC_lexicon_tags_15 },
    { 0x10acf475, SUC_lexicon_tags_11 },
    { 0xa16cf477, SUC_lexicon_tags_31 },
    { 0x126cf474, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x212cf47a, SUC_lexicon_tags_37 },
    { 0xb94cf47a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100cf482, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x348cf484, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14cf488, SUC_lexicon_tags_1 },
    { 0xadecf489, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93ccf48b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ecf48e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dacf494, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x31acf496, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x854cf498, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ecf4a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4cf4a8, SUC_lexicon_tags_1 },
    { 0xf7ccf4a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x978cf4ab, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378cf4af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4becf4b3, SUC_lexicon_tags_33 },
    { 0x2becf4b4, SUC_lexicon_tags_37 },
    { 0xfc6cf4b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3facf4b8, SUC_lexicon_tags_60 },
    { 0x312cf4b9, SUC_lexicon_tags_2 },
    { 0xbd0cf4ba, SUC_lexicon_tags_6 },
    { 0xf72cf4bb, SUC_lexicon_tags_11 },
    { 0x2ecf4bb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ecf4c0, SUC_lexicon_tags_11 },
    { 0x3e2cf4c1, SUC_lexicon_tags_4 },
    { 0xd78cf4c0, SUC_lexicon_tags_15 },
    { 0xe84cf4c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ecf4c6, SUC_lexicon_tags_15 },
    { 0xc02cf4c7, SUC_lexicon_tags_37 },
    { 0x154cf4c6, SUC_lexicon_tags_12 },
    { 0x534cf4c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x720cf4cb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x256cf4ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf62cf4d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2acf4d3, SUC_lexicon_tags_29 },
    { 0x990cf4d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x454cf4d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26cf4de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90acf4ec, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x154cf4ef, SUC_lexicon_tags_25 },
    { 0xc38cf4f0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754cf4f4, SUC_lexicon_tags_16 },
    { 0xa6acf4f5, SUC_lexicon_tags_1 },
    { 0x79ecf4f6, SUC_lexicon_tags_28 },
    { 0x1becf4f7, SUC_lexicon_tags_60 },
    { 0x9acf4f8, SUC_lexicon_tags_11 },
    { 0xd70cf4f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe72cf4fb, SUC_lexicon_tags_27 },
    { 0x70acf4fc, SUC_lexicon_tags_14 },
    { 0x1decf4fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9ccf4ff, SUC_lexicon_tags_1 },
    { 0xe90cf500, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6cf507, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ecf50a, SUC_lexicon_tags_21 },
    { 0x7b6cf50b, SUC_lexicon_tags_1 },
    { 0x60cf50b, SUC_lexicon_tags_11 },
    { 0xa78cf50c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x246cf50f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ccf513, SUC_lexicon_tags_60 },
    { 0xae6cf514, SUC_lexicon_tags_17 },
    { 0xccacf514, SUC_lexicon_tags_61 },
    { 0xcccf516, SUC_lexicon_tags_0 },
    { 0xf42cf516, SUC_lexicon_tags_47 },
    { 0x56ecf517, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe2ccf51a, SUC_lexicon_tags_15 },
    { 0x35ecf51a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f0cf51d, SUC_lexicon_tags_7 },
    { 0x1d6cf51e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6eccf520, SUC_lexicon_tags_37 },
    { 0x83ecf521, SUC_lexicon_tags_111 },
    { 0x85acf522, SUC_lexicon_tags_47 },
    { 0x418cf523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432cf527, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fccf52a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fccf52f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8cf533, SUC_lexicon_tags_17 },
    { 0x9c6cf534, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94cf537, SUC_lexicon_tags_11 },
    { 0x6a2cf537, SUC_lexicon_tags_6 },
    { 0x86ecf539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8acf53b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188cf53f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac4cf541, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112cf546, SUC_lexicon_tags_63 },
    { 0x8b2cf546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe0cf549, SUC_lexicon_tags_1 },
    { 0xcdecf54a, SUC_lexicon_tags_17 },
    { 0x63ccf54b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xef8cf54d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6cf550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8cf553, SUC_lexicon_tags_456 },
    { 0xf68cf553, SUC_lexicon_tags_14 },
    { 0x9cecf555, SUC_lexicon_tags_17 },
    { 0x4c4cf556, SUC_lexicon_tags_1 },
    { 0xfe8cf556, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49ccf559, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c4cf55b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd8cf55d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65acf561, SUC_lexicon_tags_1 },
    { 0x40acf561, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe36cf564, SUC_lexicon_tags_5 },
    { 0xe12cf564, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6cf56a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab0cf56c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8cf56f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2cf572, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0cf57d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f4cf57f, SUC_lexicon_tags_11 },
    { 0x7d4cf580, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ecf587, SUC_lexicon_tags_25 },
    { 0xac8cf588, SUC_lexicon_tags_12 },
    { 0x852cf588, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8accf58f, SUC_lexicon_tags_2 },
    { 0x618cf58f, SUC_lexicon_tags_15 },
    { 0x2f4cf58f, SUC_lexicon_tags_17 },
    { 0x68acf590, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfeccf594, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34cf598, SUC_lexicon_tags_23 },
    { 0x75ccf598, SUC_lexicon_tags_31 },
    { 0xbbacf59a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cccf59c, SUC_lexicon_tags_17 },
    { 0xb6ccf59d, SUC_lexicon_tags_19 },
    { 0xe7acf59e, SUC_lexicon_tags_8 },
    { 0x816cf59f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8acf5a1, SUC_lexicon_tags_11 },
    { 0x906cf5a2, SUC_lexicon_tags_37 },
    { 0x60ecf5a3, SUC_lexicon_tags_1 },
    { 0xd84cf5a1, SUC_lexicon_tags_25 },
    { 0x35ecf5a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x23acf5a7, SUC_lexicon_tags_1 },
    { 0xaa8cf5a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x936cf5ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa38cf5ad, SUC_lexicon_tags_7 },
    { 0xa44cf5ae, SUC_lexicon_tags_47 },
    { 0x994cf5ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cecf5b3, SUC_lexicon_tags_7 },
    { 0xb52cf5b4, SUC_lexicon_tags_12 },
    { 0x148cf5b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98cf5bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4ecf5be, SUC_lexicon_tags_7 },
    { 0xbbccf5bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57ecf5c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0cf5c4, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6cf5c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddecf5cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x316cf5cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0ccf5cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7b6cf5d1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x210cf5d3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704cf5d8, SUC_lexicon_tags_30 },
    { 0xaa4cf5d9, SUC_lexicon_tags_47 },
    { 0x3a8cf5d9, SUC_lexicon_tags_7 },
    { 0x286cf5db, SUC_lexicon_tags_12 },
    { 0xe4ccf5dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688cf5e1, SUC_lexicon_tags_37 },
    { 0xe80cf5e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60acf5e4, SUC_lexicon_tags_8 },
    { 0x514cf5e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c0cf5e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x192cf5e9, SUC_lexicon_tags_53 },
    { 0xd9acf5ea, SUC_lexicon_tags_11 },
    { 0xcccf5ea, SUC_lexicon_tags_42 },
    { 0xe06cf5e9, SUC_lexicon_tags_15 },
    { 0xc14cf5ea, SUC_lexicon_tags_63 },
    { 0x4e0cf5eb, SUC_lexicon_tags_9 },
    { 0x4facf5ef, SUC_lexicon_tags_47 },
    { 0x76ccf5e9, SUC_lexicon_tags_0 },
    { 0x884cf5f1, SUC_lexicon_tags_60 },
    { 0x4f4cf5f2, SUC_lexicon_tags_38 },
    { 0x68cf5f2, SUC_lexicon_tags_47 },
    { 0x8d0cf5ef, SUC_lexicon_tags_16 },
    { 0x804cf5ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ecf5fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24cf602, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56cf606, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5acf609, SUC_lexicon_tags_47 },
    { 0x34ccf60a, SUC_lexicon_tags_85 },
    { 0x5f6cf60b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08cf60e, SUC_lexicon_tags_74 },
    { 0x186cf60f, SUC_lexicon_tags_0 },
    { 0x9d8cf610, SUC_lexicon_tags_25 },
    { 0x6c6cf611, SUC_lexicon_tags_6 },
    { 0xfe4cf610, SUC_lexicon_tags_64 },
    { 0x8cacf613, SUC_lexicon_tags_15 },
    { 0xa22cf612, SUC_lexicon_tags_33 },
    { 0x8f6cf615, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26cf617, SUC_lexicon_tags_61 },
    { 0x174cf618, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x48ccf61a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7acf61f, SUC_lexicon_tags_25 },
    { 0x33acf61f, SUC_lexicon_tags_37 },
    { 0x346cf620, SUC_lexicon_tags_275 },
    { 0xaeecf61f, SUC_lexicon_tags_37 },
    { 0x610cf623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ccf626, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ecf62a, SUC_lexicon_tags_17 },
    { 0x4b2cf62b, SUC_lexicon_tags_29 },
    { 0x7decf62b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x410cf62f, SUC_lexicon_tags_11 },
    { 0xde2cf630, SUC_lexicon_tags_7 },
    { 0x110cf630, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8cf635, SUC_lexicon_tags_49 },
    { 0xb4ccf636, SUC_lexicon_tags_7 },
    { 0xbeacf637, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ecf63b, SUC_lexicon_tags_68 },
    { 0x88acf63b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538cf643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852cf64b, SUC_lexicon_tags_17 },
    { 0x1aecf64b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312cf650, SUC_lexicon_tags_43 },
    { 0x86ecf651, SUC_lexicon_tags_50 },
    { 0x100cf650, SUC_lexicon_tags_15 },
    { 0xe76cf650, SUC_lexicon_tags_12 },
    { 0xac0cf653, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc64cf656, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40cf65a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaecf65d, SUC_lexicon_tags_23 },
    { 0x382cf65d, SUC_lexicon_tags_12 },
    { 0x9accf65f, SUC_lexicon_tags_7 },
    { 0xea2cf660, SUC_lexicon_tags_60 },
    { 0xd68cf660, SUC_lexicon_tags_12 },
    { 0xb32cf662, SUC_lexicon_tags_17 },
    { 0xd60cf662, SUC_lexicon_tags_27 },
    { 0xdf6cf65f, SUC_lexicon_tags_39 },
    { 0xbe6cf665, SUC_lexicon_tags_25 },
    { 0x1e6cf665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6cf66b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64ccf66d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x914cf670, SUC_lexicon_tags_16 },
    { 0xfb0cf671, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6cf675, SUC_lexicon_tags_17 },
    { 0x642cf676, SUC_lexicon_tags_19 },
    { 0xa74cf675, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0acf67f, SUC_lexicon_tags_19 },
    { 0x5a0cf67f, SUC_lexicon_tags_37 },
    { 0x606cf681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba6cf683, SUC_lexicon_tags_11 },
    { 0x17acf684, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8ecf686, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98cf689, SUC_lexicon_tags_2 },
    { 0xc3ccf68a, SUC_lexicon_tags_47 },
    { 0x786cf689, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11ccf68d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ecf690, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xebecf692, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x626cf694, SUC_lexicon_tags_2 },
    { 0x752cf695, SUC_lexicon_tags_15 },
    { 0x748cf695, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ccf6a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e4cf6a4, SUC_lexicon_tags_17 },
    { 0x9aecf6a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ccf6a8, SUC_lexicon_tags_29 },
    { 0xd6acf6a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562cf6b1, SUC_lexicon_tags_14 },
    { 0x7b6cf6b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9eccf6b4, SUC_lexicon_tags_2 },
    { 0xe8ccf6b4, SUC_lexicon_tags_7 },
    { 0xf76cf6b6, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0x792cf6b8, SUC_lexicon_tags_11 },
    { 0x7a4cf6b8, SUC_lexicon_tags_28 },
    { 0x69ccf6ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fecf6bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x102cf6c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16cf6c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe10cf6c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x292cf6c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2cccf6cb, SUC_lexicon_tags_8 },
    { 0x826cf6cc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658cf6d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ecf6d5, SUC_lexicon_tags_63 },
    { 0x23ecf6d5, SUC_lexicon_tags_4 },
    { 0xc22cf6d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4cf6de, SUC_lexicon_tags_43 },
    { 0xadecf6df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeacf6e2, SUC_lexicon_tags_63 },
    { 0x40ccf6e3, SUC_lexicon_tags_33 },
    { 0xca2cf6e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ecf6e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34ccf6e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0cf6ec, SUC_lexicon_tags_79 },
    { 0xcd2cf6ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb4ecf6ef, SUC_lexicon_tags_25 },
    { 0x71ccf6ef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ccf6f4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4cf6f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccf6fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8cf6fe, SUC_lexicon_tags_61 },
    { 0x11acf6ff, SUC_lexicon_tags_47 },
    { 0x616cf700, SUC_lexicon_tags_7 },
    { 0xbfccf701, SUC_lexicon_tags_68 },
    { 0x56ccf702, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc84cf704, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaccf707, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x18cf709, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb4ccf70b, SUC_lexicon_tags_2 },
    { 0xcdacf70c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0cf710, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23acf712, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x404cf714, SUC_lexicon_tags_55 },
    { 0x5aacf715, SUC_lexicon_tags_16 },
    { 0x93ecf716, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x734cf718, SUC_lexicon_tags_18 },
    { 0x97ccf719, SUC_lexicon_tags_7 },
    { 0xcf719, SUC_lexicon_tags_12 },
    { 0x254cf719, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4cf720, SUC_lexicon_tags_37 },
    { 0xab4cf721, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x81ecf723, SUC_lexicon_tags_45 },
    { 0x46cf724, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd94cf726, SUC_lexicon_tags_68 },
    { 0x66acf727, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374cf72d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdc8cf72f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42cf733, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8ecf735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cf73b, SUC_lexicon_tags_23 },
    { 0x706cf73b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164cf741, SUC_lexicon_tags_46 },
    { 0xe22cf742, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80cf746, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204cf74b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652cf750, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ecf754, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x704cf756, SUC_lexicon_tags_84 },
    { 0xe80cf757, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3acf760, SUC_lexicon_tags_87 },
    { 0x584cf761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3acf763, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x724cf765, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae8cf767, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892cf76d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ccf776, SUC_lexicon_tags_38 },
    { 0x91ecf776, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4cf77b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeccf77f, SUC_lexicon_tags_65 },
    { 0x108cf780, SUC_lexicon_tags_7 },
    { 0xc6ccf781, SUC_lexicon_tags_15 },
    { 0xd18cf782, SUC_lexicon_tags_7 },
    { 0x44ecf783, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcecf789, SUC_lexicon_tags_10 },
    { 0xadecf78a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0cf78e, SUC_lexicon_tags_47 },
    { 0x16ccf78e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1ecf791, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28acf794, SUC_lexicon_tags_2 },
    { 0x874cf795, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x910cf797, SUC_lexicon_tags_12 },
    { 0x498cf798, SUC_lexicon_tags_66 },
    { 0xdd6cf799, SUC_lexicon_tags_47 },
    { 0xcf2cf79a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7ecf79c, SUC_lexicon_tags_55 },
    { 0x540cf79d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x932cf79f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7f0cf7a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x256cf7a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c0cf7a5, SUC_lexicon_tags_12 },
    { 0x42ccf7a6, SUC_lexicon_tags_7 },
    { 0xe40cf7a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4cf7ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x146cf7ad, SUC_lexicon_tags_31 },
    { 0x788cf7ae, SUC_lexicon_tags_12 },
    { 0xf06cf7ae, SUC_lexicon_tags_84 },
    { 0xce0cf7b0, SUC_lexicon_tags_72 },
    { 0x4d4cf7b1, SUC_lexicon_tags_30 },
    { 0xdecf7b1, SUC_lexicon_tags_37 },
    { 0x2cccf7b3, SUC_lexicon_tags_11 },
    { 0xef4cf7b2, SUC_lexicon_tags_25 },
    { 0x39acf7b5, SUC_lexicon_tags_12 },
    { 0x4facf7b3, SUC_lexicon_tags_1684 },
    { 0x89ccf7b7, SUC_lexicon_tags_15 },
    { 0x394cf7b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xafacf7ba, SUC_lexicon_tags_63 },
    { 0xd60cf7bb, SUC_lexicon_tags_39 },
    { 0x448cf7bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfcacf7be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cf7c2, SUC_lexicon_tags_27 },
    { 0x7facf7c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1a8cf7c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d8cf7c7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560cf7d0, SUC_lexicon_tags_1 },
    { 0xa76cf7d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x40ecf7d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcacf7d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cf7d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3ecf7da, SUC_lexicon_tags_46 },
    { 0xba0cf7da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170cf7e0, SUC_lexicon_tags_60 },
    { 0xea0cf7e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4cf7e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4acf7e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02cf7ea, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980cf7ee, SUC_lexicon_tags_17 },
    { 0x9dacf7ef, SUC_lexicon_tags_14 },
    { 0x838cf7ef, SUC_lexicon_tags_4 },
    { 0x66acf7ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbfecf7f3, SUC_lexicon_tags_17 },
    { 0x49ccf7f4, SUC_lexicon_tags_39 },
    { 0xd8cf7f5, SUC_lexicon_tags_1 },
    { 0x3b2cf7f6, SUC_lexicon_tags_41 },
    { 0x7f0cf7f6, SUC_lexicon_tags_8 },
    { 0x4d8cf7f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792cf7fc, SUC_lexicon_tags_27 },
    { 0x438cf7fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8cf800, SUC_lexicon_tags_15 },
    { 0xbc0cf801, SUC_lexicon_tags_70 },
    { 0x310cf800, SUC_lexicon_tags_25 },
    { 0x630cf803, SUC_lexicon_tags_49 },
    { 0x4c4cf803, SUC_lexicon_tags_0 },
    { 0xd8acf805, SUC_lexicon_tags_26 },
    { 0x410cf800, SUC_lexicon_tags_6 },
    { 0xbdacf805, SUC_lexicon_tags_15 },
    { 0x2eacf808, SUC_lexicon_tags_2 },
    { 0x8bacf802, SUC_lexicon_tags_15 },
    { 0xe7ecf80a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21acf80c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x51ccf80e, SUC_lexicon_tags_32 },
    { 0x33ccf80f, SUC_lexicon_tags_25 },
    { 0x624cf810, SUC_lexicon_tags_18 },
    { 0x904cf80f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842cf815, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8cf81b, SUC_lexicon_tags_11 },
    { 0xc10cf81c, SUC_lexicon_tags_3 },
    { 0xd74cf81d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4becf820, SUC_lexicon_tags_14 },
    { 0xf56cf821, SUC_lexicon_tags_15 },
    { 0x55ccf820, SUC_lexicon_tags_32 },
    { 0xf22cf823, SUC_lexicon_tags_2 },
    { 0x9decf820, SUC_lexicon_tags_1 },
    { 0xd36cf825, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228cf82b, SUC_lexicon_tags_12 },
    { 0xd8cf82c, SUC_lexicon_tags_37 },
    { 0xb60cf82c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb02cf82f, SUC_lexicon_tags_37 },
    { 0x79ecf830, SUC_lexicon_tags_23 },
    { 0x562cf831, SUC_lexicon_tags_40 },
    { 0xb90cf831, SUC_lexicon_tags_37 },
    { 0xbe6cf82f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358cf837, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x122cf839, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64acf83f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96cf843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ccf846, SUC_lexicon_tags_11 },
    { 0x902cf847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500cf84d, SUC_lexicon_tags_15 },
    { 0x5fccf84e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x612cf850, SUC_lexicon_tags_7 },
    { 0x792cf850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7decf85a, SUC_lexicon_tags_14 },
    { 0xd86cf85a, SUC_lexicon_tags_8 },
    { 0x316cf85b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64acf85e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8cf862, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2cf865, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x628cf868, SUC_lexicon_tags_25 },
    { 0xe4ecf869, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa30cf86b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cf875, SUC_lexicon_tags_33 },
    { 0x1aacf875, SUC_lexicon_tags_47 },
    { 0xbd4cf875, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1acf87c, SUC_lexicon_tags_25 },
    { 0x68acf87d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8cf882, SUC_lexicon_tags_45 },
    { 0x76ecf882, SUC_lexicon_tags_17 },
    { 0xf92cf883, SUC_lexicon_tags_1 },
    { 0x4fecf885, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58cf88b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaacf88e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eccf892, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc8ccf894, SUC_lexicon_tags_63 },
    { 0xd6acf895, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbf0cf897, SUC_lexicon_tags_15 },
    { 0x9b4cf897, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320cf89c, SUC_lexicon_tags_15 },
    { 0xac2cf89d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceacf8a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46cf8a5, SUC_lexicon_tags_11 },
    { 0x4fecf8a5, SUC_lexicon_tags_27 },
    { 0x306cf8a6, SUC_lexicon_tags_25 },
    { 0x3cecf8a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4cf8ac, SUC_lexicon_tags_29 },
    { 0xa74cf8ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fccf8b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9acf8b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38acf8b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x402cf8b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552cf8bc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4cf8c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82acf8c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x95ecf8c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd0cf8cb, SUC_lexicon_tags_26 },
    { 0x684cf8cc, SUC_lexicon_tags_12 },
    { 0x782cf8cc, SUC_lexicon_tags_7 },
    { 0xd0ecf8ce, SUC_lexicon_tags_37 },
    { 0x8dccf8cf, SUC_lexicon_tags_11 },
    { 0x5aacf8d0, SUC_lexicon_tags_68 },
    { 0xbd8cf8d0, SUC_lexicon_tags_37 },
    { 0x7facf8d2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdf8cf8d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8cf8d8, SUC_lexicon_tags_12 },
    { 0x982cf8d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628cf8de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2ecf8e0, SUC_lexicon_tags_11 },
    { 0xce2cf8e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9ecf8e3, SUC_lexicon_tags_23 },
    { 0x5e4cf8e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcd0cf8e6, SUC_lexicon_tags_11 },
    { 0x18ecf8e7, SUC_lexicon_tags_82 },
    { 0xab4cf8e8, SUC_lexicon_tags_33 },
    { 0xd6cf8e9, SUC_lexicon_tags_32 },
    { 0x5d4cf8ea, SUC_lexicon_tags_25 },
    { 0x3aecf8ea, SUC_lexicon_tags_31 },
    { 0xbeacf8eb, SUC_lexicon_tags_15 },
    { 0xee2cf8ed, SUC_lexicon_tags_77 },
    { 0xf38cf8ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ccf8f3, SUC_lexicon_tags_44 },
    { 0x7b6cf8f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50cf8f8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf26cf8fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cf8ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58cf902, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ecf905, SUC_lexicon_tags_81 },
    { 0x3b4cf905, SUC_lexicon_tags_55 },
    { 0xdcacf907, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296cf90b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x850cf90d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd94cf90f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4cf913, SUC_lexicon_tags_40 },
    { 0x40ecf914, SUC_lexicon_tags_2 },
    { 0x74ccf915, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ccf91c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x736cf91e, SUC_lexicon_tags_60 },
    { 0x6e4cf91f, SUC_lexicon_tags_25 },
    { 0xdd6cf920, SUC_lexicon_tags_82 },
    { 0xf24cf91f, SUC_lexicon_tags_29 },
    { 0x742cf922, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefecf926, SUC_lexicon_tags_2 },
    { 0xde2cf927, SUC_lexicon_tags_14 },
    { 0x91ccf928, SUC_lexicon_tags_28 },
    { 0x85acf926, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e2cf92b, SUC_lexicon_tags_11 },
    { 0x666cf92b, SUC_lexicon_tags_45 },
    { 0xe6cf92d, SUC_lexicon_tags_33 },
    { 0x780cf92e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f2cf930, SUC_lexicon_tags_15 },
    { 0x9e4cf930, SUC_lexicon_tags_10 },
    { 0x6e4cf932, SUC_lexicon_tags_7 },
    { 0xf0cf932, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71acf935, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6cf93a, SUC_lexicon_tags_32 },
    { 0xf46cf93b, SUC_lexicon_tags_37 },
    { 0xc80cf93a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ecf940, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160cf946, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908cf94b, SUC_lexicon_tags_1 },
    { 0xb04cf94c, SUC_lexicon_tags_12 },
    { 0xe24cf94b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x540cf950, SUC_lexicon_tags_30 },
    { 0x1f2cf951, SUC_lexicon_tags_17 },
    { 0x6fccf950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x612cf955, SUC_lexicon_tags_15 },
    { 0x9d8cf956, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8cf959, SUC_lexicon_tags_11 },
    { 0x38ecf95a, SUC_lexicon_tags_1 },
    { 0xf7ccf95a, SUC_lexicon_tags_47 },
    { 0x9d2cf95c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x966cf95e, SUC_lexicon_tags_17 },
    { 0xa6ecf95f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1acf963, SUC_lexicon_tags_37 },
    { 0xd64cf964, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x248cf967, SUC_lexicon_tags_8 },
    { 0x7bacf968, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cf96b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x780cf96d, SUC_lexicon_tags_7 },
    { 0x8d2cf96d, SUC_lexicon_tags_84 },
    { 0x3dccf96f, SUC_lexicon_tags_84 },
    { 0x4b2cf96f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ecf97b, SUC_lexicon_tags_108 },
    { 0xffecf97c, SUC_lexicon_tags_61 },
    { 0x74ccf97b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8c6cf97f, SUC_lexicon_tags_30 },
    { 0x86acf97f, SUC_lexicon_tags_17 },
    { 0xb82cf97f, SUC_lexicon_tags_1 },
    { 0xf88cf980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ecf984, SUC_lexicon_tags_17 },
    { 0x6decf985, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1ccf987, SUC_lexicon_tags_25 },
    { 0x772cf988, SUC_lexicon_tags_17 },
    { 0xcbacf988, SUC_lexicon_tags_2 },
    { 0x2aecf98a, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ccf98d, SUC_lexicon_tags_16 },
    { 0xa38cf98e, SUC_lexicon_tags_47 },
    { 0xf3ecf98f, SUC_lexicon_tags_0 },
    { 0x33acf98f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52ecf992, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x98acf994, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ecf999, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886cf99d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ecf9a1, SUC_lexicon_tags_26 },
    { 0x632cf9a1, SUC_lexicon_tags_47 },
    { 0xc0ecf9a3, SUC_lexicon_tags_1 },
    { 0x386cf9a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8ecf9a6, SUC_lexicon_tags_92 },
    { 0xc5ccf9a7, SUC_lexicon_tags_25 },
    { 0xe1ecf9a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaccf9ac, SUC_lexicon_tags_29 },
    { 0x180cf9ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dacf9b1, SUC_lexicon_tags_37 },
    { 0x546cf9b2, SUC_lexicon_tags_43 },
    { 0x606cf9b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb78cf9b5, SUC_lexicon_tags_62 },
    { 0xffccf9b6, SUC_lexicon_tags_33 },
    { 0xb7acf9b5, SUC_lexicon_tags_12 },
    { 0xc4acf9b6, SUC_lexicon_tags_21 },
    { 0x4c0cf9b9, SUC_lexicon_tags_17 },
    { 0x4c8cf9b6, SUC_lexicon_tags_1 },
    { 0xebecf9bb, SUC_lexicon_tags_12 },
    { 0x2e8cf9b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4acf9bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620cf9c4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ecf9d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ccf9d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x648cf9d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6fccf9d8, SUC_lexicon_tags_63 },
    { 0x6b6cf9d8, SUC_lexicon_tags_15 },
    { 0x5ccf9d8, SUC_lexicon_tags_11 },
    { 0x39ecf9d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0cf9de, SUC_lexicon_tags_37 },
    { 0xdaccf9df, SUC_lexicon_tags_63 },
    { 0x8eccf9e0, SUC_lexicon_tags_11 },
    { 0x2cccf9e1, SUC_lexicon_tags_0 },
    { 0x30cf9df, SUC_lexicon_tags_28 },
    { 0xd28cf9e3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8cf9e6, SUC_lexicon_tags_11 },
    { 0x9f2cf9e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa96cf9e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902cf9f0, SUC_lexicon_tags_11 },
    { 0x64ecf9f1, SUC_lexicon_tags_37 },
    { 0x62acf9f2, SUC_lexicon_tags_27 },
    { 0x3d2cf9f2, SUC_lexicon_tags_12 },
    { 0x542cf9f1, SUC_lexicon_tags_33 },
    { 0xe78cf9f5, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xabacf9f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x56ccf9f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6cf9fc, SUC_lexicon_tags_1 },
    { 0x2e6cf9fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ccfa03, SUC_lexicon_tags_1 },
    { 0x49acfa03, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26cfa08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x48ecfa0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58acfa0e, SUC_lexicon_tags_12 },
    { 0x932cfa0e, SUC_lexicon_tags_47 },
    { 0x988cfa10, SUC_lexicon_tags_17 },
    { 0x3a6cfa10, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22cfa15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4cfa18, SUC_lexicon_tags_14 },
    { 0x544cfa19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640cfa23, SUC_lexicon_tags_47 },
    { 0xd44cfa24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebecfa26, SUC_lexicon_tags_49 },
    { 0x70ccfa27, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbacfa2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ccfa31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6cfa34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedecfa3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44acfa3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32acfa3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ccfa42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94cfa48, SUC_lexicon_tags_49 },
    { 0xe88cfa49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xadecfa4c, SUC_lexicon_tags_8 },
    { 0x7b2cfa4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908cfa54, SUC_lexicon_tags_25 },
    { 0x154cfa55, SUC_lexicon_tags_38 },
    { 0x2facfa56, SUC_lexicon_tags_11 },
    { 0x16ccfa57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x974cfa59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccfa5e, SUC_lexicon_tags_38 },
    { 0x9b0cfa5f, SUC_lexicon_tags_25 },
    { 0x81ccfa60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ccfa63, SUC_lexicon_tags_17 },
    { 0x784cfa64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd4cfa66, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0cfa6a, SUC_lexicon_tags_15 },
    { 0x31ecfa6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2cfa70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dacfa74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8facfa76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x420cfa78, SUC_lexicon_tags_17 },
    { 0x2d6cfa78, SUC_lexicon_tags_46 },
    { 0x280cfa7a, SUC_lexicon_tags_63 },
    { 0x8bccfa7b, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ecfa81, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xa58cfa83, SUC_lexicon_tags_31 },
    { 0xeccfa83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcacfa8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106cfa94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80acfa97, SUC_lexicon_tags_6 },
    { 0x234cfa98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x66ccfa9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74cfaa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5aacfaa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04cfaa7, SUC_lexicon_tags_15 },
    { 0xd88cfaa7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce0cfaaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf62cfaac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278cfab2, SUC_lexicon_tags_30 },
    { 0x6d2cfab2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68cfab8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498cfabc, SUC_lexicon_tags_15 },
    { 0x7a8cfabd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4cfabf, SUC_lexicon_tags_45 },
    { 0x6c8cfabf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb8ccfac2, SUC_lexicon_tags_2 },
    { 0xf64cfac2, SUC_lexicon_tags_49 },
    { 0x33ecfac4, SUC_lexicon_tags_18 },
    { 0xbc2cfac5, SUC_lexicon_tags_46 },
    { 0xa28cfac6, SUC_lexicon_tags_25 },
    { 0xc16cfac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcacfac9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ccfacb, SUC_lexicon_tags_33 },
    { 0xe9acfacb, SUC_lexicon_tags_47 },
    { 0x360cfacd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcccfacf, SUC_lexicon_tags_91 },
    { 0x694cfacf, SUC_lexicon_tags_37 },
    { 0x270cfad1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cfad6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2cfad9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ccfadd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7aecfadf, SUC_lexicon_tags_12 },
    { 0x14cfae0, SUC_lexicon_tags_12 },
    { 0x106cfae1, SUC_lexicon_tags_43 },
    { 0xccecfae1, SUC_lexicon_tags_0 },
    { 0xc24cfae1, SUC_lexicon_tags_17 },
    { 0x442cfae2, SUC_lexicon_tags_11 },
    { 0x240cfae5, SUC_lexicon_tags_81 },
    { 0xd64cfae5, SUC_lexicon_tags_11 },
    { 0xd0acfae6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4cfaed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cccfaf2, SUC_lexicon_tags_12 },
    { 0xd8ccfaf3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d8cfaf5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134cfafa, SUC_lexicon_tags_0 },
    { 0x94cfafa, SUC_lexicon_tags_2 },
    { 0xb24cfafc, SUC_lexicon_tags_30 },
    { 0x60ecfafd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x40acfb00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xabccfb03, SUC_lexicon_tags_15 },
    { 0xe32cfb03, SUC_lexicon_tags_21 },
    { 0xb58cfb05, SUC_lexicon_tags_55 },
    { 0x2c2cfb05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a4cfb08, SUC_lexicon_tags_25 },
    { 0xfe0cfb09, SUC_lexicon_tags_1 },
    { 0xb9acfb0a, SUC_lexicon_tags_16 },
    { 0x762cfb0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668cfb1a, SUC_lexicon_tags_15 },
    { 0xe44cfb1b, SUC_lexicon_tags_38 },
    { 0xd70cfb1b, SUC_lexicon_tags_37 },
    { 0xa06cfb1a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cfb21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57acfb23, SUC_lexicon_tags_25 },
    { 0x8a0cfb24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x728cfb27, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe90cfb2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42ccfb2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722cfb30, SUC_lexicon_tags_60 },
    { 0xdc2cfb31, SUC_lexicon_tags_26 },
    { 0xc0cfb32, SUC_lexicon_tags_29 },
    { 0x7e2cfb33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ccfb37, SUC_lexicon_tags_38 },
    { 0x2b2cfb38, SUC_lexicon_tags_45 },
    { 0x2f0cfb38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfbecfb3b, SUC_lexicon_tags_12 },
    { 0x3f2cfb3b, SUC_lexicon_tags_47 },
    { 0xbf0cfb3b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaacfb41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74cfb45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc80cfb47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224cfb4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56ccfb50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402cfb56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3aacfb58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bacfb5b, SUC_lexicon_tags_37 },
    { 0x5d0cfb5c, SUC_lexicon_tags_11 },
    { 0x878cfb5d, SUC_lexicon_tags_2 },
    { 0x500cfb5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xedccfb60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6cfb65, SUC_lexicon_tags_19 },
    { 0xd84cfb65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xce8cfb68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2cfb6e, SUC_lexicon_tags_33 },
    { 0xcaccfb6f, SUC_lexicon_tags_11 },
    { 0x738cfb70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a4cfb72, SUC_lexicon_tags_94 },
    { 0xb2acfb72, SUC_lexicon_tags_37 },
    { 0x82cfb74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5acfb77, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x90cfb79, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdacfb7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc44cfb81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf94cfb83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75acfb85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6decfb88, SUC_lexicon_tags_47 },
    { 0xd2acfb88, SUC_lexicon_tags_25 },
    { 0x8bccfb88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0cfb8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa76cfb8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefccfb94, SUC_lexicon_tags_11 },
    { 0xb9ecfb94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646cfb99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdbccfb9b, SUC_lexicon_tags_2 },
    { 0x74acfb9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x806cfb9e, SUC_lexicon_tags_28 },
    { 0xd10cfb9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5acfbb2, SUC_lexicon_tags_43 },
    { 0xf20cfbb2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd4ecfbb5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfe8cfbb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92acfbc0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecfbc6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaccfbc9, SUC_lexicon_tags_11 },
    { 0xc56cfbc9, SUC_lexicon_tags_17 },
    { 0x734cfbcb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294cfbcf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286cfbd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6cfbda, SUC_lexicon_tags_12 },
    { 0x1d0cfbdb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x690cfbde, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfccfbe1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8cfbe6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb0ecfbe8, SUC_lexicon_tags_7 },
    { 0x1eecfbe9, SUC_lexicon_tags_9 },
    { 0x4d2cfbea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbed, SUC_lexicon_tags_1 },
    { 0xd1acfbee, SUC_lexicon_tags_38 },
    { 0xb0ecfbef, SUC_lexicon_tags_15 },
    { 0x146cfbef, SUC_lexicon_tags_65 },
    { 0x6fccfbf1, SUC_lexicon_tags_12 },
    { 0x9eacfbf2, SUC_lexicon_tags_2 },
    { 0x680cfbf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ccfbf6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8cfbf9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ccfbfc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44cfc02, SUC_lexicon_tags_60 },
    { 0xa08cfc02, SUC_lexicon_tags_15 },
    { 0x52ecfc04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0cfc0b, SUC_lexicon_tags_11 },
    { 0xa60cfc0c, SUC_lexicon_tags_17 },
    { 0xc66cfc0c, SUC_lexicon_tags_2 },
    { 0x8e2cfc0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaeccfc10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb34cfc12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0cfc17, SUC_lexicon_tags_27 },
    { 0x5d6cfc18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ccfc22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22cfc26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14acfc28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278cfc2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x458cfc2e, SUC_lexicon_tags_23 },
    { 0x72ccfc2e, SUC_lexicon_tags_37 },
    { 0xd24cfc2f, SUC_lexicon_tags_19 },
    { 0xaaacfc2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142cfc43, SUC_lexicon_tags_43 },
    { 0x3a2cfc44, SUC_lexicon_tags_12 },
    { 0x582cfc45, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62cfc49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ccfc4d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26cfc50, SUC_lexicon_tags_37 },
    { 0xc08cfc51, SUC_lexicon_tags_17 },
    { 0x5b2cfc52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0cfc56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x412cfc59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadecfc5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x808cfc5d, SUC_lexicon_tags_15 },
    { 0x58ecfc5e, SUC_lexicon_tags_25 },
    { 0x73ccfc5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cfc62, SUC_lexicon_tags_31 },
    { 0x4acfc63, SUC_lexicon_tags_33 },
    { 0xe2cfc63, SUC_lexicon_tags_12 },
    { 0xc2ccfc62, SUC_lexicon_tags_60 },
    { 0xd56cfc63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcccfc69, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766cfc6e, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xf0cfc70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2cfc73, SUC_lexicon_tags_41 },
    { 0xcf0cfc74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa6cfc76, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2cfc7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ecfc83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41ccfc85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cfc88, SUC_lexicon_tags_10 },
    { 0xf1ccfc89, SUC_lexicon_tags_0 },
    { 0x814cfc88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0cfc8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926cfc97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730cfc9b, SUC_lexicon_tags_23 },
    { 0xccfc9b, SUC_lexicon_tags_25 },
    { 0x40acfc9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32acfca1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x236cfca3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x26cfca5, SUC_lexicon_tags_85 },
    { 0x540cfca6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57acfcaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x564cfcad, SUC_lexicon_tags_7 },
    { 0xe22cfcad, SUC_lexicon_tags_1 },
    { 0xb78cfcaf, SUC_lexicon_tags_15 },
    { 0xbeccfcb0, SUC_lexicon_tags_7 },
    { 0xd10cfcb1, SUC_lexicon_tags_11 },
    { 0xc12cfcb0, SUC_lexicon_tags_47 },
    { 0x1fccfcb0, SUC_lexicon_tags_1 },
    { 0xc54cfcb0, SUC_lexicon_tags_19 },
    { 0xcfcb5, SUC_lexicon_tags_66 },
    { 0x5ecfcaf, SUC_lexicon_tags_60 },
    { 0x19ccfcb7, SUC_lexicon_tags_15 },
    { 0xfbecfcb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668cfcbc, SUC_lexicon_tags_79 },
    { 0x2a6cfcbd, SUC_lexicon_tags_25 },
    { 0x2eecfcbd, SUC_lexicon_tags_1474 },
    { 0, NULL },
    { 0, NULL },
    { 0x370cfcc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x30acfcc3, SUC_lexicon_tags_37 },
    { 0x982cfcc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08cfcc7, SUC_lexicon_tags_23 },
    { 0x23acfcc8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6facfcce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4cfcd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ecfcd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x730cfcda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcccfcdd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x34ccfcdf, SUC_lexicon_tags_0 },
    { 0x3b6cfce0, SUC_lexicon_tags_1 },
    { 0xdaecfce1, SUC_lexicon_tags_37 },
    { 0xd70cfce2, SUC_lexicon_tags_6 },
    { 0xeb6cfce3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594cfce7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cacfcea, SUC_lexicon_tags_43 },
    { 0xafecfceb, SUC_lexicon_tags_60 },
    { 0x3ecfcea, SUC_lexicon_tags_49 },
    { 0x6facfced, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x382cfcef, SUC_lexicon_tags_27 },
    { 0x3e0cfcef, SUC_lexicon_tags_7 },
    { 0x6eacfcef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ecfcf7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x4eecfcf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28cfcfc, SUC_lexicon_tags_112 },
    { 0x186cfcfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226cfd01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba0cfd03, SUC_lexicon_tags_9 },
    { 0x3bacfd04, SUC_lexicon_tags_7 },
    { 0x372cfd05, SUC_lexicon_tags_31 },
    { 0xaf0cfd05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb86cfd08, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xee8cfd0a, SUC_lexicon_tags_118 },
    { 0xac2cfd0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x144cfd0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9acfd12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe28cfd14, SUC_lexicon_tags_10 },
    { 0x802cfd14, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4cfd18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cfd1e, SUC_lexicon_tags_60 },
    { 0x56acfd1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfecfd22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8cfd26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x272cfd29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb52cfd2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x582cfd2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0acfd32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60cfd38, SUC_lexicon_tags_33 },
    { 0xa22cfd38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8acfd42, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x58cfd44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2a0cfd46, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x20acfd48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346cfd4f, SUC_lexicon_tags_12 },
    { 0xe74cfd50, SUC_lexicon_tags_17 },
    { 0x3c6cfd4f, SUC_lexicon_tags_63 },
    { 0x480cfd50, SUC_lexicon_tags_11 },
    { 0xd80cfd53, SUC_lexicon_tags_43 },
    { 0x372cfd50, SUC_lexicon_tags_25 },
    { 0x110cfd55, SUC_lexicon_tags_2 },
    { 0x30acfd56, SUC_lexicon_tags_6 },
    { 0x596cfd57, SUC_lexicon_tags_55 },
    { 0x776cfd56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x440cfd5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b6cfd5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82cfd5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaaecfd61, SUC_lexicon_tags_21 },
    { 0x8aecfd62, SUC_lexicon_tags_9 },
    { 0xc38cfd63, SUC_lexicon_tags_29 },
    { 0x61acfd64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x358cfd67, SUC_lexicon_tags_9 },
    { 0xcf0cfd68, SUC_lexicon_tags_38 },
    { 0xfa2cfd68, SUC_lexicon_tags_38 },
    { 0x874cfd6a, SUC_lexicon_tags_19 },
    { 0x438cfd6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0cfd6e, SUC_lexicon_tags_26 },
    { 0x85ecfd6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42cfd77, SUC_lexicon_tags_17 },
    { 0xb24cfd78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610cfd7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32cfd7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c4cfd81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x324cfd83, SUC_lexicon_tags_49 },
    { 0xa54cfd84, SUC_lexicon_tags_28 },
    { 0xc8ccfd83, SUC_lexicon_tags_39 },
    { 0xea6cfd86, SUC_lexicon_tags_31 },
    { 0x8c8cfd84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2cfd8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ecfd8d, SUC_lexicon_tags_8 },
    { 0xc2cfd8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa36cfd90, SUC_lexicon_tags_0 },
    { 0xf2ccfd91, SUC_lexicon_tags_47 },
    { 0x6d6cfd92, SUC_lexicon_tags_53 },
    { 0xd2ecfd92, SUC_lexicon_tags_47 },
    { 0x4acfd93, SUC_lexicon_tags_70 },
    { 0xe1ecfd95, SUC_lexicon_tags_6 },
    { 0xa2acfd96, SUC_lexicon_tags_28 },
    { 0xcc4cfd94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x192cfd9a, SUC_lexicon_tags_25 },
    { 0x718cfd9b, SUC_lexicon_tags_23 },
    { 0x408cfd9c, SUC_lexicon_tags_8 },
    { 0x5aacfd9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60cfda2, SUC_lexicon_tags_27 },
    { 0xcd0cfda3, SUC_lexicon_tags_31 },
    { 0xc4cfda4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbacfda8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00cfdaf, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22cfdb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93acfdb8, SUC_lexicon_tags_14 },
    { 0x2e0cfdb9, SUC_lexicon_tags_17 },
    { 0x7b2cfdba, SUC_lexicon_tags_17 },
    { 0x49ccfdbb, SUC_lexicon_tags_47 },
    { 0xc86cfdbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x666cfdbf, SUC_lexicon_tags_17 },
    { 0x1d2cfdc0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x768cfdc2, SUC_lexicon_tags_1 },
    { 0x3a8cfdc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1acfdc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1acfdca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640cfdcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bccfdd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62ccfdd8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbc0cfdda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa32cfddc, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x464cfdde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdecfde2, SUC_lexicon_tags_33 },
    { 0xf1acfde3, SUC_lexicon_tags_9 },
    { 0x8c2cfde2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b4cfde6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x68acfde8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x746cfdea, SUC_lexicon_tags_47 },
    { 0xe12cfdeb, SUC_lexicon_tags_25 },
    { 0x7becfdec, SUC_lexicon_tags_27 },
    { 0x160cfdec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ccfdf3, SUC_lexicon_tags_6 },
    { 0x248cfdf4, SUC_lexicon_tags_11 },
    { 0xa36cfdf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e0cfdf7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x438cfdfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa08cfdfd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x482cfdff, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898cfe03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x322cfe05, SUC_lexicon_tags_60 },
    { 0x6becfe05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934cfe0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810cfe11, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x14acfe13, SUC_lexicon_tags_11 },
    { 0xf02cfe13, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6d0cfe16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x908cfe18, SUC_lexicon_tags_10 },
    { 0x7decfe19, SUC_lexicon_tags_26 },
    { 0x628cfe18, SUC_lexicon_tags_2 },
    { 0xc8ecfe1b, SUC_lexicon_tags_15 },
    { 0xad2cfe1c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ecfe22, SUC_lexicon_tags_26 },
    { 0xf30cfe23, SUC_lexicon_tags_38 },
    { 0xea6cfe24, SUC_lexicon_tags_8 },
    { 0xc72cfe24, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9eecfe27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4cfe2f, SUC_lexicon_tags_63 },
    { 0x478cfe30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeacfe38, SUC_lexicon_tags_14 },
    { 0xbb2cfe39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x664cfe3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x380cfe3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8cfe47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x15ecfe49, SUC_lexicon_tags_4 },
    { 0x6dccfe4a, SUC_lexicon_tags_43 },
    { 0x4b0cfe4b, SUC_lexicon_tags_21 },
    { 0x586cfe4c, SUC_lexicon_tags_7 },
    { 0xed0cfe4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ecfe50, SUC_lexicon_tags_20 },
    { 0x7cccfe51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16cfe54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccfe5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52cfe60, SUC_lexicon_tags_15 },
    { 0x422cfe60, SUC_lexicon_tags_0 },
    { 0x782cfe60, SUC_lexicon_tags_43 },
    { 0x33acfe63, SUC_lexicon_tags_7 },
    { 0xa54cfe61, SUC_lexicon_tags_8 },
    { 0xa28cfe65, SUC_lexicon_tags_32 },
    { 0x1c6cfe65, SUC_lexicon_tags_6 },
    { 0xa10cfe67, SUC_lexicon_tags_17 },
    { 0xa40cfe68, SUC_lexicon_tags_27 },
    { 0xb68cfe69, SUC_lexicon_tags_15 },
    { 0x906cfe68, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6acfe6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8cfe72, SUC_lexicon_tags_65 },
    { 0xac8cfe73, SUC_lexicon_tags_14 },
    { 0xb76cfe74, SUC_lexicon_tags_8 },
    { 0xdb2cfe75, SUC_lexicon_tags_9 },
    { 0x86ccfe72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128cfe7a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x756cfe7d, SUC_lexicon_tags_82 },
    { 0x202cfe7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x426cfe80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcccfe82, SUC_lexicon_tags_229 },
    { 0xd68cfe82, SUC_lexicon_tags_17 },
    { 0xa12cfe84, SUC_lexicon_tags_19 },
    { 0xff0cfe85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x82acfe87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20cfe8a, SUC_lexicon_tags_17 },
    { 0xc84cfe8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3ecfe8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6cfe93, SUC_lexicon_tags_230 },
    { 0x396cfe94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6cfe96, SUC_lexicon_tags_2 },
    { 0xb6ccfe97, SUC_lexicon_tags_13 },
    { 0xc0acfe98, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x328cfe9a, SUC_lexicon_tags_29 },
    { 0x9c8cfe9b, SUC_lexicon_tags_16 },
    { 0x422cfe9c, SUC_lexicon_tags_45 },
    { 0xf2ccfe9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76cfe9f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x6d8cfea1, SUC_lexicon_tags_37 },
    { 0x5c8cfea1, SUC_lexicon_tags_17 },
    { 0xcd8cfea3, SUC_lexicon_tags_0 },
    { 0x72ecfea2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cfea8, SUC_lexicon_tags_1 },
    { 0x168cfea9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bccfeae, SUC_lexicon_tags_17 },
    { 0x832cfeae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a0cfeb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4cfeb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa40cfeb8, SUC_lexicon_tags_6 },
    { 0x4cacfeb9, SUC_lexicon_tags_37 },
    { 0x674cfeb9, SUC_lexicon_tags_12 },
    { 0x650cfebb, SUC_lexicon_tags_37 },
    { 0x6bccfeba, SUC_lexicon_tags_25 },
    { 0x3a0cfebd, SUC_lexicon_tags_47 },
    { 0xa24cfebe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3acfec2, SUC_lexicon_tags_15 },
    { 0x696cfec2, SUC_lexicon_tags_25 },
    { 0x334cfec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680cfec8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0cfecb, SUC_lexicon_tags_8 },
    { 0x2e8cfecb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96cfecf, SUC_lexicon_tags_29 },
    { 0x55ccfed0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8cfed6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x214cfed8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cfedb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecfee1, SUC_lexicon_tags_17 },
    { 0x72cfee2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x316cfee5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x9d2cfee7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7ccfee9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaecfeee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8cfef8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30cfefd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0cff05, SUC_lexicon_tags_1 },
    { 0xe4cff06, SUC_lexicon_tags_12 },
    { 0x1a6cff07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53acff09, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180cff10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf84cff12, SUC_lexicon_tags_23 },
    { 0x9c4cff13, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89acff1e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662cff25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2cff28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb32cff2a, SUC_lexicon_tags_57 },
    { 0x394cff2b, SUC_lexicon_tags_2 },
    { 0x6dccff2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2cff2f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40cff33, SUC_lexicon_tags_23 },
    { 0x456cff34, SUC_lexicon_tags_25 },
    { 0x2eecff34, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ecff3d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688cff41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a4cff43, SUC_lexicon_tags_15 },
    { 0xbc2cff44, SUC_lexicon_tags_47 },
    { 0xc3ccff45, SUC_lexicon_tags_23 },
    { 0xa5ecff46, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aacff4b, SUC_lexicon_tags_1 },
    { 0x908cff4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x344cff4e, SUC_lexicon_tags_4 },
    { 0x4aacff4f, SUC_lexicon_tags_39 },
    { 0xb06cff50, SUC_lexicon_tags_2 },
    { 0x9eecff51, SUC_lexicon_tags_33 },
    { 0x4a8cff50, SUC_lexicon_tags_2 },
    { 0xebccff53, SUC_lexicon_tags_63 },
    { 0xdfccff54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20ecff56, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9c8cff58, SUC_lexicon_tags_37 },
    { 0x55ccff58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87ecff5b, SUC_lexicon_tags_19 },
    { 0x5a0cff5c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46cff62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fccff66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00cff6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3eacff6e, SUC_lexicon_tags_12 },
    { 0xc06cff6e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x342cff71, SUC_lexicon_tags_8 },
    { 0xa68cff72, SUC_lexicon_tags_37 },
    { 0xb5ccff71, SUC_lexicon_tags_25 },
    { 0x308cff74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x698cff76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2ecff78, SUC_lexicon_tags_8 },
    { 0xe1ecff78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7acff7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0cff81, SUC_lexicon_tags_27 },
    { 0x430cff82, SUC_lexicon_tags_46 },
    { 0x786cff81, SUC_lexicon_tags_12 },
    { 0x50ccff84, SUC_lexicon_tags_11 },
    { 0x482cff85, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb54cff87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72cff8b, SUC_lexicon_tags_37 },
    { 0xcb2cff8c, SUC_lexicon_tags_92 },
    { 0x910cff8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32ccff8f, SUC_lexicon_tags_84 },
    { 0x3c0cff8f, SUC_lexicon_tags_7 },
    { 0x760cff91, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0cff96, SUC_lexicon_tags_21 },
    { 0x2cff96, SUC_lexicon_tags_15 },
    { 0xfaacff97, SUC_lexicon_tags_15 },
    { 0xe64cff99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad4cff9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3accffa2, SUC_lexicon_tags_33 },
    { 0x6e0cffa3, SUC_lexicon_tags_7 },
    { 0x47ecffa2, SUC_lexicon_tags_16 },
    { 0xf14cffa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12ccffa7, SUC_lexicon_tags_17 },
    { 0x8cecffa7, SUC_lexicon_tags_14 },
    { 0x2b2cffa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5acffac, SUC_lexicon_tags_17 },
    { 0x150cffad, SUC_lexicon_tags_17 },
    { 0x152cffad, SUC_lexicon_tags_11 },
    { 0x5acffaf, SUC_lexicon_tags_9 },
    { 0xcdacffb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x424cffb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x566cffb6, SUC_lexicon_tags_23 },
    { 0xdd0cffb6, SUC_lexicon_tags_26 },
    { 0x8fccffb8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb30cffba, SUC_lexicon_tags_25 },
    { 0x9a8cffba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaecffbf, SUC_lexicon_tags_27 },
    { 0xe06cffc0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2cffc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2cffc6, SUC_lexicon_tags_27 },
    { 0x432cffc7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ccffcb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3d6cffcd, SUC_lexicon_tags_27 },
    { 0x7a4cffcd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ccffd1, SUC_lexicon_tags_33 },
    { 0xd22cffd2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dccffd7, SUC_lexicon_tags_37 },
    { 0x66cffd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ccffdb, SUC_lexicon_tags_37 },
    { 0x1b2cffdc, SUC_lexicon_tags_17 },
    { 0x2e2cffdb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ecffe1, SUC_lexicon_tags_49 },
    { 0x3aacffe1, SUC_lexicon_tags_45 },
    { 0x37ecffe2, SUC_lexicon_tags_29 },
    { 0xca4cffe4, SUC_lexicon_tags_17 },
    { 0xbecffe5, SUC_lexicon_tags_15 },
    { 0xa66cffe5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd22cffe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40cfff1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x386cfff4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1bccfff6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540cfffa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06d0001, SUC_lexicon_tags_86 },
    { 0xc00d0001, SUC_lexicon_tags_0 },
    { 0x444d0003, SUC_lexicon_tags_37 },
    { 0x1e0d0004, SUC_lexicon_tags_60 },
    { 0xe60d0004, SUC_lexicon_tags_12 },
    { 0x188d0002, SUC_lexicon_tags_1 },
    { 0xad8d0007, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98d000f, SUC_lexicon_tags_56 },
    { 0x7cad0010, SUC_lexicon_tags_15 },
    { 0x81ad0011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394d0016, SUC_lexicon_tags_53 },
    { 0xc08d0016, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb4d0019, SUC_lexicon_tags_26 },
    { 0xecd0019, SUC_lexicon_tags_37 },
    { 0x592d001b, SUC_lexicon_tags_27 },
    { 0xf9ad001c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x80cd001e, SUC_lexicon_tags_7 },
    { 0x410d001e, SUC_lexicon_tags_236 },
    { 0xd20d0020, SUC_lexicon_tags_25 },
    { 0x8fed001f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x964d0023, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2d0026, SUC_lexicon_tags_11 },
    { 0x4c2d0027, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf16d0029, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30d002d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cd0030, SUC_lexicon_tags_15 },
    { 0xc8d0030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cd0036, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8d003b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c2d003d, SUC_lexicon_tags_38 },
    { 0x148d003e, SUC_lexicon_tags_37 },
    { 0x864d003f, SUC_lexicon_tags_85 },
    { 0x402d0040, SUC_lexicon_tags_31 },
    { 0xc42d0041, SUC_lexicon_tags_43 },
    { 0xd7cd0042, SUC_lexicon_tags_17 },
    { 0x824d0043, SUC_lexicon_tags_45 },
    { 0xf00d0042, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a4d0046, SUC_lexicon_tags_25 },
    { 0x142d0047, SUC_lexicon_tags_6 },
    { 0xc5ad0047, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608d0052, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x33cd0054, SUC_lexicon_tags_45 },
    { 0xe30d0054, SUC_lexicon_tags_19 },
    { 0x47cd0055, SUC_lexicon_tags_87 },
    { 0xb7ed0056, SUC_lexicon_tags_25 },
    { 0xba0d0058, SUC_lexicon_tags_11 },
    { 0x5a2d0059, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0d005d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef2d005f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x430d0062, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18cd0064, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ed0068, SUC_lexicon_tags_49 },
    { 0x8bad0069, SUC_lexicon_tags_57 },
    { 0x90cd006a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d0070, SUC_lexicon_tags_12 },
    { 0xf3cd0071, SUC_lexicon_tags_1 },
    { 0x1e6d0071, SUC_lexicon_tags_37 },
    { 0xc02d0073, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2d0076, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaad0079, SUC_lexicon_tags_15 },
    { 0x168d007a, SUC_lexicon_tags_11 },
    { 0xbf8d0079, SUC_lexicon_tags_11 },
    { 0x54cd0079, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2d007f, SUC_lexicon_tags_17 },
    { 0x8d6d0080, SUC_lexicon_tags_8 },
    { 0x600d007f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fed0084, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x30cd0087, SUC_lexicon_tags_48 },
    { 0xfaad0088, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46d008f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddad0095, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2d0098, SUC_lexicon_tags_12 },
    { 0xf4ad0099, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcad009e, SUC_lexicon_tags_25 },
    { 0x950d009f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fad00a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bcd00a3, SUC_lexicon_tags_57 },
    { 0xb16d00a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22d00aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb68d00ac, SUC_lexicon_tags_29 },
    { 0xef2d00ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd68d00af, SUC_lexicon_tags_38 },
    { 0x99ad00af, SUC_lexicon_tags_27 },
    { 0xe6d00b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ad00b4, SUC_lexicon_tags_9 },
    { 0x1d6d00b5, SUC_lexicon_tags_29 },
    { 0xb34d00b5, SUC_lexicon_tags_73 },
    { 0x930d00b7, SUC_lexicon_tags_26 },
    { 0x4aed00b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x370d00ba, SUC_lexicon_tags_21 },
    { 0x8d8d00ba, SUC_lexicon_tags_37 },
    { 0x46cd00bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ed00c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8d00c3, SUC_lexicon_tags_15 },
    { 0xd2ed00c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ed00c7, SUC_lexicon_tags_1 },
    { 0xdc8d00c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d00cb, SUC_lexicon_tags_23 },
    { 0x678d00cb, SUC_lexicon_tags_35 },
    { 0xd6ed00cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46ad00cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd18d00d1, SUC_lexicon_tags_19 },
    { 0xe62d00d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6ad00d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ed00d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbad00dd, SUC_lexicon_tags_2 },
    { 0x550d00de, SUC_lexicon_tags_11 },
    { 0x126d00df, SUC_lexicon_tags_15 },
    { 0x6e8d00dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd00e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984d00e9, SUC_lexicon_tags_23 },
    { 0x150d00e9, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cad00f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66d00f6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x162d00f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcd00fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6d00ff, SUC_lexicon_tags_43 },
    { 0xce4d00ff, SUC_lexicon_tags_38 },
    { 0xb06d0101, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322d0107, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf70d0109, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ed010e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cd0116, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8d0119, SUC_lexicon_tags_4 },
    { 0xd50d0119, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dad011d, SUC_lexicon_tags_12 },
    { 0x68ed011e, SUC_lexicon_tags_14 },
    { 0x17cd011e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeccd0121, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acd012a, SUC_lexicon_tags_17 },
    { 0x8a0d012b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x484d012d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecd0130, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38d0133, SUC_lexicon_tags_37 },
    { 0x838d0134, SUC_lexicon_tags_26 },
    { 0x9d2d0135, SUC_lexicon_tags_11 },
    { 0x204d0136, SUC_lexicon_tags_27 },
    { 0xf98d0134, SUC_lexicon_tags_37 },
    { 0x61cd0135, SUC_lexicon_tags_4 },
    { 0x9e2d0139, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292d013d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694d0144, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58d0149, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cd014d, SUC_lexicon_tags_112 },
    { 0x3bed014d, SUC_lexicon_tags_47 },
    { 0x614d014e, SUC_lexicon_tags_27 },
    { 0x3bad014f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x5a8d0152, SUC_lexicon_tags_11 },
    { 0x6a6d0153, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9eed0155, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x316d0157, SUC_lexicon_tags_15 },
    { 0x484d0158, SUC_lexicon_tags_8 },
    { 0xf74d0157, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x358d015c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22d015f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1cd0161, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeced0163, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdecd0166, SUC_lexicon_tags_11 },
    { 0x3e6d0167, SUC_lexicon_tags_63 },
    { 0xcc6d0168, SUC_lexicon_tags_2 },
    { 0x26cd0169, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x14d016b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2d016f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70d0173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f2d0175, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf04d0177, SUC_lexicon_tags_43 },
    { 0x79cd0177, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x750d017a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x850d017c, SUC_lexicon_tags_12 },
    { 0x256d017d, SUC_lexicon_tags_1 },
    { 0xb6ad017d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa00d0180, SUC_lexicon_tags_47 },
    { 0xb32d0180, SUC_lexicon_tags_25 },
    { 0x554d0182, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe90d0184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x816d0187, SUC_lexicon_tags_16 },
    { 0xf38d0188, SUC_lexicon_tags_17 },
    { 0xd7cd0189, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ed0190, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6d0194, SUC_lexicon_tags_12 },
    { 0x65ad0195, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60d0199, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8d019c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcf6d019e, SUC_lexicon_tags_12 },
    { 0x1a6d019f, SUC_lexicon_tags_6 },
    { 0x18d01a0, SUC_lexicon_tags_2 },
    { 0x2d8d01a1, SUC_lexicon_tags_65 },
    { 0x434d019f, SUC_lexicon_tags_0 },
    { 0x78ed01a3, SUC_lexicon_tags_37 },
    { 0x76cd019f, SUC_lexicon_tags_25 },
    { 0x3ad01a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ad01a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0d01ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd68d01af, SUC_lexicon_tags_25 },
    { 0x4bcd01b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfc2d01b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33cd01b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8d01b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2d01bb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x68d01bd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc14d01bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8d01c2, SUC_lexicon_tags_15 },
    { 0x698d01c3, SUC_lexicon_tags_13 },
    { 0xe70d01c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x200d01c7, SUC_lexicon_tags_47 },
    { 0xad2d01c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x150d01ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972d01d3, SUC_lexicon_tags_2 },
    { 0xba8d01d4, SUC_lexicon_tags_15 },
    { 0x852d01d3, SUC_lexicon_tags_37 },
    { 0xf7cd01d4, SUC_lexicon_tags_15 },
    { 0x8bcd01d7, SUC_lexicon_tags_25 },
    { 0x868d01d4, SUC_lexicon_tags_38 },
    { 0x47ed01d3, SUC_lexicon_tags_47 },
    { 0xd2cd01d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb3ad01dc, SUC_lexicon_tags_39 },
    { 0x320d01dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x816d01e0, SUC_lexicon_tags_57 },
    { 0xcaad01e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ded01e5, SUC_lexicon_tags_45 },
    { 0x40cd01e6, SUC_lexicon_tags_2 },
    { 0xd20d01e6, SUC_lexicon_tags_12 },
    { 0xe22d01e7, SUC_lexicon_tags_91 },
    { 0x138d01e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96d01ed, SUC_lexicon_tags_39 },
    { 0x91ed01ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4d01f1, SUC_lexicon_tags_32 },
    { 0x368d01f2, SUC_lexicon_tags_0 },
    { 0xb26d01f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88d01f7, SUC_lexicon_tags_7 },
    { 0x4b4d01f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6d01fc, SUC_lexicon_tags_27 },
    { 0xf62d01fd, SUC_lexicon_tags_28 },
    { 0x500d01fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0d0202, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ed020a, SUC_lexicon_tags_0 },
    { 0xb04d020b, SUC_lexicon_tags_46 },
    { 0x3b4d020c, SUC_lexicon_tags_19 },
    { 0xe0ed020d, SUC_lexicon_tags_25 },
    { 0x184d020d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x604d0210, SUC_lexicon_tags_37 },
    { 0x81cd0211, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcd0216, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8d0219, SUC_lexicon_tags_17 },
    { 0xe2ad0219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58d0220, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x766d0222, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2d0225, SUC_lexicon_tags_49 },
    { 0x4e4d0226, SUC_lexicon_tags_7 },
    { 0x3eed0227, SUC_lexicon_tags_17 },
    { 0xaded0228, SUC_lexicon_tags_6 },
    { 0x95ed0229, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8acd022b, SUC_lexicon_tags_49 },
    { 0x51cd022c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ad0232, SUC_lexicon_tags_37 },
    { 0xe64d0232, SUC_lexicon_tags_222 },
    { 0, NULL },
    { 0x5a2d0235, SUC_lexicon_tags_29 },
    { 0x664d0235, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0xc90d0238, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404d023c, SUC_lexicon_tags_11 },
    { 0x45ed023d, SUC_lexicon_tags_15 },
    { 0x152d023e, SUC_lexicon_tags_1 },
    { 0x5a2d023f, SUC_lexicon_tags_27 },
    { 0xac6d023f, SUC_lexicon_tags_25 },
    { 0xcf0d0241, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8d0247, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xafed0249, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536d024d, SUC_lexicon_tags_17 },
    { 0x80ad024e, SUC_lexicon_tags_26 },
    { 0x306d024e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960d0255, SUC_lexicon_tags_27 },
    { 0x5ced0256, SUC_lexicon_tags_33 },
    { 0x130d0257, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0d025a, SUC_lexicon_tags_62 },
    { 0x2dad025a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bed025f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2d0262, SUC_lexicon_tags_39 },
    { 0x7f2d0262, SUC_lexicon_tags_27 },
    { 0x1ecd0262, SUC_lexicon_tags_37 },
    { 0x802d0265, SUC_lexicon_tags_37 },
    { 0xb84d0265, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cad0269, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbed026c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xadad026e, SUC_lexicon_tags_7 },
    { 0xebcd026f, SUC_lexicon_tags_27 },
    { 0x760d026f, SUC_lexicon_tags_22 },
    { 0x552d026f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x174d0274, SUC_lexicon_tags_27 },
    { 0x79ad0275, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4d027b, SUC_lexicon_tags_16 },
    { 0xcd6d027b, SUC_lexicon_tags_26 },
    { 0xaa0d027d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd0280, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ad0286, SUC_lexicon_tags_16 },
    { 0x8d2d0286, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x408d0289, SUC_lexicon_tags_17 },
    { 0xb0cd028a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952d028e, SUC_lexicon_tags_66 },
    { 0x74cd028e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0d0293, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acd0297, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4c8d0299, SUC_lexicon_tags_8 },
    { 0x1aad029a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ecd029c, SUC_lexicon_tags_45 },
    { 0x438d029c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ed02a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0d02a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd72d02a8, SUC_lexicon_tags_2 },
    { 0x680d02a8, SUC_lexicon_tags_27 },
    { 0x6b8d02a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592d02b0, SUC_lexicon_tags_68 },
    { 0xc6d02b0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd14d02b3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcd02b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ad02bc, SUC_lexicon_tags_12 },
    { 0xa5cd02bd, SUC_lexicon_tags_11 },
    { 0x64cd02be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cd02c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262d02c5, SUC_lexicon_tags_25 },
    { 0xd98d02c6, SUC_lexicon_tags_87 },
    { 0x76cd02c7, SUC_lexicon_tags_22 },
    { 0xe30d02c8, SUC_lexicon_tags_11 },
    { 0xcd2d02c9, SUC_lexicon_tags_37 },
    { 0xfbad02ca, SUC_lexicon_tags_37 },
    { 0xdbad02ca, SUC_lexicon_tags_1394 },
    { 0x21cd02cc, SUC_lexicon_tags_33 },
    { 0xc38d02cd, SUC_lexicon_tags_7 },
    { 0x498d02cc, SUC_lexicon_tags_17 },
    { 0xd02d02ce, SUC_lexicon_tags_12 },
    { 0xde8d02cd, SUC_lexicon_tags_11 },
    { 0x3cd02d1, SUC_lexicon_tags_17 },
    { 0xcf6d02c5, SUC_lexicon_tags_34 },
    { 0x9fed02d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0d02d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ed02dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0d02e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d02e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd4d02e9, SUC_lexicon_tags_30 },
    { 0x17ed02e9, SUC_lexicon_tags_55 },
    { 0xcdcd02eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ecd02ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ad02f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2d02f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb0d02f8, SUC_lexicon_tags_37 },
    { 0x388d02f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76d02fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cd0300, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cd0305, SUC_lexicon_tags_38 },
    { 0xbc2d0306, SUC_lexicon_tags_15 },
    { 0xc00d0306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cd030a, SUC_lexicon_tags_7 },
    { 0x88cd030b, SUC_lexicon_tags_37 },
    { 0xe7cd030b, SUC_lexicon_tags_11 },
    { 0x8e8d030d, SUC_lexicon_tags_15 },
    { 0x4aad030e, SUC_lexicon_tags_37 },
    { 0xdfed030e, SUC_lexicon_tags_37 },
    { 0x532d0310, SUC_lexicon_tags_63 },
    { 0x9ded030a, SUC_lexicon_tags_29 },
    { 0xfe4d0312, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe02d0314, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ad031c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16d031e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa24d0320, SUC_lexicon_tags_1 },
    { 0x8fed0320, SUC_lexicon_tags_7 },
    { 0x710d0322, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20d0328, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdf6d032a, SUC_lexicon_tags_27 },
    { 0xe9ed032b, SUC_lexicon_tags_19 },
    { 0xeb0d032b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6d0331, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ced0335, SUC_lexicon_tags_15 },
    { 0xb0ed0336, SUC_lexicon_tags_1 },
    { 0x3fed0336, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ead033c, SUC_lexicon_tags_63 },
    { 0x6fcd033c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbad0346, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ad034d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cd0350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fed0355, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecd0358, SUC_lexicon_tags_14 },
    { 0x2a8d0359, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ad035c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8d0361, SUC_lexicon_tags_2 },
    { 0x432d0361, SUC_lexicon_tags_0 },
    { 0xaad0362, SUC_lexicon_tags_38 },
    { 0x5fed0364, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410d0370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacd0374, SUC_lexicon_tags_2 },
    { 0x454d0374, SUC_lexicon_tags_0 },
    { 0x70d0376, SUC_lexicon_tags_13 },
    { 0xea2d0377, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938d037e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d0385, SUC_lexicon_tags_11 },
    { 0xdf0d0385, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb0d0388, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f8d038a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ad038f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176d0396, SUC_lexicon_tags_37 },
    { 0x6b0d0397, SUC_lexicon_tags_7 },
    { 0x1f6d0397, SUC_lexicon_tags_15 },
    { 0xb64d0398, SUC_lexicon_tags_17 },
    { 0xb84d039a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4d039d, SUC_lexicon_tags_12 },
    { 0x4c8d039d, SUC_lexicon_tags_148 },
    { 0xcead039f, SUC_lexicon_tags_15 },
    { 0x39ad03a0, SUC_lexicon_tags_6 },
    { 0x534d03a1, SUC_lexicon_tags_40 },
    { 0x59ad03a2, SUC_lexicon_tags_37 },
    { 0x176d03a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcd03a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x854d03ab, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x42cd03ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ad03b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ad03b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ed03be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7cd03c0, SUC_lexicon_tags_2 },
    { 0x2eed03c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8acd03c4, SUC_lexicon_tags_47 },
    { 0xf98d03c4, SUC_lexicon_tags_77 },
    { 0xe6ad03c5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8d03cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d03d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x38ad03da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbf4d03dc, SUC_lexicon_tags_15 },
    { 0xff4d03dc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126d03e2, SUC_lexicon_tags_33 },
    { 0x4ccd03e2, SUC_lexicon_tags_15 },
    { 0xf94d03e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57cd03e6, SUC_lexicon_tags_84 },
    { 0x2cd03e7, SUC_lexicon_tags_25 },
    { 0xf7ed03e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x34cd03ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfcd03ec, SUC_lexicon_tags_7 },
    { 0xb96d03ed, SUC_lexicon_tags_23 },
    { 0x106d03ed, SUC_lexicon_tags_27 },
    { 0x5bad03ec, SUC_lexicon_tags_15 },
    { 0x47ad03f0, SUC_lexicon_tags_37 },
    { 0x97ad03f0, SUC_lexicon_tags_26 },
    { 0x408d03ec, SUC_lexicon_tags_47 },
    { 0xad8d03f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed03f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aed03fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2ad03ff, SUC_lexicon_tags_37 },
    { 0xbcd03ff, SUC_lexicon_tags_1 },
    { 0xa66d0400, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2d0405, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8d0408, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf2ad040a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4ad040c, SUC_lexicon_tags_53 },
    { 0x920d040d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccd0411, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356d041b, SUC_lexicon_tags_1 },
    { 0x432d041b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924d0420, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2fed0422, SUC_lexicon_tags_2 },
    { 0xbf0d0423, SUC_lexicon_tags_55 },
    { 0x440d0423, SUC_lexicon_tags_37 },
    { 0x310d0425, SUC_lexicon_tags_25 },
    { 0x5e8d0426, SUC_lexicon_tags_49 },
    { 0x5fcd0423, SUC_lexicon_tags_27 },
    { 0xf0d0428, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61cd042a, SUC_lexicon_tags_11 },
    { 0xa4cd042b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4cd042d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x364d042f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26d0432, SUC_lexicon_tags_66 },
    { 0xa28d0432, SUC_lexicon_tags_17 },
    { 0xb26d0434, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344d0439, SUC_lexicon_tags_49 },
    { 0xc22d043a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8d0442, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8d0444, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cd0448, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0d044b, SUC_lexicon_tags_14 },
    { 0x24d044c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x44cd044e, SUC_lexicon_tags_98 },
    { 0x62ad044e, SUC_lexicon_tags_25 },
    { 0x154d0450, SUC_lexicon_tags_47 },
    { 0x222d044f, SUC_lexicon_tags_63 },
    { 0xbbed044e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98cd0455, SUC_lexicon_tags_17 },
    { 0x610d0455, SUC_lexicon_tags_2 },
    { 0x610d0455, SUC_lexicon_tags_7 },
    { 0xa02d0455, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b8d045a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9ed045c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cd0461, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990d0465, SUC_lexicon_tags_17 },
    { 0x718d0466, SUC_lexicon_tags_25 },
    { 0x648d0467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778d046d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0d0471, SUC_lexicon_tags_47 },
    { 0xd34d0472, SUC_lexicon_tags_1 },
    { 0xfcad0473, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4d0476, SUC_lexicon_tags_12 },
    { 0x7e2d0477, SUC_lexicon_tags_17 },
    { 0xa22d0478, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xae2d047a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88d047d, SUC_lexicon_tags_87 },
    { 0x958d047d, SUC_lexicon_tags_31 },
    { 0x8fcd047e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52ed0481, SUC_lexicon_tags_65 },
    { 0x84ed0482, SUC_lexicon_tags_60 },
    { 0xc32d0483, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfd4d0485, SUC_lexicon_tags_11 },
    { 0x4c8d0485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582d048f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3cd0491, SUC_lexicon_tags_39 },
    { 0x2c4d0492, SUC_lexicon_tags_26 },
    { 0xa28d0492, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc3ad0495, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd0cd0497, SUC_lexicon_tags_25 },
    { 0xeb0d0498, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dcd049a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7cd049c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dad049f, SUC_lexicon_tags_33 },
    { 0x7a2d04a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d04a4, SUC_lexicon_tags_23 },
    { 0x45ed04a4, SUC_lexicon_tags_11 },
    { 0xa02d04a6, SUC_lexicon_tags_82 },
    { 0x116d04a6, SUC_lexicon_tags_11 },
    { 0x44d04a5, SUC_lexicon_tags_16 },
    { 0xd8ad04a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e6d04ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x162d04ad, SUC_lexicon_tags_7 },
    { 0x3e8d04ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x510d04b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ad04b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8d04b6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980d04ba, SUC_lexicon_tags_33 },
    { 0xfad04bb, SUC_lexicon_tags_37 },
    { 0x3d0d04bc, SUC_lexicon_tags_14 },
    { 0xabad04bc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x75ad04bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aad04cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaad04d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2d04d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20d04d9, SUC_lexicon_tags_84 },
    { 0x4e2d04da, SUC_lexicon_tags_170 },
    { 0x8c8d04db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846d04df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ed04e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cd04e8, SUC_lexicon_tags_27 },
    { 0x114d04e8, SUC_lexicon_tags_25 },
    { 0xcb2d04e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0d04ed, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4ccd04ef, SUC_lexicon_tags_31 },
    { 0x67ad04f0, SUC_lexicon_tags_11 },
    { 0x13ed04f1, SUC_lexicon_tags_82 },
    { 0xddad04f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358d04fa, SUC_lexicon_tags_2 },
    { 0x36ed04fb, SUC_lexicon_tags_6 },
    { 0x5aed04fc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced0500, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x130d0502, SUC_lexicon_tags_49 },
    { 0xb4ed0503, SUC_lexicon_tags_15 },
    { 0x3eed0504, SUC_lexicon_tags_4 },
    { 0xde4d0503, SUC_lexicon_tags_77 },
    { 0xb8cd0506, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16d0509, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x918d050c, SUC_lexicon_tags_19 },
    { 0x654d050d, SUC_lexicon_tags_37 },
    { 0xfacd050e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6d0515, SUC_lexicon_tags_46 },
    { 0xf2d0515, SUC_lexicon_tags_21 },
    { 0xd96d0517, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88d051a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ded051d, SUC_lexicon_tags_12 },
    { 0xd5ed051e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ed0525, SUC_lexicon_tags_68 },
    { 0xd66d0525, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cd052b, SUC_lexicon_tags_12 },
    { 0x22ed052c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd08d052e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5b0d0530, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d0535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8d0538, SUC_lexicon_tags_37 },
    { 0x3d2d0538, SUC_lexicon_tags_25 },
    { 0xe76d0538, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f4d053c, SUC_lexicon_tags_19 },
    { 0xd88d053d, SUC_lexicon_tags_11 },
    { 0xe74d053e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38d0542, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38cd0544, SUC_lexicon_tags_63 },
    { 0xd18d0545, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e6d0547, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58d054b, SUC_lexicon_tags_23 },
    { 0x81ad054b, SUC_lexicon_tags_1 },
    { 0x320d054b, SUC_lexicon_tags_46 },
    { 0xa28d054c, SUC_lexicon_tags_35 },
    { 0xe7ed054e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x602d0551, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ed0557, SUC_lexicon_tags_17 },
    { 0xb8ed0558, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ed055c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x888d055e, SUC_lexicon_tags_14 },
    { 0x646d055f, SUC_lexicon_tags_15 },
    { 0x396d0560, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aad0566, SUC_lexicon_tags_37 },
    { 0xbaad0567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdad056c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bad056f, SUC_lexicon_tags_692 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34d0572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe78d0574, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ed057c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0d0581, SUC_lexicon_tags_11 },
    { 0x7f0d0582, SUC_lexicon_tags_0 },
    { 0x6e6d0583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344d0587, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cd058e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384d0592, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962d0598, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ad059d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56d05a1, SUC_lexicon_tags_7 },
    { 0xc20d05a2, SUC_lexicon_tags_7 },
    { 0x162d05a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x110d05a6, SUC_lexicon_tags_1 },
    { 0x20d05a6, SUC_lexicon_tags_17 },
    { 0xb1cd05a6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x136d05ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee8d05ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x128d05b0, SUC_lexicon_tags_13 },
    { 0x22ed05b0, SUC_lexicon_tags_17 },
    { 0x218d05b2, SUC_lexicon_tags_174 },
    { 0x624d05b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x654d05b5, SUC_lexicon_tags_7 },
    { 0xd34d05b6, SUC_lexicon_tags_17 },
    { 0x2d05b7, SUC_lexicon_tags_2 },
    { 0x3bad05b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74d05bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816d05c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42d05c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8d05cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cd05d4, SUC_lexicon_tags_74 },
    { 0x7f2d05d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4ad05d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cd05de, SUC_lexicon_tags_43 },
    { 0x9ccd05df, SUC_lexicon_tags_19 },
    { 0x630d05e0, SUC_lexicon_tags_37 },
    { 0xa0ed05e0, SUC_lexicon_tags_25 },
    { 0x41ad05e2, SUC_lexicon_tags_49 },
    { 0xbaad05e2, SUC_lexicon_tags_37 },
    { 0x9eed05e4, SUC_lexicon_tags_47 },
    { 0x574d05e5, SUC_lexicon_tags_35 },
    { 0xd38d05e5, SUC_lexicon_tags_65 },
    { 0xf46d05e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d4d05e9, SUC_lexicon_tags_11 },
    { 0xa28d05ea, SUC_lexicon_tags_17 },
    { 0x2c0d05ea, SUC_lexicon_tags_32 },
    { 0x220d05ec, SUC_lexicon_tags_49 },
    { 0x32d05ea, SUC_lexicon_tags_11 },
    { 0xdbed05ee, SUC_lexicon_tags_60 },
    { 0x9b8d05ef, SUC_lexicon_tags_25 },
    { 0x9fad05f0, SUC_lexicon_tags_0 },
    { 0x2ad05f0, SUC_lexicon_tags_9 },
    { 0x220d05f2, SUC_lexicon_tags_107 },
    { 0x936d05f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8f6d05f5, SUC_lexicon_tags_39 },
    { 0xd0ed05f5, SUC_lexicon_tags_14 },
    { 0xcf6d05f7, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x606d05f9, SUC_lexicon_tags_12 },
    { 0x908d05fa, SUC_lexicon_tags_68 },
    { 0x5ecd05fb, SUC_lexicon_tags_17 },
    { 0xaecd05fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134d0602, SUC_lexicon_tags_38 },
    { 0xf08d0603, SUC_lexicon_tags_544 },
    { 0, NULL },
    { 0x770d0605, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x952d0607, SUC_lexicon_tags_7 },
    { 0x588d0607, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc34d060a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cd060d, SUC_lexicon_tags_25 },
    { 0xf20d060e, SUC_lexicon_tags_17 },
    { 0x89cd060e, SUC_lexicon_tags_25 },
    { 0x420d0610, SUC_lexicon_tags_60 },
    { 0xc0cd0611, SUC_lexicon_tags_33 },
    { 0x6ad0612, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x296d0614, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeaad0616, SUC_lexicon_tags_47 },
    { 0x43cd0616, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18cd0619, SUC_lexicon_tags_26 },
    { 0xd4ad061a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62d061e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfad0622, SUC_lexicon_tags_12 },
    { 0x4a2d0622, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26d0626, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bad062c, SUC_lexicon_tags_7 },
    { 0x48ad062d, SUC_lexicon_tags_4 },
    { 0x2e6d062e, SUC_lexicon_tags_26 },
    { 0xedcd062e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20d0634, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe58d0636, SUC_lexicon_tags_17 },
    { 0xd28d0637, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ed0639, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74d063d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ed0642, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74ad0644, SUC_lexicon_tags_47 },
    { 0xa70d0644, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ad0648, SUC_lexicon_tags_2 },
    { 0xa74d0649, SUC_lexicon_tags_19 },
    { 0xcd8d0648, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc92d064c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x756d064f, SUC_lexicon_tags_0 },
    { 0xf2ad0650, SUC_lexicon_tags_47 },
    { 0x992d0650, SUC_lexicon_tags_17 },
    { 0x6bcd0652, SUC_lexicon_tags_19 },
    { 0x65cd064f, SUC_lexicon_tags_113 },
    { 0x9bad064f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ed0659, SUC_lexicon_tags_25 },
    { 0x944d0659, SUC_lexicon_tags_23 },
    { 0x4f8d0659, SUC_lexicon_tags_43 },
    { 0x754d065c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38d0660, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ded0665, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ad066e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x838d0670, SUC_lexicon_tags_1 },
    { 0x436d0671, SUC_lexicon_tags_15 },
    { 0xea2d0671, SUC_lexicon_tags_65 },
    { 0x36cd0671, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ad0676, SUC_lexicon_tags_17 },
    { 0x250d0677, SUC_lexicon_tags_1 },
    { 0x92ed0676, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fed067d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2d0680, SUC_lexicon_tags_15 },
    { 0x372d0680, SUC_lexicon_tags_12 },
    { 0x994d0682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302d0686, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d068b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04d068f, SUC_lexicon_tags_4 },
    { 0x1aad0690, SUC_lexicon_tags_37 },
    { 0x134d0691, SUC_lexicon_tags_16 },
    { 0xf62d0691, SUC_lexicon_tags_1 },
    { 0x138d0690, SUC_lexicon_tags_32 },
    { 0x7e6d0691, SUC_lexicon_tags_11 },
    { 0x4dcd0692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc54d0697, SUC_lexicon_tags_26 },
    { 0xa98d0698, SUC_lexicon_tags_14 },
    { 0xe8ad0698, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850d06a2, SUC_lexicon_tags_38 },
    { 0x818d06a3, SUC_lexicon_tags_12 },
    { 0x416d06a3, SUC_lexicon_tags_34 },
    { 0x340d06a2, SUC_lexicon_tags_48 },
    { 0x1c6d06a6, SUC_lexicon_tags_37 },
    { 0xd9cd06a6, SUC_lexicon_tags_13 },
    { 0xa86d06a2, SUC_lexicon_tags_25 },
    { 0x678d06a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2d06af, SUC_lexicon_tags_12 },
    { 0xb8ad06b0, SUC_lexicon_tags_6 },
    { 0xeced06b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ded06b3, SUC_lexicon_tags_0 },
    { 0xaced06b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8d06b7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd2ad06b9, SUC_lexicon_tags_29 },
    { 0x86ad06ba, SUC_lexicon_tags_2 },
    { 0xde4d06bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb88d06bd, SUC_lexicon_tags_25 },
    { 0x81ed06be, SUC_lexicon_tags_5 },
    { 0x6fcd06be, SUC_lexicon_tags_264 },
    { 0x7ad06c0, SUC_lexicon_tags_14 },
    { 0xbbcd06bf, SUC_lexicon_tags_29 },
    { 0xeccd06c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926d06c6, SUC_lexicon_tags_13 },
    { 0x3e8d06c7, SUC_lexicon_tags_950 },
    { 0xc26d06c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cad06cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80d06d3, SUC_lexicon_tags_27 },
    { 0xb54d06d3, SUC_lexicon_tags_26 },
    { 0x1e8d06d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cd06d9, SUC_lexicon_tags_2 },
    { 0xf8ad06d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ad06df, SUC_lexicon_tags_48 },
    { 0x9ccd06e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2d06e5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784d06e9, SUC_lexicon_tags_15 },
    { 0xcb6d06ea, SUC_lexicon_tags_25 },
    { 0xfcad06eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62d06f1, SUC_lexicon_tags_7 },
    { 0x8ccd06f1, SUC_lexicon_tags_48 },
    { 0x4f6d06f3, SUC_lexicon_tags_11 },
    { 0x502d06f1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbecd06f6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4d06fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x378d06ff, SUC_lexicon_tags_7 },
    { 0xb78d0700, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcd0706, SUC_lexicon_tags_15 },
    { 0xd72d0707, SUC_lexicon_tags_37 },
    { 0x15cd0707, SUC_lexicon_tags_7 },
    { 0x9d4d0709, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x464d070b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04d070e, SUC_lexicon_tags_25 },
    { 0x19ad070f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648d0717, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ad071c, SUC_lexicon_tags_55 },
    { 0xa18d071d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100d0726, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x740d0728, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cd072b, SUC_lexicon_tags_57 },
    { 0x72ed072b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cd0732, SUC_lexicon_tags_17 },
    { 0x82cd0733, SUC_lexicon_tags_15 },
    { 0xa6cd0734, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8fad0736, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x224d0738, SUC_lexicon_tags_37 },
    { 0xcd0d0738, SUC_lexicon_tags_12 },
    { 0x720d0738, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ed0746, SUC_lexicon_tags_1 },
    { 0x4d4d0747, SUC_lexicon_tags_9 },
    { 0x46ad0747, SUC_lexicon_tags_15 },
    { 0xfd2d0749, SUC_lexicon_tags_19 },
    { 0x46ad0747, SUC_lexicon_tags_87 },
    { 0x58cd0747, SUC_lexicon_tags_25 },
    { 0x2d0d074c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6d074f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7f2d0751, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cd0754, SUC_lexicon_tags_133 },
    { 0xb98d0755, SUC_lexicon_tags_14 },
    { 0xd96d0756, SUC_lexicon_tags_8 },
    { 0x38ad0757, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6d075a, SUC_lexicon_tags_0 },
    { 0x164d075b, SUC_lexicon_tags_33 },
    { 0x540d075c, SUC_lexicon_tags_25 },
    { 0x7ded075d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ed0764, SUC_lexicon_tags_25 },
    { 0x93ed0764, SUC_lexicon_tags_43 },
    { 0xeecd0766, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa06d0768, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x326d076a, SUC_lexicon_tags_15 },
    { 0x46ed076b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18d076e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352d0774, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde6d0776, SUC_lexicon_tags_82 },
    { 0xd40d0777, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2d0784, SUC_lexicon_tags_0 },
    { 0x52ad0784, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692d0789, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422d0793, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2d0796, SUC_lexicon_tags_12 },
    { 0x538d0796, SUC_lexicon_tags_11 },
    { 0xff6d0797, SUC_lexicon_tags_11 },
    { 0x7b0d0797, SUC_lexicon_tags_25 },
    { 0x330d079a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bed07a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x824d07a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4d07a9, SUC_lexicon_tags_45 },
    { 0xd1ad07aa, SUC_lexicon_tags_37 },
    { 0xfc0d07aa, SUC_lexicon_tags_47 },
    { 0x916d07ac, SUC_lexicon_tags_10 },
    { 0x492d07ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabed07b1, SUC_lexicon_tags_15 },
    { 0x54ed07b2, SUC_lexicon_tags_15 },
    { 0x980d07b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94d07bc, SUC_lexicon_tags_47 },
    { 0x76cd07bc, SUC_lexicon_tags_47 },
    { 0x1f2d07be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06d07c1, SUC_lexicon_tags_65 },
    { 0xd64d07c2, SUC_lexicon_tags_25 },
    { 0x826d07c3, SUC_lexicon_tags_48 },
    { 0xc4cd07c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd07c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbc6d07c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3cd07cb, SUC_lexicon_tags_14 },
    { 0xaa0d07cc, SUC_lexicon_tags_23 },
    { 0x832d07cb, SUC_lexicon_tags_17 },
    { 0xc20d07cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b8d07d0, SUC_lexicon_tags_15 },
    { 0x27ed07d0, SUC_lexicon_tags_27 },
    { 0x924d07d1, SUC_lexicon_tags_27 },
    { 0xa8ad07d2, SUC_lexicon_tags_7 },
    { 0x5ead07d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x936d07d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x516d07d9, SUC_lexicon_tags_38 },
    { 0xf50d07da, SUC_lexicon_tags_11 },
    { 0xe70d07d9, SUC_lexicon_tags_32 },
    { 0xd12d07db, SUC_lexicon_tags_87 },
    { 0xda0d07dd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cd07e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0d07e8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4d07eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb60d07ee, SUC_lexicon_tags_26 },
    { 0x5a8d07ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cd07f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eed07f8, SUC_lexicon_tags_49 },
    { 0xe50d07f9, SUC_lexicon_tags_191 },
    { 0xfa8d07f9, SUC_lexicon_tags_25 },
    { 0x51ed07f8, SUC_lexicon_tags_30 },
    { 0x8a6d07fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44d0801, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d0805, SUC_lexicon_tags_60 },
    { 0x740d0806, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6d080a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x204d080c, SUC_lexicon_tags_30 },
    { 0x15ed080d, SUC_lexicon_tags_7 },
    { 0x57ed080c, SUC_lexicon_tags_17 },
    { 0xbe6d080f, SUC_lexicon_tags_29 },
    { 0x208d0810, SUC_lexicon_tags_26 },
    { 0x7b0d0810, SUC_lexicon_tags_12 },
    { 0x252d0812, SUC_lexicon_tags_15 },
    { 0x7eed0813, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8d0818, SUC_lexicon_tags_15 },
    { 0x38ad0819, SUC_lexicon_tags_29 },
    { 0x3f8d0818, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cd081e, SUC_lexicon_tags_25 },
    { 0x5ced081f, SUC_lexicon_tags_47 },
    { 0x5fed081f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72d0825, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcd082b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ad0830, SUC_lexicon_tags_64 },
    { 0x7fad0831, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330d083a, SUC_lexicon_tags_31 },
    { 0xe7cd083b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x492d083d, SUC_lexicon_tags_17 },
    { 0xc4cd083d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00d0846, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ded0849, SUC_lexicon_tags_1 },
    { 0x768d084a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60d084d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x880d0850, SUC_lexicon_tags_17 },
    { 0x336d0850, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122d0856, SUC_lexicon_tags_23 },
    { 0xb08d0856, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0d085c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x412d085e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432d0862, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684d0866, SUC_lexicon_tags_15 },
    { 0x296d0866, SUC_lexicon_tags_29 },
    { 0x7b6d0868, SUC_lexicon_tags_63 },
    { 0x41ad0869, SUC_lexicon_tags_47 },
    { 0x1f6d086a, SUC_lexicon_tags_15 },
    { 0x492d086b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cd0870, SUC_lexicon_tags_37 },
    { 0xc4ed0870, SUC_lexicon_tags_28 },
    { 0x1c2d0870, SUC_lexicon_tags_47 },
    { 0x66ad0870, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5e0d0875, SUC_lexicon_tags_2 },
    { 0x3f0d0875, SUC_lexicon_tags_38 },
    { 0xc7ad0876, SUC_lexicon_tags_12 },
    { 0x6e4d0878, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6d087d, SUC_lexicon_tags_1 },
    { 0x80ad087e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ed0883, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4ed0885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18d0889, SUC_lexicon_tags_87 },
    { 0xfbcd088a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xead088d, SUC_lexicon_tags_15 },
    { 0x756d088d, SUC_lexicon_tags_25 },
    { 0xf16d088f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x832d0892, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558d0897, SUC_lexicon_tags_7 },
    { 0x4e2d0897, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad089e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x558d08a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8d08a6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x242d08a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ed08ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x26cd08ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cd08b6, SUC_lexicon_tags_7 },
    { 0xdccd08b7, SUC_lexicon_tags_27 },
    { 0xdb2d08b7, SUC_lexicon_tags_49 },
    { 0xefad08b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ed08c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd84d08c2, SUC_lexicon_tags_17 },
    { 0x50cd08c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172d08c8, SUC_lexicon_tags_44 },
    { 0x8c0d08c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabed08cd, SUC_lexicon_tags_37 },
    { 0x54d08ce, SUC_lexicon_tags_37 },
    { 0xeeed08ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1eed08d1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbed08db, SUC_lexicon_tags_37 },
    { 0x1ed08db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ad08e2, SUC_lexicon_tags_15 },
    { 0xdecd08e3, SUC_lexicon_tags_31 },
    { 0x90ad08e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ed08eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbad08ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd08f0, SUC_lexicon_tags_38 },
    { 0xa82d08f1, SUC_lexicon_tags_37 },
    { 0x6d6d08f0, SUC_lexicon_tags_25 },
    { 0xb44d08f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d08f8, SUC_lexicon_tags_77 },
    { 0x7d6d08f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256d08fd, SUC_lexicon_tags_25 },
    { 0xe82d08fe, SUC_lexicon_tags_17 },
    { 0x51ed08ff, SUC_lexicon_tags_17 },
    { 0x174d08ff, SUC_lexicon_tags_63 },
    { 0x2aad0901, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ad090c, SUC_lexicon_tags_19 },
    { 0x546d090c, SUC_lexicon_tags_1 },
    { 0xe04d090e, SUC_lexicon_tags_25 },
    { 0xf74d090f, SUC_lexicon_tags_11 },
    { 0x46ed0910, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0d0915, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204d0921, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x996d0924, SUC_lexicon_tags_25 },
    { 0x64ad0925, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ed0932, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x962d0934, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26d0937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ded093c, SUC_lexicon_tags_37 },
    { 0xe32d093c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0d0941, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccd0945, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6d0949, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x174d094c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78ed094e, SUC_lexicon_tags_66 },
    { 0x68d094e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaf2d0951, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18cd0953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cd0959, SUC_lexicon_tags_45 },
    { 0x3b0d095a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ad095d, SUC_lexicon_tags_16 },
    { 0xbcad095d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bed0961, SUC_lexicon_tags_6 },
    { 0x5c8d0962, SUC_lexicon_tags_19 },
    { 0xdfed0962, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fcd0965, SUC_lexicon_tags_25 },
    { 0xabad0965, SUC_lexicon_tags_43 },
    { 0x9acd0967, SUC_lexicon_tags_44 },
    { 0x424d0965, SUC_lexicon_tags_55 },
    { 0x26d0969, SUC_lexicon_tags_31 },
    { 0x1f2d0966, SUC_lexicon_tags_11 },
    { 0xfb8d096b, SUC_lexicon_tags_26 },
    { 0x7ad096c, SUC_lexicon_tags_0 },
    { 0x18ad096b, SUC_lexicon_tags_12 },
    { 0x486d0965, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcecd0970, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x820d0972, SUC_lexicon_tags_1 },
    { 0xc5cd0972, SUC_lexicon_tags_23 },
    { 0xc28d0972, SUC_lexicon_tags_1 },
    { 0x468d0975, SUC_lexicon_tags_68 },
    { 0x89ed0976, SUC_lexicon_tags_1 },
    { 0x298d0975, SUC_lexicon_tags_10 },
    { 0xbaed0978, SUC_lexicon_tags_28 },
    { 0xa62d0975, SUC_lexicon_tags_14 },
    { 0xc78d097a, SUC_lexicon_tags_27 },
    { 0xed6d0976, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4f8d097d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x450d097f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6d0986, SUC_lexicon_tags_44 },
    { 0xc3ed0987, SUC_lexicon_tags_57 },
    { 0x51ed0987, SUC_lexicon_tags_37 },
    { 0xa84d0989, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97cd098b, SUC_lexicon_tags_49 },
    { 0xacad098c, SUC_lexicon_tags_37 },
    { 0xfe0d098c, SUC_lexicon_tags_38 },
    { 0xdf8d098c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928d0994, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cd0998, SUC_lexicon_tags_12 },
    { 0x324d0998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x282d099b, SUC_lexicon_tags_2 },
    { 0xc24d099c, SUC_lexicon_tags_68 },
    { 0x2f8d099c, SUC_lexicon_tags_17 },
    { 0xd0cd099e, SUC_lexicon_tags_47 },
    { 0xdcad099f, SUC_lexicon_tags_12 },
    { 0xd32d099e, SUC_lexicon_tags_35 },
    { 0xbf6d099b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ed09ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ed09b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ead09bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa3ed09bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6d09c9, SUC_lexicon_tags_3 },
    { 0xecd09ca, SUC_lexicon_tags_47 },
    { 0xc2cd09ca, SUC_lexicon_tags_2 },
    { 0x910d09cc, SUC_lexicon_tags_17 },
    { 0x1c8d09ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad09d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x288d09d8, SUC_lexicon_tags_45 },
    { 0xe0ed09d9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x942d09dc, SUC_lexicon_tags_10 },
    { 0x5dad09dd, SUC_lexicon_tags_1 },
    { 0xbc0d09dd, SUC_lexicon_tags_40 },
    { 0x250d09df, SUC_lexicon_tags_37 },
    { 0x90cd09e0, SUC_lexicon_tags_1 },
    { 0xe58d09e1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x122d09e3, SUC_lexicon_tags_11 },
    { 0x780d09e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4d09e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x844d09e9, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d09ee, SUC_lexicon_tags_39 },
    { 0xb5ed09ee, SUC_lexicon_tags_49 },
    { 0x700d09f0, SUC_lexicon_tags_15 },
    { 0x30ed09f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0d09f4, SUC_lexicon_tags_21 },
    { 0x15cd09f5, SUC_lexicon_tags_2 },
    { 0xf1ad09f6, SUC_lexicon_tags_48 },
    { 0xfc2d09f6, SUC_lexicon_tags_31 },
    { 0x392d09f6, SUC_lexicon_tags_9 },
    { 0x2f4d09f9, SUC_lexicon_tags_55 },
    { 0x3f2d09f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6d09ff, SUC_lexicon_tags_17 },
    { 0xb64d09ff, SUC_lexicon_tags_16 },
    { 0xa4cd09ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x722d0a04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ed0a0b, SUC_lexicon_tags_5 },
    { 0x1ad0a0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d0a10, SUC_lexicon_tags_1 },
    { 0x206d0a11, SUC_lexicon_tags_79 },
    { 0xfe4d0a12, SUC_lexicon_tags_11 },
    { 0x75ad0a12, SUC_lexicon_tags_15 },
    { 0x79ad0a14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2a8d0a16, SUC_lexicon_tags_25 },
    { 0x736d0a17, SUC_lexicon_tags_19 },
    { 0xa44d0a17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0d0a1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x156d0a20, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ad0a26, SUC_lexicon_tags_15 },
    { 0xc8ad0a26, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2d0a2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x838d0a32, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4eed0a34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccd0a37, SUC_lexicon_tags_15 },
    { 0x154d0a38, SUC_lexicon_tags_6 },
    { 0x2f6d0a39, SUC_lexicon_tags_30 },
    { 0xb14d0a3a, SUC_lexicon_tags_17 },
    { 0xaa0d0a3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ed0a3f, SUC_lexicon_tags_33 },
    { 0x4c2d0a40, SUC_lexicon_tags_37 },
    { 0xe12d0a40, SUC_lexicon_tags_87 },
    { 0xec2d0a3f, SUC_lexicon_tags_11 },
    { 0x1e2d0a43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0d0a49, SUC_lexicon_tags_19 },
    { 0x402d0a4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf54d0a4c, SUC_lexicon_tags_49 },
    { 0x464d0a4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cd0a52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x756d0a54, SUC_lexicon_tags_37 },
    { 0x990d0a55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5cd0a57, SUC_lexicon_tags_11 },
    { 0x2b0d0a58, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ad0a5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x356d0a5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6d0a62, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ad0a66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe20d0a68, SUC_lexicon_tags_63 },
    { 0x318d0a68, SUC_lexicon_tags_30 },
    { 0x46d0a68, SUC_lexicon_tags_66 },
    { 0x246d0a6b, SUC_lexicon_tags_25 },
    { 0xa8d0a6c, SUC_lexicon_tags_16 },
    { 0xcb6d0a6d, SUC_lexicon_tags_1 },
    { 0x3bcd0a6d, SUC_lexicon_tags_25 },
    { 0xe2ed0a69, SUC_lexicon_tags_0 },
    { 0x7aed0a70, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40d0a73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdacd0a75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0d0a7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x196d0a7d, SUC_lexicon_tags_25 },
    { 0xec4d0a7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12d0a83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe38d0a85, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xc5cd0a87, SUC_lexicon_tags_29 },
    { 0x750d0a88, SUC_lexicon_tags_11 },
    { 0x452d0a89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dad0a8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984d0a91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc10d0a93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774d0a97, SUC_lexicon_tags_15 },
    { 0x616d0a98, SUC_lexicon_tags_44 },
    { 0x6a8d0a99, SUC_lexicon_tags_32 },
    { 0xe4ad0a9a, SUC_lexicon_tags_44 },
    { 0x664d0a9a, SUC_lexicon_tags_27 },
    { 0x8e0d0a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7fad0a9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9dcd0aa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ad0aa7, SUC_lexicon_tags_30 },
    { 0x20ad0aa8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x35cd0aab, SUC_lexicon_tags_237 },
    { 0x6fed0aac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6d0aaf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206d0ab8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x142d0aba, SUC_lexicon_tags_45 },
    { 0xed6d0abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ded0ac4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ed0ac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2d0acc, SUC_lexicon_tags_25 },
    { 0x8d8d0acd, SUC_lexicon_tags_6 },
    { 0xc16d0ace, SUC_lexicon_tags_7 },
    { 0xac8d0acd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d6d0ad1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4d0ad7, SUC_lexicon_tags_1 },
    { 0xab4d0ad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ded0ada, SUC_lexicon_tags_38 },
    { 0x810d0adb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2d0add, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d0ae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3ad0ae4, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cd0aea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdead0aec, SUC_lexicon_tags_25 },
    { 0x15ad0aed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98d0af4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4d0af9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128d0afd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cd0b04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x632d0b06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2d0b0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x242d0b0f, SUC_lexicon_tags_37 },
    { 0x4a2d0b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0cd0b12, SUC_lexicon_tags_11 },
    { 0x4c0d0b13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcd0b19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348d0b1d, SUC_lexicon_tags_80 },
    { 0xaf6d0b1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabed0b27, SUC_lexicon_tags_85 },
    { 0xf56d0b27, SUC_lexicon_tags_7 },
    { 0xd8d0b29, SUC_lexicon_tags_9 },
    { 0xbc0d0b2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34d0b2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cd0b2f, SUC_lexicon_tags_13 },
    { 0x440d0b30, SUC_lexicon_tags_0 },
    { 0x366d0b2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcd0b34, SUC_lexicon_tags_31 },
    { 0xaead0b34, SUC_lexicon_tags_2 },
    { 0xf08d0b36, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cd0b40, SUC_lexicon_tags_37 },
    { 0x200d0b41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26cd0b43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ed0b47, SUC_lexicon_tags_26 },
    { 0x1e4d0b47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acd0b4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x39cd0b4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34d0b55, SUC_lexicon_tags_49 },
    { 0x5e0d0b55, SUC_lexicon_tags_17 },
    { 0x6aad0b57, SUC_lexicon_tags_17 },
    { 0xd0d0b58, SUC_lexicon_tags_846 },
    { 0xed0d0b59, SUC_lexicon_tags_28 },
    { 0xe56d0b58, SUC_lexicon_tags_11 },
    { 0xccad0b5b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x742d0b5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x994d0b60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964d0b64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ed0b6c, SUC_lexicon_tags_1 },
    { 0x5e8d0b6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2d0b75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdfcd0b77, SUC_lexicon_tags_11 },
    { 0x398d0b77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x400d0b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x360d0b7d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xded0b7f, SUC_lexicon_tags_46 },
    { 0xac0d0b80, SUC_lexicon_tags_7 },
    { 0x48cd0b7f, SUC_lexicon_tags_17 },
    { 0x19cd0b82, SUC_lexicon_tags_39 },
    { 0x5a0d0b81, SUC_lexicon_tags_7 },
    { 0xa66d0b81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684d0b8a, SUC_lexicon_tags_14 },
    { 0xb2cd0b8b, SUC_lexicon_tags_23 },
    { 0x1f2d0b8c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ad0b8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40d0b92, SUC_lexicon_tags_84 },
    { 0xe80d0b92, SUC_lexicon_tags_19 },
    { 0x654d0b94, SUC_lexicon_tags_47 },
    { 0x3a8d0b94, SUC_lexicon_tags_11 },
    { 0xeb2d0b96, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780d0ba1, SUC_lexicon_tags_9 },
    { 0x9e0d0ba2, SUC_lexicon_tags_17 },
    { 0x47ed0ba3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26d0ba9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc7ad0bab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf88d0bad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6d0bb0, SUC_lexicon_tags_11 },
    { 0x56ed0bb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cad0bb4, SUC_lexicon_tags_12 },
    { 0x21cd0bb5, SUC_lexicon_tags_43 },
    { 0x62ad0bb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24d0bba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cd0bbd, SUC_lexicon_tags_25 },
    { 0x188d0bbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ed0bc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54d0bc3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ed0bc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xffed0bc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47cd0bcb, SUC_lexicon_tags_8 },
    { 0x92cd0bcc, SUC_lexicon_tags_30 },
    { 0x626d0bcc, SUC_lexicon_tags_37 },
    { 0x614d0bcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ed0bd7, SUC_lexicon_tags_49 },
    { 0x690d0bd7, SUC_lexicon_tags_11 },
    { 0x804d0bd9, SUC_lexicon_tags_23 },
    { 0x8b2d0bda, SUC_lexicon_tags_28 },
    { 0x73ad0bda, SUC_lexicon_tags_14 },
    { 0x942d0bdc, SUC_lexicon_tags_33 },
    { 0x10ed0bdc, SUC_lexicon_tags_0 },
    { 0x30ed0bde, SUC_lexicon_tags_17 },
    { 0x346d0bdb, SUC_lexicon_tags_11 },
    { 0x794d0bd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8d0be7, SUC_lexicon_tags_68 },
    { 0xd5ed0be8, SUC_lexicon_tags_22 },
    { 0xad2d0be8, SUC_lexicon_tags_30 },
    { 0x7bad0be8, SUC_lexicon_tags_47 },
    { 0xaf4d0be9, SUC_lexicon_tags_26 },
    { 0xbc4d0bec, SUC_lexicon_tags_46 },
    { 0x868d0bed, SUC_lexicon_tags_7 },
    { 0xe46d0bed, SUC_lexicon_tags_66 },
    { 0xa32d0be8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5ead0bf1, SUC_lexicon_tags_43 },
    { 0x186d0bf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0bf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x538d0bf7, SUC_lexicon_tags_29 },
    { 0xc7ed0bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ad0c09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2d0c0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ead0c12, SUC_lexicon_tags_6 },
    { 0x298d0c12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0cd0c15, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ad0c18, SUC_lexicon_tags_92 },
    { 0xca2d0c19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4d0c22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2d0c25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x100d0c27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe60d0c29, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xce8d0c2b, SUC_lexicon_tags_9 },
    { 0xd72d0c2b, SUC_lexicon_tags_43 },
    { 0x6d0d0c2d, SUC_lexicon_tags_17 },
    { 0xc4ad0c2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56d0c31, SUC_lexicon_tags_11 },
    { 0x25cd0c32, SUC_lexicon_tags_26 },
    { 0x966d0c32, SUC_lexicon_tags_37 },
    { 0x24ad0c34, SUC_lexicon_tags_152 },
    { 0xd8ed0c35, SUC_lexicon_tags_2 },
    { 0xf1ad0c36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcd0c3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedad0c47, SUC_lexicon_tags_18 },
    { 0x938d0c48, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0x58cd0c4a, SUC_lexicon_tags_14 },
    { 0x144d0c4b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aed0c4f, SUC_lexicon_tags_25 },
    { 0x4fed0c50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bed0c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b4d0c54, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ad0c58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ad0c5b, SUC_lexicon_tags_92 },
    { 0xe18d0c5b, SUC_lexicon_tags_47 },
    { 0x22ed0c5d, SUC_lexicon_tags_17 },
    { 0x604d0c5c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xaded0c60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726d0c64, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42d0c67, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ead0c6a, SUC_lexicon_tags_13 },
    { 0xaad0c6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8d0c74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b6d0c76, SUC_lexicon_tags_37 },
    { 0x6ad0c77, SUC_lexicon_tags_26 },
    { 0xc68d0c77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdeed0c7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14d0c7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2d0c81, SUC_lexicon_tags_2 },
    { 0xbcd0c81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4d0c89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x342d0c8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8d0c91, SUC_lexicon_tags_47 },
    { 0xa06d0c92, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ad0c99, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598d0c9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138d0ca5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12d0caa, SUC_lexicon_tags_60 },
    { 0x400d0cab, SUC_lexicon_tags_37 },
    { 0x5fad0cac, SUC_lexicon_tags_14 },
    { 0x858d0cac, SUC_lexicon_tags_14 },
    { 0x1f0d0cae, SUC_lexicon_tags_12 },
    { 0x46d0cae, SUC_lexicon_tags_30 },
    { 0xe1ed0cab, SUC_lexicon_tags_12 },
    { 0x7aed0caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ad0cb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634d0cbd, SUC_lexicon_tags_63 },
    { 0x430d0cbe, SUC_lexicon_tags_27 },
    { 0x9d0d0cbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ed0cc7, SUC_lexicon_tags_14 },
    { 0xd66d0cc8, SUC_lexicon_tags_37 },
    { 0xbc2d0cc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58d0ccc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ed0ccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecd0cd2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5b4d0cd4, SUC_lexicon_tags_60 },
    { 0xd88d0cd4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2d0cdd, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74d0cea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x564d0ced, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2d0cf0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8bcd0cf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d6d0cf4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3a2d0cf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x672d0cf9, SUC_lexicon_tags_60 },
    { 0x266d0cfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502d0d04, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380d0d0b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe46d0d0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf5ed0d0f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46d0d13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fad0d15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bad0d1d, SUC_lexicon_tags_12 },
    { 0x6a6d0d1e, SUC_lexicon_tags_12 },
    { 0x6c2d0d1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5fed0d21, SUC_lexicon_tags_0 },
    { 0xa4ad0d22, SUC_lexicon_tags_17 },
    { 0x4bed0d23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cd0d27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfa6d0d29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ecd0d2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ad0d2e, SUC_lexicon_tags_47 },
    { 0xe60d0d2f, SUC_lexicon_tags_32 },
    { 0xe4cd0d30, SUC_lexicon_tags_2 },
    { 0xcdcd0d31, SUC_lexicon_tags_15 },
    { 0x7fed0d31, SUC_lexicon_tags_30 },
    { 0x336d0d2f, SUC_lexicon_tags_25 },
    { 0x8a2d0d31, SUC_lexicon_tags_7 },
    { 0x8e4d0d32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d0d38, SUC_lexicon_tags_2 },
    { 0xce4d0d39, SUC_lexicon_tags_70 },
    { 0xc60d0d3a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cd0d40, SUC_lexicon_tags_2 },
    { 0x922d0d41, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cd0d45, SUC_lexicon_tags_60 },
    { 0x260d0d46, SUC_lexicon_tags_29 },
    { 0x4fed0d47, SUC_lexicon_tags_63 },
    { 0xced0d48, SUC_lexicon_tags_55 },
    { 0xbe2d0d49, SUC_lexicon_tags_518 },
    { 0x5d8d0d48, SUC_lexicon_tags_9 },
    { 0x8b6d0d4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d0d4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82d0d5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8d0d66, SUC_lexicon_tags_1 },
    { 0xb66d0d67, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d0d6c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cd0d74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34d0d7c, SUC_lexicon_tags_7 },
    { 0xc86d0d7c, SUC_lexicon_tags_15 },
    { 0x75ed0d7d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8d0d81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00d0d85, SUC_lexicon_tags_7 },
    { 0x738d0d85, SUC_lexicon_tags_48 },
    { 0x10ad0d87, SUC_lexicon_tags_17 },
    { 0x39ad0d88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcd0d8c, SUC_lexicon_tags_82 },
    { 0xcf8d0d8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc70d0d8f, SUC_lexicon_tags_25 },
    { 0xfa4d0d90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516d0d9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52d0da1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d0daa, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x2ecd0dac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd0db0, SUC_lexicon_tags_25 },
    { 0x800d0db0, SUC_lexicon_tags_45 },
    { 0xaead0db2, SUC_lexicon_tags_31 },
    { 0x454d0db2, SUC_lexicon_tags_2 },
    { 0x562d0db0, SUC_lexicon_tags_27 },
    { 0xb3cd0db5, SUC_lexicon_tags_7 },
    { 0x9dcd0db1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcd0dba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72d0dc1, SUC_lexicon_tags_45 },
    { 0xcb8d0dc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcd0dc6, SUC_lexicon_tags_30 },
    { 0xf9ed0dc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bed0dc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ad0dcd, SUC_lexicon_tags_39 },
    { 0xa3ad0dce, SUC_lexicon_tags_37 },
    { 0xbf8d0dcf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x78cd0dd1, SUC_lexicon_tags_17 },
    { 0x788d0dd1, SUC_lexicon_tags_27 },
    { 0x518d0dd3, SUC_lexicon_tags_27 },
    { 0xe5ed0dd3, SUC_lexicon_tags_124 },
    { 0x6acd0dd2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeed0ddb, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cd0ddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x478d0de2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cd0dea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d0df2, SUC_lexicon_tags_15 },
    { 0xbacd0df3, SUC_lexicon_tags_65 },
    { 0x8a0d0df2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeead0dfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6d0e02, SUC_lexicon_tags_15 },
    { 0x2bed0e02, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf42d0e05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa02d0e07, SUC_lexicon_tags_11 },
    { 0x4cad0e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x550d0e0a, SUC_lexicon_tags_9 },
    { 0x7ead0e0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2cad0e0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3bcd0e0f, SUC_lexicon_tags_47 },
    { 0x76ed0e10, SUC_lexicon_tags_88 },
    { 0x898d0e11, SUC_lexicon_tags_1 },
    { 0xe04d0e12, SUC_lexicon_tags_17 },
    { 0xfaed0e12, SUC_lexicon_tags_549 },
    { 0x3fad0e14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e4d0e16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa28d0e18, SUC_lexicon_tags_47 },
    { 0x8e8d0e19, SUC_lexicon_tags_43 },
    { 0x96ad0e1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xea4d0e1c, SUC_lexicon_tags_77 },
    { 0x55ed0e1c, SUC_lexicon_tags_11 },
    { 0x2fad0e1e, SUC_lexicon_tags_64 },
    { 0xb6ed0e1f, SUC_lexicon_tags_37 },
    { 0xffed0e1d, SUC_lexicon_tags_28 },
    { 0x636d0e21, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d0e26, SUC_lexicon_tags_31 },
    { 0xf02d0e27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb82d0e29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x574d0e2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26ad0e2d, SUC_lexicon_tags_11 },
    { 0x8fed0e2d, SUC_lexicon_tags_8 },
    { 0x48ad0e2d, SUC_lexicon_tags_39 },
    { 0x42d0e2f, SUC_lexicon_tags_46 },
    { 0xc3ad0e2f, SUC_lexicon_tags_66 },
    { 0x846d0e31, SUC_lexicon_tags_37 },
    { 0x15cd0e33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cd0e37, SUC_lexicon_tags_26 },
    { 0xeed0e37, SUC_lexicon_tags_37 },
    { 0xdd4d0e38, SUC_lexicon_tags_87 },
    { 0x36cd0e37, SUC_lexicon_tags_31 },
    { 0xaeed0e38, SUC_lexicon_tags_17 },
    { 0xd26d0e3a, SUC_lexicon_tags_12 },
    { 0x2b2d0e3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714d0e41, SUC_lexicon_tags_11 },
    { 0x194d0e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x236d0e44, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x660d0e47, SUC_lexicon_tags_15 },
    { 0xe7ad0e48, SUC_lexicon_tags_12 },
    { 0x240d0e48, SUC_lexicon_tags_47 },
    { 0xdcd0e47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ed0e51, SUC_lexicon_tags_16 },
    { 0x64ed0e52, SUC_lexicon_tags_28 },
    { 0xeb4d0e53, SUC_lexicon_tags_15 },
    { 0xcd2d0e53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x274d0e56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08d0e5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x636d0e5d, SUC_lexicon_tags_48 },
    { 0x40ed0e5e, SUC_lexicon_tags_1 },
    { 0x3aad0e5f, SUC_lexicon_tags_19 },
    { 0xddcd0e60, SUC_lexicon_tags_17 },
    { 0x878d0e5f, SUC_lexicon_tags_37 },
    { 0xbbcd0e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x866d0e64, SUC_lexicon_tags_61 },
    { 0x2ed0e65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x120d0e67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cd0e6b, SUC_lexicon_tags_31 },
    { 0xbe8d0e6b, SUC_lexicon_tags_25 },
    { 0x52cd0e6c, SUC_lexicon_tags_2 },
    { 0x262d0e6b, SUC_lexicon_tags_14 },
    { 0x478d0e6f, SUC_lexicon_tags_8 },
    { 0x80ed0e70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ded0e72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342d0e76, SUC_lexicon_tags_47 },
    { 0xf18d0e77, SUC_lexicon_tags_151 },
    { 0x162d0e77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fed0e7f, SUC_lexicon_tags_19 },
    { 0x130d0e7f, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cd0e83, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9c6d0e85, SUC_lexicon_tags_13 },
    { 0xdaad0e86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x204d0e88, SUC_lexicon_tags_11 },
    { 0xe5ed0e89, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf22d0e8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bed0e8d, SUC_lexicon_tags_25 },
    { 0x6b8d0e8e, SUC_lexicon_tags_1 },
    { 0xd1cd0e8f, SUC_lexicon_tags_7 },
    { 0x1cad0e90, SUC_lexicon_tags_49 },
    { 0xc30d0e8f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x954d0e94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x314d0e96, SUC_lexicon_tags_38 },
    { 0xb12d0e97, SUC_lexicon_tags_60 },
    { 0x33ed0e98, SUC_lexicon_tags_47 },
    { 0xb58d0e99, SUC_lexicon_tags_19 },
    { 0x274d0e98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30d0e9d, SUC_lexicon_tags_15 },
    { 0xbc8d0e9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454d0ea3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8d0ea9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x634d0eac, SUC_lexicon_tags_37 },
    { 0xe58d0ead, SUC_lexicon_tags_37 },
    { 0x21ad0eac, SUC_lexicon_tags_25 },
    { 0x7fad0eaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2d0eb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494d0eb6, SUC_lexicon_tags_2 },
    { 0xb52d0eb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6d0eba, SUC_lexicon_tags_16 },
    { 0x5a0d0eba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x106d0ebe, SUC_lexicon_tags_37 },
    { 0x3dad0ebe, SUC_lexicon_tags_8 },
    { 0x398d0ec0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760d0ec5, SUC_lexicon_tags_74 },
    { 0xe12d0ec5, SUC_lexicon_tags_64 },
    { 0xeed0ec6, SUC_lexicon_tags_11 },
    { 0x338d0ec8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0d0ecb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82d0ed0, SUC_lexicon_tags_12 },
    { 0x320d0ed1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc00d0ed3, SUC_lexicon_tags_19 },
    { 0xcd8d0ed4, SUC_lexicon_tags_30 },
    { 0x94ed0ed5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2d0ed9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acd0edf, SUC_lexicon_tags_17 },
    { 0x3d4d0ee0, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x428d0ee3, SUC_lexicon_tags_17 },
    { 0x5bed0ee3, SUC_lexicon_tags_17 },
    { 0x74ed0ee5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x348d0ee7, SUC_lexicon_tags_2 },
    { 0xe5cd0ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98d0eee, SUC_lexicon_tags_28 },
    { 0xdced0eef, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcfad0ef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad0efb, SUC_lexicon_tags_19 },
    { 0x608d0efc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a6d0efe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ed0f05, SUC_lexicon_tags_2 },
    { 0xe88d0f06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x436d0f08, SUC_lexicon_tags_65 },
    { 0xdf8d0f09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cd0f0f, SUC_lexicon_tags_87 },
    { 0xd60d0f10, SUC_lexicon_tags_11 },
    { 0x1b4d0f0f, SUC_lexicon_tags_17 },
    { 0x6dad0f11, SUC_lexicon_tags_27 },
    { 0x2aad0f12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76d0f18, SUC_lexicon_tags_7 },
    { 0xf8ed0f18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf34d0f1b, SUC_lexicon_tags_15 },
    { 0x98d0f1b, SUC_lexicon_tags_15 },
    { 0x96ed0f1d, SUC_lexicon_tags_11 },
    { 0x6acd0f1b, SUC_lexicon_tags_11 },
    { 0xad6d0f1d, SUC_lexicon_tags_11 },
    { 0xaf0d0f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f6d0f22, SUC_lexicon_tags_43 },
    { 0x4c2d0f23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62d0f26, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ad0f29, SUC_lexicon_tags_1 },
    { 0x542d0f2a, SUC_lexicon_tags_43 },
    { 0x292d0f2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0d0f2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x720d0f30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x344d0f32, SUC_lexicon_tags_2 },
    { 0x51ad0f33, SUC_lexicon_tags_44 },
    { 0xab8d0f33, SUC_lexicon_tags_30 },
    { 0x9a8d0f35, SUC_lexicon_tags_1 },
    { 0x266d0f35, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x634d0f38, SUC_lexicon_tags_21 },
    { 0xb34d0f39, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cd0f41, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708d0f48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0d0f4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x238d0f4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cd0f54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d0f5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x820d0f5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82d0f63, SUC_lexicon_tags_11 },
    { 0xd70d0f63, SUC_lexicon_tags_12 },
    { 0xd88d0f64, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x140d0f67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98d0f6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bad0f71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d0f77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6d0f7b, SUC_lexicon_tags_15 },
    { 0x528d0f7b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x588d0f7e, SUC_lexicon_tags_23 },
    { 0x3e8d0f7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68d0f83, SUC_lexicon_tags_11 },
    { 0x2ccd0f84, SUC_lexicon_tags_23 },
    { 0x5a6d0f83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ed0f89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbad0f8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ed0f93, SUC_lexicon_tags_1 },
    { 0x1e4d0f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54d0f9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708d0f9e, SUC_lexicon_tags_37 },
    { 0xf1ad0f9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d2d0fa1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x41ad0fa3, SUC_lexicon_tags_14 },
    { 0x830d0fa3, SUC_lexicon_tags_2 },
    { 0x3ccd0fa5, SUC_lexicon_tags_0 },
    { 0xe8ad0fa6, SUC_lexicon_tags_23 },
    { 0x346d0fa6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef2d0fa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45ad0fab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf86d0fad, SUC_lexicon_tags_2 },
    { 0x97ad0fae, SUC_lexicon_tags_11 },
    { 0x550d0faf, SUC_lexicon_tags_15 },
    { 0x200d0fb0, SUC_lexicon_tags_37 },
    { 0xd22d0fb1, SUC_lexicon_tags_3 },
    { 0xc64d0fae, SUC_lexicon_tags_61 },
    { 0x7f0d0fb3, SUC_lexicon_tags_25 },
    { 0x44d0fb3, SUC_lexicon_tags_37 },
    { 0xd92d0fb5, SUC_lexicon_tags_3 },
    { 0xf60d0fb4, SUC_lexicon_tags_63 },
    { 0x4a0d0fb7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ad0fbb, SUC_lexicon_tags_4 },
    { 0x2c6d0fbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5dad0fbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0d0fc4, SUC_lexicon_tags_7 },
    { 0x22ad0fc5, SUC_lexicon_tags_25 },
    { 0xdaed0fc5, SUC_lexicon_tags_11 },
    { 0xcd0fc7, SUC_lexicon_tags_60 },
    { 0xeded0fc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ead0fcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xacd0fcd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf1cd0fcf, SUC_lexicon_tags_2 },
    { 0x428d0fcf, SUC_lexicon_tags_15 },
    { 0x3ced0fd1, SUC_lexicon_tags_85 },
    { 0x618d0fcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cd0fd7, SUC_lexicon_tags_3 },
    { 0x110d0fd8, SUC_lexicon_tags_25 },
    { 0x7d4d0fd9, SUC_lexicon_tags_65 },
    { 0xc6ad0fda, SUC_lexicon_tags_17 },
    { 0x760d0fd7, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4d0fdf, SUC_lexicon_tags_1395 },
    { 0x5a4d0fe0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4cd0fe2, SUC_lexicon_tags_8 },
    { 0x312d0fe2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920d0fe8, SUC_lexicon_tags_47 },
    { 0xa1ed0fe9, SUC_lexicon_tags_37 },
    { 0x490d0fea, SUC_lexicon_tags_39 },
    { 0xf40d0fe8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe0d0fed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cd0ff0, SUC_lexicon_tags_37 },
    { 0x834d0ff1, SUC_lexicon_tags_37 },
    { 0x216d0ff2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d0ff8, SUC_lexicon_tags_25 },
    { 0x61ed0ff8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x914d0ffb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0d0fff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfed1002, SUC_lexicon_tags_46 },
    { 0x1e6d1003, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468d1009, SUC_lexicon_tags_45 },
    { 0x952d100a, SUC_lexicon_tags_17 },
    { 0xad2d100b, SUC_lexicon_tags_25 },
    { 0x1c8d100b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfe6d100e, SUC_lexicon_tags_17 },
    { 0x510d100f, SUC_lexicon_tags_14 },
    { 0x7e0d100e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744d101a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfe0d101c, SUC_lexicon_tags_12 },
    { 0x96ad101d, SUC_lexicon_tags_37 },
    { 0x57cd101e, SUC_lexicon_tags_43 },
    { 0x474d101e, SUC_lexicon_tags_7 },
    { 0x50ad1020, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140d1024, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2d1029, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0d1034, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4d103a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2d103f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x942d1041, SUC_lexicon_tags_1 },
    { 0xd4ad1042, SUC_lexicon_tags_34 },
    { 0xbfcd1043, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4d1047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ded104d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfed1051, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8d1055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadad1058, SUC_lexicon_tags_15 },
    { 0xf62d1058, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xabad105c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2ecd105e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952d1062, SUC_lexicon_tags_7 },
    { 0x21ed1063, SUC_lexicon_tags_31 },
    { 0x972d1062, SUC_lexicon_tags_27 },
    { 0x682d1065, SUC_lexicon_tags_35 },
    { 0xd8ad1066, SUC_lexicon_tags_14 },
    { 0xd4ad1065, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a8d1069, SUC_lexicon_tags_23 },
    { 0x968d106a, SUC_lexicon_tags_79 },
    { 0x3ed106b, SUC_lexicon_tags_32 },
    { 0x63ed106c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ad106f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0d1072, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde8d1074, SUC_lexicon_tags_34 },
    { 0x408d1075, SUC_lexicon_tags_47 },
    { 0x814d1076, SUC_lexicon_tags_17 },
    { 0x596d1074, SUC_lexicon_tags_2 },
    { 0x466d1078, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710d107f, SUC_lexicon_tags_10 },
    { 0xe2d1080, SUC_lexicon_tags_38 },
    { 0xd0d1081, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8d1085, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd108a, SUC_lexicon_tags_11 },
    { 0x2c8d108a, SUC_lexicon_tags_4 },
    { 0xe32d108a, SUC_lexicon_tags_15 },
    { 0xeb6d108d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4d1091, SUC_lexicon_tags_25 },
    { 0x12ed1092, SUC_lexicon_tags_0 },
    { 0xaf6d1093, SUC_lexicon_tags_48 },
    { 0xaead1091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ad1098, SUC_lexicon_tags_33 },
    { 0xbb0d1099, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x374d109b, SUC_lexicon_tags_10 },
    { 0xb4cd109b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcd10a0, SUC_lexicon_tags_7 },
    { 0x992d10a0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x88ad10a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed10aa, SUC_lexicon_tags_9 },
    { 0x576d10aa, SUC_lexicon_tags_2 },
    { 0xdc0d10aa, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ed10b7, SUC_lexicon_tags_87 },
    { 0x906d10b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182d10be, SUC_lexicon_tags_6 },
    { 0x1ded10bf, SUC_lexicon_tags_38 },
    { 0x5e2d10bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4d10c5, SUC_lexicon_tags_0 },
    { 0xed6d10c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cd10cd, SUC_lexicon_tags_25 },
    { 0x580d10ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bad10d8, SUC_lexicon_tags_2 },
    { 0x82ad10d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0d10dd, SUC_lexicon_tags_17 },
    { 0xd26d10de, SUC_lexicon_tags_37 },
    { 0x7b4d10dd, SUC_lexicon_tags_11 },
    { 0x3b6d10e0, SUC_lexicon_tags_11 },
    { 0xa7ed10e1, SUC_lexicon_tags_19 },
    { 0x392d10e2, SUC_lexicon_tags_61 },
    { 0x43ad10e2, SUC_lexicon_tags_12 },
    { 0x228d10e1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x264d10e7, SUC_lexicon_tags_39 },
    { 0xb42d10e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92d10ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe20d10ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8d10f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf08d10f5, SUC_lexicon_tags_12 },
    { 0x422d10f5, SUC_lexicon_tags_63 },
    { 0x388d10f7, SUC_lexicon_tags_13 },
    { 0x4a2d10f8, SUC_lexicon_tags_39 },
    { 0x55ad10f9, SUC_lexicon_tags_63 },
    { 0x3b6d10fa, SUC_lexicon_tags_63 },
    { 0x67ed10f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4d10fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8a6d1100, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecd1105, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x584d1108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4d110b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xccad110d, SUC_lexicon_tags_17 },
    { 0xe5cd110d, SUC_lexicon_tags_25 },
    { 0xb94d110d, SUC_lexicon_tags_25 },
    { 0xbe0d1110, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6d1114, SUC_lexicon_tags_43 },
    { 0xea2d1115, SUC_lexicon_tags_0 },
    { 0x9d4d1115, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x468d1118, SUC_lexicon_tags_29 },
    { 0x672d1119, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x1fed111b, SUC_lexicon_tags_25 },
    { 0x9fad111c, SUC_lexicon_tags_11 },
    { 0x5b4d111b, SUC_lexicon_tags_6 },
    { 0xd3cd111e, SUC_lexicon_tags_87 },
    { 0x1e2d111f, SUC_lexicon_tags_47 },
    { 0xb52d1120, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40d1125, SUC_lexicon_tags_1 },
    { 0x370d1126, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ad1129, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70d112c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58ad112e, SUC_lexicon_tags_38 },
    { 0x50ad112f, SUC_lexicon_tags_0 },
    { 0xa06d112e, SUC_lexicon_tags_30 },
    { 0x47ed1130, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18d1137, SUC_lexicon_tags_47 },
    { 0x78cd1138, SUC_lexicon_tags_15 },
    { 0x492d1138, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432d1140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cd1144, SUC_lexicon_tags_9 },
    { 0xe6cd1145, SUC_lexicon_tags_11 },
    { 0x9ead1146, SUC_lexicon_tags_66 },
    { 0xc4ed1145, SUC_lexicon_tags_23 },
    { 0x776d1148, SUC_lexicon_tags_37 },
    { 0x5cd1148, SUC_lexicon_tags_33 },
    { 0x2ed1149, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacad1151, SUC_lexicon_tags_12 },
    { 0x2f2d1152, SUC_lexicon_tags_11 },
    { 0xae2d1153, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38d1156, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d115e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x82ed1160, SUC_lexicon_tags_0 },
    { 0x63ad1160, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6d1166, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x414d1168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606d116c, SUC_lexicon_tags_11 },
    { 0x2b0d116d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926d1172, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x672d1175, SUC_lexicon_tags_47 },
    { 0x45cd1175, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x196d1178, SUC_lexicon_tags_15 },
    { 0xf68d1178, SUC_lexicon_tags_30 },
    { 0xe94d1178, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798d117f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692d1184, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6d1187, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92d118b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ad1190, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32d1197, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942d119e, SUC_lexicon_tags_63 },
    { 0x84d119e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x11ad11a1, SUC_lexicon_tags_23 },
    { 0xce4d11a2, SUC_lexicon_tags_236 },
    { 0x762d11a3, SUC_lexicon_tags_4 },
    { 0x3c2d11a4, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cd11a8, SUC_lexicon_tags_77 },
    { 0xfc8d11a9, SUC_lexicon_tags_25 },
    { 0x1d0d11a9, SUC_lexicon_tags_37 },
    { 0xa9cd11ab, SUC_lexicon_tags_37 },
    { 0xa94d11a9, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x374d11ae, SUC_lexicon_tags_12 },
    { 0xe90d11ae, SUC_lexicon_tags_38 },
    { 0xaeed11af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x692d11b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6d11b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ccd11b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96d11bb, SUC_lexicon_tags_15 },
    { 0xc04d11bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa24d11bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ad11c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84ad11c4, SUC_lexicon_tags_37 },
    { 0x4eed11c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x990d11c7, SUC_lexicon_tags_7 },
    { 0x8d4d11c8, SUC_lexicon_tags_43 },
    { 0xafed11c9, SUC_lexicon_tags_27 },
    { 0x168d11ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cd11cf, SUC_lexicon_tags_33 },
    { 0xca6d11cf, SUC_lexicon_tags_23 },
    { 0x1d8d11d1, SUC_lexicon_tags_7 },
    { 0xd8cd11d2, SUC_lexicon_tags_32 },
    { 0x27cd11d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0d11d5, SUC_lexicon_tags_19 },
    { 0x21ed11d5, SUC_lexicon_tags_28 },
    { 0xe30d11d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x380d11da, SUC_lexicon_tags_11 },
    { 0x1e4d11db, SUC_lexicon_tags_27 },
    { 0xe5ed11dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6a4d11de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546d11e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d11e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfead11e9, SUC_lexicon_tags_47 },
    { 0x4d6d11ea, SUC_lexicon_tags_1 },
    { 0xef8d11e9, SUC_lexicon_tags_160 },
    { 0x1aed11ec, SUC_lexicon_tags_2 },
    { 0xcfcd11ed, SUC_lexicon_tags_23 },
    { 0x1ecd11ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70d11f2, SUC_lexicon_tags_11 },
    { 0x134d11f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0d11f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8d1202, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ad1206, SUC_lexicon_tags_55 },
    { 0x19cd1206, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cd1210, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44d1217, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x616d121a, SUC_lexicon_tags_28 },
    { 0x142d121a, SUC_lexicon_tags_28 },
    { 0xaccd121c, SUC_lexicon_tags_25 },
    { 0x572d121b, SUC_lexicon_tags_30 },
    { 0x4dcd121e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cd1222, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57cd1224, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x598d1226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ad1229, SUC_lexicon_tags_31 },
    { 0x954d122a, SUC_lexicon_tags_48 },
    { 0x28ad122b, SUC_lexicon_tags_37 },
    { 0x8f2d122c, SUC_lexicon_tags_12 },
    { 0xaeed122d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x196d122f, SUC_lexicon_tags_112 },
    { 0x9f2d1230, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864d1235, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa1ad1237, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6d123a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180d123e, SUC_lexicon_tags_30 },
    { 0x40ad123f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46d1245, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cad1249, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x278d124c, SUC_lexicon_tags_30 },
    { 0xe78d124d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f2d124f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb84d1251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x576d1253, SUC_lexicon_tags_33 },
    { 0xb96d1253, SUC_lexicon_tags_62 },
    { 0xd44d1253, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x916d1257, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x602d125a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcd125f, SUC_lexicon_tags_15 },
    { 0xe86d1260, SUC_lexicon_tags_43 },
    { 0xaaad1261, SUC_lexicon_tags_43 },
    { 0xeb2d125f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x494d1264, SUC_lexicon_tags_17 },
    { 0xf86d1264, SUC_lexicon_tags_12 },
    { 0x3f4d1265, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32d1269, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2d126f, SUC_lexicon_tags_2 },
    { 0x36d126f, SUC_lexicon_tags_32 },
    { 0xc5ed1271, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16ed1273, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1279, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cd127f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa48d1281, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0d1284, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd128d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cd1292, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ed1295, SUC_lexicon_tags_63 },
    { 0x484d1295, SUC_lexicon_tags_37 },
    { 0x776d1297, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcccd129a, SUC_lexicon_tags_87 },
    { 0x2a4d129b, SUC_lexicon_tags_11 },
    { 0xd94d129c, SUC_lexicon_tags_23 },
    { 0xe70d129d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62d12a3, SUC_lexicon_tags_15 },
    { 0xed12a4, SUC_lexicon_tags_5 },
    { 0x6c6d12a5, SUC_lexicon_tags_38 },
    { 0x23cd12a6, SUC_lexicon_tags_29 },
    { 0xcd8d12a7, SUC_lexicon_tags_25 },
    { 0x2c8d12a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42d12ab, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd12b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe30d12b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x220d12b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x154d12b8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d12bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x868d12c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508d12c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ead12cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26d12ce, SUC_lexicon_tags_25 },
    { 0x956d12cf, SUC_lexicon_tags_25 },
    { 0xaa2d12cf, SUC_lexicon_tags_43 },
    { 0x502d12d0, SUC_lexicon_tags_17 },
    { 0x1aad12d2, SUC_lexicon_tags_30 },
    { 0x654d12d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102d12dc, SUC_lexicon_tags_43 },
    { 0x7f8d12dd, SUC_lexicon_tags_12 },
    { 0x400d12de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e8d12e0, SUC_lexicon_tags_15 },
    { 0x33ad12e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad12e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eed12ea, SUC_lexicon_tags_11 },
    { 0x944d12eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840d12f0, SUC_lexicon_tags_9 },
    { 0xa40d12f0, SUC_lexicon_tags_38 },
    { 0x52ed12f2, SUC_lexicon_tags_43 },
    { 0xd38d12f3, SUC_lexicon_tags_7 },
    { 0xf9ed12f4, SUC_lexicon_tags_60 },
    { 0x3bcd12f5, SUC_lexicon_tags_30 },
    { 0x386d12f6, SUC_lexicon_tags_28 },
    { 0x8e6d12f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aed12f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0d12fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6d12ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816d1303, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccd1309, SUC_lexicon_tags_15 },
    { 0xa16d130a, SUC_lexicon_tags_45 },
    { 0x47ad1309, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x43ad130d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfad1312, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x142d1314, SUC_lexicon_tags_37 },
    { 0x70cd1315, SUC_lexicon_tags_12 },
    { 0x742d1314, SUC_lexicon_tags_0 },
    { 0xdcad1317, SUC_lexicon_tags_11 },
    { 0x80cd1316, SUC_lexicon_tags_14 },
    { 0x4e2d1319, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12d131c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ed1325, SUC_lexicon_tags_33 },
    { 0x5d4d1326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cad1328, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62d132e, SUC_lexicon_tags_17 },
    { 0x9ad132f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ad133b, SUC_lexicon_tags_69 },
    { 0x72cd133b, SUC_lexicon_tags_1 },
    { 0x36d133d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x92ad133f, SUC_lexicon_tags_32 },
    { 0x2d6d1340, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bed1345, SUC_lexicon_tags_25 },
    { 0x25ed1346, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88d1349, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2d134c, SUC_lexicon_tags_37 },
    { 0xff8d134c, SUC_lexicon_tags_66 },
    { 0x446d134e, SUC_lexicon_tags_25 },
    { 0x2b8d134f, SUC_lexicon_tags_1 },
    { 0xc4cd134e, SUC_lexicon_tags_23 },
    { 0x92d1351, SUC_lexicon_tags_303 },
    { 0x5d4d1351, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2b6d1354, SUC_lexicon_tags_7 },
    { 0x72ad1354, SUC_lexicon_tags_26 },
    { 0x5a0d1354, SUC_lexicon_tags_79 },
    { 0x6bad1357, SUC_lexicon_tags_6 },
    { 0xe96d1358, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ad135d, SUC_lexicon_tags_19 },
    { 0x4e2d135e, SUC_lexicon_tags_37 },
    { 0xbd2d135f, SUC_lexicon_tags_30 },
    { 0xb78d1360, SUC_lexicon_tags_63 },
    { 0xa6d1361, SUC_lexicon_tags_11 },
    { 0xd0ad1362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d1365, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1cd1367, SUC_lexicon_tags_70 },
    { 0xc6ed1368, SUC_lexicon_tags_49 },
    { 0x90cd1369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6d136f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4d1379, SUC_lexicon_tags_26 },
    { 0xff0d137a, SUC_lexicon_tags_37 },
    { 0xa78d137a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xee4d137d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912d1386, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ed138c, SUC_lexicon_tags_0 },
    { 0x376d138d, SUC_lexicon_tags_7 },
    { 0x922d138e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cd1392, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x784d1394, SUC_lexicon_tags_11 },
    { 0xcc2d1395, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224d139b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x190d139d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4d13a0, SUC_lexicon_tags_37 },
    { 0xb9cd13a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640d13a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b0d13a7, SUC_lexicon_tags_23 },
    { 0xbead13a8, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62d13ad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x922d13af, SUC_lexicon_tags_60 },
    { 0x99ad13af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecd13b4, SUC_lexicon_tags_2 },
    { 0x17ad13b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504d13bf, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662d13c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa26d13c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x678d13ca, SUC_lexicon_tags_6 },
    { 0x9fcd13ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08d13d1, SUC_lexicon_tags_15 },
    { 0x76cd13d2, SUC_lexicon_tags_151 },
    { 0xf72d13d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44d13d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ed13db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfed13e0, SUC_lexicon_tags_43 },
    { 0x5bad13e0, SUC_lexicon_tags_65 },
    { 0x8cd13e1, SUC_lexicon_tags_7 },
    { 0xa70d13e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4d2d13e5, SUC_lexicon_tags_8 },
    { 0x68ad13e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5e2d13e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5ccd13ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x288d13ed, SUC_lexicon_tags_40 },
    { 0x5c2d13ee, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268d13f2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x5d8d13f4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ed13f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6d13fb, SUC_lexicon_tags_14 },
    { 0xb76d13fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x994d13ff, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead1403, SUC_lexicon_tags_56 },
    { 0x4eed1404, SUC_lexicon_tags_11 },
    { 0x754d1405, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf8d1407, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4d140d, SUC_lexicon_tags_47 },
    { 0xb0ed140d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182d1416, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ed1418, SUC_lexicon_tags_2 },
    { 0x37ed1419, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d1420, SUC_lexicon_tags_35 },
    { 0xc84d1421, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc62d1423, SUC_lexicon_tags_14 },
    { 0xde2d1423, SUC_lexicon_tags_27 },
    { 0xf1ad1425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d2d1427, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74d142a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x37cd142c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cd1431, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d4d1433, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4d1437, SUC_lexicon_tags_37 },
    { 0xdeed1438, SUC_lexicon_tags_25 },
    { 0xa86d1437, SUC_lexicon_tags_49 },
    { 0xa02d1439, SUC_lexicon_tags_27 },
    { 0xdd4d1438, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x396d143d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c8d143f, SUC_lexicon_tags_7 },
    { 0x5acd1440, SUC_lexicon_tags_118 },
    { 0x2d0d143f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c4d1443, SUC_lexicon_tags_11 },
    { 0x49ad1443, SUC_lexicon_tags_4 },
    { 0x434d1443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x840d1447, SUC_lexicon_tags_27 },
    { 0xce6d1448, SUC_lexicon_tags_39 },
    { 0x530d1447, SUC_lexicon_tags_17 },
    { 0xbf8d144a, SUC_lexicon_tags_37 },
    { 0x3f6d144a, SUC_lexicon_tags_37 },
    { 0x6b6d144a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0d1451, SUC_lexicon_tags_17 },
    { 0xba6d1451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4d145b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ad1463, SUC_lexicon_tags_47 },
    { 0x172d1464, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46d1469, SUC_lexicon_tags_7 },
    { 0xa54d146a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x524d146c, SUC_lexicon_tags_105 },
    { 0xbecd146d, SUC_lexicon_tags_17 },
    { 0x24cd146c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d1472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202d1478, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180d147d, SUC_lexicon_tags_1 },
    { 0x93cd147e, SUC_lexicon_tags_13 },
    { 0x356d147f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2ad1481, SUC_lexicon_tags_25 },
    { 0xb84d1482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a6d1484, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x326d1486, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4d6d1488, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xafed148b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf86d148d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964d1493, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1496, SUC_lexicon_tags_29 },
    { 0xbad1496, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4d14a3, SUC_lexicon_tags_2 },
    { 0x11ed14a3, SUC_lexicon_tags_57 },
    { 0x8d4d14a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ad14a9, SUC_lexicon_tags_45 },
    { 0x9f8d14aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaad14ad, SUC_lexicon_tags_17 },
    { 0xd18d14ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x324d14b0, SUC_lexicon_tags_1492 },
    { 0, NULL },
    { 0, NULL },
    { 0x670d14b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x76cd14b5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1a0d14b7, SUC_lexicon_tags_37 },
    { 0x4e0d14b8, SUC_lexicon_tags_7 },
    { 0xef6d14b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878d14c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x610d14c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ad14cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6d14ce, SUC_lexicon_tags_29 },
    { 0xcded14cf, SUC_lexicon_tags_17 },
    { 0xff6d14d0, SUC_lexicon_tags_12 },
    { 0xf6ed14d1, SUC_lexicon_tags_45 },
    { 0x928d14d2, SUC_lexicon_tags_7 },
    { 0x2a6d14d1, SUC_lexicon_tags_7 },
    { 0xbeed14d4, SUC_lexicon_tags_15 },
    { 0x5e6d14d5, SUC_lexicon_tags_11 },
    { 0xcd6d14d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cd14de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14d14e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7b6d14e2, SUC_lexicon_tags_113 },
    { 0x3d4d14e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a8d14e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cd14ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x75cd14ee, SUC_lexicon_tags_15 },
    { 0xceed14ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cd14f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6d14f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64d14fd, SUC_lexicon_tags_15 },
    { 0x190d14fd, SUC_lexicon_tags_60 },
    { 0xef4d14fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ad1508, SUC_lexicon_tags_37 },
    { 0x3b0d1509, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x94cd150c, SUC_lexicon_tags_13 },
    { 0xd6ed150c, SUC_lexicon_tags_25 },
    { 0x690d150e, SUC_lexicon_tags_116 },
    { 0xfc0d150f, SUC_lexicon_tags_31 },
    { 0x9d6d1510, SUC_lexicon_tags_27 },
    { 0xde2d1511, SUC_lexicon_tags_2 },
    { 0x10d150e, SUC_lexicon_tags_46 },
    { 0x9bed1510, SUC_lexicon_tags_63 },
    { 0x5f0d1514, SUC_lexicon_tags_12 },
    { 0x970d1514, SUC_lexicon_tags_17 },
    { 0x714d1514, SUC_lexicon_tags_9 },
    { 0xcd8d150c, SUC_lexicon_tags_7 },
    { 0xc40d150c, SUC_lexicon_tags_17 },
    { 0xba0d1512, SUC_lexicon_tags_2 },
    { 0x7d2d151a, SUC_lexicon_tags_1 },
    { 0x1f6d151b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2d151f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x488d1522, SUC_lexicon_tags_2 },
    { 0x2ead1523, SUC_lexicon_tags_2 },
    { 0x56ad1524, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d152a, SUC_lexicon_tags_11 },
    { 0x2cad152b, SUC_lexicon_tags_17 },
    { 0x250d152b, SUC_lexicon_tags_2 },
    { 0x19ad152b, SUC_lexicon_tags_61 },
    { 0x4fed152d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81cd1530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6d1533, SUC_lexicon_tags_46 },
    { 0x6b8d1533, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0d153d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ad1544, SUC_lexicon_tags_17 },
    { 0x9fad1545, SUC_lexicon_tags_84 },
    { 0xb8ad1546, SUC_lexicon_tags_82 },
    { 0xd06d1547, SUC_lexicon_tags_15 },
    { 0xe1ed1548, SUC_lexicon_tags_19 },
    { 0x8e4d1549, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cd154e, SUC_lexicon_tags_1 },
    { 0x518d154f, SUC_lexicon_tags_13 },
    { 0x55cd154e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad1553, SUC_lexicon_tags_33 },
    { 0x1a6d1554, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cd1557, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2d155b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14d1560, SUC_lexicon_tags_17 },
    { 0x8cad1561, SUC_lexicon_tags_8 },
    { 0xa50d1561, SUC_lexicon_tags_11 },
    { 0xad2d1560, SUC_lexicon_tags_19 },
    { 0x18cd1563, SUC_lexicon_tags_25 },
    { 0x4d2d1565, SUC_lexicon_tags_9 },
    { 0x810d1566, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64d156a, SUC_lexicon_tags_11 },
    { 0xfbad156a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18d1572, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6d1577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cd157a, SUC_lexicon_tags_25 },
    { 0x4e2d157b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab0d157d, SUC_lexicon_tags_66 },
    { 0x194d157e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2d1581, SUC_lexicon_tags_23 },
    { 0xf64d1582, SUC_lexicon_tags_107 },
    { 0xec6d1583, SUC_lexicon_tags_43 },
    { 0xea8d1584, SUC_lexicon_tags_27 },
    { 0xf9ed1585, SUC_lexicon_tags_17 },
    { 0xe58d1586, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2d1589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6d158c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x366d158e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x576d1590, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320d1594, SUC_lexicon_tags_11 },
    { 0x64cd1595, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7ecd1597, SUC_lexicon_tags_39 },
    { 0xdb2d1598, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38d159b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704d159f, SUC_lexicon_tags_6 },
    { 0x7c6d159f, SUC_lexicon_tags_12 },
    { 0xd4ad15a1, SUC_lexicon_tags_37 },
    { 0xfb2d15a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20d15ab, SUC_lexicon_tags_7 },
    { 0x3ced15ac, SUC_lexicon_tags_11 },
    { 0x314d15ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8d15b0, SUC_lexicon_tags_11 },
    { 0x268d15b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10d15b6, SUC_lexicon_tags_25 },
    { 0xec4d15b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bad15b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x642d15bc, SUC_lexicon_tags_37 },
    { 0x2d2d15bd, SUC_lexicon_tags_87 },
    { 0xf56d15be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0d15c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x496d15c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd78d15c5, SUC_lexicon_tags_37 },
    { 0x53ed15c6, SUC_lexicon_tags_10 },
    { 0xc28d15c6, SUC_lexicon_tags_27 },
    { 0x16ed15c8, SUC_lexicon_tags_17 },
    { 0x4dad15c9, SUC_lexicon_tags_7 },
    { 0xd6ed15c9, SUC_lexicon_tags_55 },
    { 0xc54d15ca, SUC_lexicon_tags_38 },
    { 0x314d15cc, SUC_lexicon_tags_1 },
    { 0x5e8d15cc, SUC_lexicon_tags_46 },
    { 0xe00d15ce, SUC_lexicon_tags_7 },
    { 0xed2d15ce, SUC_lexicon_tags_7 },
    { 0xf18d15ca, SUC_lexicon_tags_33 },
    { 0x8cad15d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6d15d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6d4d15db, SUC_lexicon_tags_7 },
    { 0xb2d15dc, SUC_lexicon_tags_61 },
    { 0x970d15dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218d15e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd15eb, SUC_lexicon_tags_37 },
    { 0x6b2d15ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aad15f2, SUC_lexicon_tags_2 },
    { 0xc60d15f3, SUC_lexicon_tags_7 },
    { 0x602d15f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x480d15f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdead15f8, SUC_lexicon_tags_37 },
    { 0xef4d15f9, SUC_lexicon_tags_43 },
    { 0xbaed15f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x220d15fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe82d15fe, SUC_lexicon_tags_2 },
    { 0xc36d15ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x99ed1601, SUC_lexicon_tags_11 },
    { 0x60ad1602, SUC_lexicon_tags_68 },
    { 0x6fcd1601, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2d160a, SUC_lexicon_tags_4 },
    { 0xaf6d160a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d160f, SUC_lexicon_tags_12 },
    { 0xc0d160f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460d1614, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ded161b, SUC_lexicon_tags_17 },
    { 0x3bad161c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x26d161e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ad1626, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x346d1628, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4d162c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfdad162e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x292d1631, SUC_lexicon_tags_43 },
    { 0x3c6d1632, SUC_lexicon_tags_15 },
    { 0x434d1632, SUC_lexicon_tags_17 },
    { 0x552d1631, SUC_lexicon_tags_28 },
    { 0x6ded1635, SUC_lexicon_tags_19 },
    { 0x1f2d1634, SUC_lexicon_tags_2 },
    { 0x7e8d1631, SUC_lexicon_tags_26 },
    { 0xd2cd1638, SUC_lexicon_tags_37 },
    { 0xf12d1632, SUC_lexicon_tags_1 },
    { 0x5fed163a, SUC_lexicon_tags_2 },
    { 0x928d163b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xda0d163d, SUC_lexicon_tags_7 },
    { 0xd32d163d, SUC_lexicon_tags_1 },
    { 0xb90d163f, SUC_lexicon_tags_30 },
    { 0x1bad163d, SUC_lexicon_tags_66 },
    { 0x51ed163f, SUC_lexicon_tags_12 },
    { 0x6eed1642, SUC_lexicon_tags_11 },
    { 0x5b2d1641, SUC_lexicon_tags_17 },
    { 0xc98d163e, SUC_lexicon_tags_17 },
    { 0x12ed1642, SUC_lexicon_tags_25 },
    { 0x3e0d1640, SUC_lexicon_tags_7 },
    { 0xb06d1647, SUC_lexicon_tags_17 },
    { 0xd46d1647, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd164f, SUC_lexicon_tags_2 },
    { 0xb36d1650, SUC_lexicon_tags_7 },
    { 0x2f2d1650, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d1655, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912d165a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20d1660, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ed1666, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x212d1668, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ed166c, SUC_lexicon_tags_15 },
    { 0xee6d166c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x874d1670, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cd1674, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242d167a, SUC_lexicon_tags_11 },
    { 0xacd167b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0d1681, SUC_lexicon_tags_1 },
    { 0x92ed1682, SUC_lexicon_tags_11 },
    { 0x1d6d1682, SUC_lexicon_tags_38 },
    { 0x6b2d1684, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4aed1686, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cd1689, SUC_lexicon_tags_47 },
    { 0x2c6d168a, SUC_lexicon_tags_15 },
    { 0xa4cd168b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0d1691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ed1694, SUC_lexicon_tags_25 },
    { 0x14ad1695, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x724d1697, SUC_lexicon_tags_11 },
    { 0x218d1698, SUC_lexicon_tags_49 },
    { 0x2e8d1699, SUC_lexicon_tags_46 },
    { 0xcfcd1697, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x554d169c, SUC_lexicon_tags_11 },
    { 0x710d169c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4ed169f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90d16a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x882d16a6, SUC_lexicon_tags_25 },
    { 0x7f8d16a7, SUC_lexicon_tags_11 },
    { 0x53cd16a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00d16ab, SUC_lexicon_tags_45 },
    { 0xfad16ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x116d16ae, SUC_lexicon_tags_10 },
    { 0x6e4d16af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf94d16b1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bed16b4, SUC_lexicon_tags_15 },
    { 0x486d16b4, SUC_lexicon_tags_7 },
    { 0xf6d16b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15cd16b8, SUC_lexicon_tags_5 },
    { 0x460d16b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8d16bd, SUC_lexicon_tags_15 },
    { 0xaa0d16be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984d16c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x858d16c6, SUC_lexicon_tags_37 },
    { 0xdbad16c7, SUC_lexicon_tags_9 },
    { 0x90cd16c8, SUC_lexicon_tags_27 },
    { 0x70cd16c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470d16cf, SUC_lexicon_tags_11 },
    { 0x30ad16d0, SUC_lexicon_tags_7 },
    { 0x1c6d16d1, SUC_lexicon_tags_45 },
    { 0x716d16d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22d16d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d16e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568d16f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacd16f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ad16fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x560d16fd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2d1703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d0d1705, SUC_lexicon_tags_46 },
    { 0x544d1705, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cd170b, SUC_lexicon_tags_14 },
    { 0x116d170b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d1716, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebad171b, SUC_lexicon_tags_40 },
    { 0x708d171c, SUC_lexicon_tags_82 },
    { 0xbd8d171c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d1720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eed1723, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ded1726, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cd172a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ad1730, SUC_lexicon_tags_79 },
    { 0x7bad1731, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2d1736, SUC_lexicon_tags_1 },
    { 0x43ed1736, SUC_lexicon_tags_39 },
    { 0x216d1736, SUC_lexicon_tags_49 },
    { 0x284d1739, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x930d173b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa06d173d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ed1740, SUC_lexicon_tags_7 },
    { 0x6ad1741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2d1744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92d174a, SUC_lexicon_tags_761 },
    { 0xf8ad174a, SUC_lexicon_tags_8 },
    { 0xb18d174b, SUC_lexicon_tags_63 },
    { 0xf48d174d, SUC_lexicon_tags_37 },
    { 0x9d6d174e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xddad1750, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9f2d1752, SUC_lexicon_tags_31 },
    { 0x108d1752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aad1757, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x7bed1759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2d175d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ad1763, SUC_lexicon_tags_25 },
    { 0x366d1764, SUC_lexicon_tags_35 },
    { 0x186d1763, SUC_lexicon_tags_25 },
    { 0xb66d1764, SUC_lexicon_tags_37 },
    { 0xbf4d1764, SUC_lexicon_tags_47 },
    { 0x74ed1768, SUC_lexicon_tags_0 },
    { 0xee6d1768, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ad176c, SUC_lexicon_tags_7 },
    { 0x5a0d176d, SUC_lexicon_tags_17 },
    { 0xd4d176c, SUC_lexicon_tags_11 },
    { 0xe5ad176e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6d1772, SUC_lexicon_tags_0 },
    { 0x89ed1773, SUC_lexicon_tags_10 },
    { 0x1a2d1774, SUC_lexicon_tags_10 },
    { 0x4aed1774, SUC_lexicon_tags_2 },
    { 0xb82d1774, SUC_lexicon_tags_33 },
    { 0x328d1777, SUC_lexicon_tags_29 },
    { 0x52ad1778, SUC_lexicon_tags_27 },
    { 0xaa8d1779, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8d1780, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x904d1783, SUC_lexicon_tags_5 },
    { 0x598d1784, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc4ed1786, SUC_lexicon_tags_14 },
    { 0x1fed1786, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8d178c, SUC_lexicon_tags_12 },
    { 0xeecd178c, SUC_lexicon_tags_11 },
    { 0x754d178d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee2d1790, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a6d1792, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2d1798, SUC_lexicon_tags_23 },
    { 0x98ed1799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8ed179b, SUC_lexicon_tags_21 },
    { 0x968d179b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bcd179e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6d17a2, SUC_lexicon_tags_9 },
    { 0xdcad17a3, SUC_lexicon_tags_26 },
    { 0xb0d17a2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d17ac, SUC_lexicon_tags_17 },
    { 0x26cd17ad, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcd17b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd2d17b2, SUC_lexicon_tags_25 },
    { 0x5ecd17b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x548d17b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38d17b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ed17bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7bed17be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf10d17c0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x312d17c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda6d17c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafed17c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908d17d2, SUC_lexicon_tags_17 },
    { 0x7cd17d2, SUC_lexicon_tags_2 },
    { 0x69ad17d2, SUC_lexicon_tags_38 },
    { 0x562d17d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4d17d8, SUC_lexicon_tags_30 },
    { 0xb0ed17d8, SUC_lexicon_tags_14 },
    { 0x1d6d17da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2d17de, SUC_lexicon_tags_39 },
    { 0xa44d17df, SUC_lexicon_tags_7 },
    { 0xa9ad17df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54cd17e2, SUC_lexicon_tags_38 },
    { 0x9dcd17e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ad17e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ad17ec, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x502d17ee, SUC_lexicon_tags_25 },
    { 0x5c2d17ef, SUC_lexicon_tags_17 },
    { 0x866d17ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccd17f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322d17fa, SUC_lexicon_tags_17 },
    { 0xf94d17fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab8d17fd, SUC_lexicon_tags_1 },
    { 0x39ed17fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcd1802, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460d1806, SUC_lexicon_tags_9 },
    { 0x7cad1806, SUC_lexicon_tags_54 },
    { 0x5ad1808, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2d180b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ed180e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaad1810, SUC_lexicon_tags_46 },
    { 0x8e4d1811, SUC_lexicon_tags_43 },
    { 0x7b8d1812, SUC_lexicon_tags_11 },
    { 0x2aed1813, SUC_lexicon_tags_15 },
    { 0x670d1813, SUC_lexicon_tags_29 },
    { 0xdf0d1815, SUC_lexicon_tags_4 },
    { 0x3b6d1816, SUC_lexicon_tags_25 },
    { 0xbe8d1817, SUC_lexicon_tags_6 },
    { 0x616d1816, SUC_lexicon_tags_47 },
    { 0x54cd1818, SUC_lexicon_tags_15 },
    { 0xbfed1816, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26d181d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x222d181f, SUC_lexicon_tags_68 },
    { 0xc70d181f, SUC_lexicon_tags_88 },
    { 0xbd8d1821, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ad1826, SUC_lexicon_tags_17 },
    { 0x818d1827, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b6d1829, SUC_lexicon_tags_27 },
    { 0x9e2d1829, SUC_lexicon_tags_50 },
    { 0xe56d182a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x440d182e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16d1836, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf84d1838, SUC_lexicon_tags_8 },
    { 0x156d1839, SUC_lexicon_tags_25 },
    { 0xeaed1839, SUC_lexicon_tags_2 },
    { 0xf64d183a, SUC_lexicon_tags_37 },
    { 0xaf8d1839, SUC_lexicon_tags_12 },
    { 0xc64d183d, SUC_lexicon_tags_69 },
    { 0x2bcd183b, SUC_lexicon_tags_11 },
    { 0xd2cd183c, SUC_lexicon_tags_23 },
    { 0xe76d1840, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d1844, SUC_lexicon_tags_0 },
    { 0x968d1844, SUC_lexicon_tags_45 },
    { 0xfced1846, SUC_lexicon_tags_17 },
    { 0x876d1847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4d1850, SUC_lexicon_tags_17 },
    { 0x7d2d1850, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ed1855, SUC_lexicon_tags_17 },
    { 0xbc2d1856, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x940d1858, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26d185c, SUC_lexicon_tags_23 },
    { 0x1dcd185d, SUC_lexicon_tags_63 },
    { 0xd0d185d, SUC_lexicon_tags_15 },
    { 0x17ad185e, SUC_lexicon_tags_57 },
    { 0xf1ed1860, SUC_lexicon_tags_11 },
    { 0xc96d1861, SUC_lexicon_tags_42 },
    { 0x77cd1862, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4d1867, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x842d1869, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x952d186b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4d1872, SUC_lexicon_tags_60 },
    { 0x306d1873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4d1876, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2d1879, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4d187d, SUC_lexicon_tags_11 },
    { 0x540d187e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220d1883, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cad1886, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2d1890, SUC_lexicon_tags_27 },
    { 0xcf6d1891, SUC_lexicon_tags_17 },
    { 0xbe0d1892, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x980d1895, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22d189c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ad189f, SUC_lexicon_tags_12 },
    { 0x98cd18a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ed18a3, SUC_lexicon_tags_75 },
    { 0xf60d18a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4d18a8, SUC_lexicon_tags_47 },
    { 0x2bad18a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe00d18ab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccd18ae, SUC_lexicon_tags_31 },
    { 0x85ad18af, SUC_lexicon_tags_7 },
    { 0xbf2d18ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x864d18b2, SUC_lexicon_tags_37 },
    { 0x1c0d18b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cd18bb, SUC_lexicon_tags_29 },
    { 0xd96d18bb, SUC_lexicon_tags_11 },
    { 0xa2ad18bd, SUC_lexicon_tags_16 },
    { 0x44ad18be, SUC_lexicon_tags_15 },
    { 0x300d18bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ed18c2, SUC_lexicon_tags_38 },
    { 0xe5ad18c3, SUC_lexicon_tags_12 },
    { 0xf68d18c4, SUC_lexicon_tags_2 },
    { 0x276d18c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x110d18c7, SUC_lexicon_tags_9 },
    { 0x78ed18c8, SUC_lexicon_tags_82 },
    { 0x2bad18c7, SUC_lexicon_tags_15 },
    { 0x332d18ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12d18d7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60d18da, SUC_lexicon_tags_25 },
    { 0x456d18db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4d18df, SUC_lexicon_tags_37 },
    { 0x2f4d18e0, SUC_lexicon_tags_38 },
    { 0x826d18e0, SUC_lexicon_tags_30 },
    { 0xf86d18e2, SUC_lexicon_tags_15 },
    { 0xddcd18e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68d18ed, SUC_lexicon_tags_37 },
    { 0x9b8d18ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97ed18f0, SUC_lexicon_tags_15 },
    { 0x7c0d18f0, SUC_lexicon_tags_37 },
    { 0xb14d18f2, SUC_lexicon_tags_37 },
    { 0xa66d18f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x342d18f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d18f8, SUC_lexicon_tags_151 },
    { 0x454d18f9, SUC_lexicon_tags_84 },
    { 0xa34d18fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36d18fd, SUC_lexicon_tags_2 },
    { 0x852d18fd, SUC_lexicon_tags_17 },
    { 0x3bed18fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6d1902, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5ed1904, SUC_lexicon_tags_28 },
    { 0xacad1904, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59ed1907, SUC_lexicon_tags_33 },
    { 0xaa6d1907, SUC_lexicon_tags_15 },
    { 0xac8d1907, SUC_lexicon_tags_28 },
    { 0x810d190a, SUC_lexicon_tags_25 },
    { 0x558d190b, SUC_lexicon_tags_28 },
    { 0x3dcd190a, SUC_lexicon_tags_27 },
    { 0x704d190a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x796d1910, SUC_lexicon_tags_7 },
    { 0xaf2d1911, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eed1919, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xed191b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66d1921, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146d1928, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8d1932, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb58d1934, SUC_lexicon_tags_17 },
    { 0x5fad1935, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a6d1937, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x79cd193a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cd193e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bed1942, SUC_lexicon_tags_2 },
    { 0x446d1943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad1947, SUC_lexicon_tags_12 },
    { 0xb42d1948, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x986d194b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4d194f, SUC_lexicon_tags_12 },
    { 0xd02d194f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34d1956, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506d195a, SUC_lexicon_tags_45 },
    { 0x484d195b, SUC_lexicon_tags_7 },
    { 0xc94d195c, SUC_lexicon_tags_66 },
    { 0x2eed195d, SUC_lexicon_tags_60 },
    { 0xce8d195c, SUC_lexicon_tags_1 },
    { 0xc34d195f, SUC_lexicon_tags_60 },
    { 0xcf8d1960, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8d1965, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8d1968, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2ed196a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cad196d, SUC_lexicon_tags_53 },
    { 0x340d196e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cad1970, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bad1973, SUC_lexicon_tags_21 },
    { 0xac8d1974, SUC_lexicon_tags_19 },
    { 0x7bed1974, SUC_lexicon_tags_17 },
    { 0xb58d1973, SUC_lexicon_tags_28 },
    { 0x8f4d1977, SUC_lexicon_tags_11 },
    { 0xc0d1978, SUC_lexicon_tags_49 },
    { 0x590d1974, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6d197f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x644d1981, SUC_lexicon_tags_12 },
    { 0xd46d1982, SUC_lexicon_tags_49 },
    { 0xc4ed1983, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0d1986, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x826d1988, SUC_lexicon_tags_49 },
    { 0x8a2d1989, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x20cd198b, SUC_lexicon_tags_5 },
    { 0xb84d198c, SUC_lexicon_tags_49 },
    { 0x2ad198b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4d1990, SUC_lexicon_tags_25 },
    { 0x578d1991, SUC_lexicon_tags_14 },
    { 0x9a8d1990, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0d1995, SUC_lexicon_tags_37 },
    { 0x366d1996, SUC_lexicon_tags_8 },
    { 0xd94d1997, SUC_lexicon_tags_11 },
    { 0x328d1998, SUC_lexicon_tags_38 },
    { 0x6ecd1996, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x6b2d199b, SUC_lexicon_tags_79 },
    { 0x45cd199c, SUC_lexicon_tags_25 },
    { 0xdecd199b, SUC_lexicon_tags_17 },
    { 0xe00d199d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530d19a2, SUC_lexicon_tags_17 },
    { 0x5c4d19a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2d19a6, SUC_lexicon_tags_38 },
    { 0xa58d19a7, SUC_lexicon_tags_12 },
    { 0x6d0d19a7, SUC_lexicon_tags_21 },
    { 0xe2ed19a9, SUC_lexicon_tags_1 },
    { 0xae8d19aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a0d19ac, SUC_lexicon_tags_108 },
    { 0x6e8d19ad, SUC_lexicon_tags_30 },
    { 0x982d19ae, SUC_lexicon_tags_37 },
    { 0x8dad19af, SUC_lexicon_tags_25 },
    { 0x700d19b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4cd19b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf06d19b4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xfd8d19b6, SUC_lexicon_tags_29 },
    { 0x8a0d19b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118d19be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0d19c2, SUC_lexicon_tags_15 },
    { 0x4a2d19c3, SUC_lexicon_tags_14 },
    { 0x5cd19c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2d19cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fed19d1, SUC_lexicon_tags_11 },
    { 0x36d19d2, SUC_lexicon_tags_45 },
    { 0xb6cd19d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04d19d9, SUC_lexicon_tags_6 },
    { 0x686d19da, SUC_lexicon_tags_17 },
    { 0x2e6d19db, SUC_lexicon_tags_25 },
    { 0xd04d19dc, SUC_lexicon_tags_10 },
    { 0xd98d19dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748d19e1, SUC_lexicon_tags_4 },
    { 0xcc2d19e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd14d19e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2d19e7, SUC_lexicon_tags_84 },
    { 0xb66d19e8, SUC_lexicon_tags_38 },
    { 0xd6d19e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6d19ee, SUC_lexicon_tags_17 },
    { 0x740d19ef, SUC_lexicon_tags_27 },
    { 0xef8d19f0, SUC_lexicon_tags_48 },
    { 0x4f4d19ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xecad19f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe56d19f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecd19f9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0d19fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ed1a01, SUC_lexicon_tags_1 },
    { 0x696d1a02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570d1a0a, SUC_lexicon_tags_15 },
    { 0x410d1a0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98d1a11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17cd1a13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1cd1a15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc4d1a17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2d1a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d1a20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb62d1a22, SUC_lexicon_tags_11 },
    { 0x3a4d1a23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4d1a2a, SUC_lexicon_tags_25 },
    { 0x116d1a2a, SUC_lexicon_tags_11 },
    { 0xa82d1a2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72d1a31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cd1a34, SUC_lexicon_tags_38 },
    { 0x31cd1a34, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cd1a3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2d1a3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ad1a43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eed1a47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ad1a4f, SUC_lexicon_tags_23 },
    { 0x7aad1a50, SUC_lexicon_tags_66 },
    { 0xb88d1a51, SUC_lexicon_tags_46 },
    { 0x42cd1a51, SUC_lexicon_tags_30 },
    { 0xc00d1a52, SUC_lexicon_tags_6 },
    { 0x2d2d1a53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898d1a5a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6d1a5f, SUC_lexicon_tags_14 },
    { 0xd08d1a60, SUC_lexicon_tags_13 },
    { 0x800d1a61, SUC_lexicon_tags_25 },
    { 0x190d1a61, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe4ed1a64, SUC_lexicon_tags_37 },
    { 0xfded1a65, SUC_lexicon_tags_14 },
    { 0xbfad1a66, SUC_lexicon_tags_11 },
    { 0x71ed1a67, SUC_lexicon_tags_11 },
    { 0x590d1a66, SUC_lexicon_tags_8 },
    { 0x6ded1a68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c6d1a6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d8d1a6d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd1a70, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62d1a73, SUC_lexicon_tags_10 },
    { 0xa94d1a73, SUC_lexicon_tags_277 },
    { 0x2aad1a75, SUC_lexicon_tags_47 },
    { 0x822d1a76, SUC_lexicon_tags_9 },
    { 0x9c2d1a76, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cd1a7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xed1a7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0d1a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd8d1a82, SUC_lexicon_tags_47 },
    { 0x1e8d1a83, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8d1a88, SUC_lexicon_tags_47 },
    { 0x4bed1a89, SUC_lexicon_tags_12 },
    { 0x550d1a8a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8d1a8d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x71ed1a8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4ad1a91, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76d1a94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccd1a99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3dcd1a9b, SUC_lexicon_tags_16 },
    { 0x5acd1a9b, SUC_lexicon_tags_1 },
    { 0x63ad1a9c, SUC_lexicon_tags_27 },
    { 0xf8ed1a9e, SUC_lexicon_tags_45 },
    { 0xb30d1a9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaced1aa3, SUC_lexicon_tags_43 },
    { 0x2c8d1aa4, SUC_lexicon_tags_25 },
    { 0x5b8d1aa4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6f8d1aa7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbd6d1aa9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76d1aaf, SUC_lexicon_tags_7 },
    { 0x1a2d1ab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f2d1ab2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x548d1ab4, SUC_lexicon_tags_6 },
    { 0x40ed1ab5, SUC_lexicon_tags_15 },
    { 0xff8d1ab6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ed1abc, SUC_lexicon_tags_12 },
    { 0x532d1abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x516d1abf, SUC_lexicon_tags_11 },
    { 0x6c4d1ac0, SUC_lexicon_tags_25 },
    { 0xea4d1abf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xafcd1ac3, SUC_lexicon_tags_7 },
    { 0x53ad1ac3, SUC_lexicon_tags_1 },
    { 0xc32d1ac3, SUC_lexicon_tags_15 },
    { 0xe04d1ac4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x758d1ac8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2d1acb, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x5ead1acd, SUC_lexicon_tags_15 },
    { 0x6aad1acd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80ad1ad0, SUC_lexicon_tags_37 },
    { 0xae4d1ad0, SUC_lexicon_tags_14 },
    { 0xaf6d1ad0, SUC_lexicon_tags_43 },
    { 0xa6cd1ad1, SUC_lexicon_tags_17 },
    { 0x1cd1ad4, SUC_lexicon_tags_37 },
    { 0x6ced1ad5, SUC_lexicon_tags_12 },
    { 0xd08d1ad6, SUC_lexicon_tags_54 },
    { 0x856d1ad7, SUC_lexicon_tags_35 },
    { 0x32d1ad6, SUC_lexicon_tags_15 },
    { 0x4a8d1ad9, SUC_lexicon_tags_33 },
    { 0x85ed1ada, SUC_lexicon_tags_35 },
    { 0xf74d1ad9, SUC_lexicon_tags_37 },
    { 0x41cd1ad6, SUC_lexicon_tags_26 },
    { 0x86d1ad7, SUC_lexicon_tags_46 },
    { 0x8b2d1ade, SUC_lexicon_tags_6 },
    { 0xdfcd1adf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x884d1ae2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16d1aec, SUC_lexicon_tags_15 },
    { 0x55ed1aed, SUC_lexicon_tags_37 },
    { 0x6f0d1aed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ed1af1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4d1af4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4cd1af6, SUC_lexicon_tags_25 },
    { 0xeccd1af7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cd1afc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2d1b03, SUC_lexicon_tags_12 },
    { 0xbe6d1b04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf2d1b06, SUC_lexicon_tags_1 },
    { 0xcb2d1b07, SUC_lexicon_tags_37 },
    { 0xe7ad1b07, SUC_lexicon_tags_14 },
    { 0x1a8d1b09, SUC_lexicon_tags_15 },
    { 0x750d1b0a, SUC_lexicon_tags_17 },
    { 0x4d2d1b06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x72cd1b0e, SUC_lexicon_tags_23 },
    { 0x262d1b0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d1b11, SUC_lexicon_tags_25 },
    { 0x8eed1b12, SUC_lexicon_tags_29 },
    { 0x98ed1b11, SUC_lexicon_tags_23 },
    { 0x992d1b14, SUC_lexicon_tags_63 },
    { 0xa3ad1b14, SUC_lexicon_tags_27 },
    { 0xb58d1b16, SUC_lexicon_tags_7 },
    { 0x29cd1b15, SUC_lexicon_tags_7 },
    { 0x61ed1b17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2d1b1a, SUC_lexicon_tags_17 },
    { 0xccd1b1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712d1b1f, SUC_lexicon_tags_17 },
    { 0x51ad1b20, SUC_lexicon_tags_7 },
    { 0x372d1b21, SUC_lexicon_tags_17 },
    { 0xb6cd1b21, SUC_lexicon_tags_37 },
    { 0xed4d1b22, SUC_lexicon_tags_15 },
    { 0x982d1b22, SUC_lexicon_tags_38 },
    { 0x336d1b1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86d1b2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2d1b3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cd1b3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ed1b43, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fed1b46, SUC_lexicon_tags_14 },
    { 0x538d1b47, SUC_lexicon_tags_9 },
    { 0xac4d1b48, SUC_lexicon_tags_15 },
    { 0xc10d1b49, SUC_lexicon_tags_11 },
    { 0xa04d1b49, SUC_lexicon_tags_39 },
    { 0xf5ed1b48, SUC_lexicon_tags_45 },
    { 0xebad1b4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2d1b4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93ed1b51, SUC_lexicon_tags_47 },
    { 0xd4ad1b51, SUC_lexicon_tags_29 },
    { 0xf82d1b53, SUC_lexicon_tags_22 },
    { 0xaed1b54, SUC_lexicon_tags_17 },
    { 0xcecd1b53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2cd1b57, SUC_lexicon_tags_2 },
    { 0x286d1b58, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x25ed1b5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1ced1b5c, SUC_lexicon_tags_31 },
    { 0xa6cd1b5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x700d1b60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8d1b63, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20d1b67, SUC_lexicon_tags_29 },
    { 0x766d1b68, SUC_lexicon_tags_29 },
    { 0xc10d1b68, SUC_lexicon_tags_17 },
    { 0xb38d1b6a, SUC_lexicon_tags_25 },
    { 0xd7ad1b68, SUC_lexicon_tags_25 },
    { 0x292d1b6c, SUC_lexicon_tags_26 },
    { 0xd96d1b67, SUC_lexicon_tags_23 },
    { 0x8acd1b6e, SUC_lexicon_tags_12 },
    { 0x184d1b6e, SUC_lexicon_tags_17 },
    { 0xda8d1b6e, SUC_lexicon_tags_11 },
    { 0x556d1b71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cd1b73, SUC_lexicon_tags_12 },
    { 0xbecd1b73, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4d1b78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ed1b85, SUC_lexicon_tags_37 },
    { 0x160d1b85, SUC_lexicon_tags_54 },
    { 0x4bed1b87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880d1b8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1b8e, SUC_lexicon_tags_8 },
    { 0x614d1b8f, SUC_lexicon_tags_4 },
    { 0x3d4d1b90, SUC_lexicon_tags_7 },
    { 0x24ed1b90, SUC_lexicon_tags_17 },
    { 0xdf2d1b91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ad1b95, SUC_lexicon_tags_7 },
    { 0xf56d1b96, SUC_lexicon_tags_2 },
    { 0xc8d1b97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ed1b9a, SUC_lexicon_tags_9 },
    { 0x586d1b9a, SUC_lexicon_tags_49 },
    { 0xea6d1b9b, SUC_lexicon_tags_7 },
    { 0x382d1b9a, SUC_lexicon_tags_37 },
    { 0xb8d1b9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558d1ba6, SUC_lexicon_tags_21 },
    { 0x876d1ba6, SUC_lexicon_tags_88 },
    { 0xd62d1ba6, SUC_lexicon_tags_2 },
    { 0x98ad1ba8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad1bac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ad1bb0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88d1bb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x984d1bb6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ad1bbb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824d1bbf, SUC_lexicon_tags_15 },
    { 0x422d1bc0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d1bcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226d1bd0, SUC_lexicon_tags_47 },
    { 0xdb0d1bd1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x302d1bd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xedad1bd6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84d1bda, SUC_lexicon_tags_34 },
    { 0xe06d1bda, SUC_lexicon_tags_1 },
    { 0x5a8d1bdb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0d1be0, SUC_lexicon_tags_35 },
    { 0x13cd1be1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1ed1be3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ed1bed, SUC_lexicon_tags_2 },
    { 0xcaad1bee, SUC_lexicon_tags_12 },
    { 0x89ad1bef, SUC_lexicon_tags_1 },
    { 0x962d1bee, SUC_lexicon_tags_27 },
    { 0x2acd1bef, SUC_lexicon_tags_27 },
    { 0xe3cd1bf1, SUC_lexicon_tags_16 },
    { 0x82ed1bf2, SUC_lexicon_tags_2 },
    { 0x678d1bf1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x706d1bf7, SUC_lexicon_tags_12 },
    { 0xe54d1bf7, SUC_lexicon_tags_7 },
    { 0x54d1bf8, SUC_lexicon_tags_12 },
    { 0xf90d1bf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8d1c01, SUC_lexicon_tags_12 },
    { 0x1fcd1c01, SUC_lexicon_tags_0 },
    { 0xced1c01, SUC_lexicon_tags_1 },
    { 0xd1c04, SUC_lexicon_tags_0 },
    { 0x352d1c02, SUC_lexicon_tags_26 },
    { 0xb84d1c06, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782d1c0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86d1c15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68d1c17, SUC_lexicon_tags_28 },
    { 0x34ad1c18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedcd1c1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe2ad1c1d, SUC_lexicon_tags_14 },
    { 0x112d1c1e, SUC_lexicon_tags_25 },
    { 0x3b4d1c1e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50d1c24, SUC_lexicon_tags_11 },
    { 0x504d1c25, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484d1c2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ed1c2f, SUC_lexicon_tags_15 },
    { 0x6bed1c30, SUC_lexicon_tags_45 },
    { 0x914d1c31, SUC_lexicon_tags_11 },
    { 0xcfcd1c32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x40ed1c34, SUC_lexicon_tags_35 },
    { 0x2ed1c34, SUC_lexicon_tags_30 },
    { 0xc18d1c34, SUC_lexicon_tags_0 },
    { 0x262d1c37, SUC_lexicon_tags_2 },
    { 0x728d1c38, SUC_lexicon_tags_25 },
    { 0x75ad1c39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d1c3c, SUC_lexicon_tags_27 },
    { 0x318d1c3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10cd1c45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0d1c48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ead1c4d, SUC_lexicon_tags_70 },
    { 0xc14d1c4e, SUC_lexicon_tags_1 },
    { 0xf18d1c4f, SUC_lexicon_tags_0 },
    { 0xc78d1c50, SUC_lexicon_tags_37 },
    { 0x7aad1c51, SUC_lexicon_tags_17 },
    { 0xe62d1c51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cd1c5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe6ad1c5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c4d1c5f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcb2d1c61, SUC_lexicon_tags_29 },
    { 0x88cd1c62, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8d1c6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cd1c72, SUC_lexicon_tags_9 },
    { 0xe2cd1c73, SUC_lexicon_tags_543 },
    { 0x62d1c72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x332d1c76, SUC_lexicon_tags_37 },
    { 0xe6cd1c77, SUC_lexicon_tags_45 },
    { 0x19ad1c78, SUC_lexicon_tags_8 },
    { 0x1a4d1c79, SUC_lexicon_tags_40 },
    { 0xea4d1c76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bad1c80, SUC_lexicon_tags_17 },
    { 0x32ed1c81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9cad1c83, SUC_lexicon_tags_29 },
    { 0x392d1c84, SUC_lexicon_tags_17 },
    { 0x77cd1c85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xee0d1c87, SUC_lexicon_tags_33 },
    { 0xf20d1c88, SUC_lexicon_tags_0 },
    { 0xdb4d1c89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccd1c8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6d1c91, SUC_lexicon_tags_11 },
    { 0x960d1c91, SUC_lexicon_tags_4 },
    { 0x906d1c91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x818d1c95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388d1c99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbced1c9c, SUC_lexicon_tags_2 },
    { 0x26ed1c9d, SUC_lexicon_tags_4 },
    { 0x6aed1c9e, SUC_lexicon_tags_9 },
    { 0x602d1c9e, SUC_lexicon_tags_1 },
    { 0xc3ad1ca0, SUC_lexicon_tags_8 },
    { 0x10cd1c9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb0d1ca3, SUC_lexicon_tags_17 },
    { 0x50cd1ca3, SUC_lexicon_tags_37 },
    { 0x5b6d1ca3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ed1cae, SUC_lexicon_tags_12 },
    { 0xa9cd1caf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cd1cb2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ed1cb8, SUC_lexicon_tags_60 },
    { 0x492d1cb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc90d1cbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x876d1cbe, SUC_lexicon_tags_40 },
    { 0xbead1cbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0d1cc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a4d1cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ed1ccc, SUC_lexicon_tags_1 },
    { 0x3e8d1ccd, SUC_lexicon_tags_11 },
    { 0x9cd1ccc, SUC_lexicon_tags_12 },
    { 0xf3ad1ccf, SUC_lexicon_tags_4 },
    { 0x456d1cd0, SUC_lexicon_tags_235 },
    { 0x4c0d1cd1, SUC_lexicon_tags_26 },
    { 0x3ed1cd2, SUC_lexicon_tags_10 },
    { 0x590d1cd3, SUC_lexicon_tags_12 },
    { 0x320d1cd3, SUC_lexicon_tags_28 },
    { 0xa0ad1cd5, SUC_lexicon_tags_11 },
    { 0xadcd1cd6, SUC_lexicon_tags_55 },
    { 0x83ad1cd6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78d1cde, SUC_lexicon_tags_0 },
    { 0xd20d1cde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2d1ce4, SUC_lexicon_tags_87 },
    { 0xf80d1ce4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642d1cea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xac0d1cec, SUC_lexicon_tags_60 },
    { 0x8ed1ced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecd1cf0, SUC_lexicon_tags_15 },
    { 0x976d1cf0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6d1cf4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf2cd1cf6, SUC_lexicon_tags_6 },
    { 0xd04d1cf7, SUC_lexicon_tags_60 },
    { 0xbfcd1cf8, SUC_lexicon_tags_12 },
    { 0xab4d1cf9, SUC_lexicon_tags_30 },
    { 0x5acd1cfa, SUC_lexicon_tags_45 },
    { 0xa48d1cfb, SUC_lexicon_tags_37 },
    { 0x2a8d1cfc, SUC_lexicon_tags_0 },
    { 0xb88d1cfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8d1d01, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ed1d07, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa4ad1d09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cd1d13, SUC_lexicon_tags_77 },
    { 0x3c0d1d14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ed1d1c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cd1d27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14d1d2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x87cd1d2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fad1d31, SUC_lexicon_tags_12 },
    { 0x27ed1d32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e8d1d34, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x730d1d36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00d1d3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef8d1d3e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cd1d44, SUC_lexicon_tags_11 },
    { 0x7f4d1d44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16d1d47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d1d4d, SUC_lexicon_tags_55 },
    { 0xa8cd1d4e, SUC_lexicon_tags_7 },
    { 0x2d1d4f, SUC_lexicon_tags_1202 },
    { 0x974d1d4e, SUC_lexicon_tags_25 },
    { 0xbf4d1d51, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224d1d56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5cd1d58, SUC_lexicon_tags_33 },
    { 0x248d1d58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd88d1d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ded1d60, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cd1d6c, SUC_lexicon_tags_37 },
    { 0xfc8d1d6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6d1d71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbad1d77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96d1d7b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6d1d7d, SUC_lexicon_tags_0 },
    { 0x6f6d1d7e, SUC_lexicon_tags_25 },
    { 0xc26d1d7f, SUC_lexicon_tags_17 },
    { 0x116d1d7e, SUC_lexicon_tags_38 },
    { 0x56cd1d7d, SUC_lexicon_tags_0 },
    { 0xb3ed1d7e, SUC_lexicon_tags_2 },
    { 0x59cd1d81, SUC_lexicon_tags_12 },
    { 0x760d1d84, SUC_lexicon_tags_30 },
    { 0xc76d1d85, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf06d1d87, SUC_lexicon_tags_69 },
    { 0x62cd1d88, SUC_lexicon_tags_23 },
    { 0xbc4d1d87, SUC_lexicon_tags_25 },
    { 0x1d0d1d87, SUC_lexicon_tags_14 },
    { 0x626d1d8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd1d8e, SUC_lexicon_tags_11 },
    { 0x9f4d1d8f, SUC_lexicon_tags_14 },
    { 0xa96d1d8e, SUC_lexicon_tags_1 },
    { 0x83cd1d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8d1d95, SUC_lexicon_tags_37 },
    { 0x2ecd1d96, SUC_lexicon_tags_38 },
    { 0xa90d1d97, SUC_lexicon_tags_7 },
    { 0x664d1d98, SUC_lexicon_tags_60 },
    { 0xd4ad1d95, SUC_lexicon_tags_38 },
    { 0x4d2d1d95, SUC_lexicon_tags_47 },
    { 0x406d1d9b, SUC_lexicon_tags_15 },
    { 0x772d1d9c, SUC_lexicon_tags_23 },
    { 0x20ed1d9d, SUC_lexicon_tags_12 },
    { 0x51cd1d9e, SUC_lexicon_tags_1 },
    { 0xc8d1d9f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf46d1da1, SUC_lexicon_tags_106 },
    { 0x2aed1da2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68ed1da4, SUC_lexicon_tags_38 },
    { 0x7a6d1da4, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2d1da8, SUC_lexicon_tags_0 },
    { 0x45ed1da9, SUC_lexicon_tags_32 },
    { 0xa9cd1da9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9ad1dac, SUC_lexicon_tags_47 },
    { 0x542d1dad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4ad1daf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4d1db8, SUC_lexicon_tags_7 },
    { 0x71ed1db9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa84d1dbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbced1dc1, SUC_lexicon_tags_12 },
    { 0x754d1dc2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1ced1dc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2d1dc8, SUC_lexicon_tags_39 },
    { 0x844d1dc9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cd1dce, SUC_lexicon_tags_14 },
    { 0x78d1dcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128d1dd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ad1dde, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ad1de1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6d1de4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ed1de9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x302d1dec, SUC_lexicon_tags_154 },
    { 0x822d1dec, SUC_lexicon_tags_45 },
    { 0x4b4d1ded, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xa82d1df0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4d2d1df2, SUC_lexicon_tags_87 },
    { 0x898d1df2, SUC_lexicon_tags_7 },
    { 0xac4d1df2, SUC_lexicon_tags_25 },
    { 0xaed1df4, SUC_lexicon_tags_11 },
    { 0x368d1df4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0d1df9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d4d1dfb, SUC_lexicon_tags_2 },
    { 0xaecd1dfb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad1dff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ed1e08, SUC_lexicon_tags_26 },
    { 0x74ad1e08, SUC_lexicon_tags_23 },
    { 0x1a8d1e0a, SUC_lexicon_tags_341 },
    { 0xde2d1e0a, SUC_lexicon_tags_47 },
    { 0xf80d1e0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x706d1e0f, SUC_lexicon_tags_21 },
    { 0x752d1e0f, SUC_lexicon_tags_47 },
    { 0xaf8d1e11, SUC_lexicon_tags_17 },
    { 0xfa6d1e11, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x106d1e15, SUC_lexicon_tags_2 },
    { 0x1e0d1e16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c4d1e18, SUC_lexicon_tags_11 },
    { 0xbb0d1e18, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54d1e1c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4d1e22, SUC_lexicon_tags_15 },
    { 0x86cd1e23, SUC_lexicon_tags_47 },
    { 0x166d1e24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed1e29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acd1e30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed1e34, SUC_lexicon_tags_63 },
    { 0x8bcd1e35, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0d1e3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbd2d1e3f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1d4d1e41, SUC_lexicon_tags_11 },
    { 0x37cd1e42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaded1e44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bad1e47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe8d1e49, SUC_lexicon_tags_63 },
    { 0x7ccd1e4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe02d1e4d, SUC_lexicon_tags_15 },
    { 0xcb6d1e4d, SUC_lexicon_tags_124 },
    { 0x36d1e4f, SUC_lexicon_tags_31 },
    { 0x96ed1e4f, SUC_lexicon_tags_25 },
    { 0xfdad1e4e, SUC_lexicon_tags_8 },
    { 0x632d1e52, SUC_lexicon_tags_47 },
    { 0xb44d1e53, SUC_lexicon_tags_25 },
    { 0xd3ed1e54, SUC_lexicon_tags_60 },
    { 0x99ed1e54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e6d1e57, SUC_lexicon_tags_14 },
    { 0x44cd1e58, SUC_lexicon_tags_27 },
    { 0xf8ad1e59, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd88d1e5b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x712d1e5d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ead1e62, SUC_lexicon_tags_16 },
    { 0x4d4d1e62, SUC_lexicon_tags_25 },
    { 0xe5ad1e64, SUC_lexicon_tags_19 },
    { 0x3bed1e65, SUC_lexicon_tags_27 },
    { 0x28d1e66, SUC_lexicon_tags_15 },
    { 0x448d1e63, SUC_lexicon_tags_12 },
    { 0x2c8d1e68, SUC_lexicon_tags_33 },
    { 0x206d1e67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cd1e6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1b0d1e6f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42d1e74, SUC_lexicon_tags_1 },
    { 0xd80d1e75, SUC_lexicon_tags_27 },
    { 0x606d1e76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6d1e7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848d1e83, SUC_lexicon_tags_11 },
    { 0xfb2d1e84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48d1e8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ed1e8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46d1e97, SUC_lexicon_tags_47 },
    { 0xdfcd1e97, SUC_lexicon_tags_11 },
    { 0xd46d1e99, SUC_lexicon_tags_1 },
    { 0x5b0d1e99, SUC_lexicon_tags_25 },
    { 0x49ad1e9b, SUC_lexicon_tags_49 },
    { 0x1e0d1e9b, SUC_lexicon_tags_60 },
    { 0x2fed1e97, SUC_lexicon_tags_33 },
    { 0x356d1e9e, SUC_lexicon_tags_37 },
    { 0x922d1e9e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18d1ea7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdad1eab, SUC_lexicon_tags_15 },
    { 0x68cd1eac, SUC_lexicon_tags_25 },
    { 0x5b6d1ead, SUC_lexicon_tags_60 },
    { 0xd8d1eae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210d1eb9, SUC_lexicon_tags_7 },
    { 0x4ced1eb9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeccd1ebc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682d1ec1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d1ec4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47ed1ec6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ced1ec9, SUC_lexicon_tags_15 },
    { 0xbfcd1ec9, SUC_lexicon_tags_12 },
    { 0xf3cd1ec9, SUC_lexicon_tags_1 },
    { 0x51ed1ecc, SUC_lexicon_tags_0 },
    { 0x618d1ec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ad1ed1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ced1ed4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ad1ed7, SUC_lexicon_tags_7 },
    { 0x16ed1ed8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x31ed1eda, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ad1ede, SUC_lexicon_tags_37 },
    { 0xb70d1edf, SUC_lexicon_tags_38 },
    { 0xdbed1ee0, SUC_lexicon_tags_49 },
    { 0x636d1ee0, SUC_lexicon_tags_37 },
    { 0x73cd1ee0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcd1ee5, SUC_lexicon_tags_2 },
    { 0xf56d1ee5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fad1ee9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ad1eef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0d1ef3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d1efa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x502d1efc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d1eff, SUC_lexicon_tags_37 },
    { 0xe48d1f00, SUC_lexicon_tags_34 },
    { 0x832d1f01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x114d1f03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4d1f08, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2d1f0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc62d1f10, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cd1f14, SUC_lexicon_tags_34 },
    { 0xb0cd1f15, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70d1f18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cd1f1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30d1f24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xad6d1f26, SUC_lexicon_tags_11 },
    { 0xefed1f27, SUC_lexicon_tags_26 },
    { 0x268d1f28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2d1f2b, SUC_lexicon_tags_64 },
    { 0x354d1f2c, SUC_lexicon_tags_11 },
    { 0x9d4d1f2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208d1f37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x954d1f39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6d1f41, SUC_lexicon_tags_37 },
    { 0x4ced1f42, SUC_lexicon_tags_1 },
    { 0x4fcd1f43, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286d1f48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d1f4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cd1f52, SUC_lexicon_tags_7 },
    { 0xedcd1f52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ead1f5b, SUC_lexicon_tags_37 },
    { 0x9d2d1f5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeed1f60, SUC_lexicon_tags_55 },
    { 0x22cd1f60, SUC_lexicon_tags_29 },
    { 0x362d1f60, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x106d1f64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ed1f6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76d1f6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76ad1f6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4d1f73, SUC_lexicon_tags_38 },
    { 0xbaed1f74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cd1f79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d1f7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ad1f81, SUC_lexicon_tags_47 },
    { 0x4b2d1f81, SUC_lexicon_tags_6 },
    { 0xa04d1f83, SUC_lexicon_tags_48 },
    { 0x986d1f84, SUC_lexicon_tags_45 },
    { 0xc94d1f84, SUC_lexicon_tags_14 },
    { 0x2f8d1f85, SUC_lexicon_tags_11 },
    { 0x592d1f85, SUC_lexicon_tags_17 },
    { 0x92ed1f84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64d1f8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874d1f93, SUC_lexicon_tags_14 },
    { 0x6ed1f94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26d1f9a, SUC_lexicon_tags_2 },
    { 0xb7ed1f9b, SUC_lexicon_tags_45 },
    { 0x546d1f9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80d1f9f, SUC_lexicon_tags_12 },
    { 0xa98d1fa0, SUC_lexicon_tags_32 },
    { 0x2fad1f9f, SUC_lexicon_tags_47 },
    { 0xbc8d1f9f, SUC_lexicon_tags_60 },
    { 0x4a0d1fa3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ced1fa7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930d1fb2, SUC_lexicon_tags_37 },
    { 0x35cd1fb3, SUC_lexicon_tags_11 },
    { 0xe1ed1fb3, SUC_lexicon_tags_23 },
    { 0xde0d1fb5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ed1fb8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x16d1fba, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccd1fc1, SUC_lexicon_tags_37 },
    { 0xbc6d1fc2, SUC_lexicon_tags_17 },
    { 0x4bcd1fc2, SUC_lexicon_tags_57 },
    { 0xdb2d1fc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c0d1fc6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf54d1fc8, SUC_lexicon_tags_12 },
    { 0xb58d1fc9, SUC_lexicon_tags_17 },
    { 0xb82d1fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920d1fcf, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cd1fd2, SUC_lexicon_tags_77 },
    { 0xe70d1fd3, SUC_lexicon_tags_63 },
    { 0xa32d1fd3, SUC_lexicon_tags_25 },
    { 0xe0ed1fd2, SUC_lexicon_tags_29 },
    { 0xcf0d1fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x578d1fd8, SUC_lexicon_tags_25 },
    { 0xaad1fd8, SUC_lexicon_tags_2 },
    { 0x480d1fda, SUC_lexicon_tags_46 },
    { 0xbd0d1fdb, SUC_lexicon_tags_37 },
    { 0x920d1fdb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf2d1fde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8d1fe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x280d1feb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16d1fed, SUC_lexicon_tags_11 },
    { 0xa8cd1fed, SUC_lexicon_tags_45 },
    { 0xa36d1fed, SUC_lexicon_tags_87 },
    { 0x6acd1ff0, SUC_lexicon_tags_4 },
    { 0x1f8d1ff1, SUC_lexicon_tags_17 },
    { 0x45ed1ff1, SUC_lexicon_tags_12 },
    { 0x27ad1ff0, SUC_lexicon_tags_302 },
    { 0x450d1ff4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcd1ff9, SUC_lexicon_tags_37 },
    { 0x238d1ff9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ed2000, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bed2003, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xee2d2005, SUC_lexicon_tags_19 },
    { 0xdded2005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b6d2008, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ad2011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570d2019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe14d201b, SUC_lexicon_tags_25 },
    { 0x504d201c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418d2022, SUC_lexicon_tags_4 },
    { 0x15ad2022, SUC_lexicon_tags_15 },
    { 0xe76d2024, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x882d2027, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2d2029, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x974d202b, SUC_lexicon_tags_1 },
    { 0x138d202c, SUC_lexicon_tags_79 },
    { 0x692d202d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634d2033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14d2036, SUC_lexicon_tags_18 },
    { 0xa1ad2036, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eed203c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bed2041, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf66d2043, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cd2046, SUC_lexicon_tags_26 },
    { 0xc1cd2047, SUC_lexicon_tags_19 },
    { 0x376d2047, SUC_lexicon_tags_37 },
    { 0x2d8d2049, SUC_lexicon_tags_17 },
    { 0x5a6d204a, SUC_lexicon_tags_11 },
    { 0x92ed204a, SUC_lexicon_tags_4 },
    { 0xda0d204c, SUC_lexicon_tags_15 },
    { 0x8e0d204d, SUC_lexicon_tags_3 },
    { 0x714d204e, SUC_lexicon_tags_14 },
    { 0xab8d204f, SUC_lexicon_tags_11 },
    { 0xfc6d2050, SUC_lexicon_tags_30 },
    { 0xfb2d2051, SUC_lexicon_tags_1 },
    { 0x90ed2051, SUC_lexicon_tags_12 },
    { 0x9cd2047, SUC_lexicon_tags_63 },
    { 0x16ad204c, SUC_lexicon_tags_31 },
    { 0x778d2055, SUC_lexicon_tags_1 },
    { 0xb3ed2056, SUC_lexicon_tags_11 },
    { 0xba4d2046, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acd205c, SUC_lexicon_tags_17 },
    { 0xbfad205d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d2061, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4d2066, SUC_lexicon_tags_4 },
    { 0xb7ad2067, SUC_lexicon_tags_15 },
    { 0x9fcd2068, SUC_lexicon_tags_27 },
    { 0x71cd2069, SUC_lexicon_tags_28 },
    { 0xf6ad206a, SUC_lexicon_tags_41 },
    { 0xa68d2066, SUC_lexicon_tags_14 },
    { 0x1b8d206c, SUC_lexicon_tags_17 },
    { 0x3e2d2067, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8d2075, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x34d2077, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02d207c, SUC_lexicon_tags_87 },
    { 0xcad207d, SUC_lexicon_tags_37 },
    { 0xe38d207e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd6d2080, SUC_lexicon_tags_37 },
    { 0x56ed2081, SUC_lexicon_tags_37 },
    { 0x870d2082, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94d2085, SUC_lexicon_tags_17 },
    { 0x1e4d2086, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x52cd2088, SUC_lexicon_tags_2 },
    { 0x56cd2089, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cd208d, SUC_lexicon_tags_12 },
    { 0x2f0d208e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad2092, SUC_lexicon_tags_84 },
    { 0x58ad2093, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280d2099, SUC_lexicon_tags_38 },
    { 0xd50d2099, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x258d209d, SUC_lexicon_tags_1 },
    { 0x574d209e, SUC_lexicon_tags_12 },
    { 0x834d209e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8d20a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64d20ac, SUC_lexicon_tags_33 },
    { 0xf06d20ad, SUC_lexicon_tags_37 },
    { 0x544d20ae, SUC_lexicon_tags_7 },
    { 0xf8d20af, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf1ed20b1, SUC_lexicon_tags_11 },
    { 0x5e6d20b2, SUC_lexicon_tags_9 },
    { 0xaad20b2, SUC_lexicon_tags_38 },
    { 0x7cad20b2, SUC_lexicon_tags_33 },
    { 0x494d20b5, SUC_lexicon_tags_30 },
    { 0x810d20b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd96d20b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x388d20ba, SUC_lexicon_tags_14 },
    { 0xcc8d20bb, SUC_lexicon_tags_30 },
    { 0xb04d20bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0d20bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754d20c5, SUC_lexicon_tags_87 },
    { 0xa8ad20c5, SUC_lexicon_tags_1 },
    { 0x100d20c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ad20cc, SUC_lexicon_tags_8 },
    { 0xec8d20cc, SUC_lexicon_tags_11 },
    { 0x7fcd20ce, SUC_lexicon_tags_33 },
    { 0x888d20cf, SUC_lexicon_tags_30 },
    { 0xc4d20cc, SUC_lexicon_tags_84 },
    { 0xa46d20d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ed20dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x502d20e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfbed20e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbed20eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x978d20ed, SUC_lexicon_tags_19 },
    { 0xcf6d20ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x714d20f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3e2d20f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96d20f5, SUC_lexicon_tags_25 },
    { 0x6a6d20f6, SUC_lexicon_tags_38 },
    { 0x21cd20f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb00d20f9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ad2106, SUC_lexicon_tags_47 },
    { 0x69ad2106, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe94d2109, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x55ed210b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ad210e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfead2115, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd2119, SUC_lexicon_tags_8 },
    { 0xfa6d211a, SUC_lexicon_tags_1 },
    { 0x4ced211b, SUC_lexicon_tags_37 },
    { 0x734d211c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ed211f, SUC_lexicon_tags_17 },
    { 0x45cd211f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72d2122, SUC_lexicon_tags_2 },
    { 0x25cd2123, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50d2126, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cd2129, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78d2130, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794d2134, SUC_lexicon_tags_6 },
    { 0x612d2135, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0d213c, SUC_lexicon_tags_17 },
    { 0xbbad213d, SUC_lexicon_tags_37 },
    { 0x4fad213d, SUC_lexicon_tags_28 },
    { 0xc3ed213c, SUC_lexicon_tags_7 },
    { 0xf4ad213d, SUC_lexicon_tags_26 },
    { 0xac8d2141, SUC_lexicon_tags_37 },
    { 0x740d2140, SUC_lexicon_tags_37 },
    { 0xf2d2143, SUC_lexicon_tags_1 },
    { 0x1a0d213c, SUC_lexicon_tags_1 },
    { 0x1e6d2145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214d214f, SUC_lexicon_tags_15 },
    { 0x148d214f, SUC_lexicon_tags_25 },
    { 0x7dcd2151, SUC_lexicon_tags_7 },
    { 0x190d2151, SUC_lexicon_tags_27 },
    { 0x804d214f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602d215a, SUC_lexicon_tags_38 },
    { 0x146d215a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcd2160, SUC_lexicon_tags_38 },
    { 0x29cd2161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x628d2164, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854d2169, SUC_lexicon_tags_37 },
    { 0xb90d216a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfacd216d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe68d216f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbdad2171, SUC_lexicon_tags_12 },
    { 0x2b8d2172, SUC_lexicon_tags_87 },
    { 0xeb2d2173, SUC_lexicon_tags_2 },
    { 0x242d2173, SUC_lexicon_tags_11 },
    { 0x6ccd2175, SUC_lexicon_tags_2 },
    { 0xe5ad2176, SUC_lexicon_tags_35 },
    { 0x160d2177, SUC_lexicon_tags_37 },
    { 0xb62d2177, SUC_lexicon_tags_15 },
    { 0xcf2d2172, SUC_lexicon_tags_31 },
    { 0x602d217a, SUC_lexicon_tags_22 },
    { 0x4ded217b, SUC_lexicon_tags_15 },
    { 0xf64d2175, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x61ed217e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436d2183, SUC_lexicon_tags_11 },
    { 0x8aed2184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fed2186, SUC_lexicon_tags_37 },
    { 0x1a2d2187, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60d2189, SUC_lexicon_tags_23 },
    { 0xb78d218a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cd2192, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62d2197, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27ad2199, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8d219c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x50cd219e, SUC_lexicon_tags_7 },
    { 0x8ecd219f, SUC_lexicon_tags_37 },
    { 0xecd21a0, SUC_lexicon_tags_47 },
    { 0xdeed21a0, SUC_lexicon_tags_37 },
    { 0x6f2d21a1, SUC_lexicon_tags_82 },
    { 0xa6ad21a1, SUC_lexicon_tags_33 },
    { 0x8c8d21a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x638d21a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d21a9, SUC_lexicon_tags_3 },
    { 0xf14d21aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ad21ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2d21b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc34d21b3, SUC_lexicon_tags_12 },
    { 0xd06d21b4, SUC_lexicon_tags_40 },
    { 0xfc4d21b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4d21ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x806d21bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x602d21be, SUC_lexicon_tags_11 },
    { 0x6c6d21be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6d21c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbdad21c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130d21c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x478d21ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ad21cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29ed21d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980d21d8, SUC_lexicon_tags_1 },
    { 0x4b2d21d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6d21db, SUC_lexicon_tags_230 },
    { 0xc32d21dc, SUC_lexicon_tags_1 },
    { 0x172d21dd, SUC_lexicon_tags_1501 },
    { 0x83cd21de, SUC_lexicon_tags_2 },
    { 0xa0cd21dd, SUC_lexicon_tags_63 },
    { 0x5f6d21e0, SUC_lexicon_tags_11 },
    { 0xbdcd21e1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6d21e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42d21ed, SUC_lexicon_tags_15 },
    { 0x206d21ed, SUC_lexicon_tags_6 },
    { 0x500d21ef, SUC_lexicon_tags_2 },
    { 0x422d21ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x51ad21f2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8d21f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x922d21f8, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x158d21fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x482d21fd, SUC_lexicon_tags_17 },
    { 0x74ad21fe, SUC_lexicon_tags_57 },
    { 0xb6cd21fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x166d2201, SUC_lexicon_tags_1 },
    { 0xcd0d2202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb28d2204, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5dad2206, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x866d2209, SUC_lexicon_tags_45 },
    { 0x8f8d2209, SUC_lexicon_tags_17 },
    { 0x4dad220b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x452d220e, SUC_lexicon_tags_33 },
    { 0x336d220e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8d2214, SUC_lexicon_tags_28 },
    { 0x4f2d2215, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6d221c, SUC_lexicon_tags_34 },
    { 0xf4ed221d, SUC_lexicon_tags_17 },
    { 0xd46d221d, SUC_lexicon_tags_1 },
    { 0x746d221e, SUC_lexicon_tags_12 },
    { 0xbdad221d, SUC_lexicon_tags_11 },
    { 0xd10d2221, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x468d2223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728d2229, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cd222f, SUC_lexicon_tags_7 },
    { 0x79cd222f, SUC_lexicon_tags_2 },
    { 0x69ed222f, SUC_lexicon_tags_55 },
    { 0x8f4d222f, SUC_lexicon_tags_12 },
    { 0xc8cd2231, SUC_lexicon_tags_37 },
    { 0xc96d2234, SUC_lexicon_tags_27 },
    { 0x572d2235, SUC_lexicon_tags_26 },
    { 0xeb8d2231, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4d223b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ed2249, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x10ed224b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e4d224d, SUC_lexicon_tags_11 },
    { 0x4e4d224e, SUC_lexicon_tags_16 },
    { 0x6b2d224f, SUC_lexicon_tags_29 },
    { 0xdbcd224d, SUC_lexicon_tags_37 },
    { 0xa04d2250, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6d2253, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xca6d2255, SUC_lexicon_tags_1 },
    { 0x50ed2255, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2d2259, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfad225d, SUC_lexicon_tags_11 },
    { 0x2bad225d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80d2263, SUC_lexicon_tags_23 },
    { 0x44cd2263, SUC_lexicon_tags_12 },
    { 0xe8ad2265, SUC_lexicon_tags_30 },
    { 0xcacd2263, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82cd2269, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0d226f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x338d2271, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ded2273, SUC_lexicon_tags_23 },
    { 0xd8cd2273, SUC_lexicon_tags_7 },
    { 0x52d2275, SUC_lexicon_tags_17 },
    { 0xc82d2273, SUC_lexicon_tags_11 },
    { 0x574d2273, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccd227b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x98ad227d, SUC_lexicon_tags_22 },
    { 0xe76d227e, SUC_lexicon_tags_1 },
    { 0x676d227f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ecd2281, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4d2286, SUC_lexicon_tags_17 },
    { 0xf46d2287, SUC_lexicon_tags_33 },
    { 0xc54d2288, SUC_lexicon_tags_19 },
    { 0x744d2289, SUC_lexicon_tags_11 },
    { 0x37ad228a, SUC_lexicon_tags_65 },
    { 0xd08d228a, SUC_lexicon_tags_12 },
    { 0x428d228c, SUC_lexicon_tags_7 },
    { 0xeccd228d, SUC_lexicon_tags_14 },
    { 0x61ed228d, SUC_lexicon_tags_1 },
    { 0x764d2288, SUC_lexicon_tags_39 },
    { 0xa54d2290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ad2296, SUC_lexicon_tags_15 },
    { 0x77ad2296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66d229a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74d229d, SUC_lexicon_tags_49 },
    { 0x340d229d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440d22a3, SUC_lexicon_tags_7 },
    { 0x4dcd22a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cd22a8, SUC_lexicon_tags_25 },
    { 0xafcd22a9, SUC_lexicon_tags_71 },
    { 0xa04d22a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x770d22ac, SUC_lexicon_tags_11 },
    { 0x1fcd22ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0d22b2, SUC_lexicon_tags_12 },
    { 0xd0d22b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccd22b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97cd22b7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6c2d22b9, SUC_lexicon_tags_12 },
    { 0x8d8d22ba, SUC_lexicon_tags_14 },
    { 0xba6d22ba, SUC_lexicon_tags_17 },
    { 0xa50d22ba, SUC_lexicon_tags_2 },
    { 0x6fad22bb, SUC_lexicon_tags_46 },
    { 0x186d22bd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd46d22c0, SUC_lexicon_tags_38 },
    { 0xba0d22c1, SUC_lexicon_tags_47 },
    { 0xbaed22c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2d22c8, SUC_lexicon_tags_7 },
    { 0xbfcd22c9, SUC_lexicon_tags_19 },
    { 0x2c4d22ca, SUC_lexicon_tags_33 },
    { 0xb1ed22cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40cd22cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6cd22cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ed22d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4cd22da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7bad22dc, SUC_lexicon_tags_28 },
    { 0xef4d22dd, SUC_lexicon_tags_15 },
    { 0x4b2d22de, SUC_lexicon_tags_33 },
    { 0xd74d22dc, SUC_lexicon_tags_47 },
    { 0x1acd22e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd02d22e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbded22e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548d22f0, SUC_lexicon_tags_16 },
    { 0x9d6d22f1, SUC_lexicon_tags_1 },
    { 0x224d22f2, SUC_lexicon_tags_1 },
    { 0x568d22f2, SUC_lexicon_tags_25 },
    { 0x57ad22f1, SUC_lexicon_tags_38 },
    { 0x28d22f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xee0d22f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72cd22f9, SUC_lexicon_tags_30 },
    { 0xb1cd22fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0d22fe, SUC_lexicon_tags_17 },
    { 0xb62d22fe, SUC_lexicon_tags_7 },
    { 0x6e6d2300, SUC_lexicon_tags_2 },
    { 0x480d2301, SUC_lexicon_tags_17 },
    { 0x372d2302, SUC_lexicon_tags_29 },
    { 0xbbad2301, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cd2306, SUC_lexicon_tags_43 },
    { 0xbe0d2307, SUC_lexicon_tags_25 },
    { 0x44cd2308, SUC_lexicon_tags_17 },
    { 0x9c0d2309, SUC_lexicon_tags_38 },
    { 0x26cd230a, SUC_lexicon_tags_47 },
    { 0x4ed230b, SUC_lexicon_tags_68 },
    { 0xa74d230c, SUC_lexicon_tags_7 },
    { 0xf8d230b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x804d2310, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ad2314, SUC_lexicon_tags_12 },
    { 0x902d2315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51ed2317, SUC_lexicon_tags_55 },
    { 0xff6d2318, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0d231c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fad2320, SUC_lexicon_tags_68 },
    { 0xc5cd2321, SUC_lexicon_tags_25 },
    { 0x47ad2322, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32d2325, SUC_lexicon_tags_17 },
    { 0xb6ad2326, SUC_lexicon_tags_19 },
    { 0x780d2325, SUC_lexicon_tags_25 },
    { 0xba4d2325, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff8d232a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a0d232c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4d232f, SUC_lexicon_tags_32 },
    { 0xed0d232f, SUC_lexicon_tags_47 },
    { 0x684d232f, SUC_lexicon_tags_10 },
    { 0x302d2332, SUC_lexicon_tags_17 },
    { 0x8bed2333, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xd4ad2335, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6d233f, SUC_lexicon_tags_66 },
    { 0x282d2340, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ad2346, SUC_lexicon_tags_37 },
    { 0xc38d2346, SUC_lexicon_tags_26 },
    { 0x442d2348, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bed234d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6d2351, SUC_lexicon_tags_2 },
    { 0x6d8d2352, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d2358, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x642d235a, SUC_lexicon_tags_53 },
    { 0x806d235b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x224d235e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bed2360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106d2366, SUC_lexicon_tags_45 },
    { 0xa2cd2367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd74d2369, SUC_lexicon_tags_8 },
    { 0xc8d236a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2d236c, SUC_lexicon_tags_0 },
    { 0xe8ad236d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ed2370, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89ad2372, SUC_lexicon_tags_45 },
    { 0x50ed2373, SUC_lexicon_tags_12 },
    { 0xe2ed2374, SUC_lexicon_tags_11 },
    { 0x752d2375, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ad2377, SUC_lexicon_tags_56 },
    { 0xeb2d2377, SUC_lexicon_tags_17 },
    { 0x138d2379, SUC_lexicon_tags_51 },
    { 0x8c2d237a, SUC_lexicon_tags_37 },
    { 0x196d2379, SUC_lexicon_tags_30 },
    { 0x23ed237c, SUC_lexicon_tags_27 },
    { 0x31ed237d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3ed237f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaed2383, SUC_lexicon_tags_1 },
    { 0xc94d2384, SUC_lexicon_tags_17 },
    { 0x44cd2385, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd2388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aad238b, SUC_lexicon_tags_25 },
    { 0x95ed238c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cd2390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2d2393, SUC_lexicon_tags_26 },
    { 0x8d4d2393, SUC_lexicon_tags_7 },
    { 0xd48d2393, SUC_lexicon_tags_12 },
    { 0x536d2396, SUC_lexicon_tags_27 },
    { 0x1cd2396, SUC_lexicon_tags_11 },
    { 0x5dad2398, SUC_lexicon_tags_15 },
    { 0xff6d2399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6ad239b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9cd239d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa74d239f, SUC_lexicon_tags_11 },
    { 0xe82d239f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x770d23a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe68d23a4, SUC_lexicon_tags_55 },
    { 0xdced23a5, SUC_lexicon_tags_25 },
    { 0xf8ad23a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6d23ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62d23af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x696d23b2, SUC_lexicon_tags_30 },
    { 0x400d23b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfc8d23b5, SUC_lexicon_tags_47 },
    { 0x61ad23b6, SUC_lexicon_tags_9 },
    { 0x596d23b6, SUC_lexicon_tags_198 },
    { 0x96d23b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ad23bb, SUC_lexicon_tags_17 },
    { 0x186d23bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10d23bf, SUC_lexicon_tags_28 },
    { 0x6fcd23c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd23c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838d23c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0d23ce, SUC_lexicon_tags_43 },
    { 0x52d23cf, SUC_lexicon_tags_14 },
    { 0x392d23d0, SUC_lexicon_tags_7 },
    { 0xcfcd23d0, SUC_lexicon_tags_0 },
    { 0x642d23cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ad23d5, SUC_lexicon_tags_45 },
    { 0xd74d23d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d23db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x860d23dd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666d23e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6d23ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6acd23ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ad23f1, SUC_lexicon_tags_43 },
    { 0x752d23f2, SUC_lexicon_tags_37 },
    { 0x110d23f3, SUC_lexicon_tags_15 },
    { 0xd60d23f3, SUC_lexicon_tags_25 },
    { 0xd18d23f2, SUC_lexicon_tags_11 },
    { 0x6f0d23f3, SUC_lexicon_tags_15 },
    { 0x984d23f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6d23fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cd2401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78ad2403, SUC_lexicon_tags_15 },
    { 0x836d2404, SUC_lexicon_tags_116 },
    { 0x2eed2405, SUC_lexicon_tags_17 },
    { 0xc6cd2406, SUC_lexicon_tags_55 },
    { 0x9fcd2407, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756d240d, SUC_lexicon_tags_87 },
    { 0xcc4d240e, SUC_lexicon_tags_44 },
    { 0xa14d240d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ccd2411, SUC_lexicon_tags_28 },
    { 0xc7ad2412, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x918d2415, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ced2417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ead2419, SUC_lexicon_tags_12 },
    { 0xd32d2419, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ed241e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x308d2420, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x520d2422, SUC_lexicon_tags_12 },
    { 0xceed2423, SUC_lexicon_tags_47 },
    { 0x806d2424, SUC_lexicon_tags_0 },
    { 0x290d2423, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ed2428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x322d242a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6d242e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590d2432, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed243a, SUC_lexicon_tags_25 },
    { 0x2d6d243b, SUC_lexicon_tags_31 },
    { 0x734d243b, SUC_lexicon_tags_37 },
    { 0x324d243d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacad243f, SUC_lexicon_tags_45 },
    { 0x840d243f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46d2443, SUC_lexicon_tags_37 },
    { 0x724d2444, SUC_lexicon_tags_1 },
    { 0x8e6d2445, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cd2448, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2d244f, SUC_lexicon_tags_11 },
    { 0xfdcd2450, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3cd2452, SUC_lexicon_tags_8 },
    { 0x996d2453, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308d2457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6d2459, SUC_lexicon_tags_10 },
    { 0xa28d245a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad2462, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968d246e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x402d2471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0d2474, SUC_lexicon_tags_21 },
    { 0x556d2475, SUC_lexicon_tags_17 },
    { 0xb24d2476, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d247a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcd247f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aed2483, SUC_lexicon_tags_37 },
    { 0xc42d2484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58d2487, SUC_lexicon_tags_45 },
    { 0x44cd2487, SUC_lexicon_tags_15 },
    { 0x14d2487, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dad248f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8d2493, SUC_lexicon_tags_11 },
    { 0xe50d2493, SUC_lexicon_tags_19 },
    { 0xb82d2495, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x98ad2497, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x250d2499, SUC_lexicon_tags_57 },
    { 0x39ad249a, SUC_lexicon_tags_23 },
    { 0x330d249a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6ad249d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcc6d249f, SUC_lexicon_tags_37 },
    { 0xf8ed249f, SUC_lexicon_tags_37 },
    { 0x17ad24a1, SUC_lexicon_tags_61 },
    { 0x71ed24a0, SUC_lexicon_tags_15 },
    { 0xc5ad24a3, SUC_lexicon_tags_37 },
    { 0x7c6d24a1, SUC_lexicon_tags_38 },
    { 0xe24d24a5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcb6d24a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70ad24a9, SUC_lexicon_tags_1 },
    { 0x39cd24aa, SUC_lexicon_tags_15 },
    { 0x7ecd24aa, SUC_lexicon_tags_12 },
    { 0xf98d24ac, SUC_lexicon_tags_25 },
    { 0xf7cd24ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bed24b1, SUC_lexicon_tags_136 },
    { 0xf1cd24b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ad24b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cad24b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78d24bd, SUC_lexicon_tags_17 },
    { 0x2f0d24be, SUC_lexicon_tags_21 },
    { 0x5aad24bf, SUC_lexicon_tags_43 },
    { 0x692d24c0, SUC_lexicon_tags_79 },
    { 0xaeed24c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fcd24c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbded24c8, SUC_lexicon_tags_37 },
    { 0xb00d24c8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38d24cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5f2d24cf, SUC_lexicon_tags_35 },
    { 0x360d24d0, SUC_lexicon_tags_1 },
    { 0xabed24cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4d24d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5c6d24d6, SUC_lexicon_tags_25 },
    { 0xd68d24d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x226d24da, SUC_lexicon_tags_14 },
    { 0x82d24db, SUC_lexicon_tags_7 },
    { 0xd86d24db, SUC_lexicon_tags_7 },
    { 0xe84d24da, SUC_lexicon_tags_15 },
    { 0xce4d24de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9ad24e0, SUC_lexicon_tags_19 },
    { 0xad2d24e0, SUC_lexicon_tags_77 },
    { 0xee0d24e1, SUC_lexicon_tags_65 },
    { 0x26ad24e2, SUC_lexicon_tags_11 },
    { 0x9e0d24e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad24e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83ad24e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620d24ed, SUC_lexicon_tags_15 },
    { 0xe1cd24ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cd24f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2d24f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x516d24f6, SUC_lexicon_tags_47 },
    { 0xe4ad24f7, SUC_lexicon_tags_43 },
    { 0xda0d24f7, SUC_lexicon_tags_14 },
    { 0xcbad24f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4d24fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4d2500, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ed2505, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ad250c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8d2510, SUC_lexicon_tags_17 },
    { 0xdcad2510, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cd2517, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ded251a, SUC_lexicon_tags_23 },
    { 0x840d251a, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0d251e, SUC_lexicon_tags_79 },
    { 0x59ad251f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ad2523, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcead2526, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282d252a, SUC_lexicon_tags_17 },
    { 0xa1cd252b, SUC_lexicon_tags_47 },
    { 0x7aed252c, SUC_lexicon_tags_15 },
    { 0x860d252a, SUC_lexicon_tags_63 },
    { 0x872d252e, SUC_lexicon_tags_11 },
    { 0x12ad252f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ed2532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfad253c, SUC_lexicon_tags_12 },
    { 0xb6cd253d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306d2541, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188d2545, SUC_lexicon_tags_14 },
    { 0x2ad2545, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x958d2548, SUC_lexicon_tags_18 },
    { 0x68ed2548, SUC_lexicon_tags_47 },
    { 0x8b4d254a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4d254d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe26d254f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6b2d2551, SUC_lexicon_tags_7 },
    { 0xcd2552, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460d2557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ad255a, SUC_lexicon_tags_61 },
    { 0x5d2d255b, SUC_lexicon_tags_12 },
    { 0x964d255c, SUC_lexicon_tags_25 },
    { 0xc52d255d, SUC_lexicon_tags_11 },
    { 0x73cd255d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ad2564, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x420d2566, SUC_lexicon_tags_11 },
    { 0x626d2566, SUC_lexicon_tags_37 },
    { 0xb18d2566, SUC_lexicon_tags_15 },
    { 0xacad2569, SUC_lexicon_tags_15 },
    { 0xb42d256a, SUC_lexicon_tags_11 },
    { 0xb60d256a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ded256e, SUC_lexicon_tags_60 },
    { 0xecad256f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ad2572, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cd2576, SUC_lexicon_tags_46 },
    { 0x4a2d2577, SUC_lexicon_tags_19 },
    { 0xd36d2578, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa06d257a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aad257e, SUC_lexicon_tags_17 },
    { 0x2f4d257e, SUC_lexicon_tags_7 },
    { 0x636d2580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52ad2582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad2587, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34d258e, SUC_lexicon_tags_31 },
    { 0x82ad258e, SUC_lexicon_tags_18 },
    { 0x7d2d258e, SUC_lexicon_tags_31 },
    { 0xd3ed2591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x760d2593, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ad2596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d4d2598, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x734d259a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ad259e, SUC_lexicon_tags_38 },
    { 0xc9ed259e, SUC_lexicon_tags_49 },
    { 0xb1ed25a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ed25a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1c6d25a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14ed25a9, SUC_lexicon_tags_38 },
    { 0xafcd25aa, SUC_lexicon_tags_1 },
    { 0x736d25ab, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d25b0, SUC_lexicon_tags_15 },
    { 0xac0d25b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc10d25b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0d25b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42ed25b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcd25bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x44cd25be, SUC_lexicon_tags_47 },
    { 0xf2cd25be, SUC_lexicon_tags_19 },
    { 0xebad25be, SUC_lexicon_tags_46 },
    { 0x46ad25be, SUC_lexicon_tags_47 },
    { 0x81ad25c0, SUC_lexicon_tags_25 },
    { 0xfe4d25c0, SUC_lexicon_tags_7 },
    { 0x29cd25c4, SUC_lexicon_tags_27 },
    { 0x340d25c5, SUC_lexicon_tags_29 },
    { 0x384d25c5, SUC_lexicon_tags_1 },
    { 0xcaad25c5, SUC_lexicon_tags_7 },
    { 0xcd6d25c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8d25cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcd25d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682d25db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2d25e0, SUC_lexicon_tags_12 },
    { 0xb94d25e1, SUC_lexicon_tags_15 },
    { 0x80cd25e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa2d25e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ad25e7, SUC_lexicon_tags_26 },
    { 0xc86d25e7, SUC_lexicon_tags_2 },
    { 0xe34d25e9, SUC_lexicon_tags_27 },
    { 0x11ed25ea, SUC_lexicon_tags_87 },
    { 0x6b6d25eb, SUC_lexicon_tags_47 },
    { 0xa82d25ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f6d25ee, SUC_lexicon_tags_7 },
    { 0x6aad25ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x998d25f2, SUC_lexicon_tags_25 },
    { 0xa66d25f3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ad25f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cd25fa, SUC_lexicon_tags_37 },
    { 0x8d4d25fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcd25fe, SUC_lexicon_tags_46 },
    { 0x5f4d25ff, SUC_lexicon_tags_6 },
    { 0x902d25ff, SUC_lexicon_tags_17 },
    { 0xe6ed25ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0d2604, SUC_lexicon_tags_37 },
    { 0x824d2604, SUC_lexicon_tags_56 },
    { 0xd2605, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x158d2609, SUC_lexicon_tags_25 },
    { 0x374d260a, SUC_lexicon_tags_31 },
    { 0xd18d260a, SUC_lexicon_tags_68 },
    { 0x970d260c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4d2610, SUC_lexicon_tags_37 },
    { 0x8f4d2611, SUC_lexicon_tags_13 },
    { 0xc40d2612, SUC_lexicon_tags_55 },
    { 0xc56d2613, SUC_lexicon_tags_17 },
    { 0x4e2d2610, SUC_lexicon_tags_15 },
    { 0xd96d2615, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896d261a, SUC_lexicon_tags_11 },
    { 0xe6cd261b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8d2620, SUC_lexicon_tags_25 },
    { 0x2d8d2620, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cd2626, SUC_lexicon_tags_21 },
    { 0x62ad2627, SUC_lexicon_tags_21 },
    { 0x6a0d2628, SUC_lexicon_tags_47 },
    { 0xa8ed2626, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xaacd262b, SUC_lexicon_tags_1 },
    { 0x1a6d262c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54d2632, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ad2636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ad263b, SUC_lexicon_tags_10 },
    { 0xbb4d263b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4eed263e, SUC_lexicon_tags_12 },
    { 0xb6d263e, SUC_lexicon_tags_29 },
    { 0xf2d2640, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2d2645, SUC_lexicon_tags_63 },
    { 0xf0ad2646, SUC_lexicon_tags_63 },
    { 0x34d2646, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccd264c, SUC_lexicon_tags_82 },
    { 0x320d264d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcb6d264f, SUC_lexicon_tags_40 },
    { 0xd96d264f, SUC_lexicon_tags_27 },
    { 0x64ed2651, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0d2654, SUC_lexicon_tags_12 },
    { 0xa12d2655, SUC_lexicon_tags_32 },
    { 0x50d2656, SUC_lexicon_tags_17 },
    { 0xd52d2654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87ad2659, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182d265d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x504d2660, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf5ed2662, SUC_lexicon_tags_37 },
    { 0x2bcd2663, SUC_lexicon_tags_11 },
    { 0xf08d2664, SUC_lexicon_tags_1 },
    { 0x53ed2664, SUC_lexicon_tags_234 },
    { 0xd0ed2663, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x718d2668, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xad2d266a, SUC_lexicon_tags_11 },
    { 0x68cd266b, SUC_lexicon_tags_15 },
    { 0xe76d266a, SUC_lexicon_tags_17 },
    { 0xc02d266b, SUC_lexicon_tags_37 },
    { 0x28ed266a, SUC_lexicon_tags_28 },
    { 0xc4d266f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x216d2671, SUC_lexicon_tags_49 },
    { 0xa6ad2672, SUC_lexicon_tags_11 },
    { 0xd46d2672, SUC_lexicon_tags_61 },
    { 0x976d2673, SUC_lexicon_tags_29 },
    { 0x1d0d2675, SUC_lexicon_tags_11 },
    { 0xa6ed2675, SUC_lexicon_tags_47 },
    { 0x36ad2675, SUC_lexicon_tags_2 },
    { 0x8ed2676, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4d2680, SUC_lexicon_tags_37 },
    { 0x660d2681, SUC_lexicon_tags_11 },
    { 0x318d2681, SUC_lexicon_tags_11 },
    { 0xc94d2683, SUC_lexicon_tags_11 },
    { 0x560d2683, SUC_lexicon_tags_31 },
    { 0xd1ed2685, SUC_lexicon_tags_37 },
    { 0x3aed2686, SUC_lexicon_tags_29 },
    { 0xf7ad2687, SUC_lexicon_tags_29 },
    { 0xa76d2688, SUC_lexicon_tags_1 },
    { 0xedcd2688, SUC_lexicon_tags_68 },
    { 0x5fed2688, SUC_lexicon_tags_11 },
    { 0x6c0d2689, SUC_lexicon_tags_15 },
    { 0x58ad268c, SUC_lexicon_tags_15 },
    { 0x976d2683, SUC_lexicon_tags_15 },
    { 0xd14d2684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2d2695, SUC_lexicon_tags_27 },
    { 0xbfed2696, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842d269b, SUC_lexicon_tags_84 },
    { 0xea0d269c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x12cd269f, SUC_lexicon_tags_11 },
    { 0x13ad269f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc40d26a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d26a7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ad26aa, SUC_lexicon_tags_30 },
    { 0x9c2d26ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7e8d26ad, SUC_lexicon_tags_34 },
    { 0xf0cd26ae, SUC_lexicon_tags_1 },
    { 0xdf4d26af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68d26b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x43ed26b6, SUC_lexicon_tags_56 },
    { 0x272d26b6, SUC_lexicon_tags_31 },
    { 0xec2d26b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270d26bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d26c5, SUC_lexicon_tags_17 },
    { 0xb1cd26c6, SUC_lexicon_tags_15 },
    { 0xaeed26c6, SUC_lexicon_tags_12 },
    { 0xdecd26c7, SUC_lexicon_tags_19 },
    { 0x1a0d26c9, SUC_lexicon_tags_60 },
    { 0xcc6d26ca, SUC_lexicon_tags_23 },
    { 0xb7cd26cb, SUC_lexicon_tags_27 },
    { 0x868d26c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2d26cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0d26d2, SUC_lexicon_tags_14 },
    { 0x6d4d26d2, SUC_lexicon_tags_31 },
    { 0xc7ad26d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8d26da, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ed26e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340d26e8, SUC_lexicon_tags_19 },
    { 0x4eed26e9, SUC_lexicon_tags_15 },
    { 0x8a8d26ea, SUC_lexicon_tags_2 },
    { 0x35ed26eb, SUC_lexicon_tags_33 },
    { 0x24d26ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84d26ef, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596d26f3, SUC_lexicon_tags_148 },
    { 0xb16d26f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cd26fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48cd26ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0d2705, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf06d2707, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48ed2709, SUC_lexicon_tags_15 },
    { 0x902d270a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d270e, SUC_lexicon_tags_19 },
    { 0xf64d270f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69ad2711, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xdd0d2713, SUC_lexicon_tags_35 },
    { 0xbfad2714, SUC_lexicon_tags_17 },
    { 0x7fed2714, SUC_lexicon_tags_2 },
    { 0x7b2d2716, SUC_lexicon_tags_9 },
    { 0x554d2717, SUC_lexicon_tags_6 },
    { 0x40ad2716, SUC_lexicon_tags_160 },
    { 0x1f0d2713, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d2721, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x856d2723, SUC_lexicon_tags_19 },
    { 0xd44d2724, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ad2729, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5ad272b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d272e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cd2735, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0x946d2738, SUC_lexicon_tags_47 },
    { 0xddcd2739, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02d273e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6d2741, SUC_lexicon_tags_37 },
    { 0x906d2741, SUC_lexicon_tags_25 },
    { 0x9f6d2743, SUC_lexicon_tags_15 },
    { 0xd5ad2744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d2747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ed274e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf10d2750, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8d2755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0ed2757, SUC_lexicon_tags_17 },
    { 0x526d2758, SUC_lexicon_tags_8 },
    { 0x46d2759, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d4d275b, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60d275f, SUC_lexicon_tags_25 },
    { 0x710d2760, SUC_lexicon_tags_15 },
    { 0xed8d2760, SUC_lexicon_tags_15 },
    { 0x85ed2761, SUC_lexicon_tags_138 },
    { 0xa98d2763, SUC_lexicon_tags_37 },
    { 0xa04d2764, SUC_lexicon_tags_17 },
    { 0xd1ad2765, SUC_lexicon_tags_9 },
    { 0x1ed2765, SUC_lexicon_tags_68 },
    { 0xa14d2760, SUC_lexicon_tags_30 },
    { 0x782d2765, SUC_lexicon_tags_7 },
    { 0xc32d2769, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190d2770, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ad2774, SUC_lexicon_tags_69 },
    { 0xc6d2775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ed2778, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32cd277a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x734d277c, SUC_lexicon_tags_19 },
    { 0x9ad277d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136d2781, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40d2785, SUC_lexicon_tags_12 },
    { 0xf90d2785, SUC_lexicon_tags_47 },
    { 0xbd8d2787, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaf2d2789, SUC_lexicon_tags_19 },
    { 0xebad2789, SUC_lexicon_tags_25 },
    { 0xeccd278b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9f2d278d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb0d278f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0d2793, SUC_lexicon_tags_865 },
    { 0xb60d2793, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d2799, SUC_lexicon_tags_7 },
    { 0xe3ad2799, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42d27a0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x25ed27a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50d27a6, SUC_lexicon_tags_37 },
    { 0xdd8d27a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0d27aa, SUC_lexicon_tags_7 },
    { 0x3d0d27ab, SUC_lexicon_tags_38 },
    { 0x890d27ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x38d27ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878d27b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d0d27b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910d27c0, SUC_lexicon_tags_254 },
    { 0x11cd27c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb18d27c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6d27c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8d27ca, SUC_lexicon_tags_130 },
    { 0xd34d27cb, SUC_lexicon_tags_26 },
    { 0xe5cd27cc, SUC_lexicon_tags_15 },
    { 0x9ead27cb, SUC_lexicon_tags_8 },
    { 0x784d27cd, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd27d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cd27da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x398d27dc, SUC_lexicon_tags_2 },
    { 0xcc2d27dd, SUC_lexicon_tags_25 },
    { 0x7ad27de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x758d27e1, SUC_lexicon_tags_26 },
    { 0xbad27e2, SUC_lexicon_tags_55 },
    { 0xe36d27e1, SUC_lexicon_tags_37 },
    { 0x8b2d27e3, SUC_lexicon_tags_25 },
    { 0x6b2d27e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcd27e8, SUC_lexicon_tags_2 },
    { 0x73ed27e9, SUC_lexicon_tags_25 },
    { 0xdb2d27e9, SUC_lexicon_tags_1 },
    { 0x40d27eb, SUC_lexicon_tags_15 },
    { 0xccd27ec, SUC_lexicon_tags_48 },
    { 0xd5cd27ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850d27f3, SUC_lexicon_tags_0 },
    { 0x286d27f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104d27f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x400d27fa, SUC_lexicon_tags_15 },
    { 0xba6d27fb, SUC_lexicon_tags_37 },
    { 0xdded27fc, SUC_lexicon_tags_60 },
    { 0x644d27fb, SUC_lexicon_tags_11 },
    { 0x3ccd27fb, SUC_lexicon_tags_27 },
    { 0x9b2d27ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x778d2802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32d2808, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x5dcd280a, SUC_lexicon_tags_2 },
    { 0x7c2d280a, SUC_lexicon_tags_87 },
    { 0xc8ad280a, SUC_lexicon_tags_27 },
    { 0xf9ad280b, SUC_lexicon_tags_68 },
    { 0xa9cd280e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x318d2810, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9b6d2812, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x198d2814, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4d2817, SUC_lexicon_tags_15 },
    { 0xe88d2818, SUC_lexicon_tags_12 },
    { 0xdc6d2818, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58d281e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238d2823, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36d2827, SUC_lexicon_tags_37 },
    { 0x7b8d2827, SUC_lexicon_tags_108 },
    { 0x252d2828, SUC_lexicon_tags_37 },
    { 0xa0ed282a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbed282f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4d2834, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ed2837, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa28d2839, SUC_lexicon_tags_2 },
    { 0xf4ad2839, SUC_lexicon_tags_14 },
    { 0x4e8d283a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfe2d283d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8d2842, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986d2846, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2d284b, SUC_lexicon_tags_43 },
    { 0x49ed284b, SUC_lexicon_tags_95 },
    { 0x20d284d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa0d284f, SUC_lexicon_tags_49 },
    { 0x4f4d2850, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2d2853, SUC_lexicon_tags_7 },
    { 0x128d2854, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988d285c, SUC_lexicon_tags_1 },
    { 0x572d285d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402d2861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2aed2863, SUC_lexicon_tags_27 },
    { 0x428d2863, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204d286a, SUC_lexicon_tags_7 },
    { 0xb44d286b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x620d286d, SUC_lexicon_tags_43 },
    { 0xcced286e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768d2875, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed2878, SUC_lexicon_tags_0 },
    { 0x2b0d2878, SUC_lexicon_tags_63 },
    { 0xacd287a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x550d287c, SUC_lexicon_tags_37 },
    { 0xb9ad287c, SUC_lexicon_tags_29 },
    { 0x4ead287d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf8cd2880, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf26d2882, SUC_lexicon_tags_25 },
    { 0x100d2883, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xacad2885, SUC_lexicon_tags_2 },
    { 0x2bcd2886, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1cad2888, SUC_lexicon_tags_34 },
    { 0x930d2889, SUC_lexicon_tags_37 },
    { 0x9a0d288a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x36cd288c, SUC_lexicon_tags_29 },
    { 0x8d0d288d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4d2896, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcd2899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x454d289c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ed28a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cd28aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeccd28ac, SUC_lexicon_tags_25 },
    { 0xf90d28ac, SUC_lexicon_tags_2 },
    { 0xeb6d28ad, SUC_lexicon_tags_1553 },
    { 0, NULL },
    { 0x6d8d28b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9ed28b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2d28b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632d28bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ad28c1, SUC_lexicon_tags_10 },
    { 0xffcd28c2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd0cd28c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x922d28c7, SUC_lexicon_tags_25 },
    { 0x602d28c8, SUC_lexicon_tags_43 },
    { 0xc52d28c9, SUC_lexicon_tags_25 },
    { 0x68d28ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6d28d6, SUC_lexicon_tags_15 },
    { 0xc82d28d7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeed28d9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cd28de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x146d28e1, SUC_lexicon_tags_12 },
    { 0x596d28e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x410d28e4, SUC_lexicon_tags_445 },
    { 0x980d28e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342d28ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x202d28ec, SUC_lexicon_tags_12 },
    { 0x664d28ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6d8d28ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb42d28f1, SUC_lexicon_tags_43 },
    { 0x616d28f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2d28f7, SUC_lexicon_tags_25 },
    { 0x584d28f7, SUC_lexicon_tags_15 },
    { 0x23cd28f9, SUC_lexicon_tags_30 },
    { 0x4b4d28f9, SUC_lexicon_tags_37 },
    { 0x3dcd28f8, SUC_lexicon_tags_1 },
    { 0x70cd28fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d4d28fe, SUC_lexicon_tags_27 },
    { 0xf36d28ff, SUC_lexicon_tags_0 },
    { 0x724d2900, SUC_lexicon_tags_8 },
    { 0x4b8d2901, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x572d2904, SUC_lexicon_tags_8 },
    { 0xcf4d2905, SUC_lexicon_tags_27 },
    { 0xeaad2906, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664d290a, SUC_lexicon_tags_28 },
    { 0xcc6d290b, SUC_lexicon_tags_25 },
    { 0xcf8d290b, SUC_lexicon_tags_11 },
    { 0xba6d290a, SUC_lexicon_tags_15 },
    { 0x8cd290c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5fad2910, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aad2917, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x660d2919, SUC_lexicon_tags_37 },
    { 0x9cd291a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x110d291d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ced2924, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3ed2926, SUC_lexicon_tags_0 },
    { 0x71cd2926, SUC_lexicon_tags_1 },
    { 0x2ecd2928, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cd292d, SUC_lexicon_tags_39 },
    { 0x496d292e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x868d2931, SUC_lexicon_tags_2 },
    { 0xed2d2932, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d2939, SUC_lexicon_tags_37 },
    { 0x29ad2939, SUC_lexicon_tags_2 },
    { 0x4a8d293b, SUC_lexicon_tags_1 },
    { 0xbd0d293c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cd2948, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x128d294a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9c6d294c, SUC_lexicon_tags_37 },
    { 0xa10d294d, SUC_lexicon_tags_14 },
    { 0xa00d294c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd36d2950, SUC_lexicon_tags_19 },
    { 0xac6d2950, SUC_lexicon_tags_10 },
    { 0x81ed2952, SUC_lexicon_tags_1 },
    { 0xf7cd2952, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d2956, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6d2963, SUC_lexicon_tags_2 },
    { 0x4f8d2964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34d2967, SUC_lexicon_tags_43 },
    { 0x76ad2968, SUC_lexicon_tags_1 },
    { 0x660d2968, SUC_lexicon_tags_17 },
    { 0x9ead2967, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa9ed296c, SUC_lexicon_tags_47 },
    { 0x288d296c, SUC_lexicon_tags_41 },
    { 0xdd8d296c, SUC_lexicon_tags_0 },
    { 0x718d296d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcd2972, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe50d2974, SUC_lexicon_tags_151 },
    { 0x22ed2975, SUC_lexicon_tags_61 },
    { 0x158d2975, SUC_lexicon_tags_25 },
    { 0xe38d2977, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x58d2979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148d2981, SUC_lexicon_tags_33 },
    { 0xf24d2981, SUC_lexicon_tags_12 },
    { 0xca4d2983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ed2986, SUC_lexicon_tags_37 },
    { 0x468d2987, SUC_lexicon_tags_25 },
    { 0x368d2988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95cd298a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92d298d, SUC_lexicon_tags_25 },
    { 0x73ed298e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94d2993, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc32d2995, SUC_lexicon_tags_25 },
    { 0xe8ed2995, SUC_lexicon_tags_12 },
    { 0x516d2997, SUC_lexicon_tags_37 },
    { 0xa7ed2998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x202d299b, SUC_lexicon_tags_37 },
    { 0x722d299c, SUC_lexicon_tags_37 },
    { 0xc62d299d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aed29a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e6d29a2, SUC_lexicon_tags_15 },
    { 0x3b2d29a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4d29a6, SUC_lexicon_tags_33 },
    { 0x26ed29a7, SUC_lexicon_tags_27 },
    { 0x2f0d29a7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8d29ad, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb0cd29af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8d29bd, SUC_lexicon_tags_19 },
    { 0x994d29be, SUC_lexicon_tags_172 },
    { 0x212d29be, SUC_lexicon_tags_7 },
    { 0xeed29bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2d29c4, SUC_lexicon_tags_7 },
    { 0x2f2d29c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa24d29c7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488d29cb, SUC_lexicon_tags_65 },
    { 0xf76d29cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a0d29ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x19cd29d0, SUC_lexicon_tags_33 },
    { 0x98cd29d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20ad29d3, SUC_lexicon_tags_60 },
    { 0xaecd29d4, SUC_lexicon_tags_1 },
    { 0xebcd29d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6d29d8, SUC_lexicon_tags_11 },
    { 0x8a8d29d9, SUC_lexicon_tags_1 },
    { 0x656d29d9, SUC_lexicon_tags_46 },
    { 0x9b8d29d9, SUC_lexicon_tags_11 },
    { 0x92cd29d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ed29e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d29e7, SUC_lexicon_tags_63 },
    { 0xd29e8, SUC_lexicon_tags_107 },
    { 0x1bad29e9, SUC_lexicon_tags_48 },
    { 0xc2ad29e8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724d29f2, SUC_lexicon_tags_7 },
    { 0x688d29f2, SUC_lexicon_tags_26 },
    { 0xcfed29f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ded29f7, SUC_lexicon_tags_25 },
    { 0xf0ad29f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa58d29fa, SUC_lexicon_tags_17 },
    { 0xf50d29fb, SUC_lexicon_tags_6 },
    { 0x434d29fa, SUC_lexicon_tags_2 },
    { 0x3b8d29fb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ed2a00, SUC_lexicon_tags_37 },
    { 0xd6cd2a01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcf6d2a03, SUC_lexicon_tags_23 },
    { 0xb74d2a03, SUC_lexicon_tags_1 },
    { 0x212d2a05, SUC_lexicon_tags_11 },
    { 0x87cd2a06, SUC_lexicon_tags_49 },
    { 0x324d2a07, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d2a0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30d2a0e, SUC_lexicon_tags_25 },
    { 0x5b0d2a0f, SUC_lexicon_tags_25 },
    { 0x30ed2a0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x712d2a12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d2a16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516d2a1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x374d2a1c, SUC_lexicon_tags_37 },
    { 0xaa8d2a1d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204d2a21, SUC_lexicon_tags_7 },
    { 0x7ead2a21, SUC_lexicon_tags_87 },
    { 0xd66d2a22, SUC_lexicon_tags_11 },
    { 0xe80d2a22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542d2a31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2d2a36, SUC_lexicon_tags_12 },
    { 0x666d2a36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4d2a3b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ad2a3e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb3ed2a40, SUC_lexicon_tags_5 },
    { 0x512d2a41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x790d2a43, SUC_lexicon_tags_151 },
    { 0xd14d2a44, SUC_lexicon_tags_25 },
    { 0xe5ad2a45, SUC_lexicon_tags_15 },
    { 0x42ed2a46, SUC_lexicon_tags_35 },
    { 0x9acd2a44, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4b2d2a49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832d2a4d, SUC_lexicon_tags_37 },
    { 0x942d2a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa70d2a50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf10d2a52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2d2a5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ed2a5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2d2a62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x436d2a64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0d2a68, SUC_lexicon_tags_500 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d2a6b, SUC_lexicon_tags_12 },
    { 0x88ed2a6b, SUC_lexicon_tags_11 },
    { 0x572d2a6c, SUC_lexicon_tags_17 },
    { 0x458d2a6e, SUC_lexicon_tags_11 },
    { 0xc6ad2a6e, SUC_lexicon_tags_47 },
    { 0x268d2a70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e4d2a72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54d2a76, SUC_lexicon_tags_1 },
    { 0x36cd2a76, SUC_lexicon_tags_37 },
    { 0xba2d2a76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75cd2a7b, SUC_lexicon_tags_16 },
    { 0x9ad2a7b, SUC_lexicon_tags_38 },
    { 0x78ad2a7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc4d2a7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8cd2a81, SUC_lexicon_tags_83 },
    { 0xa38d2a82, SUC_lexicon_tags_47 },
    { 0x748d2a83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x260d2a85, SUC_lexicon_tags_25 },
    { 0x988d2a85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44d2a8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x886d2a8d, SUC_lexicon_tags_12 },
    { 0xced2a8e, SUC_lexicon_tags_107 },
    { 0x13ed2a8f, SUC_lexicon_tags_9 },
    { 0xe12d2a90, SUC_lexicon_tags_17 },
    { 0xa8ad2a91, SUC_lexicon_tags_14 },
    { 0x292d2a90, SUC_lexicon_tags_15 },
    { 0xdad2a91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fed2a97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ad2a9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccd2aa0, SUC_lexicon_tags_19 },
    { 0xf36d2aa1, SUC_lexicon_tags_31 },
    { 0xfe6d2aa1, SUC_lexicon_tags_38 },
    { 0xa54d2aa1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa80d2aa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58d2aad, SUC_lexicon_tags_35 },
    { 0xc48d2aae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2ab1, SUC_lexicon_tags_47 },
    { 0x828d2ab1, SUC_lexicon_tags_60 },
    { 0xbd8d2ab2, SUC_lexicon_tags_2 },
    { 0xf2ed2ab3, SUC_lexicon_tags_14 },
    { 0xe60d2ab5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eed2ab9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x408d2abc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14d2ac1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f4d2ac3, SUC_lexicon_tags_29 },
    { 0x8e6d2ac4, SUC_lexicon_tags_6 },
    { 0xaa4d2ac5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc5ed2ac7, SUC_lexicon_tags_42 },
    { 0x9ecd2ac8, SUC_lexicon_tags_14 },
    { 0xe48d2ac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d2acc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf2d2ace, SUC_lexicon_tags_37 },
    { 0x86ed2acf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fed2ad8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ad2adb, SUC_lexicon_tags_1 },
    { 0x63ad2adc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986d2ae0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbed2ae5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fad2aeb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234d2aef, SUC_lexicon_tags_33 },
    { 0x5c8d2af0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9f2d2af2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc72d2af4, SUC_lexicon_tags_39 },
    { 0x47ad2af5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ad2b00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4d2b07, SUC_lexicon_tags_10 },
    { 0x7fad2b08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x212d2b0b, SUC_lexicon_tags_45 },
    { 0xc82d2b0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ad2b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94cd2b13, SUC_lexicon_tags_33 },
    { 0x1bad2b14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea6d2b16, SUC_lexicon_tags_12 },
    { 0x142d2b16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf8d2b19, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x692d2b1c, SUC_lexicon_tags_1 },
    { 0xe10d2b1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50cd2b20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94cd2b22, SUC_lexicon_tags_17 },
    { 0xb0ed2b23, SUC_lexicon_tags_63 },
    { 0xde6d2b22, SUC_lexicon_tags_37 },
    { 0x92ad2b25, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34d2b28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7bed2b2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f4d2b2c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172d2b30, SUC_lexicon_tags_17 },
    { 0x602d2b31, SUC_lexicon_tags_45 },
    { 0xb6d2b31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0d2b37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x852d2b39, SUC_lexicon_tags_25 },
    { 0xa0ed2b39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cd2b3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc30d2b3f, SUC_lexicon_tags_65 },
    { 0xbe0d2b40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4d2b43, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcd2b48, SUC_lexicon_tags_7 },
    { 0xeead2b49, SUC_lexicon_tags_17 },
    { 0xec6d2b4a, SUC_lexicon_tags_11 },
    { 0x184d2b4a, SUC_lexicon_tags_4 },
    { 0xeed2b48, SUC_lexicon_tags_11 },
    { 0xd02d2b49, SUC_lexicon_tags_14 },
    { 0xd64d2b49, SUC_lexicon_tags_29 },
    { 0x5c6d2b4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836d2b55, SUC_lexicon_tags_33 },
    { 0xde8d2b56, SUC_lexicon_tags_80 },
    { 0x626d2b57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4d2b61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ad2b6a, SUC_lexicon_tags_14 },
    { 0xa3cd2b6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ed2b6e, SUC_lexicon_tags_47 },
    { 0xeed2b6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x380d2b72, SUC_lexicon_tags_27 },
    { 0xaeed2b73, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xec8d2b75, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6d2b78, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8b2d2b7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x81cd2b7c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cd2b82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9ad2b84, SUC_lexicon_tags_53 },
    { 0x42ad2b84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0d2b8a, SUC_lexicon_tags_33 },
    { 0x436d2b8b, SUC_lexicon_tags_17 },
    { 0x55ed2b8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b6d2b8e, SUC_lexicon_tags_14 },
    { 0xf10d2b8f, SUC_lexicon_tags_17 },
    { 0x816d2b90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5a0d2b92, SUC_lexicon_tags_12 },
    { 0xf38d2b93, SUC_lexicon_tags_29 },
    { 0x238d2b92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x304d2b96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb9ed2b98, SUC_lexicon_tags_15 },
    { 0xeead2b98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86d2b9d, SUC_lexicon_tags_14 },
    { 0x80d2b9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0d2ba1, SUC_lexicon_tags_82 },
    { 0xa1ad2ba2, SUC_lexicon_tags_11 },
    { 0x7b8d2ba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe38d2ba5, SUC_lexicon_tags_12 },
    { 0xc82d2ba6, SUC_lexicon_tags_11 },
    { 0xc30d2ba7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d0d2ba9, SUC_lexicon_tags_11 },
    { 0x4bad2baa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d2baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b0d2bb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2d2bb9, SUC_lexicon_tags_25 },
    { 0xd92d2bba, SUC_lexicon_tags_17 },
    { 0x3b8d2bbb, SUC_lexicon_tags_46 },
    { 0x79cd2bbc, SUC_lexicon_tags_2 },
    { 0xe8ad2bb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe10d2bc0, SUC_lexicon_tags_63 },
    { 0xe3ed2bc0, SUC_lexicon_tags_47 },
    { 0xb3cd2bc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ad2bcb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d8d2bcd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2d2bd0, SUC_lexicon_tags_6 },
    { 0x120d2bd0, SUC_lexicon_tags_16 },
    { 0x4e4d2bd2, SUC_lexicon_tags_2 },
    { 0xaced2bd1, SUC_lexicon_tags_12 },
    { 0x118d2bd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ed2bd8, SUC_lexicon_tags_1 },
    { 0x13ad2bd8, SUC_lexicon_tags_25 },
    { 0x6ad2bd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8d2be0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902d2be4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfed2be7, SUC_lexicon_tags_30 },
    { 0xa94d2be8, SUC_lexicon_tags_33 },
    { 0x87ed2be8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x234d2beb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x384d2bed, SUC_lexicon_tags_46 },
    { 0x2e2d2bee, SUC_lexicon_tags_4 },
    { 0xfbcd2bed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcd2bf2, SUC_lexicon_tags_47 },
    { 0x362d2bf2, SUC_lexicon_tags_14 },
    { 0xa90d2bf4, SUC_lexicon_tags_46 },
    { 0x1c6d2bf5, SUC_lexicon_tags_92 },
    { 0xe1cd2bf5, SUC_lexicon_tags_1 },
    { 0x7b6d2bf4, SUC_lexicon_tags_84 },
    { 0x640d2bf7, SUC_lexicon_tags_46 },
    { 0x590d2bf9, SUC_lexicon_tags_30 },
    { 0x1f8d2bf9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00d2bfd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x514d2bff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46ad2c01, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ed2c05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x884d2c07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ed2c0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00d2c14, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x47ed2c16, SUC_lexicon_tags_63 },
    { 0x598d2c17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cd2c1c, SUC_lexicon_tags_17 },
    { 0x5ced2c1d, SUC_lexicon_tags_26 },
    { 0xd2cd2c1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54d2c21, SUC_lexicon_tags_2 },
    { 0xcecd2c22, SUC_lexicon_tags_37 },
    { 0x2fed2c22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cd2c27, SUC_lexicon_tags_4 },
    { 0x414d2c28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14d2c2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ad2c30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x830d2c33, SUC_lexicon_tags_4 },
    { 0x178d2c34, SUC_lexicon_tags_47 },
    { 0xa70d2c33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x93cd2c37, SUC_lexicon_tags_8 },
    { 0xa48d2c38, SUC_lexicon_tags_43 },
    { 0x738d2c39, SUC_lexicon_tags_55 },
    { 0x590d2c3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8ed2c3c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d2c3f, SUC_lexicon_tags_8 },
    { 0xebcd2c40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24d2c47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4d2c51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4d2c56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ad2c5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6cd2c5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526d2c60, SUC_lexicon_tags_10 },
    { 0xbd0d2c60, SUC_lexicon_tags_337 },
    { 0, NULL },
    { 0x4d0d2c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x308d2c65, SUC_lexicon_tags_1 },
    { 0x82d2c66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ced2c68, SUC_lexicon_tags_37 },
    { 0x520d2c68, SUC_lexicon_tags_0 },
    { 0x362d2c6a, SUC_lexicon_tags_37 },
    { 0xa22d2c6b, SUC_lexicon_tags_7 },
    { 0xe38d2c6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca8d2c6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208d2c72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d2c75, SUC_lexicon_tags_33 },
    { 0x9b0d2c76, SUC_lexicon_tags_27 },
    { 0x9ead2c77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbed2c7a, SUC_lexicon_tags_11 },
    { 0x848d2c7a, SUC_lexicon_tags_37 },
    { 0x79ed2c7c, SUC_lexicon_tags_37 },
    { 0x830d2c7a, SUC_lexicon_tags_16 },
    { 0x57cd2c7e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb08d2c80, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28d2c85, SUC_lexicon_tags_47 },
    { 0x436d2c86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1cd2c88, SUC_lexicon_tags_25 },
    { 0x1b8d2c89, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xb8ad2c8b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ed2c92, SUC_lexicon_tags_63 },
    { 0xfd6d2c92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31ed2c95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33ad2c97, SUC_lexicon_tags_87 },
    { 0x326d2c98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e8d2c9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f2d2c9c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ed2ca1, SUC_lexicon_tags_11 },
    { 0x3ded2ca2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ead2ca4, SUC_lexicon_tags_15 },
    { 0x554d2ca5, SUC_lexicon_tags_17 },
    { 0xbdcd2ca6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xda0d2ca8, SUC_lexicon_tags_14 },
    { 0x51ad2ca8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cd2cb5, SUC_lexicon_tags_10 },
    { 0x574d2cb6, SUC_lexicon_tags_39 },
    { 0x3c2d2cb7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324d2cc2, SUC_lexicon_tags_32 },
    { 0x2ccd2cc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ad2cd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe6d2cd2, SUC_lexicon_tags_15 },
    { 0x4e6d2cd3, SUC_lexicon_tags_11 },
    { 0x972d2cd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ad2cd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36d2cdf, SUC_lexicon_tags_47 },
    { 0xf1ad2ce0, SUC_lexicon_tags_7 },
    { 0xa96d2ce0, SUC_lexicon_tags_24 },
    { 0xaf2d2ce1, SUC_lexicon_tags_26 },
    { 0xce4d2ce3, SUC_lexicon_tags_1 },
    { 0xda2d2ce3, SUC_lexicon_tags_45 },
    { 0x902d2ce0, SUC_lexicon_tags_47 },
    { 0xf52d2ce4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bed2cec, SUC_lexicon_tags_25 },
    { 0x74cd2ced, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cd2cf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f2d2cf5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0d2cf8, SUC_lexicon_tags_21 },
    { 0x442d2cf9, SUC_lexicon_tags_27 },
    { 0x26ad2cf8, SUC_lexicon_tags_33 },
    { 0x324d2cfb, SUC_lexicon_tags_785 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0d2d01, SUC_lexicon_tags_14 },
    { 0xddad2d02, SUC_lexicon_tags_60 },
    { 0xe84d2d02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6c4d2d05, SUC_lexicon_tags_17 },
    { 0x792d2d05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2d2d0f, SUC_lexicon_tags_37 },
    { 0xfc0d2d10, SUC_lexicon_tags_29 },
    { 0xc98d2d11, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xcb8d2d13, SUC_lexicon_tags_11 },
    { 0x2dcd2d13, SUC_lexicon_tags_63 },
    { 0xbed2d15, SUC_lexicon_tags_43 },
    { 0x382d2d15, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x490d2d19, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad2d1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2d2d20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x840d2d22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ad2d25, SUC_lexicon_tags_37 },
    { 0x9f2d2d26, SUC_lexicon_tags_37 },
    { 0x69ed2d26, SUC_lexicon_tags_11 },
    { 0x38ad2d28, SUC_lexicon_tags_1 },
    { 0xa82d2d29, SUC_lexicon_tags_29 },
    { 0x772d2d2a, SUC_lexicon_tags_12 },
    { 0x5ecd2d2b, SUC_lexicon_tags_37 },
    { 0xe38d2d2b, SUC_lexicon_tags_2 },
    { 0xe66d2d29, SUC_lexicon_tags_27 },
    { 0x4d2d2d2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cd2d36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x454d2d38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84d2d3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x110d2d3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2d2d42, SUC_lexicon_tags_15 },
    { 0x676d2d42, SUC_lexicon_tags_27 },
    { 0x764d2d44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bed2d46, SUC_lexicon_tags_6 },
    { 0xc08d2d47, SUC_lexicon_tags_46 },
    { 0xa88d2d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ced2d4e, SUC_lexicon_tags_48 },
    { 0xa64d2d4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc0d2d51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d2d56, SUC_lexicon_tags_1 },
    { 0x290d2d57, SUC_lexicon_tags_30 },
    { 0x56ed2d56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8d2d5f, SUC_lexicon_tags_65 },
    { 0xdfcd2d60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618d2d64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafed2d6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a8d2d6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccd2d72, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ead2d74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930d2d7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ed2d80, SUC_lexicon_tags_55 },
    { 0x186d2d80, SUC_lexicon_tags_6 },
    { 0xefad2d80, SUC_lexicon_tags_47 },
    { 0x864d2d83, SUC_lexicon_tags_79 },
    { 0x15ad2d82, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xa50d2d86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x436d2d88, SUC_lexicon_tags_69 },
    { 0xb28d2d89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe48d2d8b, SUC_lexicon_tags_25 },
    { 0xd92d2d8b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fed2d90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf22d2d92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8d2d97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cd2d9b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8ded2d9d, SUC_lexicon_tags_45 },
    { 0x20d2d9d, SUC_lexicon_tags_11 },
    { 0x92cd2d9d, SUC_lexicon_tags_33 },
    { 0x346d2d9e, SUC_lexicon_tags_15 },
    { 0x598d2da1, SUC_lexicon_tags_27 },
    { 0xb26d2d9f, SUC_lexicon_tags_84 },
    { 0x286d2da1, SUC_lexicon_tags_25 },
    { 0xc30d2da4, SUC_lexicon_tags_11 },
    { 0x506d2d9e, SUC_lexicon_tags_0 },
    { 0x88cd2da6, SUC_lexicon_tags_2 },
    { 0x1f2d2da0, SUC_lexicon_tags_11 },
    { 0x724d2da8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x348d2daa, SUC_lexicon_tags_29 },
    { 0xb62d2daa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ed2dbb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20d2dbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xceed2dc1, SUC_lexicon_tags_37 },
    { 0x8c0d2dc2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xddcd2dc4, SUC_lexicon_tags_49 },
    { 0x5b8d2dc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0d2dc9, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d2dcf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xffad2dd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x368d2dd5, SUC_lexicon_tags_43 },
    { 0x39cd2dd6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4d2dd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad2ddc, SUC_lexicon_tags_31 },
    { 0x4b6d2ddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ed2de0, SUC_lexicon_tags_1 },
    { 0xc18d2de0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acd2de6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x36cd2de9, SUC_lexicon_tags_69 },
    { 0x7b6d2de9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6a6d2dec, SUC_lexicon_tags_9 },
    { 0x476d2ded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac0d2def, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1ad2df1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0d2df9, SUC_lexicon_tags_30 },
    { 0x6f0d2dfa, SUC_lexicon_tags_17 },
    { 0xbe6d2dfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x554d2dfd, SUC_lexicon_tags_2 },
    { 0x8a6d2dfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ad2e05, SUC_lexicon_tags_37 },
    { 0x96ed2e06, SUC_lexicon_tags_46 },
    { 0x8a8d2e07, SUC_lexicon_tags_35 },
    { 0xb50d2e08, SUC_lexicon_tags_37 },
    { 0xc4ad2e08, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ad2e0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88d2e0f, SUC_lexicon_tags_25 },
    { 0x258d2e10, SUC_lexicon_tags_38 },
    { 0xdf4d2e11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876d2e15, SUC_lexicon_tags_11 },
    { 0x6ead2e16, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x3c8d2e18, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9cad2e1a, SUC_lexicon_tags_1 },
    { 0x85ed2e1b, SUC_lexicon_tags_33 },
    { 0x30cd2e1a, SUC_lexicon_tags_27 },
    { 0xefcd2e1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdead2e20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcd2e24, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290d2e2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbcd2e2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ed2e33, SUC_lexicon_tags_12 },
    { 0xeeed2e34, SUC_lexicon_tags_25 },
    { 0xc7ed2e33, SUC_lexicon_tags_12 },
    { 0xfad2e34, SUC_lexicon_tags_43 },
    { 0x9c2d2e37, SUC_lexicon_tags_9 },
    { 0xc8d2e34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d2e41, SUC_lexicon_tags_439 },
    { 0x454d2e42, SUC_lexicon_tags_17 },
    { 0xe72d2e42, SUC_lexicon_tags_11 },
    { 0x978d2e43, SUC_lexicon_tags_26 },
    { 0x9b8d2e45, SUC_lexicon_tags_14 },
    { 0xa6ed2e41, SUC_lexicon_tags_65 },
    { 0xa4ed2e47, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26d2e4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed2e5c, SUC_lexicon_tags_2 },
    { 0x352d2e5d, SUC_lexicon_tags_527 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d2e60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe1ed2e62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ad2e65, SUC_lexicon_tags_19 },
    { 0x762d2e66, SUC_lexicon_tags_0 },
    { 0xa62d2e67, SUC_lexicon_tags_2 },
    { 0xf16d2e68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334d2e6c, SUC_lexicon_tags_12 },
    { 0x98cd2e6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ad2e72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504d2e76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcd2e7a, SUC_lexicon_tags_11 },
    { 0x64ad2e7b, SUC_lexicon_tags_27 },
    { 0x3eed2e7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342d2e82, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0d2e87, SUC_lexicon_tags_7 },
    { 0x3f0d2e88, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3c6d2e8a, SUC_lexicon_tags_46 },
    { 0xa30d2e8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0d2e8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aed2e92, SUC_lexicon_tags_11 },
    { 0xd68d2e93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d2e9a, SUC_lexicon_tags_11 },
    { 0xcecd2e9b, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0x198d2e9e, SUC_lexicon_tags_17 },
    { 0x7aed2e9f, SUC_lexicon_tags_11 },
    { 0x25cd2ea0, SUC_lexicon_tags_4 },
    { 0x8d2ea0, SUC_lexicon_tags_25 },
    { 0x66ed2ea0, SUC_lexicon_tags_2 },
    { 0x8bed2ea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8cd2ea5, SUC_lexicon_tags_8 },
    { 0x664d2ea5, SUC_lexicon_tags_31 },
    { 0x24cd2ea7, SUC_lexicon_tags_25 },
    { 0xb62d2ea8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x828d2eaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56d2ead, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8d2eb3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x398d2eb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0d2ebb, SUC_lexicon_tags_25 },
    { 0x7f4d2ebc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32d2ebf, SUC_lexicon_tags_15 },
    { 0x19cd2ec0, SUC_lexicon_tags_12 },
    { 0x9aed2ec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eed2ec8, SUC_lexicon_tags_15 },
    { 0xc94d2ec9, SUC_lexicon_tags_14 },
    { 0x3aed2eca, SUC_lexicon_tags_15 },
    { 0x86cd2ec8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x770d2ecd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x508d2ecf, SUC_lexicon_tags_11 },
    { 0xac0d2ed0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12d2ed7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578d2ede, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9aed2ee0, SUC_lexicon_tags_15 },
    { 0xc32d2ee1, SUC_lexicon_tags_31 },
    { 0x214d2ee2, SUC_lexicon_tags_8 },
    { 0x468d2ee1, SUC_lexicon_tags_12 },
    { 0xda0d2ee4, SUC_lexicon_tags_82 },
    { 0x290d2ee5, SUC_lexicon_tags_0 },
    { 0xf2d2ee4, SUC_lexicon_tags_2 },
    { 0xf3ad2ee7, SUC_lexicon_tags_37 },
    { 0x6a0d2ee7, SUC_lexicon_tags_31 },
    { 0x9d2d2ee3, SUC_lexicon_tags_7 },
    { 0xfaad2ee3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4c0d2eec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf0ad2eee, SUC_lexicon_tags_15 },
    { 0x172d2eef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd2ef2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ed2ef6, SUC_lexicon_tags_12 },
    { 0x2aed2ef7, SUC_lexicon_tags_17 },
    { 0xffad2ef8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d2efb, SUC_lexicon_tags_15 },
    { 0xa32d2efc, SUC_lexicon_tags_12 },
    { 0x71ad2efd, SUC_lexicon_tags_4 },
    { 0x9bcd2efb, SUC_lexicon_tags_6 },
    { 0xfe2d2eff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa30d2f01, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x50cd2f03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd58d2f05, SUC_lexicon_tags_46 },
    { 0x7f0d2f06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8d2f09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62d2f0d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd2f14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x27ad2f16, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6d2f1b, SUC_lexicon_tags_11 },
    { 0x7cd2f1c, SUC_lexicon_tags_15 },
    { 0xb18d2f1d, SUC_lexicon_tags_68 },
    { 0x4ad2f1d, SUC_lexicon_tags_0 },
    { 0x360d2f1f, SUC_lexicon_tags_1 },
    { 0x43cd2f1f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880d2f24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644d2f28, SUC_lexicon_tags_2 },
    { 0x51ad2f29, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0d2f2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x702d2f2e, SUC_lexicon_tags_43 },
    { 0xc54d2f2e, SUC_lexicon_tags_29 },
    { 0xbecd2f30, SUC_lexicon_tags_7 },
    { 0x778d2f31, SUC_lexicon_tags_17 },
    { 0x400d2f32, SUC_lexicon_tags_2 },
    { 0x65ad2f33, SUC_lexicon_tags_37 },
    { 0xdd4d2f30, SUC_lexicon_tags_55 },
    { 0x87ed2f35, SUC_lexicon_tags_2 },
    { 0xe14d2f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2d2f39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8e2d2f3b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14d2f3f, SUC_lexicon_tags_15 },
    { 0xde8d2f40, SUC_lexicon_tags_8 },
    { 0x8a2d2f40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14d2f48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa8d2f4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36d2f4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912d2f56, SUC_lexicon_tags_11 },
    { 0xce8d2f57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x906d2f59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366d2f5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0d2f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4d2f6a, SUC_lexicon_tags_49 },
    { 0x2ad2f6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382d2f74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cd2f79, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcd2f7c, SUC_lexicon_tags_22 },
    { 0x402d2f7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0cd2f7f, SUC_lexicon_tags_6 },
    { 0xb28d2f7f, SUC_lexicon_tags_37 },
    { 0x606d2f81, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd2f85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74ad2f87, SUC_lexicon_tags_60 },
    { 0xfbed2f88, SUC_lexicon_tags_57 },
    { 0x428d2f89, SUC_lexicon_tags_14 },
    { 0xa66d2f8a, SUC_lexicon_tags_14 },
    { 0x1b6d2f8a, SUC_lexicon_tags_15 },
    { 0xb76d2f8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x678d2f8f, SUC_lexicon_tags_47 },
    { 0x662d2f90, SUC_lexicon_tags_15 },
    { 0x9eed2f91, SUC_lexicon_tags_27 },
    { 0x4a6d2f8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ad2f99, SUC_lexicon_tags_17 },
    { 0x1ed2f99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff8d2f9c, SUC_lexicon_tags_108 },
    { 0xcded2f9d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34d2fa6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18d2fab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ad2fb1, SUC_lexicon_tags_32 },
    { 0x526d2fb2, SUC_lexicon_tags_11 },
    { 0x9f6d2fb3, SUC_lexicon_tags_46 },
    { 0x9ad2fb4, SUC_lexicon_tags_2 },
    { 0xab4d2fb5, SUC_lexicon_tags_637 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756d2fbd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x73cd2fbf, SUC_lexicon_tags_12 },
    { 0xd76d2fc0, SUC_lexicon_tags_7 },
    { 0x84ad2fc1, SUC_lexicon_tags_11 },
    { 0x510d2fc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcead2fc9, SUC_lexicon_tags_15 },
    { 0xe14d2fca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44d2fcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56d2fd2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0ad2fd4, SUC_lexicon_tags_38 },
    { 0x5a4d2fd5, SUC_lexicon_tags_37 },
    { 0xf98d2fd6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x29cd2fd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x894d2fdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8d2fdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cd2fe7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34d2fea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612d2fee, SUC_lexicon_tags_11 },
    { 0xa60d2fee, SUC_lexicon_tags_27 },
    { 0xc9ed2fee, SUC_lexicon_tags_17 },
    { 0xea4d2ff1, SUC_lexicon_tags_22 },
    { 0x52d2ff1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d2ff8, SUC_lexicon_tags_4 },
    { 0x252d2ff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cd2fff, SUC_lexicon_tags_25 },
    { 0x9ced3000, SUC_lexicon_tags_87 },
    { 0xedad3000, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06d3005, SUC_lexicon_tags_25 },
    { 0x476d3005, SUC_lexicon_tags_2 },
    { 0xd2ad3007, SUC_lexicon_tags_11 },
    { 0x98cd3007, SUC_lexicon_tags_27 },
    { 0x9fed3009, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xa94d300b, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2d3010, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6d3013, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ad3016, SUC_lexicon_tags_15 },
    { 0xd8ad3017, SUC_lexicon_tags_11 },
    { 0x232d3017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6d301f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8d3023, SUC_lexicon_tags_25 },
    { 0x2b8d3024, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x906d3027, SUC_lexicon_tags_63 },
    { 0x3cd3028, SUC_lexicon_tags_45 },
    { 0xc7cd3028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x422d302b, SUC_lexicon_tags_11 },
    { 0x4ecd302c, SUC_lexicon_tags_1 },
    { 0x922d302d, SUC_lexicon_tags_12 },
    { 0xb9cd302e, SUC_lexicon_tags_65 },
    { 0x458d302c, SUC_lexicon_tags_47 },
    { 0x34ed3030, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x922d3032, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x906d3034, SUC_lexicon_tags_25 },
    { 0xe12d3035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x626d3038, SUC_lexicon_tags_8 },
    { 0xf0ad3038, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x432d303c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafed303e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d2d3040, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94d3044, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0d3049, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ed304d, SUC_lexicon_tags_375 },
    { 0x8d6d304e, SUC_lexicon_tags_12 },
    { 0xec4d304f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed3053, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76d3057, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x7a4d3059, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264d3061, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ed3064, SUC_lexicon_tags_0 },
    { 0xd9ed3065, SUC_lexicon_tags_29 },
    { 0xf5ad3064, SUC_lexicon_tags_4 },
    { 0x77cd3067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd46d3069, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8d306c, SUC_lexicon_tags_15 },
    { 0xc6cd306d, SUC_lexicon_tags_87 },
    { 0x12d306e, SUC_lexicon_tags_64 },
    { 0xb76d306f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35ed3071, SUC_lexicon_tags_17 },
    { 0x8e0d3072, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2d3075, SUC_lexicon_tags_10 },
    { 0x21ed3076, SUC_lexicon_tags_55 },
    { 0x548d3077, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38d307e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ad3081, SUC_lexicon_tags_7 },
    { 0xbc0d3082, SUC_lexicon_tags_19 },
    { 0xa86d3082, SUC_lexicon_tags_6 },
    { 0x326d3084, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ed3087, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa24d3089, SUC_lexicon_tags_26 },
    { 0x28ad3089, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xea0d308c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628d3091, SUC_lexicon_tags_0 },
    { 0x374d3091, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeeed3094, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67cd3096, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe4d3098, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14d30a2, SUC_lexicon_tags_72 },
    { 0xa4ad30a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4d30a9, SUC_lexicon_tags_15 },
    { 0xe60d30a9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d30ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0ad30af, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cd30b3, SUC_lexicon_tags_55 },
    { 0xc00d30b4, SUC_lexicon_tags_29 },
    { 0x308d30b5, SUC_lexicon_tags_25 },
    { 0x404d30b6, SUC_lexicon_tags_12 },
    { 0x46d30b7, SUC_lexicon_tags_14 },
    { 0xe70d30b4, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772d30bc, SUC_lexicon_tags_49 },
    { 0x140d30bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed30c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ad30c8, SUC_lexicon_tags_25 },
    { 0x3bcd30c8, SUC_lexicon_tags_12 },
    { 0x9cd30ca, SUC_lexicon_tags_26 },
    { 0x3c2d30cb, SUC_lexicon_tags_14 },
    { 0x6a2d30cc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6d30cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788d30d3, SUC_lexicon_tags_7 },
    { 0xfead30d4, SUC_lexicon_tags_46 },
    { 0xc8ed30d5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x11ad30d7, SUC_lexicon_tags_109 },
    { 0x68d30d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ad30db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd18d30dd, SUC_lexicon_tags_19 },
    { 0x3c2d30de, SUC_lexicon_tags_8 },
    { 0x812d30df, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x388d30e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8d30e4, SUC_lexicon_tags_17 },
    { 0x70ed30e5, SUC_lexicon_tags_33 },
    { 0x542d30e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0d30ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaed30f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x516d30f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ad30f7, SUC_lexicon_tags_26 },
    { 0x61ad30f7, SUC_lexicon_tags_68 },
    { 0xe26d30f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5ed30fb, SUC_lexicon_tags_15 },
    { 0xdecd30fc, SUC_lexicon_tags_9 },
    { 0xd88d30fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6d3102, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb4d3104, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364d3108, SUC_lexicon_tags_17 },
    { 0x3bed3109, SUC_lexicon_tags_1 },
    { 0x620d3108, SUC_lexicon_tags_1 },
    { 0xacad3108, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d3110, SUC_lexicon_tags_9 },
    { 0x120d3111, SUC_lexicon_tags_66 },
    { 0x78ed3112, SUC_lexicon_tags_27 },
    { 0xd16d3112, SUC_lexicon_tags_27 },
    { 0xe02d3112, SUC_lexicon_tags_33 },
    { 0x4a2d3112, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4cd3117, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ad3120, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ed3124, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x298d3126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cd312b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcad3130, SUC_lexicon_tags_23 },
    { 0xd26d3130, SUC_lexicon_tags_25 },
    { 0x43cd3132, SUC_lexicon_tags_37 },
    { 0xcf2d3131, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd4d3135, SUC_lexicon_tags_64 },
    { 0x572d3136, SUC_lexicon_tags_0 },
    { 0xbf4d3135, SUC_lexicon_tags_29 },
    { 0x4c6d3137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x592d313b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cd313e, SUC_lexicon_tags_15 },
    { 0x5a0d313f, SUC_lexicon_tags_17 },
    { 0x80ed3140, SUC_lexicon_tags_33 },
    { 0x28ed3140, SUC_lexicon_tags_33 },
    { 0xa7ad3140, SUC_lexicon_tags_38 },
    { 0x1c8d3143, SUC_lexicon_tags_2 },
    { 0xf6cd3144, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfb4d3146, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66d314e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36d3154, SUC_lexicon_tags_17 },
    { 0xf98d3154, SUC_lexicon_tags_16 },
    { 0x562d3156, SUC_lexicon_tags_0 },
    { 0x836d3157, SUC_lexicon_tags_60 },
    { 0xf02d3157, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaad315b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeead315f, SUC_lexicon_tags_12 },
    { 0xee6d315f, SUC_lexicon_tags_1 },
    { 0xc0ad3161, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ad3164, SUC_lexicon_tags_218 },
    { 0x6c8d3164, SUC_lexicon_tags_27 },
    { 0x99cd3166, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ed3169, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x440d316b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf2ad316d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed3170, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4e0d3172, SUC_lexicon_tags_40 },
    { 0x2ced3173, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x868d3175, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0d3178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bad317c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f2d317e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xee4d3180, SUC_lexicon_tags_29 },
    { 0xdaad3181, SUC_lexicon_tags_49 },
    { 0x42cd3181, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x996d3185, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4d318a, SUC_lexicon_tags_17 },
    { 0xf7cd318b, SUC_lexicon_tags_29 },
    { 0xc64d318a, SUC_lexicon_tags_11 },
    { 0x864d318d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa74d318f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x66ed3191, SUC_lexicon_tags_12 },
    { 0xd3ad3191, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922d3196, SUC_lexicon_tags_16 },
    { 0x2aed3197, SUC_lexicon_tags_2 },
    { 0xc06d3197, SUC_lexicon_tags_15 },
    { 0x9bad3199, SUC_lexicon_tags_33 },
    { 0xdead3196, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cd319e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398d31a2, SUC_lexicon_tags_37 },
    { 0x46cd31a2, SUC_lexicon_tags_19 },
    { 0xb26d31a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aed31ab, SUC_lexicon_tags_17 },
    { 0x914d31ac, SUC_lexicon_tags_2 },
    { 0x35ad31ad, SUC_lexicon_tags_30 },
    { 0xcbcd31ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2d31b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabed31b8, SUC_lexicon_tags_39 },
    { 0xaaed31b9, SUC_lexicon_tags_364 },
    { 0xd36d31b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f2d31bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xffad31bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acd31c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2d31cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ed31d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ad31d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ed31d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2d31db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0d31e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e6d31e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64d31ed, SUC_lexicon_tags_27 },
    { 0xe2cd31ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64d31f4, SUC_lexicon_tags_17 },
    { 0x4a2d31f5, SUC_lexicon_tags_68 },
    { 0x714d31f4, SUC_lexicon_tags_35 },
    { 0x868d31f5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaead31f9, SUC_lexicon_tags_42 },
    { 0x688d31fa, SUC_lexicon_tags_12 },
    { 0xa82d31fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4d31fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6d3202, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x228d3204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4d3207, SUC_lexicon_tags_23 },
    { 0x824d3208, SUC_lexicon_tags_47 },
    { 0x85ad3207, SUC_lexicon_tags_7 },
    { 0x52d320a, SUC_lexicon_tags_11 },
    { 0xfdcd3208, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90d3217, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2d4d3219, SUC_lexicon_tags_46 },
    { 0x47ad321a, SUC_lexicon_tags_9 },
    { 0xef6d321a, SUC_lexicon_tags_19 },
    { 0x84ad321b, SUC_lexicon_tags_49 },
    { 0x180d321d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb20d321f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x93ed3221, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x48ed3223, SUC_lexicon_tags_2 },
    { 0xf38d3224, SUC_lexicon_tags_21 },
    { 0x7c8d3224, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242d322b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474d3230, SUC_lexicon_tags_15 },
    { 0xaacd3231, SUC_lexicon_tags_38 },
    { 0x8e4d3232, SUC_lexicon_tags_1 },
    { 0x95ed3233, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108d3237, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x906d323a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298d323e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434d3242, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb12d3244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d3247, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfa0d3249, SUC_lexicon_tags_51 },
    { 0xb36d324a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcd324d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16d3250, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd3253, SUC_lexicon_tags_37 },
    { 0x8ded3254, SUC_lexicon_tags_85 },
    { 0x45ad3255, SUC_lexicon_tags_25 },
    { 0xa24d3256, SUC_lexicon_tags_37 },
    { 0x3aed3257, SUC_lexicon_tags_29 },
    { 0x7ecd3256, SUC_lexicon_tags_37 },
    { 0xc96d3258, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6d325c, SUC_lexicon_tags_25 },
    { 0x7ad325d, SUC_lexicon_tags_26 },
    { 0x294d325d, SUC_lexicon_tags_15 },
    { 0x8fcd325d, SUC_lexicon_tags_15 },
    { 0xfe4d325e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104d3266, SUC_lexicon_tags_27 },
    { 0x408d3267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8ed3269, SUC_lexicon_tags_62 },
    { 0x13cd3269, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50d326e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832d3273, SUC_lexicon_tags_17 },
    { 0xc18d3274, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676d3279, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594d327e, SUC_lexicon_tags_68 },
    { 0xe6cd327f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x374d3281, SUC_lexicon_tags_30 },
    { 0x24d3281, SUC_lexicon_tags_1167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0d3287, SUC_lexicon_tags_33 },
    { 0xee6d3287, SUC_lexicon_tags_7 },
    { 0x3f6d3287, SUC_lexicon_tags_10 },
    { 0xedad328a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb90d328c, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccd328f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4d3293, SUC_lexicon_tags_1 },
    { 0x5ded3294, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54d3299, SUC_lexicon_tags_47 },
    { 0x672d329a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x53ad329c, SUC_lexicon_tags_33 },
    { 0x35ed329d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8ad329f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc38d32a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa08d32a3, SUC_lexicon_tags_43 },
    { 0x3ed32a3, SUC_lexicon_tags_7 },
    { 0x8f6d32a5, SUC_lexicon_tags_8 },
    { 0xcd6d32a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55cd32a8, SUC_lexicon_tags_66 },
    { 0xbb4d32a9, SUC_lexicon_tags_60 },
    { 0x562d32aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cd32bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54d32c3, SUC_lexicon_tags_11 },
    { 0x61ed32c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2d32ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ed32d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x900d32d6, SUC_lexicon_tags_31 },
    { 0x2eed32d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d32e5, SUC_lexicon_tags_17 },
    { 0x670d32e6, SUC_lexicon_tags_60 },
    { 0xc8ad32e6, SUC_lexicon_tags_17 },
    { 0x6bed32e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x528d32eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e6d32ed, SUC_lexicon_tags_85 },
    { 0xd6ad32ee, SUC_lexicon_tags_16 },
    { 0xdccd32ef, SUC_lexicon_tags_15 },
    { 0x882d32ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532d32f5, SUC_lexicon_tags_37 },
    { 0x4b8d32f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cd32fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1cd32ff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb7cd3301, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44d3303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2d3308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x326d330b, SUC_lexicon_tags_11 },
    { 0x58ad330c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cad3311, SUC_lexicon_tags_12 },
    { 0x490d3312, SUC_lexicon_tags_25 },
    { 0xb50d3313, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x580d3315, SUC_lexicon_tags_12 },
    { 0x1a0d3316, SUC_lexicon_tags_7 },
    { 0x4c2d3315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x792d3319, SUC_lexicon_tags_10 },
    { 0x992d331a, SUC_lexicon_tags_11 },
    { 0xa4ad331a, SUC_lexicon_tags_39 },
    { 0x6f0d331c, SUC_lexicon_tags_170 },
    { 0xc9ad331d, SUC_lexicon_tags_12 },
    { 0x7f4d331e, SUC_lexicon_tags_25 },
    { 0xd18d3319, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ed3324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x866d3327, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110d332c, SUC_lexicon_tags_17 },
    { 0x3ced332d, SUC_lexicon_tags_21 },
    { 0x83cd332d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15ad3330, SUC_lexicon_tags_30 },
    { 0x8d6d3331, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccd333d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2d3340, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46d3344, SUC_lexicon_tags_11 },
    { 0x31cd3345, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb7ad3347, SUC_lexicon_tags_38 },
    { 0x1c0d3347, SUC_lexicon_tags_7 },
    { 0xe32d3349, SUC_lexicon_tags_33 },
    { 0xc30d334a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ad334d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64d3351, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08d3355, SUC_lexicon_tags_17 },
    { 0x1bcd3356, SUC_lexicon_tags_5 },
    { 0xcacd3355, SUC_lexicon_tags_2 },
    { 0x8c2d3357, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0d335d, SUC_lexicon_tags_1 },
    { 0x506d335e, SUC_lexicon_tags_60 },
    { 0xd50d335d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecd3365, SUC_lexicon_tags_49 },
    { 0x3d4d3365, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272d336a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x386d336c, SUC_lexicon_tags_32 },
    { 0xcb6d336c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0d3371, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xce2d3374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06d3379, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d3384, SUC_lexicon_tags_26 },
    { 0x356d3384, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x506d3388, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460d3392, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b6d3394, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x702d3396, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960d339b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0d339e, SUC_lexicon_tags_0 },
    { 0x3cd339e, SUC_lexicon_tags_17 },
    { 0x982d33a0, SUC_lexicon_tags_33 },
    { 0xe74d33a0, SUC_lexicon_tags_15 },
    { 0xbe6d33a2, SUC_lexicon_tags_11 },
    { 0x168d33a3, SUC_lexicon_tags_29 },
    { 0x634d33a3, SUC_lexicon_tags_14 },
    { 0xa8d33a5, SUC_lexicon_tags_129 },
    { 0x950d339e, SUC_lexicon_tags_47 },
    { 0xb60d33a4, SUC_lexicon_tags_26 },
    { 0x130d33a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e8d33aa, SUC_lexicon_tags_15 },
    { 0xbe2d33ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8d33af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ed33b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8d33b9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x996d33bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0d33be, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x75ad33c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfed33c4, SUC_lexicon_tags_37 },
    { 0xd40d33c5, SUC_lexicon_tags_32 },
    { 0x82cd33c4, SUC_lexicon_tags_275 },
    { 0x92d33c7, SUC_lexicon_tags_29 },
    { 0x2fcd33c8, SUC_lexicon_tags_113 },
    { 0x288d33c9, SUC_lexicon_tags_12 },
    { 0x7b6d33ca, SUC_lexicon_tags_15 },
    { 0x2bcd33c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4d33ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ad33d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a0d33d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xded33d6, SUC_lexicon_tags_2 },
    { 0xba0d33d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cd33dc, SUC_lexicon_tags_31 },
    { 0x1b2d33dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486d33e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6d33e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790d33ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ed33f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x696d33f2, SUC_lexicon_tags_11 },
    { 0xd74d33f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdad33f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e2d33f7, SUC_lexicon_tags_37 },
    { 0xad0d33f8, SUC_lexicon_tags_65 },
    { 0x522d33f9, SUC_lexicon_tags_25 },
    { 0xbfad33f8, SUC_lexicon_tags_37 },
    { 0xaecd33f8, SUC_lexicon_tags_37 },
    { 0x602d33fc, SUC_lexicon_tags_30 },
    { 0x346d33f7, SUC_lexicon_tags_39 },
    { 0x38ad33fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f6d3400, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xffad3402, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x73ed3404, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aed3408, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b6d340a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbed340d, SUC_lexicon_tags_1 },
    { 0xbced340e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92d3412, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x602d3414, SUC_lexicon_tags_2 },
    { 0xc3ed3414, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ad341a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ad341d, SUC_lexicon_tags_47 },
    { 0x4e0d341e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594d3423, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee8d3425, SUC_lexicon_tags_11 },
    { 0xd9ad3426, SUC_lexicon_tags_34 },
    { 0xdd2d3426, SUC_lexicon_tags_11 },
    { 0x730d3428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b4d342a, SUC_lexicon_tags_82 },
    { 0x21ad342a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd92d342d, SUC_lexicon_tags_17 },
    { 0x6e8d342e, SUC_lexicon_tags_11 },
    { 0xccad342f, SUC_lexicon_tags_25 },
    { 0x434d3430, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccd3435, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x508d3438, SUC_lexicon_tags_46 },
    { 0x22ad3439, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2d343d, SUC_lexicon_tags_19 },
    { 0x25cd343d, SUC_lexicon_tags_17 },
    { 0x4f6d343d, SUC_lexicon_tags_60 },
    { 0x8f8d3440, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ed3443, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xa2d3445, SUC_lexicon_tags_8 },
    { 0x16ed3445, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73cd3448, SUC_lexicon_tags_83 },
    { 0xe7ad3449, SUC_lexicon_tags_7 },
    { 0x45ed344a, SUC_lexicon_tags_12 },
    { 0x26d344b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd52d344d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x324d344f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cd3452, SUC_lexicon_tags_27 },
    { 0xbb2d3453, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36d3457, SUC_lexicon_tags_48 },
    { 0xd0ed3458, SUC_lexicon_tags_1 },
    { 0x6c6d3457, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dad345d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x966d345f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d2d3461, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84d3469, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d346e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x776d3471, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2ad3473, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ad347c, SUC_lexicon_tags_60 },
    { 0x36ad347d, SUC_lexicon_tags_11 },
    { 0x1d8d347d, SUC_lexicon_tags_12 },
    { 0x8aed347e, SUC_lexicon_tags_31 },
    { 0x73cd347d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32d3483, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6d3486, SUC_lexicon_tags_82 },
    { 0x342d3487, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4d348b, SUC_lexicon_tags_155 },
    { 0xcfed348c, SUC_lexicon_tags_47 },
    { 0xe26d348d, SUC_lexicon_tags_17 },
    { 0x330d348d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88d3491, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738d34a0, SUC_lexicon_tags_63 },
    { 0xddcd34a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892d34a6, SUC_lexicon_tags_10 },
    { 0x6ced34a6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xaa2d34a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa6d34ab, SUC_lexicon_tags_13 },
    { 0x268d34ac, SUC_lexicon_tags_37 },
    { 0xdb6d34ad, SUC_lexicon_tags_30 },
    { 0x54ed34ad, SUC_lexicon_tags_17 },
    { 0xbed34ad, SUC_lexicon_tags_4 },
    { 0x60ed34ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2d34b6, SUC_lexicon_tags_11 },
    { 0x164d34b6, SUC_lexicon_tags_33 },
    { 0x89ad34b6, SUC_lexicon_tags_26 },
    { 0x9d0d34b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfcad34bb, SUC_lexicon_tags_9 },
    { 0xb4d34bc, SUC_lexicon_tags_23 },
    { 0x99ed34bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ad34c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb8d34c2, SUC_lexicon_tags_0 },
    { 0x81cd34c3, SUC_lexicon_tags_14 },
    { 0xf28d34c3, SUC_lexicon_tags_24 },
    { 0x36cd34c2, SUC_lexicon_tags_29 },
    { 0x16ed34c6, SUC_lexicon_tags_7 },
    { 0x59ad34c6, SUC_lexicon_tags_23 },
    { 0x932d34c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60d34cb, SUC_lexicon_tags_2 },
    { 0xbdcd34cb, SUC_lexicon_tags_37 },
    { 0xbacd34cd, SUC_lexicon_tags_25 },
    { 0xf64d34cb, SUC_lexicon_tags_6 },
    { 0x4ad34cc, SUC_lexicon_tags_7 },
    { 0x732d34d0, SUC_lexicon_tags_0 },
    { 0xae4d34d0, SUC_lexicon_tags_26 },
    { 0x102d34d1, SUC_lexicon_tags_15 },
    { 0xb74d34ce, SUC_lexicon_tags_4 },
    { 0x8acd34d4, SUC_lexicon_tags_87 },
    { 0x2aad34cf, SUC_lexicon_tags_13 },
    { 0x906d34cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xffad34d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x588d34db, SUC_lexicon_tags_35 },
    { 0xccad34db, SUC_lexicon_tags_15 },
    { 0x6ead34db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d34df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe84d34e1, SUC_lexicon_tags_47 },
    { 0xb06d34e2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ed34ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x274d34f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0d34f3, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed34fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x704d34ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x33ad3501, SUC_lexicon_tags_31 },
    { 0xad6d3502, SUC_lexicon_tags_27 },
    { 0xcded3502, SUC_lexicon_tags_60 },
    { 0xf72d3504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x478d3506, SUC_lexicon_tags_8 },
    { 0xf32d3507, SUC_lexicon_tags_29 },
    { 0x64d3508, SUC_lexicon_tags_12 },
    { 0x760d3509, SUC_lexicon_tags_39 },
    { 0x6a0d350a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ed350e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108d3512, SUC_lexicon_tags_1 },
    { 0x2b8d3513, SUC_lexicon_tags_19 },
    { 0xa9ed3513, SUC_lexicon_tags_32 },
    { 0x9c2d3515, SUC_lexicon_tags_3 },
    { 0xecad3515, SUC_lexicon_tags_18 },
    { 0x3e2d3517, SUC_lexicon_tags_7 },
    { 0x170d3512, SUC_lexicon_tags_43 },
    { 0x4ced3517, SUC_lexicon_tags_7 },
    { 0x13ad351a, SUC_lexicon_tags_43 },
    { 0xeeed351a, SUC_lexicon_tags_28 },
    { 0xdd8d3515, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cd3520, SUC_lexicon_tags_48 },
    { 0x520d3521, SUC_lexicon_tags_11 },
    { 0x254d3521, SUC_lexicon_tags_25 },
    { 0xa3ed3520, SUC_lexicon_tags_27 },
    { 0x656d3524, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xacad3526, SUC_lexicon_tags_29 },
    { 0x60cd3527, SUC_lexicon_tags_25 },
    { 0x154d3528, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23ad352a, SUC_lexicon_tags_33 },
    { 0x1d0d352b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ad352f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x782d3531, SUC_lexicon_tags_1089 },
    { 0, NULL },
    { 0xb8ed3533, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34d3536, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd8cd3538, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x174d353b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cd3541, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4d3545, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x652d3547, SUC_lexicon_tags_23 },
    { 0x756d3548, SUC_lexicon_tags_42 },
    { 0x25ad3548, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0d354e, SUC_lexicon_tags_6 },
    { 0x2bcd354f, SUC_lexicon_tags_0 },
    { 0x954d3550, SUC_lexicon_tags_19 },
    { 0x63cd3550, SUC_lexicon_tags_11 },
    { 0x408d354e, SUC_lexicon_tags_45 },
    { 0x512d3553, SUC_lexicon_tags_11 },
    { 0xc0d3554, SUC_lexicon_tags_2 },
    { 0xc14d3552, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x494d3558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10ed355a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4d355e, SUC_lexicon_tags_9 },
    { 0xb3cd355f, SUC_lexicon_tags_37 },
    { 0x432d355e, SUC_lexicon_tags_27 },
    { 0x7d6d3561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280d3566, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x57ad3568, SUC_lexicon_tags_11 },
    { 0x6b6d3568, SUC_lexicon_tags_1 },
    { 0x666d356a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4d356f, SUC_lexicon_tags_3 },
    { 0xca0d3570, SUC_lexicon_tags_11 },
    { 0x61ed3571, SUC_lexicon_tags_46 },
    { 0x2f8d356f, SUC_lexicon_tags_15 },
    { 0xd76d3573, SUC_lexicon_tags_17 },
    { 0x2ced3574, SUC_lexicon_tags_8 },
    { 0xed4d3575, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xded3578, SUC_lexicon_tags_29 },
    { 0xba6d3579, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc40d357b, SUC_lexicon_tags_6 },
    { 0x6c0d357c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04d3580, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4d3586, SUC_lexicon_tags_27 },
    { 0xb64d3586, SUC_lexicon_tags_63 },
    { 0xbead3586, SUC_lexicon_tags_11 },
    { 0x834d3589, SUC_lexicon_tags_15 },
    { 0x140d3587, SUC_lexicon_tags_31 },
    { 0x8b8d3587, SUC_lexicon_tags_6 },
    { 0x2acd358c, SUC_lexicon_tags_34 },
    { 0x3d4d358d, SUC_lexicon_tags_17 },
    { 0x4bcd3586, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xedad3590, SUC_lexicon_tags_1 },
    { 0xf5ad3591, SUC_lexicon_tags_60 },
    { 0xe94d3592, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d3599, SUC_lexicon_tags_84 },
    { 0xed2d359a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4d359d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ad35a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc92d35a2, SUC_lexicon_tags_12 },
    { 0x836d35a3, SUC_lexicon_tags_15 },
    { 0xfead35a4, SUC_lexicon_tags_12 },
    { 0xa6ed35a5, SUC_lexicon_tags_2 },
    { 0xb42d35a6, SUC_lexicon_tags_29 },
    { 0x56ed35a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452d35b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99cd35b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2d35b6, SUC_lexicon_tags_61 },
    { 0xd22d35b7, SUC_lexicon_tags_4 },
    { 0x4bed35b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ad35bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x912d35be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88d35c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50d35c8, SUC_lexicon_tags_37 },
    { 0x5e4d35c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xba0d35cb, SUC_lexicon_tags_29 },
    { 0xdaad35cc, SUC_lexicon_tags_1 },
    { 0x3acd35cc, SUC_lexicon_tags_49 },
    { 0xfced35ce, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ed35d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498d35d9, SUC_lexicon_tags_17 },
    { 0x8fcd35da, SUC_lexicon_tags_14 },
    { 0xc12d35d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4d35de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ad35e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x156d35e3, SUC_lexicon_tags_26 },
    { 0x3ded35e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf8d35e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x824d35e8, SUC_lexicon_tags_45 },
    { 0xd48d35e9, SUC_lexicon_tags_37 },
    { 0x1acd35ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a0d35ec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x904d35ee, SUC_lexicon_tags_45 },
    { 0xfbcd35ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24cd35f1, SUC_lexicon_tags_37 },
    { 0x3f2d35f2, SUC_lexicon_tags_11 },
    { 0x64ed35f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdced35f5, SUC_lexicon_tags_32 },
    { 0x728d35f6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d35fa, SUC_lexicon_tags_17 },
    { 0x44ad35fb, SUC_lexicon_tags_7 },
    { 0x994d35fa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x488d35fe, SUC_lexicon_tags_4 },
    { 0xdd2d35ff, SUC_lexicon_tags_60 },
    { 0xa00d35fe, SUC_lexicon_tags_29 },
    { 0xb48d3601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca8d3603, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84d3608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c8d360a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ad360c, SUC_lexicon_tags_9 },
    { 0xf20d360d, SUC_lexicon_tags_66 },
    { 0x826d360e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cd3612, SUC_lexicon_tags_151 },
    { 0x226d3612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa42d3615, SUC_lexicon_tags_33 },
    { 0xaa4d3616, SUC_lexicon_tags_58 },
    { 0x86cd3617, SUC_lexicon_tags_25 },
    { 0xd0d3617, SUC_lexicon_tags_9 },
    { 0xbecd3619, SUC_lexicon_tags_14 },
    { 0x218d361a, SUC_lexicon_tags_5 },
    { 0x664d361b, SUC_lexicon_tags_17 },
    { 0x9fad361c, SUC_lexicon_tags_13 },
    { 0x280d361b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ed3621, SUC_lexicon_tags_12 },
    { 0x966d3622, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3f8d3624, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4d3627, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x16ed3629, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad362d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x836d362f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xde0d3631, SUC_lexicon_tags_2 },
    { 0x826d3632, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaad363b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4d363f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ed3643, SUC_lexicon_tags_4 },
    { 0x606d3644, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f0d3646, SUC_lexicon_tags_13 },
    { 0x628d3647, SUC_lexicon_tags_16 },
    { 0x7ecd3648, SUC_lexicon_tags_60 },
    { 0xcb6d3646, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ded364c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cd3655, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690d3659, SUC_lexicon_tags_0 },
    { 0xaded365a, SUC_lexicon_tags_1 },
    { 0xeb2d365b, SUC_lexicon_tags_26 },
    { 0x6acd365b, SUC_lexicon_tags_66 },
    { 0x56cd365d, SUC_lexicon_tags_66 },
    { 0xe58d365d, SUC_lexicon_tags_31 },
    { 0xf0cd365f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44d3662, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcd366a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2d366e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x12d3670, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0d367a, SUC_lexicon_tags_45 },
    { 0x16ad367a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9d8d367d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x572d367f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5c2d3681, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x760d3683, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8d3686, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x742d3688, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed3692, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcd3696, SUC_lexicon_tags_2 },
    { 0x7c6d3696, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ed36a3, SUC_lexicon_tags_31 },
    { 0x53cd36a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe16d36a6, SUC_lexicon_tags_15 },
    { 0xcdad36a7, SUC_lexicon_tags_49 },
    { 0x60d36a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab0d36aa, SUC_lexicon_tags_45 },
    { 0x3b2d36ab, SUC_lexicon_tags_55 },
    { 0x264d36ac, SUC_lexicon_tags_14 },
    { 0x93cd36ad, SUC_lexicon_tags_2 },
    { 0x3d8d36ae, SUC_lexicon_tags_2 },
    { 0x7bcd36ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a6d36b1, SUC_lexicon_tags_11 },
    { 0x728d36b2, SUC_lexicon_tags_53 },
    { 0x582d36b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf94d36b5, SUC_lexicon_tags_28 },
    { 0x120d36b6, SUC_lexicon_tags_10 },
    { 0xa46d36b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e8d36b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22d36bd, SUC_lexicon_tags_37 },
    { 0xe68d36be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8f2d36c0, SUC_lexicon_tags_15 },
    { 0x1bcd36c1, SUC_lexicon_tags_26 },
    { 0x99ed36c1, SUC_lexicon_tags_7 },
    { 0xbacd36c3, SUC_lexicon_tags_45 },
    { 0x57ad36c0, SUC_lexicon_tags_7 },
    { 0x354d36c5, SUC_lexicon_tags_16 },
    { 0x154d36c5, SUC_lexicon_tags_17 },
    { 0xe5ad36c0, SUC_lexicon_tags_25 },
    { 0xbd4d36c7, SUC_lexicon_tags_65 },
    { 0xb64d36c1, SUC_lexicon_tags_15 },
    { 0x692d36c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230d36d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4d36d6, SUC_lexicon_tags_33 },
    { 0x87ed36d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcd36df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa60d36e1, SUC_lexicon_tags_14 },
    { 0xf6d36e2, SUC_lexicon_tags_49 },
    { 0xc36d36e3, SUC_lexicon_tags_1 },
    { 0x150d36e1, SUC_lexicon_tags_23 },
    { 0x1e4d36e4, SUC_lexicon_tags_17 },
    { 0x87ad36e2, SUC_lexicon_tags_27 },
    { 0x718d36e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6d36ec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x826d36ee, SUC_lexicon_tags_60 },
    { 0x61ad36ee, SUC_lexicon_tags_51 },
    { 0xa86d36ee, SUC_lexicon_tags_218 },
    { 0x36cd36f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0d36f6, SUC_lexicon_tags_6 },
    { 0x804d36f6, SUC_lexicon_tags_34 },
    { 0xde8d36f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6ad36fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7cd36fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436d3700, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6cd3702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ecd3704, SUC_lexicon_tags_56 },
    { 0xccd3705, SUC_lexicon_tags_31 },
    { 0x542d3705, SUC_lexicon_tags_14 },
    { 0xcb0d3707, SUC_lexicon_tags_19 },
    { 0x574d3708, SUC_lexicon_tags_60 },
    { 0xcc4d3705, SUC_lexicon_tags_6 },
    { 0x104d370a, SUC_lexicon_tags_0 },
    { 0x37ed3708, SUC_lexicon_tags_17 },
    { 0x3a8d370c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52ad370e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f4d3710, SUC_lexicon_tags_28 },
    { 0xd56d3711, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x116d3713, SUC_lexicon_tags_9 },
    { 0x654d3714, SUC_lexicon_tags_27 },
    { 0x110d3715, SUC_lexicon_tags_11 },
    { 0xe34d3716, SUC_lexicon_tags_37 },
    { 0x72cd3716, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b4d3719, SUC_lexicon_tags_1 },
    { 0xf6ad3719, SUC_lexicon_tags_10 },
    { 0xe0ed371b, SUC_lexicon_tags_61 },
    { 0xb52d371c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c4d371e, SUC_lexicon_tags_37 },
    { 0xa02d371e, SUC_lexicon_tags_37 },
    { 0x134d3720, SUC_lexicon_tags_29 },
    { 0x8dad3721, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44d3724, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca2d3726, SUC_lexicon_tags_27 },
    { 0xfeed3726, SUC_lexicon_tags_13 },
    { 0xbb2d3728, SUC_lexicon_tags_23 },
    { 0xbc6d3729, SUC_lexicon_tags_2 },
    { 0x992d372a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ed372e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6d3731, SUC_lexicon_tags_25 },
    { 0x67ed3731, SUC_lexicon_tags_19 },
    { 0x3f8d3733, SUC_lexicon_tags_39 },
    { 0xcb0d3734, SUC_lexicon_tags_38 },
    { 0xb22d3733, SUC_lexicon_tags_26 },
    { 0xdf0d3731, SUC_lexicon_tags_65 },
    { 0x386d3736, SUC_lexicon_tags_12 },
    { 0xdeed3738, SUC_lexicon_tags_46 },
    { 0x95ed3739, SUC_lexicon_tags_18 },
    { 0xa9cd3731, SUC_lexicon_tags_11 },
    { 0xaf2d3732, SUC_lexicon_tags_12 },
    { 0xd12d3732, SUC_lexicon_tags_11 },
    { 0xf04d373d, SUC_lexicon_tags_25 },
    { 0xc52d373e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80d3740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8d3745, SUC_lexicon_tags_14 },
    { 0x966d3746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e2d3748, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06d374c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80d374f, SUC_lexicon_tags_37 },
    { 0xdd2d3750, SUC_lexicon_tags_15 },
    { 0x6c8d374f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88d3754, SUC_lexicon_tags_15 },
    { 0x18d3755, SUC_lexicon_tags_8 },
    { 0xe9cd3754, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ed3759, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7ad375b, SUC_lexicon_tags_7 },
    { 0xc08d375c, SUC_lexicon_tags_15 },
    { 0x844d375d, SUC_lexicon_tags_7 },
    { 0x1a4d375d, SUC_lexicon_tags_12 },
    { 0x65cd375d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xec8d3761, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52d3769, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ad376d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41cd376f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad3774, SUC_lexicon_tags_17 },
    { 0xed0d3775, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42d3779, SUC_lexicon_tags_108 },
    { 0xbe8d3779, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ed3783, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876d3788, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x294d378b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ed3790, SUC_lexicon_tags_66 },
    { 0x44ed3790, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72d379b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46d37a0, SUC_lexicon_tags_45 },
    { 0xad0d37a1, SUC_lexicon_tags_6 },
    { 0xec4d37a2, SUC_lexicon_tags_5 },
    { 0x1fed37a3, SUC_lexicon_tags_43 },
    { 0x89cd37a3, SUC_lexicon_tags_26 },
    { 0x58cd37a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fad37aa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4d37ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744d37b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98d37b6, SUC_lexicon_tags_62 },
    { 0x34d37b6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x1e6d37b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660d37c0, SUC_lexicon_tags_14 },
    { 0x4c8d37c1, SUC_lexicon_tags_1 },
    { 0xc8ad37c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0d37c6, SUC_lexicon_tags_64 },
    { 0x2ad37c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa1cd37c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8acd37cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x646d37cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x686d37d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42d37d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ed37d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x924d37db, SUC_lexicon_tags_17 },
    { 0xdad37dc, SUC_lexicon_tags_49 },
    { 0x2e4d37dd, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20d37e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ed37e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ed37ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe86d37ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d37ef, SUC_lexicon_tags_1 },
    { 0x110d37f0, SUC_lexicon_tags_14 },
    { 0x372d37f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2d37f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61ad37f6, SUC_lexicon_tags_25 },
    { 0x496d37f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598d37fc, SUC_lexicon_tags_748 },
    { 0x37ed37fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1a2d37ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x316d3801, SUC_lexicon_tags_63 },
    { 0xb72d3802, SUC_lexicon_tags_85 },
    { 0xda6d3803, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436d3807, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcd380a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa20d380c, SUC_lexicon_tags_26 },
    { 0xbded380c, SUC_lexicon_tags_15 },
    { 0x440d380e, SUC_lexicon_tags_48 },
    { 0x5b8d380f, SUC_lexicon_tags_2 },
    { 0xa5cd380e, SUC_lexicon_tags_17 },
    { 0x660d3811, SUC_lexicon_tags_37 },
    { 0xccad3811, SUC_lexicon_tags_12 },
    { 0xae0d3811, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6d3816, SUC_lexicon_tags_60 },
    { 0xa8cd3816, SUC_lexicon_tags_12 },
    { 0xae0d3818, SUC_lexicon_tags_82 },
    { 0x392d3818, SUC_lexicon_tags_12 },
    { 0x568d381a, SUC_lexicon_tags_17 },
    { 0xef8d381b, SUC_lexicon_tags_7 },
    { 0x184d3816, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cd3824, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114d3829, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662d382d, SUC_lexicon_tags_53 },
    { 0xed8d382e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x8bad3830, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd88d3832, SUC_lexicon_tags_1 },
    { 0x288d3832, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4d3836, SUC_lexicon_tags_15 },
    { 0x66cd3837, SUC_lexicon_tags_15 },
    { 0xf04d3838, SUC_lexicon_tags_61 },
    { 0x66cd3839, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98d3842, SUC_lexicon_tags_25 },
    { 0xe80d3843, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x47ad3845, SUC_lexicon_tags_32 },
    { 0xf42d3846, SUC_lexicon_tags_53 },
    { 0xc44d3846, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x53ad3849, SUC_lexicon_tags_37 },
    { 0x644d384a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fed384e, SUC_lexicon_tags_12 },
    { 0x690d384f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x52d3851, SUC_lexicon_tags_17 },
    { 0xaeed3851, SUC_lexicon_tags_305 },
    { 0x1a6d3853, SUC_lexicon_tags_800 },
    { 0xe9ad3854, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766d385d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x788d385f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04d3865, SUC_lexicon_tags_14 },
    { 0xa06d3866, SUC_lexicon_tags_16 },
    { 0xcc0d3865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x784d3869, SUC_lexicon_tags_25 },
    { 0x668d386a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x748d386c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x132d386e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50d3871, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08d3874, SUC_lexicon_tags_43 },
    { 0xced3874, SUC_lexicon_tags_19 },
    { 0x94ad3874, SUC_lexicon_tags_1 },
    { 0x4c4d3874, SUC_lexicon_tags_45 },
    { 0xd2d3878, SUC_lexicon_tags_2 },
    { 0x68ed3879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ed387d, SUC_lexicon_tags_47 },
    { 0x42ed387e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8bad3880, SUC_lexicon_tags_79 },
    { 0xd5ed3881, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156d3886, SUC_lexicon_tags_11 },
    { 0xe0cd3887, SUC_lexicon_tags_7 },
    { 0x58ad3887, SUC_lexicon_tags_0 },
    { 0x32d3889, SUC_lexicon_tags_8 },
    { 0x5e4d388a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0d388e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d0d3890, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6bad3892, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cd3896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x402d3899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d389d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ed38a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910d38ab, SUC_lexicon_tags_1283 },
    { 0x6a8d38ac, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbd8d38ae, SUC_lexicon_tags_37 },
    { 0x258d38ae, SUC_lexicon_tags_11 },
    { 0xb74d38b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33cd38b3, SUC_lexicon_tags_11 },
    { 0xe44d38b4, SUC_lexicon_tags_1 },
    { 0x2b4d38b5, SUC_lexicon_tags_11 },
    { 0xd22d38b6, SUC_lexicon_tags_7 },
    { 0x310d38b3, SUC_lexicon_tags_59 },
    { 0xa40d38b3, SUC_lexicon_tags_11 },
    { 0x7fed38b9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2d38bd, SUC_lexicon_tags_2 },
    { 0x156d38bd, SUC_lexicon_tags_7 },
    { 0x104d38be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570d38c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4d38c9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88d38d0, SUC_lexicon_tags_63 },
    { 0xbe0d38d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcd38d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88d38d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6d38e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb80d38e4, SUC_lexicon_tags_65 },
    { 0x758d38e5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd8d38e7, SUC_lexicon_tags_77 },
    { 0x176d38e8, SUC_lexicon_tags_25 },
    { 0xa18d38e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2d38ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x774d38f2, SUC_lexicon_tags_23 },
    { 0xbbed38f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcad38f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa04d38fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x774d38fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x540d38fe, SUC_lexicon_tags_43 },
    { 0x324d38ff, SUC_lexicon_tags_6 },
    { 0xd46d3900, SUC_lexicon_tags_7 },
    { 0x126d3901, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140d3906, SUC_lexicon_tags_60 },
    { 0x38cd3907, SUC_lexicon_tags_27 },
    { 0x6c4d3908, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2d390c, SUC_lexicon_tags_26 },
    { 0x4d8d390d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x582d3910, SUC_lexicon_tags_28 },
    { 0xcc8d3911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96cd3913, SUC_lexicon_tags_29 },
    { 0x722d3913, SUC_lexicon_tags_12 },
    { 0x452d3915, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbdcd3917, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed391c, SUC_lexicon_tags_45 },
    { 0x694d391d, SUC_lexicon_tags_49 },
    { 0xb92d391e, SUC_lexicon_tags_37 },
    { 0xaaad391c, SUC_lexicon_tags_15 },
    { 0x8bcd3920, SUC_lexicon_tags_66 },
    { 0xd1ed3921, SUC_lexicon_tags_97 },
    { 0xa9cd3922, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868d3926, SUC_lexicon_tags_25 },
    { 0x424d3927, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690d392b, SUC_lexicon_tags_27 },
    { 0xc18d392b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x604d392e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84d3933, SUC_lexicon_tags_25 },
    { 0xa8ad3933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb4d3936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecd3939, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x818d393b, SUC_lexicon_tags_25 },
    { 0x410d393c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d393f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96d3942, SUC_lexicon_tags_17 },
    { 0x944d3943, SUC_lexicon_tags_31 },
    { 0x752d3944, SUC_lexicon_tags_25 },
    { 0x7d4d3944, SUC_lexicon_tags_2 },
    { 0xe2ed3946, SUC_lexicon_tags_25 },
    { 0xd72d3947, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aad394b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d2d394d, SUC_lexicon_tags_17 },
    { 0x97ed394e, SUC_lexicon_tags_7 },
    { 0x4dad394f, SUC_lexicon_tags_8 },
    { 0x40d3950, SUC_lexicon_tags_65 },
    { 0xcbcd394d, SUC_lexicon_tags_17 },
    { 0x7b6d3952, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40d3957, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x58ad3959, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84d395e, SUC_lexicon_tags_11 },
    { 0x5d2d395f, SUC_lexicon_tags_23 },
    { 0x1bad3960, SUC_lexicon_tags_45 },
    { 0x9a8d3961, SUC_lexicon_tags_60 },
    { 0xe9ed395f, SUC_lexicon_tags_17 },
    { 0x500d3963, SUC_lexicon_tags_35 },
    { 0x4b0d3962, SUC_lexicon_tags_179 },
    { 0x3b0d3965, SUC_lexicon_tags_25 },
    { 0x720d3966, SUC_lexicon_tags_15 },
    { 0x6d2d3967, SUC_lexicon_tags_1 },
    { 0xd92d3968, SUC_lexicon_tags_11 },
    { 0x88d3965, SUC_lexicon_tags_41 },
    { 0x792d3965, SUC_lexicon_tags_32 },
    { 0x200d3960, SUC_lexicon_tags_11 },
    { 0xd2d3965, SUC_lexicon_tags_7 },
    { 0x956d3966, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x426d3970, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6d3974, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8d3978, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4d397c, SUC_lexicon_tags_7 },
    { 0x2ad397d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x562d397f, SUC_lexicon_tags_37 },
    { 0x5a8d3980, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4eed3982, SUC_lexicon_tags_11 },
    { 0xc2cd3982, SUC_lexicon_tags_27 },
    { 0x268d3984, SUC_lexicon_tags_3 },
    { 0x3ded3983, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2d398b, SUC_lexicon_tags_45 },
    { 0xeded398b, SUC_lexicon_tags_29 },
    { 0x266d398b, SUC_lexicon_tags_38 },
    { 0xe82d398d, SUC_lexicon_tags_737 },
    { 0xf98d398f, SUC_lexicon_tags_138 },
    { 0xf10d3990, SUC_lexicon_tags_25 },
    { 0x522d398f, SUC_lexicon_tags_10 },
    { 0x5ced398d, SUC_lexicon_tags_9 },
    { 0x72ad3993, SUC_lexicon_tags_65 },
    { 0x836d3993, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x438d3996, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2d399c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ed39a0, SUC_lexicon_tags_2 },
    { 0x342d39a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b0d39a3, SUC_lexicon_tags_49 },
    { 0xa4ed39a4, SUC_lexicon_tags_6 },
    { 0xadad39a5, SUC_lexicon_tags_25 },
    { 0xf1ad39a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb8d39a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x222d39aa, SUC_lexicon_tags_17 },
    { 0x1e4d39ab, SUC_lexicon_tags_11 },
    { 0xd18d39aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd7ad39ae, SUC_lexicon_tags_37 },
    { 0x3a2d39af, SUC_lexicon_tags_29 },
    { 0x958d39b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfded39b9, SUC_lexicon_tags_23 },
    { 0xafad39ba, SUC_lexicon_tags_17 },
    { 0xf94d39bb, SUC_lexicon_tags_1 },
    { 0x850d39bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x650d39bf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774d39ce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ad39d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374d39db, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0d39df, SUC_lexicon_tags_14 },
    { 0xf3ad39e0, SUC_lexicon_tags_15 },
    { 0xcbad39e0, SUC_lexicon_tags_43 },
    { 0x888d39df, SUC_lexicon_tags_14 },
    { 0x36d39df, SUC_lexicon_tags_0 },
    { 0xb66d39e4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6ded39e6, SUC_lexicon_tags_0 },
    { 0x8ead39e6, SUC_lexicon_tags_87 },
    { 0x9ecd39e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30d39eb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636d39ef, SUC_lexicon_tags_19 },
    { 0x65ad39f0, SUC_lexicon_tags_49 },
    { 0x2acd39f1, SUC_lexicon_tags_9 },
    { 0xca4d39ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x728d39f5, SUC_lexicon_tags_7 },
    { 0x10d39f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8d39fb, SUC_lexicon_tags_1 },
    { 0xe94d39fc, SUC_lexicon_tags_38 },
    { 0x568d39fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadad3a02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x934d3a05, SUC_lexicon_tags_45 },
    { 0x1a2d3a06, SUC_lexicon_tags_15 },
    { 0x5e6d3a06, SUC_lexicon_tags_11 },
    { 0xb14d3a08, SUC_lexicon_tags_37 },
    { 0x846d3a09, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6f6d3a0b, SUC_lexicon_tags_37 },
    { 0x3e4d3a0c, SUC_lexicon_tags_14 },
    { 0xd8ad3a0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x594d3a0f, SUC_lexicon_tags_7 },
    { 0xb14d3a10, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8d3a14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ad3a19, SUC_lexicon_tags_95 },
    { 0x3fad3a1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40d3a1e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd18d3a20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0d3a23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a8d3a25, SUC_lexicon_tags_61 },
    { 0x678d3a26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcd3a2d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bed3a30, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70d3a33, SUC_lexicon_tags_61 },
    { 0x264d3a33, SUC_lexicon_tags_30 },
    { 0xe8d3a35, SUC_lexicon_tags_17 },
    { 0x1c2d3a36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x702d3a39, SUC_lexicon_tags_15 },
    { 0x36ed3a39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ad3a40, SUC_lexicon_tags_15 },
    { 0xd86d3a41, SUC_lexicon_tags_12 },
    { 0xf4ed3a41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x614d3a44, SUC_lexicon_tags_282 },
    { 0x518d3a44, SUC_lexicon_tags_15 },
    { 0x4a4d3a46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x118d3a48, SUC_lexicon_tags_11 },
    { 0x4f8d3a48, SUC_lexicon_tags_11 },
    { 0xf56d3a4a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0d3a5d, SUC_lexicon_tags_15 },
    { 0xf2d3a5e, SUC_lexicon_tags_7 },
    { 0x7aad3a5f, SUC_lexicon_tags_8 },
    { 0x19ad3a5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5e4d3a62, SUC_lexicon_tags_19 },
    { 0x702d3a62, SUC_lexicon_tags_87 },
    { 0xcd3a62, SUC_lexicon_tags_1 },
    { 0x3ead3a62, SUC_lexicon_tags_37 },
    { 0x59cd3a66, SUC_lexicon_tags_12 },
    { 0x95ed3a66, SUC_lexicon_tags_7 },
    { 0xc66d3a68, SUC_lexicon_tags_1 },
    { 0x2aad3a67, SUC_lexicon_tags_44 },
    { 0xdc8d3a66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2d3a70, SUC_lexicon_tags_47 },
    { 0x6e8d3a71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x28ed3a73, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d3a76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1cd3a78, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54d3a7b, SUC_lexicon_tags_11 },
    { 0xaf8d3a7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ed3a7f, SUC_lexicon_tags_56 },
    { 0xf56d3a7f, SUC_lexicon_tags_55 },
    { 0x872d3a81, SUC_lexicon_tags_26 },
    { 0x928d3a81, SUC_lexicon_tags_100 },
    { 0xa04d3a83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcccd3a86, SUC_lexicon_tags_63 },
    { 0xfd8d3a87, SUC_lexicon_tags_11 },
    { 0x19ad3a88, SUC_lexicon_tags_25 },
    { 0x7e6d3a88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acd3a8d, SUC_lexicon_tags_14 },
    { 0xeb0d3a8d, SUC_lexicon_tags_87 },
    { 0x422d3a8d, SUC_lexicon_tags_38 },
    { 0xc24d3a8e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd3a97, SUC_lexicon_tags_33 },
    { 0x40ad3a98, SUC_lexicon_tags_25 },
    { 0x586d3a98, SUC_lexicon_tags_4 },
    { 0xa30d3a9a, SUC_lexicon_tags_25 },
    { 0x4d0d3a9a, SUC_lexicon_tags_11 },
    { 0xf88d3a9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xefed3a9e, SUC_lexicon_tags_8 },
    { 0x694d3a9f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2d3aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad3aa8, SUC_lexicon_tags_15 },
    { 0x7ded3aa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d3aae, SUC_lexicon_tags_2 },
    { 0x53ad3aaf, SUC_lexicon_tags_12 },
    { 0x9a6d3aaf, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16d3ab3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad3ab6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ed3abb, SUC_lexicon_tags_56 },
    { 0x366d3abc, SUC_lexicon_tags_38 },
    { 0x65ed3abd, SUC_lexicon_tags_10 },
    { 0x982d3abe, SUC_lexicon_tags_49 },
    { 0x49cd3abf, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x438d3ac2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6d3ac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x102d3ac7, SUC_lexicon_tags_43 },
    { 0x14ed3ac7, SUC_lexicon_tags_28 },
    { 0xb0ed3ac7, SUC_lexicon_tags_28 },
    { 0x95ed3aca, SUC_lexicon_tags_0 },
    { 0x670d3acb, SUC_lexicon_tags_5 },
    { 0x12ad3acb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd28d3ace, SUC_lexicon_tags_1 },
    { 0x74ad3ace, SUC_lexicon_tags_45 },
    { 0x930d3ace, SUC_lexicon_tags_8 },
    { 0xa26d3ad1, SUC_lexicon_tags_15 },
    { 0x312d3ad2, SUC_lexicon_tags_15 },
    { 0xd0cd3ad2, SUC_lexicon_tags_49 },
    { 0x998d3ace, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf24d3ad6, SUC_lexicon_tags_27 },
    { 0x5d8d3ad7, SUC_lexicon_tags_9 },
    { 0x268d3ad6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2d3adb, SUC_lexicon_tags_31 },
    { 0xa20d3adc, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x84cd3ade, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ed3ae0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cd3ae3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ad3aea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ed3aed, SUC_lexicon_tags_10 },
    { 0x686d3aee, SUC_lexicon_tags_68 },
    { 0x150d3aee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484d3af3, SUC_lexicon_tags_23 },
    { 0x82d3af3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146d3afc, SUC_lexicon_tags_11 },
    { 0xa14d3afd, SUC_lexicon_tags_8 },
    { 0xbfad3afe, SUC_lexicon_tags_60 },
    { 0x9c4d3aff, SUC_lexicon_tags_64 },
    { 0xc86d3b00, SUC_lexicon_tags_6 },
    { 0x88cd3b01, SUC_lexicon_tags_43 },
    { 0x33cd3afc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed3b05, SUC_lexicon_tags_37 },
    { 0x448d3b06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72d3b0b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ed3b0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcfcd3b11, SUC_lexicon_tags_15 },
    { 0xb24d3b12, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d3b15, SUC_lexicon_tags_3 },
    { 0x38ed3b15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ed3b1b, SUC_lexicon_tags_37 },
    { 0xc3ad3b1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8d3b23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d3b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd8d3b2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x314d3b30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x506d3b32, SUC_lexicon_tags_11 },
    { 0x200d3b33, SUC_lexicon_tags_3 },
    { 0x4c4d3b34, SUC_lexicon_tags_19 },
    { 0xd74d3b34, SUC_lexicon_tags_15 },
    { 0x458d3b32, SUC_lexicon_tags_23 },
    { 0xdf2d3b37, SUC_lexicon_tags_15 },
    { 0x65ad3b37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8d3b3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16d3b41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80d3b44, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bad3b48, SUC_lexicon_tags_9 },
    { 0xc52d3b48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61ad3b4b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf30d3b4d, SUC_lexicon_tags_23 },
    { 0xf6ad3b4e, SUC_lexicon_tags_37 },
    { 0x9ed3b4d, SUC_lexicon_tags_33 },
    { 0xa00d3b4f, SUC_lexicon_tags_11 },
    { 0xebed3b51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0d3b54, SUC_lexicon_tags_9 },
    { 0x590d3b55, SUC_lexicon_tags_63 },
    { 0x178d3b56, SUC_lexicon_tags_17 },
    { 0xc54d3b55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x790d3b5a, SUC_lexicon_tags_84 },
    { 0x870d3b5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dad3b60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc56d3b62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d4d3b64, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x352d3b67, SUC_lexicon_tags_37 },
    { 0x588d3b68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78d3b6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc14d3b6e, SUC_lexicon_tags_15 },
    { 0x9c6d3b6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bed3b73, SUC_lexicon_tags_11 },
    { 0x2ded3b74, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824d3b79, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ced3b83, SUC_lexicon_tags_13 },
    { 0x9bcd3b83, SUC_lexicon_tags_27 },
    { 0xe14d3b84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18d3b8a, SUC_lexicon_tags_170 },
    { 0xe92d3b8a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0d3b8f, SUC_lexicon_tags_7 },
    { 0xa46d3b90, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x31ad3b92, SUC_lexicon_tags_13 },
    { 0xa44d3b93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b8d3b95, SUC_lexicon_tags_25 },
    { 0x242d3b95, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8d3b9a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcd3b9f, SUC_lexicon_tags_537 },
    { 0xdd8d3b9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88d3ba2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fed3ba4, SUC_lexicon_tags_37 },
    { 0x600d3ba5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4d3bab, SUC_lexicon_tags_37 },
    { 0x77ad3bac, SUC_lexicon_tags_15 },
    { 0xedcd3bab, SUC_lexicon_tags_30 },
    { 0xc74d3bae, SUC_lexicon_tags_63 },
    { 0x8b0d3bae, SUC_lexicon_tags_33 },
    { 0xaacd3bb0, SUC_lexicon_tags_47 },
    { 0x964d3bb0, SUC_lexicon_tags_7 },
    { 0x116d3bb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eed3bb5, SUC_lexicon_tags_11 },
    { 0xab8d3bb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe4ed3bb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x156d3bba, SUC_lexicon_tags_15 },
    { 0x96ed3bbb, SUC_lexicon_tags_17 },
    { 0xa1cd3bbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4d3bc2, SUC_lexicon_tags_56 },
    { 0x18cd3bc3, SUC_lexicon_tags_10 },
    { 0xdb8d3bc3, SUC_lexicon_tags_15 },
    { 0xedcd3bc5, SUC_lexicon_tags_6 },
    { 0x72ad3bc5, SUC_lexicon_tags_9 },
    { 0xec8d3bc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1ad3bc9, SUC_lexicon_tags_2 },
    { 0x34d3bc9, SUC_lexicon_tags_10 },
    { 0xe76d3bcb, SUC_lexicon_tags_15 },
    { 0x6b2d3bcc, SUC_lexicon_tags_47 },
    { 0x5c6d3bca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa4ed3bcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ed3bd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cd3bd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cd3be2, SUC_lexicon_tags_45 },
    { 0x414d3be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x218d3be6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6d3bea, SUC_lexicon_tags_37 },
    { 0xd68d3beb, SUC_lexicon_tags_25 },
    { 0x552d3bec, SUC_lexicon_tags_45 },
    { 0xf62d3bec, SUC_lexicon_tags_33 },
    { 0xd12d3bec, SUC_lexicon_tags_29 },
    { 0x2b6d3bef, SUC_lexicon_tags_39 },
    { 0xf62d3bf0, SUC_lexicon_tags_25 },
    { 0xa4cd3bef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56d3bf5, SUC_lexicon_tags_61 },
    { 0x7cad3bf6, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aed3bfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62d3bfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1acd3bff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2d3c04, SUC_lexicon_tags_522 },
    { 0x554d3c05, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ed3c08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd2ad3c0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe2d3c0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54d3c15, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6c6d3c17, SUC_lexicon_tags_0 },
    { 0xccad3c17, SUC_lexicon_tags_37 },
    { 0x86cd3c19, SUC_lexicon_tags_81 },
    { 0x4a6d3c1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58ed3c1c, SUC_lexicon_tags_25 },
    { 0xebcd3c1c, SUC_lexicon_tags_2 },
    { 0x72cd3c1d, SUC_lexicon_tags_43 },
    { 0x9f4d3c1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8d3c27, SUC_lexicon_tags_19 },
    { 0xd56d3c28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58ad3c2a, SUC_lexicon_tags_14 },
    { 0x772d3c2b, SUC_lexicon_tags_527 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ad3c2e, SUC_lexicon_tags_27 },
    { 0x7fad3c2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6d3c3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116d3c44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88d3c47, SUC_lexicon_tags_28 },
    { 0x138d3c47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ad3c4e, SUC_lexicon_tags_27 },
    { 0xe12d3c4e, SUC_lexicon_tags_15 },
    { 0xf86d3c50, SUC_lexicon_tags_12 },
    { 0xfaad3c50, SUC_lexicon_tags_2 },
    { 0xd74d3c50, SUC_lexicon_tags_95 },
    { 0xffcd3c50, SUC_lexicon_tags_7 },
    { 0xa7cd3c50, SUC_lexicon_tags_37 },
    { 0x29cd3c55, SUC_lexicon_tags_17 },
    { 0x44cd3c56, SUC_lexicon_tags_2 },
    { 0xb62d3c56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x138d3c59, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3fcd3c5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6d3c5e, SUC_lexicon_tags_47 },
    { 0x1a8d3c5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988d3c63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6d3c6d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeb4d3c6f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76d3c73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120d3c79, SUC_lexicon_tags_27 },
    { 0xe68d3c7a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x654d3c7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a6d3c7f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x84ad3c81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x568d3c84, SUC_lexicon_tags_4 },
    { 0x1a2d3c85, SUC_lexicon_tags_12 },
    { 0x496d3c85, SUC_lexicon_tags_34 },
    { 0xc30d3c87, SUC_lexicon_tags_82 },
    { 0x4fcd3c85, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18d3c8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880d3c8f, SUC_lexicon_tags_16 },
    { 0x14d3c90, SUC_lexicon_tags_0 },
    { 0x75ed3c90, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4d3c95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d3c9a, SUC_lexicon_tags_2 },
    { 0x17ed3c9a, SUC_lexicon_tags_15 },
    { 0xed2d3c9b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d3ca0, SUC_lexicon_tags_15 },
    { 0x666d3ca1, SUC_lexicon_tags_37 },
    { 0x7b0d3ca1, SUC_lexicon_tags_31 },
    { 0xc66d3ca3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcd3ca9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2bad3cab, SUC_lexicon_tags_11 },
    { 0x5bcd3cac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70d3cb1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x964d3cb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcacd3cb7, SUC_lexicon_tags_11 },
    { 0x486d3cb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ed3cba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ad3cbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xceed3cc0, SUC_lexicon_tags_6 },
    { 0x172d3cc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04d3cc5, SUC_lexicon_tags_60 },
    { 0xe52d3cc5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ad3cc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8d3ccc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188d3cd0, SUC_lexicon_tags_17 },
    { 0x8f0d3cd1, SUC_lexicon_tags_194 },
    { 0xe1ed3cd0, SUC_lexicon_tags_26 },
    { 0x2acd3cd3, SUC_lexicon_tags_7 },
    { 0x9ecd3cd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad3cd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3cad3cda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf7ad3cdc, SUC_lexicon_tags_15 },
    { 0xf9ad3cdc, SUC_lexicon_tags_37 },
    { 0xb78d3cde, SUC_lexicon_tags_7 },
    { 0x96ed3cde, SUC_lexicon_tags_2 },
    { 0x2cd3ce0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad3ce3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ad3ce7, SUC_lexicon_tags_7 },
    { 0xf8cd3ce8, SUC_lexicon_tags_11 },
    { 0x84ed3ce9, SUC_lexicon_tags_37 },
    { 0xb02d3ce9, SUC_lexicon_tags_17 },
    { 0x2b6d3ce8, SUC_lexicon_tags_37 },
    { 0x550d3ce9, SUC_lexicon_tags_27 },
    { 0x6f4d3ced, SUC_lexicon_tags_7 },
    { 0x4e0d3cea, SUC_lexicon_tags_12 },
    { 0x686d3ce7, SUC_lexicon_tags_61 },
    { 0x22d3cf0, SUC_lexicon_tags_14 },
    { 0x5d2d3ce8, SUC_lexicon_tags_64 },
    { 0x946d3ce8, SUC_lexicon_tags_37 },
    { 0x9b2d3cf2, SUC_lexicon_tags_7 },
    { 0x9f0d3cf4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8d3cf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ed3cfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ed3d02, SUC_lexicon_tags_17 },
    { 0x536d3d02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d3d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ad3d0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a4d3d0d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8d3d11, SUC_lexicon_tags_25 },
    { 0x62ad3d12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x54ad3d14, SUC_lexicon_tags_28 },
    { 0x80ed3d14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbeed3d17, SUC_lexicon_tags_2 },
    { 0xbe6d3d17, SUC_lexicon_tags_0 },
    { 0x1a2d3d19, SUC_lexicon_tags_17 },
    { 0x888d3d1a, SUC_lexicon_tags_17 },
    { 0xeead3d1b, SUC_lexicon_tags_34 },
    { 0xa24d3d1c, SUC_lexicon_tags_7 },
    { 0xe72d3d1d, SUC_lexicon_tags_9 },
    { 0xbd2d3d1e, SUC_lexicon_tags_17 },
    { 0xae0d3d1e, SUC_lexicon_tags_38 },
    { 0x11ad3d19, SUC_lexicon_tags_23 },
    { 0xf1ad3d21, SUC_lexicon_tags_14 },
    { 0x80ed3d22, SUC_lexicon_tags_2 },
    { 0x972d3d22, SUC_lexicon_tags_29 },
    { 0x606d3d23, SUC_lexicon_tags_29 },
    { 0xed4d3d22, SUC_lexicon_tags_0 },
    { 0xa32d3d24, SUC_lexicon_tags_47 },
    { 0x20cd3d27, SUC_lexicon_tags_12 },
    { 0xb78d3d28, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xc48d3d2a, SUC_lexicon_tags_17 },
    { 0x176d3d2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x31cd3d2e, SUC_lexicon_tags_12 },
    { 0x42ad3d2e, SUC_lexicon_tags_11 },
    { 0xf0d3d2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x274d3d32, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aed3d35, SUC_lexicon_tags_56 },
    { 0x8b0d3d36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44d3d41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8d3d44, SUC_lexicon_tags_15 },
    { 0xd6ad3d45, SUC_lexicon_tags_17 },
    { 0x7d2d3d45, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0d3d4a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ad3d4d, SUC_lexicon_tags_49 },
    { 0x846d3d4e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x50d3d50, SUC_lexicon_tags_7 },
    { 0xd3ad3d51, SUC_lexicon_tags_37 },
    { 0x68cd3d52, SUC_lexicon_tags_1 },
    { 0x192d3d50, SUC_lexicon_tags_33 },
    { 0x77cd3d54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x254d3d56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aad3d5c, SUC_lexicon_tags_193 },
    { 0x2b8d3d5c, SUC_lexicon_tags_126 },
    { 0x6ccd3d5e, SUC_lexicon_tags_7 },
    { 0x370d3d5f, SUC_lexicon_tags_38 },
    { 0x618d3d5f, SUC_lexicon_tags_37 },
    { 0xa0ad3d61, SUC_lexicon_tags_19 },
    { 0x802d3d5f, SUC_lexicon_tags_49 },
    { 0xcb4d3d63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ad3d6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32d3d6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x98ed3d70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ad3d74, SUC_lexicon_tags_15 },
    { 0x170d3d75, SUC_lexicon_tags_8 },
    { 0x30ed3d76, SUC_lexicon_tags_31 },
    { 0x684d3d77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282d3d7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48d3d7e, SUC_lexicon_tags_37 },
    { 0x9e2d3d7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4d3d85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xba0d3d87, SUC_lexicon_tags_29 },
    { 0x856d3d88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x374d3d8a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb12d3d8c, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222d3d90, SUC_lexicon_tags_17 },
    { 0xabed3d91, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ad3d97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20d3d9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x650d3d9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x386d3d9e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x34cd3da0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2d3da6, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cad3dab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x298d3dad, SUC_lexicon_tags_57 },
    { 0xaeed3dad, SUC_lexicon_tags_37 },
    { 0xffcd3daf, SUC_lexicon_tags_34 },
    { 0xff0d3dad, SUC_lexicon_tags_17 },
    { 0xb14d3db1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cd3db3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcd3db8, SUC_lexicon_tags_60 },
    { 0x6f4d3db9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cd3dbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d3dc4, SUC_lexicon_tags_28 },
    { 0xc56d3dc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x878d3dc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264d3dcc, SUC_lexicon_tags_12 },
    { 0x9f8d3dcc, SUC_lexicon_tags_72 },
    { 0xedad3dce, SUC_lexicon_tags_49 },
    { 0xeaad3dcf, SUC_lexicon_tags_8 },
    { 0xa06d3dd0, SUC_lexicon_tags_27 },
    { 0x6aed3dd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd0d3dd3, SUC_lexicon_tags_63 },
    { 0xb68d3dd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d3dd8, SUC_lexicon_tags_17 },
    { 0x5f0d3dd9, SUC_lexicon_tags_43 },
    { 0xd6d3dd9, SUC_lexicon_tags_17 },
    { 0x632d3dda, SUC_lexicon_tags_31 },
    { 0x4e4d3ddc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaad3dde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfed3de1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6d3de6, SUC_lexicon_tags_7 },
    { 0x916d3de7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb7cd3de9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d3dee, SUC_lexicon_tags_11 },
    { 0x34d3def, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6d3df6, SUC_lexicon_tags_10 },
    { 0x6d2d3df6, SUC_lexicon_tags_27 },
    { 0x2c6d3df6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8d3dfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4d3dff, SUC_lexicon_tags_37 },
    { 0xfb0d3dff, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26d3e07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcd3e0c, SUC_lexicon_tags_60 },
    { 0x4b6d3e0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x338d3e0f, SUC_lexicon_tags_359 },
    { 0x18cd3e0f, SUC_lexicon_tags_61 },
    { 0x334d3e11, SUC_lexicon_tags_1 },
    { 0xc00d3e11, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xeecd3e15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x82ed3e17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984d3e1b, SUC_lexicon_tags_33 },
    { 0x45ed3e1c, SUC_lexicon_tags_39 },
    { 0xd4ad3e1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabad3e22, SUC_lexicon_tags_49 },
    { 0x1a2d3e23, SUC_lexicon_tags_47 },
    { 0x5c6d3e23, SUC_lexicon_tags_7 },
    { 0xcb2d3e25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ed3e2a, SUC_lexicon_tags_34 },
    { 0x850d3e2b, SUC_lexicon_tags_10 },
    { 0x446d3e2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6d3e31, SUC_lexicon_tags_29 },
    { 0x9a4d3e32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04d3e35, SUC_lexicon_tags_7 },
    { 0x4acd3e36, SUC_lexicon_tags_47 },
    { 0x934d3e37, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ed3e3d, SUC_lexicon_tags_7 },
    { 0x82cd3e3e, SUC_lexicon_tags_44 },
    { 0x674d3e3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d3e4e, SUC_lexicon_tags_68 },
    { 0x7ecd3e4f, SUC_lexicon_tags_0 },
    { 0xdf6d3e4e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676d3e54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74d3e58, SUC_lexicon_tags_47 },
    { 0x1f8d3e58, SUC_lexicon_tags_25 },
    { 0x942d3e5a, SUC_lexicon_tags_3 },
    { 0x6c6d3e5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d3e5e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d3e65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d3e6a, SUC_lexicon_tags_1 },
    { 0xe90d3e6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc4d3e6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d0d3e6f, SUC_lexicon_tags_98 },
    { 0x7d6d3e70, SUC_lexicon_tags_45 },
    { 0xa90d3e70, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xac8d3e73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x486d3e76, SUC_lexicon_tags_11 },
    { 0x3d4d3e76, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ed3e7f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80d3e82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e6d3e84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x314d3e87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ed3e8b, SUC_lexicon_tags_70 },
    { 0x368d3e8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ded3e90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce6d3e92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8d3e96, SUC_lexicon_tags_25 },
    { 0xa7ad3e97, SUC_lexicon_tags_3 },
    { 0x706d3e97, SUC_lexicon_tags_17 },
    { 0x3b0d3e97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x182d3e9b, SUC_lexicon_tags_27 },
    { 0x7b8d3e9b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ed3e9f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2a8d3ea1, SUC_lexicon_tags_11 },
    { 0x1ecd3ea2, SUC_lexicon_tags_37 },
    { 0x1fcd3ea3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ed3eac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeacd3eae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a8d3eb0, SUC_lexicon_tags_29 },
    { 0x49ed3eb1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68d3eb4, SUC_lexicon_tags_25 },
    { 0xf58d3eb4, SUC_lexicon_tags_12 },
    { 0x980d3eb4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cad3ebb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff4d3ebd, SUC_lexicon_tags_47 },
    { 0x532d3ebd, SUC_lexicon_tags_2 },
    { 0x864d3ebe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xead3ec2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6d3ec5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d8d3ec7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bed3eca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb24d3ecc, SUC_lexicon_tags_37 },
    { 0xf44d3ecc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x778d3ecf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acd3ed4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cd3eda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x40ed3edc, SUC_lexicon_tags_2 },
    { 0x1e4d3edd, SUC_lexicon_tags_2 },
    { 0x3ed3ede, SUC_lexicon_tags_11 },
    { 0xa2ad3ede, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86d3ee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6d3ee9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cd3eef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ed3ef4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750d3efa, SUC_lexicon_tags_21 },
    { 0x42ed3efb, SUC_lexicon_tags_19 },
    { 0x4aed3efc, SUC_lexicon_tags_35 },
    { 0xd5ed3efd, SUC_lexicon_tags_15 },
    { 0x4ced3efe, SUC_lexicon_tags_45 },
    { 0xd3cd3efa, SUC_lexicon_tags_1 },
    { 0xcbed3f00, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x290d3f02, SUC_lexicon_tags_65 },
    { 0x6bad3f03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x988d3f06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82d3f0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712d3f0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xccad3f10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4d3f13, SUC_lexicon_tags_7 },
    { 0xa22d3f14, SUC_lexicon_tags_6 },
    { 0xef6d3f15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ad3f1a, SUC_lexicon_tags_15 },
    { 0xeaed3f1a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x72ad3f1d, SUC_lexicon_tags_47 },
    { 0xd00d3f1e, SUC_lexicon_tags_11 },
    { 0xab4d3f1f, SUC_lexicon_tags_0 },
    { 0xf3cd3f1e, SUC_lexicon_tags_43 },
    { 0xa7ad3f21, SUC_lexicon_tags_8 },
    { 0xd20d3f1d, SUC_lexicon_tags_19 },
    { 0x716d3f23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x662d3f26, SUC_lexicon_tags_7 },
    { 0x4ced3f27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87cd3f29, SUC_lexicon_tags_7 },
    { 0x976d3f2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa36d3f2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45ad3f2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec8d3f30, SUC_lexicon_tags_1697 },
    { 0x5f8d3f31, SUC_lexicon_tags_0 },
    { 0x5e4d3f32, SUC_lexicon_tags_31 },
    { 0xd2cd3f33, SUC_lexicon_tags_17 },
    { 0xbd2d3f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78cd3f36, SUC_lexicon_tags_14 },
    { 0x91cd3f37, SUC_lexicon_tags_37 },
    { 0x3fed3f38, SUC_lexicon_tags_37 },
    { 0x7a6d3f39, SUC_lexicon_tags_612 },
    { 0x6ded3f38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18d3f40, SUC_lexicon_tags_7 },
    { 0xd86d3f41, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ad3f46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0d3f4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7f6d3f4c, SUC_lexicon_tags_17 },
    { 0xc02d3f4d, SUC_lexicon_tags_9 },
    { 0xdecd3f4e, SUC_lexicon_tags_703 },
    { 0, NULL },
    { 0xc3ed3f50, SUC_lexicon_tags_17 },
    { 0xb62d3f51, SUC_lexicon_tags_14 },
    { 0xffad3f51, SUC_lexicon_tags_1 },
    { 0x940d3f51, SUC_lexicon_tags_21 },
    { 0x652d3f51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf14d3f56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45ad3f58, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ad3f5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94d3f63, SUC_lexicon_tags_12 },
    { 0xa3cd3f63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc16d3f66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbecd3f69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956d3f6d, SUC_lexicon_tags_37 },
    { 0xcc6d3f6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24d3f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d3f7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6d3f7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cd3f83, SUC_lexicon_tags_7 },
    { 0x1b0d3f84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dad3f8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x540d3f8e, SUC_lexicon_tags_17 },
    { 0x410d3f8f, SUC_lexicon_tags_7 },
    { 0x998d3f8f, SUC_lexicon_tags_14 },
    { 0x3acd3f91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4d3f9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe98d3f9c, SUC_lexicon_tags_27 },
    { 0xfb2d3f9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2d3fa1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd02d3fa3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbed3fa6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x700d3fa9, SUC_lexicon_tags_2 },
    { 0x438d3faa, SUC_lexicon_tags_11 },
    { 0x226d3fab, SUC_lexicon_tags_31 },
    { 0x70ad3fa9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad0d3fae, SUC_lexicon_tags_31 },
    { 0x534d3fae, SUC_lexicon_tags_27 },
    { 0xf08d3fae, SUC_lexicon_tags_45 },
    { 0xf78d3fb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70d3fb4, SUC_lexicon_tags_37 },
    { 0x916d3fb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e0d3fb7, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d3fba, SUC_lexicon_tags_30 },
    { 0x390d3fbb, SUC_lexicon_tags_12 },
    { 0x55ed3fbc, SUC_lexicon_tags_25 },
    { 0x5dcd3fbc, SUC_lexicon_tags_37 },
    { 0x32cd3fbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2d3fc4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2f2d3fc6, SUC_lexicon_tags_47 },
    { 0x332d3fc7, SUC_lexicon_tags_12 },
    { 0x6b4d3fc8, SUC_lexicon_tags_23 },
    { 0xcc0d3fc9, SUC_lexicon_tags_0 },
    { 0xabcd3fc7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x636d3fcc, SUC_lexicon_tags_33 },
    { 0x966d3fcd, SUC_lexicon_tags_15 },
    { 0x332d3fce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc82d3fd0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x86ed3fd2, SUC_lexicon_tags_56 },
    { 0x650d3fd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbad3fd5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x42d3fd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30d3fda, SUC_lexicon_tags_2 },
    { 0x76cd3fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fad3fe0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f0d3fe2, SUC_lexicon_tags_37 },
    { 0x6a4d3fe3, SUC_lexicon_tags_8 },
    { 0xfc0d3fe4, SUC_lexicon_tags_9 },
    { 0xab4d3fe5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ad3fec, SUC_lexicon_tags_23 },
    { 0x84d3fed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cad3ff1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cd3ff6, SUC_lexicon_tags_29 },
    { 0x7a4d3ff7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ad3ffd, SUC_lexicon_tags_27 },
    { 0x8f8d3ffe, SUC_lexicon_tags_33 },
    { 0x10cd3fff, SUC_lexicon_tags_25 },
    { 0x6c6d4000, SUC_lexicon_tags_19 },
    { 0xe1cd3ffe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0d4007, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x94cd4009, SUC_lexicon_tags_6 },
    { 0xf6ad400a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466d4010, SUC_lexicon_tags_35 },
    { 0x3c6d4011, SUC_lexicon_tags_15 },
    { 0x5f4d4012, SUC_lexicon_tags_14 },
    { 0xaed4013, SUC_lexicon_tags_33 },
    { 0xe3ed4014, SUC_lexicon_tags_17 },
    { 0xdccd4015, SUC_lexicon_tags_12 },
    { 0x2cd4014, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e6d4018, SUC_lexicon_tags_39 },
    { 0x262d4018, SUC_lexicon_tags_38 },
    { 0x414d4018, SUC_lexicon_tags_39 },
    { 0x7d0d4018, SUC_lexicon_tags_36 },
    { 0x150d401c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff2d401e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x902d4020, SUC_lexicon_tags_1 },
    { 0xfe6d4020, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8d4026, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x690d4029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x122d402b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x950d402d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5e2d402f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x906d4032, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576d4036, SUC_lexicon_tags_58 },
    { 0x650d4037, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3acd4039, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee8d403b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x202d403e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6d4d4040, SUC_lexicon_tags_60 },
    { 0x5c4d4040, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2d4046, SUC_lexicon_tags_136 },
    { 0x50cd4047, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x274d4049, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ccd404b, SUC_lexicon_tags_12 },
    { 0x38ed404c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ad4053, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0d405b, SUC_lexicon_tags_17 },
    { 0x4c6d405b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414d4061, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f8d4063, SUC_lexicon_tags_37 },
    { 0x808d4064, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cd4068, SUC_lexicon_tags_32 },
    { 0x32ed4068, SUC_lexicon_tags_17 },
    { 0x6e4d406a, SUC_lexicon_tags_43 },
    { 0x1f2d406b, SUC_lexicon_tags_10 },
    { 0xd0d406c, SUC_lexicon_tags_11 },
    { 0xb96d406c, SUC_lexicon_tags_49 },
    { 0x522d406c, SUC_lexicon_tags_25 },
    { 0xdd0d406f, SUC_lexicon_tags_15 },
    { 0xf2cd4068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ad4074, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x654d4076, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1fad4078, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34d407f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856d4083, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ed408a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56ed408c, SUC_lexicon_tags_1 },
    { 0x6ccd408d, SUC_lexicon_tags_19 },
    { 0x9ded408e, SUC_lexicon_tags_11 },
    { 0x6d6d408e, SUC_lexicon_tags_4 },
    { 0x5c0d4090, SUC_lexicon_tags_12 },
    { 0xc4cd408f, SUC_lexicon_tags_104 },
    { 0x942d4092, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x594d4095, SUC_lexicon_tags_63 },
    { 0xf28d4095, SUC_lexicon_tags_12 },
    { 0x240d4097, SUC_lexicon_tags_49 },
    { 0x4ad4098, SUC_lexicon_tags_45 },
    { 0x6b4d4099, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54d409e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8d0d40a0, SUC_lexicon_tags_12 },
    { 0x370d40a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0d40a5, SUC_lexicon_tags_68 },
    { 0xff4d40a6, SUC_lexicon_tags_45 },
    { 0x9e8d40a7, SUC_lexicon_tags_2 },
    { 0x620d40a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c4d40aa, SUC_lexicon_tags_74 },
    { 0xff8d40ab, SUC_lexicon_tags_56 },
    { 0x5a6d40ac, SUC_lexicon_tags_27 },
    { 0x9d0d40ac, SUC_lexicon_tags_12 },
    { 0x824d40ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x944d40b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd40b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x904d40ba, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8a6d40bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76d40c0, SUC_lexicon_tags_9 },
    { 0xd84d40c0, SUC_lexicon_tags_60 },
    { 0x122d40c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72d40c5, SUC_lexicon_tags_7 },
    { 0x72ad40c6, SUC_lexicon_tags_65 },
    { 0x824d40c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8d40ca, SUC_lexicon_tags_17 },
    { 0xf30d40cb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516d40cf, SUC_lexicon_tags_38 },
    { 0x272d40d0, SUC_lexicon_tags_23 },
    { 0x3cd40d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644d40d6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcd40da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cd40de, SUC_lexicon_tags_82 },
    { 0x498d40df, SUC_lexicon_tags_14 },
    { 0xe28d40e0, SUC_lexicon_tags_55 },
    { 0xdced40e1, SUC_lexicon_tags_31 },
    { 0x1a6d40e0, SUC_lexicon_tags_2 },
    { 0x728d40e0, SUC_lexicon_tags_63 },
    { 0x3ccd40e0, SUC_lexicon_tags_31 },
    { 0xd88d40e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892d40e9, SUC_lexicon_tags_37 },
    { 0x34d40e9, SUC_lexicon_tags_33 },
    { 0x48cd40eb, SUC_lexicon_tags_49 },
    { 0x46ad40ec, SUC_lexicon_tags_17 },
    { 0x750d40e9, SUC_lexicon_tags_0 },
    { 0xca8d40ee, SUC_lexicon_tags_2 },
    { 0xb64d40ee, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984d40f3, SUC_lexicon_tags_11 },
    { 0xb0ad40f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe26d40f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf72d40f8, SUC_lexicon_tags_37 },
    { 0x6c0d40f8, SUC_lexicon_tags_1 },
    { 0xac6d40fa, SUC_lexicon_tags_38 },
    { 0x8b4d40fa, SUC_lexicon_tags_2 },
    { 0xfa2d40fa, SUC_lexicon_tags_3 },
    { 0x79ad40fa, SUC_lexicon_tags_45 },
    { 0x73ad40fe, SUC_lexicon_tags_15 },
    { 0x128d40ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x624d4101, SUC_lexicon_tags_14 },
    { 0xdd0d4102, SUC_lexicon_tags_33 },
    { 0xf70d4102, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50d4108, SUC_lexicon_tags_112 },
    { 0xb88d4108, SUC_lexicon_tags_33 },
    { 0x300d410a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ad410f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x114d4111, SUC_lexicon_tags_37 },
    { 0x2fcd4111, SUC_lexicon_tags_37 },
    { 0xf60d4112, SUC_lexicon_tags_5 },
    { 0xdad4114, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecd4119, SUC_lexicon_tags_17 },
    { 0x488d411a, SUC_lexicon_tags_60 },
    { 0x104d411b, SUC_lexicon_tags_37 },
    { 0x864d411c, SUC_lexicon_tags_0 },
    { 0xf9ad411c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4d4123, SUC_lexicon_tags_7 },
    { 0x7c0d4124, SUC_lexicon_tags_37 },
    { 0x45ed4124, SUC_lexicon_tags_15 },
    { 0x26d4126, SUC_lexicon_tags_33 },
    { 0x76cd4127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd412b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cd412f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e6d4131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x928d4133, SUC_lexicon_tags_51 },
    { 0x9ccd4134, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cd4138, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84d413b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xecad413d, SUC_lexicon_tags_48 },
    { 0xdd0d413e, SUC_lexicon_tags_16 },
    { 0x91cd413e, SUC_lexicon_tags_68 },
    { 0x952d413f, SUC_lexicon_tags_9 },
    { 0xcc2d4141, SUC_lexicon_tags_38 },
    { 0xe22d4140, SUC_lexicon_tags_25 },
    { 0xbc2d4143, SUC_lexicon_tags_7 },
    { 0xeb8d4141, SUC_lexicon_tags_62 },
    { 0xaecd4145, SUC_lexicon_tags_27 },
    { 0x8c8d4142, SUC_lexicon_tags_11 },
    { 0x1ecd4147, SUC_lexicon_tags_25 },
    { 0x260d4148, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ad414c, SUC_lexicon_tags_17 },
    { 0x108d414d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d4151, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9d0d4153, SUC_lexicon_tags_37 },
    { 0x20ad4153, SUC_lexicon_tags_11 },
    { 0xb70d4155, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a8d4157, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x800d4159, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836d415d, SUC_lexicon_tags_49 },
    { 0xd14d415e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2d4161, SUC_lexicon_tags_15 },
    { 0x38cd4162, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff4d4164, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2cd4166, SUC_lexicon_tags_61 },
    { 0x3d8d4166, SUC_lexicon_tags_14 },
    { 0x8f4d4168, SUC_lexicon_tags_7 },
    { 0x7e4d4166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84d416d, SUC_lexicon_tags_49 },
    { 0xa02d416e, SUC_lexicon_tags_11 },
    { 0xd26d416e, SUC_lexicon_tags_14 },
    { 0x35cd416e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ced4172, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88ed4174, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158d4178, SUC_lexicon_tags_82 },
    { 0xd18d4179, SUC_lexicon_tags_19 },
    { 0x5dad417a, SUC_lexicon_tags_31 },
    { 0x1e6d417b, SUC_lexicon_tags_33 },
    { 0x5ecd417c, SUC_lexicon_tags_9 },
    { 0x51cd417d, SUC_lexicon_tags_17 },
    { 0xd16d417c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0d4182, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x148d4184, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8d4188, SUC_lexicon_tags_27 },
    { 0xa38d4188, SUC_lexicon_tags_25 },
    { 0x26cd418a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf18d418c, SUC_lexicon_tags_7 },
    { 0x98cd418d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd3cd418f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x634d4192, SUC_lexicon_tags_12 },
    { 0xa30d4192, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c0d4195, SUC_lexicon_tags_48 },
    { 0x824d4196, SUC_lexicon_tags_63 },
    { 0x768d4196, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed419e, SUC_lexicon_tags_38 },
    { 0xd70d419e, SUC_lexicon_tags_31 },
    { 0x1b6d419f, SUC_lexicon_tags_39 },
    { 0x742d41a1, SUC_lexicon_tags_16 },
    { 0xed6d419f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cd41a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16d41ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16d41ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ced41b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28d41bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x656d41bd, SUC_lexicon_tags_17 },
    { 0x51ed41be, SUC_lexicon_tags_14 },
    { 0x3f2d41bd, SUC_lexicon_tags_37 },
    { 0x868d41c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc5cd41c2, SUC_lexicon_tags_37 },
    { 0x6e8d41c3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320d41c7, SUC_lexicon_tags_11 },
    { 0x736d41c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56d41cc, SUC_lexicon_tags_53 },
    { 0x29ed41cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746d41d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x982d41d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc6d41d7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4d41da, SUC_lexicon_tags_37 },
    { 0x1d8d41db, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xed6d41dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x720d41df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9bed41e1, SUC_lexicon_tags_39 },
    { 0x58ad41e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaed41e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ad41e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed41eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd16d41ed, SUC_lexicon_tags_33 },
    { 0x3fcd41ed, SUC_lexicon_tags_32 },
    { 0xa9ad41ee, SUC_lexicon_tags_25 },
    { 0x7a4d41f0, SUC_lexicon_tags_15 },
    { 0x6bcd41ef, SUC_lexicon_tags_49 },
    { 0xef4d41ed, SUC_lexicon_tags_8 },
    { 0x68cd41f3, SUC_lexicon_tags_15 },
    { 0x9f8d41ee, SUC_lexicon_tags_2 },
    { 0x26ad41f0, SUC_lexicon_tags_15 },
    { 0xbe0d41f6, SUC_lexicon_tags_46 },
    { 0xa20d41f6, SUC_lexicon_tags_28 },
    { 0x9a0d41f8, SUC_lexicon_tags_12 },
    { 0xa4ed41f9, SUC_lexicon_tags_25 },
    { 0x92cd41fa, SUC_lexicon_tags_26 },
    { 0x8aed41fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9ed41fd, SUC_lexicon_tags_37 },
    { 0x6dad41fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x418d4201, SUC_lexicon_tags_25 },
    { 0x292d4202, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d420a, SUC_lexicon_tags_7 },
    { 0x3e4d420b, SUC_lexicon_tags_6 },
    { 0x67ed420c, SUC_lexicon_tags_44 },
    { 0x5fcd420b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xba0d420f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64d4214, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x400d4216, SUC_lexicon_tags_17 },
    { 0x946d4217, SUC_lexicon_tags_15 },
    { 0xe56d4217, SUC_lexicon_tags_2 },
    { 0x72ed4217, SUC_lexicon_tags_25 },
    { 0x888d4216, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b2d421c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aed4220, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64d4227, SUC_lexicon_tags_37 },
    { 0x5fcd4227, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd6cd422a, SUC_lexicon_tags_9 },
    { 0x166d422b, SUC_lexicon_tags_46 },
    { 0xdd0d422c, SUC_lexicon_tags_2 },
    { 0x45ad422c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad4233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87cd4235, SUC_lexicon_tags_47 },
    { 0x576d4236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688d423c, SUC_lexicon_tags_45 },
    { 0x586d423d, SUC_lexicon_tags_25 },
    { 0x83cd423e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dad4244, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbb8d4246, SUC_lexicon_tags_56 },
    { 0x5ded4246, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x788d424a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaed4250, SUC_lexicon_tags_19 },
    { 0x45cd4251, SUC_lexicon_tags_29 },
    { 0xe58d4250, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd34d4254, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448d425b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd44d425d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ad4260, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ad4263, SUC_lexicon_tags_27 },
    { 0x1c6d4263, SUC_lexicon_tags_33 },
    { 0xc5ad4265, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978d4269, SUC_lexicon_tags_25 },
    { 0xd7cd426a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fed426f, SUC_lexicon_tags_25 },
    { 0x370d426f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284d4275, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f6d4277, SUC_lexicon_tags_17 },
    { 0xb0ed4277, SUC_lexicon_tags_37 },
    { 0xe8ad4279, SUC_lexicon_tags_25 },
    { 0xabad427a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc40d427c, SUC_lexicon_tags_1 },
    { 0xdf4d427c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6d4280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddcd4282, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ded4284, SUC_lexicon_tags_29 },
    { 0x55ad4285, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6dad4287, SUC_lexicon_tags_15 },
    { 0x2f0d4288, SUC_lexicon_tags_38 },
    { 0x8cd4289, SUC_lexicon_tags_14 },
    { 0xecad4289, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcecd428c, SUC_lexicon_tags_14 },
    { 0xa14d428d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e2d428f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6e0d4291, SUC_lexicon_tags_37 },
    { 0x6aed4292, SUC_lexicon_tags_45 },
    { 0x6b4d4292, SUC_lexicon_tags_63 },
    { 0x792d4292, SUC_lexicon_tags_7 },
    { 0xf26d4295, SUC_lexicon_tags_17 },
    { 0xadcd4296, SUC_lexicon_tags_15 },
    { 0x368d4296, SUC_lexicon_tags_25 },
    { 0x570d4297, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496d429e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0d42a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2d42a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ad42aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4d42ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x198d42b1, SUC_lexicon_tags_14 },
    { 0xf2cd42b2, SUC_lexicon_tags_46 },
    { 0x95ed42b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x940d42b5, SUC_lexicon_tags_25 },
    { 0x744d42b6, SUC_lexicon_tags_25 },
    { 0xa96d42b5, SUC_lexicon_tags_8 },
    { 0x41ad42b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4d42bd, SUC_lexicon_tags_364 },
    { 0xfb4d42bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4a8d42c0, SUC_lexicon_tags_21 },
    { 0xab4d42c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ad42c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x258d42c5, SUC_lexicon_tags_45 },
    { 0xab4d42c6, SUC_lexicon_tags_2 },
    { 0x48d42c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbed42cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d42d1, SUC_lexicon_tags_6 },
    { 0xc86d42d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d42d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652d42da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6d42de, SUC_lexicon_tags_8 },
    { 0x708d42df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc02d42e1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ad42e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4dad42ea, SUC_lexicon_tags_37 },
    { 0x738d42eb, SUC_lexicon_tags_63 },
    { 0x21ad42ea, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388d42f2, SUC_lexicon_tags_4 },
    { 0x3b6d42f3, SUC_lexicon_tags_35 },
    { 0x886d42f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x762d42f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbed42f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaa6d42fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x846d42fe, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaacd4300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1acd4302, SUC_lexicon_tags_1 },
    { 0x228d4302, SUC_lexicon_tags_1 },
    { 0x288d4304, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8d430a, SUC_lexicon_tags_32 },
    { 0x4ed430b, SUC_lexicon_tags_25 },
    { 0xb4d430b, SUC_lexicon_tags_9 },
    { 0x52ed430d, SUC_lexicon_tags_6 },
    { 0x5eed430c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46d4313, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x142d4316, SUC_lexicon_tags_17 },
    { 0x594d4317, SUC_lexicon_tags_7 },
    { 0xf2cd4318, SUC_lexicon_tags_179 },
    { 0xc8ad4319, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fad431e, SUC_lexicon_tags_17 },
    { 0xead431e, SUC_lexicon_tags_29 },
    { 0xecd4320, SUC_lexicon_tags_23 },
    { 0xadcd4320, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0d4329, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x732d432b, SUC_lexicon_tags_25 },
    { 0x6ccd432c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x82ad432e, SUC_lexicon_tags_16 },
    { 0xfb2d432e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ad4334, SUC_lexicon_tags_1 },
    { 0xb42d4335, SUC_lexicon_tags_32 },
    { 0xecd4335, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67cd4338, SUC_lexicon_tags_7 },
    { 0x96d4339, SUC_lexicon_tags_9 },
    { 0xf7ad433a, SUC_lexicon_tags_12 },
    { 0x460d4338, SUC_lexicon_tags_17 },
    { 0xd22d4338, SUC_lexicon_tags_56 },
    { 0xd78d433b, SUC_lexicon_tags_75 },
    { 0xd22d433e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf7ed4340, SUC_lexicon_tags_1 },
    { 0xffad4340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda0d4343, SUC_lexicon_tags_9 },
    { 0xe48d4344, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2d434b, SUC_lexicon_tags_37 },
    { 0xbb0d434c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf30d434e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf4d4350, SUC_lexicon_tags_37 },
    { 0xfa8d4351, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ead4356, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598d435b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dad4362, SUC_lexicon_tags_38 },
    { 0xbd6d4363, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d4366, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x924d4369, SUC_lexicon_tags_37 },
    { 0x24cd436a, SUC_lexicon_tags_19 },
    { 0xa9cd4369, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3dcd436d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30ad436f, SUC_lexicon_tags_17 },
    { 0xdc0d4370, SUC_lexicon_tags_1 },
    { 0x146d4371, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x686d4374, SUC_lexicon_tags_43 },
    { 0xc00d4375, SUC_lexicon_tags_68 },
    { 0xda0d4374, SUC_lexicon_tags_37 },
    { 0xd5ad4376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d437f, SUC_lexicon_tags_23 },
    { 0x12ad4380, SUC_lexicon_tags_27 },
    { 0xdcad4380, SUC_lexicon_tags_0 },
    { 0xc1ad4382, SUC_lexicon_tags_60 },
    { 0x522d437f, SUC_lexicon_tags_7 },
    { 0x98d4380, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x75cd4386, SUC_lexicon_tags_37 },
    { 0x6fcd4387, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaed438d, SUC_lexicon_tags_4 },
    { 0xe42d438e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92d4392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x274d4395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ead4398, SUC_lexicon_tags_28 },
    { 0xcf4d4399, SUC_lexicon_tags_37 },
    { 0xa30d439a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8d439e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d43a3, SUC_lexicon_tags_42 },
    { 0xaf6d43a4, SUC_lexicon_tags_8 },
    { 0xdcad43a5, SUC_lexicon_tags_6 },
    { 0x3dcd43a6, SUC_lexicon_tags_14 },
    { 0x73ed43a7, SUC_lexicon_tags_49 },
    { 0x994d43a7, SUC_lexicon_tags_25 },
    { 0xa86d43a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6d43b0, SUC_lexicon_tags_4 },
    { 0x16ed43b1, SUC_lexicon_tags_15 },
    { 0xad8d43b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc0cd43b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x622d43b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x662d43ba, SUC_lexicon_tags_27 },
    { 0xd54d43ba, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670d43c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d43c4, SUC_lexicon_tags_25 },
    { 0xcded43c4, SUC_lexicon_tags_46 },
    { 0xcded43c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xddad43c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x586d43ca, SUC_lexicon_tags_7 },
    { 0x3e6d43cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308d43cf, SUC_lexicon_tags_11 },
    { 0x22ed43d0, SUC_lexicon_tags_13 },
    { 0xf2ad43d0, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40d43d5, SUC_lexicon_tags_9 },
    { 0xd52d43d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76cd43d8, SUC_lexicon_tags_9 },
    { 0xf5ad43d9, SUC_lexicon_tags_45 },
    { 0x846d43d9, SUC_lexicon_tags_29 },
    { 0xf0cd43db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84d43e7, SUC_lexicon_tags_31 },
    { 0x166d43e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ed43eb, SUC_lexicon_tags_37 },
    { 0xdfed43eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2d43f1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2d43f4, SUC_lexicon_tags_44 },
    { 0xb8cd43f4, SUC_lexicon_tags_45 },
    { 0xfecd43f6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766d43fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f0d43fc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe9ed43fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xef4d4400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318d4404, SUC_lexicon_tags_27 },
    { 0x49cd4404, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ad4409, SUC_lexicon_tags_11 },
    { 0x7d8d4409, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2d440d, SUC_lexicon_tags_31 },
    { 0x928d440e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0d4412, SUC_lexicon_tags_131 },
    { 0x64d4412, SUC_lexicon_tags_39 },
    { 0x566d4414, SUC_lexicon_tags_10 },
    { 0x5a8d4415, SUC_lexicon_tags_28 },
    { 0x5b4d4415, SUC_lexicon_tags_15 },
    { 0x7bad4417, SUC_lexicon_tags_39 },
    { 0x196d4417, SUC_lexicon_tags_29 },
    { 0xb1ad4418, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4d441c, SUC_lexicon_tags_37 },
    { 0x4ad441d, SUC_lexicon_tags_25 },
    { 0x6a0d441e, SUC_lexicon_tags_48 },
    { 0xef6d441f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d4424, SUC_lexicon_tags_32 },
    { 0x4c6d4425, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xabad4428, SUC_lexicon_tags_37 },
    { 0xdbcd4429, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x78ed442b, SUC_lexicon_tags_37 },
    { 0x3f6d442c, SUC_lexicon_tags_930 },
    { 0xad2d442c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2d4430, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63cd4432, SUC_lexicon_tags_39 },
    { 0xac0d4432, SUC_lexicon_tags_1 },
    { 0x72cd4434, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd4438, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928d443c, SUC_lexicon_tags_28 },
    { 0x3f0d443d, SUC_lexicon_tags_25 },
    { 0xec6d443e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ad4441, SUC_lexicon_tags_14 },
    { 0xbfcd4442, SUC_lexicon_tags_92 },
    { 0x662d4441, SUC_lexicon_tags_15 },
    { 0xd6cd4444, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70cd4446, SUC_lexicon_tags_7 },
    { 0xe22d4447, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdead444a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ad444f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x772d4451, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x394d4453, SUC_lexicon_tags_6 },
    { 0xa9ad4453, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x534d4456, SUC_lexicon_tags_31 },
    { 0x704d4457, SUC_lexicon_tags_37 },
    { 0xe7ad4456, SUC_lexicon_tags_25 },
    { 0x236d4459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00d445c, SUC_lexicon_tags_15 },
    { 0xdaed445d, SUC_lexicon_tags_26 },
    { 0x46d445e, SUC_lexicon_tags_822 },
    { 0x662d445f, SUC_lexicon_tags_82 },
    { 0xfc8d445d, SUC_lexicon_tags_33 },
    { 0xf50d445c, SUC_lexicon_tags_4 },
    { 0xabcd445f, SUC_lexicon_tags_37 },
    { 0x3c0d4463, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444d4469, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e2d446b, SUC_lexicon_tags_70 },
    { 0x98d446c, SUC_lexicon_tags_9 },
    { 0x4a0d446d, SUC_lexicon_tags_43 },
    { 0x77cd446c, SUC_lexicon_tags_25 },
    { 0xb96d446f, SUC_lexicon_tags_23 },
    { 0x99cd4470, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cd4474, SUC_lexicon_tags_77 },
    { 0x370d4475, SUC_lexicon_tags_47 },
    { 0xd0d4475, SUC_lexicon_tags_12 },
    { 0x1e0d4477, SUC_lexicon_tags_25 },
    { 0x182d4478, SUC_lexicon_tags_12 },
    { 0x5b0d4479, SUC_lexicon_tags_38 },
    { 0x82ad4476, SUC_lexicon_tags_25 },
    { 0xfd2d4479, SUC_lexicon_tags_25 },
    { 0xbfad447c, SUC_lexicon_tags_12 },
    { 0x31ed4479, SUC_lexicon_tags_23 },
    { 0x81ed447e, SUC_lexicon_tags_37 },
    { 0xd08d447a, SUC_lexicon_tags_8 },
    { 0xb16d4475, SUC_lexicon_tags_31 },
    { 0xaded4474, SUC_lexicon_tags_2 },
    { 0xb08d4482, SUC_lexicon_tags_47 },
    { 0x70ed4483, SUC_lexicon_tags_25 },
    { 0x18cd447b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8d4488, SUC_lexicon_tags_38 },
    { 0x428d4488, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ad448c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a6d448e, SUC_lexicon_tags_15 },
    { 0xec4d448f, SUC_lexicon_tags_33 },
    { 0x9fed4490, SUC_lexicon_tags_31 },
    { 0x4a6d4490, SUC_lexicon_tags_22 },
    { 0x724d4490, SUC_lexicon_tags_30 },
    { 0x994d448e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cd4499, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50d449e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36d44a3, SUC_lexicon_tags_25 },
    { 0x854d44a4, SUC_lexicon_tags_23 },
    { 0x7e6d44a5, SUC_lexicon_tags_33 },
    { 0x226d44a4, SUC_lexicon_tags_61 },
    { 0xd90d44a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1ad44a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x690d44ab, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecd44b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702d44b5, SUC_lexicon_tags_25 },
    { 0x102d44b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd2d44b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3acd44bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfbed44bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6d44c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7cd44c3, SUC_lexicon_tags_30 },
    { 0xb60d44c4, SUC_lexicon_tags_1 },
    { 0xc72d44c5, SUC_lexicon_tags_30 },
    { 0x92ad44c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x656d44c8, SUC_lexicon_tags_37 },
    { 0x3a2d44c8, SUC_lexicon_tags_12 },
    { 0x40ed44c8, SUC_lexicon_tags_38 },
    { 0x9a8d44cb, SUC_lexicon_tags_17 },
    { 0xf4ad44cb, SUC_lexicon_tags_25 },
    { 0x872d44cd, SUC_lexicon_tags_60 },
    { 0x6a2d44ce, SUC_lexicon_tags_28 },
    { 0x426d44ce, SUC_lexicon_tags_9 },
    { 0x584d44ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608d44d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ead44d6, SUC_lexicon_tags_28 },
    { 0x7c4d44d7, SUC_lexicon_tags_60 },
    { 0xe04d44d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ed44db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4d44e0, SUC_lexicon_tags_27 },
    { 0xe98d44e0, SUC_lexicon_tags_37 },
    { 0x580d44e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4d44ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ad44f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x970d44f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cd44f9, SUC_lexicon_tags_1 },
    { 0xd9ad44fa, SUC_lexicon_tags_15 },
    { 0x500d44fa, SUC_lexicon_tags_51 },
    { 0x3b4d44fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8acd44fe, SUC_lexicon_tags_1 },
    { 0xe14d44ff, SUC_lexicon_tags_37 },
    { 0x360d4500, SUC_lexicon_tags_15 },
    { 0x714d4501, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecd4508, SUC_lexicon_tags_63 },
    { 0xe92d4509, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x3e8d450b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc26d450d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfe8d450f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cd4513, SUC_lexicon_tags_7 },
    { 0x632d4514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2d4518, SUC_lexicon_tags_14 },
    { 0x49ad4519, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106d451e, SUC_lexicon_tags_2 },
    { 0x138d451e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2d452a, SUC_lexicon_tags_15 },
    { 0xdf0d452b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50cd452d, SUC_lexicon_tags_37 },
    { 0x170d452e, SUC_lexicon_tags_63 },
    { 0x1ed452e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x618d4532, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd453c, SUC_lexicon_tags_2 },
    { 0xa30d453d, SUC_lexicon_tags_28 },
    { 0xc20d453d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374d4546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad454b, SUC_lexicon_tags_7 },
    { 0x226d454c, SUC_lexicon_tags_1 },
    { 0xd3cd454c, SUC_lexicon_tags_25 },
    { 0x7a8d454c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54d4553, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2d4d4555, SUC_lexicon_tags_31 },
    { 0xf20d4555, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d455a, SUC_lexicon_tags_17 },
    { 0xba0d455a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726d455f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ced4562, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ed4568, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc18d456a, SUC_lexicon_tags_87 },
    { 0x7a4d456b, SUC_lexicon_tags_27 },
    { 0x848d456a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cd456f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb5ad4571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63ed4573, SUC_lexicon_tags_26 },
    { 0x650d4573, SUC_lexicon_tags_7 },
    { 0x718d4575, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0d457c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46d4580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55ad4582, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dad458a, SUC_lexicon_tags_31 },
    { 0x552d458b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36d458e, SUC_lexicon_tags_6 },
    { 0xaf8d458f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92d4594, SUC_lexicon_tags_7 },
    { 0xdaad4595, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fad4599, SUC_lexicon_tags_12 },
    { 0xc1cd459a, SUC_lexicon_tags_11 },
    { 0xea6d459b, SUC_lexicon_tags_7 },
    { 0x166d459c, SUC_lexicon_tags_37 },
    { 0xe0ed459b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34d45a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x572d45a2, SUC_lexicon_tags_1 },
    { 0x758d45a2, SUC_lexicon_tags_84 },
    { 0x156d45a4, SUC_lexicon_tags_25 },
    { 0xda8d45a5, SUC_lexicon_tags_7 },
    { 0x9f0d45a5, SUC_lexicon_tags_0 },
    { 0x64ed45a7, SUC_lexicon_tags_11 },
    { 0xe2ed45a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e0d45aa, SUC_lexicon_tags_45 },
    { 0x58ed45aa, SUC_lexicon_tags_87 },
    { 0xf94d45ab, SUC_lexicon_tags_37 },
    { 0x586d45ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f2d45af, SUC_lexicon_tags_1 },
    { 0x454d45b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad45b3, SUC_lexicon_tags_25 },
    { 0xa74d45b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe4ed45b6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x68d45b8, SUC_lexicon_tags_25 },
    { 0x5c0d45b8, SUC_lexicon_tags_37 },
    { 0xb1ad45ba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xfe0d45bc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0d45bf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcd45c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x414d45c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478d45ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d0d45cc, SUC_lexicon_tags_65 },
    { 0x56ed45cc, SUC_lexicon_tags_57 },
    { 0x896d45ce, SUC_lexicon_tags_10 },
    { 0x554d45cf, SUC_lexicon_tags_10 },
    { 0xe10d45ce, SUC_lexicon_tags_53 },
    { 0x588d45d1, SUC_lexicon_tags_9 },
    { 0x27cd45d2, SUC_lexicon_tags_49 },
    { 0x570d45d3, SUC_lexicon_tags_11 },
    { 0x39ed45d4, SUC_lexicon_tags_1 },
    { 0xd92d45d5, SUC_lexicon_tags_17 },
    { 0xa8d45d4, SUC_lexicon_tags_45 },
    { 0xafed45d7, SUC_lexicon_tags_15 },
    { 0x28d45d6, SUC_lexicon_tags_17 },
    { 0xe70d45d9, SUC_lexicon_tags_3 },
    { 0x85cd45da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274d45e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d45e7, SUC_lexicon_tags_0 },
    { 0xcecd45e8, SUC_lexicon_tags_15 },
    { 0x76d45e8, SUC_lexicon_tags_47 },
    { 0xb5ed45e7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ed45ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfd0d45ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x538d45f1, SUC_lexicon_tags_47 },
    { 0x398d45f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4d45f7, SUC_lexicon_tags_15 },
    { 0xe52d45f8, SUC_lexicon_tags_22 },
    { 0x2dad45f8, SUC_lexicon_tags_30 },
    { 0x234d45fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ed45fd, SUC_lexicon_tags_16 },
    { 0x20cd45fe, SUC_lexicon_tags_47 },
    { 0x9acd45ff, SUC_lexicon_tags_34 },
    { 0xdf0d4600, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x190d4602, SUC_lexicon_tags_12 },
    { 0x9a2d4603, SUC_lexicon_tags_63 },
    { 0xe4ed4602, SUC_lexicon_tags_2 },
    { 0x23ad4605, SUC_lexicon_tags_1 },
    { 0xb58d4606, SUC_lexicon_tags_12 },
    { 0x6b8d4607, SUC_lexicon_tags_19 },
    { 0x6aad4608, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52d460b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e4d460d, SUC_lexicon_tags_19 },
    { 0x362d460e, SUC_lexicon_tags_12 },
    { 0x66cd460e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cd4615, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x406d4617, SUC_lexicon_tags_37 },
    { 0x46ed4618, SUC_lexicon_tags_12 },
    { 0x8e6d4619, SUC_lexicon_tags_45 },
    { 0xf84d4618, SUC_lexicon_tags_66 },
    { 0xb1cd4618, SUC_lexicon_tags_45 },
    { 0x344d4619, SUC_lexicon_tags_25 },
    { 0xb60d461d, SUC_lexicon_tags_11 },
    { 0xfc0d461d, SUC_lexicon_tags_37 },
    { 0xb88d461e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x308d4621, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x586d4623, SUC_lexicon_tags_37 },
    { 0x194d4624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x70d4626, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d4629, SUC_lexicon_tags_7 },
    { 0xe42d462a, SUC_lexicon_tags_39 },
    { 0x21ad4629, SUC_lexicon_tags_97 },
    { 0x2f4d462b, SUC_lexicon_tags_6 },
    { 0x56ed462d, SUC_lexicon_tags_4 },
    { 0xc48d462e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26cd4630, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd26d4632, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeded4634, SUC_lexicon_tags_30 },
    { 0xcd4d4635, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ced4639, SUC_lexicon_tags_37 },
    { 0x1e2d4639, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5ed463c, SUC_lexicon_tags_0 },
    { 0xa78d463d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ed4641, SUC_lexicon_tags_15 },
    { 0x8b4d4642, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17ed4644, SUC_lexicon_tags_2 },
    { 0xa76d4644, SUC_lexicon_tags_30 },
    { 0x294d4646, SUC_lexicon_tags_37 },
    { 0x164d4646, SUC_lexicon_tags_37 },
    { 0xcecd4644, SUC_lexicon_tags_54 },
    { 0x488d4649, SUC_lexicon_tags_108 },
    { 0xa64d464a, SUC_lexicon_tags_34 },
    { 0xb68d464b, SUC_lexicon_tags_27 },
    { 0x48d464c, SUC_lexicon_tags_15 },
    { 0xa34d464d, SUC_lexicon_tags_0 },
    { 0x5ced464e, SUC_lexicon_tags_37 },
    { 0x772d464e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb92d4651, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa3ed4653, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x596d4656, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecad4663, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36d4666, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210d466b, SUC_lexicon_tags_10 },
    { 0xca8d466c, SUC_lexicon_tags_27 },
    { 0xf0d466d, SUC_lexicon_tags_39 },
    { 0x78ed466d, SUC_lexicon_tags_5 },
    { 0x80ed466b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f0d4671, SUC_lexicon_tags_1 },
    { 0x41ed4672, SUC_lexicon_tags_25 },
    { 0x292d4672, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x974d4675, SUC_lexicon_tags_15 },
    { 0xe26d4676, SUC_lexicon_tags_2 },
    { 0xec6d4675, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74d467a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a2d467c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c0d467e, SUC_lexicon_tags_14 },
    { 0xc4ed467e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x452d4682, SUC_lexicon_tags_46 },
    { 0x9e6d4683, SUC_lexicon_tags_27 },
    { 0xf12d4684, SUC_lexicon_tags_17 },
    { 0x21ed4685, SUC_lexicon_tags_15 },
    { 0xc20d4682, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x834d4688, SUC_lexicon_tags_37 },
    { 0x984d4689, SUC_lexicon_tags_19 },
    { 0x116d4689, SUC_lexicon_tags_23 },
    { 0xd7ed4688, SUC_lexicon_tags_37 },
    { 0xf36d468b, SUC_lexicon_tags_7 },
    { 0x1f6d468c, SUC_lexicon_tags_49 },
    { 0x1cd468e, SUC_lexicon_tags_7 },
    { 0xf10d468f, SUC_lexicon_tags_27 },
    { 0x352d4690, SUC_lexicon_tags_23 },
    { 0x8d6d4691, SUC_lexicon_tags_12 },
    { 0x57ed4690, SUC_lexicon_tags_23 },
    { 0x848d468e, SUC_lexicon_tags_49 },
    { 0x80ad4692, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776d4698, SUC_lexicon_tags_44 },
    { 0x83ed4698, SUC_lexicon_tags_8 },
    { 0x866d469a, SUC_lexicon_tags_60 },
    { 0x6c2d4699, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cd469e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44d46a2, SUC_lexicon_tags_37 },
    { 0x1b6d46a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cd46a6, SUC_lexicon_tags_14 },
    { 0x58ed46a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46ab, SUC_lexicon_tags_23 },
    { 0x1fed46ab, SUC_lexicon_tags_31 },
    { 0x658d46ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352d46b2, SUC_lexicon_tags_0 },
    { 0x296d46b3, SUC_lexicon_tags_1 },
    { 0xdc4d46b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x788d46b6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4d46ba, SUC_lexicon_tags_39 },
    { 0x25cd46bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x692d46be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x730d46c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2d46c4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cd46ca, SUC_lexicon_tags_37 },
    { 0x4b4d46cb, SUC_lexicon_tags_11 },
    { 0xb36d46cc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cd46d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bed46d2, SUC_lexicon_tags_45 },
    { 0x5b8d46d3, SUC_lexicon_tags_6 },
    { 0xcb2d46d4, SUC_lexicon_tags_2 },
    { 0xe02d46d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700d46dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe66d46de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4d46e2, SUC_lexicon_tags_79 },
    { 0x55ad46e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccd46e7, SUC_lexicon_tags_65 },
    { 0x636d46e8, SUC_lexicon_tags_9 },
    { 0xcaed46e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad46ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0d46f5, SUC_lexicon_tags_43 },
    { 0x4d2d46f6, SUC_lexicon_tags_70 },
    { 0x3a4d46f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cd46fb, SUC_lexicon_tags_37 },
    { 0x58cd46fb, SUC_lexicon_tags_37 },
    { 0x1e4d46fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162d4704, SUC_lexicon_tags_27 },
    { 0xe0ed4705, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568d470b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2d4711, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeb0d4713, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630d4717, SUC_lexicon_tags_80 },
    { 0x80ed4718, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3d8d471a, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cd4721, SUC_lexicon_tags_2 },
    { 0xbf2d4722, SUC_lexicon_tags_47 },
    { 0x274d4723, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0d4726, SUC_lexicon_tags_47 },
    { 0x504d4726, SUC_lexicon_tags_224 },
    { 0xa06d4726, SUC_lexicon_tags_43 },
    { 0x31ed4729, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ed472d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdf0d472f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ed4735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0d4738, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccad473b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfb2d473d, SUC_lexicon_tags_28 },
    { 0xc0ad473d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3cad4740, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ed4743, SUC_lexicon_tags_60 },
    { 0x9ad4744, SUC_lexicon_tags_11 },
    { 0x3b2d4743, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fed4749, SUC_lexicon_tags_37 },
    { 0x55ed474a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ad474d, SUC_lexicon_tags_12 },
    { 0x5acd474e, SUC_lexicon_tags_359 },
    { 0xa62d474e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ed4752, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cd4756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf2d4758, SUC_lexicon_tags_14 },
    { 0x594d4759, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acd475d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe02d4760, SUC_lexicon_tags_15 },
    { 0xc66d4761, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x868d4764, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14d4767, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89cd476a, SUC_lexicon_tags_37 },
    { 0xa2ed476b, SUC_lexicon_tags_14 },
    { 0xf70d476b, SUC_lexicon_tags_26 },
    { 0xc9ed476d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ad4770, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3ed4772, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cad4775, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x42ad4777, SUC_lexicon_tags_19 },
    { 0x738d4777, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ed477d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ed4781, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x632d4784, SUC_lexicon_tags_23 },
    { 0xb7cd4784, SUC_lexicon_tags_23 },
    { 0xe0d4786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68d478a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60d478d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ed4794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf6d4796, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cd479e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x278d47a0, SUC_lexicon_tags_12 },
    { 0xe06d47a0, SUC_lexicon_tags_23 },
    { 0xd12d47a2, SUC_lexicon_tags_40 },
    { 0x2b6d47a3, SUC_lexicon_tags_30 },
    { 0xf50d47a4, SUC_lexicon_tags_1 },
    { 0xfacd47a5, SUC_lexicon_tags_341 },
    { 0x774d47a6, SUC_lexicon_tags_27 },
    { 0x8ced47a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6d47aa, SUC_lexicon_tags_12 },
    { 0x602d47aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668d47b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6d47b5, SUC_lexicon_tags_2 },
    { 0x8ecd47b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2d47b9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x24ed47bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf80d47bd, SUC_lexicon_tags_37 },
    { 0x876d47be, SUC_lexicon_tags_11 },
    { 0x4aad47bd, SUC_lexicon_tags_43 },
    { 0x674d47bf, SUC_lexicon_tags_25 },
    { 0x7b2d47c1, SUC_lexicon_tags_37 },
    { 0x154d47c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ed47c5, SUC_lexicon_tags_59 },
    { 0xf26d47c5, SUC_lexicon_tags_21 },
    { 0x528d47c7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540d47cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe26d47d1, SUC_lexicon_tags_84 },
    { 0x29cd47d2, SUC_lexicon_tags_37 },
    { 0xcc0d47d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cd47d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad47dc, SUC_lexicon_tags_2 },
    { 0x11ad47dc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xeced47df, SUC_lexicon_tags_45 },
    { 0xd2ed47e0, SUC_lexicon_tags_151 },
    { 0x798d47e0, SUC_lexicon_tags_12 },
    { 0x1aad47e1, SUC_lexicon_tags_7 },
    { 0xe2d47e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b2d47e5, SUC_lexicon_tags_25 },
    { 0x46ed47e5, SUC_lexicon_tags_32 },
    { 0x614d47e7, SUC_lexicon_tags_25 },
    { 0x934d47e8, SUC_lexicon_tags_7 },
    { 0x5bed47e7, SUC_lexicon_tags_25 },
    { 0xef2d47e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86d47ed, SUC_lexicon_tags_14 },
    { 0xb6ad47ee, SUC_lexicon_tags_12 },
    { 0x39cd47ef, SUC_lexicon_tags_17 },
    { 0xa7ed47f0, SUC_lexicon_tags_7 },
    { 0x560d47ed, SUC_lexicon_tags_7 },
    { 0xadcd47f2, SUC_lexicon_tags_17 },
    { 0x718d47ef, SUC_lexicon_tags_2 },
    { 0xc32d47f1, SUC_lexicon_tags_15 },
    { 0xa5cd47f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x464d47f8, SUC_lexicon_tags_47 },
    { 0xfaed47f9, SUC_lexicon_tags_17 },
    { 0x68ed47fa, SUC_lexicon_tags_14 },
    { 0x3e0d47fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7dcd47fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x264d4800, SUC_lexicon_tags_63 },
    { 0xc24d4800, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xcfed4803, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe22d4805, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ad4808, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98d480a, SUC_lexicon_tags_37 },
    { 0x704d480b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x442d480e, SUC_lexicon_tags_11 },
    { 0x620d480e, SUC_lexicon_tags_33 },
    { 0x290d480e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xda2d4812, SUC_lexicon_tags_25 },
    { 0xe0cd4813, SUC_lexicon_tags_6 },
    { 0xfb4d4814, SUC_lexicon_tags_2 },
    { 0xc76d4814, SUC_lexicon_tags_17 },
    { 0xb4d4816, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954d481a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e4d481c, SUC_lexicon_tags_12 },
    { 0x48cd481d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48d4820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x870d4823, SUC_lexicon_tags_65 },
    { 0xd14d4823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a2d4826, SUC_lexicon_tags_37 },
    { 0xfced4827, SUC_lexicon_tags_12 },
    { 0xf2ed4828, SUC_lexicon_tags_16 },
    { 0xe74d4829, SUC_lexicon_tags_37 },
    { 0x594d482a, SUC_lexicon_tags_684 },
    { 0xe4cd482a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46d482f, SUC_lexicon_tags_43 },
    { 0xcd4d4830, SUC_lexicon_tags_18 },
    { 0x766d4831, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0d4837, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72d483e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d4841, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ad4847, SUC_lexicon_tags_42 },
    { 0x264d4848, SUC_lexicon_tags_37 },
    { 0x978d4849, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528d4853, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb2d4855, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3ad4857, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716d485e, SUC_lexicon_tags_1 },
    { 0x872d485f, SUC_lexicon_tags_17 },
    { 0xd32d485f, SUC_lexicon_tags_38 },
    { 0xcacd4861, SUC_lexicon_tags_28 },
    { 0x97ed4862, SUC_lexicon_tags_17 },
    { 0x8e0d4862, SUC_lexicon_tags_2 },
    { 0x9e0d4861, SUC_lexicon_tags_37 },
    { 0xbbcd4865, SUC_lexicon_tags_45 },
    { 0xaad4865, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0d4869, SUC_lexicon_tags_38 },
    { 0x404d4869, SUC_lexicon_tags_11 },
    { 0xafcd486b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcd486e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x640d4870, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8d4874, SUC_lexicon_tags_45 },
    { 0xbc6d4875, SUC_lexicon_tags_12 },
    { 0x462d4876, SUC_lexicon_tags_1 },
    { 0x9d6d4875, SUC_lexicon_tags_14 },
    { 0x1a4d4874, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404d4880, SUC_lexicon_tags_25 },
    { 0x24d4880, SUC_lexicon_tags_2 },
    { 0x15ad4882, SUC_lexicon_tags_38 },
    { 0xef4d4882, SUC_lexicon_tags_15 },
    { 0xb4d4883, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4d488c, SUC_lexicon_tags_26 },
    { 0x10ed488c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdeed488f, SUC_lexicon_tags_45 },
    { 0x91ed4890, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00d4894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0d4897, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96d489d, SUC_lexicon_tags_19 },
    { 0x61ed489e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x740d48a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cd48a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202d48a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8d48ae, SUC_lexicon_tags_17 },
    { 0x9ded48ae, SUC_lexicon_tags_12 },
    { 0x8d4d48af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5fcd48b2, SUC_lexicon_tags_25 },
    { 0x8b0d48b3, SUC_lexicon_tags_15 },
    { 0xa0ed48b4, SUC_lexicon_tags_111 },
    { 0x3a4d48b5, SUC_lexicon_tags_45 },
    { 0x254d48b5, SUC_lexicon_tags_15 },
    { 0x690d48b3, SUC_lexicon_tags_27 },
    { 0x36d48b8, SUC_lexicon_tags_35 },
    { 0xd08d48b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b2d48bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0d48bf, SUC_lexicon_tags_37 },
    { 0x388d48c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ad48c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed48cc, SUC_lexicon_tags_162 },
    { 0xda6d48cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed48d1, SUC_lexicon_tags_17 },
    { 0xc30d48d2, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x4fcd48d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc50d48d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86d48da, SUC_lexicon_tags_28 },
    { 0xaa6d48db, SUC_lexicon_tags_63 },
    { 0x34ed48dc, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48e1, SUC_lexicon_tags_37 },
    { 0xb54d48e1, SUC_lexicon_tags_80 },
    { 0xb06d48e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a8d48e5, SUC_lexicon_tags_14 },
    { 0xf4ed48e5, SUC_lexicon_tags_12 },
    { 0xdded48e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x13ad48e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eed48ed, SUC_lexicon_tags_47 },
    { 0x6ccd48ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ad48f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x598d48f7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2d48fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x296d48fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x928d4901, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfed4905, SUC_lexicon_tags_63 },
    { 0xd34d4906, SUC_lexicon_tags_8 },
    { 0xfa4d4907, SUC_lexicon_tags_1 },
    { 0x460d4908, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d490d, SUC_lexicon_tags_23 },
    { 0x5b4d490d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8d4911, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aed4916, SUC_lexicon_tags_11 },
    { 0xa44d4916, SUC_lexicon_tags_29 },
    { 0x42cd4917, SUC_lexicon_tags_2 },
    { 0xd58d4918, SUC_lexicon_tags_12 },
    { 0x8c2d4917, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4a2d491c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfa6d491e, SUC_lexicon_tags_38 },
    { 0x5fcd491f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cd4922, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122d4928, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ed492c, SUC_lexicon_tags_6 },
    { 0xff4d492d, SUC_lexicon_tags_15 },
    { 0xf84d492e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912d4934, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42d493a, SUC_lexicon_tags_6 },
    { 0x56ed493a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2d493e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x498d4941, SUC_lexicon_tags_27 },
    { 0xadcd4942, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5c6d4944, SUC_lexicon_tags_17 },
    { 0xd0cd4945, SUC_lexicon_tags_49 },
    { 0x166d4946, SUC_lexicon_tags_60 },
    { 0xfe0d4945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742d494c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ed494f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x202d4951, SUC_lexicon_tags_37 },
    { 0x4d4d4951, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88d4957, SUC_lexicon_tags_12 },
    { 0xe6cd4957, SUC_lexicon_tags_23 },
    { 0xff8d4957, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfad495b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ad495e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ad4963, SUC_lexicon_tags_11 },
    { 0x92cd4964, SUC_lexicon_tags_10 },
    { 0x256d4965, SUC_lexicon_tags_27 },
    { 0xaa2d4966, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x946d4969, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x730d496b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cd4971, SUC_lexicon_tags_61 },
    { 0x7cd4972, SUC_lexicon_tags_37 },
    { 0x5c0d4973, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8f4d4975, SUC_lexicon_tags_26 },
    { 0x8eed4976, SUC_lexicon_tags_15 },
    { 0x6e4d4976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92cd4979, SUC_lexicon_tags_47 },
    { 0xa18d497a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f2d497c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4d4982, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb36d4984, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x816d4986, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35ad4988, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ad498c, SUC_lexicon_tags_3 },
    { 0x26cd498d, SUC_lexicon_tags_16 },
    { 0x68ad498e, SUC_lexicon_tags_207 },
    { 0x91cd498e, SUC_lexicon_tags_37 },
    { 0x96d498d, SUC_lexicon_tags_17 },
    { 0x3fcd4991, SUC_lexicon_tags_27 },
    { 0xe9ad4992, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884d4999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cd499c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x290d499e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x280d49a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286d49a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbf6d49a7, SUC_lexicon_tags_37 },
    { 0xfa2d49a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900d49ad, SUC_lexicon_tags_27 },
    { 0xa0ad49ad, SUC_lexicon_tags_1 },
    { 0xae0d49af, SUC_lexicon_tags_19 },
    { 0xfa6d49b0, SUC_lexicon_tags_37 },
    { 0x686d49b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4d49b9, SUC_lexicon_tags_11 },
    { 0x476d49ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb2d49bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeced49be, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x8a8d49c0, SUC_lexicon_tags_37 },
    { 0x188d49c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6d49c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8d49c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ed49cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcd49d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x358d49d8, SUC_lexicon_tags_37 },
    { 0xf14d49d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774d49de, SUC_lexicon_tags_21 },
    { 0x59ed49df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x138d49e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x920d49e4, SUC_lexicon_tags_17 },
    { 0x40ed49e4, SUC_lexicon_tags_15 },
    { 0x724d49e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498d49eb, SUC_lexicon_tags_25 },
    { 0x546d49ec, SUC_lexicon_tags_87 },
    { 0x1f8d49ec, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6d49f0, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0x120d49f2, SUC_lexicon_tags_1 },
    { 0x5fcd49f2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42d49f6, SUC_lexicon_tags_8 },
    { 0xa88d49f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1cd49f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54d49ff, SUC_lexicon_tags_81 },
    { 0x632d4a00, SUC_lexicon_tags_12 },
    { 0xf26d4a01, SUC_lexicon_tags_37 },
    { 0x494d4a02, SUC_lexicon_tags_27 },
    { 0x8b8d4a01, SUC_lexicon_tags_11 },
    { 0x8eed4a04, SUC_lexicon_tags_15 },
    { 0xea2d4a04, SUC_lexicon_tags_6 },
    { 0xd26d4a06, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12d4a0a, SUC_lexicon_tags_1 },
    { 0x696d4a0a, SUC_lexicon_tags_15 },
    { 0x91cd4a0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3fad4a0e, SUC_lexicon_tags_37 },
    { 0xb0d4a0f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x134d4a12, SUC_lexicon_tags_12 },
    { 0xbbcd4a13, SUC_lexicon_tags_216 },
    { 0xa58d4a12, SUC_lexicon_tags_19 },
    { 0x89ad4a12, SUC_lexicon_tags_75 },
    { 0xe8d4a16, SUC_lexicon_tags_28 },
    { 0x288d4a13, SUC_lexicon_tags_13 },
    { 0x254d4a18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x520d4a1b, SUC_lexicon_tags_37 },
    { 0x63ed4a1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x340d4a1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x840d4a21, SUC_lexicon_tags_27 },
    { 0x7c4d4a21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb64d4a24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fad4a26, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6d4a29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3ed4a2b, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602d4a30, SUC_lexicon_tags_26 },
    { 0x90d4a30, SUC_lexicon_tags_17 },
    { 0x350d4a31, SUC_lexicon_tags_11 },
    { 0xb62d4a33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5ead4a35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ed4a39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa58d4a3b, SUC_lexicon_tags_15 },
    { 0x1e6d4a3c, SUC_lexicon_tags_33 },
    { 0xc02d4a3d, SUC_lexicon_tags_33 },
    { 0x798d4a3e, SUC_lexicon_tags_15 },
    { 0x2c4d4a3e, SUC_lexicon_tags_1 },
    { 0x2ad4a3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ead4a44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef6d4a46, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972d4a4d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe3cd4a4f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbe2d4a51, SUC_lexicon_tags_3 },
    { 0x6b2d4a52, SUC_lexicon_tags_12 },
    { 0x27cd4a53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb64d4a55, SUC_lexicon_tags_37 },
    { 0x906d4a55, SUC_lexicon_tags_12 },
    { 0xa88d4a57, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94d4a5b, SUC_lexicon_tags_37 },
    { 0x92ed4a5c, SUC_lexicon_tags_7 },
    { 0x9e2d4a5b, SUC_lexicon_tags_9 },
    { 0x69ed4a5b, SUC_lexicon_tags_47 },
    { 0x68ed4a5f, SUC_lexicon_tags_30 },
    { 0x456d4a5c, SUC_lexicon_tags_47 },
    { 0xe70d4a5e, SUC_lexicon_tags_28 },
    { 0x22ed4a62, SUC_lexicon_tags_37 },
    { 0x598d4a5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2d4a68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02d4a70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x538d4a72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4d4a75, SUC_lexicon_tags_87 },
    { 0x3bcd4a76, SUC_lexicon_tags_174 },
    { 0xecd4a77, SUC_lexicon_tags_6 },
    { 0xcb8d4a78, SUC_lexicon_tags_43 },
    { 0xf6cd4a78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6d4a82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ad4a85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ead4a87, SUC_lexicon_tags_27 },
    { 0x3d4d4a88, SUC_lexicon_tags_25 },
    { 0xdd0d4a88, SUC_lexicon_tags_19 },
    { 0xd6d4a88, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2d4a8e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbd6d4a90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66d4a9d, SUC_lexicon_tags_37 },
    { 0x566d4a9d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740d4aaa, SUC_lexicon_tags_38 },
    { 0xcf0d4aaa, SUC_lexicon_tags_7 },
    { 0x8a2d4aab, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944d4ab1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cd4ab5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ed4abb, SUC_lexicon_tags_37 },
    { 0x164d4abb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc3cd4abe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610d4ac4, SUC_lexicon_tags_15 },
    { 0x4d2d4ac4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x228d4ac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ead4aca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa8ed4acc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfd0d4ace, SUC_lexicon_tags_15 },
    { 0x4d2d4acf, SUC_lexicon_tags_15 },
    { 0x674d4acf, SUC_lexicon_tags_29 },
    { 0x322d4ad1, SUC_lexicon_tags_37 },
    { 0x8d4d4ad1, SUC_lexicon_tags_38 },
    { 0x492d4ad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x834d4ad6, SUC_lexicon_tags_31 },
    { 0x616d4ad7, SUC_lexicon_tags_12 },
    { 0xb64d4ad8, SUC_lexicon_tags_37 },
    { 0xa7ad4ad9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x144d4adc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xccd4ade, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0d4ae4, SUC_lexicon_tags_15 },
    { 0x5b0d4ae5, SUC_lexicon_tags_48 },
    { 0xb68d4ae6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4e8d4ae8, SUC_lexicon_tags_28 },
    { 0xce8d4ae9, SUC_lexicon_tags_1 },
    { 0xf30d4ae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x936d4aed, SUC_lexicon_tags_16 },
    { 0x3c8d4aed, SUC_lexicon_tags_34 },
    { 0x9c4d4aee, SUC_lexicon_tags_46 },
    { 0xb84d4af0, SUC_lexicon_tags_0 },
    { 0x1ad4af1, SUC_lexicon_tags_65 },
    { 0xad4af1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2d4af5, SUC_lexicon_tags_0 },
    { 0x336d4af6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8d4afd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580d4b01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aad4b0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d4b0f, SUC_lexicon_tags_60 },
    { 0xfa6d4b10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x714d4b12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x202d4b14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6d4b1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4d4b1e, SUC_lexicon_tags_37 },
    { 0x6e2d4b1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6d4b25, SUC_lexicon_tags_25 },
    { 0x7c6d4b26, SUC_lexicon_tags_82 },
    { 0x260d4b26, SUC_lexicon_tags_23 },
    { 0xd30d4b28, SUC_lexicon_tags_1 },
    { 0x7a6d4b28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6d4b2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e2d4b2f, SUC_lexicon_tags_0 },
    { 0xe96d4b2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e4d4b32, SUC_lexicon_tags_51 },
    { 0x776d4b33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x534d4b36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6d4b39, SUC_lexicon_tags_11 },
    { 0xc60d4b3a, SUC_lexicon_tags_4 },
    { 0x932d4b39, SUC_lexicon_tags_63 },
    { 0x5bed4b3c, SUC_lexicon_tags_3 },
    { 0x1d2d4b3c, SUC_lexicon_tags_63 },
    { 0x822d4b3e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4eed4b40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ccd4b42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d2d4b44, SUC_lexicon_tags_15 },
    { 0x94cd4b45, SUC_lexicon_tags_14 },
    { 0xa7ed4b46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86d4b4c, SUC_lexicon_tags_25 },
    { 0xf06d4b4d, SUC_lexicon_tags_1 },
    { 0xc38d4b4e, SUC_lexicon_tags_49 },
    { 0x564d4b4c, SUC_lexicon_tags_65 },
    { 0x6f8d4b50, SUC_lexicon_tags_3 },
    { 0x7fcd4b50, SUC_lexicon_tags_16 },
    { 0xe24d4b52, SUC_lexicon_tags_2 },
    { 0xb86d4b53, SUC_lexicon_tags_17 },
    { 0x730d4b53, SUC_lexicon_tags_0 },
    { 0x2ded4b50, SUC_lexicon_tags_7 },
    { 0x798d4b56, SUC_lexicon_tags_11 },
    { 0x5ecd4b54, SUC_lexicon_tags_14 },
    { 0xf68d4b57, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cd4b5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdad4b64, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cd4b6e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecd4b74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ad4b78, SUC_lexicon_tags_7 },
    { 0x1c6d4b78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ded4b7c, SUC_lexicon_tags_12 },
    { 0x6f2d4b7d, SUC_lexicon_tags_7 },
    { 0xd22d4b7e, SUC_lexicon_tags_2 },
    { 0x4fad4b7f, SUC_lexicon_tags_12 },
    { 0x600d4b80, SUC_lexicon_tags_17 },
    { 0xf46d4b80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcd4b86, SUC_lexicon_tags_12 },
    { 0xa08d4b87, SUC_lexicon_tags_34 },
    { 0xec8d4b88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87cd4b8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x96ed4b8c, SUC_lexicon_tags_23 },
    { 0x38ad4b8d, SUC_lexicon_tags_31 },
    { 0x24ed4b8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf52d4b90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf90d4b92, SUC_lexicon_tags_37 },
    { 0x270d4b93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc6ad4b95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ad4b9b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2d4ba7, SUC_lexicon_tags_1 },
    { 0xac0d4ba8, SUC_lexicon_tags_47 },
    { 0xcdcd4ba9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8d4bac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0ed4bae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ad4bb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2d4bb6, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad4bb9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x476d4bbc, SUC_lexicon_tags_17 },
    { 0x67ad4bbd, SUC_lexicon_tags_51 },
    { 0x29cd4bbe, SUC_lexicon_tags_37 },
    { 0x7ead4bbd, SUC_lexicon_tags_9 },
    { 0x418d4bc0, SUC_lexicon_tags_27 },
    { 0xfcd4bc1, SUC_lexicon_tags_7 },
    { 0xafad4bbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926d4bc6, SUC_lexicon_tags_15 },
    { 0xad8d4bc7, SUC_lexicon_tags_32 },
    { 0xc7cd4bc7, SUC_lexicon_tags_5 },
    { 0x8a2d4bc9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ced4bcf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282d4bd3, SUC_lexicon_tags_38 },
    { 0xe78d4bd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ed4bd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5f4d4bda, SUC_lexicon_tags_66 },
    { 0x838d4bdb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ad4bdf, SUC_lexicon_tags_34 },
    { 0xe46d4bdf, SUC_lexicon_tags_31 },
    { 0x776d4be1, SUC_lexicon_tags_2 },
    { 0x9d6d4bdf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2d4be5, SUC_lexicon_tags_10 },
    { 0xe32d4be5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x170d4be8, SUC_lexicon_tags_17 },
    { 0x9a0d4be9, SUC_lexicon_tags_15 },
    { 0x8bed4be9, SUC_lexicon_tags_621 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2d4bf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdccd4bf2, SUC_lexicon_tags_31 },
    { 0xbcd4bf3, SUC_lexicon_tags_31 },
    { 0x7fcd4bf4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbccd4bf6, SUC_lexicon_tags_21 },
    { 0x914d4bf7, SUC_lexicon_tags_48 },
    { 0x76ed4bf8, SUC_lexicon_tags_49 },
    { 0x748d4bf7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x660d4bfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cd4c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ad4c03, SUC_lexicon_tags_38 },
    { 0x764d4c04, SUC_lexicon_tags_19 },
    { 0xabcd4c05, SUC_lexicon_tags_38 },
    { 0x596d4c06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94d4c12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56d4c16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb48d4c18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x990d4c1b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x624d4c1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832d4c22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cad4c24, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cd4c2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14d4c2e, SUC_lexicon_tags_28 },
    { 0xe82d4c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb86d4c31, SUC_lexicon_tags_1 },
    { 0x178d4c32, SUC_lexicon_tags_7 },
    { 0xc4ad4c31, SUC_lexicon_tags_31 },
    { 0x658d4c34, SUC_lexicon_tags_1 },
    { 0x640d4c34, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12d4c3b, SUC_lexicon_tags_7 },
    { 0x69cd4c3b, SUC_lexicon_tags_37 },
    { 0xc7ed4c3d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x326d4c3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458d4c46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d4c48, SUC_lexicon_tags_31 },
    { 0xd3ad4c49, SUC_lexicon_tags_7 },
    { 0x842d4c4a, SUC_lexicon_tags_92 },
    { 0x8d2d4c4b, SUC_lexicon_tags_87 },
    { 0xbd6d4c4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ed4c50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618d4c54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ed4c5b, SUC_lexicon_tags_64 },
    { 0x456d4c5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe62d4c62, SUC_lexicon_tags_6 },
    { 0xd0d4c62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d4c69, SUC_lexicon_tags_65 },
    { 0x182d4c6a, SUC_lexicon_tags_29 },
    { 0x302d4c6a, SUC_lexicon_tags_25 },
    { 0xf5ad4c6c, SUC_lexicon_tags_49 },
    { 0xb82d4c6d, SUC_lexicon_tags_66 },
    { 0xad0d4c6d, SUC_lexicon_tags_6 },
    { 0xc6ed4c6e, SUC_lexicon_tags_15 },
    { 0xaa0d4c70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x768d4c72, SUC_lexicon_tags_37 },
    { 0xc5cd4c73, SUC_lexicon_tags_2 },
    { 0x8d4c74, SUC_lexicon_tags_15 },
    { 0x39ad4c75, SUC_lexicon_tags_7 },
    { 0x61ed4c76, SUC_lexicon_tags_38 },
    { 0x89cd4c77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0d4c7a, SUC_lexicon_tags_46 },
    { 0x2e2d4c7b, SUC_lexicon_tags_27 },
    { 0xe0ed4c7b, SUC_lexicon_tags_29 },
    { 0x9bed4c7b, SUC_lexicon_tags_7 },
    { 0xcb0d4c7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cad4c80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2d4c85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6d4c89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6d4c8d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x834d4c90, SUC_lexicon_tags_25 },
    { 0xa10d4c91, SUC_lexicon_tags_7 },
    { 0x9d2d4c92, SUC_lexicon_tags_1 },
    { 0x890d4c93, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x436d4c95, SUC_lexicon_tags_25 },
    { 0x386d4c96, SUC_lexicon_tags_29 },
    { 0xb9cd4c96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe8d4c99, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x64cd4c9c, SUC_lexicon_tags_27 },
    { 0xd18d4c9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ad4ca0, SUC_lexicon_tags_9 },
    { 0xbb8d4ca1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf48d4ca3, SUC_lexicon_tags_17 },
    { 0x558d4ca4, SUC_lexicon_tags_19 },
    { 0xc1ed4ca5, SUC_lexicon_tags_35 },
    { 0xb7ad4ca5, SUC_lexicon_tags_2 },
    { 0xeb4d4ca7, SUC_lexicon_tags_31 },
    { 0x644d4ca7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad4cab, SUC_lexicon_tags_10 },
    { 0x8d4cac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946d4cb0, SUC_lexicon_tags_10 },
    { 0x24ad4cb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6d4cb6, SUC_lexicon_tags_2 },
    { 0x934d4cb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972d4cbb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x158d4cbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68d4cc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a0d4cc3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aad4cc8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc94d4cca, SUC_lexicon_tags_49 },
    { 0xd72d4cca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06d4cd6, SUC_lexicon_tags_79 },
    { 0xe86d4cd7, SUC_lexicon_tags_0 },
    { 0xf24d4cd8, SUC_lexicon_tags_2 },
    { 0x954d4cd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x254d4cdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464d4ce2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4d4ce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cd4cf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x438d4cf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x596d4cf6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x95cd4cf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4d4cfb, SUC_lexicon_tags_38 },
    { 0xee8d4cfb, SUC_lexicon_tags_27 },
    { 0x13ed4cfd, SUC_lexicon_tags_15 },
    { 0xeb6d4cfd, SUC_lexicon_tags_23 },
    { 0xfe0d4cfb, SUC_lexicon_tags_19 },
    { 0x752d4cfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2d4d03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74cd4d05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3bed4d07, SUC_lexicon_tags_15 },
    { 0x3bed4d07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cd4d10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ed4d16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ed4d1a, SUC_lexicon_tags_65 },
    { 0x5f4d4d1b, SUC_lexicon_tags_12 },
    { 0x9acd4d1a, SUC_lexicon_tags_9 },
    { 0xbb6d4d1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x574d4d20, SUC_lexicon_tags_2 },
    { 0x710d4d21, SUC_lexicon_tags_25 },
    { 0xbed4d22, SUC_lexicon_tags_39 },
    { 0x3e8d4d22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccd4d26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ad4d29, SUC_lexicon_tags_25 },
    { 0xe60d4d29, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xaf0d4d2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ed4d35, SUC_lexicon_tags_28 },
    { 0x2a6d4d36, SUC_lexicon_tags_8 },
    { 0x42d4d37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63ad4d39, SUC_lexicon_tags_0 },
    { 0x976d4d3a, SUC_lexicon_tags_25 },
    { 0x76d4d3b, SUC_lexicon_tags_45 },
    { 0xfd8d4d3a, SUC_lexicon_tags_23 },
    { 0xcf8d4d3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ed4d40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448d4d46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58d4d4a, SUC_lexicon_tags_11 },
    { 0x65cd4d4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ed4d4e, SUC_lexicon_tags_25 },
    { 0x3a8d4d4e, SUC_lexicon_tags_82 },
    { 0xde6d4d50, SUC_lexicon_tags_12 },
    { 0x576d4d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600d4d55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378d4d5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x150d4d5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ccd4d5f, SUC_lexicon_tags_47 },
    { 0x1aad4d5f, SUC_lexicon_tags_1 },
    { 0x206d4d61, SUC_lexicon_tags_3 },
    { 0x572d4d62, SUC_lexicon_tags_29 },
    { 0x84cd4d63, SUC_lexicon_tags_25 },
    { 0xcfad4d62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21cd4d67, SUC_lexicon_tags_37 },
    { 0x7aed4d68, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cd4d6b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x450d4d6d, SUC_lexicon_tags_8 },
    { 0x90ed4d6e, SUC_lexicon_tags_11 },
    { 0x3ded4d6e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x59ed4d71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6d4d74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cd4d7a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x394d4d7c, SUC_lexicon_tags_10 },
    { 0x40ed4d7c, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d4d82, SUC_lexicon_tags_27 },
    { 0xca4d4d83, SUC_lexicon_tags_39 },
    { 0x1d2d4d83, SUC_lexicon_tags_31 },
    { 0x16ad4d85, SUC_lexicon_tags_25 },
    { 0x2c8d4d86, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x734d4d88, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8a2d4d8a, SUC_lexicon_tags_25 },
    { 0x2eed4d8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a0d4d8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x542d4d8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x95cd4d91, SUC_lexicon_tags_38 },
    { 0xa4cd4d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72d4d97, SUC_lexicon_tags_43 },
    { 0x67cd4d98, SUC_lexicon_tags_16 },
    { 0xb9cd4d98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ad4d9d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccd4da1, SUC_lexicon_tags_37 },
    { 0x4b6d4da2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d4da6, SUC_lexicon_tags_193 },
    { 0xd7ed4da6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38ad4da9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x414d4dab, SUC_lexicon_tags_17 },
    { 0xfaad4dac, SUC_lexicon_tags_47 },
    { 0x7b2d4dad, SUC_lexicon_tags_7 },
    { 0x1ecd4dae, SUC_lexicon_tags_68 },
    { 0xaad4dab, SUC_lexicon_tags_0 },
    { 0xc84d4db0, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eed4db3, SUC_lexicon_tags_7 },
    { 0x89ad4db4, SUC_lexicon_tags_2 },
    { 0x44cd4db4, SUC_lexicon_tags_11 },
    { 0x540d4db3, SUC_lexicon_tags_11 },
    { 0x600d4db7, SUC_lexicon_tags_30 },
    { 0x54ed4db4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc88d4dba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ad4dbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x512d4dc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92d4dc2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x59ad4dc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcd4dca, SUC_lexicon_tags_0 },
    { 0xaaed4dcb, SUC_lexicon_tags_33 },
    { 0x66d4dcc, SUC_lexicon_tags_21 },
    { 0x18ad4dcc, SUC_lexicon_tags_0 },
    { 0x31ad4dce, SUC_lexicon_tags_68 },
    { 0xf00d4dce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x252d4dd2, SUC_lexicon_tags_31 },
    { 0x1e6d4dd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecd4ddb, SUC_lexicon_tags_15 },
    { 0xf20d4ddc, SUC_lexicon_tags_2 },
    { 0x992d4ddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ed4de1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b0d4de3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4d4de8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2d2d4dea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296d4df2, SUC_lexicon_tags_37 },
    { 0x736d4df2, SUC_lexicon_tags_14 },
    { 0x138d4df3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6b2d4df6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9ad4df8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x626d4dfa, SUC_lexicon_tags_23 },
    { 0x13ed4dfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72cd4dfd, SUC_lexicon_tags_37 },
    { 0x57ad4dfd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcd4e01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ed4e05, SUC_lexicon_tags_37 },
    { 0x29ad4e05, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x90ad4e08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0d4e0b, SUC_lexicon_tags_12 },
    { 0x552d4e0c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfd0d4e0e, SUC_lexicon_tags_37 },
    { 0xca8d4e0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb16d4e11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x280d4e13, SUC_lexicon_tags_1 },
    { 0x924d4e13, SUC_lexicon_tags_2 },
    { 0xa0d4e15, SUC_lexicon_tags_4 },
    { 0x87ed4e16, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4d4e19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dad4e1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e8d4e1e, SUC_lexicon_tags_103 },
    { 0xd60d4e1f, SUC_lexicon_tags_87 },
    { 0x640d4e1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456d4e25, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04d4e29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b6d4e2b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6d4e2e, SUC_lexicon_tags_60 },
    { 0xe84d4e2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x42ad4e31, SUC_lexicon_tags_37 },
    { 0x918d4e31, SUC_lexicon_tags_25 },
    { 0x90cd4e32, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2ed4e35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0d4e38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0d4e43, SUC_lexicon_tags_40 },
    { 0x9fad4e44, SUC_lexicon_tags_1 },
    { 0x376d4e45, SUC_lexicon_tags_14 },
    { 0x1c8d4e43, SUC_lexicon_tags_17 },
    { 0x642d4e44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6d4e4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ecd4e4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0d4e52, SUC_lexicon_tags_10 },
    { 0xde0d4e52, SUC_lexicon_tags_12 },
    { 0xd60d4e54, SUC_lexicon_tags_27 },
    { 0xb62d4e55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x608d4e57, SUC_lexicon_tags_26 },
    { 0x8f8d4e58, SUC_lexicon_tags_45 },
    { 0x900d4e58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbad4e5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114d4e61, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822d4e65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2d4e67, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200d4e6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd5ed4e6f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58d4e73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ed4e76, SUC_lexicon_tags_37 },
    { 0x9c4d4e77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12d4e7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcad4e7f, SUC_lexicon_tags_315 },
    { 0, NULL },
    { 0x10ed4e81, SUC_lexicon_tags_1 },
    { 0xcbad4e81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12d4e86, SUC_lexicon_tags_37 },
    { 0xc5ad4e87, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02d4e8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04d4e97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6d4e9c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecd4ea0, SUC_lexicon_tags_14 },
    { 0x1d6d4ea1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ed4ea4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24d4ea7, SUC_lexicon_tags_13 },
    { 0x6a8d4ea8, SUC_lexicon_tags_43 },
    { 0xd2ed4ea9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42d4ead, SUC_lexicon_tags_6 },
    { 0x586d4ead, SUC_lexicon_tags_0 },
    { 0x78cd4eaf, SUC_lexicon_tags_37 },
    { 0xcead4eae, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800d4eb6, SUC_lexicon_tags_17 },
    { 0x51cd4eb7, SUC_lexicon_tags_17 },
    { 0xfccd4eb7, SUC_lexicon_tags_57 },
    { 0xed8d4eb6, SUC_lexicon_tags_37 },
    { 0x634d4eb9, SUC_lexicon_tags_7 },
    { 0x578d4ebb, SUC_lexicon_tags_84 },
    { 0x614d4ebb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a2d4ebe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x718d4ec1, SUC_lexicon_tags_20 },
    { 0x3bad4ec2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bad4ec4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2d4ece, SUC_lexicon_tags_49 },
    { 0x82ed4ece, SUC_lexicon_tags_42 },
    { 0x124d4ed0, SUC_lexicon_tags_33 },
    { 0x918d4ece, SUC_lexicon_tags_55 },
    { 0xad4d4ed1, SUC_lexicon_tags_14 },
    { 0x638d4ece, SUC_lexicon_tags_3 },
    { 0xe0ad4ed1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x894d4ed6, SUC_lexicon_tags_6 },
    { 0x422d4ed6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8d4eda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x370d4edd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xca8d4ee0, SUC_lexicon_tags_7 },
    { 0xe24d4ee1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeded4ee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02d4eea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7b6d4eec, SUC_lexicon_tags_47 },
    { 0xa82d4eed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb12d4eef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x664d4ef1, SUC_lexicon_tags_25 },
    { 0x150d4ef1, SUC_lexicon_tags_25 },
    { 0x768d4ef3, SUC_lexicon_tags_12 },
    { 0xa32d4ef4, SUC_lexicon_tags_15 },
    { 0xbfcd4ef3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464d4efc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ead4efe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ed4f02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6d4f09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x710d4f0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x162d4f0e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ad4f12, SUC_lexicon_tags_30 },
    { 0x3aad4f13, SUC_lexicon_tags_1 },
    { 0x230d4f13, SUC_lexicon_tags_27 },
    { 0x670d4f15, SUC_lexicon_tags_28 },
    { 0x992d4f13, SUC_lexicon_tags_47 },
    { 0x7dad4f12, SUC_lexicon_tags_11 },
    { 0x20ed4f18, SUC_lexicon_tags_11 },
    { 0xf14d4f18, SUC_lexicon_tags_2 },
    { 0xc88d4f1a, SUC_lexicon_tags_34 },
    { 0x1e4d4f1a, SUC_lexicon_tags_31 },
    { 0xa88d4f1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x71ed4f1e, SUC_lexicon_tags_30 },
    { 0xaf6d4f1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36ed4f21, SUC_lexicon_tags_37 },
    { 0x988d4f22, SUC_lexicon_tags_26 },
    { 0x648d4f23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b8d4f25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4d4f28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ed4f2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96d4f2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678d4f32, SUC_lexicon_tags_21 },
    { 0xaacd4f33, SUC_lexicon_tags_25 },
    { 0xc90d4f33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2d4f39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfed4f3c, SUC_lexicon_tags_1 },
    { 0xf8ad4f3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf20d4f3f, SUC_lexicon_tags_17 },
    { 0xd06d4f40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x154d4f43, SUC_lexicon_tags_37 },
    { 0xf0cd4f44, SUC_lexicon_tags_7 },
    { 0x496d4f43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480d4f4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x934d4f4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x912d4f51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x202d4f53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb42d4f55, SUC_lexicon_tags_32 },
    { 0x634d4f56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20d4f58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ed4f5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4d4f61, SUC_lexicon_tags_15 },
    { 0x3f2d4f62, SUC_lexicon_tags_17 },
    { 0x46ad4f62, SUC_lexicon_tags_23 },
    { 0xd9cd4f64, SUC_lexicon_tags_11 },
    { 0x6d6d4f65, SUC_lexicon_tags_11 },
    { 0xfcd4f66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6d4f6a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638d4f6f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5aed4f71, SUC_lexicon_tags_9 },
    { 0x714d4f71, SUC_lexicon_tags_15 },
    { 0x40cd4f73, SUC_lexicon_tags_30 },
    { 0x38d4f74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68d4f7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabad4f80, SUC_lexicon_tags_12 },
    { 0x13ad4f81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2d4f87, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x704d4f8a, SUC_lexicon_tags_25 },
    { 0xbe4d4f8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea0d4f8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad4f90, SUC_lexicon_tags_17 },
    { 0x8ded4f91, SUC_lexicon_tags_48 },
    { 0xe94d4f90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228d4f96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ad4f9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef8d4f9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x430d4f9e, SUC_lexicon_tags_7 },
    { 0x1ded4f9e, SUC_lexicon_tags_63 },
    { 0xe82d4fa0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10d4fa6, SUC_lexicon_tags_1 },
    { 0xfa6d4fa6, SUC_lexicon_tags_104 },
    { 0x650d4fa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cd4fac, SUC_lexicon_tags_7 },
    { 0xa64d4fad, SUC_lexicon_tags_60 },
    { 0x746d4fac, SUC_lexicon_tags_37 },
    { 0xdcd4fad, SUC_lexicon_tags_30 },
    { 0xeb6d4fb0, SUC_lexicon_tags_17 },
    { 0xa6d4fad, SUC_lexicon_tags_32 },
    { 0x490d4fb2, SUC_lexicon_tags_43 },
    { 0x5c6d4fb3, SUC_lexicon_tags_47 },
    { 0x232d4fb2, SUC_lexicon_tags_17 },
    { 0xc4ed4fb3, SUC_lexicon_tags_238 },
    { 0x182d4fb1, SUC_lexicon_tags_10 },
    { 0x670d4fb7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152d4fbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0d4fbe, SUC_lexicon_tags_33 },
    { 0x29ed4fbf, SUC_lexicon_tags_17 },
    { 0x244d4fc0, SUC_lexicon_tags_38 },
    { 0xec6d4fbf, SUC_lexicon_tags_27 },
    { 0x714d4fc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39ad4fc4, SUC_lexicon_tags_28 },
    { 0xe94d4fc4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4f0d4fc7, SUC_lexicon_tags_25 },
    { 0x466d4fc7, SUC_lexicon_tags_37 },
    { 0x1bcd4fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154d4fcd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674d4fd3, SUC_lexicon_tags_8 },
    { 0x246d4fd4, SUC_lexicon_tags_68 },
    { 0xf46d4fd5, SUC_lexicon_tags_9 },
    { 0x4e6d4fd6, SUC_lexicon_tags_1 },
    { 0x5d2d4fd7, SUC_lexicon_tags_129 },
    { 0x360d4fd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4d4fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe96d4fe7, SUC_lexicon_tags_45 },
    { 0x7b2d4fe8, SUC_lexicon_tags_20 },
    { 0xde6d4fe9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38d4fef, SUC_lexicon_tags_21 },
    { 0x31ed4ff0, SUC_lexicon_tags_2 },
    { 0x2c4d4fef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794d4ff9, SUC_lexicon_tags_737 },
    { 0x24d4ff9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabed4ffc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x830d4ffe, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x3c4d5000, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d5003, SUC_lexicon_tags_17 },
    { 0xbbad5004, SUC_lexicon_tags_14 },
    { 0xfd0d5004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c4d5007, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1aed5009, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32d500c, SUC_lexicon_tags_0 },
    { 0xebad500d, SUC_lexicon_tags_17 },
    { 0x2d4d500e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668d5012, SUC_lexicon_tags_12 },
    { 0xde4d5012, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x574d5015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11ad5017, SUC_lexicon_tags_60 },
    { 0x1c8d5018, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4eed501a, SUC_lexicon_tags_28 },
    { 0x4ad501b, SUC_lexicon_tags_11 },
    { 0x354d501c, SUC_lexicon_tags_6 },
    { 0xd0d501d, SUC_lexicon_tags_31 },
    { 0x21ad501e, SUC_lexicon_tags_37 },
    { 0x41ed501c, SUC_lexicon_tags_37 },
    { 0x83ad501b, SUC_lexicon_tags_30 },
    { 0x3ded501e, SUC_lexicon_tags_5 },
    { 0xdf0d501f, SUC_lexicon_tags_104 },
    { 0xb58d5023, SUC_lexicon_tags_11 },
    { 0x634d5022, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2fcd5026, SUC_lexicon_tags_11 },
    { 0x352d5027, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd0d5029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4d502d, SUC_lexicon_tags_17 },
    { 0xff6d502d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68d5032, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4d5035, SUC_lexicon_tags_27 },
    { 0x8e6d5036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434d503d, SUC_lexicon_tags_37 },
    { 0x786d503d, SUC_lexicon_tags_11 },
    { 0x3ced503f, SUC_lexicon_tags_17 },
    { 0xed0d503f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ad5047, SUC_lexicon_tags_38 },
    { 0x9dad5048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x396d504b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eed504e, SUC_lexicon_tags_37 },
    { 0xab0d504e, SUC_lexicon_tags_2 },
    { 0x972d504e, SUC_lexicon_tags_7 },
    { 0x4dad504e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ed5057, SUC_lexicon_tags_46 },
    { 0x71ad5057, SUC_lexicon_tags_1 },
    { 0xa78d5057, SUC_lexicon_tags_11 },
    { 0xbbed5057, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x396d505d, SUC_lexicon_tags_17 },
    { 0x59ed505e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4d5063, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376d5069, SUC_lexicon_tags_9 },
    { 0x910d506a, SUC_lexicon_tags_14 },
    { 0xc30d5069, SUC_lexicon_tags_12 },
    { 0x594d506c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cd5073, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x334d5076, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4d507e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b4d5080, SUC_lexicon_tags_47 },
    { 0xb12d5081, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa8ed5083, SUC_lexicon_tags_2 },
    { 0xf3ad5084, SUC_lexicon_tags_9 },
    { 0xb5ad5085, SUC_lexicon_tags_19 },
    { 0xfcd5086, SUC_lexicon_tags_551 },
    { 0x6c0d5087, SUC_lexicon_tags_336 },
    { 0xfcd5088, SUC_lexicon_tags_22 },
    { 0x26d5089, SUC_lexicon_tags_27 },
    { 0xe04d508a, SUC_lexicon_tags_7 },
    { 0xe38d508a, SUC_lexicon_tags_10 },
    { 0x9ccd5086, SUC_lexicon_tags_27 },
    { 0x254d508d, SUC_lexicon_tags_26 },
    { 0xff2d508e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ad5097, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0d509a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6d509d, SUC_lexicon_tags_25 },
    { 0xe1ad509e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d0d50a0, SUC_lexicon_tags_7 },
    { 0xddad50a1, SUC_lexicon_tags_25 },
    { 0x3a6d50a2, SUC_lexicon_tags_17 },
    { 0x910d50a2, SUC_lexicon_tags_238 },
    { 0xbe2d50a4, SUC_lexicon_tags_30 },
    { 0xc22d50a5, SUC_lexicon_tags_4 },
    { 0x554d50a5, SUC_lexicon_tags_66 },
    { 0x584d50a7, SUC_lexicon_tags_17 },
    { 0xd6d50a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6d50ac, SUC_lexicon_tags_63 },
    { 0xcced50ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cd50b4, SUC_lexicon_tags_39 },
    { 0xd06d50b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef0d50b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2d50bc, SUC_lexicon_tags_29 },
    { 0xe64d50bd, SUC_lexicon_tags_43 },
    { 0x986d50be, SUC_lexicon_tags_37 },
    { 0x45cd50be, SUC_lexicon_tags_38 },
    { 0x13ad50c0, SUC_lexicon_tags_47 },
    { 0x72cd50c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54cd50c3, SUC_lexicon_tags_2 },
    { 0x402d50c4, SUC_lexicon_tags_15 },
    { 0xc5ad50c3, SUC_lexicon_tags_12 },
    { 0xc4ad50c6, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x152d50c9, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8d50cf, SUC_lexicon_tags_7 },
    { 0xbcad50d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50d50d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc20d50d7, SUC_lexicon_tags_7 },
    { 0x3fad50d8, SUC_lexicon_tags_7 },
    { 0xcbed50d9, SUC_lexicon_tags_11 },
    { 0x63ed50da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cd50df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x142d50e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ad50e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cd50e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4d50ed, SUC_lexicon_tags_26 },
    { 0xc10d50ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d50f2, SUC_lexicon_tags_70 },
    { 0x7f0d50f2, SUC_lexicon_tags_15 },
    { 0xc16d50f3, SUC_lexicon_tags_19 },
    { 0x812d50f2, SUC_lexicon_tags_17 },
    { 0xaa4d50f6, SUC_lexicon_tags_31 },
    { 0xfe4d50f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d50f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52d50ff, SUC_lexicon_tags_12 },
    { 0xd5ed50ff, SUC_lexicon_tags_17 },
    { 0xfe0d5101, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2d5107, SUC_lexicon_tags_43 },
    { 0xaf0d5108, SUC_lexicon_tags_66 },
    { 0x876d5109, SUC_lexicon_tags_12 },
    { 0xc84d5108, SUC_lexicon_tags_7 },
    { 0x51ad510b, SUC_lexicon_tags_11 },
    { 0x660d510c, SUC_lexicon_tags_6 },
    { 0xebcd510d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d5110, SUC_lexicon_tags_10 },
    { 0xf78d5111, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8ed5113, SUC_lexicon_tags_7 },
    { 0xf8ed5114, SUC_lexicon_tags_63 },
    { 0xbf6d5115, SUC_lexicon_tags_11 },
    { 0x82ad5116, SUC_lexicon_tags_17 },
    { 0x6d8d5114, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x11cd5119, SUC_lexicon_tags_8 },
    { 0x62ed5119, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcd511e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4d5121, SUC_lexicon_tags_46 },
    { 0x98ad5122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cad5126, SUC_lexicon_tags_19 },
    { 0x4cad5127, SUC_lexicon_tags_12 },
    { 0x80ed5128, SUC_lexicon_tags_25 },
    { 0xda6d5129, SUC_lexicon_tags_13 },
    { 0x54ad5129, SUC_lexicon_tags_37 },
    { 0xfd4d512a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350d5132, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x33ad5134, SUC_lexicon_tags_45 },
    { 0x994d5135, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766d513f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ed5144, SUC_lexicon_tags_12 },
    { 0x7f6d5145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x596d5147, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x7dad5149, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798d515c, SUC_lexicon_tags_126 },
    { 0x6f6d515d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8d5162, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b8d5164, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ecd5166, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774d516a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02d516d, SUC_lexicon_tags_25 },
    { 0x4cd516e, SUC_lexicon_tags_65 },
    { 0x3a6d516f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2d5177, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe32d5179, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x190d517c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8d517f, SUC_lexicon_tags_17 },
    { 0x45ad517f, SUC_lexicon_tags_47 },
    { 0x59ed5180, SUC_lexicon_tags_9 },
    { 0x730d5181, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cd5185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ad5190, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36d5192, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ed5196, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ad51a6, SUC_lexicon_tags_17 },
    { 0x21ed51a7, SUC_lexicon_tags_15 },
    { 0x9ded51a7, SUC_lexicon_tags_1 },
    { 0xaacd51a8, SUC_lexicon_tags_117 },
    { 0xaf8d51a6, SUC_lexicon_tags_77 },
    { 0xc16d51ab, SUC_lexicon_tags_69 },
    { 0x7e2d51a6, SUC_lexicon_tags_26 },
    { 0xca4d51ad, SUC_lexicon_tags_23 },
    { 0xe7cd51ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7ad51b0, SUC_lexicon_tags_9 },
    { 0xf58d51b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70cd51b3, SUC_lexicon_tags_1 },
    { 0x6a4d51b3, SUC_lexicon_tags_3 },
    { 0x578d51b4, SUC_lexicon_tags_37 },
    { 0x456d51b6, SUC_lexicon_tags_12 },
    { 0xf78d51b7, SUC_lexicon_tags_17 },
    { 0xf82d51b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90d51bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x228d51bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcd51c2, SUC_lexicon_tags_17 },
    { 0x8f8d51c2, SUC_lexicon_tags_37 },
    { 0x272d51c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acd51ca, SUC_lexicon_tags_47 },
    { 0x278d51ca, SUC_lexicon_tags_37 },
    { 0x68ad51cc, SUC_lexicon_tags_19 },
    { 0xe2ed51cd, SUC_lexicon_tags_33 },
    { 0x85ad51ca, SUC_lexicon_tags_10 },
    { 0x7e8d51ca, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x312d51d1, SUC_lexicon_tags_25 },
    { 0x3bcd51d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6d51d7, SUC_lexicon_tags_65 },
    { 0x98ad51d8, SUC_lexicon_tags_23 },
    { 0x80ed51d9, SUC_lexicon_tags_62 },
    { 0xa56d51da, SUC_lexicon_tags_33 },
    { 0xea2d51db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8d51e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfcd51e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16d51e7, SUC_lexicon_tags_46 },
    { 0x76ad51e7, SUC_lexicon_tags_26 },
    { 0x176d51e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dad51eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0d51ee, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x54cd51f0, SUC_lexicon_tags_25 },
    { 0x4bcd51f1, SUC_lexicon_tags_37 },
    { 0xae0d51f2, SUC_lexicon_tags_11 },
    { 0x83ad51f3, SUC_lexicon_tags_38 },
    { 0xc30d51f1, SUC_lexicon_tags_7 },
    { 0x10ed51f5, SUC_lexicon_tags_60 },
    { 0xfed51f6, SUC_lexicon_tags_17 },
    { 0x8fad51f6, SUC_lexicon_tags_35 },
    { 0x962d51f8, SUC_lexicon_tags_34 },
    { 0x368d51f5, SUC_lexicon_tags_3 },
    { 0x7ccd51f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaad51fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c6d51ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fed5202, SUC_lexicon_tags_7 },
    { 0xfaed5203, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x798d5206, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x790d5208, SUC_lexicon_tags_46 },
    { 0xd18d5209, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4d520c, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88d520f, SUC_lexicon_tags_32 },
    { 0x650d5210, SUC_lexicon_tags_37 },
    { 0x380d5210, SUC_lexicon_tags_7 },
    { 0x84ed5212, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aed5215, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x744d5217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde0d5219, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcd521d, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x9ecd521f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398d5224, SUC_lexicon_tags_2 },
    { 0x710d5225, SUC_lexicon_tags_23 },
    { 0x3bcd5226, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6d5229, SUC_lexicon_tags_7 },
    { 0x5d6d5229, SUC_lexicon_tags_19 },
    { 0x614d522b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8ad522d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4a2d522f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448d5238, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488d523c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876d5241, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d5246, SUC_lexicon_tags_21 },
    { 0x39ad5246, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24ad5249, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624d524d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x482d524f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332d5253, SUC_lexicon_tags_47 },
    { 0x1a2d5254, SUC_lexicon_tags_19 },
    { 0xc12d5254, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffed5259, SUC_lexicon_tags_15 },
    { 0xf72d525a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x204d525c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ed5265, SUC_lexicon_tags_33 },
    { 0xc24d5266, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2c4d5268, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ed526d, SUC_lexicon_tags_33 },
    { 0x772d526e, SUC_lexicon_tags_49 },
    { 0xcead526e, SUC_lexicon_tags_63 },
    { 0x4f0d526d, SUC_lexicon_tags_47 },
    { 0x292d5271, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912d5275, SUC_lexicon_tags_30 },
    { 0x44cd5275, SUC_lexicon_tags_15 },
    { 0xa84d5275, SUC_lexicon_tags_47 },
    { 0x678d5278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45ad527a, SUC_lexicon_tags_8 },
    { 0x848d527b, SUC_lexicon_tags_103 },
    { 0xb8ad527b, SUC_lexicon_tags_518 },
    { 0xcc4d527d, SUC_lexicon_tags_55 },
    { 0x574d527e, SUC_lexicon_tags_12 },
    { 0xb74d527f, SUC_lexicon_tags_28 },
    { 0x47ed527d, SUC_lexicon_tags_11 },
    { 0xf8ad5281, SUC_lexicon_tags_34 },
    { 0x5aed5282, SUC_lexicon_tags_2 },
    { 0x764d5281, SUC_lexicon_tags_11 },
    { 0x2aad5284, SUC_lexicon_tags_19 },
    { 0x614d5282, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7b4d5287, SUC_lexicon_tags_17 },
    { 0x26cd5287, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x418d528b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20d5290, SUC_lexicon_tags_11 },
    { 0xd6cd5290, SUC_lexicon_tags_47 },
    { 0x824d5292, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58d5297, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0x8a8d5299, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfaad529b, SUC_lexicon_tags_30 },
    { 0xf3ad529c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x812d529f, SUC_lexicon_tags_17 },
    { 0x714d529f, SUC_lexicon_tags_30 },
    { 0xd6ad52a1, SUC_lexicon_tags_30 },
    { 0xaccd52a0, SUC_lexicon_tags_25 },
    { 0x232d52a3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8d52a7, SUC_lexicon_tags_0 },
    { 0xd40d52a8, SUC_lexicon_tags_11 },
    { 0x63cd52a7, SUC_lexicon_tags_7 },
    { 0x28ed52aa, SUC_lexicon_tags_11 },
    { 0xd40d52ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98d52ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2d52b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ad52bb, SUC_lexicon_tags_2 },
    { 0xd68d52bb, SUC_lexicon_tags_11 },
    { 0x8fcd52bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x956d52bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ad52c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfed52c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad52c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ad52cb, SUC_lexicon_tags_19 },
    { 0x45cd52cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe62d52ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x478d52d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cd52d6, SUC_lexicon_tags_57 },
    { 0x1c2d52d7, SUC_lexicon_tags_37 },
    { 0x48d52d8, SUC_lexicon_tags_38 },
    { 0xe86d52d8, SUC_lexicon_tags_49 },
    { 0xe24d52da, SUC_lexicon_tags_23 },
    { 0x50cd52db, SUC_lexicon_tags_60 },
    { 0xead52dc, SUC_lexicon_tags_95 },
    { 0x23ed52dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfccd52df, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cd52e3, SUC_lexicon_tags_7 },
    { 0xe4cd52e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84d52e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4d52ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2d2d52f1, SUC_lexicon_tags_65 },
    { 0x734d52f1, SUC_lexicon_tags_23 },
    { 0xf7ed52f3, SUC_lexicon_tags_6 },
    { 0x560d52f4, SUC_lexicon_tags_7 },
    { 0xc92d52f2, SUC_lexicon_tags_15 },
    { 0xd7ed52f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14d52f9, SUC_lexicon_tags_37 },
    { 0x4d2d52fa, SUC_lexicon_tags_35 },
    { 0x46d52fb, SUC_lexicon_tags_200 },
    { 0x430d52fb, SUC_lexicon_tags_11 },
    { 0x9bed52fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd6ed52ff, SUC_lexicon_tags_68 },
    { 0x6d5300, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb48d5302, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed5308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x448d530a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d530d, SUC_lexicon_tags_25 },
    { 0x904d530d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x336d5310, SUC_lexicon_tags_14 },
    { 0x8dad5310, SUC_lexicon_tags_25 },
    { 0x6a4d5312, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccd531a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d531e, SUC_lexicon_tags_31 },
    { 0x922d531f, SUC_lexicon_tags_7 },
    { 0xe20d531f, SUC_lexicon_tags_15 },
    { 0xae6d5321, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x652d5324, SUC_lexicon_tags_1 },
    { 0xe84d5324, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aad5329, SUC_lexicon_tags_60 },
    { 0xc16d5329, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad5334, SUC_lexicon_tags_25 },
    { 0xe26d5335, SUC_lexicon_tags_11 },
    { 0xaf0d5335, SUC_lexicon_tags_47 },
    { 0xf84d5336, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33cd5339, SUC_lexicon_tags_12 },
    { 0x892d533a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cd533f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd2d5341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2d5346, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8bad5348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6d534b, SUC_lexicon_tags_37 },
    { 0xeccd534c, SUC_lexicon_tags_37 },
    { 0x5ced534d, SUC_lexicon_tags_31 },
    { 0x61cd534b, SUC_lexicon_tags_0 },
    { 0x99ad534f, SUC_lexicon_tags_9 },
    { 0xf46d5350, SUC_lexicon_tags_17 },
    { 0xbcd5351, SUC_lexicon_tags_11 },
    { 0xbfcd5352, SUC_lexicon_tags_11 },
    { 0x218d534e, SUC_lexicon_tags_9 },
    { 0x43ed5353, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18d535a, SUC_lexicon_tags_49 },
    { 0xda8d535a, SUC_lexicon_tags_85 },
    { 0xe94d535b, SUC_lexicon_tags_11 },
    { 0xc1cd535c, SUC_lexicon_tags_43 },
    { 0x410d535e, SUC_lexicon_tags_34 },
    { 0x7a8d535e, SUC_lexicon_tags_31 },
    { 0x9d0d5360, SUC_lexicon_tags_19 },
    { 0x930d5361, SUC_lexicon_tags_50 },
    { 0x976d5360, SUC_lexicon_tags_29 },
    { 0xe4d5361, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x78ed5365, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe4d5367, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7ad5369, SUC_lexicon_tags_64 },
    { 0x246d536a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x626d536c, SUC_lexicon_tags_87 },
    { 0x856d536c, SUC_lexicon_tags_43 },
    { 0xc3ed536e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06d5371, SUC_lexicon_tags_193 },
    { 0xe98d5372, SUC_lexicon_tags_14 },
    { 0x7bad5373, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d537c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ad5380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22d5383, SUC_lexicon_tags_60 },
    { 0x652d5384, SUC_lexicon_tags_2 },
    { 0xbb2d5385, SUC_lexicon_tags_8 },
    { 0xa4d5383, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec4d5388, SUC_lexicon_tags_17 },
    { 0xd40d5389, SUC_lexicon_tags_25 },
    { 0xe6ed538a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccd538d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7ed538f, SUC_lexicon_tags_17 },
    { 0xad4d5390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2d5394, SUC_lexicon_tags_69 },
    { 0xe0d5394, SUC_lexicon_tags_25 },
    { 0x9d4d5394, SUC_lexicon_tags_45 },
    { 0x34d5397, SUC_lexicon_tags_11 },
    { 0xc90d5398, SUC_lexicon_tags_37 },
    { 0x704d5399, SUC_lexicon_tags_28 },
    { 0xe4ad5399, SUC_lexicon_tags_1 },
    { 0xa6ad539b, SUC_lexicon_tags_43 },
    { 0xaacd539c, SUC_lexicon_tags_12 },
    { 0xbaed539d, SUC_lexicon_tags_13 },
    { 0xa26d539b, SUC_lexicon_tags_17 },
    { 0xf02d539f, SUC_lexicon_tags_12 },
    { 0x6dcd539f, SUC_lexicon_tags_1 },
    { 0xb1ad539a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa6d53a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70ed53a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6d53ab, SUC_lexicon_tags_38 },
    { 0x8b2d53ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2cd53ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x534d53b0, SUC_lexicon_tags_7 },
    { 0x480d53b1, SUC_lexicon_tags_88 },
    { 0xe56d53b2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x57ed53b4, SUC_lexicon_tags_42 },
    { 0xeb0d53b5, SUC_lexicon_tags_4 },
    { 0xd16d53b5, SUC_lexicon_tags_11 },
    { 0x924d53b6, SUC_lexicon_tags_16 },
    { 0x4ecd53b5, SUC_lexicon_tags_37 },
    { 0xf50d53b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ced53bd, SUC_lexicon_tags_17 },
    { 0x8ad53bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcd53c2, SUC_lexicon_tags_17 },
    { 0x352d53c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x372d53c6, SUC_lexicon_tags_0 },
    { 0xd1ad53c7, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8d53cc, SUC_lexicon_tags_21 },
    { 0xe56d53cd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xc14d53cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42ed53d1, SUC_lexicon_tags_12 },
    { 0xab4d53d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x818d53d5, SUC_lexicon_tags_37 },
    { 0x52cd53d5, SUC_lexicon_tags_11 },
    { 0xffed53d7, SUC_lexicon_tags_87 },
    { 0xc52d53d8, SUC_lexicon_tags_9 },
    { 0x380d53d7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ad53df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236d53e4, SUC_lexicon_tags_7 },
    { 0xfecd53e4, SUC_lexicon_tags_29 },
    { 0x496d53e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96d53e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0d53ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad53f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cd53fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ad53fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cd5406, SUC_lexicon_tags_45 },
    { 0x39ed5407, SUC_lexicon_tags_16 },
    { 0x4fad5408, SUC_lexicon_tags_1 },
    { 0x2c0d5409, SUC_lexicon_tags_27 },
    { 0xaded5407, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378d5411, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ad5417, SUC_lexicon_tags_45 },
    { 0x530d5418, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74d541b, SUC_lexicon_tags_7 },
    { 0xe2cd541b, SUC_lexicon_tags_33 },
    { 0x99ad541b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ad542a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44d5430, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfad5432, SUC_lexicon_tags_33 },
    { 0x78ad5433, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8d5437, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fed543d, SUC_lexicon_tags_12 },
    { 0x6f0d543e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0d5449, SUC_lexicon_tags_15 },
    { 0x3ccd544a, SUC_lexicon_tags_6 },
    { 0xcccd544a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcd544d, SUC_lexicon_tags_45 },
    { 0xa56d544e, SUC_lexicon_tags_33 },
    { 0x2b0d544f, SUC_lexicon_tags_79 },
    { 0x98ed5450, SUC_lexicon_tags_60 },
    { 0xdd4d5451, SUC_lexicon_tags_1 },
    { 0xf00d5451, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ad5454, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08d5457, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2d545a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x528d545c, SUC_lexicon_tags_9 },
    { 0xe90d545d, SUC_lexicon_tags_31 },
    { 0xed6d545c, SUC_lexicon_tags_7 },
    { 0xe48d545f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54d5462, SUC_lexicon_tags_6 },
    { 0x296d5463, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3cd5465, SUC_lexicon_tags_11 },
    { 0xfe2d5465, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fed5468, SUC_lexicon_tags_15 },
    { 0xca0d5468, SUC_lexicon_tags_37 },
    { 0xb48d546a, SUC_lexicon_tags_17 },
    { 0xbf8d546b, SUC_lexicon_tags_33 },
    { 0xcf6d546b, SUC_lexicon_tags_60 },
    { 0xdacd546a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4ad546f, SUC_lexicon_tags_39 },
    { 0x714d546f, SUC_lexicon_tags_11 },
    { 0xbc2d546f, SUC_lexicon_tags_48 },
    { 0x7f6d5472, SUC_lexicon_tags_11 },
    { 0xccd5472, SUC_lexicon_tags_9 },
    { 0xf08d5474, SUC_lexicon_tags_2 },
    { 0xeaad5474, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ad5478, SUC_lexicon_tags_2 },
    { 0xd36d5479, SUC_lexicon_tags_17 },
    { 0x302d5479, SUC_lexicon_tags_25 },
    { 0x280d547a, SUC_lexicon_tags_45 },
    { 0xdcad5479, SUC_lexicon_tags_27 },
    { 0x9acd547a, SUC_lexicon_tags_46 },
    { 0x1f4d547e, SUC_lexicon_tags_43 },
    { 0x40d547e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d5483, SUC_lexicon_tags_2 },
    { 0x410d5484, SUC_lexicon_tags_12 },
    { 0x4aad5485, SUC_lexicon_tags_37 },
    { 0xc5ed5483, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d548a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6d548d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa42d548f, SUC_lexicon_tags_2 },
    { 0x4cad5490, SUC_lexicon_tags_37 },
    { 0x616d5491, SUC_lexicon_tags_31 },
    { 0x5dad5490, SUC_lexicon_tags_37 },
    { 0x2b6d5493, SUC_lexicon_tags_69 },
    { 0x208d5494, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4d5499, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fad549f, SUC_lexicon_tags_37 },
    { 0xd5cd549f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed6d54a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ed54a5, SUC_lexicon_tags_1 },
    { 0xf08d54a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ed54af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x790d54b1, SUC_lexicon_tags_3 },
    { 0xbced54b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e0d54b4, SUC_lexicon_tags_27 },
    { 0x13cd54b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf6d54b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c2d54b9, SUC_lexicon_tags_43 },
    { 0xb88d54ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316d54bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced54c9, SUC_lexicon_tags_27 },
    { 0x5b0d54ca, SUC_lexicon_tags_2 },
    { 0x902d54cb, SUC_lexicon_tags_28 },
    { 0x91ad54cb, SUC_lexicon_tags_25 },
    { 0x200d54cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e2d54cf, SUC_lexicon_tags_77 },
    { 0x5cd54cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccd54d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x39cd54d6, SUC_lexicon_tags_1 },
    { 0x2c4d54d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ad54dc, SUC_lexicon_tags_33 },
    { 0x1dad54dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8fad54df, SUC_lexicon_tags_43 },
    { 0x360d54df, SUC_lexicon_tags_15 },
    { 0x9a8d54e1, SUC_lexicon_tags_37 },
    { 0x5d6d54e2, SUC_lexicon_tags_45 },
    { 0xa70d54e1, SUC_lexicon_tags_1 },
    { 0x178d54e4, SUC_lexicon_tags_23 },
    { 0xe34d54e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51ad54e7, SUC_lexicon_tags_12 },
    { 0xc20d54e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cd54eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ccd54ed, SUC_lexicon_tags_10 },
    { 0xad0d54ee, SUC_lexicon_tags_39 },
    { 0xb56d54ed, SUC_lexicon_tags_49 },
    { 0x2c0d54ef, SUC_lexicon_tags_1 },
    { 0x5dcd54f0, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaed54f4, SUC_lexicon_tags_11 },
    { 0xe74d54f5, SUC_lexicon_tags_25 },
    { 0x38cd54f4, SUC_lexicon_tags_17 },
    { 0x95ed54f7, SUC_lexicon_tags_15 },
    { 0xb82d54f6, SUC_lexicon_tags_29 },
    { 0xa84d54f9, SUC_lexicon_tags_37 },
    { 0x28ad54fa, SUC_lexicon_tags_45 },
    { 0x164d54fa, SUC_lexicon_tags_66 },
    { 0x7ced54fc, SUC_lexicon_tags_61 },
    { 0x6bcd54fd, SUC_lexicon_tags_85 },
    { 0x250d54fc, SUC_lexicon_tags_1 },
    { 0x966d54fb, SUC_lexicon_tags_17 },
    { 0xdccd54fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf0d5502, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4d5505, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa4d5507, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2d550c, SUC_lexicon_tags_0 },
    { 0xd6ad550d, SUC_lexicon_tags_25 },
    { 0xd22d550d, SUC_lexicon_tags_795 },
    { 0xe18d550f, SUC_lexicon_tags_43 },
    { 0x1e8d5510, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8cd5512, SUC_lexicon_tags_7 },
    { 0x906d5513, SUC_lexicon_tags_23 },
    { 0xb16d5513, SUC_lexicon_tags_11 },
    { 0x254d5515, SUC_lexicon_tags_39 },
    { 0x4c6d5513, SUC_lexicon_tags_49 },
    { 0x3f4d5517, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa38d5519, SUC_lexicon_tags_63 },
    { 0xbfed551a, SUC_lexicon_tags_39 },
    { 0x176d551b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x332d551d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ed5523, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x69cd5525, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbdcd5527, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e8d5529, SUC_lexicon_tags_11 },
    { 0x704d552a, SUC_lexicon_tags_15 },
    { 0x7ced5529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e8d552d, SUC_lexicon_tags_12 },
    { 0xbbad552e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f8d5530, SUC_lexicon_tags_1 },
    { 0x2f0d5531, SUC_lexicon_tags_77 },
    { 0xc3ad5531, SUC_lexicon_tags_0 },
    { 0x222d5533, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4d5536, SUC_lexicon_tags_1 },
    { 0x1cd5537, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x87ed5539, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x806d553c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20d5540, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ad5549, SUC_lexicon_tags_2 },
    { 0xfb4d554a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d5550, SUC_lexicon_tags_37 },
    { 0xcb8d5551, SUC_lexicon_tags_11 },
    { 0x506d5552, SUC_lexicon_tags_14 },
    { 0x5f8d5553, SUC_lexicon_tags_11 },
    { 0x4d8d5553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ad5557, SUC_lexicon_tags_48 },
    { 0xc54d5558, SUC_lexicon_tags_37 },
    { 0x7ccd5557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcd555c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ad5563, SUC_lexicon_tags_19 },
    { 0x3a4d5563, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd4d5566, SUC_lexicon_tags_12 },
    { 0xd92d5566, SUC_lexicon_tags_15 },
    { 0xdfcd5568, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0d556b, SUC_lexicon_tags_28 },
    { 0x6fad556c, SUC_lexicon_tags_17 },
    { 0x12ad556c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fad5576, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb1cd5578, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cd557e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ad5581, SUC_lexicon_tags_178 },
    { 0x89ad5582, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796d5589, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x410d558b, SUC_lexicon_tags_31 },
    { 0xa2cd558c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa54d558e, SUC_lexicon_tags_37 },
    { 0x48cd558f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7d8d5591, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5bad5593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8d5596, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6d559d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x944d559f, SUC_lexicon_tags_7 },
    { 0x436d55a0, SUC_lexicon_tags_47 },
    { 0x6a4d55a1, SUC_lexicon_tags_17 },
    { 0x32d559f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cd55a8, SUC_lexicon_tags_11 },
    { 0x97cd55a8, SUC_lexicon_tags_12 },
    { 0x142d55aa, SUC_lexicon_tags_1 },
    { 0xd1cd55a8, SUC_lexicon_tags_28 },
    { 0x248d55a8, SUC_lexicon_tags_46 },
    { 0xb5ad55ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6d55b2, SUC_lexicon_tags_25 },
    { 0x8f0d55b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fcd55b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e0d55b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdad55bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4d55be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468d55c5, SUC_lexicon_tags_45 },
    { 0x794d55c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ced55cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef0d55cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0d55d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf46d55d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e6d55d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206d55d9, SUC_lexicon_tags_32 },
    { 0x8e4d55da, SUC_lexicon_tags_1 },
    { 0x13ad55d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2d55e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4d55ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08d55ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ad55f4, SUC_lexicon_tags_38 },
    { 0x500d55f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d55f8, SUC_lexicon_tags_8 },
    { 0x4eed55f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594d55fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf76d5600, SUC_lexicon_tags_7 },
    { 0xf9ad5601, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96d5604, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60d560a, SUC_lexicon_tags_13 },
    { 0x632d560b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600d560f, SUC_lexicon_tags_25 },
    { 0xf6ed5610, SUC_lexicon_tags_423 },
    { 0xbced5611, SUC_lexicon_tags_65 },
    { 0x538d560f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x476d5615, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb88d5617, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x766d561a, SUC_lexicon_tags_105 },
    { 0xe82d561b, SUC_lexicon_tags_37 },
    { 0xff4d561b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfced5620, SUC_lexicon_tags_55 },
    { 0xefcd5621, SUC_lexicon_tags_38 },
    { 0xdaad5622, SUC_lexicon_tags_63 },
    { 0xc92d5623, SUC_lexicon_tags_26 },
    { 0x482d5622, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6d5627, SUC_lexicon_tags_15 },
    { 0x77cd5627, SUC_lexicon_tags_27 },
    { 0xb22d5629, SUC_lexicon_tags_62 },
    { 0x3bad562a, SUC_lexicon_tags_7 },
    { 0x64cd562b, SUC_lexicon_tags_15 },
    { 0xa9ed562c, SUC_lexicon_tags_48 },
    { 0x794d5629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2d5634, SUC_lexicon_tags_7 },
    { 0x360d5635, SUC_lexicon_tags_37 },
    { 0x212d5634, SUC_lexicon_tags_17 },
    { 0xf78d5634, SUC_lexicon_tags_27 },
    { 0x348d5637, SUC_lexicon_tags_55 },
    { 0xda6d5639, SUC_lexicon_tags_60 },
    { 0x592d5639, SUC_lexicon_tags_7 },
    { 0xb08d563a, SUC_lexicon_tags_30 },
    { 0xb16d563b, SUC_lexicon_tags_1 },
    { 0x742d5639, SUC_lexicon_tags_6 },
    { 0x208d563e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdad5641, SUC_lexicon_tags_11 },
    { 0x1dad5641, SUC_lexicon_tags_15 },
    { 0xb88d5643, SUC_lexicon_tags_63 },
    { 0xabad5644, SUC_lexicon_tags_15 },
    { 0x248d5645, SUC_lexicon_tags_87 },
    { 0x200d5645, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x40ad5648, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88d564e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x162d5650, SUC_lexicon_tags_236 },
    { 0xddad5651, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56d5655, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70d565e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1eed5660, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eed5667, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56d566c, SUC_lexicon_tags_68 },
    { 0xbead566d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e2d566f, SUC_lexicon_tags_49 },
    { 0x644d566f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cd5674, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcc4d5676, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18d567b, SUC_lexicon_tags_24 },
    { 0xbded567c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556d5680, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ed5684, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4d5687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x286d5689, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x948d568b, SUC_lexicon_tags_65 },
    { 0x2f2d568b, SUC_lexicon_tags_48 },
    { 0x244d568d, SUC_lexicon_tags_21 },
    { 0x462d568e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786d5692, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cd5696, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cd56a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad56aa, SUC_lexicon_tags_2 },
    { 0xa3ad56ab, SUC_lexicon_tags_44 },
    { 0x3ed56ab, SUC_lexicon_tags_15 },
    { 0xdbed56ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x588d56af, SUC_lexicon_tags_13 },
    { 0x1e8d56b0, SUC_lexicon_tags_6 },
    { 0x2a8d56af, SUC_lexicon_tags_33 },
    { 0xc34d56b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad56b5, SUC_lexicon_tags_7 },
    { 0xcad56b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d56bb, SUC_lexicon_tags_2 },
    { 0x51ad56bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26ad56be, SUC_lexicon_tags_2 },
    { 0x64d56bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c0d56c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90ed56c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4d56c7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecd56cc, SUC_lexicon_tags_11 },
    { 0x33ed56cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcd56d0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ad56d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ed56da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x19cd56dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0d56e4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ad56e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ced56ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110d56f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f0d56f5, SUC_lexicon_tags_25 },
    { 0x3c6d56f5, SUC_lexicon_tags_19 },
    { 0xf7ad56f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354d56fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d5701, SUC_lexicon_tags_11 },
    { 0x2f8d5702, SUC_lexicon_tags_19 },
    { 0xa48d5702, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecd5709, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8d570c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ad5710, SUC_lexicon_tags_25 },
    { 0xf86d5711, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4d5713, SUC_lexicon_tags_47 },
    { 0x14d5714, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x84ed5716, SUC_lexicon_tags_15 },
    { 0x8fad5717, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6d571c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4d571f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cd5724, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978d572d, SUC_lexicon_tags_7 },
    { 0x25ad572d, SUC_lexicon_tags_11 },
    { 0x326d572f, SUC_lexicon_tags_83 },
    { 0x414d5730, SUC_lexicon_tags_37 },
    { 0x3d4d572f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccd5733, SUC_lexicon_tags_15 },
    { 0x1f6d5734, SUC_lexicon_tags_37 },
    { 0x64ad5735, SUC_lexicon_tags_27 },
    { 0xd4ed5734, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66d573a, SUC_lexicon_tags_11 },
    { 0xf8d573b, SUC_lexicon_tags_25 },
    { 0xcaad573b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x520d573e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56d5740, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x658d5743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeed5749, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ed574f, SUC_lexicon_tags_23 },
    { 0x2b6d574f, SUC_lexicon_tags_15 },
    { 0x8dad5751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x67ed5753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeed5757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb4d5759, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3ad575b, SUC_lexicon_tags_8 },
    { 0x816d575c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aed5760, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86d5764, SUC_lexicon_tags_1 },
    { 0x8e8d5765, SUC_lexicon_tags_9 },
    { 0xbe0d5765, SUC_lexicon_tags_253 },
    { 0xdad5767, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4d5770, SUC_lexicon_tags_39 },
    { 0x76d5771, SUC_lexicon_tags_34 },
    { 0x390d5771, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0x76ad5774, SUC_lexicon_tags_19 },
    { 0x4a0d5775, SUC_lexicon_tags_0 },
    { 0xf88d5776, SUC_lexicon_tags_69 },
    { 0x442d5777, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900d577b, SUC_lexicon_tags_15 },
    { 0xba6d577c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ed5785, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf16d5787, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ad578e, SUC_lexicon_tags_61 },
    { 0x10cd578e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ed5792, SUC_lexicon_tags_1 },
    { 0x258d5793, SUC_lexicon_tags_17 },
    { 0x524d5794, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fed5796, SUC_lexicon_tags_1 },
    { 0xdaed5796, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x296d5799, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x49cd579c, SUC_lexicon_tags_1 },
    { 0xfa4d579d, SUC_lexicon_tags_30 },
    { 0x482d579e, SUC_lexicon_tags_46 },
    { 0x772d579c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x558d57a1, SUC_lexicon_tags_7 },
    { 0xcb4d57a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ad57a7, SUC_lexicon_tags_45 },
    { 0x2ecd57a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ed57ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa94d57ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb78d57af, SUC_lexicon_tags_1 },
    { 0x7ed57af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed57b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ed57b8, SUC_lexicon_tags_15 },
    { 0x6f4d57b8, SUC_lexicon_tags_37 },
    { 0x25cd57ba, SUC_lexicon_tags_11 },
    { 0x252d57bb, SUC_lexicon_tags_264 },
    { 0x442d57bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6d57be, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x834d57c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d57c6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848d57cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46d57d0, SUC_lexicon_tags_158 },
    { 0xfb8d57d1, SUC_lexicon_tags_39 },
    { 0x252d57d2, SUC_lexicon_tags_2 },
    { 0x828d57d1, SUC_lexicon_tags_33 },
    { 0xf2ad57d3, SUC_lexicon_tags_25 },
    { 0x712d57d3, SUC_lexicon_tags_7 },
    { 0x1f8d57d6, SUC_lexicon_tags_45 },
    { 0x9c8d57d7, SUC_lexicon_tags_3 },
    { 0xca8d57d8, SUC_lexicon_tags_37 },
    { 0xe72d57d9, SUC_lexicon_tags_6 },
    { 0xeb2d57da, SUC_lexicon_tags_11 },
    { 0xd10d57d9, SUC_lexicon_tags_15 },
    { 0xf80d57db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cd57e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43ad57e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x802d57e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabed57e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26d57eb, SUC_lexicon_tags_14 },
    { 0xd86d57ec, SUC_lexicon_tags_37 },
    { 0x25ad57ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74d57f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fad57f2, SUC_lexicon_tags_7 },
    { 0xb78d57f2, SUC_lexicon_tags_28 },
    { 0x2ead57f4, SUC_lexicon_tags_11 },
    { 0x8d0d57f5, SUC_lexicon_tags_46 },
    { 0x79cd57f5, SUC_lexicon_tags_7 },
    { 0x97ad57f2, SUC_lexicon_tags_17 },
    { 0xb82d57f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6d57fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x166d57fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d5801, SUC_lexicon_tags_46 },
    { 0x6bad5802, SUC_lexicon_tags_23 },
    { 0x5f8d5803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bad5806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22d580b, SUC_lexicon_tags_9 },
    { 0x29ed580b, SUC_lexicon_tags_60 },
    { 0x6e4d580d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad5810, SUC_lexicon_tags_7 },
    { 0x3f0d5810, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6d5815, SUC_lexicon_tags_31 },
    { 0xaa2d5816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x982d5818, SUC_lexicon_tags_12 },
    { 0xe36d5819, SUC_lexicon_tags_8 },
    { 0x5ed581a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x3d0d581c, SUC_lexicon_tags_37 },
    { 0xed4d581c, SUC_lexicon_tags_11 },
    { 0xb68d581e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ead5823, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d5826, SUC_lexicon_tags_79 },
    { 0xa18d5827, SUC_lexicon_tags_1 },
    { 0x1d0d5827, SUC_lexicon_tags_1 },
    { 0xe28d5829, SUC_lexicon_tags_7 },
    { 0xae4d582a, SUC_lexicon_tags_7 },
    { 0xb4ad582b, SUC_lexicon_tags_7 },
    { 0x900d582b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8e0d582e, SUC_lexicon_tags_79 },
    { 0x470d582f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ad5832, SUC_lexicon_tags_19 },
    { 0x414d5833, SUC_lexicon_tags_1 },
    { 0x50d5834, SUC_lexicon_tags_66 },
    { 0x668d5835, SUC_lexicon_tags_47 },
    { 0x2cd5835, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf40d5838, SUC_lexicon_tags_15 },
    { 0x486d5838, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x270d583b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xda6d583d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ad5841, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826d5848, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8d584c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x324d584f, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ad5854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x518d5856, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96cd5859, SUC_lexicon_tags_10 },
    { 0xa3ed5859, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0d5861, SUC_lexicon_tags_29 },
    { 0xc4cd5862, SUC_lexicon_tags_9 },
    { 0x8a8d5863, SUC_lexicon_tags_15 },
    { 0xb2ed5864, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x132d5867, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ad586a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ead5871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ad587d, SUC_lexicon_tags_57 },
    { 0x22d587e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ad5887, SUC_lexicon_tags_12 },
    { 0x4eed5887, SUC_lexicon_tags_65 },
    { 0x2e2d5887, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cd5892, SUC_lexicon_tags_30 },
    { 0x52d5893, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18d5896, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x50cd5898, SUC_lexicon_tags_23 },
    { 0xca2d5898, SUC_lexicon_tags_35 },
    { 0xc16d5899, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668d589e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2d58a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978d58a6, SUC_lexicon_tags_37 },
    { 0xcded58a6, SUC_lexicon_tags_38 },
    { 0xae0d58a8, SUC_lexicon_tags_33 },
    { 0x18ad58a6, SUC_lexicon_tags_118 },
    { 0x9ccd58a8, SUC_lexicon_tags_63 },
    { 0x73ad58ab, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406d58b0, SUC_lexicon_tags_29 },
    { 0x872d58b0, SUC_lexicon_tags_7 },
    { 0x352d58b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffed58b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x462d58b8, SUC_lexicon_tags_1 },
    { 0x700d58b8, SUC_lexicon_tags_45 },
    { 0xa7ad58b9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2d58bd, SUC_lexicon_tags_12 },
    { 0xbccd58be, SUC_lexicon_tags_27 },
    { 0x1e0d58bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f2d58c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222d58c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dad58ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf6ed58cc, SUC_lexicon_tags_11 },
    { 0x2c2d58cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cd58d3, SUC_lexicon_tags_29 },
    { 0x5ded58d4, SUC_lexicon_tags_87 },
    { 0x72ad58d5, SUC_lexicon_tags_25 },
    { 0xadcd58d5, SUC_lexicon_tags_44 },
    { 0x3acd58d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcd58d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xef4d58db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710d58e0, SUC_lexicon_tags_68 },
    { 0xcf8d58e1, SUC_lexicon_tags_31 },
    { 0x9cad58e1, SUC_lexicon_tags_30 },
    { 0x2a6d58e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8d58e6, SUC_lexicon_tags_11 },
    { 0x3e0d58e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30d58ea, SUC_lexicon_tags_29 },
    { 0x7a0d58ea, SUC_lexicon_tags_2 },
    { 0xc32d58ec, SUC_lexicon_tags_47 },
    { 0xa14d58ec, SUC_lexicon_tags_32 },
    { 0xb2cd58ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804d58f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cd58f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330d58fc, SUC_lexicon_tags_25 },
    { 0x864d58fc, SUC_lexicon_tags_103 },
    { 0x9d4d58fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810d5903, SUC_lexicon_tags_102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ad5907, SUC_lexicon_tags_30 },
    { 0x584d5907, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed590b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ad5915, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb26d5917, SUC_lexicon_tags_17 },
    { 0x7fcd5918, SUC_lexicon_tags_23 },
    { 0x3ded5917, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8d5920, SUC_lexicon_tags_5 },
    { 0x2b8d5921, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x6e6d5923, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf56d5925, SUC_lexicon_tags_28 },
    { 0x58d5926, SUC_lexicon_tags_27 },
    { 0x720d5925, SUC_lexicon_tags_15 },
    { 0xec6d5925, SUC_lexicon_tags_87 },
    { 0x42d5929, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddcd592b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe8ad592d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ed5931, SUC_lexicon_tags_1 },
    { 0x22ed5931, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfced5936, SUC_lexicon_tags_14 },
    { 0xed0d5937, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x90ed5939, SUC_lexicon_tags_62 },
    { 0x4d5939, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b4d593c, SUC_lexicon_tags_351 },
    { 0x4bad593c, SUC_lexicon_tags_7 },
    { 0x8b4d593e, SUC_lexicon_tags_37 },
    { 0xfc8d593e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7e6d5941, SUC_lexicon_tags_11 },
    { 0xdb2d5942, SUC_lexicon_tags_48 },
    { 0xdeed5943, SUC_lexicon_tags_25 },
    { 0x520d5942, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d594a, SUC_lexicon_tags_170 },
    { 0x6ced594b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36d594f, SUC_lexicon_tags_26 },
    { 0x406d594f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe30d5952, SUC_lexicon_tags_7 },
    { 0xe2cd5952, SUC_lexicon_tags_1 },
    { 0xfb0d5952, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490d5959, SUC_lexicon_tags_8 },
    { 0xdbed5959, SUC_lexicon_tags_26 },
    { 0x788d595b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ded5964, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbced596e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcccd5970, SUC_lexicon_tags_43 },
    { 0x19ed5971, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x152d5974, SUC_lexicon_tags_81 },
    { 0xa3cd5975, SUC_lexicon_tags_1 },
    { 0x646d5976, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ad597b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6d597e, SUC_lexicon_tags_15 },
    { 0xa5ad597f, SUC_lexicon_tags_7 },
    { 0x58ad597f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08d5987, SUC_lexicon_tags_37 },
    { 0x118d5988, SUC_lexicon_tags_17 },
    { 0x924d5989, SUC_lexicon_tags_47 },
    { 0xb74d598a, SUC_lexicon_tags_9 },
    { 0x204d598b, SUC_lexicon_tags_12 },
    { 0xf28d5988, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x45ed598e, SUC_lexicon_tags_1 },
    { 0x6d4d598e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x654d5991, SUC_lexicon_tags_28 },
    { 0x182d5992, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf06d5994, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ed599a, SUC_lexicon_tags_25 },
    { 0xc64d599b, SUC_lexicon_tags_0 },
    { 0xa2ed599c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4d599f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe4cd59a1, SUC_lexicon_tags_48 },
    { 0x246d59a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ed59a8, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x698d59aa, SUC_lexicon_tags_79 },
    { 0x250d59ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfed59ae, SUC_lexicon_tags_118 },
    { 0x276d59ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04d59b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedad59b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54cd59b9, SUC_lexicon_tags_49 },
    { 0x66ed59ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb80d59bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x39cd59be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaed59c0, SUC_lexicon_tags_25 },
    { 0x950d59c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcded59c5, SUC_lexicon_tags_2 },
    { 0x4cad59c5, SUC_lexicon_tags_9 },
    { 0x968d59c7, SUC_lexicon_tags_46 },
    { 0x4e4d59c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72d59ca, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64d59cd, SUC_lexicon_tags_46 },
    { 0x292d59ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0d59d4, SUC_lexicon_tags_30 },
    { 0x5dcd59d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d59da, SUC_lexicon_tags_25 },
    { 0x12cd59db, SUC_lexicon_tags_29 },
    { 0x382d59da, SUC_lexicon_tags_79 },
    { 0x5b4d59dd, SUC_lexicon_tags_25 },
    { 0xa84d59de, SUC_lexicon_tags_17 },
    { 0x4a6d59df, SUC_lexicon_tags_9 },
    { 0x7cad59e0, SUC_lexicon_tags_26 },
    { 0x93cd59e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392d59e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54d59e9, SUC_lexicon_tags_1 },
    { 0xb54d59ea, SUC_lexicon_tags_28 },
    { 0x412d59eb, SUC_lexicon_tags_37 },
    { 0x88cd59ea, SUC_lexicon_tags_12 },
    { 0xc6d59ed, SUC_lexicon_tags_43 },
    { 0x6f6d59eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xab4d59f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bed59f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c6d59f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ad5a08, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x172d5a0a, SUC_lexicon_tags_34 },
    { 0x2e8d5a0b, SUC_lexicon_tags_2 },
    { 0xc08d5a0c, SUC_lexicon_tags_91 },
    { 0xb1ad5a0b, SUC_lexicon_tags_60 },
    { 0xb6ad5a0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4d5a16, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ed5a1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d2d5a1e, SUC_lexicon_tags_17 },
    { 0x520d5a1f, SUC_lexicon_tags_0 },
    { 0x132d5a20, SUC_lexicon_tags_7 },
    { 0x81ed5a20, SUC_lexicon_tags_2 },
    { 0xd92d5a22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634d5a26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39cd5a29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cd5a2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcb0d5a2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24d5a34, SUC_lexicon_tags_1 },
    { 0x488d5a35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ad5a3d, SUC_lexicon_tags_25 },
    { 0xff2d5a3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ad5a42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc8d5a44, SUC_lexicon_tags_14 },
    { 0x67cd5a45, SUC_lexicon_tags_43 },
    { 0x7c6d5a46, SUC_lexicon_tags_2 },
    { 0x4bad5a47, SUC_lexicon_tags_6 },
    { 0x9dcd5a47, SUC_lexicon_tags_37 },
    { 0x80ed5a49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aad5a4c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10d5a4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ed5a51, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x50cd5a54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4b2d5a56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x11ad5a58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6d5a5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c0d5a5f, SUC_lexicon_tags_33 },
    { 0xf62d5a60, SUC_lexicon_tags_15 },
    { 0xcdad5a61, SUC_lexicon_tags_12 },
    { 0x400d5a62, SUC_lexicon_tags_45 },
    { 0x31ad5a63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x94d5a65, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330d5a69, SUC_lexicon_tags_10 },
    { 0xfacd5a69, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60d5a6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7ed5a71, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x272d5a73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaed5a76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad5a7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8d5a89, SUC_lexicon_tags_66 },
    { 0x454d5a89, SUC_lexicon_tags_37 },
    { 0x2aad5a8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04d5a95, SUC_lexicon_tags_49 },
    { 0xf30d5a96, SUC_lexicon_tags_30 },
    { 0xf84d5a97, SUC_lexicon_tags_46 },
    { 0x71ad5a96, SUC_lexicon_tags_17 },
    { 0xba2d5a99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88d5a9e, SUC_lexicon_tags_16 },
    { 0x722d5a9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dad5aa2, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cd5aa6, SUC_lexicon_tags_34 },
    { 0xa3ed5aa7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x374d5aa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4ad5aab, SUC_lexicon_tags_12 },
    { 0xb56d5aac, SUC_lexicon_tags_11 },
    { 0x22ad5aac, SUC_lexicon_tags_79 },
    { 0x5ded5aac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612d5ab2, SUC_lexicon_tags_39 },
    { 0x4d5ab3, SUC_lexicon_tags_28 },
    { 0xf1cd5ab4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ad5ab7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18d5aba, SUC_lexicon_tags_33 },
    { 0xe2ad5abb, SUC_lexicon_tags_23 },
    { 0x46ad5abc, SUC_lexicon_tags_38 },
    { 0x774d5aba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x962d5abf, SUC_lexicon_tags_1 },
    { 0x71ad5abf, SUC_lexicon_tags_37 },
    { 0xdacd5ac1, SUC_lexicon_tags_31 },
    { 0xf6d5ac2, SUC_lexicon_tags_2 },
    { 0x220d5ac2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa64d5ac5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e8d5ac7, SUC_lexicon_tags_276 },
    { 0x924d5ac8, SUC_lexicon_tags_25 },
    { 0x5cad5ac9, SUC_lexicon_tags_11 },
    { 0x832d5aca, SUC_lexicon_tags_49 },
    { 0x404d5acb, SUC_lexicon_tags_39 },
    { 0xf3ed5acc, SUC_lexicon_tags_7 },
    { 0x202d5acd, SUC_lexicon_tags_40 },
    { 0x512d5ace, SUC_lexicon_tags_82 },
    { 0x35cd5ace, SUC_lexicon_tags_43 },
    { 0xc00d5ace, SUC_lexicon_tags_27 },
    { 0xb86d5acd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6d5ad5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72d5ade, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3a2d5ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x164d5ae2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2d5ae9, SUC_lexicon_tags_25 },
    { 0x7fcd5ae9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xead5aec, SUC_lexicon_tags_21 },
    { 0xbd4d5aed, SUC_lexicon_tags_28 },
    { 0xfe4d5aee, SUC_lexicon_tags_27 },
    { 0xad0d5aed, SUC_lexicon_tags_12 },
    { 0xb1ad5af0, SUC_lexicon_tags_37 },
    { 0x44ad5af0, SUC_lexicon_tags_25 },
    { 0x252d5af1, SUC_lexicon_tags_27 },
    { 0x9b4d5af3, SUC_lexicon_tags_19 },
    { 0xa2cd5af2, SUC_lexicon_tags_47 },
    { 0x716d5af5, SUC_lexicon_tags_32 },
    { 0xaa8d5af6, SUC_lexicon_tags_10 },
    { 0x408d5af5, SUC_lexicon_tags_66 },
    { 0xd2ed5af8, SUC_lexicon_tags_1 },
    { 0xb00d5af9, SUC_lexicon_tags_12 },
    { 0x716d5afa, SUC_lexicon_tags_14 },
    { 0x852d5af6, SUC_lexicon_tags_1 },
    { 0xaf2d5afc, SUC_lexicon_tags_37 },
    { 0x7d4d5af1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcd5b02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ad5b06, SUC_lexicon_tags_7 },
    { 0xa5ad5b06, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cd5b0e, SUC_lexicon_tags_37 },
    { 0x7c6d5b0f, SUC_lexicon_tags_34 },
    { 0x962d5b10, SUC_lexicon_tags_17 },
    { 0x744d5b11, SUC_lexicon_tags_17 },
    { 0xcf8d5b0f, SUC_lexicon_tags_4 },
    { 0x3ead5b12, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3cad5b15, SUC_lexicon_tags_30 },
    { 0x2d6d5b16, SUC_lexicon_tags_13 },
    { 0x4ad5b17, SUC_lexicon_tags_14 },
    { 0x840d5b18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68d5b1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bed5b1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4d5b22, SUC_lexicon_tags_19 },
    { 0x9bcd5b23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebed5b2b, SUC_lexicon_tags_25 },
    { 0xbb0d5b2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d5b37, SUC_lexicon_tags_32 },
    { 0xac0d5b38, SUC_lexicon_tags_1 },
    { 0xf62d5b39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2d5b3d, SUC_lexicon_tags_1 },
    { 0x6ced5b3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2d5b44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x416d5b46, SUC_lexicon_tags_23 },
    { 0x794d5b47, SUC_lexicon_tags_45 },
    { 0x97ad5b48, SUC_lexicon_tags_2 },
    { 0xd34d5b48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d5b4c, SUC_lexicon_tags_15 },
    { 0xd98d5b4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0d5b52, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ed5b56, SUC_lexicon_tags_47 },
    { 0xecd5b56, SUC_lexicon_tags_29 },
    { 0xc88d5b58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8d5b5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42d5b60, SUC_lexicon_tags_16 },
    { 0x682d5b61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d5b67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918d5b6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4d5b73, SUC_lexicon_tags_37 },
    { 0x906d5b74, SUC_lexicon_tags_74 },
    { 0x488d5b74, SUC_lexicon_tags_7 },
    { 0xcfcd5b76, SUC_lexicon_tags_2 },
    { 0xf1ed5b75, SUC_lexicon_tags_25 },
    { 0xd74d5b78, SUC_lexicon_tags_11 },
    { 0xd22d5b78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8d5b7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dad5b86, SUC_lexicon_tags_37 },
    { 0x94cd5b87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x894d5b8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x436d5b8c, SUC_lexicon_tags_7 },
    { 0xefcd5b8c, SUC_lexicon_tags_37 },
    { 0x59cd5b8e, SUC_lexicon_tags_47 },
    { 0xcad5b8f, SUC_lexicon_tags_17 },
    { 0x8d8d5b8c, SUC_lexicon_tags_7 },
    { 0x1b0d5b8f, SUC_lexicon_tags_25 },
    { 0xd24d5b92, SUC_lexicon_tags_47 },
    { 0xa66d5b90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc6d5b95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416d5b9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62d5b9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cd5ba2, SUC_lexicon_tags_15 },
    { 0xe42d5ba2, SUC_lexicon_tags_23 },
    { 0x59cd5ba4, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0d5ba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900d5bae, SUC_lexicon_tags_38 },
    { 0x80cd5baf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bad5bba, SUC_lexicon_tags_38 },
    { 0xbf0d5bbb, SUC_lexicon_tags_32 },
    { 0xd4ed5bbc, SUC_lexicon_tags_10 },
    { 0xe00d5bbd, SUC_lexicon_tags_17 },
    { 0xb92d5bbe, SUC_lexicon_tags_2 },
    { 0x81cd5bbf, SUC_lexicon_tags_7 },
    { 0xb0ed5bbf, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4d5bc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x994d5bc9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ad5bcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7ed5bce, SUC_lexicon_tags_29 },
    { 0x80ad5bcf, SUC_lexicon_tags_1 },
    { 0x6e4d5bd0, SUC_lexicon_tags_37 },
    { 0x4d6d5bd0, SUC_lexicon_tags_87 },
    { 0x418d5bd2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664d5be0, SUC_lexicon_tags_6 },
    { 0x7e0d5be0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8d5be4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98cd5be6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40d5bea, SUC_lexicon_tags_31 },
    { 0x5bcd5bea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x648d5bed, SUC_lexicon_tags_38 },
    { 0x82ad5bed, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846d5bf4, SUC_lexicon_tags_19 },
    { 0xe54d5bf5, SUC_lexicon_tags_43 },
    { 0x6bad5bf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8d5bfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x408d5bfe, SUC_lexicon_tags_65 },
    { 0xcf0d5bff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x264d5c01, SUC_lexicon_tags_14 },
    { 0xfcad5c02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed5c07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x722d5c09, SUC_lexicon_tags_15 },
    { 0x250d5c0a, SUC_lexicon_tags_25 },
    { 0x986d5c0b, SUC_lexicon_tags_9 },
    { 0x66d5c0c, SUC_lexicon_tags_2 },
    { 0x5f8d5c0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0d5c13, SUC_lexicon_tags_17 },
    { 0xc4d5c14, SUC_lexicon_tags_68 },
    { 0xdeed5c14, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x752d5c17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8d5c1a, SUC_lexicon_tags_37 },
    { 0xcf6d5c1a, SUC_lexicon_tags_7 },
    { 0xcad5c1c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24d5c20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x304d5c23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46d5c26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86d5c2d, SUC_lexicon_tags_2 },
    { 0xe12d5c2e, SUC_lexicon_tags_17 },
    { 0xca2d5c2f, SUC_lexicon_tags_19 },
    { 0x4fed5c30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb44d5c32, SUC_lexicon_tags_77 },
    { 0xdc6d5c32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2cd5c35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856d5c3b, SUC_lexicon_tags_1539 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80d5c42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d5c4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cd5c4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x432d5c51, SUC_lexicon_tags_25 },
    { 0xdbad5c52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd5c55, SUC_lexicon_tags_28 },
    { 0xaccd5c56, SUC_lexicon_tags_0 },
    { 0xe98d5c57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d5c5a, SUC_lexicon_tags_15 },
    { 0xb34d5c5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ed5c5f, SUC_lexicon_tags_31 },
    { 0x72cd5c60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x332d5c63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672d5c69, SUC_lexicon_tags_38 },
    { 0xc0ed5c69, SUC_lexicon_tags_37 },
    { 0xdb0d5c6b, SUC_lexicon_tags_37 },
    { 0x53ed5c6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20ed5c6e, SUC_lexicon_tags_77 },
    { 0xc72d5c6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x326d5c72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8d5c79, SUC_lexicon_tags_19 },
    { 0xd34d5c79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ad5c7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x874d5c7f, SUC_lexicon_tags_61 },
    { 0x896d5c80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364d5c84, SUC_lexicon_tags_55 },
    { 0x40d5c85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44ed5c87, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2d5c8a, SUC_lexicon_tags_63 },
    { 0x3eed5c8b, SUC_lexicon_tags_16 },
    { 0x16ed5c8a, SUC_lexicon_tags_12 },
    { 0xc78d5c8d, SUC_lexicon_tags_7 },
    { 0x394d5c8e, SUC_lexicon_tags_14 },
    { 0xc5ed5c8f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc54d5c91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb8d5c93, SUC_lexicon_tags_17 },
    { 0x408d5c94, SUC_lexicon_tags_29 },
    { 0x5f0d5c93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d5c9a, SUC_lexicon_tags_7 },
    { 0xc0d5c9a, SUC_lexicon_tags_34 },
    { 0x6a2d5c9c, SUC_lexicon_tags_12 },
    { 0xa2cd5c9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8d5ca0, SUC_lexicon_tags_49 },
    { 0xbc6d5ca0, SUC_lexicon_tags_79 },
    { 0xfcad5ca2, SUC_lexicon_tags_38 },
    { 0x2a6d5ca3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96d5ca5, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0x246d5ca7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x66cd5ca9, SUC_lexicon_tags_4 },
    { 0xded5caa, SUC_lexicon_tags_17 },
    { 0x8d0d5ca9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ed5caf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ed5cb5, SUC_lexicon_tags_63 },
    { 0x7f0d5cb6, SUC_lexicon_tags_47 },
    { 0xe8d5cb7, SUC_lexicon_tags_42 },
    { 0x6a2d5cb5, SUC_lexicon_tags_39 },
    { 0xfe6d5cb9, SUC_lexicon_tags_43 },
    { 0x390d5cba, SUC_lexicon_tags_15 },
    { 0x9aad5cbb, SUC_lexicon_tags_25 },
    { 0x5f8d5cbc, SUC_lexicon_tags_60 },
    { 0xd98d5cb6, SUC_lexicon_tags_14 },
    { 0x6c0d5cbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc68d5cc0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d5cc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688d5cd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d5cd3, SUC_lexicon_tags_14 },
    { 0x8f0d5cd4, SUC_lexicon_tags_179 },
    { 0x908d5cd5, SUC_lexicon_tags_28 },
    { 0xbbad5cd6, SUC_lexicon_tags_23 },
    { 0xd28d5cd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18ed5cd9, SUC_lexicon_tags_49 },
    { 0x668d5cd9, SUC_lexicon_tags_17 },
    { 0xf06d5cdb, SUC_lexicon_tags_25 },
    { 0xc42d5cdc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d5cdf, SUC_lexicon_tags_11 },
    { 0x952d5ce0, SUC_lexicon_tags_37 },
    { 0xe3ed5ce1, SUC_lexicon_tags_160 },
    { 0x190d5ce2, SUC_lexicon_tags_1 },
    { 0x6a6d5ce3, SUC_lexicon_tags_48 },
    { 0xa1cd5ce4, SUC_lexicon_tags_16 },
    { 0xc24d5ce3, SUC_lexicon_tags_15 },
    { 0x13ad5ce6, SUC_lexicon_tags_17 },
    { 0xc0d5ce6, SUC_lexicon_tags_87 },
    { 0xbf0d5ce8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6d5ceb, SUC_lexicon_tags_77 },
    { 0x8bcd5ceb, SUC_lexicon_tags_14 },
    { 0xb5ad5ced, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62d5cf1, SUC_lexicon_tags_37 },
    { 0x51cd5cf2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2c6d5cf4, SUC_lexicon_tags_37 },
    { 0x124d5cf5, SUC_lexicon_tags_17 },
    { 0x248d5cf6, SUC_lexicon_tags_31 },
    { 0x32ed5cf5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4cd5cf9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x432d5cfc, SUC_lexicon_tags_60 },
    { 0x6c4d5cfd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4d5d06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a0d5d08, SUC_lexicon_tags_37 },
    { 0xae6d5d09, SUC_lexicon_tags_21 },
    { 0x3ead5d08, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x73cd5d0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58cd5d0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd50d5d11, SUC_lexicon_tags_15 },
    { 0xc92d5d12, SUC_lexicon_tags_4 },
    { 0x360d5d13, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5f6d5d15, SUC_lexicon_tags_27 },
    { 0xb0ed5d16, SUC_lexicon_tags_60 },
    { 0xf8ed5d17, SUC_lexicon_tags_11 },
    { 0xc3ad5d18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xabcd5d1a, SUC_lexicon_tags_92 },
    { 0x7c6d5d1a, SUC_lexicon_tags_11 },
    { 0x752d5d1c, SUC_lexicon_tags_25 },
    { 0x50cd5d1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10ad5d1f, SUC_lexicon_tags_23 },
    { 0xfa0d5d20, SUC_lexicon_tags_48 },
    { 0x992d5d21, SUC_lexicon_tags_1 },
    { 0xf6ad5d22, SUC_lexicon_tags_47 },
    { 0x120d5d20, SUC_lexicon_tags_12 },
    { 0x23cd5d24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xead5d27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad5d2b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf40d5d2d, SUC_lexicon_tags_11 },
    { 0xbb6d5d2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ed5d31, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x9fcd5d33, SUC_lexicon_tags_14 },
    { 0x7bad5d33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x358d5d36, SUC_lexicon_tags_2 },
    { 0xe3cd5d36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72cd5d39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66d5d3c, SUC_lexicon_tags_51 },
    { 0xa6ad5d3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80d5d45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ded5d4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ced5d51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300d5d55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x942d5d57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f8d5d59, SUC_lexicon_tags_17 },
    { 0x998d5d59, SUC_lexicon_tags_15 },
    { 0x980d5d59, SUC_lexicon_tags_34 },
    { 0xc7ad5d5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf56d5d5e, SUC_lexicon_tags_2 },
    { 0xcb8d5d5e, SUC_lexicon_tags_11 },
    { 0x54ad5d5e, SUC_lexicon_tags_12 },
    { 0xff8d5d61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x114d5d63, SUC_lexicon_tags_10 },
    { 0x67cd5d64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316d5d68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18d5d6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8d5d6d, SUC_lexicon_tags_25 },
    { 0x1a2d5d6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4cd5d70, SUC_lexicon_tags_47 },
    { 0xbc8d5d71, SUC_lexicon_tags_17 },
    { 0xda8d5d70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4d5d75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ed5d79, SUC_lexicon_tags_23 },
    { 0x504d5d79, SUC_lexicon_tags_17 },
    { 0xb8d5d7b, SUC_lexicon_tags_9 },
    { 0xbcad5d7c, SUC_lexicon_tags_28 },
    { 0x68ad5d79, SUC_lexicon_tags_15 },
    { 0xdced5d7e, SUC_lexicon_tags_12 },
    { 0xf5ed5d7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x586d5d82, SUC_lexicon_tags_62 },
    { 0xff4d5d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x812d5d85, SUC_lexicon_tags_25 },
    { 0xa9ed5d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fed5d8d, SUC_lexicon_tags_6 },
    { 0xdcd5d8e, SUC_lexicon_tags_15 },
    { 0xea0d5d8f, SUC_lexicon_tags_9 },
    { 0x988d5d8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd5d93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bed5d97, SUC_lexicon_tags_15 },
    { 0x754d5d98, SUC_lexicon_tags_11 },
    { 0xc50d5d99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed5d9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ed5da1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8d5da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcd5da9, SUC_lexicon_tags_0 },
    { 0x1c8d5daa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8d4d5dac, SUC_lexicon_tags_26 },
    { 0x788d5dad, SUC_lexicon_tags_46 },
    { 0x6f4d5dad, SUC_lexicon_tags_120 },
    { 0x5acd5daf, SUC_lexicon_tags_25 },
    { 0x310d5db0, SUC_lexicon_tags_37 },
    { 0xc9ed5db0, SUC_lexicon_tags_1 },
    { 0xf82d5daf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d5db7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710d5dbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12ad5dbe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cd5dc6, SUC_lexicon_tags_37 },
    { 0x3bcd5dc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bed5dca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x67ed5dcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb04d5dce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d5dd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d5dd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682d5dd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44d5ddf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbed5de1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd54d5de3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ed5de8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab8d5dea, SUC_lexicon_tags_7 },
    { 0x5b6d5deb, SUC_lexicon_tags_32 },
    { 0x200d5dec, SUC_lexicon_tags_39 },
    { 0xfbed5ded, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x854d5def, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ad5df4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2d5df7, SUC_lexicon_tags_2 },
    { 0x498d5df7, SUC_lexicon_tags_17 },
    { 0x170d5df8, SUC_lexicon_tags_25 },
    { 0x844d5dfa, SUC_lexicon_tags_12 },
    { 0xa96d5df7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4d5dfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8d5e01, SUC_lexicon_tags_4 },
    { 0x9bcd5e02, SUC_lexicon_tags_19 },
    { 0xc50d5e03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ed5e0a, SUC_lexicon_tags_7 },
    { 0xb64d5e0a, SUC_lexicon_tags_11 },
    { 0xd2cd5e0c, SUC_lexicon_tags_60 },
    { 0xdc2d5e0d, SUC_lexicon_tags_12 },
    { 0xb3ad5e0e, SUC_lexicon_tags_79 },
    { 0xd66d5e0f, SUC_lexicon_tags_49 },
    { 0xa92d5e0d, SUC_lexicon_tags_49 },
    { 0x6d0d5e11, SUC_lexicon_tags_33 },
    { 0x68ad5e12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3cd5e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bad5e17, SUC_lexicon_tags_2 },
    { 0xe6ad5e18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360d5e1c, SUC_lexicon_tags_9 },
    { 0x154d5e1c, SUC_lexicon_tags_123 },
    { 0xc52d5e1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56d5e21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf6d5e23, SUC_lexicon_tags_37 },
    { 0x64ed5e24, SUC_lexicon_tags_11 },
    { 0xe32d5e25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0d5e29, SUC_lexicon_tags_37 },
    { 0x174d5e29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264d5e2f, SUC_lexicon_tags_25 },
    { 0x8aad5e30, SUC_lexicon_tags_37 },
    { 0x9d0d5e30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecd5e34, SUC_lexicon_tags_17 },
    { 0x48d5e35, SUC_lexicon_tags_11 },
    { 0xed4d5e36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69ad5e38, SUC_lexicon_tags_14 },
    { 0x3e8d5e39, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfe8d5e3b, SUC_lexicon_tags_9 },
    { 0x2ccd5e3c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22d5e40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa68d5e42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cd5e45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x982d5e48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cd5e4c, SUC_lexicon_tags_1 },
    { 0xa14d5e4d, SUC_lexicon_tags_37 },
    { 0xdbed5e4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328d5e52, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdb0d5e54, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf92d5e56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102d5e5a, SUC_lexicon_tags_25 },
    { 0xaa6d5e5b, SUC_lexicon_tags_60 },
    { 0xdaad5e5c, SUC_lexicon_tags_17 },
    { 0x156d5e5c, SUC_lexicon_tags_2 },
    { 0x428d5e5e, SUC_lexicon_tags_14 },
    { 0xbbcd5e5f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e6d5e61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06d5e65, SUC_lexicon_tags_65 },
    { 0xf2ed5e65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d5e69, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608d5e6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x960d5e70, SUC_lexicon_tags_7 },
    { 0x5cd5e71, SUC_lexicon_tags_2 },
    { 0x434d5e71, SUC_lexicon_tags_49 },
    { 0xaf8d5e73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfeed5e75, SUC_lexicon_tags_12 },
    { 0x220d5e76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x730d5e78, SUC_lexicon_tags_57 },
    { 0x5d4d5e79, SUC_lexicon_tags_60 },
    { 0x7e2d5e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc20d5e7c, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xb70d5e7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7cd5e80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd22d5e82, SUC_lexicon_tags_49 },
    { 0xdacd5e82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ed5e86, SUC_lexicon_tags_63 },
    { 0xc88d5e87, SUC_lexicon_tags_25 },
    { 0x73ed5e87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x622d5e8a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x454d5e8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d5e91, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cd5e96, SUC_lexicon_tags_12 },
    { 0xb14d5e96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cd5e9c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408d5ea2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290d5ea6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7ed5ea8, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x8d2d5eaa, SUC_lexicon_tags_37 },
    { 0xc0cd5eaa, SUC_lexicon_tags_62 },
    { 0x802d5eac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ad5eb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60d5eb7, SUC_lexicon_tags_84 },
    { 0xa7ad5eb7, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728d5ebf, SUC_lexicon_tags_28 },
    { 0x6fad5ebf, SUC_lexicon_tags_6 },
    { 0xe56d5ec0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba8d5ec3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ed5ec6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x468d5ec8, SUC_lexicon_tags_57 },
    { 0x438d5ec9, SUC_lexicon_tags_25 },
    { 0xc9cd5eca, SUC_lexicon_tags_216 },
    { 0x5b0d5eca, SUC_lexicon_tags_14 },
    { 0xe78d5ecc, SUC_lexicon_tags_15 },
    { 0x5b6d5ecd, SUC_lexicon_tags_3 },
    { 0x460d5ecb, SUC_lexicon_tags_11 },
    { 0xf02d5ecf, SUC_lexicon_tags_7 },
    { 0xf7cd5ed0, SUC_lexicon_tags_14 },
    { 0x7b8d5ed0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe90d5ed3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ed5ed6, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x60ad5ed8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xceed5edb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacd5edf, SUC_lexicon_tags_43 },
    { 0x654d5ee0, SUC_lexicon_tags_2 },
    { 0x690d5ee1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ad5ee5, SUC_lexicon_tags_10 },
    { 0xd84d5ee6, SUC_lexicon_tags_65 },
    { 0xca2d5ee6, SUC_lexicon_tags_87 },
    { 0xb46d5ee6, SUC_lexicon_tags_14 },
    { 0x64d5ee7, SUC_lexicon_tags_14 },
    { 0x8b8d5eea, SUC_lexicon_tags_28 },
    { 0x82ed5ee5, SUC_lexicon_tags_39 },
    { 0x354d5ee6, SUC_lexicon_tags_49 },
    { 0x582d5eed, SUC_lexicon_tags_70 },
    { 0x56ad5eed, SUC_lexicon_tags_37 },
    { 0x6d5eee, SUC_lexicon_tags_48 },
    { 0x48d5eee, SUC_lexicon_tags_18 },
    { 0x2dad5ef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x134d5ef3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x126d5ef6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d6d5ef8, SUC_lexicon_tags_10 },
    { 0x10d5ef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeaad5efb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5acd5efe, SUC_lexicon_tags_2 },
    { 0x88ad5eff, SUC_lexicon_tags_45 },
    { 0xc0d5f00, SUC_lexicon_tags_37 },
    { 0x92d5f01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb30d5f03, SUC_lexicon_tags_29 },
    { 0x89cd5f03, SUC_lexicon_tags_7 },
    { 0x61ed5f05, SUC_lexicon_tags_23 },
    { 0x734d5f05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244d5f0e, SUC_lexicon_tags_29 },
    { 0x65ed5f0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x47cd5f11, SUC_lexicon_tags_47 },
    { 0x1e8d5f11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bed5f15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30d5f19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6dad5f1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad2d5f1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda8d5f1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2d5f29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d5f33, SUC_lexicon_tags_27 },
    { 0x86d5f34, SUC_lexicon_tags_15 },
    { 0x1acd5f35, SUC_lexicon_tags_8 },
    { 0x31ad5f36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccd5f3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734d5f43, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98d5f46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfbed5f48, SUC_lexicon_tags_38 },
    { 0x33cd5f48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd52d5f4b, SUC_lexicon_tags_15 },
    { 0x77cd5f4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b6d5f4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x930d5f50, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76d5f53, SUC_lexicon_tags_9 },
    { 0xc9cd5f54, SUC_lexicon_tags_79 },
    { 0xdbad5f55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ded5f5b, SUC_lexicon_tags_14 },
    { 0xce0d5f5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ad5f5f, SUC_lexicon_tags_25 },
    { 0xaf4d5f60, SUC_lexicon_tags_65 },
    { 0x9a8d5f61, SUC_lexicon_tags_66 },
    { 0xdb6d5f62, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xed4d5f64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972d5f68, SUC_lexicon_tags_49 },
    { 0x114d5f69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824d5f6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf3ad5f71, SUC_lexicon_tags_39 },
    { 0x23ed5f72, SUC_lexicon_tags_37 },
    { 0x7bcd5f72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0d5f79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ad5f7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ed5f82, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292d5f88, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x672d5f8a, SUC_lexicon_tags_2 },
    { 0x874d5f8a, SUC_lexicon_tags_72 },
    { 0x822d5f8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2d5f8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x464d5f92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ad5f95, SUC_lexicon_tags_6 },
    { 0x798d5f95, SUC_lexicon_tags_37 },
    { 0x4d0d5f97, SUC_lexicon_tags_1 },
    { 0x134d5f98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40d5f9c, SUC_lexicon_tags_25 },
    { 0x386d5f9d, SUC_lexicon_tags_9 },
    { 0xe30d5f9d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ad5fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4d5fa4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8f4d5fa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60d5fac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cd5fb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48cd5fb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ced5fb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50d5fbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1ad5fbd, SUC_lexicon_tags_0 },
    { 0xfced5fbe, SUC_lexicon_tags_11 },
    { 0x458d5fbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x720d5fc2, SUC_lexicon_tags_30 },
    { 0x3d0d5fc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6cd5fc5, SUC_lexicon_tags_15 },
    { 0x9e4d5fc6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ad5fcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6d5fce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf84d5fd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cd5fd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x542d5fd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8d5fda, SUC_lexicon_tags_118 },
    { 0x12ad5fda, SUC_lexicon_tags_37 },
    { 0x686d5fdc, SUC_lexicon_tags_15 },
    { 0xc62d5fdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x996d5fdf, SUC_lexicon_tags_1 },
    { 0x4f8d5fe0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbaad5fe2, SUC_lexicon_tags_38 },
    { 0xe64d5fe3, SUC_lexicon_tags_63 },
    { 0xa74d5fe4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d5fe7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d5feb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd8ed5fed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2f8d5fef, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x330d5ff1, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d5ff8, SUC_lexicon_tags_49 },
    { 0xa04d5ff9, SUC_lexicon_tags_29 },
    { 0x5c2d5ff8, SUC_lexicon_tags_33 },
    { 0xd2cd5ffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfcd5ffd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4ed5fff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ead6004, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4c4d6006, SUC_lexicon_tags_13 },
    { 0x9c0d6006, SUC_lexicon_tags_21 },
    { 0xa96d6008, SUC_lexicon_tags_12 },
    { 0x764d6008, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90d600c, SUC_lexicon_tags_31 },
    { 0xd16d600c, SUC_lexicon_tags_16 },
    { 0x4e8d600c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef4d6010, SUC_lexicon_tags_10 },
    { 0x802d6011, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd10d6013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ad6017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8d6021, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ed6024, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38d602a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ced602e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ad6033, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfed6035, SUC_lexicon_tags_37 },
    { 0xedcd6036, SUC_lexicon_tags_7 },
    { 0x9a2d6037, SUC_lexicon_tags_12 },
    { 0x73ed6038, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8d6042, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ad6045, SUC_lexicon_tags_2 },
    { 0xead6045, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad604e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eed6053, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ad6058, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x330d605a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6d605e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x19cd6061, SUC_lexicon_tags_12 },
    { 0xc2ad6062, SUC_lexicon_tags_33 },
    { 0x4b2d6063, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe00d6065, SUC_lexicon_tags_7 },
    { 0x63cd6066, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8cd6068, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccd606e, SUC_lexicon_tags_68 },
    { 0x9e4d606f, SUC_lexicon_tags_0 },
    { 0x254d606f, SUC_lexicon_tags_2 },
    { 0xad0d6071, SUC_lexicon_tags_31 },
    { 0xea8d6071, SUC_lexicon_tags_1 },
    { 0x772d6073, SUC_lexicon_tags_7 },
    { 0xf58d6073, SUC_lexicon_tags_30 },
    { 0x3a0d6075, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a4d6077, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9ad6079, SUC_lexicon_tags_6 },
    { 0xa26d607a, SUC_lexicon_tags_45 },
    { 0xfc4d607b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa3ed607d, SUC_lexicon_tags_56 },
    { 0x3b6d607d, SUC_lexicon_tags_11 },
    { 0x240d607f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20d6083, SUC_lexicon_tags_12 },
    { 0xa7ad6083, SUC_lexicon_tags_33 },
    { 0x6cd6085, SUC_lexicon_tags_23 },
    { 0xc7cd6086, SUC_lexicon_tags_29 },
    { 0xfd0d6083, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4d608a, SUC_lexicon_tags_0 },
    { 0x93ad608a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4d608e, SUC_lexicon_tags_17 },
    { 0xdd6d608f, SUC_lexicon_tags_23 },
    { 0x748d608e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452d6095, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98d609a, SUC_lexicon_tags_33 },
    { 0x75ed609b, SUC_lexicon_tags_25 },
    { 0xf9ed609c, SUC_lexicon_tags_23 },
    { 0x8f6d609c, SUC_lexicon_tags_17 },
    { 0xc62d609c, SUC_lexicon_tags_23 },
    { 0xa32d609f, SUC_lexicon_tags_172 },
    { 0xd50d60a0, SUC_lexicon_tags_8 },
    { 0xeced609c, SUC_lexicon_tags_33 },
    { 0xc6ed609e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28d60b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6d60b4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x350d60b6, SUC_lexicon_tags_756 },
    { 0x7f4d60b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce4d60b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64d60c1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x202d60c4, SUC_lexicon_tags_15 },
    { 0xc90d60c5, SUC_lexicon_tags_2 },
    { 0x64cd60c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d60ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73cd60cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x532d60d0, SUC_lexicon_tags_8 },
    { 0xbe2d60d1, SUC_lexicon_tags_63 },
    { 0xf54d60d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefed60d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed6d60d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ad60da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340d60e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfded60e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61cd60e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf16d60e7, SUC_lexicon_tags_23 },
    { 0x6d0d60e8, SUC_lexicon_tags_15 },
    { 0x168d60e9, SUC_lexicon_tags_37 },
    { 0xfeed60ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94cd60ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ad60f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92d60f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5fed60f8, SUC_lexicon_tags_17 },
    { 0x6bad60f9, SUC_lexicon_tags_30 },
    { 0x4d60f9, SUC_lexicon_tags_38 },
    { 0xd04d60fb, SUC_lexicon_tags_21 },
    { 0x4f0d60fb, SUC_lexicon_tags_14 },
    { 0x4fad60fd, SUC_lexicon_tags_23 },
    { 0x624d60fd, SUC_lexicon_tags_61 },
    { 0x1acd60ff, SUC_lexicon_tags_4 },
    { 0xaf2d6100, SUC_lexicon_tags_15 },
    { 0x124d6101, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120d6105, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ad6109, SUC_lexicon_tags_19 },
    { 0xc68d6109, SUC_lexicon_tags_25 },
    { 0x27cd610b, SUC_lexicon_tags_46 },
    { 0x8ded610c, SUC_lexicon_tags_8 },
    { 0xe8d610d, SUC_lexicon_tags_17 },
    { 0x514d610c, SUC_lexicon_tags_23 },
    { 0xe58d610d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa66d6111, SUC_lexicon_tags_68 },
    { 0xe04d6112, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8d611a, SUC_lexicon_tags_37 },
    { 0x3f2d611b, SUC_lexicon_tags_28 },
    { 0x4bad611c, SUC_lexicon_tags_23 },
    { 0x470d611d, SUC_lexicon_tags_26 },
    { 0x7fed611b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe42d6120, SUC_lexicon_tags_17 },
    { 0xb5ed6121, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8d6124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824d6132, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x114d6134, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xcbed6136, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3b0d6138, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x362d613a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8d613d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2b8d613f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x680d6142, SUC_lexicon_tags_7 },
    { 0x1cad6143, SUC_lexicon_tags_14 },
    { 0xcfad6144, SUC_lexicon_tags_25 },
    { 0x4f4d6143, SUC_lexicon_tags_7 },
    { 0x488d6145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x790d6148, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x766d614a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bed614e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ed6153, SUC_lexicon_tags_174 },
    { 0xc12d6154, SUC_lexicon_tags_15 },
    { 0x8e0d6154, SUC_lexicon_tags_37 },
    { 0xd08d6154, SUC_lexicon_tags_15 },
    { 0x9ccd6154, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86d615d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544d6165, SUC_lexicon_tags_70 },
    { 0xe56d6166, SUC_lexicon_tags_27 },
    { 0x9acd6165, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26d616a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d0d616c, SUC_lexicon_tags_1 },
    { 0xb16d616c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944d6174, SUC_lexicon_tags_14 },
    { 0xe00d6174, SUC_lexicon_tags_17 },
    { 0x778d6175, SUC_lexicon_tags_47 },
    { 0x2d6177, SUC_lexicon_tags_62 },
    { 0xcc0d6178, SUC_lexicon_tags_47 },
    { 0xcb0d6179, SUC_lexicon_tags_15 },
    { 0x908d617a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266d617e, SUC_lexicon_tags_7 },
    { 0xce8d617f, SUC_lexicon_tags_43 },
    { 0xaeed617f, SUC_lexicon_tags_1 },
    { 0x6b6d6180, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x80ad6183, SUC_lexicon_tags_47 },
    { 0xad0d6183, SUC_lexicon_tags_25 },
    { 0xc62d6183, SUC_lexicon_tags_82 },
    { 0xce6d6186, SUC_lexicon_tags_15 },
    { 0x78d6185, SUC_lexicon_tags_35 },
    { 0x7dcd6188, SUC_lexicon_tags_12 },
    { 0xc4cd6188, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b0d618b, SUC_lexicon_tags_1 },
    { 0x81cd618b, SUC_lexicon_tags_43 },
    { 0x5ccd618d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ed6197, SUC_lexicon_tags_17 },
    { 0x7d2d6197, SUC_lexicon_tags_1 },
    { 0xed4d6199, SUC_lexicon_tags_1 },
    { 0x668d6199, SUC_lexicon_tags_2 },
    { 0xa9ad619b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cd61a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8c8d61a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1ad61a7, SUC_lexicon_tags_7 },
    { 0xbd6d61a8, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x13cd61ab, SUC_lexicon_tags_39 },
    { 0x764d61ac, SUC_lexicon_tags_1 },
    { 0xee6d61ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3c8d61af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80d61b8, SUC_lexicon_tags_25 },
    { 0x946d61b9, SUC_lexicon_tags_49 },
    { 0x5fed61ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdf0d61bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x176d61be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628d61c3, SUC_lexicon_tags_37 },
    { 0x574d61c4, SUC_lexicon_tags_15 },
    { 0x69cd61c4, SUC_lexicon_tags_2 },
    { 0x4f8d61c6, SUC_lexicon_tags_17 },
    { 0xafcd61c7, SUC_lexicon_tags_2 },
    { 0x288d61c8, SUC_lexicon_tags_17 },
    { 0xc5ed61c8, SUC_lexicon_tags_23 },
    { 0x726d61ca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22d61d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1c8d61d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6d61db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa62d61dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88d61e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56d61e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x158d61e8, SUC_lexicon_tags_55 },
    { 0xb6ed61e9, SUC_lexicon_tags_47 },
    { 0x5acd61e8, SUC_lexicon_tags_37 },
    { 0xed61ea, SUC_lexicon_tags_18 },
    { 0xf56d61ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe56d61ee, SUC_lexicon_tags_7 },
    { 0x126d61ef, SUC_lexicon_tags_9 },
    { 0x4f6d61f0, SUC_lexicon_tags_2 },
    { 0xd62d61f1, SUC_lexicon_tags_2 },
    { 0xaaad61f2, SUC_lexicon_tags_172 },
    { 0x72ed61f3, SUC_lexicon_tags_9 },
    { 0x75ad61f0, SUC_lexicon_tags_19 },
    { 0x1ad61f5, SUC_lexicon_tags_15 },
    { 0x94ed61f6, SUC_lexicon_tags_12 },
    { 0x90ad61f5, SUC_lexicon_tags_68 },
    { 0x804d61f5, SUC_lexicon_tags_61 },
    { 0xcc4d61f9, SUC_lexicon_tags_43 },
    { 0x26cd61f6, SUC_lexicon_tags_60 },
    { 0xdf8d61fb, SUC_lexicon_tags_65 },
    { 0xb7ed61f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ad6203, SUC_lexicon_tags_11 },
    { 0x63ed6203, SUC_lexicon_tags_11 },
    { 0xf1cd6205, SUC_lexicon_tags_29 },
    { 0xaded6206, SUC_lexicon_tags_28 },
    { 0xaad6206, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad620b, SUC_lexicon_tags_27 },
    { 0x950d620c, SUC_lexicon_tags_38 },
    { 0x6a0d620d, SUC_lexicon_tags_18 },
    { 0x7b8d620e, SUC_lexicon_tags_19 },
    { 0xcded620f, SUC_lexicon_tags_7 },
    { 0x700d620d, SUC_lexicon_tags_46 },
    { 0xdbcd6211, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xbbed6213, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d6d6215, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d621b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e6d621d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb6d621f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150d6224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdded6226, SUC_lexicon_tags_93 },
    { 0xa66d6227, SUC_lexicon_tags_12 },
    { 0xb0ad6228, SUC_lexicon_tags_25 },
    { 0x84ad6228, SUC_lexicon_tags_7 },
    { 0x87cd6226, SUC_lexicon_tags_1041 },
    { 0x2e2d6226, SUC_lexicon_tags_47 },
    { 0x732d622c, SUC_lexicon_tags_9 },
    { 0xc2d6226, SUC_lexicon_tags_2 },
    { 0xd0cd622e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8d6232, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ed6235, SUC_lexicon_tags_66 },
    { 0xae8d6236, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ed6239, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8d623d, SUC_lexicon_tags_27 },
    { 0x816d623e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936d624a, SUC_lexicon_tags_8 },
    { 0xa8ad624a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c8d624d, SUC_lexicon_tags_15 },
    { 0xf78d624d, SUC_lexicon_tags_12 },
    { 0xa04d624d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8d625e, SUC_lexicon_tags_2 },
    { 0xe40d625e, SUC_lexicon_tags_140 },
    { 0x926d625f, SUC_lexicon_tags_27 },
    { 0x8cd625f, SUC_lexicon_tags_12 },
    { 0x2f0d6262, SUC_lexicon_tags_17 },
    { 0xb16d6262, SUC_lexicon_tags_27 },
    { 0x2aad6264, SUC_lexicon_tags_13 },
    { 0x838d6264, SUC_lexicon_tags_38 },
    { 0x4bed6266, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dad6269, SUC_lexicon_tags_2 },
    { 0xd02d6269, SUC_lexicon_tags_20 },
    { 0xed8d626b, SUC_lexicon_tags_12 },
    { 0x964d626c, SUC_lexicon_tags_12 },
    { 0xa16d626d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6d6271, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa3ed6273, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20d6276, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ad627b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422d6281, SUC_lexicon_tags_7 },
    { 0x4dcd6282, SUC_lexicon_tags_17 },
    { 0x6c4d6282, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2ad6285, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0d628c, SUC_lexicon_tags_1 },
    { 0x876d628c, SUC_lexicon_tags_0 },
    { 0x68ad628e, SUC_lexicon_tags_2 },
    { 0x9bad628f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d6293, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ad6298, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ad62a4, SUC_lexicon_tags_11 },
    { 0x60ad62a5, SUC_lexicon_tags_11 },
    { 0x7e6d62a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f2d62a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206d62af, SUC_lexicon_tags_37 },
    { 0x9b8d62b0, SUC_lexicon_tags_17 },
    { 0xa96d62b1, SUC_lexicon_tags_23 },
    { 0x680d62b2, SUC_lexicon_tags_15 },
    { 0x56ed62b1, SUC_lexicon_tags_68 },
    { 0xd2cd62b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ad62bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf20d62bf, SUC_lexicon_tags_63 },
    { 0x4d4d62c0, SUC_lexicon_tags_13 },
    { 0x78d62c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x992d62c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6d62c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bcd62cb, SUC_lexicon_tags_12 },
    { 0xcead62cb, SUC_lexicon_tags_37 },
    { 0x722d62cd, SUC_lexicon_tags_17 },
    { 0x5f8d62ce, SUC_lexicon_tags_0 },
    { 0xaa0d62cf, SUC_lexicon_tags_11 },
    { 0xc5cd62d0, SUC_lexicon_tags_15 },
    { 0xde6d62d0, SUC_lexicon_tags_27 },
    { 0xc1ad62d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49ad62d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cd62d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ad62dc, SUC_lexicon_tags_30 },
    { 0xcdad62dd, SUC_lexicon_tags_37 },
    { 0xa2ad62dd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8f2d62e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x408d62e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x246d62e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadad62e9, SUC_lexicon_tags_87 },
    { 0xf64d62ea, SUC_lexicon_tags_45 },
    { 0x3c0d62eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x262d62ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x102d62ef, SUC_lexicon_tags_49 },
    { 0x486d62ef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ed62f3, SUC_lexicon_tags_48 },
    { 0x580d62f4, SUC_lexicon_tags_1 },
    { 0x8ccd62f3, SUC_lexicon_tags_60 },
    { 0x690d62f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebad62fc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60d62ff, SUC_lexicon_tags_56 },
    { 0x958d62ff, SUC_lexicon_tags_15 },
    { 0x6fed6300, SUC_lexicon_tags_30 },
    { 0xe12d62ff, SUC_lexicon_tags_11 },
    { 0xd32d6303, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d6306, SUC_lexicon_tags_2 },
    { 0xf5cd6306, SUC_lexicon_tags_7 },
    { 0x782d6308, SUC_lexicon_tags_15 },
    { 0xdfad6309, SUC_lexicon_tags_112 },
    { 0x618d630a, SUC_lexicon_tags_27 },
    { 0x880d630b, SUC_lexicon_tags_0 },
    { 0x9d8d6309, SUC_lexicon_tags_12 },
    { 0xdfed630b, SUC_lexicon_tags_29 },
    { 0xf9cd630e, SUC_lexicon_tags_211 },
    { 0x81cd6309, SUC_lexicon_tags_17 },
    { 0x7a6d6306, SUC_lexicon_tags_27 },
    { 0xe98d6307, SUC_lexicon_tags_17 },
    { 0xfced6312, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaed6314, SUC_lexicon_tags_17 },
    { 0x466d6315, SUC_lexicon_tags_0 },
    { 0x968d6316, SUC_lexicon_tags_17 },
    { 0xd8d6317, SUC_lexicon_tags_33 },
    { 0xad6d6318, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeed631c, SUC_lexicon_tags_14 },
    { 0x178d631d, SUC_lexicon_tags_7 },
    { 0x44ad631e, SUC_lexicon_tags_17 },
    { 0x4ccd631f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ad6322, SUC_lexicon_tags_10 },
    { 0xfa8d6323, SUC_lexicon_tags_30 },
    { 0x180d6323, SUC_lexicon_tags_33 },
    { 0xe66d6325, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ad632d, SUC_lexicon_tags_17 },
    { 0x1f6d632e, SUC_lexicon_tags_1 },
    { 0x296d632f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0d633a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ed633e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ad6342, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2d6347, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42d634a, SUC_lexicon_tags_37 },
    { 0xc70d634b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x822d634d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06d6351, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0d6354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ed6357, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x704d6359, SUC_lexicon_tags_70 },
    { 0x130d635a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa68d635c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1c4d635e, SUC_lexicon_tags_14 },
    { 0x8dcd635f, SUC_lexicon_tags_12 },
    { 0x704d635e, SUC_lexicon_tags_27 },
    { 0x126d6361, SUC_lexicon_tags_11 },
    { 0xa96d6362, SUC_lexicon_tags_15 },
    { 0x796d6361, SUC_lexicon_tags_19 },
    { 0x6ccd6364, SUC_lexicon_tags_1 },
    { 0x576d6365, SUC_lexicon_tags_49 },
    { 0xc48d6366, SUC_lexicon_tags_2 },
    { 0xcacd6367, SUC_lexicon_tags_17 },
    { 0xabad6362, SUC_lexicon_tags_60 },
    { 0xa7ed6369, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb12d636c, SUC_lexicon_tags_26 },
    { 0x806d636d, SUC_lexicon_tags_31 },
    { 0x1f0d636e, SUC_lexicon_tags_7 },
    { 0x594d636f, SUC_lexicon_tags_37 },
    { 0x5c6d6370, SUC_lexicon_tags_17 },
    { 0x620d636e, SUC_lexicon_tags_17 },
    { 0x68cd6372, SUC_lexicon_tags_15 },
    { 0x36cd6371, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928d637a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x694d637c, SUC_lexicon_tags_72 },
    { 0x986d637c, SUC_lexicon_tags_17 },
    { 0x942d637e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x912d6381, SUC_lexicon_tags_17 },
    { 0xe84d6381, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x32cd6385, SUC_lexicon_tags_27 },
    { 0x32ed6386, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302d638a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x282d638d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ded6395, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfad639a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64d639f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ed63a5, SUC_lexicon_tags_11 },
    { 0x9d2d63a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5acd63a8, SUC_lexicon_tags_7 },
    { 0x6b2d63a8, SUC_lexicon_tags_11 },
    { 0xe6ad63aa, SUC_lexicon_tags_46 },
    { 0x230d63aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4d63ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cd63b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x852d63b5, SUC_lexicon_tags_17 },
    { 0x146d63b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2d63ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6d63c0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x222d63c3, SUC_lexicon_tags_37 },
    { 0x6ed63c4, SUC_lexicon_tags_45 },
    { 0x22ed63c5, SUC_lexicon_tags_11 },
    { 0xf4ed63c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x612d63c8, SUC_lexicon_tags_12 },
    { 0x7ad63c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8d63cd, SUC_lexicon_tags_11 },
    { 0x492d63cd, SUC_lexicon_tags_15 },
    { 0x27ed63cf, SUC_lexicon_tags_37 },
    { 0xfd2d63d0, SUC_lexicon_tags_9 },
    { 0xf46d63d0, SUC_lexicon_tags_38 },
    { 0x276d63d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0d63d6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ed63d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa16d63db, SUC_lexicon_tags_9 },
    { 0xf8ed63dc, SUC_lexicon_tags_8 },
    { 0x3fcd63dd, SUC_lexicon_tags_34 },
    { 0x232d63dd, SUC_lexicon_tags_13 },
    { 0x688d63df, SUC_lexicon_tags_17 },
    { 0x9b4d63dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x86ad63e2, SUC_lexicon_tags_66 },
    { 0xe9ed63e3, SUC_lexicon_tags_6 },
    { 0xb4ad63e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c8d63e6, SUC_lexicon_tags_32 },
    { 0xfb8d63e7, SUC_lexicon_tags_9 },
    { 0x3b6d63e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed63ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564d63f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492d63f4, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ad63f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20d63fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc70d63fc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76d63ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cd6407, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0d640c, SUC_lexicon_tags_28 },
    { 0x906d640d, SUC_lexicon_tags_39 },
    { 0x896d640d, SUC_lexicon_tags_11 },
    { 0x452d640f, SUC_lexicon_tags_375 },
    { 0xbd6d640e, SUC_lexicon_tags_0 },
    { 0x6c6d640e, SUC_lexicon_tags_27 },
    { 0xc4ad6412, SUC_lexicon_tags_2 },
    { 0x1acd6413, SUC_lexicon_tags_82 },
    { 0xad8d6414, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0d6417, SUC_lexicon_tags_30 },
    { 0x646d6417, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918d641d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x538d6420, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa72d6422, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8d6429, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ad642e, SUC_lexicon_tags_17 },
    { 0x6aed642e, SUC_lexicon_tags_28 },
    { 0x896d6430, SUC_lexicon_tags_7 },
    { 0x7aad642f, SUC_lexicon_tags_28 },
    { 0xb56d6432, SUC_lexicon_tags_62 },
    { 0xf46d6432, SUC_lexicon_tags_47 },
    { 0x1fad6434, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe76d6437, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x806d643a, SUC_lexicon_tags_7 },
    { 0xe3ed643b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ad6444, SUC_lexicon_tags_19 },
    { 0xaa4d6445, SUC_lexicon_tags_29 },
    { 0x152d6444, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x178d6449, SUC_lexicon_tags_37 },
    { 0x6aed644a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfced6457, SUC_lexicon_tags_11 },
    { 0x712d6457, SUC_lexicon_tags_10 },
    { 0xabad6458, SUC_lexicon_tags_38 },
    { 0xdf0d6457, SUC_lexicon_tags_60 },
    { 0xaf2d6459, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2d6461, SUC_lexicon_tags_17 },
    { 0xebad6462, SUC_lexicon_tags_23 },
    { 0xd98d6463, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ed6467, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372d646c, SUC_lexicon_tags_108 },
    { 0xf64d646c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166d6475, SUC_lexicon_tags_16 },
    { 0xc36d6475, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x35cd6478, SUC_lexicon_tags_11 },
    { 0xbbed6479, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1b6d647b, SUC_lexicon_tags_11 },
    { 0x6e2d647b, SUC_lexicon_tags_15 },
    { 0x2ced647d, SUC_lexicon_tags_17 },
    { 0xa36d647e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ad6481, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294d6489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2d648b, SUC_lexicon_tags_63 },
    { 0x838d648b, SUC_lexicon_tags_1 },
    { 0x286d648d, SUC_lexicon_tags_14 },
    { 0xc34d648e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x650d6491, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbfcd6493, SUC_lexicon_tags_31 },
    { 0xc74d6494, SUC_lexicon_tags_7 },
    { 0x770d6494, SUC_lexicon_tags_19 },
    { 0x8aed6496, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x700d6499, SUC_lexicon_tags_2 },
    { 0x80cd649a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cd64a1, SUC_lexicon_tags_500 },
    { 0x606d64a1, SUC_lexicon_tags_4 },
    { 0x772d64a3, SUC_lexicon_tags_11 },
    { 0xbf4d64a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aad64aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad64ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18d64b2, SUC_lexicon_tags_30 },
    { 0xf6ed64b3, SUC_lexicon_tags_11 },
    { 0xef8d64b4, SUC_lexicon_tags_12 },
    { 0x14ad64b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x136d64b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbfed64b9, SUC_lexicon_tags_37 },
    { 0xfd4d64b9, SUC_lexicon_tags_37 },
    { 0x188d64bb, SUC_lexicon_tags_2 },
    { 0xa34d64bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20d64c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ed64c5, SUC_lexicon_tags_28 },
    { 0xa6cd64c6, SUC_lexicon_tags_17 },
    { 0xe98d64c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aed64ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ed64ce, SUC_lexicon_tags_802 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ad64d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd62d64d9, SUC_lexicon_tags_2 },
    { 0x992d64d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99ed64dc, SUC_lexicon_tags_11 },
    { 0x528d64dd, SUC_lexicon_tags_25 },
    { 0x246d64de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb0ad64e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6d64e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872d64e7, SUC_lexicon_tags_0 },
    { 0x6c0d64e7, SUC_lexicon_tags_27 },
    { 0x3e8d64e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2d64f1, SUC_lexicon_tags_2 },
    { 0xd7cd64f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0d6503, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ed6507, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb5ad6509, SUC_lexicon_tags_70 },
    { 0xb38d6509, SUC_lexicon_tags_64 },
    { 0x3ccd650b, SUC_lexicon_tags_15 },
    { 0xbf0d650b, SUC_lexicon_tags_65 },
    { 0xf4d650d, SUC_lexicon_tags_17 },
    { 0x5a6d650a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5ad6510, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ad651a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaad6520, SUC_lexicon_tags_35 },
    { 0xe52d6521, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf82d6523, SUC_lexicon_tags_40 },
    { 0x316d6524, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ead652d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ded652f, SUC_lexicon_tags_37 },
    { 0x6d0d6530, SUC_lexicon_tags_7 },
    { 0x3acd6531, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x440d6533, SUC_lexicon_tags_27 },
    { 0xb16d6533, SUC_lexicon_tags_68 },
    { 0x6e0d6535, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x774d6538, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0d6541, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40d6546, SUC_lexicon_tags_11 },
    { 0x44d6547, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74d6558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224d655c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2cd655e, SUC_lexicon_tags_37 },
    { 0x7f4d655f, SUC_lexicon_tags_22 },
    { 0x3b4d6560, SUC_lexicon_tags_30 },
    { 0xab4d6560, SUC_lexicon_tags_17 },
    { 0x184d655f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ad6566, SUC_lexicon_tags_4 },
    { 0xc50d6567, SUC_lexicon_tags_37 },
    { 0x452d6567, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224d656d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x834d6570, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6d6573, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2d6576, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58d657a, SUC_lexicon_tags_17 },
    { 0xbc2d657b, SUC_lexicon_tags_12 },
    { 0x1a8d657c, SUC_lexicon_tags_37 },
    { 0x582d657a, SUC_lexicon_tags_8 },
    { 0x52ed657e, SUC_lexicon_tags_29 },
    { 0x9dad657f, SUC_lexicon_tags_7 },
    { 0x9fed6580, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17ed6582, SUC_lexicon_tags_11 },
    { 0x2d2d6582, SUC_lexicon_tags_7 },
    { 0x596d6584, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8d658b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76d6596, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfecd6598, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xde8d659a, SUC_lexicon_tags_38 },
    { 0x6d2d659b, SUC_lexicon_tags_17 },
    { 0xfbcd659b, SUC_lexicon_tags_61 },
    { 0x1b0d659d, SUC_lexicon_tags_17 },
    { 0x8eed659e, SUC_lexicon_tags_32 },
    { 0x80ad659c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344d65a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ad65a6, SUC_lexicon_tags_7 },
    { 0xba6d65a7, SUC_lexicon_tags_11 },
    { 0x882d65a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336d65ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcd65b7, SUC_lexicon_tags_7 },
    { 0xd8ed65b8, SUC_lexicon_tags_2 },
    { 0x4b8d65b7, SUC_lexicon_tags_25 },
    { 0xed6d65b7, SUC_lexicon_tags_11 },
    { 0x58d65bb, SUC_lexicon_tags_37 },
    { 0x216d65bc, SUC_lexicon_tags_108 },
    { 0xaa2d65bc, SUC_lexicon_tags_15 },
    { 0x558d65b8, SUC_lexicon_tags_38 },
    { 0x376d65bf, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xddad65c2, SUC_lexicon_tags_25 },
    { 0x9acd65c3, SUC_lexicon_tags_38 },
    { 0x23cd65c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dad65c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18d65cd, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd65d4, SUC_lexicon_tags_15 },
    { 0xd90d65d5, SUC_lexicon_tags_12 },
    { 0x41ad65d5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x51cd65d9, SUC_lexicon_tags_25 },
    { 0x5ad65da, SUC_lexicon_tags_11 },
    { 0xf42d65da, SUC_lexicon_tags_5 },
    { 0x16d65da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80d65df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4d65e2, SUC_lexicon_tags_32 },
    { 0x69ad65e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x75cd65e5, SUC_lexicon_tags_25 },
    { 0xb78d65e5, SUC_lexicon_tags_48 },
    { 0x682d65e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ad65ea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ad65ee, SUC_lexicon_tags_17 },
    { 0xd2cd65ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630d65f3, SUC_lexicon_tags_47 },
    { 0xa4ed65f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c4d65f6, SUC_lexicon_tags_0 },
    { 0x396d65f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdded65fb, SUC_lexicon_tags_11 },
    { 0xaa0d65fc, SUC_lexicon_tags_6 },
    { 0x9d2d65fb, SUC_lexicon_tags_17 },
    { 0xa0cd65fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48d6605, SUC_lexicon_tags_25 },
    { 0x16d6606, SUC_lexicon_tags_12 },
    { 0x944d6607, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f4d6609, SUC_lexicon_tags_17 },
    { 0x58cd6609, SUC_lexicon_tags_33 },
    { 0x702d660a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4d660f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf40d6611, SUC_lexicon_tags_7 },
    { 0x170d6611, SUC_lexicon_tags_37 },
    { 0xf34d6613, SUC_lexicon_tags_0 },
    { 0x8b0d6614, SUC_lexicon_tags_68 },
    { 0x46ad6615, SUC_lexicon_tags_29 },
    { 0xa9cd6616, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f4d6618, SUC_lexicon_tags_8 },
    { 0x94cd6619, SUC_lexicon_tags_26 },
    { 0xc4ed661a, SUC_lexicon_tags_2 },
    { 0x80d6619, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacd661f, SUC_lexicon_tags_37 },
    { 0xadad6620, SUC_lexicon_tags_25 },
    { 0xa78d661f, SUC_lexicon_tags_19 },
    { 0x8b0d6620, SUC_lexicon_tags_1 },
    { 0xf58d6620, SUC_lexicon_tags_17 },
    { 0x6e4d6620, SUC_lexicon_tags_158 },
    { 0xb3cd6625, SUC_lexicon_tags_0 },
    { 0x442d6626, SUC_lexicon_tags_12 },
    { 0x580d6625, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbed662b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2d662f, SUC_lexicon_tags_37 },
    { 0xd30d6630, SUC_lexicon_tags_33 },
    { 0x424d662f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdd0d6633, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ed6636, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72d6639, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860d663e, SUC_lexicon_tags_16 },
    { 0xca0d663f, SUC_lexicon_tags_14 },
    { 0xd7ad663e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe40d6642, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464d6649, SUC_lexicon_tags_15 },
    { 0x14ed6649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa36d664c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x946d664e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x248d6651, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2f6d6653, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d6659, SUC_lexicon_tags_11 },
    { 0x7e6d6659, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x344d665d, SUC_lexicon_tags_17 },
    { 0x71ed665d, SUC_lexicon_tags_15 },
    { 0xce0d665e, SUC_lexicon_tags_7 },
    { 0x712d665f, SUC_lexicon_tags_11 },
    { 0xeded6661, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ead6665, SUC_lexicon_tags_85 },
    { 0xfa0d6665, SUC_lexicon_tags_0 },
    { 0xc82d6667, SUC_lexicon_tags_10 },
    { 0xedcd6668, SUC_lexicon_tags_7 },
    { 0x974d6669, SUC_lexicon_tags_11 },
    { 0x5aed666a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26d6672, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x800d6675, SUC_lexicon_tags_2 },
    { 0xe10d6676, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ed6679, SUC_lexicon_tags_4 },
    { 0x5a2d6679, SUC_lexicon_tags_5 },
    { 0x666d6679, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2d6680, SUC_lexicon_tags_11 },
    { 0x320d6680, SUC_lexicon_tags_43 },
    { 0x7b2d6682, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8d6686, SUC_lexicon_tags_9 },
    { 0x644d6687, SUC_lexicon_tags_7 },
    { 0x2f4d6686, SUC_lexicon_tags_15 },
    { 0xafed6689, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ead668b, SUC_lexicon_tags_23 },
    { 0x85ad668c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624d6690, SUC_lexicon_tags_151 },
    { 0x2a4d6690, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8d6695, SUC_lexicon_tags_29 },
    { 0x5ced6696, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4d669c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbe0d669e, SUC_lexicon_tags_57 },
    { 0x8ecd669e, SUC_lexicon_tags_65 },
    { 0x192d669e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x492d66a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeed66a7, SUC_lexicon_tags_33 },
    { 0xf5ed66a8, SUC_lexicon_tags_7 },
    { 0x1a0d66a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec0d66ab, SUC_lexicon_tags_17 },
    { 0xed0d66ac, SUC_lexicon_tags_44 },
    { 0xe6d66ad, SUC_lexicon_tags_11 },
    { 0x8cad66ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146d66b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe90d66b4, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x1d8d66b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8d66c3, SUC_lexicon_tags_2 },
    { 0xa54d66c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ed66c7, SUC_lexicon_tags_1 },
    { 0x242d66c8, SUC_lexicon_tags_7 },
    { 0x7ced66c9, SUC_lexicon_tags_7 },
    { 0xd00d66c7, SUC_lexicon_tags_14 },
    { 0xbc2d66cb, SUC_lexicon_tags_49 },
    { 0x9b6d66cc, SUC_lexicon_tags_1 },
    { 0xfced66cd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4f2d66cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xabad66d2, SUC_lexicon_tags_33 },
    { 0x654d66d3, SUC_lexicon_tags_17 },
    { 0x10ed66d3, SUC_lexicon_tags_15 },
    { 0x248d66d2, SUC_lexicon_tags_25 },
    { 0xcc0d66d2, SUC_lexicon_tags_188 },
    { 0x87ed66d7, SUC_lexicon_tags_17 },
    { 0x44cd66d5, SUC_lexicon_tags_17 },
    { 0xb46d66d2, SUC_lexicon_tags_65 },
    { 0x32d66da, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0xae4d66dc, SUC_lexicon_tags_7 },
    { 0xeaad66dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2d66e2, SUC_lexicon_tags_1 },
    { 0x62ad66e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52d66e7, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ed66ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8d66f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ed66f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f4d66fa, SUC_lexicon_tags_19 },
    { 0xbf6d66fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ed66fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x776d6700, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00d6705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x734d6707, SUC_lexicon_tags_26 },
    { 0x14d6707, SUC_lexicon_tags_37 },
    { 0x52ed6707, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ed670e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x510d6711, SUC_lexicon_tags_32 },
    { 0x1d6d6711, SUC_lexicon_tags_37 },
    { 0x6cad6713, SUC_lexicon_tags_46 },
    { 0x188d6714, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x304d6716, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0cd6718, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5ded671a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2d6720, SUC_lexicon_tags_49 },
    { 0x32ed6721, SUC_lexicon_tags_12 },
    { 0xeed6721, SUC_lexicon_tags_51 },
    { 0x9a6d6723, SUC_lexicon_tags_37 },
    { 0xcfcd6724, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba6d6726, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe1cd6728, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccd672b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84d672e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0d6733, SUC_lexicon_tags_47 },
    { 0xb9ed6734, SUC_lexicon_tags_37 },
    { 0xa7ad6735, SUC_lexicon_tags_129 },
    { 0x208d6733, SUC_lexicon_tags_17 },
    { 0xe2cd6733, SUC_lexicon_tags_84 },
    { 0xd72d6733, SUC_lexicon_tags_16 },
    { 0x24ed6739, SUC_lexicon_tags_68 },
    { 0xb5ed673a, SUC_lexicon_tags_26 },
    { 0x5fad673b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6d6740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ed6747, SUC_lexicon_tags_15 },
    { 0x6b8d6747, SUC_lexicon_tags_11 },
    { 0xdf6d6749, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ad674d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa48d674f, SUC_lexicon_tags_46 },
    { 0xe28d6750, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ed6753, SUC_lexicon_tags_11 },
    { 0x166d6754, SUC_lexicon_tags_8 },
    { 0x2e8d6754, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x610d6758, SUC_lexicon_tags_14 },
    { 0x574d6758, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34d675e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4cd6760, SUC_lexicon_tags_45 },
    { 0xc5ed6760, SUC_lexicon_tags_7 },
    { 0x65ad6762, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x306d6764, SUC_lexicon_tags_0 },
    { 0xf70d6764, SUC_lexicon_tags_35 },
    { 0xd5ad6766, SUC_lexicon_tags_4 },
    { 0x6a6d6764, SUC_lexicon_tags_1 },
    { 0xc2cd6768, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcf4d676a, SUC_lexicon_tags_23 },
    { 0x644d676b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc6ed676d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb52d676f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x69ad6771, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33ad6773, SUC_lexicon_tags_38 },
    { 0x66cd6773, SUC_lexicon_tags_17 },
    { 0x82cd6775, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x608d6777, SUC_lexicon_tags_60 },
    { 0xefcd6777, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ad677b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916d6781, SUC_lexicon_tags_14 },
    { 0xa20d6782, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138d6786, SUC_lexicon_tags_68 },
    { 0x9bed6787, SUC_lexicon_tags_47 },
    { 0x6fed6788, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb10d678a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ad678e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c2d6790, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6d6798, SUC_lexicon_tags_16 },
    { 0xce6d6799, SUC_lexicon_tags_37 },
    { 0x194d679a, SUC_lexicon_tags_11 },
    { 0x7ccd679a, SUC_lexicon_tags_72 },
    { 0xf3ed679c, SUC_lexicon_tags_37 },
    { 0xe5ad6798, SUC_lexicon_tags_39 },
    { 0xfbed679e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcd67a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12d67a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4d67a9, SUC_lexicon_tags_7 },
    { 0x578d67aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58d67af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24d67b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370d67b5, SUC_lexicon_tags_17 },
    { 0xc80d67b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ed67ba, SUC_lexicon_tags_10 },
    { 0xfd6d67ba, SUC_lexicon_tags_14 },
    { 0x952d67ba, SUC_lexicon_tags_38 },
    { 0xd50d67ba, SUC_lexicon_tags_179 },
    { 0xd4ed67bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x814d67c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6d67c7, SUC_lexicon_tags_17 },
    { 0x95cd67c7, SUC_lexicon_tags_59 },
    { 0x46ed67c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42d67cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1b6d67ce, SUC_lexicon_tags_37 },
    { 0x484d67ce, SUC_lexicon_tags_27 },
    { 0x92d67d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90d67d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2d67da, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234d67de, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126d67ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cd67ed, SUC_lexicon_tags_12 },
    { 0x9f0d67ee, SUC_lexicon_tags_45 },
    { 0x64d67ef, SUC_lexicon_tags_47 },
    { 0xbd4d67f0, SUC_lexicon_tags_16 },
    { 0x722d67f1, SUC_lexicon_tags_27 },
    { 0xd2ed67f2, SUC_lexicon_tags_12 },
    { 0xa34d67f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2aad67f5, SUC_lexicon_tags_25 },
    { 0x528d67f6, SUC_lexicon_tags_7 },
    { 0x7c4d67f7, SUC_lexicon_tags_37 },
    { 0xb68d67f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdecd67fb, SUC_lexicon_tags_12 },
    { 0x2ecd67fc, SUC_lexicon_tags_46 },
    { 0xfc6d67fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60d6803, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x776d6805, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7b6d6807, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70d680a, SUC_lexicon_tags_92 },
    { 0x88cd680a, SUC_lexicon_tags_27 },
    { 0x2d0d680b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ad6810, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8d6813, SUC_lexicon_tags_16 },
    { 0x1c6d6813, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2d6818, SUC_lexicon_tags_82 },
    { 0x874d6819, SUC_lexicon_tags_25 },
    { 0x146d681a, SUC_lexicon_tags_29 },
    { 0x22d6819, SUC_lexicon_tags_8 },
    { 0xc3ed681c, SUC_lexicon_tags_11 },
    { 0xeecd681d, SUC_lexicon_tags_47 },
    { 0xa56d681e, SUC_lexicon_tags_336 },
    { 0x42ed681f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7cd6821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x992d6824, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4d682a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cd6830, SUC_lexicon_tags_0 },
    { 0x924d6830, SUC_lexicon_tags_2 },
    { 0xaf0d6832, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8d683d, SUC_lexicon_tags_293 },
    { 0xea8d683d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676d6844, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ed6847, SUC_lexicon_tags_2 },
    { 0xd3ad6847, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xce2d684a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92d684d, SUC_lexicon_tags_15 },
    { 0xe42d684e, SUC_lexicon_tags_25 },
    { 0xec0d684f, SUC_lexicon_tags_43 },
    { 0x590d6850, SUC_lexicon_tags_47 },
    { 0xf28d684e, SUC_lexicon_tags_7 },
    { 0xf92d684e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad6855, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6d6859, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5f6d685b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8d685e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ead6862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c4d6864, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8d6869, SUC_lexicon_tags_48 },
    { 0x28ad686a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8d686e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x380d6870, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1ded6872, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974d6878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236d687d, SUC_lexicon_tags_31 },
    { 0xaf8d687d, SUC_lexicon_tags_27 },
    { 0x480d687e, SUC_lexicon_tags_1 },
    { 0x5c6d6880, SUC_lexicon_tags_7 },
    { 0x8ead6881, SUC_lexicon_tags_37 },
    { 0x26d6882, SUC_lexicon_tags_37 },
    { 0xa1ed6880, SUC_lexicon_tags_12 },
    { 0x882d6881, SUC_lexicon_tags_33 },
    { 0xe44d6885, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xde8d6887, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ad688e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0d6895, SUC_lexicon_tags_39 },
    { 0x470d6896, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6d689a, SUC_lexicon_tags_38 },
    { 0x978d689b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4d68a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x920d68a4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccd68a8, SUC_lexicon_tags_29 },
    { 0x874d68a9, SUC_lexicon_tags_7 },
    { 0xe26d68aa, SUC_lexicon_tags_14 },
    { 0xdfcd68ab, SUC_lexicon_tags_61 },
    { 0xd9ad68aa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x84ed68ae, SUC_lexicon_tags_1 },
    { 0xefad68af, SUC_lexicon_tags_11 },
    { 0xeb2d68b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad68b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc50d68ba, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cd68be, SUC_lexicon_tags_289 },
    { 0xfcd68bf, SUC_lexicon_tags_6 },
    { 0xb4d68c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x258d68c2, SUC_lexicon_tags_33 },
    { 0xd26d68c3, SUC_lexicon_tags_38 },
    { 0xed4d68c3, SUC_lexicon_tags_2 },
    { 0x64d68c5, SUC_lexicon_tags_25 },
    { 0x60d68c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x442d68c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x544d68ca, SUC_lexicon_tags_1 },
    { 0x334d68cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ed68d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118d68d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0d68da, SUC_lexicon_tags_31 },
    { 0x4d68db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252d68df, SUC_lexicon_tags_129 },
    { 0x174d68e0, SUC_lexicon_tags_7 },
    { 0xb18d68e1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x42ed68e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b8d68e5, SUC_lexicon_tags_56 },
    { 0xcf8d68e5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7acd68e8, SUC_lexicon_tags_63 },
    { 0x892d68e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x458d68eb, SUC_lexicon_tags_12 },
    { 0x54ed68ec, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe9cd68ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e6d68f0, SUC_lexicon_tags_15 },
    { 0x74cd68f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x602d68f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ded68f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0d68f9, SUC_lexicon_tags_16 },
    { 0xd8d68fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2d6900, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x702d6902, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ead6904, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18d690a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x198d690c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46d6912, SUC_lexicon_tags_11 },
    { 0x640d6912, SUC_lexicon_tags_25 },
    { 0xbd6d6914, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc58d6916, SUC_lexicon_tags_30 },
    { 0x6aed6916, SUC_lexicon_tags_8 },
    { 0xfad6916, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498d691c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeecd691f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ed6924, SUC_lexicon_tags_26 },
    { 0x5f6d6924, SUC_lexicon_tags_17 },
    { 0x4d2d6925, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562d692b, SUC_lexicon_tags_1 },
    { 0x752d692c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6eed692e, SUC_lexicon_tags_82 },
    { 0xbe8d692e, SUC_lexicon_tags_23 },
    { 0xd4ad6930, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96d6934, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4d6937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x536d6939, SUC_lexicon_tags_7 },
    { 0x6ed693a, SUC_lexicon_tags_15 },
    { 0x66ad693a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ad693e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0d6941, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46d6945, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0d6948, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33ed694a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58d694d, SUC_lexicon_tags_37 },
    { 0x686d694d, SUC_lexicon_tags_17 },
    { 0xfcd694f, SUC_lexicon_tags_11 },
    { 0x2cad6950, SUC_lexicon_tags_15 },
    { 0xedad6951, SUC_lexicon_tags_38 },
    { 0x730d6952, SUC_lexicon_tags_39 },
    { 0x63cd6953, SUC_lexicon_tags_43 },
    { 0xbe4d6954, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fed6956, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b2d6958, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7e6d695a, SUC_lexicon_tags_33 },
    { 0xf68d695b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d6962, SUC_lexicon_tags_49 },
    { 0x892d6963, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2d696a, SUC_lexicon_tags_7 },
    { 0xd0ad696b, SUC_lexicon_tags_47 },
    { 0xa98d696b, SUC_lexicon_tags_7 },
    { 0x262d696d, SUC_lexicon_tags_1 },
    { 0x25ed696d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ad6970, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72d6978, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8dad697a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x302d697d, SUC_lexicon_tags_28 },
    { 0xacd697e, SUC_lexicon_tags_21 },
    { 0xe0ed697f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bad6983, SUC_lexicon_tags_17 },
    { 0x9c4d6984, SUC_lexicon_tags_101 },
    { 0xc62d6985, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46d6989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f0d698b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06d6990, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874d6995, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6d6998, SUC_lexicon_tags_16 },
    { 0xaaad6998, SUC_lexicon_tags_37 },
    { 0x970d699a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96d699d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe92d699f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccd69a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x174d69a5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92d69aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf20d69ac, SUC_lexicon_tags_25 },
    { 0x12ad69ad, SUC_lexicon_tags_14 },
    { 0x438d69ae, SUC_lexicon_tags_23 },
    { 0xe6ad69ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x11cd69b1, SUC_lexicon_tags_0 },
    { 0x40ed69b2, SUC_lexicon_tags_11 },
    { 0xe4ed69b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ad69b8, SUC_lexicon_tags_1 },
    { 0xe8cd69b8, SUC_lexicon_tags_37 },
    { 0x86ad69ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ad69c5, SUC_lexicon_tags_25 },
    { 0xd34d69c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x736d69c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978d69d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed69d9, SUC_lexicon_tags_29 },
    { 0x646d69da, SUC_lexicon_tags_32 },
    { 0x94ed69da, SUC_lexicon_tags_17 },
    { 0x350d69dc, SUC_lexicon_tags_15 },
    { 0xb08d69db, SUC_lexicon_tags_42 },
    { 0xf5cd69d9, SUC_lexicon_tags_9 },
    { 0x55ed69dc, SUC_lexicon_tags_38 },
    { 0x262d69e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6d69e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d69ea, SUC_lexicon_tags_23 },
    { 0xba6d69ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0d69ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992d69f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ad69f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x574d69fb, SUC_lexicon_tags_66 },
    { 0xe8d69fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9eed69fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68d6a01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94d6a04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcd6a09, SUC_lexicon_tags_6 },
    { 0xbbed6a0a, SUC_lexicon_tags_14 },
    { 0x456d6a0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12d6a0f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558d6a14, SUC_lexicon_tags_15 },
    { 0xd74d6a15, SUC_lexicon_tags_16 },
    { 0x20cd6a14, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4d6a19, SUC_lexicon_tags_12 },
    { 0xc0ed6a1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8d6a23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ed6a28, SUC_lexicon_tags_12 },
    { 0x114d6a28, SUC_lexicon_tags_100 },
    { 0x29ed6a29, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20d6a30, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc6cd6a32, SUC_lexicon_tags_63 },
    { 0x894d6a33, SUC_lexicon_tags_17 },
    { 0xd02d6a34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbd6d6a36, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ad6a39, SUC_lexicon_tags_60 },
    { 0x810d6a3a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacd6a3f, SUC_lexicon_tags_39 },
    { 0xda2d6a40, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x47cd6a42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3acd6a44, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cd6a4a, SUC_lexicon_tags_72 },
    { 0x63ed6a4b, SUC_lexicon_tags_33 },
    { 0x392d6a4c, SUC_lexicon_tags_25 },
    { 0x9aad6a4c, SUC_lexicon_tags_11 },
    { 0xcc2d6a4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cad6a51, SUC_lexicon_tags_12 },
    { 0x52d6a52, SUC_lexicon_tags_26 },
    { 0x9d8d6a52, SUC_lexicon_tags_57 },
    { 0x1ced6a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aad6a56, SUC_lexicon_tags_29 },
    { 0xec0d6a57, SUC_lexicon_tags_33 },
    { 0x5fed6a56, SUC_lexicon_tags_17 },
    { 0x4e0d6a58, SUC_lexicon_tags_25 },
    { 0x7a4d6a59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1cd6a5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x636d6a5f, SUC_lexicon_tags_113 },
    { 0x576d6a60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42d6a65, SUC_lexicon_tags_47 },
    { 0x4f2d6a66, SUC_lexicon_tags_43 },
    { 0xc86d6a65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x480d6a69, SUC_lexicon_tags_87 },
    { 0x8ecd6a6a, SUC_lexicon_tags_11 },
    { 0x9e4d6a6b, SUC_lexicon_tags_25 },
    { 0xf98d6a6c, SUC_lexicon_tags_19 },
    { 0x9e8d6a6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0d6a71, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb6ed6a73, SUC_lexicon_tags_11 },
    { 0xe6ad6a74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x892d6a77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6ed6a79, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06d6a7c, SUC_lexicon_tags_46 },
    { 0x4ced6a7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42ad6a7f, SUC_lexicon_tags_0 },
    { 0xc6ad6a80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482d6a84, SUC_lexicon_tags_6 },
    { 0x61ed6a85, SUC_lexicon_tags_27 },
    { 0x2ced6a85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ad6a8a, SUC_lexicon_tags_11 },
    { 0x17cd6a8b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x758d6a8e, SUC_lexicon_tags_48 },
    { 0x3d0d6a8e, SUC_lexicon_tags_43 },
    { 0xf36d6a90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d6a97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cd6a9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccd6aa1, SUC_lexicon_tags_12 },
    { 0xc8d6aa2, SUC_lexicon_tags_15 },
    { 0xccd6aa3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ad6aa6, SUC_lexicon_tags_60 },
    { 0x2ad6aa7, SUC_lexicon_tags_14 },
    { 0x898d6aa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6d6aac, SUC_lexicon_tags_12 },
    { 0x32cd6aad, SUC_lexicon_tags_48 },
    { 0xbfcd6aae, SUC_lexicon_tags_13 },
    { 0x24ed6aaf, SUC_lexicon_tags_17 },
    { 0xd2ed6aac, SUC_lexicon_tags_27 },
    { 0x754d6ab1, SUC_lexicon_tags_49 },
    { 0xc32d6ab2, SUC_lexicon_tags_37 },
    { 0xea8d6aae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0d6ab9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xafad6abc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6dcd6abe, SUC_lexicon_tags_28 },
    { 0x638d6abf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4e4d6ac1, SUC_lexicon_tags_1 },
    { 0xbf4d6ac1, SUC_lexicon_tags_1 },
    { 0x79ed6ac3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x930d6ac5, SUC_lexicon_tags_45 },
    { 0x4dad6ac6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4d6acc, SUC_lexicon_tags_12 },
    { 0xaaad6acd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8d2d6acf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72d6ad3, SUC_lexicon_tags_45 },
    { 0x36cd6ad4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x284d6ad6, SUC_lexicon_tags_60 },
    { 0x8ead6ad7, SUC_lexicon_tags_2 },
    { 0x114d6ad7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cd6ae1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4d6ae5, SUC_lexicon_tags_43 },
    { 0x7aed6ae6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ed6af1, SUC_lexicon_tags_6 },
    { 0xe92d6af2, SUC_lexicon_tags_11 },
    { 0x9ccd6af1, SUC_lexicon_tags_11 },
    { 0xffad6af4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764d6af9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ad6afe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x172d6b01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc76d6b03, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x828d6b05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68d6b0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d6b16, SUC_lexicon_tags_6 },
    { 0x514d6b16, SUC_lexicon_tags_63 },
    { 0x942d6b17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcad6b1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe3ad6b1c, SUC_lexicon_tags_29 },
    { 0xbed6b1d, SUC_lexicon_tags_60 },
    { 0xce6d6b1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728d6b25, SUC_lexicon_tags_279 },
    { 0x1c8d6b26, SUC_lexicon_tags_79 },
    { 0x824d6b27, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc3ad6b29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ad6b2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fad6b2f, SUC_lexicon_tags_17 },
    { 0x460d6b30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf12d6b32, SUC_lexicon_tags_27 },
    { 0xcad6b33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9a4d6b35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcd6b39, SUC_lexicon_tags_25 },
    { 0xff2d6b3a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42d6b3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84d6b43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x126d6b46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1cd6b48, SUC_lexicon_tags_55 },
    { 0x970d6b49, SUC_lexicon_tags_12 },
    { 0x48cd6b48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xecad6b4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf78d6b4f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x33ed6b51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcd6b56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdad6b5f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6b64, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8d6b67, SUC_lexicon_tags_7 },
    { 0x338d6b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x114d6b6a, SUC_lexicon_tags_15 },
    { 0x10cd6b6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ad6b6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eed6b76, SUC_lexicon_tags_33 },
    { 0x70ad6b77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80d6b7d, SUC_lexicon_tags_9 },
    { 0x7d2d6b7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad6b83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x920d6b86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4d6b89, SUC_lexicon_tags_11 },
    { 0x9a0d6b8a, SUC_lexicon_tags_43 },
    { 0x79ed6b8a, SUC_lexicon_tags_43 },
    { 0xee2d6b8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fed6b92, SUC_lexicon_tags_25 },
    { 0x582d6b93, SUC_lexicon_tags_37 },
    { 0xcd6b94, SUC_lexicon_tags_12 },
    { 0xb46d6b95, SUC_lexicon_tags_18 },
    { 0xc6cd6b95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dcd6b98, SUC_lexicon_tags_21 },
    { 0x4cd6b98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ed6b9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7cd6b9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0d6ba9, SUC_lexicon_tags_11 },
    { 0xa92d6ba9, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86d6bae, SUC_lexicon_tags_11 },
    { 0x62ed6bae, SUC_lexicon_tags_27 },
    { 0x77cd6bb0, SUC_lexicon_tags_11 },
    { 0x716d6bb0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606d6bb5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454d6bbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ad6bc1, SUC_lexicon_tags_1 },
    { 0x7fad6bc2, SUC_lexicon_tags_29 },
    { 0x4d0d6bc2, SUC_lexicon_tags_15 },
    { 0x30d6bc4, SUC_lexicon_tags_37 },
    { 0x1bed6bc2, SUC_lexicon_tags_12 },
    { 0x39cd6bc6, SUC_lexicon_tags_30 },
    { 0x70d6bc7, SUC_lexicon_tags_4 },
    { 0x78ad6bc3, SUC_lexicon_tags_170 },
    { 0xb20d6bc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x550d6bcc, SUC_lexicon_tags_15 },
    { 0x676d6bcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbed6bcf, SUC_lexicon_tags_53 },
    { 0x3dad6bcf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940d6bd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87ed6bd7, SUC_lexicon_tags_63 },
    { 0xad6d6bd8, SUC_lexicon_tags_60 },
    { 0x2ced6bd8, SUC_lexicon_tags_11 },
    { 0xafcd6bd7, SUC_lexicon_tags_17 },
    { 0xb5ad6bdb, SUC_lexicon_tags_11 },
    { 0xfc2d6bd8, SUC_lexicon_tags_25 },
    { 0x8dad6bdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0cd6bdf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d6be6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x23cd6be8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56d6beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabed6bed, SUC_lexicon_tags_1 },
    { 0x2d4d6bed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0d6bf1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccd6bf5, SUC_lexicon_tags_2 },
    { 0xcecd6bf6, SUC_lexicon_tags_1 },
    { 0x9a4d6bf7, SUC_lexicon_tags_1 },
    { 0x3aed6bf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42d6bfb, SUC_lexicon_tags_25 },
    { 0xed6bfc, SUC_lexicon_tags_38 },
    { 0x632d6bfd, SUC_lexicon_tags_45 },
    { 0x8c4d6bfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28d6c03, SUC_lexicon_tags_5 },
    { 0x16ad6c04, SUC_lexicon_tags_15 },
    { 0x3c6d6c03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4d6c08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576d6c0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x596d6c0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80ad6c10, SUC_lexicon_tags_26 },
    { 0xb8ad6c11, SUC_lexicon_tags_49 },
    { 0x40cd6c12, SUC_lexicon_tags_17 },
    { 0xd64d6c11, SUC_lexicon_tags_7 },
    { 0x8ad6c14, SUC_lexicon_tags_79 },
    { 0xa38d6c14, SUC_lexicon_tags_25 },
    { 0x340d6c11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x640d6c18, SUC_lexicon_tags_17 },
    { 0xf72d6c18, SUC_lexicon_tags_23 },
    { 0x7ecd6c1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ad6c22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76d6c26, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cd6c2a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fed6c2f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cd6c34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50d6c37, SUC_lexicon_tags_37 },
    { 0x82d6c38, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3ced6c3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac2d6c3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164d6c47, SUC_lexicon_tags_29 },
    { 0x9f4d6c47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838d6c4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x492d6c51, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d6c59, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14d6c5c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ad6c5f, SUC_lexicon_tags_63 },
    { 0x636d6c60, SUC_lexicon_tags_7 },
    { 0x9e8d6c61, SUC_lexicon_tags_17 },
    { 0x8aad6c62, SUC_lexicon_tags_2 },
    { 0xa18d6c62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0d6c6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4d6c6e, SUC_lexicon_tags_63 },
    { 0xa92d6c6e, SUC_lexicon_tags_11 },
    { 0xd26d6c6e, SUC_lexicon_tags_37 },
    { 0xdead6c6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56d6c78, SUC_lexicon_tags_45 },
    { 0x118d6c78, SUC_lexicon_tags_27 },
    { 0xee0d6c7a, SUC_lexicon_tags_14 },
    { 0x9ad6c7a, SUC_lexicon_tags_28 },
    { 0x60ad6c7c, SUC_lexicon_tags_15 },
    { 0xed0d6c7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ad6c81, SUC_lexicon_tags_7 },
    { 0x2ed6c82, SUC_lexicon_tags_29 },
    { 0x7e4d6c83, SUC_lexicon_tags_15 },
    { 0x16ed6c84, SUC_lexicon_tags_27 },
    { 0xfe4d6c85, SUC_lexicon_tags_6 },
    { 0x9d2d6c85, SUC_lexicon_tags_2 },
    { 0x23cd6c87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ad6c8d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792d6c99, SUC_lexicon_tags_47 },
    { 0x5a2d6c99, SUC_lexicon_tags_25 },
    { 0x438d6c9a, SUC_lexicon_tags_7 },
    { 0xdd0d6c9c, SUC_lexicon_tags_11 },
    { 0x79cd6c9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe1ad6c9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb40d6ca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6d6caa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cd6cae, SUC_lexicon_tags_17 },
    { 0x836d6caf, SUC_lexicon_tags_39 },
    { 0xc8ed6cb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ad6cb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654d6cba, SUC_lexicon_tags_15 },
    { 0x8a2d6cbb, SUC_lexicon_tags_15 },
    { 0x886d6cbc, SUC_lexicon_tags_1 },
    { 0xc84d6cbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x410d6cc0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf74d6cc2, SUC_lexicon_tags_4 },
    { 0x5dcd6cc3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6d6cc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76cd6cc9, SUC_lexicon_tags_37 },
    { 0xf58d6cca, SUC_lexicon_tags_45 },
    { 0x556d6cca, SUC_lexicon_tags_33 },
    { 0xad8d6ccc, SUC_lexicon_tags_12 },
    { 0x912d6cca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad6cd1, SUC_lexicon_tags_33 },
    { 0x3fcd6cd2, SUC_lexicon_tags_32 },
    { 0x6bed6cd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2dcd6cd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d6d6cd7, SUC_lexicon_tags_68 },
    { 0xad4d6cd8, SUC_lexicon_tags_38 },
    { 0xd5ed6cd9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ded6cdd, SUC_lexicon_tags_37 },
    { 0xdc2d6cdd, SUC_lexicon_tags_25 },
    { 0x8a0d6cdf, SUC_lexicon_tags_26 },
    { 0xe90d6ce0, SUC_lexicon_tags_138 },
    { 0x2d2d6ce1, SUC_lexicon_tags_17 },
    { 0xf04d6ce1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12d6ce6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc80d6ce8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ed6ceb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296d6cf0, SUC_lexicon_tags_37 },
    { 0x452d6cf1, SUC_lexicon_tags_8 },
    { 0x2bed6cf1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6d6cf5, SUC_lexicon_tags_28 },
    { 0x52d6cf6, SUC_lexicon_tags_31 },
    { 0xf04d6cf7, SUC_lexicon_tags_39 },
    { 0x54ad6cf7, SUC_lexicon_tags_6 },
    { 0x810d6cf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a2d6cfb, SUC_lexicon_tags_19 },
    { 0x7f4d6cfc, SUC_lexicon_tags_1 },
    { 0xadad6cfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962d6d01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04d6d04, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6d6d08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2d6d0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc12d6d0d, SUC_lexicon_tags_15 },
    { 0x4e4d6d0d, SUC_lexicon_tags_39 },
    { 0x288d6d0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6d6d1a, SUC_lexicon_tags_15 },
    { 0x8ead6d1b, SUC_lexicon_tags_12 },
    { 0x40d6d1c, SUC_lexicon_tags_6 },
    { 0x280d6d1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54d6d23, SUC_lexicon_tags_2 },
    { 0xc88d6d24, SUC_lexicon_tags_43 },
    { 0xab4d6d25, SUC_lexicon_tags_26 },
    { 0x6d6d6d26, SUC_lexicon_tags_29 },
    { 0x474d6d26, SUC_lexicon_tags_14 },
    { 0x70d6d28, SUC_lexicon_tags_1 },
    { 0x214d6d29, SUC_lexicon_tags_29 },
    { 0x7e6d6d2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ad6d31, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x38cd6d33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628d6d39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbad6d3d, SUC_lexicon_tags_7 },
    { 0x12ad6d3e, SUC_lexicon_tags_12 },
    { 0xb02d6d3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x532d6d41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac0d6d43, SUC_lexicon_tags_11 },
    { 0x836d6d44, SUC_lexicon_tags_17 },
    { 0xecad6d44, SUC_lexicon_tags_11 },
    { 0xdc6d6d43, SUC_lexicon_tags_63 },
    { 0x6f4d6d47, SUC_lexicon_tags_46 },
    { 0x25ad6d43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccd6d4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440d6d4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2d6d53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6d6d57, SUC_lexicon_tags_15 },
    { 0x44cd6d58, SUC_lexicon_tags_0 },
    { 0xfc0d6d58, SUC_lexicon_tags_11 },
    { 0x158d6d57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeed6d5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6d6d62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xea8d6d64, SUC_lexicon_tags_11 },
    { 0xc00d6d65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44d6d75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356d6d7a, SUC_lexicon_tags_15 },
    { 0xf6cd6d7b, SUC_lexicon_tags_62 },
    { 0xfacd6d7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc86d6d7e, SUC_lexicon_tags_10 },
    { 0x408d6d7e, SUC_lexicon_tags_37 },
    { 0x9c8d6d80, SUC_lexicon_tags_1 },
    { 0x7d8d6d7f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88d6d86, SUC_lexicon_tags_27 },
    { 0xbd0d6d87, SUC_lexicon_tags_87 },
    { 0xfaed6d87, SUC_lexicon_tags_7 },
    { 0x1dad6d89, SUC_lexicon_tags_60 },
    { 0xd5ad6d8a, SUC_lexicon_tags_27 },
    { 0x35ed6d8b, SUC_lexicon_tags_15 },
    { 0x58cd6d8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3ad6d8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeded6d90, SUC_lexicon_tags_23 },
    { 0x83ed6d91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ed6d96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ed6da1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83cd6da3, SUC_lexicon_tags_13 },
    { 0xaf0d6da4, SUC_lexicon_tags_2 },
    { 0x37ed6da5, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xd86d6da7, SUC_lexicon_tags_1 },
    { 0xa68d6da8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbded6dac, SUC_lexicon_tags_92 },
    { 0x7ad6dac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46d6db2, SUC_lexicon_tags_13 },
    { 0x89cd6db2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4d6db6, SUC_lexicon_tags_12 },
    { 0xfb4d6db7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa00d6db9, SUC_lexicon_tags_15 },
    { 0xd4d6db9, SUC_lexicon_tags_17 },
    { 0x9cad6dbb, SUC_lexicon_tags_11 },
    { 0x5b4d6dbb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x674d6dbe, SUC_lexicon_tags_60 },
    { 0x2d8d6dbf, SUC_lexicon_tags_31 },
    { 0x5ced6dc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d6dc3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cad6dc8, SUC_lexicon_tags_1 },
    { 0x3d6d6dc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd6d6dcb, SUC_lexicon_tags_38 },
    { 0x3f2d6dcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0d6dcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d8d6dd1, SUC_lexicon_tags_11 },
    { 0x784d6dd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdaad6dd4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468d6ddc, SUC_lexicon_tags_49 },
    { 0x14d6ddd, SUC_lexicon_tags_17 },
    { 0xfced6ddd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8d4d6de0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa46d6de2, SUC_lexicon_tags_11 },
    { 0x4c2d6de3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8d6de7, SUC_lexicon_tags_17 },
    { 0x6ced6de8, SUC_lexicon_tags_1 },
    { 0x292d6de8, SUC_lexicon_tags_17 },
    { 0x52ed6dea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cd6dee, SUC_lexicon_tags_37 },
    { 0x8ed6def, SUC_lexicon_tags_1 },
    { 0xfcd6def, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x156d6df3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcd6df6, SUC_lexicon_tags_69 },
    { 0x4aad6df7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeb2d6df9, SUC_lexicon_tags_4 },
    { 0x9b6d6dfa, SUC_lexicon_tags_2 },
    { 0xaaed6dfa, SUC_lexicon_tags_38 },
    { 0x54ed6dfc, SUC_lexicon_tags_17 },
    { 0xef6d6dfd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbd4d6dff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe38d6e01, SUC_lexicon_tags_11 },
    { 0x450d6e01, SUC_lexicon_tags_60 },
    { 0x1fad6e03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x250d6e05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x576d6e07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cd6e0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388d6e13, SUC_lexicon_tags_13 },
    { 0xe50d6e14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ad6e18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d6e1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6d6e27, SUC_lexicon_tags_30 },
    { 0x68ed6e28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x290d6e2a, SUC_lexicon_tags_14 },
    { 0x5cad6e2b, SUC_lexicon_tags_14 },
    { 0x8d2d6e2a, SUC_lexicon_tags_27 },
    { 0x29cd6e2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26d6e32, SUC_lexicon_tags_15 },
    { 0x548d6e33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x576d6e35, SUC_lexicon_tags_1149 },
    { 0x6eed6e36, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36d6e3b, SUC_lexicon_tags_62 },
    { 0x2aad6e3b, SUC_lexicon_tags_37 },
    { 0x2fad6e3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ed6e42, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cd6e45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0d6e49, SUC_lexicon_tags_12 },
    { 0x4f8d6e4a, SUC_lexicon_tags_16 },
    { 0x120d6e4b, SUC_lexicon_tags_2 },
    { 0xbb2d6e4b, SUC_lexicon_tags_31 },
    { 0x60cd6e4b, SUC_lexicon_tags_27 },
    { 0x564d6e4e, SUC_lexicon_tags_32 },
    { 0xa6cd6e4f, SUC_lexicon_tags_1 },
    { 0x216d6e4e, SUC_lexicon_tags_38 },
    { 0x47ed6e51, SUC_lexicon_tags_15 },
    { 0x608d6e49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34d6e56, SUC_lexicon_tags_2 },
    { 0xb0ad6e57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0d6e5a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa40d6e5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x484d6e5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74ed6e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cd6e63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x494d6e65, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad6e68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbcd6e6a, SUC_lexicon_tags_11 },
    { 0xfbed6e6a, SUC_lexicon_tags_12 },
    { 0x7eed6e6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe22d6e6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8d6e71, SUC_lexicon_tags_8 },
    { 0xebcd6e72, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x138d6e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74ad6e77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8ed6e79, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52d6e7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494d6e80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79cd6e82, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x84d6e84, SUC_lexicon_tags_43 },
    { 0x69ed6e85, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996d6e89, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad6e8e, SUC_lexicon_tags_17 },
    { 0x9c6d6e8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x17cd6e91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52d6e98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4d6ea4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fed6ea8, SUC_lexicon_tags_12 },
    { 0xc9ed6ea8, SUC_lexicon_tags_35 },
    { 0x690d6ea9, SUC_lexicon_tags_26 },
    { 0x6a8d6ea8, SUC_lexicon_tags_45 },
    { 0x7eed6eac, SUC_lexicon_tags_15 },
    { 0x4e8d6ead, SUC_lexicon_tags_2 },
    { 0x8bcd6ea9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x622d6eb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x284d6eb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8cd6eb6, SUC_lexicon_tags_17 },
    { 0xb1cd6eb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x72cd6eb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x480d6ebb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x194d6ebd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fad6ec2, SUC_lexicon_tags_1 },
    { 0xd36d6ec3, SUC_lexicon_tags_25 },
    { 0xd60d6ec4, SUC_lexicon_tags_25 },
    { 0xe68d6ec4, SUC_lexicon_tags_37 },
    { 0xf66d6ec6, SUC_lexicon_tags_45 },
    { 0xb56d6ec6, SUC_lexicon_tags_27 },
    { 0x23ed6ec8, SUC_lexicon_tags_84 },
    { 0x966d6ec9, SUC_lexicon_tags_0 },
    { 0x3f2d6ec3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d6ece, SUC_lexicon_tags_30 },
    { 0x630d6ecf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ed6ed2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ad6ed5, SUC_lexicon_tags_17 },
    { 0xb84d6ed6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe34d6ed8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cd6ede, SUC_lexicon_tags_82 },
    { 0x86ad6edf, SUC_lexicon_tags_25 },
    { 0xa6d6edf, SUC_lexicon_tags_7 },
    { 0xf36d6ede, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2d6ee5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ad6ee9, SUC_lexicon_tags_27 },
    { 0x4d6eea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x17ad6eec, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152d6ef1, SUC_lexicon_tags_87 },
    { 0x7ccd6ef1, SUC_lexicon_tags_33 },
    { 0xd38d6ef3, SUC_lexicon_tags_43 },
    { 0xe0ed6ef4, SUC_lexicon_tags_17 },
    { 0x4dcd6ef5, SUC_lexicon_tags_11 },
    { 0x870d6ef6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91cd6ef8, SUC_lexicon_tags_2 },
    { 0x7b0d6ef9, SUC_lexicon_tags_55 },
    { 0x916d6ef8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cad6efe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6d6f01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ded6f04, SUC_lexicon_tags_11 },
    { 0xfcad6f05, SUC_lexicon_tags_2 },
    { 0xe44d6f05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4fed6f08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x304d6f0a, SUC_lexicon_tags_62 },
    { 0x40ed6f0a, SUC_lexicon_tags_25 },
    { 0xf7ed6f0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6f10, SUC_lexicon_tags_0 },
    { 0x3ed6f10, SUC_lexicon_tags_17 },
    { 0x75ad6f12, SUC_lexicon_tags_17 },
    { 0x4b0d6f13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04d6f1a, SUC_lexicon_tags_12 },
    { 0x94ad6f1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52d6f22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x330d6f25, SUC_lexicon_tags_17 },
    { 0xbf2d6f26, SUC_lexicon_tags_38 },
    { 0xe28d6f27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f4d6f29, SUC_lexicon_tags_17 },
    { 0x23ed6f29, SUC_lexicon_tags_27 },
    { 0xbced6f2b, SUC_lexicon_tags_2 },
    { 0xfc0d6f29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38d6f34, SUC_lexicon_tags_37 },
    { 0x16ed6f35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ed6f38, SUC_lexicon_tags_25 },
    { 0xac6d6f39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22d6f3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9b6d6f3f, SUC_lexicon_tags_1 },
    { 0x846d6f3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782d6f44, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc2d6f46, SUC_lexicon_tags_31 },
    { 0x60ad6f47, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2d6f4a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212d6f4e, SUC_lexicon_tags_37 },
    { 0x428d6f4e, SUC_lexicon_tags_11 },
    { 0x480d6f50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x806d6f53, SUC_lexicon_tags_25 },
    { 0xb8cd6f53, SUC_lexicon_tags_29 },
    { 0x464d6f55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2d6f58, SUC_lexicon_tags_47 },
    { 0x856d6f58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcd6f64, SUC_lexicon_tags_11 },
    { 0x47ed6f65, SUC_lexicon_tags_33 },
    { 0xdc2d6f65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40d6f6d, SUC_lexicon_tags_36 },
    { 0xc54d6f6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc0d6f70, SUC_lexicon_tags_14 },
    { 0xf86d6f71, SUC_lexicon_tags_0 },
    { 0x624d6f72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acd6f80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bed6f83, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3b2d6f85, SUC_lexicon_tags_18 },
    { 0x2ecd6f86, SUC_lexicon_tags_37 },
    { 0xe30d6f87, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48d6f8a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8d6f8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fed6f91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86d6f97, SUC_lexicon_tags_17 },
    { 0x5b0d6f97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74d6f9f, SUC_lexicon_tags_12 },
    { 0x4b8d6fa0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x600d6fa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x920d6fa5, SUC_lexicon_tags_3 },
    { 0xfad6fa5, SUC_lexicon_tags_11 },
    { 0xd58d6fa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x872d6faa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccd6fae, SUC_lexicon_tags_15 },
    { 0x600d6fae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26d6fb4, SUC_lexicon_tags_19 },
    { 0x812d6fb4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcd6fb8, SUC_lexicon_tags_84 },
    { 0x240d6fb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fad6fbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ed6fc0, SUC_lexicon_tags_7 },
    { 0xa0ad6fc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0ed6fc3, SUC_lexicon_tags_12 },
    { 0x480d6fc4, SUC_lexicon_tags_14 },
    { 0xe3cd6fc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cd6fca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96ad6fcc, SUC_lexicon_tags_2 },
    { 0x4c2d6fcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a0d6fcf, SUC_lexicon_tags_32 },
    { 0x2a6d6fd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182d6fd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6d6fd9, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6d6fdc, SUC_lexicon_tags_2 },
    { 0xe36d6fdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fed6fe2, SUC_lexicon_tags_12 },
    { 0x490d6fe3, SUC_lexicon_tags_8 },
    { 0x3eed6fe4, SUC_lexicon_tags_7 },
    { 0xba2d6fe5, SUC_lexicon_tags_17 },
    { 0x5e4d6fe6, SUC_lexicon_tags_37 },
    { 0xb32d6fe7, SUC_lexicon_tags_25 },
    { 0x42ed6fe8, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6fec, SUC_lexicon_tags_15 },
    { 0x2c2d6fed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cad6ff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf0d6ff3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x576d6ff6, SUC_lexicon_tags_30 },
    { 0x390d6ff7, SUC_lexicon_tags_37 },
    { 0x562d6ff8, SUC_lexicon_tags_33 },
    { 0xcfad6ff8, SUC_lexicon_tags_17 },
    { 0x2bad6ffa, SUC_lexicon_tags_30 },
    { 0x31ed6ffa, SUC_lexicon_tags_11 },
    { 0xc82d6ffa, SUC_lexicon_tags_23 },
    { 0x1ded6ffc, SUC_lexicon_tags_19 },
    { 0x706d6ffe, SUC_lexicon_tags_15 },
    { 0x3aad6ffa, SUC_lexicon_tags_0 },
    { 0x58ad7000, SUC_lexicon_tags_4 },
    { 0x9b4d7000, SUC_lexicon_tags_12 },
    { 0xfc6d7002, SUC_lexicon_tags_25 },
    { 0x79ad7000, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24d700a, SUC_lexicon_tags_116 },
    { 0xf48d700b, SUC_lexicon_tags_1 },
    { 0x8a8d700c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed0d700e, SUC_lexicon_tags_37 },
    { 0x666d700f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x968d7011, SUC_lexicon_tags_54 },
    { 0x9f0d7012, SUC_lexicon_tags_25 },
    { 0xbc0d7013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6d7017, SUC_lexicon_tags_37 },
    { 0xd36d7018, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x556d701a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x69ed701c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858d7020, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842d7024, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaed7028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acd702c, SUC_lexicon_tags_47 },
    { 0x670d702d, SUC_lexicon_tags_47 },
    { 0x9ed702e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x886d7031, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaad7034, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x56ad7036, SUC_lexicon_tags_12 },
    { 0x348d7037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ad7039, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0d703c, SUC_lexicon_tags_38 },
    { 0x4c2d703d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d7040, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4d7046, SUC_lexicon_tags_37 },
    { 0x4a8d7046, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6d704a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810d7050, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaed7054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6d705b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40d7060, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa6ad7062, SUC_lexicon_tags_11 },
    { 0x7d4d7063, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x716d7065, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x274d7068, SUC_lexicon_tags_23 },
    { 0xb4cd7069, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd52d706b, SUC_lexicon_tags_17 },
    { 0x5aed706c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86d706f, SUC_lexicon_tags_63 },
    { 0xb4cd7070, SUC_lexicon_tags_37 },
    { 0xfa2d7071, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d7076, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06d707d, SUC_lexicon_tags_48 },
    { 0xcf6d707e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10d7089, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4cad708b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ad708e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cd7091, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdcd7093, SUC_lexicon_tags_27 },
    { 0x798d7093, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cd7098, SUC_lexicon_tags_14 },
    { 0x468d7099, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x754d709c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fad709e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6d70a2, SUC_lexicon_tags_37 },
    { 0x9b4d70a2, SUC_lexicon_tags_11 },
    { 0xe56d70a4, SUC_lexicon_tags_12 },
    { 0xc0ed70a5, SUC_lexicon_tags_15 },
    { 0xa6d70a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x550d70a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ed70b2, SUC_lexicon_tags_1 },
    { 0x790d70b3, SUC_lexicon_tags_23 },
    { 0x39ad70b2, SUC_lexicon_tags_11 },
    { 0x6b0d70b5, SUC_lexicon_tags_37 },
    { 0x458d70b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d70ba, SUC_lexicon_tags_15 },
    { 0xa80d70ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ad70c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acd70c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x70ed70c7, SUC_lexicon_tags_0 },
    { 0x61cd70c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6d70cc, SUC_lexicon_tags_15 },
    { 0x326d70cc, SUC_lexicon_tags_61 },
    { 0x8d0d70ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdacd70d1, SUC_lexicon_tags_37 },
    { 0xa52d70d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x94ad70d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa6cd70d6, SUC_lexicon_tags_29 },
    { 0x160d70d7, SUC_lexicon_tags_4 },
    { 0x8c0d70d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcd70dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2d70df, SUC_lexicon_tags_2 },
    { 0xead70e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744d70e4, SUC_lexicon_tags_11 },
    { 0x460d70e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa68d70e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cd70ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x670d70f0, SUC_lexicon_tags_57 },
    { 0xb58d70f0, SUC_lexicon_tags_9 },
    { 0xe26d70f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18d70f9, SUC_lexicon_tags_27 },
    { 0x8e8d70fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14d70fd, SUC_lexicon_tags_2 },
    { 0xd14d70fe, SUC_lexicon_tags_7 },
    { 0x2b4d70fd, SUC_lexicon_tags_46 },
    { 0xba4d70fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x242d7102, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x556d7104, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc5ed7106, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaad710c, SUC_lexicon_tags_17 },
    { 0xc8d710c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78d7111, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x474d7113, SUC_lexicon_tags_65 },
    { 0xdbad7114, SUC_lexicon_tags_31 },
    { 0x86cd7115, SUC_lexicon_tags_30 },
    { 0x21ed7116, SUC_lexicon_tags_25 },
    { 0xabad7116, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60d711c, SUC_lexicon_tags_15 },
    { 0x63cd711d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeced7122, SUC_lexicon_tags_16 },
    { 0xcccd7123, SUC_lexicon_tags_19 },
    { 0x386d7124, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560d7128, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cd712b, SUC_lexicon_tags_65 },
    { 0x620d712b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0d7130, SUC_lexicon_tags_35 },
    { 0xd4d7131, SUC_lexicon_tags_2 },
    { 0xded7132, SUC_lexicon_tags_65 },
    { 0x1e2d7133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb0d7135, SUC_lexicon_tags_37 },
    { 0x1f2d7135, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x75ed7138, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20d713b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x540d713e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0d7141, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ead7144, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8d7153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6d7158, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x252d715a, SUC_lexicon_tags_2 },
    { 0x944d715b, SUC_lexicon_tags_47 },
    { 0xcb0d715c, SUC_lexicon_tags_15 },
    { 0x4ded715c, SUC_lexicon_tags_25 },
    { 0x8cd715c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x482d7160, SUC_lexicon_tags_7 },
    { 0xd9cd7161, SUC_lexicon_tags_0 },
    { 0x90ad7160, SUC_lexicon_tags_66 },
    { 0xe8cd7163, SUC_lexicon_tags_26 },
    { 0x5e8d7164, SUC_lexicon_tags_3 },
    { 0xa32d7164, SUC_lexicon_tags_37 },
    { 0xd7cd7164, SUC_lexicon_tags_28 },
    { 0x242d7165, SUC_lexicon_tags_60 },
    { 0x99ed7168, SUC_lexicon_tags_32 },
    { 0x512d7165, SUC_lexicon_tags_7 },
    { 0xdced716a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ed716d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58d7172, SUC_lexicon_tags_27 },
    { 0xc36d7172, SUC_lexicon_tags_37 },
    { 0xb5ed7174, SUC_lexicon_tags_17 },
    { 0x510d7174, SUC_lexicon_tags_92 },
    { 0x49ed7175, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6d717b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ad717f, SUC_lexicon_tags_8 },
    { 0x84ad717f, SUC_lexicon_tags_9 },
    { 0x97cd7180, SUC_lexicon_tags_15 },
    { 0x5dad7182, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x628d7185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd7189, SUC_lexicon_tags_1 },
    { 0xcc8d718a, SUC_lexicon_tags_37 },
    { 0xc80d718b, SUC_lexicon_tags_11 },
    { 0x41cd718c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x476d718e, SUC_lexicon_tags_43 },
    { 0x6e0d718f, SUC_lexicon_tags_79 },
    { 0x322d7190, SUC_lexicon_tags_25 },
    { 0x32ed7191, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2cd7193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcd7199, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x778d719c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ed719f, SUC_lexicon_tags_15 },
    { 0x9d6d71a0, SUC_lexicon_tags_1 },
    { 0xa36d719f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x404d71a3, SUC_lexicon_tags_1 },
    { 0x74ed71a4, SUC_lexicon_tags_47 },
    { 0xdd8d71a3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2d71a9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x46ed71ab, SUC_lexicon_tags_27 },
    { 0xd2d71ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d71b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd18d71b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ed71b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8d71bb, SUC_lexicon_tags_12 },
    { 0xf06d71bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784d71c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa28d71c4, SUC_lexicon_tags_11 },
    { 0x9f2d71c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa80d71c7, SUC_lexicon_tags_12 },
    { 0x4a2d71c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x674d71ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4d71d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x692d71d2, SUC_lexicon_tags_47 },
    { 0xe0ad71d3, SUC_lexicon_tags_23 },
    { 0xe26d71d2, SUC_lexicon_tags_14 },
    { 0xab0d71d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba4d71d7, SUC_lexicon_tags_7 },
    { 0xf7ed71d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c4d71da, SUC_lexicon_tags_1187 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594d71e4, SUC_lexicon_tags_0 },
    { 0x966d71e5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4d71e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4d71ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2d71f0, SUC_lexicon_tags_10 },
    { 0xcf0d71f1, SUC_lexicon_tags_19 },
    { 0x88ed71f2, SUC_lexicon_tags_11 },
    { 0x998d71f3, SUC_lexicon_tags_2 },
    { 0x8e4d71f4, SUC_lexicon_tags_60 },
    { 0xea8d71f4, SUC_lexicon_tags_37 },
    { 0x60d71f1, SUC_lexicon_tags_7 },
    { 0xb7ed71f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6d71fb, SUC_lexicon_tags_0 },
    { 0xecad71fc, SUC_lexicon_tags_8 },
    { 0x5f0d71fb, SUC_lexicon_tags_3 },
    { 0x4fad71fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x842d7200, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d7204, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f6d7206, SUC_lexicon_tags_39 },
    { 0x780d7207, SUC_lexicon_tags_25 },
    { 0xbb8d7208, SUC_lexicon_tags_138 },
    { 0xb2ed7206, SUC_lexicon_tags_34 },
    { 0x85ad7207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120d7212, SUC_lexicon_tags_27 },
    { 0xe6d7213, SUC_lexicon_tags_11 },
    { 0x778d7212, SUC_lexicon_tags_37 },
    { 0xaccd7214, SUC_lexicon_tags_2 },
    { 0xb0cd7215, SUC_lexicon_tags_7 },
    { 0x5a8d7216, SUC_lexicon_tags_11 },
    { 0xbf8d7218, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bed721b, SUC_lexicon_tags_2 },
    { 0x22cd721c, SUC_lexicon_tags_38 },
    { 0xf5ed721d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb3ed721f, SUC_lexicon_tags_2 },
    { 0xdf0d721f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782d7226, SUC_lexicon_tags_784 },
    { 0x85ed7226, SUC_lexicon_tags_26 },
    { 0xdd4d7228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958d722c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184d7230, SUC_lexicon_tags_17 },
    { 0x410d7231, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18d7236, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x366d7239, SUC_lexicon_tags_2 },
    { 0xf20d723a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x38ad723c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x796d723f, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x292d7241, SUC_lexicon_tags_0 },
    { 0x70ad7242, SUC_lexicon_tags_17 },
    { 0x89cd7241, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe8d7245, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34d7249, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcaad724b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe72d724d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7cd724f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d7254, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd22d7256, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194d725b, SUC_lexicon_tags_25 },
    { 0x490d725c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802d7267, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xecad7269, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0d726d, SUC_lexicon_tags_38 },
    { 0xb86d726d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cd7272, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xebad7274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafad7276, SUC_lexicon_tags_29 },
    { 0xe6d7276, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe70d7279, SUC_lexicon_tags_9 },
    { 0xd2d7279, SUC_lexicon_tags_31 },
    { 0x430d727b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4d727e, SUC_lexicon_tags_35 },
    { 0x20d727e, SUC_lexicon_tags_17 },
    { 0x4d2d7280, SUC_lexicon_tags_21 },
    { 0x874d7281, SUC_lexicon_tags_87 },
    { 0x866d7281, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cd7285, SUC_lexicon_tags_81 },
    { 0x18d7285, SUC_lexicon_tags_1 },
    { 0x4e2d7287, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6d728d, SUC_lexicon_tags_12 },
    { 0xa30d728e, SUC_lexicon_tags_1 },
    { 0xfad728f, SUC_lexicon_tags_17 },
    { 0x1d0d7290, SUC_lexicon_tags_25 },
    { 0xa8ad7290, SUC_lexicon_tags_55 },
    { 0x1aad7292, SUC_lexicon_tags_0 },
    { 0x8dad7292, SUC_lexicon_tags_15 },
    { 0x6f6d7293, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x972d7296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdad729d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x942d729f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x578d72a1, SUC_lexicon_tags_31 },
    { 0xae6d72a1, SUC_lexicon_tags_40 },
    { 0x51ad72a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcd72a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe08d72a9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2f4d72ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x118d72ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x940d72af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe2d72b1, SUC_lexicon_tags_45 },
    { 0xa82d72b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ed72b7, SUC_lexicon_tags_7 },
    { 0x95ed72b7, SUC_lexicon_tags_1004 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcd72bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cd72be, SUC_lexicon_tags_17 },
    { 0xfe2d72be, SUC_lexicon_tags_17 },
    { 0x71ed72c0, SUC_lexicon_tags_6 },
    { 0xed72bf, SUC_lexicon_tags_160 },
    { 0x10ad72c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cd72c5, SUC_lexicon_tags_37 },
    { 0xb8ed72c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a2d72c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2d72cf, SUC_lexicon_tags_37 },
    { 0x37cd72cf, SUC_lexicon_tags_31 },
    { 0x4fcd72d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f8d72d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ed72d6, SUC_lexicon_tags_37 },
    { 0xf7cd72d7, SUC_lexicon_tags_12 },
    { 0x184d72d8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x484d72db, SUC_lexicon_tags_0 },
    { 0x172d72dc, SUC_lexicon_tags_12 },
    { 0x1bcd72dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4d72e2, SUC_lexicon_tags_7 },
    { 0xdacd72e3, SUC_lexicon_tags_27 },
    { 0x43ed72e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x780d72e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57ed72e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d72f2, SUC_lexicon_tags_7 },
    { 0x804d72f3, SUC_lexicon_tags_6 },
    { 0x372d72f2, SUC_lexicon_tags_27 },
    { 0x9f6d72f5, SUC_lexicon_tags_60 },
    { 0x346d72f5, SUC_lexicon_tags_2 },
    { 0x766d72f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8bcd72f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f4d72fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fad7300, SUC_lexicon_tags_37 },
    { 0x560d7301, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x428d7304, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1a8d7306, SUC_lexicon_tags_15 },
    { 0xffcd7307, SUC_lexicon_tags_1 },
    { 0x5ecd7308, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ead730f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x2bad7311, SUC_lexicon_tags_12 },
    { 0xaaad7312, SUC_lexicon_tags_60 },
    { 0xa80d7313, SUC_lexicon_tags_33 },
    { 0x448d7312, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16d7317, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d731a, SUC_lexicon_tags_17 },
    { 0xdcd731a, SUC_lexicon_tags_43 },
    { 0x53ad731a, SUC_lexicon_tags_25 },
    { 0x682d731d, SUC_lexicon_tags_17 },
    { 0x41ad731e, SUC_lexicon_tags_15 },
    { 0x2ced731f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79cd7327, SUC_lexicon_tags_11 },
    { 0x914d7328, SUC_lexicon_tags_14 },
    { 0x10cd7329, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x622d732c, SUC_lexicon_tags_2 },
    { 0xabad732c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42d7332, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe56d7334, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0d7337, SUC_lexicon_tags_45 },
    { 0xbb2d7338, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70d733b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26d7342, SUC_lexicon_tags_34 },
    { 0xdad7343, SUC_lexicon_tags_55 },
    { 0xcf8d7344, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6d7347, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x95ad7349, SUC_lexicon_tags_15 },
    { 0xdf8d734a, SUC_lexicon_tags_11 },
    { 0x612d734b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354d7353, SUC_lexicon_tags_28 },
    { 0xfa6d7354, SUC_lexicon_tags_15 },
    { 0xb28d7353, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ed735c, SUC_lexicon_tags_87 },
    { 0xe16d735d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x734d7360, SUC_lexicon_tags_19 },
    { 0x6a6d7361, SUC_lexicon_tags_38 },
    { 0xfcad7362, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50ed7364, SUC_lexicon_tags_7 },
    { 0xff4d7365, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4d7369, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1eed736b, SUC_lexicon_tags_25 },
    { 0x880d736c, SUC_lexicon_tags_15 },
    { 0x760d736c, SUC_lexicon_tags_45 },
    { 0xf12d736e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2d7372, SUC_lexicon_tags_1 },
    { 0xb30d7373, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d7379, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x286d737c, SUC_lexicon_tags_1 },
    { 0x10cd737c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x91ed737f, SUC_lexicon_tags_15 },
    { 0x678d7380, SUC_lexicon_tags_17 },
    { 0xa3ad7381, SUC_lexicon_tags_2 },
    { 0x69ed7380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ed7386, SUC_lexicon_tags_108 },
    { 0x762d7386, SUC_lexicon_tags_62 },
    { 0x5e8d7388, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6d738b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfded738d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa8d738f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x156d7391, SUC_lexicon_tags_28 },
    { 0x8fed7392, SUC_lexicon_tags_37 },
    { 0x360d7393, SUC_lexicon_tags_45 },
    { 0xf30d7393, SUC_lexicon_tags_55 },
    { 0x144d7392, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0d7398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acd739b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf90d739d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64ed739f, SUC_lexicon_tags_32 },
    { 0xbb0d73a0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ced73ad, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54d73b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0d73b3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cd73b7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8d73bb, SUC_lexicon_tags_37 },
    { 0x4b4d73bc, SUC_lexicon_tags_37 },
    { 0x196d73bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x582d73c0, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x1ecd73c2, SUC_lexicon_tags_49 },
    { 0xee2d73c2, SUC_lexicon_tags_2 },
    { 0x66cd73c4, SUC_lexicon_tags_11 },
    { 0x502d73c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218d73cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ad73cf, SUC_lexicon_tags_11 },
    { 0xdbad73d0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2a0d73d2, SUC_lexicon_tags_17 },
    { 0xcd0d73d3, SUC_lexicon_tags_15 },
    { 0x176d73d4, SUC_lexicon_tags_28 },
    { 0xfcd73d5, SUC_lexicon_tags_1 },
    { 0xee8d73d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2d73e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12d73eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ded73f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaced73f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ad73f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4ed73fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x90ed73fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ead7406, SUC_lexicon_tags_25 },
    { 0xe18d7406, SUC_lexicon_tags_28 },
    { 0x1e2d7408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48d740b, SUC_lexicon_tags_12 },
    { 0x9e4d740c, SUC_lexicon_tags_17 },
    { 0xc5ed740d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbead7411, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cd7414, SUC_lexicon_tags_48 },
    { 0xa9cd7414, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44d741d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcad7420, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6d7423, SUC_lexicon_tags_15 },
    { 0xf0d7424, SUC_lexicon_tags_4 },
    { 0x4ccd7424, SUC_lexicon_tags_5 },
    { 0x42d7426, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x684d7428, SUC_lexicon_tags_7 },
    { 0x4d7428, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf0ed742b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xedad742e, SUC_lexicon_tags_11 },
    { 0xc4ed742e, SUC_lexicon_tags_23 },
    { 0x294d7430, SUC_lexicon_tags_14 },
    { 0xd2ed7431, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ed7434, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cad7438, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x158d743a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ad743e, SUC_lexicon_tags_15 },
    { 0xda8d743f, SUC_lexicon_tags_10 },
    { 0xe4ed743f, SUC_lexicon_tags_37 },
    { 0x356d7441, SUC_lexicon_tags_25 },
    { 0xbfad7441, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb46d7444, SUC_lexicon_tags_37 },
    { 0xb2ed7445, SUC_lexicon_tags_2 },
    { 0x37cd7446, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2acd7448, SUC_lexicon_tags_17 },
    { 0x678d7449, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898d7451, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ed7454, SUC_lexicon_tags_16 },
    { 0x4f4d7455, SUC_lexicon_tags_577 },
    { 0xaaad7455, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf6d7458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ad745d, SUC_lexicon_tags_32 },
    { 0x4f6d745d, SUC_lexicon_tags_66 },
    { 0xfe6d745d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ad7462, SUC_lexicon_tags_7 },
    { 0x40ed7462, SUC_lexicon_tags_4 },
    { 0x57ad7464, SUC_lexicon_tags_14 },
    { 0xe82d7462, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ed7468, SUC_lexicon_tags_11 },
    { 0x4f4d7469, SUC_lexicon_tags_47 },
    { 0xb70d7469, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2d746e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x234d7471, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcd7474, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d7478, SUC_lexicon_tags_27 },
    { 0xf56d7479, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60ad747b, SUC_lexicon_tags_79 },
    { 0xa1cd747c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44d7481, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b2d7483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x116d7486, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18d748c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ad748e, SUC_lexicon_tags_27 },
    { 0x8b2d748f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe08d7491, SUC_lexicon_tags_91 },
    { 0xf06d7492, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0d7495, SUC_lexicon_tags_547 },
    { 0x502d7496, SUC_lexicon_tags_48 },
    { 0x112d7496, SUC_lexicon_tags_17 },
    { 0x24ed7498, SUC_lexicon_tags_8 },
    { 0x3e2d7497, SUC_lexicon_tags_11 },
    { 0x6bcd749a, SUC_lexicon_tags_151 },
    { 0x8fed7499, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ed749e, SUC_lexicon_tags_47 },
    { 0x98ed749e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xbaad74a1, SUC_lexicon_tags_2 },
    { 0x54cd74a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x406d74a4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0d74a8, SUC_lexicon_tags_66 },
    { 0x310d74a9, SUC_lexicon_tags_37 },
    { 0x612d74aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30d74ad, SUC_lexicon_tags_23 },
    { 0x156d74ae, SUC_lexicon_tags_16 },
    { 0xd54d74ae, SUC_lexicon_tags_60 },
    { 0x6d74b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ad74b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8d74b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2d74c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922d74c8, SUC_lexicon_tags_16 },
    { 0x4e0d74c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d74d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d74d5, SUC_lexicon_tags_101 },
    { 0x706d74d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ad74d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c6d74db, SUC_lexicon_tags_12 },
    { 0x1dcd74dc, SUC_lexicon_tags_2 },
    { 0x336d74dd, SUC_lexicon_tags_2 },
    { 0x206d74de, SUC_lexicon_tags_22 },
    { 0xc5cd74df, SUC_lexicon_tags_17 },
    { 0xeed74e0, SUC_lexicon_tags_27 },
    { 0xbf6d74e1, SUC_lexicon_tags_17 },
    { 0x9d0d74e2, SUC_lexicon_tags_7 },
    { 0xe70d74e2, SUC_lexicon_tags_17 },
    { 0x8eed74dd, SUC_lexicon_tags_65 },
    { 0xb3cd74e4, SUC_lexicon_tags_63 },
    { 0xbd6d74e6, SUC_lexicon_tags_33 },
    { 0xaad74e7, SUC_lexicon_tags_7 },
    { 0x5b4d74e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8d74eb, SUC_lexicon_tags_17 },
    { 0x912d74eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x830d74ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d74f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ad74fa, SUC_lexicon_tags_45 },
    { 0x2c4d74fb, SUC_lexicon_tags_17 },
    { 0xd70d74fc, SUC_lexicon_tags_1 },
    { 0x1ed74fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69ed74ff, SUC_lexicon_tags_12 },
    { 0xb6cd7500, SUC_lexicon_tags_63 },
    { 0x46d7501, SUC_lexicon_tags_0 },
    { 0x8b8d74ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x956d7505, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x728d7507, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad4d7509, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x410d750b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ad750f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78d7514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8ad7516, SUC_lexicon_tags_37 },
    { 0x44ad7517, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8d751a, SUC_lexicon_tags_60 },
    { 0xe9ad751b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40cd751d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8d7520, SUC_lexicon_tags_11 },
    { 0x30ed7520, SUC_lexicon_tags_37 },
    { 0x6eed7522, SUC_lexicon_tags_15 },
    { 0x582d7523, SUC_lexicon_tags_12 },
    { 0xff6d7524, SUC_lexicon_tags_11 },
    { 0xe14d7522, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x128d7527, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8d752b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d752e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ad7531, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24d7538, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x416d753a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952d7540, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ed7543, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cd7547, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x492d7549, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb06d754b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc4d754d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ad7552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d8d7554, SUC_lexicon_tags_19 },
    { 0x394d7555, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ed7559, SUC_lexicon_tags_48 },
    { 0x638d755a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ed755d, SUC_lexicon_tags_68 },
    { 0xfbcd755d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4d7561, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4d756d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x288d756f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x260d7571, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x66d7573, SUC_lexicon_tags_10 },
    { 0x2d0d7574, SUC_lexicon_tags_390 },
    { 0x81ad7575, SUC_lexicon_tags_7 },
    { 0xa0ed7575, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cd7579, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad757d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ed7582, SUC_lexicon_tags_2 },
    { 0xbf6d7582, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ad7589, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb4d758b, SUC_lexicon_tags_29 },
    { 0x246d758c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cd7590, SUC_lexicon_tags_37 },
    { 0x5f8d7590, SUC_lexicon_tags_27 },
    { 0x1c0d7591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cd7599, SUC_lexicon_tags_19 },
    { 0x914d759a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292d75a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd74d75a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf24d75a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x126d75a7, SUC_lexicon_tags_83 },
    { 0x640d75a8, SUC_lexicon_tags_60 },
    { 0x9c2d75a9, SUC_lexicon_tags_6 },
    { 0x73cd75aa, SUC_lexicon_tags_28 },
    { 0x1f2d75ab, SUC_lexicon_tags_0 },
    { 0xd52d75a9, SUC_lexicon_tags_47 },
    { 0xcecd75ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2d75b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450d75b7, SUC_lexicon_tags_1 },
    { 0x832d75b8, SUC_lexicon_tags_47 },
    { 0x1a6d75b7, SUC_lexicon_tags_4 },
    { 0x56cd75ba, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x966d75bd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x900d75bf, SUC_lexicon_tags_68 },
    { 0x52ed75c0, SUC_lexicon_tags_14 },
    { 0xdeed75c1, SUC_lexicon_tags_14 },
    { 0x7f0d75c2, SUC_lexicon_tags_19 },
    { 0x6bad75c3, SUC_lexicon_tags_2 },
    { 0x33ed75c4, SUC_lexicon_tags_38 },
    { 0x5a8d75c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6d75c8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5d2d75ca, SUC_lexicon_tags_29 },
    { 0x35ed75cb, SUC_lexicon_tags_10 },
    { 0xb00d75cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ad75d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112d75d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32d75d9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140d75e0, SUC_lexicon_tags_7 },
    { 0xf86d75e1, SUC_lexicon_tags_23 },
    { 0x5cad75e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28d75e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ed75eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d75ef, SUC_lexicon_tags_25 },
    { 0x5ead75f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x19ad75f2, SUC_lexicon_tags_11 },
    { 0x6dcd75f3, SUC_lexicon_tags_63 },
    { 0x5ded75f4, SUC_lexicon_tags_37 },
    { 0x90cd75f5, SUC_lexicon_tags_33 },
    { 0xda8d75f3, SUC_lexicon_tags_2 },
    { 0x4b8d75f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274d75fe, SUC_lexicon_tags_17 },
    { 0xe86d75ff, SUC_lexicon_tags_17 },
    { 0x824d7600, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48d7602, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8d0d7604, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x648d7607, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cd760a, SUC_lexicon_tags_15 },
    { 0x4f0d760b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ad7610, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4d7614, SUC_lexicon_tags_12 },
    { 0x788d7615, SUC_lexicon_tags_25 },
    { 0x62cd7616, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x770d7619, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00d761c, SUC_lexicon_tags_10 },
    { 0xa82d761d, SUC_lexicon_tags_8 },
    { 0x5aed761e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd20d7620, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80d7626, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabad7631, SUC_lexicon_tags_155 },
    { 0x4aad7631, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40ad7634, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cd763a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6d763d, SUC_lexicon_tags_15 },
    { 0x2c6d763e, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x3e8d7640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ad7643, SUC_lexicon_tags_11 },
    { 0x690d7644, SUC_lexicon_tags_63 },
    { 0x65ed7643, SUC_lexicon_tags_25 },
    { 0xc06d7645, SUC_lexicon_tags_46 },
    { 0x42d7647, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36d764c, SUC_lexicon_tags_28 },
    { 0xad4d764d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322d7651, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccd765a, SUC_lexicon_tags_46 },
    { 0x99cd765a, SUC_lexicon_tags_7 },
    { 0xfbad765c, SUC_lexicon_tags_9 },
    { 0xba4d765d, SUC_lexicon_tags_4 },
    { 0x20ad765b, SUC_lexicon_tags_37 },
    { 0xc98d765d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1cd7661, SUC_lexicon_tags_1 },
    { 0x17ed7662, SUC_lexicon_tags_0 },
    { 0x1b2d7661, SUC_lexicon_tags_10 },
    { 0x9a0d7661, SUC_lexicon_tags_15 },
    { 0x7d2d7665, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0d766b, SUC_lexicon_tags_46 },
    { 0xdf6d766c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10d767b, SUC_lexicon_tags_1 },
    { 0x62cd767c, SUC_lexicon_tags_11 },
    { 0x492d767d, SUC_lexicon_tags_15 },
    { 0x89ad767e, SUC_lexicon_tags_7 },
    { 0x76cd767c, SUC_lexicon_tags_15 },
    { 0xf1cd767d, SUC_lexicon_tags_31 },
    { 0xc9cd767b, SUC_lexicon_tags_0 },
    { 0x2aad7682, SUC_lexicon_tags_11 },
    { 0x862d767d, SUC_lexicon_tags_25 },
    { 0xfead7684, SUC_lexicon_tags_71 },
    { 0x4ead7684, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ded768b, SUC_lexicon_tags_12 },
    { 0xa44d768c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x986d768e, SUC_lexicon_tags_2 },
    { 0xf50d768f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbad7692, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6d7696, SUC_lexicon_tags_25 },
    { 0xe40d7697, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0d769c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacd76a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x49ed76a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ad76a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452d76ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xedcd76ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x33ad76af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8d76b5, SUC_lexicon_tags_12 },
    { 0x18cd76b6, SUC_lexicon_tags_47 },
    { 0x8ead76b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62d76bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeded76bf, SUC_lexicon_tags_92 },
    { 0x34cd76bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcd76c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972d76c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ed76cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d76cf, SUC_lexicon_tags_30 },
    { 0x388d76d0, SUC_lexicon_tags_28 },
    { 0xe44d76d1, SUC_lexicon_tags_37 },
    { 0x854d76d2, SUC_lexicon_tags_32 },
    { 0x106d76cf, SUC_lexicon_tags_27 },
    { 0xefcd76d4, SUC_lexicon_tags_34 },
    { 0x6a2d76d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8d76d8, SUC_lexicon_tags_7 },
    { 0xd38d76d8, SUC_lexicon_tags_8 },
    { 0xe9ed76da, SUC_lexicon_tags_26 },
    { 0x9e0d76d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cd76e7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x340d76ea, SUC_lexicon_tags_47 },
    { 0x79ed76eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8d76ef, SUC_lexicon_tags_113 },
    { 0x4dcd76f0, SUC_lexicon_tags_60 },
    { 0xa74d76f1, SUC_lexicon_tags_12 },
    { 0x9b2d76f2, SUC_lexicon_tags_65 },
    { 0xcf0d76f3, SUC_lexicon_tags_37 },
    { 0x666d76f2, SUC_lexicon_tags_37 },
    { 0x3d8d76f5, SUC_lexicon_tags_12 },
    { 0xd60d76f6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x93ed76f8, SUC_lexicon_tags_7 },
    { 0x55ed76f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf74d76fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64ad76fd, SUC_lexicon_tags_31 },
    { 0x4ecd76fd, SUC_lexicon_tags_55 },
    { 0x2e0d76ff, SUC_lexicon_tags_10 },
    { 0xd58d7700, SUC_lexicon_tags_27 },
    { 0xfbed7701, SUC_lexicon_tags_37 },
    { 0xcf8d7701, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6d7705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54d7708, SUC_lexicon_tags_60 },
    { 0x6a2d7709, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64d770d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4d7712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00d7717, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8f8d7719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x242d771b, SUC_lexicon_tags_150 },
    { 0x5e4d771c, SUC_lexicon_tags_65 },
    { 0x7d2d771c, SUC_lexicon_tags_13 },
    { 0xddcd771c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdad7723, SUC_lexicon_tags_28 },
    { 0xfa2d7724, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x492d7727, SUC_lexicon_tags_0 },
    { 0xf66d7727, SUC_lexicon_tags_15 },
    { 0xe76d7729, SUC_lexicon_tags_60 },
    { 0xe86d7729, SUC_lexicon_tags_65 },
    { 0x56ad7729, SUC_lexicon_tags_9 },
    { 0x3e4d772c, SUC_lexicon_tags_17 },
    { 0x862d772b, SUC_lexicon_tags_33 },
    { 0xae6d772e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244d7737, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x622d773a, SUC_lexicon_tags_65 },
    { 0x26cd773b, SUC_lexicon_tags_12 },
    { 0x54cd773c, SUC_lexicon_tags_48 },
    { 0x1ead773b, SUC_lexicon_tags_14 },
    { 0xcbcd773e, SUC_lexicon_tags_17 },
    { 0x41ed773f, SUC_lexicon_tags_7 },
    { 0x1cd773e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd7743, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x790d7746, SUC_lexicon_tags_38 },
    { 0xe46d7747, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6d774f, SUC_lexicon_tags_34 },
    { 0xf66d774f, SUC_lexicon_tags_37 },
    { 0xc7ed7751, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18d7754, SUC_lexicon_tags_9 },
    { 0xc3ed7755, SUC_lexicon_tags_17 },
    { 0x8e2d7754, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa1ad7758, SUC_lexicon_tags_7 },
    { 0x694d7758, SUC_lexicon_tags_29 },
    { 0x890d775a, SUC_lexicon_tags_47 },
    { 0xbe0d775b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x728d775d, SUC_lexicon_tags_17 },
    { 0x4e0d775e, SUC_lexicon_tags_912 },
    { 0, NULL },
    { 0xcf4d7760, SUC_lexicon_tags_39 },
    { 0x904d7760, SUC_lexicon_tags_25 },
    { 0x340d7761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acd7766, SUC_lexicon_tags_15 },
    { 0xc52d7766, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x490d7769, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0d7774, SUC_lexicon_tags_15 },
    { 0xfa6d7774, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ad7778, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2d7781, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc70d7783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ecd7785, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa54d7787, SUC_lexicon_tags_23 },
    { 0xa60d7788, SUC_lexicon_tags_11 },
    { 0x97ed7788, SUC_lexicon_tags_11 },
    { 0xa82d778a, SUC_lexicon_tags_47 },
    { 0x4ccd778b, SUC_lexicon_tags_14 },
    { 0x25cd7787, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cad7790, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fed7792, SUC_lexicon_tags_37 },
    { 0x5e0d7793, SUC_lexicon_tags_27 },
    { 0xb80d7792, SUC_lexicon_tags_17 },
    { 0xb92d7795, SUC_lexicon_tags_12 },
    { 0xd40d7795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b6d7798, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa60d779a, SUC_lexicon_tags_31 },
    { 0x302d779a, SUC_lexicon_tags_12 },
    { 0x7ad779c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8d77a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf14d77a2, SUC_lexicon_tags_17 },
    { 0xed0d77a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ed77a6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcd77ab, SUC_lexicon_tags_7 },
    { 0x9e6d77ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684d77b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc18d77b2, SUC_lexicon_tags_60 },
    { 0x266d77b3, SUC_lexicon_tags_11 },
    { 0x280d77b3, SUC_lexicon_tags_45 },
    { 0xf80d77b5, SUC_lexicon_tags_9 },
    { 0x7f8d77b2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988d77ba, SUC_lexicon_tags_1 },
    { 0xb0ed77ba, SUC_lexicon_tags_37 },
    { 0x4aad77bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ed77c5, SUC_lexicon_tags_47 },
    { 0x658d77c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60d77d0, SUC_lexicon_tags_14 },
    { 0xe90d77d1, SUC_lexicon_tags_38 },
    { 0x164d77d2, SUC_lexicon_tags_33 },
    { 0x476d77d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bcd77d5, SUC_lexicon_tags_1 },
    { 0x16ad77d5, SUC_lexicon_tags_106 },
    { 0x2f2d77d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaad77da, SUC_lexicon_tags_32 },
    { 0xb00d77da, SUC_lexicon_tags_61 },
    { 0x53ad77db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2d77e0, SUC_lexicon_tags_27 },
    { 0x78ed77e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ccd77e3, SUC_lexicon_tags_17 },
    { 0x2d4d77e4, SUC_lexicon_tags_17 },
    { 0xcb6d77e3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf22d77e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106d77ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d77f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x794d77f3, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x6f0d77f5, SUC_lexicon_tags_12 },
    { 0xd0cd77f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ad77f9, SUC_lexicon_tags_2 },
    { 0xbcad77f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe7ad77fc, SUC_lexicon_tags_38 },
    { 0x47cd77fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x752d7800, SUC_lexicon_tags_9 },
    { 0xffad7801, SUC_lexicon_tags_17 },
    { 0x94ad7802, SUC_lexicon_tags_37 },
    { 0x8d2d7803, SUC_lexicon_tags_17 },
    { 0x1a4d7800, SUC_lexicon_tags_60 },
    { 0xb4d7805, SUC_lexicon_tags_512 },
    { 0x190d7800, SUC_lexicon_tags_49 },
    { 0x140d7802, SUC_lexicon_tags_17 },
    { 0x786d7808, SUC_lexicon_tags_2 },
    { 0x104d7808, SUC_lexicon_tags_15 },
    { 0xe3cd7809, SUC_lexicon_tags_38 },
    { 0x6e2d7808, SUC_lexicon_tags_11 },
    { 0xc5cd780c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d780f, SUC_lexicon_tags_12 },
    { 0x9d6d7810, SUC_lexicon_tags_37 },
    { 0x97cd7811, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42d781a, SUC_lexicon_tags_12 },
    { 0xeeed781a, SUC_lexicon_tags_44 },
    { 0x768d781c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbded7822, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ed7825, SUC_lexicon_tags_11 },
    { 0x95ed7826, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa34d7828, SUC_lexicon_tags_15 },
    { 0x538d7829, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafed7832, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x200d7834, SUC_lexicon_tags_93 },
    { 0xee0d7834, SUC_lexicon_tags_7 },
    { 0xe00d7836, SUC_lexicon_tags_9 },
    { 0x344d7837, SUC_lexicon_tags_12 },
    { 0xac2d7838, SUC_lexicon_tags_37 },
    { 0x4eed7839, SUC_lexicon_tags_33 },
    { 0xfc2d783a, SUC_lexicon_tags_55 },
    { 0x292d783b, SUC_lexicon_tags_11 },
    { 0x864d783c, SUC_lexicon_tags_14 },
    { 0xa7ed7835, SUC_lexicon_tags_45 },
    { 0xc2d783e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934d7847, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6d784e, SUC_lexicon_tags_17 },
    { 0xca8d784f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x84ad7851, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cd7857, SUC_lexicon_tags_8 },
    { 0xd1cd7858, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8c4d785a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x610d785c, SUC_lexicon_tags_12 },
    { 0xa4d785d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xedcd7860, SUC_lexicon_tags_30 },
    { 0xf14d7860, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcd786a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fad786c, SUC_lexicon_tags_29 },
    { 0x68cd786d, SUC_lexicon_tags_0 },
    { 0x3d2d786c, SUC_lexicon_tags_43 },
    { 0x4f4d786d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38d7872, SUC_lexicon_tags_27 },
    { 0xa74d7872, SUC_lexicon_tags_31 },
    { 0x2b8d7874, SUC_lexicon_tags_27 },
    { 0xfaad7873, SUC_lexicon_tags_138 },
    { 0x7c8d7876, SUC_lexicon_tags_11 },
    { 0xd5ed7877, SUC_lexicon_tags_37 },
    { 0xfe8d7874, SUC_lexicon_tags_28 },
    { 0xfb4d7872, SUC_lexicon_tags_47 },
    { 0x554d7874, SUC_lexicon_tags_79 },
    { 0xfbcd7873, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4d787e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548d7888, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdfed788a, SUC_lexicon_tags_11 },
    { 0xa2ed788b, SUC_lexicon_tags_45 },
    { 0x2f4d788b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44d7890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95ad7892, SUC_lexicon_tags_61 },
    { 0x7b6d7893, SUC_lexicon_tags_55 },
    { 0xf00d7893, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ed7898, SUC_lexicon_tags_17 },
    { 0xf94d7899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ed789e, SUC_lexicon_tags_37 },
    { 0xc36d789f, SUC_lexicon_tags_8 },
    { 0xf0ad78a0, SUC_lexicon_tags_33 },
    { 0x528d78a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ead78a3, SUC_lexicon_tags_25 },
    { 0xaaad78a4, SUC_lexicon_tags_12 },
    { 0xdf0d78a5, SUC_lexicon_tags_77 },
    { 0x22ed78a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6d78a9, SUC_lexicon_tags_2 },
    { 0xa30d78aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ced78ae, SUC_lexicon_tags_25 },
    { 0x4d8d78af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764d78b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684d78b9, SUC_lexicon_tags_48 },
    { 0xcd2d78ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38ed78bc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ad78c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2d78c7, SUC_lexicon_tags_55 },
    { 0xf90d78c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfaed78ca, SUC_lexicon_tags_1 },
    { 0x698d78ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622d78cf, SUC_lexicon_tags_15 },
    { 0x9bed78d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf9ad78d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538d78d8, SUC_lexicon_tags_45 },
    { 0x216d78d9, SUC_lexicon_tags_15 },
    { 0x886d78da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3ad78dc, SUC_lexicon_tags_47 },
    { 0xae4d78dd, SUC_lexicon_tags_38 },
    { 0x4d6d78dd, SUC_lexicon_tags_37 },
    { 0x416d78df, SUC_lexicon_tags_25 },
    { 0x2e0d78dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc86d78e2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d78e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72d78ef, SUC_lexicon_tags_9 },
    { 0x8a8d78f0, SUC_lexicon_tags_46 },
    { 0x16cd78f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76ad78f3, SUC_lexicon_tags_25 },
    { 0x13ed78f3, SUC_lexicon_tags_25 },
    { 0xe70d78f5, SUC_lexicon_tags_25 },
    { 0x288d78f6, SUC_lexicon_tags_4 },
    { 0x922d78f5, SUC_lexicon_tags_45 },
    { 0xf2d78f4, SUC_lexicon_tags_11 },
    { 0x890d78f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cd7906, SUC_lexicon_tags_25 },
    { 0x9ccd7907, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcd790d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cd7911, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb16d7913, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98d7916, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x344d7918, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x478d791a, SUC_lexicon_tags_7 },
    { 0xee8d791a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8d7920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53cd7922, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1bed7924, SUC_lexicon_tags_38 },
    { 0x744d7924, SUC_lexicon_tags_2 },
    { 0x1b2d7924, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4d792c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0d7931, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d7936, SUC_lexicon_tags_45 },
    { 0x416d7937, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcf8d7939, SUC_lexicon_tags_25 },
    { 0x55ad793a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x862d793c, SUC_lexicon_tags_15 },
    { 0x2a2d793d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cd7942, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ced7944, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4d7948, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aad794a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60d794f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70d7953, SUC_lexicon_tags_15 },
    { 0xd50d7954, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4d7958, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16d795b, SUC_lexicon_tags_38 },
    { 0xb34d795b, SUC_lexicon_tags_49 },
    { 0x72cd795d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ad7961, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x476d7964, SUC_lexicon_tags_29 },
    { 0x55ed7965, SUC_lexicon_tags_6 },
    { 0xf70d7965, SUC_lexicon_tags_15 },
    { 0x102d7967, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x680d7969, SUC_lexicon_tags_37 },
    { 0x1c4d7969, SUC_lexicon_tags_25 },
    { 0xc92d796b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cd7970, SUC_lexicon_tags_7 },
    { 0x99cd7970, SUC_lexicon_tags_17 },
    { 0xf32d7972, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cd7976, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46d797b, SUC_lexicon_tags_45 },
    { 0xdaad797b, SUC_lexicon_tags_262 },
    { 0xfb8d797d, SUC_lexicon_tags_37 },
    { 0x52d797d, SUC_lexicon_tags_21 },
    { 0x786d797f, SUC_lexicon_tags_12 },
    { 0xc80d797c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe82d7982, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cd7985, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100d798c, SUC_lexicon_tags_32 },
    { 0xc34d798c, SUC_lexicon_tags_7 },
    { 0x3a4d798e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbad7994, SUC_lexicon_tags_3 },
    { 0x2eed7994, SUC_lexicon_tags_43 },
    { 0xa7ed7996, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff6d7998, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54d799b, SUC_lexicon_tags_68 },
    { 0x840d799b, SUC_lexicon_tags_11 },
    { 0x37ad799d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cd79a3, SUC_lexicon_tags_45 },
    { 0xcf4d79a4, SUC_lexicon_tags_26 },
    { 0xf82d79a5, SUC_lexicon_tags_33 },
    { 0xb76d79a6, SUC_lexicon_tags_2 },
    { 0xd3cd79a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02d79aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e6d79ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cd79b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35cd79b5, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x354d79b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x552d79b9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x522d79bc, SUC_lexicon_tags_17 },
    { 0xa12d79bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36d79c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd79ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0d79d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ad79d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ed79db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8d79de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3aed79e0, SUC_lexicon_tags_28 },
    { 0xadcd79e1, SUC_lexicon_tags_17 },
    { 0xcf4d79e2, SUC_lexicon_tags_11 },
    { 0x4c4d79e1, SUC_lexicon_tags_37 },
    { 0x784d79e4, SUC_lexicon_tags_60 },
    { 0xaecd79e1, SUC_lexicon_tags_12 },
    { 0xc44d79e6, SUC_lexicon_tags_37 },
    { 0x88d79e6, SUC_lexicon_tags_206 },
    { 0x5fcd79e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70d79eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x458d79ed, SUC_lexicon_tags_60 },
    { 0x898d79ee, SUC_lexicon_tags_17 },
    { 0x666d79ef, SUC_lexicon_tags_25 },
    { 0xc0d79ed, SUC_lexicon_tags_68 },
    { 0x50d79f1, SUC_lexicon_tags_11 },
    { 0x110d79f1, SUC_lexicon_tags_8 },
    { 0xf84d79f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4d79f6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6d79f9, SUC_lexicon_tags_87 },
    { 0xdbcd79fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x666d79fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28ed79fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fed7a01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0d7a06, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cd7a0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dad7a0f, SUC_lexicon_tags_25 },
    { 0x1f4d7a10, SUC_lexicon_tags_30 },
    { 0x2bad7a11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ed7a15, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x71ed7a17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x760d7a19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccd7a1d, SUC_lexicon_tags_43 },
    { 0xde2d7a1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf8d7a20, SUC_lexicon_tags_47 },
    { 0xae4d7a21, SUC_lexicon_tags_37 },
    { 0xf38d7a21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0d7a25, SUC_lexicon_tags_87 },
    { 0x178d7a26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9aed7a28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa22d7a2a, SUC_lexicon_tags_12 },
    { 0xe8ad7a2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42d7a2e, SUC_lexicon_tags_66 },
    { 0x9d4d7a2e, SUC_lexicon_tags_11 },
    { 0xda0d7a2e, SUC_lexicon_tags_43 },
    { 0xc18d7a2f, SUC_lexicon_tags_1 },
    { 0xe46d7a32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffed7a3a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828d7a3e, SUC_lexicon_tags_17 },
    { 0xfb6d7a3f, SUC_lexicon_tags_7 },
    { 0x966d7a40, SUC_lexicon_tags_66 },
    { 0xfe2d7a41, SUC_lexicon_tags_17 },
    { 0xa9ad7a42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbad7a44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4d7a47, SUC_lexicon_tags_11 },
    { 0x8a8d7a48, SUC_lexicon_tags_9 },
    { 0xc42d7a49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcd7a4e, SUC_lexicon_tags_23 },
    { 0xaacd7a4f, SUC_lexicon_tags_37 },
    { 0xdacd7a4f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x894d7a52, SUC_lexicon_tags_77 },
    { 0x4f8d7a53, SUC_lexicon_tags_9 },
    { 0xded7a52, SUC_lexicon_tags_148 },
    { 0xa94d7a52, SUC_lexicon_tags_2 },
    { 0xb90d7a56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x832d7a58, SUC_lexicon_tags_272 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208d7a61, SUC_lexicon_tags_17 },
    { 0xf50d7a62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7f6d7a64, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x754d7a66, SUC_lexicon_tags_14 },
    { 0xa78d7a67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x172d7a6a, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x92cd7a6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66d7a72, SUC_lexicon_tags_11 },
    { 0x48cd7a73, SUC_lexicon_tags_15 },
    { 0x76ad7a74, SUC_lexicon_tags_37 },
    { 0x5b0d7a75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16d7a78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe54d7a7a, SUC_lexicon_tags_60 },
    { 0x1dad7a7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x214d7a7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4d7a87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83ed7a89, SUC_lexicon_tags_55 },
    { 0x12ed7a8a, SUC_lexicon_tags_27 },
    { 0x266d7a89, SUC_lexicon_tags_37 },
    { 0xe2ed7a8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46d7a8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xea8d7a91, SUC_lexicon_tags_103 },
    { 0x6fed7a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38cd7a9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x754d7a9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814d7aa1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256d7aa6, SUC_lexicon_tags_37 },
    { 0x814d7aa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cd7ab1, SUC_lexicon_tags_31 },
    { 0xcc0d7ab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bad7ab4, SUC_lexicon_tags_60 },
    { 0x7d0d7ab4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ed7aba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x66d7abc, SUC_lexicon_tags_17 },
    { 0x43cd7abc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bed7ac2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ad7ac9, SUC_lexicon_tags_2 },
    { 0x9ecd7aca, SUC_lexicon_tags_27 },
    { 0x7c8d7acb, SUC_lexicon_tags_7 },
    { 0x5f8d7acc, SUC_lexicon_tags_11 },
    { 0x60ad7acc, SUC_lexicon_tags_17 },
    { 0x356d7acd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecd7ad1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ead7ad4, SUC_lexicon_tags_27 },
    { 0x536d7ad4, SUC_lexicon_tags_33 },
    { 0xd08d7ad6, SUC_lexicon_tags_43 },
    { 0x15ed7ad7, SUC_lexicon_tags_2 },
    { 0x604d7ad8, SUC_lexicon_tags_64 },
    { 0x50ad7ad7, SUC_lexicon_tags_43 },
    { 0x996d7ada, SUC_lexicon_tags_37 },
    { 0xb68d7ada, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ead7ade, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8d7aef, SUC_lexicon_tags_26 },
    { 0x48ad7af0, SUC_lexicon_tags_7 },
    { 0xa44d7af0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0ed7af3, SUC_lexicon_tags_11 },
    { 0x7a2d7af3, SUC_lexicon_tags_12 },
    { 0x5ced7af5, SUC_lexicon_tags_11 },
    { 0x14ed7af6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad7afc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6d7b01, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e6d7b03, SUC_lexicon_tags_12 },
    { 0xf4d7b04, SUC_lexicon_tags_26 },
    { 0x3a4d7b04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7b09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb52d7b0b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x3ead7b0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54d7b10, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb1ed7b12, SUC_lexicon_tags_39 },
    { 0x97ad7b13, SUC_lexicon_tags_7 },
    { 0x6c0d7b14, SUC_lexicon_tags_95 },
    { 0xfc8d7b13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c2d7b17, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ad7b1c, SUC_lexicon_tags_25 },
    { 0x822d7b1d, SUC_lexicon_tags_65 },
    { 0x7b4d7b1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x992d7b21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0d7b25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x256d7b28, SUC_lexicon_tags_45 },
    { 0xe54d7b29, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82d7b2d, SUC_lexicon_tags_8 },
    { 0x97cd7b2d, SUC_lexicon_tags_14 },
    { 0xe16d7b2f, SUC_lexicon_tags_79 },
    { 0x6c8d7b2e, SUC_lexicon_tags_27 },
    { 0x6f6d7b31, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7f4d7b33, SUC_lexicon_tags_1 },
    { 0x7c2d7b33, SUC_lexicon_tags_12 },
    { 0xf1ed7b35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4d7b38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862d7b3d, SUC_lexicon_tags_19 },
    { 0x8c6d7b3e, SUC_lexicon_tags_15 },
    { 0x7f6d7b3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47ad7b41, SUC_lexicon_tags_117 },
    { 0x3fad7b42, SUC_lexicon_tags_25 },
    { 0x19ed7b43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x992d7b46, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86d7b49, SUC_lexicon_tags_17 },
    { 0x4ead7b4a, SUC_lexicon_tags_7 },
    { 0x98d7b4b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70d7b4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98d7b51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xadad7b54, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496d7b59, SUC_lexicon_tags_133 },
    { 0xaf2d7b5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe56d7b5c, SUC_lexicon_tags_7 },
    { 0xca6d7b5c, SUC_lexicon_tags_49 },
    { 0x7ed7b5e, SUC_lexicon_tags_12 },
    { 0x362d7b5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ead7b62, SUC_lexicon_tags_23 },
    { 0x5d6d7b63, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3bed7b65, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb14d7b67, SUC_lexicon_tags_37 },
    { 0xcc2d7b68, SUC_lexicon_tags_108 },
    { 0xcd8d7b67, SUC_lexicon_tags_2 },
    { 0x6bed7b6a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xb02d7b6c, SUC_lexicon_tags_12 },
    { 0xa4ed7b6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aed7b75, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb8d7b77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xef8d7b7a, SUC_lexicon_tags_0 },
    { 0x45ad7b7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ad7b7e, SUC_lexicon_tags_17 },
    { 0xb76d7b7f, SUC_lexicon_tags_23 },
    { 0x5ccd7b7f, SUC_lexicon_tags_25 },
    { 0x3ead7b7f, SUC_lexicon_tags_47 },
    { 0xd5cd7b82, SUC_lexicon_tags_38 },
    { 0xc20d7b82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92d7b87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ead7b89, SUC_lexicon_tags_17 },
    { 0xe88d7b8a, SUC_lexicon_tags_0 },
    { 0xd30d7b89, SUC_lexicon_tags_14 },
    { 0xf2ad7b8a, SUC_lexicon_tags_28 },
    { 0x4f0d7b8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48cd7b8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcd7b96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x230d7b98, SUC_lexicon_tags_43 },
    { 0x6fad7b98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x704d7b9b, SUC_lexicon_tags_35 },
    { 0xec0d7b9c, SUC_lexicon_tags_11 },
    { 0x4b2d7b9d, SUC_lexicon_tags_47 },
    { 0x712d7b9e, SUC_lexicon_tags_29 },
    { 0xa44d7b9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6d7ba2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14d7ba5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384d7ba9, SUC_lexicon_tags_12 },
    { 0x978d7ba9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6d7bad, SUC_lexicon_tags_16 },
    { 0x6e0d7bad, SUC_lexicon_tags_38 },
    { 0xf3ed7bae, SUC_lexicon_tags_7 },
    { 0x146d7bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0d7bb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x708d7bb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8d7bba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2ed7bbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5ad7bbe, SUC_lexicon_tags_23 },
    { 0x68ed7bbf, SUC_lexicon_tags_0 },
    { 0x144d7bc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeacd7bc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68d7bc5, SUC_lexicon_tags_12 },
    { 0xf7ad7bc5, SUC_lexicon_tags_15 },
    { 0xed8d7bc7, SUC_lexicon_tags_6 },
    { 0xd52d7bc8, SUC_lexicon_tags_33 },
    { 0xa6d7bc8, SUC_lexicon_tags_11 },
    { 0x8ced7bca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466d7bd1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6d7bd7, SUC_lexicon_tags_1 },
    { 0xf0ed7bd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ad7bdb, SUC_lexicon_tags_50 },
    { 0xeb4d7bdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf8d7bde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x66cd7be1, SUC_lexicon_tags_26 },
    { 0x946d7be1, SUC_lexicon_tags_68 },
    { 0xc7ad7be3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccd7be9, SUC_lexicon_tags_25 },
    { 0x1dcd7bea, SUC_lexicon_tags_38 },
    { 0x468d7beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x120d7bed, SUC_lexicon_tags_30 },
    { 0xe8ad7bee, SUC_lexicon_tags_1 },
    { 0x6acd7bef, SUC_lexicon_tags_39 },
    { 0xd50d7bef, SUC_lexicon_tags_68 },
    { 0x4d4d7bef, SUC_lexicon_tags_6 },
    { 0x37ed7bee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4d7bfb, SUC_lexicon_tags_15 },
    { 0x4b6d7bfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0d7bff, SUC_lexicon_tags_2 },
    { 0x7cd7bff, SUC_lexicon_tags_33 },
    { 0xdd2d7bff, SUC_lexicon_tags_16 },
    { 0x9d6d7bff, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0xf2ed7c04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0d7c07, SUC_lexicon_tags_137 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6d7c0a, SUC_lexicon_tags_31 },
    { 0xe1cd7c0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4d7c12, SUC_lexicon_tags_15 },
    { 0x10d7c13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ced7c16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cd7c1a, SUC_lexicon_tags_11 },
    { 0xc24d7c1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644d7c1f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb7cd7c21, SUC_lexicon_tags_11 },
    { 0x4b6d7c22, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446d7c2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x806d7c2f, SUC_lexicon_tags_43 },
    { 0x366d7c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44d7c33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4d7c37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ed7c3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0cd7c3c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ed7c3f, SUC_lexicon_tags_81 },
    { 0x762d7c40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290d7c45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cd7c49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8d7c4f, SUC_lexicon_tags_31 },
    { 0xf36d7c50, SUC_lexicon_tags_10 },
    { 0x344d7c4f, SUC_lexicon_tags_2 },
    { 0xf3cd7c4f, SUC_lexicon_tags_505 },
    { 0x3f2d7c53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdaad7c55, SUC_lexicon_tags_23 },
    { 0x2b6d7c55, SUC_lexicon_tags_1 },
    { 0x8d6d7c55, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592d7c5c, SUC_lexicon_tags_17 },
    { 0xf62d7c5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12d7c61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cd7c70, SUC_lexicon_tags_423 },
    { 0xc92d7c71, SUC_lexicon_tags_27 },
    { 0xa90d7c72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92d7c78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed7c7b, SUC_lexicon_tags_5 },
    { 0xc96d7c7b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d7c7f, SUC_lexicon_tags_37 },
    { 0x694d7c7f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cd7c84, SUC_lexicon_tags_33 },
    { 0x876d7c84, SUC_lexicon_tags_7 },
    { 0xa4cd7c86, SUC_lexicon_tags_29 },
    { 0x6dad7c87, SUC_lexicon_tags_9 },
    { 0xbccd7c87, SUC_lexicon_tags_37 },
    { 0x9e8d7c89, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4d7c8c, SUC_lexicon_tags_47 },
    { 0xa74d7c8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ed7c90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4d7c95, SUC_lexicon_tags_27 },
    { 0x256d7c96, SUC_lexicon_tags_49 },
    { 0x6f4d7c95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x560d7c99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cd7c9d, SUC_lexicon_tags_60 },
    { 0x5d4d7c9e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcd7ca1, SUC_lexicon_tags_31 },
    { 0x23ed7ca1, SUC_lexicon_tags_19 },
    { 0x4e6d7ca3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ad7ca6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcd7ca9, SUC_lexicon_tags_21 },
    { 0x67ad7ca9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ed7caf, SUC_lexicon_tags_3 },
    { 0x410d7caf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a4d7cb2, SUC_lexicon_tags_37 },
    { 0xafcd7cb3, SUC_lexicon_tags_1 },
    { 0xa2cd7cb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bad7cb7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0d7cbb, SUC_lexicon_tags_8 },
    { 0xfe0d7cbc, SUC_lexicon_tags_60 },
    { 0xa34d7cbd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cd7cc4, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bed7cc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cd7ccb, SUC_lexicon_tags_16 },
    { 0x4f8d7ccb, SUC_lexicon_tags_15 },
    { 0x3d4d7ccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6d7cd0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cd7cd3, SUC_lexicon_tags_26 },
    { 0x1ded7cd4, SUC_lexicon_tags_45 },
    { 0x64cd7cd4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d7cd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54d7cdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8d7ce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x250d7ce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d7ce6, SUC_lexicon_tags_1 },
    { 0xa26d7ce7, SUC_lexicon_tags_5 },
    { 0x9a4d7ce7, SUC_lexicon_tags_2 },
    { 0xad0d7ce9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aed7cf6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2b6d7cf8, SUC_lexicon_tags_32 },
    { 0x55ed7cf9, SUC_lexicon_tags_1 },
    { 0x80cd7cfa, SUC_lexicon_tags_29 },
    { 0x270d7cfb, SUC_lexicon_tags_68 },
    { 0xbbed7cfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6d7cff, SUC_lexicon_tags_7 },
    { 0xcc6d7d00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaed7d08, SUC_lexicon_tags_25 },
    { 0xcdcd7d09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68d7d0c, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xa5ad7d0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cd7d12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc54d7d14, SUC_lexicon_tags_25 },
    { 0xa60d7d15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x332d7d17, SUC_lexicon_tags_30 },
    { 0x57ad7d17, SUC_lexicon_tags_11 },
    { 0x9b8d7d17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d7d1f, SUC_lexicon_tags_37 },
    { 0x654d7d20, SUC_lexicon_tags_1 },
    { 0xea0d7d21, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acd7d25, SUC_lexicon_tags_45 },
    { 0x848d7d25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x144d7d28, SUC_lexicon_tags_87 },
    { 0x2fcd7d29, SUC_lexicon_tags_37 },
    { 0x98ad7d29, SUC_lexicon_tags_32 },
    { 0xd9ad7d2a, SUC_lexicon_tags_12 },
    { 0xac4d7d2c, SUC_lexicon_tags_1 },
    { 0xb4cd7d2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0d7d31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x208d7d34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7cad7d36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe8d7d38, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x95ad7d3a, SUC_lexicon_tags_15 },
    { 0xfa4d7d3b, SUC_lexicon_tags_17 },
    { 0x31ad7d3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ad7d43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2d7d47, SUC_lexicon_tags_191 },
    { 0xeded7d48, SUC_lexicon_tags_1 },
    { 0x756d7d49, SUC_lexicon_tags_46 },
    { 0xb16d7d48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498d7d52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc18d7d54, SUC_lexicon_tags_38 },
    { 0x9a8d7d55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x844d7d57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ad7d5b, SUC_lexicon_tags_57 },
    { 0xd20d7d5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x436d7d5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x822d7d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408d7d66, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ed7d69, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x750d7d6c, SUC_lexicon_tags_60 },
    { 0x23ed7d6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656d7d71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ed7d75, SUC_lexicon_tags_7 },
    { 0x722d7d76, SUC_lexicon_tags_11 },
    { 0xf1cd7d77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6d7d7a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x480d7d7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15ed7d7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x138d7d82, SUC_lexicon_tags_14 },
    { 0xc8ed7d82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c2d7d85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544d7d8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874d7d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580d7d94, SUC_lexicon_tags_85 },
    { 0x49ad7d95, SUC_lexicon_tags_1 },
    { 0xd8ed7d95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ed7d99, SUC_lexicon_tags_15 },
    { 0xd74d7d9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8d7d9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ed7da1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf50d7da3, SUC_lexicon_tags_6 },
    { 0x78cd7da4, SUC_lexicon_tags_12 },
    { 0x42cd7da4, SUC_lexicon_tags_47 },
    { 0xea6d7da3, SUC_lexicon_tags_12 },
    { 0x1fcd7da6, SUC_lexicon_tags_28 },
    { 0xd20d7da8, SUC_lexicon_tags_30 },
    { 0xf98d7da6, SUC_lexicon_tags_40 },
    { 0x868d7daa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf56d7dac, SUC_lexicon_tags_12 },
    { 0x3d2d7dad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cd7db1, SUC_lexicon_tags_43 },
    { 0xeccd7db1, SUC_lexicon_tags_11 },
    { 0x864d7db1, SUC_lexicon_tags_108 },
    { 0x79cd7db1, SUC_lexicon_tags_29 },
    { 0xd02d7db3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x390d7db7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x218d7db9, SUC_lexicon_tags_79 },
    { 0x2a4d7dba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2d7dbe, SUC_lexicon_tags_33 },
    { 0xdb6d7dbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38d7dc9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ad7dcc, SUC_lexicon_tags_0 },
    { 0xe14d7dcd, SUC_lexicon_tags_21 },
    { 0xd34d7dce, SUC_lexicon_tags_45 },
    { 0x138d7dcf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x23ad7dd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xead7dd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x472d7dd6, SUC_lexicon_tags_11 },
    { 0xa30d7dd7, SUC_lexicon_tags_39 },
    { 0xa20d7dd8, SUC_lexicon_tags_87 },
    { 0x798d7dd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360d7dde, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fad7de1, SUC_lexicon_tags_15 },
    { 0x6ced7de1, SUC_lexicon_tags_33 },
    { 0xbb6d7de3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcd7de8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56d7deb, SUC_lexicon_tags_31 },
    { 0x4b4d7dec, SUC_lexicon_tags_27 },
    { 0x1e2d7deb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26ad7def, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8f0d7df1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x566d7df3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x296d7df5, SUC_lexicon_tags_2 },
    { 0xd18d7df6, SUC_lexicon_tags_1 },
    { 0x9ded7df7, SUC_lexicon_tags_27 },
    { 0xc12d7df5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b2d7dfa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8d7e01, SUC_lexicon_tags_87 },
    { 0x74d7e02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cd7e08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc44d7e0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ad7e12, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00d7e18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8d7e1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbded7e21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd2d7e23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb4d7e25, SUC_lexicon_tags_138 },
    { 0x58d7e26, SUC_lexicon_tags_12 },
    { 0x856d7e27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb60d7e29, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6d7e2c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x48ad7e2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40ed7e30, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x20ad7e32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4d7e36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33cd7e38, SUC_lexicon_tags_21 },
    { 0x6b0d7e39, SUC_lexicon_tags_11 },
    { 0xbd6d7e38, SUC_lexicon_tags_46 },
    { 0x612d7e3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2d7e41, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xcb8d7e43, SUC_lexicon_tags_12 },
    { 0xb6ad7e44, SUC_lexicon_tags_47 },
    { 0xbded7e45, SUC_lexicon_tags_15 },
    { 0xcc4d7e45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf9cd7e48, SUC_lexicon_tags_17 },
    { 0xc16d7e49, SUC_lexicon_tags_37 },
    { 0x58d7e49, SUC_lexicon_tags_33 },
    { 0xbe6d7e4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34d7e50, SUC_lexicon_tags_39 },
    { 0x2bed7e50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cd7e55, SUC_lexicon_tags_45 },
    { 0xe4cd7e55, SUC_lexicon_tags_65 },
    { 0x9f6d7e56, SUC_lexicon_tags_37 },
    { 0x81cd7e58, SUC_lexicon_tags_47 },
    { 0x61ad7e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370d7e5e, SUC_lexicon_tags_2 },
    { 0x256d7e5f, SUC_lexicon_tags_68 },
    { 0x3fed7e60, SUC_lexicon_tags_15 },
    { 0x92ed7e5f, SUC_lexicon_tags_17 },
    { 0xa3ed7e5f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cd7e66, SUC_lexicon_tags_25 },
    { 0x2a0d7e66, SUC_lexicon_tags_49 },
    { 0xbd8d7e67, SUC_lexicon_tags_25 },
    { 0xedcd7e69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4d7e6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234d7e74, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60d7e77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760d7e7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6d7e7f, SUC_lexicon_tags_17 },
    { 0x65cd7e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acd7e89, SUC_lexicon_tags_6 },
    { 0x2e6d7e8a, SUC_lexicon_tags_45 },
    { 0x524d7e8a, SUC_lexicon_tags_8 },
    { 0x84ed7e89, SUC_lexicon_tags_11 },
    { 0xfded7e8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9ed7e8f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ded7e91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60cd7e93, SUC_lexicon_tags_11 },
    { 0x3ed7e93, SUC_lexicon_tags_25 },
    { 0xf90d7e93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28d7e99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc18d7e9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52d7ea0, SUC_lexicon_tags_45 },
    { 0xa60d7ea1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ad7ea9, SUC_lexicon_tags_116 },
    { 0xd62d7ea9, SUC_lexicon_tags_15 },
    { 0x22ad7eab, SUC_lexicon_tags_37 },
    { 0x89cd7eac, SUC_lexicon_tags_80 },
    { 0xa24d7ead, SUC_lexicon_tags_24 },
    { 0x6dad7eae, SUC_lexicon_tags_6 },
    { 0x3e0d7ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c6d7eb1, SUC_lexicon_tags_17 },
    { 0x1a4d7eb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x856d7eb4, SUC_lexicon_tags_1 },
    { 0xe62d7eb5, SUC_lexicon_tags_70 },
    { 0x4cad7eb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecd7eba, SUC_lexicon_tags_35 },
    { 0x282d7eba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ad7ec2, SUC_lexicon_tags_11 },
    { 0x8bcd7ec3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240d7ec8, SUC_lexicon_tags_17 },
    { 0xec8d7ec8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d7ed3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6d7ed6, SUC_lexicon_tags_6 },
    { 0xdf6d7ed7, SUC_lexicon_tags_12 },
    { 0xcecd7ed6, SUC_lexicon_tags_23 },
    { 0x5a8d7ed9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6d7ee0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x48ed7ee2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cd7eea, SUC_lexicon_tags_16 },
    { 0xc72d7eeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4d7ef1, SUC_lexicon_tags_37 },
    { 0x558d7ef2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb7ed7ef4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc4d7ef6, SUC_lexicon_tags_25 },
    { 0x886d7ef7, SUC_lexicon_tags_0 },
    { 0x58ed7ef7, SUC_lexicon_tags_63 },
    { 0xb1ad7ef9, SUC_lexicon_tags_17 },
    { 0x618d7efa, SUC_lexicon_tags_15 },
    { 0x91cd7efb, SUC_lexicon_tags_87 },
    { 0xa96d7ef7, SUC_lexicon_tags_60 },
    { 0x936d7efa, SUC_lexicon_tags_55 },
    { 0x938d7efe, SUC_lexicon_tags_82 },
    { 0xd74d7ef9, SUC_lexicon_tags_31 },
    { 0x6cd7efe, SUC_lexicon_tags_67 },
    { 0x4ead7f01, SUC_lexicon_tags_48 },
    { 0x1a2d7efd, SUC_lexicon_tags_17 },
    { 0x918d7f03, SUC_lexicon_tags_12 },
    { 0xde2d7f04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x854d7f07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ead7f0a, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564d7f0e, SUC_lexicon_tags_60 },
    { 0x752d7f0e, SUC_lexicon_tags_17 },
    { 0x332d7f10, SUC_lexicon_tags_17 },
    { 0x29ad7f11, SUC_lexicon_tags_46 },
    { 0x780d7f12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa26d7f14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360d7f18, SUC_lexicon_tags_23 },
    { 0xd80d7f19, SUC_lexicon_tags_9 },
    { 0xeb2d7f1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cd7f23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x730d7f25, SUC_lexicon_tags_10 },
    { 0x3bad7f26, SUC_lexicon_tags_65 },
    { 0x2f0d7f27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422d7f31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12d7f3a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x36ad7f3c, SUC_lexicon_tags_29 },
    { 0x61cd7f3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x464d7f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ed7f45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8d7f49, SUC_lexicon_tags_60 },
    { 0x6fad7f4a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18d7f4d, SUC_lexicon_tags_68 },
    { 0xef6d7f4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfed7f51, SUC_lexicon_tags_14 },
    { 0xb2ed7f52, SUC_lexicon_tags_1 },
    { 0xf30d7f53, SUC_lexicon_tags_8 },
    { 0x63cd7f51, SUC_lexicon_tags_9 },
    { 0x4e4d7f55, SUC_lexicon_tags_37 },
    { 0x882d7f55, SUC_lexicon_tags_37 },
    { 0xef4d7f57, SUC_lexicon_tags_11 },
    { 0x7acd7f57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x832d7f5b, SUC_lexicon_tags_6 },
    { 0xa8ad7f5b, SUC_lexicon_tags_37 },
    { 0xe08d7f5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ccd7f5f, SUC_lexicon_tags_37 },
    { 0x442d7f60, SUC_lexicon_tags_2 },
    { 0x708d7f5f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd52d7f63, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280d7f68, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8d7f6b, SUC_lexicon_tags_49 },
    { 0x7f0d7f6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ad7f6f, SUC_lexicon_tags_60 },
    { 0xe7ad7f70, SUC_lexicon_tags_47 },
    { 0x156d7f70, SUC_lexicon_tags_17 },
    { 0x718d7f72, SUC_lexicon_tags_25 },
    { 0x7ced7f72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd62d7f75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b6d7f77, SUC_lexicon_tags_47 },
    { 0x90ad7f78, SUC_lexicon_tags_33 },
    { 0xe88d7f79, SUC_lexicon_tags_7 },
    { 0xa14d7f77, SUC_lexicon_tags_77 },
    { 0xd6d7f7b, SUC_lexicon_tags_2 },
    { 0xb60d7f79, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x572d7f7f, SUC_lexicon_tags_1 },
    { 0xab8d7f80, SUC_lexicon_tags_8 },
    { 0x65ad7f81, SUC_lexicon_tags_12 },
    { 0x5c4d7f82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72d7f88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x146d7f8a, SUC_lexicon_tags_12 },
    { 0x2ded7f8a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d7f8e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6d7f93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bad7f97, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc90d7f99, SUC_lexicon_tags_12 },
    { 0xba6d7f9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ed7f9e, SUC_lexicon_tags_17 },
    { 0xaecd7f9f, SUC_lexicon_tags_2 },
    { 0xafad7fa0, SUC_lexicon_tags_4 },
    { 0xf10d7fa1, SUC_lexicon_tags_1 },
    { 0xb40d7fa2, SUC_lexicon_tags_17 },
    { 0xc94d7fa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ded7fa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84d7fab, SUC_lexicon_tags_25 },
    { 0xc4d7fab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcded7fb5, SUC_lexicon_tags_37 },
    { 0x8f6d7fb5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x72d7fb8, SUC_lexicon_tags_12 },
    { 0x874d7fb9, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0xb9ad7fbb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x50cd7fbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ed7fc0, SUC_lexicon_tags_32 },
    { 0xa4d7fc0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0d7fc4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ed7fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0d7fcc, SUC_lexicon_tags_38 },
    { 0x780d7fcc, SUC_lexicon_tags_46 },
    { 0x80ed7fce, SUC_lexicon_tags_10 },
    { 0x738d7fcf, SUC_lexicon_tags_10 },
    { 0xdfcd7fce, SUC_lexicon_tags_7 },
    { 0xeecd7fd1, SUC_lexicon_tags_11 },
    { 0x11ad7fd2, SUC_lexicon_tags_38 },
    { 0x940d7fd3, SUC_lexicon_tags_60 },
    { 0x7f6d7fd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x358d7fd6, SUC_lexicon_tags_1 },
    { 0xda0d7fd7, SUC_lexicon_tags_37 },
    { 0x64ad7fd8, SUC_lexicon_tags_33 },
    { 0x2a8d7fd9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdbcd7fdb, SUC_lexicon_tags_37 },
    { 0x89ad7fdc, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d7fe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefad7fe3, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cd7fec, SUC_lexicon_tags_70 },
    { 0xe1ad7fed, SUC_lexicon_tags_15 },
    { 0x7e4d7fed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ad7ff1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xccad7ff4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x45ed7ff6, SUC_lexicon_tags_17 },
    { 0xcf6d7ff7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78d7ffc, SUC_lexicon_tags_65 },
    { 0xde6d7ffd, SUC_lexicon_tags_1 },
    { 0xfb8d7ffe, SUC_lexicon_tags_63 },
    { 0x4d2d7ffd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f6d8001, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9c4d8003, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x3e4d8005, SUC_lexicon_tags_32 },
    { 0x282d8006, SUC_lexicon_tags_45 },
    { 0xe74d8006, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadad800d, SUC_lexicon_tags_8 },
    { 0xd3cd800e, SUC_lexicon_tags_33 },
    { 0x1ded800f, SUC_lexicon_tags_6 },
    { 0x666d8010, SUC_lexicon_tags_57 },
    { 0xeeed8011, SUC_lexicon_tags_60 },
    { 0x7e2d8012, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0d8015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8d801a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30d801c, SUC_lexicon_tags_49 },
    { 0xcfed801c, SUC_lexicon_tags_7 },
    { 0xafad801d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650d8023, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x90d8025, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced802b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746d802f, SUC_lexicon_tags_29 },
    { 0x29cd802f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2d8034, SUC_lexicon_tags_16 },
    { 0x96ad8035, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf0d8037, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcecd8039, SUC_lexicon_tags_11 },
    { 0xa50d803a, SUC_lexicon_tags_15 },
    { 0x3f4d803b, SUC_lexicon_tags_329 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cd8040, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x780d8042, SUC_lexicon_tags_11 },
    { 0x946d8043, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7ad8045, SUC_lexicon_tags_12 },
    { 0xa8ad8046, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4d804a, SUC_lexicon_tags_2 },
    { 0x73ad804a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cd804f, SUC_lexicon_tags_26 },
    { 0x52ed8050, SUC_lexicon_tags_30 },
    { 0xf92d8051, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476d8059, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4a4d805b, SUC_lexicon_tags_14 },
    { 0x952d805c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ed8060, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ad8064, SUC_lexicon_tags_2 },
    { 0xd1ed8065, SUC_lexicon_tags_17 },
    { 0x9c8d8066, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0, NULL },
    { 0x404d8069, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ad806e, SUC_lexicon_tags_63 },
    { 0xb78d806f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x498d8071, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44cd8073, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf82d8075, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf4d8077, SUC_lexicon_tags_31 },
    { 0x90d8078, SUC_lexicon_tags_27 },
    { 0x9f2d8079, SUC_lexicon_tags_2 },
    { 0x8c8d8079, SUC_lexicon_tags_7 },
    { 0x8e4d8079, SUC_lexicon_tags_113 },
    { 0x264d807b, SUC_lexicon_tags_31 },
    { 0x4f2d807d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x788d8080, SUC_lexicon_tags_16 },
    { 0x968d8080, SUC_lexicon_tags_31 },
    { 0xaa0d8082, SUC_lexicon_tags_29 },
    { 0xdc2d8083, SUC_lexicon_tags_15 },
    { 0x85ed8084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfced808c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6d8090, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x798d8093, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x408d8095, SUC_lexicon_tags_19 },
    { 0x598d8095, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x612d8098, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa28d809a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbd0d809c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1e8d809e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282d80a5, SUC_lexicon_tags_15 },
    { 0x274d80a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40ed80a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x220d80aa, SUC_lexicon_tags_11 },
    { 0x87ed80ab, SUC_lexicon_tags_31 },
    { 0xbc0d80ac, SUC_lexicon_tags_7 },
    { 0x7ccd80ad, SUC_lexicon_tags_55 },
    { 0x260d80ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10d80b2, SUC_lexicon_tags_31 },
    { 0x45ed80b3, SUC_lexicon_tags_10 },
    { 0x7a4d80b4, SUC_lexicon_tags_2 },
    { 0xcd2d80b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x860d80b7, SUC_lexicon_tags_14 },
    { 0x974d80b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcd80bb, SUC_lexicon_tags_7 },
    { 0x12ed80bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcd80c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860d80c5, SUC_lexicon_tags_11 },
    { 0x9e8d80c6, SUC_lexicon_tags_33 },
    { 0xb02d80c7, SUC_lexicon_tags_31 },
    { 0xe86d80c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x436d80ca, SUC_lexicon_tags_4 },
    { 0xfad80ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6d80cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8d80d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0d80db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56d80df, SUC_lexicon_tags_25 },
    { 0xacd80e0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2d80e4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54d80e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ed80f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cd80f5, SUC_lexicon_tags_17 },
    { 0x1e0d80f6, SUC_lexicon_tags_25 },
    { 0x224d80f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372d80fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd8100, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcd8104, SUC_lexicon_tags_15 },
    { 0xc5ad8104, SUC_lexicon_tags_17 },
    { 0xf8ad8106, SUC_lexicon_tags_7 },
    { 0x146d8107, SUC_lexicon_tags_25 },
    { 0x20ed8108, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4d810e, SUC_lexicon_tags_23 },
    { 0x14d810e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x902d8111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9dcd8113, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ed8116, SUC_lexicon_tags_1 },
    { 0x816d8117, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d811e, SUC_lexicon_tags_0 },
    { 0xe38d811f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ed8122, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256d8126, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78d812a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28d8134, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x434d8136, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xef6d8138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ed813d, SUC_lexicon_tags_2 },
    { 0xeb6d813e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882d8144, SUC_lexicon_tags_33 },
    { 0x4dad8145, SUC_lexicon_tags_27 },
    { 0x4ecd8146, SUC_lexicon_tags_21 },
    { 0xb94d8145, SUC_lexicon_tags_16 },
    { 0x87cd8148, SUC_lexicon_tags_17 },
    { 0xdfcd8148, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cd8150, SUC_lexicon_tags_25 },
    { 0x750d8151, SUC_lexicon_tags_47 },
    { 0x346d8151, SUC_lexicon_tags_28 },
    { 0xe6d8153, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cd815a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x644d815d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8ed815f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x214d8161, SUC_lexicon_tags_17 },
    { 0x488d8162, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe86d8164, SUC_lexicon_tags_15 },
    { 0x2d8d8165, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362d8169, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdccd816b, SUC_lexicon_tags_315 },
    { 0x176d816c, SUC_lexicon_tags_82 },
    { 0xae2d816d, SUC_lexicon_tags_26 },
    { 0x3b6d816e, SUC_lexicon_tags_26 },
    { 0x90d816f, SUC_lexicon_tags_31 },
    { 0x1d2d816c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d8176, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfad8178, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4cad817a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x310d817d, SUC_lexicon_tags_5 },
    { 0xb7ed817d, SUC_lexicon_tags_13 },
    { 0x8f0d817e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b2d8181, SUC_lexicon_tags_7 },
    { 0xde8d8182, SUC_lexicon_tags_27 },
    { 0x464d8182, SUC_lexicon_tags_12 },
    { 0x5c0d8182, SUC_lexicon_tags_17 },
    { 0xe4ad8185, SUC_lexicon_tags_31 },
    { 0x268d8186, SUC_lexicon_tags_27 },
    { 0x730d8187, SUC_lexicon_tags_53 },
    { 0x76cd8186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f2d818a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cd818e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcced8192, SUC_lexicon_tags_49 },
    { 0x2b8d8193, SUC_lexicon_tags_31 },
    { 0xeead8194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560d8199, SUC_lexicon_tags_1 },
    { 0xa96d819a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aad819f, SUC_lexicon_tags_43 },
    { 0x892d81a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x406d81a3, SUC_lexicon_tags_23 },
    { 0x3a6d81a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cd81a8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x420d81ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd81ae, SUC_lexicon_tags_34 },
    { 0x59ed81af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276d81b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dad81bc, SUC_lexicon_tags_7 },
    { 0x58ed81bd, SUC_lexicon_tags_34 },
    { 0xeecd81bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe30d81c0, SUC_lexicon_tags_47 },
    { 0x390d81c1, SUC_lexicon_tags_37 },
    { 0xc08d81c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6d81c6, SUC_lexicon_tags_37 },
    { 0xe12d81c7, SUC_lexicon_tags_17 },
    { 0xf78d81c6, SUC_lexicon_tags_39 },
    { 0xf1ed81c9, SUC_lexicon_tags_172 },
    { 0x9e8d81ca, SUC_lexicon_tags_11 },
    { 0xb7ad81cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ed81d1, SUC_lexicon_tags_39 },
    { 0x94ed81d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0d81d7, SUC_lexicon_tags_81 },
    { 0xbcd81d7, SUC_lexicon_tags_1 },
    { 0xf96d81d9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4d81e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228d81ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650d81f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7bad81f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacd81f8, SUC_lexicon_tags_17 },
    { 0x722d81f9, SUC_lexicon_tags_23 },
    { 0x78d81f9, SUC_lexicon_tags_1 },
    { 0xd42d81f9, SUC_lexicon_tags_25 },
    { 0xc8ad81fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x196d81fe, SUC_lexicon_tags_46 },
    { 0x75ed81ff, SUC_lexicon_tags_25 },
    { 0x628d8200, SUC_lexicon_tags_12 },
    { 0x38ed81ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172d8208, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d0d820a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab6d820c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8d8210, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cd8213, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0d8217, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8d821a, SUC_lexicon_tags_25 },
    { 0xe14d821a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x678d821e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8d8222, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80d8228, SUC_lexicon_tags_14 },
    { 0x324d8229, SUC_lexicon_tags_7 },
    { 0x7e8d822a, SUC_lexicon_tags_43 },
    { 0xf2ed8229, SUC_lexicon_tags_26 },
    { 0x684d822c, SUC_lexicon_tags_17 },
    { 0x9f8d822d, SUC_lexicon_tags_13 },
    { 0xd7cd822d, SUC_lexicon_tags_45 },
    { 0xf2ed822e, SUC_lexicon_tags_11 },
    { 0xe8d8230, SUC_lexicon_tags_17 },
    { 0xaed8230, SUC_lexicon_tags_37 },
    { 0x870d8230, SUC_lexicon_tags_17 },
    { 0x3cd8233, SUC_lexicon_tags_3 },
    { 0x2c6d8234, SUC_lexicon_tags_17 },
    { 0xd00d8233, SUC_lexicon_tags_26 },
    { 0x2b2d822f, SUC_lexicon_tags_19 },
    { 0xa08d8237, SUC_lexicon_tags_37 },
    { 0xcaed822a, SUC_lexicon_tags_4 },
    { 0x450d8239, SUC_lexicon_tags_25 },
    { 0x744d8231, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcd8244, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcccd8246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecd824a, SUC_lexicon_tags_68 },
    { 0xd824b, SUC_lexicon_tags_12 },
    { 0xf0cd824c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ed8253, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x772d8255, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510d8259, SUC_lexicon_tags_113 },
    { 0x30cd825a, SUC_lexicon_tags_44 },
    { 0x426d825a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74ed825d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0d8262, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x862d8265, SUC_lexicon_tags_12 },
    { 0xe3cd8266, SUC_lexicon_tags_37 },
    { 0x46cd8267, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18d826d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf68d826f, SUC_lexicon_tags_37 },
    { 0x208d8270, SUC_lexicon_tags_28 },
    { 0xec0d8271, SUC_lexicon_tags_7 },
    { 0x7e6d8272, SUC_lexicon_tags_38 },
    { 0x2c8d8273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ed8278, SUC_lexicon_tags_7 },
    { 0x57cd8278, SUC_lexicon_tags_7 },
    { 0xed6d827a, SUC_lexicon_tags_1 },
    { 0x92cd827a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0d827e, SUC_lexicon_tags_15 },
    { 0xb24d827f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcd8282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcd8286, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe0d8288, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed828e, SUC_lexicon_tags_37 },
    { 0x194d828f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ed8292, SUC_lexicon_tags_37 },
    { 0x392d8293, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d8298, SUC_lexicon_tags_43 },
    { 0xb60d8299, SUC_lexicon_tags_37 },
    { 0xa3cd829a, SUC_lexicon_tags_43 },
    { 0xa9ed829a, SUC_lexicon_tags_62 },
    { 0xdf4d829c, SUC_lexicon_tags_1 },
    { 0xb48d829a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14d82a5, SUC_lexicon_tags_7 },
    { 0x2c8d82a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cd82aa, SUC_lexicon_tags_1 },
    { 0x112d82aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cd82ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x932d82b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b4d82b2, SUC_lexicon_tags_57 },
    { 0x7acd82b3, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x4d4d82b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62cd82b7, SUC_lexicon_tags_26 },
    { 0x434d82b8, SUC_lexicon_tags_17 },
    { 0x964d82b9, SUC_lexicon_tags_46 },
    { 0x3bad82b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0d82c1, SUC_lexicon_tags_38 },
    { 0x9a6d82c2, SUC_lexicon_tags_37 },
    { 0x494d82c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00d82ca, SUC_lexicon_tags_28 },
    { 0xbecd82ca, SUC_lexicon_tags_37 },
    { 0xa44d82cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x714d82cf, SUC_lexicon_tags_37 },
    { 0x72ed82d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828d82d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80d82da, SUC_lexicon_tags_23 },
    { 0xbc4d82db, SUC_lexicon_tags_13 },
    { 0x3a8d82dc, SUC_lexicon_tags_17 },
    { 0x56ed82dc, SUC_lexicon_tags_56 },
    { 0x974d82de, SUC_lexicon_tags_207 },
    { 0xa4d82dd, SUC_lexicon_tags_56 },
    { 0xa4cd82dc, SUC_lexicon_tags_25 },
    { 0xb66d82dc, SUC_lexicon_tags_1 },
    { 0xbd6d82dd, SUC_lexicon_tags_11 },
    { 0x716d82de, SUC_lexicon_tags_31 },
    { 0xe0cd82df, SUC_lexicon_tags_36 },
    { 0x29ed82df, SUC_lexicon_tags_21 },
    { 0x5bcd82da, SUC_lexicon_tags_38 },
    { 0x9eed82e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x248d82ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ad82f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46cd82f2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xa58d82f4, SUC_lexicon_tags_37 },
    { 0x46cd82f4, SUC_lexicon_tags_11 },
    { 0x4a6d82f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x398d82f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8ead82fa, SUC_lexicon_tags_46 },
    { 0x7dad82fb, SUC_lexicon_tags_37 },
    { 0xc48d82fc, SUC_lexicon_tags_11 },
    { 0x47ed82fc, SUC_lexicon_tags_48 },
    { 0xe90d82fa, SUC_lexicon_tags_7 },
    { 0x4e8d82ff, SUC_lexicon_tags_25 },
    { 0x8f2d82fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94d8305, SUC_lexicon_tags_30 },
    { 0x8e8d8306, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6d8309, SUC_lexicon_tags_12 },
    { 0xe5cd830a, SUC_lexicon_tags_37 },
    { 0x940d8309, SUC_lexicon_tags_1 },
    { 0x4d2d830b, SUC_lexicon_tags_33 },
    { 0xbb6d830d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500d8316, SUC_lexicon_tags_11 },
    { 0x516d8317, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cd831b, SUC_lexicon_tags_32 },
    { 0x12cd831c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126d8320, SUC_lexicon_tags_60 },
    { 0x7c4d8321, SUC_lexicon_tags_37 },
    { 0x728d8320, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950d8326, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22ad8328, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0d832b, SUC_lexicon_tags_66 },
    { 0xff0d832b, SUC_lexicon_tags_17 },
    { 0x788d832d, SUC_lexicon_tags_63 },
    { 0x20ed832d, SUC_lexicon_tags_27 },
    { 0x1bcd832d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0d8332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd8336, SUC_lexicon_tags_17 },
    { 0x65cd8337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860d833b, SUC_lexicon_tags_31 },
    { 0x9f0d833b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x996d833e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x754d8341, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x804d8343, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fad8345, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8d8349, SUC_lexicon_tags_30 },
    { 0x2b8d8349, SUC_lexicon_tags_23 },
    { 0x3fcd834b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30d8355, SUC_lexicon_tags_37 },
    { 0x88d8356, SUC_lexicon_tags_26 },
    { 0x2d0d8357, SUC_lexicon_tags_66 },
    { 0x9fed8357, SUC_lexicon_tags_17 },
    { 0x6e2d8357, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fed835d, SUC_lexicon_tags_47 },
    { 0x2c8d835e, SUC_lexicon_tags_23 },
    { 0x5a0d835d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2d8363, SUC_lexicon_tags_33 },
    { 0x43cd8364, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xd8d8366, SUC_lexicon_tags_33 },
    { 0xf8ed8366, SUC_lexicon_tags_10 },
    { 0xcaad8368, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfad836d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6d8370, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22ed8372, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6d8377, SUC_lexicon_tags_12 },
    { 0xc4cd8378, SUC_lexicon_tags_182 },
    { 0x650d8379, SUC_lexicon_tags_2 },
    { 0xcbad8379, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cd8380, SUC_lexicon_tags_49 },
    { 0xe6ed8381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706d8386, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72d8389, SUC_lexicon_tags_8 },
    { 0x520d838a, SUC_lexicon_tags_54 },
    { 0x388d838b, SUC_lexicon_tags_12 },
    { 0xac0d838c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4d8391, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ad8394, SUC_lexicon_tags_14 },
    { 0xe34d8394, SUC_lexicon_tags_27 },
    { 0xc42d8396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x102d8399, SUC_lexicon_tags_17 },
    { 0x206d839a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa62d839c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ad839f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x238d83a1, SUC_lexicon_tags_11 },
    { 0xcad83a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x404d83a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2d83aa, SUC_lexicon_tags_15 },
    { 0xf88d83ab, SUC_lexicon_tags_7 },
    { 0xa54d83ac, SUC_lexicon_tags_8 },
    { 0x692d83ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe98d83af, SUC_lexicon_tags_34 },
    { 0x578d83af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd64d83b2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x946d83b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60d83b9, SUC_lexicon_tags_15 },
    { 0x152d83ba, SUC_lexicon_tags_12 },
    { 0x824d83bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x898d83bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf18d83bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x422d83c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64d83ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd7ed83cc, SUC_lexicon_tags_11 },
    { 0xdf2d83cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bad83d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x490d83d6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xafad83d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78cd83da, SUC_lexicon_tags_17 },
    { 0x704d83db, SUC_lexicon_tags_15 },
    { 0x612d83dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4d83df, SUC_lexicon_tags_68 },
    { 0xa6ad83e0, SUC_lexicon_tags_47 },
    { 0xb92d83e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc76d83e3, SUC_lexicon_tags_45 },
    { 0xa4ad83e4, SUC_lexicon_tags_19 },
    { 0x1eed83e5, SUC_lexicon_tags_1 },
    { 0xad0d83e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ccd83e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74d83eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8d83ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0d83f2, SUC_lexicon_tags_15 },
    { 0xd68d83f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ed83f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ad83fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x520d83ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496d8407, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9e0d8409, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xccad840c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad8410, SUC_lexicon_tags_60 },
    { 0x190d8411, SUC_lexicon_tags_47 },
    { 0x818d8412, SUC_lexicon_tags_7 },
    { 0x6ead8411, SUC_lexicon_tags_45 },
    { 0xdaed8414, SUC_lexicon_tags_15 },
    { 0x6a0d8415, SUC_lexicon_tags_47 },
    { 0xd74d8416, SUC_lexicon_tags_15 },
    { 0x7eed8414, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1cad8419, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cd841e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262d8423, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1b0d8425, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbdad8427, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36d842c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x624d842e, SUC_lexicon_tags_31 },
    { 0xb16d842f, SUC_lexicon_tags_4 },
    { 0x570d8430, SUC_lexicon_tags_1 },
    { 0xc48d842e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0d8437, SUC_lexicon_tags_37 },
    { 0x854d8437, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908d843e, SUC_lexicon_tags_7 },
    { 0x37cd843e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2ad8441, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad8445, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x88cd8447, SUC_lexicon_tags_45 },
    { 0x4d4d8448, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10d844d, SUC_lexicon_tags_108 },
    { 0xcf4d844e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d8452, SUC_lexicon_tags_1 },
    { 0x57cd8453, SUC_lexicon_tags_8 },
    { 0xabad8454, SUC_lexicon_tags_25 },
    { 0x27cd8453, SUC_lexicon_tags_6 },
    { 0x960d8455, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282d845c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead8460, SUC_lexicon_tags_28 },
    { 0x1aad8460, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x910d8463, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd8ad8465, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ed8469, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x558d846b, SUC_lexicon_tags_11 },
    { 0xbd8d846c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734d8471, SUC_lexicon_tags_15 },
    { 0x7c4d8471, SUC_lexicon_tags_14 },
    { 0xa4cd8472, SUC_lexicon_tags_37 },
    { 0xf84d8474, SUC_lexicon_tags_87 },
    { 0x5f6d8475, SUC_lexicon_tags_17 },
    { 0x768d8472, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190d8485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2d8489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aed848b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd8492, SUC_lexicon_tags_37 },
    { 0xb78d8493, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72d8499, SUC_lexicon_tags_17 },
    { 0x5b2d849a, SUC_lexicon_tags_38 },
    { 0xd12d849a, SUC_lexicon_tags_31 },
    { 0x9c8d849b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58d84a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4d84a9, SUC_lexicon_tags_31 },
    { 0x1ecd84a9, SUC_lexicon_tags_49 },
    { 0x72ed84ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954d84b2, SUC_lexicon_tags_10 },
    { 0x92d84b2, SUC_lexicon_tags_0 },
    { 0x1d2d84b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa0d84b6, SUC_lexicon_tags_57 },
    { 0x204d84b6, SUC_lexicon_tags_12 },
    { 0x8f0d84b8, SUC_lexicon_tags_25 },
    { 0x388d84b8, SUC_lexicon_tags_7 },
    { 0xad2d84ba, SUC_lexicon_tags_2 },
    { 0xd34d84bb, SUC_lexicon_tags_6 },
    { 0x7b8d84bc, SUC_lexicon_tags_11 },
    { 0xbd6d84bc, SUC_lexicon_tags_70 },
    { 0xafad84bd, SUC_lexicon_tags_15 },
    { 0x18cd84b9, SUC_lexicon_tags_95 },
    { 0x9fcd84c0, SUC_lexicon_tags_25 },
    { 0xb66d84c1, SUC_lexicon_tags_12 },
    { 0x2c4d84c2, SUC_lexicon_tags_17 },
    { 0x680d84c3, SUC_lexicon_tags_11 },
    { 0x7acd84c2, SUC_lexicon_tags_47 },
    { 0xe42d84c5, SUC_lexicon_tags_17 },
    { 0x90ad84c6, SUC_lexicon_tags_19 },
    { 0x558d84c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42cd84cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0d84cf, SUC_lexicon_tags_1 },
    { 0x532d84d0, SUC_lexicon_tags_21 },
    { 0x142d84d1, SUC_lexicon_tags_25 },
    { 0x998d84d1, SUC_lexicon_tags_37 },
    { 0xc42d84d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320d84d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a4d84db, SUC_lexicon_tags_15 },
    { 0x6cad84dc, SUC_lexicon_tags_23 },
    { 0xd8cd84dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x33ed84df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xae0d84e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcd84e7, SUC_lexicon_tags_43 },
    { 0x632d84e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cd84f1, SUC_lexicon_tags_32 },
    { 0x90ad84f2, SUC_lexicon_tags_17 },
    { 0x1acd84f2, SUC_lexicon_tags_17 },
    { 0x778d84f3, SUC_lexicon_tags_37 },
    { 0x95ed84f5, SUC_lexicon_tags_25 },
    { 0x84cd84f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30d84f9, SUC_lexicon_tags_65 },
    { 0x8dcd84f9, SUC_lexicon_tags_12 },
    { 0x73ad84fb, SUC_lexicon_tags_47 },
    { 0x202d84fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ed8501, SUC_lexicon_tags_11 },
    { 0x85ed8502, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xc24d8504, SUC_lexicon_tags_8 },
    { 0xb40d8505, SUC_lexicon_tags_9 },
    { 0x31ad8506, SUC_lexicon_tags_2 },
    { 0xefcd8507, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecd850a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ad850f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978d8518, SUC_lexicon_tags_528 },
    { 0, NULL },
    { 0x7f4d851a, SUC_lexicon_tags_25 },
    { 0x320d851b, SUC_lexicon_tags_136 },
    { 0xe54d851c, SUC_lexicon_tags_9 },
    { 0xbded851b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4d8522, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x846d8524, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cd852a, SUC_lexicon_tags_37 },
    { 0x222d852b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120d8530, SUC_lexicon_tags_43 },
    { 0x876d8531, SUC_lexicon_tags_7 },
    { 0xb58d8530, SUC_lexicon_tags_25 },
    { 0x766d8533, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cd8536, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cd853a, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x94cd853c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xddad853f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16d8542, SUC_lexicon_tags_7 },
    { 0x692d8543, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc88d8545, SUC_lexicon_tags_15 },
    { 0x52ad8546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cd8549, SUC_lexicon_tags_35 },
    { 0x67ed854a, SUC_lexicon_tags_15 },
    { 0x89ad8549, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed8551, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x744d8553, SUC_lexicon_tags_29 },
    { 0xde6d8554, SUC_lexicon_tags_46 },
    { 0x9dad8555, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2ed8557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ed855d, SUC_lexicon_tags_49 },
    { 0x84ed855e, SUC_lexicon_tags_48 },
    { 0xd50d855d, SUC_lexicon_tags_17 },
    { 0x690d8560, SUC_lexicon_tags_1 },
    { 0xa64d855e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x16ed8563, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4a0d8565, SUC_lexicon_tags_7 },
    { 0x328d8566, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4d856a, SUC_lexicon_tags_43 },
    { 0x18ad856a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x196d856d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbad8573, SUC_lexicon_tags_7 },
    { 0x1c0d8574, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cd857a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51ed857c, SUC_lexicon_tags_17 },
    { 0xa5ed857c, SUC_lexicon_tags_28 },
    { 0x53ad857e, SUC_lexicon_tags_49 },
    { 0xaa6d857f, SUC_lexicon_tags_17 },
    { 0x782d857c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40ad8582, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd858a, SUC_lexicon_tags_37 },
    { 0x3eed858b, SUC_lexicon_tags_23 },
    { 0xcad858b, SUC_lexicon_tags_11 },
    { 0xbbad858a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2d8590, SUC_lexicon_tags_38 },
    { 0xb24d8591, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ed8599, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafad859b, SUC_lexicon_tags_38 },
    { 0x784d859c, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ad85a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc1ad85a2, SUC_lexicon_tags_27 },
    { 0x9ead85a3, SUC_lexicon_tags_17 },
    { 0x928d85a4, SUC_lexicon_tags_7 },
    { 0x52cd85a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8d85a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe58d85ab, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4d85af, SUC_lexicon_tags_7 },
    { 0x844d85af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x154d85b2, SUC_lexicon_tags_38 },
    { 0x9dcd85b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646d85b9, SUC_lexicon_tags_35 },
    { 0x8a4d85ba, SUC_lexicon_tags_17 },
    { 0x3eed85bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9ed85bd, SUC_lexicon_tags_48 },
    { 0xee4d85be, SUC_lexicon_tags_47 },
    { 0xfdad85bf, SUC_lexicon_tags_10 },
    { 0x792d85c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ad85c4, SUC_lexicon_tags_1 },
    { 0x590d85c5, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xcb8d85c7, SUC_lexicon_tags_11 },
    { 0xccad85c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6d85ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10cd85d2, SUC_lexicon_tags_30 },
    { 0xf1ad85d3, SUC_lexicon_tags_27 },
    { 0xe34d85d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578d85da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ad85dd, SUC_lexicon_tags_12 },
    { 0xe0d85dd, SUC_lexicon_tags_7 },
    { 0xdced85dd, SUC_lexicon_tags_9 },
    { 0x5c2d85de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4ad85e2, SUC_lexicon_tags_172 },
    { 0x32cd85e3, SUC_lexicon_tags_7 },
    { 0x296d85e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38d85e7, SUC_lexicon_tags_25 },
    { 0xd84d85e8, SUC_lexicon_tags_49 },
    { 0x816d85e9, SUC_lexicon_tags_23 },
    { 0x8c8d85e7, SUC_lexicon_tags_0 },
    { 0x1e6d85eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6d85f2, SUC_lexicon_tags_15 },
    { 0x5ced85f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x744d85f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec4d85f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ed85fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72d85fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68d8601, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9b2d8603, SUC_lexicon_tags_17 },
    { 0xe6ad8604, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26d8607, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36d860b, SUC_lexicon_tags_25 },
    { 0xce6d860c, SUC_lexicon_tags_45 },
    { 0xe7ed860d, SUC_lexicon_tags_113 },
    { 0x7eed860e, SUC_lexicon_tags_85 },
    { 0x1a0d860f, SUC_lexicon_tags_30 },
    { 0xf52d8610, SUC_lexicon_tags_25 },
    { 0x48d860b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x726d8614, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8ed8616, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1e6d8618, SUC_lexicon_tags_2 },
    { 0xaacd8619, SUC_lexicon_tags_15 },
    { 0x5d4d861a, SUC_lexicon_tags_17 },
    { 0x8bcd8619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12ad861d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2d8620, SUC_lexicon_tags_46 },
    { 0xd76d8621, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0x60cd8623, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140d8628, SUC_lexicon_tags_1 },
    { 0x436d8629, SUC_lexicon_tags_12 },
    { 0xae2d862a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc90d862c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x858d862f, SUC_lexicon_tags_1 },
    { 0x26cd862f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8d8635, SUC_lexicon_tags_4 },
    { 0x26d8636, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6bcd8638, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ed863d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4d8646, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6cd8648, SUC_lexicon_tags_37 },
    { 0xc06d8649, SUC_lexicon_tags_47 },
    { 0xeacd864a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d8650, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe10d8652, SUC_lexicon_tags_7 },
    { 0x70ad8653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2cd8655, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf22d8657, SUC_lexicon_tags_15 },
    { 0x33ed8658, SUC_lexicon_tags_12 },
    { 0xf1ad8658, SUC_lexicon_tags_12 },
    { 0xc0cd865a, SUC_lexicon_tags_13 },
    { 0x2ecd865b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d0d865d, SUC_lexicon_tags_3 },
    { 0x840d865e, SUC_lexicon_tags_12 },
    { 0x3aad865e, SUC_lexicon_tags_48 },
    { 0xf76d8660, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfbcd8662, SUC_lexicon_tags_25 },
    { 0xdfcd8663, SUC_lexicon_tags_30 },
    { 0x4d6d8664, SUC_lexicon_tags_25 },
    { 0xa0ad8665, SUC_lexicon_tags_15 },
    { 0xedad8666, SUC_lexicon_tags_14 },
    { 0x918d8666, SUC_lexicon_tags_33 },
    { 0x5b2d8667, SUC_lexicon_tags_12 },
    { 0x80d8665, SUC_lexicon_tags_25 },
    { 0x1d2d8667, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0d866d, SUC_lexicon_tags_4 },
    { 0xab6d866e, SUC_lexicon_tags_19 },
    { 0x59cd866e, SUC_lexicon_tags_33 },
    { 0x810d866e, SUC_lexicon_tags_14 },
    { 0xc26d8671, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8d8676, SUC_lexicon_tags_45 },
    { 0x4cd8677, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44d867a, SUC_lexicon_tags_1 },
    { 0x12d867a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ed8684, SUC_lexicon_tags_25 },
    { 0x960d8685, SUC_lexicon_tags_1 },
    { 0xf9ed8686, SUC_lexicon_tags_17 },
    { 0x16ed8686, SUC_lexicon_tags_12 },
    { 0xa14d8685, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6d868b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6d868e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad8697, SUC_lexicon_tags_25 },
    { 0xe1cd8697, SUC_lexicon_tags_11 },
    { 0xf40d8699, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0d86a4, SUC_lexicon_tags_11 },
    { 0x5dad86a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84ed86a7, SUC_lexicon_tags_2 },
    { 0xd9ad86a7, SUC_lexicon_tags_17 },
    { 0xe1cd86a9, SUC_lexicon_tags_37 },
    { 0xca0d86a8, SUC_lexicon_tags_23 },
    { 0x454d86a8, SUC_lexicon_tags_14 },
    { 0xb20d86ac, SUC_lexicon_tags_63 },
    { 0x10cd86ad, SUC_lexicon_tags_37 },
    { 0x306d86ad, SUC_lexicon_tags_12 },
    { 0x7eed86af, SUC_lexicon_tags_6 },
    { 0xd06d86ae, SUC_lexicon_tags_28 },
    { 0x9a4d86b1, SUC_lexicon_tags_15 },
    { 0x35ad86b2, SUC_lexicon_tags_37 },
    { 0x8e4d86b3, SUC_lexicon_tags_60 },
    { 0x82cd86b4, SUC_lexicon_tags_1 },
    { 0x608d86b4, SUC_lexicon_tags_7 },
    { 0x8b6d86b6, SUC_lexicon_tags_15 },
    { 0x8d8d86b7, SUC_lexicon_tags_61 },
    { 0xc28d86b5, SUC_lexicon_tags_23 },
    { 0x1cd86b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2b2d86bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462d86bf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ded86c7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcd86ca, SUC_lexicon_tags_45 },
    { 0xd5ed86ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad86ce, SUC_lexicon_tags_11 },
    { 0x8fed86cf, SUC_lexicon_tags_7 },
    { 0xf7cd86d0, SUC_lexicon_tags_6 },
    { 0x6a8d86d1, SUC_lexicon_tags_12 },
    { 0x1b4d86d1, SUC_lexicon_tags_6 },
    { 0xecad86d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2d86d6, SUC_lexicon_tags_11 },
    { 0x40ad86d7, SUC_lexicon_tags_28 },
    { 0x290d86d6, SUC_lexicon_tags_68 },
    { 0xea8d86d9, SUC_lexicon_tags_37 },
    { 0x85ad86d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cad86df, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x506d86e2, SUC_lexicon_tags_15 },
    { 0x34ed86e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a0d86e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ed86f1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d86fa, SUC_lexicon_tags_11 },
    { 0x2e0d86fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x538d86fd, SUC_lexicon_tags_16 },
    { 0x630d86fe, SUC_lexicon_tags_12 },
    { 0x214d86ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x59ad8701, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bed8705, SUC_lexicon_tags_43 },
    { 0x774d8706, SUC_lexicon_tags_0 },
    { 0x700d8706, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcd870b, SUC_lexicon_tags_25 },
    { 0xead870c, SUC_lexicon_tags_4 },
    { 0xe30d870d, SUC_lexicon_tags_11 },
    { 0x42ed870d, SUC_lexicon_tags_8 },
    { 0x3f8d870c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe94d8711, SUC_lexicon_tags_25 },
    { 0xd3cd8712, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd8cd8714, SUC_lexicon_tags_37 },
    { 0xf9ad8715, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacd8719, SUC_lexicon_tags_2 },
    { 0x332d871a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff6d871c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ed871f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6d8721, SUC_lexicon_tags_17 },
    { 0x7cad8722, SUC_lexicon_tags_35 },
    { 0xc30d8723, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9cd8725, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4d872c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x30cd872f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34d8734, SUC_lexicon_tags_47 },
    { 0x808d8735, SUC_lexicon_tags_7 },
    { 0x6fed8735, SUC_lexicon_tags_9 },
    { 0x10ad8737, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ed873a, SUC_lexicon_tags_61 },
    { 0x4f8d873b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ecd873d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14d8740, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896d8747, SUC_lexicon_tags_15 },
    { 0xa18d8748, SUC_lexicon_tags_28 },
    { 0x64ed8749, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe3cd874b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ed8750, SUC_lexicon_tags_13 },
    { 0x9aed8751, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb44d8753, SUC_lexicon_tags_15 },
    { 0x10ad8753, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16d8757, SUC_lexicon_tags_11 },
    { 0xa9ad8758, SUC_lexicon_tags_44 },
    { 0x9bed8759, SUC_lexicon_tags_1 },
    { 0xebcd875a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x854d875d, SUC_lexicon_tags_35 },
    { 0x744d875e, SUC_lexicon_tags_43 },
    { 0x7a6d875f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ad8764, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ad8768, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcd876b, SUC_lexicon_tags_47 },
    { 0x66ad876b, SUC_lexicon_tags_55 },
    { 0xccad876b, SUC_lexicon_tags_12 },
    { 0x4cad876e, SUC_lexicon_tags_34 },
    { 0x86ad876f, SUC_lexicon_tags_37 },
    { 0x66d8770, SUC_lexicon_tags_17 },
    { 0x2a6d876e, SUC_lexicon_tags_38 },
    { 0x40ad876f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cd8775, SUC_lexicon_tags_37 },
    { 0xdaed8775, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146d877c, SUC_lexicon_tags_7 },
    { 0x9b4d877d, SUC_lexicon_tags_12 },
    { 0xaced877c, SUC_lexicon_tags_61 },
    { 0xa8d877d, SUC_lexicon_tags_13 },
    { 0x35ad877d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x336d8783, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58ad8785, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd878a, SUC_lexicon_tags_45 },
    { 0xf6ad878a, SUC_lexicon_tags_69 },
    { 0xaeed878c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6d878f, SUC_lexicon_tags_7 },
    { 0xdb4d878f, SUC_lexicon_tags_47 },
    { 0xe2cd8791, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2dad8793, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cad8795, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48d8798, SUC_lexicon_tags_11 },
    { 0xd28d8799, SUC_lexicon_tags_60 },
    { 0x798d879a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2d879f, SUC_lexicon_tags_7 },
    { 0xeb2d879f, SUC_lexicon_tags_23 },
    { 0xc9cd87a1, SUC_lexicon_tags_11 },
    { 0x14ad87a2, SUC_lexicon_tags_11 },
    { 0xd0ad87a3, SUC_lexicon_tags_17 },
    { 0xd76d87a4, SUC_lexicon_tags_46 },
    { 0xfacd87a5, SUC_lexicon_tags_39 },
    { 0x260d87a6, SUC_lexicon_tags_39 },
    { 0xffcd87a5, SUC_lexicon_tags_10 },
    { 0xe3ed87a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf8ad87aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60d87b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b4d87b2, SUC_lexicon_tags_17 },
    { 0x710d87b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dad87b6, SUC_lexicon_tags_15 },
    { 0x472d87b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cd87bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x38cd87be, SUC_lexicon_tags_7 },
    { 0x23cd87be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498d87c7, SUC_lexicon_tags_11 },
    { 0xd52d87c8, SUC_lexicon_tags_7 },
    { 0x40d87c9, SUC_lexicon_tags_40 },
    { 0x110d87c8, SUC_lexicon_tags_3 },
    { 0xb0cd87cb, SUC_lexicon_tags_11 },
    { 0xc16d87cc, SUC_lexicon_tags_12 },
    { 0x8aed87cc, SUC_lexicon_tags_5 },
    { 0xba4d87ce, SUC_lexicon_tags_15 },
    { 0x60d87cf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74d87d2, SUC_lexicon_tags_14 },
    { 0x97ed87d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94d87d7, SUC_lexicon_tags_15 },
    { 0xf6ad87d7, SUC_lexicon_tags_12 },
    { 0x5f6d87d7, SUC_lexicon_tags_38 },
    { 0x402d87da, SUC_lexicon_tags_0 },
    { 0xe52d87d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3aad87dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6d87e0, SUC_lexicon_tags_37 },
    { 0x8b0d87e1, SUC_lexicon_tags_27 },
    { 0xac8d87e2, SUC_lexicon_tags_55 },
    { 0x304d87e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff4d87e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dad87e7, SUC_lexicon_tags_15 },
    { 0x166d87e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeed87ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccd87ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2d87f0, SUC_lexicon_tags_38 },
    { 0x806d87f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40d87f7, SUC_lexicon_tags_55 },
    { 0x2d0d87f7, SUC_lexicon_tags_27 },
    { 0x436d87f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ad87fc, SUC_lexicon_tags_2 },
    { 0xb82d87fc, SUC_lexicon_tags_17 },
    { 0xeb8d87fe, SUC_lexicon_tags_1 },
    { 0xf10d87ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ed8802, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40d8806, SUC_lexicon_tags_10 },
    { 0x5dad8807, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x656d8809, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb60d880c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22d880f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40d8814, SUC_lexicon_tags_15 },
    { 0x1d2d8814, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74d881f, SUC_lexicon_tags_49 },
    { 0xb08d8820, SUC_lexicon_tags_37 },
    { 0x236d8820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fed8829, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x556d882b, SUC_lexicon_tags_8 },
    { 0x65ed882b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x12d882e, SUC_lexicon_tags_38 },
    { 0xa8ed882f, SUC_lexicon_tags_66 },
    { 0xe16d882f, SUC_lexicon_tags_116 },
    { 0xf8d882f, SUC_lexicon_tags_27 },
    { 0xce2d8832, SUC_lexicon_tags_45 },
    { 0xf6cd8832, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6d8836, SUC_lexicon_tags_12 },
    { 0xe84d8837, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ed883c, SUC_lexicon_tags_15 },
    { 0xe7cd883c, SUC_lexicon_tags_25 },
    { 0x856d883c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdad8840, SUC_lexicon_tags_15 },
    { 0xbf2d8841, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x590d8844, SUC_lexicon_tags_6 },
    { 0x422d8845, SUC_lexicon_tags_60 },
    { 0x6e6d8845, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ad884b, SUC_lexicon_tags_22 },
    { 0x9cd884b, SUC_lexicon_tags_8 },
    { 0xab6d884d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf92d884f, SUC_lexicon_tags_17 },
    { 0xb30d884f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cd8857, SUC_lexicon_tags_37 },
    { 0xa72d8858, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ad885b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cd8860, SUC_lexicon_tags_0 },
    { 0xe16d8860, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x812d8863, SUC_lexicon_tags_42 },
    { 0xe88d8863, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xa2cd8866, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfc6d8868, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x734d886b, SUC_lexicon_tags_29 },
    { 0xed4d886c, SUC_lexicon_tags_17 },
    { 0xa22d886d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x280d886f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304d8873, SUC_lexicon_tags_29 },
    { 0x892d8873, SUC_lexicon_tags_491 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4d8878, SUC_lexicon_tags_2 },
    { 0x5c4d8879, SUC_lexicon_tags_17 },
    { 0x5f2d887a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2d887f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ad8882, SUC_lexicon_tags_1 },
    { 0x81cd8883, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22d8887, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30d8890, SUC_lexicon_tags_39 },
    { 0x412d8891, SUC_lexicon_tags_37 },
    { 0x7eed8892, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d889c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb2d889e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x118d88a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12d88a3, SUC_lexicon_tags_66 },
    { 0x912d88a3, SUC_lexicon_tags_13 },
    { 0xa36d88a5, SUC_lexicon_tags_63 },
    { 0x294d88a6, SUC_lexicon_tags_12 },
    { 0xe1ed88a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xadad88a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4d88af, SUC_lexicon_tags_37 },
    { 0xf68d88b0, SUC_lexicon_tags_185 },
    { 0xeaed88af, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cd88b4, SUC_lexicon_tags_2 },
    { 0x3c0d88b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4d88b8, SUC_lexicon_tags_37 },
    { 0x284d88b9, SUC_lexicon_tags_15 },
    { 0x526d88b8, SUC_lexicon_tags_45 },
    { 0xeb8d88bb, SUC_lexicon_tags_63 },
    { 0xeced88bb, SUC_lexicon_tags_11 },
    { 0x82d88bb, SUC_lexicon_tags_87 },
    { 0x564d88be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2d88c2, SUC_lexicon_tags_23 },
    { 0xa92d88c2, SUC_lexicon_tags_7 },
    { 0x920d88c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed88c8, SUC_lexicon_tags_7 },
    { 0x644d88c9, SUC_lexicon_tags_11 },
    { 0x57ed88c9, SUC_lexicon_tags_37 },
    { 0x600d88cb, SUC_lexicon_tags_15 },
    { 0x6fad88cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6d88d0, SUC_lexicon_tags_3 },
    { 0x98ad88d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ed88d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xad0d88d6, SUC_lexicon_tags_33 },
    { 0x114d88d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ad88da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d88dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d4d88df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x856d88e1, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0x284d88e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268d88ee, SUC_lexicon_tags_38 },
    { 0xb40d88ee, SUC_lexicon_tags_15 },
    { 0xe82d88f0, SUC_lexicon_tags_1 },
    { 0xbd8d88ee, SUC_lexicon_tags_17 },
    { 0xafad88f2, SUC_lexicon_tags_1 },
    { 0xe9cd88f0, SUC_lexicon_tags_62 },
    { 0xe28d88f4, SUC_lexicon_tags_12 },
    { 0x5f8d88f5, SUC_lexicon_tags_11 },
    { 0x332d88f6, SUC_lexicon_tags_0 },
    { 0xb2ad88f6, SUC_lexicon_tags_10 },
    { 0x204d88f8, SUC_lexicon_tags_10 },
    { 0x84d88f8, SUC_lexicon_tags_68 },
    { 0x332d88fa, SUC_lexicon_tags_37 },
    { 0xbc8d88f2, SUC_lexicon_tags_2 },
    { 0xc2ad88f5, SUC_lexicon_tags_37 },
    { 0xbd8d88fd, SUC_lexicon_tags_47 },
    { 0xf08d88fd, SUC_lexicon_tags_1 },
    { 0x656d88ff, SUC_lexicon_tags_7 },
    { 0x316d88ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x802d8902, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d890a, SUC_lexicon_tags_13 },
    { 0x65ed890a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4d890e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0d8911, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x810d8913, SUC_lexicon_tags_17 },
    { 0xe46d8914, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68ed8916, SUC_lexicon_tags_12 },
    { 0x72cd8917, SUC_lexicon_tags_11 },
    { 0x6a6d8916, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x328d891a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72d891d, SUC_lexicon_tags_31 },
    { 0x844d891e, SUC_lexicon_tags_112 },
    { 0x4d891f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04d8924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61ed8926, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2d8928, SUC_lexicon_tags_140 },
    { 0xb6cd8929, SUC_lexicon_tags_32 },
    { 0x7c2d892a, SUC_lexicon_tags_25 },
    { 0xd86d892b, SUC_lexicon_tags_88 },
    { 0x4bed892a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x894d892e, SUC_lexicon_tags_12 },
    { 0x4aad892f, SUC_lexicon_tags_37 },
    { 0x206d892e, SUC_lexicon_tags_23 },
    { 0x9b8d8931, SUC_lexicon_tags_39 },
    { 0x2c4d8931, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd0d8934, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa88d8936, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eed893c, SUC_lexicon_tags_47 },
    { 0xf40d893c, SUC_lexicon_tags_26 },
    { 0xfdcd893e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98d8947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ad894a, SUC_lexicon_tags_33 },
    { 0x64ed894b, SUC_lexicon_tags_37 },
    { 0x5eed894c, SUC_lexicon_tags_25 },
    { 0xf5ad894b, SUC_lexicon_tags_43 },
    { 0x7cd894e, SUC_lexicon_tags_8 },
    { 0x50cd894e, SUC_lexicon_tags_6 },
    { 0x89ad8950, SUC_lexicon_tags_7 },
    { 0xe5cd894e, SUC_lexicon_tags_28 },
    { 0x996d8952, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cd8958, SUC_lexicon_tags_9 },
    { 0xefcd8959, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d895e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458d8963, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x942d8965, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e8d8967, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x404d8969, SUC_lexicon_tags_167 },
    { 0xe50d896a, SUC_lexicon_tags_11 },
    { 0xf42d896a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x804d896d, SUC_lexicon_tags_17 },
    { 0x554d896e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0d8973, SUC_lexicon_tags_11 },
    { 0xd04d8974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ad8978, SUC_lexicon_tags_41 },
    { 0x512d8979, SUC_lexicon_tags_4 },
    { 0x942d8979, SUC_lexicon_tags_11 },
    { 0x514d897a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98d897e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994d8986, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532d898b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52d898f, SUC_lexicon_tags_1 },
    { 0x96ad8990, SUC_lexicon_tags_25 },
    { 0xbb0d8990, SUC_lexicon_tags_17 },
    { 0xf24d898f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f2d8994, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cd899c, SUC_lexicon_tags_26 },
    { 0xd32d899c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4d89a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa84d89a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46d89a5, SUC_lexicon_tags_37 },
    { 0x426d89a5, SUC_lexicon_tags_28 },
    { 0x694d89a7, SUC_lexicon_tags_63 },
    { 0x796d89a5, SUC_lexicon_tags_47 },
    { 0x89ad89a9, SUC_lexicon_tags_46 },
    { 0x784d89aa, SUC_lexicon_tags_17 },
    { 0x7cad89a8, SUC_lexicon_tags_46 },
    { 0xec8d89a8, SUC_lexicon_tags_1 },
    { 0x690d89ad, SUC_lexicon_tags_1 },
    { 0xcf2d89ae, SUC_lexicon_tags_11 },
    { 0xee8d89af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32d89b5, SUC_lexicon_tags_23 },
    { 0x2c8d89b6, SUC_lexicon_tags_1 },
    { 0x900d89b7, SUC_lexicon_tags_12 },
    { 0xb16d89b8, SUC_lexicon_tags_17 },
    { 0xc8cd89b8, SUC_lexicon_tags_17 },
    { 0xdf2d89ba, SUC_lexicon_tags_23 },
    { 0xbe0d89bb, SUC_lexicon_tags_7 },
    { 0xcaed89b8, SUC_lexicon_tags_28 },
    { 0x41cd89bd, SUC_lexicon_tags_33 },
    { 0x14ed89ba, SUC_lexicon_tags_77 },
    { 0xc4d89bf, SUC_lexicon_tags_60 },
    { 0x49cd89c0, SUC_lexicon_tags_37 },
    { 0x9e2d89c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8ad89c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30cd89c5, SUC_lexicon_tags_19 },
    { 0x8a2d89c5, SUC_lexicon_tags_25 },
    { 0x762d89c7, SUC_lexicon_tags_7 },
    { 0xb3ad89c8, SUC_lexicon_tags_12 },
    { 0x2b8d89c9, SUC_lexicon_tags_45 },
    { 0x282d89ca, SUC_lexicon_tags_25 },
    { 0xe54d89c6, SUC_lexicon_tags_0 },
    { 0x474d89cc, SUC_lexicon_tags_37 },
    { 0xfa0d89cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x68ed89cf, SUC_lexicon_tags_63 },
    { 0x78ed89cf, SUC_lexicon_tags_37 },
    { 0x294d89cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4d89d4, SUC_lexicon_tags_17 },
    { 0x504d89d4, SUC_lexicon_tags_12 },
    { 0xea2d89d6, SUC_lexicon_tags_23 },
    { 0x78cd89d6, SUC_lexicon_tags_45 },
    { 0x2f2d89d6, SUC_lexicon_tags_11 },
    { 0x718d89d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702d89dd, SUC_lexicon_tags_7 },
    { 0xfcd89dd, SUC_lexicon_tags_37 },
    { 0xb92d89df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cd89e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ad89e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768d89ec, SUC_lexicon_tags_7 },
    { 0x2d0d89ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc7cd89ef, SUC_lexicon_tags_25 },
    { 0x52ed89f0, SUC_lexicon_tags_25 },
    { 0x7f8d89f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dad89f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x824d89f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0d89fb, SUC_lexicon_tags_23 },
    { 0x374d89fc, SUC_lexicon_tags_60 },
    { 0x3f2d89fd, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xf54d89ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256d8a03, SUC_lexicon_tags_5 },
    { 0x402d8a04, SUC_lexicon_tags_60 },
    { 0xb8ad8a05, SUC_lexicon_tags_3 },
    { 0xde6d8a04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ed8a0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392d8a12, SUC_lexicon_tags_11 },
    { 0x49cd8a12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6d8a16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x220d8a18, SUC_lexicon_tags_85 },
    { 0x388d8a19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x420d8a1c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cd8a25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314d8a2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4d8a2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e6d8a31, SUC_lexicon_tags_11 },
    { 0x59cd8a31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ad8a35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cad8a3a, SUC_lexicon_tags_27 },
    { 0x908d8a3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8d8a3e, SUC_lexicon_tags_37 },
    { 0x8ccd8a3e, SUC_lexicon_tags_2 },
    { 0x1b2d8a40, SUC_lexicon_tags_47 },
    { 0x49cd8a3e, SUC_lexicon_tags_1 },
    { 0xf9cd8a42, SUC_lexicon_tags_11 },
    { 0x96cd8a43, SUC_lexicon_tags_31 },
    { 0x75ed8a40, SUC_lexicon_tags_15 },
    { 0xf4cd8a43, SUC_lexicon_tags_27 },
    { 0xcc8d8a46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cd8a49, SUC_lexicon_tags_25 },
    { 0xdb8d8a4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99ad8a4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572d8a51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08d8a56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda8d8a58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ad8a5c, SUC_lexicon_tags_25 },
    { 0x4ecd8a5d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30d8a60, SUC_lexicon_tags_27 },
    { 0xa76d8a61, SUC_lexicon_tags_2 },
    { 0x59cd8a61, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcad8a65, SUC_lexicon_tags_3 },
    { 0xe6d8a66, SUC_lexicon_tags_11 },
    { 0x91ad8a65, SUC_lexicon_tags_15 },
    { 0x67cd8a67, SUC_lexicon_tags_43 },
    { 0x6fcd8a69, SUC_lexicon_tags_14 },
    { 0xa88d8a68, SUC_lexicon_tags_14 },
    { 0xa2ad8a6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8d8a6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefed8a73, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d8a76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x402d8a79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ad8a84, SUC_lexicon_tags_11 },
    { 0x368d8a85, SUC_lexicon_tags_10 },
    { 0x80ad8a85, SUC_lexicon_tags_29 },
    { 0x4a4d8a87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ed8a89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed4d8a8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18d8a8e, SUC_lexicon_tags_25 },
    { 0xfb4d8a8e, SUC_lexicon_tags_26 },
    { 0x9a2d8a90, SUC_lexicon_tags_27 },
    { 0xeead8a91, SUC_lexicon_tags_47 },
    { 0x374d8a90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cd8a97, SUC_lexicon_tags_33 },
    { 0x330d8a98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52d8a9c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x478d8a9e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x56cd8aa0, SUC_lexicon_tags_55 },
    { 0x77ed8aa1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eed8aa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60d8aa9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146d8ab2, SUC_lexicon_tags_37 },
    { 0x582d8ab3, SUC_lexicon_tags_12 },
    { 0x598d8ab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x898d8ab6, SUC_lexicon_tags_7 },
    { 0x74ed8ab6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d8aba, SUC_lexicon_tags_25 },
    { 0x36d8aba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x466d8abd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82d8ac0, SUC_lexicon_tags_11 },
    { 0x66ed8ac1, SUC_lexicon_tags_29 },
    { 0xfaed8ac2, SUC_lexicon_tags_35 },
    { 0xbecd8ac3, SUC_lexicon_tags_6 },
    { 0x416d8ac4, SUC_lexicon_tags_15 },
    { 0xa46d8ac5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0d8ac8, SUC_lexicon_tags_17 },
    { 0x254d8ac9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ed8acc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294d8ad2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6d8ad5, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184d8adc, SUC_lexicon_tags_60 },
    { 0x826d8adc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8d8ae0, SUC_lexicon_tags_28 },
    { 0x702d8ae1, SUC_lexicon_tags_15 },
    { 0x722d8ae0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44d8ae5, SUC_lexicon_tags_79 },
    { 0x4e6d8ae5, SUC_lexicon_tags_7 },
    { 0xb06d8ae7, SUC_lexicon_tags_1 },
    { 0x6bed8ae5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322d8aec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8d8af0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7ad8af2, SUC_lexicon_tags_19 },
    { 0x126d8af2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8d8af7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182d8afc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10d8aff, SUC_lexicon_tags_1 },
    { 0xe5cd8b00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2aed8b02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2ad8b04, SUC_lexicon_tags_60 },
    { 0xcf0d8b05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2d8b0a, SUC_lexicon_tags_25 },
    { 0x90ed8b0b, SUC_lexicon_tags_7 },
    { 0x418d8b0b, SUC_lexicon_tags_25 },
    { 0xa7cd8b0d, SUC_lexicon_tags_14 },
    { 0x82ed8b0a, SUC_lexicon_tags_107 },
    { 0x3a2d8b0f, SUC_lexicon_tags_25 },
    { 0xbe2d8b10, SUC_lexicon_tags_11 },
    { 0xa9ed8b11, SUC_lexicon_tags_6 },
    { 0xf78d8b12, SUC_lexicon_tags_17 },
    { 0xb8cd8b13, SUC_lexicon_tags_11 },
    { 0x432d8b11, SUC_lexicon_tags_47 },
    { 0x3d2d8b15, SUC_lexicon_tags_11 },
    { 0x836d8b12, SUC_lexicon_tags_27 },
    { 0x3d6d8b17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x790d8b19, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfd0d8b1b, SUC_lexicon_tags_23 },
    { 0xe62d8b1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460d8b20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x844d8b23, SUC_lexicon_tags_61 },
    { 0xe48d8b24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0d8b2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x500d8b2e, SUC_lexicon_tags_25 },
    { 0x7b4d8b2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4d8b34, SUC_lexicon_tags_0 },
    { 0x844d8b35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa72d8b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x294d8b3a, SUC_lexicon_tags_37 },
    { 0x9fad8b3a, SUC_lexicon_tags_63 },
    { 0xd06d8b3c, SUC_lexicon_tags_7 },
    { 0x94ed8b3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc38d8b3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x99ed8b41, SUC_lexicon_tags_2 },
    { 0xe74d8b42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ed8b4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690d8b4e, SUC_lexicon_tags_37 },
    { 0xbe4d8b4f, SUC_lexicon_tags_11 },
    { 0x29cd8b50, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8bcd8b52, SUC_lexicon_tags_23 },
    { 0xa6cd8b53, SUC_lexicon_tags_9 },
    { 0x2bcd8b52, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xae8d8b56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe18d8b58, SUC_lexicon_tags_8 },
    { 0x24ed8b59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ad8b5f, SUC_lexicon_tags_17 },
    { 0xdc8d8b60, SUC_lexicon_tags_29 },
    { 0x40cd8b60, SUC_lexicon_tags_35 },
    { 0xd5ed8b61, SUC_lexicon_tags_26 },
    { 0x42ed8b63, SUC_lexicon_tags_23 },
    { 0x9fad8b63, SUC_lexicon_tags_262 },
    { 0x2ed8b64, SUC_lexicon_tags_31 },
    { 0x6acd8b66, SUC_lexicon_tags_37 },
    { 0x616d8b67, SUC_lexicon_tags_6 },
    { 0x5c6d8b66, SUC_lexicon_tags_63 },
    { 0xd94d8b69, SUC_lexicon_tags_47 },
    { 0x6dad8b6a, SUC_lexicon_tags_1 },
    { 0xa62d8b69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd8b70, SUC_lexicon_tags_32 },
    { 0x876d8b71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58d8b7a, SUC_lexicon_tags_23 },
    { 0xc7ed8b7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x340d8b7e, SUC_lexicon_tags_27 },
    { 0xccd8b7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cd8b87, SUC_lexicon_tags_25 },
    { 0x1bed8b88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b8d8b8a, SUC_lexicon_tags_38 },
    { 0xd88d8b8b, SUC_lexicon_tags_63 },
    { 0xb2ed8b8a, SUC_lexicon_tags_12 },
    { 0xf56d8b8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4ed8b8f, SUC_lexicon_tags_5 },
    { 0x6b4d8b90, SUC_lexicon_tags_37 },
    { 0x612d8b91, SUC_lexicon_tags_60 },
    { 0x1c4d8b92, SUC_lexicon_tags_57 },
    { 0x178d8b93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x382d8b96, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc54d8b98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbed8b9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf52d8b9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340d8ba0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cd8ba6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0d8ba9, SUC_lexicon_tags_31 },
    { 0x94ad8baa, SUC_lexicon_tags_37 },
    { 0xc6cd8bab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x58d8bad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x410d8baf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0d8bb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d8bb7, SUC_lexicon_tags_31 },
    { 0xb08d8bb8, SUC_lexicon_tags_23 },
    { 0x44cd8bb7, SUC_lexicon_tags_15 },
    { 0x400d8bb9, SUC_lexicon_tags_28 },
    { 0xa9ad8bbb, SUC_lexicon_tags_60 },
    { 0x6ded8bbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0d8bc1, SUC_lexicon_tags_77 },
    { 0x990d8bc2, SUC_lexicon_tags_7 },
    { 0xdbed8bc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220d8bca, SUC_lexicon_tags_80 },
    { 0x77ed8bcb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600d8bd0, SUC_lexicon_tags_37 },
    { 0x7f0d8bd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00d8bd4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd38d8bd6, SUC_lexicon_tags_37 },
    { 0x19ed8bd7, SUC_lexicon_tags_62 },
    { 0x166d8bd6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97ad8bda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x864d8bdc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ad8be8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cd8beb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ed8bf4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x93ad8bf6, SUC_lexicon_tags_17 },
    { 0x18ad8bf7, SUC_lexicon_tags_33 },
    { 0xec4d8bf8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20d8bfb, SUC_lexicon_tags_37 },
    { 0xa8ad8bfc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x334d8bff, SUC_lexicon_tags_30 },
    { 0x808d8c00, SUC_lexicon_tags_17 },
    { 0x7b0d8c00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fed8c04, SUC_lexicon_tags_17 },
    { 0xdfed8c05, SUC_lexicon_tags_15 },
    { 0x300d8c05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ed8c0c, SUC_lexicon_tags_2 },
    { 0xc58d8c0c, SUC_lexicon_tags_26 },
    { 0xb52d8c0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff2d8c10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2d8c13, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ad8c17, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984d8c1b, SUC_lexicon_tags_60 },
    { 0xdecd8c1c, SUC_lexicon_tags_62 },
    { 0x89cd8c1d, SUC_lexicon_tags_7 },
    { 0x8ad8c1d, SUC_lexicon_tags_25 },
    { 0xe06d8c1b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cd8c22, SUC_lexicon_tags_1 },
    { 0x9a6d8c23, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd8c27, SUC_lexicon_tags_45 },
    { 0xaf2d8c28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xab6d8c2a, SUC_lexicon_tags_12 },
    { 0xce2d8c2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8d8c2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cd8c33, SUC_lexicon_tags_0 },
    { 0xa74d8c34, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6d8c38, SUC_lexicon_tags_19 },
    { 0x10d8c39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6c4d8c3b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778d8c40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4acd8c43, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4aed8c45, SUC_lexicon_tags_17 },
    { 0x5a8d8c46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbccd8c48, SUC_lexicon_tags_33 },
    { 0xad2d8c49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x274d8c4b, SUC_lexicon_tags_17 },
    { 0x3d6d8c4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8d8c52, SUC_lexicon_tags_7 },
    { 0x6bed8c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8d8c56, SUC_lexicon_tags_70 },
    { 0xbb4d8c56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ed8c5a, SUC_lexicon_tags_7 },
    { 0xd6d8c5b, SUC_lexicon_tags_11 },
    { 0x3d4d8c5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5ad8c5e, SUC_lexicon_tags_17 },
    { 0x32ad8c5f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370d8c68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86ad8c6a, SUC_lexicon_tags_15 },
    { 0xeeed8c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ed8c6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafad8c73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8d8c76, SUC_lexicon_tags_49 },
    { 0x314d8c77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78d8c7c, SUC_lexicon_tags_12 },
    { 0x8a4d8c7d, SUC_lexicon_tags_43 },
    { 0x9c6d8c7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfed8c83, SUC_lexicon_tags_11 },
    { 0x896d8c84, SUC_lexicon_tags_37 },
    { 0x692d8c83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ed8c88, SUC_lexicon_tags_37 },
    { 0x31ed8c89, SUC_lexicon_tags_33 },
    { 0xebcd8c89, SUC_lexicon_tags_44 },
    { 0xfd0d8c8b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6dcd8c8d, SUC_lexicon_tags_1 },
    { 0x62d8c8e, SUC_lexicon_tags_25 },
    { 0x5ad8c8f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa2d8c91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86d8c9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfced8c9e, SUC_lexicon_tags_37 },
    { 0xfd2d8c9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512d8ca4, SUC_lexicon_tags_12 },
    { 0xf80d8ca5, SUC_lexicon_tags_23 },
    { 0x1bad8ca5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bad8ca8, SUC_lexicon_tags_45 },
    { 0x92ed8ca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18d8caf, SUC_lexicon_tags_17 },
    { 0x9f6d8cb0, SUC_lexicon_tags_15 },
    { 0x56cd8cb1, SUC_lexicon_tags_16 },
    { 0x4e6d8cb2, SUC_lexicon_tags_21 },
    { 0x5fed8cb3, SUC_lexicon_tags_45 },
    { 0xf52d8cb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4d8cb7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22d8cbb, SUC_lexicon_tags_9 },
    { 0xe88d8cbb, SUC_lexicon_tags_12 },
    { 0xdccd8cbd, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4d8cc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x254d8cc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ed8cc5, SUC_lexicon_tags_25 },
    { 0xeb2d8cc6, SUC_lexicon_tags_12 },
    { 0x250d8cc6, SUC_lexicon_tags_15 },
    { 0x288d8cc7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ed8ccb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216d8cd1, SUC_lexicon_tags_47 },
    { 0xd3ed8cd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x604d8cd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6d8cd9, SUC_lexicon_tags_47 },
    { 0x47ad8cda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00d8cdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x828d8cdf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xceed8ce2, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32d8ce5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae0d8ce7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878d8ced, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x308d8cef, SUC_lexicon_tags_0 },
    { 0x158d8cf0, SUC_lexicon_tags_1 },
    { 0x10ed8cf0, SUC_lexicon_tags_43 },
    { 0x92ed8cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x352d8cf5, SUC_lexicon_tags_13 },
    { 0x3d6d8cf5, SUC_lexicon_tags_2 },
    { 0xc58d8cf7, SUC_lexicon_tags_9 },
    { 0x8bed8cf5, SUC_lexicon_tags_1 },
    { 0x352d8cf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ced8cfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed8cfe, SUC_lexicon_tags_63 },
    { 0xe46d8cfe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d8d02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2d8d07, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x546d8d09, SUC_lexicon_tags_26 },
    { 0xee2d8d09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x694d8d0c, SUC_lexicon_tags_55 },
    { 0xa52d8d0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ed8d13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x92ad8d15, SUC_lexicon_tags_92 },
    { 0x9f4d8d15, SUC_lexicon_tags_37 },
    { 0x236d8d17, SUC_lexicon_tags_8 },
    { 0xb7cd8d15, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecd8d1c, SUC_lexicon_tags_11 },
    { 0x756d8d1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf14d8d1f, SUC_lexicon_tags_27 },
    { 0x2a4d8d20, SUC_lexicon_tags_23 },
    { 0xfbad8d21, SUC_lexicon_tags_21 },
    { 0x616d8d22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cd8d2a, SUC_lexicon_tags_7 },
    { 0x764d8d2b, SUC_lexicon_tags_77 },
    { 0xc56d8d2b, SUC_lexicon_tags_17 },
    { 0x572d8d2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88d8d30, SUC_lexicon_tags_13 },
    { 0xe8d8d31, SUC_lexicon_tags_2 },
    { 0x15ad8d32, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc9cd8d34, SUC_lexicon_tags_15 },
    { 0x8b8d8d35, SUC_lexicon_tags_12 },
    { 0x1aad8d36, SUC_lexicon_tags_17 },
    { 0x884d8d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624d8d3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ed8d3f, SUC_lexicon_tags_2 },
    { 0xcb0d8d3f, SUC_lexicon_tags_9 },
    { 0x184d8d41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85cd8d43, SUC_lexicon_tags_25 },
    { 0x17ad8d44, SUC_lexicon_tags_47 },
    { 0x554d8d43, SUC_lexicon_tags_13 },
    { 0x722d8d46, SUC_lexicon_tags_11 },
    { 0x6a2d8d46, SUC_lexicon_tags_2 },
    { 0x1d4d8d46, SUC_lexicon_tags_16 },
    { 0x9f0d8d49, SUC_lexicon_tags_37 },
    { 0x8fcd8d48, SUC_lexicon_tags_306 },
    { 0x164d8d4b, SUC_lexicon_tags_4 },
    { 0x6a2d8d4c, SUC_lexicon_tags_17 },
    { 0x3e8d8d4d, SUC_lexicon_tags_60 },
    { 0xf80d8d4d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ed8d51, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc10d8d53, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x44cd8d55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06d8d5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x608d8d5c, SUC_lexicon_tags_27 },
    { 0xbc8d8d5d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaeed8d5f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe84d8d61, SUC_lexicon_tags_8 },
    { 0xc6ed8d62, SUC_lexicon_tags_22 },
    { 0xb22d8d61, SUC_lexicon_tags_38 },
    { 0x402d8d63, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad8d67, SUC_lexicon_tags_17 },
    { 0xe7ed8d67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4b8d8d6a, SUC_lexicon_tags_6 },
    { 0x85ed8d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ad8d6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x738d8d70, SUC_lexicon_tags_60 },
    { 0x586d8d71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb0d8d73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8d8d77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ed8d7d, SUC_lexicon_tags_1 },
    { 0xa3ed8d7d, SUC_lexicon_tags_84 },
    { 0x436d8d7f, SUC_lexicon_tags_33 },
    { 0x4f6d8d7e, SUC_lexicon_tags_25 },
    { 0x7acd8d7e, SUC_lexicon_tags_38 },
    { 0xe3ed8d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66d8d85, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x456d8d87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fad8d8a, SUC_lexicon_tags_27 },
    { 0x6a8d8d8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef8d8d8d, SUC_lexicon_tags_17 },
    { 0xc34d8d8e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9ccd8d90, SUC_lexicon_tags_23 },
    { 0x9ccd8d91, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x78d8d93, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756d8d9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40d8da0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fad8da8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466d8dae, SUC_lexicon_tags_29 },
    { 0x22cd8daf, SUC_lexicon_tags_84 },
    { 0xe34d8db0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xae0d8db3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x904d8db5, SUC_lexicon_tags_63 },
    { 0xc04d8db6, SUC_lexicon_tags_27 },
    { 0x46ed8db7, SUC_lexicon_tags_17 },
    { 0x1aad8db8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4d8dbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8d8dc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ad8dcb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xded8dcd, SUC_lexicon_tags_17 },
    { 0xf00d8dcd, SUC_lexicon_tags_11 },
    { 0x408d8dcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d8dd4, SUC_lexicon_tags_37 },
    { 0x6c8d8dd5, SUC_lexicon_tags_17 },
    { 0x5c8d8dd5, SUC_lexicon_tags_37 },
    { 0x60ed8dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172d8ddd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d0d8ddf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8d8de8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ad8deb, SUC_lexicon_tags_45 },
    { 0xcdad8deb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc8d8dee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0d8df1, SUC_lexicon_tags_45 },
    { 0xd6ad8df2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d0d8df4, SUC_lexicon_tags_11 },
    { 0x418d8df4, SUC_lexicon_tags_28 },
    { 0x99cd8df6, SUC_lexicon_tags_45 },
    { 0xa94d8df7, SUC_lexicon_tags_16 },
    { 0x2d2d8df8, SUC_lexicon_tags_1 },
    { 0x1f2d8df9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2d8dfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92d8e03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6d8e09, SUC_lexicon_tags_45 },
    { 0x8e6d8e0a, SUC_lexicon_tags_11 },
    { 0x154d8e0a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x228d8e0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd8e10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb38d8e13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bad8e19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ad8e20, SUC_lexicon_tags_14 },
    { 0x178d8e21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64d8e24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ad8e27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc3cd8e29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ad8e2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4ad8e2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3f6d8e31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ad8e36, SUC_lexicon_tags_47 },
    { 0x26ed8e36, SUC_lexicon_tags_28 },
    { 0x284d8e38, SUC_lexicon_tags_23 },
    { 0xca2d8e37, SUC_lexicon_tags_30 },
    { 0x53ad8e37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6d8e3d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d8e40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe84d8e42, SUC_lexicon_tags_12 },
    { 0xea0d8e42, SUC_lexicon_tags_17 },
    { 0x83ad8e44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x242d8e47, SUC_lexicon_tags_2 },
    { 0x458d8e48, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ad8e4b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32d8e4e, SUC_lexicon_tags_23 },
    { 0xbacd8e4f, SUC_lexicon_tags_11 },
    { 0x84cd8e50, SUC_lexicon_tags_32 },
    { 0xaa0d8e4f, SUC_lexicon_tags_55 },
    { 0x1fad8e4f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8e55, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb34d8e57, SUC_lexicon_tags_1 },
    { 0x976d8e58, SUC_lexicon_tags_31 },
    { 0xebed8e59, SUC_lexicon_tags_37 },
    { 0x63ed8e59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc3cd8e5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x840d8e5e, SUC_lexicon_tags_14 },
    { 0x77cd8e5f, SUC_lexicon_tags_47 },
    { 0x310d8e60, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbced8e69, SUC_lexicon_tags_103 },
    { 0x5a0d8e69, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0d8e6d, SUC_lexicon_tags_17 },
    { 0xe90d8e6e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6c6d8e70, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56d8e77, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x546d8e7a, SUC_lexicon_tags_17 },
    { 0xcc0d8e7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4d8e83, SUC_lexicon_tags_26 },
    { 0x356d8e84, SUC_lexicon_tags_43 },
    { 0x670d8e85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dad8e8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd80d8e8c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596d8e90, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2d8e94, SUC_lexicon_tags_12 },
    { 0x7fcd8e94, SUC_lexicon_tags_8 },
    { 0x5fad8e96, SUC_lexicon_tags_6 },
    { 0x8dcd8e97, SUC_lexicon_tags_1 },
    { 0xd06d8e97, SUC_lexicon_tags_26 },
    { 0xecad8e99, SUC_lexicon_tags_9 },
    { 0x4f0d8e9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98d8ea1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x762d8ea3, SUC_lexicon_tags_9 },
    { 0x984d8ea3, SUC_lexicon_tags_47 },
    { 0x1a8d8ea5, SUC_lexicon_tags_47 },
    { 0xb94d8ea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2d8eab, SUC_lexicon_tags_63 },
    { 0x48cd8eac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba6d8eae, SUC_lexicon_tags_31 },
    { 0xb18d8eae, SUC_lexicon_tags_72 },
    { 0x274d8eb0, SUC_lexicon_tags_87 },
    { 0xebcd8eb0, SUC_lexicon_tags_31 },
    { 0x174d8eb0, SUC_lexicon_tags_75 },
    { 0xcc8d8eaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad8eb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2d8ebc, SUC_lexicon_tags_19 },
    { 0x7a8d8ebd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2d8ec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfded8ec5, SUC_lexicon_tags_11 },
    { 0xbaed8ec5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382d8ecb, SUC_lexicon_tags_5 },
    { 0xd28d8ecb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x396d8ecf, SUC_lexicon_tags_32 },
    { 0x1f6d8ed0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8d8ed5, SUC_lexicon_tags_47 },
    { 0x2d2d8ed6, SUC_lexicon_tags_60 },
    { 0xf64d8ed5, SUC_lexicon_tags_38 },
    { 0x814d8ed6, SUC_lexicon_tags_10 },
    { 0x186d8ed9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fcd8edb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4fcd8edd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaed8ee0, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0x696d8ee2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ad8ee5, SUC_lexicon_tags_12 },
    { 0x48ad8ee6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cd8ee9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578d8eed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4d8ef3, SUC_lexicon_tags_7 },
    { 0xc16d8ef3, SUC_lexicon_tags_392 },
    { 0xed4d8ef5, SUC_lexicon_tags_21 },
    { 0xcad8ef6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbad8ef9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0d8efd, SUC_lexicon_tags_37 },
    { 0x290d8efd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cd8f03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ed8f06, SUC_lexicon_tags_23 },
    { 0x716d8f07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x950d8f0a, SUC_lexicon_tags_46 },
    { 0xe5cd8f0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x704d8f0d, SUC_lexicon_tags_26 },
    { 0x5ad8f0e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x58ad8f10, SUC_lexicon_tags_28 },
    { 0x55cd8f10, SUC_lexicon_tags_15 },
    { 0xaa0d8f10, SUC_lexicon_tags_8 },
    { 0x338d8f13, SUC_lexicon_tags_7 },
    { 0x91ad8f14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x148d8f16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcd6d8f18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f4d8f1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d8f1d, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70d8f21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ed8f26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8d8f29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfced8f31, SUC_lexicon_tags_26 },
    { 0x8fed8f32, SUC_lexicon_tags_1 },
    { 0x99cd8f33, SUC_lexicon_tags_49 },
    { 0x2f0d8f34, SUC_lexicon_tags_7 },
    { 0x20ad8f35, SUC_lexicon_tags_63 },
    { 0x9f0d8f36, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76d8f3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366d8f41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x386d8f43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ead8f45, SUC_lexicon_tags_7 },
    { 0x744d8f46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d8f4a, SUC_lexicon_tags_2 },
    { 0x1bcd8f4a, SUC_lexicon_tags_38 },
    { 0xe98d8f4b, SUC_lexicon_tags_43 },
    { 0x41ad8f4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x636d8f50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b6d8f52, SUC_lexicon_tags_80 },
    { 0xe9cd8f52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fed8f55, SUC_lexicon_tags_12 },
    { 0x4acd8f55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dad8f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39ed8f5f, SUC_lexicon_tags_12 },
    { 0x1e0d8f5f, SUC_lexicon_tags_23 },
    { 0xacad8f5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22d8f64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acd8f6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad8f6c, SUC_lexicon_tags_30 },
    { 0x33ed8f6d, SUC_lexicon_tags_14 },
    { 0x4c0d8f6c, SUC_lexicon_tags_1 },
    { 0x5c8d8f6d, SUC_lexicon_tags_23 },
    { 0x7b0d8f6e, SUC_lexicon_tags_46 },
    { 0xddad8f71, SUC_lexicon_tags_25 },
    { 0xe30d8f72, SUC_lexicon_tags_11 },
    { 0x594d8f70, SUC_lexicon_tags_3 },
    { 0x858d8f72, SUC_lexicon_tags_79 },
    { 0x230d8f75, SUC_lexicon_tags_47 },
    { 0x308d8f75, SUC_lexicon_tags_46 },
    { 0x10d8f76, SUC_lexicon_tags_12 },
    { 0x922d8f76, SUC_lexicon_tags_6 },
    { 0x6a0d8f76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556d8f80, SUC_lexicon_tags_27 },
    { 0x6c0d8f81, SUC_lexicon_tags_3 },
    { 0x776d8f81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a0d8f84, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0d8f87, SUC_lexicon_tags_7 },
    { 0xa92d8f87, SUC_lexicon_tags_57 },
    { 0x43ed8f89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42d8f8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53cd8f8e, SUC_lexicon_tags_47 },
    { 0x86ad8f8f, SUC_lexicon_tags_44 },
    { 0x92cd8f8f, SUC_lexicon_tags_57 },
    { 0x54cd8f8e, SUC_lexicon_tags_15 },
    { 0xaced8f91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecd8f96, SUC_lexicon_tags_11 },
    { 0x89ed8f96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x950d8f99, SUC_lexicon_tags_42 },
    { 0x37cd8f9a, SUC_lexicon_tags_45 },
    { 0x2ead8f9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cd8f9f, SUC_lexicon_tags_25 },
    { 0x9d8d8fa0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94d8fa6, SUC_lexicon_tags_2 },
    { 0x14ed8fa6, SUC_lexicon_tags_167 },
    { 0x406d8fa7, SUC_lexicon_tags_1 },
    { 0x9fad8fa6, SUC_lexicon_tags_12 },
    { 0xb72d8fa9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6d8fad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee6d8faf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x156d8fb2, SUC_lexicon_tags_15 },
    { 0x982d8fb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc26d8fb5, SUC_lexicon_tags_42 },
    { 0xa60d8fb6, SUC_lexicon_tags_11 },
    { 0xd64d8fb5, SUC_lexicon_tags_2 },
    { 0xd9cd8fb6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x976d8fbb, SUC_lexicon_tags_61 },
    { 0x70ad8fbc, SUC_lexicon_tags_12 },
    { 0x5c0d8fbd, SUC_lexicon_tags_16 },
    { 0x2cad8fbe, SUC_lexicon_tags_47 },
    { 0xb8d8fbf, SUC_lexicon_tags_7 },
    { 0xfe6d8fc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8d8fc3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x10cd8fc5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4d8fce, SUC_lexicon_tags_1 },
    { 0xc16d8fce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa10d8fd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ad8fd5, SUC_lexicon_tags_16 },
    { 0xbd2d8fd6, SUC_lexicon_tags_15 },
    { 0xd16d8fd7, SUC_lexicon_tags_49 },
    { 0xd6d8fd6, SUC_lexicon_tags_25 },
    { 0xc42d8fd8, SUC_lexicon_tags_3 },
    { 0xeaed8fd7, SUC_lexicon_tags_27 },
    { 0xf6ad8fdb, SUC_lexicon_tags_2 },
    { 0x68cd8fd9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ad8fe3, SUC_lexicon_tags_11 },
    { 0xf84d8fe3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaad8fe9, SUC_lexicon_tags_68 },
    { 0x7dcd8fea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cad8fec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d8ff0, SUC_lexicon_tags_63 },
    { 0x60d8ff1, SUC_lexicon_tags_27 },
    { 0xd4ad8ff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cd8ffe, SUC_lexicon_tags_25 },
    { 0x956d8ffe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290d9004, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x482d9006, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb28d9008, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4f2d900a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4d9010, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ed9015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cd9018, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3b8d901a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22d901e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0d9022, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cd9025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x902d9027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476d902c, SUC_lexicon_tags_25 },
    { 0xd24d902d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bed9037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x554d903a, SUC_lexicon_tags_17 },
    { 0x630d903b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xec6d903e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ecd9040, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x428d9043, SUC_lexicon_tags_19 },
    { 0x6d9043, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa42d9046, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a4d9048, SUC_lexicon_tags_33 },
    { 0x36cd9049, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ed904c, SUC_lexicon_tags_7 },
    { 0xe24d904d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaad9050, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450d9057, SUC_lexicon_tags_31 },
    { 0x5f4d9058, SUC_lexicon_tags_25 },
    { 0x486d9058, SUC_lexicon_tags_43 },
    { 0x3f4d9059, SUC_lexicon_tags_25 },
    { 0x5ed905b, SUC_lexicon_tags_32 },
    { 0xe7cd905b, SUC_lexicon_tags_0 },
    { 0x35ad9059, SUC_lexicon_tags_2 },
    { 0x832d905e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ad9062, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aad9066, SUC_lexicon_tags_27 },
    { 0x610d9067, SUC_lexicon_tags_87 },
    { 0x122d9066, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed906e, SUC_lexicon_tags_7 },
    { 0xe06d906f, SUC_lexicon_tags_1 },
    { 0x5dad9070, SUC_lexicon_tags_12 },
    { 0x9ced9070, SUC_lexicon_tags_12 },
    { 0xa80d9070, SUC_lexicon_tags_6 },
    { 0xd56d9073, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d6d9075, SUC_lexicon_tags_82 },
    { 0x2b2d9076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452d907a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x886d907c, SUC_lexicon_tags_7 },
    { 0x3fad907d, SUC_lexicon_tags_63 },
    { 0x57ad907c, SUC_lexicon_tags_44 },
    { 0x40ad907f, SUC_lexicon_tags_6 },
    { 0xf28d907f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81cd9082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d9086, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ed908a, SUC_lexicon_tags_46 },
    { 0x92ed908a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3c4d908d, SUC_lexicon_tags_19 },
    { 0xa72d908d, SUC_lexicon_tags_1 },
    { 0x50ad908d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x790d9091, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cd9095, SUC_lexicon_tags_33 },
    { 0x1c8d9096, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ad909a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe7ed909c, SUC_lexicon_tags_49 },
    { 0x854d909d, SUC_lexicon_tags_160 },
    { 0xd80d909e, SUC_lexicon_tags_17 },
    { 0x3acd909f, SUC_lexicon_tags_23 },
    { 0x224d90a0, SUC_lexicon_tags_37 },
    { 0xfb4d909c, SUC_lexicon_tags_37 },
    { 0xa36d909d, SUC_lexicon_tags_37 },
    { 0xff6d909e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eed90a8, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446d90ac, SUC_lexicon_tags_17 },
    { 0xbfed90ac, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8d90b3, SUC_lexicon_tags_1 },
    { 0x2c0d90b3, SUC_lexicon_tags_17 },
    { 0x76d90b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44d90b9, SUC_lexicon_tags_46 },
    { 0xf26d90ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6d90c1, SUC_lexicon_tags_7 },
    { 0x15ad90c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216d90c6, SUC_lexicon_tags_27 },
    { 0x962d90c7, SUC_lexicon_tags_7 },
    { 0x97ed90c7, SUC_lexicon_tags_7 },
    { 0x3e2d90c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe90d90cc, SUC_lexicon_tags_9 },
    { 0x402d90cc, SUC_lexicon_tags_9 },
    { 0xfccd90ce, SUC_lexicon_tags_13 },
    { 0x928d90ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d90d6, SUC_lexicon_tags_28 },
    { 0x996d90d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x116d90da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160d90de, SUC_lexicon_tags_38 },
    { 0x270d90df, SUC_lexicon_tags_17 },
    { 0xf22d90e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410d90e6, SUC_lexicon_tags_15 },
    { 0x848d90e7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd56d90e9, SUC_lexicon_tags_7 },
    { 0xcb0d90ea, SUC_lexicon_tags_84 },
    { 0x34cd90eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0d90f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a2d90f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0ad90f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xead90f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x402d90f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3dcd90fb, SUC_lexicon_tags_30 },
    { 0x3a0d90fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2d9101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cd9105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4d910a, SUC_lexicon_tags_11 },
    { 0x892d910b, SUC_lexicon_tags_12 },
    { 0xa80d910b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa94d910e, SUC_lexicon_tags_33 },
    { 0x506d910f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292d9114, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aad911e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396d9123, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ed9126, SUC_lexicon_tags_3 },
    { 0x7b0d9126, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430d9130, SUC_lexicon_tags_1 },
    { 0x92ad9131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccd9135, SUC_lexicon_tags_37 },
    { 0x4f8d9136, SUC_lexicon_tags_18 },
    { 0x4bed9137, SUC_lexicon_tags_21 },
    { 0xe7cd9137, SUC_lexicon_tags_27 },
    { 0x85ed9137, SUC_lexicon_tags_17 },
    { 0xd2d913a, SUC_lexicon_tags_11 },
    { 0x9e0d913a, SUC_lexicon_tags_2 },
    { 0x7c0d913b, SUC_lexicon_tags_2 },
    { 0xf30d9139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d9144, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xee6d9146, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2d9150, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0d9155, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ad915b, SUC_lexicon_tags_12 },
    { 0xb16d915c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa0d915e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cd9161, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4d9169, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cd9172, SUC_lexicon_tags_11 },
    { 0xee8d9173, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0d9176, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2cd9178, SUC_lexicon_tags_65 },
    { 0xdbad9179, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8d9181, SUC_lexicon_tags_70 },
    { 0x7a0d9182, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed9188, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4d918c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d9190, SUC_lexicon_tags_57 },
    { 0xeb4d9190, SUC_lexicon_tags_7 },
    { 0xa52d9192, SUC_lexicon_tags_64 },
    { 0xf08d9191, SUC_lexicon_tags_6 },
    { 0xe20d9193, SUC_lexicon_tags_39 },
    { 0xf6cd9192, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x898d9198, SUC_lexicon_tags_34 },
    { 0x75ad9199, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaed91a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04d91a8, SUC_lexicon_tags_11 },
    { 0x6dad91a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x748d91ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fad91af, SUC_lexicon_tags_33 },
    { 0xf0d91af, SUC_lexicon_tags_79 },
    { 0x88d91b0, SUC_lexicon_tags_66 },
    { 0x106d91b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x920d91b4, SUC_lexicon_tags_13 },
    { 0xe32d91b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x350d91b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d8d91ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcd91bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ad91c2, SUC_lexicon_tags_15 },
    { 0xa2cd91c3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x518d91c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0d91ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xffad91cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ad91cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ad91d5, SUC_lexicon_tags_12 },
    { 0x678d91d6, SUC_lexicon_tags_23 },
    { 0x6cad91d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x87ad91d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc2d91db, SUC_lexicon_tags_12 },
    { 0x8eed91db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x870d91de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8d91e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x364d91e6, SUC_lexicon_tags_12 },
    { 0x3ecd91e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbded91eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fad91f3, SUC_lexicon_tags_29 },
    { 0x186d91f3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392d91f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ded91fb, SUC_lexicon_tags_12 },
    { 0xd28d91fb, SUC_lexicon_tags_25 },
    { 0x68d91fb, SUC_lexicon_tags_19 },
    { 0xec8d91fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9203, SUC_lexicon_tags_12 },
    { 0xfd8d9204, SUC_lexicon_tags_82 },
    { 0x782d9205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8d9209, SUC_lexicon_tags_37 },
    { 0x244d9209, SUC_lexicon_tags_61 },
    { 0x34ed920a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x110d920d, SUC_lexicon_tags_2 },
    { 0x762d920e, SUC_lexicon_tags_7 },
    { 0x1a2d920f, SUC_lexicon_tags_48 },
    { 0xb60d920f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612d921a, SUC_lexicon_tags_14 },
    { 0x2a2d921b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ad921f, SUC_lexicon_tags_51 },
    { 0xc72d921f, SUC_lexicon_tags_6 },
    { 0x676d9221, SUC_lexicon_tags_37 },
    { 0x7f4d9222, SUC_lexicon_tags_37 },
    { 0x29ad9221, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x760d9226, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86d922e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd64d9230, SUC_lexicon_tags_1 },
    { 0xcf0d9231, SUC_lexicon_tags_23 },
    { 0xaaed9232, SUC_lexicon_tags_63 },
    { 0x902d9233, SUC_lexicon_tags_42 },
    { 0x34d9232, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcd923c, SUC_lexicon_tags_17 },
    { 0x20ad923c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0d9243, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa92d9245, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfed9247, SUC_lexicon_tags_17 },
    { 0x484d9247, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ad924f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d9255, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752d9259, SUC_lexicon_tags_65 },
    { 0x20ed925a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cd9263, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90d9268, SUC_lexicon_tags_1 },
    { 0xabed9269, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280d926e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x490d9271, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6d9274, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d4d9276, SUC_lexicon_tags_12 },
    { 0xc2ad9277, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x582d9279, SUC_lexicon_tags_7 },
    { 0x4c4d927a, SUC_lexicon_tags_19 },
    { 0x9f2d927a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cd9281, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112d9285, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0d9289, SUC_lexicon_tags_33 },
    { 0x6aed928a, SUC_lexicon_tags_31 },
    { 0x260d928b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5ad928d, SUC_lexicon_tags_48 },
    { 0x1c0d928e, SUC_lexicon_tags_46 },
    { 0xf68d928f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804d9295, SUC_lexicon_tags_60 },
    { 0xd06d9296, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe26d9298, SUC_lexicon_tags_37 },
    { 0x7f0d9299, SUC_lexicon_tags_85 },
    { 0xaacd9299, SUC_lexicon_tags_17 },
    { 0xe6ed929b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x52ad929d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4dcd929f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f6d92a1, SUC_lexicon_tags_15 },
    { 0x8ccd92a1, SUC_lexicon_tags_14 },
    { 0x7a6d92a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc2d92a5, SUC_lexicon_tags_27 },
    { 0xbc8d92a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726d92ad, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcd92b3, SUC_lexicon_tags_15 },
    { 0xed92b4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x35ad92b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e4d92b8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ed92bd, SUC_lexicon_tags_12 },
    { 0xc14d92be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ed92c5, SUC_lexicon_tags_17 },
    { 0x1f6d92c6, SUC_lexicon_tags_37 },
    { 0x1ced92c7, SUC_lexicon_tags_37 },
    { 0x92ad92c8, SUC_lexicon_tags_10 },
    { 0xd46d92c9, SUC_lexicon_tags_2 },
    { 0x9c4d92c9, SUC_lexicon_tags_12 },
    { 0xfbad92cb, SUC_lexicon_tags_7 },
    { 0xeed92cb, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ad92d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20d92d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37cd92d6, SUC_lexicon_tags_14 },
    { 0xe66d92d6, SUC_lexicon_tags_11 },
    { 0x26d92d8, SUC_lexicon_tags_0 },
    { 0x2d6d92d6, SUC_lexicon_tags_342 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad92dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2d92e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeced92e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83cd92eb, SUC_lexicon_tags_42 },
    { 0xa08d92ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e8d92ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6cd92f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962d92f5, SUC_lexicon_tags_27 },
    { 0x1cd92f6, SUC_lexicon_tags_17 },
    { 0x460d92f6, SUC_lexicon_tags_84 },
    { 0x356d92f7, SUC_lexicon_tags_29 },
    { 0xf68d92f9, SUC_lexicon_tags_68 },
    { 0xa2d92fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed9302, SUC_lexicon_tags_1 },
    { 0xf56d9303, SUC_lexicon_tags_25 },
    { 0xb4d9304, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x310d9306, SUC_lexicon_tags_60 },
    { 0x80d9307, SUC_lexicon_tags_48 },
    { 0x66d9307, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x87cd930a, SUC_lexicon_tags_3 },
    { 0xfecd930a, SUC_lexicon_tags_38 },
    { 0x320d930c, SUC_lexicon_tags_7 },
    { 0x426d930d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922d9311, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34d9316, SUC_lexicon_tags_37 },
    { 0xa6d9316, SUC_lexicon_tags_1 },
    { 0x8e2d9316, SUC_lexicon_tags_12 },
    { 0xe4ed9319, SUC_lexicon_tags_15 },
    { 0x188d931a, SUC_lexicon_tags_44 },
    { 0xe6cd931a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a6d931d, SUC_lexicon_tags_57 },
    { 0x876d931d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ad9323, SUC_lexicon_tags_25 },
    { 0xf06d9323, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb24d9326, SUC_lexicon_tags_2 },
    { 0x440d9327, SUC_lexicon_tags_9 },
    { 0xc52d9326, SUC_lexicon_tags_15 },
    { 0xc02d9327, SUC_lexicon_tags_7 },
    { 0x5ad9326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd06d932c, SUC_lexicon_tags_37 },
    { 0x694d932d, SUC_lexicon_tags_15 },
    { 0xebed932c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35ad9330, SUC_lexicon_tags_17 },
    { 0xbc6d9331, SUC_lexicon_tags_45 },
    { 0x2b8d9332, SUC_lexicon_tags_17 },
    { 0x6ead9333, SUC_lexicon_tags_13 },
    { 0x80cd9334, SUC_lexicon_tags_32 },
    { 0xf20d9335, SUC_lexicon_tags_10 },
    { 0xddcd9336, SUC_lexicon_tags_25 },
    { 0xc9ed9336, SUC_lexicon_tags_87 },
    { 0xa8ed9336, SUC_lexicon_tags_60 },
    { 0xf16d9330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0d933b, SUC_lexicon_tags_0 },
    { 0x23ed933c, SUC_lexicon_tags_17 },
    { 0xbb4d933d, SUC_lexicon_tags_7 },
    { 0xc84d933e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206d9343, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478d9349, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x792d934b, SUC_lexicon_tags_12 },
    { 0x9ead934c, SUC_lexicon_tags_27 },
    { 0xbf4d934d, SUC_lexicon_tags_8 },
    { 0xbb2d934d, SUC_lexicon_tags_37 },
    { 0x8bed934e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcd9352, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x570d9355, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16d935a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552d9361, SUC_lexicon_tags_62 },
    { 0x25cd9362, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8d9365, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2cd9367, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7b6d9369, SUC_lexicon_tags_18 },
    { 0x10ad9369, SUC_lexicon_tags_12 },
    { 0x8fcd936b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4cad936d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cd9372, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cd9377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04d937a, SUC_lexicon_tags_7 },
    { 0x5b4d937b, SUC_lexicon_tags_88 },
    { 0x148d937b, SUC_lexicon_tags_77 },
    { 0xb8cd937d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4e2d937f, SUC_lexicon_tags_19 },
    { 0x226d937f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860d9388, SUC_lexicon_tags_77 },
    { 0xe58d9388, SUC_lexicon_tags_66 },
    { 0x1ad938a, SUC_lexicon_tags_25 },
    { 0x1ced938b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd00d938d, SUC_lexicon_tags_15 },
    { 0x8fad938e, SUC_lexicon_tags_23 },
    { 0xf3cd938f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850d9393, SUC_lexicon_tags_10 },
    { 0x4fad9394, SUC_lexicon_tags_154 },
    { 0x18ed9395, SUC_lexicon_tags_37 },
    { 0x936d9395, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc0cd9398, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6c4d939a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ed939d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcc4d939f, SUC_lexicon_tags_11 },
    { 0x920d93a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x788d93a3, SUC_lexicon_tags_17 },
    { 0x390d93a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4d93af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58d93b3, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ed93b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x124d93bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6d93bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x300d93c1, SUC_lexicon_tags_17 },
    { 0x800d93c2, SUC_lexicon_tags_826 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ad93c5, SUC_lexicon_tags_16 },
    { 0x20ed93c6, SUC_lexicon_tags_0 },
    { 0x154d93c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ad93cb, SUC_lexicon_tags_15 },
    { 0x45ad93cc, SUC_lexicon_tags_473 },
    { 0x3acd93cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8d93d0, SUC_lexicon_tags_7 },
    { 0x178d93d0, SUC_lexicon_tags_11 },
    { 0x9f8d93d2, SUC_lexicon_tags_14 },
    { 0xfa4d93d0, SUC_lexicon_tags_2 },
    { 0x61ed93d1, SUC_lexicon_tags_17 },
    { 0x756d93d5, SUC_lexicon_tags_4 },
    { 0x490d93d6, SUC_lexicon_tags_8 },
    { 0x7b6d93d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd6d93d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2d93de, SUC_lexicon_tags_11 },
    { 0xf16d93de, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4d93e2, SUC_lexicon_tags_152 },
    { 0xf96d93e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710d93e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ed93ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236d93f4, SUC_lexicon_tags_7 },
    { 0xfb6d93f5, SUC_lexicon_tags_12 },
    { 0xcb0d93f6, SUC_lexicon_tags_30 },
    { 0xd18d93f6, SUC_lexicon_tags_37 },
    { 0xc56d93f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0d93fe, SUC_lexicon_tags_2 },
    { 0x226d93ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ded9402, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8f2d9404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ed940b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x654d940e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0d9415, SUC_lexicon_tags_45 },
    { 0x87cd9416, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbad941b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668d9421, SUC_lexicon_tags_60 },
    { 0xda4d9422, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770d9426, SUC_lexicon_tags_12 },
    { 0x4ecd9426, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad942d, SUC_lexicon_tags_11 },
    { 0x898d942e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65cd9431, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58d943b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8d943f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaa4d9441, SUC_lexicon_tags_65 },
    { 0x846d9442, SUC_lexicon_tags_63 },
    { 0x148d9441, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce4d9445, SUC_lexicon_tags_37 },
    { 0xe86d9446, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0d9449, SUC_lexicon_tags_17 },
    { 0xd20d944a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x302d944d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50cd944f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e0d9451, SUC_lexicon_tags_37 },
    { 0x682d9452, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e4d9454, SUC_lexicon_tags_15 },
    { 0x426d9455, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62d945a, SUC_lexicon_tags_31 },
    { 0x6cad945b, SUC_lexicon_tags_26 },
    { 0xcb4d945c, SUC_lexicon_tags_12 },
    { 0xfad945c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaced9461, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d9464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x496d9467, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecd946a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa34d946c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98d946f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0d947c, SUC_lexicon_tags_62 },
    { 0x2acd947d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4d9482, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x47ed9484, SUC_lexicon_tags_38 },
    { 0xdd8d9484, SUC_lexicon_tags_31 },
    { 0x2e0d9486, SUC_lexicon_tags_17 },
    { 0x958d9485, SUC_lexicon_tags_43 },
    { 0x316d9488, SUC_lexicon_tags_39 },
    { 0x776d9489, SUC_lexicon_tags_30 },
    { 0x1d0d9489, SUC_lexicon_tags_7 },
    { 0x7c4d9489, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aed948f, SUC_lexicon_tags_71 },
    { 0xd10d9490, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2d9494, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe52d9496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d949a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x944d949c, SUC_lexicon_tags_17 },
    { 0x1f8d949c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce4d949f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54cd94a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428d94a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcd94af, SUC_lexicon_tags_2 },
    { 0x82d94b0, SUC_lexicon_tags_25 },
    { 0xdd0d94b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x576d94b4, SUC_lexicon_tags_47 },
    { 0x8b6d94b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4d94b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e6d94ba, SUC_lexicon_tags_29 },
    { 0x6fad94bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ad94be, SUC_lexicon_tags_63 },
    { 0xc20d94bf, SUC_lexicon_tags_26 },
    { 0xfaed94be, SUC_lexicon_tags_12 },
    { 0x624d94c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166d94c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f0d94c7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ad94cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x10ed94ce, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xaf8d94d0, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0d94d9, SUC_lexicon_tags_1 },
    { 0x71ed94da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36ed94dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x320d94df, SUC_lexicon_tags_87 },
    { 0xd24d94e0, SUC_lexicon_tags_39 },
    { 0x1a4d94e1, SUC_lexicon_tags_60 },
    { 0xe46d94e1, SUC_lexicon_tags_27 },
    { 0x582d94e3, SUC_lexicon_tags_12 },
    { 0x642d94e4, SUC_lexicon_tags_28 },
    { 0xef4d94e5, SUC_lexicon_tags_23 },
    { 0x424d94e5, SUC_lexicon_tags_12 },
    { 0x6eed94e5, SUC_lexicon_tags_7 },
    { 0x5ad94e1, SUC_lexicon_tags_19 },
    { 0x256d94e9, SUC_lexicon_tags_55 },
    { 0x5ccd94e9, SUC_lexicon_tags_49 },
    { 0xaed94e9, SUC_lexicon_tags_11 },
    { 0x294d94eb, SUC_lexicon_tags_25 },
    { 0x644d94ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c8d94ef, SUC_lexicon_tags_7 },
    { 0xa24d94ef, SUC_lexicon_tags_11 },
    { 0x6b6d94f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x422d94f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6ad94f5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ad94f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dad94fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x594d94fd, SUC_lexicon_tags_9 },
    { 0x602d94fe, SUC_lexicon_tags_6 },
    { 0xb0d94ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb2d9501, SUC_lexicon_tags_0 },
    { 0x486d9501, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18ad9504, SUC_lexicon_tags_25 },
    { 0x6a4d9504, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0d950e, SUC_lexicon_tags_84 },
    { 0x762d950e, SUC_lexicon_tags_26 },
    { 0x32cd9510, SUC_lexicon_tags_37 },
    { 0xec6d9511, SUC_lexicon_tags_7 },
    { 0x4aed9511, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa4d9514, SUC_lexicon_tags_113 },
    { 0xe54d9515, SUC_lexicon_tags_21 },
    { 0xbfcd9516, SUC_lexicon_tags_37 },
    { 0x162d9517, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cd951c, SUC_lexicon_tags_43 },
    { 0xb78d951d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd86d951f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2d9523, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbaad9525, SUC_lexicon_tags_33 },
    { 0xb02d9525, SUC_lexicon_tags_63 },
    { 0xe46d9527, SUC_lexicon_tags_28 },
    { 0xfa0d9527, SUC_lexicon_tags_1 },
    { 0xe66d9528, SUC_lexicon_tags_7 },
    { 0x966d952a, SUC_lexicon_tags_4 },
    { 0x942d952a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x450d952d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ed9530, SUC_lexicon_tags_60 },
    { 0xeaad9531, SUC_lexicon_tags_37 },
    { 0x9ecd9532, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922d9537, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8a6d9539, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa06d953b, SUC_lexicon_tags_63 },
    { 0x628d953c, SUC_lexicon_tags_11 },
    { 0x870d953b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2d9540, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2bad9542, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26d9545, SUC_lexicon_tags_689 },
    { 0xea0d9546, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47cd9548, SUC_lexicon_tags_10 },
    { 0xa2ed9548, SUC_lexicon_tags_12 },
    { 0xc96d9548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ad954e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c8d9550, SUC_lexicon_tags_15 },
    { 0x1ded9550, SUC_lexicon_tags_37 },
    { 0x78d9552, SUC_lexicon_tags_25 },
    { 0x85ed9552, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd9556, SUC_lexicon_tags_31 },
    { 0x8dcd9556, SUC_lexicon_tags_17 },
    { 0x2f0d9558, SUC_lexicon_tags_7 },
    { 0xe3ad9556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa60d955c, SUC_lexicon_tags_46 },
    { 0xdacd955d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0d9560, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8ed9562, SUC_lexicon_tags_12 },
    { 0xa00d9563, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc40d9565, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66d9569, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x81ad956b, SUC_lexicon_tags_15 },
    { 0xa76d956c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f8d956e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x108d9570, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6d9573, SUC_lexicon_tags_7 },
    { 0xd6d9574, SUC_lexicon_tags_79 },
    { 0xb2ad9574, SUC_lexicon_tags_0 },
    { 0x6e0d9574, SUC_lexicon_tags_38 },
    { 0x51ad9577, SUC_lexicon_tags_10 },
    { 0xc12d9578, SUC_lexicon_tags_15 },
    { 0xf40d9579, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bcd957b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642d957f, SUC_lexicon_tags_47 },
    { 0x912d957f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdfad9582, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2d958e, SUC_lexicon_tags_17 },
    { 0x234d958f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cad9592, SUC_lexicon_tags_11 },
    { 0x9e0d9592, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x922d9595, SUC_lexicon_tags_17 },
    { 0x5eed9596, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7cd9598, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29ed959a, SUC_lexicon_tags_31 },
    { 0x3b2d959b, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0d95a3, SUC_lexicon_tags_34 },
    { 0x706d95a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0d95a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcd95ab, SUC_lexicon_tags_21 },
    { 0xfe0d95ac, SUC_lexicon_tags_0 },
    { 0x354d95ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2d95b4, SUC_lexicon_tags_11 },
    { 0x50ed95b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bed95be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2d95c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16d95c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a8d95c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908d95cd, SUC_lexicon_tags_25 },
    { 0x91cd95ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6d95d2, SUC_lexicon_tags_2 },
    { 0xb74d95d3, SUC_lexicon_tags_15 },
    { 0xc18d95d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d95d8, SUC_lexicon_tags_37 },
    { 0xa7ad95d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650d95e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x28ad95e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830d95e8, SUC_lexicon_tags_65 },
    { 0xec6d95e9, SUC_lexicon_tags_11 },
    { 0x1d2d95e8, SUC_lexicon_tags_37 },
    { 0x10d95eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bad95f6, SUC_lexicon_tags_37 },
    { 0x812d95f7, SUC_lexicon_tags_17 },
    { 0x650d95f8, SUC_lexicon_tags_2 },
    { 0x556d95f9, SUC_lexicon_tags_47 },
    { 0xcfed95f7, SUC_lexicon_tags_13 },
    { 0xd46d95fb, SUC_lexicon_tags_16 },
    { 0x7fad95fc, SUC_lexicon_tags_37 },
    { 0xda4d95f7, SUC_lexicon_tags_19 },
    { 0x67cd95f7, SUC_lexicon_tags_11 },
    { 0x27ad95ff, SUC_lexicon_tags_17 },
    { 0x7b4d9600, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04d9603, SUC_lexicon_tags_29 },
    { 0x52d9603, SUC_lexicon_tags_45 },
    { 0x63cd9603, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x206d9607, SUC_lexicon_tags_2 },
    { 0xbc6d9608, SUC_lexicon_tags_45 },
    { 0x69ed9609, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x818d960b, SUC_lexicon_tags_6 },
    { 0x7e4d960c, SUC_lexicon_tags_11 },
    { 0x95ad960c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed9610, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24d961b, SUC_lexicon_tags_2 },
    { 0xef0d961b, SUC_lexicon_tags_25 },
    { 0x9ed961d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eed9620, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5bed9622, SUC_lexicon_tags_37 },
    { 0x2aed9623, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb00d9625, SUC_lexicon_tags_37 },
    { 0x90ad9626, SUC_lexicon_tags_32 },
    { 0x848d9627, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ad962a, SUC_lexicon_tags_1 },
    { 0x3d4d962b, SUC_lexicon_tags_11 },
    { 0xbb0d962c, SUC_lexicon_tags_17 },
    { 0xeacd962b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c0d962f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x550d9632, SUC_lexicon_tags_19 },
    { 0xaad9632, SUC_lexicon_tags_55 },
    { 0x9b0d9634, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ed9638, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204d963c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4d9640, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4d9644, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cad9647, SUC_lexicon_tags_3 },
    { 0xed4d9648, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d964b, SUC_lexicon_tags_68 },
    { 0xeaed964c, SUC_lexicon_tags_68 },
    { 0x160d964d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22d9653, SUC_lexicon_tags_70 },
    { 0x10d9654, SUC_lexicon_tags_84 },
    { 0xbc8d9655, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cd9659, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4d965c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0d9660, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6d9663, SUC_lexicon_tags_7 },
    { 0x1c8d9663, SUC_lexicon_tags_33 },
    { 0xb12d9665, SUC_lexicon_tags_15 },
    { 0xdad9665, SUC_lexicon_tags_0 },
    { 0x1a4d9667, SUC_lexicon_tags_15 },
    { 0x778d9665, SUC_lexicon_tags_1 },
    { 0xb12d9669, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x838d966c, SUC_lexicon_tags_6 },
    { 0x71ed966c, SUC_lexicon_tags_17 },
    { 0x79ad966e, SUC_lexicon_tags_8 },
    { 0x926d966f, SUC_lexicon_tags_12 },
    { 0x888d966f, SUC_lexicon_tags_25 },
    { 0x67ad966f, SUC_lexicon_tags_43 },
    { 0xa8ad966f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54d9675, SUC_lexicon_tags_37 },
    { 0x568d9675, SUC_lexicon_tags_31 },
    { 0x182d9677, SUC_lexicon_tags_9 },
    { 0x632d9677, SUC_lexicon_tags_15 },
    { 0x29ad9677, SUC_lexicon_tags_19 },
    { 0xed2d9677, SUC_lexicon_tags_63 },
    { 0x15cd9676, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f4d967d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8d9683, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ad968a, SUC_lexicon_tags_18 },
    { 0x5d0d968b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132d968f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcd9693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x508d9696, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22d969c, SUC_lexicon_tags_63 },
    { 0x636d969c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ad96a7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4c8d96a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4d96ac, SUC_lexicon_tags_37 },
    { 0xf68d96ac, SUC_lexicon_tags_14 },
    { 0xdf6d96ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ed96b2, SUC_lexicon_tags_20 },
    { 0x87cd96b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4d96b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaad96ba, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x36ed96bc, SUC_lexicon_tags_8 },
    { 0xe48d96bd, SUC_lexicon_tags_25 },
    { 0xe3cd96be, SUC_lexicon_tags_11 },
    { 0x1f0d96bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ed96c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8d96c7, SUC_lexicon_tags_37 },
    { 0x394d96c7, SUC_lexicon_tags_11 },
    { 0xcd4d96c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0d96cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7e2d96ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504d96d2, SUC_lexicon_tags_26 },
    { 0xe9ed96d2, SUC_lexicon_tags_11 },
    { 0x1ced96d4, SUC_lexicon_tags_11 },
    { 0x84cd96d4, SUC_lexicon_tags_63 },
    { 0x40ed96d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x996d96d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x452d96db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132d96e3, SUC_lexicon_tags_47 },
    { 0xf16d96e4, SUC_lexicon_tags_14 },
    { 0x782d96e4, SUC_lexicon_tags_21 },
    { 0xe68d96e5, SUC_lexicon_tags_17 },
    { 0x72ad96e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf68d96e9, SUC_lexicon_tags_2 },
    { 0xd28d96e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeed96ed, SUC_lexicon_tags_39 },
    { 0x686d96ed, SUC_lexicon_tags_23 },
    { 0x872d96ef, SUC_lexicon_tags_70 },
    { 0x334d96ed, SUC_lexicon_tags_60 },
    { 0x924d96f1, SUC_lexicon_tags_19 },
    { 0xb7cd96f2, SUC_lexicon_tags_1 },
    { 0x482d96ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18d96f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cd96fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ad9700, SUC_lexicon_tags_6 },
    { 0x8ead9701, SUC_lexicon_tags_17 },
    { 0x8c0d9702, SUC_lexicon_tags_4 },
    { 0xa34d9703, SUC_lexicon_tags_62 },
    { 0x47cd9701, SUC_lexicon_tags_63 },
    { 0x628d9704, SUC_lexicon_tags_16 },
    { 0x7ded9706, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366d970b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2d9710, SUC_lexicon_tags_49 },
    { 0x36ed9711, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450d9717, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d971c, SUC_lexicon_tags_12 },
    { 0xd60d971d, SUC_lexicon_tags_31 },
    { 0x10cd971e, SUC_lexicon_tags_23 },
    { 0xf2ed971e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d9725, SUC_lexicon_tags_14 },
    { 0xbc4d9726, SUC_lexicon_tags_57 },
    { 0x430d9727, SUC_lexicon_tags_11 },
    { 0x9a6d9728, SUC_lexicon_tags_38 },
    { 0x220d9729, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xefad972b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x944d972d, SUC_lexicon_tags_11 },
    { 0xd82d972d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a4d9730, SUC_lexicon_tags_2 },
    { 0x9acd9731, SUC_lexicon_tags_15 },
    { 0xf70d9732, SUC_lexicon_tags_26 },
    { 0xd4ad9730, SUC_lexicon_tags_37 },
    { 0x28ed9733, SUC_lexicon_tags_56 },
    { 0xc40d9735, SUC_lexicon_tags_63 },
    { 0x4a6d9736, SUC_lexicon_tags_1 },
    { 0x708d9737, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4d973d, SUC_lexicon_tags_2 },
    { 0xc2ad973e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2d9747, SUC_lexicon_tags_45 },
    { 0xa16d9747, SUC_lexicon_tags_1 },
    { 0xde0d9749, SUC_lexicon_tags_42 },
    { 0xa8d974a, SUC_lexicon_tags_43 },
    { 0x840d974a, SUC_lexicon_tags_7 },
    { 0x630d974c, SUC_lexicon_tags_79 },
    { 0xeaed974d, SUC_lexicon_tags_15 },
    { 0x17ad974b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ad9753, SUC_lexicon_tags_32 },
    { 0x180d9754, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xe0d9756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc54d9758, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d975b, SUC_lexicon_tags_14 },
    { 0x9f6d975c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352d9762, SUC_lexicon_tags_7 },
    { 0xb0cd9763, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d9766, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4d976d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x778d976f, SUC_lexicon_tags_43 },
    { 0xd82d9770, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390d9774, SUC_lexicon_tags_37 },
    { 0x368d9775, SUC_lexicon_tags_1 },
    { 0xd30d9775, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238d977a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fad977d, SUC_lexicon_tags_11 },
    { 0xa76d977e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x628d9780, SUC_lexicon_tags_37 },
    { 0xc30d9781, SUC_lexicon_tags_21 },
    { 0xc60d9781, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28d9785, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128d978b, SUC_lexicon_tags_28 },
    { 0x16d978c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7d2d978e, SUC_lexicon_tags_37 },
    { 0x486d978e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad8d9791, SUC_lexicon_tags_1 },
    { 0x5fed9791, SUC_lexicon_tags_60 },
    { 0x764d9793, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x246d9795, SUC_lexicon_tags_1 },
    { 0x5b4d9796, SUC_lexicon_tags_2 },
    { 0x8ad9796, SUC_lexicon_tags_18 },
    { 0x1a0d9798, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2d97a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6d97a6, SUC_lexicon_tags_57 },
    { 0x7cd97a6, SUC_lexicon_tags_33 },
    { 0xacd97a7, SUC_lexicon_tags_69 },
    { 0x9d6d97a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8d97ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80d97af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x474d97b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2fed97b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142d97c0, SUC_lexicon_tags_2 },
    { 0x1c0d97c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4d97cd, SUC_lexicon_tags_29 },
    { 0xc74d97cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x182d97d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf24d97d2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34d97d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0d97d9, SUC_lexicon_tags_1 },
    { 0x2fed97d9, SUC_lexicon_tags_19 },
    { 0xa3ed97db, SUC_lexicon_tags_4 },
    { 0x652d97dc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110d97e0, SUC_lexicon_tags_43 },
    { 0xd2ed97e0, SUC_lexicon_tags_12 },
    { 0xded97e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8d97e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4d97ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334d97f2, SUC_lexicon_tags_1 },
    { 0x30cd97f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf0cd97f5, SUC_lexicon_tags_7 },
    { 0x6b0d97f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8d97fa, SUC_lexicon_tags_19 },
    { 0x69ad97fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ded97fd, SUC_lexicon_tags_66 },
    { 0xfa4d97fe, SUC_lexicon_tags_37 },
    { 0x3ecd97fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5ad9801, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97cd9804, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ed9807, SUC_lexicon_tags_47 },
    { 0xcb0d9808, SUC_lexicon_tags_49 },
    { 0x894d9807, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4d9811, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102d981b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942d981f, SUC_lexicon_tags_43 },
    { 0xcf2d9820, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14d9823, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cad9826, SUC_lexicon_tags_28 },
    { 0x48ad9827, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6d982a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe6d982c, SUC_lexicon_tags_0 },
    { 0x622d982c, SUC_lexicon_tags_28 },
    { 0xc6cd982e, SUC_lexicon_tags_25 },
    { 0x93ad982e, SUC_lexicon_tags_33 },
    { 0x1f4d982d, SUC_lexicon_tags_11 },
    { 0x32cd9831, SUC_lexicon_tags_15 },
    { 0xec4d9832, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcc2d9834, SUC_lexicon_tags_0 },
    { 0x83cd9835, SUC_lexicon_tags_37 },
    { 0xabad9836, SUC_lexicon_tags_19 },
    { 0x1cd9837, SUC_lexicon_tags_42 },
    { 0x296d9837, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93ad983a, SUC_lexicon_tags_30 },
    { 0x5eed983a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2d983d, SUC_lexicon_tags_14 },
    { 0x216d983e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2d9844, SUC_lexicon_tags_38 },
    { 0x248d9845, SUC_lexicon_tags_60 },
    { 0xf22d9846, SUC_lexicon_tags_12 },
    { 0x398d9847, SUC_lexicon_tags_37 },
    { 0x6ad9848, SUC_lexicon_tags_1 },
    { 0xe36d9846, SUC_lexicon_tags_4 },
    { 0xbad984a, SUC_lexicon_tags_13 },
    { 0x5ead9846, SUC_lexicon_tags_11 },
    { 0x1b4d984c, SUC_lexicon_tags_1 },
    { 0xc66d984d, SUC_lexicon_tags_13 },
    { 0x4bcd984e, SUC_lexicon_tags_37 },
    { 0xea6d984d, SUC_lexicon_tags_25 },
    { 0xf1ed984f, SUC_lexicon_tags_6 },
    { 0xfead9851, SUC_lexicon_tags_15 },
    { 0x2ead984f, SUC_lexicon_tags_19 },
    { 0xd00d9849, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeed9857, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa52d9859, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16d9861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ed9864, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450d9869, SUC_lexicon_tags_25 },
    { 0x9f6d986a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcd986f, SUC_lexicon_tags_11 },
    { 0x8aad986f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0d9873, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x11ad9875, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cd987a, SUC_lexicon_tags_15 },
    { 0xeaad987b, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x1a0d987d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428d9883, SUC_lexicon_tags_27 },
    { 0x1d0d9883, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x268d9887, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x428d9889, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2d988d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cd9890, SUC_lexicon_tags_10 },
    { 0x7a2d9890, SUC_lexicon_tags_23 },
    { 0x3b6d9891, SUC_lexicon_tags_11 },
    { 0x298d9893, SUC_lexicon_tags_1 },
    { 0x93ed9893, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aed989a, SUC_lexicon_tags_60 },
    { 0x60d989a, SUC_lexicon_tags_17 },
    { 0x5c4d989b, SUC_lexicon_tags_25 },
    { 0x2bad989b, SUC_lexicon_tags_47 },
    { 0x49ad989e, SUC_lexicon_tags_14 },
    { 0x886d989f, SUC_lexicon_tags_49 },
    { 0x32ad989b, SUC_lexicon_tags_27 },
    { 0x13cd98a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x178d98a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4e2d98a5, SUC_lexicon_tags_11 },
    { 0x5dad98a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ad98aa, SUC_lexicon_tags_63 },
    { 0xe90d98ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d98b0, SUC_lexicon_tags_49 },
    { 0x3c0d98b1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ed98b4, SUC_lexicon_tags_15 },
    { 0x494d98b5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cd98bf, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386d98c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72d98c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6ad98c9, SUC_lexicon_tags_15 },
    { 0xd30d98ca, SUC_lexicon_tags_25 },
    { 0xbf4d98cb, SUC_lexicon_tags_14 },
    { 0xcd8d98cc, SUC_lexicon_tags_33 },
    { 0x34d98cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04d98d1, SUC_lexicon_tags_2 },
    { 0x91ed98d1, SUC_lexicon_tags_12 },
    { 0xb64d98d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x982d98d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf0d98d8, SUC_lexicon_tags_11 },
    { 0xc34d98d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06d98de, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2d98e1, SUC_lexicon_tags_11 },
    { 0x758d98e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71ed98e4, SUC_lexicon_tags_0 },
    { 0x89cd98e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ed98e8, SUC_lexicon_tags_37 },
    { 0x57ad98e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ed98ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8d98f1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d98f5, SUC_lexicon_tags_29 },
    { 0x6b8d98f6, SUC_lexicon_tags_1 },
    { 0xd44d98f7, SUC_lexicon_tags_8 },
    { 0xecd98f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22d98fe, SUC_lexicon_tags_1 },
    { 0x740d98fe, SUC_lexicon_tags_11 },
    { 0x206d9900, SUC_lexicon_tags_47 },
    { 0x886d98fe, SUC_lexicon_tags_14 },
    { 0x1e2d9902, SUC_lexicon_tags_17 },
    { 0x29ad9902, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ad9906, SUC_lexicon_tags_15 },
    { 0xc52d9907, SUC_lexicon_tags_1 },
    { 0xb64d9908, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ed990d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6d9913, SUC_lexicon_tags_45 },
    { 0xc42d9913, SUC_lexicon_tags_39 },
    { 0xbc8d9914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb2d9917, SUC_lexicon_tags_37 },
    { 0x678d9917, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10d991d, SUC_lexicon_tags_60 },
    { 0x8f4d991e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2d9922, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cd9929, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4d992c, SUC_lexicon_tags_37 },
    { 0x76ed992d, SUC_lexicon_tags_1 },
    { 0xce0d992d, SUC_lexicon_tags_9 },
    { 0x16ed992e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0d993d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f0d993f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x246d9942, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5dcd9944, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bad9949, SUC_lexicon_tags_37 },
    { 0x32ad994a, SUC_lexicon_tags_7 },
    { 0x4e4d994a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3bad994d, SUC_lexicon_tags_25 },
    { 0x48d994d, SUC_lexicon_tags_15 },
    { 0xa84d994f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17ed9951, SUC_lexicon_tags_1 },
    { 0xa72d9952, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x386d9954, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ad995b, SUC_lexicon_tags_3 },
    { 0x8e0d995b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfad9964, SUC_lexicon_tags_8 },
    { 0xb88d9964, SUC_lexicon_tags_70 },
    { 0x648d9966, SUC_lexicon_tags_1 },
    { 0x8eed9965, SUC_lexicon_tags_40 },
    { 0x410d9966, SUC_lexicon_tags_11 },
    { 0x1e0d9969, SUC_lexicon_tags_33 },
    { 0x924d9965, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2d996d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ad9972, SUC_lexicon_tags_12 },
    { 0x5cd9972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcd9979, SUC_lexicon_tags_107 },
    { 0x944d997a, SUC_lexicon_tags_25 },
    { 0xaced997b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394d9980, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x918d9982, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x472d9985, SUC_lexicon_tags_28 },
    { 0x656d9986, SUC_lexicon_tags_17 },
    { 0x772d9987, SUC_lexicon_tags_3 },
    { 0x134d9988, SUC_lexicon_tags_19 },
    { 0x7e6d9989, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cd998c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92d998f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x38d9991, SUC_lexicon_tags_60 },
    { 0xfd0d9992, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434d999a, SUC_lexicon_tags_7 },
    { 0xedcd999b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ed999f, SUC_lexicon_tags_2 },
    { 0xdb4d99a0, SUC_lexicon_tags_17 },
    { 0xff8d999f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1d8d99a3, SUC_lexicon_tags_14 },
    { 0x7a6d99a4, SUC_lexicon_tags_2 },
    { 0x97cd99a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cd99a8, SUC_lexicon_tags_25 },
    { 0xb2ad99a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6d99ac, SUC_lexicon_tags_15 },
    { 0x552d99ac, SUC_lexicon_tags_45 },
    { 0x14ed99ac, SUC_lexicon_tags_32 },
    { 0xbf4d99af, SUC_lexicon_tags_12 },
    { 0x840d99af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xac2d99b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6d99b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb42d99b7, SUC_lexicon_tags_38 },
    { 0xe7ed99b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeced99be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdaed99c0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452d99c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614d99c9, SUC_lexicon_tags_66 },
    { 0x6f8d99ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeed99cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf32d99d1, SUC_lexicon_tags_26 },
    { 0xb50d99d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4d99d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ad99d8, SUC_lexicon_tags_17 },
    { 0x1b0d99d9, SUC_lexicon_tags_49 },
    { 0x4a8d99da, SUC_lexicon_tags_17 },
    { 0x7fad99d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x542d99dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x156d99df, SUC_lexicon_tags_33 },
    { 0x836d99e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbed99e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa68d99e4, SUC_lexicon_tags_0 },
    { 0x92cd99e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6d99e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2d99ec, SUC_lexicon_tags_35 },
    { 0xb04d99ec, SUC_lexicon_tags_56 },
    { 0xd52d99ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ead9a01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2d9a06, SUC_lexicon_tags_37 },
    { 0x82ad9a06, SUC_lexicon_tags_11 },
    { 0x76ad9a08, SUC_lexicon_tags_25 },
    { 0x464d9a09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548d9a0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x198d9a0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb70d9a11, SUC_lexicon_tags_7 },
    { 0x758d9a12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6d9a16, SUC_lexicon_tags_47 },
    { 0x67ed9a17, SUC_lexicon_tags_29 },
    { 0x720d9a16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe50d9a1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf38d9a1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65cd9a1f, SUC_lexicon_tags_17 },
    { 0xbc6d9a1f, SUC_lexicon_tags_53 },
    { 0x4d8d9a21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaced9a25, SUC_lexicon_tags_25 },
    { 0x2fed9a25, SUC_lexicon_tags_4 },
    { 0xa0d9a25, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ed9a2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x334d9a2d, SUC_lexicon_tags_15 },
    { 0x598d9a2e, SUC_lexicon_tags_84 },
    { 0xa0cd9a2e, SUC_lexicon_tags_49 },
    { 0xd94d9a30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cd9a38, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6f2d9a3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ed9a40, SUC_lexicon_tags_0 },
    { 0x16ed9a41, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388d9a45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x21cd9a47, SUC_lexicon_tags_11 },
    { 0xc6cd9a48, SUC_lexicon_tags_37 },
    { 0x576d9a49, SUC_lexicon_tags_93 },
    { 0x106d9a49, SUC_lexicon_tags_11 },
    { 0x8fed9a49, SUC_lexicon_tags_14 },
    { 0xf68d9a4c, SUC_lexicon_tags_27 },
    { 0x774d9a4d, SUC_lexicon_tags_151 },
    { 0x56ed9a4d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14d9a52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118d9a5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x360d9a60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948d9a66, SUC_lexicon_tags_25 },
    { 0x384d9a66, SUC_lexicon_tags_7 },
    { 0xe68d9a67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4d9a6d, SUC_lexicon_tags_11 },
    { 0x2f8d9a6d, SUC_lexicon_tags_2 },
    { 0xcd6d9a6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x252d9a71, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8d9a79, SUC_lexicon_tags_975 },
    { 0xa26d9a79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870d9a81, SUC_lexicon_tags_0 },
    { 0x5acd9a81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ad9a86, SUC_lexicon_tags_56 },
    { 0xea0d9a87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x242d9a8a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5fed9a8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d9a92, SUC_lexicon_tags_37 },
    { 0xb5cd9a93, SUC_lexicon_tags_6 },
    { 0xbc2d9a94, SUC_lexicon_tags_12 },
    { 0xfbcd9a95, SUC_lexicon_tags_7 },
    { 0x89cd9a96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cd9a9c, SUC_lexicon_tags_1 },
    { 0xdc8d9a9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99cd9a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ed9aa3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6d9aa9, SUC_lexicon_tags_6 },
    { 0x90cd9aa9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ced9ab1, SUC_lexicon_tags_23 },
    { 0xa3ad9ab1, SUC_lexicon_tags_113 },
    { 0xead9ab3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3d2d9ab5, SUC_lexicon_tags_45 },
    { 0xf50d9ab5, SUC_lexicon_tags_4 },
    { 0x6dcd9ab7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x966d9aba, SUC_lexicon_tags_15 },
    { 0xa46d9abb, SUC_lexicon_tags_38 },
    { 0xeb2d9abc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0d9ac1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92d9ac4, SUC_lexicon_tags_47 },
    { 0x502d9ac5, SUC_lexicon_tags_6 },
    { 0x936d9ac4, SUC_lexicon_tags_26 },
    { 0x7ded9ac4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ead9acf, SUC_lexicon_tags_0 },
    { 0x740d9ad0, SUC_lexicon_tags_4 },
    { 0x770d9ad1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8ed9ad3, SUC_lexicon_tags_15 },
    { 0x472d9ad4, SUC_lexicon_tags_29 },
    { 0xbf4d9ad5, SUC_lexicon_tags_63 },
    { 0xa3ed9ad6, SUC_lexicon_tags_6 },
    { 0xf5cd9ad7, SUC_lexicon_tags_27 },
    { 0x248d9ad3, SUC_lexicon_tags_26 },
    { 0x604d9ad5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cd9adf, SUC_lexicon_tags_31 },
    { 0xf12d9ae0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58d9ae9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x412d9aeb, SUC_lexicon_tags_45 },
    { 0xe12d9aec, SUC_lexicon_tags_31 },
    { 0xb4ad9aec, SUC_lexicon_tags_15 },
    { 0x2c8d9aec, SUC_lexicon_tags_15 },
    { 0x704d9aef, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6d9af2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x200d9af4, SUC_lexicon_tags_25 },
    { 0x24ed9af5, SUC_lexicon_tags_27 },
    { 0x370d9af6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186d9afc, SUC_lexicon_tags_47 },
    { 0xde4d9afc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccd9b03, SUC_lexicon_tags_37 },
    { 0x832d9b03, SUC_lexicon_tags_1 },
    { 0x76cd9b05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42cd9b07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9b0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400d9b0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60cd9b14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34d9b17, SUC_lexicon_tags_37 },
    { 0x758d9b17, SUC_lexicon_tags_17 },
    { 0x372d9b19, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x250d9b1b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x114d9b1e, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838d9b24, SUC_lexicon_tags_82 },
    { 0xbb8d9b25, SUC_lexicon_tags_33 },
    { 0x5b4d9b25, SUC_lexicon_tags_35 },
    { 0x4c4d9b27, SUC_lexicon_tags_14 },
    { 0x97ad9b24, SUC_lexicon_tags_30 },
    { 0x5b6d9b25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe62d9b2b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ed9b2f, SUC_lexicon_tags_4 },
    { 0xa2ad9b2f, SUC_lexicon_tags_34 },
    { 0x340d9b31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16ad9b33, SUC_lexicon_tags_7 },
    { 0x39ad9b34, SUC_lexicon_tags_25 },
    { 0xd7cd9b35, SUC_lexicon_tags_15 },
    { 0x12ed9b33, SUC_lexicon_tags_25 },
    { 0x73ed9b34, SUC_lexicon_tags_7 },
    { 0x2a8d9b33, SUC_lexicon_tags_45 },
    { 0xfc6d9b33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ed9b3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8acd9b40, SUC_lexicon_tags_87 },
    { 0x91ed9b40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46d9b44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196d9b4e, SUC_lexicon_tags_30 },
    { 0x5d0d9b4f, SUC_lexicon_tags_37 },
    { 0x990d9b4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6d9b53, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcd9b5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4d9b63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2d9b66, SUC_lexicon_tags_33 },
    { 0x17cd9b67, SUC_lexicon_tags_1 },
    { 0x66d9b68, SUC_lexicon_tags_47 },
    { 0xe9cd9b67, SUC_lexicon_tags_9 },
    { 0x9dad9b6a, SUC_lexicon_tags_7 },
    { 0xb06d9b6b, SUC_lexicon_tags_6 },
    { 0xde4d9b6c, SUC_lexicon_tags_8 },
    { 0x34d9b6d, SUC_lexicon_tags_9 },
    { 0xfdad9b6b, SUC_lexicon_tags_25 },
    { 0x384d9b67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4d9b72, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x336d9b74, SUC_lexicon_tags_14 },
    { 0xb96d9b75, SUC_lexicon_tags_11 },
    { 0xcd4d9b74, SUC_lexicon_tags_54 },
    { 0x436d9b74, SUC_lexicon_tags_7 },
    { 0x20ad9b74, SUC_lexicon_tags_27 },
    { 0x8d9b79, SUC_lexicon_tags_88 },
    { 0xb70d9b78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4ad9b7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84d9b83, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0d9b86, SUC_lexicon_tags_7 },
    { 0xe18d9b87, SUC_lexicon_tags_12 },
    { 0x15ed9b87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168d9b8f, SUC_lexicon_tags_1 },
    { 0xcc4d9b8f, SUC_lexicon_tags_17 },
    { 0xf68d9b91, SUC_lexicon_tags_28 },
    { 0xaa0d9b92, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc98d9b94, SUC_lexicon_tags_18 },
    { 0x9b6d9b95, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4a8d9b97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ed9b9a, SUC_lexicon_tags_30 },
    { 0x37cd9b9b, SUC_lexicon_tags_1 },
    { 0x328d9b9c, SUC_lexicon_tags_49 },
    { 0xe74d9b9a, SUC_lexicon_tags_34 },
    { 0x732d9b9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x520d9ba1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cd9ba7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0d9baa, SUC_lexicon_tags_15 },
    { 0x24ad9bab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x968d9bad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2d9bb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30ed9bb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94d9bb7, SUC_lexicon_tags_23 },
    { 0x206d9bb7, SUC_lexicon_tags_15 },
    { 0xfc4d9bb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ed9bc1, SUC_lexicon_tags_1 },
    { 0x2f6d9bc2, SUC_lexicon_tags_11 },
    { 0xe3ad9bc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868d9bc8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x83ed9bca, SUC_lexicon_tags_4 },
    { 0xad9bcb, SUC_lexicon_tags_17 },
    { 0xc08d9bcc, SUC_lexicon_tags_32 },
    { 0xf90d9bcd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcced9bcf, SUC_lexicon_tags_15 },
    { 0xb2ad9bd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902d9bd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924d9bd8, SUC_lexicon_tags_63 },
    { 0xfe6d9bd9, SUC_lexicon_tags_45 },
    { 0x762d9bd9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cd9bdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa34d9bdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf4d9be1, SUC_lexicon_tags_11 },
    { 0xbd8d9be2, SUC_lexicon_tags_12 },
    { 0xd2d9be2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aed9be7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920d9bf3, SUC_lexicon_tags_7 },
    { 0x7a8d9bf4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ed9bf8, SUC_lexicon_tags_63 },
    { 0xa44d9bf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d0d9bfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2d9c03, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe06d9c05, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa66d9c07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ad9c0a, SUC_lexicon_tags_17 },
    { 0xc08d9c0b, SUC_lexicon_tags_1 },
    { 0x1aed9c0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80d9c10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x720d9c12, SUC_lexicon_tags_17 },
    { 0x722d9c12, SUC_lexicon_tags_2 },
    { 0x640d9c12, SUC_lexicon_tags_1 },
    { 0x822d9c13, SUC_lexicon_tags_17 },
    { 0xe90d9c16, SUC_lexicon_tags_25 },
    { 0x714d9c13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad9c1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0d9c24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84d9c26, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x62ad9c28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bad9c2c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcd9c36, SUC_lexicon_tags_60 },
    { 0x2c0d9c37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ed9c3f, SUC_lexicon_tags_63 },
    { 0x610d9c40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806d9c44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cd9c49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fad9c4d, SUC_lexicon_tags_17 },
    { 0x85cd9c4d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6d9c57, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872d9c5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x352d9c60, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6d9c63, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed9c67, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26d9c6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b6d9c6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x872d9c71, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ed9c75, SUC_lexicon_tags_25 },
    { 0xdbcd9c76, SUC_lexicon_tags_1 },
    { 0xabed9c77, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xb44d9c79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36ad9c7b, SUC_lexicon_tags_15 },
    { 0xc92d9c7b, SUC_lexicon_tags_80 },
    { 0xe6d9c7b, SUC_lexicon_tags_60 },
    { 0x58cd9c7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16d9c81, SUC_lexicon_tags_47 },
    { 0x7d6d9c82, SUC_lexicon_tags_46 },
    { 0x850d9c83, SUC_lexicon_tags_11 },
    { 0x8c6d9c84, SUC_lexicon_tags_25 },
    { 0x61cd9c85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x65ed9c87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98ed9c89, SUC_lexicon_tags_23 },
    { 0x3fcd9c8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cd9c8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334d9c96, SUC_lexicon_tags_47 },
    { 0xce8d9c96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ad9c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbacd9c9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4d9ca1, SUC_lexicon_tags_8 },
    { 0x138d9ca2, SUC_lexicon_tags_17 },
    { 0x1b2d9ca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258d9ca9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ad9cae, SUC_lexicon_tags_66 },
    { 0x532d9caf, SUC_lexicon_tags_19 },
    { 0xcb6d9cb0, SUC_lexicon_tags_47 },
    { 0x5f6d9cb1, SUC_lexicon_tags_25 },
    { 0x4aad9cb2, SUC_lexicon_tags_66 },
    { 0x104d9cb3, SUC_lexicon_tags_15 },
    { 0x5ed9cb3, SUC_lexicon_tags_1 },
    { 0x8b0d9cb5, SUC_lexicon_tags_2 },
    { 0x208d9cb3, SUC_lexicon_tags_27 },
    { 0x5f8d9cb3, SUC_lexicon_tags_37 },
    { 0xb44d9cb8, SUC_lexicon_tags_27 },
    { 0x344d9cb9, SUC_lexicon_tags_37 },
    { 0x560d9cb8, SUC_lexicon_tags_7 },
    { 0x124d9cbb, SUC_lexicon_tags_39 },
    { 0xebed9cb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00d9cc1, SUC_lexicon_tags_11 },
    { 0x268d9cc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe98d9cc4, SUC_lexicon_tags_17 },
    { 0x83ed9cc5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44d9cc8, SUC_lexicon_tags_1 },
    { 0x476d9cc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x36cd9ccc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d9cd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ed9cd4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2d9cd8, SUC_lexicon_tags_15 },
    { 0x7fad9cd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x968d9cdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382d9ce1, SUC_lexicon_tags_17 },
    { 0x55cd9ce2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2fad9ce4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x77cd9ce6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd0d9ce8, SUC_lexicon_tags_137 },
    { 0, NULL },
    { 0xb1ad9cea, SUC_lexicon_tags_60 },
    { 0x866d9ceb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ed9cee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacd9cf5, SUC_lexicon_tags_17 },
    { 0x290d9cf5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x130d9cf8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160d9cfc, SUC_lexicon_tags_9 },
    { 0xf40d9cfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36d9d00, SUC_lexicon_tags_37 },
    { 0x1a2d9d01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4d9d04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdead9d0a, SUC_lexicon_tags_26 },
    { 0x6f2d9d0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfc6d9d0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x92d9d0f, SUC_lexicon_tags_35 },
    { 0xd6ed9d0f, SUC_lexicon_tags_25 },
    { 0x58ed9d10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976d9d15, SUC_lexicon_tags_25 },
    { 0xe68d9d16, SUC_lexicon_tags_1 },
    { 0xe7ed9d17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ed9d1b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ed9d20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0ed9d22, SUC_lexicon_tags_46 },
    { 0x13ed9d22, SUC_lexicon_tags_15 },
    { 0x572d9d23, SUC_lexicon_tags_28 },
    { 0x268d9d23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdad9d2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230d9d35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ded9d37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccd9d3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4ed9d3c, SUC_lexicon_tags_2 },
    { 0xd3ed9d3c, SUC_lexicon_tags_27 },
    { 0x640d9d3e, SUC_lexicon_tags_37 },
    { 0xb58d9d3e, SUC_lexicon_tags_1 },
    { 0x722d9d40, SUC_lexicon_tags_1 },
    { 0xacd9d3d, SUC_lexicon_tags_4 },
    { 0x1dad9d42, SUC_lexicon_tags_33 },
    { 0x154d9d3f, SUC_lexicon_tags_7 },
    { 0x53ed9d44, SUC_lexicon_tags_11 },
    { 0x428d9d44, SUC_lexicon_tags_40 },
    { 0x93cd9d3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70d9d49, SUC_lexicon_tags_27 },
    { 0x5dcd9d4a, SUC_lexicon_tags_38 },
    { 0x732d9d4b, SUC_lexicon_tags_70 },
    { 0xe98d9d4c, SUC_lexicon_tags_17 },
    { 0xa20d9d4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x154d9d4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608d9d54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x662d9d56, SUC_lexicon_tags_7 },
    { 0x188d9d56, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x484d9d5a, SUC_lexicon_tags_48 },
    { 0x8e0d9d5b, SUC_lexicon_tags_4 },
    { 0xc80d9d5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x10ed9d5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcd9d66, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe34d9d68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcecd9d6b, SUC_lexicon_tags_3 },
    { 0x9dcd9d6c, SUC_lexicon_tags_34 },
    { 0xc3cd9d6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cd9d70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22d9d78, SUC_lexicon_tags_1 },
    { 0x29ad9d79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfced9d7b, SUC_lexicon_tags_17 },
    { 0x706d9d7c, SUC_lexicon_tags_26 },
    { 0x458d9d7d, SUC_lexicon_tags_25 },
    { 0xc7cd9d7e, SUC_lexicon_tags_17 },
    { 0xa00d9d7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d9d82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94d9d8d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe76d9d8f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x25ad9d91, SUC_lexicon_tags_37 },
    { 0xb48d9d92, SUC_lexicon_tags_44 },
    { 0x110d9d92, SUC_lexicon_tags_57 },
    { 0x2b8d9d91, SUC_lexicon_tags_11 },
    { 0xfacd9d94, SUC_lexicon_tags_9 },
    { 0x86ed9d93, SUC_lexicon_tags_4 },
    { 0xd00d9d93, SUC_lexicon_tags_25 },
    { 0xf0d9d98, SUC_lexicon_tags_282 },
    { 0xa94d9d99, SUC_lexicon_tags_34 },
    { 0x436d9d9a, SUC_lexicon_tags_7 },
    { 0xcd6d9d9b, SUC_lexicon_tags_30 },
    { 0x3b2d9d9c, SUC_lexicon_tags_63 },
    { 0xde2d9d9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x448d9d9f, SUC_lexicon_tags_47 },
    { 0x48d9da0, SUC_lexicon_tags_43 },
    { 0x10ad9da1, SUC_lexicon_tags_12 },
    { 0x46ed9da2, SUC_lexicon_tags_26 },
    { 0x10cd9da3, SUC_lexicon_tags_25 },
    { 0xe80d9da3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe9ad9da6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50d9daa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa78d9dac, SUC_lexicon_tags_27 },
    { 0xe16d9dac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52d9db1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x198d9db3, SUC_lexicon_tags_27 },
    { 0xac6d9db3, SUC_lexicon_tags_37 },
    { 0x8c6d9db5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ed9db8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ed9dbb, SUC_lexicon_tags_16 },
    { 0xfe2d9dbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68cd9dbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2d9dc2, SUC_lexicon_tags_15 },
    { 0xce4d9dc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70d9dc6, SUC_lexicon_tags_48 },
    { 0xc4ad9dc7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4d9dca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8ad9dcc, SUC_lexicon_tags_17 },
    { 0xa8d9dcd, SUC_lexicon_tags_15 },
    { 0x262d9dce, SUC_lexicon_tags_37 },
    { 0xd1ed9dcf, SUC_lexicon_tags_34 },
    { 0x402d9dd0, SUC_lexicon_tags_23 },
    { 0xa92d9dce, SUC_lexicon_tags_25 },
    { 0x984d9dd2, SUC_lexicon_tags_37 },
    { 0xa7cd9dd3, SUC_lexicon_tags_17 },
    { 0x1fad9dce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32d9dda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d9dde, SUC_lexicon_tags_60 },
    { 0x3a8d9dde, SUC_lexicon_tags_1 },
    { 0xd2d9de0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x38ed9de2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6d9de8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd34d9dea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8ed9dec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4d9df1, SUC_lexicon_tags_37 },
    { 0x96ed9df2, SUC_lexicon_tags_47 },
    { 0x7a4d9df1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ed9df6, SUC_lexicon_tags_79 },
    { 0x106d9df7, SUC_lexicon_tags_14 },
    { 0x862d9df6, SUC_lexicon_tags_17 },
    { 0xe36d9df9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x444d9dfb, SUC_lexicon_tags_38 },
    { 0x340d9dfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92d9e02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88d9e09, SUC_lexicon_tags_60 },
    { 0xce0d9e0a, SUC_lexicon_tags_30 },
    { 0x340d9e0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5cad9e0d, SUC_lexicon_tags_65 },
    { 0xf64d9e0e, SUC_lexicon_tags_17 },
    { 0x61ed9e0f, SUC_lexicon_tags_19 },
    { 0x70ed9e10, SUC_lexicon_tags_25 },
    { 0xb24d9e10, SUC_lexicon_tags_31 },
    { 0x406d9e12, SUC_lexicon_tags_25 },
    { 0xc54d9e12, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d9e1e, SUC_lexicon_tags_37 },
    { 0x5b6d9e1e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcd9e22, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x144d9e24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864d9e2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758d9e30, SUC_lexicon_tags_65 },
    { 0xc2d9e31, SUC_lexicon_tags_7 },
    { 0xa4ed9e32, SUC_lexicon_tags_17 },
    { 0xfced9e33, SUC_lexicon_tags_13 },
    { 0x9e6d9e33, SUC_lexicon_tags_49 },
    { 0x6c2d9e33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74d9e38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x594d9e3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28d9e3d, SUC_lexicon_tags_32 },
    { 0x3eed9e3d, SUC_lexicon_tags_11 },
    { 0x5bed9e3f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ad9e43, SUC_lexicon_tags_17 },
    { 0xaded9e43, SUC_lexicon_tags_17 },
    { 0xac2d9e45, SUC_lexicon_tags_24 },
    { 0xac6d9e44, SUC_lexicon_tags_1 },
    { 0x412d9e45, SUC_lexicon_tags_46 },
    { 0x316d9e43, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x634d9e4b, SUC_lexicon_tags_6 },
    { 0x396d9e4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ad9e4f, SUC_lexicon_tags_30 },
    { 0x968d9e4f, SUC_lexicon_tags_37 },
    { 0x84ed9e50, SUC_lexicon_tags_17 },
    { 0xfb8d9e52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb34d9e54, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ed9e5a, SUC_lexicon_tags_7 },
    { 0xcded9e5a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06d9e5e, SUC_lexicon_tags_37 },
    { 0x470d9e5f, SUC_lexicon_tags_63 },
    { 0xc2ad9e5f, SUC_lexicon_tags_15 },
    { 0xfe2d9e61, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ed9e65, SUC_lexicon_tags_39 },
    { 0xd9ad9e66, SUC_lexicon_tags_10 },
    { 0xe4ed9e67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ad9e6d, SUC_lexicon_tags_28 },
    { 0x5dad9e6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xebed9e70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432d9e79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ced9e7b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x31ed9e7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ed9e85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262d9e89, SUC_lexicon_tags_25 },
    { 0x13ed9e89, SUC_lexicon_tags_32 },
    { 0xd20d9e89, SUC_lexicon_tags_27 },
    { 0x994d9e89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x396d9e8e, SUC_lexicon_tags_20 },
    { 0xcdcd9e8e, SUC_lexicon_tags_2 },
    { 0xe4ed9e90, SUC_lexicon_tags_37 },
    { 0x704d9e91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b6d9e93, SUC_lexicon_tags_16 },
    { 0xcbed9e93, SUC_lexicon_tags_29 },
    { 0x1e2d9e95, SUC_lexicon_tags_38 },
    { 0x42cd9e95, SUC_lexicon_tags_19 },
    { 0xcd8d9e97, SUC_lexicon_tags_7 },
    { 0x840d9e94, SUC_lexicon_tags_4 },
    { 0x56ed9e99, SUC_lexicon_tags_60 },
    { 0x69ad9e9a, SUC_lexicon_tags_63 },
    { 0xdc2d9e9a, SUC_lexicon_tags_36 },
    { 0x81cd9e9b, SUC_lexicon_tags_7 },
    { 0x32d9e9c, SUC_lexicon_tags_10 },
    { 0x2c4d9e9b, SUC_lexicon_tags_37 },
    { 0xab2d9e9f, SUC_lexicon_tags_33 },
    { 0x4fed9ea0, SUC_lexicon_tags_28 },
    { 0x6acd9ea1, SUC_lexicon_tags_25 },
    { 0xf06d9ea0, SUC_lexicon_tags_17 },
    { 0xc6ed9ea3, SUC_lexicon_tags_28 },
    { 0x3a4d9ea2, SUC_lexicon_tags_45 },
    { 0x2dcd9e99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d9ea8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d9eb0, SUC_lexicon_tags_66 },
    { 0x210d9eb1, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08d9eb5, SUC_lexicon_tags_14 },
    { 0x948d9eb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cd9ebc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ed9ec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04d9ec5, SUC_lexicon_tags_11 },
    { 0x82ad9ec6, SUC_lexicon_tags_47 },
    { 0x61ad9ec7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0d9eca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422d9ecf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756d9ed7, SUC_lexicon_tags_37 },
    { 0x9b8d9ed7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x616d9eda, SUC_lexicon_tags_1 },
    { 0xf52d9eda, SUC_lexicon_tags_25 },
    { 0x398d9eda, SUC_lexicon_tags_45 },
    { 0x674d9eda, SUC_lexicon_tags_27 },
    { 0x792d9edc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08d9ee2, SUC_lexicon_tags_11 },
    { 0xc26d9ee3, SUC_lexicon_tags_37 },
    { 0x56ad9ee3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200d9ee8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118d9eec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10cd9eee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cd9ef2, SUC_lexicon_tags_4 },
    { 0x26d9ef3, SUC_lexicon_tags_47 },
    { 0x6ecd9ef4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x122d9ef7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0d9efa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62d9f01, SUC_lexicon_tags_0 },
    { 0xfc6d9f02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80d9f08, SUC_lexicon_tags_33 },
    { 0xe38d9f08, SUC_lexicon_tags_2 },
    { 0x784d9f09, SUC_lexicon_tags_109 },
    { 0x97ad9f0b, SUC_lexicon_tags_17 },
    { 0xe82d9f0c, SUC_lexicon_tags_17 },
    { 0x7ad9f0d, SUC_lexicon_tags_2 },
    { 0xce6d9f0e, SUC_lexicon_tags_7 },
    { 0xed2d9f0c, SUC_lexicon_tags_1 },
    { 0xf74d9f0a, SUC_lexicon_tags_8 },
    { 0xe58d9f11, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ed9f15, SUC_lexicon_tags_37 },
    { 0xf60d9f16, SUC_lexicon_tags_14 },
    { 0x46cd9f17, SUC_lexicon_tags_38 },
    { 0xaed9f18, SUC_lexicon_tags_37 },
    { 0xd6ad9f16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x272d9f1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcd9f1e, SUC_lexicon_tags_11 },
    { 0x858d9f1f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646d9f23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x48cd9f25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2d9f2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcd9f33, SUC_lexicon_tags_45 },
    { 0x994d9f33, SUC_lexicon_tags_25 },
    { 0xf60d9f35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2d9f38, SUC_lexicon_tags_2 },
    { 0x57ed9f39, SUC_lexicon_tags_49 },
    { 0x7f4d9f3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x124d9f3d, SUC_lexicon_tags_17 },
    { 0xfa0d9f3d, SUC_lexicon_tags_63 },
    { 0xc14d9f3f, SUC_lexicon_tags_345 },
    { 0, NULL },
    { 0x9ed9f41, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd80d9f43, SUC_lexicon_tags_35 },
    { 0x7fad9f44, SUC_lexicon_tags_1 },
    { 0x96cd9f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866d9f4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcced9f53, SUC_lexicon_tags_33 },
    { 0xc14d9f54, SUC_lexicon_tags_11 },
    { 0xa52d9f55, SUC_lexicon_tags_11 },
    { 0x7a4d9f56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcd9f59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ecd9f5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacd9f60, SUC_lexicon_tags_1 },
    { 0xe50d9f60, SUC_lexicon_tags_55 },
    { 0xea4d9f62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822d9f66, SUC_lexicon_tags_48 },
    { 0xb70d9f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad9f6a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76d9f71, SUC_lexicon_tags_8 },
    { 0x98ad9f72, SUC_lexicon_tags_42 },
    { 0x83cd9f72, SUC_lexicon_tags_1562 },
    { 0x844d9f74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac4d9f76, SUC_lexicon_tags_158 },
    { 0xca8d9f76, SUC_lexicon_tags_12 },
    { 0x644d9f77, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ed9f7c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x152d9f7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6d9f82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aed9f85, SUC_lexicon_tags_7 },
    { 0x81ed9f86, SUC_lexicon_tags_0 },
    { 0xb2ed9f85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9f8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ad9f8f, SUC_lexicon_tags_17 },
    { 0x28ed9f90, SUC_lexicon_tags_37 },
    { 0xb2ad9f91, SUC_lexicon_tags_45 },
    { 0xe2ed9f90, SUC_lexicon_tags_7 },
    { 0x888d9f93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ad9f9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240d9f9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6d9fa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x834d9fa5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafed9fac, SUC_lexicon_tags_30 },
    { 0xc2cd9fad, SUC_lexicon_tags_26 },
    { 0x8c4d9fad, SUC_lexicon_tags_59 },
    { 0x14cd9fae, SUC_lexicon_tags_1 },
    { 0xa2ed9fb0, SUC_lexicon_tags_30 },
    { 0x836d9fb1, SUC_lexicon_tags_17 },
    { 0x9d6d9fb1, SUC_lexicon_tags_32 },
    { 0x3d2d9fb3, SUC_lexicon_tags_55 },
    { 0x6dcd9fb2, SUC_lexicon_tags_12 },
    { 0x216d9fb4, SUC_lexicon_tags_25 },
    { 0x192d9fb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ad9fba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x408d9fbd, SUC_lexicon_tags_60 },
    { 0xdfcd9fbd, SUC_lexicon_tags_2 },
    { 0x402d9fbe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a6d9fc1, SUC_lexicon_tags_27 },
    { 0xae0d9fc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x304d9fc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xedcd9fc7, SUC_lexicon_tags_62 },
    { 0x6c8d9fc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c4d9fca, SUC_lexicon_tags_18 },
    { 0xa0d9fca, SUC_lexicon_tags_11 },
    { 0x5a0d9fcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x750d9fcf, SUC_lexicon_tags_15 },
    { 0x492d9fd0, SUC_lexicon_tags_17 },
    { 0x7b8d9fd0, SUC_lexicon_tags_26 },
    { 0x378d9fd1, SUC_lexicon_tags_27 },
    { 0x33cd9fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2d9fd6, SUC_lexicon_tags_10 },
    { 0x84d9fd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8d9fda, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ed9fdd, SUC_lexicon_tags_15 },
    { 0x380d9fde, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x356d9fe0, SUC_lexicon_tags_43 },
    { 0x19cd9fe0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xff2d9fe3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x58cd9fe5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa2cd9fe7, SUC_lexicon_tags_43 },
    { 0x868d9fe8, SUC_lexicon_tags_4 },
    { 0x27ed9fe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed9fee, SUC_lexicon_tags_43 },
    { 0x904d9fef, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406d9ff4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cd9ff8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0d9ffc, SUC_lexicon_tags_63 },
    { 0x630d9ffc, SUC_lexicon_tags_0 },
    { 0x4ded9ffd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x800da000, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ada004, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeda009, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f0da00b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358da00f, SUC_lexicon_tags_87 },
    { 0xc4cda00f, SUC_lexicon_tags_15 },
    { 0x694da010, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6da01e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc8ada020, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbfeda022, SUC_lexicon_tags_15 },
    { 0x780da023, SUC_lexicon_tags_2 },
    { 0xed8da024, SUC_lexicon_tags_62 },
    { 0x420da024, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9cda027, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d4da029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cda02d, SUC_lexicon_tags_17 },
    { 0x742da02e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97eda037, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x898da039, SUC_lexicon_tags_10 },
    { 0x76da03a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56da03d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89cda03f, SUC_lexicon_tags_7 },
    { 0x888da03f, SUC_lexicon_tags_11 },
    { 0x8fcda041, SUC_lexicon_tags_17 },
    { 0x83cda041, SUC_lexicon_tags_23 },
    { 0x150da043, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x96eda045, SUC_lexicon_tags_37 },
    { 0x18da046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d0da048, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x798da04a, SUC_lexicon_tags_25 },
    { 0xfdeda04b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ada053, SUC_lexicon_tags_7 },
    { 0x896da054, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cda058, SUC_lexicon_tags_32 },
    { 0xe86da059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf20da05b, SUC_lexicon_tags_11 },
    { 0xebada05c, SUC_lexicon_tags_23 },
    { 0x4eda05c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcda061, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x44eda063, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa16da065, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x568da067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccda06b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ada06f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b0da071, SUC_lexicon_tags_25 },
    { 0xe6ada072, SUC_lexicon_tags_29 },
    { 0xaa2da071, SUC_lexicon_tags_37 },
    { 0x57ada074, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28da07e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x664da080, SUC_lexicon_tags_15 },
    { 0x80cda081, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eda084, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0da089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd08da08b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8da095, SUC_lexicon_tags_37 },
    { 0xa3cda096, SUC_lexicon_tags_10 },
    { 0x5deda097, SUC_lexicon_tags_37 },
    { 0xe66da097, SUC_lexicon_tags_11 },
    { 0xa90da099, SUC_lexicon_tags_38 },
    { 0xa8da09a, SUC_lexicon_tags_38 },
    { 0x2b6da09b, SUC_lexicon_tags_37 },
    { 0x40da099, SUC_lexicon_tags_25 },
    { 0x4cda09c, SUC_lexicon_tags_12 },
    { 0xc50da09e, SUC_lexicon_tags_25 },
    { 0xaf2da097, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42eda0a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2ada0a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13eda0a6, SUC_lexicon_tags_39 },
    { 0x38da0a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4da0aa, SUC_lexicon_tags_19 },
    { 0xda0da0ab, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ada0b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x46cda0b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaf6da0b6, SUC_lexicon_tags_32 },
    { 0xf7eda0b6, SUC_lexicon_tags_82 },
    { 0xebada0b8, SUC_lexicon_tags_47 },
    { 0x2d8da0b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34da0bc, SUC_lexicon_tags_55 },
    { 0xca8da0bd, SUC_lexicon_tags_49 },
    { 0x410da0bd, SUC_lexicon_tags_39 },
    { 0xe1ada0bc, SUC_lexicon_tags_38 },
    { 0xaf0da0c0, SUC_lexicon_tags_6 },
    { 0x918da0c0, SUC_lexicon_tags_17 },
    { 0x6eada0bd, SUC_lexicon_tags_44 },
    { 0xe62da0bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0da0cc, SUC_lexicon_tags_12 },
    { 0xbb8da0cd, SUC_lexicon_tags_27 },
    { 0x8f4da0ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccda0d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x946da0d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8f0da0d5, SUC_lexicon_tags_14 },
    { 0xfbcda0d5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8da0dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52da0df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb86da0e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ada0e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x314da0e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x92cda0eb, SUC_lexicon_tags_63 },
    { 0x954da0ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0da0ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ada0f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcda0f4, SUC_lexicon_tags_29 },
    { 0xbecda0f5, SUC_lexicon_tags_12 },
    { 0x2c4da0f6, SUC_lexicon_tags_63 },
    { 0x8a6da0f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9eda0fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x392da0fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6da102, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ada108, SUC_lexicon_tags_37 },
    { 0x242da108, SUC_lexicon_tags_39 },
    { 0xd7ada10a, SUC_lexicon_tags_8 },
    { 0x8ada10b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4da10e, SUC_lexicon_tags_105 },
    { 0xaa8da10f, SUC_lexicon_tags_45 },
    { 0x82cda110, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce2da112, SUC_lexicon_tags_47 },
    { 0xfecda113, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690da117, SUC_lexicon_tags_17 },
    { 0xf8cda118, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46eda11c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eda122, SUC_lexicon_tags_17 },
    { 0x116da123, SUC_lexicon_tags_31 },
    { 0x864da124, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8da129, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbbcda12b, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x342da12d, SUC_lexicon_tags_0 },
    { 0x8c0da12d, SUC_lexicon_tags_15 },
    { 0x2d4da12d, SUC_lexicon_tags_37 },
    { 0x8b4da12f, SUC_lexicon_tags_60 },
    { 0xa40da131, SUC_lexicon_tags_55 },
    { 0x244da132, SUC_lexicon_tags_32 },
    { 0x3fcda133, SUC_lexicon_tags_37 },
    { 0xfaeda133, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x498da137, SUC_lexicon_tags_65 },
    { 0x250da138, SUC_lexicon_tags_12 },
    { 0xf58da139, SUC_lexicon_tags_12 },
    { 0x8d0da13a, SUC_lexicon_tags_12 },
    { 0x63eda13b, SUC_lexicon_tags_23 },
    { 0x8a8da13c, SUC_lexicon_tags_60 },
    { 0x698da13d, SUC_lexicon_tags_16 },
    { 0x554da13e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ecda140, SUC_lexicon_tags_17 },
    { 0x782da141, SUC_lexicon_tags_79 },
    { 0x70da142, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadada149, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x564da14c, SUC_lexicon_tags_47 },
    { 0xd84da14d, SUC_lexicon_tags_7 },
    { 0xb02da14e, SUC_lexicon_tags_15 },
    { 0xfa0da14e, SUC_lexicon_tags_17 },
    { 0x46cda14c, SUC_lexicon_tags_25 },
    { 0x5d0da14e, SUC_lexicon_tags_14 },
    { 0x1b8da152, SUC_lexicon_tags_68 },
    { 0xf30da14c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26cda155, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd6ada157, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596da164, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5ceda166, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xed8da168, SUC_lexicon_tags_15 },
    { 0x2deda169, SUC_lexicon_tags_17 },
    { 0x1eda169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92da16d, SUC_lexicon_tags_21 },
    { 0x86eda16e, SUC_lexicon_tags_63 },
    { 0x780da16f, SUC_lexicon_tags_9 },
    { 0x942da16f, SUC_lexicon_tags_45 },
    { 0xfbada170, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c6da173, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfecda175, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dada179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cda17d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6da184, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64da18b, SUC_lexicon_tags_15 },
    { 0xb30da18c, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x636da18f, SUC_lexicon_tags_7 },
    { 0xdaada190, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfada192, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536da197, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x640da199, SUC_lexicon_tags_45 },
    { 0xf2cda19a, SUC_lexicon_tags_17 },
    { 0xcc2da19b, SUC_lexicon_tags_14 },
    { 0xb9ada19c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8da1a0, SUC_lexicon_tags_37 },
    { 0xc42da1a0, SUC_lexicon_tags_14 },
    { 0x536da1a0, SUC_lexicon_tags_0 },
    { 0xaada1a3, SUC_lexicon_tags_33 },
    { 0x102da1a3, SUC_lexicon_tags_23 },
    { 0xa4eda1a4, SUC_lexicon_tags_37 },
    { 0x56da1a6, SUC_lexicon_tags_27 },
    { 0x552da1a7, SUC_lexicon_tags_2 },
    { 0x402da1a1, SUC_lexicon_tags_49 },
    { 0x982da1a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x67cda1ab, SUC_lexicon_tags_160 },
    { 0x47eda1ac, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0xddada1af, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xededa1b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41ada1b4, SUC_lexicon_tags_37 },
    { 0x344da1b5, SUC_lexicon_tags_14 },
    { 0xebeda1b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ada1ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5ada1bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d4da1be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f6da1c0, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8da1cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4dada1cd, SUC_lexicon_tags_11 },
    { 0x164da1ce, SUC_lexicon_tags_23 },
    { 0x42cda1ce, SUC_lexicon_tags_38 },
    { 0x364da1d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd3eda1d2, SUC_lexicon_tags_37 },
    { 0xadada1d3, SUC_lexicon_tags_2 },
    { 0xafada1d2, SUC_lexicon_tags_31 },
    { 0x412da1d5, SUC_lexicon_tags_19 },
    { 0x6c8da1d6, SUC_lexicon_tags_38 },
    { 0xd7cda1d7, SUC_lexicon_tags_46 },
    { 0xd90da1d5, SUC_lexicon_tags_0 },
    { 0x99eda1d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b4da1db, SUC_lexicon_tags_1 },
    { 0xd22da1db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f0da1de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cda1e4, SUC_lexicon_tags_3 },
    { 0xe2eda1e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d0da1e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766da1eb, SUC_lexicon_tags_4 },
    { 0x7a2da1ec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2a2da1ee, SUC_lexicon_tags_28 },
    { 0xb12da1ef, SUC_lexicon_tags_25 },
    { 0x59cda1f0, SUC_lexicon_tags_33 },
    { 0x35cda1f0, SUC_lexicon_tags_7 },
    { 0xbdada1f2, SUC_lexicon_tags_31 },
    { 0xbe2da1f1, SUC_lexicon_tags_45 },
    { 0xd7eda1f4, SUC_lexicon_tags_29 },
    { 0xd8cda1ee, SUC_lexicon_tags_82 },
    { 0xbdcda1f6, SUC_lexicon_tags_45 },
    { 0xd1eda1f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60da1fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08da1fd, SUC_lexicon_tags_21 },
    { 0x30da1fd, SUC_lexicon_tags_7 },
    { 0xfecda1fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2da216, SUC_lexicon_tags_14 },
    { 0x61ada217, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2da21b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4feda21d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4da221, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7deda223, SUC_lexicon_tags_7 },
    { 0x818da224, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eada22b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xda2da22d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46da232, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0da235, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460da23b, SUC_lexicon_tags_54 },
    { 0xd12da23c, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ada243, SUC_lexicon_tags_45 },
    { 0xc68da244, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ada247, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eda24b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4da250, SUC_lexicon_tags_3 },
    { 0x80cda251, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64eda254, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48ada256, SUC_lexicon_tags_38 },
    { 0x1f6da257, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c4da259, SUC_lexicon_tags_12 },
    { 0xb0da259, SUC_lexicon_tags_71 },
    { 0x2f6da25b, SUC_lexicon_tags_19 },
    { 0x5e2da25b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcda261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61eda26a, SUC_lexicon_tags_11 },
    { 0x6a6da26b, SUC_lexicon_tags_23 },
    { 0xe7eda26c, SUC_lexicon_tags_37 },
    { 0x50cda26a, SUC_lexicon_tags_106 },
    { 0xee4da26a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46eda270, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x608da273, SUC_lexicon_tags_130 },
    { 0xc6da274, SUC_lexicon_tags_82 },
    { 0x910da273, SUC_lexicon_tags_45 },
    { 0xe60da276, SUC_lexicon_tags_8 },
    { 0xaa4da277, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca4da279, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x462da27c, SUC_lexicon_tags_0 },
    { 0xfbcda27d, SUC_lexicon_tags_37 },
    { 0x54eda27e, SUC_lexicon_tags_16 },
    { 0xaaeda27f, SUC_lexicon_tags_47 },
    { 0xcbcda27f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92da283, SUC_lexicon_tags_14 },
    { 0xbf4da284, SUC_lexicon_tags_33 },
    { 0x77ada284, SUC_lexicon_tags_57 },
    { 0x6e6da285, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x29cda288, SUC_lexicon_tags_33 },
    { 0x45cda288, SUC_lexicon_tags_6 },
    { 0x976da28a, SUC_lexicon_tags_37 },
    { 0x24eda28b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2b6da28d, SUC_lexicon_tags_41 },
    { 0x73ada28e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4da293, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42eda296, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x546da298, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe94da29a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x384da29d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x52ada29f, SUC_lexicon_tags_65 },
    { 0x66eda29f, SUC_lexicon_tags_19 },
    { 0x32ada2a1, SUC_lexicon_tags_11 },
    { 0x2fada2a2, SUC_lexicon_tags_7 },
    { 0x598da2a3, SUC_lexicon_tags_27 },
    { 0x128da2a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xed6da2a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x154da2a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4da2ac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ceda2af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6da2b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x828da2b5, SUC_lexicon_tags_32 },
    { 0x41cda2b5, SUC_lexicon_tags_29 },
    { 0x42ada2b5, SUC_lexicon_tags_82 },
    { 0xe1eda2b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420da2c9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178da2ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18da2d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5eda2d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff4da2d8, SUC_lexicon_tags_2 },
    { 0xbb6da2d9, SUC_lexicon_tags_44 },
    { 0xd2ada2da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x816da2dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8da2e0, SUC_lexicon_tags_47 },
    { 0x88eda2e0, SUC_lexicon_tags_65 },
    { 0xa2eda2e0, SUC_lexicon_tags_37 },
    { 0x4f4da2e2, SUC_lexicon_tags_43 },
    { 0xe3ada2e0, SUC_lexicon_tags_30 },
    { 0x1ecda2e5, SUC_lexicon_tags_11 },
    { 0xc82da2e6, SUC_lexicon_tags_19 },
    { 0x70cda2e5, SUC_lexicon_tags_7 },
    { 0x8d8da2e8, SUC_lexicon_tags_14 },
    { 0xd8cda2e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458da2ee, SUC_lexicon_tags_1 },
    { 0xb1ada2ee, SUC_lexicon_tags_59 },
    { 0xd20da2f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cda2f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734da2fe, SUC_lexicon_tags_38 },
    { 0x856da2ff, SUC_lexicon_tags_25 },
    { 0x32cda2ff, SUC_lexicon_tags_47 },
    { 0x778da2ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7ada303, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf60da305, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x248da308, SUC_lexicon_tags_45 },
    { 0x9a4da309, SUC_lexicon_tags_7 },
    { 0x700da30a, SUC_lexicon_tags_17 },
    { 0xc4da30b, SUC_lexicon_tags_13 },
    { 0xbeeda30c, SUC_lexicon_tags_15 },
    { 0xa08da30d, SUC_lexicon_tags_26 },
    { 0x582da308, SUC_lexicon_tags_23 },
    { 0x898da30f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4eeda311, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbbeda313, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x456da316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcda31b, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10da326, SUC_lexicon_tags_23 },
    { 0x3c4da327, SUC_lexicon_tags_114 },
    { 0xae6da328, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0xa00da32a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e0da32c, SUC_lexicon_tags_15 },
    { 0x7c8da32d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6da331, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2da335, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550da33c, SUC_lexicon_tags_27 },
    { 0x512da33c, SUC_lexicon_tags_38 },
    { 0x3ada33c, SUC_lexicon_tags_22 },
    { 0xfd8da33c, SUC_lexicon_tags_1 },
    { 0xd64da33c, SUC_lexicon_tags_25 },
    { 0x606da341, SUC_lexicon_tags_59 },
    { 0x7dada341, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eda346, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b0da348, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2da34b, SUC_lexicon_tags_30 },
    { 0x636da34b, SUC_lexicon_tags_12 },
    { 0xfe2da34d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34da350, SUC_lexicon_tags_19 },
    { 0xee2da351, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548da355, SUC_lexicon_tags_8 },
    { 0x172da356, SUC_lexicon_tags_42 },
    { 0x46da357, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x500da35a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x888da35c, SUC_lexicon_tags_17 },
    { 0xb58da35d, SUC_lexicon_tags_19 },
    { 0xe06da35e, SUC_lexicon_tags_14 },
    { 0xe22da35c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90da361, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafcda363, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb30da365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8da36a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8da36d, SUC_lexicon_tags_2 },
    { 0x540da36d, SUC_lexicon_tags_7 },
    { 0x6deda36f, SUC_lexicon_tags_2 },
    { 0x4ceda370, SUC_lexicon_tags_37 },
    { 0x3bada370, SUC_lexicon_tags_17 },
    { 0x34ada372, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50da376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0da379, SUC_lexicon_tags_141 },
    { 0xb94da379, SUC_lexicon_tags_201 },
    { 0x558da37b, SUC_lexicon_tags_29 },
    { 0x680da37b, SUC_lexicon_tags_17 },
    { 0x812da37d, SUC_lexicon_tags_3 },
    { 0x4eeda37d, SUC_lexicon_tags_27 },
    { 0xe1ada37f, SUC_lexicon_tags_16 },
    { 0xb8cda380, SUC_lexicon_tags_1 },
    { 0x96eda37a, SUC_lexicon_tags_11 },
    { 0xc74da382, SUC_lexicon_tags_28 },
    { 0xea2da383, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e2da385, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc78da387, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x404da389, SUC_lexicon_tags_14 },
    { 0xb0cda389, SUC_lexicon_tags_47 },
    { 0x212da38a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438da38f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x95eda392, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2da395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x612da398, SUC_lexicon_tags_182 },
    { 0xc8eda399, SUC_lexicon_tags_7 },
    { 0x908da399, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8da39e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x882da3a1, SUC_lexicon_tags_15 },
    { 0x6b8da3a2, SUC_lexicon_tags_37 },
    { 0x6ceda3a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeada3a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44da3a9, SUC_lexicon_tags_49 },
    { 0x47cda3aa, SUC_lexicon_tags_41 },
    { 0x624da3ab, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928da3af, SUC_lexicon_tags_44 },
    { 0x766da3b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4deda3b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74da3b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acda3bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0da3c1, SUC_lexicon_tags_15 },
    { 0xf18da3c2, SUC_lexicon_tags_508 },
    { 0x734da3c3, SUC_lexicon_tags_42 },
    { 0x66da3c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeda3c6, SUC_lexicon_tags_17 },
    { 0x628da3c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x12eda3c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eda3d0, SUC_lexicon_tags_17 },
    { 0xc04da3d1, SUC_lexicon_tags_43 },
    { 0x702da3d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ada3db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48da3df, SUC_lexicon_tags_11 },
    { 0x1deda3df, SUC_lexicon_tags_15 },
    { 0x5ccda3e0, SUC_lexicon_tags_757 },
    { 0xf2da3e2, SUC_lexicon_tags_47 },
    { 0xdacda3e3, SUC_lexicon_tags_11 },
    { 0xdfcda3e3, SUC_lexicon_tags_28 },
    { 0x80cda3e5, SUC_lexicon_tags_0 },
    { 0xca6da3e5, SUC_lexicon_tags_60 },
    { 0x288da3e3, SUC_lexicon_tags_17 },
    { 0x8ccda3e7, SUC_lexicon_tags_37 },
    { 0x91eda3e9, SUC_lexicon_tags_12 },
    { 0x7fada3ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x834da3ec, SUC_lexicon_tags_11 },
    { 0x238da3ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x476da3ef, SUC_lexicon_tags_2 },
    { 0xdb2da3f0, SUC_lexicon_tags_3 },
    { 0xcf6da3f1, SUC_lexicon_tags_62 },
    { 0x8b0da3ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ada3fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcada401, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb08da403, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506da407, SUC_lexicon_tags_17 },
    { 0xaacda408, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18da40d, SUC_lexicon_tags_116 },
    { 0x46eda40d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cda41a, SUC_lexicon_tags_15 },
    { 0x69ada41a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fada41e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67eda421, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cda429, SUC_lexicon_tags_11 },
    { 0x8bcda42a, SUC_lexicon_tags_17 },
    { 0x8aeda42b, SUC_lexicon_tags_8 },
    { 0x2a2da42b, SUC_lexicon_tags_4 },
    { 0x336da42b, SUC_lexicon_tags_37 },
    { 0x1acda42a, SUC_lexicon_tags_11 },
    { 0x220da42f, SUC_lexicon_tags_12 },
    { 0x662da430, SUC_lexicon_tags_15 },
    { 0xfd6da431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x910da434, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x400da436, SUC_lexicon_tags_11 },
    { 0xf38da437, SUC_lexicon_tags_61 },
    { 0x592da438, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cda43e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcda441, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37eda443, SUC_lexicon_tags_6 },
    { 0xe78da443, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4da44a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcfeda44c, SUC_lexicon_tags_17 },
    { 0xeb4da44c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcda452, SUC_lexicon_tags_6 },
    { 0xbd0da452, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4da459, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x804da45b, SUC_lexicon_tags_4 },
    { 0xb52da45c, SUC_lexicon_tags_12 },
    { 0x666da45c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0da465, SUC_lexicon_tags_60 },
    { 0xec4da466, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56da469, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16eda46e, SUC_lexicon_tags_26 },
    { 0x70da46f, SUC_lexicon_tags_38 },
    { 0x612da46f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x87eda472, SUC_lexicon_tags_1 },
    { 0xbb6da473, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x654da476, SUC_lexicon_tags_14 },
    { 0xf76da477, SUC_lexicon_tags_7 },
    { 0x238da477, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x98eda47b, SUC_lexicon_tags_15 },
    { 0xc8cda47c, SUC_lexicon_tags_31 },
    { 0xc1cda47b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ada480, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27eda482, SUC_lexicon_tags_23 },
    { 0xb04da483, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x328da486, SUC_lexicon_tags_49 },
    { 0xd26da487, SUC_lexicon_tags_11 },
    { 0xacada488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x364da48a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbecda48c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82da490, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x7fada492, SUC_lexicon_tags_19 },
    { 0xc0eda493, SUC_lexicon_tags_33 },
    { 0xd12da494, SUC_lexicon_tags_66 },
    { 0xdb2da494, SUC_lexicon_tags_25 },
    { 0x184da496, SUC_lexicon_tags_15 },
    { 0xc72da496, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eeda49a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96eda49f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3eda4a1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x588da4a4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6da4ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ada4af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8da4b2, SUC_lexicon_tags_49 },
    { 0x268da4b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb24da4b5, SUC_lexicon_tags_15 },
    { 0x506da4b6, SUC_lexicon_tags_4 },
    { 0x658da4b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25cda4b9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2e4da4bb, SUC_lexicon_tags_29 },
    { 0xd78da4bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x814da4be, SUC_lexicon_tags_43 },
    { 0xfa4da4be, SUC_lexicon_tags_7 },
    { 0xab8da4bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd6cda4c2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x48eda4c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808da4cd, SUC_lexicon_tags_3 },
    { 0x638da4cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ada4d1, SUC_lexicon_tags_16 },
    { 0xa84da4d1, SUC_lexicon_tags_60 },
    { 0xdc8da4d2, SUC_lexicon_tags_113 },
    { 0xd66da4d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330da4d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4eada4da, SUC_lexicon_tags_7 },
    { 0xdcda4db, SUC_lexicon_tags_62 },
    { 0x682da4db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2da4df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4da4e2, SUC_lexicon_tags_32 },
    { 0x9aada4e2, SUC_lexicon_tags_17 },
    { 0x11cda4e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa84da4e6, SUC_lexicon_tags_37 },
    { 0xfe6da4e7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228da4ed, SUC_lexicon_tags_11 },
    { 0x9ecda4ee, SUC_lexicon_tags_25 },
    { 0xd68da4ef, SUC_lexicon_tags_34 },
    { 0xe7eda4f0, SUC_lexicon_tags_27 },
    { 0x916da4ed, SUC_lexicon_tags_31 },
    { 0xf86da4ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0da4fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4da4fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cda506, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8da509, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x628da50b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cda50f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0da511, SUC_lexicon_tags_38 },
    { 0xd60da511, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x730da515, SUC_lexicon_tags_70 },
    { 0xa6cda515, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x402da518, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa68da51a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8da51d, SUC_lexicon_tags_63 },
    { 0x7c8da51e, SUC_lexicon_tags_31 },
    { 0xcb2da51f, SUC_lexicon_tags_27 },
    { 0xab4da520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14da528, SUC_lexicon_tags_30 },
    { 0x4bada528, SUC_lexicon_tags_14 },
    { 0x9eeda528, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2da52d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x200da530, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x54da532, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40da534, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ada536, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x236da538, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18da53d, SUC_lexicon_tags_12 },
    { 0xae0da53e, SUC_lexicon_tags_10 },
    { 0x14ada53f, SUC_lexicon_tags_42 },
    { 0xba2da53e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x36eda542, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296da548, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeda54b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244da54f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6feda55c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b0da55e, SUC_lexicon_tags_32 },
    { 0xc64da55f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8eda562, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0da568, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cada56e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3feda581, SUC_lexicon_tags_3 },
    { 0xd88da582, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2da589, SUC_lexicon_tags_47 },
    { 0xeaada58a, SUC_lexicon_tags_9 },
    { 0x94da58b, SUC_lexicon_tags_25 },
    { 0x942da58a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3d2da58e, SUC_lexicon_tags_1 },
    { 0x3feda58f, SUC_lexicon_tags_38 },
    { 0x360da58e, SUC_lexicon_tags_1 },
    { 0xe98da591, SUC_lexicon_tags_47 },
    { 0x9a8da592, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6da59a, SUC_lexicon_tags_23 },
    { 0x4a4da59a, SUC_lexicon_tags_10 },
    { 0xc28da59a, SUC_lexicon_tags_17 },
    { 0xfceda59b, SUC_lexicon_tags_0 },
    { 0x7f0da59e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82da5a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64da5a6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ada5a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6da5ac, SUC_lexicon_tags_11 },
    { 0x178da5ac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1e8da5af, SUC_lexicon_tags_1 },
    { 0xe9ada5af, SUC_lexicon_tags_37 },
    { 0x648da5b1, SUC_lexicon_tags_16 },
    { 0xb22da5b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20da5b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454da5c1, SUC_lexicon_tags_12 },
    { 0x19cda5c2, SUC_lexicon_tags_11 },
    { 0x5c2da5c3, SUC_lexicon_tags_105 },
    { 0x512da5c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x924da5c7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854da5cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa70da5cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dada5d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe84da5d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2dada5d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4f8da5da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cda5de, SUC_lexicon_tags_27 },
    { 0x4c0da5de, SUC_lexicon_tags_37 },
    { 0x82cda5e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56da5e3, SUC_lexicon_tags_38 },
    { 0x678da5e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb22da5e6, SUC_lexicon_tags_33 },
    { 0x72eda5e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x832da5e9, SUC_lexicon_tags_28 },
    { 0xea4da5ea, SUC_lexicon_tags_37 },
    { 0xaceda5ea, SUC_lexicon_tags_7 },
    { 0x8feda5e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0da5ef, SUC_lexicon_tags_9 },
    { 0x1e6da5f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814da5f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x94cda5f7, SUC_lexicon_tags_0 },
    { 0x858da5f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0da5fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00da604, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ada607, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408da60c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0da60f, SUC_lexicon_tags_17 },
    { 0x964da610, SUC_lexicon_tags_17 },
    { 0x3feda610, SUC_lexicon_tags_12 },
    { 0xeb6da612, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6da617, SUC_lexicon_tags_56 },
    { 0xe2cda618, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eda61f, SUC_lexicon_tags_25 },
    { 0x908da620, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36da623, SUC_lexicon_tags_15 },
    { 0x416da624, SUC_lexicon_tags_68 },
    { 0x1fcda625, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5c2da627, SUC_lexicon_tags_33 },
    { 0x422da627, SUC_lexicon_tags_32 },
    { 0xcda629, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x740da62c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524da634, SUC_lexicon_tags_17 },
    { 0xb0da635, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad2da637, SUC_lexicon_tags_38 },
    { 0x550da637, SUC_lexicon_tags_15 },
    { 0xd8cda639, SUC_lexicon_tags_12 },
    { 0xe8cda639, SUC_lexicon_tags_29 },
    { 0x292da63b, SUC_lexicon_tags_17 },
    { 0x29ada63b, SUC_lexicon_tags_37 },
    { 0x628da63b, SUC_lexicon_tags_17 },
    { 0x8feda63a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8dcda640, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cda647, SUC_lexicon_tags_29 },
    { 0xda648, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812da64e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56da652, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeada658, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b2da65a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eda65d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x240da65f, SUC_lexicon_tags_3 },
    { 0x1c2da660, SUC_lexicon_tags_15 },
    { 0x89ada661, SUC_lexicon_tags_12 },
    { 0x1bcda662, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc40da665, SUC_lexicon_tags_1 },
    { 0x2ceda666, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bada669, SUC_lexicon_tags_43 },
    { 0x668da66a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb16da66c, SUC_lexicon_tags_17 },
    { 0x234da66d, SUC_lexicon_tags_38 },
    { 0xf1ada66e, SUC_lexicon_tags_19 },
    { 0x6a6da66d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26da67a, SUC_lexicon_tags_31 },
    { 0xd16da67b, SUC_lexicon_tags_1 },
    { 0x380da67b, SUC_lexicon_tags_68 },
    { 0x7ceda67b, SUC_lexicon_tags_53 },
    { 0xb62da67e, SUC_lexicon_tags_25 },
    { 0x8dcda67f, SUC_lexicon_tags_4 },
    { 0xa28da67d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6da683, SUC_lexicon_tags_4 },
    { 0x71eda683, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5b0da686, SUC_lexicon_tags_29 },
    { 0x9b0da687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86da68b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6da68f, SUC_lexicon_tags_17 },
    { 0x6a4da690, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6da694, SUC_lexicon_tags_114 },
    { 0x1e0da695, SUC_lexicon_tags_39 },
    { 0xa5ada696, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcda699, SUC_lexicon_tags_34 },
    { 0xa4eda69a, SUC_lexicon_tags_37 },
    { 0x4e8da699, SUC_lexicon_tags_47 },
    { 0x45cda69c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4da69f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xceeda6a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x86da6a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02da6a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278da6ad, SUC_lexicon_tags_12 },
    { 0x5c4da6ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0da6b3, SUC_lexicon_tags_12 },
    { 0x9dada6b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74da6bb, SUC_lexicon_tags_26 },
    { 0x7aeda6bc, SUC_lexicon_tags_7 },
    { 0x446da6bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbeda6c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320da6c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cda6ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70da6cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06da6d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xabeda6d6, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8da6da, SUC_lexicon_tags_25 },
    { 0x376da6db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x926da6dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacda6e3, SUC_lexicon_tags_37 },
    { 0x6f6da6e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d8da6e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978da6ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54da6ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x82cda6ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520da6f3, SUC_lexicon_tags_1 },
    { 0x568da6f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x548da6f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92da6fa, SUC_lexicon_tags_23 },
    { 0x164da6fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8da701, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdd2da703, SUC_lexicon_tags_37 },
    { 0x866da704, SUC_lexicon_tags_17 },
    { 0xf42da705, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36eda708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe54da711, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e8da713, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbcda715, SUC_lexicon_tags_66 },
    { 0xa8ada716, SUC_lexicon_tags_23 },
    { 0xc6da716, SUC_lexicon_tags_37 },
    { 0x5c6da718, SUC_lexicon_tags_32 },
    { 0x6bada719, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x472da71b, SUC_lexicon_tags_1 },
    { 0x10da71c, SUC_lexicon_tags_2 },
    { 0xa36da71d, SUC_lexicon_tags_25 },
    { 0xdeeda71e, SUC_lexicon_tags_25 },
    { 0xc78da71e, SUC_lexicon_tags_2 },
    { 0xb54da71e, SUC_lexicon_tags_14 },
    { 0x1a8da721, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cda728, SUC_lexicon_tags_68 },
    { 0x5d2da729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0da72c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef4da72e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37eda732, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6da736, SUC_lexicon_tags_17 },
    { 0xc42da737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57eda739, SUC_lexicon_tags_44 },
    { 0x26cda73a, SUC_lexicon_tags_17 },
    { 0x96cda739, SUC_lexicon_tags_12 },
    { 0xccada73c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422da741, SUC_lexicon_tags_2 },
    { 0x24cda742, SUC_lexicon_tags_11 },
    { 0xc66da742, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eda747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cda74a, SUC_lexicon_tags_1 },
    { 0x5b8da74b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaaada74d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd04da74f, SUC_lexicon_tags_15 },
    { 0x7a4da74f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3eda752, SUC_lexicon_tags_46 },
    { 0xeb2da752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494da759, SUC_lexicon_tags_151 },
    { 0xb2ada75a, SUC_lexicon_tags_10 },
    { 0x8feda75b, SUC_lexicon_tags_7 },
    { 0xa72da75c, SUC_lexicon_tags_12 },
    { 0xa16da75d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cda764, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2da767, SUC_lexicon_tags_38 },
    { 0x638da767, SUC_lexicon_tags_63 },
    { 0xdd2da767, SUC_lexicon_tags_16 },
    { 0x2e6da76a, SUC_lexicon_tags_32 },
    { 0x89ada76a, SUC_lexicon_tags_84 },
    { 0xa4cda76c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d8da76e, SUC_lexicon_tags_17 },
    { 0x8b8da76f, SUC_lexicon_tags_7 },
    { 0xbb8da770, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x930da773, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdceda775, SUC_lexicon_tags_45 },
    { 0x10ada776, SUC_lexicon_tags_39 },
    { 0x7c4da777, SUC_lexicon_tags_31 },
    { 0x4a2da778, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x128da77b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2da782, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77eda786, SUC_lexicon_tags_17 },
    { 0xe94da786, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42da78a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586da78f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x518da791, SUC_lexicon_tags_3 },
    { 0x84da792, SUC_lexicon_tags_15 },
    { 0x8fcda793, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86da799, SUC_lexicon_tags_39 },
    { 0x56eda79a, SUC_lexicon_tags_27 },
    { 0xc5eda79a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6da7a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x486da7a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0da7aa, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd4ada7ac, SUC_lexicon_tags_174 },
    { 0x85ada7ad, SUC_lexicon_tags_12 },
    { 0xce0da7ae, SUC_lexicon_tags_30 },
    { 0xd0eda7ad, SUC_lexicon_tags_27 },
    { 0x696da7af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x466da7b2, SUC_lexicon_tags_21 },
    { 0x8f0da7b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe02da7b5, SUC_lexicon_tags_19 },
    { 0x776da7b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcda7b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4da7bb, SUC_lexicon_tags_65 },
    { 0x994da7bc, SUC_lexicon_tags_17 },
    { 0xe00da7bd, SUC_lexicon_tags_45 },
    { 0x244da7be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10da7c2, SUC_lexicon_tags_16 },
    { 0x6a8da7c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8da7c8, SUC_lexicon_tags_8 },
    { 0xb2cda7c9, SUC_lexicon_tags_27 },
    { 0x2e4da7c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fada7cc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbceda7cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4da7d3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6da7d6, SUC_lexicon_tags_7 },
    { 0x380da7d6, SUC_lexicon_tags_25 },
    { 0xd4eda7d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd64da7da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76da7dd, SUC_lexicon_tags_82 },
    { 0x9dada7de, SUC_lexicon_tags_15 },
    { 0xc1eda7df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6da7e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe48da7e5, SUC_lexicon_tags_38 },
    { 0x2c6da7e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6da7ed, SUC_lexicon_tags_25 },
    { 0x538da7ee, SUC_lexicon_tags_11 },
    { 0x3b8da7ed, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178da7f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436da7fb, SUC_lexicon_tags_7 },
    { 0xd78da7fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x68da7ff, SUC_lexicon_tags_46 },
    { 0xc5cda800, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc8da802, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa42da804, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x336da806, SUC_lexicon_tags_9 },
    { 0xc4eda807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ceda809, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc66da80b, SUC_lexicon_tags_33 },
    { 0x150da80c, SUC_lexicon_tags_26 },
    { 0xb9ada80d, SUC_lexicon_tags_15 },
    { 0x600da80c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2da814, SUC_lexicon_tags_17 },
    { 0x66cda815, SUC_lexicon_tags_3 },
    { 0x1b8da815, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cda81b, SUC_lexicon_tags_37 },
    { 0x59cda81c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198da826, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb1cda828, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48da82c, SUC_lexicon_tags_2 },
    { 0xf78da82d, SUC_lexicon_tags_47 },
    { 0x880da82e, SUC_lexicon_tags_38 },
    { 0x1a4da82e, SUC_lexicon_tags_65 },
    { 0xe80da82e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08da834, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x804da836, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bada83a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf32da83c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3cda83e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x204da841, SUC_lexicon_tags_87 },
    { 0x17cda842, SUC_lexicon_tags_10 },
    { 0x4d6da843, SUC_lexicon_tags_1 },
    { 0x1deda842, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe52da846, SUC_lexicon_tags_8 },
    { 0x4aeda847, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ada84a, SUC_lexicon_tags_1 },
    { 0x57cda84a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x45cda84d, SUC_lexicon_tags_47 },
    { 0x328da84e, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0x2f8da850, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcc4da852, SUC_lexicon_tags_17 },
    { 0x9c0da853, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88da858, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbeda85a, SUC_lexicon_tags_37 },
    { 0xb02da85a, SUC_lexicon_tags_15 },
    { 0xcecda85c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cda863, SUC_lexicon_tags_25 },
    { 0x4e2da864, SUC_lexicon_tags_26 },
    { 0x304da864, SUC_lexicon_tags_23 },
    { 0x466da865, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864da86b, SUC_lexicon_tags_29 },
    { 0x564da86c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30da870, SUC_lexicon_tags_0 },
    { 0x66cda871, SUC_lexicon_tags_14 },
    { 0x1f0da872, SUC_lexicon_tags_9 },
    { 0x8a2da873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa90da875, SUC_lexicon_tags_23 },
    { 0x584da876, SUC_lexicon_tags_44 },
    { 0xbdeda876, SUC_lexicon_tags_2 },
    { 0xaecda878, SUC_lexicon_tags_10 },
    { 0xbfcda879, SUC_lexicon_tags_7 },
    { 0x61ada878, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4da87d, SUC_lexicon_tags_45 },
    { 0xd9cda87e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50ada880, SUC_lexicon_tags_2 },
    { 0xa84da880, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3c4da883, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf30da885, SUC_lexicon_tags_1 },
    { 0x594da886, SUC_lexicon_tags_28 },
    { 0x272da887, SUC_lexicon_tags_25 },
    { 0x368da885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854da892, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940da896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc4da898, SUC_lexicon_tags_1 },
    { 0x658da899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608da8a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2da8a5, SUC_lexicon_tags_62 },
    { 0x628da8a6, SUC_lexicon_tags_30 },
    { 0x99eda8a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cda8ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xed2da8ad, SUC_lexicon_tags_26 },
    { 0x200da8ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4da8b5, SUC_lexicon_tags_4 },
    { 0xde8da8b6, SUC_lexicon_tags_17 },
    { 0x938da8b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0da8ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea4da8bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cda8c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6c4da8c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54da8c6, SUC_lexicon_tags_33 },
    { 0x8f4da8c7, SUC_lexicon_tags_12 },
    { 0xfc2da8c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6da8cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4da8d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ada8d8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x94ada8da, SUC_lexicon_tags_15 },
    { 0x236da8db, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3e0da8dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae4da8df, SUC_lexicon_tags_7 },
    { 0xca6da8e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f6da8e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeada8ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x896da8ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55cda8f0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x900da8f3, SUC_lexicon_tags_1 },
    { 0xe6eda8f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x284da8f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa7ada8f9, SUC_lexicon_tags_65 },
    { 0x652da8fa, SUC_lexicon_tags_63 },
    { 0x4a0da8fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6da902, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68da909, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x410da90b, SUC_lexicon_tags_11 },
    { 0x78eda90c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7eda90f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32da912, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716da917, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24da91b, SUC_lexicon_tags_146 },
    { 0xa94da91c, SUC_lexicon_tags_302 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616da928, SUC_lexicon_tags_15 },
    { 0x808da928, SUC_lexicon_tags_66 },
    { 0xba4da928, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8da92d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xabcda92f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x20eda931, SUC_lexicon_tags_17 },
    { 0x37cda932, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cda936, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeda93b, SUC_lexicon_tags_2 },
    { 0x9d0da93c, SUC_lexicon_tags_17 },
    { 0x46eda93d, SUC_lexicon_tags_17 },
    { 0x74da93e, SUC_lexicon_tags_10 },
    { 0xdb6da93e, SUC_lexicon_tags_26 },
    { 0xd8da93b, SUC_lexicon_tags_26 },
    { 0x126da941, SUC_lexicon_tags_29 },
    { 0x680da942, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0da948, SUC_lexicon_tags_27 },
    { 0x3d4da949, SUC_lexicon_tags_79 },
    { 0xb88da94a, SUC_lexicon_tags_11 },
    { 0x4d8da94b, SUC_lexicon_tags_8 },
    { 0xe4ada94b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2da950, SUC_lexicon_tags_15 },
    { 0x240da950, SUC_lexicon_tags_43 },
    { 0xfb4da952, SUC_lexicon_tags_17 },
    { 0xba8da953, SUC_lexicon_tags_17 },
    { 0xdbeda952, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8da957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5feda95a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9eda95c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf76da95e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x40ada960, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658da964, SUC_lexicon_tags_1 },
    { 0xfa0da965, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944da96b, SUC_lexicon_tags_37 },
    { 0x264da96b, SUC_lexicon_tags_8 },
    { 0x268da96d, SUC_lexicon_tags_4 },
    { 0xa20da96e, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3eda971, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2bcda973, SUC_lexicon_tags_12 },
    { 0x84ada974, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x290da976, SUC_lexicon_tags_11 },
    { 0x496da976, SUC_lexicon_tags_37 },
    { 0xd72da977, SUC_lexicon_tags_65 },
    { 0x7b0da979, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92eda984, SUC_lexicon_tags_37 },
    { 0x196da984, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4eda987, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fada98e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eda992, SUC_lexicon_tags_12 },
    { 0xe32da993, SUC_lexicon_tags_37 },
    { 0xd98da993, SUC_lexicon_tags_17 },
    { 0xda8da994, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ada997, SUC_lexicon_tags_1 },
    { 0xd24da998, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6da99b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x986da99d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cda9a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92da9a4, SUC_lexicon_tags_4 },
    { 0x886da9a5, SUC_lexicon_tags_31 },
    { 0xf1cda9a6, SUC_lexicon_tags_1 },
    { 0x6b6da9a4, SUC_lexicon_tags_13 },
    { 0xbeeda9a6, SUC_lexicon_tags_2 },
    { 0xc50da9a9, SUC_lexicon_tags_12 },
    { 0x22cda9a8, SUC_lexicon_tags_43 },
    { 0x684da9ab, SUC_lexicon_tags_15 },
    { 0x7b4da9ab, SUC_lexicon_tags_11 },
    { 0x692da9ad, SUC_lexicon_tags_11 },
    { 0xf7ada9ae, SUC_lexicon_tags_77 },
    { 0xea0da9a7, SUC_lexicon_tags_2 },
    { 0x2b6da9af, SUC_lexicon_tags_37 },
    { 0x868da9a9, SUC_lexicon_tags_33 },
    { 0xa66da9b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82ada9b4, SUC_lexicon_tags_14 },
    { 0xf82da9b5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x430da9b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74da9ba, SUC_lexicon_tags_19 },
    { 0x3b0da9ba, SUC_lexicon_tags_55 },
    { 0xabcda9bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6da9bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ada9c2, SUC_lexicon_tags_47 },
    { 0x14eda9c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3feda9c5, SUC_lexicon_tags_27 },
    { 0xac0da9c6, SUC_lexicon_tags_2 },
    { 0xc6cda9c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72da9cb, SUC_lexicon_tags_1 },
    { 0xfb8da9cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32da9d3, SUC_lexicon_tags_37 },
    { 0x63cda9d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccda9d8, SUC_lexicon_tags_19 },
    { 0xd7ada9d9, SUC_lexicon_tags_37 },
    { 0xd44da9da, SUC_lexicon_tags_26 },
    { 0xf28da9da, SUC_lexicon_tags_30 },
    { 0xfe6da9da, SUC_lexicon_tags_5 },
    { 0xa40da9db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8da9e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ada9e3, SUC_lexicon_tags_37 },
    { 0x338da9e3, SUC_lexicon_tags_63 },
    { 0x58cda9e5, SUC_lexicon_tags_30 },
    { 0x69cda9e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512da9ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ada9ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4da9f4, SUC_lexicon_tags_15 },
    { 0x8e4da9f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2da9f8, SUC_lexicon_tags_28 },
    { 0x9aada9f9, SUC_lexicon_tags_7 },
    { 0x998da9fa, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6da9fe, SUC_lexicon_tags_11 },
    { 0x28eda9ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89adaa05, SUC_lexicon_tags_7 },
    { 0x6b2daa05, SUC_lexicon_tags_194 },
    { 0xa64daa07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x436daa09, SUC_lexicon_tags_37 },
    { 0x328daa0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x896daa0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc90daa10, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xeedaa12, SUC_lexicon_tags_3 },
    { 0x51adaa13, SUC_lexicon_tags_12 },
    { 0xac8daa13, SUC_lexicon_tags_13 },
    { 0xc06daa13, SUC_lexicon_tags_12 },
    { 0x6e4daa16, SUC_lexicon_tags_7 },
    { 0x396daa15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e4daa19, SUC_lexicon_tags_77 },
    { 0x5eedaa1a, SUC_lexicon_tags_25 },
    { 0x9b8daa1b, SUC_lexicon_tags_37 },
    { 0x490daa1c, SUC_lexicon_tags_1 },
    { 0xcd2daa1d, SUC_lexicon_tags_15 },
    { 0x67edaa1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25edaa21, SUC_lexicon_tags_33 },
    { 0xfd4daa22, SUC_lexicon_tags_15 },
    { 0x278daa23, SUC_lexicon_tags_27 },
    { 0xe5edaa24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x776daa26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22daa2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522daa2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644daa34, SUC_lexicon_tags_2 },
    { 0x2f0daa35, SUC_lexicon_tags_12 },
    { 0xdc8daa35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x232daa39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6daa3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470daa42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4daa45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x290daa47, SUC_lexicon_tags_160 },
    { 0x4c0daa47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaadaa4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2daa4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6edaa5a, SUC_lexicon_tags_27 },
    { 0xa1edaa5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94daa5e, SUC_lexicon_tags_15 },
    { 0xeedaa5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59adaa61, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2daa64, SUC_lexicon_tags_9 },
    { 0x6d8daa65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fadaa6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd36daa6c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cadaa6f, SUC_lexicon_tags_17 },
    { 0x508daa70, SUC_lexicon_tags_15 },
    { 0x392daa71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e0daa73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11adaa77, SUC_lexicon_tags_83 },
    { 0x74daa78, SUC_lexicon_tags_14 },
    { 0xa2edaa79, SUC_lexicon_tags_11 },
    { 0x5fadaa79, SUC_lexicon_tags_22 },
    { 0x2c2daa7b, SUC_lexicon_tags_16 },
    { 0xeaedaa7c, SUC_lexicon_tags_7 },
    { 0xed4daa7d, SUC_lexicon_tags_8 },
    { 0x176daa7c, SUC_lexicon_tags_63 },
    { 0x428daa79, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6fcdaa81, SUC_lexicon_tags_9 },
    { 0xdeadaa82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586daa86, SUC_lexicon_tags_25 },
    { 0xb80daa87, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x266daa89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebcdaa8b, SUC_lexicon_tags_14 },
    { 0x388daa8c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62daa90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x27cdaa93, SUC_lexicon_tags_11 },
    { 0xc0adaa93, SUC_lexicon_tags_66 },
    { 0x92edaa95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdadaa97, SUC_lexicon_tags_87 },
    { 0xe60daa98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00daa9f, SUC_lexicon_tags_29 },
    { 0x70cdaa9f, SUC_lexicon_tags_25 },
    { 0x4e2daaa0, SUC_lexicon_tags_12 },
    { 0xbbcdaa9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cdaaa5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x298daaa8, SUC_lexicon_tags_10 },
    { 0x28edaaa8, SUC_lexicon_tags_1 },
    { 0x56edaaaa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb60daaac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcadaaaf, SUC_lexicon_tags_15 },
    { 0x40cdaab0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fadaab3, SUC_lexicon_tags_15 },
    { 0x95edaab4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecedaab7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcdaabd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x94cdaabf, SUC_lexicon_tags_29 },
    { 0xa8edaabf, SUC_lexicon_tags_95 },
    { 0x228daac1, SUC_lexicon_tags_33 },
    { 0xa9edaac1, SUC_lexicon_tags_7 },
    { 0x5cdaac3, SUC_lexicon_tags_47 },
    { 0x694daac4, SUC_lexicon_tags_1 },
    { 0xcdcdaac5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56daac9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9adaacd, SUC_lexicon_tags_9 },
    { 0x5aedaace, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70daad2, SUC_lexicon_tags_14 },
    { 0x48cdaad3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1adaad9, SUC_lexicon_tags_37 },
    { 0x60cdaada, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x614daadd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590daae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11adaae4, SUC_lexicon_tags_14 },
    { 0x74edaae5, SUC_lexicon_tags_34 },
    { 0xcc8daae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cdaaec, SUC_lexicon_tags_47 },
    { 0x4cdaaed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2daaf0, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4daaf5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1badaafa, SUC_lexicon_tags_9 },
    { 0x98cdaafa, SUC_lexicon_tags_11 },
    { 0x17adaafa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbeedaafe, SUC_lexicon_tags_38 },
    { 0x1b8daaff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aadab04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4dab09, SUC_lexicon_tags_33 },
    { 0xb2edab0a, SUC_lexicon_tags_19 },
    { 0x54edab0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdab11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfacdab13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998dab18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374dab1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x340dab1e, SUC_lexicon_tags_17 },
    { 0xc06dab1f, SUC_lexicon_tags_7 },
    { 0x3d6dab1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x430dab22, SUC_lexicon_tags_15 },
    { 0x784dab23, SUC_lexicon_tags_7 },
    { 0x21adab24, SUC_lexicon_tags_8 },
    { 0xc10dab24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5cdab27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398dab2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8dab36, SUC_lexicon_tags_26 },
    { 0xdf0dab37, SUC_lexicon_tags_136 },
    { 0xa08dab38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fadab3b, SUC_lexicon_tags_8 },
    { 0x3c4dab3c, SUC_lexicon_tags_2 },
    { 0x306dab3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34dab40, SUC_lexicon_tags_32 },
    { 0xffedab40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dab45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0dab48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784dab4c, SUC_lexicon_tags_46 },
    { 0x8bcdab4d, SUC_lexicon_tags_14 },
    { 0x5e4dab4e, SUC_lexicon_tags_11 },
    { 0x540dab4f, SUC_lexicon_tags_70 },
    { 0xbb0dab4f, SUC_lexicon_tags_11 },
    { 0x2b6dab4d, SUC_lexicon_tags_11 },
    { 0xd60dab52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520dab56, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150dab5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0dab5e, SUC_lexicon_tags_62 },
    { 0xb06dab5f, SUC_lexicon_tags_11 },
    { 0x690dab5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x646dab62, SUC_lexicon_tags_37 },
    { 0x80adab63, SUC_lexicon_tags_106 },
    { 0xd0cdab64, SUC_lexicon_tags_26 },
    { 0x4cedab65, SUC_lexicon_tags_37 },
    { 0x4d8dab66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x942dab68, SUC_lexicon_tags_11 },
    { 0xcf6dab69, SUC_lexicon_tags_9 },
    { 0xf9cdab69, SUC_lexicon_tags_30 },
    { 0xb40dab6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826dab70, SUC_lexicon_tags_14 },
    { 0x2b4dab71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38dab75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9adab77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b6dab79, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd00dab7b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x784dab7e, SUC_lexicon_tags_23 },
    { 0xc26dab7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d2dab81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecdab87, SUC_lexicon_tags_7 },
    { 0x2cadab88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62adab8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4dab93, SUC_lexicon_tags_47 },
    { 0x6dedab94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf76dab96, SUC_lexicon_tags_2 },
    { 0x720dab96, SUC_lexicon_tags_30 },
    { 0xc28dab96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbfadab9a, SUC_lexicon_tags_8 },
    { 0xe5edab9b, SUC_lexicon_tags_38 },
    { 0x212dab9b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60daba3, SUC_lexicon_tags_47 },
    { 0xcc0daba4, SUC_lexicon_tags_31 },
    { 0xbaadaba4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dadaba9, SUC_lexicon_tags_4 },
    { 0xb24daba9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16dabae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edabb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea4dabb7, SUC_lexicon_tags_38 },
    { 0xefadabb8, SUC_lexicon_tags_28 },
    { 0xbcedabb8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2dabbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7eedabc0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x60dabc2, SUC_lexicon_tags_15 },
    { 0x754dabc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8dabce, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0dabd4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x52cdabd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418dabdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2edabdd, SUC_lexicon_tags_37 },
    { 0xde2dabde, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6e4dabe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x410dabe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x730dabe4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6dabe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762dabed, SUC_lexicon_tags_25 },
    { 0x6aadabee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50edabf3, SUC_lexicon_tags_29 },
    { 0x21cdabf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2badabf7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd42dabf9, SUC_lexicon_tags_12 },
    { 0x334dabf9, SUC_lexicon_tags_2 },
    { 0x470dabfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5adabff, SUC_lexicon_tags_45 },
    { 0xdd0dac00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530dac05, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8dac08, SUC_lexicon_tags_38 },
    { 0x4dac09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1d2dac0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb82dac0d, SUC_lexicon_tags_13 },
    { 0x4cedac0d, SUC_lexicon_tags_37 },
    { 0x560dac0f, SUC_lexicon_tags_1 },
    { 0x9adac0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56adac16, SUC_lexicon_tags_17 },
    { 0xd64dac17, SUC_lexicon_tags_8 },
    { 0xa14dac18, SUC_lexicon_tags_2 },
    { 0x518dac16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfa2dac1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88dac1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30dac25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfadac27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580dac2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40dac30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4edac36, SUC_lexicon_tags_7 },
    { 0x192dac36, SUC_lexicon_tags_25 },
    { 0xcb8dac38, SUC_lexicon_tags_7 },
    { 0xc4cdac39, SUC_lexicon_tags_83 },
    { 0x6a6dac39, SUC_lexicon_tags_12 },
    { 0xcb4dac3b, SUC_lexicon_tags_113 },
    { 0xcc8dac3b, SUC_lexicon_tags_15 },
    { 0x288dac3d, SUC_lexicon_tags_15 },
    { 0x88cdac3b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd48dac40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2e4dac42, SUC_lexicon_tags_12 },
    { 0xf30dac42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x202dac46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a2dac48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66dac4e, SUC_lexicon_tags_15 },
    { 0x1c0dac4f, SUC_lexicon_tags_15 },
    { 0x4fedac4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x286dac53, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x318dac56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25edac58, SUC_lexicon_tags_55 },
    { 0x77adac58, SUC_lexicon_tags_29 },
    { 0x47adac5a, SUC_lexicon_tags_7 },
    { 0x7fedac5a, SUC_lexicon_tags_11 },
    { 0xf78dac5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f6dac5e, SUC_lexicon_tags_25 },
    { 0xa04dac5f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0dac62, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf08dac64, SUC_lexicon_tags_1 },
    { 0x848dac65, SUC_lexicon_tags_39 },
    { 0xd88dac65, SUC_lexicon_tags_27 },
    { 0x9a4dac67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a4dac69, SUC_lexicon_tags_17 },
    { 0x724dac6a, SUC_lexicon_tags_68 },
    { 0xfe8dac6b, SUC_lexicon_tags_7 },
    { 0x598dac6c, SUC_lexicon_tags_45 },
    { 0x544dac6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe4dac6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acdac75, SUC_lexicon_tags_66 },
    { 0x214dac75, SUC_lexicon_tags_63 },
    { 0xafedac77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x96adac79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80edac7c, SUC_lexicon_tags_19 },
    { 0xc10dac7c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2dac85, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0dac8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cdac8f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246dac93, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x782dac96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4adac98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8cdac9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6dac9e, SUC_lexicon_tags_25 },
    { 0x8f2dac9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccdaca2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x64daca5, SUC_lexicon_tags_25 },
    { 0x8e6daca5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cdaca9, SUC_lexicon_tags_23 },
    { 0xd1adacaa, SUC_lexicon_tags_12 },
    { 0xe2edacab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cdacb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x970dacb5, SUC_lexicon_tags_39 },
    { 0x6badacb6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906dacbb, SUC_lexicon_tags_38 },
    { 0x152dacbc, SUC_lexicon_tags_6 },
    { 0xc06dacbc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02dacc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4dacc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06dacc7, SUC_lexicon_tags_79 },
    { 0xc10dacc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50adacce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x468dacd0, SUC_lexicon_tags_12 },
    { 0xb1adacd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x550dacd3, SUC_lexicon_tags_38 },
    { 0xd38dacd3, SUC_lexicon_tags_8 },
    { 0xbc6dacd5, SUC_lexicon_tags_25 },
    { 0x7e8dacd6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94dacdd, SUC_lexicon_tags_103 },
    { 0xd4cdacde, SUC_lexicon_tags_1 },
    { 0xce4dacdf, SUC_lexicon_tags_1 },
    { 0x158dace0, SUC_lexicon_tags_43 },
    { 0xc54dace1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xacdace3, SUC_lexicon_tags_26 },
    { 0x37edace4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eadace8, SUC_lexicon_tags_47 },
    { 0x72adace8, SUC_lexicon_tags_46 },
    { 0x628dace8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64cdacec, SUC_lexicon_tags_0 },
    { 0x36edaced, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadadacf5, SUC_lexicon_tags_55 },
    { 0xe44dacf6, SUC_lexicon_tags_14 },
    { 0xcf0dacf5, SUC_lexicon_tags_31 },
    { 0x2fcdacf8, SUC_lexicon_tags_65 },
    { 0xf9adacf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcdacfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcdad01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d8dad03, SUC_lexicon_tags_31 },
    { 0x9ccdad03, SUC_lexicon_tags_11 },
    { 0x3fadad05, SUC_lexicon_tags_11 },
    { 0x266dad06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86dad0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x532dad0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc68dad0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cdad12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2dad18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98adad24, SUC_lexicon_tags_8 },
    { 0x2d0dad24, SUC_lexicon_tags_2 },
    { 0x7f6dad26, SUC_lexicon_tags_11 },
    { 0x1b4dad26, SUC_lexicon_tags_15 },
    { 0x886dad25, SUC_lexicon_tags_25 },
    { 0x35edad29, SUC_lexicon_tags_8 },
    { 0xb88dad2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcfedad2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88edad37, SUC_lexicon_tags_77 },
    { 0xa5adad38, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x224dad3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2dad40, SUC_lexicon_tags_2 },
    { 0xd42dad41, SUC_lexicon_tags_2 },
    { 0x30cdad42, SUC_lexicon_tags_11 },
    { 0xd14dad42, SUC_lexicon_tags_25 },
    { 0x4dcdad41, SUC_lexicon_tags_15 },
    { 0x810dad45, SUC_lexicon_tags_60 },
    { 0xbaadad45, SUC_lexicon_tags_38 },
    { 0xa74dad46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x31edad4a, SUC_lexicon_tags_25 },
    { 0x5b2dad4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb0dad4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4dad50, SUC_lexicon_tags_48 },
    { 0x4c6dad51, SUC_lexicon_tags_9 },
    { 0xba0dad52, SUC_lexicon_tags_1 },
    { 0x99adad50, SUC_lexicon_tags_1 },
    { 0x780dad52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x19adad57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10dad62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1adad65, SUC_lexicon_tags_7 },
    { 0x642dad66, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadad6c, SUC_lexicon_tags_15 },
    { 0xb40dad6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8edad78, SUC_lexicon_tags_73 },
    { 0x512dad78, SUC_lexicon_tags_30 },
    { 0x530dad79, SUC_lexicon_tags_11 },
    { 0x330dad7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x154dad7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93cdad7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeb2dad81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa34dad83, SUC_lexicon_tags_25 },
    { 0xe88dad83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb58dad86, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaedad89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bedad91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaadad95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbf0dad97, SUC_lexicon_tags_46 },
    { 0x696dad98, SUC_lexicon_tags_37 },
    { 0xd22dad99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40edada0, SUC_lexicon_tags_23 },
    { 0xdfcdada1, SUC_lexicon_tags_12 },
    { 0xb6adada1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6dadab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecdadaf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x274dadb2, SUC_lexicon_tags_0 },
    { 0x88cdadb3, SUC_lexicon_tags_43 },
    { 0xceedadb4, SUC_lexicon_tags_17 },
    { 0xf14dadb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x724dadb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x442dadba, SUC_lexicon_tags_0 },
    { 0x4c2dadbb, SUC_lexicon_tags_25 },
    { 0x9aadadbb, SUC_lexicon_tags_39 },
    { 0x9fcdadbd, SUC_lexicon_tags_8 },
    { 0xaa6dadbd, SUC_lexicon_tags_15 },
    { 0xd3cdadbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3badadc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc4dadc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc92dadc6, SUC_lexicon_tags_37 },
    { 0xe9cdadc7, SUC_lexicon_tags_160 },
    { 0x9e8dadc8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12dadcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x470dadd1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x368dadd3, SUC_lexicon_tags_11 },
    { 0xf50dadd3, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xe3adadd6, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x6d8dadd8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4daddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefadadde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fedade7, SUC_lexicon_tags_27 },
    { 0xc16dade8, SUC_lexicon_tags_60 },
    { 0x548dade9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72dadee, SUC_lexicon_tags_16 },
    { 0x7d0dadee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4acdadf1, SUC_lexicon_tags_11 },
    { 0x316dadf2, SUC_lexicon_tags_15 },
    { 0xc64dadf3, SUC_lexicon_tags_22 },
    { 0xd18dadf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912dadf8, SUC_lexicon_tags_47 },
    { 0x39cdadf8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136dae02, SUC_lexicon_tags_38 },
    { 0xd3edae03, SUC_lexicon_tags_14 },
    { 0x28edae04, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x540dae06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccdae0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecdae0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb0dae11, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456dae17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedae1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xadedae1e, SUC_lexicon_tags_74 },
    { 0xbaadae1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4dae24, SUC_lexicon_tags_38 },
    { 0x336dae24, SUC_lexicon_tags_4 },
    { 0xf5cdae25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8c6dae28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x666dae2a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4dae2d, SUC_lexicon_tags_44 },
    { 0xbc4dae2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4dae32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2eadae34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4dae3c, SUC_lexicon_tags_236 },
    { 0xd84dae3d, SUC_lexicon_tags_38 },
    { 0xcf0dae3e, SUC_lexicon_tags_2 },
    { 0x516dae3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67adae44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f2dae46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72adae49, SUC_lexicon_tags_37 },
    { 0xe26dae4a, SUC_lexicon_tags_14 },
    { 0x190dae4b, SUC_lexicon_tags_45 },
    { 0x6c8dae4c, SUC_lexicon_tags_160 },
    { 0x4b0dae4d, SUC_lexicon_tags_31 },
    { 0x352dae4e, SUC_lexicon_tags_37 },
    { 0x572dae4f, SUC_lexicon_tags_14 },
    { 0x44edae49, SUC_lexicon_tags_11 },
    { 0x6f4dae49, SUC_lexicon_tags_66 },
    { 0xca6dae52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x956dae54, SUC_lexicon_tags_35 },
    { 0xae2dae55, SUC_lexicon_tags_29 },
    { 0x83adae56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4dae59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaadae5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4dae65, SUC_lexicon_tags_1 },
    { 0xe9cdae65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13adae6b, SUC_lexicon_tags_12 },
    { 0xb56dae6c, SUC_lexicon_tags_2 },
    { 0x29cdae6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cdae70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72dae76, SUC_lexicon_tags_2 },
    { 0xc46dae77, SUC_lexicon_tags_11 },
    { 0x52dae78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x362dae7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76dae7e, SUC_lexicon_tags_0 },
    { 0x184dae7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac4dae81, SUC_lexicon_tags_30 },
    { 0xb52dae81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2dae88, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21edae8b, SUC_lexicon_tags_63 },
    { 0xe72dae8c, SUC_lexicon_tags_29 },
    { 0xd0adae8c, SUC_lexicon_tags_25 },
    { 0xd62dae8d, SUC_lexicon_tags_17 },
    { 0xa1cdae8f, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xca4dae91, SUC_lexicon_tags_151 },
    { 0xe68dae92, SUC_lexicon_tags_10 },
    { 0xe96dae92, SUC_lexicon_tags_55 },
    { 0xb52dae93, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x700dae96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392dae9b, SUC_lexicon_tags_2 },
    { 0x12cdae9c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436daea3, SUC_lexicon_tags_11 },
    { 0xfc4daea3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x124daea6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc34daea8, SUC_lexicon_tags_214 },
    { 0x5d8daea9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x718daeab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264daeb6, SUC_lexicon_tags_27 },
    { 0x15adaeb7, SUC_lexicon_tags_43 },
    { 0x6cadaeb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978daec1, SUC_lexicon_tags_11 },
    { 0xbf6daec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e4daec4, SUC_lexicon_tags_7 },
    { 0x14daec5, SUC_lexicon_tags_21 },
    { 0xeeadaec6, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x26cdaec9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6daecc, SUC_lexicon_tags_60 },
    { 0xd8cdaecd, SUC_lexicon_tags_43 },
    { 0x978daece, SUC_lexicon_tags_15 },
    { 0xbf2daecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c6daed1, SUC_lexicon_tags_57 },
    { 0x702daed2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x704daed4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x874daed7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97adaed9, SUC_lexicon_tags_25 },
    { 0xabadaed9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778daede, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xab8daee0, SUC_lexicon_tags_10 },
    { 0x12daee1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadaee7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160daeeb, SUC_lexicon_tags_23 },
    { 0x79edaeec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c2daeee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x706daef0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4daef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6adaefd, SUC_lexicon_tags_23 },
    { 0xf1adaefe, SUC_lexicon_tags_37 },
    { 0x218daefe, SUC_lexicon_tags_56 },
    { 0x65cdaefd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4daf04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f0daf06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988daf0e, SUC_lexicon_tags_23 },
    { 0xefcdaf0f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x284daf12, SUC_lexicon_tags_8 },
    { 0xf56daf12, SUC_lexicon_tags_15 },
    { 0x986daf12, SUC_lexicon_tags_33 },
    { 0x9d4daf15, SUC_lexicon_tags_12 },
    { 0xaedaf14, SUC_lexicon_tags_12 },
    { 0xef2daf17, SUC_lexicon_tags_2 },
    { 0xfcadaf18, SUC_lexicon_tags_27 },
    { 0x696daf19, SUC_lexicon_tags_11 },
    { 0xba0daf19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cdaf1e, SUC_lexicon_tags_68 },
    { 0x1c6daf1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x160daf22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02daf27, SUC_lexicon_tags_25 },
    { 0xb28daf28, SUC_lexicon_tags_33 },
    { 0xd06daf29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6daf2c, SUC_lexicon_tags_35 },
    { 0x78cdaf2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06daf35, SUC_lexicon_tags_27 },
    { 0xe6daf35, SUC_lexicon_tags_15 },
    { 0x83cdaf37, SUC_lexicon_tags_7 },
    { 0x184daf38, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x38adaf3b, SUC_lexicon_tags_38 },
    { 0xfccdaf3b, SUC_lexicon_tags_25 },
    { 0x600daf3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62daf3f, SUC_lexicon_tags_937 },
    { 0x384daf3f, SUC_lexicon_tags_8 },
    { 0xbc2daf41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x982daf44, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf2daf46, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x410daf48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x806daf4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630daf4f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66daf52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10edaf56, SUC_lexicon_tags_25 },
    { 0xafadaf57, SUC_lexicon_tags_11 },
    { 0x510daf58, SUC_lexicon_tags_14 },
    { 0x34edaf58, SUC_lexicon_tags_12 },
    { 0x990daf5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08daf5d, SUC_lexicon_tags_17 },
    { 0xf7cdaf5e, SUC_lexicon_tags_45 },
    { 0x890daf5e, SUC_lexicon_tags_1 },
    { 0x87edaf60, SUC_lexicon_tags_4 },
    { 0x256daf61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18daf65, SUC_lexicon_tags_25 },
    { 0x114daf66, SUC_lexicon_tags_10 },
    { 0x4acdaf66, SUC_lexicon_tags_15 },
    { 0x1ccdaf66, SUC_lexicon_tags_170 },
    { 0x834daf68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2daf6b, SUC_lexicon_tags_0 },
    { 0x452daf6c, SUC_lexicon_tags_17 },
    { 0x166daf6d, SUC_lexicon_tags_37 },
    { 0x74adaf6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff4daf70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74adaf72, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cdaf77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2daf7e, SUC_lexicon_tags_14 },
    { 0x5fadaf7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6daf83, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420daf8c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4daf91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x356daf93, SUC_lexicon_tags_43 },
    { 0xbd6daf94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0daf97, SUC_lexicon_tags_25 },
    { 0x522daf98, SUC_lexicon_tags_2 },
    { 0xd12daf99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f4daf9b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80daf9e, SUC_lexicon_tags_47 },
    { 0xc6daf9f, SUC_lexicon_tags_15 },
    { 0x97edafa0, SUC_lexicon_tags_2 },
    { 0x5edaf9f, SUC_lexicon_tags_17 },
    { 0xae6dafa1, SUC_lexicon_tags_19 },
    { 0x9b6dafa3, SUC_lexicon_tags_15 },
    { 0x47cdafa1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8dafab, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xfeedafad, SUC_lexicon_tags_7 },
    { 0x2dadafae, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x310dafb0, SUC_lexicon_tags_11 },
    { 0xa70dafb0, SUC_lexicon_tags_66 },
    { 0xb80dafb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82adafb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcdafb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8dafbc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0dafc0, SUC_lexicon_tags_46 },
    { 0x68adafc0, SUC_lexicon_tags_27 },
    { 0x7fedafc1, SUC_lexicon_tags_2 },
    { 0xa32dafc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x464dafc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576dafca, SUC_lexicon_tags_82 },
    { 0x192dafcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b2dafcd, SUC_lexicon_tags_37 },
    { 0xc26dafcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268dafd4, SUC_lexicon_tags_106 },
    { 0x91edafd4, SUC_lexicon_tags_32 },
    { 0x220dafd5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82dafd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212dafdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa18dafdf, SUC_lexicon_tags_15 },
    { 0x446dafe0, SUC_lexicon_tags_1126 },
    { 0, NULL },
    { 0x42edafe2, SUC_lexicon_tags_31 },
    { 0x9e0dafe3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2dafe9, SUC_lexicon_tags_11 },
    { 0xc1edafea, SUC_lexicon_tags_25 },
    { 0x7c2dafeb, SUC_lexicon_tags_23 },
    { 0x136dafeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6daff2, SUC_lexicon_tags_38 },
    { 0x302daff3, SUC_lexicon_tags_28 },
    { 0xd1cdaff3, SUC_lexicon_tags_37 },
    { 0x386daff5, SUC_lexicon_tags_26 },
    { 0xb8edaff3, SUC_lexicon_tags_7 },
    { 0xec0daff7, SUC_lexicon_tags_70 },
    { 0x14cdaff8, SUC_lexicon_tags_12 },
    { 0xeb0daff3, SUC_lexicon_tags_25 },
    { 0x810daffa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd86daffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1edafff, SUC_lexicon_tags_11 },
    { 0xa44db000, SUC_lexicon_tags_37 },
    { 0x916dafff, SUC_lexicon_tags_12 },
    { 0x1eadb002, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xabcdb005, SUC_lexicon_tags_37 },
    { 0x7d6db006, SUC_lexicon_tags_28 },
    { 0xf30db005, SUC_lexicon_tags_29 },
    { 0x2fcdb008, SUC_lexicon_tags_25 },
    { 0x330db007, SUC_lexicon_tags_12 },
    { 0x64adb00a, SUC_lexicon_tags_46 },
    { 0x7b0db00a, SUC_lexicon_tags_63 },
    { 0x2d8db00c, SUC_lexicon_tags_2 },
    { 0xf36db00a, SUC_lexicon_tags_9 },
    { 0x964db00e, SUC_lexicon_tags_61 },
    { 0xe22db00d, SUC_lexicon_tags_42 },
    { 0xfcdb009, SUC_lexicon_tags_61 },
    { 0x27cdb011, SUC_lexicon_tags_17 },
    { 0x75edb011, SUC_lexicon_tags_37 },
    { 0x346db00d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408db018, SUC_lexicon_tags_45 },
    { 0xc1adb019, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5badb01e, SUC_lexicon_tags_7 },
    { 0x39adb01f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8db025, SUC_lexicon_tags_15 },
    { 0x204db026, SUC_lexicon_tags_43 },
    { 0x3badb025, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6db02b, SUC_lexicon_tags_32 },
    { 0x6adb02b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x828db02f, SUC_lexicon_tags_37 },
    { 0xfd2db02f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x302db033, SUC_lexicon_tags_7 },
    { 0x382db034, SUC_lexicon_tags_38 },
    { 0xcaadb035, SUC_lexicon_tags_5 },
    { 0x548db036, SUC_lexicon_tags_7 },
    { 0x76adb034, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cedb03c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d8db03e, SUC_lexicon_tags_25 },
    { 0x7b2db03f, SUC_lexicon_tags_15 },
    { 0x3bcdb040, SUC_lexicon_tags_12 },
    { 0x96cdb041, SUC_lexicon_tags_57 },
    { 0xf48db03f, SUC_lexicon_tags_19 },
    { 0xa6edb041, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7b8db045, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x884db047, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbadb049, SUC_lexicon_tags_12 },
    { 0x5ecdb04a, SUC_lexicon_tags_16 },
    { 0xa16db04b, SUC_lexicon_tags_2 },
    { 0x428db04a, SUC_lexicon_tags_7 },
    { 0xa24db04c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32adb051, SUC_lexicon_tags_27 },
    { 0xbfcdb052, SUC_lexicon_tags_27 },
    { 0xaf4db053, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f8db055, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664db05e, SUC_lexicon_tags_6 },
    { 0x7ecdb05e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd18db061, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x10edb063, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cdb068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2db06d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778db071, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598db075, SUC_lexicon_tags_15 },
    { 0xa0cdb075, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62db07a, SUC_lexicon_tags_21 },
    { 0x648db07b, SUC_lexicon_tags_11 },
    { 0xebadb07c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x362db07e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xafedb081, SUC_lexicon_tags_47 },
    { 0xa24db082, SUC_lexicon_tags_34 },
    { 0x43edb081, SUC_lexicon_tags_2 },
    { 0x308db081, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd46db086, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa0adb088, SUC_lexicon_tags_56 },
    { 0xcdedb089, SUC_lexicon_tags_46 },
    { 0x4b2db08a, SUC_lexicon_tags_7 },
    { 0xe72db08b, SUC_lexicon_tags_14 },
    { 0xca2db08c, SUC_lexicon_tags_84 },
    { 0x43edb08c, SUC_lexicon_tags_11 },
    { 0xab8db08b, SUC_lexicon_tags_11 },
    { 0x29edb08f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbedb091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x326db094, SUC_lexicon_tags_17 },
    { 0xc8edb094, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526db09a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdb09f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acdb0aa, SUC_lexicon_tags_55 },
    { 0xe2adb0ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacadb0ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edb0b4, SUC_lexicon_tags_7 },
    { 0x4dcdb0b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cdb0bb, SUC_lexicon_tags_17 },
    { 0x4a4db0bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adb0c3, SUC_lexicon_tags_17 },
    { 0x2e2db0c4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x546db0c7, SUC_lexicon_tags_17 },
    { 0xeeadb0c7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fadb0d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cdb0d5, SUC_lexicon_tags_16 },
    { 0xc46db0d6, SUC_lexicon_tags_7 },
    { 0xa1cdb0d7, SUC_lexicon_tags_37 },
    { 0x19edb0d7, SUC_lexicon_tags_198 },
    { 0x526db0d6, SUC_lexicon_tags_14 },
    { 0xafadb0da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10db0df, SUC_lexicon_tags_15 },
    { 0xe40db0df, SUC_lexicon_tags_15 },
    { 0xfe4db0e1, SUC_lexicon_tags_7 },
    { 0x878db0df, SUC_lexicon_tags_28 },
    { 0x2fedb0e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd4db0e5, SUC_lexicon_tags_2 },
    { 0x456db0e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x338db0e8, SUC_lexicon_tags_16 },
    { 0xc50db0e9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xadadb0ec, SUC_lexicon_tags_12 },
    { 0x24db0ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x584db0f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc18db0f3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9eedb0f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42db0f8, SUC_lexicon_tags_125 },
    { 0xebadb0f8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8db0fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x534db100, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2db103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410db107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94db10b, SUC_lexicon_tags_1 },
    { 0x926db10c, SUC_lexicon_tags_56 },
    { 0xb0edb10c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cdb114, SUC_lexicon_tags_47 },
    { 0x310db115, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76db11a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0db11e, SUC_lexicon_tags_2 },
    { 0x7dedb11e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x262db121, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x218db124, SUC_lexicon_tags_46 },
    { 0x9c0db125, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cdb12a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3adb12c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x250db12f, SUC_lexicon_tags_2 },
    { 0xf10db130, SUC_lexicon_tags_55 },
    { 0x5e4db131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bedb13a, SUC_lexicon_tags_38 },
    { 0x63cdb13b, SUC_lexicon_tags_1125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57adb140, SUC_lexicon_tags_11 },
    { 0x182db141, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb6db143, SUC_lexicon_tags_11 },
    { 0x268db143, SUC_lexicon_tags_37 },
    { 0xe5adb144, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98edb149, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd54db14b, SUC_lexicon_tags_37 },
    { 0x7b2db14b, SUC_lexicon_tags_66 },
    { 0x6badb14b, SUC_lexicon_tags_0 },
    { 0xac6db14e, SUC_lexicon_tags_2 },
    { 0xa28db14f, SUC_lexicon_tags_0 },
    { 0x71adb150, SUC_lexicon_tags_54 },
    { 0x108db151, SUC_lexicon_tags_55 },
    { 0x194db150, SUC_lexicon_tags_31 },
    { 0x760db153, SUC_lexicon_tags_12 },
    { 0x7d4db154, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x712db157, SUC_lexicon_tags_56 },
    { 0xf28db158, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4db15b, SUC_lexicon_tags_4 },
    { 0x6bcdb15b, SUC_lexicon_tags_40 },
    { 0xc4adb15d, SUC_lexicon_tags_25 },
    { 0x64edb15b, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xccedb161, SUC_lexicon_tags_100 },
    { 0xbd6db162, SUC_lexicon_tags_1 },
    { 0xc7adb161, SUC_lexicon_tags_49 },
    { 0x1b2db164, SUC_lexicon_tags_25 },
    { 0x8e0db163, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82db169, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa0db16b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ccdb16d, SUC_lexicon_tags_1 },
    { 0x15cdb16e, SUC_lexicon_tags_11 },
    { 0x9ccdb16e, SUC_lexicon_tags_14 },
    { 0x880db16e, SUC_lexicon_tags_8 },
    { 0xcdadb170, SUC_lexicon_tags_23 },
    { 0x55edb172, SUC_lexicon_tags_65 },
    { 0xcb6db173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cdb17b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4fadb17d, SUC_lexicon_tags_1 },
    { 0xefcdb17e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68db182, SUC_lexicon_tags_19 },
    { 0x6c6db182, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87adb187, SUC_lexicon_tags_6 },
    { 0xa06db187, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacdb18d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc72db18f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbadb193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x840db195, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8edb198, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40edb19c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3adb19e, SUC_lexicon_tags_6 },
    { 0x1fedb19f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4db1a2, SUC_lexicon_tags_37 },
    { 0xa38db1a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84db1a6, SUC_lexicon_tags_0 },
    { 0xfa4db1a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aadb1aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc62db1ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8db1b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0db1b6, SUC_lexicon_tags_45 },
    { 0x8ccdb1b6, SUC_lexicon_tags_15 },
    { 0xff8db1b8, SUC_lexicon_tags_55 },
    { 0xef2db1b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f2db1bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984db1c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa12db1c3, SUC_lexicon_tags_25 },
    { 0x62edb1c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8db1ca, SUC_lexicon_tags_2 },
    { 0xb26db1cb, SUC_lexicon_tags_13 },
    { 0xdb8db1cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x110db1ce, SUC_lexicon_tags_414 },
    { 0xd38db1cf, SUC_lexicon_tags_11 },
    { 0x54adb1d0, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26db1d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59adb1d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9adb1d9, SUC_lexicon_tags_17 },
    { 0x7aadb1da, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x32cdb1dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacdb1eb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186db1ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x44adb1f2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x652db1f5, SUC_lexicon_tags_165 },
    { 0xfdadb1f5, SUC_lexicon_tags_33 },
    { 0xb32db1f7, SUC_lexicon_tags_68 },
    { 0xd74db1f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4db1fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdb204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96db207, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb1cdb209, SUC_lexicon_tags_12 },
    { 0x80edb20a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57edb20f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30db218, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36db21f, SUC_lexicon_tags_11 },
    { 0xcb6db21f, SUC_lexicon_tags_11 },
    { 0xd86db221, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2db22a, SUC_lexicon_tags_49 },
    { 0xf8cdb22b, SUC_lexicon_tags_17 },
    { 0xef6db22a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x250db22f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46db232, SUC_lexicon_tags_25 },
    { 0xfd4db233, SUC_lexicon_tags_11 },
    { 0x230db234, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0db237, SUC_lexicon_tags_8 },
    { 0xaeadb238, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb3edb23a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440db23e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70db242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x540db245, SUC_lexicon_tags_2 },
    { 0xf2edb246, SUC_lexicon_tags_17 },
    { 0x3badb247, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32db24d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cdb250, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6edb252, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4db25b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6db25e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8dcdb260, SUC_lexicon_tags_37 },
    { 0xad0db260, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x248db264, SUC_lexicon_tags_0 },
    { 0x8a2db264, SUC_lexicon_tags_7 },
    { 0xc38db266, SUC_lexicon_tags_0 },
    { 0xe96db267, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2db26a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2db26d, SUC_lexicon_tags_37 },
    { 0x40edb26e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde2db270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x530db272, SUC_lexicon_tags_91 },
    { 0x34cdb273, SUC_lexicon_tags_27 },
    { 0x330db274, SUC_lexicon_tags_45 },
    { 0x1d0db274, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaedb277, SUC_lexicon_tags_37 },
    { 0x8dadb278, SUC_lexicon_tags_8 },
    { 0xb6db277, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xd0cdb27b, SUC_lexicon_tags_3 },
    { 0xcc6db27c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6aedb27e, SUC_lexicon_tags_15 },
    { 0xa94db27e, SUC_lexicon_tags_25 },
    { 0x756db27e, SUC_lexicon_tags_15 },
    { 0x20edb27e, SUC_lexicon_tags_45 },
    { 0x52adb280, SUC_lexicon_tags_11 },
    { 0x6d8db283, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28adb289, SUC_lexicon_tags_65 },
    { 0x70edb28a, SUC_lexicon_tags_68 },
    { 0xd26db28b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27adb28f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77edb293, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500db299, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1f8db29b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x368db29e, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31edb2a4, SUC_lexicon_tags_7 },
    { 0x60cdb2a4, SUC_lexicon_tags_33 },
    { 0xfc2db2a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e6db2a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x302db2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32db2af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aedb2b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65adb2b5, SUC_lexicon_tags_65 },
    { 0xa4adb2b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8db2b9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42db2bd, SUC_lexicon_tags_0 },
    { 0xc08db2be, SUC_lexicon_tags_37 },
    { 0x136db2be, SUC_lexicon_tags_0 },
    { 0x9fedb2bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1f2db2c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cdb2c7, SUC_lexicon_tags_11 },
    { 0x12db2c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56db2ca, SUC_lexicon_tags_77 },
    { 0xa5cdb2ca, SUC_lexicon_tags_47 },
    { 0xeeedb2cc, SUC_lexicon_tags_25 },
    { 0x90adb2cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d4db2cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cdb2d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x350db2d8, SUC_lexicon_tags_17 },
    { 0xb72db2d8, SUC_lexicon_tags_17 },
    { 0xdc0db2da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa22db2dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4db2e0, SUC_lexicon_tags_37 },
    { 0x9f4db2e0, SUC_lexicon_tags_48 },
    { 0x864db2e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebadb2ef, SUC_lexicon_tags_7 },
    { 0x1b4db2ef, SUC_lexicon_tags_29 },
    { 0x21adb2f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8edb2f4, SUC_lexicon_tags_10 },
    { 0xf92db2f5, SUC_lexicon_tags_15 },
    { 0xa36db2f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78db2f9, SUC_lexicon_tags_37 },
    { 0x4d8db2f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5dcdb2fc, SUC_lexicon_tags_2 },
    { 0x546db2fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6edb301, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x446db304, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x170db306, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x186db308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x286db30a, SUC_lexicon_tags_49 },
    { 0x666db30a, SUC_lexicon_tags_12 },
    { 0x5edb30c, SUC_lexicon_tags_12 },
    { 0xa6db30b, SUC_lexicon_tags_11 },
    { 0xdf8db30d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd08db310, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874db317, SUC_lexicon_tags_1 },
    { 0x666db318, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0db31b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x946db31d, SUC_lexicon_tags_27 },
    { 0x1f0db31d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x14cdb320, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8db322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cedb327, SUC_lexicon_tags_63 },
    { 0xf44db327, SUC_lexicon_tags_11 },
    { 0x9aadb329, SUC_lexicon_tags_15 },
    { 0xcadb329, SUC_lexicon_tags_15 },
    { 0x354db32a, SUC_lexicon_tags_11 },
    { 0x20adb329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8db330, SUC_lexicon_tags_23 },
    { 0x63cdb331, SUC_lexicon_tags_28 },
    { 0xb6db331, SUC_lexicon_tags_8 },
    { 0x258db333, SUC_lexicon_tags_46 },
    { 0x5e4db330, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5e0db336, SUC_lexicon_tags_1 },
    { 0x1fedb337, SUC_lexicon_tags_37 },
    { 0xab2db336, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x232db33a, SUC_lexicon_tags_18 },
    { 0xa78db33b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe6db33d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33edb340, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24edb342, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c2db344, SUC_lexicon_tags_16 },
    { 0x14cdb344, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xacadb347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x298db34a, SUC_lexicon_tags_25 },
    { 0x934db34b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22db350, SUC_lexicon_tags_37 },
    { 0xe90db350, SUC_lexicon_tags_1 },
    { 0xef4db351, SUC_lexicon_tags_11 },
    { 0x298db353, SUC_lexicon_tags_37 },
    { 0x54cdb354, SUC_lexicon_tags_7 },
    { 0xcdcdb355, SUC_lexicon_tags_26 },
    { 0xae2db355, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1adb358, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0db35c, SUC_lexicon_tags_23 },
    { 0x95cdb35d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x97adb35f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708db363, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74db367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x118db369, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x440db36c, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x248db36e, SUC_lexicon_tags_12 },
    { 0x3badb36f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbadb372, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952db376, SUC_lexicon_tags_62 },
    { 0xa56db376, SUC_lexicon_tags_8 },
    { 0x810db378, SUC_lexicon_tags_37 },
    { 0xcfedb378, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0db37c, SUC_lexicon_tags_4 },
    { 0x242db37c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228db383, SUC_lexicon_tags_13 },
    { 0xe16db384, SUC_lexicon_tags_3 },
    { 0x5f4db385, SUC_lexicon_tags_11 },
    { 0x75cdb386, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88db389, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24db39a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898db39e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32db3a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568db3a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00db3a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edb3ae, SUC_lexicon_tags_25 },
    { 0x3bcdb3af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b0db3b1, SUC_lexicon_tags_38 },
    { 0x964db3b2, SUC_lexicon_tags_25 },
    { 0x876db3b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cadb3b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x828db3b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb6db3ba, SUC_lexicon_tags_43 },
    { 0x8eadb3bb, SUC_lexicon_tags_27 },
    { 0xad4db3bc, SUC_lexicon_tags_14 },
    { 0x138db3bd, SUC_lexicon_tags_1 },
    { 0x20adb3bc, SUC_lexicon_tags_27 },
    { 0x662db3bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0db3c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4edb3c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbbadb3ca, SUC_lexicon_tags_66 },
    { 0xe58db3cb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x414db3cd, SUC_lexicon_tags_37 },
    { 0x7c8db3cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8db3d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6eadb3d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996db3d8, SUC_lexicon_tags_14 },
    { 0x792db3d9, SUC_lexicon_tags_43 },
    { 0xc7adb3d9, SUC_lexicon_tags_387 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6db3e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x244db3e4, SUC_lexicon_tags_14 },
    { 0x238db3e5, SUC_lexicon_tags_15 },
    { 0x82edb3e6, SUC_lexicon_tags_33 },
    { 0x65adb3e4, SUC_lexicon_tags_64 },
    { 0x1e0db3e8, SUC_lexicon_tags_29 },
    { 0x1b8db3e9, SUC_lexicon_tags_15 },
    { 0xcbadb3e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2edb3ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfedb3f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76db3fe, SUC_lexicon_tags_37 },
    { 0x456db3ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36db404, SUC_lexicon_tags_34 },
    { 0x358db405, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95edb408, SUC_lexicon_tags_66 },
    { 0x3f0db408, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72db411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8edb413, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x644db415, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaadb41c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa62db41e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x40adb420, SUC_lexicon_tags_60 },
    { 0x43adb420, SUC_lexicon_tags_17 },
    { 0x44cdb422, SUC_lexicon_tags_17 },
    { 0xa04db422, SUC_lexicon_tags_28 },
    { 0xb9cdb423, SUC_lexicon_tags_26 },
    { 0x804db425, SUC_lexicon_tags_1 },
    { 0x500db425, SUC_lexicon_tags_56 },
    { 0xc02db427, SUC_lexicon_tags_21 },
    { 0xbfedb425, SUC_lexicon_tags_23 },
    { 0x44db429, SUC_lexicon_tags_12 },
    { 0x10edb422, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x812db42c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04db430, SUC_lexicon_tags_19 },
    { 0xb4edb431, SUC_lexicon_tags_25 },
    { 0xe30db431, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830db439, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x642db43b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4db440, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34edb453, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2e6db455, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xebadb458, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc2cdb45a, SUC_lexicon_tags_2 },
    { 0xeaadb45a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148db464, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe28db466, SUC_lexicon_tags_13 },
    { 0xcb4db466, SUC_lexicon_tags_15 },
    { 0x140db468, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa96db46a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf12db46d, SUC_lexicon_tags_29 },
    { 0x9dedb46e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8db473, SUC_lexicon_tags_14 },
    { 0x45adb473, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84db477, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa28db479, SUC_lexicon_tags_15 },
    { 0xe58db479, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe80db47c, SUC_lexicon_tags_25 },
    { 0x24edb47c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2cdb47f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50db488, SUC_lexicon_tags_45 },
    { 0x492db489, SUC_lexicon_tags_2 },
    { 0xcd4db489, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40db48f, SUC_lexicon_tags_15 },
    { 0xc5edb490, SUC_lexicon_tags_7 },
    { 0xa58db48f, SUC_lexicon_tags_133 },
    { 0xc6db492, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb0db494, SUC_lexicon_tags_15 },
    { 0x47cdb495, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x87cdb497, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4db49e, SUC_lexicon_tags_37 },
    { 0xd36db49f, SUC_lexicon_tags_7 },
    { 0xb32db4a0, SUC_lexicon_tags_62 },
    { 0x476db4a0, SUC_lexicon_tags_7 },
    { 0x782db49f, SUC_lexicon_tags_11 },
    { 0xb8cdb4a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe52db4a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69edb4a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76db4ac, SUC_lexicon_tags_1 },
    { 0x440db4ad, SUC_lexicon_tags_66 },
    { 0x5fadb4ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6db4b2, SUC_lexicon_tags_21 },
    { 0xb6cdb4b3, SUC_lexicon_tags_12 },
    { 0xf8cdb4b3, SUC_lexicon_tags_17 },
    { 0x6eedb4b5, SUC_lexicon_tags_213 },
    { 0xb9adb4b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4eadb4b8, SUC_lexicon_tags_37 },
    { 0x61adb4b9, SUC_lexicon_tags_29 },
    { 0xad4db4ba, SUC_lexicon_tags_47 },
    { 0x7aedb4bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700db4bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874db4c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8db4cc, SUC_lexicon_tags_45 },
    { 0xd90db4cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8db4d1, SUC_lexicon_tags_15 },
    { 0xc58db4d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cdb4dc, SUC_lexicon_tags_31 },
    { 0x566db4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11edb4df, SUC_lexicon_tags_0 },
    { 0x81adb4df, SUC_lexicon_tags_11 },
    { 0x640db4e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99adb4e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c2db4e7, SUC_lexicon_tags_11 },
    { 0x68db4e8, SUC_lexicon_tags_19 },
    { 0xce8db4e9, SUC_lexicon_tags_12 },
    { 0x428db4e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32db4ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260db4f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x494db4f5, SUC_lexicon_tags_27 },
    { 0xd9edb4f6, SUC_lexicon_tags_40 },
    { 0x310db4f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fedb4fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2db502, SUC_lexicon_tags_11 },
    { 0x1b4db503, SUC_lexicon_tags_60 },
    { 0xf0edb504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96db507, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb60db50a, SUC_lexicon_tags_42 },
    { 0x570db50a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x738db50e, SUC_lexicon_tags_39 },
    { 0x106db50f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23edb511, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf60db513, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeedb51b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adb51f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x77edb522, SUC_lexicon_tags_25 },
    { 0xce4db523, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f6db525, SUC_lexicon_tags_1 },
    { 0xd4db526, SUC_lexicon_tags_17 },
    { 0x85cdb527, SUC_lexicon_tags_47 },
    { 0x322db526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x152db52b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee6db52d, SUC_lexicon_tags_44 },
    { 0x1fcdb52e, SUC_lexicon_tags_2 },
    { 0xfd2db52d, SUC_lexicon_tags_37 },
    { 0x45edb530, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70db535, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xceadb538, SUC_lexicon_tags_32 },
    { 0xb96db539, SUC_lexicon_tags_9 },
    { 0x3f4db538, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6db53d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8db540, SUC_lexicon_tags_53 },
    { 0x3c6db541, SUC_lexicon_tags_33 },
    { 0x504db541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2db544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x542db547, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf48db549, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692db552, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50db558, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880db55d, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adb570, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x68edb572, SUC_lexicon_tags_60 },
    { 0xe48db573, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccdb576, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4db57d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfedb582, SUC_lexicon_tags_14 },
    { 0x59adb583, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78db587, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04db58b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x288db58e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa60db590, SUC_lexicon_tags_1 },
    { 0xaadb591, SUC_lexicon_tags_1 },
    { 0xa68db592, SUC_lexicon_tags_25 },
    { 0xeecdb593, SUC_lexicon_tags_17 },
    { 0x230db594, SUC_lexicon_tags_57 },
    { 0x930db592, SUC_lexicon_tags_17 },
    { 0x180db596, SUC_lexicon_tags_1 },
    { 0xa90db596, SUC_lexicon_tags_131 },
    { 0xb80db598, SUC_lexicon_tags_7 },
    { 0x4e8db598, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184db59f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99adb5a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8db5a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91cdb5ab, SUC_lexicon_tags_2 },
    { 0x3e8db5ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4edb5ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe1adb5b0, SUC_lexicon_tags_26 },
    { 0x2fcdb5b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x190db5b3, SUC_lexicon_tags_91 },
    { 0xce2db5b3, SUC_lexicon_tags_68 },
    { 0xc56db5b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8db5b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xebadb5ba, SUC_lexicon_tags_17 },
    { 0xa0adb5bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0db5c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eadb5c7, SUC_lexicon_tags_57 },
    { 0xffcdb5c8, SUC_lexicon_tags_37 },
    { 0xff0db5c9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x452db5cc, SUC_lexicon_tags_7 },
    { 0x260db5cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56db5d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fadb5d7, SUC_lexicon_tags_23 },
    { 0xfc8db5d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342db5dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x12cdb5de, SUC_lexicon_tags_6 },
    { 0x864db5df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcd0db5e1, SUC_lexicon_tags_493 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1edb5e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8badb5ec, SUC_lexicon_tags_17 },
    { 0x1bcdb5ed, SUC_lexicon_tags_27 },
    { 0xe60db5ee, SUC_lexicon_tags_77 },
    { 0x2aadb5ed, SUC_lexicon_tags_26 },
    { 0xa6db5ed, SUC_lexicon_tags_10 },
    { 0xbedb5f1, SUC_lexicon_tags_38 },
    { 0xacedb5f1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448db5f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20adb5fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bcdb5fe, SUC_lexicon_tags_49 },
    { 0xba2db5ff, SUC_lexicon_tags_15 },
    { 0xcedb600, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302db605, SUC_lexicon_tags_33 },
    { 0x3d6db606, SUC_lexicon_tags_48 },
    { 0x410db607, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92db60a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12db60d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccdb610, SUC_lexicon_tags_7 },
    { 0xa84db611, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x316db613, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d2db615, SUC_lexicon_tags_1124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02db61a, SUC_lexicon_tags_37 },
    { 0xcc4db61b, SUC_lexicon_tags_57 },
    { 0xa66db61c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8adb61f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x674db621, SUC_lexicon_tags_37 },
    { 0xcfedb622, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cdb62a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf92db62c, SUC_lexicon_tags_9 },
    { 0x908db62c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5dadb62f, SUC_lexicon_tags_15 },
    { 0xe8db62f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x424db633, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332db63c, SUC_lexicon_tags_38 },
    { 0x854db63d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x546db63f, SUC_lexicon_tags_10 },
    { 0xa62db640, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x184db642, SUC_lexicon_tags_1 },
    { 0x6c4db643, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x934db646, SUC_lexicon_tags_25 },
    { 0xf94db647, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872db64b, SUC_lexicon_tags_47 },
    { 0x4badb64c, SUC_lexicon_tags_6 },
    { 0x42adb64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74db650, SUC_lexicon_tags_27 },
    { 0xb56db651, SUC_lexicon_tags_31 },
    { 0xa0cdb652, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93cdb655, SUC_lexicon_tags_7 },
    { 0x76db656, SUC_lexicon_tags_1 },
    { 0x944db657, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x560db659, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fadb65d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698db663, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaadb668, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136db66c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1edb66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0edb672, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18db676, SUC_lexicon_tags_29 },
    { 0x658db677, SUC_lexicon_tags_14 },
    { 0xcfadb678, SUC_lexicon_tags_30 },
    { 0x9fedb678, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb26db67b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5badb67e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712db682, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04db686, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50adb68b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7adb68f, SUC_lexicon_tags_37 },
    { 0x106db690, SUC_lexicon_tags_15 },
    { 0x4a0db690, SUC_lexicon_tags_12 },
    { 0x886db690, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6db695, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2db698, SUC_lexicon_tags_37 },
    { 0x426db699, SUC_lexicon_tags_60 },
    { 0xe12db69a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86db69e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x844db6a1, SUC_lexicon_tags_25 },
    { 0x804db6a2, SUC_lexicon_tags_37 },
    { 0x6eedb6a2, SUC_lexicon_tags_11 },
    { 0x9d0db6a3, SUC_lexicon_tags_1 },
    { 0xe24db6a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d4db6a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0db6ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4db6b6, SUC_lexicon_tags_11 },
    { 0xe6adb6b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x46adb6b9, SUC_lexicon_tags_63 },
    { 0x22db6ba, SUC_lexicon_tags_11 },
    { 0x7d8db6bb, SUC_lexicon_tags_25 },
    { 0xf8edb6bc, SUC_lexicon_tags_11 },
    { 0xe04db6bb, SUC_lexicon_tags_1 },
    { 0xe20db6be, SUC_lexicon_tags_84 },
    { 0x894db6bf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aadb6c2, SUC_lexicon_tags_11 },
    { 0x22adb6c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfc6db6c5, SUC_lexicon_tags_38 },
    { 0xe88db6c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cdb6cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ccdb6ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cdb6d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182db6d5, SUC_lexicon_tags_0 },
    { 0x5b6db6d6, SUC_lexicon_tags_65 },
    { 0x988db6d7, SUC_lexicon_tags_27 },
    { 0x4cedb6d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8db6db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138db6df, SUC_lexicon_tags_23 },
    { 0x312db6e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x90adb6e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224db6e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0db6ed, SUC_lexicon_tags_29 },
    { 0x8ccdb6ed, SUC_lexicon_tags_30 },
    { 0xf46db6ee, SUC_lexicon_tags_47 },
    { 0xfcadb6f0, SUC_lexicon_tags_11 },
    { 0xed6db6f0, SUC_lexicon_tags_14 },
    { 0x628db6f1, SUC_lexicon_tags_43 },
    { 0xeecdb6f3, SUC_lexicon_tags_37 },
    { 0x65cdb6f4, SUC_lexicon_tags_51 },
    { 0xaa4db6f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcdb6fa, SUC_lexicon_tags_12 },
    { 0xb9cdb6fb, SUC_lexicon_tags_15 },
    { 0x332db6fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0db6ff, SUC_lexicon_tags_14 },
    { 0xc2edb6ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdecdb702, SUC_lexicon_tags_17 },
    { 0x58db703, SUC_lexicon_tags_0 },
    { 0xf58db704, SUC_lexicon_tags_83 },
    { 0x650db702, SUC_lexicon_tags_11 },
    { 0x826db706, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8db709, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2db70f, SUC_lexicon_tags_33 },
    { 0x7bcdb70f, SUC_lexicon_tags_48 },
    { 0x7a6db711, SUC_lexicon_tags_201 },
    { 0x70adb712, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6cdb714, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x318db716, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1ecdb718, SUC_lexicon_tags_49 },
    { 0xa8cdb719, SUC_lexicon_tags_17 },
    { 0x602db71a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xea2db71c, SUC_lexicon_tags_47 },
    { 0xbeedb71d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aedb721, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eadb724, SUC_lexicon_tags_0 },
    { 0x3a8db725, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2edb727, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4db72a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3a8db72c, SUC_lexicon_tags_47 },
    { 0x140db72c, SUC_lexicon_tags_10 },
    { 0x26adb72e, SUC_lexicon_tags_28 },
    { 0xe34db72f, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cdb734, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12db738, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456db747, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9edb749, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a8db74b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9edb74f, SUC_lexicon_tags_9 },
    { 0xe14db74f, SUC_lexicon_tags_47 },
    { 0xdbedb751, SUC_lexicon_tags_7 },
    { 0x40cdb74f, SUC_lexicon_tags_15 },
    { 0xb12db750, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0db757, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x304db75a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x858db75d, SUC_lexicon_tags_27 },
    { 0x99adb75e, SUC_lexicon_tags_49 },
    { 0x65adb75f, SUC_lexicon_tags_11 },
    { 0x61adb75f, SUC_lexicon_tags_60 },
    { 0x552db761, SUC_lexicon_tags_2 },
    { 0xd62db762, SUC_lexicon_tags_0 },
    { 0x8ecdb763, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914db76a, SUC_lexicon_tags_2 },
    { 0x7e4db76b, SUC_lexicon_tags_23 },
    { 0x79cdb76c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30db771, SUC_lexicon_tags_577 },
    { 0x59edb772, SUC_lexicon_tags_12 },
    { 0x82db772, SUC_lexicon_tags_19 },
    { 0x4a8db773, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe44db776, SUC_lexicon_tags_30 },
    { 0xed0db777, SUC_lexicon_tags_37 },
    { 0xc2edb776, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccdb77b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8db783, SUC_lexicon_tags_63 },
    { 0x5d2db784, SUC_lexicon_tags_17 },
    { 0x2f6db783, SUC_lexicon_tags_27 },
    { 0x730db785, SUC_lexicon_tags_0 },
    { 0x60db787, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2a2db789, SUC_lexicon_tags_27 },
    { 0x642db78a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0db78f, SUC_lexicon_tags_7 },
    { 0xcaedb78f, SUC_lexicon_tags_2 },
    { 0xd0cdb790, SUC_lexicon_tags_82 },
    { 0xd78db792, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2db797, SUC_lexicon_tags_26 },
    { 0x69adb797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330db7a0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6db7a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x904db7aa, SUC_lexicon_tags_27 },
    { 0x326db7aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cdb7b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaedb7b4, SUC_lexicon_tags_87 },
    { 0xe68db7b4, SUC_lexicon_tags_522 },
    { 0x9b4db7b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8adb7c6, SUC_lexicon_tags_4 },
    { 0x56edb7c6, SUC_lexicon_tags_43 },
    { 0x6adb7c8, SUC_lexicon_tags_12 },
    { 0xefcdb7c8, SUC_lexicon_tags_12 },
    { 0x3e0db7ca, SUC_lexicon_tags_10 },
    { 0x75cdb7cb, SUC_lexicon_tags_4 },
    { 0x416db7c9, SUC_lexicon_tags_49 },
    { 0x972db7c9, SUC_lexicon_tags_4 },
    { 0x3b0db7ca, SUC_lexicon_tags_37 },
    { 0xd0edb7c9, SUC_lexicon_tags_6 },
    { 0x210db7d0, SUC_lexicon_tags_38 },
    { 0x45edb7cd, SUC_lexicon_tags_27 },
    { 0x8c0db7d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88adb7d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf14db7d6, SUC_lexicon_tags_21 },
    { 0x44adb7d7, SUC_lexicon_tags_7 },
    { 0x5f0db7d6, SUC_lexicon_tags_14 },
    { 0x678db7d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18db7df, SUC_lexicon_tags_26 },
    { 0xe90db7e0, SUC_lexicon_tags_11 },
    { 0xccadb7df, SUC_lexicon_tags_25 },
    { 0x86cdb7df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1adb7e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6db7ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4db7ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2db7f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe32db7f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcdb7f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52db7fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114db803, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaedb808, SUC_lexicon_tags_27 },
    { 0x712db809, SUC_lexicon_tags_37 },
    { 0x4a6db80a, SUC_lexicon_tags_15 },
    { 0xd90db80b, SUC_lexicon_tags_37 },
    { 0x518db80c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacedb810, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96db818, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4db81c, SUC_lexicon_tags_2 },
    { 0x560db81c, SUC_lexicon_tags_17 },
    { 0x3d6db81e, SUC_lexicon_tags_0 },
    { 0x50cdb81e, SUC_lexicon_tags_49 },
    { 0xd5edb81c, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5edb823, SUC_lexicon_tags_1 },
    { 0x4b2db824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56db828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64db82c, SUC_lexicon_tags_45 },
    { 0xedadb82d, SUC_lexicon_tags_14 },
    { 0xc42db82e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27adb834, SUC_lexicon_tags_17 },
    { 0x3e8db835, SUC_lexicon_tags_47 },
    { 0xb2cdb834, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400db83c, SUC_lexicon_tags_17 },
    { 0x606db83d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904db842, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68db84b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8db85a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820db85e, SUC_lexicon_tags_27 },
    { 0x9aadb85f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4db862, SUC_lexicon_tags_10 },
    { 0xd98db862, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe14db865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96db869, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x196db86b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6db86e, SUC_lexicon_tags_12 },
    { 0x780db86f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310db877, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x230db879, SUC_lexicon_tags_7 },
    { 0x81edb879, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x706db87c, SUC_lexicon_tags_11 },
    { 0xdfedb87d, SUC_lexicon_tags_37 },
    { 0x588db87e, SUC_lexicon_tags_60 },
    { 0x3e2db87d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdb886, SUC_lexicon_tags_37 },
    { 0x6ccdb887, SUC_lexicon_tags_25 },
    { 0x3eadb886, SUC_lexicon_tags_11 },
    { 0xdeedb889, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x122db88b, SUC_lexicon_tags_114 },
    { 0xcb2db88c, SUC_lexicon_tags_1 },
    { 0xbf0db88c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cdb890, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34db897, SUC_lexicon_tags_17 },
    { 0x680db897, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf2edb89a, SUC_lexicon_tags_37 },
    { 0xc0cdb89b, SUC_lexicon_tags_11 },
    { 0x132db89c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfedb89e, SUC_lexicon_tags_31 },
    { 0xeaadb89e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cdb8a5, SUC_lexicon_tags_47 },
    { 0x23edb8a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8db8a9, SUC_lexicon_tags_5 },
    { 0x8dcdb8aa, SUC_lexicon_tags_27 },
    { 0x41cdb8aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44db8b4, SUC_lexicon_tags_144 },
    { 0x6cdb8b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8db8b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c0db8ba, SUC_lexicon_tags_43 },
    { 0xdbcdb8ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecdb8be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x136db8c1, SUC_lexicon_tags_65 },
    { 0xcdadb8c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x250db8c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x446db8c7, SUC_lexicon_tags_49 },
    { 0x53cdb8c8, SUC_lexicon_tags_2 },
    { 0x604db8c9, SUC_lexicon_tags_33 },
    { 0x1ecdb8ca, SUC_lexicon_tags_17 },
    { 0xef6db8ca, SUC_lexicon_tags_17 },
    { 0x860db8cb, SUC_lexicon_tags_27 },
    { 0xf4cdb8c9, SUC_lexicon_tags_17 },
    { 0xfd6db8cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0db8d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x600db8d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf66db8d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106db8db, SUC_lexicon_tags_61 },
    { 0xb1adb8dc, SUC_lexicon_tags_33 },
    { 0x26adb8dd, SUC_lexicon_tags_47 },
    { 0x818db8dc, SUC_lexicon_tags_45 },
    { 0x54cdb8dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20edb8e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x632db8e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8db8ef, SUC_lexicon_tags_17 },
    { 0x6edb8ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x78edb8f2, SUC_lexicon_tags_14 },
    { 0xb4cdb8f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdcdb8f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22db8f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db8fb, SUC_lexicon_tags_7 },
    { 0xe38db8fc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8db8ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeedb902, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fadb904, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xc3edb906, SUC_lexicon_tags_14 },
    { 0x316db907, SUC_lexicon_tags_23 },
    { 0x26db908, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68db90b, SUC_lexicon_tags_12 },
    { 0x1f8db90b, SUC_lexicon_tags_63 },
    { 0x9b4db90c, SUC_lexicon_tags_12 },
    { 0x82adb90e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x320db911, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed4db913, SUC_lexicon_tags_15 },
    { 0x2bcdb913, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8db918, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0db91d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86db920, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbfedb922, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeedb924, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750db92b, SUC_lexicon_tags_37 },
    { 0x270db92c, SUC_lexicon_tags_31 },
    { 0xadedb92c, SUC_lexicon_tags_37 },
    { 0x890db92e, SUC_lexicon_tags_25 },
    { 0x7e2db92f, SUC_lexicon_tags_8 },
    { 0x33edb92f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690db936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cdb93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd74db93e, SUC_lexicon_tags_12 },
    { 0x928db93f, SUC_lexicon_tags_25 },
    { 0xc94db93e, SUC_lexicon_tags_38 },
    { 0x154db941, SUC_lexicon_tags_30 },
    { 0xd70db941, SUC_lexicon_tags_30 },
    { 0x460db941, SUC_lexicon_tags_10 },
    { 0xa38db942, SUC_lexicon_tags_37 },
    { 0x3f6db93e, SUC_lexicon_tags_8 },
    { 0xd3cdb940, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34cdb949, SUC_lexicon_tags_7 },
    { 0x3e8db949, SUC_lexicon_tags_36 },
    { 0xfa8db94b, SUC_lexicon_tags_68 },
    { 0x7d6db94b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xdd2db94e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x610db951, SUC_lexicon_tags_37 },
    { 0x240db951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d0db954, SUC_lexicon_tags_25 },
    { 0x82edb955, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xbc6db957, SUC_lexicon_tags_1 },
    { 0xabadb958, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494db95c, SUC_lexicon_tags_17 },
    { 0x964db95d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406db963, SUC_lexicon_tags_46 },
    { 0x9b6db964, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2db967, SUC_lexicon_tags_87 },
    { 0xbd6db968, SUC_lexicon_tags_8 },
    { 0x6e8db969, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10db96f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79cdb973, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9adb976, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95edb97a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbedb982, SUC_lexicon_tags_65 },
    { 0xf92db983, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x208db985, SUC_lexicon_tags_91 },
    { 0xa7adb986, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0db989, SUC_lexicon_tags_26 },
    { 0x716db989, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcf8db98c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314db991, SUC_lexicon_tags_31 },
    { 0x9adb992, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdadb996, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccdb99a, SUC_lexicon_tags_15 },
    { 0x548db99b, SUC_lexicon_tags_17 },
    { 0xebadb99c, SUC_lexicon_tags_7 },
    { 0xbc2db99c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa04db99f, SUC_lexicon_tags_9 },
    { 0x7a2db9a0, SUC_lexicon_tags_15 },
    { 0x506db99f, SUC_lexicon_tags_12 },
    { 0xd6db9a2, SUC_lexicon_tags_16 },
    { 0x74edb9a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68db9a6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xae8db9a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x238db9ab, SUC_lexicon_tags_43 },
    { 0x452db9ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfacdb9ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffadb9b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360db9bc, SUC_lexicon_tags_11 },
    { 0x6ecdb9bd, SUC_lexicon_tags_107 },
    { 0x234db9bd, SUC_lexicon_tags_15 },
    { 0x52adb9bd, SUC_lexicon_tags_0 },
    { 0x7eadb9bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66adb9c7, SUC_lexicon_tags_17 },
    { 0x52adb9c8, SUC_lexicon_tags_49 },
    { 0x326db9c7, SUC_lexicon_tags_13 },
    { 0x35adb9ca, SUC_lexicon_tags_0 },
    { 0xa1adb9cb, SUC_lexicon_tags_2 },
    { 0x88db9cc, SUC_lexicon_tags_1 },
    { 0x9e2db9cd, SUC_lexicon_tags_10 },
    { 0xc16db9ce, SUC_lexicon_tags_25 },
    { 0x730db9cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a6db9d1, SUC_lexicon_tags_11 },
    { 0x80adb9d2, SUC_lexicon_tags_15 },
    { 0xda6db9d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83edb9d5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6db9da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x702db9dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc2db9de, SUC_lexicon_tags_12 },
    { 0xc20db9df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc8adb9e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32adb9e5, SUC_lexicon_tags_38 },
    { 0xa90db9e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74db9eb, SUC_lexicon_tags_33 },
    { 0xb5adb9ec, SUC_lexicon_tags_17 },
    { 0xfcdb9eb, SUC_lexicon_tags_25 },
    { 0xb4adb9ec, SUC_lexicon_tags_25 },
    { 0xa50db9ed, SUC_lexicon_tags_2 },
    { 0xb66db9f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2db9f4, SUC_lexicon_tags_27 },
    { 0x658db9f5, SUC_lexicon_tags_22 },
    { 0xecedb9f6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812db9fa, SUC_lexicon_tags_43 },
    { 0x61adb9fb, SUC_lexicon_tags_29 },
    { 0xf40db9fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dba00, SUC_lexicon_tags_26 },
    { 0xa72dba01, SUC_lexicon_tags_31 },
    { 0x588dba01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc26dba04, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286dba0a, SUC_lexicon_tags_2 },
    { 0xdd6dba0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadadba10, SUC_lexicon_tags_45 },
    { 0xcbadba10, SUC_lexicon_tags_1 },
    { 0x532dba11, SUC_lexicon_tags_0 },
    { 0x642dba13, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x470dba15, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x516dba17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8dba1f, SUC_lexicon_tags_11 },
    { 0xbcdba1f, SUC_lexicon_tags_49 },
    { 0x8b4dba21, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa3cdba23, SUC_lexicon_tags_35 },
    { 0xe80dba24, SUC_lexicon_tags_19 },
    { 0xe66dba25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6a6dba27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6dba2c, SUC_lexicon_tags_8 },
    { 0x7d6dba2c, SUC_lexicon_tags_26 },
    { 0x8e0dba2e, SUC_lexicon_tags_25 },
    { 0x754dba2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52adba31, SUC_lexicon_tags_25 },
    { 0x502dba32, SUC_lexicon_tags_14 },
    { 0xe64dba31, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36dba37, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bedba3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8dba3d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xef6dba3f, SUC_lexicon_tags_15 },
    { 0x674dba40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a6dba42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94dba48, SUC_lexicon_tags_2 },
    { 0x166dba49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932dba4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08dba51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36dba54, SUC_lexicon_tags_35 },
    { 0x270dba55, SUC_lexicon_tags_32 },
    { 0x1dcdba56, SUC_lexicon_tags_387 },
    { 0x7a6dba57, SUC_lexicon_tags_14 },
    { 0x94dba57, SUC_lexicon_tags_23 },
    { 0x9bcdba55, SUC_lexicon_tags_97 },
    { 0x38adba5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6edba5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc24dba5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1edba67, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9adba6a, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2dba6d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2dcdba6f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x46cdba71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cdba78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16cdba7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0edba7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacdba84, SUC_lexicon_tags_23 },
    { 0x74cdba85, SUC_lexicon_tags_47 },
    { 0x66cdba86, SUC_lexicon_tags_29 },
    { 0x7f6dba86, SUC_lexicon_tags_12 },
    { 0x580dba88, SUC_lexicon_tags_117 },
    { 0x308dba86, SUC_lexicon_tags_0 },
    { 0x206dba84, SUC_lexicon_tags_37 },
    { 0x4d6dba8b, SUC_lexicon_tags_22 },
    { 0x2badba8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37adba8e, SUC_lexicon_tags_37 },
    { 0x100dba8f, SUC_lexicon_tags_18 },
    { 0xdc0dba90, SUC_lexicon_tags_15 },
    { 0x44cdba90, SUC_lexicon_tags_25 },
    { 0xc92dba92, SUC_lexicon_tags_37 },
    { 0x320dba93, SUC_lexicon_tags_4 },
    { 0x70adba94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5edba96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe52dba98, SUC_lexicon_tags_2 },
    { 0xf60dba98, SUC_lexicon_tags_2 },
    { 0x42adba98, SUC_lexicon_tags_45 },
    { 0xeb2dba99, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22dbaa0, SUC_lexicon_tags_1 },
    { 0x944dbaa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574dbaa5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x94dbaa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714dbaae, SUC_lexicon_tags_138 },
    { 0x446dbaae, SUC_lexicon_tags_15 },
    { 0xc74dbab0, SUC_lexicon_tags_87 },
    { 0xbbedbab1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edbab6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeadbaba, SUC_lexicon_tags_11 },
    { 0xfedbabb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4dbac2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdbac5, SUC_lexicon_tags_11 },
    { 0x236dbac6, SUC_lexicon_tags_10 },
    { 0xa98dbac5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742dbacc, SUC_lexicon_tags_1347 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2dbacf, SUC_lexicon_tags_25 },
    { 0x37cdbacf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442dbad4, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdbad7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13adbada, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xc8dbadc, SUC_lexicon_tags_12 },
    { 0x160dbadc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdadbadf, SUC_lexicon_tags_17 },
    { 0x370dbae0, SUC_lexicon_tags_3 },
    { 0xf86dbae0, SUC_lexicon_tags_84 },
    { 0x57adbae0, SUC_lexicon_tags_34 },
    { 0x65edbadf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9edbae5, SUC_lexicon_tags_8 },
    { 0x654dbae6, SUC_lexicon_tags_4 },
    { 0x7b2dbae7, SUC_lexicon_tags_29 },
    { 0x92adbae8, SUC_lexicon_tags_17 },
    { 0x300dbae6, SUC_lexicon_tags_1 },
    { 0x24edbae7, SUC_lexicon_tags_26 },
    { 0x818dbaeb, SUC_lexicon_tags_174 },
    { 0x768dbaec, SUC_lexicon_tags_27 },
    { 0x9b6dbaed, SUC_lexicon_tags_11 },
    { 0x5d4dbaee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x69edbaf1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xda4dbaf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddedbaf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2dbaff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadbb07, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4dbb0b, SUC_lexicon_tags_0 },
    { 0x676dbb0b, SUC_lexicon_tags_63 },
    { 0x360dbb0d, SUC_lexicon_tags_25 },
    { 0x960dbb0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x326dbb11, SUC_lexicon_tags_15 },
    { 0x102dbb11, SUC_lexicon_tags_21 },
    { 0x904dbb13, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66adbb17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83edbb19, SUC_lexicon_tags_23 },
    { 0xd94dbb1a, SUC_lexicon_tags_45 },
    { 0x90cdbb1b, SUC_lexicon_tags_0 },
    { 0x482dbb1a, SUC_lexicon_tags_25 },
    { 0x158dbb1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab0dbb1f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8b8dbb21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18dbb24, SUC_lexicon_tags_27 },
    { 0x3e2dbb25, SUC_lexicon_tags_7 },
    { 0x910dbb26, SUC_lexicon_tags_14 },
    { 0x1aedbb26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08dbb2a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988dbb2e, SUC_lexicon_tags_985 },
    { 0x1c4dbb2e, SUC_lexicon_tags_16 },
    { 0xe04dbb30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39adbb36, SUC_lexicon_tags_17 },
    { 0x6e4dbb36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbb3e, SUC_lexicon_tags_37 },
    { 0x7aadbb3e, SUC_lexicon_tags_10 },
    { 0xe12dbb40, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x21edbb42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe60dbb44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32adbb50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x964dbb53, SUC_lexicon_tags_38 },
    { 0x8b6dbb54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2dbb5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe54dbb5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258dbb64, SUC_lexicon_tags_61 },
    { 0xa6cdbb65, SUC_lexicon_tags_31 },
    { 0x50edbb65, SUC_lexicon_tags_15 },
    { 0x30dbb64, SUC_lexicon_tags_21 },
    { 0xb74dbb65, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8aedbb6a, SUC_lexicon_tags_2 },
    { 0xfe8dbb6b, SUC_lexicon_tags_1 },
    { 0xf30dbb6c, SUC_lexicon_tags_46 },
    { 0x6e0dbb6d, SUC_lexicon_tags_23 },
    { 0x4dedbb6e, SUC_lexicon_tags_17 },
    { 0x242dbb6d, SUC_lexicon_tags_32 },
    { 0x36adbb6c, SUC_lexicon_tags_12 },
    { 0xfb2dbb6f, SUC_lexicon_tags_37 },
    { 0x154dbb72, SUC_lexicon_tags_31 },
    { 0x28cdbb73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5adbb7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50dbb84, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x288dbb86, SUC_lexicon_tags_0 },
    { 0x8e2dbb87, SUC_lexicon_tags_26 },
    { 0x86adbb88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf4dbb8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7adbb8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934dbb93, SUC_lexicon_tags_17 },
    { 0x9eedbb94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5adbb97, SUC_lexicon_tags_14 },
    { 0xf44dbb98, SUC_lexicon_tags_15 },
    { 0x26dbb99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eedbb9d, SUC_lexicon_tags_55 },
    { 0x91adbb9e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496dbba2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10dbba5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcdbba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2dbbad, SUC_lexicon_tags_7 },
    { 0x2adbbad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70dbbb3, SUC_lexicon_tags_15 },
    { 0xf56dbbb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecdbbb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cdbbbb, SUC_lexicon_tags_35 },
    { 0x292dbbbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772dbbc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918dbbc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4dbbcc, SUC_lexicon_tags_11 },
    { 0xc96dbbcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378dbbd3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cdbbd7, SUC_lexicon_tags_118 },
    { 0xcadbbd8, SUC_lexicon_tags_26 },
    { 0x900dbbd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2dbbde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eedbbe1, SUC_lexicon_tags_13 },
    { 0xed0dbbe2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3badbbe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9edbbe7, SUC_lexicon_tags_44 },
    { 0x31edbbe7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518dbbec, SUC_lexicon_tags_47 },
    { 0x712dbbed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbbf1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acdbbf4, SUC_lexicon_tags_2 },
    { 0xc92dbbf5, SUC_lexicon_tags_17 },
    { 0x3aadbbf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8dbbf9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x96adbbfc, SUC_lexicon_tags_15 },
    { 0xe2adbbfc, SUC_lexicon_tags_31 },
    { 0x82cdbbfc, SUC_lexicon_tags_28 },
    { 0x912dbbfe, SUC_lexicon_tags_25 },
    { 0x40adbbfe, SUC_lexicon_tags_8 },
    { 0xb76dbc01, SUC_lexicon_tags_45 },
    { 0x94dbc01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39cdbc05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428dbc0c, SUC_lexicon_tags_46 },
    { 0xcdadbc0d, SUC_lexicon_tags_17 },
    { 0x8dadbc0d, SUC_lexicon_tags_70 },
    { 0x3d4dbc0f, SUC_lexicon_tags_70 },
    { 0x106dbc0d, SUC_lexicon_tags_12 },
    { 0xe00dbc11, SUC_lexicon_tags_25 },
    { 0x776dbc0d, SUC_lexicon_tags_19 },
    { 0xb28dbc13, SUC_lexicon_tags_12 },
    { 0x508dbc14, SUC_lexicon_tags_63 },
    { 0x788dbc12, SUC_lexicon_tags_35 },
    { 0x22adbc16, SUC_lexicon_tags_1 },
    { 0xd86dbc17, SUC_lexicon_tags_8 },
    { 0x7bcdbc0c, SUC_lexicon_tags_3 },
    { 0xe2cdbc19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x130dbc1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaecdbc1d, SUC_lexicon_tags_7 },
    { 0x652dbc1e, SUC_lexicon_tags_25 },
    { 0x76dbc1f, SUC_lexicon_tags_4 },
    { 0x7e8dbc1e, SUC_lexicon_tags_33 },
    { 0xccedbc1f, SUC_lexicon_tags_25 },
    { 0x244dbc1e, SUC_lexicon_tags_27 },
    { 0x416dbc1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790dbc28, SUC_lexicon_tags_21 },
    { 0x3adbc29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cdbc30, SUC_lexicon_tags_12 },
    { 0xad2dbc30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cdbc35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6dbc39, SUC_lexicon_tags_19 },
    { 0x32dbc3a, SUC_lexicon_tags_68 },
    { 0x344dbc3a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdadbc3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cdbc4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292dbc50, SUC_lexicon_tags_26 },
    { 0x8bcdbc50, SUC_lexicon_tags_2 },
    { 0xb28dbc52, SUC_lexicon_tags_17 },
    { 0xa1adbc53, SUC_lexicon_tags_118 },
    { 0x952dbc50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6dbc5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x49edbc5c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5eedbc5e, SUC_lexicon_tags_61 },
    { 0x282dbc5e, SUC_lexicon_tags_15 },
    { 0x28adbc60, SUC_lexicon_tags_15 },
    { 0x62edbc5e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6dbc65, SUC_lexicon_tags_9 },
    { 0xdb8dbc66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4dbc72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10edbc77, SUC_lexicon_tags_37 },
    { 0xaa8dbc78, SUC_lexicon_tags_12 },
    { 0x1b4dbc79, SUC_lexicon_tags_9 },
    { 0xf1edbc7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774dbc81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98dbc88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8edbc8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830dbc8e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88dbc92, SUC_lexicon_tags_77 },
    { 0xdc2dbc93, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x23cdbc95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66dbc9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2dbc9d, SUC_lexicon_tags_3 },
    { 0x72edbc9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6dbca1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3cadbca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0dbcaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x394dbcac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0dbcb1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecdbcbc, SUC_lexicon_tags_234 },
    { 0x312dbcbc, SUC_lexicon_tags_37 },
    { 0xa52dbcbc, SUC_lexicon_tags_33 },
    { 0xa54dbcbf, SUC_lexicon_tags_45 },
    { 0xb14dbcbf, SUC_lexicon_tags_15 },
    { 0x3cedbcbd, SUC_lexicon_tags_27 },
    { 0x660dbcc2, SUC_lexicon_tags_15 },
    { 0x76adbcc3, SUC_lexicon_tags_6 },
    { 0xd56dbcc4, SUC_lexicon_tags_16 },
    { 0xdecdbcc1, SUC_lexicon_tags_28 },
    { 0xd28dbcc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edbccb, SUC_lexicon_tags_38 },
    { 0x68dbccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482dbcd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28cdbcd3, SUC_lexicon_tags_2 },
    { 0xd1edbcd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf64dbcd6, SUC_lexicon_tags_13 },
    { 0x2b0dbcd7, SUC_lexicon_tags_11 },
    { 0xc7adbcd8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe42dbcdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee0dbcdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe56dbcdf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xda8dbce1, SUC_lexicon_tags_87 },
    { 0x286dbce1, SUC_lexicon_tags_37 },
    { 0xbfcdbce3, SUC_lexicon_tags_26 },
    { 0x53cdbce4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7adbce6, SUC_lexicon_tags_17 },
    { 0x234dbce7, SUC_lexicon_tags_37 },
    { 0x734dbce7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x782dbceb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8adbcee, SUC_lexicon_tags_1 },
    { 0x5f2dbcef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcdbcf6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd8dbcf8, SUC_lexicon_tags_0 },
    { 0x6a4dbcf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc16dbcfb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dbd01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbedbd03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa76dbd05, SUC_lexicon_tags_27 },
    { 0x3c6dbd06, SUC_lexicon_tags_61 },
    { 0xdf2dbd05, SUC_lexicon_tags_2 },
    { 0x1b0dbd07, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984dbd0f, SUC_lexicon_tags_49 },
    { 0x27adbd10, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122dbd14, SUC_lexicon_tags_37 },
    { 0x2badbd15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84dbd1b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cdbd25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb4dbd27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcc2dbd29, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x640dbd2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e0dbd2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4dbd30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5f4dbd32, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14dbd38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992dbd42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6dbd4d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88edbd52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9a2dbd54, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x728dbd56, SUC_lexicon_tags_27 },
    { 0xee6dbd57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94dbd5c, SUC_lexicon_tags_63 },
    { 0x710dbd5d, SUC_lexicon_tags_14 },
    { 0xee6dbd5d, SUC_lexicon_tags_15 },
    { 0xf64dbd5f, SUC_lexicon_tags_8 },
    { 0xca6dbd60, SUC_lexicon_tags_10 },
    { 0x36dbd5f, SUC_lexicon_tags_26 },
    { 0x696dbd62, SUC_lexicon_tags_34 },
    { 0x398dbd61, SUC_lexicon_tags_37 },
    { 0x292dbd64, SUC_lexicon_tags_2 },
    { 0x574dbd65, SUC_lexicon_tags_10 },
    { 0xdfadbd64, SUC_lexicon_tags_12 },
    { 0xb76dbd64, SUC_lexicon_tags_9 },
    { 0x64cdbd60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6cdbd6a, SUC_lexicon_tags_7 },
    { 0xb90dbd6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb80dbd6d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadbd71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04dbd74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2adbd76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc42dbd78, SUC_lexicon_tags_37 },
    { 0x480dbd79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32dbd7c, SUC_lexicon_tags_7 },
    { 0xabcdbd7d, SUC_lexicon_tags_6 },
    { 0xcdadbd7e, SUC_lexicon_tags_35 },
    { 0x2ccdbd7e, SUC_lexicon_tags_17 },
    { 0xa4edbd80, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60edbd85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26edbd8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e2dbd8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15cdbd90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958dbd95, SUC_lexicon_tags_30 },
    { 0x360dbd96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecdbd9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00dbda1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcdbda5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dbda8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1bcdbdaa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548dbdae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x408dbdb0, SUC_lexicon_tags_37 },
    { 0x2b2dbdb0, SUC_lexicon_tags_28 },
    { 0x5d6dbdb2, SUC_lexicon_tags_31 },
    { 0x370dbdb2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x408dbdb5, SUC_lexicon_tags_63 },
    { 0xea8dbdb6, SUC_lexicon_tags_25 },
    { 0x8fcdbdb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56dbdbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f8dbdbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34dbdc2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x498dbdc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cdbdd4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2dbdd7, SUC_lexicon_tags_29 },
    { 0x4c6dbdd8, SUC_lexicon_tags_33 },
    { 0xa6edbdd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x444dbddc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x764dbdde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6dbde1, SUC_lexicon_tags_37 },
    { 0xec8dbde1, SUC_lexicon_tags_29 },
    { 0x63cdbde3, SUC_lexicon_tags_37 },
    { 0x4e6dbde4, SUC_lexicon_tags_27 },
    { 0x3c2dbde4, SUC_lexicon_tags_31 },
    { 0xa2adbde6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4dbdeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83adbdf0, SUC_lexicon_tags_14 },
    { 0x678dbdf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8dbdf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcedbdf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2f6dbdfb, SUC_lexicon_tags_9 },
    { 0xd06dbdfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cdbdff, SUC_lexicon_tags_37 },
    { 0x110dbdff, SUC_lexicon_tags_11 },
    { 0x2b4dbdff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532dbe05, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x198dbe08, SUC_lexicon_tags_11 },
    { 0xbbcdbe09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80adbe0e, SUC_lexicon_tags_6 },
    { 0xd8cdbe0f, SUC_lexicon_tags_11 },
    { 0x640dbe10, SUC_lexicon_tags_25 },
    { 0x54dbe11, SUC_lexicon_tags_2 },
    { 0xb0cdbe10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edbe15, SUC_lexicon_tags_56 },
    { 0x33edbe15, SUC_lexicon_tags_48 },
    { 0x5e4dbe17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8dbe1d, SUC_lexicon_tags_37 },
    { 0x7e0dbe1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474dbe25, SUC_lexicon_tags_49 },
    { 0xcd4dbe26, SUC_lexicon_tags_23 },
    { 0xf9adbe27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4dbe2b, SUC_lexicon_tags_11 },
    { 0x8badbe2c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x184dbe2f, SUC_lexicon_tags_66 },
    { 0x184dbe30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aedbe33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0dbe38, SUC_lexicon_tags_33 },
    { 0xf18dbe39, SUC_lexicon_tags_25 },
    { 0x5e8dbe3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe98dbe3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x542dbe3f, SUC_lexicon_tags_2 },
    { 0xd1cdbe40, SUC_lexicon_tags_37 },
    { 0x6a8dbe41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf10dbe43, SUC_lexicon_tags_17 },
    { 0x84dbe44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18dbe49, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832dbe50, SUC_lexicon_tags_8 },
    { 0xf1cdbe50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78dbe57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178dbe5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28dbe64, SUC_lexicon_tags_33 },
    { 0x656dbe65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8dbe68, SUC_lexicon_tags_27 },
    { 0x24dbe69, SUC_lexicon_tags_32 },
    { 0x56cdbe6a, SUC_lexicon_tags_1 },
    { 0x7b6dbe69, SUC_lexicon_tags_17 },
    { 0x696dbe6c, SUC_lexicon_tags_12 },
    { 0x57adbe6a, SUC_lexicon_tags_37 },
    { 0x53edbe69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6dbe73, SUC_lexicon_tags_11 },
    { 0x82adbe74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aadbe79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2dbe7f, SUC_lexicon_tags_26 },
    { 0xcbadbe7f, SUC_lexicon_tags_11 },
    { 0x240dbe81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654dbe86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe18dbe88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94dbe8b, SUC_lexicon_tags_30 },
    { 0xf3adbe8c, SUC_lexicon_tags_7 },
    { 0xcd0dbe8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd6dbe8f, SUC_lexicon_tags_15 },
    { 0xea6dbe90, SUC_lexicon_tags_69 },
    { 0x520dbe91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0dbe97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94dbe9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170dbea2, SUC_lexicon_tags_21 },
    { 0x290dbea3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacdbea9, SUC_lexicon_tags_12 },
    { 0xc42dbeaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cdbeaf, SUC_lexicon_tags_10 },
    { 0x84cdbeb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcdbeb3, SUC_lexicon_tags_15 },
    { 0xa90dbeb4, SUC_lexicon_tags_11 },
    { 0x46cdbeb5, SUC_lexicon_tags_28 },
    { 0x8eedbeb6, SUC_lexicon_tags_62 },
    { 0x576dbeb7, SUC_lexicon_tags_120 },
    { 0xf06dbeb8, SUC_lexicon_tags_7 },
    { 0xdb0dbeb7, SUC_lexicon_tags_43 },
    { 0x9cedbeb7, SUC_lexicon_tags_45 },
    { 0x7cedbeb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58dbebe, SUC_lexicon_tags_17 },
    { 0x168dbebf, SUC_lexicon_tags_35 },
    { 0xe58dbebe, SUC_lexicon_tags_23 },
    { 0x848dbec1, SUC_lexicon_tags_21 },
    { 0x514dbec2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596dbec6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafadbeca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb2dbecc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x82adbecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cdbed5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ecdbed7, SUC_lexicon_tags_14 },
    { 0x4fcdbed8, SUC_lexicon_tags_37 },
    { 0xf38dbed9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30dbedc, SUC_lexicon_tags_60 },
    { 0x308dbedd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbadbee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4dbee6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96adbeea, SUC_lexicon_tags_14 },
    { 0x982dbeea, SUC_lexicon_tags_87 },
    { 0x2ecdbeeb, SUC_lexicon_tags_22 },
    { 0x5ccdbeed, SUC_lexicon_tags_14 },
    { 0xb2dbeee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x208dbef1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0dbef9, SUC_lexicon_tags_55 },
    { 0xc2dbef9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0adbf08, SUC_lexicon_tags_4 },
    { 0xb06dbf09, SUC_lexicon_tags_25 },
    { 0x5e4dbf0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa8dbf0c, SUC_lexicon_tags_179 },
    { 0x3d8dbf0d, SUC_lexicon_tags_79 },
    { 0x962dbf0d, SUC_lexicon_tags_17 },
    { 0xe74dbf0f, SUC_lexicon_tags_1 },
    { 0x84cdbf0e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508dbf16, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x2b8dbf18, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x64dbf1a, SUC_lexicon_tags_908 },
    { 0xb04dbf1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x24dbf1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x872dbf21, SUC_lexicon_tags_10 },
    { 0xc9edbf22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dedbf26, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6dbf2d, SUC_lexicon_tags_11 },
    { 0x8e2dbf2e, SUC_lexicon_tags_7 },
    { 0xeaadbf2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x828dbf32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xac8dbf34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x20adbf36, SUC_lexicon_tags_19 },
    { 0x458dbf36, SUC_lexicon_tags_12 },
    { 0xa7cdbf36, SUC_lexicon_tags_1535 },
    { 0xf6dbf38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8adbf3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x50dbf3f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x864dbf42, SUC_lexicon_tags_25 },
    { 0x2b6dbf43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cdbf49, SUC_lexicon_tags_37 },
    { 0x48dbf4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6dbf54, SUC_lexicon_tags_47 },
    { 0x452dbf55, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc2adbf57, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11adbf5d, SUC_lexicon_tags_29 },
    { 0xbcadbf5d, SUC_lexicon_tags_25 },
    { 0x29adbf5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x276dbf61, SUC_lexicon_tags_32 },
    { 0xd4dbf62, SUC_lexicon_tags_17 },
    { 0x2b4dbf61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcdbf6b, SUC_lexicon_tags_47 },
    { 0xae2dbf6c, SUC_lexicon_tags_19 },
    { 0x66cdbf6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90edbf6f, SUC_lexicon_tags_15 },
    { 0xbbcdbf6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518dbf74, SUC_lexicon_tags_0 },
    { 0xd20dbf75, SUC_lexicon_tags_2 },
    { 0xb6dbf76, SUC_lexicon_tags_26 },
    { 0xa0cdbf75, SUC_lexicon_tags_9 },
    { 0x146dbf78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58dbf7b, SUC_lexicon_tags_33 },
    { 0x8b0dbf7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0adbf80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8edbf91, SUC_lexicon_tags_71 },
    { 0xf04dbf92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6dbf95, SUC_lexicon_tags_11 },
    { 0x8dadbf96, SUC_lexicon_tags_25 },
    { 0x69cdbf97, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9bcdbf99, SUC_lexicon_tags_82 },
    { 0x750dbf9a, SUC_lexicon_tags_43 },
    { 0x654dbf9a, SUC_lexicon_tags_25 },
    { 0x8fadbf9c, SUC_lexicon_tags_23 },
    { 0x9fadbf9c, SUC_lexicon_tags_23 },
    { 0xb54dbf9e, SUC_lexicon_tags_15 },
    { 0xfcdbf9f, SUC_lexicon_tags_14 },
    { 0xdaedbf9d, SUC_lexicon_tags_37 },
    { 0xa9adbfa1, SUC_lexicon_tags_25 },
    { 0x926dbfa1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270dbfa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f2dbfaa, SUC_lexicon_tags_1 },
    { 0xac8dbfaa, SUC_lexicon_tags_37 },
    { 0xff2dbfac, SUC_lexicon_tags_32 },
    { 0x27edbfac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9b6dbfaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0adbfb1, SUC_lexicon_tags_2 },
    { 0x638dbfb2, SUC_lexicon_tags_14 },
    { 0x758dbfb3, SUC_lexicon_tags_15 },
    { 0x170dbfb3, SUC_lexicon_tags_32 },
    { 0x640dbfb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57adbfbc, SUC_lexicon_tags_8 },
    { 0xc46dbfbd, SUC_lexicon_tags_28 },
    { 0x1e6dbfbe, SUC_lexicon_tags_49 },
    { 0xcc6dbfbf, SUC_lexicon_tags_15 },
    { 0xa8dbfc0, SUC_lexicon_tags_45 },
    { 0xa32dbfbf, SUC_lexicon_tags_2 },
    { 0x51cdbfc2, SUC_lexicon_tags_1 },
    { 0xfa8dbfc0, SUC_lexicon_tags_49 },
    { 0x8a0dbfc4, SUC_lexicon_tags_31 },
    { 0x3f4dbfbf, SUC_lexicon_tags_42 },
    { 0xe98dbfc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246dbfcc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbb0dbfce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x824dbfd0, SUC_lexicon_tags_27 },
    { 0x38dbfd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8dbfd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x808dbfd8, SUC_lexicon_tags_48 },
    { 0xc1edbfd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674dbfdf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xce2dbfe1, SUC_lexicon_tags_17 },
    { 0x534dbfe2, SUC_lexicon_tags_82 },
    { 0x8c2dbfe3, SUC_lexicon_tags_46 },
    { 0x696dbfe3, SUC_lexicon_tags_63 },
    { 0x93adbfe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0dbfeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x754dbfee, SUC_lexicon_tags_9 },
    { 0xf26dbfef, SUC_lexicon_tags_2 },
    { 0xea2dbfef, SUC_lexicon_tags_37 },
    { 0xba8dbff1, SUC_lexicon_tags_8 },
    { 0x7b4dbfee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2edbff4, SUC_lexicon_tags_11 },
    { 0x914dbff5, SUC_lexicon_tags_26 },
    { 0xffcdbff6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dbff8, SUC_lexicon_tags_4 },
    { 0x580dbff8, SUC_lexicon_tags_1 },
    { 0x6a0dbff9, SUC_lexicon_tags_45 },
    { 0xda8dbff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1edc001, SUC_lexicon_tags_25 },
    { 0xd3adc002, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786dc006, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdc00b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x16cdc00e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e0dc010, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xb76dc012, SUC_lexicon_tags_65 },
    { 0x1acdc013, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda4dc015, SUC_lexicon_tags_27 },
    { 0x108dc016, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x876dc018, SUC_lexicon_tags_318 },
    { 0x8fadc019, SUC_lexicon_tags_37 },
    { 0x486dc01a, SUC_lexicon_tags_7 },
    { 0xfd2dc01b, SUC_lexicon_tags_313 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eadc01e, SUC_lexicon_tags_7 },
    { 0xcb6dc01f, SUC_lexicon_tags_64 },
    { 0x21adc020, SUC_lexicon_tags_33 },
    { 0xbc6dc01e, SUC_lexicon_tags_37 },
    { 0xb4dc022, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaadc025, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454dc02d, SUC_lexicon_tags_12 },
    { 0xed8dc02d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30dc034, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5aedc036, SUC_lexicon_tags_82 },
    { 0x72edc037, SUC_lexicon_tags_15 },
    { 0x928dc038, SUC_lexicon_tags_12 },
    { 0xb1cdc039, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edc03c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb5cdc03e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6dc042, SUC_lexicon_tags_2 },
    { 0x5b6dc043, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x810dc045, SUC_lexicon_tags_37 },
    { 0xde8dc046, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfeedc048, SUC_lexicon_tags_47 },
    { 0x6c0dc049, SUC_lexicon_tags_60 },
    { 0x6cedc048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x418dc04c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2dc051, SUC_lexicon_tags_9 },
    { 0xa52dc051, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc5cdc054, SUC_lexicon_tags_17 },
    { 0xbadc054, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8dc059, SUC_lexicon_tags_25 },
    { 0x406dc05a, SUC_lexicon_tags_23 },
    { 0xb30dc05a, SUC_lexicon_tags_26 },
    { 0x548dc05b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc58dc05e, SUC_lexicon_tags_35 },
    { 0xa44dc05f, SUC_lexicon_tags_2 },
    { 0x4eedc060, SUC_lexicon_tags_0 },
    { 0xab2dc060, SUC_lexicon_tags_12 },
    { 0x1f4dc062, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x696dc064, SUC_lexicon_tags_3 },
    { 0x3f0dc065, SUC_lexicon_tags_25 },
    { 0x58adc065, SUC_lexicon_tags_37 },
    { 0x4b6dc065, SUC_lexicon_tags_83 },
    { 0x3a8dc067, SUC_lexicon_tags_37 },
    { 0xecadc069, SUC_lexicon_tags_87 },
    { 0x698dc069, SUC_lexicon_tags_66 },
    { 0xf86dc065, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cdc06e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5adc072, SUC_lexicon_tags_80 },
    { 0xe7edc072, SUC_lexicon_tags_29 },
    { 0x522dc074, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x46edc077, SUC_lexicon_tags_12 },
    { 0x45cdc078, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc06dc07a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736dc080, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fedc083, SUC_lexicon_tags_17 },
    { 0x184dc084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0dc08b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x53cdc08d, SUC_lexicon_tags_63 },
    { 0xe76dc08e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x398dc091, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x946dc093, SUC_lexicon_tags_85 },
    { 0xfe4dc094, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdf2dc096, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4dc09c, SUC_lexicon_tags_14 },
    { 0xe80dc09d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecdc0a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cdc0a5, SUC_lexicon_tags_37 },
    { 0x894dc0a6, SUC_lexicon_tags_2 },
    { 0x9e2dc0a6, SUC_lexicon_tags_15 },
    { 0x98adc0a8, SUC_lexicon_tags_9 },
    { 0x72dc0a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcdc0ad, SUC_lexicon_tags_11 },
    { 0xc2adc0ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cdc0b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e6dc0b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fadc0bb, SUC_lexicon_tags_84 },
    { 0xe6dc0bc, SUC_lexicon_tags_37 },
    { 0x4cadc0bd, SUC_lexicon_tags_37 },
    { 0x5c2dc0be, SUC_lexicon_tags_15 },
    { 0x802dc0be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2dc0c2, SUC_lexicon_tags_12 },
    { 0x29edc0c2, SUC_lexicon_tags_1 },
    { 0xe46dc0c4, SUC_lexicon_tags_47 },
    { 0xea6dc0c5, SUC_lexicon_tags_45 },
    { 0x9d0dc0c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x632dc0c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x958dc0ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4dc0cd, SUC_lexicon_tags_17 },
    { 0x7b4dc0ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dc0d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4dc0d6, SUC_lexicon_tags_37 },
    { 0x380dc0d7, SUC_lexicon_tags_45 },
    { 0xf74dc0d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84adc0db, SUC_lexicon_tags_15 },
    { 0x3d4dc0dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f8dc0de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x128dc0e0, SUC_lexicon_tags_63 },
    { 0x6b6dc0e1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xe0adc0e3, SUC_lexicon_tags_11 },
    { 0x34cdc0e3, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130dc0e9, SUC_lexicon_tags_37 },
    { 0x1fcdc0ea, SUC_lexicon_tags_1 },
    { 0xa46dc0ea, SUC_lexicon_tags_11 },
    { 0xc5cdc0eb, SUC_lexicon_tags_17 },
    { 0xaa4dc0e9, SUC_lexicon_tags_17 },
    { 0x3bedc0ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6dc0f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288dc0ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08dc105, SUC_lexicon_tags_37 },
    { 0x52edc105, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19adc110, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cedc113, SUC_lexicon_tags_6 },
    { 0x8c6dc114, SUC_lexicon_tags_1 },
    { 0xa88dc115, SUC_lexicon_tags_14 },
    { 0xcb0dc116, SUC_lexicon_tags_26 },
    { 0x33edc117, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2dc127, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x146dc129, SUC_lexicon_tags_37 },
    { 0x26cdc129, SUC_lexicon_tags_49 },
    { 0xfedc12b, SUC_lexicon_tags_37 },
    { 0x46cdc12c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78edc130, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5adc133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bcdc135, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x424dc137, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cdc13a, SUC_lexicon_tags_3 },
    { 0xe34dc13b, SUC_lexicon_tags_12 },
    { 0x20adc13c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614dc142, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80dc146, SUC_lexicon_tags_2 },
    { 0x550dc147, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30dc14a, SUC_lexicon_tags_37 },
    { 0x75adc14b, SUC_lexicon_tags_17 },
    { 0x886dc14a, SUC_lexicon_tags_33 },
    { 0xd7edc14a, SUC_lexicon_tags_9 },
    { 0x806dc14e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dc152, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680dc157, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2dc15a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2edc15e, SUC_lexicon_tags_16 },
    { 0x790dc15e, SUC_lexicon_tags_25 },
    { 0x346dc160, SUC_lexicon_tags_17 },
    { 0x548dc161, SUC_lexicon_tags_17 },
    { 0x156dc15e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x872dc164, SUC_lexicon_tags_11 },
    { 0xaf2dc165, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3a0dc167, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2dc16f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6dc177, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4cdc179, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x75adc17c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf6adc17e, SUC_lexicon_tags_11 },
    { 0x692dc17f, SUC_lexicon_tags_9 },
    { 0x6f2dc180, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8dc184, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6dc188, SUC_lexicon_tags_12 },
    { 0x58dc188, SUC_lexicon_tags_6 },
    { 0x9d8dc18a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530dc18e, SUC_lexicon_tags_7 },
    { 0xd90dc18f, SUC_lexicon_tags_69 },
    { 0x24adc190, SUC_lexicon_tags_26 },
    { 0x622dc191, SUC_lexicon_tags_62 },
    { 0x784dc192, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53cdc194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6dc19a, SUC_lexicon_tags_17 },
    { 0xd5cdc19b, SUC_lexicon_tags_12 },
    { 0xec0dc19c, SUC_lexicon_tags_7 },
    { 0x5b8dc19d, SUC_lexicon_tags_1 },
    { 0x964dc19e, SUC_lexicon_tags_63 },
    { 0x720dc19b, SUC_lexicon_tags_21 },
    { 0x600dc1a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d0dc1a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc5edc1a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0dc1a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaeadc1a9, SUC_lexicon_tags_17 },
    { 0xa38dc1aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x306dc1ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0dc1b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4badc1b5, SUC_lexicon_tags_29 },
    { 0xbbcdc1b5, SUC_lexicon_tags_0 },
    { 0x58edc1b5, SUC_lexicon_tags_16 },
    { 0x5f8dc1b8, SUC_lexicon_tags_27 },
    { 0x592dc1b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbfadc1bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b0dc1bd, SUC_lexicon_tags_37 },
    { 0x7f6dc1be, SUC_lexicon_tags_33 },
    { 0x7dedc1be, SUC_lexicon_tags_7 },
    { 0x3c0dc1be, SUC_lexicon_tags_49 },
    { 0x98edc1c1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60dc1c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf9cdc1c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18dc1d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bedc1d5, SUC_lexicon_tags_31 },
    { 0xd8dc1d5, SUC_lexicon_tags_37 },
    { 0xc36dc1d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac2dc1d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dadc1dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96dc1e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x478dc1e3, SUC_lexicon_tags_38 },
    { 0x416dc1e4, SUC_lexicon_tags_60 },
    { 0x276dc1e3, SUC_lexicon_tags_29 },
    { 0x1a0dc1e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77cdc1e8, SUC_lexicon_tags_1 },
    { 0xba6dc1e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0dc1ef, SUC_lexicon_tags_37 },
    { 0x9dedc1f0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884dc1f4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7adc1f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2dc1fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cadc202, SUC_lexicon_tags_11 },
    { 0x6eadc202, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd26dc205, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eedc20a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x252dc20d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4dc213, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x602dc215, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebadc21b, SUC_lexicon_tags_17 },
    { 0x900dc21c, SUC_lexicon_tags_1 },
    { 0x852dc21d, SUC_lexicon_tags_65 },
    { 0x26edc21e, SUC_lexicon_tags_19 },
    { 0xd7adc21f, SUC_lexicon_tags_12 },
    { 0xb56dc21c, SUC_lexicon_tags_14 },
    { 0x364dc21f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96edc224, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224dc229, SUC_lexicon_tags_11 },
    { 0xd7cdc229, SUC_lexicon_tags_32 },
    { 0xa86dc22b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7edc22f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adc23c, SUC_lexicon_tags_43 },
    { 0x996dc23c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfeedc23f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefedc247, SUC_lexicon_tags_911 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8edc24d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cdc252, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900dc256, SUC_lexicon_tags_27 },
    { 0x4d4dc257, SUC_lexicon_tags_68 },
    { 0x25adc258, SUC_lexicon_tags_23 },
    { 0x1fadc258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafadc25d, SUC_lexicon_tags_25 },
    { 0xcedc25e, SUC_lexicon_tags_27 },
    { 0xe28dc25d, SUC_lexicon_tags_15 },
    { 0x272dc260, SUC_lexicon_tags_2 },
    { 0xaa6dc25d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8bedc263, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea6dc265, SUC_lexicon_tags_14 },
    { 0x400dc266, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4dc271, SUC_lexicon_tags_17 },
    { 0x97edc272, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62edc276, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44dc279, SUC_lexicon_tags_1 },
    { 0x41cdc27a, SUC_lexicon_tags_12 },
    { 0xcc6dc279, SUC_lexicon_tags_37 },
    { 0x906dc27c, SUC_lexicon_tags_12 },
    { 0x4dedc27d, SUC_lexicon_tags_17 },
    { 0xbd2dc27c, SUC_lexicon_tags_25 },
    { 0xd6edc27f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cdc283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dc289, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7d2dc28b, SUC_lexicon_tags_79 },
    { 0x53edc28c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0edc293, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8dc296, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bedc29e, SUC_lexicon_tags_9 },
    { 0x33adc29f, SUC_lexicon_tags_17 },
    { 0xebadc2a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa0dc2a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcdc2aa, SUC_lexicon_tags_7 },
    { 0xa4dc2ab, SUC_lexicon_tags_11 },
    { 0xbe0dc2ac, SUC_lexicon_tags_4 },
    { 0xd4adc2aa, SUC_lexicon_tags_47 },
    { 0xbe0dc2ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8dc2b2, SUC_lexicon_tags_17 },
    { 0xedadc2b2, SUC_lexicon_tags_55 },
    { 0xe5cdc2b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698dc2b9, SUC_lexicon_tags_25 },
    { 0xd2edc2ba, SUC_lexicon_tags_17 },
    { 0xea0dc2bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46dc2be, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaf6dc2c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x770dc2c2, SUC_lexicon_tags_4 },
    { 0x9d4dc2c2, SUC_lexicon_tags_31 },
    { 0x230dc2c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cdc2cb, SUC_lexicon_tags_2 },
    { 0x922dc2cb, SUC_lexicon_tags_1 },
    { 0x2d2dc2cd, SUC_lexicon_tags_37 },
    { 0x12dc2ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x638dc2d1, SUC_lexicon_tags_542 },
    { 0xd72dc2d2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0dc2d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756dc2dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x720dc2df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7adc2e5, SUC_lexicon_tags_11 },
    { 0x126dc2e6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x844dc2e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9edc2ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x520dc2f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824dc2f5, SUC_lexicon_tags_55 },
    { 0xde0dc2f5, SUC_lexicon_tags_7 },
    { 0x178dc2f6, SUC_lexicon_tags_26 },
    { 0x13adc2f8, SUC_lexicon_tags_1 },
    { 0xf22dc2f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf80dc2fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7badc2fd, SUC_lexicon_tags_39 },
    { 0x88edc2fe, SUC_lexicon_tags_2 },
    { 0x876dc2ff, SUC_lexicon_tags_11 },
    { 0xb9edc2fe, SUC_lexicon_tags_2 },
    { 0x272dc2fd, SUC_lexicon_tags_1 },
    { 0x4b0dc302, SUC_lexicon_tags_11 },
    { 0xba6dc300, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x106dc306, SUC_lexicon_tags_37 },
    { 0x70cdc306, SUC_lexicon_tags_43 },
    { 0xaf0dc308, SUC_lexicon_tags_2 },
    { 0xcdc309, SUC_lexicon_tags_7 },
    { 0xdaadc308, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e8dc30c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744dc311, SUC_lexicon_tags_11 },
    { 0xd36dc312, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2dc316, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8dc321, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6dc325, SUC_lexicon_tags_15 },
    { 0xc7adc326, SUC_lexicon_tags_25 },
    { 0x4eadc327, SUC_lexicon_tags_37 },
    { 0xf74dc326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208dc32c, SUC_lexicon_tags_2 },
    { 0xfd2dc32d, SUC_lexicon_tags_32 },
    { 0x60adc32e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x730dc330, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88dc334, SUC_lexicon_tags_11 },
    { 0x576dc334, SUC_lexicon_tags_20 },
    { 0xb88dc336, SUC_lexicon_tags_15 },
    { 0xdeadc337, SUC_lexicon_tags_2 },
    { 0xae6dc338, SUC_lexicon_tags_2 },
    { 0x402dc339, SUC_lexicon_tags_56 },
    { 0xc08dc33a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x794dc33c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44cdc33e, SUC_lexicon_tags_26 },
    { 0xda8dc33f, SUC_lexicon_tags_7 },
    { 0x62cdc340, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe80dc342, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x91adc344, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54dc347, SUC_lexicon_tags_11 },
    { 0xf66dc347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4dc34a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x876dc34c, SUC_lexicon_tags_77 },
    { 0x386dc34d, SUC_lexicon_tags_27 },
    { 0x62dc34e, SUC_lexicon_tags_37 },
    { 0x714dc34d, SUC_lexicon_tags_7 },
    { 0x6adc350, SUC_lexicon_tags_26 },
    { 0x9f2dc34e, SUC_lexicon_tags_25 },
    { 0x720dc352, SUC_lexicon_tags_15 },
    { 0x894dc353, SUC_lexicon_tags_12 },
    { 0x476dc34e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0dc357, SUC_lexicon_tags_17 },
    { 0xf02dc358, SUC_lexicon_tags_8 },
    { 0x50adc358, SUC_lexicon_tags_18 },
    { 0xd8cdc35a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136dc35f, SUC_lexicon_tags_64 },
    { 0xef0dc35f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0dc365, SUC_lexicon_tags_15 },
    { 0x25cdc365, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cdc36c, SUC_lexicon_tags_48 },
    { 0xbf4dc36d, SUC_lexicon_tags_55 },
    { 0xbf2dc36d, SUC_lexicon_tags_55 },
    { 0xae0dc36f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcdc375, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x386dc378, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0dc37c, SUC_lexicon_tags_17 },
    { 0x9c8dc37d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ccdc37f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750dc384, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd16dc386, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcdc389, SUC_lexicon_tags_17 },
    { 0x93edc389, SUC_lexicon_tags_87 },
    { 0x97adc38b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78dc38f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8dc394, SUC_lexicon_tags_11 },
    { 0xdb0dc394, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162dc39b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06dc39f, SUC_lexicon_tags_2 },
    { 0x77edc3a0, SUC_lexicon_tags_25 },
    { 0x55edc3a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61edc3a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6dc3ad, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x982dc3af, SUC_lexicon_tags_17 },
    { 0x840dc3b0, SUC_lexicon_tags_43 },
    { 0xa50dc3b0, SUC_lexicon_tags_9 },
    { 0x5ccdc3b2, SUC_lexicon_tags_37 },
    { 0x31cdc3b3, SUC_lexicon_tags_11 },
    { 0x868dc3b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8dc3b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6dc3bc, SUC_lexicon_tags_14 },
    { 0xf50dc3bd, SUC_lexicon_tags_60 },
    { 0xb1adc3bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcdc3c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xddadc3c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6cedc3c7, SUC_lexicon_tags_84 },
    { 0x1edc3c8, SUC_lexicon_tags_138 },
    { 0x784dc3c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3cd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe66dc3cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80adc3d1, SUC_lexicon_tags_16 },
    { 0xde0dc3d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4dc3dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4edc3de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cdc3e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416dc3e5, SUC_lexicon_tags_43 },
    { 0x228dc3e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dc3f6, SUC_lexicon_tags_39 },
    { 0x5e6dc3f7, SUC_lexicon_tags_7 },
    { 0x4eedc3f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87edc3fb, SUC_lexicon_tags_0 },
    { 0x280dc3fc, SUC_lexicon_tags_8 },
    { 0x6fadc3fd, SUC_lexicon_tags_15 },
    { 0x52edc3fe, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf58dc400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120dc405, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146dc40a, SUC_lexicon_tags_21 },
    { 0x9dcdc40b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9edc40d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa92dc40f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8dc418, SUC_lexicon_tags_46 },
    { 0xd60dc419, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20adc41b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc2dc41d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62dc420, SUC_lexicon_tags_21 },
    { 0x864dc421, SUC_lexicon_tags_19 },
    { 0x766dc422, SUC_lexicon_tags_45 },
    { 0xb9adc422, SUC_lexicon_tags_64 },
    { 0x888dc422, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40dc427, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8fadc429, SUC_lexicon_tags_113 },
    { 0x122dc42a, SUC_lexicon_tags_12 },
    { 0xdb2dc42a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dc431, SUC_lexicon_tags_27 },
    { 0x50dc432, SUC_lexicon_tags_15 },
    { 0xf1edc432, SUC_lexicon_tags_25 },
    { 0x9a4dc433, SUC_lexicon_tags_104 },
    { 0x20edc433, SUC_lexicon_tags_11 },
    { 0xb54dc434, SUC_lexicon_tags_11 },
    { 0x658dc437, SUC_lexicon_tags_17 },
    { 0xd96dc432, SUC_lexicon_tags_11 },
    { 0x93cdc439, SUC_lexicon_tags_1 },
    { 0xd18dc43a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeb0dc43c, SUC_lexicon_tags_14 },
    { 0x844dc43c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19adc442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf82dc444, SUC_lexicon_tags_60 },
    { 0xde2dc445, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166dc44c, SUC_lexicon_tags_81 },
    { 0x3cadc44d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd6edc44f, SUC_lexicon_tags_23 },
    { 0x666dc44f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0dc457, SUC_lexicon_tags_107 },
    { 0xfe2dc458, SUC_lexicon_tags_19 },
    { 0xa18dc457, SUC_lexicon_tags_32 },
    { 0x87cdc45a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb54dc45c, SUC_lexicon_tags_11 },
    { 0xf2dc45d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94dc461, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12dc464, SUC_lexicon_tags_7 },
    { 0xe7edc465, SUC_lexicon_tags_92 },
    { 0x564dc464, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466dc46b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x92adc46e, SUC_lexicon_tags_47 },
    { 0x914dc46f, SUC_lexicon_tags_40 },
    { 0x82adc46f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2dc478, SUC_lexicon_tags_49 },
    { 0x73adc479, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc54dc47b, SUC_lexicon_tags_38 },
    { 0xe14dc47b, SUC_lexicon_tags_12 },
    { 0xdc4dc47d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x692dc47f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc76dc482, SUC_lexicon_tags_11 },
    { 0x8a0dc483, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x426dc485, SUC_lexicon_tags_27 },
    { 0x4d0dc485, SUC_lexicon_tags_31 },
    { 0x91edc487, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x79edc489, SUC_lexicon_tags_12 },
    { 0x866dc489, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cdc48d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4dc491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0dc499, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374dc49e, SUC_lexicon_tags_26 },
    { 0x7f8dc49f, SUC_lexicon_tags_63 },
    { 0x306dc49f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfedc4a6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47adc4ad, SUC_lexicon_tags_0 },
    { 0x2b8dc4ae, SUC_lexicon_tags_17 },
    { 0xa32dc4ae, SUC_lexicon_tags_14 },
    { 0x78cdc4af, SUC_lexicon_tags_49 },
    { 0x966dc4b1, SUC_lexicon_tags_1 },
    { 0xb7adc4b1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x404dc4b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73edc4bb, SUC_lexicon_tags_17 },
    { 0x2d8dc4bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89cdc4be, SUC_lexicon_tags_14 },
    { 0x2a8dc4bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xffedc4c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4dc4c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x56edc4c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4dc4cd, SUC_lexicon_tags_7 },
    { 0x792dc4cd, SUC_lexicon_tags_295 },
    { 0x900dc4cf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaadc4d5, SUC_lexicon_tags_38 },
    { 0x8d6dc4d5, SUC_lexicon_tags_72 },
    { 0x834dc4d7, SUC_lexicon_tags_9 },
    { 0xe08dc4d8, SUC_lexicon_tags_43 },
    { 0x12dc4d8, SUC_lexicon_tags_2 },
    { 0x492dc4d9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x244dc4dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5fcdc4df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78dc4ed, SUC_lexicon_tags_2 },
    { 0xcaadc4ed, SUC_lexicon_tags_15 },
    { 0xb6dc4ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d6dc4f1, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcdc4f6, SUC_lexicon_tags_5 },
    { 0xfb6dc4f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aedc4fc, SUC_lexicon_tags_6 },
    { 0xc5cdc4fd, SUC_lexicon_tags_23 },
    { 0xa26dc4fe, SUC_lexicon_tags_25 },
    { 0x8fedc4fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22dc502, SUC_lexicon_tags_11 },
    { 0xc1cdc503, SUC_lexicon_tags_4 },
    { 0x6a8dc502, SUC_lexicon_tags_42 },
    { 0x2b0dc505, SUC_lexicon_tags_24 },
    { 0x7eedc506, SUC_lexicon_tags_11 },
    { 0xe64dc505, SUC_lexicon_tags_7 },
    { 0xedadc508, SUC_lexicon_tags_11 },
    { 0xeedc507, SUC_lexicon_tags_11 },
    { 0x902dc50a, SUC_lexicon_tags_151 },
    { 0x2dedc50b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9edc50d, SUC_lexicon_tags_12 },
    { 0xe0adc50e, SUC_lexicon_tags_31 },
    { 0xba4dc50f, SUC_lexicon_tags_12 },
    { 0x4ecdc510, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f4dc512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6dc51f, SUC_lexicon_tags_7 },
    { 0x64dc520, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fadc523, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166dc52a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62dc52d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6dc530, SUC_lexicon_tags_67 },
    { 0x7c2dc531, SUC_lexicon_tags_1 },
    { 0x41adc532, SUC_lexicon_tags_19 },
    { 0xfc6dc533, SUC_lexicon_tags_12 },
    { 0x368dc534, SUC_lexicon_tags_37 },
    { 0xb6adc535, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dc53b, SUC_lexicon_tags_1 },
    { 0x3d8dc53c, SUC_lexicon_tags_25 },
    { 0xc8adc53d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0dc540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcdc546, SUC_lexicon_tags_8 },
    { 0x996dc547, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53adc54a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2dc54e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63adc552, SUC_lexicon_tags_12 },
    { 0xb22dc553, SUC_lexicon_tags_63 },
    { 0x48dc554, SUC_lexicon_tags_14 },
    { 0x4a2dc555, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6adc55b, SUC_lexicon_tags_69 },
    { 0x2e6dc55c, SUC_lexicon_tags_66 },
    { 0xc22dc55c, SUC_lexicon_tags_49 },
    { 0xf6adc55e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92dc566, SUC_lexicon_tags_15 },
    { 0x7aedc567, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8dc56a, SUC_lexicon_tags_27 },
    { 0x16cdc56b, SUC_lexicon_tags_37 },
    { 0x4f4dc56c, SUC_lexicon_tags_30 },
    { 0x1f0dc56d, SUC_lexicon_tags_46 },
    { 0x6b0dc56e, SUC_lexicon_tags_37 },
    { 0x50adc56e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8dc574, SUC_lexicon_tags_12 },
    { 0x486dc574, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafadc57b, SUC_lexicon_tags_37 },
    { 0xd00dc57b, SUC_lexicon_tags_17 },
    { 0xd88dc57d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4dc581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x576dc583, SUC_lexicon_tags_11 },
    { 0x652dc584, SUC_lexicon_tags_0 },
    { 0x518dc584, SUC_lexicon_tags_16 },
    { 0x660dc586, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x676dc588, SUC_lexicon_tags_63 },
    { 0x66adc589, SUC_lexicon_tags_14 },
    { 0x4a0dc589, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cdc58e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x312dc591, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582dc596, SUC_lexicon_tags_40 },
    { 0x494dc597, SUC_lexicon_tags_43 },
    { 0x6bcdc597, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6dc59d, SUC_lexicon_tags_13 },
    { 0x18dc59e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5cdc5a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e0dc5a2, SUC_lexicon_tags_84 },
    { 0x96adc5a2, SUC_lexicon_tags_15 },
    { 0x61edc5a2, SUC_lexicon_tags_11 },
    { 0xcbcdc5a5, SUC_lexicon_tags_15 },
    { 0x38edc5a5, SUC_lexicon_tags_15 },
    { 0xb32dc5a6, SUC_lexicon_tags_37 },
    { 0x40cdc5a8, SUC_lexicon_tags_23 },
    { 0xce6dc5a9, SUC_lexicon_tags_11 },
    { 0xe46dc5a6, SUC_lexicon_tags_38 },
    { 0xa74dc5ab, SUC_lexicon_tags_1 },
    { 0xeaedc5ab, SUC_lexicon_tags_7 },
    { 0xe7adc5ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79edc5b5, SUC_lexicon_tags_7 },
    { 0x56edc5b6, SUC_lexicon_tags_33 },
    { 0xaa2dc5b7, SUC_lexicon_tags_6 },
    { 0x194dc5b8, SUC_lexicon_tags_44 },
    { 0xa30dc5b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdf0dc5bb, SUC_lexicon_tags_11 },
    { 0x756dc5bc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6dc5bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8dc5c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x58edc5c5, SUC_lexicon_tags_68 },
    { 0x7d4dc5c6, SUC_lexicon_tags_179 },
    { 0x4aedc5c5, SUC_lexicon_tags_1 },
    { 0x7cdc5c8, SUC_lexicon_tags_7 },
    { 0xecadc5c9, SUC_lexicon_tags_10 },
    { 0xf7cdc5ca, SUC_lexicon_tags_15 },
    { 0x86cdc5c7, SUC_lexicon_tags_8 },
    { 0xad8dc5cc, SUC_lexicon_tags_17 },
    { 0x5a0dc5c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b0dc5cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x854dc5d2, SUC_lexicon_tags_15 },
    { 0xe68dc5d2, SUC_lexicon_tags_63 },
    { 0x438dc5d2, SUC_lexicon_tags_2 },
    { 0x852dc5d5, SUC_lexicon_tags_12 },
    { 0xa82dc5d5, SUC_lexicon_tags_11 },
    { 0x6dcdc5d2, SUC_lexicon_tags_114 },
    { 0xd00dc5d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920dc5dc, SUC_lexicon_tags_60 },
    { 0x468dc5dc, SUC_lexicon_tags_271 },
    { 0x492dc5de, SUC_lexicon_tags_47 },
    { 0xcf4dc5de, SUC_lexicon_tags_2 },
    { 0x80cdc5e0, SUC_lexicon_tags_84 },
    { 0xd0cdc5e1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502dc5e5, SUC_lexicon_tags_17 },
    { 0x640dc5e6, SUC_lexicon_tags_33 },
    { 0x278dc5e7, SUC_lexicon_tags_25 },
    { 0x48cdc5e6, SUC_lexicon_tags_27 },
    { 0x166dc5e9, SUC_lexicon_tags_65 },
    { 0xd18dc5e7, SUC_lexicon_tags_25 },
    { 0x136dc5e9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452dc5fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4dc602, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edc607, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edc610, SUC_lexicon_tags_19 },
    { 0x26cdc611, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0dc614, SUC_lexicon_tags_2 },
    { 0x152dc615, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cdc61d, SUC_lexicon_tags_39 },
    { 0x46cdc61e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92dc622, SUC_lexicon_tags_61 },
    { 0xe76dc623, SUC_lexicon_tags_15 },
    { 0xedc624, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e8dc626, SUC_lexicon_tags_7 },
    { 0x9b8dc626, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcedc635, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464dc639, SUC_lexicon_tags_38 },
    { 0xd90dc63a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccdc63e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x638dc640, SUC_lexicon_tags_7 },
    { 0xfe2dc641, SUC_lexicon_tags_11 },
    { 0x560dc641, SUC_lexicon_tags_15 },
    { 0x25edc643, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16dc64a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd64dc64c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dadc654, SUC_lexicon_tags_16 },
    { 0xc3adc654, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcdc658, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26dc65d, SUC_lexicon_tags_12 },
    { 0xab6dc65e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18edc662, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6dc665, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8dc668, SUC_lexicon_tags_37 },
    { 0x664dc668, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4dc66b, SUC_lexicon_tags_27 },
    { 0x21adc66b, SUC_lexicon_tags_6 },
    { 0x88dc66d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78dc672, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf1edc674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82adc678, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2acdc67a, SUC_lexicon_tags_6 },
    { 0xfb6dc67b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd46dc67d, SUC_lexicon_tags_9 },
    { 0x8d6dc67e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x482dc681, SUC_lexicon_tags_34 },
    { 0x1e4dc682, SUC_lexicon_tags_12 },
    { 0xa2dc683, SUC_lexicon_tags_55 },
    { 0x432dc684, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x3f4dc686, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8dc68a, SUC_lexicon_tags_23 },
    { 0xfecdc68b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228dc68f, SUC_lexicon_tags_0 },
    { 0x944dc690, SUC_lexicon_tags_47 },
    { 0xacdc68f, SUC_lexicon_tags_15 },
    { 0xdf4dc68f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdadc695, SUC_lexicon_tags_33 },
    { 0xbcdc696, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234dc6a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cdc6a4, SUC_lexicon_tags_12 },
    { 0xb88dc6a4, SUC_lexicon_tags_29 },
    { 0xbbedc6a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6a4dc6a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6f2dc6aa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154dc6af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x16cdc6b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6dc6b6, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x44dc6b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x312dc6bb, SUC_lexicon_tags_7 },
    { 0x346dc6bc, SUC_lexicon_tags_16 },
    { 0xac4dc6bb, SUC_lexicon_tags_15 },
    { 0x820dc6be, SUC_lexicon_tags_7 },
    { 0xd94dc6bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0dc6c3, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0x38edc6c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478dc6cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76cdc6cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x924dc6d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadc6d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496dc6d9, SUC_lexicon_tags_14 },
    { 0x410dc6da, SUC_lexicon_tags_15 },
    { 0x32cdc6db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfd0dc6dd, SUC_lexicon_tags_1 },
    { 0x49edc6de, SUC_lexicon_tags_6 },
    { 0xa1cdc6df, SUC_lexicon_tags_37 },
    { 0x9ecdc6de, SUC_lexicon_tags_7 },
    { 0x5a0dc6e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cdc6e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x134dc6eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce4dc6ed, SUC_lexicon_tags_25 },
    { 0x4e0dc6ee, SUC_lexicon_tags_105 },
    { 0x20cdc6ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6badc6f3, SUC_lexicon_tags_25 },
    { 0xd3edc6f4, SUC_lexicon_tags_11 },
    { 0x818dc6f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9edc6f7, SUC_lexicon_tags_6 },
    { 0x366dc6f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a2dc6fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4adc700, SUC_lexicon_tags_10 },
    { 0x3c6dc701, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb8dc703, SUC_lexicon_tags_49 },
    { 0x1b6dc704, SUC_lexicon_tags_2 },
    { 0x12edc703, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2dc708, SUC_lexicon_tags_12 },
    { 0x5e8dc709, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x330dc70b, SUC_lexicon_tags_12 },
    { 0xd04dc70b, SUC_lexicon_tags_37 },
    { 0xd9cdc70d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0dc714, SUC_lexicon_tags_49 },
    { 0x784dc715, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9dedc717, SUC_lexicon_tags_14 },
    { 0xc50dc718, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc71d, SUC_lexicon_tags_2 },
    { 0xcecdc71e, SUC_lexicon_tags_7 },
    { 0xaadc71e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aedc721, SUC_lexicon_tags_31 },
    { 0x9d0dc722, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78dc727, SUC_lexicon_tags_15 },
    { 0xd98dc728, SUC_lexicon_tags_8 },
    { 0xf6edc728, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06dc72e, SUC_lexicon_tags_30 },
    { 0x786dc72e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24dc733, SUC_lexicon_tags_24 },
    { 0x4aadc733, SUC_lexicon_tags_37 },
    { 0xfc4dc735, SUC_lexicon_tags_61 },
    { 0x602dc736, SUC_lexicon_tags_37 },
    { 0xa5adc737, SUC_lexicon_tags_28 },
    { 0x7e2dc734, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1e6dc73a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64cdc73c, SUC_lexicon_tags_45 },
    { 0xb98dc73d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7c0dc73f, SUC_lexicon_tags_12 },
    { 0x504dc740, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8dc743, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaaadc745, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394dc74a, SUC_lexicon_tags_26 },
    { 0xc58dc74a, SUC_lexicon_tags_27 },
    { 0x4a4dc74c, SUC_lexicon_tags_39 },
    { 0x4f6dc74d, SUC_lexicon_tags_87 },
    { 0x328dc74e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x126dc750, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14dc753, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0dc759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940dc75f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfcdc761, SUC_lexicon_tags_23 },
    { 0x8f8dc762, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7c0dc764, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4dc767, SUC_lexicon_tags_2 },
    { 0x18adc768, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x590dc76a, SUC_lexicon_tags_7 },
    { 0xdadc76b, SUC_lexicon_tags_37 },
    { 0x26dc76b, SUC_lexicon_tags_0 },
    { 0xe3adc76d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaedc772, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cdc775, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60dc77a, SUC_lexicon_tags_23 },
    { 0x758dc77b, SUC_lexicon_tags_0 },
    { 0x37edc77c, SUC_lexicon_tags_28 },
    { 0x4a4dc77d, SUC_lexicon_tags_37 },
    { 0x58edc77a, SUC_lexicon_tags_19 },
    { 0xf32dc77f, SUC_lexicon_tags_1 },
    { 0xd12dc780, SUC_lexicon_tags_7 },
    { 0x2e2dc77b, SUC_lexicon_tags_60 },
    { 0x212dc77a, SUC_lexicon_tags_25 },
    { 0xe84dc783, SUC_lexicon_tags_37 },
    { 0xba4dc783, SUC_lexicon_tags_2 },
    { 0xb14dc785, SUC_lexicon_tags_60 },
    { 0xf26dc77e, SUC_lexicon_tags_82 },
    { 0x132dc782, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cdc78a, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48dc78d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0dc791, SUC_lexicon_tags_17 },
    { 0xd10dc791, SUC_lexicon_tags_25 },
    { 0x64dc791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8dc797, SUC_lexicon_tags_40 },
    { 0x47adc797, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1cdc79a, SUC_lexicon_tags_49 },
    { 0x36dc79b, SUC_lexicon_tags_17 },
    { 0xfe2dc79c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22cdc79e, SUC_lexicon_tags_15 },
    { 0x51edc79f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4dc7a5, SUC_lexicon_tags_66 },
    { 0x31cdc7a5, SUC_lexicon_tags_13 },
    { 0x32edc7a7, SUC_lexicon_tags_17 },
    { 0xbb0dc7a7, SUC_lexicon_tags_12 },
    { 0xf9edc7a9, SUC_lexicon_tags_23 },
    { 0xb28dc7a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90edc7af, SUC_lexicon_tags_15 },
    { 0xe7edc7b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386dc7b4, SUC_lexicon_tags_1 },
    { 0x1b0dc7b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc12dc7b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4dc7ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d8dc7bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340dc7c2, SUC_lexicon_tags_16 },
    { 0x734dc7c2, SUC_lexicon_tags_6 },
    { 0x7adc7c4, SUC_lexicon_tags_7 },
    { 0x876dc7c5, SUC_lexicon_tags_62 },
    { 0x6b4dc7c2, SUC_lexicon_tags_19 },
    { 0xbd8dc7c7, SUC_lexicon_tags_3 },
    { 0x39adc7c7, SUC_lexicon_tags_37 },
    { 0x72dc7c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf14dc7cb, SUC_lexicon_tags_25 },
    { 0xc9cdc7cc, SUC_lexicon_tags_43 },
    { 0x32edc7cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7adc7cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x56adc7d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x704dc7d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98dc7d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcaedc7db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410dc7e0, SUC_lexicon_tags_11 },
    { 0x89edc7e0, SUC_lexicon_tags_14 },
    { 0xcc2dc7e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x104dc7e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbedc7e6, SUC_lexicon_tags_17 },
    { 0xf9cdc7e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x558dc7e9, SUC_lexicon_tags_7 },
    { 0x4a0dc7ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512dc7ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726dc7f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2edc7f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfadc7f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x830dc7fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2dc7ff, SUC_lexicon_tags_37 },
    { 0xf48dc800, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32dc806, SUC_lexicon_tags_1 },
    { 0x49edc807, SUC_lexicon_tags_14 },
    { 0x882dc807, SUC_lexicon_tags_11 },
    { 0x4d2dc809, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20dc80d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88dc812, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x514dc815, SUC_lexicon_tags_40 },
    { 0x6dedc815, SUC_lexicon_tags_11 },
    { 0x8f6dc817, SUC_lexicon_tags_4 },
    { 0x248dc816, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcdc81b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9edc81e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf0dc820, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x87edc822, SUC_lexicon_tags_37 },
    { 0x4fedc823, SUC_lexicon_tags_1 },
    { 0x7cedc824, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x136dc826, SUC_lexicon_tags_9 },
    { 0xc2adc826, SUC_lexicon_tags_63 },
    { 0x14cdc828, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0dc82b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30dc82e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x18cdc830, SUC_lexicon_tags_37 },
    { 0xd18dc831, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc38dc834, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582dc838, SUC_lexicon_tags_14 },
    { 0x8d0dc838, SUC_lexicon_tags_33 },
    { 0xaa4dc83a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06dc83e, SUC_lexicon_tags_17 },
    { 0x980dc83f, SUC_lexicon_tags_12 },
    { 0x390dc83f, SUC_lexicon_tags_37 },
    { 0x154dc841, SUC_lexicon_tags_31 },
    { 0x5cadc842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bedc845, SUC_lexicon_tags_40 },
    { 0x82adc846, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506dc84a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fedc84c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c6dc84e, SUC_lexicon_tags_11 },
    { 0xea2dc84f, SUC_lexicon_tags_9 },
    { 0xc80dc84e, SUC_lexicon_tags_25 },
    { 0x360dc851, SUC_lexicon_tags_37 },
    { 0x150dc84e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6dc855, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98dc85d, SUC_lexicon_tags_1 },
    { 0xe68dc85d, SUC_lexicon_tags_38 },
    { 0xb16dc85f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc864, SUC_lexicon_tags_7 },
    { 0x82adc865, SUC_lexicon_tags_11 },
    { 0x9d0dc866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550dc86b, SUC_lexicon_tags_60 },
    { 0x4dadc86c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0dc875, SUC_lexicon_tags_26 },
    { 0xd24dc876, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8dc87a, SUC_lexicon_tags_14 },
    { 0x9d8dc87b, SUC_lexicon_tags_17 },
    { 0x424dc87a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6dc880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cdc884, SUC_lexicon_tags_8 },
    { 0x24cdc884, SUC_lexicon_tags_21 },
    { 0xaaadc886, SUC_lexicon_tags_43 },
    { 0xddadc887, SUC_lexicon_tags_17 },
    { 0x990dc888, SUC_lexicon_tags_12 },
    { 0xc70dc889, SUC_lexicon_tags_25 },
    { 0xf0cdc889, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6dc88e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7e2dc890, SUC_lexicon_tags_17 },
    { 0xdb4dc891, SUC_lexicon_tags_8 },
    { 0x47edc891, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x658dc894, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0dc898, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8dc89d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x3fadc89f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eedc8a3, SUC_lexicon_tags_14 },
    { 0xc46dc8a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8dc8a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4dc8aa, SUC_lexicon_tags_47 },
    { 0xbc4dc8ab, SUC_lexicon_tags_150 },
    { 0xe2dc8ab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9ecdc8ae, SUC_lexicon_tags_8 },
    { 0x810dc8af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8fadc8b1, SUC_lexicon_tags_38 },
    { 0x9e8dc8b2, SUC_lexicon_tags_160 },
    { 0xa12dc8b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2dc8b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5edc8ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x34edc8bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91edc8bf, SUC_lexicon_tags_25 },
    { 0xc46dc8c0, SUC_lexicon_tags_7 },
    { 0xc74dc8c1, SUC_lexicon_tags_7 },
    { 0xf84dc8c2, SUC_lexicon_tags_17 },
    { 0xd82dc8c0, SUC_lexicon_tags_25 },
    { 0x990dc8c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bedc8c9, SUC_lexicon_tags_29 },
    { 0x75edc8c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x972dc8cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70dc8d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20dc8d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dc8d6, SUC_lexicon_tags_287 },
    { 0x7ccdc8d6, SUC_lexicon_tags_1 },
    { 0xf8cdc8d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6acdc8db, SUC_lexicon_tags_11 },
    { 0xd30dc8dc, SUC_lexicon_tags_61 },
    { 0xbbadc8dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x73edc8e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748dc8e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02dc8e7, SUC_lexicon_tags_27 },
    { 0x22cdc8e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65adc8ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c0dc8ec, SUC_lexicon_tags_45 },
    { 0x88adc8ed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94dc8f0, SUC_lexicon_tags_66 },
    { 0xf32dc8f0, SUC_lexicon_tags_15 },
    { 0x640dc8f2, SUC_lexicon_tags_17 },
    { 0xdf8dc8f1, SUC_lexicon_tags_1 },
    { 0xa4adc8f3, SUC_lexicon_tags_11 },
    { 0xcf2dc8f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23cdc8f7, SUC_lexicon_tags_12 },
    { 0xa86dc8f8, SUC_lexicon_tags_29 },
    { 0xc56dc8f9, SUC_lexicon_tags_14 },
    { 0x768dc8f8, SUC_lexicon_tags_25 },
    { 0x424dc8fa, SUC_lexicon_tags_2 },
    { 0x406dc8f7, SUC_lexicon_tags_14 },
    { 0xd60dc8fd, SUC_lexicon_tags_1 },
    { 0x7b4dc8f7, SUC_lexicon_tags_2 },
    { 0x800dc8f9, SUC_lexicon_tags_7 },
    { 0x904dc900, SUC_lexicon_tags_48 },
    { 0x95cdc901, SUC_lexicon_tags_10 },
    { 0x830dc900, SUC_lexicon_tags_15 },
    { 0xa82dc903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fadc907, SUC_lexicon_tags_37 },
    { 0xfa2dc907, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cdc90b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15cdc90e, SUC_lexicon_tags_12 },
    { 0xcb4dc90e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06dc912, SUC_lexicon_tags_17 },
    { 0x7f2dc913, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x49adc916, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x772dc918, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x302dc91b, SUC_lexicon_tags_37 },
    { 0x7ecdc91c, SUC_lexicon_tags_19 },
    { 0x822dc91d, SUC_lexicon_tags_1 },
    { 0xa46dc91c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6dc921, SUC_lexicon_tags_17 },
    { 0x7d2dc922, SUC_lexicon_tags_45 },
    { 0xb12dc923, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1edc926, SUC_lexicon_tags_7 },
    { 0x312dc927, SUC_lexicon_tags_27 },
    { 0x15edc928, SUC_lexicon_tags_1 },
    { 0xd74dc929, SUC_lexicon_tags_30 },
    { 0x8c0dc927, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42dc92e, SUC_lexicon_tags_25 },
    { 0x9b0dc92f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x192dc932, SUC_lexicon_tags_66 },
    { 0xfe8dc932, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4dc938, SUC_lexicon_tags_82 },
    { 0x8aadc938, SUC_lexicon_tags_1 },
    { 0xb42dc93a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8dc93e, SUC_lexicon_tags_17 },
    { 0x16dc93f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d4dc941, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe18dc943, SUC_lexicon_tags_23 },
    { 0x7c4dc944, SUC_lexicon_tags_15 },
    { 0x95adc945, SUC_lexicon_tags_14 },
    { 0x618dc945, SUC_lexicon_tags_15 },
    { 0x628dc945, SUC_lexicon_tags_87 },
    { 0x11cdc948, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadc94b, SUC_lexicon_tags_46 },
    { 0x40dc94c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cdc950, SUC_lexicon_tags_11 },
    { 0x932dc951, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x956dc953, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2dc956, SUC_lexicon_tags_26 },
    { 0xbd0dc957, SUC_lexicon_tags_11 },
    { 0x588dc957, SUC_lexicon_tags_2 },
    { 0x76cdc959, SUC_lexicon_tags_27 },
    { 0xa6edc957, SUC_lexicon_tags_4 },
    { 0x67cdc95b, SUC_lexicon_tags_0 },
    { 0x36edc95b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dc962, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x814dc964, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dc968, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60adc96a, SUC_lexicon_tags_47 },
    { 0x4a4dc96b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb1cdc96d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70adc971, SUC_lexicon_tags_11 },
    { 0xb04dc972, SUC_lexicon_tags_25 },
    { 0xd2edc973, SUC_lexicon_tags_11 },
    { 0x224dc973, SUC_lexicon_tags_17 },
    { 0x2f4dc975, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06dc97d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134dc982, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x758dc985, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aadc989, SUC_lexicon_tags_16 },
    { 0x416dc989, SUC_lexicon_tags_25 },
    { 0x57edc989, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x378dc98d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x510dc98f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cdc996, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x51adc998, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1dadc99a, SUC_lexicon_tags_8 },
    { 0x4cadc99b, SUC_lexicon_tags_1 },
    { 0xce0dc99a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x97cdc99f, SUC_lexicon_tags_65 },
    { 0xd80dc9a0, SUC_lexicon_tags_39 },
    { 0x730dc9a0, SUC_lexicon_tags_6 },
    { 0xcdadc9a1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970dc9a7, SUC_lexicon_tags_37 },
    { 0x7c4dc9a8, SUC_lexicon_tags_31 },
    { 0x50dc9a9, SUC_lexicon_tags_7 },
    { 0x1c0dc9aa, SUC_lexicon_tags_0 },
    { 0x7d8dc9ab, SUC_lexicon_tags_2 },
    { 0xff6dc9ac, SUC_lexicon_tags_31 },
    { 0xe20dc9ad, SUC_lexicon_tags_148 },
    { 0xebedc9ab, SUC_lexicon_tags_38 },
    { 0xf4edc9ac, SUC_lexicon_tags_1 },
    { 0x136dc9aa, SUC_lexicon_tags_63 },
    { 0x16edc9b1, SUC_lexicon_tags_11 },
    { 0xbe2dc9b1, SUC_lexicon_tags_25 },
    { 0xea2dc9b3, SUC_lexicon_tags_19 },
    { 0x8d4dc9b3, SUC_lexicon_tags_17 },
    { 0x9d6dc9b5, SUC_lexicon_tags_55 },
    { 0xa36dc9b6, SUC_lexicon_tags_27 },
    { 0xc26dc9b5, SUC_lexicon_tags_12 },
    { 0x1f0dc9b6, SUC_lexicon_tags_1 },
    { 0x5a4dc9b9, SUC_lexicon_tags_33 },
    { 0x37edc9b9, SUC_lexicon_tags_39 },
    { 0x950dc9bb, SUC_lexicon_tags_15 },
    { 0x256dc9bc, SUC_lexicon_tags_43 },
    { 0x90cdc9bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5adc9bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dedc9c8, SUC_lexicon_tags_47 },
    { 0x380dc9c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28adc9d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x93adc9d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x890dc9d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8dc9d8, SUC_lexicon_tags_8 },
    { 0xfa4dc9d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cdc9dd, SUC_lexicon_tags_1 },
    { 0xd7edc9dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4dc9e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x318dc9e9, SUC_lexicon_tags_16 },
    { 0xa7cdc9ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x932dc9ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x550dc9ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x736dc9f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98dc9f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96dc9fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x458dc9fd, SUC_lexicon_tags_19 },
    { 0xbacdc9fd, SUC_lexicon_tags_2 },
    { 0xd02dc9ff, SUC_lexicon_tags_12 },
    { 0xa80dca00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16dca03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x60adca05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x314dca07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6dca0a, SUC_lexicon_tags_37 },
    { 0xdc4dca0b, SUC_lexicon_tags_60 },
    { 0x1b2dca0c, SUC_lexicon_tags_9 },
    { 0x1d6dca0b, SUC_lexicon_tags_98 },
    { 0xc98dca0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf0dca10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x812dca12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7adca15, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fedca19, SUC_lexicon_tags_2 },
    { 0xce0dca1a, SUC_lexicon_tags_7 },
    { 0xb18dca1b, SUC_lexicon_tags_3 },
    { 0x584dca1b, SUC_lexicon_tags_63 },
    { 0x6a0dca1b, SUC_lexicon_tags_25 },
    { 0x84cdca1e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940dca23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22adca29, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6dca2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x504dca2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedca32, SUC_lexicon_tags_29 },
    { 0x688dca33, SUC_lexicon_tags_12 },
    { 0x52adca34, SUC_lexicon_tags_0 },
    { 0x4d6dca35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0dca39, SUC_lexicon_tags_69 },
    { 0x974dca3a, SUC_lexicon_tags_47 },
    { 0xc8edca3a, SUC_lexicon_tags_0 },
    { 0x252dca3b, SUC_lexicon_tags_17 },
    { 0x6d2dca3d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cdca41, SUC_lexicon_tags_25 },
    { 0x62dca42, SUC_lexicon_tags_7 },
    { 0xab0dca42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8dca46, SUC_lexicon_tags_2 },
    { 0xeecdca47, SUC_lexicon_tags_18 },
    { 0x9a6dca47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd46dca4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b8dca4c, SUC_lexicon_tags_25 },
    { 0xd9adca4d, SUC_lexicon_tags_47 },
    { 0xbb6dca4d, SUC_lexicon_tags_60 },
    { 0xb68dca4e, SUC_lexicon_tags_15 },
    { 0xf6adca50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb2dca52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc72dca54, SUC_lexicon_tags_35 },
    { 0x64cdca55, SUC_lexicon_tags_26 },
    { 0x648dca56, SUC_lexicon_tags_11 },
    { 0x75edca57, SUC_lexicon_tags_11 },
    { 0x928dca58, SUC_lexicon_tags_5 },
    { 0xa60dca58, SUC_lexicon_tags_17 },
    { 0xe4dca56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x378dca5d, SUC_lexicon_tags_75 },
    { 0x46cdca5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cdca62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xed2dca64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306dca68, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86edca6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x226dca71, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x1b4dca73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x652dca76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dca79, SUC_lexicon_tags_38 },
    { 0x6cdca79, SUC_lexicon_tags_17 },
    { 0x6cadca7a, SUC_lexicon_tags_17 },
    { 0x1e8dca79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8edca81, SUC_lexicon_tags_23 },
    { 0x80dca82, SUC_lexicon_tags_17 },
    { 0x486dca83, SUC_lexicon_tags_11 },
    { 0xb76dca84, SUC_lexicon_tags_15 },
    { 0xe44dca84, SUC_lexicon_tags_19 },
    { 0x814dca82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72adca88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb6cdca8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2dca8d, SUC_lexicon_tags_46 },
    { 0x33adca8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcdca9a, SUC_lexicon_tags_12 },
    { 0x70cdca9a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24edca9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34dcaa3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12adcaa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66dcaac, SUC_lexicon_tags_17 },
    { 0x650dcaac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x13edcaaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96dcab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7adcab5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0dcab8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174dcac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59adcac5, SUC_lexicon_tags_21 },
    { 0x15cdcac5, SUC_lexicon_tags_25 },
    { 0x120dcac5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8edcaca, SUC_lexicon_tags_37 },
    { 0x444dcacb, SUC_lexicon_tags_19 },
    { 0xe8cdcaca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276dcad1, SUC_lexicon_tags_46 },
    { 0xd88dcad2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x25adcad4, SUC_lexicon_tags_60 },
    { 0x8f8dcad5, SUC_lexicon_tags_37 },
    { 0xc3cdcad6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea4dcad8, SUC_lexicon_tags_127 },
    { 0xfd0dcad9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970dcadd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8dcae0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aedcae4, SUC_lexicon_tags_25 },
    { 0x822dcae5, SUC_lexicon_tags_82 },
    { 0x61edcae5, SUC_lexicon_tags_37 },
    { 0x370dcae7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x500dcae9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74dcaec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x38adcaee, SUC_lexicon_tags_39 },
    { 0x9aedcaef, SUC_lexicon_tags_26 },
    { 0xa26dcaef, SUC_lexicon_tags_24 },
    { 0x7eadcaf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42adcaf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4dcaff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x622dcb02, SUC_lexicon_tags_38 },
    { 0x69adcb03, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23edcb0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa74dcb0f, SUC_lexicon_tags_7 },
    { 0x7c6dcb10, SUC_lexicon_tags_47 },
    { 0xcc0dcb11, SUC_lexicon_tags_2 },
    { 0x590dcb0f, SUC_lexicon_tags_17 },
    { 0xf5edcb13, SUC_lexicon_tags_47 },
    { 0xbfadcb14, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5cdcb16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa46dcb18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf08dcb1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56dcb1e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84dcb22, SUC_lexicon_tags_45 },
    { 0x9b2dcb23, SUC_lexicon_tags_1 },
    { 0xbe2dcb23, SUC_lexicon_tags_38 },
    { 0x720dcb25, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x54cdcb27, SUC_lexicon_tags_17 },
    { 0x200dcb28, SUC_lexicon_tags_32 },
    { 0xf44dcb29, SUC_lexicon_tags_14 },
    { 0x34dcb29, SUC_lexicon_tags_216 },
    { 0xa0cdcb2b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3edcb2e, SUC_lexicon_tags_29 },
    { 0xa26dcb2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cdcb36, SUC_lexicon_tags_8 },
    { 0x6aadcb37, SUC_lexicon_tags_33 },
    { 0xd42dcb37, SUC_lexicon_tags_16 },
    { 0xe74dcb36, SUC_lexicon_tags_60 },
    { 0x23cdcb36, SUC_lexicon_tags_15 },
    { 0x750dcb3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8dcb40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x74cdcb43, SUC_lexicon_tags_12 },
    { 0xaf6dcb44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0dcb47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x612dcb4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54dcb4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8edcb51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4dcb56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0adcb5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xab2dcb5e, SUC_lexicon_tags_2 },
    { 0xa64dcb5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1bcdcb61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2dcb68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaadcb6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x608dcb6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa64dcb71, SUC_lexicon_tags_11 },
    { 0x6fedcb72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcdcb75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1cdcb77, SUC_lexicon_tags_37 },
    { 0xab4dcb78, SUC_lexicon_tags_1 },
    { 0x8e0dcb79, SUC_lexicon_tags_23 },
    { 0x492dcb7a, SUC_lexicon_tags_37 },
    { 0xe44dcb7b, SUC_lexicon_tags_16 },
    { 0x486dcb7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dcb7f, SUC_lexicon_tags_77 },
    { 0x670dcb80, SUC_lexicon_tags_8 },
    { 0x470dcb81, SUC_lexicon_tags_8 },
    { 0x1acdcb81, SUC_lexicon_tags_7 },
    { 0x35adcb82, SUC_lexicon_tags_14 },
    { 0x554dcb84, SUC_lexicon_tags_37 },
    { 0x662dcb85, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72dcb8a, SUC_lexicon_tags_63 },
    { 0x280dcb8b, SUC_lexicon_tags_12 },
    { 0xa30dcb8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778dcb94, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb52dcb96, SUC_lexicon_tags_11 },
    { 0xe62dcb97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808dcba5, SUC_lexicon_tags_37 },
    { 0xa10dcba5, SUC_lexicon_tags_25 },
    { 0xd18dcba7, SUC_lexicon_tags_66 },
    { 0x95adcba8, SUC_lexicon_tags_17 },
    { 0xa04dcba9, SUC_lexicon_tags_17 },
    { 0x660dcba8, SUC_lexicon_tags_27 },
    { 0xc70dcba8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18dcbae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eadcbb4, SUC_lexicon_tags_33 },
    { 0xb18dcbb5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998dcbba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2cedcbbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0dcbbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32dcbc4, SUC_lexicon_tags_38 },
    { 0xbe8dcbc5, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x832dcbc7, SUC_lexicon_tags_17 },
    { 0x5f2dcbc8, SUC_lexicon_tags_25 },
    { 0x58edcbc7, SUC_lexicon_tags_45 },
    { 0x942dcbca, SUC_lexicon_tags_17 },
    { 0x552dcbc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf7edcbcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcedcbcf, SUC_lexicon_tags_49 },
    { 0x918dcbd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x762dcbd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x410dcbd5, SUC_lexicon_tags_17 },
    { 0x830dcbd6, SUC_lexicon_tags_15 },
    { 0xd58dcbd5, SUC_lexicon_tags_12 },
    { 0x910dcbd8, SUC_lexicon_tags_9 },
    { 0x7c4dcbd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x756dcbdb, SUC_lexicon_tags_43 },
    { 0x446dcbdc, SUC_lexicon_tags_37 },
    { 0x83adcbdc, SUC_lexicon_tags_30 },
    { 0x72dcbdd, SUC_lexicon_tags_11 },
    { 0x3eadcbdf, SUC_lexicon_tags_1 },
    { 0x7edcbdf, SUC_lexicon_tags_1 },
    { 0x3e0dcbe1, SUC_lexicon_tags_11 },
    { 0x1aadcbe2, SUC_lexicon_tags_19 },
    { 0x2ecdcbe3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0dcbe8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29adcbea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112dcbee, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x1c6dcbf0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x666dcbf3, SUC_lexicon_tags_61 },
    { 0xbcdcbf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46cdcbf7, SUC_lexicon_tags_25 },
    { 0x65edcbf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf6dcbfa, SUC_lexicon_tags_11 },
    { 0x45edcbfa, SUC_lexicon_tags_12 },
    { 0x14cdcbfc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x754dcbfe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cdcc02, SUC_lexicon_tags_37 },
    { 0xc90dcc03, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dcc06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dcdcc08, SUC_lexicon_tags_45 },
    { 0xc8dcc09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dcc0c, SUC_lexicon_tags_45 },
    { 0x624dcc0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8dcc10, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4dcc14, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x652dcc17, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x424dcc19, SUC_lexicon_tags_3 },
    { 0xc7cdcc1a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2a6dcc1c, SUC_lexicon_tags_15 },
    { 0xc24dcc1d, SUC_lexicon_tags_55 },
    { 0x172dcc1d, SUC_lexicon_tags_25 },
    { 0x690dcc1f, SUC_lexicon_tags_17 },
    { 0xfc6dcc20, SUC_lexicon_tags_39 },
    { 0xaa6dcc21, SUC_lexicon_tags_45 },
    { 0xe24dcc1f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20dcc25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x576dcc28, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x606dcc2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e4dcc2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114dcc30, SUC_lexicon_tags_17 },
    { 0x4fedcc31, SUC_lexicon_tags_2 },
    { 0xd60dcc32, SUC_lexicon_tags_49 },
    { 0x542dcc33, SUC_lexicon_tags_27 },
    { 0x338dcc33, SUC_lexicon_tags_57 },
    { 0xf62dcc35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50dcc46, SUC_lexicon_tags_37 },
    { 0xee2dcc47, SUC_lexicon_tags_7 },
    { 0x16edcc48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadcdcc4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8dcc52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceadcc58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x852dcc5b, SUC_lexicon_tags_37 },
    { 0x9c4dcc5b, SUC_lexicon_tags_1 },
    { 0xc26dcc5b, SUC_lexicon_tags_194 },
    { 0x71edcc5e, SUC_lexicon_tags_19 },
    { 0x24cdcc5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9edcc61, SUC_lexicon_tags_63 },
    { 0x7f8dcc62, SUC_lexicon_tags_0 },
    { 0xe28dcc62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462dcc6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ccdcc6e, SUC_lexicon_tags_38 },
    { 0xd7adcc6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44dcc7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cedcc7e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8dcc84, SUC_lexicon_tags_1 },
    { 0xe92dcc85, SUC_lexicon_tags_21 },
    { 0x908dcc84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x83adcc88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12dcc8b, SUC_lexicon_tags_17 },
    { 0x1f2dcc8c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1f0dcc8e, SUC_lexicon_tags_101 },
    { 0x892dcc8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6dcc94, SUC_lexicon_tags_17 },
    { 0x47cdcc95, SUC_lexicon_tags_7 },
    { 0xc6dcc96, SUC_lexicon_tags_46 },
    { 0xcaadcc97, SUC_lexicon_tags_64 },
    { 0x62edcc97, SUC_lexicon_tags_12 },
    { 0x64edcc99, SUC_lexicon_tags_65 },
    { 0x2bcdcc99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63edcca4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x850dcca6, SUC_lexicon_tags_37 },
    { 0xc90dcca7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770dccad, SUC_lexicon_tags_11 },
    { 0x2b0dccae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9f2dccb0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xaf0dccb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cdccb6, SUC_lexicon_tags_12 },
    { 0xb02dccb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x468dccb9, SUC_lexicon_tags_10 },
    { 0x2f4dccb9, SUC_lexicon_tags_57 },
    { 0x268dccb9, SUC_lexicon_tags_25 },
    { 0x432dccbc, SUC_lexicon_tags_37 },
    { 0x84edccbc, SUC_lexicon_tags_47 },
    { 0x1e2dccbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890dccc2, SUC_lexicon_tags_27 },
    { 0x72adccc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x282dccc6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2dcccc, SUC_lexicon_tags_1 },
    { 0x98dcccd, SUC_lexicon_tags_47 },
    { 0x924dcccc, SUC_lexicon_tags_11 },
    { 0x8e2dccce, SUC_lexicon_tags_1 },
    { 0x736dcccd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28dccdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc12dccde, SUC_lexicon_tags_11 },
    { 0x84adccdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618dcce4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8dcce7, SUC_lexicon_tags_1 },
    { 0xad2dcce8, SUC_lexicon_tags_60 },
    { 0x682dcce8, SUC_lexicon_tags_33 },
    { 0x1f4dcce8, SUC_lexicon_tags_30 },
    { 0x150dccea, SUC_lexicon_tags_26 },
    { 0x266dccec, SUC_lexicon_tags_63 },
    { 0x2f8dcceb, SUC_lexicon_tags_45 },
    { 0x824dccee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8dccf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cdccf3, SUC_lexicon_tags_19 },
    { 0x4e2dccf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8dccfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4dcd00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706dcd04, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x146dcd06, SUC_lexicon_tags_37 },
    { 0x876dcd06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2dcd0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8edcd0f, SUC_lexicon_tags_1 },
    { 0xca0dcd10, SUC_lexicon_tags_37 },
    { 0x9fadcd10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xceedcd13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7adcd15, SUC_lexicon_tags_191 },
    { 0x100dcd16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66dcd19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc66dcd1b, SUC_lexicon_tags_38 },
    { 0xeb6dcd1c, SUC_lexicon_tags_38 },
    { 0xb5adcd1c, SUC_lexicon_tags_141 },
    { 0xa72dcd1e, SUC_lexicon_tags_11 },
    { 0x4cedcd1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x708dcd21, SUC_lexicon_tags_65 },
    { 0xc38dcd22, SUC_lexicon_tags_7 },
    { 0x360dcd22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4dcd27, SUC_lexicon_tags_15 },
    { 0x3dadcd27, SUC_lexicon_tags_87 },
    { 0xf86dcd27, SUC_lexicon_tags_28 },
    { 0x582dcd2a, SUC_lexicon_tags_38 },
    { 0x88edcd29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34dcd35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434dcd3a, SUC_lexicon_tags_15 },
    { 0xb20dcd3b, SUC_lexicon_tags_11 },
    { 0x5fcdcd3c, SUC_lexicon_tags_12 },
    { 0xd2dcd3b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x82dcd3f, SUC_lexicon_tags_45 },
    { 0xc4adcd40, SUC_lexicon_tags_37 },
    { 0x84adcd40, SUC_lexicon_tags_44 },
    { 0xf86dcd3f, SUC_lexicon_tags_1 },
    { 0xb94dcd42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90edcd46, SUC_lexicon_tags_47 },
    { 0xbe4dcd47, SUC_lexicon_tags_267 },
    { 0x54dcd48, SUC_lexicon_tags_17 },
    { 0xcb6dcd49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2dcd4e, SUC_lexicon_tags_68 },
    { 0x714dcd4f, SUC_lexicon_tags_29 },
    { 0xee8dcd50, SUC_lexicon_tags_38 },
    { 0x908dcd51, SUC_lexicon_tags_4 },
    { 0x13edcd51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x428dcd55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cdcd5b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22dcd60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74dcd65, SUC_lexicon_tags_1 },
    { 0x788dcd66, SUC_lexicon_tags_19 },
    { 0x9eedcd66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5adcd6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe8dcd70, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3adcd73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364dcd77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdedcd7a, SUC_lexicon_tags_37 },
    { 0xa80dcd7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72dcd85, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaedcd88, SUC_lexicon_tags_19 },
    { 0x19edcd88, SUC_lexicon_tags_2 },
    { 0x944dcd89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cdcd8e, SUC_lexicon_tags_2 },
    { 0xcd8dcd8e, SUC_lexicon_tags_17 },
    { 0xf12dcd90, SUC_lexicon_tags_48 },
    { 0x7bedcd8e, SUC_lexicon_tags_3 },
    { 0x4aedcd92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42adcd95, SUC_lexicon_tags_11 },
    { 0xc00dcd96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0dcd9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5badcd9f, SUC_lexicon_tags_2 },
    { 0x53edcda0, SUC_lexicon_tags_77 },
    { 0x20dcda1, SUC_lexicon_tags_4 },
    { 0x18edcda2, SUC_lexicon_tags_11 },
    { 0xaa4dcd9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0dcda8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8dcdac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cdcdb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b0dcdb5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xefcdcdb7, SUC_lexicon_tags_12 },
    { 0xdaadcdb8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2dcdbb, SUC_lexicon_tags_84 },
    { 0xdb2dcdbc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf96dcdbe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15adcdc2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdcdc5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4dcdcb, SUC_lexicon_tags_37 },
    { 0x1fedcdcc, SUC_lexicon_tags_28 },
    { 0x8cdcdcd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacdcdd1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcdcdda, SUC_lexicon_tags_23 },
    { 0xd8edcddb, SUC_lexicon_tags_45 },
    { 0xd2adcddc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4dcde2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fadcdec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792dcdf1, SUC_lexicon_tags_87 },
    { 0xc94dcdf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45cdcdf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8edcdfb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554dcdff, SUC_lexicon_tags_12 },
    { 0x74dce00, SUC_lexicon_tags_23 },
    { 0x914dce00, SUC_lexicon_tags_17 },
    { 0x4d2dce00, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0xc7adce04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x61cdce07, SUC_lexicon_tags_7 },
    { 0x9ccdce08, SUC_lexicon_tags_37 },
    { 0xf20dce08, SUC_lexicon_tags_25 },
    { 0xeacdce0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dce0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47edce10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x188dce13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2dce16, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x500dce18, SUC_lexicon_tags_66 },
    { 0xdc0dce19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fadce1c, SUC_lexicon_tags_60 },
    { 0x91cdce1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830dce22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8dce27, SUC_lexicon_tags_25 },
    { 0x298dce28, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90cdce2a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3b8dce2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x710dce2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x882dce31, SUC_lexicon_tags_14 },
    { 0x54cdce32, SUC_lexicon_tags_31 },
    { 0x642dce33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60dce35, SUC_lexicon_tags_37 },
    { 0xa6edce36, SUC_lexicon_tags_32 },
    { 0xccedce37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20edce3d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6edce42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64dce4a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x202dce4d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470dce55, SUC_lexicon_tags_57 },
    { 0x6aadce56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dce5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x450dce5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dce60, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x164dce62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b4dce64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebedce6d, SUC_lexicon_tags_25 },
    { 0x26dce6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cedce71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadce78, SUC_lexicon_tags_37 },
    { 0xa38dce78, SUC_lexicon_tags_12 },
    { 0x3fedce78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1edce7f, SUC_lexicon_tags_9 },
    { 0x30dce80, SUC_lexicon_tags_66 },
    { 0x9f8dce80, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x51cdce83, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4dce8a, SUC_lexicon_tags_60 },
    { 0xe30dce8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x314dce8d, SUC_lexicon_tags_9 },
    { 0x6b6dce8e, SUC_lexicon_tags_8 },
    { 0x26dce8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10adce95, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x97edce97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x678dce99, SUC_lexicon_tags_39 },
    { 0x9f8dce9a, SUC_lexicon_tags_17 },
    { 0x20edce9a, SUC_lexicon_tags_11 },
    { 0xaccdce9a, SUC_lexicon_tags_34 },
    { 0xafadce9d, SUC_lexicon_tags_11 },
    { 0x2aedce9d, SUC_lexicon_tags_25 },
    { 0xbbadce9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830dcea3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe48dcea5, SUC_lexicon_tags_29 },
    { 0x3ccdcea6, SUC_lexicon_tags_14 },
    { 0xffedcea6, SUC_lexicon_tags_2 },
    { 0xfa2dcea8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3fadceaa, SUC_lexicon_tags_1 },
    { 0xf38dceab, SUC_lexicon_tags_14 },
    { 0x482dceaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27edceae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38dceb7, SUC_lexicon_tags_15 },
    { 0x4b4dceb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe06dceba, SUC_lexicon_tags_82 },
    { 0xedadceba, SUC_lexicon_tags_15 },
    { 0x1a4dcebc, SUC_lexicon_tags_17 },
    { 0x768dcebd, SUC_lexicon_tags_12 },
    { 0xd28dcebe, SUC_lexicon_tags_4 },
    { 0xef0dceba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286dcec8, SUC_lexicon_tags_13 },
    { 0x6adcec8, SUC_lexicon_tags_15 },
    { 0xe4cdceca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x804dcecd, SUC_lexicon_tags_2 },
    { 0x1bcdcecd, SUC_lexicon_tags_2 },
    { 0xe34dcecf, SUC_lexicon_tags_114 },
    { 0xfd2dcecf, SUC_lexicon_tags_2 },
    { 0xfc8dced1, SUC_lexicon_tags_27 },
    { 0xd48dcecf, SUC_lexicon_tags_14 },
    { 0x64dced0, SUC_lexicon_tags_12 },
    { 0x456dced1, SUC_lexicon_tags_33 },
    { 0xa98dced0, SUC_lexicon_tags_17 },
    { 0x9e6dced6, SUC_lexicon_tags_61 },
    { 0x7dcdced6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670dcedb, SUC_lexicon_tags_12 },
    { 0x90dcedc, SUC_lexicon_tags_118 },
    { 0x562dcedd, SUC_lexicon_tags_17 },
    { 0x358dcedd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3edcee3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x564dcee6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x77edcee8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bedceed, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860dcef1, SUC_lexicon_tags_30 },
    { 0x970dcef2, SUC_lexicon_tags_11 },
    { 0x184dcef3, SUC_lexicon_tags_101 },
    { 0x976dcef3, SUC_lexicon_tags_38 },
    { 0x5b6dcef2, SUC_lexicon_tags_1 },
    { 0xcaedcef6, SUC_lexicon_tags_63 },
    { 0x102dcef7, SUC_lexicon_tags_25 },
    { 0xedcef8, SUC_lexicon_tags_12 },
    { 0xe2adcef9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdedcefc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeeedcefe, SUC_lexicon_tags_13 },
    { 0x54cdceff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14dcf04, SUC_lexicon_tags_15 },
    { 0xa30dcf05, SUC_lexicon_tags_17 },
    { 0x832dcf06, SUC_lexicon_tags_65 },
    { 0xf8dcf06, SUC_lexicon_tags_14 },
    { 0x244dcf06, SUC_lexicon_tags_47 },
    { 0x4b6dcf08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5d6dcf0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdbcdcf0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38dcf10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2dcf13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec6dcf15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x406dcf18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x81edcf1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cdcf23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda8dcf25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fadcf27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdecdcf29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1adcf2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc4dcf2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4dcf30, SUC_lexicon_tags_69 },
    { 0xf40dcf31, SUC_lexicon_tags_113 },
    { 0x10edcf32, SUC_lexicon_tags_17 },
    { 0x4f2dcf32, SUC_lexicon_tags_11 },
    { 0x20edcf34, SUC_lexicon_tags_0 },
    { 0x3dadcf35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cedcf40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x740dcf43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfaadcf45, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cdcf4a, SUC_lexicon_tags_179 },
    { 0x2b6dcf4a, SUC_lexicon_tags_11 },
    { 0xb02dcf4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcdcf52, SUC_lexicon_tags_44 },
    { 0x8adcf53, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccdcf58, SUC_lexicon_tags_12 },
    { 0x43cdcf58, SUC_lexicon_tags_11 },
    { 0x968dcf58, SUC_lexicon_tags_14 },
    { 0xd24dcf59, SUC_lexicon_tags_7 },
    { 0x8eedcf5c, SUC_lexicon_tags_49 },
    { 0xddadcf5c, SUC_lexicon_tags_47 },
    { 0xe7edcf5c, SUC_lexicon_tags_11 },
    { 0x758dcf5b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dedcf63, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa2adcf65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x98cdcf67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830dcf6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x634dcf6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67adcf75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x382dcf77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46edcf7c, SUC_lexicon_tags_7 },
    { 0x58dcf7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad0dcf7f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cdcf83, SUC_lexicon_tags_19 },
    { 0x31cdcf84, SUC_lexicon_tags_37 },
    { 0xac2dcf85, SUC_lexicon_tags_37 },
    { 0x57edcf86, SUC_lexicon_tags_12 },
    { 0xbe4dcf87, SUC_lexicon_tags_84 },
    { 0x812dcf88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x568dcf8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b8dcf8d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x552dcf90, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6dcf94, SUC_lexicon_tags_15 },
    { 0xc6edcf95, SUC_lexicon_tags_2 },
    { 0x62edcf94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cdcf9d, SUC_lexicon_tags_39 },
    { 0xfa4dcf9d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0dcfa6, SUC_lexicon_tags_18 },
    { 0xf86dcfa6, SUC_lexicon_tags_17 },
    { 0x39cdcfa8, SUC_lexicon_tags_15 },
    { 0xc04dcfa9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x256dcfac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22dcfaf, SUC_lexicon_tags_49 },
    { 0x642dcfb0, SUC_lexicon_tags_14 },
    { 0x6d0dcfb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2adcfb3, SUC_lexicon_tags_12 },
    { 0x3a8dcfb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x958dcfb6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x23adcfb9, SUC_lexicon_tags_12 },
    { 0x4b8dcfba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x274dcfbd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x90cdcfbf, SUC_lexicon_tags_0 },
    { 0x85edcfc0, SUC_lexicon_tags_79 },
    { 0x6a0dcfbf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98dcfc5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dcfca, SUC_lexicon_tags_25 },
    { 0x31adcfcb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9edcfce, SUC_lexicon_tags_25 },
    { 0xfaadcfcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x526dcfd1, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2dcfd4, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x62edcfd6, SUC_lexicon_tags_12 },
    { 0x67edcfd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe00dcfd9, SUC_lexicon_tags_19 },
    { 0x4e2dcfd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x234dcfdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcdcfdf, SUC_lexicon_tags_1 },
    { 0xa48dcfe0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796dcfe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe2dcfe6, SUC_lexicon_tags_37 },
    { 0x890dcfe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34dcfeb, SUC_lexicon_tags_17 },
    { 0xf5cdcfeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdadcfef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160dcff4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa10dcff6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46dcffb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838dd000, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996dd004, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x930dd006, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1eedd008, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x996dd00b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x520dd00d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc24dd00f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x192dd012, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x20edd015, SUC_lexicon_tags_12 },
    { 0xadcdd016, SUC_lexicon_tags_84 },
    { 0x8d6dd017, SUC_lexicon_tags_25 },
    { 0x2ecdd018, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cdd01c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcdd01f, SUC_lexicon_tags_11 },
    { 0xb2cdd020, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0dd027, SUC_lexicon_tags_17 },
    { 0x688dd028, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecdd02c, SUC_lexicon_tags_55 },
    { 0x80add02c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6dd032, SUC_lexicon_tags_2 },
    { 0xe46dd032, SUC_lexicon_tags_19 },
    { 0x916dd034, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdd037, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55add039, SUC_lexicon_tags_17 },
    { 0xb50dd03a, SUC_lexicon_tags_1 },
    { 0x37edd03b, SUC_lexicon_tags_25 },
    { 0x140dd03a, SUC_lexicon_tags_37 },
    { 0x9b0dd03d, SUC_lexicon_tags_37 },
    { 0xf10dd03e, SUC_lexicon_tags_33 },
    { 0xe02dd03d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0dd044, SUC_lexicon_tags_43 },
    { 0x54edd045, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520dd04c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd2edd04e, SUC_lexicon_tags_25 },
    { 0xed2dd04e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30dd055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcdd058, SUC_lexicon_tags_2 },
    { 0xb74dd059, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd68dd05b, SUC_lexicon_tags_138 },
    { 0x778dd05b, SUC_lexicon_tags_0 },
    { 0x61cdd05d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42dd060, SUC_lexicon_tags_57 },
    { 0x2badd060, SUC_lexicon_tags_29 },
    { 0xa06dd062, SUC_lexicon_tags_0 },
    { 0x652dd063, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cdd067, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d8dd069, SUC_lexicon_tags_0 },
    { 0x408dd069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2dd06e, SUC_lexicon_tags_38 },
    { 0x44cdd06f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74dd073, SUC_lexicon_tags_15 },
    { 0xdcedd074, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cdd079, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cdd080, SUC_lexicon_tags_37 },
    { 0xa48dd081, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84dd084, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafedd08d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb54dd08f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e2dd091, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cedd094, SUC_lexicon_tags_6 },
    { 0xd26dd095, SUC_lexicon_tags_12 },
    { 0x2aedd096, SUC_lexicon_tags_87 },
    { 0xdc6dd097, SUC_lexicon_tags_29 },
    { 0xdaadd097, SUC_lexicon_tags_63 },
    { 0x19add097, SUC_lexicon_tags_11 },
    { 0x7cedd09a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd8dd09c, SUC_lexicon_tags_23 },
    { 0xbbadd09d, SUC_lexicon_tags_31 },
    { 0xd4add09e, SUC_lexicon_tags_23 },
    { 0x3a4dd09f, SUC_lexicon_tags_7 },
    { 0x11cdd0a0, SUC_lexicon_tags_12 },
    { 0xc64dd0a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6dd0a6, SUC_lexicon_tags_12 },
    { 0x9f6dd0a7, SUC_lexicon_tags_13 },
    { 0xb8dd0a8, SUC_lexicon_tags_37 },
    { 0xc32dd0a8, SUC_lexicon_tags_60 },
    { 0x7fadd0aa, SUC_lexicon_tags_47 },
    { 0xd00dd0ab, SUC_lexicon_tags_7 },
    { 0x360dd0a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5f0dd0ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x790dd0b0, SUC_lexicon_tags_33 },
    { 0xd3edd0b1, SUC_lexicon_tags_2 },
    { 0x84add0b2, SUC_lexicon_tags_20 },
    { 0x5badd0b1, SUC_lexicon_tags_60 },
    { 0x5dedd0b3, SUC_lexicon_tags_26 },
    { 0x614dd0b0, SUC_lexicon_tags_12 },
    { 0xf56dd0b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2dd0be, SUC_lexicon_tags_17 },
    { 0xcd0dd0bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcdd0c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacedd0c9, SUC_lexicon_tags_12 },
    { 0xc7cdd0ca, SUC_lexicon_tags_2 },
    { 0x6d8dd0ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x978dd0cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x106dd0cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6dd0d1, SUC_lexicon_tags_12 },
    { 0x7dedd0d2, SUC_lexicon_tags_15 },
    { 0x884dd0d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4dd0da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4dd0dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0edd0df, SUC_lexicon_tags_23 },
    { 0xffcdd0e0, SUC_lexicon_tags_27 },
    { 0xb1add0e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf6dd0e3, SUC_lexicon_tags_28 },
    { 0xe88dd0e3, SUC_lexicon_tags_2 },
    { 0xdecdd0e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x830dd0e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x632dd0e9, SUC_lexicon_tags_26 },
    { 0x134dd0e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x820dd0ed, SUC_lexicon_tags_60 },
    { 0xc4add0ee, SUC_lexicon_tags_37 },
    { 0xddedd0ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22dd0f2, SUC_lexicon_tags_45 },
    { 0x16dd0f3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edd0f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x40edd0fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x340dd0fe, SUC_lexicon_tags_28 },
    { 0x518dd0ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65add105, SUC_lexicon_tags_49 },
    { 0x414dd106, SUC_lexicon_tags_11 },
    { 0x186dd107, SUC_lexicon_tags_11 },
    { 0x3edd108, SUC_lexicon_tags_92 },
    { 0xc8edd109, SUC_lexicon_tags_26 },
    { 0x5a4dd10a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8dd10f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70dd112, SUC_lexicon_tags_4 },
    { 0x8a6dd112, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x14edd115, SUC_lexicon_tags_17 },
    { 0xc0add115, SUC_lexicon_tags_37 },
    { 0x45add116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41edd11b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30edd11d, SUC_lexicon_tags_7 },
    { 0x392dd11d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cdd121, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2dd124, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dadd128, SUC_lexicon_tags_2 },
    { 0xd16dd128, SUC_lexicon_tags_17 },
    { 0x248dd128, SUC_lexicon_tags_37 },
    { 0xd38dd128, SUC_lexicon_tags_37 },
    { 0x330dd12c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dd12f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe10dd131, SUC_lexicon_tags_7 },
    { 0x462dd132, SUC_lexicon_tags_37 },
    { 0x434dd133, SUC_lexicon_tags_37 },
    { 0xd86dd131, SUC_lexicon_tags_75 },
    { 0x21add134, SUC_lexicon_tags_2 },
    { 0x7a6dd135, SUC_lexicon_tags_29 },
    { 0x920dd134, SUC_lexicon_tags_25 },
    { 0x222dd136, SUC_lexicon_tags_14 },
    { 0x4ecdd139, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398dd13e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cdd145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd14f, SUC_lexicon_tags_61 },
    { 0x68edd150, SUC_lexicon_tags_1 },
    { 0x8a8dd150, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xddcdd154, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeedd158, SUC_lexicon_tags_65 },
    { 0x6edd159, SUC_lexicon_tags_33 },
    { 0x8b8dd15a, SUC_lexicon_tags_27 },
    { 0x6eadd158, SUC_lexicon_tags_19 },
    { 0xd02dd158, SUC_lexicon_tags_11 },
    { 0x38add15d, SUC_lexicon_tags_21 },
    { 0xf6add15d, SUC_lexicon_tags_6 },
    { 0x32cdd15f, SUC_lexicon_tags_68 },
    { 0x526dd160, SUC_lexicon_tags_30 },
    { 0xaeadd161, SUC_lexicon_tags_38 },
    { 0x442dd15a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214dd166, SUC_lexicon_tags_23 },
    { 0x9c8dd167, SUC_lexicon_tags_4 },
    { 0xbb2dd166, SUC_lexicon_tags_15 },
    { 0x910dd169, SUC_lexicon_tags_37 },
    { 0x1d2dd168, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11edd16d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x112dd16f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12dd174, SUC_lexicon_tags_94 },
    { 0x750dd175, SUC_lexicon_tags_27 },
    { 0x2f2dd174, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d8dd178, SUC_lexicon_tags_15 },
    { 0xc6add179, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadd17d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8edd180, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4dd183, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940dd18c, SUC_lexicon_tags_47 },
    { 0x3a4dd18c, SUC_lexicon_tags_17 },
    { 0xfbedd18c, SUC_lexicon_tags_32 },
    { 0x16add18f, SUC_lexicon_tags_49 },
    { 0x744dd190, SUC_lexicon_tags_19 },
    { 0x4f6dd191, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc4dd193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27add195, SUC_lexicon_tags_15 },
    { 0x65cdd196, SUC_lexicon_tags_28 },
    { 0xa76dd197, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x80add19a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260dd19f, SUC_lexicon_tags_179 },
    { 0x64dd1a0, SUC_lexicon_tags_43 },
    { 0xf1add1a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98dd1ab, SUC_lexicon_tags_28 },
    { 0x1fcdd1ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8dd1b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x696dd1b3, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0dd1b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0dd1c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0edd1c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56edd1c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dd1cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150dd1cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x226dd1d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf76dd1d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cdd1d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb44dd1d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x342dd1dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76add1df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7cdd1e1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06dd1ea, SUC_lexicon_tags_9 },
    { 0x25edd1eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aadd1ed, SUC_lexicon_tags_25 },
    { 0xff8dd1ed, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6dd1f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde8dd1f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb0dd1f5, SUC_lexicon_tags_19 },
    { 0x9c4dd1f6, SUC_lexicon_tags_37 },
    { 0xa0dd1f7, SUC_lexicon_tags_63 },
    { 0xc54dd1f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa40dd1fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x47cdd1fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2edd202, SUC_lexicon_tags_45 },
    { 0x7c8dd203, SUC_lexicon_tags_11 },
    { 0x3c4dd204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0edd207, SUC_lexicon_tags_63 },
    { 0x8dcdd208, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2dd20c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788dd210, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8dd216, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaf0dd218, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792dd21c, SUC_lexicon_tags_29 },
    { 0x33edd21d, SUC_lexicon_tags_14 },
    { 0xd60dd21e, SUC_lexicon_tags_15 },
    { 0x4eadd21f, SUC_lexicon_tags_32 },
    { 0x48edd21e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eedd226, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x658dd229, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250dd22e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xacadd230, SUC_lexicon_tags_27 },
    { 0x2d2dd230, SUC_lexicon_tags_63 },
    { 0x25add232, SUC_lexicon_tags_17 },
    { 0x33cdd232, SUC_lexicon_tags_7 },
    { 0x660dd234, SUC_lexicon_tags_10 },
    { 0x33edd234, SUC_lexicon_tags_2 },
    { 0x4f8dd234, SUC_lexicon_tags_12 },
    { 0x17add237, SUC_lexicon_tags_47 },
    { 0x6e6dd233, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xc8cdd23a, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x376dd23d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd3edd23f, SUC_lexicon_tags_17 },
    { 0x170dd240, SUC_lexicon_tags_40 },
    { 0x324dd240, SUC_lexicon_tags_43 },
    { 0xed8dd241, SUC_lexicon_tags_62 },
    { 0x39cdd243, SUC_lexicon_tags_14 },
    { 0xb2dd241, SUC_lexicon_tags_7 },
    { 0xe10dd245, SUC_lexicon_tags_37 },
    { 0x7dcdd246, SUC_lexicon_tags_12 },
    { 0xbb6dd243, SUC_lexicon_tags_1 },
    { 0x3eadd248, SUC_lexicon_tags_84 },
    { 0xefcdd23f, SUC_lexicon_tags_27 },
    { 0x93add241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe58dd24c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8dd251, SUC_lexicon_tags_1 },
    { 0xf50dd252, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dd25c, SUC_lexicon_tags_25 },
    { 0x46dd25d, SUC_lexicon_tags_84 },
    { 0x494dd25e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x796dd260, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dd264, SUC_lexicon_tags_7 },
    { 0xcc6dd265, SUC_lexicon_tags_15 },
    { 0x710dd266, SUC_lexicon_tags_2 },
    { 0x196dd267, SUC_lexicon_tags_2 },
    { 0x164dd266, SUC_lexicon_tags_47 },
    { 0xff0dd269, SUC_lexicon_tags_15 },
    { 0x86dd26a, SUC_lexicon_tags_11 },
    { 0x958dd268, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdedd26d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef4dd26f, SUC_lexicon_tags_29 },
    { 0x2f0dd26f, SUC_lexicon_tags_26 },
    { 0xdf2dd270, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6add273, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xacedd276, SUC_lexicon_tags_0 },
    { 0x2a8dd277, SUC_lexicon_tags_8 },
    { 0x582dd276, SUC_lexicon_tags_25 },
    { 0x8a6dd279, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412dd27d, SUC_lexicon_tags_7 },
    { 0x544dd27d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x722dd281, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe64dd283, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x710dd285, SUC_lexicon_tags_37 },
    { 0x3e2dd286, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636dd28b, SUC_lexicon_tags_39 },
    { 0xa68dd28b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x680dd28f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1add292, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x284dd295, SUC_lexicon_tags_79 },
    { 0x322dd296, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4dd299, SUC_lexicon_tags_26 },
    { 0xe08dd299, SUC_lexicon_tags_6 },
    { 0xab8dd29a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382dd29f, SUC_lexicon_tags_8 },
    { 0xe34dd2a0, SUC_lexicon_tags_2 },
    { 0xf10dd2a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2dd2a4, SUC_lexicon_tags_17 },
    { 0x14edd2a5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccdd2a8, SUC_lexicon_tags_11 },
    { 0x7edd2a8, SUC_lexicon_tags_63 },
    { 0xd36dd2aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2dd2ad, SUC_lexicon_tags_84 },
    { 0x558dd2ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadd2b6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9edd2b9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cdd2bd, SUC_lexicon_tags_17 },
    { 0x750dd2be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cdd2c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12dd2cb, SUC_lexicon_tags_45 },
    { 0x2c2dd2cc, SUC_lexicon_tags_9 },
    { 0x1cadd2cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2dd2d0, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29edd2d4, SUC_lexicon_tags_70 },
    { 0x894dd2d5, SUC_lexicon_tags_21 },
    { 0xf10dd2d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2dd2db, SUC_lexicon_tags_11 },
    { 0x7f8dd2dc, SUC_lexicon_tags_25 },
    { 0xaecdd2dc, SUC_lexicon_tags_11 },
    { 0xc3add2dd, SUC_lexicon_tags_25 },
    { 0x306dd2df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f6dd2e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cdd2e6, SUC_lexicon_tags_45 },
    { 0xef8dd2e7, SUC_lexicon_tags_2 },
    { 0x7cdd2e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff0dd2ea, SUC_lexicon_tags_37 },
    { 0xf2add2ea, SUC_lexicon_tags_3 },
    { 0x1d2dd2ec, SUC_lexicon_tags_4 },
    { 0xdf2dd2ec, SUC_lexicon_tags_2 },
    { 0xa78dd2ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x290dd2f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9edd2f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186dd2f7, SUC_lexicon_tags_23 },
    { 0x5bedd2f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8dd2fb, SUC_lexicon_tags_14 },
    { 0x4bedd2fc, SUC_lexicon_tags_4 },
    { 0xe10dd2fd, SUC_lexicon_tags_15 },
    { 0x11cdd2fe, SUC_lexicon_tags_23 },
    { 0xa8edd2fb, SUC_lexicon_tags_17 },
    { 0x9d8dd300, SUC_lexicon_tags_13 },
    { 0x4c8dd2fd, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0x23edd304, SUC_lexicon_tags_49 },
    { 0x7ecdd304, SUC_lexicon_tags_7 },
    { 0xb8add304, SUC_lexicon_tags_43 },
    { 0xb8edd304, SUC_lexicon_tags_23 },
    { 0xb80dd305, SUC_lexicon_tags_44 },
    { 0x136dd309, SUC_lexicon_tags_1 },
    { 0x968dd305, SUC_lexicon_tags_10 },
    { 0xfcedd309, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86dd30e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8dd310, SUC_lexicon_tags_49 },
    { 0xcd2dd310, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda4dd313, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec0dd315, SUC_lexicon_tags_2 },
    { 0xe68dd316, SUC_lexicon_tags_31 },
    { 0x4d6dd317, SUC_lexicon_tags_12 },
    { 0xdecdd318, SUC_lexicon_tags_1 },
    { 0xe2cdd316, SUC_lexicon_tags_49 },
    { 0x202dd31a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cdd31e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x842dd320, SUC_lexicon_tags_15 },
    { 0x3a4dd321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75cdd324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedadd326, SUC_lexicon_tags_15 },
    { 0x788dd327, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70dd32a, SUC_lexicon_tags_17 },
    { 0x334dd32b, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606dd330, SUC_lexicon_tags_13 },
    { 0x826dd331, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbb6dd333, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132dd338, SUC_lexicon_tags_38 },
    { 0x9e6dd339, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0dd33c, SUC_lexicon_tags_23 },
    { 0x484dd33d, SUC_lexicon_tags_11 },
    { 0x246dd33e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730dd348, SUC_lexicon_tags_61 },
    { 0xc88dd349, SUC_lexicon_tags_1 },
    { 0x2edd34a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dd34f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0dd353, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04dd35b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4dd35d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2dd366, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98edd36a, SUC_lexicon_tags_2 },
    { 0x9d6dd36b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92dd36e, SUC_lexicon_tags_39 },
    { 0x316dd36f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66dd375, SUC_lexicon_tags_12 },
    { 0x7badd375, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2fedd378, SUC_lexicon_tags_39 },
    { 0xf24dd378, SUC_lexicon_tags_11 },
    { 0xc6dd37a, SUC_lexicon_tags_28 },
    { 0xfd8dd37b, SUC_lexicon_tags_7 },
    { 0xfaedd37c, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cdd382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2dd385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfadd38b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56edd38d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608dd399, SUC_lexicon_tags_19 },
    { 0x74cdd399, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x720dd39c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb80dd39e, SUC_lexicon_tags_25 },
    { 0xdcadd39e, SUC_lexicon_tags_31 },
    { 0x966dd3a0, SUC_lexicon_tags_9 },
    { 0xfe8dd39f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260dd3a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd26dd3a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254dd3ab, SUC_lexicon_tags_38 },
    { 0x102dd3ac, SUC_lexicon_tags_39 },
    { 0xf26dd3ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1edd3b3, SUC_lexicon_tags_10 },
    { 0x1ecdd3b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36add3b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x878dd3b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdc2dd3bb, SUC_lexicon_tags_7 },
    { 0xbc6dd3bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186dd3c0, SUC_lexicon_tags_0 },
    { 0x5f4dd3c0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0dd3c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694dd3c8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10dd3cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40dd3d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xae2dd3d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccdd3d9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x78dd3db, SUC_lexicon_tags_11 },
    { 0x374dd3dc, SUC_lexicon_tags_1 },
    { 0xc06dd3dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x582dd3df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x424dd3e2, SUC_lexicon_tags_27 },
    { 0x7bedd3e3, SUC_lexicon_tags_17 },
    { 0x768dd3e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40dd3ea, SUC_lexicon_tags_7 },
    { 0x432dd3eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fedd3ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64dd3f3, SUC_lexicon_tags_16 },
    { 0x890dd3f4, SUC_lexicon_tags_6 },
    { 0x63edd3f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d6dd3f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbccdd3f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858dd3fd, SUC_lexicon_tags_364 },
    { 0xc5cdd3fd, SUC_lexicon_tags_19 },
    { 0xe1cdd3ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cdd402, SUC_lexicon_tags_65 },
    { 0x1a6dd402, SUC_lexicon_tags_55 },
    { 0xa06dd404, SUC_lexicon_tags_168 },
    { 0x282dd403, SUC_lexicon_tags_33 },
    { 0xdaedd404, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaedd40e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2dd412, SUC_lexicon_tags_32 },
    { 0xa22dd413, SUC_lexicon_tags_1 },
    { 0xe62dd414, SUC_lexicon_tags_87 },
    { 0x404dd414, SUC_lexicon_tags_56 },
    { 0xfa6dd416, SUC_lexicon_tags_23 },
    { 0xadcdd417, SUC_lexicon_tags_16 },
    { 0x8b0dd414, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580dd41c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4dd423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa22dd425, SUC_lexicon_tags_33 },
    { 0xa26dd426, SUC_lexicon_tags_63 },
    { 0xe9add427, SUC_lexicon_tags_19 },
    { 0x142dd427, SUC_lexicon_tags_35 },
    { 0x79cdd428, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xf84dd42b, SUC_lexicon_tags_65 },
    { 0xb94dd42b, SUC_lexicon_tags_17 },
    { 0xf66dd42d, SUC_lexicon_tags_8 },
    { 0x688dd42d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880dd434, SUC_lexicon_tags_19 },
    { 0x31edd434, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80dd43b, SUC_lexicon_tags_17 },
    { 0x742dd43b, SUC_lexicon_tags_0 },
    { 0xecedd43d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2dd440, SUC_lexicon_tags_43 },
    { 0xf72dd440, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc96dd443, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x57cdd445, SUC_lexicon_tags_84 },
    { 0x548dd446, SUC_lexicon_tags_2 },
    { 0xe56dd445, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45edd44c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cdd453, SUC_lexicon_tags_43 },
    { 0x468dd454, SUC_lexicon_tags_43 },
    { 0x55edd454, SUC_lexicon_tags_35 },
    { 0xb54dd454, SUC_lexicon_tags_11 },
    { 0xce8dd457, SUC_lexicon_tags_25 },
    { 0x65edd458, SUC_lexicon_tags_25 },
    { 0xadcdd459, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9add460, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x78cdd462, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560dd466, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x392dd468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd62dd46a, SUC_lexicon_tags_7 },
    { 0xb0add46b, SUC_lexicon_tags_27 },
    { 0xd34dd46b, SUC_lexicon_tags_25 },
    { 0x9f0dd46d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976dd471, SUC_lexicon_tags_32 },
    { 0xd6cdd472, SUC_lexicon_tags_15 },
    { 0x8dadd472, SUC_lexicon_tags_14 },
    { 0x9c8dd474, SUC_lexicon_tags_7 },
    { 0x62add472, SUC_lexicon_tags_79 },
    { 0xddadd476, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86add47a, SUC_lexicon_tags_0 },
    { 0x21cdd47a, SUC_lexicon_tags_30 },
    { 0x1b4dd47c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2dd480, SUC_lexicon_tags_45 },
    { 0x450dd481, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cdd489, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2a8dd48b, SUC_lexicon_tags_12 },
    { 0xbb2dd48c, SUC_lexicon_tags_343 },
    { 0xc9edd48d, SUC_lexicon_tags_2 },
    { 0xdc6dd48e, SUC_lexicon_tags_27 },
    { 0xc16dd48c, SUC_lexicon_tags_6 },
    { 0x474dd490, SUC_lexicon_tags_1 },
    { 0x1edd491, SUC_lexicon_tags_81 },
    { 0xb92dd492, SUC_lexicon_tags_30 },
    { 0x152dd493, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4dd498, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5acdd49a, SUC_lexicon_tags_19 },
    { 0xf5cdd49b, SUC_lexicon_tags_27 },
    { 0x6e6dd49a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0dd4a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cedd4a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aedd4a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6dd4ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x120dd4ae, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb12dd4b0, SUC_lexicon_tags_14 },
    { 0x86edd4b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4add4b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8dd4ba, SUC_lexicon_tags_15 },
    { 0x56edd4bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6dd4bf, SUC_lexicon_tags_32 },
    { 0x8fcdd4bf, SUC_lexicon_tags_32 },
    { 0x358dd4c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234dd4c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x724dd4c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cadd4cf, SUC_lexicon_tags_69 },
    { 0xc10dd4d0, SUC_lexicon_tags_25 },
    { 0x2e4dd4d1, SUC_lexicon_tags_37 },
    { 0x96cdd4d2, SUC_lexicon_tags_32 },
    { 0x1dcdd4d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x758dd4d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cdd4da, SUC_lexicon_tags_11 },
    { 0x14add4da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8dd4de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2a6dd4e0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21add4e4, SUC_lexicon_tags_32 },
    { 0xfadd4e5, SUC_lexicon_tags_31 },
    { 0x32edd4e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4dd4ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6dd4ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806dd4f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x270dd4f4, SUC_lexicon_tags_81 },
    { 0x2cedd4f5, SUC_lexicon_tags_47 },
    { 0x2a4dd4f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7add4f8, SUC_lexicon_tags_1 },
    { 0x708dd4f9, SUC_lexicon_tags_6 },
    { 0x2fadd4f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0dd4fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40dd500, SUC_lexicon_tags_46 },
    { 0x1e2dd501, SUC_lexicon_tags_6 },
    { 0x768dd501, SUC_lexicon_tags_24 },
    { 0xd84dd503, SUC_lexicon_tags_34 },
    { 0x88dd503, SUC_lexicon_tags_1 },
    { 0x146dd500, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0dd509, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8c4dd50b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x532dd50d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32dd511, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6b4dd513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814dd518, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2dd51b, SUC_lexicon_tags_23 },
    { 0xbd2dd51b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x268dd51f, SUC_lexicon_tags_43 },
    { 0x3a8dd520, SUC_lexicon_tags_62 },
    { 0x4badd521, SUC_lexicon_tags_16 },
    { 0x7aadd520, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cdd526, SUC_lexicon_tags_1 },
    { 0x3e6dd526, SUC_lexicon_tags_30 },
    { 0xb74dd528, SUC_lexicon_tags_25 },
    { 0x60add529, SUC_lexicon_tags_985 },
    { 0x5ecdd52a, SUC_lexicon_tags_2 },
    { 0x640dd52b, SUC_lexicon_tags_91 },
    { 0x5dadd52a, SUC_lexicon_tags_9 },
    { 0xb5edd52d, SUC_lexicon_tags_15 },
    { 0x8c0dd52e, SUC_lexicon_tags_17 },
    { 0x8d4dd52c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdacdd532, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4dd536, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6fcdd538, SUC_lexicon_tags_17 },
    { 0x474dd539, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52dd53e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0dd545, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66dd54b, SUC_lexicon_tags_84 },
    { 0x1fedd54c, SUC_lexicon_tags_0 },
    { 0x16dd54d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69cdd54f, SUC_lexicon_tags_17 },
    { 0xd5cdd550, SUC_lexicon_tags_14 },
    { 0xfbcdd550, SUC_lexicon_tags_15 },
    { 0x2c6dd552, SUC_lexicon_tags_63 },
    { 0x8a6dd552, SUC_lexicon_tags_37 },
    { 0x780dd553, SUC_lexicon_tags_48 },
    { 0x600dd550, SUC_lexicon_tags_11 },
    { 0xe94dd556, SUC_lexicon_tags_1 },
    { 0x57add551, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbadd55a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c8dd55c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74dd55f, SUC_lexicon_tags_71 },
    { 0x236dd560, SUC_lexicon_tags_25 },
    { 0x4bedd55f, SUC_lexicon_tags_87 },
    { 0x7eadd560, SUC_lexicon_tags_19 },
    { 0x624dd55f, SUC_lexicon_tags_70 },
    { 0x71edd564, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdd568, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdd570, SUC_lexicon_tags_2 },
    { 0xdfadd570, SUC_lexicon_tags_1 },
    { 0x8badd572, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274dd577, SUC_lexicon_tags_11 },
    { 0x144dd578, SUC_lexicon_tags_1 },
    { 0x9c6dd578, SUC_lexicon_tags_17 },
    { 0x16add57a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616dd580, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x654dd583, SUC_lexicon_tags_11 },
    { 0x29add583, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2dd588, SUC_lexicon_tags_25 },
    { 0xb8cdd589, SUC_lexicon_tags_26 },
    { 0x8c4dd58a, SUC_lexicon_tags_38 },
    { 0xfa8dd58b, SUC_lexicon_tags_17 },
    { 0x15cdd588, SUC_lexicon_tags_37 },
    { 0x906dd58d, SUC_lexicon_tags_27 },
    { 0x3c0dd58e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4cdd590, SUC_lexicon_tags_12 },
    { 0x5d4dd591, SUC_lexicon_tags_37 },
    { 0x4c2dd591, SUC_lexicon_tags_19 },
    { 0xa4add592, SUC_lexicon_tags_25 },
    { 0x728dd590, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x686dd597, SUC_lexicon_tags_17 },
    { 0x896dd598, SUC_lexicon_tags_11 },
    { 0xa9cdd599, SUC_lexicon_tags_27 },
    { 0xdb8dd598, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf10dd59c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x19add59e, SUC_lexicon_tags_49 },
    { 0x2a2dd59e, SUC_lexicon_tags_11 },
    { 0xf46dd59e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dd5a3, SUC_lexicon_tags_29 },
    { 0x998dd5a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6dd5a8, SUC_lexicon_tags_45 },
    { 0xe42dd5a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f0dd5ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdc6dd5ad, SUC_lexicon_tags_23 },
    { 0xea6dd5ae, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968dd5b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56dd5b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658dd5c4, SUC_lexicon_tags_11 },
    { 0x1b6dd5c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd40dd5c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadd5cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616dd5d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4dd5e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0dd5e6, SUC_lexicon_tags_38 },
    { 0xf98dd5e7, SUC_lexicon_tags_35 },
    { 0x878dd5e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd48dd5ea, SUC_lexicon_tags_14 },
    { 0x948dd5eb, SUC_lexicon_tags_11 },
    { 0x2f8dd5ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcadd5f0, SUC_lexicon_tags_91 },
    { 0xef6dd5f1, SUC_lexicon_tags_9 },
    { 0x446dd5f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8edd5f8, SUC_lexicon_tags_12 },
    { 0x9d4dd5f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcadd5fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x490dd5ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cdd604, SUC_lexicon_tags_19 },
    { 0xe4dd605, SUC_lexicon_tags_35 },
    { 0x9a0dd606, SUC_lexicon_tags_60 },
    { 0xa18dd606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x870dd609, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cdd60d, SUC_lexicon_tags_11 },
    { 0xa4add60e, SUC_lexicon_tags_23 },
    { 0xe24dd60f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12dd616, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8dd61e, SUC_lexicon_tags_47 },
    { 0xf4cdd61f, SUC_lexicon_tags_33 },
    { 0x7a6dd61f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88edd626, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecdd62b, SUC_lexicon_tags_17 },
    { 0x44edd62b, SUC_lexicon_tags_15 },
    { 0xef0dd62d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e6dd62f, SUC_lexicon_tags_84 },
    { 0xc80dd630, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100dd635, SUC_lexicon_tags_23 },
    { 0x852dd636, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aadd639, SUC_lexicon_tags_79 },
    { 0xc1add63a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x666dd63d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6add644, SUC_lexicon_tags_25 },
    { 0xb42dd645, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68dd64d, SUC_lexicon_tags_193 },
    { 0x7eadd64e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150dd652, SUC_lexicon_tags_62 },
    { 0x926dd653, SUC_lexicon_tags_47 },
    { 0x7dcdd654, SUC_lexicon_tags_41 },
    { 0x49cdd655, SUC_lexicon_tags_0 },
    { 0x8d4dd656, SUC_lexicon_tags_11 },
    { 0x232dd656, SUC_lexicon_tags_33 },
    { 0x4b4dd655, SUC_lexicon_tags_61 },
    { 0x1aadd659, SUC_lexicon_tags_10 },
    { 0x13add659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2dd65d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0dd660, SUC_lexicon_tags_43 },
    { 0x85add661, SUC_lexicon_tags_9 },
    { 0x504dd662, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf54dd664, SUC_lexicon_tags_31 },
    { 0x254dd665, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6dd66c, SUC_lexicon_tags_11 },
    { 0x322dd66d, SUC_lexicon_tags_45 },
    { 0x9a8dd66d, SUC_lexicon_tags_17 },
    { 0x88cdd66f, SUC_lexicon_tags_11 },
    { 0xe90dd670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x526dd672, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112dd676, SUC_lexicon_tags_38 },
    { 0xefedd676, SUC_lexicon_tags_37 },
    { 0x17edd676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264dd67c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9eadd67e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0dd681, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbedd684, SUC_lexicon_tags_12 },
    { 0xad2dd684, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97edd68a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2dd68f, SUC_lexicon_tags_17 },
    { 0xe02dd690, SUC_lexicon_tags_47 },
    { 0xce8dd690, SUC_lexicon_tags_0 },
    { 0x54dd692, SUC_lexicon_tags_47 },
    { 0xd66dd691, SUC_lexicon_tags_136 },
    { 0xf32dd694, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf50dd696, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576dd69b, SUC_lexicon_tags_25 },
    { 0x68dd69b, SUC_lexicon_tags_25 },
    { 0x24dd69c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x710dd69f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ecdd6a1, SUC_lexicon_tags_23 },
    { 0x764dd6a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4dd6ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dd6b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcdedd6b2, SUC_lexicon_tags_11 },
    { 0x8c8dd6b2, SUC_lexicon_tags_8 },
    { 0x758dd6b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x394dd6b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866dd6bd, SUC_lexicon_tags_39 },
    { 0x17edd6bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81edd6c0, SUC_lexicon_tags_17 },
    { 0x96cdd6c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdcedd6c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x78add6c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x746dd6c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680dd6d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6e5, SUC_lexicon_tags_0 },
    { 0xb3add6e6, SUC_lexicon_tags_23 },
    { 0x13edd6e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfaedd6e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x45edd6ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdd6f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2dd6f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x766dd6f9, SUC_lexicon_tags_1 },
    { 0xc90dd6fa, SUC_lexicon_tags_33 },
    { 0x7c8dd6fb, SUC_lexicon_tags_65 },
    { 0x7e0dd6fb, SUC_lexicon_tags_15 },
    { 0xf4dd6f9, SUC_lexicon_tags_5 },
    { 0x92add6fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eedd701, SUC_lexicon_tags_15 },
    { 0x688dd702, SUC_lexicon_tags_42 },
    { 0x7a6dd701, SUC_lexicon_tags_88 },
    { 0xa9cdd703, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x15edd706, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862dd70e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264dd712, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc6edd714, SUC_lexicon_tags_18 },
    { 0xa12dd715, SUC_lexicon_tags_18 },
    { 0x760dd716, SUC_lexicon_tags_49 },
    { 0xededd715, SUC_lexicon_tags_37 },
    { 0x45edd718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb4dd71a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc0dd71c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5edd71e, SUC_lexicon_tags_31 },
    { 0xa36dd71e, SUC_lexicon_tags_9 },
    { 0x20dd720, SUC_lexicon_tags_11 },
    { 0x91cdd721, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4dadd723, SUC_lexicon_tags_11 },
    { 0xd64dd723, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32dd727, SUC_lexicon_tags_47 },
    { 0x6edd728, SUC_lexicon_tags_10 },
    { 0x846dd727, SUC_lexicon_tags_38 },
    { 0xf0add72a, SUC_lexicon_tags_2 },
    { 0xcd6dd72b, SUC_lexicon_tags_60 },
    { 0x68add72a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8dd72f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318dd734, SUC_lexicon_tags_43 },
    { 0x2b6dd734, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32dd73b, SUC_lexicon_tags_671 },
    { 0xcccdd73b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0dd73f, SUC_lexicon_tags_37 },
    { 0xdf4dd73f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x792dd742, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x62edd744, SUC_lexicon_tags_17 },
    { 0x990dd744, SUC_lexicon_tags_4 },
    { 0x138dd744, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2dd749, SUC_lexicon_tags_65 },
    { 0x190dd749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaedd74d, SUC_lexicon_tags_33 },
    { 0x2dcdd74e, SUC_lexicon_tags_9 },
    { 0x2d8dd74d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dd753, SUC_lexicon_tags_11 },
    { 0xed6dd754, SUC_lexicon_tags_87 },
    { 0x58edd755, SUC_lexicon_tags_8 },
    { 0x93cdd755, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4dd75e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe94dd760, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26dd763, SUC_lexicon_tags_59 },
    { 0xae4dd763, SUC_lexicon_tags_14 },
    { 0x662dd765, SUC_lexicon_tags_262 },
    { 0x578dd766, SUC_lexicon_tags_11 },
    { 0xd22dd766, SUC_lexicon_tags_16 },
    { 0x1aedd768, SUC_lexicon_tags_38 },
    { 0x934dd769, SUC_lexicon_tags_45 },
    { 0x416dd765, SUC_lexicon_tags_28 },
    { 0xf58dd768, SUC_lexicon_tags_108 },
    { 0x8b6dd76a, SUC_lexicon_tags_2 },
    { 0xde0dd76b, SUC_lexicon_tags_26 },
    { 0xd58dd76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75edd770, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00dd773, SUC_lexicon_tags_21 },
    { 0x186dd774, SUC_lexicon_tags_29 },
    { 0x72edd775, SUC_lexicon_tags_17 },
    { 0x4fadd774, SUC_lexicon_tags_77 },
    { 0x9eadd777, SUC_lexicon_tags_17 },
    { 0xc18dd774, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97edd77a, SUC_lexicon_tags_45 },
    { 0xe76dd77a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb02dd77d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x564dd77f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45edd783, SUC_lexicon_tags_17 },
    { 0x6fedd784, SUC_lexicon_tags_16 },
    { 0xe96dd785, SUC_lexicon_tags_38 },
    { 0xc06dd786, SUC_lexicon_tags_17 },
    { 0x7e4dd787, SUC_lexicon_tags_14 },
    { 0xb44dd788, SUC_lexicon_tags_47 },
    { 0x598dd789, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x538dd78b, SUC_lexicon_tags_1 },
    { 0x15add78c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2dd78f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cdd793, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa44dd795, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108dd799, SUC_lexicon_tags_11 },
    { 0xb42dd79a, SUC_lexicon_tags_26 },
    { 0xb1edd79a, SUC_lexicon_tags_14 },
    { 0xa72dd79c, SUC_lexicon_tags_17 },
    { 0xbb6dd79c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2add7a2, SUC_lexicon_tags_2 },
    { 0x5cedd7a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x32edd7a6, SUC_lexicon_tags_23 },
    { 0x6add7a7, SUC_lexicon_tags_23 },
    { 0x87edd7a8, SUC_lexicon_tags_15 },
    { 0xacadd7a8, SUC_lexicon_tags_45 },
    { 0xfccdd7a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84dd7b0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eedd7b3, SUC_lexicon_tags_45 },
    { 0x212dd7b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618dd7b8, SUC_lexicon_tags_34 },
    { 0x956dd7b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0dd7bb, SUC_lexicon_tags_27 },
    { 0x57cdd7bc, SUC_lexicon_tags_49 },
    { 0xde4dd7bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0dd7c2, SUC_lexicon_tags_91 },
    { 0x9c8dd7c2, SUC_lexicon_tags_43 },
    { 0x780dd7c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0dd7c7, SUC_lexicon_tags_4 },
    { 0xcecdd7c7, SUC_lexicon_tags_17 },
    { 0xbbedd7c9, SUC_lexicon_tags_63 },
    { 0x8f4dd7ca, SUC_lexicon_tags_4 },
    { 0x894dd7ca, SUC_lexicon_tags_336 },
    { 0x726dd7cc, SUC_lexicon_tags_29 },
    { 0xaadd7c9, SUC_lexicon_tags_25 },
    { 0x5a0dd7cb, SUC_lexicon_tags_87 },
    { 0xd76dd7c7, SUC_lexicon_tags_43 },
    { 0x80add7d0, SUC_lexicon_tags_29 },
    { 0x53cdd7d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc20dd7d3, SUC_lexicon_tags_38 },
    { 0xe2add7d4, SUC_lexicon_tags_17 },
    { 0xe88dd7d4, SUC_lexicon_tags_61 },
    { 0x66cdd7d6, SUC_lexicon_tags_7 },
    { 0x38edd7d4, SUC_lexicon_tags_37 },
    { 0x482dd7d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6dd7df, SUC_lexicon_tags_17 },
    { 0x710dd7e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63edd7e5, SUC_lexicon_tags_22 },
    { 0x2d8dd7e6, SUC_lexicon_tags_25 },
    { 0xbc8dd7e6, SUC_lexicon_tags_11 },
    { 0x540dd7e7, SUC_lexicon_tags_40 },
    { 0xeb8dd7e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0dd7ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9dadd7ef, SUC_lexicon_tags_7 },
    { 0x5aadd7f0, SUC_lexicon_tags_26 },
    { 0xfa8dd7ef, SUC_lexicon_tags_25 },
    { 0xceedd7ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6edd7f5, SUC_lexicon_tags_0 },
    { 0xa78dd7f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82dd7fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0add809, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x464dd80c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x668dd80f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2add811, SUC_lexicon_tags_28 },
    { 0xeb6dd812, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dedd815, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2dd818, SUC_lexicon_tags_45 },
    { 0x38edd819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd80dd81b, SUC_lexicon_tags_146 },
    { 0xa24dd81c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49edd81f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x838dd821, SUC_lexicon_tags_17 },
    { 0xf0edd822, SUC_lexicon_tags_11 },
    { 0xe42dd823, SUC_lexicon_tags_37 },
    { 0x70add823, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb40dd826, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e2dd828, SUC_lexicon_tags_28 },
    { 0x430dd829, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2dd82c, SUC_lexicon_tags_11 },
    { 0xc8edd82c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcdd833, SUC_lexicon_tags_1 },
    { 0x6edd833, SUC_lexicon_tags_38 },
    { 0x13cdd835, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298dd839, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0dd83e, SUC_lexicon_tags_49 },
    { 0xabadd83f, SUC_lexicon_tags_23 },
    { 0x56add83f, SUC_lexicon_tags_25 },
    { 0x974dd83e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3add847, SUC_lexicon_tags_0 },
    { 0x996dd847, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4dd850, SUC_lexicon_tags_2 },
    { 0x3dedd850, SUC_lexicon_tags_45 },
    { 0x5f4dd851, SUC_lexicon_tags_12 },
    { 0x1e8dd853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4dd85f, SUC_lexicon_tags_21 },
    { 0x910dd860, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2dd864, SUC_lexicon_tags_17 },
    { 0xcfedd864, SUC_lexicon_tags_11 },
    { 0xa2edd865, SUC_lexicon_tags_10 },
    { 0x422dd865, SUC_lexicon_tags_17 },
    { 0x204dd868, SUC_lexicon_tags_16 },
    { 0xc78dd869, SUC_lexicon_tags_65 },
    { 0xca6dd865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f6dd86c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92dd86f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x560dd871, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf0dd873, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6dd878, SUC_lexicon_tags_10 },
    { 0xda0dd878, SUC_lexicon_tags_38 },
    { 0x770dd87a, SUC_lexicon_tags_120 },
    { 0x474dd879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59edd87d, SUC_lexicon_tags_19 },
    { 0x906dd87e, SUC_lexicon_tags_46 },
    { 0x8e2dd87d, SUC_lexicon_tags_15 },
    { 0x2d4dd87d, SUC_lexicon_tags_9 },
    { 0x9d6dd881, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccdd885, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f8dd887, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9edd889, SUC_lexicon_tags_37 },
    { 0xa62dd889, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x646dd88d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3fedd88f, SUC_lexicon_tags_12 },
    { 0x860dd890, SUC_lexicon_tags_74 },
    { 0x3c6dd891, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296dd896, SUC_lexicon_tags_8 },
    { 0x5b8dd897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98dd8a2, SUC_lexicon_tags_45 },
    { 0x934dd8a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf88dd8a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234dd8aa, SUC_lexicon_tags_70 },
    { 0xae4dd8ab, SUC_lexicon_tags_45 },
    { 0x694dd8aa, SUC_lexicon_tags_18 },
    { 0xa08dd8ad, SUC_lexicon_tags_86 },
    { 0xb66dd8ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cdd8b4, SUC_lexicon_tags_37 },
    { 0x626dd8b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72dd8b9, SUC_lexicon_tags_30 },
    { 0x134dd8ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8dd8bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0dd8c5, SUC_lexicon_tags_60 },
    { 0xbcadd8c5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5f2dd8c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcdd8cf, SUC_lexicon_tags_37 },
    { 0x7bedd8d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8dd8d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80dd8de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe8dd8e0, SUC_lexicon_tags_9 },
    { 0x2a2dd8e1, SUC_lexicon_tags_41 },
    { 0x63cdd8e2, SUC_lexicon_tags_15 },
    { 0x3d4dd8e0, SUC_lexicon_tags_2 },
    { 0x1f4dd8e4, SUC_lexicon_tags_9 },
    { 0x130dd8e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1fcdd8e7, SUC_lexicon_tags_8 },
    { 0x6a4dd8e8, SUC_lexicon_tags_95 },
    { 0x61add8e9, SUC_lexicon_tags_7 },
    { 0x6eadd8ea, SUC_lexicon_tags_32 },
    { 0xaa0dd8eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0dd8ee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6dd8f1, SUC_lexicon_tags_17 },
    { 0xf52dd8f2, SUC_lexicon_tags_25 },
    { 0x94dd8f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x532dd8f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0dd8f8, SUC_lexicon_tags_8 },
    { 0x500dd8f8, SUC_lexicon_tags_2 },
    { 0x360dd8fa, SUC_lexicon_tags_63 },
    { 0x588dd8fb, SUC_lexicon_tags_37 },
    { 0x258dd8fa, SUC_lexicon_tags_11 },
    { 0x34dd8fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1cdd8ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17edd904, SUC_lexicon_tags_60 },
    { 0x1fedd905, SUC_lexicon_tags_19 },
    { 0x990dd905, SUC_lexicon_tags_12 },
    { 0xe24dd906, SUC_lexicon_tags_12 },
    { 0x718dd908, SUC_lexicon_tags_49 },
    { 0x17cdd909, SUC_lexicon_tags_28 },
    { 0xa16dd908, SUC_lexicon_tags_26 },
    { 0xedcdd908, SUC_lexicon_tags_66 },
    { 0xc50dd90c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28dd910, SUC_lexicon_tags_11 },
    { 0x5cedd910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8badd914, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97edd916, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x462dd919, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94add91c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0dd920, SUC_lexicon_tags_25 },
    { 0x3acdd920, SUC_lexicon_tags_62 },
    { 0xe6cdd920, SUC_lexicon_tags_95 },
    { 0xce0dd921, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c8dd925, SUC_lexicon_tags_9 },
    { 0xa7edd925, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24add92b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x710dd92d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc3add92f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324dd933, SUC_lexicon_tags_45 },
    { 0xf76dd934, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeadd938, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180dd93c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x83add93e, SUC_lexicon_tags_33 },
    { 0x382dd93e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662dd944, SUC_lexicon_tags_47 },
    { 0xeb6dd945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe42dd947, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfdedd949, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe30dd94b, SUC_lexicon_tags_6 },
    { 0x986dd94c, SUC_lexicon_tags_594 },
    { 0xcedd94b, SUC_lexicon_tags_5 },
    { 0xdc8dd94e, SUC_lexicon_tags_27 },
    { 0xa12dd94f, SUC_lexicon_tags_17 },
    { 0x51edd950, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34edd955, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x496dd957, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9add959, SUC_lexicon_tags_37 },
    { 0xa06dd95a, SUC_lexicon_tags_60 },
    { 0x926dd95b, SUC_lexicon_tags_8 },
    { 0xd48dd95c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cdd968, SUC_lexicon_tags_31 },
    { 0x514dd968, SUC_lexicon_tags_19 },
    { 0x6dedd96a, SUC_lexicon_tags_43 },
    { 0x9a0dd96b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6ecdd96d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x460dd96f, SUC_lexicon_tags_0 },
    { 0xabcdd970, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5ecdd972, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdd977, SUC_lexicon_tags_45 },
    { 0xa08dd978, SUC_lexicon_tags_2 },
    { 0x15add977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x724dd97c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6dd980, SUC_lexicon_tags_27 },
    { 0xc42dd981, SUC_lexicon_tags_37 },
    { 0xba8dd982, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92dd985, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x98add987, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61edd98b, SUC_lexicon_tags_37 },
    { 0xa14dd98c, SUC_lexicon_tags_37 },
    { 0x484dd98b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x158dd98f, SUC_lexicon_tags_38 },
    { 0xcd6dd98f, SUC_lexicon_tags_33 },
    { 0x34cdd991, SUC_lexicon_tags_40 },
    { 0x2dedd992, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcedd995, SUC_lexicon_tags_27 },
    { 0x3cadd996, SUC_lexicon_tags_15 },
    { 0x6e8dd997, SUC_lexicon_tags_1 },
    { 0x16add995, SUC_lexicon_tags_68 },
    { 0x20dd998, SUC_lexicon_tags_40 },
    { 0xef8dd99a, SUC_lexicon_tags_25 },
    { 0xd4add99a, SUC_lexicon_tags_37 },
    { 0xbcadd99a, SUC_lexicon_tags_37 },
    { 0x92add999, SUC_lexicon_tags_37 },
    { 0x2b4dd997, SUC_lexicon_tags_25 },
    { 0xb92dd997, SUC_lexicon_tags_23 },
    { 0x348dd997, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aadd9a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x574dd9a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83edd9a7, SUC_lexicon_tags_2 },
    { 0xdfadd9a8, SUC_lexicon_tags_29 },
    { 0x85edd9a8, SUC_lexicon_tags_25 },
    { 0x4cedd9aa, SUC_lexicon_tags_17 },
    { 0x398dd9a8, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x73edd9ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x990dd9b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecedd9b2, SUC_lexicon_tags_65 },
    { 0xe0dd9b3, SUC_lexicon_tags_47 },
    { 0x3e8dd9b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88dd9bb, SUC_lexicon_tags_26 },
    { 0xb24dd9bc, SUC_lexicon_tags_1 },
    { 0x2d6dd9bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35cdd9bf, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x50add9c2, SUC_lexicon_tags_37 },
    { 0xe32dd9c2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130dd9c7, SUC_lexicon_tags_25 },
    { 0xd44dd9c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34edd9d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x522dd9d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cdd9d6, SUC_lexicon_tags_28 },
    { 0xc50dd9d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdadd9da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0dd9e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf16dd9e3, SUC_lexicon_tags_25 },
    { 0xaadd9e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfedd9ea, SUC_lexicon_tags_32 },
    { 0x2badd9eb, SUC_lexicon_tags_23 },
    { 0x2aedd9ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10dd9f0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55add9f4, SUC_lexicon_tags_29 },
    { 0xd4edd9f5, SUC_lexicon_tags_12 },
    { 0x1fcdd9f4, SUC_lexicon_tags_37 },
    { 0x65edd9f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe6edd9f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18dd9fe, SUC_lexicon_tags_37 },
    { 0x190dd9fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2dda03, SUC_lexicon_tags_7 },
    { 0x512dda04, SUC_lexicon_tags_79 },
    { 0x916dda05, SUC_lexicon_tags_11 },
    { 0x222dda06, SUC_lexicon_tags_16 },
    { 0x7d0dda06, SUC_lexicon_tags_15 },
    { 0x236dda08, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x20dda0a, SUC_lexicon_tags_37 },
    { 0x7b8dda0b, SUC_lexicon_tags_17 },
    { 0x9aedda0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dda12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74dda17, SUC_lexicon_tags_63 },
    { 0x694dda17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe8cdda1a, SUC_lexicon_tags_17 },
    { 0x7c4dda1b, SUC_lexicon_tags_17 },
    { 0xdda1b, SUC_lexicon_tags_47 },
    { 0xcf6dda1b, SUC_lexicon_tags_19 },
    { 0x762dda1e, SUC_lexicon_tags_14 },
    { 0xab6dda1e, SUC_lexicon_tags_14 },
    { 0xa04dda20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596dda27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6badda31, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8dda3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90dda3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380dda43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x252dda46, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x234dda49, SUC_lexicon_tags_2 },
    { 0x25adda4a, SUC_lexicon_tags_38 },
    { 0xf7adda4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfa2dda4d, SUC_lexicon_tags_25 },
    { 0xf16dda4e, SUC_lexicon_tags_17 },
    { 0xb10dda4f, SUC_lexicon_tags_4 },
    { 0xb08dda50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2dda54, SUC_lexicon_tags_1 },
    { 0x7f4dda54, SUC_lexicon_tags_33 },
    { 0x3dadda56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd82dda58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40dda61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cdda66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x20dda69, SUC_lexicon_tags_0 },
    { 0xa0edda6a, SUC_lexicon_tags_12 },
    { 0xe98dda6b, SUC_lexicon_tags_30 },
    { 0x25adda6b, SUC_lexicon_tags_45 },
    { 0x1e2dda6d, SUC_lexicon_tags_8 },
    { 0x4dcdda6e, SUC_lexicon_tags_92 },
    { 0x6c8dda6f, SUC_lexicon_tags_0 },
    { 0x132dda6c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92dda73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecdda75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72dda79, SUC_lexicon_tags_32 },
    { 0x3c0dda7a, SUC_lexicon_tags_37 },
    { 0x554dda7b, SUC_lexicon_tags_27 },
    { 0xc4adda7c, SUC_lexicon_tags_17 },
    { 0x1fedda7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26adda80, SUC_lexicon_tags_11 },
    { 0xaeedda81, SUC_lexicon_tags_11 },
    { 0xd0dda82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9adda85, SUC_lexicon_tags_12 },
    { 0xa4adda86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58edda8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2dda8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecdda96, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x376dda98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6dda9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874dda9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x180ddaa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ceddaa4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492ddaae, SUC_lexicon_tags_45 },
    { 0xbeaddaaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faddab4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3feddaba, SUC_lexicon_tags_49 },
    { 0x94eddabb, SUC_lexicon_tags_11 },
    { 0x7c6ddabc, SUC_lexicon_tags_57 },
    { 0xe06ddabd, SUC_lexicon_tags_10 },
    { 0x172ddabe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7addac3, SUC_lexicon_tags_27 },
    { 0x13addac3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8ddac9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4caddacc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972ddad4, SUC_lexicon_tags_17 },
    { 0x5aeddad5, SUC_lexicon_tags_21 },
    { 0xbdaddad5, SUC_lexicon_tags_40 },
    { 0xd8addad5, SUC_lexicon_tags_21 },
    { 0x280ddad8, SUC_lexicon_tags_49 },
    { 0x48cddad9, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0x7c0ddadb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cddadf, SUC_lexicon_tags_1 },
    { 0x126ddadf, SUC_lexicon_tags_37 },
    { 0x5f2ddae0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36ddae4, SUC_lexicon_tags_7 },
    { 0x32addae5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7addae7, SUC_lexicon_tags_10 },
    { 0x7ceddae7, SUC_lexicon_tags_15 },
    { 0xef0ddae7, SUC_lexicon_tags_1 },
    { 0xe8eddae9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x580ddaec, SUC_lexicon_tags_9 },
    { 0xc70ddaec, SUC_lexicon_tags_2 },
    { 0xdccddaed, SUC_lexicon_tags_0 },
    { 0x68ddaed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7addaf3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x64addaf5, SUC_lexicon_tags_5 },
    { 0x75cddaf5, SUC_lexicon_tags_37 },
    { 0xb0addaf7, SUC_lexicon_tags_21 },
    { 0x6beddaf7, SUC_lexicon_tags_68 },
    { 0xaa0ddaf9, SUC_lexicon_tags_35 },
    { 0xeeeddafa, SUC_lexicon_tags_65 },
    { 0xacaddaf6, SUC_lexicon_tags_27 },
    { 0x59cddafc, SUC_lexicon_tags_1 },
    { 0x75eddafc, SUC_lexicon_tags_14 },
    { 0xeceddafe, SUC_lexicon_tags_19 },
    { 0xee0ddaff, SUC_lexicon_tags_39 },
    { 0x9c6ddb00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30ddb02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa5eddb04, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512ddb09, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cddb0f, SUC_lexicon_tags_69 },
    { 0x4d0ddb0f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x826ddb12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9feddb16, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00ddb19, SUC_lexicon_tags_31 },
    { 0x564ddb1a, SUC_lexicon_tags_25 },
    { 0x112ddb1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7addb21, SUC_lexicon_tags_11 },
    { 0x3a8ddb22, SUC_lexicon_tags_37 },
    { 0x51addb23, SUC_lexicon_tags_141 },
    { 0x47addb24, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x814ddb26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cddb2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x730ddb2c, SUC_lexicon_tags_57 },
    { 0xe76ddb2c, SUC_lexicon_tags_1 },
    { 0x400ddb2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37eddb31, SUC_lexicon_tags_11 },
    { 0xf1cddb32, SUC_lexicon_tags_11 },
    { 0xda8ddb32, SUC_lexicon_tags_9 },
    { 0xda6ddb34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe3cddb36, SUC_lexicon_tags_45 },
    { 0x1eaddb37, SUC_lexicon_tags_87 },
    { 0x94ddb38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4ddb3b, SUC_lexicon_tags_7 },
    { 0x830ddb3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4ddb40, SUC_lexicon_tags_25 },
    { 0xb10ddb41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96eddb43, SUC_lexicon_tags_17 },
    { 0x3b6ddb44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x918ddb46, SUC_lexicon_tags_17 },
    { 0xef0ddb47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x136ddb49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x376ddb4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faddb5b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ddb69, SUC_lexicon_tags_26 },
    { 0xfb2ddb69, SUC_lexicon_tags_1 },
    { 0x676ddb6b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8ddb6f, SUC_lexicon_tags_1 },
    { 0xfd0ddb70, SUC_lexicon_tags_23 },
    { 0xf86ddb71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x79eddb73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5addb7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42ddb81, SUC_lexicon_tags_17 },
    { 0x69cddb82, SUC_lexicon_tags_17 },
    { 0xaaeddb83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4cddb85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96ddb8a, SUC_lexicon_tags_23 },
    { 0xfaeddb8a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x248ddb8d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2c6ddb8f, SUC_lexicon_tags_21 },
    { 0xba2ddb8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xacddb92, SUC_lexicon_tags_25 },
    { 0xb06ddb92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb4ddb95, SUC_lexicon_tags_43 },
    { 0x946ddb95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ddb9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3addb9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eddba4, SUC_lexicon_tags_31 },
    { 0xddcddba4, SUC_lexicon_tags_33 },
    { 0x6b4ddba6, SUC_lexicon_tags_2 },
    { 0x90eddba7, SUC_lexicon_tags_14 },
    { 0x88ddba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2ddbab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8ddbb1, SUC_lexicon_tags_47 },
    { 0x42addbb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64ddbb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4eddbb8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaddbbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x830ddbc0, SUC_lexicon_tags_7 },
    { 0xcd4ddbc0, SUC_lexicon_tags_23 },
    { 0x58eddbc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb42ddbc4, SUC_lexicon_tags_63 },
    { 0x3deddbc5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e2ddbc7, SUC_lexicon_tags_7 },
    { 0x7c2ddbc8, SUC_lexicon_tags_17 },
    { 0x614ddbc8, SUC_lexicon_tags_10 },
    { 0xf34ddbc8, SUC_lexicon_tags_12 },
    { 0x6ceddbcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14eddbce, SUC_lexicon_tags_45 },
    { 0xedcddbcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x270ddbd2, SUC_lexicon_tags_25 },
    { 0xca6ddbd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x328ddbd5, SUC_lexicon_tags_15 },
    { 0xe3cddbd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7aeddbd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6ddbdb, SUC_lexicon_tags_11 },
    { 0xf70ddbdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4ddbe0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xec8ddbe2, SUC_lexicon_tags_7 },
    { 0xea8ddbe2, SUC_lexicon_tags_15 },
    { 0xce2ddbe2, SUC_lexicon_tags_31 },
    { 0x752ddbe5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x656ddbe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaddbec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3addbef, SUC_lexicon_tags_11 },
    { 0xbfeddbf0, SUC_lexicon_tags_37 },
    { 0x9c8ddbf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2ddbf4, SUC_lexicon_tags_12 },
    { 0x732ddbf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a8ddbf7, SUC_lexicon_tags_12 },
    { 0xdc2ddbf7, SUC_lexicon_tags_31 },
    { 0xeb0ddbf9, SUC_lexicon_tags_11 },
    { 0x4caddbf9, SUC_lexicon_tags_37 },
    { 0x2f4ddbf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6ddbfe, SUC_lexicon_tags_15 },
    { 0xc8addbfe, SUC_lexicon_tags_11 },
    { 0x3a4ddbfe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x700ddc03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2ddc06, SUC_lexicon_tags_36 },
    { 0xc46ddc06, SUC_lexicon_tags_27 },
    { 0x426ddc08, SUC_lexicon_tags_57 },
    { 0x548ddc08, SUC_lexicon_tags_26 },
    { 0x488ddc0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18ddc0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786ddc18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x366ddc1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x290ddc1e, SUC_lexicon_tags_2 },
    { 0x394ddc1e, SUC_lexicon_tags_11 },
    { 0xb46ddc20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6addc23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cddc26, SUC_lexicon_tags_21 },
    { 0x174ddc26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x332ddc2a, SUC_lexicon_tags_25 },
    { 0x8deddc2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf02ddc2d, SUC_lexicon_tags_31 },
    { 0x6b8ddc2e, SUC_lexicon_tags_43 },
    { 0xb6eddc2e, SUC_lexicon_tags_12 },
    { 0xb3eddc2e, SUC_lexicon_tags_57 },
    { 0x6ddc2f, SUC_lexicon_tags_17 },
    { 0xe62ddc2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa72ddc34, SUC_lexicon_tags_1 },
    { 0x3faddc35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66ddc3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b6ddc41, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62ddc47, SUC_lexicon_tags_12 },
    { 0x6acddc48, SUC_lexicon_tags_45 },
    { 0x1feddc47, SUC_lexicon_tags_28 },
    { 0xdeeddc4a, SUC_lexicon_tags_10 },
    { 0x96addc49, SUC_lexicon_tags_37 },
    { 0x1eaddc47, SUC_lexicon_tags_65 },
    { 0xe2cddc4d, SUC_lexicon_tags_27 },
    { 0x29addc4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8ddc52, SUC_lexicon_tags_17 },
    { 0x1f2ddc52, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614ddc58, SUC_lexicon_tags_15 },
    { 0xe84ddc59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ddc5e, SUC_lexicon_tags_0 },
    { 0x26eddc5f, SUC_lexicon_tags_25 },
    { 0xf34ddc60, SUC_lexicon_tags_66 },
    { 0x298ddc61, SUC_lexicon_tags_33 },
    { 0xf6ddc62, SUC_lexicon_tags_17 },
    { 0x91eddc61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1acddc66, SUC_lexicon_tags_25 },
    { 0xc12ddc67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x290ddc6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95cddc6c, SUC_lexicon_tags_17 },
    { 0x8e0ddc6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82addc70, SUC_lexicon_tags_11 },
    { 0x6aaddc71, SUC_lexicon_tags_23 },
    { 0xb6eddc72, SUC_lexicon_tags_1 },
    { 0x26cddc72, SUC_lexicon_tags_25 },
    { 0x1d6ddc74, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x592ddc77, SUC_lexicon_tags_62 },
    { 0x794ddc77, SUC_lexicon_tags_25 },
    { 0x41cddc78, SUC_lexicon_tags_11 },
    { 0xe7eddc78, SUC_lexicon_tags_47 },
    { 0x496ddc7b, SUC_lexicon_tags_0 },
    { 0x7d8ddc7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f4ddc7e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3deddc81, SUC_lexicon_tags_2 },
    { 0x4a0ddc82, SUC_lexicon_tags_56 },
    { 0x548ddc83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd5addc85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe68ddc87, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8ddc8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaddc90, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x46addc92, SUC_lexicon_tags_14 },
    { 0xb72ddc92, SUC_lexicon_tags_7 },
    { 0x94eddc94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x442ddc96, SUC_lexicon_tags_37 },
    { 0x4beddc96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2addc9b, SUC_lexicon_tags_28 },
    { 0x376ddc9c, SUC_lexicon_tags_12 },
    { 0xa9eddc9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cddca0, SUC_lexicon_tags_47 },
    { 0x91eddca1, SUC_lexicon_tags_7 },
    { 0x8e4ddca2, SUC_lexicon_tags_17 },
    { 0xa0cddca3, SUC_lexicon_tags_45 },
    { 0xf14ddca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64ddca6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310ddcaa, SUC_lexicon_tags_68 },
    { 0xaf8ddcab, SUC_lexicon_tags_21 },
    { 0xb6eddcab, SUC_lexicon_tags_60 },
    { 0x4d0ddcad, SUC_lexicon_tags_11 },
    { 0x8eaddcac, SUC_lexicon_tags_38 },
    { 0xfaeddcad, SUC_lexicon_tags_15 },
    { 0x8a2ddcaf, SUC_lexicon_tags_12 },
    { 0x284ddcb1, SUC_lexicon_tags_23 },
    { 0xb1eddcaf, SUC_lexicon_tags_37 },
    { 0x582ddcb3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x96addcb6, SUC_lexicon_tags_1 },
    { 0x7bcddcb7, SUC_lexicon_tags_38 },
    { 0xe8eddcb8, SUC_lexicon_tags_30 },
    { 0xedaddcb9, SUC_lexicon_tags_49 },
    { 0x3f6ddcb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x132ddcbc, SUC_lexicon_tags_14 },
    { 0xaa2ddcbc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6ddcc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x572ddcc2, SUC_lexicon_tags_14 },
    { 0xcd6ddcc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf12ddcc5, SUC_lexicon_tags_31 },
    { 0xff4ddcc6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ddcc9, SUC_lexicon_tags_16 },
    { 0x8beddcca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe46ddccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598ddcd0, SUC_lexicon_tags_60 },
    { 0x17cddcd1, SUC_lexicon_tags_43 },
    { 0xe7cddcd0, SUC_lexicon_tags_8 },
    { 0x502ddcd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1eddcd5, SUC_lexicon_tags_45 },
    { 0xee4ddcd6, SUC_lexicon_tags_7 },
    { 0x712ddcd6, SUC_lexicon_tags_39 },
    { 0x1bcddcd7, SUC_lexicon_tags_23 },
    { 0x504ddcd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb64ddcdb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x77cddcdd, SUC_lexicon_tags_7 },
    { 0x290ddcdd, SUC_lexicon_tags_0 },
    { 0x894ddcdf, SUC_lexicon_tags_23 },
    { 0x6e2ddce0, SUC_lexicon_tags_64 },
    { 0x1d2ddce1, SUC_lexicon_tags_2 },
    { 0xea0ddcde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x286ddce4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466ddce8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40eddcec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568ddcf2, SUC_lexicon_tags_138 },
    { 0xf0ddcf3, SUC_lexicon_tags_141 },
    { 0x51eddcf3, SUC_lexicon_tags_17 },
    { 0x42eddcf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5eddcf7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0ddcfa, SUC_lexicon_tags_17 },
    { 0x8d2ddcfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8ddcfe, SUC_lexicon_tags_1 },
    { 0x17cddcff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8ddd07, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe82ddd09, SUC_lexicon_tags_32 },
    { 0x516ddd0a, SUC_lexicon_tags_193 },
    { 0xe82ddd0b, SUC_lexicon_tags_26 },
    { 0x10cddd0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12cddd0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0ddd13, SUC_lexicon_tags_28 },
    { 0xa12ddd14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb38ddd16, SUC_lexicon_tags_17 },
    { 0xa10ddd16, SUC_lexicon_tags_79 },
    { 0x424ddd17, SUC_lexicon_tags_49 },
    { 0x444ddd19, SUC_lexicon_tags_2 },
    { 0x552ddd1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cddd1f, SUC_lexicon_tags_2 },
    { 0x498ddd20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab8ddd22, SUC_lexicon_tags_23 },
    { 0x404ddd23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80ddd26, SUC_lexicon_tags_2 },
    { 0x528ddd26, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2eddd29, SUC_lexicon_tags_15 },
    { 0x966ddd2a, SUC_lexicon_tags_37 },
    { 0xdb8ddd2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa22ddd2d, SUC_lexicon_tags_46 },
    { 0x516ddd2d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90ddd34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6ddd38, SUC_lexicon_tags_15 },
    { 0x9bcddd38, SUC_lexicon_tags_50 },
    { 0xeeaddd3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8b8ddd3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72cddd3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfacddd40, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbdeddd42, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecddd46, SUC_lexicon_tags_15 },
    { 0x37eddd47, SUC_lexicon_tags_17 },
    { 0x868ddd48, SUC_lexicon_tags_12 },
    { 0x2f8ddd48, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x834ddd4b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xaacddd4e, SUC_lexicon_tags_37 },
    { 0xc78ddd4e, SUC_lexicon_tags_10 },
    { 0x1a2ddd50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8ddd54, SUC_lexicon_tags_14 },
    { 0xb0cddd54, SUC_lexicon_tags_1 },
    { 0x542ddd56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224ddd5b, SUC_lexicon_tags_37 },
    { 0xb40ddd5c, SUC_lexicon_tags_60 },
    { 0x3e8ddd5c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe22ddd5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20ddd62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ddd66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cddd6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab2ddd6d, SUC_lexicon_tags_1 },
    { 0x37addd6e, SUC_lexicon_tags_11 },
    { 0x9eddd6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900ddd76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cddd79, SUC_lexicon_tags_15 },
    { 0x7b8ddd7a, SUC_lexicon_tags_7 },
    { 0xbb4ddd7b, SUC_lexicon_tags_4 },
    { 0x2acddd7b, SUC_lexicon_tags_88 },
    { 0x622ddd79, SUC_lexicon_tags_45 },
    { 0x7f0ddd7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6ddd83, SUC_lexicon_tags_17 },
    { 0x918ddd84, SUC_lexicon_tags_37 },
    { 0x29eddd84, SUC_lexicon_tags_15 },
    { 0x262ddd86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69addd89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaddd8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0ddd93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xef4ddd95, SUC_lexicon_tags_37 },
    { 0x7a8ddd95, SUC_lexicon_tags_11 },
    { 0x66addd95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcddd9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb4ddd9e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cddda1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cddda9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462dddad, SUC_lexicon_tags_17 },
    { 0x3f2dddae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84dddb1, SUC_lexicon_tags_11 },
    { 0x34cdddb2, SUC_lexicon_tags_32 },
    { 0x422dddb2, SUC_lexicon_tags_49 },
    { 0x30adddb1, SUC_lexicon_tags_2 },
    { 0x806dddb4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x862dddb7, SUC_lexicon_tags_71 },
    { 0x6dddb7, SUC_lexicon_tags_25 },
    { 0x1b8dddb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x820dddbb, SUC_lexicon_tags_14 },
    { 0x1ecdddbc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cdddc0, SUC_lexicon_tags_7 },
    { 0x96edddc1, SUC_lexicon_tags_38 },
    { 0xf26dddc2, SUC_lexicon_tags_55 },
    { 0x4e8dddc3, SUC_lexicon_tags_18 },
    { 0xfccdddc4, SUC_lexicon_tags_47 },
    { 0xe7cdddc4, SUC_lexicon_tags_12 },
    { 0x314dddc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6dddc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd6dddcb, SUC_lexicon_tags_17 },
    { 0x6b0dddcc, SUC_lexicon_tags_11 },
    { 0x912dddcb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xeb0dddcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0edddd1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40dddd5, SUC_lexicon_tags_6 },
    { 0x96adddd5, SUC_lexicon_tags_15 },
    { 0xaa0dddd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d0dddd9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44ddddc, SUC_lexicon_tags_25 },
    { 0x16cddddd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cddde0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33addde2, SUC_lexicon_tags_17 },
    { 0xf28ddde3, SUC_lexicon_tags_37 },
    { 0xbf0ddde4, SUC_lexicon_tags_12 },
    { 0x28addde4, SUC_lexicon_tags_19 },
    { 0xf64ddde6, SUC_lexicon_tags_27 },
    { 0x418ddde4, SUC_lexicon_tags_7 },
    { 0x2b8ddde8, SUC_lexicon_tags_47 },
    { 0x986ddde9, SUC_lexicon_tags_25 },
    { 0x6b8ddde5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2ddded, SUC_lexicon_tags_11 },
    { 0xe9cdddee, SUC_lexicon_tags_37 },
    { 0xe0adddef, SUC_lexicon_tags_2 },
    { 0x270dddee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51adddf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdddf9, SUC_lexicon_tags_17 },
    { 0x84edddfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1edddfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb34dde00, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dde0c, SUC_lexicon_tags_26 },
    { 0x13cdde0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb8edde0f, SUC_lexicon_tags_12 },
    { 0xc4edde10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a8dde12, SUC_lexicon_tags_17 },
    { 0xeedde13, SUC_lexicon_tags_25 },
    { 0x3eedde12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1edde17, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0dde1a, SUC_lexicon_tags_21 },
    { 0x4d4dde1b, SUC_lexicon_tags_7 },
    { 0x7a0dde1c, SUC_lexicon_tags_77 },
    { 0x716dde1d, SUC_lexicon_tags_0 },
    { 0x3b0dde1c, SUC_lexicon_tags_13 },
    { 0x640dde1e, SUC_lexicon_tags_15 },
    { 0x29adde20, SUC_lexicon_tags_0 },
    { 0x368dde21, SUC_lexicon_tags_6 },
    { 0x348dde21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68dde25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8dde28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xea8dde2a, SUC_lexicon_tags_12 },
    { 0x394dde2b, SUC_lexicon_tags_11 },
    { 0x2e4dde2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x474dde2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6dde31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3edde3a, SUC_lexicon_tags_26 },
    { 0x28edde3b, SUC_lexicon_tags_235 },
    { 0xdb8dde3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48dde46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b6dde48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0dde4b, SUC_lexicon_tags_11 },
    { 0xda0dde4c, SUC_lexicon_tags_45 },
    { 0x58edde4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24dde52, SUC_lexicon_tags_25 },
    { 0x810dde52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4dde56, SUC_lexicon_tags_51 },
    { 0x8b2dde57, SUC_lexicon_tags_17 },
    { 0x9eedde58, SUC_lexicon_tags_34 },
    { 0x538dde58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04dde5d, SUC_lexicon_tags_37 },
    { 0x4e8dde5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254dde62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x186dde64, SUC_lexicon_tags_37 },
    { 0x7a4dde65, SUC_lexicon_tags_43 },
    { 0xa9cdde66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x206dde68, SUC_lexicon_tags_11 },
    { 0x356dde68, SUC_lexicon_tags_19 },
    { 0xae4dde6a, SUC_lexicon_tags_12 },
    { 0xec4dde6b, SUC_lexicon_tags_45 },
    { 0x246dde6b, SUC_lexicon_tags_31 },
    { 0xd5cdde6d, SUC_lexicon_tags_7 },
    { 0x6eedde6e, SUC_lexicon_tags_63 },
    { 0x58adde6e, SUC_lexicon_tags_45 },
    { 0xf9adde70, SUC_lexicon_tags_60 },
    { 0x8d2dde6f, SUC_lexicon_tags_13 },
    { 0x7aedde6a, SUC_lexicon_tags_69 },
    { 0x89adde70, SUC_lexicon_tags_31 },
    { 0x898dde71, SUC_lexicon_tags_25 },
    { 0xbfadde68, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70dde7a, SUC_lexicon_tags_15 },
    { 0xf7adde7a, SUC_lexicon_tags_5 },
    { 0xa1edde7c, SUC_lexicon_tags_26 },
    { 0x44dde7d, SUC_lexicon_tags_37 },
    { 0xb54dde7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7adde80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x27cdde82, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6a2dde84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x44cdde87, SUC_lexicon_tags_30 },
    { 0x948dde87, SUC_lexicon_tags_43 },
    { 0xe5adde89, SUC_lexicon_tags_29 },
    { 0x47edde88, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccdde8e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x628dde90, SUC_lexicon_tags_68 },
    { 0x1f2dde91, SUC_lexicon_tags_11 },
    { 0x5a4dde92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x560dde94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410dde98, SUC_lexicon_tags_37 },
    { 0x44edde98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edde9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32ddea6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x108ddea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99addeaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592ddeb0, SUC_lexicon_tags_27 },
    { 0x528ddeb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558ddeb5, SUC_lexicon_tags_27 },
    { 0xa9addeb6, SUC_lexicon_tags_7 },
    { 0xceaddeb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922ddebd, SUC_lexicon_tags_92 },
    { 0x76eddebd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x836ddec0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0ddec3, SUC_lexicon_tags_17 },
    { 0xbdcddec3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1addec8, SUC_lexicon_tags_37 },
    { 0x3c4ddec9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc8addecb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe50ddecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58dded1, SUC_lexicon_tags_82 },
    { 0xc00dded2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0dded5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x806dded7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566ddedb, SUC_lexicon_tags_16 },
    { 0x662ddedb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ceddede, SUC_lexicon_tags_0 },
    { 0x3ceddedf, SUC_lexicon_tags_25 },
    { 0x6addee0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680ddee5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x588ddee7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40ddeec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8ddef0, SUC_lexicon_tags_43 },
    { 0x72ddef1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136ddef5, SUC_lexicon_tags_35 },
    { 0xfe8ddef6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68ddefb, SUC_lexicon_tags_17 },
    { 0xa1cddefb, SUC_lexicon_tags_27 },
    { 0x724ddefc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0ddf00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04ddf06, SUC_lexicon_tags_25 },
    { 0xfeeddf07, SUC_lexicon_tags_2 },
    { 0x24cddf07, SUC_lexicon_tags_24 },
    { 0x79addf08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cddf0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x934ddf10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x344ddf12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa54ddf14, SUC_lexicon_tags_19 },
    { 0x7e4ddf15, SUC_lexicon_tags_14 },
    { 0x652ddf14, SUC_lexicon_tags_29 },
    { 0x7d2ddf17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30ddf1b, SUC_lexicon_tags_2 },
    { 0xce4ddf1c, SUC_lexicon_tags_82 },
    { 0x59cddf1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9baddf1f, SUC_lexicon_tags_25 },
    { 0x3c8ddf20, SUC_lexicon_tags_38 },
    { 0x4c8ddf21, SUC_lexicon_tags_2 },
    { 0x7cddf22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa56ddf24, SUC_lexicon_tags_49 },
    { 0x4aeddf25, SUC_lexicon_tags_11 },
    { 0x37addf26, SUC_lexicon_tags_29 },
    { 0x328ddf27, SUC_lexicon_tags_65 },
    { 0xd58ddf24, SUC_lexicon_tags_27 },
    { 0x466ddf29, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eddf2c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62ddf33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8ddf37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x396ddf39, SUC_lexicon_tags_37 },
    { 0x298ddf3a, SUC_lexicon_tags_7 },
    { 0x544ddf3b, SUC_lexicon_tags_2 },
    { 0x7f6ddf3b, SUC_lexicon_tags_19 },
    { 0xddf3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4ddf44, SUC_lexicon_tags_46 },
    { 0x558ddf44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882ddf49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3eddf4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2ddf51, SUC_lexicon_tags_11 },
    { 0x154ddf52, SUC_lexicon_tags_4 },
    { 0xc2eddf51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90ddf57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe80ddf59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826ddf5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868ddf63, SUC_lexicon_tags_25 },
    { 0x958ddf64, SUC_lexicon_tags_67 },
    { 0xdf4ddf64, SUC_lexicon_tags_1 },
    { 0xa2ddf65, SUC_lexicon_tags_87 },
    { 0x91eddf64, SUC_lexicon_tags_30 },
    { 0x5a8ddf68, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ddf6b, SUC_lexicon_tags_12 },
    { 0xa60ddf6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde0ddf6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b2ddf70, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22ddf77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x420ddf79, SUC_lexicon_tags_12 },
    { 0xeecddf79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0addf80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x544ddf82, SUC_lexicon_tags_60 },
    { 0xdbcddf83, SUC_lexicon_tags_69 },
    { 0x9dcddf82, SUC_lexicon_tags_47 },
    { 0x854ddf85, SUC_lexicon_tags_28 },
    { 0x83cddf83, SUC_lexicon_tags_19 },
    { 0x476ddf87, SUC_lexicon_tags_60 },
    { 0xb18ddf84, SUC_lexicon_tags_62 },
    { 0x34ddf89, SUC_lexicon_tags_17 },
    { 0x736ddf89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcddf91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36ddf97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eddf9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x81addf9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58ddfa2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x548ddfa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618ddfa8, SUC_lexicon_tags_7 },
    { 0xc1addfa8, SUC_lexicon_tags_29 },
    { 0xf2ddfaa, SUC_lexicon_tags_7 },
    { 0xf48ddfaa, SUC_lexicon_tags_15 },
    { 0x752ddfac, SUC_lexicon_tags_15 },
    { 0xbe0ddfad, SUC_lexicon_tags_18 },
    { 0x98addfae, SUC_lexicon_tags_25 },
    { 0xc98ddfaf, SUC_lexicon_tags_19 },
    { 0xf12ddfaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38addfb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78ddfb8, SUC_lexicon_tags_2 },
    { 0xa5cddfb9, SUC_lexicon_tags_11 },
    { 0xdbeddfba, SUC_lexicon_tags_29 },
    { 0x78eddfbb, SUC_lexicon_tags_0 },
    { 0xe58ddfbc, SUC_lexicon_tags_151 },
    { 0xa64ddfbd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a2ddfbf, SUC_lexicon_tags_60 },
    { 0x13cddfc0, SUC_lexicon_tags_66 },
    { 0x33eddfc0, SUC_lexicon_tags_12 },
    { 0x756ddfc0, SUC_lexicon_tags_37 },
    { 0x5ceddfc3, SUC_lexicon_tags_15 },
    { 0x8aeddfc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8acddfc6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe2addfc8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cddfcd, SUC_lexicon_tags_45 },
    { 0x8f8ddfce, SUC_lexicon_tags_2 },
    { 0xd8addfcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22ddfd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcddfd4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cddfd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0ddfdd, SUC_lexicon_tags_79 },
    { 0x1daddfdd, SUC_lexicon_tags_29 },
    { 0x516ddfdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92ddfe2, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x214ddfe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6ddfe7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x112ddfea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278ddfee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x950ddff0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0ddff4, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730ddffa, SUC_lexicon_tags_26 },
    { 0x746ddffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42de001, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfeede003, SUC_lexicon_tags_25 },
    { 0x93ede004, SUC_lexicon_tags_0 },
    { 0xed2de005, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ede00a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918de015, SUC_lexicon_tags_37 },
    { 0xfb4de016, SUC_lexicon_tags_37 },
    { 0x40ade017, SUC_lexicon_tags_7 },
    { 0xe86de018, SUC_lexicon_tags_11 },
    { 0xce0de019, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2de01c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890de021, SUC_lexicon_tags_10 },
    { 0xac6de022, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd24de024, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504de02a, SUC_lexicon_tags_11 },
    { 0x998de02a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x838de02d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe32de02f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x646de032, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cde037, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88de03f, SUC_lexicon_tags_77 },
    { 0xc66de03f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570de046, SUC_lexicon_tags_11 },
    { 0x7e0de047, SUC_lexicon_tags_4 },
    { 0xa40de046, SUC_lexicon_tags_27 },
    { 0x21ede049, SUC_lexicon_tags_47 },
    { 0x10de048, SUC_lexicon_tags_11 },
    { 0x624de04b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ede053, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4de057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x166de05a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeade05e, SUC_lexicon_tags_15 },
    { 0xdd0de05f, SUC_lexicon_tags_23 },
    { 0x45cde060, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x77ade062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ede069, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x948de06c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe2de06e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2de074, SUC_lexicon_tags_37 },
    { 0x86ade074, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4de07a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ade07f, SUC_lexicon_tags_25 },
    { 0x40ade07f, SUC_lexicon_tags_1 },
    { 0xd2ede07f, SUC_lexicon_tags_14 },
    { 0xc4ade082, SUC_lexicon_tags_25 },
    { 0x4f4de083, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ade08a, SUC_lexicon_tags_56 },
    { 0x100de08a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa0cde08d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840de093, SUC_lexicon_tags_29 },
    { 0x78ede094, SUC_lexicon_tags_27 },
    { 0x3e6de094, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688de099, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeede09d, SUC_lexicon_tags_33 },
    { 0xe70de09e, SUC_lexicon_tags_3 },
    { 0x23cde09e, SUC_lexicon_tags_17 },
    { 0x320de0a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecede0a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344de0a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa2de0ab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xde2de0ad, SUC_lexicon_tags_33 },
    { 0x3a8de0ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08de0b4, SUC_lexicon_tags_27 },
    { 0xe22de0b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x396de0b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcde0bf, SUC_lexicon_tags_47 },
    { 0x948de0bf, SUC_lexicon_tags_542 },
    { 0x136de0c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288de0c5, SUC_lexicon_tags_28 },
    { 0x200de0c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cde0cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22de0d3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecde0d9, SUC_lexicon_tags_12 },
    { 0x71cde0da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24de0e0, SUC_lexicon_tags_30 },
    { 0xb98de0e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe5cde0e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x63ade0e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92de0ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ede0f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cade0f4, SUC_lexicon_tags_17 },
    { 0xacade0f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95cde0f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacde0fd, SUC_lexicon_tags_11 },
    { 0xe86de0fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5cde100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10de107, SUC_lexicon_tags_64 },
    { 0x640de108, SUC_lexicon_tags_11 },
    { 0x676de109, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdd2de10b, SUC_lexicon_tags_1 },
    { 0x250de10b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36de110, SUC_lexicon_tags_12 },
    { 0xa16de111, SUC_lexicon_tags_1 },
    { 0xb0de110, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eade115, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14de11c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cde122, SUC_lexicon_tags_17 },
    { 0x2a2de123, SUC_lexicon_tags_1 },
    { 0x6d8de124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ede128, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92de12d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x838de130, SUC_lexicon_tags_60 },
    { 0x784de130, SUC_lexicon_tags_37 },
    { 0xdb6de132, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6de134, SUC_lexicon_tags_25 },
    { 0xfacde135, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ade13b, SUC_lexicon_tags_15 },
    { 0xa4ede13c, SUC_lexicon_tags_14 },
    { 0x2aede13c, SUC_lexicon_tags_30 },
    { 0x8cade13b, SUC_lexicon_tags_32 },
    { 0x706de13f, SUC_lexicon_tags_19 },
    { 0x914de13f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6de144, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ede148, SUC_lexicon_tags_33 },
    { 0x8bcde149, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46de14b, SUC_lexicon_tags_14 },
    { 0x5dede14c, SUC_lexicon_tags_38 },
    { 0x3bcde14d, SUC_lexicon_tags_12 },
    { 0xa80de14d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192de155, SUC_lexicon_tags_15 },
    { 0xbf0de155, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0de15b, SUC_lexicon_tags_22 },
    { 0xe4ade15c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6de160, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16de16f, SUC_lexicon_tags_18 },
    { 0x47ede16f, SUC_lexicon_tags_39 },
    { 0xddcde171, SUC_lexicon_tags_8 },
    { 0xdb6de171, SUC_lexicon_tags_14 },
    { 0xd08de172, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c0de175, SUC_lexicon_tags_37 },
    { 0x1a4de176, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceede17b, SUC_lexicon_tags_15 },
    { 0xe2de17c, SUC_lexicon_tags_12 },
    { 0x7ade17c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabede182, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cde185, SUC_lexicon_tags_2 },
    { 0xb98de185, SUC_lexicon_tags_63 },
    { 0xa92de185, SUC_lexicon_tags_44 },
    { 0x4e8de187, SUC_lexicon_tags_55 },
    { 0x438de185, SUC_lexicon_tags_12 },
    { 0xe82de185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x898de18d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cde193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6de195, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914de199, SUC_lexicon_tags_55 },
    { 0xd40de19a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8de19d, SUC_lexicon_tags_7 },
    { 0xf6ade19d, SUC_lexicon_tags_60 },
    { 0xb60de19f, SUC_lexicon_tags_14 },
    { 0xe0de19d, SUC_lexicon_tags_6 },
    { 0x36de1a1, SUC_lexicon_tags_84 },
    { 0xaa0de1a0, SUC_lexicon_tags_48 },
    { 0xb38de1a3, SUC_lexicon_tags_35 },
    { 0xe5cde19d, SUC_lexicon_tags_49 },
    { 0xe82de1a5, SUC_lexicon_tags_2 },
    { 0x750de1a6, SUC_lexicon_tags_12 },
    { 0x522de1a6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56de1ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416de1b2, SUC_lexicon_tags_29 },
    { 0x9d4de1b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ade1b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x594de1b9, SUC_lexicon_tags_55 },
    { 0x252de1b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dede1bc, SUC_lexicon_tags_11 },
    { 0x3d6de1bd, SUC_lexicon_tags_15 },
    { 0x95ade1bd, SUC_lexicon_tags_37 },
    { 0x794de1be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e2de1c1, SUC_lexicon_tags_37 },
    { 0xb80de1c2, SUC_lexicon_tags_15 },
    { 0xf40de1c3, SUC_lexicon_tags_55 },
    { 0xf5cde1c4, SUC_lexicon_tags_63 },
    { 0xfacde1c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacade1ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x28ede1cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafade1d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858de1dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa66de1df, SUC_lexicon_tags_12 },
    { 0x804de1e0, SUC_lexicon_tags_21 },
    { 0x1acde1e0, SUC_lexicon_tags_0 },
    { 0xb04de1e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92de1e4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcde1e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8de1ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf7ede1ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bede1f0, SUC_lexicon_tags_7 },
    { 0x4f2de1f1, SUC_lexicon_tags_27 },
    { 0x7f6de1f0, SUC_lexicon_tags_26 },
    { 0xa56de1f3, SUC_lexicon_tags_14 },
    { 0xcb4de1f4, SUC_lexicon_tags_29 },
    { 0x156de1f5, SUC_lexicon_tags_3 },
    { 0xd70de1f3, SUC_lexicon_tags_11 },
    { 0xbcde1f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24de1fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x606de1fe, SUC_lexicon_tags_37 },
    { 0x68ede1ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ede204, SUC_lexicon_tags_49 },
    { 0xf6ade204, SUC_lexicon_tags_9 },
    { 0x5cde206, SUC_lexicon_tags_12 },
    { 0x36de207, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690de20d, SUC_lexicon_tags_7 },
    { 0x3acde20d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4de210, SUC_lexicon_tags_138 },
    { 0x8de211, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aede216, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x686de219, SUC_lexicon_tags_47 },
    { 0x674de219, SUC_lexicon_tags_11 },
    { 0x4aade21b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74de222, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x880de225, SUC_lexicon_tags_14 },
    { 0x690de225, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcde229, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924de22f, SUC_lexicon_tags_19 },
    { 0xab0de22f, SUC_lexicon_tags_30 },
    { 0xd4ade231, SUC_lexicon_tags_87 },
    { 0x4cde231, SUC_lexicon_tags_1 },
    { 0x196de231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8de236, SUC_lexicon_tags_34 },
    { 0x6c8de237, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9cde239, SUC_lexicon_tags_12 },
    { 0x736de239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x860de23c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x500de23e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ede242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ede245, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1aade247, SUC_lexicon_tags_25 },
    { 0xba8de248, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f0de24a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4f2de24c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb6ade24e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb22de250, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0de253, SUC_lexicon_tags_60 },
    { 0xfaade253, SUC_lexicon_tags_11 },
    { 0xf1cde254, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x34ede257, SUC_lexicon_tags_9 },
    { 0x702de258, SUC_lexicon_tags_46 },
    { 0x4bede259, SUC_lexicon_tags_31 },
    { 0xde8de258, SUC_lexicon_tags_2 },
    { 0xa16de258, SUC_lexicon_tags_15 },
    { 0x4de25c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6de25f, SUC_lexicon_tags_38 },
    { 0x2fcde260, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1f4de262, SUC_lexicon_tags_15 },
    { 0xff0de263, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x494de265, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dade268, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x134de26a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14de26d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x586de26f, SUC_lexicon_tags_11 },
    { 0x3d2de270, SUC_lexicon_tags_25 },
    { 0x65ede271, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d2de273, SUC_lexicon_tags_104 },
    { 0x726de274, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cde27a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660de27e, SUC_lexicon_tags_17 },
    { 0x812de27f, SUC_lexicon_tags_8 },
    { 0xc42de280, SUC_lexicon_tags_66 },
    { 0x77ade27f, SUC_lexicon_tags_12 },
    { 0x6ede282, SUC_lexicon_tags_7 },
    { 0xffcde281, SUC_lexicon_tags_87 },
    { 0xee2de281, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708de288, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08de291, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a6de293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cade298, SUC_lexicon_tags_51 },
    { 0xf6de299, SUC_lexicon_tags_16 },
    { 0x5c0de299, SUC_lexicon_tags_49 },
    { 0xbaede29a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaade29e, SUC_lexicon_tags_11 },
    { 0x70de29f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962de2a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x622de2a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x876de2a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84de2ae, SUC_lexicon_tags_46 },
    { 0x2bede2af, SUC_lexicon_tags_2 },
    { 0x1acde2ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00de2b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374de2b9, SUC_lexicon_tags_12 },
    { 0x9aede2b9, SUC_lexicon_tags_47 },
    { 0x772de2bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cde2be, SUC_lexicon_tags_35 },
    { 0xfc4de2bf, SUC_lexicon_tags_11 },
    { 0x56de2c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9cde2c2, SUC_lexicon_tags_17 },
    { 0x8aede2c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x77ede2c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2cede2c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb2de2c9, SUC_lexicon_tags_14 },
    { 0x13ede2ca, SUC_lexicon_tags_63 },
    { 0x1a4de2ca, SUC_lexicon_tags_60 },
    { 0x82cde2cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40de2cf, SUC_lexicon_tags_79 },
    { 0xe16de2d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdc8de2d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcede2d7, SUC_lexicon_tags_25 },
    { 0x75ade2d8, SUC_lexicon_tags_63 },
    { 0xc92de2d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x984de2dc, SUC_lexicon_tags_27 },
    { 0x708de2dc, SUC_lexicon_tags_5 },
    { 0xccade2de, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x456de2e1, SUC_lexicon_tags_25 },
    { 0x4e6de2e2, SUC_lexicon_tags_23 },
    { 0x3a8de2e3, SUC_lexicon_tags_25 },
    { 0x472de2e4, SUC_lexicon_tags_15 },
    { 0xd84de2e5, SUC_lexicon_tags_23 },
    { 0x364de2e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854de2f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458de2f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2de2f7, SUC_lexicon_tags_68 },
    { 0xd3cde2f7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xfd8de2fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x630de2fd, SUC_lexicon_tags_25 },
    { 0x81cde2fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x70cde300, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2de303, SUC_lexicon_tags_15 },
    { 0x370de304, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xde2de307, SUC_lexicon_tags_27 },
    { 0x49ede308, SUC_lexicon_tags_17 },
    { 0x2bcde308, SUC_lexicon_tags_17 },
    { 0xe66de30a, SUC_lexicon_tags_17 },
    { 0x864de30a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x77ede30d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ede313, SUC_lexicon_tags_114 },
    { 0xc44de313, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x620de317, SUC_lexicon_tags_11 },
    { 0x368de318, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5dade31a, SUC_lexicon_tags_12 },
    { 0x93ade31b, SUC_lexicon_tags_39 },
    { 0x2b8de31c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8ede31e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd0ede320, SUC_lexicon_tags_35 },
    { 0x3b2de321, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492de325, SUC_lexicon_tags_23 },
    { 0xc46de325, SUC_lexicon_tags_91 },
    { 0xc5ade325, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dede32b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6de333, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaaade335, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2de338, SUC_lexicon_tags_39 },
    { 0x284de339, SUC_lexicon_tags_26 },
    { 0xb40de33a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2de33d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb94de33f, SUC_lexicon_tags_60 },
    { 0x882de340, SUC_lexicon_tags_25 },
    { 0xaf2de33f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x320de343, SUC_lexicon_tags_11 },
    { 0xf48de343, SUC_lexicon_tags_9 },
    { 0xb20de345, SUC_lexicon_tags_6 },
    { 0x222de346, SUC_lexicon_tags_50 },
    { 0x172de347, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ade34c, SUC_lexicon_tags_11 },
    { 0xa56de34d, SUC_lexicon_tags_37 },
    { 0x6f4de34e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9aede350, SUC_lexicon_tags_14 },
    { 0x264de350, SUC_lexicon_tags_25 },
    { 0x4f6de351, SUC_lexicon_tags_25 },
    { 0x4e8de353, SUC_lexicon_tags_25 },
    { 0xeacde351, SUC_lexicon_tags_17 },
    { 0xf9cde352, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x426de358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec2de35a, SUC_lexicon_tags_53 },
    { 0xb80de35a, SUC_lexicon_tags_11 },
    { 0xf2de35b, SUC_lexicon_tags_28 },
    { 0x352de35d, SUC_lexicon_tags_11 },
    { 0x544de35d, SUC_lexicon_tags_23 },
    { 0xc88de35f, SUC_lexicon_tags_84 },
    { 0x65ede35e, SUC_lexicon_tags_79 },
    { 0x346de35d, SUC_lexicon_tags_15 },
    { 0x672de35e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ade368, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4de36b, SUC_lexicon_tags_14 },
    { 0xec4de36c, SUC_lexicon_tags_11 },
    { 0xd24de36d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cade372, SUC_lexicon_tags_23 },
    { 0x2ecde373, SUC_lexicon_tags_38 },
    { 0xa6ade374, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60de37b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844de37f, SUC_lexicon_tags_63 },
    { 0xc3cde380, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76de386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60de389, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88de38c, SUC_lexicon_tags_57 },
    { 0xd7ade38c, SUC_lexicon_tags_37 },
    { 0x1d8de38e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ade391, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4de395, SUC_lexicon_tags_49 },
    { 0x714de396, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2de39d, SUC_lexicon_tags_27 },
    { 0x826de39e, SUC_lexicon_tags_14 },
    { 0x2a0de39e, SUC_lexicon_tags_9 },
    { 0x728de39d, SUC_lexicon_tags_11 },
    { 0xaecde3a0, SUC_lexicon_tags_77 },
    { 0x6dcde3a2, SUC_lexicon_tags_25 },
    { 0x2eede3a2, SUC_lexicon_tags_25 },
    { 0x30ede3a4, SUC_lexicon_tags_17 },
    { 0xaede3a5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6de3a8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ede3af, SUC_lexicon_tags_23 },
    { 0xa78de3b0, SUC_lexicon_tags_13 },
    { 0x690de3b0, SUC_lexicon_tags_11 },
    { 0x5fade3b2, SUC_lexicon_tags_133 },
    { 0xc86de3b0, SUC_lexicon_tags_1 },
    { 0x962de3b0, SUC_lexicon_tags_7 },
    { 0x224de3b5, SUC_lexicon_tags_16 },
    { 0xba2de3b5, SUC_lexicon_tags_17 },
    { 0xefcde3b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad0de3b9, SUC_lexicon_tags_61 },
    { 0x6dade3b9, SUC_lexicon_tags_47 },
    { 0x89ade3bb, SUC_lexicon_tags_25 },
    { 0xee2de3b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8cde3be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cde3c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cde3c9, SUC_lexicon_tags_17 },
    { 0xfd8de3ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fede3cc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x494de3ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24ade3d0, SUC_lexicon_tags_25 },
    { 0x564de3d1, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0x9d8de3d3, SUC_lexicon_tags_1 },
    { 0x6cede3d3, SUC_lexicon_tags_484 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18de3d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x320de3da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcede3e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde4de3e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae6de3e4, SUC_lexicon_tags_17 },
    { 0x4aede3e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22de3e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8de3ed, SUC_lexicon_tags_1 },
    { 0xcbade3ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4de3f4, SUC_lexicon_tags_19 },
    { 0xebede3f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa22de3f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bade3fd, SUC_lexicon_tags_15 },
    { 0xf88de3fd, SUC_lexicon_tags_35 },
    { 0xd42de3fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68de403, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cde409, SUC_lexicon_tags_21 },
    { 0x47ade40a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64de40d, SUC_lexicon_tags_15 },
    { 0x11ede40e, SUC_lexicon_tags_37 },
    { 0x90de40e, SUC_lexicon_tags_34 },
    { 0x328de410, SUC_lexicon_tags_9 },
    { 0xe3ade40d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514de416, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cede41e, SUC_lexicon_tags_2 },
    { 0x2e8de41e, SUC_lexicon_tags_11 },
    { 0x6a0de420, SUC_lexicon_tags_37 },
    { 0x700de420, SUC_lexicon_tags_11 },
    { 0xc54de422, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf6de424, SUC_lexicon_tags_6 },
    { 0xa7ade424, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2de428, SUC_lexicon_tags_7 },
    { 0xecade428, SUC_lexicon_tags_0 },
    { 0xc46de42a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242de430, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2dade432, SUC_lexicon_tags_12 },
    { 0xb6ede432, SUC_lexicon_tags_6 },
    { 0xa6de434, SUC_lexicon_tags_79 },
    { 0x8fcde434, SUC_lexicon_tags_25 },
    { 0x5e8de436, SUC_lexicon_tags_11 },
    { 0x6c4de436, SUC_lexicon_tags_38 },
    { 0x72de438, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdcede43a, SUC_lexicon_tags_37 },
    { 0x1bcde43b, SUC_lexicon_tags_25 },
    { 0xaa0de43b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbaede43e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ede442, SUC_lexicon_tags_37 },
    { 0x3b0de443, SUC_lexicon_tags_1 },
    { 0xc1ede444, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76de44b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2de44f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ade453, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf4de455, SUC_lexicon_tags_9 },
    { 0x688de456, SUC_lexicon_tags_60 },
    { 0xd60de455, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x174de45a, SUC_lexicon_tags_37 },
    { 0x344de45b, SUC_lexicon_tags_25 },
    { 0x122de45c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x772de45f, SUC_lexicon_tags_45 },
    { 0xa92de460, SUC_lexicon_tags_45 },
    { 0x6e8de461, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670de467, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cde46c, SUC_lexicon_tags_39 },
    { 0x43ede46d, SUC_lexicon_tags_17 },
    { 0x1fcde46e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ade473, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x564de475, SUC_lexicon_tags_37 },
    { 0x81ade476, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x262de479, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4de47f, SUC_lexicon_tags_19 },
    { 0x6fede480, SUC_lexicon_tags_96 },
    { 0x216de47f, SUC_lexicon_tags_243 },
    { 0xfa8de482, SUC_lexicon_tags_37 },
    { 0x898de480, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624de487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4de48a, SUC_lexicon_tags_7 },
    { 0x36ade48b, SUC_lexicon_tags_15 },
    { 0xd04de48b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff6de48e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16de493, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ade496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84de499, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x810de49b, SUC_lexicon_tags_172 },
    { 0xc1ede49c, SUC_lexicon_tags_46 },
    { 0x2a8de49d, SUC_lexicon_tags_66 },
    { 0x4f4de49d, SUC_lexicon_tags_11 },
    { 0xfc4de49f, SUC_lexicon_tags_11 },
    { 0xed8de49b, SUC_lexicon_tags_14 },
    { 0xe8cde4a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76de4a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8cde4a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf44de4a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134de4ad, SUC_lexicon_tags_1 },
    { 0xb84de4ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e0de4b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618de4b9, SUC_lexicon_tags_29 },
    { 0x964de4b9, SUC_lexicon_tags_17 },
    { 0x5eade4bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00de4bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8ade4c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x51ade4c3, SUC_lexicon_tags_35 },
    { 0x576de4c4, SUC_lexicon_tags_11 },
    { 0xdf8de4c3, SUC_lexicon_tags_25 },
    { 0xbb6de4c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe72de4c9, SUC_lexicon_tags_46 },
    { 0x3b6de4ca, SUC_lexicon_tags_17 },
    { 0x55ade4c9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6de4cf, SUC_lexicon_tags_0 },
    { 0xcf2de4d0, SUC_lexicon_tags_82 },
    { 0x1e6de4d0, SUC_lexicon_tags_12 },
    { 0xd44de4cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x878de4d4, SUC_lexicon_tags_1 },
    { 0xff4de4d5, SUC_lexicon_tags_14 },
    { 0xe62de4d6, SUC_lexicon_tags_11 },
    { 0xccade4d6, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cde4da, SUC_lexicon_tags_15 },
    { 0x22cde4db, SUC_lexicon_tags_25 },
    { 0x50cde4db, SUC_lexicon_tags_47 },
    { 0x976de4dd, SUC_lexicon_tags_61 },
    { 0x818de4da, SUC_lexicon_tags_47 },
    { 0x9a4de4db, SUC_lexicon_tags_66 },
    { 0xa50de4e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x138de4e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0de4e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd82de4e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcde4eb, SUC_lexicon_tags_45 },
    { 0xa3cde4ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x150de4ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeede4f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ade4f4, SUC_lexicon_tags_23 },
    { 0x7eade4f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12de4f8, SUC_lexicon_tags_0 },
    { 0x87ede4f9, SUC_lexicon_tags_15 },
    { 0xd8de4f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb8de4fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ade501, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35cde503, SUC_lexicon_tags_18 },
    { 0x2ccde504, SUC_lexicon_tags_19 },
    { 0xb44de505, SUC_lexicon_tags_34 },
    { 0xb9ade505, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6de509, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee4de50b, SUC_lexicon_tags_14 },
    { 0xf96de50b, SUC_lexicon_tags_45 },
    { 0xafade50d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb36de50f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x416de511, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4de514, SUC_lexicon_tags_23 },
    { 0xafade515, SUC_lexicon_tags_26 },
    { 0xc64de516, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x126de518, SUC_lexicon_tags_61 },
    { 0xf06de518, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cde520, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91cde522, SUC_lexicon_tags_15 },
    { 0x5fade522, SUC_lexicon_tags_49 },
    { 0x588de522, SUC_lexicon_tags_29 },
    { 0xc00de523, SUC_lexicon_tags_2 },
    { 0xb6cde526, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724de52e, SUC_lexicon_tags_55 },
    { 0xe44de52e, SUC_lexicon_tags_37 },
    { 0x180de52f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14de533, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50de537, SUC_lexicon_tags_29 },
    { 0xf2ede537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x922de53a, SUC_lexicon_tags_25 },
    { 0x79cde53b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa0de53d, SUC_lexicon_tags_47 },
    { 0xceede53d, SUC_lexicon_tags_31 },
    { 0xf48de53f, SUC_lexicon_tags_7 },
    { 0x54de53f, SUC_lexicon_tags_2 },
    { 0xce6de541, SUC_lexicon_tags_1 },
    { 0xc08de541, SUC_lexicon_tags_14 },
    { 0x3fede543, SUC_lexicon_tags_17 },
    { 0x77ade544, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5eade546, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18de549, SUC_lexicon_tags_47 },
    { 0x9eede54a, SUC_lexicon_tags_47 },
    { 0x6ecde54b, SUC_lexicon_tags_25 },
    { 0x6ede54c, SUC_lexicon_tags_2 },
    { 0xa70de54d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90ede54f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0de554, SUC_lexicon_tags_25 },
    { 0x4dede555, SUC_lexicon_tags_45 },
    { 0x3f4de555, SUC_lexicon_tags_44 },
    { 0x7f6de557, SUC_lexicon_tags_12 },
    { 0xaade555, SUC_lexicon_tags_1 },
    { 0xf8de559, SUC_lexicon_tags_27 },
    { 0xfe0de55a, SUC_lexicon_tags_10 },
    { 0xafade555, SUC_lexicon_tags_86 },
    { 0x3b6de55c, SUC_lexicon_tags_15 },
    { 0x3a0de55d, SUC_lexicon_tags_11 },
    { 0xbdcde55b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x23ede560, SUC_lexicon_tags_13 },
    { 0xf00de561, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2de567, SUC_lexicon_tags_1 },
    { 0x70cde568, SUC_lexicon_tags_12 },
    { 0x9cede567, SUC_lexicon_tags_33 },
    { 0xdaade56a, SUC_lexicon_tags_2 },
    { 0x1e2de56b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2de572, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0de576, SUC_lexicon_tags_38 },
    { 0xe18de576, SUC_lexicon_tags_7 },
    { 0x58cde576, SUC_lexicon_tags_13 },
    { 0xa90de579, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x574de57b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35ede57d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654de581, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06de585, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xd30de587, SUC_lexicon_tags_17 },
    { 0x764de588, SUC_lexicon_tags_11 },
    { 0xde2de588, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72de58c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4de591, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acde595, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb98de597, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4de59c, SUC_lexicon_tags_28 },
    { 0x8a6de59c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0de5a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cde5a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24de5ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0ede5ad, SUC_lexicon_tags_117 },
    { 0xe58de5ae, SUC_lexicon_tags_11 },
    { 0x420de5af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4de5b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4de5b6, SUC_lexicon_tags_11 },
    { 0xab0de5b6, SUC_lexicon_tags_28 },
    { 0x1f0de5b8, SUC_lexicon_tags_25 },
    { 0xa8de5b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x702de5bc, SUC_lexicon_tags_14 },
    { 0x96ade5bc, SUC_lexicon_tags_14 },
    { 0xdb6de5be, SUC_lexicon_tags_25 },
    { 0x530de5be, SUC_lexicon_tags_11 },
    { 0xa3ade5c0, SUC_lexicon_tags_45 },
    { 0xec2de5c1, SUC_lexicon_tags_92 },
    { 0xa0ede5bd, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcde5c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x656de5c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaade5ca, SUC_lexicon_tags_7 },
    { 0x4d4de5cb, SUC_lexicon_tags_30 },
    { 0xa92de5cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cde5d0, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0de5d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc28de5d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ade5df, SUC_lexicon_tags_2 },
    { 0xb6de5e0, SUC_lexicon_tags_22 },
    { 0x814de5e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98de5eb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8de5ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ede5f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cde5f9, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76de5fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4de601, SUC_lexicon_tags_11 },
    { 0x66ade602, SUC_lexicon_tags_12 },
    { 0x8e0de603, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb7ade605, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x196de607, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3e4de609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x706de60c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4de612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21cde615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fade618, SUC_lexicon_tags_17 },
    { 0x6a0de619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cde61e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6de625, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ede629, SUC_lexicon_tags_25 },
    { 0xa72de62a, SUC_lexicon_tags_2 },
    { 0x84de629, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cde630, SUC_lexicon_tags_35 },
    { 0x3b8de631, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70de634, SUC_lexicon_tags_38 },
    { 0x2ecde635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96ade637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x630de639, SUC_lexicon_tags_17 },
    { 0x52ede63a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8de63f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e4de641, SUC_lexicon_tags_2 },
    { 0x618de642, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2d6de644, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906de649, SUC_lexicon_tags_114 },
    { 0x526de64a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc6ade64c, SUC_lexicon_tags_11 },
    { 0xa6de64d, SUC_lexicon_tags_2 },
    { 0x73cde64e, SUC_lexicon_tags_33 },
    { 0x294de64d, SUC_lexicon_tags_12 },
    { 0xb24de64c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82de656, SUC_lexicon_tags_39 },
    { 0xbbade657, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45ade659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x880de65c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cde660, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806de664, SUC_lexicon_tags_37 },
    { 0x46ade664, SUC_lexicon_tags_47 },
    { 0xe22de666, SUC_lexicon_tags_29 },
    { 0xeaede666, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc08de669, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa14de66b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0de66f, SUC_lexicon_tags_37 },
    { 0x8fade670, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8de677, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ade67a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7ade67c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624de68a, SUC_lexicon_tags_8 },
    { 0x858de68a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cde68e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8de693, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8de69a, SUC_lexicon_tags_7 },
    { 0x7dcde69b, SUC_lexicon_tags_14 },
    { 0xd08de69c, SUC_lexicon_tags_10 },
    { 0xda2de69d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6ede69f, SUC_lexicon_tags_66 },
    { 0x422de6a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0de6a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8de6a8, SUC_lexicon_tags_4 },
    { 0xfcade6a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6de6ad, SUC_lexicon_tags_14 },
    { 0x34cde6ae, SUC_lexicon_tags_23 },
    { 0xe20de6af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8de6b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafade6ba, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc9cde6bc, SUC_lexicon_tags_27 },
    { 0xe8cde6bd, SUC_lexicon_tags_2 },
    { 0xf9cde6be, SUC_lexicon_tags_8 },
    { 0x152de6bc, SUC_lexicon_tags_47 },
    { 0x5d6de6c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62de6c3, SUC_lexicon_tags_0 },
    { 0xde2de6c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x712de6c6, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66de6cb, SUC_lexicon_tags_13 },
    { 0xec4de6cc, SUC_lexicon_tags_1 },
    { 0xce0de6cd, SUC_lexicon_tags_26 },
    { 0x512de6ce, SUC_lexicon_tags_7 },
    { 0xf34de6cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ede6d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd8ede6d5, SUC_lexicon_tags_12 },
    { 0xb2de6d6, SUC_lexicon_tags_27 },
    { 0x396de6d7, SUC_lexicon_tags_6 },
    { 0x244de6d8, SUC_lexicon_tags_21 },
    { 0xe66de6d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdb8de6db, SUC_lexicon_tags_37 },
    { 0x37ade6dc, SUC_lexicon_tags_11 },
    { 0xf18de6dd, SUC_lexicon_tags_25 },
    { 0x666de6dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa3cde6e0, SUC_lexicon_tags_55 },
    { 0x6f6de6e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xca8de6e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ade6e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xede6eb, SUC_lexicon_tags_12 },
    { 0xadade6eb, SUC_lexicon_tags_17 },
    { 0x8a4de6ed, SUC_lexicon_tags_4 },
    { 0x828de6ee, SUC_lexicon_tags_1 },
    { 0x6aade6ee, SUC_lexicon_tags_1 },
    { 0x7c2de6f0, SUC_lexicon_tags_29 },
    { 0x9c4de6f0, SUC_lexicon_tags_15 },
    { 0x4d4de6f2, SUC_lexicon_tags_45 },
    { 0x3a6de6f1, SUC_lexicon_tags_37 },
    { 0x816de6f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4b2de6f6, SUC_lexicon_tags_1 },
    { 0x794de6f7, SUC_lexicon_tags_13 },
    { 0xa2ade6f8, SUC_lexicon_tags_6 },
    { 0xe8ede6f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cde6fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952de701, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x680de704, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862de70b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe74de70d, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x150de70f, SUC_lexicon_tags_17 },
    { 0x86de710, SUC_lexicon_tags_39 },
    { 0x610de710, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796de717, SUC_lexicon_tags_23 },
    { 0x1dcde718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4cde71a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4de71d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4fade71f, SUC_lexicon_tags_15 },
    { 0xe72de720, SUC_lexicon_tags_23 },
    { 0x8d6de721, SUC_lexicon_tags_62 },
    { 0xc88de721, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8de72e, SUC_lexicon_tags_0 },
    { 0x6a8de72f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x20de731, SUC_lexicon_tags_12 },
    { 0x556de732, SUC_lexicon_tags_38 },
    { 0x5b8de733, SUC_lexicon_tags_7 },
    { 0x432de733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44de73a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9f6de73c, SUC_lexicon_tags_9 },
    { 0x4e8de73d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x676de73f, SUC_lexicon_tags_66 },
    { 0x8a4de740, SUC_lexicon_tags_37 },
    { 0xc8ade741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe62de743, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ade747, SUC_lexicon_tags_26 },
    { 0x96ade748, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b4de74a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcecde74d, SUC_lexicon_tags_64 },
    { 0x918de74e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x170de750, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876de754, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9a4de756, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0de758, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22de75c, SUC_lexicon_tags_15 },
    { 0xb64de75d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x758de760, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc38de762, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8ade764, SUC_lexicon_tags_63 },
    { 0x490de765, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d0de767, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cde76a, SUC_lexicon_tags_23 },
    { 0xd92de76b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x766de76d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbede771, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x340de773, SUC_lexicon_tags_37 },
    { 0xf8de773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x564de777, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87cde77a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0de780, SUC_lexicon_tags_1 },
    { 0x5ede781, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6de785, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ade789, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6de78d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852de791, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218de798, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb82de79a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ede79e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2de7a4, SUC_lexicon_tags_17 },
    { 0x55ade7a5, SUC_lexicon_tags_25 },
    { 0x15ede7a5, SUC_lexicon_tags_1 },
    { 0xf42de7a4, SUC_lexicon_tags_87 },
    { 0x910de7a7, SUC_lexicon_tags_37 },
    { 0x8cade7a5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00de7ad, SUC_lexicon_tags_37 },
    { 0x77ede7ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852de7b3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98de7b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218de7ba, SUC_lexicon_tags_8 },
    { 0xc8ede7bb, SUC_lexicon_tags_37 },
    { 0xaaade7ba, SUC_lexicon_tags_32 },
    { 0xb16de7ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37ade7bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc4ede7c1, SUC_lexicon_tags_47 },
    { 0xe2de7c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3cde7c4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74de7c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x358de7c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xceade7cc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x130de7cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194de7d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442de7d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ede7e1, SUC_lexicon_tags_27 },
    { 0x734de7e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504de7e6, SUC_lexicon_tags_12 },
    { 0x922de7e7, SUC_lexicon_tags_6 },
    { 0xfcade7e8, SUC_lexicon_tags_37 },
    { 0x82de7e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ade7ee, SUC_lexicon_tags_12 },
    { 0xd58de7ee, SUC_lexicon_tags_57 },
    { 0x770de7f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfc0de7f2, SUC_lexicon_tags_27 },
    { 0x86ade7f3, SUC_lexicon_tags_56 },
    { 0x23ade7f4, SUC_lexicon_tags_138 },
    { 0xbd4de7f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x950de7f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80de7fa, SUC_lexicon_tags_63 },
    { 0x76ade7fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x418de7fe, SUC_lexicon_tags_10 },
    { 0xf2de7ff, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0de804, SUC_lexicon_tags_55 },
    { 0x60ede805, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21ade807, SUC_lexicon_tags_14 },
    { 0x714de808, SUC_lexicon_tags_68 },
    { 0xa18de807, SUC_lexicon_tags_37 },
    { 0xceade807, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cde80e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x890de810, SUC_lexicon_tags_70 },
    { 0x4d6de810, SUC_lexicon_tags_37 },
    { 0xfeede812, SUC_lexicon_tags_53 },
    { 0x2eede813, SUC_lexicon_tags_7 },
    { 0x98cde814, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ede819, SUC_lexicon_tags_30 },
    { 0x7dede819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x898de81c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x9bcde81e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98de822, SUC_lexicon_tags_7 },
    { 0x502de823, SUC_lexicon_tags_25 },
    { 0x362de823, SUC_lexicon_tags_29 },
    { 0xa54de825, SUC_lexicon_tags_2 },
    { 0x4a6de825, SUC_lexicon_tags_11 },
    { 0xc72de823, SUC_lexicon_tags_11 },
    { 0x266de823, SUC_lexicon_tags_70 },
    { 0x718de827, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722de82e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80de833, SUC_lexicon_tags_25 },
    { 0x618de833, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2de837, SUC_lexicon_tags_17 },
    { 0x1cede837, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75ade83a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf74de83c, SUC_lexicon_tags_88 },
    { 0xe7ede83c, SUC_lexicon_tags_25 },
    { 0xb7cde83e, SUC_lexicon_tags_6 },
    { 0x7d6de83f, SUC_lexicon_tags_28 },
    { 0xd6de83f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18de846, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0de84c, SUC_lexicon_tags_9 },
    { 0xe0cde84d, SUC_lexicon_tags_33 },
    { 0x48ade84e, SUC_lexicon_tags_11 },
    { 0xe56de84e, SUC_lexicon_tags_37 },
    { 0xca2de84e, SUC_lexicon_tags_17 },
    { 0xe30de851, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f4de853, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf12de855, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55cde857, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x662de85a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616de860, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec6de862, SUC_lexicon_tags_37 },
    { 0x7dade863, SUC_lexicon_tags_49 },
    { 0xb20de864, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb8ede866, SUC_lexicon_tags_47 },
    { 0xb88de866, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccde86a, SUC_lexicon_tags_107 },
    { 0x724de86b, SUC_lexicon_tags_25 },
    { 0xc72de86c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc18de86e, SUC_lexicon_tags_47 },
    { 0x21cde86f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4de872, SUC_lexicon_tags_23 },
    { 0xf40de873, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ade879, SUC_lexicon_tags_43 },
    { 0xc2ede87a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5bade87c, SUC_lexicon_tags_15 },
    { 0x2d0de87c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6de881, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc9ade883, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x38de886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x722de888, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0de890, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ede898, SUC_lexicon_tags_257 },
    { 0x6f0de899, SUC_lexicon_tags_17 },
    { 0x204de899, SUC_lexicon_tags_86 },
    { 0x13ede898, SUC_lexicon_tags_7 },
    { 0xd56de898, SUC_lexicon_tags_15 },
    { 0xe2ede89c, SUC_lexicon_tags_12 },
    { 0x354de89e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dade8a3, SUC_lexicon_tags_23 },
    { 0xad8de8a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ede8aa, SUC_lexicon_tags_1 },
    { 0x5e6de8ab, SUC_lexicon_tags_23 },
    { 0xbf6de8ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51ede8ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cde8b1, SUC_lexicon_tags_17 },
    { 0xc0ede8b2, SUC_lexicon_tags_25 },
    { 0x75ede8b3, SUC_lexicon_tags_16 },
    { 0xbc0de8b3, SUC_lexicon_tags_88 },
    { 0xddade8b2, SUC_lexicon_tags_37 },
    { 0x652de8b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2de8bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x876de8bf, SUC_lexicon_tags_33 },
    { 0xfc6de8c0, SUC_lexicon_tags_37 },
    { 0xa3ade8c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cde8c4, SUC_lexicon_tags_60 },
    { 0x7e6de8c4, SUC_lexicon_tags_55 },
    { 0x788de8c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f8de8c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a2de8ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x668de8cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x16cde8cf, SUC_lexicon_tags_11 },
    { 0xac6de8d0, SUC_lexicon_tags_14 },
    { 0x21cde8d0, SUC_lexicon_tags_33 },
    { 0x10de8cf, SUC_lexicon_tags_49 },
    { 0x9fede8d3, SUC_lexicon_tags_2 },
    { 0xf9cde8d4, SUC_lexicon_tags_17 },
    { 0xa1ede8d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bade8e0, SUC_lexicon_tags_7 },
    { 0xbdcde8e1, SUC_lexicon_tags_25 },
    { 0x122de8e1, SUC_lexicon_tags_11 },
    { 0xcd2de8e3, SUC_lexicon_tags_151 },
    { 0xad4de8e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48de8ea, SUC_lexicon_tags_38 },
    { 0x48ede8eb, SUC_lexicon_tags_27 },
    { 0xa20de8ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x644de8ef, SUC_lexicon_tags_10 },
    { 0xeb0de8f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84de8f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x376de8f8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4bade8fa, SUC_lexicon_tags_25 },
    { 0xdcade8fb, SUC_lexicon_tags_24 },
    { 0x282de8fa, SUC_lexicon_tags_11 },
    { 0xf5ede8fd, SUC_lexicon_tags_15 },
    { 0xdf2de8fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2ede900, SUC_lexicon_tags_43 },
    { 0xfd4de901, SUC_lexicon_tags_2 },
    { 0xdd2de900, SUC_lexicon_tags_30 },
    { 0xff4de901, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ade909, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5c0de90b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786de90f, SUC_lexicon_tags_3 },
    { 0xb7cde910, SUC_lexicon_tags_7 },
    { 0x1e6de910, SUC_lexicon_tags_11 },
    { 0x58cde911, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8de915, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ede91b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38de921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd6de923, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540de927, SUC_lexicon_tags_125 },
    { 0x5aade928, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2de92e, SUC_lexicon_tags_26 },
    { 0x572de92f, SUC_lexicon_tags_44 },
    { 0x4cade930, SUC_lexicon_tags_19 },
    { 0x86ade92f, SUC_lexicon_tags_1 },
    { 0xdfcde932, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ede935, SUC_lexicon_tags_60 },
    { 0x378de936, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dede93e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcede948, SUC_lexicon_tags_27 },
    { 0x76de949, SUC_lexicon_tags_2 },
    { 0x59ede94a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x446de94c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ade94f, SUC_lexicon_tags_33 },
    { 0xfe4de950, SUC_lexicon_tags_23 },
    { 0x3f8de951, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4aede953, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ade95c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x406de95e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2de962, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe34de964, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x95ade966, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ede96b, SUC_lexicon_tags_34 },
    { 0xb6de96b, SUC_lexicon_tags_224 },
    { 0xc52de96d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198de97c, SUC_lexicon_tags_11 },
    { 0x580de97d, SUC_lexicon_tags_27 },
    { 0x6cde97e, SUC_lexicon_tags_37 },
    { 0xf2ede97c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ede982, SUC_lexicon_tags_45 },
    { 0x716de983, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabede988, SUC_lexicon_tags_17 },
    { 0xd66de988, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850de98e, SUC_lexicon_tags_7 },
    { 0x2a6de98e, SUC_lexicon_tags_1 },
    { 0x3f6de990, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cde998, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960de9a9, SUC_lexicon_tags_17 },
    { 0xcd6de9a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x214de9ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e4de9ae, SUC_lexicon_tags_37 },
    { 0xef8de9af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x728de9b1, SUC_lexicon_tags_59 },
    { 0x63ede9b2, SUC_lexicon_tags_15 },
    { 0x464de9b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0de9b6, SUC_lexicon_tags_8 },
    { 0xc82de9b7, SUC_lexicon_tags_37 },
    { 0x462de9b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8de9bb, SUC_lexicon_tags_25 },
    { 0x6fede9bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690de9c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cde9c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x94cde9c9, SUC_lexicon_tags_44 },
    { 0x2d4de9ca, SUC_lexicon_tags_66 },
    { 0x4dade9c9, SUC_lexicon_tags_16 },
    { 0x1eede9cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258de9d1, SUC_lexicon_tags_39 },
    { 0x132de9d2, SUC_lexicon_tags_11 },
    { 0x310de9d2, SUC_lexicon_tags_17 },
    { 0x42cde9d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6eade9d6, SUC_lexicon_tags_7 },
    { 0xc84de9d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2de9db, SUC_lexicon_tags_46 },
    { 0xa3cde9dc, SUC_lexicon_tags_12 },
    { 0xd22de9dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ecde9df, SUC_lexicon_tags_17 },
    { 0x1ede9e0, SUC_lexicon_tags_8 },
    { 0x688de9df, SUC_lexicon_tags_87 },
    { 0x66ade9e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10de9e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48de9ec, SUC_lexicon_tags_28 },
    { 0x6d2de9ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x806de9ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x142de9f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8de9f4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6de9f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12de9fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x402de9fe, SUC_lexicon_tags_12 },
    { 0xf6de9ff, SUC_lexicon_tags_32 },
    { 0xe5ede9ff, SUC_lexicon_tags_17 },
    { 0x400dea00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cadea06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82adea0a, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cdea0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdea12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x636dea14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0dea1b, SUC_lexicon_tags_111 },
    { 0xf8cdea1c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbc8dea1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69edea20, SUC_lexicon_tags_17 },
    { 0x2a2dea21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a8dea23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8dea29, SUC_lexicon_tags_45 },
    { 0x514dea29, SUC_lexicon_tags_2 },
    { 0x49cdea2a, SUC_lexicon_tags_4 },
    { 0x1b2dea2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99edea31, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x896dea34, SUC_lexicon_tags_60 },
    { 0x67adea35, SUC_lexicon_tags_37 },
    { 0x6a0dea36, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5b4dea38, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x428dea3b, SUC_lexicon_tags_2 },
    { 0xd22dea3b, SUC_lexicon_tags_37 },
    { 0xcb4dea3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6edea3f, SUC_lexicon_tags_77 },
    { 0xdc6dea40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a6dea42, SUC_lexicon_tags_82 },
    { 0x7b6dea43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeadea4b, SUC_lexicon_tags_14 },
    { 0xbfedea4c, SUC_lexicon_tags_12 },
    { 0xd6edea4d, SUC_lexicon_tags_15 },
    { 0xf3adea4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22adea52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fadea55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xef8dea57, SUC_lexicon_tags_6 },
    { 0x396dea57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8dea63, SUC_lexicon_tags_69 },
    { 0xa98dea64, SUC_lexicon_tags_6 },
    { 0xc84dea65, SUC_lexicon_tags_12 },
    { 0xe76dea66, SUC_lexicon_tags_35 },
    { 0x400dea63, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcdea6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26dea70, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cedea78, SUC_lexicon_tags_12 },
    { 0xfa0dea79, SUC_lexicon_tags_37 },
    { 0x678dea7a, SUC_lexicon_tags_11 },
    { 0xd88dea79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d8dea7d, SUC_lexicon_tags_23 },
    { 0xe06dea7e, SUC_lexicon_tags_13 },
    { 0xfdcdea7f, SUC_lexicon_tags_47 },
    { 0xd48dea80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dea85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cdea88, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262dea91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92adea93, SUC_lexicon_tags_1 },
    { 0x2aadea93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x786dea97, SUC_lexicon_tags_15 },
    { 0x2badea98, SUC_lexicon_tags_26 },
    { 0xb58dea98, SUC_lexicon_tags_11 },
    { 0x63edea98, SUC_lexicon_tags_11 },
    { 0x52edea9a, SUC_lexicon_tags_60 },
    { 0x86edea9c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x95adea9f, SUC_lexicon_tags_16 },
    { 0x3cdeaa0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x958deaa3, SUC_lexicon_tags_28 },
    { 0x7d2deaa3, SUC_lexicon_tags_15 },
    { 0x242deaa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4deaaa, SUC_lexicon_tags_32 },
    { 0x77cdeaaa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4deab5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x890deab7, SUC_lexicon_tags_45 },
    { 0xf84deab8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2e6deaba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73adeabe, SUC_lexicon_tags_11 },
    { 0x100deabf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedeac3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcdeaca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x52edeacd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2dead3, SUC_lexicon_tags_14 },
    { 0x784dead4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9cedead6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf0edead8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9edeada, SUC_lexicon_tags_11 },
    { 0xea0deada, SUC_lexicon_tags_45 },
    { 0x56edeada, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe14deade, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc6deae0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x50deae3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe84deae5, SUC_lexicon_tags_1 },
    { 0x3b4deae6, SUC_lexicon_tags_46 },
    { 0xfb6deae7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18deaeb, SUC_lexicon_tags_13 },
    { 0x440deaec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcdeaf8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8deafc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4edeafe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732deb09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4deb0d, SUC_lexicon_tags_12 },
    { 0x68edeb0e, SUC_lexicon_tags_1 },
    { 0x408deb0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaacdeb12, SUC_lexicon_tags_38 },
    { 0x826deb13, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24deb16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2deb1b, SUC_lexicon_tags_2 },
    { 0x4e2deb1c, SUC_lexicon_tags_37 },
    { 0x6c6deb1c, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360deb21, SUC_lexicon_tags_29 },
    { 0x2badeb22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78deb25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x35adeb28, SUC_lexicon_tags_11 },
    { 0xcdedeb28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8deb2e, SUC_lexicon_tags_27 },
    { 0x3a2deb2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6deb32, SUC_lexicon_tags_9 },
    { 0xd04deb33, SUC_lexicon_tags_0 },
    { 0xeb0deb32, SUC_lexicon_tags_134 },
    { 0x48deb35, SUC_lexicon_tags_17 },
    { 0x154deb36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcdeb3b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8deb40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2adeb42, SUC_lexicon_tags_11 },
    { 0x664deb43, SUC_lexicon_tags_47 },
    { 0xa14deb44, SUC_lexicon_tags_65 },
    { 0x772deb44, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60deb4f, SUC_lexicon_tags_77 },
    { 0xfd2deb50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0deb53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcdeb57, SUC_lexicon_tags_27 },
    { 0xf54deb58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b6deb5a, SUC_lexicon_tags_21 },
    { 0x3fadeb5b, SUC_lexicon_tags_35 },
    { 0x534deb5c, SUC_lexicon_tags_12 },
    { 0xb6cdeb5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908deb62, SUC_lexicon_tags_17 },
    { 0x9b6deb62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabadeb65, SUC_lexicon_tags_1 },
    { 0x6e4deb66, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4deb69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcadeb6d, SUC_lexicon_tags_43 },
    { 0xae4deb6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6deb72, SUC_lexicon_tags_1 },
    { 0x440deb73, SUC_lexicon_tags_29 },
    { 0xf60deb74, SUC_lexicon_tags_4 },
    { 0x574deb73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbadeb7b, SUC_lexicon_tags_7 },
    { 0x8dadeb7c, SUC_lexicon_tags_664 },
    { 0x65cdeb7d, SUC_lexicon_tags_27 },
    { 0xf2cdeb7e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0deb81, SUC_lexicon_tags_19 },
    { 0xc8edeb82, SUC_lexicon_tags_25 },
    { 0xd6edeb83, SUC_lexicon_tags_282 },
    { 0x2aadeb84, SUC_lexicon_tags_25 },
    { 0xc92deb85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750deb8a, SUC_lexicon_tags_1 },
    { 0x2fcdeb8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658deb94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedeb9a, SUC_lexicon_tags_9 },
    { 0x458deb9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f0deb9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6dcdeb9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608deba7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6debae, SUC_lexicon_tags_14 },
    { 0x2eadebae, SUC_lexicon_tags_43 },
    { 0xa7adebb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17edebb2, SUC_lexicon_tags_19 },
    { 0xd1edebb3, SUC_lexicon_tags_7 },
    { 0x86edebb2, SUC_lexicon_tags_179 },
    { 0xc7adebb5, SUC_lexicon_tags_26 },
    { 0x8dedebb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcdebba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x106debbc, SUC_lexicon_tags_7 },
    { 0x1b4debbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eedebc1, SUC_lexicon_tags_7 },
    { 0x124debc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x648debc5, SUC_lexicon_tags_11 },
    { 0xe4cdebc6, SUC_lexicon_tags_38 },
    { 0x8ecdebc7, SUC_lexicon_tags_0 },
    { 0x52edebc7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8debce, SUC_lexicon_tags_17 },
    { 0xa6debce, SUC_lexicon_tags_60 },
    { 0xee2debce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26debd6, SUC_lexicon_tags_25 },
    { 0x20cdebd7, SUC_lexicon_tags_47 },
    { 0xa68debd7, SUC_lexicon_tags_47 },
    { 0xae2debd8, SUC_lexicon_tags_19 },
    { 0x734debda, SUC_lexicon_tags_118 },
    { 0x9b6debdb, SUC_lexicon_tags_12 },
    { 0xafadebdc, SUC_lexicon_tags_11 },
    { 0x8badebdd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb12debdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2debe2, SUC_lexicon_tags_82 },
    { 0x902debe3, SUC_lexicon_tags_1 },
    { 0xfadebe4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24debec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd90debee, SUC_lexicon_tags_2 },
    { 0xd3edebee, SUC_lexicon_tags_0 },
    { 0x33edebf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164debfa, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcadebfd, SUC_lexicon_tags_1 },
    { 0x316debfe, SUC_lexicon_tags_25 },
    { 0x686debfd, SUC_lexicon_tags_2 },
    { 0xfb8debfd, SUC_lexicon_tags_46 },
    { 0x4b0dec01, SUC_lexicon_tags_25 },
    { 0x9fcdec02, SUC_lexicon_tags_85 },
    { 0x934debff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cdec06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0adec08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa44dec0a, SUC_lexicon_tags_25 },
    { 0x4fcdec0a, SUC_lexicon_tags_37 },
    { 0x47adec0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56adec14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eedec18, SUC_lexicon_tags_37 },
    { 0xbbadec19, SUC_lexicon_tags_11 },
    { 0x9f6dec19, SUC_lexicon_tags_30 },
    { 0xa2cdec1b, SUC_lexicon_tags_27 },
    { 0x3d6dec1b, SUC_lexicon_tags_11 },
    { 0xed0dec1d, SUC_lexicon_tags_37 },
    { 0x4ccdec1d, SUC_lexicon_tags_2 },
    { 0x354dec1a, SUC_lexicon_tags_7 },
    { 0x138dec1c, SUC_lexicon_tags_7 },
    { 0xdb8dec21, SUC_lexicon_tags_45 },
    { 0xbecdec1d, SUC_lexicon_tags_37 },
    { 0x598dec20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38dec28, SUC_lexicon_tags_11 },
    { 0xda6dec29, SUC_lexicon_tags_40 },
    { 0xa2edec28, SUC_lexicon_tags_17 },
    { 0xa6edec29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79adec31, SUC_lexicon_tags_29 },
    { 0x64dec32, SUC_lexicon_tags_11 },
    { 0xe4edec33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7d0dec35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x632dec37, SUC_lexicon_tags_60 },
    { 0x7b2dec37, SUC_lexicon_tags_37 },
    { 0xf90dec39, SUC_lexicon_tags_37 },
    { 0x6dadec37, SUC_lexicon_tags_0 },
    { 0xb6cdec39, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94edec44, SUC_lexicon_tags_37 },
    { 0x6bedec45, SUC_lexicon_tags_45 },
    { 0xafedec45, SUC_lexicon_tags_9 },
    { 0x536dec47, SUC_lexicon_tags_26 },
    { 0x8e2dec45, SUC_lexicon_tags_10 },
    { 0x4badec44, SUC_lexicon_tags_31 },
    { 0xaedec48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cdec50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x570dec53, SUC_lexicon_tags_63 },
    { 0x20adec54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8cedec56, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890dec64, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeaadec66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd82dec68, SUC_lexicon_tags_39 },
    { 0x1bcdec68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02dec6f, SUC_lexicon_tags_1 },
    { 0xf98dec70, SUC_lexicon_tags_16 },
    { 0x30dec6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29adec73, SUC_lexicon_tags_19 },
    { 0x56dec74, SUC_lexicon_tags_17 },
    { 0x230dec75, SUC_lexicon_tags_82 },
    { 0xfe6dec76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x59cdec78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72dec7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd10dec7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcdec81, SUC_lexicon_tags_37 },
    { 0x340dec82, SUC_lexicon_tags_1 },
    { 0x848dec83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68edec88, SUC_lexicon_tags_68 },
    { 0x606dec89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x176dec8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x342dec8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdec93, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x37adec96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352dec9d, SUC_lexicon_tags_11 },
    { 0xda0dec9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x690deca0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x594deca2, SUC_lexicon_tags_11 },
    { 0x526deca3, SUC_lexicon_tags_31 },
    { 0x8d4deca4, SUC_lexicon_tags_9 },
    { 0x92cdeca5, SUC_lexicon_tags_43 },
    { 0xe4deca6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2deca9, SUC_lexicon_tags_1 },
    { 0xadcdecaa, SUC_lexicon_tags_37 },
    { 0xabcdecaa, SUC_lexicon_tags_12 },
    { 0xfcadecac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcdecbb, SUC_lexicon_tags_17 },
    { 0xb88decbc, SUC_lexicon_tags_63 },
    { 0xca8decbd, SUC_lexicon_tags_42 },
    { 0xedadecbe, SUC_lexicon_tags_104 },
    { 0x704decbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1badecc3, SUC_lexicon_tags_15 },
    { 0x5bcdecc4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94decca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3c0deccc, SUC_lexicon_tags_29 },
    { 0x986deccd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1edecd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134decd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2decdb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1adecde, SUC_lexicon_tags_25 },
    { 0xe5adecde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fedece2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88dece7, SUC_lexicon_tags_47 },
    { 0xcfcdece8, SUC_lexicon_tags_15 },
    { 0x9e0dece8, SUC_lexicon_tags_23 },
    { 0xf62decea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x960deced, SUC_lexicon_tags_15 },
    { 0xcecdecee, SUC_lexicon_tags_30 },
    { 0xfdedecef, SUC_lexicon_tags_7 },
    { 0x5f6decee, SUC_lexicon_tags_39 },
    { 0xa2decf1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadecf4, SUC_lexicon_tags_19 },
    { 0xc2edecf5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6decf8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd6cdecfa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbe6decfc, SUC_lexicon_tags_7 },
    { 0xb62decfc, SUC_lexicon_tags_47 },
    { 0xc74decfd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x670ded01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2ded05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaded0f, SUC_lexicon_tags_37 },
    { 0x44ded10, SUC_lexicon_tags_63 },
    { 0xbeded0f, SUC_lexicon_tags_15 },
    { 0x956ded12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8ded17, SUC_lexicon_tags_32 },
    { 0x7cded18, SUC_lexicon_tags_29 },
    { 0xcaeded18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x828ded1b, SUC_lexicon_tags_25 },
    { 0x49aded1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa72ded1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe32ded20, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acded25, SUC_lexicon_tags_1 },
    { 0x4b2ded26, SUC_lexicon_tags_47 },
    { 0x46cded26, SUC_lexicon_tags_107 },
    { 0xcb2ded28, SUC_lexicon_tags_19 },
    { 0x8a4ded28, SUC_lexicon_tags_25 },
    { 0x2f6ded29, SUC_lexicon_tags_15 },
    { 0xa7eded2b, SUC_lexicon_tags_31 },
    { 0xb8ded2c, SUC_lexicon_tags_15 },
    { 0xf2cded2c, SUC_lexicon_tags_47 },
    { 0x954ded25, SUC_lexicon_tags_56 },
    { 0x3faded2a, SUC_lexicon_tags_15 },
    { 0x6c0ded2c, SUC_lexicon_tags_15 },
    { 0x8f8ded31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc28ded33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44aded35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60ded38, SUC_lexicon_tags_12 },
    { 0xa4aded38, SUC_lexicon_tags_10 },
    { 0xd4aded39, SUC_lexicon_tags_11 },
    { 0x64aded3b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x184ded3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2eded3f, SUC_lexicon_tags_14 },
    { 0xae0ded40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cded43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x232ded46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6ded4c, SUC_lexicon_tags_27 },
    { 0xfecded4c, SUC_lexicon_tags_1114 },
    { 0xe12ded4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3daded50, SUC_lexicon_tags_11 },
    { 0x282ded50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ded55, SUC_lexicon_tags_30 },
    { 0x7ccded56, SUC_lexicon_tags_7 },
    { 0x7daded56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecded5a, SUC_lexicon_tags_1 },
    { 0x474ded5a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x1ccded5d, SUC_lexicon_tags_61 },
    { 0x102ded5e, SUC_lexicon_tags_15 },
    { 0x79aded5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18ded62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5f6ded64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4ded69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9eded6b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x788ded6e, SUC_lexicon_tags_23 },
    { 0x12cded6f, SUC_lexicon_tags_60 },
    { 0x100ded70, SUC_lexicon_tags_51 },
    { 0x2a2ded70, SUC_lexicon_tags_6 },
    { 0xa42ded70, SUC_lexicon_tags_87 },
    { 0x83cded72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366ded7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1aded89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x89aded8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4ded8e, SUC_lexicon_tags_31 },
    { 0xd36ded8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f6ded91, SUC_lexicon_tags_16 },
    { 0x9f2ded92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2ded96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x616ded98, SUC_lexicon_tags_2 },
    { 0x9b4ded98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3aded9c, SUC_lexicon_tags_2 },
    { 0x59aded9d, SUC_lexicon_tags_16 },
    { 0x1eaded9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2deda1, SUC_lexicon_tags_1 },
    { 0xb9ededa2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6e0deda4, SUC_lexicon_tags_8 },
    { 0xf62deda4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4dedaa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x96adedac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cededaf, SUC_lexicon_tags_11 },
    { 0xeccdedaf, SUC_lexicon_tags_11 },
    { 0x576dedb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4adedb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x32dedb5, SUC_lexicon_tags_37 },
    { 0xc9ededb6, SUC_lexicon_tags_64 },
    { 0xdcdedb7, SUC_lexicon_tags_25 },
    { 0xa6cdedb6, SUC_lexicon_tags_11 },
    { 0xde2dedb7, SUC_lexicon_tags_25 },
    { 0x1f0dedb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67adedbe, SUC_lexicon_tags_12 },
    { 0x25cdedbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222dedc3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258dedce, SUC_lexicon_tags_37 },
    { 0xd2ededcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4dedd2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe46dedd4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88dedd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x648dedda, SUC_lexicon_tags_25 },
    { 0x83ededdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92edede0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x15edede2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0dede8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcf0dedea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cdeded, SUC_lexicon_tags_14 },
    { 0x4bcdedee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff6dedf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbadedf3, SUC_lexicon_tags_345 },
    { 0xa1cdedf4, SUC_lexicon_tags_28 },
    { 0xbecdedf3, SUC_lexicon_tags_4 },
    { 0x998dedf4, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0x376dedf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd2dedfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52dedfd, SUC_lexicon_tags_103 },
    { 0xb26dedfe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4dee01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x778dee04, SUC_lexicon_tags_87 },
    { 0x570dee04, SUC_lexicon_tags_8 },
    { 0x1adee06, SUC_lexicon_tags_11 },
    { 0x69edee06, SUC_lexicon_tags_66 },
    { 0xf72dee07, SUC_lexicon_tags_14 },
    { 0xdf8dee06, SUC_lexicon_tags_15 },
    { 0xa3edee09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x632dee0c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0dee0f, SUC_lexicon_tags_43 },
    { 0x884dee0f, SUC_lexicon_tags_19 },
    { 0x266dee10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12dee15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1edee17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aedee1c, SUC_lexicon_tags_63 },
    { 0x9b2dee1d, SUC_lexicon_tags_11 },
    { 0x3b0dee1e, SUC_lexicon_tags_109 },
    { 0xc38dee1e, SUC_lexicon_tags_14 },
    { 0x65edee1e, SUC_lexicon_tags_25 },
    { 0x20adee1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ecdee23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcdee2b, SUC_lexicon_tags_7 },
    { 0x58dee2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fedee2f, SUC_lexicon_tags_89 },
    { 0x58adee30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67edee36, SUC_lexicon_tags_45 },
    { 0x46edee37, SUC_lexicon_tags_112 },
    { 0x406dee38, SUC_lexicon_tags_15 },
    { 0xb1cdee39, SUC_lexicon_tags_12 },
    { 0xaacdee3a, SUC_lexicon_tags_91 },
    { 0x154dee39, SUC_lexicon_tags_38 },
    { 0x98edee3c, SUC_lexicon_tags_7 },
    { 0x5b6dee3d, SUC_lexicon_tags_213 },
    { 0x6a0dee3c, SUC_lexicon_tags_38 },
    { 0x8aadee3f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x602dee42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cdee46, SUC_lexicon_tags_15 },
    { 0x38edee46, SUC_lexicon_tags_46 },
    { 0xf5cdee48, SUC_lexicon_tags_17 },
    { 0x166dee46, SUC_lexicon_tags_12 },
    { 0xde0dee4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2dee4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38dee52, SUC_lexicon_tags_1 },
    { 0x620dee52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3c2dee55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcdee5f, SUC_lexicon_tags_44 },
    { 0x802dee60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2dee66, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590dee6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65cdee6d, SUC_lexicon_tags_23 },
    { 0x886dee6e, SUC_lexicon_tags_19 },
    { 0xc8edee6f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x35adee71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dee74, SUC_lexicon_tags_11 },
    { 0x744dee75, SUC_lexicon_tags_37 },
    { 0x41cdee75, SUC_lexicon_tags_38 },
    { 0x478dee77, SUC_lexicon_tags_29 },
    { 0x2d0dee78, SUC_lexicon_tags_15 },
    { 0x9fcdee79, SUC_lexicon_tags_11 },
    { 0xbeedee79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf68dee7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0dee80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd64dee82, SUC_lexicon_tags_37 },
    { 0xa2cdee83, SUC_lexicon_tags_15 },
    { 0x56adee84, SUC_lexicon_tags_9 },
    { 0x956dee85, SUC_lexicon_tags_8 },
    { 0xa30dee86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cdee8b, SUC_lexicon_tags_26 },
    { 0x606dee8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x340dee8e, SUC_lexicon_tags_27 },
    { 0xe5cdee8f, SUC_lexicon_tags_1 },
    { 0x28cdee8f, SUC_lexicon_tags_37 },
    { 0x2b0dee90, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xef0dee93, SUC_lexicon_tags_38 },
    { 0xc94dee93, SUC_lexicon_tags_61 },
    { 0xd16dee94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed0dee97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x810dee99, SUC_lexicon_tags_37 },
    { 0x4dadee9a, SUC_lexicon_tags_11 },
    { 0x504dee9b, SUC_lexicon_tags_14 },
    { 0x42cdee9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8deea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68deea4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc8deea6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68deead, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06deeb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28adeeb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cdeebb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x596deebe, SUC_lexicon_tags_37 },
    { 0xfcdeebe, SUC_lexicon_tags_25 },
    { 0x6a2deec0, SUC_lexicon_tags_37 },
    { 0x90edeec1, SUC_lexicon_tags_47 },
    { 0x110deec0, SUC_lexicon_tags_25 },
    { 0x334deec1, SUC_lexicon_tags_3 },
    { 0x7e0deec1, SUC_lexicon_tags_19 },
    { 0xd9edeec1, SUC_lexicon_tags_12 },
    { 0xafadeec6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13edeecc, SUC_lexicon_tags_11 },
    { 0x97cdeecc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38deed6, SUC_lexicon_tags_25 },
    { 0xed4deed6, SUC_lexicon_tags_27 },
    { 0xf28deed8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbedeeda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8deee2, SUC_lexicon_tags_1 },
    { 0x75edeee3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x410deee6, SUC_lexicon_tags_2 },
    { 0xd6cdeee7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4deeeb, SUC_lexicon_tags_17 },
    { 0x91adeeec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcdeeef, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x26cdeef1, SUC_lexicon_tags_17 },
    { 0xa78deef2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0deef5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x974deef7, SUC_lexicon_tags_25 },
    { 0x4a2deef8, SUC_lexicon_tags_14 },
    { 0x1b0deef9, SUC_lexicon_tags_27 },
    { 0x982deef9, SUC_lexicon_tags_26 },
    { 0x354deefa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3edef00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2def0a, SUC_lexicon_tags_0 },
    { 0xbdadef0b, SUC_lexicon_tags_37 },
    { 0x3e8def0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe52def0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26edef11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28def16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8def19, SUC_lexicon_tags_43 },
    { 0x388def1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcadef1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xff6def20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adef25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0edef27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49edef2e, SUC_lexicon_tags_11 },
    { 0xe8edef2f, SUC_lexicon_tags_37 },
    { 0xc16def2f, SUC_lexicon_tags_37 },
    { 0xc32def30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe06def33, SUC_lexicon_tags_27 },
    { 0x136def34, SUC_lexicon_tags_0 },
    { 0xe5adef33, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688def3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9edef42, SUC_lexicon_tags_26 },
    { 0x45adef42, SUC_lexicon_tags_37 },
    { 0xb60def43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97cdef48, SUC_lexicon_tags_2 },
    { 0x624def49, SUC_lexicon_tags_252 },
    { 0x98adef4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480def51, SUC_lexicon_tags_10 },
    { 0x5b6def52, SUC_lexicon_tags_19 },
    { 0xc5adef51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x32edef55, SUC_lexicon_tags_2 },
    { 0x222def55, SUC_lexicon_tags_61 },
    { 0xe7edef57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadef5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4def61, SUC_lexicon_tags_32 },
    { 0xbcedef62, SUC_lexicon_tags_49 },
    { 0x92edef63, SUC_lexicon_tags_1 },
    { 0x928def62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xebcdef66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aadef6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2def71, SUC_lexicon_tags_14 },
    { 0xd86def72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2def78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59edef7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe52def7c, SUC_lexicon_tags_28 },
    { 0x2b8def7d, SUC_lexicon_tags_37 },
    { 0x6dedef7e, SUC_lexicon_tags_47 },
    { 0xa5cdef7f, SUC_lexicon_tags_9 },
    { 0xa90def80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x418def82, SUC_lexicon_tags_25 },
    { 0xd16def83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b4def85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2adef89, SUC_lexicon_tags_14 },
    { 0xffedef8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18edef8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbadef94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7edef96, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x21cdef98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x346def9a, SUC_lexicon_tags_33 },
    { 0x900def9b, SUC_lexicon_tags_48 },
    { 0x128def9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44defa0, SUC_lexicon_tags_28 },
    { 0xdf8defa1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282defa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28cdefa7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820defab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44edefad, SUC_lexicon_tags_17 },
    { 0xd90defae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaadefb1, SUC_lexicon_tags_15 },
    { 0x7c0defb2, SUC_lexicon_tags_30 },
    { 0xceadefb3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2defb7, SUC_lexicon_tags_15 },
    { 0x6c2defb8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaadefbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2edefc0, SUC_lexicon_tags_15 },
    { 0xc82defc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6aadefc3, SUC_lexicon_tags_25 },
    { 0xca6defc4, SUC_lexicon_tags_32 },
    { 0xef4defc4, SUC_lexicon_tags_11 },
    { 0x7fcdefc6, SUC_lexicon_tags_14 },
    { 0x152defc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0defca, SUC_lexicon_tags_37 },
    { 0x2dedefcb, SUC_lexicon_tags_7 },
    { 0x5c8defca, SUC_lexicon_tags_8 },
    { 0xd04defca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x904defcf, SUC_lexicon_tags_1 },
    { 0xeaadefd0, SUC_lexicon_tags_2 },
    { 0xe22defd1, SUC_lexicon_tags_42 },
    { 0xc6cdefd1, SUC_lexicon_tags_2 },
    { 0x69cdefd3, SUC_lexicon_tags_37 },
    { 0x3eedefd3, SUC_lexicon_tags_25 },
    { 0x46cdefd5, SUC_lexicon_tags_12 },
    { 0x858defcf, SUC_lexicon_tags_6 },
    { 0xa82defd7, SUC_lexicon_tags_15 },
    { 0xa82defd4, SUC_lexicon_tags_25 },
    { 0x2dedefd9, SUC_lexicon_tags_57 },
    { 0x9cdefd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4defdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cdefe5, SUC_lexicon_tags_33 },
    { 0x278defe6, SUC_lexicon_tags_29 },
    { 0x7edefe6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858defec, SUC_lexicon_tags_37 },
    { 0xcb6defed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcdeff0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0deff4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x438deff6, SUC_lexicon_tags_32 },
    { 0x4d4deff7, SUC_lexicon_tags_1 },
    { 0xd4adeff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fadeffb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcadeffe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xebcdf000, SUC_lexicon_tags_23 },
    { 0x9fadf001, SUC_lexicon_tags_7 },
    { 0xf8df002, SUC_lexicon_tags_37 },
    { 0x8aadf000, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98df007, SUC_lexicon_tags_26 },
    { 0x576df007, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x95edf00a, SUC_lexicon_tags_25 },
    { 0x7adf00b, SUC_lexicon_tags_25 },
    { 0x120df00c, SUC_lexicon_tags_77 },
    { 0x766df00d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcadf011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cdf015, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcdf019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354df01f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x24cdf022, SUC_lexicon_tags_2 },
    { 0x47edf023, SUC_lexicon_tags_25 },
    { 0xa14df023, SUC_lexicon_tags_23 },
    { 0xf06df024, SUC_lexicon_tags_2 },
    { 0xc8edf026, SUC_lexicon_tags_11 },
    { 0xa7edf026, SUC_lexicon_tags_7 },
    { 0x5cadf026, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xededf030, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4df035, SUC_lexicon_tags_21 },
    { 0x444df036, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefadf03a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6df03e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6edf042, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38df046, SUC_lexicon_tags_7 },
    { 0x2f4df046, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cdf04d, SUC_lexicon_tags_37 },
    { 0xe8cdf04e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x30edf051, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20df05b, SUC_lexicon_tags_158 },
    { 0xd0cdf05c, SUC_lexicon_tags_33 },
    { 0xce0df05d, SUC_lexicon_tags_25 },
    { 0xda4df05c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5adf060, SUC_lexicon_tags_7 },
    { 0x7fcdf061, SUC_lexicon_tags_15 },
    { 0xe88df062, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2df067, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88df06b, SUC_lexicon_tags_0 },
    { 0xc0cdf06c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8adf06f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786df078, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29adf07c, SUC_lexicon_tags_45 },
    { 0x306df07c, SUC_lexicon_tags_25 },
    { 0x734df07e, SUC_lexicon_tags_35 },
    { 0x8eedf07c, SUC_lexicon_tags_7 },
    { 0x3fadf07f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3bedf082, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cdf085, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bcdf087, SUC_lexicon_tags_46 },
    { 0xab0df087, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5badf092, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x384df095, SUC_lexicon_tags_4 },
    { 0xcd8df096, SUC_lexicon_tags_7 },
    { 0x5b6df097, SUC_lexicon_tags_14 },
    { 0x632df095, SUC_lexicon_tags_38 },
    { 0xb98df099, SUC_lexicon_tags_33 },
    { 0xc4df095, SUC_lexicon_tags_37 },
    { 0x41edf096, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5adf09e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x878df0a1, SUC_lexicon_tags_5 },
    { 0xa2edf0a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x958df0a4, SUC_lexicon_tags_46 },
    { 0x586df0a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54edf0a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa8df0a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8df0ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17adf0b4, SUC_lexicon_tags_37 },
    { 0x7f6df0b4, SUC_lexicon_tags_23 },
    { 0xca0df0b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x902df0b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8df0bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x334df0be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354df0c7, SUC_lexicon_tags_7 },
    { 0x4e4df0c8, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xd1edf0ca, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7aadf0cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0df0d6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xba4df0d8, SUC_lexicon_tags_15 },
    { 0x44edf0d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0df0df, SUC_lexicon_tags_14 },
    { 0x61edf0e0, SUC_lexicon_tags_43 },
    { 0x960df0e0, SUC_lexicon_tags_7 },
    { 0xd8cdf0e1, SUC_lexicon_tags_32 },
    { 0x44edf0e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x32cdf0e5, SUC_lexicon_tags_37 },
    { 0xd44df0e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2eedf0e8, SUC_lexicon_tags_17 },
    { 0x926df0e9, SUC_lexicon_tags_47 },
    { 0x8df0e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7f0df0ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0edf0f0, SUC_lexicon_tags_49 },
    { 0xc78df0f1, SUC_lexicon_tags_17 },
    { 0x200df0f0, SUC_lexicon_tags_12 },
    { 0x22edf0f1, SUC_lexicon_tags_27 },
    { 0x6fcdf0f4, SUC_lexicon_tags_8 },
    { 0x602df0f4, SUC_lexicon_tags_8 },
    { 0x9f0df0f6, SUC_lexicon_tags_8 },
    { 0x110df0f7, SUC_lexicon_tags_7 },
    { 0xe30df0f7, SUC_lexicon_tags_25 },
    { 0x664df0f9, SUC_lexicon_tags_66 },
    { 0xd46df0f2, SUC_lexicon_tags_19 },
    { 0x12cdf0fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c8df0fd, SUC_lexicon_tags_25 },
    { 0x7e6df0fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aedf100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5cdf102, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd64df104, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6df107, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2df10a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2df10e, SUC_lexicon_tags_15 },
    { 0x586df10e, SUC_lexicon_tags_19 },
    { 0x8adf110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf14df112, SUC_lexicon_tags_17 },
    { 0xaa6df113, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232df118, SUC_lexicon_tags_45 },
    { 0x2aadf118, SUC_lexicon_tags_11 },
    { 0xc08df119, SUC_lexicon_tags_6 },
    { 0xb74df11b, SUC_lexicon_tags_23 },
    { 0x36edf11c, SUC_lexicon_tags_47 },
    { 0x14df11d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bedf121, SUC_lexicon_tags_25 },
    { 0x288df122, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcdf128, SUC_lexicon_tags_10 },
    { 0x736df128, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x59edf12b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x87adf12e, SUC_lexicon_tags_11 },
    { 0x304df12e, SUC_lexicon_tags_1 },
    { 0x7b2df130, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x22df132, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33edf136, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xadadf138, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df13b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818df141, SUC_lexicon_tags_49 },
    { 0x802df142, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58adf144, SUC_lexicon_tags_25 },
    { 0xbdcdf145, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x406df147, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fcdf149, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x960df14b, SUC_lexicon_tags_35 },
    { 0x6b4df14b, SUC_lexicon_tags_11 },
    { 0x830df14b, SUC_lexicon_tags_43 },
    { 0x670df14b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464df152, SUC_lexicon_tags_100 },
    { 0x274df152, SUC_lexicon_tags_1 },
    { 0xf42df154, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790df159, SUC_lexicon_tags_8 },
    { 0xb52df159, SUC_lexicon_tags_12 },
    { 0xdc2df15b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56df15e, SUC_lexicon_tags_15 },
    { 0xbfedf15f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0df162, SUC_lexicon_tags_17 },
    { 0x8e6df163, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8edf165, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684df169, SUC_lexicon_tags_6 },
    { 0x1a2df16a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x988df16d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8adf171, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542df176, SUC_lexicon_tags_46 },
    { 0xe86df177, SUC_lexicon_tags_0 },
    { 0x6cadf177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x294df17b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52df183, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x11adf185, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb48df187, SUC_lexicon_tags_31 },
    { 0x65adf188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x588df18b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe74df18d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6df191, SUC_lexicon_tags_29 },
    { 0x2cdf192, SUC_lexicon_tags_10 },
    { 0x278df193, SUC_lexicon_tags_1 },
    { 0x362df194, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60df198, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62df19e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x390df1a0, SUC_lexicon_tags_37 },
    { 0xc7adf1a0, SUC_lexicon_tags_7 },
    { 0xf9adf1a2, SUC_lexicon_tags_11 },
    { 0x64df1a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64df1a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2df1b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeedf1b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x334df1b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98df1be, SUC_lexicon_tags_48 },
    { 0x10edf1bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56adf1c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcacdf1c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6df1c8, SUC_lexicon_tags_15 },
    { 0x6b8df1c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0df1cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cedf1cf, SUC_lexicon_tags_14 },
    { 0xe00df1d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46df1d9, SUC_lexicon_tags_49 },
    { 0x454df1d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x120df1dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82df1e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd2df1e3, SUC_lexicon_tags_2 },
    { 0x32edf1e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1adf1e7, SUC_lexicon_tags_106 },
    { 0xb32df1e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7adf1ea, SUC_lexicon_tags_11 },
    { 0xf92df1ea, SUC_lexicon_tags_25 },
    { 0xb6df1ec, SUC_lexicon_tags_37 },
    { 0x672df1ed, SUC_lexicon_tags_6 },
    { 0xe0cdf1ec, SUC_lexicon_tags_64 },
    { 0xf0edf1eb, SUC_lexicon_tags_1 },
    { 0x9b0df1f0, SUC_lexicon_tags_11 },
    { 0xc9edf1f1, SUC_lexicon_tags_39 },
    { 0xaf4df1f0, SUC_lexicon_tags_17 },
    { 0x2a8df1f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4df1f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xba4df1fa, SUC_lexicon_tags_12 },
    { 0x720df1fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x962df1fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf0edf1ff, SUC_lexicon_tags_30 },
    { 0x7aedf1ff, SUC_lexicon_tags_1 },
    { 0xedcdf201, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58df204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8df208, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938df20c, SUC_lexicon_tags_33 },
    { 0x760df20d, SUC_lexicon_tags_12 },
    { 0x912df20e, SUC_lexicon_tags_85 },
    { 0x924df20c, SUC_lexicon_tags_87 },
    { 0xa50df20d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99adf212, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6df215, SUC_lexicon_tags_11 },
    { 0x1b0df216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978df21a, SUC_lexicon_tags_25 },
    { 0xbc4df21b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cdf21f, SUC_lexicon_tags_47 },
    { 0x562df21f, SUC_lexicon_tags_47 },
    { 0xca6df220, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x786df224, SUC_lexicon_tags_17 },
    { 0x300df224, SUC_lexicon_tags_25 },
    { 0xd78df224, SUC_lexicon_tags_8 },
    { 0xe2cdf225, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb10df229, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90df22f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x45edf232, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0x4c0df234, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0df23d, SUC_lexicon_tags_138 },
    { 0x65edf23e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xec6df240, SUC_lexicon_tags_2 },
    { 0xf6df240, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe38df243, SUC_lexicon_tags_1 },
    { 0x75cdf243, SUC_lexicon_tags_65 },
    { 0xf2edf245, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8df24c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecdf252, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x730df254, SUC_lexicon_tags_15 },
    { 0x8d4df254, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x47adf258, SUC_lexicon_tags_7 },
    { 0x346df259, SUC_lexicon_tags_45 },
    { 0x344df25a, SUC_lexicon_tags_28 },
    { 0xd9adf25b, SUC_lexicon_tags_25 },
    { 0xb88df25c, SUC_lexicon_tags_65 },
    { 0x854df25c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdbadf25f, SUC_lexicon_tags_2 },
    { 0xa16df260, SUC_lexicon_tags_11 },
    { 0xb52df260, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28df264, SUC_lexicon_tags_2 },
    { 0x784df265, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56df269, SUC_lexicon_tags_19 },
    { 0xb64df26a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0df26f, SUC_lexicon_tags_3 },
    { 0xb7edf270, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe2cdf272, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180df276, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x750df278, SUC_lexicon_tags_17 },
    { 0xaecdf279, SUC_lexicon_tags_19 },
    { 0xf62df27a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaedf27f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x98edf282, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376df286, SUC_lexicon_tags_14 },
    { 0x218df287, SUC_lexicon_tags_0 },
    { 0x9a4df288, SUC_lexicon_tags_31 },
    { 0xe9edf289, SUC_lexicon_tags_65 },
    { 0x2b8df287, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58df28e, SUC_lexicon_tags_12 },
    { 0xfa8df28e, SUC_lexicon_tags_17 },
    { 0x30cdf290, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0xbb4df292, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x306df295, SUC_lexicon_tags_1 },
    { 0x932df295, SUC_lexicon_tags_37 },
    { 0xfbcdf296, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddedf299, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb48df29c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798df2a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18df2a8, SUC_lexicon_tags_66 },
    { 0x456df2a8, SUC_lexicon_tags_15 },
    { 0x3dcdf2aa, SUC_lexicon_tags_48 },
    { 0xe4edf2ab, SUC_lexicon_tags_29 },
    { 0xd82df2a9, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38df2b2, SUC_lexicon_tags_27 },
    { 0x50edf2b2, SUC_lexicon_tags_151 },
    { 0x296df2b4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4edf2b9, SUC_lexicon_tags_31 },
    { 0xa8df2b9, SUC_lexicon_tags_11 },
    { 0xfe0df2ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31adf2be, SUC_lexicon_tags_2 },
    { 0x652df2bf, SUC_lexicon_tags_37 },
    { 0xcc0df2c0, SUC_lexicon_tags_29 },
    { 0x166df2c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x32cdf2c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4df2c6, SUC_lexicon_tags_39 },
    { 0x574df2c6, SUC_lexicon_tags_11 },
    { 0xf30df2c7, SUC_lexicon_tags_3 },
    { 0x7b4df2c7, SUC_lexicon_tags_47 },
    { 0x8d0df2ca, SUC_lexicon_tags_49 },
    { 0xbf0df2c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa90df2cd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x474df2d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dadf2d4, SUC_lexicon_tags_25 },
    { 0x54adf2d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf64df2d7, SUC_lexicon_tags_12 },
    { 0x2a0df2d7, SUC_lexicon_tags_0 },
    { 0x4adf2d8, SUC_lexicon_tags_26 },
    { 0x8dcdf2da, SUC_lexicon_tags_47 },
    { 0x4acdf2d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe78df2dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04df2e1, SUC_lexicon_tags_39 },
    { 0xefadf2e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x720df2e4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66df2e8, SUC_lexicon_tags_15 },
    { 0xd46df2e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4df2ed, SUC_lexicon_tags_15 },
    { 0xddedf2ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2dcdf2f0, SUC_lexicon_tags_31 },
    { 0x3bcdf2f1, SUC_lexicon_tags_31 },
    { 0xeacdf2f2, SUC_lexicon_tags_1 },
    { 0x8d2df2f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aedf2fb, SUC_lexicon_tags_37 },
    { 0xeb6df2fc, SUC_lexicon_tags_17 },
    { 0x4f0df2fd, SUC_lexicon_tags_1 },
    { 0x2e2df2fe, SUC_lexicon_tags_25 },
    { 0xe8cdf2ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cdf304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a0df306, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6df313, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb16df315, SUC_lexicon_tags_11 },
    { 0x9c2df316, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68df31a, SUC_lexicon_tags_12 },
    { 0xd6edf31b, SUC_lexicon_tags_74 },
    { 0xae8df31c, SUC_lexicon_tags_11 },
    { 0x572df31c, SUC_lexicon_tags_19 },
    { 0xca6df31c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4df321, SUC_lexicon_tags_29 },
    { 0x5a6df322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8df325, SUC_lexicon_tags_17 },
    { 0xccadf326, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5e4df328, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8df32c, SUC_lexicon_tags_11 },
    { 0x4c4df32d, SUC_lexicon_tags_2 },
    { 0x65cdf32e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508df332, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc58df334, SUC_lexicon_tags_3 },
    { 0x30cdf335, SUC_lexicon_tags_37 },
    { 0x3ccdf336, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744df33b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0df33f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aadf345, SUC_lexicon_tags_2 },
    { 0x1b4df346, SUC_lexicon_tags_10 },
    { 0x9cdf345, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe72df349, SUC_lexicon_tags_8 },
    { 0xc0cdf34a, SUC_lexicon_tags_11 },
    { 0xe5edf34b, SUC_lexicon_tags_32 },
    { 0x36adf34a, SUC_lexicon_tags_6 },
    { 0x978df34d, SUC_lexicon_tags_1 },
    { 0x8ecdf34e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4edf356, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0df35b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97edf35e, SUC_lexicon_tags_7 },
    { 0x1ecdf35e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180df367, SUC_lexicon_tags_17 },
    { 0x410df368, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x732df36a, SUC_lexicon_tags_0 },
    { 0xa54df36b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888df36f, SUC_lexicon_tags_25 },
    { 0x820df370, SUC_lexicon_tags_37 },
    { 0x36cdf371, SUC_lexicon_tags_23 },
    { 0x474df372, SUC_lexicon_tags_46 },
    { 0xe12df36f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa72df375, SUC_lexicon_tags_12 },
    { 0x454df375, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8adf378, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb78df37a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aadf37f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2df384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8df38a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x994df38d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x678df390, SUC_lexicon_tags_1 },
    { 0xab4df391, SUC_lexicon_tags_8 },
    { 0x40cdf392, SUC_lexicon_tags_17 },
    { 0xab2df393, SUC_lexicon_tags_1 },
    { 0x37cdf394, SUC_lexicon_tags_25 },
    { 0x4ccdf395, SUC_lexicon_tags_25 },
    { 0xeb6df396, SUC_lexicon_tags_23 },
    { 0x2c0df397, SUC_lexicon_tags_35 },
    { 0x96df395, SUC_lexicon_tags_11 },
    { 0xd6adf394, SUC_lexicon_tags_37 },
    { 0xc7adf39a, SUC_lexicon_tags_39 },
    { 0xd32df392, SUC_lexicon_tags_48 },
    { 0x630df39a, SUC_lexicon_tags_17 },
    { 0x576df39d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11adf3a3, SUC_lexicon_tags_25 },
    { 0x3ecdf3a4, SUC_lexicon_tags_2 },
    { 0xeb6df3a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x754df3a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758df3b0, SUC_lexicon_tags_43 },
    { 0xe00df3b0, SUC_lexicon_tags_23 },
    { 0x664df3b0, SUC_lexicon_tags_19 },
    { 0x178df3b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922df3b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x494df3bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282df3c0, SUC_lexicon_tags_13 },
    { 0x442df3c1, SUC_lexicon_tags_2 },
    { 0xdccdf3c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dedf3c6, SUC_lexicon_tags_17 },
    { 0xc1adf3c6, SUC_lexicon_tags_10 },
    { 0x1aedf3c8, SUC_lexicon_tags_2 },
    { 0xc52df3c9, SUC_lexicon_tags_12 },
    { 0xa68df3ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0adf3cf, SUC_lexicon_tags_11 },
    { 0x10adf3d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a8df3d2, SUC_lexicon_tags_23 },
    { 0x6c0df3d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376df3db, SUC_lexicon_tags_2 },
    { 0xbeedf3db, SUC_lexicon_tags_37 },
    { 0x7d2df3db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeedf3e2, SUC_lexicon_tags_1 },
    { 0xf26df3e2, SUC_lexicon_tags_17 },
    { 0xde0df3e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x838df3e7, SUC_lexicon_tags_19 },
    { 0x894df3e8, SUC_lexicon_tags_0 },
    { 0x768df3e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x536df3ec, SUC_lexicon_tags_25 },
    { 0x5aadf3ed, SUC_lexicon_tags_37 },
    { 0x2e2df3ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5aedf3f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa86df3f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37edf3f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cdf3fa, SUC_lexicon_tags_7 },
    { 0x576df3fb, SUC_lexicon_tags_2 },
    { 0x386df3fc, SUC_lexicon_tags_32 },
    { 0xc48df3fb, SUC_lexicon_tags_0 },
    { 0xd84df3fe, SUC_lexicon_tags_63 },
    { 0xf1edf3ff, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94edf40a, SUC_lexicon_tags_37 },
    { 0x9b2df40a, SUC_lexicon_tags_45 },
    { 0xec0df40b, SUC_lexicon_tags_1 },
    { 0xf1adf40b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x878df410, SUC_lexicon_tags_18 },
    { 0x58edf411, SUC_lexicon_tags_1 },
    { 0x2a4df411, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6df415, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eadf41b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2df420, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0df424, SUC_lexicon_tags_31 },
    { 0x3d6df424, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0x2d4df427, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x978df429, SUC_lexicon_tags_23 },
    { 0xa48df429, SUC_lexicon_tags_12 },
    { 0x904df42a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bcdf42d, SUC_lexicon_tags_37 },
    { 0xc2edf42e, SUC_lexicon_tags_25 },
    { 0xc26df42d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcdf432, SUC_lexicon_tags_43 },
    { 0xdadf432, SUC_lexicon_tags_25 },
    { 0x588df434, SUC_lexicon_tags_37 },
    { 0xc78df433, SUC_lexicon_tags_14 },
    { 0x2ccdf436, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0df43a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550df440, SUC_lexicon_tags_17 },
    { 0x510df440, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73edf44f, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6df453, SUC_lexicon_tags_40 },
    { 0xf90df453, SUC_lexicon_tags_27 },
    { 0x680df454, SUC_lexicon_tags_6 },
    { 0xa28df453, SUC_lexicon_tags_45 },
    { 0x11adf457, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed4df459, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbccdf45b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cadf45f, SUC_lexicon_tags_37 },
    { 0x47cdf45f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb64df462, SUC_lexicon_tags_1 },
    { 0x9d8df463, SUC_lexicon_tags_7 },
    { 0xa5edf464, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xedf466, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7edf468, SUC_lexicon_tags_5 },
    { 0x148df469, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8df46f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30df472, SUC_lexicon_tags_25 },
    { 0x30cdf472, SUC_lexicon_tags_72 },
    { 0x2a0df472, SUC_lexicon_tags_37 },
    { 0x3d0df475, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4df479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d2df47b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaedf480, SUC_lexicon_tags_21 },
    { 0xc2adf481, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8df483, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x606df485, SUC_lexicon_tags_1 },
    { 0x310df486, SUC_lexicon_tags_15 },
    { 0xfe8df487, SUC_lexicon_tags_34 },
    { 0xf28df485, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcdf48c, SUC_lexicon_tags_15 },
    { 0xd1cdf48d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeedf492, SUC_lexicon_tags_11 },
    { 0xf86df492, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678df497, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2df49d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90edf4a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd88df4a3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cdf4a6, SUC_lexicon_tags_2 },
    { 0x5ccdf4a7, SUC_lexicon_tags_17 },
    { 0xb7adf4a8, SUC_lexicon_tags_31 },
    { 0xc7cdf4a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4df4ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe10df4af, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eedf4b2, SUC_lexicon_tags_14 },
    { 0x36df4b3, SUC_lexicon_tags_85 },
    { 0xa8adf4b2, SUC_lexicon_tags_1 },
    { 0xba6df4b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc94df4b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8df4ba, SUC_lexicon_tags_27 },
    { 0x55edf4bb, SUC_lexicon_tags_31 },
    { 0x60df4ba, SUC_lexicon_tags_9 },
    { 0x8c0df4bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x34adf4bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7dcdf4c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62adf4c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26df4cb, SUC_lexicon_tags_28 },
    { 0xaf2df4cb, SUC_lexicon_tags_3 },
    { 0xe88df4cd, SUC_lexicon_tags_38 },
    { 0x584df4ce, SUC_lexicon_tags_25 },
    { 0xe2df4cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81cdf4d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7edf4d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7edf4dd, SUC_lexicon_tags_15 },
    { 0x92df4dd, SUC_lexicon_tags_27 },
    { 0x458df4df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xadf4e1, SUC_lexicon_tags_25 },
    { 0xf9adf4e2, SUC_lexicon_tags_39 },
    { 0x1a6df4e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97cdf4e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424df4ed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0adf4f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8df4f4, SUC_lexicon_tags_25 },
    { 0x566df4f5, SUC_lexicon_tags_19 },
    { 0xcd2df4f5, SUC_lexicon_tags_15 },
    { 0x3f0df4f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53adf4f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36df4fb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21edf501, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdb2df503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10df505, SUC_lexicon_tags_19 },
    { 0x852df506, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70edf50e, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298df515, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeedf51d, SUC_lexicon_tags_11 },
    { 0x722df51e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x144df520, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0x480df522, SUC_lexicon_tags_19 },
    { 0x308df523, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x226df525, SUC_lexicon_tags_45 },
    { 0x38df525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x904df528, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804df531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22df537, SUC_lexicon_tags_15 },
    { 0xd20df538, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cadf53a, SUC_lexicon_tags_1 },
    { 0x5c2df53b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x868df53e, SUC_lexicon_tags_17 },
    { 0x47adf53f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32cdf541, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc2df543, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x55cdf545, SUC_lexicon_tags_45 },
    { 0x260df546, SUC_lexicon_tags_35 },
    { 0xcdcdf547, SUC_lexicon_tags_1 },
    { 0xeaedf548, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fadf54a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe44df54c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88df551, SUC_lexicon_tags_60 },
    { 0xf70df552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2df557, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49edf55b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x136df55e, SUC_lexicon_tags_23 },
    { 0x19cdf55f, SUC_lexicon_tags_47 },
    { 0xf1cdf55f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x810df562, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x50edf565, SUC_lexicon_tags_14 },
    { 0xd10df566, SUC_lexicon_tags_46 },
    { 0xad6df566, SUC_lexicon_tags_60 },
    { 0x1c8df568, SUC_lexicon_tags_6 },
    { 0x4cdf569, SUC_lexicon_tags_33 },
    { 0xa8cdf56a, SUC_lexicon_tags_8 },
    { 0xdacdf56b, SUC_lexicon_tags_1 },
    { 0x588df56b, SUC_lexicon_tags_56 },
    { 0x7f0df56d, SUC_lexicon_tags_11 },
    { 0xa42df56e, SUC_lexicon_tags_25 },
    { 0xf04df56f, SUC_lexicon_tags_0 },
    { 0xefadf570, SUC_lexicon_tags_37 },
    { 0xd4edf571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ecdf573, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x814df575, SUC_lexicon_tags_63 },
    { 0x442df576, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81edf578, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8a4df57a, SUC_lexicon_tags_7 },
    { 0x4c8df57b, SUC_lexicon_tags_43 },
    { 0x890df57a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42df57f, SUC_lexicon_tags_19 },
    { 0xc08df580, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2adf587, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x554df589, SUC_lexicon_tags_37 },
    { 0x65cdf58a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8df58d, SUC_lexicon_tags_11 },
    { 0xbb0df58d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32adf590, SUC_lexicon_tags_0 },
    { 0xcd6df590, SUC_lexicon_tags_14 },
    { 0xfcadf592, SUC_lexicon_tags_68 },
    { 0x368df592, SUC_lexicon_tags_38 },
    { 0xa82df592, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbedf59a, SUC_lexicon_tags_91 },
    { 0x604df59b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6df59f, SUC_lexicon_tags_12 },
    { 0xe0adf5a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850df5a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48df5aa, SUC_lexicon_tags_69 },
    { 0x838df5ab, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84df5b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96df5b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54adf5b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcdf5c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68adf5cb, SUC_lexicon_tags_236 },
    { 0x53cdf5cc, SUC_lexicon_tags_25 },
    { 0x32edf5cc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xecedf5d0, SUC_lexicon_tags_17 },
    { 0xb58df5d0, SUC_lexicon_tags_15 },
    { 0x9dedf5d1, SUC_lexicon_tags_47 },
    { 0x16edf5d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3cadf5d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x57adf5d7, SUC_lexicon_tags_6 },
    { 0x926df5d8, SUC_lexicon_tags_17 },
    { 0x1cadf5d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c2df5db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72df5e0, SUC_lexicon_tags_38 },
    { 0x5acdf5e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb46df5e3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf66df5e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36df5e7, SUC_lexicon_tags_37 },
    { 0x3e0df5e7, SUC_lexicon_tags_15 },
    { 0xaa0df5e9, SUC_lexicon_tags_7 },
    { 0xd0adf5ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98df5ed, SUC_lexicon_tags_27 },
    { 0x424df5ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb0df5f0, SUC_lexicon_tags_14 },
    { 0xd82df5f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54adf5f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2df5f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82edf5fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4df600, SUC_lexicon_tags_34 },
    { 0xf2df601, SUC_lexicon_tags_3 },
    { 0x13edf602, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xed0df604, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xeedf606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc72df608, SUC_lexicon_tags_14 },
    { 0x39cdf609, SUC_lexicon_tags_60 },
    { 0x6c8df60a, SUC_lexicon_tags_29 },
    { 0x7b6df609, SUC_lexicon_tags_26 },
    { 0xb8adf60c, SUC_lexicon_tags_7 },
    { 0xa62df60d, SUC_lexicon_tags_25 },
    { 0x3fcdf60a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29cdf610, SUC_lexicon_tags_48 },
    { 0x37edf611, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45cdf613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x830df615, SUC_lexicon_tags_72 },
    { 0xb90df616, SUC_lexicon_tags_1 },
    { 0x7acdf617, SUC_lexicon_tags_84 },
    { 0xc2cdf618, SUC_lexicon_tags_45 },
    { 0x5b0df619, SUC_lexicon_tags_45 },
    { 0xd86df61a, SUC_lexicon_tags_12 },
    { 0x4adf61a, SUC_lexicon_tags_17 },
    { 0x106df618, SUC_lexicon_tags_4 },
    { 0x846df617, SUC_lexicon_tags_14 },
    { 0xde6df61e, SUC_lexicon_tags_1 },
    { 0x280df61f, SUC_lexicon_tags_45 },
    { 0xbeedf620, SUC_lexicon_tags_17 },
    { 0xd32df620, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0df626, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cdf62e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x798df630, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282df634, SUC_lexicon_tags_87 },
    { 0x51adf634, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff4df637, SUC_lexicon_tags_15 },
    { 0x6c2df638, SUC_lexicon_tags_11 },
    { 0xe64df638, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34df63e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40df641, SUC_lexicon_tags_11 },
    { 0xddadf642, SUC_lexicon_tags_66 },
    { 0x12edf643, SUC_lexicon_tags_103 },
    { 0x894df641, SUC_lexicon_tags_48 },
    { 0xf44df643, SUC_lexicon_tags_47 },
    { 0x8a2df643, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0df649, SUC_lexicon_tags_33 },
    { 0xc28df64a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e8df64c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76df64f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcadf652, SUC_lexicon_tags_17 },
    { 0x60adf653, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4df656, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8edf65d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb84df65f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10adf664, SUC_lexicon_tags_12 },
    { 0xfc0df665, SUC_lexicon_tags_0 },
    { 0x8badf666, SUC_lexicon_tags_64 },
    { 0x3dcdf665, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x276df66a, SUC_lexicon_tags_63 },
    { 0x51cdf66a, SUC_lexicon_tags_9 },
    { 0xf2df66c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62cdf66e, SUC_lexicon_tags_11 },
    { 0x99edf66f, SUC_lexicon_tags_2 },
    { 0x550df66f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x516df673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadf679, SUC_lexicon_tags_1 },
    { 0xbdcdf67a, SUC_lexicon_tags_25 },
    { 0x5f4df67b, SUC_lexicon_tags_14 },
    { 0x64cdf67c, SUC_lexicon_tags_43 },
    { 0xc10df67c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf24df67f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x800df681, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2df686, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x290df688, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240df68c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc28df68e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42df691, SUC_lexicon_tags_28 },
    { 0xd52df691, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486df69c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe40df69e, SUC_lexicon_tags_49 },
    { 0x14cdf69f, SUC_lexicon_tags_27 },
    { 0x93edf6a0, SUC_lexicon_tags_46 },
    { 0xb06df6a1, SUC_lexicon_tags_63 },
    { 0xfe4df6a1, SUC_lexicon_tags_103 },
    { 0x6fedf6a3, SUC_lexicon_tags_253 },
    { 0xefcdf6a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422df6ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48edf6ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8df6b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38df6b9, SUC_lexicon_tags_7 },
    { 0x498df6ba, SUC_lexicon_tags_24 },
    { 0x8badf6bb, SUC_lexicon_tags_0 },
    { 0x14cdf6bb, SUC_lexicon_tags_37 },
    { 0x872df6ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd4df6bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2df6c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756df6d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06df6dc, SUC_lexicon_tags_3 },
    { 0xd52df6dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9adf6e2, SUC_lexicon_tags_21 },
    { 0x214df6e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6adf6e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38df6ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x508df6f0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36df6fb, SUC_lexicon_tags_33 },
    { 0xafedf6fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x194df6fe, SUC_lexicon_tags_1 },
    { 0xd48df6ff, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83adf705, SUC_lexicon_tags_1 },
    { 0xb24df706, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4df709, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32edf70d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccdf713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44df716, SUC_lexicon_tags_37 },
    { 0x532df717, SUC_lexicon_tags_61 },
    { 0x596df716, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aedf71f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x15adf722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bedf727, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd0adf729, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84df72e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x20df731, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7badf735, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2fadf737, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x12df739, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cdf742, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106df747, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656df74b, SUC_lexicon_tags_11 },
    { 0xe1adf74c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfcadf74e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x86edf750, SUC_lexicon_tags_11 },
    { 0x44adf751, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d8df753, SUC_lexicon_tags_30 },
    { 0xe40df754, SUC_lexicon_tags_7 },
    { 0xbeedf755, SUC_lexicon_tags_1 },
    { 0xc6edf755, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a4df758, SUC_lexicon_tags_11 },
    { 0x346df759, SUC_lexicon_tags_49 },
    { 0x74edf759, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2df764, SUC_lexicon_tags_26 },
    { 0x122df765, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x940df767, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34df76a, SUC_lexicon_tags_37 },
    { 0xa14df76a, SUC_lexicon_tags_0 },
    { 0x604df76c, SUC_lexicon_tags_25 },
    { 0x4fcdf76c, SUC_lexicon_tags_11 },
    { 0x96df76c, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62df771, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2df774, SUC_lexicon_tags_11 },
    { 0x21edf775, SUC_lexicon_tags_104 },
    { 0xfcedf776, SUC_lexicon_tags_6 },
    { 0xc2df775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12df77c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acdf781, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34df787, SUC_lexicon_tags_15 },
    { 0x176df788, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878df78c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85edf78e, SUC_lexicon_tags_1030 },
    { 0xa4edf78e, SUC_lexicon_tags_60 },
    { 0x6cadf790, SUC_lexicon_tags_4 },
    { 0xbdadf791, SUC_lexicon_tags_2 },
    { 0x34adf791, SUC_lexicon_tags_17 },
    { 0xa16df790, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4df79e, SUC_lexicon_tags_559 },
    { 0x97adf79e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42cdf7a2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x456df7a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf00df7a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x322df7aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424df7b3, SUC_lexicon_tags_29 },
    { 0x89cdf7b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d4df7b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6df7ba, SUC_lexicon_tags_12 },
    { 0xd90df7bb, SUC_lexicon_tags_17 },
    { 0x1c2df7bb, SUC_lexicon_tags_11 },
    { 0xba0df7bd, SUC_lexicon_tags_33 },
    { 0x2adf7bd, SUC_lexicon_tags_9 },
    { 0x4f2df7bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99adf7c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x354df7c4, SUC_lexicon_tags_8 },
    { 0xf7cdf7c4, SUC_lexicon_tags_47 },
    { 0xd8df7c6, SUC_lexicon_tags_26 },
    { 0xba8df7c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x554df7ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadf7d0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2df7d9, SUC_lexicon_tags_12 },
    { 0xb30df7d9, SUC_lexicon_tags_35 },
    { 0xbcdf7db, SUC_lexicon_tags_84 },
    { 0x6eedf7dc, SUC_lexicon_tags_17 },
    { 0x77cdf7dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc88df7e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76df7e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2cedf7e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfcedf7e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccdf7ea, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cdf7f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160df7f6, SUC_lexicon_tags_12 },
    { 0x868df7f7, SUC_lexicon_tags_35 },
    { 0xc38df7f6, SUC_lexicon_tags_11 },
    { 0x80edf7f6, SUC_lexicon_tags_7 },
    { 0xf90df7fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e0df7fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9edf7fe, SUC_lexicon_tags_63 },
    { 0x5f4df7ff, SUC_lexicon_tags_37 },
    { 0xce2df800, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x356df803, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2df806, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2df80b, SUC_lexicon_tags_49 },
    { 0x84adf80b, SUC_lexicon_tags_0 },
    { 0x554df80d, SUC_lexicon_tags_6 },
    { 0xec8df80c, SUC_lexicon_tags_47 },
    { 0xefedf80d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2df814, SUC_lexicon_tags_140 },
    { 0xc84df814, SUC_lexicon_tags_17 },
    { 0x30adf816, SUC_lexicon_tags_17 },
    { 0xc3adf816, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44df81e, SUC_lexicon_tags_11 },
    { 0xe64df81e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04df823, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbb4df825, SUC_lexicon_tags_7 },
    { 0x7b2df825, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10df82a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43edf832, SUC_lexicon_tags_1 },
    { 0x3f8df833, SUC_lexicon_tags_13 },
    { 0xf24df834, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4df837, SUC_lexicon_tags_2 },
    { 0x8b8df838, SUC_lexicon_tags_77 },
    { 0x3d6df839, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc42df83b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4adf83d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cdf840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454df845, SUC_lexicon_tags_60 },
    { 0x574df845, SUC_lexicon_tags_47 },
    { 0x50edf846, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dedf84b, SUC_lexicon_tags_47 },
    { 0xcdadf84b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x384df84e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210df853, SUC_lexicon_tags_48 },
    { 0x582df854, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd54df857, SUC_lexicon_tags_2 },
    { 0x932df858, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eadf85b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360df861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322df866, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586df86b, SUC_lexicon_tags_14 },
    { 0x5cadf86c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x66df86e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaadf871, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98adf873, SUC_lexicon_tags_12 },
    { 0x29edf874, SUC_lexicon_tags_77 },
    { 0x804df875, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59edf877, SUC_lexicon_tags_11 },
    { 0x9f6df878, SUC_lexicon_tags_14 },
    { 0x2fadf878, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd72df87b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28adf880, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x98adf883, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12df887, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x344df889, SUC_lexicon_tags_33 },
    { 0xa78df88a, SUC_lexicon_tags_33 },
    { 0xa14df88a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x75cdf88d, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cdf891, SUC_lexicon_tags_1 },
    { 0x928df892, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5adf894, SUC_lexicon_tags_2 },
    { 0x86df895, SUC_lexicon_tags_19 },
    { 0x24cdf894, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63edf89b, SUC_lexicon_tags_28 },
    { 0x1c2df89c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6df89f, SUC_lexicon_tags_23 },
    { 0xb62df8a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x13adf8a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7bcdf8a4, SUC_lexicon_tags_30 },
    { 0x3e6df8a5, SUC_lexicon_tags_7 },
    { 0x1f6df8a6, SUC_lexicon_tags_37 },
    { 0xe4edf8a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cadf8aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x792df8ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920df8b5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe66df8b7, SUC_lexicon_tags_82 },
    { 0x4d2df8b8, SUC_lexicon_tags_11 },
    { 0x4bedf8b7, SUC_lexicon_tags_9 },
    { 0xffedf8b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb28df8bd, SUC_lexicon_tags_1 },
    { 0xd0edf8be, SUC_lexicon_tags_74 },
    { 0xd94df8bf, SUC_lexicon_tags_65 },
    { 0x894df8c0, SUC_lexicon_tags_2 },
    { 0xcdf8bf, SUC_lexicon_tags_33 },
    { 0x39cdf8c2, SUC_lexicon_tags_11 },
    { 0xab0df8c1, SUC_lexicon_tags_17 },
    { 0x114df8c4, SUC_lexicon_tags_77 },
    { 0x9c0df8c5, SUC_lexicon_tags_25 },
    { 0xda6df8c1, SUC_lexicon_tags_23 },
    { 0xdd8df8c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14adf8ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6df8cf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cdf8d6, SUC_lexicon_tags_60 },
    { 0x2a6df8d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64df8da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310df8e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e2df8e5, SUC_lexicon_tags_1 },
    { 0x7eedf8e5, SUC_lexicon_tags_66 },
    { 0xa48df8e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e4df8e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54edf8eb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24df8f1, SUC_lexicon_tags_25 },
    { 0xa48df8f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86df8f7, SUC_lexicon_tags_74 },
    { 0x806df8f8, SUC_lexicon_tags_33 },
    { 0x462df8f9, SUC_lexicon_tags_7 },
    { 0x724df8f7, SUC_lexicon_tags_9 },
    { 0x2c8df8f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7edf8ff, SUC_lexicon_tags_37 },
    { 0x380df900, SUC_lexicon_tags_2 },
    { 0x8aadf901, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x754df903, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x868df906, SUC_lexicon_tags_25 },
    { 0x5badf906, SUC_lexicon_tags_29 },
    { 0x68edf906, SUC_lexicon_tags_28 },
    { 0x9b6df906, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4b8df90b, SUC_lexicon_tags_29 },
    { 0xdd0df90c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88df90f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7eadf911, SUC_lexicon_tags_45 },
    { 0xca8df912, SUC_lexicon_tags_25 },
    { 0xd5edf911, SUC_lexicon_tags_48 },
    { 0xdf4df911, SUC_lexicon_tags_25 },
    { 0x65cdf915, SUC_lexicon_tags_10 },
    { 0x28df911, SUC_lexicon_tags_35 },
    { 0x9f2df917, SUC_lexicon_tags_1 },
    { 0x118df918, SUC_lexicon_tags_37 },
    { 0xb94df919, SUC_lexicon_tags_63 },
    { 0xef4df91a, SUC_lexicon_tags_11 },
    { 0xaa8df91b, SUC_lexicon_tags_15 },
    { 0x4aedf919, SUC_lexicon_tags_27 },
    { 0x27edf914, SUC_lexicon_tags_11 },
    { 0x70cdf91e, SUC_lexicon_tags_49 },
    { 0x336df91f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x858df922, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x456df924, SUC_lexicon_tags_8 },
    { 0xebadf924, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcdf92c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452df932, SUC_lexicon_tags_37 },
    { 0xa00df933, SUC_lexicon_tags_55 },
    { 0x228df934, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8edf93a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acdf93e, SUC_lexicon_tags_61 },
    { 0x264df93e, SUC_lexicon_tags_27 },
    { 0xf5adf940, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb0df942, SUC_lexicon_tags_17 },
    { 0xe28df943, SUC_lexicon_tags_6 },
    { 0xeccdf943, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22cdf946, SUC_lexicon_tags_37 },
    { 0xf7cdf947, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436df94d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8df953, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0df959, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df95d, SUC_lexicon_tags_31 },
    { 0x4a4df95d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16adf960, SUC_lexicon_tags_23 },
    { 0x330df961, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5e0df963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8df975, SUC_lexicon_tags_46 },
    { 0xc92df976, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17edf978, SUC_lexicon_tags_116 },
    { 0xcacdf979, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776df97d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x848df97f, SUC_lexicon_tags_37 },
    { 0xf34df980, SUC_lexicon_tags_12 },
    { 0xd5edf980, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xcacdf984, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56df988, SUC_lexicon_tags_65 },
    { 0x1cedf989, SUC_lexicon_tags_23 },
    { 0x84adf98a, SUC_lexicon_tags_12 },
    { 0x40cdf989, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8df990, SUC_lexicon_tags_2 },
    { 0x822df990, SUC_lexicon_tags_37 },
    { 0x546df992, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x71cdf995, SUC_lexicon_tags_14 },
    { 0x180df996, SUC_lexicon_tags_7 },
    { 0xd88df996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8df9a0, SUC_lexicon_tags_63 },
    { 0xfe2df9a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94df9a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cdf9ab, SUC_lexicon_tags_25 },
    { 0xabcdf9ac, SUC_lexicon_tags_17 },
    { 0xfe6df9ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc68df9af, SUC_lexicon_tags_7 },
    { 0x146df9af, SUC_lexicon_tags_10 },
    { 0x9eedf9af, SUC_lexicon_tags_15 },
    { 0x99cdf9b2, SUC_lexicon_tags_38 },
    { 0xcadf9b0, SUC_lexicon_tags_47 },
    { 0x15adf9af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134df9ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848df9be, SUC_lexicon_tags_37 },
    { 0xbd6df9be, SUC_lexicon_tags_25 },
    { 0x842df9c0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4d8df9c2, SUC_lexicon_tags_14 },
    { 0x4f0df9c3, SUC_lexicon_tags_11 },
    { 0xb0df9c3, SUC_lexicon_tags_7 },
    { 0xc60df9c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce0df9c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd48df9c9, SUC_lexicon_tags_37 },
    { 0x15edf9c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdadf9ce, SUC_lexicon_tags_43 },
    { 0x276df9ce, SUC_lexicon_tags_7 },
    { 0x126df9d0, SUC_lexicon_tags_29 },
    { 0x8cdf9d0, SUC_lexicon_tags_25 },
    { 0x18df9d2, SUC_lexicon_tags_40 },
    { 0xe96df9d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530df9d7, SUC_lexicon_tags_8 },
    { 0x8d8df9d8, SUC_lexicon_tags_1 },
    { 0x7e0df9d9, SUC_lexicon_tags_16 },
    { 0x2e0df9d8, SUC_lexicon_tags_14 },
    { 0xf12df9db, SUC_lexicon_tags_21 },
    { 0x832df9dc, SUC_lexicon_tags_37 },
    { 0xd3adf9dc, SUC_lexicon_tags_47 },
    { 0x1c0df9d9, SUC_lexicon_tags_68 },
    { 0x3e8df9df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74df9e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32edf9e7, SUC_lexicon_tags_7 },
    { 0xd80df9e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd92df9ea, SUC_lexicon_tags_25 },
    { 0x976df9eb, SUC_lexicon_tags_11 },
    { 0xd20df9eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x352df9ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652df9f4, SUC_lexicon_tags_28 },
    { 0x8bcdf9f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2df9f8, SUC_lexicon_tags_9 },
    { 0x9d8df9f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff2df9fb, SUC_lexicon_tags_34 },
    { 0x134df9fc, SUC_lexicon_tags_43 },
    { 0x138df9fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x318dfa00, SUC_lexicon_tags_15 },
    { 0xd30dfa00, SUC_lexicon_tags_14 },
    { 0xbbadfa02, SUC_lexicon_tags_47 },
    { 0xa1edfa02, SUC_lexicon_tags_1 },
    { 0x7aadfa04, SUC_lexicon_tags_27 },
    { 0x716dfa03, SUC_lexicon_tags_23 },
    { 0xfdedfa06, SUC_lexicon_tags_37 },
    { 0x3dcdfa04, SUC_lexicon_tags_15 },
    { 0x76cdfa08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c8dfa0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4dfa11, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388dfa15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x522dfa18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xba2dfa1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x976dfa1c, SUC_lexicon_tags_11 },
    { 0xf7edfa1d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42dfa22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672dfa26, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3c8dfa28, SUC_lexicon_tags_17 },
    { 0xed0dfa29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e0dfa2b, SUC_lexicon_tags_17 },
    { 0xd84dfa2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x254dfa2f, SUC_lexicon_tags_27 },
    { 0x982dfa2f, SUC_lexicon_tags_48 },
    { 0x998dfa2f, SUC_lexicon_tags_29 },
    { 0x9dedfa32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62dfa36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178dfa3a, SUC_lexicon_tags_61 },
    { 0xa04dfa3b, SUC_lexicon_tags_12 },
    { 0x102dfa3a, SUC_lexicon_tags_47 },
    { 0xa68dfa3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882dfa41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaeedfa43, SUC_lexicon_tags_17 },
    { 0x94adfa44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2dfa47, SUC_lexicon_tags_17 },
    { 0x264dfa47, SUC_lexicon_tags_33 },
    { 0x368dfa49, SUC_lexicon_tags_140 },
    { 0xefcdfa49, SUC_lexicon_tags_12 },
    { 0x102dfa4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb0dfa4d, SUC_lexicon_tags_19 },
    { 0xe0edfa4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78dfa52, SUC_lexicon_tags_37 },
    { 0xf30dfa53, SUC_lexicon_tags_3 },
    { 0xafcdfa54, SUC_lexicon_tags_12 },
    { 0x2e6dfa55, SUC_lexicon_tags_33 },
    { 0xf8cdfa55, SUC_lexicon_tags_415 },
    { 0xe24dfa57, SUC_lexicon_tags_17 },
    { 0x7fcdfa56, SUC_lexicon_tags_2 },
    { 0xccedfa59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e4dfa5b, SUC_lexicon_tags_11 },
    { 0x534dfa5b, SUC_lexicon_tags_37 },
    { 0x3f0dfa5d, SUC_lexicon_tags_14 },
    { 0x96cdfa5d, SUC_lexicon_tags_12 },
    { 0xafcdfa5d, SUC_lexicon_tags_25 },
    { 0xaccdfa5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7cdfa62, SUC_lexicon_tags_1 },
    { 0xa54dfa62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd0dfa65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7adfa6a, SUC_lexicon_tags_12 },
    { 0xa30dfa6b, SUC_lexicon_tags_2 },
    { 0xc76dfa6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aedfa6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ccdfa71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa90dfa73, SUC_lexicon_tags_17 },
    { 0x514dfa74, SUC_lexicon_tags_33 },
    { 0x45edfa75, SUC_lexicon_tags_6 },
    { 0x972dfa75, SUC_lexicon_tags_11 },
    { 0x2b0dfa77, SUC_lexicon_tags_17 },
    { 0x922dfa78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08dfa7f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3adfa81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0edfa8e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7edfa92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x718dfa95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cdfa98, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124dfa9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x324dfa9f, SUC_lexicon_tags_17 },
    { 0x830dfaa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecedfaa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x748dfaaa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x660dfaac, SUC_lexicon_tags_15 },
    { 0x4aedfaad, SUC_lexicon_tags_15 },
    { 0x2aadfaac, SUC_lexicon_tags_49 },
    { 0xe12dfaad, SUC_lexicon_tags_35 },
    { 0x274dfab0, SUC_lexicon_tags_17 },
    { 0xf70dfab0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4edfab3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa46dfab5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8dfab8, SUC_lexicon_tags_25 },
    { 0x7fedfab9, SUC_lexicon_tags_17 },
    { 0xda4dfab8, SUC_lexicon_tags_33 },
    { 0x528dfabb, SUC_lexicon_tags_10 },
    { 0xfb2dfabc, SUC_lexicon_tags_28 },
    { 0xf98dfabd, SUC_lexicon_tags_87 },
    { 0x2eadfabb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0dfac2, SUC_lexicon_tags_65 },
    { 0x60dfac3, SUC_lexicon_tags_0 },
    { 0xf78dfac4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0dfacb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8cdfacd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x946dfacf, SUC_lexicon_tags_124 },
    { 0xe9cdfacf, SUC_lexicon_tags_25 },
    { 0x6e0dfad0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cdfad8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aadfada, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x824dfadc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534dfae8, SUC_lexicon_tags_26 },
    { 0x304dfae9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x214dfaeb, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4dfaf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96dfaf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cdfafe, SUC_lexicon_tags_49 },
    { 0x55cdfaff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4dfb08, SUC_lexicon_tags_11 },
    { 0x72cdfb08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cdfb0d, SUC_lexicon_tags_7 },
    { 0x7e0dfb0d, SUC_lexicon_tags_57 },
    { 0xae0dfb0d, SUC_lexicon_tags_11 },
    { 0x2dadfb10, SUC_lexicon_tags_16 },
    { 0xadfb11, SUC_lexicon_tags_43 },
    { 0xe6dfb12, SUC_lexicon_tags_45 },
    { 0xeeadfb12, SUC_lexicon_tags_17 },
    { 0x53cdfb13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x902dfb16, SUC_lexicon_tags_8 },
    { 0xcb2dfb16, SUC_lexicon_tags_17 },
    { 0xc46dfb16, SUC_lexicon_tags_28 },
    { 0xacedfb18, SUC_lexicon_tags_39 },
    { 0xe6cdfb1a, SUC_lexicon_tags_11 },
    { 0x3f2dfb1a, SUC_lexicon_tags_1 },
    { 0x488dfb1c, SUC_lexicon_tags_27 },
    { 0x56cdfb1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dfb24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb88dfb26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xedcdfb29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x51adfb2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8dfb2e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcbedfb30, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76dfb33, SUC_lexicon_tags_15 },
    { 0xffedfb34, SUC_lexicon_tags_39 },
    { 0x482dfb34, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf6dfb37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4dfb3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd90dfb3d, SUC_lexicon_tags_1091 },
    { 0, NULL },
    { 0x2c4dfb3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8dfb45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8dfb4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe28dfb4d, SUC_lexicon_tags_25 },
    { 0x928dfb4d, SUC_lexicon_tags_1 },
    { 0x804dfb4f, SUC_lexicon_tags_25 },
    { 0x2cdfb50, SUC_lexicon_tags_1 },
    { 0xfe4dfb51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaaadfb53, SUC_lexicon_tags_60 },
    { 0xb38dfb54, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eadfb57, SUC_lexicon_tags_14 },
    { 0x824dfb58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb90dfb5a, SUC_lexicon_tags_17 },
    { 0x120dfb5b, SUC_lexicon_tags_25 },
    { 0x6adfb5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec4dfb5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0dfb61, SUC_lexicon_tags_38 },
    { 0x6f6dfb61, SUC_lexicon_tags_25 },
    { 0x60adfb61, SUC_lexicon_tags_34 },
    { 0x9cadfb64, SUC_lexicon_tags_15 },
    { 0xc4dfb64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x404dfb67, SUC_lexicon_tags_46 },
    { 0x566dfb67, SUC_lexicon_tags_43 },
    { 0x742dfb69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7adfb6b, SUC_lexicon_tags_11 },
    { 0xd86dfb6c, SUC_lexicon_tags_19 },
    { 0x828dfb6d, SUC_lexicon_tags_87 },
    { 0xe6adfb6c, SUC_lexicon_tags_30 },
    { 0xf0adfb6f, SUC_lexicon_tags_11 },
    { 0x4c4dfb6f, SUC_lexicon_tags_19 },
    { 0xae8dfb71, SUC_lexicon_tags_15 },
    { 0x508dfb6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66dfb74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf22dfb76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67adfb7b, SUC_lexicon_tags_18 },
    { 0xc2cdfb7c, SUC_lexicon_tags_17 },
    { 0x6dcdfb7d, SUC_lexicon_tags_37 },
    { 0x1c4dfb7e, SUC_lexicon_tags_49 },
    { 0xacedfb7e, SUC_lexicon_tags_27 },
    { 0x9e0dfb80, SUC_lexicon_tags_9 },
    { 0xd58dfb81, SUC_lexicon_tags_23 },
    { 0x10cdfb7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a4dfb84, SUC_lexicon_tags_17 },
    { 0x40edfb85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424dfb8d, SUC_lexicon_tags_17 },
    { 0xa48dfb8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdacdfb90, SUC_lexicon_tags_1 },
    { 0xfeedfb90, SUC_lexicon_tags_7 },
    { 0x142dfb90, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1eadfb94, SUC_lexicon_tags_68 },
    { 0x5dadfb95, SUC_lexicon_tags_29 },
    { 0x2e8dfb94, SUC_lexicon_tags_47 },
    { 0xca0dfb94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8dfb9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe20dfb9e, SUC_lexicon_tags_30 },
    { 0xae4dfb9e, SUC_lexicon_tags_87 },
    { 0x38cdfba0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x512dfba2, SUC_lexicon_tags_10 },
    { 0xccdfba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d0dfba5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x822dfba8, SUC_lexicon_tags_7 },
    { 0x140dfba9, SUC_lexicon_tags_26 },
    { 0xc10dfbaa, SUC_lexicon_tags_14 },
    { 0x284dfbaa, SUC_lexicon_tags_63 },
    { 0x614dfbaa, SUC_lexicon_tags_17 },
    { 0x3e8dfbad, SUC_lexicon_tags_37 },
    { 0x154dfbaa, SUC_lexicon_tags_14 },
    { 0x4f8dfbaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1edfbb7, SUC_lexicon_tags_45 },
    { 0x576dfbb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x710dfbba, SUC_lexicon_tags_37 },
    { 0xfc6dfbba, SUC_lexicon_tags_23 },
    { 0x1adfbbc, SUC_lexicon_tags_11 },
    { 0xd24dfbbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x81adfbbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4badfbc2, SUC_lexicon_tags_7 },
    { 0x51cdfbc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc40dfbc5, SUC_lexicon_tags_37 },
    { 0x11edfbc5, SUC_lexicon_tags_60 },
    { 0x380dfbc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc6dfbc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69edfbcd, SUC_lexicon_tags_63 },
    { 0xc2edfbce, SUC_lexicon_tags_14 },
    { 0x666dfbcf, SUC_lexicon_tags_17 },
    { 0x136dfbd0, SUC_lexicon_tags_103 },
    { 0x360dfbd1, SUC_lexicon_tags_82 },
    { 0x954dfbd0, SUC_lexicon_tags_17 },
    { 0xcdfbd3, SUC_lexicon_tags_6 },
    { 0x4f8dfbd2, SUC_lexicon_tags_23 },
    { 0x30cdfbd0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x59adfbd8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12adfbdc, SUC_lexicon_tags_25 },
    { 0xc62dfbdc, SUC_lexicon_tags_17 },
    { 0x3bcdfbdc, SUC_lexicon_tags_70 },
    { 0x330dfbdf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dfbe2, SUC_lexicon_tags_39 },
    { 0x780dfbe3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d6dfbe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35cdfbe8, SUC_lexicon_tags_15 },
    { 0x73adfbe9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd4adfbeb, SUC_lexicon_tags_33 },
    { 0x890dfbec, SUC_lexicon_tags_48 },
    { 0x93cdfbec, SUC_lexicon_tags_60 },
    { 0x2badfbec, SUC_lexicon_tags_1 },
    { 0x97adfbef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6dfbf2, SUC_lexicon_tags_42 },
    { 0x3d2dfbf3, SUC_lexicon_tags_17 },
    { 0x6acdfbf4, SUC_lexicon_tags_20 },
    { 0x43cdfbf4, SUC_lexicon_tags_29 },
    { 0x49edfbf4, SUC_lexicon_tags_57 },
    { 0x58cdfbf7, SUC_lexicon_tags_37 },
    { 0x440dfbf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cdfbfb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8cdfbfd, SUC_lexicon_tags_17 },
    { 0xe92dfbfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cdfc01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bedfc04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8dfc0b, SUC_lexicon_tags_23 },
    { 0xf9edfc0b, SUC_lexicon_tags_26 },
    { 0x818dfc0d, SUC_lexicon_tags_28 },
    { 0xa36dfc0e, SUC_lexicon_tags_1 },
    { 0x20dfc0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cedfc1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6dfc21, SUC_lexicon_tags_46 },
    { 0x9d8dfc22, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98dfc26, SUC_lexicon_tags_15 },
    { 0xb2dfc26, SUC_lexicon_tags_37 },
    { 0x396dfc28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88dfc33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18edfc35, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adfc38, SUC_lexicon_tags_1 },
    { 0x87adfc39, SUC_lexicon_tags_11 },
    { 0xba8dfc38, SUC_lexicon_tags_12 },
    { 0x8f0dfc38, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfc3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeadfc41, SUC_lexicon_tags_19 },
    { 0xd78dfc42, SUC_lexicon_tags_66 },
    { 0x4a8dfc42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x884dfc45, SUC_lexicon_tags_0 },
    { 0x594dfc46, SUC_lexicon_tags_2 },
    { 0xb7edfc46, SUC_lexicon_tags_582 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692dfc4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcdfc53, SUC_lexicon_tags_49 },
    { 0x64dfc54, SUC_lexicon_tags_15 },
    { 0xc2cdfc54, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x174dfc58, SUC_lexicon_tags_12 },
    { 0xd50dfc59, SUC_lexicon_tags_27 },
    { 0x1d8dfc5a, SUC_lexicon_tags_4 },
    { 0xeaadfc58, SUC_lexicon_tags_1 },
    { 0xb8adfc5c, SUC_lexicon_tags_14 },
    { 0x930dfc5d, SUC_lexicon_tags_43 },
    { 0xd2dfc5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x406dfc63, SUC_lexicon_tags_12 },
    { 0x88cdfc64, SUC_lexicon_tags_47 },
    { 0xd66dfc64, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdc0dfc67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca6dfc69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x326dfc6b, SUC_lexicon_tags_1 },
    { 0xbb0dfc6c, SUC_lexicon_tags_12 },
    { 0x654dfc6d, SUC_lexicon_tags_57 },
    { 0xb1adfc6e, SUC_lexicon_tags_582 },
    { 0xc00dfc6f, SUC_lexicon_tags_38 },
    { 0x2cadfc6f, SUC_lexicon_tags_27 },
    { 0xab4dfc6f, SUC_lexicon_tags_229 },
    { 0xfd8dfc70, SUC_lexicon_tags_84 },
    { 0x990dfc70, SUC_lexicon_tags_38 },
    { 0x830dfc72, SUC_lexicon_tags_23 },
    { 0xc18dfc75, SUC_lexicon_tags_61 },
    { 0x61adfc6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x720dfc79, SUC_lexicon_tags_32 },
    { 0x572dfc79, SUC_lexicon_tags_46 },
    { 0x9cadfc7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4dfc7e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8dfc81, SUC_lexicon_tags_45 },
    { 0x424dfc81, SUC_lexicon_tags_25 },
    { 0xb00dfc83, SUC_lexicon_tags_4 },
    { 0x9a8dfc84, SUC_lexicon_tags_37 },
    { 0x8cadfc84, SUC_lexicon_tags_15 },
    { 0xe2edfc83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12dfc8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32dfc8e, SUC_lexicon_tags_25 },
    { 0xc18dfc8f, SUC_lexicon_tags_25 },
    { 0x7b6dfc90, SUC_lexicon_tags_66 },
    { 0x11edfc91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3badfc94, SUC_lexicon_tags_25 },
    { 0x782dfc95, SUC_lexicon_tags_37 },
    { 0xe2dfc96, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcdfc99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cdfc9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaecdfca0, SUC_lexicon_tags_25 },
    { 0xf06dfca0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9cdfca3, SUC_lexicon_tags_29 },
    { 0x48dfca4, SUC_lexicon_tags_21 },
    { 0x2c0dfca5, SUC_lexicon_tags_12 },
    { 0x5c0dfca5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22dfca9, SUC_lexicon_tags_1 },
    { 0xba4dfca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb98dfcac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x264dfcae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8dfcb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d8dfcb5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274dfcb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0dfcbd, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14dfcc0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374dfcc4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x230dfcc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcdfccc, SUC_lexicon_tags_15 },
    { 0xbd6dfccd, SUC_lexicon_tags_66 },
    { 0x852dfcce, SUC_lexicon_tags_57 },
    { 0xad6dfccf, SUC_lexicon_tags_11 },
    { 0x682dfcd0, SUC_lexicon_tags_17 },
    { 0x516dfcd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x878dfcd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccdfcd7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x400dfcd9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x938dfcdb, SUC_lexicon_tags_14 },
    { 0x40adfcdc, SUC_lexicon_tags_8 },
    { 0xaeedfcdd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x43adfcdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594dfcea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf20dfcec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2dfcee, SUC_lexicon_tags_47 },
    { 0x22adfcef, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x23adfcf1, SUC_lexicon_tags_11 },
    { 0x6cadfcf2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf66dfcf4, SUC_lexicon_tags_7 },
    { 0x2b6dfcf5, SUC_lexicon_tags_5 },
    { 0xc12dfcf6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4dfcfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1adfd02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf4edfd04, SUC_lexicon_tags_25 },
    { 0x5f8dfd05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0dfd08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30dfd0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eedfd17, SUC_lexicon_tags_3 },
    { 0xcb0dfd18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adfd1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa0edfd22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe6edfd24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf76dfd26, SUC_lexicon_tags_0 },
    { 0xeacdfd27, SUC_lexicon_tags_6 },
    { 0x9e0dfd28, SUC_lexicon_tags_47 },
    { 0x2f4dfd27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9edfd2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccdfd30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f2dfd32, SUC_lexicon_tags_15 },
    { 0xcfcdfd33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x832dfd35, SUC_lexicon_tags_1 },
    { 0x246dfd36, SUC_lexicon_tags_11 },
    { 0xf16dfd37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8dfd3b, SUC_lexicon_tags_49 },
    { 0xb0cdfd3c, SUC_lexicon_tags_17 },
    { 0x2a8dfd3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa96dfd3f, SUC_lexicon_tags_47 },
    { 0x510dfd3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4adfd44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x66cdfd46, SUC_lexicon_tags_17 },
    { 0x754dfd46, SUC_lexicon_tags_17 },
    { 0xde2dfd47, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644dfd4c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x774dfd4e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468dfd54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6dfd57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97adfd59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb8dfd5b, SUC_lexicon_tags_11 },
    { 0x53adfd5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a0dfd5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x222dfd60, SUC_lexicon_tags_43 },
    { 0x922dfd61, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fadfd6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x906dfd6d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x214dfd6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66cdfd71, SUC_lexicon_tags_14 },
    { 0x734dfd71, SUC_lexicon_tags_46 },
    { 0x544dfd71, SUC_lexicon_tags_32 },
    { 0xc9edfd74, SUC_lexicon_tags_27 },
    { 0xb4dfd75, SUC_lexicon_tags_23 },
    { 0x3b4dfd76, SUC_lexicon_tags_11 },
    { 0x31cdfd77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cdfd7c, SUC_lexicon_tags_26 },
    { 0x450dfd7d, SUC_lexicon_tags_1 },
    { 0x1edfd7e, SUC_lexicon_tags_9 },
    { 0xe56dfd7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18adfd81, SUC_lexicon_tags_25 },
    { 0x69cdfd82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x780dfd85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196dfd89, SUC_lexicon_tags_55 },
    { 0x2bcdfd89, SUC_lexicon_tags_2 },
    { 0xe30dfd89, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760dfd8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cdfda0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92dfda3, SUC_lexicon_tags_11 },
    { 0xc90dfda3, SUC_lexicon_tags_15 },
    { 0xf50dfda5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dfda9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x312dfdac, SUC_lexicon_tags_2 },
    { 0x27edfdac, SUC_lexicon_tags_37 },
    { 0xef2dfdad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cdfdb2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32dfdb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82dfdc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cadfdc7, SUC_lexicon_tags_1 },
    { 0x51adfdc8, SUC_lexicon_tags_27 },
    { 0x788dfdc9, SUC_lexicon_tags_29 },
    { 0x56adfdc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8dfdd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacdfdd7, SUC_lexicon_tags_37 },
    { 0x60adfdd8, SUC_lexicon_tags_25 },
    { 0x426dfdd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xafedfddc, SUC_lexicon_tags_12 },
    { 0xc76dfddd, SUC_lexicon_tags_1 },
    { 0x116dfdde, SUC_lexicon_tags_12 },
    { 0xb2adfdde, SUC_lexicon_tags_16 },
    { 0xb1cdfde0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2dfde6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa30dfde8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x50dfdeb, SUC_lexicon_tags_27 },
    { 0x7eedfdec, SUC_lexicon_tags_2 },
    { 0x81edfded, SUC_lexicon_tags_37 },
    { 0xa10dfdee, SUC_lexicon_tags_37 },
    { 0xc22dfdee, SUC_lexicon_tags_12 },
    { 0x4d0dfdf0, SUC_lexicon_tags_7 },
    { 0x772dfdee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18adfdf6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0dfdfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dfdff, SUC_lexicon_tags_0 },
    { 0x5c0dfdff, SUC_lexicon_tags_25 },
    { 0xe5edfe01, SUC_lexicon_tags_11 },
    { 0x41cdfe02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82edfe05, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcd2dfe07, SUC_lexicon_tags_87 },
    { 0x80cdfe07, SUC_lexicon_tags_61 },
    { 0x262dfe09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28dfe12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30dfe16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23adfe1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x18adfe21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dfe26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576dfe2c, SUC_lexicon_tags_26 },
    { 0xaadfe2c, SUC_lexicon_tags_35 },
    { 0x1b4dfe2e, SUC_lexicon_tags_2 },
    { 0x770dfe2d, SUC_lexicon_tags_15 },
    { 0x89edfe30, SUC_lexicon_tags_15 },
    { 0xcd4dfe31, SUC_lexicon_tags_2 },
    { 0xf16dfe30, SUC_lexicon_tags_105 },
    { 0x9e8dfe33, SUC_lexicon_tags_2 },
    { 0xd92dfe34, SUC_lexicon_tags_15 },
    { 0x734dfe35, SUC_lexicon_tags_57 },
    { 0xaa8dfe36, SUC_lexicon_tags_10 },
    { 0xa00dfe35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4dfe3c, SUC_lexicon_tags_37 },
    { 0x16cdfe3c, SUC_lexicon_tags_46 },
    { 0xd0cdfe3e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50dfe42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf98dfe44, SUC_lexicon_tags_25 },
    { 0xc04dfe45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96dfe49, SUC_lexicon_tags_30 },
    { 0xe76dfe49, SUC_lexicon_tags_11 },
    { 0xa76dfe4a, SUC_lexicon_tags_81 },
    { 0x90dfe4c, SUC_lexicon_tags_15 },
    { 0x2a6dfe4a, SUC_lexicon_tags_38 },
    { 0x672dfe4e, SUC_lexicon_tags_25 },
    { 0xdccdfe4a, SUC_lexicon_tags_45 },
    { 0xff2dfe4f, SUC_lexicon_tags_22 },
    { 0xb0cdfe50, SUC_lexicon_tags_19 },
    { 0x304dfe52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd1edfe54, SUC_lexicon_tags_15 },
    { 0x352dfe55, SUC_lexicon_tags_37 },
    { 0x8aedfe56, SUC_lexicon_tags_7 },
    { 0x5e2dfe55, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcdfe5f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x72edfe61, SUC_lexicon_tags_19 },
    { 0xec6dfe61, SUC_lexicon_tags_38 },
    { 0xaf2dfe63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194dfe68, SUC_lexicon_tags_8 },
    { 0x308dfe68, SUC_lexicon_tags_45 },
    { 0x4ccdfe6a, SUC_lexicon_tags_37 },
    { 0xc5adfe6b, SUC_lexicon_tags_14 },
    { 0xa3edfe6b, SUC_lexicon_tags_25 },
    { 0x8a4dfe6d, SUC_lexicon_tags_0 },
    { 0x30cdfe6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x124dfe71, SUC_lexicon_tags_27 },
    { 0x3cdfe72, SUC_lexicon_tags_46 },
    { 0x336dfe73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x420dfe75, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00dfe7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x540dfe7f, SUC_lexicon_tags_9 },
    { 0x3b8dfe80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbf8dfe82, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8adfe8c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd74dfe8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8dfe98, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21edfe9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa22dfea0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8dfea3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80dfea7, SUC_lexicon_tags_37 },
    { 0xc14dfea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x858dfeaa, SUC_lexicon_tags_46 },
    { 0xfd4dfeab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x18dfead, SUC_lexicon_tags_2 },
    { 0xe86dfeae, SUC_lexicon_tags_28 },
    { 0x788dfeaf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f6dfeb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48adfeb4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd86dfeb6, SUC_lexicon_tags_2 },
    { 0x118dfeb7, SUC_lexicon_tags_27 },
    { 0xd84dfeb7, SUC_lexicon_tags_1 },
    { 0x300dfeb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cdfebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31adfec0, SUC_lexicon_tags_106 },
    { 0xb7edfec1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4edfec4, SUC_lexicon_tags_45 },
    { 0x5bcdfec5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42dfec7, SUC_lexicon_tags_17 },
    { 0x432dfec8, SUC_lexicon_tags_12 },
    { 0x726dfec9, SUC_lexicon_tags_198 },
    { 0xe04dfeca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfadfecd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc0dfecf, SUC_lexicon_tags_10 },
    { 0xaccdfed0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccdfed6, SUC_lexicon_tags_45 },
    { 0xb8dfed7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf50dfed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b8dfedb, SUC_lexicon_tags_14 },
    { 0x688dfedc, SUC_lexicon_tags_46 },
    { 0x5d4dfedc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x936dfedf, SUC_lexicon_tags_11 },
    { 0xeb0dfee0, SUC_lexicon_tags_25 },
    { 0xf6cdfee1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcdfee5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe38dfee7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664dfeec, SUC_lexicon_tags_37 },
    { 0xff8dfeed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56dfef1, SUC_lexicon_tags_23 },
    { 0xdd2dfef2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36adfef9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ccdfefb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2cdfefd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf28dfeff, SUC_lexicon_tags_7 },
    { 0x2a4dff00, SUC_lexicon_tags_7 },
    { 0x596dff01, SUC_lexicon_tags_11 },
    { 0x53cdff01, SUC_lexicon_tags_37 },
    { 0x46adff02, SUC_lexicon_tags_2 },
    { 0x856dff00, SUC_lexicon_tags_46 },
    { 0x1d0dff05, SUC_lexicon_tags_2 },
    { 0x696dff05, SUC_lexicon_tags_12 },
    { 0xf56dff01, SUC_lexicon_tags_79 },
    { 0xcaedff06, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18dff0c, SUC_lexicon_tags_34 },
    { 0x68edff0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60dff12, SUC_lexicon_tags_47 },
    { 0xd8adff12, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8dff17, SUC_lexicon_tags_27 },
    { 0x792dff18, SUC_lexicon_tags_27 },
    { 0x3ccdff18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938dff1e, SUC_lexicon_tags_30 },
    { 0x21cdff1f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72dff24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9badff28, SUC_lexicon_tags_17 },
    { 0xdb0dff29, SUC_lexicon_tags_19 },
    { 0xf0edff2a, SUC_lexicon_tags_17 },
    { 0xf88dff2b, SUC_lexicon_tags_6 },
    { 0xbaedff2c, SUC_lexicon_tags_10 },
    { 0x766dff2d, SUC_lexicon_tags_0 },
    { 0x4c6dff2e, SUC_lexicon_tags_740 },
    { 0x70adff2f, SUC_lexicon_tags_11 },
    { 0x442dff2f, SUC_lexicon_tags_17 },
    { 0x6a6dff31, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadedff39, SUC_lexicon_tags_11 },
    { 0x42adff3a, SUC_lexicon_tags_60 },
    { 0x486dff3b, SUC_lexicon_tags_4 },
    { 0x1f0dff3a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574dff40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558dff47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x550dff49, SUC_lexicon_tags_61 },
    { 0x84edff4a, SUC_lexicon_tags_13 },
    { 0x984dff4b, SUC_lexicon_tags_613 },
    { 0xfd2dff4c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9bcdff4e, SUC_lexicon_tags_25 },
    { 0x49edff4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xededff52, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cdff57, SUC_lexicon_tags_47 },
    { 0x30dff58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8dff5d, SUC_lexicon_tags_23 },
    { 0x152dff5d, SUC_lexicon_tags_19 },
    { 0xb08dff5e, SUC_lexicon_tags_11 },
    { 0xf16dff60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98dff65, SUC_lexicon_tags_9 },
    { 0xf4adff66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdff6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x72edff6d, SUC_lexicon_tags_47 },
    { 0x9f8dff6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88edff70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7adff72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57edff74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4dff77, SUC_lexicon_tags_82 },
    { 0x330dff78, SUC_lexicon_tags_158 },
    { 0xd20dff79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78adff7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaadff7d, SUC_lexicon_tags_160 },
    { 0x6d8dff7e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2a2dff80, SUC_lexicon_tags_66 },
    { 0xdc4dff81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1edff84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x372dff86, SUC_lexicon_tags_54 },
    { 0x408dff87, SUC_lexicon_tags_25 },
    { 0x44edff88, SUC_lexicon_tags_0 },
    { 0x670dff88, SUC_lexicon_tags_25 },
    { 0x226dff8a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284dff90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22dff99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22edff9f, SUC_lexicon_tags_17 },
    { 0x7a0dffa0, SUC_lexicon_tags_2 },
    { 0x370dffa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa38dffa3, SUC_lexicon_tags_19 },
    { 0x958dffa3, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x8b6dffa6, SUC_lexicon_tags_4 },
    { 0xe8cdffa7, SUC_lexicon_tags_57 },
    { 0xb2dffa8, SUC_lexicon_tags_12 },
    { 0xd16dffa9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x866dffab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fedffb2, SUC_lexicon_tags_55 },
    { 0x160dffb3, SUC_lexicon_tags_47 },
    { 0x3c2dffb3, SUC_lexicon_tags_15 },
    { 0x166dffb5, SUC_lexicon_tags_17 },
    { 0x118dffb5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6dadffb8, SUC_lexicon_tags_77 },
    { 0x9adffb8, SUC_lexicon_tags_25 },
    { 0x50edffb9, SUC_lexicon_tags_2 },
    { 0x7aedffbb, SUC_lexicon_tags_15 },
    { 0x47adffbc, SUC_lexicon_tags_1 },
    { 0xbdcdffbd, SUC_lexicon_tags_47 },
    { 0xc0dffba, SUC_lexicon_tags_45 },
    { 0x96dffbf, SUC_lexicon_tags_32 },
    { 0x580dffbc, SUC_lexicon_tags_13 },
    { 0x5dedffba, SUC_lexicon_tags_25 },
    { 0x4a0dffb8, SUC_lexicon_tags_12 },
    { 0xbcadffc0, SUC_lexicon_tags_37 },
    { 0x5dadffc3, SUC_lexicon_tags_12 },
    { 0x306dffc5, SUC_lexicon_tags_11 },
    { 0x814dffba, SUC_lexicon_tags_19 },
    { 0xe16dffc4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538dffce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2dffd2, SUC_lexicon_tags_39 },
    { 0x418dffd3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3b2dffd5, SUC_lexicon_tags_12 },
    { 0x318dffd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeacdffd9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36dffdf, SUC_lexicon_tags_7 },
    { 0xd18dffdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4dffe3, SUC_lexicon_tags_40 },
    { 0xd3edffe3, SUC_lexicon_tags_2 },
    { 0x5f6dffe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540dffeb, SUC_lexicon_tags_19 },
    { 0x1b4dffeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe0dffee, SUC_lexicon_tags_46 },
    { 0x996dffef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7dcdfff1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43adfff7, SUC_lexicon_tags_25 },
    { 0x8d8dfff8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab8dfffa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x798dfffc, SUC_lexicon_tags_56 },
    { 0xb9adfffd, SUC_lexicon_tags_33 },
    { 0x344dfffe, SUC_lexicon_tags_18 },
    { 0xc80dffff, SUC_lexicon_tags_43 },
    { 0xfc6dffff, SUC_lexicon_tags_6 },
    { 0xabce0001, SUC_lexicon_tags_4 },
    { 0xcf2e0002, SUC_lexicon_tags_25 },
    { 0xa0e0003, SUC_lexicon_tags_37 },
    { 0x4c6e0004, SUC_lexicon_tags_28 },
    { 0xce2e0005, SUC_lexicon_tags_11 },
    { 0x2a2e0006, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x39ae0008, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeee000b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ee000f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x878e0011, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x64ee0013, SUC_lexicon_tags_15 },
    { 0xb6e0014, SUC_lexicon_tags_23 },
    { 0xcd8e0015, SUC_lexicon_tags_11 },
    { 0xca2e0014, SUC_lexicon_tags_12 },
    { 0x79ce0017, SUC_lexicon_tags_49 },
    { 0x4dce0018, SUC_lexicon_tags_12 },
    { 0xc56e0018, SUC_lexicon_tags_11 },
    { 0xf5ee0018, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ce001d, SUC_lexicon_tags_11 },
    { 0x3dce001e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x234e0021, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ee0024, SUC_lexicon_tags_37 },
    { 0xc5ee0025, SUC_lexicon_tags_17 },
    { 0xaee0026, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x808e0028, SUC_lexicon_tags_23 },
    { 0x7c8e0028, SUC_lexicon_tags_218 },
    { 0x21ee002a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258e0034, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dae0037, SUC_lexicon_tags_60 },
    { 0x20ce0038, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ae003e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x264e0040, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x78ee0042, SUC_lexicon_tags_27 },
    { 0xf76e0043, SUC_lexicon_tags_11 },
    { 0xc6e0044, SUC_lexicon_tags_6 },
    { 0xd98e0043, SUC_lexicon_tags_62 },
    { 0x54ee0046, SUC_lexicon_tags_15 },
    { 0x5dee0047, SUC_lexicon_tags_15 },
    { 0x8eee0048, SUC_lexicon_tags_14 },
    { 0x9b6e0048, SUC_lexicon_tags_14 },
    { 0x2e4e004a, SUC_lexicon_tags_1 },
    { 0x7dee0044, SUC_lexicon_tags_25 },
    { 0x848e004c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ce0053, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe68e0055, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd74e0057, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50e005c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x622e005e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8e0061, SUC_lexicon_tags_14 },
    { 0x648e0062, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc8e0064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x440e0067, SUC_lexicon_tags_1 },
    { 0x31ee0067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b8e006a, SUC_lexicon_tags_43 },
    { 0xb7ee006a, SUC_lexicon_tags_11 },
    { 0x5b0e006c, SUC_lexicon_tags_38 },
    { 0x15ce006d, SUC_lexicon_tags_2 },
    { 0x73ae006d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dce0071, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894e0075, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x988e0078, SUC_lexicon_tags_1 },
    { 0xacce0079, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae007d, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4e0084, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ce0089, SUC_lexicon_tags_18 },
    { 0xf6ce008a, SUC_lexicon_tags_12 },
    { 0x94ee008b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428e0090, SUC_lexicon_tags_12 },
    { 0x87ae0091, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee0096, SUC_lexicon_tags_33 },
    { 0x7dce0096, SUC_lexicon_tags_55 },
    { 0xf84e0096, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2e00a0, SUC_lexicon_tags_49 },
    { 0x7fae00a0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfdee00a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ce00a7, SUC_lexicon_tags_10 },
    { 0xb96e00a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x39ae00aa, SUC_lexicon_tags_14 },
    { 0x8ee00ab, SUC_lexicon_tags_77 },
    { 0x210e00ac, SUC_lexicon_tags_14 },
    { 0x31ce00ad, SUC_lexicon_tags_17 },
    { 0xffee00aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb96e00b0, SUC_lexicon_tags_38 },
    { 0x492e00b1, SUC_lexicon_tags_79 },
    { 0xb42e00b2, SUC_lexicon_tags_16 },
    { 0x3fee00b1, SUC_lexicon_tags_37 },
    { 0x6ee00b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626e00b8, SUC_lexicon_tags_77 },
    { 0x804e00b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x860e00bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44e00be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dee00c8, SUC_lexicon_tags_1 },
    { 0x30ee00c9, SUC_lexicon_tags_2 },
    { 0x628e00ca, SUC_lexicon_tags_6 },
    { 0x952e00ca, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0xc2ce00cd, SUC_lexicon_tags_63 },
    { 0x16ce00cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd24e00d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e00e2, SUC_lexicon_tags_28 },
    { 0x27ce00e3, SUC_lexicon_tags_49 },
    { 0x898e00e4, SUC_lexicon_tags_23 },
    { 0xd32e00e3, SUC_lexicon_tags_15 },
    { 0x780e00e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e00ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x666e00ef, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0e00f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xccae00fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x306e00fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x61ae00ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0e0108, SUC_lexicon_tags_63 },
    { 0x83ae0109, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516e010d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cee0112, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6e011c, SUC_lexicon_tags_17 },
    { 0xb6e011c, SUC_lexicon_tags_9 },
    { 0x8bce011c, SUC_lexicon_tags_28 },
    { 0xc6ce011f, SUC_lexicon_tags_28 },
    { 0xc48e011d, SUC_lexicon_tags_17 },
    { 0x438e0121, SUC_lexicon_tags_16 },
    { 0xe02e0122, SUC_lexicon_tags_53 },
    { 0x2c4e0122, SUC_lexicon_tags_5 },
    { 0xaace0124, SUC_lexicon_tags_11 },
    { 0x5e8e0122, SUC_lexicon_tags_7 },
    { 0x782e0126, SUC_lexicon_tags_37 },
    { 0x8bee0127, SUC_lexicon_tags_1 },
    { 0xea0e0126, SUC_lexicon_tags_10 },
    { 0xf40e0126, SUC_lexicon_tags_39 },
    { 0x9d2e012a, SUC_lexicon_tags_6 },
    { 0x72ee012b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x658e012d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x888e0130, SUC_lexicon_tags_25 },
    { 0x804e0131, SUC_lexicon_tags_2 },
    { 0x836e0132, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42e0136, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ce013a, SUC_lexicon_tags_8 },
    { 0xab2e013a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe42e013d, SUC_lexicon_tags_23 },
    { 0xb42e013e, SUC_lexicon_tags_32 },
    { 0x3fee013f, SUC_lexicon_tags_37 },
    { 0x4e4e0140, SUC_lexicon_tags_25 },
    { 0x512e0140, SUC_lexicon_tags_6 },
    { 0x3d4e0142, SUC_lexicon_tags_2 },
    { 0x11ce0142, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452e0147, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x12e014a, SUC_lexicon_tags_1 },
    { 0x2b4e014a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24e014e, SUC_lexicon_tags_25 },
    { 0x684e014f, SUC_lexicon_tags_10 },
    { 0xa6ae0150, SUC_lexicon_tags_12 },
    { 0xaf2e014f, SUC_lexicon_tags_37 },
    { 0xfece0152, SUC_lexicon_tags_25 },
    { 0xc04e0151, SUC_lexicon_tags_26 },
    { 0x864e0154, SUC_lexicon_tags_4 },
    { 0x58ae0155, SUC_lexicon_tags_45 },
    { 0xef8e0156, SUC_lexicon_tags_25 },
    { 0xd10e0157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac8e0159, SUC_lexicon_tags_60 },
    { 0x640e015a, SUC_lexicon_tags_11 },
    { 0x8f8e0159, SUC_lexicon_tags_84 },
    { 0x562e015a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb96e015e, SUC_lexicon_tags_17 },
    { 0xcdae015f, SUC_lexicon_tags_9 },
    { 0x172e015f, SUC_lexicon_tags_2 },
    { 0x95ae015e, SUC_lexicon_tags_25 },
    { 0xae4e0162, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6e0165, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4e0169, SUC_lexicon_tags_2 },
    { 0xff4e016a, SUC_lexicon_tags_1455 },
    { 0, NULL },
    { 0, NULL },
    { 0x438e016d, SUC_lexicon_tags_37 },
    { 0xe90e016e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x902e0171, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xae6e0174, SUC_lexicon_tags_7 },
    { 0x408e0174, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x352e017d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02e0180, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4e0184, SUC_lexicon_tags_18 },
    { 0x326e0184, SUC_lexicon_tags_28 },
    { 0x828e0185, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8e018b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768e0190, SUC_lexicon_tags_7 },
    { 0x26ce0191, SUC_lexicon_tags_15 },
    { 0x192e0192, SUC_lexicon_tags_62 },
    { 0x884e0193, SUC_lexicon_tags_68 },
    { 0xa6ce0193, SUC_lexicon_tags_46 },
    { 0xdd8e0195, SUC_lexicon_tags_27 },
    { 0x746e0195, SUC_lexicon_tags_37 },
    { 0x534e0197, SUC_lexicon_tags_26 },
    { 0xf2e0194, SUC_lexicon_tags_44 },
    { 0x5e2e0196, SUC_lexicon_tags_2 },
    { 0x9c8e019a, SUC_lexicon_tags_19 },
    { 0x644e0199, SUC_lexicon_tags_14 },
    { 0x638e019c, SUC_lexicon_tags_11 },
    { 0xe6ee0193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54e01a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bae01a6, SUC_lexicon_tags_37 },
    { 0xe02e01a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e6e01a9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x85ce01ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ce01b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ee01b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x604e01b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938e01be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aee01c1, SUC_lexicon_tags_37 },
    { 0x492e01c2, SUC_lexicon_tags_63 },
    { 0x100e01c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19ae01c5, SUC_lexicon_tags_12 },
    { 0x780e01c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75ae01c8, SUC_lexicon_tags_14 },
    { 0x39ce01c8, SUC_lexicon_tags_34 },
    { 0x124e01c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x42ee01cc, SUC_lexicon_tags_15 },
    { 0x132e01cd, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ae01da, SUC_lexicon_tags_44 },
    { 0xf3ee01db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784e01df, SUC_lexicon_tags_17 },
    { 0xbce01df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444e01e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ee01e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x542e01ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40e01ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336e01f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb48e01f6, SUC_lexicon_tags_37 },
    { 0x3d4e01f7, SUC_lexicon_tags_16 },
    { 0x7f6e01f8, SUC_lexicon_tags_11 },
    { 0xa8ae01f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7eee01fb, SUC_lexicon_tags_46 },
    { 0x320e01fc, SUC_lexicon_tags_25 },
    { 0x4eee01fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ce0200, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622e0205, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fae020a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30e020d, SUC_lexicon_tags_17 },
    { 0xc02e020d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cee0211, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ee0217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4e021b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fce021e, SUC_lexicon_tags_11 },
    { 0x834e021e, SUC_lexicon_tags_32 },
    { 0xe18e0220, SUC_lexicon_tags_38 },
    { 0x8bae0221, SUC_lexicon_tags_11 },
    { 0x804e0221, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x61ae0224, SUC_lexicon_tags_23 },
    { 0xa40e0224, SUC_lexicon_tags_27 },
    { 0xe96e0224, SUC_lexicon_tags_8 },
    { 0x470e0227, SUC_lexicon_tags_25 },
    { 0x34ae0228, SUC_lexicon_tags_63 },
    { 0xf4ee0227, SUC_lexicon_tags_25 },
    { 0xd8ae0228, SUC_lexicon_tags_26 },
    { 0xbdae0225, SUC_lexicon_tags_15 },
    { 0x302e022c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x544e022f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf86e0231, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108e0239, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37ae023b, SUC_lexicon_tags_15 },
    { 0xf1ce023c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eae0241, SUC_lexicon_tags_37 },
    { 0x44ae0242, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16e0246, SUC_lexicon_tags_34 },
    { 0x670e0247, SUC_lexicon_tags_7 },
    { 0xd44e0248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72e024b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8e0252, SUC_lexicon_tags_22 },
    { 0x59ee0253, SUC_lexicon_tags_148 },
    { 0x72e0254, SUC_lexicon_tags_11 },
    { 0x4f0e0254, SUC_lexicon_tags_65 },
    { 0x1aae0254, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x416e0258, SUC_lexicon_tags_1 },
    { 0x66ae0259, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x692e025c, SUC_lexicon_tags_7 },
    { 0xea8e025d, SUC_lexicon_tags_7 },
    { 0xd3ae025e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8e0262, SUC_lexicon_tags_57 },
    { 0xd4e0263, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce0266, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8e026b, SUC_lexicon_tags_15 },
    { 0xec6e026c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x694e026f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ce0278, SUC_lexicon_tags_14 },
    { 0x46ee0279, SUC_lexicon_tags_23 },
    { 0x3bee0279, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2e0289, SUC_lexicon_tags_34 },
    { 0x176e028a, SUC_lexicon_tags_19 },
    { 0x70ce028b, SUC_lexicon_tags_25 },
    { 0xe80e028c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ae0291, SUC_lexicon_tags_7 },
    { 0xb46e0292, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ce0297, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0e029e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e02a1, SUC_lexicon_tags_613 },
    { 0xebee02a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6e02a5, SUC_lexicon_tags_87 },
    { 0x35ee02a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6e02ad, SUC_lexicon_tags_13 },
    { 0x9e0e02ae, SUC_lexicon_tags_60 },
    { 0xce2e02ad, SUC_lexicon_tags_14 },
    { 0x8ace02b0, SUC_lexicon_tags_12 },
    { 0x6f4e02b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9ce02b3, SUC_lexicon_tags_0 },
    { 0xe26e02b4, SUC_lexicon_tags_1 },
    { 0xf76e02b5, SUC_lexicon_tags_61 },
    { 0x606e02b6, SUC_lexicon_tags_37 },
    { 0x702e02b7, SUC_lexicon_tags_1 },
    { 0x738e02b8, SUC_lexicon_tags_1 },
    { 0xffae02b9, SUC_lexicon_tags_8 },
    { 0xdd0e02b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98e02c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x530e02c5, SUC_lexicon_tags_26 },
    { 0x4d0e02c6, SUC_lexicon_tags_60 },
    { 0xec2e02c7, SUC_lexicon_tags_38 },
    { 0xd04e02c8, SUC_lexicon_tags_21 },
    { 0x8bee02c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4e02ce, SUC_lexicon_tags_17 },
    { 0x366e02cf, SUC_lexicon_tags_38 },
    { 0x514e02cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x850e02d2, SUC_lexicon_tags_62 },
    { 0x99ee02d3, SUC_lexicon_tags_37 },
    { 0x6e6e02d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa7ae02d6, SUC_lexicon_tags_104 },
    { 0xf7ee02d6, SUC_lexicon_tags_37 },
    { 0x7c2e02d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x438e02db, SUC_lexicon_tags_262 },
    { 0x368e02db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32e02e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ce02e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598e02e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50e02f1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540e02f5, SUC_lexicon_tags_10 },
    { 0x3f0e02f6, SUC_lexicon_tags_70 },
    { 0xf3ae02f7, SUC_lexicon_tags_15 },
    { 0x44e02f8, SUC_lexicon_tags_60 },
    { 0x4c4e02f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ee02fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8e02ff, SUC_lexicon_tags_2 },
    { 0x9c0e02ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e8e0302, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ae0305, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x398e0308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x420e030b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x76e030d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8e0313, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x81ee0315, SUC_lexicon_tags_54 },
    { 0x48e0315, SUC_lexicon_tags_27 },
    { 0xbeee0317, SUC_lexicon_tags_23 },
    { 0xd08e0315, SUC_lexicon_tags_38 },
    { 0x26ae0319, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x298e031b, SUC_lexicon_tags_14 },
    { 0x4eee031c, SUC_lexicon_tags_27 },
    { 0xed8e031d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2e0321, SUC_lexicon_tags_7 },
    { 0x6fae0322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126e0326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ace0328, SUC_lexicon_tags_37 },
    { 0x47ce0328, SUC_lexicon_tags_32 },
    { 0xa00e0328, SUC_lexicon_tags_17 },
    { 0x6ee032b, SUC_lexicon_tags_101 },
    { 0xdd0e0328, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xec6e032f, SUC_lexicon_tags_30 },
    { 0x8b2e0330, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb00e0332, SUC_lexicon_tags_25 },
    { 0x874e0333, SUC_lexicon_tags_19 },
    { 0xace0332, SUC_lexicon_tags_15 },
    { 0xa70e0334, SUC_lexicon_tags_59 },
    { 0xc8ee0334, SUC_lexicon_tags_29 },
    { 0xda6e0332, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ee033a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826e0341, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xab0e0343, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ce0347, SUC_lexicon_tags_47 },
    { 0x1d8e0348, SUC_lexicon_tags_19 },
    { 0x884e0349, SUC_lexicon_tags_37 },
    { 0xf08e034a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9e8e034c, SUC_lexicon_tags_2 },
    { 0xa36e034d, SUC_lexicon_tags_11 },
    { 0x694e034c, SUC_lexicon_tags_11 },
    { 0xa82e034d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6e0357, SUC_lexicon_tags_8 },
    { 0xb94e0357, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe20e035a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2e035d, SUC_lexicon_tags_24 },
    { 0x744e035e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8e0363, SUC_lexicon_tags_37 },
    { 0xda4e0364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44e036a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4e036d, SUC_lexicon_tags_15 },
    { 0x14ee036e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48e0371, SUC_lexicon_tags_6 },
    { 0xbbce0372, SUC_lexicon_tags_85 },
    { 0x192e0373, SUC_lexicon_tags_1 },
    { 0x55ae0374, SUC_lexicon_tags_1 },
    { 0x584e0373, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf58e0377, SUC_lexicon_tags_31 },
    { 0x548e0378, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ce037b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ce0381, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x448e0384, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x948e0386, SUC_lexicon_tags_75 },
    { 0xde8e0387, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee038d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4e0390, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ce0394, SUC_lexicon_tags_7 },
    { 0x23ee0395, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa5ce0397, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae8e0399, SUC_lexicon_tags_7 },
    { 0x80ce039a, SUC_lexicon_tags_8 },
    { 0x76ce039b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60e039e, SUC_lexicon_tags_2 },
    { 0x908e039e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x744e03a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e03a9, SUC_lexicon_tags_2 },
    { 0x2d0e03a9, SUC_lexicon_tags_15 },
    { 0x4c8e03aa, SUC_lexicon_tags_27 },
    { 0x22ee03ac, SUC_lexicon_tags_17 },
    { 0x326e03ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fce03b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf82e03b2, SUC_lexicon_tags_2 },
    { 0xfb6e03b3, SUC_lexicon_tags_15 },
    { 0x2fae03b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb76e03b6, SUC_lexicon_tags_37 },
    { 0xda4e03b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688e03bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8e03c2, SUC_lexicon_tags_7 },
    { 0x122e03c2, SUC_lexicon_tags_22 },
    { 0x24ee03c4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e03c9, SUC_lexicon_tags_56 },
    { 0xfeae03c9, SUC_lexicon_tags_23 },
    { 0xbe8e03ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e03ce, SUC_lexicon_tags_29 },
    { 0x9bce03ce, SUC_lexicon_tags_26 },
    { 0x324e03cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ae03d5, SUC_lexicon_tags_14 },
    { 0x1d6e03d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafae03d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x550e03da, SUC_lexicon_tags_15 },
    { 0xe7ae03db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x732e03dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44ee03df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e03e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d0e03e5, SUC_lexicon_tags_48 },
    { 0xf78e03e6, SUC_lexicon_tags_25 },
    { 0x24ce03e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fae03eb, SUC_lexicon_tags_402 },
    { 0x15ce03ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74e03f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35ee03f3, SUC_lexicon_tags_15 },
    { 0x612e03f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x160e03f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8e03fa, SUC_lexicon_tags_25 },
    { 0x72ee03fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x930e03fd, SUC_lexicon_tags_17 },
    { 0x7d8e03fe, SUC_lexicon_tags_19 },
    { 0xc1ae03fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902e0403, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42e0407, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dee0409, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460e040e, SUC_lexicon_tags_25 },
    { 0x148e040f, SUC_lexicon_tags_1 },
    { 0x9aae040f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ae0414, SUC_lexicon_tags_12 },
    { 0xdfce0415, SUC_lexicon_tags_1 },
    { 0x758e0415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x392e0419, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ae041d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ce0422, SUC_lexicon_tags_11 },
    { 0xb0ce0422, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8e0426, SUC_lexicon_tags_37 },
    { 0x38ae0427, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b6e0429, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e042e, SUC_lexicon_tags_47 },
    { 0x870e042f, SUC_lexicon_tags_16 },
    { 0x672e042e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14e0433, SUC_lexicon_tags_47 },
    { 0x104e0434, SUC_lexicon_tags_23 },
    { 0xd02e0435, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa54e0437, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fee043a, SUC_lexicon_tags_527 },
    { 0x73ce043b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376e043f, SUC_lexicon_tags_44 },
    { 0x12ee0440, SUC_lexicon_tags_1 },
    { 0x8d6e0440, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eee0447, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948e044b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e044f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0e0451, SUC_lexicon_tags_47 },
    { 0x134e0452, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x108e0455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8e0457, SUC_lexicon_tags_14 },
    { 0x8bae0458, SUC_lexicon_tags_55 },
    { 0x472e0459, SUC_lexicon_tags_1 },
    { 0x160e0459, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae4e045c, SUC_lexicon_tags_31 },
    { 0x906e045d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ce0462, SUC_lexicon_tags_15 },
    { 0x970e0462, SUC_lexicon_tags_27 },
    { 0x438e0464, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e046b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86e0473, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e0477, SUC_lexicon_tags_84 },
    { 0x254e0477, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x628e047b, SUC_lexicon_tags_16 },
    { 0x34ee047b, SUC_lexicon_tags_11 },
    { 0x580e047d, SUC_lexicon_tags_16 },
    { 0x32e047c, SUC_lexicon_tags_33 },
    { 0xe4ae047c, SUC_lexicon_tags_26 },
    { 0x594e0480, SUC_lexicon_tags_31 },
    { 0x426e047c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x330e0484, SUC_lexicon_tags_37 },
    { 0xa84e0485, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4e048e, SUC_lexicon_tags_37 },
    { 0xd6ce048f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70e0493, SUC_lexicon_tags_17 },
    { 0x3e2e0493, SUC_lexicon_tags_16 },
    { 0xbd0e0493, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ce0499, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206e049d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x238e04a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ee04a7, SUC_lexicon_tags_26 },
    { 0xee0e04a8, SUC_lexicon_tags_1 },
    { 0x826e04a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6e04ac, SUC_lexicon_tags_27 },
    { 0xcf0e04ad, SUC_lexicon_tags_14 },
    { 0xf2e04ae, SUC_lexicon_tags_4 },
    { 0x49ce04ad, SUC_lexicon_tags_60 },
    { 0xd3ee04b0, SUC_lexicon_tags_1 },
    { 0x946e04b1, SUC_lexicon_tags_25 },
    { 0x74e04b0, SUC_lexicon_tags_1 },
    { 0x422e04b3, SUC_lexicon_tags_8 },
    { 0x7a2e04b0, SUC_lexicon_tags_63 },
    { 0xb7ce04af, SUC_lexicon_tags_7 },
    { 0xcb4e04b1, SUC_lexicon_tags_26 },
    { 0x2e8e04b5, SUC_lexicon_tags_7 },
    { 0x6ace04b8, SUC_lexicon_tags_16 },
    { 0xc0ee04ad, SUC_lexicon_tags_2 },
    { 0x33ee04ba, SUC_lexicon_tags_15 },
    { 0xfae04ba, SUC_lexicon_tags_27 },
    { 0xa3ae04b2, SUC_lexicon_tags_60 },
    { 0xa0e04bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce04c1, SUC_lexicon_tags_2 },
    { 0xaae04c2, SUC_lexicon_tags_11 },
    { 0x230e04c2, SUC_lexicon_tags_8 },
    { 0xa2ee04c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eee04c7, SUC_lexicon_tags_47 },
    { 0x1d0e04c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30e04cf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148e04d4, SUC_lexicon_tags_8 },
    { 0x6fee04d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb2e04d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888e04df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cee04e4, SUC_lexicon_tags_25 },
    { 0x7d0e04e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ce04f3, SUC_lexicon_tags_55 },
    { 0xe28e04f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x856e04f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84ee04f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8e04fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ee0505, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0e050f, SUC_lexicon_tags_37 },
    { 0xdee0510, SUC_lexicon_tags_45 },
    { 0x904e0511, SUC_lexicon_tags_7 },
    { 0x188e0512, SUC_lexicon_tags_25 },
    { 0x200e0513, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8e0516, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd94e0518, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab2e051a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48e051f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ce052a, SUC_lexicon_tags_27 },
    { 0xa56e052b, SUC_lexicon_tags_11 },
    { 0x8d0e052c, SUC_lexicon_tags_46 },
    { 0x920e052d, SUC_lexicon_tags_12 },
    { 0x84ae052e, SUC_lexicon_tags_11 },
    { 0xfaae052f, SUC_lexicon_tags_43 },
    { 0x410e052e, SUC_lexicon_tags_23 },
    { 0x14ee0531, SUC_lexicon_tags_2 },
    { 0xaece0531, SUC_lexicon_tags_19 },
    { 0xd7ee0530, SUC_lexicon_tags_5 },
    { 0x7d2e052e, SUC_lexicon_tags_13 },
    { 0x43ee0535, SUC_lexicon_tags_8 },
    { 0x14ee0536, SUC_lexicon_tags_2 },
    { 0xbf2e0533, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172e053e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844e0542, SUC_lexicon_tags_2 },
    { 0x406e0543, SUC_lexicon_tags_68 },
    { 0x270e0544, SUC_lexicon_tags_21 },
    { 0xfece0545, SUC_lexicon_tags_25 },
    { 0xb4ce0546, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6e054c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252e0550, SUC_lexicon_tags_25 },
    { 0xf2ce0551, SUC_lexicon_tags_7 },
    { 0x47ee0552, SUC_lexicon_tags_15 },
    { 0xcb2e0553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x520e0556, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748e055c, SUC_lexicon_tags_11 },
    { 0xdece055d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754e0568, SUC_lexicon_tags_29 },
    { 0x2eee0569, SUC_lexicon_tags_1 },
    { 0xe94e056a, SUC_lexicon_tags_87 },
    { 0xe056a, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ce056e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x10ce0570, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14ae0572, SUC_lexicon_tags_12 },
    { 0x7eee0572, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ce057c, SUC_lexicon_tags_32 },
    { 0x792e057c, SUC_lexicon_tags_11 },
    { 0x63ee057e, SUC_lexicon_tags_6 },
    { 0xc7ae057c, SUC_lexicon_tags_10 },
    { 0xa9ce057e, SUC_lexicon_tags_1 },
    { 0x388e0581, SUC_lexicon_tags_11 },
    { 0xe8ee0581, SUC_lexicon_tags_12 },
    { 0x242e0581, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9bce0585, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734e058a, SUC_lexicon_tags_17 },
    { 0x7cee058a, SUC_lexicon_tags_11 },
    { 0x3fae058c, SUC_lexicon_tags_26 },
    { 0x9f4e058d, SUC_lexicon_tags_816 },
    { 0x1e8e058c, SUC_lexicon_tags_7 },
    { 0x634e058c, SUC_lexicon_tags_17 },
    { 0xa52e058b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638e0595, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aae0599, SUC_lexicon_tags_45 },
    { 0x39ee059a, SUC_lexicon_tags_15 },
    { 0xd4e0599, SUC_lexicon_tags_17 },
    { 0x632e059c, SUC_lexicon_tags_45 },
    { 0x2aee059d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x268e059f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x616e05a1, SUC_lexicon_tags_38 },
    { 0x2bee05a2, SUC_lexicon_tags_7 },
    { 0x542e05a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0e05a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ce05a9, SUC_lexicon_tags_23 },
    { 0xb3ae05aa, SUC_lexicon_tags_15 },
    { 0x34ae05ab, SUC_lexicon_tags_14 },
    { 0xdb4e05aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00e05b2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee05b8, SUC_lexicon_tags_17 },
    { 0x7c2e05b9, SUC_lexicon_tags_19 },
    { 0x20ae05b8, SUC_lexicon_tags_14 },
    { 0xabee05bb, SUC_lexicon_tags_14 },
    { 0xa2ce05bb, SUC_lexicon_tags_65 },
    { 0x72ee05bd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954e05c2, SUC_lexicon_tags_47 },
    { 0x6b8e05c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8e05c9, SUC_lexicon_tags_25 },
    { 0x17ee05ca, SUC_lexicon_tags_63 },
    { 0xe3ee05cb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3b2e05cd, SUC_lexicon_tags_49 },
    { 0x33ce05cd, SUC_lexicon_tags_1 },
    { 0xd66e05cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2e05d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe8ce05d3, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0x902e05d5, SUC_lexicon_tags_10 },
    { 0xcd2e05d6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x5d4e05d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ee05db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d4e05dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686e05e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dee05e5, SUC_lexicon_tags_31 },
    { 0xdece05e6, SUC_lexicon_tags_29 },
    { 0x18ee05e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6e05ea, SUC_lexicon_tags_45 },
    { 0xca8e05ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456e05ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd66e05f1, SUC_lexicon_tags_29 },
    { 0x4b8e05f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeaae05f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4e05f8, SUC_lexicon_tags_17 },
    { 0x36ee05f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x122e05fb, SUC_lexicon_tags_60 },
    { 0x312e05fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce0600, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ce0604, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf2e0606, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x610e0608, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e060b, SUC_lexicon_tags_57 },
    { 0x53ae060c, SUC_lexicon_tags_22 },
    { 0x53ae060d, SUC_lexicon_tags_37 },
    { 0x4c6e060c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fae0612, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4ce0614, SUC_lexicon_tags_14 },
    { 0x4cce0615, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ee0618, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30ce061a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5ee061c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ce061f, SUC_lexicon_tags_2 },
    { 0xfeae0620, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xda4e0622, SUC_lexicon_tags_49 },
    { 0xeee0623, SUC_lexicon_tags_11 },
    { 0xd9ce0624, SUC_lexicon_tags_37 },
    { 0x8b0e0623, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafce0637, SUC_lexicon_tags_37 },
    { 0x804e0637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x874e063a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356e0645, SUC_lexicon_tags_37 },
    { 0x6d4e0645, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ce0649, SUC_lexicon_tags_60 },
    { 0xab8e0649, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6e064e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x246e0650, SUC_lexicon_tags_48 },
    { 0x7ece0651, SUC_lexicon_tags_0 },
    { 0x9ae0652, SUC_lexicon_tags_38 },
    { 0x20ee0653, SUC_lexicon_tags_35 },
    { 0x818e0654, SUC_lexicon_tags_44 },
    { 0x9fce0651, SUC_lexicon_tags_1 },
    { 0xfa2e0656, SUC_lexicon_tags_11 },
    { 0x6b8e0656, SUC_lexicon_tags_17 },
    { 0x9f6e0658, SUC_lexicon_tags_23 },
    { 0xf20e0651, SUC_lexicon_tags_37 },
    { 0x26ce0658, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730e0660, SUC_lexicon_tags_39 },
    { 0x594e0661, SUC_lexicon_tags_63 },
    { 0x108e0662, SUC_lexicon_tags_27 },
    { 0x94ee0660, SUC_lexicon_tags_5 },
    { 0xb54e0664, SUC_lexicon_tags_55 },
    { 0xd42e0664, SUC_lexicon_tags_30 },
    { 0xdf8e0663, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ee066a, SUC_lexicon_tags_55 },
    { 0x390e066b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83ce066d, SUC_lexicon_tags_83 },
    { 0xb48e066e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe6ce0670, SUC_lexicon_tags_37 },
    { 0x8a4e0671, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x764e0673, SUC_lexicon_tags_25 },
    { 0x5b0e0673, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2e0676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14ee0678, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa78e067a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf92e067c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888e0681, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6e0684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610e0688, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8f8e068a, SUC_lexicon_tags_25 },
    { 0x790e068b, SUC_lexicon_tags_37 },
    { 0x682e068b, SUC_lexicon_tags_17 },
    { 0xfc6e068c, SUC_lexicon_tags_32 },
    { 0x9ae068b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x234e0690, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ce0694, SUC_lexicon_tags_9 },
    { 0xb96e0695, SUC_lexicon_tags_11 },
    { 0x410e0696, SUC_lexicon_tags_33 },
    { 0xb6ee0695, SUC_lexicon_tags_7 },
    { 0xa1ce0698, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6e06a2, SUC_lexicon_tags_8 },
    { 0x736e06a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x936e06a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ce06a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598e06ae, SUC_lexicon_tags_7 },
    { 0xf9ae06af, SUC_lexicon_tags_38 },
    { 0x972e06af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620e06b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x136e06b7, SUC_lexicon_tags_21 },
    { 0x884e06b8, SUC_lexicon_tags_49 },
    { 0x286e06b9, SUC_lexicon_tags_144 },
    { 0x882e06ba, SUC_lexicon_tags_23 },
    { 0xa6ce06ba, SUC_lexicon_tags_16 },
    { 0x9fce06b8, SUC_lexicon_tags_1 },
    { 0x878e06bc, SUC_lexicon_tags_87 },
    { 0x64ce06be, SUC_lexicon_tags_37 },
    { 0x54e06bf, SUC_lexicon_tags_38 },
    { 0xd30e06ba, SUC_lexicon_tags_1 },
    { 0xa14e06ba, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x446e06c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4e06cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc6e06ce, SUC_lexicon_tags_2 },
    { 0xfc6e06ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ee06d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x516e06d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220e06dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1dce06df, SUC_lexicon_tags_1 },
    { 0xd6e06e0, SUC_lexicon_tags_45 },
    { 0x2b4e06e0, SUC_lexicon_tags_1 },
    { 0x912e06e2, SUC_lexicon_tags_2 },
    { 0xc2e06e2, SUC_lexicon_tags_8 },
    { 0x38ee06e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc8ae06e6, SUC_lexicon_tags_29 },
    { 0x11ce06e6, SUC_lexicon_tags_2 },
    { 0x6a6e06e8, SUC_lexicon_tags_31 },
    { 0x4fee06e9, SUC_lexicon_tags_11 },
    { 0xbfae06e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910e06ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ce06f0, SUC_lexicon_tags_17 },
    { 0x4ece06f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0e06f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38e06fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x982e06fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb7ce06fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ae0701, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0e0704, SUC_lexicon_tags_0 },
    { 0xafce0704, SUC_lexicon_tags_7 },
    { 0xc42e0704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2e070d, SUC_lexicon_tags_27 },
    { 0x3dae070d, SUC_lexicon_tags_0 },
    { 0xf66e070f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8fae0711, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ee0717, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0e071a, SUC_lexicon_tags_18 },
    { 0x2bae071a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae0720, SUC_lexicon_tags_1 },
    { 0x340e0721, SUC_lexicon_tags_17 },
    { 0x692e0721, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf6ce0724, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92e0726, SUC_lexicon_tags_17 },
    { 0x3d0e0727, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ee072b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56e072e, SUC_lexicon_tags_25 },
    { 0x25ae072e, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6e0732, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e0737, SUC_lexicon_tags_17 },
    { 0x60ee0737, SUC_lexicon_tags_11 },
    { 0xbaee0739, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6e0745, SUC_lexicon_tags_23 },
    { 0xf36e0745, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e0749, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0e074c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6e0753, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ce0756, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbce0759, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x302e075b, SUC_lexicon_tags_25 },
    { 0x616e075b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ae075f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e0765, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ace0767, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50e076b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ce076e, SUC_lexicon_tags_30 },
    { 0x5d0e076e, SUC_lexicon_tags_43 },
    { 0x4f6e0770, SUC_lexicon_tags_37 },
    { 0x93ce0771, SUC_lexicon_tags_65 },
    { 0xce0e0772, SUC_lexicon_tags_11 },
    { 0x6d6e0772, SUC_lexicon_tags_30 },
    { 0xc7ce0773, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x202e0777, SUC_lexicon_tags_0 },
    { 0xdbee0777, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaee0780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32e0783, SUC_lexicon_tags_37 },
    { 0x362e0784, SUC_lexicon_tags_45 },
    { 0x92ae0785, SUC_lexicon_tags_0 },
    { 0xb96e0786, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4ae0788, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x906e078b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32e0790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6e0794, SUC_lexicon_tags_17 },
    { 0x39ee0795, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd78e0797, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd12e0799, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376e07a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa20e07a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xea8e07a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x378e07a8, SUC_lexicon_tags_23 },
    { 0xae4e07a9, SUC_lexicon_tags_25 },
    { 0xb9ee07a9, SUC_lexicon_tags_6 },
    { 0xf52e07ab, SUC_lexicon_tags_68 },
    { 0xfbae07aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa72e07ae, SUC_lexicon_tags_37 },
    { 0x836e07af, SUC_lexicon_tags_37 },
    { 0x10ae07b0, SUC_lexicon_tags_27 },
    { 0x596e07b1, SUC_lexicon_tags_32 },
    { 0x40ee07b0, SUC_lexicon_tags_60 },
    { 0xc7ae07b0, SUC_lexicon_tags_16 },
    { 0xba0e07b1, SUC_lexicon_tags_25 },
    { 0x518e07b5, SUC_lexicon_tags_17 },
    { 0xc0e07b5, SUC_lexicon_tags_21 },
    { 0xadce07b5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ee07ba, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xf3ce07bc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x10e07be, SUC_lexicon_tags_23 },
    { 0x548e07bf, SUC_lexicon_tags_14 },
    { 0x7ece07c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4e07c7, SUC_lexicon_tags_16 },
    { 0xc56e07c8, SUC_lexicon_tags_27 },
    { 0xd68e07c9, SUC_lexicon_tags_25 },
    { 0xfaee07c8, SUC_lexicon_tags_15 },
    { 0xb50e07ca, SUC_lexicon_tags_6 },
    { 0xa60e07ca, SUC_lexicon_tags_14 },
    { 0x444e07cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78e07d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ae07d4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd0ae07d6, SUC_lexicon_tags_37 },
    { 0x5c6e07d6, SUC_lexicon_tags_28 },
    { 0xa6ee07d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x762e07da, SUC_lexicon_tags_17 },
    { 0x5ace07da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae07de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60e07e2, SUC_lexicon_tags_27 },
    { 0xc98e07e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee07e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6e07eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee07f2, SUC_lexicon_tags_25 },
    { 0x718e07f2, SUC_lexicon_tags_70 },
    { 0xdc2e07f2, SUC_lexicon_tags_11 },
    { 0x9ace07f2, SUC_lexicon_tags_17 },
    { 0xcb8e07f4, SUC_lexicon_tags_47 },
    { 0xd6e07f7, SUC_lexicon_tags_25 },
    { 0x62ce07f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6e07fe, SUC_lexicon_tags_17 },
    { 0x23ee07fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4e0803, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ae0806, SUC_lexicon_tags_2 },
    { 0x19ee0807, SUC_lexicon_tags_87 },
    { 0x3b0e0808, SUC_lexicon_tags_11 },
    { 0xd22e0808, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeae080d, SUC_lexicon_tags_32 },
    { 0xfc8e080d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0e0812, SUC_lexicon_tags_66 },
    { 0x680e0813, SUC_lexicon_tags_37 },
    { 0x58ce0814, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ae0817, SUC_lexicon_tags_44 },
    { 0x9bae0817, SUC_lexicon_tags_15 },
    { 0x68e0819, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce081d, SUC_lexicon_tags_25 },
    { 0xfd8e081e, SUC_lexicon_tags_60 },
    { 0x5cee081f, SUC_lexicon_tags_47 },
    { 0xc4ee081e, SUC_lexicon_tags_37 },
    { 0x592e081d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8e0826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x496e0828, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68e082f, SUC_lexicon_tags_47 },
    { 0x24ae0830, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430e0835, SUC_lexicon_tags_63 },
    { 0xc2ae0835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0e083d, SUC_lexicon_tags_25 },
    { 0xc5ce083e, SUC_lexicon_tags_11 },
    { 0xd2ce083d, SUC_lexicon_tags_12 },
    { 0x410e083d, SUC_lexicon_tags_7 },
    { 0xd8e0841, SUC_lexicon_tags_11 },
    { 0x656e0841, SUC_lexicon_tags_7 },
    { 0x54e0842, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x2c8e0845, SUC_lexicon_tags_37 },
    { 0xf34e0846, SUC_lexicon_tags_8 },
    { 0x59ee0847, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb64e0849, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6e084e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0e0851, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ce0858, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x542e085a, SUC_lexicon_tags_15 },
    { 0x1f2e085a, SUC_lexicon_tags_17 },
    { 0xdfee085c, SUC_lexicon_tags_37 },
    { 0xfc4e085c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54e0860, SUC_lexicon_tags_62 },
    { 0xfee0861, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbace0865, SUC_lexicon_tags_17 },
    { 0x63ae0866, SUC_lexicon_tags_43 },
    { 0x14e0865, SUC_lexicon_tags_39 },
    { 0x122e0867, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4e086b, SUC_lexicon_tags_33 },
    { 0xcce086c, SUC_lexicon_tags_113 },
    { 0x1a2e086d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc68e086f, SUC_lexicon_tags_876 },
    { 0, NULL },
    { 0xb5ae0871, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ee0875, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ae0878, SUC_lexicon_tags_25 },
    { 0xe36e0879, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6d4e087b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x484e087e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabae0883, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2e0887, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x328e0889, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ce088d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6e0890, SUC_lexicon_tags_26 },
    { 0xd22e0891, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446e089a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642e089e, SUC_lexicon_tags_14 },
    { 0x284e089f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8e08a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ee08a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8e08ae, SUC_lexicon_tags_15 },
    { 0x98ce08ae, SUC_lexicon_tags_63 },
    { 0xe6e08b0, SUC_lexicon_tags_9 },
    { 0x8d4e08b0, SUC_lexicon_tags_25 },
    { 0x604e08b0, SUC_lexicon_tags_26 },
    { 0x5e8e08af, SUC_lexicon_tags_1 },
    { 0x20ae08b0, SUC_lexicon_tags_7 },
    { 0x216e08b5, SUC_lexicon_tags_49 },
    { 0x696e08b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ce08b9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4e08bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bce08bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ee08c3, SUC_lexicon_tags_15 },
    { 0x46e08c4, SUC_lexicon_tags_31 },
    { 0x7d6e08c5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864e08ca, SUC_lexicon_tags_60 },
    { 0x82e08cb, SUC_lexicon_tags_0 },
    { 0xe08e08cc, SUC_lexicon_tags_11 },
    { 0x932e08cc, SUC_lexicon_tags_11 },
    { 0xdc8e08ce, SUC_lexicon_tags_37 },
    { 0xef0e08cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ce08d5, SUC_lexicon_tags_17 },
    { 0xfeae08d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14e08dc, SUC_lexicon_tags_8 },
    { 0x2c2e08dd, SUC_lexicon_tags_37 },
    { 0x26ee08dc, SUC_lexicon_tags_27 },
    { 0x392e08df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ce08e2, SUC_lexicon_tags_37 },
    { 0x640e08e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70e08ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24e08ee, SUC_lexicon_tags_8 },
    { 0x30e08ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4e08f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd4ae08f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2e08fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34ce08ff, SUC_lexicon_tags_29 },
    { 0xc8e0900, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58e0904, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2e0909, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2e0912, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4e0915, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce0919, SUC_lexicon_tags_2 },
    { 0xdaae0919, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0e091d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174e0922, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee0929, SUC_lexicon_tags_37 },
    { 0xf46e092a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dee0932, SUC_lexicon_tags_19 },
    { 0x79ae0932, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2e0937, SUC_lexicon_tags_10 },
    { 0x3aae0938, SUC_lexicon_tags_25 },
    { 0xc78e0937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12e0944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2e0948, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e0951, SUC_lexicon_tags_45 },
    { 0x1ece0952, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ae0955, SUC_lexicon_tags_37 },
    { 0xcc4e0956, SUC_lexicon_tags_0 },
    { 0x48ce0955, SUC_lexicon_tags_17 },
    { 0x626e0958, SUC_lexicon_tags_55 },
    { 0xd8ee0959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8ae095b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf54e095d, SUC_lexicon_tags_71 },
    { 0x52e095e, SUC_lexicon_tags_37 },
    { 0xb6ae095f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52e0963, SUC_lexicon_tags_25 },
    { 0x8f8e0964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64e096b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68e096f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72e0973, SUC_lexicon_tags_32 },
    { 0xe20e0973, SUC_lexicon_tags_63 },
    { 0xf80e0975, SUC_lexicon_tags_21 },
    { 0x1e6e0976, SUC_lexicon_tags_1 },
    { 0x7bae0977, SUC_lexicon_tags_33 },
    { 0x1eae0977, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60e097c, SUC_lexicon_tags_57 },
    { 0xb86e097d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fae0986, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42e098a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x176e098d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x96e098f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x540e0991, SUC_lexicon_tags_14 },
    { 0x6bee0992, SUC_lexicon_tags_26 },
    { 0xddae0993, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x766e0996, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x984e0998, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x46ae099a, SUC_lexicon_tags_25 },
    { 0x13ee099b, SUC_lexicon_tags_47 },
    { 0xa60e099c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51ce099e, SUC_lexicon_tags_39 },
    { 0x40ae099f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ce09a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106e09a9, SUC_lexicon_tags_37 },
    { 0x886e09aa, SUC_lexicon_tags_17 },
    { 0x9c8e09ab, SUC_lexicon_tags_35 },
    { 0xa00e09aa, SUC_lexicon_tags_12 },
    { 0x9ce09ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcf0e09af, SUC_lexicon_tags_1 },
    { 0x5d8e09b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaace09b4, SUC_lexicon_tags_27 },
    { 0xc0e09b4, SUC_lexicon_tags_15 },
    { 0xdf6e09b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeee09ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x44ae09bc, SUC_lexicon_tags_11 },
    { 0x4fae09bd, SUC_lexicon_tags_55 },
    { 0x106e09bd, SUC_lexicon_tags_7 },
    { 0x3eee09bd, SUC_lexicon_tags_17 },
    { 0x422e09bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758e09c4, SUC_lexicon_tags_15 },
    { 0xca6e09c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ce09c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270e09cc, SUC_lexicon_tags_60 },
    { 0xcfae09cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7ee09cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80e09d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ae09d5, SUC_lexicon_tags_47 },
    { 0x294e09d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc2e09d8, SUC_lexicon_tags_17 },
    { 0xaa2e09d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bce09db, SUC_lexicon_tags_68 },
    { 0xca4e09dc, SUC_lexicon_tags_2 },
    { 0x7e4e09dd, SUC_lexicon_tags_11 },
    { 0x1dee09de, SUC_lexicon_tags_2 },
    { 0x8c8e09df, SUC_lexicon_tags_10 },
    { 0x232e09df, SUC_lexicon_tags_11 },
    { 0x1f2e09dd, SUC_lexicon_tags_40 },
    { 0x53ae09e2, SUC_lexicon_tags_9 },
    { 0x748e09e3, SUC_lexicon_tags_32 },
    { 0x4b8e09e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4e09ea, SUC_lexicon_tags_12 },
    { 0xb40e09eb, SUC_lexicon_tags_17 },
    { 0xd6ae09ec, SUC_lexicon_tags_21 },
    { 0x738e09ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe66e09ef, SUC_lexicon_tags_11 },
    { 0xb50e09f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e09f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x704e09f6, SUC_lexicon_tags_9 },
    { 0xa88e09f6, SUC_lexicon_tags_9 },
    { 0xbdee09f7, SUC_lexicon_tags_31 },
    { 0xee0e09f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ae0a00, SUC_lexicon_tags_46 },
    { 0x5f2e0a01, SUC_lexicon_tags_16 },
    { 0x1f8e0a01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c2e0a04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd36e0a06, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6e0a09, SUC_lexicon_tags_15 },
    { 0x6aae0a09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8e0a0f, SUC_lexicon_tags_37 },
    { 0xe5ee0a0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50e0a15, SUC_lexicon_tags_33 },
    { 0x9ce0a16, SUC_lexicon_tags_15 },
    { 0x77ee0a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60e0a1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x338e0a1d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ce0a25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b6e0a27, SUC_lexicon_tags_15 },
    { 0x7c0e0a27, SUC_lexicon_tags_27 },
    { 0x3ae0a29, SUC_lexicon_tags_25 },
    { 0x722e0a2a, SUC_lexicon_tags_28 },
    { 0x92ae0a29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ae0a32, SUC_lexicon_tags_28 },
    { 0xc60e0a33, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ee0a37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x742e0a3a, SUC_lexicon_tags_0 },
    { 0x276e0a3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ae0a40, SUC_lexicon_tags_33 },
    { 0x2e0a40, SUC_lexicon_tags_7 },
    { 0x120e0a42, SUC_lexicon_tags_148 },
    { 0xd50e0a43, SUC_lexicon_tags_29 },
    { 0xcd2e0a42, SUC_lexicon_tags_25 },
    { 0x6c0e0a40, SUC_lexicon_tags_37 },
    { 0x914e0a42, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74e0a49, SUC_lexicon_tags_19 },
    { 0x302e0a4a, SUC_lexicon_tags_16 },
    { 0x798e0a4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde2e0a4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80e0a50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ce0a56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x632e0a59, SUC_lexicon_tags_79 },
    { 0xbbae0a5a, SUC_lexicon_tags_37 },
    { 0xd30e0a59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384e0a69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa80e0a6b, SUC_lexicon_tags_27 },
    { 0xc9ee0a6c, SUC_lexicon_tags_32 },
    { 0xb40e0a6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2e0a76, SUC_lexicon_tags_7 },
    { 0x506e0a77, SUC_lexicon_tags_4 },
    { 0x910e0a78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x916e0a7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d0e0a7c, SUC_lexicon_tags_30 },
    { 0x88ee0a7d, SUC_lexicon_tags_25 },
    { 0xcb4e0a7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce0a83, SUC_lexicon_tags_7 },
    { 0x5dee0a83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x270e0a86, SUC_lexicon_tags_29 },
    { 0x300e0a87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb0e0a89, SUC_lexicon_tags_37 },
    { 0xad6e0a89, SUC_lexicon_tags_7 },
    { 0x458e0a8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2e0a91, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ee0a95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x294e0a98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4e0a9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccae0a9f, SUC_lexicon_tags_37 },
    { 0x7c0e0aa0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc2ee0aa2, SUC_lexicon_tags_29 },
    { 0x85ee0aa3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbce0aa6, SUC_lexicon_tags_0 },
    { 0xdd8e0aa7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb8ee0aa9, SUC_lexicon_tags_23 },
    { 0xd10e0aaa, SUC_lexicon_tags_15 },
    { 0x2ee0aa9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x11ae0aad, SUC_lexicon_tags_4 },
    { 0x2cce0aae, SUC_lexicon_tags_160 },
    { 0x8ce0aad, SUC_lexicon_tags_301 },
    { 0xb16e0ab0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6e0ab5, SUC_lexicon_tags_27 },
    { 0xecee0ab6, SUC_lexicon_tags_37 },
    { 0xfe0e0ab7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0e0abd, SUC_lexicon_tags_9 },
    { 0x844e0abe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6a0e0ac0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22e0ac3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e0ac7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0e0acc, SUC_lexicon_tags_65 },
    { 0xee0acc, SUC_lexicon_tags_37 },
    { 0xf54e0ace, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe76e0ad0, SUC_lexicon_tags_2 },
    { 0xa0e0ad1, SUC_lexicon_tags_25 },
    { 0x46ee0ad2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44e0ad9, SUC_lexicon_tags_47 },
    { 0x24ce0ad9, SUC_lexicon_tags_29 },
    { 0x836e0adb, SUC_lexicon_tags_207 },
    { 0x15ce0adc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x340e0ade, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4e0ae4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6e0aef, SUC_lexicon_tags_6 },
    { 0x5e4e0af0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed4e0af2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x994e0af4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fae0af7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ce0afb, SUC_lexicon_tags_11 },
    { 0x4fae0afc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38ee0afe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e0b08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe08e0b0a, SUC_lexicon_tags_15 },
    { 0xb32e0b0b, SUC_lexicon_tags_2 },
    { 0xff6e0b0c, SUC_lexicon_tags_3 },
    { 0xadce0b0d, SUC_lexicon_tags_11 },
    { 0xf52e0b0b, SUC_lexicon_tags_16 },
    { 0x552e0b0f, SUC_lexicon_tags_25 },
    { 0xadee0b10, SUC_lexicon_tags_33 },
    { 0x474e0b0b, SUC_lexicon_tags_23 },
    { 0x5aae0b11, SUC_lexicon_tags_66 },
    { 0x5aae0b13, SUC_lexicon_tags_15 },
    { 0x696e0b0a, SUC_lexicon_tags_26 },
    { 0xe2ce0b15, SUC_lexicon_tags_37 },
    { 0x122e0b16, SUC_lexicon_tags_0 },
    { 0xe4ae0b14, SUC_lexicon_tags_3 },
    { 0x98ce0b18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x582e0b1b, SUC_lexicon_tags_3 },
    { 0x68ae0b1b, SUC_lexicon_tags_49 },
    { 0x24e0b1d, SUC_lexicon_tags_17 },
    { 0xdd6e0b1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6e0b22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eee0b26, SUC_lexicon_tags_56 },
    { 0xffae0b26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ae0b2a, SUC_lexicon_tags_39 },
    { 0x4bae0b2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x73ce0b2d, SUC_lexicon_tags_14 },
    { 0xd3ce0b2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ae0b31, SUC_lexicon_tags_49 },
    { 0x65ee0b31, SUC_lexicon_tags_7 },
    { 0x5c6e0b33, SUC_lexicon_tags_31 },
    { 0x342e0b34, SUC_lexicon_tags_25 },
    { 0x8c4e0b33, SUC_lexicon_tags_65 },
    { 0xc60e0b31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xecce0b39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3d0e0b3b, SUC_lexicon_tags_17 },
    { 0xa5ae0b3c, SUC_lexicon_tags_37 },
    { 0xa46e0b3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3dce0b3f, SUC_lexicon_tags_1313 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8e0b42, SUC_lexicon_tags_12 },
    { 0xe52e0b43, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740e0b4a, SUC_lexicon_tags_17 },
    { 0xabce0b4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e0b4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ee0b51, SUC_lexicon_tags_15 },
    { 0x1d8e0b52, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdce0b54, SUC_lexicon_tags_15 },
    { 0x5e6e0b55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8e0b5b, SUC_lexicon_tags_37 },
    { 0x6ae0b5c, SUC_lexicon_tags_10 },
    { 0x286e0b5b, SUC_lexicon_tags_12 },
    { 0x4cee0b5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2e0b62, SUC_lexicon_tags_37 },
    { 0xd6ae0b63, SUC_lexicon_tags_8 },
    { 0x7b8e0b64, SUC_lexicon_tags_17 },
    { 0xa9ee0b63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbce0b72, SUC_lexicon_tags_1 },
    { 0xa6ae0b73, SUC_lexicon_tags_38 },
    { 0x760e0b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e0b77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e0b7d, SUC_lexicon_tags_6 },
    { 0x136e0b7e, SUC_lexicon_tags_2 },
    { 0xc46e0b7e, SUC_lexicon_tags_37 },
    { 0x3f2e0b80, SUC_lexicon_tags_68 },
    { 0xe28e0b81, SUC_lexicon_tags_25 },
    { 0x388e0b82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ee0b85, SUC_lexicon_tags_0 },
    { 0x6b4e0b85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4e0b8c, SUC_lexicon_tags_49 },
    { 0xe58e0b8d, SUC_lexicon_tags_37 },
    { 0x5eae0b8e, SUC_lexicon_tags_27 },
    { 0x348e0b8e, SUC_lexicon_tags_11 },
    { 0x708e0b90, SUC_lexicon_tags_27 },
    { 0x592e0b8f, SUC_lexicon_tags_23 },
    { 0x64ce0b92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696e0b9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ae0b9d, SUC_lexicon_tags_3 },
    { 0x2d6e0b9e, SUC_lexicon_tags_82 },
    { 0xe46e0b9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfd8e0ba1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88e0ba6, SUC_lexicon_tags_55 },
    { 0xee8e0ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bee0bac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4e0bb1, SUC_lexicon_tags_14 },
    { 0xc0ee0bb1, SUC_lexicon_tags_19 },
    { 0x474e0bb3, SUC_lexicon_tags_9 },
    { 0x9aae0bb4, SUC_lexicon_tags_0 },
    { 0x464e0bb4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee6e0bb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb20e0bb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e0bbc, SUC_lexicon_tags_8 },
    { 0xa48e0bbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4e0bc0, SUC_lexicon_tags_27 },
    { 0x84ce0bc1, SUC_lexicon_tags_12 },
    { 0x84ee0bc2, SUC_lexicon_tags_19 },
    { 0x184e0bc3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ce0bc6, SUC_lexicon_tags_31 },
    { 0xe32e0bc7, SUC_lexicon_tags_11 },
    { 0xfe0e0bc6, SUC_lexicon_tags_39 },
    { 0x8d8e0bc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x434e0bcb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6a8e0bcd, SUC_lexicon_tags_1 },
    { 0xefae0bcd, SUC_lexicon_tags_19 },
    { 0x6dee0bcf, SUC_lexicon_tags_23 },
    { 0x1f4e0bcd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5e8e0bd2, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x9ae0bd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f8e0bd6, SUC_lexicon_tags_1 },
    { 0x94ee0bd7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56e0bdd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ae0be0, SUC_lexicon_tags_37 },
    { 0x658e0be1, SUC_lexicon_tags_23 },
    { 0xd3ce0be2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x130e0be5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8e0bea, SUC_lexicon_tags_12 },
    { 0x4fee0beb, SUC_lexicon_tags_28 },
    { 0xbf0e0beb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x432e0bef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dae0bf5, SUC_lexicon_tags_14 },
    { 0xa92e0bf5, SUC_lexicon_tags_38 },
    { 0x364e0bf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4ce0bf9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x872e0bfb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08e0c01, SUC_lexicon_tags_43 },
    { 0x1f4e0c02, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaa8e0c04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ee0c08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ee0c0c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0e0c14, SUC_lexicon_tags_401 },
    { 0xd9ce0c15, SUC_lexicon_tags_15 },
    { 0x9f8e0c15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ae0c19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x77ee0c1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ee0c20, SUC_lexicon_tags_2 },
    { 0xd6e0c21, SUC_lexicon_tags_61 },
    { 0x58ce0c21, SUC_lexicon_tags_29 },
    { 0x9aee0c23, SUC_lexicon_tags_37 },
    { 0x19ee0c24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680e0c29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e0c2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ce0c31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ace0c35, SUC_lexicon_tags_1 },
    { 0x698e0c36, SUC_lexicon_tags_2 },
    { 0xff2e0c37, SUC_lexicon_tags_12 },
    { 0x3d0e0c37, SUC_lexicon_tags_25 },
    { 0x812e0c38, SUC_lexicon_tags_26 },
    { 0x1d2e0c3a, SUC_lexicon_tags_84 },
    { 0x62ae0c39, SUC_lexicon_tags_49 },
    { 0xbae0c3c, SUC_lexicon_tags_7 },
    { 0x33ce0c39, SUC_lexicon_tags_25 },
    { 0x3d4e0c3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x948e0c40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x920e0c42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63ae0c44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd48e0c46, SUC_lexicon_tags_1 },
    { 0x830e0c46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756e0c4b, SUC_lexicon_tags_10 },
    { 0xb06e0c4b, SUC_lexicon_tags_7 },
    { 0x1a6e0c4d, SUC_lexicon_tags_63 },
    { 0x6f0e0c4b, SUC_lexicon_tags_61 },
    { 0x67ee0c4f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd54e0c51, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4e0c55, SUC_lexicon_tags_27 },
    { 0x76ae0c55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4e0c5c, SUC_lexicon_tags_60 },
    { 0x226e0c5d, SUC_lexicon_tags_30 },
    { 0xf62e0c5e, SUC_lexicon_tags_7 },
    { 0x2b2e0c5f, SUC_lexicon_tags_18 },
    { 0xa82e0c5f, SUC_lexicon_tags_48 },
    { 0x114e0c60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6ee0c63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x458e0c65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x694e0c67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72e0c6d, SUC_lexicon_tags_55 },
    { 0x858e0c6e, SUC_lexicon_tags_15 },
    { 0xd56e0c6f, SUC_lexicon_tags_19 },
    { 0xa84e0c6d, SUC_lexicon_tags_45 },
    { 0x81ae0c71, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x6a6e0c73, SUC_lexicon_tags_12 },
    { 0xb6e0c74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e0c79, SUC_lexicon_tags_37 },
    { 0x21ee0c7a, SUC_lexicon_tags_37 },
    { 0xee2e0c7b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x420e0c7d, SUC_lexicon_tags_37 },
    { 0x360e0c7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bae0c85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ee0c88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbd4e0c8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56e0c8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dee0c90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12e0c95, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x920e0c98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58e0c9b, SUC_lexicon_tags_37 },
    { 0x148e0c9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6e0ca0, SUC_lexicon_tags_64 },
    { 0xbf4e0ca1, SUC_lexicon_tags_23 },
    { 0x2e0e0ca0, SUC_lexicon_tags_15 },
    { 0xb46e0ca3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ae0ca7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18e0caa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0e0cad, SUC_lexicon_tags_33 },
    { 0x9a0e0cae, SUC_lexicon_tags_27 },
    { 0xad4e0caf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x792e0cb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xace0cb4, SUC_lexicon_tags_14 },
    { 0x87ee0cb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x114e0cb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x724e0cba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136e0cbe, SUC_lexicon_tags_38 },
    { 0x20e0cbf, SUC_lexicon_tags_15 },
    { 0x83ee0cc0, SUC_lexicon_tags_62 },
    { 0x7d4e0cc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacae0cc9, SUC_lexicon_tags_11 },
    { 0xa36e0cc9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xee4e0ccc, SUC_lexicon_tags_19 },
    { 0xe5ee0ccd, SUC_lexicon_tags_11 },
    { 0x624e0cce, SUC_lexicon_tags_65 },
    { 0x4fae0ccf, SUC_lexicon_tags_11 },
    { 0x70ce0cd0, SUC_lexicon_tags_14 },
    { 0xf9ce0cd1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x184e0cd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82e0cd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x118e0cd9, SUC_lexicon_tags_37 },
    { 0xf2ee0cda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x322e0cdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0e0ce5, SUC_lexicon_tags_2 },
    { 0xc4ee0ce6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3c8e0ce8, SUC_lexicon_tags_25 },
    { 0x1f0e0ce9, SUC_lexicon_tags_63 },
    { 0xd2e0cea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ae0ced, SUC_lexicon_tags_37 },
    { 0x12ae0cee, SUC_lexicon_tags_17 },
    { 0x978e0ced, SUC_lexicon_tags_37 },
    { 0xce0e0cf0, SUC_lexicon_tags_1 },
    { 0x10ee0cf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12ae0cf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba8e0cf5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4e2e0cf7, SUC_lexicon_tags_2 },
    { 0xae4e0cf7, SUC_lexicon_tags_29 },
    { 0x4ae0cf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x200e0cfc, SUC_lexicon_tags_37 },
    { 0x3c0e0cfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86e0d01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf90e0d03, SUC_lexicon_tags_28 },
    { 0xa72e0d03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6e0d07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ce0d0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e0d0f, SUC_lexicon_tags_5 },
    { 0x960e0d0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf52e0d12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3ace0d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ae0d18, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4e0d1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8e0d1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566e0d23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ae0d27, SUC_lexicon_tags_10 },
    { 0xe48e0d28, SUC_lexicon_tags_27 },
    { 0x23ce0d29, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x882e0d2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8e0d2e, SUC_lexicon_tags_25 },
    { 0x6c2e0d2f, SUC_lexicon_tags_19 },
    { 0x7ee0d2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258e0d35, SUC_lexicon_tags_37 },
    { 0x824e0d36, SUC_lexicon_tags_25 },
    { 0x3bee0d37, SUC_lexicon_tags_2 },
    { 0xb14e0d37, SUC_lexicon_tags_12 },
    { 0x286e0d36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xce8e0d3b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc92e0d3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa08e0d3f, SUC_lexicon_tags_8 },
    { 0x888e0d40, SUC_lexicon_tags_7 },
    { 0x598e0d41, SUC_lexicon_tags_37 },
    { 0xe10e0d42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ae0d48, SUC_lexicon_tags_11 },
    { 0x364e0d49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ee0d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x610e0d4e, SUC_lexicon_tags_47 },
    { 0x618e0d4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478e0d5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfae0d5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62e0d61, SUC_lexicon_tags_9 },
    { 0xa28e0d61, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x522e0d64, SUC_lexicon_tags_17 },
    { 0x888e0d65, SUC_lexicon_tags_46 },
    { 0x8b6e0d65, SUC_lexicon_tags_25 },
    { 0xee2e0d67, SUC_lexicon_tags_17 },
    { 0xb12e0d67, SUC_lexicon_tags_19 },
    { 0xad6e0d65, SUC_lexicon_tags_2 },
    { 0x416e0d6a, SUC_lexicon_tags_98 },
    { 0x80ee0d6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2b6e0d6d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192e0d71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcae0d76, SUC_lexicon_tags_68 },
    { 0x15ae0d77, SUC_lexicon_tags_23 },
    { 0x5aae0d78, SUC_lexicon_tags_27 },
    { 0x32ae0d79, SUC_lexicon_tags_45 },
    { 0xe92e0d79, SUC_lexicon_tags_28 },
    { 0x35ae0d79, SUC_lexicon_tags_60 },
    { 0x1d8e0d7c, SUC_lexicon_tags_23 },
    { 0x4e4e0d7c, SUC_lexicon_tags_25 },
    { 0x5aae0d77, SUC_lexicon_tags_0 },
    { 0xc56e0d7f, SUC_lexicon_tags_45 },
    { 0x7f0e0d76, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6e0d83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ee0d86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51ae0d88, SUC_lexicon_tags_26 },
    { 0xdf4e0d88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f0e0d8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb2e0d8d, SUC_lexicon_tags_7 },
    { 0x2b6e0d8d, SUC_lexicon_tags_46 },
    { 0x732e0d8d, SUC_lexicon_tags_63 },
    { 0xfd2e0d90, SUC_lexicon_tags_201 },
    { 0xd60e0d91, SUC_lexicon_tags_49 },
    { 0x680e0d90, SUC_lexicon_tags_1 },
    { 0x48ae0d93, SUC_lexicon_tags_15 },
    { 0x5c0e0d94, SUC_lexicon_tags_15 },
    { 0x712e0d95, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4aee0d97, SUC_lexicon_tags_15 },
    { 0x610e0d98, SUC_lexicon_tags_9 },
    { 0x3dae0d99, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x91ee0d9b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5c6e0d9d, SUC_lexicon_tags_57 },
    { 0x12ce0d9e, SUC_lexicon_tags_381 },
    { 0xa2ce0d9d, SUC_lexicon_tags_126 },
    { 0x87ee0da0, SUC_lexicon_tags_403 },
    { 0x320e0da1, SUC_lexicon_tags_15 },
    { 0x6fae0da2, SUC_lexicon_tags_906 },
    { 0xc82e0da3, SUC_lexicon_tags_66 },
    { 0x8d6e0da4, SUC_lexicon_tags_12 },
    { 0x98e0da4, SUC_lexicon_tags_30 },
    { 0x660e0da6, SUC_lexicon_tags_7 },
    { 0xe7ae0da2, SUC_lexicon_tags_15 },
    { 0x1b4e0da8, SUC_lexicon_tags_17 },
    { 0xb90e0da9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62e0dac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x92e0dae, SUC_lexicon_tags_42 },
    { 0x794e0dae, SUC_lexicon_tags_8 },
    { 0x14e0dae, SUC_lexicon_tags_8 },
    { 0x268e0db1, SUC_lexicon_tags_8 },
    { 0xe5ae0db2, SUC_lexicon_tags_14 },
    { 0x5d4e0db1, SUC_lexicon_tags_0 },
    { 0x23ae0db0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730e0db9, SUC_lexicon_tags_21 },
    { 0xd74e0db9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0e0dbd, SUC_lexicon_tags_15 },
    { 0x4f2e0dbd, SUC_lexicon_tags_37 },
    { 0xebce0dbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac4e0dc1, SUC_lexicon_tags_65 },
    { 0xf2ae0dc1, SUC_lexicon_tags_47 },
    { 0x3b8e0dc3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd8ae0dc5, SUC_lexicon_tags_7 },
    { 0x3a4e0dc5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x5d8e0dc8, SUC_lexicon_tags_33 },
    { 0xdc8e0dc9, SUC_lexicon_tags_43 },
    { 0xaaae0dca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x444e0dcd, SUC_lexicon_tags_27 },
    { 0x50ce0dce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd6e0dd0, SUC_lexicon_tags_17 },
    { 0xce4e0dd0, SUC_lexicon_tags_32 },
    { 0x3b0e0dd2, SUC_lexicon_tags_34 },
    { 0x2ece0dd3, SUC_lexicon_tags_7 },
    { 0x86ce0dd4, SUC_lexicon_tags_27 },
    { 0x2c2e0dd2, SUC_lexicon_tags_25 },
    { 0x31ee0dd2, SUC_lexicon_tags_25 },
    { 0x704e0dd7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x30e0dd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7eae0ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20e0dde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0e0de1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x49ee0de3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cee0de6, SUC_lexicon_tags_6 },
    { 0x70ce0de7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516e0dec, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x38ae0dee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44e0df1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832e0df5, SUC_lexicon_tags_2 },
    { 0x814e0df6, SUC_lexicon_tags_1 },
    { 0x10ee0df7, SUC_lexicon_tags_1 },
    { 0x58ae0df8, SUC_lexicon_tags_7 },
    { 0x38ce0df7, SUC_lexicon_tags_15 },
    { 0x864e0dfa, SUC_lexicon_tags_1 },
    { 0x3b0e0dfa, SUC_lexicon_tags_17 },
    { 0x36ce0dfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dee0e00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb92e0e02, SUC_lexicon_tags_7 },
    { 0x30ee0e03, SUC_lexicon_tags_82 },
    { 0x58ae0e04, SUC_lexicon_tags_37 },
    { 0xba2e0e05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18ae0e07, SUC_lexicon_tags_2 },
    { 0x8d2e0e08, SUC_lexicon_tags_12 },
    { 0xeb4e0e09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ee0e0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb38e0e0e, SUC_lexicon_tags_7 },
    { 0xc76e0e0e, SUC_lexicon_tags_61 },
    { 0xb1ee0e0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380e0e14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce0e17, SUC_lexicon_tags_50 },
    { 0xd54e0e18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56e0e1e, SUC_lexicon_tags_7 },
    { 0x596e0e1f, SUC_lexicon_tags_11 },
    { 0x1dae0e20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4cee0e22, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xecce0e24, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ae0e28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ce0e2d, SUC_lexicon_tags_57 },
    { 0x51ce0e2e, SUC_lexicon_tags_47 },
    { 0xfd2e0e2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e0e32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x392e0e34, SUC_lexicon_tags_43 },
    { 0x5dee0e34, SUC_lexicon_tags_2 },
    { 0x6b8e0e36, SUC_lexicon_tags_47 },
    { 0x626e0e37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee0e3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ae0e41, SUC_lexicon_tags_37 },
    { 0xa7ce0e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4ce0e44, SUC_lexicon_tags_1 },
    { 0xae0e45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x166e0e48, SUC_lexicon_tags_29 },
    { 0x268e0e49, SUC_lexicon_tags_38 },
    { 0x4bae0e4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8e0e4d, SUC_lexicon_tags_1 },
    { 0x778e0e4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338e0e55, SUC_lexicon_tags_17 },
    { 0x3bce0e55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d0e0e58, SUC_lexicon_tags_46 },
    { 0x78ee0e59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbee0e5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fee0e60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x380e0e62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0e0e65, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ce0e68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x802e0e6a, SUC_lexicon_tags_25 },
    { 0x196e0e6b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xeece0e6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf56e0e6f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xaae0e71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x942e0e73, SUC_lexicon_tags_160 },
    { 0x88e0e73, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878e0e78, SUC_lexicon_tags_17 },
    { 0x902e0e79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8e0e7c, SUC_lexicon_tags_25 },
    { 0xc24e0e7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf52e0e7f, SUC_lexicon_tags_4 },
    { 0x9a0e0e80, SUC_lexicon_tags_75 },
    { 0x98ce0e81, SUC_lexicon_tags_11 },
    { 0xd1ae0e80, SUC_lexicon_tags_2 },
    { 0x214e0e81, SUC_lexicon_tags_6 },
    { 0x42ee0e81, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7dce0e86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0e0e8a, SUC_lexicon_tags_37 },
    { 0xf00e0e8b, SUC_lexicon_tags_15 },
    { 0x6e2e0e8c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914e0e92, SUC_lexicon_tags_12 },
    { 0xc4ce0e92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96e0e97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x724e0e9a, SUC_lexicon_tags_15 },
    { 0x374e0e9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ee0e9f, SUC_lexicon_tags_1 },
    { 0xf1ce0e9f, SUC_lexicon_tags_11 },
    { 0x7e0e0e9f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x81ce0ea3, SUC_lexicon_tags_15 },
    { 0xf12e0ea4, SUC_lexicon_tags_8 },
    { 0xf6e0ea4, SUC_lexicon_tags_79 },
    { 0x778e0ea6, SUC_lexicon_tags_8 },
    { 0x9b0e0ea7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826e0eb0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x100e0eb3, SUC_lexicon_tags_25 },
    { 0x58ae0eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc42e0eb7, SUC_lexicon_tags_43 },
    { 0xadee0eb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd32e0eba, SUC_lexicon_tags_17 },
    { 0xfb0e0ebb, SUC_lexicon_tags_1 },
    { 0x9d6e0eba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ee0ebf, SUC_lexicon_tags_25 },
    { 0x5ae0ec0, SUC_lexicon_tags_63 },
    { 0x41ce0ec1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928e0ec9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x248e0ecb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0e0ecd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x374e0ed0, SUC_lexicon_tags_2 },
    { 0xc6ae0ed1, SUC_lexicon_tags_37 },
    { 0x7bee0ed1, SUC_lexicon_tags_19 },
    { 0x7c0e0ed3, SUC_lexicon_tags_27 },
    { 0x7fee0ed0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4e0ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a4e0eda, SUC_lexicon_tags_15 },
    { 0xe6ce0eda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972e0ee0, SUC_lexicon_tags_27 },
    { 0x974e0ee0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670e0ee6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67ae0ee8, SUC_lexicon_tags_15 },
    { 0x4a6e0ee8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b4e0eeb, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6e0ef0, SUC_lexicon_tags_7 },
    { 0x5ece0ef0, SUC_lexicon_tags_29 },
    { 0x7fee0ef0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818e0ef7, SUC_lexicon_tags_10 },
    { 0xd9ee0ef8, SUC_lexicon_tags_37 },
    { 0xc86e0ef8, SUC_lexicon_tags_12 },
    { 0xcece0ef7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574e0f01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee0f04, SUC_lexicon_tags_37 },
    { 0xe60e0f05, SUC_lexicon_tags_2 },
    { 0x752e0f04, SUC_lexicon_tags_30 },
    { 0xc80e0f07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ce0f0d, SUC_lexicon_tags_7 },
    { 0xf28e0f0e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26e0f12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ee0f16, SUC_lexicon_tags_26 },
    { 0x6a0e0f17, SUC_lexicon_tags_17 },
    { 0x3c0e0f17, SUC_lexicon_tags_17 },
    { 0xa90e0f17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ae0f1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48ee0f1e, SUC_lexicon_tags_38 },
    { 0x26ee0f1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888e0f25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ce0f28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dee0f2a, SUC_lexicon_tags_37 },
    { 0x70e0f2b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8e0f2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8e0f34, SUC_lexicon_tags_45 },
    { 0x2c4e0f35, SUC_lexicon_tags_11 },
    { 0xeb6e0f36, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x860e0f38, SUC_lexicon_tags_15 },
    { 0xaf4e0f39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d2e0f3b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e2e0f3d, SUC_lexicon_tags_49 },
    { 0x71ae0f3e, SUC_lexicon_tags_0 },
    { 0x16ce0f3f, SUC_lexicon_tags_39 },
    { 0x91ce0f3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ce0f45, SUC_lexicon_tags_60 },
    { 0x114e0f45, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee0f4a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x104e0f4d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x568e0f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ae0f52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4e0f56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0e0f5c, SUC_lexicon_tags_31 },
    { 0x3c8e0f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce0f60, SUC_lexicon_tags_28 },
    { 0x75ee0f60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40e0f66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46e0f68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x246e0f6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29ce0f6d, SUC_lexicon_tags_7 },
    { 0xabae0f6d, SUC_lexicon_tags_170 },
    { 0x656e0f6e, SUC_lexicon_tags_30 },
    { 0xc9ae0f70, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce0f78, SUC_lexicon_tags_27 },
    { 0x99ee0f79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2e0f7e, SUC_lexicon_tags_10 },
    { 0x2cae0f7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1a4e0f81, SUC_lexicon_tags_23 },
    { 0xfbee0f82, SUC_lexicon_tags_37 },
    { 0x80ee0f81, SUC_lexicon_tags_7 },
    { 0xac0e0f84, SUC_lexicon_tags_25 },
    { 0x95ae0f84, SUC_lexicon_tags_26 },
    { 0xc1ae0f86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ee0f8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976e0f92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24e0f95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x212e0f98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabae0f9c, SUC_lexicon_tags_1 },
    { 0x81ae0f9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b6e0f9f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186e0fa3, SUC_lexicon_tags_26 },
    { 0xb96e0fa4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf04e0fa6, SUC_lexicon_tags_25 },
    { 0xcf6e0fa7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa9ce0fa9, SUC_lexicon_tags_6 },
    { 0x456e0faa, SUC_lexicon_tags_7 },
    { 0xe64e0fab, SUC_lexicon_tags_54 },
    { 0x496e0fac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dee0faf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cae0fb2, SUC_lexicon_tags_12 },
    { 0xd0ee0fb3, SUC_lexicon_tags_6 },
    { 0xc9ce0fb3, SUC_lexicon_tags_14 },
    { 0x9b6e0fb5, SUC_lexicon_tags_25 },
    { 0x938e0fb5, SUC_lexicon_tags_31 },
    { 0x66ce0fb6, SUC_lexicon_tags_6 },
    { 0xecce0fb3, SUC_lexicon_tags_63 },
    { 0xf68e0fb9, SUC_lexicon_tags_15 },
    { 0x522e0fba, SUC_lexicon_tags_0 },
    { 0x4cee0fbb, SUC_lexicon_tags_27 },
    { 0x9d2e0fba, SUC_lexicon_tags_17 },
    { 0xad4e0fb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268e0fc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ee0fc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebae0fcb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6e0fce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72e0fd1, SUC_lexicon_tags_7 },
    { 0x7d0e0fd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3f2e0fd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ae0fd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x100e0fdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xceae0fdd, SUC_lexicon_tags_0 },
    { 0x182e0fdd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bee0fe1, SUC_lexicon_tags_37 },
    { 0xe92e0fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ae0fea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e0ff3, SUC_lexicon_tags_29 },
    { 0x808e0ff3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0e0ff8, SUC_lexicon_tags_38 },
    { 0x9e2e0ff8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc64e0ffb, SUC_lexicon_tags_11 },
    { 0xad8e0ffb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x408e0ffe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ce1001, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6e1005, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6e1010, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6e1017, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654e101c, SUC_lexicon_tags_7 },
    { 0x1bce101c, SUC_lexicon_tags_11 },
    { 0x382e101e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xffee1020, SUC_lexicon_tags_191 },
    { 0xa6ee1021, SUC_lexicon_tags_49 },
    { 0x3ece1022, SUC_lexicon_tags_8 },
    { 0x282e1023, SUC_lexicon_tags_27 },
    { 0x3c0e1024, SUC_lexicon_tags_15 },
    { 0x828e1022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xface102b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ee1036, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x33ee1038, SUC_lexicon_tags_45 },
    { 0xe5ae1039, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6e103c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76e1040, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc04e1042, SUC_lexicon_tags_7 },
    { 0x7dce1043, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf04e1045, SUC_lexicon_tags_7 },
    { 0x574e1045, SUC_lexicon_tags_56 },
    { 0xaf6e1047, SUC_lexicon_tags_30 },
    { 0x93ce1048, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xefae104a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ae104e, SUC_lexicon_tags_37 },
    { 0xb2ae104f, SUC_lexicon_tags_37 },
    { 0x17ce1050, SUC_lexicon_tags_45 },
    { 0xadee1050, SUC_lexicon_tags_0 },
    { 0x4c4e1052, SUC_lexicon_tags_15 },
    { 0x3bce1052, SUC_lexicon_tags_11 },
    { 0x424e1052, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ae1058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e105c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x952e105f, SUC_lexicon_tags_37 },
    { 0x8f6e1060, SUC_lexicon_tags_19 },
    { 0xe68e1061, SUC_lexicon_tags_11 },
    { 0x45ae1061, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa1ee1064, SUC_lexicon_tags_35 },
    { 0x7bce1065, SUC_lexicon_tags_14 },
    { 0x2d2e1066, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ae106a, SUC_lexicon_tags_1 },
    { 0x702e106b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258e1076, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22e107a, SUC_lexicon_tags_19 },
    { 0x65ee107b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ce107f, SUC_lexicon_tags_26 },
    { 0xebae1080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x284e1082, SUC_lexicon_tags_70 },
    { 0x31ce1082, SUC_lexicon_tags_11 },
    { 0xecee1084, SUC_lexicon_tags_68 },
    { 0xdf2e1085, SUC_lexicon_tags_47 },
    { 0x686e1086, SUC_lexicon_tags_7 },
    { 0xf38e1087, SUC_lexicon_tags_25 },
    { 0xf5ee1082, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x450e108b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa00e108d, SUC_lexicon_tags_6 },
    { 0xbd8e108e, SUC_lexicon_tags_56 },
    { 0x4f6e108f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230e1093, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30e1096, SUC_lexicon_tags_47 },
    { 0xe8ee1097, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ace109b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x882e109d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e10a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc62e10a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242e10a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ce10aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cee10b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x12ce10b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae10bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ce10c2, SUC_lexicon_tags_47 },
    { 0xe2ce10c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e10c8, SUC_lexicon_tags_64 },
    { 0x384e10c8, SUC_lexicon_tags_30 },
    { 0xcdae10ca, SUC_lexicon_tags_66 },
    { 0x5dae10cb, SUC_lexicon_tags_11 },
    { 0xe6ae10cb, SUC_lexicon_tags_64 },
    { 0x20ae10cb, SUC_lexicon_tags_32 },
    { 0xe30e10ce, SUC_lexicon_tags_87 },
    { 0x180e10cd, SUC_lexicon_tags_13 },
    { 0x12e10d0, SUC_lexicon_tags_45 },
    { 0xc38e10d0, SUC_lexicon_tags_35 },
    { 0x9d8e10d2, SUC_lexicon_tags_3 },
    { 0x8bce10cb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ee10d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50e10da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ee10dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ce10e2, SUC_lexicon_tags_9 },
    { 0x40e10e3, SUC_lexicon_tags_7 },
    { 0x8ee10e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534e10e9, SUC_lexicon_tags_82 },
    { 0xcc2e10e9, SUC_lexicon_tags_35 },
    { 0xf08e10eb, SUC_lexicon_tags_28 },
    { 0xe2ee10ec, SUC_lexicon_tags_20 },
    { 0xb70e10ed, SUC_lexicon_tags_11 },
    { 0xc2ee10e9, SUC_lexicon_tags_37 },
    { 0xeaae10ef, SUC_lexicon_tags_63 },
    { 0x3d0e10ef, SUC_lexicon_tags_25 },
    { 0xb5ae10f0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e10f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c8e10f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826e10fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee1101, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e1104, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4e1109, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bce110d, SUC_lexicon_tags_63 },
    { 0x49ee110e, SUC_lexicon_tags_32 },
    { 0x9e2e110f, SUC_lexicon_tags_37 },
    { 0x624e1110, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2ae1112, SUC_lexicon_tags_15 },
    { 0xa50e1112, SUC_lexicon_tags_38 },
    { 0x630e1114, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeae1117, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762e111c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe94e111e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x564e1120, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdbae1122, SUC_lexicon_tags_45 },
    { 0x9c2e1122, SUC_lexicon_tags_6 },
    { 0x27ae1123, SUC_lexicon_tags_11 },
    { 0xbb6e1125, SUC_lexicon_tags_0 },
    { 0xf1ae1125, SUC_lexicon_tags_52 },
    { 0xfb4e1125, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x852e1129, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80e112d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6ce112f, SUC_lexicon_tags_7 },
    { 0x400e112f, SUC_lexicon_tags_32 },
    { 0xf16e1131, SUC_lexicon_tags_17 },
    { 0x618e1132, SUC_lexicon_tags_17 },
    { 0x7bee1133, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cee1138, SUC_lexicon_tags_17 },
    { 0xe5ce1139, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ce113f, SUC_lexicon_tags_15 },
    { 0x4aae1140, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bee1146, SUC_lexicon_tags_64 },
    { 0x532e1147, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eee114a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13ce114c, SUC_lexicon_tags_0 },
    { 0x144e114c, SUC_lexicon_tags_37 },
    { 0x1fce114e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6e1152, SUC_lexicon_tags_37 },
    { 0x70e1153, SUC_lexicon_tags_0 },
    { 0x12ce1154, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f0e1156, SUC_lexicon_tags_11 },
    { 0x19ce1156, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156e115b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcf0e115d, SUC_lexicon_tags_17 },
    { 0x230e115e, SUC_lexicon_tags_33 },
    { 0x440e115f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x79ae1161, SUC_lexicon_tags_12 },
    { 0x610e1162, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4e1168, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a4e116a, SUC_lexicon_tags_21 },
    { 0xd14e116a, SUC_lexicon_tags_23 },
    { 0x28ae116c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64e1171, SUC_lexicon_tags_9 },
    { 0x74e1172, SUC_lexicon_tags_11 },
    { 0x40ae1171, SUC_lexicon_tags_27 },
    { 0xe3ae1171, SUC_lexicon_tags_43 },
    { 0x4f6e1175, SUC_lexicon_tags_59 },
    { 0x164e1176, SUC_lexicon_tags_39 },
    { 0x1fee1176, SUC_lexicon_tags_27 },
    { 0x784e1177, SUC_lexicon_tags_23 },
    { 0xd06e1179, SUC_lexicon_tags_7 },
    { 0xb08e117a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342e117e, SUC_lexicon_tags_2 },
    { 0xe6ee117f, SUC_lexicon_tags_7 },
    { 0x4e4e1180, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ae1183, SUC_lexicon_tags_28 },
    { 0x400e1184, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaae118a, SUC_lexicon_tags_46 },
    { 0xa42e118b, SUC_lexicon_tags_25 },
    { 0xee2e118c, SUC_lexicon_tags_63 },
    { 0xba8e118a, SUC_lexicon_tags_11 },
    { 0xde6e118d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89ee1190, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ee1196, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x498e1198, SUC_lexicon_tags_611 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80e119c, SUC_lexicon_tags_1 },
    { 0xb0e119d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fce11a0, SUC_lexicon_tags_11 },
    { 0xa2ce11a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x650e11a4, SUC_lexicon_tags_12 },
    { 0x1a6e11a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb4e11a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xface11b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44e11b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd94e11b7, SUC_lexicon_tags_17 },
    { 0x288e11b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcee11bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78e11c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04e11c6, SUC_lexicon_tags_39 },
    { 0xce8e11c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506e11cf, SUC_lexicon_tags_33 },
    { 0xa2ee11d0, SUC_lexicon_tags_10 },
    { 0xd48e11d1, SUC_lexicon_tags_65 },
    { 0xb5ee11d1, SUC_lexicon_tags_25 },
    { 0x1cce11d0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fee11d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18e11d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x946e11db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ae11de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x586e11e1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4e11e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x94ce11e7, SUC_lexicon_tags_10 },
    { 0x50ee11e7, SUC_lexicon_tags_11 },
    { 0xcd8e11e8, SUC_lexicon_tags_178 },
    { 0xf84e11ea, SUC_lexicon_tags_65 },
    { 0xace11eb, SUC_lexicon_tags_7 },
    { 0x7d0e11eb, SUC_lexicon_tags_29 },
    { 0x506e11ed, SUC_lexicon_tags_23 },
    { 0xb96e11ee, SUC_lexicon_tags_33 },
    { 0x376e11ef, SUC_lexicon_tags_23 },
    { 0xc44e11f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fce11f4, SUC_lexicon_tags_19 },
    { 0x50ce11f5, SUC_lexicon_tags_17 },
    { 0x4d6e11f6, SUC_lexicon_tags_66 },
    { 0xd4ce11f7, SUC_lexicon_tags_12 },
    { 0x604e11f5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x506e11fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24e1204, SUC_lexicon_tags_37 },
    { 0xf5ee1205, SUC_lexicon_tags_65 },
    { 0xdfae1205, SUC_lexicon_tags_37 },
    { 0xa04e1207, SUC_lexicon_tags_84 },
    { 0xb82e1208, SUC_lexicon_tags_15 },
    { 0x24ce1209, SUC_lexicon_tags_7 },
    { 0xb6ee120a, SUC_lexicon_tags_23 },
    { 0xcb6e1207, SUC_lexicon_tags_43 },
    { 0x8ae1206, SUC_lexicon_tags_10 },
    { 0x306e120d, SUC_lexicon_tags_38 },
    { 0xebae120e, SUC_lexicon_tags_30 },
    { 0x8eee120f, SUC_lexicon_tags_30 },
    { 0xccae1210, SUC_lexicon_tags_14 },
    { 0x6f8e1211, SUC_lexicon_tags_23 },
    { 0xee8e1211, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xeeae1214, SUC_lexicon_tags_28 },
    { 0x8fce1215, SUC_lexicon_tags_37 },
    { 0x8ce1216, SUC_lexicon_tags_66 },
    { 0x864e1217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x578e121a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860e121f, SUC_lexicon_tags_12 },
    { 0x892e1220, SUC_lexicon_tags_62 },
    { 0x70ee1221, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720e1225, SUC_lexicon_tags_40 },
    { 0xb58e1226, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ae122a, SUC_lexicon_tags_11 },
    { 0x532e122b, SUC_lexicon_tags_12 },
    { 0x79ee122b, SUC_lexicon_tags_49 },
    { 0xde2e122a, SUC_lexicon_tags_11 },
    { 0xc34e122e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74e1234, SUC_lexicon_tags_23 },
    { 0x4c8e1235, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4eee1237, SUC_lexicon_tags_11 },
    { 0xe68e1238, SUC_lexicon_tags_1 },
    { 0xf6ae1238, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9ae123b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x112e123e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x376e1241, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x160e1243, SUC_lexicon_tags_37 },
    { 0x8a4e1244, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ae1247, SUC_lexicon_tags_63 },
    { 0xd20e1248, SUC_lexicon_tags_16 },
    { 0x5f2e1248, SUC_lexicon_tags_26 },
    { 0x352e124a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee124e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ae1253, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x832e1255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabae1257, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06e125b, SUC_lexicon_tags_33 },
    { 0x40ee125c, SUC_lexicon_tags_39 },
    { 0x2c2e125d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfce1260, SUC_lexicon_tags_1 },
    { 0x394e1261, SUC_lexicon_tags_17 },
    { 0xe34e1262, SUC_lexicon_tags_25 },
    { 0xcf8e1263, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x934e1265, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a6e1267, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x696e1269, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ee126c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x756e126e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ee1270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeee1273, SUC_lexicon_tags_26 },
    { 0xcf8e1273, SUC_lexicon_tags_0 },
    { 0xad2e1273, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdb6e1277, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x120e1279, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfee127d, SUC_lexicon_tags_2 },
    { 0xbf2e127e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x230e1281, SUC_lexicon_tags_9 },
    { 0x8b6e1282, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaa0e1284, SUC_lexicon_tags_11 },
    { 0xebee1285, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298e128c, SUC_lexicon_tags_104 },
    { 0x35ce128c, SUC_lexicon_tags_17 },
    { 0xd6ee128e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4e1291, SUC_lexicon_tags_45 },
    { 0x68ce1292, SUC_lexicon_tags_28 },
    { 0x510e1291, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff6e1295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e1298, SUC_lexicon_tags_9 },
    { 0x52e1298, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446e12a0, SUC_lexicon_tags_14 },
    { 0xb08e12a1, SUC_lexicon_tags_23 },
    { 0x5a2e12a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ece12a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x734e12a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4e12ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4e12b3, SUC_lexicon_tags_19 },
    { 0xed2e12b3, SUC_lexicon_tags_26 },
    { 0xe9ae12b3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece12b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38e12c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x548e12c4, SUC_lexicon_tags_3 },
    { 0xb88e12c5, SUC_lexicon_tags_26 },
    { 0xfb8e12c6, SUC_lexicon_tags_1 },
    { 0x610e12c7, SUC_lexicon_tags_10 },
    { 0x32ee12c8, SUC_lexicon_tags_12 },
    { 0xf90e12c5, SUC_lexicon_tags_31 },
    { 0xf52e12c5, SUC_lexicon_tags_520 },
    { 0xd8ce12c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fce12cd, SUC_lexicon_tags_36 },
    { 0xc7ee12cd, SUC_lexicon_tags_12 },
    { 0x798e12cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd28e12d1, SUC_lexicon_tags_12 },
    { 0xddce12d2, SUC_lexicon_tags_8 },
    { 0x428e12d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x622e12d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd46e12d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28e12db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62e12dd, SUC_lexicon_tags_9 },
    { 0xf70e12de, SUC_lexicon_tags_25 },
    { 0x840e12de, SUC_lexicon_tags_25 },
    { 0xcc6e12e0, SUC_lexicon_tags_48 },
    { 0x64ee12e1, SUC_lexicon_tags_45 },
    { 0x994e12e2, SUC_lexicon_tags_1 },
    { 0x44ae12e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56e12e5, SUC_lexicon_tags_60 },
    { 0xefce12e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x36e12e9, SUC_lexicon_tags_47 },
    { 0xd62e12ea, SUC_lexicon_tags_47 },
    { 0xa42e12e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61ce12ed, SUC_lexicon_tags_37 },
    { 0x4e2e12ee, SUC_lexicon_tags_27 },
    { 0x68e12ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1b6e12f1, SUC_lexicon_tags_7 },
    { 0x726e12f2, SUC_lexicon_tags_26 },
    { 0xf2ee12f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ae12f8, SUC_lexicon_tags_7 },
    { 0xff0e12f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ee12fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310e1301, SUC_lexicon_tags_1 },
    { 0xca0e1302, SUC_lexicon_tags_15 },
    { 0x6f2e1303, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814e1308, SUC_lexicon_tags_19 },
    { 0x358e1309, SUC_lexicon_tags_60 },
    { 0xf54e1309, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ce130e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88e1310, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a4e1312, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2e1315, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46e1318, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebce131d, SUC_lexicon_tags_32 },
    { 0xb62e131e, SUC_lexicon_tags_43 },
    { 0x430e131f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ee1323, SUC_lexicon_tags_17 },
    { 0x664e1324, SUC_lexicon_tags_10 },
    { 0xf00e1325, SUC_lexicon_tags_37 },
    { 0x2e0e1324, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4e132c, SUC_lexicon_tags_9 },
    { 0x6d8e132c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7c0e132f, SUC_lexicon_tags_1 },
    { 0xc8ae1330, SUC_lexicon_tags_7 },
    { 0x892e132f, SUC_lexicon_tags_88 },
    { 0x288e1331, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52e1335, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376e133c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff2e133e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2e2e1340, SUC_lexicon_tags_7 },
    { 0xafce1341, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x682e1344, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa94e1346, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xedee1349, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ae134c, SUC_lexicon_tags_15 },
    { 0x366e134c, SUC_lexicon_tags_11 },
    { 0xae4e134e, SUC_lexicon_tags_12 },
    { 0xff0e134f, SUC_lexicon_tags_2 },
    { 0x29ee134e, SUC_lexicon_tags_60 },
    { 0xc4ce134e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ce1358, SUC_lexicon_tags_37 },
    { 0xff8e1359, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce135d, SUC_lexicon_tags_23 },
    { 0x24ce135d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8e1366, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e136b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6e136e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x296e1370, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x142e137e, SUC_lexicon_tags_15 },
    { 0x1bce137f, SUC_lexicon_tags_8 },
    { 0xdf8e1380, SUC_lexicon_tags_28 },
    { 0xf22e1381, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124e1385, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370e138c, SUC_lexicon_tags_28 },
    { 0x3cee138c, SUC_lexicon_tags_7 },
    { 0x9f4e138e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0e1391, SUC_lexicon_tags_14 },
    { 0xa5ee1391, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ae1399, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564e139f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd3ce13a1, SUC_lexicon_tags_14 },
    { 0x822e13a1, SUC_lexicon_tags_17 },
    { 0x2e0e13a1, SUC_lexicon_tags_25 },
    { 0x676e13a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf36e13a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e13aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x758e13ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82e13af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x398e13b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5aee13b4, SUC_lexicon_tags_63 },
    { 0xaa4e13b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82e13b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78e13be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856e13c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ee13ca, SUC_lexicon_tags_11 },
    { 0xb88e13ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c6e13cd, SUC_lexicon_tags_15 },
    { 0xb22e13ce, SUC_lexicon_tags_11 },
    { 0x114e13ce, SUC_lexicon_tags_31 },
    { 0x826e13d0, SUC_lexicon_tags_31 },
    { 0x518e13d0, SUC_lexicon_tags_2 },
    { 0x856e13cf, SUC_lexicon_tags_32 },
    { 0x490e13cf, SUC_lexicon_tags_7 },
    { 0x8d4e13d4, SUC_lexicon_tags_17 },
    { 0xe00e13d4, SUC_lexicon_tags_37 },
    { 0xfc8e13d4, SUC_lexicon_tags_11 },
    { 0xbaee13d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680e13db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04e13df, SUC_lexicon_tags_25 },
    { 0xe02e13e0, SUC_lexicon_tags_109 },
    { 0xcdee13e1, SUC_lexicon_tags_17 },
    { 0x1aae13e0, SUC_lexicon_tags_7 },
    { 0xe28e13e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x840e13e5, SUC_lexicon_tags_63 },
    { 0x258e13e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26ce13e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744e13ed, SUC_lexicon_tags_28 },
    { 0x9dae13ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8ce13f0, SUC_lexicon_tags_7 },
    { 0x264e13f1, SUC_lexicon_tags_43 },
    { 0x3e0e13f1, SUC_lexicon_tags_30 },
    { 0x1c0e13f0, SUC_lexicon_tags_45 },
    { 0xf2ee13f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffee13f9, SUC_lexicon_tags_17 },
    { 0x3dce13f9, SUC_lexicon_tags_56 },
    { 0x982e13fa, SUC_lexicon_tags_37 },
    { 0xcce13fc, SUC_lexicon_tags_37 },
    { 0x28ae13f9, SUC_lexicon_tags_23 },
    { 0x2c2e13fe, SUC_lexicon_tags_15 },
    { 0x9fee13fc, SUC_lexicon_tags_37 },
    { 0x84e1400, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa34e1402, SUC_lexicon_tags_15 },
    { 0x404e1403, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ae140a, SUC_lexicon_tags_1 },
    { 0x3c0e140b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ce1410, SUC_lexicon_tags_2 },
    { 0xa78e1411, SUC_lexicon_tags_38 },
    { 0xe7ce1410, SUC_lexicon_tags_5 },
    { 0x87ae1410, SUC_lexicon_tags_11 },
    { 0x6ce1412, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd7ae1416, SUC_lexicon_tags_1 },
    { 0x712e1417, SUC_lexicon_tags_11 },
    { 0xfb0e1418, SUC_lexicon_tags_60 },
    { 0xaf0e1419, SUC_lexicon_tags_12 },
    { 0x218e1419, SUC_lexicon_tags_66 },
    { 0x390e141b, SUC_lexicon_tags_60 },
    { 0x63ce141b, SUC_lexicon_tags_19 },
    { 0xfeee141d, SUC_lexicon_tags_87 },
    { 0x92ae141e, SUC_lexicon_tags_65 },
    { 0x35ee141e, SUC_lexicon_tags_38 },
    { 0x982e141e, SUC_lexicon_tags_25 },
    { 0x4fae1421, SUC_lexicon_tags_65 },
    { 0x4c6e1422, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ce1429, SUC_lexicon_tags_436 },
    { 0x6b6e142a, SUC_lexicon_tags_25 },
    { 0xf1ae1429, SUC_lexicon_tags_11 },
    { 0xc24e142c, SUC_lexicon_tags_17 },
    { 0x5b4e142a, SUC_lexicon_tags_45 },
    { 0x1d6e142e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabae1430, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb50e1432, SUC_lexicon_tags_17 },
    { 0x1eae1433, SUC_lexicon_tags_23 },
    { 0xca8e1433, SUC_lexicon_tags_47 },
    { 0x698e1433, SUC_lexicon_tags_9 },
    { 0xf68e1436, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce143a, SUC_lexicon_tags_25 },
    { 0x844e143b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd4e143d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808e1443, SUC_lexicon_tags_60 },
    { 0xc8ce1443, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x530e1446, SUC_lexicon_tags_55 },
    { 0xb76e1446, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x142e1449, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e1452, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x696e1454, SUC_lexicon_tags_21 },
    { 0x70e1455, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82e145a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x720e145d, SUC_lexicon_tags_14 },
    { 0x78ee145e, SUC_lexicon_tags_33 },
    { 0xa78e145f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x588e1462, SUC_lexicon_tags_12 },
    { 0xb3ee1462, SUC_lexicon_tags_9 },
    { 0xc52e1464, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x892e1467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69ee1469, SUC_lexicon_tags_298 },
    { 0x4d6e146a, SUC_lexicon_tags_55 },
    { 0x2b2e146b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x512e146e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294e1476, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82e147f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x112e1482, SUC_lexicon_tags_84 },
    { 0xf50e1482, SUC_lexicon_tags_11 },
    { 0xc2ce1482, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6e1488, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x982e148b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ce148e, SUC_lexicon_tags_25 },
    { 0x936e148f, SUC_lexicon_tags_43 },
    { 0x396e148e, SUC_lexicon_tags_12 },
    { 0xf02e1491, SUC_lexicon_tags_2 },
    { 0xf30e1491, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x200e1495, SUC_lexicon_tags_25 },
    { 0x13ee1496, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fce149b, SUC_lexicon_tags_2 },
    { 0x624e149c, SUC_lexicon_tags_7 },
    { 0x45ae149d, SUC_lexicon_tags_29 },
    { 0x78e149e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0e14a1, SUC_lexicon_tags_11 },
    { 0x260e14a2, SUC_lexicon_tags_14 },
    { 0xefce14a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148e14a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae14ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264e14b2, SUC_lexicon_tags_15 },
    { 0xd4ae14b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448e14bb, SUC_lexicon_tags_1 },
    { 0x4aee14bb, SUC_lexicon_tags_11 },
    { 0x4c4e14bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896e14c3, SUC_lexicon_tags_25 },
    { 0x88ee14c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ae14c8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bae14cd, SUC_lexicon_tags_46 },
    { 0xcc6e14cd, SUC_lexicon_tags_28 },
    { 0xc72e14ce, SUC_lexicon_tags_2 },
    { 0xdce14d0, SUC_lexicon_tags_37 },
    { 0x760e14d0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dee14db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4e14e2, SUC_lexicon_tags_25 },
    { 0xc4ae14e2, SUC_lexicon_tags_37 },
    { 0x298e14e4, SUC_lexicon_tags_1 },
    { 0x34ae14e4, SUC_lexicon_tags_7 },
    { 0xa78e14e6, SUC_lexicon_tags_15 },
    { 0xd30e14e5, SUC_lexicon_tags_11 },
    { 0x2e8e14e8, SUC_lexicon_tags_17 },
    { 0xae8e14e9, SUC_lexicon_tags_8 },
    { 0x618e14e9, SUC_lexicon_tags_1 },
    { 0xf52e14ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ee14ee, SUC_lexicon_tags_17 },
    { 0xf4ce14ef, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bce14f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9ae14f7, SUC_lexicon_tags_37 },
    { 0x5aee14f8, SUC_lexicon_tags_37 },
    { 0xe22e14f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ee14fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2e1501, SUC_lexicon_tags_8 },
    { 0x2c0e1502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18ee1504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x270e1507, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2e150f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8e1513, SUC_lexicon_tags_43 },
    { 0x1e8e1514, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8e1518, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf8e151a, SUC_lexicon_tags_13 },
    { 0x7e0e151a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ae1521, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x104e1523, SUC_lexicon_tags_7 },
    { 0x9f0e1523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf4e1526, SUC_lexicon_tags_1 },
    { 0xa6ee1527, SUC_lexicon_tags_0 },
    { 0x912e1527, SUC_lexicon_tags_2 },
    { 0x936e1527, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6e152f, SUC_lexicon_tags_55 },
    { 0x13ae152f, SUC_lexicon_tags_50 },
    { 0xfa0e1530, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x8fce1533, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ee1537, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64e153c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ee153f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bee1542, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x146e1545, SUC_lexicon_tags_68 },
    { 0x9a8e1546, SUC_lexicon_tags_85 },
    { 0xcb8e1547, SUC_lexicon_tags_45 },
    { 0xab2e1548, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cae1551, SUC_lexicon_tags_8 },
    { 0x880e1552, SUC_lexicon_tags_43 },
    { 0xee4e1553, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ae1556, SUC_lexicon_tags_37 },
    { 0x3bee1556, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce155a, SUC_lexicon_tags_43 },
    { 0x260e155a, SUC_lexicon_tags_0 },
    { 0x67ce155a, SUC_lexicon_tags_11 },
    { 0xf72e155d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ee1561, SUC_lexicon_tags_30 },
    { 0x852e1562, SUC_lexicon_tags_63 },
    { 0x98ce1562, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6e1568, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572e1572, SUC_lexicon_tags_15 },
    { 0xfbce1573, SUC_lexicon_tags_15 },
    { 0x728e1574, SUC_lexicon_tags_37 },
    { 0x2dae1575, SUC_lexicon_tags_26 },
    { 0x1e8e1573, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8e1579, SUC_lexicon_tags_17 },
    { 0x5c6e1579, SUC_lexicon_tags_18 },
    { 0x806e157a, SUC_lexicon_tags_23 },
    { 0x26ae157c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ee1581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b6e1583, SUC_lexicon_tags_16 },
    { 0xf02e1584, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ee1587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x412e1589, SUC_lexicon_tags_11 },
    { 0x9ee158a, SUC_lexicon_tags_37 },
    { 0xa6ce1589, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce158e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0e1594, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6e159a, SUC_lexicon_tags_17 },
    { 0xf4e159b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6e159e, SUC_lexicon_tags_40 },
    { 0xefce159f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x63ce15a1, SUC_lexicon_tags_26 },
    { 0x23ce15a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dae15a5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1d0e15a7, SUC_lexicon_tags_22 },
    { 0xf44e15a7, SUC_lexicon_tags_26 },
    { 0x92ce15a9, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e15ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x12ee15ae, SUC_lexicon_tags_343 },
    { 0x14e15ae, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36e15b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ce15b5, SUC_lexicon_tags_27 },
    { 0xf48e15b6, SUC_lexicon_tags_49 },
    { 0x524e15b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfcce15b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae15bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574e15cd, SUC_lexicon_tags_152 },
    { 0xa3ae15ce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72e15d1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920e15d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994e15dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x656e15df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ae15ea, SUC_lexicon_tags_6 },
    { 0x6e0e15eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ee15ef, SUC_lexicon_tags_62 },
    { 0xd8ee15f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ee15f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ae15fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xface15fe, SUC_lexicon_tags_15 },
    { 0xfcee15ff, SUC_lexicon_tags_66 },
    { 0x1d4e15ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a6e1602, SUC_lexicon_tags_45 },
    { 0x7e2e1602, SUC_lexicon_tags_2 },
    { 0x53ce1604, SUC_lexicon_tags_13 },
    { 0xb06e1604, SUC_lexicon_tags_15 },
    { 0xeb8e1606, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee160c, SUC_lexicon_tags_1 },
    { 0x6c2e160d, SUC_lexicon_tags_23 },
    { 0x260e160e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4e1612, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ee1617, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ae161e, SUC_lexicon_tags_66 },
    { 0x460e161f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ee1626, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0e162f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838e1633, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x512e1636, SUC_lexicon_tags_60 },
    { 0xc62e1637, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8e163b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4e1640, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ace1642, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e1646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2e164b, SUC_lexicon_tags_61 },
    { 0x35ae164c, SUC_lexicon_tags_14 },
    { 0xe40e164c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x396e164f, SUC_lexicon_tags_7 },
    { 0x900e164f, SUC_lexicon_tags_1514 },
    { 0x68ce1650, SUC_lexicon_tags_19 },
    { 0x212e1652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac4e1654, SUC_lexicon_tags_27 },
    { 0xbaae1654, SUC_lexicon_tags_37 },
    { 0xdeae1654, SUC_lexicon_tags_46 },
    { 0x414e1656, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x292e1659, SUC_lexicon_tags_1 },
    { 0xd2ce1659, SUC_lexicon_tags_43 },
    { 0x762e165b, SUC_lexicon_tags_12 },
    { 0x330e165c, SUC_lexicon_tags_11 },
    { 0x350e165a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc7ee165f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2e1662, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ee1672, SUC_lexicon_tags_112 },
    { 0xd0ce1672, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x120e1675, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ae1678, SUC_lexicon_tags_15 },
    { 0x1eee1678, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00e1685, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x190e1687, SUC_lexicon_tags_0 },
    { 0xe08e1688, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb88e168a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa0ae168c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fce1693, SUC_lexicon_tags_6 },
    { 0xa48e1693, SUC_lexicon_tags_0 },
    { 0x32e1695, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc30e1697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78e169a, SUC_lexicon_tags_28 },
    { 0x2ace169b, SUC_lexicon_tags_25 },
    { 0xa8ce169c, SUC_lexicon_tags_68 },
    { 0xa96e169c, SUC_lexicon_tags_14 },
    { 0xd0ee169d, SUC_lexicon_tags_17 },
    { 0xc36e169f, SUC_lexicon_tags_47 },
    { 0xe9ce16a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d2e16a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94e16ab, SUC_lexicon_tags_12 },
    { 0x13ae16ac, SUC_lexicon_tags_37 },
    { 0x9e6e16ad, SUC_lexicon_tags_45 },
    { 0x232e16ae, SUC_lexicon_tags_25 },
    { 0x728e16af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x614e16b1, SUC_lexicon_tags_37 },
    { 0xbf8e16b2, SUC_lexicon_tags_47 },
    { 0xfbee16b2, SUC_lexicon_tags_9 },
    { 0xd10e16b4, SUC_lexicon_tags_79 },
    { 0x666e16b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94e16b8, SUC_lexicon_tags_559 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ee16bb, SUC_lexicon_tags_60 },
    { 0x118e16bc, SUC_lexicon_tags_9 },
    { 0xd1ee16bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2e16c1, SUC_lexicon_tags_37 },
    { 0xc4ce16c1, SUC_lexicon_tags_7 },
    { 0x364e16c1, SUC_lexicon_tags_7 },
    { 0x1f6e16c1, SUC_lexicon_tags_16 },
    { 0xdeee16c5, SUC_lexicon_tags_11 },
    { 0x1c6e16c6, SUC_lexicon_tags_12 },
    { 0xce6e16c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58e16cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8e6e16ce, SUC_lexicon_tags_21 },
    { 0x51ce16cf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990e16d6, SUC_lexicon_tags_55 },
    { 0x88ae16d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aae16e1, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dae16e4, SUC_lexicon_tags_37 },
    { 0x706e16e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8e16e9, SUC_lexicon_tags_9 },
    { 0x5d6e16e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x588e16ec, SUC_lexicon_tags_7 },
    { 0xe72e16ed, SUC_lexicon_tags_15 },
    { 0x330e16ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x372e16f0, SUC_lexicon_tags_37 },
    { 0x452e16f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x152e16f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e16f9, SUC_lexicon_tags_17 },
    { 0x418e16fa, SUC_lexicon_tags_11 },
    { 0x840e16fa, SUC_lexicon_tags_8 },
    { 0x56e16f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2e1704, SUC_lexicon_tags_27 },
    { 0x952e1705, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdc8e1707, SUC_lexicon_tags_28 },
    { 0x904e1708, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfce170a, SUC_lexicon_tags_7 },
    { 0x650e170b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb74e170d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94ae170f, SUC_lexicon_tags_37 },
    { 0x9f0e1710, SUC_lexicon_tags_19 },
    { 0x56ae1710, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5ce1713, SUC_lexicon_tags_11 },
    { 0xcdee1713, SUC_lexicon_tags_75 },
    { 0x452e1713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ce1719, SUC_lexicon_tags_54 },
    { 0xb1ce171a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fee1720, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4e1723, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcce1725, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x296e1728, SUC_lexicon_tags_25 },
    { 0xe3ce1729, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e172d, SUC_lexicon_tags_68 },
    { 0x848e172d, SUC_lexicon_tags_66 },
    { 0x810e172d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4e1733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfce1738, SUC_lexicon_tags_30 },
    { 0x980e1739, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fce173d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x80ce173f, SUC_lexicon_tags_6 },
    { 0x5f2e173f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6e1743, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ee1746, SUC_lexicon_tags_4 },
    { 0x8f0e1747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4ae1749, SUC_lexicon_tags_65 },
    { 0x46ee174a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66e175b, SUC_lexicon_tags_47 },
    { 0xac8e175c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1ce175e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786e1762, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ece176b, SUC_lexicon_tags_37 },
    { 0xae8e176c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x972e176f, SUC_lexicon_tags_25 },
    { 0xca4e1770, SUC_lexicon_tags_19 },
    { 0x3cae1771, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ce1774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2e1778, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x214e177b, SUC_lexicon_tags_30 },
    { 0x38e177b, SUC_lexicon_tags_31 },
    { 0x998e177d, SUC_lexicon_tags_17 },
    { 0x2eee177e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14e1781, SUC_lexicon_tags_27 },
    { 0xe9ae1782, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a2e1784, SUC_lexicon_tags_37 },
    { 0xa6ce1785, SUC_lexicon_tags_15 },
    { 0x698e1786, SUC_lexicon_tags_116 },
    { 0xd00e1787, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34e178d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x100e1790, SUC_lexicon_tags_0 },
    { 0xc4e1791, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ee179c, SUC_lexicon_tags_11 },
    { 0x71ee179c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00e17a0, SUC_lexicon_tags_26 },
    { 0x4cae17a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8e17a4, SUC_lexicon_tags_29 },
    { 0x758e17a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46e17a8, SUC_lexicon_tags_1364 },
    { 0, NULL },
    { 0x390e17aa, SUC_lexicon_tags_0 },
    { 0x9a8e17aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae17b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d2e17b2, SUC_lexicon_tags_46 },
    { 0xfb2e17b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ae17b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4e17ba, SUC_lexicon_tags_12 },
    { 0x4f2e17bb, SUC_lexicon_tags_37 },
    { 0xfcee17bc, SUC_lexicon_tags_38 },
    { 0x80ae17bd, SUC_lexicon_tags_12 },
    { 0x36e17bb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316e17c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39ae17c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200e17cf, SUC_lexicon_tags_47 },
    { 0xc0ce17cf, SUC_lexicon_tags_30 },
    { 0xa26e17d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6ae17d3, SUC_lexicon_tags_17 },
    { 0xf6ee17d3, SUC_lexicon_tags_11 },
    { 0xa5ae17d3, SUC_lexicon_tags_23 },
    { 0x596e17d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790e17da, SUC_lexicon_tags_11 },
    { 0xe12e17db, SUC_lexicon_tags_43 },
    { 0x2bee17dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186e17e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcdce17e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf18e17e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8e17eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8e17ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x496e17ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce17f5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd58e17f7, SUC_lexicon_tags_7 },
    { 0xb10e17f8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x294e17fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874e1800, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x78ee1802, SUC_lexicon_tags_77 },
    { 0x72e1803, SUC_lexicon_tags_27 },
    { 0x22e1804, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e1808, SUC_lexicon_tags_66 },
    { 0x5dce1808, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ae180f, SUC_lexicon_tags_25 },
    { 0xa66e1810, SUC_lexicon_tags_31 },
    { 0xc3ce1810, SUC_lexicon_tags_23 },
    { 0x9cce1812, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb90e1814, SUC_lexicon_tags_7 },
    { 0xcaee1815, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb6ce1817, SUC_lexicon_tags_105 },
    { 0x234e1818, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2ae181a, SUC_lexicon_tags_31 },
    { 0xd06e181b, SUC_lexicon_tags_28 },
    { 0x3f8e181c, SUC_lexicon_tags_14 },
    { 0x788e181d, SUC_lexicon_tags_47 },
    { 0x8dce181c, SUC_lexicon_tags_47 },
    { 0xd12e181d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe44e1821, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0e1825, SUC_lexicon_tags_35 },
    { 0xa34e1826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc44e1829, SUC_lexicon_tags_37 },
    { 0xa12e182a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2e182d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8e1830, SUC_lexicon_tags_4 },
    { 0xf84e1830, SUC_lexicon_tags_53 },
    { 0x99ee1830, SUC_lexicon_tags_16 },
    { 0x834e1831, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7bee1835, SUC_lexicon_tags_17 },
    { 0xdae1836, SUC_lexicon_tags_45 },
    { 0xd08e1836, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54e183a, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xe2ce183c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd28e183e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae1841, SUC_lexicon_tags_6 },
    { 0xf46e1842, SUC_lexicon_tags_25 },
    { 0x1c0e1843, SUC_lexicon_tags_126 },
    { 0x798e1843, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8a0e1846, SUC_lexicon_tags_25 },
    { 0x2a8e1847, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x858e184a, SUC_lexicon_tags_16 },
    { 0x16ce184b, SUC_lexicon_tags_29 },
    { 0xcdae184c, SUC_lexicon_tags_45 },
    { 0x120e184d, SUC_lexicon_tags_60 },
    { 0xad0e184e, SUC_lexicon_tags_1 },
    { 0x72ee184a, SUC_lexicon_tags_15 },
    { 0x4bee184e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ee1854, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44e1858, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x534e185b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492e185f, SUC_lexicon_tags_45 },
    { 0x5c6e185f, SUC_lexicon_tags_0 },
    { 0xedee1861, SUC_lexicon_tags_30 },
    { 0xc62e1861, SUC_lexicon_tags_21 },
    { 0x5d2e1863, SUC_lexicon_tags_14 },
    { 0x772e1862, SUC_lexicon_tags_1 },
    { 0x6ae185f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444e186a, SUC_lexicon_tags_17 },
    { 0x74ae186b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd82e186d, SUC_lexicon_tags_25 },
    { 0x810e186e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x280e1870, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ee1879, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14e187d, SUC_lexicon_tags_37 },
    { 0xe1ce187e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912e1882, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aee1887, SUC_lexicon_tags_15 },
    { 0xd26e1888, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9eee188a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcae188c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae1896, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc2ce1898, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf32e189a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x424e189c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf70e189e, SUC_lexicon_tags_17 },
    { 0x1f6e189e, SUC_lexicon_tags_17 },
    { 0xb8ce18a0, SUC_lexicon_tags_10 },
    { 0xadee18a0, SUC_lexicon_tags_37 },
    { 0x108e18a1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce18aa, SUC_lexicon_tags_12 },
    { 0x6bee18ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xffee18ad, SUC_lexicon_tags_141 },
    { 0x960e18ae, SUC_lexicon_tags_19 },
    { 0x324e18ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x210e18b1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8e18b5, SUC_lexicon_tags_0 },
    { 0x6f6e18b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf34e18b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2e18ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ae18bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b2e18bf, SUC_lexicon_tags_47 },
    { 0x5e8e18bf, SUC_lexicon_tags_25 },
    { 0x106e18c1, SUC_lexicon_tags_7 },
    { 0xe48e18c0, SUC_lexicon_tags_17 },
    { 0xbdae18c3, SUC_lexicon_tags_212 },
    { 0x248e18c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e18cb, SUC_lexicon_tags_65 },
    { 0xd8ce18cb, SUC_lexicon_tags_47 },
    { 0x748e18cb, SUC_lexicon_tags_34 },
    { 0x8b0e18ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee18d4, SUC_lexicon_tags_23 },
    { 0x6b2e18d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76e18d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x652e18da, SUC_lexicon_tags_83 },
    { 0x78e18da, SUC_lexicon_tags_8 },
    { 0xe4ae18db, SUC_lexicon_tags_33 },
    { 0x61ae18dd, SUC_lexicon_tags_31 },
    { 0xd20e18de, SUC_lexicon_tags_1 },
    { 0x124e18dc, SUC_lexicon_tags_0 },
    { 0x6e18e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ee18e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ce18ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0e18ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36e18f2, SUC_lexicon_tags_8 },
    { 0xc64e18f3, SUC_lexicon_tags_2 },
    { 0x994e18f3, SUC_lexicon_tags_82 },
    { 0x1c4e18f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e18f8, SUC_lexicon_tags_92 },
    { 0x7eee18f9, SUC_lexicon_tags_43 },
    { 0x606e18fa, SUC_lexicon_tags_38 },
    { 0xc72e18fb, SUC_lexicon_tags_37 },
    { 0x4cee18fc, SUC_lexicon_tags_17 },
    { 0x406e18fa, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10e1901, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74e1904, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8e1908, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ae190d, SUC_lexicon_tags_45 },
    { 0xdfce190e, SUC_lexicon_tags_45 },
    { 0x97ae190d, SUC_lexicon_tags_17 },
    { 0x7b2e1910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14ae1912, SUC_lexicon_tags_9 },
    { 0x8e8e1913, SUC_lexicon_tags_11 },
    { 0x900e1913, SUC_lexicon_tags_14 },
    { 0x9cee1915, SUC_lexicon_tags_60 },
    { 0x27ce1916, SUC_lexicon_tags_15 },
    { 0x508e1916, SUC_lexicon_tags_29 },
    { 0x8e1918, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcb4e191a, SUC_lexicon_tags_37 },
    { 0xb58e191b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ae191d, SUC_lexicon_tags_0 },
    { 0xa18e191e, SUC_lexicon_tags_60 },
    { 0x608e191d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x820e1921, SUC_lexicon_tags_48 },
    { 0x228e1921, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2e2e1924, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6e1933, SUC_lexicon_tags_68 },
    { 0xcd6e1933, SUC_lexicon_tags_27 },
    { 0x12e1935, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1fee1937, SUC_lexicon_tags_25 },
    { 0xde2e1938, SUC_lexicon_tags_6 },
    { 0xcfae1939, SUC_lexicon_tags_25 },
    { 0x56ee1939, SUC_lexicon_tags_65 },
    { 0xe20e1939, SUC_lexicon_tags_15 },
    { 0x706e193c, SUC_lexicon_tags_11 },
    { 0x7e0e193d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2e1940, SUC_lexicon_tags_2 },
    { 0x2bce1941, SUC_lexicon_tags_15 },
    { 0xca4e1942, SUC_lexicon_tags_19 },
    { 0x83ae1943, SUC_lexicon_tags_15 },
    { 0xde6e1944, SUC_lexicon_tags_33 },
    { 0x51ee1944, SUC_lexicon_tags_65 },
    { 0xae1946, SUC_lexicon_tags_45 },
    { 0x860e1947, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x548e194a, SUC_lexicon_tags_14 },
    { 0x3a0e194b, SUC_lexicon_tags_1 },
    { 0xe0ee194a, SUC_lexicon_tags_17 },
    { 0xdbee194b, SUC_lexicon_tags_12 },
    { 0xc2ee194d, SUC_lexicon_tags_2 },
    { 0x15ce194f, SUC_lexicon_tags_27 },
    { 0x806e194f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114e1958, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x156e195a, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224e1962, SUC_lexicon_tags_23 },
    { 0x276e1963, SUC_lexicon_tags_64 },
    { 0xdc0e1963, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e1968, SUC_lexicon_tags_27 },
    { 0xaeae1969, SUC_lexicon_tags_37 },
    { 0xac2e1968, SUC_lexicon_tags_7 },
    { 0x6dee196b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ae1970, SUC_lexicon_tags_7 },
    { 0x432e1970, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddae1973, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eee1977, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce197a, SUC_lexicon_tags_37 },
    { 0x896e197b, SUC_lexicon_tags_17 },
    { 0x2e4e197a, SUC_lexicon_tags_15 },
    { 0x328e197a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4e1981, SUC_lexicon_tags_15 },
    { 0xb58e1982, SUC_lexicon_tags_11 },
    { 0x842e1981, SUC_lexicon_tags_15 },
    { 0x9f6e1984, SUC_lexicon_tags_9 },
    { 0x656e1985, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x954e1987, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2e198d, SUC_lexicon_tags_34 },
    { 0xb8ee198e, SUC_lexicon_tags_17 },
    { 0xf18e198e, SUC_lexicon_tags_11 },
    { 0xf3ee1990, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43ae1992, SUC_lexicon_tags_7 },
    { 0x402e1993, SUC_lexicon_tags_17 },
    { 0x492e1993, SUC_lexicon_tags_30 },
    { 0x398e1995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fae1999, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8e199e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584e19a5, SUC_lexicon_tags_29 },
    { 0xa22e19a6, SUC_lexicon_tags_28 },
    { 0xe1ae19a7, SUC_lexicon_tags_17 },
    { 0xe44e19a8, SUC_lexicon_tags_26 },
    { 0x16e19a7, SUC_lexicon_tags_37 },
    { 0x7c4e19a6, SUC_lexicon_tags_8 },
    { 0x760e19ab, SUC_lexicon_tags_11 },
    { 0xd02e19ac, SUC_lexicon_tags_37 },
    { 0x96ee19ac, SUC_lexicon_tags_47 },
    { 0xd30e19ab, SUC_lexicon_tags_11 },
    { 0x304e19af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0e19b2, SUC_lexicon_tags_11 },
    { 0x154e19b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc68e19b5, SUC_lexicon_tags_12 },
    { 0xf74e19b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb50e19b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952e19be, SUC_lexicon_tags_7 },
    { 0x5b4e19bf, SUC_lexicon_tags_37 },
    { 0x154e19bf, SUC_lexicon_tags_7 },
    { 0xadae19c1, SUC_lexicon_tags_25 },
    { 0x310e19c2, SUC_lexicon_tags_11 },
    { 0x706e19c1, SUC_lexicon_tags_45 },
    { 0xe04e19c2, SUC_lexicon_tags_37 },
    { 0x71ee19c5, SUC_lexicon_tags_60 },
    { 0x214e19c6, SUC_lexicon_tags_15 },
    { 0x82ae19c6, SUC_lexicon_tags_39 },
    { 0x3b4e19c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x208e19ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76e19cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512e19d1, SUC_lexicon_tags_37 },
    { 0x474e19d2, SUC_lexicon_tags_0 },
    { 0x6ce19d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafee19d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xffce19da, SUC_lexicon_tags_56 },
    { 0x2e4e19da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ce19de, SUC_lexicon_tags_11 },
    { 0x89ee19df, SUC_lexicon_tags_11 },
    { 0xd8e19e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eee19ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8b6e19f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6e19f4, SUC_lexicon_tags_87 },
    { 0xbfce19f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf06e19f7, SUC_lexicon_tags_17 },
    { 0xe0ee19f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb88e19fa, SUC_lexicon_tags_37 },
    { 0x3bce19fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x790e19fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadce1a00, SUC_lexicon_tags_47 },
    { 0x206e1a00, SUC_lexicon_tags_12 },
    { 0xa1ae1a02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ce1a05, SUC_lexicon_tags_17 },
    { 0xefce1a05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x358e1a09, SUC_lexicon_tags_5 },
    { 0x5f2e1a0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ae1a0d, SUC_lexicon_tags_14 },
    { 0xa1ee1a0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce1a12, SUC_lexicon_tags_19 },
    { 0x85ee1a13, SUC_lexicon_tags_11 },
    { 0xb8ce1a12, SUC_lexicon_tags_224 },
    { 0xc78e1a15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee1a19, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4e1a1d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ae1a20, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdce1a25, SUC_lexicon_tags_66 },
    { 0x8cce1a26, SUC_lexicon_tags_15 },
    { 0x21ee1a27, SUC_lexicon_tags_7 },
    { 0x918e1a28, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xebee1a2b, SUC_lexicon_tags_2 },
    { 0x8d8e1a2c, SUC_lexicon_tags_9 },
    { 0xfb4e1a2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6e1a31, SUC_lexicon_tags_1 },
    { 0xcce1a32, SUC_lexicon_tags_23 },
    { 0x582e1a33, SUC_lexicon_tags_37 },
    { 0x8c4e1a33, SUC_lexicon_tags_48 },
    { 0xb9ee1a35, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x522e1a37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764e1a3b, SUC_lexicon_tags_11 },
    { 0x530e1a3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566e1a43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb22e1a45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50e1a49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4e1a4c, SUC_lexicon_tags_16 },
    { 0x95ce1a4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x730e1a50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8e1a54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x882e1a57, SUC_lexicon_tags_17 },
    { 0x520e1a57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ee1a5d, SUC_lexicon_tags_1 },
    { 0x374e1a5e, SUC_lexicon_tags_47 },
    { 0x954e1a5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9ae1a61, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd4ce1a63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e8e1a65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180e1a69, SUC_lexicon_tags_68 },
    { 0x74ae1a6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2e1a6e, SUC_lexicon_tags_28 },
    { 0xa82e1a6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x570e1a71, SUC_lexicon_tags_10 },
    { 0x32ce1a72, SUC_lexicon_tags_3 },
    { 0x3a2e1a73, SUC_lexicon_tags_2 },
    { 0xbeae1a74, SUC_lexicon_tags_38 },
    { 0xe12e1a75, SUC_lexicon_tags_14 },
    { 0x9ece1a76, SUC_lexicon_tags_2 },
    { 0x34ee1a77, SUC_lexicon_tags_37 },
    { 0xdc8e1a74, SUC_lexicon_tags_31 },
    { 0x34ae1a72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8c0e1a7b, SUC_lexicon_tags_37 },
    { 0x976e1a7b, SUC_lexicon_tags_49 },
    { 0x70ce1a7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ee1a82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x330e1a85, SUC_lexicon_tags_6 },
    { 0xc7ee1a86, SUC_lexicon_tags_26 },
    { 0xb82e1a85, SUC_lexicon_tags_37 },
    { 0x4b2e1a86, SUC_lexicon_tags_29 },
    { 0x304e1a87, SUC_lexicon_tags_9 },
    { 0x5cee1a8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x60ce1a8c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x336e1a8f, SUC_lexicon_tags_70 },
    { 0x9dce1a8f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ee1a94, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ae1a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0e1a9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92ce1a9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2e1aa2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ae1aa6, SUC_lexicon_tags_62 },
    { 0xd36e1aa6, SUC_lexicon_tags_25 },
    { 0xdbae1aa7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ee1aab, SUC_lexicon_tags_82 },
    { 0xfcce1aab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dee1ab5, SUC_lexicon_tags_17 },
    { 0x96ee1ab5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818e1abc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ce1abf, SUC_lexicon_tags_65 },
    { 0x27ae1ac0, SUC_lexicon_tags_47 },
    { 0x904e1ac1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2e1ac4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ce1ac8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeae1acb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x412e1ace, SUC_lexicon_tags_1 },
    { 0xa26e1acf, SUC_lexicon_tags_15 },
    { 0xb0ae1ad0, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xd4ae1ad2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbbce1ad4, SUC_lexicon_tags_8 },
    { 0xdeee1ad5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef2e1ad7, SUC_lexicon_tags_63 },
    { 0x9f2e1ad8, SUC_lexicon_tags_11 },
    { 0xdeae1ad9, SUC_lexicon_tags_47 },
    { 0xe04e1ad8, SUC_lexicon_tags_11 },
    { 0xde8e1adb, SUC_lexicon_tags_19 },
    { 0xa82e1ad9, SUC_lexicon_tags_38 },
    { 0x16e1adb, SUC_lexicon_tags_15 },
    { 0x8dae1ad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ee1ae1, SUC_lexicon_tags_8 },
    { 0xe74e1ae1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ce1ae6, SUC_lexicon_tags_60 },
    { 0x1e6e1ae7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20e1aeb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36e1aef, SUC_lexicon_tags_17 },
    { 0xc98e1aef, SUC_lexicon_tags_15 },
    { 0x482e1af1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3b8e1af3, SUC_lexicon_tags_79 },
    { 0xf9ce1af3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4e1af8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774e1afd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf14e1aff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e8e1b01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e1b04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0ee1b06, SUC_lexicon_tags_38 },
    { 0x360e1b07, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2eee1b09, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x502e1b0b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x400e1b0d, SUC_lexicon_tags_2 },
    { 0x560e1b0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x980e1b11, SUC_lexicon_tags_15 },
    { 0x748e1b12, SUC_lexicon_tags_9 },
    { 0x308e1b13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4e1b16, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb6ce1b18, SUC_lexicon_tags_37 },
    { 0x8bee1b18, SUC_lexicon_tags_6 },
    { 0xedce1b1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x264e1b1d, SUC_lexicon_tags_11 },
    { 0x5eee1b1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ce1b21, SUC_lexicon_tags_7 },
    { 0xdd0e1b22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468e1b2a, SUC_lexicon_tags_60 },
    { 0x9cae1b2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bce1b2e, SUC_lexicon_tags_53 },
    { 0xf0e1b2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2e1b33, SUC_lexicon_tags_2 },
    { 0xa28e1b34, SUC_lexicon_tags_2 },
    { 0x8d0e1b34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e1b39, SUC_lexicon_tags_22 },
    { 0xf3ce1b3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ee1b45, SUC_lexicon_tags_0 },
    { 0xefce1b46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312e1b4c, SUC_lexicon_tags_1 },
    { 0x67ee1b4d, SUC_lexicon_tags_29 },
    { 0x692e1b4d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce1b51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fae1b54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8c4e1b56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ce1b5d, SUC_lexicon_tags_27 },
    { 0x46ae1b5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fee1b60, SUC_lexicon_tags_37 },
    { 0x808e1b61, SUC_lexicon_tags_1 },
    { 0x4f0e1b62, SUC_lexicon_tags_12 },
    { 0xe12e1b61, SUC_lexicon_tags_25 },
    { 0x88ce1b64, SUC_lexicon_tags_45 },
    { 0x37ee1b65, SUC_lexicon_tags_4 },
    { 0x7eee1b66, SUC_lexicon_tags_49 },
    { 0xd36e1b66, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x314e1b6a, SUC_lexicon_tags_12 },
    { 0x24ce1b6b, SUC_lexicon_tags_11 },
    { 0x29ce1b6c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x274e1b6f, SUC_lexicon_tags_25 },
    { 0xfa0e1b6f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xffee1b72, SUC_lexicon_tags_2 },
    { 0x696e1b73, SUC_lexicon_tags_1 },
    { 0xae8e1b72, SUC_lexicon_tags_17 },
    { 0xa0ae1b75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636e1b7d, SUC_lexicon_tags_37 },
    { 0x65ae1b7d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ae1b83, SUC_lexicon_tags_35 },
    { 0xf92e1b83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef0e1b86, SUC_lexicon_tags_17 },
    { 0xee8e1b87, SUC_lexicon_tags_31 },
    { 0xfcae1b87, SUC_lexicon_tags_31 },
    { 0xc06e1b88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaae1b8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fce1b90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cce1b95, SUC_lexicon_tags_12 },
    { 0x6a4e1b96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26e1b98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ee1b9b, SUC_lexicon_tags_45 },
    { 0x328e1b9b, SUC_lexicon_tags_15 },
    { 0x69ce1b9d, SUC_lexicon_tags_15 },
    { 0x58ce1b9b, SUC_lexicon_tags_1 },
    { 0x846e1b9f, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xc06e1ba1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6e1ba5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2e1ba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf6e1baa, SUC_lexicon_tags_28 },
    { 0xb48e1bab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x666e1bad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ae1bb1, SUC_lexicon_tags_29 },
    { 0x820e1bb2, SUC_lexicon_tags_47 },
    { 0x216e1bb3, SUC_lexicon_tags_9 },
    { 0xd3ee1bb4, SUC_lexicon_tags_17 },
    { 0xfb2e1bb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc24e1bb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4e1bbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2e1bc3, SUC_lexicon_tags_11 },
    { 0xd8ae1bc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ce1bc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcee1bc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a4e1bcb, SUC_lexicon_tags_25 },
    { 0x89ce1bcb, SUC_lexicon_tags_6 },
    { 0x758e1bcd, SUC_lexicon_tags_34 },
    { 0x874e1bce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616e1bd2, SUC_lexicon_tags_47 },
    { 0x46e1bd3, SUC_lexicon_tags_0 },
    { 0x72e1bd3, SUC_lexicon_tags_1 },
    { 0x41ae1bd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ae1bd7, SUC_lexicon_tags_28 },
    { 0xd0ae1bd8, SUC_lexicon_tags_12 },
    { 0x566e1bd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e1bdd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe72e1be0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeee1be3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ce1be7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508e1bee, SUC_lexicon_tags_11 },
    { 0xb98e1bee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee1bf2, SUC_lexicon_tags_17 },
    { 0x964e1bf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x518e1bf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd8e1bf7, SUC_lexicon_tags_1 },
    { 0x9e8e1bf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128e1bfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30e1c00, SUC_lexicon_tags_0 },
    { 0x2a8e1c00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332e1c05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16e1c0a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe92e1c0c, SUC_lexicon_tags_8 },
    { 0x4c2e1c0d, SUC_lexicon_tags_13 },
    { 0x6f0e1c0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ee1c11, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x512e1c13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee6e1c15, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0e1c1a, SUC_lexicon_tags_14 },
    { 0xa8ae1c1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2e1c1e, SUC_lexicon_tags_11 },
    { 0x368e1c1f, SUC_lexicon_tags_49 },
    { 0xdae1c1f, SUC_lexicon_tags_17 },
    { 0xe18e1c1f, SUC_lexicon_tags_11 },
    { 0x162e1c1e, SUC_lexicon_tags_9 },
    { 0xcd2e1c1f, SUC_lexicon_tags_11 },
    { 0xac6e1c24, SUC_lexicon_tags_37 },
    { 0x92ae1c25, SUC_lexicon_tags_2 },
    { 0xcb2e1c26, SUC_lexicon_tags_65 },
    { 0x52ee1c27, SUC_lexicon_tags_31 },
    { 0x67ee1c27, SUC_lexicon_tags_17 },
    { 0xe66e1c29, SUC_lexicon_tags_45 },
    { 0x12ee1c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce1c2d, SUC_lexicon_tags_30 },
    { 0x9f4e1c2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6e1c35, SUC_lexicon_tags_12 },
    { 0xf88e1c35, SUC_lexicon_tags_25 },
    { 0x352e1c37, SUC_lexicon_tags_114 },
    { 0x9e8e1c35, SUC_lexicon_tags_25 },
    { 0x388e1c39, SUC_lexicon_tags_37 },
    { 0xee2e1c3a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x630e1c3c, SUC_lexicon_tags_25 },
    { 0xd54e1c3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2e1c43, SUC_lexicon_tags_43 },
    { 0x508e1c43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd30e1c46, SUC_lexicon_tags_25 },
    { 0xccee1c46, SUC_lexicon_tags_2 },
    { 0x460e1c47, SUC_lexicon_tags_23 },
    { 0xbd8e1c49, SUC_lexicon_tags_12 },
    { 0xbf2e1c49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8e1c4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x900e1c51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8e1c55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400e1c5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cae1c5f, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccae1c66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06e1c6a, SUC_lexicon_tags_19 },
    { 0x44e1c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6e1c6f, SUC_lexicon_tags_12 },
    { 0x834e1c70, SUC_lexicon_tags_11 },
    { 0x4dce1c71, SUC_lexicon_tags_7 },
    { 0xfb2e1c72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ee1c75, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x944e1c78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52e1c7b, SUC_lexicon_tags_38 },
    { 0xe5ee1c7c, SUC_lexicon_tags_14 },
    { 0x134e1c7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68e1c80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4e1c83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8e1c86, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30e1c89, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ee1c8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf02e1c90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76e1c9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e1c9e, SUC_lexicon_tags_85 },
    { 0xbc6e1c9f, SUC_lexicon_tags_45 },
    { 0xf56e1ca0, SUC_lexicon_tags_0 },
    { 0x776e1ca1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ae1ca4, SUC_lexicon_tags_29 },
    { 0x45ae1ca5, SUC_lexicon_tags_87 },
    { 0x11ae1ca6, SUC_lexicon_tags_17 },
    { 0x26ce1ca7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xadce1caa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7eee1cac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd42e1cae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfae1cb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264e1cb9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968e1cbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd08e1cc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ee1cc6, SUC_lexicon_tags_37 },
    { 0x8a4e1cc7, SUC_lexicon_tags_30 },
    { 0x124e1cc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9ae1cca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dce1cce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4e1cd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe88e1cd4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690e1cd8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x84ce1cda, SUC_lexicon_tags_1 },
    { 0x1a0e1cdb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ae1ce0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee1ce3, SUC_lexicon_tags_30 },
    { 0x8cee1ce4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc28e1ce6, SUC_lexicon_tags_17 },
    { 0xa32e1ce7, SUC_lexicon_tags_60 },
    { 0xe8ce1ce8, SUC_lexicon_tags_7 },
    { 0xb44e1ce9, SUC_lexicon_tags_33 },
    { 0x108e1cea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ee1cf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ee1cfa, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xc10e1cfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ce1d03, SUC_lexicon_tags_19 },
    { 0x95ee1d03, SUC_lexicon_tags_25 },
    { 0xe8ae1d03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78e1d09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x288e1d0b, SUC_lexicon_tags_26 },
    { 0xcb0e1d0c, SUC_lexicon_tags_6 },
    { 0xc6ae1d0d, SUC_lexicon_tags_11 },
    { 0xb50e1d0e, SUC_lexicon_tags_60 },
    { 0x68ce1d0e, SUC_lexicon_tags_11 },
    { 0x754e1d10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb4e1d12, SUC_lexicon_tags_37 },
    { 0xbd6e1d13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36e1d15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcce1d18, SUC_lexicon_tags_57 },
    { 0xb16e1d18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22e1d1f, SUC_lexicon_tags_17 },
    { 0x780e1d20, SUC_lexicon_tags_7 },
    { 0x4bee1d1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4e1d25, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x824e1d28, SUC_lexicon_tags_0 },
    { 0xc8ee1d28, SUC_lexicon_tags_39 },
    { 0x6bce1d28, SUC_lexicon_tags_2 },
    { 0xcaae1d2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cce1d2f, SUC_lexicon_tags_63 },
    { 0xe68e1d2f, SUC_lexicon_tags_25 },
    { 0xd32e1d30, SUC_lexicon_tags_12 },
    { 0x3fce1d32, SUC_lexicon_tags_49 },
    { 0x164e1d31, SUC_lexicon_tags_7 },
    { 0xccae1d34, SUC_lexicon_tags_37 },
    { 0xcd6e1d35, SUC_lexicon_tags_14 },
    { 0xa64e1d34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40e1d3b, SUC_lexicon_tags_8 },
    { 0x978e1d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302e1d43, SUC_lexicon_tags_60 },
    { 0x8bee1d44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ee1d47, SUC_lexicon_tags_1 },
    { 0xebce1d48, SUC_lexicon_tags_17 },
    { 0xfe4e1d49, SUC_lexicon_tags_15 },
    { 0xa14e1d49, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78e1d4e, SUC_lexicon_tags_6 },
    { 0xb50e1d4f, SUC_lexicon_tags_1 },
    { 0x37ce1d50, SUC_lexicon_tags_37 },
    { 0x6dae1d51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894e1d55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372e1d59, SUC_lexicon_tags_1 },
    { 0xb0e1d5a, SUC_lexicon_tags_6 },
    { 0xd7ae1d5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4e1d63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6e1d66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e1d69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132e1d72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x45ee1d74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ee1d77, SUC_lexicon_tags_16 },
    { 0x46e1d77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e1d7c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf76e1d7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ce1d81, SUC_lexicon_tags_27 },
    { 0xa2e1d82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a6e1d84, SUC_lexicon_tags_19 },
    { 0xda4e1d85, SUC_lexicon_tags_0 },
    { 0x564e1d86, SUC_lexicon_tags_34 },
    { 0x9e4e1d85, SUC_lexicon_tags_68 },
    { 0x2a8e1d88, SUC_lexicon_tags_33 },
    { 0xc40e1d84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ce1d8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0e1d92, SUC_lexicon_tags_19 },
    { 0x436e1d93, SUC_lexicon_tags_27 },
    { 0x708e1d93, SUC_lexicon_tags_25 },
    { 0x41ce1d95, SUC_lexicon_tags_33 },
    { 0x4fce1d95, SUC_lexicon_tags_38 },
    { 0x268e1d94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58e1d9a, SUC_lexicon_tags_12 },
    { 0x30ae1d9b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08e1d9e, SUC_lexicon_tags_19 },
    { 0x55ce1d9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe02e1da2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccce1da6, SUC_lexicon_tags_47 },
    { 0x976e1da6, SUC_lexicon_tags_7 },
    { 0x4b2e1da7, SUC_lexicon_tags_98 },
    { 0x650e1da9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcce1dad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ae1db1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2e1db4, SUC_lexicon_tags_4 },
    { 0xa38e1db4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbeee1db7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ae1dbd, SUC_lexicon_tags_37 },
    { 0x840e1dbe, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2e1dc8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e1dca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4e1dcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f8e1dd1, SUC_lexicon_tags_0 },
    { 0x512e1dd2, SUC_lexicon_tags_29 },
    { 0x1d6e1dd3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0e1dd6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9b2e1dd8, SUC_lexicon_tags_37 },
    { 0x6fce1dd9, SUC_lexicon_tags_53 },
    { 0xc2ee1dda, SUC_lexicon_tags_47 },
    { 0x8c2e1ddb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240e1de5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36e1def, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5a0e1df1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaee1df3, SUC_lexicon_tags_11 },
    { 0xace1df4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc8e1df6, SUC_lexicon_tags_43 },
    { 0xaee1df7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe0e1df9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x968e1dfb, SUC_lexicon_tags_0 },
    { 0x96e1dfc, SUC_lexicon_tags_430 },
    { 0, NULL },
    { 0xbcae1dfe, SUC_lexicon_tags_2 },
    { 0xe1ae1dff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6e1e02, SUC_lexicon_tags_25 },
    { 0x76ee1e03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6e1e08, SUC_lexicon_tags_14 },
    { 0xce4e1e09, SUC_lexicon_tags_12 },
    { 0x596e1e0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x720e1e0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59ce1e0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaae1e12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0e1e19, SUC_lexicon_tags_7 },
    { 0x1dae1e19, SUC_lexicon_tags_1 },
    { 0x906e1e19, SUC_lexicon_tags_11 },
    { 0x320e1e1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb6e1e1e, SUC_lexicon_tags_33 },
    { 0xea4e1e1e, SUC_lexicon_tags_14 },
    { 0xa8ce1e20, SUC_lexicon_tags_12 },
    { 0xcfae1e20, SUC_lexicon_tags_14 },
    { 0x904e1e22, SUC_lexicon_tags_60 },
    { 0xddce1e22, SUC_lexicon_tags_23 },
    { 0x2c2e1e24, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476e1e28, SUC_lexicon_tags_17 },
    { 0x9fae1e29, SUC_lexicon_tags_14 },
    { 0x30ae1e29, SUC_lexicon_tags_7 },
    { 0x9bce1e2b, SUC_lexicon_tags_11 },
    { 0x526e1e2b, SUC_lexicon_tags_25 },
    { 0x220e1e2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d8e1e2f, SUC_lexicon_tags_23 },
    { 0x448e1e30, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76e1e34, SUC_lexicon_tags_0 },
    { 0x8ae1e35, SUC_lexicon_tags_14 },
    { 0xa36e1e36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e1e41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dce1e45, SUC_lexicon_tags_27 },
    { 0x428e1e46, SUC_lexicon_tags_75 },
    { 0x508e1e47, SUC_lexicon_tags_450 },
    { 0xd84e1e47, SUC_lexicon_tags_0 },
    { 0xf5ce1e48, SUC_lexicon_tags_47 },
    { 0x8ece1e4a, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xfece1e4d, SUC_lexicon_tags_25 },
    { 0x914e1e4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ee1e51, SUC_lexicon_tags_37 },
    { 0x93ae1e51, SUC_lexicon_tags_47 },
    { 0x756e1e53, SUC_lexicon_tags_87 },
    { 0xc98e1e53, SUC_lexicon_tags_9 },
    { 0xfb6e1e55, SUC_lexicon_tags_43 },
    { 0x618e1e56, SUC_lexicon_tags_15 },
    { 0x6e1e57, SUC_lexicon_tags_37 },
    { 0x312e1e58, SUC_lexicon_tags_5 },
    { 0xf26e1e53, SUC_lexicon_tags_37 },
    { 0x192e1e54, SUC_lexicon_tags_54 },
    { 0x2b2e1e5b, SUC_lexicon_tags_17 },
    { 0xc1ce1e5c, SUC_lexicon_tags_27 },
    { 0x7eae1e5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xae8e1e5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec0e1e61, SUC_lexicon_tags_28 },
    { 0x6ce1e62, SUC_lexicon_tags_11 },
    { 0x27ce1e63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x534e1e65, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0e1e69, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xacae1e6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecae1e6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc8e1e70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464e1e74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ee1e78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x32e1e7a, SUC_lexicon_tags_37 },
    { 0x4fce1e7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ae1e81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cce1e84, SUC_lexicon_tags_14 },
    { 0x596e1e85, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe50e1e87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0e1e8a, SUC_lexicon_tags_25 },
    { 0x8e1e8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd0ae1e8d, SUC_lexicon_tags_28 },
    { 0xc0ee1e8d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986e1e95, SUC_lexicon_tags_1 },
    { 0x8d8e1e95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e1e99, SUC_lexicon_tags_1 },
    { 0x18e1e9a, SUC_lexicon_tags_23 },
    { 0x1dae1e9b, SUC_lexicon_tags_6 },
    { 0xef4e1e9a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08e1ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b2e1ea7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ee1eaa, SUC_lexicon_tags_40 },
    { 0x5a2e1eaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ace1eb0, SUC_lexicon_tags_2 },
    { 0xe3ae1eb1, SUC_lexicon_tags_4 },
    { 0x264e1eb1, SUC_lexicon_tags_60 },
    { 0x92ee1eb3, SUC_lexicon_tags_1 },
    { 0x500e1eb4, SUC_lexicon_tags_8 },
    { 0xf60e1eb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x380e1eb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ae1ebd, SUC_lexicon_tags_37 },
    { 0xcd2e1ebe, SUC_lexicon_tags_43 },
    { 0xfa0e1ebd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692e1ec4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8e1ec7, SUC_lexicon_tags_14 },
    { 0xf54e1ec8, SUC_lexicon_tags_23 },
    { 0xe2ae1ec7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ee1ecc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ace1ece, SUC_lexicon_tags_243 },
    { 0xe0ce1ecf, SUC_lexicon_tags_2 },
    { 0xf2e1ed0, SUC_lexicon_tags_21 },
    { 0xd94e1ecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ae1ed7, SUC_lexicon_tags_14 },
    { 0x58ee1ed8, SUC_lexicon_tags_38 },
    { 0x514e1ed8, SUC_lexicon_tags_5 },
    { 0x720e1eda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66e1ee1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc8ae1ee3, SUC_lexicon_tags_44 },
    { 0xd58e1ee4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0e1ee7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x152e1ee9, SUC_lexicon_tags_29 },
    { 0xb66e1eea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xddce1eed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ae1ef2, SUC_lexicon_tags_7 },
    { 0xeb2e1ef3, SUC_lexicon_tags_1 },
    { 0x720e1ef4, SUC_lexicon_tags_289 },
    { 0xe20e1ef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ace1f07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4e1f0b, SUC_lexicon_tags_7 },
    { 0xa68e1f0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ee1f13, SUC_lexicon_tags_23 },
    { 0xfa8e1f13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a2e1f16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcece1f19, SUC_lexicon_tags_26 },
    { 0xc92e1f19, SUC_lexicon_tags_25 },
    { 0x608e1f19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6e1f20, SUC_lexicon_tags_50 },
    { 0x834e1f20, SUC_lexicon_tags_10 },
    { 0x93ce1f22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x592e1f24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0e1f27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416e1f2b, SUC_lexicon_tags_7 },
    { 0x8ee1f2b, SUC_lexicon_tags_23 },
    { 0x2dae1f2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea0e1f2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x880e1f31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd02e1f33, SUC_lexicon_tags_23 },
    { 0xa1ce1f33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a4e1f36, SUC_lexicon_tags_15 },
    { 0x27ae1f37, SUC_lexicon_tags_79 },
    { 0xcd4e1f38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dce1f3b, SUC_lexicon_tags_8 },
    { 0xe24e1f3c, SUC_lexicon_tags_49 },
    { 0x814e1f3d, SUC_lexicon_tags_12 },
    { 0xc56e1f3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344e1f4d, SUC_lexicon_tags_11 },
    { 0xcd0e1f4e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeece1f53, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce1f56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618e1f5e, SUC_lexicon_tags_25 },
    { 0xdce1f5f, SUC_lexicon_tags_11 },
    { 0xec8e1f5e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370e1f65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ee1f6b, SUC_lexicon_tags_63 },
    { 0x4b6e1f6b, SUC_lexicon_tags_37 },
    { 0xddce1f6c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x146e1f6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8e1f78, SUC_lexicon_tags_55 },
    { 0x20ce1f79, SUC_lexicon_tags_12 },
    { 0xae2e1f7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdae1f7e, SUC_lexicon_tags_1 },
    { 0xe2ce1f7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaee1f81, SUC_lexicon_tags_47 },
    { 0x8c4e1f82, SUC_lexicon_tags_17 },
    { 0x9cae1f83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ee1f8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x930e1f8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x296e1f8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e8e1f91, SUC_lexicon_tags_37 },
    { 0x79ae1f92, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96e1f97, SUC_lexicon_tags_7 },
    { 0xd24e1f98, SUC_lexicon_tags_11 },
    { 0x4d2e1f99, SUC_lexicon_tags_148 },
    { 0x788e1f9a, SUC_lexicon_tags_37 },
    { 0xf30e1f97, SUC_lexicon_tags_30 },
    { 0x198e1f9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20e1fa7, SUC_lexicon_tags_37 },
    { 0xa1ae1fa7, SUC_lexicon_tags_29 },
    { 0x2d0e1fa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x216e1fac, SUC_lexicon_tags_17 },
    { 0x57ce1fac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440e1fb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbee1fb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402e1fba, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x856e1fbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93ae1fbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd28e1fc1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3c6e1fc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd58e1fc5, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbce1fca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x608e1fcc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4e1fd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x158e1fd3, SUC_lexicon_tags_45 },
    { 0x5dce1fd4, SUC_lexicon_tags_12 },
    { 0xb44e1fd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dee1fd8, SUC_lexicon_tags_60 },
    { 0x1aae1fd9, SUC_lexicon_tags_15 },
    { 0xca8e1fda, SUC_lexicon_tags_1 },
    { 0xe22e1fda, SUC_lexicon_tags_33 },
    { 0xae2e1fdc, SUC_lexicon_tags_25 },
    { 0xbaae1fdd, SUC_lexicon_tags_16 },
    { 0x354e1fde, SUC_lexicon_tags_45 },
    { 0x974e1fde, SUC_lexicon_tags_32 },
    { 0x610e1fe0, SUC_lexicon_tags_17 },
    { 0x438e1fe1, SUC_lexicon_tags_29 },
    { 0xe14e1fe1, SUC_lexicon_tags_25 },
    { 0x7ae1fe1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d4e1fe5, SUC_lexicon_tags_37 },
    { 0x62ae1fe6, SUC_lexicon_tags_17 },
    { 0xa16e1fe6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c8e1fe9, SUC_lexicon_tags_25 },
    { 0xc44e1fea, SUC_lexicon_tags_47 },
    { 0xda4e1fe9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ee1fee, SUC_lexicon_tags_66 },
    { 0x6ace1fef, SUC_lexicon_tags_46 },
    { 0x7c4e1fef, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x320e1ff2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8e1ff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2e1ff9, SUC_lexicon_tags_15 },
    { 0x554e1ffa, SUC_lexicon_tags_60 },
    { 0x190e1ffa, SUC_lexicon_tags_59 },
    { 0x75ce1ffb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10e2000, SUC_lexicon_tags_44 },
    { 0xf52e2001, SUC_lexicon_tags_1 },
    { 0x5a8e2002, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f0e2004, SUC_lexicon_tags_0 },
    { 0x294e2005, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce200f, SUC_lexicon_tags_62 },
    { 0x25ae200f, SUC_lexicon_tags_37 },
    { 0xbbce200f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae0e2013, SUC_lexicon_tags_4 },
    { 0x3cce2014, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4e2017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ce2019, SUC_lexicon_tags_29 },
    { 0x63ae201a, SUC_lexicon_tags_1 },
    { 0x9cce201b, SUC_lexicon_tags_27 },
    { 0x34ae201c, SUC_lexicon_tags_16 },
    { 0x5e8e201c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee2022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2e2028, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bce202c, SUC_lexicon_tags_15 },
    { 0x772e202d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348e2032, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa88e2034, SUC_lexicon_tags_27 },
    { 0xe8ce2035, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82e2038, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff6e203a, SUC_lexicon_tags_10 },
    { 0x27ce203b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904e2040, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dee2044, SUC_lexicon_tags_2 },
    { 0x760e2045, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6e2048, SUC_lexicon_tags_12 },
    { 0x38ae2048, SUC_lexicon_tags_26 },
    { 0x300e204a, SUC_lexicon_tags_25 },
    { 0x1d2e204b, SUC_lexicon_tags_25 },
    { 0x7bae204b, SUC_lexicon_tags_60 },
    { 0x90e204b, SUC_lexicon_tags_17 },
    { 0x56ae204e, SUC_lexicon_tags_243 },
    { 0xc60e2049, SUC_lexicon_tags_8 },
    { 0x8eee2050, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fae205e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x594e2061, SUC_lexicon_tags_11 },
    { 0x54e2062, SUC_lexicon_tags_4 },
    { 0xf1ee2063, SUC_lexicon_tags_1 },
    { 0xe0ee2063, SUC_lexicon_tags_38 },
    { 0x880e2061, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cee2068, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522e206c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dce2072, SUC_lexicon_tags_23 },
    { 0xbb8e2073, SUC_lexicon_tags_12 },
    { 0x64ce2072, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabce207b, SUC_lexicon_tags_7 },
    { 0x948e207b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14e2080, SUC_lexicon_tags_9 },
    { 0x900e2080, SUC_lexicon_tags_8 },
    { 0x9b0e2081, SUC_lexicon_tags_7 },
    { 0x178e2083, SUC_lexicon_tags_18 },
    { 0xef0e2084, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dce2088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6e2091, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2e2094, SUC_lexicon_tags_8 },
    { 0x40ae2094, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x848e2097, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ae209a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x846e209c, SUC_lexicon_tags_23 },
    { 0x2b2e209c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf6e209f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x952e20a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58ee20a3, SUC_lexicon_tags_29 },
    { 0xcae20a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x562e20a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2e20aa, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x868e20ad, SUC_lexicon_tags_1 },
    { 0xc0ae20ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d2e20b0, SUC_lexicon_tags_83 },
    { 0x692e20b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4e20b7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90e20bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932e20bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44e20c2, SUC_lexicon_tags_2 },
    { 0x3d2e20c3, SUC_lexicon_tags_55 },
    { 0xde2e20c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4e20cc, SUC_lexicon_tags_32 },
    { 0x2c4e20cd, SUC_lexicon_tags_27 },
    { 0x2a0e20ce, SUC_lexicon_tags_11 },
    { 0x33ce20cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912e20d4, SUC_lexicon_tags_25 },
    { 0x642e20d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75ee20d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd8e20d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0e20db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2e20e0, SUC_lexicon_tags_37 },
    { 0x76ce20e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee20e6, SUC_lexicon_tags_39 },
    { 0x94ce20e6, SUC_lexicon_tags_49 },
    { 0xd5ee20e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7aee20ea, SUC_lexicon_tags_80 },
    { 0x62ae20eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37ae20ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae20f0, SUC_lexicon_tags_37 },
    { 0x3d0e20f0, SUC_lexicon_tags_825 },
    { 0, NULL },
    { 0xbb0e20f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168e20f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2e20fb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ce2100, SUC_lexicon_tags_11 },
    { 0x724e2100, SUC_lexicon_tags_37 },
    { 0x7b6e2102, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0e2108, SUC_lexicon_tags_33 },
    { 0xf0e2108, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x68ae210b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8e2110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x556e2113, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeace211a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7e0e211c, SUC_lexicon_tags_37 },
    { 0x9aee211d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c6e211f, SUC_lexicon_tags_41 },
    { 0xe40e211f, SUC_lexicon_tags_39 },
    { 0xed6e2121, SUC_lexicon_tags_11 },
    { 0x268e2122, SUC_lexicon_tags_12 },
    { 0x2f8e2123, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd8e2125, SUC_lexicon_tags_25 },
    { 0xfd4e2126, SUC_lexicon_tags_45 },
    { 0x710e2127, SUC_lexicon_tags_53 },
    { 0x3a8e2125, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb4e212a, SUC_lexicon_tags_87 },
    { 0x44ee212b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dce212f, SUC_lexicon_tags_23 },
    { 0xa90e212f, SUC_lexicon_tags_15 },
    { 0x946e2131, SUC_lexicon_tags_31 },
    { 0x74ee2131, SUC_lexicon_tags_1 },
    { 0x18e2133, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x998e2135, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x742e2137, SUC_lexicon_tags_17 },
    { 0xd0ce2138, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398e213e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0e2144, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ee214d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcce2151, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf26e2153, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96e2159, SUC_lexicon_tags_37 },
    { 0xa2e2159, SUC_lexicon_tags_19 },
    { 0x222e2159, SUC_lexicon_tags_2 },
    { 0x332e215c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ee215f, SUC_lexicon_tags_118 },
    { 0x8e215f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x772e2162, SUC_lexicon_tags_23 },
    { 0xe22e2162, SUC_lexicon_tags_65 },
    { 0x31ce2162, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x498e2167, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcee2169, SUC_lexicon_tags_9 },
    { 0x3a0e216a, SUC_lexicon_tags_87 },
    { 0x692e216b, SUC_lexicon_tags_15 },
    { 0xfae216b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x422e216f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e2173, SUC_lexicon_tags_0 },
    { 0xc18e2174, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670e2178, SUC_lexicon_tags_11 },
    { 0xa36e2178, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ce217e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558e2187, SUC_lexicon_tags_14 },
    { 0x8c8e2187, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee218c, SUC_lexicon_tags_32 },
    { 0x936e218d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x186e2190, SUC_lexicon_tags_39 },
    { 0xfc0e2191, SUC_lexicon_tags_23 },
    { 0x938e2190, SUC_lexicon_tags_12 },
    { 0x50ee2192, SUC_lexicon_tags_31 },
    { 0xee2194, SUC_lexicon_tags_3 },
    { 0xf70e2195, SUC_lexicon_tags_1 },
    { 0xb56e2191, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8e219c, SUC_lexicon_tags_63 },
    { 0x718e219c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ee21a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e21a7, SUC_lexicon_tags_11 },
    { 0xe4e21a7, SUC_lexicon_tags_32 },
    { 0xd64e21a8, SUC_lexicon_tags_63 },
    { 0x898e21a8, SUC_lexicon_tags_37 },
    { 0x8e0e21a8, SUC_lexicon_tags_39 },
    { 0x644e21a9, SUC_lexicon_tags_60 },
    { 0x7b0e21ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302e21b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ae21b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eae21b7, SUC_lexicon_tags_12 },
    { 0xfce21b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dee21ba, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112e21be, SUC_lexicon_tags_42 },
    { 0xce0e21be, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424e21c5, SUC_lexicon_tags_33 },
    { 0x15ce21c6, SUC_lexicon_tags_4 },
    { 0x57ce21c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ae21cf, SUC_lexicon_tags_23 },
    { 0x6c6e21d0, SUC_lexicon_tags_7 },
    { 0xae21d1, SUC_lexicon_tags_25 },
    { 0x3e8e21d2, SUC_lexicon_tags_37 },
    { 0xbbce21d1, SUC_lexicon_tags_45 },
    { 0x3cae21d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dee21d7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x758e21d9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e21e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x590e21e7, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x8bae21e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ee21f4, SUC_lexicon_tags_7 },
    { 0xd14e21f4, SUC_lexicon_tags_27 },
    { 0xf22e21f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ce21f9, SUC_lexicon_tags_47 },
    { 0x2dae21f9, SUC_lexicon_tags_15 },
    { 0x484e21fb, SUC_lexicon_tags_12 },
    { 0x308e21fc, SUC_lexicon_tags_11 },
    { 0x140e21fd, SUC_lexicon_tags_37 },
    { 0xbe2e21fe, SUC_lexicon_tags_37 },
    { 0xfcce21fc, SUC_lexicon_tags_31 },
    { 0x2c8e2200, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ee2204, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ee2208, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ee220d, SUC_lexicon_tags_28 },
    { 0xc1ae220d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ae221a, SUC_lexicon_tags_682 },
    { 0x5f0e221a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4a8e221d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e4e221f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x542e2222, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ee2229, SUC_lexicon_tags_2 },
    { 0x2dee222a, SUC_lexicon_tags_61 },
    { 0x8c0e222b, SUC_lexicon_tags_32 },
    { 0xfdce222c, SUC_lexicon_tags_11 },
    { 0x15ce222d, SUC_lexicon_tags_37 },
    { 0xdaee222a, SUC_lexicon_tags_31 },
    { 0xae4e222f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ee2234, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0e2238, SUC_lexicon_tags_17 },
    { 0xf16e2239, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ce223f, SUC_lexicon_tags_31 },
    { 0x550e2240, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x792e2242, SUC_lexicon_tags_17 },
    { 0x298e2243, SUC_lexicon_tags_17 },
    { 0x9ace2244, SUC_lexicon_tags_27 },
    { 0x818e2245, SUC_lexicon_tags_14 },
    { 0xe68e2244, SUC_lexicon_tags_15 },
    { 0x89ae2243, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6e224c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2e224f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf60e2251, SUC_lexicon_tags_11 },
    { 0x8b2e2252, SUC_lexicon_tags_14 },
    { 0x5c4e2251, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10e2256, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7c4e2258, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ee225b, SUC_lexicon_tags_14 },
    { 0x1dce225c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7ce225e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136e2263, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0e2266, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2e226f, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6e2272, SUC_lexicon_tags_49 },
    { 0x218e2273, SUC_lexicon_tags_19 },
    { 0x8a6e2274, SUC_lexicon_tags_25 },
    { 0x652e2272, SUC_lexicon_tags_12 },
    { 0xb4ee2276, SUC_lexicon_tags_63 },
    { 0xbc6e2276, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3d8e2279, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ee2280, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6e2287, SUC_lexicon_tags_3 },
    { 0xe2ae2288, SUC_lexicon_tags_17 },
    { 0xa16e2289, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706e228d, SUC_lexicon_tags_32 },
    { 0x524e228d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2e2292, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ee2296, SUC_lexicon_tags_101 },
    { 0x13ce2297, SUC_lexicon_tags_351 },
    { 0xfc2e2298, SUC_lexicon_tags_25 },
    { 0x9d6e2299, SUC_lexicon_tags_34 },
    { 0x6ce2299, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6eae229c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc86e229e, SUC_lexicon_tags_11 },
    { 0xd52e229f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x938e22a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990e22a5, SUC_lexicon_tags_17 },
    { 0x7d6e22a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6e22ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ae22b1, SUC_lexicon_tags_47 },
    { 0xbdae22b1, SUC_lexicon_tags_37 },
    { 0xc68e22b3, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0e22b6, SUC_lexicon_tags_37 },
    { 0x20ee22b6, SUC_lexicon_tags_0 },
    { 0x57ee22b7, SUC_lexicon_tags_15 },
    { 0x1eee22b6, SUC_lexicon_tags_15 },
    { 0xd56e22b8, SUC_lexicon_tags_301 },
    { 0xc24e22bb, SUC_lexicon_tags_1 },
    { 0x9dce22b7, SUC_lexicon_tags_1 },
    { 0x10ce22bd, SUC_lexicon_tags_63 },
    { 0xe42e22bc, SUC_lexicon_tags_37 },
    { 0xbc0e22ba, SUC_lexicon_tags_7 },
    { 0x2c4e22c0, SUC_lexicon_tags_11 },
    { 0xa0e22c1, SUC_lexicon_tags_2 },
    { 0xed4e22c0, SUC_lexicon_tags_19 },
    { 0xa56e22c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e2e22c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd18e22c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0e22ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ce22d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x528e22d5, SUC_lexicon_tags_27 },
    { 0x42e22d5, SUC_lexicon_tags_24 },
    { 0x930e22d7, SUC_lexicon_tags_68 },
    { 0xf5ae22d8, SUC_lexicon_tags_37 },
    { 0x1d0e22d9, SUC_lexicon_tags_27 },
    { 0xdc4e22d8, SUC_lexicon_tags_31 },
    { 0x2ce22d9, SUC_lexicon_tags_1 },
    { 0xdc6e22dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x66e22de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426e22e7, SUC_lexicon_tags_60 },
    { 0xb38e22e8, SUC_lexicon_tags_11 },
    { 0x32ce22e8, SUC_lexicon_tags_79 },
    { 0xd7ee22ea, SUC_lexicon_tags_37 },
    { 0x6cae22e8, SUC_lexicon_tags_17 },
    { 0x1c4e22ec, SUC_lexicon_tags_46 },
    { 0xdc0e22ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2e22f0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0e22f3, SUC_lexicon_tags_7 },
    { 0x34ae22f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4e22f7, SUC_lexicon_tags_33 },
    { 0x598e22f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46ae22fa, SUC_lexicon_tags_7 },
    { 0xfe6e22fa, SUC_lexicon_tags_15 },
    { 0x4e4e22fb, SUC_lexicon_tags_43 },
    { 0x854e22fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8e2300, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ee230a, SUC_lexicon_tags_38 },
    { 0x482e230a, SUC_lexicon_tags_7 },
    { 0xbdce230c, SUC_lexicon_tags_1 },
    { 0x1eee230a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae2312, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7fae2314, SUC_lexicon_tags_28 },
    { 0x5bee2315, SUC_lexicon_tags_19 },
    { 0x734e2316, SUC_lexicon_tags_12 },
    { 0x1d2e2315, SUC_lexicon_tags_29 },
    { 0x106e2317, SUC_lexicon_tags_7 },
    { 0x718e2319, SUC_lexicon_tags_23 },
    { 0x4b4e2315, SUC_lexicon_tags_25 },
    { 0x970e231b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2e231f, SUC_lexicon_tags_37 },
    { 0x6b6e2320, SUC_lexicon_tags_37 },
    { 0x7c6e2321, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbae2324, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ce2329, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xceee232c, SUC_lexicon_tags_7 },
    { 0x510e232d, SUC_lexicon_tags_17 },
    { 0x422e232e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e2331, SUC_lexicon_tags_45 },
    { 0xa80e2331, SUC_lexicon_tags_11 },
    { 0x7b0e2333, SUC_lexicon_tags_0 },
    { 0x17ae2334, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954e2339, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce233c, SUC_lexicon_tags_17 },
    { 0x2d6e233d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ce2340, SUC_lexicon_tags_37 },
    { 0x1aae2341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e2350, SUC_lexicon_tags_28 },
    { 0xd18e2351, SUC_lexicon_tags_37 },
    { 0xb72e2351, SUC_lexicon_tags_9 },
    { 0x420e2352, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58e235f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88e2368, SUC_lexicon_tags_2 },
    { 0x10ce2368, SUC_lexicon_tags_37 },
    { 0xb12e236a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14e236d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xf6ae236f, SUC_lexicon_tags_27 },
    { 0x32ee236f, SUC_lexicon_tags_7 },
    { 0x2d0e2371, SUC_lexicon_tags_34 },
    { 0x83ae2371, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8e2376, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9ee2378, SUC_lexicon_tags_37 },
    { 0x56ee2379, SUC_lexicon_tags_15 },
    { 0xae237a, SUC_lexicon_tags_19 },
    { 0xaace237b, SUC_lexicon_tags_14 },
    { 0x430e237b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe50e237e, SUC_lexicon_tags_63 },
    { 0x17ae237f, SUC_lexicon_tags_30 },
    { 0x8bae2380, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ae2384, SUC_lexicon_tags_60 },
    { 0x318e2384, SUC_lexicon_tags_25 },
    { 0x2fce2384, SUC_lexicon_tags_27 },
    { 0x49ee2385, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x736e2389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x110e238c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb18e238e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638e2395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x146e2398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a4e239a, SUC_lexicon_tags_34 },
    { 0x358e239b, SUC_lexicon_tags_25 },
    { 0x3a8e239c, SUC_lexicon_tags_21 },
    { 0x68ae239d, SUC_lexicon_tags_63 },
    { 0xd30e239e, SUC_lexicon_tags_11 },
    { 0xd72e239e, SUC_lexicon_tags_43 },
    { 0x2c8e23a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xca2e23a2, SUC_lexicon_tags_9 },
    { 0xeace23a2, SUC_lexicon_tags_47 },
    { 0xa6e23a2, SUC_lexicon_tags_37 },
    { 0x64e23a3, SUC_lexicon_tags_17 },
    { 0x63ee23a6, SUC_lexicon_tags_17 },
    { 0x74ce23a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x912e23aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ae23b1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x70ee23b3, SUC_lexicon_tags_399 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ee23b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x332e23bb, SUC_lexicon_tags_17 },
    { 0x83ce23bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722e23c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4ee23c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2e23c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ee23c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeae23cd, SUC_lexicon_tags_27 },
    { 0xf90e23ce, SUC_lexicon_tags_25 },
    { 0x51ee23cf, SUC_lexicon_tags_10 },
    { 0x9d6e23cf, SUC_lexicon_tags_15 },
    { 0x8cae23d0, SUC_lexicon_tags_68 },
    { 0x84ee23d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2e23d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940e23d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4e23dd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ae23e0, SUC_lexicon_tags_11 },
    { 0xb04e23e1, SUC_lexicon_tags_57 },
    { 0x818e23e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2e23e9, SUC_lexicon_tags_25 },
    { 0xba2e23e9, SUC_lexicon_tags_33 },
    { 0x76ce23eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46e23ee, SUC_lexicon_tags_55 },
    { 0x9c6e23ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96ae23f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17ee23f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26e23f5, SUC_lexicon_tags_27 },
    { 0x79ae23f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32ee23f8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ae23fe, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56e2403, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6e2406, SUC_lexicon_tags_3 },
    { 0x4c2e2406, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ee240a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x258e240d, SUC_lexicon_tags_1 },
    { 0x1c8e240d, SUC_lexicon_tags_37 },
    { 0x172e240f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc9ee2411, SUC_lexicon_tags_29 },
    { 0xe18e2412, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bae241a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52e241d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dee2421, SUC_lexicon_tags_48 },
    { 0x9aae2422, SUC_lexicon_tags_7 },
    { 0x732e2423, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ce242c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300e2431, SUC_lexicon_tags_11 },
    { 0xa68e2432, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cce2437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ce243b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65ee243d, SUC_lexicon_tags_0 },
    { 0x2f6e243e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa08e2440, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40e2444, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x98e2446, SUC_lexicon_tags_243 },
    { 0x232e2447, SUC_lexicon_tags_151 },
    { 0xc18e2447, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ee244c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefce2450, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10e2456, SUC_lexicon_tags_1 },
    { 0x1eae2457, SUC_lexicon_tags_43 },
    { 0xa5ce2456, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ce245e, SUC_lexicon_tags_37 },
    { 0xbae245e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed4e2461, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8e2465, SUC_lexicon_tags_31 },
    { 0xb04e2466, SUC_lexicon_tags_37 },
    { 0x6ae2467, SUC_lexicon_tags_12 },
    { 0x3cae2468, SUC_lexicon_tags_12 },
    { 0xa22e2468, SUC_lexicon_tags_30 },
    { 0xc60e246a, SUC_lexicon_tags_7 },
    { 0x74e2469, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdece246d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xebce2470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4e2473, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d8e2475, SUC_lexicon_tags_35 },
    { 0x5e6e2476, SUC_lexicon_tags_12 },
    { 0x864e2475, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dce2479, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414e247d, SUC_lexicon_tags_43 },
    { 0x688e247e, SUC_lexicon_tags_151 },
    { 0x5a2e247e, SUC_lexicon_tags_37 },
    { 0xec6e2480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddce2482, SUC_lexicon_tags_15 },
    { 0x890e2483, SUC_lexicon_tags_33 },
    { 0x3a6e2484, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46e2487, SUC_lexicon_tags_77 },
    { 0xe3ce2488, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71ce248a, SUC_lexicon_tags_27 },
    { 0xa3ae248b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ce248e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c6e2490, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x2f4e2492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x260e2494, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ce2498, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ae249b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8e24a9, SUC_lexicon_tags_25 },
    { 0x7bee24aa, SUC_lexicon_tags_30 },
    { 0xceae24ab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8e24ae, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x724e24b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4e24bb, SUC_lexicon_tags_17 },
    { 0x1e2e24bc, SUC_lexicon_tags_19 },
    { 0xece24bd, SUC_lexicon_tags_25 },
    { 0x2cce24bb, SUC_lexicon_tags_14 },
    { 0xfdee24bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ce24c2, SUC_lexicon_tags_3 },
    { 0xbd6e24c3, SUC_lexicon_tags_30 },
    { 0x9c8e24c2, SUC_lexicon_tags_66 },
    { 0x21ae24c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80ee24c7, SUC_lexicon_tags_5 },
    { 0xf0ae24c8, SUC_lexicon_tags_15 },
    { 0xd68e24c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5aae24cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ce24d4, SUC_lexicon_tags_2 },
    { 0x724e24d4, SUC_lexicon_tags_47 },
    { 0xe58e24d6, SUC_lexicon_tags_37 },
    { 0xbf8e24d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48e24db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9b8e24dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ae24e0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e24e3, SUC_lexicon_tags_15 },
    { 0x36e24e4, SUC_lexicon_tags_11 },
    { 0x384e24e3, SUC_lexicon_tags_21 },
    { 0x558e24e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ae24eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98e24f1, SUC_lexicon_tags_7 },
    { 0xd48e24f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0e24f7, SUC_lexicon_tags_55 },
    { 0x714e24f8, SUC_lexicon_tags_2 },
    { 0xab6e24f9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9d0e24fb, SUC_lexicon_tags_28 },
    { 0x84ee24fc, SUC_lexicon_tags_28 },
    { 0xcfae24fd, SUC_lexicon_tags_12 },
    { 0x544e24fd, SUC_lexicon_tags_14 },
    { 0x734e24ff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8b2e2501, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fee2507, SUC_lexicon_tags_27 },
    { 0xba0e2507, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x164e250b, SUC_lexicon_tags_25 },
    { 0x44ae250c, SUC_lexicon_tags_66 },
    { 0x42ee250c, SUC_lexicon_tags_19 },
    { 0x144e250e, SUC_lexicon_tags_1 },
    { 0xa00e250f, SUC_lexicon_tags_15 },
    { 0x4eee2510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x802e2512, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6e2519, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8e251d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50e2522, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x404e2524, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ae252e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e2533, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e4e2535, SUC_lexicon_tags_26 },
    { 0x35ee2535, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aee2539, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x95ae253b, SUC_lexicon_tags_3 },
    { 0x7d2e253c, SUC_lexicon_tags_12 },
    { 0x23ae253d, SUC_lexicon_tags_7 },
    { 0xcace253d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18e2541, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf0e2543, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbae2546, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e254c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe08e254f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28e2552, SUC_lexicon_tags_31 },
    { 0x182e2553, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570e2557, SUC_lexicon_tags_127 },
    { 0xdfae2557, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ce255d, SUC_lexicon_tags_47 },
    { 0xe48e255d, SUC_lexicon_tags_47 },
    { 0x6fee255f, SUC_lexicon_tags_20 },
    { 0x5c4e255f, SUC_lexicon_tags_63 },
    { 0x5d4e2560, SUC_lexicon_tags_16 },
    { 0xfa6e2562, SUC_lexicon_tags_2 },
    { 0xb82e2562, SUC_lexicon_tags_25 },
    { 0x926e2563, SUC_lexicon_tags_1 },
    { 0xcc4e2563, SUC_lexicon_tags_2 },
    { 0x266e255e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc18e2568, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dce256b, SUC_lexicon_tags_1 },
    { 0x162e256c, SUC_lexicon_tags_0 },
    { 0x8f2e256d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0e2571, SUC_lexicon_tags_45 },
    { 0xcd2e2572, SUC_lexicon_tags_6 },
    { 0x934e2572, SUC_lexicon_tags_51 },
    { 0x87ae2572, SUC_lexicon_tags_37 },
    { 0x4d4e2575, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x772e2578, SUC_lexicon_tags_1 },
    { 0x8e4e2578, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26e257c, SUC_lexicon_tags_0 },
    { 0x810e257d, SUC_lexicon_tags_56 },
    { 0x2ae257e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8e2586, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee2589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x858e258b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30e258e, SUC_lexicon_tags_12 },
    { 0xe6ce258f, SUC_lexicon_tags_45 },
    { 0x4e2590, SUC_lexicon_tags_11 },
    { 0x84ee2591, SUC_lexicon_tags_162 },
    { 0x778e2591, SUC_lexicon_tags_34 },
    { 0x48ce2592, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x282e2595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x460e2597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416e259d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd2e259f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x882e25a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0e25a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fae25ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51ee25ae, SUC_lexicon_tags_57 },
    { 0x16ce25af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ee25b6, SUC_lexicon_tags_38 },
    { 0x104e25b6, SUC_lexicon_tags_15 },
    { 0x92ee25b7, SUC_lexicon_tags_13 },
    { 0x89ae25b7, SUC_lexicon_tags_12 },
    { 0x17ae25ba, SUC_lexicon_tags_66 },
    { 0x270e25bb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa0e25bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe80e25bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6e25c3, SUC_lexicon_tags_3 },
    { 0x266e25c4, SUC_lexicon_tags_25 },
    { 0x70ee25c5, SUC_lexicon_tags_6 },
    { 0x112e25c5, SUC_lexicon_tags_1 },
    { 0xa44e25c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fce25ca, SUC_lexicon_tags_44 },
    { 0xce6e25cb, SUC_lexicon_tags_32 },
    { 0x72ce25cc, SUC_lexicon_tags_2 },
    { 0x1c4e25cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0e25d0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2b4e25d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x274e25d4, SUC_lexicon_tags_1 },
    { 0x298e25d5, SUC_lexicon_tags_60 },
    { 0x5f2e25d4, SUC_lexicon_tags_64 },
    { 0xbbee25d7, SUC_lexicon_tags_49 },
    { 0xbe4e25d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30e25db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34e25de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0e25e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2c8e25e6, SUC_lexicon_tags_34 },
    { 0x8e2e25e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77ce25e9, SUC_lexicon_tags_2 },
    { 0x570e25e9, SUC_lexicon_tags_47 },
    { 0x618e25ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe72e25ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x864e25ef, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x548e25f1, SUC_lexicon_tags_9 },
    { 0x7b4e25f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc06e25f4, SUC_lexicon_tags_4 },
    { 0xe44e25f5, SUC_lexicon_tags_38 },
    { 0x112e25f6, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cee25fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcae2602, SUC_lexicon_tags_17 },
    { 0x260e2603, SUC_lexicon_tags_45 },
    { 0x47ee2604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7ce2606, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc94e2608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298e260e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ce2610, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x2c6e2612, SUC_lexicon_tags_1 },
    { 0x26ae2612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98ce2615, SUC_lexicon_tags_25 },
    { 0xb04e2616, SUC_lexicon_tags_38 },
    { 0x1f8e2616, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346e261d, SUC_lexicon_tags_4 },
    { 0x2eae261d, SUC_lexicon_tags_2 },
    { 0x198e261f, SUC_lexicon_tags_17 },
    { 0xcece2620, SUC_lexicon_tags_33 },
    { 0x996e2621, SUC_lexicon_tags_25 },
    { 0x30ee2622, SUC_lexicon_tags_15 },
    { 0x16e2623, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ae2627, SUC_lexicon_tags_47 },
    { 0x376e2628, SUC_lexicon_tags_30 },
    { 0x91ae2629, SUC_lexicon_tags_0 },
    { 0x8dce262a, SUC_lexicon_tags_26 },
    { 0xf5ae262a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x330e262e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x488e2631, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eee2637, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4e263a, SUC_lexicon_tags_15 },
    { 0xdf0e263b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566e2641, SUC_lexicon_tags_42 },
    { 0xd9ce2642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe86e2644, SUC_lexicon_tags_6 },
    { 0x7e8e2645, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512e2649, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4e264c, SUC_lexicon_tags_31 },
    { 0xbc0e264d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350e2652, SUC_lexicon_tags_7 },
    { 0xc38e2653, SUC_lexicon_tags_16 },
    { 0xcb4e2654, SUC_lexicon_tags_32 },
    { 0x522e2652, SUC_lexicon_tags_17 },
    { 0x8c6e2654, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x464e2658, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x500e265a, SUC_lexicon_tags_7 },
    { 0xfbce265b, SUC_lexicon_tags_46 },
    { 0x7fee265a, SUC_lexicon_tags_46 },
    { 0x568e265d, SUC_lexicon_tags_63 },
    { 0x242e265e, SUC_lexicon_tags_11 },
    { 0x212e265d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xfe2e2661, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad4e2663, SUC_lexicon_tags_17 },
    { 0xd48e2664, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fae2666, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31ae2668, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc8e266a, SUC_lexicon_tags_19 },
    { 0x72ce266a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22e266d, SUC_lexicon_tags_65 },
    { 0xccae266d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0e267a, SUC_lexicon_tags_15 },
    { 0x626e267a, SUC_lexicon_tags_23 },
    { 0x7e0e267c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x66ae267e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x854e2681, SUC_lexicon_tags_7 },
    { 0xfa0e2681, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176e268c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x138e268e, SUC_lexicon_tags_30 },
    { 0x7a8e268e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x956e2691, SUC_lexicon_tags_12 },
    { 0xec8e2692, SUC_lexicon_tags_7 },
    { 0x80e2692, SUC_lexicon_tags_17 },
    { 0x27ce2694, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6e2696, SUC_lexicon_tags_12 },
    { 0xdb6e2697, SUC_lexicon_tags_26 },
    { 0xac2e2697, SUC_lexicon_tags_201 },
    { 0xd9ae2698, SUC_lexicon_tags_7 },
    { 0xa0ce2696, SUC_lexicon_tags_39 },
    { 0x5ee269b, SUC_lexicon_tags_17 },
    { 0x574e269c, SUC_lexicon_tags_0 },
    { 0x39ae269b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2e26a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0e26a4, SUC_lexicon_tags_25 },
    { 0xbe4e26a5, SUC_lexicon_tags_10 },
    { 0x664e26a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x478e26a8, SUC_lexicon_tags_12 },
    { 0xb34e26a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ce26ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634e26b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926e26ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbe6e26bc, SUC_lexicon_tags_160 },
    { 0xe16e26bd, SUC_lexicon_tags_25 },
    { 0x4cee26be, SUC_lexicon_tags_33 },
    { 0xd7ae26bf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8e26c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56e26c7, SUC_lexicon_tags_7 },
    { 0xdbae26c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x958e26ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ee26ce, SUC_lexicon_tags_1 },
    { 0xdcce26cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ee26d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbd2e26d4, SUC_lexicon_tags_19 },
    { 0xf74e26d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12ce26d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x856e26d9, SUC_lexicon_tags_7 },
    { 0x51ce26da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e8e26dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ee26e2, SUC_lexicon_tags_69 },
    { 0x280e26e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22e26e5, SUC_lexicon_tags_37 },
    { 0xa3ee26e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3d6e26e8, SUC_lexicon_tags_37 },
    { 0xb62e26e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62e26ec, SUC_lexicon_tags_11 },
    { 0xf40e26ed, SUC_lexicon_tags_855 },
    { 0, NULL },
    { 0x6cce26ef, SUC_lexicon_tags_17 },
    { 0xb3ce26f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa18e26f2, SUC_lexicon_tags_12 },
    { 0x128e26f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ee26f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x856e26f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6e26ff, SUC_lexicon_tags_37 },
    { 0xeaee26ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2e2705, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbae2708, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5a0e270a, SUC_lexicon_tags_230 },
    { 0x46ae270b, SUC_lexicon_tags_30 },
    { 0x206e270c, SUC_lexicon_tags_66 },
    { 0x182e270d, SUC_lexicon_tags_60 },
    { 0xda4e270e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5d0e2710, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21ae2712, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x762e2715, SUC_lexicon_tags_151 },
    { 0x58ee2716, SUC_lexicon_tags_10 },
    { 0x1b4e2716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf26e2719, SUC_lexicon_tags_31 },
    { 0xacee271a, SUC_lexicon_tags_49 },
    { 0x48ee271b, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0x366e271d, SUC_lexicon_tags_11 },
    { 0x840e271d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8e2721, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11ce2723, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf46e2725, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedae272b, SUC_lexicon_tags_1 },
    { 0xfcae272c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0e2731, SUC_lexicon_tags_25 },
    { 0x268e2732, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e2738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfeae273a, SUC_lexicon_tags_88 },
    { 0x88ce273b, SUC_lexicon_tags_35 },
    { 0xf0ae273b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ce2741, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed8e2743, SUC_lexicon_tags_35 },
    { 0xb0ee2743, SUC_lexicon_tags_8 },
    { 0xa7ae2745, SUC_lexicon_tags_6 },
    { 0xdc4e2746, SUC_lexicon_tags_25 },
    { 0x84e2747, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x944e274a, SUC_lexicon_tags_158 },
    { 0x824e274b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdaee274d, SUC_lexicon_tags_46 },
    { 0x700e274e, SUC_lexicon_tags_19 },
    { 0xa88e274f, SUC_lexicon_tags_1 },
    { 0x37ae274d, SUC_lexicon_tags_37 },
    { 0x448e2751, SUC_lexicon_tags_12 },
    { 0x758e2752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fae2755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fce2757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x292e2759, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e275c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0ae275e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49ae2760, SUC_lexicon_tags_2 },
    { 0xc76e2760, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bee2765, SUC_lexicon_tags_33 },
    { 0xb32e2765, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x508e2769, SUC_lexicon_tags_25 },
    { 0xcace276a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce276d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd26e276f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2e2774, SUC_lexicon_tags_25 },
    { 0x86ce2774, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb18e2777, SUC_lexicon_tags_11 },
    { 0xf66e2777, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ee277c, SUC_lexicon_tags_1 },
    { 0xe30e277c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ee2787, SUC_lexicon_tags_1 },
    { 0xef0e2788, SUC_lexicon_tags_25 },
    { 0x9f4e2789, SUC_lexicon_tags_11 },
    { 0xc88e2789, SUC_lexicon_tags_17 },
    { 0xb1ce278b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30ce278d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eee2790, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72ce2792, SUC_lexicon_tags_902 },
    { 0x4e0e2793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfae2796, SUC_lexicon_tags_22 },
    { 0xaf6e2797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x676e2799, SUC_lexicon_tags_11 },
    { 0xa68e279a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ae27a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220e27a8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40e27ae, SUC_lexicon_tags_6 },
    { 0x330e27af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6e27b2, SUC_lexicon_tags_0 },
    { 0xc90e27b2, SUC_lexicon_tags_47 },
    { 0x902e27b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeee27b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82e27bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ee27be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2e27c2, SUC_lexicon_tags_23 },
    { 0x9fee27c2, SUC_lexicon_tags_9 },
    { 0xac2e27c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558e27c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x44e27cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8e27ce, SUC_lexicon_tags_37 },
    { 0xa90e27ce, SUC_lexicon_tags_12 },
    { 0x2ace27cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0e27d4, SUC_lexicon_tags_14 },
    { 0xfa0e27d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee27d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f2e27db, SUC_lexicon_tags_30 },
    { 0x1bce27db, SUC_lexicon_tags_0 },
    { 0x86ce27dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeae27e3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16e27e6, SUC_lexicon_tags_12 },
    { 0xb2e27e7, SUC_lexicon_tags_65 },
    { 0xf12e27e8, SUC_lexicon_tags_12 },
    { 0x660e27e6, SUC_lexicon_tags_15 },
    { 0x886e27ea, SUC_lexicon_tags_23 },
    { 0xca2e27ea, SUC_lexicon_tags_7 },
    { 0x9c8e27eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ce27f1, SUC_lexicon_tags_33 },
    { 0x2dae27f1, SUC_lexicon_tags_5 },
    { 0xf54e27f3, SUC_lexicon_tags_2 },
    { 0xb88e27f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xefee27f6, SUC_lexicon_tags_25 },
    { 0x1b4e27f7, SUC_lexicon_tags_15 },
    { 0x8e2e27f7, SUC_lexicon_tags_13 },
    { 0x3ae27f8, SUC_lexicon_tags_1 },
    { 0x32e27fa, SUC_lexicon_tags_17 },
    { 0xfa4e27fb, SUC_lexicon_tags_62 },
    { 0x516e27fb, SUC_lexicon_tags_33 },
    { 0xa56e27f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa90e27ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404e2804, SUC_lexicon_tags_25 },
    { 0xcc2e2805, SUC_lexicon_tags_25 },
    { 0xae2804, SUC_lexicon_tags_65 },
    { 0xd50e2807, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x238e2809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4e280f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10e2812, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bce2816, SUC_lexicon_tags_11 },
    { 0x802e2817, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e281a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ae281f, SUC_lexicon_tags_7 },
    { 0x3c4e2820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ce2823, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ae282a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3fae282c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162e2831, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ce2836, SUC_lexicon_tags_31 },
    { 0xf4ee2837, SUC_lexicon_tags_4 },
    { 0x8ae2838, SUC_lexicon_tags_27 },
    { 0xa02e2839, SUC_lexicon_tags_19 },
    { 0x990e2838, SUC_lexicon_tags_19 },
    { 0x1f8e2838, SUC_lexicon_tags_15 },
    { 0x348e283c, SUC_lexicon_tags_4 },
    { 0x50ee283b, SUC_lexicon_tags_1 },
    { 0xcf0e283e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c4e2840, SUC_lexicon_tags_29 },
    { 0x36ae2841, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ce2843, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfae2845, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3bae2847, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd88e2849, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ce284d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x452e284f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cce2852, SUC_lexicon_tags_14 },
    { 0xc0ee2853, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4e285b, SUC_lexicon_tags_4 },
    { 0xa82e285b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x166e285e, SUC_lexicon_tags_7 },
    { 0xbe2e285f, SUC_lexicon_tags_47 },
    { 0x518e2860, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8e2867, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x36e2869, SUC_lexicon_tags_2 },
    { 0xafee2869, SUC_lexicon_tags_25 },
    { 0xc8ae286b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592e2873, SUC_lexicon_tags_44 },
    { 0x1e2e2873, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42e2877, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbce287a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc82e287c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902e2883, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0e2886, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ae288d, SUC_lexicon_tags_38 },
    { 0xa2ae288d, SUC_lexicon_tags_37 },
    { 0xe1ce288d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86e2896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ae289b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80e289e, SUC_lexicon_tags_19 },
    { 0x82e289f, SUC_lexicon_tags_6 },
    { 0x7e6e28a0, SUC_lexicon_tags_8 },
    { 0x2d6e28a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3b4e28a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee28a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30e28aa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6ece28ac, SUC_lexicon_tags_46 },
    { 0x496e28ad, SUC_lexicon_tags_17 },
    { 0x606e28ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ce28b1, SUC_lexicon_tags_17 },
    { 0xb18e28b2, SUC_lexicon_tags_16 },
    { 0x576e28b3, SUC_lexicon_tags_61 },
    { 0x5c8e28b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x866e28b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f2e28b9, SUC_lexicon_tags_17 },
    { 0xc6ae28ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81ae28bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6e28bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396e28c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x554e28c7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceae28d2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ce28d7, SUC_lexicon_tags_17 },
    { 0xeb8e28d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6e28db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64ae28dd, SUC_lexicon_tags_47 },
    { 0x78e28de, SUC_lexicon_tags_9 },
    { 0x8ce28de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb4e28e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898e28ea, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0e28ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ce28f6, SUC_lexicon_tags_33 },
    { 0x30ae28f6, SUC_lexicon_tags_23 },
    { 0x694e28f8, SUC_lexicon_tags_34 },
    { 0x9a0e28f9, SUC_lexicon_tags_6 },
    { 0x714e28f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bee28fd, SUC_lexicon_tags_14 },
    { 0x8ae28fe, SUC_lexicon_tags_15 },
    { 0xf6ee28fe, SUC_lexicon_tags_60 },
    { 0x524e28fe, SUC_lexicon_tags_37 },
    { 0x22e28fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0e2904, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e0e2906, SUC_lexicon_tags_27 },
    { 0xd94e2906, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd0e2909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64e290d, SUC_lexicon_tags_25 },
    { 0xe4ae290e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbae2910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff4e2912, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc82e2914, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb10e2916, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542e291c, SUC_lexicon_tags_61 },
    { 0x80ce291d, SUC_lexicon_tags_1 },
    { 0x84ce291d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ce2921, SUC_lexicon_tags_7 },
    { 0xc28e2921, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48e2926, SUC_lexicon_tags_17 },
    { 0x734e2927, SUC_lexicon_tags_15 },
    { 0x704e2927, SUC_lexicon_tags_172 },
    { 0x3ece2929, SUC_lexicon_tags_32 },
    { 0x24ce292a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe86e292c, SUC_lexicon_tags_14 },
    { 0x8bee292d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84e2933, SUC_lexicon_tags_21 },
    { 0x4f0e2934, SUC_lexicon_tags_193 },
    { 0x896e2934, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x58ae2937, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696e293c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ce2940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760e294a, SUC_lexicon_tags_38 },
    { 0x778e294a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce2950, SUC_lexicon_tags_46 },
    { 0x9d4e2950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812e2958, SUC_lexicon_tags_136 },
    { 0xe02e2958, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x220e295b, SUC_lexicon_tags_300 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110e295f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ee2962, SUC_lexicon_tags_10 },
    { 0xac2e2963, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8e2966, SUC_lexicon_tags_25 },
    { 0x13ce2966, SUC_lexicon_tags_49 },
    { 0x416e2968, SUC_lexicon_tags_23 },
    { 0x792e2969, SUC_lexicon_tags_7 },
    { 0x816e296a, SUC_lexicon_tags_1 },
    { 0x34ae296b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8e296e, SUC_lexicon_tags_25 },
    { 0x21ee296f, SUC_lexicon_tags_77 },
    { 0x604e296f, SUC_lexicon_tags_39 },
    { 0x1d2e296f, SUC_lexicon_tags_15 },
    { 0xf84e2972, SUC_lexicon_tags_60 },
    { 0xa4e296e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86e2977, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2e297a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x778e297c, SUC_lexicon_tags_25 },
    { 0x262e297d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x144e297f, SUC_lexicon_tags_19 },
    { 0xef8e297f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ae2986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2e298a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec6e298d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ee2992, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ae2995, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88e2997, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a8e2999, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6e299e, SUC_lexicon_tags_23 },
    { 0xc36e299f, SUC_lexicon_tags_11 },
    { 0xf96e29a0, SUC_lexicon_tags_11 },
    { 0xb7ee29a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2fae29a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x660e29a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ce29aa, SUC_lexicon_tags_19 },
    { 0xa9ce29ab, SUC_lexicon_tags_11 },
    { 0xadce29ac, SUC_lexicon_tags_37 },
    { 0x6f8e29ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f0e29af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff4e29b1, SUC_lexicon_tags_37 },
    { 0x274e29b1, SUC_lexicon_tags_11 },
    { 0xfd2e29b1, SUC_lexicon_tags_17 },
    { 0x8a8e29b4, SUC_lexicon_tags_37 },
    { 0x564e29b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc56e29b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5e4e29b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12e29bd, SUC_lexicon_tags_22 },
    { 0xe0ae29be, SUC_lexicon_tags_12 },
    { 0xece29bd, SUC_lexicon_tags_7 },
    { 0x172e29c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0e29c3, SUC_lexicon_tags_17 },
    { 0x77ee29c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x970e29c7, SUC_lexicon_tags_37 },
    { 0x274e29c8, SUC_lexicon_tags_23 },
    { 0xd5ee29c8, SUC_lexicon_tags_17 },
    { 0x8a6e29c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x574e29cc, SUC_lexicon_tags_37 },
    { 0xecae29cd, SUC_lexicon_tags_0 },
    { 0xa96e29ce, SUC_lexicon_tags_65 },
    { 0xb10e29ce, SUC_lexicon_tags_30 },
    { 0xc10e29d0, SUC_lexicon_tags_2 },
    { 0x2e4e29d1, SUC_lexicon_tags_57 },
    { 0xc42e29d1, SUC_lexicon_tags_160 },
    { 0xcbee29d3, SUC_lexicon_tags_3 },
    { 0x3cee29d3, SUC_lexicon_tags_1 },
    { 0x468e29d4, SUC_lexicon_tags_14 },
    { 0x352e29d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8e29d9, SUC_lexicon_tags_1 },
    { 0xb06e29d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce29de, SUC_lexicon_tags_26 },
    { 0xe0ce29de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8cce29e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46e29e6, SUC_lexicon_tags_23 },
    { 0x9ece29e7, SUC_lexicon_tags_19 },
    { 0x950e29e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0e29ed, SUC_lexicon_tags_12 },
    { 0xd96e29ee, SUC_lexicon_tags_37 },
    { 0xad2e29ee, SUC_lexicon_tags_37 },
    { 0xb7ee29f0, SUC_lexicon_tags_5 },
    { 0xb86e29f1, SUC_lexicon_tags_6 },
    { 0x3d4e29f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ce29f6, SUC_lexicon_tags_15 },
    { 0xc3ce29f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x184e29f9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9b6e29fb, SUC_lexicon_tags_23 },
    { 0x7f4e29fb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x862e29ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce2a05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4e2a0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e2a0f, SUC_lexicon_tags_39 },
    { 0x540e2a0f, SUC_lexicon_tags_45 },
    { 0x442e2a10, SUC_lexicon_tags_12 },
    { 0x4e0e2a12, SUC_lexicon_tags_15 },
    { 0xe02e2a10, SUC_lexicon_tags_47 },
    { 0xa44e2a0f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182e2a18, SUC_lexicon_tags_387 },
    { 0x92ce2a19, SUC_lexicon_tags_37 },
    { 0xd16e2a1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ae2a1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1ce2a20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb4e2a22, SUC_lexicon_tags_17 },
    { 0xbeee2a22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d6e2a25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a0e2a27, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x234e2a29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62e2a2b, SUC_lexicon_tags_4 },
    { 0x74ee2a2b, SUC_lexicon_tags_1 },
    { 0x44e2a2b, SUC_lexicon_tags_22 },
    { 0x7ae2a2c, SUC_lexicon_tags_37 },
    { 0xc04e2a2f, SUC_lexicon_tags_38 },
    { 0xf66e2a2d, SUC_lexicon_tags_6 },
    { 0x89ce2a31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ce2a35, SUC_lexicon_tags_1 },
    { 0x26e2a35, SUC_lexicon_tags_32 },
    { 0x2d8e2a37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0ae2a39, SUC_lexicon_tags_11 },
    { 0x472e2a3a, SUC_lexicon_tags_32 },
    { 0x7d8e2a3b, SUC_lexicon_tags_2 },
    { 0xdd8e2a3a, SUC_lexicon_tags_7 },
    { 0x2b0e2a3d, SUC_lexicon_tags_17 },
    { 0xad2e2a39, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4a8e2a40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ae2a43, SUC_lexicon_tags_30 },
    { 0xed8e2a43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x558e2a47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e2a4a, SUC_lexicon_tags_65 },
    { 0xe70e2a4b, SUC_lexicon_tags_14 },
    { 0x648e2a4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e2a54, SUC_lexicon_tags_12 },
    { 0x99ae2a55, SUC_lexicon_tags_17 },
    { 0xb7ce2a56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ee2a59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d4e2a5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872e2a60, SUC_lexicon_tags_5 },
    { 0x690e2a60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698e2a68, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf98e2a6a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6e2a71, SUC_lexicon_tags_11 },
    { 0x1e2e2a71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51ee2a74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce2a7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x674e2a7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954e2a81, SUC_lexicon_tags_37 },
    { 0x7a4e2a82, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26e2a85, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ce2a8c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a8e2a8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2e2a94, SUC_lexicon_tags_51 },
    { 0x894e2a94, SUC_lexicon_tags_37 },
    { 0x9dee2a96, SUC_lexicon_tags_47 },
    { 0xb44e2a97, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2e2a9c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb68e2a9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534e2aa2, SUC_lexicon_tags_15 },
    { 0x2c0e2aa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830e2aa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ae2aaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dce2ab2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0e2ab5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2e2ab7, SUC_lexicon_tags_40 },
    { 0x136e2ab7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8e2abd, SUC_lexicon_tags_14 },
    { 0x18e2abd, SUC_lexicon_tags_9 },
    { 0x468e2abd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ae2ac2, SUC_lexicon_tags_17 },
    { 0x3fae2ac3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x512e2ac5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x874e2ac8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50e2acd, SUC_lexicon_tags_5 },
    { 0x82ee2acd, SUC_lexicon_tags_25 },
    { 0xb74e2acf, SUC_lexicon_tags_2 },
    { 0xa3ae2ad0, SUC_lexicon_tags_11 },
    { 0xf82e2ad1, SUC_lexicon_tags_82 },
    { 0xcf6e2ad1, SUC_lexicon_tags_0 },
    { 0x37ae2ad0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480e2ad8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ee2add, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52e2ae3, SUC_lexicon_tags_32 },
    { 0xbeae2ae4, SUC_lexicon_tags_15 },
    { 0x708e2ae5, SUC_lexicon_tags_3 },
    { 0x2e2ae3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ae2aee, SUC_lexicon_tags_26 },
    { 0x446e2aee, SUC_lexicon_tags_1 },
    { 0x1cce2aef, SUC_lexicon_tags_705 },
    { 0x9f0e2aef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84ae2af3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0ae2af5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x172e2af8, SUC_lexicon_tags_37 },
    { 0x19ce2af9, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ece2afc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x864e2aff, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x346e2b02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42ae2b04, SUC_lexicon_tags_7 },
    { 0xdf4e2b05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae2b09, SUC_lexicon_tags_25 },
    { 0x42e2b09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776e2b11, SUC_lexicon_tags_26 },
    { 0x74e2b12, SUC_lexicon_tags_11 },
    { 0x19ce2b13, SUC_lexicon_tags_37 },
    { 0x83ce2b14, SUC_lexicon_tags_46 },
    { 0x5d8e2b15, SUC_lexicon_tags_37 },
    { 0x9dae2b16, SUC_lexicon_tags_37 },
    { 0x77ae2b17, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0x28ce2b19, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2e2b20, SUC_lexicon_tags_45 },
    { 0x20ae2b20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ae2b26, SUC_lexicon_tags_38 },
    { 0x3bee2b26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbae2b2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4e2b33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc06e2b35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14e2b39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x42ae2b3b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582e2b3f, SUC_lexicon_tags_15 },
    { 0x770e2b40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68ee2b42, SUC_lexicon_tags_12 },
    { 0xcae2b43, SUC_lexicon_tags_38 },
    { 0xe08e2b42, SUC_lexicon_tags_87 },
    { 0xabce2b43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce2b48, SUC_lexicon_tags_0 },
    { 0xb5ce2b49, SUC_lexicon_tags_47 },
    { 0xa26e2b48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96e2b4d, SUC_lexicon_tags_140 },
    { 0xa6ae2b4e, SUC_lexicon_tags_33 },
    { 0xd3ee2b4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04e2b53, SUC_lexicon_tags_87 },
    { 0xbf6e2b54, SUC_lexicon_tags_37 },
    { 0x128e2b53, SUC_lexicon_tags_63 },
    { 0xc6ce2b56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6e2b5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cae2b5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712e2b69, SUC_lexicon_tags_11 },
    { 0x4a2e2b6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbae2b6c, SUC_lexicon_tags_10 },
    { 0xac8e2b6c, SUC_lexicon_tags_46 },
    { 0x3eae2b6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x924e2b71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8e2b74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e2b7a, SUC_lexicon_tags_3 },
    { 0xcfae2b7b, SUC_lexicon_tags_6 },
    { 0x322e2b7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8e2b80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf36e2b82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544e2b89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaee2b8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ae2b91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12e2b96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4e2b9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee2b9e, SUC_lexicon_tags_12 },
    { 0xacee2b9f, SUC_lexicon_tags_14 },
    { 0xa1ce2ba0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8e2ba4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8e2ba7, SUC_lexicon_tags_11 },
    { 0x952e2ba8, SUC_lexicon_tags_42 },
    { 0x4c0e2ba9, SUC_lexicon_tags_25 },
    { 0x26ce2baa, SUC_lexicon_tags_7 },
    { 0x15ce2bab, SUC_lexicon_tags_15 },
    { 0x880e2bac, SUC_lexicon_tags_27 },
    { 0x350e2bad, SUC_lexicon_tags_11 },
    { 0xa60e2bad, SUC_lexicon_tags_725 },
    { 0xa52e2bad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8e2bb2, SUC_lexicon_tags_25 },
    { 0xa82e2bb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116e2bb8, SUC_lexicon_tags_37 },
    { 0x646e2bb9, SUC_lexicon_tags_25 },
    { 0x26ae2bba, SUC_lexicon_tags_13 },
    { 0x22ee2bbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28e2bc4, SUC_lexicon_tags_66 },
    { 0x5ce2bc5, SUC_lexicon_tags_68 },
    { 0x812e2bc6, SUC_lexicon_tags_28 },
    { 0x4fce2bc7, SUC_lexicon_tags_6 },
    { 0x51ae2bc7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3e4e2bca, SUC_lexicon_tags_25 },
    { 0xf0ee2bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178e2bcf, SUC_lexicon_tags_23 },
    { 0xb84e2bd0, SUC_lexicon_tags_10 },
    { 0x152e2bcf, SUC_lexicon_tags_17 },
    { 0x886e2bd2, SUC_lexicon_tags_37 },
    { 0x18ae2bd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ae2bd8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24ce2bda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2e2be0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffae2be5, SUC_lexicon_tags_25 },
    { 0x2dee2be5, SUC_lexicon_tags_60 },
    { 0xd04e2be7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bee2bea, SUC_lexicon_tags_37 },
    { 0xf3ce2beb, SUC_lexicon_tags_15 },
    { 0xda8e2bec, SUC_lexicon_tags_2 },
    { 0x18e2bed, SUC_lexicon_tags_15 },
    { 0xb56e2beb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x85ee2bf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86ce2bf2, SUC_lexicon_tags_7 },
    { 0x61ee2bf3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x986e2bf5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94e2bf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96e2bfb, SUC_lexicon_tags_37 },
    { 0xdd6e2bfc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98e2c08, SUC_lexicon_tags_11 },
    { 0x54ae2c09, SUC_lexicon_tags_14 },
    { 0xbe4e2c09, SUC_lexicon_tags_2 },
    { 0xbe4e2c0b, SUC_lexicon_tags_13 },
    { 0x53ce2c0b, SUC_lexicon_tags_11 },
    { 0xf1ce2c0d, SUC_lexicon_tags_43 },
    { 0x820e2c0e, SUC_lexicon_tags_15 },
    { 0xec6e2c0e, SUC_lexicon_tags_66 },
    { 0x43ae2c10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaace2c12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0e2c15, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eae2c19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x424e2c1b, SUC_lexicon_tags_118 },
    { 0xfd8e2c1c, SUC_lexicon_tags_1 },
    { 0xaa4e2c1d, SUC_lexicon_tags_28 },
    { 0x954e2c1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ae2c29, SUC_lexicon_tags_28 },
    { 0x23ce2c2a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd90e2c2c, SUC_lexicon_tags_61 },
    { 0xd7ce2c2d, SUC_lexicon_tags_11 },
    { 0x36e2c2e, SUC_lexicon_tags_12 },
    { 0x520e2c2f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862e2c3a, SUC_lexicon_tags_37 },
    { 0x38e2c3b, SUC_lexicon_tags_103 },
    { 0x97ae2c3c, SUC_lexicon_tags_0 },
    { 0x3bce2c3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2e2c45, SUC_lexicon_tags_11 },
    { 0xbb6e2c46, SUC_lexicon_tags_11 },
    { 0xa8ae2c45, SUC_lexicon_tags_37 },
    { 0x4b6e2c46, SUC_lexicon_tags_37 },
    { 0x8aae2c49, SUC_lexicon_tags_0 },
    { 0xf6ce2c45, SUC_lexicon_tags_17 },
    { 0xfd2e2c4b, SUC_lexicon_tags_2 },
    { 0xd24e2c4c, SUC_lexicon_tags_2 },
    { 0x782e2c45, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x264e2c4f, SUC_lexicon_tags_29 },
    { 0x288e2c50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaae2c53, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x984e2c56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02e2c62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ee2c67, SUC_lexicon_tags_17 },
    { 0x932e2c67, SUC_lexicon_tags_14 },
    { 0xf04e2c69, SUC_lexicon_tags_17 },
    { 0xb0ce2c69, SUC_lexicon_tags_7 },
    { 0xcee2c6b, SUC_lexicon_tags_27 },
    { 0x6cce2c6c, SUC_lexicon_tags_25 },
    { 0x674e2c68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x528e2c70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34e2c75, SUC_lexicon_tags_392 },
    { 0x664e2c76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cce2c78, SUC_lexicon_tags_37 },
    { 0x1aee2c79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x596e2c7c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4e2c7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x912e2c81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe2ce2c83, SUC_lexicon_tags_7 },
    { 0x5ece2c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432e2c8b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ee2c8e, SUC_lexicon_tags_2 },
    { 0x7bee2c8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ece2c92, SUC_lexicon_tags_33 },
    { 0x4dae2c92, SUC_lexicon_tags_33 },
    { 0x470e2c94, SUC_lexicon_tags_26 },
    { 0xd48e2c94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56e2c9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15ee2ca1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12e2ca3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x600e2ca6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x61ae2ca8, SUC_lexicon_tags_1 },
    { 0xae0e2ca8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2cb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eae2cb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x916e2cb7, SUC_lexicon_tags_63 },
    { 0x40e2cb8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xac4e2cba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ee2cbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ce2cc2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08e2ccb, SUC_lexicon_tags_28 },
    { 0x12e2ccb, SUC_lexicon_tags_31 },
    { 0xe48e2ccd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4ae2ccf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344e2cd3, SUC_lexicon_tags_45 },
    { 0x7c4e2cd4, SUC_lexicon_tags_2 },
    { 0x7dae2cd5, SUC_lexicon_tags_103 },
    { 0x3f8e2cd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12e2cdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2eee2cdd, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8e2ce1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x214e2ce3, SUC_lexicon_tags_179 },
    { 0x744e2ce4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82e2ce7, SUC_lexicon_tags_60 },
    { 0xa6e2ce8, SUC_lexicon_tags_23 },
    { 0xd4ce2ce9, SUC_lexicon_tags_23 },
    { 0xefae2cea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20e2cf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234e2cfa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x716e2cfc, SUC_lexicon_tags_25 },
    { 0xc48e2cfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x984e2cff, SUC_lexicon_tags_60 },
    { 0x1f6e2d00, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcce2d03, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486e2d08, SUC_lexicon_tags_25 },
    { 0x46e2d08, SUC_lexicon_tags_8 },
    { 0x5bae2d0a, SUC_lexicon_tags_30 },
    { 0xd1ce2d08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ce2d0e, SUC_lexicon_tags_26 },
    { 0x328e2d0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbc0e2d11, SUC_lexicon_tags_15 },
    { 0x9f2e2d12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce2d16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71ae2d18, SUC_lexicon_tags_11 },
    { 0x1a2e2d18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd3ae2d1b, SUC_lexicon_tags_15 },
    { 0x68ee2d1c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eee2d23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4e2d27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2e2d2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5c6e2d2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28e2d31, SUC_lexicon_tags_11 },
    { 0xde0e2d31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7ce2d34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66e2d37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x168e2d39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ee2d3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x574e2d40, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ee2d44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x536e2d47, SUC_lexicon_tags_0 },
    { 0x4eae2d48, SUC_lexicon_tags_31 },
    { 0xbbae2d49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbae2d4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ee2d51, SUC_lexicon_tags_43 },
    { 0x36ee2d51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ee2d5c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ae2d5f, SUC_lexicon_tags_16 },
    { 0x538e2d60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82e2d64, SUC_lexicon_tags_17 },
    { 0x14ce2d65, SUC_lexicon_tags_17 },
    { 0xe58e2d66, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x312e2d68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ae2d6b, SUC_lexicon_tags_124 },
    { 0xbdee2d6c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca0e2d70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462e2d76, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e2d79, SUC_lexicon_tags_9 },
    { 0x4cee2d79, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x74ee2d7c, SUC_lexicon_tags_45 },
    { 0xc80e2d7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92e2d80, SUC_lexicon_tags_11 },
    { 0xdce2d81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3ee2d83, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796e2d89, SUC_lexicon_tags_3 },
    { 0xd2ae2d8a, SUC_lexicon_tags_15 },
    { 0x126e2d8a, SUC_lexicon_tags_2 },
    { 0xa00e2d8c, SUC_lexicon_tags_7 },
    { 0x610e2d8d, SUC_lexicon_tags_17 },
    { 0xdb0e2d8d, SUC_lexicon_tags_4 },
    { 0x43ce2d8f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dee2d93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ee2d96, SUC_lexicon_tags_2 },
    { 0xe5ee2d96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ce2d9a, SUC_lexicon_tags_60 },
    { 0x648e2d9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x486e2d9e, SUC_lexicon_tags_2 },
    { 0x492e2d9e, SUC_lexicon_tags_68 },
    { 0x702e2d9e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ce2da5, SUC_lexicon_tags_33 },
    { 0xa1ae2da6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98e2da9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ae2dad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860e2db1, SUC_lexicon_tags_37 },
    { 0x2bce2db2, SUC_lexicon_tags_11 },
    { 0x18e2db3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0e2db6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7ee2db8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x194e2dba, SUC_lexicon_tags_33 },
    { 0x72e2dba, SUC_lexicon_tags_8 },
    { 0xa26e2dba, SUC_lexicon_tags_14 },
    { 0xc0ce2dbd, SUC_lexicon_tags_7 },
    { 0xea0e2dbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dae2dc1, SUC_lexicon_tags_29 },
    { 0x49ce2dc2, SUC_lexicon_tags_0 },
    { 0x36ae2dc3, SUC_lexicon_tags_63 },
    { 0x64ce2dc4, SUC_lexicon_tags_9 },
    { 0x12ce2dc5, SUC_lexicon_tags_11 },
    { 0x674e2dc3, SUC_lexicon_tags_31 },
    { 0xefce2dc6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x706e2dca, SUC_lexicon_tags_25 },
    { 0x2ee2dcb, SUC_lexicon_tags_23 },
    { 0x7b4e2dcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8e2dcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x940e2dd2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ce2dd5, SUC_lexicon_tags_7 },
    { 0x536e2dd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x418e2dd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808e2ddd, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ce2de6, SUC_lexicon_tags_7 },
    { 0xadee2de6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ae2deb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bce2dee, SUC_lexicon_tags_14 },
    { 0xab4e2dee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bae2df4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4ae2df6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4e2dfb, SUC_lexicon_tags_17 },
    { 0xc62e2dfc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ee2e00, SUC_lexicon_tags_33 },
    { 0xcfee2e01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xface2e0b, SUC_lexicon_tags_49 },
    { 0x1aee2e0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32e2e12, SUC_lexicon_tags_15 },
    { 0xa10e2e13, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2f4e2e15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884e2e1a, SUC_lexicon_tags_47 },
    { 0x94ce2e1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xca2e2e1d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8e2e1f, SUC_lexicon_tags_35 },
    { 0xc78e2e20, SUC_lexicon_tags_1251 },
    { 0, NULL },
    { 0xd9ce2e22, SUC_lexicon_tags_1 },
    { 0x34ae2e22, SUC_lexicon_tags_38 },
    { 0x624e2e24, SUC_lexicon_tags_37 },
    { 0x474e2e23, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ae2e28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f4e2e2a, SUC_lexicon_tags_9 },
    { 0x3bae2e2b, SUC_lexicon_tags_55 },
    { 0x8bae2e2a, SUC_lexicon_tags_39 },
    { 0x54ce2e2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0e2e30, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xceee2e32, SUC_lexicon_tags_10 },
    { 0x932e2e32, SUC_lexicon_tags_16 },
    { 0x716e2e34, SUC_lexicon_tags_27 },
    { 0x800e2e35, SUC_lexicon_tags_17 },
    { 0xfbae2e36, SUC_lexicon_tags_25 },
    { 0xcfae2e37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x434e2e3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6e2e3d, SUC_lexicon_tags_21 },
    { 0xe28e2e3e, SUC_lexicon_tags_37 },
    { 0x54e2e3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ce2e42, SUC_lexicon_tags_2 },
    { 0x748e2e43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2e2e46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdce2e4a, SUC_lexicon_tags_55 },
    { 0x648e2e4b, SUC_lexicon_tags_12 },
    { 0x16ee2e4b, SUC_lexicon_tags_1 },
    { 0xc30e2e4d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74e2e52, SUC_lexicon_tags_57 },
    { 0xb6ae2e53, SUC_lexicon_tags_11 },
    { 0xbace2e54, SUC_lexicon_tags_32 },
    { 0x668e2e55, SUC_lexicon_tags_17 },
    { 0x7c6e2e55, SUC_lexicon_tags_12 },
    { 0x656e2e57, SUC_lexicon_tags_37 },
    { 0xfa4e2e58, SUC_lexicon_tags_3 },
    { 0xc5ae2e59, SUC_lexicon_tags_25 },
    { 0xcb0e2e54, SUC_lexicon_tags_17 },
    { 0xcdce2e57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260e2e5f, SUC_lexicon_tags_60 },
    { 0xab4e2e60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4e2e63, SUC_lexicon_tags_17 },
    { 0xd1ce2e63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7aee2e66, SUC_lexicon_tags_14 },
    { 0x82e2e67, SUC_lexicon_tags_63 },
    { 0x2e6e2e66, SUC_lexicon_tags_43 },
    { 0xface2e69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x204e2e6b, SUC_lexicon_tags_88 },
    { 0x9bae2e6c, SUC_lexicon_tags_1 },
    { 0xa64e2e6d, SUC_lexicon_tags_11 },
    { 0x3f2e2e6d, SUC_lexicon_tags_17 },
    { 0xd42e2e6f, SUC_lexicon_tags_7 },
    { 0x12e2e70, SUC_lexicon_tags_34 },
    { 0x700e2e6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef0e2e73, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fce2e76, SUC_lexicon_tags_38 },
    { 0x360e2e76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd90e2e7a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x26ae2e7c, SUC_lexicon_tags_27 },
    { 0x2d8e2e7d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0e2e80, SUC_lexicon_tags_37 },
    { 0x4ae2e81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412e2e92, SUC_lexicon_tags_30 },
    { 0xe9ce2e93, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x270e2e96, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126e2e9f, SUC_lexicon_tags_47 },
    { 0x2c8e2ea0, SUC_lexicon_tags_19 },
    { 0x63ee2e9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x964e2ea4, SUC_lexicon_tags_14 },
    { 0x480e2ea5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e4e2ea7, SUC_lexicon_tags_15 },
    { 0x7eae2ea8, SUC_lexicon_tags_99 },
    { 0xcbee2ea9, SUC_lexicon_tags_2 },
    { 0x8b8e2ea8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeee2eae, SUC_lexicon_tags_17 },
    { 0x3b8e2eae, SUC_lexicon_tags_68 },
    { 0x110e2eaf, SUC_lexicon_tags_30 },
    { 0xe8ce2eb1, SUC_lexicon_tags_17 },
    { 0x8dce2eb2, SUC_lexicon_tags_2 },
    { 0x6e4e2eb2, SUC_lexicon_tags_8 },
    { 0x332e2eb4, SUC_lexicon_tags_19 },
    { 0x120e2eb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde0e2eb7, SUC_lexicon_tags_87 },
    { 0xe44e2eb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24e2ec0, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432e2ec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10e2ec7, SUC_lexicon_tags_522 },
    { 0x1c8e2ec8, SUC_lexicon_tags_22 },
    { 0x2b4e2ec7, SUC_lexicon_tags_60 },
    { 0xdbae2eca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dce2ece, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732e2ed2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ee2ed6, SUC_lexicon_tags_10 },
    { 0xd4ee2ed6, SUC_lexicon_tags_30 },
    { 0x38ae2ed8, SUC_lexicon_tags_28 },
    { 0x4dee2ed8, SUC_lexicon_tags_37 },
    { 0x46ee2ed7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146e2ede, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e2ee1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77ae2ee3, SUC_lexicon_tags_13 },
    { 0xae8e2ee4, SUC_lexicon_tags_38 },
    { 0x504e2ee5, SUC_lexicon_tags_43 },
    { 0x912e2ee5, SUC_lexicon_tags_17 },
    { 0xdd2e2ee4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90e2eec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2ef2, SUC_lexicon_tags_10 },
    { 0x2d2e2ef3, SUC_lexicon_tags_2 },
    { 0xb58e2ef4, SUC_lexicon_tags_27 },
    { 0x93ee2ef4, SUC_lexicon_tags_39 },
    { 0xaf8e2ef6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6e2efb, SUC_lexicon_tags_45 },
    { 0x150e2efc, SUC_lexicon_tags_61 },
    { 0x6f0e2efb, SUC_lexicon_tags_47 },
    { 0xf3ee2efe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e2f01, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x634e2f04, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68e2f08, SUC_lexicon_tags_109 },
    { 0x6b8e2f09, SUC_lexicon_tags_25 },
    { 0x7fae2f0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x30ce2f0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x288e2f0e, SUC_lexicon_tags_25 },
    { 0xf6ae2f0f, SUC_lexicon_tags_26 },
    { 0xa32e2f0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x624e2f12, SUC_lexicon_tags_27 },
    { 0xb36e2f12, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508e2f17, SUC_lexicon_tags_8 },
    { 0xf66e2f18, SUC_lexicon_tags_25 },
    { 0x754e2f17, SUC_lexicon_tags_11 },
    { 0xddae2f18, SUC_lexicon_tags_0 },
    { 0x3eee2f1a, SUC_lexicon_tags_7 },
    { 0x2dae2f1a, SUC_lexicon_tags_9 },
    { 0x694e2f1d, SUC_lexicon_tags_37 },
    { 0x3ee2f1e, SUC_lexicon_tags_31 },
    { 0xb7ee2f1b, SUC_lexicon_tags_61 },
    { 0xd8ce2f19, SUC_lexicon_tags_25 },
    { 0xf56e2f19, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ce2f25, SUC_lexicon_tags_25 },
    { 0xf2ae2f26, SUC_lexicon_tags_30 },
    { 0x78e2f27, SUC_lexicon_tags_10 },
    { 0xd3ae2f28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a8e2f2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92e2f31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ee2f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70e2f39, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e2f3c, SUC_lexicon_tags_37 },
    { 0xeb2e2f3d, SUC_lexicon_tags_17 },
    { 0x43ae2f3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874e2f43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2e2f4b, SUC_lexicon_tags_104 },
    { 0x4a6e2f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa22e2f4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x804e2f50, SUC_lexicon_tags_17 },
    { 0xb0ce2f51, SUC_lexicon_tags_25 },
    { 0x9fee2f52, SUC_lexicon_tags_17 },
    { 0x114e2f52, SUC_lexicon_tags_0 },
    { 0xacae2f52, SUC_lexicon_tags_17 },
    { 0x318e2f55, SUC_lexicon_tags_15 },
    { 0x1f6e2f50, SUC_lexicon_tags_25 },
    { 0x11ce2f57, SUC_lexicon_tags_7 },
    { 0xb8ae2f54, SUC_lexicon_tags_27 },
    { 0x34e2f59, SUC_lexicon_tags_85 },
    { 0xe44e2f5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ce2f5e, SUC_lexicon_tags_38 },
    { 0x71ae2f5f, SUC_lexicon_tags_12 },
    { 0x838e2f60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb8ee2f62, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0e2f67, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96e2f6b, SUC_lexicon_tags_60 },
    { 0x5e4e2f6c, SUC_lexicon_tags_87 },
    { 0x792e2f6c, SUC_lexicon_tags_11 },
    { 0x558e2f6e, SUC_lexicon_tags_27 },
    { 0xfe2e2f6d, SUC_lexicon_tags_8 },
    { 0x6dce2f70, SUC_lexicon_tags_1 },
    { 0xd14e2f71, SUC_lexicon_tags_27 },
    { 0xf0ae2f71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd54e2f74, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e2f77, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ae2f7a, SUC_lexicon_tags_15 },
    { 0xbcae2f7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bee2f7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dee2f80, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x808e2f82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f0e2f84, SUC_lexicon_tags_7 },
    { 0x9a8e2f85, SUC_lexicon_tags_31 },
    { 0xb6ce2f86, SUC_lexicon_tags_2 },
    { 0xd74e2f87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74e2f8a, SUC_lexicon_tags_21 },
    { 0x94ce2f8b, SUC_lexicon_tags_2 },
    { 0xc18e2f8c, SUC_lexicon_tags_14 },
    { 0x672e2f8c, SUC_lexicon_tags_26 },
    { 0x7cce2f8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20e2f92, SUC_lexicon_tags_59 },
    { 0xda2e2f93, SUC_lexicon_tags_28 },
    { 0x258e2f94, SUC_lexicon_tags_19 },
    { 0x54e2f95, SUC_lexicon_tags_12 },
    { 0xe82e2f96, SUC_lexicon_tags_11 },
    { 0x9f2e2f96, SUC_lexicon_tags_8 },
    { 0xa8ce2f98, SUC_lexicon_tags_15 },
    { 0x5dee2f97, SUC_lexicon_tags_1 },
    { 0x25ae2f9a, SUC_lexicon_tags_2 },
    { 0xa14e2f96, SUC_lexicon_tags_11 },
    { 0x140e2f9c, SUC_lexicon_tags_2 },
    { 0xaf2e2f9d, SUC_lexicon_tags_7 },
    { 0x6d0e2f9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ce2fa4, SUC_lexicon_tags_66 },
    { 0xf3ee2fa4, SUC_lexicon_tags_11 },
    { 0xcc8e2fa6, SUC_lexicon_tags_15 },
    { 0x808e2fa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ee2fb3, SUC_lexicon_tags_2 },
    { 0xb08e2fb4, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50e2fb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ee2fba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ae2fbe, SUC_lexicon_tags_12 },
    { 0x960e2fbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e2fc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4e2fcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x578e2fd0, SUC_lexicon_tags_7 },
    { 0xe22e2fd0, SUC_lexicon_tags_53 },
    { 0xd7ae2fd2, SUC_lexicon_tags_55 },
    { 0x956e2fd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b8e2fd5, SUC_lexicon_tags_25 },
    { 0xd22e2fd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238e2fdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4e2fe4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ae2fe7, SUC_lexicon_tags_25 },
    { 0x2dee2fe7, SUC_lexicon_tags_45 },
    { 0x476e2fe9, SUC_lexicon_tags_10 },
    { 0x370e2fe9, SUC_lexicon_tags_28 },
    { 0x954e2fe9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xee4e2fed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x778e2fef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x408e2ff1, SUC_lexicon_tags_12 },
    { 0x58ce2ff1, SUC_lexicon_tags_11 },
    { 0x38ce2ff3, SUC_lexicon_tags_23 },
    { 0x434e2ff4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd26e2ff6, SUC_lexicon_tags_198 },
    { 0xd94e2ff7, SUC_lexicon_tags_37 },
    { 0x536e2ff8, SUC_lexicon_tags_17 },
    { 0x3a2e2ff9, SUC_lexicon_tags_113 },
    { 0xa04e2ffa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ce3001, SUC_lexicon_tags_11 },
    { 0x976e3002, SUC_lexicon_tags_2 },
    { 0xc1ce3001, SUC_lexicon_tags_15 },
    { 0xb14e3004, SUC_lexicon_tags_15 },
    { 0x656e3001, SUC_lexicon_tags_40 },
    { 0x654e3006, SUC_lexicon_tags_82 },
    { 0x62ce3007, SUC_lexicon_tags_11 },
    { 0x6f4e3007, SUC_lexicon_tags_14 },
    { 0x4ace3008, SUC_lexicon_tags_6 },
    { 0x58ce3008, SUC_lexicon_tags_37 },
    { 0x796e3008, SUC_lexicon_tags_26 },
    { 0xe1ce3007, SUC_lexicon_tags_79 },
    { 0x81ce3009, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ce3012, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdeae3014, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48ee3016, SUC_lexicon_tags_26 },
    { 0x9a2e3017, SUC_lexicon_tags_16 },
    { 0xa72e3016, SUC_lexicon_tags_37 },
    { 0x8a6e3016, SUC_lexicon_tags_16 },
    { 0x5a4e301a, SUC_lexicon_tags_12 },
    { 0x1eae301a, SUC_lexicon_tags_56 },
    { 0x350e301b, SUC_lexicon_tags_27 },
    { 0x766e301d, SUC_lexicon_tags_26 },
    { 0x928e301e, SUC_lexicon_tags_37 },
    { 0x99ee301e, SUC_lexicon_tags_37 },
    { 0x5eee301d, SUC_lexicon_tags_15 },
    { 0xe38e3021, SUC_lexicon_tags_17 },
    { 0x19ce3017, SUC_lexicon_tags_14 },
    { 0x30ae301f, SUC_lexicon_tags_64 },
    { 0xb80e3024, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x682e3026, SUC_lexicon_tags_14 },
    { 0xcd0e3026, SUC_lexicon_tags_15 },
    { 0x4ee3028, SUC_lexicon_tags_27 },
    { 0x196e3029, SUC_lexicon_tags_87 },
    { 0x318e3029, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4ce302c, SUC_lexicon_tags_2 },
    { 0x16ae302c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d2e302f, SUC_lexicon_tags_25 },
    { 0xa20e3030, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcd6e3032, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce303b, SUC_lexicon_tags_15 },
    { 0x672e303b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548e3041, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324e3049, SUC_lexicon_tags_23 },
    { 0x15ce304a, SUC_lexicon_tags_4 },
    { 0x922e304b, SUC_lexicon_tags_23 },
    { 0x5d8e304c, SUC_lexicon_tags_1 },
    { 0x8f8e304c, SUC_lexicon_tags_55 },
    { 0x8c0e3049, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e3051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54e305a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72e305d, SUC_lexicon_tags_39 },
    { 0x82ce305e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f4e3060, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8e3063, SUC_lexicon_tags_19 },
    { 0xa4ce3064, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744e3068, SUC_lexicon_tags_31 },
    { 0xcfce3069, SUC_lexicon_tags_45 },
    { 0x6a8e306a, SUC_lexicon_tags_39 },
    { 0x18ae306b, SUC_lexicon_tags_45 },
    { 0x596e306a, SUC_lexicon_tags_43 },
    { 0x208e306d, SUC_lexicon_tags_82 },
    { 0x270e306b, SUC_lexicon_tags_7 },
    { 0x8c8e3068, SUC_lexicon_tags_11 },
    { 0x10ae306a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x898e3072, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab0e3074, SUC_lexicon_tags_27 },
    { 0xf9ee3074, SUC_lexicon_tags_37 },
    { 0xa78e3074, SUC_lexicon_tags_23 },
    { 0x3dee3077, SUC_lexicon_tags_33 },
    { 0xda8e3077, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdd0e307a, SUC_lexicon_tags_23 },
    { 0xa8e307b, SUC_lexicon_tags_95 },
    { 0x8e2e307a, SUC_lexicon_tags_10 },
    { 0x4eae307c, SUC_lexicon_tags_48 },
    { 0xf72e307e, SUC_lexicon_tags_31 },
    { 0xfa4e307e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfce3082, SUC_lexicon_tags_37 },
    { 0x942e3083, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ace3087, SUC_lexicon_tags_2 },
    { 0xc26e3088, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x48e308a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ee308f, SUC_lexicon_tags_50 },
    { 0xcdce308f, SUC_lexicon_tags_12 },
    { 0xb36e3091, SUC_lexicon_tags_16 },
    { 0xf02e3091, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4a4e3094, SUC_lexicon_tags_45 },
    { 0xa32e3095, SUC_lexicon_tags_1 },
    { 0x554e3094, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ee309a, SUC_lexicon_tags_10 },
    { 0xfc4e309b, SUC_lexicon_tags_15 },
    { 0x936e309a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866e30a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bee30a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x856e30a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x302e30a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ce30aa, SUC_lexicon_tags_45 },
    { 0x284e30ab, SUC_lexicon_tags_1 },
    { 0x4ae30ac, SUC_lexicon_tags_33 },
    { 0x256e30ad, SUC_lexicon_tags_17 },
    { 0x590e30ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc82e30b0, SUC_lexicon_tags_38 },
    { 0x728e30b1, SUC_lexicon_tags_0 },
    { 0x1f0e30b2, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xefee30b5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x7e0e30b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970e30be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84e30c2, SUC_lexicon_tags_12 },
    { 0x918e30c3, SUC_lexicon_tags_87 },
    { 0x216e30c3, SUC_lexicon_tags_33 },
    { 0x13ae30c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0e30ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aee30cc, SUC_lexicon_tags_17 },
    { 0x2e8e30cc, SUC_lexicon_tags_18 },
    { 0x33ae30cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848e30d9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x378e30dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e30df, SUC_lexicon_tags_1 },
    { 0xa10e30e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece30e4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ee30e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e30eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8e30f0, SUC_lexicon_tags_7 },
    { 0xd3ce30f0, SUC_lexicon_tags_2 },
    { 0xd80e30f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6e30f7, SUC_lexicon_tags_48 },
    { 0xc56e30f8, SUC_lexicon_tags_23 },
    { 0x41ee30f9, SUC_lexicon_tags_11 },
    { 0xa88e30f9, SUC_lexicon_tags_37 },
    { 0xa1ee30fb, SUC_lexicon_tags_1 },
    { 0xa66e30fc, SUC_lexicon_tags_63 },
    { 0xe70e30fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cce30ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfce3101, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff2e3103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0e3109, SUC_lexicon_tags_1 },
    { 0x298e3109, SUC_lexicon_tags_7 },
    { 0x3b0e310b, SUC_lexicon_tags_15 },
    { 0x636e310c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740e3110, SUC_lexicon_tags_11 },
    { 0x1bce3111, SUC_lexicon_tags_15 },
    { 0xf4ce3111, SUC_lexicon_tags_11 },
    { 0x442e3112, SUC_lexicon_tags_81 },
    { 0x3eee3114, SUC_lexicon_tags_25 },
    { 0x43ce3114, SUC_lexicon_tags_0 },
    { 0x972e3116, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6e311f, SUC_lexicon_tags_43 },
    { 0x67ce3120, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e4e3122, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x542e3125, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ce3128, SUC_lexicon_tags_38 },
    { 0x440e3128, SUC_lexicon_tags_21 },
    { 0xf58e3129, SUC_lexicon_tags_42 },
    { 0x534e312b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e8e312d, SUC_lexicon_tags_1 },
    { 0x37ee312e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x336e3131, SUC_lexicon_tags_22 },
    { 0x5bae3131, SUC_lexicon_tags_30 },
    { 0x4ace3132, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ae3138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354e313d, SUC_lexicon_tags_77 },
    { 0x928e313e, SUC_lexicon_tags_60 },
    { 0x604e313e, SUC_lexicon_tags_25 },
    { 0x942e3140, SUC_lexicon_tags_63 },
    { 0x712e313d, SUC_lexicon_tags_13 },
    { 0xdb0e3140, SUC_lexicon_tags_29 },
    { 0x878e3143, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6e3148, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x654e314b, SUC_lexicon_tags_25 },
    { 0x986e314c, SUC_lexicon_tags_2 },
    { 0x5e8e314c, SUC_lexicon_tags_15 },
    { 0xfd4e314e, SUC_lexicon_tags_23 },
    { 0x490e314f, SUC_lexicon_tags_43 },
    { 0x88ae314c, SUC_lexicon_tags_9 },
    { 0xd56e314c, SUC_lexicon_tags_37 },
    { 0x208e3152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cae3157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cae315c, SUC_lexicon_tags_28 },
    { 0x6aee315c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aae3160, SUC_lexicon_tags_27 },
    { 0xd4e3161, SUC_lexicon_tags_45 },
    { 0x3aee3162, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e4e3164, SUC_lexicon_tags_31 },
    { 0x2b4e3165, SUC_lexicon_tags_55 },
    { 0xdfae3165, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50e3169, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3a4e316b, SUC_lexicon_tags_83 },
    { 0xb5ae316c, SUC_lexicon_tags_1 },
    { 0xa16e316d, SUC_lexicon_tags_37 },
    { 0xf38e316c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504e3172, SUC_lexicon_tags_47 },
    { 0x2d8e3172, SUC_lexicon_tags_37 },
    { 0x770e3172, SUC_lexicon_tags_9 },
    { 0x172e3175, SUC_lexicon_tags_11 },
    { 0x9fee3175, SUC_lexicon_tags_2 },
    { 0xb58e3175, SUC_lexicon_tags_9 },
    { 0xa22e3178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ee317b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcae317e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54e3183, SUC_lexicon_tags_14 },
    { 0xedce3183, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf70e3186, SUC_lexicon_tags_12 },
    { 0xbe0e3186, SUC_lexicon_tags_49 },
    { 0xb10e3188, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ee318b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa14e318d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe3ce318f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bae3193, SUC_lexicon_tags_30 },
    { 0x4b8e3193, SUC_lexicon_tags_38 },
    { 0xce8e3194, SUC_lexicon_tags_1 },
    { 0x48ae3193, SUC_lexicon_tags_25 },
    { 0xd2ee3197, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2e319a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x184e319d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636e31a1, SUC_lexicon_tags_55 },
    { 0xe98e31a1, SUC_lexicon_tags_1 },
    { 0xf76e31a3, SUC_lexicon_tags_27 },
    { 0x25ee31a4, SUC_lexicon_tags_11 },
    { 0x80e31a3, SUC_lexicon_tags_7 },
    { 0x878e31a6, SUC_lexicon_tags_25 },
    { 0xf02e31a7, SUC_lexicon_tags_11 },
    { 0xa62e31a5, SUC_lexicon_tags_13 },
    { 0x4c6e31a9, SUC_lexicon_tags_15 },
    { 0x7a0e31a9, SUC_lexicon_tags_7 },
    { 0xba0e31ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ae31ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x510e31b0, SUC_lexicon_tags_59 },
    { 0x37ce31b1, SUC_lexicon_tags_11 },
    { 0xc34e31b2, SUC_lexicon_tags_21 },
    { 0xbb8e31b0, SUC_lexicon_tags_25 },
    { 0xd9ee31b4, SUC_lexicon_tags_10 },
    { 0x2dae31b5, SUC_lexicon_tags_1 },
    { 0x94ee31b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c4e31b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76e31bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5d6e31bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04e31c6, SUC_lexicon_tags_17 },
    { 0xb38e31c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6e31cd, SUC_lexicon_tags_60 },
    { 0xd68e31cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfee31d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00e31d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18e31d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf76e31da, SUC_lexicon_tags_30 },
    { 0x172e31db, SUC_lexicon_tags_26 },
    { 0x450e31dc, SUC_lexicon_tags_28 },
    { 0x6dae31dd, SUC_lexicon_tags_64 },
    { 0xa1ce31dd, SUC_lexicon_tags_9 },
    { 0xad2e31dc, SUC_lexicon_tags_65 },
    { 0x3ee31df, SUC_lexicon_tags_11 },
    { 0x530e31e1, SUC_lexicon_tags_17 },
    { 0xaee31e2, SUC_lexicon_tags_37 },
    { 0x294e31e3, SUC_lexicon_tags_25 },
    { 0x994e31e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x174e31e7, SUC_lexicon_tags_14 },
    { 0x49ae31e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6e31ef, SUC_lexicon_tags_2 },
    { 0x87ae31f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa3ee31f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66ee31f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eae31fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e2e31fc, SUC_lexicon_tags_21 },
    { 0xc2ee31fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56e3201, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf12e3203, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x87ee3205, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84ae3207, SUC_lexicon_tags_45 },
    { 0xd24e3208, SUC_lexicon_tags_60 },
    { 0xe0ee3207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9aae320b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9f8e320d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee3213, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a4e3215, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x16ae3217, SUC_lexicon_tags_45 },
    { 0xd98e3217, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ce3222, SUC_lexicon_tags_2 },
    { 0x14e3223, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6e3226, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbae3231, SUC_lexicon_tags_12 },
    { 0x5d4e3232, SUC_lexicon_tags_11 },
    { 0x470e3233, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ae3236, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbd0e3238, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2e323c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ae323f, SUC_lexicon_tags_21 },
    { 0x23ce3240, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e3246, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378e324a, SUC_lexicon_tags_1 },
    { 0x9f2e324b, SUC_lexicon_tags_45 },
    { 0x814e324a, SUC_lexicon_tags_27 },
    { 0xbb4e324d, SUC_lexicon_tags_38 },
    { 0x3cce324d, SUC_lexicon_tags_7 },
    { 0x5e2e324f, SUC_lexicon_tags_2 },
    { 0x7cce324e, SUC_lexicon_tags_30 },
    { 0x69ae3251, SUC_lexicon_tags_29 },
    { 0xd56e3251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662e3256, SUC_lexicon_tags_12 },
    { 0x560e3256, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8a6e3259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6ce325b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96e325f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ae3262, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410e3268, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878e326f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0e3273, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2e3276, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cee3281, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d4e3283, SUC_lexicon_tags_26 },
    { 0x624e3284, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x738e3286, SUC_lexicon_tags_69 },
    { 0x296e3287, SUC_lexicon_tags_129 },
    { 0x69ce3287, SUC_lexicon_tags_43 },
    { 0x5b8e3286, SUC_lexicon_tags_17 },
    { 0xacae328a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f8e328c, SUC_lexicon_tags_34 },
    { 0x742e328d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc4e328f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62e3293, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4e3297, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ee329a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae329d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe4ae329f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab4e32a1, SUC_lexicon_tags_37 },
    { 0x980e32a2, SUC_lexicon_tags_6 },
    { 0xbae32a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa26e32a5, SUC_lexicon_tags_39 },
    { 0x75ce32a6, SUC_lexicon_tags_9 },
    { 0xfa0e32a6, SUC_lexicon_tags_43 },
    { 0xd86e32a8, SUC_lexicon_tags_17 },
    { 0xf74e32a9, SUC_lexicon_tags_25 },
    { 0x19ee32a5, SUC_lexicon_tags_17 },
    { 0xb38e32a9, SUC_lexicon_tags_11 },
    { 0xb56e32ac, SUC_lexicon_tags_37 },
    { 0x22ee32a8, SUC_lexicon_tags_11 },
    { 0x424e32ae, SUC_lexicon_tags_116 },
    { 0x37ce32af, SUC_lexicon_tags_14 },
    { 0xa10e32af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2e32b4, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x2cee32b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ce32b9, SUC_lexicon_tags_4 },
    { 0x856e32ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x754e32bd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8e32c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x968e32c4, SUC_lexicon_tags_56 },
    { 0x9e2e32c4, SUC_lexicon_tags_75 },
    { 0xecee32c6, SUC_lexicon_tags_13 },
    { 0x628e32c7, SUC_lexicon_tags_49 },
    { 0x360e32c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x378e32ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2e32d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26e32d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0ae32d9, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccee32dd, SUC_lexicon_tags_37 },
    { 0x90ee32de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eee32e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd30e32ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66e32ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806e32f2, SUC_lexicon_tags_1 },
    { 0xb50e32f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8ae32f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ae32f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86e3300, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4e3303, SUC_lexicon_tags_15 },
    { 0x332e3304, SUC_lexicon_tags_23 },
    { 0x6cae3303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ae330b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8e6e330d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dae3310, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce3316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ae3320, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87ae3322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42e3325, SUC_lexicon_tags_11 },
    { 0x816e3326, SUC_lexicon_tags_15 },
    { 0xc10e3327, SUC_lexicon_tags_1 },
    { 0xf74e3325, SUC_lexicon_tags_37 },
    { 0x16ae3329, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772e3330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecae3334, SUC_lexicon_tags_68 },
    { 0x45ce3335, SUC_lexicon_tags_37 },
    { 0x91ee3335, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0e333b, SUC_lexicon_tags_15 },
    { 0xec4e333c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16e3342, SUC_lexicon_tags_32 },
    { 0x5b6e3342, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8e3347, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd12e3349, SUC_lexicon_tags_13 },
    { 0x672e334a, SUC_lexicon_tags_11 },
    { 0xbaae334b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccae334e, SUC_lexicon_tags_68 },
    { 0xec6e334f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb2e3351, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbae335b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa76e335d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x852e3360, SUC_lexicon_tags_2 },
    { 0xa76e3360, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa24e3364, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ee3367, SUC_lexicon_tags_61 },
    { 0xf18e3367, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x666e336b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4e3371, SUC_lexicon_tags_30 },
    { 0x5bce3372, SUC_lexicon_tags_87 },
    { 0x2f2e3373, SUC_lexicon_tags_19 },
    { 0x654e3371, SUC_lexicon_tags_19 },
    { 0x538e3375, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x42e3377, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ee337b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x842e337d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fee3382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784e3388, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8e338b, SUC_lexicon_tags_21 },
    { 0xcd4e338c, SUC_lexicon_tags_38 },
    { 0x4e6e338c, SUC_lexicon_tags_17 },
    { 0xface338d, SUC_lexicon_tags_11 },
    { 0x898e338e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408e3393, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x598e3396, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ee33a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ae33a5, SUC_lexicon_tags_3 },
    { 0x502e33a6, SUC_lexicon_tags_17 },
    { 0x380e33a7, SUC_lexicon_tags_10 },
    { 0xb92e33a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2e33ad, SUC_lexicon_tags_27 },
    { 0xe96e33ae, SUC_lexicon_tags_14 },
    { 0x6ece33ad, SUC_lexicon_tags_17 },
    { 0xd02e33b0, SUC_lexicon_tags_47 },
    { 0x164e33b0, SUC_lexicon_tags_25 },
    { 0x3f0e33b2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf8e33b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcece33b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ce33ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576e33be, SUC_lexicon_tags_11 },
    { 0xd1ee33bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ce33c4, SUC_lexicon_tags_14 },
    { 0xd70e33c5, SUC_lexicon_tags_60 },
    { 0x116e33c5, SUC_lexicon_tags_234 },
    { 0x228e33c5, SUC_lexicon_tags_45 },
    { 0xa26e33c8, SUC_lexicon_tags_63 },
    { 0x192e33c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dae33d0, SUC_lexicon_tags_87 },
    { 0x50ae33d0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3cae33d3, SUC_lexicon_tags_17 },
    { 0x24ee33d4, SUC_lexicon_tags_17 },
    { 0xd3ce33d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2e33d9, SUC_lexicon_tags_99 },
    { 0xddce33da, SUC_lexicon_tags_2 },
    { 0xfc6e33db, SUC_lexicon_tags_17 },
    { 0xb9ae33dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcee33e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x614e33e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x630e33ea, SUC_lexicon_tags_15 },
    { 0x3bae33eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcdae33ed, SUC_lexicon_tags_60 },
    { 0xfc0e33ee, SUC_lexicon_tags_0 },
    { 0xfc8e33ef, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x622e33f1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x210e33f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4ee33f6, SUC_lexicon_tags_15 },
    { 0xdce33f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b2e33f9, SUC_lexicon_tags_37 },
    { 0x300e33fa, SUC_lexicon_tags_54 },
    { 0x2b0e33f9, SUC_lexicon_tags_2 },
    { 0x21ce33fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ae3401, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8e340b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8f6e340d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ee3410, SUC_lexicon_tags_92 },
    { 0xcf8e3410, SUC_lexicon_tags_2 },
    { 0x39ce3412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2e3415, SUC_lexicon_tags_63 },
    { 0x5a2e3416, SUC_lexicon_tags_15 },
    { 0x876e3416, SUC_lexicon_tags_28 },
    { 0x462e3418, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3dce341a, SUC_lexicon_tags_2 },
    { 0xd66e341a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xee341e, SUC_lexicon_tags_33 },
    { 0x378e341f, SUC_lexicon_tags_37 },
    { 0xa38e341e, SUC_lexicon_tags_14 },
    { 0x6e6e341f, SUC_lexicon_tags_37 },
    { 0x906e3422, SUC_lexicon_tags_27 },
    { 0x930e341e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e342a, SUC_lexicon_tags_2 },
    { 0xcd8e342b, SUC_lexicon_tags_37 },
    { 0xd10e342c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x814e342e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44e3431, SUC_lexicon_tags_27 },
    { 0x63ae3431, SUC_lexicon_tags_7 },
    { 0x2eee3433, SUC_lexicon_tags_82 },
    { 0x4f4e3434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96e3438, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46e343b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f4e343d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54e3440, SUC_lexicon_tags_25 },
    { 0x43ee3441, SUC_lexicon_tags_37 },
    { 0xb6ce3442, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56e3446, SUC_lexicon_tags_16 },
    { 0x390e3447, SUC_lexicon_tags_27 },
    { 0x880e3448, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0e344c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ae3451, SUC_lexicon_tags_17 },
    { 0x644e3452, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40e3455, SUC_lexicon_tags_19 },
    { 0x14e3456, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02e345c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56e3460, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34ce3462, SUC_lexicon_tags_11 },
    { 0x6c6e3463, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8e3466, SUC_lexicon_tags_19 },
    { 0x29ee3466, SUC_lexicon_tags_17 },
    { 0x7d4e3468, SUC_lexicon_tags_7 },
    { 0x6e8e3469, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fae346e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf36e3470, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86e347a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xecce347c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ae3480, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462e3485, SUC_lexicon_tags_229 },
    { 0x2c0e3485, SUC_lexicon_tags_1 },
    { 0x2f8e3487, SUC_lexicon_tags_65 },
    { 0xd08e3487, SUC_lexicon_tags_37 },
    { 0xe6ce3489, SUC_lexicon_tags_2 },
    { 0x6bae348a, SUC_lexicon_tags_2 },
    { 0xc70e348b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eee348e, SUC_lexicon_tags_37 },
    { 0x352e348f, SUC_lexicon_tags_2 },
    { 0xd9ee3490, SUC_lexicon_tags_1 },
    { 0xe0ce3490, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd50e3493, SUC_lexicon_tags_11 },
    { 0x39ce3494, SUC_lexicon_tags_1 },
    { 0x4eae3495, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3f2e3497, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43ce3499, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948e349f, SUC_lexicon_tags_12 },
    { 0x5c6e349f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x82ae34a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeee34ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02e34af, SUC_lexicon_tags_37 },
    { 0x754e34af, SUC_lexicon_tags_77 },
    { 0x908e34b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x102e34b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00e34bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf06e34bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ae34c3, SUC_lexicon_tags_80 },
    { 0xdb6e34c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4e34c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x202e34c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ae34cd, SUC_lexicon_tags_7 },
    { 0x34ae34ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fee34d1, SUC_lexicon_tags_23 },
    { 0x1d8e34d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf80e34d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61ee34d6, SUC_lexicon_tags_17 },
    { 0x13ae34d6, SUC_lexicon_tags_12 },
    { 0xa66e34d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb62e34da, SUC_lexicon_tags_11 },
    { 0x66ce34db, SUC_lexicon_tags_43 },
    { 0x65ce34dc, SUC_lexicon_tags_30 },
    { 0xe4ae34db, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4e34e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e34e9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918e34f0, SUC_lexicon_tags_2 },
    { 0x936e34f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ae34f5, SUC_lexicon_tags_37 },
    { 0x220e34f6, SUC_lexicon_tags_8 },
    { 0xbcae34f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e34f9, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x3ace34fb, SUC_lexicon_tags_27 },
    { 0xf2e34fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e3503, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd5ee3505, SUC_lexicon_tags_17 },
    { 0x510e3506, SUC_lexicon_tags_37 },
    { 0x880e3505, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeece3509, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ce3510, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd2e3512, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc50e3514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316e351b, SUC_lexicon_tags_11 },
    { 0x1f4e351b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cee3520, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84e3523, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x394e3525, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08e352a, SUC_lexicon_tags_66 },
    { 0x32ee352b, SUC_lexicon_tags_27 },
    { 0x55ae352c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2e352f, SUC_lexicon_tags_8 },
    { 0x5cae3530, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0e3534, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66e3537, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x120e353a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x860e353d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10ae353f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e354a, SUC_lexicon_tags_17 },
    { 0xa14e354a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6e3551, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68ae3553, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb60e3555, SUC_lexicon_tags_47 },
    { 0x110e3556, SUC_lexicon_tags_35 },
    { 0xc50e3557, SUC_lexicon_tags_0 },
    { 0x9ce3557, SUC_lexicon_tags_14 },
    { 0xd44e3556, SUC_lexicon_tags_1 },
    { 0xd42e355a, SUC_lexicon_tags_15 },
    { 0xb9ae355b, SUC_lexicon_tags_2 },
    { 0x3bee355a, SUC_lexicon_tags_298 },
    { 0x8f4e355d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ce3563, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89ae3565, SUC_lexicon_tags_33 },
    { 0x326e3565, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820e356a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e3570, SUC_lexicon_tags_7 },
    { 0x348e3571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e3579, SUC_lexicon_tags_70 },
    { 0x9d0e357a, SUC_lexicon_tags_33 },
    { 0x2ee357b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x68e357d, SUC_lexicon_tags_25 },
    { 0xff2e357e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ee3586, SUC_lexicon_tags_0 },
    { 0xeeae3587, SUC_lexicon_tags_7 },
    { 0xdc0e3587, SUC_lexicon_tags_12 },
    { 0xd86e3589, SUC_lexicon_tags_63 },
    { 0x108e3589, SUC_lexicon_tags_29 },
    { 0xca6e358b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944e358f, SUC_lexicon_tags_30 },
    { 0x9a8e358f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6e359a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ee359d, SUC_lexicon_tags_8 },
    { 0xc2ae359e, SUC_lexicon_tags_1 },
    { 0x10e359f, SUC_lexicon_tags_33 },
    { 0x484e35a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x312e35a2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe8ee35a4, SUC_lexicon_tags_2 },
    { 0x700e35a5, SUC_lexicon_tags_37 },
    { 0x2ce35a5, SUC_lexicon_tags_17 },
    { 0xecae35a7, SUC_lexicon_tags_21 },
    { 0xdc0e35a6, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8e35b0, SUC_lexicon_tags_32 },
    { 0x4bee35b1, SUC_lexicon_tags_49 },
    { 0x410e35b2, SUC_lexicon_tags_0 },
    { 0xd70e35b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36e35b8, SUC_lexicon_tags_11 },
    { 0x1bee35b9, SUC_lexicon_tags_47 },
    { 0x886e35ba, SUC_lexicon_tags_37 },
    { 0x3f2e35bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fce35bd, SUC_lexicon_tags_2 },
    { 0xa22e35bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880e35c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e35cb, SUC_lexicon_tags_11 },
    { 0x8c4e35cc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdece35cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9ee35d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0e35d7, SUC_lexicon_tags_7 },
    { 0x194e35d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12e35e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbce35e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x322e35e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8e35eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x444e35ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b6e35ef, SUC_lexicon_tags_37 },
    { 0xde0e35f0, SUC_lexicon_tags_4 },
    { 0xc6e35ef, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ae35f7, SUC_lexicon_tags_11 },
    { 0x302e35f7, SUC_lexicon_tags_58 },
    { 0x732e35f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ee3601, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ce3604, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68e360a, SUC_lexicon_tags_38 },
    { 0xa68e360b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce360f, SUC_lexicon_tags_11 },
    { 0xace3610, SUC_lexicon_tags_9 },
    { 0x34e3611, SUC_lexicon_tags_10 },
    { 0xa48e3610, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9ee3614, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdae3617, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bae361b, SUC_lexicon_tags_25 },
    { 0xc58e361c, SUC_lexicon_tags_2 },
    { 0x942e361c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ee3627, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x40e3629, SUC_lexicon_tags_47 },
    { 0xf78e362a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dce362c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaee3632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x572e3635, SUC_lexicon_tags_7 },
    { 0xbfee3636, SUC_lexicon_tags_33 },
    { 0xf8ae3637, SUC_lexicon_tags_12 },
    { 0x72ae3636, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8e3641, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ee3644, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa4e3646, SUC_lexicon_tags_1 },
    { 0xcee3647, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x292e3649, SUC_lexicon_tags_27 },
    { 0xb34e364a, SUC_lexicon_tags_60 },
    { 0x1ae364b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x94e364d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece3656, SUC_lexicon_tags_65 },
    { 0xfb0e3657, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e365b, SUC_lexicon_tags_29 },
    { 0xc8e365c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dce365f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8ee3661, SUC_lexicon_tags_43 },
    { 0x35ce3662, SUC_lexicon_tags_11 },
    { 0xdcee3663, SUC_lexicon_tags_23 },
    { 0x5d6e3663, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2e3668, SUC_lexicon_tags_45 },
    { 0x4c8e3669, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214e3671, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ee3674, SUC_lexicon_tags_17 },
    { 0x806e3675, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x656e3677, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60e367b, SUC_lexicon_tags_57 },
    { 0x76ce367c, SUC_lexicon_tags_19 },
    { 0x946e367d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ae3684, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7b2e3686, SUC_lexicon_tags_17 },
    { 0x35ae3687, SUC_lexicon_tags_0 },
    { 0xe8ae3688, SUC_lexicon_tags_11 },
    { 0x38e3688, SUC_lexicon_tags_7 },
    { 0xb6ee3688, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478e3690, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6e3696, SUC_lexicon_tags_37 },
    { 0xdce3697, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4e369a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x668e369d, SUC_lexicon_tags_15 },
    { 0x3a6e369d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eae36a2, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0x4dae36a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7ee36a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x818e36a8, SUC_lexicon_tags_21 },
    { 0xe36e36a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90e36af, SUC_lexicon_tags_32 },
    { 0x978e36b0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xe84e36b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x734e36b5, SUC_lexicon_tags_10 },
    { 0x72ce36b5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ee36c0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x658e36c2, SUC_lexicon_tags_0 },
    { 0x6e4e36c2, SUC_lexicon_tags_65 },
    { 0xad2e36c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaee36c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63ae36c9, SUC_lexicon_tags_27 },
    { 0x93ae36ca, SUC_lexicon_tags_25 },
    { 0xf06e36ca, SUC_lexicon_tags_17 },
    { 0xc14e36c9, SUC_lexicon_tags_47 },
    { 0x992e36ca, SUC_lexicon_tags_60 },
    { 0xe8ae36ce, SUC_lexicon_tags_12 },
    { 0xabae36ce, SUC_lexicon_tags_12 },
    { 0xbb8e36d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81ce36d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6e36d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x182e36d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x79ce36d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460e36de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476e36e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe68e36e4, SUC_lexicon_tags_35 },
    { 0x3f8e36e4, SUC_lexicon_tags_12 },
    { 0x956e36e6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ae36f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bce36f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e6e36f9, SUC_lexicon_tags_4 },
    { 0xbe6e36fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x148e36fd, SUC_lexicon_tags_28 },
    { 0x14ee36fd, SUC_lexicon_tags_2 },
    { 0x52ee36ff, SUC_lexicon_tags_60 },
    { 0x8ee36fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ee3704, SUC_lexicon_tags_7 },
    { 0x34ce3705, SUC_lexicon_tags_2 },
    { 0xf06e3706, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcee370a, SUC_lexicon_tags_45 },
    { 0x15ae370a, SUC_lexicon_tags_39 },
    { 0x500e370a, SUC_lexicon_tags_797 },
    { 0, NULL },
    { 0x37ce370e, SUC_lexicon_tags_47 },
    { 0xe18e370f, SUC_lexicon_tags_45 },
    { 0xa2ce3710, SUC_lexicon_tags_18 },
    { 0xcfce370e, SUC_lexicon_tags_23 },
    { 0xfe0e370e, SUC_lexicon_tags_17 },
    { 0xd74e3713, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x580e3715, SUC_lexicon_tags_0 },
    { 0x886e3716, SUC_lexicon_tags_31 },
    { 0x364e3716, SUC_lexicon_tags_2 },
    { 0x204e3718, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6e371e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ae3724, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2e3728, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfae372e, SUC_lexicon_tags_38 },
    { 0xcf8e372f, SUC_lexicon_tags_33 },
    { 0x8ece372f, SUC_lexicon_tags_10 },
    { 0xb56e372e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ae3736, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc02e3738, SUC_lexicon_tags_43 },
    { 0xec4e3738, SUC_lexicon_tags_28 },
    { 0xf5ee373a, SUC_lexicon_tags_25 },
    { 0xee6e3738, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ce3740, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd22e3742, SUC_lexicon_tags_23 },
    { 0x908e3743, SUC_lexicon_tags_6 },
    { 0x502e3744, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ce3748, SUC_lexicon_tags_38 },
    { 0x74ae3748, SUC_lexicon_tags_17 },
    { 0xa28e374a, SUC_lexicon_tags_11 },
    { 0x43ee374b, SUC_lexicon_tags_11 },
    { 0xb86e374c, SUC_lexicon_tags_25 },
    { 0x3f0e374d, SUC_lexicon_tags_26 },
    { 0xfcce374d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ee3752, SUC_lexicon_tags_12 },
    { 0x420e3753, SUC_lexicon_tags_30 },
    { 0xb2ee3754, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbae3756, SUC_lexicon_tags_45 },
    { 0x1ace3756, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x818e3759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26e375c, SUC_lexicon_tags_9 },
    { 0xc72e375c, SUC_lexicon_tags_15 },
    { 0x3f6e375e, SUC_lexicon_tags_75 },
    { 0x728e375e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce3762, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x260e3765, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfae3767, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0e376a, SUC_lexicon_tags_26 },
    { 0xbbee376a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x664e376d, SUC_lexicon_tags_48 },
    { 0x1a2e376e, SUC_lexicon_tags_7 },
    { 0xb12e376f, SUC_lexicon_tags_12 },
    { 0xfbee376f, SUC_lexicon_tags_0 },
    { 0xbeee3771, SUC_lexicon_tags_11 },
    { 0x702e3772, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe36e3774, SUC_lexicon_tags_9 },
    { 0x454e3775, SUC_lexicon_tags_2 },
    { 0x690e3776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4e3779, SUC_lexicon_tags_49 },
    { 0x3ace377a, SUC_lexicon_tags_11 },
    { 0x9fee377b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0e377f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ee3782, SUC_lexicon_tags_6 },
    { 0x77ee3782, SUC_lexicon_tags_37 },
    { 0x37ee3782, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542e378b, SUC_lexicon_tags_3 },
    { 0x7b4e378b, SUC_lexicon_tags_0 },
    { 0x6cce378b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x120e378f, SUC_lexicon_tags_12 },
    { 0x73ae3790, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eae3793, SUC_lexicon_tags_37 },
    { 0xe6e3793, SUC_lexicon_tags_63 },
    { 0x5ce3793, SUC_lexicon_tags_1 },
    { 0xc2ee3796, SUC_lexicon_tags_33 },
    { 0x296e3797, SUC_lexicon_tags_0 },
    { 0x36ae3797, SUC_lexicon_tags_37 },
    { 0xda4e3799, SUC_lexicon_tags_7 },
    { 0x1fae379a, SUC_lexicon_tags_15 },
    { 0xbd4e3794, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe08e379d, SUC_lexicon_tags_11 },
    { 0x272e379e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4fce37a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6e37a4, SUC_lexicon_tags_13 },
    { 0x774e37a5, SUC_lexicon_tags_0 },
    { 0x8cce37a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x67ae37a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb32e37aa, SUC_lexicon_tags_25 },
    { 0x3c8e37ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdee37ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ae37b4, SUC_lexicon_tags_48 },
    { 0x158e37b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ce37b9, SUC_lexicon_tags_11 },
    { 0xdb2e37ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xae2e37bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bae37bf, SUC_lexicon_tags_15 },
    { 0xe80e37bf, SUC_lexicon_tags_113 },
    { 0xe92e37c1, SUC_lexicon_tags_7 },
    { 0x23ae37c2, SUC_lexicon_tags_60 },
    { 0xb6e37c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966e37c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06e37cc, SUC_lexicon_tags_17 },
    { 0xebae37cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8e37d1, SUC_lexicon_tags_37 },
    { 0xad8e37d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6e37d6, SUC_lexicon_tags_6 },
    { 0x2e8e37d7, SUC_lexicon_tags_37 },
    { 0xad2e37d8, SUC_lexicon_tags_17 },
    { 0x6d2e37d9, SUC_lexicon_tags_27 },
    { 0x7aee37d9, SUC_lexicon_tags_66 },
    { 0x6dce37da, SUC_lexicon_tags_17 },
    { 0x370e37d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed0e37de, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x774e37e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8e37e4, SUC_lexicon_tags_198 },
    { 0x496e37e4, SUC_lexicon_tags_65 },
    { 0xf76e37e6, SUC_lexicon_tags_13 },
    { 0x714e37e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ce37eb, SUC_lexicon_tags_30 },
    { 0x456e37ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebce37f0, SUC_lexicon_tags_10 },
    { 0xac8e37f1, SUC_lexicon_tags_15 },
    { 0xe1ee37f2, SUC_lexicon_tags_79 },
    { 0x3a8e37f3, SUC_lexicon_tags_25 },
    { 0x2e8e37f4, SUC_lexicon_tags_43 },
    { 0xc5ee37f5, SUC_lexicon_tags_2 },
    { 0xfeae37f6, SUC_lexicon_tags_23 },
    { 0x4e0e37f7, SUC_lexicon_tags_37 },
    { 0x5e8e37f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc94e37fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c8e37fc, SUC_lexicon_tags_49 },
    { 0x4fce37fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb58e37ff, SUC_lexicon_tags_14 },
    { 0xec0e37ff, SUC_lexicon_tags_46 },
    { 0xa0ce3801, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0e3808, SUC_lexicon_tags_2 },
    { 0x74ce3809, SUC_lexicon_tags_37 },
    { 0xc74e380a, SUC_lexicon_tags_47 },
    { 0x69ee380b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6fce380d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x392e380f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x150e3811, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6e3813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea2e3815, SUC_lexicon_tags_70 },
    { 0x4d2e3815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x560e3818, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0e381c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14e381f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cce3822, SUC_lexicon_tags_15 },
    { 0x900e3823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x140e3825, SUC_lexicon_tags_8 },
    { 0xb26e3825, SUC_lexicon_tags_7 },
    { 0x752e3826, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ee382b, SUC_lexicon_tags_77 },
    { 0x666e382b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462e3831, SUC_lexicon_tags_39 },
    { 0x120e3831, SUC_lexicon_tags_14 },
    { 0x1c0e3832, SUC_lexicon_tags_15 },
    { 0xc46e3834, SUC_lexicon_tags_17 },
    { 0xb3ce3831, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ce383c, SUC_lexicon_tags_27 },
    { 0x160e383d, SUC_lexicon_tags_38 },
    { 0xe04e383d, SUC_lexicon_tags_17 },
    { 0x2cae383f, SUC_lexicon_tags_37 },
    { 0xd30e383f, SUC_lexicon_tags_47 },
    { 0xfa8e3840, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fae3844, SUC_lexicon_tags_13 },
    { 0xd6ee3845, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6e3849, SUC_lexicon_tags_30 },
    { 0xf3ee384a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x522e384d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9bce384f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x410e3851, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ee3854, SUC_lexicon_tags_25 },
    { 0x48ae3854, SUC_lexicon_tags_11 },
    { 0x31ae3856, SUC_lexicon_tags_38 },
    { 0xee2e3857, SUC_lexicon_tags_11 },
    { 0x2fee3857, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ee385b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x800e385d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2e3860, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcee3862, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30e3865, SUC_lexicon_tags_27 },
    { 0xd08e3866, SUC_lexicon_tags_14 },
    { 0x7f0e3866, SUC_lexicon_tags_37 },
    { 0x8cee3866, SUC_lexicon_tags_77 },
    { 0xda8e3866, SUC_lexicon_tags_14 },
    { 0x5bee3867, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x228e386d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ce3871, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc2e3873, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x63ae3875, SUC_lexicon_tags_45 },
    { 0xe58e3875, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x154e3878, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46e3882, SUC_lexicon_tags_21 },
    { 0xd0e3883, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ee3889, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbee388f, SUC_lexicon_tags_38 },
    { 0xa4ae388f, SUC_lexicon_tags_4 },
    { 0xb6ee3890, SUC_lexicon_tags_28 },
    { 0x11ce3892, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdce3895, SUC_lexicon_tags_11 },
    { 0x6f2e3896, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc3ce3898, SUC_lexicon_tags_27 },
    { 0x11ce3899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ce389e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4e38a1, SUC_lexicon_tags_39 },
    { 0xf46e38a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ce38a6, SUC_lexicon_tags_27 },
    { 0x3e0e38a7, SUC_lexicon_tags_15 },
    { 0xcce38a7, SUC_lexicon_tags_2 },
    { 0xe6ee38a9, SUC_lexicon_tags_7 },
    { 0x980e38aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2e38b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ce38b6, SUC_lexicon_tags_7 },
    { 0x56e38b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e38bb, SUC_lexicon_tags_37 },
    { 0xaa4e38bc, SUC_lexicon_tags_17 },
    { 0xfb4e38bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbae38c0, SUC_lexicon_tags_27 },
    { 0xaeee38c0, SUC_lexicon_tags_17 },
    { 0x8b2e38c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x89ee38c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ce38ce, SUC_lexicon_tags_39 },
    { 0xd98e38cf, SUC_lexicon_tags_2 },
    { 0x372e38d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4e38d3, SUC_lexicon_tags_48 },
    { 0x532e38d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202e38d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ce38de, SUC_lexicon_tags_9 },
    { 0x21ce38df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce38e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4c0e38e5, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08e38e8, SUC_lexicon_tags_39 },
    { 0xb9ce38e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ce38ed, SUC_lexicon_tags_16 },
    { 0xb3ce38ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372e38f2, SUC_lexicon_tags_25 },
    { 0x888e38f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6e38f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e38fa, SUC_lexicon_tags_19 },
    { 0x564e38fa, SUC_lexicon_tags_84 },
    { 0x778e38fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xafee38ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x430e3901, SUC_lexicon_tags_25 },
    { 0xf6ee3901, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc98e3904, SUC_lexicon_tags_8 },
    { 0xa4e3905, SUC_lexicon_tags_11 },
    { 0x81ee3905, SUC_lexicon_tags_9 },
    { 0x2cee3907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4e390a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bee390d, SUC_lexicon_tags_0 },
    { 0x6aee390e, SUC_lexicon_tags_11 },
    { 0x85ae390f, SUC_lexicon_tags_7 },
    { 0xb0ee3910, SUC_lexicon_tags_21 },
    { 0x3d6e390e, SUC_lexicon_tags_25 },
    { 0xe56e3912, SUC_lexicon_tags_13 },
    { 0x65ae3912, SUC_lexicon_tags_29 },
    { 0xbdce390e, SUC_lexicon_tags_6 },
    { 0xd00e3915, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ee3918, SUC_lexicon_tags_0 },
    { 0xf7ee3919, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8ce391b, SUC_lexicon_tags_60 },
    { 0xefee391c, SUC_lexicon_tags_25 },
    { 0xc10e391d, SUC_lexicon_tags_15 },
    { 0x9c2e391e, SUC_lexicon_tags_0 },
    { 0x17ae391d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2e3924, SUC_lexicon_tags_15 },
    { 0x870e3925, SUC_lexicon_tags_11 },
    { 0x726e3926, SUC_lexicon_tags_1 },
    { 0x49ae3927, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20e392a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x816e392d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10e392f, SUC_lexicon_tags_37 },
    { 0x7f6e3930, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16e3934, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0e3937, SUC_lexicon_tags_15 },
    { 0x674e3938, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x234e393a, SUC_lexicon_tags_77 },
    { 0x7bae393b, SUC_lexicon_tags_63 },
    { 0x7fae393c, SUC_lexicon_tags_11 },
    { 0xa7ae393d, SUC_lexicon_tags_1 },
    { 0x2fae393e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2e3943, SUC_lexicon_tags_2 },
    { 0xbc0e3943, SUC_lexicon_tags_46 },
    { 0x7b2e3945, SUC_lexicon_tags_27 },
    { 0xe6ee3946, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x108e3948, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x456e394b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4e3951, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf68e3953, SUC_lexicon_tags_46 },
    { 0x5a4e3953, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeae3958, SUC_lexicon_tags_37 },
    { 0x77ee3959, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xda2e395b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ee3964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dce3968, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aee396a, SUC_lexicon_tags_6 },
    { 0x4d2e396a, SUC_lexicon_tags_68 },
    { 0x3e4e396c, SUC_lexicon_tags_21 },
    { 0xddae396b, SUC_lexicon_tags_17 },
    { 0xfd2e396e, SUC_lexicon_tags_48 },
    { 0x55ce396f, SUC_lexicon_tags_7 },
    { 0xd2e3970, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa32e3972, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ce3976, SUC_lexicon_tags_7 },
    { 0x200e3976, SUC_lexicon_tags_69 },
    { 0xef2e3978, SUC_lexicon_tags_104 },
    { 0xa66e3979, SUC_lexicon_tags_15 },
    { 0x60e3978, SUC_lexicon_tags_25 },
    { 0x464e397b, SUC_lexicon_tags_11 },
    { 0x760e397c, SUC_lexicon_tags_43 },
    { 0x2a0e397d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x890e397f, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58e3984, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43ae3986, SUC_lexicon_tags_15 },
    { 0x1aae3986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cae3989, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82e398e, SUC_lexicon_tags_5 },
    { 0x3f6e398f, SUC_lexicon_tags_14 },
    { 0x71ce3990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cee3992, SUC_lexicon_tags_79 },
    { 0x836e3992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734e3997, SUC_lexicon_tags_46 },
    { 0x1c4e3998, SUC_lexicon_tags_30 },
    { 0xa38e3998, SUC_lexicon_tags_39 },
    { 0xc6ee3999, SUC_lexicon_tags_37 },
    { 0xc3ae399a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ee39a0, SUC_lexicon_tags_15 },
    { 0xc1ce39a1, SUC_lexicon_tags_17 },
    { 0xaf2e39a1, SUC_lexicon_tags_1 },
    { 0x674e39a3, SUC_lexicon_tags_2 },
    { 0xf44e39a4, SUC_lexicon_tags_38 },
    { 0x262e39a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6e39a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e39b0, SUC_lexicon_tags_68 },
    { 0x8e39b0, SUC_lexicon_tags_45 },
    { 0x152e39b2, SUC_lexicon_tags_18 },
    { 0xb2ce39b3, SUC_lexicon_tags_31 },
    { 0xd8ae39b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ce39bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22e39bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x64ee39bf, SUC_lexicon_tags_45 },
    { 0xe4e39c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810e39c4, SUC_lexicon_tags_25 },
    { 0x636e39c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4e39cc, SUC_lexicon_tags_17 },
    { 0xa54e39cd, SUC_lexicon_tags_82 },
    { 0xcb4e39cc, SUC_lexicon_tags_10 },
    { 0x6dce39cf, SUC_lexicon_tags_63 },
    { 0x9b2e39cc, SUC_lexicon_tags_1 },
    { 0xd04e39d1, SUC_lexicon_tags_11 },
    { 0x8e8e39d2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ee39d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ee39dc, SUC_lexicon_tags_25 },
    { 0x4c0e39dd, SUC_lexicon_tags_10 },
    { 0xdcee39de, SUC_lexicon_tags_37 },
    { 0xcce39df, SUC_lexicon_tags_47 },
    { 0x2fae39de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ce39e6, SUC_lexicon_tags_37 },
    { 0x5f8e39e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee4e39e9, SUC_lexicon_tags_0 },
    { 0x97ce39ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02e39f0, SUC_lexicon_tags_11 },
    { 0x55ae39f1, SUC_lexicon_tags_39 },
    { 0x43ee39f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbeee39f4, SUC_lexicon_tags_17 },
    { 0xf0ee39f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ae39fa, SUC_lexicon_tags_49 },
    { 0x738e39fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ae39fe, SUC_lexicon_tags_10 },
    { 0xa16e39ff, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ae3a03, SUC_lexicon_tags_17 },
    { 0x4e4e3a04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cae3a09, SUC_lexicon_tags_11 },
    { 0xcf2e3a09, SUC_lexicon_tags_63 },
    { 0xb56e3a0b, SUC_lexicon_tags_45 },
    { 0xec8e3a0c, SUC_lexicon_tags_15 },
    { 0x6ece3a0a, SUC_lexicon_tags_14 },
    { 0x432e3a0e, SUC_lexicon_tags_12 },
    { 0xa1ee3a0f, SUC_lexicon_tags_38 },
    { 0xb7ee3a10, SUC_lexicon_tags_17 },
    { 0xd90e3a0f, SUC_lexicon_tags_17 },
    { 0xb16e3a10, SUC_lexicon_tags_38 },
    { 0x46ee3a13, SUC_lexicon_tags_37 },
    { 0x244e3a14, SUC_lexicon_tags_39 },
    { 0xba4e3a15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ace3a17, SUC_lexicon_tags_7 },
    { 0xaf8e3a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bee3a1e, SUC_lexicon_tags_2 },
    { 0xc90e3a1f, SUC_lexicon_tags_0 },
    { 0x7f4e3a1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x96ae3a22, SUC_lexicon_tags_48 },
    { 0x4e8e3a23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528e3a2b, SUC_lexicon_tags_43 },
    { 0x9eee3a2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x444e3a2e, SUC_lexicon_tags_43 },
    { 0xbeae3a2f, SUC_lexicon_tags_11 },
    { 0x86e3a2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x180e3a33, SUC_lexicon_tags_37 },
    { 0x5cee3a34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e3a38, SUC_lexicon_tags_15 },
    { 0x7eee3a39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180e3a3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdee3a4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x598e3a4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd2e3a4f, SUC_lexicon_tags_43 },
    { 0x6ee3a50, SUC_lexicon_tags_171 },
    { 0x724e3a4f, SUC_lexicon_tags_15 },
    { 0xd6ee3a52, SUC_lexicon_tags_15 },
    { 0xb0ce3a52, SUC_lexicon_tags_8 },
    { 0x538e3a54, SUC_lexicon_tags_39 },
    { 0x224e3a55, SUC_lexicon_tags_37 },
    { 0xc0ae3a56, SUC_lexicon_tags_25 },
    { 0x120e3a54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecee3a5e, SUC_lexicon_tags_24 },
    { 0x708e3a5f, SUC_lexicon_tags_34 },
    { 0xbaae3a5f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ce3a65, SUC_lexicon_tags_8 },
    { 0xeb8e3a65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42ae3a68, SUC_lexicon_tags_60 },
    { 0x5b0e3a69, SUC_lexicon_tags_7 },
    { 0x2a8e3a68, SUC_lexicon_tags_37 },
    { 0xf06e3a6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ee3a6e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502e3a72, SUC_lexicon_tags_37 },
    { 0x514e3a72, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e3a78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dee3a7b, SUC_lexicon_tags_35 },
    { 0x4e4e3a7c, SUC_lexicon_tags_68 },
    { 0x2cae3a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8e3a80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ae3a83, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x780e3a85, SUC_lexicon_tags_8 },
    { 0xbdae3a86, SUC_lexicon_tags_30 },
    { 0x8e0e3a86, SUC_lexicon_tags_68 },
    { 0x1eee3a88, SUC_lexicon_tags_37 },
    { 0xd24e3a86, SUC_lexicon_tags_37 },
    { 0x8e2e3a8a, SUC_lexicon_tags_27 },
    { 0xc98e3a8a, SUC_lexicon_tags_13 },
    { 0x8aee3a8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ee3a8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fae3a92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758e3a99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x618e3a9b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x148e3a9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6e3aa0, SUC_lexicon_tags_23 },
    { 0xff8e3aa1, SUC_lexicon_tags_43 },
    { 0x41ae3aa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x184e3aa4, SUC_lexicon_tags_146 },
    { 0x800e3aa4, SUC_lexicon_tags_16 },
    { 0x648e3aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6e3aab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e3ab1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bce3ab6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58e3aba, SUC_lexicon_tags_2 },
    { 0x1bee3abb, SUC_lexicon_tags_12 },
    { 0x604e3abc, SUC_lexicon_tags_15 },
    { 0x594e3abc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514e3ac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28e3ac5, SUC_lexicon_tags_264 },
    { 0x8a0e3ac5, SUC_lexicon_tags_23 },
    { 0xd98e3ac6, SUC_lexicon_tags_47 },
    { 0xe2e3ac8, SUC_lexicon_tags_79 },
    { 0x9e6e3ac9, SUC_lexicon_tags_71 },
    { 0xc3ee3aca, SUC_lexicon_tags_1 },
    { 0x5aae3acb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ae3ad1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x76e3ad3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10e3ad7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40ae3ad9, SUC_lexicon_tags_17 },
    { 0xe44e3ada, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82e3add, SUC_lexicon_tags_17 },
    { 0x36ce3add, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38e3ae3, SUC_lexicon_tags_198 },
    { 0x162e3ae4, SUC_lexicon_tags_37 },
    { 0x34ee3ae5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbe8e3ae7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2e3aec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb76e3aee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x280e3af1, SUC_lexicon_tags_31 },
    { 0xc20e3af2, SUC_lexicon_tags_6 },
    { 0xc56e3af3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834e3af9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98e3afc, SUC_lexicon_tags_1 },
    { 0x762e3afc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x980e3b00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596e3b0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fce3b10, SUC_lexicon_tags_1 },
    { 0x294e3b11, SUC_lexicon_tags_48 },
    { 0x4cee3b10, SUC_lexicon_tags_12 },
    { 0x916e3b13, SUC_lexicon_tags_2 },
    { 0xece3b14, SUC_lexicon_tags_3 },
    { 0xc94e3b15, SUC_lexicon_tags_3 },
    { 0x110e3b13, SUC_lexicon_tags_25 },
    { 0xf3ce3b17, SUC_lexicon_tags_68 },
    { 0x74ee3b18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4e3b1e, SUC_lexicon_tags_1 },
    { 0xdaee3b1e, SUC_lexicon_tags_11 },
    { 0x43ee3b20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466e3b24, SUC_lexicon_tags_45 },
    { 0xd18e3b25, SUC_lexicon_tags_2 },
    { 0x30ee3b26, SUC_lexicon_tags_25 },
    { 0xddce3b26, SUC_lexicon_tags_11 },
    { 0x112e3b28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf84e3b2a, SUC_lexicon_tags_17 },
    { 0xecee3b2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2e3b2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6e3b32, SUC_lexicon_tags_1 },
    { 0xffce3b33, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x7c0e3b35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x574e3b37, SUC_lexicon_tags_13 },
    { 0x96ae3b37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce3b3d, SUC_lexicon_tags_4 },
    { 0x47ae3b3e, SUC_lexicon_tags_108 },
    { 0xa5ae3b3f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e3b43, SUC_lexicon_tags_7 },
    { 0x524e3b44, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ae3b48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750e3b4f, SUC_lexicon_tags_11 },
    { 0xc0ce3b50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x416e3b52, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa0ae3b54, SUC_lexicon_tags_1 },
    { 0xd50e3b55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8e3b58, SUC_lexicon_tags_29 },
    { 0x83ce3b58, SUC_lexicon_tags_25 },
    { 0x1dce3b58, SUC_lexicon_tags_14 },
    { 0x5a0e3b5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584e3b67, SUC_lexicon_tags_119 },
    { 0xd32e3b67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbee3b6c, SUC_lexicon_tags_12 },
    { 0x5c6e3b6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16e3b74, SUC_lexicon_tags_37 },
    { 0x440e3b75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50e3b7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfe8e3b81, SUC_lexicon_tags_35 },
    { 0xb90e3b82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e3b89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0e3b8c, SUC_lexicon_tags_11 },
    { 0xccee3b8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa74e3b8f, SUC_lexicon_tags_61 },
    { 0x4fee3b8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e0e3b92, SUC_lexicon_tags_16 },
    { 0xcb4e3b93, SUC_lexicon_tags_11 },
    { 0x6f2e3b94, SUC_lexicon_tags_257 },
    { 0xa4e3b93, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x520e3b97, SUC_lexicon_tags_7 },
    { 0xa7ee3b97, SUC_lexicon_tags_2 },
    { 0x596e3b99, SUC_lexicon_tags_40 },
    { 0x166e3b99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6e3b9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ee3ba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ace3ba5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2e6e3ba7, SUC_lexicon_tags_7 },
    { 0xbc0e3ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738e3baf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb24e3bb1, SUC_lexicon_tags_10 },
    { 0x596e3bb1, SUC_lexicon_tags_1 },
    { 0xefce3bb1, SUC_lexicon_tags_29 },
    { 0xa8ae3bb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad0e3bb6, SUC_lexicon_tags_25 },
    { 0xcf2e3bb6, SUC_lexicon_tags_34 },
    { 0x978e3bb8, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ee3bbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4e3bc0, SUC_lexicon_tags_7 },
    { 0xa0ce3bc0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2e3bc4, SUC_lexicon_tags_17 },
    { 0x4ee3bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x380e3bc7, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaee3bca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712e3bd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d6e3bd2, SUC_lexicon_tags_37 },
    { 0x68ce3bd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bee3bd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x872e3bd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8e3bdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ee3be3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x676e3be5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ae3be8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdace3beb, SUC_lexicon_tags_4 },
    { 0x234e3bec, SUC_lexicon_tags_17 },
    { 0x9c6e3bed, SUC_lexicon_tags_43 },
    { 0x45ce3bec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60e3bf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfee3bf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c8e3bf6, SUC_lexicon_tags_4 },
    { 0xa78e3bf7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xae0e3bf9, SUC_lexicon_tags_33 },
    { 0xc6ce3bfa, SUC_lexicon_tags_27 },
    { 0x41ee3bfb, SUC_lexicon_tags_2 },
    { 0xaf6e3bfc, SUC_lexicon_tags_15 },
    { 0x9cae3bfc, SUC_lexicon_tags_37 },
    { 0xdf2e3bf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5eae3c00, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x462e3c03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e3c06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x582e3c08, SUC_lexicon_tags_37 },
    { 0xd0ce3c09, SUC_lexicon_tags_77 },
    { 0xfb2e3c0a, SUC_lexicon_tags_17 },
    { 0x988e3c0b, SUC_lexicon_tags_87 },
    { 0x7f2e3c0a, SUC_lexicon_tags_16 },
    { 0x938e3c0d, SUC_lexicon_tags_7 },
    { 0xa60e3c0c, SUC_lexicon_tags_17 },
    { 0x60ae3c0f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2e3c14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cae3c19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcae3c1c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404e3c24, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e3c28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390e3c30, SUC_lexicon_tags_2 },
    { 0x2eae3c31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82e3c39, SUC_lexicon_tags_12 },
    { 0x86ae3c3a, SUC_lexicon_tags_11 },
    { 0xe32e3c3b, SUC_lexicon_tags_77 },
    { 0x61ee3c3b, SUC_lexicon_tags_37 },
    { 0xefce3c3d, SUC_lexicon_tags_11 },
    { 0x3c2e3c3e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6e3c42, SUC_lexicon_tags_11 },
    { 0xaece3c42, SUC_lexicon_tags_21 },
    { 0x2bce3c44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e3c47, SUC_lexicon_tags_7 },
    { 0x18e3c47, SUC_lexicon_tags_25 },
    { 0xabae3c47, SUC_lexicon_tags_1 },
    { 0xafee3c48, SUC_lexicon_tags_11 },
    { 0x14ce3c4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894e3c56, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ce3c5c, SUC_lexicon_tags_79 },
    { 0xfe0e3c5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa36e3c60, SUC_lexicon_tags_33 },
    { 0x142e3c60, SUC_lexicon_tags_37 },
    { 0x156e3c62, SUC_lexicon_tags_16 },
    { 0x64e3c61, SUC_lexicon_tags_12 },
    { 0xa90e3c64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380e3c68, SUC_lexicon_tags_15 },
    { 0x24e3c69, SUC_lexicon_tags_39 },
    { 0x61ae3c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x636e3c6d, SUC_lexicon_tags_11 },
    { 0x898e3c6e, SUC_lexicon_tags_7 },
    { 0x200e3c6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84e3c76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e3c7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702e3c7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x532e3c81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdaae3c83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb14e3c85, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bce3c89, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6e3c91, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb62e3c93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x352e3c95, SUC_lexicon_tags_25 },
    { 0xcdae3c96, SUC_lexicon_tags_57 },
    { 0x2b2e3c97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514e3c9e, SUC_lexicon_tags_27 },
    { 0xd6ce3c9e, SUC_lexicon_tags_31 },
    { 0x628e3c9f, SUC_lexicon_tags_11 },
    { 0x612e3ca1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ae3caa, SUC_lexicon_tags_2 },
    { 0x792e3cab, SUC_lexicon_tags_47 },
    { 0xda4e3cac, SUC_lexicon_tags_3 },
    { 0x6b0e3cad, SUC_lexicon_tags_17 },
    { 0x3b4e3cad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfee3cb1, SUC_lexicon_tags_10 },
    { 0xcc4e3cb2, SUC_lexicon_tags_3 },
    { 0x616e3cb3, SUC_lexicon_tags_17 },
    { 0xb7ae3cb3, SUC_lexicon_tags_11 },
    { 0xc62e3cb3, SUC_lexicon_tags_11 },
    { 0x4e8e3cb3, SUC_lexicon_tags_0 },
    { 0x2bee3cb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36e3cba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666e3cc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2e3cc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x476e3cc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x488e3ccb, SUC_lexicon_tags_11 },
    { 0x6a8e3ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb84e3cce, SUC_lexicon_tags_23 },
    { 0x4fce3cce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ae3cd2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aae3cd5, SUC_lexicon_tags_39 },
    { 0x7b6e3cd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2cae3cd8, SUC_lexicon_tags_12 },
    { 0xfc2e3cd8, SUC_lexicon_tags_19 },
    { 0x7d2e3cd9, SUC_lexicon_tags_27 },
    { 0x9d4e3cdb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc60e3cdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8e3ce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00e3ce7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648e3cec, SUC_lexicon_tags_1 },
    { 0x38e3ced, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ae3cf3, SUC_lexicon_tags_45 },
    { 0xe8ee3cf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7cee3cf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ce3cf9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986e3cfd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2e3d00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dee3d05, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fee3d08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7ae3d0a, SUC_lexicon_tags_38 },
    { 0x202e3d0b, SUC_lexicon_tags_25 },
    { 0x88ce3d0c, SUC_lexicon_tags_33 },
    { 0x2a2e3d0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13ae3d0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x586e3d12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c6e3d14, SUC_lexicon_tags_37 },
    { 0xfc0e3d14, SUC_lexicon_tags_3 },
    { 0xd8e3d16, SUC_lexicon_tags_38 },
    { 0x1ae3d15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xafae3d1a, SUC_lexicon_tags_23 },
    { 0xd78e3d1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaaee3d1d, SUC_lexicon_tags_25 },
    { 0x306e3d1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184e3d23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800e3d2a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x80e3d2d, SUC_lexicon_tags_15 },
    { 0xc22e3d2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcf2e3d30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978e3d38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718e3d3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71ee3d3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56ae3d41, SUC_lexicon_tags_17 },
    { 0xea4e3d41, SUC_lexicon_tags_63 },
    { 0x56ae3d43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242e3d47, SUC_lexicon_tags_14 },
    { 0x772e3d48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6e3d4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x846e3d4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ce3d56, SUC_lexicon_tags_82 },
    { 0xa66e3d57, SUC_lexicon_tags_21 },
    { 0xe12e3d58, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1fae3d5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd36e3d5c, SUC_lexicon_tags_9 },
    { 0xe68e3d5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230e3d62, SUC_lexicon_tags_1 },
    { 0x3a0e3d62, SUC_lexicon_tags_23 },
    { 0x8fae3d64, SUC_lexicon_tags_17 },
    { 0xefee3d62, SUC_lexicon_tags_19 },
    { 0x6ace3d66, SUC_lexicon_tags_17 },
    { 0x2b6e3d66, SUC_lexicon_tags_27 },
    { 0x858e3d67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8e3d72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0e3d79, SUC_lexicon_tags_37 },
    { 0xc52e3d7a, SUC_lexicon_tags_32 },
    { 0xe86e3d7b, SUC_lexicon_tags_17 },
    { 0xc24e3d7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0e3d82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ee3d85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880e3d8b, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x420e3d8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0e3d93, SUC_lexicon_tags_11 },
    { 0x6e2e3d94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdae3d97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeee3d9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc7ae3da0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf88e3da3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce3da9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c0e3dab, SUC_lexicon_tags_15 },
    { 0x730e3dac, SUC_lexicon_tags_38 },
    { 0x788e3dac, SUC_lexicon_tags_9 },
    { 0x57ee3dad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3db7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbee3db9, SUC_lexicon_tags_26 },
    { 0x36e3dba, SUC_lexicon_tags_56 },
    { 0xd10e3db9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610e3dc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66e3dc6, SUC_lexicon_tags_1 },
    { 0x2cce3dc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa64e3dc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40ce3dcb, SUC_lexicon_tags_25 },
    { 0x2ae3dcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0e3dd8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200e3ddd, SUC_lexicon_tags_12 },
    { 0xaa8e3ddd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ee3de2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcee3ded, SUC_lexicon_tags_11 },
    { 0x98ae3dee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4dee3df0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b6e3df2, SUC_lexicon_tags_23 },
    { 0xd00e3df3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548e3df8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x832e3dfa, SUC_lexicon_tags_8 },
    { 0xd28e3dfa, SUC_lexicon_tags_28 },
    { 0x608e3dfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ae3dfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510e3e02, SUC_lexicon_tags_47 },
    { 0x65ee3e02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ee3e07, SUC_lexicon_tags_7 },
    { 0x2eae3e08, SUC_lexicon_tags_53 },
    { 0x600e3e09, SUC_lexicon_tags_61 },
    { 0x6bee3e09, SUC_lexicon_tags_7 },
    { 0x4cae3e0b, SUC_lexicon_tags_37 },
    { 0xa98e3e0c, SUC_lexicon_tags_6 },
    { 0x750e3e0c, SUC_lexicon_tags_37 },
    { 0xf2ee3e0e, SUC_lexicon_tags_17 },
    { 0x5ace3e09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fce3e13, SUC_lexicon_tags_85 },
    { 0x43ee3e13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xedce3e17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbee3e19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf40e3e1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x30e3e1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0e3e24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0e3e27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ee3e2b, SUC_lexicon_tags_33 },
    { 0x804e3e2b, SUC_lexicon_tags_19 },
    { 0x612e3e2d, SUC_lexicon_tags_7 },
    { 0xe3ae3e2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33ae3e30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ce3e34, SUC_lexicon_tags_25 },
    { 0xeb6e3e35, SUC_lexicon_tags_37 },
    { 0xcf4e3e35, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4e3e3a, SUC_lexicon_tags_30 },
    { 0xa6ce3e3b, SUC_lexicon_tags_110 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692e3e3f, SUC_lexicon_tags_43 },
    { 0xdf4e3e40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x492e3e43, SUC_lexicon_tags_19 },
    { 0xc1ae3e43, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcace3e47, SUC_lexicon_tags_2 },
    { 0x44ee3e48, SUC_lexicon_tags_0 },
    { 0xbb2e3e48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcdce3e4b, SUC_lexicon_tags_12 },
    { 0x5e6e3e4c, SUC_lexicon_tags_19 },
    { 0x4aae3e4d, SUC_lexicon_tags_7 },
    { 0x44e3e4e, SUC_lexicon_tags_37 },
    { 0x6b4e3e4f, SUC_lexicon_tags_48 },
    { 0x83ae3e50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa6e3e52, SUC_lexicon_tags_42 },
    { 0xed2e3e52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90e3e56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4e3e59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2e3e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240e3e60, SUC_lexicon_tags_15 },
    { 0x618e3e61, SUC_lexicon_tags_25 },
    { 0x872e3e60, SUC_lexicon_tags_12 },
    { 0xfb0e3e63, SUC_lexicon_tags_7 },
    { 0xb8e3e64, SUC_lexicon_tags_27 },
    { 0xe82e3e65, SUC_lexicon_tags_29 },
    { 0xe3ee3e66, SUC_lexicon_tags_28 },
    { 0xd92e3e67, SUC_lexicon_tags_39 },
    { 0x304e3e68, SUC_lexicon_tags_43 },
    { 0xc2ce3e68, SUC_lexicon_tags_2 },
    { 0x324e3e6a, SUC_lexicon_tags_14 },
    { 0xc0e3e6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0e3e6e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xccae3e70, SUC_lexicon_tags_10 },
    { 0x6f6e3e71, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x484e3e73, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440e3e7b, SUC_lexicon_tags_56 },
    { 0xd8ee3e7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50e3e81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ece3e83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18e3e85, SUC_lexicon_tags_1 },
    { 0xf44e3e86, SUC_lexicon_tags_60 },
    { 0xd3ee3e87, SUC_lexicon_tags_47 },
    { 0x176e3e88, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e3e8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ce3e8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecee3e93, SUC_lexicon_tags_65 },
    { 0xa16e3e94, SUC_lexicon_tags_60 },
    { 0x580e3e94, SUC_lexicon_tags_17 },
    { 0xda2e3e95, SUC_lexicon_tags_27 },
    { 0xbb2e3e97, SUC_lexicon_tags_63 },
    { 0xf2ee3e94, SUC_lexicon_tags_15 },
    { 0x5e6e3e99, SUC_lexicon_tags_23 },
    { 0x98e3e99, SUC_lexicon_tags_8 },
    { 0x53ae3e99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fce3ea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51ce3ea2, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ae3ea5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd6e3ea7, SUC_lexicon_tags_141 },
    { 0x776e3ea8, SUC_lexicon_tags_28 },
    { 0xe58e3ea9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8e3eaf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76e3eb2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x434e3eb4, SUC_lexicon_tags_37 },
    { 0xdbce3eb5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ce3eb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x636e3ebb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd82e3ebe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14e3ec0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8e3ec4, SUC_lexicon_tags_15 },
    { 0x7b4e3ec5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dce3eca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf0ee3ecc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13ee3ece, SUC_lexicon_tags_60 },
    { 0x9b0e3ecf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dce3ed3, SUC_lexicon_tags_7 },
    { 0x2dae3ed4, SUC_lexicon_tags_7 },
    { 0x6e6e3ed4, SUC_lexicon_tags_9 },
    { 0xd4e3ed6, SUC_lexicon_tags_12 },
    { 0x96ae3ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x122e3ed9, SUC_lexicon_tags_0 },
    { 0xb58e3ed9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x542e3edd, SUC_lexicon_tags_30 },
    { 0xd8ce3ede, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08e3ee4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180e3ee9, SUC_lexicon_tags_49 },
    { 0xc1ce3eea, SUC_lexicon_tags_77 },
    { 0x11ee3eea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cee3eed, SUC_lexicon_tags_11 },
    { 0xe66e3eed, SUC_lexicon_tags_1 },
    { 0x6b6e3eef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e3ef8, SUC_lexicon_tags_60 },
    { 0xdeae3ef8, SUC_lexicon_tags_49 },
    { 0xab4e3ef8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf64e3efc, SUC_lexicon_tags_43 },
    { 0xdc2e3efd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6e3f03, SUC_lexicon_tags_7 },
    { 0xc4ae3f04, SUC_lexicon_tags_4 },
    { 0x460e3f04, SUC_lexicon_tags_38 },
    { 0x560e3f06, SUC_lexicon_tags_7 },
    { 0x32ee3f06, SUC_lexicon_tags_31 },
    { 0xc00e3f08, SUC_lexicon_tags_23 },
    { 0x7cce3f09, SUC_lexicon_tags_64 },
    { 0xd34e3f09, SUC_lexicon_tags_31 },
    { 0x29ae3f0b, SUC_lexicon_tags_23 },
    { 0xbace3f0c, SUC_lexicon_tags_18 },
    { 0x4d2e3f0d, SUC_lexicon_tags_14 },
    { 0x694e3f0e, SUC_lexicon_tags_17 },
    { 0x682e3f06, SUC_lexicon_tags_60 },
    { 0xfb6e3f0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632e3f19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ae3f1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x450e3f20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xda8e3f22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffae3f2c, SUC_lexicon_tags_37 },
    { 0xface3f2d, SUC_lexicon_tags_17 },
    { 0x85ce3f2d, SUC_lexicon_tags_11 },
    { 0x2a2e3f2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6aee3f31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18e3f34, SUC_lexicon_tags_63 },
    { 0x414e3f34, SUC_lexicon_tags_15 },
    { 0x8aae3f36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ae3f3b, SUC_lexicon_tags_43 },
    { 0x2f8e3f3b, SUC_lexicon_tags_25 },
    { 0x4a8e3f3c, SUC_lexicon_tags_14 },
    { 0x820e3f3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92ee3f40, SUC_lexicon_tags_32 },
    { 0x990e3f41, SUC_lexicon_tags_3 },
    { 0x2e6e3f41, SUC_lexicon_tags_1 },
    { 0x606e3f43, SUC_lexicon_tags_63 },
    { 0x138e3f44, SUC_lexicon_tags_25 },
    { 0xa6ee3f43, SUC_lexicon_tags_11 },
    { 0xe4ee3f46, SUC_lexicon_tags_11 },
    { 0xb34e3f46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x138e3f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22e3f4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e4e3f51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cee3f53, SUC_lexicon_tags_2 },
    { 0xe8ce3f54, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ae3f59, SUC_lexicon_tags_7 },
    { 0x1a8e3f5a, SUC_lexicon_tags_0 },
    { 0x6eee3f5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4e3f5f, SUC_lexicon_tags_2 },
    { 0x1b2e3f5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c6e3f62, SUC_lexicon_tags_15 },
    { 0x59ce3f63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cce3f6d, SUC_lexicon_tags_30 },
    { 0x3ae3f6e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfdee3f70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ae3f74, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc54e3f76, SUC_lexicon_tags_37 },
    { 0x5dae3f77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe4e3f79, SUC_lexicon_tags_8 },
    { 0x80e3f7a, SUC_lexicon_tags_1 },
    { 0xc02e3f7a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0e3f7e, SUC_lexicon_tags_43 },
    { 0xaeae3f7f, SUC_lexicon_tags_15 },
    { 0x6bce3f80, SUC_lexicon_tags_30 },
    { 0x3aae3f80, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xe6ee3f83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778e3f8d, SUC_lexicon_tags_136 },
    { 0xb1ee3f8e, SUC_lexicon_tags_7 },
    { 0xcfae3f8f, SUC_lexicon_tags_57 },
    { 0x35ee3f90, SUC_lexicon_tags_8 },
    { 0x3ae3f90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ce3f94, SUC_lexicon_tags_1 },
    { 0x3d2e3f94, SUC_lexicon_tags_10 },
    { 0x1f4e3f96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ace3f98, SUC_lexicon_tags_2 },
    { 0x1d2e3f98, SUC_lexicon_tags_11 },
    { 0x920e3f99, SUC_lexicon_tags_15 },
    { 0xa0ce3f9b, SUC_lexicon_tags_80 },
    { 0x776e3f9c, SUC_lexicon_tags_1 },
    { 0xb0ce3f9d, SUC_lexicon_tags_15 },
    { 0x6a8e3f9c, SUC_lexicon_tags_34 },
    { 0x9dae3f9f, SUC_lexicon_tags_2 },
    { 0xce2e3f9b, SUC_lexicon_tags_1 },
    { 0xd56e3f9d, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x404e3fa4, SUC_lexicon_tags_12 },
    { 0xff6e3fa4, SUC_lexicon_tags_6 },
    { 0x954e3fa6, SUC_lexicon_tags_11 },
    { 0x9c2e3fa6, SUC_lexicon_tags_25 },
    { 0x9eae3fa8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ee3faf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76e3fb3, SUC_lexicon_tags_4 },
    { 0x71ce3fb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40e3fb7, SUC_lexicon_tags_43 },
    { 0x8fae3fb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aae3fbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ee3fc0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x50ce3fc2, SUC_lexicon_tags_3 },
    { 0x41ee3fc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6ce3fc5, SUC_lexicon_tags_1 },
    { 0x156e3fc6, SUC_lexicon_tags_37 },
    { 0xaa0e3fc5, SUC_lexicon_tags_66 },
    { 0xe12e3fc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28e3fca, SUC_lexicon_tags_7 },
    { 0xd54e3fcb, SUC_lexicon_tags_8 },
    { 0x31ee3fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbae3fce, SUC_lexicon_tags_11 },
    { 0xb62e3fcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728e3fd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd76e3fd5, SUC_lexicon_tags_12 },
    { 0x5b0e3fd5, SUC_lexicon_tags_11 },
    { 0x79ee3fd5, SUC_lexicon_tags_17 },
    { 0xf0ce3fd6, SUC_lexicon_tags_26 },
    { 0x682e3fd9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ee3fdc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cae3fe1, SUC_lexicon_tags_27 },
    { 0xc76e3fe1, SUC_lexicon_tags_33 },
    { 0x86ae3fe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3ce3fe5, SUC_lexicon_tags_2 },
    { 0xdace3fe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0ce3fe8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd5ce3fea, SUC_lexicon_tags_17 },
    { 0x544e3fea, SUC_lexicon_tags_44 },
    { 0x2fce3fec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372e3ff0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x132e3ff2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1ace3ff4, SUC_lexicon_tags_47 },
    { 0x544e3ff5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0e3ffb, SUC_lexicon_tags_28 },
    { 0x61ee3ffc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68ce3ffe, SUC_lexicon_tags_11 },
    { 0xbc4e3fff, SUC_lexicon_tags_45 },
    { 0xd86e3fff, SUC_lexicon_tags_12 },
    { 0x82e4001, SUC_lexicon_tags_2 },
    { 0xa5ee4000, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cee4006, SUC_lexicon_tags_1 },
    { 0xbc8e4007, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6e400c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf14e400e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x7a4e4010, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564e4015, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57ae4017, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158e401b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a2e401d, SUC_lexicon_tags_84 },
    { 0xf28e401e, SUC_lexicon_tags_6 },
    { 0xab4e401e, SUC_lexicon_tags_65 },
    { 0xdb2e4020, SUC_lexicon_tags_6 },
    { 0xd84e4021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadee4026, SUC_lexicon_tags_11 },
    { 0x3fee4026, SUC_lexicon_tags_0 },
    { 0x41ce4028, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772e402d, SUC_lexicon_tags_6 },
    { 0xe64e402e, SUC_lexicon_tags_12 },
    { 0x87ae402f, SUC_lexicon_tags_25 },
    { 0x91ae4030, SUC_lexicon_tags_0 },
    { 0x306e4030, SUC_lexicon_tags_28 },
    { 0x442e4031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x872e4034, SUC_lexicon_tags_38 },
    { 0x4a2e4034, SUC_lexicon_tags_9 },
    { 0xe8e4034, SUC_lexicon_tags_3 },
    { 0xf84e4037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16e403a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f4e403c, SUC_lexicon_tags_3 },
    { 0xa90e403d, SUC_lexicon_tags_27 },
    { 0x2b6e403d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bce4048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506e404c, SUC_lexicon_tags_56 },
    { 0x16ae404d, SUC_lexicon_tags_1 },
    { 0x716e404e, SUC_lexicon_tags_15 },
    { 0x980e404f, SUC_lexicon_tags_11 },
    { 0x78ce404f, SUC_lexicon_tags_37 },
    { 0xfd0e4051, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ce4055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cae4059, SUC_lexicon_tags_49 },
    { 0x424e405a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0e405d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd02e405f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x60ee4061, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1a2e4063, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ee4068, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ae406b, SUC_lexicon_tags_98 },
    { 0x47ae406b, SUC_lexicon_tags_7 },
    { 0xe38e406c, SUC_lexicon_tags_10 },
    { 0x9b6e406e, SUC_lexicon_tags_1 },
    { 0x9a4e406f, SUC_lexicon_tags_1 },
    { 0xa8e4070, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xa58e4072, SUC_lexicon_tags_7 },
    { 0x212e4072, SUC_lexicon_tags_35 },
    { 0x8e0e4074, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ee4078, SUC_lexicon_tags_42 },
    { 0x734e4078, SUC_lexicon_tags_68 },
    { 0xef4e4079, SUC_lexicon_tags_25 },
    { 0x85ae407b, SUC_lexicon_tags_27 },
    { 0x4d0e407c, SUC_lexicon_tags_12 },
    { 0xa1ce407d, SUC_lexicon_tags_23 },
    { 0xebae407c, SUC_lexicon_tags_14 },
    { 0x7a0e407a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e4082, SUC_lexicon_tags_9 },
    { 0xe2ae4083, SUC_lexicon_tags_60 },
    { 0x7e2e4083, SUC_lexicon_tags_10 },
    { 0xf04e4083, SUC_lexicon_tags_47 },
    { 0x5f6e4086, SUC_lexicon_tags_9 },
    { 0x268e4087, SUC_lexicon_tags_12 },
    { 0x7dae4088, SUC_lexicon_tags_11 },
    { 0x45ee4087, SUC_lexicon_tags_1 },
    { 0xc52e408a, SUC_lexicon_tags_6 },
    { 0xfeee408b, SUC_lexicon_tags_21 },
    { 0x9e8e408a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ce4090, SUC_lexicon_tags_1 },
    { 0xab8e4091, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0e4094, SUC_lexicon_tags_37 },
    { 0x7cee4095, SUC_lexicon_tags_45 },
    { 0x46ae4096, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8e409c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68e40a3, SUC_lexicon_tags_30 },
    { 0x9b8e40a3, SUC_lexicon_tags_38 },
    { 0xd88e40a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd20e40a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x14ee40a9, SUC_lexicon_tags_25 },
    { 0x516e40a9, SUC_lexicon_tags_11 },
    { 0x6a2e40ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x190e40ad, SUC_lexicon_tags_45 },
    { 0x48ce40ad, SUC_lexicon_tags_15 },
    { 0xe48e40ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dae40b3, SUC_lexicon_tags_1 },
    { 0xe1ae40b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x754e40b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd8e40b8, SUC_lexicon_tags_7 },
    { 0x1f0e40b9, SUC_lexicon_tags_25 },
    { 0xa8ee40b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe34e40bc, SUC_lexicon_tags_29 },
    { 0x50ee40bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa30e40bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb4ce40c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbce40c4, SUC_lexicon_tags_47 },
    { 0x282e40c5, SUC_lexicon_tags_17 },
    { 0x368e40c6, SUC_lexicon_tags_21 },
    { 0xc8ce40c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6e40c9, SUC_lexicon_tags_11 },
    { 0xbbce40ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6e40cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ace40d2, SUC_lexicon_tags_17 },
    { 0xc68e40d3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x966e40d5, SUC_lexicon_tags_30 },
    { 0xe60e40d5, SUC_lexicon_tags_60 },
    { 0xf0ae40d7, SUC_lexicon_tags_38 },
    { 0xbdae40d8, SUC_lexicon_tags_25 },
    { 0x1ae40d8, SUC_lexicon_tags_1 },
    { 0x596e40da, SUC_lexicon_tags_33 },
    { 0x1a8e40db, SUC_lexicon_tags_11 },
    { 0x7f8e40db, SUC_lexicon_tags_69 },
    { 0x23ee40dd, SUC_lexicon_tags_26 },
    { 0xd2e40db, SUC_lexicon_tags_250 },
    { 0x5aee40df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbece40e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7fce40e3, SUC_lexicon_tags_11 },
    { 0x1c2e40e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686e40eb, SUC_lexicon_tags_900 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74e40ee, SUC_lexicon_tags_6 },
    { 0xdbee40ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd0ee40f1, SUC_lexicon_tags_11 },
    { 0xe5ce40f1, SUC_lexicon_tags_107 },
    { 0xbbce40f3, SUC_lexicon_tags_19 },
    { 0xa72e40f4, SUC_lexicon_tags_2 },
    { 0x8d4e40f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee40fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e410d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae4112, SUC_lexicon_tags_25 },
    { 0x79ae4113, SUC_lexicon_tags_2 },
    { 0x540e4114, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x498e4116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ace411a, SUC_lexicon_tags_25 },
    { 0x670e411b, SUC_lexicon_tags_23 },
    { 0xd48e411c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc2e411e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76e4122, SUC_lexicon_tags_12 },
    { 0x936e4123, SUC_lexicon_tags_15 },
    { 0x392e4124, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ce412b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ee4130, SUC_lexicon_tags_19 },
    { 0xf8ae4131, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x938e4133, SUC_lexicon_tags_12 },
    { 0xf18e4134, SUC_lexicon_tags_25 },
    { 0x994e4134, SUC_lexicon_tags_15 },
    { 0x8bce4133, SUC_lexicon_tags_19 },
    { 0x164e4133, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x39ae4139, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2e413e, SUC_lexicon_tags_63 },
    { 0xc26e413e, SUC_lexicon_tags_33 },
    { 0x91ee413e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ee4144, SUC_lexicon_tags_47 },
    { 0xc00e4145, SUC_lexicon_tags_37 },
    { 0xe58e4144, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x302e4149, SUC_lexicon_tags_2 },
    { 0xd36e414a, SUC_lexicon_tags_23 },
    { 0x868e414b, SUC_lexicon_tags_9 },
    { 0x136e414c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2e4155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e4e4157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2e415b, SUC_lexicon_tags_38 },
    { 0x2d2e415c, SUC_lexicon_tags_2 },
    { 0x53ce415d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x716e415f, SUC_lexicon_tags_17 },
    { 0xbd8e4160, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04e4163, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ae4167, SUC_lexicon_tags_37 },
    { 0x888e4167, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478e416d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832e4175, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda0e4177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888e4180, SUC_lexicon_tags_7 },
    { 0xb30e4181, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ae4185, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaae4188, SUC_lexicon_tags_25 },
    { 0xe4188, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x542e418b, SUC_lexicon_tags_1 },
    { 0x6cee418b, SUC_lexicon_tags_17 },
    { 0x344e418d, SUC_lexicon_tags_17 },
    { 0x820e418d, SUC_lexicon_tags_2 },
    { 0xd88e418c, SUC_lexicon_tags_33 },
    { 0xffee418c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d0e4192, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc3ae4194, SUC_lexicon_tags_30 },
    { 0x19ae4194, SUC_lexicon_tags_25 },
    { 0x7f2e4196, SUC_lexicon_tags_63 },
    { 0x5eae4197, SUC_lexicon_tags_65 },
    { 0xfb0e4198, SUC_lexicon_tags_11 },
    { 0x24ce4199, SUC_lexicon_tags_23 },
    { 0xeae419a, SUC_lexicon_tags_37 },
    { 0xe6e4198, SUC_lexicon_tags_14 },
    { 0x2ee4196, SUC_lexicon_tags_0 },
    { 0x174e419d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8e41a1, SUC_lexicon_tags_9 },
    { 0x4fee41a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc02e41a4, SUC_lexicon_tags_11 },
    { 0x428e41a4, SUC_lexicon_tags_7 },
    { 0xe6ae41a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x650e41a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b6e41ab, SUC_lexicon_tags_19 },
    { 0x7f6e41ab, SUC_lexicon_tags_44 },
    { 0xed8e41ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x644e41b0, SUC_lexicon_tags_37 },
    { 0x846e41b1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd76e41b3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ce41b9, SUC_lexicon_tags_43 },
    { 0x9a8e41ba, SUC_lexicon_tags_82 },
    { 0x682e41bb, SUC_lexicon_tags_12 },
    { 0x296e41bb, SUC_lexicon_tags_25 },
    { 0x890e41bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bce41c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4ee41c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796e41c8, SUC_lexicon_tags_23 },
    { 0xfd4e41c8, SUC_lexicon_tags_12 },
    { 0x234e41c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaf2e41cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90e41d0, SUC_lexicon_tags_47 },
    { 0x9f2e41d1, SUC_lexicon_tags_49 },
    { 0x1ee41d2, SUC_lexicon_tags_37 },
    { 0xbd4e41d2, SUC_lexicon_tags_113 },
    { 0x320e41d4, SUC_lexicon_tags_60 },
    { 0xa0ce41d5, SUC_lexicon_tags_17 },
    { 0x91ce41d4, SUC_lexicon_tags_9 },
    { 0x340e41d7, SUC_lexicon_tags_8 },
    { 0x8c0e41d3, SUC_lexicon_tags_146 },
    { 0xe40e41d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28e41dc, SUC_lexicon_tags_25 },
    { 0x174e41dd, SUC_lexicon_tags_25 },
    { 0xc6ae41dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4e41e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce41e8, SUC_lexicon_tags_19 },
    { 0x3fee41e9, SUC_lexicon_tags_17 },
    { 0xa8ee41e9, SUC_lexicon_tags_2 },
    { 0x936e41eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x642e41ee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420e41f2, SUC_lexicon_tags_68 },
    { 0x150e41f3, SUC_lexicon_tags_38 },
    { 0x6f8e41f4, SUC_lexicon_tags_27 },
    { 0x2ee41f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccee41fc, SUC_lexicon_tags_0 },
    { 0xa4ee41fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x120e41ff, SUC_lexicon_tags_47 },
    { 0x1f0e41ff, SUC_lexicon_tags_37 },
    { 0x506e41ff, SUC_lexicon_tags_16 },
    { 0x15ee4202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ae4209, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c0e420b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6e0e420d, SUC_lexicon_tags_68 },
    { 0xaece420e, SUC_lexicon_tags_38 },
    { 0xc9ee420f, SUC_lexicon_tags_68 },
    { 0x780e420d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3aae4212, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ce4218, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x12ae421a, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x866e421c, SUC_lexicon_tags_7 },
    { 0xa6ae421d, SUC_lexicon_tags_11 },
    { 0xa34e421d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430e4222, SUC_lexicon_tags_83 },
    { 0x62ce4222, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x620e4225, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ee422a, SUC_lexicon_tags_2 },
    { 0xeb2e422b, SUC_lexicon_tags_121 },
    { 0x2e4e422c, SUC_lexicon_tags_25 },
    { 0x36ce422d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x106e4230, SUC_lexicon_tags_15 },
    { 0x446e4231, SUC_lexicon_tags_1 },
    { 0x536e4231, SUC_lexicon_tags_29 },
    { 0x690e4232, SUC_lexicon_tags_7 },
    { 0x48ee4231, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78e4237, SUC_lexicon_tags_25 },
    { 0xb8ce4238, SUC_lexicon_tags_27 },
    { 0x72e4238, SUC_lexicon_tags_11 },
    { 0xee0e4238, SUC_lexicon_tags_63 },
    { 0xa7ce423b, SUC_lexicon_tags_14 },
    { 0x154e423c, SUC_lexicon_tags_33 },
    { 0xf2ae423d, SUC_lexicon_tags_26 },
    { 0xc5ce423e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24e4241, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf2e4243, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ee424c, SUC_lexicon_tags_55 },
    { 0x306e424d, SUC_lexicon_tags_13 },
    { 0x496e424e, SUC_lexicon_tags_15 },
    { 0xcf8e424c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38e425a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2e425e, SUC_lexicon_tags_37 },
    { 0x562e425f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ace4261, SUC_lexicon_tags_25 },
    { 0xb56e4262, SUC_lexicon_tags_14 },
    { 0xed4e4261, SUC_lexicon_tags_60 },
    { 0xd4ae4264, SUC_lexicon_tags_63 },
    { 0xfa2e4262, SUC_lexicon_tags_2 },
    { 0xfbae4261, SUC_lexicon_tags_12 },
    { 0x95ee4266, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4e426c, SUC_lexicon_tags_40 },
    { 0xcb8e426d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8e4271, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4e4278, SUC_lexicon_tags_7 },
    { 0xc60e4279, SUC_lexicon_tags_12 },
    { 0x57ce427a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e427d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54e427f, SUC_lexicon_tags_66 },
    { 0x850e4280, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x590e4283, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfee4285, SUC_lexicon_tags_11 },
    { 0xfa2e4285, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ce4293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ce4297, SUC_lexicon_tags_17 },
    { 0x176e4298, SUC_lexicon_tags_37 },
    { 0x370e4297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ce429c, SUC_lexicon_tags_2 },
    { 0x772e429d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2e42a1, SUC_lexicon_tags_10 },
    { 0x25ce42a2, SUC_lexicon_tags_17 },
    { 0x3fae42a3, SUC_lexicon_tags_54 },
    { 0xce6e42a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x38ae42a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x306e42a8, SUC_lexicon_tags_37 },
    { 0x768e42a9, SUC_lexicon_tags_46 },
    { 0xeb6e42a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96e42ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44ee42b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe08e42b2, SUC_lexicon_tags_11 },
    { 0x776e42b3, SUC_lexicon_tags_10 },
    { 0xa98e42b4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68e42b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcace42c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10e42c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926e42cc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ce42d2, SUC_lexicon_tags_114 },
    { 0xf1ce42d3, SUC_lexicon_tags_4 },
    { 0xbfee42d3, SUC_lexicon_tags_21 },
    { 0x55ee42d2, SUC_lexicon_tags_9 },
    { 0x9fee42d6, SUC_lexicon_tags_48 },
    { 0xfe2e42d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ee42da, SUC_lexicon_tags_45 },
    { 0x64ce42db, SUC_lexicon_tags_27 },
    { 0xb7ee42db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ce42e2, SUC_lexicon_tags_15 },
    { 0xe52e42e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9d0e42e5, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0e42e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ee42ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976e42f3, SUC_lexicon_tags_2 },
    { 0x97ae42f3, SUC_lexicon_tags_87 },
    { 0x270e42f5, SUC_lexicon_tags_28 },
    { 0x5a4e42f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32e42fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924e42fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aae4301, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x458e4304, SUC_lexicon_tags_1 },
    { 0xf34e4304, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aee4308, SUC_lexicon_tags_37 },
    { 0x754e4309, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bae4312, SUC_lexicon_tags_2 },
    { 0x516e4312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x906e4315, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f2e4317, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4e431b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x506e431d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8e4320, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2e4327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8e4330, SUC_lexicon_tags_12 },
    { 0x732e4330, SUC_lexicon_tags_33 },
    { 0xffce4330, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ce4335, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bae433c, SUC_lexicon_tags_15 },
    { 0x40ee433d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebae4342, SUC_lexicon_tags_37 },
    { 0xebae4342, SUC_lexicon_tags_77 },
    { 0x820e4343, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cae4347, SUC_lexicon_tags_12 },
    { 0x8ece4348, SUC_lexicon_tags_35 },
    { 0x8b6e4349, SUC_lexicon_tags_23 },
    { 0x780e4349, SUC_lexicon_tags_37 },
    { 0xeb2e434b, SUC_lexicon_tags_48 },
    { 0xb54e434a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8e4350, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6e4353, SUC_lexicon_tags_31 },
    { 0xb5ce4353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9ae4356, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56e435a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x450e435d, SUC_lexicon_tags_38 },
    { 0xc2ce435e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ce4361, SUC_lexicon_tags_37 },
    { 0xb6ee4362, SUC_lexicon_tags_37 },
    { 0x836e4361, SUC_lexicon_tags_28 },
    { 0xc24e4364, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0e436a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbae436f, SUC_lexicon_tags_7 },
    { 0x2ee436f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a6e4372, SUC_lexicon_tags_1 },
    { 0x114e4372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94e437d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448e4381, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ae4387, SUC_lexicon_tags_19 },
    { 0x682e4388, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962e438d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5bee438f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x532e4391, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276e4395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320e43a0, SUC_lexicon_tags_37 },
    { 0x1bee43a0, SUC_lexicon_tags_6 },
    { 0x52ce43a0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ae43a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b4e43a7, SUC_lexicon_tags_33 },
    { 0xa72e43a8, SUC_lexicon_tags_15 },
    { 0x24ce43a8, SUC_lexicon_tags_33 },
    { 0x412e43aa, SUC_lexicon_tags_23 },
    { 0xb9ee43ab, SUC_lexicon_tags_25 },
    { 0x8bee43aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7ae43ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bee43b1, SUC_lexicon_tags_23 },
    { 0xe4ee43b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ce43b7, SUC_lexicon_tags_2 },
    { 0x2b2e43b8, SUC_lexicon_tags_17 },
    { 0xe18e43b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80e43bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ae43c0, SUC_lexicon_tags_2 },
    { 0xeeee43c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e43c7, SUC_lexicon_tags_1 },
    { 0x19ae43c8, SUC_lexicon_tags_56 },
    { 0xc1ae43c8, SUC_lexicon_tags_16 },
    { 0xee8e43ca, SUC_lexicon_tags_0 },
    { 0xafae43ca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1fee43cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6e43d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2ce43d4, SUC_lexicon_tags_30 },
    { 0x1a4e43d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48e43df, SUC_lexicon_tags_4 },
    { 0xa6ce43e0, SUC_lexicon_tags_33 },
    { 0xfc6e43e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ee43ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2e43ed, SUC_lexicon_tags_7 },
    { 0xbe8e43ee, SUC_lexicon_tags_55 },
    { 0xc32e43ef, SUC_lexicon_tags_7 },
    { 0x6ece43f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190e43f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e43f9, SUC_lexicon_tags_55 },
    { 0xd02e43fa, SUC_lexicon_tags_14 },
    { 0x26ee43fb, SUC_lexicon_tags_19 },
    { 0x4e43fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00e43ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaee4401, SUC_lexicon_tags_23 },
    { 0xa6e4401, SUC_lexicon_tags_3 },
    { 0xd6ae4403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x150e4406, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x450e4409, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838e440e, SUC_lexicon_tags_7 },
    { 0x33ae440f, SUC_lexicon_tags_39 },
    { 0x8a6e440e, SUC_lexicon_tags_1 },
    { 0x7f8e4411, SUC_lexicon_tags_71 },
    { 0x3d8e4412, SUC_lexicon_tags_12 },
    { 0x1cce4413, SUC_lexicon_tags_10 },
    { 0x9d6e4414, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb1ce4416, SUC_lexicon_tags_8 },
    { 0x3b0e4417, SUC_lexicon_tags_1 },
    { 0x976e4417, SUC_lexicon_tags_37 },
    { 0xc0ce4419, SUC_lexicon_tags_43 },
    { 0xc5ae4416, SUC_lexicon_tags_17 },
    { 0x43ae441b, SUC_lexicon_tags_2 },
    { 0x662e4417, SUC_lexicon_tags_31 },
    { 0x930e4416, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e4421, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ee4424, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0e442a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dce4433, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecce4437, SUC_lexicon_tags_33 },
    { 0x3b8e4438, SUC_lexicon_tags_14 },
    { 0x30ce4439, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdee443b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cee443e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ee4445, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae4449, SUC_lexicon_tags_2 },
    { 0x102e444a, SUC_lexicon_tags_28 },
    { 0x24ae444a, SUC_lexicon_tags_47 },
    { 0x818e444a, SUC_lexicon_tags_66 },
    { 0x2cee4449, SUC_lexicon_tags_17 },
    { 0xd0ce444e, SUC_lexicon_tags_12 },
    { 0x2d0e444e, SUC_lexicon_tags_31 },
    { 0x342e444f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8a6e4452, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0e4455, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e445e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x936e4461, SUC_lexicon_tags_29 },
    { 0x59ae4462, SUC_lexicon_tags_7 },
    { 0xa3ee4463, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0e446a, SUC_lexicon_tags_47 },
    { 0x578e446b, SUC_lexicon_tags_43 },
    { 0xce2e446a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2e4470, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x512e4473, SUC_lexicon_tags_17 },
    { 0xa96e4474, SUC_lexicon_tags_12 },
    { 0xabce4475, SUC_lexicon_tags_31 },
    { 0xe12e4476, SUC_lexicon_tags_30 },
    { 0x13ee4474, SUC_lexicon_tags_48 },
    { 0xa32e4477, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x806e447b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ae448a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce448f, SUC_lexicon_tags_11 },
    { 0x6ee4490, SUC_lexicon_tags_53 },
    { 0x202e4490, SUC_lexicon_tags_7 },
    { 0x204e4490, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcee4496, SUC_lexicon_tags_38 },
    { 0x47ce4497, SUC_lexicon_tags_30 },
    { 0xce4498, SUC_lexicon_tags_12 },
    { 0x2dee4499, SUC_lexicon_tags_1 },
    { 0x87ee449a, SUC_lexicon_tags_27 },
    { 0x136e449b, SUC_lexicon_tags_27 },
    { 0xfce449b, SUC_lexicon_tags_79 },
    { 0x79ee449d, SUC_lexicon_tags_107 },
    { 0xda6e449e, SUC_lexicon_tags_7 },
    { 0x226e449f, SUC_lexicon_tags_9 },
    { 0xe2ae449c, SUC_lexicon_tags_25 },
    { 0x176e449c, SUC_lexicon_tags_37 },
    { 0x166e44a0, SUC_lexicon_tags_29 },
    { 0x3e6e4498, SUC_lexicon_tags_33 },
    { 0x66ee44a4, SUC_lexicon_tags_15 },
    { 0x984e44a4, SUC_lexicon_tags_13 },
    { 0x606e449e, SUC_lexicon_tags_40 },
    { 0x52ee44a7, SUC_lexicon_tags_46 },
    { 0xf00e44a8, SUC_lexicon_tags_11 },
    { 0x22ae44a9, SUC_lexicon_tags_0 },
    { 0xbdae44aa, SUC_lexicon_tags_22 },
    { 0xe94e44a9, SUC_lexicon_tags_1 },
    { 0x6cce449a, SUC_lexicon_tags_25 },
    { 0xabee4497, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e44b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6e44b5, SUC_lexicon_tags_37 },
    { 0xb92e44b5, SUC_lexicon_tags_2 },
    { 0x23ae44b6, SUC_lexicon_tags_63 },
    { 0x132e44b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22e44bc, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xf60e44be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x78e44c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ee44c4, SUC_lexicon_tags_60 },
    { 0xd4e44c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eee44c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64e44cc, SUC_lexicon_tags_32 },
    { 0xd2ae44cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28e44d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6e44d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0e44e2, SUC_lexicon_tags_1 },
    { 0x4aee44e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cce44e7, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42e44ed, SUC_lexicon_tags_2 },
    { 0xa2ee44ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae44f6, SUC_lexicon_tags_31 },
    { 0xbce44f7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e44fc, SUC_lexicon_tags_45 },
    { 0x8e2e44fc, SUC_lexicon_tags_4 },
    { 0x222e44fe, SUC_lexicon_tags_0 },
    { 0x3f6e44fe, SUC_lexicon_tags_15 },
    { 0x2eae4500, SUC_lexicon_tags_25 },
    { 0xb4ae44ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x30ce4503, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xf20e4505, SUC_lexicon_tags_37 },
    { 0xceee4505, SUC_lexicon_tags_87 },
    { 0xfb4e4507, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2e450c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78e450f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ee4512, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2e4518, SUC_lexicon_tags_44 },
    { 0xa4e4519, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ae451c, SUC_lexicon_tags_63 },
    { 0x59ee451c, SUC_lexicon_tags_25 },
    { 0x588e451e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12e4528, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ce452c, SUC_lexicon_tags_27 },
    { 0x8c6e452d, SUC_lexicon_tags_32 },
    { 0xc0ce452d, SUC_lexicon_tags_7 },
    { 0x9bee452f, SUC_lexicon_tags_39 },
    { 0xc7ce452f, SUC_lexicon_tags_27 },
    { 0xf30e4531, SUC_lexicon_tags_19 },
    { 0x7a2e452d, SUC_lexicon_tags_17 },
    { 0x53ce452c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668e4537, SUC_lexicon_tags_14 },
    { 0xf8ae4538, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ce453c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ee453f, SUC_lexicon_tags_17 },
    { 0xe1ae4540, SUC_lexicon_tags_49 },
    { 0x5bce4541, SUC_lexicon_tags_136 },
    { 0x16ae4542, SUC_lexicon_tags_0 },
    { 0xc86e4543, SUC_lexicon_tags_65 },
    { 0x400e4543, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x434e4547, SUC_lexicon_tags_37 },
    { 0xc90e4547, SUC_lexicon_tags_56 },
    { 0x31ee4549, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ce454d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x132e4550, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828e4554, SUC_lexicon_tags_31 },
    { 0xff6e4555, SUC_lexicon_tags_2 },
    { 0xb0ee4554, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd58e4558, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ee455b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e4560, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdae4565, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcce4569, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x634e456b, SUC_lexicon_tags_37 },
    { 0x69ae456c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23ce456e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x32ee4570, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4e4573, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258e4577, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0e457b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bae4580, SUC_lexicon_tags_25 },
    { 0x2dce4581, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66e4589, SUC_lexicon_tags_68 },
    { 0x52e458a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4e458e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436e4592, SUC_lexicon_tags_32 },
    { 0x8eee4593, SUC_lexicon_tags_14 },
    { 0x4b8e4594, SUC_lexicon_tags_15 },
    { 0x9d8e4595, SUC_lexicon_tags_0 },
    { 0xc44e4595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4e459c, SUC_lexicon_tags_17 },
    { 0x8fce459d, SUC_lexicon_tags_87 },
    { 0xda0e459e, SUC_lexicon_tags_26 },
    { 0xa86e459f, SUC_lexicon_tags_17 },
    { 0x1d4e459e, SUC_lexicon_tags_2 },
    { 0x2bce45a0, SUC_lexicon_tags_2 },
    { 0xf26e459f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x276e45a4, SUC_lexicon_tags_1170 },
    { 0x368e45a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96e45aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2e45b2, SUC_lexicon_tags_38 },
    { 0x48ce45b3, SUC_lexicon_tags_136 },
    { 0xe98e45b4, SUC_lexicon_tags_62 },
    { 0x32e45b5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8e45ba, SUC_lexicon_tags_25 },
    { 0xc54e45bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ee45c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce45cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788e45d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcae45dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ae45df, SUC_lexicon_tags_12 },
    { 0xa42e45df, SUC_lexicon_tags_25 },
    { 0x5b6e45e1, SUC_lexicon_tags_12 },
    { 0x1d4e45e2, SUC_lexicon_tags_38 },
    { 0xe0e45e2, SUC_lexicon_tags_23 },
    { 0x34ae45e0, SUC_lexicon_tags_37 },
    { 0x544e45e5, SUC_lexicon_tags_26 },
    { 0xc46e45e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e45f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ee45f5, SUC_lexicon_tags_25 },
    { 0x45ee45f6, SUC_lexicon_tags_23 },
    { 0xc5ce45f7, SUC_lexicon_tags_12 },
    { 0xa44e45f8, SUC_lexicon_tags_15 },
    { 0x4a6e45f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4600, SUC_lexicon_tags_2 },
    { 0x3c0e4601, SUC_lexicon_tags_35 },
    { 0xffce4602, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee4608, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94e460b, SUC_lexicon_tags_2 },
    { 0x9d0e460c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7e8e460e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x128e4611, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2e4615, SUC_lexicon_tags_37 },
    { 0xb6ee4616, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724e4620, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb02e4622, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x318e4624, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24e4629, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0e462d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916e4631, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x18ce4633, SUC_lexicon_tags_60 },
    { 0xcc0e4634, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fee4639, SUC_lexicon_tags_1 },
    { 0xf7ee463a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x722e463c, SUC_lexicon_tags_66 },
    { 0x82ce463d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfece4643, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xabce4646, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4ae4648, SUC_lexicon_tags_61 },
    { 0xe96e4649, SUC_lexicon_tags_47 },
    { 0x8d8e464a, SUC_lexicon_tags_7 },
    { 0xa86e4649, SUC_lexicon_tags_43 },
    { 0xa4ae464c, SUC_lexicon_tags_8 },
    { 0x456e464d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68e4654, SUC_lexicon_tags_29 },
    { 0x5ee4655, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x724e4657, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90e465a, SUC_lexicon_tags_11 },
    { 0x832e465b, SUC_lexicon_tags_12 },
    { 0xd0e465c, SUC_lexicon_tags_12 },
    { 0x880e465a, SUC_lexicon_tags_15 },
    { 0x5b6e465e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x5eee4660, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ce4667, SUC_lexicon_tags_310 },
    { 0x4f4e4667, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbdee466a, SUC_lexicon_tags_7 },
    { 0x75ee466a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704e4670, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e4673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6e4676, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b8e4678, SUC_lexicon_tags_25 },
    { 0xc0e4679, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ae467c, SUC_lexicon_tags_8 },
    { 0x2a6e467d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2e4680, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c2e4682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eee4689, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6e468d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaf0e468f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ce4692, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e4696, SUC_lexicon_tags_6 },
    { 0x426e4697, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ce469a, SUC_lexicon_tags_12 },
    { 0x52ae469b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb6e469d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ce46a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e46a2, SUC_lexicon_tags_11 },
    { 0x9d2e46a2, SUC_lexicon_tags_63 },
    { 0xf24e46a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5a4e46a6, SUC_lexicon_tags_55 },
    { 0xce6e46a7, SUC_lexicon_tags_25 },
    { 0x64ee46a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x784e46aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00e46ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d0e46b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x97ae46b2, SUC_lexicon_tags_7 },
    { 0xc02e46b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466e46ba, SUC_lexicon_tags_25 },
    { 0xdf0e46ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778e46c1, SUC_lexicon_tags_1 },
    { 0xc06e46c2, SUC_lexicon_tags_7 },
    { 0xf50e46c3, SUC_lexicon_tags_95 },
    { 0xe50e46c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa18e46c6, SUC_lexicon_tags_2 },
    { 0x33ce46c7, SUC_lexicon_tags_2 },
    { 0xfa2e46c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x644e46cb, SUC_lexicon_tags_2 },
    { 0x8eae46cb, SUC_lexicon_tags_6 },
    { 0x50ae46cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1e6e46cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692e46d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aae46df, SUC_lexicon_tags_44 },
    { 0x87ae46e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8e46e3, SUC_lexicon_tags_37 },
    { 0x318e46e3, SUC_lexicon_tags_37 },
    { 0x94ae46e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x90e46e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x266e46ea, SUC_lexicon_tags_1 },
    { 0xd54e46ea, SUC_lexicon_tags_1 },
    { 0x328e46ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502e46f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x402e46f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e6e46f4, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xd54e46f6, SUC_lexicon_tags_38 },
    { 0xfb4e46f6, SUC_lexicon_tags_12 },
    { 0x4b4e46f7, SUC_lexicon_tags_68 },
    { 0x1f8e46f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6e46ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2e4703, SUC_lexicon_tags_37 },
    { 0x75ae4703, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8b6e4706, SUC_lexicon_tags_14 },
    { 0x862e4707, SUC_lexicon_tags_11 },
    { 0x10e4707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ee470a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc8ae470c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe82e470e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x154e4710, SUC_lexicon_tags_2 },
    { 0x334e4711, SUC_lexicon_tags_47 },
    { 0x7e2e4711, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xabae4715, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ce4719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x230e471b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0e471e, SUC_lexicon_tags_37 },
    { 0x948e471f, SUC_lexicon_tags_9 },
    { 0x974e471e, SUC_lexicon_tags_15 },
    { 0x8ce4720, SUC_lexicon_tags_1 },
    { 0x460e4721, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb2e4724, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4e4727, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae472c, SUC_lexicon_tags_6 },
    { 0x310e472d, SUC_lexicon_tags_17 },
    { 0x6d0e472e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4e4733, SUC_lexicon_tags_7 },
    { 0x2b4e4734, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdae473b, SUC_lexicon_tags_18 },
    { 0xc4ae473c, SUC_lexicon_tags_68 },
    { 0xc7ae473c, SUC_lexicon_tags_63 },
    { 0x918e473d, SUC_lexicon_tags_27 },
    { 0xc1ce473f, SUC_lexicon_tags_7 },
    { 0x9a4e4740, SUC_lexicon_tags_11 },
    { 0x438e4741, SUC_lexicon_tags_23 },
    { 0xa7ae473e, SUC_lexicon_tags_85 },
    { 0xfcce4743, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ce4746, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce4749, SUC_lexicon_tags_14 },
    { 0xb18e474a, SUC_lexicon_tags_34 },
    { 0xd02e474a, SUC_lexicon_tags_47 },
    { 0x77ae474a, SUC_lexicon_tags_69 },
    { 0x6e0e474d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xedae474f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x81ee4751, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4e4754, SUC_lexicon_tags_29 },
    { 0x8c8e4754, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6e475a, SUC_lexicon_tags_34 },
    { 0x82e475b, SUC_lexicon_tags_7 },
    { 0xdb4e475c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164e4760, SUC_lexicon_tags_80 },
    { 0x226e4761, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2ce4763, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ee4769, SUC_lexicon_tags_2 },
    { 0x818e4769, SUC_lexicon_tags_11 },
    { 0x40ce4769, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ee4770, SUC_lexicon_tags_8 },
    { 0xa92e4771, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x136e4773, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594e477f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb98e4781, SUC_lexicon_tags_60 },
    { 0x3fae4781, SUC_lexicon_tags_23 },
    { 0x88ee4783, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56ee4785, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ce4789, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ce478c, SUC_lexicon_tags_19 },
    { 0x518e478d, SUC_lexicon_tags_46 },
    { 0x960e478e, SUC_lexicon_tags_17 },
    { 0xbcae478f, SUC_lexicon_tags_23 },
    { 0x7dce478c, SUC_lexicon_tags_14 },
    { 0xc98e478e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb80e4793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x466e4795, SUC_lexicon_tags_45 },
    { 0xcc2e4795, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1d0e4798, SUC_lexicon_tags_63 },
    { 0x88ee4799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15ee479b, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfce479f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576e47a4, SUC_lexicon_tags_7 },
    { 0x47ae47a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964e47a9, SUC_lexicon_tags_11 },
    { 0x5f0e47aa, SUC_lexicon_tags_11 },
    { 0xe28e47aa, SUC_lexicon_tags_25 },
    { 0xdbee47ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x450e47ae, SUC_lexicon_tags_17 },
    { 0x7c8e47ae, SUC_lexicon_tags_25 },
    { 0x54e47b0, SUC_lexicon_tags_15 },
    { 0xc44e47b1, SUC_lexicon_tags_1 },
    { 0xd06e47ae, SUC_lexicon_tags_11 },
    { 0xdfae47b3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa18e47b5, SUC_lexicon_tags_104 },
    { 0x836e47b5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ee47bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccae47be, SUC_lexicon_tags_14 },
    { 0xa0ee47bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ee47c2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4e47c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ce47cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ae47d9, SUC_lexicon_tags_25 },
    { 0x6aee47da, SUC_lexicon_tags_15 },
    { 0x710e47db, SUC_lexicon_tags_62 },
    { 0x756e47dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ee47e0, SUC_lexicon_tags_47 },
    { 0x2e6e47e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x268e47e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x908e47e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dae47e9, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ee47f0, SUC_lexicon_tags_31 },
    { 0x36e47f1, SUC_lexicon_tags_33 },
    { 0xb6ee47f2, SUC_lexicon_tags_11 },
    { 0xd94e47f3, SUC_lexicon_tags_11 },
    { 0x94ce47f4, SUC_lexicon_tags_35 },
    { 0x806e47f5, SUC_lexicon_tags_34 },
    { 0x846e47f2, SUC_lexicon_tags_34 },
    { 0x784e47f7, SUC_lexicon_tags_11 },
    { 0xd06e47f8, SUC_lexicon_tags_27 },
    { 0x332e47f2, SUC_lexicon_tags_11 },
    { 0xd56e47fa, SUC_lexicon_tags_37 },
    { 0x28e47fa, SUC_lexicon_tags_37 },
    { 0x5e2e47fc, SUC_lexicon_tags_25 },
    { 0xf02e47fd, SUC_lexicon_tags_17 },
    { 0xaee47fb, SUC_lexicon_tags_25 },
    { 0x4a6e47f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22e4803, SUC_lexicon_tags_9 },
    { 0xaf4e4804, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2a0e4806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x624e4808, SUC_lexicon_tags_17 },
    { 0x89ee4808, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x176e480c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x678e480e, SUC_lexicon_tags_29 },
    { 0x5d0e480e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x426e4811, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcee4819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x298e481c, SUC_lexicon_tags_64 },
    { 0xbfae481c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdee4820, SUC_lexicon_tags_37 },
    { 0x220e4821, SUC_lexicon_tags_37 },
    { 0x446e4820, SUC_lexicon_tags_45 },
    { 0xf30e4820, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8e4826, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fce4829, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee482e, SUC_lexicon_tags_38 },
    { 0xe12e482e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ce4835, SUC_lexicon_tags_46 },
    { 0xbb6e4836, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce8e4838, SUC_lexicon_tags_37 },
    { 0x8b8e4839, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558e483f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e4847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24e484e, SUC_lexicon_tags_11 },
    { 0xd76e484f, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x17ce4851, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264e4859, SUC_lexicon_tags_7 },
    { 0x1bae4859, SUC_lexicon_tags_60 },
    { 0x62ae485b, SUC_lexicon_tags_8 },
    { 0x514e485a, SUC_lexicon_tags_158 },
    { 0x838e485d, SUC_lexicon_tags_32 },
    { 0x1a0e485c, SUC_lexicon_tags_29 },
    { 0x81ee485a, SUC_lexicon_tags_21 },
    { 0x574e485e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ace4867, SUC_lexicon_tags_43 },
    { 0x89ee4868, SUC_lexicon_tags_1 },
    { 0x4a6e4869, SUC_lexicon_tags_15 },
    { 0x848e4869, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dae4872, SUC_lexicon_tags_37 },
    { 0x974e4873, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bce4876, SUC_lexicon_tags_87 },
    { 0x6e4876, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbcee4879, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ae487e, SUC_lexicon_tags_11 },
    { 0x79ce487f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64e4883, SUC_lexicon_tags_9 },
    { 0xaace4884, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x672e4886, SUC_lexicon_tags_25 },
    { 0x58ae4887, SUC_lexicon_tags_44 },
    { 0x23ee4888, SUC_lexicon_tags_17 },
    { 0xf04e4886, SUC_lexicon_tags_25 },
    { 0x8ee488a, SUC_lexicon_tags_37 },
    { 0x9a0e488b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa56e488d, SUC_lexicon_tags_17 },
    { 0x922e488d, SUC_lexicon_tags_25 },
    { 0x398e488f, SUC_lexicon_tags_16 },
    { 0xace488e, SUC_lexicon_tags_11 },
    { 0x878e4891, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ee489a, SUC_lexicon_tags_106 },
    { 0x93ae489b, SUC_lexicon_tags_46 },
    { 0x862e489c, SUC_lexicon_tags_48 },
    { 0x994e489c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98e48a2, SUC_lexicon_tags_15 },
    { 0xd86e48a3, SUC_lexicon_tags_28 },
    { 0x3cae48a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ce48a9, SUC_lexicon_tags_38 },
    { 0x5bce48aa, SUC_lexicon_tags_60 },
    { 0xcc2e48ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bce48b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa94e48b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134e48b6, SUC_lexicon_tags_11 },
    { 0xf10e48b7, SUC_lexicon_tags_1 },
    { 0x12ae48b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426e48c1, SUC_lexicon_tags_23 },
    { 0x3b8e48c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ce48c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ee48c9, SUC_lexicon_tags_28 },
    { 0x79ce48ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0e48cd, SUC_lexicon_tags_3 },
    { 0xd66e48ce, SUC_lexicon_tags_17 },
    { 0xd36e48cf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc04e48d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xefae48d3, SUC_lexicon_tags_37 },
    { 0xf90e48d4, SUC_lexicon_tags_30 },
    { 0x780e48d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd58e48d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaee48dd, SUC_lexicon_tags_1 },
    { 0xe18e48dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ee48e1, SUC_lexicon_tags_37 },
    { 0x432e48e2, SUC_lexicon_tags_12 },
    { 0xc04e48e3, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bce48e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x252e48ea, SUC_lexicon_tags_60 },
    { 0xe6ee48eb, SUC_lexicon_tags_2 },
    { 0x1a2e48ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce48f1, SUC_lexicon_tags_0 },
    { 0xce4e48f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ce48f5, SUC_lexicon_tags_38 },
    { 0xb76e48f6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7cee48f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8e48fc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x580e4902, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454e490a, SUC_lexicon_tags_43 },
    { 0x3b8e490a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96e490e, SUC_lexicon_tags_45 },
    { 0x60ce490f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4e491b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x35ee491d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62e4921, SUC_lexicon_tags_17 },
    { 0x534e4921, SUC_lexicon_tags_12 },
    { 0x7e6e4923, SUC_lexicon_tags_28 },
    { 0x4f4e4923, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54e4928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790e4933, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce4938, SUC_lexicon_tags_11 },
    { 0xb56e4939, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42e493d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0e4941, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bce4945, SUC_lexicon_tags_235 },
    { 0x8ee4946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x716e4948, SUC_lexicon_tags_39 },
    { 0x11ee4949, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x73ee494b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ae4950, SUC_lexicon_tags_17 },
    { 0xfdce4950, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e4955, SUC_lexicon_tags_47 },
    { 0x26ee4956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56e495e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6e4962, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ee4966, SUC_lexicon_tags_17 },
    { 0xbd0e4967, SUC_lexicon_tags_5 },
    { 0x796e4968, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78e4971, SUC_lexicon_tags_48 },
    { 0x324e4972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0e4976, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8e4979, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0e497d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ce4984, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8e4987, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e498a, SUC_lexicon_tags_7 },
    { 0x1f2e498b, SUC_lexicon_tags_10 },
    { 0x100e498a, SUC_lexicon_tags_0 },
    { 0xb02e498d, SUC_lexicon_tags_1 },
    { 0x19ae498e, SUC_lexicon_tags_63 },
    { 0xafee498c, SUC_lexicon_tags_43 },
    { 0x4e4990, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314e499a, SUC_lexicon_tags_23 },
    { 0xa52e499a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254e499f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c8e49a1, SUC_lexicon_tags_16 },
    { 0xaae49a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x384e49a5, SUC_lexicon_tags_45 },
    { 0xda4e49a6, SUC_lexicon_tags_83 },
    { 0x638e49a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46ce49a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ae49ae, SUC_lexicon_tags_106 },
    { 0x40e49af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576e49b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8e49b7, SUC_lexicon_tags_14 },
    { 0x684e49b8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e49bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ce49be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64e49c2, SUC_lexicon_tags_12 },
    { 0xb88e49c2, SUC_lexicon_tags_27 },
    { 0xc7ce49c4, SUC_lexicon_tags_13 },
    { 0x58ce49c5, SUC_lexicon_tags_43 },
    { 0xb20e49c5, SUC_lexicon_tags_64 },
    { 0xed0e49c7, SUC_lexicon_tags_2 },
    { 0xd1ee49c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa22e49ca, SUC_lexicon_tags_25 },
    { 0x51ee49ca, SUC_lexicon_tags_63 },
    { 0x586e49cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2e49d0, SUC_lexicon_tags_37 },
    { 0x322e49d1, SUC_lexicon_tags_15 },
    { 0xfd6e49d2, SUC_lexicon_tags_38 },
    { 0xfce49d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978e49da, SUC_lexicon_tags_45 },
    { 0x1b8e49db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf50e49dd, SUC_lexicon_tags_1 },
    { 0xdeee49de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56e49e4, SUC_lexicon_tags_146 },
    { 0x90ae49e5, SUC_lexicon_tags_7 },
    { 0x984e49e5, SUC_lexicon_tags_15 },
    { 0x388e49e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae49f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dce49f5, SUC_lexicon_tags_27 },
    { 0x814e49f5, SUC_lexicon_tags_38 },
    { 0xa68e49f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x944e49f9, SUC_lexicon_tags_65 },
    { 0x4dee49fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ee49fd, SUC_lexicon_tags_2 },
    { 0x4aee49fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfee4a02, SUC_lexicon_tags_15 },
    { 0x8a0e4a03, SUC_lexicon_tags_31 },
    { 0xa44e4a03, SUC_lexicon_tags_12 },
    { 0xd2ee4a04, SUC_lexicon_tags_25 },
    { 0x40ce4a06, SUC_lexicon_tags_37 },
    { 0x574e4a07, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x780e4a09, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e4a0f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cee4a17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x268e4a1a, SUC_lexicon_tags_27 },
    { 0x43ee4a1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664e4a25, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8e4a2a, SUC_lexicon_tags_15 },
    { 0x9b4e4a2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaee4a30, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9e8e4a32, SUC_lexicon_tags_23 },
    { 0x970e4a33, SUC_lexicon_tags_30 },
    { 0x5bce4a34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ce4a38, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12e4a3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2e4a42, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4e4a46, SUC_lexicon_tags_47 },
    { 0x5a6e4a46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ce4a4e, SUC_lexicon_tags_61 },
    { 0x188e4a4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ce4a53, SUC_lexicon_tags_98 },
    { 0x1fce4a54, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x650e4a57, SUC_lexicon_tags_12 },
    { 0x1a8e4a57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x444e4a5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22e4a63, SUC_lexicon_tags_1 },
    { 0x228e4a63, SUC_lexicon_tags_27 },
    { 0x2bce4a65, SUC_lexicon_tags_8 },
    { 0x1f0e4a66, SUC_lexicon_tags_15 },
    { 0xb9ee4a66, SUC_lexicon_tags_39 },
    { 0xbd4e4a68, SUC_lexicon_tags_37 },
    { 0xa50e4a69, SUC_lexicon_tags_47 },
    { 0x8c8e4a6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe82e4a6c, SUC_lexicon_tags_25 },
    { 0x6d2e4a6c, SUC_lexicon_tags_29 },
    { 0x84ee4a6d, SUC_lexicon_tags_81 },
    { 0x814e4a6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ee4a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc72e4a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8e4a77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x916e4a79, SUC_lexicon_tags_0 },
    { 0x890e4a7a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464e4a7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8e4a81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfee4a87, SUC_lexicon_tags_47 },
    { 0x226e4a88, SUC_lexicon_tags_30 },
    { 0x9dee4a89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ee4a8f, SUC_lexicon_tags_12 },
    { 0x1ce4a90, SUC_lexicon_tags_0 },
    { 0xa22e4a8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78e4a95, SUC_lexicon_tags_24 },
    { 0x3cee4a96, SUC_lexicon_tags_49 },
    { 0x104e4a95, SUC_lexicon_tags_37 },
    { 0x96ce4a98, SUC_lexicon_tags_7 },
    { 0xfb4e4a99, SUC_lexicon_tags_57 },
    { 0x556e4a96, SUC_lexicon_tags_6 },
    { 0x214e4a95, SUC_lexicon_tags_0 },
    { 0xcaee4a99, SUC_lexicon_tags_16 },
    { 0xa9ce4a9d, SUC_lexicon_tags_45 },
    { 0x486e4a9d, SUC_lexicon_tags_15 },
    { 0x5d0e4a96, SUC_lexicon_tags_2 },
    { 0x43ce4aa0, SUC_lexicon_tags_58 },
    { 0x6cce4aa1, SUC_lexicon_tags_31 },
    { 0xadce4aa1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12e4aa5, SUC_lexicon_tags_17 },
    { 0xda2e4aa5, SUC_lexicon_tags_1 },
    { 0x20e4aa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ce4aaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ae4ab0, SUC_lexicon_tags_15 },
    { 0x540e4ab0, SUC_lexicon_tags_7 },
    { 0x62ee4ab1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x814e4ab5, SUC_lexicon_tags_43 },
    { 0x380e4ab6, SUC_lexicon_tags_141 },
    { 0x938e4ab5, SUC_lexicon_tags_64 },
    { 0x4fee4ab8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6bee4aba, SUC_lexicon_tags_47 },
    { 0x4bce4abb, SUC_lexicon_tags_17 },
    { 0x20e4abc, SUC_lexicon_tags_1 },
    { 0x5ace4abc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dae4ac0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8e4ac7, SUC_lexicon_tags_0 },
    { 0xc2ee4ac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2e4acc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ae4ad2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0e4ad5, SUC_lexicon_tags_1 },
    { 0x8f4e4ad6, SUC_lexicon_tags_17 },
    { 0xc8ae4ad7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6e4ada, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0e4adf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44ee4ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98e4ae4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4e4ae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x132e4aea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ce4aef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x97ee4af1, SUC_lexicon_tags_32 },
    { 0xdbee4af1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4e4af8, SUC_lexicon_tags_39 },
    { 0x592e4af9, SUC_lexicon_tags_48 },
    { 0x884e4af9, SUC_lexicon_tags_28 },
    { 0xafee4afa, SUC_lexicon_tags_48 },
    { 0xcc0e4af8, SUC_lexicon_tags_10 },
    { 0x1e8e4afd, SUC_lexicon_tags_12 },
    { 0x7ece4afe, SUC_lexicon_tags_23 },
    { 0x4a8e4aff, SUC_lexicon_tags_17 },
    { 0xaf8e4b00, SUC_lexicon_tags_84 },
    { 0x2cce4b00, SUC_lexicon_tags_1 },
    { 0x6ece4b02, SUC_lexicon_tags_28 },
    { 0x8e6e4aff, SUC_lexicon_tags_65 },
    { 0x9bce4b04, SUC_lexicon_tags_38 },
    { 0xa8e4aff, SUC_lexicon_tags_32 },
    { 0x258e4b05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78e4b09, SUC_lexicon_tags_11 },
    { 0x15ae4b09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678e4b13, SUC_lexicon_tags_17 },
    { 0x84e4b13, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4e4b18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938e4b1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ee4b22, SUC_lexicon_tags_15 },
    { 0x3e8e4b22, SUC_lexicon_tags_25 },
    { 0xb6ce4b23, SUC_lexicon_tags_47 },
    { 0xb1ce4b25, SUC_lexicon_tags_16 },
    { 0x7f8e4b23, SUC_lexicon_tags_29 },
    { 0x65ae4b25, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe98e4b29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ce4b2c, SUC_lexicon_tags_12 },
    { 0x3d6e4b2c, SUC_lexicon_tags_7 },
    { 0x2a4e4b2e, SUC_lexicon_tags_23 },
    { 0xa1ce4b2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ee4b35, SUC_lexicon_tags_25 },
    { 0xd6ae4b36, SUC_lexicon_tags_37 },
    { 0x47ae4b35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcee4b39, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2bee4b3b, SUC_lexicon_tags_0 },
    { 0x9e6e4b3c, SUC_lexicon_tags_39 },
    { 0x784e4b3d, SUC_lexicon_tags_11 },
    { 0x942e4b3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x284e4b41, SUC_lexicon_tags_17 },
    { 0x3d4e4b42, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24e4b45, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0e4b4a, SUC_lexicon_tags_11 },
    { 0xaf6e4b4a, SUC_lexicon_tags_994 },
    { 0, NULL },
    { 0x7b8e4b4d, SUC_lexicon_tags_1 },
    { 0xf1ae4b4d, SUC_lexicon_tags_29 },
    { 0xdaae4b4f, SUC_lexicon_tags_43 },
    { 0xfc2e4b50, SUC_lexicon_tags_15 },
    { 0x864e4b50, SUC_lexicon_tags_46 },
    { 0x8e6e4b52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x35ae4b54, SUC_lexicon_tags_8 },
    { 0xb6ce4b55, SUC_lexicon_tags_21 },
    { 0x462e4b55, SUC_lexicon_tags_28 },
    { 0x534e4b57, SUC_lexicon_tags_17 },
    { 0x952e4b58, SUC_lexicon_tags_66 },
    { 0xd40e4b54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb9ae4b5b, SUC_lexicon_tags_19 },
    { 0x956e4b5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4aee4b5e, SUC_lexicon_tags_12 },
    { 0xc8e4b5f, SUC_lexicon_tags_23 },
    { 0x290e4b60, SUC_lexicon_tags_45 },
    { 0x798e4b60, SUC_lexicon_tags_8 },
    { 0x886e4b62, SUC_lexicon_tags_14 },
    { 0xa38e4b63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ee4b67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbce4b6b, SUC_lexicon_tags_14 },
    { 0xae4e4b6b, SUC_lexicon_tags_63 },
    { 0xafae4b6d, SUC_lexicon_tags_47 },
    { 0x946e4b6c, SUC_lexicon_tags_29 },
    { 0xa24e4b6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x260e4b72, SUC_lexicon_tags_65 },
    { 0xe12e4b73, SUC_lexicon_tags_46 },
    { 0xc54e4b74, SUC_lexicon_tags_17 },
    { 0xc2ae4b74, SUC_lexicon_tags_9 },
    { 0x8b8e4b74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae4b7b, SUC_lexicon_tags_1 },
    { 0xc5ce4b7c, SUC_lexicon_tags_1 },
    { 0x9c6e4b7d, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ae4b82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18e4b85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74e4b88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fae4b8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc82e4b8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42e4b8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3ae4b91, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa04e4b93, SUC_lexicon_tags_37 },
    { 0x850e4b93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246e4b99, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x75ce4b9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2e4ba2, SUC_lexicon_tags_4 },
    { 0x896e4ba3, SUC_lexicon_tags_45 },
    { 0xa98e4ba3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22e4ba8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x770e4baa, SUC_lexicon_tags_37 },
    { 0xf6ee4baa, SUC_lexicon_tags_17 },
    { 0xc72e4bac, SUC_lexicon_tags_17 },
    { 0xf72e4bad, SUC_lexicon_tags_27 },
    { 0xb3ce4bac, SUC_lexicon_tags_31 },
    { 0x840e4baf, SUC_lexicon_tags_2 },
    { 0x9d4e4bb0, SUC_lexicon_tags_4 },
    { 0x492e4bb1, SUC_lexicon_tags_6 },
    { 0x80ee4bb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bee4bb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f4e4bb6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe12e4bb8, SUC_lexicon_tags_66 },
    { 0xb7ee4bb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964e4bbf, SUC_lexicon_tags_9 },
    { 0x196e4bc0, SUC_lexicon_tags_15 },
    { 0x5a4e4bc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4e4bc6, SUC_lexicon_tags_49 },
    { 0xeece4bc7, SUC_lexicon_tags_0 },
    { 0xf8e4bc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e4bce, SUC_lexicon_tags_2 },
    { 0xc64e4bce, SUC_lexicon_tags_33 },
    { 0x7d8e4bd0, SUC_lexicon_tags_15 },
    { 0x264e4bd1, SUC_lexicon_tags_17 },
    { 0x63ce4bd2, SUC_lexicon_tags_47 },
    { 0x41ae4bd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ce4bd7, SUC_lexicon_tags_47 },
    { 0xd2ce4bd8, SUC_lexicon_tags_15 },
    { 0xe46e4bd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae4be0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372e4be5, SUC_lexicon_tags_7 },
    { 0xc24e4be5, SUC_lexicon_tags_8 },
    { 0xff6e4be7, SUC_lexicon_tags_17 },
    { 0xd72e4be7, SUC_lexicon_tags_7 },
    { 0x9b8e4be9, SUC_lexicon_tags_48 },
    { 0x130e4be9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d6e4bec, SUC_lexicon_tags_25 },
    { 0x25ae4bec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ee4bf1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ae4bf6, SUC_lexicon_tags_60 },
    { 0x212e4bf7, SUC_lexicon_tags_47 },
    { 0xbc0e4bf6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0e4bfc, SUC_lexicon_tags_77 },
    { 0x9fee4bfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32e4c00, SUC_lexicon_tags_0 },
    { 0xe02e4c01, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80e4c04, SUC_lexicon_tags_37 },
    { 0xad6e4c05, SUC_lexicon_tags_47 },
    { 0xdfce4c04, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6e4c0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacee4c15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8e4c1b, SUC_lexicon_tags_37 },
    { 0x614e4c1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x11ae4c1e, SUC_lexicon_tags_2 },
    { 0x1b6e4c1f, SUC_lexicon_tags_10 },
    { 0xd22e4c20, SUC_lexicon_tags_33 },
    { 0xacae4c21, SUC_lexicon_tags_46 },
    { 0x4fae4c22, SUC_lexicon_tags_38 },
    { 0xe50e4c20, SUC_lexicon_tags_30 },
    { 0x412e4c24, SUC_lexicon_tags_15 },
    { 0x2eee4c24, SUC_lexicon_tags_37 },
    { 0x744e4c26, SUC_lexicon_tags_23 },
    { 0x912e4c1f, SUC_lexicon_tags_17 },
    { 0xf8e4c28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x868e4c2a, SUC_lexicon_tags_8 },
    { 0xceae4c2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x332e4c2d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518e4c32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x386e4c34, SUC_lexicon_tags_49 },
    { 0xd70e4c34, SUC_lexicon_tags_47 },
    { 0xdae4c36, SUC_lexicon_tags_17 },
    { 0x44ee4c37, SUC_lexicon_tags_46 },
    { 0xf92e4c35, SUC_lexicon_tags_7 },
    { 0xf6e4c39, SUC_lexicon_tags_1 },
    { 0xd02e4c35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x948e4c3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2e4c40, SUC_lexicon_tags_12 },
    { 0xf32e4c41, SUC_lexicon_tags_12 },
    { 0x1d0e4c42, SUC_lexicon_tags_6 },
    { 0xbaee4c42, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd66e4c45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74e4c48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfc6e4c4a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28e4c4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aae4c50, SUC_lexicon_tags_55 },
    { 0x8eae4c51, SUC_lexicon_tags_6 },
    { 0x7d6e4c52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4e4c56, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16e4c62, SUC_lexicon_tags_2 },
    { 0x3cce4c63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eae4c66, SUC_lexicon_tags_9 },
    { 0x87ce4c67, SUC_lexicon_tags_4 },
    { 0xb54e4c67, SUC_lexicon_tags_25 },
    { 0x57ae4c67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce4c6d, SUC_lexicon_tags_37 },
    { 0x920e4c6e, SUC_lexicon_tags_37 },
    { 0xc0ee4c6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d0e4c71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x954e4c74, SUC_lexicon_tags_55 },
    { 0xfcae4c75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0e4c7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ee4c81, SUC_lexicon_tags_23 },
    { 0xe12e4c82, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148e4c87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ae4c8b, SUC_lexicon_tags_17 },
    { 0xc64e4c8b, SUC_lexicon_tags_15 },
    { 0x12ae4c8d, SUC_lexicon_tags_21 },
    { 0xc2ce4c8e, SUC_lexicon_tags_8 },
    { 0x9d6e4c8f, SUC_lexicon_tags_11 },
    { 0xd92e4c8e, SUC_lexicon_tags_27 },
    { 0xaeee4c90, SUC_lexicon_tags_17 },
    { 0xeb8e4c8b, SUC_lexicon_tags_68 },
    { 0xf14e4c8e, SUC_lexicon_tags_17 },
    { 0xe14e4c94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e4e4c96, SUC_lexicon_tags_27 },
    { 0xdae4c96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4e4ca3, SUC_lexicon_tags_49 },
    { 0xb1ee4ca4, SUC_lexicon_tags_7 },
    { 0x116e4ca5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4ce4ca7, SUC_lexicon_tags_11 },
    { 0x156e4ca7, SUC_lexicon_tags_63 },
    { 0x664e4ca9, SUC_lexicon_tags_37 },
    { 0x70ee4ca9, SUC_lexicon_tags_33 },
    { 0xdcce4ca9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ae4cae, SUC_lexicon_tags_21 },
    { 0xae4e4caf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8e4cb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fee4cb6, SUC_lexicon_tags_11 },
    { 0x3fce4cb7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc32e4cb9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6e4cbe, SUC_lexicon_tags_60 },
    { 0x53ee4cbf, SUC_lexicon_tags_82 },
    { 0xe40e4cc0, SUC_lexicon_tags_15 },
    { 0x988e4cc1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf1ee4cc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9aee4cc5, SUC_lexicon_tags_16 },
    { 0x63ee4cc6, SUC_lexicon_tags_14 },
    { 0x70ce4cc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x788e4cca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ce4ccd, SUC_lexicon_tags_29 },
    { 0x138e4cce, SUC_lexicon_tags_17 },
    { 0x17ae4ccf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x400e4cd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ce4cd6, SUC_lexicon_tags_0 },
    { 0x92ae4cd7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68e4cde, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14e4ce4, SUC_lexicon_tags_12 },
    { 0xe4ae4ce4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918e4ceb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334e4cf3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc50e4cf6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2e4cfe, SUC_lexicon_tags_60 },
    { 0x4ece4cff, SUC_lexicon_tags_54 },
    { 0x804e4cff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8e4d07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4e4d0b, SUC_lexicon_tags_33 },
    { 0x698e4d0b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914e4d10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996e4d17, SUC_lexicon_tags_47 },
    { 0xb52e4d18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb7ae4d1a, SUC_lexicon_tags_14 },
    { 0x586e4d1b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffae4d1f, SUC_lexicon_tags_11 },
    { 0x720e4d20, SUC_lexicon_tags_25 },
    { 0xce2e4d21, SUC_lexicon_tags_82 },
    { 0xf4ee4d21, SUC_lexicon_tags_37 },
    { 0x1f0e4d20, SUC_lexicon_tags_12 },
    { 0xc88e4d20, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50e4d27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6e4d2f, SUC_lexicon_tags_17 },
    { 0x362e4d30, SUC_lexicon_tags_47 },
    { 0x6dae4d2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ae4d34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ae4d37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e4d3b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44e4d3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a0e4d40, SUC_lexicon_tags_21 },
    { 0x730e4d41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ae4d47, SUC_lexicon_tags_38 },
    { 0xbb6e4d48, SUC_lexicon_tags_12 },
    { 0xde4e4d49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50e4d4d, SUC_lexicon_tags_33 },
    { 0xe6ae4d4e, SUC_lexicon_tags_61 },
    { 0x8e0e4d4e, SUC_lexicon_tags_45 },
    { 0x4f8e4d50, SUC_lexicon_tags_47 },
    { 0x702e4d51, SUC_lexicon_tags_8 },
    { 0xb46e4d4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2e4d56, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x638e4d59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278e4d5e, SUC_lexicon_tags_14 },
    { 0xa84e4d5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868e4d63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4e4d67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8d2e4d69, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x120e4d6b, SUC_lexicon_tags_47 },
    { 0xbeae4d6c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ae4d6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x246e4d72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84e4d75, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x392e4d78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78e4d7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa60e4d7d, SUC_lexicon_tags_2 },
    { 0x3eae4d7d, SUC_lexicon_tags_14 },
    { 0xe14e4d7f, SUC_lexicon_tags_8 },
    { 0x192e4d80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ee4d8a, SUC_lexicon_tags_39 },
    { 0x64ae4d8b, SUC_lexicon_tags_14 },
    { 0xa36e4d8c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506e4d91, SUC_lexicon_tags_37 },
    { 0x946e4d91, SUC_lexicon_tags_11 },
    { 0x906e4d93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8e4d99, SUC_lexicon_tags_43 },
    { 0xd76e4d9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fae4d9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e4da0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200e4da4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5ce4da6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x284e4da9, SUC_lexicon_tags_2 },
    { 0x2aee4daa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8e4dad, SUC_lexicon_tags_35 },
    { 0x97ee4dad, SUC_lexicon_tags_16 },
    { 0x832e4dad, SUC_lexicon_tags_55 },
    { 0x32ee4dad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xba0e4db2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e8e4db4, SUC_lexicon_tags_49 },
    { 0x762e4db5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc6e4db7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9ce4db9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632e4dbe, SUC_lexicon_tags_11 },
    { 0xec0e4dbf, SUC_lexicon_tags_1438 },
    { 0xf68e4dc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ce4dc4, SUC_lexicon_tags_33 },
    { 0x762e4dc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0e4dca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x726e4dcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x756e4dcf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x734e4dd1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28e4dd5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee4dda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x750e4ddc, SUC_lexicon_tags_45 },
    { 0x96ce4ddd, SUC_lexicon_tags_2 },
    { 0xd04e4dde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e2e4de0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed8e4de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc74e4de4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90e4de8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee4dec, SUC_lexicon_tags_17 },
    { 0x8c8e4ded, SUC_lexicon_tags_7 },
    { 0x2a6e4dee, SUC_lexicon_tags_15 },
    { 0x92ce4dee, SUC_lexicon_tags_2 },
    { 0x4b2e4dee, SUC_lexicon_tags_30 },
    { 0x2fae4df1, SUC_lexicon_tags_25 },
    { 0x7cae4df2, SUC_lexicon_tags_6 },
    { 0xb6e4df1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232e4df8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4ee4dfa, SUC_lexicon_tags_13 },
    { 0x610e4dfa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce4dfe, SUC_lexicon_tags_19 },
    { 0x422e4dfe, SUC_lexicon_tags_29 },
    { 0xe4ee4e00, SUC_lexicon_tags_54 },
    { 0x8fae4e01, SUC_lexicon_tags_39 },
    { 0x44ee4dff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ee4e05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e4e10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x782e4e12, SUC_lexicon_tags_37 },
    { 0x87ae4e12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960e4e1a, SUC_lexicon_tags_11 },
    { 0x794e4e1b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xbbae4e1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52e4e20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c2e4e22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4e4e26, SUC_lexicon_tags_11 },
    { 0x844e4e27, SUC_lexicon_tags_1 },
    { 0x72ae4e28, SUC_lexicon_tags_29 },
    { 0xc76e4e29, SUC_lexicon_tags_7 },
    { 0x692e4e2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f6e4e2c, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xcf0e4e2e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x5f6e4e30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9ae4e32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50e4e36, SUC_lexicon_tags_29 },
    { 0xfeee4e37, SUC_lexicon_tags_2 },
    { 0x97ce4e38, SUC_lexicon_tags_138 },
    { 0x8c6e4e39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x288e4e3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ee4e3e, SUC_lexicon_tags_13 },
    { 0xd7ce4e3f, SUC_lexicon_tags_87 },
    { 0x248e4e3e, SUC_lexicon_tags_11 },
    { 0x3e0e4e41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ce4e47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490e4e4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x936e4e51, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4e4e5c, SUC_lexicon_tags_14 },
    { 0x20ae4e5c, SUC_lexicon_tags_11 },
    { 0x1b6e4e5e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ce4e61, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x372e4e64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x806e4e66, SUC_lexicon_tags_1 },
    { 0x200e4e67, SUC_lexicon_tags_9 },
    { 0x8f2e4e66, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682e4e6d, SUC_lexicon_tags_38 },
    { 0x74ee4e6d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70e4e71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240e4e76, SUC_lexicon_tags_17 },
    { 0x826e4e77, SUC_lexicon_tags_14 },
    { 0x562e4e77, SUC_lexicon_tags_37 },
    { 0x786e4e79, SUC_lexicon_tags_11 },
    { 0x87ee4e7a, SUC_lexicon_tags_15 },
    { 0xe18e4e79, SUC_lexicon_tags_11 },
    { 0xc6ce4e77, SUC_lexicon_tags_17 },
    { 0x7eee4e7d, SUC_lexicon_tags_12 },
    { 0xe42e4e7d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfae4e82, SUC_lexicon_tags_63 },
    { 0x86e4e82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2bce4e85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x790e4e88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30e4e8c, SUC_lexicon_tags_62 },
    { 0x29ce4e8d, SUC_lexicon_tags_11 },
    { 0x27ae4e8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdee4e92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e4e96, SUC_lexicon_tags_1 },
    { 0x1ace4e96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80e4e9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e4e9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e2e4ea0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bee4ea3, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ce4ea8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0e4eab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792e4eb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ae4eb6, SUC_lexicon_tags_63 },
    { 0x1bae4eb7, SUC_lexicon_tags_11 },
    { 0x726e4eb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106e4ec3, SUC_lexicon_tags_65 },
    { 0x816e4ec4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf32e4ec6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac4e4ec8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4e4ecb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x938e4ece, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bee4ed9, SUC_lexicon_tags_5 },
    { 0x812e4eda, SUC_lexicon_tags_91 },
    { 0xfa8e4edb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaa6e4edd, SUC_lexicon_tags_14 },
    { 0x6ece4edd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2e4ee6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x54ae4ee8, SUC_lexicon_tags_37 },
    { 0x38ae4ee9, SUC_lexicon_tags_0 },
    { 0x856e4eea, SUC_lexicon_tags_14 },
    { 0x7f4e4eeb, SUC_lexicon_tags_56 },
    { 0xbdae4eec, SUC_lexicon_tags_7 },
    { 0x93ae4eeb, SUC_lexicon_tags_33 },
    { 0x278e4ee9, SUC_lexicon_tags_25 },
    { 0xf5ee4eef, SUC_lexicon_tags_11 },
    { 0xfd4e4ef0, SUC_lexicon_tags_15 },
    { 0xd30e4ef0, SUC_lexicon_tags_37 },
    { 0x560e4ef2, SUC_lexicon_tags_11 },
    { 0xd8ce4ef2, SUC_lexicon_tags_34 },
    { 0x454e4ee8, SUC_lexicon_tags_2 },
    { 0x990e4ef5, SUC_lexicon_tags_23 },
    { 0x87ce4eec, SUC_lexicon_tags_17 },
    { 0xd4ce4eed, SUC_lexicon_tags_2 },
    { 0x6cae4eed, SUC_lexicon_tags_15 },
    { 0x512e4ef6, SUC_lexicon_tags_7 },
    { 0xbbee4ef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xefae4efd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114e4f02, SUC_lexicon_tags_17 },
    { 0xa34e4f03, SUC_lexicon_tags_19 },
    { 0xbee4f04, SUC_lexicon_tags_141 },
    { 0x138e4f05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ee4f08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4e4f0d, SUC_lexicon_tags_19 },
    { 0xbb4e4f0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf56e4f10, SUC_lexicon_tags_87 },
    { 0x10ee4f10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ee4f14, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82e4f17, SUC_lexicon_tags_13 },
    { 0xda4e4f18, SUC_lexicon_tags_34 },
    { 0xbf0e4f19, SUC_lexicon_tags_7 },
    { 0xe86e4f1a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0e4f23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208e4f2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ce4f2f, SUC_lexicon_tags_11 },
    { 0xb96e4f30, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xddce4f32, SUC_lexicon_tags_5 },
    { 0xc18e4f33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68e4f3a, SUC_lexicon_tags_15 },
    { 0xdf6e4f3a, SUC_lexicon_tags_11 },
    { 0xab8e4f3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x824e4f3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8e4f45, SUC_lexicon_tags_17 },
    { 0x6ece4f46, SUC_lexicon_tags_25 },
    { 0xf64e4f45, SUC_lexicon_tags_23 },
    { 0x49ce4f48, SUC_lexicon_tags_1 },
    { 0xee0e4f49, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x23ae4f4b, SUC_lexicon_tags_44 },
    { 0x870e4f4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeae4f50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56e4f54, SUC_lexicon_tags_23 },
    { 0xd42e4f55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ee4f58, SUC_lexicon_tags_43 },
    { 0x80e4f58, SUC_lexicon_tags_80 },
    { 0xad2e4f5a, SUC_lexicon_tags_33 },
    { 0xb2ae4f5a, SUC_lexicon_tags_23 },
    { 0xe9ee4f5c, SUC_lexicon_tags_32 },
    { 0x676e4f5c, SUC_lexicon_tags_31 },
    { 0x7bae4f5e, SUC_lexicon_tags_229 },
    { 0x22e4f5d, SUC_lexicon_tags_12 },
    { 0xe08e4f60, SUC_lexicon_tags_15 },
    { 0xf9ee4f5a, SUC_lexicon_tags_47 },
    { 0xbfee4f5f, SUC_lexicon_tags_12 },
    { 0xd00e4f63, SUC_lexicon_tags_70 },
    { 0x8ae4f5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1ee4f66, SUC_lexicon_tags_37 },
    { 0xdb2e4f67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc28e4f69, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x560e4f6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfce4f71, SUC_lexicon_tags_27 },
    { 0xf92e4f71, SUC_lexicon_tags_35 },
    { 0x49ce4f71, SUC_lexicon_tags_19 },
    { 0x746e4f74, SUC_lexicon_tags_0 },
    { 0x43ae4f75, SUC_lexicon_tags_33 },
    { 0xfa6e4f74, SUC_lexicon_tags_17 },
    { 0x6eee4f77, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x93ae4f79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d0e4f7b, SUC_lexicon_tags_37 },
    { 0x946e4f7c, SUC_lexicon_tags_43 },
    { 0xe0ee4f7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20e4f82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00e4f87, SUC_lexicon_tags_38 },
    { 0x81ae4f88, SUC_lexicon_tags_12 },
    { 0xdc4e4f88, SUC_lexicon_tags_43 },
    { 0xacce4f8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8e4f8f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x72ee4f91, SUC_lexicon_tags_67 },
    { 0x8b4e4f92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0e4f95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328e4f99, SUC_lexicon_tags_26 },
    { 0x3f8e4f9a, SUC_lexicon_tags_30 },
    { 0x8ae4f9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868e4fa0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1ce4fa2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ae4fa9, SUC_lexicon_tags_96 },
    { 0xf78e4faa, SUC_lexicon_tags_7 },
    { 0x5c2e4fab, SUC_lexicon_tags_37 },
    { 0x1dce4fac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x476e4faf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50e4fb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6e4fbf, SUC_lexicon_tags_11 },
    { 0xb04e4fc0, SUC_lexicon_tags_11 },
    { 0xe36e4fbf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd52e4fc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ce4fc8, SUC_lexicon_tags_47 },
    { 0xedee4fc9, SUC_lexicon_tags_49 },
    { 0x79ce4fca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcae4fcd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8e4fd2, SUC_lexicon_tags_15 },
    { 0x146e4fd3, SUC_lexicon_tags_17 },
    { 0x19ce4fd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85ce4fd6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x25ae4fd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86e4fdc, SUC_lexicon_tags_15 },
    { 0xb42e4fdc, SUC_lexicon_tags_37 },
    { 0xe62e4fde, SUC_lexicon_tags_31 },
    { 0xec4e4fdf, SUC_lexicon_tags_25 },
    { 0xba2e4fe0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x922e4fe3, SUC_lexicon_tags_22 },
    { 0xae0e4fe4, SUC_lexicon_tags_17 },
    { 0xabee4fe5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262e4fe9, SUC_lexicon_tags_37 },
    { 0xac6e4fea, SUC_lexicon_tags_37 },
    { 0x120e4feb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x318e4fee, SUC_lexicon_tags_48 },
    { 0x74ee4fee, SUC_lexicon_tags_11 },
    { 0xebae4fef, SUC_lexicon_tags_17 },
    { 0xa5ee4fef, SUC_lexicon_tags_17 },
    { 0x9c0e4ff0, SUC_lexicon_tags_12 },
    { 0xc48e4ff3, SUC_lexicon_tags_33 },
    { 0x58ae4fef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x446e4ff7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ce4ffb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe76e4ffe, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae5001, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ae5004, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ce5009, SUC_lexicon_tags_15 },
    { 0xa68e500a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90e5011, SUC_lexicon_tags_15 },
    { 0xa80e5012, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb30e5014, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288e5018, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe7ce501a, SUC_lexicon_tags_65 },
    { 0xa66e501b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cce5020, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4ce5022, SUC_lexicon_tags_7 },
    { 0xe72e5022, SUC_lexicon_tags_2 },
    { 0xab2e5024, SUC_lexicon_tags_68 },
    { 0x68ae5025, SUC_lexicon_tags_17 },
    { 0x2dce5026, SUC_lexicon_tags_62 },
    { 0x618e5025, SUC_lexicon_tags_29 },
    { 0x612e5024, SUC_lexicon_tags_41 },
    { 0x6ece5028, SUC_lexicon_tags_47 },
    { 0x74e502a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ae502d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0e5032, SUC_lexicon_tags_14 },
    { 0xdf6e5032, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd0e5035, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0e5038, SUC_lexicon_tags_17 },
    { 0x918e5039, SUC_lexicon_tags_25 },
    { 0x9cee5039, SUC_lexicon_tags_1 },
    { 0x416e503b, SUC_lexicon_tags_45 },
    { 0xad4e5038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf98e503e, SUC_lexicon_tags_17 },
    { 0x648e503e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24ce5041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeae5044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfee504d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcce5050, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24e5059, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdaae505b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd94e505d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x546e505f, SUC_lexicon_tags_21 },
    { 0xda4e5060, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ee5064, SUC_lexicon_tags_33 },
    { 0xb34e5065, SUC_lexicon_tags_27 },
    { 0x28e5065, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136e506a, SUC_lexicon_tags_37 },
    { 0x78ce506b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fce506f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf8ce5071, SUC_lexicon_tags_37 },
    { 0x638e5072, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ace5074, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x42e5077, SUC_lexicon_tags_15 },
    { 0xd10e5077, SUC_lexicon_tags_16 },
    { 0xa86e5078, SUC_lexicon_tags_0 },
    { 0xc44e5079, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908e507e, SUC_lexicon_tags_17 },
    { 0x1a6e507f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ce5082, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1d8e5084, SUC_lexicon_tags_9 },
    { 0xb92e5085, SUC_lexicon_tags_19 },
    { 0xce0e5085, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdee508d, SUC_lexicon_tags_60 },
    { 0x7fae508e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcee5092, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e5096, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeee5098, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x984e509a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x556e509c, SUC_lexicon_tags_37 },
    { 0xfae509d, SUC_lexicon_tags_17 },
    { 0x29ee509e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ae50a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30e50a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae50a9, SUC_lexicon_tags_11 },
    { 0xd32e50aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x72ee50ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce50b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbee50b3, SUC_lexicon_tags_114 },
    { 0x488e50b4, SUC_lexicon_tags_27 },
    { 0x462e50b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ce50ba, SUC_lexicon_tags_4 },
    { 0xff8e50ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422e50c3, SUC_lexicon_tags_588 },
    { 0, NULL },
    { 0x12ee50c5, SUC_lexicon_tags_12 },
    { 0x24ce50c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9fce50c8, SUC_lexicon_tags_39 },
    { 0x2a4e50c9, SUC_lexicon_tags_27 },
    { 0x8e2e50c9, SUC_lexicon_tags_262 },
    { 0x86e50ca, SUC_lexicon_tags_66 },
    { 0xdbee50cc, SUC_lexicon_tags_1 },
    { 0xf8e50c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0e50d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6e50d3, SUC_lexicon_tags_14 },
    { 0xcc2e50d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ee50d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e50db, SUC_lexicon_tags_6 },
    { 0x75ee50dc, SUC_lexicon_tags_38 },
    { 0xd54e50db, SUC_lexicon_tags_6 },
    { 0xebce50db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826e50e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x716e50e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ee50eb, SUC_lexicon_tags_9 },
    { 0x57ae50ec, SUC_lexicon_tags_25 },
    { 0x1cee50ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8e50f2, SUC_lexicon_tags_37 },
    { 0xab8e50f3, SUC_lexicon_tags_14 },
    { 0xd88e50f4, SUC_lexicon_tags_32 },
    { 0xbfce50f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2ce50f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b2e50f9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6b6e50fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ce50fe, SUC_lexicon_tags_48 },
    { 0x1c2e50ff, SUC_lexicon_tags_3 },
    { 0x4a2e50ff, SUC_lexicon_tags_1 },
    { 0x84e5101, SUC_lexicon_tags_44 },
    { 0x6f6e5102, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64e5105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338e5109, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14e510c, SUC_lexicon_tags_8 },
    { 0xfcae510c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ce5110, SUC_lexicon_tags_7 },
    { 0xb84e5111, SUC_lexicon_tags_15 },
    { 0x14ae5112, SUC_lexicon_tags_17 },
    { 0x374e5113, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8e5118, SUC_lexicon_tags_35 },
    { 0x12ce5118, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x478e511b, SUC_lexicon_tags_16 },
    { 0x2c6e511c, SUC_lexicon_tags_33 },
    { 0x448e511d, SUC_lexicon_tags_11 },
    { 0xe48e511b, SUC_lexicon_tags_2 },
    { 0x242e511f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb88e5121, SUC_lexicon_tags_15 },
    { 0x450e5122, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ae5125, SUC_lexicon_tags_39 },
    { 0xca6e5126, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ee512a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ae5132, SUC_lexicon_tags_1 },
    { 0xb86e5132, SUC_lexicon_tags_43 },
    { 0xceae5134, SUC_lexicon_tags_48 },
    { 0x6f0e5135, SUC_lexicon_tags_11 },
    { 0x250e5134, SUC_lexicon_tags_25 },
    { 0x232e5135, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce513a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ce513e, SUC_lexicon_tags_15 },
    { 0xac4e513f, SUC_lexicon_tags_14 },
    { 0xed4e5140, SUC_lexicon_tags_7 },
    { 0x2eee5140, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd4ce5143, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcae5147, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xb90e5149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22ae514b, SUC_lexicon_tags_6 },
    { 0xdaee514b, SUC_lexicon_tags_7 },
    { 0x984e514d, SUC_lexicon_tags_23 },
    { 0x3ae514e, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x76ae5150, SUC_lexicon_tags_4 },
    { 0x36ce5151, SUC_lexicon_tags_22 },
    { 0xf02e5152, SUC_lexicon_tags_2 },
    { 0xd02e5153, SUC_lexicon_tags_23 },
    { 0x292e5152, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x602e5156, SUC_lexicon_tags_193 },
    { 0x408e5156, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ae515a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x904e515c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x254e515e, SUC_lexicon_tags_4 },
    { 0x5eae515f, SUC_lexicon_tags_2 },
    { 0xdcae5160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x996e5162, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32e5165, SUC_lexicon_tags_7 },
    { 0xf04e5166, SUC_lexicon_tags_15 },
    { 0x78e5165, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x598e5169, SUC_lexicon_tags_47 },
    { 0xa60e5169, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee516d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2e5172, SUC_lexicon_tags_0 },
    { 0x83ee5172, SUC_lexicon_tags_63 },
    { 0x860e5174, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4e5177, SUC_lexicon_tags_160 },
    { 0x3a4e5177, SUC_lexicon_tags_7 },
    { 0x4e0e5178, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12e517c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774e5180, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6e5184, SUC_lexicon_tags_1 },
    { 0x88ee5185, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eae518d, SUC_lexicon_tags_27 },
    { 0x9d8e518e, SUC_lexicon_tags_14 },
    { 0x908e518f, SUC_lexicon_tags_6 },
    { 0xe2ce518e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974e5194, SUC_lexicon_tags_2 },
    { 0xe1ae5194, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcaee5197, SUC_lexicon_tags_37 },
    { 0x536e5198, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bee51a2, SUC_lexicon_tags_25 },
    { 0xba4e51a3, SUC_lexicon_tags_11 },
    { 0x354e51a3, SUC_lexicon_tags_37 },
    { 0x304e51a5, SUC_lexicon_tags_1 },
    { 0x678e51a6, SUC_lexicon_tags_11 },
    { 0x3cee51a7, SUC_lexicon_tags_7 },
    { 0xc70e51a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4e51b0, SUC_lexicon_tags_29 },
    { 0xad2e51b0, SUC_lexicon_tags_25 },
    { 0x1e0e51b2, SUC_lexicon_tags_43 },
    { 0x746e51b3, SUC_lexicon_tags_1 },
    { 0x608e51b4, SUC_lexicon_tags_12 },
    { 0x1eee51b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8e51b9, SUC_lexicon_tags_17 },
    { 0xf6e51ba, SUC_lexicon_tags_29 },
    { 0xb3ce51ba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x12e51bd, SUC_lexicon_tags_79 },
    { 0xdfee51be, SUC_lexicon_tags_57 },
    { 0x3cae51bf, SUC_lexicon_tags_7 },
    { 0x872e51c0, SUC_lexicon_tags_0 },
    { 0x1b2e51c1, SUC_lexicon_tags_7 },
    { 0xc20e51c1, SUC_lexicon_tags_11 },
    { 0xad6e51c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb2e51c5, SUC_lexicon_tags_7 },
    { 0x7b0e51c6, SUC_lexicon_tags_7 },
    { 0x4f2e51c5, SUC_lexicon_tags_31 },
    { 0xcee51c5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90e51cb, SUC_lexicon_tags_2 },
    { 0x36ae51cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x32ae51ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafce51d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30e51d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12e51dd, SUC_lexicon_tags_28 },
    { 0x11ce51de, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fee51e6, SUC_lexicon_tags_35 },
    { 0x3cce51e7, SUC_lexicon_tags_37 },
    { 0x4dce51e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x750e51ea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e51ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ae51f3, SUC_lexicon_tags_7 },
    { 0xa58e51f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950e51f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90e5200, SUC_lexicon_tags_17 },
    { 0xf36e5201, SUC_lexicon_tags_37 },
    { 0x7dee5201, SUC_lexicon_tags_10 },
    { 0xb94e5203, SUC_lexicon_tags_17 },
    { 0xe8ce5204, SUC_lexicon_tags_12 },
    { 0xb42e5203, SUC_lexicon_tags_14 },
    { 0x9bae5205, SUC_lexicon_tags_37 },
    { 0xd2ce5207, SUC_lexicon_tags_9 },
    { 0x46e5208, SUC_lexicon_tags_12 },
    { 0x924e5209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4e520c, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x9eae520e, SUC_lexicon_tags_68 },
    { 0x622e520e, SUC_lexicon_tags_2 },
    { 0xf42e520e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946e5214, SUC_lexicon_tags_47 },
    { 0x736e5215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eee521b, SUC_lexicon_tags_15 },
    { 0x686e521c, SUC_lexicon_tags_49 },
    { 0x13ee521d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4e5223, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x980e5226, SUC_lexicon_tags_15 },
    { 0x774e5226, SUC_lexicon_tags_2 },
    { 0x326e5228, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x658e522d, SUC_lexicon_tags_30 },
    { 0x226e522d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4e5231, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbf0e5233, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x928e5235, SUC_lexicon_tags_2 },
    { 0x194e5236, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf82e5238, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28e523c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14ae523e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4ee5240, SUC_lexicon_tags_8 },
    { 0x3b4e5241, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcfee5243, SUC_lexicon_tags_1 },
    { 0x8f2e5244, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea6e5246, SUC_lexicon_tags_4 },
    { 0x66e5247, SUC_lexicon_tags_994 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854e524b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x604e524e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x738e5250, SUC_lexicon_tags_1 },
    { 0xe8ee5251, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x382e5253, SUC_lexicon_tags_7 },
    { 0x480e5254, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0e5258, SUC_lexicon_tags_27 },
    { 0x31ce5258, SUC_lexicon_tags_45 },
    { 0xdcce525a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ee5260, SUC_lexicon_tags_10 },
    { 0xe50e5260, SUC_lexicon_tags_17 },
    { 0x1d4e5262, SUC_lexicon_tags_4 },
    { 0x636e5262, SUC_lexicon_tags_40 },
    { 0xeb6e5264, SUC_lexicon_tags_14 },
    { 0x79ee5265, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x390e5267, SUC_lexicon_tags_63 },
    { 0xdb6e5268, SUC_lexicon_tags_28 },
    { 0xb96e5269, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e526d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x990e526f, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80e5276, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x47ae5278, SUC_lexicon_tags_30 },
    { 0x52ae5279, SUC_lexicon_tags_13 },
    { 0xecee527a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66e527e, SUC_lexicon_tags_12 },
    { 0x444e527f, SUC_lexicon_tags_45 },
    { 0xa92e5280, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x646e5283, SUC_lexicon_tags_14 },
    { 0x698e5284, SUC_lexicon_tags_49 },
    { 0xaee5285, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4e5289, SUC_lexicon_tags_15 },
    { 0x186e5289, SUC_lexicon_tags_2 },
    { 0x4b0e528b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fae528f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x63ce5291, SUC_lexicon_tags_6 },
    { 0xd7ee5292, SUC_lexicon_tags_17 },
    { 0x52ce5293, SUC_lexicon_tags_32 },
    { 0xe00e5294, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ae5297, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbae529a, SUC_lexicon_tags_37 },
    { 0xd68e529b, SUC_lexicon_tags_2 },
    { 0x2fce529a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8e52a4, SUC_lexicon_tags_25 },
    { 0xb86e52a4, SUC_lexicon_tags_15 },
    { 0x92ce52a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff0e52a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff4e52aa, SUC_lexicon_tags_43 },
    { 0xf50e52ab, SUC_lexicon_tags_7 },
    { 0x5a2e52ac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ee52b0, SUC_lexicon_tags_37 },
    { 0x104e52b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8e52b5, SUC_lexicon_tags_31 },
    { 0x716e52b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x468e52b8, SUC_lexicon_tags_15 },
    { 0x9cae52b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b2e52bb, SUC_lexicon_tags_6 },
    { 0x152e52bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x546e52be, SUC_lexicon_tags_1 },
    { 0x758e52bf, SUC_lexicon_tags_10 },
    { 0x7a6e52c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8e0e52c2, SUC_lexicon_tags_23 },
    { 0x684e52c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fce52c8, SUC_lexicon_tags_12 },
    { 0xcdce52c9, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16e52cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6e52cf, SUC_lexicon_tags_15 },
    { 0x972e52cf, SUC_lexicon_tags_15 },
    { 0x950e52d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d2e52d3, SUC_lexicon_tags_28 },
    { 0x5d2e52d3, SUC_lexicon_tags_60 },
    { 0xbdee52d5, SUC_lexicon_tags_8 },
    { 0x636e52d6, SUC_lexicon_tags_3 },
    { 0x7eae52d6, SUC_lexicon_tags_1 },
    { 0x274e52d8, SUC_lexicon_tags_17 },
    { 0x1b8e52d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786e52df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48e52e2, SUC_lexicon_tags_45 },
    { 0x5a6e52e3, SUC_lexicon_tags_1 },
    { 0x156e52e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fee52eb, SUC_lexicon_tags_7 },
    { 0xa3ae52eb, SUC_lexicon_tags_37 },
    { 0xe1ce52ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ece52ef, SUC_lexicon_tags_31 },
    { 0x380e52f0, SUC_lexicon_tags_26 },
    { 0x704e52ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6e52f6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6e52fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ce52fd, SUC_lexicon_tags_17 },
    { 0x6dae52fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ae5304, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaee5309, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e530b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566e530f, SUC_lexicon_tags_37 },
    { 0x930e5310, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0e5315, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ece5317, SUC_lexicon_tags_23 },
    { 0xda6e5318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66e531a, SUC_lexicon_tags_45 },
    { 0x4ece531b, SUC_lexicon_tags_0 },
    { 0x3cee531c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80e5322, SUC_lexicon_tags_20 },
    { 0xab0e5322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630e5329, SUC_lexicon_tags_33 },
    { 0x33ee532a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6e532d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26e5333, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8e5338, SUC_lexicon_tags_43 },
    { 0xea4e5339, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6e533f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ae5344, SUC_lexicon_tags_0 },
    { 0xd48e5345, SUC_lexicon_tags_12 },
    { 0x272e5345, SUC_lexicon_tags_47 },
    { 0x572e5347, SUC_lexicon_tags_11 },
    { 0xb7ce5347, SUC_lexicon_tags_45 },
    { 0xc92e5349, SUC_lexicon_tags_25 },
    { 0x90ae5349, SUC_lexicon_tags_2 },
    { 0xde0e534b, SUC_lexicon_tags_30 },
    { 0xab4e534c, SUC_lexicon_tags_26 },
    { 0x55ce534c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ee5354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4cee5356, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36e535c, SUC_lexicon_tags_900 },
    { 0xf5ee535c, SUC_lexicon_tags_13 },
    { 0xdc4e535e, SUC_lexicon_tags_92 },
    { 0xed8e535f, SUC_lexicon_tags_15 },
    { 0x17ee5360, SUC_lexicon_tags_40 },
    { 0x8cce5360, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58e5365, SUC_lexicon_tags_11 },
    { 0x530e5365, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0e5369, SUC_lexicon_tags_2 },
    { 0x2cee536a, SUC_lexicon_tags_11 },
    { 0x9f8e536a, SUC_lexicon_tags_11 },
    { 0x5ace536c, SUC_lexicon_tags_37 },
    { 0xfeae536b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x42e536f, SUC_lexicon_tags_47 },
    { 0x1cae5370, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x558e5372, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdee5374, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dce5379, SUC_lexicon_tags_79 },
    { 0x838e537a, SUC_lexicon_tags_77 },
    { 0x34e537b, SUC_lexicon_tags_29 },
    { 0x8d0e537c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cae5380, SUC_lexicon_tags_63 },
    { 0x72ce5380, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02e5384, SUC_lexicon_tags_31 },
    { 0xb94e5385, SUC_lexicon_tags_37 },
    { 0x7d4e5386, SUC_lexicon_tags_4 },
    { 0xc0ee5387, SUC_lexicon_tags_49 },
    { 0xcf4e5388, SUC_lexicon_tags_2 },
    { 0xe54e5388, SUC_lexicon_tags_19 },
    { 0x3fae538a, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6e538d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0e5394, SUC_lexicon_tags_43 },
    { 0xf5ae5395, SUC_lexicon_tags_0 },
    { 0x306e5396, SUC_lexicon_tags_94 },
    { 0xe70e5397, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebae539e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378e53a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x818e53a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac8e53ab, SUC_lexicon_tags_17 },
    { 0xebae53ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ce53af, SUC_lexicon_tags_35 },
    { 0x6fee53b0, SUC_lexicon_tags_69 },
    { 0x2b2e53af, SUC_lexicon_tags_11 },
    { 0xfe8e53b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x792e53b4, SUC_lexicon_tags_47 },
    { 0xfcae53b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x268e53b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16ee53ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfee53be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838e53c3, SUC_lexicon_tags_37 },
    { 0x2f6e53c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4e53c8, SUC_lexicon_tags_87 },
    { 0x2e8e53c8, SUC_lexicon_tags_0 },
    { 0x4ae53ca, SUC_lexicon_tags_25 },
    { 0x1c4e53c9, SUC_lexicon_tags_15 },
    { 0xdaae53c9, SUC_lexicon_tags_19 },
    { 0xa5ee53cd, SUC_lexicon_tags_55 },
    { 0x3c2e53ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0e53d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x496e53d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2e53d9, SUC_lexicon_tags_2 },
    { 0x65ae53da, SUC_lexicon_tags_17 },
    { 0x2bce53d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e53df, SUC_lexicon_tags_1 },
    { 0x33ce53df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ae53e7, SUC_lexicon_tags_2 },
    { 0xd14e53e8, SUC_lexicon_tags_151 },
    { 0x5b2e53e7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ae53ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1aae53ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ae53f1, SUC_lexicon_tags_87 },
    { 0xf34e53f2, SUC_lexicon_tags_29 },
    { 0xf0ce53f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ee53f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc24e53f8, SUC_lexicon_tags_27 },
    { 0x624e53f9, SUC_lexicon_tags_15 },
    { 0x840e53f9, SUC_lexicon_tags_29 },
    { 0xa30e53fb, SUC_lexicon_tags_39 },
    { 0x26e53fb, SUC_lexicon_tags_0 },
    { 0xd7ae53fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cee5400, SUC_lexicon_tags_17 },
    { 0xd70e5401, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ae5404, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46e5407, SUC_lexicon_tags_37 },
    { 0xfb2e5408, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60e540b, SUC_lexicon_tags_2 },
    { 0x540e540c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb76e540e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dee5411, SUC_lexicon_tags_23 },
    { 0x29ce5412, SUC_lexicon_tags_29 },
    { 0x63ae5412, SUC_lexicon_tags_1 },
    { 0xd66e5414, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ae5417, SUC_lexicon_tags_14 },
    { 0x1a2e5418, SUC_lexicon_tags_37 },
    { 0x970e5419, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x948e541c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240e5420, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2e5424, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26e5427, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e542a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90e542e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbbae5430, SUC_lexicon_tags_3 },
    { 0x10e5431, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ae5436, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e543a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6e543d, SUC_lexicon_tags_1 },
    { 0x53ae543e, SUC_lexicon_tags_12 },
    { 0x7b6e543f, SUC_lexicon_tags_22 },
    { 0x86ce5440, SUC_lexicon_tags_1 },
    { 0xbe0e5441, SUC_lexicon_tags_10 },
    { 0xd44e5442, SUC_lexicon_tags_63 },
    { 0x22ae5441, SUC_lexicon_tags_37 },
    { 0xaf8e543d, SUC_lexicon_tags_5 },
    { 0x65ce5445, SUC_lexicon_tags_47 },
    { 0x442e543f, SUC_lexicon_tags_25 },
    { 0x516e5447, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x546e5449, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4dce544b, SUC_lexicon_tags_14 },
    { 0xa28e544c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50e544f, SUC_lexicon_tags_17 },
    { 0x1f2e5450, SUC_lexicon_tags_29 },
    { 0x436e5451, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ce5455, SUC_lexicon_tags_39 },
    { 0xface5456, SUC_lexicon_tags_9 },
    { 0xea6e5457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eee545b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc66e545d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ce5460, SUC_lexicon_tags_15 },
    { 0xb96e5460, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc2e5463, SUC_lexicon_tags_11 },
    { 0xc7ae5464, SUC_lexicon_tags_0 },
    { 0x884e5465, SUC_lexicon_tags_98 },
    { 0xbb4e5466, SUC_lexicon_tags_33 },
    { 0xb84e5467, SUC_lexicon_tags_1314 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594e546d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dae5470, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbce5474, SUC_lexicon_tags_37 },
    { 0x296e5474, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726e5479, SUC_lexicon_tags_23 },
    { 0x8e4e5479, SUC_lexicon_tags_7 },
    { 0x74ce547b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ee5482, SUC_lexicon_tags_6 },
    { 0xa4ce5483, SUC_lexicon_tags_2 },
    { 0x294e5484, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ace5486, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0e548b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x728e548e, SUC_lexicon_tags_39 },
    { 0xe02e548e, SUC_lexicon_tags_37 },
    { 0x4f0e548f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98e5494, SUC_lexicon_tags_49 },
    { 0x3e2e5495, SUC_lexicon_tags_11 },
    { 0x682e5496, SUC_lexicon_tags_27 },
    { 0xcb6e5497, SUC_lexicon_tags_15 },
    { 0xbc2e5497, SUC_lexicon_tags_16 },
    { 0xd7ae5494, SUC_lexicon_tags_15 },
    { 0x3f8e5498, SUC_lexicon_tags_45 },
    { 0x30ee549b, SUC_lexicon_tags_25 },
    { 0x86ae549b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2e54a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa76e54a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236e54a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeae54a9, SUC_lexicon_tags_87 },
    { 0x7dee54aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x246e54ac, SUC_lexicon_tags_33 },
    { 0xe22e54ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802e54b2, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ae54b5, SUC_lexicon_tags_14 },
    { 0xdf6e54b6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86e54be, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4e54c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36e54c4, SUC_lexicon_tags_45 },
    { 0xbe8e54c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbb0e54c7, SUC_lexicon_tags_87 },
    { 0xea8e54c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fce54ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcc6e54d0, SUC_lexicon_tags_11 },
    { 0x8bae54d1, SUC_lexicon_tags_12 },
    { 0x28ee54d1, SUC_lexicon_tags_38 },
    { 0x532e54d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e54d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fce54db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ee54de, SUC_lexicon_tags_25 },
    { 0xb5ae54de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbbee54e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2e54e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2e54e8, SUC_lexicon_tags_57 },
    { 0x898e54e9, SUC_lexicon_tags_25 },
    { 0xfd2e54ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0e54fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc06e54fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ae54ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4e5503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80e5505, SUC_lexicon_tags_23 },
    { 0xd8ee5505, SUC_lexicon_tags_55 },
    { 0xc0e5507, SUC_lexicon_tags_46 },
    { 0x602e5507, SUC_lexicon_tags_43 },
    { 0x39ee5509, SUC_lexicon_tags_17 },
    { 0x1b8e5505, SUC_lexicon_tags_1 },
    { 0x1b8e5509, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02e5513, SUC_lexicon_tags_74 },
    { 0xd32e5513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188e551c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fce5520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce5524, SUC_lexicon_tags_2 },
    { 0x65ae5525, SUC_lexicon_tags_23 },
    { 0x144e5525, SUC_lexicon_tags_7 },
    { 0xfb8e5525, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cee552b, SUC_lexicon_tags_12 },
    { 0x3f0e552c, SUC_lexicon_tags_37 },
    { 0x96ee552b, SUC_lexicon_tags_47 },
    { 0x4c6e552e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x990e5530, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1ae5532, SUC_lexicon_tags_0 },
    { 0xc82e5533, SUC_lexicon_tags_12 },
    { 0xf36e5533, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfae5538, SUC_lexicon_tags_17 },
    { 0x230e5539, SUC_lexicon_tags_1 },
    { 0x872e553a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860e5541, SUC_lexicon_tags_29 },
    { 0x6ae5542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ee5546, SUC_lexicon_tags_25 },
    { 0xc42e5547, SUC_lexicon_tags_0 },
    { 0xc12e5548, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30e554c, SUC_lexicon_tags_26 },
    { 0x592e554d, SUC_lexicon_tags_26 },
    { 0x468e554c, SUC_lexicon_tags_38 },
    { 0xc4ae554e, SUC_lexicon_tags_12 },
    { 0xc9ce5550, SUC_lexicon_tags_9 },
    { 0x434e5551, SUC_lexicon_tags_21 },
    { 0x5b6e5552, SUC_lexicon_tags_39 },
    { 0xf86e5550, SUC_lexicon_tags_37 },
    { 0x146e554f, SUC_lexicon_tags_25 },
    { 0x360e5555, SUC_lexicon_tags_38 },
    { 0xe1ee554d, SUC_lexicon_tags_26 },
    { 0x424e5557, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60e555b, SUC_lexicon_tags_5 },
    { 0xdb4e555b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x70ae555e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x328e5560, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae5565, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf5ee5567, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ee556b, SUC_lexicon_tags_15 },
    { 0xf68e556c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf5ce556e, SUC_lexicon_tags_33 },
    { 0x38ae556f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x844e5571, SUC_lexicon_tags_37 },
    { 0x4f4e5571, SUC_lexicon_tags_61 },
    { 0x8b4e5571, SUC_lexicon_tags_30 },
    { 0x42ae5574, SUC_lexicon_tags_37 },
    { 0xd06e5575, SUC_lexicon_tags_25 },
    { 0x702e5573, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ae557b, SUC_lexicon_tags_11 },
    { 0x6ece557c, SUC_lexicon_tags_14 },
    { 0xfc8e557d, SUC_lexicon_tags_0 },
    { 0x9fae557d, SUC_lexicon_tags_0 },
    { 0x950e557f, SUC_lexicon_tags_45 },
    { 0xb3ee5580, SUC_lexicon_tags_11 },
    { 0x1ce5580, SUC_lexicon_tags_17 },
    { 0x2e0e5582, SUC_lexicon_tags_107 },
    { 0xdace5582, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x246e5585, SUC_lexicon_tags_61 },
    { 0xd6ee5586, SUC_lexicon_tags_10 },
    { 0x486e5586, SUC_lexicon_tags_37 },
    { 0x3ce5588, SUC_lexicon_tags_12 },
    { 0x93ee5586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340e558e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe5ce5590, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4e5593, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd6e5595, SUC_lexicon_tags_28 },
    { 0x8fae5596, SUC_lexicon_tags_25 },
    { 0x722e5596, SUC_lexicon_tags_17 },
    { 0x98ae5596, SUC_lexicon_tags_63 },
    { 0x95ce5599, SUC_lexicon_tags_62 },
    { 0x838e559a, SUC_lexicon_tags_37 },
    { 0xe2ae559a, SUC_lexicon_tags_14 },
    { 0x222e559b, SUC_lexicon_tags_33 },
    { 0xd80e559d, SUC_lexicon_tags_11 },
    { 0x30e559d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2e55a2, SUC_lexicon_tags_69 },
    { 0x4a8e55a3, SUC_lexicon_tags_9 },
    { 0x984e55a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x672e55a6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x986e55a8, SUC_lexicon_tags_1 },
    { 0xf9ae55a9, SUC_lexicon_tags_27 },
    { 0xbf0e55a9, SUC_lexicon_tags_15 },
    { 0x74e55ab, SUC_lexicon_tags_60 },
    { 0xa34e55ac, SUC_lexicon_tags_11 },
    { 0x2bae55a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92ee55af, SUC_lexicon_tags_26 },
    { 0xf02e55af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0e55b3, SUC_lexicon_tags_27 },
    { 0xa6ee55b4, SUC_lexicon_tags_17 },
    { 0xbb6e55b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbee55b7, SUC_lexicon_tags_46 },
    { 0x91ce55b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56ce55ba, SUC_lexicon_tags_15 },
    { 0x266e55bb, SUC_lexicon_tags_23 },
    { 0x14ee55bc, SUC_lexicon_tags_17 },
    { 0x57ae55bd, SUC_lexicon_tags_7 },
    { 0xe88e55bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4eee55c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9ae55c2, SUC_lexicon_tags_1 },
    { 0x4b6e55c3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xdeae55c5, SUC_lexicon_tags_1 },
    { 0x398e55c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4e55c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dce55ce, SUC_lexicon_tags_2 },
    { 0xdc6e55ce, SUC_lexicon_tags_25 },
    { 0x7b6e55d0, SUC_lexicon_tags_23 },
    { 0x388e55d0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaae55d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e55de, SUC_lexicon_tags_15 },
    { 0x298e55df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8e55e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ce55e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ce55ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8e55ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bce55f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34e55f4, SUC_lexicon_tags_17 },
    { 0xf7ee55f4, SUC_lexicon_tags_12 },
    { 0xf0ce55f6, SUC_lexicon_tags_25 },
    { 0x290e55f5, SUC_lexicon_tags_11 },
    { 0xd06e55f8, SUC_lexicon_tags_39 },
    { 0xb94e55f9, SUC_lexicon_tags_25 },
    { 0x702e55f9, SUC_lexicon_tags_17 },
    { 0x332e55fb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ece55fe, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf5ee5600, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5cce5602, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x144e5605, SUC_lexicon_tags_45 },
    { 0x464e5606, SUC_lexicon_tags_32 },
    { 0xb3ee5606, SUC_lexicon_tags_49 },
    { 0x16ce5607, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ee560d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818e5614, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2e5619, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586e561d, SUC_lexicon_tags_31 },
    { 0x686e561e, SUC_lexicon_tags_17 },
    { 0xbe8e561d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe68e5621, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ae5625, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4e562e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe32e5630, SUC_lexicon_tags_12 },
    { 0x1e6e5631, SUC_lexicon_tags_2 },
    { 0x9dce5632, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x432e5634, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ce563a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x144e563c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02e563f, SUC_lexicon_tags_43 },
    { 0xe12e563f, SUC_lexicon_tags_61 },
    { 0xca8e563f, SUC_lexicon_tags_28 },
    { 0x662e5642, SUC_lexicon_tags_0 },
    { 0x96ae5643, SUC_lexicon_tags_23 },
    { 0xf38e5644, SUC_lexicon_tags_14 },
    { 0xe6ee5642, SUC_lexicon_tags_1 },
    { 0x1b2e5646, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e564b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86e564f, SUC_lexicon_tags_21 },
    { 0x2f6e564f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974e5658, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52e565c, SUC_lexicon_tags_25 },
    { 0x8cce565d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe6e565f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cee5661, SUC_lexicon_tags_31 },
    { 0x2d0e5662, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966e566a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x674e566c, SUC_lexicon_tags_63 },
    { 0x43ee566d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20ae566f, SUC_lexicon_tags_27 },
    { 0x998e5670, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4e5675, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998e567c, SUC_lexicon_tags_19 },
    { 0xa6ae567d, SUC_lexicon_tags_23 },
    { 0x9f6e567d, SUC_lexicon_tags_37 },
    { 0xa8e567d, SUC_lexicon_tags_39 },
    { 0xbf4e5680, SUC_lexicon_tags_12 },
    { 0x838e5681, SUC_lexicon_tags_61 },
    { 0x21ee5681, SUC_lexicon_tags_5 },
    { 0x376e5683, SUC_lexicon_tags_12 },
    { 0xbc4e5684, SUC_lexicon_tags_25 },
    { 0x6f4e5683, SUC_lexicon_tags_15 },
    { 0x2cae567e, SUC_lexicon_tags_37 },
    { 0x876e567f, SUC_lexicon_tags_25 },
    { 0xd3ce5686, SUC_lexicon_tags_68 },
    { 0xb64e5689, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ee5690, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x570e5693, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0e5697, SUC_lexicon_tags_61 },
    { 0x8d4e5698, SUC_lexicon_tags_38 },
    { 0x412e5699, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x352e569b, SUC_lexicon_tags_10 },
    { 0xb4e569c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6e569f, SUC_lexicon_tags_2 },
    { 0x228e569f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a2e56a2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e56a5, SUC_lexicon_tags_71 },
    { 0xaae56a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8e56a9, SUC_lexicon_tags_60 },
    { 0x88e56a9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x7f8e56ac, SUC_lexicon_tags_11 },
    { 0xbece56ad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdbae56af, SUC_lexicon_tags_138 },
    { 0xa1ce56af, SUC_lexicon_tags_7 },
    { 0x868e56b1, SUC_lexicon_tags_11 },
    { 0xebae56b2, SUC_lexicon_tags_47 },
    { 0xd8e56b2, SUC_lexicon_tags_7 },
    { 0x978e56b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34e56bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300e56c0, SUC_lexicon_tags_15 },
    { 0xddce56c0, SUC_lexicon_tags_8 },
    { 0xd92e56c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ee56c6, SUC_lexicon_tags_17 },
    { 0x132e56c7, SUC_lexicon_tags_35 },
    { 0x446e56c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ae56cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9c8e56cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912e56d6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6fce56d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0e56e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdee56ea, SUC_lexicon_tags_17 },
    { 0xce56ea, SUC_lexicon_tags_25 },
    { 0x296e56eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ee56f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dee56fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8c6e56fc, SUC_lexicon_tags_7 },
    { 0xf90e56fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ae5705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ce5709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fee5710, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06e5713, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2e5718, SUC_lexicon_tags_30 },
    { 0xf04e5719, SUC_lexicon_tags_10 },
    { 0xa28e5718, SUC_lexicon_tags_21 },
    { 0xcdee571b, SUC_lexicon_tags_17 },
    { 0x506e571b, SUC_lexicon_tags_25 },
    { 0xcdae571d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba4e571f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824e5723, SUC_lexicon_tags_60 },
    { 0x472e5723, SUC_lexicon_tags_31 },
    { 0x632e5725, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbee5727, SUC_lexicon_tags_53 },
    { 0xe6e5728, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf28e572a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28e572f, SUC_lexicon_tags_17 },
    { 0x60e5730, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ee5734, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376e573b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf8e573d, SUC_lexicon_tags_17 },
    { 0xaa0e573e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86e5744, SUC_lexicon_tags_45 },
    { 0x92e5744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36e574a, SUC_lexicon_tags_17 },
    { 0x51ae574a, SUC_lexicon_tags_11 },
    { 0xcaae574a, SUC_lexicon_tags_63 },
    { 0xfcee574d, SUC_lexicon_tags_1 },
    { 0x7dce574b, SUC_lexicon_tags_43 },
    { 0x478e574f, SUC_lexicon_tags_1 },
    { 0x17ae574f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26e5756, SUC_lexicon_tags_10 },
    { 0x5ae5756, SUC_lexicon_tags_1 },
    { 0x776e5757, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a4e575a, SUC_lexicon_tags_47 },
    { 0x89ce575b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x894e575d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6e5760, SUC_lexicon_tags_25 },
    { 0x4b6e5761, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22e5767, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7ee5769, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x510e576b, SUC_lexicon_tags_37 },
    { 0x5f8e576c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ece576f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeee5772, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bae5774, SUC_lexicon_tags_1 },
    { 0x678e5775, SUC_lexicon_tags_38 },
    { 0x830e5774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee5779, SUC_lexicon_tags_12 },
    { 0x5bce5779, SUC_lexicon_tags_25 },
    { 0x9fce577b, SUC_lexicon_tags_8 },
    { 0x45ce577b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bae5786, SUC_lexicon_tags_17 },
    { 0x6d8e5787, SUC_lexicon_tags_148 },
    { 0x6a8e5787, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe26e578d, SUC_lexicon_tags_7 },
    { 0xe1ee578d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6e5796, SUC_lexicon_tags_49 },
    { 0x8f8e5797, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x65ee5799, SUC_lexicon_tags_45 },
    { 0xeae579a, SUC_lexicon_tags_48 },
    { 0xb90e579a, SUC_lexicon_tags_27 },
    { 0x150e579b, SUC_lexicon_tags_3 },
    { 0x90e579d, SUC_lexicon_tags_37 },
    { 0xbeae5799, SUC_lexicon_tags_1 },
    { 0x732e579f, SUC_lexicon_tags_11 },
    { 0x376e579c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2e57a4, SUC_lexicon_tags_25 },
    { 0xcd8e57a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbae57aa, SUC_lexicon_tags_1 },
    { 0x49ee57aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724e57b0, SUC_lexicon_tags_15 },
    { 0x9f4e57b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd04e57b3, SUC_lexicon_tags_47 },
    { 0xdfee57b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842e57b9, SUC_lexicon_tags_43 },
    { 0xec0e57ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x69ee57bc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x460e57be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28e57c4, SUC_lexicon_tags_9 },
    { 0x1aee57c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3c4e57c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564e57cc, SUC_lexicon_tags_17 },
    { 0x3c8e57cd, SUC_lexicon_tags_18 },
    { 0xbeae57ce, SUC_lexicon_tags_49 },
    { 0x376e57cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6dae57d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02e57d6, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x1b0e57d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2e57dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44ee57de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ee57e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x274e57e4, SUC_lexicon_tags_15 },
    { 0x430e57e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70e57eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x660e57ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34e57ef, SUC_lexicon_tags_38 },
    { 0x9e4e57ef, SUC_lexicon_tags_37 },
    { 0xd56e57f1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8e57f9, SUC_lexicon_tags_1 },
    { 0x742e57f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270e5806, SUC_lexicon_tags_63 },
    { 0x522e5807, SUC_lexicon_tags_15 },
    { 0x188e5808, SUC_lexicon_tags_37 },
    { 0x4e6e5809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd0e580b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e5811, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdce5813, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ace5819, SUC_lexicon_tags_6 },
    { 0x624e581a, SUC_lexicon_tags_37 },
    { 0x880e581a, SUC_lexicon_tags_21 },
    { 0xe74e581c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8bee581e, SUC_lexicon_tags_124 },
    { 0xc26e581e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426e5826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870e5830, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d8e5832, SUC_lexicon_tags_17 },
    { 0x316e5833, SUC_lexicon_tags_15 },
    { 0xd8ae5833, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x574e5836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53ce5838, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8e583c, SUC_lexicon_tags_14 },
    { 0xa3ee583d, SUC_lexicon_tags_114 },
    { 0x350e583e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x158e5841, SUC_lexicon_tags_45 },
    { 0x5cee5842, SUC_lexicon_tags_33 },
    { 0xb90e5843, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee584a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cee584d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284e5852, SUC_lexicon_tags_2 },
    { 0xaaae5852, SUC_lexicon_tags_0 },
    { 0xf9ee5854, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d6e5856, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x43ae5858, SUC_lexicon_tags_14 },
    { 0x148e5858, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e585f, SUC_lexicon_tags_34 },
    { 0x934e5860, SUC_lexicon_tags_87 },
    { 0xf2ee5860, SUC_lexicon_tags_17 },
    { 0x144e585f, SUC_lexicon_tags_15 },
    { 0xb84e5863, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ae5866, SUC_lexicon_tags_25 },
    { 0x276e5867, SUC_lexicon_tags_45 },
    { 0xc36e5868, SUC_lexicon_tags_188 },
    { 0x860e5868, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8e586c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57ce586e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ce5875, SUC_lexicon_tags_29 },
    { 0x4cee5876, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5fce5878, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3fce587a, SUC_lexicon_tags_64 },
    { 0x49ee587b, SUC_lexicon_tags_7 },
    { 0x4fee587c, SUC_lexicon_tags_14 },
    { 0x296e587d, SUC_lexicon_tags_82 },
    { 0x3b6e587e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xafee5880, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ae5884, SUC_lexicon_tags_37 },
    { 0x684e5885, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc72e5887, SUC_lexicon_tags_12 },
    { 0x226e5887, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x714e588a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34e5893, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2e589a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccce589c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6e589f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ae58a2, SUC_lexicon_tags_37 },
    { 0x944e58a2, SUC_lexicon_tags_34 },
    { 0xcb2e58a4, SUC_lexicon_tags_27 },
    { 0xe5ee58a5, SUC_lexicon_tags_70 },
    { 0xd6ce58a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x970e58a8, SUC_lexicon_tags_57 },
    { 0xf6ee58a9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6e58ac, SUC_lexicon_tags_16 },
    { 0xb50e58ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x48e58af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee58b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd42e58b7, SUC_lexicon_tags_15 },
    { 0xa02e58b8, SUC_lexicon_tags_19 },
    { 0x746e58b9, SUC_lexicon_tags_11 },
    { 0x910e58ba, SUC_lexicon_tags_65 },
    { 0x9a2e58bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526e58c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ce58c7, SUC_lexicon_tags_12 },
    { 0xe32e58c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cce58d1, SUC_lexicon_tags_17 },
    { 0x9d4e58d2, SUC_lexicon_tags_11 },
    { 0x7dee58d2, SUC_lexicon_tags_68 },
    { 0x9b2e58d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4e58dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9bce58de, SUC_lexicon_tags_16 },
    { 0xa2e58df, SUC_lexicon_tags_17 },
    { 0xd26e58e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200e58e4, SUC_lexicon_tags_13 },
    { 0x57ce58e4, SUC_lexicon_tags_15 },
    { 0xd8ee58e6, SUC_lexicon_tags_25 },
    { 0x86e58e7, SUC_lexicon_tags_28 },
    { 0x4e58e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc16e58ea, SUC_lexicon_tags_46 },
    { 0x1e2e58ea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x74ce58ed, SUC_lexicon_tags_9 },
    { 0xe44e58ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84e58f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402e58f5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ae58f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee5900, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x208e5903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x702e5906, SUC_lexicon_tags_17 },
    { 0x928e5907, SUC_lexicon_tags_43 },
    { 0xad6e5908, SUC_lexicon_tags_25 },
    { 0x8ee5907, SUC_lexicon_tags_10 },
    { 0x75ee590a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x154e590d, SUC_lexicon_tags_37 },
    { 0x76e590d, SUC_lexicon_tags_56 },
    { 0xfe6e590f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80e5913, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ce5916, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdae591c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72e5922, SUC_lexicon_tags_0 },
    { 0x396e5923, SUC_lexicon_tags_45 },
    { 0xe32e5924, SUC_lexicon_tags_15 },
    { 0x6ace5924, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6e5928, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0e592d, SUC_lexicon_tags_87 },
    { 0x556e592e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6e5932, SUC_lexicon_tags_12 },
    { 0x108e5932, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8e5937, SUC_lexicon_tags_11 },
    { 0xc8ee5937, SUC_lexicon_tags_53 },
    { 0xdeee5939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ce593f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ce5942, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x48e5944, SUC_lexicon_tags_47 },
    { 0xa9ae5944, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x518e5947, SUC_lexicon_tags_63 },
    { 0xe64e5948, SUC_lexicon_tags_25 },
    { 0x5b8e5948, SUC_lexicon_tags_15 },
    { 0xe60e594a, SUC_lexicon_tags_47 },
    { 0x6a6e5948, SUC_lexicon_tags_26 },
    { 0x608e594b, SUC_lexicon_tags_11 },
    { 0x1ace594d, SUC_lexicon_tags_2 },
    { 0x216e594e, SUC_lexicon_tags_14 },
    { 0xf8ce594f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6e5955, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16e5959, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x818e595c, SUC_lexicon_tags_9 },
    { 0x7ce595c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaae5961, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320e5967, SUC_lexicon_tags_8 },
    { 0x116e5967, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08e596d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0e5972, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ce5975, SUC_lexicon_tags_43 },
    { 0x7fee5975, SUC_lexicon_tags_131 },
    { 0x5b0e5977, SUC_lexicon_tags_43 },
    { 0x554e5978, SUC_lexicon_tags_8 },
    { 0xb44e5977, SUC_lexicon_tags_216 },
    { 0xe90e597a, SUC_lexicon_tags_11 },
    { 0x480e597a, SUC_lexicon_tags_27 },
    { 0x7e8e597c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cce597e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ae598c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b6e598e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662e5992, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa86e5994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x100e5996, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622e599c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebce599e, SUC_lexicon_tags_56 },
    { 0x688e599f, SUC_lexicon_tags_2 },
    { 0x536e599f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x618e59a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ace59a6, SUC_lexicon_tags_1 },
    { 0x81ee59a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ace59b0, SUC_lexicon_tags_6 },
    { 0x8e59b1, SUC_lexicon_tags_178 },
    { 0xc0ce59b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7bee59b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3fee59b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfdce59b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee59bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x498e59bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4e59c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x72ae59c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x920e59c4, SUC_lexicon_tags_27 },
    { 0x50ae59c4, SUC_lexicon_tags_35 },
    { 0xd6ae59c4, SUC_lexicon_tags_14 },
    { 0x2a8e59c4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6e59ca, SUC_lexicon_tags_60 },
    { 0x1dce59cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee59ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75ce59d0, SUC_lexicon_tags_9 },
    { 0xea0e59d0, SUC_lexicon_tags_31 },
    { 0x9d0e59d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee59d6, SUC_lexicon_tags_10 },
    { 0xff6e59d7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ee59db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x896e59de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cee59e8, SUC_lexicon_tags_0 },
    { 0xf76e59e9, SUC_lexicon_tags_49 },
    { 0xa08e59ea, SUC_lexicon_tags_2 },
    { 0xef8e59eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cee59ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4e59f2, SUC_lexicon_tags_12 },
    { 0xa92e59f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80ce59f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce59f8, SUC_lexicon_tags_57 },
    { 0x3e6e59f8, SUC_lexicon_tags_43 },
    { 0x3cee59f9, SUC_lexicon_tags_7 },
    { 0xed6e59f8, SUC_lexicon_tags_38 },
    { 0xf82e59fc, SUC_lexicon_tags_17 },
    { 0xdb6e59fc, SUC_lexicon_tags_2 },
    { 0x14e59f9, SUC_lexicon_tags_45 },
    { 0x3a4e59fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4e5a04, SUC_lexicon_tags_45 },
    { 0xb96e5a05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcece5a08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30e5a0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4e5a0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e5a12, SUC_lexicon_tags_47 },
    { 0x16ae5a13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x50ce5a15, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x510e5a18, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae5a1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e5a24, SUC_lexicon_tags_17 },
    { 0x6aae5a25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cae5a27, SUC_lexicon_tags_12 },
    { 0x1c0e5a28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0e5a2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefae5a30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x158e5a32, SUC_lexicon_tags_65 },
    { 0x1ee5a33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4e5a37, SUC_lexicon_tags_26 },
    { 0xc12e5a38, SUC_lexicon_tags_11 },
    { 0x8f8e5a39, SUC_lexicon_tags_27 },
    { 0x1b4e5a39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf2ee5a3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x674e5a3e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x364e5a40, SUC_lexicon_tags_0 },
    { 0xf72e5a41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x494e5a43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ae5a46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4e5a4a, SUC_lexicon_tags_15 },
    { 0xc16e5a4b, SUC_lexicon_tags_125 },
    { 0xc14e5a4b, SUC_lexicon_tags_55 },
    { 0xc5ce5a4d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ee5a52, SUC_lexicon_tags_1 },
    { 0xe56e5a52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fee5a58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16e5a5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dee5a5f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x430e5a61, SUC_lexicon_tags_43 },
    { 0xa4ce5a62, SUC_lexicon_tags_0 },
    { 0x99ae5a63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa2e5a65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8e5a68, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ae5a6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe80e5a6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6e5a78, SUC_lexicon_tags_6 },
    { 0x37ce5a79, SUC_lexicon_tags_11 },
    { 0xb90e5a7a, SUC_lexicon_tags_25 },
    { 0x802e5a7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfee5a7f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ee5a84, SUC_lexicon_tags_60 },
    { 0x1b6e5a85, SUC_lexicon_tags_12 },
    { 0x1fae5a86, SUC_lexicon_tags_25 },
    { 0x2a4e5a87, SUC_lexicon_tags_17 },
    { 0x22e5a88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xecae5a8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4e5a8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2e5a91, SUC_lexicon_tags_30 },
    { 0x14ce5a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd70e5a95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660e5a9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ae5a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa40e5aa1, SUC_lexicon_tags_37 },
    { 0xf3ae5aa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2ee5aa4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x218e5aa6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378e5aaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8e5ab2, SUC_lexicon_tags_2 },
    { 0xb78e5ab2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x506e5ab6, SUC_lexicon_tags_7 },
    { 0xff2e5ab7, SUC_lexicon_tags_38 },
    { 0x4e2e5ab6, SUC_lexicon_tags_1 },
    { 0x370e5ab9, SUC_lexicon_tags_14 },
    { 0x16ce5aba, SUC_lexicon_tags_12 },
    { 0x9ace5abb, SUC_lexicon_tags_25 },
    { 0xe36e5abb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ae5abf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ae5ac2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x382e5ac4, SUC_lexicon_tags_7 },
    { 0xb66e5ac5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140e5ac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8e5acd, SUC_lexicon_tags_23 },
    { 0x598e5acd, SUC_lexicon_tags_17 },
    { 0xfece5acf, SUC_lexicon_tags_11 },
    { 0xb14e5ace, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2e5ad4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x312e5ad6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3f2e5ad8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68e5adb, SUC_lexicon_tags_104 },
    { 0x90ce5adc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b8e5ade, SUC_lexicon_tags_15 },
    { 0x204e5ade, SUC_lexicon_tags_9 },
    { 0xf2ee5ae0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9bae5ae2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39ae5ae4, SUC_lexicon_tags_16 },
    { 0xaaee5ae5, SUC_lexicon_tags_17 },
    { 0x4ace5ae6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87ae5ae8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9ee5aea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e5aee, SUC_lexicon_tags_55 },
    { 0xfa2e5aef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece5af3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aae5af5, SUC_lexicon_tags_17 },
    { 0x710e5af6, SUC_lexicon_tags_37 },
    { 0xc42e5af7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ee5afd, SUC_lexicon_tags_1 },
    { 0xd9ee5afe, SUC_lexicon_tags_7 },
    { 0xf30e5aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4e5b03, SUC_lexicon_tags_47 },
    { 0x21ae5b04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40e5b07, SUC_lexicon_tags_0 },
    { 0x634e5b08, SUC_lexicon_tags_37 },
    { 0x5cee5b09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7b0e5b0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x852e5b0e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdae5b13, SUC_lexicon_tags_65 },
    { 0x84ee5b13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12e5b18, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dce5b1b, SUC_lexicon_tags_49 },
    { 0x77ee5b1c, SUC_lexicon_tags_43 },
    { 0x23ae5b1d, SUC_lexicon_tags_11 },
    { 0x1f8e5b1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84e5b21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ee5b24, SUC_lexicon_tags_14 },
    { 0x662e5b25, SUC_lexicon_tags_2 },
    { 0x7fce5b26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x244e5b28, SUC_lexicon_tags_6 },
    { 0xdc6e5b29, SUC_lexicon_tags_49 },
    { 0x9f8e5b2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ce5b2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440e5b33, SUC_lexicon_tags_47 },
    { 0x812e5b34, SUC_lexicon_tags_17 },
    { 0x96e5b34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd6e5b37, SUC_lexicon_tags_1 },
    { 0xf34e5b37, SUC_lexicon_tags_15 },
    { 0xabce5b39, SUC_lexicon_tags_1 },
    { 0x676e5b39, SUC_lexicon_tags_38 },
    { 0x416e5b3b, SUC_lexicon_tags_30 },
    { 0x6d2e5b3c, SUC_lexicon_tags_151 },
    { 0x274e5b3d, SUC_lexicon_tags_75 },
    { 0x65ce5b3d, SUC_lexicon_tags_25 },
    { 0xd80e5b3f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddee5b47, SUC_lexicon_tags_1 },
    { 0x6e4e5b47, SUC_lexicon_tags_65 },
    { 0x336e5b49, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932e5b4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ae5b58, SUC_lexicon_tags_26 },
    { 0x738e5b59, SUC_lexicon_tags_60 },
    { 0x180e5b5a, SUC_lexicon_tags_49 },
    { 0xc5ce5b5b, SUC_lexicon_tags_34 },
    { 0x760e5b5a, SUC_lexicon_tags_9 },
    { 0x104e5b5d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e5b61, SUC_lexicon_tags_10 },
    { 0x5e0e5b62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10e5b67, SUC_lexicon_tags_10 },
    { 0xcb4e5b68, SUC_lexicon_tags_2 },
    { 0x4dae5b69, SUC_lexicon_tags_29 },
    { 0x72e5b6a, SUC_lexicon_tags_21 },
    { 0x350e5b6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc24e5b6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xef2e5b6f, SUC_lexicon_tags_7 },
    { 0x6ce5b70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90e5b75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e6e5b77, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ee5b7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8e5b80, SUC_lexicon_tags_37 },
    { 0x4dae5b80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x45ee5b83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x516e5b85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46e5b88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x442e5b8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8e5b8e, SUC_lexicon_tags_45 },
    { 0x11ce5b8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbce5b97, SUC_lexicon_tags_0 },
    { 0x4dce5b98, SUC_lexicon_tags_27 },
    { 0xd94e5b99, SUC_lexicon_tags_47 },
    { 0x9e2e5b99, SUC_lexicon_tags_60 },
    { 0x3fae5b97, SUC_lexicon_tags_11 },
    { 0x7ae5b9a, SUC_lexicon_tags_3 },
    { 0xce2e5b9d, SUC_lexicon_tags_28 },
    { 0x570e5b9e, SUC_lexicon_tags_16 },
    { 0x43ee5b9f, SUC_lexicon_tags_46 },
    { 0x54ae5ba0, SUC_lexicon_tags_2 },
    { 0x7ce5b9d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280e5ba7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x84ae5ba9, SUC_lexicon_tags_11 },
    { 0x31ce5baa, SUC_lexicon_tags_45 },
    { 0x166e5baa, SUC_lexicon_tags_37 },
    { 0xa86e5bab, SUC_lexicon_tags_17 },
    { 0xc0ee5bad, SUC_lexicon_tags_15 },
    { 0x5ce5bae, SUC_lexicon_tags_8 },
    { 0x6bce5bae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbae5bb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e5bb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bae5bba, SUC_lexicon_tags_15 },
    { 0xd7ce5bbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x660e5bbd, SUC_lexicon_tags_2 },
    { 0xa98e5bbe, SUC_lexicon_tags_37 },
    { 0xe54e5bbd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e5bc3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ce5bc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd08e5bcb, SUC_lexicon_tags_65 },
    { 0x324e5bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec6e5bce, SUC_lexicon_tags_65 },
    { 0xeace5bcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cce5bd1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ae5bd6, SUC_lexicon_tags_49 },
    { 0x47ae5bd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x118e5bd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdce5bdc, SUC_lexicon_tags_13 },
    { 0xab0e5bdd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8e5be1, SUC_lexicon_tags_17 },
    { 0x6cce5be2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c4e5be4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2e5be9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46ae5beb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988e5bef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2e5bf2, SUC_lexicon_tags_10 },
    { 0x342e5bf3, SUC_lexicon_tags_1 },
    { 0x508e5bf4, SUC_lexicon_tags_34 },
    { 0x900e5bf5, SUC_lexicon_tags_37 },
    { 0xe08e5bf5, SUC_lexicon_tags_1 },
    { 0x392e5bf6, SUC_lexicon_tags_30 },
    { 0x61ce5bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x608e5bfa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458e5bfe, SUC_lexicon_tags_9 },
    { 0xab4e5bfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x278e5c01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ae5c04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412e5c0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x694e5c0c, SUC_lexicon_tags_3 },
    { 0xac4e5c0d, SUC_lexicon_tags_18 },
    { 0xbce5c0e, SUC_lexicon_tags_26 },
    { 0xe8ae5c0d, SUC_lexicon_tags_1368 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236e5c13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4e5c18, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46e5c1e, SUC_lexicon_tags_43 },
    { 0x69ae5c1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46e5c23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6e5c25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ae5c2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0e5c34, SUC_lexicon_tags_11 },
    { 0x648e5c35, SUC_lexicon_tags_11 },
    { 0x60ce5c36, SUC_lexicon_tags_19 },
    { 0xbd2e5c36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x508e5c39, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcce5c3c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ae5c40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ee5c44, SUC_lexicon_tags_25 },
    { 0xcce5c44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4e5c48, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ce5c4c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32e5c50, SUC_lexicon_tags_68 },
    { 0x710e5c51, SUC_lexicon_tags_66 },
    { 0x460e5c52, SUC_lexicon_tags_37 },
    { 0x9f8e5c53, SUC_lexicon_tags_38 },
    { 0x506e5c54, SUC_lexicon_tags_19 },
    { 0x542e5c53, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48e5c5a, SUC_lexicon_tags_9 },
    { 0x1f6e5c5b, SUC_lexicon_tags_25 },
    { 0x47ae5c5b, SUC_lexicon_tags_43 },
    { 0x10ee5c5d, SUC_lexicon_tags_6 },
    { 0x42e5c5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95ce5c60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d2e5c62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eae5c6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ae5c73, SUC_lexicon_tags_0 },
    { 0x142e5c74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756e5c7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x650e5c7e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ee5c85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96e5c8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24e5c90, SUC_lexicon_tags_17 },
    { 0xcb6e5c91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9ce5c93, SUC_lexicon_tags_63 },
    { 0xe04e5c94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecee5c98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62e5c9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc36e5c9d, SUC_lexicon_tags_11 },
    { 0x548e5c9d, SUC_lexicon_tags_37 },
    { 0xd44e5c9e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782e5ca5, SUC_lexicon_tags_7 },
    { 0x1a2e5ca5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb4ee5ca8, SUC_lexicon_tags_17 },
    { 0x86ee5ca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ace5caf, SUC_lexicon_tags_15 },
    { 0xb4ee5caf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee5cb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x960e5cb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ee5cba, SUC_lexicon_tags_8 },
    { 0x664e5cbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x634e5cbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95ae5cbf, SUC_lexicon_tags_63 },
    { 0xc3ae5cc0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ae5cc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacce5cc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00e5ccb, SUC_lexicon_tags_179 },
    { 0x206e5ccc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0e5ccf, SUC_lexicon_tags_15 },
    { 0x3cce5ccf, SUC_lexicon_tags_11 },
    { 0xe1ee5cd1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd50e5cd3, SUC_lexicon_tags_2 },
    { 0xd18e5cd4, SUC_lexicon_tags_8 },
    { 0xd26e5cd5, SUC_lexicon_tags_7 },
    { 0x3b6e5cd6, SUC_lexicon_tags_36 },
    { 0xfdce5cd6, SUC_lexicon_tags_47 },
    { 0x1e4e5cd8, SUC_lexicon_tags_7 },
    { 0xa64e5cd5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6d6e5cdb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcee5ce1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ce5ce4, SUC_lexicon_tags_0 },
    { 0xa2ce5ce5, SUC_lexicon_tags_63 },
    { 0xc6ae5ce6, SUC_lexicon_tags_45 },
    { 0x454e5ce4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdce5cea, SUC_lexicon_tags_1121 },
    { 0, NULL },
    { 0x768e5cec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16e5cef, SUC_lexicon_tags_13 },
    { 0x31ae5cef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26e5cf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe2e5cf5, SUC_lexicon_tags_17 },
    { 0x78ae5cf6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e5d06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ae5d0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cae5d0f, SUC_lexicon_tags_15 },
    { 0x878e5d10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x654e5d13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3eae5d15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4e5d1c, SUC_lexicon_tags_22 },
    { 0x43ce5d1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2dae5d1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ae5d25, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbcae5d27, SUC_lexicon_tags_710 },
    { 0, NULL },
    { 0, NULL },
    { 0x82e5d2a, SUC_lexicon_tags_11 },
    { 0xcd8e5d2b, SUC_lexicon_tags_30 },
    { 0xfcae5d2c, SUC_lexicon_tags_17 },
    { 0x528e5d2d, SUC_lexicon_tags_25 },
    { 0xfdee5d2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96e5d31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd60e5d33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce5d38, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x140e5d3a, SUC_lexicon_tags_87 },
    { 0x602e5d3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308e5d3f, SUC_lexicon_tags_12 },
    { 0xb26e5d40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19ce5d42, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ae5d45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e5d4c, SUC_lexicon_tags_2 },
    { 0x640e5d4d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabee5d51, SUC_lexicon_tags_35 },
    { 0x142e5d52, SUC_lexicon_tags_45 },
    { 0xa12e5d52, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xffee5d56, SUC_lexicon_tags_105 },
    { 0xed0e5d56, SUC_lexicon_tags_43 },
    { 0x5a0e5d58, SUC_lexicon_tags_43 },
    { 0xf76e5d58, SUC_lexicon_tags_15 },
    { 0x122e5d57, SUC_lexicon_tags_37 },
    { 0x15ce5d58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc6e5d5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x208e5d5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xac4e5d61, SUC_lexicon_tags_1 },
    { 0x7ece5d62, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbae5d6c, SUC_lexicon_tags_37 },
    { 0x204e5d6d, SUC_lexicon_tags_23 },
    { 0x93ce5d6d, SUC_lexicon_tags_47 },
    { 0xc96e5d6f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9fee5d71, SUC_lexicon_tags_17 },
    { 0xe14e5d71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x724e5d74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4e5d7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78e5d7f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0e5d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e5d86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ee5d8c, SUC_lexicon_tags_39 },
    { 0x20ce5d8d, SUC_lexicon_tags_17 },
    { 0xdbae5d8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd4e5d90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eae5d94, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x90ee5d96, SUC_lexicon_tags_68 },
    { 0x20ee5d97, SUC_lexicon_tags_33 },
    { 0x812e5d98, SUC_lexicon_tags_47 },
    { 0xef4e5d98, SUC_lexicon_tags_37 },
    { 0xbdee5d96, SUC_lexicon_tags_19 },
    { 0x506e5d96, SUC_lexicon_tags_37 },
    { 0xcf8e5d9c, SUC_lexicon_tags_66 },
    { 0xa7ae5d9d, SUC_lexicon_tags_11 },
    { 0xe62e5d9d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6e5da1, SUC_lexicon_tags_1 },
    { 0xbb6e5da1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cce5da5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ae5dad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ae5db0, SUC_lexicon_tags_32 },
    { 0x3bce5db0, SUC_lexicon_tags_23 },
    { 0xf4ce5db2, SUC_lexicon_tags_37 },
    { 0xca0e5db3, SUC_lexicon_tags_60 },
    { 0xf80e5db0, SUC_lexicon_tags_3 },
    { 0xea2e5db0, SUC_lexicon_tags_25 },
    { 0x4e4e5db6, SUC_lexicon_tags_27 },
    { 0xa04e5db7, SUC_lexicon_tags_11 },
    { 0x70e5db8, SUC_lexicon_tags_17 },
    { 0xf0e5db9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22ce5dbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd8e5dbd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614e5dc7, SUC_lexicon_tags_1 },
    { 0xb5ce5dc7, SUC_lexicon_tags_87 },
    { 0x56ae5dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x788e5dcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10e5dcf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x17ee5dd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0e5dd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14e5ddb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8e5de1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8e5de7, SUC_lexicon_tags_17 },
    { 0xaae5de8, SUC_lexicon_tags_61 },
    { 0x6d2e5de8, SUC_lexicon_tags_7 },
    { 0xd8ee5de9, SUC_lexicon_tags_18 },
    { 0xa74e5de8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadee5dee, SUC_lexicon_tags_39 },
    { 0xcf0e5def, SUC_lexicon_tags_9 },
    { 0x80e5def, SUC_lexicon_tags_1 },
    { 0x78ee5df1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6e5df4, SUC_lexicon_tags_7 },
    { 0x31ce5df5, SUC_lexicon_tags_57 },
    { 0xee8e5df6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa50e5df8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x266e5dfa, SUC_lexicon_tags_17 },
    { 0xed8e5dfb, SUC_lexicon_tags_45 },
    { 0x88e5dfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656e5e05, SUC_lexicon_tags_38 },
    { 0xba2e5e06, SUC_lexicon_tags_47 },
    { 0x6c0e5e07, SUC_lexicon_tags_4 },
    { 0xa76e5e08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826e5e0c, SUC_lexicon_tags_60 },
    { 0xa4ee5e0c, SUC_lexicon_tags_13 },
    { 0x338e5e0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92e5e11, SUC_lexicon_tags_38 },
    { 0xcc2e5e12, SUC_lexicon_tags_15 },
    { 0x902e5e12, SUC_lexicon_tags_64 },
    { 0xeb8e5e13, SUC_lexicon_tags_15 },
    { 0x586e5e15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x906e5e17, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dee5e1b, SUC_lexicon_tags_11 },
    { 0xeace5e1b, SUC_lexicon_tags_0 },
    { 0xe9ee5e1d, SUC_lexicon_tags_4 },
    { 0xca6e5e1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x934e5e20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54ee5e22, SUC_lexicon_tags_160 },
    { 0x71ee5e22, SUC_lexicon_tags_19 },
    { 0xca0e5e24, SUC_lexicon_tags_95 },
    { 0x18ae5e22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ae5e29, SUC_lexicon_tags_31 },
    { 0xd68e5e2a, SUC_lexicon_tags_14 },
    { 0x6e5e2b, SUC_lexicon_tags_30 },
    { 0xd3ce5e2c, SUC_lexicon_tags_13 },
    { 0x238e5e2a, SUC_lexicon_tags_20 },
    { 0xb7ce5e2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604e5e34, SUC_lexicon_tags_70 },
    { 0x90ee5e34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x282e5e38, SUC_lexicon_tags_11 },
    { 0x52ee5e39, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x352e5e3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ee5e3f, SUC_lexicon_tags_37 },
    { 0x84e5e3f, SUC_lexicon_tags_17 },
    { 0x7aae5e40, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908e5e45, SUC_lexicon_tags_25 },
    { 0x4f2e5e45, SUC_lexicon_tags_2 },
    { 0x9a2e5e46, SUC_lexicon_tags_31 },
    { 0x338e5e46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x710e5e4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe84e5e4c, SUC_lexicon_tags_710 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28e5e50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ee5e58, SUC_lexicon_tags_29 },
    { 0x5a2e5e59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x224e5e5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ee5e5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650e5e65, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadae5e69, SUC_lexicon_tags_17 },
    { 0x534e5e69, SUC_lexicon_tags_1 },
    { 0xe18e5e6b, SUC_lexicon_tags_37 },
    { 0x254e5e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64e5e70, SUC_lexicon_tags_47 },
    { 0xa86e5e70, SUC_lexicon_tags_33 },
    { 0x86ee5e70, SUC_lexicon_tags_5 },
    { 0xfe4e5e72, SUC_lexicon_tags_49 },
    { 0xcce5e74, SUC_lexicon_tags_2 },
    { 0x478e5e70, SUC_lexicon_tags_46 },
    { 0x48ce5e76, SUC_lexicon_tags_27 },
    { 0x950e5e76, SUC_lexicon_tags_37 },
    { 0x33ae5e78, SUC_lexicon_tags_85 },
    { 0x394e5e79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08e5e7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaae5e85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2e5e8b, SUC_lexicon_tags_38 },
    { 0x222e5e8c, SUC_lexicon_tags_2 },
    { 0x696e5e8d, SUC_lexicon_tags_23 },
    { 0x462e5e8e, SUC_lexicon_tags_37 },
    { 0xc4ae5e8c, SUC_lexicon_tags_84 },
    { 0x53ce5e90, SUC_lexicon_tags_7 },
    { 0x6eee5e8c, SUC_lexicon_tags_1 },
    { 0x9e0e5e90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612e5e98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89ee5e9a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62e5e9f, SUC_lexicon_tags_38 },
    { 0x5dae5ea0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eee5ea5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548e5eac, SUC_lexicon_tags_28 },
    { 0x520e5eac, SUC_lexicon_tags_3 },
    { 0x6d8e5ead, SUC_lexicon_tags_11 },
    { 0x678e5ead, SUC_lexicon_tags_60 },
    { 0x190e5eaf, SUC_lexicon_tags_7 },
    { 0x758e5eac, SUC_lexicon_tags_10 },
    { 0x984e5eb2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x464e5eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ce5eb7, SUC_lexicon_tags_70 },
    { 0x61ee5eb7, SUC_lexicon_tags_4 },
    { 0xddae5eb9, SUC_lexicon_tags_354 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6e5ebc, SUC_lexicon_tags_37 },
    { 0xbd2e5ebd, SUC_lexicon_tags_19 },
    { 0x620e5ebc, SUC_lexicon_tags_104 },
    { 0x52ae5ebf, SUC_lexicon_tags_65 },
    { 0xc64e5ec0, SUC_lexicon_tags_11 },
    { 0xc74e5ec0, SUC_lexicon_tags_27 },
    { 0xeb8e5ec2, SUC_lexicon_tags_37 },
    { 0xb6ae5ec2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbece5ecb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ce5ecf, SUC_lexicon_tags_9 },
    { 0xef4e5ed0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808e5ed9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568e5edd, SUC_lexicon_tags_14 },
    { 0x5d8e5edd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae5ee1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabce5ee6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc6e5ee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa22e5eea, SUC_lexicon_tags_17 },
    { 0x71ee5eeb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa98e5eed, SUC_lexicon_tags_4 },
    { 0xf06e5eee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510e5ef4, SUC_lexicon_tags_68 },
    { 0x7f2e5ef5, SUC_lexicon_tags_25 },
    { 0x2d2e5ef5, SUC_lexicon_tags_47 },
    { 0x1b6e5ef6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf7ee5ef9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80e5efe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76e5f06, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30e5f0e, SUC_lexicon_tags_2 },
    { 0xfb6e5f0f, SUC_lexicon_tags_33 },
    { 0x5e2e5f0e, SUC_lexicon_tags_11 },
    { 0x46ae5f11, SUC_lexicon_tags_11 },
    { 0xee8e5f10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60ae5f14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66e5f17, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8e5f1a, SUC_lexicon_tags_25 },
    { 0xb1ce5f1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ae5f22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e5f24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x892e5f26, SUC_lexicon_tags_12 },
    { 0x2c6e5f27, SUC_lexicon_tags_18 },
    { 0x8eae5f26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d2e5f2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x122e5f2c, SUC_lexicon_tags_47 },
    { 0x802e5f2c, SUC_lexicon_tags_16 },
    { 0xcd6e5f2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eae5f31, SUC_lexicon_tags_17 },
    { 0xc20e5f31, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x84e5f35, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ae5f3c, SUC_lexicon_tags_11 },
    { 0x604e5f3d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe90e5f3f, SUC_lexicon_tags_47 },
    { 0x8bee5f40, SUC_lexicon_tags_2 },
    { 0xb4ee5f40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532e5f47, SUC_lexicon_tags_49 },
    { 0x366e5f47, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38e5f4d, SUC_lexicon_tags_17 },
    { 0xe40e5f4e, SUC_lexicon_tags_17 },
    { 0xa2ae5f4e, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x94e5f52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59ce5f54, SUC_lexicon_tags_49 },
    { 0x42e5f54, SUC_lexicon_tags_11 },
    { 0xe2ae5f54, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1dae5f58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaae5f5b, SUC_lexicon_tags_11 },
    { 0x914e5f5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa34e5f5e, SUC_lexicon_tags_25 },
    { 0xa1ae5f5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4e5f63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d8e5f65, SUC_lexicon_tags_10 },
    { 0x610e5f66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0e5f6d, SUC_lexicon_tags_29 },
    { 0x14ce5f6e, SUC_lexicon_tags_47 },
    { 0x2e4e5f6f, SUC_lexicon_tags_21 },
    { 0xe10e5f70, SUC_lexicon_tags_2 },
    { 0x6b4e5f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x924e5f74, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ee5f7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x366e5f7e, SUC_lexicon_tags_0 },
    { 0x386e5f7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x402e5f82, SUC_lexicon_tags_11 },
    { 0x99ae5f83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ee5f89, SUC_lexicon_tags_37 },
    { 0x346e5f89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee5f8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1ce5f8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162e5f93, SUC_lexicon_tags_37 },
    { 0x230e5f93, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ae5f9d, SUC_lexicon_tags_37 },
    { 0x4e4e5f9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70ee5fa0, SUC_lexicon_tags_11 },
    { 0x6f6e5fa0, SUC_lexicon_tags_15 },
    { 0x61ce5fa0, SUC_lexicon_tags_17 },
    { 0xcae5fa2, SUC_lexicon_tags_16 },
    { 0x5b6e5fa4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2e5fa8, SUC_lexicon_tags_93 },
    { 0x868e5fa9, SUC_lexicon_tags_79 },
    { 0xa5ae5faa, SUC_lexicon_tags_2 },
    { 0x274e5fab, SUC_lexicon_tags_60 },
    { 0x6fae5fa8, SUC_lexicon_tags_70 },
    { 0xbf0e5fad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fce5fb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd8e5fb2, SUC_lexicon_tags_29 },
    { 0xff0e5fb2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x120e5fb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfee5fb9, SUC_lexicon_tags_17 },
    { 0x39ce5fba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x112e5fbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd62e5fbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2e5fc3, SUC_lexicon_tags_16 },
    { 0xf64e5fc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb66e5fc6, SUC_lexicon_tags_106 },
    { 0x7cce5fc6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8cae5fc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c4e5fcb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6e5fce, SUC_lexicon_tags_80 },
    { 0xeb8e5fce, SUC_lexicon_tags_15 },
    { 0x970e5fd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ae5fd4, SUC_lexicon_tags_37 },
    { 0x17ce5fd4, SUC_lexicon_tags_25 },
    { 0xaeae5fd4, SUC_lexicon_tags_1 },
    { 0xf86e5fd7, SUC_lexicon_tags_61 },
    { 0xef0e5fd8, SUC_lexicon_tags_8 },
    { 0x8ece5fd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cae5fde, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14e5fe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58e5fe5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ce5fea, SUC_lexicon_tags_37 },
    { 0xdb2e5feb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x60ae5fed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90e5ff2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x392e5ff4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xddce5ff6, SUC_lexicon_tags_91 },
    { 0xab6e5ff6, SUC_lexicon_tags_29 },
    { 0x43ee5ff8, SUC_lexicon_tags_25 },
    { 0x7a4e5ff9, SUC_lexicon_tags_151 },
    { 0x832e5ff9, SUC_lexicon_tags_14 },
    { 0x38ee5ffb, SUC_lexicon_tags_25 },
    { 0x772e5ffb, SUC_lexicon_tags_8 },
    { 0x3a8e5ff8, SUC_lexicon_tags_91 },
    { 0xf26e5ffe, SUC_lexicon_tags_49 },
    { 0x19ee5fff, SUC_lexicon_tags_8 },
    { 0x276e5fff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114e6007, SUC_lexicon_tags_39 },
    { 0x94e6008, SUC_lexicon_tags_26 },
    { 0x294e6009, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6e6010, SUC_lexicon_tags_17 },
    { 0xb62e6011, SUC_lexicon_tags_3 },
    { 0x852e6012, SUC_lexicon_tags_9 },
    { 0x912e6010, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0e601d, SUC_lexicon_tags_14 },
    { 0x200e601e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ce6025, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb4e6027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4e602d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e6030, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcce6034, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88e6038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ece603a, SUC_lexicon_tags_60 },
    { 0xe64e603a, SUC_lexicon_tags_30 },
    { 0xab6e603b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4e6040, SUC_lexicon_tags_1 },
    { 0x238e6041, SUC_lexicon_tags_7 },
    { 0x344e6041, SUC_lexicon_tags_47 },
    { 0xfc4e6043, SUC_lexicon_tags_37 },
    { 0x62ae6043, SUC_lexicon_tags_47 },
    { 0x8e2e6045, SUC_lexicon_tags_1 },
    { 0xd96e6045, SUC_lexicon_tags_17 },
    { 0xa50e6046, SUC_lexicon_tags_19 },
    { 0xa74e6048, SUC_lexicon_tags_17 },
    { 0x630e6043, SUC_lexicon_tags_12 },
    { 0xce6e604a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9e0e604c, SUC_lexicon_tags_16 },
    { 0x3c0e604d, SUC_lexicon_tags_11 },
    { 0xb46e604e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ae6052, SUC_lexicon_tags_16 },
    { 0x20ce6052, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2e6056, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xefce6058, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc12e605a, SUC_lexicon_tags_25 },
    { 0xc86e605b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e605e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfce6060, SUC_lexicon_tags_7 },
    { 0xcd6e6060, SUC_lexicon_tags_1 },
    { 0xb24e6062, SUC_lexicon_tags_87 },
    { 0xf98e6061, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246e606f, SUC_lexicon_tags_15 },
    { 0x7d2e6070, SUC_lexicon_tags_47 },
    { 0x736e6071, SUC_lexicon_tags_65 },
    { 0xe98e6072, SUC_lexicon_tags_42 },
    { 0xe94e6073, SUC_lexicon_tags_43 },
    { 0x5d0e6073, SUC_lexicon_tags_45 },
    { 0x56ee6071, SUC_lexicon_tags_17 },
    { 0xa30e6070, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34ae6078, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc62e607a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e8e607c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x636e607f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952e6083, SUC_lexicon_tags_68 },
    { 0x1fae6083, SUC_lexicon_tags_0 },
    { 0xefce6084, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc0e6087, SUC_lexicon_tags_87 },
    { 0xc84e6087, SUC_lexicon_tags_17 },
    { 0x7b4e6089, SUC_lexicon_tags_25 },
    { 0xcdae6087, SUC_lexicon_tags_17 },
    { 0x4a6e608b, SUC_lexicon_tags_12 },
    { 0x570e6088, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ee608f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120e6093, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x492e6095, SUC_lexicon_tags_0 },
    { 0x9a8e6096, SUC_lexicon_tags_33 },
    { 0xccae6096, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616e609b, SUC_lexicon_tags_43 },
    { 0xd46e609b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x696e609e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12e60a1, SUC_lexicon_tags_0 },
    { 0xec8e60a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ee60a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26ee60a8, SUC_lexicon_tags_15 },
    { 0x35ae60a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ee60b1, SUC_lexicon_tags_37 },
    { 0x5ce60b2, SUC_lexicon_tags_28 },
    { 0x530e60b2, SUC_lexicon_tags_9 },
    { 0x81ae60b2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622e60b8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeee60bc, SUC_lexicon_tags_7 },
    { 0xd44e60bd, SUC_lexicon_tags_17 },
    { 0xffae60be, SUC_lexicon_tags_7 },
    { 0xe28e60bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60ae60c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64e60c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x65ce60c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe14e60c9, SUC_lexicon_tags_12 },
    { 0xdd4e60c9, SUC_lexicon_tags_12 },
    { 0x5fee60cb, SUC_lexicon_tags_9 },
    { 0x986e60cb, SUC_lexicon_tags_37 },
    { 0x69ce60cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0e60d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x452e60d9, SUC_lexicon_tags_11 },
    { 0x262e60d9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78e60dd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ae60e1, SUC_lexicon_tags_0 },
    { 0x9c2e60e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ae60e7, SUC_lexicon_tags_25 },
    { 0x800e60e8, SUC_lexicon_tags_25 },
    { 0xbdee60e9, SUC_lexicon_tags_10 },
    { 0xbce60e9, SUC_lexicon_tags_43 },
    { 0x644e60eb, SUC_lexicon_tags_32 },
    { 0xee60eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fee60ef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeae60f2, SUC_lexicon_tags_25 },
    { 0x31ce60f3, SUC_lexicon_tags_49 },
    { 0x8c4e60f4, SUC_lexicon_tags_15 },
    { 0x51ce60f3, SUC_lexicon_tags_7 },
    { 0x84ae60f3, SUC_lexicon_tags_25 },
    { 0xd74e60f3, SUC_lexicon_tags_48 },
    { 0xe7ce60f8, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270e60fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc0ce60fe, SUC_lexicon_tags_17 },
    { 0x88e60ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156e6103, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x654e6106, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2ce6108, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ce610b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12e610e, SUC_lexicon_tags_17 },
    { 0xd04e610f, SUC_lexicon_tags_33 },
    { 0x326e610e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0e611a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd1ae611c, SUC_lexicon_tags_45 },
    { 0x26e611d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4e6122, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xab0e6124, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x2aae6126, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15ee6128, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ee612b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782e6133, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4e613b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34e613f, SUC_lexicon_tags_6 },
    { 0x1b4e6140, SUC_lexicon_tags_8 },
    { 0x10ee6141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9dee6143, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0e6149, SUC_lexicon_tags_14 },
    { 0x376e614a, SUC_lexicon_tags_8 },
    { 0x7f8e614a, SUC_lexicon_tags_11 },
    { 0xd2e614c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0e6150, SUC_lexicon_tags_79 },
    { 0xe02e6151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ae615a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52e6160, SUC_lexicon_tags_17 },
    { 0x44ce6161, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2e6165, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24e616a, SUC_lexicon_tags_15 },
    { 0x3fee616a, SUC_lexicon_tags_37 },
    { 0xadce616c, SUC_lexicon_tags_84 },
    { 0x960e616c, SUC_lexicon_tags_37 },
    { 0xe04e616d, SUC_lexicon_tags_108 },
    { 0xa48e616f, SUC_lexicon_tags_46 },
    { 0x110e616f, SUC_lexicon_tags_0 },
    { 0x7fee6171, SUC_lexicon_tags_19 },
    { 0xa7ae6171, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd0e6174, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x454e6176, SUC_lexicon_tags_7 },
    { 0x33ee6177, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x17ee6179, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xddce617b, SUC_lexicon_tags_37 },
    { 0x4f2e617c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x514e617e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4e6181, SUC_lexicon_tags_28 },
    { 0xc54e6182, SUC_lexicon_tags_26 },
    { 0x3dae6182, SUC_lexicon_tags_351 },
    { 0x696e6182, SUC_lexicon_tags_26 },
    { 0x6e0e6183, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce618b, SUC_lexicon_tags_11 },
    { 0x5cee618c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0e6190, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ce6194, SUC_lexicon_tags_7 },
    { 0x4e0e6195, SUC_lexicon_tags_12 },
    { 0x8cae6196, SUC_lexicon_tags_6 },
    { 0x162e6196, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee619a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ace61a9, SUC_lexicon_tags_224 },
    { 0xa90e61aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0e61ad, SUC_lexicon_tags_0 },
    { 0xeeae61ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0e61b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386e61b7, SUC_lexicon_tags_91 },
    { 0x2e4e61b7, SUC_lexicon_tags_7 },
    { 0x472e61b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa62e61bb, SUC_lexicon_tags_26 },
    { 0x880e61bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6e61c1, SUC_lexicon_tags_29 },
    { 0x698e61c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dce61c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ce61d3, SUC_lexicon_tags_31 },
    { 0xe98e61d4, SUC_lexicon_tags_6 },
    { 0x304e61d4, SUC_lexicon_tags_7 },
    { 0xd86e61d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x894e61d8, SUC_lexicon_tags_77 },
    { 0x8d6e61d9, SUC_lexicon_tags_37 },
    { 0xd9ce61da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc28e61dc, SUC_lexicon_tags_1 },
    { 0xd54e61dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbee61e3, SUC_lexicon_tags_17 },
    { 0x990e61e4, SUC_lexicon_tags_8 },
    { 0x23ce61e3, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x194e61e7, SUC_lexicon_tags_8 },
    { 0xd2ce61e7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xf6ee61ea, SUC_lexicon_tags_15 },
    { 0x7fae61ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ce61f3, SUC_lexicon_tags_61 },
    { 0xadae61f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe7ae61f6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ce61fa, SUC_lexicon_tags_11 },
    { 0x18e61fa, SUC_lexicon_tags_31 },
    { 0x1bae61fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bae6201, SUC_lexicon_tags_7 },
    { 0xc04e6201, SUC_lexicon_tags_37 },
    { 0xece6203, SUC_lexicon_tags_11 },
    { 0x404e6204, SUC_lexicon_tags_19 },
    { 0x4cee6205, SUC_lexicon_tags_37 },
    { 0x1bee6205, SUC_lexicon_tags_47 },
    { 0xf4e6203, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2e620b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae620f, SUC_lexicon_tags_29 },
    { 0xf18e6210, SUC_lexicon_tags_27 },
    { 0x8f8e6211, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92e621c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ce6220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8e6223, SUC_lexicon_tags_48 },
    { 0xdfae6224, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x33ee6226, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x556e6229, SUC_lexicon_tags_38 },
    { 0x8b0e6229, SUC_lexicon_tags_63 },
    { 0x1e4e6229, SUC_lexicon_tags_17 },
    { 0xa22e622a, SUC_lexicon_tags_49 },
    { 0xbd4e6229, SUC_lexicon_tags_23 },
    { 0x6ce622e, SUC_lexicon_tags_37 },
    { 0xace622f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24ce6231, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa12e6233, SUC_lexicon_tags_17 },
    { 0xd5ee6233, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x85ee6236, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aee623a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc74e623c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x666e623e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ae6243, SUC_lexicon_tags_2 },
    { 0xd34e6244, SUC_lexicon_tags_0 },
    { 0x7eae6243, SUC_lexicon_tags_17 },
    { 0x30e6245, SUC_lexicon_tags_37 },
    { 0xcf2e6247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacee6249, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6ce624b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ae624f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0e6253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ae6258, SUC_lexicon_tags_38 },
    { 0xb4e6258, SUC_lexicon_tags_47 },
    { 0x476e6259, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce6266, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecae626a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4e626d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe90e626f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce6273, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x484e6275, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550e627a, SUC_lexicon_tags_3 },
    { 0xdcae627b, SUC_lexicon_tags_1 },
    { 0x9f6e627c, SUC_lexicon_tags_17 },
    { 0x196e627d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14e6281, SUC_lexicon_tags_170 },
    { 0xdd0e6282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c8e6284, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce6288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2e628b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc50e628d, SUC_lexicon_tags_25 },
    { 0xecee628d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e6291, SUC_lexicon_tags_9 },
    { 0xc58e6292, SUC_lexicon_tags_11 },
    { 0x58e6293, SUC_lexicon_tags_11 },
    { 0x2cce6294, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa4ee6296, SUC_lexicon_tags_1 },
    { 0x1f8e6297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d8e6299, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x758e629c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70e62a0, SUC_lexicon_tags_7 },
    { 0xb6ce62a0, SUC_lexicon_tags_2 },
    { 0xa74e62a0, SUC_lexicon_tags_46 },
    { 0xf26e62a0, SUC_lexicon_tags_0 },
    { 0xd2ce62a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x386e62a7, SUC_lexicon_tags_87 },
    { 0x82e62a8, SUC_lexicon_tags_77 },
    { 0x6b8e62a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ee62ac, SUC_lexicon_tags_15 },
    { 0xb2ce62ad, SUC_lexicon_tags_11 },
    { 0xfd2e62ad, SUC_lexicon_tags_12 },
    { 0x116e62af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79ae62b1, SUC_lexicon_tags_91 },
    { 0xd00e62b2, SUC_lexicon_tags_45 },
    { 0x76ce62b2, SUC_lexicon_tags_39 },
    { 0xe64e62b2, SUC_lexicon_tags_15 },
    { 0xae62b5, SUC_lexicon_tags_15 },
    { 0xf5ee62b1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce62b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcece62bb, SUC_lexicon_tags_11 },
    { 0xb3ce62bc, SUC_lexicon_tags_9 },
    { 0xeaee62bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5ee62bf, SUC_lexicon_tags_6 },
    { 0x3bee62c0, SUC_lexicon_tags_35 },
    { 0x5a8e62c1, SUC_lexicon_tags_25 },
    { 0xe10e62c0, SUC_lexicon_tags_12 },
    { 0x8e6e62c3, SUC_lexicon_tags_980 },
    { 0x6aae62c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x240e62c6, SUC_lexicon_tags_37 },
    { 0xee0e62c7, SUC_lexicon_tags_1 },
    { 0x200e62c8, SUC_lexicon_tags_7 },
    { 0xe84e62c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04e62d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x772e62d3, SUC_lexicon_tags_30 },
    { 0x62ce62d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cce62dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cee62df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00e62e9, SUC_lexicon_tags_37 },
    { 0x882e62ea, SUC_lexicon_tags_17 },
    { 0xec2e62eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dce62f1, SUC_lexicon_tags_25 },
    { 0x942e62f2, SUC_lexicon_tags_8 },
    { 0x1f4e62f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee62f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0e62ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x402e6301, SUC_lexicon_tags_15 },
    { 0xace6302, SUC_lexicon_tags_12 },
    { 0xcf6e6303, SUC_lexicon_tags_47 },
    { 0xbee6304, SUC_lexicon_tags_28 },
    { 0x188e6305, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ace630a, SUC_lexicon_tags_65 },
    { 0x654e630b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2e6310, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x854e6313, SUC_lexicon_tags_27 },
    { 0xbf0e6313, SUC_lexicon_tags_25 },
    { 0x444e6315, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294e631d, SUC_lexicon_tags_19 },
    { 0x46ce631e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ae6322, SUC_lexicon_tags_17 },
    { 0xcb4e6323, SUC_lexicon_tags_7 },
    { 0x8d4e6323, SUC_lexicon_tags_12 },
    { 0xfd6e6323, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6e6328, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e632e, SUC_lexicon_tags_37 },
    { 0x24ce632e, SUC_lexicon_tags_22 },
    { 0xa66e632e, SUC_lexicon_tags_7 },
    { 0xae0e6331, SUC_lexicon_tags_165 },
    { 0x68ae6332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2e6335, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ee6338, SUC_lexicon_tags_47 },
    { 0x708e6339, SUC_lexicon_tags_7 },
    { 0x114e633a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68e633e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x372e6340, SUC_lexicon_tags_27 },
    { 0x9dee6341, SUC_lexicon_tags_70 },
    { 0x38e6340, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ae6346, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe82e6348, SUC_lexicon_tags_11 },
    { 0xf04e6348, SUC_lexicon_tags_25 },
    { 0x45ae634a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6e6354, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8e635b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ee635e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x700e6360, SUC_lexicon_tags_18 },
    { 0xc3ee6361, SUC_lexicon_tags_49 },
    { 0x73ee6362, SUC_lexicon_tags_33 },
    { 0x7d6e6360, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ee6366, SUC_lexicon_tags_27 },
    { 0x53ae6367, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4e636c, SUC_lexicon_tags_13 },
    { 0x4bee636d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e6371, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bee6379, SUC_lexicon_tags_47 },
    { 0x3bae637a, SUC_lexicon_tags_45 },
    { 0x872e637a, SUC_lexicon_tags_26 },
    { 0x172e637c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x472e637e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37ee6380, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x978e6383, SUC_lexicon_tags_47 },
    { 0x932e6384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76e6388, SUC_lexicon_tags_14 },
    { 0xb12e6389, SUC_lexicon_tags_761 },
    { 0xd3ce638a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30e6390, SUC_lexicon_tags_11 },
    { 0xdbae6390, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf48e6394, SUC_lexicon_tags_38 },
    { 0xffae6395, SUC_lexicon_tags_17 },
    { 0x400e6396, SUC_lexicon_tags_45 },
    { 0x8ce6397, SUC_lexicon_tags_38 },
    { 0x49ae6395, SUC_lexicon_tags_17 },
    { 0xe64e6394, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x510e639b, SUC_lexicon_tags_16 },
    { 0xdd0e639c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ce63a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e0e63a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8e63a5, SUC_lexicon_tags_37 },
    { 0xe52e63a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd9ce63a8, SUC_lexicon_tags_0 },
    { 0x7f0e63a9, SUC_lexicon_tags_26 },
    { 0x4e6e63aa, SUC_lexicon_tags_2 },
    { 0xadce63a9, SUC_lexicon_tags_1 },
    { 0x8a6e63ab, SUC_lexicon_tags_55 },
    { 0x9ece63aa, SUC_lexicon_tags_12 },
    { 0x12ae63a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeaae63b0, SUC_lexicon_tags_15 },
    { 0xd14e63b1, SUC_lexicon_tags_15 },
    { 0x4b8e63b0, SUC_lexicon_tags_3 },
    { 0x75ae63b3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54e63b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ae63be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x450e63c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x700e63c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d0e63c5, SUC_lexicon_tags_61 },
    { 0x266e63c6, SUC_lexicon_tags_15 },
    { 0xb46e63c7, SUC_lexicon_tags_15 },
    { 0xf5ce63c8, SUC_lexicon_tags_55 },
    { 0x26ee63c9, SUC_lexicon_tags_37 },
    { 0x4bce63ca, SUC_lexicon_tags_12 },
    { 0xdfee63cb, SUC_lexicon_tags_10 },
    { 0xf30e63cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8e63d2, SUC_lexicon_tags_17 },
    { 0x5a8e63d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdeae63d5, SUC_lexicon_tags_11 },
    { 0x504e63d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8e63dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0e63e2, SUC_lexicon_tags_30 },
    { 0x4b0e63e3, SUC_lexicon_tags_37 },
    { 0xcce63e2, SUC_lexicon_tags_18 },
    { 0x8d2e63e5, SUC_lexicon_tags_55 },
    { 0x2ace63e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4e63e9, SUC_lexicon_tags_114 },
    { 0x14ce63e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa8e63ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94ee63ee, SUC_lexicon_tags_1 },
    { 0x558e63ef, SUC_lexicon_tags_27 },
    { 0x74ae63ee, SUC_lexicon_tags_15 },
    { 0x11ce63f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0e63fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fce6404, SUC_lexicon_tags_31 },
    { 0x10e6405, SUC_lexicon_tags_43 },
    { 0x42e6404, SUC_lexicon_tags_25 },
    { 0xfd6e6405, SUC_lexicon_tags_27 },
    { 0x9a6e6406, SUC_lexicon_tags_31 },
    { 0xdeae6409, SUC_lexicon_tags_12 },
    { 0x692e640a, SUC_lexicon_tags_33 },
    { 0xd6ee640a, SUC_lexicon_tags_17 },
    { 0xa42e640c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x988e640e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0e6412, SUC_lexicon_tags_8 },
    { 0xffce6413, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6e6416, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96e641a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd46e641c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ee6422, SUC_lexicon_tags_23 },
    { 0x306e6423, SUC_lexicon_tags_12 },
    { 0xa48e6424, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eae6427, SUC_lexicon_tags_2 },
    { 0x794e6428, SUC_lexicon_tags_1 },
    { 0x194e6429, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdcae642b, SUC_lexicon_tags_11 },
    { 0x542e642c, SUC_lexicon_tags_254 },
    { 0x460e642d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0ae642f, SUC_lexicon_tags_11 },
    { 0x820e6430, SUC_lexicon_tags_42 },
    { 0x9e2e6431, SUC_lexicon_tags_25 },
    { 0xc22e6431, SUC_lexicon_tags_15 },
    { 0xe90e6430, SUC_lexicon_tags_17 },
    { 0x402e6431, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadce6438, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3ee643a, SUC_lexicon_tags_11 },
    { 0x638e643a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0e643f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x106e6442, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6e6446, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20e644d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4e6456, SUC_lexicon_tags_49 },
    { 0x726e6457, SUC_lexicon_tags_14 },
    { 0xa4e6458, SUC_lexicon_tags_33 },
    { 0x21ee6456, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x872e645b, SUC_lexicon_tags_15 },
    { 0x5d6e645c, SUC_lexicon_tags_7 },
    { 0x1fce645d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666e6464, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264e646d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536e6471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970e647a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa58e647c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ee647f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x194e6481, SUC_lexicon_tags_7 },
    { 0x5b4e6481, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0e648a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x126e648c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04e649a, SUC_lexicon_tags_31 },
    { 0x4c4e649a, SUC_lexicon_tags_4 },
    { 0x73ce649a, SUC_lexicon_tags_0 },
    { 0x184e649b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffae649f, SUC_lexicon_tags_44 },
    { 0x1eee64a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0e64a3, SUC_lexicon_tags_34 },
    { 0xae64a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74e64a9, SUC_lexicon_tags_2 },
    { 0xb5ae64aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63ee64ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x738e64ae, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ae64b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ce64b5, SUC_lexicon_tags_42 },
    { 0x8e0e64b5, SUC_lexicon_tags_15 },
    { 0x3c8e64b7, SUC_lexicon_tags_15 },
    { 0x50e64b6, SUC_lexicon_tags_2 },
    { 0x390e64b9, SUC_lexicon_tags_12 },
    { 0x81ae64b9, SUC_lexicon_tags_25 },
    { 0xb54e64ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6dae64bd, SUC_lexicon_tags_37 },
    { 0x7c0e64be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ce64c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0e64c8, SUC_lexicon_tags_2 },
    { 0x564e64c9, SUC_lexicon_tags_10 },
    { 0x532e64ca, SUC_lexicon_tags_30 },
    { 0x97ee64c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb4e64cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdace64d1, SUC_lexicon_tags_56 },
    { 0xf24e64d1, SUC_lexicon_tags_11 },
    { 0xee4e64d1, SUC_lexicon_tags_37 },
    { 0x3ae64d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x596e64d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7f8e64d9, SUC_lexicon_tags_11 },
    { 0x780e64da, SUC_lexicon_tags_21 },
    { 0x3bce64da, SUC_lexicon_tags_85 },
    { 0x6b0e64dc, SUC_lexicon_tags_60 },
    { 0x6fee64dc, SUC_lexicon_tags_25 },
    { 0x7f6e64de, SUC_lexicon_tags_7 },
    { 0xcfce64df, SUC_lexicon_tags_11 },
    { 0xc7ce64e0, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6e64e8, SUC_lexicon_tags_2 },
    { 0xf22e64e8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96e64ec, SUC_lexicon_tags_37 },
    { 0x784e64ec, SUC_lexicon_tags_59 },
    { 0xb5ae64ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe48e64f0, SUC_lexicon_tags_17 },
    { 0xca0e64f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b4e64f3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xefee64f6, SUC_lexicon_tags_28 },
    { 0xf1ee64f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xface64fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe64fc, SUC_lexicon_tags_35 },
    { 0x1a6e64fd, SUC_lexicon_tags_21 },
    { 0x1f0e64fe, SUC_lexicon_tags_11 },
    { 0x5b0e64fd, SUC_lexicon_tags_7 },
    { 0x15ce64ff, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce6503, SUC_lexicon_tags_6 },
    { 0xc12e6504, SUC_lexicon_tags_14 },
    { 0x32ee6505, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x2b4e6507, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2e6509, SUC_lexicon_tags_27 },
    { 0xd84e650a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc1ee650c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02e6512, SUC_lexicon_tags_25 },
    { 0x976e6513, SUC_lexicon_tags_12 },
    { 0x2bce6514, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56e6517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27ce6519, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdcee651b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1d8e651d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2e6521, SUC_lexicon_tags_15 },
    { 0x29ee6521, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74ae6524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x548e6527, SUC_lexicon_tags_26 },
    { 0xd70e6528, SUC_lexicon_tags_39 },
    { 0x674e6528, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x60e652b, SUC_lexicon_tags_37 },
    { 0x5f4e652c, SUC_lexicon_tags_65 },
    { 0x440e652d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ee6530, SUC_lexicon_tags_4 },
    { 0x942e6531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb44e6534, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbce6537, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8e6544, SUC_lexicon_tags_10 },
    { 0x144e6545, SUC_lexicon_tags_11 },
    { 0x128e6545, SUC_lexicon_tags_14 },
    { 0x77ce6547, SUC_lexicon_tags_60 },
    { 0xa4ce6546, SUC_lexicon_tags_12 },
    { 0x5fee6544, SUC_lexicon_tags_0 },
    { 0x8c2e654a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40e6550, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454e6554, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x762e6557, SUC_lexicon_tags_27 },
    { 0x924e6558, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe70e655a, SUC_lexicon_tags_11 },
    { 0x972e655b, SUC_lexicon_tags_12 },
    { 0xcaee655b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x24ee655e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x958e6560, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf8e6562, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0e656a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2e656d, SUC_lexicon_tags_2 },
    { 0xf52e656d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70e6572, SUC_lexicon_tags_8 },
    { 0x740e6572, SUC_lexicon_tags_17 },
    { 0xb72e6574, SUC_lexicon_tags_47 },
    { 0x3e4e6572, SUC_lexicon_tags_49 },
    { 0x122e6576, SUC_lexicon_tags_25 },
    { 0xefce6576, SUC_lexicon_tags_57 },
    { 0x762e6578, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x182e657b, SUC_lexicon_tags_17 },
    { 0x550e657b, SUC_lexicon_tags_27 },
    { 0x90e657d, SUC_lexicon_tags_1 },
    { 0xd50e657d, SUC_lexicon_tags_1 },
    { 0x17ee657f, SUC_lexicon_tags_32 },
    { 0xc08e6580, SUC_lexicon_tags_27 },
    { 0x49ee6581, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0e6584, SUC_lexicon_tags_11 },
    { 0x7ce6585, SUC_lexicon_tags_17 },
    { 0xf0ae6586, SUC_lexicon_tags_23 },
    { 0x2b2e6587, SUC_lexicon_tags_25 },
    { 0xcee6588, SUC_lexicon_tags_12 },
    { 0xd34e6589, SUC_lexicon_tags_37 },
    { 0xcd2e6589, SUC_lexicon_tags_63 },
    { 0xa16e658b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b4e658d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xacee658f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x642e6591, SUC_lexicon_tags_37 },
    { 0x20ce6591, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99ee6594, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x60e6597, SUC_lexicon_tags_12 },
    { 0x464e6598, SUC_lexicon_tags_12 },
    { 0x5c6e6599, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0e65a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x652e65a3, SUC_lexicon_tags_31 },
    { 0x390e65a4, SUC_lexicon_tags_61 },
    { 0x3b8e65a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ee65ab, SUC_lexicon_tags_194 },
    { 0xa7ae65ab, SUC_lexicon_tags_12 },
    { 0x59ce65ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36e65b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86e65b6, SUC_lexicon_tags_33 },
    { 0x536e65b7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebae65bc, SUC_lexicon_tags_2 },
    { 0xbd8e65bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ee65c0, SUC_lexicon_tags_28 },
    { 0xe36e65c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x748e65c3, SUC_lexicon_tags_437 },
    { 0xa0e65c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6e65c6, SUC_lexicon_tags_68 },
    { 0xf80e65c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ee65cd, SUC_lexicon_tags_49 },
    { 0x696e65ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ce65d1, SUC_lexicon_tags_14 },
    { 0xec8e65d1, SUC_lexicon_tags_60 },
    { 0xfece65d3, SUC_lexicon_tags_1 },
    { 0x132e65d4, SUC_lexicon_tags_45 },
    { 0xa94e65d5, SUC_lexicon_tags_45 },
    { 0x9ee65d6, SUC_lexicon_tags_37 },
    { 0x686e65d5, SUC_lexicon_tags_47 },
    { 0xc2ae65d8, SUC_lexicon_tags_35 },
    { 0x35ee65d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7ee65db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde6e65dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x44ae65df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee2e65e1, SUC_lexicon_tags_48 },
    { 0x73ae65e1, SUC_lexicon_tags_7 },
    { 0x35ce65e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ace65ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x67ee65ec, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xde2e65ee, SUC_lexicon_tags_60 },
    { 0x85ce65ef, SUC_lexicon_tags_2 },
    { 0x182e65ee, SUC_lexicon_tags_60 },
    { 0x7cee65f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdce65f3, SUC_lexicon_tags_39 },
    { 0x5a2e65f4, SUC_lexicon_tags_5 },
    { 0xd90e65f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29ee65f7, SUC_lexicon_tags_25 },
    { 0xc96e65f7, SUC_lexicon_tags_80 },
    { 0xe4e65f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x67ae65fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35ae65fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4e6602, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bae660d, SUC_lexicon_tags_31 },
    { 0x3c0e660e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8e6616, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bee6618, SUC_lexicon_tags_8 },
    { 0x4a2e6619, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x598e661b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750e6620, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf08e6622, SUC_lexicon_tags_14 },
    { 0xa16e6623, SUC_lexicon_tags_25 },
    { 0x3a2e6624, SUC_lexicon_tags_47 },
    { 0x414e6625, SUC_lexicon_tags_27 },
    { 0x63ae6625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704e662a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272e662e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x118e6631, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386e6635, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebee6639, SUC_lexicon_tags_64 },
    { 0xa0ce663a, SUC_lexicon_tags_7 },
    { 0x4a4e663b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4e663e, SUC_lexicon_tags_17 },
    { 0xa1ae663f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee6643, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bae6646, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8e664d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ee6654, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ce665b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x490e665d, SUC_lexicon_tags_57 },
    { 0xf12e665e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xdee6660, SUC_lexicon_tags_1 },
    { 0x940e6661, SUC_lexicon_tags_30 },
    { 0x882e6661, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0e6665, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee6669, SUC_lexicon_tags_3 },
    { 0xba4e666a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dce6670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x624e6673, SUC_lexicon_tags_17 },
    { 0x118e6674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4e6679, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dce667d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa84e667f, SUC_lexicon_tags_24 },
    { 0xddce667f, SUC_lexicon_tags_34 },
    { 0x1c2e6681, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56e6684, SUC_lexicon_tags_37 },
    { 0x924e6685, SUC_lexicon_tags_12 },
    { 0x72ae6685, SUC_lexicon_tags_25 },
    { 0x7bce6686, SUC_lexicon_tags_0 },
    { 0xe24e6688, SUC_lexicon_tags_29 },
    { 0x214e6685, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x606e668b, SUC_lexicon_tags_15 },
    { 0x750e668b, SUC_lexicon_tags_39 },
    { 0xfb2e668c, SUC_lexicon_tags_129 },
    { 0x40ae668d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1f4e6690, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeee6693, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x940e6695, SUC_lexicon_tags_33 },
    { 0xfb0e6696, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4f6e6698, SUC_lexicon_tags_43 },
    { 0xd14e6699, SUC_lexicon_tags_14 },
    { 0x7d8e6698, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724e669e, SUC_lexicon_tags_10 },
    { 0x25ce669f, SUC_lexicon_tags_25 },
    { 0x3d8e669e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfae66a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ee66a5, SUC_lexicon_tags_16 },
    { 0x428e66a6, SUC_lexicon_tags_15 },
    { 0x8cae66a7, SUC_lexicon_tags_234 },
    { 0x100e66a6, SUC_lexicon_tags_43 },
    { 0xd9ce66a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x762e66ac, SUC_lexicon_tags_7 },
    { 0x50ae66ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x30ae66af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ee66b2, SUC_lexicon_tags_29 },
    { 0x8f2e66b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddce66b8, SUC_lexicon_tags_11 },
    { 0xf1ce66b9, SUC_lexicon_tags_11 },
    { 0xacae66b9, SUC_lexicon_tags_38 },
    { 0x9a6e66bb, SUC_lexicon_tags_2 },
    { 0x928e66bc, SUC_lexicon_tags_25 },
    { 0x1cce66bb, SUC_lexicon_tags_15 },
    { 0xa76e66bb, SUC_lexicon_tags_1 },
    { 0x896e66bb, SUC_lexicon_tags_27 },
    { 0x8fae66b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7ce66c2, SUC_lexicon_tags_35 },
    { 0xd86e66c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ce66cb, SUC_lexicon_tags_15 },
    { 0xe92e66cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ae66d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ce66d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8e66d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4e66dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce66ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aee66f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48e66f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8e6706, SUC_lexicon_tags_88 },
    { 0x410e6707, SUC_lexicon_tags_12 },
    { 0x232e6707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ce670e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ae6711, SUC_lexicon_tags_37 },
    { 0x83ae6712, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6dee6714, SUC_lexicon_tags_104 },
    { 0x5c4e6714, SUC_lexicon_tags_23 },
    { 0x592e6715, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ce671a, SUC_lexicon_tags_11 },
    { 0x464e671b, SUC_lexicon_tags_43 },
    { 0x3fce671c, SUC_lexicon_tags_525 },
    { 0x78ce671c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cae6720, SUC_lexicon_tags_10 },
    { 0xbe2e6721, SUC_lexicon_tags_31 },
    { 0x64ae6722, SUC_lexicon_tags_63 },
    { 0xb62e6723, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76e6737, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4e673b, SUC_lexicon_tags_33 },
    { 0x170e673c, SUC_lexicon_tags_11 },
    { 0xaa2e673b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f4e673f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ae6743, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84e674e, SUC_lexicon_tags_60 },
    { 0xcb0e674e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496e6753, SUC_lexicon_tags_15 },
    { 0x5dae6753, SUC_lexicon_tags_46 },
    { 0x6cce6753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74ae6758, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6e675b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8e675f, SUC_lexicon_tags_15 },
    { 0xe92e6760, SUC_lexicon_tags_21 },
    { 0x9d0e6761, SUC_lexicon_tags_49 },
    { 0xeeee6762, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf2e6764, SUC_lexicon_tags_6 },
    { 0x512e6764, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ae676a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8e676e, SUC_lexicon_tags_25 },
    { 0xbb4e676e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfce6775, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6778, SUC_lexicon_tags_1 },
    { 0x3e8e6778, SUC_lexicon_tags_1 },
    { 0x92ee6778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00e677d, SUC_lexicon_tags_60 },
    { 0xa88e677e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e6782, SUC_lexicon_tags_56 },
    { 0xf2ee6783, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4e6786, SUC_lexicon_tags_29 },
    { 0xc52e6786, SUC_lexicon_tags_21 },
    { 0xbdce6788, SUC_lexicon_tags_108 },
    { 0xbfee6789, SUC_lexicon_tags_63 },
    { 0x6d6e678a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ace678d, SUC_lexicon_tags_25 },
    { 0x112e678d, SUC_lexicon_tags_7 },
    { 0xd78e678d, SUC_lexicon_tags_18 },
    { 0x4d2e678e, SUC_lexicon_tags_32 },
    { 0x4fee6791, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ce6794, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5dee6796, SUC_lexicon_tags_1 },
    { 0xe52e6797, SUC_lexicon_tags_19 },
    { 0x7f8e6797, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ae67a0, SUC_lexicon_tags_17 },
    { 0xbb8e67a0, SUC_lexicon_tags_19 },
    { 0xe5ee67a1, SUC_lexicon_tags_68 },
    { 0xd6e67a3, SUC_lexicon_tags_11 },
    { 0xe24e67a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2e67a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5ce67a9, SUC_lexicon_tags_11 },
    { 0xdd0e67a9, SUC_lexicon_tags_18 },
    { 0xf3ae67a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74e67af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x462e67b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf02e67b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aae67b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x244e67bb, SUC_lexicon_tags_55 },
    { 0x9f0e67bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6a8e67be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa5ae67c0, SUC_lexicon_tags_11 },
    { 0xbb2e67c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bce67c4, SUC_lexicon_tags_15 },
    { 0xedce67c5, SUC_lexicon_tags_47 },
    { 0xda0e67c5, SUC_lexicon_tags_63 },
    { 0xe7ee67c6, SUC_lexicon_tags_25 },
    { 0x972e67c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96e67ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0e67d7, SUC_lexicon_tags_15 },
    { 0xaa0e67d8, SUC_lexicon_tags_47 },
    { 0xd0ae67d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27ce67db, SUC_lexicon_tags_30 },
    { 0x3aee67dc, SUC_lexicon_tags_14 },
    { 0x1c4e67db, SUC_lexicon_tags_87 },
    { 0x1c2e67dc, SUC_lexicon_tags_11 },
    { 0xfe8e67df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2e67e8, SUC_lexicon_tags_56 },
    { 0x760e67e8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ae67f5, SUC_lexicon_tags_47 },
    { 0x8dce67f5, SUC_lexicon_tags_37 },
    { 0x92e67f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8e67fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x13ee67fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94ae67fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0e6802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8e6807, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ae680b, SUC_lexicon_tags_68 },
    { 0x28e680c, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ee6811, SUC_lexicon_tags_371 },
    { 0, NULL },
    { 0, NULL },
    { 0x14e6814, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc06e6816, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02e681a, SUC_lexicon_tags_33 },
    { 0x654e681a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b0e681d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ae6820, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x348e6822, SUC_lexicon_tags_46 },
    { 0x89ce6822, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bae6828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960e682f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ce6832, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaee683b, SUC_lexicon_tags_12 },
    { 0x908e683c, SUC_lexicon_tags_7 },
    { 0x4eee683c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x76ee683f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ee6842, SUC_lexicon_tags_10 },
    { 0xde2e6842, SUC_lexicon_tags_25 },
    { 0x1d6e6844, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914e6849, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4e6852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52e6857, SUC_lexicon_tags_31 },
    { 0x57ae6858, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ee685b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8b0e685d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53ee685f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ae686f, SUC_lexicon_tags_27 },
    { 0x46e6870, SUC_lexicon_tags_0 },
    { 0xa66e6870, SUC_lexicon_tags_25 },
    { 0x4e4e6870, SUC_lexicon_tags_21 },
    { 0xfeee6870, SUC_lexicon_tags_77 },
    { 0x92ee6874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ee6877, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdce687b, SUC_lexicon_tags_13 },
    { 0xbb2e687c, SUC_lexicon_tags_3 },
    { 0x1ae687b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe72e687f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ae688f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ce6892, SUC_lexicon_tags_17 },
    { 0x93ae6892, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc4ce6895, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370e6899, SUC_lexicon_tags_15 },
    { 0xdc6e6899, SUC_lexicon_tags_12 },
    { 0xdeee689a, SUC_lexicon_tags_31 },
    { 0x2f0e689c, SUC_lexicon_tags_38 },
    { 0x97ee689d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48e68a5, SUC_lexicon_tags_11 },
    { 0xcee68a6, SUC_lexicon_tags_34 },
    { 0x6ace68a7, SUC_lexicon_tags_27 },
    { 0x892e68a8, SUC_lexicon_tags_69 },
    { 0x46e68a7, SUC_lexicon_tags_63 },
    { 0x774e68a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8aee68ac, SUC_lexicon_tags_37 },
    { 0xfb2e68ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad8e68af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6e68b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e68bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffee68c3, SUC_lexicon_tags_5 },
    { 0xf4ce68c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4e68c8, SUC_lexicon_tags_49 },
    { 0x8aee68c9, SUC_lexicon_tags_37 },
    { 0xa1ee68ca, SUC_lexicon_tags_8 },
    { 0xecae68cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ce68ce, SUC_lexicon_tags_12 },
    { 0x3c4e68ce, SUC_lexicon_tags_0 },
    { 0x26ce68ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2e68d7, SUC_lexicon_tags_1 },
    { 0xcc8e68d8, SUC_lexicon_tags_2 },
    { 0xe30e68d7, SUC_lexicon_tags_15 },
    { 0xd18e68d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x982e68dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470e68e0, SUC_lexicon_tags_1 },
    { 0x66ee68e0, SUC_lexicon_tags_14 },
    { 0xd34e68e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72e68e5, SUC_lexicon_tags_30 },
    { 0xc98e68e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964e68ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x164e68ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2e68f1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100e68f6, SUC_lexicon_tags_2 },
    { 0x25ce68f6, SUC_lexicon_tags_459 },
    { 0xe28e68f8, SUC_lexicon_tags_37 },
    { 0x572e68f9, SUC_lexicon_tags_11 },
    { 0x4c6e68fa, SUC_lexicon_tags_17 },
    { 0xff6e68fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ce68fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354e6903, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x976e6905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6e6909, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x11ae690b, SUC_lexicon_tags_94 },
    { 0x58ae690c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x2cee690e, SUC_lexicon_tags_0 },
    { 0x2e6e690f, SUC_lexicon_tags_15 },
    { 0x49ee6910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb4e6912, SUC_lexicon_tags_17 },
    { 0x3ece6912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd00e6915, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268e691b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884e6923, SUC_lexicon_tags_7 },
    { 0xd3ae6924, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6e6929, SUC_lexicon_tags_37 },
    { 0x41ae6929, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4e6931, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92e6935, SUC_lexicon_tags_60 },
    { 0xfbae6936, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e6939, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xff0e693b, SUC_lexicon_tags_33 },
    { 0x56ee693b, SUC_lexicon_tags_37 },
    { 0x56ce693b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6e6945, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456e6949, SUC_lexicon_tags_17 },
    { 0xb38e694a, SUC_lexicon_tags_15 },
    { 0xe8e694b, SUC_lexicon_tags_29 },
    { 0x272e694b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0e6952, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x446e6955, SUC_lexicon_tags_2 },
    { 0xf4ae6956, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e4e6958, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0e695b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6e695e, SUC_lexicon_tags_12 },
    { 0xf3ce695e, SUC_lexicon_tags_28 },
    { 0x4ae695f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x79ee6962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee6968, SUC_lexicon_tags_17 },
    { 0x7cee6969, SUC_lexicon_tags_37 },
    { 0x182e6969, SUC_lexicon_tags_37 },
    { 0x522e696a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98e696d, SUC_lexicon_tags_17 },
    { 0x5bce696e, SUC_lexicon_tags_38 },
    { 0x266e696f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x566e6972, SUC_lexicon_tags_30 },
    { 0xfdae6972, SUC_lexicon_tags_25 },
    { 0x1f0e6973, SUC_lexicon_tags_25 },
    { 0xe3ee6975, SUC_lexicon_tags_3 },
    { 0xf80e6976, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94ce6978, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ee697b, SUC_lexicon_tags_35 },
    { 0xf52e697b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4dae697e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0e6983, SUC_lexicon_tags_12 },
    { 0xdfae6983, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c2e6986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46e6989, SUC_lexicon_tags_19 },
    { 0x6a4e698a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2e698f, SUC_lexicon_tags_11 },
    { 0xeaae698f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8e6994, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6e699a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6e69a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23ce69a6, SUC_lexicon_tags_4 },
    { 0x320e69a7, SUC_lexicon_tags_19 },
    { 0x312e69a8, SUC_lexicon_tags_31 },
    { 0x206e69a6, SUC_lexicon_tags_12 },
    { 0xfd2e69aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4e69b1, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0xb0ae69b3, SUC_lexicon_tags_9 },
    { 0xfeae69b4, SUC_lexicon_tags_17 },
    { 0xff2e69b3, SUC_lexicon_tags_17 },
    { 0xcbae69b5, SUC_lexicon_tags_92 },
    { 0xa7ce69b7, SUC_lexicon_tags_12 },
    { 0x20ee69b7, SUC_lexicon_tags_17 },
    { 0xa32e69b6, SUC_lexicon_tags_1 },
    { 0x378e69b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356e69be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x282e69c0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8e69c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68e69c8, SUC_lexicon_tags_1 },
    { 0xd2ae69c9, SUC_lexicon_tags_15 },
    { 0x60ae69ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6a6e69cc, SUC_lexicon_tags_11 },
    { 0x52ce69cc, SUC_lexicon_tags_12 },
    { 0x924e69cc, SUC_lexicon_tags_40 },
    { 0xfd2e69cf, SUC_lexicon_tags_12 },
    { 0x69ee69cf, SUC_lexicon_tags_11 },
    { 0x790e69ce, SUC_lexicon_tags_14 },
    { 0x3f4e69cd, SUC_lexicon_tags_19 },
    { 0x542e69d3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ce69d6, SUC_lexicon_tags_14 },
    { 0xae0e69d7, SUC_lexicon_tags_25 },
    { 0x704e69d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece69df, SUC_lexicon_tags_37 },
    { 0xde2e69df, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28e69e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6e69e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0e69ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ae69f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf62e69f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80e69f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836e69fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0e6a00, SUC_lexicon_tags_37 },
    { 0x802e6a01, SUC_lexicon_tags_1 },
    { 0x460e6a01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14e6a04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660e6a0c, SUC_lexicon_tags_10 },
    { 0x20ce6a0c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x822e6a0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34e6a13, SUC_lexicon_tags_27 },
    { 0x462e6a14, SUC_lexicon_tags_17 },
    { 0x392e6a15, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2e6a18, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266e6a1c, SUC_lexicon_tags_17 },
    { 0x550e6a1d, SUC_lexicon_tags_38 },
    { 0x9ee6a1d, SUC_lexicon_tags_29 },
    { 0x2b0e6a1f, SUC_lexicon_tags_31 },
    { 0xa3ee6a20, SUC_lexicon_tags_11 },
    { 0xd3ee6a21, SUC_lexicon_tags_27 },
    { 0xdc0e6a22, SUC_lexicon_tags_47 },
    { 0x7c2e6a23, SUC_lexicon_tags_23 },
    { 0x390e6a22, SUC_lexicon_tags_60 },
    { 0x656e6a22, SUC_lexicon_tags_25 },
    { 0x4fae6a1f, SUC_lexicon_tags_7 },
    { 0xa84e6a27, SUC_lexicon_tags_12 },
    { 0x136e6a28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x690e6a2a, SUC_lexicon_tags_33 },
    { 0x5c4e6a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4e6a2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaae6a32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546e6a37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe78e6a39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6e6a3c, SUC_lexicon_tags_84 },
    { 0xd5ee6a3d, SUC_lexicon_tags_84 },
    { 0xc6ae6a3e, SUC_lexicon_tags_48 },
    { 0x51ae6a3d, SUC_lexicon_tags_7 },
    { 0xd26e6a3f, SUC_lexicon_tags_37 },
    { 0x70e6a41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ce6a47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa6e6a49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fce6a4b, SUC_lexicon_tags_11 },
    { 0x16ae6a4b, SUC_lexicon_tags_7 },
    { 0xeeae6a4b, SUC_lexicon_tags_10 },
    { 0x8e6e6a4e, SUC_lexicon_tags_26 },
    { 0xddae6a4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58ee6a51, SUC_lexicon_tags_17 },
    { 0xfc6e6a52, SUC_lexicon_tags_43 },
    { 0x470e6a51, SUC_lexicon_tags_27 },
    { 0xd10e6a54, SUC_lexicon_tags_2 },
    { 0x520e6a55, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54e6a58, SUC_lexicon_tags_47 },
    { 0x566e6a59, SUC_lexicon_tags_0 },
    { 0xae6a58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796e6a5e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0e6a61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eae6a65, SUC_lexicon_tags_29 },
    { 0x5e0e6a66, SUC_lexicon_tags_33 },
    { 0x42ee6a67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156e6a6b, SUC_lexicon_tags_12 },
    { 0x84ee6a6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdee6a6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0e6a72, SUC_lexicon_tags_7 },
    { 0xad4e6a73, SUC_lexicon_tags_84 },
    { 0xf14e6a74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4e6a77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x366e6a79, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0e6a7e, SUC_lexicon_tags_0 },
    { 0xe9ae6a7e, SUC_lexicon_tags_1 },
    { 0xcf6e6a7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ce6a89, SUC_lexicon_tags_3 },
    { 0x4a8e6a8a, SUC_lexicon_tags_2 },
    { 0xbc8e6a8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52e6a8f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa36e6a91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6e6a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bae6a9c, SUC_lexicon_tags_15 },
    { 0x9a2e6a9d, SUC_lexicon_tags_12 },
    { 0xe0ee6a9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee6aa6, SUC_lexicon_tags_25 },
    { 0xbee6aa7, SUC_lexicon_tags_15 },
    { 0xdb6e6aa8, SUC_lexicon_tags_21 },
    { 0x8bae6aa9, SUC_lexicon_tags_28 },
    { 0xc8ae6aaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4ce6aac, SUC_lexicon_tags_45 },
    { 0x7b4e6aad, SUC_lexicon_tags_7 },
    { 0xad2e6aad, SUC_lexicon_tags_6 },
    { 0x372e6aac, SUC_lexicon_tags_11 },
    { 0x15ee6aad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aee6ab4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27ee6ab6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224e6abc, SUC_lexicon_tags_33 },
    { 0x258e6abd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464e6ac2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7fae6ac4, SUC_lexicon_tags_19 },
    { 0xcf4e6ac4, SUC_lexicon_tags_11 },
    { 0xdf0e6ac5, SUC_lexicon_tags_45 },
    { 0xed8e6ac7, SUC_lexicon_tags_32 },
    { 0x8e8e6ac8, SUC_lexicon_tags_37 },
    { 0x1e2e6ac9, SUC_lexicon_tags_1 },
    { 0x710e6ac9, SUC_lexicon_tags_9 },
    { 0x254e6ac7, SUC_lexicon_tags_56 },
    { 0x334e6acc, SUC_lexicon_tags_11 },
    { 0x29ee6ac5, SUC_lexicon_tags_38 },
    { 0x7aee6ac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x728e6ad1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac0e6ad3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbace6ad8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccee6add, SUC_lexicon_tags_10 },
    { 0xb2ce6add, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eae6ae2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2e6ae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c2e6ae8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68e6aeb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6e6af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee6af4, SUC_lexicon_tags_1 },
    { 0xb36e6af4, SUC_lexicon_tags_37 },
    { 0x84ce6af6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0e6af9, SUC_lexicon_tags_9 },
    { 0x8bee6afa, SUC_lexicon_tags_11 },
    { 0x514e6afb, SUC_lexicon_tags_1 },
    { 0xb10e6afc, SUC_lexicon_tags_37 },
    { 0x832e6afd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ce6b00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3aae6b02, SUC_lexicon_tags_25 },
    { 0x6cee6b03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bae6b05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858e6b09, SUC_lexicon_tags_15 },
    { 0x9a6e6b0a, SUC_lexicon_tags_1 },
    { 0x694e6b0a, SUC_lexicon_tags_11 },
    { 0xdb2e6b0a, SUC_lexicon_tags_30 },
    { 0xe46e6b0b, SUC_lexicon_tags_1 },
    { 0x472e6b0e, SUC_lexicon_tags_37 },
    { 0x8b2e6b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ce6b14, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x51ce6b16, SUC_lexicon_tags_1 },
    { 0xc5ae6b17, SUC_lexicon_tags_7 },
    { 0xacae6b18, SUC_lexicon_tags_34 },
    { 0x9a6e6b19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69ee6b1b, SUC_lexicon_tags_14 },
    { 0x7c0e6b1c, SUC_lexicon_tags_1015 },
    { 0x3d2e6b1b, SUC_lexicon_tags_23 },
    { 0x222e6b1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9cae6b20, SUC_lexicon_tags_37 },
    { 0xa3ae6b20, SUC_lexicon_tags_26 },
    { 0x918e6b22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24e6b25, SUC_lexicon_tags_45 },
    { 0x37ee6b26, SUC_lexicon_tags_1 },
    { 0x2e8e6b26, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ce6b2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994e6b2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32e6b34, SUC_lexicon_tags_1 },
    { 0xd72e6b35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ae6b3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0e6b4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde4e6b4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bae6b4f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f0e6b51, SUC_lexicon_tags_82 },
    { 0xe2ee6b51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd80e6b54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6e6b57, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcce6b5c, SUC_lexicon_tags_23 },
    { 0x254e6b5c, SUC_lexicon_tags_25 },
    { 0x2c6e6b5c, SUC_lexicon_tags_25 },
    { 0x852e6b5e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e6b63, SUC_lexicon_tags_37 },
    { 0xc8ae6b64, SUC_lexicon_tags_25 },
    { 0xb64e6b65, SUC_lexicon_tags_31 },
    { 0xc26e6b65, SUC_lexicon_tags_1 },
    { 0xa9ae6b67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ae6b6b, SUC_lexicon_tags_15 },
    { 0x8a8e6b6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ee6b6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bae6b7e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb26e6b80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e6b83, SUC_lexicon_tags_37 },
    { 0x95ee6b84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586e6b88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38e6b8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10e6b8f, SUC_lexicon_tags_47 },
    { 0x8f4e6b8f, SUC_lexicon_tags_101 },
    { 0xff2e6b8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ae6b94, SUC_lexicon_tags_18 },
    { 0xb6e6b95, SUC_lexicon_tags_17 },
    { 0x620e6b96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82ae6b98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaae6b9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb48e6b9d, SUC_lexicon_tags_57 },
    { 0xe28e6b9d, SUC_lexicon_tags_12 },
    { 0xc56e6b9f, SUC_lexicon_tags_68 },
    { 0x5eee6ba0, SUC_lexicon_tags_29 },
    { 0x81ae6ba1, SUC_lexicon_tags_11 },
    { 0x322e6ba2, SUC_lexicon_tags_55 },
    { 0x42ce6ba3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cae6ba6, SUC_lexicon_tags_15 },
    { 0xdbee6ba6, SUC_lexicon_tags_17 },
    { 0xe90e6ba6, SUC_lexicon_tags_17 },
    { 0x4b6e6ba9, SUC_lexicon_tags_38 },
    { 0x8f2e6ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fce6baf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecae6bb5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ae6bb8, SUC_lexicon_tags_29 },
    { 0xbf6e6bb8, SUC_lexicon_tags_1 },
    { 0x2bee6bba, SUC_lexicon_tags_79 },
    { 0xb6e6bba, SUC_lexicon_tags_37 },
    { 0x278e6bba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb0e6bbe, SUC_lexicon_tags_47 },
    { 0x118e6bbf, SUC_lexicon_tags_11 },
    { 0xf9ae6bc0, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ae6bc7, SUC_lexicon_tags_47 },
    { 0x34ae6bc7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf8e6bca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4e6bd2, SUC_lexicon_tags_25 },
    { 0xbcee6bd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2e6bd6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x108e6bd8, SUC_lexicon_tags_37 },
    { 0x272e6bd8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ae6bdf, SUC_lexicon_tags_31 },
    { 0xd8ee6be0, SUC_lexicon_tags_25 },
    { 0x840e6be1, SUC_lexicon_tags_29 },
    { 0x860e6be2, SUC_lexicon_tags_15 },
    { 0xce4e6be1, SUC_lexicon_tags_30 },
    { 0xbcce6be3, SUC_lexicon_tags_33 },
    { 0xbace6be0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90ce6be7, SUC_lexicon_tags_3 },
    { 0x29ae6be8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e6bef, SUC_lexicon_tags_14 },
    { 0x8a0e6bf0, SUC_lexicon_tags_38 },
    { 0x5d6e6bf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6ae6bf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x522e6bf6, SUC_lexicon_tags_25 },
    { 0x5dae6bf7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2e6c02, SUC_lexicon_tags_11 },
    { 0xf88e6c02, SUC_lexicon_tags_193 },
    { 0xa8ae6c03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dae6c06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ee6c09, SUC_lexicon_tags_25 },
    { 0x786e6c09, SUC_lexicon_tags_25 },
    { 0x952e6c0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456e6c0f, SUC_lexicon_tags_148 },
    { 0x9c4e6c10, SUC_lexicon_tags_26 },
    { 0xf18e6c11, SUC_lexicon_tags_25 },
    { 0x626e6c12, SUC_lexicon_tags_63 },
    { 0xcc8e6c12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36e6c18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cce6c1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d4e6c1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea6e6c1e, SUC_lexicon_tags_1 },
    { 0xe6ee6c1f, SUC_lexicon_tags_11 },
    { 0x76e6c20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110e6c2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x84ce6c2e, SUC_lexicon_tags_7 },
    { 0x6a6e6c2f, SUC_lexicon_tags_14 },
    { 0xe12e6c30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c6e6c32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40e6c36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x626e6c38, SUC_lexicon_tags_85 },
    { 0x668e6c39, SUC_lexicon_tags_102 },
    { 0x95ee6c3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8e6c3d, SUC_lexicon_tags_12 },
    { 0x1d6e6c3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x106e6c40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ee6c44, SUC_lexicon_tags_45 },
    { 0x950e6c44, SUC_lexicon_tags_25 },
    { 0x53ee6c46, SUC_lexicon_tags_33 },
    { 0xeaae6c47, SUC_lexicon_tags_41 },
    { 0xe16e6c45, SUC_lexicon_tags_16 },
    { 0x312e6c45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e6c4d, SUC_lexicon_tags_14 },
    { 0x150e6c4e, SUC_lexicon_tags_7 },
    { 0x2d4e6c4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928e6c58, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf6e6c5a, SUC_lexicon_tags_25 },
    { 0x6f6e6c5b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x880e6c5d, SUC_lexicon_tags_11 },
    { 0x7d6e6c5e, SUC_lexicon_tags_31 },
    { 0x7e4e6c5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xffce6c62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2e6c65, SUC_lexicon_tags_17 },
    { 0xf3ae6c66, SUC_lexicon_tags_23 },
    { 0x1c8e6c66, SUC_lexicon_tags_25 },
    { 0xb9ee6c68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ae6c6b, SUC_lexicon_tags_1 },
    { 0x32ae6c6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6e6c71, SUC_lexicon_tags_2 },
    { 0x2d8e6c71, SUC_lexicon_tags_26 },
    { 0x250e6c73, SUC_lexicon_tags_28 },
    { 0xe5ae6c74, SUC_lexicon_tags_17 },
    { 0x42ae6c71, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x982e6c77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x252e6c7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7b8e6c7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x340e6c7f, SUC_lexicon_tags_1 },
    { 0x916e6c80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204e6c86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86e6c88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x962e6c8b, SUC_lexicon_tags_31 },
    { 0x320e6c8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604e6c90, SUC_lexicon_tags_38 },
    { 0x94ce6c91, SUC_lexicon_tags_12 },
    { 0x52ae6c91, SUC_lexicon_tags_25 },
    { 0x622e6c93, SUC_lexicon_tags_11 },
    { 0xa68e6c94, SUC_lexicon_tags_120 },
    { 0x406e6c95, SUC_lexicon_tags_1 },
    { 0x5f6e6c96, SUC_lexicon_tags_55 },
    { 0xf76e6c97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3ae6c99, SUC_lexicon_tags_30 },
    { 0xf16e6c9a, SUC_lexicon_tags_19 },
    { 0xe3ae6c99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32e6ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0e6ca7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ce6cb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2e6cb9, SUC_lexicon_tags_33 },
    { 0x178e6cb9, SUC_lexicon_tags_25 },
    { 0x978e6cbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38e6cbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75ee6cc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ae6cc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x616e6cc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564e6ccb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6ae6ccd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xdc4e6ccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600e6cd3, SUC_lexicon_tags_18 },
    { 0xe40e6cd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcae6cd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6e6cdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cae6cdd, SUC_lexicon_tags_21 },
    { 0xcace6cde, SUC_lexicon_tags_33 },
    { 0x9a2e6cdf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacce6ce8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x114e6ceb, SUC_lexicon_tags_13 },
    { 0xcdce6ceb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516e6cf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0e6cff, SUC_lexicon_tags_1 },
    { 0x686e6d00, SUC_lexicon_tags_5 },
    { 0x750e6d01, SUC_lexicon_tags_2 },
    { 0xa6ae6d02, SUC_lexicon_tags_3 },
    { 0xfeee6d03, SUC_lexicon_tags_214 },
    { 0xa40e6d00, SUC_lexicon_tags_25 },
    { 0xdf4e6d03, SUC_lexicon_tags_12 },
    { 0xfbae6d01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ee6d09, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf10e6d0c, SUC_lexicon_tags_2 },
    { 0xf1ee6d0d, SUC_lexicon_tags_37 },
    { 0x1c8e6d0e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4c8e6d10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x226e6d12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586e6d17, SUC_lexicon_tags_31 },
    { 0x798e6d17, SUC_lexicon_tags_12 },
    { 0x496e6d19, SUC_lexicon_tags_43 },
    { 0xdee6d19, SUC_lexicon_tags_25 },
    { 0xb6e6d1b, SUC_lexicon_tags_15 },
    { 0x31ae6d19, SUC_lexicon_tags_15 },
    { 0x5ace6d1d, SUC_lexicon_tags_25 },
    { 0x380e6d1c, SUC_lexicon_tags_45 },
    { 0xdd6e6d1f, SUC_lexicon_tags_28 },
    { 0xa5ee6d19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbece6d27, SUC_lexicon_tags_25 },
    { 0x646e6d28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf62e6d2a, SUC_lexicon_tags_30 },
    { 0xd2ce6d2a, SUC_lexicon_tags_12 },
    { 0xfdce6d2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788e6d31, SUC_lexicon_tags_48 },
    { 0x272e6d31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x242e6d35, SUC_lexicon_tags_25 },
    { 0x460e6d35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fce6d38, SUC_lexicon_tags_25 },
    { 0xcd8e6d38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8e6d3d, SUC_lexicon_tags_10 },
    { 0x79ee6d3d, SUC_lexicon_tags_46 },
    { 0xc94e6d3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0e6d43, SUC_lexicon_tags_26 },
    { 0xd4ae6d44, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd76e6d46, SUC_lexicon_tags_63 },
    { 0x362e6d47, SUC_lexicon_tags_25 },
    { 0x24e6d48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2e6d4d, SUC_lexicon_tags_47 },
    { 0xef4e6d4d, SUC_lexicon_tags_65 },
    { 0x438e6d4d, SUC_lexicon_tags_31 },
    { 0xe70e6d4d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6e6d53, SUC_lexicon_tags_82 },
    { 0xa8e6d54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08e6d5c, SUC_lexicon_tags_37 },
    { 0x85ee6d5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ee6d61, SUC_lexicon_tags_25 },
    { 0x420e6d61, SUC_lexicon_tags_17 },
    { 0xdcee6d62, SUC_lexicon_tags_15 },
    { 0xc14e6d64, SUC_lexicon_tags_30 },
    { 0xdb0e6d65, SUC_lexicon_tags_32 },
    { 0x654e6d61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e6d6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x326e6d6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x558e6d70, SUC_lexicon_tags_11 },
    { 0x9d0e6d71, SUC_lexicon_tags_25 },
    { 0x8eee6d72, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8e6d77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf2e6d79, SUC_lexicon_tags_12 },
    { 0x48ae6d79, SUC_lexicon_tags_1 },
    { 0x7b4e6d7b, SUC_lexicon_tags_2 },
    { 0xab0e6d7b, SUC_lexicon_tags_37 },
    { 0xef6e6d79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef4e6d7f, SUC_lexicon_tags_1 },
    { 0x27ee6d7f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ae6d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2e6d89, SUC_lexicon_tags_26 },
    { 0x7c0e6d89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce6d8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x288e6d90, SUC_lexicon_tags_49 },
    { 0xda4e6d91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ee6d96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e6d99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ae6da6, SUC_lexicon_tags_151 },
    { 0xcaae6da7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca6e6da9, SUC_lexicon_tags_1 },
    { 0x5e8e6daa, SUC_lexicon_tags_27 },
    { 0x77ae6daa, SUC_lexicon_tags_8 },
    { 0x4ae6da9, SUC_lexicon_tags_45 },
    { 0x3dae6dad, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x81ce6daf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac8e6db1, SUC_lexicon_tags_15 },
    { 0x23ae6db2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccce6dba, SUC_lexicon_tags_37 },
    { 0x956e6dba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x308e6dbe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fae6dc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576e6dc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96ae6dc9, SUC_lexicon_tags_15 },
    { 0x596e6dca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278e6dcf, SUC_lexicon_tags_47 },
    { 0x524e6dcf, SUC_lexicon_tags_37 },
    { 0x4e8e6dcf, SUC_lexicon_tags_25 },
    { 0x962e6dd2, SUC_lexicon_tags_109 },
    { 0xc4ee6dd3, SUC_lexicon_tags_19 },
    { 0xe40e6dd3, SUC_lexicon_tags_14 },
    { 0x60ce6dd5, SUC_lexicon_tags_27 },
    { 0xa72e6dd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc0ee6dd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4dce6dda, SUC_lexicon_tags_9 },
    { 0xda2e6dda, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282e6ddf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bae6de2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5c4e6de4, SUC_lexicon_tags_12 },
    { 0xc20e6de5, SUC_lexicon_tags_17 },
    { 0xb00e6de5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2e6def, SUC_lexicon_tags_0 },
    { 0x3ae6df0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2e6df7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ee6dfc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dee6dff, SUC_lexicon_tags_27 },
    { 0xd24e6e00, SUC_lexicon_tags_100 },
    { 0x2d4e6e00, SUC_lexicon_tags_10 },
    { 0xd36e6e02, SUC_lexicon_tags_70 },
    { 0x902e6e02, SUC_lexicon_tags_9 },
    { 0x2f8e6e04, SUC_lexicon_tags_13 },
    { 0xee6e05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ce6e08, SUC_lexicon_tags_0 },
    { 0x80ae6e09, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4e6e0d, SUC_lexicon_tags_26 },
    { 0x374e6e0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x644e6e10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x560e6e12, SUC_lexicon_tags_37 },
    { 0xac8e6e13, SUC_lexicon_tags_1 },
    { 0xd28e6e13, SUC_lexicon_tags_14 },
    { 0xd98e6e15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cce6e17, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886e6e1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75ae6e20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ae6e26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616e6e33, SUC_lexicon_tags_25 },
    { 0xe6e6e34, SUC_lexicon_tags_13 },
    { 0x978e6e34, SUC_lexicon_tags_47 },
    { 0x150e6e36, SUC_lexicon_tags_4 },
    { 0x426e6e37, SUC_lexicon_tags_14 },
    { 0x9e4e6e38, SUC_lexicon_tags_17 },
    { 0x648e6e39, SUC_lexicon_tags_17 },
    { 0xdae6e38, SUC_lexicon_tags_15 },
    { 0x6cee6e3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ace6e3f, SUC_lexicon_tags_38 },
    { 0x410e6e40, SUC_lexicon_tags_37 },
    { 0xe64e6e41, SUC_lexicon_tags_12 },
    { 0x77ae6e42, SUC_lexicon_tags_49 },
    { 0x272e6e42, SUC_lexicon_tags_17 },
    { 0xce6e43, SUC_lexicon_tags_14 },
    { 0x528e6e44, SUC_lexicon_tags_29 },
    { 0xef0e6e43, SUC_lexicon_tags_23 },
    { 0x36ee6e47, SUC_lexicon_tags_82 },
    { 0x610e6e48, SUC_lexicon_tags_11 },
    { 0x93ae6e49, SUC_lexicon_tags_9 },
    { 0x466e6e48, SUC_lexicon_tags_47 },
    { 0x7ee6e4a, SUC_lexicon_tags_5 },
    { 0x380e6e42, SUC_lexicon_tags_11 },
    { 0x6aee6e4d, SUC_lexicon_tags_37 },
    { 0x6e4e6e42, SUC_lexicon_tags_16 },
    { 0x61ce6e4f, SUC_lexicon_tags_33 },
    { 0x85ee6e50, SUC_lexicon_tags_25 },
    { 0xd02e6e51, SUC_lexicon_tags_8 },
    { 0xd4e6e4d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x98ee6e54, SUC_lexicon_tags_17 },
    { 0x18ce6e55, SUC_lexicon_tags_66 },
    { 0x6f0e6e56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2e6e5a, SUC_lexicon_tags_25 },
    { 0x1b2e6e5a, SUC_lexicon_tags_65 },
    { 0x94ae6e5b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800e6e62, SUC_lexicon_tags_65 },
    { 0x828e6e62, SUC_lexicon_tags_12 },
    { 0x366e6e64, SUC_lexicon_tags_92 },
    { 0x9b2e6e64, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd7ae6e67, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ae6e6c, SUC_lexicon_tags_0 },
    { 0x8cae6e6d, SUC_lexicon_tags_41 },
    { 0x7eee6e6c, SUC_lexicon_tags_146 },
    { 0xba6e6e6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaace6e71, SUC_lexicon_tags_11 },
    { 0xb08e6e71, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6fee6e74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x258e6e76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0e6e79, SUC_lexicon_tags_37 },
    { 0x866e6e7a, SUC_lexicon_tags_23 },
    { 0x83ce6e7b, SUC_lexicon_tags_15 },
    { 0xa74e6e7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad6e6e7e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x56e6e80, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb50e6e82, SUC_lexicon_tags_0 },
    { 0x29ee6e83, SUC_lexicon_tags_13 },
    { 0xeb2e6e83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8e6e87, SUC_lexicon_tags_25 },
    { 0xae6e6e88, SUC_lexicon_tags_2 },
    { 0xfdce6e89, SUC_lexicon_tags_6 },
    { 0xbd4e6e8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xface6e8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ae6e91, SUC_lexicon_tags_49 },
    { 0xdd6e6e92, SUC_lexicon_tags_15 },
    { 0xd12e6e91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ce6e9b, SUC_lexicon_tags_28 },
    { 0xae4e6e9c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecee6ea0, SUC_lexicon_tags_28 },
    { 0x8cae6ea1, SUC_lexicon_tags_4 },
    { 0xe24e6ea2, SUC_lexicon_tags_77 },
    { 0x298e6ea3, SUC_lexicon_tags_60 },
    { 0xda2e6ea4, SUC_lexicon_tags_32 },
    { 0xa5ce6ea1, SUC_lexicon_tags_17 },
    { 0x5dce6ea6, SUC_lexicon_tags_17 },
    { 0xe5ee6ea7, SUC_lexicon_tags_2 },
    { 0x9f6e6ea7, SUC_lexicon_tags_68 },
    { 0xdb0e6ea9, SUC_lexicon_tags_38 },
    { 0xc22e6ea9, SUC_lexicon_tags_27 },
    { 0xdfae6ea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x380e6eae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ae6eb3, SUC_lexicon_tags_13 },
    { 0x4cce6eb3, SUC_lexicon_tags_2 },
    { 0x6dae6eb5, SUC_lexicon_tags_49 },
    { 0x53ce6eb5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6e6ebb, SUC_lexicon_tags_27 },
    { 0x126e6ebc, SUC_lexicon_tags_2 },
    { 0x3c4e6ebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6e6ec0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ce6ec3, SUC_lexicon_tags_15 },
    { 0x634e6ec4, SUC_lexicon_tags_11 },
    { 0xace6ec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784e6ecb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16e6ecf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4e6ed2, SUC_lexicon_tags_30 },
    { 0x4d6e6ed3, SUC_lexicon_tags_26 },
    { 0x4cce6ed2, SUC_lexicon_tags_2 },
    { 0xdf2e6ed5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802e6eda, SUC_lexicon_tags_37 },
    { 0x3f8e6edb, SUC_lexicon_tags_28 },
    { 0x4a4e6edc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6e6ee2, SUC_lexicon_tags_61 },
    { 0xa36e6ee3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfc6e6ee5, SUC_lexicon_tags_26 },
    { 0x22e6ee6, SUC_lexicon_tags_61 },
    { 0xcae6ee7, SUC_lexicon_tags_27 },
    { 0x9ee6ee8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e0e6eea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dae6eed, SUC_lexicon_tags_66 },
    { 0x940e6eee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x144e6ef0, SUC_lexicon_tags_17 },
    { 0x23ce6ef1, SUC_lexicon_tags_46 },
    { 0x380e6ef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee6ef6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6aee6ef8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dae6f00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858e6f05, SUC_lexicon_tags_25 },
    { 0xaf6e6f06, SUC_lexicon_tags_55 },
    { 0x4d0e6f07, SUC_lexicon_tags_141 },
    { 0x296e6f08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ace6f0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ee6f13, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4e6f18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4e6f1d, SUC_lexicon_tags_1 },
    { 0x6e2e6f1e, SUC_lexicon_tags_33 },
    { 0x1ce6f1f, SUC_lexicon_tags_65 },
    { 0x89ce6f1f, SUC_lexicon_tags_28 },
    { 0x37ee6f21, SUC_lexicon_tags_49 },
    { 0xa44e6f20, SUC_lexicon_tags_17 },
    { 0x326e6f23, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ae6f27, SUC_lexicon_tags_87 },
    { 0x7a6e6f27, SUC_lexicon_tags_2 },
    { 0x30e6f29, SUC_lexicon_tags_13 },
    { 0x736e6f28, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88e6f2e, SUC_lexicon_tags_37 },
    { 0xfdae6f2f, SUC_lexicon_tags_39 },
    { 0x4d0e6f2f, SUC_lexicon_tags_17 },
    { 0xf48e6f31, SUC_lexicon_tags_25 },
    { 0x2fae6f31, SUC_lexicon_tags_60 },
    { 0x386e6f33, SUC_lexicon_tags_4 },
    { 0x320e6f33, SUC_lexicon_tags_14 },
    { 0x5d4e6f35, SUC_lexicon_tags_30 },
    { 0x7b0e6f33, SUC_lexicon_tags_9 },
    { 0xf7ee6f34, SUC_lexicon_tags_6 },
    { 0x3ace6f36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x708e6f3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64e6f3f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x632e6f41, SUC_lexicon_tags_17 },
    { 0xf52e6f42, SUC_lexicon_tags_87 },
    { 0x59ee6f41, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccee6f48, SUC_lexicon_tags_17 },
    { 0x3eae6f49, SUC_lexicon_tags_46 },
    { 0x97ee6f48, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28e6f4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3ce6f4f, SUC_lexicon_tags_18 },
    { 0xb74e6f50, SUC_lexicon_tags_26 },
    { 0xd24e6f50, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ce6f55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70e6f58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ae6f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x206e6f5f, SUC_lexicon_tags_14 },
    { 0xc84e6f60, SUC_lexicon_tags_2 },
    { 0xdee6f61, SUC_lexicon_tags_37 },
    { 0xae6e6f61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e6f65, SUC_lexicon_tags_0 },
    { 0x1a0e6f66, SUC_lexicon_tags_83 },
    { 0x50e6f66, SUC_lexicon_tags_25 },
    { 0x976e6f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ae6f6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72e6f71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ae6f73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6e6f76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xed6e6f78, SUC_lexicon_tags_37 },
    { 0x1ace6f79, SUC_lexicon_tags_16 },
    { 0xbfae6f79, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6e6f7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bee6f80, SUC_lexicon_tags_26 },
    { 0x464e6f80, SUC_lexicon_tags_15 },
    { 0x350e6f82, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc50e6f84, SUC_lexicon_tags_1 },
    { 0x78ce6f85, SUC_lexicon_tags_26 },
    { 0x3a8e6f86, SUC_lexicon_tags_151 },
    { 0xf0e6f87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xecee6f8a, SUC_lexicon_tags_1 },
    { 0x8dae6f8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ae6f8f, SUC_lexicon_tags_37 },
    { 0x40ae6f90, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x196e6f93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6e6f96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x27ee6f98, SUC_lexicon_tags_46 },
    { 0xb9ae6f99, SUC_lexicon_tags_27 },
    { 0x91ee6f99, SUC_lexicon_tags_26 },
    { 0x71ce6f99, SUC_lexicon_tags_19 },
    { 0x148e6f9c, SUC_lexicon_tags_23 },
    { 0xb72e6f9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8e6fac, SUC_lexicon_tags_47 },
    { 0x34e6fad, SUC_lexicon_tags_34 },
    { 0xb6ce6fae, SUC_lexicon_tags_3 },
    { 0xbdae6fae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee2e6fbc, SUC_lexicon_tags_46 },
    { 0x214e6fbd, SUC_lexicon_tags_60 },
    { 0xdc0e6fbe, SUC_lexicon_tags_104 },
    { 0xc1ce6fbe, SUC_lexicon_tags_1 },
    { 0x792e6fc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd90e6fc2, SUC_lexicon_tags_11 },
    { 0xf9ee6fc2, SUC_lexicon_tags_25 },
    { 0x53ce6fc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6e6fc8, SUC_lexicon_tags_2 },
    { 0x364e6fc9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518e6fcf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x694e6fd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8e6fd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36e6fd6, SUC_lexicon_tags_25 },
    { 0x47ee6fd6, SUC_lexicon_tags_48 },
    { 0x552e6fd8, SUC_lexicon_tags_7 },
    { 0xea2e6fd8, SUC_lexicon_tags_39 },
    { 0xd06e6fda, SUC_lexicon_tags_13 },
    { 0xa04e6fdb, SUC_lexicon_tags_29 },
    { 0x1ae6fdb, SUC_lexicon_tags_0 },
    { 0xebce6fd6, SUC_lexicon_tags_38 },
    { 0x572e6fde, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefee6fe3, SUC_lexicon_tags_37 },
    { 0x374e6fe4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x23ce6fe6, SUC_lexicon_tags_17 },
    { 0x1dae6fe7, SUC_lexicon_tags_10 },
    { 0xeeae6fe8, SUC_lexicon_tags_32 },
    { 0xe0ce6fe9, SUC_lexicon_tags_82 },
    { 0x7fee6fe9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8e6fef, SUC_lexicon_tags_47 },
    { 0xfd6e6ff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x130e6ff3, SUC_lexicon_tags_43 },
    { 0xd9ee6ff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ee6ff8, SUC_lexicon_tags_27 },
    { 0x1ace6ff8, SUC_lexicon_tags_37 },
    { 0xeb4e6ff9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30e6ffe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf5ae7000, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xce2e7002, SUC_lexicon_tags_12 },
    { 0xfb0e7003, SUC_lexicon_tags_25 },
    { 0x2aee7002, SUC_lexicon_tags_11 },
    { 0xe46e7004, SUC_lexicon_tags_71 },
    { 0x6eae7006, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe9ae7008, SUC_lexicon_tags_37 },
    { 0xd98e7009, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee700c, SUC_lexicon_tags_60 },
    { 0x25ce700d, SUC_lexicon_tags_38 },
    { 0xf1ce700d, SUC_lexicon_tags_11 },
    { 0x68e700f, SUC_lexicon_tags_0 },
    { 0x426e700f, SUC_lexicon_tags_11 },
    { 0x9c4e7011, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370e7016, SUC_lexicon_tags_14 },
    { 0xc0e7017, SUC_lexicon_tags_2 },
    { 0xd10e7016, SUC_lexicon_tags_25 },
    { 0x392e7019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f6e701b, SUC_lexicon_tags_15 },
    { 0xce701c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdce701e, SUC_lexicon_tags_14 },
    { 0xdf8e701f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4e0e7021, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48e7026, SUC_lexicon_tags_1 },
    { 0xb80e7026, SUC_lexicon_tags_47 },
    { 0x7ace7028, SUC_lexicon_tags_38 },
    { 0x21ae7027, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18e702e, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xdc6e7030, SUC_lexicon_tags_11 },
    { 0x48ce7030, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32e7034, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x682e7036, SUC_lexicon_tags_23 },
    { 0x24ee7037, SUC_lexicon_tags_2 },
    { 0xd7ee7038, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dae703a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e8e703c, SUC_lexicon_tags_11 },
    { 0x796e703d, SUC_lexicon_tags_45 },
    { 0xbaee703d, SUC_lexicon_tags_12 },
    { 0xb58e703f, SUC_lexicon_tags_15 },
    { 0xf94e7040, SUC_lexicon_tags_45 },
    { 0x4d6e7040, SUC_lexicon_tags_49 },
    { 0x8cae7041, SUC_lexicon_tags_26 },
    { 0xcce7043, SUC_lexicon_tags_15 },
    { 0xda2e7044, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ace704a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x922e704d, SUC_lexicon_tags_2 },
    { 0x318e704d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce7051, SUC_lexicon_tags_0 },
    { 0xc04e7052, SUC_lexicon_tags_26 },
    { 0xc32e7053, SUC_lexicon_tags_17 },
    { 0x218e7054, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1eae7056, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x188e7058, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16ce705a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48e705e, SUC_lexicon_tags_30 },
    { 0xadce705f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd8ee7061, SUC_lexicon_tags_47 },
    { 0xee8e7062, SUC_lexicon_tags_17 },
    { 0x138e7061, SUC_lexicon_tags_11 },
    { 0x424e7061, SUC_lexicon_tags_3 },
    { 0x73ee7065, SUC_lexicon_tags_45 },
    { 0x8a6e7062, SUC_lexicon_tags_17 },
    { 0x960e7067, SUC_lexicon_tags_28 },
    { 0x386e7068, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc48e706a, SUC_lexicon_tags_1 },
    { 0x498e706a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6e7074, SUC_lexicon_tags_23 },
    { 0xf74e7075, SUC_lexicon_tags_28 },
    { 0xdd8e7075, SUC_lexicon_tags_15 },
    { 0xe98e7076, SUC_lexicon_tags_1 },
    { 0x5a6e7076, SUC_lexicon_tags_60 },
    { 0x60ce7075, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ce707f, SUC_lexicon_tags_37 },
    { 0x4a2e707f, SUC_lexicon_tags_23 },
    { 0x438e7081, SUC_lexicon_tags_14 },
    { 0x870e7082, SUC_lexicon_tags_21 },
    { 0x4e8e7081, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4e7089, SUC_lexicon_tags_37 },
    { 0x33ee708a, SUC_lexicon_tags_11 },
    { 0x70e708b, SUC_lexicon_tags_45 },
    { 0x50ae708b, SUC_lexicon_tags_0 },
    { 0xcae708d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64e7096, SUC_lexicon_tags_7 },
    { 0xacce7097, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xac4e7099, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf2e709b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96ee709d, SUC_lexicon_tags_14 },
    { 0x590e709e, SUC_lexicon_tags_1 },
    { 0x98e709f, SUC_lexicon_tags_31 },
    { 0x90ce709f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x300e70a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ee70a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6e70ae, SUC_lexicon_tags_0 },
    { 0xcce70af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8e70b2, SUC_lexicon_tags_0 },
    { 0x152e70b2, SUC_lexicon_tags_37 },
    { 0xa9ce70b4, SUC_lexicon_tags_5 },
    { 0xc08e70b5, SUC_lexicon_tags_47 },
    { 0x9c4e70b3, SUC_lexicon_tags_14 },
    { 0x14ce70b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35ce70b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34e70bc, SUC_lexicon_tags_17 },
    { 0xe18e70bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2e70c1, SUC_lexicon_tags_15 },
    { 0xb68e70c2, SUC_lexicon_tags_1 },
    { 0xda8e70c3, SUC_lexicon_tags_160 },
    { 0xa06e70c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x698e70c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa16e70c8, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70e70cf, SUC_lexicon_tags_15 },
    { 0xa28e70d0, SUC_lexicon_tags_15 },
    { 0x880e70d1, SUC_lexicon_tags_28 },
    { 0x2a4e70d2, SUC_lexicon_tags_1 },
    { 0x92ce70d3, SUC_lexicon_tags_47 },
    { 0x1c6e70d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffee70d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0e70dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ae70e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72e70e7, SUC_lexicon_tags_55 },
    { 0xefce70e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aae70ec, SUC_lexicon_tags_25 },
    { 0xe6e70ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aae70ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x862e70f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ae70f6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfb2e70f8, SUC_lexicon_tags_23 },
    { 0xdc8e70f8, SUC_lexicon_tags_15 },
    { 0xa54e70f9, SUC_lexicon_tags_48 },
    { 0xfeae70fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x158e70fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6c0e7100, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdee7106, SUC_lexicon_tags_49 },
    { 0xe24e7107, SUC_lexicon_tags_55 },
    { 0xf7ae7108, SUC_lexicon_tags_12 },
    { 0xe58e7109, SUC_lexicon_tags_25 },
    { 0x52ce710a, SUC_lexicon_tags_43 },
    { 0xaaee710b, SUC_lexicon_tags_68 },
    { 0x574e710b, SUC_lexicon_tags_13 },
    { 0x7aae710d, SUC_lexicon_tags_1 },
    { 0xee4e710d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4e7115, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96e7117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4e711b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ae711d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf80e711f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1b6e7121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aae7125, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4aae7127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ce712b, SUC_lexicon_tags_37 },
    { 0x96ae712c, SUC_lexicon_tags_17 },
    { 0xfe6e712d, SUC_lexicon_tags_1 },
    { 0x818e712b, SUC_lexicon_tags_38 },
    { 0xc5ce712f, SUC_lexicon_tags_37 },
    { 0x11ce7130, SUC_lexicon_tags_11 },
    { 0x75ce712b, SUC_lexicon_tags_47 },
    { 0x1bae7132, SUC_lexicon_tags_1 },
    { 0xaf8e7133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ae7137, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282e713c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ee713f, SUC_lexicon_tags_158 },
    { 0xa06e7140, SUC_lexicon_tags_63 },
    { 0x138e7140, SUC_lexicon_tags_29 },
    { 0x486e7142, SUC_lexicon_tags_14 },
    { 0xa40e713f, SUC_lexicon_tags_15 },
    { 0x962e713f, SUC_lexicon_tags_1 },
    { 0x5e6e7145, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x268e7147, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2e714d, SUC_lexicon_tags_37 },
    { 0xc30e714d, SUC_lexicon_tags_63 },
    { 0x2b6e714d, SUC_lexicon_tags_37 },
    { 0x4bee7150, SUC_lexicon_tags_7 },
    { 0x1d2e7151, SUC_lexicon_tags_25 },
    { 0x68ce714e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa2e7154, SUC_lexicon_tags_13 },
    { 0x7ece7155, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936e7159, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428e7162, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8e7169, SUC_lexicon_tags_23 },
    { 0x4e6e716a, SUC_lexicon_tags_47 },
    { 0xdaee7169, SUC_lexicon_tags_47 },
    { 0xe0ce716c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83ee716e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ae7173, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x338e7175, SUC_lexicon_tags_60 },
    { 0x846e7175, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1dce7178, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8e717c, SUC_lexicon_tags_34 },
    { 0x178e717c, SUC_lexicon_tags_15 },
    { 0x2d8e717c, SUC_lexicon_tags_7 },
    { 0x628e717f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeee7184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92e7188, SUC_lexicon_tags_37 },
    { 0x2dee7189, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa6ae718b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e0e718d, SUC_lexicon_tags_31 },
    { 0x1b6e718d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d6e7190, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc8ee7192, SUC_lexicon_tags_32 },
    { 0x4fae7192, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x182e7196, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x15ee7198, SUC_lexicon_tags_8 },
    { 0xda2e7199, SUC_lexicon_tags_112 },
    { 0xdc8e7198, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x934e719d, SUC_lexicon_tags_50 },
    { 0xcdee719d, SUC_lexicon_tags_12 },
    { 0xdcee719f, SUC_lexicon_tags_31 },
    { 0xace719f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2e71a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48e71a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ee71a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4e71ac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ae71af, SUC_lexicon_tags_27 },
    { 0x774e71b0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc2e71b2, SUC_lexicon_tags_2 },
    { 0xdae71b3, SUC_lexicon_tags_37 },
    { 0x76e71b3, SUC_lexicon_tags_63 },
    { 0xe26e71b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x292e71b7, SUC_lexicon_tags_17 },
    { 0x6f4e71b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x248e71ba, SUC_lexicon_tags_28 },
    { 0x7a0e71bb, SUC_lexicon_tags_7 },
    { 0x8eae71bb, SUC_lexicon_tags_60 },
    { 0x3c4e71bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2e71c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6e71c3, SUC_lexicon_tags_9 },
    { 0x680e71c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ce71cb, SUC_lexicon_tags_45 },
    { 0x482e71cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0e71cf, SUC_lexicon_tags_12 },
    { 0xfe8e71d0, SUC_lexicon_tags_17 },
    { 0x24ce71d1, SUC_lexicon_tags_6 },
    { 0x1c0e71d0, SUC_lexicon_tags_12 },
    { 0xb80e71cf, SUC_lexicon_tags_65 },
    { 0x88ee71d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ce71d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48e71db, SUC_lexicon_tags_37 },
    { 0xb22e71db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4e71e1, SUC_lexicon_tags_8 },
    { 0x37ae71e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14e71e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdce71e7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ce71ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748e71ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x886e71f1, SUC_lexicon_tags_77 },
    { 0x140e71f2, SUC_lexicon_tags_12 },
    { 0x578e71f1, SUC_lexicon_tags_25 },
    { 0xd1ae71f4, SUC_lexicon_tags_34 },
    { 0x516e71f4, SUC_lexicon_tags_79 },
    { 0x898e71f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdb6e71f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ce71fb, SUC_lexicon_tags_2 },
    { 0x864e71fc, SUC_lexicon_tags_94 },
    { 0xe08e71fd, SUC_lexicon_tags_11 },
    { 0xee8e71fe, SUC_lexicon_tags_37 },
    { 0x74ce71fe, SUC_lexicon_tags_47 },
    { 0xb28e7200, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x454e7202, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54e7206, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2ce7208, SUC_lexicon_tags_63 },
    { 0x13ee7209, SUC_lexicon_tags_45 },
    { 0x158e720a, SUC_lexicon_tags_23 },
    { 0x42e7208, SUC_lexicon_tags_2 },
    { 0xd9ae720c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0e720f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba0e7211, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8e7214, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x488e7216, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa72e7218, SUC_lexicon_tags_201 },
    { 0xe7ae7219, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fae721b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb78e721d, SUC_lexicon_tags_11 },
    { 0x6f6e721e, SUC_lexicon_tags_12 },
    { 0x2a4e721f, SUC_lexicon_tags_43 },
    { 0xfd8e721e, SUC_lexicon_tags_49 },
    { 0x776e721f, SUC_lexicon_tags_47 },
    { 0x338e721d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4e7227, SUC_lexicon_tags_17 },
    { 0x3a8e7227, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x226e722b, SUC_lexicon_tags_2 },
    { 0x14e722b, SUC_lexicon_tags_47 },
    { 0xf8e722c, SUC_lexicon_tags_7 },
    { 0x6c6e722b, SUC_lexicon_tags_32 },
    { 0xa82e722e, SUC_lexicon_tags_49 },
    { 0x8bee722d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf58e7232, SUC_lexicon_tags_17 },
    { 0xaa8e7233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ce723c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78e7240, SUC_lexicon_tags_64 },
    { 0x7f2e7241, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ee7244, SUC_lexicon_tags_11 },
    { 0x7a6e7245, SUC_lexicon_tags_19 },
    { 0xa52e7246, SUC_lexicon_tags_27 },
    { 0x798e7247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2e724a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f6e724c, SUC_lexicon_tags_25 },
    { 0x22ce724d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x762e7250, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x866e7253, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xeeae7255, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ee725b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ce725f, SUC_lexicon_tags_1 },
    { 0xb66e7260, SUC_lexicon_tags_37 },
    { 0xb3ce7261, SUC_lexicon_tags_1 },
    { 0x5cce7262, SUC_lexicon_tags_525 },
    { 0xea2e7263, SUC_lexicon_tags_82 },
    { 0x774e7261, SUC_lexicon_tags_7 },
    { 0xf24e7265, SUC_lexicon_tags_11 },
    { 0xde8e7264, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92e726d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e7275, SUC_lexicon_tags_17 },
    { 0x82ce7275, SUC_lexicon_tags_37 },
    { 0xfa6e7277, SUC_lexicon_tags_1 },
    { 0x63ee7276, SUC_lexicon_tags_12 },
    { 0x91ee7279, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xdd4e727b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4e727e, SUC_lexicon_tags_19 },
    { 0xc28e727f, SUC_lexicon_tags_60 },
    { 0xfc8e7280, SUC_lexicon_tags_43 },
    { 0x428e7280, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xebce7283, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8e728a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12ee728c, SUC_lexicon_tags_2 },
    { 0x96ce728d, SUC_lexicon_tags_17 },
    { 0x638e728e, SUC_lexicon_tags_1 },
    { 0x6aee728f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48e7293, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe48e7295, SUC_lexicon_tags_23 },
    { 0x300e7295, SUC_lexicon_tags_37 },
    { 0xc04e7296, SUC_lexicon_tags_23 },
    { 0xc90e7296, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9eee729a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98ee729c, SUC_lexicon_tags_27 },
    { 0x9a2e729c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6e72a2, SUC_lexicon_tags_38 },
    { 0xb80e72a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb18e72a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfae72a9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688e72ad, SUC_lexicon_tags_11 },
    { 0xe5ce72ae, SUC_lexicon_tags_1055 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ce72b1, SUC_lexicon_tags_25 },
    { 0x22ae72b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ce72b5, SUC_lexicon_tags_1 },
    { 0xa72e72b6, SUC_lexicon_tags_27 },
    { 0x834e72b7, SUC_lexicon_tags_6 },
    { 0x26ae72b8, SUC_lexicon_tags_17 },
    { 0xf74e72b7, SUC_lexicon_tags_65 },
    { 0x20ce72b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc24e72bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2b8e72be, SUC_lexicon_tags_33 },
    { 0xc62e72bf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0e72c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ce72c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae72c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe82e72cb, SUC_lexicon_tags_37 },
    { 0x6aee72cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88e72cf, SUC_lexicon_tags_15 },
    { 0x30e72d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4e72d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ece72d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3aee72d8, SUC_lexicon_tags_13 },
    { 0x4f8e72d9, SUC_lexicon_tags_37 },
    { 0x7c6e72da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92e72dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x12ae72df, SUC_lexicon_tags_26 },
    { 0xab6e72df, SUC_lexicon_tags_82 },
    { 0x646e72e1, SUC_lexicon_tags_27 },
    { 0x87ce72e2, SUC_lexicon_tags_23 },
    { 0xa78e72e0, SUC_lexicon_tags_1 },
    { 0xa4e72e1, SUC_lexicon_tags_77 },
    { 0x938e72e4, SUC_lexicon_tags_26 },
    { 0xe7ce72e6, SUC_lexicon_tags_55 },
    { 0x2c4e72e7, SUC_lexicon_tags_17 },
    { 0xdbee72e8, SUC_lexicon_tags_23 },
    { 0x666e72e0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e72f5, SUC_lexicon_tags_47 },
    { 0xf7ee72f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xebee72f8, SUC_lexicon_tags_33 },
    { 0x5f0e72f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146e72fd, SUC_lexicon_tags_12 },
    { 0x608e72fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff8e7300, SUC_lexicon_tags_38 },
    { 0x5ee7301, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bee7305, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0e730d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ee7311, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8e7314, SUC_lexicon_tags_21 },
    { 0x7a2e7314, SUC_lexicon_tags_0 },
    { 0xa7ee7314, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6e731a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece731e, SUC_lexicon_tags_63 },
    { 0x7d6e731f, SUC_lexicon_tags_55 },
    { 0x2a2e7320, SUC_lexicon_tags_16 },
    { 0x876e731e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ce7325, SUC_lexicon_tags_14 },
    { 0xa74e7325, SUC_lexicon_tags_25 },
    { 0x1e4e7327, SUC_lexicon_tags_88 },
    { 0x1eee7325, SUC_lexicon_tags_1 },
    { 0x98e7329, SUC_lexicon_tags_25 },
    { 0x98ce732a, SUC_lexicon_tags_4 },
    { 0xb52e7327, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78e732f, SUC_lexicon_tags_2 },
    { 0xc1ee732f, SUC_lexicon_tags_85 },
    { 0x74e732f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebae7339, SUC_lexicon_tags_25 },
    { 0xdeee733a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920e733e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ce7342, SUC_lexicon_tags_11 },
    { 0x7d6e7342, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8e7347, SUC_lexicon_tags_27 },
    { 0x5dee7348, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x790e734b, SUC_lexicon_tags_13 },
    { 0xd9ae734c, SUC_lexicon_tags_17 },
    { 0x996e734c, SUC_lexicon_tags_25 },
    { 0x2d0e734d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fce7351, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x102e7353, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6e735a, SUC_lexicon_tags_38 },
    { 0xe34e735a, SUC_lexicon_tags_53 },
    { 0xdce735c, SUC_lexicon_tags_7 },
    { 0x8ece735d, SUC_lexicon_tags_7 },
    { 0x4c8e735c, SUC_lexicon_tags_25 },
    { 0xd1ce735f, SUC_lexicon_tags_33 },
    { 0xe10e735d, SUC_lexicon_tags_37 },
    { 0x7bce735a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e2e7363, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ae7368, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102e736c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x454e736f, SUC_lexicon_tags_15 },
    { 0x4a2e7370, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e7373, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992e7377, SUC_lexicon_tags_25 },
    { 0xcaee7378, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd92e737a, SUC_lexicon_tags_45 },
    { 0xc0ce737a, SUC_lexicon_tags_30 },
    { 0xb88e737c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb34e737e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfece7383, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x526e7385, SUC_lexicon_tags_37 },
    { 0x808e7386, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee738a, SUC_lexicon_tags_8 },
    { 0xc56e738b, SUC_lexicon_tags_23 },
    { 0x768e738c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44e7392, SUC_lexicon_tags_43 },
    { 0xe6ae7392, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dee73a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ee73a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66e73aa, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x46ae73ac, SUC_lexicon_tags_2 },
    { 0x7e6e73ac, SUC_lexicon_tags_26 },
    { 0xef2e73ac, SUC_lexicon_tags_14 },
    { 0x9aae73af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x394e73b2, SUC_lexicon_tags_37 },
    { 0x780e73b3, SUC_lexicon_tags_32 },
    { 0xece73b4, SUC_lexicon_tags_37 },
    { 0x854e73b5, SUC_lexicon_tags_7 },
    { 0x910e73b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebee73bb, SUC_lexicon_tags_57 },
    { 0x82ee73bc, SUC_lexicon_tags_96 },
    { 0x9ace73bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12e73c0, SUC_lexicon_tags_23 },
    { 0x402e73c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48ce73c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338e73ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee73cd, SUC_lexicon_tags_14 },
    { 0x5a6e73cd, SUC_lexicon_tags_49 },
    { 0x7a6e73cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32e73d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe96e73d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fae73d6, SUC_lexicon_tags_49 },
    { 0xa46e73d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36e73db, SUC_lexicon_tags_1 },
    { 0xc8ee73dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5ae73de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564e73e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73ae73e4, SUC_lexicon_tags_40 },
    { 0xa40e73e4, SUC_lexicon_tags_19 },
    { 0xb3ae73e6, SUC_lexicon_tags_25 },
    { 0x9f4e73e6, SUC_lexicon_tags_37 },
    { 0xbb0e73e8, SUC_lexicon_tags_65 },
    { 0x6dee73e9, SUC_lexicon_tags_17 },
    { 0x3d4e73e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562e73ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80e73f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ee73f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ce73fb, SUC_lexicon_tags_14 },
    { 0xb66e73fc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x136e73fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54e7401, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ce740b, SUC_lexicon_tags_228 },
    { 0x98ae740b, SUC_lexicon_tags_37 },
    { 0x300e740c, SUC_lexicon_tags_37 },
    { 0xd14e740e, SUC_lexicon_tags_37 },
    { 0x77ae740f, SUC_lexicon_tags_12 },
    { 0xd12e7410, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ee7416, SUC_lexicon_tags_31 },
    { 0x87ce7417, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ce741c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd62e741e, SUC_lexicon_tags_0 },
    { 0x6f4e741f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8d6e7421, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d8e7423, SUC_lexicon_tags_28 },
    { 0x46e7423, SUC_lexicon_tags_17 },
    { 0x62ee7425, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ae7428, SUC_lexicon_tags_12 },
    { 0xcd2e7429, SUC_lexicon_tags_25 },
    { 0xae2e7429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ae7433, SUC_lexicon_tags_38 },
    { 0xdbce7433, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52e7444, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106e7448, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6e744b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4e7455, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x17ce7457, SUC_lexicon_tags_49 },
    { 0xb28e7458, SUC_lexicon_tags_43 },
    { 0x6b4e7459, SUC_lexicon_tags_49 },
    { 0x26ee7459, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6e745d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x502e745f, SUC_lexicon_tags_7 },
    { 0x388e7460, SUC_lexicon_tags_37 },
    { 0x4cae7460, SUC_lexicon_tags_61 },
    { 0xacee7462, SUC_lexicon_tags_60 },
    { 0xd68e7462, SUC_lexicon_tags_11 },
    { 0x684e7464, SUC_lexicon_tags_25 },
    { 0xac8e7460, SUC_lexicon_tags_27 },
    { 0x728e7466, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb38e7468, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xacae746a, SUC_lexicon_tags_7 },
    { 0x990e746b, SUC_lexicon_tags_462 },
    { 0, NULL },
    { 0xd54e746d, SUC_lexicon_tags_17 },
    { 0x414e746d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ce7473, SUC_lexicon_tags_19 },
    { 0x99ce7473, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x306e7476, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870e747c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dee747f, SUC_lexicon_tags_60 },
    { 0x8ce7480, SUC_lexicon_tags_37 },
    { 0x842e7480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd2e7483, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ce748b, SUC_lexicon_tags_45 },
    { 0x1aae748c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ce748f, SUC_lexicon_tags_17 },
    { 0x596e7490, SUC_lexicon_tags_30 },
    { 0x956e7491, SUC_lexicon_tags_11 },
    { 0xc20e7491, SUC_lexicon_tags_28 },
    { 0x4cae7493, SUC_lexicon_tags_19 },
    { 0xc36e7491, SUC_lexicon_tags_32 },
    { 0x40e7495, SUC_lexicon_tags_13 },
    { 0x948e7494, SUC_lexicon_tags_27 },
    { 0xfa4e7491, SUC_lexicon_tags_37 },
    { 0x420e7498, SUC_lexicon_tags_11 },
    { 0xa3ae7491, SUC_lexicon_tags_33 },
    { 0xeace7494, SUC_lexicon_tags_27 },
    { 0x848e7495, SUC_lexicon_tags_37 },
    { 0xf8ce7490, SUC_lexicon_tags_57 },
    { 0xb62e7495, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ce74a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8e74a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfbee74a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ce74aa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ce74b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6e74bb, SUC_lexicon_tags_43 },
    { 0x360e74bb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cee74bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76e74c4, SUC_lexicon_tags_11 },
    { 0xf54e74c4, SUC_lexicon_tags_17 },
    { 0x856e74c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0e74c9, SUC_lexicon_tags_11 },
    { 0x636e74ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4e74ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aee74d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932e74d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x470e74d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994e74dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce74e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578e74e5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206e74e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40e74ed, SUC_lexicon_tags_15 },
    { 0x226e74ee, SUC_lexicon_tags_12 },
    { 0x1e8e74ef, SUC_lexicon_tags_1 },
    { 0xf56e74ef, SUC_lexicon_tags_15 },
    { 0xe4e74f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xac6e74f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6e74f6, SUC_lexicon_tags_15 },
    { 0x92ae74f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268e74fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x952e74fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60e7500, SUC_lexicon_tags_38 },
    { 0xe58e7501, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x996e7503, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dee7508, SUC_lexicon_tags_16 },
    { 0x91ae7508, SUC_lexicon_tags_25 },
    { 0x16ee750a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x282e750c, SUC_lexicon_tags_23 },
    { 0xd30e750d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa92e750f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fce7513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa92e7515, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8e751a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x814e751c, SUC_lexicon_tags_1 },
    { 0x46ae751d, SUC_lexicon_tags_63 },
    { 0x9dee751c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ae7521, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbfce7523, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dae752a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ece752d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xface752f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedae7537, SUC_lexicon_tags_28 },
    { 0x4ece7537, SUC_lexicon_tags_38 },
    { 0x902e7539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ee753d, SUC_lexicon_tags_82 },
    { 0x6b8e753e, SUC_lexicon_tags_25 },
    { 0x684e753e, SUC_lexicon_tags_31 },
    { 0x686e753e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0e7547, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77ae7549, SUC_lexicon_tags_0 },
    { 0x542e754a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0e754d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cce7551, SUC_lexicon_tags_66 },
    { 0x6bee7552, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa84e7554, SUC_lexicon_tags_37 },
    { 0x826e7555, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa64e7557, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ce755a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52e755d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x20ee755f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x326e7562, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0e7565, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b4e7567, SUC_lexicon_tags_27 },
    { 0xa3ae7568, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad0e756a, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0e7570, SUC_lexicon_tags_27 },
    { 0xcc8e7571, SUC_lexicon_tags_37 },
    { 0xafce7572, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ce7575, SUC_lexicon_tags_37 },
    { 0xd9ee7576, SUC_lexicon_tags_12 },
    { 0x1e0e7576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e757e, SUC_lexicon_tags_293 },
    { 0x4b8e757e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eae7585, SUC_lexicon_tags_21 },
    { 0x88ee7585, SUC_lexicon_tags_46 },
    { 0x9fee7587, SUC_lexicon_tags_1 },
    { 0x29ee7587, SUC_lexicon_tags_15 },
    { 0xe8e7589, SUC_lexicon_tags_15 },
    { 0x368e758a, SUC_lexicon_tags_63 },
    { 0xc1ee758b, SUC_lexicon_tags_68 },
    { 0x3b6e758a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a4e758e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x718e7590, SUC_lexicon_tags_15 },
    { 0x942e7591, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc8e7593, SUC_lexicon_tags_11 },
    { 0xc9ee7594, SUC_lexicon_tags_25 },
    { 0x1dae7595, SUC_lexicon_tags_14 },
    { 0x1e8e7595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88ce7598, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4e759b, SUC_lexicon_tags_7 },
    { 0x9dce759c, SUC_lexicon_tags_11 },
    { 0xfd8e759d, SUC_lexicon_tags_25 },
    { 0xa46e759e, SUC_lexicon_tags_37 },
    { 0x68ee759f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456e75a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48e75ab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7c4e75ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34e75b1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf72e75b3, SUC_lexicon_tags_27 },
    { 0xa6ae75b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2e75b7, SUC_lexicon_tags_11 },
    { 0x97ee75b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ace75bb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988e75c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e75c8, SUC_lexicon_tags_37 },
    { 0xbc6e75c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ace75ce, SUC_lexicon_tags_1 },
    { 0x576e75cf, SUC_lexicon_tags_11 },
    { 0x588e75d0, SUC_lexicon_tags_27 },
    { 0x818e75d0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x160e75d3, SUC_lexicon_tags_9 },
    { 0x92e75d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cae75d6, SUC_lexicon_tags_14 },
    { 0x57ee75d7, SUC_lexicon_tags_25 },
    { 0x632e75d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2e75e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ce75e6, SUC_lexicon_tags_23 },
    { 0x232e75e6, SUC_lexicon_tags_9 },
    { 0xcb4e75e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44e75ec, SUC_lexicon_tags_11 },
    { 0xa20e75ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x194e75ef, SUC_lexicon_tags_17 },
    { 0x4e0e75f0, SUC_lexicon_tags_7 },
    { 0x386e75f1, SUC_lexicon_tags_12 },
    { 0x3aae75f2, SUC_lexicon_tags_1 },
    { 0x114e75f2, SUC_lexicon_tags_19 },
    { 0xc1ce75f4, SUC_lexicon_tags_15 },
    { 0x14ae75f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x886e75f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390e75fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x886e75fd, SUC_lexicon_tags_60 },
    { 0x7eae75fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcd0e7600, SUC_lexicon_tags_26 },
    { 0xb42e7601, SUC_lexicon_tags_29 },
    { 0xed8e7602, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x350e7605, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6e7609, SUC_lexicon_tags_11 },
    { 0xb48e760a, SUC_lexicon_tags_11 },
    { 0x762e760b, SUC_lexicon_tags_5 },
    { 0xb88e760c, SUC_lexicon_tags_2 },
    { 0xdf2e760a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92ee760f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe10e7611, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2ee7613, SUC_lexicon_tags_31 },
    { 0xc5ae7614, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e4e7616, SUC_lexicon_tags_2 },
    { 0x372e7617, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe3ce7619, SUC_lexicon_tags_1 },
    { 0xde2e7619, SUC_lexicon_tags_25 },
    { 0xd2ae761b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870e7622, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82e7626, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14e7629, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6e762e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14e7632, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x612e7635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0e763c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ae7640, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534e7644, SUC_lexicon_tags_17 },
    { 0xfeae7645, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382e764c, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2e7651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2ce7653, SUC_lexicon_tags_11 },
    { 0xf0e7654, SUC_lexicon_tags_8 },
    { 0x5c6e7655, SUC_lexicon_tags_299 },
    { 0xa14e7656, SUC_lexicon_tags_11 },
    { 0xcbae7656, SUC_lexicon_tags_7 },
    { 0x19ae7657, SUC_lexicon_tags_27 },
    { 0x2fce7655, SUC_lexicon_tags_1 },
    { 0x910e765a, SUC_lexicon_tags_1 },
    { 0xa14e7658, SUC_lexicon_tags_9 },
    { 0x8d2e7658, SUC_lexicon_tags_30 },
    { 0x80e765a, SUC_lexicon_tags_49 },
    { 0xb10e765e, SUC_lexicon_tags_43 },
    { 0x2e765e, SUC_lexicon_tags_194 },
    { 0x704e7660, SUC_lexicon_tags_7 },
    { 0xb6e7661, SUC_lexicon_tags_12 },
    { 0x4bae7660, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb72e7664, SUC_lexicon_tags_17 },
    { 0x6d4e7664, SUC_lexicon_tags_27 },
    { 0x8e0e7665, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8e766a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x46ee766c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x448e766f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa52e7671, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dee7675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236e767c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa0e767e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510e7688, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ae768c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ece768e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x798e7690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ce7695, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a6e7697, SUC_lexicon_tags_25 },
    { 0x52e7697, SUC_lexicon_tags_38 },
    { 0xa98e7698, SUC_lexicon_tags_2 },
    { 0x7b8e7697, SUC_lexicon_tags_62 },
    { 0x170e7697, SUC_lexicon_tags_41 },
    { 0x94e7697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x552e769e, SUC_lexicon_tags_29 },
    { 0xfbee769f, SUC_lexicon_tags_33 },
    { 0x338e76a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b6e76a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6ce76a4, SUC_lexicon_tags_12 },
    { 0xd8ae76a5, SUC_lexicon_tags_62 },
    { 0xfe0e76a6, SUC_lexicon_tags_12 },
    { 0x280e76a7, SUC_lexicon_tags_23 },
    { 0xabee76a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x636e76aa, SUC_lexicon_tags_14 },
    { 0x54e76ab, SUC_lexicon_tags_37 },
    { 0x4e6e76ac, SUC_lexicon_tags_19 },
    { 0xc82e76ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeee76b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768e76b6, SUC_lexicon_tags_23 },
    { 0x88ee76b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb58e76b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdbce76bb, SUC_lexicon_tags_16 },
    { 0x6fae76bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3eae76be, SUC_lexicon_tags_49 },
    { 0x6cae76be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ae76c6, SUC_lexicon_tags_7 },
    { 0xa74e76c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x852e76c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4e76cc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x63ce76ce, SUC_lexicon_tags_12 },
    { 0xd2ee76ce, SUC_lexicon_tags_3 },
    { 0x4e0e76d0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0e76d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e2e76d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a0e76d7, SUC_lexicon_tags_37 },
    { 0x902e76d8, SUC_lexicon_tags_6 },
    { 0x84ae76d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae76de, SUC_lexicon_tags_43 },
    { 0x3ae76df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18e76e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa0e76e4, SUC_lexicon_tags_17 },
    { 0x998e76e5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xce2e76e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea2e76e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70e76ec, SUC_lexicon_tags_26 },
    { 0x54ce76ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4e76f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xadae76f2, SUC_lexicon_tags_19 },
    { 0x252e76f3, SUC_lexicon_tags_14 },
    { 0x1a4e76f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34e76fc, SUC_lexicon_tags_23 },
    { 0x4d2e76fd, SUC_lexicon_tags_60 },
    { 0xb76e76fe, SUC_lexicon_tags_34 },
    { 0x878e76fd, SUC_lexicon_tags_33 },
    { 0xbae7700, SUC_lexicon_tags_17 },
    { 0xf2ee7701, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ae7704, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfc6e7706, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338e770a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8ae770c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0e770f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd92e7711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd62e7713, SUC_lexicon_tags_43 },
    { 0x198e7714, SUC_lexicon_tags_28 },
    { 0x85ce7715, SUC_lexicon_tags_38 },
    { 0x218e7716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b2e7718, SUC_lexicon_tags_46 },
    { 0x138e7719, SUC_lexicon_tags_7 },
    { 0x258e771a, SUC_lexicon_tags_45 },
    { 0xa28e7718, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x96ee771d, SUC_lexicon_tags_25 },
    { 0x6ace771d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876e7722, SUC_lexicon_tags_27 },
    { 0xf46e7723, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86e7727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x670e7729, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6e772c, SUC_lexicon_tags_29 },
    { 0xc86e772c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8e7733, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6e7736, SUC_lexicon_tags_17 },
    { 0xaf8e7737, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ee7741, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ee7745, SUC_lexicon_tags_25 },
    { 0x29ae7746, SUC_lexicon_tags_28 },
    { 0x36e7746, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xadce774a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x716e774c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dce7753, SUC_lexicon_tags_33 },
    { 0x802e7754, SUC_lexicon_tags_25 },
    { 0xee7755, SUC_lexicon_tags_11 },
    { 0x552e7755, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596e7763, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dee7768, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6e776c, SUC_lexicon_tags_11 },
    { 0x57ae776d, SUC_lexicon_tags_23 },
    { 0xd5ae776e, SUC_lexicon_tags_14 },
    { 0xc7ee776c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ee7776, SUC_lexicon_tags_11 },
    { 0x53ee7777, SUC_lexicon_tags_65 },
    { 0xf20e7776, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x328e777b, SUC_lexicon_tags_114 },
    { 0xe58e777b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbe4e777e, SUC_lexicon_tags_2 },
    { 0xc7ae777f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ee7783, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ce778d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4e7790, SUC_lexicon_tags_39 },
    { 0x926e7791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4e7793, SUC_lexicon_tags_56 },
    { 0x6ece7793, SUC_lexicon_tags_54 },
    { 0xfe8e7795, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e779c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8e779f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x570e77a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ae77ad, SUC_lexicon_tags_2 },
    { 0xc1ae77ad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ee77b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24e77b8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe16e77ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ae77bd, SUC_lexicon_tags_10 },
    { 0x27ae77be, SUC_lexicon_tags_47 },
    { 0x42ae77bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ee77c2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaae77c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fce77c8, SUC_lexicon_tags_37 },
    { 0xec4e77c8, SUC_lexicon_tags_124 },
    { 0x56ae77ca, SUC_lexicon_tags_37 },
    { 0xf2ee77ca, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ee77cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fee77d8, SUC_lexicon_tags_17 },
    { 0xceee77d9, SUC_lexicon_tags_2 },
    { 0x9ace77d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ce77de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ee77e1, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x340e77e3, SUC_lexicon_tags_33 },
    { 0x79ee77e4, SUC_lexicon_tags_38 },
    { 0xe74e77e5, SUC_lexicon_tags_7 },
    { 0x102e77e6, SUC_lexicon_tags_26 },
    { 0x4e6e77e6, SUC_lexicon_tags_12 },
    { 0x4d2e77e5, SUC_lexicon_tags_16 },
    { 0x682e77e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ae77ed, SUC_lexicon_tags_14 },
    { 0x9c0e77ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x580e77f0, SUC_lexicon_tags_14 },
    { 0x1f2e77f0, SUC_lexicon_tags_8 },
    { 0x87ce77f1, SUC_lexicon_tags_79 },
    { 0x568e77f3, SUC_lexicon_tags_27 },
    { 0x612e77f3, SUC_lexicon_tags_198 },
    { 0xc48e77f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594e77f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ce77fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x24ce77ff, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6f4e7801, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4e7805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe4e7807, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x684e7809, SUC_lexicon_tags_27 },
    { 0x7ece780a, SUC_lexicon_tags_11 },
    { 0x5bee780b, SUC_lexicon_tags_43 },
    { 0x4d6e780c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cee780f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ae7813, SUC_lexicon_tags_43 },
    { 0x5c8e7814, SUC_lexicon_tags_4 },
    { 0x620e7815, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292e7819, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecee7821, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d2e7823, SUC_lexicon_tags_37 },
    { 0x620e7823, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dce7828, SUC_lexicon_tags_30 },
    { 0x4b6e7829, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ee7832, SUC_lexicon_tags_11 },
    { 0xe32e7833, SUC_lexicon_tags_29 },
    { 0xdbce7833, SUC_lexicon_tags_27 },
    { 0x648e7834, SUC_lexicon_tags_2 },
    { 0xe42e7832, SUC_lexicon_tags_10 },
    { 0x10e7836, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190e783b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ce7841, SUC_lexicon_tags_37 },
    { 0xa42e7842, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42e7844, SUC_lexicon_tags_0 },
    { 0xc62e7845, SUC_lexicon_tags_17 },
    { 0xaf2e7846, SUC_lexicon_tags_42 },
    { 0xc02e7847, SUC_lexicon_tags_15 },
    { 0x424e7847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64e784c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210e7851, SUC_lexicon_tags_37 },
    { 0x76ae7852, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86ee7854, SUC_lexicon_tags_68 },
    { 0xb40e7855, SUC_lexicon_tags_12 },
    { 0xbc0e7856, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc8e7858, SUC_lexicon_tags_23 },
    { 0xb08e7859, SUC_lexicon_tags_27 },
    { 0x56ae7858, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7ace785c, SUC_lexicon_tags_49 },
    { 0xf50e785c, SUC_lexicon_tags_12 },
    { 0x6a4e785e, SUC_lexicon_tags_2 },
    { 0xf40e785e, SUC_lexicon_tags_15 },
    { 0x260e7860, SUC_lexicon_tags_30 },
    { 0x638e7861, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf2ae7863, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd7ce7865, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e2e7867, SUC_lexicon_tags_15 },
    { 0x60e7868, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x852e786b, SUC_lexicon_tags_47 },
    { 0x4f2e786c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7f4e786e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1c4e7870, SUC_lexicon_tags_37 },
    { 0x142e7870, SUC_lexicon_tags_11 },
    { 0x6ee7870, SUC_lexicon_tags_37 },
    { 0xd4ce7870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bce7875, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982e7879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebee787b, SUC_lexicon_tags_39 },
    { 0x5c0e787c, SUC_lexicon_tags_63 },
    { 0x4bee787d, SUC_lexicon_tags_106 },
    { 0x292e787d, SUC_lexicon_tags_1136 },
    { 0, NULL },
    { 0x8e4e7880, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa2ee7882, SUC_lexicon_tags_23 },
    { 0xc56e7882, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ee7888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x854e788a, SUC_lexicon_tags_11 },
    { 0x578e788b, SUC_lexicon_tags_8 },
    { 0x328e788b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4e7890, SUC_lexicon_tags_17 },
    { 0x7f4e7891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99ce7893, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0e7896, SUC_lexicon_tags_1 },
    { 0x4d8e7896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf46e7899, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7cae789b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x872e789d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x542e78a2, SUC_lexicon_tags_25 },
    { 0xb9ae78a2, SUC_lexicon_tags_25 },
    { 0xa00e78a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8e78a6, SUC_lexicon_tags_1 },
    { 0x44ae78a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bae78ae, SUC_lexicon_tags_17 },
    { 0xd2ce78ae, SUC_lexicon_tags_46 },
    { 0xe10e78ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4e78b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cee78b9, SUC_lexicon_tags_14 },
    { 0x2a8e78b9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ae78bd, SUC_lexicon_tags_2 },
    { 0xfa2e78be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aee78c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0e78c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fae78cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf9ae78ce, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x20e78d1, SUC_lexicon_tags_7 },
    { 0x28ae78d2, SUC_lexicon_tags_1 },
    { 0xd56e78d3, SUC_lexicon_tags_0 },
    { 0x38e78d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66e78d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf86e78d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8e78dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfd2e78de, SUC_lexicon_tags_15 },
    { 0x8aee78df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19ce78e1, SUC_lexicon_tags_63 },
    { 0xd44e78e1, SUC_lexicon_tags_17 },
    { 0x49ce78e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc04e78e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x77ce78e8, SUC_lexicon_tags_23 },
    { 0x300e78e9, SUC_lexicon_tags_7 },
    { 0xc3ae78ea, SUC_lexicon_tags_15 },
    { 0x5cee78ea, SUC_lexicon_tags_49 },
    { 0xa8ae78ec, SUC_lexicon_tags_68 },
    { 0x10e78ed, SUC_lexicon_tags_42 },
    { 0xad6e78ec, SUC_lexicon_tags_16 },
    { 0x99ee78ec, SUC_lexicon_tags_29 },
    { 0x47ce78f0, SUC_lexicon_tags_42 },
    { 0x372e78f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768e78f5, SUC_lexicon_tags_11 },
    { 0x676e78f6, SUC_lexicon_tags_0 },
    { 0xa00e78f7, SUC_lexicon_tags_15 },
    { 0xe66e78f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212e78fc, SUC_lexicon_tags_15 },
    { 0xefee78fd, SUC_lexicon_tags_17 },
    { 0x620e78fe, SUC_lexicon_tags_11 },
    { 0xc5ce78fe, SUC_lexicon_tags_14 },
    { 0xa3ee78fd, SUC_lexicon_tags_7 },
    { 0xe58e78fe, SUC_lexicon_tags_65 },
    { 0x44e7902, SUC_lexicon_tags_1 },
    { 0x708e7903, SUC_lexicon_tags_12 },
    { 0x908e7902, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2e7908, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbae790d, SUC_lexicon_tags_2 },
    { 0x988e790e, SUC_lexicon_tags_4 },
    { 0x234e790f, SUC_lexicon_tags_33 },
    { 0x50ae790f, SUC_lexicon_tags_0 },
    { 0x62ae7911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3ce7913, SUC_lexicon_tags_25 },
    { 0x892e7913, SUC_lexicon_tags_33 },
    { 0x264e7915, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8ae7917, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda2e7919, SUC_lexicon_tags_7 },
    { 0x9ae791a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78e791d, SUC_lexicon_tags_8 },
    { 0x5c0e791e, SUC_lexicon_tags_65 },
    { 0x812e791f, SUC_lexicon_tags_25 },
    { 0xe10e791f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaece7922, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xabce7925, SUC_lexicon_tags_2 },
    { 0x1f6e7926, SUC_lexicon_tags_26 },
    { 0x424e7926, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12e792a, SUC_lexicon_tags_16 },
    { 0x830e792a, SUC_lexicon_tags_61 },
    { 0x14e792b, SUC_lexicon_tags_17 },
    { 0xc4e792b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd36e792f, SUC_lexicon_tags_77 },
    { 0xe5ae7930, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04e7933, SUC_lexicon_tags_4 },
    { 0x30ae7934, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x442e7937, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ee793d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce7944, SUC_lexicon_tags_17 },
    { 0x874e7944, SUC_lexicon_tags_31 },
    { 0xd6ae7946, SUC_lexicon_tags_18 },
    { 0x1e6e7944, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x394e7949, SUC_lexicon_tags_11 },
    { 0x540e7949, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36e794d, SUC_lexicon_tags_17 },
    { 0xd28e794e, SUC_lexicon_tags_45 },
    { 0x50ae794d, SUC_lexicon_tags_25 },
    { 0x20ce794e, SUC_lexicon_tags_39 },
    { 0xa80e794f, SUC_lexicon_tags_7 },
    { 0xd16e794e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dce7955, SUC_lexicon_tags_62 },
    { 0xce6e7956, SUC_lexicon_tags_18 },
    { 0xd70e7957, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2e795e, SUC_lexicon_tags_68 },
    { 0x74ae795f, SUC_lexicon_tags_31 },
    { 0xaf6e7960, SUC_lexicon_tags_26 },
    { 0x28ae7961, SUC_lexicon_tags_1 },
    { 0x384e7962, SUC_lexicon_tags_10 },
    { 0x50e7962, SUC_lexicon_tags_25 },
    { 0x5c6e7960, SUC_lexicon_tags_100 },
    { 0xffae7960, SUC_lexicon_tags_0 },
    { 0x98ae7966, SUC_lexicon_tags_27 },
    { 0x424e7966, SUC_lexicon_tags_17 },
    { 0xa80e7968, SUC_lexicon_tags_11 },
    { 0x8b6e7969, SUC_lexicon_tags_62 },
    { 0x14ce796a, SUC_lexicon_tags_27 },
    { 0xef0e796b, SUC_lexicon_tags_37 },
    { 0x540e796c, SUC_lexicon_tags_103 },
    { 0xda0e7969, SUC_lexicon_tags_1 },
    { 0xb44e796a, SUC_lexicon_tags_47 },
    { 0xdd4e796a, SUC_lexicon_tags_2 },
    { 0xb8ce796f, SUC_lexicon_tags_28 },
    { 0x9c2e7968, SUC_lexicon_tags_0 },
    { 0xb76e7972, SUC_lexicon_tags_63 },
    { 0x58ce7972, SUC_lexicon_tags_15 },
    { 0x92ae7967, SUC_lexicon_tags_63 },
    { 0x68e796d, SUC_lexicon_tags_198 },
    { 0x5ece7976, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314e797c, SUC_lexicon_tags_37 },
    { 0x652e797c, SUC_lexicon_tags_47 },
    { 0x7aae797e, SUC_lexicon_tags_29 },
    { 0x466e797f, SUC_lexicon_tags_29 },
    { 0xdf6e7980, SUC_lexicon_tags_28 },
    { 0xe9ae797c, SUC_lexicon_tags_48 },
    { 0xd4ee7982, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96e7987, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x752e798a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4e0e798c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x418e798f, SUC_lexicon_tags_23 },
    { 0x1ce798f, SUC_lexicon_tags_68 },
    { 0xabce7991, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8e7995, SUC_lexicon_tags_8 },
    { 0xc36e7995, SUC_lexicon_tags_62 },
    { 0xc82e7997, SUC_lexicon_tags_23 },
    { 0x798e7998, SUC_lexicon_tags_45 },
    { 0x398e7996, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcece799b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6e79a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x976e79a3, SUC_lexicon_tags_9 },
    { 0x3fee79a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe00e79a6, SUC_lexicon_tags_6 },
    { 0xec0e79a7, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512e79ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0e79af, SUC_lexicon_tags_17 },
    { 0x99ee79af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x412e79b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x764e79b4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x262e79b7, SUC_lexicon_tags_54 },
    { 0xa52e79b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x894e79bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x628e79be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ece79c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33ae79c3, SUC_lexicon_tags_66 },
    { 0x3aee79c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ee79c7, SUC_lexicon_tags_146 },
    { 0x206e79c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46e79ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ce79cd, SUC_lexicon_tags_37 },
    { 0x56ee79cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ee79d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32ee79da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10ae79dc, SUC_lexicon_tags_26 },
    { 0x4f4e79dd, SUC_lexicon_tags_1 },
    { 0xa10e79de, SUC_lexicon_tags_14 },
    { 0x8f2e79df, SUC_lexicon_tags_1 },
    { 0x574e79e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe96e79e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x910e79e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ce79e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146e79ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd00e79f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ae79f5, SUC_lexicon_tags_38 },
    { 0x8bae79f5, SUC_lexicon_tags_11 },
    { 0x242e79f7, SUC_lexicon_tags_27 },
    { 0x968e79f8, SUC_lexicon_tags_34 },
    { 0x7cce79f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aee79fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x760e79ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62e7a08, SUC_lexicon_tags_12 },
    { 0xfdee7a09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x276e7a0c, SUC_lexicon_tags_7 },
    { 0x814e7a0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ee7a12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcae7a14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6e7a18, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626e7a1c, SUC_lexicon_tags_12 },
    { 0xb4ae7a1c, SUC_lexicon_tags_17 },
    { 0x7e8e7a1e, SUC_lexicon_tags_79 },
    { 0x6bce7a1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e7a25, SUC_lexicon_tags_11 },
    { 0xd84e7a25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x486e7a28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea4e7a2a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcee7a2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874e7a32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x312e7a34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6e7a3d, SUC_lexicon_tags_33 },
    { 0xe16e7a3d, SUC_lexicon_tags_37 },
    { 0x90ce7a3f, SUC_lexicon_tags_2 },
    { 0xe4e7a3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272e7a47, SUC_lexicon_tags_47 },
    { 0xe00e7a48, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8e7a50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36e7a53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0e7a56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ce7a59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4e7a5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x160e7a5f, SUC_lexicon_tags_49 },
    { 0x50ae7a60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654e7a68, SUC_lexicon_tags_9 },
    { 0xfd2e7a68, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450e7a70, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0e7a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa06e7a75, SUC_lexicon_tags_9 },
    { 0xcb2e7a75, SUC_lexicon_tags_43 },
    { 0xddee7a77, SUC_lexicon_tags_45 },
    { 0x7eae7a75, SUC_lexicon_tags_6 },
    { 0x3e6e7a79, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78e7a7d, SUC_lexicon_tags_12 },
    { 0xb1ce7a7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x756e7a80, SUC_lexicon_tags_25 },
    { 0x1e0e7a81, SUC_lexicon_tags_49 },
    { 0x48ae7a81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02e7a89, SUC_lexicon_tags_19 },
    { 0xd22e7a8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554e7a90, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6e7a95, SUC_lexicon_tags_29 },
    { 0x708e7a95, SUC_lexicon_tags_17 },
    { 0x960e7a97, SUC_lexicon_tags_14 },
    { 0x62ee7a98, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ee7aa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x826e7aa3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6e7aaa, SUC_lexicon_tags_19 },
    { 0x96ee7aab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ee7aae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4e7ab3, SUC_lexicon_tags_12 },
    { 0xfeae7ab4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4ae7ab6, SUC_lexicon_tags_69 },
    { 0x142e7ab7, SUC_lexicon_tags_37 },
    { 0x378e7ab7, SUC_lexicon_tags_12 },
    { 0xb9ee7ab9, SUC_lexicon_tags_6 },
    { 0x7dce7aba, SUC_lexicon_tags_19 },
    { 0x320e7aba, SUC_lexicon_tags_49 },
    { 0xf8ce7abc, SUC_lexicon_tags_15 },
    { 0xd6ce7abd, SUC_lexicon_tags_1 },
    { 0xa54e7abe, SUC_lexicon_tags_45 },
    { 0xa7ee7abd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862e7ac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8cae7ac9, SUC_lexicon_tags_11 },
    { 0x91ee7aca, SUC_lexicon_tags_21 },
    { 0x870e7acb, SUC_lexicon_tags_11 },
    { 0xea8e7acc, SUC_lexicon_tags_31 },
    { 0x440e7acd, SUC_lexicon_tags_29 },
    { 0xd8ce7acd, SUC_lexicon_tags_15 },
    { 0x3d2e7acb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe30e7ad1, SUC_lexicon_tags_7 },
    { 0xa56e7ad1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80e7ad4, SUC_lexicon_tags_17 },
    { 0x616e7ad5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc78e7ad7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xccce7ad9, SUC_lexicon_tags_14 },
    { 0x4fee7ad9, SUC_lexicon_tags_37 },
    { 0x3cce7adb, SUC_lexicon_tags_15 },
    { 0x706e7adc, SUC_lexicon_tags_11 },
    { 0x4eae7adc, SUC_lexicon_tags_25 },
    { 0xab8e7ade, SUC_lexicon_tags_108 },
    { 0x52ee7adf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ee7ae6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ce7aea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x444e7aed, SUC_lexicon_tags_19 },
    { 0xe50e7aee, SUC_lexicon_tags_7 },
    { 0x9ce7aef, SUC_lexicon_tags_63 },
    { 0x67ce7af0, SUC_lexicon_tags_25 },
    { 0xaf2e7af1, SUC_lexicon_tags_12 },
    { 0xf18e7aee, SUC_lexicon_tags_37 },
    { 0xde6e7af1, SUC_lexicon_tags_25 },
    { 0xf86e7af4, SUC_lexicon_tags_49 },
    { 0x902e7af5, SUC_lexicon_tags_37 },
    { 0x6dae7af4, SUC_lexicon_tags_46 },
    { 0xedce7af3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe16e7af9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ce7afe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dce7b01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa10e7b03, SUC_lexicon_tags_37 },
    { 0xd4ae7b04, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcce7b07, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aee7b0d, SUC_lexicon_tags_19 },
    { 0xeece7b0e, SUC_lexicon_tags_26 },
    { 0x10ae7b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76e7b13, SUC_lexicon_tags_19 },
    { 0x40ae7b13, SUC_lexicon_tags_25 },
    { 0x72e7b15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fce7b18, SUC_lexicon_tags_14 },
    { 0xee4e7b19, SUC_lexicon_tags_16 },
    { 0xfae7b1a, SUC_lexicon_tags_12 },
    { 0x890e7b1b, SUC_lexicon_tags_37 },
    { 0xcf8e7b1c, SUC_lexicon_tags_15 },
    { 0xab0e7b19, SUC_lexicon_tags_17 },
    { 0xc9ce7b1e, SUC_lexicon_tags_17 },
    { 0x35ce7b1f, SUC_lexicon_tags_37 },
    { 0x910e7b1e, SUC_lexicon_tags_61 },
    { 0x646e7b1f, SUC_lexicon_tags_6 },
    { 0x63ce7b1f, SUC_lexicon_tags_37 },
    { 0xac2e7b1e, SUC_lexicon_tags_38 },
    { 0xbaae7b24, SUC_lexicon_tags_2 },
    { 0x622e7b1f, SUC_lexicon_tags_29 },
    { 0x814e7b1f, SUC_lexicon_tags_27 },
    { 0xbeee7b25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4dee7b29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ae7b2c, SUC_lexicon_tags_29 },
    { 0xf4ee7b2d, SUC_lexicon_tags_141 },
    { 0x526e7b2e, SUC_lexicon_tags_0 },
    { 0x3dae7b2f, SUC_lexicon_tags_1 },
    { 0x5cee7b30, SUC_lexicon_tags_37 },
    { 0x1aae7b2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ee7b34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06e7b37, SUC_lexicon_tags_14 },
    { 0x516e7b38, SUC_lexicon_tags_31 },
    { 0xc40e7b39, SUC_lexicon_tags_17 },
    { 0xbe6e7b3a, SUC_lexicon_tags_82 },
    { 0x7cee7b3b, SUC_lexicon_tags_84 },
    { 0xc4ae7b3c, SUC_lexicon_tags_21 },
    { 0x582e7b3c, SUC_lexicon_tags_7 },
    { 0x5bae7b3d, SUC_lexicon_tags_43 },
    { 0x290e7b3f, SUC_lexicon_tags_16 },
    { 0x8d6e7b40, SUC_lexicon_tags_37 },
    { 0x852e7b3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ce7b44, SUC_lexicon_tags_11 },
    { 0x98e7b44, SUC_lexicon_tags_7 },
    { 0xe98e7b46, SUC_lexicon_tags_38 },
    { 0x450e7b45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e7b4d, SUC_lexicon_tags_17 },
    { 0xd08e7b4e, SUC_lexicon_tags_19 },
    { 0xb3ce7b4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8e7b52, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8e7b58, SUC_lexicon_tags_11 },
    { 0xfe0e7b58, SUC_lexicon_tags_27 },
    { 0x3a6e7b5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x214e7b5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eae7b5f, SUC_lexicon_tags_9 },
    { 0x414e7b60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4e7b63, SUC_lexicon_tags_13 },
    { 0x8dee7b64, SUC_lexicon_tags_15 },
    { 0xe4ae7b64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0e7b6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe66e7b6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54e7b72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e7b79, SUC_lexicon_tags_29 },
    { 0x5e8e7b79, SUC_lexicon_tags_15 },
    { 0x44ce7b7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ee7b83, SUC_lexicon_tags_47 },
    { 0xf54e7b84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e7b88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344e7b8d, SUC_lexicon_tags_13 },
    { 0x6ee7b8e, SUC_lexicon_tags_29 },
    { 0xd22e7b8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x412e7b91, SUC_lexicon_tags_2 },
    { 0x2fce7b92, SUC_lexicon_tags_13 },
    { 0xb84e7b92, SUC_lexicon_tags_1 },
    { 0x7cce7b93, SUC_lexicon_tags_37 },
    { 0xc1ce7b95, SUC_lexicon_tags_65 },
    { 0x2f8e7b96, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4ece7b98, SUC_lexicon_tags_224 },
    { 0x5e6e7b99, SUC_lexicon_tags_17 },
    { 0xff6e7b98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4e7b9e, SUC_lexicon_tags_1 },
    { 0x45ce7b9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacee7ba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a4e7ba5, SUC_lexicon_tags_1 },
    { 0x3c8e7ba6, SUC_lexicon_tags_37 },
    { 0x1f0e7ba7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x420e7baa, SUC_lexicon_tags_65 },
    { 0x792e7bab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x150e7bae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8e7bb3, SUC_lexicon_tags_17 },
    { 0x51ce7bb4, SUC_lexicon_tags_2 },
    { 0xd92e7bb4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bce7bba, SUC_lexicon_tags_42 },
    { 0xd92e7bba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x984e7bbd, SUC_lexicon_tags_23 },
    { 0x970e7bbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ae7bc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfee7bc8, SUC_lexicon_tags_37 },
    { 0x1ce7bc8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2e7bce, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892e7bd2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fee7bd5, SUC_lexicon_tags_17 },
    { 0x520e7bd5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8e7bd9, SUC_lexicon_tags_16 },
    { 0x432e7bda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb7ee7bdc, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x658e7bdf, SUC_lexicon_tags_7 },
    { 0xbe4e7be0, SUC_lexicon_tags_12 },
    { 0x40e7be0, SUC_lexicon_tags_33 },
    { 0x718e7be2, SUC_lexicon_tags_11 },
    { 0xfdee7be3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bee7be6, SUC_lexicon_tags_7 },
    { 0x726e7be7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7bed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x486e7bf0, SUC_lexicon_tags_25 },
    { 0x702e7bf1, SUC_lexicon_tags_39 },
    { 0x150e7bf1, SUC_lexicon_tags_12 },
    { 0xacce7bf3, SUC_lexicon_tags_12 },
    { 0xee6e7bf3, SUC_lexicon_tags_11 },
    { 0x2bae7bf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6ee7bf7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00e7bfa, SUC_lexicon_tags_56 },
    { 0xc20e7bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2e7c01, SUC_lexicon_tags_60 },
    { 0x4b2e7c01, SUC_lexicon_tags_60 },
    { 0x410e7c03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eee7c08, SUC_lexicon_tags_17 },
    { 0xf86e7c09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xacae7c0b, SUC_lexicon_tags_87 },
    { 0x164e7c0c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x30e7c0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174e7c12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf86e7c14, SUC_lexicon_tags_63 },
    { 0xadce7c15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6e7c18, SUC_lexicon_tags_15 },
    { 0xdeee7c19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf20e7c1b, SUC_lexicon_tags_25 },
    { 0xe58e7c1c, SUC_lexicon_tags_4 },
    { 0x27ce7c1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2e7c27, SUC_lexicon_tags_33 },
    { 0x42ce7c28, SUC_lexicon_tags_11 },
    { 0xe18e7c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90e7c2d, SUC_lexicon_tags_8 },
    { 0x99ae7c2e, SUC_lexicon_tags_12 },
    { 0x74e7c2f, SUC_lexicon_tags_28 },
    { 0xd40e7c2d, SUC_lexicon_tags_87 },
    { 0x8e2e7c2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe5ce7c33, SUC_lexicon_tags_37 },
    { 0x406e7c34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9ae7c36, SUC_lexicon_tags_33 },
    { 0xcb2e7c37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfece7c3b, SUC_lexicon_tags_8 },
    { 0xfb0e7c3c, SUC_lexicon_tags_14 },
    { 0xf2e7c3d, SUC_lexicon_tags_47 },
    { 0xd6ae7c3e, SUC_lexicon_tags_37 },
    { 0x62ae7c3f, SUC_lexicon_tags_46 },
    { 0xc6ae7c3f, SUC_lexicon_tags_7 },
    { 0xa5ce7c3e, SUC_lexicon_tags_34 },
    { 0x1bee7c42, SUC_lexicon_tags_49 },
    { 0xb64e7c3d, SUC_lexicon_tags_22 },
    { 0xcb2e7c44, SUC_lexicon_tags_37 },
    { 0x590e7c45, SUC_lexicon_tags_15 },
    { 0x614e7c46, SUC_lexicon_tags_31 },
    { 0x190e7c42, SUC_lexicon_tags_12 },
    { 0xe02e7c48, SUC_lexicon_tags_4 },
    { 0x36ce7c48, SUC_lexicon_tags_1 },
    { 0x28ee7c48, SUC_lexicon_tags_29 },
    { 0x864e7c4b, SUC_lexicon_tags_2 },
    { 0xe9ae7c4b, SUC_lexicon_tags_21 },
    { 0x62ee7c48, SUC_lexicon_tags_80 },
    { 0x9eae7c4e, SUC_lexicon_tags_48 },
    { 0xa06e7c4f, SUC_lexicon_tags_29 },
    { 0xfb0e7c4f, SUC_lexicon_tags_9 },
    { 0x8cee7c51, SUC_lexicon_tags_12 },
    { 0xf46e7c4c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x68ee7c54, SUC_lexicon_tags_9 },
    { 0x302e7c54, SUC_lexicon_tags_61 },
    { 0x434e7c55, SUC_lexicon_tags_11 },
    { 0xb38e7c57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4e7c5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72e7c60, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670e7c6b, SUC_lexicon_tags_0 },
    { 0x508e7c6c, SUC_lexicon_tags_11 },
    { 0xb76e7c6d, SUC_lexicon_tags_11 },
    { 0x936e7c6d, SUC_lexicon_tags_246 },
    { 0x974e7c6f, SUC_lexicon_tags_19 },
    { 0xa1ae7c6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4e7c78, SUC_lexicon_tags_23 },
    { 0xebae7c78, SUC_lexicon_tags_14 },
    { 0x724e7c78, SUC_lexicon_tags_37 },
    { 0x892e7c7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd0e7c7d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x32ee7c7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aae7c82, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46e7c86, SUC_lexicon_tags_95 },
    { 0xab6e7c86, SUC_lexicon_tags_31 },
    { 0x69ce7c88, SUC_lexicon_tags_25 },
    { 0xe12e7c86, SUC_lexicon_tags_12 },
    { 0x46ee7c88, SUC_lexicon_tags_1 },
    { 0x608e7c8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x912e7c8d, SUC_lexicon_tags_32 },
    { 0x222e7c8d, SUC_lexicon_tags_19 },
    { 0x274e7c8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x186e7c92, SUC_lexicon_tags_12 },
    { 0xcb8e7c93, SUC_lexicon_tags_37 },
    { 0xd3ee7c94, SUC_lexicon_tags_45 },
    { 0xbece7c95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe14e7c97, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8e7c9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9ee7c9c, SUC_lexicon_tags_15 },
    { 0xe1ce7c9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2e7c9f, SUC_lexicon_tags_1 },
    { 0xdb2e7ca0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ce7ca5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ae7cab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0e7cb3, SUC_lexicon_tags_4 },
    { 0xab4e7cb3, SUC_lexicon_tags_35 },
    { 0x980e7cb5, SUC_lexicon_tags_45 },
    { 0x49ae7cb5, SUC_lexicon_tags_47 },
    { 0x55ae7cb3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x20e7cb9, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0xface7cbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8e7cc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22e7cc3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80e7cc6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb72e7cc8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52e7ccb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x432e7cce, SUC_lexicon_tags_2 },
    { 0x486e7cce, SUC_lexicon_tags_8 },
    { 0xba6e7cd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x298e7cd2, SUC_lexicon_tags_15 },
    { 0x912e7cd2, SUC_lexicon_tags_2 },
    { 0x1f0e7cd4, SUC_lexicon_tags_25 },
    { 0xd80e7cd4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2e7cdc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa8ae7cde, SUC_lexicon_tags_0 },
    { 0xe94e7cdf, SUC_lexicon_tags_8 },
    { 0x432e7cde, SUC_lexicon_tags_37 },
    { 0xbf2e7ce1, SUC_lexicon_tags_25 },
    { 0x330e7ce2, SUC_lexicon_tags_7 },
    { 0x22ae7ce1, SUC_lexicon_tags_25 },
    { 0xf9ee7ce2, SUC_lexicon_tags_37 },
    { 0xdd6e7ce2, SUC_lexicon_tags_1 },
    { 0x30ae7ce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ee7ce9, SUC_lexicon_tags_45 },
    { 0x99ee7cea, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x82ce7cec, SUC_lexicon_tags_31 },
    { 0x67ae7ced, SUC_lexicon_tags_14 },
    { 0x550e7cee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ee7cf1, SUC_lexicon_tags_47 },
    { 0x9f6e7cf2, SUC_lexicon_tags_29 },
    { 0x8b4e7cf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d0e7cf5, SUC_lexicon_tags_15 },
    { 0x9b8e7cf6, SUC_lexicon_tags_7 },
    { 0xd74e7cf7, SUC_lexicon_tags_17 },
    { 0xad4e7cf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4e7cfc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ee7d03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x420e7d05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe78e7d08, SUC_lexicon_tags_83 },
    { 0x3a0e7d09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x630e7d0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ce7d0f, SUC_lexicon_tags_2 },
    { 0xf8ee7d10, SUC_lexicon_tags_63 },
    { 0xa24e7d11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bce7d16, SUC_lexicon_tags_47 },
    { 0x7ee7d16, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd02e7d19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918e7d1f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xddee7d21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2e7d24, SUC_lexicon_tags_637 },
    { 0, NULL },
    { 0xaaae7d26, SUC_lexicon_tags_83 },
    { 0x76ae7d26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40e7d2a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64e7d2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x804e7d31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2eae7d33, SUC_lexicon_tags_29 },
    { 0xca4e7d34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734e7d40, SUC_lexicon_tags_87 },
    { 0xddee7d40, SUC_lexicon_tags_7 },
    { 0xbae7d42, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362e7d52, SUC_lexicon_tags_45 },
    { 0x896e7d53, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe24e7d55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ee7d5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb34e7d5d, SUC_lexicon_tags_9 },
    { 0x830e7d5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ce7d63, SUC_lexicon_tags_42 },
    { 0xe4e7d63, SUC_lexicon_tags_16 },
    { 0x126e7d65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ee7d68, SUC_lexicon_tags_6 },
    { 0xeace7d69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26e7d75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadce7d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536e7d86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e7d89, SUC_lexicon_tags_33 },
    { 0xceae7d8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4e7d8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x368e7d8f, SUC_lexicon_tags_1 },
    { 0x50ae7d90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1aae7d92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1cee7d94, SUC_lexicon_tags_15 },
    { 0x40ce7d95, SUC_lexicon_tags_45 },
    { 0x942e7d95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7ee7d98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562e7da2, SUC_lexicon_tags_11 },
    { 0xa32e7da2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dee7da7, SUC_lexicon_tags_33 },
    { 0xfb0e7da8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ce7dae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb8e7db0, SUC_lexicon_tags_47 },
    { 0x5a0e7db1, SUC_lexicon_tags_79 },
    { 0x820e7db0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x124e7db4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xface7dbb, SUC_lexicon_tags_49 },
    { 0xbae7dbc, SUC_lexicon_tags_27 },
    { 0xf4ae7dbb, SUC_lexicon_tags_8 },
    { 0x5d0e7dbb, SUC_lexicon_tags_28 },
    { 0x9e4e7dbf, SUC_lexicon_tags_46 },
    { 0xb22e7dc0, SUC_lexicon_tags_87 },
    { 0x160e7dc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x690e7dc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ae7dca, SUC_lexicon_tags_21 },
    { 0xe06e7dcb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x894e7dcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ee7dd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ee7dd4, SUC_lexicon_tags_31 },
    { 0xface7dd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xebce7dd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26e7dda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4e7de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546e7dea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe2ce7dec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54e7df0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ce7df4, SUC_lexicon_tags_2 },
    { 0x808e7df4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce6e7df7, SUC_lexicon_tags_62 },
    { 0x914e7df8, SUC_lexicon_tags_8 },
    { 0x278e7df7, SUC_lexicon_tags_43 },
    { 0xe64e7df8, SUC_lexicon_tags_47 },
    { 0x7fce7df7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ce7dfe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4c0e7e00, SUC_lexicon_tags_11 },
    { 0xe3ee7e00, SUC_lexicon_tags_8 },
    { 0xed2e7e02, SUC_lexicon_tags_11 },
    { 0x8f6e7e03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffae7e05, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x532e7e07, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ee7e0f, SUC_lexicon_tags_15 },
    { 0xf48e7e10, SUC_lexicon_tags_11 },
    { 0x9ce7e11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd8ae7e13, SUC_lexicon_tags_65 },
    { 0xc4ae7e13, SUC_lexicon_tags_32 },
    { 0xdbae7e15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc6e7e17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfce7e19, SUC_lexicon_tags_7 },
    { 0x69ee7e1a, SUC_lexicon_tags_17 },
    { 0x2b8e7e1b, SUC_lexicon_tags_11 },
    { 0xaaae7e1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0e7e1f, SUC_lexicon_tags_38 },
    { 0x4e0e7e20, SUC_lexicon_tags_17 },
    { 0xd64e7e21, SUC_lexicon_tags_7 },
    { 0x392e7e22, SUC_lexicon_tags_56 },
    { 0xd60e7e21, SUC_lexicon_tags_27 },
    { 0x330e7e24, SUC_lexicon_tags_43 },
    { 0x622e7e24, SUC_lexicon_tags_17 },
    { 0xc54e7e26, SUC_lexicon_tags_31 },
    { 0x5c2e7e21, SUC_lexicon_tags_16 },
    { 0xcce7e25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd6e7e2a, SUC_lexicon_tags_15 },
    { 0xaf6e7e2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228e7e2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae8e7e31, SUC_lexicon_tags_84 },
    { 0x462e7e31, SUC_lexicon_tags_8 },
    { 0x766e7e33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8e7e37, SUC_lexicon_tags_17 },
    { 0xc9ae7e38, SUC_lexicon_tags_38 },
    { 0x8b8e7e39, SUC_lexicon_tags_27 },
    { 0xc0ee7e3a, SUC_lexicon_tags_12 },
    { 0x302e7e38, SUC_lexicon_tags_81 },
    { 0x564e7e3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x160e7e3e, SUC_lexicon_tags_8 },
    { 0x2e8e7e3f, SUC_lexicon_tags_25 },
    { 0xf6ae7e3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x326e7e42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x850e7e45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ce7e4e, SUC_lexicon_tags_12 },
    { 0xd74e7e4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ce7e53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8ce7e55, SUC_lexicon_tags_63 },
    { 0x77ee7e56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cee7e5b, SUC_lexicon_tags_12 },
    { 0xc0ee7e5c, SUC_lexicon_tags_55 },
    { 0xd1ee7e5d, SUC_lexicon_tags_25 },
    { 0x7d4e7e5e, SUC_lexicon_tags_32 },
    { 0xf32e7e5f, SUC_lexicon_tags_63 },
    { 0x706e7e60, SUC_lexicon_tags_9 },
    { 0x7aee7e5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e8e7e63, SUC_lexicon_tags_46 },
    { 0x31ae7e63, SUC_lexicon_tags_34 },
    { 0xde8e7e65, SUC_lexicon_tags_60 },
    { 0x5d8e7e66, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492e7e6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20e7e71, SUC_lexicon_tags_547 },
    { 0, NULL },
    { 0x57ee7e73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6e7e77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936e7e80, SUC_lexicon_tags_33 },
    { 0x26ae7e80, SUC_lexicon_tags_11 },
    { 0x61ae7e82, SUC_lexicon_tags_11 },
    { 0x63ae7e83, SUC_lexicon_tags_23 },
    { 0xc68e7e83, SUC_lexicon_tags_18 },
    { 0xe38e7e84, SUC_lexicon_tags_0 },
    { 0x1fae7e83, SUC_lexicon_tags_107 },
    { 0xb24e7e87, SUC_lexicon_tags_38 },
    { 0x634e7e87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e6e7e8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30e7e90, SUC_lexicon_tags_25 },
    { 0xd58e7e90, SUC_lexicon_tags_7 },
    { 0x98e7e92, SUC_lexicon_tags_37 },
    { 0x974e7e92, SUC_lexicon_tags_37 },
    { 0x542e7e94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x856e7e96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ce7e99, SUC_lexicon_tags_15 },
    { 0x5f2e7e9a, SUC_lexicon_tags_8 },
    { 0xfbae7e9b, SUC_lexicon_tags_14 },
    { 0xf2ae7e9b, SUC_lexicon_tags_43 },
    { 0x3cee7e9d, SUC_lexicon_tags_17 },
    { 0x28ae7e9e, SUC_lexicon_tags_47 },
    { 0x36ce7e99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddee7ea3, SUC_lexicon_tags_14 },
    { 0x60ee7ea4, SUC_lexicon_tags_33 },
    { 0x4bce7ea5, SUC_lexicon_tags_46 },
    { 0xd8ce7ea4, SUC_lexicon_tags_48 },
    { 0x8eae7ea3, SUC_lexicon_tags_33 },
    { 0xefee7ea3, SUC_lexicon_tags_11 },
    { 0x200e7ea9, SUC_lexicon_tags_99 },
    { 0xe84e7eaa, SUC_lexicon_tags_38 },
    { 0xff0e7eaa, SUC_lexicon_tags_35 },
    { 0x728e7eaa, SUC_lexicon_tags_35 },
    { 0x692e7eac, SUC_lexicon_tags_1 },
    { 0x1a0e7ea6, SUC_lexicon_tags_25 },
    { 0xd48e7eaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0e7eb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x726e7eb7, SUC_lexicon_tags_47 },
    { 0xbe8e7eb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4e7ec0, SUC_lexicon_tags_1 },
    { 0xc84e7ec0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10ee7ec3, SUC_lexicon_tags_2 },
    { 0x6aae7ec4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76e7ec8, SUC_lexicon_tags_14 },
    { 0xa0ce7ec8, SUC_lexicon_tags_25 },
    { 0x6e8e7eca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dce7ed1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x56e7ed3, SUC_lexicon_tags_15 },
    { 0x8d0e7ed3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbae7ed7, SUC_lexicon_tags_87 },
    { 0xd8ae7ed8, SUC_lexicon_tags_9 },
    { 0xb7ce7ed9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ae7edf, SUC_lexicon_tags_17 },
    { 0x5cce7ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x242e7ee2, SUC_lexicon_tags_11 },
    { 0xe64e7ee3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bce7eed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbee7eef, SUC_lexicon_tags_38 },
    { 0x27ee7ef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb68e7ef2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80e7ef5, SUC_lexicon_tags_37 },
    { 0x3bae7ef5, SUC_lexicon_tags_19 },
    { 0xe2ae7ef6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x106e7ef9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6f4e7efb, SUC_lexicon_tags_8 },
    { 0xee7efb, SUC_lexicon_tags_43 },
    { 0x16e7efc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70e7f00, SUC_lexicon_tags_38 },
    { 0xbe8e7f00, SUC_lexicon_tags_12 },
    { 0x914e7f02, SUC_lexicon_tags_17 },
    { 0x5a8e7f03, SUC_lexicon_tags_23 },
    { 0xfeae7f04, SUC_lexicon_tags_1 },
    { 0xa3ae7f04, SUC_lexicon_tags_15 },
    { 0x2e7f00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab6e7f08, SUC_lexicon_tags_25 },
    { 0x76e7f08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714e7f0d, SUC_lexicon_tags_37 },
    { 0x8c4e7f0e, SUC_lexicon_tags_30 },
    { 0xc6e7f0f, SUC_lexicon_tags_66 },
    { 0xe28e7f10, SUC_lexicon_tags_69 },
    { 0x39ee7f0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x500e7f14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8e7f19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ae7f20, SUC_lexicon_tags_25 },
    { 0x78ce7f21, SUC_lexicon_tags_7 },
    { 0xc18e7f20, SUC_lexicon_tags_37 },
    { 0x614e7f20, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8e7f26, SUC_lexicon_tags_2 },
    { 0x742e7f27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0ce7f29, SUC_lexicon_tags_14 },
    { 0x356e7f2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xceae7f2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xab2e7f2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6e7f33, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x216e7f36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x830e7f38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4e7f46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ae7f49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x970e7f4b, SUC_lexicon_tags_1 },
    { 0x990e7f4c, SUC_lexicon_tags_43 },
    { 0xd6ae7f4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdaee7f4f, SUC_lexicon_tags_47 },
    { 0xc38e7f4f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8e7f56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ee7f59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9ae7f5b, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x7aee7f5d, SUC_lexicon_tags_63 },
    { 0xbc8e7f5d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ce7f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc80e7f66, SUC_lexicon_tags_31 },
    { 0x87ae7f67, SUC_lexicon_tags_33 },
    { 0xc10e7f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ce7f6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe04e7f6f, SUC_lexicon_tags_2 },
    { 0x524e7f70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30e7f74, SUC_lexicon_tags_13 },
    { 0x8e7f74, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06e7f78, SUC_lexicon_tags_45 },
    { 0x160e7f79, SUC_lexicon_tags_39 },
    { 0x170e7f7a, SUC_lexicon_tags_6 },
    { 0x430e7f7b, SUC_lexicon_tags_17 },
    { 0xbd6e7f7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ee7f84, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00e7f88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ee7f8b, SUC_lexicon_tags_17 },
    { 0x310e7f8c, SUC_lexicon_tags_12 },
    { 0x978e7f8d, SUC_lexicon_tags_23 },
    { 0x8ce7f8e, SUC_lexicon_tags_86 },
    { 0xba2e7f8c, SUC_lexicon_tags_2 },
    { 0x2ce7f90, SUC_lexicon_tags_7 },
    { 0xce7f91, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8e7f97, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf4e7f99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb6e7f9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a6e7f9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72ee7f9f, SUC_lexicon_tags_37 },
    { 0x14e7f9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacce7fa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcee7fa8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x8a2e7faa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ae7fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812e7fbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fae7fbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6e7fc0, SUC_lexicon_tags_28 },
    { 0x520e7fc0, SUC_lexicon_tags_63 },
    { 0x650e7fc0, SUC_lexicon_tags_15 },
    { 0x7ce7fc3, SUC_lexicon_tags_11 },
    { 0x98ce7fc4, SUC_lexicon_tags_17 },
    { 0x842e7fc3, SUC_lexicon_tags_19 },
    { 0xc26e7fc6, SUC_lexicon_tags_17 },
    { 0x8fce7fc6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aee7fca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ae7fd5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x936e7fd7, SUC_lexicon_tags_2 },
    { 0x9f4e7fd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf02e7fda, SUC_lexicon_tags_1 },
    { 0xbcae7fdb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ce7feb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ae7fef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x176e7ff2, SUC_lexicon_tags_47 },
    { 0xc40e7ff2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6f2e7ff5, SUC_lexicon_tags_37 },
    { 0xf28e7ff5, SUC_lexicon_tags_29 },
    { 0x444e7ff7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ee7ffa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0e8002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52e8007, SUC_lexicon_tags_7 },
    { 0xf18e8008, SUC_lexicon_tags_12 },
    { 0xd44e8008, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ce8011, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7bee8013, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466e801c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb50e801e, SUC_lexicon_tags_12 },
    { 0xe0ee801f, SUC_lexicon_tags_34 },
    { 0x1eae8020, SUC_lexicon_tags_11 },
    { 0xc82e8020, SUC_lexicon_tags_14 },
    { 0xa0e8022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x516e8024, SUC_lexicon_tags_47 },
    { 0x340e8024, SUC_lexicon_tags_6 },
    { 0xd68e8026, SUC_lexicon_tags_1 },
    { 0x72ce8027, SUC_lexicon_tags_12 },
    { 0xef0e8028, SUC_lexicon_tags_17 },
    { 0x372e8028, SUC_lexicon_tags_17 },
    { 0x9b4e8027, SUC_lexicon_tags_12 },
    { 0x5d4e8029, SUC_lexicon_tags_0 },
    { 0xc66e802c, SUC_lexicon_tags_60 },
    { 0x420e802d, SUC_lexicon_tags_43 },
    { 0xa70e802e, SUC_lexicon_tags_11 },
    { 0xc96e802c, SUC_lexicon_tags_12 },
    { 0x8cee802d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb06e8032, SUC_lexicon_tags_2 },
    { 0xd38e8033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecae8037, SUC_lexicon_tags_47 },
    { 0x98e8038, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xac2e803a, SUC_lexicon_tags_2 },
    { 0x802e803a, SUC_lexicon_tags_11 },
    { 0x770e803c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x370e803f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190e8045, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0e8048, SUC_lexicon_tags_15 },
    { 0xc02e8049, SUC_lexicon_tags_0 },
    { 0xc1ce804a, SUC_lexicon_tags_30 },
    { 0x362e804b, SUC_lexicon_tags_14 },
    { 0x436e804c, SUC_lexicon_tags_17 },
    { 0x376e8049, SUC_lexicon_tags_1 },
    { 0x4f0e804c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62ce8050, SUC_lexicon_tags_26 },
    { 0x98ae8050, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ce8056, SUC_lexicon_tags_8 },
    { 0xf4ae8057, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcd4e8059, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204e8063, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84e8066, SUC_lexicon_tags_12 },
    { 0xf6ae8067, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6e806a, SUC_lexicon_tags_25 },
    { 0x150e806b, SUC_lexicon_tags_12 },
    { 0x6eae806a, SUC_lexicon_tags_15 },
    { 0xc4ce806d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4e8070, SUC_lexicon_tags_17 },
    { 0xb56e8071, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ae8074, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8e807a, SUC_lexicon_tags_12 },
    { 0xa74e807b, SUC_lexicon_tags_9 },
    { 0x770e807c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b4e807e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bce8084, SUC_lexicon_tags_28 },
    { 0x38ee8084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e2e8087, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518e808c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02e8091, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ee8099, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8e809d, SUC_lexicon_tags_45 },
    { 0x348e809e, SUC_lexicon_tags_47 },
    { 0xa16e809d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304e80a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x854e80aa, SUC_lexicon_tags_43 },
    { 0x2c8e80ab, SUC_lexicon_tags_19 },
    { 0x4e6e80ac, SUC_lexicon_tags_7 },
    { 0x42ce80aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc86e80af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x20ae80b2, SUC_lexicon_tags_15 },
    { 0xa34e80b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12e80b7, SUC_lexicon_tags_2 },
    { 0xdf2e80b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae80ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08e80c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98e80c4, SUC_lexicon_tags_68 },
    { 0xdf8e80c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe02e80c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2e80ca, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x472e80cd, SUC_lexicon_tags_179 },
    { 0x5fee80cd, SUC_lexicon_tags_17 },
    { 0x140e80cf, SUC_lexicon_tags_23 },
    { 0x1b2e80d0, SUC_lexicon_tags_43 },
    { 0x2cee80ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad4e80d3, SUC_lexicon_tags_43 },
    { 0x62ae80d4, SUC_lexicon_tags_25 },
    { 0xadae80d3, SUC_lexicon_tags_23 },
    { 0x856e80d5, SUC_lexicon_tags_7 },
    { 0x9fae80d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50e80d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ae80df, SUC_lexicon_tags_34 },
    { 0xc3ce80e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4e80eb, SUC_lexicon_tags_27 },
    { 0xccce80ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x798e80ee, SUC_lexicon_tags_15 },
    { 0x7c4e80ef, SUC_lexicon_tags_28 },
    { 0x114e80f0, SUC_lexicon_tags_14 },
    { 0x542e80f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330e80f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ae80fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ee8102, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x278e8104, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x648e8106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb54e8108, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x474e810b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74e8110, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x90ae8112, SUC_lexicon_tags_21 },
    { 0xe72e8112, SUC_lexicon_tags_11 },
    { 0xf82e8113, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x12e8116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaee8119, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ce811c, SUC_lexicon_tags_55 },
    { 0x7ae811d, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652e8121, SUC_lexicon_tags_0 },
    { 0xa4ee8121, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2e8130, SUC_lexicon_tags_68 },
    { 0x364e8130, SUC_lexicon_tags_39 },
    { 0xbb2e8130, SUC_lexicon_tags_10 },
    { 0x432e8131, SUC_lexicon_tags_12 },
    { 0x6f2e8134, SUC_lexicon_tags_2 },
    { 0xa7ce8135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x322e8138, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x132e813b, SUC_lexicon_tags_1 },
    { 0x8fce813b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78e8143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32e8147, SUC_lexicon_tags_15 },
    { 0x4dee8147, SUC_lexicon_tags_11 },
    { 0xc3ee8149, SUC_lexicon_tags_31 },
    { 0x158e8147, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88e814d, SUC_lexicon_tags_12 },
    { 0x830e814d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaece8151, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ce8157, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ce815d, SUC_lexicon_tags_48 },
    { 0xe06e815d, SUC_lexicon_tags_7 },
    { 0xf78e815e, SUC_lexicon_tags_38 },
    { 0xfd6e8160, SUC_lexicon_tags_12 },
    { 0x89ae8161, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ee8165, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6ece8167, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44e816a, SUC_lexicon_tags_17 },
    { 0x86ae816b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ae816e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce8171, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x158e8174, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ce8178, SUC_lexicon_tags_17 },
    { 0x5e0e8179, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2e817c, SUC_lexicon_tags_25 },
    { 0xd04e817d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ee8181, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f8e8183, SUC_lexicon_tags_21 },
    { 0x672e8183, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce8189, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x302e818c, SUC_lexicon_tags_7 },
    { 0x4d6e818d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36e8192, SUC_lexicon_tags_30 },
    { 0x3dae8192, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376e819a, SUC_lexicon_tags_29 },
    { 0xc6ce819b, SUC_lexicon_tags_63 },
    { 0xbc0e819c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x63ce819e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd78e81a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e81a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28e81a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x41ae81a9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670e81b6, SUC_lexicon_tags_27 },
    { 0x92e81b7, SUC_lexicon_tags_37 },
    { 0x958e81b6, SUC_lexicon_tags_17 },
    { 0x7b2e81b9, SUC_lexicon_tags_15 },
    { 0xe7ce81b9, SUC_lexicon_tags_68 },
    { 0x9b0e81bb, SUC_lexicon_tags_32 },
    { 0x740e81bc, SUC_lexicon_tags_37 },
    { 0x430e81b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x634e81bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3ce81c1, SUC_lexicon_tags_31 },
    { 0x3bae81c1, SUC_lexicon_tags_10 },
    { 0xae81c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe6ae81c5, SUC_lexicon_tags_49 },
    { 0x206e81c6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x19ce81c8, SUC_lexicon_tags_53 },
    { 0x184e81c8, SUC_lexicon_tags_2 },
    { 0x342e81ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0e81d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e81d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4e81d7, SUC_lexicon_tags_37 },
    { 0x14ee81d8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdb6e81da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6e81dd, SUC_lexicon_tags_37 },
    { 0xad8e81de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xce0e81e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ae81e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64e81e9, SUC_lexicon_tags_61 },
    { 0x6c0e81ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffee81ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ae81f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ce81f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4ce81f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ee81fb, SUC_lexicon_tags_41 },
    { 0xd30e81fc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x354e81fe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x84ae8200, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd40e8202, SUC_lexicon_tags_7 },
    { 0x696e8203, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x266e8206, SUC_lexicon_tags_1 },
    { 0x926e8206, SUC_lexicon_tags_37 },
    { 0x51ee8206, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeee820b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8e8210, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eae8213, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x896e8216, SUC_lexicon_tags_11 },
    { 0xb72e8217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86e821e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbae8221, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae8226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108e822a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18e822f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2e8232, SUC_lexicon_tags_47 },
    { 0x2f4e8233, SUC_lexicon_tags_25 },
    { 0x47ce8233, SUC_lexicon_tags_1579 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744e8238, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0e823b, SUC_lexicon_tags_49 },
    { 0xe02e823c, SUC_lexicon_tags_0 },
    { 0x810e823c, SUC_lexicon_tags_114 },
    { 0xc6ce823e, SUC_lexicon_tags_2 },
    { 0xc32e823f, SUC_lexicon_tags_11 },
    { 0x558e8240, SUC_lexicon_tags_14 },
    { 0x186e8240, SUC_lexicon_tags_38 },
    { 0x330e8242, SUC_lexicon_tags_14 },
    { 0xedee823e, SUC_lexicon_tags_12 },
    { 0x476e823f, SUC_lexicon_tags_2 },
    { 0xc06e8245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154e824c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeee8252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6e825d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38e8260, SUC_lexicon_tags_11 },
    { 0x57ee8260, SUC_lexicon_tags_2 },
    { 0xd9ee8260, SUC_lexicon_tags_45 },
    { 0xadee8263, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x92ae8265, SUC_lexicon_tags_32 },
    { 0x94e8266, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4e826a, SUC_lexicon_tags_46 },
    { 0xd4ae826a, SUC_lexicon_tags_11 },
    { 0x46e826c, SUC_lexicon_tags_12 },
    { 0xd0ee826c, SUC_lexicon_tags_2 },
    { 0x808e826d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2e8276, SUC_lexicon_tags_15 },
    { 0x86ae8276, SUC_lexicon_tags_11 },
    { 0xea8e8278, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32ae827a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ee8280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cee8282, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ee8285, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2e8289, SUC_lexicon_tags_49 },
    { 0xae0e8289, SUC_lexicon_tags_380 },
    { 0, NULL },
    { 0xfb2e828c, SUC_lexicon_tags_11 },
    { 0x97ae828d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888e8291, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ae8296, SUC_lexicon_tags_27 },
    { 0x398e8296, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeee829b, SUC_lexicon_tags_28 },
    { 0x398e829c, SUC_lexicon_tags_11 },
    { 0xc3ee829d, SUC_lexicon_tags_37 },
    { 0x76e829e, SUC_lexicon_tags_63 },
    { 0xe54e829f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfece82a1, SUC_lexicon_tags_39 },
    { 0x914e82a2, SUC_lexicon_tags_25 },
    { 0xff4e82a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4e82aa, SUC_lexicon_tags_43 },
    { 0x392e82aa, SUC_lexicon_tags_91 },
    { 0x51ee82ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e82af, SUC_lexicon_tags_19 },
    { 0xfb8e82b0, SUC_lexicon_tags_11 },
    { 0xc9ae82b1, SUC_lexicon_tags_69 },
    { 0x1b8e82af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8e82b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dee82b9, SUC_lexicon_tags_2 },
    { 0x42ae82ba, SUC_lexicon_tags_2 },
    { 0xf0ee82bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96e82be, SUC_lexicon_tags_11 },
    { 0xebee82bf, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa0ae82c1, SUC_lexicon_tags_25 },
    { 0x5d0e82c2, SUC_lexicon_tags_15 },
    { 0x6f2e82c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x966e82c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ae82c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x950e82cb, SUC_lexicon_tags_46 },
    { 0x63ce82cc, SUC_lexicon_tags_34 },
    { 0xd60e82cc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4e4e82cf, SUC_lexicon_tags_0 },
    { 0xc90e82d0, SUC_lexicon_tags_10 },
    { 0x55ae82d1, SUC_lexicon_tags_12 },
    { 0xd12e82d1, SUC_lexicon_tags_25 },
    { 0xc2ae82d1, SUC_lexicon_tags_23 },
    { 0x14e82d3, SUC_lexicon_tags_38 },
    { 0x65ce82d2, SUC_lexicon_tags_160 },
    { 0xb04e82d6, SUC_lexicon_tags_49 },
    { 0xafae82d7, SUC_lexicon_tags_12 },
    { 0x724e82d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ae82db, SUC_lexicon_tags_37 },
    { 0xb4ce82db, SUC_lexicon_tags_37 },
    { 0x48e82dd, SUC_lexicon_tags_18 },
    { 0x6a2e82de, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4e82e1, SUC_lexicon_tags_11 },
    { 0xf64e82e2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ee82e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc32e82e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ae82ee, SUC_lexicon_tags_23 },
    { 0x2bce82ef, SUC_lexicon_tags_37 },
    { 0x266e82ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8e82f2, SUC_lexicon_tags_25 },
    { 0x64e82f3, SUC_lexicon_tags_17 },
    { 0x420e82f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ae82f7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4b0e82f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ae82fc, SUC_lexicon_tags_30 },
    { 0x14ae82fd, SUC_lexicon_tags_0 },
    { 0xfaae82fd, SUC_lexicon_tags_37 },
    { 0xa6ae82ff, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96e8304, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x30ee8306, SUC_lexicon_tags_8 },
    { 0xb60e8307, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeee830c, SUC_lexicon_tags_84 },
    { 0x950e830d, SUC_lexicon_tags_17 },
    { 0x1ee830e, SUC_lexicon_tags_10 },
    { 0x4ce830e, SUC_lexicon_tags_7 },
    { 0x648e8310, SUC_lexicon_tags_25 },
    { 0x260e8310, SUC_lexicon_tags_97 },
    { 0x50ae8312, SUC_lexicon_tags_21 },
    { 0xdaae830d, SUC_lexicon_tags_11 },
    { 0xed2e8313, SUC_lexicon_tags_1 },
    { 0x54ae8315, SUC_lexicon_tags_37 },
    { 0xc26e8316, SUC_lexicon_tags_12 },
    { 0x2c8e8314, SUC_lexicon_tags_45 },
    { 0xf9ce8318, SUC_lexicon_tags_7 },
    { 0x9bee830f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ce831d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x362e8320, SUC_lexicon_tags_13 },
    { 0xd24e8321, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x276e8324, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4e8328, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74e832b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x58e832d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2e8331, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6e8334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ae833a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ce8342, SUC_lexicon_tags_6 },
    { 0x7ee8342, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6e8346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2e834a, SUC_lexicon_tags_7 },
    { 0xa6ce834b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x716e834d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4e8351, SUC_lexicon_tags_33 },
    { 0x378e8351, SUC_lexicon_tags_37 },
    { 0x2a2e8353, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2e8359, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32e835e, SUC_lexicon_tags_11 },
    { 0x74ae835e, SUC_lexicon_tags_17 },
    { 0xb20e8360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ae8363, SUC_lexicon_tags_87 },
    { 0xb56e8364, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6e8367, SUC_lexicon_tags_37 },
    { 0x2a8e8368, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ce836c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x6f6e836e, SUC_lexicon_tags_27 },
    { 0xb3ae836e, SUC_lexicon_tags_19 },
    { 0xa88e8370, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc06e8372, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x188e8375, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26e8378, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ce837e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850e8383, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbce8386, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0e838b, SUC_lexicon_tags_17 },
    { 0xd76e838c, SUC_lexicon_tags_14 },
    { 0x7a6e838d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8e8390, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48e8396, SUC_lexicon_tags_57 },
    { 0x27ee8397, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0e839d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856e83a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c0e83a3, SUC_lexicon_tags_63 },
    { 0x428e83a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1e2e83a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10e83aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e0e83ac, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb7ee83ae, SUC_lexicon_tags_25 },
    { 0x9e8e83af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaeee83b1, SUC_lexicon_tags_12 },
    { 0x844e83b2, SUC_lexicon_tags_75 },
    { 0x3a0e83b2, SUC_lexicon_tags_37 },
    { 0x868e83b1, SUC_lexicon_tags_0 },
    { 0xd2ce83b5, SUC_lexicon_tags_1 },
    { 0x532e83b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x932e83b8, SUC_lexicon_tags_68 },
    { 0xbe2e83b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ce83c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cce83c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aae83c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04e83cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4e83d0, SUC_lexicon_tags_17 },
    { 0xe2ce83d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x540e83d4, SUC_lexicon_tags_32 },
    { 0x606e83d5, SUC_lexicon_tags_85 },
    { 0x2e83d4, SUC_lexicon_tags_7 },
    { 0xdece83d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2e83da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ae83dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x574e83df, SUC_lexicon_tags_27 },
    { 0xd88e83e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ce83e3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ece83e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf9ce83e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeee83eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbae83ed, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ae83f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22ee83f2, SUC_lexicon_tags_2 },
    { 0x41ee83f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938e83fa, SUC_lexicon_tags_43 },
    { 0x740e83fa, SUC_lexicon_tags_25 },
    { 0xa0ce83fa, SUC_lexicon_tags_56 },
    { 0x1a8e83fd, SUC_lexicon_tags_42 },
    { 0xeb6e83fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8e8405, SUC_lexicon_tags_27 },
    { 0xb5ae8406, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaae840e, SUC_lexicon_tags_17 },
    { 0xe840e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ee8412, SUC_lexicon_tags_37 },
    { 0x7f4e8412, SUC_lexicon_tags_39 },
    { 0x8cae8414, SUC_lexicon_tags_136 },
    { 0x7b8e8415, SUC_lexicon_tags_87 },
    { 0xce8e8415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x428e841d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90e8422, SUC_lexicon_tags_15 },
    { 0x9f6e8422, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82e8429, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe8e842b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4a2e842d, SUC_lexicon_tags_25 },
    { 0x682e842d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5e4e8430, SUC_lexicon_tags_33 },
    { 0x7ee8430, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x388e8434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56e8437, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd68e8439, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ae843e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32e8447, SUC_lexicon_tags_52 },
    { 0xc12e8448, SUC_lexicon_tags_15 },
    { 0x848e8448, SUC_lexicon_tags_37 },
    { 0x9e0e844a, SUC_lexicon_tags_43 },
    { 0xe76e844b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484e8450, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84ae8452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20e8458, SUC_lexicon_tags_47 },
    { 0x7c2e8458, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbee845c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x710e845f, SUC_lexicon_tags_19 },
    { 0x124e8460, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ae846e, SUC_lexicon_tags_11 },
    { 0xe4e846f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ece8473, SUC_lexicon_tags_31 },
    { 0x6b8e8474, SUC_lexicon_tags_45 },
    { 0xfdae8475, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x152e8477, SUC_lexicon_tags_37 },
    { 0x78ce8477, SUC_lexicon_tags_92 },
    { 0x4bce8479, SUC_lexicon_tags_3 },
    { 0x59ae847a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ae8480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf0e8482, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2e8485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabae848d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x462e8490, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xace8492, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x204e8494, SUC_lexicon_tags_63 },
    { 0x34e8495, SUC_lexicon_tags_4 },
    { 0xe9ae8496, SUC_lexicon_tags_8 },
    { 0xf80e8496, SUC_lexicon_tags_30 },
    { 0xaaee8498, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ee849b, SUC_lexicon_tags_33 },
    { 0xb10e849c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94e84a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb18e84a5, SUC_lexicon_tags_31 },
    { 0xe80e84a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746e84b0, SUC_lexicon_tags_10 },
    { 0x9cce84b1, SUC_lexicon_tags_23 },
    { 0xb80e84b2, SUC_lexicon_tags_7 },
    { 0x15ce84b2, SUC_lexicon_tags_37 },
    { 0xb8e84b4, SUC_lexicon_tags_49 },
    { 0x346e84b5, SUC_lexicon_tags_5 },
    { 0xea0e84b6, SUC_lexicon_tags_14 },
    { 0x40ce84b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ee84ba, SUC_lexicon_tags_49 },
    { 0xfd8e84bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9bae84bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532e84c3, SUC_lexicon_tags_27 },
    { 0xf96e84c4, SUC_lexicon_tags_37 },
    { 0xa00e84c5, SUC_lexicon_tags_6 },
    { 0xc50e84c6, SUC_lexicon_tags_10 },
    { 0xece84c7, SUC_lexicon_tags_12 },
    { 0x438e84c4, SUC_lexicon_tags_11 },
    { 0x416e84c8, SUC_lexicon_tags_78 },
    { 0x2a8e84c7, SUC_lexicon_tags_11 },
    { 0x440e84c6, SUC_lexicon_tags_23 },
    { 0x3a6e84cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b2e84ce, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdb4e84d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ee84d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36e84d8, SUC_lexicon_tags_15 },
    { 0x962e84d9, SUC_lexicon_tags_17 },
    { 0xbf2e84da, SUC_lexicon_tags_11 },
    { 0xfc4e84d9, SUC_lexicon_tags_56 },
    { 0x86ce84dc, SUC_lexicon_tags_11 },
    { 0xa8ce84dc, SUC_lexicon_tags_1 },
    { 0x80ee84db, SUC_lexicon_tags_29 },
    { 0xccce84df, SUC_lexicon_tags_7 },
    { 0x67ae84df, SUC_lexicon_tags_17 },
    { 0xdd2e84df, SUC_lexicon_tags_9 },
    { 0x7a6e84e2, SUC_lexicon_tags_17 },
    { 0x170e84e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59ce84e5, SUC_lexicon_tags_1 },
    { 0x5b4e84e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96e84ea, SUC_lexicon_tags_30 },
    { 0x348e84eb, SUC_lexicon_tags_1 },
    { 0xff0e84eb, SUC_lexicon_tags_1 },
    { 0x840e84eb, SUC_lexicon_tags_43 },
    { 0x7dee84ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8e84f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc34e84f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ce84f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4e84fc, SUC_lexicon_tags_27 },
    { 0xacce84fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaae8500, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4e8503, SUC_lexicon_tags_11 },
    { 0x232e8503, SUC_lexicon_tags_63 },
    { 0xedce8504, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x532e8507, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f0e8509, SUC_lexicon_tags_11 },
    { 0x870e8509, SUC_lexicon_tags_15 },
    { 0x2e850a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eee8514, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9eae8516, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd56e8518, SUC_lexicon_tags_6 },
    { 0xd96e8519, SUC_lexicon_tags_26 },
    { 0x768e851a, SUC_lexicon_tags_46 },
    { 0xba2e851b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36e851d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x378e851f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee8524, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460e852f, SUC_lexicon_tags_6 },
    { 0x6c0e8530, SUC_lexicon_tags_39 },
    { 0xb86e8531, SUC_lexicon_tags_37 },
    { 0xe8532, SUC_lexicon_tags_12 },
    { 0x964e8531, SUC_lexicon_tags_49 },
    { 0x9ee8533, SUC_lexicon_tags_129 },
    { 0x18ee8535, SUC_lexicon_tags_15 },
    { 0x3b4e8535, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54e853a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ae8542, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6dee8544, SUC_lexicon_tags_6 },
    { 0x804e8545, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7aae8547, SUC_lexicon_tags_55 },
    { 0xe62e8548, SUC_lexicon_tags_12 },
    { 0xc96e8548, SUC_lexicon_tags_4 },
    { 0x908e854a, SUC_lexicon_tags_49 },
    { 0x1dee854b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788e855a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366e8562, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ee8568, SUC_lexicon_tags_12 },
    { 0xdb8e8569, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7ee856b, SUC_lexicon_tags_23 },
    { 0xf0e856c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ae8575, SUC_lexicon_tags_15 },
    { 0x73ce8575, SUC_lexicon_tags_7 },
    { 0x6a8e8577, SUC_lexicon_tags_8 },
    { 0x95ae8578, SUC_lexicon_tags_40 },
    { 0xcb0e8579, SUC_lexicon_tags_47 },
    { 0x168e857a, SUC_lexicon_tags_7 },
    { 0xf08e8577, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656e8580, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ae8583, SUC_lexicon_tags_1 },
    { 0xcb4e8584, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12e858d, SUC_lexicon_tags_17 },
    { 0x888e858d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddce8593, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31ae8595, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ee859b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ce85a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426e85a6, SUC_lexicon_tags_15 },
    { 0xd74e85a6, SUC_lexicon_tags_37 },
    { 0x604e85a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e85b2, SUC_lexicon_tags_14 },
    { 0x592e85b3, SUC_lexicon_tags_12 },
    { 0xab8e85b4, SUC_lexicon_tags_16 },
    { 0x290e85b5, SUC_lexicon_tags_60 },
    { 0x8f6e85b5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x40ee85b8, SUC_lexicon_tags_17 },
    { 0xec6e85b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ce85bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce85c4, SUC_lexicon_tags_17 },
    { 0xd1ae85c5, SUC_lexicon_tags_19 },
    { 0x780e85c4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42e85cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ae85d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2ce85d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x512e85d5, SUC_lexicon_tags_31 },
    { 0x6eae85d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ee85db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71ce85dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea0e85df, SUC_lexicon_tags_25 },
    { 0xd72e85df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x56ee85e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x630e85e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2e85eb, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x700e85ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ce85f2, SUC_lexicon_tags_17 },
    { 0x13ae85f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fce85f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4e85f8, SUC_lexicon_tags_12 },
    { 0x124e85f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ae85ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x994e8602, SUC_lexicon_tags_68 },
    { 0x9ce8603, SUC_lexicon_tags_60 },
    { 0xf32e8604, SUC_lexicon_tags_28 },
    { 0xbe0e8605, SUC_lexicon_tags_30 },
    { 0xd36e8606, SUC_lexicon_tags_27 },
    { 0x492e8607, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fce860b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce860f, SUC_lexicon_tags_23 },
    { 0x46e8610, SUC_lexicon_tags_27 },
    { 0x950e8611, SUC_lexicon_tags_60 },
    { 0x144e860f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808e8622, SUC_lexicon_tags_87 },
    { 0x94ae8622, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dae8627, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa38e8629, SUC_lexicon_tags_37 },
    { 0xed2e862a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bae8635, SUC_lexicon_tags_17 },
    { 0xc16e8636, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0x9f2e8638, SUC_lexicon_tags_17 },
    { 0xd36e8639, SUC_lexicon_tags_65 },
    { 0xd12e8639, SUC_lexicon_tags_26 },
    { 0x2e6e863b, SUC_lexicon_tags_17 },
    { 0xd14e863c, SUC_lexicon_tags_35 },
    { 0xd94e863c, SUC_lexicon_tags_39 },
    { 0x4d0e863d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ee8644, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x102e8646, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd44e8648, SUC_lexicon_tags_11 },
    { 0x776e8649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x312e864b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefae864d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1aee864f, SUC_lexicon_tags_28 },
    { 0x520e864f, SUC_lexicon_tags_25 },
    { 0x9e0e8650, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x50ae8653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x768e8656, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6e865c, SUC_lexicon_tags_37 },
    { 0xe08e865d, SUC_lexicon_tags_199 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862e8661, SUC_lexicon_tags_11 },
    { 0x446e8661, SUC_lexicon_tags_13 },
    { 0x5a6e8663, SUC_lexicon_tags_23 },
    { 0x560e8664, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bee866a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418e866f, SUC_lexicon_tags_47 },
    { 0x4ece8670, SUC_lexicon_tags_7 },
    { 0xdb4e8671, SUC_lexicon_tags_11 },
    { 0x6d6e8672, SUC_lexicon_tags_1 },
    { 0x342e8673, SUC_lexicon_tags_15 },
    { 0x80e8670, SUC_lexicon_tags_38 },
    { 0xd3ee8672, SUC_lexicon_tags_80 },
    { 0xbe0e8673, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bee867a, SUC_lexicon_tags_1 },
    { 0x43ae867b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7ee867d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422e8681, SUC_lexicon_tags_2 },
    { 0xd8ae8682, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc7ae8684, SUC_lexicon_tags_271 },
    { 0, NULL },
    { 0x19ee8686, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ee868b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ace868d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42ae868f, SUC_lexicon_tags_14 },
    { 0xcbce868f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aae8694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18ce8696, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda4e8698, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ae86a5, SUC_lexicon_tags_63 },
    { 0xb42e86a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f6e86a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eee86ae, SUC_lexicon_tags_33 },
    { 0xffce86af, SUC_lexicon_tags_30 },
    { 0x62ee86ae, SUC_lexicon_tags_33 },
    { 0x9c4e86b0, SUC_lexicon_tags_37 },
    { 0xb18e86b1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa06e86b5, SUC_lexicon_tags_47 },
    { 0x1fee86b5, SUC_lexicon_tags_37 },
    { 0x30ae86b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaae86bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x524e86c1, SUC_lexicon_tags_37 },
    { 0xa70e86c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ce86c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38e86cd, SUC_lexicon_tags_17 },
    { 0x5cee86ce, SUC_lexicon_tags_1 },
    { 0x978e86cf, SUC_lexicon_tags_62 },
    { 0x45ee86cf, SUC_lexicon_tags_20 },
    { 0x432e86d1, SUC_lexicon_tags_4 },
    { 0xc18e86d0, SUC_lexicon_tags_11 },
    { 0x72ee86d3, SUC_lexicon_tags_38 },
    { 0x80ee86d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8e86d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dae86dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bee86df, SUC_lexicon_tags_11 },
    { 0x420e86df, SUC_lexicon_tags_7 },
    { 0xd0e86e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ce86e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734e86ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3ae86f1, SUC_lexicon_tags_16 },
    { 0x5cae86f2, SUC_lexicon_tags_17 },
    { 0x32ee86f3, SUC_lexicon_tags_43 },
    { 0x7eee86f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10e86f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72e86fd, SUC_lexicon_tags_7 },
    { 0xdf2e86fd, SUC_lexicon_tags_37 },
    { 0x59ee86ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97ce8701, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3dee8703, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4e870c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x188e870e, SUC_lexicon_tags_39 },
    { 0x704e870f, SUC_lexicon_tags_6 },
    { 0xd1ae8710, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ee8713, SUC_lexicon_tags_15 },
    { 0x6cae8714, SUC_lexicon_tags_37 },
    { 0x94ee8714, SUC_lexicon_tags_17 },
    { 0x6a4e8716, SUC_lexicon_tags_1 },
    { 0x768e8715, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58e871b, SUC_lexicon_tags_112 },
    { 0xcace871c, SUC_lexicon_tags_1 },
    { 0x60ae871d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bce871f, SUC_lexicon_tags_151 },
    { 0x8fae8720, SUC_lexicon_tags_63 },
    { 0xb3ee871f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ee8726, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x82ce8728, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6e872d, SUC_lexicon_tags_12 },
    { 0xcf2e872e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x890e8731, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x16ae8733, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eae8737, SUC_lexicon_tags_11 },
    { 0x97ee8738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x158e873a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34ee873c, SUC_lexicon_tags_19 },
    { 0x98ae873d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeae873f, SUC_lexicon_tags_68 },
    { 0xc06e873f, SUC_lexicon_tags_60 },
    { 0x654e8741, SUC_lexicon_tags_6 },
    { 0x2c4e8741, SUC_lexicon_tags_60 },
    { 0x2dce8743, SUC_lexicon_tags_37 },
    { 0xc44e8742, SUC_lexicon_tags_55 },
    { 0xd50e8745, SUC_lexicon_tags_12 },
    { 0x85ae8745, SUC_lexicon_tags_39 },
    { 0x7e0e8744, SUC_lexicon_tags_55 },
    { 0xf96e8741, SUC_lexicon_tags_17 },
    { 0xf2ee8749, SUC_lexicon_tags_50 },
    { 0xe5ce874a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02e874d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd22e874f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52e8753, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892e8759, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8e8761, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc26e8764, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ce8767, SUC_lexicon_tags_55 },
    { 0x918e8767, SUC_lexicon_tags_25 },
    { 0x4dae8769, SUC_lexicon_tags_43 },
    { 0x8dae8769, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x640e876d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cae8772, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58e8776, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x66ce8778, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98ee877a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6dae877c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ce877f, SUC_lexicon_tags_7 },
    { 0x56ae877f, SUC_lexicon_tags_2 },
    { 0xe58e8781, SUC_lexicon_tags_1 },
    { 0x930e8782, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x912e8784, SUC_lexicon_tags_15 },
    { 0xccae8785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08e8788, SUC_lexicon_tags_45 },
    { 0x28ee8789, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ae8791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32e8794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd4e8796, SUC_lexicon_tags_6 },
    { 0x236e8797, SUC_lexicon_tags_31 },
    { 0x20e8798, SUC_lexicon_tags_19 },
    { 0x16ae8799, SUC_lexicon_tags_0 },
    { 0x294e8799, SUC_lexicon_tags_41 },
    { 0x97ee879a, SUC_lexicon_tags_1 },
    { 0xe46e8799, SUC_lexicon_tags_33 },
    { 0x4ce8799, SUC_lexicon_tags_23 },
    { 0xae8e879b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0e87a2, SUC_lexicon_tags_112 },
    { 0x94e87a3, SUC_lexicon_tags_33 },
    { 0x548e87a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ce87a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6e87ad, SUC_lexicon_tags_82 },
    { 0x26ae87ad, SUC_lexicon_tags_48 },
    { 0x1cae87ad, SUC_lexicon_tags_26 },
    { 0x882e87ae, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ece87bd, SUC_lexicon_tags_15 },
    { 0xf14e87bd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x43ee87c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa72e87c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e87c8, SUC_lexicon_tags_160 },
    { 0xf0e87c8, SUC_lexicon_tags_15 },
    { 0x9cce87ca, SUC_lexicon_tags_7 },
    { 0x424e87cb, SUC_lexicon_tags_28 },
    { 0xf34e87cc, SUC_lexicon_tags_55 },
    { 0xf96e87cd, SUC_lexicon_tags_5 },
    { 0x5b2e87ce, SUC_lexicon_tags_19 },
    { 0x24ae87cf, SUC_lexicon_tags_47 },
    { 0xaf0e87d0, SUC_lexicon_tags_11 },
    { 0x864e87ce, SUC_lexicon_tags_25 },
    { 0x282e87cd, SUC_lexicon_tags_11 },
    { 0xcdae87cf, SUC_lexicon_tags_19 },
    { 0x21ee87d4, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4e87d7, SUC_lexicon_tags_47 },
    { 0xca8e87d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8e87db, SUC_lexicon_tags_30 },
    { 0xebee87db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0e87e0, SUC_lexicon_tags_25 },
    { 0xdd8e87e0, SUC_lexicon_tags_29 },
    { 0xb88e87e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160e87e6, SUC_lexicon_tags_25 },
    { 0xc42e87e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8e87eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76e87ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ce87f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4e87f4, SUC_lexicon_tags_7 },
    { 0x840e87f4, SUC_lexicon_tags_60 },
    { 0x98ee87f6, SUC_lexicon_tags_0 },
    { 0x7a0e87f6, SUC_lexicon_tags_55 },
    { 0xcc2e87f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ee8801, SUC_lexicon_tags_11 },
    { 0xe20e8801, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd90e8804, SUC_lexicon_tags_17 },
    { 0x834e8804, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4e8808, SUC_lexicon_tags_25 },
    { 0xcece8808, SUC_lexicon_tags_11 },
    { 0x190e880a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x544e880c, SUC_lexicon_tags_17 },
    { 0xaa6e880d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ee8810, SUC_lexicon_tags_25 },
    { 0x860e8810, SUC_lexicon_tags_28 },
    { 0xdb2e8812, SUC_lexicon_tags_33 },
    { 0xdcee8811, SUC_lexicon_tags_47 },
    { 0x2c6e8814, SUC_lexicon_tags_14 },
    { 0x54e8815, SUC_lexicon_tags_40 },
    { 0x23ce8814, SUC_lexicon_tags_63 },
    { 0xd16e8811, SUC_lexicon_tags_19 },
    { 0xdb8e8818, SUC_lexicon_tags_0 },
    { 0x2d0e8819, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd62e881b, SUC_lexicon_tags_16 },
    { 0x260e881c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ae8829, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596e8830, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x3a4e8832, SUC_lexicon_tags_12 },
    { 0x95ee8833, SUC_lexicon_tags_17 },
    { 0x7fee8834, SUC_lexicon_tags_39 },
    { 0x1eae8833, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbece8837, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5fce8839, SUC_lexicon_tags_15 },
    { 0x656e883a, SUC_lexicon_tags_37 },
    { 0x93ee8839, SUC_lexicon_tags_11 },
    { 0x4a2e883c, SUC_lexicon_tags_17 },
    { 0xb50e883d, SUC_lexicon_tags_21 },
    { 0xba4e883e, SUC_lexicon_tags_80 },
    { 0x722e8839, SUC_lexicon_tags_17 },
    { 0xa54e8840, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8e8845, SUC_lexicon_tags_21 },
    { 0x1d0e8846, SUC_lexicon_tags_17 },
    { 0x634e8846, SUC_lexicon_tags_16 },
    { 0x518e8848, SUC_lexicon_tags_39 },
    { 0x4e0e8846, SUC_lexicon_tags_11 },
    { 0x13ce8849, SUC_lexicon_tags_26 },
    { 0x6d6e884b, SUC_lexicon_tags_47 },
    { 0x2b8e884b, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368e8854, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3cee8856, SUC_lexicon_tags_27 },
    { 0x404e8857, SUC_lexicon_tags_4 },
    { 0x520e8858, SUC_lexicon_tags_15 },
    { 0xdc0e8859, SUC_lexicon_tags_47 },
    { 0x5e2e8857, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ae885e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52e8864, SUC_lexicon_tags_8 },
    { 0xe46e8865, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xca4e8867, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46ee8869, SUC_lexicon_tags_8 },
    { 0xff2e8869, SUC_lexicon_tags_13 },
    { 0xa9ee886a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ee886e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73ee8870, SUC_lexicon_tags_7 },
    { 0x922e8871, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa0ce8873, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ee8877, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408e887b, SUC_lexicon_tags_8 },
    { 0xb68e887c, SUC_lexicon_tags_47 },
    { 0x642e887d, SUC_lexicon_tags_27 },
    { 0x2b8e887d, SUC_lexicon_tags_2 },
    { 0x1b6e887c, SUC_lexicon_tags_27 },
    { 0x7aee887d, SUC_lexicon_tags_33 },
    { 0xbc4e8881, SUC_lexicon_tags_11 },
    { 0x55ee8881, SUC_lexicon_tags_82 },
    { 0xf7ce8883, SUC_lexicon_tags_10 },
    { 0xb3ee887d, SUC_lexicon_tags_5 },
    { 0x14e8885, SUC_lexicon_tags_39 },
    { 0x956e8885, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe6ee8888, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xface888b, SUC_lexicon_tags_7 },
    { 0x430e888b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd4e888e, SUC_lexicon_tags_35 },
    { 0x58ee888e, SUC_lexicon_tags_27 },
    { 0x8a2e8890, SUC_lexicon_tags_7 },
    { 0xb2ce888e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0e8899, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce889c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80e88a0, SUC_lexicon_tags_2 },
    { 0x5c2e88a1, SUC_lexicon_tags_23 },
    { 0x650e88a2, SUC_lexicon_tags_23 },
    { 0x54ae88a3, SUC_lexicon_tags_82 },
    { 0x750e88a2, SUC_lexicon_tags_49 },
    { 0x9fee88a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc5ae88a7, SUC_lexicon_tags_29 },
    { 0xf94e88a8, SUC_lexicon_tags_25 },
    { 0xda4e88a8, SUC_lexicon_tags_17 },
    { 0x198e88aa, SUC_lexicon_tags_7 },
    { 0xb0ae88a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x446e88ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe44e88b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x156e88b3, SUC_lexicon_tags_23 },
    { 0xa4e88b4, SUC_lexicon_tags_45 },
    { 0x120e88b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50e88bc, SUC_lexicon_tags_2 },
    { 0x684e88bd, SUC_lexicon_tags_6 },
    { 0xddce88bd, SUC_lexicon_tags_28 },
    { 0x890e88bf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138e88c5, SUC_lexicon_tags_6 },
    { 0x310e88c6, SUC_lexicon_tags_11 },
    { 0xa1ce88c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ae88cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82e88cf, SUC_lexicon_tags_12 },
    { 0xf30e88d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4e88d3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ee88e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x776e88e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c4e88e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d0e88eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x952e88ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8e88f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78ce88f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd0ee88fb, SUC_lexicon_tags_38 },
    { 0xab0e88fc, SUC_lexicon_tags_11 },
    { 0x128e88fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e8900, SUC_lexicon_tags_11 },
    { 0xf0ee8901, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe10e8903, SUC_lexicon_tags_12 },
    { 0x4ae8904, SUC_lexicon_tags_70 },
    { 0x514e8905, SUC_lexicon_tags_27 },
    { 0x870e8904, SUC_lexicon_tags_33 },
    { 0x310e8907, SUC_lexicon_tags_6 },
    { 0x4fce8908, SUC_lexicon_tags_95 },
    { 0xb18e8906, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70e8910, SUC_lexicon_tags_381 },
    { 0xb34e8910, SUC_lexicon_tags_26 },
    { 0x388e8912, SUC_lexicon_tags_42 },
    { 0xc30e8913, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04e8916, SUC_lexicon_tags_6 },
    { 0x86ee8916, SUC_lexicon_tags_38 },
    { 0x68ee8918, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ae891c, SUC_lexicon_tags_12 },
    { 0xfc8e891d, SUC_lexicon_tags_66 },
    { 0x410e891e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8ee8920, SUC_lexicon_tags_47 },
    { 0x7cae8921, SUC_lexicon_tags_12 },
    { 0x480e8920, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e8927, SUC_lexicon_tags_37 },
    { 0x3bee8927, SUC_lexicon_tags_107 },
    { 0xfcce8927, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cae892e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbee8931, SUC_lexicon_tags_7 },
    { 0x14e8931, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf4e8934, SUC_lexicon_tags_25 },
    { 0x84e8935, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618e8939, SUC_lexicon_tags_28 },
    { 0x244e893a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ce893f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4e8943, SUC_lexicon_tags_33 },
    { 0x34ae8943, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x158e8946, SUC_lexicon_tags_2 },
    { 0x448e8946, SUC_lexicon_tags_1 },
    { 0xa2e8948, SUC_lexicon_tags_47 },
    { 0xe74e8948, SUC_lexicon_tags_37 },
    { 0xd58e8948, SUC_lexicon_tags_17 },
    { 0x67ee8948, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798e8954, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee8958, SUC_lexicon_tags_56 },
    { 0x900e8959, SUC_lexicon_tags_23 },
    { 0x8fce8958, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ee895e, SUC_lexicon_tags_37 },
    { 0x23ce895f, SUC_lexicon_tags_2 },
    { 0x75ae895f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x62ae8962, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ce8965, SUC_lexicon_tags_17 },
    { 0x67ee8966, SUC_lexicon_tags_2 },
    { 0x850e8965, SUC_lexicon_tags_25 },
    { 0xfae8968, SUC_lexicon_tags_27 },
    { 0xb22e8968, SUC_lexicon_tags_16 },
    { 0x5dce896a, SUC_lexicon_tags_29 },
    { 0xc9ce896b, SUC_lexicon_tags_37 },
    { 0x7b2e8966, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aee896f, SUC_lexicon_tags_9 },
    { 0xedee8970, SUC_lexicon_tags_2 },
    { 0xff6e896f, SUC_lexicon_tags_15 },
    { 0xbf6e8972, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ae8976, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08e897a, SUC_lexicon_tags_25 },
    { 0x86ce897a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf40e897d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf2ae897f, SUC_lexicon_tags_26 },
    { 0x8c8e8980, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe74e8982, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xece8985, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0e8989, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ae898d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x258e898f, SUC_lexicon_tags_45 },
    { 0x2fae898f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120e8994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806e8999, SUC_lexicon_tags_23 },
    { 0xd88e899a, SUC_lexicon_tags_37 },
    { 0x7f6e899b, SUC_lexicon_tags_25 },
    { 0xe4ce899c, SUC_lexicon_tags_7 },
    { 0xc3ce899d, SUC_lexicon_tags_11 },
    { 0x178e899e, SUC_lexicon_tags_15 },
    { 0x75ee899b, SUC_lexicon_tags_4 },
    { 0x3ace899d, SUC_lexicon_tags_15 },
    { 0x684e89a1, SUC_lexicon_tags_1 },
    { 0x5e6e8999, SUC_lexicon_tags_1640 },
    { 0, NULL },
    { 0x3aae89a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x444e89a7, SUC_lexicon_tags_23 },
    { 0xacce89a8, SUC_lexicon_tags_27 },
    { 0x430e89a8, SUC_lexicon_tags_30 },
    { 0x1f8e89a9, SUC_lexicon_tags_19 },
    { 0xa1ce89a9, SUC_lexicon_tags_12 },
    { 0xb34e89ac, SUC_lexicon_tags_27 },
    { 0x458e89aa, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x966e89af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea6e89b1, SUC_lexicon_tags_7 },
    { 0x38ae89b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1ee89b4, SUC_lexicon_tags_23 },
    { 0x9c2e89b5, SUC_lexicon_tags_8 },
    { 0x6aee89b5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cae89b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5fae89bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24e89be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d8e89c0, SUC_lexicon_tags_37 },
    { 0x2f6e89c1, SUC_lexicon_tags_33 },
    { 0xb42e89c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xabee89c4, SUC_lexicon_tags_12 },
    { 0x8e2e89c4, SUC_lexicon_tags_17 },
    { 0x1cee89c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ce89c9, SUC_lexicon_tags_47 },
    { 0x184e89c9, SUC_lexicon_tags_39 },
    { 0xdf4e89c9, SUC_lexicon_tags_10 },
    { 0xeb4e89cc, SUC_lexicon_tags_1 },
    { 0xf7ae89cd, SUC_lexicon_tags_15 },
    { 0xd3ee89cb, SUC_lexicon_tags_72 },
    { 0xa62e89cf, SUC_lexicon_tags_48 },
    { 0x702e89d0, SUC_lexicon_tags_17 },
    { 0x9c0e89d0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fae89d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ee89dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa2e89df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe42e89e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e89e6, SUC_lexicon_tags_79 },
    { 0x6c2e89e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dae89eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e89f0, SUC_lexicon_tags_31 },
    { 0xe2ae89f1, SUC_lexicon_tags_15 },
    { 0xa84e89f1, SUC_lexicon_tags_15 },
    { 0xb7ee89f1, SUC_lexicon_tags_46 },
    { 0x5ee89f4, SUC_lexicon_tags_47 },
    { 0xef4e89f4, SUC_lexicon_tags_60 },
    { 0x22ee89f6, SUC_lexicon_tags_37 },
    { 0x94ce89f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c8e89f9, SUC_lexicon_tags_1 },
    { 0x124e89fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70e89fd, SUC_lexicon_tags_12 },
    { 0x3e2e89fe, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ae8a09, SUC_lexicon_tags_47 },
    { 0xae8e8a09, SUC_lexicon_tags_82 },
    { 0x1fae8a09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a6e8a0d, SUC_lexicon_tags_1 },
    { 0xbd6e8a0e, SUC_lexicon_tags_46 },
    { 0xca4e8a0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34e8a12, SUC_lexicon_tags_47 },
    { 0x7f2e8a13, SUC_lexicon_tags_59 },
    { 0xbcce8a14, SUC_lexicon_tags_37 },
    { 0x2d2e8a14, SUC_lexicon_tags_30 },
    { 0xde8e8a13, SUC_lexicon_tags_19 },
    { 0x974e8a12, SUC_lexicon_tags_11 },
    { 0x3ece8a14, SUC_lexicon_tags_38 },
    { 0xe2e8a18, SUC_lexicon_tags_11 },
    { 0x5f6e8a1a, SUC_lexicon_tags_15 },
    { 0x7c0e8a1b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8e8a1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc74e8a20, SUC_lexicon_tags_13 },
    { 0x46e8a21, SUC_lexicon_tags_27 },
    { 0x940e8a22, SUC_lexicon_tags_38 },
    { 0x6e2e8a23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x936e8a26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb26e8a28, SUC_lexicon_tags_30 },
    { 0x968e8a28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc46e8a2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbf2e8a2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72e8a30, SUC_lexicon_tags_15 },
    { 0x5f2e8a31, SUC_lexicon_tags_13 },
    { 0xd98e8a31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bee8a38, SUC_lexicon_tags_17 },
    { 0x5bee8a38, SUC_lexicon_tags_2 },
    { 0x8b0e8a3a, SUC_lexicon_tags_9 },
    { 0xd0ae8a3b, SUC_lexicon_tags_7 },
    { 0x2d6e8a3c, SUC_lexicon_tags_11 },
    { 0x8dee8a3c, SUC_lexicon_tags_23 },
    { 0x832e8a3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ae8a43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xad8e8a45, SUC_lexicon_tags_21 },
    { 0x3fee8a46, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xca6e8a49, SUC_lexicon_tags_55 },
    { 0x7ee8a49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c4e8a4c, SUC_lexicon_tags_39 },
    { 0xb68e8a4d, SUC_lexicon_tags_7 },
    { 0x6ae8a4d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88e8a51, SUC_lexicon_tags_2 },
    { 0x8dee8a52, SUC_lexicon_tags_148 },
    { 0xaa0e8a53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42e8a56, SUC_lexicon_tags_8 },
    { 0x10ce8a56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8e8a5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8e8a5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73ee8a60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec4e8a62, SUC_lexicon_tags_17 },
    { 0xa9ce8a63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaee8a6a, SUC_lexicon_tags_4 },
    { 0x9dce8a6b, SUC_lexicon_tags_77 },
    { 0xcb2e8a6c, SUC_lexicon_tags_59 },
    { 0xc8ae8a6d, SUC_lexicon_tags_68 },
    { 0x59ce8a6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ce8a71, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe12e8a73, SUC_lexicon_tags_7 },
    { 0x9a2e8a73, SUC_lexicon_tags_37 },
    { 0x8ace8a75, SUC_lexicon_tags_66 },
    { 0x9fce8a74, SUC_lexicon_tags_0 },
    { 0x564e8a77, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ce8a7e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6e8a82, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04e8a87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6e8a8b, SUC_lexicon_tags_2 },
    { 0x8d0e8a8b, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8e8a90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ae8a99, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ae8a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x616e8aa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fce8aa3, SUC_lexicon_tags_587 },
    { 0xfd8e8aa3, SUC_lexicon_tags_17 },
    { 0x33ee8aa5, SUC_lexicon_tags_34 },
    { 0x89ae8aa5, SUC_lexicon_tags_49 },
    { 0xb44e8aa7, SUC_lexicon_tags_49 },
    { 0x1d2e8aa8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840e8aac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ae8aae, SUC_lexicon_tags_11 },
    { 0xd98e8aae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x344e8ab1, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ee8ab6, SUC_lexicon_tags_11 },
    { 0xe22e8ab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ee8ac0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6e8ac4, SUC_lexicon_tags_84 },
    { 0xc76e8ac4, SUC_lexicon_tags_13 },
    { 0x62ce8ac4, SUC_lexicon_tags_32 },
    { 0x91ae8ac7, SUC_lexicon_tags_7 },
    { 0x216e8ac7, SUC_lexicon_tags_60 },
    { 0x510e8ac9, SUC_lexicon_tags_37 },
    { 0x956e8ac6, SUC_lexicon_tags_12 },
    { 0x8ce8acb, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0xf5ce8acd, SUC_lexicon_tags_53 },
    { 0xf32e8acd, SUC_lexicon_tags_2 },
    { 0xe86e8acf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x624e8ad2, SUC_lexicon_tags_25 },
    { 0xb98e8ad3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x376e8ad5, SUC_lexicon_tags_7 },
    { 0x524e8ad6, SUC_lexicon_tags_13 },
    { 0xf9ee8ad7, SUC_lexicon_tags_39 },
    { 0x6aae8ad5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x602e8ada, SUC_lexicon_tags_17 },
    { 0xd14e8adb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eae8ae2, SUC_lexicon_tags_63 },
    { 0x7fee8ae2, SUC_lexicon_tags_44 },
    { 0xbb8e8ae3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x770e8ae6, SUC_lexicon_tags_25 },
    { 0xfb6e8ae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ae8aeb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x622e8aee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ae8af1, SUC_lexicon_tags_121 },
    { 0x160e8af2, SUC_lexicon_tags_15 },
    { 0x248e8af3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb64e8af5, SUC_lexicon_tags_26 },
    { 0x124e8af5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x750e8af9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44e8afd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdee8b02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212e8b09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30e8b0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380e8b11, SUC_lexicon_tags_65 },
    { 0x1b8e8b11, SUC_lexicon_tags_82 },
    { 0xae6e8b13, SUC_lexicon_tags_38 },
    { 0x636e8b13, SUC_lexicon_tags_27 },
    { 0xbdce8b15, SUC_lexicon_tags_2 },
    { 0xb5ce8b15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ae8b1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38ae8b1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444e8b21, SUC_lexicon_tags_40 },
    { 0x18ae8b22, SUC_lexicon_tags_2 },
    { 0xb70e8b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ae8b26, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2e8b2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcae8b2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x26e8b31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabee8b33, SUC_lexicon_tags_12 },
    { 0xc1ce8b33, SUC_lexicon_tags_182 },
    { 0xecee8b34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d6e8b37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8e8b3a, SUC_lexicon_tags_29 },
    { 0xcdce8b3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6e8b3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18ae8b41, SUC_lexicon_tags_19 },
    { 0x342e8b42, SUC_lexicon_tags_15 },
    { 0xe92e8b42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4e8b47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6e8b4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790e8b50, SUC_lexicon_tags_37 },
    { 0xb42e8b51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x110e8b54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x500e8b57, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0xfc4e8b59, SUC_lexicon_tags_14 },
    { 0x2eee8b5a, SUC_lexicon_tags_11 },
    { 0xa28e8b5b, SUC_lexicon_tags_14 },
    { 0x48ae8b5c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae8b64, SUC_lexicon_tags_63 },
    { 0x60ee8b64, SUC_lexicon_tags_61 },
    { 0x52ce8b64, SUC_lexicon_tags_2 },
    { 0xd7ce8b66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22e8b6b, SUC_lexicon_tags_33 },
    { 0xc92e8b6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516e8b74, SUC_lexicon_tags_25 },
    { 0xbd2e8b74, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2f6e8b77, SUC_lexicon_tags_47 },
    { 0xf36e8b78, SUC_lexicon_tags_37 },
    { 0x236e8b77, SUC_lexicon_tags_7 },
    { 0x454e8b7a, SUC_lexicon_tags_39 },
    { 0x9f4e8b78, SUC_lexicon_tags_2 },
    { 0x252e8b78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946e8b81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6e8b84, SUC_lexicon_tags_15 },
    { 0xfae8b85, SUC_lexicon_tags_0 },
    { 0xeee8b86, SUC_lexicon_tags_12 },
    { 0x89ce8b87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc06e8b89, SUC_lexicon_tags_1 },
    { 0x3e6e8b8a, SUC_lexicon_tags_12 },
    { 0x172e8b8a, SUC_lexicon_tags_7 },
    { 0xf56e8b8c, SUC_lexicon_tags_25 },
    { 0xc72e8b8c, SUC_lexicon_tags_37 },
    { 0xaf6e8b8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabee8b90, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x290e8b93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ee8b97, SUC_lexicon_tags_37 },
    { 0x302e8b97, SUC_lexicon_tags_38 },
    { 0xf90e8b99, SUC_lexicon_tags_27 },
    { 0xf5ce8b9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ee8ba1, SUC_lexicon_tags_7 },
    { 0xce0e8ba1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836e8ba8, SUC_lexicon_tags_92 },
    { 0x136e8ba9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xedae8bab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x128e8bad, SUC_lexicon_tags_58 },
    { 0x70ee8bad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16e8bb1, SUC_lexicon_tags_1 },
    { 0x3bee8bb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ae8bb5, SUC_lexicon_tags_16 },
    { 0xf0ce8bb6, SUC_lexicon_tags_17 },
    { 0xc98e8bb7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94e8bba, SUC_lexicon_tags_3 },
    { 0x5e8e8bbb, SUC_lexicon_tags_33 },
    { 0xed2e8bbb, SUC_lexicon_tags_28 },
    { 0xe32e8bbc, SUC_lexicon_tags_37 },
    { 0x8f6e8bbe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xeaae8bc0, SUC_lexicon_tags_7 },
    { 0x486e8bc0, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556e8bc6, SUC_lexicon_tags_23 },
    { 0x6ace8bc7, SUC_lexicon_tags_35 },
    { 0xc6ae8bc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72e8bcb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40e8bd1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x440e8bd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944e8bdf, SUC_lexicon_tags_0 },
    { 0x576e8be0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeae8be5, SUC_lexicon_tags_21 },
    { 0x51ee8be6, SUC_lexicon_tags_1 },
    { 0x26ce8be7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x38ce8be9, SUC_lexicon_tags_10 },
    { 0x4c6e8bea, SUC_lexicon_tags_62 },
    { 0xd48e8beb, SUC_lexicon_tags_1 },
    { 0x45ee8beb, SUC_lexicon_tags_8 },
    { 0xf6e8bed, SUC_lexicon_tags_12 },
    { 0xde4e8bec, SUC_lexicon_tags_37 },
    { 0xece8bec, SUC_lexicon_tags_46 },
    { 0xbdce8bf0, SUC_lexicon_tags_17 },
    { 0xe2e8bf0, SUC_lexicon_tags_11 },
    { 0x3c2e8bf1, SUC_lexicon_tags_2 },
    { 0xe9ae8bf3, SUC_lexicon_tags_31 },
    { 0x984e8bf4, SUC_lexicon_tags_37 },
    { 0x338e8bf4, SUC_lexicon_tags_0 },
    { 0xe40e8bf6, SUC_lexicon_tags_7 },
    { 0x47ae8bf3, SUC_lexicon_tags_11 },
    { 0xfc6e8bf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6e8bfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31ce8bfd, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0x92e8c00, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x328e8c02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ee8c05, SUC_lexicon_tags_60 },
    { 0x222e8c06, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e8c0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eee8c11, SUC_lexicon_tags_11 },
    { 0x7e4e8c12, SUC_lexicon_tags_92 },
    { 0x3a6e8c13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ee8c18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5ee8c1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90e8c1d, SUC_lexicon_tags_31 },
    { 0x900e8c1e, SUC_lexicon_tags_16 },
    { 0x492e8c1f, SUC_lexicon_tags_1 },
    { 0x7fce8c1d, SUC_lexicon_tags_15 },
    { 0xc6ee8c1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc0e8c23, SUC_lexicon_tags_0 },
    { 0x41ae8c23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x906e8c26, SUC_lexicon_tags_2 },
    { 0xf86e8c27, SUC_lexicon_tags_60 },
    { 0x4c2e8c26, SUC_lexicon_tags_8 },
    { 0x750e8c29, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0e8c2c, SUC_lexicon_tags_17 },
    { 0xc7ce8c2c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ce8c30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf48e8c32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xebee8c34, SUC_lexicon_tags_4 },
    { 0xbb0e8c35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ce8c38, SUC_lexicon_tags_25 },
    { 0x8eae8c39, SUC_lexicon_tags_11 },
    { 0xf34e8c38, SUC_lexicon_tags_19 },
    { 0xba2e8c38, SUC_lexicon_tags_38 },
    { 0x4fee8c38, SUC_lexicon_tags_42 },
    { 0x802e8c3b, SUC_lexicon_tags_55 },
    { 0x414e8c3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ce8c43, SUC_lexicon_tags_27 },
    { 0xc16e8c43, SUC_lexicon_tags_43 },
    { 0x708e8c43, SUC_lexicon_tags_63 },
    { 0x1f0e8c43, SUC_lexicon_tags_7 },
    { 0xe88e8c47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8c4a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ee8c4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x424e8c51, SUC_lexicon_tags_48 },
    { 0x850e8c52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x478e8c54, SUC_lexicon_tags_77 },
    { 0x5cee8c55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x588e8c57, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ee8c5c, SUC_lexicon_tags_17 },
    { 0xa0e8c5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb70e8c5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ae8c62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328e8c66, SUC_lexicon_tags_151 },
    { 0x948e8c66, SUC_lexicon_tags_8 },
    { 0x7c2e8c68, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ee8c6e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172e8c79, SUC_lexicon_tags_14 },
    { 0xcf6e8c79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc2e8c7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x594e8c7f, SUC_lexicon_tags_7 },
    { 0x394e8c7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ee8c83, SUC_lexicon_tags_79 },
    { 0x9e0e8c84, SUC_lexicon_tags_13 },
    { 0x6ace8c84, SUC_lexicon_tags_23 },
    { 0x7dce8c86, SUC_lexicon_tags_0 },
    { 0x19ee8c85, SUC_lexicon_tags_2 },
    { 0xb16e8c88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66e8c8d, SUC_lexicon_tags_33 },
    { 0x720e8c8e, SUC_lexicon_tags_7 },
    { 0xc36e8c8f, SUC_lexicon_tags_12 },
    { 0x95ae8c8f, SUC_lexicon_tags_43 },
    { 0xc2ee8c91, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92e8c96, SUC_lexicon_tags_25 },
    { 0x120e8c97, SUC_lexicon_tags_117 },
    { 0xd94e8c98, SUC_lexicon_tags_77 },
    { 0xbd0e8c99, SUC_lexicon_tags_28 },
    { 0x998e8c9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6e8c9d, SUC_lexicon_tags_7 },
    { 0x46ae8c9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eae8ca4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ee8ca8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x112e8caa, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6e8cad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ae8cb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6ee8cb6, SUC_lexicon_tags_64 },
    { 0xc16e8cb6, SUC_lexicon_tags_44 },
    { 0xedee8cb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2e8cbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdce8cbf, SUC_lexicon_tags_37 },
    { 0xa32e8cc0, SUC_lexicon_tags_8 },
    { 0xff4e8cc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85ce8cc3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8e8cc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x14ae8cc8, SUC_lexicon_tags_43 },
    { 0xc60e8cc8, SUC_lexicon_tags_7 },
    { 0xed8e8cca, SUC_lexicon_tags_7 },
    { 0x86ae8ccb, SUC_lexicon_tags_43 },
    { 0x56ee8ccb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ee8cd2, SUC_lexicon_tags_33 },
    { 0x826e8cd3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1e4e8cd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabee8cd7, SUC_lexicon_tags_8 },
    { 0x716e8cd7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384e8ce1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc0e8ce3, SUC_lexicon_tags_5 },
    { 0xf0e8ce4, SUC_lexicon_tags_4 },
    { 0x810e8ce4, SUC_lexicon_tags_48 },
    { 0x5fee8ce3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0e8cec, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12e8cef, SUC_lexicon_tags_36 },
    { 0x9ece8cef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52e8cf3, SUC_lexicon_tags_12 },
    { 0x622e8cf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2e8cf9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf62e8cfb, SUC_lexicon_tags_37 },
    { 0xdbae8cfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912e8d02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0e8d07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8e8d0b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x432e8d0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ee8d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x542e8d14, SUC_lexicon_tags_19 },
    { 0x7dae8d15, SUC_lexicon_tags_9 },
    { 0x40ae8d16, SUC_lexicon_tags_4 },
    { 0xe86e8d15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6e8d1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0ee8d1e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x9f2e8d20, SUC_lexicon_tags_37 },
    { 0x4d4e8d21, SUC_lexicon_tags_0 },
    { 0x1e0e8d22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0e8d2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fee8d30, SUC_lexicon_tags_47 },
    { 0xfb4e8d30, SUC_lexicon_tags_9 },
    { 0xc88e8d32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0ce8d34, SUC_lexicon_tags_63 },
    { 0x6a6e8d34, SUC_lexicon_tags_37 },
    { 0x5c0e8d36, SUC_lexicon_tags_35 },
    { 0x4f2e8d37, SUC_lexicon_tags_25 },
    { 0x67ae8d36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x472e8d3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6e8d3e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ae8d45, SUC_lexicon_tags_25 },
    { 0xef0e8d46, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x420e8d48, SUC_lexicon_tags_15 },
    { 0xa70e8d49, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ae8d4f, SUC_lexicon_tags_1 },
    { 0xe40e8d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd86e8d52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6ae8d54, SUC_lexicon_tags_23 },
    { 0x30ee8d55, SUC_lexicon_tags_61 },
    { 0xf62e8d56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86e8d5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186e8d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8e8d6b, SUC_lexicon_tags_15 },
    { 0x2dae8d6c, SUC_lexicon_tags_25 },
    { 0x9c8e8d6d, SUC_lexicon_tags_26 },
    { 0x302e8d6e, SUC_lexicon_tags_47 },
    { 0xf86e8d6f, SUC_lexicon_tags_2 },
    { 0xac8e8d6f, SUC_lexicon_tags_47 },
    { 0xb92e8d70, SUC_lexicon_tags_60 },
    { 0x6cae8d72, SUC_lexicon_tags_55 },
    { 0x28e8d73, SUC_lexicon_tags_37 },
    { 0x6aee8d74, SUC_lexicon_tags_11 },
    { 0xa38e8d75, SUC_lexicon_tags_13 },
    { 0xe8ee8d76, SUC_lexicon_tags_11 },
    { 0xefae8d77, SUC_lexicon_tags_2 },
    { 0x26ee8d72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ee8d7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cae8d7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x27ee8d81, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x442e8d84, SUC_lexicon_tags_38 },
    { 0x8b6e8d84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4e8d8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7d8e8d8d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6e8d98, SUC_lexicon_tags_1 },
    { 0x468e8d99, SUC_lexicon_tags_15 },
    { 0x1e4e8d9a, SUC_lexicon_tags_77 },
    { 0x92ee8d9b, SUC_lexicon_tags_12 },
    { 0x1c8e8d9c, SUC_lexicon_tags_11 },
    { 0xcece8d9c, SUC_lexicon_tags_12 },
    { 0x894e8d9c, SUC_lexicon_tags_1 },
    { 0x330e8d9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ee8da4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02e8da9, SUC_lexicon_tags_27 },
    { 0xbdee8daa, SUC_lexicon_tags_0 },
    { 0x246e8dab, SUC_lexicon_tags_87 },
    { 0x3d8e8dac, SUC_lexicon_tags_15 },
    { 0x646e8dad, SUC_lexicon_tags_15 },
    { 0x63ce8dac, SUC_lexicon_tags_1 },
    { 0xe86e8dad, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x358e8db2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258e8db6, SUC_lexicon_tags_17 },
    { 0x6c4e8db7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24e8dba, SUC_lexicon_tags_1 },
    { 0x1e8e8dba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e8dc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0e8dc4, SUC_lexicon_tags_37 },
    { 0x66ae8dc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ae8dcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaae8dce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ce8dd4, SUC_lexicon_tags_1 },
    { 0x2e8e8dd5, SUC_lexicon_tags_15 },
    { 0x1d6e8dd4, SUC_lexicon_tags_6 },
    { 0xea0e8dd7, SUC_lexicon_tags_11 },
    { 0x4a6e8dd7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ece8de1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbaae8de3, SUC_lexicon_tags_10 },
    { 0x11ae8de4, SUC_lexicon_tags_37 },
    { 0xa48e8de5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ae8dea, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e8ded, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffae8df0, SUC_lexicon_tags_37 },
    { 0x15ee8df1, SUC_lexicon_tags_1 },
    { 0x172e8df2, SUC_lexicon_tags_19 },
    { 0x1dae8df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44e8df7, SUC_lexicon_tags_43 },
    { 0xdaee8df7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4e8dfb, SUC_lexicon_tags_178 },
    { 0xecce8dfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78e8e02, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ece8e05, SUC_lexicon_tags_23 },
    { 0xc8e8e06, SUC_lexicon_tags_0 },
    { 0x34e8e07, SUC_lexicon_tags_141 },
    { 0x1eae8e08, SUC_lexicon_tags_9 },
    { 0x1d8e8e09, SUC_lexicon_tags_248 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ee8e0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x720e8e10, SUC_lexicon_tags_25 },
    { 0x468e8e11, SUC_lexicon_tags_0 },
    { 0xb0ae8e12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724e8e18, SUC_lexicon_tags_136 },
    { 0x5eae8e19, SUC_lexicon_tags_27 },
    { 0x38ee8e19, SUC_lexicon_tags_47 },
    { 0xa4ae8e18, SUC_lexicon_tags_11 },
    { 0xd98e8e1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12e8e1e, SUC_lexicon_tags_53 },
    { 0x7ae8e1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x946e8e21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x346e8e23, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadee8e2b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x82ee8e2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x620e8e2f, SUC_lexicon_tags_29 },
    { 0x592e8e2f, SUC_lexicon_tags_29 },
    { 0xe82e8e31, SUC_lexicon_tags_21 },
    { 0x8bee8e32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66e8e35, SUC_lexicon_tags_32 },
    { 0xa4ae8e36, SUC_lexicon_tags_17 },
    { 0xeae8e36, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4a8e8e39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bce8e3c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x522e8e3e, SUC_lexicon_tags_17 },
    { 0x244e8e3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4e8e42, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fae8e45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e4e8e47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x452e8e4a, SUC_lexicon_tags_10 },
    { 0xe18e8e4b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8e8e52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa12e8e54, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf48e8e56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dce8e65, SUC_lexicon_tags_0 },
    { 0x44ae8e65, SUC_lexicon_tags_17 },
    { 0xefee8e67, SUC_lexicon_tags_37 },
    { 0x666e8e68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c2e8e6a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aee8e6e, SUC_lexicon_tags_18 },
    { 0xf5ee8e6f, SUC_lexicon_tags_2 },
    { 0xe0ce8e70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x38e8e72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366e8e76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x12ee8e78, SUC_lexicon_tags_7 },
    { 0xcaee8e78, SUC_lexicon_tags_25 },
    { 0x7a0e8e7a, SUC_lexicon_tags_2 },
    { 0x140e8e7b, SUC_lexicon_tags_7 },
    { 0x6b0e8e7c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18e8e83, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x688e8e85, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x890e8e87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ee8e8f, SUC_lexicon_tags_37 },
    { 0x71ee8e90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb14e8e92, SUC_lexicon_tags_1 },
    { 0xebae8e93, SUC_lexicon_tags_16 },
    { 0xa04e8e93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xadce8e96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d6e8e98, SUC_lexicon_tags_27 },
    { 0x4a6e8e99, SUC_lexicon_tags_1 },
    { 0x31ce8e9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6e8e9d, SUC_lexicon_tags_13 },
    { 0x2cee8e9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4e8ea4, SUC_lexicon_tags_46 },
    { 0x6c2e8ea5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x530e8ea7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6e8ea9, SUC_lexicon_tags_36 },
    { 0xba6e8eaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e8ead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4e8eb0, SUC_lexicon_tags_37 },
    { 0xb7ee8eb1, SUC_lexicon_tags_11 },
    { 0x994e8eb2, SUC_lexicon_tags_85 },
    { 0xd30e8eb0, SUC_lexicon_tags_31 },
    { 0xcbce8eb0, SUC_lexicon_tags_60 },
    { 0x5a2e8eb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0e8ebe, SUC_lexicon_tags_7 },
    { 0x900e8ebf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xca2e8ec1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4e8ec4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776e8eca, SUC_lexicon_tags_15 },
    { 0xf24e8ecb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x522e8ecd, SUC_lexicon_tags_7 },
    { 0x5c4e8ecd, SUC_lexicon_tags_31 },
    { 0xdf0e8ece, SUC_lexicon_tags_13 },
    { 0xb7ae8ed0, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xeb8e8ed2, SUC_lexicon_tags_87 },
    { 0x6d0e8ed3, SUC_lexicon_tags_118 },
    { 0xa50e8ed3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf98e8ed6, SUC_lexicon_tags_13 },
    { 0x59ae8ed6, SUC_lexicon_tags_37 },
    { 0x5a4e8ed8, SUC_lexicon_tags_28 },
    { 0xfa6e8ed9, SUC_lexicon_tags_25 },
    { 0xfaae8ed9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x420e8edc, SUC_lexicon_tags_43 },
    { 0x552e8edc, SUC_lexicon_tags_1 },
    { 0xde8e8ede, SUC_lexicon_tags_82 },
    { 0x8a0e8edf, SUC_lexicon_tags_49 },
    { 0xceae8ede, SUC_lexicon_tags_30 },
    { 0x92e8ee1, SUC_lexicon_tags_21 },
    { 0x2eae8ee1, SUC_lexicon_tags_37 },
    { 0x95ce8ee3, SUC_lexicon_tags_15 },
    { 0x184e8ee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e8eee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2e8ef1, SUC_lexicon_tags_11 },
    { 0xfc6e8ef2, SUC_lexicon_tags_11 },
    { 0xb88e8ef3, SUC_lexicon_tags_84 },
    { 0xae4e8ef4, SUC_lexicon_tags_1 },
    { 0xba4e8ef2, SUC_lexicon_tags_17 },
    { 0x1aae8ef6, SUC_lexicon_tags_9 },
    { 0x790e8ef5, SUC_lexicon_tags_27 },
    { 0xc32e8ef8, SUC_lexicon_tags_175 },
    { 0x648e8ef9, SUC_lexicon_tags_14 },
    { 0xbf4e8ef8, SUC_lexicon_tags_17 },
    { 0x4e8efb, SUC_lexicon_tags_47 },
    { 0xa1ce8efb, SUC_lexicon_tags_2 },
    { 0x97ce8efc, SUC_lexicon_tags_31 },
    { 0xacee8efd, SUC_lexicon_tags_68 },
    { 0xa40e8eff, SUC_lexicon_tags_8 },
    { 0xb18e8f00, SUC_lexicon_tags_17 },
    { 0x276e8efb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe22e8f03, SUC_lexicon_tags_1 },
    { 0x618e8f03, SUC_lexicon_tags_28 },
    { 0x4cae8f05, SUC_lexicon_tags_33 },
    { 0xbee8f05, SUC_lexicon_tags_12 },
    { 0xf7ee8f07, SUC_lexicon_tags_63 },
    { 0x16ee8f08, SUC_lexicon_tags_27 },
    { 0xf6ae8f08, SUC_lexicon_tags_12 },
    { 0x50ee8f04, SUC_lexicon_tags_27 },
    { 0x134e8f0b, SUC_lexicon_tags_15 },
    { 0x5e8e8f0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ce8f0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8e8f15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88e8f1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b6e8f1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160e8f26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990e8f2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778e8f33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24e8f35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xae2e8f37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc16e8f39, SUC_lexicon_tags_25 },
    { 0x3b4e8f3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb4e8f3c, SUC_lexicon_tags_39 },
    { 0x834e8f3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8e8f40, SUC_lexicon_tags_33 },
    { 0x91ee8f41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd96e8f43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaeae8f45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ee8f49, SUC_lexicon_tags_49 },
    { 0x64ae8f4a, SUC_lexicon_tags_37 },
    { 0xcc2e8f4b, SUC_lexicon_tags_124 },
    { 0xa08e8f4c, SUC_lexicon_tags_63 },
    { 0xb2ae8f4d, SUC_lexicon_tags_14 },
    { 0x5c6e8f4d, SUC_lexicon_tags_80 },
    { 0xd38e8f4e, SUC_lexicon_tags_17 },
    { 0xfa0e8f50, SUC_lexicon_tags_26 },
    { 0x862e8f49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab2e8f53, SUC_lexicon_tags_30 },
    { 0x31ce8f53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6e8f57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4ee8f59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x730e8f5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4e8f64, SUC_lexicon_tags_6 },
    { 0x4b2e8f64, SUC_lexicon_tags_0 },
    { 0x138e8f66, SUC_lexicon_tags_15 },
    { 0x680e8f67, SUC_lexicon_tags_2 },
    { 0x35ce8f67, SUC_lexicon_tags_17 },
    { 0x5a4e8f64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16e8f6c, SUC_lexicon_tags_17 },
    { 0x2d4e8f6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0ee8f6f, SUC_lexicon_tags_30 },
    { 0x530e8f70, SUC_lexicon_tags_11 },
    { 0x6a2e8f6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8e8f75, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x494e8f78, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ee8f7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x202e8f7d, SUC_lexicon_tags_63 },
    { 0x6e4e8f7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf46e8f80, SUC_lexicon_tags_1 },
    { 0x3dce8f81, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ae8f85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6e8f89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe84e8f8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ae8f90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ae8f96, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xb9ee8f98, SUC_lexicon_tags_25 },
    { 0x996e8f99, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770e8f9e, SUC_lexicon_tags_63 },
    { 0xe32e8f9e, SUC_lexicon_tags_79 },
    { 0xee8e8fa0, SUC_lexicon_tags_7 },
    { 0x84ce8fa1, SUC_lexicon_tags_25 },
    { 0xbdae8f9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830e8fa7, SUC_lexicon_tags_28 },
    { 0xaf0e8fa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x872e8faa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadae8fae, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aae8fb6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7a6e8fb8, SUC_lexicon_tags_658 },
    { 0xc3ce8fb9, SUC_lexicon_tags_0 },
    { 0x108e8fb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ee8fbe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ae8fc2, SUC_lexicon_tags_37 },
    { 0xc78e8fc3, SUC_lexicon_tags_2 },
    { 0xbd4e8fc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc12e8fc6, SUC_lexicon_tags_25 },
    { 0x958e8fc7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488e8fcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6e8fcf, SUC_lexicon_tags_37 },
    { 0xe8ee8fd0, SUC_lexicon_tags_11 },
    { 0xa64e8fd0, SUC_lexicon_tags_37 },
    { 0x464e8fd2, SUC_lexicon_tags_4 },
    { 0xbc2e8fd2, SUC_lexicon_tags_7 },
    { 0xdf2e8fd4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ee8fd7, SUC_lexicon_tags_37 },
    { 0x544e8fd8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8e8fdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aee8fe4, SUC_lexicon_tags_33 },
    { 0xf26e8fe5, SUC_lexicon_tags_37 },
    { 0x9c0e8fe6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ce8fe9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2e8fef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4ae8ff1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786e8ff5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf34e8ff7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ce8ffe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04e9002, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x16ce9004, SUC_lexicon_tags_17 },
    { 0x760e9004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ee900b, SUC_lexicon_tags_0 },
    { 0x480e900c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644e9012, SUC_lexicon_tags_65 },
    { 0x2ece9012, SUC_lexicon_tags_4 },
    { 0xf70e9013, SUC_lexicon_tags_11 },
    { 0x238e9015, SUC_lexicon_tags_38 },
    { 0x7c8e9016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x326e9018, SUC_lexicon_tags_11 },
    { 0xcf8e9019, SUC_lexicon_tags_7 },
    { 0x314e901a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfdce901c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x182e901f, SUC_lexicon_tags_93 },
    { 0x484e901f, SUC_lexicon_tags_7 },
    { 0x4ae9020, SUC_lexicon_tags_17 },
    { 0x4b0e9022, SUC_lexicon_tags_37 },
    { 0x89ae9023, SUC_lexicon_tags_19 },
    { 0xa32e9024, SUC_lexicon_tags_220 },
    { 0x57ae9020, SUC_lexicon_tags_7 },
    { 0x9f6e9026, SUC_lexicon_tags_6 },
    { 0x3e2e9027, SUC_lexicon_tags_11 },
    { 0x372e9024, SUC_lexicon_tags_114 },
    { 0xad6e9027, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22e902c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb4e902e, SUC_lexicon_tags_25 },
    { 0x708e902f, SUC_lexicon_tags_8 },
    { 0x814e9030, SUC_lexicon_tags_66 },
    { 0xb52e9030, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3fae9033, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe90e9036, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x800e9039, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd88e903c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedee9042, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbee9045, SUC_lexicon_tags_11 },
    { 0xb14e9045, SUC_lexicon_tags_15 },
    { 0xace9047, SUC_lexicon_tags_15 },
    { 0x8b6e9045, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708e9050, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ae9055, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bee9059, SUC_lexicon_tags_77 },
    { 0xfd0e905a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5ce905c, SUC_lexicon_tags_8 },
    { 0xe94e905d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6e905f, SUC_lexicon_tags_2 },
    { 0xaa4e905f, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0e9064, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa66e9066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x462e9068, SUC_lexicon_tags_7 },
    { 0xdee9069, SUC_lexicon_tags_6 },
    { 0x37ce906a, SUC_lexicon_tags_23 },
    { 0xd38e9068, SUC_lexicon_tags_60 },
    { 0x2d4e906c, SUC_lexicon_tags_17 },
    { 0x536e906c, SUC_lexicon_tags_17 },
    { 0x3bae906e, SUC_lexicon_tags_25 },
    { 0xdace906a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a2e9071, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x488e9079, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342e9082, SUC_lexicon_tags_25 },
    { 0x1aae9083, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0e9087, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf24e9089, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712e908d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ae9091, SUC_lexicon_tags_12 },
    { 0x8d8e9091, SUC_lexicon_tags_33 },
    { 0x53ce9093, SUC_lexicon_tags_60 },
    { 0xa98e9093, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846e9099, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d6e909b, SUC_lexicon_tags_25 },
    { 0x368e909c, SUC_lexicon_tags_1 },
    { 0x9e6e909d, SUC_lexicon_tags_11 },
    { 0x388e909e, SUC_lexicon_tags_4 },
    { 0x3b8e909f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeee90a5, SUC_lexicon_tags_27 },
    { 0x2dce90a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcee90a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa9ee90ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70e90ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632e90b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbb0e90b8, SUC_lexicon_tags_29 },
    { 0xe66e90b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fce90c0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xee4e90c2, SUC_lexicon_tags_32 },
    { 0xcdae90c2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ce90c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ee90ca, SUC_lexicon_tags_12 },
    { 0xdb4e90cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0e90ce, SUC_lexicon_tags_68 },
    { 0x962e90ce, SUC_lexicon_tags_17 },
    { 0x80ee90ce, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x23ce90d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed0e90d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ee90d8, SUC_lexicon_tags_10 },
    { 0x230e90d8, SUC_lexicon_tags_17 },
    { 0xe6ce90da, SUC_lexicon_tags_37 },
    { 0x734e90da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x520e90dd, SUC_lexicon_tags_43 },
    { 0x712e90de, SUC_lexicon_tags_33 },
    { 0x998e90de, SUC_lexicon_tags_28 },
    { 0xda2e90e0, SUC_lexicon_tags_26 },
    { 0x372e90e1, SUC_lexicon_tags_39 },
    { 0x282e90dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8e90e8, SUC_lexicon_tags_29 },
    { 0xb84e90e9, SUC_lexicon_tags_25 },
    { 0x632e90e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd96e90ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ce90f2, SUC_lexicon_tags_33 },
    { 0xc4ae90f2, SUC_lexicon_tags_25 },
    { 0x688e90f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ce90fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962e9103, SUC_lexicon_tags_87 },
    { 0x952e9104, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fee9107, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fce910a, SUC_lexicon_tags_32 },
    { 0x1d4e910b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2e910e, SUC_lexicon_tags_31 },
    { 0x6eae910f, SUC_lexicon_tags_6 },
    { 0xf8ee910f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934e9114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6e911b, SUC_lexicon_tags_17 },
    { 0x812e911c, SUC_lexicon_tags_29 },
    { 0xa78e911d, SUC_lexicon_tags_16 },
    { 0x96ce911e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb82e9120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8e9124, SUC_lexicon_tags_27 },
    { 0x99ee9124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88e9129, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ee912d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6e9130, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ece9134, SUC_lexicon_tags_7 },
    { 0xd5ee9134, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2e9138, SUC_lexicon_tags_15 },
    { 0x8e9139, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ae913e, SUC_lexicon_tags_27 },
    { 0x3b6e913f, SUC_lexicon_tags_11 },
    { 0x764e9140, SUC_lexicon_tags_1 },
    { 0x7d8e9141, SUC_lexicon_tags_17 },
    { 0x394e9142, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ce9146, SUC_lexicon_tags_2 },
    { 0x62ae9147, SUC_lexicon_tags_0 },
    { 0xddae9148, SUC_lexicon_tags_11 },
    { 0xb28e9146, SUC_lexicon_tags_48 },
    { 0xbc0e9148, SUC_lexicon_tags_25 },
    { 0xfe4e9146, SUC_lexicon_tags_28 },
    { 0x4a0e914c, SUC_lexicon_tags_47 },
    { 0xf8ee9146, SUC_lexicon_tags_15 },
    { 0xfeae914e, SUC_lexicon_tags_0 },
    { 0xe0e914f, SUC_lexicon_tags_47 },
    { 0xcdee914e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0e9154, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ae915e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xcb0e9160, SUC_lexicon_tags_2 },
    { 0x3c6e9161, SUC_lexicon_tags_7 },
    { 0xfcae9161, SUC_lexicon_tags_25 },
    { 0x76e9161, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf6e9165, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x638e9168, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69ee916a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x532e916d, SUC_lexicon_tags_13 },
    { 0xaeae916d, SUC_lexicon_tags_1 },
    { 0xf1ae916f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490e9173, SUC_lexicon_tags_37 },
    { 0x616e9174, SUC_lexicon_tags_0 },
    { 0xad2e9175, SUC_lexicon_tags_6 },
    { 0x226e9176, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4e9179, SUC_lexicon_tags_11 },
    { 0x78ae9179, SUC_lexicon_tags_28 },
    { 0x454e917b, SUC_lexicon_tags_37 },
    { 0x40ae917c, SUC_lexicon_tags_17 },
    { 0x43ee917d, SUC_lexicon_tags_25 },
    { 0xc86e917c, SUC_lexicon_tags_11 },
    { 0xc00e917e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0e9182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8e9185, SUC_lexicon_tags_37 },
    { 0x6fae9186, SUC_lexicon_tags_45 },
    { 0xaaae9187, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968e918b, SUC_lexicon_tags_3 },
    { 0x228e918c, SUC_lexicon_tags_29 },
    { 0x37ee918b, SUC_lexicon_tags_11 },
    { 0xa86e918e, SUC_lexicon_tags_15 },
    { 0x6f0e918f, SUC_lexicon_tags_17 },
    { 0x396e918c, SUC_lexicon_tags_31 },
    { 0xcf8e9191, SUC_lexicon_tags_31 },
    { 0xec6e9192, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6e9196, SUC_lexicon_tags_11 },
    { 0x8ae9197, SUC_lexicon_tags_26 },
    { 0xb10e9198, SUC_lexicon_tags_85 },
    { 0xeb4e9199, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x632e919b, SUC_lexicon_tags_613 },
    { 0x8fce919c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde4e919e, SUC_lexicon_tags_28 },
    { 0xa18e919e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee91a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x298e91a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x508e91ab, SUC_lexicon_tags_33 },
    { 0x2ece91ac, SUC_lexicon_tags_46 },
    { 0x9d4e91ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x53ee91af, SUC_lexicon_tags_15 },
    { 0x464e91b0, SUC_lexicon_tags_68 },
    { 0xf9ee91b0, SUC_lexicon_tags_15 },
    { 0x42ae91b1, SUC_lexicon_tags_46 },
    { 0xa54e91b3, SUC_lexicon_tags_15 },
    { 0x7e6e91b2, SUC_lexicon_tags_43 },
    { 0xa64e91b2, SUC_lexicon_tags_37 },
    { 0x11ee91b6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x236e91b9, SUC_lexicon_tags_77 },
    { 0x670e91ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1bce91bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a2e91be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x198e91c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeee91c4, SUC_lexicon_tags_16 },
    { 0x4e91c5, SUC_lexicon_tags_2 },
    { 0x93ee91c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ee91c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ae91ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82e91d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ce91dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf54e91de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64e91ea, SUC_lexicon_tags_45 },
    { 0x82ae91eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fce91ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf24e91f0, SUC_lexicon_tags_11 },
    { 0xb0ae91f1, SUC_lexicon_tags_14 },
    { 0xf76e91f0, SUC_lexicon_tags_43 },
    { 0xe32e91f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ce91f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4e91ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80e9203, SUC_lexicon_tags_45 },
    { 0x1f4e9204, SUC_lexicon_tags_2 },
    { 0xea4e9205, SUC_lexicon_tags_15 },
    { 0x7bee9206, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfeae9208, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x19ae920a, SUC_lexicon_tags_25 },
    { 0x7cae920a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x910e920d, SUC_lexicon_tags_7 },
    { 0xc78e920e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb96e9210, SUC_lexicon_tags_38 },
    { 0x67ae9211, SUC_lexicon_tags_45 },
    { 0xd02e9212, SUC_lexicon_tags_6 },
    { 0x82ce9213, SUC_lexicon_tags_7 },
    { 0xb16e9214, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x394e9216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x556e9218, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526e921c, SUC_lexicon_tags_37 },
    { 0x2e6e921d, SUC_lexicon_tags_16 },
    { 0x4ace921c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2e9225, SUC_lexicon_tags_6 },
    { 0x1bee9226, SUC_lexicon_tags_1 },
    { 0xb70e9226, SUC_lexicon_tags_37 },
    { 0x9fee9228, SUC_lexicon_tags_11 },
    { 0x734e9229, SUC_lexicon_tags_19 },
    { 0xd06e9226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee922f, SUC_lexicon_tags_37 },
    { 0xd58e9230, SUC_lexicon_tags_17 },
    { 0x7b2e922f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb34e9233, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790e9238, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6e923d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x932e9240, SUC_lexicon_tags_11 },
    { 0x758e9241, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00e9247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448e9250, SUC_lexicon_tags_15 },
    { 0x59ce9250, SUC_lexicon_tags_65 },
    { 0x3c6e9251, SUC_lexicon_tags_6 },
    { 0xce6e9250, SUC_lexicon_tags_65 },
    { 0x5e2e9251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdee9258, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18e925b, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xeaee925d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fee9262, SUC_lexicon_tags_31 },
    { 0x80ce9263, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ee9267, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x948e926a, SUC_lexicon_tags_1 },
    { 0x628e926a, SUC_lexicon_tags_77 },
    { 0xa66e926c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ee9271, SUC_lexicon_tags_31 },
    { 0x890e9272, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeece9274, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46ce9276, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd60e9278, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x566e927a, SUC_lexicon_tags_17 },
    { 0x41ce927b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0e9281, SUC_lexicon_tags_37 },
    { 0x8c4e9282, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f8e9284, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8e9289, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34e928d, SUC_lexicon_tags_60 },
    { 0xfaee928e, SUC_lexicon_tags_13 },
    { 0xf98e928e, SUC_lexicon_tags_49 },
    { 0x3cee9290, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdee9292, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84ee9294, SUC_lexicon_tags_37 },
    { 0x4fee9295, SUC_lexicon_tags_37 },
    { 0x78ae9294, SUC_lexicon_tags_26 },
    { 0x982e9294, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb08e9299, SUC_lexicon_tags_27 },
    { 0xf6ee9299, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x12e929c, SUC_lexicon_tags_15 },
    { 0xd50e929d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72e92a0, SUC_lexicon_tags_11 },
    { 0x60ee92a1, SUC_lexicon_tags_1 },
    { 0x70e92a2, SUC_lexicon_tags_25 },
    { 0xaaae92a2, SUC_lexicon_tags_11 },
    { 0xb40e92a2, SUC_lexicon_tags_15 },
    { 0x49ee92a5, SUC_lexicon_tags_9 },
    { 0x9ece92a6, SUC_lexicon_tags_5 },
    { 0xb18e92a2, SUC_lexicon_tags_11 },
    { 0xe02e92a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ce92ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x308e92ae, SUC_lexicon_tags_30 },
    { 0xb14e92ae, SUC_lexicon_tags_31 },
    { 0x982e92b0, SUC_lexicon_tags_1 },
    { 0xa22e92b1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6e92b6, SUC_lexicon_tags_62 },
    { 0xad2e92b6, SUC_lexicon_tags_29 },
    { 0xff4e92b7, SUC_lexicon_tags_7 },
    { 0x980e92b9, SUC_lexicon_tags_43 },
    { 0x1bae92b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ce92c2, SUC_lexicon_tags_15 },
    { 0xcc4e92c3, SUC_lexicon_tags_8 },
    { 0x2bce92c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x82e92c6, SUC_lexicon_tags_25 },
    { 0x580e92c6, SUC_lexicon_tags_13 },
    { 0x44ae92c8, SUC_lexicon_tags_17 },
    { 0x7f2e92c9, SUC_lexicon_tags_1 },
    { 0xd52e92ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc20e92cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82e92d5, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x754e92d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4e92e0, SUC_lexicon_tags_7 },
    { 0x37ae92e0, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0x790e92e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x248e92e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbee92ea, SUC_lexicon_tags_45 },
    { 0xc54e92eb, SUC_lexicon_tags_85 },
    { 0x170e92ec, SUC_lexicon_tags_11 },
    { 0x4a0e92ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x714e92f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ace92f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x602e92f6, SUC_lexicon_tags_15 },
    { 0xd48e92f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18e92fc, SUC_lexicon_tags_2 },
    { 0x236e92fd, SUC_lexicon_tags_82 },
    { 0x8f6e92fc, SUC_lexicon_tags_15 },
    { 0xf3ee92fd, SUC_lexicon_tags_49 },
    { 0xf2ce9300, SUC_lexicon_tags_38 },
    { 0x97ce9301, SUC_lexicon_tags_8 },
    { 0x844e92fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6e9305, SUC_lexicon_tags_31 },
    { 0x398e9305, SUC_lexicon_tags_13 },
    { 0x80e9305, SUC_lexicon_tags_2 },
    { 0x56ee9305, SUC_lexicon_tags_25 },
    { 0x5f8e9306, SUC_lexicon_tags_15 },
    { 0x312e9305, SUC_lexicon_tags_23 },
    { 0xdb6e930b, SUC_lexicon_tags_27 },
    { 0xe0e930b, SUC_lexicon_tags_11 },
    { 0xdc8e930b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1fae930f, SUC_lexicon_tags_9 },
    { 0x49ee9310, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x814e9312, SUC_lexicon_tags_85 },
    { 0x9cce9313, SUC_lexicon_tags_15 },
    { 0xb12e9314, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeeee9316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eee931b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee2e931d, SUC_lexicon_tags_37 },
    { 0x2a0e931e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ce9322, SUC_lexicon_tags_6 },
    { 0xa26e9322, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xde4e9325, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x166e9327, SUC_lexicon_tags_49 },
    { 0x9aae9328, SUC_lexicon_tags_12 },
    { 0x128e9329, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xecae932b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534e932f, SUC_lexicon_tags_1 },
    { 0x580e9330, SUC_lexicon_tags_27 },
    { 0xb36e9330, SUC_lexicon_tags_7 },
    { 0xb8ee9332, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf42e9334, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xdbce9336, SUC_lexicon_tags_1 },
    { 0x938e9337, SUC_lexicon_tags_25 },
    { 0x9bee9337, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4e933d, SUC_lexicon_tags_1 },
    { 0x584e933e, SUC_lexicon_tags_8 },
    { 0xd6ae933e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40e9344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabee9346, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x816e9349, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93ee934b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bee9351, SUC_lexicon_tags_15 },
    { 0xd36e9352, SUC_lexicon_tags_7 },
    { 0xe3ae9352, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x694e9355, SUC_lexicon_tags_2 },
    { 0x52ae9356, SUC_lexicon_tags_64 },
    { 0xcd4e9355, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x854e9359, SUC_lexicon_tags_57 },
    { 0xe2e935a, SUC_lexicon_tags_27 },
    { 0x42ce935a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x606e935d, SUC_lexicon_tags_47 },
    { 0x87ae935e, SUC_lexicon_tags_29 },
    { 0xda2e935f, SUC_lexicon_tags_103 },
    { 0x88ae9360, SUC_lexicon_tags_0 },
    { 0xb1ae9361, SUC_lexicon_tags_30 },
    { 0x9d4e9362, SUC_lexicon_tags_15 },
    { 0xa54e9361, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0e9368, SUC_lexicon_tags_11 },
    { 0x6c6e9369, SUC_lexicon_tags_8 },
    { 0xc3ce936a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ae9371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6e9376, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x840e9378, SUC_lexicon_tags_55 },
    { 0x10ae9379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f8e937b, SUC_lexicon_tags_49 },
    { 0xf50e937c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0e9388, SUC_lexicon_tags_0 },
    { 0x650e9389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40e938c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x106e938e, SUC_lexicon_tags_12 },
    { 0xbcae938f, SUC_lexicon_tags_28 },
    { 0x810e9390, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0e9396, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5ee9398, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4e939b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd0e939d, SUC_lexicon_tags_12 },
    { 0xc24e939e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe46e93a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e93a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ce93a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0e93ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6e93b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84e93b3, SUC_lexicon_tags_31 },
    { 0xbe0e93b4, SUC_lexicon_tags_9 },
    { 0x976e93b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf56e93b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6e93be, SUC_lexicon_tags_37 },
    { 0x77ce93be, SUC_lexicon_tags_0 },
    { 0x99ae93c0, SUC_lexicon_tags_12 },
    { 0xa30e93be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf08e93c3, SUC_lexicon_tags_2 },
    { 0x934e93c3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9d6e93c6, SUC_lexicon_tags_37 },
    { 0xc4ee93c6, SUC_lexicon_tags_552 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844e93cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ae93ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ae93d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bee93d7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc06e93d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644e93df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ae93e4, SUC_lexicon_tags_7 },
    { 0x31ce93e4, SUC_lexicon_tags_0 },
    { 0xc84e93e6, SUC_lexicon_tags_27 },
    { 0x3b4e93e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ae93ec, SUC_lexicon_tags_37 },
    { 0x434e93ed, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x672e93ef, SUC_lexicon_tags_15 },
    { 0xf0ee93f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc9ee93f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2dee93f4, SUC_lexicon_tags_43 },
    { 0x38e93f5, SUC_lexicon_tags_2 },
    { 0xe7ce93f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ce93fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf48e93fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x436e9400, SUC_lexicon_tags_39 },
    { 0xee2e9401, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980e940b, SUC_lexicon_tags_1 },
    { 0x862e940b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dee940f, SUC_lexicon_tags_27 },
    { 0x256e9410, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ee9414, SUC_lexicon_tags_28 },
    { 0x8fae9414, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe76e9417, SUC_lexicon_tags_2 },
    { 0x108e9418, SUC_lexicon_tags_15 },
    { 0x3c2e9419, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xed6e941b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ee941e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4e9421, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8e9424, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dce9429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefae942d, SUC_lexicon_tags_66 },
    { 0xf88e942d, SUC_lexicon_tags_27 },
    { 0x9a6e942d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284e9435, SUC_lexicon_tags_43 },
    { 0x21ce9436, SUC_lexicon_tags_48 },
    { 0x3a4e9437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0e943c, SUC_lexicon_tags_12 },
    { 0x5aae943c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6e9441, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4e944a, SUC_lexicon_tags_17 },
    { 0xfce944b, SUC_lexicon_tags_1 },
    { 0xc24e944a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90e9455, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ae9458, SUC_lexicon_tags_25 },
    { 0x2cce9459, SUC_lexicon_tags_34 },
    { 0xea2e945a, SUC_lexicon_tags_23 },
    { 0xba8e945b, SUC_lexicon_tags_7 },
    { 0xb92e945c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcdae945e, SUC_lexicon_tags_11 },
    { 0x53ae945f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6e9465, SUC_lexicon_tags_37 },
    { 0x126e9466, SUC_lexicon_tags_13 },
    { 0x48ce9467, SUC_lexicon_tags_2 },
    { 0x2fae9466, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2e4e946a, SUC_lexicon_tags_16 },
    { 0x9dae946a, SUC_lexicon_tags_26 },
    { 0x13ce946c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf04e946e, SUC_lexicon_tags_17 },
    { 0xcb8e946e, SUC_lexicon_tags_11 },
    { 0x190e9470, SUC_lexicon_tags_12 },
    { 0xf9ae9471, SUC_lexicon_tags_7 },
    { 0x398e9470, SUC_lexicon_tags_47 },
    { 0xfd0e9473, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ee9476, SUC_lexicon_tags_141 },
    { 0xcd8e9477, SUC_lexicon_tags_7 },
    { 0x450e9477, SUC_lexicon_tags_60 },
    { 0x738e9479, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x686e947c, SUC_lexicon_tags_11 },
    { 0x7a2e947c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x768e947f, SUC_lexicon_tags_1 },
    { 0x93ee947f, SUC_lexicon_tags_12 },
    { 0x482e9481, SUC_lexicon_tags_8 },
    { 0x6d6e9482, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebce9485, SUC_lexicon_tags_25 },
    { 0x20ce9485, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4e948b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272e948f, SUC_lexicon_tags_15 },
    { 0x5b4e9490, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4e9495, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aae949a, SUC_lexicon_tags_7 },
    { 0xadae949b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x268e949e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22e94a5, SUC_lexicon_tags_11 },
    { 0xd66e94a5, SUC_lexicon_tags_11 },
    { 0x900e94a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8e94aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bee94ac, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ee94af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eae94b2, SUC_lexicon_tags_17 },
    { 0x394e94b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6e94b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x134e94bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebae94be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe3ee94c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2e94ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256e94ce, SUC_lexicon_tags_17 },
    { 0x66ee94cf, SUC_lexicon_tags_25 },
    { 0xcd6e94d0, SUC_lexicon_tags_15 },
    { 0xc46e94d0, SUC_lexicon_tags_21 },
    { 0xfc2e94d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bae94d5, SUC_lexicon_tags_66 },
    { 0xa86e94d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8e94d8, SUC_lexicon_tags_25 },
    { 0xf8ee94d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ee94dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6e94e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5bee94e8, SUC_lexicon_tags_47 },
    { 0x5eae94e9, SUC_lexicon_tags_17 },
    { 0x288e94ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd4e94ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500e94f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ee94f3, SUC_lexicon_tags_17 },
    { 0xe0ae94f4, SUC_lexicon_tags_26 },
    { 0x2cae94f5, SUC_lexicon_tags_37 },
    { 0x5e6e94f5, SUC_lexicon_tags_37 },
    { 0xac8e94f6, SUC_lexicon_tags_25 },
    { 0x292e94f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148e94fc, SUC_lexicon_tags_12 },
    { 0x9b4e94fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8e9500, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ee9503, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8e9507, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182e950d, SUC_lexicon_tags_2 },
    { 0xcf0e950d, SUC_lexicon_tags_39 },
    { 0x792e950e, SUC_lexicon_tags_30 },
    { 0x210e950f, SUC_lexicon_tags_42 },
    { 0xceee950f, SUC_lexicon_tags_1 },
    { 0x742e9511, SUC_lexicon_tags_29 },
    { 0xece9513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadae9515, SUC_lexicon_tags_11 },
    { 0xf40e9515, SUC_lexicon_tags_7 },
    { 0x8ece9517, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0x660e9519, SUC_lexicon_tags_25 },
    { 0x15ce951a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ee9520, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aee952a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x698e952d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40e9531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x644e9534, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7ce9536, SUC_lexicon_tags_7 },
    { 0xa42e9537, SUC_lexicon_tags_24 },
    { 0x7f8e9538, SUC_lexicon_tags_63 },
    { 0x32ae9536, SUC_lexicon_tags_9 },
    { 0x1cee953a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd08e953c, SUC_lexicon_tags_0 },
    { 0x7a6e953d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82ae953f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20ee9541, SUC_lexicon_tags_23 },
    { 0xc20e9542, SUC_lexicon_tags_46 },
    { 0xbe4e9543, SUC_lexicon_tags_25 },
    { 0x4b6e9543, SUC_lexicon_tags_37 },
    { 0x1cce9545, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88e954a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ce954d, SUC_lexicon_tags_11 },
    { 0x75ce954e, SUC_lexicon_tags_33 },
    { 0xa38e954f, SUC_lexicon_tags_62 },
    { 0x38ce954d, SUC_lexicon_tags_11 },
    { 0xa2e954e, SUC_lexicon_tags_19 },
    { 0xcc2e9552, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ee9555, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a4e9557, SUC_lexicon_tags_27 },
    { 0x8fee9558, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc4e955a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa8ce955c, SUC_lexicon_tags_17 },
    { 0x710e955d, SUC_lexicon_tags_15 },
    { 0x43ae955e, SUC_lexicon_tags_50 },
    { 0xbbae955f, SUC_lexicon_tags_11 },
    { 0xa4e955c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70e9562, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ee9566, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x114e9569, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ae956e, SUC_lexicon_tags_17 },
    { 0xeece956f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefae9571, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54e9579, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x296e957c, SUC_lexicon_tags_45 },
    { 0x534e957d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ce9582, SUC_lexicon_tags_17 },
    { 0xd04e9583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40e9585, SUC_lexicon_tags_1 },
    { 0x3aae9585, SUC_lexicon_tags_45 },
    { 0x30e9587, SUC_lexicon_tags_28 },
    { 0x934e9588, SUC_lexicon_tags_15 },
    { 0x492e9589, SUC_lexicon_tags_25 },
    { 0xb34e9588, SUC_lexicon_tags_31 },
    { 0xe5ce9585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62e9592, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f4e9594, SUC_lexicon_tags_23 },
    { 0x95ce9594, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x934e9597, SUC_lexicon_tags_7 },
    { 0xb14e9598, SUC_lexicon_tags_12 },
    { 0xe92e9599, SUC_lexicon_tags_37 },
    { 0x69ce9599, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c4e959c, SUC_lexicon_tags_25 },
    { 0xdcce959d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1f2e959f, SUC_lexicon_tags_7 },
    { 0x8fce95a0, SUC_lexicon_tags_47 },
    { 0x886e95a1, SUC_lexicon_tags_23 },
    { 0xb0ce95a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ce95ad, SUC_lexicon_tags_25 },
    { 0x9f6e95ad, SUC_lexicon_tags_25 },
    { 0xe3ee95af, SUC_lexicon_tags_37 },
    { 0x17ae95b0, SUC_lexicon_tags_7 },
    { 0xb5ee95ae, SUC_lexicon_tags_60 },
    { 0xe68e95af, SUC_lexicon_tags_34 },
    { 0x36e95b0, SUC_lexicon_tags_8 },
    { 0x9ce95b4, SUC_lexicon_tags_133 },
    { 0xc2e95b5, SUC_lexicon_tags_2 },
    { 0x7dae95b5, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4e95b9, SUC_lexicon_tags_27 },
    { 0xb70e95b9, SUC_lexicon_tags_45 },
    { 0xad8e95bb, SUC_lexicon_tags_28 },
    { 0x884e95bb, SUC_lexicon_tags_2 },
    { 0xcee95b9, SUC_lexicon_tags_37 },
    { 0x424e95be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a6e95c0, SUC_lexicon_tags_15 },
    { 0xbce95c1, SUC_lexicon_tags_49 },
    { 0xfae95c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84e95c6, SUC_lexicon_tags_12 },
    { 0xedee95c7, SUC_lexicon_tags_100 },
    { 0x36ee95c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8e95cb, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x6c8e95cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582e95d3, SUC_lexicon_tags_68 },
    { 0x23ee95d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30e95d7, SUC_lexicon_tags_30 },
    { 0x392e95d7, SUC_lexicon_tags_14 },
    { 0x640e95d9, SUC_lexicon_tags_6 },
    { 0x9ace95da, SUC_lexicon_tags_23 },
    { 0x8c8e95db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d8e95dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8e95e2, SUC_lexicon_tags_11 },
    { 0xa3ae95e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24e95e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66e95ee, SUC_lexicon_tags_26 },
    { 0x784e95ee, SUC_lexicon_tags_191 },
    { 0x8e2e95ee, SUC_lexicon_tags_37 },
    { 0x584e95f1, SUC_lexicon_tags_15 },
    { 0xf9ce95f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x408e95f4, SUC_lexicon_tags_26 },
    { 0xcbee95f5, SUC_lexicon_tags_27 },
    { 0x126e95f4, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ce95fc, SUC_lexicon_tags_25 },
    { 0x9e0e95fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538e9603, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06e9606, SUC_lexicon_tags_7 },
    { 0x9d4e9607, SUC_lexicon_tags_17 },
    { 0x3e8e9608, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x788e960b, SUC_lexicon_tags_37 },
    { 0x6c4e960c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeae9610, SUC_lexicon_tags_7 },
    { 0xf0ce9611, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x614e9613, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d4e9615, SUC_lexicon_tags_1 },
    { 0xe84e9615, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8dce9618, SUC_lexicon_tags_19 },
    { 0x7bee9618, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60e9620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xace9628, SUC_lexicon_tags_37 },
    { 0xb32e9629, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xbc8e962b, SUC_lexicon_tags_2 },
    { 0xff4e962b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x840e962e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3d0e9630, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc9ce9632, SUC_lexicon_tags_17 },
    { 0x2e0e9633, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe98e9635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d8e9637, SUC_lexicon_tags_15 },
    { 0xf70e9638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44e963c, SUC_lexicon_tags_7 },
    { 0x748e963c, SUC_lexicon_tags_17 },
    { 0xeb0e963e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d6e9640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6e9645, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f6e9647, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eee964b, SUC_lexicon_tags_1 },
    { 0x9ae964c, SUC_lexicon_tags_25 },
    { 0x512e964d, SUC_lexicon_tags_33 },
    { 0xe26e964e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ae9651, SUC_lexicon_tags_6 },
    { 0xa24e9652, SUC_lexicon_tags_45 },
    { 0x1d6e9652, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x42ae9655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32ae9657, SUC_lexicon_tags_14 },
    { 0x8a8e9658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4e9661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfae9663, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce9666, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dce9669, SUC_lexicon_tags_2 },
    { 0x8a6e966a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ee966f, SUC_lexicon_tags_19 },
    { 0xf46e9670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ee9673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9ee9675, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ee9679, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xce967b, SUC_lexicon_tags_12 },
    { 0x680e967c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8c8e967e, SUC_lexicon_tags_47 },
    { 0x788e967e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x496e9681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36e9688, SUC_lexicon_tags_55 },
    { 0xa94e9689, SUC_lexicon_tags_25 },
    { 0x982e9689, SUC_lexicon_tags_27 },
    { 0xcdae9688, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00e9690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ae9693, SUC_lexicon_tags_11 },
    { 0x550e9694, SUC_lexicon_tags_14 },
    { 0xfbae9693, SUC_lexicon_tags_79 },
    { 0x968e9695, SUC_lexicon_tags_8 },
    { 0x3aae9697, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x442e9699, SUC_lexicon_tags_17 },
    { 0x17ee9699, SUC_lexicon_tags_25 },
    { 0x9ece969a, SUC_lexicon_tags_19 },
    { 0xd5ee969a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2fae969e, SUC_lexicon_tags_46 },
    { 0x146e969f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140e96a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ee96a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80e96ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ae96af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ee96b3, SUC_lexicon_tags_13 },
    { 0xce96b4, SUC_lexicon_tags_1 },
    { 0x7b6e96b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff6e96b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680e96c0, SUC_lexicon_tags_25 },
    { 0x270e96c0, SUC_lexicon_tags_17 },
    { 0xca4e96c2, SUC_lexicon_tags_87 },
    { 0x60e96c3, SUC_lexicon_tags_40 },
    { 0x340e96c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e96c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dce96ca, SUC_lexicon_tags_15 },
    { 0xe1ee96cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dee96d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aee96db, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280e96e2, SUC_lexicon_tags_12 },
    { 0x676e96e3, SUC_lexicon_tags_70 },
    { 0xcb8e96e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x174e96e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe32e96e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6e96ea, SUC_lexicon_tags_15 },
    { 0x194e96ea, SUC_lexicon_tags_32 },
    { 0x722e96ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6e96f0, SUC_lexicon_tags_25 },
    { 0x748e96f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0ee96f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ee96f6, SUC_lexicon_tags_63 },
    { 0xb76e96f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8e96fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa98e96fe, SUC_lexicon_tags_1 },
    { 0xba0e96ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe28e9701, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0e9705, SUC_lexicon_tags_11 },
    { 0x244e9706, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ae970a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x180e970d, SUC_lexicon_tags_9 },
    { 0xe6ee970e, SUC_lexicon_tags_0 },
    { 0x4bee970f, SUC_lexicon_tags_15 },
    { 0xa9ee970e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ee971b, SUC_lexicon_tags_2 },
    { 0xd90e971c, SUC_lexicon_tags_7 },
    { 0x8f4e971d, SUC_lexicon_tags_47 },
    { 0x920e971d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2e9723, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ce9726, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4e972b, SUC_lexicon_tags_12 },
    { 0xb4e972c, SUC_lexicon_tags_25 },
    { 0x930e972d, SUC_lexicon_tags_26 },
    { 0xafee972d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0e9731, SUC_lexicon_tags_22 },
    { 0x936e9731, SUC_lexicon_tags_85 },
    { 0x438e9733, SUC_lexicon_tags_44 },
    { 0x538e9733, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80ae9736, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x778e9738, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1ee973a, SUC_lexicon_tags_48 },
    { 0xd8ae973a, SUC_lexicon_tags_12 },
    { 0xf0e973c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92e973f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8e9743, SUC_lexicon_tags_37 },
    { 0xe70e9743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa7ee9746, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8e9749, SUC_lexicon_tags_22 },
    { 0x29ee974a, SUC_lexicon_tags_0 },
    { 0xd38e974b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ae9751, SUC_lexicon_tags_7 },
    { 0x278e9751, SUC_lexicon_tags_19 },
    { 0x40e9752, SUC_lexicon_tags_7 },
    { 0x2cae9754, SUC_lexicon_tags_17 },
    { 0xdaee9751, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8e975c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x28e975f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ee9763, SUC_lexicon_tags_45 },
    { 0x210e9763, SUC_lexicon_tags_47 },
    { 0xbdae9765, SUC_lexicon_tags_47 },
    { 0x9b6e9765, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ae976c, SUC_lexicon_tags_75 },
    { 0x6ae976d, SUC_lexicon_tags_2 },
    { 0x4dae976e, SUC_lexicon_tags_2 },
    { 0xc8ce976f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492e9773, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x384e9775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x464e9778, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560e9780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0e9784, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x454e9786, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2e9789, SUC_lexicon_tags_33 },
    { 0x80e978a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820e978f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2e9792, SUC_lexicon_tags_15 },
    { 0x482e9792, SUC_lexicon_tags_33 },
    { 0xc8e9794, SUC_lexicon_tags_15 },
    { 0x3c8e9794, SUC_lexicon_tags_12 },
    { 0xbb0e9796, SUC_lexicon_tags_15 },
    { 0x78ce9795, SUC_lexicon_tags_0 },
    { 0x858e9792, SUC_lexicon_tags_37 },
    { 0x356e9799, SUC_lexicon_tags_11 },
    { 0xd12e979a, SUC_lexicon_tags_11 },
    { 0xee9796, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1ace979d, SUC_lexicon_tags_23 },
    { 0x478e979d, SUC_lexicon_tags_2 },
    { 0x5c0e979f, SUC_lexicon_tags_19 },
    { 0x198e97a0, SUC_lexicon_tags_63 },
    { 0xdece97a1, SUC_lexicon_tags_12 },
    { 0x76e97a1, SUC_lexicon_tags_33 },
    { 0xbfce97a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64ee97a5, SUC_lexicon_tags_7 },
    { 0x7a2e97a5, SUC_lexicon_tags_30 },
    { 0xdcee97a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8e97ac, SUC_lexicon_tags_15 },
    { 0xaeee97ad, SUC_lexicon_tags_31 },
    { 0x5aee97ae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92e97b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabce97b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x94ae97bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c4e97bd, SUC_lexicon_tags_1 },
    { 0x99ee97be, SUC_lexicon_tags_11 },
    { 0x104e97bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa70e97c1, SUC_lexicon_tags_32 },
    { 0x5a6e97c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeeae97c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ce97c7, SUC_lexicon_tags_33 },
    { 0x54ae97c7, SUC_lexicon_tags_17 },
    { 0xe5ee97c7, SUC_lexicon_tags_15 },
    { 0x802e97c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bce97cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e6e97cf, SUC_lexicon_tags_2 },
    { 0xd5ce97cf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810e97da, SUC_lexicon_tags_1 },
    { 0x6d0e97db, SUC_lexicon_tags_17 },
    { 0x3bae97dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6e97df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x216e97e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16e97e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24e97ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x940e97f2, SUC_lexicon_tags_11 },
    { 0x984e97f2, SUC_lexicon_tags_3 },
    { 0x172e97f4, SUC_lexicon_tags_21 },
    { 0x886e97f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4ae97f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ce97ff, SUC_lexicon_tags_89 },
    { 0xd50e9800, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0e9809, SUC_lexicon_tags_25 },
    { 0xacee9809, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ce980d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5ce980f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32e9814, SUC_lexicon_tags_25 },
    { 0x170e9814, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22e981a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4e9820, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ae9824, SUC_lexicon_tags_25 },
    { 0xedce9825, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322e982a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x262e982d, SUC_lexicon_tags_10 },
    { 0x880e982d, SUC_lexicon_tags_28 },
    { 0x722e982f, SUC_lexicon_tags_5 },
    { 0x872e982d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd24e9832, SUC_lexicon_tags_31 },
    { 0x69ce9833, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340e9837, SUC_lexicon_tags_1 },
    { 0xad0e9837, SUC_lexicon_tags_27 },
    { 0x7cee9839, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6e983f, SUC_lexicon_tags_11 },
    { 0xab6e9840, SUC_lexicon_tags_49 },
    { 0x57ce983f, SUC_lexicon_tags_37 },
    { 0xa14e9841, SUC_lexicon_tags_11 },
    { 0xe62e9843, SUC_lexicon_tags_2 },
    { 0x450e9844, SUC_lexicon_tags_14 },
    { 0xe1ae9844, SUC_lexicon_tags_0 },
    { 0xc98e9843, SUC_lexicon_tags_7 },
    { 0xb4ae9847, SUC_lexicon_tags_34 },
    { 0xfa8e9848, SUC_lexicon_tags_2 },
    { 0xdfce9848, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ae984c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb84e984e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30e9851, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6e9855, SUC_lexicon_tags_37 },
    { 0x412e9855, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114e985a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5bae985c, SUC_lexicon_tags_2 },
    { 0xe1ce985d, SUC_lexicon_tags_15 },
    { 0x9fee985e, SUC_lexicon_tags_15 },
    { 0xcc2e985e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xff6e9861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe60e9863, SUC_lexicon_tags_1 },
    { 0xe78e9864, SUC_lexicon_tags_11 },
    { 0x4a4e9864, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x864e9867, SUC_lexicon_tags_7 },
    { 0xbcce9867, SUC_lexicon_tags_17 },
    { 0xceae9869, SUC_lexicon_tags_24 },
    { 0x5e0e986a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804e986e, SUC_lexicon_tags_11 },
    { 0x33ae986f, SUC_lexicon_tags_2 },
    { 0x248e986f, SUC_lexicon_tags_13 },
    { 0xf2ee9871, SUC_lexicon_tags_6 },
    { 0xf86e9872, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0e9875, SUC_lexicon_tags_6 },
    { 0x1d4e9876, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814e987a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x264e987c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac4e987e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90e9882, SUC_lexicon_tags_17 },
    { 0x204e9883, SUC_lexicon_tags_68 },
    { 0x526e9884, SUC_lexicon_tags_17 },
    { 0x38e9884, SUC_lexicon_tags_2 },
    { 0x9fae9885, SUC_lexicon_tags_14 },
    { 0x334e9887, SUC_lexicon_tags_66 },
    { 0x6d4e9883, SUC_lexicon_tags_26 },
    { 0x516e9889, SUC_lexicon_tags_25 },
    { 0x550e988a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc82e988c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbd4e988e, SUC_lexicon_tags_25 },
    { 0xed8e988f, SUC_lexicon_tags_7 },
    { 0x9b2e9890, SUC_lexicon_tags_63 },
    { 0x582e9891, SUC_lexicon_tags_39 },
    { 0xc3ce9892, SUC_lexicon_tags_11 },
    { 0x9cae988e, SUC_lexicon_tags_12 },
    { 0x6d6e9893, SUC_lexicon_tags_8 },
    { 0x504e9895, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6e9899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0e989c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x218e989e, SUC_lexicon_tags_23 },
    { 0xcdce989f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x72e98a1, SUC_lexicon_tags_49 },
    { 0x62ee98a2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9f0e98a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ce98a9, SUC_lexicon_tags_46 },
    { 0x478e98a9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922e98ae, SUC_lexicon_tags_30 },
    { 0x158e98af, SUC_lexicon_tags_65 },
    { 0xa14e98af, SUC_lexicon_tags_25 },
    { 0xd4ee98b1, SUC_lexicon_tags_17 },
    { 0x512e98b2, SUC_lexicon_tags_60 },
    { 0x10ae98b3, SUC_lexicon_tags_7 },
    { 0x7c6e98b4, SUC_lexicon_tags_14 },
    { 0xff6e98b5, SUC_lexicon_tags_1 },
    { 0xbc0e98b6, SUC_lexicon_tags_12 },
    { 0xf4e98b6, SUC_lexicon_tags_37 },
    { 0xaae98b8, SUC_lexicon_tags_17 },
    { 0x4aee98b9, SUC_lexicon_tags_17 },
    { 0x46ae98ba, SUC_lexicon_tags_55 },
    { 0x798e98ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ae98be, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2e98c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80e98cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ee98cf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ce98d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8fee98da, SUC_lexicon_tags_46 },
    { 0x9a4e98da, SUC_lexicon_tags_17 },
    { 0x848e98da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682e98e0, SUC_lexicon_tags_8 },
    { 0x438e98e1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xcbce98e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4e98e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ce98ea, SUC_lexicon_tags_38 },
    { 0xcbee98eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3fee98ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22e98ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0e98f3, SUC_lexicon_tags_6 },
    { 0x700e98f4, SUC_lexicon_tags_11 },
    { 0x66ee98f5, SUC_lexicon_tags_114 },
    { 0x46ce98f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb2ce98f8, SUC_lexicon_tags_11 },
    { 0xa52e98f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x154e98fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762e9904, SUC_lexicon_tags_12 },
    { 0xd46e9905, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370e9913, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8e9917, SUC_lexicon_tags_25 },
    { 0x742e9917, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x19ee991a, SUC_lexicon_tags_26 },
    { 0x41ae991a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1ae991d, SUC_lexicon_tags_2 },
    { 0x11ee991d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbace9922, SUC_lexicon_tags_2 },
    { 0xc48e9923, SUC_lexicon_tags_2 },
    { 0x46e9924, SUC_lexicon_tags_15 },
    { 0x18e9924, SUC_lexicon_tags_7 },
    { 0xfbae9926, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4ee9928, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6e992d, SUC_lexicon_tags_7 },
    { 0xc0ee992e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe34e9930, SUC_lexicon_tags_37 },
    { 0xd74e9930, SUC_lexicon_tags_29 },
    { 0xe3ee9931, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaae9934, SUC_lexicon_tags_37 },
    { 0xb18e9935, SUC_lexicon_tags_1 },
    { 0x47ee9935, SUC_lexicon_tags_1 },
    { 0x47ee9936, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x324e9939, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630e993f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ee9942, SUC_lexicon_tags_37 },
    { 0x3a4e9943, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54e9946, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x56e9949, SUC_lexicon_tags_37 },
    { 0xbdce994a, SUC_lexicon_tags_19 },
    { 0x8f2e9949, SUC_lexicon_tags_29 },
    { 0x282e9949, SUC_lexicon_tags_25 },
    { 0xd4e994d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4e9952, SUC_lexicon_tags_25 },
    { 0x130e9953, SUC_lexicon_tags_4 },
    { 0x89ae9954, SUC_lexicon_tags_15 },
    { 0xc4ce9955, SUC_lexicon_tags_15 },
    { 0xf5ee9956, SUC_lexicon_tags_15 },
    { 0xe30e9956, SUC_lexicon_tags_63 },
    { 0x612e9958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66e995d, SUC_lexicon_tags_4 },
    { 0x4d2e995e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ee9961, SUC_lexicon_tags_25 },
    { 0xa02e9961, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40e9965, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602e996e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x732e9970, SUC_lexicon_tags_37 },
    { 0x756e9970, SUC_lexicon_tags_11 },
    { 0x7cce9972, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ee9978, SUC_lexicon_tags_27 },
    { 0x5f4e9978, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x660e997b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xceae997d, SUC_lexicon_tags_25 },
    { 0xc50e997e, SUC_lexicon_tags_33 },
    { 0x5dae997e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2e9987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb0e9989, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93ae998b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaee9990, SUC_lexicon_tags_9 },
    { 0xc98e9990, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308e999a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312e999e, SUC_lexicon_tags_55 },
    { 0xe54e999f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x500e99a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf7ae99a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ce99ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x958e99af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x476e99b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67ce99b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344e99ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4e99be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x434e99c1, SUC_lexicon_tags_11 },
    { 0x8c4e99c2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x102e99c5, SUC_lexicon_tags_15 },
    { 0x5ace99c6, SUC_lexicon_tags_7 },
    { 0x2ace99c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80e99ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cce99d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6e99d6, SUC_lexicon_tags_17 },
    { 0x41ee99d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4e99de, SUC_lexicon_tags_11 },
    { 0x200e99de, SUC_lexicon_tags_25 },
    { 0x41ae99e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbae99e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ee99e5, SUC_lexicon_tags_8 },
    { 0xb9ee99e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfeee99e8, SUC_lexicon_tags_21 },
    { 0x55ae99e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ce99ee, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeee99f1, SUC_lexicon_tags_21 },
    { 0xb72e99f2, SUC_lexicon_tags_37 },
    { 0xcc8e99f3, SUC_lexicon_tags_17 },
    { 0x324e99f4, SUC_lexicon_tags_64 },
    { 0xebce99f5, SUC_lexicon_tags_25 },
    { 0xe9ae99f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ae99fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78e9a00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8e9a04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b4e9a06, SUC_lexicon_tags_10 },
    { 0xc3ce9a06, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x54e9a0a, SUC_lexicon_tags_7 },
    { 0xb4ae9a0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x168e9a0d, SUC_lexicon_tags_1 },
    { 0x6d8e9a0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ce9a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8e9a1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0e9a1e, SUC_lexicon_tags_12 },
    { 0xcf4e9a1e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764e9a23, SUC_lexicon_tags_25 },
    { 0x4e9a24, SUC_lexicon_tags_30 },
    { 0x5e8e9a25, SUC_lexicon_tags_11 },
    { 0xf98e9a23, SUC_lexicon_tags_1 },
    { 0xc8ee9a27, SUC_lexicon_tags_37 },
    { 0x934e9a27, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0e9a2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86e9a31, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612e9a38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9ae9a3a, SUC_lexicon_tags_37 },
    { 0x136e9a3a, SUC_lexicon_tags_57 },
    { 0x53ae9a3c, SUC_lexicon_tags_37 },
    { 0xe60e9a3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18e9a41, SUC_lexicon_tags_48 },
    { 0x180e9a42, SUC_lexicon_tags_1 },
    { 0x66ae9a43, SUC_lexicon_tags_2 },
    { 0x39ae9a44, SUC_lexicon_tags_25 },
    { 0x19ee9a42, SUC_lexicon_tags_47 },
    { 0x310e9a45, SUC_lexicon_tags_10 },
    { 0x9c6e9a45, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8e9a4a, SUC_lexicon_tags_9 },
    { 0xb08e9a4a, SUC_lexicon_tags_15 },
    { 0xfce9a4a, SUC_lexicon_tags_37 },
    { 0xcf6e9a4d, SUC_lexicon_tags_2 },
    { 0xaae9a4a, SUC_lexicon_tags_47 },
    { 0xaeee9a4b, SUC_lexicon_tags_12 },
    { 0xcd8e9a50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ee9a56, SUC_lexicon_tags_82 },
    { 0xae6e9a57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ae9a5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe46e9a5d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ae9a60, SUC_lexicon_tags_9 },
    { 0x9a6e9a60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5ce9a63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ce9a68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276e9a6c, SUC_lexicon_tags_14 },
    { 0xa18e9a6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bce9a72, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4e9a79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xccae9a7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74e9a7f, SUC_lexicon_tags_17 },
    { 0xbd6e9a7f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x972e9a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994e9a86, SUC_lexicon_tags_37 },
    { 0xca4e9a86, SUC_lexicon_tags_28 },
    { 0x4eee9a86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfee9a8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x862e9a8d, SUC_lexicon_tags_25 },
    { 0x8c2e9a8d, SUC_lexicon_tags_21 },
    { 0x79ae9a8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3a4e9a91, SUC_lexicon_tags_23 },
    { 0x1ee9a92, SUC_lexicon_tags_2 },
    { 0x66e9a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b6e9a95, SUC_lexicon_tags_663 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0e9a9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684e9aa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cee9aa5, SUC_lexicon_tags_11 },
    { 0xfa8e9aa6, SUC_lexicon_tags_15 },
    { 0xc70e9aa7, SUC_lexicon_tags_37 },
    { 0x49ce9aa5, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x3f2e9aaa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x16e9aad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56e9ab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x742e9ab7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70ce9ab9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bee9abf, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170e9ac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c0e9ac6, SUC_lexicon_tags_17 },
    { 0x6b0e9ac7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaee9acb, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2e9ad0, SUC_lexicon_tags_25 },
    { 0xafee9ad1, SUC_lexicon_tags_19 },
    { 0x594e9ad2, SUC_lexicon_tags_11 },
    { 0x32ae9ad3, SUC_lexicon_tags_28 },
    { 0x124e9ad4, SUC_lexicon_tags_71 },
    { 0x284e9ad2, SUC_lexicon_tags_12 },
    { 0x20e9ad5, SUC_lexicon_tags_26 },
    { 0xbe0e9ad6, SUC_lexicon_tags_33 },
    { 0x504e9ad8, SUC_lexicon_tags_19 },
    { 0xebae9ad6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18e9adc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8e9adf, SUC_lexicon_tags_6 },
    { 0xa6ce9adf, SUC_lexicon_tags_37 },
    { 0x724e9ae1, SUC_lexicon_tags_23 },
    { 0x71ce9adf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6e9ae5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e9ae8, SUC_lexicon_tags_85 },
    { 0xac0e9ae8, SUC_lexicon_tags_45 },
    { 0x982e9aea, SUC_lexicon_tags_61 },
    { 0x510e9aea, SUC_lexicon_tags_37 },
    { 0x490e9ae8, SUC_lexicon_tags_39 },
    { 0x1f6e9ae8, SUC_lexicon_tags_57 },
    { 0x1aee9aee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310e9af2, SUC_lexicon_tags_11 },
    { 0x52ee9af2, SUC_lexicon_tags_57 },
    { 0xa78e9af3, SUC_lexicon_tags_27 },
    { 0xc2ce9af5, SUC_lexicon_tags_17 },
    { 0xfcee9af6, SUC_lexicon_tags_11 },
    { 0xdaae9af6, SUC_lexicon_tags_79 },
    { 0xf50e9af8, SUC_lexicon_tags_87 },
    { 0x646e9af8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ee9afd, SUC_lexicon_tags_45 },
    { 0xbce9afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc4e9b00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802e9b0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34e9b13, SUC_lexicon_tags_37 },
    { 0x1b0e9b13, SUC_lexicon_tags_31 },
    { 0x604e9b15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6ae9b17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x806e9b19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa0e9b1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c0e9b1d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6e9b25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ce9b27, SUC_lexicon_tags_37 },
    { 0x582e9b28, SUC_lexicon_tags_49 },
    { 0xf42e9b29, SUC_lexicon_tags_102 },
    { 0xefce9b29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb9ae9b2c, SUC_lexicon_tags_23 },
    { 0x89ce9b2d, SUC_lexicon_tags_11 },
    { 0xc24e9b2e, SUC_lexicon_tags_16 },
    { 0x19ee9b2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ae9b34, SUC_lexicon_tags_33 },
    { 0x58ce9b35, SUC_lexicon_tags_16 },
    { 0x4b0e9b35, SUC_lexicon_tags_7 },
    { 0xc5ee9b37, SUC_lexicon_tags_15 },
    { 0xcd0e9b38, SUC_lexicon_tags_27 },
    { 0x262e9b38, SUC_lexicon_tags_23 },
    { 0xc22e9b38, SUC_lexicon_tags_15 },
    { 0xf6e9b37, SUC_lexicon_tags_43 },
    { 0x512e9b39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bee9b3f, SUC_lexicon_tags_25 },
    { 0xb6e9b40, SUC_lexicon_tags_2 },
    { 0x9bce9b41, SUC_lexicon_tags_29 },
    { 0x1b8e9b40, SUC_lexicon_tags_7 },
    { 0x206e9b42, SUC_lexicon_tags_37 },
    { 0xa62e9b43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x210e9b46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc36e9b48, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd50e9b4b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500e9b4f, SUC_lexicon_tags_31 },
    { 0xc6ce9b50, SUC_lexicon_tags_29 },
    { 0x9eee9b50, SUC_lexicon_tags_2 },
    { 0x9f2e9b50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x448e9b54, SUC_lexicon_tags_10 },
    { 0xc12e9b54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426e9b59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48e9b5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276e9b62, SUC_lexicon_tags_11 },
    { 0xa90e9b63, SUC_lexicon_tags_32 },
    { 0x66ce9b64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2e9b69, SUC_lexicon_tags_16 },
    { 0x29ce9b69, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd84e9b6c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0e9b71, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb74e9b73, SUC_lexicon_tags_14 },
    { 0x320e9b74, SUC_lexicon_tags_29 },
    { 0x1b0e9b75, SUC_lexicon_tags_57 },
    { 0x7b2e9b76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e9b7b, SUC_lexicon_tags_19 },
    { 0xcd4e9b7c, SUC_lexicon_tags_33 },
    { 0x156e9b7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2e9b81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd0e9b83, SUC_lexicon_tags_56 },
    { 0xe7ce9b84, SUC_lexicon_tags_23 },
    { 0x93ce9b85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0e9b88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ce9b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x768e9b93, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800e9b98, SUC_lexicon_tags_11 },
    { 0xd36e9b98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43ae9b9b, SUC_lexicon_tags_33 },
    { 0xb06e9b9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ee9ba1, SUC_lexicon_tags_15 },
    { 0xc7ae9ba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bee9ba5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90ce9ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ace9bab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416e9baf, SUC_lexicon_tags_28 },
    { 0xe88e9bb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40e9bb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x290e9bb5, SUC_lexicon_tags_7 },
    { 0x278e9bb6, SUC_lexicon_tags_12 },
    { 0x332e9bb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ce9bbb, SUC_lexicon_tags_38 },
    { 0x1eae9bbb, SUC_lexicon_tags_27 },
    { 0x33ae9bbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4e9bc0, SUC_lexicon_tags_30 },
    { 0x180e9bc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcace9bc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6e9bc7, SUC_lexicon_tags_11 },
    { 0x55ee9bc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e0e9bca, SUC_lexicon_tags_37 },
    { 0x13ce9bca, SUC_lexicon_tags_497 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88e9bd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820e9bda, SUC_lexicon_tags_33 },
    { 0xcb0e9bdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x142e9bdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ee9be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ce9be6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xccae9be9, SUC_lexicon_tags_25 },
    { 0x8cce9bea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x334e9bed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4e9bfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ce9c01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72e9c05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x280e9c08, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb6ae9c0a, SUC_lexicon_tags_55 },
    { 0x5f8e9c0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb68e9c0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x440e9c0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ae9c1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384e9c27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x894e9c2a, SUC_lexicon_tags_31 },
    { 0xecce9c2b, SUC_lexicon_tags_15 },
    { 0xcc2e9c2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66e9c2f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ce9c33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x992e9c36, SUC_lexicon_tags_43 },
    { 0xdaae9c37, SUC_lexicon_tags_63 },
    { 0x442e9c38, SUC_lexicon_tags_7 },
    { 0xe9ae9c36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5ee9c3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x966e9c3e, SUC_lexicon_tags_25 },
    { 0x5e0e9c3e, SUC_lexicon_tags_18 },
    { 0xb82e9c40, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4e9c43, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x876e9c45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb4e9c47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x856e9c4a, SUC_lexicon_tags_63 },
    { 0x6fce9c4a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f6e9c4d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ae9c53, SUC_lexicon_tags_12 },
    { 0x6fee9c54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdee9c5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304e9c63, SUC_lexicon_tags_1 },
    { 0x454e9c63, SUC_lexicon_tags_27 },
    { 0x31ee9c65, SUC_lexicon_tags_12 },
    { 0x750e9c66, SUC_lexicon_tags_27 },
    { 0x902e9c67, SUC_lexicon_tags_38 },
    { 0x7ace9c68, SUC_lexicon_tags_37 },
    { 0x45ae9c67, SUC_lexicon_tags_9 },
    { 0xa9ce9c6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ee9c6e, SUC_lexicon_tags_54 },
    { 0x6eee9c6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf14e9c71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x938e9c73, SUC_lexicon_tags_11 },
    { 0xf24e9c74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcce9c76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3ee9c78, SUC_lexicon_tags_8 },
    { 0xf1ce9c78, SUC_lexicon_tags_47 },
    { 0x1d2e9c7a, SUC_lexicon_tags_39 },
    { 0x26e9c79, SUC_lexicon_tags_4 },
    { 0x454e9c7c, SUC_lexicon_tags_14 },
    { 0x3dee9c7a, SUC_lexicon_tags_35 },
    { 0x3e4e9c7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2e9c82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ce9c87, SUC_lexicon_tags_17 },
    { 0x55ce9c87, SUC_lexicon_tags_11 },
    { 0xf1ce9c89, SUC_lexicon_tags_31 },
    { 0xd0ce9c89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabee9c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ce9c93, SUC_lexicon_tags_740 },
    { 0xa38e9c93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd46e9c96, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eae9c9a, SUC_lexicon_tags_1 },
    { 0x836e9c9b, SUC_lexicon_tags_47 },
    { 0x1f8e9c9c, SUC_lexicon_tags_15 },
    { 0xe32e9c9a, SUC_lexicon_tags_15 },
    { 0x6e2e9c9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x164e9ca1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfeae9ca3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x258e9ca6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbd8e9ca8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690e9cb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x878e9cb4, SUC_lexicon_tags_7 },
    { 0xbd8e9cb4, SUC_lexicon_tags_47 },
    { 0x27ae9cb5, SUC_lexicon_tags_64 },
    { 0x38ae9cb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ae9cba, SUC_lexicon_tags_60 },
    { 0xc98e9cbb, SUC_lexicon_tags_48 },
    { 0xe66e9cbb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x586e9cbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcce9cd1, SUC_lexicon_tags_10 },
    { 0xa0ee9cd1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x684e9cd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe58e9cd6, SUC_lexicon_tags_37 },
    { 0xc32e9cd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ee9cdb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa6e9cdd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4e9ce2, SUC_lexicon_tags_25 },
    { 0x160e9ce2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xddce9ce6, SUC_lexicon_tags_15 },
    { 0x2c8e9ce7, SUC_lexicon_tags_15 },
    { 0x24ce9ce8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x664e9cea, SUC_lexicon_tags_48 },
    { 0x502e9cea, SUC_lexicon_tags_8 },
    { 0x2c8e9ceb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0e9cf5, SUC_lexicon_tags_27 },
    { 0x6f2e9cf5, SUC_lexicon_tags_115 },
    { 0x7a2e9cf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc4e9cf9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2ece9cfb, SUC_lexicon_tags_7 },
    { 0xee0e9cfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x220e9cff, SUC_lexicon_tags_37 },
    { 0xa76e9d00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x210e9d02, SUC_lexicon_tags_17 },
    { 0x7fae9d03, SUC_lexicon_tags_37 },
    { 0x1a6e9d04, SUC_lexicon_tags_133 },
    { 0xafce9d05, SUC_lexicon_tags_38 },
    { 0xc56e9d03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb30e9d08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54ae9d0a, SUC_lexicon_tags_91 },
    { 0x1eae9d0a, SUC_lexicon_tags_37 },
    { 0xfc4e9d0c, SUC_lexicon_tags_87 },
    { 0xf28e9d0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918e9d11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ce9d15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2e9d18, SUC_lexicon_tags_12 },
    { 0xef8e9d18, SUC_lexicon_tags_9 },
    { 0xcf4e9d1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ae9d23, SUC_lexicon_tags_17 },
    { 0x83ae9d24, SUC_lexicon_tags_15 },
    { 0x4fce9d25, SUC_lexicon_tags_25 },
    { 0xed8e9d25, SUC_lexicon_tags_60 },
    { 0x636e9d23, SUC_lexicon_tags_55 },
    { 0xd96e9d28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ee9d2b, SUC_lexicon_tags_32 },
    { 0x946e9d2c, SUC_lexicon_tags_7 },
    { 0xb4e9d2d, SUC_lexicon_tags_27 },
    { 0xc7ee9d2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56e9d31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778e9d36, SUC_lexicon_tags_39 },
    { 0xff2e9d37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92ce9d39, SUC_lexicon_tags_15 },
    { 0x47ee9d39, SUC_lexicon_tags_150 },
    { 0xeaae9d3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc4e9d3d, SUC_lexicon_tags_65 },
    { 0xe06e9d3e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ee9d41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ae9d45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72e9d4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92e9d53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d4e9d55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46e9d59, SUC_lexicon_tags_17 },
    { 0x83ae9d59, SUC_lexicon_tags_38 },
    { 0xa9ce9d5b, SUC_lexicon_tags_33 },
    { 0xe20e9d5c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ee9d63, SUC_lexicon_tags_15 },
    { 0xc66e9d64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ce9d68, SUC_lexicon_tags_70 },
    { 0x89ce9d69, SUC_lexicon_tags_8 },
    { 0x29ee9d69, SUC_lexicon_tags_125 },
    { 0xecee9d6b, SUC_lexicon_tags_27 },
    { 0xbd2e9d6c, SUC_lexicon_tags_37 },
    { 0xacae9d6d, SUC_lexicon_tags_25 },
    { 0xc06e9d6e, SUC_lexicon_tags_24 },
    { 0x30ce9d6f, SUC_lexicon_tags_23 },
    { 0x282e9d6f, SUC_lexicon_tags_64 },
    { 0x514e9d70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeace9d73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ae9d79, SUC_lexicon_tags_6 },
    { 0x32ae9d7a, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ee9d7e, SUC_lexicon_tags_59 },
    { 0xbae9d7e, SUC_lexicon_tags_21 },
    { 0x5b4e9d80, SUC_lexicon_tags_17 },
    { 0x780e9d80, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x272e9d83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112e9d88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0ce9d8a, SUC_lexicon_tags_11 },
    { 0xcb0e9d8b, SUC_lexicon_tags_11 },
    { 0x120e9d8c, SUC_lexicon_tags_25 },
    { 0x9fce9d8d, SUC_lexicon_tags_2 },
    { 0x3f8e9d8b, SUC_lexicon_tags_33 },
    { 0x36ee9d8e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x950e9d91, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834e9d9b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12e9da1, SUC_lexicon_tags_44 },
    { 0xeeee9da2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238e9da6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd66e9da8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24e9dab, SUC_lexicon_tags_77 },
    { 0xc52e9dac, SUC_lexicon_tags_1 },
    { 0x100e9dad, SUC_lexicon_tags_1 },
    { 0x1b0e9dab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ee9db5, SUC_lexicon_tags_15 },
    { 0xc90e9db6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ce9dbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6e9dc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe68e9dc6, SUC_lexicon_tags_45 },
    { 0xf2ce9dc7, SUC_lexicon_tags_17 },
    { 0xe6e9dc7, SUC_lexicon_tags_63 },
    { 0x1b6e9dc7, SUC_lexicon_tags_19 },
    { 0x854e9dc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32ee9dcc, SUC_lexicon_tags_9 },
    { 0xcbce9dcd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xedae9dd0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20ae9dd2, SUC_lexicon_tags_2 },
    { 0x4aee9dd2, SUC_lexicon_tags_47 },
    { 0x652e9dd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8e9dd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x220e9ddb, SUC_lexicon_tags_7 },
    { 0xfe8e9ddc, SUC_lexicon_tags_25 },
    { 0xf06e9ddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654e9de3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cae9dee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ce9df1, SUC_lexicon_tags_1 },
    { 0x6a2e9df2, SUC_lexicon_tags_49 },
    { 0x42ee9df3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06e9df7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94e9dfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbce9e00, SUC_lexicon_tags_17 },
    { 0xc88e9e01, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726e9e06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x778e9e09, SUC_lexicon_tags_1 },
    { 0xd88e9e0a, SUC_lexicon_tags_0 },
    { 0xdcae9e0b, SUC_lexicon_tags_37 },
    { 0xf30e9e0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ee9e12, SUC_lexicon_tags_1 },
    { 0x2d6e9e13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26e9e17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42e9e1c, SUC_lexicon_tags_7 },
    { 0x5d0e9e1d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x448e9e1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60e9e24, SUC_lexicon_tags_7 },
    { 0x196e9e25, SUC_lexicon_tags_25 },
    { 0xccce9e26, SUC_lexicon_tags_43 },
    { 0xdd2e9e27, SUC_lexicon_tags_14 },
    { 0x204e9e27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d8e9e2a, SUC_lexicon_tags_16 },
    { 0xff0e9e2b, SUC_lexicon_tags_30 },
    { 0xb34e9e2c, SUC_lexicon_tags_32 },
    { 0xab6e9e2d, SUC_lexicon_tags_12 },
    { 0x8e9e2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342e9e32, SUC_lexicon_tags_37 },
    { 0x150e9e33, SUC_lexicon_tags_63 },
    { 0x186e9e34, SUC_lexicon_tags_14 },
    { 0xa64e9e35, SUC_lexicon_tags_7 },
    { 0xedee9e32, SUC_lexicon_tags_11 },
    { 0x6b2e9e37, SUC_lexicon_tags_24 },
    { 0x204e9e34, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xfc4e9e3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636e9e40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38e9e42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fee9e45, SUC_lexicon_tags_15 },
    { 0x2fee9e45, SUC_lexicon_tags_30 },
    { 0xb2ae9e45, SUC_lexicon_tags_26 },
    { 0x7aee9e48, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ae9e50, SUC_lexicon_tags_2 },
    { 0x8cae9e50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6e9e57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6ce9e59, SUC_lexicon_tags_34 },
    { 0x9d2e9e59, SUC_lexicon_tags_25 },
    { 0x440e9e5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x462e9e5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x742e9e61, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x280e9e63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4e9e67, SUC_lexicon_tags_65 },
    { 0x60ce9e68, SUC_lexicon_tags_65 },
    { 0x2c6e9e67, SUC_lexicon_tags_25 },
    { 0x61ce9e6a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6e9e6e, SUC_lexicon_tags_25 },
    { 0xdeae9e6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d6e9e71, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bee9e76, SUC_lexicon_tags_31 },
    { 0xdeee9e77, SUC_lexicon_tags_19 },
    { 0xbf8e9e78, SUC_lexicon_tags_47 },
    { 0x20ce9e78, SUC_lexicon_tags_29 },
    { 0x444e9e77, SUC_lexicon_tags_25 },
    { 0x5b8e9e7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1aee9e7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x710e9e7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddee9e87, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ae9e8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bae9e93, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1f6e9e95, SUC_lexicon_tags_11 },
    { 0x534e9e96, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xce6e9e98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f4e9e9a, SUC_lexicon_tags_14 },
    { 0xb34e9e9b, SUC_lexicon_tags_7 },
    { 0x766e9e9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86e9e9e, SUC_lexicon_tags_11 },
    { 0x9e6e9e9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdae9ea3, SUC_lexicon_tags_17 },
    { 0x104e9ea4, SUC_lexicon_tags_0 },
    { 0x78ae9ea5, SUC_lexicon_tags_12 },
    { 0xd4ae9ea5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74e9ead, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d6e9eaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ae9eb2, SUC_lexicon_tags_44 },
    { 0x5bee9eb3, SUC_lexicon_tags_19 },
    { 0xfc6e9eb4, SUC_lexicon_tags_2 },
    { 0x66ae9eb5, SUC_lexicon_tags_7 },
    { 0x41ee9eb3, SUC_lexicon_tags_33 },
    { 0xf88e9eb5, SUC_lexicon_tags_71 },
    { 0xfdee9eb2, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xa3ee9eba, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xb62e9ebc, SUC_lexicon_tags_14 },
    { 0x824e9ebc, SUC_lexicon_tags_66 },
    { 0xe46e9ebe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68e9ecd, SUC_lexicon_tags_29 },
    { 0x870e9ecd, SUC_lexicon_tags_49 },
    { 0x7b6e9ecf, SUC_lexicon_tags_25 },
    { 0x5cee9ed0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ce9ed4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa92e9ed6, SUC_lexicon_tags_37 },
    { 0x7eee9ed6, SUC_lexicon_tags_19 },
    { 0xaa8e9ed6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104e9edc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0e9ee0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x624e9ee2, SUC_lexicon_tags_12 },
    { 0xf7ce9ee3, SUC_lexicon_tags_198 },
    { 0x9fae9ee4, SUC_lexicon_tags_29 },
    { 0x3c0e9ee5, SUC_lexicon_tags_47 },
    { 0xa5ce9ee6, SUC_lexicon_tags_38 },
    { 0x170e9ee7, SUC_lexicon_tags_36 },
    { 0xbc0e9ee5, SUC_lexicon_tags_9 },
    { 0x2e0e9ee9, SUC_lexicon_tags_10 },
    { 0x9b6e9eea, SUC_lexicon_tags_25 },
    { 0x3aee9eeb, SUC_lexicon_tags_2 },
    { 0x37ce9eec, SUC_lexicon_tags_31 },
    { 0x466e9ee9, SUC_lexicon_tags_37 },
    { 0x384e9eee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8e9ef2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2e9ef7, SUC_lexicon_tags_60 },
    { 0xfc8e9ef7, SUC_lexicon_tags_6 },
    { 0xb2e9ef9, SUC_lexicon_tags_25 },
    { 0xc9ae9ef9, SUC_lexicon_tags_29 },
    { 0x4fce9efb, SUC_lexicon_tags_17 },
    { 0x8e0e9efb, SUC_lexicon_tags_25 },
    { 0x24ce9efd, SUC_lexicon_tags_282 },
    { 0x66e9efd, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ce9f03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x622e9f06, SUC_lexicon_tags_12 },
    { 0xc2ae9f07, SUC_lexicon_tags_17 },
    { 0xd66e9f08, SUC_lexicon_tags_12 },
    { 0x4ace9f06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ce9f12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fae9f15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2e9f18, SUC_lexicon_tags_7 },
    { 0x1f0e9f18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bae9f1f, SUC_lexicon_tags_9 },
    { 0x33ee9f1f, SUC_lexicon_tags_38 },
    { 0xd50e9f20, SUC_lexicon_tags_14 },
    { 0xb12e9f21, SUC_lexicon_tags_63 },
    { 0xcc2e9f22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692e9f2d, SUC_lexicon_tags_28 },
    { 0xd12e9f2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x598e9f31, SUC_lexicon_tags_30 },
    { 0xcdae9f32, SUC_lexicon_tags_7 },
    { 0x7eee9f33, SUC_lexicon_tags_7 },
    { 0xdeae9f32, SUC_lexicon_tags_7 },
    { 0xbeee9f35, SUC_lexicon_tags_10 },
    { 0xc82e9f36, SUC_lexicon_tags_41 },
    { 0x45ce9f31, SUC_lexicon_tags_25 },
    { 0x3b6e9f38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70e9f3b, SUC_lexicon_tags_15 },
    { 0xa9ce9f3b, SUC_lexicon_tags_32 },
    { 0x428e9f3d, SUC_lexicon_tags_28 },
    { 0x94ce9f3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x824e9f40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eee9f43, SUC_lexicon_tags_84 },
    { 0xb96e9f44, SUC_lexicon_tags_25 },
    { 0x91ee9f44, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4e9f48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x802e9f4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c4e9f4c, SUC_lexicon_tags_19 },
    { 0x1a2e9f4c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x1e8e9f4f, SUC_lexicon_tags_15 },
    { 0x34ae9f50, SUC_lexicon_tags_43 },
    { 0x1f2e9f50, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ae9f55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ce9f59, SUC_lexicon_tags_14 },
    { 0x10ce9f5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0e9f5d, SUC_lexicon_tags_2 },
    { 0x116e9f5e, SUC_lexicon_tags_12 },
    { 0x5c0e9f5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96e9f65, SUC_lexicon_tags_49 },
    { 0x4c2e9f65, SUC_lexicon_tags_55 },
    { 0x95ee9f66, SUC_lexicon_tags_17 },
    { 0xec6e9f68, SUC_lexicon_tags_437 },
    { 0xb5ee9f68, SUC_lexicon_tags_1 },
    { 0x580e9f6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ce9f6f, SUC_lexicon_tags_34 },
    { 0xc3ce9f70, SUC_lexicon_tags_37 },
    { 0x40e9f71, SUC_lexicon_tags_12 },
    { 0xc48e9f6f, SUC_lexicon_tags_0 },
    { 0x238e9f73, SUC_lexicon_tags_17 },
    { 0x4b4e9f74, SUC_lexicon_tags_27 },
    { 0x4dce9f75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ee9f79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaae9f81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dae9f86, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0e9f89, SUC_lexicon_tags_17 },
    { 0x4d0e9f89, SUC_lexicon_tags_1 },
    { 0xdbee9f8b, SUC_lexicon_tags_2 },
    { 0xd8ae9f8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8e9f8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x962e9f92, SUC_lexicon_tags_2 },
    { 0x1bee9f93, SUC_lexicon_tags_9 },
    { 0x590e9f94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ece9f96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6e9f9b, SUC_lexicon_tags_12 },
    { 0xb12e9f9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16e9f9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x464e9fb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1ee9fb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4e9fba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x142e9fbc, SUC_lexicon_tags_7 },
    { 0x5bee9fbc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8e9fc0, SUC_lexicon_tags_15 },
    { 0xaeae9fc0, SUC_lexicon_tags_11 },
    { 0x868e9fc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898e9fc6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x506e9fc9, SUC_lexicon_tags_3 },
    { 0x9b2e9fca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11ae9fcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x578e9fcf, SUC_lexicon_tags_25 },
    { 0x9c8e9fcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd72e9fd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bce9fd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78e9fd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4e9fdb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x22ae9fdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x128e9fdf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x964e9fe1, SUC_lexicon_tags_38 },
    { 0x8f6e9fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6ae9fe4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5c2e9fe6, SUC_lexicon_tags_48 },
    { 0xc7ee9fe6, SUC_lexicon_tags_33 },
    { 0x628e9fe7, SUC_lexicon_tags_27 },
    { 0x430e9fe9, SUC_lexicon_tags_1 },
    { 0x90ce9fe8, SUC_lexicon_tags_63 },
    { 0xe0ae9feb, SUC_lexicon_tags_9 },
    { 0x2d4e9feb, SUC_lexicon_tags_6 },
    { 0xf62e9fe7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffce9fef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00e9ff2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ae9ff5, SUC_lexicon_tags_11 },
    { 0xd58e9ff6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7bae9ff8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0e9ffc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412ea001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36ea009, SUC_lexicon_tags_1 },
    { 0xb24ea00a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108ea00e, SUC_lexicon_tags_25 },
    { 0xb2eea00f, SUC_lexicon_tags_82 },
    { 0x914ea00e, SUC_lexicon_tags_11 },
    { 0x15eea00e, SUC_lexicon_tags_16 },
    { 0xda4ea012, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x486ea014, SUC_lexicon_tags_31 },
    { 0xb86ea014, SUC_lexicon_tags_19 },
    { 0xdb6ea015, SUC_lexicon_tags_13 },
    { 0x2a8ea017, SUC_lexicon_tags_25 },
    { 0x998ea018, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920ea022, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784ea02c, SUC_lexicon_tags_36 },
    { 0xb16ea02d, SUC_lexicon_tags_4 },
    { 0x16cea02e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aea031, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aea036, SUC_lexicon_tags_30 },
    { 0xbeea037, SUC_lexicon_tags_6 },
    { 0x2c8ea037, SUC_lexicon_tags_33 },
    { 0x290ea039, SUC_lexicon_tags_11 },
    { 0xf62ea039, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ecea03c, SUC_lexicon_tags_15 },
    { 0x4a0ea03c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcea043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18aea047, SUC_lexicon_tags_37 },
    { 0x69cea048, SUC_lexicon_tags_11 },
    { 0xc56ea048, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482ea04e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8ea053, SUC_lexicon_tags_17 },
    { 0x3faea054, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9eea058, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cea05d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x58ea05f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x690ea062, SUC_lexicon_tags_60 },
    { 0xa80ea062, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x21cea065, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6aea067, SUC_lexicon_tags_7 },
    { 0x1aeea068, SUC_lexicon_tags_12 },
    { 0x64ea069, SUC_lexicon_tags_82 },
    { 0x3b2ea069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94ea070, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0ea073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b2ea075, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1aea077, SUC_lexicon_tags_27 },
    { 0x2a8ea078, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56ea07d, SUC_lexicon_tags_7 },
    { 0x38aea07d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00ea082, SUC_lexicon_tags_6 },
    { 0x7b8ea083, SUC_lexicon_tags_49 },
    { 0x19aea082, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92eea086, SUC_lexicon_tags_39 },
    { 0xdaea086, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x320ea08a, SUC_lexicon_tags_107 },
    { 0xfa6ea08b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x57eea08e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x36eea091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x656ea093, SUC_lexicon_tags_60 },
    { 0xe50ea094, SUC_lexicon_tags_43 },
    { 0xc06ea094, SUC_lexicon_tags_31 },
    { 0xebeea093, SUC_lexicon_tags_31 },
    { 0xe38ea097, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0ea09e, SUC_lexicon_tags_12 },
    { 0xa02ea09f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932ea0a9, SUC_lexicon_tags_2 },
    { 0xba4ea0aa, SUC_lexicon_tags_26 },
    { 0x6b4ea0ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x228ea0ad, SUC_lexicon_tags_25 },
    { 0x1d8ea0ad, SUC_lexicon_tags_27 },
    { 0xd48ea0ae, SUC_lexicon_tags_1 },
    { 0xa2aea0ae, SUC_lexicon_tags_15 },
    { 0x432ea0af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510ea0b6, SUC_lexicon_tags_23 },
    { 0x980ea0b7, SUC_lexicon_tags_55 },
    { 0x490ea0b8, SUC_lexicon_tags_6 },
    { 0x52aea0b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aea0bc, SUC_lexicon_tags_47 },
    { 0xe9eea0bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x366ea0c0, SUC_lexicon_tags_7 },
    { 0x436ea0c1, SUC_lexicon_tags_29 },
    { 0x3eeea0c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34eea0c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388ea0d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46ea0d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaea0d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x418ea0d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc28ea0db, SUC_lexicon_tags_43 },
    { 0xe0ea0dc, SUC_lexicon_tags_1 },
    { 0xd32ea0dd, SUC_lexicon_tags_23 },
    { 0xaeaea0dc, SUC_lexicon_tags_11 },
    { 0x300ea0dd, SUC_lexicon_tags_15 },
    { 0x11aea0dc, SUC_lexicon_tags_23 },
    { 0xd66ea0e1, SUC_lexicon_tags_37 },
    { 0xe96ea0e1, SUC_lexicon_tags_15 },
    { 0x898ea0e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaea0e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x820ea0e8, SUC_lexicon_tags_37 },
    { 0x400ea0e8, SUC_lexicon_tags_178 },
    { 0x3ecea0ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18ea0f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x622ea0f5, SUC_lexicon_tags_8 },
    { 0x762ea0f5, SUC_lexicon_tags_8 },
    { 0xdfcea0f7, SUC_lexicon_tags_32 },
    { 0x42ea0f7, SUC_lexicon_tags_55 },
    { 0xcb0ea0f9, SUC_lexicon_tags_12 },
    { 0x206ea0fa, SUC_lexicon_tags_15 },
    { 0x456ea0fb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42ea0fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cea101, SUC_lexicon_tags_27 },
    { 0xdcaea101, SUC_lexicon_tags_15 },
    { 0xed6ea102, SUC_lexicon_tags_27 },
    { 0x9a6ea104, SUC_lexicon_tags_49 },
    { 0xcc0ea105, SUC_lexicon_tags_30 },
    { 0x370ea104, SUC_lexicon_tags_11 },
    { 0xff8ea106, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x248ea10a, SUC_lexicon_tags_37 },
    { 0x2c0ea10a, SUC_lexicon_tags_25 },
    { 0xcb8ea10a, SUC_lexicon_tags_87 },
    { 0xb34ea10a, SUC_lexicon_tags_11 },
    { 0x98eea10e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc2ea110, SUC_lexicon_tags_15 },
    { 0x24ea111, SUC_lexicon_tags_49 },
    { 0x964ea111, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x186ea114, SUC_lexicon_tags_1 },
    { 0xabcea114, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x998ea118, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2ea11c, SUC_lexicon_tags_2 },
    { 0x916ea11d, SUC_lexicon_tags_31 },
    { 0xd32ea11c, SUC_lexicon_tags_1 },
    { 0x1cea11e, SUC_lexicon_tags_17 },
    { 0xd5eea120, SUC_lexicon_tags_49 },
    { 0xe42ea121, SUC_lexicon_tags_77 },
    { 0x254ea122, SUC_lexicon_tags_23 },
    { 0xb6eea122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50ea128, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x794ea12b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14ea134, SUC_lexicon_tags_60 },
    { 0x52aea135, SUC_lexicon_tags_1 },
    { 0x54eea136, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2ea13a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3eea13c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb08ea13e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54ea14c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64ea151, SUC_lexicon_tags_26 },
    { 0x95aea152, SUC_lexicon_tags_7 },
    { 0x4aaea153, SUC_lexicon_tags_1 },
    { 0x1c8ea153, SUC_lexicon_tags_27 },
    { 0xbbcea151, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e0ea157, SUC_lexicon_tags_29 },
    { 0x4b0ea157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x944ea15b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726ea15f, SUC_lexicon_tags_1 },
    { 0x5a2ea160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416ea165, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06ea171, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26ea176, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf30ea178, SUC_lexicon_tags_38 },
    { 0x616ea179, SUC_lexicon_tags_31 },
    { 0x18cea179, SUC_lexicon_tags_1 },
    { 0xdf4ea17b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aea17e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00ea188, SUC_lexicon_tags_45 },
    { 0xe30ea188, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe68ea18c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x466ea18f, SUC_lexicon_tags_25 },
    { 0xdbcea18f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2ea193, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cea19a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x500ea19c, SUC_lexicon_tags_11 },
    { 0x5eeea19c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2ea1a2, SUC_lexicon_tags_26 },
    { 0x4a2ea1a3, SUC_lexicon_tags_14 },
    { 0x276ea1a2, SUC_lexicon_tags_19 },
    { 0xbf0ea1a3, SUC_lexicon_tags_15 },
    { 0x7d8ea1a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x236ea1a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46ea1ab, SUC_lexicon_tags_14 },
    { 0x796ea1ac, SUC_lexicon_tags_30 },
    { 0xab6ea1ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98ea1b2, SUC_lexicon_tags_27 },
    { 0x3b0ea1b2, SUC_lexicon_tags_14 },
    { 0xcd4ea1b3, SUC_lexicon_tags_23 },
    { 0xe68ea1b3, SUC_lexicon_tags_17 },
    { 0x27eea1b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf54ea1b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcea1bb, SUC_lexicon_tags_47 },
    { 0xfaea1bb, SUC_lexicon_tags_47 },
    { 0x902ea1bd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aea1c1, SUC_lexicon_tags_8 },
    { 0xd22ea1c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52cea1c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb42ea1ca, SUC_lexicon_tags_29 },
    { 0x55cea1cb, SUC_lexicon_tags_7 },
    { 0x6a2ea1cb, SUC_lexicon_tags_4 },
    { 0x868ea1cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2faea1cf, SUC_lexicon_tags_21 },
    { 0x706ea1d0, SUC_lexicon_tags_28 },
    { 0xb12ea1d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdeea1d3, SUC_lexicon_tags_33 },
    { 0xb0ea1d4, SUC_lexicon_tags_28 },
    { 0x4eaea1d5, SUC_lexicon_tags_31 },
    { 0xb2aea1d3, SUC_lexicon_tags_29 },
    { 0x932ea1d7, SUC_lexicon_tags_15 },
    { 0x8d4ea1d8, SUC_lexicon_tags_25 },
    { 0x86ea1d6, SUC_lexicon_tags_37 },
    { 0x1f8ea1da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaea1df, SUC_lexicon_tags_61 },
    { 0x60aea1df, SUC_lexicon_tags_29 },
    { 0x716ea1df, SUC_lexicon_tags_84 },
    { 0xd24ea1e2, SUC_lexicon_tags_1 },
    { 0x120ea1e3, SUC_lexicon_tags_29 },
    { 0x7d6ea1e4, SUC_lexicon_tags_2 },
    { 0x646ea1e0, SUC_lexicon_tags_17 },
    { 0x56aea1e6, SUC_lexicon_tags_15 },
    { 0x20aea1e2, SUC_lexicon_tags_5 },
    { 0x478ea1e8, SUC_lexicon_tags_25 },
    { 0x40ea1e9, SUC_lexicon_tags_4 },
    { 0x1feea1e8, SUC_lexicon_tags_15 },
    { 0xf96ea1eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972ea1f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecea1f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf48ea1f6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aea1f9, SUC_lexicon_tags_60 },
    { 0x8b2ea1fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcea1fd, SUC_lexicon_tags_2 },
    { 0x170ea1fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x786ea200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0aea206, SUC_lexicon_tags_15 },
    { 0x8aeea207, SUC_lexicon_tags_7 },
    { 0x5f2ea208, SUC_lexicon_tags_37 },
    { 0x11eea208, SUC_lexicon_tags_12 },
    { 0x16ea209, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbe4ea20c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78eea210, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcea215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc10ea217, SUC_lexicon_tags_14 },
    { 0x1c0ea218, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x266ea21a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ea21d, SUC_lexicon_tags_43 },
    { 0x7f2ea21e, SUC_lexicon_tags_11 },
    { 0x7a2ea21e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80ea224, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96ea22c, SUC_lexicon_tags_17 },
    { 0xdbcea22d, SUC_lexicon_tags_42 },
    { 0xaaea22e, SUC_lexicon_tags_38 },
    { 0x7d6ea22d, SUC_lexicon_tags_6 },
    { 0x582ea230, SUC_lexicon_tags_7 },
    { 0xcb2ea22e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72aea235, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4ea23c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58ea23f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182ea244, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaea247, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5aea24e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6ea252, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8ea25d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x32eea25f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44eea261, SUC_lexicon_tags_11 },
    { 0xf50ea261, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76ea268, SUC_lexicon_tags_17 },
    { 0xa98ea269, SUC_lexicon_tags_87 },
    { 0xe5cea26a, SUC_lexicon_tags_37 },
    { 0x1d4ea26b, SUC_lexicon_tags_12 },
    { 0x432ea26c, SUC_lexicon_tags_7 },
    { 0xba4ea26b, SUC_lexicon_tags_23 },
    { 0xa9cea26e, SUC_lexicon_tags_25 },
    { 0x1a4ea26f, SUC_lexicon_tags_17 },
    { 0x436ea270, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aea273, SUC_lexicon_tags_33 },
    { 0xa4eea273, SUC_lexicon_tags_140 },
    { 0x960ea274, SUC_lexicon_tags_13 },
    { 0x7d4ea275, SUC_lexicon_tags_33 },
    { 0xceeea277, SUC_lexicon_tags_11 },
    { 0x2aea276, SUC_lexicon_tags_10 },
    { 0xeb2ea279, SUC_lexicon_tags_37 },
    { 0x240ea27a, SUC_lexicon_tags_15 },
    { 0xd62ea279, SUC_lexicon_tags_55 },
    { 0xe8eea277, SUC_lexicon_tags_38 },
    { 0x628ea27d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x79aea280, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0x9f6ea282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cea286, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260ea28a, SUC_lexicon_tags_12 },
    { 0x27aea28b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10aea291, SUC_lexicon_tags_28 },
    { 0xadeea292, SUC_lexicon_tags_14 },
    { 0x86ea291, SUC_lexicon_tags_85 },
    { 0x4b6ea294, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb06ea296, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7e0ea298, SUC_lexicon_tags_1 },
    { 0xfdcea299, SUC_lexicon_tags_15 },
    { 0xf72ea299, SUC_lexicon_tags_11 },
    { 0xdacea29b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98ea29d, SUC_lexicon_tags_37 },
    { 0xea4ea29e, SUC_lexicon_tags_12 },
    { 0x6b2ea29f, SUC_lexicon_tags_7 },
    { 0x360ea29d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff4ea2a2, SUC_lexicon_tags_9 },
    { 0xd84ea2a3, SUC_lexicon_tags_33 },
    { 0xbc6ea2a4, SUC_lexicon_tags_55 },
    { 0x3f2ea2a5, SUC_lexicon_tags_14 },
    { 0xfbeea2a6, SUC_lexicon_tags_17 },
    { 0xe96ea2a5, SUC_lexicon_tags_65 },
    { 0xd74ea2a3, SUC_lexicon_tags_46 },
    { 0x518ea2a9, SUC_lexicon_tags_27 },
    { 0xa3eea2aa, SUC_lexicon_tags_2 },
    { 0x7eaea2a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31aea2ad, SUC_lexicon_tags_7 },
    { 0xb9eea2ae, SUC_lexicon_tags_37 },
    { 0x37aea2ae, SUC_lexicon_tags_12 },
    { 0xb2cea2ae, SUC_lexicon_tags_17 },
    { 0x1aea2b1, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ea2b4, SUC_lexicon_tags_25 },
    { 0x300ea2b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd40ea2b7, SUC_lexicon_tags_69 },
    { 0x7d4ea2b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2b4ea2ba, SUC_lexicon_tags_25 },
    { 0xa1cea2ba, SUC_lexicon_tags_31 },
    { 0x12aea2bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eea2bf, SUC_lexicon_tags_37 },
    { 0x5b0ea2c0, SUC_lexicon_tags_62 },
    { 0xf58ea2c1, SUC_lexicon_tags_82 },
    { 0xa4eea2c2, SUC_lexicon_tags_27 },
    { 0x686ea2c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aea2c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ea2cc, SUC_lexicon_tags_42 },
    { 0xd7eea2cc, SUC_lexicon_tags_13 },
    { 0xde8ea2cd, SUC_lexicon_tags_17 },
    { 0x1deea2cd, SUC_lexicon_tags_15 },
    { 0xb6ea2ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa18ea2d3, SUC_lexicon_tags_19 },
    { 0xe10ea2d4, SUC_lexicon_tags_3 },
    { 0x6daea2d4, SUC_lexicon_tags_37 },
    { 0xfc8ea2d3, SUC_lexicon_tags_8 },
    { 0x98eea2d7, SUC_lexicon_tags_33 },
    { 0x716ea2d8, SUC_lexicon_tags_31 },
    { 0xedeea2d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334ea2dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b2ea2df, SUC_lexicon_tags_68 },
    { 0x92eea2e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8ea2e3, SUC_lexicon_tags_37 },
    { 0x7f4ea2e4, SUC_lexicon_tags_26 },
    { 0xe92ea2e3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ea2e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2ea2ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844ea2f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49eea2ff, SUC_lexicon_tags_29 },
    { 0x270ea300, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34ea304, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cea307, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa0ea309, SUC_lexicon_tags_43 },
    { 0xe48ea30a, SUC_lexicon_tags_37 },
    { 0xa22ea30b, SUC_lexicon_tags_37 },
    { 0x4baea30b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920ea310, SUC_lexicon_tags_17 },
    { 0x946ea311, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8ea315, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2baea317, SUC_lexicon_tags_30 },
    { 0x3e4ea318, SUC_lexicon_tags_1 },
    { 0x688ea319, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x714ea31c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x54ea31e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a4ea320, SUC_lexicon_tags_48 },
    { 0xab4ea321, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd04ea324, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cea32b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98ea330, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x728ea332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcea335, SUC_lexicon_tags_87 },
    { 0xc8aea336, SUC_lexicon_tags_23 },
    { 0xf66ea337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4ea33a, SUC_lexicon_tags_62 },
    { 0x36cea33a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48ea33f, SUC_lexicon_tags_25 },
    { 0xb00ea340, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ea343, SUC_lexicon_tags_19 },
    { 0x730ea344, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cea349, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cea351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4ea355, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x104ea357, SUC_lexicon_tags_43 },
    { 0x73aea358, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0ea35b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x74eea35d, SUC_lexicon_tags_4 },
    { 0x86eea35e, SUC_lexicon_tags_37 },
    { 0xefcea35f, SUC_lexicon_tags_12 },
    { 0x33cea35f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4eea363, SUC_lexicon_tags_8 },
    { 0x912ea364, SUC_lexicon_tags_87 },
    { 0x4acea365, SUC_lexicon_tags_12 },
    { 0x4fcea365, SUC_lexicon_tags_37 },
    { 0x6c4ea367, SUC_lexicon_tags_33 },
    { 0x44cea368, SUC_lexicon_tags_37 },
    { 0xf14ea369, SUC_lexicon_tags_2 },
    { 0x51cea36a, SUC_lexicon_tags_30 },
    { 0x954ea365, SUC_lexicon_tags_60 },
    { 0xf08ea36c, SUC_lexicon_tags_38 },
    { 0x232ea367, SUC_lexicon_tags_17 },
    { 0xcc6ea366, SUC_lexicon_tags_43 },
    { 0x96eea365, SUC_lexicon_tags_11 },
    { 0x93eea370, SUC_lexicon_tags_15 },
    { 0x780ea366, SUC_lexicon_tags_0 },
    { 0x70aea371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2eea375, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12ea378, SUC_lexicon_tags_32 },
    { 0x41aea379, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414ea383, SUC_lexicon_tags_30 },
    { 0x5b8ea384, SUC_lexicon_tags_47 },
    { 0xb6ea383, SUC_lexicon_tags_34 },
    { 0x15aea385, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0ea389, SUC_lexicon_tags_65 },
    { 0x604ea389, SUC_lexicon_tags_2 },
    { 0x5e2ea38b, SUC_lexicon_tags_16 },
    { 0x3eea38a, SUC_lexicon_tags_14 },
    { 0x5f4ea38d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8beea38f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36cea391, SUC_lexicon_tags_1 },
    { 0x50ea392, SUC_lexicon_tags_1 },
    { 0x90aea393, SUC_lexicon_tags_13 },
    { 0x2d0ea393, SUC_lexicon_tags_62 },
    { 0xdceea392, SUC_lexicon_tags_151 },
    { 0xd72ea393, SUC_lexicon_tags_28 },
    { 0x42eea393, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x214ea39a, SUC_lexicon_tags_11 },
    { 0xfaeea39b, SUC_lexicon_tags_15 },
    { 0x9b8ea39c, SUC_lexicon_tags_1 },
    { 0x372ea39b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaaaea39f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4ea3a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e0ea3a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38eea3a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x430ea3ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988ea3af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eea3b2, SUC_lexicon_tags_37 },
    { 0xca8ea3b3, SUC_lexicon_tags_1 },
    { 0x61eea3b3, SUC_lexicon_tags_68 },
    { 0xc56ea3b3, SUC_lexicon_tags_82 },
    { 0x584ea3b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2ea3bb, SUC_lexicon_tags_15 },
    { 0x26ea3bc, SUC_lexicon_tags_66 },
    { 0x300ea3bd, SUC_lexicon_tags_1506 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2ea3c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ea3c8, SUC_lexicon_tags_8 },
    { 0xf64ea3c9, SUC_lexicon_tags_53 },
    { 0x754ea3c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422ea3cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cea3d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x58ea3d7, SUC_lexicon_tags_15 },
    { 0x4f6ea3d8, SUC_lexicon_tags_60 },
    { 0xc80ea3d7, SUC_lexicon_tags_17 },
    { 0x3c6ea3da, SUC_lexicon_tags_11 },
    { 0x30aea3da, SUC_lexicon_tags_84 },
    { 0x49aea3db, SUC_lexicon_tags_33 },
    { 0xd9cea3da, SUC_lexicon_tags_15 },
    { 0x136ea3de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4ea3e2, SUC_lexicon_tags_25 },
    { 0x896ea3e3, SUC_lexicon_tags_11 },
    { 0xf70ea3e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3feea3e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6ea3e9, SUC_lexicon_tags_31 },
    { 0xa4cea3ea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994ea3ee, SUC_lexicon_tags_12 },
    { 0xed4ea3ee, SUC_lexicon_tags_61 },
    { 0xf4ea3f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62ea3f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec4ea3f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf6ea3fa, SUC_lexicon_tags_17 },
    { 0x5aea3fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd46ea400, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aeea403, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x546ea405, SUC_lexicon_tags_2 },
    { 0x37cea406, SUC_lexicon_tags_25 },
    { 0x10ea407, SUC_lexicon_tags_25 },
    { 0xe68ea408, SUC_lexicon_tags_37 },
    { 0x26eea409, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1caea40b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x972ea40d, SUC_lexicon_tags_7 },
    { 0xa02ea40e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x118ea410, SUC_lexicon_tags_15 },
    { 0x824ea411, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89aea415, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02ea418, SUC_lexicon_tags_46 },
    { 0x6c2ea419, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ceea41b, SUC_lexicon_tags_7 },
    { 0x5a4ea41c, SUC_lexicon_tags_49 },
    { 0x934ea41b, SUC_lexicon_tags_63 },
    { 0x386ea41c, SUC_lexicon_tags_37 },
    { 0x1a8ea41d, SUC_lexicon_tags_33 },
    { 0xedaea41d, SUC_lexicon_tags_37 },
    { 0x4a8ea41e, SUC_lexicon_tags_37 },
    { 0x536ea41b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xefeea424, SUC_lexicon_tags_47 },
    { 0xefeea424, SUC_lexicon_tags_46 },
    { 0x15eea424, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xcd6ea428, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa12ea42a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4ea42f, SUC_lexicon_tags_34 },
    { 0xd1cea42f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2ea434, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48ea43a, SUC_lexicon_tags_7 },
    { 0x5faea43a, SUC_lexicon_tags_136 },
    { 0xd9aea43c, SUC_lexicon_tags_60 },
    { 0xceea43c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aea440, SUC_lexicon_tags_2 },
    { 0x90aea440, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cea445, SUC_lexicon_tags_37 },
    { 0x8a2ea446, SUC_lexicon_tags_17 },
    { 0x932ea446, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89aea44a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0ea44d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x526ea450, SUC_lexicon_tags_37 },
    { 0xc86ea451, SUC_lexicon_tags_0 },
    { 0x532ea451, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638ea456, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600ea45c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcea465, SUC_lexicon_tags_91 },
    { 0x7c4ea465, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaea46a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaea46d, SUC_lexicon_tags_11 },
    { 0xad0ea46e, SUC_lexicon_tags_26 },
    { 0x68aea46d, SUC_lexicon_tags_6 },
    { 0x9e8ea470, SUC_lexicon_tags_48 },
    { 0x588ea46d, SUC_lexicon_tags_9 },
    { 0xdcea46d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2a8ea474, SUC_lexicon_tags_7 },
    { 0xd84ea475, SUC_lexicon_tags_11 },
    { 0x22ea476, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26cea478, SUC_lexicon_tags_19 },
    { 0xf6aea479, SUC_lexicon_tags_87 },
    { 0xc38ea47a, SUC_lexicon_tags_95 },
    { 0x166ea47b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2ea483, SUC_lexicon_tags_14 },
    { 0x72aea484, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x226ea486, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82ea489, SUC_lexicon_tags_45 },
    { 0x77aea48a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccea48d, SUC_lexicon_tags_12 },
    { 0x2c6ea48d, SUC_lexicon_tags_63 },
    { 0x540ea48f, SUC_lexicon_tags_141 },
    { 0x7a6ea490, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe20ea492, SUC_lexicon_tags_74 },
    { 0x320ea493, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4ea496, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4ea498, SUC_lexicon_tags_11 },
    { 0xc3cea498, SUC_lexicon_tags_11 },
    { 0xae8ea49a, SUC_lexicon_tags_25 },
    { 0x572ea498, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baea4a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x420ea4a3, SUC_lexicon_tags_6 },
    { 0x3daea4a3, SUC_lexicon_tags_60 },
    { 0x244ea4a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7eea4ab, SUC_lexicon_tags_17 },
    { 0x5feea4ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x872ea4ae, SUC_lexicon_tags_11 },
    { 0xd58ea4af, SUC_lexicon_tags_32 },
    { 0xe60ea4b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xaa8ea4b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0ea4b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74ea4ba, SUC_lexicon_tags_29 },
    { 0x142ea4bb, SUC_lexicon_tags_27 },
    { 0xf3eea4bb, SUC_lexicon_tags_1 },
    { 0x3feea4bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa2eea4bf, SUC_lexicon_tags_37 },
    { 0xa96ea4c0, SUC_lexicon_tags_15 },
    { 0x32ea4c0, SUC_lexicon_tags_27 },
    { 0xc1aea4c0, SUC_lexicon_tags_1 },
    { 0x624ea4c0, SUC_lexicon_tags_17 },
    { 0x1ecea4c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72eea4c9, SUC_lexicon_tags_8 },
    { 0x794ea4ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x36cea4cd, SUC_lexicon_tags_25 },
    { 0x4eaea4ce, SUC_lexicon_tags_224 },
    { 0x608ea4cf, SUC_lexicon_tags_32 },
    { 0x314ea4cf, SUC_lexicon_tags_15 },
    { 0xba0ea4d1, SUC_lexicon_tags_5 },
    { 0xd0eea4d1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50aea4d6, SUC_lexicon_tags_21 },
    { 0x9b8ea4d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0ea4da, SUC_lexicon_tags_1 },
    { 0x116ea4db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceea4e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26cea4e7, SUC_lexicon_tags_39 },
    { 0x822ea4e8, SUC_lexicon_tags_12 },
    { 0xa52ea4e8, SUC_lexicon_tags_17 },
    { 0xa66ea4e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86ea4ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeea4f2, SUC_lexicon_tags_37 },
    { 0xfc8ea4f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaea4f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b6ea4f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ea4fb, SUC_lexicon_tags_37 },
    { 0x35eea4fb, SUC_lexicon_tags_11 },
    { 0xc7eea4fd, SUC_lexicon_tags_25 },
    { 0x122ea4fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5faea501, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fcea503, SUC_lexicon_tags_1 },
    { 0xdd4ea504, SUC_lexicon_tags_26 },
    { 0xd0ea504, SUC_lexicon_tags_23 },
    { 0xc74ea505, SUC_lexicon_tags_17 },
    { 0xa3cea503, SUC_lexicon_tags_2 },
    { 0x602ea508, SUC_lexicon_tags_45 },
    { 0xd14ea506, SUC_lexicon_tags_29 },
    { 0xd42ea50a, SUC_lexicon_tags_30 },
    { 0xff6ea50a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6eea50d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6aea510, SUC_lexicon_tags_12 },
    { 0x5baea510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeceea516, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22ea51b, SUC_lexicon_tags_25 },
    { 0xa32ea51c, SUC_lexicon_tags_47 },
    { 0x8a6ea51c, SUC_lexicon_tags_19 },
    { 0xc5aea51c, SUC_lexicon_tags_1 },
    { 0x73aea51d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe14ea521, SUC_lexicon_tags_11 },
    { 0x83cea521, SUC_lexicon_tags_17 },
    { 0x53aea523, SUC_lexicon_tags_5 },
    { 0x7beea524, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4ea528, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cea52c, SUC_lexicon_tags_11 },
    { 0x22aea52d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4ea532, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10ea535, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x922ea537, SUC_lexicon_tags_7 },
    { 0x502ea537, SUC_lexicon_tags_12 },
    { 0xa2cea537, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448ea540, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08ea545, SUC_lexicon_tags_47 },
    { 0x212ea546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8ea54c, SUC_lexicon_tags_30 },
    { 0x5c2ea54d, SUC_lexicon_tags_19 },
    { 0xe0ea54d, SUC_lexicon_tags_23 },
    { 0x540ea54f, SUC_lexicon_tags_25 },
    { 0xd8aea550, SUC_lexicon_tags_48 },
    { 0x494ea54f, SUC_lexicon_tags_19 },
    { 0x2ecea552, SUC_lexicon_tags_11 },
    { 0x8cea553, SUC_lexicon_tags_45 },
    { 0xbeaea554, SUC_lexicon_tags_15 },
    { 0x604ea553, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xfbeea557, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cea55a, SUC_lexicon_tags_27 },
    { 0xf32ea55b, SUC_lexicon_tags_8 },
    { 0x3eea55b, SUC_lexicon_tags_12 },
    { 0x28ea55a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f4ea55f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40ea562, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804ea568, SUC_lexicon_tags_59 },
    { 0xa42ea569, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed6ea56b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x47cea56d, SUC_lexicon_tags_1 },
    { 0x7c4ea56d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ea571, SUC_lexicon_tags_47 },
    { 0x760ea571, SUC_lexicon_tags_34 },
    { 0x928ea572, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694ea57b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6aea57e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39cea580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc40ea582, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcecea58a, SUC_lexicon_tags_1 },
    { 0xe94ea58b, SUC_lexicon_tags_0 },
    { 0xa06ea58c, SUC_lexicon_tags_11 },
    { 0x7c8ea58d, SUC_lexicon_tags_25 },
    { 0xaf4ea58d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18ea594, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daea59b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbdeea59d, SUC_lexicon_tags_17 },
    { 0x620ea59e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5baea5a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360ea5a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8ea5ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8f8ea5ae, SUC_lexicon_tags_37 },
    { 0x30eea5af, SUC_lexicon_tags_7 },
    { 0x7beea5af, SUC_lexicon_tags_17 },
    { 0x5a2ea5ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22ea5b7, SUC_lexicon_tags_17 },
    { 0xc36ea5b8, SUC_lexicon_tags_0 },
    { 0x778ea5b7, SUC_lexicon_tags_25 },
    { 0x37cea5b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8ea5bf, SUC_lexicon_tags_130 },
    { 0x3eaea5c0, SUC_lexicon_tags_15 },
    { 0xe46ea5c1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecea5c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaea5c8, SUC_lexicon_tags_7 },
    { 0x52cea5c9, SUC_lexicon_tags_29 },
    { 0x602ea5ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2ea5d6, SUC_lexicon_tags_87 },
    { 0xb8ea5d6, SUC_lexicon_tags_65 },
    { 0x62aea5d8, SUC_lexicon_tags_28 },
    { 0x2d2ea5d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0cea5db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x152ea5dd, SUC_lexicon_tags_7 },
    { 0x130ea5de, SUC_lexicon_tags_15 },
    { 0x7dcea5dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8daea5e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e2ea5e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904ea5ed, SUC_lexicon_tags_63 },
    { 0xebcea5ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90aea5f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320ea5fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x86ea5fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x762ea5fe, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xf1aea600, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6ea603, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65cea605, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2c4ea607, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6aaea609, SUC_lexicon_tags_64 },
    { 0x926ea609, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcea60f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4cea611, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6ea619, SUC_lexicon_tags_15 },
    { 0x7bcea61a, SUC_lexicon_tags_17 },
    { 0x2faea61b, SUC_lexicon_tags_6 },
    { 0x9aeea61b, SUC_lexicon_tags_1 },
    { 0xd34ea619, SUC_lexicon_tags_2 },
    { 0xaecea61e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2ea622, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4aea624, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998ea629, SUC_lexicon_tags_49 },
    { 0xe3cea62a, SUC_lexicon_tags_12 },
    { 0x916ea62a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502ea62f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7eea634, SUC_lexicon_tags_11 },
    { 0x6deea634, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6cea637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaea63b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x56cea63d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daea645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912ea649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14ea64d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cea653, SUC_lexicon_tags_1 },
    { 0xeecea654, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6ea657, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe96ea659, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeea65d, SUC_lexicon_tags_2 },
    { 0x358ea65e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718ea665, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2cea667, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ea66b, SUC_lexicon_tags_0 },
    { 0xda4ea66c, SUC_lexicon_tags_16 },
    { 0x180ea66d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66eea674, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5d2ea676, SUC_lexicon_tags_2 },
    { 0x5e4ea676, SUC_lexicon_tags_60 },
    { 0x958ea678, SUC_lexicon_tags_37 },
    { 0x5f6ea678, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x340ea67b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6ea683, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf2ea685, SUC_lexicon_tags_2 },
    { 0x74ea686, SUC_lexicon_tags_33 },
    { 0x522ea686, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x82eea68a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cea68e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28ea694, SUC_lexicon_tags_7 },
    { 0x9beea694, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ecea697, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aea69b, SUC_lexicon_tags_38 },
    { 0xd84ea69b, SUC_lexicon_tags_2 },
    { 0x8feea69b, SUC_lexicon_tags_33 },
    { 0x70cea69e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60ea6a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508ea6a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4aea6a8, SUC_lexicon_tags_11 },
    { 0xc46ea6a8, SUC_lexicon_tags_47 },
    { 0x2b2ea6aa, SUC_lexicon_tags_66 },
    { 0x69cea6ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49aea6b1, SUC_lexicon_tags_33 },
    { 0xd20ea6b2, SUC_lexicon_tags_9 },
    { 0xf18ea6b1, SUC_lexicon_tags_12 },
    { 0x704ea6b4, SUC_lexicon_tags_13 },
    { 0xdb2ea6b5, SUC_lexicon_tags_77 },
    { 0x408ea6b5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06ea6bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda0ea6bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2ea6c2, SUC_lexicon_tags_15 },
    { 0x66eea6c3, SUC_lexicon_tags_28 },
    { 0x6f4ea6c3, SUC_lexicon_tags_21 },
    { 0x75cea6c5, SUC_lexicon_tags_43 },
    { 0x1daea6c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcea6ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba4ea6cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cea6d0, SUC_lexicon_tags_55 },
    { 0x3baea6d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8ea6d5, SUC_lexicon_tags_38 },
    { 0xa8cea6d5, SUC_lexicon_tags_18 },
    { 0x4eeea6d6, SUC_lexicon_tags_65 },
    { 0x8d4ea6d8, SUC_lexicon_tags_2 },
    { 0x9f0ea6d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6ea6df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0ea6e5, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882ea6e9, SUC_lexicon_tags_38 },
    { 0x486ea6e9, SUC_lexicon_tags_1 },
    { 0x8d2ea6eb, SUC_lexicon_tags_24 },
    { 0x376ea6ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90ea6f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60ea6f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc12ea6f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864ea6fe, SUC_lexicon_tags_17 },
    { 0x450ea6ff, SUC_lexicon_tags_177 },
    { 0xa4ea700, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cea703, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ea706, SUC_lexicon_tags_2 },
    { 0xfd4ea707, SUC_lexicon_tags_29 },
    { 0xdc8ea708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x318ea70b, SUC_lexicon_tags_7 },
    { 0x174ea70c, SUC_lexicon_tags_7 },
    { 0xfd2ea70d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0ea717, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdf4ea719, SUC_lexicon_tags_17 },
    { 0x1e4ea719, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98ea71e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366ea722, SUC_lexicon_tags_37 },
    { 0x44eea723, SUC_lexicon_tags_37 },
    { 0x4e8ea724, SUC_lexicon_tags_30 },
    { 0xa0eea723, SUC_lexicon_tags_47 },
    { 0xd96ea726, SUC_lexicon_tags_87 },
    { 0x724ea723, SUC_lexicon_tags_50 },
    { 0x51eea728, SUC_lexicon_tags_2 },
    { 0xdcea724, SUC_lexicon_tags_25 },
    { 0x3e4ea72a, SUC_lexicon_tags_25 },
    { 0x37aea72b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3deea72e, SUC_lexicon_tags_1 },
    { 0x146ea72f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cea732, SUC_lexicon_tags_14 },
    { 0xd28ea733, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff0ea735, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18ea73b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2ea743, SUC_lexicon_tags_1 },
    { 0x810ea743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8ea74e, SUC_lexicon_tags_2 },
    { 0x7d6ea74e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33aea755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ecea757, SUC_lexicon_tags_7 },
    { 0x48ea758, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0ea75e, SUC_lexicon_tags_37 },
    { 0xbd6ea75e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50ea764, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1acea766, SUC_lexicon_tags_3 },
    { 0xaa8ea767, SUC_lexicon_tags_87 },
    { 0x56aea768, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x752ea76b, SUC_lexicon_tags_27 },
    { 0xc76ea76b, SUC_lexicon_tags_15 },
    { 0xc7aea76b, SUC_lexicon_tags_25 },
    { 0xe9cea76e, SUC_lexicon_tags_15 },
    { 0xec0ea76b, SUC_lexicon_tags_84 },
    { 0xf52ea76c, SUC_lexicon_tags_1 },
    { 0x708ea771, SUC_lexicon_tags_15 },
    { 0xe8ea772, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4ea776, SUC_lexicon_tags_7 },
    { 0xbcea777, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58ea77f, SUC_lexicon_tags_43 },
    { 0x51aea780, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc82ea782, SUC_lexicon_tags_47 },
    { 0xb7eea783, SUC_lexicon_tags_15 },
    { 0x508ea784, SUC_lexicon_tags_55 },
    { 0xbcaea785, SUC_lexicon_tags_66 },
    { 0x1e4ea785, SUC_lexicon_tags_30 },
    { 0x2bcea785, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9feea78b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4ea78f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x416ea792, SUC_lexicon_tags_6 },
    { 0x446ea793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cea797, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x462ea799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x886ea79c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aea7a4, SUC_lexicon_tags_2 },
    { 0x2d6ea7a5, SUC_lexicon_tags_19 },
    { 0xaecea7a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x494ea7a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aea7ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96ea7af, SUC_lexicon_tags_0 },
    { 0xa08ea7af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8ea7b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b8ea7b6, SUC_lexicon_tags_2 },
    { 0x3c4ea7b7, SUC_lexicon_tags_1 },
    { 0x29aea7b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52ea7bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6ea7c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ea7cb, SUC_lexicon_tags_2 },
    { 0xc6aea7cc, SUC_lexicon_tags_63 },
    { 0xd30ea7cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eea7d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2aea7d3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x140ea7d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814ea7de, SUC_lexicon_tags_7 },
    { 0x2d6ea7de, SUC_lexicon_tags_11 },
    { 0x398ea7de, SUC_lexicon_tags_42 },
    { 0x96ea7e1, SUC_lexicon_tags_7 },
    { 0xcacea7e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6ea7e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x210ea7e8, SUC_lexicon_tags_14 },
    { 0x660ea7e9, SUC_lexicon_tags_11 },
    { 0x8feea7e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x874ea7ec, SUC_lexicon_tags_12 },
    { 0x3aaea7ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5e6ea7ef, SUC_lexicon_tags_7 },
    { 0xfa0ea7ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cea7f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x694ea7f5, SUC_lexicon_tags_15 },
    { 0xc62ea7f5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cea7fb, SUC_lexicon_tags_54 },
    { 0x1b0ea7fb, SUC_lexicon_tags_56 },
    { 0x8c4ea7fb, SUC_lexicon_tags_37 },
    { 0x334ea7fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x21eea800, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc10ea802, SUC_lexicon_tags_47 },
    { 0x192ea803, SUC_lexicon_tags_37 },
    { 0xb58ea803, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x218ea806, SUC_lexicon_tags_31 },
    { 0x4deea806, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e4ea809, SUC_lexicon_tags_27 },
    { 0x80eea80a, SUC_lexicon_tags_17 },
    { 0xf5cea80b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x508ea80e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0ea812, SUC_lexicon_tags_7 },
    { 0x2b6ea813, SUC_lexicon_tags_31 },
    { 0xfeeea812, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf94ea816, SUC_lexicon_tags_15 },
    { 0xca0ea817, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5a6ea819, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cea81f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3caea821, SUC_lexicon_tags_37 },
    { 0xe64ea821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x832ea824, SUC_lexicon_tags_72 },
    { 0xe1aea825, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cea828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x442ea82b, SUC_lexicon_tags_11 },
    { 0x934ea82b, SUC_lexicon_tags_48 },
    { 0xbe6ea82d, SUC_lexicon_tags_37 },
    { 0x44ea82d, SUC_lexicon_tags_25 },
    { 0xba2ea82f, SUC_lexicon_tags_19 },
    { 0x784ea830, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2ea836, SUC_lexicon_tags_2 },
    { 0x61eea837, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd8ea839, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2ea843, SUC_lexicon_tags_15 },
    { 0xecea843, SUC_lexicon_tags_1 },
    { 0x6ecea845, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52ea848, SUC_lexicon_tags_2 },
    { 0xb06ea848, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ea84f, SUC_lexicon_tags_59 },
    { 0xdfeea84f, SUC_lexicon_tags_11 },
    { 0x358ea851, SUC_lexicon_tags_26 },
    { 0xb14ea852, SUC_lexicon_tags_37 },
    { 0xa06ea853, SUC_lexicon_tags_37 },
    { 0x66ea854, SUC_lexicon_tags_37 },
    { 0x96ea854, SUC_lexicon_tags_23 },
    { 0xf8eea853, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750ea85b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x96ea85d, SUC_lexicon_tags_14 },
    { 0xa56ea85e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eea863, SUC_lexicon_tags_49 },
    { 0x5bcea863, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x31cea866, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44ea86b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaea86e, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cea872, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x504ea874, SUC_lexicon_tags_44 },
    { 0xb8eea875, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeea87b, SUC_lexicon_tags_25 },
    { 0xccaea87c, SUC_lexicon_tags_25 },
    { 0xed0ea87b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb8ea87f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4ea885, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4b4ea887, SUC_lexicon_tags_28 },
    { 0x17cea888, SUC_lexicon_tags_14 },
    { 0x714ea889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800ea88d, SUC_lexicon_tags_63 },
    { 0x56aea88d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cea895, SUC_lexicon_tags_15 },
    { 0xc7eea896, SUC_lexicon_tags_33 },
    { 0xf2ea896, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688ea89b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224ea8a0, SUC_lexicon_tags_11 },
    { 0x6faea8a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6ea8a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444ea8a8, SUC_lexicon_tags_9 },
    { 0x634ea8a8, SUC_lexicon_tags_7 },
    { 0xa94ea8a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa78ea8ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2acea8ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x460ea8b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7eea8b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8ea8b5, SUC_lexicon_tags_34 },
    { 0xb12ea8b5, SUC_lexicon_tags_37 },
    { 0x800ea8b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4ea8bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14ea8c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4ea8cc, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x578ea8ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96ea8d4, SUC_lexicon_tags_11 },
    { 0x170ea8d4, SUC_lexicon_tags_25 },
    { 0x820ea8d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2ea8da, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3d8ea8dc, SUC_lexicon_tags_33 },
    { 0xa3aea8dd, SUC_lexicon_tags_37 },
    { 0xc08ea8de, SUC_lexicon_tags_28 },
    { 0xf54ea8df, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc28ea8e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0ea8eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x168ea8ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2baea8ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68aea8fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe4ea8fe, SUC_lexicon_tags_6 },
    { 0x224ea8ff, SUC_lexicon_tags_11 },
    { 0x73eea8fe, SUC_lexicon_tags_27 },
    { 0x87eea901, SUC_lexicon_tags_25 },
    { 0x5d2ea902, SUC_lexicon_tags_14 },
    { 0x858ea903, SUC_lexicon_tags_79 },
    { 0x260ea902, SUC_lexicon_tags_25 },
    { 0x2aea901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eeea90a, SUC_lexicon_tags_5 },
    { 0xd56ea90b, SUC_lexicon_tags_12 },
    { 0xb34ea90c, SUC_lexicon_tags_16 },
    { 0xb86ea90a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcea911, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0ea914, SUC_lexicon_tags_11 },
    { 0x19cea914, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x298ea918, SUC_lexicon_tags_88 },
    { 0x3d4ea919, SUC_lexicon_tags_11 },
    { 0x178ea91a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x552ea91d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6aea922, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7beea925, SUC_lexicon_tags_63 },
    { 0xe1eea925, SUC_lexicon_tags_31 },
    { 0xea0ea927, SUC_lexicon_tags_17 },
    { 0x248ea928, SUC_lexicon_tags_18 },
    { 0x3ecea926, SUC_lexicon_tags_37 },
    { 0xb78ea92a, SUC_lexicon_tags_7 },
    { 0x360ea92b, SUC_lexicon_tags_25 },
    { 0x938ea92c, SUC_lexicon_tags_25 },
    { 0x798ea92d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24ea934, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x93eea937, SUC_lexicon_tags_65 },
    { 0xd4cea938, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x408ea93b, SUC_lexicon_tags_11 },
    { 0xf9eea93c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cea941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6ea946, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acea950, SUC_lexicon_tags_1 },
    { 0xadcea950, SUC_lexicon_tags_15 },
    { 0x33eea952, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4ea956, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4ea95a, SUC_lexicon_tags_2 },
    { 0x194ea95b, SUC_lexicon_tags_19 },
    { 0xc6aea95b, SUC_lexicon_tags_70 },
    { 0x634ea95d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12ea965, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5eea967, SUC_lexicon_tags_28 },
    { 0x7baea968, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58ea96b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146ea970, SUC_lexicon_tags_37 },
    { 0xab4ea970, SUC_lexicon_tags_49 },
    { 0x7f6ea970, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaea976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x234ea978, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94eea97c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ea97f, SUC_lexicon_tags_1 },
    { 0x564ea980, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaea985, SUC_lexicon_tags_19 },
    { 0xec8ea985, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eea989, SUC_lexicon_tags_14 },
    { 0x60cea989, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cea98e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68ea992, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00ea995, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecea998, SUC_lexicon_tags_0 },
    { 0x928ea999, SUC_lexicon_tags_1151 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeea99c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348ea9a4, SUC_lexicon_tags_7 },
    { 0x846ea9a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16ea9aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39aea9ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea6ea9af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9aea9b7, SUC_lexicon_tags_34 },
    { 0xcd6ea9b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56ea9bb, SUC_lexicon_tags_25 },
    { 0xe3eea9bc, SUC_lexicon_tags_85 },
    { 0x7cea9bc, SUC_lexicon_tags_22 },
    { 0xb16ea9bc, SUC_lexicon_tags_43 },
    { 0x3f8ea9bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x16aea9c2, SUC_lexicon_tags_37 },
    { 0x494ea9c2, SUC_lexicon_tags_14 },
    { 0x152ea9c2, SUC_lexicon_tags_28 },
    { 0x790ea9c2, SUC_lexicon_tags_15 },
    { 0x8f8ea9c5, SUC_lexicon_tags_33 },
    { 0xee6ea9c7, SUC_lexicon_tags_25 },
    { 0x3cea9c7, SUC_lexicon_tags_23 },
    { 0x5f6ea9c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8ea9cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd50ea9ce, SUC_lexicon_tags_33 },
    { 0xd04ea9ce, SUC_lexicon_tags_12 },
    { 0x526ea9d0, SUC_lexicon_tags_136 },
    { 0xa6aea9d1, SUC_lexicon_tags_37 },
    { 0xe8eea9d0, SUC_lexicon_tags_37 },
    { 0xff0ea9d3, SUC_lexicon_tags_17 },
    { 0x4ceea9d4, SUC_lexicon_tags_7 },
    { 0xf0cea9d5, SUC_lexicon_tags_19 },
    { 0x212ea9d5, SUC_lexicon_tags_10 },
    { 0x366ea9d4, SUC_lexicon_tags_7 },
    { 0x87cea9d8, SUC_lexicon_tags_42 },
    { 0xdacea9d9, SUC_lexicon_tags_39 },
    { 0x158ea9d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2ea9df, SUC_lexicon_tags_2 },
    { 0xaeaea9e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4ea9e3, SUC_lexicon_tags_14 },
    { 0x684ea9e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520ea9e9, SUC_lexicon_tags_463 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4ea9ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1c4ea9ef, SUC_lexicon_tags_11 },
    { 0xbb8ea9ef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc3cea9f2, SUC_lexicon_tags_7 },
    { 0x740ea9f3, SUC_lexicon_tags_19 },
    { 0x336ea9f4, SUC_lexicon_tags_0 },
    { 0x4b8ea9f5, SUC_lexicon_tags_49 },
    { 0xf3aea9f6, SUC_lexicon_tags_6 },
    { 0x160ea9f5, SUC_lexicon_tags_9 },
    { 0x576ea9f8, SUC_lexicon_tags_6 },
    { 0xf60ea9f8, SUC_lexicon_tags_14 },
    { 0x270ea9fa, SUC_lexicon_tags_14 },
    { 0x2f2ea9fb, SUC_lexicon_tags_29 },
    { 0x77aea9fc, SUC_lexicon_tags_47 },
    { 0x170ea9fd, SUC_lexicon_tags_47 },
    { 0xb74ea9fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa96eaa01, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aeaa04, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8eaa07, SUC_lexicon_tags_19 },
    { 0xd90eaa07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6eaa0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ceaa12, SUC_lexicon_tags_15 },
    { 0x38eeaa13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36eeaa15, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc18eaa17, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x69ceaa19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834eaa1d, SUC_lexicon_tags_47 },
    { 0xde8eaa1e, SUC_lexicon_tags_17 },
    { 0xd42eaa1f, SUC_lexicon_tags_11 },
    { 0x78aeaa1f, SUC_lexicon_tags_17 },
    { 0x43aeaa1e, SUC_lexicon_tags_63 },
    { 0x4c4eaa22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0eaa29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8eaa2d, SUC_lexicon_tags_12 },
    { 0x600eaa2e, SUC_lexicon_tags_27 },
    { 0x94eaa2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x552eaa31, SUC_lexicon_tags_7 },
    { 0x1feeaa32, SUC_lexicon_tags_79 },
    { 0xde4eaa33, SUC_lexicon_tags_165 },
    { 0x7c8eaa31, SUC_lexicon_tags_10 },
    { 0x564eaa35, SUC_lexicon_tags_88 },
    { 0x76eaa34, SUC_lexicon_tags_25 },
    { 0x1b2eaa37, SUC_lexicon_tags_1 },
    { 0x6daeaa38, SUC_lexicon_tags_45 },
    { 0xba0eaa38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ceaa3f, SUC_lexicon_tags_2 },
    { 0xc56eaa40, SUC_lexicon_tags_27 },
    { 0x14aeaa41, SUC_lexicon_tags_27 },
    { 0x138eaa41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x314eaa45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6eaa49, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3deeaa4f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2eaa52, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe4aeaa54, SUC_lexicon_tags_57 },
    { 0x316eaa55, SUC_lexicon_tags_34 },
    { 0x3caeaa56, SUC_lexicon_tags_17 },
    { 0x348eaa55, SUC_lexicon_tags_19 },
    { 0xcb8eaa58, SUC_lexicon_tags_33 },
    { 0xcc6eaa58, SUC_lexicon_tags_12 },
    { 0xb48eaa57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30eaa5d, SUC_lexicon_tags_37 },
    { 0xbcceaa5e, SUC_lexicon_tags_17 },
    { 0xaa2eaa5f, SUC_lexicon_tags_1 },
    { 0xf2eaa5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57aeaa63, SUC_lexicon_tags_46 },
    { 0x4a4eaa63, SUC_lexicon_tags_63 },
    { 0x7f8eaa65, SUC_lexicon_tags_1 },
    { 0x20eaa65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x564eaa69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aeaa6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4aeaa6e, SUC_lexicon_tags_1 },
    { 0xa6aeaa6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98ceaa71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322eaa7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x808eaa80, SUC_lexicon_tags_23 },
    { 0xabceaa81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfeeaa84, SUC_lexicon_tags_37 },
    { 0xc5aeaa85, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x2d6eaa87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4eaa8c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518eaa90, SUC_lexicon_tags_25 },
    { 0xbdceaa91, SUC_lexicon_tags_14 },
    { 0x350eaa92, SUC_lexicon_tags_1 },
    { 0xc40eaa93, SUC_lexicon_tags_31 },
    { 0x800eaa94, SUC_lexicon_tags_7 },
    { 0xcd6eaa93, SUC_lexicon_tags_31 },
    { 0x1d0eaa96, SUC_lexicon_tags_37 },
    { 0x6c6eaa97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98eaa99, SUC_lexicon_tags_12 },
    { 0x8e6eaa9a, SUC_lexicon_tags_37 },
    { 0xe84eaa9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x288eaa9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29eeaaa2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ceaaa7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0eaaab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa92eaaad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa28eaab0, SUC_lexicon_tags_7 },
    { 0x6d4eaab0, SUC_lexicon_tags_14 },
    { 0x63ceaab2, SUC_lexicon_tags_29 },
    { 0x156eaab2, SUC_lexicon_tags_8 },
    { 0xc52eaab4, SUC_lexicon_tags_17 },
    { 0x874eaab4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x74aeaab7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa90eaab9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806eaac1, SUC_lexicon_tags_27 },
    { 0xf6eaac2, SUC_lexicon_tags_19 },
    { 0x9cceaac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924eaac8, SUC_lexicon_tags_10 },
    { 0x5b8eaac9, SUC_lexicon_tags_48 },
    { 0x908eaaca, SUC_lexicon_tags_37 },
    { 0x900eaaca, SUC_lexicon_tags_27 },
    { 0x80aeaacb, SUC_lexicon_tags_15 },
    { 0x1e4eaac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x578eaad0, SUC_lexicon_tags_63 },
    { 0x67aeaad1, SUC_lexicon_tags_63 },
    { 0xe54eaad2, SUC_lexicon_tags_23 },
    { 0x3cceaad1, SUC_lexicon_tags_29 },
    { 0x3e0eaad4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x406eaad6, SUC_lexicon_tags_14 },
    { 0x486eaad7, SUC_lexicon_tags_17 },
    { 0x76ceaad7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00eaae3, SUC_lexicon_tags_34 },
    { 0x786eaae4, SUC_lexicon_tags_37 },
    { 0x6eeaae5, SUC_lexicon_tags_37 },
    { 0x4ceeaae5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x940eaae8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x326eaaeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aeaaf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54eaaf4, SUC_lexicon_tags_118 },
    { 0x3b2eaaf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51aeaaf8, SUC_lexicon_tags_12 },
    { 0x3beeaaf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90eaafc, SUC_lexicon_tags_33 },
    { 0x1c0eaafd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ceab00, SUC_lexicon_tags_48 },
    { 0xe98eab01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ceab04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0eab08, SUC_lexicon_tags_11 },
    { 0x652eab09, SUC_lexicon_tags_17 },
    { 0xd4eeab09, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676eab0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x984eab11, SUC_lexicon_tags_2 },
    { 0x520eab12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74eab16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x836eab19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6eab1e, SUC_lexicon_tags_37 },
    { 0x9d2eab1e, SUC_lexicon_tags_43 },
    { 0x602eab1f, SUC_lexicon_tags_12 },
    { 0xfdceab21, SUC_lexicon_tags_7 },
    { 0xf78eab20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8faeab24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedeeab29, SUC_lexicon_tags_31 },
    { 0x792eab2a, SUC_lexicon_tags_26 },
    { 0x4d8eab2b, SUC_lexicon_tags_63 },
    { 0xb7ceab2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xadaeab2f, SUC_lexicon_tags_55 },
    { 0x63ceab2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06eab34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140eab38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8eab3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x990eab3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8a4eab40, SUC_lexicon_tags_7 },
    { 0xe88eab41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22eab46, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xf34eab48, SUC_lexicon_tags_12 },
    { 0x208eab49, SUC_lexicon_tags_30 },
    { 0x2ceeab4a, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xa34eab4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72eab50, SUC_lexicon_tags_15 },
    { 0x78aeab50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaceab54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6aeab56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aeab5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86ceab5d, SUC_lexicon_tags_3 },
    { 0x24eeab5e, SUC_lexicon_tags_17 },
    { 0xffaeab5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b2eab61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2eab64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x844eab67, SUC_lexicon_tags_15 },
    { 0x262eab68, SUC_lexicon_tags_7 },
    { 0x5a4eab67, SUC_lexicon_tags_25 },
    { 0xd4aeab6a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8eab6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eeab70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4ceab72, SUC_lexicon_tags_17 },
    { 0xb3ceab72, SUC_lexicon_tags_43 },
    { 0x8aeeab74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d4eab76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82eab7a, SUC_lexicon_tags_38 },
    { 0x724eab7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8eab7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554eab82, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286eab86, SUC_lexicon_tags_28 },
    { 0x8d0eab86, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eeab8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4eab94, SUC_lexicon_tags_25 },
    { 0x57aeab94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76aeab97, SUC_lexicon_tags_37 },
    { 0x180eab97, SUC_lexicon_tags_33 },
    { 0x69ceab98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60eab9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8eaba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b8eaba5, SUC_lexicon_tags_28 },
    { 0x7baeaba6, SUC_lexicon_tags_25 },
    { 0xd5eeaba7, SUC_lexicon_tags_2 },
    { 0x34eeaba7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722eabae, SUC_lexicon_tags_12 },
    { 0xc04eabaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3feeabb1, SUC_lexicon_tags_38 },
    { 0xaaaeabb2, SUC_lexicon_tags_2 },
    { 0x3beeabb3, SUC_lexicon_tags_37 },
    { 0x6ceabb4, SUC_lexicon_tags_15 },
    { 0xaeaeabb2, SUC_lexicon_tags_15 },
    { 0x60eabb6, SUC_lexicon_tags_25 },
    { 0x868eabb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x530eabba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x900eabbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x674eabbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x94eeabc0, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06eabc5, SUC_lexicon_tags_15 },
    { 0x346eabc6, SUC_lexicon_tags_41 },
    { 0xbeeeabc6, SUC_lexicon_tags_25 },
    { 0x6d2eabc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf44eabca, SUC_lexicon_tags_7 },
    { 0x56eabcb, SUC_lexicon_tags_29 },
    { 0x4a0eabcc, SUC_lexicon_tags_37 },
    { 0x84eabcd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0eabd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93eeabd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8eabdc, SUC_lexicon_tags_35 },
    { 0xffeeabdd, SUC_lexicon_tags_40 },
    { 0x978eabde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20aeabe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x188eabe6, SUC_lexicon_tags_11 },
    { 0x2c4eabe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6eabf0, SUC_lexicon_tags_63 },
    { 0x112eabf0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38eabf7, SUC_lexicon_tags_10 },
    { 0xe5aeabf8, SUC_lexicon_tags_37 },
    { 0x944eabf8, SUC_lexicon_tags_2 },
    { 0xfa2eabf9, SUC_lexicon_tags_2 },
    { 0x86eabf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790eac00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baeac04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaaceac06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78eeac0b, SUC_lexicon_tags_2 },
    { 0x1beeac0c, SUC_lexicon_tags_15 },
    { 0x6beeac0d, SUC_lexicon_tags_77 },
    { 0x31ceac0d, SUC_lexicon_tags_87 },
    { 0x342eac0c, SUC_lexicon_tags_19 },
    { 0xdceac10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb64eac13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x63eeac16, SUC_lexicon_tags_38 },
    { 0xfaeac16, SUC_lexicon_tags_12 },
    { 0x1aceac18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122eac23, SUC_lexicon_tags_17 },
    { 0xfe4eac24, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeceeac29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fceac31, SUC_lexicon_tags_8 },
    { 0x57eeac32, SUC_lexicon_tags_14 },
    { 0xdf6eac33, SUC_lexicon_tags_37 },
    { 0x7f6eac34, SUC_lexicon_tags_17 },
    { 0x214eac34, SUC_lexicon_tags_7 },
    { 0xa48eac34, SUC_lexicon_tags_17 },
    { 0x286eac34, SUC_lexicon_tags_38 },
    { 0xb4ceac38, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbeeac3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x692eac3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6eac43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380eac47, SUC_lexicon_tags_11 },
    { 0x28eac48, SUC_lexicon_tags_4 },
    { 0x6f0eac49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f6eac4b, SUC_lexicon_tags_25 },
    { 0x5e6eac4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ceac50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ceac57, SUC_lexicon_tags_37 },
    { 0xefaeac58, SUC_lexicon_tags_0 },
    { 0xa08eac58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306eac63, SUC_lexicon_tags_11 },
    { 0x474eac63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcceac68, SUC_lexicon_tags_68 },
    { 0x9fceac69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42eac6d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbb0eac6f, SUC_lexicon_tags_15 },
    { 0x838eac70, SUC_lexicon_tags_15 },
    { 0xb52eac71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb74eac73, SUC_lexicon_tags_1 },
    { 0x3c4eac74, SUC_lexicon_tags_57 },
    { 0xa22eac73, SUC_lexicon_tags_45 },
    { 0x306eac75, SUC_lexicon_tags_1520 },
    { 0x6b4eac77, SUC_lexicon_tags_7 },
    { 0xd4eac78, SUC_lexicon_tags_49 },
    { 0xd3ceac79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x968eac7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72eac80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22eeac86, SUC_lexicon_tags_12 },
    { 0x7baeac86, SUC_lexicon_tags_25 },
    { 0xbbeeac88, SUC_lexicon_tags_15 },
    { 0xb74eac89, SUC_lexicon_tags_46 },
    { 0x92aeac8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6c2eac8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x422eac8e, SUC_lexicon_tags_23 },
    { 0x752eac8e, SUC_lexicon_tags_37 },
    { 0x3a4eac90, SUC_lexicon_tags_15 },
    { 0x802eac91, SUC_lexicon_tags_25 },
    { 0xd08eac91, SUC_lexicon_tags_11 },
    { 0x950eac93, SUC_lexicon_tags_29 },
    { 0x22eac8e, SUC_lexicon_tags_6 },
    { 0x53ceac92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86eac9a, SUC_lexicon_tags_7 },
    { 0x512eac9a, SUC_lexicon_tags_56 },
    { 0x268eac9b, SUC_lexicon_tags_33 },
    { 0xd52eac9d, SUC_lexicon_tags_17 },
    { 0xff2eac9e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd2ceaca0, SUC_lexicon_tags_77 },
    { 0xf9eeaca1, SUC_lexicon_tags_63 },
    { 0x396eaca2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828eaca7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85eeacaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0eacae, SUC_lexicon_tags_12 },
    { 0xe20eacaf, SUC_lexicon_tags_12 },
    { 0xc6aeacaf, SUC_lexicon_tags_15 },
    { 0x8e2eacb1, SUC_lexicon_tags_25 },
    { 0xdf2eacae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4eacb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39eeacb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3bceacbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x430eacbe, SUC_lexicon_tags_37 },
    { 0x7f0eacbf, SUC_lexicon_tags_38 },
    { 0x5d6eacc0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18eacc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x422eacc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170eaccd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x44aeaccf, SUC_lexicon_tags_33 },
    { 0xf40eacd0, SUC_lexicon_tags_2 },
    { 0x39aeacd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f0eacd3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x916eacd5, SUC_lexicon_tags_37 },
    { 0xe06eacd6, SUC_lexicon_tags_33 },
    { 0x5a2eacd7, SUC_lexicon_tags_0 },
    { 0x7f2eacd8, SUC_lexicon_tags_2 },
    { 0x70ceacd9, SUC_lexicon_tags_30 },
    { 0x28eeacd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x190eacdd, SUC_lexicon_tags_7 },
    { 0x7fceacde, SUC_lexicon_tags_66 },
    { 0xf6eeacde, SUC_lexicon_tags_13 },
    { 0xdd2eacde, SUC_lexicon_tags_8 },
    { 0x482eace1, SUC_lexicon_tags_37 },
    { 0xccaeacdd, SUC_lexicon_tags_31 },
    { 0x3e8eace3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6ceace5, SUC_lexicon_tags_39 },
    { 0xd60eace6, SUC_lexicon_tags_33 },
    { 0xe8eace7, SUC_lexicon_tags_63 },
    { 0x6e6eace7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eeacec, SUC_lexicon_tags_49 },
    { 0x6cceacec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2eacf0, SUC_lexicon_tags_19 },
    { 0xf64eacf1, SUC_lexicon_tags_17 },
    { 0x390eacf1, SUC_lexicon_tags_8 },
    { 0x164eacf2, SUC_lexicon_tags_0 },
    { 0xca8eacf1, SUC_lexicon_tags_15 },
    { 0xf00eacf5, SUC_lexicon_tags_34 },
    { 0xeb2eacf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca2eacf8, SUC_lexicon_tags_12 },
    { 0xe72eacf9, SUC_lexicon_tags_2 },
    { 0xedeeacfa, SUC_lexicon_tags_7 },
    { 0xb0aeacfb, SUC_lexicon_tags_11 },
    { 0x440eacfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8ead00, SUC_lexicon_tags_37 },
    { 0x3daead01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0ead04, SUC_lexicon_tags_2 },
    { 0x128ead05, SUC_lexicon_tags_25 },
    { 0x80ead06, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eead0a, SUC_lexicon_tags_17 },
    { 0x43cead0a, SUC_lexicon_tags_47 },
    { 0x5daead0c, SUC_lexicon_tags_45 },
    { 0x800ead0a, SUC_lexicon_tags_25 },
    { 0x162ead0d, SUC_lexicon_tags_16 },
    { 0x90ead0a, SUC_lexicon_tags_9 },
    { 0xb2eead0f, SUC_lexicon_tags_37 },
    { 0x6e0ead11, SUC_lexicon_tags_13 },
    { 0x204ead0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848ead1b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eead1f, SUC_lexicon_tags_19 },
    { 0xf56ead20, SUC_lexicon_tags_23 },
    { 0x742ead21, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6aead26, SUC_lexicon_tags_1 },
    { 0x6baead26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0ead2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4ead2e, SUC_lexicon_tags_1 },
    { 0xdaaead2f, SUC_lexicon_tags_17 },
    { 0x978ead2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24ead33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2ead39, SUC_lexicon_tags_22 },
    { 0x318ead3a, SUC_lexicon_tags_85 },
    { 0xe84ead39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62ead3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eead42, SUC_lexicon_tags_23 },
    { 0x748ead43, SUC_lexicon_tags_2 },
    { 0x9f4ead43, SUC_lexicon_tags_38 },
    { 0x9aead44, SUC_lexicon_tags_22 },
    { 0x2d0ead44, SUC_lexicon_tags_4 },
    { 0x142ead42, SUC_lexicon_tags_14 },
    { 0xbd6ead42, SUC_lexicon_tags_37 },
    { 0x19cead49, SUC_lexicon_tags_60 },
    { 0x1dcead49, SUC_lexicon_tags_10 },
    { 0x558ead4b, SUC_lexicon_tags_37 },
    { 0xc7aead4c, SUC_lexicon_tags_15 },
    { 0x252ead4b, SUC_lexicon_tags_31 },
    { 0x92ead4d, SUC_lexicon_tags_47 },
    { 0x9f2ead4f, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccead52, SUC_lexicon_tags_14 },
    { 0x13aead53, SUC_lexicon_tags_2 },
    { 0xaceead52, SUC_lexicon_tags_16 },
    { 0x764ead55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0ead58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe68ead5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56aead5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x488ead5e, SUC_lexicon_tags_15 },
    { 0x8b8ead5f, SUC_lexicon_tags_19 },
    { 0xc20ead5f, SUC_lexicon_tags_12 },
    { 0x1c2ead60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5feead6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82ead6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0ead72, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cead75, SUC_lexicon_tags_11 },
    { 0x72cead76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d8ead78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418ead80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2ead83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6ead8b, SUC_lexicon_tags_61 },
    { 0x9b4ead8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfdeead8e, SUC_lexicon_tags_15 },
    { 0x97cead8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4ead93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44ead96, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcead99, SUC_lexicon_tags_37 },
    { 0x798ead9a, SUC_lexicon_tags_31 },
    { 0xb26ead9a, SUC_lexicon_tags_43 },
    { 0xb92ead9c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3cead9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x644eada0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48eada9, SUC_lexicon_tags_7 },
    { 0x264eada9, SUC_lexicon_tags_160 },
    { 0x9aceadab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3deeadad, SUC_lexicon_tags_47 },
    { 0x756eadae, SUC_lexicon_tags_37 },
    { 0xbe0eadae, SUC_lexicon_tags_43 },
    { 0x4feeadad, SUC_lexicon_tags_17 },
    { 0x786eadb1, SUC_lexicon_tags_11 },
    { 0xd70eadb0, SUC_lexicon_tags_0 },
    { 0xe96eadad, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xff4eadb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x344eadb8, SUC_lexicon_tags_30 },
    { 0xca8eadb9, SUC_lexicon_tags_17 },
    { 0x612eadb8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0eadc2, SUC_lexicon_tags_15 },
    { 0x5b4eadc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4eadc6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x926eadc8, SUC_lexicon_tags_39 },
    { 0xcd8eadc9, SUC_lexicon_tags_12 },
    { 0xe22eadca, SUC_lexicon_tags_38 },
    { 0xff8eadcb, SUC_lexicon_tags_47 },
    { 0x128eadcc, SUC_lexicon_tags_38 },
    { 0xc12eadcc, SUC_lexicon_tags_0 },
    { 0x7f8eadcb, SUC_lexicon_tags_25 },
    { 0xf68eadcf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12eadd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38eadd9, SUC_lexicon_tags_38 },
    { 0xa66eadd9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4eaddc, SUC_lexicon_tags_29 },
    { 0x9c4eaddd, SUC_lexicon_tags_23 },
    { 0x302eadde, SUC_lexicon_tags_25 },
    { 0x934eaddf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82ceade1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1beeade4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda4eade6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2eadeb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362eadf0, SUC_lexicon_tags_1 },
    { 0x16eadf1, SUC_lexicon_tags_7 },
    { 0x754eadf1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70eeae00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78eae07, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x90ceae09, SUC_lexicon_tags_65 },
    { 0x96eae0a, SUC_lexicon_tags_30 },
    { 0x376eae0b, SUC_lexicon_tags_46 },
    { 0x380eae0b, SUC_lexicon_tags_15 },
    { 0x7aeeae0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faeae11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45eeae14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8eeae16, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x222eae18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68aeae1a, SUC_lexicon_tags_65 },
    { 0x856eae1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x856eae1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x216eae21, SUC_lexicon_tags_27 },
    { 0x1b4eae22, SUC_lexicon_tags_27 },
    { 0xc7ceae22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef6eae25, SUC_lexicon_tags_49 },
    { 0x650eae25, SUC_lexicon_tags_7 },
    { 0x7f4eae26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40eae2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ceae2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ceae32, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9aeae34, SUC_lexicon_tags_4 },
    { 0xd52eae34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe92eae37, SUC_lexicon_tags_15 },
    { 0xb70eae38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xae8eae3a, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x4deeae3d, SUC_lexicon_tags_11 },
    { 0xf78eae3d, SUC_lexicon_tags_33 },
    { 0xd6eeae3d, SUC_lexicon_tags_28 },
    { 0x364eae40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12eae43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x428eae45, SUC_lexicon_tags_49 },
    { 0xe66eae46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x16aeae49, SUC_lexicon_tags_11 },
    { 0x288eae49, SUC_lexicon_tags_29 },
    { 0xc58eae4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fceae51, SUC_lexicon_tags_46 },
    { 0x90ceae52, SUC_lexicon_tags_65 },
    { 0x892eae53, SUC_lexicon_tags_11 },
    { 0xb1ceae51, SUC_lexicon_tags_25 },
    { 0x892eae55, SUC_lexicon_tags_87 },
    { 0x686eae52, SUC_lexicon_tags_15 },
    { 0x25ceae57, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x672eae59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9aeae5f, SUC_lexicon_tags_37 },
    { 0x12eeae60, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34eae65, SUC_lexicon_tags_47 },
    { 0x104eae66, SUC_lexicon_tags_12 },
    { 0x312eae67, SUC_lexicon_tags_25 },
    { 0xe66eae67, SUC_lexicon_tags_0 },
    { 0xa12eae69, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x980eae6b, SUC_lexicon_tags_15 },
    { 0x4bceae6b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x554eae6e, SUC_lexicon_tags_0 },
    { 0x922eae6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d6eae71, SUC_lexicon_tags_19 },
    { 0xd0aeae72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x246eae74, SUC_lexicon_tags_15 },
    { 0xee8eae75, SUC_lexicon_tags_2 },
    { 0x1eeeae76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6eae7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2eae7d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9e4eae7f, SUC_lexicon_tags_25 },
    { 0xd64eae80, SUC_lexicon_tags_16 },
    { 0x90aeae81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2eae87, SUC_lexicon_tags_1236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74eae8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7aeae92, SUC_lexicon_tags_27 },
    { 0xf08eae93, SUC_lexicon_tags_10 },
    { 0xa2eae94, SUC_lexicon_tags_7 },
    { 0x48eeae94, SUC_lexicon_tags_68 },
    { 0x78ceae93, SUC_lexicon_tags_37 },
    { 0x5c6eae95, SUC_lexicon_tags_43 },
    { 0xaa4eae98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faeae9f, SUC_lexicon_tags_33 },
    { 0xf3eeaea0, SUC_lexicon_tags_29 },
    { 0xfcaeaea1, SUC_lexicon_tags_23 },
    { 0xfb2eaea2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64eaea6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff4eaea8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe10eaeaa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dceaead, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498eaeb4, SUC_lexicon_tags_14 },
    { 0xc08eaeb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466eaeb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaeaebb, SUC_lexicon_tags_43 },
    { 0x8eaeaebc, SUC_lexicon_tags_17 },
    { 0x64eaebb, SUC_lexicon_tags_9 },
    { 0x672eaebd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2eaec1, SUC_lexicon_tags_17 },
    { 0xc52eaec1, SUC_lexicon_tags_29 },
    { 0x3c8eaec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70eaec6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x370eaec8, SUC_lexicon_tags_25 },
    { 0x13eeaec9, SUC_lexicon_tags_25 },
    { 0x9faeaec9, SUC_lexicon_tags_66 },
    { 0x9e0eaecb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708eaed2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x796eaed4, SUC_lexicon_tags_32 },
    { 0x4e4eaed5, SUC_lexicon_tags_17 },
    { 0x4c0eaed6, SUC_lexicon_tags_14 },
    { 0x9a0eaed7, SUC_lexicon_tags_12 },
    { 0x1d4eaed7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b6eaeda, SUC_lexicon_tags_91 },
    { 0xf4aeaedb, SUC_lexicon_tags_4 },
    { 0xb7ceaedc, SUC_lexicon_tags_62 },
    { 0xcc4eaeda, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x432eaee0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d2eaee2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00eaeea, SUC_lexicon_tags_17 },
    { 0x2aceaeea, SUC_lexicon_tags_33 },
    { 0xb98eaeeb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeaeeaeee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2eaef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714eaefe, SUC_lexicon_tags_2 },
    { 0xad0eaeff, SUC_lexicon_tags_17 },
    { 0x98eaf00, SUC_lexicon_tags_0 },
    { 0x964eaf01, SUC_lexicon_tags_7 },
    { 0xc6eaeff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdceaf07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44eaf0d, SUC_lexicon_tags_25 },
    { 0xa48eaf0e, SUC_lexicon_tags_27 },
    { 0x352eaf0d, SUC_lexicon_tags_33 },
    { 0xca0eaf0e, SUC_lexicon_tags_1 },
    { 0x7e6eaf11, SUC_lexicon_tags_74 },
    { 0x358eaf0f, SUC_lexicon_tags_1 },
    { 0x78eaf13, SUC_lexicon_tags_39 },
    { 0xba0eaf14, SUC_lexicon_tags_37 },
    { 0x7f0eaf15, SUC_lexicon_tags_29 },
    { 0xda6eaf14, SUC_lexicon_tags_1 },
    { 0x98eeaf11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ceaf1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48eeaf1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eeaf20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238eaf2c, SUC_lexicon_tags_37 },
    { 0xdc2eaf2d, SUC_lexicon_tags_48 },
    { 0x93aeaf2c, SUC_lexicon_tags_37 },
    { 0xf7ceaf2f, SUC_lexicon_tags_37 },
    { 0x272eaf30, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2eaf34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x150eaf36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8feeaf39, SUC_lexicon_tags_151 },
    { 0xf0eaf3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8eeaf3c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ceaf43, SUC_lexicon_tags_37 },
    { 0x2f4eaf43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x900eaf47, SUC_lexicon_tags_97 },
    { 0x1b8eaf48, SUC_lexicon_tags_37 },
    { 0x94aeaf49, SUC_lexicon_tags_46 },
    { 0xa50eaf48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5baeaf4c, SUC_lexicon_tags_30 },
    { 0x53eeaf4c, SUC_lexicon_tags_19 },
    { 0x11aeaf4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0eaf51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaeaf55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6eeaf57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x502eaf59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x646eaf5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61aeaf5d, SUC_lexicon_tags_2 },
    { 0xbe6eaf5d, SUC_lexicon_tags_0 },
    { 0xe2eaf5e, SUC_lexicon_tags_15 },
    { 0xc3aeaf60, SUC_lexicon_tags_0 },
    { 0xbaaeaf5d, SUC_lexicon_tags_7 },
    { 0xcc4eaf62, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6eaf65, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6eaf68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6dceaf6a, SUC_lexicon_tags_9 },
    { 0x220eaf6b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aeaf6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408eaf73, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeeeaf77, SUC_lexicon_tags_17 },
    { 0xcaeeaf78, SUC_lexicon_tags_23 },
    { 0x556eaf79, SUC_lexicon_tags_1 },
    { 0xfcceaf79, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4aeaf7f, SUC_lexicon_tags_26 },
    { 0xca8eaf7f, SUC_lexicon_tags_7 },
    { 0x906eaf80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcaeaf83, SUC_lexicon_tags_37 },
    { 0xd9eeaf83, SUC_lexicon_tags_11 },
    { 0x80eeaf83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8e8eaf87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1eeaf89, SUC_lexicon_tags_25 },
    { 0xbdaeaf8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ceaf90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x98eaf93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4eaf97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182eaf9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe92eaf9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xea8eafa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa14eafa3, SUC_lexicon_tags_60 },
    { 0x23ceafa4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xffceafa6, SUC_lexicon_tags_25 },
    { 0xd8eeafa7, SUC_lexicon_tags_25 },
    { 0x5fceafa8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda2eafaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x774eafad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d4eafaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe48eafb1, SUC_lexicon_tags_23 },
    { 0xfc4eafb2, SUC_lexicon_tags_37 },
    { 0xe0ceafb3, SUC_lexicon_tags_11 },
    { 0xa2eeafb3, SUC_lexicon_tags_14 },
    { 0x3f8eafb4, SUC_lexicon_tags_7 },
    { 0x18aeafb6, SUC_lexicon_tags_37 },
    { 0x538eafb3, SUC_lexicon_tags_64 },
    { 0xeeceafb8, SUC_lexicon_tags_55 },
    { 0xe0aeafb8, SUC_lexicon_tags_25 },
    { 0x4aceafba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96eafbe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660eafc4, SUC_lexicon_tags_29 },
    { 0xfa6eafc4, SUC_lexicon_tags_15 },
    { 0x65aeafc6, SUC_lexicon_tags_11 },
    { 0xbe8eafc7, SUC_lexicon_tags_64 },
    { 0x876eafc7, SUC_lexicon_tags_2 },
    { 0x3e2eafc9, SUC_lexicon_tags_55 },
    { 0x9cceafc6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaeafd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeafd5, SUC_lexicon_tags_25 },
    { 0xd8ceafd6, SUC_lexicon_tags_19 },
    { 0xc0aeafd7, SUC_lexicon_tags_25 },
    { 0x538eafd6, SUC_lexicon_tags_63 },
    { 0x4f2eafd9, SUC_lexicon_tags_48 },
    { 0x116eafd8, SUC_lexicon_tags_2 },
    { 0xe16eafdb, SUC_lexicon_tags_7 },
    { 0x2c8eafd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x804eafdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x708eafe1, SUC_lexicon_tags_15 },
    { 0xe64eafe2, SUC_lexicon_tags_31 },
    { 0x156eafe2, SUC_lexicon_tags_16 },
    { 0x134eafe4, SUC_lexicon_tags_61 },
    { 0x928eafe5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x576eafe7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66eafea, SUC_lexicon_tags_18 },
    { 0x62ceafeb, SUC_lexicon_tags_2 },
    { 0x71aeafeb, SUC_lexicon_tags_7 },
    { 0xe02eafed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8eaff0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf0aeaff2, SUC_lexicon_tags_11 },
    { 0x67ceaff3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbceb001, SUC_lexicon_tags_2 },
    { 0x180eb002, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x28eeb005, SUC_lexicon_tags_15 },
    { 0x72ceb006, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50eb00a, SUC_lexicon_tags_0 },
    { 0xec6eb00a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x942eb00d, SUC_lexicon_tags_17 },
    { 0x4dceb00e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbceb010, SUC_lexicon_tags_8 },
    { 0x54eb011, SUC_lexicon_tags_38 },
    { 0x3c2eb012, SUC_lexicon_tags_7 },
    { 0xce6eb012, SUC_lexicon_tags_12 },
    { 0xbf4eb014, SUC_lexicon_tags_82 },
    { 0x23eeb011, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4eb01a, SUC_lexicon_tags_21 },
    { 0x14eb01b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdf2eb01d, SUC_lexicon_tags_9 },
    { 0x16aeb01e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89aeb020, SUC_lexicon_tags_17 },
    { 0x548eb021, SUC_lexicon_tags_15 },
    { 0x3b0eb021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56eb024, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2ceb026, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x50ceb028, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6eb02e, SUC_lexicon_tags_37 },
    { 0x996eb02f, SUC_lexicon_tags_47 },
    { 0x9c6eb02f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaceb034, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x752eb037, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaceb03a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4eb03d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502eb041, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0eb045, SUC_lexicon_tags_20 },
    { 0x72eeb046, SUC_lexicon_tags_12 },
    { 0xd24eb045, SUC_lexicon_tags_2 },
    { 0xb5ceb046, SUC_lexicon_tags_11 },
    { 0xee6eb049, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7ceb04b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaeb051, SUC_lexicon_tags_48 },
    { 0x4eceb052, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46eb055, SUC_lexicon_tags_10 },
    { 0x74ceb056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x394eb058, SUC_lexicon_tags_12 },
    { 0x978eb058, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76eb05c, SUC_lexicon_tags_1 },
    { 0x14aeb05d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eceb060, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbceeb067, SUC_lexicon_tags_25 },
    { 0xd22eb068, SUC_lexicon_tags_19 },
    { 0xd14eb069, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23eeb06d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ceeb06f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeb072, SUC_lexicon_tags_14 },
    { 0x74ceb073, SUC_lexicon_tags_37 },
    { 0x576eb074, SUC_lexicon_tags_13 },
    { 0x3d4eb075, SUC_lexicon_tags_31 },
    { 0xc50eb073, SUC_lexicon_tags_19 },
    { 0x5feeb076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eceb07b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x37aeb07d, SUC_lexicon_tags_15 },
    { 0x102eb07e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x32eeb081, SUC_lexicon_tags_7 },
    { 0x49ceb082, SUC_lexicon_tags_63 },
    { 0xc8ceb083, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6eb089, SUC_lexicon_tags_17 },
    { 0xeceeb08a, SUC_lexicon_tags_136 },
    { 0x19ceb089, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8eb08e, SUC_lexicon_tags_1 },
    { 0x6aeeb08e, SUC_lexicon_tags_15 },
    { 0xf1aeb090, SUC_lexicon_tags_39 },
    { 0x2e4eb091, SUC_lexicon_tags_7 },
    { 0xa4aeb092, SUC_lexicon_tags_27 },
    { 0xbd8eb092, SUC_lexicon_tags_17 },
    { 0x238eb090, SUC_lexicon_tags_2 },
    { 0x350eb090, SUC_lexicon_tags_46 },
    { 0x16aeb094, SUC_lexicon_tags_11 },
    { 0xf4eb097, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ceb09e, SUC_lexicon_tags_25 },
    { 0x4c4eb09e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0eb0a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2eb0a9, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60eb0b0, SUC_lexicon_tags_7 },
    { 0x626eb0b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x988eb0b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10aeb0b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0aeb0ba, SUC_lexicon_tags_27 },
    { 0xf02eb0ba, SUC_lexicon_tags_7 },
    { 0xee2eb0bc, SUC_lexicon_tags_37 },
    { 0xd52eb0bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc4eb0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x238eb0c1, SUC_lexicon_tags_11 },
    { 0xaa0eb0c2, SUC_lexicon_tags_15 },
    { 0xb42eb0c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x51eeb0c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446eb0ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36eb0cc, SUC_lexicon_tags_9 },
    { 0xb52eb0cd, SUC_lexicon_tags_0 },
    { 0x312eb0cc, SUC_lexicon_tags_38 },
    { 0xa2aeb0cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8eeb0d2, SUC_lexicon_tags_17 },
    { 0x6d4eb0d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ceb0d8, SUC_lexicon_tags_61 },
    { 0x8a2eb0d9, SUC_lexicon_tags_7 },
    { 0xa4aeb0d9, SUC_lexicon_tags_11 },
    { 0x546eb0db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x466eb0de, SUC_lexicon_tags_14 },
    { 0x298eb0df, SUC_lexicon_tags_14 },
    { 0x46ceb0e0, SUC_lexicon_tags_45 },
    { 0x19eeb0e0, SUC_lexicon_tags_38 },
    { 0xbf6eb0e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8eb0eb, SUC_lexicon_tags_23 },
    { 0x8b2eb0ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x990eb0ef, SUC_lexicon_tags_17 },
    { 0xfeeb0ef, SUC_lexicon_tags_7 },
    { 0x83eeb0ef, SUC_lexicon_tags_614 },
    { 0xf76eb0f1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7d0eb0f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940eb0fb, SUC_lexicon_tags_2 },
    { 0x940eb0fc, SUC_lexicon_tags_12 },
    { 0x160eb0fd, SUC_lexicon_tags_59 },
    { 0x4a0eb0fd, SUC_lexicon_tags_12 },
    { 0x32ceb0ff, SUC_lexicon_tags_49 },
    { 0x778eb0fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ceb105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13eeb108, SUC_lexicon_tags_25 },
    { 0x738eb109, SUC_lexicon_tags_951 },
    { 0, NULL },
    { 0xb8aeb10b, SUC_lexicon_tags_10 },
    { 0xc4ceb10c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x138eb10e, SUC_lexicon_tags_2 },
    { 0x508eb10e, SUC_lexicon_tags_23 },
    { 0x6feeb10f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eeb115, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc18eb118, SUC_lexicon_tags_31 },
    { 0x152eb119, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50eb11d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aeb125, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd44eb127, SUC_lexicon_tags_15 },
    { 0xbc8eb127, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0x94eeb12a, SUC_lexicon_tags_47 },
    { 0xf4eb12b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6eb137, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x248eb13a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69aeb13f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e4eb141, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62eb145, SUC_lexicon_tags_1 },
    { 0x79eeb146, SUC_lexicon_tags_7 },
    { 0x8eeeb147, SUC_lexicon_tags_4 },
    { 0x7f0eb148, SUC_lexicon_tags_37 },
    { 0x328eb149, SUC_lexicon_tags_19 },
    { 0xa86eb14a, SUC_lexicon_tags_33 },
    { 0xda6eb145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d4eb14d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2eb150, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ceb159, SUC_lexicon_tags_0 },
    { 0xbceeb159, SUC_lexicon_tags_49 },
    { 0x73aeb15b, SUC_lexicon_tags_12 },
    { 0xcb8eb15b, SUC_lexicon_tags_15 },
    { 0x1e2eb15d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46eb160, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x214eb163, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x78ceb165, SUC_lexicon_tags_14 },
    { 0x5aceb165, SUC_lexicon_tags_49 },
    { 0xad4eb167, SUC_lexicon_tags_37 },
    { 0x79aeb167, SUC_lexicon_tags_60 },
    { 0x2feeb168, SUC_lexicon_tags_29 },
    { 0xb2ceb169, SUC_lexicon_tags_15 },
    { 0xf4eeb166, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaeb16e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f6eb170, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faeb178, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aeb17b, SUC_lexicon_tags_15 },
    { 0x4b8eb17c, SUC_lexicon_tags_2 },
    { 0xd96eb17d, SUC_lexicon_tags_11 },
    { 0xde0eb17d, SUC_lexicon_tags_151 },
    { 0x8a8eb17e, SUC_lexicon_tags_85 },
    { 0xc48eb17e, SUC_lexicon_tags_1 },
    { 0x6f8eb181, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf76eb183, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ceb187, SUC_lexicon_tags_19 },
    { 0xfe6eb188, SUC_lexicon_tags_25 },
    { 0xf58eb187, SUC_lexicon_tags_31 },
    { 0xd7aeb188, SUC_lexicon_tags_29 },
    { 0x1c0eb18b, SUC_lexicon_tags_49 },
    { 0x9f0eb18b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x296eb18e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57eeb192, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6eb197, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf62eb19b, SUC_lexicon_tags_37 },
    { 0xab8eb19b, SUC_lexicon_tags_27 },
    { 0x710eb19d, SUC_lexicon_tags_7 },
    { 0x81aeb19b, SUC_lexicon_tags_34 },
    { 0x324eb19f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eb1a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19eeb1a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40eb1b4, SUC_lexicon_tags_1 },
    { 0x24aeb1b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aeb1be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68eb1c1, SUC_lexicon_tags_43 },
    { 0x99ceb1c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7f6eb1c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d8eb1c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f2eb1c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x346eb1cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbceb1cd, SUC_lexicon_tags_17 },
    { 0x392eb1ce, SUC_lexicon_tags_1 },
    { 0x39ceb1cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x48ceb1d1, SUC_lexicon_tags_46 },
    { 0x2deeb1d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaeb1d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ceb1da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb62eb1dc, SUC_lexicon_tags_17 },
    { 0x8d6eb1dd, SUC_lexicon_tags_63 },
    { 0x39aeb1dc, SUC_lexicon_tags_17 },
    { 0x9deeb1df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f0eb1e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568eb1e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51aeb1ea, SUC_lexicon_tags_18 },
    { 0xfc2eb1eb, SUC_lexicon_tags_15 },
    { 0xc5ceb1ec, SUC_lexicon_tags_26 },
    { 0x8a6eb1ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6ceb1ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ceb1f3, SUC_lexicon_tags_31 },
    { 0xd04eb1f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02eb1f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc44eb1fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304eb1fe, SUC_lexicon_tags_30 },
    { 0x240eb1fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684eb203, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x874eb205, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x360eb207, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ceb20b, SUC_lexicon_tags_24 },
    { 0x4feeb20c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eceb20f, SUC_lexicon_tags_45 },
    { 0x64eeb210, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ceb214, SUC_lexicon_tags_163 },
    { 0x214eb215, SUC_lexicon_tags_25 },
    { 0xe50eb216, SUC_lexicon_tags_37 },
    { 0x4eeeb217, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x736eb219, SUC_lexicon_tags_25 },
    { 0xdbceb21a, SUC_lexicon_tags_79 },
    { 0x4c8eb21b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaeb221, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x390eb223, SUC_lexicon_tags_29 },
    { 0x270eb223, SUC_lexicon_tags_15 },
    { 0x6cceb224, SUC_lexicon_tags_37 },
    { 0xf60eb224, SUC_lexicon_tags_32 },
    { 0x70aeb227, SUC_lexicon_tags_38 },
    { 0x8daeb225, SUC_lexicon_tags_65 },
    { 0x616eb229, SUC_lexicon_tags_44 },
    { 0x5eaeb22a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x50eeb22c, SUC_lexicon_tags_3 },
    { 0x43eeb22c, SUC_lexicon_tags_32 },
    { 0x530eb22e, SUC_lexicon_tags_15 },
    { 0x776eb22f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1aeb231, SUC_lexicon_tags_15 },
    { 0x272eb232, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0eb237, SUC_lexicon_tags_35 },
    { 0x3eeeb238, SUC_lexicon_tags_25 },
    { 0x4d2eb239, SUC_lexicon_tags_0 },
    { 0x5f8eb23a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ceb23d, SUC_lexicon_tags_0 },
    { 0xb6eeb23e, SUC_lexicon_tags_15 },
    { 0x51eeb23f, SUC_lexicon_tags_25 },
    { 0xfd4eb240, SUC_lexicon_tags_28 },
    { 0x986eb241, SUC_lexicon_tags_35 },
    { 0x912eb241, SUC_lexicon_tags_1 },
    { 0x4b2eb242, SUC_lexicon_tags_23 },
    { 0x93eeb244, SUC_lexicon_tags_25 },
    { 0xaa2eb245, SUC_lexicon_tags_84 },
    { 0xc9ceb23e, SUC_lexicon_tags_27 },
    { 0xcc8eb244, SUC_lexicon_tags_37 },
    { 0x720eb246, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa86eb24b, SUC_lexicon_tags_17 },
    { 0x2e2eb24b, SUC_lexicon_tags_10 },
    { 0x6f6eb24d, SUC_lexicon_tags_49 },
    { 0x662eb24d, SUC_lexicon_tags_17 },
    { 0x8deeb24e, SUC_lexicon_tags_66 },
    { 0x23ceb24b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54eb253, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x696eb255, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbfaeb257, SUC_lexicon_tags_25 },
    { 0xe2eb258, SUC_lexicon_tags_29 },
    { 0x1bceb257, SUC_lexicon_tags_15 },
    { 0x548eb25a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eeb25e, SUC_lexicon_tags_49 },
    { 0xc54eb25e, SUC_lexicon_tags_37 },
    { 0xb0aeb260, SUC_lexicon_tags_60 },
    { 0xd4eeb25f, SUC_lexicon_tags_4 },
    { 0x156eb261, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64eb266, SUC_lexicon_tags_19 },
    { 0x5deeb267, SUC_lexicon_tags_19 },
    { 0x1daeb267, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4c6eb26a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0eb26f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ceb272, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78eeb27a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96eb282, SUC_lexicon_tags_141 },
    { 0x280eb282, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8eb286, SUC_lexicon_tags_9 },
    { 0x922eb287, SUC_lexicon_tags_15 },
    { 0x9aceb287, SUC_lexicon_tags_37 },
    { 0x82ceb288, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x39aeb28c, SUC_lexicon_tags_31 },
    { 0x532eb28c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0eb293, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0eb297, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f8eb299, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8eb29e, SUC_lexicon_tags_17 },
    { 0xd86eb29f, SUC_lexicon_tags_11 },
    { 0x68eeb2a0, SUC_lexicon_tags_63 },
    { 0x81aeb2a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942eb2a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2faeb2b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe14eb2b2, SUC_lexicon_tags_87 },
    { 0x9f6eb2b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8eb2b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ceb2bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce2eb2bf, SUC_lexicon_tags_14 },
    { 0xadceb2bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8eb2c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1aeeb2c5, SUC_lexicon_tags_23 },
    { 0x75eeb2c6, SUC_lexicon_tags_7 },
    { 0xf58eb2c7, SUC_lexicon_tags_21 },
    { 0x5d8eb2c5, SUC_lexicon_tags_60 },
    { 0x988eb2c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d0eb2cb, SUC_lexicon_tags_23 },
    { 0x728eb2cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312eb2d1, SUC_lexicon_tags_22 },
    { 0x6c2eb2d2, SUC_lexicon_tags_28 },
    { 0x510eb2d3, SUC_lexicon_tags_60 },
    { 0xcbceb2d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692eb2d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4eb2dc, SUC_lexicon_tags_1 },
    { 0xaeeeb2dd, SUC_lexicon_tags_12 },
    { 0x8f0eb2de, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x594eb2e1, SUC_lexicon_tags_45 },
    { 0x75aeb2e2, SUC_lexicon_tags_12 },
    { 0xb80eb2e3, SUC_lexicon_tags_2 },
    { 0x578eb2e2, SUC_lexicon_tags_11 },
    { 0x240eb2e5, SUC_lexicon_tags_26 },
    { 0xdcceb2e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790eb2eb, SUC_lexicon_tags_3 },
    { 0x3ceb2eb, SUC_lexicon_tags_25 },
    { 0x636eb2ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x626eb2ef, SUC_lexicon_tags_259 },
    { 0x59aeb2f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x750eb2f3, SUC_lexicon_tags_2 },
    { 0xa0ceb2f4, SUC_lexicon_tags_25 },
    { 0x12aeb2f5, SUC_lexicon_tags_15 },
    { 0xd7eeb2f6, SUC_lexicon_tags_1 },
    { 0xb9aeb2f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4eb2fa, SUC_lexicon_tags_14 },
    { 0xe9eeb2fa, SUC_lexicon_tags_12 },
    { 0x986eb2fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x950eb2ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60eb303, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x21ceb305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4eb308, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddceb30d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ceb310, SUC_lexicon_tags_11 },
    { 0x5b8eb310, SUC_lexicon_tags_7 },
    { 0xd6eb312, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaeb316, SUC_lexicon_tags_63 },
    { 0x8b6eb317, SUC_lexicon_tags_17 },
    { 0xc32eb318, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476eb31c, SUC_lexicon_tags_17 },
    { 0x5beeb31d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eeb322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x368eb324, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf72eb326, SUC_lexicon_tags_63 },
    { 0x254eb327, SUC_lexicon_tags_2 },
    { 0xc8eb327, SUC_lexicon_tags_31 },
    { 0xb94eb326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x406eb32c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcaeb332, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450eb336, SUC_lexicon_tags_37 },
    { 0x828eb337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744eb346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8eb34a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ceb34d, SUC_lexicon_tags_45 },
    { 0x662eb34d, SUC_lexicon_tags_42 },
    { 0x434eb34e, SUC_lexicon_tags_0 },
    { 0x87ceb34e, SUC_lexicon_tags_25 },
    { 0x22eeb34d, SUC_lexicon_tags_17 },
    { 0x496eb350, SUC_lexicon_tags_17 },
    { 0x110eb353, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57eeb35b, SUC_lexicon_tags_15 },
    { 0x5e0eb35c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ceb35f, SUC_lexicon_tags_2 },
    { 0x66aeb360, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96eb364, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eeb369, SUC_lexicon_tags_37 },
    { 0x8d0eb36a, SUC_lexicon_tags_25 },
    { 0xc9eeb369, SUC_lexicon_tags_14 },
    { 0xd34eb36a, SUC_lexicon_tags_9 },
    { 0x8daeb36d, SUC_lexicon_tags_9 },
    { 0x50eeb36e, SUC_lexicon_tags_17 },
    { 0xe8aeb36f, SUC_lexicon_tags_30 },
    { 0xcceeb370, SUC_lexicon_tags_0 },
    { 0x92aeb369, SUC_lexicon_tags_45 },
    { 0x16eeb36c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc04eb374, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6beeb376, SUC_lexicon_tags_30 },
    { 0x3a4eb377, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722eb37b, SUC_lexicon_tags_19 },
    { 0x7aaeb37c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1beeb381, SUC_lexicon_tags_28 },
    { 0x11ceb382, SUC_lexicon_tags_6 },
    { 0x69aeb383, SUC_lexicon_tags_15 },
    { 0xa20eb383, SUC_lexicon_tags_15 },
    { 0x592eb385, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c0eb387, SUC_lexicon_tags_15 },
    { 0x71ceb388, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4ceb38a, SUC_lexicon_tags_32 },
    { 0x194eb38b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xde6eb38d, SUC_lexicon_tags_25 },
    { 0x556eb38e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eeb391, SUC_lexicon_tags_17 },
    { 0x812eb391, SUC_lexicon_tags_47 },
    { 0x3d4eb392, SUC_lexicon_tags_6 },
    { 0xebeeb394, SUC_lexicon_tags_9 },
    { 0x214eb394, SUC_lexicon_tags_46 },
    { 0x514eb396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x980eb398, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964eb39f, SUC_lexicon_tags_47 },
    { 0xe70eb3a0, SUC_lexicon_tags_7 },
    { 0x362eb3a1, SUC_lexicon_tags_17 },
    { 0x2aaeb3a0, SUC_lexicon_tags_84 },
    { 0x98eb3a0, SUC_lexicon_tags_3 },
    { 0x6faeb3a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9cceb3a6, SUC_lexicon_tags_49 },
    { 0xf8aeb3a7, SUC_lexicon_tags_25 },
    { 0xcdceb3a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6eeb3ab, SUC_lexicon_tags_46 },
    { 0x2feeb3ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30eb3af, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x680eb3b2, SUC_lexicon_tags_23 },
    { 0xb3aeb3b2, SUC_lexicon_tags_38 },
    { 0xad2eb3b4, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8eb3b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa7eeb3b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2eb3bd, SUC_lexicon_tags_17 },
    { 0xb58eb3be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0eeb3c0, SUC_lexicon_tags_2 },
    { 0x8e4eb3c1, SUC_lexicon_tags_11 },
    { 0x46aeb3c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fceb3c4, SUC_lexicon_tags_15 },
    { 0x588eb3c5, SUC_lexicon_tags_37 },
    { 0x74ceb3c4, SUC_lexicon_tags_37 },
    { 0xe9eeb3c7, SUC_lexicon_tags_17 },
    { 0x5f8eb3c8, SUC_lexicon_tags_1 },
    { 0xd5ceb3c8, SUC_lexicon_tags_17 },
    { 0x54ceb3ca, SUC_lexicon_tags_37 },
    { 0x60ceb3ca, SUC_lexicon_tags_25 },
    { 0x684eb3cc, SUC_lexicon_tags_60 },
    { 0xea6eb3c9, SUC_lexicon_tags_22 },
    { 0xd0eeb3ce, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8eb3d1, SUC_lexicon_tags_17 },
    { 0xa66eb3d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaeb3d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xda2eb3da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c0eb3dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ceb3e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44eb3e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4eb3e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aaeb3ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9ceb3f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50eb3f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70eb3f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6eb3fc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70eb3ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2eb403, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1cceb405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x360eb407, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50eb40a, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620eb40f, SUC_lexicon_tags_1 },
    { 0x53ceb40f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73aeb414, SUC_lexicon_tags_15 },
    { 0x50ceb414, SUC_lexicon_tags_7 },
    { 0xeaaeb415, SUC_lexicon_tags_15 },
    { 0x7f0eb417, SUC_lexicon_tags_23 },
    { 0x9f8eb418, SUC_lexicon_tags_23 },
    { 0x9f2eb417, SUC_lexicon_tags_53 },
    { 0x242eb41a, SUC_lexicon_tags_49 },
    { 0x7b6eb41b, SUC_lexicon_tags_15 },
    { 0xe2eeb41c, SUC_lexicon_tags_1 },
    { 0xb68eb41d, SUC_lexicon_tags_38 },
    { 0x34aeb41d, SUC_lexicon_tags_1 },
    { 0xf70eb417, SUC_lexicon_tags_1 },
    { 0x46eeb418, SUC_lexicon_tags_7 },
    { 0x8a8eb421, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc2eb423, SUC_lexicon_tags_8 },
    { 0x668eb424, SUC_lexicon_tags_17 },
    { 0xa8eeb425, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18eb42d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344eb436, SUC_lexicon_tags_0 },
    { 0x6f8eb436, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ceb43d, SUC_lexicon_tags_25 },
    { 0x30eb43e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ceb441, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeeb444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2eb448, SUC_lexicon_tags_30 },
    { 0xa2eb448, SUC_lexicon_tags_12 },
    { 0x410eb44a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x460eb44c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1eeb44e, SUC_lexicon_tags_0 },
    { 0x268eb44e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9caeb452, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24eb455, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1fceb457, SUC_lexicon_tags_12 },
    { 0xad4eb458, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0eb45b, SUC_lexicon_tags_1 },
    { 0x77eeb45b, SUC_lexicon_tags_45 },
    { 0x8c6eb45c, SUC_lexicon_tags_23 },
    { 0xcb2eb45c, SUC_lexicon_tags_11 },
    { 0x61aeb45d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e4eb461, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36eb464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4aeb466, SUC_lexicon_tags_25 },
    { 0x292eb466, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x950eb46a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6eb46f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aceb475, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf06eb477, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7aeb47a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404eb47e, SUC_lexicon_tags_11 },
    { 0x7eceb47e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9f4eb481, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78eb483, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xae6eb485, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8eb489, SUC_lexicon_tags_11 },
    { 0xe8aeb48a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1dceb48c, SUC_lexicon_tags_26 },
    { 0x9aeb48c, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1eeb490, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x982eb492, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e8eb494, SUC_lexicon_tags_1 },
    { 0x11ceb494, SUC_lexicon_tags_222 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8eb49b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eeb49e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aeb4a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x176eb4a8, SUC_lexicon_tags_15 },
    { 0x3eaeb4a9, SUC_lexicon_tags_16 },
    { 0x54eb4aa, SUC_lexicon_tags_37 },
    { 0xdfaeb4ab, SUC_lexicon_tags_79 },
    { 0x220eb4ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d4eb4ae, SUC_lexicon_tags_25 },
    { 0x202eb4ae, SUC_lexicon_tags_25 },
    { 0xab8eb4af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e4eb4b2, SUC_lexicon_tags_47 },
    { 0x5c8eb4b3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe28eb4b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x44aeb4b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4eb4bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x450eb4bd, SUC_lexicon_tags_2 },
    { 0xab4eb4be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596eb4c5, SUC_lexicon_tags_26 },
    { 0x2e2eb4c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf12eb4c9, SUC_lexicon_tags_17 },
    { 0x8deeb4ca, SUC_lexicon_tags_2 },
    { 0x8c4eb4cb, SUC_lexicon_tags_39 },
    { 0xebeeb4cb, SUC_lexicon_tags_1 },
    { 0x9d0eb4cb, SUC_lexicon_tags_61 },
    { 0x3caeb4ce, SUC_lexicon_tags_32 },
    { 0xc76eb4cc, SUC_lexicon_tags_47 },
    { 0xdc2eb4c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4deeb4d4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8eb4d7, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefeeb4e1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdceb4e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x224eb4e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316eb4ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1ceb4ef, SUC_lexicon_tags_1 },
    { 0xe9aeb4f0, SUC_lexicon_tags_72 },
    { 0xf28eb4f1, SUC_lexicon_tags_6 },
    { 0x588eb4ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434eb4fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30eeb501, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0eb506, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6eeeb508, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf48eb50a, SUC_lexicon_tags_116 },
    { 0x6b8eb50b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0eb510, SUC_lexicon_tags_84 },
    { 0x8c8eb511, SUC_lexicon_tags_15 },
    { 0xbe0eb511, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe38eb514, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x77eeb516, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x594eb519, SUC_lexicon_tags_1 },
    { 0xe34eb51a, SUC_lexicon_tags_15 },
    { 0x1e6eb51b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x35eeb51e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28eeb521, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x72aeb524, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf92eb526, SUC_lexicon_tags_11 },
    { 0x8fceb526, SUC_lexicon_tags_37 },
    { 0xc2aeb528, SUC_lexicon_tags_23 },
    { 0xd74eb529, SUC_lexicon_tags_37 },
    { 0xbeeeb52a, SUC_lexicon_tags_15 },
    { 0xf9eeb52b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x38eeb52d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x310eb530, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a8eb532, SUC_lexicon_tags_175 },
    { 0x79ceb533, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52eeb538, SUC_lexicon_tags_31 },
    { 0x506eb538, SUC_lexicon_tags_8 },
    { 0x34eeb539, SUC_lexicon_tags_11 },
    { 0x240eb539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43eeb53d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2eb540, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4eb549, SUC_lexicon_tags_37 },
    { 0xe82eb54a, SUC_lexicon_tags_59 },
    { 0x3e2eb54a, SUC_lexicon_tags_17 },
    { 0x1d2eb54c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5beeb54f, SUC_lexicon_tags_39 },
    { 0x40aeb54f, SUC_lexicon_tags_23 },
    { 0x2d4eb550, SUC_lexicon_tags_38 },
    { 0x8baeb551, SUC_lexicon_tags_11 },
    { 0x85ceb553, SUC_lexicon_tags_46 },
    { 0x1f0eb550, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9aeb559, SUC_lexicon_tags_46 },
    { 0x312eb559, SUC_lexicon_tags_32 },
    { 0x356eb55b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40aeb55d, SUC_lexicon_tags_2 },
    { 0xd26eb55e, SUC_lexicon_tags_27 },
    { 0x38eb55e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aeeb562, SUC_lexicon_tags_19 },
    { 0xa82eb563, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eceb566, SUC_lexicon_tags_7 },
    { 0x7bceb566, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x600eb56a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42eb56c, SUC_lexicon_tags_12 },
    { 0x42ceb56c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e8eb56f, SUC_lexicon_tags_1 },
    { 0xe4ceb56f, SUC_lexicon_tags_26 },
    { 0xe4eeb571, SUC_lexicon_tags_17 },
    { 0xdcceb570, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eb577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x748eb579, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x338eb57c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26eb57f, SUC_lexicon_tags_37 },
    { 0xc6aeb580, SUC_lexicon_tags_25 },
    { 0x664eb581, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0aeb585, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbbeeb587, SUC_lexicon_tags_19 },
    { 0xaaeb588, SUC_lexicon_tags_17 },
    { 0x8aeb589, SUC_lexicon_tags_60 },
    { 0x46eb58a, SUC_lexicon_tags_33 },
    { 0x8e4eb588, SUC_lexicon_tags_33 },
    { 0x27ceb58c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc32eb58e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2d6eb590, SUC_lexicon_tags_23 },
    { 0x1f8eb591, SUC_lexicon_tags_29 },
    { 0x23aeb591, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8eb595, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x672eb598, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ceb59c, SUC_lexicon_tags_15 },
    { 0x462eb59d, SUC_lexicon_tags_68 },
    { 0xcdaeb59d, SUC_lexicon_tags_21 },
    { 0x36eeb59f, SUC_lexicon_tags_2 },
    { 0x538eb5a0, SUC_lexicon_tags_12 },
    { 0xedceb5a1, SUC_lexicon_tags_29 },
    { 0x180eb5a2, SUC_lexicon_tags_2 },
    { 0xb52eb5a3, SUC_lexicon_tags_11 },
    { 0xdd8eb5a2, SUC_lexicon_tags_17 },
    { 0xf6ceb5a1, SUC_lexicon_tags_11 },
    { 0x896eb5a1, SUC_lexicon_tags_26 },
    { 0x81eeb5a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42eb5ab, SUC_lexicon_tags_23 },
    { 0xaeceb5ac, SUC_lexicon_tags_0 },
    { 0x80eb5ad, SUC_lexicon_tags_49 },
    { 0xa40eb5ae, SUC_lexicon_tags_29 },
    { 0x7e0eb5af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x614eb5b2, SUC_lexicon_tags_11 },
    { 0xbb6eb5b2, SUC_lexicon_tags_10 },
    { 0x7eeb5b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadeeb5b7, SUC_lexicon_tags_7 },
    { 0x856eb5b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21eeb5bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76eb5bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6feeb5c2, SUC_lexicon_tags_65 },
    { 0x828eb5c2, SUC_lexicon_tags_17 },
    { 0xb16eb5c4, SUC_lexicon_tags_37 },
    { 0x49eeb5c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf3ceb5c7, SUC_lexicon_tags_35 },
    { 0x8a4eb5c8, SUC_lexicon_tags_12 },
    { 0xc88eb5c9, SUC_lexicon_tags_37 },
    { 0x3daeb5c7, SUC_lexicon_tags_14 },
    { 0x76eb5cb, SUC_lexicon_tags_87 },
    { 0x832eb5cc, SUC_lexicon_tags_49 },
    { 0x48aeb5cd, SUC_lexicon_tags_19 },
    { 0x98aeb5ce, SUC_lexicon_tags_1 },
    { 0x11aeb5cf, SUC_lexicon_tags_49 },
    { 0xf22eb5d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb34eb5d2, SUC_lexicon_tags_2 },
    { 0x18aeb5d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54eb5d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72eb5db, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7caeb5de, SUC_lexicon_tags_14 },
    { 0x736eb5de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742eb5e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x848eb5e7, SUC_lexicon_tags_61 },
    { 0xebceb5e8, SUC_lexicon_tags_13 },
    { 0x702eb5e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a0eb5eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8eb5ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15ceb5f0, SUC_lexicon_tags_11 },
    { 0x1bceb5f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370eb5f6, SUC_lexicon_tags_15 },
    { 0x43ceb5f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6eb5fb, SUC_lexicon_tags_25 },
    { 0xf98eb5fb, SUC_lexicon_tags_19 },
    { 0x770eb5fd, SUC_lexicon_tags_17 },
    { 0xaf6eb5fe, SUC_lexicon_tags_11 },
    { 0x728eb5ff, SUC_lexicon_tags_6 },
    { 0xca6eb600, SUC_lexicon_tags_23 },
    { 0xbe4eb600, SUC_lexicon_tags_28 },
    { 0x8f4eb602, SUC_lexicon_tags_28 },
    { 0x7c6eb5fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x902eb606, SUC_lexicon_tags_65 },
    { 0x7b6eb607, SUC_lexicon_tags_7 },
    { 0xcb8eb608, SUC_lexicon_tags_43 },
    { 0x1f4eb609, SUC_lexicon_tags_45 },
    { 0xceeb609, SUC_lexicon_tags_12 },
    { 0x3b0eb609, SUC_lexicon_tags_6 },
    { 0x98ceb60c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eeb612, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0eb616, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x678eb618, SUC_lexicon_tags_4 },
    { 0xa2aeb618, SUC_lexicon_tags_10 },
    { 0xb2eeb61a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeb61e, SUC_lexicon_tags_6 },
    { 0x84eb61f, SUC_lexicon_tags_47 },
    { 0x24aeb620, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20ceb622, SUC_lexicon_tags_25 },
    { 0x6a4eb622, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eceb626, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x888eb629, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4baeb62e, SUC_lexicon_tags_43 },
    { 0x86eb62f, SUC_lexicon_tags_17 },
    { 0xe1eeb630, SUC_lexicon_tags_39 },
    { 0x9a6eb631, SUC_lexicon_tags_21 },
    { 0xae4eb631, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaeb63b, SUC_lexicon_tags_68 },
    { 0x5a6eb63c, SUC_lexicon_tags_37 },
    { 0x2e0eb63b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118eb641, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8eb646, SUC_lexicon_tags_7 },
    { 0xcc4eb647, SUC_lexicon_tags_15 },
    { 0x6ceb646, SUC_lexicon_tags_15 },
    { 0x182eb649, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x74aeb64c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83aeb650, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688eb654, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4eb657, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892eb65e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb80eb660, SUC_lexicon_tags_38 },
    { 0xce2eb661, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812eb667, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x598eb669, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6eb66e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfdceb670, SUC_lexicon_tags_19 },
    { 0x20aeb671, SUC_lexicon_tags_15 },
    { 0x26eeb672, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72eb67b, SUC_lexicon_tags_25 },
    { 0xa7eeb67b, SUC_lexicon_tags_33 },
    { 0x6b2eb67c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6eb682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ceb687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaceb68e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2eb692, SUC_lexicon_tags_7 },
    { 0x7eeeb693, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbcaeb695, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b0eb697, SUC_lexicon_tags_0 },
    { 0x2ceb698, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8aeb69d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdaeeb69f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe72eb6a1, SUC_lexicon_tags_25 },
    { 0xf12eb6a2, SUC_lexicon_tags_8 },
    { 0x866eb6a2, SUC_lexicon_tags_11 },
    { 0x6a8eb6a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68eb6a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faeb6ad, SUC_lexicon_tags_46 },
    { 0xaceeb6ae, SUC_lexicon_tags_33 },
    { 0x968eb6ae, SUC_lexicon_tags_25 },
    { 0xd6eb6b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178eb6b4, SUC_lexicon_tags_7 },
    { 0xf3eeb6b5, SUC_lexicon_tags_27 },
    { 0x48eb6b6, SUC_lexicon_tags_17 },
    { 0x22eb6b7, SUC_lexicon_tags_32 },
    { 0x602eb6b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9eeb6bd, SUC_lexicon_tags_32 },
    { 0xbb2eb6be, SUC_lexicon_tags_47 },
    { 0x97eeb6bf, SUC_lexicon_tags_23 },
    { 0xbf6eb6c0, SUC_lexicon_tags_0 },
    { 0x47eeb6c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eeb6c7, SUC_lexicon_tags_39 },
    { 0x47ceb6c8, SUC_lexicon_tags_74 },
    { 0xee2eb6c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4eb6cc, SUC_lexicon_tags_2 },
    { 0xdaeeb6cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6aeb6d2, SUC_lexicon_tags_8 },
    { 0xdf0eb6d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6aeb6d6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9eaeb6d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ceb6dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6eb6e1, SUC_lexicon_tags_14 },
    { 0x5b4eb6e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbeaeb6e4, SUC_lexicon_tags_17 },
    { 0x1b6eb6e4, SUC_lexicon_tags_30 },
    { 0xafceb6e6, SUC_lexicon_tags_37 },
    { 0x504eb6e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x51ceb6e9, SUC_lexicon_tags_43 },
    { 0xfeeeb6ea, SUC_lexicon_tags_23 },
    { 0xc8aeb6eb, SUC_lexicon_tags_17 },
    { 0xb8ceb6ec, SUC_lexicon_tags_27 },
    { 0x976eb6ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52eeb6ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0eb6f5, SUC_lexicon_tags_37 },
    { 0x2e0eb6f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96eb6fc, SUC_lexicon_tags_48 },
    { 0x2f2eb6fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc02eb6ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9eeb701, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ceb705, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeeeeb707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc42eb709, SUC_lexicon_tags_17 },
    { 0xb9ceb70a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450eb710, SUC_lexicon_tags_11 },
    { 0x24aeb710, SUC_lexicon_tags_96 },
    { 0xc14eb710, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x684eb714, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x56aeb716, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eeb719, SUC_lexicon_tags_7 },
    { 0x25ceb71a, SUC_lexicon_tags_1 },
    { 0x420eb719, SUC_lexicon_tags_14 },
    { 0xfbaeb71b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6eb71f, SUC_lexicon_tags_23 },
    { 0x266eb720, SUC_lexicon_tags_26 },
    { 0x352eb71f, SUC_lexicon_tags_2 },
    { 0xb6eeb722, SUC_lexicon_tags_15 },
    { 0x8eaeb723, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652eb728, SUC_lexicon_tags_14 },
    { 0xc66eb729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0eb72e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43eeb732, SUC_lexicon_tags_32 },
    { 0xd00eb732, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8eb737, SUC_lexicon_tags_17 },
    { 0xd34eb737, SUC_lexicon_tags_66 },
    { 0x17ceb739, SUC_lexicon_tags_47 },
    { 0x3d8eb73a, SUC_lexicon_tags_503 },
    { 0x290eb73b, SUC_lexicon_tags_37 },
    { 0x14eeb73c, SUC_lexicon_tags_2 },
    { 0x29aeb73d, SUC_lexicon_tags_29 },
    { 0x304eb73d, SUC_lexicon_tags_17 },
    { 0xfb0eb73f, SUC_lexicon_tags_11 },
    { 0xfaceb740, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94aeb742, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584eb74a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x586eb74c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26eb752, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2eb759, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc36eb75b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe4eb75d, SUC_lexicon_tags_37 },
    { 0x8daeb75e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9ceb760, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaeeb763, SUC_lexicon_tags_15 },
    { 0xc94eb764, SUC_lexicon_tags_37 },
    { 0x632eb765, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6eb769, SUC_lexicon_tags_17 },
    { 0x600eb76a, SUC_lexicon_tags_8 },
    { 0xabaeb76b, SUC_lexicon_tags_22 },
    { 0x456eb76c, SUC_lexicon_tags_33 },
    { 0x154eb76a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84eb770, SUC_lexicon_tags_29 },
    { 0x1beeb771, SUC_lexicon_tags_3 },
    { 0xc90eb771, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x80aeb775, SUC_lexicon_tags_17 },
    { 0xb4eeb776, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56aeb77c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2eb780, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04eb788, SUC_lexicon_tags_17 },
    { 0x30ceb789, SUC_lexicon_tags_14 },
    { 0xb60eb789, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xcc8eb78c, SUC_lexicon_tags_138 },
    { 0xc1aeb78d, SUC_lexicon_tags_15 },
    { 0xa2ceb78e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x140eb790, SUC_lexicon_tags_17 },
    { 0x274eb791, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320eb797, SUC_lexicon_tags_34 },
    { 0xc58eb797, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17aeb79a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x142eb79d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6eb7a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e4eb7a2, SUC_lexicon_tags_32 },
    { 0x778eb7a3, SUC_lexicon_tags_49 },
    { 0x844eb7a4, SUC_lexicon_tags_8 },
    { 0xb46eb7a3, SUC_lexicon_tags_12 },
    { 0x59ceb7a6, SUC_lexicon_tags_4 },
    { 0xe54eb7a7, SUC_lexicon_tags_28 },
    { 0x850eb7a2, SUC_lexicon_tags_66 },
    { 0x54eeb7a9, SUC_lexicon_tags_9 },
    { 0xe94eb7a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaaeeb7ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74eb7af, SUC_lexicon_tags_25 },
    { 0xf5aeb7af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0eb7b3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeb7b8, SUC_lexicon_tags_1 },
    { 0xa86eb7b9, SUC_lexicon_tags_28 },
    { 0xc9eeb7b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe6eb7bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0eb7c0, SUC_lexicon_tags_25 },
    { 0xf4ceb7c1, SUC_lexicon_tags_29 },
    { 0x29aeb7c2, SUC_lexicon_tags_12 },
    { 0x69eeb7c3, SUC_lexicon_tags_29 },
    { 0xbfaeb7c1, SUC_lexicon_tags_17 },
    { 0xbd8eb7c0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x216eb7c7, SUC_lexicon_tags_25 },
    { 0x628eb7c8, SUC_lexicon_tags_2 },
    { 0x6a2eb7c9, SUC_lexicon_tags_25 },
    { 0x56eb7ca, SUC_lexicon_tags_79 },
    { 0x910eb7cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd8aeb7cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceb7d1, SUC_lexicon_tags_2 },
    { 0x85ceb7d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x858eb7d4, SUC_lexicon_tags_37 },
    { 0x4c8eb7d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64eb7dc, SUC_lexicon_tags_12 },
    { 0x5baeb7dc, SUC_lexicon_tags_1 },
    { 0x5d2eb7de, SUC_lexicon_tags_11 },
    { 0xe2ceb7dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8eb7e6, SUC_lexicon_tags_63 },
    { 0xa28eb7e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaeb7ea, SUC_lexicon_tags_43 },
    { 0x384eb7ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15aeb7f2, SUC_lexicon_tags_2 },
    { 0x6f6eb7f3, SUC_lexicon_tags_12 },
    { 0xeb4eb7f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ceb7f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7daeb7f9, SUC_lexicon_tags_27 },
    { 0xf4eb7f9, SUC_lexicon_tags_28 },
    { 0xaeceb7fb, SUC_lexicon_tags_65 },
    { 0x770eb7fc, SUC_lexicon_tags_27 },
    { 0x5aceb7fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aeb800, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x860eb802, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70eb805, SUC_lexicon_tags_7 },
    { 0x92ceb805, SUC_lexicon_tags_17 },
    { 0x4aceb805, SUC_lexicon_tags_72 },
    { 0xbd8eb808, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aeb80e, SUC_lexicon_tags_1 },
    { 0xb30eb80f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c2eb811, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684eb817, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4eb81a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daeb81e, SUC_lexicon_tags_3 },
    { 0x918eb81f, SUC_lexicon_tags_23 },
    { 0xe7aeb820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ceb825, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18eb828, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x220eb82a, SUC_lexicon_tags_12 },
    { 0xf20eb82a, SUC_lexicon_tags_11 },
    { 0xd82eb82c, SUC_lexicon_tags_6 },
    { 0x6beeb82a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506eb831, SUC_lexicon_tags_2 },
    { 0x684eb832, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5eeb838, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86eb83c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436eb842, SUC_lexicon_tags_28 },
    { 0xa82eb843, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x88eeb845, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42eeb84b, SUC_lexicon_tags_26 },
    { 0x2eb84c, SUC_lexicon_tags_26 },
    { 0x9b6eb84d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x456eb84f, SUC_lexicon_tags_37 },
    { 0x6beeb84f, SUC_lexicon_tags_21 },
    { 0xfc2eb850, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faeb855, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xf86eb857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ceb85c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890eb862, SUC_lexicon_tags_37 },
    { 0x8e6eb862, SUC_lexicon_tags_61 },
    { 0x9aaeb862, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaeb867, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5fceb869, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaeb86e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3bceb870, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3eaeb872, SUC_lexicon_tags_1 },
    { 0x986eb873, SUC_lexicon_tags_8 },
    { 0xa8ceb872, SUC_lexicon_tags_11 },
    { 0xc54eb874, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fceb87b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ceeb87e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x522eb880, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74eb886, SUC_lexicon_tags_11 },
    { 0xaeeb886, SUC_lexicon_tags_6 },
    { 0x482eb886, SUC_lexicon_tags_37 },
    { 0xa78eb887, SUC_lexicon_tags_37 },
    { 0x8f2eb889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x540eb88c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836eb890, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe52eb893, SUC_lexicon_tags_2 },
    { 0xac4eb893, SUC_lexicon_tags_69 },
    { 0xfe2eb895, SUC_lexicon_tags_49 },
    { 0xc78eb896, SUC_lexicon_tags_15 },
    { 0x866eb896, SUC_lexicon_tags_37 },
    { 0x438eb898, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81aeb89c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb98eb89e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c8eb8a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8eb8a8, SUC_lexicon_tags_15 },
    { 0x5cceb8a9, SUC_lexicon_tags_33 },
    { 0x41aeb8aa, SUC_lexicon_tags_29 },
    { 0x69eeb8aa, SUC_lexicon_tags_25 },
    { 0x3deeb8ac, SUC_lexicon_tags_8 },
    { 0xf00eb8aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308eb8b1, SUC_lexicon_tags_37 },
    { 0xf22eb8b2, SUC_lexicon_tags_24 },
    { 0x966eb8b3, SUC_lexicon_tags_37 },
    { 0x7f2eb8b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf88eb8b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70eb8b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x610eb8bb, SUC_lexicon_tags_68 },
    { 0x5c2eb8bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50eeb8be, SUC_lexicon_tags_138 },
    { 0x1d8eb8be, SUC_lexicon_tags_60 },
    { 0x628eb8bf, SUC_lexicon_tags_62 },
    { 0x4fceb8c1, SUC_lexicon_tags_39 },
    { 0xcc0eb8c2, SUC_lexicon_tags_17 },
    { 0x1b6eb8c1, SUC_lexicon_tags_25 },
    { 0x9a8eb8c0, SUC_lexicon_tags_14 },
    { 0xc34eb8c0, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x542eb8c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2eeb8ca, SUC_lexicon_tags_84 },
    { 0xc96eb8ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2eb8ce, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00eb8d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafeeb8db, SUC_lexicon_tags_15 },
    { 0x32eeb8dc, SUC_lexicon_tags_17 },
    { 0xcb2eb8db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x198eb8e0, SUC_lexicon_tags_37 },
    { 0x1d0eb8e1, SUC_lexicon_tags_7 },
    { 0x78ceb8e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf2eb8e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5ceb8e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcaeeb8e8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2eb8eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x77ceb8ed, SUC_lexicon_tags_37 },
    { 0xbf8eb8ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x46ceb8f0, SUC_lexicon_tags_1 },
    { 0x726eb8f1, SUC_lexicon_tags_26 },
    { 0x9caeb8f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeeb8f5, SUC_lexicon_tags_17 },
    { 0x6f4eb8f6, SUC_lexicon_tags_80 },
    { 0xa74eb8f7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aceb8fa, SUC_lexicon_tags_1 },
    { 0xe92eb8fb, SUC_lexicon_tags_10 },
    { 0x858eb8fb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xcaeb8fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94eb901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74aeb906, SUC_lexicon_tags_17 },
    { 0xe98eb907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd76eb909, SUC_lexicon_tags_45 },
    { 0x79ceb90a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93eeb911, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2deeb913, SUC_lexicon_tags_60 },
    { 0x392eb914, SUC_lexicon_tags_14 },
    { 0xd16eb915, SUC_lexicon_tags_47 },
    { 0xe8ceb916, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2eb91a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2eb91d, SUC_lexicon_tags_38 },
    { 0xd00eb91d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaeb922, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x220eb924, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b8eb926, SUC_lexicon_tags_37 },
    { 0x4ceb927, SUC_lexicon_tags_45 },
    { 0xeceeb928, SUC_lexicon_tags_47 },
    { 0x2d8eb929, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe6aeb92b, SUC_lexicon_tags_17 },
    { 0x34ceb92b, SUC_lexicon_tags_2 },
    { 0x63aeb92b, SUC_lexicon_tags_25 },
    { 0x4d2eb92c, SUC_lexicon_tags_1 },
    { 0xa2ceb92f, SUC_lexicon_tags_37 },
    { 0x934eb930, SUC_lexicon_tags_17 },
    { 0xf6eb931, SUC_lexicon_tags_609 },
    { 0xa22eb931, SUC_lexicon_tags_19 },
    { 0x9e0eb92f, SUC_lexicon_tags_37 },
    { 0x1a6eb934, SUC_lexicon_tags_9 },
    { 0x984eb934, SUC_lexicon_tags_0 },
    { 0x4eceb92c, SUC_lexicon_tags_68 },
    { 0x38eeb937, SUC_lexicon_tags_15 },
    { 0x88aeb938, SUC_lexicon_tags_17 },
    { 0xfbceb935, SUC_lexicon_tags_25 },
    { 0xf34eb93a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72eb944, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xe30eb946, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26eb949, SUC_lexicon_tags_12 },
    { 0x100eb949, SUC_lexicon_tags_82 },
    { 0x39ceb94a, SUC_lexicon_tags_7 },
    { 0xe52eb94c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eb953, SUC_lexicon_tags_437 },
    { 0xc00eb954, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x200eb956, SUC_lexicon_tags_12 },
    { 0x974eb957, SUC_lexicon_tags_25 },
    { 0x848eb958, SUC_lexicon_tags_282 },
    { 0x9f4eb956, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x768eb95b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ceb95e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40eeb964, SUC_lexicon_tags_19 },
    { 0x3d2eb965, SUC_lexicon_tags_12 },
    { 0x70eb966, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aeb96f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ceb973, SUC_lexicon_tags_17 },
    { 0x9b2eb974, SUC_lexicon_tags_37 },
    { 0xb06eb973, SUC_lexicon_tags_17 },
    { 0xbe2eb975, SUC_lexicon_tags_7 },
    { 0x5f8eb977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x622eb979, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x44eeb97b, SUC_lexicon_tags_47 },
    { 0xe42eb97b, SUC_lexicon_tags_33 },
    { 0x236eb97b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98eb982, SUC_lexicon_tags_12 },
    { 0x89eeb983, SUC_lexicon_tags_0 },
    { 0x47eeb984, SUC_lexicon_tags_9 },
    { 0x17eeb984, SUC_lexicon_tags_25 },
    { 0x606eb986, SUC_lexicon_tags_17 },
    { 0xeeaeb987, SUC_lexicon_tags_63 },
    { 0xeaeeb986, SUC_lexicon_tags_23 },
    { 0x4aeeb989, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4eb98f, SUC_lexicon_tags_33 },
    { 0x6f0eb990, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02eb993, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x558eb995, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebaeb997, SUC_lexicon_tags_49 },
    { 0x210eb998, SUC_lexicon_tags_1 },
    { 0xbd6eb999, SUC_lexicon_tags_14 },
    { 0x45eeb999, SUC_lexicon_tags_9 },
    { 0x624eb99b, SUC_lexicon_tags_25 },
    { 0xdd6eb99a, SUC_lexicon_tags_11 },
    { 0x19ceb99d, SUC_lexicon_tags_19 },
    { 0x1eeb99e, SUC_lexicon_tags_29 },
    { 0xd2aeb99f, SUC_lexicon_tags_15 },
    { 0x7aeb9a0, SUC_lexicon_tags_2 },
    { 0xb2eeb9a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbfceb9a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eeeb9a6, SUC_lexicon_tags_7 },
    { 0x5b8eb9a6, SUC_lexicon_tags_11 },
    { 0x14ceb9a6, SUC_lexicon_tags_60 },
    { 0x7f2eb9a9, SUC_lexicon_tags_7 },
    { 0x1daeb9a6, SUC_lexicon_tags_1 },
    { 0xad2eb9a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9eeb9b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52eb9b5, SUC_lexicon_tags_25 },
    { 0x1e4eb9b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c8eb9b8, SUC_lexicon_tags_92 },
    { 0xb38eb9b8, SUC_lexicon_tags_1 },
    { 0xdd8eb9ba, SUC_lexicon_tags_37 },
    { 0xcbeeb9bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38eb9c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972eb9c7, SUC_lexicon_tags_15 },
    { 0xadceb9c8, SUC_lexicon_tags_37 },
    { 0x7dceb9c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6eb9cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x994eb9d0, SUC_lexicon_tags_55 },
    { 0x6aeb9d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8eeb9d4, SUC_lexicon_tags_15 },
    { 0x178eb9d5, SUC_lexicon_tags_47 },
    { 0x738eb9d6, SUC_lexicon_tags_45 },
    { 0x922eb9d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eeb9dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4feeb9de, SUC_lexicon_tags_15 },
    { 0x88ceb9df, SUC_lexicon_tags_26 },
    { 0xdeeb9df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546eb9e5, SUC_lexicon_tags_63 },
    { 0x43aeb9e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0eb9ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafeeb9ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8faeb9f1, SUC_lexicon_tags_47 },
    { 0xc2eeb9f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72eb9f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ceb9f9, SUC_lexicon_tags_26 },
    { 0x296eb9fa, SUC_lexicon_tags_956 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2eba00, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ceba03, SUC_lexicon_tags_33 },
    { 0x7ceba03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ceba08, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe0eba0a, SUC_lexicon_tags_21 },
    { 0xcfaeba0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e6eba0d, SUC_lexicon_tags_12 },
    { 0x52eeba0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650eba13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2eba17, SUC_lexicon_tags_34 },
    { 0x694eba18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194eba1d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42eba21, SUC_lexicon_tags_56 },
    { 0x12eba22, SUC_lexicon_tags_23 },
    { 0xd26eba23, SUC_lexicon_tags_11 },
    { 0xc7ceba24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x188eba26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eba29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbeeba2e, SUC_lexicon_tags_63 },
    { 0x460eba2f, SUC_lexicon_tags_2 },
    { 0x184eba30, SUC_lexicon_tags_15 },
    { 0x93ceba31, SUC_lexicon_tags_38 },
    { 0x4aeeba30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ceba35, SUC_lexicon_tags_28 },
    { 0x81aeba35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30eba39, SUC_lexicon_tags_19 },
    { 0xad4eba3a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94eba40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaceba45, SUC_lexicon_tags_11 },
    { 0x628eba46, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaeba49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6eba4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x650eba4f, SUC_lexicon_tags_37 },
    { 0xc48eba50, SUC_lexicon_tags_37 },
    { 0x6b0eba51, SUC_lexicon_tags_35 },
    { 0x6dceba51, SUC_lexicon_tags_91 },
    { 0xca0eba51, SUC_lexicon_tags_29 },
    { 0x68aeba54, SUC_lexicon_tags_39 },
    { 0xb10eba54, SUC_lexicon_tags_25 },
    { 0xe04eba56, SUC_lexicon_tags_30 },
    { 0xb0aeba57, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x424eba59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4eba5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda2eba5e, SUC_lexicon_tags_11 },
    { 0xa74eba5f, SUC_lexicon_tags_47 },
    { 0x6d2eba60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6aeeba62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeaeeba64, SUC_lexicon_tags_37 },
    { 0xa40eba65, SUC_lexicon_tags_14 },
    { 0x9deeba66, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x91ceba68, SUC_lexicon_tags_27 },
    { 0x836eba69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x126eba6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0eba70, SUC_lexicon_tags_11 },
    { 0x180eba71, SUC_lexicon_tags_11 },
    { 0xd86eba71, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7c4eba74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6aeba79, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x584eba7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6eba7f, SUC_lexicon_tags_15 },
    { 0x9aceba80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80eba83, SUC_lexicon_tags_37 },
    { 0xf76eba84, SUC_lexicon_tags_60 },
    { 0xc70eba85, SUC_lexicon_tags_1 },
    { 0x82eeba86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaceba89, SUC_lexicon_tags_998 },
    { 0x9a2eba8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc90eba8c, SUC_lexicon_tags_2 },
    { 0x48ceba8c, SUC_lexicon_tags_17 },
    { 0x2d8eba8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84aeba90, SUC_lexicon_tags_15 },
    { 0xfd2eba90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eeeba96, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2aeba9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x44aeba9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00ebaa6, SUC_lexicon_tags_1 },
    { 0xc98ebaa7, SUC_lexicon_tags_25 },
    { 0x96eebaa7, SUC_lexicon_tags_2 },
    { 0x4a0ebaa8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18ebaae, SUC_lexicon_tags_15 },
    { 0x578ebaaf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aebab2, SUC_lexicon_tags_48 },
    { 0xf82ebab2, SUC_lexicon_tags_26 },
    { 0x88eebab4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63eebab7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6ebaba, SUC_lexicon_tags_25 },
    { 0xa64ebabb, SUC_lexicon_tags_27 },
    { 0xdeeebabc, SUC_lexicon_tags_25 },
    { 0xe9eebabd, SUC_lexicon_tags_11 },
    { 0x5e8ebaba, SUC_lexicon_tags_19 },
    { 0x742ebabf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33eebac1, SUC_lexicon_tags_6 },
    { 0x7ccebac2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4caebac4, SUC_lexicon_tags_7 },
    { 0xa02ebac5, SUC_lexicon_tags_63 },
    { 0x1e4ebac5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4ebac9, SUC_lexicon_tags_33 },
    { 0x81eebac9, SUC_lexicon_tags_25 },
    { 0x888ebacb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8ebad2, SUC_lexicon_tags_68 },
    { 0x4e6ebad3, SUC_lexicon_tags_17 },
    { 0x486ebad3, SUC_lexicon_tags_33 },
    { 0xaeaebad5, SUC_lexicon_tags_27 },
    { 0x1d0ebad5, SUC_lexicon_tags_27 },
    { 0xfd2ebad7, SUC_lexicon_tags_2 },
    { 0x210ebad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93eebada, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16ebadd, SUC_lexicon_tags_12 },
    { 0x542ebadd, SUC_lexicon_tags_6 },
    { 0x5f6ebadf, SUC_lexicon_tags_15 },
    { 0xc46ebae0, SUC_lexicon_tags_27 },
    { 0x916ebae1, SUC_lexicon_tags_47 },
    { 0x4ebae0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc98ebae4, SUC_lexicon_tags_43 },
    { 0xb04ebae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x572ebae7, SUC_lexicon_tags_11 },
    { 0xc58ebae8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2ebaee, SUC_lexicon_tags_43 },
    { 0xf2eebaef, SUC_lexicon_tags_25 },
    { 0x5acebaf0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ebaf3, SUC_lexicon_tags_14 },
    { 0x726ebaf4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0ebaf8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462ebafe, SUC_lexicon_tags_17 },
    { 0x6a0ebafe, SUC_lexicon_tags_33 },
    { 0xfa6ebb00, SUC_lexicon_tags_19 },
    { 0x678ebb01, SUC_lexicon_tags_23 },
    { 0xc72ebb01, SUC_lexicon_tags_47 },
    { 0x3c0ebb03, SUC_lexicon_tags_45 },
    { 0x8d8ebb04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf32ebb06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x828ebb08, SUC_lexicon_tags_37 },
    { 0xb6eebb09, SUC_lexicon_tags_1 },
    { 0x844ebb0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366ebb0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x992ebb10, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x392ebb12, SUC_lexicon_tags_23 },
    { 0xccaebb13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97aebb15, SUC_lexicon_tags_27 },
    { 0xb0ebb15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eebb19, SUC_lexicon_tags_37 },
    { 0xf32ebb1a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12ebb20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x582ebb22, SUC_lexicon_tags_38 },
    { 0x3b6ebb23, SUC_lexicon_tags_7 },
    { 0xebb24, SUC_lexicon_tags_29 },
    { 0xb8ebb23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04ebb2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73cebb2d, SUC_lexicon_tags_2 },
    { 0x8aebb2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648ebb32, SUC_lexicon_tags_17 },
    { 0x3eeebb32, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcebb38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf8ebb3a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12ebb42, SUC_lexicon_tags_9 },
    { 0x3c0ebb42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952ebb48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf4ebb4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28ebb4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4ebb54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcebb58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x288ebb5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1aebb5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25aebb5f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x980ebb61, SUC_lexicon_tags_45 },
    { 0xb3eebb62, SUC_lexicon_tags_108 },
    { 0x270ebb61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67aebb66, SUC_lexicon_tags_8 },
    { 0xc3aebb67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5fcebb69, SUC_lexicon_tags_38 },
    { 0x182ebb6a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cebb73, SUC_lexicon_tags_25 },
    { 0x3f8ebb74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x694ebb76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf80ebb78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x898ebb7a, SUC_lexicon_tags_7 },
    { 0xd56ebb7b, SUC_lexicon_tags_1 },
    { 0xc7cebb7b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aebb80, SUC_lexicon_tags_14 },
    { 0xfdaebb81, SUC_lexicon_tags_7 },
    { 0xe14ebb82, SUC_lexicon_tags_23 },
    { 0x482ebb83, SUC_lexicon_tags_234 },
    { 0xf1eebb84, SUC_lexicon_tags_25 },
    { 0x130ebb80, SUC_lexicon_tags_11 },
    { 0x220ebb86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ebb8b, SUC_lexicon_tags_60 },
    { 0x4b4ebb8c, SUC_lexicon_tags_6 },
    { 0x39eebb8b, SUC_lexicon_tags_12 },
    { 0x930ebb8e, SUC_lexicon_tags_7 },
    { 0x5d2ebb8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfecebb92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x290ebb94, SUC_lexicon_tags_47 },
    { 0x236ebb95, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38aebb9b, SUC_lexicon_tags_60 },
    { 0x1caebb9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7beebba0, SUC_lexicon_tags_11 },
    { 0xca8ebba1, SUC_lexicon_tags_22 },
    { 0x27aebba1, SUC_lexicon_tags_46 },
    { 0xbe0ebba3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84ebba6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce4ebba8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x95aebbab, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6ebbb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a6ebbb7, SUC_lexicon_tags_14 },
    { 0xca8ebbb7, SUC_lexicon_tags_27 },
    { 0xb58ebbb8, SUC_lexicon_tags_17 },
    { 0x5f6ebbba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf2ebbbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5cebbbe, SUC_lexicon_tags_27 },
    { 0xd76ebbbf, SUC_lexicon_tags_40 },
    { 0xc4aebbbe, SUC_lexicon_tags_60 },
    { 0xba8ebbc0, SUC_lexicon_tags_3 },
    { 0x4f2ebbc2, SUC_lexicon_tags_37 },
    { 0x4e6ebbc3, SUC_lexicon_tags_49 },
    { 0xe26ebbc4, SUC_lexicon_tags_25 },
    { 0x422ebbc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8ebbc9, SUC_lexicon_tags_160 },
    { 0xe8eebbc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c8ebbcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a6ebbce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcebbd1, SUC_lexicon_tags_17 },
    { 0xc50ebbd2, SUC_lexicon_tags_19 },
    { 0x1ccebbd3, SUC_lexicon_tags_43 },
    { 0xb3eebbd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ebbd9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x87eebbdc, SUC_lexicon_tags_60 },
    { 0x6d6ebbdc, SUC_lexicon_tags_6 },
    { 0x826ebbde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aaebbe3, SUC_lexicon_tags_0 },
    { 0xdeaebbe4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2ebbe7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ebbea, SUC_lexicon_tags_15 },
    { 0x724ebbeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20ebbef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x198ebbf2, SUC_lexicon_tags_11 },
    { 0x32ebbf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28ebbfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b4ebbff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308ebc07, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acebc0a, SUC_lexicon_tags_28 },
    { 0x5e6ebc0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe80ebc0e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeecebc10, SUC_lexicon_tags_37 },
    { 0xba6ebc10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x980ebc13, SUC_lexicon_tags_66 },
    { 0xcb6ebc14, SUC_lexicon_tags_7 },
    { 0xaa0ebc15, SUC_lexicon_tags_60 },
    { 0x788ebc16, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ebc19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x776ebc1b, SUC_lexicon_tags_47 },
    { 0x904ebc1c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44ebc20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2ebc23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x184ebc25, SUC_lexicon_tags_68 },
    { 0x450ebc25, SUC_lexicon_tags_55 },
    { 0xde2ebc27, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50ebc2e, SUC_lexicon_tags_3 },
    { 0x1e0ebc2e, SUC_lexicon_tags_29 },
    { 0x24eebc30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272ebc34, SUC_lexicon_tags_8 },
    { 0x2e8ebc35, SUC_lexicon_tags_39 },
    { 0x25cebc36, SUC_lexicon_tags_31 },
    { 0x21eebc37, SUC_lexicon_tags_37 },
    { 0x2ecebc38, SUC_lexicon_tags_45 },
    { 0x388ebc37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ebc3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02ebc40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60ebc47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8ebc4a, SUC_lexicon_tags_63 },
    { 0x500ebc4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508ebc4f, SUC_lexicon_tags_14 },
    { 0x680ebc50, SUC_lexicon_tags_34 },
    { 0xd1cebc51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976ebc55, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256ebc5e, SUC_lexicon_tags_4 },
    { 0xb34ebc5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x568ebc62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3aebc64, SUC_lexicon_tags_48 },
    { 0xd5cebc65, SUC_lexicon_tags_25 },
    { 0x6f8ebc65, SUC_lexicon_tags_17 },
    { 0x42aebc64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x79eebc69, SUC_lexicon_tags_11 },
    { 0xf0ebc6a, SUC_lexicon_tags_21 },
    { 0xaf6ebc6a, SUC_lexicon_tags_11 },
    { 0xef6ebc69, SUC_lexicon_tags_63 },
    { 0xbf4ebc6a, SUC_lexicon_tags_7 },
    { 0xcdaebc6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd84ebc70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f6ebc72, SUC_lexicon_tags_7 },
    { 0x410ebc73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x558ebc79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x934ebc7b, SUC_lexicon_tags_6 },
    { 0xd10ebc7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56aebc7e, SUC_lexicon_tags_29 },
    { 0x3a2ebc7e, SUC_lexicon_tags_12 },
    { 0xd10ebc80, SUC_lexicon_tags_0 },
    { 0xd80ebc7f, SUC_lexicon_tags_17 },
    { 0xbf2ebc82, SUC_lexicon_tags_68 },
    { 0x7c6ebc83, SUC_lexicon_tags_17 },
    { 0xf46ebc84, SUC_lexicon_tags_123 },
    { 0x1c2ebc84, SUC_lexicon_tags_62 },
    { 0x974ebc85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170ebc8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x732ebc8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xec2ebc91, SUC_lexicon_tags_63 },
    { 0x660ebc92, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204ebc9b, SUC_lexicon_tags_18 },
    { 0xa6cebc9c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4ebca1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82ebca4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cebca9, SUC_lexicon_tags_29 },
    { 0x3eebcaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394ebcb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccebcb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x608ebcba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118ebcc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ebcc8, SUC_lexicon_tags_19 },
    { 0xc72ebcc9, SUC_lexicon_tags_11 },
    { 0x416ebcca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5aebccd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cebcd2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceebcdc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ebcdf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2ebce2, SUC_lexicon_tags_19 },
    { 0xa8ebce3, SUC_lexicon_tags_11 },
    { 0xe80ebce3, SUC_lexicon_tags_47 },
    { 0xfaebce5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58eebce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x352ebce9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x392ebceb, SUC_lexicon_tags_39 },
    { 0xa52ebcec, SUC_lexicon_tags_11 },
    { 0x2aeebced, SUC_lexicon_tags_15 },
    { 0x548ebcec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eebcf4, SUC_lexicon_tags_12 },
    { 0xab2ebcf5, SUC_lexicon_tags_12 },
    { 0xbaaebcf4, SUC_lexicon_tags_17 },
    { 0x774ebcf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87eebcfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306ebd01, SUC_lexicon_tags_8 },
    { 0x226ebd01, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cebd08, SUC_lexicon_tags_17 },
    { 0xf94ebd09, SUC_lexicon_tags_57 },
    { 0x9e8ebd0a, SUC_lexicon_tags_68 },
    { 0xd82ebd0b, SUC_lexicon_tags_37 },
    { 0xa02ebd0c, SUC_lexicon_tags_10 },
    { 0xc88ebd0d, SUC_lexicon_tags_11 },
    { 0x952ebd0e, SUC_lexicon_tags_17 },
    { 0x910ebd0f, SUC_lexicon_tags_17 },
    { 0x116ebd0f, SUC_lexicon_tags_25 },
    { 0xe6cebd11, SUC_lexicon_tags_45 },
    { 0x750ebd12, SUC_lexicon_tags_60 },
    { 0x930ebd12, SUC_lexicon_tags_12 },
    { 0x9a8ebd0d, SUC_lexicon_tags_64 },
    { 0xd16ebd15, SUC_lexicon_tags_0 },
    { 0x4a2ebd16, SUC_lexicon_tags_13 },
    { 0x746ebd0e, SUC_lexicon_tags_23 },
    { 0xa9aebd12, SUC_lexicon_tags_37 },
    { 0x442ebd0d, SUC_lexicon_tags_17 },
    { 0x266ebd10, SUC_lexicon_tags_17 },
    { 0x19eebd1b, SUC_lexicon_tags_25 },
    { 0xf50ebd1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84ebd1f, SUC_lexicon_tags_15 },
    { 0x534ebd1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04ebd23, SUC_lexicon_tags_1 },
    { 0xbf4ebd24, SUC_lexicon_tags_43 },
    { 0xb98ebd25, SUC_lexicon_tags_16 },
    { 0x5c6ebd25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c6ebd28, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x772ebd2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cebd31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x95eebd33, SUC_lexicon_tags_43 },
    { 0x168ebd33, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xb9cebd36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12aebd3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x862ebd3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810ebd40, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148ebd47, SUC_lexicon_tags_1058 },
    { 0x206ebd48, SUC_lexicon_tags_47 },
    { 0x446ebd49, SUC_lexicon_tags_37 },
    { 0x5caebd48, SUC_lexicon_tags_14 },
    { 0x4e4ebd48, SUC_lexicon_tags_23 },
    { 0xf32ebd4b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58ebd52, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450ebd56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc2aebd58, SUC_lexicon_tags_17 },
    { 0xd00ebd58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x616ebd5b, SUC_lexicon_tags_27 },
    { 0x430ebd5c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312ebd69, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724ebd76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97eebd79, SUC_lexicon_tags_7 },
    { 0xfc8ebd7a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x93eebd7c, SUC_lexicon_tags_45 },
    { 0x234ebd7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65cebd80, SUC_lexicon_tags_2 },
    { 0x446ebd80, SUC_lexicon_tags_45 },
    { 0x6faebd82, SUC_lexicon_tags_9 },
    { 0xf92ebd83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aebd87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89cebd8a, SUC_lexicon_tags_6 },
    { 0xb6eebd8b, SUC_lexicon_tags_37 },
    { 0x490ebd8c, SUC_lexicon_tags_25 },
    { 0xa52ebd8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2ebd90, SUC_lexicon_tags_27 },
    { 0x5daebd90, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ceebd93, SUC_lexicon_tags_15 },
    { 0x644ebd94, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xbfcebd96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cebd98, SUC_lexicon_tags_19 },
    { 0x128ebd98, SUC_lexicon_tags_1 },
    { 0x2e6ebd99, SUC_lexicon_tags_82 },
    { 0xee4ebd9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc90ebd9d, SUC_lexicon_tags_30 },
    { 0xb2eebd9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x112ebda1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2ebda5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbacebda8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4ebdaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4ebdae, SUC_lexicon_tags_7 },
    { 0x430ebdaf, SUC_lexicon_tags_107 },
    { 0x5d8ebdb0, SUC_lexicon_tags_37 },
    { 0xa28ebdb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daebdb5, SUC_lexicon_tags_9 },
    { 0xd78ebdb6, SUC_lexicon_tags_83 },
    { 0xc9eebdb6, SUC_lexicon_tags_15 },
    { 0x24aebdb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424ebdbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5aebdc1, SUC_lexicon_tags_49 },
    { 0x682ebdc2, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0x83aebdc5, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaebdca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cebdcf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cebdd4, SUC_lexicon_tags_32 },
    { 0x9c4ebdd5, SUC_lexicon_tags_15 },
    { 0x73cebdd6, SUC_lexicon_tags_27 },
    { 0xc16ebdd7, SUC_lexicon_tags_23 },
    { 0x4d4ebdd8, SUC_lexicon_tags_31 },
    { 0x310ebdd9, SUC_lexicon_tags_85 },
    { 0x498ebdd7, SUC_lexicon_tags_61 },
    { 0x32eebddb, SUC_lexicon_tags_17 },
    { 0xf06ebdd9, SUC_lexicon_tags_56 },
    { 0x1caebddd, SUC_lexicon_tags_17 },
    { 0xf3eebdda, SUC_lexicon_tags_1 },
    { 0x3a6ebddf, SUC_lexicon_tags_12 },
    { 0xcd6ebde0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164ebde4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888ebde9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08ebdef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc18ebdf1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15aebdf5, SUC_lexicon_tags_47 },
    { 0x8dcebdf6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60ebdfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cebdfe, SUC_lexicon_tags_0 },
    { 0x906ebdff, SUC_lexicon_tags_12 },
    { 0xee0ebe00, SUC_lexicon_tags_141 },
    { 0x2f4ebe01, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbe6ebe03, SUC_lexicon_tags_17 },
    { 0x736ebe04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aebe08, SUC_lexicon_tags_27 },
    { 0x2f8ebe08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x542ebe0c, SUC_lexicon_tags_55 },
    { 0xa72ebe0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2ebe11, SUC_lexicon_tags_7 },
    { 0x614ebe12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40ebe1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7cebe1c, SUC_lexicon_tags_40 },
    { 0x882ebe1d, SUC_lexicon_tags_13 },
    { 0xf9eebe1e, SUC_lexicon_tags_38 },
    { 0x8e4ebe1f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfceebe21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1eebe26, SUC_lexicon_tags_37 },
    { 0xd78ebe27, SUC_lexicon_tags_12 },
    { 0x22aebe28, SUC_lexicon_tags_25 },
    { 0x2e4ebe27, SUC_lexicon_tags_2 },
    { 0x8f2ebe28, SUC_lexicon_tags_47 },
    { 0x98aebe2b, SUC_lexicon_tags_17 },
    { 0x96aebe29, SUC_lexicon_tags_47 },
    { 0xcf0ebe2d, SUC_lexicon_tags_12 },
    { 0x56cebe29, SUC_lexicon_tags_103 },
    { 0x628ebe2f, SUC_lexicon_tags_26 },
    { 0x86cebe29, SUC_lexicon_tags_8 },
    { 0xfbaebe27, SUC_lexicon_tags_47 },
    { 0xab2ebe30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94aebe34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0ebe37, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90eebe3c, SUC_lexicon_tags_44 },
    { 0x83aebe3c, SUC_lexicon_tags_14 },
    { 0x324ebe3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c8ebe40, SUC_lexicon_tags_23 },
    { 0x600ebe41, SUC_lexicon_tags_28 },
    { 0x5cebe42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb76ebe44, SUC_lexicon_tags_37 },
    { 0x31cebe45, SUC_lexicon_tags_21 },
    { 0x5c6ebe45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9eebe4b, SUC_lexicon_tags_2 },
    { 0xc94ebe4c, SUC_lexicon_tags_92 },
    { 0x9c2ebe4b, SUC_lexicon_tags_45 },
    { 0x17cebe4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4ebe52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0ebe58, SUC_lexicon_tags_55 },
    { 0x8e6ebe59, SUC_lexicon_tags_25 },
    { 0xb1aebe59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2ebe5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd8ebe5f, SUC_lexicon_tags_7 },
    { 0x192ebe60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16ebe63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6ebe68, SUC_lexicon_tags_27 },
    { 0xae2ebe69, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0x98eebe6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712ebe74, SUC_lexicon_tags_13 },
    { 0xf94ebe75, SUC_lexicon_tags_17 },
    { 0x44aebe76, SUC_lexicon_tags_55 },
    { 0xd10ebe75, SUC_lexicon_tags_1 },
    { 0x244ebe78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x634ebe7a, SUC_lexicon_tags_31 },
    { 0x15aebe7b, SUC_lexicon_tags_65 },
    { 0x790ebe7c, SUC_lexicon_tags_28 },
    { 0x6eaebe7d, SUC_lexicon_tags_23 },
    { 0x19eebe7d, SUC_lexicon_tags_47 },
    { 0x2deebe7f, SUC_lexicon_tags_55 },
    { 0x48eebe80, SUC_lexicon_tags_11 },
    { 0xf6ebe80, SUC_lexicon_tags_15 },
    { 0x83aebe82, SUC_lexicon_tags_15 },
    { 0x52cebe82, SUC_lexicon_tags_37 },
    { 0x5c2ebe84, SUC_lexicon_tags_2 },
    { 0x3aaebe83, SUC_lexicon_tags_33 },
    { 0x864ebe86, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x82eebe88, SUC_lexicon_tags_25 },
    { 0x5fcebe88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x292ebe8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab0ebe8d, SUC_lexicon_tags_11 },
    { 0xe00ebe8e, SUC_lexicon_tags_12 },
    { 0xc4aebe8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc4ebe91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6ebe97, SUC_lexicon_tags_11 },
    { 0x90ebe98, SUC_lexicon_tags_9 },
    { 0x34eebe99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5aebe9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec6ebe9e, SUC_lexicon_tags_37 },
    { 0x992ebe9f, SUC_lexicon_tags_11 },
    { 0x9bcebea0, SUC_lexicon_tags_63 },
    { 0x994ebea0, SUC_lexicon_tags_12 },
    { 0x84aebe9e, SUC_lexicon_tags_60 },
    { 0xf38ebea0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb68ebea5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x582ebea8, SUC_lexicon_tags_11 },
    { 0xb4eebea8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2ebeb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21cebeb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ebebc, SUC_lexicon_tags_15 },
    { 0x354ebebd, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cebec0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeebec6, SUC_lexicon_tags_17 },
    { 0xda0ebec7, SUC_lexicon_tags_15 },
    { 0xea8ebec8, SUC_lexicon_tags_1 },
    { 0xaeeebec7, SUC_lexicon_tags_12 },
    { 0xbd4ebeca, SUC_lexicon_tags_149 },
    { 0x772ebec7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc04ebecd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60ebed4, SUC_lexicon_tags_2 },
    { 0x5f2ebed5, SUC_lexicon_tags_17 },
    { 0x53eebed6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2ebeda, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb68ebedc, SUC_lexicon_tags_27 },
    { 0x26eebedc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x106ebee0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x396ebee2, SUC_lexicon_tags_27 },
    { 0x196ebee3, SUC_lexicon_tags_37 },
    { 0xaebee4, SUC_lexicon_tags_77 },
    { 0x236ebee3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4aaebee7, SUC_lexicon_tags_7 },
    { 0x3b2ebee8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8ebeeb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5feebeef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aebef5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c2ebef7, SUC_lexicon_tags_11 },
    { 0x4beebef8, SUC_lexicon_tags_30 },
    { 0xfe8ebef7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x2f8ebefb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794ebeff, SUC_lexicon_tags_19 },
    { 0xc00ebf00, SUC_lexicon_tags_15 },
    { 0x9ceebf01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eebf05, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb04ebf07, SUC_lexicon_tags_47 },
    { 0x442ebf08, SUC_lexicon_tags_43 },
    { 0x650ebf09, SUC_lexicon_tags_12 },
    { 0x312ebf07, SUC_lexicon_tags_25 },
    { 0xd46ebf0b, SUC_lexicon_tags_26 },
    { 0x776ebf0b, SUC_lexicon_tags_2 },
    { 0x97cebf0a, SUC_lexicon_tags_30 },
    { 0x7feebf0b, SUC_lexicon_tags_6 },
    { 0x9aaebf0b, SUC_lexicon_tags_68 },
    { 0xeceebf0c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x948ebf13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdfcebf15, SUC_lexicon_tags_1 },
    { 0x73aebf15, SUC_lexicon_tags_25 },
    { 0x5ceebf15, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8ebf1c, SUC_lexicon_tags_17 },
    { 0x866ebf1d, SUC_lexicon_tags_7 },
    { 0x4caebf1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcebf29, SUC_lexicon_tags_118 },
    { 0x11aebf2a, SUC_lexicon_tags_15 },
    { 0x35cebf2b, SUC_lexicon_tags_12 },
    { 0xc76ebf2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2eebf2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5caebf35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecebf38, SUC_lexicon_tags_61 },
    { 0x1d8ebf38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa26ebf3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcebf44, SUC_lexicon_tags_15 },
    { 0x2a2ebf45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906ebf49, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa00ebf4b, SUC_lexicon_tags_7 },
    { 0x170ebf4c, SUC_lexicon_tags_45 },
    { 0xdd8ebf4d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50ebf55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x172ebf57, SUC_lexicon_tags_65 },
    { 0xc7aebf57, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2bcebf5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80ebf5e, SUC_lexicon_tags_17 },
    { 0xcaebf5f, SUC_lexicon_tags_46 },
    { 0x698ebf5f, SUC_lexicon_tags_0 },
    { 0x31aebf61, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcebf65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3caebf67, SUC_lexicon_tags_15 },
    { 0x3ecebf68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84aebf70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefeebf79, SUC_lexicon_tags_25 },
    { 0x57aebf79, SUC_lexicon_tags_1 },
    { 0xd4aebf7b, SUC_lexicon_tags_33 },
    { 0xdd2ebf7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d0ebf7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eebf81, SUC_lexicon_tags_17 },
    { 0x1a4ebf82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd34ebf84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19eebf87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb0ebf89, SUC_lexicon_tags_56 },
    { 0xcd4ebf8a, SUC_lexicon_tags_17 },
    { 0x142ebf8b, SUC_lexicon_tags_82 },
    { 0x6beebf8c, SUC_lexicon_tags_25 },
    { 0xc34ebf8b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x660ebf8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4ebf93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeeebf97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe46ebf99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x176ebf9b, SUC_lexicon_tags_19 },
    { 0x8e6ebf9c, SUC_lexicon_tags_7 },
    { 0xcb8ebf9d, SUC_lexicon_tags_37 },
    { 0x5d8ebf9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeebfa1, SUC_lexicon_tags_12 },
    { 0x45aebfa2, SUC_lexicon_tags_28 },
    { 0x37eebfa3, SUC_lexicon_tags_10 },
    { 0xf72ebfa4, SUC_lexicon_tags_160 },
    { 0x5a8ebfa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86ebfaa, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34ebfb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39aebfbc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x646ebfbe, SUC_lexicon_tags_174 },
    { 0xf8cebfbf, SUC_lexicon_tags_4 },
    { 0xe48ebfc0, SUC_lexicon_tags_8 },
    { 0xd5eebfbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eebfc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04ebfc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x164ebfca, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x988ebfcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2ebfd1, SUC_lexicon_tags_56 },
    { 0xdc0ebfd2, SUC_lexicon_tags_66 },
    { 0xb62ebfd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626ebfdb, SUC_lexicon_tags_85 },
    { 0xec2ebfdc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x25cebfde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6deebfe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eebfe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56aebfeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x112ebfee, SUC_lexicon_tags_28 },
    { 0xe5cebfef, SUC_lexicon_tags_17 },
    { 0x22ebff0, SUC_lexicon_tags_26 },
    { 0x55aebfee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcebff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb0ebff6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20ebff9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c2ebffb, SUC_lexicon_tags_47 },
    { 0xe4ebffc, SUC_lexicon_tags_47 },
    { 0xc64ebffc, SUC_lexicon_tags_1 },
    { 0x850ebffe, SUC_lexicon_tags_7 },
    { 0xfacebfff, SUC_lexicon_tags_11 },
    { 0x1c6ec000, SUC_lexicon_tags_55 },
    { 0xc04ebffe, SUC_lexicon_tags_158 },
    { 0xf0eec002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaec00a, SUC_lexicon_tags_1 },
    { 0xd3eec00b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9aec013, SUC_lexicon_tags_13 },
    { 0x702ec013, SUC_lexicon_tags_84 },
    { 0xf84ec013, SUC_lexicon_tags_9 },
    { 0x630ec016, SUC_lexicon_tags_12 },
    { 0x200ec013, SUC_lexicon_tags_11 },
    { 0x5eeec018, SUC_lexicon_tags_39 },
    { 0x9bcec013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0ec01c, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04ec01f, SUC_lexicon_tags_27 },
    { 0xaeeec020, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ec023, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9aec02b, SUC_lexicon_tags_12 },
    { 0xa36ec02b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8feec02e, SUC_lexicon_tags_98 },
    { 0x81cec02f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c4ec031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfecec033, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cec038, SUC_lexicon_tags_37 },
    { 0x93aec038, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x284ec03b, SUC_lexicon_tags_30 },
    { 0xfc6ec03c, SUC_lexicon_tags_17 },
    { 0xf60ec03c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x350ec03f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8ec045, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6cec047, SUC_lexicon_tags_56 },
    { 0xd84ec047, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16ec04c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x940ec04e, SUC_lexicon_tags_1 },
    { 0x3e0ec04f, SUC_lexicon_tags_3 },
    { 0xb6cec04e, SUC_lexicon_tags_1 },
    { 0xe5cec051, SUC_lexicon_tags_39 },
    { 0xe7aec052, SUC_lexicon_tags_11 },
    { 0x6fcec052, SUC_lexicon_tags_17 },
    { 0x3f0ec052, SUC_lexicon_tags_1 },
    { 0x46aec053, SUC_lexicon_tags_26 },
    { 0x934ec056, SUC_lexicon_tags_38 },
    { 0xb16ec057, SUC_lexicon_tags_14 },
    { 0xf34ec058, SUC_lexicon_tags_39 },
    { 0xd50ec058, SUC_lexicon_tags_15 },
    { 0x15aec058, SUC_lexicon_tags_17 },
    { 0xf62ec059, SUC_lexicon_tags_14 },
    { 0x54aec059, SUC_lexicon_tags_9 },
    { 0x156ec05a, SUC_lexicon_tags_37 },
    { 0x7a8ec05e, SUC_lexicon_tags_12 },
    { 0xd34ec054, SUC_lexicon_tags_37 },
    { 0x822ec060, SUC_lexicon_tags_7 },
    { 0xc60ec060, SUC_lexicon_tags_15 },
    { 0x316ec061, SUC_lexicon_tags_31 },
    { 0xc68ec061, SUC_lexicon_tags_10 },
    { 0xd2aec058, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf4ec066, SUC_lexicon_tags_8 },
    { 0xe86ec066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb64ec069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb42ec06b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2ec06e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6ec077, SUC_lexicon_tags_12 },
    { 0x8f0ec077, SUC_lexicon_tags_57 },
    { 0x95cec079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3eec07b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x636ec07d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17aec082, SUC_lexicon_tags_55 },
    { 0x5a4ec082, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ec08f, SUC_lexicon_tags_25 },
    { 0x234ec08f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2ec093, SUC_lexicon_tags_25 },
    { 0xf9aec094, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb6ec096, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x41aec098, SUC_lexicon_tags_60 },
    { 0x514ec098, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x430ec09b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcec0a1, SUC_lexicon_tags_15 },
    { 0x2b2ec0a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x140ec0a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc20ec0a6, SUC_lexicon_tags_33 },
    { 0xf2cec0a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786ec0b1, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0x1b8ec0b3, SUC_lexicon_tags_37 },
    { 0xee6ec0b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x50ec0b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4ec0bb, SUC_lexicon_tags_43 },
    { 0xe5cec0bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9feec0be, SUC_lexicon_tags_15 },
    { 0x78eec0bf, SUC_lexicon_tags_19 },
    { 0xdaeec0c0, SUC_lexicon_tags_47 },
    { 0xb56ec0c0, SUC_lexicon_tags_14 },
    { 0x76cec0c2, SUC_lexicon_tags_35 },
    { 0x158ec0c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4ec0c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa64ec0c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38cec0ce, SUC_lexicon_tags_2 },
    { 0x54eec0cf, SUC_lexicon_tags_11 },
    { 0xfc4ec0cf, SUC_lexicon_tags_38 },
    { 0x12ec0d1, SUC_lexicon_tags_37 },
    { 0xc06ec0d2, SUC_lexicon_tags_37 },
    { 0xd4cec0d3, SUC_lexicon_tags_45 },
    { 0xa22ec0cf, SUC_lexicon_tags_9 },
    { 0x1d2ec0d5, SUC_lexicon_tags_26 },
    { 0xe8ec0d4, SUC_lexicon_tags_13 },
    { 0x442ec0d2, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x684ec0d9, SUC_lexicon_tags_949 },
    { 0xe2cec0d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22eec0df, SUC_lexicon_tags_2 },
    { 0xfccec0e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2ec0e3, SUC_lexicon_tags_31 },
    { 0x59aec0e4, SUC_lexicon_tags_24 },
    { 0xe6ec0e4, SUC_lexicon_tags_15 },
    { 0x994ec0e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf16ec0e8, SUC_lexicon_tags_160 },
    { 0x78aec0e9, SUC_lexicon_tags_37 },
    { 0xfdeec0e9, SUC_lexicon_tags_35 },
    { 0x7eeec0eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2a6ec0ed, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd50ec0ef, SUC_lexicon_tags_19 },
    { 0xfd4ec0ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4ec0f3, SUC_lexicon_tags_127 },
    { 0x334ec0f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4ec0f8, SUC_lexicon_tags_15 },
    { 0xe74ec0f9, SUC_lexicon_tags_37 },
    { 0xcb4ec0f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394ec0ff, SUC_lexicon_tags_77 },
    { 0x396ec0ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x936ec102, SUC_lexicon_tags_40 },
    { 0x898ec102, SUC_lexicon_tags_65 },
    { 0x45cec102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d0ec106, SUC_lexicon_tags_15 },
    { 0xa06ec106, SUC_lexicon_tags_25 },
    { 0xfceec108, SUC_lexicon_tags_29 },
    { 0xc1aec109, SUC_lexicon_tags_3 },
    { 0x3aaec10a, SUC_lexicon_tags_43 },
    { 0xbccec109, SUC_lexicon_tags_14 },
    { 0xfc8ec10c, SUC_lexicon_tags_0 },
    { 0x994ec10d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9b2ec10f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x472ec112, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446ec116, SUC_lexicon_tags_8 },
    { 0xcf8ec117, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4ec11c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4ec11f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x358ec122, SUC_lexicon_tags_0 },
    { 0xb18ec123, SUC_lexicon_tags_9 },
    { 0x6daec124, SUC_lexicon_tags_2 },
    { 0xcceec125, SUC_lexicon_tags_46 },
    { 0xfaec124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40ec129, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52ec130, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdaeec132, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x426ec134, SUC_lexicon_tags_49 },
    { 0x212ec135, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x628ec138, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6aec13a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4ec13d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd8ec13f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696ec146, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x91aec148, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x710ec14b, SUC_lexicon_tags_2 },
    { 0x1e0ec14c, SUC_lexicon_tags_25 },
    { 0x82ec14d, SUC_lexicon_tags_1 },
    { 0x630ec14e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x268ec150, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79aec156, SUC_lexicon_tags_42 },
    { 0x238ec156, SUC_lexicon_tags_8 },
    { 0xb4aec158, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37aec15a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6deec15d, SUC_lexicon_tags_8 },
    { 0xab2ec15e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cec164, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4ec16a, SUC_lexicon_tags_1 },
    { 0xfe2ec16b, SUC_lexicon_tags_12 },
    { 0x6a0ec16c, SUC_lexicon_tags_34 },
    { 0xed6ec16b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x832ec16f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3faec172, SUC_lexicon_tags_27 },
    { 0xfa2ec173, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccec179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79eec180, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4ec186, SUC_lexicon_tags_2 },
    { 0x46ec187, SUC_lexicon_tags_7 },
    { 0x2c4ec186, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8daec18a, SUC_lexicon_tags_28 },
    { 0x90ec18a, SUC_lexicon_tags_49 },
    { 0x72eec18c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdaec192, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5eec196, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1aec198, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48ec19c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45aec19e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc48ec1a0, SUC_lexicon_tags_25 },
    { 0xd6eec1a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620ec1a5, SUC_lexicon_tags_37 },
    { 0xae8ec1a6, SUC_lexicon_tags_27 },
    { 0x18eec1a7, SUC_lexicon_tags_38 },
    { 0x8bcec1a8, SUC_lexicon_tags_37 },
    { 0x4f2ec1a9, SUC_lexicon_tags_61 },
    { 0xd66ec1a8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738ec1ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4eec1b2, SUC_lexicon_tags_27 },
    { 0xb38ec1b2, SUC_lexicon_tags_151 },
    { 0x8d6ec1b4, SUC_lexicon_tags_37 },
    { 0xb92ec1b3, SUC_lexicon_tags_15 },
    { 0x9b0ec1b6, SUC_lexicon_tags_13 },
    { 0xaa0ec1b7, SUC_lexicon_tags_55 },
    { 0x2a0ec1b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x380ec1ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44ec1be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0ec1c2, SUC_lexicon_tags_15 },
    { 0x68cec1c2, SUC_lexicon_tags_7 },
    { 0x570ec1c4, SUC_lexicon_tags_12 },
    { 0xf6cec1c5, SUC_lexicon_tags_17 },
    { 0x23cec1c3, SUC_lexicon_tags_65 },
    { 0x41eec1c5, SUC_lexicon_tags_37 },
    { 0xed2ec1c5, SUC_lexicon_tags_37 },
    { 0x99cec1c5, SUC_lexicon_tags_15 },
    { 0xd18ec1ca, SUC_lexicon_tags_7 },
    { 0x50cec1c8, SUC_lexicon_tags_1 },
    { 0xa2cec1cb, SUC_lexicon_tags_27 },
    { 0x170ec1cd, SUC_lexicon_tags_42 },
    { 0xe9eec1cd, SUC_lexicon_tags_2 },
    { 0x64aec1cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcec1d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79cec1d6, SUC_lexicon_tags_37 },
    { 0x4feec1d7, SUC_lexicon_tags_30 },
    { 0xa98ec1d8, SUC_lexicon_tags_29 },
    { 0x5a6ec1d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6ec1e0, SUC_lexicon_tags_17 },
    { 0x2b8ec1e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54ec1e6, SUC_lexicon_tags_33 },
    { 0x7b2ec1e7, SUC_lexicon_tags_1 },
    { 0xc20ec1e8, SUC_lexicon_tags_10 },
    { 0x8b8ec1e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb0ec1eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf48ec1ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34eec1f1, SUC_lexicon_tags_25 },
    { 0x3f6ec1f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634ec1f6, SUC_lexicon_tags_61 },
    { 0x21aec1f7, SUC_lexicon_tags_0 },
    { 0x9aeec1f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10ec1fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2ec200, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36ec204, SUC_lexicon_tags_47 },
    { 0x42aec205, SUC_lexicon_tags_15 },
    { 0x5ccec206, SUC_lexicon_tags_37 },
    { 0xaa2ec207, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cec20a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eec210, SUC_lexicon_tags_37 },
    { 0x538ec210, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2aec213, SUC_lexicon_tags_12 },
    { 0x8e2ec214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4ec21b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5aec220, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xacec222, SUC_lexicon_tags_49 },
    { 0xdaec223, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9feec228, SUC_lexicon_tags_25 },
    { 0x566ec229, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2aec22f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14ec232, SUC_lexicon_tags_17 },
    { 0x98eec233, SUC_lexicon_tags_12 },
    { 0x5baec234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x212ec236, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cec23a, SUC_lexicon_tags_40 },
    { 0xa76ec23b, SUC_lexicon_tags_168 },
    { 0x524ec23c, SUC_lexicon_tags_15 },
    { 0xee0ec23d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8ec243, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb60ec246, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356ec24f, SUC_lexicon_tags_7 },
    { 0xea0ec250, SUC_lexicon_tags_15 },
    { 0x4a6ec250, SUC_lexicon_tags_31 },
    { 0xc5eec252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eec258, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecec25e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1cec260, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43cec262, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xda6ec264, SUC_lexicon_tags_92 },
    { 0x2b6ec265, SUC_lexicon_tags_4 },
    { 0x182ec265, SUC_lexicon_tags_11 },
    { 0xceec265, SUC_lexicon_tags_7 },
    { 0xb02ec264, SUC_lexicon_tags_76 },
    { 0xa4eec268, SUC_lexicon_tags_17 },
    { 0xe3eec26a, SUC_lexicon_tags_1004 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ec270, SUC_lexicon_tags_14 },
    { 0xeb6ec270, SUC_lexicon_tags_37 },
    { 0x3a0ec271, SUC_lexicon_tags_37 },
    { 0x558ec273, SUC_lexicon_tags_2 },
    { 0x52aec273, SUC_lexicon_tags_15 },
    { 0x310ec275, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0ec27c, SUC_lexicon_tags_87 },
    { 0x3d6ec27d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2ec286, SUC_lexicon_tags_9 },
    { 0x23aec286, SUC_lexicon_tags_26 },
    { 0xd30ec288, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcceec28a, SUC_lexicon_tags_43 },
    { 0x3d2ec28b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668ec293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eec299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7eeec29b, SUC_lexicon_tags_33 },
    { 0xd8cec29c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6ec29f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4ec2a4, SUC_lexicon_tags_34 },
    { 0xe48ec2a5, SUC_lexicon_tags_14 },
    { 0x4a4ec2a6, SUC_lexicon_tags_39 },
    { 0x802ec2a6, SUC_lexicon_tags_3 },
    { 0x6d6ec2a8, SUC_lexicon_tags_17 },
    { 0xe5eec2a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f4ec2ab, SUC_lexicon_tags_11 },
    { 0x244ec2ac, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6ec2b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82eec2ba, SUC_lexicon_tags_11 },
    { 0xe34ec2ba, SUC_lexicon_tags_2 },
    { 0xcacec2bc, SUC_lexicon_tags_31 },
    { 0x72eec2bd, SUC_lexicon_tags_12 },
    { 0xad0ec2bd, SUC_lexicon_tags_27 },
    { 0x36eec2be, SUC_lexicon_tags_1 },
    { 0xf4eec2c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856ec2c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x312ec2c8, SUC_lexicon_tags_15 },
    { 0xe22ec2c8, SUC_lexicon_tags_11 },
    { 0xcc0ec2ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68ec2cd, SUC_lexicon_tags_15 },
    { 0x6aeec2ce, SUC_lexicon_tags_2 },
    { 0x884ec2cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc08ec2d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x464ec2d4, SUC_lexicon_tags_17 },
    { 0x62aec2d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7baec2d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaec2dc, SUC_lexicon_tags_2 },
    { 0x966ec2dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6ec2e0, SUC_lexicon_tags_15 },
    { 0x690ec2e1, SUC_lexicon_tags_63 },
    { 0x60eec2e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4ec2e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60cec2e7, SUC_lexicon_tags_47 },
    { 0x420ec2e7, SUC_lexicon_tags_7 },
    { 0x526ec2e9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x328ec2eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x170ec2ee, SUC_lexicon_tags_38 },
    { 0xc44ec2ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x146ec2f1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x888ec2f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6aeec2f6, SUC_lexicon_tags_1 },
    { 0x5d4ec2f7, SUC_lexicon_tags_17 },
    { 0x3deec2f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14ec2fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x564ec300, SUC_lexicon_tags_148 },
    { 0xf00ec301, SUC_lexicon_tags_7 },
    { 0x104ec302, SUC_lexicon_tags_63 },
    { 0xeb4ec303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cec307, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x954ec309, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1aec30b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8daec30e, SUC_lexicon_tags_274 },
    { 0xa86ec30f, SUC_lexicon_tags_17 },
    { 0x370ec310, SUC_lexicon_tags_1 },
    { 0x832ec311, SUC_lexicon_tags_17 },
    { 0xc3eec311, SUC_lexicon_tags_19 },
    { 0x81cec311, SUC_lexicon_tags_70 },
    { 0x142ec313, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x278ec317, SUC_lexicon_tags_44 },
    { 0x152ec318, SUC_lexicon_tags_45 },
    { 0x48eec317, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28ec31c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76ec321, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8ec327, SUC_lexicon_tags_87 },
    { 0xe3eec328, SUC_lexicon_tags_1 },
    { 0x334ec328, SUC_lexicon_tags_12 },
    { 0xe2aec32a, SUC_lexicon_tags_87 },
    { 0x2f8ec32a, SUC_lexicon_tags_12 },
    { 0xf44ec32b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70ec335, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa12ec337, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdceec339, SUC_lexicon_tags_38 },
    { 0xc9aec339, SUC_lexicon_tags_23 },
    { 0x1aec33b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4ec33f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x998ec342, SUC_lexicon_tags_12 },
    { 0x39cec342, SUC_lexicon_tags_17 },
    { 0xd2cec343, SUC_lexicon_tags_3 },
    { 0x6c8ec342, SUC_lexicon_tags_8 },
    { 0x2ecec344, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ec349, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80ec34e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccec351, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x824ec354, SUC_lexicon_tags_11 },
    { 0xa56ec355, SUC_lexicon_tags_15 },
    { 0x672ec356, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2acec358, SUC_lexicon_tags_15 },
    { 0x6c0ec358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb08ec35b, SUC_lexicon_tags_15 },
    { 0xb20ec35b, SUC_lexicon_tags_55 },
    { 0xf24ec35d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6ec361, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4ec365, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32ec368, SUC_lexicon_tags_19 },
    { 0xf58ec369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea4ec36b, SUC_lexicon_tags_9 },
    { 0x936ec36c, SUC_lexicon_tags_65 },
    { 0x52aec36c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608ec372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefeec376, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aec379, SUC_lexicon_tags_11 },
    { 0x7f8ec37a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6ec382, SUC_lexicon_tags_37 },
    { 0xa78ec383, SUC_lexicon_tags_37 },
    { 0x3c4ec384, SUC_lexicon_tags_37 },
    { 0x4e0ec384, SUC_lexicon_tags_4 },
    { 0x8c6ec385, SUC_lexicon_tags_42 },
    { 0xe46ec383, SUC_lexicon_tags_60 },
    { 0x816ec388, SUC_lexicon_tags_27 },
    { 0x868ec388, SUC_lexicon_tags_25 },
    { 0x4aaec38a, SUC_lexicon_tags_52 },
    { 0x8cec38b, SUC_lexicon_tags_47 },
    { 0x160ec389, SUC_lexicon_tags_15 },
    { 0x742ec38d, SUC_lexicon_tags_11 },
    { 0x8cec382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x402ec391, SUC_lexicon_tags_47 },
    { 0x19aec392, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26ec398, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x248ec39b, SUC_lexicon_tags_38 },
    { 0x576ec39c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594ec3a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9baec3a2, SUC_lexicon_tags_26 },
    { 0xb4cec3a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa2ec3a5, SUC_lexicon_tags_14 },
    { 0x8a0ec3a5, SUC_lexicon_tags_43 },
    { 0xfccec3a7, SUC_lexicon_tags_26 },
    { 0x894ec3a8, SUC_lexicon_tags_29 },
    { 0x312ec3a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd90ec3ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cec3b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962ec3bc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cec3c5, SUC_lexicon_tags_43 },
    { 0xcd2ec3c6, SUC_lexicon_tags_23 },
    { 0x218ec3c6, SUC_lexicon_tags_65 },
    { 0xa16ec3c8, SUC_lexicon_tags_572 },
    { 0x824ec3c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860ec3ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd86ec3d0, SUC_lexicon_tags_12 },
    { 0xc2aec3d1, SUC_lexicon_tags_47 },
    { 0xff4ec3d2, SUC_lexicon_tags_84 },
    { 0x9f4ec3d0, SUC_lexicon_tags_2 },
    { 0x30ec3d4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x77aec3d6, SUC_lexicon_tags_14 },
    { 0x5d2ec3d7, SUC_lexicon_tags_11 },
    { 0x50aec3d8, SUC_lexicon_tags_43 },
    { 0x90cec3d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1a2ec3db, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ec3e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e6ec3e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f4ec3e7, SUC_lexicon_tags_28 },
    { 0x1dcec3e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ecec3ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x91aec3ed, SUC_lexicon_tags_30 },
    { 0x3a8ec3ee, SUC_lexicon_tags_14 },
    { 0x76cec3ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc18ec3f1, SUC_lexicon_tags_27 },
    { 0xaaec3f1, SUC_lexicon_tags_7 },
    { 0xaec3f1, SUC_lexicon_tags_32 },
    { 0x27cec3f2, SUC_lexicon_tags_65 },
    { 0x9a6ec3f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336ec3f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x90eec3fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cec403, SUC_lexicon_tags_25 },
    { 0x606ec404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ec40a, SUC_lexicon_tags_31 },
    { 0x9caec40b, SUC_lexicon_tags_35 },
    { 0xbd6ec40c, SUC_lexicon_tags_11 },
    { 0x622ec40c, SUC_lexicon_tags_1 },
    { 0xee6ec40e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x600ec411, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x732ec413, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf82ec416, SUC_lexicon_tags_47 },
    { 0x33eec417, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc5cec419, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626ec420, SUC_lexicon_tags_47 },
    { 0x732ec420, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b4ec423, SUC_lexicon_tags_45 },
    { 0xcd8ec424, SUC_lexicon_tags_15 },
    { 0xf4aec423, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92eec429, SUC_lexicon_tags_37 },
    { 0xa00ec429, SUC_lexicon_tags_69 },
    { 0x90cec42a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600ec42f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196ec433, SUC_lexicon_tags_65 },
    { 0x362ec433, SUC_lexicon_tags_49 },
    { 0x40ec435, SUC_lexicon_tags_12 },
    { 0xf10ec435, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684ec43a, SUC_lexicon_tags_37 },
    { 0x4c0ec43b, SUC_lexicon_tags_1 },
    { 0xd06ec43b, SUC_lexicon_tags_10 },
    { 0xe06ec43c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5cec43f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0ec442, SUC_lexicon_tags_23 },
    { 0x31aec443, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58eec447, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c0ec449, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8ec44d, SUC_lexicon_tags_1 },
    { 0xe36ec44e, SUC_lexicon_tags_9 },
    { 0x1faec44f, SUC_lexicon_tags_17 },
    { 0x72aec44d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x604ec452, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4eec457, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1deec45b, SUC_lexicon_tags_23 },
    { 0x7d4ec45c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb48ec45f, SUC_lexicon_tags_15 },
    { 0xa30ec460, SUC_lexicon_tags_15 },
    { 0x80eec461, SUC_lexicon_tags_7 },
    { 0x532ec460, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40aec464, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cec468, SUC_lexicon_tags_15 },
    { 0x370ec468, SUC_lexicon_tags_19 },
    { 0x7b6ec468, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906ec470, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58ec475, SUC_lexicon_tags_17 },
    { 0x1baec476, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x834ec479, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x402ec47f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf02ec481, SUC_lexicon_tags_17 },
    { 0xa1cec482, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x814ec485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8ec489, SUC_lexicon_tags_29 },
    { 0x208ec48a, SUC_lexicon_tags_25 },
    { 0x6faec48b, SUC_lexicon_tags_1 },
    { 0x65eec489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x514ec48f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x354ec491, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6ec495, SUC_lexicon_tags_11 },
    { 0x1f6ec495, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0aec49a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36ec49d, SUC_lexicon_tags_30 },
    { 0x6b2ec49d, SUC_lexicon_tags_14 },
    { 0x636ec49e, SUC_lexicon_tags_60 },
    { 0xf6aec4a0, SUC_lexicon_tags_27 },
    { 0xaceec4a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08ec4a5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2caec4ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0ec4af, SUC_lexicon_tags_15 },
    { 0x8c2ec4af, SUC_lexicon_tags_63 },
    { 0x35eec4b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba6ec4b3, SUC_lexicon_tags_22 },
    { 0x8dcec4b4, SUC_lexicon_tags_7 },
    { 0xcf2ec4b3, SUC_lexicon_tags_37 },
    { 0xbbaec4b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x464ec4b8, SUC_lexicon_tags_25 },
    { 0x4eeec4b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cec4bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296ec4c3, SUC_lexicon_tags_32 },
    { 0xae2ec4c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ec4c9, SUC_lexicon_tags_19 },
    { 0x22aec4ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd96ec4cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92ec4d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x892ec4d6, SUC_lexicon_tags_148 },
    { 0x9fcec4d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660ec4df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0ec4e2, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcec4e8, SUC_lexicon_tags_15 },
    { 0x52ec4e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44ec4ec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf10ec4ee, SUC_lexicon_tags_1 },
    { 0xc40ec4ef, SUC_lexicon_tags_14 },
    { 0x892ec4ef, SUC_lexicon_tags_33 },
    { 0xc66ec4f0, SUC_lexicon_tags_37 },
    { 0xb92ec4f1, SUC_lexicon_tags_8 },
    { 0x4f6ec4f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbeec4f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcec4fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aec4ff, SUC_lexicon_tags_1 },
    { 0xec4ec500, SUC_lexicon_tags_39 },
    { 0xb1eec500, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaec505, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4ec509, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cec50e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b6ec510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69eec513, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aec517, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcec51b, SUC_lexicon_tags_50 },
    { 0x4b2ec51c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2ec526, SUC_lexicon_tags_11 },
    { 0x6ec527, SUC_lexicon_tags_65 },
    { 0xf24ec528, SUC_lexicon_tags_1 },
    { 0xf46ec527, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76ec52d, SUC_lexicon_tags_25 },
    { 0x2d6ec52e, SUC_lexicon_tags_47 },
    { 0x516ec52d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602ec534, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a0ec536, SUC_lexicon_tags_0 },
    { 0x444ec536, SUC_lexicon_tags_15 },
    { 0x116ec536, SUC_lexicon_tags_17 },
    { 0xbc4ec539, SUC_lexicon_tags_60 },
    { 0xc5cec53a, SUC_lexicon_tags_13 },
    { 0x3eec53b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x580ec53d, SUC_lexicon_tags_0 },
    { 0x5daec53d, SUC_lexicon_tags_7 },
    { 0x4a4ec53d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaec548, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978ec54c, SUC_lexicon_tags_37 },
    { 0x434ec54d, SUC_lexicon_tags_28 },
    { 0x8caec54e, SUC_lexicon_tags_11 },
    { 0x906ec54d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86eec553, SUC_lexicon_tags_63 },
    { 0x2f0ec554, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cec559, SUC_lexicon_tags_1 },
    { 0xf28ec559, SUC_lexicon_tags_47 },
    { 0x222ec55a, SUC_lexicon_tags_15 },
    { 0x198ec559, SUC_lexicon_tags_27 },
    { 0xbaeec55c, SUC_lexicon_tags_15 },
    { 0x44eec559, SUC_lexicon_tags_6 },
    { 0x808ec55f, SUC_lexicon_tags_6 },
    { 0xa82ec55f, SUC_lexicon_tags_261 },
    { 0x12aec55d, SUC_lexicon_tags_29 },
    { 0xa4cec562, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe4ec564, SUC_lexicon_tags_12 },
    { 0x858ec565, SUC_lexicon_tags_160 },
    { 0x92aec566, SUC_lexicon_tags_5 },
    { 0x6dcec567, SUC_lexicon_tags_11 },
    { 0xc7aec564, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2ec56b, SUC_lexicon_tags_17 },
    { 0xda0ec56c, SUC_lexicon_tags_48 },
    { 0x96eec56c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38ec575, SUC_lexicon_tags_21 },
    { 0x89eec576, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7caec578, SUC_lexicon_tags_1 },
    { 0x462ec579, SUC_lexicon_tags_23 },
    { 0x696ec57a, SUC_lexicon_tags_17 },
    { 0xe28ec57a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa64ec57d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6aec57f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8ec584, SUC_lexicon_tags_7 },
    { 0x7f6ec584, SUC_lexicon_tags_11 },
    { 0xed8ec586, SUC_lexicon_tags_4 },
    { 0xca0ec585, SUC_lexicon_tags_25 },
    { 0x76eec588, SUC_lexicon_tags_12 },
    { 0xe72ec588, SUC_lexicon_tags_11 },
    { 0x674ec588, SUC_lexicon_tags_10 },
    { 0x7b8ec58b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcdaec58d, SUC_lexicon_tags_11 },
    { 0x3f8ec58d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc32ec590, SUC_lexicon_tags_43 },
    { 0x16aec590, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcec597, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x61eec59a, SUC_lexicon_tags_11 },
    { 0x87aec59b, SUC_lexicon_tags_85 },
    { 0xbfaec59a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd72ec59e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaec5a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ec5a6, SUC_lexicon_tags_25 },
    { 0x4feec5a7, SUC_lexicon_tags_12 },
    { 0x764ec5a7, SUC_lexicon_tags_7 },
    { 0x55aec5a9, SUC_lexicon_tags_12 },
    { 0x3c2ec5a7, SUC_lexicon_tags_11 },
    { 0x39cec5ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x960ec5ad, SUC_lexicon_tags_11 },
    { 0xa4cec5ae, SUC_lexicon_tags_57 },
    { 0x16cec5ae, SUC_lexicon_tags_11 },
    { 0x988ec5af, SUC_lexicon_tags_37 },
    { 0x298ec5af, SUC_lexicon_tags_31 },
    { 0x70ec5ae, SUC_lexicon_tags_17 },
    { 0xbdeec5b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86ec5b6, SUC_lexicon_tags_131 },
    { 0x228ec5b7, SUC_lexicon_tags_6 },
    { 0xdbeec5b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aec5bb, SUC_lexicon_tags_10 },
    { 0x942ec5bb, SUC_lexicon_tags_60 },
    { 0x184ec5bb, SUC_lexicon_tags_49 },
    { 0xb12ec5bc, SUC_lexicon_tags_33 },
    { 0x7daec5bf, SUC_lexicon_tags_14 },
    { 0x928ec5bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cec5c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50eec5c8, SUC_lexicon_tags_7 },
    { 0xdeeec5c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372ec5ce, SUC_lexicon_tags_23 },
    { 0x496ec5ce, SUC_lexicon_tags_16 },
    { 0xb20ec5d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daec5d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba2ec5d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27eec5da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceec5de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x102ec5e1, SUC_lexicon_tags_17 },
    { 0x73cec5e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c6ec5e4, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74ec5e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2ec5ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cec5ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe0ec5f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd72ec5f3, SUC_lexicon_tags_77 },
    { 0x95eec5f3, SUC_lexicon_tags_12 },
    { 0x458ec5f5, SUC_lexicon_tags_42 },
    { 0xdd4ec5f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x736ec5f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8ec601, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782ec605, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2ec60d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfeec611, SUC_lexicon_tags_21 },
    { 0xc88ec611, SUC_lexicon_tags_63 },
    { 0xa7cec611, SUC_lexicon_tags_70 },
    { 0x5eec614, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6ec619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafeec61b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cec621, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45aec623, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x882ec625, SUC_lexicon_tags_11 },
    { 0xfb8ec626, SUC_lexicon_tags_15 },
    { 0x734ec625, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdd6ec629, SUC_lexicon_tags_95 },
    { 0x5ccec629, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264ec62e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ec631, SUC_lexicon_tags_15 },
    { 0xdaeec631, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x58aec634, SUC_lexicon_tags_17 },
    { 0x250ec635, SUC_lexicon_tags_26 },
    { 0xf28ec636, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb90ec638, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec642, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2ec646, SUC_lexicon_tags_47 },
    { 0x6a4ec646, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x798ec64a, SUC_lexicon_tags_21 },
    { 0xc42ec64b, SUC_lexicon_tags_10 },
    { 0xd14ec64c, SUC_lexicon_tags_15 },
    { 0xe94ec64d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64ec651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8acec653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cec656, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ccec658, SUC_lexicon_tags_25 },
    { 0x62ec659, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aec65e, SUC_lexicon_tags_1 },
    { 0x264ec65e, SUC_lexicon_tags_9 },
    { 0x78eec660, SUC_lexicon_tags_7 },
    { 0x6deec660, SUC_lexicon_tags_63 },
    { 0xa8aec661, SUC_lexicon_tags_37 },
    { 0x44eec660, SUC_lexicon_tags_25 },
    { 0xc88ec65e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78ec669, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8ec66c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ec66f, SUC_lexicon_tags_17 },
    { 0xad4ec670, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eec674, SUC_lexicon_tags_60 },
    { 0xac8ec675, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6ec678, SUC_lexicon_tags_38 },
    { 0x630ec678, SUC_lexicon_tags_25 },
    { 0x304ec67a, SUC_lexicon_tags_46 },
    { 0xdc6ec67b, SUC_lexicon_tags_1 },
    { 0xe5cec67c, SUC_lexicon_tags_14 },
    { 0xfa6ec67d, SUC_lexicon_tags_60 },
    { 0x534ec67d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66ec681, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4ec68b, SUC_lexicon_tags_7 },
    { 0x66aec68c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4aec68e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x104ec690, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x650ec693, SUC_lexicon_tags_1 },
    { 0x540ec694, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38ec699, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x342ec69c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d4ec69e, SUC_lexicon_tags_15 },
    { 0x6e8ec69e, SUC_lexicon_tags_12 },
    { 0x458ec6a0, SUC_lexicon_tags_25 },
    { 0x878ec6a1, SUC_lexicon_tags_48 },
    { 0x340ec6a2, SUC_lexicon_tags_8 },
    { 0x7a2ec6a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb4eec6a5, SUC_lexicon_tags_63 },
    { 0x2aec6a6, SUC_lexicon_tags_17 },
    { 0x280ec6a7, SUC_lexicon_tags_77 },
    { 0x188ec6a5, SUC_lexicon_tags_37 },
    { 0xb9aec6a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaec6af, SUC_lexicon_tags_204 },
    { 0x818ec6af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ec6b4, SUC_lexicon_tags_127 },
    { 0x67eec6b5, SUC_lexicon_tags_31 },
    { 0x2aaec6b6, SUC_lexicon_tags_43 },
    { 0xcb6ec6b5, SUC_lexicon_tags_17 },
    { 0x76ec6b8, SUC_lexicon_tags_29 },
    { 0xc8eec6b8, SUC_lexicon_tags_1 },
    { 0xe9cec6b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258ec6c5, SUC_lexicon_tags_19 },
    { 0x710ec6c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x954ec6c8, SUC_lexicon_tags_25 },
    { 0x2d4ec6c9, SUC_lexicon_tags_12 },
    { 0x758ec6c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8ec6cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764ec6d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe56ec6d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6ec6db, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ec6e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8ec6e7, SUC_lexicon_tags_11 },
    { 0xfdaec6e8, SUC_lexicon_tags_37 },
    { 0x148ec6e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3ceec6eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x258ec6ed, SUC_lexicon_tags_77 },
    { 0x2c0ec6ee, SUC_lexicon_tags_11 },
    { 0x8a4ec6ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x486ec6f2, SUC_lexicon_tags_15 },
    { 0x89eec6f3, SUC_lexicon_tags_11 },
    { 0x170ec6f2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ec6f8, SUC_lexicon_tags_8 },
    { 0xba2ec6f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaec6fc, SUC_lexicon_tags_30 },
    { 0xb84ec6fd, SUC_lexicon_tags_15 },
    { 0xdeaec6fd, SUC_lexicon_tags_18 },
    { 0x894ec6ff, SUC_lexicon_tags_61 },
    { 0x27eec700, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecec703, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6ec70b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x118ec70e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8ec714, SUC_lexicon_tags_6 },
    { 0xd0ec715, SUC_lexicon_tags_2 },
    { 0x2e4ec715, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x552ec719, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79eec722, SUC_lexicon_tags_17 },
    { 0x220ec723, SUC_lexicon_tags_65 },
    { 0xdeaec724, SUC_lexicon_tags_14 },
    { 0xf30ec725, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x830ec728, SUC_lexicon_tags_60 },
    { 0x13cec729, SUC_lexicon_tags_17 },
    { 0xf76ec729, SUC_lexicon_tags_25 },
    { 0x764ec72b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22ec72f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4ec732, SUC_lexicon_tags_15 },
    { 0xb6eec733, SUC_lexicon_tags_37 },
    { 0x7d6ec734, SUC_lexicon_tags_2 },
    { 0xac6ec733, SUC_lexicon_tags_53 },
    { 0xf1cec736, SUC_lexicon_tags_15 },
    { 0xcceec734, SUC_lexicon_tags_77 },
    { 0x958ec733, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8ec740, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76aec746, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eec749, SUC_lexicon_tags_37 },
    { 0x16cec749, SUC_lexicon_tags_37 },
    { 0x2e0ec74b, SUC_lexicon_tags_0 },
    { 0x56aec74c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304ec750, SUC_lexicon_tags_17 },
    { 0x678ec751, SUC_lexicon_tags_26 },
    { 0x528ec751, SUC_lexicon_tags_104 },
    { 0xb60ec753, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8ec759, SUC_lexicon_tags_37 },
    { 0x2daec75a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe86ec75c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x484ec75e, SUC_lexicon_tags_12 },
    { 0x51eec75f, SUC_lexicon_tags_17 },
    { 0x8e4ec760, SUC_lexicon_tags_26 },
    { 0x964ec761, SUC_lexicon_tags_85 },
    { 0x500ec761, SUC_lexicon_tags_25 },
    { 0x5d8ec763, SUC_lexicon_tags_15 },
    { 0x978ec764, SUC_lexicon_tags_6 },
    { 0xe28ec764, SUC_lexicon_tags_54 },
    { 0x770ec766, SUC_lexicon_tags_17 },
    { 0xa86ec767, SUC_lexicon_tags_26 },
    { 0x154ec768, SUC_lexicon_tags_15 },
    { 0x902ec769, SUC_lexicon_tags_2 },
    { 0xa36ec767, SUC_lexicon_tags_25 },
    { 0xc80ec768, SUC_lexicon_tags_2 },
    { 0xac8ec76c, SUC_lexicon_tags_66 },
    { 0x1e2ec766, SUC_lexicon_tags_17 },
    { 0xe78ec76e, SUC_lexicon_tags_43 },
    { 0x34cec76f, SUC_lexicon_tags_19 },
    { 0xeecec76f, SUC_lexicon_tags_7 },
    { 0x380ec76a, SUC_lexicon_tags_17 },
    { 0xcf0ec772, SUC_lexicon_tags_17 },
    { 0xb8eec773, SUC_lexicon_tags_21 },
    { 0x5f4ec772, SUC_lexicon_tags_45 },
    { 0x222ec775, SUC_lexicon_tags_37 },
    { 0x202ec776, SUC_lexicon_tags_29 },
    { 0x6f6ec772, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53eec779, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110ec781, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eec785, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb0ec787, SUC_lexicon_tags_19 },
    { 0x854ec787, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23eec78c, SUC_lexicon_tags_11 },
    { 0xc9aec78c, SUC_lexicon_tags_1 },
    { 0x1bcec78c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274ec792, SUC_lexicon_tags_25 },
    { 0x832ec792, SUC_lexicon_tags_37 },
    { 0x3dcec792, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60eec798, SUC_lexicon_tags_25 },
    { 0xd60ec799, SUC_lexicon_tags_45 },
    { 0xb58ec79a, SUC_lexicon_tags_11 },
    { 0xf56ec799, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x97cec79d, SUC_lexicon_tags_19 },
    { 0x984ec79d, SUC_lexicon_tags_0 },
    { 0xa34ec79e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204ec7a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4ec7a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32ec7ad, SUC_lexicon_tags_27 },
    { 0x57cec7ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aec7b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcec7b9, SUC_lexicon_tags_37 },
    { 0xa36ec7ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08ec7be, SUC_lexicon_tags_11 },
    { 0x75aec7bf, SUC_lexicon_tags_30 },
    { 0x7faec7c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cec7c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404ec7ca, SUC_lexicon_tags_15 },
    { 0x628ec7cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x564ec7cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaec7d9, SUC_lexicon_tags_45 },
    { 0xad8ec7da, SUC_lexicon_tags_7 },
    { 0x2faec7da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2ec7e1, SUC_lexicon_tags_15 },
    { 0xa8aec7e1, SUC_lexicon_tags_19 },
    { 0xe5aec7e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a4ec7e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8ec7e8, SUC_lexicon_tags_133 },
    { 0x962ec7e8, SUC_lexicon_tags_152 },
    { 0x536ec7e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaec7f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceec7f3, SUC_lexicon_tags_25 },
    { 0xea6ec7f4, SUC_lexicon_tags_37 },
    { 0x256ec7f3, SUC_lexicon_tags_11 },
    { 0x55aec7f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x962ec7f9, SUC_lexicon_tags_3 },
    { 0x39aec7fa, SUC_lexicon_tags_2 },
    { 0xc92ec7fa, SUC_lexicon_tags_11 },
    { 0xab4ec7fc, SUC_lexicon_tags_27 },
    { 0x3d0ec7fb, SUC_lexicon_tags_26 },
    { 0x28aec7fe, SUC_lexicon_tags_1 },
    { 0x3deec7ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ec802, SUC_lexicon_tags_17 },
    { 0xb56ec802, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430ec807, SUC_lexicon_tags_47 },
    { 0x4e6ec808, SUC_lexicon_tags_68 },
    { 0xaccec809, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83eec811, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68ec816, SUC_lexicon_tags_10 },
    { 0x1f8ec817, SUC_lexicon_tags_1 },
    { 0x3deec816, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbeec81c, SUC_lexicon_tags_37 },
    { 0xf84ec81d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaec824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72aec827, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaec82b, SUC_lexicon_tags_37 },
    { 0x9f6ec82c, SUC_lexicon_tags_37 },
    { 0x5c2ec82d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb8eec82f, SUC_lexicon_tags_21 },
    { 0x1e0ec830, SUC_lexicon_tags_39 },
    { 0x7c2ec831, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xedcec834, SUC_lexicon_tags_46 },
    { 0x54aec835, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcec838, SUC_lexicon_tags_49 },
    { 0xaa0ec838, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca4ec83b, SUC_lexicon_tags_30 },
    { 0x6c2ec83c, SUC_lexicon_tags_15 },
    { 0xa3aec83c, SUC_lexicon_tags_37 },
    { 0xbfeec83e, SUC_lexicon_tags_31 },
    { 0x72eec83d, SUC_lexicon_tags_39 },
    { 0xacaec840, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeec845, SUC_lexicon_tags_9 },
    { 0x398ec845, SUC_lexicon_tags_7 },
    { 0x972ec847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8ec84d, SUC_lexicon_tags_9 },
    { 0xb52ec84e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x190ec851, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0ec855, SUC_lexicon_tags_65 },
    { 0xd9aec855, SUC_lexicon_tags_45 },
    { 0xfc4ec857, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eec85b, SUC_lexicon_tags_25 },
    { 0x3f0ec85c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bcec85e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66eec863, SUC_lexicon_tags_45 },
    { 0x634ec864, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10ec869, SUC_lexicon_tags_4 },
    { 0x11eec86a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4cec86c, SUC_lexicon_tags_2 },
    { 0xdd4ec86d, SUC_lexicon_tags_82 },
    { 0x7b2ec86c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa6ec870, SUC_lexicon_tags_30 },
    { 0xe18ec871, SUC_lexicon_tags_46 },
    { 0xab8ec872, SUC_lexicon_tags_39 },
    { 0x9e8ec872, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x850ec875, SUC_lexicon_tags_11 },
    { 0x406ec876, SUC_lexicon_tags_45 },
    { 0xe92ec877, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686ec87b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302ec881, SUC_lexicon_tags_68 },
    { 0xf2ec881, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe44ec884, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x820ec887, SUC_lexicon_tags_25 },
    { 0x10ec888, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4ec88c, SUC_lexicon_tags_34 },
    { 0x498ec88d, SUC_lexicon_tags_22 },
    { 0x1aeec88e, SUC_lexicon_tags_2 },
    { 0x644ec88d, SUC_lexicon_tags_28 },
    { 0x744ec890, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cec895, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2aec89a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf4ec89c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aec8a0, SUC_lexicon_tags_23 },
    { 0x796ec8a1, SUC_lexicon_tags_103 },
    { 0xe1aec8a2, SUC_lexicon_tags_14 },
    { 0x6aaec8a3, SUC_lexicon_tags_0 },
    { 0x680ec8a4, SUC_lexicon_tags_7 },
    { 0xe52ec8a5, SUC_lexicon_tags_45 },
    { 0x946ec8a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cec8a9, SUC_lexicon_tags_30 },
    { 0x6f4ec8aa, SUC_lexicon_tags_30 },
    { 0x780ec8aa, SUC_lexicon_tags_25 },
    { 0x2f8ec8aa, SUC_lexicon_tags_2 },
    { 0xc0aec8ac, SUC_lexicon_tags_25 },
    { 0x3ecec8ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8ec8b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaec8b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20aec8b8, SUC_lexicon_tags_3 },
    { 0xb4eec8b9, SUC_lexicon_tags_16 },
    { 0x2e4ec8ba, SUC_lexicon_tags_12 },
    { 0x7b2ec8b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x594ec8be, SUC_lexicon_tags_31 },
    { 0x5e0ec8bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x958ec8c1, SUC_lexicon_tags_38 },
    { 0x392ec8c2, SUC_lexicon_tags_47 },
    { 0x41aec8c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540ec8c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe22ec8c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe14ec8cb, SUC_lexicon_tags_42 },
    { 0x65cec8cb, SUC_lexicon_tags_17 },
    { 0xdaec8cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258ec8da, SUC_lexicon_tags_11 },
    { 0x59aec8da, SUC_lexicon_tags_33 },
    { 0xd42ec8da, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4ec8df, SUC_lexicon_tags_65 },
    { 0x58aec8e0, SUC_lexicon_tags_37 },
    { 0xa92ec8e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38cec8e3, SUC_lexicon_tags_15 },
    { 0x2eeec8e4, SUC_lexicon_tags_54 },
    { 0x94cec8e4, SUC_lexicon_tags_1 },
    { 0x406ec8e3, SUC_lexicon_tags_1 },
    { 0xb10ec8e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478ec8eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cec8ee, SUC_lexicon_tags_1 },
    { 0x44aec8ef, SUC_lexicon_tags_114 },
    { 0x102ec8f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4ec8f8, SUC_lexicon_tags_17 },
    { 0x8d8ec8f9, SUC_lexicon_tags_6 },
    { 0xcbeec8fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24ec8fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfccec8ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19cec902, SUC_lexicon_tags_43 },
    { 0x70eec903, SUC_lexicon_tags_27 },
    { 0x608ec904, SUC_lexicon_tags_11 },
    { 0x2e0ec904, SUC_lexicon_tags_48 },
    { 0x65cec906, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x578ec908, SUC_lexicon_tags_23 },
    { 0x250ec908, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff0ec90b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc92ec90d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446ec911, SUC_lexicon_tags_8 },
    { 0x634ec911, SUC_lexicon_tags_23 },
    { 0x5a2ec913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb66ec915, SUC_lexicon_tags_27 },
    { 0x5eec916, SUC_lexicon_tags_18 },
    { 0x84ec917, SUC_lexicon_tags_80 },
    { 0x2b0ec916, SUC_lexicon_tags_0 },
    { 0x5ecec919, SUC_lexicon_tags_7 },
    { 0x44cec91a, SUC_lexicon_tags_19 },
    { 0x34cec91b, SUC_lexicon_tags_7 },
    { 0xe4aec91b, SUC_lexicon_tags_8 },
    { 0x770ec91d, SUC_lexicon_tags_25 },
    { 0xb3eec91e, SUC_lexicon_tags_14 },
    { 0x5b4ec91f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61cec921, SUC_lexicon_tags_10 },
    { 0x1eaec922, SUC_lexicon_tags_218 },
    { 0xdceec923, SUC_lexicon_tags_14 },
    { 0xd16ec924, SUC_lexicon_tags_37 },
    { 0x4b8ec923, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cec929, SUC_lexicon_tags_15 },
    { 0xb68ec92a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6ec92d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1aeec92f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd36ec931, SUC_lexicon_tags_15 },
    { 0x938ec932, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210ec936, SUC_lexicon_tags_15 },
    { 0xe92ec937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3cec939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacec942, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa00ec944, SUC_lexicon_tags_62 },
    { 0x38cec945, SUC_lexicon_tags_45 },
    { 0x358ec946, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234ec94a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfaec94c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acec953, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x542ec955, SUC_lexicon_tags_65 },
    { 0x1c4ec956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe4ec958, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d2ec95a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88eec961, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8ec966, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206ec96a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x820ec96d, SUC_lexicon_tags_2 },
    { 0x79eec96d, SUC_lexicon_tags_66 },
    { 0x756ec96d, SUC_lexicon_tags_14 },
    { 0xe6aec96f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2feec977, SUC_lexicon_tags_25 },
    { 0xe08ec977, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aec97d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8ec980, SUC_lexicon_tags_0 },
    { 0xdc0ec981, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6ec988, SUC_lexicon_tags_7 },
    { 0x3b8ec989, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd64ec98b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8ec98f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x358ec991, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cec995, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34ec998, SUC_lexicon_tags_17 },
    { 0xfeaec998, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3eec99d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x614ec99f, SUC_lexicon_tags_50 },
    { 0xcdaec9a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cec9a6, SUC_lexicon_tags_48 },
    { 0x32eec9a7, SUC_lexicon_tags_47 },
    { 0xef8ec9a8, SUC_lexicon_tags_33 },
    { 0x7f6ec9a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504ec9ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd9aec9b0, SUC_lexicon_tags_11 },
    { 0x1d6ec9b1, SUC_lexicon_tags_7 },
    { 0xa66ec9b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd2cec9b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6ec9b9, SUC_lexicon_tags_26 },
    { 0x2eeec9b9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790ec9c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4ec9cc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6ec9d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x618ec9d3, SUC_lexicon_tags_37 },
    { 0x122ec9d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd0cec9d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87aec9d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcec9db, SUC_lexicon_tags_19 },
    { 0x67cec9dc, SUC_lexicon_tags_12 },
    { 0xe0ec9dd, SUC_lexicon_tags_30 },
    { 0x63eec9dd, SUC_lexicon_tags_17 },
    { 0xc68ec9dc, SUC_lexicon_tags_7 },
    { 0x8e6ec9df, SUC_lexicon_tags_2 },
    { 0x4eaec9df, SUC_lexicon_tags_21 },
    { 0xb5aec9e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ec9ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65eec9f1, SUC_lexicon_tags_10 },
    { 0xa04ec9f2, SUC_lexicon_tags_17 },
    { 0x884ec9f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704ec9fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x26eec9fd, SUC_lexicon_tags_12 },
    { 0x9fcec9fd, SUC_lexicon_tags_1 },
    { 0xf34ec9ff, SUC_lexicon_tags_31 },
    { 0x1bcec9fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0eca03, SUC_lexicon_tags_0 },
    { 0x360eca04, SUC_lexicon_tags_33 },
    { 0xf92eca03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0eca0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb04eca0d, SUC_lexicon_tags_17 },
    { 0xa16eca0e, SUC_lexicon_tags_11 },
    { 0x2b2eca0d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xea2eca11, SUC_lexicon_tags_26 },
    { 0xe78eca12, SUC_lexicon_tags_37 },
    { 0x592eca12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0eca18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe42eca1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54eca1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ceca25, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x918eca27, SUC_lexicon_tags_37 },
    { 0xc46eca28, SUC_lexicon_tags_16 },
    { 0x50eca29, SUC_lexicon_tags_61 },
    { 0x80aeca2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3aaeca2c, SUC_lexicon_tags_2 },
    { 0x462eca2d, SUC_lexicon_tags_33 },
    { 0xb58eca2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ceca34, SUC_lexicon_tags_11 },
    { 0xe5ceca34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x550eca38, SUC_lexicon_tags_15 },
    { 0x30aeca38, SUC_lexicon_tags_11 },
    { 0x9aeca38, SUC_lexicon_tags_8 },
    { 0xde6eca3a, SUC_lexicon_tags_30 },
    { 0x45eeca38, SUC_lexicon_tags_7 },
    { 0x6bceca3d, SUC_lexicon_tags_1 },
    { 0x680eca3e, SUC_lexicon_tags_45 },
    { 0x26eeca3a, SUC_lexicon_tags_23 },
    { 0xd62eca3f, SUC_lexicon_tags_14 },
    { 0x28ceca41, SUC_lexicon_tags_31 },
    { 0xd58eca42, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8dceca44, SUC_lexicon_tags_19 },
    { 0x148eca44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62eca48, SUC_lexicon_tags_0 },
    { 0xadeeca49, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8eca4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x59ceca4e, SUC_lexicon_tags_60 },
    { 0x1cceca4f, SUC_lexicon_tags_61 },
    { 0x91aeca4e, SUC_lexicon_tags_33 },
    { 0x5aeca51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172eca55, SUC_lexicon_tags_33 },
    { 0x888eca56, SUC_lexicon_tags_28 },
    { 0x5b4eca55, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x960eca59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0aeca5f, SUC_lexicon_tags_133 },
    { 0x2c4eca60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfceeca67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eeeca6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cceca6d, SUC_lexicon_tags_27 },
    { 0x7f6eca6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98aeca72, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x158eca74, SUC_lexicon_tags_33 },
    { 0x532eca75, SUC_lexicon_tags_42 },
    { 0x842eca76, SUC_lexicon_tags_47 },
    { 0x1beeca77, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xe24eca79, SUC_lexicon_tags_178 },
    { 0xafceca79, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790eca80, SUC_lexicon_tags_45 },
    { 0x376eca80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc38eca83, SUC_lexicon_tags_47 },
    { 0xffeeca84, SUC_lexicon_tags_4 },
    { 0xf22eca84, SUC_lexicon_tags_43 },
    { 0xf1ceca84, SUC_lexicon_tags_16 },
    { 0x266eca87, SUC_lexicon_tags_28 },
    { 0x2aceca88, SUC_lexicon_tags_25 },
    { 0xe1aeca89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8eca8c, SUC_lexicon_tags_17 },
    { 0x624eca8c, SUC_lexicon_tags_9 },
    { 0x52eca8e, SUC_lexicon_tags_46 },
    { 0x340eca8f, SUC_lexicon_tags_38 },
    { 0x874eca90, SUC_lexicon_tags_12 },
    { 0x5f2eca91, SUC_lexicon_tags_37 },
    { 0x332eca92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0eca97, SUC_lexicon_tags_47 },
    { 0x512eca97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x722eca9b, SUC_lexicon_tags_2 },
    { 0xc08eca9c, SUC_lexicon_tags_13 },
    { 0xb6ceca9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeecaa1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9aecaa4, SUC_lexicon_tags_68 },
    { 0xb02ecaa4, SUC_lexicon_tags_77 },
    { 0x626ecaa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eecaab, SUC_lexicon_tags_37 },
    { 0xdceecaab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x260ecaaf, SUC_lexicon_tags_6 },
    { 0x1fcecab0, SUC_lexicon_tags_25 },
    { 0xbcaecab0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836ecab5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae6ecab7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xee2ecab9, SUC_lexicon_tags_63 },
    { 0x78aecaba, SUC_lexicon_tags_15 },
    { 0xec8ecab9, SUC_lexicon_tags_7 },
    { 0x4f4ecabc, SUC_lexicon_tags_10 },
    { 0xc2cecabc, SUC_lexicon_tags_8 },
    { 0x654ecaba, SUC_lexicon_tags_23 },
    { 0xc06ecabb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4ecac2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa92ecac4, SUC_lexicon_tags_7 },
    { 0x338ecac5, SUC_lexicon_tags_25 },
    { 0xbdaecac4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84eecac8, SUC_lexicon_tags_29 },
    { 0x36ecac9, SUC_lexicon_tags_37 },
    { 0xd28ecaca, SUC_lexicon_tags_15 },
    { 0x214ecacb, SUC_lexicon_tags_37 },
    { 0x270ecaca, SUC_lexicon_tags_56 },
    { 0xc4aecaca, SUC_lexicon_tags_34 },
    { 0xd5aecace, SUC_lexicon_tags_21 },
    { 0x4eecacf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaeecad4, SUC_lexicon_tags_1 },
    { 0x79cecad5, SUC_lexicon_tags_37 },
    { 0x574ecad6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x11cecad8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eecae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8baecae6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eecaec, SUC_lexicon_tags_17 },
    { 0xe32ecaec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eecaf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ecaf5, SUC_lexicon_tags_29 },
    { 0x860ecaf5, SUC_lexicon_tags_49 },
    { 0x87aecaf7, SUC_lexicon_tags_2 },
    { 0x818ecaf8, SUC_lexicon_tags_11 },
    { 0x3b2ecaf9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x488ecafb, SUC_lexicon_tags_68 },
    { 0xa86ecafb, SUC_lexicon_tags_12 },
    { 0xc06ecafb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e8ecaff, SUC_lexicon_tags_12 },
    { 0xe6aecb00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546ecb06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8ecb09, SUC_lexicon_tags_25 },
    { 0xa56ecb0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaecb0e, SUC_lexicon_tags_11 },
    { 0x43eecb0f, SUC_lexicon_tags_11 },
    { 0xab8ecb0f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x19cecb12, SUC_lexicon_tags_11 },
    { 0x80ecb12, SUC_lexicon_tags_14 },
    { 0x2feecb14, SUC_lexicon_tags_7 },
    { 0x938ecb14, SUC_lexicon_tags_25 },
    { 0x90eecb15, SUC_lexicon_tags_79 },
    { 0x368ecb14, SUC_lexicon_tags_27 },
    { 0x3aecb15, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaecb1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x984ecb1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624ecb22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aecb29, SUC_lexicon_tags_63 },
    { 0x354ecb29, SUC_lexicon_tags_1 },
    { 0xdcaecb2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x800ecb2d, SUC_lexicon_tags_16 },
    { 0xe02ecb2e, SUC_lexicon_tags_9 },
    { 0x9beecb2d, SUC_lexicon_tags_45 },
    { 0xfaaecb30, SUC_lexicon_tags_715 },
    { 0x708ecb31, SUC_lexicon_tags_56 },
    { 0x8aecb32, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38ecb36, SUC_lexicon_tags_1 },
    { 0x7e2ecb36, SUC_lexicon_tags_15 },
    { 0x8f8ecb38, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0ecb3e, SUC_lexicon_tags_23 },
    { 0x8c0ecb3f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8daecb41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9aecb43, SUC_lexicon_tags_69 },
    { 0x8a2ecb44, SUC_lexicon_tags_27 },
    { 0xcf6ecb45, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daecb49, SUC_lexicon_tags_79 },
    { 0x822ecb4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32ecb4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x796ecb50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d2ecb52, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x53eecb54, SUC_lexicon_tags_2 },
    { 0xb6cecb55, SUC_lexicon_tags_31 },
    { 0xb5aecb55, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xc88ecb58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0ecb5b, SUC_lexicon_tags_17 },
    { 0x8e8ecb5c, SUC_lexicon_tags_37 },
    { 0x88cecb5d, SUC_lexicon_tags_11 },
    { 0x49cecb5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2a8ecb60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554ecb68, SUC_lexicon_tags_11 },
    { 0xfeaecb69, SUC_lexicon_tags_16 },
    { 0x9c4ecb68, SUC_lexicon_tags_10 },
    { 0x298ecb6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faecb6f, SUC_lexicon_tags_21 },
    { 0x9aeecb70, SUC_lexicon_tags_38 },
    { 0x5eecb70, SUC_lexicon_tags_33 },
    { 0x6d2ecb72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aecb79, SUC_lexicon_tags_19 },
    { 0x340ecb79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92ecb7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xee6ecb7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8ecb82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184ecb89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96aecb8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4ecb8e, SUC_lexicon_tags_9 },
    { 0xa88ecb8f, SUC_lexicon_tags_47 },
    { 0x5beecb90, SUC_lexicon_tags_37 },
    { 0x43cecb90, SUC_lexicon_tags_15 },
    { 0x438ecb92, SUC_lexicon_tags_77 },
    { 0x790ecb90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x912ecb95, SUC_lexicon_tags_62 },
    { 0x724ecb95, SUC_lexicon_tags_15 },
    { 0xcb8ecb95, SUC_lexicon_tags_1 },
    { 0x410ecb98, SUC_lexicon_tags_17 },
    { 0x5ececb96, SUC_lexicon_tags_37 },
    { 0x652ecb9a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf8ecb9c, SUC_lexicon_tags_13 },
    { 0x1ececb9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2ecb9f, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0ecba4, SUC_lexicon_tags_25 },
    { 0x15aecba4, SUC_lexicon_tags_15 },
    { 0x40cecba6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c8ecba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x910ecbab, SUC_lexicon_tags_47 },
    { 0xf50ecbab, SUC_lexicon_tags_45 },
    { 0x4aecbad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e4ecbaf, SUC_lexicon_tags_17 },
    { 0xc7cecbb0, SUC_lexicon_tags_47 },
    { 0xc5cecbb1, SUC_lexicon_tags_64 },
    { 0x746ecbb2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320ecbb7, SUC_lexicon_tags_1310 },
    { 0x8caecbb8, SUC_lexicon_tags_19 },
    { 0x14eecbb9, SUC_lexicon_tags_7 },
    { 0x780ecbba, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xda8ecbbc, SUC_lexicon_tags_48 },
    { 0xb6aecbbd, SUC_lexicon_tags_45 },
    { 0xdc8ecbbe, SUC_lexicon_tags_9 },
    { 0x9acecbbe, SUC_lexicon_tags_17 },
    { 0xadeecbc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10ecbc7, SUC_lexicon_tags_29 },
    { 0x288ecbc8, SUC_lexicon_tags_7 },
    { 0x3a0ecbc7, SUC_lexicon_tags_2 },
    { 0x42aecbc8, SUC_lexicon_tags_45 },
    { 0x214ecbcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceecbcf, SUC_lexicon_tags_65 },
    { 0x480ecbcf, SUC_lexicon_tags_84 },
    { 0x3faecbd1, SUC_lexicon_tags_37 },
    { 0x9e4ecbd2, SUC_lexicon_tags_16 },
    { 0xcdcecbd3, SUC_lexicon_tags_37 },
    { 0xa8ecbd3, SUC_lexicon_tags_25 },
    { 0x7f4ecbd5, SUC_lexicon_tags_35 },
    { 0x43cecbd6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8ecbd9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ecbdc, SUC_lexicon_tags_40 },
    { 0x140ecbdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08ecbe1, SUC_lexicon_tags_25 },
    { 0x384ecbe2, SUC_lexicon_tags_7 },
    { 0x65cecbe3, SUC_lexicon_tags_49 },
    { 0x1c4ecbe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4ecbe8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa90ecbea, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xeacecbec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaecbf1, SUC_lexicon_tags_23 },
    { 0x52cecbf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4ecbf6, SUC_lexicon_tags_84 },
    { 0x1eeecbf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8ecbfe, SUC_lexicon_tags_20 },
    { 0x8c0ecbff, SUC_lexicon_tags_37 },
    { 0xa7eecbfe, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2ecc06, SUC_lexicon_tags_2 },
    { 0xfeeecc07, SUC_lexicon_tags_1 },
    { 0x234ecc06, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0ecc0b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x640ecc0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x576ecc10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432ecc17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752ecc1b, SUC_lexicon_tags_48 },
    { 0xa9eecc1c, SUC_lexicon_tags_47 },
    { 0xbfcecc1d, SUC_lexicon_tags_60 },
    { 0x180ecc1c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80cecc22, SUC_lexicon_tags_1 },
    { 0xe8ecc22, SUC_lexicon_tags_4 },
    { 0xbb8ecc24, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xca6ecc26, SUC_lexicon_tags_5 },
    { 0xa5aecc27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x594ecc2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332ecc30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55eecc37, SUC_lexicon_tags_25 },
    { 0x7c8ecc38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14aecc3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0ecc45, SUC_lexicon_tags_37 },
    { 0xfeeecc46, SUC_lexicon_tags_37 },
    { 0x80cecc47, SUC_lexicon_tags_57 },
    { 0xb38ecc48, SUC_lexicon_tags_2 },
    { 0xca6ecc49, SUC_lexicon_tags_25 },
    { 0xbfaecc49, SUC_lexicon_tags_60 },
    { 0x4e8ecc4a, SUC_lexicon_tags_25 },
    { 0x252ecc4b, SUC_lexicon_tags_44 },
    { 0xd88ecc4a, SUC_lexicon_tags_4 },
    { 0x8aecc47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc0ecc50, SUC_lexicon_tags_15 },
    { 0xfb0ecc51, SUC_lexicon_tags_25 },
    { 0xc60ecc52, SUC_lexicon_tags_17 },
    { 0x9b8ecc53, SUC_lexicon_tags_1 },
    { 0xf2aecc51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcecc5a, SUC_lexicon_tags_15 },
    { 0x6f6ecc5a, SUC_lexicon_tags_14 },
    { 0x326ecc5c, SUC_lexicon_tags_37 },
    { 0xd84ecc5d, SUC_lexicon_tags_37 },
    { 0x74eecc5c, SUC_lexicon_tags_11 },
    { 0x2c8ecc5f, SUC_lexicon_tags_11 },
    { 0x612ecc60, SUC_lexicon_tags_17 },
    { 0x1feecc61, SUC_lexicon_tags_15 },
    { 0xb6eecc5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0ecc65, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cecc68, SUC_lexicon_tags_77 },
    { 0xa86ecc69, SUC_lexicon_tags_62 },
    { 0x6f2ecc69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20ecc6e, SUC_lexicon_tags_17 },
    { 0x556ecc6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eecc77, SUC_lexicon_tags_23 },
    { 0xa42ecc78, SUC_lexicon_tags_28 },
    { 0xf02ecc79, SUC_lexicon_tags_37 },
    { 0xa2aecc78, SUC_lexicon_tags_68 },
    { 0x3a2ecc7b, SUC_lexicon_tags_17 },
    { 0xc12ecc7c, SUC_lexicon_tags_27 },
    { 0xd6ecc7d, SUC_lexicon_tags_37 },
    { 0xaa0ecc7e, SUC_lexicon_tags_27 },
    { 0x852ecc7f, SUC_lexicon_tags_14 },
    { 0x1ceecc7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aecc83, SUC_lexicon_tags_7 },
    { 0x84aecc84, SUC_lexicon_tags_19 },
    { 0xe02ecc85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0ecc88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87eecc8e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24ecc91, SUC_lexicon_tags_7 },
    { 0xfd6ecc92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58ecc95, SUC_lexicon_tags_11 },
    { 0x952ecc95, SUC_lexicon_tags_2 },
    { 0x4deecc97, SUC_lexicon_tags_79 },
    { 0x576ecc98, SUC_lexicon_tags_25 },
    { 0xc1cecc96, SUC_lexicon_tags_57 },
    { 0x39eecc99, SUC_lexicon_tags_16 },
    { 0x16cecc9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744ecc9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x76aecca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ceccac, SUC_lexicon_tags_33 },
    { 0xe64eccad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5feeccb7, SUC_lexicon_tags_7 },
    { 0x38eeccb8, SUC_lexicon_tags_3 },
    { 0x61aeccb9, SUC_lexicon_tags_45 },
    { 0x256eccba, SUC_lexicon_tags_522 },
    { 0xc2ceccba, SUC_lexicon_tags_49 },
    { 0x8ceccbb, SUC_lexicon_tags_35 },
    { 0x628eccbb, SUC_lexicon_tags_33 },
    { 0xb9ceccb9, SUC_lexicon_tags_3 },
    { 0x116eccbf, SUC_lexicon_tags_7 },
    { 0x63eeccb9, SUC_lexicon_tags_7 },
    { 0x7fceccc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda4eccc3, SUC_lexicon_tags_53 },
    { 0xa1aeccc4, SUC_lexicon_tags_60 },
    { 0x232eccc5, SUC_lexicon_tags_49 },
    { 0xd32eccc6, SUC_lexicon_tags_33 },
    { 0x12ceccc7, SUC_lexicon_tags_38 },
    { 0xf42eccc3, SUC_lexicon_tags_38 },
    { 0xf40eccc9, SUC_lexicon_tags_21 },
    { 0x778eccc9, SUC_lexicon_tags_15 },
    { 0x9aeecccb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50eccd0, SUC_lexicon_tags_31 },
    { 0x6eccd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe60eccd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x354eccd7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4f4eccd9, SUC_lexicon_tags_17 },
    { 0x99aeccda, SUC_lexicon_tags_29 },
    { 0x186eccdb, SUC_lexicon_tags_38 },
    { 0x3d2eccdc, SUC_lexicon_tags_25 },
    { 0x59aeccdb, SUC_lexicon_tags_64 },
    { 0x90ceccdd, SUC_lexicon_tags_8 },
    { 0xb3ceccdf, SUC_lexicon_tags_37 },
    { 0x14aecce0, SUC_lexicon_tags_37 },
    { 0x43eeccd9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ecce7, SUC_lexicon_tags_17 },
    { 0xa08ecce8, SUC_lexicon_tags_49 },
    { 0x11eecce8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7c0ecceb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760eccef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56ceccf1, SUC_lexicon_tags_2 },
    { 0x7b2eccf1, SUC_lexicon_tags_1 },
    { 0xc0aeccf1, SUC_lexicon_tags_1 },
    { 0x296eccf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0eccfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16ecd01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cecd10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6ecd14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ecd22, SUC_lexicon_tags_25 },
    { 0x8eeecd23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2ecd28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeecd2c, SUC_lexicon_tags_29 },
    { 0x1c4ecd2d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1b4ecd2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116ecd33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82ecd37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x64aecd39, SUC_lexicon_tags_29 },
    { 0x990ecd3a, SUC_lexicon_tags_6 },
    { 0x2b2ecd3b, SUC_lexicon_tags_47 },
    { 0x972ecd3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5aecd40, SUC_lexicon_tags_2 },
    { 0xf0ecd41, SUC_lexicon_tags_11 },
    { 0x14cecd41, SUC_lexicon_tags_7 },
    { 0x696ecd42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78ecd4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa46ecd4f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8eecd52, SUC_lexicon_tags_37 },
    { 0x9b0ecd53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eecd56, SUC_lexicon_tags_17 },
    { 0xd30ecd57, SUC_lexicon_tags_33 },
    { 0xf82ecd56, SUC_lexicon_tags_29 },
    { 0x5aeecd59, SUC_lexicon_tags_160 },
    { 0x8daecd57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb96ecd5c, SUC_lexicon_tags_7 },
    { 0xe0aecd5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba2ecd5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86aecd63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ceecd69, SUC_lexicon_tags_2 },
    { 0xcc4ecd6a, SUC_lexicon_tags_39 },
    { 0x6c4ecd6b, SUC_lexicon_tags_11 },
    { 0xbd6ecd6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8ecd70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xca2ecd72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88eecd78, SUC_lexicon_tags_1 },
    { 0xe50ecd79, SUC_lexicon_tags_15 },
    { 0x878ecd78, SUC_lexicon_tags_37 },
    { 0x230ecd7a, SUC_lexicon_tags_111 },
    { 0x1deecd78, SUC_lexicon_tags_17 },
    { 0x7c0ecd7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x750ecd81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4ecd83, SUC_lexicon_tags_15 },
    { 0xcb2ecd84, SUC_lexicon_tags_37 },
    { 0x5baecd84, SUC_lexicon_tags_1 },
    { 0x570ecd86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe96ecd89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2ecd8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c2ecd8e, SUC_lexicon_tags_1263 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8ecd99, SUC_lexicon_tags_63 },
    { 0x8b6ecd9a, SUC_lexicon_tags_10 },
    { 0xfb4ecd9b, SUC_lexicon_tags_556 },
    { 0x288ecd9a, SUC_lexicon_tags_11 },
    { 0xaf2ecd9c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x990ecd9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634ecda5, SUC_lexicon_tags_49 },
    { 0x7ccecda5, SUC_lexicon_tags_27 },
    { 0xe0cecda7, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x12aecdaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeecdaf, SUC_lexicon_tags_1 },
    { 0xe3eecdb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73eecdb3, SUC_lexicon_tags_37 },
    { 0x948ecdb4, SUC_lexicon_tags_11 },
    { 0xc94ecdb5, SUC_lexicon_tags_19 },
    { 0xdceecdb4, SUC_lexicon_tags_37 },
    { 0x1d6ecdb7, SUC_lexicon_tags_82 },
    { 0x7faecdb6, SUC_lexicon_tags_4 },
    { 0x538ecdb9, SUC_lexicon_tags_11 },
    { 0x192ecdba, SUC_lexicon_tags_17 },
    { 0x9e0ecdbb, SUC_lexicon_tags_21 },
    { 0x51eecdbc, SUC_lexicon_tags_30 },
    { 0x15eecdbd, SUC_lexicon_tags_25 },
    { 0x578ecdbe, SUC_lexicon_tags_7 },
    { 0xc76ecdbf, SUC_lexicon_tags_2 },
    { 0x3ceecdb3, SUC_lexicon_tags_14 },
    { 0x68eecdc1, SUC_lexicon_tags_37 },
    { 0x5faecdb6, SUC_lexicon_tags_15 },
    { 0x796ecdbc, SUC_lexicon_tags_17 },
    { 0x14ecdc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8ecdc8, SUC_lexicon_tags_15 },
    { 0x6b2ecdc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8eaecdcb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2ecdce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa1cecdd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8deecdd4, SUC_lexicon_tags_105 },
    { 0x6a2ecdd4, SUC_lexicon_tags_0 },
    { 0xd44ecdd5, SUC_lexicon_tags_87 },
    { 0xe92ecdd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa96ecdd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaecde0, SUC_lexicon_tags_39 },
    { 0xa60ecde0, SUC_lexicon_tags_29 },
    { 0x8f4ecde0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462ecde8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeecdeb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2ecdef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8acecdf1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322ecdf6, SUC_lexicon_tags_12 },
    { 0xa9cecdf6, SUC_lexicon_tags_27 },
    { 0xc5eecdf8, SUC_lexicon_tags_45 },
    { 0x4a6ecdf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654ece02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70ece06, SUC_lexicon_tags_17 },
    { 0xad4ece06, SUC_lexicon_tags_16 },
    { 0xf52ece08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x740ece0a, SUC_lexicon_tags_11 },
    { 0xe4ece0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ece16, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8ece19, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678ece1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0eece23, SUC_lexicon_tags_33 },
    { 0xc0cece24, SUC_lexicon_tags_25 },
    { 0x666ece24, SUC_lexicon_tags_1 },
    { 0x798ece23, SUC_lexicon_tags_28 },
    { 0x22ece26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd84ece2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16cece2c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340ece30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4ece36, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184ece3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9eece46, SUC_lexicon_tags_21 },
    { 0x522ece46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28ece4c, SUC_lexicon_tags_38 },
    { 0x206ece4d, SUC_lexicon_tags_43 },
    { 0xf1aece4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf08ece50, SUC_lexicon_tags_27 },
    { 0x7f0ece50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0ece58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x886ece5b, SUC_lexicon_tags_15 },
    { 0xc16ece5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd5eece5e, SUC_lexicon_tags_30 },
    { 0x900ece5f, SUC_lexicon_tags_40 },
    { 0xb76ece5e, SUC_lexicon_tags_47 },
    { 0x1d6ece5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcece66, SUC_lexicon_tags_15 },
    { 0x480ece66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8ece6a, SUC_lexicon_tags_7 },
    { 0x68eece6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcece6f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xc82ece71, SUC_lexicon_tags_14 },
    { 0x9acece72, SUC_lexicon_tags_1 },
    { 0xad2ece73, SUC_lexicon_tags_17 },
    { 0xb86ece74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe44ece76, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aece79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956ece81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x216ece83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88aece85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96aece88, SUC_lexicon_tags_37 },
    { 0xa1eece89, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x79aece8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb4eece8d, SUC_lexicon_tags_11 },
    { 0xbd2ece8d, SUC_lexicon_tags_39 },
    { 0xb7eece8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a0ece91, SUC_lexicon_tags_31 },
    { 0x46eece92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceece96, SUC_lexicon_tags_17 },
    { 0xc92ece97, SUC_lexicon_tags_47 },
    { 0x69eece98, SUC_lexicon_tags_0 },
    { 0x96cece99, SUC_lexicon_tags_16 },
    { 0xcccece98, SUC_lexicon_tags_1 },
    { 0x6caece99, SUC_lexicon_tags_38 },
    { 0x688ece9c, SUC_lexicon_tags_13 },
    { 0x4c0ece9d, SUC_lexicon_tags_63 },
    { 0x9a0ece9e, SUC_lexicon_tags_62 },
    { 0xc4eece9e, SUC_lexicon_tags_8 },
    { 0x574ece9f, SUC_lexicon_tags_6 },
    { 0x7a4ece9f, SUC_lexicon_tags_4 },
    { 0x85cece99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd22ecea4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc3eecea6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06ecea9, SUC_lexicon_tags_14 },
    { 0xb9cecea9, SUC_lexicon_tags_11 },
    { 0x2c4eceab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aeceae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eeceb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x582eceb3, SUC_lexicon_tags_18 },
    { 0xceeeceb4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6eceb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x51eeceb9, SUC_lexicon_tags_56 },
    { 0x3f8eceba, SUC_lexicon_tags_27 },
    { 0x6d2eceb9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f8ecebd, SUC_lexicon_tags_25 },
    { 0xa40ecebe, SUC_lexicon_tags_1 },
    { 0x304ecebf, SUC_lexicon_tags_25 },
    { 0xca8ecec0, SUC_lexicon_tags_31 },
    { 0x836ecec1, SUC_lexicon_tags_45 },
    { 0xc4ecebe, SUC_lexicon_tags_14 },
    { 0x858ecec3, SUC_lexicon_tags_32 },
    { 0x35eecec4, SUC_lexicon_tags_43 },
    { 0x792ecebf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe0ecec7, SUC_lexicon_tags_47 },
    { 0x48eecec8, SUC_lexicon_tags_23 },
    { 0x132ecec7, SUC_lexicon_tags_133 },
    { 0x32cecec9, SUC_lexicon_tags_79 },
    { 0xc7eececb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9b2ececd, SUC_lexicon_tags_7 },
    { 0x256ecece, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xedaeced1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8eceda, SUC_lexicon_tags_47 },
    { 0x4f6ecedb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4ecee4, SUC_lexicon_tags_12 },
    { 0x634ecee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6eceea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aeceef, SUC_lexicon_tags_37 },
    { 0x354eceef, SUC_lexicon_tags_31 },
    { 0x4e4ecef0, SUC_lexicon_tags_11 },
    { 0xce0eceef, SUC_lexicon_tags_27 },
    { 0x2aeecef3, SUC_lexicon_tags_63 },
    { 0xe8ecef4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x714ecef6, SUC_lexicon_tags_19 },
    { 0x6e4ecef7, SUC_lexicon_tags_17 },
    { 0x870ecef8, SUC_lexicon_tags_47 },
    { 0x618ecef9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xca2ecefb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x248ecefd, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6ecf00, SUC_lexicon_tags_7 },
    { 0xcbeecf01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4aecf04, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0xf86ecf06, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cecf09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8baecf0c, SUC_lexicon_tags_38 },
    { 0xb02ecf0d, SUC_lexicon_tags_37 },
    { 0xad8ecf0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x182ecf10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0aecf13, SUC_lexicon_tags_47 },
    { 0xcbcecf13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x732ecf16, SUC_lexicon_tags_7 },
    { 0xf1cecf16, SUC_lexicon_tags_12 },
    { 0x97eecf16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7d6ecf1a, SUC_lexicon_tags_9 },
    { 0xd90ecf1b, SUC_lexicon_tags_13 },
    { 0x66cecf1c, SUC_lexicon_tags_2 },
    { 0x52eecf1b, SUC_lexicon_tags_37 },
    { 0x648ecf1e, SUC_lexicon_tags_14 },
    { 0xc70ecf1f, SUC_lexicon_tags_37 },
    { 0xf62ecf1e, SUC_lexicon_tags_11 },
    { 0xf30ecf20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80aecf23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x596ecf25, SUC_lexicon_tags_2 },
    { 0x352ecf26, SUC_lexicon_tags_23 },
    { 0x764ecf27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfaecf2c, SUC_lexicon_tags_17 },
    { 0xc66ecf2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaecf32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9eecf34, SUC_lexicon_tags_56 },
    { 0x610ecf35, SUC_lexicon_tags_45 },
    { 0xf8ecf35, SUC_lexicon_tags_11 },
    { 0xa22ecf37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48eecf3d, SUC_lexicon_tags_7 },
    { 0x71eecf3e, SUC_lexicon_tags_14 },
    { 0xdf8ecf3f, SUC_lexicon_tags_45 },
    { 0x98cecf40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef0ecf42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442ecf4e, SUC_lexicon_tags_106 },
    { 0x1fcecf4f, SUC_lexicon_tags_37 },
    { 0x22cecf50, SUC_lexicon_tags_17 },
    { 0x218ecf51, SUC_lexicon_tags_82 },
    { 0xd4cecf52, SUC_lexicon_tags_63 },
    { 0x156ecf53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x278ecf56, SUC_lexicon_tags_17 },
    { 0xf78ecf56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272ecf5c, SUC_lexicon_tags_65 },
    { 0x7f2ecf5c, SUC_lexicon_tags_1 },
    { 0x5bcecf5e, SUC_lexicon_tags_6 },
    { 0x9caecf5c, SUC_lexicon_tags_32 },
    { 0x9cecf60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2ecf68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55eecf6c, SUC_lexicon_tags_37 },
    { 0x318ecf6d, SUC_lexicon_tags_17 },
    { 0xd5cecf6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf20ecf70, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x62eecf72, SUC_lexicon_tags_83 },
    { 0xc74ecf73, SUC_lexicon_tags_11 },
    { 0xe3aecf74, SUC_lexicon_tags_17 },
    { 0x5d0ecf75, SUC_lexicon_tags_30 },
    { 0xc86ecf76, SUC_lexicon_tags_48 },
    { 0xb24ecf75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344ecf7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eecf7f, SUC_lexicon_tags_2 },
    { 0x938ecf80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eecf83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58ecf86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaecf89, SUC_lexicon_tags_63 },
    { 0x436ecf8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27eecf8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596ecf91, SUC_lexicon_tags_65 },
    { 0xf0cecf92, SUC_lexicon_tags_12 },
    { 0x2eecf92, SUC_lexicon_tags_29 },
    { 0x218ecf93, SUC_lexicon_tags_39 },
    { 0x128ecf94, SUC_lexicon_tags_47 },
    { 0x6deecf92, SUC_lexicon_tags_0 },
    { 0x4a8ecf97, SUC_lexicon_tags_50 },
    { 0xe84ecf98, SUC_lexicon_tags_37 },
    { 0x8eeecf95, SUC_lexicon_tags_47 },
    { 0x6daecf9a, SUC_lexicon_tags_23 },
    { 0xb82ecf9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb34ecf9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84ecfa0, SUC_lexicon_tags_1 },
    { 0xdd2ecfa1, SUC_lexicon_tags_46 },
    { 0x466ecfa2, SUC_lexicon_tags_2 },
    { 0x8ececfa3, SUC_lexicon_tags_37 },
    { 0xdf0ecfa3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeecfad, SUC_lexicon_tags_49 },
    { 0x266ecfae, SUC_lexicon_tags_55 },
    { 0xbe6ecfad, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd78ecfb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42cecfb3, SUC_lexicon_tags_55 },
    { 0x2beecfb4, SUC_lexicon_tags_3 },
    { 0x54eecfb4, SUC_lexicon_tags_12 },
    { 0x838ecfb5, SUC_lexicon_tags_31 },
    { 0x986ecfb7, SUC_lexicon_tags_25 },
    { 0x42aecfb8, SUC_lexicon_tags_17 },
    { 0x88aecfb9, SUC_lexicon_tags_45 },
    { 0xd26ecfb9, SUC_lexicon_tags_1 },
    { 0xb60ecfb5, SUC_lexicon_tags_31 },
    { 0x6b8ecfb9, SUC_lexicon_tags_25 },
    { 0xfa2ecfbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafeecfc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28ecfc6, SUC_lexicon_tags_35 },
    { 0x614ecfc7, SUC_lexicon_tags_46 },
    { 0x64eecfc8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x338ecfcb, SUC_lexicon_tags_0 },
    { 0xaaecfcb, SUC_lexicon_tags_0 },
    { 0x598ecfcd, SUC_lexicon_tags_8 },
    { 0x57cecfce, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2ecfd5, SUC_lexicon_tags_0 },
    { 0x55aecfd6, SUC_lexicon_tags_6 },
    { 0x14aecfd7, SUC_lexicon_tags_14 },
    { 0x42eecfd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe4ecfda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa60ecfdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2ecfe1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33aecfe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69eecfe7, SUC_lexicon_tags_33 },
    { 0x89eecfe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x962ecfeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0ecfef, SUC_lexicon_tags_46 },
    { 0x7c6ecff0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8acecff2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa44ecff4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38aecff6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xacaecff8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0aed000, SUC_lexicon_tags_1 },
    { 0xea4ed001, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1daed003, SUC_lexicon_tags_8 },
    { 0xda8ed003, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f2ed006, SUC_lexicon_tags_91 },
    { 0xb30ed007, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0ed010, SUC_lexicon_tags_15 },
    { 0x37ced011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aeed016, SUC_lexicon_tags_37 },
    { 0x27aed017, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2ed01b, SUC_lexicon_tags_17 },
    { 0x122ed01c, SUC_lexicon_tags_15 },
    { 0x30ed01d, SUC_lexicon_tags_15 },
    { 0x4bced01e, SUC_lexicon_tags_43 },
    { 0x930ed01d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x756ed021, SUC_lexicon_tags_17 },
    { 0xcbeed022, SUC_lexicon_tags_11 },
    { 0x4aeed022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c0ed025, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fced028, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eed02c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fced033, SUC_lexicon_tags_15 },
    { 0x190ed034, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91ced036, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x902ed039, SUC_lexicon_tags_11 },
    { 0xa9aed03a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd1ced03c, SUC_lexicon_tags_25 },
    { 0x40aed03d, SUC_lexicon_tags_2 },
    { 0x422ed03e, SUC_lexicon_tags_11 },
    { 0x82ed03f, SUC_lexicon_tags_1 },
    { 0xabeed040, SUC_lexicon_tags_19 },
    { 0xdcaed03f, SUC_lexicon_tags_37 },
    { 0x9eaed03f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78aed044, SUC_lexicon_tags_14 },
    { 0x68eed045, SUC_lexicon_tags_29 },
    { 0xbcced046, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f8ed048, SUC_lexicon_tags_12 },
    { 0x5aced049, SUC_lexicon_tags_34 },
    { 0xcd2ed049, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b6ed04c, SUC_lexicon_tags_65 },
    { 0x944ed04d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840ed051, SUC_lexicon_tags_46 },
    { 0xa02ed051, SUC_lexicon_tags_17 },
    { 0x3b6ed053, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00ed05a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x42aed05d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85eed05f, SUC_lexicon_tags_23 },
    { 0x3eced060, SUC_lexicon_tags_84 },
    { 0x272ed060, SUC_lexicon_tags_17 },
    { 0x804ed062, SUC_lexicon_tags_1 },
    { 0x858ed060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5aed065, SUC_lexicon_tags_12 },
    { 0xe8aed065, SUC_lexicon_tags_25 },
    { 0x994ed067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaed06a, SUC_lexicon_tags_10 },
    { 0x56ced06b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95aed073, SUC_lexicon_tags_46 },
    { 0x91aed073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418ed07a, SUC_lexicon_tags_2 },
    { 0x6eeed07a, SUC_lexicon_tags_17 },
    { 0xea8ed07a, SUC_lexicon_tags_11 },
    { 0xf72ed07d, SUC_lexicon_tags_60 },
    { 0x63aed07d, SUC_lexicon_tags_45 },
    { 0x98aed07a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeed082, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294ed088, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b0ed08a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14ed08d, SUC_lexicon_tags_21 },
    { 0x392ed08e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66ed094, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x744ed097, SUC_lexicon_tags_11 },
    { 0xf7eed098, SUC_lexicon_tags_33 },
    { 0x8d6ed098, SUC_lexicon_tags_14 },
    { 0x574ed09a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2ed09d, SUC_lexicon_tags_27 },
    { 0x836ed09e, SUC_lexicon_tags_7 },
    { 0xaa2ed09f, SUC_lexicon_tags_7 },
    { 0x61eed09d, SUC_lexicon_tags_1 },
    { 0xa0ed09f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34ed0a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82eed0af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14ed0b2, SUC_lexicon_tags_1 },
    { 0x2b6ed0b3, SUC_lexicon_tags_19 },
    { 0x2faed0b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724ed0bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5aeed0bf, SUC_lexicon_tags_45 },
    { 0x200ed0c0, SUC_lexicon_tags_43 },
    { 0x13ced0c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x126ed0c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84ed0c8, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5daed0cc, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8ed0cf, SUC_lexicon_tags_2 },
    { 0xe16ed0d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2eed0d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98eed0d6, SUC_lexicon_tags_0 },
    { 0x138ed0d7, SUC_lexicon_tags_15 },
    { 0xe92ed0d7, SUC_lexicon_tags_37 },
    { 0xfb4ed0d9, SUC_lexicon_tags_29 },
    { 0xe86ed0da, SUC_lexicon_tags_4 },
    { 0xe3ced0db, SUC_lexicon_tags_12 },
    { 0x67ced0d9, SUC_lexicon_tags_11 },
    { 0x6ed0d8, SUC_lexicon_tags_35 },
    { 0x22eed0de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2aced0e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78eed0e8, SUC_lexicon_tags_37 },
    { 0xedeed0e9, SUC_lexicon_tags_11 },
    { 0x8bced0ea, SUC_lexicon_tags_60 },
    { 0xa52ed0eb, SUC_lexicon_tags_37 },
    { 0x10eed0ec, SUC_lexicon_tags_1 },
    { 0x6b8ed0eb, SUC_lexicon_tags_11 },
    { 0x354ed0e8, SUC_lexicon_tags_25 },
    { 0x42ed0ef, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xe1aed0f1, SUC_lexicon_tags_12 },
    { 0x4aaed0f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe06ed0f4, SUC_lexicon_tags_19 },
    { 0x110ed0f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcaed0f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c2ed0fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x532ed0fd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3e2ed0ff, SUC_lexicon_tags_55 },
    { 0xb14ed100, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x360ed102, SUC_lexicon_tags_17 },
    { 0x276ed102, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa46ed105, SUC_lexicon_tags_1 },
    { 0xf5ced106, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6beed114, SUC_lexicon_tags_63 },
    { 0x882ed115, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd66ed117, SUC_lexicon_tags_25 },
    { 0x9baed117, SUC_lexicon_tags_56 },
    { 0xc30ed119, SUC_lexicon_tags_2 },
    { 0xcc8ed117, SUC_lexicon_tags_45 },
    { 0xcd0ed11b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x910ed11d, SUC_lexicon_tags_43 },
    { 0x184ed11d, SUC_lexicon_tags_28 },
    { 0xf9ced11f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc8ed121, SUC_lexicon_tags_11 },
    { 0x58aed122, SUC_lexicon_tags_2 },
    { 0xe8ed121, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0ed127, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc2eed129, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1beed12c, SUC_lexicon_tags_23 },
    { 0x488ed12d, SUC_lexicon_tags_2 },
    { 0xe7aed12e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6feed131, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0ed135, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x70ced137, SUC_lexicon_tags_1 },
    { 0x976ed138, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2ed13d, SUC_lexicon_tags_19 },
    { 0xfd8ed13d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ced141, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ced143, SUC_lexicon_tags_25 },
    { 0xb5eed143, SUC_lexicon_tags_67 },
    { 0x53aed143, SUC_lexicon_tags_60 },
    { 0x300ed144, SUC_lexicon_tags_4 },
    { 0xd24ed147, SUC_lexicon_tags_12 },
    { 0x92ed143, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620ed14c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74ed150, SUC_lexicon_tags_31 },
    { 0xa22ed151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2ed155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6faed157, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaaeed159, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x388ed15b, SUC_lexicon_tags_7 },
    { 0x988ed15b, SUC_lexicon_tags_1 },
    { 0x94aed15d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2beed160, SUC_lexicon_tags_39 },
    { 0xe72ed161, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x422ed163, SUC_lexicon_tags_2 },
    { 0x1a2ed163, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0aed16f, SUC_lexicon_tags_37 },
    { 0x5caed170, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69aed177, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd7eed179, SUC_lexicon_tags_37 },
    { 0x2d4ed17a, SUC_lexicon_tags_15 },
    { 0x596ed17b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadced180, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92ed184, SUC_lexicon_tags_26 },
    { 0x1b4ed184, SUC_lexicon_tags_47 },
    { 0x7d2ed186, SUC_lexicon_tags_15 },
    { 0xc84ed187, SUC_lexicon_tags_25 },
    { 0x2beed187, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4ed18c, SUC_lexicon_tags_45 },
    { 0x708ed18c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x55aed190, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x676ed193, SUC_lexicon_tags_2 },
    { 0x8ed194, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6ed198, SUC_lexicon_tags_33 },
    { 0x93eed199, SUC_lexicon_tags_12 },
    { 0xd74ed199, SUC_lexicon_tags_2 },
    { 0x5e6ed198, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4ed19e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ced1a2, SUC_lexicon_tags_5 },
    { 0xb48ed1a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc6ed1a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x380ed1a7, SUC_lexicon_tags_27 },
    { 0xe16ed1a8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aed1ad, SUC_lexicon_tags_1 },
    { 0xeaaed1ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20ed1b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x392ed1ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704ed1be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8ed1ca, SUC_lexicon_tags_7 },
    { 0xe1aed1ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x418ed1cd, SUC_lexicon_tags_160 },
    { 0xc38ed1cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40aed1d8, SUC_lexicon_tags_49 },
    { 0x582ed1d8, SUC_lexicon_tags_37 },
    { 0xfbeed1da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x642ed1de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262ed1e4, SUC_lexicon_tags_56 },
    { 0xfa0ed1e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28aed1e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x164ed1ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d2ed1ec, SUC_lexicon_tags_10 },
    { 0xb3aed1ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3f8ed1ef, SUC_lexicon_tags_1 },
    { 0xf60ed1f0, SUC_lexicon_tags_1 },
    { 0x334ed1f1, SUC_lexicon_tags_12 },
    { 0xc7eed1f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a4ed1f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bced1f6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x462ed1f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ced1ff, SUC_lexicon_tags_26 },
    { 0xa92ed200, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50ed204, SUC_lexicon_tags_15 },
    { 0x9e6ed205, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x518ed207, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaed20a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x460ed20c, SUC_lexicon_tags_47 },
    { 0x618ed20c, SUC_lexicon_tags_0 },
    { 0xb3ced20e, SUC_lexicon_tags_15 },
    { 0xa74ed20f, SUC_lexicon_tags_17 },
    { 0x54ced20e, SUC_lexicon_tags_11 },
    { 0xe62ed20f, SUC_lexicon_tags_12 },
    { 0x448ed212, SUC_lexicon_tags_59 },
    { 0x590ed212, SUC_lexicon_tags_14 },
    { 0x788ed20d, SUC_lexicon_tags_26 },
    { 0xceed215, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f4ed217, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278ed21c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2aced21e, SUC_lexicon_tags_1 },
    { 0x2c0ed21e, SUC_lexicon_tags_1 },
    { 0xf9aed220, SUC_lexicon_tags_47 },
    { 0x93aed221, SUC_lexicon_tags_8 },
    { 0x2eed21f, SUC_lexicon_tags_11 },
    { 0x456ed223, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xacaed225, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ced228, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aed22f, SUC_lexicon_tags_28 },
    { 0x580ed22f, SUC_lexicon_tags_28 },
    { 0xba2ed231, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x39eed234, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ced238, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd7aed23a, SUC_lexicon_tags_63 },
    { 0x85ced23b, SUC_lexicon_tags_66 },
    { 0xb5aed23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfeed23e, SUC_lexicon_tags_54 },
    { 0xf20ed23f, SUC_lexicon_tags_11 },
    { 0xfe2ed240, SUC_lexicon_tags_30 },
    { 0x996ed23e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61aed243, SUC_lexicon_tags_806 },
    { 0, NULL },
    { 0, NULL },
    { 0x962ed246, SUC_lexicon_tags_30 },
    { 0xcfaed247, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x80ced249, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc06ed24b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bced24e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x55aed251, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4eed254, SUC_lexicon_tags_25 },
    { 0xe36ed255, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262ed262, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02ed268, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x442ed26a, SUC_lexicon_tags_12 },
    { 0x6b4ed26b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x34aed26d, SUC_lexicon_tags_15 },
    { 0xca6ed26d, SUC_lexicon_tags_19 },
    { 0xe26ed26f, SUC_lexicon_tags_12 },
    { 0x5eed26d, SUC_lexicon_tags_49 },
    { 0x7f8ed271, SUC_lexicon_tags_4 },
    { 0x83ced270, SUC_lexicon_tags_17 },
    { 0xc1eed273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eed279, SUC_lexicon_tags_27 },
    { 0x8b8ed27a, SUC_lexicon_tags_27 },
    { 0x3cced279, SUC_lexicon_tags_37 },
    { 0x9b0ed27c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bced280, SUC_lexicon_tags_27 },
    { 0x5f2ed281, SUC_lexicon_tags_5 },
    { 0x356ed281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92aed284, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7faed286, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1f4ed288, SUC_lexicon_tags_9 },
    { 0x536ed289, SUC_lexicon_tags_46 },
    { 0xbc6ed288, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21aed28c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed290, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82aed294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8ed298, SUC_lexicon_tags_12 },
    { 0x7d0ed299, SUC_lexicon_tags_49 },
    { 0x24eed29a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8ed2a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4ed2a4, SUC_lexicon_tags_37 },
    { 0x506ed2a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfdeed2a7, SUC_lexicon_tags_0 },
    { 0x8f8ed2a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00ed2ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aed2b0, SUC_lexicon_tags_11 },
    { 0x3aced2b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ced2b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x134ed2ba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aed2bd, SUC_lexicon_tags_33 },
    { 0xf9aed2be, SUC_lexicon_tags_63 },
    { 0xf12ed2bf, SUC_lexicon_tags_39 },
    { 0xe24ed2c0, SUC_lexicon_tags_23 },
    { 0xcc2ed2c0, SUC_lexicon_tags_2 },
    { 0x1aced2c2, SUC_lexicon_tags_25 },
    { 0x366ed2c2, SUC_lexicon_tags_44 },
    { 0xb12ed2c2, SUC_lexicon_tags_15 },
    { 0x286ed2c4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0ed2c9, SUC_lexicon_tags_37 },
    { 0x880ed2ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6ed2cd, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ed2d0, SUC_lexicon_tags_49 },
    { 0x766ed2d0, SUC_lexicon_tags_118 },
    { 0xadeed2d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26aed2dd, SUC_lexicon_tags_45 },
    { 0xc7ced2dd, SUC_lexicon_tags_2 },
    { 0x8c8ed2df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2ed2e3, SUC_lexicon_tags_1 },
    { 0xdeed2e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf04ed2e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39eed2ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x824ed2f0, SUC_lexicon_tags_19 },
    { 0xed8ed2f0, SUC_lexicon_tags_45 },
    { 0x852ed2f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04ed2f5, SUC_lexicon_tags_11 },
    { 0xf3aed2f5, SUC_lexicon_tags_25 },
    { 0xb8eed2f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ed2fc, SUC_lexicon_tags_42 },
    { 0x1e0ed2fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aed300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dced303, SUC_lexicon_tags_15 },
    { 0x6f0ed304, SUC_lexicon_tags_15 },
    { 0xe82ed304, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x74aed307, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3deed309, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbfced30b, SUC_lexicon_tags_25 },
    { 0x1a8ed30b, SUC_lexicon_tags_38 },
    { 0x75aed30d, SUC_lexicon_tags_12 },
    { 0x6f0ed30d, SUC_lexicon_tags_37 },
    { 0x9b0ed30e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x162ed312, SUC_lexicon_tags_60 },
    { 0xb92ed313, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf52ed316, SUC_lexicon_tags_15 },
    { 0x7b6ed317, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xefced319, SUC_lexicon_tags_11 },
    { 0x6eed31a, SUC_lexicon_tags_25 },
    { 0x634ed31b, SUC_lexicon_tags_1 },
    { 0x896ed319, SUC_lexicon_tags_60 },
    { 0xfb6ed31d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6ed320, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x674ed322, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59eed324, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ced32a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26ed32d, SUC_lexicon_tags_26 },
    { 0x374ed32e, SUC_lexicon_tags_107 },
    { 0xcaced32e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294ed334, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ed33b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedaed340, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x51eed342, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ced346, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eed34a, SUC_lexicon_tags_27 },
    { 0xd44ed34a, SUC_lexicon_tags_15 },
    { 0x99eed34c, SUC_lexicon_tags_36 },
    { 0x99ced34d, SUC_lexicon_tags_17 },
    { 0x66aed34e, SUC_lexicon_tags_66 },
    { 0x402ed34b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2ed353, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b6ed355, SUC_lexicon_tags_687 },
    { 0x9c2ed356, SUC_lexicon_tags_0 },
    { 0xa34ed356, SUC_lexicon_tags_15 },
    { 0x74ed355, SUC_lexicon_tags_32 },
    { 0x110ed359, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ced35e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2ed361, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87aed36c, SUC_lexicon_tags_49 },
    { 0xe9aed36d, SUC_lexicon_tags_25 },
    { 0xaeced36e, SUC_lexicon_tags_17 },
    { 0x694ed36e, SUC_lexicon_tags_10 },
    { 0x882ed370, SUC_lexicon_tags_11 },
    { 0xa48ed370, SUC_lexicon_tags_11 },
    { 0x408ed36e, SUC_lexicon_tags_174 },
    { 0x89ced372, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98ed376, SUC_lexicon_tags_2 },
    { 0x97aed377, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x824ed379, SUC_lexicon_tags_1 },
    { 0x250ed379, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbfeed37c, SUC_lexicon_tags_141 },
    { 0xc66ed37d, SUC_lexicon_tags_12 },
    { 0x474ed37d, SUC_lexicon_tags_15 },
    { 0x4f8ed37f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bced384, SUC_lexicon_tags_2 },
    { 0xaed385, SUC_lexicon_tags_1 },
    { 0xf36ed386, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370ed38c, SUC_lexicon_tags_39 },
    { 0x17aed38c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ced391, SUC_lexicon_tags_1 },
    { 0x5f2ed392, SUC_lexicon_tags_11 },
    { 0x79aed393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8ed398, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12eed39c, SUC_lexicon_tags_13 },
    { 0xfbeed39d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaced39f, SUC_lexicon_tags_45 },
    { 0xba0ed3a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe7eed3a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44ed3a4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02ed3a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30ed3ab, SUC_lexicon_tags_37 },
    { 0x46aed3ac, SUC_lexicon_tags_10 },
    { 0xda4ed3ad, SUC_lexicon_tags_25 },
    { 0x95ced3ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4ed3b3, SUC_lexicon_tags_63 },
    { 0xd3aed3b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850ed3ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdeaed3bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x690ed3be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2eed3c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7eed3c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ced3d1, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58ed3d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16ed3dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2c0ed3de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8ed3e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e8ed3e6, SUC_lexicon_tags_39 },
    { 0x1d8ed3e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30eed3e9, SUC_lexicon_tags_27 },
    { 0x2aeed3e9, SUC_lexicon_tags_39 },
    { 0x158ed3e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8beed3ee, SUC_lexicon_tags_14 },
    { 0xe8aed3ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2ed3f7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x508ed3f9, SUC_lexicon_tags_47 },
    { 0xa20ed3fa, SUC_lexicon_tags_37 },
    { 0x9aced3fb, SUC_lexicon_tags_12 },
    { 0x5ceed3fc, SUC_lexicon_tags_11 },
    { 0xd22ed3fd, SUC_lexicon_tags_16 },
    { 0x326ed3fe, SUC_lexicon_tags_37 },
    { 0xecced3ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x594ed401, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ced404, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x98ced406, SUC_lexicon_tags_17 },
    { 0x9e2ed406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eed40b, SUC_lexicon_tags_7 },
    { 0x702ed40c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ced411, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x120ed414, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966ed419, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98ed41d, SUC_lexicon_tags_19 },
    { 0xc54ed41e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac2ed420, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaed423, SUC_lexicon_tags_1001 },
    { 0x9f8ed424, SUC_lexicon_tags_1 },
    { 0xe06ed424, SUC_lexicon_tags_35 },
    { 0xa60ed426, SUC_lexicon_tags_65 },
    { 0x904ed424, SUC_lexicon_tags_17 },
    { 0x48aed428, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eed42f, SUC_lexicon_tags_65 },
    { 0xf02ed42f, SUC_lexicon_tags_24 },
    { 0xa7ced430, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76ed435, SUC_lexicon_tags_12 },
    { 0xce6ed436, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8ed43f, SUC_lexicon_tags_43 },
    { 0x37ced440, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xb8eed442, SUC_lexicon_tags_37 },
    { 0xf14ed442, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728ed44a, SUC_lexicon_tags_45 },
    { 0x7f6ed44b, SUC_lexicon_tags_15 },
    { 0x4f8ed44b, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8ed44f, SUC_lexicon_tags_0 },
    { 0xff8ed450, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baed456, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa40ed458, SUC_lexicon_tags_25 },
    { 0xd88ed458, SUC_lexicon_tags_2 },
    { 0xb78ed459, SUC_lexicon_tags_15 },
    { 0x456ed45b, SUC_lexicon_tags_37 },
    { 0x652ed45c, SUC_lexicon_tags_1 },
    { 0xcdeed45b, SUC_lexicon_tags_37 },
    { 0x7eaed45c, SUC_lexicon_tags_66 },
    { 0x2d4ed45f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7b4ed461, SUC_lexicon_tags_12 },
    { 0xfeeed461, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1eed46a, SUC_lexicon_tags_28 },
    { 0xc9aed46a, SUC_lexicon_tags_25 },
    { 0x402ed46a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaed46f, SUC_lexicon_tags_7 },
    { 0xf9aed470, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aced472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6ed475, SUC_lexicon_tags_2 },
    { 0xf0ed475, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b8ed478, SUC_lexicon_tags_1 },
    { 0xb8ed479, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb56ed47b, SUC_lexicon_tags_9 },
    { 0x316ed47c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760ed480, SUC_lexicon_tags_17 },
    { 0xa26ed480, SUC_lexicon_tags_11 },
    { 0x534ed482, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d6ed484, SUC_lexicon_tags_113 },
    { 0xa1eed485, SUC_lexicon_tags_46 },
    { 0xba8ed485, SUC_lexicon_tags_12 },
    { 0xc8ed487, SUC_lexicon_tags_40 },
    { 0x332ed488, SUC_lexicon_tags_15 },
    { 0xf88ed489, SUC_lexicon_tags_27 },
    { 0x86ed489, SUC_lexicon_tags_2 },
    { 0x5b4ed484, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x126ed48d, SUC_lexicon_tags_23 },
    { 0x84ced48e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a6ed490, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf8ed492, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19aed49e, SUC_lexicon_tags_16 },
    { 0xb40ed49f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75eed4a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7beed4a7, SUC_lexicon_tags_15 },
    { 0xe2ed4a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8ed4ac, SUC_lexicon_tags_37 },
    { 0xcc4ed4ad, SUC_lexicon_tags_11 },
    { 0xaf4ed4ac, SUC_lexicon_tags_60 },
    { 0xbf8ed4af, SUC_lexicon_tags_62 },
    { 0x2eeed4ae, SUC_lexicon_tags_45 },
    { 0x5a0ed4b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7eed4b3, SUC_lexicon_tags_12 },
    { 0x7aed4b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x14aed4b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x574ed4b9, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16ed4bf, SUC_lexicon_tags_22 },
    { 0xec6ed4c0, SUC_lexicon_tags_15 },
    { 0xa2ed4c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd62ed4c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x694ed4c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x898ed4c8, SUC_lexicon_tags_15 },
    { 0x1beed4c9, SUC_lexicon_tags_16 },
    { 0x32ed4c8, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefeed4d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ced4d5, SUC_lexicon_tags_25 },
    { 0xda8ed4d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfeed4db, SUC_lexicon_tags_1 },
    { 0x100ed4dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbeced4de, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9feed4e1, SUC_lexicon_tags_34 },
    { 0x896ed4e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7c2ed4e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48ed4e9, SUC_lexicon_tags_2 },
    { 0xf38ed4e9, SUC_lexicon_tags_16 },
    { 0xca6ed4eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58eed4ee, SUC_lexicon_tags_11 },
    { 0x20aed4ef, SUC_lexicon_tags_11 },
    { 0x3c4ed4ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6ed4f4, SUC_lexicon_tags_23 },
    { 0x586ed4f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2ed4fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b2ed4fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56aed500, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ced503, SUC_lexicon_tags_17 },
    { 0xfdaed504, SUC_lexicon_tags_15 },
    { 0xcd8ed504, SUC_lexicon_tags_2 },
    { 0xfc6ed506, SUC_lexicon_tags_43 },
    { 0x9a0ed504, SUC_lexicon_tags_33 },
    { 0xaa0ed504, SUC_lexicon_tags_14 },
    { 0x6d8ed509, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x516ed50b, SUC_lexicon_tags_62 },
    { 0x94ced50c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x764ed50f, SUC_lexicon_tags_16 },
    { 0xbaced510, SUC_lexicon_tags_15 },
    { 0x388ed510, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x644ed514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ced519, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x474ed51c, SUC_lexicon_tags_49 },
    { 0x88aed51d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dced51f, SUC_lexicon_tags_0 },
    { 0xc30ed520, SUC_lexicon_tags_30 },
    { 0x990ed521, SUC_lexicon_tags_0 },
    { 0x268ed521, SUC_lexicon_tags_15 },
    { 0x72aed523, SUC_lexicon_tags_17 },
    { 0x864ed51f, SUC_lexicon_tags_17 },
    { 0x9f4ed520, SUC_lexicon_tags_25 },
    { 0xe48ed526, SUC_lexicon_tags_7 },
    { 0x8d8ed527, SUC_lexicon_tags_23 },
    { 0xea4ed528, SUC_lexicon_tags_1 },
    { 0x230ed525, SUC_lexicon_tags_25 },
    { 0xbd0ed52a, SUC_lexicon_tags_23 },
    { 0x8aaed52a, SUC_lexicon_tags_14 },
    { 0x574ed52a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ced530, SUC_lexicon_tags_17 },
    { 0x816ed531, SUC_lexicon_tags_67 },
    { 0x640ed532, SUC_lexicon_tags_37 },
    { 0xe68ed532, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf4ed535, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x266ed537, SUC_lexicon_tags_9 },
    { 0xad0ed538, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4ed53b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26ed53f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb88ed541, SUC_lexicon_tags_17 },
    { 0xd3aed541, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x460ed545, SUC_lexicon_tags_38 },
    { 0xe90ed545, SUC_lexicon_tags_4 },
    { 0x1aed547, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8ed54c, SUC_lexicon_tags_17 },
    { 0x83eed54c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa4ed54f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ced553, SUC_lexicon_tags_37 },
    { 0xcfeed554, SUC_lexicon_tags_23 },
    { 0xdfced555, SUC_lexicon_tags_25 },
    { 0xee8ed556, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44eed55a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0ed55d, SUC_lexicon_tags_12 },
    { 0xa80ed55e, SUC_lexicon_tags_11 },
    { 0xe82ed55f, SUC_lexicon_tags_7 },
    { 0xfced560, SUC_lexicon_tags_7 },
    { 0x5f4ed560, SUC_lexicon_tags_21 },
    { 0x5feed560, SUC_lexicon_tags_11 },
    { 0xa8aed563, SUC_lexicon_tags_12 },
    { 0x45aed564, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16ed567, SUC_lexicon_tags_12 },
    { 0x3e4ed568, SUC_lexicon_tags_38 },
    { 0xb88ed568, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb62ed56b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32eed570, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe32ed572, SUC_lexicon_tags_32 },
    { 0x842ed573, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ced576, SUC_lexicon_tags_4 },
    { 0x20ced577, SUC_lexicon_tags_84 },
    { 0x716ed578, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422ed57c, SUC_lexicon_tags_1 },
    { 0xaa6ed57c, SUC_lexicon_tags_17 },
    { 0x868ed57c, SUC_lexicon_tags_33 },
    { 0x24ced57c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46ed586, SUC_lexicon_tags_12 },
    { 0x404ed586, SUC_lexicon_tags_11 },
    { 0xee4ed586, SUC_lexicon_tags_25 },
    { 0xbeaed589, SUC_lexicon_tags_7 },
    { 0x580ed58a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832ed590, SUC_lexicon_tags_2 },
    { 0xd40ed590, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x286ed593, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd50ed595, SUC_lexicon_tags_17 },
    { 0x9eeed596, SUC_lexicon_tags_30 },
    { 0xeb2ed597, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabced59a, SUC_lexicon_tags_15 },
    { 0xd48ed59a, SUC_lexicon_tags_11 },
    { 0x8faed59a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2ed5a2, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xa4ced5a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34eed5a7, SUC_lexicon_tags_14 },
    { 0x43ced5a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x596ed5aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562ed5b1, SUC_lexicon_tags_12 },
    { 0xc5eed5b1, SUC_lexicon_tags_45 },
    { 0xd98ed5b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24ed5bb, SUC_lexicon_tags_64 },
    { 0x390ed5bb, SUC_lexicon_tags_37 },
    { 0x912ed5bc, SUC_lexicon_tags_27 },
    { 0x4f6ed5be, SUC_lexicon_tags_23 },
    { 0x56ed5bf, SUC_lexicon_tags_343 },
    { 0x9baed5bd, SUC_lexicon_tags_14 },
    { 0xa24ed5c1, SUC_lexicon_tags_17 },
    { 0xa6eed5bd, SUC_lexicon_tags_7 },
    { 0x1faed5c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44ed5c6, SUC_lexicon_tags_17 },
    { 0xaf4ed5c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc12ed5c9, SUC_lexicon_tags_7 },
    { 0xde2ed5c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2ed5cf, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4b6ed5d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ced5d5, SUC_lexicon_tags_2 },
    { 0xcf2ed5d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9fced5d8, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x7dced5da, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb7aed5dc, SUC_lexicon_tags_30 },
    { 0x12eed5dc, SUC_lexicon_tags_26 },
    { 0x302ed5dd, SUC_lexicon_tags_10 },
    { 0xfcaed5df, SUC_lexicon_tags_14 },
    { 0xbb0ed5e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820ed5e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x318ed5e7, SUC_lexicon_tags_37 },
    { 0x2d6ed5e8, SUC_lexicon_tags_45 },
    { 0xcfced5e9, SUC_lexicon_tags_17 },
    { 0x4e4ed5ea, SUC_lexicon_tags_5 },
    { 0xf26ed5eb, SUC_lexicon_tags_94 },
    { 0x174ed5ec, SUC_lexicon_tags_30 },
    { 0x76aed5e9, SUC_lexicon_tags_11 },
    { 0x1b6ed5ee, SUC_lexicon_tags_11 },
    { 0x468ed5ec, SUC_lexicon_tags_35 },
    { 0xf8eed5eb, SUC_lexicon_tags_84 },
    { 0x9c6ed5ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656ed5f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2aed5f8, SUC_lexicon_tags_30 },
    { 0x97eed5f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aed5fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88ed5ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96ed605, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x368ed607, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x784ed60d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcced60f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ced613, SUC_lexicon_tags_46 },
    { 0x5ceed614, SUC_lexicon_tags_2 },
    { 0xb68ed615, SUC_lexicon_tags_14 },
    { 0x758ed615, SUC_lexicon_tags_77 },
    { 0x580ed617, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ced61a, SUC_lexicon_tags_424 },
    { 0xb14ed61b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd40ed61d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84ed620, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aed623, SUC_lexicon_tags_15 },
    { 0x446ed624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94aed628, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8ed62c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6aed630, SUC_lexicon_tags_38 },
    { 0xf98ed631, SUC_lexicon_tags_46 },
    { 0xfceed632, SUC_lexicon_tags_63 },
    { 0x8b8ed633, SUC_lexicon_tags_65 },
    { 0x486ed634, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dced637, SUC_lexicon_tags_15 },
    { 0x6a8ed638, SUC_lexicon_tags_33 },
    { 0xa30ed637, SUC_lexicon_tags_17 },
    { 0xd8aed63a, SUC_lexicon_tags_29 },
    { 0xbd0ed63a, SUC_lexicon_tags_45 },
    { 0x812ed63a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18ed63f, SUC_lexicon_tags_56 },
    { 0x724ed63f, SUC_lexicon_tags_6 },
    { 0x402ed641, SUC_lexicon_tags_25 },
    { 0x888ed63f, SUC_lexicon_tags_0 },
    { 0x928ed641, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6ed646, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1deed649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2eed64b, SUC_lexicon_tags_33 },
    { 0xaeed64b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0ed650, SUC_lexicon_tags_40 },
    { 0xd46ed651, SUC_lexicon_tags_33 },
    { 0x8a8ed652, SUC_lexicon_tags_25 },
    { 0xf30ed651, SUC_lexicon_tags_16 },
    { 0xebaed654, SUC_lexicon_tags_23 },
    { 0x462ed655, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2ed659, SUC_lexicon_tags_11 },
    { 0x1a0ed65a, SUC_lexicon_tags_23 },
    { 0x278ed65b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3deed662, SUC_lexicon_tags_2 },
    { 0x5a8ed663, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4ed66b, SUC_lexicon_tags_25 },
    { 0x5e4ed66c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480ed673, SUC_lexicon_tags_11 },
    { 0x724ed674, SUC_lexicon_tags_27 },
    { 0x2bced675, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0eed678, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x254ed67a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2ed681, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x81eed684, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85eed68a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14ed68f, SUC_lexicon_tags_11 },
    { 0xf7eed68f, SUC_lexicon_tags_27 },
    { 0x986ed691, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xeceed694, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b4ed696, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xff8ed698, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a4ed69a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23eed6a0, SUC_lexicon_tags_38 },
    { 0xa8aed6a1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0ed6a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120ed6ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b4ed6ad, SUC_lexicon_tags_33 },
    { 0x3d2ed6ae, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438ed6b6, SUC_lexicon_tags_21 },
    { 0x8b8ed6b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31eed6bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeed6c3, SUC_lexicon_tags_14 },
    { 0x2f0ed6c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaed6cf, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea4ed6d5, SUC_lexicon_tags_11 },
    { 0x818ed6d6, SUC_lexicon_tags_31 },
    { 0xfdeed6d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ced6dd, SUC_lexicon_tags_17 },
    { 0x67eed6de, SUC_lexicon_tags_15 },
    { 0x8d4ed6de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468ed6e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842ed6e9, SUC_lexicon_tags_14 },
    { 0x7eced6ea, SUC_lexicon_tags_8 },
    { 0x6c0ed6eb, SUC_lexicon_tags_30 },
    { 0xfc8ed6ec, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380ed6f1, SUC_lexicon_tags_60 },
    { 0xc3eed6f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d4ed6f4, SUC_lexicon_tags_1 },
    { 0xafced6f4, SUC_lexicon_tags_0 },
    { 0x70ced6f5, SUC_lexicon_tags_7 },
    { 0x276ed6f6, SUC_lexicon_tags_32 },
    { 0x76aed6f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16ed6fd, SUC_lexicon_tags_26 },
    { 0x5c2ed6fe, SUC_lexicon_tags_1 },
    { 0xb3aed6fe, SUC_lexicon_tags_0 },
    { 0xf0eed700, SUC_lexicon_tags_12 },
    { 0xa8aed700, SUC_lexicon_tags_39 },
    { 0x75eed6fd, SUC_lexicon_tags_109 },
    { 0x628ed703, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x130ed705, SUC_lexicon_tags_17 },
    { 0x7c8ed705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2ed70f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ced713, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ced718, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876ed720, SUC_lexicon_tags_28 },
    { 0x308ed721, SUC_lexicon_tags_12 },
    { 0x8bced722, SUC_lexicon_tags_1 },
    { 0x134ed720, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178ed727, SUC_lexicon_tags_1 },
    { 0xea0ed728, SUC_lexicon_tags_6 },
    { 0x4a6ed729, SUC_lexicon_tags_11 },
    { 0xc60ed728, SUC_lexicon_tags_29 },
    { 0x2b8ed72b, SUC_lexicon_tags_6 },
    { 0x204ed72c, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854ed730, SUC_lexicon_tags_35 },
    { 0x49ced731, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3c4ed733, SUC_lexicon_tags_25 },
    { 0x2aeed733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ed73a, SUC_lexicon_tags_25 },
    { 0x266ed73b, SUC_lexicon_tags_17 },
    { 0x3caed73c, SUC_lexicon_tags_57 },
    { 0xe36ed73d, SUC_lexicon_tags_12 },
    { 0x6aed73e, SUC_lexicon_tags_7 },
    { 0x2d4ed73f, SUC_lexicon_tags_15 },
    { 0x2c0ed740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ced742, SUC_lexicon_tags_19 },
    { 0x146ed742, SUC_lexicon_tags_116 },
    { 0xacced744, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8ed747, SUC_lexicon_tags_29 },
    { 0x8b6ed748, SUC_lexicon_tags_17 },
    { 0x6c2ed749, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eced74c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ed754, SUC_lexicon_tags_94 },
    { 0x99ced755, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x302ed758, SUC_lexicon_tags_27 },
    { 0xcb6ed759, SUC_lexicon_tags_38 },
    { 0xf02ed75a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34ed75f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78ed763, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x340ed768, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4ed76b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776ed770, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x978ed772, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148ed779, SUC_lexicon_tags_0 },
    { 0x1b4ed77a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1a2ed77c, SUC_lexicon_tags_47 },
    { 0x96ced77d, SUC_lexicon_tags_33 },
    { 0xc94ed77e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5faed783, SUC_lexicon_tags_65 },
    { 0x94ed783, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdd0ed786, SUC_lexicon_tags_17 },
    { 0xe28ed787, SUC_lexicon_tags_37 },
    { 0x872ed788, SUC_lexicon_tags_107 },
    { 0xd46ed789, SUC_lexicon_tags_25 },
    { 0xe06ed78a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaced78e, SUC_lexicon_tags_45 },
    { 0xa3aed78f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36eed793, SUC_lexicon_tags_11 },
    { 0x182ed793, SUC_lexicon_tags_37 },
    { 0xc32ed794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70ed799, SUC_lexicon_tags_23 },
    { 0xaeced79a, SUC_lexicon_tags_28 },
    { 0xbbced79b, SUC_lexicon_tags_17 },
    { 0x318ed79b, SUC_lexicon_tags_17 },
    { 0x3c8ed79d, SUC_lexicon_tags_15 },
    { 0xe8ced79e, SUC_lexicon_tags_23 },
    { 0xa3aed79e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xec8ed7a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe08ed7a3, SUC_lexicon_tags_2 },
    { 0x8ced7a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6eaed7a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0ed7ac, SUC_lexicon_tags_22 },
    { 0x574ed7ad, SUC_lexicon_tags_6 },
    { 0xe0ced7ad, SUC_lexicon_tags_174 },
    { 0xd30ed7af, SUC_lexicon_tags_0 },
    { 0x604ed7af, SUC_lexicon_tags_0 },
    { 0xdaaed7af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2ced7b3, SUC_lexicon_tags_43 },
    { 0x8a0ed7b3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ed7b7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568ed7bd, SUC_lexicon_tags_773 },
    { 0, NULL },
    { 0x20ed7bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770ed7c4, SUC_lexicon_tags_0 },
    { 0xa8aed7c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x634ed7c8, SUC_lexicon_tags_27 },
    { 0x53eed7c9, SUC_lexicon_tags_2 },
    { 0x4aced7ca, SUC_lexicon_tags_14 },
    { 0xbcced7cb, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546ed7d1, SUC_lexicon_tags_37 },
    { 0x792ed7d2, SUC_lexicon_tags_38 },
    { 0x5eed7d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16ed7d5, SUC_lexicon_tags_2 },
    { 0x33eed7d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3eed7d8, SUC_lexicon_tags_23 },
    { 0xa2ed7d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x224ed7dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x734ed7de, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x62ced7e0, SUC_lexicon_tags_11 },
    { 0x36ced7e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2aed7e4, SUC_lexicon_tags_69 },
    { 0x376ed7e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ced7eb, SUC_lexicon_tags_17 },
    { 0x378ed7eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x940ed7ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b4ed7f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x118ed7f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xab6ed7f5, SUC_lexicon_tags_47 },
    { 0x190ed7f6, SUC_lexicon_tags_23 },
    { 0x342ed7f6, SUC_lexicon_tags_6 },
    { 0xa34ed7f8, SUC_lexicon_tags_19 },
    { 0xeaced7f8, SUC_lexicon_tags_15 },
    { 0x3e2ed7fa, SUC_lexicon_tags_21 },
    { 0x7a0ed7f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7aed7fd, SUC_lexicon_tags_9 },
    { 0x832ed7fd, SUC_lexicon_tags_2 },
    { 0x22eed7ff, SUC_lexicon_tags_65 },
    { 0xc90ed800, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ced805, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ced808, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaed80b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fced80f, SUC_lexicon_tags_32 },
    { 0xf9aed810, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652ed816, SUC_lexicon_tags_23 },
    { 0x536ed817, SUC_lexicon_tags_71 },
    { 0x8d6ed816, SUC_lexicon_tags_79 },
    { 0x728ed818, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6ed81c, SUC_lexicon_tags_92 },
    { 0x3a4ed81d, SUC_lexicon_tags_7 },
    { 0xff4ed81d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2ed824, SUC_lexicon_tags_118 },
    { 0x38ed825, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd90ed828, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daed830, SUC_lexicon_tags_63 },
    { 0x904ed830, SUC_lexicon_tags_226 },
    { 0x2beed831, SUC_lexicon_tags_10 },
    { 0x4f0ed833, SUC_lexicon_tags_29 },
    { 0xd4ed832, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584ed83a, SUC_lexicon_tags_33 },
    { 0x662ed83b, SUC_lexicon_tags_37 },
    { 0xc32ed83c, SUC_lexicon_tags_16 },
    { 0xba4ed83d, SUC_lexicon_tags_0 },
    { 0x722ed83d, SUC_lexicon_tags_7 },
    { 0x86aed83f, SUC_lexicon_tags_4 },
    { 0x668ed83e, SUC_lexicon_tags_25 },
    { 0xe2eed841, SUC_lexicon_tags_29 },
    { 0xa80ed842, SUC_lexicon_tags_37 },
    { 0x5c6ed83c, SUC_lexicon_tags_16 },
    { 0xd8aed844, SUC_lexicon_tags_15 },
    { 0x912ed845, SUC_lexicon_tags_26 },
    { 0x6b2ed846, SUC_lexicon_tags_79 },
    { 0xe1eed842, SUC_lexicon_tags_9 },
    { 0x97aed848, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aced84c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504ed856, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc66ed858, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68ed85b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51aed85d, SUC_lexicon_tags_19 },
    { 0x4d8ed85e, SUC_lexicon_tags_49 },
    { 0xe5eed85e, SUC_lexicon_tags_55 },
    { 0xf42ed860, SUC_lexicon_tags_11 },
    { 0x364ed861, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x54aed863, SUC_lexicon_tags_15 },
    { 0x510ed863, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xea2ed867, SUC_lexicon_tags_23 },
    { 0x3e4ed868, SUC_lexicon_tags_11 },
    { 0x668ed868, SUC_lexicon_tags_18 },
    { 0xbc0ed86a, SUC_lexicon_tags_12 },
    { 0xd92ed869, SUC_lexicon_tags_47 },
    { 0x91aed868, SUC_lexicon_tags_7 },
    { 0xb4ced86d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x890ed86f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4baed871, SUC_lexicon_tags_12 },
    { 0xc22ed871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ced87e, SUC_lexicon_tags_41 },
    { 0x71eed87e, SUC_lexicon_tags_82 },
    { 0x9f8ed880, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ced88a, SUC_lexicon_tags_8 },
    { 0x9faed88b, SUC_lexicon_tags_37 },
    { 0x47aed88c, SUC_lexicon_tags_61 },
    { 0x79aed88d, SUC_lexicon_tags_45 },
    { 0x102ed88e, SUC_lexicon_tags_25 },
    { 0xa32ed88f, SUC_lexicon_tags_60 },
    { 0x5eaed88e, SUC_lexicon_tags_37 },
    { 0xb88ed891, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20ed896, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ced899, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x106ed89b, SUC_lexicon_tags_0 },
    { 0x314ed89b, SUC_lexicon_tags_47 },
    { 0x1dced89b, SUC_lexicon_tags_61 },
    { 0x654ed89e, SUC_lexicon_tags_37 },
    { 0x9a0ed89b, SUC_lexicon_tags_17 },
    { 0x11aed89e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x79aed8a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728ed8a7, SUC_lexicon_tags_8 },
    { 0x9aced8a8, SUC_lexicon_tags_12 },
    { 0x56ed8a7, SUC_lexicon_tags_32 },
    { 0xa02ed8aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4ed8ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcc0ed8b0, SUC_lexicon_tags_25 },
    { 0x672ed8b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xde8ed8b3, SUC_lexicon_tags_37 },
    { 0x8b4ed8b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72eed8ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x292ed8bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43eed8c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2ed8c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4ed8ca, SUC_lexicon_tags_23 },
    { 0x400ed8cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00ed8d0, SUC_lexicon_tags_25 },
    { 0x86ed8d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2feed8d3, SUC_lexicon_tags_11 },
    { 0x4e0ed8d4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x976ed8d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbced8d8, SUC_lexicon_tags_37 },
    { 0x6b2ed8d9, SUC_lexicon_tags_37 },
    { 0x4d8ed8d9, SUC_lexicon_tags_17 },
    { 0x54aed8d8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x450ed8de, SUC_lexicon_tags_1 },
    { 0x1ceed8df, SUC_lexicon_tags_48 },
    { 0x188ed8df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ced8e3, SUC_lexicon_tags_7 },
    { 0x468ed8e3, SUC_lexicon_tags_33 },
    { 0xf36ed8e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16eed8e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54ed8ea, SUC_lexicon_tags_92 },
    { 0x40ed8eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4deed8ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd62ed8f0, SUC_lexicon_tags_14 },
    { 0xb8eed8f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ced8f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6ed8f8, SUC_lexicon_tags_10 },
    { 0xacced8f9, SUC_lexicon_tags_37 },
    { 0x69ced8f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x258ed8fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8eed8ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12ed906, SUC_lexicon_tags_69 },
    { 0xab6ed907, SUC_lexicon_tags_17 },
    { 0x652ed907, SUC_lexicon_tags_174 },
    { 0x5b0ed909, SUC_lexicon_tags_23 },
    { 0x7faed90a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa48ed90c, SUC_lexicon_tags_37 },
    { 0x6a2ed90d, SUC_lexicon_tags_30 },
    { 0x4e8ed90e, SUC_lexicon_tags_2 },
    { 0x41eed90d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ced912, SUC_lexicon_tags_77 },
    { 0x6c8ed913, SUC_lexicon_tags_7 },
    { 0xd70ed914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0ed917, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x786ed919, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0ed91d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x806ed920, SUC_lexicon_tags_33 },
    { 0x128ed921, SUC_lexicon_tags_47 },
    { 0xb60ed922, SUC_lexicon_tags_77 },
    { 0xfdced922, SUC_lexicon_tags_28 },
    { 0x7e4ed923, SUC_lexicon_tags_11 },
    { 0x5feed925, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5aed927, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa46ed929, SUC_lexicon_tags_38 },
    { 0x976ed92a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26eed92e, SUC_lexicon_tags_37 },
    { 0x6a4ed92f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb2eed931, SUC_lexicon_tags_11 },
    { 0x5aaed932, SUC_lexicon_tags_29 },
    { 0x704ed931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240ed938, SUC_lexicon_tags_26 },
    { 0xf00ed939, SUC_lexicon_tags_32 },
    { 0x7eced93a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84ced93c, SUC_lexicon_tags_1 },
    { 0x434ed93d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaf2ed93f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc5eed941, SUC_lexicon_tags_0 },
    { 0x3d4ed942, SUC_lexicon_tags_92 },
    { 0x424ed943, SUC_lexicon_tags_151 },
    { 0xd82ed944, SUC_lexicon_tags_11 },
    { 0xb2ed943, SUC_lexicon_tags_37 },
    { 0x70ced946, SUC_lexicon_tags_87 },
    { 0xf06ed941, SUC_lexicon_tags_43 },
    { 0x4baed948, SUC_lexicon_tags_66 },
    { 0xadeed947, SUC_lexicon_tags_19 },
    { 0x8d8ed941, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872ed950, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ed955, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa86ed957, SUC_lexicon_tags_7 },
    { 0xeaced958, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x214ed95a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876ed964, SUC_lexicon_tags_19 },
    { 0x262ed965, SUC_lexicon_tags_38 },
    { 0xf46ed966, SUC_lexicon_tags_15 },
    { 0xd4aed967, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3a0ed969, SUC_lexicon_tags_25 },
    { 0x364ed96a, SUC_lexicon_tags_62 },
    { 0xe58ed96a, SUC_lexicon_tags_8 },
    { 0xa96ed96c, SUC_lexicon_tags_6 },
    { 0x6cced96b, SUC_lexicon_tags_37 },
    { 0xe86ed96c, SUC_lexicon_tags_29 },
    { 0x1f2ed969, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf8aed971, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4ed975, SUC_lexicon_tags_19 },
    { 0x276ed976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x396ed978, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x16ced97a, SUC_lexicon_tags_17 },
    { 0x1d8ed97b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe7ced97d, SUC_lexicon_tags_38 },
    { 0xb4ced97e, SUC_lexicon_tags_17 },
    { 0x18ced97f, SUC_lexicon_tags_30 },
    { 0x104ed97f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceed983, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d8ed985, SUC_lexicon_tags_28 },
    { 0x966ed986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4ed98b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ced994, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1cced996, SUC_lexicon_tags_11 },
    { 0xbe6ed997, SUC_lexicon_tags_15 },
    { 0xec4ed998, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x502ed99a, SUC_lexicon_tags_6 },
    { 0x6ed99b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0ed99e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x964ed9a0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aced9a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x838ed9a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73eed9a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ced9af, SUC_lexicon_tags_4 },
    { 0x8baed9b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ced9bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4ed9bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6ed9c3, SUC_lexicon_tags_30 },
    { 0x12aed9c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216ed9d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94ed9d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ed9d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xabaed9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4ed9e7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52aed9ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x816ed9ef, SUC_lexicon_tags_45 },
    { 0x306ed9f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa16ed9f2, SUC_lexicon_tags_154 },
    { 0x5b2ed9f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ced9f8, SUC_lexicon_tags_63 },
    { 0xdaed9f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x686ed9fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x680ed9ff, SUC_lexicon_tags_7 },
    { 0x9b6eda00, SUC_lexicon_tags_11 },
    { 0xe6eed9ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0ceda03, SUC_lexicon_tags_77 },
    { 0x1f2eda04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb6eda06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4eda09, SUC_lexicon_tags_37 },
    { 0x114eda0a, SUC_lexicon_tags_40 },
    { 0x178eda0a, SUC_lexicon_tags_61 },
    { 0xbe6eda0b, SUC_lexicon_tags_11 },
    { 0x3b6eda0d, SUC_lexicon_tags_47 },
    { 0xe92eda0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ceda13, SUC_lexicon_tags_27 },
    { 0x984eda13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aeda1c, SUC_lexicon_tags_141 },
    { 0x44ceda1d, SUC_lexicon_tags_17 },
    { 0xea4eda1e, SUC_lexicon_tags_9 },
    { 0xad6eda1e, SUC_lexicon_tags_0 },
    { 0x8d0eda1d, SUC_lexicon_tags_19 },
    { 0x890eda21, SUC_lexicon_tags_37 },
    { 0x6aeda1f, SUC_lexicon_tags_2 },
    { 0x532eda22, SUC_lexicon_tags_15 },
    { 0xd2aeda1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x150eda26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0eda2a, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x496eda2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9eeda2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1a2eda30, SUC_lexicon_tags_21 },
    { 0x37ceda30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x13ceda33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73aeda35, SUC_lexicon_tags_25 },
    { 0x90ceda35, SUC_lexicon_tags_30 },
    { 0xa0ceda37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x440eda3a, SUC_lexicon_tags_11 },
    { 0x320eda3b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xeb2eda3d, SUC_lexicon_tags_11 },
    { 0xec2eda3d, SUC_lexicon_tags_17 },
    { 0xb4eeda3d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x452eda41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x96aeda43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a8eda45, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbcceda47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe46eda49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdceda4d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1f2eda4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeda53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556eda57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb74eda59, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8eda5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x928eda60, SUC_lexicon_tags_0 },
    { 0xc7ceda61, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6a6eda63, SUC_lexicon_tags_60 },
    { 0x860eda64, SUC_lexicon_tags_27 },
    { 0x12eeda65, SUC_lexicon_tags_43 },
    { 0x8cceda66, SUC_lexicon_tags_15 },
    { 0x35eeda67, SUC_lexicon_tags_37 },
    { 0xfb0eda68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94eda6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faeda74, SUC_lexicon_tags_15 },
    { 0x442eda75, SUC_lexicon_tags_27 },
    { 0x492eda76, SUC_lexicon_tags_33 },
    { 0xe1eeda77, SUC_lexicon_tags_94 },
    { 0xd1ceda78, SUC_lexicon_tags_14 },
    { 0x566eda79, SUC_lexicon_tags_33 },
    { 0xef2eda79, SUC_lexicon_tags_47 },
    { 0xc96eda78, SUC_lexicon_tags_2 },
    { 0x4c0eda7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x426eda7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xec6eda81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x222eda83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49aeda85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc64eda87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6eda8a, SUC_lexicon_tags_6 },
    { 0x9c0eda8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x222eda8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d4eda8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x448eda92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4eda96, SUC_lexicon_tags_1 },
    { 0x86ceda96, SUC_lexicon_tags_15 },
    { 0x56aeda98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x88eda9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9eeda9c, SUC_lexicon_tags_30 },
    { 0xaceda9d, SUC_lexicon_tags_7 },
    { 0x3eceda9d, SUC_lexicon_tags_11 },
    { 0x910eda9c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x956edaa2, SUC_lexicon_tags_15 },
    { 0x730edaa3, SUC_lexicon_tags_17 },
    { 0x5e2edaa4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2edaaa, SUC_lexicon_tags_7 },
    { 0xf60edaab, SUC_lexicon_tags_6 },
    { 0x1d0edaac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x944edaaf, SUC_lexicon_tags_29 },
    { 0x904edaaf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaedab5, SUC_lexicon_tags_33 },
    { 0x246edab6, SUC_lexicon_tags_55 },
    { 0x434edab6, SUC_lexicon_tags_14 },
    { 0x48aedab7, SUC_lexicon_tags_35 },
    { 0x964edab6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0aedabb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06edabf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a2edac1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc36edac3, SUC_lexicon_tags_63 },
    { 0xeacedac3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2daedac6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x570edac9, SUC_lexicon_tags_47 },
    { 0x5e8edaca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30edad0, SUC_lexicon_tags_31 },
    { 0xcfeedad1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x154edad4, SUC_lexicon_tags_25 },
    { 0x850edad5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74edada, SUC_lexicon_tags_17 },
    { 0xe7eedadb, SUC_lexicon_tags_11 },
    { 0x55cedadc, SUC_lexicon_tags_32 },
    { 0x3e6edadd, SUC_lexicon_tags_60 },
    { 0x1cedadb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb2edae0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5e8edae2, SUC_lexicon_tags_37 },
    { 0xc30edae3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcedae7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8edaea, SUC_lexicon_tags_12 },
    { 0xdd0edaeb, SUC_lexicon_tags_28 },
    { 0xd8eedaec, SUC_lexicon_tags_18 },
    { 0xed0edaed, SUC_lexicon_tags_31 },
    { 0xe2edaec, SUC_lexicon_tags_7 },
    { 0xc18edaef, SUC_lexicon_tags_37 },
    { 0xc6edaee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88edaf9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6edafc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc7aedafe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaedb01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6edb04, SUC_lexicon_tags_49 },
    { 0xf4cedb04, SUC_lexicon_tags_7 },
    { 0x724edb04, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2edb0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cedb0f, SUC_lexicon_tags_5 },
    { 0xec8edb10, SUC_lexicon_tags_26 },
    { 0xe24edb11, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x742edb13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabcedb16, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1aedb1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf9eedb1f, SUC_lexicon_tags_87 },
    { 0x428edb1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6aedb22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168edb29, SUC_lexicon_tags_15 },
    { 0xb52edb2a, SUC_lexicon_tags_8 },
    { 0xb92edb2b, SUC_lexicon_tags_62 },
    { 0x31eedb2c, SUC_lexicon_tags_25 },
    { 0x832edb2c, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x622edb2f, SUC_lexicon_tags_14 },
    { 0x59eedb2f, SUC_lexicon_tags_30 },
    { 0xa6cedb2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d6edb33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6eedb35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50cedb38, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5c8edb3a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe74edb3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12edb3e, SUC_lexicon_tags_27 },
    { 0xdb2edb3e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaedb43, SUC_lexicon_tags_38 },
    { 0x922edb44, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8edb48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2aedb4d, SUC_lexicon_tags_29 },
    { 0xf10edb4e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x65cedb50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6edb59, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888edb5d, SUC_lexicon_tags_56 },
    { 0x7b6edb5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6edb62, SUC_lexicon_tags_47 },
    { 0x106edb63, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x262edb6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62edb70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf50edb72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748edb77, SUC_lexicon_tags_7 },
    { 0x39eedb78, SUC_lexicon_tags_63 },
    { 0x2f8edb79, SUC_lexicon_tags_14 },
    { 0x206edb79, SUC_lexicon_tags_31 },
    { 0x2b4edb7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0edb80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ecedb82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x206edb85, SUC_lexicon_tags_39 },
    { 0x264edb86, SUC_lexicon_tags_45 },
    { 0xa7cedb87, SUC_lexicon_tags_43 },
    { 0x150edb88, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54edb8b, SUC_lexicon_tags_28 },
    { 0xf6cedb8c, SUC_lexicon_tags_11 },
    { 0x1b8edb8d, SUC_lexicon_tags_25 },
    { 0x622edb8e, SUC_lexicon_tags_6 },
    { 0x190edb8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8edb94, SUC_lexicon_tags_82 },
    { 0x2f4edb95, SUC_lexicon_tags_21 },
    { 0xf7cedb96, SUC_lexicon_tags_38 },
    { 0xfc8edb96, SUC_lexicon_tags_17 },
    { 0x7d2edb98, SUC_lexicon_tags_66 },
    { 0xf4eedb99, SUC_lexicon_tags_16 },
    { 0xc96edb9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x728edb9c, SUC_lexicon_tags_12 },
    { 0x5f8edb9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaedba0, SUC_lexicon_tags_25 },
    { 0xe56edba0, SUC_lexicon_tags_43 },
    { 0xdeaedba1, SUC_lexicon_tags_79 },
    { 0x17cedba3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa16edba5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x550edba7, SUC_lexicon_tags_44 },
    { 0x31cedba7, SUC_lexicon_tags_19 },
    { 0x52aedba9, SUC_lexicon_tags_19 },
    { 0x76eedbaa, SUC_lexicon_tags_60 },
    { 0x52aedbab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8faedbae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6aedbb0, SUC_lexicon_tags_17 },
    { 0x8a2edbb0, SUC_lexicon_tags_24 },
    { 0x436edbb0, SUC_lexicon_tags_45 },
    { 0x2e0edbb3, SUC_lexicon_tags_25 },
    { 0x542edbb4, SUC_lexicon_tags_25 },
    { 0x4a4edbb5, SUC_lexicon_tags_15 },
    { 0x86edbb3, SUC_lexicon_tags_37 },
    { 0x65aedbb7, SUC_lexicon_tags_38 },
    { 0x84edbb7, SUC_lexicon_tags_1 },
    { 0x188edbb2, SUC_lexicon_tags_49 },
    { 0x8eaedbba, SUC_lexicon_tags_47 },
    { 0xf58edbba, SUC_lexicon_tags_33 },
    { 0xaf0edbb2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8edbc0, SUC_lexicon_tags_216 },
    { 0xf9cedbc1, SUC_lexicon_tags_0 },
    { 0x668edbc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cedbc6, SUC_lexicon_tags_37 },
    { 0xc26edbc6, SUC_lexicon_tags_15 },
    { 0x4eedbc8, SUC_lexicon_tags_47 },
    { 0x542edbc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14edbcd, SUC_lexicon_tags_2 },
    { 0x688edbce, SUC_lexicon_tags_33 },
    { 0xb64edbcf, SUC_lexicon_tags_60 },
    { 0x888edbd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf6edbd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaedbd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156edbda, SUC_lexicon_tags_87 },
    { 0xb88edbdb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xceedbdd, SUC_lexicon_tags_31 },
    { 0x562edbde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeedbe1, SUC_lexicon_tags_19 },
    { 0xcdaedbe1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9ccedbe4, SUC_lexicon_tags_19 },
    { 0x32edbe5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8fcedbe7, SUC_lexicon_tags_37 },
    { 0x1f8edbe8, SUC_lexicon_tags_13 },
    { 0x27eedbe7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x658edbec, SUC_lexicon_tags_36 },
    { 0xa98edbed, SUC_lexicon_tags_28 },
    { 0x40eedbee, SUC_lexicon_tags_28 },
    { 0x41cedbef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x172edbf1, SUC_lexicon_tags_17 },
    { 0xefeedbf2, SUC_lexicon_tags_9 },
    { 0x18cedbf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58edbf6, SUC_lexicon_tags_32 },
    { 0x382edbf7, SUC_lexicon_tags_7 },
    { 0x986edbf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeedc00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384edc0a, SUC_lexicon_tags_2 },
    { 0xeb8edc0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccedc0d, SUC_lexicon_tags_13 },
    { 0x8ceedc0e, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x968edc10, SUC_lexicon_tags_7 },
    { 0x49eedc10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22edc16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2edc1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8edc29, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcd4edc2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaceedc2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eedc37, SUC_lexicon_tags_27 },
    { 0x248edc38, SUC_lexicon_tags_17 },
    { 0xb42edc37, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20edc40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2edc4c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb02edc4e, SUC_lexicon_tags_11 },
    { 0x7eedc4f, SUC_lexicon_tags_34 },
    { 0xcd4edc4f, SUC_lexicon_tags_16 },
    { 0x718edc4f, SUC_lexicon_tags_23 },
    { 0xd2eedc52, SUC_lexicon_tags_14 },
    { 0x9b6edc4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x89eedc56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63eedc59, SUC_lexicon_tags_18 },
    { 0x1fcedc5a, SUC_lexicon_tags_27 },
    { 0xc3cedc5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x466edc5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caedc64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2edc69, SUC_lexicon_tags_11 },
    { 0x72eedc69, SUC_lexicon_tags_207 },
    { 0x6baedc6b, SUC_lexicon_tags_45 },
    { 0xfd4edc6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf62edc6e, SUC_lexicon_tags_43 },
    { 0xb2edc6f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cedc72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6edc79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6edc7f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa92edc81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66edc8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62aedc91, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb8aedc93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x808edc95, SUC_lexicon_tags_17 },
    { 0xb68edc96, SUC_lexicon_tags_16 },
    { 0xfb2edc97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42edc9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808edca1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcedca5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756edcaa, SUC_lexicon_tags_108 },
    { 0x6ecedcab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x62cedcad, SUC_lexicon_tags_34 },
    { 0xeeedcae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5acedcb0, SUC_lexicon_tags_86 },
    { 0x558edcb0, SUC_lexicon_tags_39 },
    { 0xc46edcb1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e0edcb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29cedcb6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154edcbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4edcbe, SUC_lexicon_tags_26 },
    { 0xb9aedcbf, SUC_lexicon_tags_17 },
    { 0x76edcc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddeedcc5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7b6edcc7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796edccc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x766edccf, SUC_lexicon_tags_19 },
    { 0x296edccf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34edcd4, SUC_lexicon_tags_10 },
    { 0xc02edcd4, SUC_lexicon_tags_28 },
    { 0x61cedcd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaedcda, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9aedcde, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8ccedce0, SUC_lexicon_tags_14 },
    { 0x540edce1, SUC_lexicon_tags_23 },
    { 0xd64edce1, SUC_lexicon_tags_178 },
    { 0x460edce3, SUC_lexicon_tags_4 },
    { 0x270edce1, SUC_lexicon_tags_27 },
    { 0xd14edce5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79eedce7, SUC_lexicon_tags_15 },
    { 0xe36edce8, SUC_lexicon_tags_12 },
    { 0x2f6edce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8edceb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992edcef, SUC_lexicon_tags_19 },
    { 0x870edcf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x448edcf3, SUC_lexicon_tags_2 },
    { 0x8d4edcf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2edcfe, SUC_lexicon_tags_38 },
    { 0xaaedcfe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x464edd02, SUC_lexicon_tags_25 },
    { 0x854edd03, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecedd06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0edd09, SUC_lexicon_tags_3 },
    { 0x2fcedd0a, SUC_lexicon_tags_12 },
    { 0x43cedd0b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36edd0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4feedd14, SUC_lexicon_tags_2 },
    { 0x5a0edd15, SUC_lexicon_tags_7 },
    { 0xa10edd16, SUC_lexicon_tags_7 },
    { 0x51eedd17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2edd1e, SUC_lexicon_tags_0 },
    { 0x806edd1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8b8edd21, SUC_lexicon_tags_45 },
    { 0xb78edd22, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaedd26, SUC_lexicon_tags_55 },
    { 0x82cedd26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6edd2d, SUC_lexicon_tags_50 },
    { 0xa0edd2e, SUC_lexicon_tags_49 },
    { 0xaf8edd2f, SUC_lexicon_tags_45 },
    { 0x774edd30, SUC_lexicon_tags_37 },
    { 0xc6cedd30, SUC_lexicon_tags_2 },
    { 0x61cedd31, SUC_lexicon_tags_37 },
    { 0x5b8edd2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eedd38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x920edd3b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x212edd3e, SUC_lexicon_tags_61 },
    { 0xbc4edd3f, SUC_lexicon_tags_60 },
    { 0xe62edd40, SUC_lexicon_tags_17 },
    { 0x904edd41, SUC_lexicon_tags_27 },
    { 0xc6edd40, SUC_lexicon_tags_25 },
    { 0x5a4edd41, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd18edd45, SUC_lexicon_tags_23 },
    { 0x96edd46, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6caedd49, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22edd50, SUC_lexicon_tags_77 },
    { 0x4f4edd50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78edd57, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4edd5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x878edd5e, SUC_lexicon_tags_17 },
    { 0xca2edd5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8edd61, SUC_lexicon_tags_6 },
    { 0x4c0edd62, SUC_lexicon_tags_14 },
    { 0xe82edd63, SUC_lexicon_tags_63 },
    { 0x2e4edd64, SUC_lexicon_tags_34 },
    { 0x7a4edd65, SUC_lexicon_tags_37 },
    { 0xc8eedd64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6edd6b, SUC_lexicon_tags_17 },
    { 0x8b4edd6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2c2edd6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2edd73, SUC_lexicon_tags_2 },
    { 0xeb6edd73, SUC_lexicon_tags_37 },
    { 0x66eedd75, SUC_lexicon_tags_2 },
    { 0x37aedd76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4b2edd78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eedd7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888edd82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2edd86, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe9aedd88, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792edd8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x500edd8f, SUC_lexicon_tags_31 },
    { 0x27cedd90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf76edd92, SUC_lexicon_tags_9 },
    { 0xd3cedd92, SUC_lexicon_tags_11 },
    { 0x44edd92, SUC_lexicon_tags_17 },
    { 0xcaaedd95, SUC_lexicon_tags_11 },
    { 0xa4edd96, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x40eedd98, SUC_lexicon_tags_25 },
    { 0xd0eedd99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7aedd9b, SUC_lexicon_tags_2 },
    { 0xc5eedd9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58edda1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24cedda8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0eddab, SUC_lexicon_tags_0 },
    { 0x25eeddab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86eeddb3, SUC_lexicon_tags_1 },
    { 0x1f0eddb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2eddbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aceddbf, SUC_lexicon_tags_2 },
    { 0x44ceddbf, SUC_lexicon_tags_8 },
    { 0x706eddc1, SUC_lexicon_tags_43 },
    { 0x602eddc2, SUC_lexicon_tags_10 },
    { 0x602eddc2, SUC_lexicon_tags_49 },
    { 0x2daeddc4, SUC_lexicon_tags_25 },
    { 0x402eddbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2ceddc7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ceddca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2eddcd, SUC_lexicon_tags_65 },
    { 0x57aeddce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36eddd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236eddd8, SUC_lexicon_tags_56 },
    { 0x62eeddd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0eddde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94aedde0, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6edde5, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x454edde7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaacedde9, SUC_lexicon_tags_15 },
    { 0xa5aedde9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4deedded, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364eddf1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeeddf5, SUC_lexicon_tags_33 },
    { 0xeceddf6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58aede05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6ede0b, SUC_lexicon_tags_6 },
    { 0x3eede0c, SUC_lexicon_tags_23 },
    { 0x3b6ede0b, SUC_lexicon_tags_17 },
    { 0xb64ede0b, SUC_lexicon_tags_60 },
    { 0x788ede0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4ede12, SUC_lexicon_tags_10 },
    { 0x996ede12, SUC_lexicon_tags_49 },
    { 0x212ede12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4a0ede16, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x790ede18, SUC_lexicon_tags_63 },
    { 0xd2aede18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48ede1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1eede21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40ede2b, SUC_lexicon_tags_25 },
    { 0x5b8ede2b, SUC_lexicon_tags_37 },
    { 0xab2ede2d, SUC_lexicon_tags_38 },
    { 0xe5eede2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0ede34, SUC_lexicon_tags_61 },
    { 0x3c8ede35, SUC_lexicon_tags_12 },
    { 0xa28ede36, SUC_lexicon_tags_17 },
    { 0x45aede34, SUC_lexicon_tags_114 },
    { 0x2c6ede37, SUC_lexicon_tags_47 },
    { 0x438ede39, SUC_lexicon_tags_4 },
    { 0x89eede3a, SUC_lexicon_tags_37 },
    { 0xd34ede3b, SUC_lexicon_tags_1 },
    { 0xb2ede3a, SUC_lexicon_tags_37 },
    { 0xcb2ede35, SUC_lexicon_tags_25 },
    { 0x1aaede3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654ede47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcede4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16aede4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946ede54, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0eede58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48ede5e, SUC_lexicon_tags_25 },
    { 0xcf6ede5f, SUC_lexicon_tags_66 },
    { 0xd5cede60, SUC_lexicon_tags_6 },
    { 0xde2ede61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x514ede63, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6eede68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefeede6d, SUC_lexicon_tags_37 },
    { 0xb8aede6d, SUC_lexicon_tags_15 },
    { 0xbdaede6e, SUC_lexicon_tags_2 },
    { 0xe68ede70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0ede73, SUC_lexicon_tags_11 },
    { 0xbb2ede74, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8aede78, SUC_lexicon_tags_25 },
    { 0xa70ede78, SUC_lexicon_tags_2 },
    { 0xaacede7a, SUC_lexicon_tags_15 },
    { 0x31cede7a, SUC_lexicon_tags_7 },
    { 0x93aede7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceede82, SUC_lexicon_tags_17 },
    { 0x28ede83, SUC_lexicon_tags_7 },
    { 0xf0aede84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa70ede86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78aede88, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd98ede8a, SUC_lexicon_tags_11 },
    { 0xc0ede8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf06ede8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3faede8f, SUC_lexicon_tags_1 },
    { 0xafcede90, SUC_lexicon_tags_12 },
    { 0x5b6ede90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0ede95, SUC_lexicon_tags_45 },
    { 0x6b0ede95, SUC_lexicon_tags_14 },
    { 0x81aede97, SUC_lexicon_tags_17 },
    { 0x976ede98, SUC_lexicon_tags_17 },
    { 0x264ede99, SUC_lexicon_tags_15 },
    { 0xe4ede9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36aede9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0edea0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x948edea2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61cedea4, SUC_lexicon_tags_38 },
    { 0x3dcedea5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaedea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238edeac, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908edeb0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36edeb8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc0eedeba, SUC_lexicon_tags_37 },
    { 0xd1eedeba, SUC_lexicon_tags_27 },
    { 0x6cedebc, SUC_lexicon_tags_0 },
    { 0xb18edebd, SUC_lexicon_tags_45 },
    { 0x848edebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffcedec0, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xc38edec2, SUC_lexicon_tags_38 },
    { 0x97cedec3, SUC_lexicon_tags_30 },
    { 0xf2aedec4, SUC_lexicon_tags_27 },
    { 0x116edec3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cedecd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08edee3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32eedee5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91aedeef, SUC_lexicon_tags_55 },
    { 0x3aedef0, SUC_lexicon_tags_43 },
    { 0x90aedef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cedef4, SUC_lexicon_tags_37 },
    { 0xf52edef5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcedef8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x284edefa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaedefd, SUC_lexicon_tags_32 },
    { 0x5c8edefe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3e4edf00, SUC_lexicon_tags_11 },
    { 0x91eedf01, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e4edf03, SUC_lexicon_tags_11 },
    { 0x9e6edf03, SUC_lexicon_tags_2 },
    { 0xde6edf04, SUC_lexicon_tags_6 },
    { 0xfa4edf05, SUC_lexicon_tags_47 },
    { 0x530edf05, SUC_lexicon_tags_43 },
    { 0x8a4edf06, SUC_lexicon_tags_49 },
    { 0x6baedf06, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66edf10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf82edf12, SUC_lexicon_tags_25 },
    { 0xe28edf13, SUC_lexicon_tags_31 },
    { 0xd86edf14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeedf18, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4baedf1a, SUC_lexicon_tags_9 },
    { 0xd50edf1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28aedf24, SUC_lexicon_tags_37 },
    { 0x632edf25, SUC_lexicon_tags_37 },
    { 0x2b0edf24, SUC_lexicon_tags_39 },
    { 0xf58edf27, SUC_lexicon_tags_65 },
    { 0x888edf28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7aedf2a, SUC_lexicon_tags_3 },
    { 0x5daedf2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72edf2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7aedf38, SUC_lexicon_tags_37 },
    { 0x326edf39, SUC_lexicon_tags_66 },
    { 0x68edf3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ceedf40, SUC_lexicon_tags_15 },
    { 0xba4edf41, SUC_lexicon_tags_87 },
    { 0xe36edf42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2edf45, SUC_lexicon_tags_38 },
    { 0x1a6edf46, SUC_lexicon_tags_1 },
    { 0x584edf45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89eedf49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aedf4d, SUC_lexicon_tags_29 },
    { 0x564edf4d, SUC_lexicon_tags_15 },
    { 0xdb6edf4d, SUC_lexicon_tags_9 },
    { 0x158edf50, SUC_lexicon_tags_30 },
    { 0x35cedf51, SUC_lexicon_tags_9 },
    { 0x43eedf52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540edf5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2edf5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacedf66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f2edf68, SUC_lexicon_tags_30 },
    { 0xd9aedf69, SUC_lexicon_tags_26 },
    { 0xd9aedf69, SUC_lexicon_tags_25 },
    { 0x4ccedf69, SUC_lexicon_tags_60 },
    { 0xc34edf6b, SUC_lexicon_tags_1 },
    { 0x3caedf6b, SUC_lexicon_tags_21 },
    { 0x316edf6e, SUC_lexicon_tags_1 },
    { 0x5b6edf6f, SUC_lexicon_tags_37 },
    { 0x63cedf70, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cedf79, SUC_lexicon_tags_1 },
    { 0xfdeedf7a, SUC_lexicon_tags_23 },
    { 0x248edf7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x498edf7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x190edf7f, SUC_lexicon_tags_28 },
    { 0x4f0edf7f, SUC_lexicon_tags_1 },
    { 0x41cedf80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2edf85, SUC_lexicon_tags_17 },
    { 0x8acedf85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42cedf8a, SUC_lexicon_tags_57 },
    { 0x630edf8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990edf91, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8edf96, SUC_lexicon_tags_45 },
    { 0x3f0edf97, SUC_lexicon_tags_27 },
    { 0xbaeedf98, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6eedf9b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8edf9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eedfa4, SUC_lexicon_tags_30 },
    { 0x8bcedfa5, SUC_lexicon_tags_10 },
    { 0x592edfa5, SUC_lexicon_tags_11 },
    { 0x200edfa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcedfab, SUC_lexicon_tags_27 },
    { 0x780edfac, SUC_lexicon_tags_37 },
    { 0x830edfac, SUC_lexicon_tags_37 },
    { 0xecaedfad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72aedfb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x504edfb2, SUC_lexicon_tags_2 },
    { 0xcedfb2, SUC_lexicon_tags_30 },
    { 0xee2edfb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88edfb8, SUC_lexicon_tags_26 },
    { 0x22edfb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94edfbc, SUC_lexicon_tags_12 },
    { 0x7f4edfbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24aedfc1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eedfc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x75eedfc9, SUC_lexicon_tags_1 },
    { 0xafeedfca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cedfd0, SUC_lexicon_tags_2 },
    { 0xc9cedfd1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8edfdf, SUC_lexicon_tags_43 },
    { 0x3b6edfdf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24edfef, SUC_lexicon_tags_32 },
    { 0xa3aedff0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6edff3, SUC_lexicon_tags_45 },
    { 0xebaedff4, SUC_lexicon_tags_17 },
    { 0x30cedff5, SUC_lexicon_tags_4 },
    { 0xb60edff6, SUC_lexicon_tags_0 },
    { 0xf74edff6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8c6edff9, SUC_lexicon_tags_4 },
    { 0x178edffa, SUC_lexicon_tags_1 },
    { 0x8f2edffb, SUC_lexicon_tags_17 },
    { 0xdb4edffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4edfff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xba4ee001, SUC_lexicon_tags_15 },
    { 0x134ee001, SUC_lexicon_tags_6 },
    { 0x504ee002, SUC_lexicon_tags_17 },
    { 0x8a8ee004, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50ee00b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf64ee00d, SUC_lexicon_tags_25 },
    { 0x898ee00e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x156ee011, SUC_lexicon_tags_15 },
    { 0x9c4ee011, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8ee015, SUC_lexicon_tags_1 },
    { 0x3a2ee015, SUC_lexicon_tags_4 },
    { 0x7b0ee015, SUC_lexicon_tags_8 },
    { 0x42eee018, SUC_lexicon_tags_77 },
    { 0x48eee016, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x78aee01c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x458ee01e, SUC_lexicon_tags_14 },
    { 0xa5cee01e, SUC_lexicon_tags_56 },
    { 0x3ccee01f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00ee023, SUC_lexicon_tags_25 },
    { 0x5beee024, SUC_lexicon_tags_38 },
    { 0xde8ee025, SUC_lexicon_tags_14 },
    { 0xa20ee025, SUC_lexicon_tags_37 },
    { 0xe50ee025, SUC_lexicon_tags_7 },
    { 0x40ee024, SUC_lexicon_tags_12 },
    { 0x35aee029, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x516ee02b, SUC_lexicon_tags_37 },
    { 0x8b8ee02c, SUC_lexicon_tags_65 },
    { 0x8caee02d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3aee034, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6ee03a, SUC_lexicon_tags_25 },
    { 0x6ceee03b, SUC_lexicon_tags_27 },
    { 0xa00ee03c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e4ee03e, SUC_lexicon_tags_1 },
    { 0x656ee03e, SUC_lexicon_tags_37 },
    { 0x2dcee03f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8ee043, SUC_lexicon_tags_37 },
    { 0xd06ee044, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc8ee046, SUC_lexicon_tags_11 },
    { 0x222ee047, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaceee04a, SUC_lexicon_tags_23 },
    { 0xf56ee04a, SUC_lexicon_tags_17 },
    { 0x8e8ee04c, SUC_lexicon_tags_11 },
    { 0xc9eee04d, SUC_lexicon_tags_1 },
    { 0xadcee04e, SUC_lexicon_tags_17 },
    { 0xf20ee04f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcaee051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5caee054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x502ee056, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x91cee058, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976ee05f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99aee062, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8ee066, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8ee069, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3aaee06b, SUC_lexicon_tags_17 },
    { 0x62eee06b, SUC_lexicon_tags_6 },
    { 0x19eee06b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaee070, SUC_lexicon_tags_39 },
    { 0x7bcee070, SUC_lexicon_tags_37 },
    { 0x3d6ee072, SUC_lexicon_tags_28 },
    { 0xa98ee071, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x86aee075, SUC_lexicon_tags_30 },
    { 0xaf4ee075, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x79eee078, SUC_lexicon_tags_15 },
    { 0x1c2ee079, SUC_lexicon_tags_7 },
    { 0xb54ee079, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eee082, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eeee08b, SUC_lexicon_tags_54 },
    { 0x97eee08c, SUC_lexicon_tags_2 },
    { 0xf0aee08d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eee092, SUC_lexicon_tags_0 },
    { 0x768ee092, SUC_lexicon_tags_112 },
    { 0x556ee094, SUC_lexicon_tags_23 },
    { 0xc3eee094, SUC_lexicon_tags_15 },
    { 0xd74ee096, SUC_lexicon_tags_25 },
    { 0x638ee092, SUC_lexicon_tags_11 },
    { 0xa22ee093, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1eee09a, SUC_lexicon_tags_39 },
    { 0x68aee09b, SUC_lexicon_tags_0 },
    { 0xdbcee09c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4ee0a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeeee0a5, SUC_lexicon_tags_14 },
    { 0x52eee0a6, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50eee0ab, SUC_lexicon_tags_38 },
    { 0x2e0ee0ac, SUC_lexicon_tags_2 },
    { 0xc18ee0ab, SUC_lexicon_tags_2 },
    { 0xd9aee0ae, SUC_lexicon_tags_79 },
    { 0xaf6ee0ab, SUC_lexicon_tags_68 },
    { 0x666ee0b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa86ee0b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xceaee0b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588ee0bf, SUC_lexicon_tags_60 },
    { 0x4f0ee0bf, SUC_lexicon_tags_2 },
    { 0xf9eee0c1, SUC_lexicon_tags_11 },
    { 0x986ee0c0, SUC_lexicon_tags_38 },
    { 0x9acee0c3, SUC_lexicon_tags_38 },
    { 0x4ceee0c4, SUC_lexicon_tags_6 },
    { 0xfdeee0c5, SUC_lexicon_tags_39 },
    { 0x872ee0c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16ee0cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b8ee0ce, SUC_lexicon_tags_33 },
    { 0x552ee0ce, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6ee0d7, SUC_lexicon_tags_17 },
    { 0xe60ee0d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeee0db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cee0de, SUC_lexicon_tags_33 },
    { 0x2e6ee0de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc0ee0e1, SUC_lexicon_tags_26 },
    { 0x136ee0e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35eee0e6, SUC_lexicon_tags_30 },
    { 0xba0ee0e6, SUC_lexicon_tags_0 },
    { 0x1a4ee0e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedaee0ef, SUC_lexicon_tags_33 },
    { 0x29eee0f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe98ee0f2, SUC_lexicon_tags_27 },
    { 0x9ecee0f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18ee0f8, SUC_lexicon_tags_63 },
    { 0x362ee0f8, SUC_lexicon_tags_30 },
    { 0x8dcee0fa, SUC_lexicon_tags_15 },
    { 0xacee0fb, SUC_lexicon_tags_30 },
    { 0xd1eee0fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54aee0ff, SUC_lexicon_tags_418 },
    { 0xfcee100, SUC_lexicon_tags_6 },
    { 0x142ee100, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ee104, SUC_lexicon_tags_63 },
    { 0x58ee105, SUC_lexicon_tags_11 },
    { 0x50eee106, SUC_lexicon_tags_23 },
    { 0xe02ee107, SUC_lexicon_tags_2 },
    { 0xfccee107, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4aee10a, SUC_lexicon_tags_15 },
    { 0x92ee10b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0eee10f, SUC_lexicon_tags_11 },
    { 0x4f6ee110, SUC_lexicon_tags_47 },
    { 0x78ee111, SUC_lexicon_tags_0 },
    { 0xea8ee111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaee118, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8ee11e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754ee122, SUC_lexicon_tags_12 },
    { 0xac2ee123, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbe8ee125, SUC_lexicon_tags_25 },
    { 0x1e2ee126, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x188ee129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ee12d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6acee12f, SUC_lexicon_tags_42 },
    { 0x3f0ee130, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf98ee132, SUC_lexicon_tags_49 },
    { 0xdfeee133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8ee139, SUC_lexicon_tags_30 },
    { 0xb84ee13a, SUC_lexicon_tags_25 },
    { 0xbacee13a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd6ee13d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80eee13f, SUC_lexicon_tags_37 },
    { 0x470ee13f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x80ee142, SUC_lexicon_tags_4 },
    { 0x2daee142, SUC_lexicon_tags_37 },
    { 0x358ee144, SUC_lexicon_tags_11 },
    { 0x4f6ee145, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10cee14e, SUC_lexicon_tags_30 },
    { 0xeee14f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x89aee151, SUC_lexicon_tags_37 },
    { 0xe20ee152, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff6ee154, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298ee158, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x176ee15b, SUC_lexicon_tags_25 },
    { 0x306ee15b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3eee15f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4daee162, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32ee165, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecee16d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfeee16f, SUC_lexicon_tags_2 },
    { 0x57cee16f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58ee174, SUC_lexicon_tags_38 },
    { 0xe8cee175, SUC_lexicon_tags_30 },
    { 0xbecee174, SUC_lexicon_tags_2 },
    { 0xadaee177, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x152ee179, SUC_lexicon_tags_17 },
    { 0x7c4ee17a, SUC_lexicon_tags_37 },
    { 0xf92ee17b, SUC_lexicon_tags_32 },
    { 0x2cee17a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcaaee17e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfdcee180, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ee185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x696ee187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x894ee189, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x274ee18b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8ee192, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4e2ee194, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976ee19b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb84ee19d, SUC_lexicon_tags_17 },
    { 0x590ee19e, SUC_lexicon_tags_60 },
    { 0x8eeee19f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61eee1a6, SUC_lexicon_tags_60 },
    { 0xd08ee1a7, SUC_lexicon_tags_67 },
    { 0xb8aee1a6, SUC_lexicon_tags_60 },
    { 0x26aee1a9, SUC_lexicon_tags_151 },
    { 0x206ee1aa, SUC_lexicon_tags_12 },
    { 0x466ee1a7, SUC_lexicon_tags_15 },
    { 0x838ee1ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ee1af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0ee1b1, SUC_lexicon_tags_17 },
    { 0xb16ee1b1, SUC_lexicon_tags_23 },
    { 0xf80ee1b1, SUC_lexicon_tags_19 },
    { 0x9ecee1b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdeee1b8, SUC_lexicon_tags_2 },
    { 0x202ee1b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4ee1bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecee1bf, SUC_lexicon_tags_17 },
    { 0x5ccee1bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6ee1c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2feee1c7, SUC_lexicon_tags_37 },
    { 0xeb6ee1c7, SUC_lexicon_tags_29 },
    { 0x6a6ee1c9, SUC_lexicon_tags_15 },
    { 0x28aee1c8, SUC_lexicon_tags_2 },
    { 0x33eee1ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x326ee1cd, SUC_lexicon_tags_34 },
    { 0x71aee1ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146ee1d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd8aee1d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x44eee1d8, SUC_lexicon_tags_13 },
    { 0xca8ee1d8, SUC_lexicon_tags_15 },
    { 0x55eee1da, SUC_lexicon_tags_63 },
    { 0xcbcee1da, SUC_lexicon_tags_40 },
    { 0xbe8ee1dc, SUC_lexicon_tags_37 },
    { 0x680ee1dd, SUC_lexicon_tags_15 },
    { 0x14ee1dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730ee1e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6caee1e5, SUC_lexicon_tags_48 },
    { 0xe3aee1e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2ee1ee, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5b6ee1f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccee1f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x400ee1f9, SUC_lexicon_tags_261 },
    { 0xaa4ee1f9, SUC_lexicon_tags_8 },
    { 0xc12ee1fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64cee1fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6aee201, SUC_lexicon_tags_2 },
    { 0x6fcee201, SUC_lexicon_tags_9 },
    { 0x90ee203, SUC_lexicon_tags_28 },
    { 0xe8eee203, SUC_lexicon_tags_11 },
    { 0xcd6ee202, SUC_lexicon_tags_23 },
    { 0xc72ee206, SUC_lexicon_tags_35 },
    { 0x34aee207, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x49eee209, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14eee20e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6ee212, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64ee215, SUC_lexicon_tags_68 },
    { 0xaecee215, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x286ee219, SUC_lexicon_tags_11 },
    { 0xd8cee21a, SUC_lexicon_tags_13 },
    { 0x238ee21b, SUC_lexicon_tags_15 },
    { 0xc6aee21c, SUC_lexicon_tags_2 },
    { 0x684ee21a, SUC_lexicon_tags_25 },
    { 0x4b6ee21c, SUC_lexicon_tags_16 },
    { 0x9eee21f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe64ee222, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcee226, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8ee230, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472ee234, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x316ee237, SUC_lexicon_tags_7 },
    { 0x446ee238, SUC_lexicon_tags_22 },
    { 0x8c8ee237, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeceee23c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cee23f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cee243, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x510ee246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73aee250, SUC_lexicon_tags_8 },
    { 0x260ee250, SUC_lexicon_tags_33 },
    { 0xe52ee252, SUC_lexicon_tags_172 },
    { 0x73eee251, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd96ee255, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8faee257, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaee25e, SUC_lexicon_tags_21 },
    { 0xaf4ee25e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740ee266, SUC_lexicon_tags_37 },
    { 0xb72ee266, SUC_lexicon_tags_15 },
    { 0x500ee268, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420ee26c, SUC_lexicon_tags_60 },
    { 0x476ee26d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2ee273, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0ee27a, SUC_lexicon_tags_12 },
    { 0x152ee27b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4ee286, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6ee28c, SUC_lexicon_tags_37 },
    { 0xabcee28d, SUC_lexicon_tags_17 },
    { 0xfe0ee28d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0ee293, SUC_lexicon_tags_7 },
    { 0x3e8ee294, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92ee299, SUC_lexicon_tags_17 },
    { 0x354ee29a, SUC_lexicon_tags_27 },
    { 0x8d2ee29b, SUC_lexicon_tags_12 },
    { 0xd02ee29c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x246ee29e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2eee2a6, SUC_lexicon_tags_2 },
    { 0xb4eee2a7, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0ee2ad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x81eee2af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36cee2b2, SUC_lexicon_tags_7 },
    { 0x21aee2b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5beee2b6, SUC_lexicon_tags_12 },
    { 0x450ee2b7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xcf8ee2b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178ee2be, SUC_lexicon_tags_23 },
    { 0x572ee2bf, SUC_lexicon_tags_38 },
    { 0x8eaee2bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cee2c4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34ee2c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x848ee2c9, SUC_lexicon_tags_23 },
    { 0x3d0ee2c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38cee2cc, SUC_lexicon_tags_37 },
    { 0xf4ee2cd, SUC_lexicon_tags_15 },
    { 0xae0ee2cc, SUC_lexicon_tags_11 },
    { 0x2d4ee2ce, SUC_lexicon_tags_7 },
    { 0xe4eee2d0, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400ee2d9, SUC_lexicon_tags_7 },
    { 0x808ee2da, SUC_lexicon_tags_26 },
    { 0xc76ee2db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2ee2e0, SUC_lexicon_tags_29 },
    { 0x46cee2e1, SUC_lexicon_tags_60 },
    { 0x9eaee2e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x414ee2e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaee2e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67aee2ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92ee2ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x820ee2f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7faee2f4, SUC_lexicon_tags_9 },
    { 0xf0ee2f5, SUC_lexicon_tags_17 },
    { 0xc36ee2f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0eee2fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x306ee2ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12ee304, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24eee30b, SUC_lexicon_tags_30 },
    { 0x176ee30c, SUC_lexicon_tags_45 },
    { 0x46aee30b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2ee310, SUC_lexicon_tags_15 },
    { 0xfbcee310, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6ee318, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbceee31c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12ee325, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84eee32b, SUC_lexicon_tags_37 },
    { 0xb66ee32c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaee32f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeee333, SUC_lexicon_tags_9 },
    { 0x7eee333, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x24eee336, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x124ee338, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cee33c, SUC_lexicon_tags_8 },
    { 0x7e6ee33c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9baee349, SUC_lexicon_tags_0 },
    { 0xb5eee34a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd60ee34c, SUC_lexicon_tags_26 },
    { 0xae6ee34c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d4ee34f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaee352, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xd90ee354, SUC_lexicon_tags_17 },
    { 0xeb6ee355, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x972ee357, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31aee35b, SUC_lexicon_tags_5 },
    { 0x562ee35b, SUC_lexicon_tags_25 },
    { 0x490ee35c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x362ee35f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326ee364, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7eee367, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6ee36a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc4aee36c, SUC_lexicon_tags_27 },
    { 0xc12ee36d, SUC_lexicon_tags_2 },
    { 0xd14ee36e, SUC_lexicon_tags_0 },
    { 0x26cee36f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3aee375, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ceee37c, SUC_lexicon_tags_14 },
    { 0x60aee37d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92ee380, SUC_lexicon_tags_8 },
    { 0xb5cee381, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9eee383, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67eee387, SUC_lexicon_tags_14 },
    { 0x264ee388, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6ee38b, SUC_lexicon_tags_0 },
    { 0xaeeee38b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf84ee38e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e8ee390, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0ee395, SUC_lexicon_tags_12 },
    { 0x924ee395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28ee39e, SUC_lexicon_tags_49 },
    { 0xc6aee39f, SUC_lexicon_tags_47 },
    { 0xf52ee3a0, SUC_lexicon_tags_39 },
    { 0x5fcee3a1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcee3a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8ee3ab, SUC_lexicon_tags_1 },
    { 0xe86ee3ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748ee3b1, SUC_lexicon_tags_17 },
    { 0x804ee3b2, SUC_lexicon_tags_15 },
    { 0x198ee3b3, SUC_lexicon_tags_65 },
    { 0x48eee3b4, SUC_lexicon_tags_7 },
    { 0x790ee3b5, SUC_lexicon_tags_26 },
    { 0xd4aee3b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea6ee3b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6caee3ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaceee3bd, SUC_lexicon_tags_11 },
    { 0x890ee3bd, SUC_lexicon_tags_90 },
    { 0x6eeee3bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a2ee3c1, SUC_lexicon_tags_163 },
    { 0xb54ee3c2, SUC_lexicon_tags_7 },
    { 0xf54ee3c3, SUC_lexicon_tags_34 },
    { 0x9f6ee3c4, SUC_lexicon_tags_2 },
    { 0xd20ee3c5, SUC_lexicon_tags_49 },
    { 0x128ee3c5, SUC_lexicon_tags_7 },
    { 0x608ee3c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caee3cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x516ee3d2, SUC_lexicon_tags_11 },
    { 0x5e6ee3d3, SUC_lexicon_tags_17 },
    { 0xb28ee3d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x73eee3d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32aee3d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2ee3df, SUC_lexicon_tags_45 },
    { 0xcc0ee3df, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x53cee3e2, SUC_lexicon_tags_33 },
    { 0xfeeee3e2, SUC_lexicon_tags_35 },
    { 0xc60ee3e3, SUC_lexicon_tags_29 },
    { 0xb3cee3e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x756ee3e8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x32cee3ea, SUC_lexicon_tags_25 },
    { 0xf4cee3eb, SUC_lexicon_tags_63 },
    { 0xa8cee3ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa5aee3ee, SUC_lexicon_tags_16 },
    { 0xdb6ee3ef, SUC_lexicon_tags_47 },
    { 0xc9cee3f0, SUC_lexicon_tags_21 },
    { 0xccaee3ee, SUC_lexicon_tags_21 },
    { 0x6acee3f1, SUC_lexicon_tags_2 },
    { 0xa1cee3f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4ee3f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76ee3fc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66aee400, SUC_lexicon_tags_2 },
    { 0x6e6ee401, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x586ee403, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x458ee405, SUC_lexicon_tags_92 },
    { 0x66eee406, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ceee408, SUC_lexicon_tags_10 },
    { 0xb9eee409, SUC_lexicon_tags_27 },
    { 0x6ceee40a, SUC_lexicon_tags_12 },
    { 0x926ee40b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238ee411, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x5dcee413, SUC_lexicon_tags_44 },
    { 0x2e4ee413, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62ee41b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40ee41e, SUC_lexicon_tags_47 },
    { 0xe0eee41f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6ee422, SUC_lexicon_tags_25 },
    { 0x1d2ee423, SUC_lexicon_tags_8 },
    { 0xd9eee424, SUC_lexicon_tags_60 },
    { 0x49cee425, SUC_lexicon_tags_1 },
    { 0x170ee426, SUC_lexicon_tags_37 },
    { 0x4f2ee425, SUC_lexicon_tags_14 },
    { 0x3a8ee427, SUC_lexicon_tags_7 },
    { 0xf76ee429, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe7aee42b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71aee430, SUC_lexicon_tags_12 },
    { 0xcdcee431, SUC_lexicon_tags_37 },
    { 0x962ee432, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x49aee435, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee438, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x912ee43a, SUC_lexicon_tags_656 },
    { 0xb8ee43b, SUC_lexicon_tags_1 },
    { 0xaaeee43c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb06ee43e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cee442, SUC_lexicon_tags_104 },
    { 0xf04ee443, SUC_lexicon_tags_1 },
    { 0x2eaee444, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xaceee447, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eee44d, SUC_lexicon_tags_15 },
    { 0xd90ee44e, SUC_lexicon_tags_8 },
    { 0x16ee44f, SUC_lexicon_tags_14 },
    { 0x148ee44d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cee455, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x824ee457, SUC_lexicon_tags_17 },
    { 0x12aee458, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83aee45d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812ee463, SUC_lexicon_tags_1 },
    { 0x456ee464, SUC_lexicon_tags_25 },
    { 0x232ee464, SUC_lexicon_tags_11 },
    { 0x982ee464, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5a8ee468, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6ee46b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfaaee46d, SUC_lexicon_tags_11 },
    { 0xa60ee46e, SUC_lexicon_tags_11 },
    { 0xde8ee46f, SUC_lexicon_tags_7 },
    { 0x5ccee46d, SUC_lexicon_tags_34 },
    { 0xefeee46d, SUC_lexicon_tags_57 },
    { 0xb14ee470, SUC_lexicon_tags_37 },
    { 0x878ee470, SUC_lexicon_tags_25 },
    { 0x56ee473, SUC_lexicon_tags_47 },
    { 0xbeee475, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3c0ee477, SUC_lexicon_tags_17 },
    { 0x678ee477, SUC_lexicon_tags_8 },
    { 0x838ee479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf20ee47b, SUC_lexicon_tags_15 },
    { 0x82ee47c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x202ee47f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0ee48e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910ee493, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ee496, SUC_lexicon_tags_37 },
    { 0x900ee497, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78ee49f, SUC_lexicon_tags_2 },
    { 0x618ee4a0, SUC_lexicon_tags_37 },
    { 0x610ee4a1, SUC_lexicon_tags_14 },
    { 0xfdeee4a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xecaee4a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x828ee4a8, SUC_lexicon_tags_188 },
    { 0x1a0ee4a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1eee4ab, SUC_lexicon_tags_55 },
    { 0xc84ee4ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb60ee4ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cee4b1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x994ee4b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x47cee4b6, SUC_lexicon_tags_19 },
    { 0x9d0ee4b6, SUC_lexicon_tags_39 },
    { 0x952ee4b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cee4bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x800ee4bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2ee4c2, SUC_lexicon_tags_17 },
    { 0xfcaee4c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x224ee4c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2ee4c9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cee4cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaee4d5, SUC_lexicon_tags_25 },
    { 0xb06ee4d6, SUC_lexicon_tags_12 },
    { 0xf26ee4d6, SUC_lexicon_tags_54 },
    { 0xc4ee4d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48ee4db, SUC_lexicon_tags_11 },
    { 0xe0aee4dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44ee4e3, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9aee4e6, SUC_lexicon_tags_7 },
    { 0x718ee4e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75aee4ea, SUC_lexicon_tags_2 },
    { 0xc34ee4eb, SUC_lexicon_tags_30 },
    { 0xc34ee4ec, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0x2eaee4ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ee4f1, SUC_lexicon_tags_17 },
    { 0xa9eee4f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4ee4fd, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458ee503, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6ee50d, SUC_lexicon_tags_28 },
    { 0x416ee50e, SUC_lexicon_tags_8 },
    { 0xbeaee50e, SUC_lexicon_tags_26 },
    { 0xdaee50f, SUC_lexicon_tags_58 },
    { 0xd1aee50e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa16ee513, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82ee518, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166ee51f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78aee523, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25eee527, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4ee52e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb00ee530, SUC_lexicon_tags_17 },
    { 0x5a2ee530, SUC_lexicon_tags_33 },
    { 0x29eee532, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x70eee535, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a2ee537, SUC_lexicon_tags_6 },
    { 0x308ee538, SUC_lexicon_tags_7 },
    { 0x5b4ee538, SUC_lexicon_tags_11 },
    { 0x2d4ee53a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x666ee53c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x978ee53e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcee544, SUC_lexicon_tags_25 },
    { 0xdacee544, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb9aee547, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xee54a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0ee553, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ee559, SUC_lexicon_tags_12 },
    { 0xa2eee559, SUC_lexicon_tags_12 },
    { 0xf84ee559, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdeaee55d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ee561, SUC_lexicon_tags_2 },
    { 0xa62ee562, SUC_lexicon_tags_8 },
    { 0x8f2ee563, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9aee56c, SUC_lexicon_tags_17 },
    { 0xbd2ee56d, SUC_lexicon_tags_15 },
    { 0x6acee56e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4ee570, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88aee575, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceee578, SUC_lexicon_tags_66 },
    { 0xa6aee579, SUC_lexicon_tags_60 },
    { 0x38cee57a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974ee584, SUC_lexicon_tags_12 },
    { 0x18eee585, SUC_lexicon_tags_63 },
    { 0xcfaee586, SUC_lexicon_tags_38 },
    { 0x822ee586, SUC_lexicon_tags_43 },
    { 0xd04ee586, SUC_lexicon_tags_46 },
    { 0xaecee589, SUC_lexicon_tags_34 },
    { 0x908ee585, SUC_lexicon_tags_7 },
    { 0xa1cee58b, SUC_lexicon_tags_37 },
    { 0x43aee587, SUC_lexicon_tags_63 },
    { 0xf24ee58d, SUC_lexicon_tags_37 },
    { 0xbaee58e, SUC_lexicon_tags_8 },
    { 0x99eee58e, SUC_lexicon_tags_1 },
    { 0x94cee590, SUC_lexicon_tags_2 },
    { 0xfd8ee58e, SUC_lexicon_tags_2 },
    { 0xf4ee58c, SUC_lexicon_tags_43 },
    { 0x800ee593, SUC_lexicon_tags_229 },
    { 0x88cee594, SUC_lexicon_tags_17 },
    { 0x530ee594, SUC_lexicon_tags_17 },
    { 0x72ee594, SUC_lexicon_tags_39 },
    { 0x6e2ee597, SUC_lexicon_tags_26 },
    { 0x984ee585, SUC_lexicon_tags_87 },
    { 0x968ee599, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12cee59b, SUC_lexicon_tags_7 },
    { 0x5d2ee59c, SUC_lexicon_tags_15 },
    { 0x678ee59d, SUC_lexicon_tags_37 },
    { 0x218ee59c, SUC_lexicon_tags_12 },
    { 0xbfaee59f, SUC_lexicon_tags_26 },
    { 0x3cee5a0, SUC_lexicon_tags_37 },
    { 0x74ee59f, SUC_lexicon_tags_154 },
    { 0x1c8ee59e, SUC_lexicon_tags_31 },
    { 0x23cee5a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd8ee5a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ee5a9, SUC_lexicon_tags_33 },
    { 0x212ee5aa, SUC_lexicon_tags_1 },
    { 0x636ee5ab, SUC_lexicon_tags_6 },
    { 0x546ee5ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ee5b1, SUC_lexicon_tags_15 },
    { 0xcaaee5b1, SUC_lexicon_tags_15 },
    { 0xf42ee5b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2b0ee5b5, SUC_lexicon_tags_17 },
    { 0x166ee5b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7ecee5b8, SUC_lexicon_tags_1 },
    { 0x52cee5b8, SUC_lexicon_tags_25 },
    { 0xc20ee5ba, SUC_lexicon_tags_17 },
    { 0x512ee5ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x22aee5be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ee5c2, SUC_lexicon_tags_11 },
    { 0x8d0ee5c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1cee5c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4ee5c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aee5cb, SUC_lexicon_tags_6 },
    { 0x97eee5cb, SUC_lexicon_tags_65 },
    { 0xe66ee5cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x740ee5d0, SUC_lexicon_tags_19 },
    { 0x41eee5d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x68cee5d3, SUC_lexicon_tags_33 },
    { 0xdaeee5d3, SUC_lexicon_tags_25 },
    { 0xc46ee5d4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176ee5df, SUC_lexicon_tags_17 },
    { 0x2ecee5e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86ee5e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cee5ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4ee5ed, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xae0ee5f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd42ee5f2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16ee5f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280ee5fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46ee602, SUC_lexicon_tags_19 },
    { 0x1beee602, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638ee607, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48ee60b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12eee611, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eee617, SUC_lexicon_tags_17 },
    { 0xd60ee618, SUC_lexicon_tags_26 },
    { 0xd78ee619, SUC_lexicon_tags_233 },
    { 0x16cee617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcee61e, SUC_lexicon_tags_75 },
    { 0x720ee61f, SUC_lexicon_tags_1 },
    { 0xf3eee620, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22ee623, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6ee626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9daee628, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cee62c, SUC_lexicon_tags_7 },
    { 0x16aee62d, SUC_lexicon_tags_46 },
    { 0xf6ee62d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62ee631, SUC_lexicon_tags_11 },
    { 0x28cee631, SUC_lexicon_tags_6 },
    { 0xf52ee631, SUC_lexicon_tags_40 },
    { 0x966ee634, SUC_lexicon_tags_2 },
    { 0x6f6ee635, SUC_lexicon_tags_1 },
    { 0x7cee635, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106ee63b, SUC_lexicon_tags_29 },
    { 0xf06ee63b, SUC_lexicon_tags_9 },
    { 0xbb6ee63d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcee640, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9baee645, SUC_lexicon_tags_12 },
    { 0xa54ee646, SUC_lexicon_tags_7 },
    { 0xcc2ee645, SUC_lexicon_tags_12 },
    { 0xf8aee648, SUC_lexicon_tags_1 },
    { 0x428ee648, SUC_lexicon_tags_12 },
    { 0x12cee646, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x234ee64c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1b4ee64e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x808ee650, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0ee654, SUC_lexicon_tags_8 },
    { 0xbd8ee654, SUC_lexicon_tags_10 },
    { 0x408ee656, SUC_lexicon_tags_23 },
    { 0x938ee657, SUC_lexicon_tags_37 },
    { 0x33aee658, SUC_lexicon_tags_2 },
    { 0x92cee656, SUC_lexicon_tags_63 },
    { 0xb02ee658, SUC_lexicon_tags_37 },
    { 0xc1cee65b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6ee65f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7aeee661, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb22ee663, SUC_lexicon_tags_12 },
    { 0xef4ee663, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91aee666, SUC_lexicon_tags_1 },
    { 0xa4ee667, SUC_lexicon_tags_15 },
    { 0xe2eee667, SUC_lexicon_tags_12 },
    { 0x76eee669, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacee66e, SUC_lexicon_tags_11 },
    { 0xa74ee66f, SUC_lexicon_tags_25 },
    { 0xf92ee66f, SUC_lexicon_tags_25 },
    { 0x1cee66f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3baee676, SUC_lexicon_tags_66 },
    { 0xd50ee677, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69eee679, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88ee67c, SUC_lexicon_tags_12 },
    { 0xa0eee67d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccee684, SUC_lexicon_tags_37 },
    { 0xa5aee685, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160ee68b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x460ee68e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4ee691, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58ee698, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x860ee69a, SUC_lexicon_tags_32 },
    { 0x8a6ee69a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ee6a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x34eee6a2, SUC_lexicon_tags_7 },
    { 0xfa8ee6a2, SUC_lexicon_tags_10 },
    { 0x46cee6a4, SUC_lexicon_tags_38 },
    { 0xac6ee6a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cee6aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7cee6ac, SUC_lexicon_tags_14 },
    { 0x850ee6ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf1cee6af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118ee6b3, SUC_lexicon_tags_37 },
    { 0xcb0ee6b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xadeee6b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1c2ee6b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34ee6bd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaee6c0, SUC_lexicon_tags_160 },
    { 0x802ee6c1, SUC_lexicon_tags_12 },
    { 0xacee6c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6ee6c4, SUC_lexicon_tags_17 },
    { 0x7d0ee6c4, SUC_lexicon_tags_47 },
    { 0xb24ee6c6, SUC_lexicon_tags_8 },
    { 0xd6aee6c6, SUC_lexicon_tags_138 },
    { 0x1c2ee6c8, SUC_lexicon_tags_33 },
    { 0xa4cee6c9, SUC_lexicon_tags_11 },
    { 0x280ee6c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x354ee6cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x556ee6cf, SUC_lexicon_tags_37 },
    { 0x6e8ee6d0, SUC_lexicon_tags_4 },
    { 0x862ee6d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc60ee6d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x468ee6d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x420ee6d7, SUC_lexicon_tags_0 },
    { 0x9d2ee6d8, SUC_lexicon_tags_48 },
    { 0x16cee6d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ee6dc, SUC_lexicon_tags_15 },
    { 0x140ee6dd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840ee6e1, SUC_lexicon_tags_7 },
    { 0x31aee6e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6ee6e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x414ee6e8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3c4ee6ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x44eee6ec, SUC_lexicon_tags_43 },
    { 0x5b4ee6ec, SUC_lexicon_tags_17 },
    { 0x794ee6ee, SUC_lexicon_tags_7 },
    { 0x2f0ee6ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6ee6f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcee6f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7faee6fc, SUC_lexicon_tags_25 },
    { 0xb60ee6fd, SUC_lexicon_tags_16 },
    { 0xd46ee6fe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb4aee700, SUC_lexicon_tags_66 },
    { 0x1a2ee701, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934ee707, SUC_lexicon_tags_28 },
    { 0x502ee707, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4daee70a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9beee70d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396ee711, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xbc6ee713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4ee717, SUC_lexicon_tags_21 },
    { 0xcc6ee717, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3f8ee71a, SUC_lexicon_tags_33 },
    { 0xcdaee71a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492ee720, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eeee724, SUC_lexicon_tags_27 },
    { 0x666ee725, SUC_lexicon_tags_55 },
    { 0xad8ee724, SUC_lexicon_tags_37 },
    { 0x22aee726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4beee729, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8ee72d, SUC_lexicon_tags_49 },
    { 0x88aee72d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6eee733, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4ceee735, SUC_lexicon_tags_2 },
    { 0xb6eee735, SUC_lexicon_tags_2 },
    { 0xdeee737, SUC_lexicon_tags_61 },
    { 0xd42ee737, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7feee73b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4ee73e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744ee743, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x78eee746, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36ee74b, SUC_lexicon_tags_1 },
    { 0xc7aee74b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x144ee74e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8ee753, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x64eee755, SUC_lexicon_tags_31 },
    { 0x810ee755, SUC_lexicon_tags_23 },
    { 0x3ccee757, SUC_lexicon_tags_18 },
    { 0x4e6ee758, SUC_lexicon_tags_37 },
    { 0xc4ee759, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31cee75b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x180ee75e, SUC_lexicon_tags_9 },
    { 0x88cee75f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdccee761, SUC_lexicon_tags_2 },
    { 0xa8aee762, SUC_lexicon_tags_4 },
    { 0x568ee763, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x632ee765, SUC_lexicon_tags_14 },
    { 0xac0ee765, SUC_lexicon_tags_10 },
    { 0xe2aee767, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc0ee769, SUC_lexicon_tags_7 },
    { 0x746ee76a, SUC_lexicon_tags_82 },
    { 0x7a0ee76a, SUC_lexicon_tags_25 },
    { 0x620ee76c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14ee76f, SUC_lexicon_tags_12 },
    { 0x99eee770, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x126ee773, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53cee775, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb66ee777, SUC_lexicon_tags_37 },
    { 0x5eeee777, SUC_lexicon_tags_0 },
    { 0xf1eee778, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x660ee77c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x712ee77e, SUC_lexicon_tags_47 },
    { 0x4eaee77f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeeee786, SUC_lexicon_tags_11 },
    { 0xb7eee786, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2ee792, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeee796, SUC_lexicon_tags_11 },
    { 0x552ee797, SUC_lexicon_tags_4 },
    { 0x1daee796, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2ee79c, SUC_lexicon_tags_1 },
    { 0xb5eee79d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ee7a0, SUC_lexicon_tags_6 },
    { 0x8d2ee7a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x446ee7a3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66aee7a7, SUC_lexicon_tags_45 },
    { 0x77aee7a8, SUC_lexicon_tags_3 },
    { 0xe0ee7a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a0ee7ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44eee7ad, SUC_lexicon_tags_17 },
    { 0x950ee7ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x182ee7b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xddaee7b3, SUC_lexicon_tags_6 },
    { 0x76ee7b4, SUC_lexicon_tags_27 },
    { 0x778ee7b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x488ee7b8, SUC_lexicon_tags_45 },
    { 0x140ee7b9, SUC_lexicon_tags_37 },
    { 0xd06ee7b9, SUC_lexicon_tags_87 },
    { 0x552ee7bb, SUC_lexicon_tags_55 },
    { 0xd94ee7bc, SUC_lexicon_tags_49 },
    { 0x5dcee7b8, SUC_lexicon_tags_30 },
    { 0xd34ee7bc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeee7c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfc2ee7c3, SUC_lexicon_tags_9 },
    { 0x1e4ee7c4, SUC_lexicon_tags_11 },
    { 0x422ee7c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442ee7ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0ee7d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788ee7db, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572ee7e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6ee7f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34ee7f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcee7f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0ee7fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1aee7ff, SUC_lexicon_tags_27 },
    { 0x55eee800, SUC_lexicon_tags_8 },
    { 0xc02ee800, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4ee806, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0ee809, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f6ee80b, SUC_lexicon_tags_27 },
    { 0x6c4ee80c, SUC_lexicon_tags_23 },
    { 0x21cee80c, SUC_lexicon_tags_7 },
    { 0x15aee80b, SUC_lexicon_tags_17 },
    { 0xe16ee80f, SUC_lexicon_tags_60 },
    { 0x106ee810, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x98eee813, SUC_lexicon_tags_23 },
    { 0xbceee813, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf0aee816, SUC_lexicon_tags_2 },
    { 0x41eee817, SUC_lexicon_tags_77 },
    { 0xbc8ee818, SUC_lexicon_tags_35 },
    { 0xe78ee819, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x31cee81b, SUC_lexicon_tags_7 },
    { 0xb9aee81c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ee81f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd2ee821, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f0ee823, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5baee825, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52ee82a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82aee82f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4ee835, SUC_lexicon_tags_17 },
    { 0xfc6ee836, SUC_lexicon_tags_84 },
    { 0x460ee837, SUC_lexicon_tags_37 },
    { 0x22ee838, SUC_lexicon_tags_37 },
    { 0x288ee837, SUC_lexicon_tags_8 },
    { 0x168ee839, SUC_lexicon_tags_64 },
    { 0x28ee83b, SUC_lexicon_tags_14 },
    { 0x56ee83b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8b4ee83e, SUC_lexicon_tags_2 },
    { 0x52eee83f, SUC_lexicon_tags_65 },
    { 0x7b4ee840, SUC_lexicon_tags_87 },
    { 0x4b2ee841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc84ee843, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b0ee845, SUC_lexicon_tags_39 },
    { 0x67eee846, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2ee84a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebcee84c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe30ee84e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31eee851, SUC_lexicon_tags_15 },
    { 0x72ee852, SUC_lexicon_tags_7 },
    { 0xd3eee851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48ee85c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cee860, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eee863, SUC_lexicon_tags_37 },
    { 0xa0ee864, SUC_lexicon_tags_49 },
    { 0xaf4ee865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62eee868, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xc86ee86a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8beee86e, SUC_lexicon_tags_27 },
    { 0x938ee86e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaecee871, SUC_lexicon_tags_60 },
    { 0xa6eee871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcee878, SUC_lexicon_tags_45 },
    { 0x9aaee879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aee87d, SUC_lexicon_tags_23 },
    { 0xa46ee87e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x864ee880, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceee886, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaee889, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1daee88b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48ee88e, SUC_lexicon_tags_37 },
    { 0x9f4ee88f, SUC_lexicon_tags_17 },
    { 0x42ee890, SUC_lexicon_tags_17 },
    { 0xea6ee891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x790ee894, SUC_lexicon_tags_86 },
    { 0xdc2ee895, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x252ee898, SUC_lexicon_tags_63 },
    { 0xa88ee899, SUC_lexicon_tags_47 },
    { 0xceeee89a, SUC_lexicon_tags_11 },
    { 0xc9aee899, SUC_lexicon_tags_29 },
    { 0x438ee89c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x790ee89e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56aee8a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0ee8a5, SUC_lexicon_tags_45 },
    { 0x71aee8a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84ee8aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ee8ae, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaee8b2, SUC_lexicon_tags_17 },
    { 0x5deee8b2, SUC_lexicon_tags_17 },
    { 0x26ee8b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x890ee8b7, SUC_lexicon_tags_47 },
    { 0x26cee8b8, SUC_lexicon_tags_60 },
    { 0x6e4ee8b9, SUC_lexicon_tags_17 },
    { 0xa68ee8b8, SUC_lexicon_tags_27 },
    { 0xc24ee8bb, SUC_lexicon_tags_37 },
    { 0xfbcee8bc, SUC_lexicon_tags_0 },
    { 0xf64ee8bd, SUC_lexicon_tags_11 },
    { 0x1ceee8ba, SUC_lexicon_tags_11 },
    { 0x222ee8bf, SUC_lexicon_tags_6 },
    { 0xa28ee8b9, SUC_lexicon_tags_14 },
    { 0xc3eee8bf, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcccee8d0, SUC_lexicon_tags_2 },
    { 0x24cee8d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xecee8d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0ee8d6, SUC_lexicon_tags_1 },
    { 0x2eaee8d7, SUC_lexicon_tags_43 },
    { 0xa68ee8d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x10aee8db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc50ee8dd, SUC_lexicon_tags_10 },
    { 0xd10ee8de, SUC_lexicon_tags_12 },
    { 0x234ee8de, SUC_lexicon_tags_23 },
    { 0x698ee8df, SUC_lexicon_tags_6 },
    { 0x6d0ee8de, SUC_lexicon_tags_12 },
    { 0xc56ee8e2, SUC_lexicon_tags_33 },
    { 0x186ee8e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eee8e8, SUC_lexicon_tags_39 },
    { 0x35cee8e8, SUC_lexicon_tags_1 },
    { 0x718ee8ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x108ee8ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cee8ef, SUC_lexicon_tags_32 },
    { 0x9fcee8f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x658ee8f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x506ee8f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x29eee8f8, SUC_lexicon_tags_11 },
    { 0xee2ee8f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9eee8fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932ee906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d4ee908, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaaee90a, SUC_lexicon_tags_35 },
    { 0x28cee90b, SUC_lexicon_tags_6 },
    { 0x49aee90c, SUC_lexicon_tags_34 },
    { 0x35cee90d, SUC_lexicon_tags_45 },
    { 0xb64ee90e, SUC_lexicon_tags_25 },
    { 0xd2aee90d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ceee913, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0ee916, SUC_lexicon_tags_34 },
    { 0x7f4ee917, SUC_lexicon_tags_32 },
    { 0x406ee918, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738ee91c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcee91f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceeee923, SUC_lexicon_tags_30 },
    { 0xae8ee923, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73aee928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73eee92d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120ee933, SUC_lexicon_tags_6 },
    { 0xb3aee933, SUC_lexicon_tags_25 },
    { 0x70aee935, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x550ee937, SUC_lexicon_tags_16 },
    { 0x972ee938, SUC_lexicon_tags_19 },
    { 0xb26ee937, SUC_lexicon_tags_1 },
    { 0xeccee938, SUC_lexicon_tags_0 },
    { 0xd54ee93b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84ee940, SUC_lexicon_tags_10 },
    { 0x7a4ee941, SUC_lexicon_tags_17 },
    { 0x3c6ee940, SUC_lexicon_tags_25 },
    { 0x66cee943, SUC_lexicon_tags_32 },
    { 0xec8ee943, SUC_lexicon_tags_6 },
    { 0xf46ee943, SUC_lexicon_tags_91 },
    { 0x4b0ee945, SUC_lexicon_tags_12 },
    { 0x392ee941, SUC_lexicon_tags_16 },
    { 0xf0aee947, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66ee94b, SUC_lexicon_tags_47 },
    { 0xc4eee94c, SUC_lexicon_tags_66 },
    { 0x9e6ee94d, SUC_lexicon_tags_15 },
    { 0x3acee94d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x75eee951, SUC_lexicon_tags_25 },
    { 0xa54ee951, SUC_lexicon_tags_25 },
    { 0xb68ee952, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2ee958, SUC_lexicon_tags_38 },
    { 0x6ecee959, SUC_lexicon_tags_43 },
    { 0x940ee959, SUC_lexicon_tags_22 },
    { 0x2f0ee95b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa4ee95d, SUC_lexicon_tags_17 },
    { 0x7b4ee95e, SUC_lexicon_tags_30 },
    { 0x76cee95f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ee965, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x480ee967, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a0ee969, SUC_lexicon_tags_2 },
    { 0x560ee96a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeee971, SUC_lexicon_tags_108 },
    { 0x9eee972, SUC_lexicon_tags_7 },
    { 0xf86ee973, SUC_lexicon_tags_12 },
    { 0x6feee973, SUC_lexicon_tags_12 },
    { 0x1f2ee971, SUC_lexicon_tags_25 },
    { 0xfb0ee975, SUC_lexicon_tags_0 },
    { 0xfb8ee971, SUC_lexicon_tags_29 },
    { 0x5eaee978, SUC_lexicon_tags_7 },
    { 0xbf6ee979, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd20ee97b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaee97f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xadaee981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cee986, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1aaee988, SUC_lexicon_tags_38 },
    { 0xcb8ee989, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0ee98c, SUC_lexicon_tags_47 },
    { 0x738ee98c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96ee996, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15eee99b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x566ee99e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4ee9a6, SUC_lexicon_tags_47 },
    { 0xd04ee9a7, SUC_lexicon_tags_2 },
    { 0xf10ee9a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aee9ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7daee9af, SUC_lexicon_tags_21 },
    { 0x6dcee9af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb96ee9b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04ee9b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x710ee9b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdccee9bb, SUC_lexicon_tags_15 },
    { 0xb2aee9bc, SUC_lexicon_tags_12 },
    { 0x988ee9bd, SUC_lexicon_tags_67 },
    { 0x6a0ee9be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x306ee9c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84aee9c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acee9cb, SUC_lexicon_tags_29 },
    { 0x20aee9cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6eee9d1, SUC_lexicon_tags_4 },
    { 0x15cee9d2, SUC_lexicon_tags_25 },
    { 0xe02ee9d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212ee9d8, SUC_lexicon_tags_47 },
    { 0xbb2ee9d9, SUC_lexicon_tags_69 },
    { 0x2f0ee9d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4ee9dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76aee9e1, SUC_lexicon_tags_3 },
    { 0x5aee9e2, SUC_lexicon_tags_49 },
    { 0x96eee9e3, SUC_lexicon_tags_15 },
    { 0x446ee9e4, SUC_lexicon_tags_4 },
    { 0xba2ee9e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596ee9ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x850ee9f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeee9f4, SUC_lexicon_tags_1 },
    { 0xe7cee9f5, SUC_lexicon_tags_37 },
    { 0xa4ee9f6, SUC_lexicon_tags_1 },
    { 0xee0ee9f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x216ee9f9, SUC_lexicon_tags_25 },
    { 0xff4ee9f9, SUC_lexicon_tags_2 },
    { 0x45eee9fb, SUC_lexicon_tags_17 },
    { 0xb94ee9fc, SUC_lexicon_tags_11 },
    { 0xdbcee9fd, SUC_lexicon_tags_47 },
    { 0x77cee9fe, SUC_lexicon_tags_49 },
    { 0xbe6ee9ff, SUC_lexicon_tags_11 },
    { 0x8e0ee9fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52eeea04, SUC_lexicon_tags_37 },
    { 0x93ceea05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91eeea07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3aeeea09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x544eea0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4eea14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ceea23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8caeea25, SUC_lexicon_tags_2 },
    { 0x22eea25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8dceea28, SUC_lexicon_tags_25 },
    { 0x9a4eea28, SUC_lexicon_tags_1 },
    { 0x23eeea28, SUC_lexicon_tags_61 },
    { 0x1fceea2b, SUC_lexicon_tags_15 },
    { 0x96ceea2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7aceea2e, SUC_lexicon_tags_45 },
    { 0x636eea2f, SUC_lexicon_tags_23 },
    { 0x580eea30, SUC_lexicon_tags_0 },
    { 0xac0eea31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7d0eea33, SUC_lexicon_tags_124 },
    { 0xe2eeea34, SUC_lexicon_tags_27 },
    { 0xc6eeea35, SUC_lexicon_tags_495 },
    { 0, NULL },
    { 0xe02eea37, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9beeea3a, SUC_lexicon_tags_11 },
    { 0xda6eea3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2eea42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2d6eea44, SUC_lexicon_tags_29 },
    { 0xd8eeea45, SUC_lexicon_tags_7 },
    { 0xc88eea45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16aeea4c, SUC_lexicon_tags_17 },
    { 0x1feeea4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0eea51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x246eea54, SUC_lexicon_tags_57 },
    { 0xd54eea55, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8eea59, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0aeea5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88eea62, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6eea66, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14aeea6d, SUC_lexicon_tags_11 },
    { 0x390eea6e, SUC_lexicon_tags_1 },
    { 0x42eea6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaeea72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x49ceea74, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdc8eea76, SUC_lexicon_tags_38 },
    { 0xee4eea77, SUC_lexicon_tags_15 },
    { 0xa72eea78, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bceea7b, SUC_lexicon_tags_43 },
    { 0xfb0eea7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48eea81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7eeea84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8eea89, SUC_lexicon_tags_56 },
    { 0x400eea8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22eea8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x38eea8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b8eea91, SUC_lexicon_tags_37 },
    { 0x7eeea91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x14eeea94, SUC_lexicon_tags_7 },
    { 0x31aeea95, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2eea99, SUC_lexicon_tags_25 },
    { 0xc7eeea99, SUC_lexicon_tags_60 },
    { 0xc40eea99, SUC_lexicon_tags_14 },
    { 0x1a2eea9c, SUC_lexicon_tags_27 },
    { 0x4e6eea9d, SUC_lexicon_tags_15 },
    { 0xb94eea9d, SUC_lexicon_tags_10 },
    { 0xcbaeea9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2caeeaa4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xd74eeaa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfeeeaa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcceeeaaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbdceeaac, SUC_lexicon_tags_39 },
    { 0x6b8eeaad, SUC_lexicon_tags_37 },
    { 0xe6ceeaac, SUC_lexicon_tags_11 },
    { 0x148eeaaf, SUC_lexicon_tags_25 },
    { 0x5caeeab0, SUC_lexicon_tags_10 },
    { 0x226eeaae, SUC_lexicon_tags_8 },
    { 0xa76eeab2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ceeab9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868eeabd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ceeac0, SUC_lexicon_tags_15 },
    { 0xfd0eeac0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x850eeac3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafeeeac7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x782eeac9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0eeacd, SUC_lexicon_tags_31 },
    { 0xa74eeace, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8beeead2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x466eead5, SUC_lexicon_tags_17 },
    { 0xfc0eead6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4eeadb, SUC_lexicon_tags_27 },
    { 0x530eeadc, SUC_lexicon_tags_17 },
    { 0xf76eeadc, SUC_lexicon_tags_15 },
    { 0xf6ceeadc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd96eeae1, SUC_lexicon_tags_8 },
    { 0x3bceeae2, SUC_lexicon_tags_7 },
    { 0x45aeeae3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6eeae8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d4eeaea, SUC_lexicon_tags_15 },
    { 0xaceeaea, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936eeaf0, SUC_lexicon_tags_26 },
    { 0xbfceeaf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85eeeaf3, SUC_lexicon_tags_29 },
    { 0x996eeaf4, SUC_lexicon_tags_11 },
    { 0xbc4eeaf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x256eeaf7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeeeaf9, SUC_lexicon_tags_7 },
    { 0x15aeeafa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6eeaff, SUC_lexicon_tags_53 },
    { 0xc02eeaff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2eeb03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfd2eeb05, SUC_lexicon_tags_12 },
    { 0x64ceeb06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70eeb0b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x184eeb0e, SUC_lexicon_tags_25 },
    { 0x66eeb0e, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x24eeb11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12eeb14, SUC_lexicon_tags_15 },
    { 0x4e8eeb14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x890eeb18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd0eeb1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8eeb22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4eeb24, SUC_lexicon_tags_17 },
    { 0x46eeeb25, SUC_lexicon_tags_8 },
    { 0x8dceeb25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2beeeb2c, SUC_lexicon_tags_11 },
    { 0xb50eeb2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eeeb30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458eeb34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x308eeb37, SUC_lexicon_tags_9 },
    { 0xf38eeb38, SUC_lexicon_tags_14 },
    { 0xb20eeb38, SUC_lexicon_tags_37 },
    { 0x39eeeb3a, SUC_lexicon_tags_12 },
    { 0xd54eeb39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2eeb3f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc06eeb41, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x994eeb44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f6eeb46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81aeeb49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb68eeb4d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cceeb50, SUC_lexicon_tags_27 },
    { 0xc1ceeb51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2b6eeb53, SUC_lexicon_tags_25 },
    { 0x378eeb54, SUC_lexicon_tags_12 },
    { 0x332eeb55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48aeeb57, SUC_lexicon_tags_2 },
    { 0x74eeeb58, SUC_lexicon_tags_1 },
    { 0xe78eeb58, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x348eeb5c, SUC_lexicon_tags_63 },
    { 0x466eeb5c, SUC_lexicon_tags_1 },
    { 0xfb0eeb5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70eeb61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64eeb66, SUC_lexicon_tags_12 },
    { 0x5faeeb67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74aeeb69, SUC_lexicon_tags_15 },
    { 0x986eeb6a, SUC_lexicon_tags_7 },
    { 0x52eeeb6b, SUC_lexicon_tags_13 },
    { 0x3f4eeb6c, SUC_lexicon_tags_8 },
    { 0x964eeb6c, SUC_lexicon_tags_12 },
    { 0xdeceeb6c, SUC_lexicon_tags_61 },
    { 0x6e8eeb6f, SUC_lexicon_tags_66 },
    { 0xc3ceeb70, SUC_lexicon_tags_12 },
    { 0x95ceeb6a, SUC_lexicon_tags_25 },
    { 0x99ceeb71, SUC_lexicon_tags_0 },
    { 0xf16eeb73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaeeb75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74eeb7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca0eeb80, SUC_lexicon_tags_45 },
    { 0xfe4eeb81, SUC_lexicon_tags_17 },
    { 0xa22eeb82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaeeb85, SUC_lexicon_tags_2 },
    { 0x93ceeb85, SUC_lexicon_tags_8 },
    { 0x710eeb87, SUC_lexicon_tags_82 },
    { 0x818eeb88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322eeb8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ceeb93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb0eeb95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x912eeb97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62aeeb99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x580eeb9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaceeeb9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0eeba1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x248eeba4, SUC_lexicon_tags_37 },
    { 0x3f4eeba5, SUC_lexicon_tags_17 },
    { 0x7a2eeba6, SUC_lexicon_tags_63 },
    { 0x218eeba5, SUC_lexicon_tags_2 },
    { 0x688eeba8, SUC_lexicon_tags_12 },
    { 0xabceeba5, SUC_lexicon_tags_12 },
    { 0x40aeebaa, SUC_lexicon_tags_6 },
    { 0xb80eebab, SUC_lexicon_tags_19 },
    { 0xdd2eebac, SUC_lexicon_tags_4 },
    { 0x88aeebac, SUC_lexicon_tags_27 },
    { 0x328eebae, SUC_lexicon_tags_25 },
    { 0x304eebaf, SUC_lexicon_tags_49 },
    { 0x5cceeba7, SUC_lexicon_tags_25 },
    { 0x604eebb1, SUC_lexicon_tags_15 },
    { 0xeeeebb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4eebb7, SUC_lexicon_tags_62 },
    { 0xe50eebb8, SUC_lexicon_tags_21 },
    { 0xc06eebb9, SUC_lexicon_tags_31 },
    { 0x5c2eebba, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4eebbd, SUC_lexicon_tags_43 },
    { 0x65aeebbd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8eebc3, SUC_lexicon_tags_55 },
    { 0x1e0eebc4, SUC_lexicon_tags_2 },
    { 0xaeceebc5, SUC_lexicon_tags_25 },
    { 0x778eebc3, SUC_lexicon_tags_224 },
    { 0xe7ceebc4, SUC_lexicon_tags_1 },
    { 0xb3eeebc8, SUC_lexicon_tags_87 },
    { 0x4d8eebc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14eebdb, SUC_lexicon_tags_28 },
    { 0xfe8eebdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8eebdf, SUC_lexicon_tags_2 },
    { 0x968eebe0, SUC_lexicon_tags_12 },
    { 0x548eebe1, SUC_lexicon_tags_15 },
    { 0xb0eebe2, SUC_lexicon_tags_7 },
    { 0x558eebdf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x82aeebe6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62eeebea, SUC_lexicon_tags_60 },
    { 0xefaeebeb, SUC_lexicon_tags_126 },
    { 0xd1eeebec, SUC_lexicon_tags_3 },
    { 0x404eebed, SUC_lexicon_tags_45 },
    { 0x532eebeb, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xae0eebf0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28eebf5, SUC_lexicon_tags_14 },
    { 0xa28eebf6, SUC_lexicon_tags_141 },
    { 0x3e2eebf7, SUC_lexicon_tags_7 },
    { 0xcb0eebf6, SUC_lexicon_tags_17 },
    { 0xb8eebf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdaeebfb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852eebff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dceec0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aeec1a, SUC_lexicon_tags_1 },
    { 0x5b6eec1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceec23, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x14ceec25, SUC_lexicon_tags_25 },
    { 0xbe6eec25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0eec2c, SUC_lexicon_tags_11 },
    { 0x6d8eec2d, SUC_lexicon_tags_31 },
    { 0x790eec2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x204eec30, SUC_lexicon_tags_12 },
    { 0xdd6eec31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ceec3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x39aeec42, SUC_lexicon_tags_37 },
    { 0xa30eec43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530eec47, SUC_lexicon_tags_423 },
    { 0x404eec47, SUC_lexicon_tags_37 },
    { 0xd00eec49, SUC_lexicon_tags_25 },
    { 0x882eec4a, SUC_lexicon_tags_11 },
    { 0x1beeec4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefceec4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0eec52, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0eec56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a0eec58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4eec5d, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x312eec60, SUC_lexicon_tags_8 },
    { 0xa98eec61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14eec63, SUC_lexicon_tags_29 },
    { 0xbaeeec64, SUC_lexicon_tags_63 },
    { 0x420eec63, SUC_lexicon_tags_8 },
    { 0x342eec64, SUC_lexicon_tags_1 },
    { 0xbe0eec64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeeaeec69, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x804eec6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ceec72, SUC_lexicon_tags_68 },
    { 0xca8eec72, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0eec81, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x152eec83, SUC_lexicon_tags_38 },
    { 0x54aeec84, SUC_lexicon_tags_17 },
    { 0xc36eec83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4deeec8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36ceec8d, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0x58eec90, SUC_lexicon_tags_63 },
    { 0x452eec91, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee6eec93, SUC_lexicon_tags_9 },
    { 0xd72eec94, SUC_lexicon_tags_57 },
    { 0x576eec94, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78eec98, SUC_lexicon_tags_160 },
    { 0x552eec99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x86ceec9b, SUC_lexicon_tags_2 },
    { 0xadceec9c, SUC_lexicon_tags_60 },
    { 0x3eeec9b, SUC_lexicon_tags_17 },
    { 0x61eeec9e, SUC_lexicon_tags_30 },
    { 0x72eec9e, SUC_lexicon_tags_46 },
    { 0xaa0eeca0, SUC_lexicon_tags_12 },
    { 0x81eeeca1, SUC_lexicon_tags_17 },
    { 0xe92eeca2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51aeeca9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc74eecab, SUC_lexicon_tags_2 },
    { 0x386eecab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16ceecae, SUC_lexicon_tags_12 },
    { 0x700eecae, SUC_lexicon_tags_26 },
    { 0x5c4eecb0, SUC_lexicon_tags_13 },
    { 0x35ceecb1, SUC_lexicon_tags_26 },
    { 0x5f6eecb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2eecb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1c2eecb7, SUC_lexicon_tags_2 },
    { 0x1ceecb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55ceecba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba0eecbd, SUC_lexicon_tags_33 },
    { 0x25eeecbe, SUC_lexicon_tags_1 },
    { 0xe38eecbf, SUC_lexicon_tags_88 },
    { 0x3b2eecbf, SUC_lexicon_tags_49 },
    { 0x1d4eecbf, SUC_lexicon_tags_11 },
    { 0x430eecc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94eecc6, SUC_lexicon_tags_34 },
    { 0x8caeecc7, SUC_lexicon_tags_14 },
    { 0x3e4eecc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14ceecca, SUC_lexicon_tags_12 },
    { 0x67eeeccb, SUC_lexicon_tags_7 },
    { 0x706eeccb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x440eeccf, SUC_lexicon_tags_12 },
    { 0x32eeccf, SUC_lexicon_tags_31 },
    { 0xa32eeccf, SUC_lexicon_tags_0 },
    { 0xea6eecd2, SUC_lexicon_tags_12 },
    { 0x80eeecd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296eecd8, SUC_lexicon_tags_15 },
    { 0xa98eecd9, SUC_lexicon_tags_16 },
    { 0x61eeecda, SUC_lexicon_tags_43 },
    { 0x46ceecdb, SUC_lexicon_tags_38 },
    { 0x87eeecdb, SUC_lexicon_tags_43 },
    { 0x766eecdc, SUC_lexicon_tags_15 },
    { 0xb56eecde, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eeeece2, SUC_lexicon_tags_37 },
    { 0x610eece2, SUC_lexicon_tags_61 },
    { 0x966eece3, SUC_lexicon_tags_28 },
    { 0xd8ceece5, SUC_lexicon_tags_27 },
    { 0x754eece6, SUC_lexicon_tags_11 },
    { 0x95aeece5, SUC_lexicon_tags_7 },
    { 0xa44eece5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ceecf5, SUC_lexicon_tags_11 },
    { 0x33aeecf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x680eecf9, SUC_lexicon_tags_15 },
    { 0x294eecfa, SUC_lexicon_tags_66 },
    { 0xd0ceecfb, SUC_lexicon_tags_74 },
    { 0x254eecfc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80eed03, SUC_lexicon_tags_7 },
    { 0xcf2eed04, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10eed08, SUC_lexicon_tags_45 },
    { 0x166eed09, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28eed0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32eed15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700eed20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182eed28, SUC_lexicon_tags_17 },
    { 0x5f2eed29, SUC_lexicon_tags_38 },
    { 0x954eed2a, SUC_lexicon_tags_12 },
    { 0x888eed2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x410eed2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86eed36, SUC_lexicon_tags_25 },
    { 0xfdeeed37, SUC_lexicon_tags_7 },
    { 0x8f2eed38, SUC_lexicon_tags_1 },
    { 0x460eed38, SUC_lexicon_tags_25 },
    { 0x3bceed39, SUC_lexicon_tags_31 },
    { 0x2aaeed3b, SUC_lexicon_tags_234 },
    { 0x4c6eed3c, SUC_lexicon_tags_12 },
    { 0x93eeed3c, SUC_lexicon_tags_12 },
    { 0xc54eed36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eeed42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770eed49, SUC_lexicon_tags_14 },
    { 0x594eed4a, SUC_lexicon_tags_17 },
    { 0x94ceed4b, SUC_lexicon_tags_37 },
    { 0x534eed4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210eed51, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x768eed53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8eed59, SUC_lexicon_tags_2 },
    { 0xee6eed5a, SUC_lexicon_tags_37 },
    { 0x93ceed5b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0eed5f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84eed62, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230eed70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62aeed74, SUC_lexicon_tags_11 },
    { 0x34eed75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6a4eed77, SUC_lexicon_tags_12 },
    { 0xe08eed78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ceed83, SUC_lexicon_tags_37 },
    { 0x23ceed84, SUC_lexicon_tags_2 },
    { 0x498eed85, SUC_lexicon_tags_19 },
    { 0x2b2eed86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414eed8a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbf8eed8c, SUC_lexicon_tags_42 },
    { 0xbfceed8c, SUC_lexicon_tags_49 },
    { 0x5d8eed8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x632eed90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92eed94, SUC_lexicon_tags_68 },
    { 0xe04eed94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x304eed9b, SUC_lexicon_tags_11 },
    { 0xedceed9c, SUC_lexicon_tags_7 },
    { 0x9b4eed9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eeeda0, SUC_lexicon_tags_7 },
    { 0x344eeda0, SUC_lexicon_tags_15 },
    { 0xf1ceeda2, SUC_lexicon_tags_19 },
    { 0x92eeda3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6eedad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaeedb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aceedb4, SUC_lexicon_tags_21 },
    { 0xd2aeedb5, SUC_lexicon_tags_49 },
    { 0xb72eedb6, SUC_lexicon_tags_69 },
    { 0x9b4eedb7, SUC_lexicon_tags_11 },
    { 0x1b0eedb8, SUC_lexicon_tags_49 },
    { 0xc2aeedb9, SUC_lexicon_tags_65 },
    { 0xe0eeedba, SUC_lexicon_tags_32 },
    { 0xfceeedb6, SUC_lexicon_tags_2 },
    { 0xf2aeedbc, SUC_lexicon_tags_25 },
    { 0xda8eedb8, SUC_lexicon_tags_45 },
    { 0x134eedbb, SUC_lexicon_tags_8 },
    { 0x72eedbf, SUC_lexicon_tags_7 },
    { 0xa3aeedb9, SUC_lexicon_tags_60 },
    { 0xcdaeedc1, SUC_lexicon_tags_17 },
    { 0xb6eeedc1, SUC_lexicon_tags_38 },
    { 0x48eedbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ceedcd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdaceedcf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0eedd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceeedd5, SUC_lexicon_tags_31 },
    { 0x728eedd5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2eedda, SUC_lexicon_tags_30 },
    { 0xdf4eeddb, SUC_lexicon_tags_280 },
    { 0xf8eeddb, SUC_lexicon_tags_2 },
    { 0xb04eeddc, SUC_lexicon_tags_39 },
    { 0x380eeddd, SUC_lexicon_tags_14 },
    { 0xd82eeddf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ceede3, SUC_lexicon_tags_37 },
    { 0xc6eeede4, SUC_lexicon_tags_12 },
    { 0x102eede5, SUC_lexicon_tags_28 },
    { 0x2f8eede4, SUC_lexicon_tags_65 },
    { 0xb00eede5, SUC_lexicon_tags_33 },
    { 0x978eede8, SUC_lexicon_tags_17 },
    { 0x2e0eede4, SUC_lexicon_tags_33 },
    { 0xa72eedea, SUC_lexicon_tags_7 },
    { 0x686eedea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2ceeded, SUC_lexicon_tags_25 },
    { 0x470eedee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4eeedf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x442eedf2, SUC_lexicon_tags_11 },
    { 0x9b6eedf2, SUC_lexicon_tags_208 },
    { 0x85aeedf4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cceedf7, SUC_lexicon_tags_1 },
    { 0xcfaeedf8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ceedfb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30eedfe, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x2a4eee00, SUC_lexicon_tags_23 },
    { 0x174eee01, SUC_lexicon_tags_2 },
    { 0xca0eee00, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6eee07, SUC_lexicon_tags_43 },
    { 0x7a2eee08, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb02eee0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2eee0d, SUC_lexicon_tags_4 },
    { 0x1eceee0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x784eee10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20aeee13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474eee17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ceee1a, SUC_lexicon_tags_4 },
    { 0xdc6eee1b, SUC_lexicon_tags_38 },
    { 0xfe4eee1b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58eee23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ceee26, SUC_lexicon_tags_37 },
    { 0xf0aeee27, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72eee2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92eee2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aceee32, SUC_lexicon_tags_23 },
    { 0xaa6eee33, SUC_lexicon_tags_21 },
    { 0x46eeee32, SUC_lexicon_tags_2 },
    { 0x3deeee35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1c2eee37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34eee3b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce8eee3d, SUC_lexicon_tags_68 },
    { 0x4c8eee3e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06eee41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1aeee43, SUC_lexicon_tags_8 },
    { 0xd2eeee44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2deeee46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cceee48, SUC_lexicon_tags_81 },
    { 0x812eee49, SUC_lexicon_tags_12 },
    { 0x58aeee49, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xb2ceee4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2eee54, SUC_lexicon_tags_31 },
    { 0x21eeee54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeee5d, SUC_lexicon_tags_26 },
    { 0x4b0eee5e, SUC_lexicon_tags_2 },
    { 0x396eee5f, SUC_lexicon_tags_12 },
    { 0x4c6eee60, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14eee6c, SUC_lexicon_tags_37 },
    { 0x3a2eee6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeeee72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x380eee75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4eee78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aeee81, SUC_lexicon_tags_15 },
    { 0xfe8eee81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eeee87, SUC_lexicon_tags_37 },
    { 0xdfeeee88, SUC_lexicon_tags_91 },
    { 0x878eee87, SUC_lexicon_tags_16 },
    { 0x30eeee88, SUC_lexicon_tags_38 },
    { 0x818eee8b, SUC_lexicon_tags_27 },
    { 0xd9ceee8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64eee90, SUC_lexicon_tags_63 },
    { 0x8eeeee91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf56eee93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x634eee96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816eee9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xceeee9d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x57aeee9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ceeea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236eeea8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76eeeac, SUC_lexicon_tags_15 },
    { 0xe8eeeac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816eeeb2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eeeeb8, SUC_lexicon_tags_15 },
    { 0x684eeeb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eeeebe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1eeeec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18eeec7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844eeecb, SUC_lexicon_tags_2 },
    { 0xc5ceeecb, SUC_lexicon_tags_1 },
    { 0x10eeecd, SUC_lexicon_tags_17 },
    { 0x490eeece, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85aeeed1, SUC_lexicon_tags_17 },
    { 0xed0eeed2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8eeeed5, SUC_lexicon_tags_4 },
    { 0xe06eeed5, SUC_lexicon_tags_23 },
    { 0x59ceeed7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f6eeed9, SUC_lexicon_tags_25 },
    { 0x6eeeeeda, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aeeee0, SUC_lexicon_tags_43 },
    { 0x6bceeee0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54eeeee7, SUC_lexicon_tags_47 },
    { 0x2a8eeee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736eeeed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7d8eeeef, SUC_lexicon_tags_47 },
    { 0x456eeef0, SUC_lexicon_tags_33 },
    { 0xcaaeeef1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x966eeef3, SUC_lexicon_tags_28 },
    { 0x124eeef4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaeeef7, SUC_lexicon_tags_17 },
    { 0x180eeef7, SUC_lexicon_tags_7 },
    { 0x1c0eeef8, SUC_lexicon_tags_37 },
    { 0xbdceeefa, SUC_lexicon_tags_6 },
    { 0x4e0eeefb, SUC_lexicon_tags_2 },
    { 0x97ceeef8, SUC_lexicon_tags_63 },
    { 0x4a0eeefc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5baeef02, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdb4eef04, SUC_lexicon_tags_9 },
    { 0x440eef05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d4eef07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaeef0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ceef10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4eef13, SUC_lexicon_tags_124 },
    { 0x216eef14, SUC_lexicon_tags_63 },
    { 0xbf6eef15, SUC_lexicon_tags_47 },
    { 0xcaeeef16, SUC_lexicon_tags_15 },
    { 0xf80eef17, SUC_lexicon_tags_26 },
    { 0x270eef16, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3d4eef1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x540eef1c, SUC_lexicon_tags_61 },
    { 0xa12eef1c, SUC_lexicon_tags_12 },
    { 0x9bceef1e, SUC_lexicon_tags_55 },
    { 0x9d4eef1e, SUC_lexicon_tags_0 },
    { 0x98eeef1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30aeef22, SUC_lexicon_tags_15 },
    { 0x442eef22, SUC_lexicon_tags_1 },
    { 0xb70eef23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6deeef28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeef2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd48eef2d, SUC_lexicon_tags_68 },
    { 0x836eef2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x778eef31, SUC_lexicon_tags_16 },
    { 0xd32eef32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22eef36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30eef39, SUC_lexicon_tags_1 },
    { 0x24ceef39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa36eef3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2eef41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62eef44, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa32eef46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33aeef4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18eef51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938eef56, SUC_lexicon_tags_0 },
    { 0x958eef57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf54eef5a, SUC_lexicon_tags_38 },
    { 0xb96eef5a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2eef60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dceef62, SUC_lexicon_tags_63 },
    { 0x13aeef63, SUC_lexicon_tags_11 },
    { 0xaf2eef62, SUC_lexicon_tags_46 },
    { 0xa7ceef65, SUC_lexicon_tags_60 },
    { 0x896eef63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806eef6e, SUC_lexicon_tags_6 },
    { 0x95aeef6f, SUC_lexicon_tags_56 },
    { 0xe16eef70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x212eef72, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcbceef74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ceef79, SUC_lexicon_tags_23 },
    { 0x17eeef79, SUC_lexicon_tags_21 },
    { 0x9c8eef7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eeef7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34eef85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d8eef87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7eeef8b, SUC_lexicon_tags_26 },
    { 0xdaaeef8b, SUC_lexicon_tags_33 },
    { 0xc0aeef8b, SUC_lexicon_tags_9 },
    { 0x2a4eef8e, SUC_lexicon_tags_37 },
    { 0xa56eef8f, SUC_lexicon_tags_17 },
    { 0x4f0eef8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcceef98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17aeef9a, SUC_lexicon_tags_47 },
    { 0xc4eeef9b, SUC_lexicon_tags_19 },
    { 0x8e6eef9c, SUC_lexicon_tags_17 },
    { 0xf28eef9d, SUC_lexicon_tags_46 },
    { 0xc6ceef9b, SUC_lexicon_tags_37 },
    { 0x7a2eef9f, SUC_lexicon_tags_45 },
    { 0xb16eef9f, SUC_lexicon_tags_61 },
    { 0x942eefa0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe5aeefa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf84eefa5, SUC_lexicon_tags_17 },
    { 0x648eefa5, SUC_lexicon_tags_30 },
    { 0x5aeefa7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9eeefa9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad6eefab, SUC_lexicon_tags_17 },
    { 0xd5ceefab, SUC_lexicon_tags_15 },
    { 0x89ceefab, SUC_lexicon_tags_68 },
    { 0xa1eeefae, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78eefb6, SUC_lexicon_tags_63 },
    { 0xe34eefb6, SUC_lexicon_tags_37 },
    { 0x1b4eefb8, SUC_lexicon_tags_37 },
    { 0x4daeefb9, SUC_lexicon_tags_25 },
    { 0x250eefba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aeeefc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86eefca, SUC_lexicon_tags_33 },
    { 0xce2eefcb, SUC_lexicon_tags_14 },
    { 0x9e6eefcc, SUC_lexicon_tags_37 },
    { 0x60aeefcb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8aaeefcf, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceaeefd4, SUC_lexicon_tags_11 },
    { 0x134eefd5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8aeefdb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4aeefe2, SUC_lexicon_tags_17 },
    { 0xafeeefe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eeefe7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0eefea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd8eefec, SUC_lexicon_tags_65 },
    { 0xa3aeefed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfdeeefef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf10eeff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2eeff7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364eefff, SUC_lexicon_tags_55 },
    { 0x9bceefff, SUC_lexicon_tags_7 },
    { 0x3f4ef001, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296ef006, SUC_lexicon_tags_12 },
    { 0xd34ef007, SUC_lexicon_tags_9 },
    { 0xf00ef007, SUC_lexicon_tags_12 },
    { 0xc78ef009, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668ef010, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x644ef012, SUC_lexicon_tags_11 },
    { 0x45cef013, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x268ef016, SUC_lexicon_tags_7 },
    { 0xad6ef016, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8ef01a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2ef01e, SUC_lexicon_tags_12 },
    { 0x3f4ef01e, SUC_lexicon_tags_33 },
    { 0x844ef01f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eef025, SUC_lexicon_tags_7 },
    { 0x1baef026, SUC_lexicon_tags_0 },
    { 0x922ef026, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30ef02a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4ef036, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa10ef038, SUC_lexicon_tags_25 },
    { 0xd26ef039, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cef043, SUC_lexicon_tags_11 },
    { 0x750ef044, SUC_lexicon_tags_23 },
    { 0x524ef044, SUC_lexicon_tags_70 },
    { 0x2d0ef046, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24ef04f, SUC_lexicon_tags_2 },
    { 0x4b4ef04f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14ef054, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15aef056, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eef059, SUC_lexicon_tags_16 },
    { 0x35eef05a, SUC_lexicon_tags_25 },
    { 0x63eef05b, SUC_lexicon_tags_37 },
    { 0x3b0ef05c, SUC_lexicon_tags_46 },
    { 0x122ef05d, SUC_lexicon_tags_17 },
    { 0xf0cef05e, SUC_lexicon_tags_27 },
    { 0x88cef05e, SUC_lexicon_tags_25 },
    { 0x4a6ef060, SUC_lexicon_tags_8 },
    { 0x67eef061, SUC_lexicon_tags_15 },
    { 0xde8ef062, SUC_lexicon_tags_61 },
    { 0xacaef061, SUC_lexicon_tags_63 },
    { 0xdf8ef064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcaaef066, SUC_lexicon_tags_25 },
    { 0x5beef066, SUC_lexicon_tags_14 },
    { 0xb56ef068, SUC_lexicon_tags_43 },
    { 0x370ef068, SUC_lexicon_tags_18 },
    { 0x11aef069, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8daef06f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xccef072, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf32ef074, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69eef076, SUC_lexicon_tags_15 },
    { 0x8d6ef076, SUC_lexicon_tags_12 },
    { 0xd2cef078, SUC_lexicon_tags_172 },
    { 0xbdcef079, SUC_lexicon_tags_2 },
    { 0x77cef078, SUC_lexicon_tags_9 },
    { 0x482ef07b, SUC_lexicon_tags_26 },
    { 0x74cef079, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74eef081, SUC_lexicon_tags_24 },
    { 0x2f6ef082, SUC_lexicon_tags_7 },
    { 0x9b0ef083, SUC_lexicon_tags_27 },
    { 0x29eef084, SUC_lexicon_tags_37 },
    { 0x5daef084, SUC_lexicon_tags_17 },
    { 0x5d2ef082, SUC_lexicon_tags_4 },
    { 0x53cef087, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26ef08c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56ef091, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5aeef093, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89eef095, SUC_lexicon_tags_28 },
    { 0x636ef095, SUC_lexicon_tags_27 },
    { 0x68eef096, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33cef09a, SUC_lexicon_tags_19 },
    { 0x450ef09a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cef09e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56ef0a2, SUC_lexicon_tags_12 },
    { 0xfaef0a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448ef0a7, SUC_lexicon_tags_47 },
    { 0x32ef0a8, SUC_lexicon_tags_68 },
    { 0x832ef0a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6ef0ad, SUC_lexicon_tags_37 },
    { 0x404ef0ad, SUC_lexicon_tags_17 },
    { 0xe4eef0af, SUC_lexicon_tags_25 },
    { 0xf2cef0b0, SUC_lexicon_tags_1485 },
    { 0xef2ef0b1, SUC_lexicon_tags_29 },
    { 0xc52ef0b2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0ef0b6, SUC_lexicon_tags_4 },
    { 0x7aef0b7, SUC_lexicon_tags_40 },
    { 0x4b4ef0b8, SUC_lexicon_tags_77 },
    { 0x296ef0b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37aef0bb, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfeef0be, SUC_lexicon_tags_26 },
    { 0x392ef0bf, SUC_lexicon_tags_7 },
    { 0xc14ef0c0, SUC_lexicon_tags_37 },
    { 0xc6cef0bf, SUC_lexicon_tags_15 },
    { 0xe50ef0be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54ef0c5, SUC_lexicon_tags_25 },
    { 0xcaaef0c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460ef0cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9eef0cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55cef0cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x404ef0d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x346ef0d4, SUC_lexicon_tags_7 },
    { 0xfe0ef0d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb28ef0d7, SUC_lexicon_tags_26 },
    { 0xeaef0d7, SUC_lexicon_tags_40 },
    { 0xa26ef0d9, SUC_lexicon_tags_2 },
    { 0x90cef0da, SUC_lexicon_tags_25 },
    { 0xb0aef0db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0ef0e2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41eef0e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19cef0ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x564ef0ec, SUC_lexicon_tags_0 },
    { 0x206ef0ec, SUC_lexicon_tags_37 },
    { 0x60ef0ee, SUC_lexicon_tags_79 },
    { 0x47eef0ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6ef0f4, SUC_lexicon_tags_61 },
    { 0x476ef0f5, SUC_lexicon_tags_11 },
    { 0xdd6ef0f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2ef0f9, SUC_lexicon_tags_32 },
    { 0x9d6ef0f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2ef0fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe2ef0fe, SUC_lexicon_tags_11 },
    { 0xa92ef0ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8ef103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaef106, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184ef10b, SUC_lexicon_tags_0 },
    { 0xd36ef10c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0ef10f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x652ef111, SUC_lexicon_tags_14 },
    { 0x3e0ef112, SUC_lexicon_tags_55 },
    { 0x83cef113, SUC_lexicon_tags_15 },
    { 0x51aef113, SUC_lexicon_tags_57 },
    { 0x4ef114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51eef117, SUC_lexicon_tags_11 },
    { 0xd18ef118, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbeef11f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18ef124, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6ef128, SUC_lexicon_tags_11 },
    { 0x218ef129, SUC_lexicon_tags_14 },
    { 0x480ef129, SUC_lexicon_tags_37 },
    { 0x9eef129, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78eef132, SUC_lexicon_tags_1 },
    { 0x588ef133, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x10ef135, SUC_lexicon_tags_8 },
    { 0x502ef136, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97eef139, SUC_lexicon_tags_17 },
    { 0xc4aef13a, SUC_lexicon_tags_25 },
    { 0xe74ef139, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468ef145, SUC_lexicon_tags_12 },
    { 0x816ef146, SUC_lexicon_tags_37 },
    { 0x494ef147, SUC_lexicon_tags_6 },
    { 0xe9cef148, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa8ef14a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3bcef14c, SUC_lexicon_tags_15 },
    { 0xd64ef14d, SUC_lexicon_tags_17 },
    { 0x326ef14e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc0ef150, SUC_lexicon_tags_47 },
    { 0x308ef150, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeef154, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cef15b, SUC_lexicon_tags_0 },
    { 0xb4aef15c, SUC_lexicon_tags_39 },
    { 0x3baef15d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x19aef15f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704ef166, SUC_lexicon_tags_45 },
    { 0x91eef167, SUC_lexicon_tags_37 },
    { 0x458ef168, SUC_lexicon_tags_14 },
    { 0xe4ef169, SUC_lexicon_tags_65 },
    { 0x48ef167, SUC_lexicon_tags_19 },
    { 0xccaef16b, SUC_lexicon_tags_4 },
    { 0x29aef16c, SUC_lexicon_tags_56 },
    { 0xeeaef167, SUC_lexicon_tags_59 },
    { 0x114ef16e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57eef170, SUC_lexicon_tags_29 },
    { 0x69aef170, SUC_lexicon_tags_35 },
    { 0x6b0ef170, SUC_lexicon_tags_17 },
    { 0x36ef172, SUC_lexicon_tags_279 },
    { 0x596ef174, SUC_lexicon_tags_15 },
    { 0x54eef175, SUC_lexicon_tags_45 },
    { 0xcfaef175, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97aef17b, SUC_lexicon_tags_2 },
    { 0x5eeef17c, SUC_lexicon_tags_60 },
    { 0xdd6ef17d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdceef182, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceef187, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd56ef189, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeeef18e, SUC_lexicon_tags_12 },
    { 0x190ef18f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9f2ef191, SUC_lexicon_tags_27 },
    { 0x51eef191, SUC_lexicon_tags_49 },
    { 0x94ef193, SUC_lexicon_tags_66 },
    { 0xe5aef193, SUC_lexicon_tags_15 },
    { 0xbceef195, SUC_lexicon_tags_15 },
    { 0x4e8ef194, SUC_lexicon_tags_12 },
    { 0xe20ef192, SUC_lexicon_tags_8 },
    { 0xd5aef198, SUC_lexicon_tags_37 },
    { 0x270ef199, SUC_lexicon_tags_2 },
    { 0x284ef194, SUC_lexicon_tags_10 },
    { 0x9ecef19b, SUC_lexicon_tags_1 },
    { 0x3ceef19c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0ef1a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1fcef1a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb30ef1a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0ef1ab, SUC_lexicon_tags_15 },
    { 0xfeeef1ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4ef1b0, SUC_lexicon_tags_25 },
    { 0x224ef1b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28ef1b5, SUC_lexicon_tags_15 },
    { 0x434ef1b6, SUC_lexicon_tags_25 },
    { 0x488ef1b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cef1bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c8ef1bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8eef1c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf62ef1c3, SUC_lexicon_tags_49 },
    { 0xcef1c4, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xce4ef1c6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8ef1cd, SUC_lexicon_tags_65 },
    { 0x960ef1ce, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x546ef1d1, SUC_lexicon_tags_15 },
    { 0xfecef1d2, SUC_lexicon_tags_14 },
    { 0xea2ef1d2, SUC_lexicon_tags_25 },
    { 0xbc8ef1d4, SUC_lexicon_tags_29 },
    { 0x80ef1d3, SUC_lexicon_tags_37 },
    { 0xf5cef1d4, SUC_lexicon_tags_29 },
    { 0x4ceef1d3, SUC_lexicon_tags_15 },
    { 0xa3cef1d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4eef1dd, SUC_lexicon_tags_29 },
    { 0x954ef1de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39aef1e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5aef1e7, SUC_lexicon_tags_15 },
    { 0x292ef1e8, SUC_lexicon_tags_10 },
    { 0xc74ef1e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918ef1ee, SUC_lexicon_tags_25 },
    { 0x8fcef1ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1d8ef1f1, SUC_lexicon_tags_27 },
    { 0x3a4ef1f2, SUC_lexicon_tags_23 },
    { 0x47aef1f2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0ef1f9, SUC_lexicon_tags_12 },
    { 0x7eeef1fa, SUC_lexicon_tags_15 },
    { 0x16cef1fa, SUC_lexicon_tags_33 },
    { 0x8feef1fc, SUC_lexicon_tags_14 },
    { 0x874ef1fc, SUC_lexicon_tags_12 },
    { 0x822ef1fe, SUC_lexicon_tags_17 },
    { 0xff8ef1fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18eef204, SUC_lexicon_tags_26 },
    { 0xe8aef205, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aef209, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48ef20d, SUC_lexicon_tags_15 },
    { 0xc30ef20e, SUC_lexicon_tags_27 },
    { 0x8faef20e, SUC_lexicon_tags_7 },
    { 0x180ef20f, SUC_lexicon_tags_43 },
    { 0xb7eef20d, SUC_lexicon_tags_39 },
    { 0x700ef20e, SUC_lexicon_tags_30 },
    { 0x54cef213, SUC_lexicon_tags_42 },
    { 0x6fcef214, SUC_lexicon_tags_11 },
    { 0x1a4ef215, SUC_lexicon_tags_11 },
    { 0x670ef20f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ceef21a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa62ef21c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2ef21f, SUC_lexicon_tags_27 },
    { 0x52aef220, SUC_lexicon_tags_27 },
    { 0xdf6ef221, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ef228, SUC_lexicon_tags_1 },
    { 0xc52ef229, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x590ef22c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ef22f, SUC_lexicon_tags_49 },
    { 0x516ef230, SUC_lexicon_tags_12 },
    { 0x668ef231, SUC_lexicon_tags_21 },
    { 0xd1aef232, SUC_lexicon_tags_11 },
    { 0x566ef233, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2faef238, SUC_lexicon_tags_1 },
    { 0x446ef239, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26ef23d, SUC_lexicon_tags_7 },
    { 0x5d4ef23e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaeef242, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26ef247, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x930ef24a, SUC_lexicon_tags_29 },
    { 0x9beef24a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaef252, SUC_lexicon_tags_6 },
    { 0x7e6ef253, SUC_lexicon_tags_36 },
    { 0x60ef253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcef257, SUC_lexicon_tags_109 },
    { 0x43cef257, SUC_lexicon_tags_29 },
    { 0xbd0ef259, SUC_lexicon_tags_25 },
    { 0x730ef25a, SUC_lexicon_tags_2 },
    { 0xcaaef25b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb80ef25d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x10cef25f, SUC_lexicon_tags_9 },
    { 0xb34ef260, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2eef264, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x880ef267, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682ef26e, SUC_lexicon_tags_7 },
    { 0x378ef26e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418ef274, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x31cef277, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704ef27b, SUC_lexicon_tags_25 },
    { 0x152ef27b, SUC_lexicon_tags_3 },
    { 0x41eef27c, SUC_lexicon_tags_25 },
    { 0x3b8ef27d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1aef281, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324ef286, SUC_lexicon_tags_37 },
    { 0x89cef286, SUC_lexicon_tags_23 },
    { 0x2e2ef288, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c0ef28a, SUC_lexicon_tags_29 },
    { 0xabeef28a, SUC_lexicon_tags_7 },
    { 0x6c4ef28c, SUC_lexicon_tags_2 },
    { 0x698ef28c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8ef291, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ef293, SUC_lexicon_tags_1 },
    { 0x66eef293, SUC_lexicon_tags_17 },
    { 0x2feef293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcef29a, SUC_lexicon_tags_83 },
    { 0xb00ef29b, SUC_lexicon_tags_1190 },
    { 0, NULL },
    { 0, NULL },
    { 0x600ef29e, SUC_lexicon_tags_19 },
    { 0x2a6ef29e, SUC_lexicon_tags_25 },
    { 0x7d4ef29f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdbaef2a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb78ef2a4, SUC_lexicon_tags_31 },
    { 0x908ef2a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242ef2a9, SUC_lexicon_tags_43 },
    { 0xfe8ef2a9, SUC_lexicon_tags_15 },
    { 0x67aef2ab, SUC_lexicon_tags_62 },
    { 0xb2eef2ab, SUC_lexicon_tags_39 },
    { 0x5f6ef2ad, SUC_lexicon_tags_0 },
    { 0x750ef2ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8ef2b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef2b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfc8ef2b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24ef2c5, SUC_lexicon_tags_113 },
    { 0xa64ef2c6, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228ef2ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x728ef2d1, SUC_lexicon_tags_17 },
    { 0xe00ef2d1, SUC_lexicon_tags_47 },
    { 0x4c0ef2d2, SUC_lexicon_tags_11 },
    { 0x6baef2d4, SUC_lexicon_tags_74 },
    { 0x278ef2d4, SUC_lexicon_tags_27 },
    { 0xb94ef2d6, SUC_lexicon_tags_63 },
    { 0xc02ef2d7, SUC_lexicon_tags_14 },
    { 0xc8aef2d8, SUC_lexicon_tags_37 },
    { 0x3a2ef2d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0ef2dc, SUC_lexicon_tags_0 },
    { 0x204ef2dd, SUC_lexicon_tags_0 },
    { 0x5aef2de, SUC_lexicon_tags_47 },
    { 0xb78ef2df, SUC_lexicon_tags_71 },
    { 0x1acef2e0, SUC_lexicon_tags_13 },
    { 0x90cef2df, SUC_lexicon_tags_11 },
    { 0xb4eef2e2, SUC_lexicon_tags_45 },
    { 0xb08ef2dd, SUC_lexicon_tags_28 },
    { 0x506ef2e4, SUC_lexicon_tags_1 },
    { 0x77aef2e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeecef2e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8ef2ea, SUC_lexicon_tags_37 },
    { 0x44cef2eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x660ef2ed, SUC_lexicon_tags_198 },
    { 0x8e8ef2ee, SUC_lexicon_tags_37 },
    { 0x288ef2ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8caef2f1, SUC_lexicon_tags_2 },
    { 0xd08ef2f2, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118ef2fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cef308, SUC_lexicon_tags_49 },
    { 0x9f2ef309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79aef30b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xad6ef30d, SUC_lexicon_tags_17 },
    { 0x672ef30e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x330ef310, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcef313, SUC_lexicon_tags_19 },
    { 0xcd0ef314, SUC_lexicon_tags_9 },
    { 0x20ef315, SUC_lexicon_tags_14 },
    { 0x1b8ef315, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb90ef318, SUC_lexicon_tags_11 },
    { 0xaf2ef318, SUC_lexicon_tags_12 },
    { 0x586ef319, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6ef31f, SUC_lexicon_tags_47 },
    { 0xef4ef320, SUC_lexicon_tags_17 },
    { 0xbeaef321, SUC_lexicon_tags_48 },
    { 0xe68ef321, SUC_lexicon_tags_31 },
    { 0x85aef322, SUC_lexicon_tags_27 },
    { 0x352ef321, SUC_lexicon_tags_26 },
    { 0x5a4ef325, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2ef32d, SUC_lexicon_tags_618 },
    { 0x92eef32e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2ef337, SUC_lexicon_tags_37 },
    { 0x7aaef338, SUC_lexicon_tags_15 },
    { 0x148ef339, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacef33d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58aef341, SUC_lexicon_tags_26 },
    { 0xe4cef341, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4deef345, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x820ef347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcef34a, SUC_lexicon_tags_77 },
    { 0xf94ef34b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0ef350, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5eef356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeef35a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94cef35c, SUC_lexicon_tags_17 },
    { 0x504ef35d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x920ef35f, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xa62ef361, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5aef366, SUC_lexicon_tags_65 },
    { 0x298ef366, SUC_lexicon_tags_79 },
    { 0xeb6ef368, SUC_lexicon_tags_37 },
    { 0xbf6ef368, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0ef36c, SUC_lexicon_tags_25 },
    { 0x1f0ef36d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cef371, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef374, SUC_lexicon_tags_21 },
    { 0x468ef375, SUC_lexicon_tags_17 },
    { 0xb6eef374, SUC_lexicon_tags_8 },
    { 0x4c8ef376, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6ef37c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8ef37f, SUC_lexicon_tags_2 },
    { 0x90eef380, SUC_lexicon_tags_32 },
    { 0xce2ef37f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb24ef383, SUC_lexicon_tags_7 },
    { 0x628ef383, SUC_lexicon_tags_4 },
    { 0x16ef385, SUC_lexicon_tags_62 },
    { 0xd58ef385, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc9eef388, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4aef38b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5acef38d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8eef390, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50ef394, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x666ef397, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x748ef39a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4ef3a3, SUC_lexicon_tags_12 },
    { 0xecaef3a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568ef3ad, SUC_lexicon_tags_11 },
    { 0x6f8ef3ae, SUC_lexicon_tags_47 },
    { 0xc32ef3ae, SUC_lexicon_tags_37 },
    { 0x8caef3b0, SUC_lexicon_tags_47 },
    { 0xb96ef3b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e6ef3b3, SUC_lexicon_tags_1 },
    { 0xaecef3b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cef3b8, SUC_lexicon_tags_47 },
    { 0x9d8ef3b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x39cef3bc, SUC_lexicon_tags_27 },
    { 0x1acef3bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xefeef3c0, SUC_lexicon_tags_15 },
    { 0x926ef3c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812ef3c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d8ef3c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x366ef3c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12ef3cc, SUC_lexicon_tags_3 },
    { 0x3ecef3cd, SUC_lexicon_tags_60 },
    { 0x7a8ef3ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18ef3d2, SUC_lexicon_tags_37 },
    { 0x3c8ef3d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ef3d7, SUC_lexicon_tags_38 },
    { 0xd68ef3d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb9cef3da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aef3df, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982ef3e3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaeef3e6, SUC_lexicon_tags_17 },
    { 0xad2ef3e7, SUC_lexicon_tags_17 },
    { 0x222ef3e8, SUC_lexicon_tags_61 },
    { 0xa08ef3e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25eef3ed, SUC_lexicon_tags_49 },
    { 0x334ef3ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8aef3f2, SUC_lexicon_tags_11 },
    { 0xa62ef3f3, SUC_lexicon_tags_7 },
    { 0xffaef3f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20ef3f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eeef3fb, SUC_lexicon_tags_11 },
    { 0xfeef3fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8cef3fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x700ef400, SUC_lexicon_tags_37 },
    { 0x610ef400, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aef409, SUC_lexicon_tags_43 },
    { 0x442ef409, SUC_lexicon_tags_2 },
    { 0x97eef40b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40ef411, SUC_lexicon_tags_49 },
    { 0xc7cef411, SUC_lexicon_tags_38 },
    { 0xfb6ef412, SUC_lexicon_tags_23 },
    { 0xe6eef414, SUC_lexicon_tags_6 },
    { 0x16ef415, SUC_lexicon_tags_722 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceef419, SUC_lexicon_tags_49 },
    { 0xbf8ef41a, SUC_lexicon_tags_25 },
    { 0xd34ef41b, SUC_lexicon_tags_49 },
    { 0x19aef41b, SUC_lexicon_tags_12 },
    { 0x2eaef41d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72cef41f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6ef424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaef428, SUC_lexicon_tags_38 },
    { 0xd9aef428, SUC_lexicon_tags_7 },
    { 0xeaef428, SUC_lexicon_tags_37 },
    { 0xa60ef42b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec2ef430, SUC_lexicon_tags_23 },
    { 0xb5eef431, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6ef436, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcef43b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cef442, SUC_lexicon_tags_8 },
    { 0xc22ef443, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2eef44a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cef44d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6ef453, SUC_lexicon_tags_38 },
    { 0xda8ef454, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcef458, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ef461, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68eef463, SUC_lexicon_tags_29 },
    { 0xd9eef464, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa7eef466, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8ef469, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48cef46c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x900ef46e, SUC_lexicon_tags_25 },
    { 0xa06ef46f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50ef474, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61eef47c, SUC_lexicon_tags_12 },
    { 0xd6aef47d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa26ef47f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaef488, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510ef48d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x502ef48f, SUC_lexicon_tags_49 },
    { 0x1f2ef490, SUC_lexicon_tags_27 },
    { 0x704ef48f, SUC_lexicon_tags_11 },
    { 0xec0ef492, SUC_lexicon_tags_65 },
    { 0xe52ef493, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ef496, SUC_lexicon_tags_37 },
    { 0x414ef497, SUC_lexicon_tags_12 },
    { 0x908ef496, SUC_lexicon_tags_25 },
    { 0xfaaef499, SUC_lexicon_tags_11 },
    { 0xf82ef49a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8ef4a0, SUC_lexicon_tags_45 },
    { 0x568ef4a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa38ef4a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4ef4a6, SUC_lexicon_tags_29 },
    { 0x466ef4a7, SUC_lexicon_tags_68 },
    { 0x9f4ef4a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x33eef4aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x324ef4ad, SUC_lexicon_tags_31 },
    { 0x18cef4ae, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8ef4b2, SUC_lexicon_tags_12 },
    { 0x84aef4b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x996ef4b6, SUC_lexicon_tags_38 },
    { 0xef4ef4b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13cef4b9, SUC_lexicon_tags_8 },
    { 0xe64ef4ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcaaef4bc, SUC_lexicon_tags_45 },
    { 0x9d4ef4bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568ef4c1, SUC_lexicon_tags_12 },
    { 0xaaaef4c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0ef4ce, SUC_lexicon_tags_6 },
    { 0xb50ef4ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96ef4d2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cef4d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8ef4dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4ef4e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4c0ef4e2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa94ef4e4, SUC_lexicon_tags_49 },
    { 0x68aef4e5, SUC_lexicon_tags_0 },
    { 0x8aef4e6, SUC_lexicon_tags_64 },
    { 0x4aef4e7, SUC_lexicon_tags_66 },
    { 0xd14ef4e8, SUC_lexicon_tags_43 },
    { 0x10cef4e6, SUC_lexicon_tags_15 },
    { 0x534ef4e8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4ef4ed, SUC_lexicon_tags_27 },
    { 0x9daef4ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cef4f5, SUC_lexicon_tags_45 },
    { 0x4cef4f6, SUC_lexicon_tags_6 },
    { 0xa72ef4f7, SUC_lexicon_tags_77 },
    { 0xfeaef4f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef4fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cef505, SUC_lexicon_tags_65 },
    { 0x400ef506, SUC_lexicon_tags_1 },
    { 0x8a4ef507, SUC_lexicon_tags_14 },
    { 0xddcef508, SUC_lexicon_tags_201 },
    { 0xc3aef509, SUC_lexicon_tags_8 },
    { 0x42ef50a, SUC_lexicon_tags_14 },
    { 0xc9aef50a, SUC_lexicon_tags_36 },
    { 0x24eef506, SUC_lexicon_tags_65 },
    { 0xbfcef507, SUC_lexicon_tags_60 },
    { 0x246ef507, SUC_lexicon_tags_17 },
    { 0xb16ef50f, SUC_lexicon_tags_29 },
    { 0x61aef510, SUC_lexicon_tags_26 },
    { 0x58eef511, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x86ef513, SUC_lexicon_tags_63 },
    { 0x31aef514, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0ef51a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04ef51e, SUC_lexicon_tags_70 },
    { 0x440ef51f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x68eef521, SUC_lexicon_tags_15 },
    { 0xf4cef522, SUC_lexicon_tags_23 },
    { 0x334ef523, SUC_lexicon_tags_49 },
    { 0xcd2ef523, SUC_lexicon_tags_19 },
    { 0x9dcef522, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x964ef527, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22aef52d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cef531, SUC_lexicon_tags_1 },
    { 0x874ef532, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x99aef534, SUC_lexicon_tags_2 },
    { 0x700ef534, SUC_lexicon_tags_23 },
    { 0x234ef536, SUC_lexicon_tags_25 },
    { 0x240ef536, SUC_lexicon_tags_104 },
    { 0x5b2ef536, SUC_lexicon_tags_1 },
    { 0x660ef536, SUC_lexicon_tags_2 },
    { 0x282ef534, SUC_lexicon_tags_27 },
    { 0x104ef535, SUC_lexicon_tags_15 },
    { 0xcf4ef53c, SUC_lexicon_tags_37 },
    { 0x9aaef53c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x424ef540, SUC_lexicon_tags_37 },
    { 0x4e8ef540, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18ef544, SUC_lexicon_tags_30 },
    { 0x350ef545, SUC_lexicon_tags_19 },
    { 0x6d0ef544, SUC_lexicon_tags_1 },
    { 0xc4aef546, SUC_lexicon_tags_12 },
    { 0xd1aef548, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae8ef54a, SUC_lexicon_tags_14 },
    { 0x60eef54a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aef54e, SUC_lexicon_tags_7 },
    { 0xeb8ef54f, SUC_lexicon_tags_12 },
    { 0x554ef54e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6ef554, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ceef55b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116ef565, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2ef56a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4aef56c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfaeef56e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cef571, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0ef575, SUC_lexicon_tags_11 },
    { 0xf34ef576, SUC_lexicon_tags_12 },
    { 0x392ef576, SUC_lexicon_tags_27 },
    { 0x2d6ef578, SUC_lexicon_tags_15 },
    { 0xce6ef579, SUC_lexicon_tags_1 },
    { 0x330ef579, SUC_lexicon_tags_10 },
    { 0x33eef57a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf8ef57d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cef581, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd28ef583, SUC_lexicon_tags_37 },
    { 0xe16ef583, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x804ef586, SUC_lexicon_tags_25 },
    { 0x6c0ef587, SUC_lexicon_tags_55 },
    { 0x8c8ef587, SUC_lexicon_tags_46 },
    { 0x926ef589, SUC_lexicon_tags_41 },
    { 0x240ef589, SUC_lexicon_tags_38 },
    { 0x3a0ef588, SUC_lexicon_tags_37 },
    { 0x406ef58b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x78eef58f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccef593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x602ef596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42eef59a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110ef59e, SUC_lexicon_tags_108 },
    { 0x98cef59f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cef5a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34eef5a7, SUC_lexicon_tags_0 },
    { 0x60aef5a7, SUC_lexicon_tags_19 },
    { 0x37eef5a7, SUC_lexicon_tags_1 },
    { 0xcd0ef5a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcecef5ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc32ef5af, SUC_lexicon_tags_33 },
    { 0x33eef5b0, SUC_lexicon_tags_38 },
    { 0x620ef5af, SUC_lexicon_tags_252 },
    { 0xad0ef5b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56ef5b5, SUC_lexicon_tags_11 },
    { 0x44cef5b6, SUC_lexicon_tags_6 },
    { 0x454ef5b6, SUC_lexicon_tags_1 },
    { 0x3f8ef5b8, SUC_lexicon_tags_1 },
    { 0x684ef5b7, SUC_lexicon_tags_8 },
    { 0x41eef5ba, SUC_lexicon_tags_17 },
    { 0x14aef5bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50ef5bd, SUC_lexicon_tags_39 },
    { 0xb3eef5be, SUC_lexicon_tags_12 },
    { 0x1eef5bd, SUC_lexicon_tags_7 },
    { 0x8aaef5bf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8ef5cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2ef5d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeeef5dc, SUC_lexicon_tags_11 },
    { 0x7b8ef5dd, SUC_lexicon_tags_31 },
    { 0xa08ef5de, SUC_lexicon_tags_47 },
    { 0x6d2ef5dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aef5e3, SUC_lexicon_tags_65 },
    { 0xbe4ef5e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18ef5e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcef5ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x876ef5ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaef5f3, SUC_lexicon_tags_1 },
    { 0xbd0ef5f4, SUC_lexicon_tags_15 },
    { 0x1b6ef5f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594ef5f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4aef5fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe7aef5ff, SUC_lexicon_tags_14 },
    { 0xec0ef5ff, SUC_lexicon_tags_15 },
    { 0xf9cef600, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcef606, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2ef609, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x762ef60c, SUC_lexicon_tags_17 },
    { 0xc36ef60c, SUC_lexicon_tags_33 },
    { 0x84aef60e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xceeef610, SUC_lexicon_tags_25 },
    { 0xf7aef610, SUC_lexicon_tags_11 },
    { 0xe62ef612, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf0aef614, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7eef617, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x884ef61a, SUC_lexicon_tags_1 },
    { 0x826ef61b, SUC_lexicon_tags_33 },
    { 0x544ef61a, SUC_lexicon_tags_25 },
    { 0xd92ef61d, SUC_lexicon_tags_19 },
    { 0xf28ef61d, SUC_lexicon_tags_30 },
    { 0x1f2ef61a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x61cef621, SUC_lexicon_tags_38 },
    { 0xee8ef622, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa6cef624, SUC_lexicon_tags_16 },
    { 0xc8aef624, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2ef628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c2ef62a, SUC_lexicon_tags_11 },
    { 0x384ef62b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfeef62f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4daef635, SUC_lexicon_tags_46 },
    { 0x746ef635, SUC_lexicon_tags_10 },
    { 0x69aef637, SUC_lexicon_tags_2 },
    { 0xcf4ef638, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb4ef63a, SUC_lexicon_tags_28 },
    { 0x668ef63b, SUC_lexicon_tags_27 },
    { 0xc42ef63c, SUC_lexicon_tags_17 },
    { 0x62ef63d, SUC_lexicon_tags_1 },
    { 0xe72ef63e, SUC_lexicon_tags_2 },
    { 0xdbeef63f, SUC_lexicon_tags_46 },
    { 0x5b6ef640, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2ef643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480ef647, SUC_lexicon_tags_15 },
    { 0xccef648, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x524ef64b, SUC_lexicon_tags_30 },
    { 0x9c4ef64c, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xf34ef64e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc64ef650, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926ef655, SUC_lexicon_tags_9 },
    { 0x580ef656, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2ef659, SUC_lexicon_tags_39 },
    { 0x7fcef659, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8ef65d, SUC_lexicon_tags_37 },
    { 0x132ef65e, SUC_lexicon_tags_12 },
    { 0xe8ef65d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x48ef661, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438ef668, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292ef671, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x742ef673, SUC_lexicon_tags_11 },
    { 0x92ef674, SUC_lexicon_tags_2 },
    { 0xb6cef674, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2ef679, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x51cef67b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ef682, SUC_lexicon_tags_1710 },
    { 0x1deef683, SUC_lexicon_tags_49 },
    { 0x1a2ef684, SUC_lexicon_tags_25 },
    { 0xb62ef685, SUC_lexicon_tags_7 },
    { 0x764ef683, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xc72ef688, SUC_lexicon_tags_106 },
    { 0x522ef689, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8ef68e, SUC_lexicon_tags_234 },
    { 0x344ef68f, SUC_lexicon_tags_25 },
    { 0x2baef68e, SUC_lexicon_tags_11 },
    { 0x5dcef690, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaef695, SUC_lexicon_tags_25 },
    { 0x26ef696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5eef69a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6ef69e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4ef6a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1acef6a3, SUC_lexicon_tags_8 },
    { 0x7e8ef6a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1feef6a6, SUC_lexicon_tags_37 },
    { 0x9caef6a7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x760ef6aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61aef6ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefeef6b1, SUC_lexicon_tags_30 },
    { 0xfe6ef6b2, SUC_lexicon_tags_17 },
    { 0x1d2ef6b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x862ef6b6, SUC_lexicon_tags_12 },
    { 0x58aef6b7, SUC_lexicon_tags_15 },
    { 0x1eef6b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cef6bc, SUC_lexicon_tags_15 },
    { 0x276ef6bd, SUC_lexicon_tags_21 },
    { 0x52ef6be, SUC_lexicon_tags_17 },
    { 0xe9cef6bf, SUC_lexicon_tags_33 },
    { 0x60eef6bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcef6c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882ef6c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4eef6d0, SUC_lexicon_tags_85 },
    { 0x27eef6d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736ef6d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866ef6df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6d8ef6e1, SUC_lexicon_tags_25 },
    { 0x1ccef6e2, SUC_lexicon_tags_9 },
    { 0x24cef6e3, SUC_lexicon_tags_19 },
    { 0xc32ef6e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baef6e9, SUC_lexicon_tags_37 },
    { 0x628ef6ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6ef6f5, SUC_lexicon_tags_9 },
    { 0x1eaef6f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cef6fe, SUC_lexicon_tags_11 },
    { 0x104ef6ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ef702, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aeef706, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586ef70a, SUC_lexicon_tags_47 },
    { 0x81eef70b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86ef70f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcdeef711, SUC_lexicon_tags_33 },
    { 0x890ef712, SUC_lexicon_tags_17 },
    { 0xfacef712, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe8ef716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf8ef718, SUC_lexicon_tags_0 },
    { 0xc7cef719, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4beef71c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886ef721, SUC_lexicon_tags_34 },
    { 0x352ef721, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x882ef724, SUC_lexicon_tags_45 },
    { 0xcf2ef725, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x466ef727, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x818ef729, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ef72e, SUC_lexicon_tags_63 },
    { 0x954ef72f, SUC_lexicon_tags_7 },
    { 0x5a0ef730, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8aeef732, SUC_lexicon_tags_26 },
    { 0xecef733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x174ef735, SUC_lexicon_tags_25 },
    { 0x79cef736, SUC_lexicon_tags_179 },
    { 0xd82ef736, SUC_lexicon_tags_62 },
    { 0xb22ef738, SUC_lexicon_tags_49 },
    { 0x26aef735, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecef73d, SUC_lexicon_tags_45 },
    { 0x324ef73d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3beef745, SUC_lexicon_tags_45 },
    { 0xef0ef746, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554ef750, SUC_lexicon_tags_2 },
    { 0x8f6ef751, SUC_lexicon_tags_49 },
    { 0x27eef751, SUC_lexicon_tags_103 },
    { 0x512ef751, SUC_lexicon_tags_13 },
    { 0xf6ef753, SUC_lexicon_tags_47 },
    { 0x8c4ef755, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6eef757, SUC_lexicon_tags_63 },
    { 0x976ef758, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2faef75e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ef761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6aef763, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaef76d, SUC_lexicon_tags_25 },
    { 0x328ef76d, SUC_lexicon_tags_31 },
    { 0xd02ef76e, SUC_lexicon_tags_17 },
    { 0xa8aef770, SUC_lexicon_tags_3 },
    { 0x94eef771, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6a0ef773, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x770ef776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45cef778, SUC_lexicon_tags_17 },
    { 0x672ef779, SUC_lexicon_tags_47 },
    { 0x9feef779, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82ef77f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8aef782, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5eef784, SUC_lexicon_tags_10 },
    { 0xa46ef785, SUC_lexicon_tags_15 },
    { 0xbfcef785, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332ef78a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330ef791, SUC_lexicon_tags_28 },
    { 0x102ef792, SUC_lexicon_tags_1 },
    { 0xa86ef793, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb00ef795, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94ef79a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcaef79c, SUC_lexicon_tags_27 },
    { 0xcfcef79d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc58ef79f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96ef7a2, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cef7a9, SUC_lexicon_tags_43 },
    { 0x966ef7aa, SUC_lexicon_tags_37 },
    { 0x1f4ef7ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8ef7b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43eef7b6, SUC_lexicon_tags_7 },
    { 0xaeef7b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46ef7ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706ef7c7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x634ef7ca, SUC_lexicon_tags_1 },
    { 0xf24ef7cb, SUC_lexicon_tags_25 },
    { 0x166ef7ca, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x920ef7ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0aef7d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eef7d5, SUC_lexicon_tags_87 },
    { 0xe50ef7d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59eef7e2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x916ef7e5, SUC_lexicon_tags_37 },
    { 0x3a4ef7e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97cef7e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x290ef7ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f6ef7ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x948ef7ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x762ef7f7, SUC_lexicon_tags_23 },
    { 0x6d6ef7f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8ef7fe, SUC_lexicon_tags_11 },
    { 0xfeef7fe, SUC_lexicon_tags_4 },
    { 0xeb2ef800, SUC_lexicon_tags_33 },
    { 0x4c0ef800, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ef805, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52aef809, SUC_lexicon_tags_45 },
    { 0xdeef80a, SUC_lexicon_tags_37 },
    { 0xe7eef80a, SUC_lexicon_tags_6 },
    { 0x34ef809, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccef817, SUC_lexicon_tags_68 },
    { 0x870ef817, SUC_lexicon_tags_17 },
    { 0x892ef819, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6eef81b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcef81e, SUC_lexicon_tags_30 },
    { 0x834ef81f, SUC_lexicon_tags_31 },
    { 0x6e4ef820, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aef823, SUC_lexicon_tags_15 },
    { 0x97eef824, SUC_lexicon_tags_17 },
    { 0xe82ef825, SUC_lexicon_tags_12 },
    { 0x842ef824, SUC_lexicon_tags_27 },
    { 0x368ef827, SUC_lexicon_tags_12 },
    { 0xc56ef824, SUC_lexicon_tags_1 },
    { 0x22aef826, SUC_lexicon_tags_8 },
    { 0xab0ef82a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86ef82d, SUC_lexicon_tags_19 },
    { 0xe7cef82e, SUC_lexicon_tags_33 },
    { 0x988ef82e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2ef832, SUC_lexicon_tags_1 },
    { 0x9f2ef833, SUC_lexicon_tags_23 },
    { 0x9e6ef834, SUC_lexicon_tags_37 },
    { 0x5e6ef835, SUC_lexicon_tags_7 },
    { 0x6daef836, SUC_lexicon_tags_25 },
    { 0xe18ef837, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8aef83e, SUC_lexicon_tags_14 },
    { 0xdecef83e, SUC_lexicon_tags_30 },
    { 0x5c4ef83f, SUC_lexicon_tags_55 },
    { 0x50eef841, SUC_lexicon_tags_104 },
    { 0x810ef840, SUC_lexicon_tags_49 },
    { 0x118ef843, SUC_lexicon_tags_46 },
    { 0xe18ef83e, SUC_lexicon_tags_1 },
    { 0x850ef845, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbe4ef847, SUC_lexicon_tags_25 },
    { 0x4d0ef848, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50ef84c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x854ef84f, SUC_lexicon_tags_5 },
    { 0xaf8ef84f, SUC_lexicon_tags_11 },
    { 0xde8ef84f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabaef854, SUC_lexicon_tags_28 },
    { 0x7b4ef855, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x734ef857, SUC_lexicon_tags_2 },
    { 0xb0cef857, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416ef85c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d4ef85e, SUC_lexicon_tags_16 },
    { 0xd78ef85f, SUC_lexicon_tags_8 },
    { 0xcfeef860, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x394ef863, SUC_lexicon_tags_4 },
    { 0x80eef864, SUC_lexicon_tags_15 },
    { 0x8f4ef865, SUC_lexicon_tags_56 },
    { 0xb0ef866, SUC_lexicon_tags_29 },
    { 0xad6ef867, SUC_lexicon_tags_17 },
    { 0xdb8ef868, SUC_lexicon_tags_2 },
    { 0x4aaef869, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbc0ef86b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0ef871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x820ef874, SUC_lexicon_tags_7 },
    { 0xd86ef875, SUC_lexicon_tags_1 },
    { 0x6e4ef875, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0ef880, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35aef884, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62cef889, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aef88d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540ef891, SUC_lexicon_tags_17 },
    { 0x680ef892, SUC_lexicon_tags_49 },
    { 0x81cef893, SUC_lexicon_tags_12 },
    { 0x12cef894, SUC_lexicon_tags_11 },
    { 0xea8ef895, SUC_lexicon_tags_12 },
    { 0x9bcef895, SUC_lexicon_tags_11 },
    { 0x21eef895, SUC_lexicon_tags_198 },
    { 0x58ef895, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x35eef89b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x46aef89e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aeef8a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x966ef8a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62aef8aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cef8ae, SUC_lexicon_tags_8 },
    { 0x3baef8af, SUC_lexicon_tags_2 },
    { 0xe9aef8ae, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x586ef8b2, SUC_lexicon_tags_16 },
    { 0x54cef8b2, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x8b4ef8b5, SUC_lexicon_tags_1 },
    { 0xe2cef8b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22ef8b9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9e6ef8bb, SUC_lexicon_tags_25 },
    { 0x68eef8bb, SUC_lexicon_tags_34 },
    { 0x268ef8bd, SUC_lexicon_tags_32 },
    { 0x63eef8be, SUC_lexicon_tags_0 },
    { 0x410ef8bf, SUC_lexicon_tags_6 },
    { 0x370ef8c0, SUC_lexicon_tags_27 },
    { 0xa9eef8c0, SUC_lexicon_tags_17 },
    { 0x826ef8c2, SUC_lexicon_tags_37 },
    { 0x424ef8c3, SUC_lexicon_tags_84 },
    { 0xf5cef8c4, SUC_lexicon_tags_15 },
    { 0x758ef8c4, SUC_lexicon_tags_45 },
    { 0xa52ef8c6, SUC_lexicon_tags_12 },
    { 0x4d6ef8c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2ef8cc, SUC_lexicon_tags_706 },
    { 0x318ef8cd, SUC_lexicon_tags_47 },
    { 0xbbaef8cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430ef8d2, SUC_lexicon_tags_15 },
    { 0x832ef8d3, SUC_lexicon_tags_7 },
    { 0xd64ef8d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x680ef8d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0ef8d9, SUC_lexicon_tags_141 },
    { 0x112ef8da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40aef8df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc68ef8e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe68ef8e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x71aef8ea, SUC_lexicon_tags_42 },
    { 0xf3cef8eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccef8ee, SUC_lexicon_tags_13 },
    { 0x344ef8ee, SUC_lexicon_tags_2 },
    { 0x9a2ef8ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6ef8f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66ef8f8, SUC_lexicon_tags_6 },
    { 0x746ef8f9, SUC_lexicon_tags_55 },
    { 0xa38ef8fa, SUC_lexicon_tags_2 },
    { 0x1baef8fb, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcef8fe, SUC_lexicon_tags_26 },
    { 0xd5eef8ff, SUC_lexicon_tags_47 },
    { 0x3c4ef8ff, SUC_lexicon_tags_27 },
    { 0x696ef901, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8eef904, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe00ef906, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ef908, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeef90b, SUC_lexicon_tags_15 },
    { 0x33aef90b, SUC_lexicon_tags_23 },
    { 0x3feef90d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x126ef910, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cef919, SUC_lexicon_tags_28 },
    { 0xf96ef919, SUC_lexicon_tags_14 },
    { 0x67cef91b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4ef91e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x398ef920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5aef927, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8ef92a, SUC_lexicon_tags_49 },
    { 0x6aaef92b, SUC_lexicon_tags_12 },
    { 0x514ef92c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674ef932, SUC_lexicon_tags_12 },
    { 0xd96ef933, SUC_lexicon_tags_20 },
    { 0x46eef933, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2ef937, SUC_lexicon_tags_14 },
    { 0x396ef937, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2d2ef93a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbeef93c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x664ef93f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eef947, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb96ef94a, SUC_lexicon_tags_66 },
    { 0xafcef94a, SUC_lexicon_tags_38 },
    { 0x8ef94b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6ef950, SUC_lexicon_tags_26 },
    { 0x718ef951, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0ef956, SUC_lexicon_tags_2 },
    { 0x230ef956, SUC_lexicon_tags_25 },
    { 0x6c0ef957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ccef95d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18ef964, SUC_lexicon_tags_47 },
    { 0x838ef965, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644ef96b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e2ef96d, SUC_lexicon_tags_12 },
    { 0xa52ef96e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cef974, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xca6ef976, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53aef97b, SUC_lexicon_tags_1 },
    { 0x68aef97c, SUC_lexicon_tags_32 },
    { 0x7daef97d, SUC_lexicon_tags_14 },
    { 0x504ef97e, SUC_lexicon_tags_57 },
    { 0x19aef97f, SUC_lexicon_tags_39 },
    { 0x12eef97d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46ef983, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa2aef985, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6eef98b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x508ef98d, SUC_lexicon_tags_63 },
    { 0xe76ef98d, SUC_lexicon_tags_22 },
    { 0xc9cef98d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcef992, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73cef994, SUC_lexicon_tags_66 },
    { 0xbdaef994, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x87aef997, SUC_lexicon_tags_14 },
    { 0x60eef997, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58eef99e, SUC_lexicon_tags_0 },
    { 0x9bcef99f, SUC_lexicon_tags_44 },
    { 0x8d0ef9a0, SUC_lexicon_tags_15 },
    { 0x148ef9a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a0ef9a3, SUC_lexicon_tags_37 },
    { 0xd60ef9a4, SUC_lexicon_tags_21 },
    { 0x57cef9a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52ef9a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70cef9ab, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320ef9af, SUC_lexicon_tags_15 },
    { 0x548ef9af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc8ef9b2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cef9b7, SUC_lexicon_tags_37 },
    { 0xe24ef9b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6feef9bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14ef9be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51eef9c2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x188ef9c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06ef9c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x786ef9cb, SUC_lexicon_tags_37 },
    { 0xaa0ef9cb, SUC_lexicon_tags_30 },
    { 0xba0ef9cb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecef9d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x562ef9d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58eef9d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f2ef9d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaef9db, SUC_lexicon_tags_37 },
    { 0x648ef9dc, SUC_lexicon_tags_45 },
    { 0xa54ef9dd, SUC_lexicon_tags_14 },
    { 0xe98ef9de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226ef9e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf7eef9e9, SUC_lexicon_tags_54 },
    { 0xd90ef9ea, SUC_lexicon_tags_25 },
    { 0x990ef9ea, SUC_lexicon_tags_12 },
    { 0xdbeef9ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842ef9f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2ef9fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2ef9fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8efa04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30aefa09, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x24aefa0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29eefa0d, SUC_lexicon_tags_9 },
    { 0x4deefa0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42efa12, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4efa15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaefa19, SUC_lexicon_tags_15 },
    { 0xf5aefa19, SUC_lexicon_tags_7 },
    { 0x37aefa1b, SUC_lexicon_tags_37 },
    { 0x4ceefa1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcefa1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabaefa21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efa28, SUC_lexicon_tags_7 },
    { 0xa4aefa28, SUC_lexicon_tags_26 },
    { 0xe76efa2a, SUC_lexicon_tags_27 },
    { 0xd24efa29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598efa2f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x882efa31, SUC_lexicon_tags_26 },
    { 0xa9cefa31, SUC_lexicon_tags_1 },
    { 0x7c6efa32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4efa39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfeefa3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b6efa3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xba2efa40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16efa43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb90efa45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766efa49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136efa50, SUC_lexicon_tags_2 },
    { 0xaccefa50, SUC_lexicon_tags_15 },
    { 0xd90efa52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed8efa54, SUC_lexicon_tags_37 },
    { 0x8d0efa55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4efa59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0efa61, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaefa65, SUC_lexicon_tags_113 },
    { 0xde6efa66, SUC_lexicon_tags_17 },
    { 0x892efa65, SUC_lexicon_tags_17 },
    { 0xde2efa65, SUC_lexicon_tags_25 },
    { 0x2caefa69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cefa70, SUC_lexicon_tags_7 },
    { 0xbacefa71, SUC_lexicon_tags_14 },
    { 0xb02efa71, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddeefa76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe20efa78, SUC_lexicon_tags_16 },
    { 0x82cefa79, SUC_lexicon_tags_7 },
    { 0xee6efa79, SUC_lexicon_tags_37 },
    { 0x2a2efa7a, SUC_lexicon_tags_9 },
    { 0xddaefa79, SUC_lexicon_tags_25 },
    { 0x5e0efa7c, SUC_lexicon_tags_68 },
    { 0x7b4efa7c, SUC_lexicon_tags_11 },
    { 0x126efa7f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x910efa81, SUC_lexicon_tags_31 },
    { 0xa1eefa82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf42efa84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80eefa89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48efa8d, SUC_lexicon_tags_30 },
    { 0xb8cefa8e, SUC_lexicon_tags_61 },
    { 0x466efa8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbeefa96, SUC_lexicon_tags_15 },
    { 0x576efa96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc70efa99, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x48efa9b, SUC_lexicon_tags_2 },
    { 0xa52efa9c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efaa0, SUC_lexicon_tags_27 },
    { 0x584efaa1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4efaa7, SUC_lexicon_tags_25 },
    { 0xd04efaa8, SUC_lexicon_tags_32 },
    { 0x256efaa8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6efaad, SUC_lexicon_tags_12 },
    { 0x3a2efaad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20efab5, SUC_lexicon_tags_17 },
    { 0xb2cefab6, SUC_lexicon_tags_26 },
    { 0x832efab7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40efabb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x516efabe, SUC_lexicon_tags_7 },
    { 0x4c8efabf, SUC_lexicon_tags_23 },
    { 0x8cefabf, SUC_lexicon_tags_25 },
    { 0x710efabe, SUC_lexicon_tags_32 },
    { 0x36efac1, SUC_lexicon_tags_7 },
    { 0x52efabf, SUC_lexicon_tags_14 },
    { 0xe78efac3, SUC_lexicon_tags_9 },
    { 0xa72efac1, SUC_lexicon_tags_7 },
    { 0x14aefac3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x83eefac8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa32efaca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4efad4, SUC_lexicon_tags_25 },
    { 0x6ecefad5, SUC_lexicon_tags_17 },
    { 0xcb4efad6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeefad9, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xc46efadb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb84efadd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x864efae0, SUC_lexicon_tags_79 },
    { 0xd10efae1, SUC_lexicon_tags_38 },
    { 0x214efae1, SUC_lexicon_tags_21 },
    { 0xbaaefae3, SUC_lexicon_tags_11 },
    { 0x2e8efae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28efae8, SUC_lexicon_tags_2 },
    { 0x556efae9, SUC_lexicon_tags_25 },
    { 0x87aefaea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x484efaec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2aefaef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94efaf4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x900efaf6, SUC_lexicon_tags_17 },
    { 0xe26efaf7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf68efaf9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24efb05, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaefb08, SUC_lexicon_tags_10 },
    { 0x4f8efb09, SUC_lexicon_tags_37 },
    { 0x7ceefb0a, SUC_lexicon_tags_87 },
    { 0x1c6efb0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd66efb0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcefb16, SUC_lexicon_tags_7 },
    { 0x122efb16, SUC_lexicon_tags_23 },
    { 0x886efb17, SUC_lexicon_tags_25 },
    { 0x760efb16, SUC_lexicon_tags_33 },
    { 0x9eaefb1a, SUC_lexicon_tags_27 },
    { 0xff2efb1b, SUC_lexicon_tags_10 },
    { 0x22cefb1c, SUC_lexicon_tags_15 },
    { 0xff2efb1d, SUC_lexicon_tags_15 },
    { 0x33eefb1c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb0aefb20, SUC_lexicon_tags_248 },
    { 0x26cefb21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x584efb24, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4ecefb26, SUC_lexicon_tags_15 },
    { 0x8c4efb26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x376efb29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eefb2f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0efb39, SUC_lexicon_tags_17 },
    { 0x28aefb39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x930efb3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56eefb42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeefb46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1aaefb48, SUC_lexicon_tags_37 },
    { 0x77eefb49, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x79aefb4c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x540efb4f, SUC_lexicon_tags_21 },
    { 0x254efb4f, SUC_lexicon_tags_15 },
    { 0xd66efb51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8efb57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf4efb59, SUC_lexicon_tags_1 },
    { 0xe32efb5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cefb5d, SUC_lexicon_tags_45 },
    { 0xa9aefb5e, SUC_lexicon_tags_212 },
    { 0xf8cefb5f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4efb69, SUC_lexicon_tags_27 },
    { 0x856efb6a, SUC_lexicon_tags_7 },
    { 0xd60efb69, SUC_lexicon_tags_27 },
    { 0x4a6efb6c, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xb92efb6e, SUC_lexicon_tags_7 },
    { 0xa24efb6f, SUC_lexicon_tags_16 },
    { 0x9c8efb6f, SUC_lexicon_tags_29 },
    { 0x910efb70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0efb75, SUC_lexicon_tags_23 },
    { 0x7a2efb76, SUC_lexicon_tags_47 },
    { 0xb7aefb76, SUC_lexicon_tags_1 },
    { 0x382efb78, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34efb7d, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x29aefb7f, SUC_lexicon_tags_23 },
    { 0x3ceefb7f, SUC_lexicon_tags_17 },
    { 0x6eaefb81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50aefb89, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf12efb8b, SUC_lexicon_tags_14 },
    { 0xd44efb8c, SUC_lexicon_tags_35 },
    { 0xe88efb8d, SUC_lexicon_tags_11 },
    { 0x11eefb8e, SUC_lexicon_tags_37 },
    { 0x8d0efb8f, SUC_lexicon_tags_38 },
    { 0x44cefb90, SUC_lexicon_tags_1 },
    { 0x2a8efb8b, SUC_lexicon_tags_2 },
    { 0x624efb90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86efb96, SUC_lexicon_tags_38 },
    { 0x2caefb97, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x926efb9a, SUC_lexicon_tags_37 },
    { 0x38eefb9b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2efba3, SUC_lexicon_tags_23 },
    { 0x662efba4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cefba8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480efbac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x958efbae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994efbb4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x776efbb6, SUC_lexicon_tags_35 },
    { 0xf2efbb6, SUC_lexicon_tags_0 },
    { 0x520efbb8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f4efbba, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8aefbbe, SUC_lexicon_tags_84 },
    { 0xc70efbbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4efbc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x622efbc4, SUC_lexicon_tags_61 },
    { 0xe8cefbc5, SUC_lexicon_tags_17 },
    { 0x514efbc5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4efbc9, SUC_lexicon_tags_3 },
    { 0xa6efbca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4cefbcc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x384efbce, SUC_lexicon_tags_19 },
    { 0x88aefbce, SUC_lexicon_tags_48 },
    { 0xaeaefbce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0efbd4, SUC_lexicon_tags_11 },
    { 0x4f2efbd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724efbda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaefbe1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0aefbe3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1caefbe6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd76efbe8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6efbed, SUC_lexicon_tags_11 },
    { 0x1feefbed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67aefbf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98efbfa, SUC_lexicon_tags_9 },
    { 0xc08efbfa, SUC_lexicon_tags_1 },
    { 0x388efbfc, SUC_lexicon_tags_2 },
    { 0x98efbfb, SUC_lexicon_tags_12 },
    { 0xc58efbfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa42efc00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0efc0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d6efc0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cefc11, SUC_lexicon_tags_15 },
    { 0x94cefc11, SUC_lexicon_tags_30 },
    { 0x4b6efc13, SUC_lexicon_tags_7 },
    { 0x5ecefc13, SUC_lexicon_tags_11 },
    { 0x912efc15, SUC_lexicon_tags_15 },
    { 0xcd2efc15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10aefc19, SUC_lexicon_tags_34 },
    { 0x8d8efc19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecefc1d, SUC_lexicon_tags_17 },
    { 0xdaefc1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8efc23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0efc26, SUC_lexicon_tags_1 },
    { 0xcf8efc27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79aefc29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88efc2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0efc33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694efc38, SUC_lexicon_tags_12 },
    { 0x294efc39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb30efc3e, SUC_lexicon_tags_65 },
    { 0x4d4efc3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82cefc41, SUC_lexicon_tags_8 },
    { 0x63eefc41, SUC_lexicon_tags_17 },
    { 0xac6efc41, SUC_lexicon_tags_17 },
    { 0x2d2efc44, SUC_lexicon_tags_1 },
    { 0xb5cefc45, SUC_lexicon_tags_45 },
    { 0x838efc42, SUC_lexicon_tags_36 },
    { 0x79eefc45, SUC_lexicon_tags_149 },
    { 0xb0eefc48, SUC_lexicon_tags_6 },
    { 0xb3aefc49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38efc4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeefc4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eeefc54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe3eefc56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59cefc58, SUC_lexicon_tags_60 },
    { 0x26cefc59, SUC_lexicon_tags_12 },
    { 0xdfcefc58, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xaeefc5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1eefc61, SUC_lexicon_tags_15 },
    { 0x4d0efc62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c6efc64, SUC_lexicon_tags_11 },
    { 0x6d8efc64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4aefc68, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebeefc74, SUC_lexicon_tags_1 },
    { 0xdacefc74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8efc79, SUC_lexicon_tags_7 },
    { 0x764efc7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x646efc7d, SUC_lexicon_tags_37 },
    { 0x5acefc7d, SUC_lexicon_tags_66 },
    { 0x42efc7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5feefc82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4efc8b, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x244efc8d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xec6efc8f, SUC_lexicon_tags_25 },
    { 0x68cefc90, SUC_lexicon_tags_30 },
    { 0x126efc91, SUC_lexicon_tags_23 },
    { 0x7a6efc90, SUC_lexicon_tags_60 },
    { 0x3fcefc93, SUC_lexicon_tags_44 },
    { 0x20efc93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6aefc9b, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x3e8efc9d, SUC_lexicon_tags_50 },
    { 0x8feefc9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x114efca0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x20eefca2, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0x836efca4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae4efca6, SUC_lexicon_tags_37 },
    { 0x61eefca7, SUC_lexicon_tags_15 },
    { 0x7e2efca8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cefcac, SUC_lexicon_tags_7 },
    { 0x9c6efcad, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2efcb0, SUC_lexicon_tags_38 },
    { 0x302efcb1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7d2efcb3, SUC_lexicon_tags_19 },
    { 0x83eefcb3, SUC_lexicon_tags_63 },
    { 0xf7cefcb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc12efcb7, SUC_lexicon_tags_2 },
    { 0x4c0efcb8, SUC_lexicon_tags_47 },
    { 0x294efcb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966efcbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88efcc0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aefcc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30efcc9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd8efccb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116efcd1, SUC_lexicon_tags_20 },
    { 0x692efcd2, SUC_lexicon_tags_25 },
    { 0xf84efcd3, SUC_lexicon_tags_15 },
    { 0x6f4efcd3, SUC_lexicon_tags_118 },
    { 0x3d4efcd4, SUC_lexicon_tags_1 },
    { 0x9a2efcd6, SUC_lexicon_tags_15 },
    { 0xf9cefcd2, SUC_lexicon_tags_16 },
    { 0xf48efcd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56efcdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76eefce0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a2efce2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26efce8, SUC_lexicon_tags_15 },
    { 0xa62efce8, SUC_lexicon_tags_7 },
    { 0x4aeefce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x108efcec, SUC_lexicon_tags_55 },
    { 0x4d8efcec, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x19eefcef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa16efcf1, SUC_lexicon_tags_38 },
    { 0x42aefcf1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6fcefcf4, SUC_lexicon_tags_66 },
    { 0xa9cefcf5, SUC_lexicon_tags_37 },
    { 0xf70efcf6, SUC_lexicon_tags_1 },
    { 0x272efcf7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cefcfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef8efcfe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aefd06, SUC_lexicon_tags_0 },
    { 0xa04efd07, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6efd0a, SUC_lexicon_tags_47 },
    { 0xfeefd0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2efd0e, SUC_lexicon_tags_7 },
    { 0x30efd0f, SUC_lexicon_tags_29 },
    { 0xe28efd0f, SUC_lexicon_tags_28 },
    { 0x4c0efd11, SUC_lexicon_tags_12 },
    { 0x8b4efd11, SUC_lexicon_tags_37 },
    { 0xc44efd13, SUC_lexicon_tags_27 },
    { 0x464efd14, SUC_lexicon_tags_61 },
    { 0xf22efd15, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7bcefd17, SUC_lexicon_tags_3 },
    { 0x29eefd17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x200efd1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb6efd1e, SUC_lexicon_tags_15 },
    { 0x64eefd1f, SUC_lexicon_tags_25 },
    { 0x6caefd1f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cefd23, SUC_lexicon_tags_33 },
    { 0x310efd23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92cefd27, SUC_lexicon_tags_11 },
    { 0x1b2efd28, SUC_lexicon_tags_23 },
    { 0x2a2efd29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162efd2d, SUC_lexicon_tags_19 },
    { 0x874efd2d, SUC_lexicon_tags_25 },
    { 0xd0eefd2f, SUC_lexicon_tags_43 },
    { 0xb7eefd30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x570efd33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6efd3c, SUC_lexicon_tags_10 },
    { 0x39aefd3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94eefd43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150efd49, SUC_lexicon_tags_20 },
    { 0xd02efd4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96efd4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcefd53, SUC_lexicon_tags_215 },
    { 0x994efd53, SUC_lexicon_tags_63 },
    { 0x10aefd53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16efd59, SUC_lexicon_tags_9 },
    { 0x244efd5a, SUC_lexicon_tags_65 },
    { 0xf56efd5b, SUC_lexicon_tags_28 },
    { 0xe2efd5c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284efd61, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68efd64, SUC_lexicon_tags_23 },
    { 0x732efd65, SUC_lexicon_tags_47 },
    { 0x652efd65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb46efd68, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8d2efd6a, SUC_lexicon_tags_119 },
    { 0x336efd6b, SUC_lexicon_tags_148 },
    { 0xaacefd6c, SUC_lexicon_tags_23 },
    { 0xf28efd6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x874efd6f, SUC_lexicon_tags_25 },
    { 0xe32efd70, SUC_lexicon_tags_3 },
    { 0x78efd71, SUC_lexicon_tags_1 },
    { 0xd42efd6f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc1aefd74, SUC_lexicon_tags_43 },
    { 0xff6efd75, SUC_lexicon_tags_31 },
    { 0x45cefd74, SUC_lexicon_tags_25 },
    { 0x9e2efd74, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22efd79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02efd7d, SUC_lexicon_tags_11 },
    { 0xd78efd7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40aefd82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90eefd84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4efd88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7eefd8d, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x302efd8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a2efd91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x502efd94, SUC_lexicon_tags_15 },
    { 0xb74efd94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5eefd98, SUC_lexicon_tags_38 },
    { 0x1daefd99, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x558efd9c, SUC_lexicon_tags_14 },
    { 0xa38efd9d, SUC_lexicon_tags_25 },
    { 0xb32efd9e, SUC_lexicon_tags_33 },
    { 0xa12efd9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x852efda1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x170efda4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe18efda6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6efdae, SUC_lexicon_tags_11 },
    { 0xd5cefdaf, SUC_lexicon_tags_14 },
    { 0x4b4efdaf, SUC_lexicon_tags_11 },
    { 0x34cefdb1, SUC_lexicon_tags_82 },
    { 0x726efdb2, SUC_lexicon_tags_37 },
    { 0x5aefdb3, SUC_lexicon_tags_47 },
    { 0x66efdb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x664efdb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cefdbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x934efdc3, SUC_lexicon_tags_1 },
    { 0xb96efdc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0efdc7, SUC_lexicon_tags_25 },
    { 0x4e4efdc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb50efdca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x51eefdcd, SUC_lexicon_tags_26 },
    { 0x174efdce, SUC_lexicon_tags_12 },
    { 0x97aefdce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x356efdd2, SUC_lexicon_tags_25 },
    { 0x4b4efdd3, SUC_lexicon_tags_12 },
    { 0xce0efdd3, SUC_lexicon_tags_416 },
    { 0x4f8efdd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ecefdd7, SUC_lexicon_tags_49 },
    { 0x524efdd8, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x21cefdda, SUC_lexicon_tags_1 },
    { 0x844efddb, SUC_lexicon_tags_83 },
    { 0x23cefddc, SUC_lexicon_tags_25 },
    { 0xdb4efddb, SUC_lexicon_tags_8 },
    { 0xafcefddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1aefde0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36aefde2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x278efde5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xad0efde7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc7eefde9, SUC_lexicon_tags_11 },
    { 0x900efde9, SUC_lexicon_tags_38 },
    { 0x856efdeb, SUC_lexicon_tags_15 },
    { 0x40aefdea, SUC_lexicon_tags_0 },
    { 0x42efded, SUC_lexicon_tags_37 },
    { 0x5fcefdee, SUC_lexicon_tags_2 },
    { 0x11aefdef, SUC_lexicon_tags_6 },
    { 0xb32efdeb, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0xb3aefdf2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x698efdf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd26efdf7, SUC_lexicon_tags_2 },
    { 0xccefdf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00efdfc, SUC_lexicon_tags_25 },
    { 0xd96efdfc, SUC_lexicon_tags_49 },
    { 0x580efdfc, SUC_lexicon_tags_47 },
    { 0xc9cefdff, SUC_lexicon_tags_77 },
    { 0x376efe00, SUC_lexicon_tags_37 },
    { 0x984efdfc, SUC_lexicon_tags_31 },
    { 0x4c0efe02, SUC_lexicon_tags_12 },
    { 0x984efe03, SUC_lexicon_tags_712 },
    { 0xdc0efe04, SUC_lexicon_tags_12 },
    { 0xdeaefe03, SUC_lexicon_tags_11 },
    { 0x440efe06, SUC_lexicon_tags_7 },
    { 0x3d4efe07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x436efe0a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2efe0e, SUC_lexicon_tags_12 },
    { 0xfa0efe0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1deefe13, SUC_lexicon_tags_2 },
    { 0x3aaefe14, SUC_lexicon_tags_458 },
    { 0xe1cefe15, SUC_lexicon_tags_38 },
    { 0x472efe15, SUC_lexicon_tags_11 },
    { 0x400efe14, SUC_lexicon_tags_7 },
    { 0xde2efe18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdc6efe1a, SUC_lexicon_tags_12 },
    { 0x4efe1b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cefe1e, SUC_lexicon_tags_47 },
    { 0xd58efe1f, SUC_lexicon_tags_11 },
    { 0xd40efe20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27aefe22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434efe2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x634efe2e, SUC_lexicon_tags_0 },
    { 0x860efe2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840efe33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x142efe35, SUC_lexicon_tags_14 },
    { 0xfecefe36, SUC_lexicon_tags_3 },
    { 0xc68efe35, SUC_lexicon_tags_65 },
    { 0xce2efe37, SUC_lexicon_tags_2 },
    { 0x50aefe39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8efe3f, SUC_lexicon_tags_1 },
    { 0x5d8efe40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc74efe42, SUC_lexicon_tags_17 },
    { 0x8feefe43, SUC_lexicon_tags_43 },
    { 0x744efe43, SUC_lexicon_tags_15 },
    { 0x8d0efe43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1aefe4b, SUC_lexicon_tags_17 },
    { 0xe0efe4b, SUC_lexicon_tags_23 },
    { 0x314efe4b, SUC_lexicon_tags_60 },
    { 0xfa0efe4b, SUC_lexicon_tags_11 },
    { 0xa94efe4f, SUC_lexicon_tags_37 },
    { 0x6a8efe50, SUC_lexicon_tags_23 },
    { 0x2acefe51, SUC_lexicon_tags_21 },
    { 0x8baefe51, SUC_lexicon_tags_14 },
    { 0xbaefe4f, SUC_lexicon_tags_7 },
    { 0xe98efe54, SUC_lexicon_tags_16 },
    { 0xde0efe54, SUC_lexicon_tags_45 },
    { 0x50eefe53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8efe59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40efe5c, SUC_lexicon_tags_25 },
    { 0x76cefe5c, SUC_lexicon_tags_11 },
    { 0x3fcefe5e, SUC_lexicon_tags_15 },
    { 0x432efe5e, SUC_lexicon_tags_27 },
    { 0x1b0efe60, SUC_lexicon_tags_33 },
    { 0x9aeefe5f, SUC_lexicon_tags_87 },
    { 0x2f6efe62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf2efe64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e0efe66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x186efe69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7faefe6d, SUC_lexicon_tags_38 },
    { 0xb1cefe6d, SUC_lexicon_tags_1 },
    { 0xcaeefe6e, SUC_lexicon_tags_17 },
    { 0xd7eefe6d, SUC_lexicon_tags_19 },
    { 0x720efe71, SUC_lexicon_tags_37 },
    { 0xc86efe6f, SUC_lexicon_tags_21 },
    { 0x1c0efe73, SUC_lexicon_tags_74 },
    { 0x6beefe74, SUC_lexicon_tags_11 },
    { 0x3baefe6e, SUC_lexicon_tags_1 },
    { 0xeacefe76, SUC_lexicon_tags_47 },
    { 0xc06efe75, SUC_lexicon_tags_15 },
    { 0xa34efe78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x934efe7b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50efe81, SUC_lexicon_tags_37 },
    { 0x4b4efe81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31aefe84, SUC_lexicon_tags_32 },
    { 0x5acefe84, SUC_lexicon_tags_37 },
    { 0x96eefe84, SUC_lexicon_tags_79 },
    { 0x81eefe87, SUC_lexicon_tags_1 },
    { 0x8f0efe87, SUC_lexicon_tags_64 },
    { 0x994efe89, SUC_lexicon_tags_7 },
    { 0xe84efe89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x356efe8c, SUC_lexicon_tags_23 },
    { 0x944efe8d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x388efe8f, SUC_lexicon_tags_60 },
    { 0x5a8efe90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x978efe93, SUC_lexicon_tags_8 },
    { 0xa2eefe94, SUC_lexicon_tags_11 },
    { 0x466efe93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe14efe97, SUC_lexicon_tags_6 },
    { 0x48cefe98, SUC_lexicon_tags_1 },
    { 0x4c0efe98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384efea0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcf6efea2, SUC_lexicon_tags_12 },
    { 0x888efea3, SUC_lexicon_tags_23 },
    { 0xd3eefea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80eefea6, SUC_lexicon_tags_23 },
    { 0x3baefea7, SUC_lexicon_tags_47 },
    { 0x252efea7, SUC_lexicon_tags_65 },
    { 0x40eefea7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaefeb0, SUC_lexicon_tags_37 },
    { 0x82aefeb1, SUC_lexicon_tags_10 },
    { 0xe6eefeb0, SUC_lexicon_tags_1 },
    { 0x382efeb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8eefebb, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7beefec1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1e2efec3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaacefec6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63cefecb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce4efecd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eeefed0, SUC_lexicon_tags_47 },
    { 0xa5cefed0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aeefed4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36cefed6, SUC_lexicon_tags_2 },
    { 0x430efed7, SUC_lexicon_tags_60 },
    { 0xea4efed6, SUC_lexicon_tags_9 },
    { 0xe20efed7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56efee2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x848efee4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x87cefee6, SUC_lexicon_tags_17 },
    { 0xb62efee6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0efeea, SUC_lexicon_tags_61 },
    { 0xbfeefeeb, SUC_lexicon_tags_25 },
    { 0x2baefeeb, SUC_lexicon_tags_17 },
    { 0x7dcefeed, SUC_lexicon_tags_17 },
    { 0x474efeee, SUC_lexicon_tags_3 },
    { 0xedaefeec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50efef3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68efef6, SUC_lexicon_tags_25 },
    { 0x764efef6, SUC_lexicon_tags_1191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98efefb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc9cefefd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x254efeff, SUC_lexicon_tags_14 },
    { 0xae6eff00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06eff0c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a4eff0e, SUC_lexicon_tags_49 },
    { 0x628eff0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ceff12, SUC_lexicon_tags_17 },
    { 0x780eff13, SUC_lexicon_tags_47 },
    { 0xb1eeff14, SUC_lexicon_tags_37 },
    { 0xd94eff14, SUC_lexicon_tags_42 },
    { 0x204eff13, SUC_lexicon_tags_33 },
    { 0xd0aeff17, SUC_lexicon_tags_37 },
    { 0xf48eff18, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8daeff1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93aeff1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ceff23, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8eff26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964eff2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06eff2e, SUC_lexicon_tags_27 },
    { 0xfbceff2e, SUC_lexicon_tags_14 },
    { 0x58eeff30, SUC_lexicon_tags_53 },
    { 0xa8eeff31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820eff35, SUC_lexicon_tags_45 },
    { 0xfa0eff36, SUC_lexicon_tags_12 },
    { 0xb48eff35, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc5eeff39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eeff3c, SUC_lexicon_tags_16 },
    { 0xa9aeff3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310eff42, SUC_lexicon_tags_15 },
    { 0xb02eff42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xddceff45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0eff49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2eff4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2daeff51, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ceff57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x51aeff59, SUC_lexicon_tags_2 },
    { 0x71eeff5a, SUC_lexicon_tags_11 },
    { 0x592eff5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x86eeff5e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646eff62, SUC_lexicon_tags_11 },
    { 0x8ceeff63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36eff67, SUC_lexicon_tags_31 },
    { 0xc76eff67, SUC_lexicon_tags_1 },
    { 0x45ceff69, SUC_lexicon_tags_29 },
    { 0x36eff69, SUC_lexicon_tags_49 },
    { 0x370eff6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aceff70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaeff74, SUC_lexicon_tags_1 },
    { 0x8a2eff75, SUC_lexicon_tags_12 },
    { 0x9baeff76, SUC_lexicon_tags_27 },
    { 0xd7aeff77, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aaeff7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cceff7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2eff85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eceff88, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370eff8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbceff90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664eff95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x736eff98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36eff9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd74eff9e, SUC_lexicon_tags_11 },
    { 0x992eff9f, SUC_lexicon_tags_1 },
    { 0x324effa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40effa3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaeffa6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xee8effa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x722effaa, SUC_lexicon_tags_29 },
    { 0xa50effaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364effb0, SUC_lexicon_tags_32 },
    { 0x104effb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe20effb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0effb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8eeffbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98effca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x138effcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaeeffcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x442effd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd16effd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aeffda, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xed4effdc, SUC_lexicon_tags_23 },
    { 0x4ceeffdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2effe0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4effe3, SUC_lexicon_tags_26 },
    { 0x556effe3, SUC_lexicon_tags_1 },
    { 0x190effe3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e6effe7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95eefff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1caefff3, SUC_lexicon_tags_14 },
    { 0x994efff4, SUC_lexicon_tags_91 },
    { 0xf2efff5, SUC_lexicon_tags_15 },
    { 0x3bcefff4, SUC_lexicon_tags_17 },
    { 0xf30efff7, SUC_lexicon_tags_32 },
    { 0x60aefff7, SUC_lexicon_tags_15 },
    { 0x54cefff9, SUC_lexicon_tags_26 },
    { 0xda6efffa, SUC_lexicon_tags_7 },
    { 0x4aaefffb, SUC_lexicon_tags_23 },
    { 0xd2aefffa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40f0003, SUC_lexicon_tags_37 },
    { 0x42f0004, SUC_lexicon_tags_79 },
    { 0x718f0005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792f000a, SUC_lexicon_tags_1 },
    { 0x6a6f000a, SUC_lexicon_tags_21 },
    { 0x1fcf000a, SUC_lexicon_tags_37 },
    { 0xb5cf000d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa72f000f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6f0013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x640f0015, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ef0018, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x24cf001a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbef001f, SUC_lexicon_tags_25 },
    { 0x410f001f, SUC_lexicon_tags_217 },
    { 0xeeef0021, SUC_lexicon_tags_108 },
    { 0x156f0022, SUC_lexicon_tags_103 },
    { 0x48ef0023, SUC_lexicon_tags_27 },
    { 0x93af0024, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x446f0027, SUC_lexicon_tags_31 },
    { 0x26ef0027, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2f0030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd30f0032, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9d8f0034, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x49cf0036, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x736f0038, SUC_lexicon_tags_0 },
    { 0x2a2f0038, SUC_lexicon_tags_37 },
    { 0xab2f003a, SUC_lexicon_tags_45 },
    { 0x6acf003a, SUC_lexicon_tags_12 },
    { 0xecef003a, SUC_lexicon_tags_38 },
    { 0xb42f0039, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe24f003f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52f0042, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74af004a, SUC_lexicon_tags_6 },
    { 0x348f004b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa38f004d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc2f004f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460f0054, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4c0f0056, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x998f0058, SUC_lexicon_tags_47 },
    { 0x7b6f0059, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42f005c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66af005e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04f0062, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdef0066, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ef006d, SUC_lexicon_tags_87 },
    { 0x1baf006d, SUC_lexicon_tags_33 },
    { 0xe26f006e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0f0072, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x200f0075, SUC_lexicon_tags_49 },
    { 0xb20f0076, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f2f0078, SUC_lexicon_tags_1 },
    { 0xafcf0079, SUC_lexicon_tags_37 },
    { 0xc9af007a, SUC_lexicon_tags_37 },
    { 0x402f007b, SUC_lexicon_tags_1 },
    { 0xcdaf0078, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7cf007e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504f0082, SUC_lexicon_tags_21 },
    { 0x248f0083, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x982f0085, SUC_lexicon_tags_11 },
    { 0x9d6f0086, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaf0089, SUC_lexicon_tags_29 },
    { 0x240f008a, SUC_lexicon_tags_7 },
    { 0x4baf008b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x98cf008e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26ef0090, SUC_lexicon_tags_12 },
    { 0x4bcf0091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92f0098, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74f009c, SUC_lexicon_tags_27 },
    { 0x948f009d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f00a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xebaf00a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d8f00a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25af00a7, SUC_lexicon_tags_1 },
    { 0x838f00a8, SUC_lexicon_tags_15 },
    { 0xa8af00a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2f00ac, SUC_lexicon_tags_17 },
    { 0xe56f00ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7ef00af, SUC_lexicon_tags_30 },
    { 0xad8f00b0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x716f00b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872f00ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1def00c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ef00ca, SUC_lexicon_tags_25 },
    { 0x708f00ca, SUC_lexicon_tags_25 },
    { 0xf12f00cb, SUC_lexicon_tags_27 },
    { 0x10af00cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ef00d1, SUC_lexicon_tags_12 },
    { 0x38cf00d2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf00db, SUC_lexicon_tags_69 },
    { 0x32cf00db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x560f00df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x896f00e2, SUC_lexicon_tags_40 },
    { 0x61af00e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x530f00e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcaf00e7, SUC_lexicon_tags_12 },
    { 0x6caf00e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802f00ee, SUC_lexicon_tags_15 },
    { 0x17af00ee, SUC_lexicon_tags_27 },
    { 0xbef00ee, SUC_lexicon_tags_37 },
    { 0x5eef00ef, SUC_lexicon_tags_23 },
    { 0x4a8f00f2, SUC_lexicon_tags_25 },
    { 0x62cf00f2, SUC_lexicon_tags_11 },
    { 0x574f00f4, SUC_lexicon_tags_94 },
    { 0x97ef00f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf00fb, SUC_lexicon_tags_29 },
    { 0x58ef00fc, SUC_lexicon_tags_17 },
    { 0x132f00fd, SUC_lexicon_tags_45 },
    { 0x9aaf00fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cf0103, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66f0108, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72f010b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6f0111, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548f0119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89af011c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x61ef011e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb60f0120, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cf0123, SUC_lexicon_tags_16 },
    { 0x654f0123, SUC_lexicon_tags_30 },
    { 0x9cf0123, SUC_lexicon_tags_63 },
    { 0x8d6f0123, SUC_lexicon_tags_21 },
    { 0xf92f0127, SUC_lexicon_tags_1 },
    { 0xd9cf0128, SUC_lexicon_tags_33 },
    { 0xf96f0127, SUC_lexicon_tags_2 },
    { 0x56af0128, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916f012f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa4ef0131, SUC_lexicon_tags_9 },
    { 0xcf2f0132, SUC_lexicon_tags_23 },
    { 0x2faf0131, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92cf013a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe22f013c, SUC_lexicon_tags_33 },
    { 0x610f013c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9c8f013f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf0143, SUC_lexicon_tags_1361 },
    { 0x7bef0144, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cf0147, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664f014f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0f0153, SUC_lexicon_tags_33 },
    { 0xdacf0154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ef0157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964f015c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1af0162, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef016c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c4f016e, SUC_lexicon_tags_37 },
    { 0x4e2f016f, SUC_lexicon_tags_17 },
    { 0x6f6f016e, SUC_lexicon_tags_53 },
    { 0xc92f0171, SUC_lexicon_tags_19 },
    { 0xfecf0172, SUC_lexicon_tags_27 },
    { 0x94ef0173, SUC_lexicon_tags_63 },
    { 0x632f0172, SUC_lexicon_tags_47 },
    { 0x328f0174, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x152f0179, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6f017c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0f0186, SUC_lexicon_tags_12 },
    { 0xb50f0187, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2daf018c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4f018f, SUC_lexicon_tags_2 },
    { 0xe64f0190, SUC_lexicon_tags_45 },
    { 0x38ef0191, SUC_lexicon_tags_37 },
    { 0x81af0192, SUC_lexicon_tags_15 },
    { 0x9f0f0193, SUC_lexicon_tags_63 },
    { 0x1d6f0194, SUC_lexicon_tags_2 },
    { 0xa68f0193, SUC_lexicon_tags_31 },
    { 0xf2ef0195, SUC_lexicon_tags_5 },
    { 0x308f0193, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcf019f, SUC_lexicon_tags_37 },
    { 0x662f01a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb98f01a2, SUC_lexicon_tags_7 },
    { 0x10cf01a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x45ef01a5, SUC_lexicon_tags_49 },
    { 0x53af01a6, SUC_lexicon_tags_47 },
    { 0xfd6f01a7, SUC_lexicon_tags_184 },
    { 0x7acf01a7, SUC_lexicon_tags_49 },
    { 0xcfcf01a8, SUC_lexicon_tags_5 },
    { 0x550f01aa, SUC_lexicon_tags_12 },
    { 0x82af01aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cf01ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x464f01b0, SUC_lexicon_tags_0 },
    { 0x674f01b1, SUC_lexicon_tags_45 },
    { 0xa18f01b1, SUC_lexicon_tags_62 },
    { 0xcbcf01b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x144f01b5, SUC_lexicon_tags_0 },
    { 0x7c6f01b5, SUC_lexicon_tags_0 },
    { 0x25cf01b7, SUC_lexicon_tags_37 },
    { 0x802f01b8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe5af01ba, SUC_lexicon_tags_25 },
    { 0x16af01ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8f01c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe28f01c4, SUC_lexicon_tags_17 },
    { 0xa64f01c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x898f01c7, SUC_lexicon_tags_25 },
    { 0xdbcf01c8, SUC_lexicon_tags_2 },
    { 0x65cf01c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cf01cd, SUC_lexicon_tags_12 },
    { 0xe5af01cd, SUC_lexicon_tags_63 },
    { 0xd9ef01cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8f01d2, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58f01d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc6f01db, SUC_lexicon_tags_19 },
    { 0xfacf01dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8f01e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x47cf01ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680f01f1, SUC_lexicon_tags_0 },
    { 0xb18f01f1, SUC_lexicon_tags_31 },
    { 0x286f01f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60f01f6, SUC_lexicon_tags_9 },
    { 0x6ccf01f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8f01fd, SUC_lexicon_tags_2 },
    { 0x22af01fe, SUC_lexicon_tags_77 },
    { 0x780f01fe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd76f0202, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0x724f0204, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0206, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28af0208, SUC_lexicon_tags_26 },
    { 0xd24f0209, SUC_lexicon_tags_15 },
    { 0x9a6f0208, SUC_lexicon_tags_17 },
    { 0xb56f020b, SUC_lexicon_tags_11 },
    { 0xdd4f020c, SUC_lexicon_tags_15 },
    { 0xc58f020b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff4f0213, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0f0216, SUC_lexicon_tags_31 },
    { 0xfb0f0216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8aef0219, SUC_lexicon_tags_35 },
    { 0x9a4f0219, SUC_lexicon_tags_1182 },
    { 0x798f0219, SUC_lexicon_tags_7 },
    { 0x64f021a, SUC_lexicon_tags_37 },
    { 0xe3af021c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b6f021f, SUC_lexicon_tags_25 },
    { 0x3eaf0220, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50f0223, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb12f0225, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97af0229, SUC_lexicon_tags_47 },
    { 0x5e4f0229, SUC_lexicon_tags_60 },
    { 0xdf0f022a, SUC_lexicon_tags_12 },
    { 0xe5ef022b, SUC_lexicon_tags_33 },
    { 0x416f022d, SUC_lexicon_tags_69 },
    { 0x814f022a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x96af0230, SUC_lexicon_tags_0 },
    { 0x66f0231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ef0234, SUC_lexicon_tags_15 },
    { 0xc0ef0235, SUC_lexicon_tags_15 },
    { 0xe6ef0234, SUC_lexicon_tags_37 },
    { 0x9ecf0237, SUC_lexicon_tags_37 },
    { 0x3b2f0237, SUC_lexicon_tags_92 },
    { 0x738f0239, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x706f023b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x392f023d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4f0240, SUC_lexicon_tags_17 },
    { 0xb0f0240, SUC_lexicon_tags_11 },
    { 0x616f0242, SUC_lexicon_tags_61 },
    { 0x5b6f0243, SUC_lexicon_tags_17 },
    { 0x2a8f0243, SUC_lexicon_tags_25 },
    { 0xdc2f0245, SUC_lexicon_tags_14 },
    { 0xfe6f0242, SUC_lexicon_tags_30 },
    { 0xefcf0246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c8f0249, SUC_lexicon_tags_27 },
    { 0xac4f0249, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x988f024c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118f0252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc36f0254, SUC_lexicon_tags_72 },
    { 0xb0f0255, SUC_lexicon_tags_17 },
    { 0xb38f0256, SUC_lexicon_tags_43 },
    { 0xad4f0257, SUC_lexicon_tags_2 },
    { 0xe4f0254, SUC_lexicon_tags_31 },
    { 0xdfef0259, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1b4f025b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506f0262, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0x342f0265, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4f0269, SUC_lexicon_tags_63 },
    { 0xb56f026a, SUC_lexicon_tags_32 },
    { 0x274f026b, SUC_lexicon_tags_49 },
    { 0xa94f026b, SUC_lexicon_tags_14 },
    { 0x2d2f026b, SUC_lexicon_tags_44 },
    { 0x5b6f026e, SUC_lexicon_tags_17 },
    { 0x30f026f, SUC_lexicon_tags_26 },
    { 0xf5cf026c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d0f0272, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacf0278, SUC_lexicon_tags_626 },
    { 0x670f0279, SUC_lexicon_tags_17 },
    { 0xfeef027a, SUC_lexicon_tags_87 },
    { 0xbdcf0279, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252f027f, SUC_lexicon_tags_11 },
    { 0x30cf0280, SUC_lexicon_tags_2 },
    { 0xba2f027f, SUC_lexicon_tags_14 },
    { 0xa16f0282, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4af0288, SUC_lexicon_tags_28 },
    { 0x428f0289, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570f0290, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3baf0292, SUC_lexicon_tags_19 },
    { 0x592f0292, SUC_lexicon_tags_47 },
    { 0x2d6f0294, SUC_lexicon_tags_2 },
    { 0xb68f0294, SUC_lexicon_tags_38 },
    { 0x4fef0296, SUC_lexicon_tags_1 },
    { 0xa98f0297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ef029c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cf02a2, SUC_lexicon_tags_79 },
    { 0x812f02a3, SUC_lexicon_tags_12 },
    { 0xd3af02a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f02a7, SUC_lexicon_tags_45 },
    { 0x96ef02a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fef02b2, SUC_lexicon_tags_47 },
    { 0xcb4f02b3, SUC_lexicon_tags_7 },
    { 0xdb2f02b4, SUC_lexicon_tags_3 },
    { 0x20f02b5, SUC_lexicon_tags_12 },
    { 0xe04f02b6, SUC_lexicon_tags_19 },
    { 0x862f02b7, SUC_lexicon_tags_1 },
    { 0xc7cf02b4, SUC_lexicon_tags_30 },
    { 0xc14f02b9, SUC_lexicon_tags_29 },
    { 0xbfef02b3, SUC_lexicon_tags_30 },
    { 0x1fef02bb, SUC_lexicon_tags_17 },
    { 0xd8f02bb, SUC_lexicon_tags_7 },
    { 0x4f8f02b3, SUC_lexicon_tags_72 },
    { 0xa18f02b7, SUC_lexicon_tags_1 },
    { 0xd5ef02b8, SUC_lexicon_tags_47 },
    { 0x960f02c0, SUC_lexicon_tags_28 },
    { 0xee8f02c1, SUC_lexicon_tags_31 },
    { 0x664f02c2, SUC_lexicon_tags_10 },
    { 0x312f02c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cf02c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaf02ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ef02ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x212f02d1, SUC_lexicon_tags_33 },
    { 0x89ef02d2, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6f02d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8af02d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32f02db, SUC_lexicon_tags_66 },
    { 0xabef02dc, SUC_lexicon_tags_37 },
    { 0xcf0f02db, SUC_lexicon_tags_17 },
    { 0x8d8f02de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736f02e4, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976f02e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ef02ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2faf02f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe50f02f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320f02f8, SUC_lexicon_tags_191 },
    { 0x904f02f9, SUC_lexicon_tags_457 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ef02ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf30f0301, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e8f0303, SUC_lexicon_tags_38 },
    { 0xea8f0303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5caf0306, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52f030b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd72f030e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cf0315, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6f0319, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x168f031b, SUC_lexicon_tags_129 },
    { 0xe2cf031c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1af0323, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1cef0325, SUC_lexicon_tags_11 },
    { 0xb1cf0326, SUC_lexicon_tags_43 },
    { 0x290f0327, SUC_lexicon_tags_15 },
    { 0xceef0326, SUC_lexicon_tags_37 },
    { 0x52f0326, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04f0333, SUC_lexicon_tags_9 },
    { 0x8c4f0333, SUC_lexicon_tags_37 },
    { 0x250f0334, SUC_lexicon_tags_38 },
    { 0xe8ef0336, SUC_lexicon_tags_33 },
    { 0x3b2f0333, SUC_lexicon_tags_7 },
    { 0x874f0334, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6f033d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x45af033f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ef0342, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb42f0344, SUC_lexicon_tags_15 },
    { 0x468f0344, SUC_lexicon_tags_17 },
    { 0xe70f0346, SUC_lexicon_tags_15 },
    { 0x57af0347, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed6f0349, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x97cf034b, SUC_lexicon_tags_17 },
    { 0xc0f034c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72f0351, SUC_lexicon_tags_11 },
    { 0x14ef0352, SUC_lexicon_tags_47 },
    { 0x672f0353, SUC_lexicon_tags_6 },
    { 0x71af0351, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6bef0356, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eaf0359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38f035c, SUC_lexicon_tags_19 },
    { 0x5fcf035d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x196f035f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcf0363, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692f036b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790f0371, SUC_lexicon_tags_60 },
    { 0x244f0371, SUC_lexicon_tags_49 },
    { 0x5c8f0373, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1af0376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524f037a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a0f037c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9af037f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89af0381, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb46f0383, SUC_lexicon_tags_95 },
    { 0x9b8f0384, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7faf0386, SUC_lexicon_tags_6 },
    { 0x576f0387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98f038d, SUC_lexicon_tags_1 },
    { 0x6d0f038d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80f0392, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd38f0394, SUC_lexicon_tags_38 },
    { 0x982f0395, SUC_lexicon_tags_15 },
    { 0xb4ef0396, SUC_lexicon_tags_38 },
    { 0xa90f0396, SUC_lexicon_tags_45 },
    { 0xecef0398, SUC_lexicon_tags_25 },
    { 0x1d4f0396, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81af039b, SUC_lexicon_tags_66 },
    { 0x76af039c, SUC_lexicon_tags_23 },
    { 0x426f039b, SUC_lexicon_tags_11 },
    { 0xb92f039b, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x192f03a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53cf03a2, SUC_lexicon_tags_26 },
    { 0xb9ef03a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcf03a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x874f03a9, SUC_lexicon_tags_12 },
    { 0x300f03a9, SUC_lexicon_tags_11 },
    { 0x458f03ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x842f03ae, SUC_lexicon_tags_37 },
    { 0x4f8f03af, SUC_lexicon_tags_17 },
    { 0x10cf03b0, SUC_lexicon_tags_5 },
    { 0xb02f03b0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46f03b9, SUC_lexicon_tags_61 },
    { 0xabcf03b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64af03bf, SUC_lexicon_tags_23 },
    { 0x8f8f03c0, SUC_lexicon_tags_37 },
    { 0x946f03c1, SUC_lexicon_tags_42 },
    { 0x456f03c2, SUC_lexicon_tags_38 },
    { 0xc16f03c1, SUC_lexicon_tags_34 },
    { 0x62f03c4, SUC_lexicon_tags_17 },
    { 0xc8cf03c4, SUC_lexicon_tags_32 },
    { 0x700f03c4, SUC_lexicon_tags_17 },
    { 0x4d4f03c7, SUC_lexicon_tags_2 },
    { 0x20af03c4, SUC_lexicon_tags_25 },
    { 0xfb0f03c4, SUC_lexicon_tags_2 },
    { 0xe90f03ca, SUC_lexicon_tags_19 },
    { 0x206f03c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa44f03cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2c2f03cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5e8f03d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa0f03d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30f03d6, SUC_lexicon_tags_55 },
    { 0xacef03d6, SUC_lexicon_tags_65 },
    { 0x318f03d7, SUC_lexicon_tags_12 },
    { 0x8c6f03d9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x542f03dc, SUC_lexicon_tags_1 },
    { 0x186f03dc, SUC_lexicon_tags_25 },
    { 0xee4f03de, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122f03e5, SUC_lexicon_tags_31 },
    { 0x2c6f03e6, SUC_lexicon_tags_33 },
    { 0xad8f03e6, SUC_lexicon_tags_0 },
    { 0x584f03e8, SUC_lexicon_tags_0 },
    { 0x78af03e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78f03ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50f03fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76f03ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaf0402, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1a2f0404, SUC_lexicon_tags_8 },
    { 0x36f0405, SUC_lexicon_tags_17 },
    { 0x830f0404, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16af040a, SUC_lexicon_tags_7 },
    { 0xee6f040b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72f0410, SUC_lexicon_tags_28 },
    { 0x57ef0410, SUC_lexicon_tags_40 },
    { 0xc4ef0412, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf82f0414, SUC_lexicon_tags_37 },
    { 0x5d2f0415, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x690f0417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672f041d, SUC_lexicon_tags_11 },
    { 0x890f041e, SUC_lexicon_tags_30 },
    { 0x6eaf041f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0af0423, SUC_lexicon_tags_5 },
    { 0x898f0424, SUC_lexicon_tags_35 },
    { 0x72f0425, SUC_lexicon_tags_11 },
    { 0xfb4f0424, SUC_lexicon_tags_1 },
    { 0x66f0425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x590f0429, SUC_lexicon_tags_47 },
    { 0x628f042a, SUC_lexicon_tags_37 },
    { 0x5a2f042b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b8f042d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0cf042f, SUC_lexicon_tags_30 },
    { 0xc2ef0430, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2f0434, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3e8f0436, SUC_lexicon_tags_17 },
    { 0x412f0437, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x472f0439, SUC_lexicon_tags_14 },
    { 0xb92f0439, SUC_lexicon_tags_61 },
    { 0x6c6f043b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6f0441, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cf0444, SUC_lexicon_tags_47 },
    { 0x63af0444, SUC_lexicon_tags_7 },
    { 0x6ecf0446, SUC_lexicon_tags_15 },
    { 0xeaef0446, SUC_lexicon_tags_1 },
    { 0x2c6f0444, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ef044d, SUC_lexicon_tags_6 },
    { 0x796f044d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2f0451, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf76f0453, SUC_lexicon_tags_11 },
    { 0x87af0453, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cf0459, SUC_lexicon_tags_60 },
    { 0x7baf045a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdaef045c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x818f045e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63af0465, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cf0468, SUC_lexicon_tags_33 },
    { 0xe26f0469, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e0f046b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71cf046f, SUC_lexicon_tags_32 },
    { 0xaf6f0470, SUC_lexicon_tags_2 },
    { 0x10f0470, SUC_lexicon_tags_25 },
    { 0xeeaf0471, SUC_lexicon_tags_29 },
    { 0x19cf0473, SUC_lexicon_tags_14 },
    { 0xdc6f0473, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cf0479, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78f047d, SUC_lexicon_tags_11 },
    { 0x71cf047e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46f0480, SUC_lexicon_tags_60 },
    { 0x80af0481, SUC_lexicon_tags_25 },
    { 0x1aef0482, SUC_lexicon_tags_11 },
    { 0xf72f0483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x630f0485, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6f0488, SUC_lexicon_tags_15 },
    { 0x9f6f0488, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x352f048b, SUC_lexicon_tags_14 },
    { 0xa06f048c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x26cf048e, SUC_lexicon_tags_14 },
    { 0x346f048f, SUC_lexicon_tags_47 },
    { 0xb0cf048e, SUC_lexicon_tags_25 },
    { 0x99cf048e, SUC_lexicon_tags_66 },
    { 0x7f2f0492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaf0498, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x43af049b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36f049e, SUC_lexicon_tags_21 },
    { 0xb08f049f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc46f04a1, SUC_lexicon_tags_11 },
    { 0x6f0f04a2, SUC_lexicon_tags_37 },
    { 0x64af04a2, SUC_lexicon_tags_0 },
    { 0x58f04a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaf04a7, SUC_lexicon_tags_25 },
    { 0x34ef04a7, SUC_lexicon_tags_25 },
    { 0xcd2f04a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x174f04ac, SUC_lexicon_tags_13 },
    { 0xba2f04ad, SUC_lexicon_tags_39 },
    { 0x33cf04ad, SUC_lexicon_tags_37 },
    { 0x36ef04ae, SUC_lexicon_tags_193 },
    { 0x41af04b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa00f04b2, SUC_lexicon_tags_26 },
    { 0x5f6f04b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26f04b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2f04bd, SUC_lexicon_tags_43 },
    { 0x1f8f04bd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x6f0f04c0, SUC_lexicon_tags_37 },
    { 0xe4f04c0, SUC_lexicon_tags_87 },
    { 0x548f04c2, SUC_lexicon_tags_15 },
    { 0x85ef04c3, SUC_lexicon_tags_16 },
    { 0x8f4f04c4, SUC_lexicon_tags_1 },
    { 0x66cf04c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2f04cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x42af04ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x734f04d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858f04d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc80f04db, SUC_lexicon_tags_45 },
    { 0x47ef04db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cf04e0, SUC_lexicon_tags_26 },
    { 0x992f04e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cf04e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x688f04eb, SUC_lexicon_tags_22 },
    { 0x21ef04ec, SUC_lexicon_tags_17 },
    { 0x802f04eb, SUC_lexicon_tags_33 },
    { 0xc6ef04ee, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdb2f04f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2daf04f3, SUC_lexicon_tags_15 },
    { 0xb16f04f4, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648f04f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ef04fd, SUC_lexicon_tags_17 },
    { 0x28cf04fd, SUC_lexicon_tags_83 },
    { 0x1fcf04ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450f0507, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbaf0509, SUC_lexicon_tags_11 },
    { 0x212f0509, SUC_lexicon_tags_60 },
    { 0xe78f050b, SUC_lexicon_tags_17 },
    { 0x78cf050b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38f0510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f0513, SUC_lexicon_tags_11 },
    { 0x532f0514, SUC_lexicon_tags_239 },
    { 0xab6f0514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x550f0517, SUC_lexicon_tags_0 },
    { 0xacf0518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d0f051a, SUC_lexicon_tags_26 },
    { 0x9d2f051a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x32af051d, SUC_lexicon_tags_1 },
    { 0xe9ef051e, SUC_lexicon_tags_17 },
    { 0x766f051e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74f0525, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64f0527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81ef0529, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x240f052b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa14f052d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x790f052f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb3af0531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0f0534, SUC_lexicon_tags_32 },
    { 0xa2af0535, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x780f0537, SUC_lexicon_tags_30 },
    { 0x812f0538, SUC_lexicon_tags_14 },
    { 0x666f0539, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xacef053b, SUC_lexicon_tags_37 },
    { 0x55cf053c, SUC_lexicon_tags_0 },
    { 0xa52f053d, SUC_lexicon_tags_15 },
    { 0xaf4f053e, SUC_lexicon_tags_28 },
    { 0xc04f053f, SUC_lexicon_tags_17 },
    { 0xd80f0540, SUC_lexicon_tags_9 },
    { 0x64f053d, SUC_lexicon_tags_37 },
    { 0x7f4f053d, SUC_lexicon_tags_47 },
    { 0x928f0543, SUC_lexicon_tags_17 },
    { 0xe94f0544, SUC_lexicon_tags_13 },
    { 0x232f053b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6af0549, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8def054c, SUC_lexicon_tags_59 },
    { 0xeeaf054c, SUC_lexicon_tags_0 },
    { 0x608f054e, SUC_lexicon_tags_121 },
    { 0x218f054f, SUC_lexicon_tags_17 },
    { 0xd62f054e, SUC_lexicon_tags_17 },
    { 0xd70f0551, SUC_lexicon_tags_33 },
    { 0x9c0f054d, SUC_lexicon_tags_37 },
    { 0x8b8f0553, SUC_lexicon_tags_17 },
    { 0xbcaf0553, SUC_lexicon_tags_7 },
    { 0x91af0555, SUC_lexicon_tags_36 },
    { 0x3d8f0556, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb04f0558, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cf055b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccf055e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x74af0561, SUC_lexicon_tags_15 },
    { 0x5fcf0562, SUC_lexicon_tags_17 },
    { 0x338f0563, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc26f0567, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f6f0569, SUC_lexicon_tags_39 },
    { 0x506f056a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574f056e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa6cf0570, SUC_lexicon_tags_27 },
    { 0xd66f0571, SUC_lexicon_tags_11 },
    { 0x2baf0571, SUC_lexicon_tags_2 },
    { 0xc7af0573, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132f0578, SUC_lexicon_tags_15 },
    { 0xb96f0579, SUC_lexicon_tags_1 },
    { 0x850f057a, SUC_lexicon_tags_7 },
    { 0xa72f057b, SUC_lexicon_tags_11 },
    { 0x810f057c, SUC_lexicon_tags_47 },
    { 0x3c2f057d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8def0584, SUC_lexicon_tags_19 },
    { 0x68ef0585, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518f058a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x10af058c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaf058f, SUC_lexicon_tags_25 },
    { 0x64cf0590, SUC_lexicon_tags_9 },
    { 0x6c0f0590, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xb44f0593, SUC_lexicon_tags_8 },
    { 0x198f0593, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f0598, SUC_lexicon_tags_25 },
    { 0xb00f0598, SUC_lexicon_tags_26 },
    { 0xad8f059a, SUC_lexicon_tags_11 },
    { 0x488f059b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9faf059d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9c0f059f, SUC_lexicon_tags_106 },
    { 0xe4f05a0, SUC_lexicon_tags_47 },
    { 0xec8f05a1, SUC_lexicon_tags_27 },
    { 0x31af05a2, SUC_lexicon_tags_38 },
    { 0x404f05a0, SUC_lexicon_tags_17 },
    { 0x1b4f05a4, SUC_lexicon_tags_29 },
    { 0x318f05a0, SUC_lexicon_tags_49 },
    { 0x5baf05a6, SUC_lexicon_tags_8 },
    { 0xe98f05a7, SUC_lexicon_tags_6 },
    { 0x3d2f05a6, SUC_lexicon_tags_11 },
    { 0xafcf05a0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8e6f05ab, SUC_lexicon_tags_27 },
    { 0x67af05ab, SUC_lexicon_tags_60 },
    { 0xef8f05ad, SUC_lexicon_tags_14 },
    { 0x94af05ae, SUC_lexicon_tags_1 },
    { 0x232f05ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaf05b2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xdf6f05b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf6f05b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48f05b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d8f05bb, SUC_lexicon_tags_46 },
    { 0x8ef05bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ef05c0, SUC_lexicon_tags_9 },
    { 0xe96f05c1, SUC_lexicon_tags_11 },
    { 0x118f05c2, SUC_lexicon_tags_15 },
    { 0xbd4f05c0, SUC_lexicon_tags_81 },
    { 0x41af05c3, SUC_lexicon_tags_10 },
    { 0xa6cf05c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef05c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2f05cc, SUC_lexicon_tags_12 },
    { 0xe0f05cc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x82af05d0, SUC_lexicon_tags_31 },
    { 0x1ef05d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f05d4, SUC_lexicon_tags_1 },
    { 0x51cf05d5, SUC_lexicon_tags_7 },
    { 0xf92f05d4, SUC_lexicon_tags_43 },
    { 0x918f05d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4f05da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eef05df, SUC_lexicon_tags_17 },
    { 0xb88f05e0, SUC_lexicon_tags_17 },
    { 0xe5af05e1, SUC_lexicon_tags_7 },
    { 0xedcf05df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cf05e5, SUC_lexicon_tags_2 },
    { 0xbc4f05e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f05eb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74f05ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4af05f0, SUC_lexicon_tags_45 },
    { 0xf9ef05f1, SUC_lexicon_tags_36 },
    { 0xe9cf05f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdccf05f4, SUC_lexicon_tags_25 },
    { 0xe14f05f4, SUC_lexicon_tags_15 },
    { 0xb38f05f6, SUC_lexicon_tags_12 },
    { 0x9ccf05f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54f05fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292f0601, SUC_lexicon_tags_2 },
    { 0xbeef0602, SUC_lexicon_tags_65 },
    { 0x424f0603, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceaf060d, SUC_lexicon_tags_17 },
    { 0x924f060e, SUC_lexicon_tags_12 },
    { 0x752f060f, SUC_lexicon_tags_17 },
    { 0x536f0610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3cf0612, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c2f0614, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04f0618, SUC_lexicon_tags_15 },
    { 0x37ef0619, SUC_lexicon_tags_38 },
    { 0x9caf061a, SUC_lexicon_tags_23 },
    { 0x9d4f061b, SUC_lexicon_tags_23 },
    { 0x448f061c, SUC_lexicon_tags_0 },
    { 0xb1af061b, SUC_lexicon_tags_15 },
    { 0x17af061b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a4f0620, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xf28f0622, SUC_lexicon_tags_47 },
    { 0x40cf0623, SUC_lexicon_tags_43 },
    { 0xbfef0624, SUC_lexicon_tags_7 },
    { 0x55ef0625, SUC_lexicon_tags_0 },
    { 0xeb8f0626, SUC_lexicon_tags_2 },
    { 0x612f0624, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ef062a, SUC_lexicon_tags_24 },
    { 0x3cf062b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6f0631, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa68f0633, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x85cf0635, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eef0638, SUC_lexicon_tags_26 },
    { 0x830f0639, SUC_lexicon_tags_17 },
    { 0x964f0639, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x188f063c, SUC_lexicon_tags_138 },
    { 0xb7af063d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9d6f063f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56f0641, SUC_lexicon_tags_33 },
    { 0x5bcf0642, SUC_lexicon_tags_7 },
    { 0x39ef0641, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cef0646, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x870f0648, SUC_lexicon_tags_27 },
    { 0x102f0649, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf40f064b, SUC_lexicon_tags_15 },
    { 0x792f064b, SUC_lexicon_tags_605 },
    { 0, NULL },
    { 0, NULL },
    { 0x302f064f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cf0655, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcf065c, SUC_lexicon_tags_11 },
    { 0x728f065d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa08f065f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x518f0662, SUC_lexicon_tags_17 },
    { 0x8b2f0663, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xbe6f0665, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x294f0667, SUC_lexicon_tags_46 },
    { 0x89cf0667, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99ef066a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bef066e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x572f0670, SUC_lexicon_tags_25 },
    { 0x8f2f0671, SUC_lexicon_tags_9 },
    { 0x48ef0672, SUC_lexicon_tags_27 },
    { 0xab8f0672, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcb2f0675, SUC_lexicon_tags_26 },
    { 0x8daf0676, SUC_lexicon_tags_11 },
    { 0xddcf0677, SUC_lexicon_tags_1 },
    { 0x2b4f0677, SUC_lexicon_tags_37 },
    { 0x10f0675, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfef067c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6f0684, SUC_lexicon_tags_37 },
    { 0x836f0685, SUC_lexicon_tags_15 },
    { 0x6af0684, SUC_lexicon_tags_1 },
    { 0xc2f0687, SUC_lexicon_tags_15 },
    { 0x620f0688, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceaf0695, SUC_lexicon_tags_25 },
    { 0x982f0696, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acf069c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x774f069f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7af06a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87af06a3, SUC_lexicon_tags_27 },
    { 0xbb2f06a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744f06a8, SUC_lexicon_tags_1 },
    { 0xc16f06a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66cf06ac, SUC_lexicon_tags_1 },
    { 0x842f06ad, SUC_lexicon_tags_7 },
    { 0x52cf06ae, SUC_lexicon_tags_43 },
    { 0xa0cf06ae, SUC_lexicon_tags_10 },
    { 0x98f06b0, SUC_lexicon_tags_6 },
    { 0x762f06b0, SUC_lexicon_tags_25 },
    { 0xdaef06ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x628f06b6, SUC_lexicon_tags_33 },
    { 0xbfcf06b6, SUC_lexicon_tags_38 },
    { 0x9e4f06b8, SUC_lexicon_tags_25 },
    { 0xc4f06b8, SUC_lexicon_tags_25 },
    { 0xa3ef06b7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x770f06bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180f06c1, SUC_lexicon_tags_25 },
    { 0x88cf06c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8f06c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56af06cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x246f06ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4f06d1, SUC_lexicon_tags_11 },
    { 0xb24f06d2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcd6f06d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90f06da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcf4f06dc, SUC_lexicon_tags_113 },
    { 0x40f06dd, SUC_lexicon_tags_15 },
    { 0x4f0f06dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x880f06e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabaf06e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x788f06e5, SUC_lexicon_tags_17 },
    { 0x1ef06e6, SUC_lexicon_tags_3 },
    { 0x446f06e5, SUC_lexicon_tags_22 },
    { 0x67af06e8, SUC_lexicon_tags_7 },
    { 0xa1ef06e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bcf06eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19af06ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0af06f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86f06f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccef06f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528f06fd, SUC_lexicon_tags_25 },
    { 0x178f06fe, SUC_lexicon_tags_21 },
    { 0x46cf06fe, SUC_lexicon_tags_17 },
    { 0xb3af0700, SUC_lexicon_tags_47 },
    { 0xcf2f0701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aef0704, SUC_lexicon_tags_30 },
    { 0x4baf0705, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc78f0707, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14f070b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216f070f, SUC_lexicon_tags_14 },
    { 0x90ef0710, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6f0714, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cf071b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x284f071f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cf0725, SUC_lexicon_tags_9 },
    { 0x9a4f0726, SUC_lexicon_tags_40 },
    { 0x8baf0727, SUC_lexicon_tags_46 },
    { 0x9fcf0726, SUC_lexicon_tags_23 },
    { 0xbcf0729, SUC_lexicon_tags_14 },
    { 0x2e8f072a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x406f072d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x326f0730, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cf0735, SUC_lexicon_tags_1 },
    { 0x1f8f0736, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2f0f0738, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ef073c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64f0740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788f0746, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda8f0748, SUC_lexicon_tags_27 },
    { 0x7a4f0749, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714f074d, SUC_lexicon_tags_60 },
    { 0xc2ef074d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38ef0750, SUC_lexicon_tags_55 },
    { 0xb7cf0750, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7af075d, SUC_lexicon_tags_45 },
    { 0x810f075e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9af0760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ef0765, SUC_lexicon_tags_141 },
    { 0xb3af0765, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x102f0768, SUC_lexicon_tags_8 },
    { 0x64cf0768, SUC_lexicon_tags_38 },
    { 0x9eef076a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8f076d, SUC_lexicon_tags_37 },
    { 0x32ef076d, SUC_lexicon_tags_11 },
    { 0xe22f076d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fef0772, SUC_lexicon_tags_8 },
    { 0xa26f0773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d6f0775, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678f077b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668f077f, SUC_lexicon_tags_15 },
    { 0x5cf077f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6f0784, SUC_lexicon_tags_38 },
    { 0x73cf0785, SUC_lexicon_tags_12 },
    { 0x5f2f0786, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xedaf0789, SUC_lexicon_tags_25 },
    { 0x2d6f078a, SUC_lexicon_tags_12 },
    { 0xadaf078a, SUC_lexicon_tags_33 },
    { 0xe88f078a, SUC_lexicon_tags_25 },
    { 0x16cf078a, SUC_lexicon_tags_15 },
    { 0x310f078e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08f0792, SUC_lexicon_tags_45 },
    { 0x82af0792, SUC_lexicon_tags_2 },
    { 0x358f0794, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f0799, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcf079d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10f07a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f07a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x154f07ab, SUC_lexicon_tags_1 },
    { 0x5eef07ac, SUC_lexicon_tags_12 },
    { 0x54ef07ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8f07b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x234f07b4, SUC_lexicon_tags_55 },
    { 0x7acf07b4, SUC_lexicon_tags_12 },
    { 0x474f07b5, SUC_lexicon_tags_7 },
    { 0x698f07b7, SUC_lexicon_tags_4 },
    { 0x140f07b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88af07c0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7c0f07c2, SUC_lexicon_tags_7 },
    { 0x62af07c2, SUC_lexicon_tags_17 },
    { 0xdbcf07c4, SUC_lexicon_tags_12 },
    { 0xf24f07c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f07c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6f07cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ef07d1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x538f07d6, SUC_lexicon_tags_17 },
    { 0x5cf07d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37af07dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60f07e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330f07e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4f07ea, SUC_lexicon_tags_37 },
    { 0xb38f07eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90f07ef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x34f07f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2c0f07f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x160f07f5, SUC_lexicon_tags_60 },
    { 0x4a0f07f6, SUC_lexicon_tags_47 },
    { 0x63cf07f6, SUC_lexicon_tags_48 },
    { 0x848f07f8, SUC_lexicon_tags_435 },
    { 0xa00f07f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330f07fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8f0806, SUC_lexicon_tags_8 },
    { 0x832f0807, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4f080a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b0f080c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500f0811, SUC_lexicon_tags_15 },
    { 0x464f0812, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86f0816, SUC_lexicon_tags_31 },
    { 0x72ef0817, SUC_lexicon_tags_46 },
    { 0x8ccf0816, SUC_lexicon_tags_9 },
    { 0xad8f0819, SUC_lexicon_tags_8 },
    { 0x2b0f0819, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418f081e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d0f0820, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcf0825, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb80f0827, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124f082e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcf0830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd0f0832, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ecf0834, SUC_lexicon_tags_55 },
    { 0xa32f0835, SUC_lexicon_tags_48 },
    { 0x8b8f0836, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf083b, SUC_lexicon_tags_14 },
    { 0xadaf083c, SUC_lexicon_tags_25 },
    { 0xac2f083d, SUC_lexicon_tags_15 },
    { 0xe3cf083d, SUC_lexicon_tags_17 },
    { 0xf80f083f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf0842, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ef0846, SUC_lexicon_tags_26 },
    { 0xcdaf0847, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444f084d, SUC_lexicon_tags_4 },
    { 0xe78f084e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf6f0850, SUC_lexicon_tags_63 },
    { 0xf5af0851, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x514f0853, SUC_lexicon_tags_9 },
    { 0x432f0853, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8baf0857, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ef085a, SUC_lexicon_tags_7 },
    { 0xcd2f085b, SUC_lexicon_tags_37 },
    { 0xe24f085c, SUC_lexicon_tags_2 },
    { 0x184f085d, SUC_lexicon_tags_17 },
    { 0xd5cf085c, SUC_lexicon_tags_26 },
    { 0x2d4f085f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd2f0861, SUC_lexicon_tags_23 },
    { 0x34f0862, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x832f0865, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdccf0867, SUC_lexicon_tags_1 },
    { 0xfd6f0867, SUC_lexicon_tags_37 },
    { 0x5fcf0869, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24cf086b, SUC_lexicon_tags_66 },
    { 0x8c4f086c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef086f, SUC_lexicon_tags_27 },
    { 0x680f0870, SUC_lexicon_tags_27 },
    { 0x488f086f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d8f0873, SUC_lexicon_tags_25 },
    { 0xb5af0874, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x4c4f0876, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8c2f0878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6f087c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcf0885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6f0887, SUC_lexicon_tags_31 },
    { 0x340f0888, SUC_lexicon_tags_65 },
    { 0xcf2f0887, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x36f088b, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88f0892, SUC_lexicon_tags_66 },
    { 0x208f0892, SUC_lexicon_tags_25 },
    { 0x336f0894, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea0f0896, SUC_lexicon_tags_11 },
    { 0xfd0f0897, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438f089c, SUC_lexicon_tags_27 },
    { 0xfa8f089d, SUC_lexicon_tags_37 },
    { 0x690f089d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f08a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924f08aa, SUC_lexicon_tags_25 },
    { 0xf66f08aa, SUC_lexicon_tags_1 },
    { 0xad8f08aa, SUC_lexicon_tags_7 },
    { 0x96ef08aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f08b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb8cf08b4, SUC_lexicon_tags_38 },
    { 0xa88f08b5, SUC_lexicon_tags_37 },
    { 0x1d4f08b6, SUC_lexicon_tags_7 },
    { 0xf74f08b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ef08bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x138f08bf, SUC_lexicon_tags_28 },
    { 0x408f08c0, SUC_lexicon_tags_47 },
    { 0x266f08c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ef08c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acf08cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x39cf08ce, SUC_lexicon_tags_0 },
    { 0x2daf08cf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcaf08d2, SUC_lexicon_tags_19 },
    { 0x2f8f08d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x830f08d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc6f08d7, SUC_lexicon_tags_4 },
    { 0x522f08d7, SUC_lexicon_tags_63 },
    { 0x230f08d9, SUC_lexicon_tags_1146 },
    { 0, NULL },
    { 0xa94f08db, SUC_lexicon_tags_17 },
    { 0xe8ef08dc, SUC_lexicon_tags_19 },
    { 0xdaf08dd, SUC_lexicon_tags_9 },
    { 0x3c6f08dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20cf08e0, SUC_lexicon_tags_47 },
    { 0x2b2f08e1, SUC_lexicon_tags_22 },
    { 0xaeaf08e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf08ee, SUC_lexicon_tags_2 },
    { 0xd6f08ef, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x848f08f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd72f08f3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7aaf08f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4f08f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6f08fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292f0901, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x160f0903, SUC_lexicon_tags_12 },
    { 0x98ef0904, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940f0909, SUC_lexicon_tags_47 },
    { 0xef6f090a, SUC_lexicon_tags_44 },
    { 0x64af090b, SUC_lexicon_tags_37 },
    { 0xcb6f090c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cf0913, SUC_lexicon_tags_7 },
    { 0x88ef0914, SUC_lexicon_tags_7 },
    { 0x8acf0915, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcf0918, SUC_lexicon_tags_7 },
    { 0x622f0918, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xd34f091b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x446f0928, SUC_lexicon_tags_26 },
    { 0xc56f0929, SUC_lexicon_tags_11 },
    { 0xbfaf092a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2f092d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602f0933, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16f0936, SUC_lexicon_tags_60 },
    { 0xef0937, SUC_lexicon_tags_10 },
    { 0x852f0938, SUC_lexicon_tags_14 },
    { 0x9baf0939, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd52f093b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ef0942, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4f0946, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa7cf0948, SUC_lexicon_tags_15 },
    { 0xd90f0948, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb50f094b, SUC_lexicon_tags_17 },
    { 0xfd4f094b, SUC_lexicon_tags_25 },
    { 0xaa0f094b, SUC_lexicon_tags_2 },
    { 0x57cf094d, SUC_lexicon_tags_37 },
    { 0xbc4f094f, SUC_lexicon_tags_8 },
    { 0x744f0950, SUC_lexicon_tags_17 },
    { 0xdbcf0950, SUC_lexicon_tags_25 },
    { 0x3d2f0952, SUC_lexicon_tags_29 },
    { 0x7af094f, SUC_lexicon_tags_29 },
    { 0xe5ef0954, SUC_lexicon_tags_28 },
    { 0xf88f0955, SUC_lexicon_tags_40 },
    { 0x8ef0956, SUC_lexicon_tags_32 },
    { 0xef0f0957, SUC_lexicon_tags_1344 },
    { 0x3bef0958, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x516f095a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bef0960, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f0965, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x888f0967, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6cf0969, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61af096d, SUC_lexicon_tags_118 },
    { 0xfeef096d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2f0974, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f0977, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbc6f0979, SUC_lexicon_tags_17 },
    { 0x2caf0979, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430f0980, SUC_lexicon_tags_17 },
    { 0x6b2f0981, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x352f0984, SUC_lexicon_tags_39 },
    { 0x744f0984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696f0989, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xed4f098b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x24ef098d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4f0990, SUC_lexicon_tags_37 },
    { 0x1d4f0990, SUC_lexicon_tags_11 },
    { 0x946f0991, SUC_lexicon_tags_1102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4f099a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61af099e, SUC_lexicon_tags_37 },
    { 0x8f6f099e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78f09a3, SUC_lexicon_tags_47 },
    { 0xd2f09a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8f09a7, SUC_lexicon_tags_1 },
    { 0x34af09a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa6ef09aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84f09ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38f09b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x828f09b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcd4f09b5, SUC_lexicon_tags_9 },
    { 0xd28f09b6, SUC_lexicon_tags_25 },
    { 0x530f09b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814f09bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x494f09bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x20af09c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9def09c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490f09ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9def09d0, SUC_lexicon_tags_44 },
    { 0xd6af09d0, SUC_lexicon_tags_32 },
    { 0x14f09d2, SUC_lexicon_tags_64 },
    { 0x186f09d1, SUC_lexicon_tags_15 },
    { 0xd9cf09d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f09da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c0f09dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698f09e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb30f09e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4ef09e7, SUC_lexicon_tags_37 },
    { 0x2eef09e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb26f09eb, SUC_lexicon_tags_55 },
    { 0x4b6f09ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc6f09ee, SUC_lexicon_tags_38 },
    { 0x4a0f09ef, SUC_lexicon_tags_2 },
    { 0xabef09ef, SUC_lexicon_tags_33 },
    { 0x3e4f09f1, SUC_lexicon_tags_12 },
    { 0xf78f09f2, SUC_lexicon_tags_2 },
    { 0xee8f09f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cf09f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7af09fc, SUC_lexicon_tags_17 },
    { 0xc36f09fc, SUC_lexicon_tags_0 },
    { 0x78af09fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd2f0a00, SUC_lexicon_tags_33 },
    { 0x540f0a01, SUC_lexicon_tags_12 },
    { 0xb6f0a02, SUC_lexicon_tags_45 },
    { 0x692f0a01, SUC_lexicon_tags_29 },
    { 0x3d0f0a04, SUC_lexicon_tags_17 },
    { 0xbc6f0a05, SUC_lexicon_tags_43 },
    { 0x158f0a06, SUC_lexicon_tags_28 },
    { 0x2b4f0a07, SUC_lexicon_tags_23 },
    { 0x4f2f0a08, SUC_lexicon_tags_28 },
    { 0xf3cf0a08, SUC_lexicon_tags_33 },
    { 0x2cf0a0a, SUC_lexicon_tags_7 },
    { 0x73ef0a0b, SUC_lexicon_tags_51 },
    { 0xc1af0a0c, SUC_lexicon_tags_82 },
    { 0x166f0a02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xedaf0a10, SUC_lexicon_tags_25 },
    { 0x77cf0a11, SUC_lexicon_tags_60 },
    { 0xe80f0a12, SUC_lexicon_tags_5 },
    { 0xe6ef0a13, SUC_lexicon_tags_9 },
    { 0xed2f0a13, SUC_lexicon_tags_22 },
    { 0x8b8f0a15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x260f0a17, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fef0a1c, SUC_lexicon_tags_60 },
    { 0x978f0a1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29cf0a20, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdecf0a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28f0a27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0f0a2a, SUC_lexicon_tags_37 },
    { 0x716f0a2a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4f0a30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf0a33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f0a37, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x482f0a39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd1cf0a3b, SUC_lexicon_tags_25 },
    { 0x94f0a3b, SUC_lexicon_tags_14 },
    { 0xc66f0a3b, SUC_lexicon_tags_45 },
    { 0x444f0a3e, SUC_lexicon_tags_63 },
    { 0xbc4f0a3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424f0a43, SUC_lexicon_tags_66 },
    { 0x8cef0a44, SUC_lexicon_tags_57 },
    { 0x412f0a44, SUC_lexicon_tags_2 },
    { 0x208f0a45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcd6f0a48, SUC_lexicon_tags_17 },
    { 0x46ef0a49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66af0a4b, SUC_lexicon_tags_82 },
    { 0xf1af0a4c, SUC_lexicon_tags_2 },
    { 0xe90f0a4d, SUC_lexicon_tags_47 },
    { 0xcecf0a4c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6c8f0a50, SUC_lexicon_tags_15 },
    { 0x166f0a50, SUC_lexicon_tags_12 },
    { 0x4e4f0a52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cf0a55, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc48f0a58, SUC_lexicon_tags_160 },
    { 0xeeaf0a58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcf0a5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57ef0a5e, SUC_lexicon_tags_27 },
    { 0xe5ef0a5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0af0a61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x19cf0a63, SUC_lexicon_tags_25 },
    { 0x9f8f0a63, SUC_lexicon_tags_47 },
    { 0x578f0a65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x24cf0a67, SUC_lexicon_tags_15 },
    { 0xedcf0a68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aaf0a6d, SUC_lexicon_tags_17 },
    { 0x682f0a6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x696f0a70, SUC_lexicon_tags_63 },
    { 0x766f0a71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8f0a7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc14f0a80, SUC_lexicon_tags_49 },
    { 0x95cf0a81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660f0a85, SUC_lexicon_tags_17 },
    { 0x72af0a85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad8f0a88, SUC_lexicon_tags_14 },
    { 0x36cf0a89, SUC_lexicon_tags_6 },
    { 0x100f0a89, SUC_lexicon_tags_12 },
    { 0x912f0a89, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x190f0a8e, SUC_lexicon_tags_25 },
    { 0x2eef0a8f, SUC_lexicon_tags_11 },
    { 0x8eaf0a90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cf0a94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cf0a9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b8f0a9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1daf0aa0, SUC_lexicon_tags_18 },
    { 0xa56f0aa1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa9cf0aa3, SUC_lexicon_tags_45 },
    { 0xae4f0aa4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808f0aa8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x59cf0aaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0ab2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82f0ab4, SUC_lexicon_tags_0 },
    { 0xfc2f0ab5, SUC_lexicon_tags_12 },
    { 0xde0f0ab4, SUC_lexicon_tags_7 },
    { 0xfb8f0ab7, SUC_lexicon_tags_0 },
    { 0x990f0ab8, SUC_lexicon_tags_47 },
    { 0x5b2f0ab8, SUC_lexicon_tags_7 },
    { 0xb8f0ab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42f0ac0, SUC_lexicon_tags_28 },
    { 0x702f0ac0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22f0ac4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0f0acb, SUC_lexicon_tags_37 },
    { 0xcbcf0acb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10f0acf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x932f0ad2, SUC_lexicon_tags_12 },
    { 0x8acf0ad2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1e4f0ad5, SUC_lexicon_tags_37 },
    { 0xf46f0ad6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbf2f0ad8, SUC_lexicon_tags_15 },
    { 0x690f0ad9, SUC_lexicon_tags_37 },
    { 0xc10f0ada, SUC_lexicon_tags_16 },
    { 0x4e2f0ad9, SUC_lexicon_tags_2 },
    { 0x342f0ad9, SUC_lexicon_tags_1 },
    { 0xfdef0add, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb00f0adf, SUC_lexicon_tags_60 },
    { 0xad4f0ae0, SUC_lexicon_tags_62 },
    { 0x576f0adf, SUC_lexicon_tags_14 },
    { 0x2b4f0ae2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52f0ae6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x600f0ae8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x222f0aeb, SUC_lexicon_tags_19 },
    { 0xa5cf0aeb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cf0af1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a6f0af3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3aef0af5, SUC_lexicon_tags_33 },
    { 0xb08f0af5, SUC_lexicon_tags_1 },
    { 0x818f0af5, SUC_lexicon_tags_7 },
    { 0x88f0af8, SUC_lexicon_tags_69 },
    { 0x95ef0af7, SUC_lexicon_tags_65 },
    { 0x568f0afa, SUC_lexicon_tags_38 },
    { 0x844f0afb, SUC_lexicon_tags_12 },
    { 0x166f0afc, SUC_lexicon_tags_0 },
    { 0x3cf0afd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56f0b09, SUC_lexicon_tags_15 },
    { 0xf1ef0b0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6f0b10, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d8f0b12, SUC_lexicon_tags_8 },
    { 0x366f0b13, SUC_lexicon_tags_1 },
    { 0xee2f0b14, SUC_lexicon_tags_25 },
    { 0xd0ef0b15, SUC_lexicon_tags_11 },
    { 0xe80f0b15, SUC_lexicon_tags_37 },
    { 0xdef0b16, SUC_lexicon_tags_1 },
    { 0x336f0b18, SUC_lexicon_tags_0 },
    { 0x45cf0b19, SUC_lexicon_tags_43 },
    { 0xe8cf0b18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xca4f0b1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f0b20, SUC_lexicon_tags_12 },
    { 0xbc8f0b21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0f0b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10af0b31, SUC_lexicon_tags_37 },
    { 0x19ef0b32, SUC_lexicon_tags_26 },
    { 0xe60f0b32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0f0b36, SUC_lexicon_tags_33 },
    { 0x2acf0b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x720f0b39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x88cf0b3c, SUC_lexicon_tags_31 },
    { 0x6aaf0b3d, SUC_lexicon_tags_25 },
    { 0x8eaf0b3c, SUC_lexicon_tags_12 },
    { 0x646f0b3f, SUC_lexicon_tags_0 },
    { 0x576f0b3c, SUC_lexicon_tags_45 },
    { 0x48f0b41, SUC_lexicon_tags_56 },
    { 0xbe0f0b42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa02f0b45, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x700f0b48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xee4f0b4b, SUC_lexicon_tags_40 },
    { 0xb3af0b4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ef0b51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b0f0b53, SUC_lexicon_tags_27 },
    { 0xc02f0b54, SUC_lexicon_tags_11 },
    { 0xe86f0b53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6f0b59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ef0b5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6f0b5f, SUC_lexicon_tags_15 },
    { 0xc22f0b5f, SUC_lexicon_tags_7 },
    { 0x338f0b5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb94f0b63, SUC_lexicon_tags_208 },
    { 0x358f0b64, SUC_lexicon_tags_87 },
    { 0x27ef0b63, SUC_lexicon_tags_55 },
    { 0x360f0b64, SUC_lexicon_tags_60 },
    { 0x53cf0b65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20f0b6a, SUC_lexicon_tags_17 },
    { 0xcacf0b6a, SUC_lexicon_tags_66 },
    { 0x344f0b6a, SUC_lexicon_tags_7 },
    { 0xa42f0b6b, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0f0b72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbef0b76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0b7b, SUC_lexicon_tags_14 },
    { 0x7f6f0b7b, SUC_lexicon_tags_7 },
    { 0x7c0f0b7d, SUC_lexicon_tags_40 },
    { 0xd50f0b7d, SUC_lexicon_tags_2 },
    { 0x31cf0b7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486f0b83, SUC_lexicon_tags_15 },
    { 0x1bef0b84, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x45ef0b86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46af0b8b, SUC_lexicon_tags_25 },
    { 0xcc4f0b8c, SUC_lexicon_tags_401 },
    { 0, NULL },
    { 0xff0f0b8e, SUC_lexicon_tags_7 },
    { 0x85cf0b8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8f0b94, SUC_lexicon_tags_45 },
    { 0x770f0b94, SUC_lexicon_tags_17 },
    { 0x9f8f0b96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x984f0b99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66f0b9c, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66f0b9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76af0ba1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xac2f0ba4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x360f0ba6, SUC_lexicon_tags_25 },
    { 0xcf8f0ba6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72af0bab, SUC_lexicon_tags_63 },
    { 0x3e6f0bac, SUC_lexicon_tags_30 },
    { 0xdd4f0bad, SUC_lexicon_tags_47 },
    { 0x9f8f0bae, SUC_lexicon_tags_33 },
    { 0xc18f0bae, SUC_lexicon_tags_60 },
    { 0xd9af0bac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cf0bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52cf0bbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104f0bc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cf0bc5, SUC_lexicon_tags_47 },
    { 0xcb4f0bc6, SUC_lexicon_tags_262 },
    { 0xb86f0bc7, SUC_lexicon_tags_1 },
    { 0x526f0bc7, SUC_lexicon_tags_7 },
    { 0x57ef0bc7, SUC_lexicon_tags_38 },
    { 0x52f0bc5, SUC_lexicon_tags_9 },
    { 0xd5ef0bcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe10f0bcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeef0bd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cf0bd6, SUC_lexicon_tags_21 },
    { 0xa7ef0bd7, SUC_lexicon_tags_1164 },
    { 0x5a4f0bd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef0bdb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30f0bde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f0be1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4f0be5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650f0beb, SUC_lexicon_tags_43 },
    { 0x980f0bec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8af0bee, SUC_lexicon_tags_28 },
    { 0x86f0bef, SUC_lexicon_tags_17 },
    { 0x7eef0bf0, SUC_lexicon_tags_0 },
    { 0x306f0bf0, SUC_lexicon_tags_1 },
    { 0x31af0bee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42f0bf5, SUC_lexicon_tags_38 },
    { 0x234f0bf6, SUC_lexicon_tags_17 },
    { 0xd48f0bf5, SUC_lexicon_tags_17 },
    { 0x8ecf0bf8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ecf0bfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218f0bfe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x13ef0c00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc4f0c02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56f0c0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f0c13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ef0c17, SUC_lexicon_tags_14 },
    { 0xacaf0c17, SUC_lexicon_tags_17 },
    { 0x34f0c17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93af0c1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaecf0c22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe28f0c24, SUC_lexicon_tags_8 },
    { 0xb82f0c24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeb4f0c27, SUC_lexicon_tags_61 },
    { 0xe2cf0c27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8f0c2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118f0c31, SUC_lexicon_tags_19 },
    { 0xc5ef0c32, SUC_lexicon_tags_11 },
    { 0x38af0c33, SUC_lexicon_tags_7 },
    { 0x2d8f0c33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f0c38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a2f0c3a, SUC_lexicon_tags_23 },
    { 0x1e6f0c3b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x208f0c3d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202f0c44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cf0c48, SUC_lexicon_tags_1 },
    { 0x17cf0c48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90f0c4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42f0c50, SUC_lexicon_tags_1 },
    { 0xd78f0c50, SUC_lexicon_tags_19 },
    { 0x944f0c51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ef0c57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62f0c5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46f0c5f, SUC_lexicon_tags_234 },
    { 0x85cf0c5f, SUC_lexicon_tags_1 },
    { 0xd8af0c61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cf0c64, SUC_lexicon_tags_42 },
    { 0xb10f0c64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf0c68, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x252f0c6a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x350f0c6c, SUC_lexicon_tags_36 },
    { 0x43af0c6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fef0c6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1f2f0c71, SUC_lexicon_tags_16 },
    { 0xd1af0c72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8f8f0c74, SUC_lexicon_tags_11 },
    { 0x3dcf0c74, SUC_lexicon_tags_19 },
    { 0x2bef0c76, SUC_lexicon_tags_2 },
    { 0xe7ef0c77, SUC_lexicon_tags_27 },
    { 0x2d2f0c78, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206f0c7d, SUC_lexicon_tags_25 },
    { 0xcb0f0c7d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xedaf0c80, SUC_lexicon_tags_11 },
    { 0xb52f0c81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6f0c85, SUC_lexicon_tags_12 },
    { 0xfb0f0c86, SUC_lexicon_tags_2 },
    { 0x276f0c87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbef0c90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x882f0c92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2f0c96, SUC_lexicon_tags_25 },
    { 0xe7ef0c97, SUC_lexicon_tags_63 },
    { 0xffaf0c98, SUC_lexicon_tags_2 },
    { 0x6a6f0c99, SUC_lexicon_tags_87 },
    { 0xfcef0c9a, SUC_lexicon_tags_60 },
    { 0xd1ef0c97, SUC_lexicon_tags_34 },
    { 0x992f0c9c, SUC_lexicon_tags_17 },
    { 0xf5af0c9b, SUC_lexicon_tags_15 },
    { 0x9d2f0c9e, SUC_lexicon_tags_53 },
    { 0xb92f0c9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x618f0ca2, SUC_lexicon_tags_4 },
    { 0x260f0ca3, SUC_lexicon_tags_11 },
    { 0x92ef0ca4, SUC_lexicon_tags_15 },
    { 0x1acf0ca5, SUC_lexicon_tags_16 },
    { 0xab0f0ca6, SUC_lexicon_tags_37 },
    { 0xf0ca7, SUC_lexicon_tags_27 },
    { 0x7dcf0ca7, SUC_lexicon_tags_19 },
    { 0x53af0ca6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x150f0cab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958f0cb2, SUC_lexicon_tags_2 },
    { 0xa3cf0cb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f0cb7, SUC_lexicon_tags_15 },
    { 0x748f0cb7, SUC_lexicon_tags_32 },
    { 0x224f0cb8, SUC_lexicon_tags_118 },
    { 0x548f0cb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x706f0cbc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8f0cc1, SUC_lexicon_tags_151 },
    { 0x974f0cc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x20af0cc4, SUC_lexicon_tags_1 },
    { 0x56f0cc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f0cc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acf0ccb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47af0cd1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954f0cd7, SUC_lexicon_tags_23 },
    { 0x64cf0cd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9caf0cdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74f0cdf, SUC_lexicon_tags_14 },
    { 0xb62f0cdf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2f0ce6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x38af0ce9, SUC_lexicon_tags_17 },
    { 0xa7cf0ce9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x616f0cec, SUC_lexicon_tags_1 },
    { 0xedaf0cec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0f0cf5, SUC_lexicon_tags_47 },
    { 0x506f0cf6, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0f0cfe, SUC_lexicon_tags_15 },
    { 0x676f0cff, SUC_lexicon_tags_27 },
    { 0x8d8f0d00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65cf0d02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cef0d05, SUC_lexicon_tags_10 },
    { 0x2f8f0d06, SUC_lexicon_tags_14 },
    { 0x38f0d07, SUC_lexicon_tags_2 },
    { 0xc9ef0d08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260f0d0d, SUC_lexicon_tags_15 },
    { 0x110f0d0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5fcf0d10, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa7af0d12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74f0d15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf0d19, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x284f0d1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3af0d23, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x984f0d26, SUC_lexicon_tags_19 },
    { 0x3cf0d26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa98f0d29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2f0d32, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x72cf0d34, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8f0d39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe02f0d3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2f0d45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cf0d49, SUC_lexicon_tags_17 },
    { 0xb0cf0d4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa12f0d4c, SUC_lexicon_tags_10 },
    { 0x280f0d4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7acf0d4f, SUC_lexicon_tags_1 },
    { 0xdef0d50, SUC_lexicon_tags_3 },
    { 0x574f0d51, SUC_lexicon_tags_25 },
    { 0xd8cf0d50, SUC_lexicon_tags_2 },
    { 0xf00f0d53, SUC_lexicon_tags_7 },
    { 0xff4f0d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdecf0d56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8f0d5a, SUC_lexicon_tags_7 },
    { 0x330f0d5b, SUC_lexicon_tags_17 },
    { 0xb4f0d5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cef0d5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8af0d61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0f0d66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58f0d6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xddef0d6e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeccf0d78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x726f0d7b, SUC_lexicon_tags_17 },
    { 0xab6f0d7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06f0d7f, SUC_lexicon_tags_8 },
    { 0xf1af0d80, SUC_lexicon_tags_49 },
    { 0xe8ef0d81, SUC_lexicon_tags_19 },
    { 0x1f8f0d82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620f0d8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaf0d94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x37af0d97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb90f0d99, SUC_lexicon_tags_23 },
    { 0xbcf0d9a, SUC_lexicon_tags_66 },
    { 0xf42f0d9a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08f0da0, SUC_lexicon_tags_15 },
    { 0xb8ef0da1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe24f0da3, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cf0da7, SUC_lexicon_tags_30 },
    { 0x998f0da7, SUC_lexicon_tags_15 },
    { 0xb16f0da9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4cf0dab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa34f0dad, SUC_lexicon_tags_34 },
    { 0x6b0f0dae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416f0db2, SUC_lexicon_tags_12 },
    { 0x33cf0db2, SUC_lexicon_tags_11 },
    { 0x55ef0db3, SUC_lexicon_tags_46 },
    { 0xb6af0db5, SUC_lexicon_tags_13 },
    { 0xf42f0db5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3acf0db9, SUC_lexicon_tags_26 },
    { 0x910f0dba, SUC_lexicon_tags_11 },
    { 0x2f0f0dbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fef0dbf, SUC_lexicon_tags_7 },
    { 0x9fef0dbf, SUC_lexicon_tags_25 },
    { 0x96cf0dc1, SUC_lexicon_tags_63 },
    { 0xf6f0dc1, SUC_lexicon_tags_49 },
    { 0xebcf0dc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30f0dc7, SUC_lexicon_tags_27 },
    { 0x724f0dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6f0dcc, SUC_lexicon_tags_11 },
    { 0xfd0f0dcc, SUC_lexicon_tags_21 },
    { 0xfe4f0dce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83af0dd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf10f0dd5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeaf0dd7, SUC_lexicon_tags_37 },
    { 0xc7cf0dd8, SUC_lexicon_tags_4 },
    { 0xf4f0dd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526f0de0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400f0dea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d4f0dec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x412f0dee, SUC_lexicon_tags_18 },
    { 0x322f0dee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x520f0df1, SUC_lexicon_tags_11 },
    { 0x852f0df2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12f0dfa, SUC_lexicon_tags_11 },
    { 0x190f0dfb, SUC_lexicon_tags_10 },
    { 0xf8f0dfc, SUC_lexicon_tags_8 },
    { 0xd7af0dfc, SUC_lexicon_tags_60 },
    { 0x6e4f0dfc, SUC_lexicon_tags_27 },
    { 0x44cf0dfe, SUC_lexicon_tags_7 },
    { 0x694f0dfd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x75cf0e02, SUC_lexicon_tags_37 },
    { 0x9f6f0e03, SUC_lexicon_tags_28 },
    { 0x2e0f0e03, SUC_lexicon_tags_5 },
    { 0x902f0e04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x390f0e08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2f0e0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62f0e14, SUC_lexicon_tags_49 },
    { 0x74f0e14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cf0e1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcf0e21, SUC_lexicon_tags_42 },
    { 0x9d0f0e22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4f0e27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1baf0e29, SUC_lexicon_tags_63 },
    { 0xcbcf0e2a, SUC_lexicon_tags_11 },
    { 0xe4ef0e2a, SUC_lexicon_tags_8 },
    { 0x326f0e2c, SUC_lexicon_tags_40 },
    { 0x144f0e2d, SUC_lexicon_tags_12 },
    { 0xf88f0e2d, SUC_lexicon_tags_30 },
    { 0xea8f0e2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ef0e35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfaf0e38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc08f0e3b, SUC_lexicon_tags_31 },
    { 0x600f0e3c, SUC_lexicon_tags_7 },
    { 0x9bcf0e3d, SUC_lexicon_tags_37 },
    { 0xf5af0e3b, SUC_lexicon_tags_46 },
    { 0x4b4f0e3c, SUC_lexicon_tags_25 },
    { 0x778f0e3e, SUC_lexicon_tags_15 },
    { 0xaa8f0e3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x836f0e43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x766f0e46, SUC_lexicon_tags_29 },
    { 0xe62f0e47, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa9cf0e49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe60f0e4b, SUC_lexicon_tags_15 },
    { 0x738f0e4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x806f0e4e, SUC_lexicon_tags_25 },
    { 0x4f0e4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a0f0e54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76af0e5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cf0e5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6f0e61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b6f0e63, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x49cf0e66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04f0e74, SUC_lexicon_tags_63 },
    { 0x276f0e74, SUC_lexicon_tags_32 },
    { 0x80af0e76, SUC_lexicon_tags_11 },
    { 0x57cf0e77, SUC_lexicon_tags_1 },
    { 0x880f0e77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cf0e7b, SUC_lexicon_tags_26 },
    { 0x860f0e7b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cef0e86, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714f0e8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8f0e91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56ef0e93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4acf0e95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f0f0e97, SUC_lexicon_tags_7 },
    { 0x9b0f0e98, SUC_lexicon_tags_61 },
    { 0x33ef0e98, SUC_lexicon_tags_19 },
    { 0xb84f0e9a, SUC_lexicon_tags_33 },
    { 0x238f0e9b, SUC_lexicon_tags_27 },
    { 0x3c4f0e9a, SUC_lexicon_tags_17 },
    { 0x7faf0e9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0f0ea3, SUC_lexicon_tags_23 },
    { 0x61cf0ea4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8f0ea8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62f0eac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34f0eb1, SUC_lexicon_tags_11 },
    { 0xfe2f0eb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf06f0eb4, SUC_lexicon_tags_25 },
    { 0x5eaf0eb5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe34f0eb7, SUC_lexicon_tags_60 },
    { 0xe62f0eb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ef0ebb, SUC_lexicon_tags_21 },
    { 0x98cf0ebc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa34f0ebe, SUC_lexicon_tags_47 },
    { 0xc00f0ebe, SUC_lexicon_tags_38 },
    { 0x270f0ebe, SUC_lexicon_tags_35 },
    { 0x69ef0ebf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd1cf0ec3, SUC_lexicon_tags_14 },
    { 0x662f0ec4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baf0ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cf0ecb, SUC_lexicon_tags_31 },
    { 0xd94f0ecc, SUC_lexicon_tags_19 },
    { 0x148f0ecd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa9af0ecf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2f0ed3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebef0ed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97ef0ed9, SUC_lexicon_tags_11 },
    { 0xb8cf0eda, SUC_lexicon_tags_38 },
    { 0x3e6f0edb, SUC_lexicon_tags_9 },
    { 0xbfaf0edc, SUC_lexicon_tags_38 },
    { 0xc90f0edb, SUC_lexicon_tags_11 },
    { 0xbcf0ede, SUC_lexicon_tags_31 },
    { 0x2e0f0ede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04f0ee5, SUC_lexicon_tags_47 },
    { 0x456f0ee6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x876f0ee8, SUC_lexicon_tags_12 },
    { 0xbfaf0ee9, SUC_lexicon_tags_17 },
    { 0x792f0ee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1cf0eec, SUC_lexicon_tags_63 },
    { 0x2cef0eec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236f0ef4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4f0eff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16f0f05, SUC_lexicon_tags_11 },
    { 0xdecf0f05, SUC_lexicon_tags_4 },
    { 0xff2f0f06, SUC_lexicon_tags_33 },
    { 0x97cf0f07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x380f0f0b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cf0f0e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0f0f14, SUC_lexicon_tags_2 },
    { 0xfa0f0f15, SUC_lexicon_tags_37 },
    { 0xb96f0f14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50af0f18, SUC_lexicon_tags_65 },
    { 0x812f0f18, SUC_lexicon_tags_34 },
    { 0x52cf0f18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeef0f1f, SUC_lexicon_tags_55 },
    { 0x504f0f20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbe6f0f22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0f0f28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6f0f2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280f0f30, SUC_lexicon_tags_27 },
    { 0x29af0f30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x960f0f33, SUC_lexicon_tags_17 },
    { 0xea0f0f34, SUC_lexicon_tags_12 },
    { 0xa60f0f34, SUC_lexicon_tags_136 },
    { 0x466f0f36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64af0f3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72ef0f3d, SUC_lexicon_tags_45 },
    { 0xca8f0f3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bef0f40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe2f0f42, SUC_lexicon_tags_26 },
    { 0xfccf0f42, SUC_lexicon_tags_25 },
    { 0xb30f0f43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b4f0f46, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x756f0f48, SUC_lexicon_tags_15 },
    { 0xa42f0f49, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98f0f4e, SUC_lexicon_tags_17 },
    { 0x82ef0f4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668f0f53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2cf0f55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26f0f57, SUC_lexicon_tags_11 },
    { 0x398f0f58, SUC_lexicon_tags_23 },
    { 0xafaf0f57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12cf0f5b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x6f0f0f5d, SUC_lexicon_tags_1 },
    { 0x562f0f5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4f0f61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfcf0f63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cf0f67, SUC_lexicon_tags_11 },
    { 0x5e0f0f68, SUC_lexicon_tags_12 },
    { 0x8c8f0f68, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa8af0f6b, SUC_lexicon_tags_37 },
    { 0x9ef0f6b, SUC_lexicon_tags_35 },
    { 0xbfaf0f6d, SUC_lexicon_tags_37 },
    { 0xf18f0f6e, SUC_lexicon_tags_2 },
    { 0x3b2f0f6f, SUC_lexicon_tags_49 },
    { 0x52af0f70, SUC_lexicon_tags_37 },
    { 0xfe6f0f71, SUC_lexicon_tags_33 },
    { 0x18f0f72, SUC_lexicon_tags_29 },
    { 0x426f0f6e, SUC_lexicon_tags_19 },
    { 0x338f0f74, SUC_lexicon_tags_45 },
    { 0x7c4f0f75, SUC_lexicon_tags_11 },
    { 0xc92f0f76, SUC_lexicon_tags_4 },
    { 0x9b0f0f77, SUC_lexicon_tags_171 },
    { 0x57af0f78, SUC_lexicon_tags_37 },
    { 0x334f0f79, SUC_lexicon_tags_28 },
    { 0xf50f0f7a, SUC_lexicon_tags_2 },
    { 0x208f0f7a, SUC_lexicon_tags_6 },
    { 0x6daf0f7b, SUC_lexicon_tags_9 },
    { 0xb0af0f77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe42f0f80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af0f83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8f0f87, SUC_lexicon_tags_56 },
    { 0xe34f0f88, SUC_lexicon_tags_55 },
    { 0x1def0f88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0f0f8d, SUC_lexicon_tags_25 },
    { 0xd5cf0f8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46f0f90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf44f0f92, SUC_lexicon_tags_12 },
    { 0x530f0f93, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfcef0f95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdd8f0f97, SUC_lexicon_tags_14 },
    { 0xf98f0f97, SUC_lexicon_tags_27 },
    { 0xa0cf0f97, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ef0f9d, SUC_lexicon_tags_15 },
    { 0x400f0f9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd02f0fa0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208f0fac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43cf0fae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23af0fb2, SUC_lexicon_tags_8 },
    { 0x424f0fb2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766f0fbb, SUC_lexicon_tags_44 },
    { 0x190f0fbc, SUC_lexicon_tags_1 },
    { 0x648f0fbd, SUC_lexicon_tags_11 },
    { 0xeacf0fbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72f0fc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2af0fc3, SUC_lexicon_tags_46 },
    { 0x75af0fc4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cf0fc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe68f0fca, SUC_lexicon_tags_37 },
    { 0x86ef0fcb, SUC_lexicon_tags_12 },
    { 0x168f0fcc, SUC_lexicon_tags_0 },
    { 0x118f0fcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e4f0fcf, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ef0fd3, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72f0fd9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8f0fde, SUC_lexicon_tags_29 },
    { 0x75ef0fde, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0fe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x940f0fe5, SUC_lexicon_tags_48 },
    { 0x4ecf0fe6, SUC_lexicon_tags_7 },
    { 0x9bcf0fe5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x722f0fea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaf0fed, SUC_lexicon_tags_38 },
    { 0xd8ef0fed, SUC_lexicon_tags_34 },
    { 0xc3cf0fef, SUC_lexicon_tags_30 },
    { 0x62af0ff0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cf0ff4, SUC_lexicon_tags_1 },
    { 0x95af0ff4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xea4f0ff7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0f0ffc, SUC_lexicon_tags_12 },
    { 0xacf0ffd, SUC_lexicon_tags_2 },
    { 0x72f0ffe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf4cf1000, SUC_lexicon_tags_61 },
    { 0x23af1001, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x92af1003, SUC_lexicon_tags_11 },
    { 0x816f1004, SUC_lexicon_tags_11 },
    { 0xbeaf1004, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9af100a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0f100e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb12f1010, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cf1015, SUC_lexicon_tags_60 },
    { 0x5f6f1016, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x696f1018, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5af101a, SUC_lexicon_tags_15 },
    { 0x42af101b, SUC_lexicon_tags_19 },
    { 0xdfef101b, SUC_lexicon_tags_49 },
    { 0xa10f101d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcf1020, SUC_lexicon_tags_17 },
    { 0xd8cf1021, SUC_lexicon_tags_68 },
    { 0xc96f1022, SUC_lexicon_tags_2 },
    { 0x768f1022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cf1028, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6f102d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360f1033, SUC_lexicon_tags_43 },
    { 0x88f1034, SUC_lexicon_tags_33 },
    { 0xf54f1034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d4f1037, SUC_lexicon_tags_43 },
    { 0x388f1038, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa28f103a, SUC_lexicon_tags_26 },
    { 0xeacf103b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f103d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x840f103f, SUC_lexicon_tags_12 },
    { 0x25cf1040, SUC_lexicon_tags_61 },
    { 0xdc6f103f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6f1044, SUC_lexicon_tags_14 },
    { 0xe0af1045, SUC_lexicon_tags_15 },
    { 0xed4f1046, SUC_lexicon_tags_11 },
    { 0xae0f1047, SUC_lexicon_tags_37 },
    { 0xd44f1048, SUC_lexicon_tags_25 },
    { 0x61af1044, SUC_lexicon_tags_15 },
    { 0x656f104a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e4f104c, SUC_lexicon_tags_37 },
    { 0x276f104c, SUC_lexicon_tags_37 },
    { 0xa2af104c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0f1053, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ef105b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaf1063, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48af1066, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdb4f1068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6f106b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2bcf106d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e2f106f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6f1073, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd4f1075, SUC_lexicon_tags_15 },
    { 0x8c8f1076, SUC_lexicon_tags_7 },
    { 0x13cf1077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4f107e, SUC_lexicon_tags_15 },
    { 0x8e0f107e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcfef1081, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf108c, SUC_lexicon_tags_10 },
    { 0x3d8f108d, SUC_lexicon_tags_11 },
    { 0xc90f108e, SUC_lexicon_tags_0 },
    { 0x750f108e, SUC_lexicon_tags_25 },
    { 0xfacf1090, SUC_lexicon_tags_56 },
    { 0xc5cf1090, SUC_lexicon_tags_43 },
    { 0x980f1091, SUC_lexicon_tags_107 },
    { 0x970f1093, SUC_lexicon_tags_1 },
    { 0x902f1094, SUC_lexicon_tags_11 },
    { 0xbb6f1095, SUC_lexicon_tags_37 },
    { 0xd5cf1095, SUC_lexicon_tags_17 },
    { 0x960f1097, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaeef1099, SUC_lexicon_tags_37 },
    { 0xcbaf109a, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8f109e, SUC_lexicon_tags_30 },
    { 0xc86f109f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61af10a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x282f10a7, SUC_lexicon_tags_7 },
    { 0xfcef10a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba2f10aa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaf10ad, SUC_lexicon_tags_62 },
    { 0x1ef10ad, SUC_lexicon_tags_7 },
    { 0x71af10af, SUC_lexicon_tags_37 },
    { 0xe5ef10b0, SUC_lexicon_tags_45 },
    { 0x6eaf10b1, SUC_lexicon_tags_37 },
    { 0x148f10b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ef10b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf98f10b9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162f10bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x574f10bf, SUC_lexicon_tags_40 },
    { 0xbb4f10bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaf10ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902f10d0, SUC_lexicon_tags_11 },
    { 0xbfef10d1, SUC_lexicon_tags_4 },
    { 0x9f8f10d0, SUC_lexicon_tags_39 },
    { 0xf86f10d3, SUC_lexicon_tags_7 },
    { 0x4dcf10d4, SUC_lexicon_tags_37 },
    { 0x8f6f10d3, SUC_lexicon_tags_17 },
    { 0xc9ef10d6, SUC_lexicon_tags_7 },
    { 0xc40f10d6, SUC_lexicon_tags_56 },
    { 0x692f10d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12f10de, SUC_lexicon_tags_30 },
    { 0xc18f10de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdef10e1, SUC_lexicon_tags_37 },
    { 0xa18f10e2, SUC_lexicon_tags_11 },
    { 0xd5af10e2, SUC_lexicon_tags_47 },
    { 0x264f10e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586f10e9, SUC_lexicon_tags_37 },
    { 0x10cf10ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f10ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x54af10f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x634f10f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46f10f6, SUC_lexicon_tags_33 },
    { 0x762f10f6, SUC_lexicon_tags_7 },
    { 0x726f10f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34f10fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52f10fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef10ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62f1103, SUC_lexicon_tags_38 },
    { 0xa6ef1104, SUC_lexicon_tags_7 },
    { 0xcacf1103, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c2f1107, SUC_lexicon_tags_30 },
    { 0x7f8f1107, SUC_lexicon_tags_14 },
    { 0x3e0f1109, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8f110d, SUC_lexicon_tags_7 },
    { 0x946f110d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x256f1110, SUC_lexicon_tags_2 },
    { 0x778f1111, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd4f1113, SUC_lexicon_tags_25 },
    { 0x7f0f1113, SUC_lexicon_tags_31 },
    { 0xc4ef1115, SUC_lexicon_tags_27 },
    { 0xd60f1116, SUC_lexicon_tags_15 },
    { 0xddaf1115, SUC_lexicon_tags_11 },
    { 0x9a4f1118, SUC_lexicon_tags_25 },
    { 0xa48f1114, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7faf111e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360f1122, SUC_lexicon_tags_7 },
    { 0x280f1123, SUC_lexicon_tags_14 },
    { 0x2c0f1123, SUC_lexicon_tags_1 },
    { 0x946f1124, SUC_lexicon_tags_21 },
    { 0x396f1126, SUC_lexicon_tags_1 },
    { 0x458f1127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22f112a, SUC_lexicon_tags_37 },
    { 0x84cf112b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5cf112d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cf1130, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9af1134, SUC_lexicon_tags_396 },
    { 0x2b4f1134, SUC_lexicon_tags_10 },
    { 0xe48f1136, SUC_lexicon_tags_25 },
    { 0xd72f1137, SUC_lexicon_tags_11 },
    { 0x930f1136, SUC_lexicon_tags_37 },
    { 0x83ef1136, SUC_lexicon_tags_63 },
    { 0xa80f113a, SUC_lexicon_tags_182 },
    { 0x4fef113b, SUC_lexicon_tags_1 },
    { 0x4fef113a, SUC_lexicon_tags_28 },
    { 0xf82f113c, SUC_lexicon_tags_55 },
    { 0x5eef113d, SUC_lexicon_tags_15 },
    { 0xfd0f113a, SUC_lexicon_tags_2 },
    { 0x864f113d, SUC_lexicon_tags_12 },
    { 0xe6af113e, SUC_lexicon_tags_15 },
    { 0x6daf113c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2f1146, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46cf1149, SUC_lexicon_tags_46 },
    { 0x18ef114a, SUC_lexicon_tags_11 },
    { 0x20ef114b, SUC_lexicon_tags_62 },
    { 0xc3af114c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x406f114f, SUC_lexicon_tags_0 },
    { 0xd24f1150, SUC_lexicon_tags_7 },
    { 0x288f1150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87ef1153, SUC_lexicon_tags_68 },
    { 0xdc6f1153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38f1158, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x79ef115a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x378f115d, SUC_lexicon_tags_105 },
    { 0x324f115e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816f1163, SUC_lexicon_tags_12 },
    { 0x6bef1164, SUC_lexicon_tags_12 },
    { 0x550f1164, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14f116b, SUC_lexicon_tags_37 },
    { 0x63af116c, SUC_lexicon_tags_0 },
    { 0xa0cf116d, SUC_lexicon_tags_31 },
    { 0xbbcf116b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x784f1170, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecf1175, SUC_lexicon_tags_37 },
    { 0x4b8f1176, SUC_lexicon_tags_17 },
    { 0xe48f1177, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330f117d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1184, SUC_lexicon_tags_49 },
    { 0xacef1184, SUC_lexicon_tags_25 },
    { 0x1f6f1186, SUC_lexicon_tags_12 },
    { 0x5d2f1185, SUC_lexicon_tags_0 },
    { 0x930f1188, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990f1191, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd28f1193, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8f1196, SUC_lexicon_tags_25 },
    { 0x25ef1197, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xceaf1199, SUC_lexicon_tags_40 },
    { 0xc76f1199, SUC_lexicon_tags_7 },
    { 0x3b6f1199, SUC_lexicon_tags_17 },
    { 0xdaaf119b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf5af119e, SUC_lexicon_tags_12 },
    { 0xfc4f119e, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480f11ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114f11b2, SUC_lexicon_tags_7 },
    { 0xa2cf11b3, SUC_lexicon_tags_1 },
    { 0x282f11b2, SUC_lexicon_tags_11 },
    { 0x382f11b5, SUC_lexicon_tags_6 },
    { 0xe30f11b5, SUC_lexicon_tags_17 },
    { 0xb8cf11b5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbcaf11b9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24f11bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6f11c5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xdb6f11c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8f11cc, SUC_lexicon_tags_30 },
    { 0x1af11cd, SUC_lexicon_tags_17 },
    { 0xc10f11cc, SUC_lexicon_tags_37 },
    { 0x2def11cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6f11d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6f11dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ef11e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x304f11e6, SUC_lexicon_tags_11 },
    { 0x242f11e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x460f11ea, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02f11ed, SUC_lexicon_tags_2 },
    { 0xd56f11ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8f11f1, SUC_lexicon_tags_37 },
    { 0xfc8f11f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf11f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59ef11f7, SUC_lexicon_tags_28 },
    { 0x9c2f11f8, SUC_lexicon_tags_14 },
    { 0x96af11f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8f11fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daf1203, SUC_lexicon_tags_8 },
    { 0x48cf1203, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54f1208, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6f120d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x508f120f, SUC_lexicon_tags_2 },
    { 0xf38f1210, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112f1214, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cf1218, SUC_lexicon_tags_6 },
    { 0x936f1219, SUC_lexicon_tags_15 },
    { 0xda8f1218, SUC_lexicon_tags_37 },
    { 0xaa4f121a, SUC_lexicon_tags_80 },
    { 0x392f121c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42f1221, SUC_lexicon_tags_17 },
    { 0x9aaf1222, SUC_lexicon_tags_17 },
    { 0x174f1222, SUC_lexicon_tags_11 },
    { 0xcf1224, SUC_lexicon_tags_38 },
    { 0x260f1225, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6f1229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf34f122b, SUC_lexicon_tags_38 },
    { 0xa6cf122c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50cf1234, SUC_lexicon_tags_37 },
    { 0xa06f1235, SUC_lexicon_tags_5 },
    { 0xf9ef1235, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x140f1238, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b2f123a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x512f123f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cf1244, SUC_lexicon_tags_19 },
    { 0x5dcf1245, SUC_lexicon_tags_43 },
    { 0x2e0f1246, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ecf1248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2f124e, SUC_lexicon_tags_17 },
    { 0x730f124f, SUC_lexicon_tags_11 },
    { 0x756f1250, SUC_lexicon_tags_11 },
    { 0xd8cf1251, SUC_lexicon_tags_28 },
    { 0x328f124f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf62f1254, SUC_lexicon_tags_12 },
    { 0x48ef1255, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ef1258, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41af125c, SUC_lexicon_tags_31 },
    { 0x264f125d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32f1266, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x342f1269, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7b4f126b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6f1270, SUC_lexicon_tags_15 },
    { 0xa24f1271, SUC_lexicon_tags_7 },
    { 0xd9af1271, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8f1278, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x344f127a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcf127e, SUC_lexicon_tags_46 },
    { 0x31ef127f, SUC_lexicon_tags_11 },
    { 0x16f127f, SUC_lexicon_tags_82 },
    { 0x69ef1281, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe2f1283, SUC_lexicon_tags_43 },
    { 0x8def1284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04f1289, SUC_lexicon_tags_17 },
    { 0x534f128a, SUC_lexicon_tags_11 },
    { 0x9b6f128b, SUC_lexicon_tags_21 },
    { 0x65ef128b, SUC_lexicon_tags_17 },
    { 0x7eef128d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c6f128f, SUC_lexicon_tags_6 },
    { 0x3d6f1290, SUC_lexicon_tags_11 },
    { 0xb0cf1291, SUC_lexicon_tags_56 },
    { 0x58cf1291, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x586f1294, SUC_lexicon_tags_1 },
    { 0x5acf1294, SUC_lexicon_tags_7 },
    { 0x556f1296, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x226f1298, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4af129b, SUC_lexicon_tags_28 },
    { 0xd40f129c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x406f129e, SUC_lexicon_tags_27 },
    { 0x5e4f129e, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6f12a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82af12a7, SUC_lexicon_tags_37 },
    { 0xa96f12a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cf12ac, SUC_lexicon_tags_7 },
    { 0xb4af12ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2f12b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf12b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd30f12b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962f12c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cf12c3, SUC_lexicon_tags_30 },
    { 0x40af12c4, SUC_lexicon_tags_2 },
    { 0xf4cf12c5, SUC_lexicon_tags_25 },
    { 0xe14f12c6, SUC_lexicon_tags_71 },
    { 0xa8cf12c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c6f12c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x600f12cb, SUC_lexicon_tags_9 },
    { 0xc1af12cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27cf12ce, SUC_lexicon_tags_32 },
    { 0x160f12cf, SUC_lexicon_tags_17 },
    { 0x2cf12cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x876f12d3, SUC_lexicon_tags_26 },
    { 0x7ccf12d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcf12db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8f12df, SUC_lexicon_tags_17 },
    { 0x904f12df, SUC_lexicon_tags_1 },
    { 0xd8f12e1, SUC_lexicon_tags_7 },
    { 0xbacf12e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61af12e6, SUC_lexicon_tags_106 },
    { 0xdd0f12e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x422f12ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f12f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eef12f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7caf12fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfa0f12fe, SUC_lexicon_tags_31 },
    { 0x29ef12ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xac0f1302, SUC_lexicon_tags_45 },
    { 0x378f1303, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdc6f1305, SUC_lexicon_tags_49 },
    { 0x7b2f1305, SUC_lexicon_tags_37 },
    { 0xc68f1307, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x382f130a, SUC_lexicon_tags_43 },
    { 0xd3ef130b, SUC_lexicon_tags_37 },
    { 0xf16f130b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4f130f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f1313, SUC_lexicon_tags_35 },
    { 0x88cf1314, SUC_lexicon_tags_15 },
    { 0xd9ef1314, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6e2f1317, SUC_lexicon_tags_31 },
    { 0x5cf1317, SUC_lexicon_tags_49 },
    { 0xa7af1319, SUC_lexicon_tags_27 },
    { 0xc28f1317, SUC_lexicon_tags_7 },
    { 0xacf131b, SUC_lexicon_tags_7 },
    { 0xadef131c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f131f, SUC_lexicon_tags_1 },
    { 0xb26f1320, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37cf1322, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318f1326, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0f1329, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468f132d, SUC_lexicon_tags_19 },
    { 0x996f132e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d8f1330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x250f1332, SUC_lexicon_tags_26 },
    { 0x61cf1332, SUC_lexicon_tags_0 },
    { 0x168f1334, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa66f1337, SUC_lexicon_tags_2 },
    { 0xcb4f1338, SUC_lexicon_tags_70 },
    { 0x31af1338, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0f133d, SUC_lexicon_tags_10 },
    { 0xbe2f133e, SUC_lexicon_tags_46 },
    { 0x944f133e, SUC_lexicon_tags_33 },
    { 0x2daf133e, SUC_lexicon_tags_19 },
    { 0x4f8f1341, SUC_lexicon_tags_34 },
    { 0xbfaf1342, SUC_lexicon_tags_11 },
    { 0xdccf1343, SUC_lexicon_tags_26 },
    { 0x9f0f133d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656f1348, SUC_lexicon_tags_19 },
    { 0x8d4f1349, SUC_lexicon_tags_38 },
    { 0x41af1349, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cf134f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52f1352, SUC_lexicon_tags_84 },
    { 0xe0cf1353, SUC_lexicon_tags_80 },
    { 0xd22f1353, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8ef1356, SUC_lexicon_tags_1 },
    { 0x8baf1357, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5f2f1359, SUC_lexicon_tags_69 },
    { 0x51af1359, SUC_lexicon_tags_12 },
    { 0x2d2f135b, SUC_lexicon_tags_17 },
    { 0x306f135c, SUC_lexicon_tags_7 },
    { 0x5acf135c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ef1361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f6f1363, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cf136a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3def1374, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x362f1377, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46f137b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72f1380, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0af1388, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4f138e, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58f1394, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0f1399, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb0f139b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4f139e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x31cf13a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712f13a4, SUC_lexicon_tags_0 },
    { 0x5d8f13a5, SUC_lexicon_tags_23 },
    { 0x494f13a5, SUC_lexicon_tags_27 },
    { 0xd3af13a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350f13ad, SUC_lexicon_tags_23 },
    { 0xfaef13ae, SUC_lexicon_tags_5 },
    { 0x5e2f13af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x874f13b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68cf13b3, SUC_lexicon_tags_38 },
    { 0x660f13b4, SUC_lexicon_tags_25 },
    { 0x77cf13b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf70f13b7, SUC_lexicon_tags_13 },
    { 0xbb6f13b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592f13bd, SUC_lexicon_tags_13 },
    { 0x2af13bd, SUC_lexicon_tags_10 },
    { 0xef4f13bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0f13c4, SUC_lexicon_tags_19 },
    { 0xc62f13c5, SUC_lexicon_tags_33 },
    { 0x4b6f13c6, SUC_lexicon_tags_37 },
    { 0x40f13c7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0f13cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb46f13ce, SUC_lexicon_tags_17 },
    { 0x98af13cf, SUC_lexicon_tags_7 },
    { 0x14cf13d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6f13d3, SUC_lexicon_tags_29 },
    { 0x1f2f13d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaf13da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbaaf13dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718f13e0, SUC_lexicon_tags_23 },
    { 0x35ef13e1, SUC_lexicon_tags_47 },
    { 0xe14f13e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f13e5, SUC_lexicon_tags_37 },
    { 0x606f13e6, SUC_lexicon_tags_25 },
    { 0x352f13e7, SUC_lexicon_tags_15 },
    { 0x16af13e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xec8f13ea, SUC_lexicon_tags_17 },
    { 0x32f13eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ef13f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896f13f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0af13fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x95af13fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12f1402, SUC_lexicon_tags_27 },
    { 0xee8f1403, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe54f1405, SUC_lexicon_tags_17 },
    { 0xb76f1406, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cef1408, SUC_lexicon_tags_19 },
    { 0x4ccf1409, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8f140c, SUC_lexicon_tags_66 },
    { 0x842f140d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xefaf1410, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87af1415, SUC_lexicon_tags_33 },
    { 0x250f1415, SUC_lexicon_tags_1 },
    { 0x826f1415, SUC_lexicon_tags_27 },
    { 0xe44f1415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x560f141b, SUC_lexicon_tags_45 },
    { 0xc2f141c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8f1420, SUC_lexicon_tags_37 },
    { 0x21ef1420, SUC_lexicon_tags_17 },
    { 0x13cf1422, SUC_lexicon_tags_7 },
    { 0x272f1421, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6f1427, SUC_lexicon_tags_3 },
    { 0x69ef1428, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfa4f142a, SUC_lexicon_tags_25 },
    { 0x870f142a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93ef142d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d6f142f, SUC_lexicon_tags_12 },
    { 0xc90f142f, SUC_lexicon_tags_25 },
    { 0xe24f142f, SUC_lexicon_tags_12 },
    { 0x7f0f1432, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f1437, SUC_lexicon_tags_43 },
    { 0xda8f1438, SUC_lexicon_tags_38 },
    { 0xa8af1439, SUC_lexicon_tags_15 },
    { 0x7c2f143a, SUC_lexicon_tags_35 },
    { 0xf40f1438, SUC_lexicon_tags_25 },
    { 0xc8ef1437, SUC_lexicon_tags_37 },
    { 0x118f143d, SUC_lexicon_tags_10 },
    { 0x7bcf1438, SUC_lexicon_tags_29 },
    { 0xaa2f143f, SUC_lexicon_tags_12 },
    { 0xa92f1440, SUC_lexicon_tags_37 },
    { 0x9def1440, SUC_lexicon_tags_83 },
    { 0x2c8f1442, SUC_lexicon_tags_25 },
    { 0x2ecf1442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40f1448, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe66f144b, SUC_lexicon_tags_12 },
    { 0xb1cf144b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79ef144e, SUC_lexicon_tags_56 },
    { 0xe7af144e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfef1451, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x410f1454, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51af145c, SUC_lexicon_tags_2 },
    { 0x9b2f145d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af1460, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4f1463, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38f146b, SUC_lexicon_tags_7 },
    { 0x576f146c, SUC_lexicon_tags_48 },
    { 0xca6f146c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2f1470, SUC_lexicon_tags_28 },
    { 0xa7ef1471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f1474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626f1479, SUC_lexicon_tags_2 },
    { 0x62af1479, SUC_lexicon_tags_37 },
    { 0x746f147b, SUC_lexicon_tags_10 },
    { 0x738f147c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfeaf147e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84f1483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe6f1485, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920f148a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7af148d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4caf1490, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8f1494, SUC_lexicon_tags_60 },
    { 0x5c0f1494, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0af149a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2f149d, SUC_lexicon_tags_95 },
    { 0x8e2f149e, SUC_lexicon_tags_23 },
    { 0xef6f149f, SUC_lexicon_tags_70 },
    { 0x72f14a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4f14a4, SUC_lexicon_tags_17 },
    { 0x648f14a5, SUC_lexicon_tags_15 },
    { 0x17ef14a6, SUC_lexicon_tags_37 },
    { 0x86ef14a7, SUC_lexicon_tags_1 },
    { 0x526f14a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x534f14ab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x806f14ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee2f14af, SUC_lexicon_tags_48 },
    { 0x950f14b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x914f14b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x118f14b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6f14b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6f14bc, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xf92f14be, SUC_lexicon_tags_60 },
    { 0x1b4f14be, SUC_lexicon_tags_12 },
    { 0xde6f14be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8f14c7, SUC_lexicon_tags_11 },
    { 0x8eaf14c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a2f14ca, SUC_lexicon_tags_7 },
    { 0xaa6f14ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe2cf14cd, SUC_lexicon_tags_37 },
    { 0x618f14ce, SUC_lexicon_tags_60 },
    { 0xc60f14cd, SUC_lexicon_tags_30 },
    { 0x402f14ce, SUC_lexicon_tags_15 },
    { 0xb08f14d1, SUC_lexicon_tags_36 },
    { 0x6def14d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xacaf14d4, SUC_lexicon_tags_11 },
    { 0x714f14d5, SUC_lexicon_tags_23 },
    { 0x69cf14d5, SUC_lexicon_tags_13 },
    { 0x260f14d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210f14dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bef14e5, SUC_lexicon_tags_237 },
    { 0x3cef14e6, SUC_lexicon_tags_1 },
    { 0xaa0f14e7, SUC_lexicon_tags_49 },
    { 0xca4f14e7, SUC_lexicon_tags_6 },
    { 0x658f14e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cf14ec, SUC_lexicon_tags_15 },
    { 0xc1ef14ed, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f14f9, SUC_lexicon_tags_11 },
    { 0xf74f14fa, SUC_lexicon_tags_17 },
    { 0xb26f14fb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aef14ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6f150c, SUC_lexicon_tags_27 },
    { 0x880f150c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ef1510, SUC_lexicon_tags_15 },
    { 0x528f1511, SUC_lexicon_tags_26 },
    { 0x718f1512, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ef151c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804f1526, SUC_lexicon_tags_46 },
    { 0xf3cf1526, SUC_lexicon_tags_1 },
    { 0x1c4f1527, SUC_lexicon_tags_21 },
    { 0x1baf1526, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf1ef152b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82ef152d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x30af1530, SUC_lexicon_tags_63 },
    { 0xc50f1531, SUC_lexicon_tags_45 },
    { 0x822f1532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaf1538, SUC_lexicon_tags_4 },
    { 0x9eef1539, SUC_lexicon_tags_7 },
    { 0x470f1539, SUC_lexicon_tags_37 },
    { 0x7fcf153a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x954f153d, SUC_lexicon_tags_43 },
    { 0x634f153e, SUC_lexicon_tags_56 },
    { 0xc96f153e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf0ef1541, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cf1546, SUC_lexicon_tags_3 },
    { 0x588f1546, SUC_lexicon_tags_151 },
    { 0xc5cf1548, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cef154c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754f1553, SUC_lexicon_tags_47 },
    { 0xbeaf1553, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3eaf1556, SUC_lexicon_tags_7 },
    { 0x7f2f1556, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xa88f1559, SUC_lexicon_tags_12 },
    { 0x54af155a, SUC_lexicon_tags_17 },
    { 0x16f155a, SUC_lexicon_tags_15 },
    { 0xd3af155c, SUC_lexicon_tags_2 },
    { 0x4c0f155d, SUC_lexicon_tags_10 },
    { 0xacaf155e, SUC_lexicon_tags_26 },
    { 0xc4cf155f, SUC_lexicon_tags_66 },
    { 0x17af1560, SUC_lexicon_tags_15 },
    { 0xd1af155e, SUC_lexicon_tags_25 },
    { 0xed0f155e, SUC_lexicon_tags_2 },
    { 0xfc6f155d, SUC_lexicon_tags_11 },
    { 0x8f1561, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x806f1566, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30af1568, SUC_lexicon_tags_9 },
    { 0x26f1568, SUC_lexicon_tags_83 },
    { 0x396f156a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78f156d, SUC_lexicon_tags_16 },
    { 0x58af156e, SUC_lexicon_tags_10 },
    { 0x97cf156f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456f1574, SUC_lexicon_tags_49 },
    { 0x122f1575, SUC_lexicon_tags_47 },
    { 0x208f1576, SUC_lexicon_tags_27 },
    { 0x74ef1577, SUC_lexicon_tags_17 },
    { 0x5af1578, SUC_lexicon_tags_19 },
    { 0x26ef1578, SUC_lexicon_tags_7 },
    { 0xf18f157a, SUC_lexicon_tags_12 },
    { 0x460f157a, SUC_lexicon_tags_25 },
    { 0x72cf157c, SUC_lexicon_tags_17 },
    { 0x6e8f157c, SUC_lexicon_tags_60 },
    { 0x9c8f157c, SUC_lexicon_tags_17 },
    { 0x3d4f157e, SUC_lexicon_tags_60 },
    { 0x7d2f157e, SUC_lexicon_tags_12 },
    { 0xaf6f157a, SUC_lexicon_tags_17 },
    { 0x618f1582, SUC_lexicon_tags_17 },
    { 0x464f1582, SUC_lexicon_tags_60 },
    { 0x8b0f1584, SUC_lexicon_tags_11 },
    { 0x3a6f1585, SUC_lexicon_tags_14 },
    { 0x902f1586, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x316f1589, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf158c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7def1592, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2cf1594, SUC_lexicon_tags_30 },
    { 0xeef1595, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcf1598, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2f15a1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18f15a7, SUC_lexicon_tags_29 },
    { 0x548f15a8, SUC_lexicon_tags_1 },
    { 0x980f15a8, SUC_lexicon_tags_25 },
    { 0x8a0f15a7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xdd8f15ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578f15b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cf15ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f15bd, SUC_lexicon_tags_146 },
    { 0x832f15be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f15c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90f15c6, SUC_lexicon_tags_49 },
    { 0xc74f15c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x950f15c9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82f15cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2f15d3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74af15dc, SUC_lexicon_tags_60 },
    { 0x9a8f15dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd74f15e0, SUC_lexicon_tags_27 },
    { 0xfdaf15e0, SUC_lexicon_tags_25 },
    { 0x80cf15e2, SUC_lexicon_tags_25 },
    { 0xce0f15e3, SUC_lexicon_tags_25 },
    { 0xf28f15e3, SUC_lexicon_tags_0 },
    { 0xda4f15e5, SUC_lexicon_tags_9 },
    { 0x74af15e6, SUC_lexicon_tags_1 },
    { 0x202f15e2, SUC_lexicon_tags_7 },
    { 0x68af15e8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x1baf15ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x16af15ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2f15f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452f15f4, SUC_lexicon_tags_27 },
    { 0xc5af15f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c8f15f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b0f15f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x67af15fb, SUC_lexicon_tags_7 },
    { 0x7c8f15fc, SUC_lexicon_tags_17 },
    { 0x41cf15fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cf1600, SUC_lexicon_tags_17 },
    { 0x3dcf1600, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0f1608, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x734f160a, SUC_lexicon_tags_23 },
    { 0xe60f160b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2f160e, SUC_lexicon_tags_1 },
    { 0x434f160f, SUC_lexicon_tags_66 },
    { 0x788f1610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8f1616, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38f1619, SUC_lexicon_tags_17 },
    { 0x8f8f161a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6f161c, SUC_lexicon_tags_6 },
    { 0xc1af161d, SUC_lexicon_tags_11 },
    { 0xcd6f161e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x210f1620, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48f1626, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50f1628, SUC_lexicon_tags_43 },
    { 0xe46f1629, SUC_lexicon_tags_17 },
    { 0xf18f1629, SUC_lexicon_tags_27 },
    { 0xa8ef162a, SUC_lexicon_tags_43 },
    { 0xf30f1629, SUC_lexicon_tags_1 },
    { 0xbc4f162d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x466f1630, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cf1635, SUC_lexicon_tags_7 },
    { 0x3c6f1636, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85af163a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7acf163c, SUC_lexicon_tags_17 },
    { 0x80cf163d, SUC_lexicon_tags_14 },
    { 0x124f163c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab6f1641, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4ef1643, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x95cf1645, SUC_lexicon_tags_29 },
    { 0x910f1646, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xccef1648, SUC_lexicon_tags_37 },
    { 0x8a6f1649, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cf164e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788f1654, SUC_lexicon_tags_29 },
    { 0xbcef1655, SUC_lexicon_tags_2 },
    { 0xc0f1656, SUC_lexicon_tags_7 },
    { 0x940f1657, SUC_lexicon_tags_8 },
    { 0x642f1658, SUC_lexicon_tags_7 },
    { 0x76af1659, SUC_lexicon_tags_45 },
    { 0x7f4f165a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4f1660, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110f1666, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x542f1668, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd06f166a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcf166f, SUC_lexicon_tags_26 },
    { 0x76cf166f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44f1674, SUC_lexicon_tags_2 },
    { 0x974f1675, SUC_lexicon_tags_49 },
    { 0x8d0f1676, SUC_lexicon_tags_9 },
    { 0x4c8f1677, SUC_lexicon_tags_7 },
    { 0xe96f1678, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79af167d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x492f167f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x774f1682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcaaf1684, SUC_lexicon_tags_36 },
    { 0xfaef1685, SUC_lexicon_tags_23 },
    { 0xc5ef1686, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xcaf1688, SUC_lexicon_tags_0 },
    { 0x58af1689, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x982f168b, SUC_lexicon_tags_25 },
    { 0x6f4f168c, SUC_lexicon_tags_1 },
    { 0xec8f168b, SUC_lexicon_tags_4 },
    { 0x5f4f168e, SUC_lexicon_tags_25 },
    { 0xf26f168f, SUC_lexicon_tags_12 },
    { 0x128f1690, SUC_lexicon_tags_32 },
    { 0x4aef1691, SUC_lexicon_tags_2 },
    { 0x6aaf1692, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ef1695, SUC_lexicon_tags_9 },
    { 0xb8af1696, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8f169b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2faf16a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae6f16a3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaef16a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32f16ab, SUC_lexicon_tags_12 },
    { 0x642f16ab, SUC_lexicon_tags_25 },
    { 0x148f16ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x566f16b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11af16b2, SUC_lexicon_tags_0 },
    { 0x9a8f16b3, SUC_lexicon_tags_37 },
    { 0xc0f16b4, SUC_lexicon_tags_12 },
    { 0x74ef16b5, SUC_lexicon_tags_7 },
    { 0x4dcf16b4, SUC_lexicon_tags_25 },
    { 0x5ecf16b3, SUC_lexicon_tags_2 },
    { 0x990f16b5, SUC_lexicon_tags_17 },
    { 0x81cf16b8, SUC_lexicon_tags_47 },
    { 0x82f16ba, SUC_lexicon_tags_65 },
    { 0x99ef16b5, SUC_lexicon_tags_12 },
    { 0x3caf16b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d2f16be, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cf16c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ef16cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x17af16cd, SUC_lexicon_tags_38 },
    { 0x2eaf16cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2f16d2, SUC_lexicon_tags_17 },
    { 0x518f16d3, SUC_lexicon_tags_17 },
    { 0x692f16d3, SUC_lexicon_tags_33 },
    { 0xcc6f16d5, SUC_lexicon_tags_1 },
    { 0x5ccf16d5, SUC_lexicon_tags_19 },
    { 0xbfaf16d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x542f16d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cf16dc, SUC_lexicon_tags_46 },
    { 0x6cf16dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x35cf16df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ccf16e6, SUC_lexicon_tags_59 },
    { 0x62ef16e7, SUC_lexicon_tags_11 },
    { 0xe98f16e8, SUC_lexicon_tags_14 },
    { 0x600f16e6, SUC_lexicon_tags_2 },
    { 0xce2f16e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x758f16ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x70af16ee, SUC_lexicon_tags_37 },
    { 0xc02f16ee, SUC_lexicon_tags_27 },
    { 0x3b4f16f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcf16f5, SUC_lexicon_tags_37 },
    { 0x918f16f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc80f16f8, SUC_lexicon_tags_2 },
    { 0xb4cf16f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76f16fd, SUC_lexicon_tags_7 },
    { 0xbacf16fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47cf1703, SUC_lexicon_tags_37 },
    { 0x30f1704, SUC_lexicon_tags_17 },
    { 0x9baf1704, SUC_lexicon_tags_37 },
    { 0x90f1706, SUC_lexicon_tags_2 },
    { 0xbf4f1706, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74cf170c, SUC_lexicon_tags_35 },
    { 0xa86f170c, SUC_lexicon_tags_15 },
    { 0x48af170c, SUC_lexicon_tags_14 },
    { 0x87ef170c, SUC_lexicon_tags_10 },
    { 0xde8f170d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x956f1712, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f1715, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ef171a, SUC_lexicon_tags_30 },
    { 0xbc6f171b, SUC_lexicon_tags_815 },
    { 0, NULL },
    { 0, NULL },
    { 0x384f171e, SUC_lexicon_tags_11 },
    { 0xb3af171f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584f1725, SUC_lexicon_tags_38 },
    { 0x63ef1726, SUC_lexicon_tags_14 },
    { 0xa3cf1727, SUC_lexicon_tags_32 },
    { 0x60f1725, SUC_lexicon_tags_1 },
    { 0x21ef1729, SUC_lexicon_tags_49 },
    { 0x72cf172a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x720f172c, SUC_lexicon_tags_25 },
    { 0xefaf172d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x898f1730, SUC_lexicon_tags_29 },
    { 0x5ef1731, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2f1736, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf3cf1738, SUC_lexicon_tags_12 },
    { 0x2d6f1739, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16af173e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe04f1740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0f1744, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834f1748, SUC_lexicon_tags_14 },
    { 0x3f2f1749, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2f174c, SUC_lexicon_tags_46 },
    { 0x12ef174d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcf1751, SUC_lexicon_tags_1 },
    { 0xf00f1751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0af1754, SUC_lexicon_tags_13 },
    { 0x24af1754, SUC_lexicon_tags_15 },
    { 0xbfaf1754, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f175a, SUC_lexicon_tags_47 },
    { 0x8c0f175a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc28f175d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa60f1761, SUC_lexicon_tags_66 },
    { 0x2acf1762, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc40f1764, SUC_lexicon_tags_2 },
    { 0x268f1764, SUC_lexicon_tags_31 },
    { 0xcf6f1766, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46f176d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd74f176f, SUC_lexicon_tags_25 },
    { 0xbdcf176f, SUC_lexicon_tags_23 },
    { 0xb52f176f, SUC_lexicon_tags_27 },
    { 0x8f1772, SUC_lexicon_tags_15 },
    { 0x66cf1772, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7acf1776, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3ef1778, SUC_lexicon_tags_2 },
    { 0x5fcf1778, SUC_lexicon_tags_7 },
    { 0xfcf177a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244f177f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffef1783, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cf178c, SUC_lexicon_tags_15 },
    { 0xc1cf178d, SUC_lexicon_tags_77 },
    { 0x8d6f178e, SUC_lexicon_tags_60 },
    { 0x8f4f178f, SUC_lexicon_tags_2 },
    { 0x108f178f, SUC_lexicon_tags_19 },
    { 0xb5cf1791, SUC_lexicon_tags_43 },
    { 0x89cf178f, SUC_lexicon_tags_15 },
    { 0x4a8f1793, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdccf1795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56f1798, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0f179c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cf17a2, SUC_lexicon_tags_43 },
    { 0x846f17a3, SUC_lexicon_tags_2 },
    { 0xa4cf17a4, SUC_lexicon_tags_11 },
    { 0x744f17a5, SUC_lexicon_tags_11 },
    { 0x8dcf17a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb14f17a8, SUC_lexicon_tags_16 },
    { 0xe3af17a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe2f17ab, SUC_lexicon_tags_28 },
    { 0xab0f17ac, SUC_lexicon_tags_1 },
    { 0xbcef17ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x526f17af, SUC_lexicon_tags_9 },
    { 0x4d4f17b0, SUC_lexicon_tags_37 },
    { 0x468f17b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x136f17b3, SUC_lexicon_tags_15 },
    { 0x4a0f17b4, SUC_lexicon_tags_23 },
    { 0x77af17b5, SUC_lexicon_tags_19 },
    { 0x3c0f17b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x218f17b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf84f17bb, SUC_lexicon_tags_2 },
    { 0x486f17bb, SUC_lexicon_tags_9 },
    { 0x948f17bd, SUC_lexicon_tags_113 },
    { 0x76af17bc, SUC_lexicon_tags_25 },
    { 0xa0ef17bf, SUC_lexicon_tags_25 },
    { 0xa52f17c0, SUC_lexicon_tags_47 },
    { 0x2bef17c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbcf17c3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x63cf17c5, SUC_lexicon_tags_32 },
    { 0x7d6f17c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878f17ce, SUC_lexicon_tags_11 },
    { 0xc68f17cf, SUC_lexicon_tags_26 },
    { 0xbd8f17d0, SUC_lexicon_tags_30 },
    { 0xb2ef17ce, SUC_lexicon_tags_25 },
    { 0xa24f17d1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfc8f17d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6caf17d6, SUC_lexicon_tags_15 },
    { 0x30cf17d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ef17da, SUC_lexicon_tags_38 },
    { 0xdaf17db, SUC_lexicon_tags_45 },
    { 0x76cf17da, SUC_lexicon_tags_15 },
    { 0xeacf17dd, SUC_lexicon_tags_38 },
    { 0x99af17dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x706f17e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8cf17e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ef17e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62f17ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98f17f0, SUC_lexicon_tags_2 },
    { 0xf5cf17f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba0f17f3, SUC_lexicon_tags_38 },
    { 0x1a2f17f3, SUC_lexicon_tags_77 },
    { 0xeaaf17f4, SUC_lexicon_tags_49 },
    { 0x366f17f6, SUC_lexicon_tags_37 },
    { 0xe48f17f4, SUC_lexicon_tags_27 },
    { 0xc18f17f7, SUC_lexicon_tags_0 },
    { 0x5fef17f7, SUC_lexicon_tags_38 },
    { 0x96ef17f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4ef17fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x882f17ff, SUC_lexicon_tags_1 },
    { 0xfd6f1800, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34f180a, SUC_lexicon_tags_48 },
    { 0x96cf180a, SUC_lexicon_tags_43 },
    { 0x714f180c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cf1815, SUC_lexicon_tags_30 },
    { 0xf04f1816, SUC_lexicon_tags_21 },
    { 0x93cf1817, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x83ef1819, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbccf181b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6f181f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56f1824, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8f1828, SUC_lexicon_tags_45 },
    { 0x106f1829, SUC_lexicon_tags_37 },
    { 0xebef1828, SUC_lexicon_tags_25 },
    { 0x6c4f182b, SUC_lexicon_tags_31 },
    { 0x6d8f182a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4f1832, SUC_lexicon_tags_8 },
    { 0xda4f1832, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x736f1836, SUC_lexicon_tags_17 },
    { 0x344f1837, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f183c, SUC_lexicon_tags_66 },
    { 0xf16f183d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4f1842, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaf184b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ef184f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7af1853, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x608f185a, SUC_lexicon_tags_38 },
    { 0x3d0f185b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x926f185d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08f1863, SUC_lexicon_tags_84 },
    { 0x8f4f1864, SUC_lexicon_tags_0 },
    { 0x5a4f1864, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf8f1867, SUC_lexicon_tags_65 },
    { 0xd36f1868, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218f186c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ef1871, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cf1875, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2f187b, SUC_lexicon_tags_2 },
    { 0x40cf187c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92f1883, SUC_lexicon_tags_140 },
    { 0x8c6f1883, SUC_lexicon_tags_121 },
    { 0x8daf1884, SUC_lexicon_tags_25 },
    { 0x908f1886, SUC_lexicon_tags_60 },
    { 0x112f1887, SUC_lexicon_tags_17 },
    { 0x994f1886, SUC_lexicon_tags_37 },
    { 0xfe2f1889, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8f188d, SUC_lexicon_tags_29 },
    { 0xf6cf188e, SUC_lexicon_tags_0 },
    { 0xc78f188e, SUC_lexicon_tags_15 },
    { 0x4bcf188d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52f1893, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34f1895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1af1897, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x164f189a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc72f189c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x666f189f, SUC_lexicon_tags_47 },
    { 0xb3ef189f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baf18a9, SUC_lexicon_tags_17 },
    { 0xc48f18aa, SUC_lexicon_tags_63 },
    { 0x1eaf18a9, SUC_lexicon_tags_17 },
    { 0x8b8f18ab, SUC_lexicon_tags_1 },
    { 0xb32f18ad, SUC_lexicon_tags_63 },
    { 0x44af18ae, SUC_lexicon_tags_28 },
    { 0xaf2f18ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77af18b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ef18bf, SUC_lexicon_tags_47 },
    { 0xf98f18c0, SUC_lexicon_tags_33 },
    { 0x4aef18c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbef18c3, SUC_lexicon_tags_6 },
    { 0x8a6f18c4, SUC_lexicon_tags_27 },
    { 0xcecf18c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ef18c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcf18cb, SUC_lexicon_tags_17 },
    { 0x880f18cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4ef18ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfef18d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6f18e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce6f18e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cf18eb, SUC_lexicon_tags_8 },
    { 0x50ef18eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ef18f0, SUC_lexicon_tags_28 },
    { 0xdb6f18f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4f18f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cf18f8, SUC_lexicon_tags_2 },
    { 0x290f18f9, SUC_lexicon_tags_124 },
    { 0xd44f18f9, SUC_lexicon_tags_47 },
    { 0xda4f18fb, SUC_lexicon_tags_1 },
    { 0x44af18fc, SUC_lexicon_tags_25 },
    { 0x914f18fc, SUC_lexicon_tags_0 },
    { 0xc42f18fd, SUC_lexicon_tags_107 },
    { 0x91cf18fe, SUC_lexicon_tags_45 },
    { 0x926f1900, SUC_lexicon_tags_31 },
    { 0x46f1901, SUC_lexicon_tags_65 },
    { 0xb2ef18fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xacef1905, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418f190d, SUC_lexicon_tags_30 },
    { 0xadcf190e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x84af1911, SUC_lexicon_tags_29 },
    { 0x4c6f1912, SUC_lexicon_tags_64 },
    { 0x6a0f1911, SUC_lexicon_tags_1 },
    { 0xaf0f1913, SUC_lexicon_tags_0 },
    { 0x26ef1911, SUC_lexicon_tags_12 },
    { 0x1e8f1916, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26af1919, SUC_lexicon_tags_15 },
    { 0x40af1919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77cf191c, SUC_lexicon_tags_1 },
    { 0xa64f191d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfdef191f, SUC_lexicon_tags_11 },
    { 0xf48f1920, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1924, SUC_lexicon_tags_49 },
    { 0xe82f1925, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb22f1927, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cf192a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caf192f, SUC_lexicon_tags_22 },
    { 0x52ef1930, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4f1933, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xda8f1935, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20f1938, SUC_lexicon_tags_37 },
    { 0xd04f1939, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128f193d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64f1943, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x232f1946, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef194c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2f194f, SUC_lexicon_tags_37 },
    { 0xe90f1950, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf1953, SUC_lexicon_tags_25 },
    { 0xaf4f1954, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x750f1956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13af1958, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x108f195b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322f195f, SUC_lexicon_tags_37 },
    { 0xa2cf195f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f1964, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82f1969, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x21ef196b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33af196d, SUC_lexicon_tags_47 },
    { 0x2baf196d, SUC_lexicon_tags_15 },
    { 0x1cf196d, SUC_lexicon_tags_11 },
    { 0x8af1970, SUC_lexicon_tags_9 },
    { 0xe90f1971, SUC_lexicon_tags_33 },
    { 0x3b8f1972, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x71cf1974, SUC_lexicon_tags_3 },
    { 0xe84f1975, SUC_lexicon_tags_15 },
    { 0x1dcf1975, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0f1979, SUC_lexicon_tags_11 },
    { 0x68af197a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x224f197d, SUC_lexicon_tags_23 },
    { 0xcef197e, SUC_lexicon_tags_25 },
    { 0xea0f197f, SUC_lexicon_tags_18 },
    { 0xd2af1980, SUC_lexicon_tags_33 },
    { 0x328f1981, SUC_lexicon_tags_7 },
    { 0x46af1980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95cf1984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a0f1986, SUC_lexicon_tags_37 },
    { 0xfb6f1986, SUC_lexicon_tags_9 },
    { 0x984f1988, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccf198c, SUC_lexicon_tags_30 },
    { 0xcdaf198c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x368f1990, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bef1993, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ef199a, SUC_lexicon_tags_11 },
    { 0xa62f199b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x576f199d, SUC_lexicon_tags_198 },
    { 0xa58f199e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30f19a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x230f19a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0f19ad, SUC_lexicon_tags_47 },
    { 0x24af19ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41ef19b0, SUC_lexicon_tags_37 },
    { 0x7c2f19b1, SUC_lexicon_tags_25 },
    { 0x148f19b1, SUC_lexicon_tags_12 },
    { 0xf66f19b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daf19b8, SUC_lexicon_tags_25 },
    { 0xd7ef19b8, SUC_lexicon_tags_17 },
    { 0xd2f19ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc98f19bc, SUC_lexicon_tags_9 },
    { 0x67af19bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7eaf19bf, SUC_lexicon_tags_0 },
    { 0x8f2f19c0, SUC_lexicon_tags_48 },
    { 0x532f19c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae8f19c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x644f19c8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcf19cc, SUC_lexicon_tags_7 },
    { 0x204f19cc, SUC_lexicon_tags_12 },
    { 0xaef19ce, SUC_lexicon_tags_37 },
    { 0x268f19cf, SUC_lexicon_tags_49 },
    { 0xdc2f19cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb7cf19d2, SUC_lexicon_tags_66 },
    { 0xe04f19d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfef19d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa72f19d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x466f19d9, SUC_lexicon_tags_23 },
    { 0x2af19d9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14f19df, SUC_lexicon_tags_26 },
    { 0x2a6f19df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x340f19e2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfa4f19e4, SUC_lexicon_tags_16 },
    { 0x76cf19e5, SUC_lexicon_tags_27 },
    { 0x25cf19e6, SUC_lexicon_tags_31 },
    { 0x450f19e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x58cf19e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f19ec, SUC_lexicon_tags_116 },
    { 0x6b2f19ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9af19ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccf19f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2f19f7, SUC_lexicon_tags_43 },
    { 0x49af19f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04f19fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x438f1a00, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd7af1a02, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74af1a06, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ef1a09, SUC_lexicon_tags_74 },
    { 0x3b8f1a09, SUC_lexicon_tags_17 },
    { 0xd46f1a0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda0f1a0d, SUC_lexicon_tags_1 },
    { 0xa48f1a0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44ef1a10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb06f1a12, SUC_lexicon_tags_47 },
    { 0xb0cf1a12, SUC_lexicon_tags_37 },
    { 0xb18f1a13, SUC_lexicon_tags_27 },
    { 0xd4cf1a15, SUC_lexicon_tags_1 },
    { 0xe94f1a16, SUC_lexicon_tags_11 },
    { 0xd1af1a17, SUC_lexicon_tags_17 },
    { 0x220f1a18, SUC_lexicon_tags_1 },
    { 0x6ef1a16, SUC_lexicon_tags_527 },
    { 0, NULL },
    { 0x5dcf1a1b, SUC_lexicon_tags_1 },
    { 0xa76f1a1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x836f1a1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834f1a22, SUC_lexicon_tags_13 },
    { 0x49af1a23, SUC_lexicon_tags_17 },
    { 0x7d8f1a22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf1a28, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x688f1a2b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6f1a2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcf1a32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8bcf1a34, SUC_lexicon_tags_27 },
    { 0x526f1a35, SUC_lexicon_tags_1 },
    { 0x654f1a35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x668f1a39, SUC_lexicon_tags_11 },
    { 0x120f1a39, SUC_lexicon_tags_11 },
    { 0x69cf1a3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaf1a3f, SUC_lexicon_tags_49 },
    { 0x74af1a3f, SUC_lexicon_tags_65 },
    { 0xca4f1a41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca4f1a43, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f1a47, SUC_lexicon_tags_12 },
    { 0xc0f1a48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4f1a4d, SUC_lexicon_tags_15 },
    { 0x19af1a4e, SUC_lexicon_tags_17 },
    { 0xdd6f1a4f, SUC_lexicon_tags_63 },
    { 0x730f1a4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0f1a5a, SUC_lexicon_tags_140 },
    { 0x116f1a5b, SUC_lexicon_tags_23 },
    { 0xfdcf1a5b, SUC_lexicon_tags_57 },
    { 0x842f1a5d, SUC_lexicon_tags_19 },
    { 0xba2f1a5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4f1a61, SUC_lexicon_tags_114 },
    { 0x7eef1a61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76cf1a64, SUC_lexicon_tags_31 },
    { 0x56af1a65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40f1a67, SUC_lexicon_tags_11 },
    { 0xf9af1a68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cf1a71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x790f1a73, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74f1a77, SUC_lexicon_tags_25 },
    { 0x8dcf1a78, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2f1a7b, SUC_lexicon_tags_25 },
    { 0xdf2f1a7c, SUC_lexicon_tags_140 },
    { 0x7aaf1a7d, SUC_lexicon_tags_77 },
    { 0xeb2f1a7e, SUC_lexicon_tags_11 },
    { 0x560f1a7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd6f1a81, SUC_lexicon_tags_6 },
    { 0xecef1a82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x442f1a85, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8f1a89, SUC_lexicon_tags_31 },
    { 0x8eef1a8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32f1a8d, SUC_lexicon_tags_11 },
    { 0xf00f1a8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8f1a91, SUC_lexicon_tags_46 },
    { 0x606f1a92, SUC_lexicon_tags_63 },
    { 0xd0f1a91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdef1a98, SUC_lexicon_tags_87 },
    { 0xdf6f1a98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc1ef1a9b, SUC_lexicon_tags_12 },
    { 0x4a4f1a9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaf1a9f, SUC_lexicon_tags_29 },
    { 0x910f1aa0, SUC_lexicon_tags_17 },
    { 0x150f1aa0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xad0f1aa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55cf1aa6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x680f1aa9, SUC_lexicon_tags_19 },
    { 0xa5ef1aaa, SUC_lexicon_tags_14 },
    { 0x68af1aab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32f1aae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f1ab1, SUC_lexicon_tags_16 },
    { 0x26af1ab2, SUC_lexicon_tags_11 },
    { 0x21ef1ab3, SUC_lexicon_tags_1 },
    { 0xd1af1ab4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14f1ab8, SUC_lexicon_tags_2 },
    { 0xf72f1ab9, SUC_lexicon_tags_43 },
    { 0x8fef1aba, SUC_lexicon_tags_26 },
    { 0x496f1aba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e6f1abd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8f1ac1, SUC_lexicon_tags_7 },
    { 0x742f1ac2, SUC_lexicon_tags_23 },
    { 0x740f1ac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fef1ac6, SUC_lexicon_tags_23 },
    { 0xc7ef1ac7, SUC_lexicon_tags_7 },
    { 0x1c6f1ac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02f1acf, SUC_lexicon_tags_68 },
    { 0x2af1ad0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6f1ad4, SUC_lexicon_tags_45 },
    { 0x448f1ad5, SUC_lexicon_tags_11 },
    { 0xaf6f1ad4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3e2f1ad8, SUC_lexicon_tags_87 },
    { 0x73cf1ad8, SUC_lexicon_tags_30 },
    { 0xb0ef1ada, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f1add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0f1ae2, SUC_lexicon_tags_0 },
    { 0x880f1ae3, SUC_lexicon_tags_23 },
    { 0x14cf1ae3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8f1ae8, SUC_lexicon_tags_17 },
    { 0xee4f1ae9, SUC_lexicon_tags_1 },
    { 0x8d4f1ae8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca4f1aec, SUC_lexicon_tags_49 },
    { 0xfb8f1aed, SUC_lexicon_tags_31 },
    { 0xa2cf1aed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2f1af1, SUC_lexicon_tags_25 },
    { 0x20cf1af1, SUC_lexicon_tags_25 },
    { 0xef6f1af3, SUC_lexicon_tags_15 },
    { 0xb82f1af2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe58f1af6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x904f1af8, SUC_lexicon_tags_38 },
    { 0x998f1af8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x834f1afb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6f1afe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x224f1b00, SUC_lexicon_tags_11 },
    { 0x904f1b00, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x162f1b04, SUC_lexicon_tags_7 },
    { 0x980f1b05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ef1b08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc42f1b0a, SUC_lexicon_tags_23 },
    { 0xe9ef1b0b, SUC_lexicon_tags_29 },
    { 0xf9ef1b0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832f1b10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb2f1b12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xebaf1b14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356f1b18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16f1b1c, SUC_lexicon_tags_0 },
    { 0xd08f1b1d, SUC_lexicon_tags_25 },
    { 0x9baf1b1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6cf1b20, SUC_lexicon_tags_37 },
    { 0xd96f1b21, SUC_lexicon_tags_15 },
    { 0xf06f1b22, SUC_lexicon_tags_16 },
    { 0x182f1b21, SUC_lexicon_tags_1 },
    { 0x450f1b24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a2f1b26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794f1b2c, SUC_lexicon_tags_40 },
    { 0xcbaf1b2d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x498f1b2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ef1b35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbccf1b37, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2f1b40, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x720f1b42, SUC_lexicon_tags_14 },
    { 0x78af1b42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaef1b46, SUC_lexicon_tags_13 },
    { 0xb40f1b47, SUC_lexicon_tags_37 },
    { 0xd22f1b47, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44f1b4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bef1b51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecf1b59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecf1b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x942f1b60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32f1b63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x66ef1b65, SUC_lexicon_tags_26 },
    { 0x6aaf1b65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe42f1b68, SUC_lexicon_tags_37 },
    { 0x7dcf1b68, SUC_lexicon_tags_15 },
    { 0x4d0f1b6a, SUC_lexicon_tags_49 },
    { 0xd24f1b6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14af1b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6daf1b76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ef1b79, SUC_lexicon_tags_12 },
    { 0x93cf1b79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2f1b7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x372f1b7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1dcf1b81, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x218f1b83, SUC_lexicon_tags_14 },
    { 0xf3af1b84, SUC_lexicon_tags_63 },
    { 0x6b6f1b85, SUC_lexicon_tags_2 },
    { 0x3aef1b85, SUC_lexicon_tags_0 },
    { 0x7e4f1b86, SUC_lexicon_tags_25 },
    { 0x5baf1b88, SUC_lexicon_tags_1 },
    { 0x786f1b89, SUC_lexicon_tags_17 },
    { 0xb8af1b88, SUC_lexicon_tags_2 },
    { 0xaa8f1b87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f1b8e, SUC_lexicon_tags_37 },
    { 0x4eaf1b8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2f1b92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4ef1b94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf1b98, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x672f1b9a, SUC_lexicon_tags_8 },
    { 0x152f1b9b, SUC_lexicon_tags_37 },
    { 0xd4ef1b9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f2f1ba5, SUC_lexicon_tags_25 },
    { 0xd86f1ba6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ef1ba9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278f1bad, SUC_lexicon_tags_12 },
    { 0x3ccf1bae, SUC_lexicon_tags_11 },
    { 0x4a4f1baf, SUC_lexicon_tags_174 },
    { 0x41af1bae, SUC_lexicon_tags_1 },
    { 0x6e4f1bae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x260f1bb3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4f1bb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4f1bba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ef1bc0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa32f1bc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd38f1bc4, SUC_lexicon_tags_48 },
    { 0x1b8f1bc5, SUC_lexicon_tags_11 },
    { 0xa9ef1bc6, SUC_lexicon_tags_17 },
    { 0x302f1bc6, SUC_lexicon_tags_37 },
    { 0x338f1bc8, SUC_lexicon_tags_12 },
    { 0x8c6f1bc9, SUC_lexicon_tags_28 },
    { 0x866f1bc6, SUC_lexicon_tags_80 },
    { 0x970f1bc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cf1bcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7f8f1bd1, SUC_lexicon_tags_16 },
    { 0x5aaf1bd1, SUC_lexicon_tags_45 },
    { 0x276f1bd2, SUC_lexicon_tags_38 },
    { 0xd38f1bd4, SUC_lexicon_tags_17 },
    { 0xbcf1bd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6f1bdb, SUC_lexicon_tags_19 },
    { 0xe26f1bdb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9faf1bde, SUC_lexicon_tags_33 },
    { 0xc18f1bde, SUC_lexicon_tags_25 },
    { 0x85ef1bde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6f1be3, SUC_lexicon_tags_11 },
    { 0x1fef1be4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x378f1be7, SUC_lexicon_tags_12 },
    { 0x742f1be8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34af1bee, SUC_lexicon_tags_33 },
    { 0x834f1bef, SUC_lexicon_tags_12 },
    { 0x506f1bee, SUC_lexicon_tags_34 },
    { 0xb6f1bf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc26f1bf3, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeef1bf8, SUC_lexicon_tags_37 },
    { 0x816f1bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ef1bfc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x388f1bfe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476f1c06, SUC_lexicon_tags_11 },
    { 0xe7ef1c07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x468f1c0a, SUC_lexicon_tags_43 },
    { 0x112f1c0b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdef1c0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb4cf1c0f, SUC_lexicon_tags_11 },
    { 0x484f1c0f, SUC_lexicon_tags_40 },
    { 0xb64f1c0f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1f4f1c13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x258f1c16, SUC_lexicon_tags_27 },
    { 0xa5af1c16, SUC_lexicon_tags_37 },
    { 0xc12f1c18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60f1c1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8f1c22, SUC_lexicon_tags_12 },
    { 0x9b6f1c23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4f1c2d, SUC_lexicon_tags_2 },
    { 0x7b0f1c2e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x228f1c34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eef1c3a, SUC_lexicon_tags_37 },
    { 0xea2f1c3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4f1c44, SUC_lexicon_tags_38 },
    { 0x86cf1c45, SUC_lexicon_tags_23 },
    { 0x4a0f1c46, SUC_lexicon_tags_10 },
    { 0x782f1c45, SUC_lexicon_tags_7 },
    { 0x236f1c48, SUC_lexicon_tags_49 },
    { 0xbf0f1c49, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0f1c4e, SUC_lexicon_tags_27 },
    { 0x48cf1c4f, SUC_lexicon_tags_17 },
    { 0xf12f1c4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720f1c58, SUC_lexicon_tags_12 },
    { 0x120f1c58, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x328f1c5b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3e2f1c5d, SUC_lexicon_tags_47 },
    { 0x630f1c5e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ef1c62, SUC_lexicon_tags_31 },
    { 0xe4af1c63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0cf1c65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398f1c6c, SUC_lexicon_tags_19 },
    { 0x64f1c6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412f1c72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x516f1c74, SUC_lexicon_tags_27 },
    { 0x262f1c74, SUC_lexicon_tags_1 },
    { 0xe94f1c76, SUC_lexicon_tags_49 },
    { 0x382f1c75, SUC_lexicon_tags_28 },
    { 0x210f1c78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb92f1c7a, SUC_lexicon_tags_37 },
    { 0x952f1c7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x122f1c7e, SUC_lexicon_tags_9 },
    { 0xcb6f1c7f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754f1c83, SUC_lexicon_tags_766 },
    { 0x872f1c83, SUC_lexicon_tags_2 },
    { 0x76af1c85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ef1c88, SUC_lexicon_tags_55 },
    { 0x766f1c89, SUC_lexicon_tags_1 },
    { 0x368f1c88, SUC_lexicon_tags_64 },
    { 0x35ef1c8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568f1c8f, SUC_lexicon_tags_47 },
    { 0xc4cf1c90, SUC_lexicon_tags_87 },
    { 0x382f1c91, SUC_lexicon_tags_13 },
    { 0xe50f1c91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12f1c98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x35ef1c9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbef1c9c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbcef1c9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34f1ca0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6f1ca3, SUC_lexicon_tags_4 },
    { 0xcbcf1ca4, SUC_lexicon_tags_11 },
    { 0x268f1ca3, SUC_lexicon_tags_3 },
    { 0xe3cf1ca6, SUC_lexicon_tags_10 },
    { 0x918f1ca6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8f1caa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x890f1cad, SUC_lexicon_tags_12 },
    { 0xe4af1cae, SUC_lexicon_tags_11 },
    { 0x278f1caf, SUC_lexicon_tags_37 },
    { 0xd90f1cb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0f1cb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfef1cb9, SUC_lexicon_tags_12 },
    { 0x7f6f1cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x462f1cbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x138f1cc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fef1cc2, SUC_lexicon_tags_29 },
    { 0xe40f1cc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648f1cc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x442f1cc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74f1cce, SUC_lexicon_tags_46 },
    { 0xf2ef1cce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ef1cd7, SUC_lexicon_tags_21 },
    { 0xfc0f1cd8, SUC_lexicon_tags_45 },
    { 0xc3ef1cd9, SUC_lexicon_tags_57 },
    { 0x502f1cda, SUC_lexicon_tags_60 },
    { 0x6b6f1cda, SUC_lexicon_tags_27 },
    { 0x334f1cdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x566f1cde, SUC_lexicon_tags_38 },
    { 0x47ef1cdf, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xc24f1ce1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xbecf1ce3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x24ef1ce5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcaaf1ce7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x31cf1cea, SUC_lexicon_tags_37 },
    { 0x26cf1ceb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcf1cef, SUC_lexicon_tags_15 },
    { 0xb64f1cf0, SUC_lexicon_tags_0 },
    { 0x818f1cf1, SUC_lexicon_tags_37 },
    { 0x674f1cf2, SUC_lexicon_tags_43 },
    { 0x3ef1cf2, SUC_lexicon_tags_19 },
    { 0xcd6f1cf3, SUC_lexicon_tags_47 },
    { 0xe96f1cf4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8f1cf8, SUC_lexicon_tags_1 },
    { 0x590f1cf9, SUC_lexicon_tags_0 },
    { 0xd8af1cf8, SUC_lexicon_tags_14 },
    { 0x39cf1cfb, SUC_lexicon_tags_37 },
    { 0x65cf1cfc, SUC_lexicon_tags_10 },
    { 0xdbef1cfd, SUC_lexicon_tags_47 },
    { 0x798f1cfa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x478f1d00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51af1d02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2f1d05, SUC_lexicon_tags_2 },
    { 0xafef1d06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0f1d09, SUC_lexicon_tags_35 },
    { 0x590f1d0a, SUC_lexicon_tags_2 },
    { 0x2ef1d0a, SUC_lexicon_tags_11 },
    { 0x54ef1d0c, SUC_lexicon_tags_17 },
    { 0x312f1d0d, SUC_lexicon_tags_17 },
    { 0x1daf1d0b, SUC_lexicon_tags_79 },
    { 0xa8ef1d0e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44f1d15, SUC_lexicon_tags_12 },
    { 0x506f1d15, SUC_lexicon_tags_63 },
    { 0x5aef1d15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610f1d1d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x720f1d1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3acf1d21, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x50f1d23, SUC_lexicon_tags_6 },
    { 0x67af1d24, SUC_lexicon_tags_9 },
    { 0x924f1d24, SUC_lexicon_tags_17 },
    { 0x658f1d25, SUC_lexicon_tags_11 },
    { 0x6b8f1d24, SUC_lexicon_tags_37 },
    { 0x80af1d26, SUC_lexicon_tags_6 },
    { 0x10cf1d28, SUC_lexicon_tags_27 },
    { 0xcd2f1d29, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552f1d2f, SUC_lexicon_tags_37 },
    { 0x3cef1d30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x596f1d32, SUC_lexicon_tags_12 },
    { 0x308f1d33, SUC_lexicon_tags_12 },
    { 0x4d2f1d32, SUC_lexicon_tags_17 },
    { 0x39ef1d32, SUC_lexicon_tags_37 },
    { 0xaa8f1d36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa0f1d38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ecf1d3a, SUC_lexicon_tags_17 },
    { 0xa84f1d3b, SUC_lexicon_tags_17 },
    { 0xab6f1d3a, SUC_lexicon_tags_2 },
    { 0x61af1d3d, SUC_lexicon_tags_0 },
    { 0x7acf1d3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50f1d40, SUC_lexicon_tags_66 },
    { 0x92f1d40, SUC_lexicon_tags_63 },
    { 0x884f1d42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa2f1d4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x582f1d50, SUC_lexicon_tags_25 },
    { 0x1d4f1d51, SUC_lexicon_tags_2 },
    { 0x636f1d52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70af1d57, SUC_lexicon_tags_12 },
    { 0xe40f1d58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2f1d5c, SUC_lexicon_tags_2 },
    { 0x3dcf1d5d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x81ef1d5f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7cf1d61, SUC_lexicon_tags_31 },
    { 0x79cf1d62, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834f1d67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8f1d6b, SUC_lexicon_tags_11 },
    { 0xe94f1d6c, SUC_lexicon_tags_1 },
    { 0x85cf1d6b, SUC_lexicon_tags_11 },
    { 0xcbaf1d6d, SUC_lexicon_tags_37 },
    { 0x53cf1d6f, SUC_lexicon_tags_65 },
    { 0x56ef1d70, SUC_lexicon_tags_13 },
    { 0x714f1d6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6f1d7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6cf1d7d, SUC_lexicon_tags_17 },
    { 0x2ef1d7e, SUC_lexicon_tags_60 },
    { 0x63af1d7f, SUC_lexicon_tags_25 },
    { 0xe0cf1d80, SUC_lexicon_tags_38 },
    { 0x428f1d7e, SUC_lexicon_tags_17 },
    { 0xc4f1d82, SUC_lexicon_tags_48 },
    { 0xbb0f1d7d, SUC_lexicon_tags_37 },
    { 0x592f1d84, SUC_lexicon_tags_7 },
    { 0x1f8f1d84, SUC_lexicon_tags_64 },
    { 0x78af1d86, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16af1d8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfef1d91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ef1d95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ccf1d97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc10f1d99, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaf1d9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0f1da0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624f1da5, SUC_lexicon_tags_37 },
    { 0x736f1da6, SUC_lexicon_tags_7 },
    { 0xc3ef1da5, SUC_lexicon_tags_11 },
    { 0x8e6f1da7, SUC_lexicon_tags_11 },
    { 0x1b4f1da8, SUC_lexicon_tags_0 },
    { 0x8af1daa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77af1dae, SUC_lexicon_tags_69 },
    { 0xd3ef1daf, SUC_lexicon_tags_29 },
    { 0x3c0f1dae, SUC_lexicon_tags_2 },
    { 0x858f1db0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x974f1db3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6f1dbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4f1dbe, SUC_lexicon_tags_1 },
    { 0xc1ef1dbe, SUC_lexicon_tags_26 },
    { 0xb8cf1dc0, SUC_lexicon_tags_21 },
    { 0xa40f1dbe, SUC_lexicon_tags_21 },
    { 0xc52f1dc2, SUC_lexicon_tags_8 },
    { 0x510f1dc1, SUC_lexicon_tags_47 },
    { 0x684f1dc4, SUC_lexicon_tags_25 },
    { 0xa40f1dbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x592f1dc8, SUC_lexicon_tags_37 },
    { 0xceaf1dc9, SUC_lexicon_tags_12 },
    { 0x53af1dca, SUC_lexicon_tags_17 },
    { 0xe84f1dcb, SUC_lexicon_tags_45 },
    { 0xcacf1dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x676f1dce, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570f1dd7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf10f1dd9, SUC_lexicon_tags_37 },
    { 0x42f1dd9, SUC_lexicon_tags_17 },
    { 0x9d2f1ddb, SUC_lexicon_tags_7 },
    { 0x2caf1ddc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x294f1ddf, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x364f1de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e2f1de3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6f1de6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0f1de9, SUC_lexicon_tags_6 },
    { 0xb5ef1de9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3dcf1dec, SUC_lexicon_tags_26 },
    { 0x8aaf1dec, SUC_lexicon_tags_25 },
    { 0xea2f1ded, SUC_lexicon_tags_8 },
    { 0x3e4f1def, SUC_lexicon_tags_68 },
    { 0xfeef1def, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20f1df4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92af1df8, SUC_lexicon_tags_96 },
    { 0x600f1df8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x630f1dfc, SUC_lexicon_tags_49 },
    { 0x280f1dfd, SUC_lexicon_tags_14 },
    { 0xdf6f1dfe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cf1e05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x470f1e08, SUC_lexicon_tags_15 },
    { 0x982f1e08, SUC_lexicon_tags_37 },
    { 0x732f1e0a, SUC_lexicon_tags_25 },
    { 0xc7ef1e0b, SUC_lexicon_tags_17 },
    { 0x71cf1e0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x31af1e0f, SUC_lexicon_tags_282 },
    { 0xc04f1e10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa30f1e12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fef1e16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x936f1e19, SUC_lexicon_tags_60 },
    { 0xc2cf1e19, SUC_lexicon_tags_17 },
    { 0xd94f1e1b, SUC_lexicon_tags_27 },
    { 0x898f1e19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ef1e25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ef1e28, SUC_lexicon_tags_4 },
    { 0x8e8f1e29, SUC_lexicon_tags_6 },
    { 0xf1ef1e2a, SUC_lexicon_tags_18 },
    { 0x238f1e28, SUC_lexicon_tags_47 },
    { 0x770f1e2c, SUC_lexicon_tags_87 },
    { 0x3e2f1e2d, SUC_lexicon_tags_23 },
    { 0x6cf1e2e, SUC_lexicon_tags_29 },
    { 0x690f1e2c, SUC_lexicon_tags_2 },
    { 0xdfaf1e30, SUC_lexicon_tags_7 },
    { 0x11cf1e2e, SUC_lexicon_tags_11 },
    { 0x5a6f1e32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x340f1e35, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x814f1e38, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf1e3b, SUC_lexicon_tags_45 },
    { 0x7a0f1e3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1cf1e3e, SUC_lexicon_tags_151 },
    { 0xbd8f1e3f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36f1e43, SUC_lexicon_tags_63 },
    { 0xf96f1e43, SUC_lexicon_tags_2 },
    { 0xbaf1e45, SUC_lexicon_tags_29 },
    { 0x37cf1e43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d6f1e48, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4f1e4b, SUC_lexicon_tags_8 },
    { 0xb8cf1e4c, SUC_lexicon_tags_7 },
    { 0x5e8f1e4c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x366f1e4f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaf1e52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9af1e54, SUC_lexicon_tags_47 },
    { 0xde2f1e55, SUC_lexicon_tags_14 },
    { 0x772f1e56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96f1e5b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a4f1e60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78f1e63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x332f1e66, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcef1e6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f1e6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41cf1e70, SUC_lexicon_tags_25 },
    { 0xbe6f1e71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366f1e75, SUC_lexicon_tags_7 },
    { 0x3c2f1e76, SUC_lexicon_tags_7 },
    { 0x616f1e75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x182f1e79, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7af1e7e, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8f1e88, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x658f1e8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f1e8e, SUC_lexicon_tags_43 },
    { 0x5aef1e8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eef1e95, SUC_lexicon_tags_15 },
    { 0xd98f1e96, SUC_lexicon_tags_12 },
    { 0x20f1e96, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xc96f1e99, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86f1ea5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c2f1ea7, SUC_lexicon_tags_12 },
    { 0xf1ef1ea8, SUC_lexicon_tags_68 },
    { 0x728f1ea7, SUC_lexicon_tags_13 },
    { 0x608f1eaa, SUC_lexicon_tags_6 },
    { 0xc78f1ea8, SUC_lexicon_tags_17 },
    { 0x704f1ea9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90f1eb0, SUC_lexicon_tags_69 },
    { 0x364f1eb0, SUC_lexicon_tags_11 },
    { 0xab2f1eb2, SUC_lexicon_tags_47 },
    { 0xeaef1eb3, SUC_lexicon_tags_7 },
    { 0xb48f1eb3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6f1eb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cf1ebe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684f1ec2, SUC_lexicon_tags_27 },
    { 0xa5cf1ec3, SUC_lexicon_tags_11 },
    { 0x9f8f1ec4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226f1eca, SUC_lexicon_tags_29 },
    { 0x22cf1eca, SUC_lexicon_tags_82 },
    { 0xa0ef1eca, SUC_lexicon_tags_5 },
    { 0xc66f1ecb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x59ef1ecf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ef1ed2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1eda, SUC_lexicon_tags_21 },
    { 0xccaf1edb, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xd4f1edd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746f1ee1, SUC_lexicon_tags_29 },
    { 0x9a0f1ee1, SUC_lexicon_tags_14 },
    { 0xa30f1ee1, SUC_lexicon_tags_11 },
    { 0x682f1ee4, SUC_lexicon_tags_37 },
    { 0x5cf1ee4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50f1ee8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf88f1eea, SUC_lexicon_tags_37 },
    { 0x708f1eeb, SUC_lexicon_tags_63 },
    { 0x706f1eeb, SUC_lexicon_tags_39 },
    { 0x98f1eeb, SUC_lexicon_tags_19 },
    { 0xf1ef1eec, SUC_lexicon_tags_2 },
    { 0x442f1eef, SUC_lexicon_tags_11 },
    { 0x130f1ef0, SUC_lexicon_tags_17 },
    { 0xc12f1eec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x624f1ef3, SUC_lexicon_tags_19 },
    { 0xc00f1ef4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cf1ef8, SUC_lexicon_tags_7 },
    { 0x8f6f1ef9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862f1efd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x200f1eff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfef1f02, SUC_lexicon_tags_11 },
    { 0xddcf1f03, SUC_lexicon_tags_11 },
    { 0x198f1f03, SUC_lexicon_tags_9 },
    { 0xb62f1f05, SUC_lexicon_tags_13 },
    { 0x966f1f02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcf1f0b, SUC_lexicon_tags_17 },
    { 0x33cf1f0c, SUC_lexicon_tags_68 },
    { 0xdf6f1f0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8f1f11, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa6af1f13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2f1f18, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910f1f1f, SUC_lexicon_tags_68 },
    { 0x386f1f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x856f1f22, SUC_lexicon_tags_2 },
    { 0x474f1f23, SUC_lexicon_tags_15 },
    { 0x68ef1f24, SUC_lexicon_tags_1 },
    { 0x276f1f24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162f1f29, SUC_lexicon_tags_45 },
    { 0x784f1f2a, SUC_lexicon_tags_25 },
    { 0x84cf1f2b, SUC_lexicon_tags_14 },
    { 0x116f1f2c, SUC_lexicon_tags_31 },
    { 0xf12f1f2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x276f1f30, SUC_lexicon_tags_25 },
    { 0x40f1f31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cf1f38, SUC_lexicon_tags_17 },
    { 0x8e4f1f38, SUC_lexicon_tags_3 },
    { 0xfcf1f39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd76f1f3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcef1f40, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250f1f44, SUC_lexicon_tags_2 },
    { 0xae4f1f44, SUC_lexicon_tags_47 },
    { 0xca8f1f44, SUC_lexicon_tags_27 },
    { 0x766f1f44, SUC_lexicon_tags_56 },
    { 0xc7af1f48, SUC_lexicon_tags_5 },
    { 0x172f1f49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x852f1f4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8f1f55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170f1f63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa5cf1f65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0f1f69, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8f1f6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cf1f75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x202f1f77, SUC_lexicon_tags_13 },
    { 0x40cf1f78, SUC_lexicon_tags_31 },
    { 0x3e4f1f79, SUC_lexicon_tags_7 },
    { 0xdd8f1f7a, SUC_lexicon_tags_37 },
    { 0xd5cf1f7b, SUC_lexicon_tags_0 },
    { 0xda4f1f7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cf1f83, SUC_lexicon_tags_11 },
    { 0xc88f1f84, SUC_lexicon_tags_25 },
    { 0x5a2f1f85, SUC_lexicon_tags_66 },
    { 0xc08f1f85, SUC_lexicon_tags_1316 },
    { 0xa8ef1f87, SUC_lexicon_tags_25 },
    { 0x5c8f1f88, SUC_lexicon_tags_4 },
    { 0x954f1f89, SUC_lexicon_tags_14 },
    { 0x586f1f85, SUC_lexicon_tags_17 },
    { 0x354f1f8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1c0f1f8d, SUC_lexicon_tags_15 },
    { 0x262f1f8e, SUC_lexicon_tags_19 },
    { 0xb26f1f8f, SUC_lexicon_tags_11 },
    { 0x702f1f90, SUC_lexicon_tags_2 },
    { 0xe90f1f91, SUC_lexicon_tags_10 },
    { 0xcc0f1f91, SUC_lexicon_tags_6 },
    { 0xb3ef1f92, SUC_lexicon_tags_32 },
    { 0x3fcf1f91, SUC_lexicon_tags_27 },
    { 0xb24f1f95, SUC_lexicon_tags_15 },
    { 0x67cf1f95, SUC_lexicon_tags_12 },
    { 0xe10f1f97, SUC_lexicon_tags_25 },
    { 0x634f1f97, SUC_lexicon_tags_25 },
    { 0x2d0f1f97, SUC_lexicon_tags_25 },
    { 0x1b6f1f99, SUC_lexicon_tags_57 },
    { 0x252f1f97, SUC_lexicon_tags_16 },
    { 0xd12f1f8e, SUC_lexicon_tags_1 },
    { 0x69ef1f90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ccf1f9f, SUC_lexicon_tags_39 },
    { 0x84ef1fa0, SUC_lexicon_tags_60 },
    { 0x134f1f9f, SUC_lexicon_tags_11 },
    { 0x6bcf1fa1, SUC_lexicon_tags_16 },
    { 0x80af1f9f, SUC_lexicon_tags_28 },
    { 0x83cf1fa4, SUC_lexicon_tags_14 },
    { 0x486f1fa5, SUC_lexicon_tags_15 },
    { 0xfcef1fa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fef1fa9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f1fb4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6daf1fb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x158f1fb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x474f1fba, SUC_lexicon_tags_1 },
    { 0x50cf1fbb, SUC_lexicon_tags_26 },
    { 0xa04f1fbc, SUC_lexicon_tags_33 },
    { 0x782f1fbb, SUC_lexicon_tags_54 },
    { 0xf28f1fbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x518f1fc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x580f1fc2, SUC_lexicon_tags_11 },
    { 0xcaaf1fc3, SUC_lexicon_tags_87 },
    { 0xbf8f1fc4, SUC_lexicon_tags_27 },
    { 0x8c8f1fc4, SUC_lexicon_tags_7 },
    { 0x328f1fc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40f1fcc, SUC_lexicon_tags_2 },
    { 0x868f1fcd, SUC_lexicon_tags_37 },
    { 0xd48f1fce, SUC_lexicon_tags_0 },
    { 0xee6f1fcf, SUC_lexicon_tags_43 },
    { 0xdeaf1fcf, SUC_lexicon_tags_15 },
    { 0x590f1fd1, SUC_lexicon_tags_17 },
    { 0xb84f1fd1, SUC_lexicon_tags_101 },
    { 0x37cf1fd3, SUC_lexicon_tags_21 },
    { 0x6cf1fd3, SUC_lexicon_tags_87 },
    { 0x3ef1fd5, SUC_lexicon_tags_7 },
    { 0x782f1fd6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4f1fd9, SUC_lexicon_tags_66 },
    { 0x7b2f1fd9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x316f1fdd, SUC_lexicon_tags_79 },
    { 0x180f1fde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb80f1fe0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x128f1fe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc12f1fe4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622f1fea, SUC_lexicon_tags_43 },
    { 0xa62f1feb, SUC_lexicon_tags_11 },
    { 0x21cf1fec, SUC_lexicon_tags_1 },
    { 0x446f1fed, SUC_lexicon_tags_37 },
    { 0x2eaf1fee, SUC_lexicon_tags_14 },
    { 0x60f1fee, SUC_lexicon_tags_60 },
    { 0xec4f1feb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4f1ff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58f1ffe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d0f2000, SUC_lexicon_tags_45 },
    { 0xcf0f2001, SUC_lexicon_tags_15 },
    { 0x7f2f2002, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x164f2004, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312f2008, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x648f200b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x634f200d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb6af200f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70f2012, SUC_lexicon_tags_1158 },
    { 0xe54f2013, SUC_lexicon_tags_29 },
    { 0xfa2f2014, SUC_lexicon_tags_7 },
    { 0x7d6f2014, SUC_lexicon_tags_7 },
    { 0x150f2016, SUC_lexicon_tags_17 },
    { 0xf98f2017, SUC_lexicon_tags_7 },
    { 0xce0f2017, SUC_lexicon_tags_15 },
    { 0xdc8f2019, SUC_lexicon_tags_17 },
    { 0x7d6f201a, SUC_lexicon_tags_39 },
    { 0xed6f201a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x942f2020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x790f2024, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x48cf2027, SUC_lexicon_tags_12 },
    { 0x7ccf2027, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc80f202a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x380f202c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x664f202e, SUC_lexicon_tags_11 },
    { 0x734f202f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f2032, SUC_lexicon_tags_25 },
    { 0xe42f2032, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cf2037, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1cf2039, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ef203c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x678f203e, SUC_lexicon_tags_15 },
    { 0x258f203f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f2045, SUC_lexicon_tags_1 },
    { 0x6dcf2046, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6def204d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ef2055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa70f2057, SUC_lexicon_tags_42 },
    { 0xa5ef2057, SUC_lexicon_tags_17 },
    { 0x366f2059, SUC_lexicon_tags_29 },
    { 0xf82f205a, SUC_lexicon_tags_29 },
    { 0x9d6f205b, SUC_lexicon_tags_37 },
    { 0x22ef2057, SUC_lexicon_tags_37 },
    { 0xcfcf2057, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2f2065, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0f2068, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x992f206b, SUC_lexicon_tags_17 },
    { 0x210f206c, SUC_lexicon_tags_0 },
    { 0x522f206d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x406f206f, SUC_lexicon_tags_26 },
    { 0xd72f2070, SUC_lexicon_tags_23 },
    { 0x1e2f2070, SUC_lexicon_tags_17 },
    { 0xf0ef2072, SUC_lexicon_tags_4 },
    { 0xa4f2071, SUC_lexicon_tags_45 },
    { 0x5bcf2072, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8faf2076, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x992f207b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6f207f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0f2084, SUC_lexicon_tags_2 },
    { 0x66f2085, SUC_lexicon_tags_27 },
    { 0x5aef2086, SUC_lexicon_tags_2 },
    { 0x4aef2087, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x964f2089, SUC_lexicon_tags_37 },
    { 0x920f208a, SUC_lexicon_tags_25 },
    { 0x774f208b, SUC_lexicon_tags_16 },
    { 0xdc6f208c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x704f2094, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x906f2096, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70f2099, SUC_lexicon_tags_17 },
    { 0x166f209a, SUC_lexicon_tags_14 },
    { 0x93ef209b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918f209f, SUC_lexicon_tags_2 },
    { 0xa7cf20a0, SUC_lexicon_tags_16 },
    { 0x70f20a1, SUC_lexicon_tags_45 },
    { 0xde6f20a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59cf20a4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc58f20a6, SUC_lexicon_tags_47 },
    { 0xecef20a7, SUC_lexicon_tags_61 },
    { 0xb6af20a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6f20ab, SUC_lexicon_tags_37 },
    { 0xc32f20ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd52f20ae, SUC_lexicon_tags_17 },
    { 0x4acf20ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4f20b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6f20b7, SUC_lexicon_tags_12 },
    { 0x504f20b8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfc6f20ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x832f20bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x772f20c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8f20c9, SUC_lexicon_tags_10 },
    { 0x162f20ca, SUC_lexicon_tags_6 },
    { 0x954f20c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x846f20cd, SUC_lexicon_tags_14 },
    { 0x816f20ce, SUC_lexicon_tags_37 },
    { 0x84f20cf, SUC_lexicon_tags_47 },
    { 0xb5cf20d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612f20d5, SUC_lexicon_tags_72 },
    { 0xa20f20d5, SUC_lexicon_tags_1 },
    { 0x3ccf20d6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2f20dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebef20e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa26f20e3, SUC_lexicon_tags_37 },
    { 0xd0cf20e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ef20e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4f20ea, SUC_lexicon_tags_65 },
    { 0xe62f20ea, SUC_lexicon_tags_45 },
    { 0x21ef20ea, SUC_lexicon_tags_37 },
    { 0x384f20ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324f20f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f20f4, SUC_lexicon_tags_37 },
    { 0x1ef20f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6af20f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef20fc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x2f2f20fe, SUC_lexicon_tags_77 },
    { 0x2e6f20ff, SUC_lexicon_tags_25 },
    { 0x506f20ff, SUC_lexicon_tags_60 },
    { 0x2ccf20ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670f2107, SUC_lexicon_tags_14 },
    { 0xa48f2108, SUC_lexicon_tags_12 },
    { 0x2c4f2108, SUC_lexicon_tags_65 },
    { 0xebcf210a, SUC_lexicon_tags_47 },
    { 0x512f210b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3af210f, SUC_lexicon_tags_20 },
    { 0xc5ef2110, SUC_lexicon_tags_37 },
    { 0xb34f2110, SUC_lexicon_tags_7 },
    { 0x832f210f, SUC_lexicon_tags_29 },
    { 0xde8f2113, SUC_lexicon_tags_25 },
    { 0x224f2112, SUC_lexicon_tags_3 },
    { 0x22f210f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x27cf2118, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02f211b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdecf211d, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06f2121, SUC_lexicon_tags_38 },
    { 0xdecf2122, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f2126, SUC_lexicon_tags_64 },
    { 0x782f2127, SUC_lexicon_tags_47 },
    { 0xfb0f2127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a0f212a, SUC_lexicon_tags_30 },
    { 0xa8f212a, SUC_lexicon_tags_2 },
    { 0xcccf212c, SUC_lexicon_tags_8 },
    { 0xec8f212d, SUC_lexicon_tags_25 },
    { 0x1af212d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cf2133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef2136, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f213a, SUC_lexicon_tags_1 },
    { 0xf5cf213a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4f213e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8af2142, SUC_lexicon_tags_60 },
    { 0xb1ef2143, SUC_lexicon_tags_79 },
    { 0x8ecf2144, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caf2149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f6f214b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a8f214e, SUC_lexicon_tags_37 },
    { 0xa16f214f, SUC_lexicon_tags_31 },
    { 0x2f0f214e, SUC_lexicon_tags_25 },
    { 0xeb2f2150, SUC_lexicon_tags_12 },
    { 0x368f2152, SUC_lexicon_tags_6 },
    { 0xb4ef2152, SUC_lexicon_tags_10 },
    { 0x28f2151, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8f2158, SUC_lexicon_tags_17 },
    { 0x30cf2158, SUC_lexicon_tags_32 },
    { 0xf08f2158, SUC_lexicon_tags_25 },
    { 0x164f215b, SUC_lexicon_tags_12 },
    { 0xbb8f215a, SUC_lexicon_tags_34 },
    { 0x5d6f215d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108f2161, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf216b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9cf216d, SUC_lexicon_tags_60 },
    { 0x736f216e, SUC_lexicon_tags_11 },
    { 0x6daf216d, SUC_lexicon_tags_11 },
    { 0x1fcf2170, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612f2175, SUC_lexicon_tags_6 },
    { 0x724f2176, SUC_lexicon_tags_14 },
    { 0x118f2177, SUC_lexicon_tags_55 },
    { 0xf2ef2178, SUC_lexicon_tags_11 },
    { 0x1ccf2179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc62f217b, SUC_lexicon_tags_28 },
    { 0xa34f217c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396f218b, SUC_lexicon_tags_302 },
    { 0xe58f218b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x776f2192, SUC_lexicon_tags_25 },
    { 0x86cf2193, SUC_lexicon_tags_9 },
    { 0xbc4f2193, SUC_lexicon_tags_11 },
    { 0xc46f2195, SUC_lexicon_tags_38 },
    { 0xdf2f2196, SUC_lexicon_tags_71 },
    { 0x404f2196, SUC_lexicon_tags_45 },
    { 0xe7af2196, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8f219b, SUC_lexicon_tags_1 },
    { 0xfccf219b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346f21a0, SUC_lexicon_tags_1 },
    { 0x73af21a0, SUC_lexicon_tags_46 },
    { 0x494f21a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa76f21a4, SUC_lexicon_tags_1 },
    { 0x54f21a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cf21a8, SUC_lexicon_tags_38 },
    { 0xcaef21a9, SUC_lexicon_tags_38 },
    { 0x560f21a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864f21b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcccf21b5, SUC_lexicon_tags_28 },
    { 0xaeaf21b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404f21bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46f21bf, SUC_lexicon_tags_12 },
    { 0x42f21bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64f21c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856f21ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faf21cf, SUC_lexicon_tags_8 },
    { 0x5b2f21d0, SUC_lexicon_tags_25 },
    { 0x820f21d0, SUC_lexicon_tags_17 },
    { 0x1ecf21cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x304f21d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeaef21d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe74f21d8, SUC_lexicon_tags_29 },
    { 0x794f21d9, SUC_lexicon_tags_34 },
    { 0xf40f21da, SUC_lexicon_tags_2 },
    { 0xb2cf21da, SUC_lexicon_tags_16 },
    { 0x68cf21dc, SUC_lexicon_tags_11 },
    { 0xa46f21dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ef21e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ef21e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42f21e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcaf21e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb08f21f0, SUC_lexicon_tags_56 },
    { 0x852f21f1, SUC_lexicon_tags_0 },
    { 0x6fcf21f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x522f21f5, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86f21f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaef21fd, SUC_lexicon_tags_29 },
    { 0xc1af21fe, SUC_lexicon_tags_21 },
    { 0x356f21fe, SUC_lexicon_tags_37 },
    { 0x586f2200, SUC_lexicon_tags_2 },
    { 0xe56f21ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44f2204, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fcf2206, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674f220a, SUC_lexicon_tags_26 },
    { 0x168f220b, SUC_lexicon_tags_46 },
    { 0xd76f220b, SUC_lexicon_tags_87 },
    { 0xacf220b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57cf220f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cf2212, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76af2215, SUC_lexicon_tags_27 },
    { 0x58cf2215, SUC_lexicon_tags_37 },
    { 0xc72f2217, SUC_lexicon_tags_37 },
    { 0x7c4f2215, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x454f221a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58f221f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eaf2224, SUC_lexicon_tags_28 },
    { 0x98af2225, SUC_lexicon_tags_15 },
    { 0x6f6f2224, SUC_lexicon_tags_56 },
    { 0x6af2226, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cf222a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0f2232, SUC_lexicon_tags_55 },
    { 0xd2af2233, SUC_lexicon_tags_15 },
    { 0xcdef2233, SUC_lexicon_tags_47 },
    { 0x882f2235, SUC_lexicon_tags_39 },
    { 0xec0f2236, SUC_lexicon_tags_11 },
    { 0xe16f2236, SUC_lexicon_tags_37 },
    { 0xd4af2238, SUC_lexicon_tags_7 },
    { 0x8f6f2239, SUC_lexicon_tags_65 },
    { 0xc54f223a, SUC_lexicon_tags_33 },
    { 0xba6f2234, SUC_lexicon_tags_82 },
    { 0x226f223c, SUC_lexicon_tags_84 },
    { 0x350f2238, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x594f2240, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732f2248, SUC_lexicon_tags_7 },
    { 0xd50f2249, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92f224e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4f2251, SUC_lexicon_tags_12 },
    { 0xe0cf2252, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cf225c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7af225e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x450f2260, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x83af2262, SUC_lexicon_tags_19 },
    { 0x4cf2263, SUC_lexicon_tags_92 },
    { 0x63ef2263, SUC_lexicon_tags_17 },
    { 0xa0cf2265, SUC_lexicon_tags_25 },
    { 0x7cef2264, SUC_lexicon_tags_75 },
    { 0xb68f2267, SUC_lexicon_tags_0 },
    { 0x234f2268, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ef226c, SUC_lexicon_tags_17 },
    { 0x3a8f226d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5af2270, SUC_lexicon_tags_37 },
    { 0x69af2271, SUC_lexicon_tags_33 },
    { 0x8d2f2271, SUC_lexicon_tags_127 },
    { 0x394f2273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cf2278, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc2f227a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4f2280, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x57cf2282, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56af2287, SUC_lexicon_tags_31 },
    { 0x9dcf2288, SUC_lexicon_tags_26 },
    { 0xbb4f2289, SUC_lexicon_tags_38 },
    { 0x266f228a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ef228d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca6f228f, SUC_lexicon_tags_60 },
    { 0xfb8f2290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98f2297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef229c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x67cf229e, SUC_lexicon_tags_53 },
    { 0xa04f229e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ef22a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ef22b0, SUC_lexicon_tags_15 },
    { 0x768f22b0, SUC_lexicon_tags_30 },
    { 0x22f22b2, SUC_lexicon_tags_79 },
    { 0x8af22b3, SUC_lexicon_tags_27 },
    { 0xb28f22b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf90f22b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8f22ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef22be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81af22c0, SUC_lexicon_tags_38 },
    { 0x844f22c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6fef22c3, SUC_lexicon_tags_29 },
    { 0x166f22c3, SUC_lexicon_tags_27 },
    { 0x780f22c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe86f22c7, SUC_lexicon_tags_17 },
    { 0x300f22c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe54f22ca, SUC_lexicon_tags_38 },
    { 0x378f22cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8f22d4, SUC_lexicon_tags_37 },
    { 0x2c8f22d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72f22d8, SUC_lexicon_tags_17 },
    { 0x520f22d8, SUC_lexicon_tags_19 },
    { 0x61af22da, SUC_lexicon_tags_33 },
    { 0x4f4f22db, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810f22e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6f22e5, SUC_lexicon_tags_17 },
    { 0x7c6f22e6, SUC_lexicon_tags_30 },
    { 0x6e6f22e7, SUC_lexicon_tags_23 },
    { 0xf36f22e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94f22eb, SUC_lexicon_tags_1 },
    { 0x178f22eb, SUC_lexicon_tags_15 },
    { 0x376f22eb, SUC_lexicon_tags_37 },
    { 0xff0f22ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2f22f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0f22f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08f22fd, SUC_lexicon_tags_7 },
    { 0x314f22fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x622f2300, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12af2306, SUC_lexicon_tags_12 },
    { 0x114f2307, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x940f2309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x874f230b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2f230e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x612f2310, SUC_lexicon_tags_7 },
    { 0x66cf2310, SUC_lexicon_tags_60 },
    { 0x98f2312, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f2315, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x270f2318, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0f231d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18f2322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4f2325, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf2f2327, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944f2331, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74f2334, SUC_lexicon_tags_37 },
    { 0x7b0f2335, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ef2338, SUC_lexicon_tags_15 },
    { 0x932f2339, SUC_lexicon_tags_54 },
    { 0x69af233a, SUC_lexicon_tags_1 },
    { 0x2af2338, SUC_lexicon_tags_11 },
    { 0xee8f233b, SUC_lexicon_tags_11 },
    { 0xec2f2338, SUC_lexicon_tags_14 },
    { 0x7c4f233e, SUC_lexicon_tags_19 },
    { 0x930f2339, SUC_lexicon_tags_16 },
    { 0xf8cf2340, SUC_lexicon_tags_0 },
    { 0xa12f233f, SUC_lexicon_tags_12 },
    { 0x94cf2342, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcf2349, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ef2353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57ef2355, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ef2358, SUC_lexicon_tags_151 },
    { 0xf5ef2358, SUC_lexicon_tags_14 },
    { 0x17af235a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54f2360, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4f2367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd60f236b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f4f236d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266f2373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ef237a, SUC_lexicon_tags_17 },
    { 0x42af237a, SUC_lexicon_tags_30 },
    { 0xbd4f237c, SUC_lexicon_tags_17 },
    { 0x12f237d, SUC_lexicon_tags_11 },
    { 0xf68f237c, SUC_lexicon_tags_38 },
    { 0x626f237a, SUC_lexicon_tags_11 },
    { 0xb24f237a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f2385, SUC_lexicon_tags_43 },
    { 0xa76f2386, SUC_lexicon_tags_30 },
    { 0x9faf2386, SUC_lexicon_tags_11 },
    { 0xfb4f2385, SUC_lexicon_tags_27 },
    { 0x52cf2389, SUC_lexicon_tags_293 },
    { 0xbef2389, SUC_lexicon_tags_17 },
    { 0x1f0f2389, SUC_lexicon_tags_25 },
    { 0xa38f2389, SUC_lexicon_tags_1 },
    { 0x6fef238c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18af2392, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbc6f2394, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa88f2396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cf23a0, SUC_lexicon_tags_37 },
    { 0x3d4f23a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f23a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x822f23a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0af23aa, SUC_lexicon_tags_40 },
    { 0x500f23ab, SUC_lexicon_tags_30 },
    { 0x78ef23ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482f23b3, SUC_lexicon_tags_11 },
    { 0x6a4f23b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccf23b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2f23bd, SUC_lexicon_tags_7 },
    { 0x71ef23bd, SUC_lexicon_tags_11 },
    { 0x988f23bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d2f23c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f23c9, SUC_lexicon_tags_82 },
    { 0x114f23ca, SUC_lexicon_tags_29 },
    { 0xbdef23cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x722f23cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x134f23d0, SUC_lexicon_tags_15 },
    { 0x674f23d1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xcb8f23d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2f23da, SUC_lexicon_tags_7 },
    { 0x8c0f23da, SUC_lexicon_tags_19 },
    { 0x3ecf23dc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30f23e8, SUC_lexicon_tags_31 },
    { 0x580f23e9, SUC_lexicon_tags_11 },
    { 0x8eef23ea, SUC_lexicon_tags_14 },
    { 0x4e8f23eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe98f23ed, SUC_lexicon_tags_15 },
    { 0x7c6f23ed, SUC_lexicon_tags_41 },
    { 0x86af23ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc00f23f1, SUC_lexicon_tags_16 },
    { 0x91ef23f2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x478f23f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaef23f7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6c0f23f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f23ff, SUC_lexicon_tags_30 },
    { 0x35af23ff, SUC_lexicon_tags_85 },
    { 0x82ef2401, SUC_lexicon_tags_1 },
    { 0x26cf2402, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x36af2405, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcf240a, SUC_lexicon_tags_25 },
    { 0xd90f240b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbbaf240d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xefef240f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ef2413, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426f241a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x456f241c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2f2424, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5eef2426, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf9cf2428, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0x154f242a, SUC_lexicon_tags_46 },
    { 0xdeef242b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x406f242d, SUC_lexicon_tags_33 },
    { 0x616f242e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f2432, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00f2435, SUC_lexicon_tags_33 },
    { 0x732f2436, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af243c, SUC_lexicon_tags_28 },
    { 0xd8ef243c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9dcf243f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4def2444, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2f2447, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cf244a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x334f244c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x49cf244e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x852f2451, SUC_lexicon_tags_15 },
    { 0xf46f2451, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6af2455, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x322f2457, SUC_lexicon_tags_25 },
    { 0xbb6f2458, SUC_lexicon_tags_11 },
    { 0xea0f2459, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59af245e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422f2462, SUC_lexicon_tags_16 },
    { 0x1a6f2463, SUC_lexicon_tags_7 },
    { 0x1b6f2464, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaeaf2466, SUC_lexicon_tags_1 },
    { 0x222f2466, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcccf246a, SUC_lexicon_tags_34 },
    { 0xcaef246a, SUC_lexicon_tags_21 },
    { 0x7b6f246c, SUC_lexicon_tags_7 },
    { 0x4ef246a, SUC_lexicon_tags_2 },
    { 0x2b2f246c, SUC_lexicon_tags_19 },
    { 0xd26f246f, SUC_lexicon_tags_17 },
    { 0xc98f2470, SUC_lexicon_tags_49 },
    { 0x7a8f2470, SUC_lexicon_tags_23 },
    { 0x2f246c, SUC_lexicon_tags_7 },
    { 0x5a8f2473, SUC_lexicon_tags_2 },
    { 0x880f2474, SUC_lexicon_tags_37 },
    { 0xe10f2470, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x126f2477, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434f247b, SUC_lexicon_tags_15 },
    { 0x734f247c, SUC_lexicon_tags_37 },
    { 0x182f247d, SUC_lexicon_tags_0 },
    { 0xb8cf247d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3def2480, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x178f2483, SUC_lexicon_tags_27 },
    { 0x900f2484, SUC_lexicon_tags_37 },
    { 0x8c6f2484, SUC_lexicon_tags_48 },
    { 0x438f2486, SUC_lexicon_tags_21 },
    { 0xe46f2487, SUC_lexicon_tags_14 },
    { 0x874f2486, SUC_lexicon_tags_37 },
    { 0xe16f2489, SUC_lexicon_tags_31 },
    { 0x41cf248a, SUC_lexicon_tags_17 },
    { 0xf36f2484, SUC_lexicon_tags_35 },
    { 0x816f248c, SUC_lexicon_tags_19 },
    { 0x70f248d, SUC_lexicon_tags_16 },
    { 0xf6ef2487, SUC_lexicon_tags_30 },
    { 0xf64f248f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5baf2492, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50f2497, SUC_lexicon_tags_17 },
    { 0x992f2497, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1daf249a, SUC_lexicon_tags_44 },
    { 0x16cf249b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6f249f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3b2f24a1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x968f24a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf90f24a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94af24b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cef24b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2f24bd, SUC_lexicon_tags_21 },
    { 0x9caf24be, SUC_lexicon_tags_7 },
    { 0x6f4f24bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08f24c8, SUC_lexicon_tags_17 },
    { 0xac4f24c9, SUC_lexicon_tags_12 },
    { 0x394f24c8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x888f24cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85cf24cf, SUC_lexicon_tags_55 },
    { 0xd00f24d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1cf24d2, SUC_lexicon_tags_28 },
    { 0xd94f24d3, SUC_lexicon_tags_30 },
    { 0x4d8f24d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6af24d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaf24db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cf24de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x180f24e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14af24e8, SUC_lexicon_tags_15 },
    { 0xc60f24e8, SUC_lexicon_tags_37 },
    { 0x92af24ea, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe80f24ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cf24f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148f24f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x146f24f8, SUC_lexicon_tags_255 },
    { 0xa32f24f9, SUC_lexicon_tags_63 },
    { 0x1f6f24f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35ef24fc, SUC_lexicon_tags_11 },
    { 0x6baf24fd, SUC_lexicon_tags_32 },
    { 0xb4f24fe, SUC_lexicon_tags_16 },
    { 0xaaaf24fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f2505, SUC_lexicon_tags_4 },
    { 0xb74f2506, SUC_lexicon_tags_19 },
    { 0xe7cf2506, SUC_lexicon_tags_46 },
    { 0x9fef2506, SUC_lexicon_tags_11 },
    { 0x1d4f2508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeaaf250b, SUC_lexicon_tags_34 },
    { 0xdf8f250c, SUC_lexicon_tags_28 },
    { 0xa74f250c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88f2512, SUC_lexicon_tags_37 },
    { 0xbcef2512, SUC_lexicon_tags_38 },
    { 0xd62f2514, SUC_lexicon_tags_37 },
    { 0x6fcf2514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58f251a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecf251f, SUC_lexicon_tags_23 },
    { 0x2c4f2520, SUC_lexicon_tags_32 },
    { 0x9f6f2521, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326f2529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x296f252c, SUC_lexicon_tags_16 },
    { 0xd6ef252d, SUC_lexicon_tags_43 },
    { 0x2a2f252e, SUC_lexicon_tags_8 },
    { 0xe6cf252f, SUC_lexicon_tags_25 },
    { 0xa9ef2530, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2f2534, SUC_lexicon_tags_17 },
    { 0xf6f2535, SUC_lexicon_tags_17 },
    { 0x87af2535, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ef2539, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f253d, SUC_lexicon_tags_77 },
    { 0x186f253e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x85cf2540, SUC_lexicon_tags_25 },
    { 0x3b4f2541, SUC_lexicon_tags_11 },
    { 0xde0f2541, SUC_lexicon_tags_27 },
    { 0x924f2543, SUC_lexicon_tags_14 },
    { 0x17af2543, SUC_lexicon_tags_95 },
    { 0x788f2540, SUC_lexicon_tags_27 },
    { 0x5c2f2546, SUC_lexicon_tags_2 },
    { 0xdb2f2543, SUC_lexicon_tags_17 },
    { 0xa1ef2540, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf254c, SUC_lexicon_tags_14 },
    { 0x158f254c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x308f2550, SUC_lexicon_tags_17 },
    { 0xfa8f2551, SUC_lexicon_tags_46 },
    { 0x8b4f2551, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810f2557, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e6f2559, SUC_lexicon_tags_31 },
    { 0x3aaf2559, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cf255d, SUC_lexicon_tags_51 },
    { 0xa7af255e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4c0f2560, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00f2567, SUC_lexicon_tags_37 },
    { 0x838f2568, SUC_lexicon_tags_23 },
    { 0xc50f2569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaaf256b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xffaf256d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84f2577, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2f257c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x480f257e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe34f2580, SUC_lexicon_tags_32 },
    { 0x22ef2580, SUC_lexicon_tags_11 },
    { 0x904f2580, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6f258a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc4f258c, SUC_lexicon_tags_50 },
    { 0x7def258d, SUC_lexicon_tags_0 },
    { 0x954f258e, SUC_lexicon_tags_1 },
    { 0x3f4f258f, SUC_lexicon_tags_11 },
    { 0xd76f258f, SUC_lexicon_tags_31 },
    { 0x76f2591, SUC_lexicon_tags_45 },
    { 0xc40f2592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9af2596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38f259a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xddaf259c, SUC_lexicon_tags_17 },
    { 0xcf4f259c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x194f259f, SUC_lexicon_tags_27 },
    { 0xdaaf25a0, SUC_lexicon_tags_38 },
    { 0x5eaf25a1, SUC_lexicon_tags_25 },
    { 0x966f25a2, SUC_lexicon_tags_60 },
    { 0x38af25a3, SUC_lexicon_tags_37 },
    { 0x1c8f25a3, SUC_lexicon_tags_11 },
    { 0x928f25a3, SUC_lexicon_tags_230 },
    { 0xfd8f25a4, SUC_lexicon_tags_12 },
    { 0xee0f25a5, SUC_lexicon_tags_26 },
    { 0x514f25a7, SUC_lexicon_tags_1 },
    { 0x362f25a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x940f25ab, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8f25b5, SUC_lexicon_tags_45 },
    { 0x568f25b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x16ef25b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e0f25ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aef25bd, SUC_lexicon_tags_25 },
    { 0xd58f25be, SUC_lexicon_tags_11 },
    { 0xaeaf25be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74cf25c1, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7af25c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8f25c8, SUC_lexicon_tags_38 },
    { 0xb10f25c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x828f25cc, SUC_lexicon_tags_11 },
    { 0x59ef25cc, SUC_lexicon_tags_100 },
    { 0xe48f25cc, SUC_lexicon_tags_30 },
    { 0x172f25cd, SUC_lexicon_tags_3 },
    { 0x1d2f25cf, SUC_lexicon_tags_10 },
    { 0x99ef25d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x950f25d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaf25db, SUC_lexicon_tags_6 },
    { 0xf60f25dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6caf25df, SUC_lexicon_tags_11 },
    { 0x84f25e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95af25e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0f25eb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcc8f25ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8f25f0, SUC_lexicon_tags_1 },
    { 0x8cf25f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cf25f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0f25fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x208f25fc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x630f25ff, SUC_lexicon_tags_2 },
    { 0xb36f2600, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236f2606, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf4cf2608, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f260e, SUC_lexicon_tags_27 },
    { 0x590f260f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0af2617, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30f261f, SUC_lexicon_tags_45 },
    { 0x44f261f, SUC_lexicon_tags_23 },
    { 0xabef2621, SUC_lexicon_tags_49 },
    { 0x54ef2622, SUC_lexicon_tags_12 },
    { 0xfc6f2621, SUC_lexicon_tags_13 },
    { 0x726f2624, SUC_lexicon_tags_25 },
    { 0xd12f2625, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8f2628, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x782f262c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3af262e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x370f2630, SUC_lexicon_tags_17 },
    { 0xb7ef2630, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8f4f2633, SUC_lexicon_tags_1 },
    { 0x534f2633, SUC_lexicon_tags_43 },
    { 0x52ef2633, SUC_lexicon_tags_11 },
    { 0x266f2636, SUC_lexicon_tags_2 },
    { 0x8f4f2637, SUC_lexicon_tags_1 },
    { 0xf8ef2637, SUC_lexicon_tags_1 },
    { 0x766f2637, SUC_lexicon_tags_27 },
    { 0xbe2f263a, SUC_lexicon_tags_7 },
    { 0x26af263b, SUC_lexicon_tags_35 },
    { 0xe20f2637, SUC_lexicon_tags_11 },
    { 0x384f263c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d8f263f, SUC_lexicon_tags_25 },
    { 0xb82f263f, SUC_lexicon_tags_1 },
    { 0x15ef2640, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xedef2643, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaf2649, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x740f264b, SUC_lexicon_tags_87 },
    { 0xefef264c, SUC_lexicon_tags_17 },
    { 0xdcef264b, SUC_lexicon_tags_12 },
    { 0x586f264e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa82f2651, SUC_lexicon_tags_12 },
    { 0x63ef2651, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2caf265a, SUC_lexicon_tags_22 },
    { 0x6e4f265b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedef2664, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2cf2666, SUC_lexicon_tags_7 },
    { 0x51cf2667, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2f266b, SUC_lexicon_tags_17 },
    { 0x54cf266c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7a2f266e, SUC_lexicon_tags_15 },
    { 0x1f8f266f, SUC_lexicon_tags_26 },
    { 0x3cef266f, SUC_lexicon_tags_87 },
    { 0x9ecf266f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x708f2673, SUC_lexicon_tags_7 },
    { 0xe30f2673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e4f2676, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x534f2679, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xb4ef267b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f267f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f2683, SUC_lexicon_tags_37 },
    { 0xff6f2684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10f2688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6af268a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf42f268c, SUC_lexicon_tags_7 },
    { 0x4daf268d, SUC_lexicon_tags_38 },
    { 0x138f268e, SUC_lexicon_tags_11 },
    { 0x9bef268f, SUC_lexicon_tags_2 },
    { 0x8e8f268f, SUC_lexicon_tags_0 },
    { 0x8c0f268f, SUC_lexicon_tags_19 },
    { 0x26f2692, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb16f2694, SUC_lexicon_tags_27 },
    { 0xb2cf2695, SUC_lexicon_tags_0 },
    { 0x1f8f2696, SUC_lexicon_tags_17 },
    { 0xb0af2697, SUC_lexicon_tags_17 },
    { 0xeecf2698, SUC_lexicon_tags_14 },
    { 0xd54f2699, SUC_lexicon_tags_40 },
    { 0xa1ef2699, SUC_lexicon_tags_9 },
    { 0x592f2698, SUC_lexicon_tags_47 },
    { 0xc6f2699, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x148f269f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c8f26a1, SUC_lexicon_tags_37 },
    { 0x286f26a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cf26a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6acf26a7, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf26ab, SUC_lexicon_tags_37 },
    { 0xb76f26ac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf68f26b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfaf26b2, SUC_lexicon_tags_25 },
    { 0xc20f26b2, SUC_lexicon_tags_15 },
    { 0xecf26b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3caf26b6, SUC_lexicon_tags_63 },
    { 0x6c0f26b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2f26ba, SUC_lexicon_tags_23 },
    { 0xbfcf26bb, SUC_lexicon_tags_14 },
    { 0xaf2f26bc, SUC_lexicon_tags_4 },
    { 0xdcf26bc, SUC_lexicon_tags_37 },
    { 0x534f26be, SUC_lexicon_tags_29 },
    { 0xd34f26be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcf26c2, SUC_lexicon_tags_43 },
    { 0xe94f26c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe40f26c5, SUC_lexicon_tags_25 },
    { 0x44cf26c6, SUC_lexicon_tags_60 },
    { 0x2e0f26c7, SUC_lexicon_tags_7 },
    { 0xd76f26c8, SUC_lexicon_tags_2 },
    { 0x82ef26c8, SUC_lexicon_tags_61 },
    { 0x716f26ca, SUC_lexicon_tags_77 },
    { 0xb4ef26ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952f26cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc34f26d6, SUC_lexicon_tags_7 },
    { 0xc4cf26d6, SUC_lexicon_tags_65 },
    { 0xb9ef26d7, SUC_lexicon_tags_61 },
    { 0x8dcf26d8, SUC_lexicon_tags_2 },
    { 0xf2cf26da, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xc90f26dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70f26e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26af26e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc94f26e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecf26f0, SUC_lexicon_tags_25 },
    { 0x97ef26f0, SUC_lexicon_tags_38 },
    { 0x80af26f2, SUC_lexicon_tags_15 },
    { 0xafcf26f3, SUC_lexicon_tags_37 },
    { 0x368f26f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35cf26f7, SUC_lexicon_tags_60 },
    { 0x14cf26f8, SUC_lexicon_tags_3 },
    { 0xd58f26f9, SUC_lexicon_tags_1 },
    { 0xeaf26f9, SUC_lexicon_tags_38 },
    { 0xa84f26fb, SUC_lexicon_tags_37 },
    { 0xd28f26fc, SUC_lexicon_tags_63 },
    { 0xdccf26fd, SUC_lexicon_tags_17 },
    { 0xa72f26fb, SUC_lexicon_tags_138 },
    { 0x87cf26fe, SUC_lexicon_tags_3 },
    { 0xd02f26f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x118f2702, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6f2706, SUC_lexicon_tags_2 },
    { 0x2daf2707, SUC_lexicon_tags_17 },
    { 0x6bcf2707, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x774f270b, SUC_lexicon_tags_8 },
    { 0xba6f270c, SUC_lexicon_tags_23 },
    { 0x2c4f270c, SUC_lexicon_tags_12 },
    { 0x3b0f270e, SUC_lexicon_tags_14 },
    { 0x5f2f270e, SUC_lexicon_tags_33 },
    { 0xd06f2710, SUC_lexicon_tags_19 },
    { 0x320f2710, SUC_lexicon_tags_18 },
    { 0xebaf2712, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b0f2714, SUC_lexicon_tags_2 },
    { 0xdbaf2714, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cf2718, SUC_lexicon_tags_5 },
    { 0x6caf2719, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46f271c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c6f271e, SUC_lexicon_tags_37 },
    { 0x8b8f271e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332f272a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828f272f, SUC_lexicon_tags_160 },
    { 0x7a2f2730, SUC_lexicon_tags_10 },
    { 0xe2cf272f, SUC_lexicon_tags_1 },
    { 0x61cf2732, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef273a, SUC_lexicon_tags_12 },
    { 0xa2f273b, SUC_lexicon_tags_25 },
    { 0x748f273c, SUC_lexicon_tags_37 },
    { 0x14f273d, SUC_lexicon_tags_37 },
    { 0x768f273d, SUC_lexicon_tags_9 },
    { 0xc44f273f, SUC_lexicon_tags_15 },
    { 0x512f273d, SUC_lexicon_tags_33 },
    { 0xd68f2741, SUC_lexicon_tags_45 },
    { 0x16f2742, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x576f2745, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x398f2748, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14f274d, SUC_lexicon_tags_0 },
    { 0xda2f274e, SUC_lexicon_tags_11 },
    { 0x78af274f, SUC_lexicon_tags_2 },
    { 0xcecf2750, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba4f2752, SUC_lexicon_tags_16 },
    { 0x7caf2752, SUC_lexicon_tags_14 },
    { 0x122f2754, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x758f2756, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f2759, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2f275c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf50f275e, SUC_lexicon_tags_32 },
    { 0xbd2f275f, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0x404f2761, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x550f2763, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6f2766, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614f276c, SUC_lexicon_tags_37 },
    { 0x3def276d, SUC_lexicon_tags_47 },
    { 0x22ef276d, SUC_lexicon_tags_9 },
    { 0xeb2f276f, SUC_lexicon_tags_37 },
    { 0xb5ef276f, SUC_lexicon_tags_12 },
    { 0xcbaf276f, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xb2cf2773, SUC_lexicon_tags_45 },
    { 0x8d0f2774, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f2777, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2f277a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4f2782, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x950f2785, SUC_lexicon_tags_25 },
    { 0x808f2786, SUC_lexicon_tags_6 },
    { 0x8daf2787, SUC_lexicon_tags_33 },
    { 0xda0f2786, SUC_lexicon_tags_25 },
    { 0xb46f2785, SUC_lexicon_tags_65 },
    { 0xc2cf2787, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aaf2790, SUC_lexicon_tags_69 },
    { 0x996f2791, SUC_lexicon_tags_15 },
    { 0x370f2792, SUC_lexicon_tags_26 },
    { 0xff8f2792, SUC_lexicon_tags_37 },
    { 0x144f2793, SUC_lexicon_tags_17 },
    { 0x30ef2795, SUC_lexicon_tags_10 },
    { 0xda2f2796, SUC_lexicon_tags_37 },
    { 0xf94f2794, SUC_lexicon_tags_10 },
    { 0xd16f2794, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40f279f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x62ef27a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27af27a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40f27a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344f27af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4f27b4, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3af27bb, SUC_lexicon_tags_66 },
    { 0x56af27bc, SUC_lexicon_tags_12 },
    { 0x4def27bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4f27c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cf27c7, SUC_lexicon_tags_15 },
    { 0x7f4f27c7, SUC_lexicon_tags_5 },
    { 0x336f27c9, SUC_lexicon_tags_11 },
    { 0xd3af27ca, SUC_lexicon_tags_16 },
    { 0x132f27c7, SUC_lexicon_tags_7 },
    { 0xe04f27c7, SUC_lexicon_tags_63 },
    { 0x1d6f27cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cf27d7, SUC_lexicon_tags_30 },
    { 0x602f27d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53af27da, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x644f27de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f4f27e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98af27e6, SUC_lexicon_tags_7 },
    { 0xd6cf27e7, SUC_lexicon_tags_12 },
    { 0x806f27e8, SUC_lexicon_tags_10 },
    { 0x10cf27e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa7ef27eb, SUC_lexicon_tags_1 },
    { 0xad4f27ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ef27f2, SUC_lexicon_tags_63 },
    { 0xbd6f27f2, SUC_lexicon_tags_29 },
    { 0xc28f27f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4f27f9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7caf27fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1b8f27fe, SUC_lexicon_tags_0 },
    { 0x56af27ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2f2802, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864f2807, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf280d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6f2812, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222f281e, SUC_lexicon_tags_1 },
    { 0x9aef281e, SUC_lexicon_tags_12 },
    { 0x200f2820, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3a6f2822, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaf2826, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960f282d, SUC_lexicon_tags_37 },
    { 0x87cf282e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38f2832, SUC_lexicon_tags_56 },
    { 0x664f2832, SUC_lexicon_tags_12 },
    { 0x376f2832, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2837, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x840f2839, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7faf283d, SUC_lexicon_tags_23 },
    { 0x64ef283e, SUC_lexicon_tags_7 },
    { 0xeef283f, SUC_lexicon_tags_15 },
    { 0xdcf283d, SUC_lexicon_tags_16 },
    { 0x786f2841, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaf2844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe74f2847, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2f284a, SUC_lexicon_tags_61 },
    { 0xdc4f284a, SUC_lexicon_tags_31 },
    { 0x37cf284c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8f2850, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4f2853, SUC_lexicon_tags_23 },
    { 0x3af2853, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x886f2856, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ef2859, SUC_lexicon_tags_1 },
    { 0x1e4f285a, SUC_lexicon_tags_25 },
    { 0x1caf285b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6af285e, SUC_lexicon_tags_31 },
    { 0x180f285f, SUC_lexicon_tags_87 },
    { 0xfc6f2860, SUC_lexicon_tags_8 },
    { 0xbbaf2861, SUC_lexicon_tags_15 },
    { 0x706f2862, SUC_lexicon_tags_1 },
    { 0xbdaf2860, SUC_lexicon_tags_45 },
    { 0x37ef2864, SUC_lexicon_tags_16 },
    { 0xd40f2865, SUC_lexicon_tags_17 },
    { 0xee4f2862, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3fef2868, SUC_lexicon_tags_15 },
    { 0x178f2868, SUC_lexicon_tags_16 },
    { 0xafcf2868, SUC_lexicon_tags_60 },
    { 0x1b0f286b, SUC_lexicon_tags_11 },
    { 0x3b0f286c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c2f286e, SUC_lexicon_tags_148 },
    { 0x712f286f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xf30f2871, SUC_lexicon_tags_25 },
    { 0x95af2872, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0f2879, SUC_lexicon_tags_7 },
    { 0xceaf287a, SUC_lexicon_tags_23 },
    { 0xd8af287b, SUC_lexicon_tags_2 },
    { 0x58ef287c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42f2883, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x74cf2885, SUC_lexicon_tags_61 },
    { 0xefef2885, SUC_lexicon_tags_30 },
    { 0x42cf2887, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf288b, SUC_lexicon_tags_1 },
    { 0xc94f288b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff2f288e, SUC_lexicon_tags_25 },
    { 0xec6f288f, SUC_lexicon_tags_19 },
    { 0x1f6f2890, SUC_lexicon_tags_1 },
    { 0xcb8f288f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ef2894, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6dcf2896, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x58cf2899, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8f28a1, SUC_lexicon_tags_25 },
    { 0x1a6f28a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508f28ad, SUC_lexicon_tags_29 },
    { 0xeef28ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8d0f28b0, SUC_lexicon_tags_127 },
    { 0xe24f28b1, SUC_lexicon_tags_26 },
    { 0x744f28b1, SUC_lexicon_tags_27 },
    { 0xf5ef28b3, SUC_lexicon_tags_27 },
    { 0xf5cf28b2, SUC_lexicon_tags_45 },
    { 0xbf8f28b5, SUC_lexicon_tags_160 },
    { 0xd7ef28b5, SUC_lexicon_tags_2 },
    { 0x22cf28b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4f28bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4f28c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbef28c4, SUC_lexicon_tags_43 },
    { 0xbdcf28c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7af28c7, SUC_lexicon_tags_1 },
    { 0xe20f28c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf66f28ca, SUC_lexicon_tags_27 },
    { 0xb1ef28cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cef28d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4f28d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x858f28d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58f28d9, SUC_lexicon_tags_37 },
    { 0xef6f28d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa2f28dc, SUC_lexicon_tags_12 },
    { 0x23af28dd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4f28e1, SUC_lexicon_tags_25 },
    { 0x12cf28e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cf28e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0f28eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff4f28ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cef28ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a0f28f1, SUC_lexicon_tags_37 },
    { 0x90f28f2, SUC_lexicon_tags_14 },
    { 0x1aef28f3, SUC_lexicon_tags_60 },
    { 0x610f28f4, SUC_lexicon_tags_37 },
    { 0x20af28f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2ef28f7, SUC_lexicon_tags_37 },
    { 0x1c0f28f8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccf28fd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612f2901, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0f2905, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x39af2907, SUC_lexicon_tags_15 },
    { 0x916f2908, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6f290c, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46af2912, SUC_lexicon_tags_12 },
    { 0xd80f2912, SUC_lexicon_tags_11 },
    { 0x39cf2912, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74f2917, SUC_lexicon_tags_22 },
    { 0xedcf2918, SUC_lexicon_tags_131 },
    { 0xb80f2919, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ef291c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cf292c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x102f292e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x34cf2931, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586f293a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x6e6f293c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49ef293e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x48af2940, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa6f2942, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516f2947, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f294b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac0f294d, SUC_lexicon_tags_37 },
    { 0x72ef294d, SUC_lexicon_tags_15 },
    { 0xfd6f294f, SUC_lexicon_tags_7 },
    { 0x76af2950, SUC_lexicon_tags_33 },
    { 0x15af2951, SUC_lexicon_tags_26 },
    { 0xa16f2952, SUC_lexicon_tags_60 },
    { 0x978f2953, SUC_lexicon_tags_37 },
    { 0xce0f2953, SUC_lexicon_tags_43 },
    { 0x1c0f2954, SUC_lexicon_tags_23 },
    { 0x100f2956, SUC_lexicon_tags_148 },
    { 0xa3ef2952, SUC_lexicon_tags_9 },
    { 0xa98f2952, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6f295e, SUC_lexicon_tags_63 },
    { 0x6b8f295f, SUC_lexicon_tags_13 },
    { 0xfb6f295f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214f2964, SUC_lexicon_tags_37 },
    { 0x42ef2965, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0f296b, SUC_lexicon_tags_15 },
    { 0xc04f296b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74af296e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6f2974, SUC_lexicon_tags_37 },
    { 0x92af2975, SUC_lexicon_tags_15 },
    { 0x818f2976, SUC_lexicon_tags_9 },
    { 0xac6f2977, SUC_lexicon_tags_11 },
    { 0xa20f2978, SUC_lexicon_tags_60 },
    { 0xcf8f2978, SUC_lexicon_tags_37 },
    { 0x6def2978, SUC_lexicon_tags_1 },
    { 0x5eef2979, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f2980, SUC_lexicon_tags_6 },
    { 0x972f2981, SUC_lexicon_tags_12 },
    { 0x106f2982, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70f2986, SUC_lexicon_tags_21 },
    { 0xebcf2987, SUC_lexicon_tags_17 },
    { 0x386f2986, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ef2991, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2af2994, SUC_lexicon_tags_11 },
    { 0xbc6f2994, SUC_lexicon_tags_4 },
    { 0x872f2994, SUC_lexicon_tags_12 },
    { 0x63cf2997, SUC_lexicon_tags_1 },
    { 0x120f2997, SUC_lexicon_tags_7 },
    { 0x412f2995, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafcf299b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ef299e, SUC_lexicon_tags_63 },
    { 0xc84f299f, SUC_lexicon_tags_11 },
    { 0xfa8f29a0, SUC_lexicon_tags_15 },
    { 0xe22f29a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x318f29a3, SUC_lexicon_tags_26 },
    { 0x7d8f29a4, SUC_lexicon_tags_28 },
    { 0xc78f29a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180f29ad, SUC_lexicon_tags_2 },
    { 0xc0af29ae, SUC_lexicon_tags_47 },
    { 0xd8f29ad, SUC_lexicon_tags_17 },
    { 0x94cf29b0, SUC_lexicon_tags_68 },
    { 0x43ef29b1, SUC_lexicon_tags_1 },
    { 0xd2af29b0, SUC_lexicon_tags_23 },
    { 0xa6cf29b3, SUC_lexicon_tags_2 },
    { 0x316f29b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef29bc, SUC_lexicon_tags_28 },
    { 0xfd8f29bd, SUC_lexicon_tags_25 },
    { 0xcf0f29bd, SUC_lexicon_tags_55 },
    { 0x8af29bf, SUC_lexicon_tags_30 },
    { 0x860f29bc, SUC_lexicon_tags_26 },
    { 0x292f29bc, SUC_lexicon_tags_28 },
    { 0x3f6f29c2, SUC_lexicon_tags_11 },
    { 0x950f29c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecef29c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde8f29ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270f29ce, SUC_lexicon_tags_12 },
    { 0x76cf29cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcf29d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2baf29d5, SUC_lexicon_tags_2 },
    { 0x186f29d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976f29da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fef29dd, SUC_lexicon_tags_8 },
    { 0x9ef29dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f29e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad2f29e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccf29e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74cf29ea, SUC_lexicon_tags_19 },
    { 0x7f0f29eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a6f29f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcf29f7, SUC_lexicon_tags_1 },
    { 0x76ef29f8, SUC_lexicon_tags_40 },
    { 0x9baf29f8, SUC_lexicon_tags_25 },
    { 0xa1ef29fa, SUC_lexicon_tags_30 },
    { 0x4f0f29fb, SUC_lexicon_tags_6 },
    { 0x4a4f29fb, SUC_lexicon_tags_17 },
    { 0x68cf29fd, SUC_lexicon_tags_31 },
    { 0xd22f29fe, SUC_lexicon_tags_54 },
    { 0x9c6f29fd, SUC_lexicon_tags_7 },
    { 0x9c0f2a00, SUC_lexicon_tags_45 },
    { 0xb0cf2a00, SUC_lexicon_tags_6 },
    { 0x59ef2a02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cef2a05, SUC_lexicon_tags_21 },
    { 0x80cf2a06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cf2a0a, SUC_lexicon_tags_1 },
    { 0x63ef2a0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8f2a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x468f2a17, SUC_lexicon_tags_37 },
    { 0x5e6f2a18, SUC_lexicon_tags_30 },
    { 0x93cf2a17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebcf2a1b, SUC_lexicon_tags_37 },
    { 0xf54f2a1c, SUC_lexicon_tags_63 },
    { 0x32f2a1d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x97cf2a1f, SUC_lexicon_tags_15 },
    { 0xa26f2a1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x342f2a23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x416f2a26, SUC_lexicon_tags_7 },
    { 0x88af2a27, SUC_lexicon_tags_32 },
    { 0xbfef2a28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x666f2a2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6f2a2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26f2a32, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xa3af2a34, SUC_lexicon_tags_0 },
    { 0x5bcf2a35, SUC_lexicon_tags_63 },
    { 0xce2f2a35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf2a39, SUC_lexicon_tags_7 },
    { 0x47ef2a39, SUC_lexicon_tags_25 },
    { 0x97af2a3b, SUC_lexicon_tags_104 },
    { 0xdd0f2a3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850f2a49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cf2a51, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaf2a54, SUC_lexicon_tags_46 },
    { 0xfe8f2a54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868f2a5a, SUC_lexicon_tags_39 },
    { 0x4b6f2a5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f2a65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeaaf2a67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548f2a6d, SUC_lexicon_tags_15 },
    { 0xe12f2a6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc82f2a70, SUC_lexicon_tags_45 },
    { 0x5aef2a71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cf2a75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2f2a7a, SUC_lexicon_tags_49 },
    { 0x34ef2a7a, SUC_lexicon_tags_55 },
    { 0xcaef2a7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6f2a80, SUC_lexicon_tags_37 },
    { 0x998f2a80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43af2a84, SUC_lexicon_tags_0 },
    { 0x72ef2a84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0f2a8e, SUC_lexicon_tags_23 },
    { 0x53ef2a8f, SUC_lexicon_tags_43 },
    { 0x54ef2a8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4f2a95, SUC_lexicon_tags_25 },
    { 0x3d8f2a96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcb2f2a98, SUC_lexicon_tags_28 },
    { 0x900f2a99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0f2aa2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x988f2aa4, SUC_lexicon_tags_37 },
    { 0xa52f2aa5, SUC_lexicon_tags_9 },
    { 0x456f2aa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76f2aab, SUC_lexicon_tags_43 },
    { 0x4f2f2aab, SUC_lexicon_tags_1 },
    { 0x7a4f2aad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce4f2ab0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f2ab3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ef2ab9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ef2abc, SUC_lexicon_tags_45 },
    { 0xf5cf2abd, SUC_lexicon_tags_15 },
    { 0x7c8f2abe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91af2ac6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf2acc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484f2ad2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x414f2ad5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18f2adc, SUC_lexicon_tags_82 },
    { 0x31cf2adc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480f2ae3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5a2f2ae5, SUC_lexicon_tags_2 },
    { 0x4caf2ae6, SUC_lexicon_tags_39 },
    { 0xa64f2ae6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f2aed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x48af2aef, SUC_lexicon_tags_13 },
    { 0x5b6f2af0, SUC_lexicon_tags_25 },
    { 0xb04f2af0, SUC_lexicon_tags_5 },
    { 0x918f2af0, SUC_lexicon_tags_47 },
    { 0xa48f2af3, SUC_lexicon_tags_39 },
    { 0x514f2af4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364f2afd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1caf2aff, SUC_lexicon_tags_12 },
    { 0xbaef2b00, SUC_lexicon_tags_17 },
    { 0x468f2b01, SUC_lexicon_tags_29 },
    { 0x6f2f2b00, SUC_lexicon_tags_26 },
    { 0x73af2b00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x878f2b05, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xe2ef2b07, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf92f2b09, SUC_lexicon_tags_32 },
    { 0x3bef2b09, SUC_lexicon_tags_15 },
    { 0x3d8f2b0b, SUC_lexicon_tags_65 },
    { 0xaef2b0a, SUC_lexicon_tags_45 },
    { 0x524f2b0d, SUC_lexicon_tags_31 },
    { 0xd00f2b0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbef2b18, SUC_lexicon_tags_17 },
    { 0x516f2b19, SUC_lexicon_tags_6 },
    { 0xeef2b1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ef2b1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x758f2b21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60cf2b23, SUC_lexicon_tags_27 },
    { 0x444f2b23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef2b2a, SUC_lexicon_tags_2 },
    { 0x2e0f2b2b, SUC_lexicon_tags_0 },
    { 0x5def2b2a, SUC_lexicon_tags_15 },
    { 0x2c6f2b2b, SUC_lexicon_tags_7 },
    { 0x67ef2b2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bef2b31, SUC_lexicon_tags_2 },
    { 0x786f2b32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x664f2b35, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x46cf2b38, SUC_lexicon_tags_65 },
    { 0xa3cf2b38, SUC_lexicon_tags_37 },
    { 0x6b6f2b3a, SUC_lexicon_tags_15 },
    { 0xb8ef2b39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62af2b3f, SUC_lexicon_tags_37 },
    { 0x75af2b3f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaf2b47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaf2b4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x988f2b4d, SUC_lexicon_tags_45 },
    { 0x16af2b4e, SUC_lexicon_tags_77 },
    { 0xccaf2b4f, SUC_lexicon_tags_37 },
    { 0xea0f2b50, SUC_lexicon_tags_108 },
    { 0x2b0f2b50, SUC_lexicon_tags_16 },
    { 0x4d4f2b52, SUC_lexicon_tags_31 },
    { 0x304f2b53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194f2b57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44f2b5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb6f2b5d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe52f2b5f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x35ef2b61, SUC_lexicon_tags_37 },
    { 0x4fef2b61, SUC_lexicon_tags_7 },
    { 0x396f2b63, SUC_lexicon_tags_9 },
    { 0x3d6f2b63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x30cf2b67, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ef2b6c, SUC_lexicon_tags_7 },
    { 0x686f2b6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698f2b74, SUC_lexicon_tags_17 },
    { 0x62cf2b74, SUC_lexicon_tags_47 },
    { 0xc36f2b76, SUC_lexicon_tags_1 },
    { 0xb2af2b77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aef2b7c, SUC_lexicon_tags_34 },
    { 0xffef2b7d, SUC_lexicon_tags_17 },
    { 0x744f2b7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b83, SUC_lexicon_tags_12 },
    { 0xb72f2b83, SUC_lexicon_tags_63 },
    { 0x236f2b85, SUC_lexicon_tags_23 },
    { 0xc6af2b86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde8f2b89, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480f2b8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x208f2b91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xccaf2b94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08f2b9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf74f2b9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa9af2b9f, SUC_lexicon_tags_14 },
    { 0xedef2b9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeef2ba4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ef2ba8, SUC_lexicon_tags_43 },
    { 0x1bcf2ba9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x142f2bac, SUC_lexicon_tags_8 },
    { 0x894f2bad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f2bb0, SUC_lexicon_tags_17 },
    { 0xcf2f2bb1, SUC_lexicon_tags_53 },
    { 0xf16f2bb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea6f2bb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7af2bb7, SUC_lexicon_tags_15 },
    { 0x1e2f2bb8, SUC_lexicon_tags_12 },
    { 0xfa8f2bb9, SUC_lexicon_tags_60 },
    { 0x472f2bba, SUC_lexicon_tags_46 },
    { 0x684f2bb8, SUC_lexicon_tags_25 },
    { 0x906f2bb9, SUC_lexicon_tags_19 },
    { 0xc0af2bb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6f2bc0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9fcf2bc2, SUC_lexicon_tags_11 },
    { 0xe34f2bc3, SUC_lexicon_tags_43 },
    { 0xbdaf2bc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef2bc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b6f2bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaf2bd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x66af2bd4, SUC_lexicon_tags_25 },
    { 0xbf0f2bd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3af2bda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28cf2bdc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52f2bdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e8f2be1, SUC_lexicon_tags_45 },
    { 0xce2f2be2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ef2be8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xccaf2beb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30f2bef, SUC_lexicon_tags_67 },
    { 0xf82f2bf0, SUC_lexicon_tags_11 },
    { 0xd78f2bf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8aef2bf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70f2bf7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2f2bfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe14f2bfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ef2c03, SUC_lexicon_tags_39 },
    { 0x5b6f2c03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x282f2c07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f8f2c09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124f2c0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ef2c11, SUC_lexicon_tags_37 },
    { 0x4af2c12, SUC_lexicon_tags_60 },
    { 0x9eaf2c13, SUC_lexicon_tags_12 },
    { 0xeb4f2c14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x706f2c17, SUC_lexicon_tags_12 },
    { 0xd9cf2c18, SUC_lexicon_tags_31 },
    { 0x9bcf2c19, SUC_lexicon_tags_15 },
    { 0xcacf2c18, SUC_lexicon_tags_15 },
    { 0xbfcf2c18, SUC_lexicon_tags_2 },
    { 0xa6cf2c18, SUC_lexicon_tags_10 },
    { 0xb26f2c1b, SUC_lexicon_tags_45 },
    { 0x15ef2c1e, SUC_lexicon_tags_65 },
    { 0xaef2c1e, SUC_lexicon_tags_14 },
    { 0x660f2c1f, SUC_lexicon_tags_11 },
    { 0x5b2f2c21, SUC_lexicon_tags_68 },
    { 0x250f2c22, SUC_lexicon_tags_7 },
    { 0xa12f2c23, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cf2c27, SUC_lexicon_tags_25 },
    { 0x2b6f2c27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x226f2c2b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5faf2c2d, SUC_lexicon_tags_12 },
    { 0x86af2c2e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe0cf2c30, SUC_lexicon_tags_99 },
    { 0xa64f2c30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06f2c35, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x56af2c37, SUC_lexicon_tags_15 },
    { 0x33af2c37, SUC_lexicon_tags_12 },
    { 0x4b6f2c39, SUC_lexicon_tags_25 },
    { 0x10cf2c39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148f2c3f, SUC_lexicon_tags_12 },
    { 0xe02f2c40, SUC_lexicon_tags_7 },
    { 0x452f2c40, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe04f2c4e, SUC_lexicon_tags_7 },
    { 0xa64f2c4e, SUC_lexicon_tags_17 },
    { 0x124f2c50, SUC_lexicon_tags_19 },
    { 0xa40f2c51, SUC_lexicon_tags_17 },
    { 0x6cf2c52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x830f2c54, SUC_lexicon_tags_15 },
    { 0xe48f2c54, SUC_lexicon_tags_2 },
    { 0xa3cf2c55, SUC_lexicon_tags_17 },
    { 0xa7ef2c57, SUC_lexicon_tags_63 },
    { 0xcc0f2c58, SUC_lexicon_tags_6 },
    { 0x83af2c59, SUC_lexicon_tags_43 },
    { 0x82cf2c5a, SUC_lexicon_tags_27 },
    { 0xb70f2c5a, SUC_lexicon_tags_15 },
    { 0x322f2c5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6f2c64, SUC_lexicon_tags_6 },
    { 0x2aef2c65, SUC_lexicon_tags_2 },
    { 0x888f2c66, SUC_lexicon_tags_66 },
    { 0x33af2c67, SUC_lexicon_tags_11 },
    { 0x220f2c65, SUC_lexicon_tags_25 },
    { 0x938f2c69, SUC_lexicon_tags_1 },
    { 0x4b2f2c69, SUC_lexicon_tags_63 },
    { 0x576f2c6a, SUC_lexicon_tags_10 },
    { 0x1e8f2c6c, SUC_lexicon_tags_158 },
    { 0xff2f2c67, SUC_lexicon_tags_15 },
    { 0x856f2c64, SUC_lexicon_tags_29 },
    { 0xf62f2c6f, SUC_lexicon_tags_17 },
    { 0x3b6f2c69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f2c76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692f2c7e, SUC_lexicon_tags_25 },
    { 0x12ef2c7e, SUC_lexicon_tags_65 },
    { 0x57ef2c80, SUC_lexicon_tags_25 },
    { 0x7e2f2c7f, SUC_lexicon_tags_15 },
    { 0x178f2c82, SUC_lexicon_tags_31 },
    { 0x20ef2c83, SUC_lexicon_tags_34 },
    { 0x93af2c84, SUC_lexicon_tags_17 },
    { 0xc4f2c82, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696f2c89, SUC_lexicon_tags_0 },
    { 0x29cf2c8a, SUC_lexicon_tags_2 },
    { 0xe2cf2c89, SUC_lexicon_tags_33 },
    { 0xa60f2c8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f2c98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x384f2c9a, SUC_lexicon_tags_1 },
    { 0xe3ef2c9a, SUC_lexicon_tags_23 },
    { 0x52af2c9c, SUC_lexicon_tags_2 },
    { 0x1b6f2c9b, SUC_lexicon_tags_1 },
    { 0x926f2c9e, SUC_lexicon_tags_11 },
    { 0xcd6f2c9f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5f0f2ca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6f2ca7, SUC_lexicon_tags_37 },
    { 0x5daf2ca7, SUC_lexicon_tags_37 },
    { 0xfb0f2ca8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa34f2cab, SUC_lexicon_tags_6 },
    { 0x8cf2cac, SUC_lexicon_tags_11 },
    { 0xd74f2cab, SUC_lexicon_tags_19 },
    { 0xf86f2cae, SUC_lexicon_tags_26 },
    { 0xac4f2cab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x594f2cb2, SUC_lexicon_tags_47 },
    { 0x2d6f2cb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74f2cb7, SUC_lexicon_tags_47 },
    { 0x94ef2cb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94f2cba, SUC_lexicon_tags_27 },
    { 0x9cef2cbb, SUC_lexicon_tags_23 },
    { 0x932f2cbc, SUC_lexicon_tags_0 },
    { 0x9c4f2cba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46cf2ccb, SUC_lexicon_tags_11 },
    { 0x88af2ccc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x59af2cce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af2cd1, SUC_lexicon_tags_7 },
    { 0x600f2cd1, SUC_lexicon_tags_85 },
    { 0xa6cf2cd3, SUC_lexicon_tags_33 },
    { 0xfbcf2cd4, SUC_lexicon_tags_21 },
    { 0xadef2cd5, SUC_lexicon_tags_11 },
    { 0xf0af2cd6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6f2cd9, SUC_lexicon_tags_15 },
    { 0xfb8f2cda, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa28f2cdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0f2ce1, SUC_lexicon_tags_19 },
    { 0xe84f2ce2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xba2f2ce4, SUC_lexicon_tags_28 },
    { 0x826f2ce4, SUC_lexicon_tags_60 },
    { 0x96cf2ce6, SUC_lexicon_tags_32 },
    { 0x24af2ce7, SUC_lexicon_tags_47 },
    { 0xbbcf2ce8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbef2cea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaf2ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x466f2cf0, SUC_lexicon_tags_26 },
    { 0x30f2cf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4caf2cf5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1af2cf8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0f2cff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55af2d05, SUC_lexicon_tags_1 },
    { 0x42af2d06, SUC_lexicon_tags_1 },
    { 0xd48f2d05, SUC_lexicon_tags_12 },
    { 0xe68f2d05, SUC_lexicon_tags_25 },
    { 0xf6ef2d08, SUC_lexicon_tags_25 },
    { 0x534f2d07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x364f2d0e, SUC_lexicon_tags_8 },
    { 0x7acf2d0e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb24f2d11, SUC_lexicon_tags_25 },
    { 0x758f2d11, SUC_lexicon_tags_22 },
    { 0xb4ef2d13, SUC_lexicon_tags_34 },
    { 0xa3ef2d13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x888f2d17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44f2d19, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6f2d1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4cf2d21, SUC_lexicon_tags_8 },
    { 0xcb0f2d22, SUC_lexicon_tags_12 },
    { 0x166f2d21, SUC_lexicon_tags_15 },
    { 0x852f2d24, SUC_lexicon_tags_30 },
    { 0xbbcf2d24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22f2d2b, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x908f2d2d, SUC_lexicon_tags_47 },
    { 0xbc8f2d2d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8f2d32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764f2d3d, SUC_lexicon_tags_8 },
    { 0x350f2d3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccf2d42, SUC_lexicon_tags_2 },
    { 0x3c8f2d42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216f2d4a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe4af2d4c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2f2d50, SUC_lexicon_tags_4 },
    { 0xf54f2d51, SUC_lexicon_tags_23 },
    { 0x190f2d52, SUC_lexicon_tags_7 },
    { 0x6a0f2d53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x424f2d56, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ef2d59, SUC_lexicon_tags_48 },
    { 0x300f2d5a, SUC_lexicon_tags_11 },
    { 0xfef2d5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x966f2d5f, SUC_lexicon_tags_43 },
    { 0xe96f2d60, SUC_lexicon_tags_14 },
    { 0x2ef2d5f, SUC_lexicon_tags_15 },
    { 0x4c4f2d62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72f2d65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8af2d68, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ef2d6c, SUC_lexicon_tags_0 },
    { 0x7e6f2d6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb54f2d6f, SUC_lexicon_tags_45 },
    { 0x50af2d6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8af2d73, SUC_lexicon_tags_48 },
    { 0x46f2d74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8d2f2d76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b0f2d78, SUC_lexicon_tags_26 },
    { 0x6ecf2d78, SUC_lexicon_tags_10 },
    { 0x36f2d7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1aaf2d7c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76af2d81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x452f2d83, SUC_lexicon_tags_7 },
    { 0xf84f2d84, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76f2d88, SUC_lexicon_tags_11 },
    { 0xee2f2d88, SUC_lexicon_tags_7 },
    { 0x326f2d8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450f2d8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5e6f2d90, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8def2d92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2baf2d94, SUC_lexicon_tags_11 },
    { 0xe44f2d94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe26f2d97, SUC_lexicon_tags_1 },
    { 0x99ef2d98, SUC_lexicon_tags_49 },
    { 0x3f2f2d99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700f2da1, SUC_lexicon_tags_30 },
    { 0x212f2da2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57cf2da8, SUC_lexicon_tags_15 },
    { 0xbaf2da8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x11cf2dab, SUC_lexicon_tags_25 },
    { 0x16ef2dac, SUC_lexicon_tags_5 },
    { 0x9c2f2dad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f2db0, SUC_lexicon_tags_27 },
    { 0x39ef2db1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480f2db9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2f2dcb, SUC_lexicon_tags_38 },
    { 0x944f2dcc, SUC_lexicon_tags_12 },
    { 0x860f2dcd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ef2dd1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6f2dd8, SUC_lexicon_tags_14 },
    { 0x2f2f2dd9, SUC_lexicon_tags_0 },
    { 0x4c4f2dd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2f2de1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cf2de4, SUC_lexicon_tags_26 },
    { 0x996f2de4, SUC_lexicon_tags_30 },
    { 0xafaf2de5, SUC_lexicon_tags_11 },
    { 0x4d2f2de5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xffef2dea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118f2dee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94f2df3, SUC_lexicon_tags_17 },
    { 0x74cf2df3, SUC_lexicon_tags_37 },
    { 0x16af2df4, SUC_lexicon_tags_60 },
    { 0xa52f2df4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240f2dfa, SUC_lexicon_tags_7 },
    { 0x212f2dfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x574f2dfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x962f2dff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf8af2e01, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6baf2e04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacf2e0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ef2e0f, SUC_lexicon_tags_61 },
    { 0x318f2e10, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x458f2e12, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80f2e15, SUC_lexicon_tags_6 },
    { 0x1a4f2e15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd58f2e18, SUC_lexicon_tags_47 },
    { 0x402f2e18, SUC_lexicon_tags_17 },
    { 0x734f2e18, SUC_lexicon_tags_15 },
    { 0x172f2e1a, SUC_lexicon_tags_11 },
    { 0xd66f2e1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd60f2e1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x602f2e20, SUC_lexicon_tags_45 },
    { 0x316f2e20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa92f2e23, SUC_lexicon_tags_47 },
    { 0xb32f2e23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d2f2e26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26f2e2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5af2e2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2f2e30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06f2e34, SUC_lexicon_tags_37 },
    { 0x7ef2e35, SUC_lexicon_tags_37 },
    { 0x5f6f2e35, SUC_lexicon_tags_25 },
    { 0x33cf2e37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484f2e3b, SUC_lexicon_tags_31 },
    { 0x1e2f2e3c, SUC_lexicon_tags_11 },
    { 0x40cf2e3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14f2e40, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498f2e47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94af2e4b, SUC_lexicon_tags_11 },
    { 0x8f2e4b, SUC_lexicon_tags_37 },
    { 0x444f2e4b, SUC_lexicon_tags_14 },
    { 0x808f2e4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7e8f2e50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16f2e55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eef2e5a, SUC_lexicon_tags_65 },
    { 0x634f2e5b, SUC_lexicon_tags_0 },
    { 0x54cf2e5c, SUC_lexicon_tags_25 },
    { 0xbc6f2e5d, SUC_lexicon_tags_9 },
    { 0xcd6f2e5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aef2e61, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cf2e64, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ef2e6a, SUC_lexicon_tags_23 },
    { 0xa50f2e6b, SUC_lexicon_tags_1 },
    { 0x33af2e6c, SUC_lexicon_tags_84 },
    { 0x6c8f2e6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52cf2e6f, SUC_lexicon_tags_19 },
    { 0xfccf2e70, SUC_lexicon_tags_17 },
    { 0x2f6f2e6f, SUC_lexicon_tags_17 },
    { 0x18f2e6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x900f2e75, SUC_lexicon_tags_12 },
    { 0xbcaf2e76, SUC_lexicon_tags_34 },
    { 0x238f2e76, SUC_lexicon_tags_12 },
    { 0x692f2e77, SUC_lexicon_tags_61 },
    { 0x5caf2e75, SUC_lexicon_tags_0 },
    { 0xbdaf2e7a, SUC_lexicon_tags_60 },
    { 0x35cf2e7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd44f2e7e, SUC_lexicon_tags_8 },
    { 0x3e4f2e7f, SUC_lexicon_tags_23 },
    { 0xc6cf2e7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ef2e83, SUC_lexicon_tags_38 },
    { 0xc62f2e83, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4f2e87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcf2e8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x358f2e8e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672f2e95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618f2e99, SUC_lexicon_tags_19 },
    { 0x426f2e99, SUC_lexicon_tags_2 },
    { 0x5aaf2e99, SUC_lexicon_tags_25 },
    { 0x828f2e9a, SUC_lexicon_tags_27 },
    { 0xb70f2e9a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaf2ea2, SUC_lexicon_tags_2 },
    { 0xe10f2ea3, SUC_lexicon_tags_2 },
    { 0xb06f2ea4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15af2ea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd48f2eaa, SUC_lexicon_tags_20 },
    { 0x538f2eab, SUC_lexicon_tags_9 },
    { 0x6f2eab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab0f2eae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92f2eb1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf2eb6, SUC_lexicon_tags_7 },
    { 0x354f2eb6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aef2ebc, SUC_lexicon_tags_33 },
    { 0x424f2ebd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x998f2ec0, SUC_lexicon_tags_27 },
    { 0xc1ef2ec1, SUC_lexicon_tags_2 },
    { 0x566f2ec2, SUC_lexicon_tags_43 },
    { 0x7f6f2ec3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2ec8, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cf2ece, SUC_lexicon_tags_15 },
    { 0xf72f2ecf, SUC_lexicon_tags_27 },
    { 0x8eaf2ed0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556f2ed4, SUC_lexicon_tags_42 },
    { 0xcfef2ed5, SUC_lexicon_tags_11 },
    { 0x36af2ed6, SUC_lexicon_tags_140 },
    { 0x616f2ed5, SUC_lexicon_tags_21 },
    { 0x3cef2ed7, SUC_lexicon_tags_15 },
    { 0x466f2ed6, SUC_lexicon_tags_25 },
    { 0xf0f2eda, SUC_lexicon_tags_29 },
    { 0x5a0f2ed6, SUC_lexicon_tags_56 },
    { 0xf7cf2edc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0f2edf, SUC_lexicon_tags_37 },
    { 0x79ef2ee0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae2f2ee3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc38f2ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f2ee8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6f2eec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cf2eef, SUC_lexicon_tags_68 },
    { 0x40af2ef0, SUC_lexicon_tags_17 },
    { 0x480f2ef1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2f2ef4, SUC_lexicon_tags_6 },
    { 0x660f2ef5, SUC_lexicon_tags_14 },
    { 0x612f2ef6, SUC_lexicon_tags_25 },
    { 0x38ef2ef7, SUC_lexicon_tags_17 },
    { 0xea6f2ef8, SUC_lexicon_tags_17 },
    { 0x658f2ef5, SUC_lexicon_tags_25 },
    { 0x900f2efa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95af2efe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ef2f01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e6f2f03, SUC_lexicon_tags_63 },
    { 0x2c6f2f03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x59cf2f06, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x522f2f08, SUC_lexicon_tags_66 },
    { 0x8f4f2f09, SUC_lexicon_tags_0 },
    { 0x8eef2f0a, SUC_lexicon_tags_17 },
    { 0xabef2f0b, SUC_lexicon_tags_48 },
    { 0x2b2f2f0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91af2f13, SUC_lexicon_tags_45 },
    { 0x1af2f13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876f2f18, SUC_lexicon_tags_59 },
    { 0x5baf2f19, SUC_lexicon_tags_23 },
    { 0xcc0f2f19, SUC_lexicon_tags_23 },
    { 0x356f2f19, SUC_lexicon_tags_15 },
    { 0xb4af2f1c, SUC_lexicon_tags_29 },
    { 0xcf8f2f1d, SUC_lexicon_tags_25 },
    { 0x90af2f1e, SUC_lexicon_tags_25 },
    { 0x21ef2f1e, SUC_lexicon_tags_59 },
    { 0x18f2f1d, SUC_lexicon_tags_3 },
    { 0xa4cf2f21, SUC_lexicon_tags_17 },
    { 0xdfcf2f1e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0af2f2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2f2f2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x956f2f31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x886f2f34, SUC_lexicon_tags_6 },
    { 0xe70f2f35, SUC_lexicon_tags_62 },
    { 0x6cf2f36, SUC_lexicon_tags_7 },
    { 0x178f2f37, SUC_lexicon_tags_37 },
    { 0xc62f2f38, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x336f2f3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2f2f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e2f2f41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd82f2f43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23ef2f45, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2af2f4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x436f2f4d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbdaf2f4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28f2f53, SUC_lexicon_tags_17 },
    { 0x502f2f53, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5a0f2f56, SUC_lexicon_tags_1 },
    { 0x46ef2f57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ecf2f59, SUC_lexicon_tags_0 },
    { 0xaa4f2f59, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcf2f62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ef2f64, SUC_lexicon_tags_33 },
    { 0x7aaf2f65, SUC_lexicon_tags_0 },
    { 0xb34f2f66, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x87cf2f69, SUC_lexicon_tags_12 },
    { 0x7b0f2f69, SUC_lexicon_tags_12 },
    { 0xeb0f2f6b, SUC_lexicon_tags_25 },
    { 0xaaf2f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39cf2f6e, SUC_lexicon_tags_37 },
    { 0xb7af2f6e, SUC_lexicon_tags_44 },
    { 0xf10f2f70, SUC_lexicon_tags_30 },
    { 0xbf8f2f70, SUC_lexicon_tags_60 },
    { 0xb7af2f6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e2f2f74, SUC_lexicon_tags_12 },
    { 0xaef2f75, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf46f2f77, SUC_lexicon_tags_35 },
    { 0x35ef2f77, SUC_lexicon_tags_4 },
    { 0x302f2f79, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x6def2f7c, SUC_lexicon_tags_2 },
    { 0x4eaf2f7d, SUC_lexicon_tags_47 },
    { 0x832f2f7d, SUC_lexicon_tags_33 },
    { 0x9d4f2f7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8f2f83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x184f2f85, SUC_lexicon_tags_4 },
    { 0xe54f2f86, SUC_lexicon_tags_26 },
    { 0x59ef2f87, SUC_lexicon_tags_7 },
    { 0xd08f2f88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0f2f91, SUC_lexicon_tags_19 },
    { 0xe4af2f92, SUC_lexicon_tags_198 },
    { 0x80ef2f92, SUC_lexicon_tags_25 },
    { 0x98f2f94, SUC_lexicon_tags_25 },
    { 0x318f2f92, SUC_lexicon_tags_1 },
    { 0xdfcf2f96, SUC_lexicon_tags_45 },
    { 0xdd6f2f95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548f2f9c, SUC_lexicon_tags_18 },
    { 0x5a4f2f9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cf2fa1, SUC_lexicon_tags_25 },
    { 0x8c0f2fa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fef2fa4, SUC_lexicon_tags_6 },
    { 0x4c6f2fa5, SUC_lexicon_tags_229 },
    { 0x614f2fa6, SUC_lexicon_tags_44 },
    { 0x682f2fa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e0f2fa9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ef2fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc20f2fae, SUC_lexicon_tags_26 },
    { 0x71cf2faf, SUC_lexicon_tags_25 },
    { 0xb9af2fb0, SUC_lexicon_tags_35 },
    { 0x600f2faf, SUC_lexicon_tags_17 },
    { 0xb4f2fb2, SUC_lexicon_tags_37 },
    { 0x8cf2fb2, SUC_lexicon_tags_61 },
    { 0xe30f2fb4, SUC_lexicon_tags_116 },
    { 0xea8f2fb5, SUC_lexicon_tags_1 },
    { 0xfbef2fb6, SUC_lexicon_tags_55 },
    { 0x646f2fb6, SUC_lexicon_tags_60 },
    { 0xe6cf2fb8, SUC_lexicon_tags_14 },
    { 0x146f2fb9, SUC_lexicon_tags_7 },
    { 0xbf8f2fb6, SUC_lexicon_tags_19 },
    { 0x2a2f2fb5, SUC_lexicon_tags_1 },
    { 0x91af2fae, SUC_lexicon_tags_37 },
    { 0x5daf2fbd, SUC_lexicon_tags_37 },
    { 0x71af2fb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd72f2fc0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x36f2fc2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ef2fd9, SUC_lexicon_tags_85 },
    { 0x19cf2fda, SUC_lexicon_tags_17 },
    { 0xe2f2fda, SUC_lexicon_tags_54 },
    { 0xcf6f2fdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffef2fe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac0f2fe3, SUC_lexicon_tags_13 },
    { 0xcbef2fe4, SUC_lexicon_tags_33 },
    { 0xf9ef2fe5, SUC_lexicon_tags_11 },
    { 0x4cef2fe3, SUC_lexicon_tags_37 },
    { 0x90af2fe7, SUC_lexicon_tags_2 },
    { 0x3c2f2fe4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5af2fee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc66f2ff2, SUC_lexicon_tags_38 },
    { 0x61cf2ff2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00f2ff8, SUC_lexicon_tags_2 },
    { 0xb16f2ff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1eef2ffb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa38f2ffe, SUC_lexicon_tags_49 },
    { 0x6caf2ffe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86ef3001, SUC_lexicon_tags_8 },
    { 0x594f3002, SUC_lexicon_tags_28 },
    { 0xa7ef3002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaf3006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd34f3008, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504f300e, SUC_lexicon_tags_49 },
    { 0x9a8f300f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x588f3012, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cf3015, SUC_lexicon_tags_27 },
    { 0x622f3015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x744f3018, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecf301b, SUC_lexicon_tags_15 },
    { 0x132f301c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x7d2f301e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6f3021, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bef3024, SUC_lexicon_tags_25 },
    { 0xd24f3024, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88f3028, SUC_lexicon_tags_29 },
    { 0x5e2f3028, SUC_lexicon_tags_25 },
    { 0xe48f302a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x700f302c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x522f302e, SUC_lexicon_tags_37 },
    { 0x510f302e, SUC_lexicon_tags_82 },
    { 0xae8f3030, SUC_lexicon_tags_37 },
    { 0x6aaf3030, SUC_lexicon_tags_224 },
    { 0x9af3032, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcf3036, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62f3039, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6f303c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4f303f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf6ef3041, SUC_lexicon_tags_35 },
    { 0xf5ef3042, SUC_lexicon_tags_60 },
    { 0x440f3041, SUC_lexicon_tags_2 },
    { 0xe88f3043, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e8f3046, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaf304b, SUC_lexicon_tags_87 },
    { 0x6c8f304c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x856f304f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe3ef3051, SUC_lexicon_tags_15 },
    { 0x3c8f3051, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2f305d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624f3063, SUC_lexicon_tags_16 },
    { 0x1c8f3064, SUC_lexicon_tags_26 },
    { 0xd86f3063, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef3068, SUC_lexicon_tags_151 },
    { 0xb42f3069, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccf306c, SUC_lexicon_tags_1 },
    { 0xde0f306d, SUC_lexicon_tags_30 },
    { 0x87cf306e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaef3071, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a0f3073, SUC_lexicon_tags_37 },
    { 0x96af3073, SUC_lexicon_tags_37 },
    { 0xc0cf3075, SUC_lexicon_tags_12 },
    { 0x79af3076, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cf307d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42cf3081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ecf3083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6f308a, SUC_lexicon_tags_2 },
    { 0x118f308b, SUC_lexicon_tags_37 },
    { 0xa9cf308c, SUC_lexicon_tags_17 },
    { 0x1ef308a, SUC_lexicon_tags_56 },
    { 0x958f308c, SUC_lexicon_tags_47 },
    { 0xcbcf308e, SUC_lexicon_tags_15 },
    { 0x410f308d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0af3095, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d6f3097, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764f309c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x708f309e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16f30a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b2f30a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf7cf30a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af30ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ef30ae, SUC_lexicon_tags_2 },
    { 0x1c0f30af, SUC_lexicon_tags_17 },
    { 0x64f30b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98af30b2, SUC_lexicon_tags_25 },
    { 0xff0f30b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ef30b8, SUC_lexicon_tags_19 },
    { 0xeb4f30b9, SUC_lexicon_tags_14 },
    { 0x5dcf30b9, SUC_lexicon_tags_37 },
    { 0xa32f30ba, SUC_lexicon_tags_112 },
    { 0xb60f30ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0f30c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64f30cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2f30d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08f30d6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2daf30db, SUC_lexicon_tags_37 },
    { 0x278f30dc, SUC_lexicon_tags_15 },
    { 0xc04f30dd, SUC_lexicon_tags_32 },
    { 0x432f30de, SUC_lexicon_tags_15 },
    { 0x402f30df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2f30e1, SUC_lexicon_tags_38 },
    { 0x776f30e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236f30e7, SUC_lexicon_tags_1 },
    { 0xb86f30e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4f30ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee2f30f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd38f30f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82f30f6, SUC_lexicon_tags_11 },
    { 0xc4af30f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8f3106, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568f310a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1caf310f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50f3113, SUC_lexicon_tags_11 },
    { 0x18ef3114, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd0f3116, SUC_lexicon_tags_29 },
    { 0xbf8f3117, SUC_lexicon_tags_35 },
    { 0x30af3118, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19af3125, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6f3128, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x374f312a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f312e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf88f3130, SUC_lexicon_tags_31 },
    { 0x5c2f3131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baf3136, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb10f3139, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba4f313b, SUC_lexicon_tags_30 },
    { 0x8a6f313c, SUC_lexicon_tags_7 },
    { 0x938f313d, SUC_lexicon_tags_37 },
    { 0x4b8f313b, SUC_lexicon_tags_66 },
    { 0xe98f313f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4caf3146, SUC_lexicon_tags_15 },
    { 0x8bef3146, SUC_lexicon_tags_60 },
    { 0x882f3148, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404f314c, SUC_lexicon_tags_72 },
    { 0x29cf314d, SUC_lexicon_tags_19 },
    { 0x662f314e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2f3151, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f3154, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24af3156, SUC_lexicon_tags_14 },
    { 0x164f3157, SUC_lexicon_tags_7 },
    { 0x856f3158, SUC_lexicon_tags_28 },
    { 0x6fef3159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dcf315b, SUC_lexicon_tags_16 },
    { 0x8aaf315b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x376f315e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ef3164, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0f3168, SUC_lexicon_tags_17 },
    { 0x638f3169, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548f316d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb82f316f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x24af3172, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x244f3174, SUC_lexicon_tags_29 },
    { 0xebaf3174, SUC_lexicon_tags_6 },
    { 0xd7af3176, SUC_lexicon_tags_15 },
    { 0xd9af3177, SUC_lexicon_tags_17 },
    { 0xc5ef3176, SUC_lexicon_tags_37 },
    { 0xb0f3175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0f317f, SUC_lexicon_tags_68 },
    { 0x48af3180, SUC_lexicon_tags_351 },
    { 0xcf0f317f, SUC_lexicon_tags_77 },
    { 0xd1ef3181, SUC_lexicon_tags_1 },
    { 0x7eaf317f, SUC_lexicon_tags_19 },
    { 0xf7ef317f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60f3188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f318e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e6f3190, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6cf3192, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48f3195, SUC_lexicon_tags_61 },
    { 0x5c4f3196, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaf3199, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2af319c, SUC_lexicon_tags_37 },
    { 0xf36f319d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x46ef319f, SUC_lexicon_tags_37 },
    { 0x48cf31a0, SUC_lexicon_tags_37 },
    { 0x85ef31a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x652f31a3, SUC_lexicon_tags_63 },
    { 0x9fcf31a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ef31a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7af31af, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cf31b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4f31b6, SUC_lexicon_tags_4 },
    { 0x548f31b7, SUC_lexicon_tags_15 },
    { 0x2f6f31b8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6f31be, SUC_lexicon_tags_21 },
    { 0xae6f31bf, SUC_lexicon_tags_37 },
    { 0x91ef31c0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x58ef31c2, SUC_lexicon_tags_37 },
    { 0xef2f31c3, SUC_lexicon_tags_23 },
    { 0xb1cf31c4, SUC_lexicon_tags_11 },
    { 0x38af31c3, SUC_lexicon_tags_11 },
    { 0x2e6f31c3, SUC_lexicon_tags_37 },
    { 0x85af31c4, SUC_lexicon_tags_15 },
    { 0xf42f31c8, SUC_lexicon_tags_49 },
    { 0x956f31c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x946f31cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18cf31d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f31db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x986f31e7, SUC_lexicon_tags_37 },
    { 0x200f31e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x692f31ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe6cf31ec, SUC_lexicon_tags_25 },
    { 0xd2af31ec, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548f31f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06f31f8, SUC_lexicon_tags_11 },
    { 0x734f31f8, SUC_lexicon_tags_9 },
    { 0x3eef31f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f31fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x902f31ff, SUC_lexicon_tags_14 },
    { 0x9acf31ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04f3205, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf320a, SUC_lexicon_tags_11 },
    { 0x234f320b, SUC_lexicon_tags_15 },
    { 0xfaaf320c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92f320f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c8f3211, SUC_lexicon_tags_17 },
    { 0x88f3212, SUC_lexicon_tags_25 },
    { 0x51cf3212, SUC_lexicon_tags_11 },
    { 0xeb8f3214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6f3219, SUC_lexicon_tags_37 },
    { 0x4e6f321a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ef321e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6f3224, SUC_lexicon_tags_104 },
    { 0xe56f3224, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ef3228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x646f322b, SUC_lexicon_tags_27 },
    { 0xe4ef322c, SUC_lexicon_tags_14 },
    { 0x686f322d, SUC_lexicon_tags_8 },
    { 0x64ef322c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceaf3232, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15cf3234, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972f323e, SUC_lexicon_tags_25 },
    { 0x292f323f, SUC_lexicon_tags_48 },
    { 0x278f3240, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x250f3243, SUC_lexicon_tags_25 },
    { 0xed6f3244, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa16f3246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cf3249, SUC_lexicon_tags_13 },
    { 0x3def324a, SUC_lexicon_tags_79 },
    { 0x88af324b, SUC_lexicon_tags_17 },
    { 0x75af324a, SUC_lexicon_tags_19 },
    { 0xc6f324a, SUC_lexicon_tags_31 },
    { 0x738f324e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4def3252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af3255, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62ef3257, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x830f3259, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfbcf325b, SUC_lexicon_tags_13 },
    { 0xbbcf325b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb58f325e, SUC_lexicon_tags_49 },
    { 0x588f325e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75ef3261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44ef3263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24af3265, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4f326b, SUC_lexicon_tags_45 },
    { 0xe8cf326b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ef3272, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3ecf3274, SUC_lexicon_tags_15 },
    { 0x286f3275, SUC_lexicon_tags_37 },
    { 0x228f3275, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06f327e, SUC_lexicon_tags_25 },
    { 0x30af327e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf3282, SUC_lexicon_tags_33 },
    { 0x590f3283, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96af3285, SUC_lexicon_tags_38 },
    { 0x598f3285, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748f328f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6f3295, SUC_lexicon_tags_2 },
    { 0x99ef3296, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x632f3298, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfcf329a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf96f329c, SUC_lexicon_tags_63 },
    { 0x352f329d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00f32a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d4f32a3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe58f32a5, SUC_lexicon_tags_38 },
    { 0x24af32a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecf32aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6f32ad, SUC_lexicon_tags_0 },
    { 0x9e8f32ad, SUC_lexicon_tags_23 },
    { 0x52af32ad, SUC_lexicon_tags_7 },
    { 0xbdef32ad, SUC_lexicon_tags_14 },
    { 0xf76f32b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0f32b7, SUC_lexicon_tags_17 },
    { 0xb7ef32b8, SUC_lexicon_tags_25 },
    { 0x754f32b8, SUC_lexicon_tags_9 },
    { 0x90af32ba, SUC_lexicon_tags_60 },
    { 0x8a0f32b8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x902f32bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6dcf32bf, SUC_lexicon_tags_33 },
    { 0xf64f32c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2f32c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa1af32c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x85af32c8, SUC_lexicon_tags_25 },
    { 0x574f32c9, SUC_lexicon_tags_22 },
    { 0xd96f32ca, SUC_lexicon_tags_65 },
    { 0xf9cf32cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4aef32ce, SUC_lexicon_tags_17 },
    { 0x87cf32ce, SUC_lexicon_tags_7 },
    { 0x738f32d0, SUC_lexicon_tags_11 },
    { 0xab0f32d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x984f32d3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x84cf32d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce0f32d7, SUC_lexicon_tags_4 },
    { 0x112f32d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08f32dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc6f32de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6f32e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694f32ea, SUC_lexicon_tags_15 },
    { 0x32ef32eb, SUC_lexicon_tags_17 },
    { 0x52cf32ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8f32ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87cf32f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9c8f32f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cf32f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0f3301, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2f3309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ef3310, SUC_lexicon_tags_12 },
    { 0xd44f3311, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff2f3313, SUC_lexicon_tags_7 },
    { 0x2e4f3313, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd4af3316, SUC_lexicon_tags_7 },
    { 0x82af3316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4af3319, SUC_lexicon_tags_0 },
    { 0x538f331a, SUC_lexicon_tags_27 },
    { 0x2e6f3319, SUC_lexicon_tags_14 },
    { 0xaeaf331c, SUC_lexicon_tags_39 },
    { 0xadaf331b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f3321, SUC_lexicon_tags_25 },
    { 0x11cf3322, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f3326, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x63af3328, SUC_lexicon_tags_0 },
    { 0xc9ef3328, SUC_lexicon_tags_23 },
    { 0xd40f332a, SUC_lexicon_tags_23 },
    { 0x80f332a, SUC_lexicon_tags_28 },
    { 0x5a6f332c, SUC_lexicon_tags_37 },
    { 0x722f3329, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68f3330, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94f3333, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x842f3335, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x65af3338, SUC_lexicon_tags_38 },
    { 0x5b4f3339, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8f333c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x622f333f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea4f3342, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x750f3345, SUC_lexicon_tags_15 },
    { 0xca8f3346, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00f334b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6eaf334d, SUC_lexicon_tags_6 },
    { 0xa46f334e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0f3352, SUC_lexicon_tags_15 },
    { 0x468f3353, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcf3357, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb92f3359, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x848f335b, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x256f335e, SUC_lexicon_tags_46 },
    { 0xa8af335f, SUC_lexicon_tags_28 },
    { 0x386f335e, SUC_lexicon_tags_39 },
    { 0xd1cf3361, SUC_lexicon_tags_13 },
    { 0x968f3362, SUC_lexicon_tags_31 },
    { 0x34af3362, SUC_lexicon_tags_0 },
    { 0x2d4f335e, SUC_lexicon_tags_11 },
    { 0x9ccf3365, SUC_lexicon_tags_27 },
    { 0xcd2f3366, SUC_lexicon_tags_28 },
    { 0x622f3366, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x578f3369, SUC_lexicon_tags_37 },
    { 0xf3af336a, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86f3370, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xeb2f3372, SUC_lexicon_tags_19 },
    { 0x89ef3372, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdef3376, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a6f3378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20f337b, SUC_lexicon_tags_15 },
    { 0x110f337c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x588f337f, SUC_lexicon_tags_27 },
    { 0xcd0f3380, SUC_lexicon_tags_37 },
    { 0xa24f3380, SUC_lexicon_tags_43 },
    { 0xeaaf3382, SUC_lexicon_tags_30 },
    { 0xf72f3383, SUC_lexicon_tags_11 },
    { 0x832f3384, SUC_lexicon_tags_27 },
    { 0xba6f3385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a0f3387, SUC_lexicon_tags_12 },
    { 0xc64f3387, SUC_lexicon_tags_24 },
    { 0x89ef3389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1caf338d, SUC_lexicon_tags_2 },
    { 0xaaf338d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d4f3390, SUC_lexicon_tags_37 },
    { 0xb8cf3391, SUC_lexicon_tags_47 },
    { 0xbe2f3392, SUC_lexicon_tags_53 },
    { 0xc0cf3390, SUC_lexicon_tags_43 },
    { 0xc82f3394, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724f3398, SUC_lexicon_tags_25 },
    { 0xf8af3398, SUC_lexicon_tags_8 },
    { 0x51cf3398, SUC_lexicon_tags_25 },
    { 0xa02f339b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916f33a0, SUC_lexicon_tags_25 },
    { 0x562f33a1, SUC_lexicon_tags_38 },
    { 0x590f33a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4f33a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722f33ac, SUC_lexicon_tags_7 },
    { 0x928f33ac, SUC_lexicon_tags_12 },
    { 0x852f33ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184f33b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f33b5, SUC_lexicon_tags_17 },
    { 0xbfcf33b6, SUC_lexicon_tags_11 },
    { 0xcc4f33b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786f33c0, SUC_lexicon_tags_12 },
    { 0xbfcf33c1, SUC_lexicon_tags_662 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccf33c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8f33cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x27ef33cd, SUC_lexicon_tags_37 },
    { 0xddef33ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x198f33d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecef33d2, SUC_lexicon_tags_47 },
    { 0xd8cf33d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x234f33d5, SUC_lexicon_tags_27 },
    { 0x530f33d5, SUC_lexicon_tags_3 },
    { 0xe40f33d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230f33e0, SUC_lexicon_tags_17 },
    { 0x7e4f33e1, SUC_lexicon_tags_19 },
    { 0xa52f33e0, SUC_lexicon_tags_12 },
    { 0xb82f33e2, SUC_lexicon_tags_17 },
    { 0xdd4f33e1, SUC_lexicon_tags_47 },
    { 0x476f33e1, SUC_lexicon_tags_26 },
    { 0x2fef33e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbacf33e8, SUC_lexicon_tags_25 },
    { 0x772f33e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd4f33eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56f33ed, SUC_lexicon_tags_47 },
    { 0x2fef33ee, SUC_lexicon_tags_9 },
    { 0x706f33ed, SUC_lexicon_tags_15 },
    { 0xa38f33f0, SUC_lexicon_tags_37 },
    { 0xa46f33f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556f33f5, SUC_lexicon_tags_61 },
    { 0x360f33f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ef33f9, SUC_lexicon_tags_7 },
    { 0x6c4f33f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f3400, SUC_lexicon_tags_23 },
    { 0x26af3401, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x168f3403, SUC_lexicon_tags_63 },
    { 0x64cf3403, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ef340b, SUC_lexicon_tags_11 },
    { 0xa3ef340c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4f3414, SUC_lexicon_tags_37 },
    { 0xb72f3415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf3418, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66f341a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a8f341c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240f3421, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84cf3423, SUC_lexicon_tags_7 },
    { 0x21af3424, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f3427, SUC_lexicon_tags_17 },
    { 0x226f3428, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62f342b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aef342e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8f3431, SUC_lexicon_tags_19 },
    { 0x44ef3431, SUC_lexicon_tags_77 },
    { 0xcaaf3433, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2f3436, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0f3439, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70f343f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x158f3441, SUC_lexicon_tags_32 },
    { 0xd28f3442, SUC_lexicon_tags_25 },
    { 0xbf6f3442, SUC_lexicon_tags_60 },
    { 0x65cf3443, SUC_lexicon_tags_1 },
    { 0x9f8f3442, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8f3449, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaf3451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x230f3453, SUC_lexicon_tags_37 },
    { 0x840f3454, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cf3458, SUC_lexicon_tags_27 },
    { 0xdc0f3459, SUC_lexicon_tags_44 },
    { 0x532f345a, SUC_lexicon_tags_49 },
    { 0x7daf345b, SUC_lexicon_tags_25 },
    { 0x88f345c, SUC_lexicon_tags_46 },
    { 0x69af3458, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ef3462, SUC_lexicon_tags_29 },
    { 0x89ef3463, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf2cf3465, SUC_lexicon_tags_27 },
    { 0x3f6f3465, SUC_lexicon_tags_16 },
    { 0xb90f3465, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce4f3469, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f346d, SUC_lexicon_tags_65 },
    { 0x56cf346e, SUC_lexicon_tags_10 },
    { 0xcf6f346e, SUC_lexicon_tags_25 },
    { 0x44ef3470, SUC_lexicon_tags_87 },
    { 0x4dcf3471, SUC_lexicon_tags_18 },
    { 0x43cf346e, SUC_lexicon_tags_11 },
    { 0x52af3473, SUC_lexicon_tags_70 },
    { 0x1f2f3473, SUC_lexicon_tags_2 },
    { 0x80cf3472, SUC_lexicon_tags_17 },
    { 0x162f3471, SUC_lexicon_tags_7 },
    { 0x71cf3477, SUC_lexicon_tags_12 },
    { 0xbb2f3470, SUC_lexicon_tags_12 },
    { 0xd4f3479, SUC_lexicon_tags_27 },
    { 0xdaf3479, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72f347e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc76f3480, SUC_lexicon_tags_27 },
    { 0xd52f3481, SUC_lexicon_tags_39 },
    { 0x2eef3482, SUC_lexicon_tags_11 },
    { 0x926f3480, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f3487, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720f348c, SUC_lexicon_tags_11 },
    { 0x648f348d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x558f348f, SUC_lexicon_tags_60 },
    { 0x4c6f348f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb02f3492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4f3495, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f3499, SUC_lexicon_tags_37 },
    { 0xe3cf349a, SUC_lexicon_tags_37 },
    { 0x872f349b, SUC_lexicon_tags_11 },
    { 0xb3af349c, SUC_lexicon_tags_37 },
    { 0xb04f349d, SUC_lexicon_tags_12 },
    { 0xeb2f349e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56f34a2, SUC_lexicon_tags_11 },
    { 0x78f34a3, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6f34a7, SUC_lexicon_tags_11 },
    { 0x734f34a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe48f34aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5daf34af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f34b4, SUC_lexicon_tags_12 },
    { 0xb22f34b5, SUC_lexicon_tags_7 },
    { 0xda6f34b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x910f34b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ef34c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x146f34c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f34cc, SUC_lexicon_tags_17 },
    { 0xf38f34cc, SUC_lexicon_tags_17 },
    { 0x956f34ce, SUC_lexicon_tags_23 },
    { 0x626f34cf, SUC_lexicon_tags_1 },
    { 0x396f34d0, SUC_lexicon_tags_17 },
    { 0x464f34d1, SUC_lexicon_tags_43 },
    { 0xb14f34d2, SUC_lexicon_tags_37 },
    { 0xdf4f34d3, SUC_lexicon_tags_0 },
    { 0x9c0f34d3, SUC_lexicon_tags_17 },
    { 0x9fef34d5, SUC_lexicon_tags_48 },
    { 0xd76f34d6, SUC_lexicon_tags_11 },
    { 0xc5ef34d4, SUC_lexicon_tags_66 },
    { 0x680f34cf, SUC_lexicon_tags_6 },
    { 0xc5cf34d0, SUC_lexicon_tags_1 },
    { 0x4aaf34da, SUC_lexicon_tags_26 },
    { 0xd3ef34db, SUC_lexicon_tags_11 },
    { 0x692f34d9, SUC_lexicon_tags_38 },
    { 0xb6ef34d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2af34eb, SUC_lexicon_tags_65 },
    { 0x858f34eb, SUC_lexicon_tags_10 },
    { 0x588f34eb, SUC_lexicon_tags_37 },
    { 0xa58f34ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ef34f1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684f34f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf86f34fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68f34ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd8f3501, SUC_lexicon_tags_2 },
    { 0xb30f3502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ef3508, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2f350d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0f3510, SUC_lexicon_tags_15 },
    { 0x316f3511, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ef3515, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3aaf3517, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf96f3519, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4f351c, SUC_lexicon_tags_11 },
    { 0x53af351d, SUC_lexicon_tags_37 },
    { 0x708f351e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6f3521, SUC_lexicon_tags_31 },
    { 0x6f0f3522, SUC_lexicon_tags_37 },
    { 0x62af3523, SUC_lexicon_tags_25 },
    { 0x998f3524, SUC_lexicon_tags_33 },
    { 0x96ef3524, SUC_lexicon_tags_7 },
    { 0xc2ef3526, SUC_lexicon_tags_19 },
    { 0x654f3526, SUC_lexicon_tags_27 },
    { 0xbe0f3523, SUC_lexicon_tags_23 },
    { 0x1e6f3529, SUC_lexicon_tags_1 },
    { 0xf2f3528, SUC_lexicon_tags_37 },
    { 0x80f352b, SUC_lexicon_tags_31 },
    { 0xdbef352b, SUC_lexicon_tags_47 },
    { 0x1d8f352b, SUC_lexicon_tags_30 },
    { 0x2e2f352c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0f3537, SUC_lexicon_tags_3 },
    { 0xafcf3537, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126f353f, SUC_lexicon_tags_48 },
    { 0xc84f3540, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96f354c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70af3550, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3a6f3552, SUC_lexicon_tags_38 },
    { 0x718f3552, SUC_lexicon_tags_23 },
    { 0x53cf3554, SUC_lexicon_tags_282 },
    { 0xc86f3555, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e8f3557, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x23ef3559, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x42f355b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x226f355d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31af355f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x68f3561, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0f3564, SUC_lexicon_tags_32 },
    { 0xfbaf3564, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16f3569, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93cf356c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42cf3577, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ef357c, SUC_lexicon_tags_82 },
    { 0x30cf357d, SUC_lexicon_tags_12 },
    { 0xa8f357e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4f3583, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x594f3586, SUC_lexicon_tags_8 },
    { 0xe70f3587, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x702f3589, SUC_lexicon_tags_8 },
    { 0x81ef358a, SUC_lexicon_tags_12 },
    { 0xd24f358a, SUC_lexicon_tags_25 },
    { 0xbccf358a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab2f358e, SUC_lexicon_tags_5 },
    { 0x730f358e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf3592, SUC_lexicon_tags_39 },
    { 0x398f3593, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x124f3595, SUC_lexicon_tags_1 },
    { 0x73af3595, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2f359a, SUC_lexicon_tags_49 },
    { 0x3c6f359a, SUC_lexicon_tags_31 },
    { 0x16f359a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2f359f, SUC_lexicon_tags_14 },
    { 0x218f35a0, SUC_lexicon_tags_49 },
    { 0x6dcf35a1, SUC_lexicon_tags_15 },
    { 0xf2ef35a2, SUC_lexicon_tags_17 },
    { 0x66af35a2, SUC_lexicon_tags_10 },
    { 0x47af35a4, SUC_lexicon_tags_93 },
    { 0x99af35a4, SUC_lexicon_tags_1 },
    { 0x974f35a6, SUC_lexicon_tags_37 },
    { 0xf06f35a6, SUC_lexicon_tags_16 },
    { 0x4daf35a8, SUC_lexicon_tags_12 },
    { 0x26af35a9, SUC_lexicon_tags_27 },
    { 0xe9ef35aa, SUC_lexicon_tags_37 },
    { 0xee2f35ab, SUC_lexicon_tags_25 },
    { 0xb74f35ab, SUC_lexicon_tags_19 },
    { 0xa24f35ac, SUC_lexicon_tags_4 },
    { 0x286f35a4, SUC_lexicon_tags_14 },
    { 0xf32f35a7, SUC_lexicon_tags_65 },
    { 0xf06f35b0, SUC_lexicon_tags_30 },
    { 0x4bef35b0, SUC_lexicon_tags_23 },
    { 0x944f35b2, SUC_lexicon_tags_2 },
    { 0x218f35b3, SUC_lexicon_tags_25 },
    { 0x9bcf35b3, SUC_lexicon_tags_37 },
    { 0x202f35a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64f35b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0f35c1, SUC_lexicon_tags_60 },
    { 0xe2cf35c2, SUC_lexicon_tags_14 },
    { 0x24f35c3, SUC_lexicon_tags_11 },
    { 0xf2cf35c2, SUC_lexicon_tags_37 },
    { 0xa9ef35c5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ef35ca, SUC_lexicon_tags_2 },
    { 0xcdaf35cb, SUC_lexicon_tags_25 },
    { 0xb92f35cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18af35d0, SUC_lexicon_tags_37 },
    { 0xdaef35d1, SUC_lexicon_tags_79 },
    { 0xc08f35d2, SUC_lexicon_tags_3 },
    { 0xe7cf35d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2cf35d5, SUC_lexicon_tags_0 },
    { 0x33af35d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfecf35d8, SUC_lexicon_tags_25 },
    { 0x440f35d9, SUC_lexicon_tags_2 },
    { 0x5a0f35d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd3cf35dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x782f35df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eef35e4, SUC_lexicon_tags_40 },
    { 0x678f35e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x222f35e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456f35ec, SUC_lexicon_tags_11 },
    { 0x4acf35ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff0f35ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6f35f2, SUC_lexicon_tags_21 },
    { 0x44ef35f3, SUC_lexicon_tags_33 },
    { 0x31ef35f3, SUC_lexicon_tags_30 },
    { 0x570f35f3, SUC_lexicon_tags_15 },
    { 0xa9cf35f5, SUC_lexicon_tags_66 },
    { 0x3fef35f2, SUC_lexicon_tags_28 },
    { 0xac2f35f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf35fb, SUC_lexicon_tags_12 },
    { 0x864f35fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9e8f35fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe90f3601, SUC_lexicon_tags_17 },
    { 0xb9af3602, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50af3604, SUC_lexicon_tags_7 },
    { 0x8caf3604, SUC_lexicon_tags_118 },
    { 0xaf8f3606, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0xaf6f3608, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x256f360b, SUC_lexicon_tags_17 },
    { 0x4bef360b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x892f360e, SUC_lexicon_tags_11 },
    { 0x64cf360e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ef3612, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcf3617, SUC_lexicon_tags_14 },
    { 0x1c0f3618, SUC_lexicon_tags_7 },
    { 0xd4af3619, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xae8f361b, SUC_lexicon_tags_12 },
    { 0xf20f361b, SUC_lexicon_tags_487 },
    { 0xef0f361d, SUC_lexicon_tags_49 },
    { 0xdd4f361e, SUC_lexicon_tags_33 },
    { 0x3e0f361d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc8f3621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc4f3623, SUC_lexicon_tags_1 },
    { 0x4faf3624, SUC_lexicon_tags_43 },
    { 0x4fef3625, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf04f3627, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaf362a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf362d, SUC_lexicon_tags_37 },
    { 0x4e8f362e, SUC_lexicon_tags_27 },
    { 0xac4f362f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x678f3632, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa22f363a, SUC_lexicon_tags_68 },
    { 0x83af363b, SUC_lexicon_tags_68 },
    { 0xc9af363c, SUC_lexicon_tags_45 },
    { 0x9b6f363d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0f3646, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854f364a, SUC_lexicon_tags_47 },
    { 0xe06f364b, SUC_lexicon_tags_17 },
    { 0xa50f364c, SUC_lexicon_tags_27 },
    { 0xfecf364d, SUC_lexicon_tags_17 },
    { 0x72cf364e, SUC_lexicon_tags_2 },
    { 0x7d6f364d, SUC_lexicon_tags_107 },
    { 0xccaf364d, SUC_lexicon_tags_6 },
    { 0xcb0f364e, SUC_lexicon_tags_17 },
    { 0x6def364d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f3655, SUC_lexicon_tags_25 },
    { 0xac0f3656, SUC_lexicon_tags_33 },
    { 0x24af3657, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f365b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27af365f, SUC_lexicon_tags_43 },
    { 0x2eaf365f, SUC_lexicon_tags_12 },
    { 0xfa0f3661, SUC_lexicon_tags_13 },
    { 0x5f8f3661, SUC_lexicon_tags_25 },
    { 0x2a8f3662, SUC_lexicon_tags_2 },
    { 0x7b8f365f, SUC_lexicon_tags_38 },
    { 0x378f3660, SUC_lexicon_tags_55 },
    { 0x4cef3665, SUC_lexicon_tags_170 },
    { 0xe48f3664, SUC_lexicon_tags_37 },
    { 0x540f3668, SUC_lexicon_tags_53 },
    { 0x1c0f3669, SUC_lexicon_tags_14 },
    { 0x5eaf3668, SUC_lexicon_tags_38 },
    { 0x612f3668, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f366e, SUC_lexicon_tags_1 },
    { 0xbfcf366e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38f3674, SUC_lexicon_tags_6 },
    { 0xe5cf3674, SUC_lexicon_tags_38 },
    { 0xdc2f3674, SUC_lexicon_tags_37 },
    { 0x462f3677, SUC_lexicon_tags_45 },
    { 0x9ecf3678, SUC_lexicon_tags_11 },
    { 0x414f3678, SUC_lexicon_tags_14 },
    { 0xa8f3679, SUC_lexicon_tags_17 },
    { 0x7fcf3677, SUC_lexicon_tags_2 },
    { 0x36cf3677, SUC_lexicon_tags_31 },
    { 0x878f367d, SUC_lexicon_tags_66 },
    { 0x708f367e, SUC_lexicon_tags_37 },
    { 0x78af3676, SUC_lexicon_tags_15 },
    { 0x5aaf3680, SUC_lexicon_tags_37 },
    { 0xd94f3681, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0f3684, SUC_lexicon_tags_1 },
    { 0xd62f3685, SUC_lexicon_tags_33 },
    { 0x9eaf3684, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa9af3688, SUC_lexicon_tags_45 },
    { 0xb68f3689, SUC_lexicon_tags_65 },
    { 0xeef3689, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8def368d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacf3691, SUC_lexicon_tags_23 },
    { 0x990f3692, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c2f3694, SUC_lexicon_tags_224 },
    { 0x170f3695, SUC_lexicon_tags_1 },
    { 0x244f3695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34f369a, SUC_lexicon_tags_19 },
    { 0x744f369a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6f36a0, SUC_lexicon_tags_11 },
    { 0x510f36a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x142f36a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeacf36a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cf36ac, SUC_lexicon_tags_47 },
    { 0x25cf36ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76f36b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fef36b9, SUC_lexicon_tags_18 },
    { 0xaf36ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8f36c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x448f36c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20f36c8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xf74f36ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ef36d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412f36d8, SUC_lexicon_tags_1 },
    { 0x584f36d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24f36dc, SUC_lexicon_tags_11 },
    { 0x31af36dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11af36e1, SUC_lexicon_tags_49 },
    { 0x8a0f36e1, SUC_lexicon_tags_29 },
    { 0x34cf36e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56cf36e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2f36eb, SUC_lexicon_tags_55 },
    { 0x5d6f36ec, SUC_lexicon_tags_138 },
    { 0x34ef36eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x818f36f0, SUC_lexicon_tags_236 },
    { 0xbeaf36f0, SUC_lexicon_tags_17 },
    { 0xfc6f36f0, SUC_lexicon_tags_38 },
    { 0x12cf36f3, SUC_lexicon_tags_48 },
    { 0xfa2f36f4, SUC_lexicon_tags_29 },
    { 0x250f36f0, SUC_lexicon_tags_62 },
    { 0x256f36f6, SUC_lexicon_tags_12 },
    { 0xa14f36f6, SUC_lexicon_tags_11 },
    { 0x15cf36f8, SUC_lexicon_tags_2 },
    { 0xe8ef36f9, SUC_lexicon_tags_25 },
    { 0x6f0f36fa, SUC_lexicon_tags_12 },
    { 0x62af36fa, SUC_lexicon_tags_6 },
    { 0x8d2f36fc, SUC_lexicon_tags_0 },
    { 0xc34f36fb, SUC_lexicon_tags_37 },
    { 0xc66f36fe, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x378f3701, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bef3703, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ef370d, SUC_lexicon_tags_12 },
    { 0x402f370e, SUC_lexicon_tags_15 },
    { 0x646f370e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb44f3711, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0f3714, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x678f3716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ef3719, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xee2f371b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498f3721, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654f3725, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xce4f3727, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x116f3729, SUC_lexicon_tags_12 },
    { 0xa7ef372a, SUC_lexicon_tags_70 },
    { 0x372f372a, SUC_lexicon_tags_1 },
    { 0xa8f372c, SUC_lexicon_tags_7 },
    { 0xa9ef372d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x23af372f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b6f3731, SUC_lexicon_tags_17 },
    { 0xf18f3731, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9ef3734, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9af3737, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x744f373a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792f3740, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfeaf3742, SUC_lexicon_tags_37 },
    { 0xa18f3743, SUC_lexicon_tags_0 },
    { 0xba8f3742, SUC_lexicon_tags_84 },
    { 0x1c8f3745, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34f3749, SUC_lexicon_tags_38 },
    { 0xf6ef3749, SUC_lexicon_tags_12 },
    { 0x3dcf374b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c8f374d, SUC_lexicon_tags_26 },
    { 0x7f2f374e, SUC_lexicon_tags_11 },
    { 0x560f374d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb36f3751, SUC_lexicon_tags_11 },
    { 0x288f3751, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc98f3756, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90f375a, SUC_lexicon_tags_343 },
    { 0x654f375b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86f375f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f6f3761, SUC_lexicon_tags_8 },
    { 0x36ef3762, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa48f3765, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36af3769, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x47af376c, SUC_lexicon_tags_38 },
    { 0x270f376c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f3771, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x23ef3773, SUC_lexicon_tags_35 },
    { 0x876f3774, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecf3777, SUC_lexicon_tags_27 },
    { 0xae6f3778, SUC_lexicon_tags_49 },
    { 0x56af3777, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ef377c, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718f3780, SUC_lexicon_tags_37 },
    { 0xd06f3781, SUC_lexicon_tags_47 },
    { 0xdaaf3782, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68f3789, SUC_lexicon_tags_38 },
    { 0xc3cf378a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf6f378c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef378f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcf3792, SUC_lexicon_tags_49 },
    { 0xf4ef3793, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64f3796, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe52f3798, SUC_lexicon_tags_7 },
    { 0x174f3799, SUC_lexicon_tags_2 },
    { 0x69ef379a, SUC_lexicon_tags_21 },
    { 0xd02f379b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc22f379d, SUC_lexicon_tags_1 },
    { 0x8a0f379d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ef37a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cf37a8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xed6f37aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7b8f37ac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfacf37af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94f37b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe9cf37b4, SUC_lexicon_tags_25 },
    { 0x262f37b5, SUC_lexicon_tags_48 },
    { 0xc2cf37b6, SUC_lexicon_tags_1 },
    { 0xac4f37b6, SUC_lexicon_tags_25 },
    { 0xa28f37b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ef37bb, SUC_lexicon_tags_49 },
    { 0xafef37bc, SUC_lexicon_tags_12 },
    { 0x2a6f37bd, SUC_lexicon_tags_11 },
    { 0x406f37be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x146f37c0, SUC_lexicon_tags_17 },
    { 0x6f37c1, SUC_lexicon_tags_12 },
    { 0x8baf37c2, SUC_lexicon_tags_7 },
    { 0xa44f37c3, SUC_lexicon_tags_28 },
    { 0x380f37c4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xbf2f37c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7eaf37c8, SUC_lexicon_tags_25 },
    { 0x442f37c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ef37ce, SUC_lexicon_tags_82 },
    { 0x97af37cf, SUC_lexicon_tags_2 },
    { 0x154f37cf, SUC_lexicon_tags_16 },
    { 0x81ef37cf, SUC_lexicon_tags_2 },
    { 0x910f37d2, SUC_lexicon_tags_1 },
    { 0x93af37d3, SUC_lexicon_tags_14 },
    { 0x896f37d3, SUC_lexicon_tags_12 },
    { 0x702f37d5, SUC_lexicon_tags_74 },
    { 0x73cf37d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104f37da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x622f37dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6f37e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf20f37e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaef37e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6e6f37e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa68f37e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd88f37eb, SUC_lexicon_tags_49 },
    { 0x836f37ec, SUC_lexicon_tags_0 },
    { 0x482f37ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902f37f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaf37fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f37ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc2f3801, SUC_lexicon_tags_4 },
    { 0x32af3801, SUC_lexicon_tags_8 },
    { 0xe02f3803, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc04f3805, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc86f3807, SUC_lexicon_tags_49 },
    { 0x84f3808, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172f380c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x39af380f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846f3813, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd90f3815, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6f3818, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf0ef381a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe22f3820, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6cf3822, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ef3828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cf3830, SUC_lexicon_tags_29 },
    { 0x2e0f3831, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb4f3833, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80ef3835, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56af383b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acf3840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x402f3843, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf54f3845, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x37ef3847, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4ef3849, SUC_lexicon_tags_9 },
    { 0x488f384a, SUC_lexicon_tags_17 },
    { 0x602f384b, SUC_lexicon_tags_7 },
    { 0xcf6f384c, SUC_lexicon_tags_11 },
    { 0xb50f384d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x844f384f, SUC_lexicon_tags_17 },
    { 0x870f3850, SUC_lexicon_tags_25 },
    { 0xcd6f3850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72f3856, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af385b, SUC_lexicon_tags_30 },
    { 0x488f385c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44f3861, SUC_lexicon_tags_7 },
    { 0x17af3861, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2f386b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2f3870, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af3875, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34f387c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88f3880, SUC_lexicon_tags_10 },
    { 0xa9cf3881, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cf3887, SUC_lexicon_tags_11 },
    { 0x8c4f3887, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe72f388a, SUC_lexicon_tags_11 },
    { 0x1def388b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f388f, SUC_lexicon_tags_0 },
    { 0x8a4f3890, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2f3894, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2f3899, SUC_lexicon_tags_65 },
    { 0xdb6f389a, SUC_lexicon_tags_26 },
    { 0x52ef389b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x114f389e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f38a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2f38a7, SUC_lexicon_tags_11 },
    { 0xf8ef38a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa6cf38aa, SUC_lexicon_tags_17 },
    { 0xaaaf38ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x614f38ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbeef38af, SUC_lexicon_tags_1 },
    { 0x602f38b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaf38b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2baf38b8, SUC_lexicon_tags_25 },
    { 0x94ef38b8, SUC_lexicon_tags_16 },
    { 0x8b4f38ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cf38c5, SUC_lexicon_tags_9 },
    { 0xa84f38c6, SUC_lexicon_tags_60 },
    { 0x3f4f38c7, SUC_lexicon_tags_39 },
    { 0x272f38c8, SUC_lexicon_tags_11 },
    { 0xddcf38c8, SUC_lexicon_tags_33 },
    { 0x61af38c7, SUC_lexicon_tags_37 },
    { 0x46ef38c9, SUC_lexicon_tags_3 },
    { 0x988f38c6, SUC_lexicon_tags_11 },
    { 0x4b6f38cc, SUC_lexicon_tags_2 },
    { 0x4b0f38ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f38d1, SUC_lexicon_tags_55 },
    { 0x81cf38d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde8f38d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ef38d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcecf38d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafaf38db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6f38df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef38e2, SUC_lexicon_tags_37 },
    { 0xecef38e3, SUC_lexicon_tags_0 },
    { 0x11cf38e3, SUC_lexicon_tags_27 },
    { 0xac4f38e5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xe0af38e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef8f38e9, SUC_lexicon_tags_17 },
    { 0xed0f38ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72f38f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0f38f7, SUC_lexicon_tags_11 },
    { 0x6c8f38f8, SUC_lexicon_tags_11 },
    { 0x88ef38f9, SUC_lexicon_tags_60 },
    { 0x752f38fa, SUC_lexicon_tags_12 },
    { 0x74f38fb, SUC_lexicon_tags_63 },
    { 0x68af38f8, SUC_lexicon_tags_11 },
    { 0xcc6f38fd, SUC_lexicon_tags_45 },
    { 0xaa4f38fd, SUC_lexicon_tags_11 },
    { 0xebef38ff, SUC_lexicon_tags_11 },
    { 0x432f3900, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x804f3902, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7caf3906, SUC_lexicon_tags_43 },
    { 0x1c6f3907, SUC_lexicon_tags_32 },
    { 0x318f3907, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf6f390a, SUC_lexicon_tags_25 },
    { 0x846f390b, SUC_lexicon_tags_31 },
    { 0xc6f390c, SUC_lexicon_tags_11 },
    { 0x6cf390d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cf3912, SUC_lexicon_tags_16 },
    { 0xb64f3912, SUC_lexicon_tags_39 },
    { 0xeccf3914, SUC_lexicon_tags_8 },
    { 0x394f3915, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16af3919, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cf391c, SUC_lexicon_tags_47 },
    { 0x248f391c, SUC_lexicon_tags_13 },
    { 0x95af391e, SUC_lexicon_tags_21 },
    { 0x61af391e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x108f3925, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x474f3927, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f392b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaf392f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x858f3932, SUC_lexicon_tags_37 },
    { 0x732f3932, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x730f3935, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcf3938, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00f393d, SUC_lexicon_tags_23 },
    { 0x594f393e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8ef3940, SUC_lexicon_tags_23 },
    { 0xf66f3940, SUC_lexicon_tags_55 },
    { 0x3daf3940, SUC_lexicon_tags_77 },
    { 0x234f3943, SUC_lexicon_tags_1 },
    { 0xc8af3944, SUC_lexicon_tags_45 },
    { 0xcd8f3944, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0f3949, SUC_lexicon_tags_27 },
    { 0x68cf394a, SUC_lexicon_tags_45 },
    { 0xed2f394b, SUC_lexicon_tags_60 },
    { 0x8fcf394c, SUC_lexicon_tags_17 },
    { 0xfeef3949, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6f3951, SUC_lexicon_tags_37 },
    { 0x5fef3951, SUC_lexicon_tags_49 },
    { 0xb2cf3953, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x29cf3955, SUC_lexicon_tags_27 },
    { 0xb06f3956, SUC_lexicon_tags_17 },
    { 0xe40f3957, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f395c, SUC_lexicon_tags_7 },
    { 0x124f395d, SUC_lexicon_tags_1 },
    { 0xd16f395e, SUC_lexicon_tags_68 },
    { 0xbb0f395d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6ef3961, SUC_lexicon_tags_17 },
    { 0x208f3962, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c2f3967, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838f3970, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94f3973, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6f3976, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cf3979, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ef397c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc4f397e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7a2f3980, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f3983, SUC_lexicon_tags_26 },
    { 0xa78f3983, SUC_lexicon_tags_17 },
    { 0xc90f3983, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x188f398c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ef3991, SUC_lexicon_tags_2 },
    { 0x342f3992, SUC_lexicon_tags_25 },
    { 0x53ef3991, SUC_lexicon_tags_43 },
    { 0xf80f3991, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x22ef3996, SUC_lexicon_tags_65 },
    { 0x1acf3997, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2f399a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccf399f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8f39a3, SUC_lexicon_tags_32 },
    { 0x400f39a4, SUC_lexicon_tags_0 },
    { 0xdecf39a5, SUC_lexicon_tags_46 },
    { 0x944f39a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x960f39a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e4f39ad, SUC_lexicon_tags_25 },
    { 0x608f39ad, SUC_lexicon_tags_37 },
    { 0x464f39af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa78f39b1, SUC_lexicon_tags_37 },
    { 0xecef39b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f39b5, SUC_lexicon_tags_38 },
    { 0xd88f39b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5baf39b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef39bd, SUC_lexicon_tags_17 },
    { 0x37cf39be, SUC_lexicon_tags_26 },
    { 0xe8f39bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb10f39c1, SUC_lexicon_tags_26 },
    { 0x884f39c2, SUC_lexicon_tags_17 },
    { 0x98af39c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f2f39c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552f39c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62f39cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92f39cf, SUC_lexicon_tags_37 },
    { 0x112f39d0, SUC_lexicon_tags_1 },
    { 0x2a4f39cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x182f39d4, SUC_lexicon_tags_37 },
    { 0xe92f39d4, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x666f39d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f39dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6f39e1, SUC_lexicon_tags_27 },
    { 0x29af39e2, SUC_lexicon_tags_37 },
    { 0x584f39e3, SUC_lexicon_tags_1 },
    { 0xb44f39e4, SUC_lexicon_tags_1 },
    { 0x9fcf39e3, SUC_lexicon_tags_34 },
    { 0x9d2f39e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60af39ee, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f39f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28f39f6, SUC_lexicon_tags_30 },
    { 0x4fef39f7, SUC_lexicon_tags_16 },
    { 0x37ef39f6, SUC_lexicon_tags_29 },
    { 0x7c8f39f7, SUC_lexicon_tags_17 },
    { 0x44af39f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c8f39fc, SUC_lexicon_tags_30 },
    { 0xd36f39fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35af39ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ef3a05, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4f3a08, SUC_lexicon_tags_55 },
    { 0xbbaf3a08, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x252f3a0b, SUC_lexicon_tags_12 },
    { 0x7eef3a0c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176f3a12, SUC_lexicon_tags_37 },
    { 0x16ef3a12, SUC_lexicon_tags_1 },
    { 0xc08f3a12, SUC_lexicon_tags_33 },
    { 0xdf8f3a14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82f3a1a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598f3a20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5af3a23, SUC_lexicon_tags_30 },
    { 0xf2cf3a24, SUC_lexicon_tags_77 },
    { 0x7e8f3a25, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x238f3a27, SUC_lexicon_tags_16 },
    { 0xf8ef3a27, SUC_lexicon_tags_34 },
    { 0x86ef3a29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd66f3a2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x78ef3a2d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x886f3a32, SUC_lexicon_tags_27 },
    { 0x428f3a32, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x526f3a35, SUC_lexicon_tags_11 },
    { 0x1fcf3a35, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x184f3a38, SUC_lexicon_tags_31 },
    { 0x614f3a39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x372f3a3b, SUC_lexicon_tags_26 },
    { 0x852f3a3c, SUC_lexicon_tags_7 },
    { 0x7a4f3a3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4f3a42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x718f3a44, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92f3a4a, SUC_lexicon_tags_1 },
    { 0x346f3a4a, SUC_lexicon_tags_27 },
    { 0xa06f3a4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabef3a57, SUC_lexicon_tags_33 },
    { 0x38cf3a58, SUC_lexicon_tags_31 },
    { 0x9f0f3a59, SUC_lexicon_tags_37 },
    { 0x916f3a57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204f3a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd4f3a66, SUC_lexicon_tags_31 },
    { 0xc4ef3a67, SUC_lexicon_tags_49 },
    { 0x98ef3a68, SUC_lexicon_tags_11 },
    { 0x1af3a69, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x534f3a6c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9caf3a6e, SUC_lexicon_tags_7 },
    { 0x44ef3a6f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x324f3a71, SUC_lexicon_tags_2 },
    { 0xd02f3a72, SUC_lexicon_tags_15 },
    { 0xcacf3a73, SUC_lexicon_tags_17 },
    { 0x982f3a72, SUC_lexicon_tags_14 },
    { 0x3ccf3a71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374f3a7c, SUC_lexicon_tags_47 },
    { 0xf58f3a7c, SUC_lexicon_tags_31 },
    { 0x502f3a7c, SUC_lexicon_tags_31 },
    { 0x268f3a7f, SUC_lexicon_tags_69 },
    { 0x34cf3a7f, SUC_lexicon_tags_6 },
    { 0x224f3a81, SUC_lexicon_tags_60 },
    { 0xc80f3a7d, SUC_lexicon_tags_1 },
    { 0xbbaf3a82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0f3a87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8f3a8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51ef3a8f, SUC_lexicon_tags_14 },
    { 0xd98f3a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95ef3a92, SUC_lexicon_tags_16 },
    { 0xe84f3a92, SUC_lexicon_tags_37 },
    { 0x896f3a94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x956f3a96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x118f3a99, SUC_lexicon_tags_45 },
    { 0x4a0f3a9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42af3a9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690f3aa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22f3aa4, SUC_lexicon_tags_525 },
    { 0xa04f3aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76af3aa8, SUC_lexicon_tags_1 },
    { 0xbc0f3aa9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x480f3aab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x614f3aad, SUC_lexicon_tags_63 },
    { 0xc9cf3aae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x406f3ab1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cf3ab6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4af3ab9, SUC_lexicon_tags_7 },
    { 0xfc8f3aba, SUC_lexicon_tags_27 },
    { 0x7fef3aba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4f3ac0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0f3acc, SUC_lexicon_tags_45 },
    { 0x9e8f3acd, SUC_lexicon_tags_15 },
    { 0x2c0f3acd, SUC_lexicon_tags_25 },
    { 0xe10f3ace, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60f3ad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15cf3ad5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0f3add, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x952f3adf, SUC_lexicon_tags_45 },
    { 0xa9cf3ae0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x498f3ae3, SUC_lexicon_tags_17 },
    { 0xbdcf3ae4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0f3ae9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecf3aef, SUC_lexicon_tags_9 },
    { 0xff8f3aef, SUC_lexicon_tags_2 },
    { 0x422f3af1, SUC_lexicon_tags_81 },
    { 0x3b0f3af2, SUC_lexicon_tags_7 },
    { 0xb28f3af3, SUC_lexicon_tags_12 },
    { 0x7aaf3af4, SUC_lexicon_tags_17 },
    { 0x430f3af5, SUC_lexicon_tags_9 },
    { 0xb4af3af5, SUC_lexicon_tags_23 },
    { 0x994f3af6, SUC_lexicon_tags_2 },
    { 0xc86f3af8, SUC_lexicon_tags_69 },
    { 0x43cf3af4, SUC_lexicon_tags_14 },
    { 0x4ccf3af5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2f3afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f3b01, SUC_lexicon_tags_23 },
    { 0x95af3b02, SUC_lexicon_tags_43 },
    { 0xb2cf3b03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8f2f3b05, SUC_lexicon_tags_1 },
    { 0xb44f3b06, SUC_lexicon_tags_61 },
    { 0x18f3b07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6f3b0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ef3b11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0f3b18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cf3b1b, SUC_lexicon_tags_30 },
    { 0x146f3b1c, SUC_lexicon_tags_19 },
    { 0x7daf3b1d, SUC_lexicon_tags_38 },
    { 0x4b8f3b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb00f3b21, SUC_lexicon_tags_7 },
    { 0xde8f3b22, SUC_lexicon_tags_17 },
    { 0x936f3b23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6af3b25, SUC_lexicon_tags_32 },
    { 0x226f3b25, SUC_lexicon_tags_31 },
    { 0x4eaf3b27, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92f3b2d, SUC_lexicon_tags_10 },
    { 0xeacf3b2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64f3b35, SUC_lexicon_tags_47 },
    { 0x3b4f3b36, SUC_lexicon_tags_8 },
    { 0x4faf3b36, SUC_lexicon_tags_92 },
    { 0xb2f3b38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x674f3b3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c8f3b3c, SUC_lexicon_tags_12 },
    { 0x362f3b3d, SUC_lexicon_tags_2 },
    { 0x79cf3b3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f2f3b40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386f3b47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92ef3b49, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0f3b52, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8f3b58, SUC_lexicon_tags_70 },
    { 0x2cef3b59, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x690f3b5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe80f3b5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f3b64, SUC_lexicon_tags_32 },
    { 0x79ef3b64, SUC_lexicon_tags_37 },
    { 0xc4cf3b66, SUC_lexicon_tags_7 },
    { 0xd98f3b67, SUC_lexicon_tags_94 },
    { 0x45af3b67, SUC_lexicon_tags_17 },
    { 0x1c6f3b68, SUC_lexicon_tags_34 },
    { 0x2e2f3b68, SUC_lexicon_tags_8 },
    { 0x36ef3b65, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcf3b6f, SUC_lexicon_tags_3 },
    { 0x726f3b70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28f3b78, SUC_lexicon_tags_27 },
    { 0x586f3b79, SUC_lexicon_tags_68 },
    { 0x1e2f3b78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9af3b7c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2f3b82, SUC_lexicon_tags_63 },
    { 0xefcf3b82, SUC_lexicon_tags_70 },
    { 0xc36f3b84, SUC_lexicon_tags_23 },
    { 0x5f2f3b85, SUC_lexicon_tags_25 },
    { 0xeaf3b86, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9f0f3b88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d0f3b8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa86f3b8c, SUC_lexicon_tags_9 },
    { 0x9d6f3b8d, SUC_lexicon_tags_6 },
    { 0xfd8f3b8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0af3b92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x540f3b95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8f3b97, SUC_lexicon_tags_4 },
    { 0xa5af3b98, SUC_lexicon_tags_60 },
    { 0xdcaf3b97, SUC_lexicon_tags_16 },
    { 0x1d6f3b98, SUC_lexicon_tags_71 },
    { 0xd68f3b9b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6a8f3b9d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6f3ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaf3ba4, SUC_lexicon_tags_82 },
    { 0x9ccf3ba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x874f3ba7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4e4f3ba9, SUC_lexicon_tags_21 },
    { 0x3cef3baa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106f3bb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ef3bb4, SUC_lexicon_tags_37 },
    { 0xb6ef3bb4, SUC_lexicon_tags_37 },
    { 0x918f3bb6, SUC_lexicon_tags_6 },
    { 0x83ef3bb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x912f3bba, SUC_lexicon_tags_45 },
    { 0x9a4f3bbb, SUC_lexicon_tags_11 },
    { 0xf70f3bbb, SUC_lexicon_tags_7 },
    { 0xd3ef3bbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabef3bbf, SUC_lexicon_tags_45 },
    { 0x736f3bc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf3bc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cef3bc9, SUC_lexicon_tags_11 },
    { 0xc1af3bc9, SUC_lexicon_tags_1 },
    { 0xd02f3bca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0f3bd1, SUC_lexicon_tags_15 },
    { 0x100f3bd2, SUC_lexicon_tags_15 },
    { 0x71cf3bd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8f3bd6, SUC_lexicon_tags_2 },
    { 0x4a2f3bd7, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0x3b4f3bd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39cf3bdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f3bdf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb7cf3be1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb74f3be3, SUC_lexicon_tags_37 },
    { 0x1daf3be3, SUC_lexicon_tags_8 },
    { 0x728f3be5, SUC_lexicon_tags_31 },
    { 0xfeef3be6, SUC_lexicon_tags_77 },
    { 0x60f3be3, SUC_lexicon_tags_94 },
    { 0x784f3be8, SUC_lexicon_tags_37 },
    { 0x402f3be6, SUC_lexicon_tags_2 },
    { 0xefef3be9, SUC_lexicon_tags_27 },
    { 0x5a8f3be5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb12f3bed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x568f3bef, SUC_lexicon_tags_0 },
    { 0xa34f3bf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x516f3bf2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x63ef3bf4, SUC_lexicon_tags_11 },
    { 0x7eef3bf5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x89af3bf7, SUC_lexicon_tags_49 },
    { 0x7b0f3bf8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe5af3bfa, SUC_lexicon_tags_25 },
    { 0x522f3bfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6f3bfe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486f3c02, SUC_lexicon_tags_11 },
    { 0xb2af3c02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b4f3c05, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x13ef3c07, SUC_lexicon_tags_37 },
    { 0xecaf3c08, SUC_lexicon_tags_77 },
    { 0x1cef3c09, SUC_lexicon_tags_12 },
    { 0x672f3c08, SUC_lexicon_tags_27 },
    { 0x47cf3c09, SUC_lexicon_tags_13 },
    { 0xe7af3c0c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70f3c10, SUC_lexicon_tags_33 },
    { 0x5baf3c11, SUC_lexicon_tags_38 },
    { 0xa46f3c11, SUC_lexicon_tags_11 },
    { 0xf30f3c13, SUC_lexicon_tags_107 },
    { 0xa4ef3c12, SUC_lexicon_tags_31 },
    { 0xc5ef3c15, SUC_lexicon_tags_45 },
    { 0xa74f3c10, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110f3c1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4cf3c1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552f3c23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd24f3c25, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34f3c28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48f3c2c, SUC_lexicon_tags_23 },
    { 0x1cf3c2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x836f3c2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x200f3c32, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8f3c36, SUC_lexicon_tags_37 },
    { 0x282f3c36, SUC_lexicon_tags_38 },
    { 0x2a6f3c38, SUC_lexicon_tags_34 },
    { 0x70ef3c39, SUC_lexicon_tags_91 },
    { 0x292f3c38, SUC_lexicon_tags_12 },
    { 0x5f8f3c3b, SUC_lexicon_tags_10 },
    { 0xc94f3c3c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23af3c41, SUC_lexicon_tags_22 },
    { 0xb4af3c41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4f3c46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc76f3c48, SUC_lexicon_tags_2 },
    { 0xbd8f3c49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x298f3c4c, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6f3c50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3af3c56, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe1af3c58, SUC_lexicon_tags_613 },
    { 0xefef3c59, SUC_lexicon_tags_39 },
    { 0xaaef3c5a, SUC_lexicon_tags_17 },
    { 0x294f3c5a, SUC_lexicon_tags_7 },
    { 0xef0f3c5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5cf3c5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f3c62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf3c65, SUC_lexicon_tags_19 },
    { 0x566f3c66, SUC_lexicon_tags_0 },
    { 0x940f3c65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0f3c6b, SUC_lexicon_tags_45 },
    { 0xdacf3c6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaef3c6f, SUC_lexicon_tags_12 },
    { 0x52ef3c6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36f3c73, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4f3c76, SUC_lexicon_tags_55 },
    { 0x968f3c77, SUC_lexicon_tags_12 },
    { 0xe7cf3c78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xebaf3c7a, SUC_lexicon_tags_2 },
    { 0x53cf3c7b, SUC_lexicon_tags_47 },
    { 0x75ef3c7c, SUC_lexicon_tags_17 },
    { 0xce4f3c7d, SUC_lexicon_tags_27 },
    { 0xfa0f3c7d, SUC_lexicon_tags_0 },
    { 0x6aaf3c7f, SUC_lexicon_tags_2 },
    { 0xe1af3c7f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2f3c85, SUC_lexicon_tags_14 },
    { 0xf32f3c86, SUC_lexicon_tags_37 },
    { 0x76ef3c87, SUC_lexicon_tags_7 },
    { 0xfdaf3c87, SUC_lexicon_tags_4 },
    { 0xd1af3c89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0f3c8c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0f3c94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2af3c96, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708f3c9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222f3c9e, SUC_lexicon_tags_0 },
    { 0xbe0f3c9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f3ca6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x998f3ca9, SUC_lexicon_tags_11 },
    { 0x2fcf3ca9, SUC_lexicon_tags_37 },
    { 0xc8cf3ca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba0f3cad, SUC_lexicon_tags_23 },
    { 0x9dcf3cad, SUC_lexicon_tags_7 },
    { 0xc60f3caf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x800f3cb3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe3ef3cb5, SUC_lexicon_tags_37 },
    { 0xe70f3cb6, SUC_lexicon_tags_43 },
    { 0xb4af3cb5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948f3cbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eef3cc2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79af3cc6, SUC_lexicon_tags_15 },
    { 0xbd8f3cc7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe24f3cc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe34f3ccb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x880f3cd2, SUC_lexicon_tags_11 },
    { 0xb34f3cd2, SUC_lexicon_tags_1 },
    { 0xff8f3cd4, SUC_lexicon_tags_37 },
    { 0x85af3cd5, SUC_lexicon_tags_11 },
    { 0x502f3cd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cef3ce5, SUC_lexicon_tags_37 },
    { 0xf70f3ce6, SUC_lexicon_tags_12 },
    { 0x38cf3ce6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22f3cec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba8f3cef, SUC_lexicon_tags_17 },
    { 0x162f3cf0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6f3cf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x842f3cf5, SUC_lexicon_tags_11 },
    { 0xc6cf3cf6, SUC_lexicon_tags_3 },
    { 0x184f3cf6, SUC_lexicon_tags_25 },
    { 0x58f3cf8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cf3cfc, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x2ecf3cfe, SUC_lexicon_tags_7 },
    { 0x69af3cfe, SUC_lexicon_tags_27 },
    { 0xbc6f3d00, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x48af3d03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cf3d06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732f3d0b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x766f3d0e, SUC_lexicon_tags_55 },
    { 0x712f3d0f, SUC_lexicon_tags_2 },
    { 0x58ef3d10, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60f3d13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cf3d17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8f3d1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358f3d21, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xee0f3d23, SUC_lexicon_tags_54 },
    { 0xdc4f3d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe14f3d26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdd4f3d28, SUC_lexicon_tags_25 },
    { 0x7e2f3d29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316f3d2d, SUC_lexicon_tags_47 },
    { 0xb42f3d2d, SUC_lexicon_tags_35 },
    { 0x796f3d2f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8f3d33, SUC_lexicon_tags_45 },
    { 0x5b6f3d34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690f3d39, SUC_lexicon_tags_46 },
    { 0xbbcf3d3a, SUC_lexicon_tags_17 },
    { 0xce0f3d3b, SUC_lexicon_tags_37 },
    { 0x842f3d3b, SUC_lexicon_tags_37 },
    { 0xfb4f3d3b, SUC_lexicon_tags_38 },
    { 0xd6cf3d3e, SUC_lexicon_tags_27 },
    { 0x9e0f3d3f, SUC_lexicon_tags_60 },
    { 0xc5cf3d40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ef3d43, SUC_lexicon_tags_1 },
    { 0x5f0f3d44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b0f3d46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ef3d49, SUC_lexicon_tags_25 },
    { 0xd2cf3d49, SUC_lexicon_tags_1 },
    { 0x158f3d4b, SUC_lexicon_tags_11 },
    { 0x1dcf3d4b, SUC_lexicon_tags_12 },
    { 0x5c2f3d4a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5aaf3d4f, SUC_lexicon_tags_8 },
    { 0x6eaf3d50, SUC_lexicon_tags_22 },
    { 0x7baf3d50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ecf3d53, SUC_lexicon_tags_0 },
    { 0xdc0f3d53, SUC_lexicon_tags_17 },
    { 0xdc8f3d53, SUC_lexicon_tags_33 },
    { 0xacaf3d54, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa44f3d58, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x892f3d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cef3d62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cf3d65, SUC_lexicon_tags_37 },
    { 0xb2f3d66, SUC_lexicon_tags_17 },
    { 0x1fcf3d67, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4f3d6d, SUC_lexicon_tags_19 },
    { 0xefaf3d6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a4f3d70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8f3d79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x908f3d7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cf3d82, SUC_lexicon_tags_29 },
    { 0x7ccf3d82, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x3f6f3d85, SUC_lexicon_tags_37 },
    { 0x38af3d86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf18f3d89, SUC_lexicon_tags_27 },
    { 0xc08f3d8a, SUC_lexicon_tags_15 },
    { 0xc90f3d8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ef3d8e, SUC_lexicon_tags_10 },
    { 0x108f3d8f, SUC_lexicon_tags_31 },
    { 0xebaf3d90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x728f3d93, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x88af3d95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102f3d99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb68f3d9b, SUC_lexicon_tags_11 },
    { 0xf3af3d9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cf3da1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x414f3da3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc5ef3da5, SUC_lexicon_tags_15 },
    { 0x57ef3da6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe9cf3da8, SUC_lexicon_tags_25 },
    { 0x6d8f3da9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed8f3dab, SUC_lexicon_tags_17 },
    { 0x25ef3dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbbcf3dae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38f3db1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736f3db7, SUC_lexicon_tags_19 },
    { 0xecf3db8, SUC_lexicon_tags_11 },
    { 0x6f3db9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xecaf3dbb, SUC_lexicon_tags_2 },
    { 0xd8cf3dbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfbaf3dbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af3dc3, SUC_lexicon_tags_12 },
    { 0xd44f3dc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2f3dca, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ef3dcf, SUC_lexicon_tags_49 },
    { 0xdcaf3dcf, SUC_lexicon_tags_2 },
    { 0x12f3dcf, SUC_lexicon_tags_46 },
    { 0xad8f3dd2, SUC_lexicon_tags_17 },
    { 0xe64f3dd2, SUC_lexicon_tags_15 },
    { 0xd00f3dd4, SUC_lexicon_tags_25 },
    { 0x1d6f3dd5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x738f3dd7, SUC_lexicon_tags_17 },
    { 0x30af3dd8, SUC_lexicon_tags_15 },
    { 0x8b4f3dd9, SUC_lexicon_tags_6 },
    { 0x8e8f3dda, SUC_lexicon_tags_47 },
    { 0x526f3dda, SUC_lexicon_tags_216 },
    { 0x3faf3dda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5def3dde, SUC_lexicon_tags_462 },
    { 0xed6f3ddf, SUC_lexicon_tags_65 },
    { 0xec6f3de0, SUC_lexicon_tags_7 },
    { 0x12af3ddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3aef3de3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1b2f3de5, SUC_lexicon_tags_28 },
    { 0xc62f3de6, SUC_lexicon_tags_11 },
    { 0xf8ef3de7, SUC_lexicon_tags_37 },
    { 0xb76f3de7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf3ded, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xa4cf3def, SUC_lexicon_tags_17 },
    { 0x882f3def, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99af3df2, SUC_lexicon_tags_8 },
    { 0xe76f3df3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956f3df8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaf3dfb, SUC_lexicon_tags_8 },
    { 0x66f3dfc, SUC_lexicon_tags_46 },
    { 0xaa6f3dfd, SUC_lexicon_tags_15 },
    { 0x396f3dfd, SUC_lexicon_tags_4 },
    { 0xa26f3dff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xff4f3e01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470f3e09, SUC_lexicon_tags_2 },
    { 0xd7cf3e0a, SUC_lexicon_tags_29 },
    { 0x54f3e0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x434f3e0d, SUC_lexicon_tags_54 },
    { 0x436f3e0e, SUC_lexicon_tags_29 },
    { 0xfbcf3e0f, SUC_lexicon_tags_25 },
    { 0x816f3e0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f3e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0f3e1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x856f3e1d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x20af3e20, SUC_lexicon_tags_30 },
    { 0x64af3e20, SUC_lexicon_tags_10 },
    { 0x994f3e21, SUC_lexicon_tags_63 },
    { 0x2f0f3e23, SUC_lexicon_tags_1 },
    { 0xa76f3e24, SUC_lexicon_tags_45 },
    { 0x3caf3e25, SUC_lexicon_tags_17 },
    { 0xc98f3e26, SUC_lexicon_tags_10 },
    { 0xdd8f3e24, SUC_lexicon_tags_19 },
    { 0x5aef3e27, SUC_lexicon_tags_7 },
    { 0xff8f3e29, SUC_lexicon_tags_11 },
    { 0xe92f3e2a, SUC_lexicon_tags_2 },
    { 0x2c8f3e2b, SUC_lexicon_tags_49 },
    { 0x22cf3e2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858f3e30, SUC_lexicon_tags_2 },
    { 0xaaf3e30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ef3e39, SUC_lexicon_tags_68 },
    { 0x3d8f3e3a, SUC_lexicon_tags_7 },
    { 0x454f3e3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af3e3f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x296f3e42, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x680f3e45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90af3e4a, SUC_lexicon_tags_30 },
    { 0x41cf3e4b, SUC_lexicon_tags_19 },
    { 0x24ef3e4c, SUC_lexicon_tags_1 },
    { 0x474f3e4d, SUC_lexicon_tags_194 },
    { 0xe6cf3e4e, SUC_lexicon_tags_38 },
    { 0x686f3e4f, SUC_lexicon_tags_77 },
    { 0xba4f3e4f, SUC_lexicon_tags_25 },
    { 0x328f3e51, SUC_lexicon_tags_1 },
    { 0xd8f3e4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cef3e57, SUC_lexicon_tags_7 },
    { 0x366f3e57, SUC_lexicon_tags_15 },
    { 0xf40f3e59, SUC_lexicon_tags_37 },
    { 0xb2ef3e5a, SUC_lexicon_tags_37 },
    { 0x83cf3e58, SUC_lexicon_tags_63 },
    { 0xecef3e58, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aef3e5f, SUC_lexicon_tags_39 },
    { 0xfb8f3e5f, SUC_lexicon_tags_37 },
    { 0x2def3e61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x59ef3e63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x94cf3e65, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924f3e6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcf3e72, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6f3e7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16af3e81, SUC_lexicon_tags_63 },
    { 0xab0f3e82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4f3e88, SUC_lexicon_tags_12 },
    { 0xf1af3e88, SUC_lexicon_tags_79 },
    { 0xd88f3e8a, SUC_lexicon_tags_37 },
    { 0x58af3e8a, SUC_lexicon_tags_16 },
    { 0xdc0f3e8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ef3e8f, SUC_lexicon_tags_25 },
    { 0x43af3e90, SUC_lexicon_tags_25 },
    { 0xaf2f3e91, SUC_lexicon_tags_2 },
    { 0x1d0f3e91, SUC_lexicon_tags_47 },
    { 0x8c4f3e93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9af3e97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe68f3e99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x806f3e9b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cf3ea3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd0ef3ea5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6d2f3ea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36ef3ea9, SUC_lexicon_tags_7 },
    { 0x74ef3eaa, SUC_lexicon_tags_30 },
    { 0x4eaf3eab, SUC_lexicon_tags_37 },
    { 0x5def3eac, SUC_lexicon_tags_16 },
    { 0x1def3ead, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ef3eb2, SUC_lexicon_tags_2 },
    { 0xbb6f3eb3, SUC_lexicon_tags_7 },
    { 0xff4f3eb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacaf3eb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d6f3eba, SUC_lexicon_tags_49 },
    { 0x1baf3eba, SUC_lexicon_tags_11 },
    { 0x830f3ebc, SUC_lexicon_tags_7 },
    { 0x92af3ebb, SUC_lexicon_tags_25 },
    { 0x4b0f3eba, SUC_lexicon_tags_32 },
    { 0x3e6f3ebb, SUC_lexicon_tags_2 },
    { 0x33af3ec0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04f3ec3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc9cf3ec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2caf3ec7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712f3ecd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa30f3ecf, SUC_lexicon_tags_11 },
    { 0xf42f3ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36f3ed3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fef3ed6, SUC_lexicon_tags_21 },
    { 0xe1cf3ed6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62f3edc, SUC_lexicon_tags_26 },
    { 0x6f3edc, SUC_lexicon_tags_7 },
    { 0x6cf3ede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f3ee2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ef3ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfef3eed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x558f3eef, SUC_lexicon_tags_43 },
    { 0x738f3ef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cf3ef6, SUC_lexicon_tags_11 },
    { 0xf38f3ef7, SUC_lexicon_tags_30 },
    { 0x410f3ef8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8c0f3efa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22f3eff, SUC_lexicon_tags_0 },
    { 0xe54f3f00, SUC_lexicon_tags_37 },
    { 0x6c8f3f01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2f3f0d, SUC_lexicon_tags_12 },
    { 0x116f3f0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb8f3f10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x340f3f13, SUC_lexicon_tags_7 },
    { 0x500f3f14, SUC_lexicon_tags_11 },
    { 0xaef3f14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44f3f20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af3f24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0f3f29, SUC_lexicon_tags_10 },
    { 0x3e4f3f29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x472f3f2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b4f3f2e, SUC_lexicon_tags_25 },
    { 0x7eef3f2f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x782f3f31, SUC_lexicon_tags_6 },
    { 0x96af3f31, SUC_lexicon_tags_43 },
    { 0x5d0f3f33, SUC_lexicon_tags_21 },
    { 0x272f3f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7def3f37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaf3f3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefaf3f3c, SUC_lexicon_tags_0 },
    { 0x8acf3f3d, SUC_lexicon_tags_2 },
    { 0x4b2f3f3d, SUC_lexicon_tags_15 },
    { 0x7ef3f3f, SUC_lexicon_tags_27 },
    { 0xae8f3f3d, SUC_lexicon_tags_37 },
    { 0x438f3f41, SUC_lexicon_tags_43 },
    { 0x460f3f42, SUC_lexicon_tags_103 },
    { 0x18f3f3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6f3f48, SUC_lexicon_tags_43 },
    { 0xc4af3f49, SUC_lexicon_tags_15 },
    { 0xa78f3f4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe84f3f4c, SUC_lexicon_tags_30 },
    { 0xda2f3f4d, SUC_lexicon_tags_11 },
    { 0x382f3f4d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x298f3f50, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fef3f53, SUC_lexicon_tags_27 },
    { 0xdfaf3f54, SUC_lexicon_tags_9 },
    { 0xc1ef3f53, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf84f3f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50af3f5a, SUC_lexicon_tags_37 },
    { 0x408f3f5b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2f3f60, SUC_lexicon_tags_17 },
    { 0x14ef3f61, SUC_lexicon_tags_63 },
    { 0xb62f3f62, SUC_lexicon_tags_29 },
    { 0x2c0f3f61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x816f3f65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x646f3f68, SUC_lexicon_tags_11 },
    { 0x934f3f69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x576f3f6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cf3f6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x992f3f70, SUC_lexicon_tags_11 },
    { 0x600f3f71, SUC_lexicon_tags_2 },
    { 0x1faf3f72, SUC_lexicon_tags_16 },
    { 0xb22f3f71, SUC_lexicon_tags_15 },
    { 0x5aef3f71, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcf3f79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf3cf3f7b, SUC_lexicon_tags_1 },
    { 0x11ef3f7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06f3f81, SUC_lexicon_tags_38 },
    { 0x5c0f3f82, SUC_lexicon_tags_23 },
    { 0x9b0f3f83, SUC_lexicon_tags_25 },
    { 0x9acf3f84, SUC_lexicon_tags_29 },
    { 0x510f3f82, SUC_lexicon_tags_33 },
    { 0x5a0f3f85, SUC_lexicon_tags_6 },
    { 0xef4f3f87, SUC_lexicon_tags_1 },
    { 0xc26f3f88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1ef3f8a, SUC_lexicon_tags_60 },
    { 0xd30f3f8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cf3f8e, SUC_lexicon_tags_27 },
    { 0xa94f3f8f, SUC_lexicon_tags_4 },
    { 0xf32f3f90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5af3f92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250f3f96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2f3f99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x704f3f9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a4f3f9e, SUC_lexicon_tags_19 },
    { 0xc60f3f9e, SUC_lexicon_tags_11 },
    { 0x15cf3fa0, SUC_lexicon_tags_0 },
    { 0x8cef3fa1, SUC_lexicon_tags_23 },
    { 0x258f3fa2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x388f3fa5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc76f3fa7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51ef3fa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe52f3fac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134f3fb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53af3fb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9acf3fba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ef3fbe, SUC_lexicon_tags_10 },
    { 0x466f3fbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccf3fc2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x93af3fc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf3fc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4f3fcb, SUC_lexicon_tags_31 },
    { 0xfe4f3fcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89af3fd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4f3fd4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4f3fdb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788f3fdf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x474f3fe1, SUC_lexicon_tags_47 },
    { 0xd6ef3fe2, SUC_lexicon_tags_37 },
    { 0xad4f3fe3, SUC_lexicon_tags_30 },
    { 0x6b0f3fe4, SUC_lexicon_tags_65 },
    { 0x5a4f3fe5, SUC_lexicon_tags_7 },
    { 0xed2f3fe6, SUC_lexicon_tags_18 },
    { 0x8baf3fe7, SUC_lexicon_tags_7 },
    { 0x3dcf3fe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f8f3fea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x87af3fed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648f3ff2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e0f3ff4, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6f3ff8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0f3ffb, SUC_lexicon_tags_7 },
    { 0xec0f3ffb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186f4007, SUC_lexicon_tags_37 },
    { 0xd72f4008, SUC_lexicon_tags_17 },
    { 0xdc8f4009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92f400f, SUC_lexicon_tags_38 },
    { 0x770f400f, SUC_lexicon_tags_9 },
    { 0x170f4011, SUC_lexicon_tags_28 },
    { 0x982f4012, SUC_lexicon_tags_65 },
    { 0xe50f4013, SUC_lexicon_tags_17 },
    { 0x526f4014, SUC_lexicon_tags_17 },
    { 0x4caf4015, SUC_lexicon_tags_2 },
    { 0x59ef4016, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4f4019, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcf4021, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x534f4024, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x572f4026, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x768f4028, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32f402f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x7c4f4031, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602f4035, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a0f4037, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe06f4039, SUC_lexicon_tags_12 },
    { 0xce8f4039, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bef403d, SUC_lexicon_tags_17 },
    { 0xde0f403d, SUC_lexicon_tags_9 },
    { 0x88f403f, SUC_lexicon_tags_13 },
    { 0xaf403d, SUC_lexicon_tags_63 },
    { 0x480f4041, SUC_lexicon_tags_47 },
    { 0x6c6f403e, SUC_lexicon_tags_12 },
    { 0x432f403d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d2f4045, SUC_lexicon_tags_15 },
    { 0x350f4046, SUC_lexicon_tags_31 },
    { 0xe26f4047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x162f404a, SUC_lexicon_tags_64 },
    { 0x3c2f404b, SUC_lexicon_tags_37 },
    { 0x574f404b, SUC_lexicon_tags_15 },
    { 0x2d8f404d, SUC_lexicon_tags_11 },
    { 0x86f404b, SUC_lexicon_tags_4 },
    { 0x62f404e, SUC_lexicon_tags_17 },
    { 0x922f404d, SUC_lexicon_tags_13 },
    { 0xdccf404e, SUC_lexicon_tags_113 },
    { 0x8baf404e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36f4057, SUC_lexicon_tags_66 },
    { 0x31ef4058, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3acf405a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0af405d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaef4064, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2cf406b, SUC_lexicon_tags_154 },
    { 0x3bcf406c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90ef406e, SUC_lexicon_tags_17 },
    { 0x9e6f406f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf28f4071, SUC_lexicon_tags_23 },
    { 0x556f4072, SUC_lexicon_tags_2 },
    { 0x1c4f4073, SUC_lexicon_tags_17 },
    { 0xcd6f4073, SUC_lexicon_tags_25 },
    { 0x6aaf4075, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ef407c, SUC_lexicon_tags_7 },
    { 0x5f0f407d, SUC_lexicon_tags_28 },
    { 0xb7cf407e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30cf4082, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8fcf4084, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5f0f4086, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72f408a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcf4090, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92f4094, SUC_lexicon_tags_9 },
    { 0x478f4094, SUC_lexicon_tags_6 },
    { 0x616f4096, SUC_lexicon_tags_25 },
    { 0xac2f4097, SUC_lexicon_tags_17 },
    { 0xbe8f4098, SUC_lexicon_tags_15 },
    { 0xf80f4095, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9af409d, SUC_lexicon_tags_19 },
    { 0x534f409d, SUC_lexicon_tags_7 },
    { 0x1a6f409f, SUC_lexicon_tags_69 },
    { 0x636f409e, SUC_lexicon_tags_69 },
    { 0x614f40a0, SUC_lexicon_tags_17 },
    { 0x3eef40a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4f40a8, SUC_lexicon_tags_68 },
    { 0xe0f40a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10af40ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x644f40b0, SUC_lexicon_tags_19 },
    { 0x742f40b1, SUC_lexicon_tags_9 },
    { 0x2f8f40b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcf40b7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f40bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f40bf, SUC_lexicon_tags_25 },
    { 0x8b8f40c0, SUC_lexicon_tags_7 },
    { 0x3fef40c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x75ef40c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8f40c6, SUC_lexicon_tags_63 },
    { 0xc96f40c7, SUC_lexicon_tags_45 },
    { 0xce6f40c7, SUC_lexicon_tags_87 },
    { 0xe0f40c9, SUC_lexicon_tags_82 },
    { 0xf72f40c9, SUC_lexicon_tags_7 },
    { 0x5e8f40cb, SUC_lexicon_tags_27 },
    { 0xb5af40cb, SUC_lexicon_tags_17 },
    { 0xb3af40cd, SUC_lexicon_tags_65 },
    { 0x1d6f40ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa4af40d0, SUC_lexicon_tags_47 },
    { 0x410f40d1, SUC_lexicon_tags_12 },
    { 0x714f40d2, SUC_lexicon_tags_53 },
    { 0x3a0f40d3, SUC_lexicon_tags_60 },
    { 0x84cf40d3, SUC_lexicon_tags_38 },
    { 0x5b2f40d5, SUC_lexicon_tags_13 },
    { 0x6f4f40d0, SUC_lexicon_tags_30 },
    { 0xf46f40d7, SUC_lexicon_tags_82 },
    { 0xf0ef40d7, SUC_lexicon_tags_4 },
    { 0xedcf40d5, SUC_lexicon_tags_65 },
    { 0x77af40d1, SUC_lexicon_tags_37 },
    { 0xd3cf40d0, SUC_lexicon_tags_30 },
    { 0xba4f40dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbc8f40de, SUC_lexicon_tags_4 },
    { 0xa88f40de, SUC_lexicon_tags_46 },
    { 0x6c0f40e0, SUC_lexicon_tags_43 },
    { 0xae6f40e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa04f40e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ef40eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c8f40ed, SUC_lexicon_tags_19 },
    { 0xea2f40ee, SUC_lexicon_tags_15 },
    { 0x52cf40ef, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x7b6f40f1, SUC_lexicon_tags_49 },
    { 0xb16f40f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc1cf40f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd38f40f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6aef40f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0f40fb, SUC_lexicon_tags_23 },
    { 0x628f40fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4e6f40fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4f4107, SUC_lexicon_tags_47 },
    { 0x72ef4108, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x646f410a, SUC_lexicon_tags_46 },
    { 0xf80f410a, SUC_lexicon_tags_15 },
    { 0x1dcf410a, SUC_lexicon_tags_37 },
    { 0x4a6f410c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4f4113, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4f4116, SUC_lexicon_tags_38 },
    { 0xe0ef4117, SUC_lexicon_tags_12 },
    { 0xb4ef4118, SUC_lexicon_tags_74 },
    { 0xd92f4117, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73af411f, SUC_lexicon_tags_1 },
    { 0x66af4120, SUC_lexicon_tags_45 },
    { 0x5bcf411f, SUC_lexicon_tags_3 },
    { 0x2b0f4122, SUC_lexicon_tags_29 },
    { 0x56f4122, SUC_lexicon_tags_46 },
    { 0x17cf4122, SUC_lexicon_tags_11 },
    { 0x3cf4125, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f4129, SUC_lexicon_tags_2 },
    { 0xe8af412a, SUC_lexicon_tags_37 },
    { 0x81cf4129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x944f412d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0f4136, SUC_lexicon_tags_26 },
    { 0x99ef4136, SUC_lexicon_tags_1 },
    { 0x834f4137, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792f413f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd56f4143, SUC_lexicon_tags_56 },
    { 0x71ef4144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f4148, SUC_lexicon_tags_46 },
    { 0x9c2f4149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2f414f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42f4152, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14f4158, SUC_lexicon_tags_1 },
    { 0x37ef4158, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfeaf415b, SUC_lexicon_tags_30 },
    { 0x2e2f415c, SUC_lexicon_tags_0 },
    { 0x45af415d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808f4161, SUC_lexicon_tags_1 },
    { 0xc98f4162, SUC_lexicon_tags_9 },
    { 0x714f4163, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ef4166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546f416a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda6f416c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x172f416e, SUC_lexicon_tags_12 },
    { 0x2a2f416f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd82f4171, SUC_lexicon_tags_66 },
    { 0x470f4172, SUC_lexicon_tags_37 },
    { 0x4daf4171, SUC_lexicon_tags_0 },
    { 0xa22f4174, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366f4179, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0f417f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x652f4182, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80af4185, SUC_lexicon_tags_34 },
    { 0xd94f4186, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f4189, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x774f418c, SUC_lexicon_tags_0 },
    { 0x432f418d, SUC_lexicon_tags_14 },
    { 0x896f418e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f4194, SUC_lexicon_tags_15 },
    { 0x3aaf4195, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3af419d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x26f419f, SUC_lexicon_tags_23 },
    { 0x88cf41a0, SUC_lexicon_tags_30 },
    { 0xdb2f41a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2def41a5, SUC_lexicon_tags_37 },
    { 0x39ef41a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ef41ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2f41af, SUC_lexicon_tags_48 },
    { 0x36cf41b0, SUC_lexicon_tags_6 },
    { 0xe20f41b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c2f41b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f41b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda8f41b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x422f41ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa88f41bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984f41c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0f41c5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42f41cc, SUC_lexicon_tags_56 },
    { 0x956f41cd, SUC_lexicon_tags_1 },
    { 0x6af41cd, SUC_lexicon_tags_17 },
    { 0x20cf41cd, SUC_lexicon_tags_7 },
    { 0xe9cf41cf, SUC_lexicon_tags_38 },
    { 0xd82f41d1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aef41d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7bcf41d6, SUC_lexicon_tags_7 },
    { 0x88af41d6, SUC_lexicon_tags_17 },
    { 0x622f41d8, SUC_lexicon_tags_15 },
    { 0xed4f41d6, SUC_lexicon_tags_1 },
    { 0x1b6f41da, SUC_lexicon_tags_53 },
    { 0x6c2f41d8, SUC_lexicon_tags_45 },
    { 0x3eaf41d8, SUC_lexicon_tags_77 },
    { 0xf80f41dd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656f41e1, SUC_lexicon_tags_12 },
    { 0x106f41e1, SUC_lexicon_tags_21 },
    { 0x920f41e3, SUC_lexicon_tags_61 },
    { 0x644f41e3, SUC_lexicon_tags_0 },
    { 0x6fef41e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62ef41e7, SUC_lexicon_tags_2 },
    { 0x134f41e7, SUC_lexicon_tags_79 },
    { 0x918f41e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x71cf41ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6f41f3, SUC_lexicon_tags_11 },
    { 0x9aef41f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc88f41f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef0f41f9, SUC_lexicon_tags_17 },
    { 0x58af41f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ef41fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a4f41ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cf4206, SUC_lexicon_tags_15 },
    { 0x1d8f4207, SUC_lexicon_tags_3 },
    { 0x910f4207, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22f420f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ef4214, SUC_lexicon_tags_32 },
    { 0x94f4215, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb42f4217, SUC_lexicon_tags_15 },
    { 0xc04f4218, SUC_lexicon_tags_33 },
    { 0xaf8f4218, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36f421e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4ef4220, SUC_lexicon_tags_19 },
    { 0x9e2f4221, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8f4224, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58f4227, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8f422e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e4f4230, SUC_lexicon_tags_2 },
    { 0xfd4f4231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcf4237, SUC_lexicon_tags_12 },
    { 0xbf8f4238, SUC_lexicon_tags_15 },
    { 0x91cf4238, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x926f423b, SUC_lexicon_tags_19 },
    { 0xff6f423c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee0f423e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ef4244, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x852f4247, SUC_lexicon_tags_14 },
    { 0x286f4248, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0f424c, SUC_lexicon_tags_3 },
    { 0xeaf424d, SUC_lexicon_tags_30 },
    { 0x910f424e, SUC_lexicon_tags_25 },
    { 0x3bef424e, SUC_lexicon_tags_25 },
    { 0x548f424e, SUC_lexicon_tags_27 },
    { 0x4b8f424f, SUC_lexicon_tags_15 },
    { 0xe1ef424f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5daf4254, SUC_lexicon_tags_47 },
    { 0x8d6f4255, SUC_lexicon_tags_37 },
    { 0x67ef4256, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcf425b, SUC_lexicon_tags_19 },
    { 0x372f425b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x77cf425e, SUC_lexicon_tags_68 },
    { 0x910f425f, SUC_lexicon_tags_7 },
    { 0x4c4f425f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f4266, SUC_lexicon_tags_87 },
    { 0x7baf4267, SUC_lexicon_tags_2 },
    { 0xca6f4267, SUC_lexicon_tags_25 },
    { 0xab0f4269, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x724f426c, SUC_lexicon_tags_28 },
    { 0x5def426d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cf4272, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x892f4274, SUC_lexicon_tags_37 },
    { 0xab4f4275, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6f4279, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa08f427f, SUC_lexicon_tags_61 },
    { 0x99cf4280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746f4285, SUC_lexicon_tags_19 },
    { 0xaa2f4286, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624f4290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70f4295, SUC_lexicon_tags_37 },
    { 0x29cf4296, SUC_lexicon_tags_15 },
    { 0x2bcf4296, SUC_lexicon_tags_29 },
    { 0xf82f4298, SUC_lexicon_tags_49 },
    { 0x24f4299, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8f429d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x454f42a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf42a3, SUC_lexicon_tags_25 },
    { 0x67af42a3, SUC_lexicon_tags_56 },
    { 0x308f42a5, SUC_lexicon_tags_39 },
    { 0xef0f42a5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ef42b2, SUC_lexicon_tags_12 },
    { 0x9cf42b2, SUC_lexicon_tags_45 },
    { 0xef6f42b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebaf42b9, SUC_lexicon_tags_37 },
    { 0x860f42b9, SUC_lexicon_tags_31 },
    { 0x2a4f42bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64f42c1, SUC_lexicon_tags_7 },
    { 0x8d4f42c1, SUC_lexicon_tags_19 },
    { 0x208f42c3, SUC_lexicon_tags_11 },
    { 0xe02f42c4, SUC_lexicon_tags_47 },
    { 0x3b4f42c1, SUC_lexicon_tags_17 },
    { 0x2e4f42c5, SUC_lexicon_tags_12 },
    { 0xbeaf42c3, SUC_lexicon_tags_32 },
    { 0x734f42c1, SUC_lexicon_tags_27 },
    { 0x9baf42c9, SUC_lexicon_tags_72 },
    { 0xca4f42ca, SUC_lexicon_tags_1 },
    { 0x51ef42cb, SUC_lexicon_tags_2 },
    { 0xcf42cb, SUC_lexicon_tags_2 },
    { 0x898f42cd, SUC_lexicon_tags_1 },
    { 0xc50f42cc, SUC_lexicon_tags_2 },
    { 0xd66f42cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0f42da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6f42dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f42e1, SUC_lexicon_tags_1218 },
    { 0x300f42e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82f42e7, SUC_lexicon_tags_2 },
    { 0xe34f42e8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xefaf42ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42ee, SUC_lexicon_tags_15 },
    { 0xf78f42ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42f42f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x800f42f4, SUC_lexicon_tags_25 },
    { 0xf2cf42f5, SUC_lexicon_tags_29 },
    { 0xe72f42f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73ef42f8, SUC_lexicon_tags_26 },
    { 0xe80f42f8, SUC_lexicon_tags_15 },
    { 0xd80f42fa, SUC_lexicon_tags_25 },
    { 0x888f42fb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb66f42fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0f4302, SUC_lexicon_tags_15 },
    { 0x3baf4302, SUC_lexicon_tags_70 },
    { 0x91ef4302, SUC_lexicon_tags_11 },
    { 0xa62f4303, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860f430a, SUC_lexicon_tags_12 },
    { 0x58cf430b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x110f430d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x826f4310, SUC_lexicon_tags_49 },
    { 0x9def4311, SUC_lexicon_tags_11 },
    { 0x630f4312, SUC_lexicon_tags_10 },
    { 0xc24f4311, SUC_lexicon_tags_11 },
    { 0x58af4314, SUC_lexicon_tags_10 },
    { 0x5ef4314, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdf6f4317, SUC_lexicon_tags_11 },
    { 0xacef4318, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b2f431a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2f431e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4ef4320, SUC_lexicon_tags_39 },
    { 0xef8f4320, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4324, SUC_lexicon_tags_29 },
    { 0x18ef4325, SUC_lexicon_tags_11 },
    { 0xbf8f4325, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cf432b, SUC_lexicon_tags_11 },
    { 0x34ef432c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf432f, SUC_lexicon_tags_15 },
    { 0x4ef4330, SUC_lexicon_tags_30 },
    { 0x1ef4331, SUC_lexicon_tags_68 },
    { 0x7f2f4332, SUC_lexicon_tags_2 },
    { 0x5d2f4331, SUC_lexicon_tags_27 },
    { 0x94ef432f, SUC_lexicon_tags_1 },
    { 0xf3af432f, SUC_lexicon_tags_49 },
    { 0x3e0f4331, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2f4339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248f433e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbd2f4340, SUC_lexicon_tags_27 },
    { 0x6cef4341, SUC_lexicon_tags_25 },
    { 0x478f4342, SUC_lexicon_tags_28 },
    { 0x230f4343, SUC_lexicon_tags_2 },
    { 0x686f4343, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x386f4346, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc94f4348, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4f434c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd2f434e, SUC_lexicon_tags_15 },
    { 0x38f434f, SUC_lexicon_tags_54 },
    { 0x7f6f434f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c6f4352, SUC_lexicon_tags_33 },
    { 0x70ef4353, SUC_lexicon_tags_72 },
    { 0xdcaf4354, SUC_lexicon_tags_25 },
    { 0x330f4355, SUC_lexicon_tags_27 },
    { 0xabaf4356, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x90ef4358, SUC_lexicon_tags_68 },
    { 0x744f4359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf435d, SUC_lexicon_tags_37 },
    { 0x526f435e, SUC_lexicon_tags_8 },
    { 0x3e4f435f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda8f4363, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7caf4365, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb4ef4367, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0f436a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136f4371, SUC_lexicon_tags_23 },
    { 0xc86f4372, SUC_lexicon_tags_14 },
    { 0x98ef4372, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4f437d, SUC_lexicon_tags_5 },
    { 0x906f437e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6f4381, SUC_lexicon_tags_66 },
    { 0x678f4382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cf4385, SUC_lexicon_tags_14 },
    { 0x9f2f4385, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312f438d, SUC_lexicon_tags_27 },
    { 0xd8cf438d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x296f4391, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330f4395, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf96f4397, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x810f4399, SUC_lexicon_tags_28 },
    { 0x548f4399, SUC_lexicon_tags_27 },
    { 0xf96f439b, SUC_lexicon_tags_107 },
    { 0xd30f439c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x614f439e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfdaf43a0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6f4f43a2, SUC_lexicon_tags_1497 },
    { 0x3af43a3, SUC_lexicon_tags_47 },
    { 0xde8f43a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ef43ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60f43af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7cf43b1, SUC_lexicon_tags_37 },
    { 0xff2f43b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0f43b7, SUC_lexicon_tags_1 },
    { 0xe76f43b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4f43bc, SUC_lexicon_tags_30 },
    { 0x92ef43bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ef43c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92f43c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4eaf43c4, SUC_lexicon_tags_11 },
    { 0x244f43c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f43cb, SUC_lexicon_tags_7 },
    { 0x9caf43cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cf43d1, SUC_lexicon_tags_38 },
    { 0xc7ef43d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8f43d5, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f43da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x800f43dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f43e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x760f43e4, SUC_lexicon_tags_7 },
    { 0x684f43e5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4f43e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4f43f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14f43f4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x7daf43f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x30af43f9, SUC_lexicon_tags_21 },
    { 0x4fef43fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x254f43fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcf4400, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cf4405, SUC_lexicon_tags_7 },
    { 0xbef4406, SUC_lexicon_tags_15 },
    { 0xa14f4406, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cf440b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426f4413, SUC_lexicon_tags_47 },
    { 0x4e2f4414, SUC_lexicon_tags_26 },
    { 0xc36f4413, SUC_lexicon_tags_37 },
    { 0x80f4413, SUC_lexicon_tags_14 },
    { 0xa54f4417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x168f441a, SUC_lexicon_tags_48 },
    { 0xe7ef441b, SUC_lexicon_tags_12 },
    { 0x6d6f441c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f4421, SUC_lexicon_tags_7 },
    { 0xf16f4422, SUC_lexicon_tags_2 },
    { 0x598f4422, SUC_lexicon_tags_68 },
    { 0xb5cf4422, SUC_lexicon_tags_18 },
    { 0x7af4423, SUC_lexicon_tags_77 },
    { 0xd72f4426, SUC_lexicon_tags_14 },
    { 0xff6f4427, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96f442c, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14f4435, SUC_lexicon_tags_20 },
    { 0xd98f4436, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf72f443a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46af443c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124f4440, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcf4443, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f4448, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x190f444b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x604f4450, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6f4453, SUC_lexicon_tags_27 },
    { 0xe84f4454, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x48ef4456, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ef4459, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x526f445b, SUC_lexicon_tags_11 },
    { 0xda6f445c, SUC_lexicon_tags_26 },
    { 0x6e8f445d, SUC_lexicon_tags_28 },
    { 0x922f445d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474f4464, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae4f4466, SUC_lexicon_tags_25 },
    { 0xeeef4466, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74af446b, SUC_lexicon_tags_10 },
    { 0xbe2f446c, SUC_lexicon_tags_25 },
    { 0x2c0f446d, SUC_lexicon_tags_95 },
    { 0xc00f446c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b6f4470, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2f4476, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4f4479, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x900f447b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf447f, SUC_lexicon_tags_124 },
    { 0x826f4480, SUC_lexicon_tags_17 },
    { 0xf16f4481, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x834f4484, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf4487, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa86f4490, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x5e8f4492, SUC_lexicon_tags_0 },
    { 0x41ef4492, SUC_lexicon_tags_434 },
    { 0x2af4493, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x670f4497, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x432f4499, SUC_lexicon_tags_65 },
    { 0xbfaf449a, SUC_lexicon_tags_64 },
    { 0xbd4f449b, SUC_lexicon_tags_9 },
    { 0xfb8f449c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cf449f, SUC_lexicon_tags_28 },
    { 0x5a4f44a0, SUC_lexicon_tags_1 },
    { 0xa50f44a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xebaf44a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x424f44a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x734f44a8, SUC_lexicon_tags_27 },
    { 0x3eef44a9, SUC_lexicon_tags_25 },
    { 0x70ef44a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0f44b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb46f44b4, SUC_lexicon_tags_11 },
    { 0xaeaf44b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2f44b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0f44ba, SUC_lexicon_tags_14 },
    { 0x63af44ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaf44be, SUC_lexicon_tags_11 },
    { 0x5f4f44bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe0f44c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00f44c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfbaf44c7, SUC_lexicon_tags_14 },
    { 0x508f44c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x51af44cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x558f44cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x79ef44cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccf44d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x554f44d4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf26f44d6, SUC_lexicon_tags_1 },
    { 0x246f44d6, SUC_lexicon_tags_15 },
    { 0x5a8f44d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x168f44db, SUC_lexicon_tags_25 },
    { 0xd26f44dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cf44df, SUC_lexicon_tags_29 },
    { 0x3ef44e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x582f44e3, SUC_lexicon_tags_29 },
    { 0x9e6f44e4, SUC_lexicon_tags_28 },
    { 0xab4f44e5, SUC_lexicon_tags_16 },
    { 0x6d6f44e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40cf44e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x778f44eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x246f44ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x456f44f1, SUC_lexicon_tags_37 },
    { 0x9f6f44f1, SUC_lexicon_tags_17 },
    { 0x998f44f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936f44f9, SUC_lexicon_tags_4 },
    { 0x532f44fa, SUC_lexicon_tags_1 },
    { 0xbf4f44fa, SUC_lexicon_tags_2 },
    { 0x392f44fb, SUC_lexicon_tags_17 },
    { 0xf40f44fd, SUC_lexicon_tags_39 },
    { 0xbe8f44fe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x45af4500, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x28cf4502, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6f4505, SUC_lexicon_tags_59 },
    { 0x5f2f4505, SUC_lexicon_tags_26 },
    { 0xd00f4507, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xacaf450a, SUC_lexicon_tags_15 },
    { 0xe3ef450a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9bcf450d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ef4511, SUC_lexicon_tags_12 },
    { 0xecf4512, SUC_lexicon_tags_6 },
    { 0x9aaf4512, SUC_lexicon_tags_8 },
    { 0x552f4513, SUC_lexicon_tags_11 },
    { 0xeb4f4513, SUC_lexicon_tags_47 },
    { 0x8a0f4516, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6f4519, SUC_lexicon_tags_151 },
    { 0x29ef451a, SUC_lexicon_tags_43 },
    { 0xcacf451b, SUC_lexicon_tags_15 },
    { 0xd94f451a, SUC_lexicon_tags_11 },
    { 0xfccf4519, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc96f451f, SUC_lexicon_tags_28 },
    { 0x566f4520, SUC_lexicon_tags_14 },
    { 0x402f4520, SUC_lexicon_tags_25 },
    { 0x2def4521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb0f4524, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdfcf4526, SUC_lexicon_tags_45 },
    { 0xb40f4527, SUC_lexicon_tags_11 },
    { 0x770f4527, SUC_lexicon_tags_48 },
    { 0xddef4529, SUC_lexicon_tags_2 },
    { 0x5f0f452a, SUC_lexicon_tags_9 },
    { 0x17cf4529, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba2f4530, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf4535, SUC_lexicon_tags_37 },
    { 0x576f4536, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ef453a, SUC_lexicon_tags_11 },
    { 0x8cf453b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe30f453d, SUC_lexicon_tags_397 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23af4541, SUC_lexicon_tags_30 },
    { 0xeeaf4542, SUC_lexicon_tags_811 },
    { 0x2c8f4543, SUC_lexicon_tags_11 },
    { 0x24f4544, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8ef4546, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236f454c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72f4551, SUC_lexicon_tags_39 },
    { 0x234f4552, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874f4556, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x944f4559, SUC_lexicon_tags_0 },
    { 0x38ef455a, SUC_lexicon_tags_45 },
    { 0x442f455b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x650f455e, SUC_lexicon_tags_25 },
    { 0x542f455f, SUC_lexicon_tags_11 },
    { 0x6c4f4560, SUC_lexicon_tags_26 },
    { 0x58f4561, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8f4564, SUC_lexicon_tags_475 },
    { 0x346f4565, SUC_lexicon_tags_61 },
    { 0xea8f4566, SUC_lexicon_tags_25 },
    { 0x9d6f4566, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4f456b, SUC_lexicon_tags_2 },
    { 0x71cf456b, SUC_lexicon_tags_45 },
    { 0x476f456d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf32f456f, SUC_lexicon_tags_17 },
    { 0x9d8f4570, SUC_lexicon_tags_1 },
    { 0xf54f4570, SUC_lexicon_tags_25 },
    { 0xe88f4571, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf48f4574, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaf457b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeaaf457d, SUC_lexicon_tags_60 },
    { 0x412f457e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f4582, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd7ef4584, SUC_lexicon_tags_27 },
    { 0xd86f4585, SUC_lexicon_tags_15 },
    { 0xa56f4586, SUC_lexicon_tags_0 },
    { 0xf66f4587, SUC_lexicon_tags_8 },
    { 0xa1af4588, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8f458b, SUC_lexicon_tags_25 },
    { 0x276f458c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596f4590, SUC_lexicon_tags_21 },
    { 0x8eaf4591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb84f4593, SUC_lexicon_tags_37 },
    { 0x574f4593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52f4597, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f459a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0f459e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ef45a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4f45a8, SUC_lexicon_tags_15 },
    { 0x844f45a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482f45b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cf45b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb46f45bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9faf45c1, SUC_lexicon_tags_63 },
    { 0x40af45c2, SUC_lexicon_tags_61 },
    { 0x332f45c3, SUC_lexicon_tags_11 },
    { 0x332f45c4, SUC_lexicon_tags_9 },
    { 0xf90f45c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53af45c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32af45cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66f45d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ccf45d3, SUC_lexicon_tags_1 },
    { 0xde6f45d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4f45e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2faf45e8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcaf45ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370f45f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866f45f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc0af45f9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbef4600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cf4605, SUC_lexicon_tags_9 },
    { 0x86ef4605, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab4f4608, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3caf460a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cf460d, SUC_lexicon_tags_45 },
    { 0xdcef460e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88f4613, SUC_lexicon_tags_32 },
    { 0x432f4613, SUC_lexicon_tags_17 },
    { 0x612f4615, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x310f4617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13cf4619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56f461c, SUC_lexicon_tags_30 },
    { 0x19cf461d, SUC_lexicon_tags_11 },
    { 0x960f461d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x662f4621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9af4623, SUC_lexicon_tags_8 },
    { 0xd24f4624, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6f4627, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3af462b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646f4630, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa38f4632, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79cf4635, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x410f4637, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x138f463a, SUC_lexicon_tags_12 },
    { 0x1ef463b, SUC_lexicon_tags_37 },
    { 0xe9ef463c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2f4641, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x236f4643, SUC_lexicon_tags_29 },
    { 0xd6cf4644, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b8f4646, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ef464b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf4651, SUC_lexicon_tags_1 },
    { 0x2f4652, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x306f4655, SUC_lexicon_tags_1 },
    { 0xef8f4656, SUC_lexicon_tags_45 },
    { 0xc04f4657, SUC_lexicon_tags_9 },
    { 0xb56f4658, SUC_lexicon_tags_15 },
    { 0x5baf4659, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x96cf465c, SUC_lexicon_tags_12 },
    { 0x26af465d, SUC_lexicon_tags_0 },
    { 0x950f465c, SUC_lexicon_tags_65 },
    { 0x812f465d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xedaf4661, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf82f4663, SUC_lexicon_tags_46 },
    { 0x16cf4664, SUC_lexicon_tags_29 },
    { 0x7a6f4664, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2f4668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf92f466a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb70f466c, SUC_lexicon_tags_43 },
    { 0xab6f466d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x752f466f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef4672, SUC_lexicon_tags_23 },
    { 0x42ef4672, SUC_lexicon_tags_25 },
    { 0x9a2f4673, SUC_lexicon_tags_14 },
    { 0x506f4674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1af4677, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x298f4679, SUC_lexicon_tags_15 },
    { 0xec0f467a, SUC_lexicon_tags_31 },
    { 0x9d4f467a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08f4681, SUC_lexicon_tags_19 },
    { 0x5aaf4682, SUC_lexicon_tags_37 },
    { 0x6d8f4683, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cf468a, SUC_lexicon_tags_15 },
    { 0x8cf468b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c6f4694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xefaf4697, SUC_lexicon_tags_25 },
    { 0x6def4698, SUC_lexicon_tags_48 },
    { 0xaeef4698, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90f46a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ef46a9, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0xe8f46ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0f46b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a6f46b6, SUC_lexicon_tags_11 },
    { 0x600f46b7, SUC_lexicon_tags_45 },
    { 0x9f8f46b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f46bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5ef46bd, SUC_lexicon_tags_33 },
    { 0x302f46bd, SUC_lexicon_tags_3 },
    { 0xd96f46bf, SUC_lexicon_tags_47 },
    { 0x868f46be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x342f46c2, SUC_lexicon_tags_34 },
    { 0xcdcf46c2, SUC_lexicon_tags_15 },
    { 0x7a2f46c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88f46c9, SUC_lexicon_tags_25 },
    { 0x756f46c9, SUC_lexicon_tags_47 },
    { 0xfb6f46ca, SUC_lexicon_tags_13 },
    { 0xb4af46cc, SUC_lexicon_tags_21 },
    { 0xecaf46c9, SUC_lexicon_tags_31 },
    { 0x4e0f46c9, SUC_lexicon_tags_37 },
    { 0xeb6f46ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98f46d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ef46da, SUC_lexicon_tags_11 },
    { 0xf88f46da, SUC_lexicon_tags_68 },
    { 0x4f46dc, SUC_lexicon_tags_7 },
    { 0xf82f46dd, SUC_lexicon_tags_7 },
    { 0x188f46de, SUC_lexicon_tags_29 },
    { 0x5ef46dd, SUC_lexicon_tags_65 },
    { 0x116f46e0, SUC_lexicon_tags_23 },
    { 0xf16f46e0, SUC_lexicon_tags_87 },
    { 0xe0ef46e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x756f46e4, SUC_lexicon_tags_2 },
    { 0x33af46e5, SUC_lexicon_tags_19 },
    { 0x76af46e5, SUC_lexicon_tags_1 },
    { 0x822f46e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ef46eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x898f46ed, SUC_lexicon_tags_33 },
    { 0x62cf46ed, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0f46f6, SUC_lexicon_tags_23 },
    { 0x8caf46f6, SUC_lexicon_tags_25 },
    { 0x40f46f8, SUC_lexicon_tags_14 },
    { 0x1f4f46f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568f46fe, SUC_lexicon_tags_38 },
    { 0xa5ef46fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93cf4707, SUC_lexicon_tags_49 },
    { 0x9b0f4708, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7f0f470a, SUC_lexicon_tags_17 },
    { 0xdc8f470a, SUC_lexicon_tags_0 },
    { 0x2dcf470b, SUC_lexicon_tags_17 },
    { 0x712f470b, SUC_lexicon_tags_62 },
    { 0xf32f470e, SUC_lexicon_tags_25 },
    { 0x46cf470f, SUC_lexicon_tags_69 },
    { 0x96ef470c, SUC_lexicon_tags_70 },
    { 0x808f4711, SUC_lexicon_tags_12 },
    { 0xecf4712, SUC_lexicon_tags_15 },
    { 0x84af470f, SUC_lexicon_tags_43 },
    { 0x1d4f4714, SUC_lexicon_tags_37 },
    { 0xbcf4715, SUC_lexicon_tags_31 },
    { 0xf8cf4711, SUC_lexicon_tags_9 },
    { 0x768f4717, SUC_lexicon_tags_48 },
    { 0xa0cf4718, SUC_lexicon_tags_11 },
    { 0x82cf4719, SUC_lexicon_tags_1 },
    { 0x7d4f471a, SUC_lexicon_tags_17 },
    { 0xaf4717, SUC_lexicon_tags_65 },
    { 0xd5ef471c, SUC_lexicon_tags_65 },
    { 0x138f471d, SUC_lexicon_tags_8 },
    { 0x722f471e, SUC_lexicon_tags_17 },
    { 0x450f471c, SUC_lexicon_tags_12 },
    { 0x91cf471d, SUC_lexicon_tags_19 },
    { 0x142f4721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe60f4723, SUC_lexicon_tags_17 },
    { 0xd52f4723, SUC_lexicon_tags_12 },
    { 0x7d8f4725, SUC_lexicon_tags_11 },
    { 0x4d4f4725, SUC_lexicon_tags_1 },
    { 0x1b2f4726, SUC_lexicon_tags_43 },
    { 0xccf4724, SUC_lexicon_tags_23 },
    { 0xbe8f4727, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c2f472b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x192f472d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x288f472f, SUC_lexicon_tags_37 },
    { 0x2f8f4730, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cf4735, SUC_lexicon_tags_0 },
    { 0xc0cf4736, SUC_lexicon_tags_21 },
    { 0x342f4736, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ef473b, SUC_lexicon_tags_37 },
    { 0x278f473c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06f473f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cf4744, SUC_lexicon_tags_19 },
    { 0x79cf4744, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x14cf4747, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x576f4749, SUC_lexicon_tags_18 },
    { 0xad4f4749, SUC_lexicon_tags_15 },
    { 0x78af474b, SUC_lexicon_tags_11 },
    { 0xd64f474c, SUC_lexicon_tags_29 },
    { 0xae0f474c, SUC_lexicon_tags_14 },
    { 0x722f474d, SUC_lexicon_tags_1 },
    { 0x66cf474d, SUC_lexicon_tags_63 },
    { 0x58ef474d, SUC_lexicon_tags_48 },
    { 0x9e6f474e, SUC_lexicon_tags_13 },
    { 0xd14f4752, SUC_lexicon_tags_37 },
    { 0xbd8f4750, SUC_lexicon_tags_1 },
    { 0x786f4754, SUC_lexicon_tags_17 },
    { 0x67cf4755, SUC_lexicon_tags_38 },
    { 0xbdef4756, SUC_lexicon_tags_7 },
    { 0xb52f4757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5def475f, SUC_lexicon_tags_7 },
    { 0x79af4760, SUC_lexicon_tags_28 },
    { 0x27cf4760, SUC_lexicon_tags_37 },
    { 0x438f4762, SUC_lexicon_tags_80 },
    { 0xd0f4763, SUC_lexicon_tags_15 },
    { 0xce2f4763, SUC_lexicon_tags_45 },
    { 0x2c0f4765, SUC_lexicon_tags_19 },
    { 0xd5ef4762, SUC_lexicon_tags_8 },
    { 0xf9ef4764, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf476a, SUC_lexicon_tags_63 },
    { 0x74ef476b, SUC_lexicon_tags_12 },
    { 0x86ef476b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348f4771, SUC_lexicon_tags_12 },
    { 0xa52f4771, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53af4774, SUC_lexicon_tags_30 },
    { 0xa0f4775, SUC_lexicon_tags_7 },
    { 0x8e2f4776, SUC_lexicon_tags_66 },
    { 0x5c8f4774, SUC_lexicon_tags_10 },
    { 0x7b2f4778, SUC_lexicon_tags_23 },
    { 0x672f4778, SUC_lexicon_tags_37 },
    { 0xfbaf477a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dcf477c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930f4781, SUC_lexicon_tags_11 },
    { 0x3c8f4782, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cf4785, SUC_lexicon_tags_12 },
    { 0xf2cf4786, SUC_lexicon_tags_2 },
    { 0x70ef4787, SUC_lexicon_tags_18 },
    { 0xbef4787, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6f478b, SUC_lexicon_tags_17 },
    { 0x7e0f478c, SUC_lexicon_tags_27 },
    { 0x1e0f478d, SUC_lexicon_tags_0 },
    { 0x310f478b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48f4797, SUC_lexicon_tags_15 },
    { 0xa16f4798, SUC_lexicon_tags_2 },
    { 0xe1ef4798, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x96ef479c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8f479f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb98f47a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830f47a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd38f47a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cf47ad, SUC_lexicon_tags_5 },
    { 0x34f47ad, SUC_lexicon_tags_11 },
    { 0x3f6f47af, SUC_lexicon_tags_47 },
    { 0x5c0f47b0, SUC_lexicon_tags_14 },
    { 0xcc0f47af, SUC_lexicon_tags_7 },
    { 0xc30f47b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6f47b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd2ef47b7, SUC_lexicon_tags_16 },
    { 0x264f47b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af47c0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8e0f47c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x62af47c5, SUC_lexicon_tags_12 },
    { 0x2bef47c6, SUC_lexicon_tags_17 },
    { 0xdb0f47c6, SUC_lexicon_tags_37 },
    { 0x670f47c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6aaf47ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2f47ce, SUC_lexicon_tags_25 },
    { 0xed6f47cf, SUC_lexicon_tags_25 },
    { 0x6b8f47cf, SUC_lexicon_tags_37 },
    { 0x226f47d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31cf47d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcaf47d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x21af47d7, SUC_lexicon_tags_25 },
    { 0x502f47d7, SUC_lexicon_tags_63 },
    { 0x55af47d7, SUC_lexicon_tags_28 },
    { 0x35cf47da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714f47df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4f47e3, SUC_lexicon_tags_2 },
    { 0x7d6f47e4, SUC_lexicon_tags_63 },
    { 0xa18f47e5, SUC_lexicon_tags_320 },
    { 0x73ef47e5, SUC_lexicon_tags_14 },
    { 0x934f47e6, SUC_lexicon_tags_23 },
    { 0x710f47e8, SUC_lexicon_tags_15 },
    { 0xdd8f47e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78f47f1, SUC_lexicon_tags_31 },
    { 0x6b8f47f2, SUC_lexicon_tags_84 },
    { 0x178f47f2, SUC_lexicon_tags_1 },
    { 0x2b0f47f4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2fef47f6, SUC_lexicon_tags_165 },
    { 0xf90f47f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0f47fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aef47fd, SUC_lexicon_tags_11 },
    { 0x540f47fe, SUC_lexicon_tags_60 },
    { 0x7cf47ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982f4803, SUC_lexicon_tags_25 },
    { 0x704f4804, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x694f4807, SUC_lexicon_tags_23 },
    { 0xfb8f4808, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cf480d, SUC_lexicon_tags_48 },
    { 0xd44f480e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf4812, SUC_lexicon_tags_1 },
    { 0xb52f4813, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732f4817, SUC_lexicon_tags_2 },
    { 0x8bef4818, SUC_lexicon_tags_25 },
    { 0xadef4818, SUC_lexicon_tags_2 },
    { 0x144f4819, SUC_lexicon_tags_3 },
    { 0xee2f481b, SUC_lexicon_tags_14 },
    { 0xb56f481b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cf4822, SUC_lexicon_tags_4 },
    { 0x8daf4822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4f4827, SUC_lexicon_tags_27 },
    { 0x242f4828, SUC_lexicon_tags_1612 },
    { 0x602f4829, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acf4830, SUC_lexicon_tags_17 },
    { 0x7b0f4831, SUC_lexicon_tags_12 },
    { 0x60af4831, SUC_lexicon_tags_7 },
    { 0x486f4833, SUC_lexicon_tags_31 },
    { 0x296f4832, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742f4838, SUC_lexicon_tags_3 },
    { 0x4b8f4839, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190f483f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x258f4841, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816f4849, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb72f484b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54cf484d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabef4851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb48f4853, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x754f4855, SUC_lexicon_tags_17 },
    { 0x810f4856, SUC_lexicon_tags_79 },
    { 0xaf2f4856, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x846f485a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6af485d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5caf4865, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8af4867, SUC_lexicon_tags_7 },
    { 0x55cf4867, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cf4871, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548f4876, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700f487e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ef4884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f4887, SUC_lexicon_tags_45 },
    { 0xb7cf4888, SUC_lexicon_tags_25 },
    { 0xb34f4889, SUC_lexicon_tags_6 },
    { 0xf48f488a, SUC_lexicon_tags_1 },
    { 0xfd6f488a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6faf488d, SUC_lexicon_tags_27 },
    { 0x602f488e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30cf4890, SUC_lexicon_tags_17 },
    { 0xefef4891, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ef4895, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502f4899, SUC_lexicon_tags_21 },
    { 0xe06f489a, SUC_lexicon_tags_47 },
    { 0xebef489a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x89ef489d, SUC_lexicon_tags_62 },
    { 0xa4cf489e, SUC_lexicon_tags_16 },
    { 0xbf6f489f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x31cf48a1, SUC_lexicon_tags_15 },
    { 0xb0ef48a2, SUC_lexicon_tags_11 },
    { 0xabef48a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf48a6, SUC_lexicon_tags_14 },
    { 0xea2f48a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8f48a9, SUC_lexicon_tags_38 },
    { 0xbdcf48aa, SUC_lexicon_tags_1 },
    { 0xc0af48ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0f48af, SUC_lexicon_tags_30 },
    { 0xaf2f48af, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4f48b7, SUC_lexicon_tags_17 },
    { 0xccef48b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd76f48ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4f48bd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaf48c5, SUC_lexicon_tags_17 },
    { 0xdd4f48c5, SUC_lexicon_tags_26 },
    { 0xc38f48c7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x930f48c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf48cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6f48d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6f48da, SUC_lexicon_tags_35 },
    { 0x5b4f48db, SUC_lexicon_tags_38 },
    { 0xdc6f48da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc6ef48de, SUC_lexicon_tags_140 },
    { 0x582f48df, SUC_lexicon_tags_0 },
    { 0x40f48df, SUC_lexicon_tags_12 },
    { 0xd36f48e1, SUC_lexicon_tags_1 },
    { 0xf04f48e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354f48e6, SUC_lexicon_tags_49 },
    { 0x5def48e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8f48ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x576f48f0, SUC_lexicon_tags_25 },
    { 0xf80f48f1, SUC_lexicon_tags_12 },
    { 0x8c2f48f2, SUC_lexicon_tags_15 },
    { 0xfb6f48f3, SUC_lexicon_tags_29 },
    { 0xa1cf48f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x738f48f7, SUC_lexicon_tags_19 },
    { 0xffaf48f7, SUC_lexicon_tags_14 },
    { 0x65cf48f8, SUC_lexicon_tags_10 },
    { 0x8d4f48f8, SUC_lexicon_tags_38 },
    { 0x700f48fb, SUC_lexicon_tags_11 },
    { 0xc28f48fb, SUC_lexicon_tags_2 },
    { 0x7a8f48fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadef4901, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6f4906, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2f490c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a6f490e, SUC_lexicon_tags_19 },
    { 0x84af490e, SUC_lexicon_tags_17 },
    { 0xfb6f490f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912f4916, SUC_lexicon_tags_29 },
    { 0x698f4917, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732f491c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44cf491e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b8f4920, SUC_lexicon_tags_1 },
    { 0x5caf4921, SUC_lexicon_tags_17 },
    { 0xb32f4921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18f4925, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70f492a, SUC_lexicon_tags_47 },
    { 0xb38f492a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f492e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572f4933, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xb58f4936, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4f4939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cef4940, SUC_lexicon_tags_26 },
    { 0x22ef4941, SUC_lexicon_tags_92 },
    { 0xaa4f4942, SUC_lexicon_tags_37 },
    { 0xab4f4941, SUC_lexicon_tags_6 },
    { 0xa60f4943, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a2f4946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87cf4948, SUC_lexicon_tags_60 },
    { 0xb3ef4948, SUC_lexicon_tags_27 },
    { 0x32f4949, SUC_lexicon_tags_12 },
    { 0x784f494b, SUC_lexicon_tags_25 },
    { 0x532f494a, SUC_lexicon_tags_1393 },
    { 0x4bef494d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x656f4950, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f2f4952, SUC_lexicon_tags_25 },
    { 0x9b8f4953, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x86af4955, SUC_lexicon_tags_45 },
    { 0x194f4956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac6f495d, SUC_lexicon_tags_37 },
    { 0x582f495e, SUC_lexicon_tags_27 },
    { 0xd5ef495f, SUC_lexicon_tags_87 },
    { 0xcbcf495e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38f4966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122f496c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0f4975, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0f4978, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f8f497a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f497d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910f4985, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c0f4987, SUC_lexicon_tags_11 },
    { 0xe82f4987, SUC_lexicon_tags_28 },
    { 0x59af4988, SUC_lexicon_tags_15 },
    { 0x1f0f498a, SUC_lexicon_tags_11 },
    { 0xdc4f4987, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x358f498d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6f4996, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x81af4999, SUC_lexicon_tags_26 },
    { 0xa32f499a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0af499d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2f49a0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416f49a4, SUC_lexicon_tags_49 },
    { 0x6ccf49a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec8f49a7, SUC_lexicon_tags_12 },
    { 0x822f49a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99ef49aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842f49b0, SUC_lexicon_tags_11 },
    { 0xb4cf49b1, SUC_lexicon_tags_17 },
    { 0xef0f49b2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4f49ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf50f49bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52ef49be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecf49c1, SUC_lexicon_tags_12 },
    { 0x608f49c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28f49de, SUC_lexicon_tags_204 },
    { 0xe0ef49de, SUC_lexicon_tags_17 },
    { 0xd96f49df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x924f49e2, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2f49e7, SUC_lexicon_tags_25 },
    { 0xef2f49e8, SUC_lexicon_tags_12 },
    { 0xf54f49e9, SUC_lexicon_tags_27 },
    { 0xd28f49e9, SUC_lexicon_tags_11 },
    { 0xdf4f49eb, SUC_lexicon_tags_19 },
    { 0x10ef49ec, SUC_lexicon_tags_29 },
    { 0xfcf49e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x100f49ef, SUC_lexicon_tags_17 },
    { 0x142f49ef, SUC_lexicon_tags_30 },
    { 0x3b4f49f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e2f49f3, SUC_lexicon_tags_33 },
    { 0xe2cf49f4, SUC_lexicon_tags_27 },
    { 0x612f49f5, SUC_lexicon_tags_14 },
    { 0x38f49f4, SUC_lexicon_tags_2 },
    { 0x7c0f49f7, SUC_lexicon_tags_17 },
    { 0xee0f49f8, SUC_lexicon_tags_29 },
    { 0xeaf49f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64ef49fb, SUC_lexicon_tags_55 },
    { 0x360f49fb, SUC_lexicon_tags_1 },
    { 0xa2ef49fd, SUC_lexicon_tags_17 },
    { 0x494f49fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ef4a02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f4f4a04, SUC_lexicon_tags_2 },
    { 0xe60f4a05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50f4a08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684f4a0c, SUC_lexicon_tags_12 },
    { 0x31ef4a0d, SUC_lexicon_tags_107 },
    { 0x9b4f4a0d, SUC_lexicon_tags_17 },
    { 0x876f4a0d, SUC_lexicon_tags_11 },
    { 0x5bef4a0f, SUC_lexicon_tags_15 },
    { 0xdb0f4a0f, SUC_lexicon_tags_17 },
    { 0x55ef4a12, SUC_lexicon_tags_37 },
    { 0x980f4a0d, SUC_lexicon_tags_47 },
    { 0x8daf4a0e, SUC_lexicon_tags_60 },
    { 0x5f2f4a15, SUC_lexicon_tags_63 },
    { 0x17ef4a16, SUC_lexicon_tags_14 },
    { 0x22f4a16, SUC_lexicon_tags_0 },
    { 0x4c4f4a18, SUC_lexicon_tags_56 },
    { 0x20cf4a19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x264f4a1b, SUC_lexicon_tags_19 },
    { 0x624f4a1b, SUC_lexicon_tags_13 },
    { 0x496f4a1d, SUC_lexicon_tags_15 },
    { 0xb14f4a1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f8f4a20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb98f4a22, SUC_lexicon_tags_25 },
    { 0xe86f4a23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fef4a29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x24af4a2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ef4a2e, SUC_lexicon_tags_21 },
    { 0xf22f4a2e, SUC_lexicon_tags_27 },
    { 0x2dcf4a30, SUC_lexicon_tags_25 },
    { 0x248f4a30, SUC_lexicon_tags_15 },
    { 0x500f4a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554f4a38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb6f4a3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972f4a3e, SUC_lexicon_tags_7 },
    { 0x2a4f4a3e, SUC_lexicon_tags_17 },
    { 0xbeef4a3f, SUC_lexicon_tags_30 },
    { 0xcd0f4a3f, SUC_lexicon_tags_15 },
    { 0xd0f4a3f, SUC_lexicon_tags_25 },
    { 0x1d6f4a43, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3c0f4a45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x738f4a48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4faf4a4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaf4a4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40ef4a51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd4f4a53, SUC_lexicon_tags_37 },
    { 0x6f8f4a54, SUC_lexicon_tags_12 },
    { 0x9cef4a55, SUC_lexicon_tags_17 },
    { 0x6c2f4a55, SUC_lexicon_tags_15 },
    { 0x9b6f4a57, SUC_lexicon_tags_37 },
    { 0x146f4a58, SUC_lexicon_tags_17 },
    { 0x8f4f4a59, SUC_lexicon_tags_16 },
    { 0x116f4a58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48f4a5d, SUC_lexicon_tags_12 },
    { 0x4d4f4a5d, SUC_lexicon_tags_55 },
    { 0x670f4a5f, SUC_lexicon_tags_11 },
    { 0x35af4a5f, SUC_lexicon_tags_27 },
    { 0x446f4a5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa16f4a63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf4a6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0f4a6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb90f4a71, SUC_lexicon_tags_2 },
    { 0x5dcf4a72, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x574f4a74, SUC_lexicon_tags_16 },
    { 0xc96f4a75, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ef4a79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34ef4a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ef4a7e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4f4a82, SUC_lexicon_tags_31 },
    { 0x962f4a83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250f4a8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf4a8f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3a4f4a91, SUC_lexicon_tags_1 },
    { 0xe3cf4a91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x624f4a94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4f4a99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46f4a9d, SUC_lexicon_tags_160 },
    { 0xb68f4a9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x298f4aa1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eaf4aa6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6f4aa9, SUC_lexicon_tags_40 },
    { 0x398f4aaa, SUC_lexicon_tags_39 },
    { 0x88ef4aa9, SUC_lexicon_tags_82 },
    { 0x21ef4aac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a2f4aae, SUC_lexicon_tags_23 },
    { 0x30ef4aaf, SUC_lexicon_tags_25 },
    { 0xfd6f4aae, SUC_lexicon_tags_64 },
    { 0x494f4aae, SUC_lexicon_tags_2 },
    { 0xa04f4ab2, SUC_lexicon_tags_12 },
    { 0x604f4aaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12f4ab6, SUC_lexicon_tags_250 },
    { 0x480f4ab7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5d4f4ab9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f4f4abb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50f4abf, SUC_lexicon_tags_25 },
    { 0x1e2f4ac0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccf4ac3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56af4ad0, SUC_lexicon_tags_25 },
    { 0x270f4ad1, SUC_lexicon_tags_8 },
    { 0xe30f4ad2, SUC_lexicon_tags_60 },
    { 0xc9ef4ad3, SUC_lexicon_tags_15 },
    { 0x280f4ad4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18f4ada, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeacf4ade, SUC_lexicon_tags_25 },
    { 0x4ecf4adf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x554f4ae1, SUC_lexicon_tags_17 },
    { 0x734f4ae2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc60f4aeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaf4aef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ef4af2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258f4af6, SUC_lexicon_tags_1 },
    { 0x5baf4af7, SUC_lexicon_tags_7 },
    { 0x554f4af7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4f4afd, SUC_lexicon_tags_5 },
    { 0x4f0f4afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554f4b05, SUC_lexicon_tags_2 },
    { 0xbbcf4b05, SUC_lexicon_tags_47 },
    { 0xa32f4b07, SUC_lexicon_tags_25 },
    { 0xd74f4b08, SUC_lexicon_tags_25 },
    { 0x49cf4b09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2f4b0c, SUC_lexicon_tags_56 },
    { 0x77af4b0c, SUC_lexicon_tags_37 },
    { 0x46af4b0e, SUC_lexicon_tags_38 },
    { 0x106f4b0e, SUC_lexicon_tags_2 },
    { 0xf6cf4b0d, SUC_lexicon_tags_33 },
    { 0xc4f4b0f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216f4b15, SUC_lexicon_tags_17 },
    { 0x710f4b15, SUC_lexicon_tags_25 },
    { 0x3faf4b17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84f4b1b, SUC_lexicon_tags_2 },
    { 0xd8f4b1c, SUC_lexicon_tags_19 },
    { 0x4d2f4b1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8f4b21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcf4b24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb30f4b27, SUC_lexicon_tags_25 },
    { 0xd4ef4b28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8f4b2b, SUC_lexicon_tags_14 },
    { 0xf1af4b2b, SUC_lexicon_tags_0 },
    { 0xb24f4b2d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650f4b37, SUC_lexicon_tags_6 },
    { 0x1def4b38, SUC_lexicon_tags_49 },
    { 0xb6cf4b38, SUC_lexicon_tags_138 },
    { 0x4d8f4b3a, SUC_lexicon_tags_66 },
    { 0x704f4b39, SUC_lexicon_tags_43 },
    { 0x752f4b3c, SUC_lexicon_tags_77 },
    { 0x44cf4b3d, SUC_lexicon_tags_79 },
    { 0xf30f4b3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3af4b40, SUC_lexicon_tags_27 },
    { 0xd94f4b41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898f4b48, SUC_lexicon_tags_7 },
    { 0xd10f4b49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x764f4b4c, SUC_lexicon_tags_60 },
    { 0xfd0f4b4c, SUC_lexicon_tags_12 },
    { 0xa90f4b4e, SUC_lexicon_tags_60 },
    { 0x2b2f4b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6caf4b51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x636f4b53, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc04f4b55, SUC_lexicon_tags_6 },
    { 0x444f4b55, SUC_lexicon_tags_2 },
    { 0x89cf4b57, SUC_lexicon_tags_39 },
    { 0x1f4f4b58, SUC_lexicon_tags_37 },
    { 0xc74f4b58, SUC_lexicon_tags_6 },
    { 0x8eaf4b55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f4b61, SUC_lexicon_tags_27 },
    { 0x1faf4b61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3daf4b64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cef4b66, SUC_lexicon_tags_2 },
    { 0x63af4b66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x77af4b69, SUC_lexicon_tags_63 },
    { 0xb08f4b6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf9cf4b6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00f4b6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422f4b7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cf4b82, SUC_lexicon_tags_31 },
    { 0x836f4b82, SUC_lexicon_tags_0 },
    { 0x476f4b84, SUC_lexicon_tags_27 },
    { 0x28af4b83, SUC_lexicon_tags_33 },
    { 0x3d2f4b86, SUC_lexicon_tags_23 },
    { 0x570f4b86, SUC_lexicon_tags_34 },
    { 0xe0af4b87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38f4b8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2aaf4b91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8f4b95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x630f4b98, SUC_lexicon_tags_7 },
    { 0xf88f4b99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3af4b9b, SUC_lexicon_tags_15 },
    { 0xd52f4b9b, SUC_lexicon_tags_15 },
    { 0x688f4b9b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26f4ba3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1daf4ba5, SUC_lexicon_tags_12 },
    { 0x6f2f4ba6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4f4baa, SUC_lexicon_tags_1 },
    { 0xd84f4bab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83af4baf, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456f4bba, SUC_lexicon_tags_63 },
    { 0xce0f4bbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cf4bc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa14f4bc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788f4bc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76af4bcd, SUC_lexicon_tags_55 },
    { 0xaf0f4bcd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa18f4bd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4f4bd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86ef4bd6, SUC_lexicon_tags_25 },
    { 0x846f4bd7, SUC_lexicon_tags_25 },
    { 0xd1ef4bd8, SUC_lexicon_tags_17 },
    { 0xbf2f4bd7, SUC_lexicon_tags_15 },
    { 0x114f4bd9, SUC_lexicon_tags_11 },
    { 0x6f8f4bd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaf4bdf, SUC_lexicon_tags_25 },
    { 0x9def4be0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x93af4be3, SUC_lexicon_tags_43 },
    { 0x278f4be4, SUC_lexicon_tags_11 },
    { 0x1d4f4be3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94af4bea, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2f4bee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc42f4bf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aef4bf3, SUC_lexicon_tags_8 },
    { 0x1f6f4bf3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74f4bf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dcf4bfa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x67af4bfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2f4bff, SUC_lexicon_tags_25 },
    { 0x2c4f4c00, SUC_lexicon_tags_4 },
    { 0x404f4c01, SUC_lexicon_tags_17 },
    { 0xd24f4bff, SUC_lexicon_tags_15 },
    { 0x994f4c03, SUC_lexicon_tags_34 },
    { 0x3baf4c04, SUC_lexicon_tags_39 },
    { 0xc2f4c04, SUC_lexicon_tags_97 },
    { 0xc4cf4c03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ef4c09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecaf4c0e, SUC_lexicon_tags_17 },
    { 0x64af4c0f, SUC_lexicon_tags_11 },
    { 0xbfcf4c0f, SUC_lexicon_tags_12 },
    { 0x55ef4c11, SUC_lexicon_tags_38 },
    { 0xc0f4c12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31af4c19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x418f4c1b, SUC_lexicon_tags_1 },
    { 0xbc2f4c1c, SUC_lexicon_tags_15 },
    { 0xaa8f4c1c, SUC_lexicon_tags_15 },
    { 0x1ef4c1d, SUC_lexicon_tags_47 },
    { 0x6f8f4c1f, SUC_lexicon_tags_49 },
    { 0xa08f4c20, SUC_lexicon_tags_10 },
    { 0xfb0f4c1f, SUC_lexicon_tags_37 },
    { 0xfdaf4c1e, SUC_lexicon_tags_10 },
    { 0xb62f4c23, SUC_lexicon_tags_25 },
    { 0xdb0f4c1f, SUC_lexicon_tags_30 },
    { 0xf10f4c25, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaf4c2b, SUC_lexicon_tags_10 },
    { 0x638f4c2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12f4c33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842f4c37, SUC_lexicon_tags_66 },
    { 0x520f4c38, SUC_lexicon_tags_14 },
    { 0xe8ef4c39, SUC_lexicon_tags_37 },
    { 0xb6ef4c39, SUC_lexicon_tags_39 },
    { 0xcc2f4c3b, SUC_lexicon_tags_33 },
    { 0xbf4f4c38, SUC_lexicon_tags_60 },
    { 0x492f4c3a, SUC_lexicon_tags_62 },
    { 0xc6f4c3e, SUC_lexicon_tags_34 },
    { 0xd6cf4c3e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ef4c42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x240f4c45, SUC_lexicon_tags_65 },
    { 0x962f4c45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x236f4c49, SUC_lexicon_tags_25 },
    { 0x69af4c4a, SUC_lexicon_tags_37 },
    { 0x8e2f4c4b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6f4c4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfcaf4c51, SUC_lexicon_tags_17 },
    { 0xa04f4c51, SUC_lexicon_tags_21 },
    { 0x4c4f4c51, SUC_lexicon_tags_25 },
    { 0xa94f4c54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a4f4c56, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2f4c5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x894f4c5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f4c61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870f4c67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6cf4c69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0f4c6d, SUC_lexicon_tags_32 },
    { 0x194f4c6e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f4c77, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ef4c7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef4c7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f4c81, SUC_lexicon_tags_103 },
    { 0x33af4c82, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbef4c85, SUC_lexicon_tags_4 },
    { 0xe3af4c86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8f4c89, SUC_lexicon_tags_17 },
    { 0x8d4f4c8a, SUC_lexicon_tags_4 },
    { 0x8e8f4c89, SUC_lexicon_tags_28 },
    { 0x262f4c8c, SUC_lexicon_tags_37 },
    { 0xe2ef4c8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d0f4c8f, SUC_lexicon_tags_1 },
    { 0x32f4c8f, SUC_lexicon_tags_66 },
    { 0x46f4c8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0ef4c93, SUC_lexicon_tags_33 },
    { 0xed2f4c94, SUC_lexicon_tags_68 },
    { 0x4caf4c95, SUC_lexicon_tags_17 },
    { 0xbaf4c96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xba6f4c99, SUC_lexicon_tags_7 },
    { 0xa04f4c9a, SUC_lexicon_tags_3 },
    { 0x156f4c9a, SUC_lexicon_tags_8 },
    { 0xca4f4c9c, SUC_lexicon_tags_12 },
    { 0xab8f4c9c, SUC_lexicon_tags_37 },
    { 0xf86f4c9a, SUC_lexicon_tags_129 },
    { 0x3ecf4c9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd2af4ca1, SUC_lexicon_tags_1 },
    { 0x846f4ca1, SUC_lexicon_tags_56 },
    { 0x8c4f4ca2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470f4ca7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f4caa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ef4cad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f4f4caf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840f4cb3, SUC_lexicon_tags_15 },
    { 0x15cf4cb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd64f4cb7, SUC_lexicon_tags_87 },
    { 0xf1cf4cb7, SUC_lexicon_tags_88 },
    { 0x56ef4cb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd8f4cbb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb7ef4cbd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe80f4cbf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f4cc3, SUC_lexicon_tags_25 },
    { 0x4f0f4cc3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66af4ccb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f4cd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14f4cd3, SUC_lexicon_tags_17 },
    { 0xf8cf4cd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe96f4cd6, SUC_lexicon_tags_7 },
    { 0x27af4cd7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6a4f4cd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acf4cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612f4ce5, SUC_lexicon_tags_38 },
    { 0xf98f4ce6, SUC_lexicon_tags_26 },
    { 0x3bef4ce5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfccf4ce9, SUC_lexicon_tags_68 },
    { 0x5f2f4cea, SUC_lexicon_tags_1 },
    { 0x722f4ce9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd58f4cee, SUC_lexicon_tags_25 },
    { 0x3dcf4cee, SUC_lexicon_tags_47 },
    { 0x60ef4cf0, SUC_lexicon_tags_5 },
    { 0x2acf4cee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80f4cf7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90f4cfb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x186f4cfd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x982f4d00, SUC_lexicon_tags_60 },
    { 0xd8f4d01, SUC_lexicon_tags_30 },
    { 0x3c8f4d02, SUC_lexicon_tags_14 },
    { 0xbf8f4d01, SUC_lexicon_tags_39 },
    { 0x44f4d02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x132f4d06, SUC_lexicon_tags_1 },
    { 0xc10f4d07, SUC_lexicon_tags_15 },
    { 0xaaef4d08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0f4d0b, SUC_lexicon_tags_37 },
    { 0x412f4d0c, SUC_lexicon_tags_11 },
    { 0x382f4d0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8f4d14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccf4d17, SUC_lexicon_tags_25 },
    { 0xa52f4d18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x286f4d1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870f4d1f, SUC_lexicon_tags_11 },
    { 0xb92f4d20, SUC_lexicon_tags_38 },
    { 0xf5af4d1f, SUC_lexicon_tags_12 },
    { 0x5d6f4d22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc8f4d24, SUC_lexicon_tags_47 },
    { 0x6b0f4d25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x610f4d27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6f4d2c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xafaf4d2e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16f4d31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ef4d3c, SUC_lexicon_tags_14 },
    { 0x634f4d3d, SUC_lexicon_tags_46 },
    { 0x208f4d3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x862f4d40, SUC_lexicon_tags_68 },
    { 0x972f4d41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2f4d45, SUC_lexicon_tags_29 },
    { 0xd18f4d46, SUC_lexicon_tags_11 },
    { 0x4af4d47, SUC_lexicon_tags_9 },
    { 0xd54f4d48, SUC_lexicon_tags_29 },
    { 0x84f4d46, SUC_lexicon_tags_2 },
    { 0xfcf4d4a, SUC_lexicon_tags_23 },
    { 0xe3ef4d47, SUC_lexicon_tags_14 },
    { 0x7bcf4d4c, SUC_lexicon_tags_28 },
    { 0x3acf4d48, SUC_lexicon_tags_21 },
    { 0x4a4f4d4e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x20cf4d51, SUC_lexicon_tags_6 },
    { 0x1acf4d52, SUC_lexicon_tags_11 },
    { 0x266f4d52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd2af4d55, SUC_lexicon_tags_3 },
    { 0xf90f4d55, SUC_lexicon_tags_2 },
    { 0x95cf4d57, SUC_lexicon_tags_7 },
    { 0xdbaf4d58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2f4d5b, SUC_lexicon_tags_14 },
    { 0x438f4d5c, SUC_lexicon_tags_37 },
    { 0xb4f4d5b, SUC_lexicon_tags_63 },
    { 0x602f4d5e, SUC_lexicon_tags_65 },
    { 0x798f4d5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ef4d63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ef4d67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe00f4d6a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20f4d71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8f4d74, SUC_lexicon_tags_23 },
    { 0xf3ef4d75, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cf4d78, SUC_lexicon_tags_25 },
    { 0x198f4d78, SUC_lexicon_tags_10 },
    { 0xa5cf4d78, SUC_lexicon_tags_0 },
    { 0xa4ef4d78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ef4d7f, SUC_lexicon_tags_32 },
    { 0x11cf4d80, SUC_lexicon_tags_12 },
    { 0xb70f4d81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818f4d86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x920f4d89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38cf4d8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xebef4d8e, SUC_lexicon_tags_1 },
    { 0xe5af4d8f, SUC_lexicon_tags_1 },
    { 0x9eef4d90, SUC_lexicon_tags_12 },
    { 0x65ef4d90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06f4d96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x754f4d98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2eef4d9a, SUC_lexicon_tags_25 },
    { 0x54af4d9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46f4da0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaef4da3, SUC_lexicon_tags_11 },
    { 0x3bef4da4, SUC_lexicon_tags_19 },
    { 0x38af4da4, SUC_lexicon_tags_2 },
    { 0xd08f4da6, SUC_lexicon_tags_47 },
    { 0x4def4da5, SUC_lexicon_tags_11 },
    { 0x368f4da7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6f4dad, SUC_lexicon_tags_84 },
    { 0xb90f4dae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7ef4db0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2f4db4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224f4db8, SUC_lexicon_tags_60 },
    { 0x6a6f4db9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56f4dbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x908f4dbf, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60f4dc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f4dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x682f4dcb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x122f4dce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540f4dd2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8f4dd8, SUC_lexicon_tags_47 },
    { 0xe78f4dd9, SUC_lexicon_tags_12 },
    { 0xf88f4dd9, SUC_lexicon_tags_27 },
    { 0x70f4dd9, SUC_lexicon_tags_17 },
    { 0xd3af4ddc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ef4de0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324f4de6, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xa14f4de8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676f4dec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdef4def, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2f4df4, SUC_lexicon_tags_2 },
    { 0xe4cf4df5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788f4df9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf4dfd, SUC_lexicon_tags_79 },
    { 0x7c8f4dfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30ef4e00, SUC_lexicon_tags_9 },
    { 0xd70f4e01, SUC_lexicon_tags_3 },
    { 0x598f4e00, SUC_lexicon_tags_15 },
    { 0x654f4e03, SUC_lexicon_tags_61 },
    { 0xae4f4e03, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232f4e0a, SUC_lexicon_tags_66 },
    { 0x7e2f4e0b, SUC_lexicon_tags_32 },
    { 0x71af4e0c, SUC_lexicon_tags_37 },
    { 0xd1ef4e0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2bef4e0f, SUC_lexicon_tags_30 },
    { 0x610f4e10, SUC_lexicon_tags_47 },
    { 0xe68f4e11, SUC_lexicon_tags_25 },
    { 0xc1cf4e11, SUC_lexicon_tags_37 },
    { 0x788f4e11, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xebef4e15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf34f4e17, SUC_lexicon_tags_64 },
    { 0x400f4e18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5af4e1a, SUC_lexicon_tags_84 },
    { 0x514f4e1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ef4e1f, SUC_lexicon_tags_37 },
    { 0x664f4e1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6f4e26, SUC_lexicon_tags_35 },
    { 0x796f4e26, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xc60f4e29, SUC_lexicon_tags_8 },
    { 0xb6cf4e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9eef4e2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250f4e34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x144f4e3a, SUC_lexicon_tags_17 },
    { 0x148f4e3b, SUC_lexicon_tags_6 },
    { 0x366f4e3c, SUC_lexicon_tags_79 },
    { 0xe06f4e3b, SUC_lexicon_tags_17 },
    { 0xd2af4e3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6f4e48, SUC_lexicon_tags_2 },
    { 0xe32f4e49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6f4e4d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14f4e50, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682f4e54, SUC_lexicon_tags_37 },
    { 0x8ef4e54, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8f4e5b, SUC_lexicon_tags_37 },
    { 0x982f4e5b, SUC_lexicon_tags_17 },
    { 0xce4f4e5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecf4e64, SUC_lexicon_tags_0 },
    { 0x652f4e64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa72f4e67, SUC_lexicon_tags_8 },
    { 0x596f4e67, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xed0f4e6b, SUC_lexicon_tags_45 },
    { 0x504f4e6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf4e74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0af4e78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ef4e7c, SUC_lexicon_tags_0 },
    { 0xe5cf4e7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x482f4e80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf58f4e82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cf4e86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc68f4e88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69af4e8b, SUC_lexicon_tags_19 },
    { 0xc2cf4e8c, SUC_lexicon_tags_26 },
    { 0x356f4e8b, SUC_lexicon_tags_47 },
    { 0x8d8f4e8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29af4e93, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f4e99, SUC_lexicon_tags_7 },
    { 0x1fef4e9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330f4e9f, SUC_lexicon_tags_15 },
    { 0x8e0f4ea0, SUC_lexicon_tags_14 },
    { 0x42af4ea0, SUC_lexicon_tags_6 },
    { 0x3cf4ea0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4f4eab, SUC_lexicon_tags_30 },
    { 0x2dcf4eac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x166f4eb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8af4eb6, SUC_lexicon_tags_64 },
    { 0x97cf4eb7, SUC_lexicon_tags_87 },
    { 0x2def4eb8, SUC_lexicon_tags_14 },
    { 0xed0f4eb9, SUC_lexicon_tags_12 },
    { 0xfa0f4eba, SUC_lexicon_tags_1 },
    { 0x28ef4ebb, SUC_lexicon_tags_38 },
    { 0x65ef4ebc, SUC_lexicon_tags_3 },
    { 0x962f4ebc, SUC_lexicon_tags_12 },
    { 0x8b2f4ebb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc4f4ec0, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xfc0f4ec2, SUC_lexicon_tags_21 },
    { 0xab4f4ec3, SUC_lexicon_tags_2 },
    { 0x9fcf4ec4, SUC_lexicon_tags_12 },
    { 0x562f4ec5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadef4ece, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ef4ed2, SUC_lexicon_tags_11 },
    { 0xa56f4ed3, SUC_lexicon_tags_17 },
    { 0x772f4ed2, SUC_lexicon_tags_9 },
    { 0x2fcf4ed2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x506f4ed8, SUC_lexicon_tags_23 },
    { 0xccf4ed9, SUC_lexicon_tags_2 },
    { 0xcc0f4ed9, SUC_lexicon_tags_65 },
    { 0x762f4edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0af4ee0, SUC_lexicon_tags_37 },
    { 0x9a2f4ee1, SUC_lexicon_tags_12 },
    { 0x98ef4ee2, SUC_lexicon_tags_17 },
    { 0xb14f4ee2, SUC_lexicon_tags_30 },
    { 0xf88f4ee2, SUC_lexicon_tags_7 },
    { 0x946f4ee4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x648f4ee8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46f4eeb, SUC_lexicon_tags_45 },
    { 0x820f4eeb, SUC_lexicon_tags_160 },
    { 0x1f6f4eed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6f4ef3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x4d0f4ef5, SUC_lexicon_tags_17 },
    { 0xb82f4ef6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x358f4ef8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0f4efb, SUC_lexicon_tags_12 },
    { 0x81cf4efc, SUC_lexicon_tags_17 },
    { 0xd46f4efd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9cf4eff, SUC_lexicon_tags_33 },
    { 0x4a0f4f00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50f4f06, SUC_lexicon_tags_37 },
    { 0x6c0f4f06, SUC_lexicon_tags_75 },
    { 0xe88f4f08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ef4f0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ef4f10, SUC_lexicon_tags_16 },
    { 0xc8cf4f10, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7baf4f13, SUC_lexicon_tags_26 },
    { 0x54ef4f14, SUC_lexicon_tags_17 },
    { 0xcfaf4f13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x226f4f17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50f4f1d, SUC_lexicon_tags_15 },
    { 0x870f4f1d, SUC_lexicon_tags_45 },
    { 0xcfef4f1f, SUC_lexicon_tags_4 },
    { 0x54ef4f20, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710f4f26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29af4f28, SUC_lexicon_tags_19 },
    { 0x144f4f29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f4f2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x626f4f2f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7f4f4f31, SUC_lexicon_tags_19 },
    { 0xedaf4f32, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4cef4f34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f4f39, SUC_lexicon_tags_48 },
    { 0x3d8f4f3a, SUC_lexicon_tags_62 },
    { 0x520f4f3b, SUC_lexicon_tags_17 },
    { 0x82cf4f39, SUC_lexicon_tags_17 },
    { 0x6f4f39, SUC_lexicon_tags_27 },
    { 0xa4ef4f39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546f4f42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386f4f47, SUC_lexicon_tags_7 },
    { 0xdecf4f48, SUC_lexicon_tags_31 },
    { 0x302f4f49, SUC_lexicon_tags_11 },
    { 0x10f4f48, SUC_lexicon_tags_47 },
    { 0x1cf4f4b, SUC_lexicon_tags_29 },
    { 0x868f4f4a, SUC_lexicon_tags_49 },
    { 0x9d4f4f48, SUC_lexicon_tags_12 },
    { 0x71af4f49, SUC_lexicon_tags_11 },
    { 0xb8ef4f4f, SUC_lexicon_tags_151 },
    { 0xf6af4f50, SUC_lexicon_tags_14 },
    { 0x4c4f4f51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6f4f55, SUC_lexicon_tags_2 },
    { 0x98ef4f55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0f4f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x526f4f5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8f4f62, SUC_lexicon_tags_82 },
    { 0xaf8f4f63, SUC_lexicon_tags_19 },
    { 0x104f4f64, SUC_lexicon_tags_307 },
    { 0x1af4f65, SUC_lexicon_tags_17 },
    { 0x528f4f63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f4f6b, SUC_lexicon_tags_87 },
    { 0x14af4f6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xef0f4f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb48f4f70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe22f4f73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0f4f75, SUC_lexicon_tags_257 },
    { 0x978f4f76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x55cf4f79, SUC_lexicon_tags_35 },
    { 0x182f4f7a, SUC_lexicon_tags_11 },
    { 0xc02f4f7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6af4f81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x864f4f83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef4f86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0f4f8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xacef4f8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd2af4f90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04f4f93, SUC_lexicon_tags_69 },
    { 0x822f4f94, SUC_lexicon_tags_19 },
    { 0x5ccf4f93, SUC_lexicon_tags_17 },
    { 0x762f4f96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce0f4f98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820f4f9c, SUC_lexicon_tags_17 },
    { 0xb72f4f9d, SUC_lexicon_tags_33 },
    { 0xe84f4f9e, SUC_lexicon_tags_11 },
    { 0x22cf4f9e, SUC_lexicon_tags_2 },
    { 0x108f4f9f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482f4fa6, SUC_lexicon_tags_37 },
    { 0x952f4fa6, SUC_lexicon_tags_11 },
    { 0x7acf4fa8, SUC_lexicon_tags_7 },
    { 0x85af4fa9, SUC_lexicon_tags_9 },
    { 0xee4f4fa8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1c6f4fac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112f4fb1, SUC_lexicon_tags_7 },
    { 0xebef4fb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f4fb9, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x368f4fbc, SUC_lexicon_tags_25 },
    { 0x4a2f4fbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf2af4fbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x232f4fc2, SUC_lexicon_tags_16 },
    { 0x556f4fc2, SUC_lexicon_tags_27 },
    { 0xcaf4fc4, SUC_lexicon_tags_49 },
    { 0xfe0f4fc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x440f4fc7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8c2f4fc9, SUC_lexicon_tags_1 },
    { 0x91ef4fca, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x95cf4fcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf4fd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x286f4fd6, SUC_lexicon_tags_68 },
    { 0xb52f4fd7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eaf4fe4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc92f4fe6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f4fe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4af4fea, SUC_lexicon_tags_35 },
    { 0x5d2f4feb, SUC_lexicon_tags_37 },
    { 0x776f4fea, SUC_lexicon_tags_32 },
    { 0x14ef4fed, SUC_lexicon_tags_1 },
    { 0xdeaf4fee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cf4ff1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94f4ff8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd58f4ffa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd22f5000, SUC_lexicon_tags_17 },
    { 0x18ef5001, SUC_lexicon_tags_60 },
    { 0xccef5002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60af5005, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ef500e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf5010, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2def5017, SUC_lexicon_tags_26 },
    { 0xad0f5018, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0f501f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf5021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556f5027, SUC_lexicon_tags_17 },
    { 0xaaf5028, SUC_lexicon_tags_45 },
    { 0xff8f5027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2faf502b, SUC_lexicon_tags_15 },
    { 0xf88f502b, SUC_lexicon_tags_423 },
    { 0xc88f502d, SUC_lexicon_tags_7 },
    { 0xe42f502d, SUC_lexicon_tags_11 },
    { 0xff0f502f, SUC_lexicon_tags_28 },
    { 0xec0f5030, SUC_lexicon_tags_12 },
    { 0xd7af5031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb38f5033, SUC_lexicon_tags_1 },
    { 0x924f5033, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24f5038, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430f503f, SUC_lexicon_tags_38 },
    { 0x54cf5040, SUC_lexicon_tags_30 },
    { 0xa7af5041, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b6f5043, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9af5045, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320f504a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfecf504d, SUC_lexicon_tags_900 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32f5051, SUC_lexicon_tags_25 },
    { 0xeef5052, SUC_lexicon_tags_11 },
    { 0x744f5053, SUC_lexicon_tags_17 },
    { 0x74cf5052, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf5058, SUC_lexicon_tags_30 },
    { 0xc84f5059, SUC_lexicon_tags_62 },
    { 0x462f505a, SUC_lexicon_tags_1 },
    { 0x616f505b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66f5060, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaef5063, SUC_lexicon_tags_2 },
    { 0xfe8f5064, SUC_lexicon_tags_34 },
    { 0x2b0f5065, SUC_lexicon_tags_63 },
    { 0x73af5066, SUC_lexicon_tags_45 },
    { 0x412f5067, SUC_lexicon_tags_12 },
    { 0x89cf5067, SUC_lexicon_tags_49 },
    { 0x22cf5069, SUC_lexicon_tags_46 },
    { 0x234f506a, SUC_lexicon_tags_47 },
    { 0x8bef506a, SUC_lexicon_tags_15 },
    { 0x606f5069, SUC_lexicon_tags_14 },
    { 0xef8f506d, SUC_lexicon_tags_15 },
    { 0x4bcf506e, SUC_lexicon_tags_49 },
    { 0xa2f506f, SUC_lexicon_tags_29 },
    { 0x7ef5070, SUC_lexicon_tags_17 },
    { 0x24af5068, SUC_lexicon_tags_38 },
    { 0x7c4f5072, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90f5075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0af507d, SUC_lexicon_tags_6 },
    { 0x414f507e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80af5082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0f5089, SUC_lexicon_tags_29 },
    { 0xcd8f508a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafaf508c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x708f508e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560f5092, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24f5095, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbaef5097, SUC_lexicon_tags_15 },
    { 0x934f5098, SUC_lexicon_tags_2 },
    { 0x3d2f5099, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cf50a2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40f50a6, SUC_lexicon_tags_25 },
    { 0xf7cf50a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130f50af, SUC_lexicon_tags_15 },
    { 0xa0cf50af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76f50b4, SUC_lexicon_tags_17 },
    { 0x666f50b5, SUC_lexicon_tags_68 },
    { 0xce8f50b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x242f50b8, SUC_lexicon_tags_2 },
    { 0xef6f50b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x228f50bc, SUC_lexicon_tags_7 },
    { 0xfaf50bd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc7cf50bf, SUC_lexicon_tags_37 },
    { 0xa38f50bf, SUC_lexicon_tags_1 },
    { 0x12af50c1, SUC_lexicon_tags_18 },
    { 0x630f50bf, SUC_lexicon_tags_87 },
    { 0xdaef50c0, SUC_lexicon_tags_2 },
    { 0x15cf50c4, SUC_lexicon_tags_0 },
    { 0xe28f50c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadef50cb, SUC_lexicon_tags_13 },
    { 0xfcaf50cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28f50d4, SUC_lexicon_tags_14 },
    { 0x7caf50d5, SUC_lexicon_tags_106 },
    { 0x554f50d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132f50da, SUC_lexicon_tags_39 },
    { 0xf52f50db, SUC_lexicon_tags_31 },
    { 0x29ef50dc, SUC_lexicon_tags_123 },
    { 0x6e8f50dd, SUC_lexicon_tags_47 },
    { 0xcc2f50da, SUC_lexicon_tags_229 },
    { 0x8fcf50dd, SUC_lexicon_tags_49 },
    { 0xb74f50e0, SUC_lexicon_tags_17 },
    { 0x90f50e0, SUC_lexicon_tags_17 },
    { 0xf8ef50e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caf50e8, SUC_lexicon_tags_7 },
    { 0xaa0f50e8, SUC_lexicon_tags_17 },
    { 0x818f50ea, SUC_lexicon_tags_49 },
    { 0xc0ef50eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232f50ef, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc38f50f1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x784f50f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eaf50f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90af50f9, SUC_lexicon_tags_15 },
    { 0x7eaf50fa, SUC_lexicon_tags_45 },
    { 0x194f50fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0f50ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8f5103, SUC_lexicon_tags_25 },
    { 0xdbcf5104, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4f510a, SUC_lexicon_tags_3 },
    { 0x60af510b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x896f510d, SUC_lexicon_tags_28 },
    { 0x3faf510e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552f5112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b2f5114, SUC_lexicon_tags_48 },
    { 0x9a8f5115, SUC_lexicon_tags_30 },
    { 0xd12f5116, SUC_lexicon_tags_4 },
    { 0x552f5116, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x816f5119, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28f511e, SUC_lexicon_tags_57 },
    { 0x82cf511e, SUC_lexicon_tags_7 },
    { 0xaf2f511f, SUC_lexicon_tags_25 },
    { 0x138f5121, SUC_lexicon_tags_37 },
    { 0x7af511f, SUC_lexicon_tags_11 },
    { 0x30af5120, SUC_lexicon_tags_0 },
    { 0x130f5124, SUC_lexicon_tags_65 },
    { 0xfccf5125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4f5128, SUC_lexicon_tags_56 },
    { 0x2f6f5128, SUC_lexicon_tags_23 },
    { 0xbeef512a, SUC_lexicon_tags_17 },
    { 0xbd6f512b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f5132, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2f5137, SUC_lexicon_tags_17 },
    { 0xb44f5137, SUC_lexicon_tags_1 },
    { 0x6e6f5137, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f513f, SUC_lexicon_tags_8 },
    { 0xcb2f513f, SUC_lexicon_tags_37 },
    { 0x9d2f5141, SUC_lexicon_tags_25 },
    { 0x932f5140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a2f5144, SUC_lexicon_tags_30 },
    { 0x642f5144, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60af514c, SUC_lexicon_tags_65 },
    { 0x936f514d, SUC_lexicon_tags_2 },
    { 0x46af514c, SUC_lexicon_tags_16 },
    { 0x186f514f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x552f5151, SUC_lexicon_tags_37 },
    { 0x172f5152, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b8f5154, SUC_lexicon_tags_60 },
    { 0x1faf5154, SUC_lexicon_tags_60 },
    { 0x6e4f5156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58cf5158, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa14f515a, SUC_lexicon_tags_30 },
    { 0x8baf515b, SUC_lexicon_tags_1 },
    { 0x6d0f515b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad8f515e, SUC_lexicon_tags_79 },
    { 0x576f515e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6f5164, SUC_lexicon_tags_26 },
    { 0xcaf5165, SUC_lexicon_tags_17 },
    { 0x2d0f5165, SUC_lexicon_tags_12 },
    { 0xbd4f5166, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3acf516a, SUC_lexicon_tags_12 },
    { 0x78ef516b, SUC_lexicon_tags_2 },
    { 0x484f516c, SUC_lexicon_tags_2 },
    { 0xf96f516c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcf5174, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadcf5178, SUC_lexicon_tags_47 },
    { 0x592f5178, SUC_lexicon_tags_15 },
    { 0x97af5178, SUC_lexicon_tags_25 },
    { 0xe3cf517b, SUC_lexicon_tags_11 },
    { 0x736f517c, SUC_lexicon_tags_32 },
    { 0x134f517d, SUC_lexicon_tags_17 },
    { 0x336f517e, SUC_lexicon_tags_63 },
    { 0x874f517a, SUC_lexicon_tags_2 },
    { 0xbcef5180, SUC_lexicon_tags_244 },
    { 0x328f517d, SUC_lexicon_tags_37 },
    { 0x6d2f5182, SUC_lexicon_tags_15 },
    { 0xc64f5180, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x200f5185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cf5189, SUC_lexicon_tags_56 },
    { 0x3d0f518a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8f518d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268f5196, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf5199, SUC_lexicon_tags_19 },
    { 0x6dcf519a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ef519e, SUC_lexicon_tags_65 },
    { 0xaef519f, SUC_lexicon_tags_39 },
    { 0xb4ef51a0, SUC_lexicon_tags_25 },
    { 0x3f6f519e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930f51a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7cf51a9, SUC_lexicon_tags_1 },
    { 0xe58f51a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x150f51ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920f51b1, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x3b8f51b3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xce4f51b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77af51b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe2af51ba, SUC_lexicon_tags_12 },
    { 0xdf6f51bb, SUC_lexicon_tags_29 },
    { 0xf7cf51bc, SUC_lexicon_tags_10 },
    { 0xe66f51bc, SUC_lexicon_tags_37 },
    { 0xa80f51be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cf51c4, SUC_lexicon_tags_11 },
    { 0x5fcf51c5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cf51c8, SUC_lexicon_tags_16 },
    { 0x6a6f51c8, SUC_lexicon_tags_34 },
    { 0xb72f51ca, SUC_lexicon_tags_3 },
    { 0x7c4f51c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x580f51cd, SUC_lexicon_tags_64 },
    { 0xc64f51ce, SUC_lexicon_tags_17 },
    { 0x280f51cf, SUC_lexicon_tags_37 },
    { 0x396f51d0, SUC_lexicon_tags_14 },
    { 0x66f51cf, SUC_lexicon_tags_38 },
    { 0x8d6f51d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccaf51d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x566f51d9, SUC_lexicon_tags_47 },
    { 0x476f51d9, SUC_lexicon_tags_25 },
    { 0x1bcf51db, SUC_lexicon_tags_37 },
    { 0xa94f51d9, SUC_lexicon_tags_4 },
    { 0xbe2f51d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82f51df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ef51e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x876f51e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6f51eb, SUC_lexicon_tags_7 },
    { 0x9bcf51eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cf51f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x422f51f3, SUC_lexicon_tags_5 },
    { 0xb4f51f4, SUC_lexicon_tags_542 },
    { 0xe10f51f5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef5200, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f5203, SUC_lexicon_tags_60 },
    { 0x1a6f5203, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962f520a, SUC_lexicon_tags_11 },
    { 0x800f520b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0f5215, SUC_lexicon_tags_29 },
    { 0xc3cf5215, SUC_lexicon_tags_12 },
    { 0x678f5217, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fef521b, SUC_lexicon_tags_630 },
    { 0xbb4f521b, SUC_lexicon_tags_507 },
    { 0x690f521d, SUC_lexicon_tags_12 },
    { 0x982f521e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cef5221, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x176f5223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eef5225, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ef5227, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6f522b, SUC_lexicon_tags_27 },
    { 0xd6f522c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592f523a, SUC_lexicon_tags_1 },
    { 0xa5ef523a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xcfcf523d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd3af523f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ef5242, SUC_lexicon_tags_17 },
    { 0x792f5242, SUC_lexicon_tags_1 },
    { 0xee6f5244, SUC_lexicon_tags_2 },
    { 0xef6f5245, SUC_lexicon_tags_2 },
    { 0xb96f5244, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8aef5248, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ef524a, SUC_lexicon_tags_118 },
    { 0xf3af524a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9af524d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bef524f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc5af5251, SUC_lexicon_tags_13 },
    { 0xc10f5252, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f5256, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcecf5258, SUC_lexicon_tags_235 },
    { 0x5e8f5259, SUC_lexicon_tags_19 },
    { 0xcccf525a, SUC_lexicon_tags_2 },
    { 0x3b6f5259, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248f525f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x830f5262, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5af5265, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2af5269, SUC_lexicon_tags_17 },
    { 0x84f526a, SUC_lexicon_tags_8 },
    { 0x1bef5269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x248f526e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7b4f5270, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x44f5272, SUC_lexicon_tags_25 },
    { 0xa8f5273, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f527b, SUC_lexicon_tags_1 },
    { 0xcbcf527b, SUC_lexicon_tags_28 },
    { 0xdf4f527c, SUC_lexicon_tags_37 },
    { 0xed2f527e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8f5285, SUC_lexicon_tags_1 },
    { 0x448f5286, SUC_lexicon_tags_57 },
    { 0x71af5287, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640f528b, SUC_lexicon_tags_15 },
    { 0x87ef528c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2baf5290, SUC_lexicon_tags_17 },
    { 0xbbcf5290, SUC_lexicon_tags_14 },
    { 0x904f5292, SUC_lexicon_tags_15 },
    { 0xcb6f5293, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf94f5295, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc24f5297, SUC_lexicon_tags_37 },
    { 0x1d6f5298, SUC_lexicon_tags_2 },
    { 0xaaf5297, SUC_lexicon_tags_22 },
    { 0xd00f529a, SUC_lexicon_tags_17 },
    { 0xb74f529a, SUC_lexicon_tags_15 },
    { 0x952f529b, SUC_lexicon_tags_47 },
    { 0x25af529b, SUC_lexicon_tags_47 },
    { 0x8acf529c, SUC_lexicon_tags_1 },
    { 0x32af529a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2def52a2, SUC_lexicon_tags_103 },
    { 0x4caf52a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68f52a8, SUC_lexicon_tags_29 },
    { 0x852f52a9, SUC_lexicon_tags_12 },
    { 0x29cf52aa, SUC_lexicon_tags_14 },
    { 0x9aef52ab, SUC_lexicon_tags_1 },
    { 0xae8f52aa, SUC_lexicon_tags_1 },
    { 0x212f52aa, SUC_lexicon_tags_2 },
    { 0xa72f52a9, SUC_lexicon_tags_7 },
    { 0x8a4f52af, SUC_lexicon_tags_33 },
    { 0x3b6f52ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x492f52b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x666f52b5, SUC_lexicon_tags_11 },
    { 0xad0f52b6, SUC_lexicon_tags_35 },
    { 0x14cf52b7, SUC_lexicon_tags_43 },
    { 0x902f52b8, SUC_lexicon_tags_4 },
    { 0x488f52b9, SUC_lexicon_tags_17 },
    { 0xb98f52b5, SUC_lexicon_tags_26 },
    { 0xf64f52bb, SUC_lexicon_tags_1 },
    { 0xd56f52bc, SUC_lexicon_tags_28 },
    { 0x460f52bc, SUC_lexicon_tags_144 },
    { 0xa98f52b5, SUC_lexicon_tags_15 },
    { 0x52af52bf, SUC_lexicon_tags_1 },
    { 0x626f52b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x600f52c3, SUC_lexicon_tags_39 },
    { 0x46af52c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64f52ce, SUC_lexicon_tags_26 },
    { 0x3a0f52ce, SUC_lexicon_tags_25 },
    { 0xd02f52d0, SUC_lexicon_tags_19 },
    { 0x946f52ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd4f52d3, SUC_lexicon_tags_11 },
    { 0x472f52d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x380f52d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38f52dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ef52e0, SUC_lexicon_tags_11 },
    { 0x79ef52e0, SUC_lexicon_tags_8 },
    { 0x964f52e2, SUC_lexicon_tags_33 },
    { 0xccef52e0, SUC_lexicon_tags_17 },
    { 0x1ef52e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ef52e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa6ef52e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6f52ef, SUC_lexicon_tags_43 },
    { 0x1bcf52f0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdecf52f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x744f52fa, SUC_lexicon_tags_28 },
    { 0x866f52fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf52fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x470f52ff, SUC_lexicon_tags_1 },
    { 0x7b0f52ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2f5306, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ef5309, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62ef530b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ef530e, SUC_lexicon_tags_4 },
    { 0x410f530f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e8f5313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8caf5316, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb88f5318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738f531d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28f5323, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f5326, SUC_lexicon_tags_11 },
    { 0x6caf5326, SUC_lexicon_tags_148 },
    { 0x75af5328, SUC_lexicon_tags_23 },
    { 0x8c6f5329, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x9b4f532b, SUC_lexicon_tags_19 },
    { 0x2a0f532c, SUC_lexicon_tags_15 },
    { 0xb12f532c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6f5331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddef5337, SUC_lexicon_tags_17 },
    { 0x772f5337, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8f5340, SUC_lexicon_tags_9 },
    { 0xd78f5340, SUC_lexicon_tags_0 },
    { 0xd58f5342, SUC_lexicon_tags_49 },
    { 0xa60f5342, SUC_lexicon_tags_15 },
    { 0xef2f5344, SUC_lexicon_tags_37 },
    { 0x6a8f5345, SUC_lexicon_tags_26 },
    { 0x384f5344, SUC_lexicon_tags_42 },
    { 0x660f5342, SUC_lexicon_tags_17 },
    { 0xb94f5347, SUC_lexicon_tags_7 },
    { 0x15ef5349, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ef534d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x6caf534f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a4f5351, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x768f5353, SUC_lexicon_tags_11 },
    { 0xd9cf5353, SUC_lexicon_tags_43 },
    { 0x282f5354, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cf535f, SUC_lexicon_tags_67 },
    { 0xc8f5360, SUC_lexicon_tags_136 },
    { 0x6baf5361, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61ef5363, SUC_lexicon_tags_69 },
    { 0xd9cf5363, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x356f5367, SUC_lexicon_tags_45 },
    { 0x7aaf5367, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2e0f536a, SUC_lexicon_tags_1 },
    { 0x8ef536b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaf536f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcf5373, SUC_lexicon_tags_19 },
    { 0x2bcf5373, SUC_lexicon_tags_25 },
    { 0x2f2f5374, SUC_lexicon_tags_23 },
    { 0xac4f5376, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e4f5378, SUC_lexicon_tags_21 },
    { 0x2c8f5378, SUC_lexicon_tags_14 },
    { 0xaeef5378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0f5380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ef5383, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a6f5385, SUC_lexicon_tags_8 },
    { 0x50f5386, SUC_lexicon_tags_9 },
    { 0x8a0f5387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31af538e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x51cf5391, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x128f5393, SUC_lexicon_tags_48 },
    { 0x588f5394, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0f5396, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f539d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6f53a3, SUC_lexicon_tags_38 },
    { 0x7f8f53a4, SUC_lexicon_tags_33 },
    { 0x2c8f53a5, SUC_lexicon_tags_25 },
    { 0xb8f53a6, SUC_lexicon_tags_0 },
    { 0xba4f53a5, SUC_lexicon_tags_33 },
    { 0xe58f53a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8f53ad, SUC_lexicon_tags_1 },
    { 0x4e2f53ae, SUC_lexicon_tags_65 },
    { 0x19cf53ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcccf53b1, SUC_lexicon_tags_1 },
    { 0xfd0f53b2, SUC_lexicon_tags_27 },
    { 0x1baf53b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0f53b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0f53ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cf53c2, SUC_lexicon_tags_25 },
    { 0x5aef53c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb14f53c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2f53c9, SUC_lexicon_tags_2 },
    { 0xdbef53ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30af53cf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90f53d3, SUC_lexicon_tags_23 },
    { 0x268f53d4, SUC_lexicon_tags_43 },
    { 0x31cf53d4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1dcf53d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf56f53d9, SUC_lexicon_tags_45 },
    { 0x3c8f53da, SUC_lexicon_tags_7 },
    { 0xa2f53da, SUC_lexicon_tags_12 },
    { 0xfe6f53dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4f53e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c0f53e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8af53ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cf53f2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc68f53f4, SUC_lexicon_tags_87 },
    { 0x778f53f5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x99cf53f8, SUC_lexicon_tags_37 },
    { 0x17cf53f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54cf53fd, SUC_lexicon_tags_37 },
    { 0x3e6f53fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478f5406, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8f540c, SUC_lexicon_tags_38 },
    { 0x484f540d, SUC_lexicon_tags_7 },
    { 0x94af540e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98cf5411, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7af5415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ef5419, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x700f541c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ef5424, SUC_lexicon_tags_7 },
    { 0x47af5425, SUC_lexicon_tags_80 },
    { 0x2f8f5424, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8f5429, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6f5434, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ef5438, SUC_lexicon_tags_1 },
    { 0xb96f5439, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6cef543b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8f543f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0af5442, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x388f5445, SUC_lexicon_tags_2 },
    { 0x9aaf5446, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcf5450, SUC_lexicon_tags_30 },
    { 0xbfef5450, SUC_lexicon_tags_19 },
    { 0x9c4f5451, SUC_lexicon_tags_11 },
    { 0x96f5453, SUC_lexicon_tags_25 },
    { 0xd1cf5453, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ef5457, SUC_lexicon_tags_1 },
    { 0x436f5458, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ef545c, SUC_lexicon_tags_2 },
    { 0xbb4f545d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ef5460, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58ef5462, SUC_lexicon_tags_37 },
    { 0x37af5463, SUC_lexicon_tags_13 },
    { 0xb80f5464, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2f546b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe12f546d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccf5471, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb56f5473, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86ef5475, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9af5478, SUC_lexicon_tags_21 },
    { 0xb24f5478, SUC_lexicon_tags_43 },
    { 0x24cf547a, SUC_lexicon_tags_55 },
    { 0x66cf547b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8af5483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256f5488, SUC_lexicon_tags_37 },
    { 0xff4f5489, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124f548e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5caf5491, SUC_lexicon_tags_27 },
    { 0xa88f5491, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x476f5494, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64af5496, SUC_lexicon_tags_0 },
    { 0xec0f5496, SUC_lexicon_tags_16 },
    { 0xd36f5498, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x558f549a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4cf549c, SUC_lexicon_tags_12 },
    { 0x4ccf549c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cf54a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f54ab, SUC_lexicon_tags_37 },
    { 0xdd0f54ac, SUC_lexicon_tags_12 },
    { 0x760f54ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90f54b0, SUC_lexicon_tags_14 },
    { 0x29af54b1, SUC_lexicon_tags_8 },
    { 0xc24f54b2, SUC_lexicon_tags_17 },
    { 0xb62f54b3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f54b7, SUC_lexicon_tags_37 },
    { 0x4baf54b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf16f54ba, SUC_lexicon_tags_3 },
    { 0x6aef54bb, SUC_lexicon_tags_49 },
    { 0x68f54bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff0f54be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230f54c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x184f54c7, SUC_lexicon_tags_1 },
    { 0xbe4f54c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef54cb, SUC_lexicon_tags_49 },
    { 0x750f54cc, SUC_lexicon_tags_1 },
    { 0xa6ef54cd, SUC_lexicon_tags_11 },
    { 0xa40f54cc, SUC_lexicon_tags_48 },
    { 0x124f54cf, SUC_lexicon_tags_0 },
    { 0x79ef54d0, SUC_lexicon_tags_11 },
    { 0x5f2f54d1, SUC_lexicon_tags_7 },
    { 0x850f54d2, SUC_lexicon_tags_11 },
    { 0xdb0f54d3, SUC_lexicon_tags_6 },
    { 0x644f54d4, SUC_lexicon_tags_23 },
    { 0x3ccf54cf, SUC_lexicon_tags_505 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4f54d9, SUC_lexicon_tags_10 },
    { 0x29ef54d9, SUC_lexicon_tags_39 },
    { 0x3d0f54db, SUC_lexicon_tags_1 },
    { 0xb8cf54dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x794f54df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c8f54e1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xe14f54e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaf54ea, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x6c4f54ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd0f54ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb7ef54f0, SUC_lexicon_tags_25 },
    { 0x594f54f1, SUC_lexicon_tags_38 },
    { 0x4e0f54f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bef54fd, SUC_lexicon_tags_70 },
    { 0x732f54fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0f5502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46f5504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcf5508, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806f550c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0f550f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910f5513, SUC_lexicon_tags_14 },
    { 0x132f5514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120f551d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2f5520, SUC_lexicon_tags_47 },
    { 0xa6ef5521, SUC_lexicon_tags_11 },
    { 0xef2f5522, SUC_lexicon_tags_37 },
    { 0x8a6f5520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x856f5526, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78f5529, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca4f552b, SUC_lexicon_tags_11 },
    { 0x44ef552b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccf5531, SUC_lexicon_tags_39 },
    { 0xde8f5532, SUC_lexicon_tags_70 },
    { 0x87ef5533, SUC_lexicon_tags_63 },
    { 0x132f5531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36f553f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c0f5541, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe16f5543, SUC_lexicon_tags_55 },
    { 0x428f5544, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccf5548, SUC_lexicon_tags_7 },
    { 0xaaef5549, SUC_lexicon_tags_235 },
    { 0xbf2f554a, SUC_lexicon_tags_37 },
    { 0x41ef5548, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2af554d, SUC_lexicon_tags_7 },
    { 0x63ef554e, SUC_lexicon_tags_27 },
    { 0x734f554f, SUC_lexicon_tags_84 },
    { 0x976f5550, SUC_lexicon_tags_35 },
    { 0x680f5551, SUC_lexicon_tags_25 },
    { 0x3d2f5552, SUC_lexicon_tags_37 },
    { 0x4b2f5552, SUC_lexicon_tags_25 },
    { 0x4ccf5553, SUC_lexicon_tags_37 },
    { 0xd70f5555, SUC_lexicon_tags_37 },
    { 0x5d6f5556, SUC_lexicon_tags_2 },
    { 0xd6f554d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf555b, SUC_lexicon_tags_57 },
    { 0x724f555b, SUC_lexicon_tags_11 },
    { 0x91cf555b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x722f555f, SUC_lexicon_tags_43 },
    { 0x130f555f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x79af5563, SUC_lexicon_tags_19 },
    { 0xaa4f5564, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124f556c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8f556f, SUC_lexicon_tags_63 },
    { 0x692f556f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2f5574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ef557b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1acf557d, SUC_lexicon_tags_39 },
    { 0x38f557e, SUC_lexicon_tags_19 },
    { 0xbe8f557e, SUC_lexicon_tags_31 },
    { 0xc9af557d, SUC_lexicon_tags_47 },
    { 0x8b8f557d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3f4f5583, SUC_lexicon_tags_25 },
    { 0x2b4f5584, SUC_lexicon_tags_47 },
    { 0x41cf5584, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba4f5589, SUC_lexicon_tags_9 },
    { 0xbaef558a, SUC_lexicon_tags_43 },
    { 0x95cf558b, SUC_lexicon_tags_14 },
    { 0x310f5589, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38f558f, SUC_lexicon_tags_17 },
    { 0xc88f5590, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cf5593, SUC_lexicon_tags_53 },
    { 0xa26f5593, SUC_lexicon_tags_66 },
    { 0xf22f5594, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x302f5598, SUC_lexicon_tags_4 },
    { 0x75cf5599, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954f559e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe72f55a0, SUC_lexicon_tags_25 },
    { 0x368f55a1, SUC_lexicon_tags_15 },
    { 0xa44f55a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1af55a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67af55aa, SUC_lexicon_tags_7 },
    { 0x684f55aa, SUC_lexicon_tags_1 },
    { 0xd10f55ac, SUC_lexicon_tags_57 },
    { 0xda4f55ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aef55b2, SUC_lexicon_tags_25 },
    { 0xf8f55b2, SUC_lexicon_tags_38 },
    { 0xc04f55b2, SUC_lexicon_tags_17 },
    { 0xaf0f55b3, SUC_lexicon_tags_82 },
    { 0xc50f55b6, SUC_lexicon_tags_37 },
    { 0xe98f55b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6f55bb, SUC_lexicon_tags_17 },
    { 0x494f55bc, SUC_lexicon_tags_43 },
    { 0xa14f55bc, SUC_lexicon_tags_25 },
    { 0xc7ef55bb, SUC_lexicon_tags_7 },
    { 0x8e4f55bc, SUC_lexicon_tags_23 },
    { 0xc4ef55c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ef55c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55af55c5, SUC_lexicon_tags_47 },
    { 0xfc8f55c6, SUC_lexicon_tags_12 },
    { 0x9fcf55c7, SUC_lexicon_tags_14 },
    { 0x46af55c8, SUC_lexicon_tags_37 },
    { 0xf3af55c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15af55cd, SUC_lexicon_tags_17 },
    { 0xe42f55ce, SUC_lexicon_tags_49 },
    { 0x60cf55ce, SUC_lexicon_tags_27 },
    { 0x126f55ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91cf55d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x254f55d5, SUC_lexicon_tags_12 },
    { 0x27cf55d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8f55df, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828f55e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0f55e9, SUC_lexicon_tags_37 },
    { 0x3a8f55ea, SUC_lexicon_tags_84 },
    { 0x140f55eb, SUC_lexicon_tags_25 },
    { 0x50ef55ec, SUC_lexicon_tags_1 },
    { 0x910f55ed, SUC_lexicon_tags_23 },
    { 0x600f55ed, SUC_lexicon_tags_0 },
    { 0xf50f55ed, SUC_lexicon_tags_30 },
    { 0xdaef55f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32f55f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x932f55f5, SUC_lexicon_tags_29 },
    { 0xb3ef55f5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ef55fb, SUC_lexicon_tags_15 },
    { 0x98f55fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ef5603, SUC_lexicon_tags_2 },
    { 0x46f5603, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0f5607, SUC_lexicon_tags_43 },
    { 0x6b4f5608, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0f560b, SUC_lexicon_tags_31 },
    { 0x784f560c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2f5610, SUC_lexicon_tags_27 },
    { 0xb4cf5610, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74af5616, SUC_lexicon_tags_10 },
    { 0x2d4f5617, SUC_lexicon_tags_23 },
    { 0x968f5618, SUC_lexicon_tags_114 },
    { 0x9baf5619, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaf561f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x10af5621, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f0f5623, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8f5627, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962f5630, SUC_lexicon_tags_29 },
    { 0x80f5630, SUC_lexicon_tags_19 },
    { 0x3caf5631, SUC_lexicon_tags_37 },
    { 0xdc8f5632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67af5635, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cf5639, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x39ef563b, SUC_lexicon_tags_19 },
    { 0x822f563c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0af5641, SUC_lexicon_tags_0 },
    { 0x2d8f5641, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cf5645, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76f564a, SUC_lexicon_tags_38 },
    { 0x9a0f564b, SUC_lexicon_tags_17 },
    { 0x52ef564c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x538f564e, SUC_lexicon_tags_14 },
    { 0x436f564f, SUC_lexicon_tags_11 },
    { 0x66ef564f, SUC_lexicon_tags_1 },
    { 0xccef5651, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x312f5654, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94f565c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48f5660, SUC_lexicon_tags_134 },
    { 0xc28f5661, SUC_lexicon_tags_25 },
    { 0xd2f5661, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2f5665, SUC_lexicon_tags_0 },
    { 0xafaf5666, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x380f5668, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cf567a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f567e, SUC_lexicon_tags_25 },
    { 0x182f567f, SUC_lexicon_tags_1 },
    { 0x964f5680, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x512f5683, SUC_lexicon_tags_49 },
    { 0x9bcf5684, SUC_lexicon_tags_41 },
    { 0x86ef5685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf568b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae8f568d, SUC_lexicon_tags_17 },
    { 0x342f568e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786f5692, SUC_lexicon_tags_77 },
    { 0x66ef5693, SUC_lexicon_tags_10 },
    { 0x346f5694, SUC_lexicon_tags_11 },
    { 0x406f5692, SUC_lexicon_tags_45 },
    { 0xa2f5696, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824f569a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb56f569c, SUC_lexicon_tags_31 },
    { 0x196f569d, SUC_lexicon_tags_0 },
    { 0xde0f569e, SUC_lexicon_tags_15 },
    { 0xd8af569d, SUC_lexicon_tags_15 },
    { 0x33ef569d, SUC_lexicon_tags_26 },
    { 0x8b6f56a1, SUC_lexicon_tags_37 },
    { 0x89ef569e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf56a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74ef56a8, SUC_lexicon_tags_17 },
    { 0x302f56a9, SUC_lexicon_tags_30 },
    { 0xd98f56aa, SUC_lexicon_tags_15 },
    { 0x7e6f56aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08f56b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cf56b9, SUC_lexicon_tags_14 },
    { 0xd22f56b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9acf56bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde0f56bf, SUC_lexicon_tags_15 },
    { 0xe38f56c0, SUC_lexicon_tags_23 },
    { 0x84ef56c1, SUC_lexicon_tags_2 },
    { 0x7b0f56c2, SUC_lexicon_tags_2 },
    { 0xf08f56bf, SUC_lexicon_tags_11 },
    { 0xa7af56c4, SUC_lexicon_tags_63 },
    { 0xd6cf56c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c6f56c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x522f56c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeef56cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe0ef56ce, SUC_lexicon_tags_12 },
    { 0x1e8f56cf, SUC_lexicon_tags_38 },
    { 0x6d6f56cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cf56d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x91af56d5, SUC_lexicon_tags_31 },
    { 0xe4af56d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x314f56d9, SUC_lexicon_tags_45 },
    { 0x79af56da, SUC_lexicon_tags_12 },
    { 0x792f56db, SUC_lexicon_tags_62 },
    { 0x5ecf56da, SUC_lexicon_tags_25 },
    { 0x45ef56dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x308f56df, SUC_lexicon_tags_12 },
    { 0xbe2f56e0, SUC_lexicon_tags_37 },
    { 0xf60f56e0, SUC_lexicon_tags_2 },
    { 0xa1af56e2, SUC_lexicon_tags_14 },
    { 0xa94f56e3, SUC_lexicon_tags_2 },
    { 0xb2af56e3, SUC_lexicon_tags_17 },
    { 0x446f56e3, SUC_lexicon_tags_32 },
    { 0x26cf56e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0f56eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x874f56ee, SUC_lexicon_tags_2 },
    { 0xad6f56ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26f56f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13af56f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x840f56fc, SUC_lexicon_tags_17 },
    { 0x606f56fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460f5701, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xebef5703, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0f5706, SUC_lexicon_tags_0 },
    { 0x5ccf5707, SUC_lexicon_tags_11 },
    { 0x534f5708, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faf570d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1aef570f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cef5715, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86f5719, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7daf571b, SUC_lexicon_tags_2 },
    { 0xed2f571b, SUC_lexicon_tags_23 },
    { 0x414f571d, SUC_lexicon_tags_42 },
    { 0x8f2f571e, SUC_lexicon_tags_17 },
    { 0x6a2f571f, SUC_lexicon_tags_28 },
    { 0xb84f5720, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xceaf5722, SUC_lexicon_tags_29 },
    { 0x650f5723, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cf5727, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd6cf5729, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4f572c, SUC_lexicon_tags_9 },
    { 0x642f572d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42cf5734, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0f5737, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122f573e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46af5740, SUC_lexicon_tags_2 },
    { 0x9aef5741, SUC_lexicon_tags_15 },
    { 0xc7ef5742, SUC_lexicon_tags_37 },
    { 0xc08f5743, SUC_lexicon_tags_55 },
    { 0x2d4f5744, SUC_lexicon_tags_11 },
    { 0xf08f5745, SUC_lexicon_tags_60 },
    { 0xfcef5746, SUC_lexicon_tags_31 },
    { 0xef6f5740, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dcf5749, SUC_lexicon_tags_79 },
    { 0xdeef574a, SUC_lexicon_tags_19 },
    { 0x62cf5749, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec6f5752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cf5755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb14f5763, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252f576a, SUC_lexicon_tags_11 },
    { 0x222f576b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2faf576d, SUC_lexicon_tags_12 },
    { 0x23cf576d, SUC_lexicon_tags_6 },
    { 0xe88f576f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa70f5771, SUC_lexicon_tags_48 },
    { 0x2eef5772, SUC_lexicon_tags_25 },
    { 0xde0f5772, SUC_lexicon_tags_63 },
    { 0x3e2f5771, SUC_lexicon_tags_11 },
    { 0x510f5775, SUC_lexicon_tags_12 },
    { 0x292f5776, SUC_lexicon_tags_48 },
    { 0x596f5774, SUC_lexicon_tags_37 },
    { 0x8def5778, SUC_lexicon_tags_31 },
    { 0xf9cf5779, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x204f577d, SUC_lexicon_tags_2 },
    { 0xe6cf577d, SUC_lexicon_tags_27 },
    { 0xac4f577f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faf5783, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdaf5786, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe42f5788, SUC_lexicon_tags_17 },
    { 0xdc2f5788, SUC_lexicon_tags_5 },
    { 0x3d6f5789, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x670f578d, SUC_lexicon_tags_14 },
    { 0xb42f578e, SUC_lexicon_tags_32 },
    { 0x8ecf578e, SUC_lexicon_tags_7 },
    { 0x580f5790, SUC_lexicon_tags_63 },
    { 0x976f5791, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cf5795, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8f579c, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xafcf579e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324f57a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2dcf57a7, SUC_lexicon_tags_77 },
    { 0x846f57a8, SUC_lexicon_tags_19 },
    { 0xc00f57a8, SUC_lexicon_tags_11 },
    { 0xee2f57aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe50f57ac, SUC_lexicon_tags_17 },
    { 0x18cf57ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x278f57b0, SUC_lexicon_tags_14 },
    { 0x7ef57b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaf57b8, SUC_lexicon_tags_56 },
    { 0x484f57b9, SUC_lexicon_tags_45 },
    { 0xbfcf57b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa52f57bc, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x896f57bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74f57c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf57c9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712f57cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06f57d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bcf57d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ef57dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36cf57de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaf57e1, SUC_lexicon_tags_33 },
    { 0x9baf57e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6af57e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x200f57e8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2f57ef, SUC_lexicon_tags_68 },
    { 0x718f57f0, SUC_lexicon_tags_29 },
    { 0xc04f57f1, SUC_lexicon_tags_8 },
    { 0x820f57f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf58f57f4, SUC_lexicon_tags_60 },
    { 0x2f57f4, SUC_lexicon_tags_27 },
    { 0xcf57f6, SUC_lexicon_tags_17 },
    { 0xbc0f57f7, SUC_lexicon_tags_37 },
    { 0xab6f57f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426f57ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x708f5801, SUC_lexicon_tags_23 },
    { 0xf7af5801, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cef5806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cf5811, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcf581a, SUC_lexicon_tags_32 },
    { 0xbb6f581b, SUC_lexicon_tags_63 },
    { 0xb76f581b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4f5820, SUC_lexicon_tags_11 },
    { 0xd3cf5821, SUC_lexicon_tags_1 },
    { 0xbecf5821, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x432f5824, SUC_lexicon_tags_0 },
    { 0x130f5824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed4f5827, SUC_lexicon_tags_15 },
    { 0xe16f5828, SUC_lexicon_tags_14 },
    { 0x512f5829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aef582c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x922f5830, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22f5833, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2b4f5835, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ef5839, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6ecf583b, SUC_lexicon_tags_26 },
    { 0x5b6f583b, SUC_lexicon_tags_0 },
    { 0x758f583c, SUC_lexicon_tags_43 },
    { 0xb0af583e, SUC_lexicon_tags_45 },
    { 0x172f583e, SUC_lexicon_tags_37 },
    { 0x7e4f583f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3af5846, SUC_lexicon_tags_37 },
    { 0x38af5847, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ef584a, SUC_lexicon_tags_14 },
    { 0x3a0f584b, SUC_lexicon_tags_12 },
    { 0x6cf584b, SUC_lexicon_tags_17 },
    { 0xd1cf584b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8cf584f, SUC_lexicon_tags_12 },
    { 0x2bef5850, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x198f5852, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd18f5854, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe8cf5856, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x564f585d, SUC_lexicon_tags_28 },
    { 0x846f585e, SUC_lexicon_tags_0 },
    { 0x64cf585e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x506f5861, SUC_lexicon_tags_91 },
    { 0xc9cf5861, SUC_lexicon_tags_37 },
    { 0x53cf5863, SUC_lexicon_tags_25 },
    { 0xa38f5864, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2f586a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8f586e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2af5871, SUC_lexicon_tags_26 },
    { 0x60f5871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970f587b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xad6f587d, SUC_lexicon_tags_1 },
    { 0x9d2f587d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9cef5880, SUC_lexicon_tags_12 },
    { 0xde4f5881, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcaaf5883, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaf5888, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x910f588a, SUC_lexicon_tags_43 },
    { 0xb86f588b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760f5890, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52f5897, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x87af5899, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x612f589b, SUC_lexicon_tags_10 },
    { 0xc6f589c, SUC_lexicon_tags_32 },
    { 0x890f589d, SUC_lexicon_tags_17 },
    { 0x628f589d, SUC_lexicon_tags_17 },
    { 0x8acf589d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f58aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcf8f58ac, SUC_lexicon_tags_17 },
    { 0x9b8f58ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ef58b4, SUC_lexicon_tags_6 },
    { 0xa2cf58b4, SUC_lexicon_tags_63 },
    { 0x47cf58b6, SUC_lexicon_tags_47 },
    { 0xe6ef58b7, SUC_lexicon_tags_60 },
    { 0x412f58b6, SUC_lexicon_tags_37 },
    { 0x2d4f58b7, SUC_lexicon_tags_7 },
    { 0x7c2f58b4, SUC_lexicon_tags_28 },
    { 0xf64f58b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70f58be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x562f58c0, SUC_lexicon_tags_45 },
    { 0x7cf58c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8f58c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8f58ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9faf58d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73af58d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d4f58d7, SUC_lexicon_tags_11 },
    { 0x632f58d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbdef58da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x240f58dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x14ef58df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cf58e8, SUC_lexicon_tags_0 },
    { 0x518f58e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcecf58eb, SUC_lexicon_tags_15 },
    { 0x2d2f58eb, SUC_lexicon_tags_23 },
    { 0x2d4f58ed, SUC_lexicon_tags_7 },
    { 0x596f58ed, SUC_lexicon_tags_7 },
    { 0x7e0f58ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ef58f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2f58f9, SUC_lexicon_tags_25 },
    { 0x7fef58fa, SUC_lexicon_tags_11 },
    { 0x9bcf58fa, SUC_lexicon_tags_15 },
    { 0xc54f58fb, SUC_lexicon_tags_7 },
    { 0x864f58fd, SUC_lexicon_tags_14 },
    { 0x2f6f58fd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfef5903, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0f5907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9af5909, SUC_lexicon_tags_38 },
    { 0xd56f590a, SUC_lexicon_tags_11 },
    { 0xd82f590b, SUC_lexicon_tags_1 },
    { 0x110f5909, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80cf590e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x552f5911, SUC_lexicon_tags_33 },
    { 0x590f5912, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f591c, SUC_lexicon_tags_77 },
    { 0xc84f591d, SUC_lexicon_tags_37 },
    { 0x36cf591d, SUC_lexicon_tags_63 },
    { 0x374f591d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c8f5921, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x784f5924, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910f5928, SUC_lexicon_tags_7 },
    { 0x178f5929, SUC_lexicon_tags_19 },
    { 0xf5af592a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaf5932, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1daf5934, SUC_lexicon_tags_12 },
    { 0x592f5935, SUC_lexicon_tags_7 },
    { 0x55ef5934, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2f593c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5af593e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314f5942, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0f5946, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x23cf5948, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2f594b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x68ef594d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe2ef594f, SUC_lexicon_tags_7 },
    { 0xb56f594f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84f5954, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x822f5957, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdd8f5959, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38f595d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530f5966, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4f5969, SUC_lexicon_tags_2 },
    { 0xdcef596a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddef596f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8f5972, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7faf5974, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d2f5976, SUC_lexicon_tags_37 },
    { 0x57cf5976, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd80f597a, SUC_lexicon_tags_7 },
    { 0x360f597b, SUC_lexicon_tags_8 },
    { 0x40f597c, SUC_lexicon_tags_7 },
    { 0xbf2f597d, SUC_lexicon_tags_46 },
    { 0xe1af597a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd70f5980, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc90f5982, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaf598b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x806f598d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae6f598f, SUC_lexicon_tags_15 },
    { 0xce6f5990, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97af599f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cef59a7, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4f59aa, SUC_lexicon_tags_11 },
    { 0x180f59ab, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x3ecf59ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52f59b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5af59b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cf59ba, SUC_lexicon_tags_26 },
    { 0x31cf59bb, SUC_lexicon_tags_17 },
    { 0x1af59bc, SUC_lexicon_tags_26 },
    { 0x6b4f59bd, SUC_lexicon_tags_11 },
    { 0x79af59be, SUC_lexicon_tags_23 },
    { 0x9fef59bf, SUC_lexicon_tags_30 },
    { 0x642f59ba, SUC_lexicon_tags_49 },
    { 0x4cef59bb, SUC_lexicon_tags_1 },
    { 0xd3cf59bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106f59ca, SUC_lexicon_tags_43 },
    { 0x19af59cb, SUC_lexicon_tags_2 },
    { 0x1b4f59ca, SUC_lexicon_tags_49 },
    { 0x9b6f59cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ef59d7, SUC_lexicon_tags_12 },
    { 0x6f2f59d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5baf59df, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6f59e4, SUC_lexicon_tags_17 },
    { 0x69cf59e5, SUC_lexicon_tags_12 },
    { 0x678f59e5, SUC_lexicon_tags_2 },
    { 0x4aaf59e7, SUC_lexicon_tags_21 },
    { 0x324f59e7, SUC_lexicon_tags_19 },
    { 0x5caf59e8, SUC_lexicon_tags_43 },
    { 0x55ef59ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ef59f1, SUC_lexicon_tags_15 },
    { 0xfb4f59f1, SUC_lexicon_tags_17 },
    { 0x75ef59f2, SUC_lexicon_tags_14 },
    { 0x54ef59f4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcfaf59f6, SUC_lexicon_tags_60 },
    { 0x696f59f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdeaf59f9, SUC_lexicon_tags_7 },
    { 0x9cef59fa, SUC_lexicon_tags_0 },
    { 0x3f6f59fb, SUC_lexicon_tags_11 },
    { 0xab8f59fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe62f59fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2af5a03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b2f5a05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6f5a08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650f5a0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46af5a1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe06f5a1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5baf5a21, SUC_lexicon_tags_23 },
    { 0x2f6f5a22, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f5a28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63af5a2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x176f5a2d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982f5a31, SUC_lexicon_tags_2 },
    { 0x312f5a32, SUC_lexicon_tags_27 },
    { 0x4e0f5a32, SUC_lexicon_tags_13 },
    { 0x9d6f5a33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672f5a3b, SUC_lexicon_tags_12 },
    { 0xd9af5a3c, SUC_lexicon_tags_38 },
    { 0xce8f5a3d, SUC_lexicon_tags_64 },
    { 0xba2f5a3e, SUC_lexicon_tags_213 },
    { 0x416f5a3d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfdaf5a41, SUC_lexicon_tags_19 },
    { 0x196f5a42, SUC_lexicon_tags_19 },
    { 0xd3cf5a43, SUC_lexicon_tags_33 },
    { 0xd6cf5a43, SUC_lexicon_tags_2 },
    { 0x4e8f5a45, SUC_lexicon_tags_11 },
    { 0xe4f5a46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x384f5a48, SUC_lexicon_tags_63 },
    { 0x93af5a49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa56f5a4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x994f5a4d, SUC_lexicon_tags_1 },
    { 0xc98f5a4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ef5a54, SUC_lexicon_tags_17 },
    { 0x604f5a55, SUC_lexicon_tags_12 },
    { 0x76af5a55, SUC_lexicon_tags_43 },
    { 0x2c4f5a55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa0af5a59, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd0f5a5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41af5a5d, SUC_lexicon_tags_1 },
    { 0xf2cf5a5d, SUC_lexicon_tags_19 },
    { 0x222f5a5f, SUC_lexicon_tags_28 },
    { 0x4f2f5a60, SUC_lexicon_tags_79 },
    { 0xcb2f5a60, SUC_lexicon_tags_3 },
    { 0xfbef5a62, SUC_lexicon_tags_25 },
    { 0xc72f5a61, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x152f5a65, SUC_lexicon_tags_38 },
    { 0x998f5a66, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34f5a6c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ef5a6f, SUC_lexicon_tags_2 },
    { 0xf12f5a70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498f5a74, SUC_lexicon_tags_7 },
    { 0xbb4f5a74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf62f5a77, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x230f5a7a, SUC_lexicon_tags_37 },
    { 0x1b4f5a7b, SUC_lexicon_tags_44 },
    { 0xf8ef5a7b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa94f5a7f, SUC_lexicon_tags_23 },
    { 0x344f5a80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22cf5a88, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110f5a8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x332f5a91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd56f5a93, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aaf5a9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2f5aa0, SUC_lexicon_tags_87 },
    { 0x784f5aa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf5aa4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162f5aa8, SUC_lexicon_tags_8 },
    { 0x45ef5aa9, SUC_lexicon_tags_7 },
    { 0xb0f5aaa, SUC_lexicon_tags_0 },
    { 0xeb2f5aa9, SUC_lexicon_tags_43 },
    { 0x6e4f5aac, SUC_lexicon_tags_1 },
    { 0x494f5aad, SUC_lexicon_tags_25 },
    { 0x4bef5aae, SUC_lexicon_tags_17 },
    { 0x904f5aae, SUC_lexicon_tags_84 },
    { 0x9b4f5ab0, SUC_lexicon_tags_7 },
    { 0xaf8f5ab1, SUC_lexicon_tags_215 },
    { 0xd70f5ab2, SUC_lexicon_tags_40 },
    { 0x466f5ab2, SUC_lexicon_tags_2 },
    { 0x6ccf5ab2, SUC_lexicon_tags_37 },
    { 0x72cf5aad, SUC_lexicon_tags_25 },
    { 0xad4f5aae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264f5aba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728f5ac0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf0cf5ac2, SUC_lexicon_tags_37 },
    { 0x784f5ac2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35af5ac6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ef5acb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6f5ad0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaf5ad8, SUC_lexicon_tags_37 },
    { 0x142f5ad9, SUC_lexicon_tags_2 },
    { 0xbcef5ada, SUC_lexicon_tags_37 },
    { 0x328f5adb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1af5ae1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738f5ae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaef5aec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aef5aef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x27cf5af1, SUC_lexicon_tags_63 },
    { 0x58af5af2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1c2f5af4, SUC_lexicon_tags_45 },
    { 0x988f5af5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106f5afb, SUC_lexicon_tags_12 },
    { 0x848f5afc, SUC_lexicon_tags_37 },
    { 0x1d2f5afd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cf5b01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472f5b05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720f5b09, SUC_lexicon_tags_37 },
    { 0xceaf5b0a, SUC_lexicon_tags_25 },
    { 0x6bcf5b0a, SUC_lexicon_tags_27 },
    { 0x5b4f5b0b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6d4f5b0e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf46f5b10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cf5b16, SUC_lexicon_tags_63 },
    { 0x5eaf5b16, SUC_lexicon_tags_2 },
    { 0x936f5b18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70f5b1b, SUC_lexicon_tags_45 },
    { 0x97af5b1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x672f5b1e, SUC_lexicon_tags_23 },
    { 0x72af5b1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ef5b26, SUC_lexicon_tags_11 },
    { 0x246f5b27, SUC_lexicon_tags_12 },
    { 0xa5af5b28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9eaf5b2a, SUC_lexicon_tags_61 },
    { 0x872f5b2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0f5b30, SUC_lexicon_tags_32 },
    { 0x7ef5b31, SUC_lexicon_tags_19 },
    { 0x3bef5b32, SUC_lexicon_tags_39 },
    { 0xb34f5b31, SUC_lexicon_tags_25 },
    { 0xab0f5b34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6f5b37, SUC_lexicon_tags_2 },
    { 0x2e2f5b37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x810f5b3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x174f5b3d, SUC_lexicon_tags_26 },
    { 0xb4cf5b3d, SUC_lexicon_tags_95 },
    { 0x440f5b3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x154f5b4d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x16af5b50, SUC_lexicon_tags_9 },
    { 0xe0ef5b51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ef5b55, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xedaf5b57, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x62ef5b59, SUC_lexicon_tags_38 },
    { 0x362f5b5a, SUC_lexicon_tags_63 },
    { 0x760f5b59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6f5b65, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ef5b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2f5b6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2f5b73, SUC_lexicon_tags_151 },
    { 0x670f5b73, SUC_lexicon_tags_14 },
    { 0xfacf5b75, SUC_lexicon_tags_27 },
    { 0x86f5b76, SUC_lexicon_tags_485 },
    { 0x606f5b76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x980f5b7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f5b80, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc46f5b82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4f5b8a, SUC_lexicon_tags_48 },
    { 0xcfef5b8a, SUC_lexicon_tags_37 },
    { 0xdeaf5b8b, SUC_lexicon_tags_33 },
    { 0x682f5b8d, SUC_lexicon_tags_17 },
    { 0x8fef5b8b, SUC_lexicon_tags_25 },
    { 0x2ef5b8f, SUC_lexicon_tags_16 },
    { 0x7d8f5b90, SUC_lexicon_tags_2 },
    { 0x57ef5b8a, SUC_lexicon_tags_25 },
    { 0x6acf5b91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x350f5b94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ef5b97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1a0f5b99, SUC_lexicon_tags_2 },
    { 0x120f5b9a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd40f5b9c, SUC_lexicon_tags_11 },
    { 0x13cf5b9d, SUC_lexicon_tags_48 },
    { 0x4bcf5b9d, SUC_lexicon_tags_38 },
    { 0x160f5b9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a8f5ba1, SUC_lexicon_tags_27 },
    { 0xbf2f5ba2, SUC_lexicon_tags_19 },
    { 0x272f5ba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0f5ba9, SUC_lexicon_tags_12 },
    { 0x658f5baa, SUC_lexicon_tags_49 },
    { 0x38ef5bab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdd4f5bad, SUC_lexicon_tags_69 },
    { 0xc5ef5bae, SUC_lexicon_tags_25 },
    { 0x3b0f5baf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36f5bbb, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664f5bbf, SUC_lexicon_tags_38 },
    { 0xd54f5bbf, SUC_lexicon_tags_17 },
    { 0x958f5bc1, SUC_lexicon_tags_12 },
    { 0xff0f5bc2, SUC_lexicon_tags_14 },
    { 0x552f5bc3, SUC_lexicon_tags_7 },
    { 0x88af5bc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f5bca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306f5bd0, SUC_lexicon_tags_65 },
    { 0x17af5bd1, SUC_lexicon_tags_27 },
    { 0x184f5bd2, SUC_lexicon_tags_56 },
    { 0x67af5bd2, SUC_lexicon_tags_31 },
    { 0x496f5bd4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x264f5bd7, SUC_lexicon_tags_951 },
    { 0x774f5bd8, SUC_lexicon_tags_15 },
    { 0xf8ef5bd9, SUC_lexicon_tags_1 },
    { 0x76f5bda, SUC_lexicon_tags_7 },
    { 0xf92f5bda, SUC_lexicon_tags_91 },
    { 0x6a8f5bd8, SUC_lexicon_tags_2 },
    { 0x342f5bdd, SUC_lexicon_tags_12 },
    { 0xe14f5bd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb48f5be0, SUC_lexicon_tags_0 },
    { 0xad4f5be0, SUC_lexicon_tags_12 },
    { 0xeecf5be0, SUC_lexicon_tags_68 },
    { 0x5aaf5be1, SUC_lexicon_tags_21 },
    { 0xb56f5be0, SUC_lexicon_tags_14 },
    { 0xd84f5be5, SUC_lexicon_tags_46 },
    { 0xefcf5be4, SUC_lexicon_tags_11 },
    { 0xb34f5be5, SUC_lexicon_tags_61 },
    { 0xbcaf5be3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x218f5bea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18f5bf0, SUC_lexicon_tags_47 },
    { 0xa02f5bf0, SUC_lexicon_tags_12 },
    { 0xba0f5bf2, SUC_lexicon_tags_65 },
    { 0x48ef5bf3, SUC_lexicon_tags_12 },
    { 0xe48f5bf3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72f5bf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cef5bfa, SUC_lexicon_tags_12 },
    { 0x772f5bfb, SUC_lexicon_tags_37 },
    { 0xdcef5bfc, SUC_lexicon_tags_11 },
    { 0xf06f5bfc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xba0f5bff, SUC_lexicon_tags_15 },
    { 0xf7cf5c00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeeaf5c02, SUC_lexicon_tags_45 },
    { 0xaa2f5c02, SUC_lexicon_tags_68 },
    { 0xe20f5c03, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248f5c08, SUC_lexicon_tags_60 },
    { 0xb2ef5c09, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cf5c0d, SUC_lexicon_tags_65 },
    { 0x18f5c0e, SUC_lexicon_tags_24 },
    { 0x19ef5c0e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58f5c12, SUC_lexicon_tags_17 },
    { 0x66ef5c13, SUC_lexicon_tags_10 },
    { 0xde0f5c12, SUC_lexicon_tags_77 },
    { 0x5ccf5c15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0cf5c17, SUC_lexicon_tags_15 },
    { 0x400f5c17, SUC_lexicon_tags_7 },
    { 0xe6ef5c19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4f5c1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cf5c20, SUC_lexicon_tags_0 },
    { 0x748f5c21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf5c24, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x368f5c26, SUC_lexicon_tags_17 },
    { 0x628f5c26, SUC_lexicon_tags_230 },
    { 0xdaf5c27, SUC_lexicon_tags_2 },
    { 0x166f5c29, SUC_lexicon_tags_63 },
    { 0xc2f5c29, SUC_lexicon_tags_17 },
    { 0xf5ef5c2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948f5c2f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1bef5c31, SUC_lexicon_tags_25 },
    { 0x97ef5c32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452f5c38, SUC_lexicon_tags_11 },
    { 0x9ecf5c38, SUC_lexicon_tags_17 },
    { 0x85ef5c39, SUC_lexicon_tags_11 },
    { 0x352f5c3b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc2cf5c3d, SUC_lexicon_tags_85 },
    { 0xb76f5c3e, SUC_lexicon_tags_48 },
    { 0x60f5c3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x232f5c42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46f5c46, SUC_lexicon_tags_4 },
    { 0x386f5c46, SUC_lexicon_tags_45 },
    { 0x76cf5c48, SUC_lexicon_tags_25 },
    { 0x71af5c48, SUC_lexicon_tags_11 },
    { 0x380f5c4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cf5c53, SUC_lexicon_tags_17 },
    { 0x996f5c54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2af5c58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdef5c5a, SUC_lexicon_tags_104 },
    { 0x5ccf5c5b, SUC_lexicon_tags_7 },
    { 0x49af5c5c, SUC_lexicon_tags_16 },
    { 0x4e8f5c5c, SUC_lexicon_tags_37 },
    { 0x90af5c5e, SUC_lexicon_tags_47 },
    { 0xf98f5c5b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842f5c63, SUC_lexicon_tags_87 },
    { 0xed4f5c64, SUC_lexicon_tags_6 },
    { 0xe70f5c65, SUC_lexicon_tags_23 },
    { 0x2e4f5c66, SUC_lexicon_tags_16 },
    { 0x1e0f5c66, SUC_lexicon_tags_23 },
    { 0x712f5c68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4f5c6a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xdc2f5c6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cf5c71, SUC_lexicon_tags_25 },
    { 0xfbef5c71, SUC_lexicon_tags_25 },
    { 0xdb0f5c72, SUC_lexicon_tags_26 },
    { 0xc18f5c74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x372f5c76, SUC_lexicon_tags_17 },
    { 0x11af5c77, SUC_lexicon_tags_12 },
    { 0x61cf5c78, SUC_lexicon_tags_63 },
    { 0x6f5c77, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4f5c7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x11af5c81, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192f5c8f, SUC_lexicon_tags_12 },
    { 0x674f5c90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecf5c98, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cf5c9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5a6f5ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91af5ca2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa66f5ca4, SUC_lexicon_tags_55 },
    { 0xa26f5ca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x604f5ca8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4f5cad, SUC_lexicon_tags_3 },
    { 0x6eef5cae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x598f5cb1, SUC_lexicon_tags_2 },
    { 0x9e8f5cb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4f5cb9, SUC_lexicon_tags_44 },
    { 0x3b8f5cb9, SUC_lexicon_tags_11 },
    { 0x8ccf5cba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f6f5cbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf5cc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2f5cc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6f5cce, SUC_lexicon_tags_26 },
    { 0xe3cf5cce, SUC_lexicon_tags_32 },
    { 0x6b6f5cd0, SUC_lexicon_tags_17 },
    { 0xfaef5cd1, SUC_lexicon_tags_38 },
    { 0xbc0f5cd2, SUC_lexicon_tags_25 },
    { 0x4acf5cd3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef5cd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620f5cde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbdcf5ce0, SUC_lexicon_tags_31 },
    { 0x5dcf5ce1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6f5ce9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x222f5ceb, SUC_lexicon_tags_0 },
    { 0x6ccf5ceb, SUC_lexicon_tags_11 },
    { 0x992f5ced, SUC_lexicon_tags_26 },
    { 0xcacf5cee, SUC_lexicon_tags_43 },
    { 0x64ef5cef, SUC_lexicon_tags_13 },
    { 0x878f5cf0, SUC_lexicon_tags_23 },
    { 0x6aef5cf1, SUC_lexicon_tags_4 },
    { 0xffaf5cef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12af5cfa, SUC_lexicon_tags_19 },
    { 0x22cf5cfa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x634f5cfd, SUC_lexicon_tags_37 },
    { 0x728f5cfe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaf5d04, SUC_lexicon_tags_476 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370f5d0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dcf5d0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x810f5d11, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f5d18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86f5d1a, SUC_lexicon_tags_10 },
    { 0x59ef5d1a, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaf5d23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x20ef5d25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbef5d2b, SUC_lexicon_tags_32 },
    { 0x55cf5d2b, SUC_lexicon_tags_82 },
    { 0x58f5d2d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ef5d30, SUC_lexicon_tags_11 },
    { 0xc9ef5d30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x14f5d33, SUC_lexicon_tags_1 },
    { 0x5d6f5d34, SUC_lexicon_tags_11 },
    { 0xa86f5d35, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804f5d3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x636f5d3f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1af5d42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd86f5d45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50f5d4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfb4f5d51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x628f5d53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf6f5d55, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa4af5d57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78f5d5d, SUC_lexicon_tags_21 },
    { 0xc4ef5d5d, SUC_lexicon_tags_1 },
    { 0xc72f5d5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ef5d63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd18f5d65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5a6f5d67, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840f5d6b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcf5d76, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798f5d7d, SUC_lexicon_tags_17 },
    { 0x6b2f5d7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08f5d81, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824f5d8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916f5d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ef5d9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d8f5d9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92f5d9f, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32f5da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf5dab, SUC_lexicon_tags_30 },
    { 0xa3af5dab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64f5daf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0f5db3, SUC_lexicon_tags_27 },
    { 0xfb2f5db3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaf5db9, SUC_lexicon_tags_17 },
    { 0xd04f5db9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ef5dbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60f5dc2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc8f5dc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46f5dc9, SUC_lexicon_tags_45 },
    { 0xb9cf5dca, SUC_lexicon_tags_17 },
    { 0xc78f5dcb, SUC_lexicon_tags_47 },
    { 0x88cf5dcc, SUC_lexicon_tags_1 },
    { 0x924f5dcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef5dd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x556f5dd3, SUC_lexicon_tags_60 },
    { 0x38af5dd4, SUC_lexicon_tags_0 },
    { 0x896f5dd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89cf5dd8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x392f5dda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89af5ddd, SUC_lexicon_tags_2 },
    { 0x57ef5ddd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bef5de1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6f5de4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04f5ded, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaf5df3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cf5df7, SUC_lexicon_tags_25 },
    { 0x7f4f5df7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaf5dff, SUC_lexicon_tags_7 },
    { 0xc96f5e00, SUC_lexicon_tags_53 },
    { 0xdecf5dff, SUC_lexicon_tags_7 },
    { 0xdb8f5dff, SUC_lexicon_tags_37 },
    { 0x1acf5e03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7aaf5e05, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ef5e08, SUC_lexicon_tags_25 },
    { 0x2a2f5e08, SUC_lexicon_tags_12 },
    { 0x9bcf5e0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86cf5e0c, SUC_lexicon_tags_32 },
    { 0x316f5e0c, SUC_lexicon_tags_25 },
    { 0xf34f5e0e, SUC_lexicon_tags_813 },
    { 0xd74f5e0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9caf5e11, SUC_lexicon_tags_11 },
    { 0x988f5e11, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49af5e18, SUC_lexicon_tags_12 },
    { 0xbb4f5e19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcf5e1e, SUC_lexicon_tags_11 },
    { 0xb76f5e1f, SUC_lexicon_tags_17 },
    { 0x12ef5e1f, SUC_lexicon_tags_1 },
    { 0xcef5e21, SUC_lexicon_tags_43 },
    { 0x798f5e22, SUC_lexicon_tags_14 },
    { 0x5daf5e23, SUC_lexicon_tags_30 },
    { 0x6daf5e23, SUC_lexicon_tags_108 },
    { 0x830f5e22, SUC_lexicon_tags_2 },
    { 0x7ccf5e26, SUC_lexicon_tags_79 },
    { 0x848f5e27, SUC_lexicon_tags_37 },
    { 0x93cf5e28, SUC_lexicon_tags_2 },
    { 0x12f5e29, SUC_lexicon_tags_87 },
    { 0x794f5e28, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x6c8f5e2c, SUC_lexicon_tags_2 },
    { 0x744f5e2d, SUC_lexicon_tags_26 },
    { 0xb8cf5e2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2f5e32, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcf5e3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf46f5e42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90f5e45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ecf5e47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x284f5e4a, SUC_lexicon_tags_33 },
    { 0xa16f5e4a, SUC_lexicon_tags_25 },
    { 0x5dcf5e4c, SUC_lexicon_tags_37 },
    { 0xd50f5e4a, SUC_lexicon_tags_15 },
    { 0xaf6f5e4e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xadef5e50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eef5e56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x508f5e58, SUC_lexicon_tags_12 },
    { 0x14af5e59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72f5e5b, SUC_lexicon_tags_64 },
    { 0x4f6f5e5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x588f5e5e, SUC_lexicon_tags_15 },
    { 0xb0af5e5f, SUC_lexicon_tags_30 },
    { 0x71cf5e5f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240f5e67, SUC_lexicon_tags_11 },
    { 0x8bef5e68, SUC_lexicon_tags_2 },
    { 0xa86f5e69, SUC_lexicon_tags_1 },
    { 0x25af5e6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa70f5e6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcf5e6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd52f5e71, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cf5e76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a0f5e83, SUC_lexicon_tags_1 },
    { 0x7d6f5e84, SUC_lexicon_tags_23 },
    { 0x4dcf5e83, SUC_lexicon_tags_27 },
    { 0xc6cf5e86, SUC_lexicon_tags_33 },
    { 0x542f5e87, SUC_lexicon_tags_17 },
    { 0xa4af5e87, SUC_lexicon_tags_23 },
    { 0xdbef5e88, SUC_lexicon_tags_27 },
    { 0x568f5e8a, SUC_lexicon_tags_10 },
    { 0x3af5e8b, SUC_lexicon_tags_17 },
    { 0x344f5e8a, SUC_lexicon_tags_37 },
    { 0x786f5e89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68cf5e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82f5e98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06f5e9b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x94ef5e9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa40f5e9f, SUC_lexicon_tags_11 },
    { 0xfcef5ea0, SUC_lexicon_tags_68 },
    { 0x104f5ea1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2f5ea4, SUC_lexicon_tags_47 },
    { 0xfd0f5ea5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fef5eab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2f5eb0, SUC_lexicon_tags_11 },
    { 0x164f5eb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e4f5eb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb6f5eb5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x35ef5eb7, SUC_lexicon_tags_7 },
    { 0x64cf5eb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecf5ebd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bcf5ebf, SUC_lexicon_tags_4 },
    { 0xc34f5ec0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb66f5ec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ef5ec7, SUC_lexicon_tags_0 },
    { 0xe52f5ec8, SUC_lexicon_tags_1 },
    { 0xbe4f5ec8, SUC_lexicon_tags_1 },
    { 0x57ef5ec8, SUC_lexicon_tags_0 },
    { 0xd8ef5ecb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x992f5ecd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x138f5ecf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x526f5ed1, SUC_lexicon_tags_52 },
    { 0xeb2f5ed2, SUC_lexicon_tags_7 },
    { 0x886f5ed2, SUC_lexicon_tags_45 },
    { 0x578f5ed4, SUC_lexicon_tags_38 },
    { 0x49cf5ed4, SUC_lexicon_tags_82 },
    { 0xea0f5ed4, SUC_lexicon_tags_7 },
    { 0x376f5ed1, SUC_lexicon_tags_66 },
    { 0x58cf5ed4, SUC_lexicon_tags_30 },
    { 0xd7af5ed9, SUC_lexicon_tags_1 },
    { 0x19af5eda, SUC_lexicon_tags_25 },
    { 0xc92f5eda, SUC_lexicon_tags_57 },
    { 0x544f5edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7af5ee3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84af5ee5, SUC_lexicon_tags_11 },
    { 0xd78f5ee6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2aef5ee8, SUC_lexicon_tags_37 },
    { 0xf8f5ee9, SUC_lexicon_tags_12 },
    { 0x2b6f5eea, SUC_lexicon_tags_17 },
    { 0xb7ef5ee9, SUC_lexicon_tags_37 },
    { 0x534f5eec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x27af5eee, SUC_lexicon_tags_2 },
    { 0x29ef5eef, SUC_lexicon_tags_16 },
    { 0x294f5eef, SUC_lexicon_tags_21 },
    { 0x3eef5ef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d2f5ef3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652f5efa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x936f5efc, SUC_lexicon_tags_12 },
    { 0x634f5efd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x922f5eff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe18f5f02, SUC_lexicon_tags_25 },
    { 0x908f5f02, SUC_lexicon_tags_26 },
    { 0x348f5f04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x510f5f06, SUC_lexicon_tags_77 },
    { 0x568f5f07, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8f5f0b, SUC_lexicon_tags_26 },
    { 0x2def5f0c, SUC_lexicon_tags_26 },
    { 0xff0f5f0c, SUC_lexicon_tags_37 },
    { 0x130f5f0c, SUC_lexicon_tags_25 },
    { 0x746f5f0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6f5f1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ef5f1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2d4f5f1f, SUC_lexicon_tags_43 },
    { 0x2acf5f1f, SUC_lexicon_tags_35 },
    { 0xa52f5f21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x526f5f24, SUC_lexicon_tags_37 },
    { 0x4b0f5f24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x412f5f27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ef5f2e, SUC_lexicon_tags_29 },
    { 0xcf8f5f2e, SUC_lexicon_tags_33 },
    { 0xb10f5f30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120f5f37, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756f5f3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x560f5f41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x106f5f43, SUC_lexicon_tags_26 },
    { 0xe20f5f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eaf5f47, SUC_lexicon_tags_37 },
    { 0x5b6f5f47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc0af5f4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x896f5f4c, SUC_lexicon_tags_25 },
    { 0xbc8f5f4d, SUC_lexicon_tags_28 },
    { 0x60af5f4e, SUC_lexicon_tags_12 },
    { 0x602f5f4f, SUC_lexicon_tags_65 },
    { 0x30af5f4f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x36f5f52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa66f5f54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98f5f5d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cf5f63, SUC_lexicon_tags_11 },
    { 0xdecf5f63, SUC_lexicon_tags_4 },
    { 0x5d0f5f63, SUC_lexicon_tags_46 },
    { 0xd2ef5f64, SUC_lexicon_tags_26 },
    { 0x37ef5f66, SUC_lexicon_tags_15 },
    { 0xb5cf5f68, SUC_lexicon_tags_51 },
    { 0x452f5f69, SUC_lexicon_tags_25 },
    { 0x9cf5f6a, SUC_lexicon_tags_15 },
    { 0xc5cf5f6a, SUC_lexicon_tags_7 },
    { 0xf48f5f66, SUC_lexicon_tags_15 },
    { 0xe98f5f67, SUC_lexicon_tags_8 },
    { 0xee8f5f6e, SUC_lexicon_tags_4 },
    { 0x3a8f5f6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2d6f5f71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794f5f76, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46f5f7c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf0af5f7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94cf5f80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x638f5f82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acf5f87, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46f5f8c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14af5f94, SUC_lexicon_tags_64 },
    { 0x168f5f94, SUC_lexicon_tags_38 },
    { 0x546f5f94, SUC_lexicon_tags_45 },
    { 0x8ecf5f97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4f5f9c, SUC_lexicon_tags_17 },
    { 0xe56f5f9c, SUC_lexicon_tags_1 },
    { 0x688f5f9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb2f5fa0, SUC_lexicon_tags_37 },
    { 0x12af5fa1, SUC_lexicon_tags_9 },
    { 0x2c8f5fa2, SUC_lexicon_tags_10 },
    { 0x4a6f5fa1, SUC_lexicon_tags_25 },
    { 0xe32f5fa2, SUC_lexicon_tags_15 },
    { 0x6acf5fa5, SUC_lexicon_tags_34 },
    { 0x8eef5fa1, SUC_lexicon_tags_34 },
    { 0x660f5fa7, SUC_lexicon_tags_15 },
    { 0x7c4f5fa7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92af5fac, SUC_lexicon_tags_29 },
    { 0x6e4f5fad, SUC_lexicon_tags_8 },
    { 0xc9cf5fae, SUC_lexicon_tags_37 },
    { 0xb16f5faf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cf5fb2, SUC_lexicon_tags_38 },
    { 0x474f5fb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4f5fb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24f5fba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf5fbd, SUC_lexicon_tags_87 },
    { 0x3a0f5fbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85cf5fc1, SUC_lexicon_tags_17 },
    { 0x4caf5fc2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72f5fc5, SUC_lexicon_tags_9 },
    { 0x730f5fc6, SUC_lexicon_tags_43 },
    { 0x562f5fc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00f5fcc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x42cf5fce, SUC_lexicon_tags_0 },
    { 0xb56f5fcf, SUC_lexicon_tags_17 },
    { 0x7c0f5fcf, SUC_lexicon_tags_29 },
    { 0x452f5fd1, SUC_lexicon_tags_12 },
    { 0x1a2f5fcf, SUC_lexicon_tags_2 },
    { 0xdc4f5fd2, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x474f5fd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58f5fda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x186f5fe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8f5fe7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bef5fed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29af5ff2, SUC_lexicon_tags_28 },
    { 0xf5ff3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4e2f5ff5, SUC_lexicon_tags_19 },
    { 0x8e4f5ff6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8af5ff8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ef5ffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x376f5fff, SUC_lexicon_tags_47 },
    { 0x902f5fff, SUC_lexicon_tags_14 },
    { 0xfd0f6001, SUC_lexicon_tags_19 },
    { 0x24af6002, SUC_lexicon_tags_19 },
    { 0x1b6f6003, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4f600a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ef600e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x68af6011, SUC_lexicon_tags_7 },
    { 0x37af6012, SUC_lexicon_tags_25 },
    { 0xdaf6012, SUC_lexicon_tags_11 },
    { 0x9e8f6011, SUC_lexicon_tags_108 },
    { 0x31cf6015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fef6017, SUC_lexicon_tags_11 },
    { 0x5f8f6018, SUC_lexicon_tags_25 },
    { 0xd48f6019, SUC_lexicon_tags_85 },
    { 0xb1af601a, SUC_lexicon_tags_17 },
    { 0xfc6f6018, SUC_lexicon_tags_23 },
    { 0xaf4f601a, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80f6021, SUC_lexicon_tags_30 },
    { 0x10cf6022, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2f602f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x176f6031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4f6037, SUC_lexicon_tags_39 },
    { 0x5f4f6038, SUC_lexicon_tags_26 },
    { 0x696f6039, SUC_lexicon_tags_12 },
    { 0x642f6039, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb74f603c, SUC_lexicon_tags_4 },
    { 0xfe6f603d, SUC_lexicon_tags_82 },
    { 0xc6f603d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4f6049, SUC_lexicon_tags_45 },
    { 0x5c6f6049, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x844f604c, SUC_lexicon_tags_56 },
    { 0xfb0f604d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6f6050, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912f6054, SUC_lexicon_tags_64 },
    { 0x178f6054, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbacf605b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0f6062, SUC_lexicon_tags_27 },
    { 0x276f6062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8f606e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa0f6070, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ef6072, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x99af6074, SUC_lexicon_tags_65 },
    { 0xec8f6074, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a0f6079, SUC_lexicon_tags_31 },
    { 0x370f6079, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeecf607c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a2f607e, SUC_lexicon_tags_37 },
    { 0x96cf607e, SUC_lexicon_tags_1 },
    { 0xe74f6080, SUC_lexicon_tags_12 },
    { 0xc36f6081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56f608a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734f608f, SUC_lexicon_tags_37 },
    { 0xbecf6090, SUC_lexicon_tags_23 },
    { 0x2b4f6091, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8f6096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x696f6099, SUC_lexicon_tags_15 },
    { 0x5e2f609a, SUC_lexicon_tags_15 },
    { 0x304f609b, SUC_lexicon_tags_11 },
    { 0xc48f609c, SUC_lexicon_tags_39 },
    { 0x5dcf609d, SUC_lexicon_tags_65 },
    { 0x90af609e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcf60a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf58f60a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x256f60a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x158f60a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x676f60aa, SUC_lexicon_tags_12 },
    { 0xdd6f60aa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x226f60ad, SUC_lexicon_tags_14 },
    { 0x58ef60ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc62f60b0, SUC_lexicon_tags_7 },
    { 0xaf4f60b0, SUC_lexicon_tags_7 },
    { 0x8c8f60b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222f60b7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2f60bb, SUC_lexicon_tags_55 },
    { 0xa64f60bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x334f60be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196f60c2, SUC_lexicon_tags_27 },
    { 0x5fcf60c2, SUC_lexicon_tags_9 },
    { 0xc90f60c4, SUC_lexicon_tags_55 },
    { 0xebcf60c5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x94af60c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8daf60ca, SUC_lexicon_tags_12 },
    { 0xda4f60ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa34f60cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52af60cf, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x632f60d1, SUC_lexicon_tags_12 },
    { 0xb9af60d2, SUC_lexicon_tags_27 },
    { 0x422f60d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4f60d6, SUC_lexicon_tags_12 },
    { 0xa2ef60d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0f60da, SUC_lexicon_tags_47 },
    { 0xe0cf60db, SUC_lexicon_tags_65 },
    { 0xb68f60dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x262f60df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438f60e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734f60ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110f60f3, SUC_lexicon_tags_31 },
    { 0xdaf60f3, SUC_lexicon_tags_63 },
    { 0xa54f60f5, SUC_lexicon_tags_36 },
    { 0x990f60f3, SUC_lexicon_tags_11 },
    { 0xe9ef60f7, SUC_lexicon_tags_6 },
    { 0xbc4f60f8, SUC_lexicon_tags_25 },
    { 0xddaf60f8, SUC_lexicon_tags_11 },
    { 0xa32f60fa, SUC_lexicon_tags_129 },
    { 0xd32f60fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f60ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe94f6101, SUC_lexicon_tags_25 },
    { 0xba2f6102, SUC_lexicon_tags_1 },
    { 0xbf6f6103, SUC_lexicon_tags_1 },
    { 0x1d8f6104, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8f6107, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fef610b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa84f610d, SUC_lexicon_tags_15 },
    { 0x196f610e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2f6113, SUC_lexicon_tags_6 },
    { 0xf38f6113, SUC_lexicon_tags_65 },
    { 0x864f6115, SUC_lexicon_tags_1 },
    { 0xd10f6116, SUC_lexicon_tags_1 },
    { 0xd26f6115, SUC_lexicon_tags_17 },
    { 0x988f6118, SUC_lexicon_tags_8 },
    { 0xd78f6119, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ccf611f, SUC_lexicon_tags_43 },
    { 0xb74f611f, SUC_lexicon_tags_9 },
    { 0xd34f611f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418f6126, SUC_lexicon_tags_17 },
    { 0x7d2f6127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24af612c, SUC_lexicon_tags_27 },
    { 0x1f8f612d, SUC_lexicon_tags_118 },
    { 0x7c4f612d, SUC_lexicon_tags_85 },
    { 0xa68f612e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf6135, SUC_lexicon_tags_11 },
    { 0x20ef6136, SUC_lexicon_tags_17 },
    { 0x71ef6135, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6f613d, SUC_lexicon_tags_19 },
    { 0xda4f613e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fef6140, SUC_lexicon_tags_43 },
    { 0x9fef6141, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daf6146, SUC_lexicon_tags_109 },
    { 0xf60f6146, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618f614b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44f6151, SUC_lexicon_tags_45 },
    { 0x36f6152, SUC_lexicon_tags_19 },
    { 0xef0f6153, SUC_lexicon_tags_4 },
    { 0x3e6f6154, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28f6157, SUC_lexicon_tags_14 },
    { 0xbef6157, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ef615b, SUC_lexicon_tags_8 },
    { 0x518f615c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71ef615e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea0f616d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc4f616f, SUC_lexicon_tags_29 },
    { 0xa4cf6170, SUC_lexicon_tags_79 },
    { 0xb08f6171, SUC_lexicon_tags_11 },
    { 0xd4f6172, SUC_lexicon_tags_25 },
    { 0x8c0f6173, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x378f6176, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ef6179, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78f617e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ef6181, SUC_lexicon_tags_37 },
    { 0x912f6182, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4f6186, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x514f6188, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x496f618b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb62f618f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f6193, SUC_lexicon_tags_11 },
    { 0xbfef6194, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x75ef6196, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecf619c, SUC_lexicon_tags_11 },
    { 0x33cf619c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f61a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf61a8, SUC_lexicon_tags_13 },
    { 0x14af61a9, SUC_lexicon_tags_54 },
    { 0x7a4f61aa, SUC_lexicon_tags_17 },
    { 0x358f61ab, SUC_lexicon_tags_63 },
    { 0x762f61aa, SUC_lexicon_tags_33 },
    { 0x7c4f61ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ef61b0, SUC_lexicon_tags_19 },
    { 0xcf0f61b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6f61b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8f61bb, SUC_lexicon_tags_43 },
    { 0xb26f61bc, SUC_lexicon_tags_37 },
    { 0x37af61bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xca0f61bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8f61c2, SUC_lexicon_tags_45 },
    { 0x742f61c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22f61c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114f61cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x13cf61cf, SUC_lexicon_tags_25 },
    { 0x5cef61d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6f61d3, SUC_lexicon_tags_0 },
    { 0x45af61d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6f61d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef61da, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890f61df, SUC_lexicon_tags_26 },
    { 0xef61e0, SUC_lexicon_tags_68 },
    { 0x894f61e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe22f61e3, SUC_lexicon_tags_11 },
    { 0xe94f61e3, SUC_lexicon_tags_14 },
    { 0xb62f61e4, SUC_lexicon_tags_37 },
    { 0x6af61e6, SUC_lexicon_tags_39 },
    { 0x39af61e3, SUC_lexicon_tags_30 },
    { 0x44af61e5, SUC_lexicon_tags_27 },
    { 0x47af61e9, SUC_lexicon_tags_964 },
    { 0xf8f61ea, SUC_lexicon_tags_31 },
    { 0xc0f61eb, SUC_lexicon_tags_37 },
    { 0x26cf61eb, SUC_lexicon_tags_8 },
    { 0xb2ef61e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268f61f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6f61f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f61f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb72f61fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x388f61fe, SUC_lexicon_tags_14 },
    { 0xf60f61ff, SUC_lexicon_tags_45 },
    { 0x4a6f6200, SUC_lexicon_tags_12 },
    { 0x6ccf6201, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaf6208, SUC_lexicon_tags_10 },
    { 0x730f6209, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac6f620b, SUC_lexicon_tags_25 },
    { 0x69cf620b, SUC_lexicon_tags_49 },
    { 0x44cf620d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af6210, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2d0f6212, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2f6216, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6f6219, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98f621f, SUC_lexicon_tags_12 },
    { 0xa44f6220, SUC_lexicon_tags_1 },
    { 0x70ef6220, SUC_lexicon_tags_14 },
    { 0xf10f621f, SUC_lexicon_tags_6 },
    { 0xd9af6223, SUC_lexicon_tags_14 },
    { 0x1a4f6224, SUC_lexicon_tags_26 },
    { 0xa68f6224, SUC_lexicon_tags_6 },
    { 0x71af6226, SUC_lexicon_tags_26 },
    { 0xba0f6227, SUC_lexicon_tags_87 },
    { 0x7daf6227, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638f622d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50f6233, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfcf6235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8cf6237, SUC_lexicon_tags_31 },
    { 0xa48f6238, SUC_lexicon_tags_38 },
    { 0xcb6f6239, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x55cf623b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47ef623d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6f6240, SUC_lexicon_tags_29 },
    { 0x9b6f6241, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98f6244, SUC_lexicon_tags_19 },
    { 0xa68f6244, SUC_lexicon_tags_26 },
    { 0x6daf6246, SUC_lexicon_tags_37 },
    { 0x846f6245, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x614f624a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4dcf624c, SUC_lexicon_tags_1 },
    { 0x4f4f624c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x224f624f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d6f6251, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0f6254, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcf6258, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2bef625a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfaf625c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ef6261, SUC_lexicon_tags_11 },
    { 0x40af6262, SUC_lexicon_tags_19 },
    { 0x412f6261, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1e6f6265, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x370f6268, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ef626e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc96f6270, SUC_lexicon_tags_25 },
    { 0xc4af6271, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cf627f, SUC_lexicon_tags_14 },
    { 0x1caf6280, SUC_lexicon_tags_33 },
    { 0xb48f6281, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1ecf6283, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc24f6285, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260f6292, SUC_lexicon_tags_17 },
    { 0xcfaf6292, SUC_lexicon_tags_1 },
    { 0xdb2f6294, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaef6298, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7eef629a, SUC_lexicon_tags_21 },
    { 0xae0f629b, SUC_lexicon_tags_1 },
    { 0xefcf629c, SUC_lexicon_tags_17 },
    { 0xfb4f629c, SUC_lexicon_tags_17 },
    { 0x8bef629b, SUC_lexicon_tags_2 },
    { 0xe0cf629f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x412f62a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xebef62a7, SUC_lexicon_tags_11 },
    { 0x682f62a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384f62ad, SUC_lexicon_tags_27 },
    { 0xd6f62ae, SUC_lexicon_tags_1 },
    { 0xf24f62ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3caf62b1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaef62b5, SUC_lexicon_tags_7 },
    { 0x99af62b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732f62c1, SUC_lexicon_tags_25 },
    { 0x654f62c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xafcf62c4, SUC_lexicon_tags_17 },
    { 0x590f62c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668f62c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe90f62cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268f62d0, SUC_lexicon_tags_2 },
    { 0xcfef62d1, SUC_lexicon_tags_11 },
    { 0x1a0f62d2, SUC_lexicon_tags_9 },
    { 0x49ef62d2, SUC_lexicon_tags_7 },
    { 0xb92f62d2, SUC_lexicon_tags_11 },
    { 0xc7ef62d5, SUC_lexicon_tags_14 },
    { 0xe4ef62d0, SUC_lexicon_tags_11 },
    { 0x540f62d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xff0f62d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6f62de, SUC_lexicon_tags_60 },
    { 0x450f62de, SUC_lexicon_tags_65 },
    { 0xe06f62df, SUC_lexicon_tags_27 },
    { 0x2d4f62df, SUC_lexicon_tags_9 },
    { 0xc4cf62e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99af62e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5cf62e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71af62ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcf62f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4f62fd, SUC_lexicon_tags_37 },
    { 0xf4ef62fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb94f6300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4f6303, SUC_lexicon_tags_87 },
    { 0xdbcf6304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe46f6306, SUC_lexicon_tags_1 },
    { 0x59ef6307, SUC_lexicon_tags_23 },
    { 0x900f6308, SUC_lexicon_tags_65 },
    { 0xe18f6306, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8f630d, SUC_lexicon_tags_7 },
    { 0x1faf630e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06f6312, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2af6314, SUC_lexicon_tags_1 },
    { 0xb92f6314, SUC_lexicon_tags_17 },
    { 0x782f6316, SUC_lexicon_tags_37 },
    { 0x584f6315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226f631b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d8f631d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c2f631f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164f6323, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ef6326, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948f632a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2f632e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202f6332, SUC_lexicon_tags_15 },
    { 0x882f6333, SUC_lexicon_tags_11 },
    { 0x118f6334, SUC_lexicon_tags_8 },
    { 0xc12f6333, SUC_lexicon_tags_1 },
    { 0x2e0f6336, SUC_lexicon_tags_25 },
    { 0x672f6335, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d2f6339, SUC_lexicon_tags_57 },
    { 0x288f6339, SUC_lexicon_tags_17 },
    { 0x2cf6339, SUC_lexicon_tags_25 },
    { 0x9f4f633c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaecf633e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x182f6340, SUC_lexicon_tags_17 },
    { 0x9d0f6341, SUC_lexicon_tags_12 },
    { 0x4a6f6342, SUC_lexicon_tags_27 },
    { 0x77cf6342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6af6346, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb54f6349, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ef634f, SUC_lexicon_tags_2 },
    { 0xafaf6350, SUC_lexicon_tags_1 },
    { 0xce8f6351, SUC_lexicon_tags_26 },
    { 0x494f6352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbef6354, SUC_lexicon_tags_8 },
    { 0x56f6355, SUC_lexicon_tags_28 },
    { 0xc42f6355, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cf635a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd26f635c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2af635e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06f6362, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3dcf6364, SUC_lexicon_tags_25 },
    { 0x43ef6365, SUC_lexicon_tags_37 },
    { 0xdb0f6366, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30af6368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394f636c, SUC_lexicon_tags_47 },
    { 0x95ef636d, SUC_lexicon_tags_66 },
    { 0xf42f636d, SUC_lexicon_tags_28 },
    { 0x27af636f, SUC_lexicon_tags_11 },
    { 0x7fef636d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cf6374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4f6378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f637f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97ef6381, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848f6387, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cf638b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16f638e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120f6395, SUC_lexicon_tags_14 },
    { 0xdbef6396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888f639a, SUC_lexicon_tags_14 },
    { 0x6c4f639a, SUC_lexicon_tags_25 },
    { 0xe56f639b, SUC_lexicon_tags_7 },
    { 0xe7ef639d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4f63a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x104f63a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39cf63a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4aef63a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56f63ac, SUC_lexicon_tags_8 },
    { 0xac2f63ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d2f63af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0f63b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x246f63b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2f63b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4f63bd, SUC_lexicon_tags_1010 },
    { 0xaf0f63be, SUC_lexicon_tags_7 },
    { 0x56cf63bf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaf63c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968f63ca, SUC_lexicon_tags_14 },
    { 0xc4af63ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc74f63ce, SUC_lexicon_tags_14 },
    { 0x92f63ce, SUC_lexicon_tags_37 },
    { 0x91ef63ce, SUC_lexicon_tags_31 },
    { 0xf63d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xba6f63d3, SUC_lexicon_tags_14 },
    { 0xdc2f63d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffcf63d8, SUC_lexicon_tags_29 },
    { 0xc4af63d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86f63de, SUC_lexicon_tags_2 },
    { 0x9f8f63de, SUC_lexicon_tags_11 },
    { 0x9c4f63df, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x526f63e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfc8f63e5, SUC_lexicon_tags_32 },
    { 0xec2f63e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2f63ed, SUC_lexicon_tags_45 },
    { 0x364f63ee, SUC_lexicon_tags_7 },
    { 0x410f63ed, SUC_lexicon_tags_25 },
    { 0x586f63f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x416f63f2, SUC_lexicon_tags_60 },
    { 0xe82f63f3, SUC_lexicon_tags_37 },
    { 0x116f63f4, SUC_lexicon_tags_1 },
    { 0xcb2f63f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fef63f9, SUC_lexicon_tags_2 },
    { 0xeacf63fa, SUC_lexicon_tags_25 },
    { 0xe20f63fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc80f63fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72af6402, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a4f6404, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11af6409, SUC_lexicon_tags_9 },
    { 0x31af640a, SUC_lexicon_tags_25 },
    { 0x9e6f640a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x260f640f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c8f6411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdeef6413, SUC_lexicon_tags_7 },
    { 0x140f6413, SUC_lexicon_tags_19 },
    { 0x556f6415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed6f6417, SUC_lexicon_tags_0 },
    { 0xf52f6418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616f641d, SUC_lexicon_tags_7 },
    { 0xcc4f641e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0f6422, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf70f6424, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5acf6427, SUC_lexicon_tags_17 },
    { 0x230f6427, SUC_lexicon_tags_37 },
    { 0x198f6429, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458f642f, SUC_lexicon_tags_11 },
    { 0x1b8f642f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaf6435, SUC_lexicon_tags_16 },
    { 0x2b0f6436, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c0f6438, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3af643b, SUC_lexicon_tags_87 },
    { 0xd5af643c, SUC_lexicon_tags_7 },
    { 0xe30f643d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f6441, SUC_lexicon_tags_7 },
    { 0x3b8f6441, SUC_lexicon_tags_7 },
    { 0x9aef6441, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x136f6445, SUC_lexicon_tags_38 },
    { 0xba4f6445, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x42af6449, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe12f644b, SUC_lexicon_tags_47 },
    { 0x136f644c, SUC_lexicon_tags_0 },
    { 0x988f644d, SUC_lexicon_tags_7 },
    { 0x5a4f644e, SUC_lexicon_tags_60 },
    { 0x25af644e, SUC_lexicon_tags_45 },
    { 0x5bef644d, SUC_lexicon_tags_12 },
    { 0x770f6451, SUC_lexicon_tags_14 },
    { 0x102f6452, SUC_lexicon_tags_38 },
    { 0x450f6452, SUC_lexicon_tags_29 },
    { 0x34af6454, SUC_lexicon_tags_37 },
    { 0xc7af6452, SUC_lexicon_tags_29 },
    { 0x326f6456, SUC_lexicon_tags_11 },
    { 0x602f6456, SUC_lexicon_tags_6 },
    { 0x15af6457, SUC_lexicon_tags_25 },
    { 0xdef6459, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cf645e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa28f6460, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274f6466, SUC_lexicon_tags_46 },
    { 0xa52f6467, SUC_lexicon_tags_25 },
    { 0x11ef6468, SUC_lexicon_tags_61 },
    { 0xf90f6469, SUC_lexicon_tags_14 },
    { 0xb10f6469, SUC_lexicon_tags_31 },
    { 0x84af646b, SUC_lexicon_tags_37 },
    { 0x892f646c, SUC_lexicon_tags_30 },
    { 0xe0af6468, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cf6475, SUC_lexicon_tags_17 },
    { 0xc52f6475, SUC_lexicon_tags_7 },
    { 0x3e8f6475, SUC_lexicon_tags_27 },
    { 0x38af6478, SUC_lexicon_tags_47 },
    { 0xb68f6479, SUC_lexicon_tags_29 },
    { 0xfa2f647a, SUC_lexicon_tags_33 },
    { 0x68ef6478, SUC_lexicon_tags_25 },
    { 0x5ef6477, SUC_lexicon_tags_37 },
    { 0xa94f647d, SUC_lexicon_tags_1 },
    { 0x970f647d, SUC_lexicon_tags_62 },
    { 0xf26f647f, SUC_lexicon_tags_47 },
    { 0x396f647e, SUC_lexicon_tags_29 },
    { 0x1a8f6481, SUC_lexicon_tags_37 },
    { 0xb28f6482, SUC_lexicon_tags_46 },
    { 0x2b4f6481, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x982f648b, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xdb6f648d, SUC_lexicon_tags_7 },
    { 0x4b0f648e, SUC_lexicon_tags_7 },
    { 0x306f648e, SUC_lexicon_tags_25 },
    { 0xb2ef6490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a0f6492, SUC_lexicon_tags_12 },
    { 0x8b4f6492, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc36f6495, SUC_lexicon_tags_8 },
    { 0x702f6496, SUC_lexicon_tags_4 },
    { 0xff8f6495, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2f649a, SUC_lexicon_tags_17 },
    { 0xe16f649a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf14f649d, SUC_lexicon_tags_11 },
    { 0x16f649e, SUC_lexicon_tags_17 },
    { 0x39af649e, SUC_lexicon_tags_33 },
    { 0xefcf64a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48f64a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cf64a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536f64ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b4f64af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4f64b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d8f64b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x876f64b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fef64b9, SUC_lexicon_tags_16 },
    { 0xcc4f64ba, SUC_lexicon_tags_19 },
    { 0xb44f64bb, SUC_lexicon_tags_11 },
    { 0x21ef64bb, SUC_lexicon_tags_37 },
    { 0x8f8f64b9, SUC_lexicon_tags_33 },
    { 0x7f2f64be, SUC_lexicon_tags_11 },
    { 0xea6f64be, SUC_lexicon_tags_31 },
    { 0xffef64bd, SUC_lexicon_tags_43 },
    { 0xe64f64c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c4f64c3, SUC_lexicon_tags_4 },
    { 0x500f64c3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x846f64c6, SUC_lexicon_tags_12 },
    { 0xc08f64c7, SUC_lexicon_tags_1 },
    { 0xfd0f64c8, SUC_lexicon_tags_25 },
    { 0xb4cf64c9, SUC_lexicon_tags_1 },
    { 0x786f64ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416f64d0, SUC_lexicon_tags_2 },
    { 0xc14f64d1, SUC_lexicon_tags_14 },
    { 0xc76f64d2, SUC_lexicon_tags_6 },
    { 0x180f64d2, SUC_lexicon_tags_12 },
    { 0xdc6f64d4, SUC_lexicon_tags_2 },
    { 0xaeaf64d2, SUC_lexicon_tags_63 },
    { 0x332f64d0, SUC_lexicon_tags_55 },
    { 0xefef64d7, SUC_lexicon_tags_43 },
    { 0xdd8f64d8, SUC_lexicon_tags_25 },
    { 0x716f64d3, SUC_lexicon_tags_1440 },
    { 0x8fef64d7, SUC_lexicon_tags_17 },
    { 0x13af64d0, SUC_lexicon_tags_37 },
    { 0x728f64dc, SUC_lexicon_tags_12 },
    { 0x266f64dd, SUC_lexicon_tags_17 },
    { 0x2b8f64dc, SUC_lexicon_tags_662 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16f64e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x692f64e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fef64ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72f64ee, SUC_lexicon_tags_8 },
    { 0xcf64ef, SUC_lexicon_tags_53 },
    { 0x2faf64f0, SUC_lexicon_tags_1 },
    { 0xcecf64f1, SUC_lexicon_tags_2 },
    { 0xfdaf64f1, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194f64f6, SUC_lexicon_tags_79 },
    { 0xac0f64f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56f64fb, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aef64fe, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x986f6500, SUC_lexicon_tags_17 },
    { 0x812f6500, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980f6507, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x340f6509, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4f650c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x540f650f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f6514, SUC_lexicon_tags_3 },
    { 0x88af6514, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfd0f6517, SUC_lexicon_tags_37 },
    { 0x50ef6518, SUC_lexicon_tags_37 },
    { 0xab6f6518, SUC_lexicon_tags_1 },
    { 0xca6f6519, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808f651e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cf6521, SUC_lexicon_tags_11 },
    { 0x2f8f6522, SUC_lexicon_tags_25 },
    { 0x33cf6523, SUC_lexicon_tags_8 },
    { 0xc86f6521, SUC_lexicon_tags_191 },
    { 0x1eaf6525, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6f6528, SUC_lexicon_tags_14 },
    { 0x390f6529, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38af652d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbcf6531, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd4f6533, SUC_lexicon_tags_17 },
    { 0x76ef6534, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc32f6537, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52f6541, SUC_lexicon_tags_47 },
    { 0xe38f6542, SUC_lexicon_tags_11 },
    { 0x79af6543, SUC_lexicon_tags_4 },
    { 0x3cef6544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf654e, SUC_lexicon_tags_0 },
    { 0xa26f654f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf6552, SUC_lexicon_tags_179 },
    { 0xe60f6553, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8f6558, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882f655e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd8f6560, SUC_lexicon_tags_2 },
    { 0x462f6560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf6564, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x932f6567, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa40f6569, SUC_lexicon_tags_26 },
    { 0x82cf6569, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaf656d, SUC_lexicon_tags_37 },
    { 0x9a8f656e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4acf6573, SUC_lexicon_tags_9 },
    { 0xb1af6574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5cf6576, SUC_lexicon_tags_35 },
    { 0x110f6577, SUC_lexicon_tags_61 },
    { 0xb7ef6577, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66cf657a, SUC_lexicon_tags_23 },
    { 0xc40f657b, SUC_lexicon_tags_47 },
    { 0x9aaf657c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1af6582, SUC_lexicon_tags_7 },
    { 0x856f6582, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b4f6585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf6589, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcf658d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8f6590, SUC_lexicon_tags_2 },
    { 0x5fef6590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x958f6593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8f6597, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf28f6599, SUC_lexicon_tags_25 },
    { 0x8d4f659a, SUC_lexicon_tags_85 },
    { 0xb1ef659b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20f659f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2f65a8, SUC_lexicon_tags_5 },
    { 0x28f65a8, SUC_lexicon_tags_11 },
    { 0x676f65aa, SUC_lexicon_tags_33 },
    { 0x228f65aa, SUC_lexicon_tags_7 },
    { 0xb78f65aa, SUC_lexicon_tags_11 },
    { 0x8dcf65ad, SUC_lexicon_tags_17 },
    { 0x826f65ac, SUC_lexicon_tags_43 },
    { 0x44ef65a8, SUC_lexicon_tags_49 },
    { 0xa40f65a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8f65b4, SUC_lexicon_tags_151 },
    { 0xc54f65b5, SUC_lexicon_tags_46 },
    { 0x728f65b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e0f65b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4f65be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4faf65c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cf65c3, SUC_lexicon_tags_2 },
    { 0x14ef65c4, SUC_lexicon_tags_25 },
    { 0x532f65c3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44af65ca, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x35ef65cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cf65d2, SUC_lexicon_tags_48 },
    { 0xf6cf65d3, SUC_lexicon_tags_12 },
    { 0x708f65d3, SUC_lexicon_tags_47 },
    { 0x4daf65d5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x660f65d8, SUC_lexicon_tags_17 },
    { 0x1a6f65d8, SUC_lexicon_tags_23 },
    { 0x950f65da, SUC_lexicon_tags_17 },
    { 0xdc8f65da, SUC_lexicon_tags_14 },
    { 0xa22f65db, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0, NULL },
    { 0x990f65df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482f65e3, SUC_lexicon_tags_49 },
    { 0xe24f65e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3af65e7, SUC_lexicon_tags_11 },
    { 0x8ecf65e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cf65ec, SUC_lexicon_tags_6 },
    { 0xcb4f65ed, SUC_lexicon_tags_30 },
    { 0x718f65ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59ef65f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cf65f4, SUC_lexicon_tags_17 },
    { 0xe80f65f4, SUC_lexicon_tags_11 },
    { 0xae4f65f6, SUC_lexicon_tags_83 },
    { 0xf72f65f7, SUC_lexicon_tags_60 },
    { 0x474f65f8, SUC_lexicon_tags_1 },
    { 0x83af65f6, SUC_lexicon_tags_8 },
    { 0xc7cf65f4, SUC_lexicon_tags_7 },
    { 0x60f65fb, SUC_lexicon_tags_12 },
    { 0x274f65fc, SUC_lexicon_tags_82 },
    { 0x914f65f8, SUC_lexicon_tags_68 },
    { 0xd36f65fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x254f6600, SUC_lexicon_tags_2 },
    { 0xb2ef6601, SUC_lexicon_tags_47 },
    { 0x992f6602, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecf6605, SUC_lexicon_tags_8 },
    { 0x5a0f6606, SUC_lexicon_tags_11 },
    { 0x904f6607, SUC_lexicon_tags_55 },
    { 0x96af6606, SUC_lexicon_tags_49 },
    { 0xe0af6609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc24f660c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6f6610, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2f6615, SUC_lexicon_tags_11 },
    { 0xa26f6616, SUC_lexicon_tags_17 },
    { 0x688f6617, SUC_lexicon_tags_27 },
    { 0xa5ef6617, SUC_lexicon_tags_29 },
    { 0xf58f6616, SUC_lexicon_tags_26 },
    { 0xb5cf6618, SUC_lexicon_tags_27 },
    { 0xef6f661b, SUC_lexicon_tags_1 },
    { 0xee2f661c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0f661f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x410f6622, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2c6f6624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6f6629, SUC_lexicon_tags_47 },
    { 0x206f662a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0af662c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ef6636, SUC_lexicon_tags_12 },
    { 0x11ef6637, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6f663c, SUC_lexicon_tags_47 },
    { 0x1fef663d, SUC_lexicon_tags_2 },
    { 0x2acf663d, SUC_lexicon_tags_23 },
    { 0x282f663c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x460f6642, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad4f6644, SUC_lexicon_tags_25 },
    { 0x3e8f6645, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cf6650, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf66f6656, SUC_lexicon_tags_60 },
    { 0x12af6656, SUC_lexicon_tags_25 },
    { 0x33af6658, SUC_lexicon_tags_11 },
    { 0xbf0f6658, SUC_lexicon_tags_25 },
    { 0xa8f6657, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcf665f, SUC_lexicon_tags_77 },
    { 0x4eef6660, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2cf6662, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aef6669, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712f666e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6f6673, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf64f6675, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f6679, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaf667c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5f0f667e, SUC_lexicon_tags_37 },
    { 0xbaef667f, SUC_lexicon_tags_542 },
    { 0x208f6680, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2f6688, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf14f668a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32f668e, SUC_lexicon_tags_65 },
    { 0x598f668f, SUC_lexicon_tags_3 },
    { 0x244f6690, SUC_lexicon_tags_8 },
    { 0x924f6691, SUC_lexicon_tags_79 },
    { 0xa76f668f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502f669d, SUC_lexicon_tags_2 },
    { 0xd62f669e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4f66a7, SUC_lexicon_tags_32 },
    { 0x680f66a8, SUC_lexicon_tags_60 },
    { 0xf00f66a9, SUC_lexicon_tags_12 },
    { 0xbd4f66a7, SUC_lexicon_tags_49 },
    { 0xeccf66ab, SUC_lexicon_tags_19 },
    { 0x2aaf66ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x358f66ae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218f66b2, SUC_lexicon_tags_87 },
    { 0xd98f66b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34f66ba, SUC_lexicon_tags_12 },
    { 0xe68f66bb, SUC_lexicon_tags_117 },
    { 0xca8f66bc, SUC_lexicon_tags_12 },
    { 0xcc6f66bc, SUC_lexicon_tags_11 },
    { 0x3d0f66be, SUC_lexicon_tags_63 },
    { 0xd86f66bd, SUC_lexicon_tags_27 },
    { 0x892f66c0, SUC_lexicon_tags_11 },
    { 0x6b4f66c0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xbe2f66c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cf66cb, SUC_lexicon_tags_9 },
    { 0x9cf66cc, SUC_lexicon_tags_2 },
    { 0x5fcf66cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac2f66cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe66f66d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6f66d6, SUC_lexicon_tags_31 },
    { 0xce0f66d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc8cf66d9, SUC_lexicon_tags_47 },
    { 0x53af66da, SUC_lexicon_tags_47 },
    { 0x92f66da, SUC_lexicon_tags_70 },
    { 0x6a8f66dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf66df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5af66e5, SUC_lexicon_tags_12 },
    { 0x364f66e6, SUC_lexicon_tags_2 },
    { 0xe2ef66e7, SUC_lexicon_tags_25 },
    { 0xf36f66e7, SUC_lexicon_tags_14 },
    { 0xedaf66e6, SUC_lexicon_tags_17 },
    { 0x9eaf66e7, SUC_lexicon_tags_38 },
    { 0xa30f66e7, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0x8def66ed, SUC_lexicon_tags_63 },
    { 0xab6f66ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x184f66f0, SUC_lexicon_tags_6 },
    { 0xfcef66f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x22ef66f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2bef66f5, SUC_lexicon_tags_14 },
    { 0xc78f66f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf2f66f8, SUC_lexicon_tags_31 },
    { 0x5fef66f9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34f66ff, SUC_lexicon_tags_25 },
    { 0x688f6700, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1f8f6702, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f6f6704, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc4af6706, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ef670a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe82f670c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ef6713, SUC_lexicon_tags_63 },
    { 0xbc4f6713, SUC_lexicon_tags_11 },
    { 0xf58f6715, SUC_lexicon_tags_1 },
    { 0x134f6716, SUC_lexicon_tags_45 },
    { 0x416f6715, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x230f671a, SUC_lexicon_tags_15 },
    { 0xdc6f671b, SUC_lexicon_tags_11 },
    { 0x7e4f671c, SUC_lexicon_tags_2 },
    { 0xf1cf671d, SUC_lexicon_tags_15 },
    { 0x4a4f671e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaf6721, SUC_lexicon_tags_27 },
    { 0x306f6722, SUC_lexicon_tags_66 },
    { 0x7ccf6722, SUC_lexicon_tags_11 },
    { 0x878f6724, SUC_lexicon_tags_49 },
    { 0x6f8f6725, SUC_lexicon_tags_45 },
    { 0xfbcf6724, SUC_lexicon_tags_9 },
    { 0xb00f6727, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xef672a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xebef672c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf28f672e, SUC_lexicon_tags_204 },
    { 0x53af672f, SUC_lexicon_tags_8 },
    { 0xd82f6730, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x68f6732, SUC_lexicon_tags_0 },
    { 0xea4f6732, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6f6739, SUC_lexicon_tags_15 },
    { 0x7eef673a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf26f673c, SUC_lexicon_tags_14 },
    { 0xc9cf673c, SUC_lexicon_tags_49 },
    { 0x67af673e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6f6741, SUC_lexicon_tags_37 },
    { 0x96ef6742, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x43af6745, SUC_lexicon_tags_48 },
    { 0xbfaf6745, SUC_lexicon_tags_7 },
    { 0x794f6747, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ef674a, SUC_lexicon_tags_19 },
    { 0x620f674b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf84f6753, SUC_lexicon_tags_45 },
    { 0xfbef6754, SUC_lexicon_tags_77 },
    { 0x294f6755, SUC_lexicon_tags_7 },
    { 0xadef6756, SUC_lexicon_tags_29 },
    { 0x62af6756, SUC_lexicon_tags_11 },
    { 0xa04f6756, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6f675f, SUC_lexicon_tags_15 },
    { 0x720f675f, SUC_lexicon_tags_0 },
    { 0x494f6761, SUC_lexicon_tags_7 },
    { 0xb66f6762, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0f6765, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eef6768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaef6772, SUC_lexicon_tags_26 },
    { 0x592f6772, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f6776, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984f6781, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bcf6783, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cf6786, SUC_lexicon_tags_2 },
    { 0x326f6787, SUC_lexicon_tags_10 },
    { 0xc20f6786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0f678d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618f6797, SUC_lexicon_tags_15 },
    { 0xf9af6798, SUC_lexicon_tags_12 },
    { 0x7aef6797, SUC_lexicon_tags_64 },
    { 0x84af679a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x9a2f679c, SUC_lexicon_tags_25 },
    { 0x4d8f679c, SUC_lexicon_tags_2 },
    { 0x182f679e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8f67a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x720f67a6, SUC_lexicon_tags_25 },
    { 0x4a0f67a7, SUC_lexicon_tags_37 },
    { 0x62af67a8, SUC_lexicon_tags_47 },
    { 0x8cf67a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc8f67ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x662f67af, SUC_lexicon_tags_8 },
    { 0xe98f67b0, SUC_lexicon_tags_26 },
    { 0x6daf67b0, SUC_lexicon_tags_11 },
    { 0x6a4f67b0, SUC_lexicon_tags_23 },
    { 0x26af67b3, SUC_lexicon_tags_27 },
    { 0x208f67b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2b2f67b6, SUC_lexicon_tags_7 },
    { 0xca8f67b6, SUC_lexicon_tags_6 },
    { 0x2def67b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968f67bc, SUC_lexicon_tags_19 },
    { 0x25af67bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314f67c2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa02f67c4, SUC_lexicon_tags_1 },
    { 0xb5cf67c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06f67ce, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x23ef67d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x102f67d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124f67df, SUC_lexicon_tags_47 },
    { 0xfb0f67e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb62f67e2, SUC_lexicon_tags_33 },
    { 0x880f67e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc2f67e5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaf67e8, SUC_lexicon_tags_1 },
    { 0x64ef67e9, SUC_lexicon_tags_27 },
    { 0x3faf67e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe3cf67ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b0f67ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f67f1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x31af67f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x626f67f7, SUC_lexicon_tags_27 },
    { 0x140f67f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7eaf67fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cf67fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83af6803, SUC_lexicon_tags_43 },
    { 0xa52f6803, SUC_lexicon_tags_54 },
    { 0xd50f6805, SUC_lexicon_tags_25 },
    { 0xe0cf6805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f6809, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf98f680c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79cf6810, SUC_lexicon_tags_7 },
    { 0x51ef6810, SUC_lexicon_tags_48 },
    { 0xd44f6811, SUC_lexicon_tags_28 },
    { 0xd58f6813, SUC_lexicon_tags_31 },
    { 0x58cf6814, SUC_lexicon_tags_12 },
    { 0x2b2f6812, SUC_lexicon_tags_32 },
    { 0x83cf6810, SUC_lexicon_tags_37 },
    { 0xc6af6817, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x794f6819, SUC_lexicon_tags_17 },
    { 0x826f6819, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x534f681c, SUC_lexicon_tags_23 },
    { 0xeaaf681d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06f6823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf0f6825, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cf6829, SUC_lexicon_tags_8 },
    { 0x2aaf682a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9aef682c, SUC_lexicon_tags_17 },
    { 0x846f682d, SUC_lexicon_tags_38 },
    { 0x9ecf682e, SUC_lexicon_tags_15 },
    { 0xa6ef682f, SUC_lexicon_tags_25 },
    { 0xb14f6830, SUC_lexicon_tags_49 },
    { 0xc86f682f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf14f6833, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x706f6835, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cf6838, SUC_lexicon_tags_4 },
    { 0xfa2f6839, SUC_lexicon_tags_0 },
    { 0x942f6838, SUC_lexicon_tags_15 },
    { 0x262f683b, SUC_lexicon_tags_0 },
    { 0xa06f683c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ef683f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdc8f6841, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8eef6843, SUC_lexicon_tags_9 },
    { 0xc66f6844, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4f684d, SUC_lexicon_tags_17 },
    { 0x5b0f684e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998f6853, SUC_lexicon_tags_19 },
    { 0xb98f6854, SUC_lexicon_tags_10 },
    { 0x210f6855, SUC_lexicon_tags_15 },
    { 0xffef6855, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278f685a, SUC_lexicon_tags_201 },
    { 0x674f685b, SUC_lexicon_tags_37 },
    { 0x52ef685b, SUC_lexicon_tags_7 },
    { 0x428f685c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6cf685f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcef6862, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x666f6864, SUC_lexicon_tags_46 },
    { 0xcaaf6865, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc24f6867, SUC_lexicon_tags_9 },
    { 0x192f6868, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbc0f686a, SUC_lexicon_tags_37 },
    { 0xc22f686b, SUC_lexicon_tags_39 },
    { 0x6c0f686b, SUC_lexicon_tags_60 },
    { 0xda2f686d, SUC_lexicon_tags_8 },
    { 0x93af686a, SUC_lexicon_tags_19 },
    { 0xfeef686d, SUC_lexicon_tags_15 },
    { 0xd4ef6870, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6f6875, SUC_lexicon_tags_32 },
    { 0xdbef6875, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666f687e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x242f6880, SUC_lexicon_tags_30 },
    { 0x446f6880, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaacf6883, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd84f6885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16ef6887, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb92f6889, SUC_lexicon_tags_66 },
    { 0x81ef6889, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8f688d, SUC_lexicon_tags_7 },
    { 0x5fcf688d, SUC_lexicon_tags_37 },
    { 0x62cf688f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258f6893, SUC_lexicon_tags_0 },
    { 0xe7cf6894, SUC_lexicon_tags_1 },
    { 0xbd6f6895, SUC_lexicon_tags_37 },
    { 0x2bcf6894, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x670f6898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74f689b, SUC_lexicon_tags_25 },
    { 0x792f689c, SUC_lexicon_tags_1280 },
    { 0x31ef689c, SUC_lexicon_tags_37 },
    { 0x3def689e, SUC_lexicon_tags_15 },
    { 0x3baf689c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2f68a6, SUC_lexicon_tags_11 },
    { 0xad2f68a7, SUC_lexicon_tags_68 },
    { 0x8a2f68a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54f68ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb26f68ae, SUC_lexicon_tags_25 },
    { 0xc54f68af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ef68b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6f68b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cef68b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44f68bd, SUC_lexicon_tags_88 },
    { 0x1acf68bd, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0x92ef68c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8f68c3, SUC_lexicon_tags_3 },
    { 0x3eef68c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cf68ca, SUC_lexicon_tags_47 },
    { 0xbd6f68cb, SUC_lexicon_tags_46 },
    { 0x1aaf68cb, SUC_lexicon_tags_25 },
    { 0xf36f68cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2f68d2, SUC_lexicon_tags_11 },
    { 0x656f68d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0f68d8, SUC_lexicon_tags_17 },
    { 0x6cf68d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x91af68db, SUC_lexicon_tags_17 },
    { 0xc5cf68dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cf68e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ef68e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af68ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cf68ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaf68f3, SUC_lexicon_tags_536 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaf68f8, SUC_lexicon_tags_37 },
    { 0x16af68f9, SUC_lexicon_tags_23 },
    { 0xb30f68f8, SUC_lexicon_tags_16 },
    { 0x2e2f68f8, SUC_lexicon_tags_11 },
    { 0xecef68fc, SUC_lexicon_tags_37 },
    { 0xf4f68fc, SUC_lexicon_tags_28 },
    { 0x42cf68fb, SUC_lexicon_tags_1 },
    { 0x48af68fd, SUC_lexicon_tags_38 },
    { 0x94ef6900, SUC_lexicon_tags_12 },
    { 0x8f6f68fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33af6906, SUC_lexicon_tags_65 },
    { 0xef6f6907, SUC_lexicon_tags_37 },
    { 0xeb0f6908, SUC_lexicon_tags_181 },
    { 0xd2f6906, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04f690d, SUC_lexicon_tags_39 },
    { 0xe48f690e, SUC_lexicon_tags_47 },
    { 0xc50f690f, SUC_lexicon_tags_14 },
    { 0xc7af6910, SUC_lexicon_tags_27 },
    { 0x3e8f6911, SUC_lexicon_tags_68 },
    { 0xb34f690e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6af691e, SUC_lexicon_tags_17 },
    { 0x2e4f691f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x484f6926, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cf6929, SUC_lexicon_tags_48 },
    { 0x5f0f692a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45cf692c, SUC_lexicon_tags_7 },
    { 0x3fcf692c, SUC_lexicon_tags_82 },
    { 0x320f692e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4f6931, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8f6937, SUC_lexicon_tags_201 },
    { 0xe6af6937, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaf693d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0af6941, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940f6946, SUC_lexicon_tags_15 },
    { 0xdaaf6947, SUC_lexicon_tags_4 },
    { 0xf3cf6947, SUC_lexicon_tags_15 },
    { 0xe74f6949, SUC_lexicon_tags_6 },
    { 0x5eaf6947, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0af694e, SUC_lexicon_tags_9 },
    { 0x788f694f, SUC_lexicon_tags_28 },
    { 0x6bef6950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584f6955, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd4f6957, SUC_lexicon_tags_12 },
    { 0x732f6958, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c8f695a, SUC_lexicon_tags_38 },
    { 0xe0f695b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36f6961, SUC_lexicon_tags_47 },
    { 0xa4f6961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdccf6964, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4acf6967, SUC_lexicon_tags_65 },
    { 0x2caf6967, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ef696d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x650f696f, SUC_lexicon_tags_12 },
    { 0x680f6970, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef6973, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fef6975, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976f697a, SUC_lexicon_tags_25 },
    { 0xd2ef697a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95af697d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x708f6980, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x328f6983, SUC_lexicon_tags_47 },
    { 0xe86f6983, SUC_lexicon_tags_17 },
    { 0x452f6985, SUC_lexicon_tags_56 },
    { 0x996f6986, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc72f6989, SUC_lexicon_tags_15 },
    { 0xdc0f698a, SUC_lexicon_tags_8 },
    { 0xbef698a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10cf698d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6f6990, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd90f6992, SUC_lexicon_tags_17 },
    { 0x34af6993, SUC_lexicon_tags_19 },
    { 0x8c6f6994, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3ef6996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e0f6998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedef699c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbb2f699e, SUC_lexicon_tags_15 },
    { 0xd9cf699e, SUC_lexicon_tags_47 },
    { 0xf1cf699e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x346f69a4, SUC_lexicon_tags_37 },
    { 0xf00f69a4, SUC_lexicon_tags_65 },
    { 0xb58f69a6, SUC_lexicon_tags_14 },
    { 0xd50f69a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x262f69aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf92f69ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d4f69ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x424f69b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8f69b5, SUC_lexicon_tags_37 },
    { 0xb4cf69b6, SUC_lexicon_tags_30 },
    { 0x23cf69b6, SUC_lexicon_tags_9 },
    { 0x9cef69b5, SUC_lexicon_tags_138 },
    { 0x64cf69b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2b4f69bb, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x904f69be, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cf69c3, SUC_lexicon_tags_6 },
    { 0xbef69c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988f69c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x978f69ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182f69ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x478f69d1, SUC_lexicon_tags_30 },
    { 0xd2ef69d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92f69d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaaf69db, SUC_lexicon_tags_27 },
    { 0x998f69dc, SUC_lexicon_tags_1 },
    { 0x8eef69dd, SUC_lexicon_tags_55 },
    { 0x3bcf69de, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2f69e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896f69e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x55cf69e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0f69ec, SUC_lexicon_tags_45 },
    { 0x3ccf69ed, SUC_lexicon_tags_1 },
    { 0x6d8f69ed, SUC_lexicon_tags_37 },
    { 0xe6cf69ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162f69f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802f69fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfef6a01, SUC_lexicon_tags_39 },
    { 0x128f6a02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x802f6a04, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xd80f6a06, SUC_lexicon_tags_17 },
    { 0xbcf6a06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12cf6a09, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5a2f6a0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97cf6a0d, SUC_lexicon_tags_14 },
    { 0x5faf6a0d, SUC_lexicon_tags_13 },
    { 0x9acf6a0d, SUC_lexicon_tags_1 },
    { 0x234f6a10, SUC_lexicon_tags_49 },
    { 0x664f6a11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaf6a17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1af6a1c, SUC_lexicon_tags_2 },
    { 0x56af6a1c, SUC_lexicon_tags_16 },
    { 0xf9ef6a1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8f6a24, SUC_lexicon_tags_9 },
    { 0x228f6a25, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x62cf6a27, SUC_lexicon_tags_29 },
    { 0xe6cf6a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2f6a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fcf6a2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700f6a31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56cf6a36, SUC_lexicon_tags_1 },
    { 0x788f6a37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x140f6a39, SUC_lexicon_tags_12 },
    { 0x6c4f6a39, SUC_lexicon_tags_38 },
    { 0x706f6a3a, SUC_lexicon_tags_37 },
    { 0xee0f6a39, SUC_lexicon_tags_12 },
    { 0xcc8f6a3d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x710f6a3f, SUC_lexicon_tags_56 },
    { 0x7ecf6a40, SUC_lexicon_tags_37 },
    { 0x660f6a40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80af6a45, SUC_lexicon_tags_30 },
    { 0xc58f6a46, SUC_lexicon_tags_37 },
    { 0x792f6a45, SUC_lexicon_tags_37 },
    { 0xd68f6a48, SUC_lexicon_tags_65 },
    { 0xfbaf6a49, SUC_lexicon_tags_25 },
    { 0xab8f6a4a, SUC_lexicon_tags_33 },
    { 0x7fef6a49, SUC_lexicon_tags_14 },
    { 0xddaf6a4c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ef6a53, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1acf6a55, SUC_lexicon_tags_37 },
    { 0x77af6a56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4f6a5e, SUC_lexicon_tags_17 },
    { 0x5f2f6a5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7faf6a62, SUC_lexicon_tags_11 },
    { 0x74af6a62, SUC_lexicon_tags_669 },
    { 0xb58f6a64, SUC_lexicon_tags_14 },
    { 0x67af6a65, SUC_lexicon_tags_30 },
    { 0x1daf6a66, SUC_lexicon_tags_13 },
    { 0x74cf6a67, SUC_lexicon_tags_47 },
    { 0x2eef6a68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xda0f6a6a, SUC_lexicon_tags_21 },
    { 0x662f6a6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa0f6a6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cf6a73, SUC_lexicon_tags_31 },
    { 0xeb8f6a74, SUC_lexicon_tags_21 },
    { 0x81ef6a74, SUC_lexicon_tags_33 },
    { 0x8faf6a76, SUC_lexicon_tags_25 },
    { 0xae8f6a76, SUC_lexicon_tags_2 },
    { 0x7aaf6a78, SUC_lexicon_tags_19 },
    { 0xd9cf6a79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf56f6a7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8af6a7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x920f6a80, SUC_lexicon_tags_1 },
    { 0xec6f6a81, SUC_lexicon_tags_17 },
    { 0xd8af6a81, SUC_lexicon_tags_2 },
    { 0x214f6a83, SUC_lexicon_tags_17 },
    { 0x3def6a84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x986f6a86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742f6a8b, SUC_lexicon_tags_2 },
    { 0xaacf6a8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x622f6a8f, SUC_lexicon_tags_37 },
    { 0x360f6a90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcf6a93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b0f6a95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89ef6a97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf78f6a99, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bef6a9f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x20cf6aa2, SUC_lexicon_tags_0 },
    { 0x92af6aa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf6aa7, SUC_lexicon_tags_1 },
    { 0x264f6aa7, SUC_lexicon_tags_34 },
    { 0x134f6aa9, SUC_lexicon_tags_37 },
    { 0x60cf6aaa, SUC_lexicon_tags_45 },
    { 0x72ef6aab, SUC_lexicon_tags_12 },
    { 0x5daf6aaa, SUC_lexicon_tags_7 },
    { 0xd7ef6aa7, SUC_lexicon_tags_60 },
    { 0x510f6aa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6f6ab2, SUC_lexicon_tags_25 },
    { 0x602f6ab3, SUC_lexicon_tags_37 },
    { 0x888f6ab4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98f6abd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcf6ac0, SUC_lexicon_tags_27 },
    { 0x214f6ac0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666f6ac7, SUC_lexicon_tags_17 },
    { 0x20ef6ac7, SUC_lexicon_tags_30 },
    { 0x304f6ac9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcf6ad3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2f6ad7, SUC_lexicon_tags_31 },
    { 0xf6f6ad7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772f6adf, SUC_lexicon_tags_9 },
    { 0xfc0f6adf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f6ae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x636f6ae8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x244f6aea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2f6af0, SUC_lexicon_tags_17 },
    { 0x34ef6af1, SUC_lexicon_tags_14 },
    { 0xd7af6af0, SUC_lexicon_tags_105 },
    { 0xa42f6af3, SUC_lexicon_tags_11 },
    { 0xab2f6af4, SUC_lexicon_tags_37 },
    { 0x5bef6af5, SUC_lexicon_tags_12 },
    { 0xb2af6af6, SUC_lexicon_tags_38 },
    { 0x2b4f6af5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404f6b01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9baf6b03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x168f6b05, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fef6b0b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc06f6b0e, SUC_lexicon_tags_4 },
    { 0xd00f6b0e, SUC_lexicon_tags_7 },
    { 0x914f6b10, SUC_lexicon_tags_15 },
    { 0x96ef6b11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6f6b15, SUC_lexicon_tags_25 },
    { 0xdd0f6b16, SUC_lexicon_tags_12 },
    { 0xf1ef6b15, SUC_lexicon_tags_28 },
    { 0xa86f6b18, SUC_lexicon_tags_44 },
    { 0xe72f6b15, SUC_lexicon_tags_64 },
    { 0xfb0f6b1a, SUC_lexicon_tags_27 },
    { 0x754f6b1b, SUC_lexicon_tags_11 },
    { 0xb80f6b1c, SUC_lexicon_tags_2 },
    { 0xdf0f6b1a, SUC_lexicon_tags_38 },
    { 0xee2f6b19, SUC_lexicon_tags_7 },
    { 0x3a2f6b1f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734f6b29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1def6b2b, SUC_lexicon_tags_17 },
    { 0xd60f6b2c, SUC_lexicon_tags_17 },
    { 0x50ef6b2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf74f6b2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50af6b32, SUC_lexicon_tags_47 },
    { 0x7eaf6b32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc62f6b35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x938f6b37, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83af6b3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620f6b41, SUC_lexicon_tags_28 },
    { 0x942f6b41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x67ef6b44, SUC_lexicon_tags_15 },
    { 0x90ef6b45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1af6b47, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x136f6b49, SUC_lexicon_tags_37 },
    { 0x6f0f6b49, SUC_lexicon_tags_37 },
    { 0x34ef6b4b, SUC_lexicon_tags_7 },
    { 0xd70f6b4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x83cf6b4f, SUC_lexicon_tags_49 },
    { 0x846f6b4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x272f6b52, SUC_lexicon_tags_19 },
    { 0xa8cf6b53, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x498f6b55, SUC_lexicon_tags_8 },
    { 0xf6b55, SUC_lexicon_tags_15 },
    { 0x6d4f6b55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x184f6b5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14f6b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedcf6b61, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ef6b64, SUC_lexicon_tags_11 },
    { 0xfb2f6b65, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86f6b68, SUC_lexicon_tags_7 },
    { 0x714f6b69, SUC_lexicon_tags_7 },
    { 0x286f6b6a, SUC_lexicon_tags_25 },
    { 0x34ef6b69, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10f6b70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6f6b73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x37af6b75, SUC_lexicon_tags_27 },
    { 0x23cf6b76, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60f6b79, SUC_lexicon_tags_12 },
    { 0x65af6b79, SUC_lexicon_tags_0 },
    { 0x7e8f6b7b, SUC_lexicon_tags_17 },
    { 0x7d8f6b79, SUC_lexicon_tags_13 },
    { 0xd32f6b7c, SUC_lexicon_tags_29 },
    { 0xf14f6b7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ef6b83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12f6b86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eef6b8e, SUC_lexicon_tags_29 },
    { 0x558f6b8f, SUC_lexicon_tags_2 },
    { 0xa68f6b8f, SUC_lexicon_tags_6 },
    { 0x4fef6b90, SUC_lexicon_tags_17 },
    { 0x1f4f6b92, SUC_lexicon_tags_17 },
    { 0x4b4f6b93, SUC_lexicon_tags_31 },
    { 0xa30f6b94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4f6b98, SUC_lexicon_tags_37 },
    { 0x6c4f6b99, SUC_lexicon_tags_66 },
    { 0x598f6b9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cf6b9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4f6ba1, SUC_lexicon_tags_11 },
    { 0xbcaf6ba1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6f6baa, SUC_lexicon_tags_972 },
    { 0xca2f6baa, SUC_lexicon_tags_262 },
    { 0x920f6bac, SUC_lexicon_tags_17 },
    { 0x8f4f6bac, SUC_lexicon_tags_29 },
    { 0xdbcf6bad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x72cf6bb1, SUC_lexicon_tags_25 },
    { 0x2f6bb2, SUC_lexicon_tags_45 },
    { 0x23af6bb1, SUC_lexicon_tags_2 },
    { 0x90cf6bb2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cf6bbc, SUC_lexicon_tags_31 },
    { 0x716f6bbd, SUC_lexicon_tags_1 },
    { 0x3aef6bbe, SUC_lexicon_tags_17 },
    { 0x5fef6bbd, SUC_lexicon_tags_55 },
    { 0x130f6bbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6f6bc3, SUC_lexicon_tags_0 },
    { 0xb7ef6bc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28cf6bc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe50f6bc8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0f6bcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2af6bce, SUC_lexicon_tags_63 },
    { 0x868f6bcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798f6bd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b0f6bd8, SUC_lexicon_tags_13 },
    { 0x77ef6bd8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x96cf6bdb, SUC_lexicon_tags_61 },
    { 0x4b4f6bdc, SUC_lexicon_tags_60 },
    { 0x400f6bdc, SUC_lexicon_tags_31 },
    { 0x994f6bdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9aef6be0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef6be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf6be6, SUC_lexicon_tags_33 },
    { 0xe96f6be7, SUC_lexicon_tags_17 },
    { 0x7e4f6be6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd90f6bea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8f6bed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x254f6bf0, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xa00f6bf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x830f6bf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30f6bfb, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x3d6f6bfd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0f6c02, SUC_lexicon_tags_15 },
    { 0x358f6c03, SUC_lexicon_tags_63 },
    { 0xc3ef6c03, SUC_lexicon_tags_77 },
    { 0xf9ef6c05, SUC_lexicon_tags_23 },
    { 0x96f6c06, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x28f6c08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfaf6c0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd06f6c0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aef6c0f, SUC_lexicon_tags_12 },
    { 0xf14f6c0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eef6c15, SUC_lexicon_tags_549 },
    { 0xafef6c15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2aef6c18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4f6c1c, SUC_lexicon_tags_72 },
    { 0x52f6c1d, SUC_lexicon_tags_66 },
    { 0x1a2f6c1d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0f6c24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0f6c27, SUC_lexicon_tags_295 },
    { 0x6c6f6c27, SUC_lexicon_tags_15 },
    { 0xe4af6c28, SUC_lexicon_tags_11 },
    { 0xdf4f6c2a, SUC_lexicon_tags_7 },
    { 0x696f6c2b, SUC_lexicon_tags_1 },
    { 0x826f6c2c, SUC_lexicon_tags_49 },
    { 0xcc6f6c2d, SUC_lexicon_tags_61 },
    { 0x3b0f6c2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ef6c31, SUC_lexicon_tags_17 },
    { 0xb6ef6c31, SUC_lexicon_tags_27 },
    { 0x77cf6c33, SUC_lexicon_tags_11 },
    { 0xfb4f6c34, SUC_lexicon_tags_21 },
    { 0xed8f6c35, SUC_lexicon_tags_14 },
    { 0x116f6c35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4f6c39, SUC_lexicon_tags_104 },
    { 0xc40f6c39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cf6c3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0f6c40, SUC_lexicon_tags_30 },
    { 0xcfcf6c41, SUC_lexicon_tags_11 },
    { 0x19ef6c40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea8f6c45, SUC_lexicon_tags_2 },
    { 0x136f6c46, SUC_lexicon_tags_37 },
    { 0x1faf6c45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x51cf6c49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ef6c4e, SUC_lexicon_tags_19 },
    { 0x962f6c4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x71cf6c52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ef6c5b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f6c5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ef6c61, SUC_lexicon_tags_25 },
    { 0x456f6c61, SUC_lexicon_tags_70 },
    { 0x20f6c61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1ccf6c65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eef6c6d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6f6c74, SUC_lexicon_tags_15 },
    { 0x22ef6c74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258f6c7e, SUC_lexicon_tags_43 },
    { 0xc8cf6c7f, SUC_lexicon_tags_8 },
    { 0x994f6c7f, SUC_lexicon_tags_23 },
    { 0xe86f6c81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ef6c84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x748f6c87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8f6c8b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8f6c92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaf6c97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae6f6c99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9a2f6c9b, SUC_lexicon_tags_23 },
    { 0x55af6c9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0f6ca0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f6ca4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160f6ca8, SUC_lexicon_tags_15 },
    { 0xbccf6ca8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdaf6cab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5af6cae, SUC_lexicon_tags_0 },
    { 0xc5ef6cae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x262f6cb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5caf6cb4, SUC_lexicon_tags_25 },
    { 0xd2cf6cb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7af6cb7, SUC_lexicon_tags_26 },
    { 0xdd2f6cb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a6f6cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7ef6cbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680f6cc3, SUC_lexicon_tags_9 },
    { 0xa40f6cc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48f6cc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea8f6ccb, SUC_lexicon_tags_30 },
    { 0x54f6ccc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x872f6ccf, SUC_lexicon_tags_63 },
    { 0x11af6ccf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cf6cd3, SUC_lexicon_tags_68 },
    { 0xdcaf6cd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc70f6cd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328f6cdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cf6ce2, SUC_lexicon_tags_31 },
    { 0x518f6ce3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cf6ce6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x124f6ce8, SUC_lexicon_tags_1585 },
    { 0, NULL },
    { 0x836f6cea, SUC_lexicon_tags_37 },
    { 0xc20f6cea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaf6cf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4f6cfb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496f6d01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ccf6d03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2a4f6d05, SUC_lexicon_tags_12 },
    { 0xb78f6d06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95cf6d08, SUC_lexicon_tags_1 },
    { 0x676f6d09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af6d10, SUC_lexicon_tags_25 },
    { 0x7e8f6d11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ef6d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bef6d16, SUC_lexicon_tags_11 },
    { 0xd6f6d17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69cf6d19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f8f6d1b, SUC_lexicon_tags_7 },
    { 0x314f6d1c, SUC_lexicon_tags_32 },
    { 0x492f6d1d, SUC_lexicon_tags_34 },
    { 0x6a6f6d1e, SUC_lexicon_tags_25 },
    { 0x1f4f6d1f, SUC_lexicon_tags_66 },
    { 0x8b8f6d20, SUC_lexicon_tags_7 },
    { 0x1baf6d21, SUC_lexicon_tags_37 },
    { 0x374f6d1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8f6d25, SUC_lexicon_tags_911 },
    { 0x6ccf6d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6f6d29, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd82f6d32, SUC_lexicon_tags_55 },
    { 0x1a0f6d33, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68af6d3a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366f6d42, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc4f6d46, SUC_lexicon_tags_22 },
    { 0x1a0f6d47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2f6d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ef6d52, SUC_lexicon_tags_37 },
    { 0xef6d52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ef6d57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96f6d5b, SUC_lexicon_tags_25 },
    { 0x240f6d5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1cef6d5e, SUC_lexicon_tags_30 },
    { 0xa2af6d5f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eef6d63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7af6d6f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2f6d71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8f6d76, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2f6d79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f6d7c, SUC_lexicon_tags_13 },
    { 0xc3cf6d7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cf6d81, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42f6d84, SUC_lexicon_tags_65 },
    { 0xf12f6d85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f6d91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x31ef6d93, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f6d96, SUC_lexicon_tags_1 },
    { 0xbc2f6d97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456f6d9b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ef6da2, SUC_lexicon_tags_25 },
    { 0xcf0f6da2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e0f6da5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf78f6da7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4f6daa, SUC_lexicon_tags_27 },
    { 0x616f6dab, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0f6db1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2a6f6db3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eef6db6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc04f6dbd, SUC_lexicon_tags_12 },
    { 0xa06f6dbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136f6dc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868f6dc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60f6dc9, SUC_lexicon_tags_12 },
    { 0xab2f6dca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6f6dcf, SUC_lexicon_tags_124 },
    { 0xf2cf6dcf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2f2f6dd2, SUC_lexicon_tags_14 },
    { 0x6b4f6dd2, SUC_lexicon_tags_28 },
    { 0x5fcf6dd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8af6dd9, SUC_lexicon_tags_23 },
    { 0x966f6dda, SUC_lexicon_tags_17 },
    { 0x12af6ddb, SUC_lexicon_tags_101 },
    { 0xf08f6ddc, SUC_lexicon_tags_16 },
    { 0xc7af6ddd, SUC_lexicon_tags_25 },
    { 0x8d0f6ddd, SUC_lexicon_tags_1 },
    { 0x592f6dda, SUC_lexicon_tags_7 },
    { 0xa4cf6de0, SUC_lexicon_tags_56 },
    { 0xba8f6de1, SUC_lexicon_tags_45 },
    { 0x334f6de2, SUC_lexicon_tags_37 },
    { 0x66f6dd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8f6de6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x310f6de8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8f6def, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x994f6df1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ef6df8, SUC_lexicon_tags_7 },
    { 0x4bcf6df9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4f6dfd, SUC_lexicon_tags_2 },
    { 0x620f6dfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3ef6e00, SUC_lexicon_tags_35 },
    { 0x7b2f6e01, SUC_lexicon_tags_47 },
    { 0xe38f6e02, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2f6e09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x544f6e0b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeaf6e0e, SUC_lexicon_tags_12 },
    { 0x466f6e0f, SUC_lexicon_tags_9 },
    { 0xbe2f6e0e, SUC_lexicon_tags_45 },
    { 0xe68f6e10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812f6e15, SUC_lexicon_tags_11 },
    { 0xdf8f6e15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510f6e1a, SUC_lexicon_tags_3 },
    { 0x746f6e1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2d0f6e1d, SUC_lexicon_tags_6 },
    { 0x7bef6e1e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38f6e21, SUC_lexicon_tags_11 },
    { 0x96ef6e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4f6e2b, SUC_lexicon_tags_31 },
    { 0xcf4f6e2c, SUC_lexicon_tags_6 },
    { 0x6c4f6e2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cf6e32, SUC_lexicon_tags_39 },
    { 0xc36f6e33, SUC_lexicon_tags_37 },
    { 0x8ef6e34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fef6e37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ef6e3a, SUC_lexicon_tags_30 },
    { 0x500f6e3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2cf6e3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0f6e40, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a8f6e44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72f6e48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x69cf6e4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634f6e55, SUC_lexicon_tags_28 },
    { 0x79ef6e56, SUC_lexicon_tags_28 },
    { 0xeef6e55, SUC_lexicon_tags_14 },
    { 0x686f6e55, SUC_lexicon_tags_2 },
    { 0xe0cf6e59, SUC_lexicon_tags_2 },
    { 0x928f6e5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38cf6e5c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cf6e60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456f6e66, SUC_lexicon_tags_25 },
    { 0xa1ef6e67, SUC_lexicon_tags_1 },
    { 0x8e8f6e68, SUC_lexicon_tags_17 },
    { 0x6aef6e68, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f6e6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53af6e6e, SUC_lexicon_tags_17 },
    { 0xca0f6e6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cf6e72, SUC_lexicon_tags_25 },
    { 0x21af6e72, SUC_lexicon_tags_65 },
    { 0xffaf6e74, SUC_lexicon_tags_11 },
    { 0xe96f6e75, SUC_lexicon_tags_8 },
    { 0xb50f6e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196f6e7a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbf4f6e7c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x3aaf6e7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950f6e82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbaef6e84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06f6e88, SUC_lexicon_tags_9 },
    { 0xc94f6e89, SUC_lexicon_tags_25 },
    { 0xc92f6e8a, SUC_lexicon_tags_28 },
    { 0x49cf6e8b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa1ef6e8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a0f6e8f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cf6e93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14f6e9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52af6ea0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd60f6ea2, SUC_lexicon_tags_17 },
    { 0xfb4f6ea2, SUC_lexicon_tags_31 },
    { 0x972f6ea4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x624f6ea7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918f6eaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4f6eb2, SUC_lexicon_tags_26 },
    { 0xd62f6eb3, SUC_lexicon_tags_31 },
    { 0xc30f6eb4, SUC_lexicon_tags_11 },
    { 0x8b2f6eb3, SUC_lexicon_tags_11 },
    { 0x1aef6eb6, SUC_lexicon_tags_15 },
    { 0x37cf6eb7, SUC_lexicon_tags_1 },
    { 0xbc4f6eb7, SUC_lexicon_tags_11 },
    { 0x74f6eb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4f6ebc, SUC_lexicon_tags_17 },
    { 0x756f6ebc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa82f6ec3, SUC_lexicon_tags_23 },
    { 0x514f6ec4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cf6ec8, SUC_lexicon_tags_14 },
    { 0x2b0f6ec8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabef6ecb, SUC_lexicon_tags_62 },
    { 0x7b8f6ecc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x634f6ecf, SUC_lexicon_tags_15 },
    { 0xe30f6ecf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x22cf6ed2, SUC_lexicon_tags_27 },
    { 0x9fef6ed3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21ef6ed5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22cf6ed7, SUC_lexicon_tags_25 },
    { 0x326f6ed8, SUC_lexicon_tags_9 },
    { 0xeef6ed9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x698f6edb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x420f6ede, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xe24f6ee1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6f6ee9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ef6eed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47ef6eef, SUC_lexicon_tags_37 },
    { 0xaa0f6ef0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6f6ef3, SUC_lexicon_tags_14 },
    { 0xcd4f6ef4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64f6ef9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf6f00, SUC_lexicon_tags_33 },
    { 0x112f6f01, SUC_lexicon_tags_127 },
    { 0x798f6f00, SUC_lexicon_tags_14 },
    { 0xea0f6f01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2f6f09, SUC_lexicon_tags_15 },
    { 0x9a8f6f09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe8cf6f0c, SUC_lexicon_tags_25 },
    { 0x3ccf6f0d, SUC_lexicon_tags_11 },
    { 0x66af6f0c, SUC_lexicon_tags_11 },
    { 0x446f6f0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100f6f13, SUC_lexicon_tags_25 },
    { 0x898f6f14, SUC_lexicon_tags_27 },
    { 0x48af6f15, SUC_lexicon_tags_37 },
    { 0xa0af6f16, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2f6f19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69af6f1c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6f6f24, SUC_lexicon_tags_23 },
    { 0xcdcf6f25, SUC_lexicon_tags_15 },
    { 0x7c0f6f24, SUC_lexicon_tags_12 },
    { 0x4fcf6f26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x440f6f2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7af6f2f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x930f6f32, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf5ef6f34, SUC_lexicon_tags_7 },
    { 0xd46f6f34, SUC_lexicon_tags_1 },
    { 0xb68f6f34, SUC_lexicon_tags_7 },
    { 0xa54f6f37, SUC_lexicon_tags_47 },
    { 0x436f6f38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x218f6f3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4af6f3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78f6f47, SUC_lexicon_tags_8 },
    { 0x134f6f48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaf6f4b, SUC_lexicon_tags_25 },
    { 0xf62f6f4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98f6f4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2f6f52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x450f6f55, SUC_lexicon_tags_23 },
    { 0x1bcf6f56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x958f6f59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x788f6f5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2f6f5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cf6f69, SUC_lexicon_tags_43 },
    { 0xce2f6f6a, SUC_lexicon_tags_47 },
    { 0x5e6f6f6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6cf6f6d, SUC_lexicon_tags_37 },
    { 0xa24f6f6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faf6f73, SUC_lexicon_tags_15 },
    { 0x64af6f74, SUC_lexicon_tags_15 },
    { 0xad8f6f74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faf6f7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e6f6f7c, SUC_lexicon_tags_63 },
    { 0xcd6f6f7d, SUC_lexicon_tags_7 },
    { 0xe0af6f7e, SUC_lexicon_tags_29 },
    { 0x976f6f7f, SUC_lexicon_tags_25 },
    { 0x81af6f7f, SUC_lexicon_tags_4 },
    { 0x1daf6f81, SUC_lexicon_tags_615 },
    { 0x784f6f7e, SUC_lexicon_tags_49 },
    { 0x4fcf6f83, SUC_lexicon_tags_1 },
    { 0x276f6f84, SUC_lexicon_tags_1 },
    { 0x6a2f6f83, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af6f88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88af6f8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18af6f92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb44f6f94, SUC_lexicon_tags_7 },
    { 0x57af6f95, SUC_lexicon_tags_1 },
    { 0x18f6f95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcf6f9a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4f6f9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648f6fa2, SUC_lexicon_tags_1 },
    { 0x30cf6fa2, SUC_lexicon_tags_11 },
    { 0xffcf6fa2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52f6fa7, SUC_lexicon_tags_7 },
    { 0x376f6fa8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8f6fb0, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0x44cf6fb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf6f6fb4, SUC_lexicon_tags_11 },
    { 0x95cf6fb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2f6fbb, SUC_lexicon_tags_37 },
    { 0x1e6f6fbc, SUC_lexicon_tags_1 },
    { 0xde0f6fbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x292f6fbf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6f6fc2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x180f6fc4, SUC_lexicon_tags_49 },
    { 0xed6f6fc5, SUC_lexicon_tags_2 },
    { 0x51ef6fc5, SUC_lexicon_tags_63 },
    { 0xa8ef6fc7, SUC_lexicon_tags_19 },
    { 0x638f6fc8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x22cf6fca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5af6fcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcf6fd0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x928f6fd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3baf6fd6, SUC_lexicon_tags_6 },
    { 0x1ecf6fd7, SUC_lexicon_tags_29 },
    { 0xde4f6fd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xacef6fda, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xacf6fdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57af6fde, SUC_lexicon_tags_27 },
    { 0xac4f6fdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cf6fe3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8f6fe9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00f6fef, SUC_lexicon_tags_25 },
    { 0x120f6fef, SUC_lexicon_tags_11 },
    { 0xc6cf6fef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa40f6ffe, SUC_lexicon_tags_6 },
    { 0x91af6fff, SUC_lexicon_tags_1 },
    { 0x458f7000, SUC_lexicon_tags_7 },
    { 0x9faf6fff, SUC_lexicon_tags_17 },
    { 0xf26f6fff, SUC_lexicon_tags_15 },
    { 0x300f7003, SUC_lexicon_tags_49 },
    { 0x738f7004, SUC_lexicon_tags_45 },
    { 0xceaf7002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3af7008, SUC_lexicon_tags_11 },
    { 0x79af7009, SUC_lexicon_tags_37 },
    { 0xcdef700a, SUC_lexicon_tags_23 },
    { 0x47af7008, SUC_lexicon_tags_60 },
    { 0xd68f700c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb5cf700e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc4f7010, SUC_lexicon_tags_21 },
    { 0xae0f7010, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2f701e, SUC_lexicon_tags_17 },
    { 0x2ef701e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794f7025, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cf702c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136f7034, SUC_lexicon_tags_15 },
    { 0x246f7035, SUC_lexicon_tags_63 },
    { 0x296f7036, SUC_lexicon_tags_49 },
    { 0xac0f7036, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x76cf703a, SUC_lexicon_tags_15 },
    { 0xf8f703b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x516f703e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88f7045, SUC_lexicon_tags_25 },
    { 0xbd8f7046, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe58f7048, SUC_lexicon_tags_12 },
    { 0xd8f7049, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x660f704b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccf704e, SUC_lexicon_tags_43 },
    { 0xe66f704f, SUC_lexicon_tags_11 },
    { 0x9a0f7050, SUC_lexicon_tags_8 },
    { 0xa6cf7051, SUC_lexicon_tags_46 },
    { 0x978f7052, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfccf7055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964f7059, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ef7063, SUC_lexicon_tags_15 },
    { 0x90ef7064, SUC_lexicon_tags_26 },
    { 0xfa2f7064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cf706b, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x784f706e, SUC_lexicon_tags_26 },
    { 0x632f706e, SUC_lexicon_tags_11 },
    { 0xd54f706e, SUC_lexicon_tags_32 },
    { 0x444f7071, SUC_lexicon_tags_87 },
    { 0xb0ef7070, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xda4f7074, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc18f7076, SUC_lexicon_tags_63 },
    { 0x954f7077, SUC_lexicon_tags_27 },
    { 0x3aaf7078, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cef707e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88f7083, SUC_lexicon_tags_1 },
    { 0x69cf7084, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504f7088, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cf708c, SUC_lexicon_tags_25 },
    { 0x476f708c, SUC_lexicon_tags_5 },
    { 0xc3cf708e, SUC_lexicon_tags_32 },
    { 0x1e8f708f, SUC_lexicon_tags_37 },
    { 0x238f708f, SUC_lexicon_tags_37 },
    { 0x35af7091, SUC_lexicon_tags_40 },
    { 0xc3ef708f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2f7096, SUC_lexicon_tags_14 },
    { 0xb30f7097, SUC_lexicon_tags_37 },
    { 0x974f7098, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cf709b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x484f709d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f70a3, SUC_lexicon_tags_38 },
    { 0xf24f70a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34f70a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cf70ab, SUC_lexicon_tags_1 },
    { 0xbb0f70ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x93ef70ae, SUC_lexicon_tags_37 },
    { 0x15cf70af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecf70b5, SUC_lexicon_tags_2 },
    { 0x770f70b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc64f70bc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a2f70c0, SUC_lexicon_tags_2 },
    { 0xf22f70c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x542f70c3, SUC_lexicon_tags_49 },
    { 0x618f70c4, SUC_lexicon_tags_56 },
    { 0xd44f70c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10af70c7, SUC_lexicon_tags_25 },
    { 0x51cf70c8, SUC_lexicon_tags_7 },
    { 0x7cf70c7, SUC_lexicon_tags_0 },
    { 0xe62f70ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50af70cc, SUC_lexicon_tags_37 },
    { 0xc56f70cd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388f70df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcf70e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f70e8, SUC_lexicon_tags_11 },
    { 0x41cf70e9, SUC_lexicon_tags_17 },
    { 0xb78f70e8, SUC_lexicon_tags_33 },
    { 0xd2cf70e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf88f70ee, SUC_lexicon_tags_38 },
    { 0x8e0f70ef, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acf70f5, SUC_lexicon_tags_21 },
    { 0x9ccf70f6, SUC_lexicon_tags_43 },
    { 0x52ef70f6, SUC_lexicon_tags_37 },
    { 0xd98f70f7, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ef70fb, SUC_lexicon_tags_7 },
    { 0x570f70fc, SUC_lexicon_tags_46 },
    { 0x214f70fd, SUC_lexicon_tags_1 },
    { 0xff6f70fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95af7104, SUC_lexicon_tags_32 },
    { 0x8bcf7104, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc56f7108, SUC_lexicon_tags_1 },
    { 0x79cf7108, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6af710d, SUC_lexicon_tags_1026 },
    { 0xa28f710e, SUC_lexicon_tags_55 },
    { 0x25cf710d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecf7113, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356f7117, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6eaf7119, SUC_lexicon_tags_4 },
    { 0xc72f711a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaf7120, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51af7129, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ef712e, SUC_lexicon_tags_37 },
    { 0x58cf712f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4a6f7131, SUC_lexicon_tags_37 },
    { 0x900f7132, SUC_lexicon_tags_17 },
    { 0x7b8f7133, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298f7138, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb16f713a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4f713f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc72f7141, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568f714a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828f7153, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2f7157, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x296f715a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafaf715c, SUC_lexicon_tags_3 },
    { 0x7def715d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f2f715f, SUC_lexicon_tags_0 },
    { 0x4a0f7160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f7166, SUC_lexicon_tags_19 },
    { 0x338f7166, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x540f716a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266f716e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6cf7170, SUC_lexicon_tags_91 },
    { 0xe76f7171, SUC_lexicon_tags_37 },
    { 0xd86f7171, SUC_lexicon_tags_25 },
    { 0x97af7172, SUC_lexicon_tags_485 },
    { 0x6c6f7174, SUC_lexicon_tags_11 },
    { 0xb48f7174, SUC_lexicon_tags_17 },
    { 0x9caf7175, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x172f7178, SUC_lexicon_tags_7 },
    { 0xc66f7179, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cf717c, SUC_lexicon_tags_15 },
    { 0x11cf717d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf8f717f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ef7182, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x476f7185, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74af7187, SUC_lexicon_tags_22 },
    { 0x2d6f7188, SUC_lexicon_tags_37 },
    { 0xc42f7187, SUC_lexicon_tags_9 },
    { 0x5aef718a, SUC_lexicon_tags_63 },
    { 0xf9af718b, SUC_lexicon_tags_60 },
    { 0xbc0f718c, SUC_lexicon_tags_23 },
    { 0xa10f718d, SUC_lexicon_tags_16 },
    { 0x4f6f718e, SUC_lexicon_tags_45 },
    { 0x4c8f7188, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4f7191, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46af7193, SUC_lexicon_tags_11 },
    { 0xc70f7194, SUC_lexicon_tags_11 },
    { 0x63cf7193, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4ef7197, SUC_lexicon_tags_0 },
    { 0x6c2f7197, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x714f719b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f71a1, SUC_lexicon_tags_1 },
    { 0x63cf71a1, SUC_lexicon_tags_11 },
    { 0xb64f71a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d8f71a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6f71a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cf71ac, SUC_lexicon_tags_38 },
    { 0xb18f71ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8f71b8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe80f71ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69af71c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x540f71c5, SUC_lexicon_tags_79 },
    { 0x140f71c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714f71cb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaf71ce, SUC_lexicon_tags_12 },
    { 0x804f71cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f71d2, SUC_lexicon_tags_30 },
    { 0xedef71d3, SUC_lexicon_tags_12 },
    { 0xf6cf71d4, SUC_lexicon_tags_30 },
    { 0xf18f71d5, SUC_lexicon_tags_25 },
    { 0x798f71d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44f71da, SUC_lexicon_tags_45 },
    { 0xc10f71da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97ef71dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2f71e1, SUC_lexicon_tags_25 },
    { 0xd5cf71e1, SUC_lexicon_tags_15 },
    { 0x2b4f71e3, SUC_lexicon_tags_2 },
    { 0x1cef71e4, SUC_lexicon_tags_8 },
    { 0x396f71e2, SUC_lexicon_tags_5 },
    { 0xf1cf71e6, SUC_lexicon_tags_29 },
    { 0xd8af71e6, SUC_lexicon_tags_55 },
    { 0xc5af71e6, SUC_lexicon_tags_91 },
    { 0xbbaf71e2, SUC_lexicon_tags_68 },
    { 0x846f71ea, SUC_lexicon_tags_27 },
    { 0x36cf71e2, SUC_lexicon_tags_19 },
    { 0x6c6f71e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cf71ef, SUC_lexicon_tags_26 },
    { 0x194f71ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f71f3, SUC_lexicon_tags_66 },
    { 0xf0f71f3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2f71f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaf71fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x186f71ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x782f7202, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1b4f7204, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x486f7206, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7af720f, SUC_lexicon_tags_2 },
    { 0x6a6f7210, SUC_lexicon_tags_19 },
    { 0xf1cf720f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x780f7214, SUC_lexicon_tags_652 },
    { 0, NULL },
    { 0xfd8f7216, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988f721c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90f7227, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc14f7229, SUC_lexicon_tags_11 },
    { 0x4e4f722a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x564f722c, SUC_lexicon_tags_27 },
    { 0x60cf722d, SUC_lexicon_tags_11 },
    { 0x25ef722e, SUC_lexicon_tags_17 },
    { 0xf80f722e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20f7234, SUC_lexicon_tags_11 },
    { 0x6c6f7234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19cf7237, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28f723a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72f723d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58cf723f, SUC_lexicon_tags_30 },
    { 0x85cf7240, SUC_lexicon_tags_12 },
    { 0x4c2f7241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4f7244, SUC_lexicon_tags_12 },
    { 0x418f7245, SUC_lexicon_tags_12 },
    { 0x858f7244, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xabcf7249, SUC_lexicon_tags_7 },
    { 0x5a8f724a, SUC_lexicon_tags_6 },
    { 0x322f724b, SUC_lexicon_tags_12 },
    { 0x400f724a, SUC_lexicon_tags_11 },
    { 0xd28f724d, SUC_lexicon_tags_11 },
    { 0x768f724d, SUC_lexicon_tags_623 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f7253, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d8f7255, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x156f7257, SUC_lexicon_tags_55 },
    { 0x444f7257, SUC_lexicon_tags_7 },
    { 0x42ef7259, SUC_lexicon_tags_11 },
    { 0x3f2f725a, SUC_lexicon_tags_37 },
    { 0x324f7259, SUC_lexicon_tags_15 },
    { 0x8a4f725a, SUC_lexicon_tags_38 },
    { 0xa30f725d, SUC_lexicon_tags_37 },
    { 0x38af725e, SUC_lexicon_tags_16 },
    { 0xc3cf725e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bef726a, SUC_lexicon_tags_33 },
    { 0xc94f726a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f7272, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ef7276, SUC_lexicon_tags_55 },
    { 0x720f7277, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26f727b, SUC_lexicon_tags_37 },
    { 0xe04f727c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecf7280, SUC_lexicon_tags_30 },
    { 0xd8ef7281, SUC_lexicon_tags_2 },
    { 0xc10f7282, SUC_lexicon_tags_11 },
    { 0x92cf7283, SUC_lexicon_tags_40 },
    { 0x492f7282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x308f7286, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178f728c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9af728e, SUC_lexicon_tags_37 },
    { 0xc36f728e, SUC_lexicon_tags_17 },
    { 0x6c4f728f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50f7293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f8f7295, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe0ef7297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e8f7299, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcf72a6, SUC_lexicon_tags_7 },
    { 0xd6f72a7, SUC_lexicon_tags_12 },
    { 0x640f72a8, SUC_lexicon_tags_360 },
    { 0x3c8f72a8, SUC_lexicon_tags_25 },
    { 0xc54f72a9, SUC_lexicon_tags_1 },
    { 0xe1ef72aa, SUC_lexicon_tags_14 },
    { 0x942f72ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef72b4, SUC_lexicon_tags_710 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaf72b9, SUC_lexicon_tags_17 },
    { 0x9c6f72ba, SUC_lexicon_tags_46 },
    { 0xfa6f72bb, SUC_lexicon_tags_35 },
    { 0xa04f72bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21af72bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0af72c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x730f72c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80f72c9, SUC_lexicon_tags_47 },
    { 0xe14f72c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9faf72cc, SUC_lexicon_tags_1 },
    { 0xde8f72cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f6f72cf, SUC_lexicon_tags_14 },
    { 0x11cf72d0, SUC_lexicon_tags_26 },
    { 0x266f72d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4e2f72d3, SUC_lexicon_tags_55 },
    { 0x846f72d4, SUC_lexicon_tags_52 },
    { 0x52af72d5, SUC_lexicon_tags_6 },
    { 0x2d4f72d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1af72d8, SUC_lexicon_tags_37 },
    { 0xbbef72d8, SUC_lexicon_tags_47 },
    { 0x6f4f72da, SUC_lexicon_tags_12 },
    { 0x58cf72db, SUC_lexicon_tags_28 },
    { 0xf8ef72db, SUC_lexicon_tags_37 },
    { 0x2c2f72da, SUC_lexicon_tags_2 },
    { 0x6c0f72de, SUC_lexicon_tags_23 },
    { 0x464f72df, SUC_lexicon_tags_31 },
    { 0xf0f72de, SUC_lexicon_tags_17 },
    { 0x944f72dc, SUC_lexicon_tags_17 },
    { 0x484f72e2, SUC_lexicon_tags_26 },
    { 0xc6f72dd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f72e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcf72eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23af72ee, SUC_lexicon_tags_17 },
    { 0x8b4f72ef, SUC_lexicon_tags_29 },
    { 0xfa6f72ef, SUC_lexicon_tags_11 },
    { 0x33cf72ee, SUC_lexicon_tags_12 },
    { 0x652f72f2, SUC_lexicon_tags_14 },
    { 0xdbcf72f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94ef72f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cf72f9, SUC_lexicon_tags_26 },
    { 0xeecf72fa, SUC_lexicon_tags_80 },
    { 0xd98f72fb, SUC_lexicon_tags_16 },
    { 0x290f72fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc3af72fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1eaf7300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20f7308, SUC_lexicon_tags_6 },
    { 0xdaf7308, SUC_lexicon_tags_60 },
    { 0x66f7309, SUC_lexicon_tags_37 },
    { 0xb02f730a, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80f730f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6d4f7311, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x64af7313, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ef7318, SUC_lexicon_tags_7 },
    { 0xac6f7319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x842f731f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120f7325, SUC_lexicon_tags_16 },
    { 0x364f7325, SUC_lexicon_tags_250 },
    { 0x742f7326, SUC_lexicon_tags_15 },
    { 0xfa4f7328, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee4f732d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f7332, SUC_lexicon_tags_15 },
    { 0x902f7333, SUC_lexicon_tags_12 },
    { 0xc26f7333, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbef7336, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa06f733a, SUC_lexicon_tags_87 },
    { 0x162f733b, SUC_lexicon_tags_17 },
    { 0xcc2f733c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xe04f733e, SUC_lexicon_tags_37 },
    { 0xaaaf733f, SUC_lexicon_tags_25 },
    { 0x142f733f, SUC_lexicon_tags_17 },
    { 0x8c8f7341, SUC_lexicon_tags_37 },
    { 0xc44f733e, SUC_lexicon_tags_23 },
    { 0x6ecf7340, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52f7346, SUC_lexicon_tags_7 },
    { 0x7b4f7346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10ef7349, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7def734b, SUC_lexicon_tags_2 },
    { 0x9b4f734c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856f7354, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3af7356, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x178f7359, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06f7360, SUC_lexicon_tags_14 },
    { 0x320f7360, SUC_lexicon_tags_48 },
    { 0x304f7362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58af7364, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcaef7366, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x122f7368, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5e4f736a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8bef736c, SUC_lexicon_tags_68 },
    { 0xa30f736d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632f7374, SUC_lexicon_tags_31 },
    { 0x2dcf7374, SUC_lexicon_tags_14 },
    { 0xa28f7375, SUC_lexicon_tags_12 },
    { 0xd9cf7377, SUC_lexicon_tags_19 },
    { 0x610f7378, SUC_lexicon_tags_25 },
    { 0x1faf7379, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6cf737b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44f7380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccf7386, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x48f7388, SUC_lexicon_tags_0 },
    { 0x7f4f7389, SUC_lexicon_tags_15 },
    { 0xb26f738a, SUC_lexicon_tags_9 },
    { 0x5eaf738b, SUC_lexicon_tags_12 },
    { 0x8e8f738c, SUC_lexicon_tags_11 },
    { 0xf80f738d, SUC_lexicon_tags_2 },
    { 0x86cf738e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb74f7390, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfacf7392, SUC_lexicon_tags_8 },
    { 0xf36f7392, SUC_lexicon_tags_17 },
    { 0x690f7393, SUC_lexicon_tags_17 },
    { 0x316f7395, SUC_lexicon_tags_15 },
    { 0xad6f7396, SUC_lexicon_tags_33 },
    { 0x1c8f7397, SUC_lexicon_tags_60 },
    { 0x460f7398, SUC_lexicon_tags_47 },
    { 0xd2f7399, SUC_lexicon_tags_31 },
    { 0x5f6f7397, SUC_lexicon_tags_151 },
    { 0x248f739b, SUC_lexicon_tags_12 },
    { 0x1ef739b, SUC_lexicon_tags_15 },
    { 0xb3af7392, SUC_lexicon_tags_29 },
    { 0x3a2f739c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cf73a2, SUC_lexicon_tags_84 },
    { 0xf54f73a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x818f73a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f73a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7af73b0, SUC_lexicon_tags_27 },
    { 0xdaf73b1, SUC_lexicon_tags_208 },
    { 0xe4f73b2, SUC_lexicon_tags_9 },
    { 0x41ef73b3, SUC_lexicon_tags_37 },
    { 0xc9af73b4, SUC_lexicon_tags_21 },
    { 0x8e6f73b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83af73b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x654f73ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5ccf73bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x746f73be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfaf73c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36f73c4, SUC_lexicon_tags_15 },
    { 0xcbef73c5, SUC_lexicon_tags_15 },
    { 0x490f73c5, SUC_lexicon_tags_37 },
    { 0xf84f73c7, SUC_lexicon_tags_28 },
    { 0xecef73c8, SUC_lexicon_tags_12 },
    { 0x20f73c7, SUC_lexicon_tags_15 },
    { 0x8f2f73c9, SUC_lexicon_tags_0 },
    { 0x1f6f73c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x728f73d0, SUC_lexicon_tags_38 },
    { 0xe7ef73d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccf73d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2def73d8, SUC_lexicon_tags_12 },
    { 0x6a6f73d8, SUC_lexicon_tags_2 },
    { 0x60cf73d8, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x7c0f73dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x764f73de, SUC_lexicon_tags_4 },
    { 0x64f73df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6f73e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x170f73e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8f73e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8f73eb, SUC_lexicon_tags_32 },
    { 0x7c8f73ec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x514f73ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x844f73f3, SUC_lexicon_tags_37 },
    { 0xcb2f73f4, SUC_lexicon_tags_56 },
    { 0x7b2f73f4, SUC_lexicon_tags_4 },
    { 0xfa6f73f5, SUC_lexicon_tags_63 },
    { 0xf70f73f7, SUC_lexicon_tags_7 },
    { 0xf14f73f5, SUC_lexicon_tags_37 },
    { 0xfd0f73f8, SUC_lexicon_tags_87 },
    { 0xc38f73f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0af73fd, SUC_lexicon_tags_33 },
    { 0x4c6f73fe, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x132f7401, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x474f7404, SUC_lexicon_tags_11 },
    { 0x93ef7405, SUC_lexicon_tags_26 },
    { 0xd8cf7406, SUC_lexicon_tags_25 },
    { 0x4baf7407, SUC_lexicon_tags_1 },
    { 0x110f7408, SUC_lexicon_tags_12 },
    { 0x53cf7405, SUC_lexicon_tags_31 },
    { 0x920f7405, SUC_lexicon_tags_37 },
    { 0xfe2f740b, SUC_lexicon_tags_37 },
    { 0xeb4f740a, SUC_lexicon_tags_17 },
    { 0xc90f740d, SUC_lexicon_tags_106 },
    { 0x6c6f740d, SUC_lexicon_tags_18 },
    { 0x31cf740f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xcacf7411, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6af7413, SUC_lexicon_tags_2 },
    { 0xdacf7414, SUC_lexicon_tags_15 },
    { 0xc12f7415, SUC_lexicon_tags_49 },
    { 0xc30f7415, SUC_lexicon_tags_26 },
    { 0x4aaf7414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf741b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcf741d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2f7420, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x35ef7422, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbdcf7424, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cf7428, SUC_lexicon_tags_11 },
    { 0xfacf7428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2f742c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0f7432, SUC_lexicon_tags_66 },
    { 0x90ef7432, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6f7437, SUC_lexicon_tags_7 },
    { 0xa6af7438, SUC_lexicon_tags_11 },
    { 0x4dcf7439, SUC_lexicon_tags_8 },
    { 0xca0f7439, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde0f743e, SUC_lexicon_tags_14 },
    { 0x1f8f743f, SUC_lexicon_tags_2 },
    { 0xb0f743f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f7443, SUC_lexicon_tags_4 },
    { 0x8af7443, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0f7447, SUC_lexicon_tags_0 },
    { 0x2f4f7447, SUC_lexicon_tags_37 },
    { 0x70af7447, SUC_lexicon_tags_7 },
    { 0xf6af744a, SUC_lexicon_tags_43 },
    { 0x1d8f744b, SUC_lexicon_tags_15 },
    { 0xa14f744c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76f744e, SUC_lexicon_tags_11 },
    { 0x21af744f, SUC_lexicon_tags_29 },
    { 0xc34f7450, SUC_lexicon_tags_83 },
    { 0x6bcf744f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x510f7455, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9def7457, SUC_lexicon_tags_14 },
    { 0xae8f7458, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460f7462, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x302f7464, SUC_lexicon_tags_31 },
    { 0x16cf7464, SUC_lexicon_tags_9 },
    { 0x92cf7465, SUC_lexicon_tags_25 },
    { 0xcacf7466, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80f746b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a0f746d, SUC_lexicon_tags_9 },
    { 0xdaef746e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86f7471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f7475, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6ccf7477, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b6f7479, SUC_lexicon_tags_17 },
    { 0xff4f747a, SUC_lexicon_tags_14 },
    { 0x860f7479, SUC_lexicon_tags_37 },
    { 0xf22f747c, SUC_lexicon_tags_15 },
    { 0xf28f747d, SUC_lexicon_tags_37 },
    { 0xbeaf747c, SUC_lexicon_tags_201 },
    { 0xd14f747f, SUC_lexicon_tags_11 },
    { 0x61af7479, SUC_lexicon_tags_13 },
    { 0x478f7481, SUC_lexicon_tags_27 },
    { 0xbd8f7481, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb64f7486, SUC_lexicon_tags_11 },
    { 0xf9cf7487, SUC_lexicon_tags_17 },
    { 0xfd4f7488, SUC_lexicon_tags_49 },
    { 0x232f7489, SUC_lexicon_tags_55 },
    { 0x930f7489, SUC_lexicon_tags_11 },
    { 0xd5ef748b, SUC_lexicon_tags_7 },
    { 0x26f748a, SUC_lexicon_tags_47 },
    { 0x19cf748d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x8a0f748f, SUC_lexicon_tags_66 },
    { 0x65ef7490, SUC_lexicon_tags_7 },
    { 0xe76f748f, SUC_lexicon_tags_12 },
    { 0x6fcf7492, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5a8f7494, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2f7497, SUC_lexicon_tags_14 },
    { 0x8f6f7498, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8f749b, SUC_lexicon_tags_7 },
    { 0x46af749b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4af749f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x684f74a1, SUC_lexicon_tags_45 },
    { 0xd8ef74a1, SUC_lexicon_tags_37 },
    { 0x5ef74a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf18f74a5, SUC_lexicon_tags_2 },
    { 0x878f74a5, SUC_lexicon_tags_498 },
    { 0xbcf74a7, SUC_lexicon_tags_37 },
    { 0x7a8f74a6, SUC_lexicon_tags_37 },
    { 0xbc0f74a9, SUC_lexicon_tags_49 },
    { 0x9b0f74a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde2f74af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00f74b5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ef74b8, SUC_lexicon_tags_37 },
    { 0x6d2f74b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8f74bf, SUC_lexicon_tags_15 },
    { 0x584f74c0, SUC_lexicon_tags_57 },
    { 0x418f74bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb44f74c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f74c8, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ef74cb, SUC_lexicon_tags_136 },
    { 0x7a8f74cc, SUC_lexicon_tags_37 },
    { 0x99af74cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910f74db, SUC_lexicon_tags_69 },
    { 0x56cf74dc, SUC_lexicon_tags_3 },
    { 0x40af74db, SUC_lexicon_tags_12 },
    { 0x9d6f74de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x25cf74e0, SUC_lexicon_tags_9 },
    { 0x172f74e1, SUC_lexicon_tags_11 },
    { 0x332f74e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x796f74e4, SUC_lexicon_tags_27 },
    { 0x7a8f74e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4def74e9, SUC_lexicon_tags_15 },
    { 0xd80f74e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaf74ed, SUC_lexicon_tags_63 },
    { 0x438f74ed, SUC_lexicon_tags_1 },
    { 0xe56f74ee, SUC_lexicon_tags_14 },
    { 0x540f74ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0f74f2, SUC_lexicon_tags_14 },
    { 0x20af74f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd6f74f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2cf74f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadcf74f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed6f7501, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x634f7503, SUC_lexicon_tags_6 },
    { 0xffcf7504, SUC_lexicon_tags_2 },
    { 0x71cf7505, SUC_lexicon_tags_23 },
    { 0x8a0f7503, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83ef7508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8f750c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xadef750f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d2f7511, SUC_lexicon_tags_46 },
    { 0x6d2f7512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36f7518, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53cf751a, SUC_lexicon_tags_0 },
    { 0x182f751a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x830f751d, SUC_lexicon_tags_49 },
    { 0xec6f751e, SUC_lexicon_tags_84 },
    { 0x786f751e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2af7521, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54f7527, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6f752b, SUC_lexicon_tags_87 },
    { 0x3e8f752b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cef7539, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xb84f753b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bef753d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208f7543, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb44f7548, SUC_lexicon_tags_2 },
    { 0x4f7549, SUC_lexicon_tags_37 },
    { 0x82cf7548, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69ef754c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x65cf754e, SUC_lexicon_tags_12 },
    { 0xf24f754f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316f7553, SUC_lexicon_tags_66 },
    { 0x7eaf7554, SUC_lexicon_tags_68 },
    { 0xc80f7555, SUC_lexicon_tags_17 },
    { 0x1c6f7556, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f7561, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8f7564, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0f7568, SUC_lexicon_tags_1 },
    { 0x7d8f7569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f756d, SUC_lexicon_tags_0 },
    { 0xe98f756e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6f7571, SUC_lexicon_tags_21 },
    { 0x9bcf7571, SUC_lexicon_tags_7 },
    { 0xe38f7571, SUC_lexicon_tags_104 },
    { 0x12cf7571, SUC_lexicon_tags_45 },
    { 0x920f7572, SUC_lexicon_tags_15 },
    { 0xdfcf7576, SUC_lexicon_tags_29 },
    { 0x6f8f7577, SUC_lexicon_tags_25 },
    { 0x918f7578, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0f757f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ef7585, SUC_lexicon_tags_43 },
    { 0xf10f7586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeef7590, SUC_lexicon_tags_33 },
    { 0xcfef7590, SUC_lexicon_tags_14 },
    { 0xdd4f7592, SUC_lexicon_tags_46 },
    { 0x6b4f7593, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2f7596, SUC_lexicon_tags_37 },
    { 0x2acf7597, SUC_lexicon_tags_61 },
    { 0x744f7596, SUC_lexicon_tags_17 },
    { 0x20cf7599, SUC_lexicon_tags_12 },
    { 0x824f759a, SUC_lexicon_tags_17 },
    { 0x2ccf7599, SUC_lexicon_tags_43 },
    { 0x4b6f759b, SUC_lexicon_tags_14 },
    { 0x88cf759d, SUC_lexicon_tags_9 },
    { 0xd04f759d, SUC_lexicon_tags_15 },
    { 0x43af759f, SUC_lexicon_tags_3 },
    { 0x520f759e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x88af75a2, SUC_lexicon_tags_17 },
    { 0x642f75a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x65cf75a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30af75aa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4acf75ac, SUC_lexicon_tags_30 },
    { 0x7cf75ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x970f75af, SUC_lexicon_tags_45 },
    { 0x226f75af, SUC_lexicon_tags_12 },
    { 0x66af75af, SUC_lexicon_tags_47 },
    { 0xaa4f75b1, SUC_lexicon_tags_63 },
    { 0x24f75b2, SUC_lexicon_tags_4 },
    { 0xe0ef75b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400f75b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82f75bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f0f75bd, SUC_lexicon_tags_37 },
    { 0xa26f75bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418f75c3, SUC_lexicon_tags_37 },
    { 0xbccf75c3, SUC_lexicon_tags_34 },
    { 0xa68f75c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ef75c9, SUC_lexicon_tags_71 },
    { 0xb80f75ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3af75cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fef75d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe56f75d3, SUC_lexicon_tags_25 },
    { 0x43ef75d4, SUC_lexicon_tags_27 },
    { 0x904f75d4, SUC_lexicon_tags_46 },
    { 0x77cf75d3, SUC_lexicon_tags_11 },
    { 0x7eaf75d7, SUC_lexicon_tags_7 },
    { 0xa2ef75d8, SUC_lexicon_tags_17 },
    { 0x92cf75d3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x536f75dc, SUC_lexicon_tags_11 },
    { 0xd6ef75dd, SUC_lexicon_tags_1 },
    { 0xb2f75de, SUC_lexicon_tags_16 },
    { 0x8e8f75df, SUC_lexicon_tags_28 },
    { 0x946f75e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd28f75e2, SUC_lexicon_tags_29 },
    { 0xab2f75e3, SUC_lexicon_tags_0 },
    { 0x442f75e4, SUC_lexicon_tags_38 },
    { 0x408f75e4, SUC_lexicon_tags_33 },
    { 0x6af75e5, SUC_lexicon_tags_8 },
    { 0xe90f75e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2f75eb, SUC_lexicon_tags_0 },
    { 0xfa2f75eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f75f1, SUC_lexicon_tags_11 },
    { 0xfeaf75f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe20f75f4, SUC_lexicon_tags_45 },
    { 0x75ef75f5, SUC_lexicon_tags_0 },
    { 0x21af75f6, SUC_lexicon_tags_12 },
    { 0x454f75f6, SUC_lexicon_tags_8 },
    { 0x6af75f7, SUC_lexicon_tags_23 },
    { 0x562f75f4, SUC_lexicon_tags_43 },
    { 0x300f75fa, SUC_lexicon_tags_0 },
    { 0x388f75fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad8f75fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c0f75ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf12f7601, SUC_lexicon_tags_25 },
    { 0x1d8f7602, SUC_lexicon_tags_43 },
    { 0x65cf7603, SUC_lexicon_tags_12 },
    { 0x9c2f7604, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6f7609, SUC_lexicon_tags_146 },
    { 0x4f8f760a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe18f760c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4f7610, SUC_lexicon_tags_21 },
    { 0xcecf7610, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x208f7614, SUC_lexicon_tags_15 },
    { 0xfeef7614, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe4cf7617, SUC_lexicon_tags_60 },
    { 0x592f7617, SUC_lexicon_tags_88 },
    { 0xe86f7619, SUC_lexicon_tags_2 },
    { 0x488f761a, SUC_lexicon_tags_1 },
    { 0x2eaf7619, SUC_lexicon_tags_2 },
    { 0x6af7618, SUC_lexicon_tags_1 },
    { 0xd38f761c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12af761f, SUC_lexicon_tags_27 },
    { 0x4aef761f, SUC_lexicon_tags_11 },
    { 0xe76f7621, SUC_lexicon_tags_12 },
    { 0x7c4f7620, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6f7625, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97af762c, SUC_lexicon_tags_47 },
    { 0x75cf762d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3d2f762f, SUC_lexicon_tags_11 },
    { 0xc94f7630, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eef7635, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4f763d, SUC_lexicon_tags_28 },
    { 0xad6f763d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf6ef7640, SUC_lexicon_tags_12 },
    { 0x29ef7640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2baf7644, SUC_lexicon_tags_338 },
    { 0, NULL },
    { 0x510f7646, SUC_lexicon_tags_1 },
    { 0x2b8f7646, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae4f764c, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0xe22f764e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cf7653, SUC_lexicon_tags_63 },
    { 0x63ef7653, SUC_lexicon_tags_47 },
    { 0x9b6f7655, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0x696f7658, SUC_lexicon_tags_45 },
    { 0xf42f7658, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6f765c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4f765f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f7662, SUC_lexicon_tags_14 },
    { 0xac6f7663, SUC_lexicon_tags_2 },
    { 0xdb0f7664, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90f766e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf28f7670, SUC_lexicon_tags_15 },
    { 0xa46f7671, SUC_lexicon_tags_37 },
    { 0xa78f7671, SUC_lexicon_tags_38 },
    { 0x4baf7671, SUC_lexicon_tags_39 },
    { 0x43af7673, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aef767d, SUC_lexicon_tags_37 },
    { 0x82cf767e, SUC_lexicon_tags_49 },
    { 0x8def767e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa98f7683, SUC_lexicon_tags_7 },
    { 0x660f7683, SUC_lexicon_tags_26 },
    { 0xd20f7684, SUC_lexicon_tags_17 },
    { 0xb86f7686, SUC_lexicon_tags_107 },
    { 0x448f7686, SUC_lexicon_tags_37 },
    { 0xfeef7683, SUC_lexicon_tags_17 },
    { 0x4eef7689, SUC_lexicon_tags_47 },
    { 0x454f7688, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32f768d, SUC_lexicon_tags_43 },
    { 0xc58f768e, SUC_lexicon_tags_12 },
    { 0x7a0f768d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2f769a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x192f769c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0f769f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbeef76a1, SUC_lexicon_tags_49 },
    { 0x95af76a2, SUC_lexicon_tags_85 },
    { 0x2a0f76a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ef76a7, SUC_lexicon_tags_14 },
    { 0x3a2f76a8, SUC_lexicon_tags_45 },
    { 0x26af76a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaef76ac, SUC_lexicon_tags_43 },
    { 0x4a4f76ad, SUC_lexicon_tags_15 },
    { 0x7eaf76ae, SUC_lexicon_tags_0 },
    { 0x5baf76ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19ef76b1, SUC_lexicon_tags_23 },
    { 0xf8cf76b1, SUC_lexicon_tags_63 },
    { 0xf5ef76b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb0f76b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ef76b9, SUC_lexicon_tags_37 },
    { 0x85ef76ba, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ef76c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc34f76c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec0f76c7, SUC_lexicon_tags_14 },
    { 0x866f76c8, SUC_lexicon_tags_12 },
    { 0x2fcf76c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222f76d2, SUC_lexicon_tags_8 },
    { 0x5d0f76d3, SUC_lexicon_tags_15 },
    { 0x50af76d4, SUC_lexicon_tags_59 },
    { 0xe08f76d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c4f76d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x774f76d9, SUC_lexicon_tags_1 },
    { 0xf0cf76da, SUC_lexicon_tags_6 },
    { 0xb68f76da, SUC_lexicon_tags_15 },
    { 0xe12f76d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91af76e0, SUC_lexicon_tags_25 },
    { 0xb50f76e1, SUC_lexicon_tags_15 },
    { 0x7e8f76e2, SUC_lexicon_tags_144 },
    { 0xcc6f76e3, SUC_lexicon_tags_8 },
    { 0x314f76e1, SUC_lexicon_tags_10 },
    { 0x47cf76e5, SUC_lexicon_tags_8 },
    { 0x11af76e6, SUC_lexicon_tags_29 },
    { 0x6d2f76e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x272f76e9, SUC_lexicon_tags_23 },
    { 0xe10f76ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x724f76ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496f76f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0f76f6, SUC_lexicon_tags_6 },
    { 0xd54f76f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630f76ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68af7704, SUC_lexicon_tags_12 },
    { 0xe4cf7705, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100f7709, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccf7711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f7714, SUC_lexicon_tags_27 },
    { 0x390f7715, SUC_lexicon_tags_2 },
    { 0x626f7716, SUC_lexicon_tags_85 },
    { 0x746f7717, SUC_lexicon_tags_2 },
    { 0x8cf7716, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x804f771f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7726, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe50f7728, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa48f772a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x93cf772c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6d4f772e, SUC_lexicon_tags_11 },
    { 0xe4cf772f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe92f7734, SUC_lexicon_tags_4 },
    { 0xacaf7734, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x590f7737, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8f773a, SUC_lexicon_tags_37 },
    { 0xdb6f773b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86af7741, SUC_lexicon_tags_11 },
    { 0xd4f7742, SUC_lexicon_tags_82 },
    { 0x658f7742, SUC_lexicon_tags_15 },
    { 0xe0f7744, SUC_lexicon_tags_25 },
    { 0xcbaf7745, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528f7749, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8def774f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e0f7751, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x640f7758, SUC_lexicon_tags_17 },
    { 0x94cf7758, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91af775d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf7762, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd34f7764, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x132f7767, SUC_lexicon_tags_9 },
    { 0xc9af7768, SUC_lexicon_tags_25 },
    { 0x34af7767, SUC_lexicon_tags_24 },
    { 0xb88f7769, SUC_lexicon_tags_11 },
    { 0x7def776b, SUC_lexicon_tags_15 },
    { 0x9ccf776c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666f7775, SUC_lexicon_tags_0 },
    { 0xbfcf7776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8af7778, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc92f777a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9c2f777c, SUC_lexicon_tags_27 },
    { 0xf9cf777d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x176f777f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ef7782, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ef778b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe76f778d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x338f778f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58f7793, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe66f7795, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4aaf7797, SUC_lexicon_tags_15 },
    { 0x4a6f7797, SUC_lexicon_tags_32 },
    { 0x73af7797, SUC_lexicon_tags_77 },
    { 0x3b6f779a, SUC_lexicon_tags_45 },
    { 0xb7af779b, SUC_lexicon_tags_158 },
    { 0xde4f779a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0f77a0, SUC_lexicon_tags_38 },
    { 0x304f77a1, SUC_lexicon_tags_23 },
    { 0x7ccf77a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcf77a6, SUC_lexicon_tags_49 },
    { 0x932f77a7, SUC_lexicon_tags_43 },
    { 0xe44f77a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02f77ae, SUC_lexicon_tags_25 },
    { 0xb36f77af, SUC_lexicon_tags_27 },
    { 0x354f77ae, SUC_lexicon_tags_49 },
    { 0x31cf77b1, SUC_lexicon_tags_9 },
    { 0xa84f77b1, SUC_lexicon_tags_27 },
    { 0x842f77b1, SUC_lexicon_tags_13 },
    { 0xdd8f77af, SUC_lexicon_tags_1 },
    { 0xc86f77b5, SUC_lexicon_tags_65 },
    { 0x154f77b6, SUC_lexicon_tags_11 },
    { 0x9e4f77b5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x124f77ba, SUC_lexicon_tags_11 },
    { 0x794f77bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6e8f77bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaacf77c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacf77cc, SUC_lexicon_tags_238 },
    { 0xfef77cd, SUC_lexicon_tags_17 },
    { 0x4c2f77ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62f77d1, SUC_lexicon_tags_9 },
    { 0x1e6f77d1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8f77d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1af77db, SUC_lexicon_tags_11 },
    { 0xba6f77dc, SUC_lexicon_tags_28 },
    { 0x902f77dd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3c8f77df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe20f77e1, SUC_lexicon_tags_63 },
    { 0x38ef77e2, SUC_lexicon_tags_1 },
    { 0x4b8f77e2, SUC_lexicon_tags_37 },
    { 0x9a8f77e2, SUC_lexicon_tags_11 },
    { 0xea6f77e3, SUC_lexicon_tags_12 },
    { 0x31cf77e6, SUC_lexicon_tags_30 },
    { 0x46ef77e6, SUC_lexicon_tags_37 },
    { 0x588f77e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x246f77ea, SUC_lexicon_tags_49 },
    { 0xfb4f77ea, SUC_lexicon_tags_28 },
    { 0xb2f77ec, SUC_lexicon_tags_12 },
    { 0x416f77ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90f77f2, SUC_lexicon_tags_40 },
    { 0xc00f77f3, SUC_lexicon_tags_0 },
    { 0xe86f77f4, SUC_lexicon_tags_37 },
    { 0x99af77f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96ef77f7, SUC_lexicon_tags_15 },
    { 0x26cf77f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x210f77fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d6f77fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96f780a, SUC_lexicon_tags_33 },
    { 0xde4f780b, SUC_lexicon_tags_37 },
    { 0x790f780b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x606f780f, SUC_lexicon_tags_17 },
    { 0x80ef7810, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x34cf7812, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6ccf7814, SUC_lexicon_tags_27 },
    { 0xe16f7815, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0f7818, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cf781b, SUC_lexicon_tags_60 },
    { 0x570f781c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708f7823, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516f782a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ef782d, SUC_lexicon_tags_60 },
    { 0x90af782e, SUC_lexicon_tags_7 },
    { 0xd00f782f, SUC_lexicon_tags_30 },
    { 0x568f782f, SUC_lexicon_tags_27 },
    { 0x64af782f, SUC_lexicon_tags_23 },
    { 0xe3ef7831, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x722f7839, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x248f783c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2f7842, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x202f7844, SUC_lexicon_tags_43 },
    { 0x3f6f7844, SUC_lexicon_tags_37 },
    { 0xf70f7846, SUC_lexicon_tags_56 },
    { 0xca8f7846, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe66f7849, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbccf784b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31cf784d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43cf784f, SUC_lexicon_tags_1 },
    { 0xbbef7850, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd0cf7852, SUC_lexicon_tags_80 },
    { 0xc1cf7853, SUC_lexicon_tags_14 },
    { 0x444f7854, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2f7858, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x218f785a, SUC_lexicon_tags_40 },
    { 0xb40f785b, SUC_lexicon_tags_1 },
    { 0x12f785c, SUC_lexicon_tags_46 },
    { 0xbc8f785c, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x316f785f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3cf7861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6f7864, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cef7867, SUC_lexicon_tags_12 },
    { 0xa5cf7868, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x696f786b, SUC_lexicon_tags_35 },
    { 0x470f786b, SUC_lexicon_tags_8 },
    { 0xde6f786d, SUC_lexicon_tags_28 },
    { 0x622f786c, SUC_lexicon_tags_15 },
    { 0xc48f786d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6f7872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138f7876, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd2f7878, SUC_lexicon_tags_194 },
    { 0xe32f7879, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe04f787b, SUC_lexicon_tags_336 },
    { 0xdcf787c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cf7881, SUC_lexicon_tags_136 },
    { 0xc2cf7882, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cf7887, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c6f7889, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x75af788b, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2f7898, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaf789b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6f789e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd0f78a0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x130f78a5, SUC_lexicon_tags_12 },
    { 0x37af78a6, SUC_lexicon_tags_1 },
    { 0x41af78a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x248f78aa, SUC_lexicon_tags_9 },
    { 0x846f78aa, SUC_lexicon_tags_15 },
    { 0x572f78ac, SUC_lexicon_tags_6 },
    { 0x4cf78ab, SUC_lexicon_tags_25 },
    { 0x22cf78aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f6f78b0, SUC_lexicon_tags_37 },
    { 0x65cf78b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4f78b4, SUC_lexicon_tags_11 },
    { 0x1e4f78b5, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d4f78ba, SUC_lexicon_tags_12 },
    { 0xe6af78bb, SUC_lexicon_tags_17 },
    { 0x210f78bc, SUC_lexicon_tags_15 },
    { 0x388f78bc, SUC_lexicon_tags_17 },
    { 0x840f78bb, SUC_lexicon_tags_25 },
    { 0xcc0f78bc, SUC_lexicon_tags_25 },
    { 0xd16f78c0, SUC_lexicon_tags_17 },
    { 0xf84f78c1, SUC_lexicon_tags_87 },
    { 0x2c4f78bb, SUC_lexicon_tags_33 },
    { 0xf60f78c3, SUC_lexicon_tags_87 },
    { 0xb74f78c4, SUC_lexicon_tags_15 },
    { 0xb7cf78c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78f78cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf78d2, SUC_lexicon_tags_23 },
    { 0x69cf78d3, SUC_lexicon_tags_37 },
    { 0xf2ef78d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5ef78d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38af78dd, SUC_lexicon_tags_47 },
    { 0xd8f78dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0cf78e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc02f78e3, SUC_lexicon_tags_63 },
    { 0xe72f78e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbc0f78e6, SUC_lexicon_tags_2 },
    { 0xf68f78e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb14f78e9, SUC_lexicon_tags_62 },
    { 0x8a4f78ea, SUC_lexicon_tags_5 },
    { 0x7ccf78eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2f78ee, SUC_lexicon_tags_45 },
    { 0xd22f78ef, SUC_lexicon_tags_60 },
    { 0xfccf78ef, SUC_lexicon_tags_1 },
    { 0xca4f78f1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaf78f5, SUC_lexicon_tags_47 },
    { 0x3ef78f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310f78fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6f78fd, SUC_lexicon_tags_7 },
    { 0x6bcf78fe, SUC_lexicon_tags_45 },
    { 0x496f78fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ef7902, SUC_lexicon_tags_21 },
    { 0x2bcf7903, SUC_lexicon_tags_53 },
    { 0xa4ef7904, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf62f7907, SUC_lexicon_tags_11 },
    { 0xbb2f7908, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f790f, SUC_lexicon_tags_12 },
    { 0xf2ef7910, SUC_lexicon_tags_25 },
    { 0x91ef790f, SUC_lexicon_tags_7 },
    { 0x846f790f, SUC_lexicon_tags_17 },
    { 0xdb6f7913, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc8af7915, SUC_lexicon_tags_17 },
    { 0x11cf7916, SUC_lexicon_tags_2 },
    { 0x7b0f7915, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x970f791a, SUC_lexicon_tags_95 },
    { 0xbaef791a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2f7923, SUC_lexicon_tags_25 },
    { 0x4c4f7924, SUC_lexicon_tags_1 },
    { 0x408f7925, SUC_lexicon_tags_27 },
    { 0xcfaf7926, SUC_lexicon_tags_2 },
    { 0x558f7925, SUC_lexicon_tags_45 },
    { 0x4d0f7927, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ccf792a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4f7931, SUC_lexicon_tags_4 },
    { 0x56af7932, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcf7935, SUC_lexicon_tags_37 },
    { 0x184f7936, SUC_lexicon_tags_26 },
    { 0x894f7936, SUC_lexicon_tags_2 },
    { 0x4cef7938, SUC_lexicon_tags_32 },
    { 0x7daf7939, SUC_lexicon_tags_2 },
    { 0xf10f793a, SUC_lexicon_tags_14 },
    { 0xcf4f793a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454f793f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadaf7942, SUC_lexicon_tags_25 },
    { 0xd68f7942, SUC_lexicon_tags_275 },
    { 0x9fef7943, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x762f7947, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x292f7949, SUC_lexicon_tags_43 },
    { 0x6a6f7949, SUC_lexicon_tags_18 },
    { 0xfeef794a, SUC_lexicon_tags_4 },
    { 0xaf4f7949, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cf7951, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xecf7953, SUC_lexicon_tags_15 },
    { 0x5a4f7954, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cf7959, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x934f795c, SUC_lexicon_tags_1 },
    { 0xae6f795d, SUC_lexicon_tags_41 },
    { 0x3a0f795e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990f7962, SUC_lexicon_tags_2 },
    { 0x89cf7963, SUC_lexicon_tags_15 },
    { 0xd0af7964, SUC_lexicon_tags_47 },
    { 0x42af7965, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd2f7967, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8baf7969, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x754f796b, SUC_lexicon_tags_17 },
    { 0xfcef796c, SUC_lexicon_tags_14 },
    { 0x250f796c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eef7970, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18ef7972, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x842f7974, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cf797f, SUC_lexicon_tags_12 },
    { 0xe3af7980, SUC_lexicon_tags_6 },
    { 0xb52f7981, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef7984, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43ef7986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8f798d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecf7990, SUC_lexicon_tags_39 },
    { 0xa3ef7990, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf7994, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96af7997, SUC_lexicon_tags_2 },
    { 0xa78f7998, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8f799c, SUC_lexicon_tags_24 },
    { 0xd94f799d, SUC_lexicon_tags_11 },
    { 0x4cf799d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0f79a6, SUC_lexicon_tags_12 },
    { 0x44f79a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ef79ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0f79af, SUC_lexicon_tags_11 },
    { 0xef6f79b0, SUC_lexicon_tags_37 },
    { 0x266f79b1, SUC_lexicon_tags_4 },
    { 0xd3cf79b1, SUC_lexicon_tags_12 },
    { 0xeaef79b1, SUC_lexicon_tags_17 },
    { 0x418f79b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2c0f79b6, SUC_lexicon_tags_12 },
    { 0xb10f79b7, SUC_lexicon_tags_30 },
    { 0x87af79b8, SUC_lexicon_tags_14 },
    { 0x18cf79b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370f79c3, SUC_lexicon_tags_15 },
    { 0xf2f79c3, SUC_lexicon_tags_25 },
    { 0x60f79c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2f79c9, SUC_lexicon_tags_1 },
    { 0x6d0f79c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3af79d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ef79d5, SUC_lexicon_tags_37 },
    { 0x63cf79d6, SUC_lexicon_tags_15 },
    { 0x3ef79d6, SUC_lexicon_tags_25 },
    { 0x998f79d6, SUC_lexicon_tags_26 },
    { 0x814f79d6, SUC_lexicon_tags_12 },
    { 0x2f79da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32f79dd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f79e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cf79e6, SUC_lexicon_tags_27 },
    { 0x464f79e7, SUC_lexicon_tags_27 },
    { 0x220f79e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf50f79ea, SUC_lexicon_tags_11 },
    { 0x930f79eb, SUC_lexicon_tags_24 },
    { 0xc48f79ec, SUC_lexicon_tags_10 },
    { 0xc8cf79ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18f79f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f79fa, SUC_lexicon_tags_37 },
    { 0xa56f79fb, SUC_lexicon_tags_17 },
    { 0xa8af79fa, SUC_lexicon_tags_15 },
    { 0x786f79fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x98cf7a00, SUC_lexicon_tags_30 },
    { 0xfc4f7a01, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38f7a0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78af7a10, SUC_lexicon_tags_38 },
    { 0x650f7a11, SUC_lexicon_tags_12 },
    { 0x7bef7a12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4f6f7a14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ef7a18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x39af7a1a, SUC_lexicon_tags_82 },
    { 0x44cf7a1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x888f7a1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94f7a21, SUC_lexicon_tags_1 },
    { 0x4c4f7a21, SUC_lexicon_tags_37 },
    { 0x202f7a22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f7a26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2f7a2b, SUC_lexicon_tags_33 },
    { 0x272f7a2b, SUC_lexicon_tags_43 },
    { 0xe4af7a2d, SUC_lexicon_tags_7 },
    { 0xe56f7a2e, SUC_lexicon_tags_26 },
    { 0x8f4f7a2c, SUC_lexicon_tags_66 },
    { 0xb00f7a2e, SUC_lexicon_tags_61 },
    { 0x232f7a31, SUC_lexicon_tags_17 },
    { 0x1f0f7a31, SUC_lexicon_tags_14 },
    { 0x40ef7a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2f7a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0f7a3c, SUC_lexicon_tags_15 },
    { 0xf0af7a3c, SUC_lexicon_tags_25 },
    { 0x200f7a3d, SUC_lexicon_tags_83 },
    { 0x186f7a3f, SUC_lexicon_tags_6 },
    { 0x1baf7a40, SUC_lexicon_tags_62 },
    { 0x8f0f7a40, SUC_lexicon_tags_60 },
    { 0xcbaf7a3f, SUC_lexicon_tags_17 },
    { 0xa48f7a43, SUC_lexicon_tags_12 },
    { 0x3c6f7a44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434f7a4b, SUC_lexicon_tags_80 },
    { 0x808f7a4c, SUC_lexicon_tags_37 },
    { 0xdb8f7a4d, SUC_lexicon_tags_11 },
    { 0x4a8f7a4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676f7a52, SUC_lexicon_tags_11 },
    { 0x80ef7a52, SUC_lexicon_tags_15 },
    { 0xda8f7a54, SUC_lexicon_tags_37 },
    { 0x116f7a55, SUC_lexicon_tags_0 },
    { 0x1aaf7a56, SUC_lexicon_tags_37 },
    { 0x352f7a56, SUC_lexicon_tags_14 },
    { 0xec2f7a56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd38f7a5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x656f7a5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ef7a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaf7a67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x816f7a69, SUC_lexicon_tags_25 },
    { 0xdf0f7a6a, SUC_lexicon_tags_17 },
    { 0x122f7a6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50f7a6f, SUC_lexicon_tags_45 },
    { 0x228f7a70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd6ef7a72, SUC_lexicon_tags_7 },
    { 0xe10f7a73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316f7a7d, SUC_lexicon_tags_15 },
    { 0x1d4f7a7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa80f7a80, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96f7a84, SUC_lexicon_tags_11 },
    { 0x75af7a85, SUC_lexicon_tags_7 },
    { 0xb4cf7a86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedaf7a89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72f7a91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360f7a95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f7a9a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42f7aa1, SUC_lexicon_tags_38 },
    { 0x50ef7aa2, SUC_lexicon_tags_33 },
    { 0xb5ef7aa3, SUC_lexicon_tags_7 },
    { 0xe98f7aa4, SUC_lexicon_tags_12 },
    { 0xad2f7aa5, SUC_lexicon_tags_17 },
    { 0x33ef7aa5, SUC_lexicon_tags_9 },
    { 0x71af7aa7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81ef7aa9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x356f7aab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ecf7aae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2d0f7ab0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7af7ab5, SUC_lexicon_tags_37 },
    { 0xc5cf7ab6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb50f7ab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc14f7abb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x320f7abd, SUC_lexicon_tags_37 },
    { 0x934f7abe, SUC_lexicon_tags_23 },
    { 0xbfaf7abf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88cf7ac3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x108f7ac6, SUC_lexicon_tags_15 },
    { 0x3dcf7ac7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85cf7ac9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x830f7acb, SUC_lexicon_tags_1 },
    { 0xe9ef7acc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ef7ad1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7daf7ad3, SUC_lexicon_tags_0 },
    { 0x39af7ad3, SUC_lexicon_tags_3 },
    { 0xceef7ad5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x458f7ad8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4af7ada, SUC_lexicon_tags_0 },
    { 0xbf8f7adb, SUC_lexicon_tags_17 },
    { 0xaf7ada, SUC_lexicon_tags_37 },
    { 0x4c0f7add, SUC_lexicon_tags_19 },
    { 0xb06f7ade, SUC_lexicon_tags_45 },
    { 0x75af7ade, SUC_lexicon_tags_37 },
    { 0xac2f7add, SUC_lexicon_tags_19 },
    { 0xaa0f7ade, SUC_lexicon_tags_33 },
    { 0x224f7ae1, SUC_lexicon_tags_19 },
    { 0xae2f7ae3, SUC_lexicon_tags_12 },
    { 0x948f7ae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x548f7aeb, SUC_lexicon_tags_46 },
    { 0xc1cf7aeb, SUC_lexicon_tags_56 },
    { 0x3c0f7aed, SUC_lexicon_tags_23 },
    { 0x746f7aeb, SUC_lexicon_tags_12 },
    { 0x4daf7aef, SUC_lexicon_tags_15 },
    { 0xa2af7af0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x148f7af2, SUC_lexicon_tags_23 },
    { 0x2e4f7af3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78f7af6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94af7b03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8f7b08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7b6f7b0a, SUC_lexicon_tags_2 },
    { 0xe7af7b0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a6f7b0d, SUC_lexicon_tags_2 },
    { 0xdb2f7b0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0f7b12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2f7b18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ef7b1d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bcf7b22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf8f7b24, SUC_lexicon_tags_79 },
    { 0x434f7b25, SUC_lexicon_tags_75 },
    { 0xea8f7b26, SUC_lexicon_tags_18 },
    { 0x480f7b27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6f7b32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbd0f7b34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cf7b39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cf7b3c, SUC_lexicon_tags_68 },
    { 0xefcf7b3d, SUC_lexicon_tags_15 },
    { 0xf04f7b3e, SUC_lexicon_tags_37 },
    { 0x2e0f7b3e, SUC_lexicon_tags_37 },
    { 0xdd0f7b3e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12f7b47, SUC_lexicon_tags_15 },
    { 0xe38f7b48, SUC_lexicon_tags_19 },
    { 0xa72f7b47, SUC_lexicon_tags_45 },
    { 0xc9ef7b4a, SUC_lexicon_tags_0 },
    { 0x1f2f7b47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e2f7b4d, SUC_lexicon_tags_47 },
    { 0xf5af7b4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x712f7b50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x714f7b52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69cf7b54, SUC_lexicon_tags_26 },
    { 0x7aaf7b55, SUC_lexicon_tags_2 },
    { 0xb96f7b56, SUC_lexicon_tags_25 },
    { 0x1b8f7b57, SUC_lexicon_tags_12 },
    { 0x8b2f7b58, SUC_lexicon_tags_21 },
    { 0xf96f7b59, SUC_lexicon_tags_2 },
    { 0x8f4f7b59, SUC_lexicon_tags_31 },
    { 0xbbef7b5b, SUC_lexicon_tags_12 },
    { 0x50f7b5c, SUC_lexicon_tags_33 },
    { 0x264f7b5b, SUC_lexicon_tags_38 },
    { 0x318f7b5b, SUC_lexicon_tags_27 },
    { 0x6def7b57, SUC_lexicon_tags_66 },
    { 0x56cf7b60, SUC_lexicon_tags_17 },
    { 0x592f7b5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x684f7b64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6f7b73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf24f7b75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f7b7c, SUC_lexicon_tags_12 },
    { 0x4b0f7b7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x644f7b7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aef7b8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2f7b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x778f7b97, SUC_lexicon_tags_15 },
    { 0x7a2f7b97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4f7b9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2f7b9d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x930f7b9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecf7ba4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ef7ba8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bef7bab, SUC_lexicon_tags_1 },
    { 0xd56f7bab, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eef7baf, SUC_lexicon_tags_39 },
    { 0x376f7bb0, SUC_lexicon_tags_29 },
    { 0x75ef7bb1, SUC_lexicon_tags_37 },
    { 0x73af7bb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecf7bb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56af7bbb, SUC_lexicon_tags_3 },
    { 0x878f7bbb, SUC_lexicon_tags_61 },
    { 0x9b4f7bbc, SUC_lexicon_tags_11 },
    { 0xffef7bbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0cf7bc0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x64f7bc2, SUC_lexicon_tags_7 },
    { 0x4e2f7bc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x618f7bc6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x382f7bc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed4f7bcb, SUC_lexicon_tags_2 },
    { 0x44cf7bcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeecf7bce, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0f7bd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0f7bd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ef7bdc, SUC_lexicon_tags_29 },
    { 0x9baf7bdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd0f7bdf, SUC_lexicon_tags_0 },
    { 0xc90f7bdf, SUC_lexicon_tags_33 },
    { 0x2aef7bdf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f7be4, SUC_lexicon_tags_14 },
    { 0xd74f7be5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0f7be9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f7bf0, SUC_lexicon_tags_15 },
    { 0xd8ef7bf1, SUC_lexicon_tags_28 },
    { 0xadcf7bf1, SUC_lexicon_tags_15 },
    { 0x60f7bf0, SUC_lexicon_tags_8 },
    { 0x94ef7bf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x266f7bf9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfef7bfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda8f7bfe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f7c05, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xea8f7c07, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2f7c0b, SUC_lexicon_tags_19 },
    { 0xf50f7c0c, SUC_lexicon_tags_38 },
    { 0xc28f7c0c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x684f7c0f, SUC_lexicon_tags_12 },
    { 0x3b0f7c0f, SUC_lexicon_tags_18 },
    { 0xa88f7c11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78f7c14, SUC_lexicon_tags_1 },
    { 0xee0f7c15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ef7c1d, SUC_lexicon_tags_25 },
    { 0xc6ef7c1e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x414f7c20, SUC_lexicon_tags_26 },
    { 0x920f7c21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x330f7c24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6f7c29, SUC_lexicon_tags_7 },
    { 0xb90f7c29, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe96f7c2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaf7c2f, SUC_lexicon_tags_79 },
    { 0xd8cf7c30, SUC_lexicon_tags_37 },
    { 0x962f7c31, SUC_lexicon_tags_9 },
    { 0x1f2f7c32, SUC_lexicon_tags_47 },
    { 0xa90f7c33, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50f7c36, SUC_lexicon_tags_60 },
    { 0xb2cf7c37, SUC_lexicon_tags_29 },
    { 0x1caf7c38, SUC_lexicon_tags_37 },
    { 0x936f7c38, SUC_lexicon_tags_47 },
    { 0x7b4f7c39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x306f7c3e, SUC_lexicon_tags_60 },
    { 0xc0af7c3f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ef7c42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918f7c49, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x138f7c4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540f7c4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9af7c58, SUC_lexicon_tags_9 },
    { 0xa42f7c59, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1eaf7c5b, SUC_lexicon_tags_2 },
    { 0x434f7c5b, SUC_lexicon_tags_12 },
    { 0x6c8f7c5c, SUC_lexicon_tags_2 },
    { 0x724f7c5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f7c61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x738f7c64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1daf7c69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8f7c70, SUC_lexicon_tags_25 },
    { 0x580f7c71, SUC_lexicon_tags_16 },
    { 0x45cf7c72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0f7c79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4faf7c7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fcf7c7d, SUC_lexicon_tags_10 },
    { 0x38f7c7d, SUC_lexicon_tags_11 },
    { 0xb84f7c7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454f7c85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8f7c89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef4f7c8b, SUC_lexicon_tags_43 },
    { 0x4a6f7c8c, SUC_lexicon_tags_17 },
    { 0x604f7c8d, SUC_lexicon_tags_37 },
    { 0x358f7c8e, SUC_lexicon_tags_4 },
    { 0x7b4f7c8e, SUC_lexicon_tags_25 },
    { 0x73af7c8b, SUC_lexicon_tags_19 },
    { 0xedaf7c91, SUC_lexicon_tags_19 },
    { 0x550f7c8e, SUC_lexicon_tags_12 },
    { 0x26f7c93, SUC_lexicon_tags_45 },
    { 0xc3ef7c94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb58f7c96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ef7c99, SUC_lexicon_tags_27 },
    { 0xf70f7c99, SUC_lexicon_tags_1 },
    { 0x994f7c99, SUC_lexicon_tags_37 },
    { 0x2f0f7c9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22af7ca1, SUC_lexicon_tags_11 },
    { 0xde6f7ca1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x896f7ca4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb02f7ca6, SUC_lexicon_tags_6 },
    { 0xc20f7ca7, SUC_lexicon_tags_17 },
    { 0x39cf7ca8, SUC_lexicon_tags_4 },
    { 0xe78f7ca9, SUC_lexicon_tags_37 },
    { 0xa02f7caa, SUC_lexicon_tags_10 },
    { 0x6d2f7caa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x582f7cad, SUC_lexicon_tags_11 },
    { 0x12cf7cae, SUC_lexicon_tags_1 },
    { 0xc26f7caf, SUC_lexicon_tags_19 },
    { 0xd02f7cad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x670f7cb2, SUC_lexicon_tags_37 },
    { 0xf48f7cb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38f7cb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06f7cbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcf7cc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeef7cc3, SUC_lexicon_tags_38 },
    { 0x96ef7cc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccf7cc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x856f7cc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2f7ccf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xae0f7cd1, SUC_lexicon_tags_11 },
    { 0x308f7cd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2cf7cd4, SUC_lexicon_tags_16 },
    { 0xf9ef7cd5, SUC_lexicon_tags_23 },
    { 0xac8f7cd6, SUC_lexicon_tags_6 },
    { 0xd42f7cd6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaf7cd9, SUC_lexicon_tags_26 },
    { 0xfb0f7cda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cf7cdf, SUC_lexicon_tags_25 },
    { 0x19ef7ce0, SUC_lexicon_tags_27 },
    { 0x162f7ce1, SUC_lexicon_tags_25 },
    { 0x2ecf7ce2, SUC_lexicon_tags_0 },
    { 0x73cf7ce2, SUC_lexicon_tags_2 },
    { 0x6c2f7ce3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd60f7ce7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x704f7cea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa4f7cf0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1aaf7cf2, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0x838f7cf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70af7cf7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaf2f7cf9, SUC_lexicon_tags_8 },
    { 0x782f7cf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ef7cfd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2af7d00, SUC_lexicon_tags_17 },
    { 0x1c4f7d01, SUC_lexicon_tags_19 },
    { 0xae8f7d01, SUC_lexicon_tags_47 },
    { 0x988f7d03, SUC_lexicon_tags_8 },
    { 0xd64f7d04, SUC_lexicon_tags_6 },
    { 0x7c8f7d05, SUC_lexicon_tags_2 },
    { 0x6b6f7d05, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd44f7d08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2f7d0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf2f7d0d, SUC_lexicon_tags_15 },
    { 0xf2ef7d0e, SUC_lexicon_tags_15 },
    { 0xf20f7d0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836f7d14, SUC_lexicon_tags_28 },
    { 0x9f4f7d15, SUC_lexicon_tags_15 },
    { 0xc7ef7d16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcf7d1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x888f7d1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a2f7d20, SUC_lexicon_tags_48 },
    { 0x72f7d21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ef7d24, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x566f7d26, SUC_lexicon_tags_13 },
    { 0x406f7d27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ef7d2d, SUC_lexicon_tags_11 },
    { 0xbe4f7d2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4f7d37, SUC_lexicon_tags_38 },
    { 0x9eef7d37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1aef7d3a, SUC_lexicon_tags_33 },
    { 0x9e2f7d3b, SUC_lexicon_tags_47 },
    { 0x288f7d3c, SUC_lexicon_tags_17 },
    { 0x7f2f7d3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8f7d40, SUC_lexicon_tags_38 },
    { 0x3b2f7d40, SUC_lexicon_tags_65 },
    { 0xfaef7d40, SUC_lexicon_tags_12 },
    { 0xeb8f7d43, SUC_lexicon_tags_8 },
    { 0x3c8f7d43, SUC_lexicon_tags_7 },
    { 0xdaaf7d43, SUC_lexicon_tags_63 },
    { 0x272f7d46, SUC_lexicon_tags_19 },
    { 0x78cf7d47, SUC_lexicon_tags_7 },
    { 0xeef7d48, SUC_lexicon_tags_14 },
    { 0xfb4f7d49, SUC_lexicon_tags_25 },
    { 0x6a8f7d48, SUC_lexicon_tags_37 },
    { 0x6eef7d4b, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0xc48f7d4d, SUC_lexicon_tags_30 },
    { 0x94cf7d4e, SUC_lexicon_tags_17 },
    { 0xaf4f7d4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ef7d52, SUC_lexicon_tags_27 },
    { 0xef0f7d53, SUC_lexicon_tags_15 },
    { 0x83cf7d54, SUC_lexicon_tags_31 },
    { 0xe18f7d54, SUC_lexicon_tags_37 },
    { 0x562f7d55, SUC_lexicon_tags_65 },
    { 0x594f7d56, SUC_lexicon_tags_3 },
    { 0x1d2f7d58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x998f7d5b, SUC_lexicon_tags_91 },
    { 0xbcef7d5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5fcf7d5e, SUC_lexicon_tags_15 },
    { 0x4b2f7d5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0f7d63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80af7d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7d6e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58af7d73, SUC_lexicon_tags_25 },
    { 0x8a6f7d74, SUC_lexicon_tags_8 },
    { 0x86af7d74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7baf7d77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7fef7d79, SUC_lexicon_tags_7 },
    { 0x56cf7d7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ef7d81, SUC_lexicon_tags_1 },
    { 0xa7cf7d82, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7a0f7d84, SUC_lexicon_tags_28 },
    { 0x320f7d84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5af7d88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1daf7d8b, SUC_lexicon_tags_31 },
    { 0x694f7d8c, SUC_lexicon_tags_8 },
    { 0x3a4f7d8d, SUC_lexicon_tags_11 },
    { 0x470f7d8e, SUC_lexicon_tags_2 },
    { 0xfe2f7d8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456f7d98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77ef7d9a, SUC_lexicon_tags_37 },
    { 0x474f7d9b, SUC_lexicon_tags_15 },
    { 0x1ef7d9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf7da2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6f7da6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2f7daa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860f7daf, SUC_lexicon_tags_29 },
    { 0x660f7daf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x338f7db3, SUC_lexicon_tags_25 },
    { 0xd5af7db4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaef7dbb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6faf7dbd, SUC_lexicon_tags_12 },
    { 0xd3ef7dbe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe06f7dc6, SUC_lexicon_tags_3 },
    { 0x91cf7dc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf8f7dc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6f7dcc, SUC_lexicon_tags_23 },
    { 0x746f7dcd, SUC_lexicon_tags_9 },
    { 0x682f7dce, SUC_lexicon_tags_25 },
    { 0x4d8f7dcf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x758f7dd1, SUC_lexicon_tags_17 },
    { 0x5a6f7dd2, SUC_lexicon_tags_1 },
    { 0x8bcf7dd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4af7dd6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4ef7dd8, SUC_lexicon_tags_40 },
    { 0x79af7dd8, SUC_lexicon_tags_2 },
    { 0xef0f7dd8, SUC_lexicon_tags_36 },
    { 0x918f7ddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa22f7dde, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6f7de1, SUC_lexicon_tags_5 },
    { 0xa22f7de2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6f7de7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f7df0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834f7df4, SUC_lexicon_tags_18 },
    { 0x3eef7df4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcf7df9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfa6f7dfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x400f7dfd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0f7e04, SUC_lexicon_tags_2 },
    { 0xe56f7e05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8f7e08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0f7e0d, SUC_lexicon_tags_40 },
    { 0xe4ef7e0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x424f7e10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd52f7e1d, SUC_lexicon_tags_60 },
    { 0xbecf7e1e, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x6dcf7e20, SUC_lexicon_tags_113 },
    { 0x4d8f7e20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0f7e27, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4a2f7e29, SUC_lexicon_tags_23 },
    { 0x68ef7e2a, SUC_lexicon_tags_43 },
    { 0xb4cf7e2b, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8f7e31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f2f7e33, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fef7e36, SUC_lexicon_tags_32 },
    { 0x6caf7e37, SUC_lexicon_tags_38 },
    { 0x320f7e38, SUC_lexicon_tags_30 },
    { 0xc86f7e38, SUC_lexicon_tags_11 },
    { 0xa0af7e39, SUC_lexicon_tags_116 },
    { 0x212f7e3b, SUC_lexicon_tags_50 },
    { 0x1d4f7e3c, SUC_lexicon_tags_15 },
    { 0xb44f7e3b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12f7e41, SUC_lexicon_tags_1 },
    { 0x8fcf7e42, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xffaf7e44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4f7e48, SUC_lexicon_tags_160 },
    { 0x9d4f7e49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2f7e50, SUC_lexicon_tags_7 },
    { 0xc0f7e50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e2f7e53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ef7e58, SUC_lexicon_tags_27 },
    { 0xd4af7e58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadaf7e61, SUC_lexicon_tags_2 },
    { 0xa68f7e62, SUC_lexicon_tags_17 },
    { 0xec2f7e62, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7e66, SUC_lexicon_tags_63 },
    { 0xe06f7e67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x382f7e69, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x858f7e6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x302f7e6f, SUC_lexicon_tags_12 },
    { 0x92f7e6f, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xb50f7e72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68cf7e74, SUC_lexicon_tags_1 },
    { 0x48cf7e74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf42f7e77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1aef7e79, SUC_lexicon_tags_17 },
    { 0xb56f7e7a, SUC_lexicon_tags_21 },
    { 0xc72f7e7b, SUC_lexicon_tags_59 },
    { 0x7caf7e7c, SUC_lexicon_tags_7 },
    { 0xcb8f7e7d, SUC_lexicon_tags_37 },
    { 0xadaf7e7d, SUC_lexicon_tags_23 },
    { 0x572f7e7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2f7e84, SUC_lexicon_tags_17 },
    { 0x834f7e84, SUC_lexicon_tags_0 },
    { 0x1e8f7e85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38f7e88, SUC_lexicon_tags_10 },
    { 0xcb4f7e89, SUC_lexicon_tags_12 },
    { 0x6ecf7e89, SUC_lexicon_tags_1 },
    { 0x7a0f7e8b, SUC_lexicon_tags_46 },
    { 0xf26f7e89, SUC_lexicon_tags_25 },
    { 0xcc8f7e88, SUC_lexicon_tags_27 },
    { 0xff4f7e8c, SUC_lexicon_tags_27 },
    { 0x25af7e8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x382f7e92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2f7e99, SUC_lexicon_tags_16 },
    { 0x998f7e99, SUC_lexicon_tags_2 },
    { 0xf0f7e9b, SUC_lexicon_tags_7 },
    { 0x166f7e9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ef7ea0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740f7ea4, SUC_lexicon_tags_37 },
    { 0x6eef7ea5, SUC_lexicon_tags_37 },
    { 0x17cf7ea6, SUC_lexicon_tags_15 },
    { 0x7d4f7ea7, SUC_lexicon_tags_0 },
    { 0x882f7ea6, SUC_lexicon_tags_25 },
    { 0x7bcf7ea6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf8ef7eab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcf7eb1, SUC_lexicon_tags_11 },
    { 0xbe8f7eb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8f7eb5, SUC_lexicon_tags_43 },
    { 0xac0f7eb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8f7eb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12f7ebd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766f7ec1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84f7ec5, SUC_lexicon_tags_2 },
    { 0x20cf7ec6, SUC_lexicon_tags_7 },
    { 0xd90f7ec7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cf7ecd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb6af7ecf, SUC_lexicon_tags_0 },
    { 0x7e4f7ed0, SUC_lexicon_tags_1 },
    { 0xa9af7ecf, SUC_lexicon_tags_65 },
    { 0xac0f7ed2, SUC_lexicon_tags_7 },
    { 0x4b2f7ed2, SUC_lexicon_tags_14 },
    { 0x2fcf7ed3, SUC_lexicon_tags_31 },
    { 0xe58f7ed5, SUC_lexicon_tags_664 },
    { 0xea6f7ed3, SUC_lexicon_tags_37 },
    { 0xc4cf7ed7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6f7eda, SUC_lexicon_tags_47 },
    { 0x528f7edb, SUC_lexicon_tags_7 },
    { 0xba0f7eda, SUC_lexicon_tags_1 },
    { 0x7aaf7edd, SUC_lexicon_tags_9 },
    { 0xd18f7ede, SUC_lexicon_tags_12 },
    { 0xbd4f7eda, SUC_lexicon_tags_15 },
    { 0x750f7edc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daf7ee8, SUC_lexicon_tags_12 },
    { 0x542f7ee9, SUC_lexicon_tags_25 },
    { 0x608f7ee9, SUC_lexicon_tags_25 },
    { 0x286f7eeb, SUC_lexicon_tags_1 },
    { 0xddcf7eea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7cf7eee, SUC_lexicon_tags_47 },
    { 0x696f7eef, SUC_lexicon_tags_21 },
    { 0xb8f7ef0, SUC_lexicon_tags_25 },
    { 0x3caf7eee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe0f7ef3, SUC_lexicon_tags_68 },
    { 0xecf7ef4, SUC_lexicon_tags_7 },
    { 0x2baf7ef5, SUC_lexicon_tags_37 },
    { 0x1ef7ef6, SUC_lexicon_tags_21 },
    { 0xf60f7ef7, SUC_lexicon_tags_7 },
    { 0x430f7ef7, SUC_lexicon_tags_25 },
    { 0x7e8f7ef9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38f7efe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182f7f02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af7f0c, SUC_lexicon_tags_62 },
    { 0x926f7f0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x134f7f10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6d2f7f12, SUC_lexicon_tags_37 },
    { 0xe9af7f13, SUC_lexicon_tags_11 },
    { 0xf02f7f14, SUC_lexicon_tags_65 },
    { 0x7aaf7f15, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc4af7f17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90cf7f1f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562f7f2a, SUC_lexicon_tags_17 },
    { 0x5eef7f2a, SUC_lexicon_tags_6 },
    { 0xa96f7f2a, SUC_lexicon_tags_15 },
    { 0xc0f7f2d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe84f7f2f, SUC_lexicon_tags_25 },
    { 0x812f7f2f, SUC_lexicon_tags_44 },
    { 0x720f7f2f, SUC_lexicon_tags_4 },
    { 0xaa4f7f32, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8d6f7f34, SUC_lexicon_tags_17 },
    { 0x4c2f7f35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c8f7f37, SUC_lexicon_tags_2 },
    { 0xf1ef7f37, SUC_lexicon_tags_13 },
    { 0x8e6f7f39, SUC_lexicon_tags_12 },
    { 0xafcf7f39, SUC_lexicon_tags_14 },
    { 0xa2af7f37, SUC_lexicon_tags_15 },
    { 0xef0f7f37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d2f7f3e, SUC_lexicon_tags_63 },
    { 0x2b4f7f3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6f7f42, SUC_lexicon_tags_37 },
    { 0x25af7f43, SUC_lexicon_tags_17 },
    { 0x10f7f42, SUC_lexicon_tags_25 },
    { 0x152f7f44, SUC_lexicon_tags_29 },
    { 0x38f7f42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8f7f4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0cf7f4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0f7f5d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x674f7f5f, SUC_lexicon_tags_12 },
    { 0x4af7f5f, SUC_lexicon_tags_7 },
    { 0x9c6f7f61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ef7f64, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef7f6f, SUC_lexicon_tags_11 },
    { 0x78af7f70, SUC_lexicon_tags_19 },
    { 0x336f7f71, SUC_lexicon_tags_77 },
    { 0x188f7f70, SUC_lexicon_tags_17 },
    { 0x73cf7f73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50af7f78, SUC_lexicon_tags_36 },
    { 0x258f7f79, SUC_lexicon_tags_12 },
    { 0xdc4f7f7a, SUC_lexicon_tags_25 },
    { 0x1a0f7f7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8ef7f7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x504f7f7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x116f7f82, SUC_lexicon_tags_7 },
    { 0x61cf7f83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2f7f88, SUC_lexicon_tags_64 },
    { 0xb98f7f89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe04f7f8c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9d2f7f8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0f7f90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f7f94, SUC_lexicon_tags_3 },
    { 0x16cf7f95, SUC_lexicon_tags_49 },
    { 0x20f7f96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf7f9d, SUC_lexicon_tags_31 },
    { 0xa56f7f9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ef7fa9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x518f7fac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47ef7fae, SUC_lexicon_tags_63 },
    { 0x896f7faf, SUC_lexicon_tags_5 },
    { 0xce0f7fb0, SUC_lexicon_tags_37 },
    { 0x24af7fb1, SUC_lexicon_tags_1 },
    { 0x2def7fb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x242f7fb4, SUC_lexicon_tags_23 },
    { 0xb88f7fb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2f7fb9, SUC_lexicon_tags_15 },
    { 0xc54f7fba, SUC_lexicon_tags_1 },
    { 0xbb8f7fba, SUC_lexicon_tags_27 },
    { 0xde6f7fbc, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ef7fbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfef7fc2, SUC_lexicon_tags_11 },
    { 0x9b6f7fc2, SUC_lexicon_tags_27 },
    { 0xf88f7fc4, SUC_lexicon_tags_6 },
    { 0x450f7fc5, SUC_lexicon_tags_28 },
    { 0xd08f7fc5, SUC_lexicon_tags_4 },
    { 0x556f7fc7, SUC_lexicon_tags_3 },
    { 0xa2f7fc7, SUC_lexicon_tags_68 },
    { 0x91ef7fc8, SUC_lexicon_tags_13 },
    { 0xf36f7fca, SUC_lexicon_tags_17 },
    { 0x5c4f7fca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8f7fd8, SUC_lexicon_tags_17 },
    { 0xc20f7fd9, SUC_lexicon_tags_19 },
    { 0xb7ef7fda, SUC_lexicon_tags_25 },
    { 0x258f7fdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104f7fe2, SUC_lexicon_tags_25 },
    { 0xb42f7fe3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xebaf7fe5, SUC_lexicon_tags_4 },
    { 0xd7ef7fe6, SUC_lexicon_tags_1 },
    { 0x6d0f7fe6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56f7fee, SUC_lexicon_tags_63 },
    { 0x80f7fef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0f7ff2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa72f7ff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8ef7ff6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530f7ffa, SUC_lexicon_tags_33 },
    { 0xa84f7ffb, SUC_lexicon_tags_33 },
    { 0x3c8f7ffc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x87cf7ffe, SUC_lexicon_tags_25 },
    { 0x44af7ffe, SUC_lexicon_tags_37 },
    { 0xc00f8000, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cf8006, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5bcf8008, SUC_lexicon_tags_11 },
    { 0xb2ef8009, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaf8f800b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x37cf800e, SUC_lexicon_tags_15 },
    { 0xd26f800f, SUC_lexicon_tags_23 },
    { 0xe2af8010, SUC_lexicon_tags_17 },
    { 0x780f8011, SUC_lexicon_tags_37 },
    { 0xc9ef8012, SUC_lexicon_tags_47 },
    { 0xff2f8013, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x640f8015, SUC_lexicon_tags_55 },
    { 0xbfcf8016, SUC_lexicon_tags_11 },
    { 0xc00f8015, SUC_lexicon_tags_12 },
    { 0x728f8017, SUC_lexicon_tags_25 },
    { 0x8b6f8015, SUC_lexicon_tags_43 },
    { 0x8f0f801a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24f801f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4caf8023, SUC_lexicon_tags_4 },
    { 0x394f8023, SUC_lexicon_tags_25 },
    { 0xd02f8025, SUC_lexicon_tags_15 },
    { 0xffaf8025, SUC_lexicon_tags_9 },
    { 0x838f8025, SUC_lexicon_tags_37 },
    { 0x528f8028, SUC_lexicon_tags_11 },
    { 0xbb2f8029, SUC_lexicon_tags_26 },
    { 0xe58f8029, SUC_lexicon_tags_37 },
    { 0x308f8029, SUC_lexicon_tags_23 },
    { 0xb72f802c, SUC_lexicon_tags_12 },
    { 0xaaef802d, SUC_lexicon_tags_17 },
    { 0x424f802d, SUC_lexicon_tags_38 },
    { 0xf74f802f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55af8034, SUC_lexicon_tags_25 },
    { 0xfd6f8035, SUC_lexicon_tags_2 },
    { 0x82f8036, SUC_lexicon_tags_15 },
    { 0xe6ef8037, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5af803c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4f8041, SUC_lexicon_tags_60 },
    { 0xd2ef8042, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2f8048, SUC_lexicon_tags_7 },
    { 0xc2cf8049, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f804d, SUC_lexicon_tags_47 },
    { 0x264f804e, SUC_lexicon_tags_4 },
    { 0xbaef804f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ef8056, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90f805b, SUC_lexicon_tags_25 },
    { 0xcdef805b, SUC_lexicon_tags_11 },
    { 0xa2f805d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x360f8062, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620f8068, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35af806b, SUC_lexicon_tags_28 },
    { 0x22af806b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x276f806f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7dcf8071, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x84af8074, SUC_lexicon_tags_60 },
    { 0x216f8075, SUC_lexicon_tags_9 },
    { 0xfd0f8075, SUC_lexicon_tags_12 },
    { 0xca0f8077, SUC_lexicon_tags_37 },
    { 0x1e8f8078, SUC_lexicon_tags_8 },
    { 0xfb8f8077, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cef807d, SUC_lexicon_tags_15 },
    { 0x30af807e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e4f8080, SUC_lexicon_tags_12 },
    { 0x2b6f8081, SUC_lexicon_tags_37 },
    { 0x54ef8080, SUC_lexicon_tags_12 },
    { 0xdc6f8083, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e6f8085, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa50f8087, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6f808a, SUC_lexicon_tags_7 },
    { 0xeaef808a, SUC_lexicon_tags_36 },
    { 0xceef808a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe28f808e, SUC_lexicon_tags_38 },
    { 0x522f808f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192f8097, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2baf8099, SUC_lexicon_tags_9 },
    { 0x1e8f809a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8f80a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee8f80a7, SUC_lexicon_tags_92 },
    { 0xc32f80a8, SUC_lexicon_tags_12 },
    { 0xcb0f80a9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x722f80ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8f80ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944f80b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x680f80b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ccf80bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x996f80be, SUC_lexicon_tags_45 },
    { 0xe14f80bf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd0f80c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacf80c6, SUC_lexicon_tags_15 },
    { 0xb12f80c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65af80c9, SUC_lexicon_tags_25 },
    { 0xc84f80ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb74f80cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0f80d2, SUC_lexicon_tags_30 },
    { 0x3aaf80d3, SUC_lexicon_tags_7 },
    { 0xdb8f80d4, SUC_lexicon_tags_17 },
    { 0xd52f80d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770f80da, SUC_lexicon_tags_17 },
    { 0xe12f80db, SUC_lexicon_tags_26 },
    { 0xe4cf80dc, SUC_lexicon_tags_1 },
    { 0x620f80dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x944f80df, SUC_lexicon_tags_11 },
    { 0x83af80e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5af80e4, SUC_lexicon_tags_15 },
    { 0xb36f80e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef80ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acf80f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ccf80f5, SUC_lexicon_tags_2 },
    { 0x9bcf80f5, SUC_lexicon_tags_11 },
    { 0x334f80f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef80fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392f8101, SUC_lexicon_tags_2 },
    { 0x33af8101, SUC_lexicon_tags_47 },
    { 0xceaf8101, SUC_lexicon_tags_28 },
    { 0x854f8104, SUC_lexicon_tags_17 },
    { 0x946f8103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16f810a, SUC_lexicon_tags_11 },
    { 0x9b6f810b, SUC_lexicon_tags_38 },
    { 0x43af810b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3af810e, SUC_lexicon_tags_25 },
    { 0xb66f810f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x494f8111, SUC_lexicon_tags_49 },
    { 0x98af8112, SUC_lexicon_tags_47 },
    { 0x8aaf8113, SUC_lexicon_tags_35 },
    { 0xce4f8114, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb96f8116, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758f811f, SUC_lexicon_tags_12 },
    { 0x34af8120, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf812a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f812e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x894f8131, SUC_lexicon_tags_17 },
    { 0xab8f8131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf50f8134, SUC_lexicon_tags_33 },
    { 0x35af8135, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93af8137, SUC_lexicon_tags_44 },
    { 0xe86f8137, SUC_lexicon_tags_15 },
    { 0xc5ef8138, SUC_lexicon_tags_10 },
    { 0xaeaf8137, SUC_lexicon_tags_63 },
    { 0x99ef813b, SUC_lexicon_tags_12 },
    { 0x5cf813a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56f813f, SUC_lexicon_tags_11 },
    { 0x7fef813f, SUC_lexicon_tags_55 },
    { 0xb34f8141, SUC_lexicon_tags_6 },
    { 0x60af8142, SUC_lexicon_tags_17 },
    { 0x9def8141, SUC_lexicon_tags_25 },
    { 0x532f8141, SUC_lexicon_tags_66 },
    { 0x930f8145, SUC_lexicon_tags_34 },
    { 0xf3cf8146, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x618f8148, SUC_lexicon_tags_63 },
    { 0x2acf8148, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1a8f814b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fcf814d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc82f814f, SUC_lexicon_tags_282 },
    { 0xabef8150, SUC_lexicon_tags_0 },
    { 0x476f8150, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aef8159, SUC_lexicon_tags_25 },
    { 0x95af815a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62af815e, SUC_lexicon_tags_26 },
    { 0x2b6f815e, SUC_lexicon_tags_66 },
    { 0x236f8160, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xffef8163, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738f816b, SUC_lexicon_tags_11 },
    { 0x6def816b, SUC_lexicon_tags_11 },
    { 0xd54f816c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8f8174, SUC_lexicon_tags_45 },
    { 0xfecf8175, SUC_lexicon_tags_37 },
    { 0x1cf8175, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af817c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664f8182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98f818a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b4f818c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x440f818f, SUC_lexicon_tags_7 },
    { 0x4e8f8190, SUC_lexicon_tags_9 },
    { 0x5af818f, SUC_lexicon_tags_23 },
    { 0x86f8190, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0f8198, SUC_lexicon_tags_11 },
    { 0x4e8f8199, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6f819e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd44f81a0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x172f81a2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ef81a5, SUC_lexicon_tags_50 },
    { 0x3b4f81a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cf81aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x796f81ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcef81b2, SUC_lexicon_tags_2 },
    { 0x838f81b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaaf81b7, SUC_lexicon_tags_29 },
    { 0x2cf81b8, SUC_lexicon_tags_7 },
    { 0xcd2f81b7, SUC_lexicon_tags_43 },
    { 0xa08f81ba, SUC_lexicon_tags_2 },
    { 0x54ef81bb, SUC_lexicon_tags_7 },
    { 0x374f81bc, SUC_lexicon_tags_14 },
    { 0xc6af81bd, SUC_lexicon_tags_37 },
    { 0x836f81be, SUC_lexicon_tags_49 },
    { 0x188f81ba, SUC_lexicon_tags_14 },
    { 0xe7ef81c0, SUC_lexicon_tags_19 },
    { 0x2f8f81ba, SUC_lexicon_tags_37 },
    { 0x854f81bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2f81c8, SUC_lexicon_tags_66 },
    { 0x7b4f81c9, SUC_lexicon_tags_11 },
    { 0xbeaf81c9, SUC_lexicon_tags_33 },
    { 0xea2f81c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8f81d7, SUC_lexicon_tags_15 },
    { 0x8b8f81d8, SUC_lexicon_tags_30 },
    { 0x150f81d7, SUC_lexicon_tags_6 },
    { 0x18ef81da, SUC_lexicon_tags_12 },
    { 0x7c8f81da, SUC_lexicon_tags_17 },
    { 0xa2ef81dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x970f81de, SUC_lexicon_tags_27 },
    { 0xa36f81df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6f81e2, SUC_lexicon_tags_7 },
    { 0xa12f81e3, SUC_lexicon_tags_27 },
    { 0x926f81e4, SUC_lexicon_tags_16 },
    { 0xd7af81e5, SUC_lexicon_tags_10 },
    { 0x3ccf81e4, SUC_lexicon_tags_1 },
    { 0x22af81e2, SUC_lexicon_tags_25 },
    { 0x9af81e2, SUC_lexicon_tags_15 },
    { 0x670f81e9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7e2f81eb, SUC_lexicon_tags_0 },
    { 0x4c2f81ec, SUC_lexicon_tags_15 },
    { 0xd0cf81eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc16f81ef, SUC_lexicon_tags_14 },
    { 0x234f81f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62af81f4, SUC_lexicon_tags_2 },
    { 0xfc6f81f5, SUC_lexicon_tags_38 },
    { 0x3acf81f5, SUC_lexicon_tags_33 },
    { 0xbe6f81f7, SUC_lexicon_tags_28 },
    { 0x154f81f8, SUC_lexicon_tags_17 },
    { 0xdd2f81f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126f81fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf06f8200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2f8203, SUC_lexicon_tags_0 },
    { 0xc5ef8203, SUC_lexicon_tags_15 },
    { 0x440f8204, SUC_lexicon_tags_0 },
    { 0x47ef8206, SUC_lexicon_tags_27 },
    { 0x522f8207, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97af8209, SUC_lexicon_tags_17 },
    { 0x9f4f820a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x636f820d, SUC_lexicon_tags_0 },
    { 0xa82f820e, SUC_lexicon_tags_1 },
    { 0xaf0f820f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea6f8211, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x686f8214, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x18af8217, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea8f821b, SUC_lexicon_tags_28 },
    { 0xd9af821c, SUC_lexicon_tags_252 },
    { 0x86ef821c, SUC_lexicon_tags_4 },
    { 0x6e2f821c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf14f8220, SUC_lexicon_tags_68 },
    { 0x3cf8221, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0f8224, SUC_lexicon_tags_82 },
    { 0xc4f8224, SUC_lexicon_tags_473 },
    { 0x56af8224, SUC_lexicon_tags_17 },
    { 0x6d4f8227, SUC_lexicon_tags_37 },
    { 0xfd6f8227, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14f822b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820f8230, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ef8233, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2f8238, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2a6f823a, SUC_lexicon_tags_11 },
    { 0xc84f823b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x778f823d, SUC_lexicon_tags_2 },
    { 0x2f823d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x902f8241, SUC_lexicon_tags_11 },
    { 0x4b8f8242, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cf8246, SUC_lexicon_tags_37 },
    { 0x654f8246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca8f8249, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8f824c, SUC_lexicon_tags_0 },
    { 0x4cef824d, SUC_lexicon_tags_19 },
    { 0xc1af824e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x11cf8251, SUC_lexicon_tags_30 },
    { 0x604f8252, SUC_lexicon_tags_60 },
    { 0xf1cf8253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cf8259, SUC_lexicon_tags_33 },
    { 0x676f825a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x600f825c, SUC_lexicon_tags_7 },
    { 0x3b2f825d, SUC_lexicon_tags_37 },
    { 0x328f825e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950f8265, SUC_lexicon_tags_15 },
    { 0x58f8265, SUC_lexicon_tags_33 },
    { 0xfc8f8265, SUC_lexicon_tags_10 },
    { 0x16f8268, SUC_lexicon_tags_23 },
    { 0x7e4f8269, SUC_lexicon_tags_43 },
    { 0xe04f8269, SUC_lexicon_tags_37 },
    { 0x2def8265, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1b2f826d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9b8f826f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0f8279, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x87cf827c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92f827f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ef8284, SUC_lexicon_tags_1 },
    { 0xc48f8285, SUC_lexicon_tags_26 },
    { 0x21af8286, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x480f828f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d4f8291, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f8296, SUC_lexicon_tags_11 },
    { 0xac6f8297, SUC_lexicon_tags_15 },
    { 0x45cf8298, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71af829c, SUC_lexicon_tags_12 },
    { 0x886f829c, SUC_lexicon_tags_6 },
    { 0x302f829e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fef82a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14cf82a4, SUC_lexicon_tags_1 },
    { 0xb98f82a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdaef82a7, SUC_lexicon_tags_37 },
    { 0x3f2f82a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x47af82aa, SUC_lexicon_tags_12 },
    { 0x408f82ab, SUC_lexicon_tags_23 },
    { 0xcb0f82ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310f82b1, SUC_lexicon_tags_0 },
    { 0xe4cf82b1, SUC_lexicon_tags_32 },
    { 0xd4f82b3, SUC_lexicon_tags_49 },
    { 0xd9ef82b4, SUC_lexicon_tags_28 },
    { 0x32af82b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566f82ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4f82be, SUC_lexicon_tags_106 },
    { 0xe8af82bf, SUC_lexicon_tags_17 },
    { 0x4aaf82c0, SUC_lexicon_tags_15 },
    { 0x996f82c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfc6f82c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcf82c7, SUC_lexicon_tags_12 },
    { 0x5caf82c7, SUC_lexicon_tags_60 },
    { 0xed6f82c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80cf82cb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbd6f82cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6f82d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2e2f82d2, SUC_lexicon_tags_31 },
    { 0x914f82d3, SUC_lexicon_tags_30 },
    { 0xc1ef82d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420f82dd, SUC_lexicon_tags_6 },
    { 0x4f0f82dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02f82e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6f82f2, SUC_lexicon_tags_11 },
    { 0xfc4f82f3, SUC_lexicon_tags_7 },
    { 0xfdcf82f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74af82f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828f82fb, SUC_lexicon_tags_56 },
    { 0xb8cf82fc, SUC_lexicon_tags_12 },
    { 0x58ef82fb, SUC_lexicon_tags_55 },
    { 0x9b6f82fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9e2f8300, SUC_lexicon_tags_25 },
    { 0x6daf8300, SUC_lexicon_tags_25 },
    { 0x74f8300, SUC_lexicon_tags_45 },
    { 0x736f8303, SUC_lexicon_tags_6 },
    { 0xf00f8301, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962f830c, SUC_lexicon_tags_12 },
    { 0x528f830c, SUC_lexicon_tags_1 },
    { 0x580f830e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75af8312, SUC_lexicon_tags_26 },
    { 0x1fef8312, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916f8319, SUC_lexicon_tags_7 },
    { 0x40ef831a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724f831e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfef8321, SUC_lexicon_tags_2 },
    { 0xc4cf8322, SUC_lexicon_tags_81 },
    { 0xa2cf8322, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a8f8327, SUC_lexicon_tags_37 },
    { 0x312f8328, SUC_lexicon_tags_179 },
    { 0x7e0f8327, SUC_lexicon_tags_37 },
    { 0xe2f8327, SUC_lexicon_tags_14 },
    { 0xbe0f8329, SUC_lexicon_tags_45 },
    { 0x7faf8327, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714f8330, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x844f8333, SUC_lexicon_tags_25 },
    { 0xf64f8334, SUC_lexicon_tags_2 },
    { 0x308f8335, SUC_lexicon_tags_10 },
    { 0x5aef8336, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x114f8338, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x212f833b, SUC_lexicon_tags_33 },
    { 0x194f833b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51cf833e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ef834d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcaf8350, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf54f8352, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4bef8354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0f835d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2def8362, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa2f8364, SUC_lexicon_tags_11 },
    { 0xc6ef8365, SUC_lexicon_tags_1 },
    { 0x58f8364, SUC_lexicon_tags_15 },
    { 0xdb6f8367, SUC_lexicon_tags_33 },
    { 0x854f8368, SUC_lexicon_tags_26 },
    { 0xa76f8367, SUC_lexicon_tags_38 },
    { 0xc8ef836a, SUC_lexicon_tags_28 },
    { 0x7e4f836b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f8371, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6faf8374, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x846f8379, SUC_lexicon_tags_7 },
    { 0xdf8f8379, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940f8381, SUC_lexicon_tags_9 },
    { 0x1b4f8382, SUC_lexicon_tags_2 },
    { 0xdcef8383, SUC_lexicon_tags_37 },
    { 0x4e6f8383, SUC_lexicon_tags_39 },
    { 0xe8af8383, SUC_lexicon_tags_25 },
    { 0x52af8381, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84f838a, SUC_lexicon_tags_37 },
    { 0x912f838b, SUC_lexicon_tags_6 },
    { 0x2ecf838c, SUC_lexicon_tags_35 },
    { 0x352f838d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fef838f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x244f8394, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8f839a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd14f839d, SUC_lexicon_tags_0 },
    { 0x6eaf839e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x436f83a0, SUC_lexicon_tags_37 },
    { 0xe2f83a1, SUC_lexicon_tags_46 },
    { 0xd74f83a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdeef83a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0af83ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4f83b0, SUC_lexicon_tags_2 },
    { 0x41af83b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f2f83b3, SUC_lexicon_tags_49 },
    { 0x682f83b4, SUC_lexicon_tags_63 },
    { 0x874f83b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8af83b8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0f83bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90f83c0, SUC_lexicon_tags_15 },
    { 0x442f83c1, SUC_lexicon_tags_38 },
    { 0x604f83c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cf83c5, SUC_lexicon_tags_47 },
    { 0x6fcf83c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd86f83c8, SUC_lexicon_tags_64 },
    { 0x790f83c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1af83d0, SUC_lexicon_tags_17 },
    { 0xb6cf83d1, SUC_lexicon_tags_1 },
    { 0x29cf83d2, SUC_lexicon_tags_63 },
    { 0x76cf83d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x232f83d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65cf83d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2def83d9, SUC_lexicon_tags_733 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6f83dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32cf83de, SUC_lexicon_tags_33 },
    { 0x63cf83df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974f83e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x60af83e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cf83e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1caf83ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaef83ef, SUC_lexicon_tags_118 },
    { 0xa0ef83f0, SUC_lexicon_tags_27 },
    { 0x334f83f0, SUC_lexicon_tags_63 },
    { 0xecaf83f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832f83f6, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8f83fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6f83ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa58f8401, SUC_lexicon_tags_8 },
    { 0x64cf8402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdecf8404, SUC_lexicon_tags_17 },
    { 0x396f8405, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x66ef8407, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55af840b, SUC_lexicon_tags_27 },
    { 0xad8f840c, SUC_lexicon_tags_15 },
    { 0x68f840b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf8411, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b2f8413, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18f8419, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44f841d, SUC_lexicon_tags_63 },
    { 0x1a0f841d, SUC_lexicon_tags_26 },
    { 0x9f8f841f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x312f8421, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf8424, SUC_lexicon_tags_17 },
    { 0xac8f8425, SUC_lexicon_tags_25 },
    { 0xcc2f8424, SUC_lexicon_tags_37 },
    { 0x66af8427, SUC_lexicon_tags_60 },
    { 0xdecf8427, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee0f842a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42cf842c, SUC_lexicon_tags_1 },
    { 0xe72f842c, SUC_lexicon_tags_47 },
    { 0x2af842e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95af8435, SUC_lexicon_tags_37 },
    { 0x498f8435, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xf8ef8438, SUC_lexicon_tags_49 },
    { 0xb5ef8438, SUC_lexicon_tags_49 },
    { 0xdd2f843a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2acf843c, SUC_lexicon_tags_8 },
    { 0x750f843d, SUC_lexicon_tags_37 },
    { 0xdf8f843c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4c8f8440, SUC_lexicon_tags_7 },
    { 0x70af8440, SUC_lexicon_tags_692 },
    { 0x28ef8442, SUC_lexicon_tags_37 },
    { 0xd3ef8442, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce6f8445, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf10f8447, SUC_lexicon_tags_38 },
    { 0x5ecf8448, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x98f844a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cf844e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66af8455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x702f8457, SUC_lexicon_tags_92 },
    { 0xd0af8458, SUC_lexicon_tags_34 },
    { 0xf32f8457, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3b0f845b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0f8461, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xafef8463, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x424f8466, SUC_lexicon_tags_1 },
    { 0x176f8467, SUC_lexicon_tags_25 },
    { 0xb5af8466, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x940f846b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xca2f846e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9e0f8470, SUC_lexicon_tags_7 },
    { 0x832f8471, SUC_lexicon_tags_29 },
    { 0x106f8472, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa28f8474, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x242f8477, SUC_lexicon_tags_25 },
    { 0xdd4f8477, SUC_lexicon_tags_63 },
    { 0x87ef8477, SUC_lexicon_tags_254 },
    { 0xdb8f847a, SUC_lexicon_tags_4 },
    { 0xebef8477, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f6f8481, SUC_lexicon_tags_7 },
    { 0xd7af8482, SUC_lexicon_tags_60 },
    { 0x3ecf8481, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88f8487, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2f848c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd54f848e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ef8492, SUC_lexicon_tags_272 },
    { 0x516f8493, SUC_lexicon_tags_8 },
    { 0x664f8494, SUC_lexicon_tags_17 },
    { 0xa6af8493, SUC_lexicon_tags_19 },
    { 0x41ef8493, SUC_lexicon_tags_14 },
    { 0x1f8f8494, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8ef8499, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cf849d, SUC_lexicon_tags_9 },
    { 0xb24f849d, SUC_lexicon_tags_112 },
    { 0xe0cf849d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ef84a3, SUC_lexicon_tags_2 },
    { 0xadaf84a4, SUC_lexicon_tags_83 },
    { 0x4f4f84a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cf84aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42ef84ac, SUC_lexicon_tags_4 },
    { 0x734f84ad, SUC_lexicon_tags_11 },
    { 0xe4ef84ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4af84b1, SUC_lexicon_tags_25 },
    { 0x464f84b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6f84b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712f84bd, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xce8f84bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea2f84c1, SUC_lexicon_tags_21 },
    { 0x104f84c2, SUC_lexicon_tags_31 },
    { 0xb0f84c2, SUC_lexicon_tags_11 },
    { 0xf8f84c3, SUC_lexicon_tags_47 },
    { 0xb8af84c2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x69ef84c7, SUC_lexicon_tags_12 },
    { 0x8d6f84c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x836f84cb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x154f84cd, SUC_lexicon_tags_31 },
    { 0x776f84cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7baf84d0, SUC_lexicon_tags_48 },
    { 0xf62f84d0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x600f84d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6f84d7, SUC_lexicon_tags_7 },
    { 0xa04f84d8, SUC_lexicon_tags_43 },
    { 0x4d0f84d8, SUC_lexicon_tags_47 },
    { 0x874f84da, SUC_lexicon_tags_47 },
    { 0x72f84db, SUC_lexicon_tags_11 },
    { 0xbc2f84dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe42f84e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf84e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6def84e7, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x638f84ea, SUC_lexicon_tags_7 },
    { 0x416f84ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x49ef84ed, SUC_lexicon_tags_31 },
    { 0xe22f84ee, SUC_lexicon_tags_73 },
    { 0x1a8f84ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcf84f2, SUC_lexicon_tags_0 },
    { 0xfc2f84f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a4f84f6, SUC_lexicon_tags_23 },
    { 0xd0f84f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e6f84f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x794f84fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccf8501, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6f8504, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cf8509, SUC_lexicon_tags_11 },
    { 0xaaef850a, SUC_lexicon_tags_14 },
    { 0xdbaf850b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xff0f850d, SUC_lexicon_tags_25 },
    { 0x914f850d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c4f8510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62cf8513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f8517, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2daf851a, SUC_lexicon_tags_2 },
    { 0xa5af851b, SUC_lexicon_tags_4 },
    { 0x1eaf851c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccf851f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x936f8521, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cf8524, SUC_lexicon_tags_1 },
    { 0x976f8525, SUC_lexicon_tags_11 },
    { 0x338f8526, SUC_lexicon_tags_17 },
    { 0x57ef8524, SUC_lexicon_tags_37 },
    { 0xb20f8528, SUC_lexicon_tags_1 },
    { 0xfeaf8525, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x614f852b, SUC_lexicon_tags_43 },
    { 0x204f852b, SUC_lexicon_tags_11 },
    { 0x1a6f852d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x438f852f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ef8535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696f853b, SUC_lexicon_tags_14 },
    { 0x1fef853c, SUC_lexicon_tags_39 },
    { 0xd50f853d, SUC_lexicon_tags_56 },
    { 0x73af853d, SUC_lexicon_tags_28 },
    { 0x7bcf853b, SUC_lexicon_tags_1 },
    { 0xb16f8540, SUC_lexicon_tags_0 },
    { 0x96ef8541, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766f8545, SUC_lexicon_tags_19 },
    { 0xf86f8545, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8f854b, SUC_lexicon_tags_60 },
    { 0xa10f854b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccf8553, SUC_lexicon_tags_0 },
    { 0x50cf8554, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9ccf8556, SUC_lexicon_tags_13 },
    { 0x696f8557, SUC_lexicon_tags_37 },
    { 0xc88f8558, SUC_lexicon_tags_108 },
    { 0xfa8f8559, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x63af855b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec8f8560, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x314f8563, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cf856a, SUC_lexicon_tags_0 },
    { 0x6cf856b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x100f856d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172f8573, SUC_lexicon_tags_8 },
    { 0x54af8573, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6f857a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a8f857c, SUC_lexicon_tags_63 },
    { 0x9f2f857c, SUC_lexicon_tags_37 },
    { 0xca4f857e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x102f8580, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08f8584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ef8588, SUC_lexicon_tags_106 },
    { 0x87af8589, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x318f858b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf8597, SUC_lexicon_tags_12 },
    { 0x142f8597, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110f859f, SUC_lexicon_tags_47 },
    { 0x57cf85a0, SUC_lexicon_tags_28 },
    { 0x740f85a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40f85a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfa2f85a6, SUC_lexicon_tags_30 },
    { 0xa1cf85a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1baf85a9, SUC_lexicon_tags_38 },
    { 0xed0f85a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x954f85ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x798f85af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0f85b3, SUC_lexicon_tags_14 },
    { 0x24f85b3, SUC_lexicon_tags_25 },
    { 0xfd2f85b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa8f85b7, SUC_lexicon_tags_14 },
    { 0xe00f85b8, SUC_lexicon_tags_37 },
    { 0x38f85b8, SUC_lexicon_tags_17 },
    { 0x7c8f85ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38af85c0, SUC_lexicon_tags_7 },
    { 0x752f85c0, SUC_lexicon_tags_0 },
    { 0xc84f85c1, SUC_lexicon_tags_25 },
    { 0x3f2f85c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90cf85c5, SUC_lexicon_tags_2 },
    { 0xea6f85c6, SUC_lexicon_tags_11 },
    { 0x5c2f85c6, SUC_lexicon_tags_2 },
    { 0xb90f85c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f85d0, SUC_lexicon_tags_57 },
    { 0x798f85d1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbacf85d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0f85e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc66f85e6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xaa6f85e8, SUC_lexicon_tags_33 },
    { 0x6d4f85e9, SUC_lexicon_tags_48 },
    { 0xdacf85e9, SUC_lexicon_tags_55 },
    { 0x660f85e9, SUC_lexicon_tags_17 },
    { 0xf4ef85e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856f85f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d0f85f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f85fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f85fe, SUC_lexicon_tags_17 },
    { 0xe48f85ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4f8603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd74f8605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ef8607, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434f860d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8f8613, SUC_lexicon_tags_19 },
    { 0xf92f8614, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492f8619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e6f861b, SUC_lexicon_tags_45 },
    { 0x346f861b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3af861f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8af8624, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x950f8627, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbe0f8629, SUC_lexicon_tags_11 },
    { 0xcc8f862a, SUC_lexicon_tags_45 },
    { 0xf4cf862b, SUC_lexicon_tags_17 },
    { 0xcb6f862a, SUC_lexicon_tags_77 },
    { 0xf20f8629, SUC_lexicon_tags_19 },
    { 0xaf2f862e, SUC_lexicon_tags_37 },
    { 0xf74f862e, SUC_lexicon_tags_37 },
    { 0xaaf862a, SUC_lexicon_tags_19 },
    { 0xe82f8631, SUC_lexicon_tags_0 },
    { 0xe1ef8631, SUC_lexicon_tags_27 },
    { 0x1b6f8629, SUC_lexicon_tags_25 },
    { 0xab6f8629, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29af8639, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7ef863b, SUC_lexicon_tags_12 },
    { 0x128f863c, SUC_lexicon_tags_2 },
    { 0x632f863d, SUC_lexicon_tags_31 },
    { 0xdb0f863d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8644, SUC_lexicon_tags_5 },
    { 0xffcf8645, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8f864c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x622f864f, SUC_lexicon_tags_25 },
    { 0xcaef8650, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf8653, SUC_lexicon_tags_17 },
    { 0xa6cf8654, SUC_lexicon_tags_11 },
    { 0x2b2f8653, SUC_lexicon_tags_39 },
    { 0xc8af8656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa90f8659, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x36cf865b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1baf865f, SUC_lexicon_tags_40 },
    { 0xc0cf8660, SUC_lexicon_tags_39 },
    { 0xb22f8660, SUC_lexicon_tags_45 },
    { 0xfeef8662, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x550f8665, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276f866a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaa4f866c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0f8671, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd0f8673, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cf8677, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x450f867a, SUC_lexicon_tags_12 },
    { 0x9a4f867a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2f867f, SUC_lexicon_tags_68 },
    { 0xc90f867f, SUC_lexicon_tags_21 },
    { 0xbdef8680, SUC_lexicon_tags_2 },
    { 0xe20f867f, SUC_lexicon_tags_37 },
    { 0xaf4f8683, SUC_lexicon_tags_37 },
    { 0xd22f8680, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd12f8686, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f8689, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220f868d, SUC_lexicon_tags_39 },
    { 0x46f868e, SUC_lexicon_tags_25 },
    { 0xbf4f868f, SUC_lexicon_tags_9 },
    { 0x5def868e, SUC_lexicon_tags_15 },
    { 0xf22f8690, SUC_lexicon_tags_15 },
    { 0x746f868e, SUC_lexicon_tags_49 },
    { 0x336f868f, SUC_lexicon_tags_12 },
    { 0xbaaf8692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62ef8696, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cf869f, SUC_lexicon_tags_33 },
    { 0x826f869f, SUC_lexicon_tags_11 },
    { 0xaf0f86a1, SUC_lexicon_tags_15 },
    { 0x360f86a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14f86a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7aaf86a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268f86ad, SUC_lexicon_tags_32 },
    { 0x31ef86ae, SUC_lexicon_tags_2 },
    { 0xc8ef86ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc6cf86b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcf86b4, SUC_lexicon_tags_8 },
    { 0x40ef86b5, SUC_lexicon_tags_17 },
    { 0x4c6f86b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x134f86b8, SUC_lexicon_tags_2 },
    { 0xfbaf86b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9daf86bb, SUC_lexicon_tags_25 },
    { 0x2b6f86bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ef86c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf88f86ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecef86cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888f86d2, SUC_lexicon_tags_2 },
    { 0x798f86d2, SUC_lexicon_tags_25 },
    { 0x13cf86d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x298f86d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cf86d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0f86e1, SUC_lexicon_tags_32 },
    { 0x6d8f86e1, SUC_lexicon_tags_11 },
    { 0x16ef86e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x82af86e5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f86e8, SUC_lexicon_tags_11 },
    { 0xfb0f86e9, SUC_lexicon_tags_6 },
    { 0x7b2f86ea, SUC_lexicon_tags_31 },
    { 0xde8f86eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x14af86ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acf86f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34f86f5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x22af86f7, SUC_lexicon_tags_21 },
    { 0x556f86f8, SUC_lexicon_tags_26 },
    { 0xc04f86f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x772f86fb, SUC_lexicon_tags_60 },
    { 0x820f86fc, SUC_lexicon_tags_11 },
    { 0xaaf86fd, SUC_lexicon_tags_11 },
    { 0xb60f86fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a4f8700, SUC_lexicon_tags_47 },
    { 0xc7cf8701, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9def8704, SUC_lexicon_tags_45 },
    { 0x9a2f8704, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x248f8707, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f870f, SUC_lexicon_tags_14 },
    { 0xf8ef8710, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8f8716, SUC_lexicon_tags_1 },
    { 0x96af8717, SUC_lexicon_tags_11 },
    { 0xc02f8717, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x31af871b, SUC_lexicon_tags_10 },
    { 0x39af871c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02f8720, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8cf8722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ccf8726, SUC_lexicon_tags_17 },
    { 0x8f4f8727, SUC_lexicon_tags_12 },
    { 0x7b6f8726, SUC_lexicon_tags_1 },
    { 0x5d8f8728, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5c2f872b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b4f872d, SUC_lexicon_tags_7 },
    { 0x500f872e, SUC_lexicon_tags_17 },
    { 0x88af872e, SUC_lexicon_tags_47 },
    { 0x218f872e, SUC_lexicon_tags_57 },
    { 0x358f872f, SUC_lexicon_tags_37 },
    { 0xf5ef8732, SUC_lexicon_tags_62 },
    { 0xcc0f8730, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x32ef8735, SUC_lexicon_tags_37 },
    { 0x694f8736, SUC_lexicon_tags_2 },
    { 0x258f8737, SUC_lexicon_tags_1 },
    { 0xebcf8736, SUC_lexicon_tags_14 },
    { 0xa00f8739, SUC_lexicon_tags_12 },
    { 0xa50f8739, SUC_lexicon_tags_194 },
    { 0xd74f873b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3daf873f, SUC_lexicon_tags_25 },
    { 0x23cf8740, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x24af8743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x334f8745, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecf874c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ef874f, SUC_lexicon_tags_7 },
    { 0x6f2f874f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30f8753, SUC_lexicon_tags_6 },
    { 0x1a6f8753, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88f8758, SUC_lexicon_tags_56 },
    { 0xc48f8759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc22f875b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc70f875d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x924f8761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eef8769, SUC_lexicon_tags_12 },
    { 0x428f8769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52ef876c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef8771, SUC_lexicon_tags_77 },
    { 0xd8cf8771, SUC_lexicon_tags_47 },
    { 0x25cf8771, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4f8776, SUC_lexicon_tags_23 },
    { 0x396f8777, SUC_lexicon_tags_48 },
    { 0x340f8776, SUC_lexicon_tags_7 },
    { 0x382f8776, SUC_lexicon_tags_107 },
    { 0x6d4f8778, SUC_lexicon_tags_15 },
    { 0x6daf8777, SUC_lexicon_tags_17 },
    { 0xe02f8779, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27af8780, SUC_lexicon_tags_9 },
    { 0x6fef8781, SUC_lexicon_tags_15 },
    { 0x2b4f8781, SUC_lexicon_tags_27 },
    { 0x8e8f8783, SUC_lexicon_tags_25 },
    { 0x116f8784, SUC_lexicon_tags_13 },
    { 0xf16f8784, SUC_lexicon_tags_12 },
    { 0xb66f8786, SUC_lexicon_tags_37 },
    { 0x790f8787, SUC_lexicon_tags_60 },
    { 0xf1cf8788, SUC_lexicon_tags_0 },
    { 0x1a6f8789, SUC_lexicon_tags_104 },
    { 0x34af878a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60f878e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f8791, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f4f8793, SUC_lexicon_tags_14 },
    { 0xd8af8793, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x832f8797, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518f879f, SUC_lexicon_tags_28 },
    { 0xe70f87a0, SUC_lexicon_tags_11 },
    { 0xcccf87a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceef87a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37af87ac, SUC_lexicon_tags_77 },
    { 0x886f87ad, SUC_lexicon_tags_17 },
    { 0xc8ef87ad, SUC_lexicon_tags_140 },
    { 0xe08f87ad, SUC_lexicon_tags_351 },
    { 0x348f87af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf87b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2f87b7, SUC_lexicon_tags_2 },
    { 0xca8f87b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0f87bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x602f87be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x526f87c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe8f87c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6f87c7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00f87cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10f87d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x508f87d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x188f87d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caf87de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb86f87e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf87e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0f87ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8f87f1, SUC_lexicon_tags_39 },
    { 0x2f4f87f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fef87f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x208f87fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4f8801, SUC_lexicon_tags_43 },
    { 0x69cf8802, SUC_lexicon_tags_33 },
    { 0x662f8801, SUC_lexicon_tags_25 },
    { 0xe94f8804, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acf880d, SUC_lexicon_tags_14 },
    { 0x1d0f880e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0f8811, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54f8816, SUC_lexicon_tags_2 },
    { 0x444f8817, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x51af881a, SUC_lexicon_tags_27 },
    { 0xacf881b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f8820, SUC_lexicon_tags_12 },
    { 0x246f8820, SUC_lexicon_tags_31 },
    { 0xecef8820, SUC_lexicon_tags_47 },
    { 0x99af8823, SUC_lexicon_tags_15 },
    { 0x59cf8823, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5af8827, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x950f882a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9f6f882c, SUC_lexicon_tags_29 },
    { 0x954f882c, SUC_lexicon_tags_49 },
    { 0xfd0f882d, SUC_lexicon_tags_11 },
    { 0xdeef882d, SUC_lexicon_tags_30 },
    { 0x488f882e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b4f8832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x118f8834, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56cf8836, SUC_lexicon_tags_7 },
    { 0x692f8836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4f883c, SUC_lexicon_tags_25 },
    { 0x44af883d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa48f883f, SUC_lexicon_tags_17 },
    { 0x996f8840, SUC_lexicon_tags_11 },
    { 0xeb0f8841, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb8ef8843, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fef884a, SUC_lexicon_tags_11 },
    { 0xccef884b, SUC_lexicon_tags_29 },
    { 0x6ccf884c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0f8852, SUC_lexicon_tags_39 },
    { 0xfd6f8853, SUC_lexicon_tags_37 },
    { 0x98cf8853, SUC_lexicon_tags_12 },
    { 0x4b4f8853, SUC_lexicon_tags_639 },
    { 0xe90f8856, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32af885a, SUC_lexicon_tags_11 },
    { 0x644f885b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bef8860, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x556f8864, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8f8868, SUC_lexicon_tags_30 },
    { 0x1a2f8869, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4f8872, SUC_lexicon_tags_12 },
    { 0xfa2f8872, SUC_lexicon_tags_4 },
    { 0xd72f8874, SUC_lexicon_tags_33 },
    { 0xcc0f8875, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ecf8877, SUC_lexicon_tags_37 },
    { 0x5b6f8878, SUC_lexicon_tags_23 },
    { 0x7baf8877, SUC_lexicon_tags_11 },
    { 0xa2af8879, SUC_lexicon_tags_7 },
    { 0xa88f887b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xecef887e, SUC_lexicon_tags_19 },
    { 0x520f887f, SUC_lexicon_tags_70 },
    { 0x9a2f8880, SUC_lexicon_tags_296 },
    { 0x306f8881, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39ef8883, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ef8886, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d8f8888, SUC_lexicon_tags_11 },
    { 0xff6f8889, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcf8891, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xab2f8894, SUC_lexicon_tags_25 },
    { 0x896f8894, SUC_lexicon_tags_53 },
    { 0x3fef8894, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb42f8898, SUC_lexicon_tags_12 },
    { 0x13af8899, SUC_lexicon_tags_25 },
    { 0x312f8898, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02f88a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcf88a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932f88ac, SUC_lexicon_tags_37 },
    { 0x4c4f88ad, SUC_lexicon_tags_19 },
    { 0x94f88ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5def88b1, SUC_lexicon_tags_45 },
    { 0x39ef88b2, SUC_lexicon_tags_33 },
    { 0x89cf88b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f88b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa6f88b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f88bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf3ef88be, SUC_lexicon_tags_11 },
    { 0x1a8f88bf, SUC_lexicon_tags_37 },
    { 0xb4af88c0, SUC_lexicon_tags_17 },
    { 0x988f88c0, SUC_lexicon_tags_37 },
    { 0x564f88be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeaf88c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58af88ca, SUC_lexicon_tags_63 },
    { 0x978f88ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x106f88cd, SUC_lexicon_tags_2 },
    { 0xa8ef88ce, SUC_lexicon_tags_27 },
    { 0x79cf88ce, SUC_lexicon_tags_0 },
    { 0x920f88d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4f88d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ef88da, SUC_lexicon_tags_7 },
    { 0xf32f88db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafcf88dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8f88e5, SUC_lexicon_tags_12 },
    { 0xb2cf88e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6f88ec, SUC_lexicon_tags_33 },
    { 0x940f88ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410f88f3, SUC_lexicon_tags_60 },
    { 0x33cf88f4, SUC_lexicon_tags_15 },
    { 0x1c8f88f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x776f88f8, SUC_lexicon_tags_25 },
    { 0x9daf88f8, SUC_lexicon_tags_105 },
    { 0x892f88fa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2af8900, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c0f8903, SUC_lexicon_tags_0 },
    { 0xc7af8904, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6f2f8906, SUC_lexicon_tags_63 },
    { 0x4e6f8907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x864f890d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a0f890f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ef8917, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14f891a, SUC_lexicon_tags_25 },
    { 0xcc2f891b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5fef891d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10af8920, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88af8922, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x7ccf8924, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b4f8926, SUC_lexicon_tags_63 },
    { 0xf44f8927, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x638f892b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aaf892d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf18f892f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2f8933, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f8936, SUC_lexicon_tags_26 },
    { 0x5f8f8937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ecf8939, SUC_lexicon_tags_17 },
    { 0x39cf893a, SUC_lexicon_tags_14 },
    { 0x43ef893a, SUC_lexicon_tags_15 },
    { 0xf38f893c, SUC_lexicon_tags_1 },
    { 0x64cf893d, SUC_lexicon_tags_12 },
    { 0xf58f893e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd26f8940, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd68f8942, SUC_lexicon_tags_25 },
    { 0x89cf8942, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83ef8945, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x328f8947, SUC_lexicon_tags_117 },
    { 0x9e8f8948, SUC_lexicon_tags_15 },
    { 0x3bef8949, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa96f894d, SUC_lexicon_tags_12 },
    { 0x2ccf894e, SUC_lexicon_tags_2 },
    { 0x7e2f894e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3a8f8951, SUC_lexicon_tags_6 },
    { 0xdeaf8952, SUC_lexicon_tags_129 },
    { 0x77ef8952, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af8957, SUC_lexicon_tags_13 },
    { 0x904f8958, SUC_lexicon_tags_57 },
    { 0x4d4f8959, SUC_lexicon_tags_65 },
    { 0x30ef895a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x154f895d, SUC_lexicon_tags_11 },
    { 0x792f895e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58af8961, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf7ef8963, SUC_lexicon_tags_1004 },
    { 0xadaf8963, SUC_lexicon_tags_2 },
    { 0x102f8965, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836f896b, SUC_lexicon_tags_0 },
    { 0x398f896c, SUC_lexicon_tags_85 },
    { 0xa0af896d, SUC_lexicon_tags_23 },
    { 0x33af896d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ef8972, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf8976, SUC_lexicon_tags_37 },
    { 0xc5cf8977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8f897b, SUC_lexicon_tags_26 },
    { 0xd66f897b, SUC_lexicon_tags_1 },
    { 0xf9ef897d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cf8980, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf38f8982, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafef8986, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a6f8988, SUC_lexicon_tags_25 },
    { 0x2faf8989, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54f898b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ef898f, SUC_lexicon_tags_38 },
    { 0x1f6f8990, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2def8992, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x264f8994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7daf8997, SUC_lexicon_tags_7 },
    { 0x7faf8998, SUC_lexicon_tags_12 },
    { 0x72f8999, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b0f899b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d2f899d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ef89a5, SUC_lexicon_tags_17 },
    { 0x6c4f89a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cf89ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x912f89b0, SUC_lexicon_tags_7 },
    { 0x41cf89b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f89b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8f89b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74ef89b9, SUC_lexicon_tags_38 },
    { 0xe2af89ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2f89c3, SUC_lexicon_tags_27 },
    { 0x6d6f89c3, SUC_lexicon_tags_7 },
    { 0xcbaf89c5, SUC_lexicon_tags_25 },
    { 0xcaf89c6, SUC_lexicon_tags_65 },
    { 0xbdcf89c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb96f89c9, SUC_lexicon_tags_21 },
    { 0xa4af89ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6f89cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x482f89cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70f89da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x432f89dd, SUC_lexicon_tags_17 },
    { 0xdbcf89de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cf89e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4f89e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6f89ee, SUC_lexicon_tags_16 },
    { 0xb82f89ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ef89f2, SUC_lexicon_tags_37 },
    { 0x27cf89f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde8f89f8, SUC_lexicon_tags_8 },
    { 0xb28f89f9, SUC_lexicon_tags_25 },
    { 0xbecf89f9, SUC_lexicon_tags_1 },
    { 0xd32f89fb, SUC_lexicon_tags_27 },
    { 0x616f89f9, SUC_lexicon_tags_63 },
    { 0xae8f89f9, SUC_lexicon_tags_25 },
    { 0xa68f89fa, SUC_lexicon_tags_43 },
    { 0xa10f89f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62f8a07, SUC_lexicon_tags_45 },
    { 0x838f8a08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd28f8a0c, SUC_lexicon_tags_17 },
    { 0x280f8a0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcf8a11, SUC_lexicon_tags_10 },
    { 0x4aef8a12, SUC_lexicon_tags_15 },
    { 0xe60f8a13, SUC_lexicon_tags_204 },
    { 0xb16f8a11, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xda6f8a17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19af8a1b, SUC_lexicon_tags_48 },
    { 0xc38f8a1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe82f8a1e, SUC_lexicon_tags_17 },
    { 0x40ef8a1f, SUC_lexicon_tags_61 },
    { 0xa98f8a1e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x25af8a23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf8cf8a25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81cf8a2c, SUC_lexicon_tags_2 },
    { 0x72af8a2d, SUC_lexicon_tags_11 },
    { 0x2b6f8a2e, SUC_lexicon_tags_69 },
    { 0xe8f8a2f, SUC_lexicon_tags_33 },
    { 0x784f8a30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcf8a38, SUC_lexicon_tags_12 },
    { 0xd66f8a38, SUC_lexicon_tags_15 },
    { 0x75cf8a3a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe78f8a3c, SUC_lexicon_tags_9 },
    { 0xd4af8a3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f4f8a46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4f8a4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3af8a4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43af8a4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ef8a51, SUC_lexicon_tags_17 },
    { 0x8f4f8a52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x384f8a54, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x764f8a57, SUC_lexicon_tags_14 },
    { 0x72ef8a58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa98f8a5a, SUC_lexicon_tags_29 },
    { 0x6aef8a5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6cf8a5f, SUC_lexicon_tags_1 },
    { 0xa32f8a60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdccf8a62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x708f8a64, SUC_lexicon_tags_11 },
    { 0x266f8a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618f8a6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faf8a71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xadaf8a74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x702f8a76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226f8a7b, SUC_lexicon_tags_82 },
    { 0xd06f8a7b, SUC_lexicon_tags_11 },
    { 0x166f8a7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x438f8a7f, SUC_lexicon_tags_12 },
    { 0xe3af8a7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82af8a87, SUC_lexicon_tags_1 },
    { 0xfe8f8a88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0f8a8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34f8a8f, SUC_lexicon_tags_37 },
    { 0xe7af8a8f, SUC_lexicon_tags_49 },
    { 0x918f8a90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910f8a96, SUC_lexicon_tags_1 },
    { 0x358f8a96, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xea8f8a99, SUC_lexicon_tags_25 },
    { 0xadaf8a9a, SUC_lexicon_tags_14 },
    { 0xa9cf8a99, SUC_lexicon_tags_60 },
    { 0x47ef8a99, SUC_lexicon_tags_31 },
    { 0x9b8f8a9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eaf8aa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe40f8aa6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aef8ab1, SUC_lexicon_tags_17 },
    { 0x346f8ab2, SUC_lexicon_tags_12 },
    { 0xc84f8ab2, SUC_lexicon_tags_96 },
    { 0xcbef8ab4, SUC_lexicon_tags_27 },
    { 0xe80f8ab4, SUC_lexicon_tags_8 },
    { 0xe66f8ab2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcf8aba, SUC_lexicon_tags_87 },
    { 0xbbcf8abb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438f8ac0, SUC_lexicon_tags_25 },
    { 0x40af8ac1, SUC_lexicon_tags_68 },
    { 0x5f6f8ac1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf4f8ac4, SUC_lexicon_tags_43 },
    { 0x67ef8ac5, SUC_lexicon_tags_7 },
    { 0x152f8ac6, SUC_lexicon_tags_37 },
    { 0x608f8ac5, SUC_lexicon_tags_7 },
    { 0x182f8ac8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cf8ad1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0f8ad4, SUC_lexicon_tags_28 },
    { 0xdbaf8ad5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda8f8ad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76f8ada, SUC_lexicon_tags_55 },
    { 0xeb6f8adb, SUC_lexicon_tags_37 },
    { 0x638f8adb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414f8ae1, SUC_lexicon_tags_65 },
    { 0x280f8ae2, SUC_lexicon_tags_35 },
    { 0xf98f8ae1, SUC_lexicon_tags_48 },
    { 0x820f8ae3, SUC_lexicon_tags_3 },
    { 0x380f8ae5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26f8ae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x222f8aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8f8aed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd0f8aef, SUC_lexicon_tags_79 },
    { 0x81af8af0, SUC_lexicon_tags_10 },
    { 0xe04f8af1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56f8af6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0f8afa, SUC_lexicon_tags_25 },
    { 0x41cf8afb, SUC_lexicon_tags_10 },
    { 0xe72f8afa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292f8b02, SUC_lexicon_tags_15 },
    { 0x980f8b02, SUC_lexicon_tags_37 },
    { 0xbfcf8b04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6f8b0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2f8b11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30f8b15, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962f8b1f, SUC_lexicon_tags_17 },
    { 0xbdaf8b20, SUC_lexicon_tags_7 },
    { 0x2e4f8b20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c0f8b25, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92af8b27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x602f8b29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9ef8b2b, SUC_lexicon_tags_37 },
    { 0x50f8b2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa34f8b2f, SUC_lexicon_tags_25 },
    { 0xd8ef8b30, SUC_lexicon_tags_7 },
    { 0x9def8b30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694f8b37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf2f8b39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37af8b3c, SUC_lexicon_tags_17 },
    { 0xeecf8b3c, SUC_lexicon_tags_84 },
    { 0x8e4f8b3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x874f8b40, SUC_lexicon_tags_7 },
    { 0x1c2f8b41, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ef8b44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26af8b49, SUC_lexicon_tags_3 },
    { 0x1a0f8b4a, SUC_lexicon_tags_26 },
    { 0x158f8b4a, SUC_lexicon_tags_22 },
    { 0x4aaf8b4c, SUC_lexicon_tags_12 },
    { 0xe84f8b4d, SUC_lexicon_tags_11 },
    { 0x594f8b4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ef8b52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8a4f8b54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbe2f8b56, SUC_lexicon_tags_11 },
    { 0x1aaf8b56, SUC_lexicon_tags_25 },
    { 0x596f8b58, SUC_lexicon_tags_26 },
    { 0xa44f8b59, SUC_lexicon_tags_2 },
    { 0x1bef8b5a, SUC_lexicon_tags_63 },
    { 0xa74f8b5b, SUC_lexicon_tags_17 },
    { 0xd54f8b5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x740f8b5e, SUC_lexicon_tags_6 },
    { 0xaaf8b5f, SUC_lexicon_tags_60 },
    { 0x250f8b5f, SUC_lexicon_tags_37 },
    { 0xb1ef8b61, SUC_lexicon_tags_25 },
    { 0xcfaf8b62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8f8b69, SUC_lexicon_tags_37 },
    { 0xfa4f8b6a, SUC_lexicon_tags_37 },
    { 0x95cf8b6b, SUC_lexicon_tags_39 },
    { 0xaecf8b6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9af8b6e, SUC_lexicon_tags_23 },
    { 0x51cf8b6f, SUC_lexicon_tags_55 },
    { 0x70af8b70, SUC_lexicon_tags_23 },
    { 0x4a8f8b6f, SUC_lexicon_tags_46 },
    { 0xe32f8b70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x860f8b74, SUC_lexicon_tags_31 },
    { 0xc40f8b74, SUC_lexicon_tags_10 },
    { 0x2d0f8b76, SUC_lexicon_tags_17 },
    { 0xca2f8b77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x812f8b79, SUC_lexicon_tags_19 },
    { 0x1d4f8b7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eef8b7e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x37ef8b80, SUC_lexicon_tags_11 },
    { 0x74cf8b81, SUC_lexicon_tags_32 },
    { 0xec2f8b82, SUC_lexicon_tags_11 },
    { 0xb7cf8b81, SUC_lexicon_tags_11 },
    { 0x5fef8b84, SUC_lexicon_tags_25 },
    { 0x294f8b85, SUC_lexicon_tags_17 },
    { 0xf26f8b86, SUC_lexicon_tags_46 },
    { 0x6d6f8b85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8f8b8c, SUC_lexicon_tags_17 },
    { 0xaaef8b8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1baf8b95, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xce6f8b98, SUC_lexicon_tags_25 },
    { 0xbaf8b99, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f8ba0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74f8ba4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c8f8baa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24f8bb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17af8bb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8f8bb9, SUC_lexicon_tags_28 },
    { 0xb30f8bba, SUC_lexicon_tags_6 },
    { 0x2cf8bb9, SUC_lexicon_tags_299 },
    { 0x142f8bbb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xda2f8bbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710f8bc4, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ef8bca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a6f8bcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900f8bd2, SUC_lexicon_tags_77 },
    { 0x690f8bd3, SUC_lexicon_tags_0 },
    { 0x672f8bd4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cf8bd7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xba4f8bd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fcf8bdb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb76f8bdd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x736f8bdf, SUC_lexicon_tags_1 },
    { 0xd00f8bdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d4f8be2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x918f8be4, SUC_lexicon_tags_26 },
    { 0x1fef8be5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x932f8be7, SUC_lexicon_tags_37 },
    { 0x94af8be8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94f8beb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x82af8bed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42af8bf0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdbcf8bf2, SUC_lexicon_tags_84 },
    { 0x82f8bf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8f8bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea0f8bfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x346f8bfe, SUC_lexicon_tags_23 },
    { 0x5a8f8bfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bef8c03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26f8c08, SUC_lexicon_tags_28 },
    { 0x4c8f8c09, SUC_lexicon_tags_15 },
    { 0xb2cf8c09, SUC_lexicon_tags_17 },
    { 0xefcf8c09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150f8c10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x738f8c12, SUC_lexicon_tags_55 },
    { 0xcacf8c13, SUC_lexicon_tags_27 },
    { 0x422f8c12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcf8c1d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x512f8c1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0f8c26, SUC_lexicon_tags_45 },
    { 0x5c0f8c26, SUC_lexicon_tags_17 },
    { 0x39cf8c28, SUC_lexicon_tags_11 },
    { 0x384f8c26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0f8c2c, SUC_lexicon_tags_31 },
    { 0xb38f8c2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x862f8c34, SUC_lexicon_tags_7 },
    { 0xb68f8c35, SUC_lexicon_tags_60 },
    { 0x47cf8c36, SUC_lexicon_tags_31 },
    { 0x452f8c37, SUC_lexicon_tags_26 },
    { 0x8aef8c38, SUC_lexicon_tags_37 },
    { 0x500f8c39, SUC_lexicon_tags_27 },
    { 0xab2f8c38, SUC_lexicon_tags_37 },
    { 0x458f8c38, SUC_lexicon_tags_0 },
    { 0xf0ef8c39, SUC_lexicon_tags_19 },
    { 0x172f8c3d, SUC_lexicon_tags_29 },
    { 0x824f8c3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ef8c44, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x26ef8c46, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724f8c4c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbbcf8c4e, SUC_lexicon_tags_47 },
    { 0x53ef8c4f, SUC_lexicon_tags_14 },
    { 0xdeef8c50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x286f8c52, SUC_lexicon_tags_12 },
    { 0x9f2f8c53, SUC_lexicon_tags_17 },
    { 0x11ef8c53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x882f8c57, SUC_lexicon_tags_17 },
    { 0x4b6f8c58, SUC_lexicon_tags_21 },
    { 0xe7cf8c59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9faf8c60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88ef8c62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x378f8c64, SUC_lexicon_tags_27 },
    { 0x53af8c65, SUC_lexicon_tags_39 },
    { 0x258f8c66, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14af8c6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccf8c73, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930f8c77, SUC_lexicon_tags_60 },
    { 0xfd6f8c78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486f8c7f, SUC_lexicon_tags_2 },
    { 0x88af8c80, SUC_lexicon_tags_7 },
    { 0x8fef8c81, SUC_lexicon_tags_37 },
    { 0xf72f8c82, SUC_lexicon_tags_43 },
    { 0x2a6f8c7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ef8c86, SUC_lexicon_tags_25 },
    { 0x76f8c87, SUC_lexicon_tags_9 },
    { 0xfd8f8c88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x536f8c8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bef8c93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeaf8c99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aef8c9c, SUC_lexicon_tags_33 },
    { 0xb72f8c9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eef8ca2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x37ef8ca4, SUC_lexicon_tags_45 },
    { 0x5f8f8ca4, SUC_lexicon_tags_47 },
    { 0x8a6f8ca5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4f8ca9, SUC_lexicon_tags_8 },
    { 0x6e0f8ca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62f8cb1, SUC_lexicon_tags_17 },
    { 0xf2cf8cb2, SUC_lexicon_tags_60 },
    { 0x96ef8cb3, SUC_lexicon_tags_1 },
    { 0xf7ef8cb4, SUC_lexicon_tags_37 },
    { 0x5aef8cb4, SUC_lexicon_tags_11 },
    { 0x110f8cb3, SUC_lexicon_tags_11 },
    { 0x630f8cb3, SUC_lexicon_tags_12 },
    { 0xcf4f8cb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94f8cbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd0af8cbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed6f8cc1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x35cf8cc3, SUC_lexicon_tags_11 },
    { 0xd74f8cc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6f8cc7, SUC_lexicon_tags_14 },
    { 0x728f8cc8, SUC_lexicon_tags_12 },
    { 0x606f8cc7, SUC_lexicon_tags_37 },
    { 0x2a8f8cc9, SUC_lexicon_tags_37 },
    { 0xd2cf8cca, SUC_lexicon_tags_82 },
    { 0x80cf8cc9, SUC_lexicon_tags_250 },
    { 0x70f8ccd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cf8cd2, SUC_lexicon_tags_51 },
    { 0x750f8cd3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7ccf8cd5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x356f8cd8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf8cdb, SUC_lexicon_tags_15 },
    { 0x4e8f8cdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8f8ce1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x300f8ce4, SUC_lexicon_tags_12 },
    { 0xe06f8ce4, SUC_lexicon_tags_25 },
    { 0xf6af8ce5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6f8ce9, SUC_lexicon_tags_40 },
    { 0xf38f8cea, SUC_lexicon_tags_14 },
    { 0x9fcf8ceb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30ef8ced, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ef8cf1, SUC_lexicon_tags_7 },
    { 0x2c0f8cf2, SUC_lexicon_tags_7 },
    { 0x126f8cf3, SUC_lexicon_tags_25 },
    { 0xb7cf8cf4, SUC_lexicon_tags_22 },
    { 0xc8f8cf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6f8cfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f8f8cfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x328f8cff, SUC_lexicon_tags_30 },
    { 0x574f8d00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x34f8d02, SUC_lexicon_tags_37 },
    { 0xc76f8d03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ef8d0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x124f8d0c, SUC_lexicon_tags_7 },
    { 0xa32f8d0d, SUC_lexicon_tags_2 },
    { 0x74ef8d0c, SUC_lexicon_tags_27 },
    { 0xa3ef8d0f, SUC_lexicon_tags_64 },
    { 0xd84f8d10, SUC_lexicon_tags_70 },
    { 0x466f8d0e, SUC_lexicon_tags_43 },
    { 0x852f8d0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c0f8d14, SUC_lexicon_tags_10 },
    { 0x678f8d14, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0x5e6f8d17, SUC_lexicon_tags_12 },
    { 0xf3ef8d18, SUC_lexicon_tags_2 },
    { 0xf9af8d18, SUC_lexicon_tags_2 },
    { 0x9dcf8d1a, SUC_lexicon_tags_15 },
    { 0xd2cf8d1a, SUC_lexicon_tags_56 },
    { 0x458f8d1a, SUC_lexicon_tags_25 },
    { 0x800f8d1d, SUC_lexicon_tags_1 },
    { 0x2d8f8d1d, SUC_lexicon_tags_68 },
    { 0x294f8d1f, SUC_lexicon_tags_23 },
    { 0x47af8d1e, SUC_lexicon_tags_25 },
    { 0xe4ef8d21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0f8d27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8f8d2a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700f8d35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6f8d38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe20f8d3a, SUC_lexicon_tags_8 },
    { 0x718f8d3a, SUC_lexicon_tags_6 },
    { 0xbdef8d3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92f8d41, SUC_lexicon_tags_193 },
    { 0x7a4f8d42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96cf8d45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4c8f8d47, SUC_lexicon_tags_14 },
    { 0x81ef8d47, SUC_lexicon_tags_23 },
    { 0x144f8d49, SUC_lexicon_tags_2 },
    { 0x2e6f8d4a, SUC_lexicon_tags_17 },
    { 0x5c0f8d4b, SUC_lexicon_tags_33 },
    { 0x49cf8d4b, SUC_lexicon_tags_17 },
    { 0xc02f8d4d, SUC_lexicon_tags_38 },
    { 0x662f8d4d, SUC_lexicon_tags_12 },
    { 0x240f8d4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968f8d53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10cf8d55, SUC_lexicon_tags_11 },
    { 0xb38f8d55, SUC_lexicon_tags_45 },
    { 0xa8cf8d57, SUC_lexicon_tags_28 },
    { 0x26af8d57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2bef8d5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe66f8d62, SUC_lexicon_tags_13 },
    { 0xa74f8d63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798f8d67, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc92f8d69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13af8d6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6f8d72, SUC_lexicon_tags_29 },
    { 0xfd6f8d72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe2f8d75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cf8d79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcf8d7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc40f8d80, SUC_lexicon_tags_85 },
    { 0x5c6f8d81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780f8d87, SUC_lexicon_tags_59 },
    { 0xd70f8d88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc50f8d8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b4f8d8c, SUC_lexicon_tags_25 },
    { 0xa2ef8d8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ef8d90, SUC_lexicon_tags_19 },
    { 0x82ef8d90, SUC_lexicon_tags_1 },
    { 0x2b6f8d92, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaf8d98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x964f8d9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccf8da1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96cf8da6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfa2f8da8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce6f8daa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0f8dad, SUC_lexicon_tags_87 },
    { 0x360f8dae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ef8db1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cf8db5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2f8db9, SUC_lexicon_tags_25 },
    { 0x86cf8dba, SUC_lexicon_tags_47 },
    { 0x482f8dba, SUC_lexicon_tags_216 },
    { 0x630f8dbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ef8dbe, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd4f8dc0, SUC_lexicon_tags_2 },
    { 0xdccf8dc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0f8dc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x766f8dc8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2cf8dce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b4f8dd0, SUC_lexicon_tags_7 },
    { 0xb28f8dd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf16f8dd3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970f8dd7, SUC_lexicon_tags_91 },
    { 0x184f8dd7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92f8de0, SUC_lexicon_tags_23 },
    { 0xdbaf8de1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbecf8de7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8f8dea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c4f8dec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4f8df3, SUC_lexicon_tags_11 },
    { 0xb7af8df4, SUC_lexicon_tags_25 },
    { 0xc72f8df5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x300f8df7, SUC_lexicon_tags_32 },
    { 0x264f8df8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2f8dfd, SUC_lexicon_tags_11 },
    { 0x2c6f8dfe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x648f8e07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20af8e11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eaf8e16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae0f8e18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2f8e1b, SUC_lexicon_tags_37 },
    { 0x6fef8e1b, SUC_lexicon_tags_27 },
    { 0x38cf8e1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd26f8e22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af8e25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44f8e2d, SUC_lexicon_tags_43 },
    { 0x872f8e2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44f8e32, SUC_lexicon_tags_25 },
    { 0xe12f8e32, SUC_lexicon_tags_4 },
    { 0x1c2f8e34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890f8e3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf8e3f, SUC_lexicon_tags_30 },
    { 0x420f8e40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaef8e44, SUC_lexicon_tags_38 },
    { 0x29cf8e45, SUC_lexicon_tags_26 },
    { 0x9d2f8e44, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa28f8e4a, SUC_lexicon_tags_25 },
    { 0x62af8e4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc02f8e4e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x564f8e50, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0x434f8e52, SUC_lexicon_tags_68 },
    { 0xe70f8e53, SUC_lexicon_tags_49 },
    { 0xcf4f8e54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6f8e59, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf8e5c, SUC_lexicon_tags_23 },
    { 0x6f6f8e5d, SUC_lexicon_tags_17 },
    { 0x2eef8e5e, SUC_lexicon_tags_45 },
    { 0xd68f8e5f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48cf8e63, SUC_lexicon_tags_38 },
    { 0x46f8e64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55af8e67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85ef8e69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698f8e6e, SUC_lexicon_tags_29 },
    { 0x91af8e6e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xed6f8e71, SUC_lexicon_tags_17 },
    { 0xa88f8e71, SUC_lexicon_tags_15 },
    { 0x5caf8e73, SUC_lexicon_tags_25 },
    { 0x86cf8e74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10f8e7d, SUC_lexicon_tags_7 },
    { 0xb90f8e7e, SUC_lexicon_tags_7 },
    { 0x57cf8e7f, SUC_lexicon_tags_46 },
    { 0x99ef8e7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x782f8e83, SUC_lexicon_tags_14 },
    { 0xfbaf8e84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x680f8e87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23ef8e89, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb6f8e8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52ef8e8d, SUC_lexicon_tags_29 },
    { 0x29cf8e8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5cef8e90, SUC_lexicon_tags_136 },
    { 0xf6cf8e91, SUC_lexicon_tags_68 },
    { 0x4b8f8e92, SUC_lexicon_tags_1 },
    { 0x720f8e93, SUC_lexicon_tags_43 },
    { 0xa0af8e94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8f8e9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f8ea0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faf8ea5, SUC_lexicon_tags_1 },
    { 0xa6f8ea6, SUC_lexicon_tags_39 },
    { 0x3a0f8ea7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdef8ead, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x158f8eaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f8eb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4f8eb6, SUC_lexicon_tags_32 },
    { 0x2f4f8eb7, SUC_lexicon_tags_12 },
    { 0x910f8eb8, SUC_lexicon_tags_63 },
    { 0xa4cf8eb9, SUC_lexicon_tags_17 },
    { 0x236f8eba, SUC_lexicon_tags_47 },
    { 0xe20f8ebb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa30f8ebe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68af8ec0, SUC_lexicon_tags_25 },
    { 0xdf0f8ec1, SUC_lexicon_tags_43 },
    { 0x8b6f8ec2, SUC_lexicon_tags_17 },
    { 0xea2f8ec3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2f8ec6, SUC_lexicon_tags_27 },
    { 0x420f8ec7, SUC_lexicon_tags_66 },
    { 0x252f8ec8, SUC_lexicon_tags_25 },
    { 0x3c0f8ec8, SUC_lexicon_tags_27 },
    { 0xb2ef8eca, SUC_lexicon_tags_26 },
    { 0xe12f8ecb, SUC_lexicon_tags_2 },
    { 0x6c4f8ecb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2f8ecf, SUC_lexicon_tags_2 },
    { 0xa56f8ecf, SUC_lexicon_tags_26 },
    { 0xeb0f8ed0, SUC_lexicon_tags_23 },
    { 0x356f8ed2, SUC_lexicon_tags_13 },
    { 0xfb4f8ecf, SUC_lexicon_tags_30 },
    { 0xe4f8ecf, SUC_lexicon_tags_43 },
    { 0xb8af8ed5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ef8ed9, SUC_lexicon_tags_832 },
    { 0x472f8eda, SUC_lexicon_tags_60 },
    { 0x90f8edb, SUC_lexicon_tags_23 },
    { 0xf7af8edc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62af8ede, SUC_lexicon_tags_1 },
    { 0x6d4f8edf, SUC_lexicon_tags_25 },
    { 0x44f8ee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88af8ee2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cef8eea, SUC_lexicon_tags_17 },
    { 0xa7cf8eeb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf56f8eed, SUC_lexicon_tags_65 },
    { 0x780f8eed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc02f8ef0, SUC_lexicon_tags_25 },
    { 0x352f8ef0, SUC_lexicon_tags_0 },
    { 0x6f8ef2, SUC_lexicon_tags_2 },
    { 0x536f8ef0, SUC_lexicon_tags_11 },
    { 0x978f8ef0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8f8ef8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6f8efb, SUC_lexicon_tags_29 },
    { 0xe50f8efb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00f8f03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6f8f07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef8f0a, SUC_lexicon_tags_1 },
    { 0x474f8f0b, SUC_lexicon_tags_29 },
    { 0x85ef8f0c, SUC_lexicon_tags_10 },
    { 0x44ef8f0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6f8f10, SUC_lexicon_tags_1 },
    { 0xc00f8f10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84f8f14, SUC_lexicon_tags_11 },
    { 0x2c4f8f15, SUC_lexicon_tags_63 },
    { 0x1baf8f15, SUC_lexicon_tags_92 },
    { 0x614f8f17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a8f8f19, SUC_lexicon_tags_12 },
    { 0xacf8f1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x49cf8f1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430f8f20, SUC_lexicon_tags_37 },
    { 0x6c6f8f20, SUC_lexicon_tags_25 },
    { 0x2c2f8f22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x482f8f24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x618f8f27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52cf8f2b, SUC_lexicon_tags_31 },
    { 0x310f8f2b, SUC_lexicon_tags_11 },
    { 0xd94f8f2b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3def8f31, SUC_lexicon_tags_37 },
    { 0x95ef8f32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2af8f38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x554f8f3a, SUC_lexicon_tags_4 },
    { 0x63cf8f3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6f8f3f, SUC_lexicon_tags_191 },
    { 0xeb2f8f40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5af8f42, SUC_lexicon_tags_66 },
    { 0x3a8f8f43, SUC_lexicon_tags_37 },
    { 0xeb8f8f43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9d2f8f46, SUC_lexicon_tags_26 },
    { 0x2fcf8f47, SUC_lexicon_tags_30 },
    { 0x706f8f48, SUC_lexicon_tags_28 },
    { 0x524f8f46, SUC_lexicon_tags_17 },
    { 0xf7af8f46, SUC_lexicon_tags_37 },
    { 0x518f8f4b, SUC_lexicon_tags_79 },
    { 0xf58f8f46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e4f8f4e, SUC_lexicon_tags_25 },
    { 0x102f8f4e, SUC_lexicon_tags_19 },
    { 0x70ef8f50, SUC_lexicon_tags_36 },
    { 0x664f8f51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c8f8f53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382f8f5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x786f8f5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdaf8f5e, SUC_lexicon_tags_32 },
    { 0x614f8f5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73af8f61, SUC_lexicon_tags_66 },
    { 0xcdcf8f62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4f8f68, SUC_lexicon_tags_43 },
    { 0x8aef8f69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef8f6f, SUC_lexicon_tags_9 },
    { 0x7cef8f70, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x390f8f72, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ef8f79, SUC_lexicon_tags_11 },
    { 0x634f8f7a, SUC_lexicon_tags_17 },
    { 0x76cf8f7b, SUC_lexicon_tags_45 },
    { 0xd10f8f7c, SUC_lexicon_tags_2 },
    { 0x878f8f7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712f8f82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ef8f87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f8f8a, SUC_lexicon_tags_7 },
    { 0x862f8f8b, SUC_lexicon_tags_37 },
    { 0x280f8f8b, SUC_lexicon_tags_33 },
    { 0xb3ef8f8c, SUC_lexicon_tags_37 },
    { 0xa76f8f8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7af8f92, SUC_lexicon_tags_66 },
    { 0x2e0f8f93, SUC_lexicon_tags_14 },
    { 0x75ef8f94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798f8f9f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x990f8fa2, SUC_lexicon_tags_11 },
    { 0xf4ef8fa2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x248f8fa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40cf8fa8, SUC_lexicon_tags_30 },
    { 0xff0f8fa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54cf8fab, SUC_lexicon_tags_55 },
    { 0x24cf8fac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x756f8fae, SUC_lexicon_tags_37 },
    { 0x57af8faf, SUC_lexicon_tags_25 },
    { 0xa58f8fae, SUC_lexicon_tags_7 },
    { 0x54cf8fb1, SUC_lexicon_tags_61 },
    { 0xe76f8fb2, SUC_lexicon_tags_28 },
    { 0xc58f8fb3, SUC_lexicon_tags_11 },
    { 0x1f4f8fb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ef8fb7, SUC_lexicon_tags_17 },
    { 0x22f8fb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf14f8fba, SUC_lexicon_tags_17 },
    { 0x92af8fbb, SUC_lexicon_tags_37 },
    { 0x24f8fbc, SUC_lexicon_tags_46 },
    { 0x9e8f8fbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2f8fc0, SUC_lexicon_tags_25 },
    { 0x300f8fc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8fc7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44f8fcf, SUC_lexicon_tags_2 },
    { 0xf92f8fcf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6f8fd2, SUC_lexicon_tags_4 },
    { 0x3f2f8fd2, SUC_lexicon_tags_15 },
    { 0x8f0f8fd4, SUC_lexicon_tags_37 },
    { 0xcaef8fd3, SUC_lexicon_tags_63 },
    { 0x54f8fd4, SUC_lexicon_tags_11 },
    { 0x6d4f8fd7, SUC_lexicon_tags_15 },
    { 0x79af8fd8, SUC_lexicon_tags_8 },
    { 0x6d0f8fd9, SUC_lexicon_tags_15 },
    { 0xafcf8fd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7af8fdd, SUC_lexicon_tags_6 },
    { 0xecef8fde, SUC_lexicon_tags_11 },
    { 0xa76f8fdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc72f8fe1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d4f8fe3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ef8fe7, SUC_lexicon_tags_47 },
    { 0x276f8fe8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ef8feb, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6f8ffc, SUC_lexicon_tags_43 },
    { 0xfcaf8ffc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10cf8fff, SUC_lexicon_tags_2 },
    { 0x7f2f9000, SUC_lexicon_tags_37 },
    { 0xacf9001, SUC_lexicon_tags_12 },
    { 0x9cef9002, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb1cf9004, SUC_lexicon_tags_9 },
    { 0xbacf9005, SUC_lexicon_tags_16 },
    { 0xbbef9005, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe00f9008, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ef900a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f0f900c, SUC_lexicon_tags_13 },
    { 0x74ef900d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce6f9011, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x214f9014, SUC_lexicon_tags_7 },
    { 0xec8f9014, SUC_lexicon_tags_28 },
    { 0x2ccf9016, SUC_lexicon_tags_12 },
    { 0x764f9017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55cf9019, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2f9022, SUC_lexicon_tags_11 },
    { 0x32cf9023, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xad4f9026, SUC_lexicon_tags_11 },
    { 0x660f9027, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18af902d, SUC_lexicon_tags_43 },
    { 0x6cf902e, SUC_lexicon_tags_17 },
    { 0x344f902e, SUC_lexicon_tags_15 },
    { 0x140f9030, SUC_lexicon_tags_47 },
    { 0x446f9031, SUC_lexicon_tags_45 },
    { 0x1af9030, SUC_lexicon_tags_25 },
    { 0x5ef9033, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70f903a, SUC_lexicon_tags_66 },
    { 0xd78f903b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19af9044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d2f9046, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64ef9048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888f904c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf32f904f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x582f9051, SUC_lexicon_tags_7 },
    { 0xb78f9051, SUC_lexicon_tags_25 },
    { 0x46f9053, SUC_lexicon_tags_33 },
    { 0x57ef9054, SUC_lexicon_tags_37 },
    { 0xc82f9054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc90f9057, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84f905b, SUC_lexicon_tags_29 },
    { 0x9eef905c, SUC_lexicon_tags_1 },
    { 0xdc4f905d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e6f905f, SUC_lexicon_tags_45 },
    { 0x1f6f905f, SUC_lexicon_tags_60 },
    { 0x5f0f9061, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196f906a, SUC_lexicon_tags_47 },
    { 0xb04f906b, SUC_lexicon_tags_12 },
    { 0xfbef906b, SUC_lexicon_tags_68 },
    { 0x7baf906c, SUC_lexicon_tags_15 },
    { 0x4d6f906e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd4f9070, SUC_lexicon_tags_27 },
    { 0x36af9071, SUC_lexicon_tags_1 },
    { 0xa5ef9072, SUC_lexicon_tags_55 },
    { 0x392f9071, SUC_lexicon_tags_1 },
    { 0xd0f9074, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a8f9078, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0af907b, SUC_lexicon_tags_14 },
    { 0xc70f907c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74cf907e, SUC_lexicon_tags_42 },
    { 0x4ef907e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x884f9081, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cf9086, SUC_lexicon_tags_12 },
    { 0x1ecf9086, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9af9089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602f908e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96f9096, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8def9099, SUC_lexicon_tags_14 },
    { 0x91af909a, SUC_lexicon_tags_27 },
    { 0xeb0f909a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe10f909d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5af90a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x744f90a6, SUC_lexicon_tags_23 },
    { 0x748f90a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x984f90aa, SUC_lexicon_tags_7 },
    { 0xfd8f90ab, SUC_lexicon_tags_65 },
    { 0x23af90ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x568f90ae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24af90b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0f90b9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaf6f90bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504f90c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04f90c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f6f90c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x986f90cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0f90ce, SUC_lexicon_tags_28 },
    { 0x82cf90cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8f90d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x320f90d5, SUC_lexicon_tags_11 },
    { 0x332f90d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16cf90d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6f90dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe02f90e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cef90e3, SUC_lexicon_tags_1 },
    { 0xe3af90e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92f90e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x416f90e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x300f90eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x214f90ed, SUC_lexicon_tags_15 },
    { 0x5f8f90ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71af90f2, SUC_lexicon_tags_2 },
    { 0x3f0f90f2, SUC_lexicon_tags_12 },
    { 0xba8f90f4, SUC_lexicon_tags_82 },
    { 0x8c6f90f5, SUC_lexicon_tags_26 },
    { 0x400f90f3, SUC_lexicon_tags_87 },
    { 0x8b0f90f3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8a6f90f9, SUC_lexicon_tags_8 },
    { 0xffef90fa, SUC_lexicon_tags_33 },
    { 0x622f90fb, SUC_lexicon_tags_14 },
    { 0x874f90fa, SUC_lexicon_tags_53 },
    { 0x940f90fd, SUC_lexicon_tags_2 },
    { 0xf52f90fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64af9101, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccf9105, SUC_lexicon_tags_10 },
    { 0xd40f9105, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cf910e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa62f9111, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8af9117, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6faf9119, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aaf911c, SUC_lexicon_tags_29 },
    { 0x15cf911d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8f9120, SUC_lexicon_tags_17 },
    { 0x438f9120, SUC_lexicon_tags_136 },
    { 0xfecf9122, SUC_lexicon_tags_2 },
    { 0xa94f9123, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8a2f9125, SUC_lexicon_tags_29 },
    { 0x1fcf9125, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af9129, SUC_lexicon_tags_11 },
    { 0x216f912a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93cf912c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x920f9131, SUC_lexicon_tags_16 },
    { 0x91ef9132, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x364f9134, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66af9138, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cf913b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3dcf913d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ef9140, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394f9146, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0af914b, SUC_lexicon_tags_84 },
    { 0x15cf914c, SUC_lexicon_tags_12 },
    { 0x470f914c, SUC_lexicon_tags_1 },
    { 0x49cf914b, SUC_lexicon_tags_2 },
    { 0xae8f914b, SUC_lexicon_tags_49 },
    { 0x14ef914f, SUC_lexicon_tags_12 },
    { 0x544f9151, SUC_lexicon_tags_8 },
    { 0x594f9151, SUC_lexicon_tags_14 },
    { 0x288f914e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ccf9155, SUC_lexicon_tags_37 },
    { 0x28af9156, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacaf915d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x494f915f, SUC_lexicon_tags_11 },
    { 0x78f9160, SUC_lexicon_tags_1 },
    { 0x598f9160, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3c6f9163, SUC_lexicon_tags_33 },
    { 0x2e8f9164, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2f9167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafef9169, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x330f916b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56f916f, SUC_lexicon_tags_114 },
    { 0x41cf9170, SUC_lexicon_tags_17 },
    { 0xa10f9171, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4f9176, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x68cf9178, SUC_lexicon_tags_11 },
    { 0xccaf9179, SUC_lexicon_tags_17 },
    { 0x54cf917a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5caf917c, SUC_lexicon_tags_15 },
    { 0x6ccf917d, SUC_lexicon_tags_17 },
    { 0x3d4f917d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cf9187, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x794f9189, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ef918e, SUC_lexicon_tags_25 },
    { 0xaaaf918f, SUC_lexicon_tags_28 },
    { 0x2b2f9190, SUC_lexicon_tags_7 },
    { 0xf34f9191, SUC_lexicon_tags_53 },
    { 0x414f9191, SUC_lexicon_tags_126 },
    { 0x4af9193, SUC_lexicon_tags_43 },
    { 0x942f9193, SUC_lexicon_tags_68 },
    { 0x630f9195, SUC_lexicon_tags_17 },
    { 0x456f9196, SUC_lexicon_tags_35 },
    { 0x3def9197, SUC_lexicon_tags_12 },
    { 0x6c2f9198, SUC_lexicon_tags_1 },
    { 0x106f9191, SUC_lexicon_tags_6 },
    { 0xb70f9197, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0f919d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x31af91a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ef91a7, SUC_lexicon_tags_11 },
    { 0xe1ef91a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x98af91aa, SUC_lexicon_tags_79 },
    { 0x2d0f91aa, SUC_lexicon_tags_11 },
    { 0xadef91aa, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fcf91b0, SUC_lexicon_tags_33 },
    { 0xe7cf91b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccf91ba, SUC_lexicon_tags_106 },
    { 0x16af91ba, SUC_lexicon_tags_12 },
    { 0xc4f91ba, SUC_lexicon_tags_25 },
    { 0xc48f91bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc8f91c0, SUC_lexicon_tags_15 },
    { 0x56af91c0, SUC_lexicon_tags_46 },
    { 0x1d4f91c0, SUC_lexicon_tags_18 },
    { 0xadaf91c3, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522f91c9, SUC_lexicon_tags_29 },
    { 0xafcf91ca, SUC_lexicon_tags_11 },
    { 0x15cf91c9, SUC_lexicon_tags_30 },
    { 0xd38f91cc, SUC_lexicon_tags_23 },
    { 0xc06f91cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4af91d3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59af91d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x466f91e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x972f91e2, SUC_lexicon_tags_62 },
    { 0x718f91e3, SUC_lexicon_tags_15 },
    { 0xc9ef91e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2f91e9, SUC_lexicon_tags_43 },
    { 0x90ef91e9, SUC_lexicon_tags_37 },
    { 0xca2f91eb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cf91ef, SUC_lexicon_tags_65 },
    { 0xcfef91f0, SUC_lexicon_tags_47 },
    { 0x334f91f1, SUC_lexicon_tags_41 },
    { 0x19cf91f2, SUC_lexicon_tags_37 },
    { 0xc80f91f0, SUC_lexicon_tags_31 },
    { 0xaaf91f4, SUC_lexicon_tags_4 },
    { 0xbd4f91f5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8b0f91f7, SUC_lexicon_tags_63 },
    { 0xcbcf91f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8f91fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe2ef91fd, SUC_lexicon_tags_37 },
    { 0x7a0f91fe, SUC_lexicon_tags_25 },
    { 0x8fef91fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcf9203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cf9207, SUC_lexicon_tags_8 },
    { 0xf04f9208, SUC_lexicon_tags_57 },
    { 0x99ef9207, SUC_lexicon_tags_2 },
    { 0x196f9207, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00f920f, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cf9212, SUC_lexicon_tags_23 },
    { 0xd6af9212, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccef9219, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32ef921b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa20f921f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccf9226, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xec6f9229, SUC_lexicon_tags_38 },
    { 0x512f922a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7caf9230, SUC_lexicon_tags_12 },
    { 0x190f9231, SUC_lexicon_tags_37 },
    { 0x73ef9232, SUC_lexicon_tags_37 },
    { 0xa9af9233, SUC_lexicon_tags_12 },
    { 0x5f6f9234, SUC_lexicon_tags_15 },
    { 0x306f9234, SUC_lexicon_tags_9 },
    { 0x280f9236, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88f923b, SUC_lexicon_tags_12 },
    { 0x41ef923c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x664f923e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cf9246, SUC_lexicon_tags_7 },
    { 0x3faf9247, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706f924b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328f9250, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x268f9253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0f9256, SUC_lexicon_tags_31 },
    { 0xde4f9257, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184f925b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8f9260, SUC_lexicon_tags_2 },
    { 0x560f9260, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x38ef9263, SUC_lexicon_tags_77 },
    { 0x39ef9264, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ef926b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb74f926e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ef9277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf94f9279, SUC_lexicon_tags_14 },
    { 0x600f9279, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdd0f927c, SUC_lexicon_tags_27 },
    { 0x790f927c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7af927f, SUC_lexicon_tags_68 },
    { 0xd9af9280, SUC_lexicon_tags_14 },
    { 0xdfaf9281, SUC_lexicon_tags_30 },
    { 0x6acf9282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x980f9284, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebef9286, SUC_lexicon_tags_16 },
    { 0x214f9287, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8f928a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x770f928d, SUC_lexicon_tags_37 },
    { 0x900f928e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15cf9291, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ef9294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1baf929a, SUC_lexicon_tags_11 },
    { 0x68cf929a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50ef929d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cf92a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4f92a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27cf92a8, SUC_lexicon_tags_7 },
    { 0x34f92a9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfef92ad, SUC_lexicon_tags_79 },
    { 0xab2f92ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cf92b1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68f92b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8f92b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a6f92bc, SUC_lexicon_tags_37 },
    { 0xfb0f92bc, SUC_lexicon_tags_11 },
    { 0xe4ef92bd, SUC_lexicon_tags_11 },
    { 0xf14f92bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x66cf92c1, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9af92c5, SUC_lexicon_tags_0 },
    { 0xf14f92c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6faf92c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71ef92cb, SUC_lexicon_tags_19 },
    { 0x176f92cc, SUC_lexicon_tags_15 },
    { 0x228f92cd, SUC_lexicon_tags_14 },
    { 0x2b0f92ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9aef92d0, SUC_lexicon_tags_11 },
    { 0xe10f92d0, SUC_lexicon_tags_87 },
    { 0xf4af92d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d6f92d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x396f92d6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x214f92d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x792f92db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300f92df, SUC_lexicon_tags_43 },
    { 0x4c2f92df, SUC_lexicon_tags_25 },
    { 0x340f92e1, SUC_lexicon_tags_31 },
    { 0x9d0f92e1, SUC_lexicon_tags_9 },
    { 0x836f92e1, SUC_lexicon_tags_15 },
    { 0x52ef92e4, SUC_lexicon_tags_15 },
    { 0x59cf92e4, SUC_lexicon_tags_25 },
    { 0xa5ef92e6, SUC_lexicon_tags_45 },
    { 0x83cf92e2, SUC_lexicon_tags_37 },
    { 0xf70f92e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf90f92ea, SUC_lexicon_tags_28 },
    { 0xb0af92eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd66f92f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7baf92fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xabaf92ff, SUC_lexicon_tags_79 },
    { 0x8dcf9300, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60f9308, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0xde6f930b, SUC_lexicon_tags_17 },
    { 0xb1cf930b, SUC_lexicon_tags_2 },
    { 0xa18f930d, SUC_lexicon_tags_27 },
    { 0xcc8f930d, SUC_lexicon_tags_53 },
    { 0x67cf930b, SUC_lexicon_tags_84 },
    { 0xf0f930d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbaf9312, SUC_lexicon_tags_54 },
    { 0x712f9313, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x42ef9315, SUC_lexicon_tags_33 },
    { 0xb96f9316, SUC_lexicon_tags_2 },
    { 0xf36f9317, SUC_lexicon_tags_55 },
    { 0x71cf9318, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a0f931a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0f931f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa44f9324, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24f932c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48f9334, SUC_lexicon_tags_43 },
    { 0x668f9335, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x914f9337, SUC_lexicon_tags_11 },
    { 0x79cf9338, SUC_lexicon_tags_30 },
    { 0x362f9339, SUC_lexicon_tags_2 },
    { 0xf80f9337, SUC_lexicon_tags_0 },
    { 0x2e8f933b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6f9343, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bef934d, SUC_lexicon_tags_2 },
    { 0x13cf934e, SUC_lexicon_tags_7 },
    { 0x7ccf934e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bef9352, SUC_lexicon_tags_34 },
    { 0x534f9352, SUC_lexicon_tags_10 },
    { 0x56cf9354, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe46f9357, SUC_lexicon_tags_25 },
    { 0x36cf9357, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2f935b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ef935f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f9363, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8f9367, SUC_lexicon_tags_27 },
    { 0xc4f9368, SUC_lexicon_tags_2 },
    { 0xa30f9368, SUC_lexicon_tags_60 },
    { 0x66ef936a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x358f936d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3af936f, SUC_lexicon_tags_32 },
    { 0x87af936f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaef9373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8f9377, SUC_lexicon_tags_25 },
    { 0xab4f9378, SUC_lexicon_tags_26 },
    { 0x59ef9379, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8f937e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x426f9380, SUC_lexicon_tags_2 },
    { 0x772f9380, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40f9385, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98f9388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef938c, SUC_lexicon_tags_47 },
    { 0xe6f938d, SUC_lexicon_tags_355 },
    { 0x980f938d, SUC_lexicon_tags_14 },
    { 0x230f938e, SUC_lexicon_tags_25 },
    { 0xf8cf9390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cf9394, SUC_lexicon_tags_17 },
    { 0xfbcf9395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8f9398, SUC_lexicon_tags_49 },
    { 0xa32f9399, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x69cf939c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78f939f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294f93a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64f93a6, SUC_lexicon_tags_1 },
    { 0xc4ef93a7, SUC_lexicon_tags_2 },
    { 0x38af93a8, SUC_lexicon_tags_39 },
    { 0xd82f93a9, SUC_lexicon_tags_6 },
    { 0x44cf93aa, SUC_lexicon_tags_138 },
    { 0xc40f93ab, SUC_lexicon_tags_17 },
    { 0x894f93a9, SUC_lexicon_tags_4 },
    { 0x4eaf93a9, SUC_lexicon_tags_44 },
    { 0xb30f93ab, SUC_lexicon_tags_14 },
    { 0xd54f93ae, SUC_lexicon_tags_1 },
    { 0x48ef93b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48f93b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46f93bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd38f93bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14cf93c1, SUC_lexicon_tags_4 },
    { 0x7e4f93c1, SUC_lexicon_tags_25 },
    { 0x80ef93c3, SUC_lexicon_tags_26 },
    { 0x61af93c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaf93c8, SUC_lexicon_tags_42 },
    { 0x532f93c9, SUC_lexicon_tags_2 },
    { 0x2acf93c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20f93cf, SUC_lexicon_tags_49 },
    { 0x73af93d0, SUC_lexicon_tags_12 },
    { 0xfc4f93d1, SUC_lexicon_tags_27 },
    { 0x644f93d2, SUC_lexicon_tags_37 },
    { 0x2d2f93d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4f93d6, SUC_lexicon_tags_12 },
    { 0xd00f93d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf80f93d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x408f93dc, SUC_lexicon_tags_37 },
    { 0x942f93dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6f93e0, SUC_lexicon_tags_1 },
    { 0xfc0f93e1, SUC_lexicon_tags_2 },
    { 0xe92f93e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8d6f93e4, SUC_lexicon_tags_1 },
    { 0xeb8f93e5, SUC_lexicon_tags_12 },
    { 0x328f93e5, SUC_lexicon_tags_151 },
    { 0x28f93e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2f93ea, SUC_lexicon_tags_9 },
    { 0xdbcf93ea, SUC_lexicon_tags_15 },
    { 0x55ef93ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc6f93ee, SUC_lexicon_tags_66 },
    { 0x610f93ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6def93f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52f93fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x270f93fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc86f93ff, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x292f9402, SUC_lexicon_tags_29 },
    { 0xbe0f9403, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c2f9406, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x624f9408, SUC_lexicon_tags_151 },
    { 0x524f9409, SUC_lexicon_tags_2 },
    { 0x70cf9409, SUC_lexicon_tags_82 },
    { 0x42f9409, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2f940e, SUC_lexicon_tags_1 },
    { 0x592f940f, SUC_lexicon_tags_11 },
    { 0xd76f9410, SUC_lexicon_tags_12 },
    { 0xf9cf9410, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xffef9414, SUC_lexicon_tags_33 },
    { 0x796f9415, SUC_lexicon_tags_19 },
    { 0x4c8f9414, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cf941c, SUC_lexicon_tags_12 },
    { 0x89af941c, SUC_lexicon_tags_9 },
    { 0x6c0f941e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0f9421, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdc0f9423, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ef9428, SUC_lexicon_tags_12 },
    { 0x2c2f9429, SUC_lexicon_tags_46 },
    { 0x58ef9428, SUC_lexicon_tags_2 },
    { 0x60cf942a, SUC_lexicon_tags_7 },
    { 0xde2f942c, SUC_lexicon_tags_22 },
    { 0xfaef942d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30f942f, SUC_lexicon_tags_25 },
    { 0x3af9430, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x836f9433, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ef9437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f943c, SUC_lexicon_tags_10 },
    { 0x22af943d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686f9444, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7af9448, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4f944d, SUC_lexicon_tags_25 },
    { 0xb34f944e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496f9454, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5caf9456, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x524f9458, SUC_lexicon_tags_12 },
    { 0x768f9459, SUC_lexicon_tags_1 },
    { 0x50ef945a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb82f945e, SUC_lexicon_tags_17 },
    { 0xe36f945f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb4f9461, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ef9464, SUC_lexicon_tags_2 },
    { 0x390f9465, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1ef9467, SUC_lexicon_tags_2 },
    { 0x5e0f9467, SUC_lexicon_tags_8 },
    { 0xd64f9468, SUC_lexicon_tags_19 },
    { 0x44ef9467, SUC_lexicon_tags_27 },
    { 0x72ef946b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0f9471, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8f9474, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ef9477, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1af947a, SUC_lexicon_tags_1 },
    { 0x8e8f947b, SUC_lexicon_tags_2 },
    { 0xf0f947b, SUC_lexicon_tags_2 },
    { 0x1daf947d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefef9481, SUC_lexicon_tags_16 },
    { 0x52ef9482, SUC_lexicon_tags_23 },
    { 0x48f9481, SUC_lexicon_tags_1 },
    { 0x1cef9484, SUC_lexicon_tags_1 },
    { 0xd74f9484, SUC_lexicon_tags_98 },
    { 0x5c0f9483, SUC_lexicon_tags_2 },
    { 0x72af9487, SUC_lexicon_tags_30 },
    { 0xe8af9488, SUC_lexicon_tags_12 },
    { 0x764f9487, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504f948f, SUC_lexicon_tags_25 },
    { 0xa50f9490, SUC_lexicon_tags_25 },
    { 0xfd6f9491, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0f9494, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xae6f9496, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfeef9498, SUC_lexicon_tags_0 },
    { 0x43ef9499, SUC_lexicon_tags_8 },
    { 0x5c2f9499, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x690f949c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60f949f, SUC_lexicon_tags_9 },
    { 0x336f949f, SUC_lexicon_tags_45 },
    { 0x1e6f94a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ef94a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x380f94a8, SUC_lexicon_tags_11 },
    { 0xe4af94a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6def94ad, SUC_lexicon_tags_31 },
    { 0x9d2f94ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83cf94b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaf94b8, SUC_lexicon_tags_63 },
    { 0xd0ef94b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ef94bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9ef94be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaaf94c1, SUC_lexicon_tags_33 },
    { 0x3faf94c2, SUC_lexicon_tags_14 },
    { 0xb9cf94c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6af94c5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x568f94c7, SUC_lexicon_tags_92 },
    { 0xe86f94c7, SUC_lexicon_tags_12 },
    { 0xafef94c9, SUC_lexicon_tags_25 },
    { 0xb06f94ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc38f94cc, SUC_lexicon_tags_0 },
    { 0x456f94cd, SUC_lexicon_tags_37 },
    { 0x9cf94cd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23af94d7, SUC_lexicon_tags_7 },
    { 0x156f94d8, SUC_lexicon_tags_15 },
    { 0xdf8f94d8, SUC_lexicon_tags_45 },
    { 0xf26f94d9, SUC_lexicon_tags_44 },
    { 0x8aaf94da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80f94df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3af94e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe4f94e3, SUC_lexicon_tags_17 },
    { 0x1b0f94e3, SUC_lexicon_tags_25 },
    { 0xc0f94e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x736f94e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37af94e9, SUC_lexicon_tags_445 },
    { 0x4d6f94ea, SUC_lexicon_tags_2 },
    { 0xf6af94eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x21cf94ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ef94f1, SUC_lexicon_tags_15 },
    { 0x7aaf94f2, SUC_lexicon_tags_7 },
    { 0x108f94f2, SUC_lexicon_tags_2 },
    { 0xb56f94f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ef94fd, SUC_lexicon_tags_30 },
    { 0xde6f94fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78af9502, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f9505, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2cf9507, SUC_lexicon_tags_7 },
    { 0x394f9508, SUC_lexicon_tags_33 },
    { 0xca0f9509, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x444f950b, SUC_lexicon_tags_0 },
    { 0x504f950c, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0x834f950f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6f9513, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4f9516, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80f9519, SUC_lexicon_tags_1 },
    { 0x15ef951a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ef951d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81cf9522, SUC_lexicon_tags_11 },
    { 0x87ef9523, SUC_lexicon_tags_68 },
    { 0xfb4f9522, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eaf9529, SUC_lexicon_tags_15 },
    { 0xf3ef9529, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ef952d, SUC_lexicon_tags_63 },
    { 0x20af952e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x36af9530, SUC_lexicon_tags_38 },
    { 0xe6f9530, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7a8f9533, SUC_lexicon_tags_29 },
    { 0x3acf9534, SUC_lexicon_tags_14 },
    { 0xc1af9534, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc84f953a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x130f953d, SUC_lexicon_tags_30 },
    { 0x79af953e, SUC_lexicon_tags_14 },
    { 0xd8ef953f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cf9546, SUC_lexicon_tags_11 },
    { 0x38ef9547, SUC_lexicon_tags_27 },
    { 0x246f9548, SUC_lexicon_tags_17 },
    { 0x592f9548, SUC_lexicon_tags_42 },
    { 0x35cf9548, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcf9551, SUC_lexicon_tags_6 },
    { 0x306f9551, SUC_lexicon_tags_33 },
    { 0x29cf9553, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3cf9555, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc8f9559, SUC_lexicon_tags_38 },
    { 0x4b0f955a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabaf955f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddef9561, SUC_lexicon_tags_11 },
    { 0x31af9562, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9faf9567, SUC_lexicon_tags_23 },
    { 0x284f9567, SUC_lexicon_tags_27 },
    { 0xfe0f9569, SUC_lexicon_tags_11 },
    { 0x4c4f956a, SUC_lexicon_tags_28 },
    { 0xe28f956b, SUC_lexicon_tags_12 },
    { 0xf58f9567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14af9570, SUC_lexicon_tags_26 },
    { 0xda8f9570, SUC_lexicon_tags_25 },
    { 0x84f9572, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa84f9574, SUC_lexicon_tags_12 },
    { 0x44cf9575, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118f957b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ef957e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99af9580, SUC_lexicon_tags_37 },
    { 0xc0af9580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbccf9584, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa28f9586, SUC_lexicon_tags_25 },
    { 0xda6f9586, SUC_lexicon_tags_30 },
    { 0x7d0f9587, SUC_lexicon_tags_17 },
    { 0x654f9588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92f958b, SUC_lexicon_tags_15 },
    { 0x52cf958c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbef9597, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cf959a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3c4f959c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3ef959e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d6f95a0, SUC_lexicon_tags_14 },
    { 0xe66f95a0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x928f95a3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a6f95a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x750f95ab, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x216f95ad, SUC_lexicon_tags_127 },
    { 0x10af95ad, SUC_lexicon_tags_60 },
    { 0x4f6f95ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4baf95bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x842f95be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ef95c2, SUC_lexicon_tags_4 },
    { 0xf6ef95c2, SUC_lexicon_tags_11 },
    { 0xae8f95c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8f95c9, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0x3acf95cb, SUC_lexicon_tags_47 },
    { 0x690f95cc, SUC_lexicon_tags_1 },
    { 0x4bef95cd, SUC_lexicon_tags_2 },
    { 0x3af95cb, SUC_lexicon_tags_33 },
    { 0x7c0f95cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fef95d2, SUC_lexicon_tags_29 },
    { 0xa58f95d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90f95d8, SUC_lexicon_tags_17 },
    { 0x4ccf95d9, SUC_lexicon_tags_65 },
    { 0x578f95da, SUC_lexicon_tags_23 },
    { 0xef4f95db, SUC_lexicon_tags_82 },
    { 0x6b6f95db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2f95eb, SUC_lexicon_tags_11 },
    { 0x11af95ec, SUC_lexicon_tags_23 },
    { 0x9d6f95ec, SUC_lexicon_tags_77 },
    { 0xf4af95ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f95f2, SUC_lexicon_tags_19 },
    { 0xd74f95f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990f95f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaef95ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x824f9606, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf9609, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63cf960b, SUC_lexicon_tags_1 },
    { 0xda8f960b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606f9612, SUC_lexicon_tags_26 },
    { 0xa08f9612, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe8af9615, SUC_lexicon_tags_47 },
    { 0xbcef9615, SUC_lexicon_tags_17 },
    { 0x80af9617, SUC_lexicon_tags_47 },
    { 0x4d0f9618, SUC_lexicon_tags_25 },
    { 0x8eef9617, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79cf961b, SUC_lexicon_tags_27 },
    { 0x28f961b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16f9621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x924f9623, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x824f9625, SUC_lexicon_tags_15 },
    { 0xae4f9626, SUC_lexicon_tags_25 },
    { 0x684f9627, SUC_lexicon_tags_17 },
    { 0x138f9626, SUC_lexicon_tags_15 },
    { 0x7d4f9628, SUC_lexicon_tags_15 },
    { 0x24af9629, SUC_lexicon_tags_55 },
    { 0x630f962b, SUC_lexicon_tags_2 },
    { 0xb82f962b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4af962e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x306f9630, SUC_lexicon_tags_11 },
    { 0x110f9631, SUC_lexicon_tags_28 },
    { 0x33ef9632, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6f9636, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6f963b, SUC_lexicon_tags_4 },
    { 0xa50f963c, SUC_lexicon_tags_60 },
    { 0x608f963d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caf9644, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4bcf9646, SUC_lexicon_tags_57 },
    { 0x972f9647, SUC_lexicon_tags_11 },
    { 0x2f2f9647, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578f964d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x192f9651, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cf9656, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b4f9658, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cf965b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc2af965d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2f9660, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb56f9662, SUC_lexicon_tags_37 },
    { 0xfc2f9663, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x518f9666, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792f9671, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xadaf9673, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2f967f, SUC_lexicon_tags_133 },
    { 0x824f9680, SUC_lexicon_tags_25 },
    { 0x394f9680, SUC_lexicon_tags_15 },
    { 0xaa4f9682, SUC_lexicon_tags_68 },
    { 0x91ef9683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x570f9687, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4f968a, SUC_lexicon_tags_12 },
    { 0xc20f968a, SUC_lexicon_tags_47 },
    { 0xfc6f968c, SUC_lexicon_tags_37 },
    { 0xc6ef968b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414f9697, SUC_lexicon_tags_12 },
    { 0xf0f9698, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52f969d, SUC_lexicon_tags_8 },
    { 0x192f969e, SUC_lexicon_tags_15 },
    { 0x44ef969d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2af96a2, SUC_lexicon_tags_47 },
    { 0xe48f96a3, SUC_lexicon_tags_8 },
    { 0xe7ef96a3, SUC_lexicon_tags_33 },
    { 0x27af96a3, SUC_lexicon_tags_8 },
    { 0x396f96a3, SUC_lexicon_tags_7 },
    { 0xdb4f96a7, SUC_lexicon_tags_10 },
    { 0x9d4f96a8, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aaf96ab, SUC_lexicon_tags_14 },
    { 0xdc6f96ac, SUC_lexicon_tags_18 },
    { 0x1af96ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc42f96af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x11cf96b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290f96b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7acf96b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f8f96b9, SUC_lexicon_tags_63 },
    { 0x5b2f96b9, SUC_lexicon_tags_7 },
    { 0x7fef96bb, SUC_lexicon_tags_5 },
    { 0x510f96bb, SUC_lexicon_tags_26 },
    { 0x7a6f96bd, SUC_lexicon_tags_47 },
    { 0xa10f96bd, SUC_lexicon_tags_31 },
    { 0x55ef96bf, SUC_lexicon_tags_11 },
    { 0xafef96c0, SUC_lexicon_tags_17 },
    { 0x28af96bc, SUC_lexicon_tags_17 },
    { 0xbdaf96c0, SUC_lexicon_tags_63 },
    { 0x162f96bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56cf96c5, SUC_lexicon_tags_7 },
    { 0xc4ef96c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560f96ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61af96cc, SUC_lexicon_tags_10 },
    { 0xfb4f96cc, SUC_lexicon_tags_15 },
    { 0x9acf96ce, SUC_lexicon_tags_37 },
    { 0x154f96cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c8f96d1, SUC_lexicon_tags_2 },
    { 0xb78f96d1, SUC_lexicon_tags_2 },
    { 0xaaf96d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f96d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x302f96d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc22f96dc, SUC_lexicon_tags_61 },
    { 0xbfaf96dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x442f96df, SUC_lexicon_tags_17 },
    { 0x826f96df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24f96e4, SUC_lexicon_tags_55 },
    { 0x87af96e4, SUC_lexicon_tags_25 },
    { 0x256f96e4, SUC_lexicon_tags_19 },
    { 0x2fef96e5, SUC_lexicon_tags_45 },
    { 0x620f96e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcf96ec, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf96f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6f96f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x244f96f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cf96fb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106f9700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04f9704, SUC_lexicon_tags_7 },
    { 0xb30f9704, SUC_lexicon_tags_3 },
    { 0x6faf9704, SUC_lexicon_tags_25 },
    { 0xd2af9707, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74f970b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b2f970d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb24f970f, SUC_lexicon_tags_37 },
    { 0x994f970f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4f9715, SUC_lexicon_tags_26 },
    { 0xc8cf9715, SUC_lexicon_tags_14 },
    { 0xf58f9717, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ecf9719, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb16f971b, SUC_lexicon_tags_15 },
    { 0x820f971c, SUC_lexicon_tags_4 },
    { 0xaeef971b, SUC_lexicon_tags_45 },
    { 0x7c0f971d, SUC_lexicon_tags_37 },
    { 0x6aef971f, SUC_lexicon_tags_25 },
    { 0xa94f971f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22f9723, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8f9726, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cf972c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d2f972f, SUC_lexicon_tags_5 },
    { 0x910f972f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeecf973a, SUC_lexicon_tags_81 },
    { 0xd7ef973b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32f9740, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ef9743, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecf9747, SUC_lexicon_tags_29 },
    { 0xa4f9748, SUC_lexicon_tags_17 },
    { 0x926f9749, SUC_lexicon_tags_0 },
    { 0x540f9749, SUC_lexicon_tags_14 },
    { 0x1f0f974b, SUC_lexicon_tags_16 },
    { 0xa2cf974c, SUC_lexicon_tags_17 },
    { 0x46af974d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ef9752, SUC_lexicon_tags_37 },
    { 0x2f6f9752, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd02f9756, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51ef9758, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa20f975a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x29cf975d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6af9762, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a4f9764, SUC_lexicon_tags_49 },
    { 0xff4f9764, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678f976c, SUC_lexicon_tags_25 },
    { 0xad6f976c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35ef976f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bef9776, SUC_lexicon_tags_19 },
    { 0x31ef9777, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca2f9779, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x95cf977c, SUC_lexicon_tags_15 },
    { 0xb52f977c, SUC_lexicon_tags_4 },
    { 0x944f977c, SUC_lexicon_tags_12 },
    { 0x102f977f, SUC_lexicon_tags_33 },
    { 0xe0cf9780, SUC_lexicon_tags_28 },
    { 0x35ef9781, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6f9791, SUC_lexicon_tags_17 },
    { 0x33af9792, SUC_lexicon_tags_37 },
    { 0xd34f9793, SUC_lexicon_tags_17 },
    { 0xe10f9794, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4f9798, SUC_lexicon_tags_23 },
    { 0x7d8f9798, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70f979e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5d4f97a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47af97a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bef97ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48af97b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce8f97b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1af97b8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x154f97bb, SUC_lexicon_tags_37 },
    { 0x8e8f97bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x266f97be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca4f97c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1d4f97cb, SUC_lexicon_tags_10 },
    { 0x180f97cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672f97d4, SUC_lexicon_tags_25 },
    { 0xb86f97d4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x346f97d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2f97da, SUC_lexicon_tags_84 },
    { 0xc74f97da, SUC_lexicon_tags_19 },
    { 0x43af97da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50f97e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422f97e5, SUC_lexicon_tags_7 },
    { 0x2c4f97e6, SUC_lexicon_tags_672 },
    { 0x360f97e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcf97ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6f97ef, SUC_lexicon_tags_27 },
    { 0x886f97ef, SUC_lexicon_tags_1 },
    { 0x59af97f1, SUC_lexicon_tags_28 },
    { 0x9acf97f2, SUC_lexicon_tags_15 },
    { 0x740f97f3, SUC_lexicon_tags_584 },
    { 0, NULL },
    { 0, NULL },
    { 0x864f97f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbfef97f8, SUC_lexicon_tags_68 },
    { 0x18ef97f9, SUC_lexicon_tags_9 },
    { 0x1c0f97f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4f97fe, SUC_lexicon_tags_15 },
    { 0x2ef97ff, SUC_lexicon_tags_49 },
    { 0xa0ef9800, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0f9803, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcf9806, SUC_lexicon_tags_15 },
    { 0xae2f9807, SUC_lexicon_tags_19 },
    { 0xd20f9807, SUC_lexicon_tags_77 },
    { 0x7caf9809, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492f9811, SUC_lexicon_tags_12 },
    { 0x646f9812, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecf981f, SUC_lexicon_tags_25 },
    { 0xb66f9820, SUC_lexicon_tags_29 },
    { 0x6baf9821, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328f9825, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ef982b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20f9832, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffef9836, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62f9839, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41cf983c, SUC_lexicon_tags_9 },
    { 0x5a2f983d, SUC_lexicon_tags_10 },
    { 0xc76f983e, SUC_lexicon_tags_1 },
    { 0x9eaf983c, SUC_lexicon_tags_11 },
    { 0x35ef9840, SUC_lexicon_tags_12 },
    { 0xcc4f9841, SUC_lexicon_tags_14 },
    { 0xd80f9842, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af9846, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x960f9849, SUC_lexicon_tags_15 },
    { 0x8d8f984a, SUC_lexicon_tags_38 },
    { 0x53ef984b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2af984f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462f9856, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef9860, SUC_lexicon_tags_23 },
    { 0x4aaf9861, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ef9869, SUC_lexicon_tags_6 },
    { 0xed0f986a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x252f986e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270f9875, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8ef9877, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb06f9879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ef987c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc6f9880, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xc4af9882, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc40f9888, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x94af988a, SUC_lexicon_tags_28 },
    { 0xd3af988b, SUC_lexicon_tags_2 },
    { 0x414f988a, SUC_lexicon_tags_68 },
    { 0xfd0f988c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87af988f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcb8f9891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0cf9894, SUC_lexicon_tags_0 },
    { 0xc12f9895, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f9897, SUC_lexicon_tags_1 },
    { 0x334f9898, SUC_lexicon_tags_11 },
    { 0xd18f9898, SUC_lexicon_tags_43 },
    { 0xcacf9899, SUC_lexicon_tags_11 },
    { 0x4cef989b, SUC_lexicon_tags_26 },
    { 0xe0f989b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc60f989e, SUC_lexicon_tags_37 },
    { 0xdf4f989e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75cf98a2, SUC_lexicon_tags_11 },
    { 0xf16f98a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cf98ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84f98ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f98b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8f98b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11af98b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x626f98b9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xda0f98bb, SUC_lexicon_tags_37 },
    { 0x71cf98bc, SUC_lexicon_tags_17 },
    { 0x610f98bd, SUC_lexicon_tags_1 },
    { 0xd76f98bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60f98c2, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cf98c5, SUC_lexicon_tags_0 },
    { 0xbd0f98c6, SUC_lexicon_tags_60 },
    { 0xafef98c6, SUC_lexicon_tags_37 },
    { 0xffef98c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ef98cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7cf98ce, SUC_lexicon_tags_1 },
    { 0xa98f98cf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x952f98d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666f98d6, SUC_lexicon_tags_4 },
    { 0x978f98d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e8f98da, SUC_lexicon_tags_15 },
    { 0x2aef98db, SUC_lexicon_tags_17 },
    { 0x38cf98dc, SUC_lexicon_tags_60 },
    { 0x43cf98dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x88af98e0, SUC_lexicon_tags_37 },
    { 0x9c6f98e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cf98e4, SUC_lexicon_tags_37 },
    { 0x5e8f98e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cf98e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34f98ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x730f98f1, SUC_lexicon_tags_25 },
    { 0x36af98f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84af98f8, SUC_lexicon_tags_9 },
    { 0x1faf98f8, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116f9901, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x404f9903, SUC_lexicon_tags_1 },
    { 0xfb0f9903, SUC_lexicon_tags_17 },
    { 0x4a0f9903, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ef990a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaf9911, SUC_lexicon_tags_25 },
    { 0xfb0f9912, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82f9918, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x398f991b, SUC_lexicon_tags_27 },
    { 0x458f991b, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcf9924, SUC_lexicon_tags_4 },
    { 0x214f9925, SUC_lexicon_tags_63 },
    { 0x6af9924, SUC_lexicon_tags_7 },
    { 0x8c8f9924, SUC_lexicon_tags_85 },
    { 0x946f9925, SUC_lexicon_tags_2 },
    { 0x1d6f9925, SUC_lexicon_tags_65 },
    { 0xcf4f992a, SUC_lexicon_tags_45 },
    { 0xa2ef992a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cf9930, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x898f9932, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870f9937, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534f993b, SUC_lexicon_tags_3 },
    { 0x590f993c, SUC_lexicon_tags_17 },
    { 0x7e8f993d, SUC_lexicon_tags_61 },
    { 0x7f0f993e, SUC_lexicon_tags_11 },
    { 0x850f993e, SUC_lexicon_tags_2 },
    { 0x70ef993e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76af9945, SUC_lexicon_tags_63 },
    { 0x464f9946, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x326f9949, SUC_lexicon_tags_29 },
    { 0x846f9949, SUC_lexicon_tags_37 },
    { 0xbfcf994b, SUC_lexicon_tags_23 },
    { 0x840f9949, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1af994f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6aef9951, SUC_lexicon_tags_12 },
    { 0x986f9951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40f9955, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6f9959, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aef995f, SUC_lexicon_tags_11 },
    { 0x61af9960, SUC_lexicon_tags_56 },
    { 0x12ef995f, SUC_lexicon_tags_9 },
    { 0xc0f995f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f9965, SUC_lexicon_tags_11 },
    { 0xfecf9966, SUC_lexicon_tags_25 },
    { 0xc88f9967, SUC_lexicon_tags_2 },
    { 0xb4cf9966, SUC_lexicon_tags_24 },
    { 0x55ef9968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f996e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x160f9970, SUC_lexicon_tags_11 },
    { 0x8c0f9970, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeaf9974, SUC_lexicon_tags_31 },
    { 0xb26f9975, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19af9977, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89af997e, SUC_lexicon_tags_53 },
    { 0x560f997e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854f9983, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26af9989, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0f9990, SUC_lexicon_tags_17 },
    { 0x7cf9990, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92f9994, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ef9997, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ef999b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5acf999e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28f99a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14f99a4, SUC_lexicon_tags_16 },
    { 0x97ef99a5, SUC_lexicon_tags_263 },
    { 0x44f99a6, SUC_lexicon_tags_12 },
    { 0xec0f99a5, SUC_lexicon_tags_11 },
    { 0x824f99a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238f99ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x618f99b1, SUC_lexicon_tags_17 },
    { 0x762f99b2, SUC_lexicon_tags_27 },
    { 0x618f99b3, SUC_lexicon_tags_12 },
    { 0xac2f99b4, SUC_lexicon_tags_17 },
    { 0xe4ef99b5, SUC_lexicon_tags_25 },
    { 0x302f99b5, SUC_lexicon_tags_201 },
    { 0x9c8f99b6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x946f99b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcf99bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x77af99c0, SUC_lexicon_tags_7 },
    { 0xc4cf99c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf08f99c8, SUC_lexicon_tags_60 },
    { 0x48ef99c8, SUC_lexicon_tags_37 },
    { 0xb64f99c9, SUC_lexicon_tags_17 },
    { 0x198f99cb, SUC_lexicon_tags_17 },
    { 0xff0f99cc, SUC_lexicon_tags_15 },
    { 0x1caf99cd, SUC_lexicon_tags_63 },
    { 0xd70f99cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61af99d1, SUC_lexicon_tags_9 },
    { 0x550f99d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x810f99d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x51cf99d6, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x988f99dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x650f99df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74f99e4, SUC_lexicon_tags_0 },
    { 0x142f99e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc2f99e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf99ea, SUC_lexicon_tags_77 },
    { 0x1c6f99eb, SUC_lexicon_tags_17 },
    { 0xea4f99ec, SUC_lexicon_tags_12 },
    { 0xab4f99eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x458f99f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17af99f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8f99f7, SUC_lexicon_tags_11 },
    { 0xb66f99f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29af99fa, SUC_lexicon_tags_15 },
    { 0xce6f99fb, SUC_lexicon_tags_12 },
    { 0x6d6f99fc, SUC_lexicon_tags_15 },
    { 0x5cf99fc, SUC_lexicon_tags_63 },
    { 0xaeaf99fc, SUC_lexicon_tags_87 },
    { 0xb8af99ff, SUC_lexicon_tags_22 },
    { 0x5faf9a00, SUC_lexicon_tags_87 },
    { 0xc3af9a00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ef9a05, SUC_lexicon_tags_2 },
    { 0x5ecf9a06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740f9a0e, SUC_lexicon_tags_30 },
    { 0x4d6f9a0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x346f9a11, SUC_lexicon_tags_2 },
    { 0x4def9a11, SUC_lexicon_tags_33 },
    { 0x91af9a11, SUC_lexicon_tags_25 },
    { 0x5faf9a14, SUC_lexicon_tags_26 },
    { 0x19cf9a14, SUC_lexicon_tags_47 },
    { 0xb12f9a15, SUC_lexicon_tags_15 },
    { 0x488f9a14, SUC_lexicon_tags_37 },
    { 0x236f9a14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa0cf9a1a, SUC_lexicon_tags_7 },
    { 0xcb6f9a1b, SUC_lexicon_tags_1 },
    { 0xf0ef9a1c, SUC_lexicon_tags_65 },
    { 0x74f9a1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620f9a23, SUC_lexicon_tags_103 },
    { 0x3acf9a24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x59cf9a26, SUC_lexicon_tags_12 },
    { 0x1caf9a26, SUC_lexicon_tags_11 },
    { 0x546f9a26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6def9a2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220f9a31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e4f9a33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69af9a35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa00f9a37, SUC_lexicon_tags_17 },
    { 0xdfef9a37, SUC_lexicon_tags_39 },
    { 0xdecf9a38, SUC_lexicon_tags_27 },
    { 0x638f9a3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefcf9a3c, SUC_lexicon_tags_17 },
    { 0x654f9a3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2f9a41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34f9a44, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8f9a4c, SUC_lexicon_tags_1 },
    { 0x91ef9a4d, SUC_lexicon_tags_17 },
    { 0x56ef9a4e, SUC_lexicon_tags_111 },
    { 0x4aaf9a4f, SUC_lexicon_tags_83 },
    { 0x236f9a4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x106f9a53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cf9a56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8f9a5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d0f9a5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x624f9a61, SUC_lexicon_tags_10 },
    { 0xac2f9a62, SUC_lexicon_tags_15 },
    { 0xbd8f9a63, SUC_lexicon_tags_11 },
    { 0x660f9a63, SUC_lexicon_tags_11 },
    { 0x4c6f9a65, SUC_lexicon_tags_25 },
    { 0xf32f9a66, SUC_lexicon_tags_13 },
    { 0x564f9a66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10f9a6b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33af9a70, SUC_lexicon_tags_25 },
    { 0x358f9a71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6af9a73, SUC_lexicon_tags_5 },
    { 0x844f9a74, SUC_lexicon_tags_17 },
    { 0xd50f9a74, SUC_lexicon_tags_15 },
    { 0xe04f9a76, SUC_lexicon_tags_15 },
    { 0x31af9a77, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ef9a7a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x692f9a7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46f9a81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8f9a84, SUC_lexicon_tags_12 },
    { 0x3caf9a84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06f9a88, SUC_lexicon_tags_27 },
    { 0x866f9a88, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946f9a90, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498f9a94, SUC_lexicon_tags_42 },
    { 0xe30f9a95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x946f9a99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cf9a9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbaf9a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5caf9aa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa46f9aa6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ef9aa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x380f9aab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202f9aaf, SUC_lexicon_tags_49 },
    { 0x2c0f9ab0, SUC_lexicon_tags_12 },
    { 0x366f9ab1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50cf9ab3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x47cf9ab6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4f9ab8, SUC_lexicon_tags_15 },
    { 0xa5ef9ab9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ccf9abb, SUC_lexicon_tags_25 },
    { 0xe36f9abc, SUC_lexicon_tags_3 },
    { 0xc00f9abd, SUC_lexicon_tags_17 },
    { 0xccaf9abc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8f9ac2, SUC_lexicon_tags_11 },
    { 0x154f9ac3, SUC_lexicon_tags_23 },
    { 0x9e2f9ac4, SUC_lexicon_tags_12 },
    { 0x78cf9ac5, SUC_lexicon_tags_19 },
    { 0xd44f9ac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0f9ac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ef9acc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8bef9ace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd20f9ad1, SUC_lexicon_tags_16 },
    { 0x5a4f9ad2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cf9ad6, SUC_lexicon_tags_1 },
    { 0x7e8f9ad7, SUC_lexicon_tags_36 },
    { 0x648f9ad8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa5cf9ada, SUC_lexicon_tags_14 },
    { 0x7c6f9adb, SUC_lexicon_tags_19 },
    { 0xef9adc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ef9adf, SUC_lexicon_tags_28 },
    { 0x59af9ae0, SUC_lexicon_tags_17 },
    { 0x538f9ae1, SUC_lexicon_tags_12 },
    { 0xbfcf9ae0, SUC_lexicon_tags_30 },
    { 0xcb8f9ae3, SUC_lexicon_tags_45 },
    { 0xba6f9ae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a0f9ae6, SUC_lexicon_tags_60 },
    { 0x640f9ae6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ef9aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbaf9aed, SUC_lexicon_tags_46 },
    { 0x6f4f9aee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbecf9af0, SUC_lexicon_tags_66 },
    { 0x556f9af1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9af9af4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca2f9af8, SUC_lexicon_tags_22 },
    { 0x7a6f9af8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20cf9aff, SUC_lexicon_tags_11 },
    { 0xaa6f9b00, SUC_lexicon_tags_30 },
    { 0xe92f9b01, SUC_lexicon_tags_63 },
    { 0xf78f9b02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96f9b04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadef9b09, SUC_lexicon_tags_24 },
    { 0x5ef9b0a, SUC_lexicon_tags_38 },
    { 0xa76f9b0b, SUC_lexicon_tags_1 },
    { 0x148f9b0c, SUC_lexicon_tags_37 },
    { 0x69cf9b0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe0f9b0f, SUC_lexicon_tags_65 },
    { 0xbbcf9b10, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cf9b14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcccf9b16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6af9b19, SUC_lexicon_tags_61 },
    { 0x40ef9b19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x84ef9b1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578f9b23, SUC_lexicon_tags_35 },
    { 0x1cf9b23, SUC_lexicon_tags_38 },
    { 0xf4ef9b25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b28, SUC_lexicon_tags_15 },
    { 0x752f9b28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2cf9b2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x850f9b2e, SUC_lexicon_tags_31 },
    { 0x400f9b2f, SUC_lexicon_tags_25 },
    { 0x3cf9b2f, SUC_lexicon_tags_12 },
    { 0x5c4f9b30, SUC_lexicon_tags_33 },
    { 0x2b6f9b30, SUC_lexicon_tags_45 },
    { 0x5d4f9b32, SUC_lexicon_tags_14 },
    { 0x77cf9b2f, SUC_lexicon_tags_26 },
    { 0x2f0f9b35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8bcf9b37, SUC_lexicon_tags_11 },
    { 0x882f9b38, SUC_lexicon_tags_14 },
    { 0xcaf9b39, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6af9b3d, SUC_lexicon_tags_17 },
    { 0x4f4f9b3e, SUC_lexicon_tags_30 },
    { 0x58ef9b3e, SUC_lexicon_tags_25 },
    { 0x4f8f9b3f, SUC_lexicon_tags_15 },
    { 0x82f9b3d, SUC_lexicon_tags_17 },
    { 0xd6ef9b42, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4f9b46, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0xe1ef9b48, SUC_lexicon_tags_45 },
    { 0xd1ef9b48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0f9b4b, SUC_lexicon_tags_318 },
    { 0xed4f9b4c, SUC_lexicon_tags_19 },
    { 0xed0f9b4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x93cf9b4f, SUC_lexicon_tags_25 },
    { 0x10ef9b4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2f9b55, SUC_lexicon_tags_25 },
    { 0x80cf9b56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf9b59, SUC_lexicon_tags_31 },
    { 0xc12f9b5a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874f9b5e, SUC_lexicon_tags_12 },
    { 0x92af9b5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70f9b65, SUC_lexicon_tags_15 },
    { 0x798f9b66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb52f9b68, SUC_lexicon_tags_27 },
    { 0xd0cf9b69, SUC_lexicon_tags_1 },
    { 0xea4f9b69, SUC_lexicon_tags_33 },
    { 0x38ef9b68, SUC_lexicon_tags_17 },
    { 0xcc8f9b69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x142f9b6e, SUC_lexicon_tags_25 },
    { 0xf30f9b6e, SUC_lexicon_tags_14 },
    { 0x6ef9b70, SUC_lexicon_tags_14 },
    { 0x6ecf9b71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98f9b74, SUC_lexicon_tags_11 },
    { 0x406f9b75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6f9b78, SUC_lexicon_tags_1 },
    { 0xee8f9b79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x59af9b7c, SUC_lexicon_tags_2 },
    { 0xcc4f9b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4af9b81, SUC_lexicon_tags_15 },
    { 0x902f9b82, SUC_lexicon_tags_11 },
    { 0x310f9b83, SUC_lexicon_tags_25 },
    { 0x2e0f9b84, SUC_lexicon_tags_15 },
    { 0x1ecf9b85, SUC_lexicon_tags_2 },
    { 0x472f9b86, SUC_lexicon_tags_45 },
    { 0x8acf9b87, SUC_lexicon_tags_1 },
    { 0xe92f9b81, SUC_lexicon_tags_28 },
    { 0x4f2f9b89, SUC_lexicon_tags_28 },
    { 0x94af9b88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f9b8d, SUC_lexicon_tags_21 },
    { 0x4b0f9b8d, SUC_lexicon_tags_14 },
    { 0xaa2f9b8f, SUC_lexicon_tags_33 },
    { 0xd86f9b8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1cf9b92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x732f9b94, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7a6f9b96, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xde4f9b98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a2f9b9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4f9b9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x152f9ba1, SUC_lexicon_tags_37 },
    { 0xa8ef9ba1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc86f9ba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4af9ba6, SUC_lexicon_tags_0 },
    { 0x848f9ba7, SUC_lexicon_tags_25 },
    { 0x578f9ba8, SUC_lexicon_tags_27 },
    { 0x4f9ba7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4f9bae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0f9bb1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6c6f9bb3, SUC_lexicon_tags_27 },
    { 0xc6cf9bb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe78f9bb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ef9bbc, SUC_lexicon_tags_48 },
    { 0xcfaf9bbd, SUC_lexicon_tags_45 },
    { 0xc0ef9bbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81af9bc3, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0f9bc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x800f9bca, SUC_lexicon_tags_77 },
    { 0x980f9bcb, SUC_lexicon_tags_37 },
    { 0x1f6f9bca, SUC_lexicon_tags_23 },
    { 0x5c6f9bcd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x64cf9bcf, SUC_lexicon_tags_10 },
    { 0x254f9bd0, SUC_lexicon_tags_55 },
    { 0x1a0f9bd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacf9bd7, SUC_lexicon_tags_27 },
    { 0xb90f9bd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae6f9bdd, SUC_lexicon_tags_37 },
    { 0x6a8f9bde, SUC_lexicon_tags_13 },
    { 0x380f9bdd, SUC_lexicon_tags_1 },
    { 0xc1af9bde, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2f9be4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42af9be6, SUC_lexicon_tags_18 },
    { 0x844f9be7, SUC_lexicon_tags_31 },
    { 0x314f9be7, SUC_lexicon_tags_14 },
    { 0x37cf9be9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318f9bf2, SUC_lexicon_tags_27 },
    { 0xda6f9bf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0f9bf5, SUC_lexicon_tags_15 },
    { 0xdf4f9bf6, SUC_lexicon_tags_24 },
    { 0x5c8f9bf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cf9bfc, SUC_lexicon_tags_3 },
    { 0x444f9bfd, SUC_lexicon_tags_37 },
    { 0xec0f9bfe, SUC_lexicon_tags_49 },
    { 0xc54f9bff, SUC_lexicon_tags_45 },
    { 0x1dcf9c00, SUC_lexicon_tags_23 },
    { 0x6f2f9bff, SUC_lexicon_tags_0 },
    { 0xc4f9c01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4f9c0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25af9c18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48f9c1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x230f9c1d, SUC_lexicon_tags_11 },
    { 0xf7cf9c1d, SUC_lexicon_tags_49 },
    { 0x480f9c1f, SUC_lexicon_tags_31 },
    { 0x6aef9c1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4f9c28, SUC_lexicon_tags_28 },
    { 0xb4f9c29, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4c0f9c2b, SUC_lexicon_tags_20 },
    { 0xbe4f9c2c, SUC_lexicon_tags_23 },
    { 0xa46f9c2d, SUC_lexicon_tags_63 },
    { 0xfc4f9c2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59cf9c30, SUC_lexicon_tags_1 },
    { 0xe76f9c30, SUC_lexicon_tags_11 },
    { 0xdb2f9c32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x38ef9c34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994f9c39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884f9c3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fcf9c40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2f9c44, SUC_lexicon_tags_12 },
    { 0x290f9c45, SUC_lexicon_tags_0 },
    { 0x6acf9c46, SUC_lexicon_tags_37 },
    { 0xbfef9c45, SUC_lexicon_tags_23 },
    { 0x26ef9c48, SUC_lexicon_tags_45 },
    { 0xe58f9c49, SUC_lexicon_tags_11 },
    { 0xdc4f9c47, SUC_lexicon_tags_15 },
    { 0xa2f9c4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x964f9c4d, SUC_lexicon_tags_17 },
    { 0x842f9c4e, SUC_lexicon_tags_43 },
    { 0xdd6f9c4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68cf9c55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x264f9c57, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xca0f9c59, SUC_lexicon_tags_12 },
    { 0xaf8f9c5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc6f9c5c, SUC_lexicon_tags_37 },
    { 0x5a4f9c5d, SUC_lexicon_tags_3 },
    { 0xdc2f9c5e, SUC_lexicon_tags_82 },
    { 0x96af9c5f, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2f9c63, SUC_lexicon_tags_27 },
    { 0xb7ef9c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x224f9c67, SUC_lexicon_tags_34 },
    { 0x59ef9c67, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ef9c6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbaf9c70, SUC_lexicon_tags_49 },
    { 0xf2ef9c70, SUC_lexicon_tags_35 },
    { 0x980f9c71, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb02f9c76, SUC_lexicon_tags_2 },
    { 0x28af9c77, SUC_lexicon_tags_47 },
    { 0x3c4f9c77, SUC_lexicon_tags_15 },
    { 0x4e2f9c77, SUC_lexicon_tags_1 },
    { 0x692f9c7a, SUC_lexicon_tags_38 },
    { 0xe16f9c78, SUC_lexicon_tags_11 },
    { 0x7e0f9c77, SUC_lexicon_tags_47 },
    { 0x2caf9c7d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd42f9c80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16cf9c82, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70f9c86, SUC_lexicon_tags_7 },
    { 0x538f9c87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaef9c8c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x59af9c8f, SUC_lexicon_tags_10 },
    { 0xdbaf9c90, SUC_lexicon_tags_39 },
    { 0x2cf9c91, SUC_lexicon_tags_37 },
    { 0xaaaf9c92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x310f9c94, SUC_lexicon_tags_28 },
    { 0x4bcf9c94, SUC_lexicon_tags_30 },
    { 0x3c6f9c94, SUC_lexicon_tags_77 },
    { 0x52cf9c97, SUC_lexicon_tags_25 },
    { 0xbf4f9c98, SUC_lexicon_tags_26 },
    { 0xd78f9c98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf8f9c9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x110f9c9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8f9ca1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe18f9ca3, SUC_lexicon_tags_1 },
    { 0x802f9ca4, SUC_lexicon_tags_8 },
    { 0xd42f9ca4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65ef9ca7, SUC_lexicon_tags_37 },
    { 0x30af9ca7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76f9cac, SUC_lexicon_tags_6 },
    { 0x7c4f9cac, SUC_lexicon_tags_2 },
    { 0x90cf9cae, SUC_lexicon_tags_19 },
    { 0x94f9cae, SUC_lexicon_tags_19 },
    { 0xa18f9cb0, SUC_lexicon_tags_60 },
    { 0x730f9cb0, SUC_lexicon_tags_6 },
    { 0x81ef9cb2, SUC_lexicon_tags_31 },
    { 0x634f9cb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd10f9cb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2f9cb8, SUC_lexicon_tags_19 },
    { 0x956f9cb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef9cbd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xadef9cc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4f9cc8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6f9cce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc96f9cd0, SUC_lexicon_tags_8 },
    { 0x1a6f9cd1, SUC_lexicon_tags_37 },
    { 0xecf9cd2, SUC_lexicon_tags_37 },
    { 0x82ef9cd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8f9cd6, SUC_lexicon_tags_7 },
    { 0x5f4f9cd7, SUC_lexicon_tags_7 },
    { 0x802f9cd8, SUC_lexicon_tags_45 },
    { 0x61cf9cd8, SUC_lexicon_tags_65 },
    { 0x810f9cda, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f9cdd, SUC_lexicon_tags_1 },
    { 0x674f9cde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17cf9ce0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x520f9ce4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cf9ce7, SUC_lexicon_tags_2 },
    { 0x220f9ce8, SUC_lexicon_tags_25 },
    { 0x4a0f9ce9, SUC_lexicon_tags_11 },
    { 0x70ef9cea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x40af9cec, SUC_lexicon_tags_34 },
    { 0xa80f9ced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd2f9cef, SUC_lexicon_tags_42 },
    { 0x522f9cf0, SUC_lexicon_tags_30 },
    { 0xd2af9cf1, SUC_lexicon_tags_37 },
    { 0x96cf9cf2, SUC_lexicon_tags_26 },
    { 0xe96f9cf1, SUC_lexicon_tags_9 },
    { 0x44f9cef, SUC_lexicon_tags_229 },
    { 0xee8f9cf5, SUC_lexicon_tags_33 },
    { 0x132f9cf6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36f9cfb, SUC_lexicon_tags_25 },
    { 0xbbef9cfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cf9d06, SUC_lexicon_tags_11 },
    { 0xb38f9d07, SUC_lexicon_tags_15 },
    { 0x800f9d08, SUC_lexicon_tags_20 },
    { 0x4c8f9d08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48f9d0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8ef9d11, SUC_lexicon_tags_170 },
    { 0x26cf9d12, SUC_lexicon_tags_253 },
    { 0xb22f9d12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ef9d1d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ecf9d1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8f9d23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f9d26, SUC_lexicon_tags_37 },
    { 0x7f6f9d26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcdcf9d29, SUC_lexicon_tags_23 },
    { 0x8a6f9d2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736f9d2e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x960f9d30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eef9d33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaf9d38, SUC_lexicon_tags_7 },
    { 0xf6f9d39, SUC_lexicon_tags_12 },
    { 0x35ef9d3a, SUC_lexicon_tags_21 },
    { 0x33ef9d3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd38f9d3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc64f9d3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70af9d41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94f9d44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x272f9d46, SUC_lexicon_tags_47 },
    { 0xe00f9d46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08f9d4a, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0xf2cf9d4c, SUC_lexicon_tags_37 },
    { 0xd78f9d4d, SUC_lexicon_tags_7 },
    { 0x55cf9d4e, SUC_lexicon_tags_12 },
    { 0xbe8f9d4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x870f9d56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2f9d5b, SUC_lexicon_tags_9 },
    { 0x628f9d5b, SUC_lexicon_tags_48 },
    { 0x4f4f9d5d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x334f9d5f, SUC_lexicon_tags_25 },
    { 0xd64f9d60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x110f9d62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc84f9d64, SUC_lexicon_tags_2 },
    { 0xcc4f9d64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcaf9d69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x372f9d6b, SUC_lexicon_tags_11 },
    { 0x706f9d6b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7a0f9d6e, SUC_lexicon_tags_0 },
    { 0x516f9d6e, SUC_lexicon_tags_47 },
    { 0x10af9d6f, SUC_lexicon_tags_63 },
    { 0x3fcf9d71, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfe2f9d73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaa0f9d75, SUC_lexicon_tags_15 },
    { 0x158f9d76, SUC_lexicon_tags_1 },
    { 0x9ecf9d77, SUC_lexicon_tags_4 },
    { 0xb98f9d78, SUC_lexicon_tags_27 },
    { 0x6a0f9d77, SUC_lexicon_tags_13 },
    { 0x48cf9d75, SUC_lexicon_tags_17 },
    { 0x212f9d7b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8caf9d80, SUC_lexicon_tags_11 },
    { 0x2fef9d80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86f9d86, SUC_lexicon_tags_4 },
    { 0x660f9d87, SUC_lexicon_tags_63 },
    { 0x8b6f9d87, SUC_lexicon_tags_0 },
    { 0xedaf9d87, SUC_lexicon_tags_3 },
    { 0xac6f9d86, SUC_lexicon_tags_66 },
    { 0xa24f9d89, SUC_lexicon_tags_87 },
    { 0xbb2f9d8c, SUC_lexicon_tags_12 },
    { 0xf08f9d8c, SUC_lexicon_tags_28 },
    { 0x9eef9d8c, SUC_lexicon_tags_7 },
    { 0x526f9d8c, SUC_lexicon_tags_14 },
    { 0x15af9d8d, SUC_lexicon_tags_11 },
    { 0x8f9d89, SUC_lexicon_tags_17 },
    { 0x166f9d92, SUC_lexicon_tags_38 },
    { 0xda6f9d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfef9d97, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcf4f9d99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79cf9d9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50af9d9e, SUC_lexicon_tags_1 },
    { 0x172f9d9f, SUC_lexicon_tags_231 },
    { 0x626f9d9e, SUC_lexicon_tags_12 },
    { 0x112f9da1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x802f9da8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x786f9daa, SUC_lexicon_tags_18 },
    { 0x4e6f9dab, SUC_lexicon_tags_60 },
    { 0x2f0f9dac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0f9daf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee0f9db6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6f9db9, SUC_lexicon_tags_12 },
    { 0x2faf9dba, SUC_lexicon_tags_141 },
    { 0xd78f9dba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x776f9dbe, SUC_lexicon_tags_1 },
    { 0x920f9dbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8f9dc6, SUC_lexicon_tags_38 },
    { 0x262f9dc7, SUC_lexicon_tags_8 },
    { 0xca0f9dc8, SUC_lexicon_tags_17 },
    { 0xe24f9dc9, SUC_lexicon_tags_63 },
    { 0xd92f9dca, SUC_lexicon_tags_15 },
    { 0xf14f9dca, SUC_lexicon_tags_8 },
    { 0x140f9dcc, SUC_lexicon_tags_6 },
    { 0xf58f9dcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x852f9dcf, SUC_lexicon_tags_25 },
    { 0xc9ef9dd0, SUC_lexicon_tags_17 },
    { 0x23af9dd1, SUC_lexicon_tags_6 },
    { 0x26cf9dd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0f9dd8, SUC_lexicon_tags_29 },
    { 0xb28f9dd9, SUC_lexicon_tags_11 },
    { 0x1e6f9dda, SUC_lexicon_tags_0 },
    { 0x1b4f9dd9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ef9ddf, SUC_lexicon_tags_87 },
    { 0xfe8f9de0, SUC_lexicon_tags_33 },
    { 0x942f9ddf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x528f9de5, SUC_lexicon_tags_25 },
    { 0x196f9de6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6f9deb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28f9dee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef9df1, SUC_lexicon_tags_1 },
    { 0x238f9df2, SUC_lexicon_tags_14 },
    { 0x43af9df3, SUC_lexicon_tags_11 },
    { 0x23ef9df4, SUC_lexicon_tags_60 },
    { 0xb20f9df4, SUC_lexicon_tags_43 },
    { 0xfb6f9df6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35cf9df8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1af9dff, SUC_lexicon_tags_17 },
    { 0x4cf9e00, SUC_lexicon_tags_11 },
    { 0x442f9e01, SUC_lexicon_tags_37 },
    { 0x832f9dff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3dcf9e04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cf9e0a, SUC_lexicon_tags_7 },
    { 0x7e8f9e0b, SUC_lexicon_tags_2 },
    { 0x196f9e0c, SUC_lexicon_tags_31 },
    { 0x5e0f9e0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe86f9e10, SUC_lexicon_tags_4 },
    { 0xec8f9e11, SUC_lexicon_tags_7 },
    { 0x41af9e11, SUC_lexicon_tags_30 },
    { 0x7ecf9e13, SUC_lexicon_tags_11 },
    { 0xf92f9e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f9e1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2f9e1d, SUC_lexicon_tags_25 },
    { 0xf9cf9e1e, SUC_lexicon_tags_11 },
    { 0xa8f9e1f, SUC_lexicon_tags_28 },
    { 0x280f9e1f, SUC_lexicon_tags_15 },
    { 0x9c2f9e21, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x128f9e24, SUC_lexicon_tags_14 },
    { 0x754f9e25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee2f9e28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ccf9e2a, SUC_lexicon_tags_11 },
    { 0xbe0f9e2b, SUC_lexicon_tags_37 },
    { 0x822f9e2c, SUC_lexicon_tags_1 },
    { 0x176f9e2c, SUC_lexicon_tags_6 },
    { 0x680f9e2e, SUC_lexicon_tags_19 },
    { 0xd2af9e2e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cf9e34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x546f9e37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x914f9e39, SUC_lexicon_tags_60 },
    { 0xc2ef9e39, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x252f9e3c, SUC_lexicon_tags_7 },
    { 0xc16f9e3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc28f9e41, SUC_lexicon_tags_26 },
    { 0x89af9e41, SUC_lexicon_tags_65 },
    { 0x4eaf9e43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ef9e48, SUC_lexicon_tags_25 },
    { 0xdd8f9e49, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ef9e4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x712f9e4f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x83cf9e52, SUC_lexicon_tags_16 },
    { 0xa38f9e53, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cf9e58, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4f9e5b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x178f9e5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54f9e63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04f9e68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4f9e6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf76f9e72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4f9e76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9af9e7a, SUC_lexicon_tags_2 },
    { 0x1c4f9e7b, SUC_lexicon_tags_68 },
    { 0xc4af9e7c, SUC_lexicon_tags_38 },
    { 0xe9af9e7b, SUC_lexicon_tags_37 },
    { 0xd9ef9e7e, SUC_lexicon_tags_108 },
    { 0xa74f9e7c, SUC_lexicon_tags_1647 },
    { 0x8aef9e7d, SUC_lexicon_tags_27 },
    { 0x818f9e81, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x224f9e84, SUC_lexicon_tags_17 },
    { 0x1eef9e84, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24f9e8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ef9e91, SUC_lexicon_tags_11 },
    { 0xc64f9e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8f9e97, SUC_lexicon_tags_20 },
    { 0x81ef9e97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2f9e9b, SUC_lexicon_tags_19 },
    { 0x888f9e9c, SUC_lexicon_tags_1 },
    { 0xd94f9e9d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x172f9e9f, SUC_lexicon_tags_33 },
    { 0x5aef9e9f, SUC_lexicon_tags_0 },
    { 0x2bef9ea1, SUC_lexicon_tags_11 },
    { 0x7b2f9ea2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cf9ea5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x938f9ea8, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840f9eac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40af9eae, SUC_lexicon_tags_27 },
    { 0x6e6f9eaf, SUC_lexicon_tags_7 },
    { 0x5daf9eb0, SUC_lexicon_tags_7 },
    { 0x41cf9eb0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24f9eb6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4f9eba, SUC_lexicon_tags_15 },
    { 0xdd0f9ebb, SUC_lexicon_tags_29 },
    { 0xee2f9ebb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0cf9ebe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6f9ec4, SUC_lexicon_tags_5 },
    { 0xeb4f9ec5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9acf9ec7, SUC_lexicon_tags_11 },
    { 0xd60f9ec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x586f9eca, SUC_lexicon_tags_17 },
    { 0xaa8f9ecb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x59af9ecd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x186f9ed0, SUC_lexicon_tags_15 },
    { 0x720f9ed1, SUC_lexicon_tags_8 },
    { 0x8d8f9ed2, SUC_lexicon_tags_17 },
    { 0x88cf9ed0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ef9ed7, SUC_lexicon_tags_47 },
    { 0x972f9ed8, SUC_lexicon_tags_15 },
    { 0x2f8f9ed8, SUC_lexicon_tags_15 },
    { 0xdc6f9eda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe8f9edc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x130f9ede, SUC_lexicon_tags_47 },
    { 0xc04f9ede, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x324f9ee1, SUC_lexicon_tags_2 },
    { 0x5caf9ee2, SUC_lexicon_tags_6 },
    { 0x480f9ee3, SUC_lexicon_tags_11 },
    { 0x5a8f9ee4, SUC_lexicon_tags_49 },
    { 0x6f2f9ee4, SUC_lexicon_tags_12 },
    { 0xd96f9ee6, SUC_lexicon_tags_11 },
    { 0xa6ef9ee6, SUC_lexicon_tags_47 },
    { 0xbc2f9ee4, SUC_lexicon_tags_47 },
    { 0xe54f9ee7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xba2f9eeb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76f9ef0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2f9ef5, SUC_lexicon_tags_11 },
    { 0xee2f9ef6, SUC_lexicon_tags_14 },
    { 0x164f9ef6, SUC_lexicon_tags_28 },
    { 0xefaf9ef7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10af9efe, SUC_lexicon_tags_17 },
    { 0xceaf9eff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0f9f02, SUC_lexicon_tags_25 },
    { 0xa7cf9f03, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x512f9f06, SUC_lexicon_tags_25 },
    { 0xe58f9f07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcf9f0b, SUC_lexicon_tags_7 },
    { 0x684f9f0c, SUC_lexicon_tags_15 },
    { 0xd98f9f0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc74f9f0f, SUC_lexicon_tags_29 },
    { 0xce4f9f10, SUC_lexicon_tags_25 },
    { 0x256f9f0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0f9f18, SUC_lexicon_tags_19 },
    { 0x18ef9f18, SUC_lexicon_tags_37 },
    { 0x3bef9f1a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb2cf9f1c, SUC_lexicon_tags_2 },
    { 0xefcf9f1c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96f9f22, SUC_lexicon_tags_14 },
    { 0xf76f9f23, SUC_lexicon_tags_2 },
    { 0x2e8f9f23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ef9f29, SUC_lexicon_tags_23 },
    { 0x44ef9f2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x21af9f2c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ef9f33, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xed8f9f35, SUC_lexicon_tags_48 },
    { 0x82ef9f36, SUC_lexicon_tags_43 },
    { 0x6def9f37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a2f9f3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82cf9f42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4f9f45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ccf9f47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5acf9f4a, SUC_lexicon_tags_23 },
    { 0x4faf9f4b, SUC_lexicon_tags_224 },
    { 0x33af9f4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ef9f58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ef9f5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ef9f61, SUC_lexicon_tags_37 },
    { 0xe3cf9f62, SUC_lexicon_tags_62 },
    { 0x5a6f9f63, SUC_lexicon_tags_11 },
    { 0x4b8f9f64, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb76f9f67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae4f9f69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f9f6f, SUC_lexicon_tags_56 },
    { 0xb1ef9f70, SUC_lexicon_tags_38 },
    { 0x98cf9f71, SUC_lexicon_tags_63 },
    { 0xaf8f9f72, SUC_lexicon_tags_15 },
    { 0xdbef9f73, SUC_lexicon_tags_27 },
    { 0x47af9f73, SUC_lexicon_tags_25 },
    { 0x3c4f9f75, SUC_lexicon_tags_11 },
    { 0x5e6f9f73, SUC_lexicon_tags_12 },
    { 0xf74f9f77, SUC_lexicon_tags_48 },
    { 0x48cf9f77, SUC_lexicon_tags_12 },
    { 0x136f9f77, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3baf9f7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10cf9f81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5af9f84, SUC_lexicon_tags_7 },
    { 0xe1ef9f84, SUC_lexicon_tags_79 },
    { 0xe4cf9f86, SUC_lexicon_tags_4 },
    { 0x112f9f85, SUC_lexicon_tags_25 },
    { 0x886f9f88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90af9f8b, SUC_lexicon_tags_15 },
    { 0x492f9f8c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d8f9f8e, SUC_lexicon_tags_23 },
    { 0xc30f9f8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5af9f91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcf2f9f93, SUC_lexicon_tags_26 },
    { 0xb20f9f94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaaf9f97, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x874f9f99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45ef9f9b, SUC_lexicon_tags_4 },
    { 0x6aef9f9b, SUC_lexicon_tags_15 },
    { 0xa2f9f9d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x44f9f9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70f9fa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91ef9fa4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80f9fa9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa4f9faf, SUC_lexicon_tags_19 },
    { 0x834f9fb0, SUC_lexicon_tags_29 },
    { 0x27cf9fb0, SUC_lexicon_tags_7 },
    { 0xed2f9fb1, SUC_lexicon_tags_43 },
    { 0xd4ef9fb3, SUC_lexicon_tags_25 },
    { 0xe7cf9fb4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xde4f9fb6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc5ef9fb8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968f9fbc, SUC_lexicon_tags_1 },
    { 0x24cf9fbd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xceaf9fbf, SUC_lexicon_tags_55 },
    { 0x8caf9fc0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x546f9fc2, SUC_lexicon_tags_43 },
    { 0x5cef9fc3, SUC_lexicon_tags_0 },
    { 0xfe4f9fc3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdf8f9fc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x774f9fc8, SUC_lexicon_tags_49 },
    { 0x7d4f9fc9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x980f9fcb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48f9fd0, SUC_lexicon_tags_17 },
    { 0xc56f9fd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f9fd6, SUC_lexicon_tags_45 },
    { 0x28af9fd7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4f9fda, SUC_lexicon_tags_29 },
    { 0x3cef9fdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x264f9fdf, SUC_lexicon_tags_12 },
    { 0x73cf9fe0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x850f9fe2, SUC_lexicon_tags_2 },
    { 0x89ef9fe3, SUC_lexicon_tags_8 },
    { 0xe8f9fe4, SUC_lexicon_tags_27 },
    { 0xc7af9fe5, SUC_lexicon_tags_34 },
    { 0x578f9fe5, SUC_lexicon_tags_37 },
    { 0xd46f9fe3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e2f9fe9, SUC_lexicon_tags_11 },
    { 0xf90f9fea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x798f9fed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ecf9fef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb40f9ff2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd38f9ff4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ef9ff8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x92ef9ffa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fcf9ffc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00fa002, SUC_lexicon_tags_60 },
    { 0x152fa003, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6afa007, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43afa00a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd70fa00c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x44cfa00f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x498fa016, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x334fa018, SUC_lexicon_tags_0 },
    { 0x8bafa019, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac0fa01e, SUC_lexicon_tags_141 },
    { 0x70efa01e, SUC_lexicon_tags_11 },
    { 0xe1afa020, SUC_lexicon_tags_45 },
    { 0x13efa021, SUC_lexicon_tags_3 },
    { 0xe20fa022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda0fa024, SUC_lexicon_tags_33 },
    { 0x584fa025, SUC_lexicon_tags_82 },
    { 0x3e4fa025, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676fa02b, SUC_lexicon_tags_12 },
    { 0x7a2fa02c, SUC_lexicon_tags_11 },
    { 0xceafa02c, SUC_lexicon_tags_7 },
    { 0x632fa02c, SUC_lexicon_tags_30 },
    { 0x746fa02f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efa033, SUC_lexicon_tags_8 },
    { 0x844fa033, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9dcfa036, SUC_lexicon_tags_33 },
    { 0x16efa036, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0fa03c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2efa03f, SUC_lexicon_tags_7 },
    { 0x92efa040, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x338fa043, SUC_lexicon_tags_17 },
    { 0x736fa044, SUC_lexicon_tags_25 },
    { 0x434fa045, SUC_lexicon_tags_7 },
    { 0x5fcfa045, SUC_lexicon_tags_27 },
    { 0x116fa043, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x16efa049, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1befa04f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf2fa051, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x54afa053, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7cfa055, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa92fa057, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8fa05a, SUC_lexicon_tags_69 },
    { 0xf86fa05b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe70fa05d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcefa062, SUC_lexicon_tags_25 },
    { 0xc90fa062, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab4fa065, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x540fa068, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0fa06c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342fa074, SUC_lexicon_tags_23 },
    { 0x434fa075, SUC_lexicon_tags_11 },
    { 0x786fa074, SUC_lexicon_tags_11 },
    { 0xe68fa074, SUC_lexicon_tags_1 },
    { 0x42cfa074, SUC_lexicon_tags_17 },
    { 0x332fa076, SUC_lexicon_tags_17 },
    { 0x302fa07a, SUC_lexicon_tags_1 },
    { 0x90efa07b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaafa07e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1afa081, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf8fa083, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x934fa085, SUC_lexicon_tags_15 },
    { 0xa48fa085, SUC_lexicon_tags_1 },
    { 0x182fa085, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba4fa089, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdacfa08b, SUC_lexicon_tags_49 },
    { 0xc4efa08b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x726fa08e, SUC_lexicon_tags_0 },
    { 0x9befa08f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x344fa091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaafa09b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58fa0a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fafa0a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x574fa0a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7defa0aa, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x698fa0b1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cfa0b7, SUC_lexicon_tags_91 },
    { 0xd4afa0b7, SUC_lexicon_tags_33 },
    { 0xe34fa0b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf64fa0bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3efa0c2, SUC_lexicon_tags_37 },
    { 0xad8fa0c3, SUC_lexicon_tags_45 },
    { 0xc62fa0c4, SUC_lexicon_tags_12 },
    { 0x360fa0c5, SUC_lexicon_tags_45 },
    { 0x228fa0c6, SUC_lexicon_tags_37 },
    { 0xf0fa0c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54fa0cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48fa0d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x494fa0d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2cfa0d8, SUC_lexicon_tags_25 },
    { 0x626fa0d9, SUC_lexicon_tags_56 },
    { 0x804fa0d9, SUC_lexicon_tags_17 },
    { 0xd9efa0da, SUC_lexicon_tags_14 },
    { 0xe88fa0dc, SUC_lexicon_tags_2 },
    { 0x966fa0dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0fa0e0, SUC_lexicon_tags_17 },
    { 0x42afa0e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc52fa0e4, SUC_lexicon_tags_1 },
    { 0x87cfa0e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb22fa0e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b4fa0ea, SUC_lexicon_tags_25 },
    { 0x33afa0ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5efa0ed, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbd4fa0ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x52cfa0f2, SUC_lexicon_tags_2 },
    { 0x37cfa0f3, SUC_lexicon_tags_96 },
    { 0x15efa0f4, SUC_lexicon_tags_30 },
    { 0xbeefa0f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4fa0fb, SUC_lexicon_tags_25 },
    { 0xf94fa0fc, SUC_lexicon_tags_21 },
    { 0x6a6fa0fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4fa101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620fa105, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64fa10d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc08fa10f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x126fa114, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb26fa116, SUC_lexicon_tags_11 },
    { 0xba8fa117, SUC_lexicon_tags_37 },
    { 0x652fa118, SUC_lexicon_tags_15 },
    { 0xd4cfa118, SUC_lexicon_tags_108 },
    { 0xabefa118, SUC_lexicon_tags_37 },
    { 0xffefa11b, SUC_lexicon_tags_26 },
    { 0xaeefa11c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48fa122, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fa126, SUC_lexicon_tags_15 },
    { 0x398fa126, SUC_lexicon_tags_1 },
    { 0x518fa128, SUC_lexicon_tags_7 },
    { 0xe72fa126, SUC_lexicon_tags_27 },
    { 0xd4fa128, SUC_lexicon_tags_11 },
    { 0xaf4fa129, SUC_lexicon_tags_45 },
    { 0xa7afa12b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660fa130, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccfa138, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4fa13b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0fa13e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cfa141, SUC_lexicon_tags_48 },
    { 0x25cfa142, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe92fa144, SUC_lexicon_tags_47 },
    { 0x594fa144, SUC_lexicon_tags_12 },
    { 0xb1afa145, SUC_lexicon_tags_19 },
    { 0xcf8fa145, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x270fa14a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x536fa14d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60afa14f, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x6eefa151, SUC_lexicon_tags_43 },
    { 0x5f0fa152, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8fa157, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4fa15a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ecfa15d, SUC_lexicon_tags_137 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158fa161, SUC_lexicon_tags_43 },
    { 0xbcefa162, SUC_lexicon_tags_37 },
    { 0xa10fa163, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa90fa165, SUC_lexicon_tags_2 },
    { 0xf92fa165, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x60fa168, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff8fa16a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc30fa16c, SUC_lexicon_tags_15 },
    { 0xa56fa16c, SUC_lexicon_tags_26 },
    { 0xe8cfa16d, SUC_lexicon_tags_148 },
    { 0x56afa16e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbdcfa171, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fa174, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x958fa176, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cfa17f, SUC_lexicon_tags_17 },
    { 0x452fa180, SUC_lexicon_tags_7 },
    { 0x1d4fa181, SUC_lexicon_tags_15 },
    { 0xf2fa17f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x418fa184, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786fa18a, SUC_lexicon_tags_32 },
    { 0x5befa18a, SUC_lexicon_tags_25 },
    { 0xab4fa18c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecfa191, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6afa194, SUC_lexicon_tags_23 },
    { 0xb78fa195, SUC_lexicon_tags_29 },
    { 0x45efa195, SUC_lexicon_tags_65 },
    { 0xb1cfa195, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b8fa199, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30cfa19c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x248fa19e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fa1a3, SUC_lexicon_tags_38 },
    { 0xe70fa1a3, SUC_lexicon_tags_17 },
    { 0x64fa1a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xceafa1a8, SUC_lexicon_tags_49 },
    { 0xdaafa1a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaafa1ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4fa1af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54fa1b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6fa1b5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afa1b8, SUC_lexicon_tags_37 },
    { 0x154fa1b9, SUC_lexicon_tags_25 },
    { 0x1a0fa1ba, SUC_lexicon_tags_47 },
    { 0xa68fa1b9, SUC_lexicon_tags_48 },
    { 0xa34fa1bc, SUC_lexicon_tags_15 },
    { 0xc14fa1bd, SUC_lexicon_tags_43 },
    { 0xc5afa1be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cfa1c2, SUC_lexicon_tags_37 },
    { 0x1ecfa1c2, SUC_lexicon_tags_82 },
    { 0x41efa1c4, SUC_lexicon_tags_14 },
    { 0xfcafa1c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aafa1c8, SUC_lexicon_tags_7 },
    { 0x190fa1c9, SUC_lexicon_tags_27 },
    { 0xe8cfa1ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74fa1cc, SUC_lexicon_tags_17 },
    { 0xa64fa1cd, SUC_lexicon_tags_63 },
    { 0x2a0fa1ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168fa1d2, SUC_lexicon_tags_37 },
    { 0x69afa1d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x706fa1d6, SUC_lexicon_tags_2 },
    { 0x7b8fa1d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce6fa1d9, SUC_lexicon_tags_42 },
    { 0x556fa1da, SUC_lexicon_tags_0 },
    { 0x6bafa1db, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51afa1df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6fa1e5, SUC_lexicon_tags_70 },
    { 0x2c0fa1e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4fa1e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4fa1ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfefa1ee, SUC_lexicon_tags_4 },
    { 0xa44fa1ee, SUC_lexicon_tags_23 },
    { 0x176fa1f0, SUC_lexicon_tags_17 },
    { 0xd44fa1f0, SUC_lexicon_tags_12 },
    { 0x1d8fa1f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fafa1f5, SUC_lexicon_tags_14 },
    { 0x76efa1f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8cafa1f8, SUC_lexicon_tags_23 },
    { 0xb42fa1f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x56fa1fb, SUC_lexicon_tags_1 },
    { 0x914fa1fc, SUC_lexicon_tags_7 },
    { 0xc92fa1fd, SUC_lexicon_tags_12 },
    { 0xf48fa1fe, SUC_lexicon_tags_1 },
    { 0xe6fa1fc, SUC_lexicon_tags_9 },
    { 0xea6fa1fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2fa204, SUC_lexicon_tags_14 },
    { 0x69efa204, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabefa20f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe24fa211, SUC_lexicon_tags_82 },
    { 0x4a4fa212, SUC_lexicon_tags_63 },
    { 0x8acfa211, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa80fa215, SUC_lexicon_tags_11 },
    { 0xb56fa216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe12fa218, SUC_lexicon_tags_37 },
    { 0x7cfa218, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x450fa21c, SUC_lexicon_tags_27 },
    { 0xa9afa21d, SUC_lexicon_tags_6 },
    { 0xd98fa21d, SUC_lexicon_tags_0 },
    { 0x53efa21f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6fa221, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aefa228, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958fa22c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82cfa22f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40fa232, SUC_lexicon_tags_43 },
    { 0xc90fa233, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x834fa239, SUC_lexicon_tags_14 },
    { 0x664fa23a, SUC_lexicon_tags_28 },
    { 0x246fa23b, SUC_lexicon_tags_47 },
    { 0x2cefa23b, SUC_lexicon_tags_79 },
    { 0x7efa23d, SUC_lexicon_tags_15 },
    { 0x172fa23e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x466fa240, SUC_lexicon_tags_25 },
    { 0x176fa241, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24afa243, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64afa245, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x898fa24a, SUC_lexicon_tags_83 },
    { 0xfe6fa24a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcfa24e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x36efa250, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7efa253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d2fa255, SUC_lexicon_tags_66 },
    { 0xce6fa256, SUC_lexicon_tags_38 },
    { 0x496fa256, SUC_lexicon_tags_25 },
    { 0xcfafa258, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52efa25c, SUC_lexicon_tags_158 },
    { 0xb4afa25d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eefa261, SUC_lexicon_tags_7 },
    { 0x97cfa261, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x884fa265, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76afa269, SUC_lexicon_tags_3 },
    { 0xbaefa26a, SUC_lexicon_tags_56 },
    { 0xbccfa26b, SUC_lexicon_tags_25 },
    { 0xa06fa26c, SUC_lexicon_tags_10 },
    { 0xba8fa26b, SUC_lexicon_tags_2 },
    { 0x2cafa26b, SUC_lexicon_tags_7 },
    { 0xd20fa26f, SUC_lexicon_tags_37 },
    { 0x388fa270, SUC_lexicon_tags_14 },
    { 0x3c2fa271, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xabafa274, SUC_lexicon_tags_11 },
    { 0x57efa275, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8fa278, SUC_lexicon_tags_1 },
    { 0x80efa279, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x364fa27c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec4fa27e, SUC_lexicon_tags_2 },
    { 0xea8fa27f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8acfa281, SUC_lexicon_tags_27 },
    { 0x8fa282, SUC_lexicon_tags_37 },
    { 0x3a0fa281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x326fa285, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98fa288, SUC_lexicon_tags_1 },
    { 0x10cfa289, SUC_lexicon_tags_37 },
    { 0xc52fa28a, SUC_lexicon_tags_28 },
    { 0x4befa28b, SUC_lexicon_tags_1 },
    { 0x22efa28b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a2fa28e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x924fa291, SUC_lexicon_tags_21 },
    { 0x176fa292, SUC_lexicon_tags_26 },
    { 0x5e0fa293, SUC_lexicon_tags_1 },
    { 0xa6afa291, SUC_lexicon_tags_2 },
    { 0x23cfa293, SUC_lexicon_tags_83 },
    { 0xedafa292, SUC_lexicon_tags_37 },
    { 0xe2afa297, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc0fa299, SUC_lexicon_tags_17 },
    { 0x442fa29a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5f4fa29c, SUC_lexicon_tags_25 },
    { 0xf4afa29d, SUC_lexicon_tags_12 },
    { 0x1d4fa29e, SUC_lexicon_tags_14 },
    { 0x8fcfa29f, SUC_lexicon_tags_1 },
    { 0xc0cfa29c, SUC_lexicon_tags_15 },
    { 0x46efa2a1, SUC_lexicon_tags_6 },
    { 0xbaefa2a1, SUC_lexicon_tags_47 },
    { 0xdc2fa2a3, SUC_lexicon_tags_22 },
    { 0x77efa2a3, SUC_lexicon_tags_12 },
    { 0x1cefa2a5, SUC_lexicon_tags_7 },
    { 0xf0afa2a5, SUC_lexicon_tags_30 },
    { 0x506fa29e, SUC_lexicon_tags_45 },
    { 0x45efa2a8, SUC_lexicon_tags_8 },
    { 0xc4efa29d, SUC_lexicon_tags_14 },
    { 0x82efa2aa, SUC_lexicon_tags_5 },
    { 0x754fa2a3, SUC_lexicon_tags_246 },
    { 0x628fa2a7, SUC_lexicon_tags_17 },
    { 0xde2fa2a2, SUC_lexicon_tags_61 },
    { 0x70fa2ae, SUC_lexicon_tags_7 },
    { 0x954fa2af, SUC_lexicon_tags_11 },
    { 0xc3cfa2b0, SUC_lexicon_tags_9 },
    { 0xb08fa2b0, SUC_lexicon_tags_25 },
    { 0x94afa2b2, SUC_lexicon_tags_38 },
    { 0x3f8fa2af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cfa2b8, SUC_lexicon_tags_6 },
    { 0x86fa2b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x762fa2bc, SUC_lexicon_tags_11 },
    { 0xf30fa2bc, SUC_lexicon_tags_63 },
    { 0x1a8fa2be, SUC_lexicon_tags_25 },
    { 0xcefa2bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65afa2c1, SUC_lexicon_tags_25 },
    { 0xc18fa2c2, SUC_lexicon_tags_1 },
    { 0xcaafa2c1, SUC_lexicon_tags_37 },
    { 0xe26fa2c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfefa2c6, SUC_lexicon_tags_30 },
    { 0x56fa2c6, SUC_lexicon_tags_384 },
    { 0x868fa2c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cfa2cd, SUC_lexicon_tags_5 },
    { 0x98efa2cd, SUC_lexicon_tags_11 },
    { 0x952fa2cf, SUC_lexicon_tags_23 },
    { 0x94fa2d0, SUC_lexicon_tags_25 },
    { 0xd0fa2d0, SUC_lexicon_tags_47 },
    { 0xf38fa2d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37efa2d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f4fa2d9, SUC_lexicon_tags_46 },
    { 0x214fa2da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cfa2de, SUC_lexicon_tags_17 },
    { 0x374fa2df, SUC_lexicon_tags_27 },
    { 0x164fa2e0, SUC_lexicon_tags_38 },
    { 0x218fa2e0, SUC_lexicon_tags_60 },
    { 0xa2efa2de, SUC_lexicon_tags_27 },
    { 0x99cfa2df, SUC_lexicon_tags_38 },
    { 0x24efa2e1, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xd46fa2e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772fa2ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4fa2f0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8fa2f5, SUC_lexicon_tags_21 },
    { 0x4d0fa2f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x130fa2f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x516fa2fa, SUC_lexicon_tags_19 },
    { 0xf10fa2fb, SUC_lexicon_tags_1 },
    { 0x58afa2fc, SUC_lexicon_tags_65 },
    { 0x26fa2fb, SUC_lexicon_tags_37 },
    { 0xcf2fa2fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x120fa301, SUC_lexicon_tags_17 },
    { 0xe96fa302, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdafa306, SUC_lexicon_tags_37 },
    { 0x696fa307, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8fa30b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x756fa310, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cfa314, SUC_lexicon_tags_29 },
    { 0xb7cfa314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53cfa318, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaefa31a, SUC_lexicon_tags_17 },
    { 0x498fa31b, SUC_lexicon_tags_25 },
    { 0x61afa31c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15efa31e, SUC_lexicon_tags_82 },
    { 0x3dafa31f, SUC_lexicon_tags_11 },
    { 0x18cfa320, SUC_lexicon_tags_49 },
    { 0x656fa321, SUC_lexicon_tags_8 },
    { 0x112fa320, SUC_lexicon_tags_0 },
    { 0xa2fa31f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3afa327, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe2fa329, SUC_lexicon_tags_15 },
    { 0x4d6fa32a, SUC_lexicon_tags_2 },
    { 0xe36fa32b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc80fa32d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeefa332, SUC_lexicon_tags_1 },
    { 0x7eafa333, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a8fa337, SUC_lexicon_tags_12 },
    { 0x8a4fa337, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe46fa33a, SUC_lexicon_tags_1 },
    { 0xbfefa33a, SUC_lexicon_tags_7 },
    { 0x8a8fa33a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x424fa33f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1b2fa341, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4e0fa343, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9befa346, SUC_lexicon_tags_2 },
    { 0x79efa346, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x740fa34a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeacfa34c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774fa352, SUC_lexicon_tags_17 },
    { 0x262fa352, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x186fa355, SUC_lexicon_tags_12 },
    { 0x436fa356, SUC_lexicon_tags_25 },
    { 0xc58fa356, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cfa35b, SUC_lexicon_tags_7 },
    { 0xf1afa35c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecfa35f, SUC_lexicon_tags_33 },
    { 0xcf6fa360, SUC_lexicon_tags_12 },
    { 0x7a4fa361, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5d2fa363, SUC_lexicon_tags_37 },
    { 0x680fa364, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfd4fa366, SUC_lexicon_tags_6 },
    { 0x14cfa367, SUC_lexicon_tags_17 },
    { 0xc60fa368, SUC_lexicon_tags_7 },
    { 0x548fa369, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52afa370, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914fa374, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe86fa378, SUC_lexicon_tags_1 },
    { 0xd8efa378, SUC_lexicon_tags_25 },
    { 0xd9cfa378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb04fa37c, SUC_lexicon_tags_18 },
    { 0x2ccfa37d, SUC_lexicon_tags_46 },
    { 0x47afa37c, SUC_lexicon_tags_1 },
    { 0xe9cfa37d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x620fa381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17cfa383, SUC_lexicon_tags_15 },
    { 0x25afa384, SUC_lexicon_tags_15 },
    { 0xbc4fa385, SUC_lexicon_tags_37 },
    { 0x596fa386, SUC_lexicon_tags_37 },
    { 0x3b6fa387, SUC_lexicon_tags_15 },
    { 0x1d8fa388, SUC_lexicon_tags_43 },
    { 0x2dafa389, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x340fa38b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82afa38e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6fa394, SUC_lexicon_tags_18 },
    { 0x24efa395, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104fa39b, SUC_lexicon_tags_2 },
    { 0xb24fa39c, SUC_lexicon_tags_7 },
    { 0x7a2fa39d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdafa3a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe94fa3a2, SUC_lexicon_tags_26 },
    { 0x478fa3a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb32fa3a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250fa3aa, SUC_lexicon_tags_39 },
    { 0x1d6fa3ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43cfa3ad, SUC_lexicon_tags_12 },
    { 0x598fa3ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6fa3b2, SUC_lexicon_tags_25 },
    { 0xf5cfa3b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2fa3b6, SUC_lexicon_tags_359 },
    { 0x77efa3b7, SUC_lexicon_tags_37 },
    { 0xb6fa3b8, SUC_lexicon_tags_21 },
    { 0x106fa3b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x51cfa3bb, SUC_lexicon_tags_2 },
    { 0xc0cfa3bb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79efa3c2, SUC_lexicon_tags_1 },
    { 0xb9cfa3c2, SUC_lexicon_tags_65 },
    { 0xd0cfa3c4, SUC_lexicon_tags_2 },
    { 0x17efa3c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59efa3cc, SUC_lexicon_tags_32 },
    { 0xb1afa3cd, SUC_lexicon_tags_7 },
    { 0x74fa3cd, SUC_lexicon_tags_0 },
    { 0x152fa3ce, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x8e6fa3d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85efa3d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eafa3dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3afa3e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x936fa3e3, SUC_lexicon_tags_23 },
    { 0x87efa3e3, SUC_lexicon_tags_15 },
    { 0xe34fa3e4, SUC_lexicon_tags_30 },
    { 0xd84fa3e4, SUC_lexicon_tags_37 },
    { 0x90efa3e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0fa3ea, SUC_lexicon_tags_37 },
    { 0xf0afa3ea, SUC_lexicon_tags_11 },
    { 0x4e2fa3eb, SUC_lexicon_tags_61 },
    { 0x9fafa3ea, SUC_lexicon_tags_7 },
    { 0x234fa3ee, SUC_lexicon_tags_15 },
    { 0x3fefa3ef, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x952fa3f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43afa3f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0fa3fe, SUC_lexicon_tags_25 },
    { 0x8b8fa3fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44fa404, SUC_lexicon_tags_17 },
    { 0xdacfa404, SUC_lexicon_tags_6 },
    { 0x52efa406, SUC_lexicon_tags_12 },
    { 0x212fa404, SUC_lexicon_tags_7 },
    { 0x4efa408, SUC_lexicon_tags_7 },
    { 0x3dafa409, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x816fa40b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546fa40f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77cfa412, SUC_lexicon_tags_25 },
    { 0x27cfa413, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14afa418, SUC_lexicon_tags_48 },
    { 0xd30fa418, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420fa41d, SUC_lexicon_tags_31 },
    { 0x94cfa41d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0efa421, SUC_lexicon_tags_27 },
    { 0x74efa421, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xabcfa425, SUC_lexicon_tags_33 },
    { 0x554fa426, SUC_lexicon_tags_28 },
    { 0xe08fa425, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3dafa429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978fa42e, SUC_lexicon_tags_11 },
    { 0x36cfa42f, SUC_lexicon_tags_8 },
    { 0x8fefa42f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37afa432, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x174fa435, SUC_lexicon_tags_30 },
    { 0x60afa435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cfa439, SUC_lexicon_tags_25 },
    { 0xcaefa43a, SUC_lexicon_tags_2 },
    { 0x40afa43a, SUC_lexicon_tags_39 },
    { 0xe86fa43c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2fa43f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cfa442, SUC_lexicon_tags_49 },
    { 0x4b2fa443, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ecfa445, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea4fa447, SUC_lexicon_tags_10 },
    { 0x108fa447, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59cfa44a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8fa44e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc0fa450, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcafa452, SUC_lexicon_tags_37 },
    { 0x202fa453, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8cfa455, SUC_lexicon_tags_37 },
    { 0x7befa455, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x442fa45a, SUC_lexicon_tags_11 },
    { 0xc36fa45b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54fa460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2afa462, SUC_lexicon_tags_6 },
    { 0x13afa462, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa8fa465, SUC_lexicon_tags_29 },
    { 0x1e8fa466, SUC_lexicon_tags_15 },
    { 0x594fa465, SUC_lexicon_tags_43 },
    { 0xb3cfa468, SUC_lexicon_tags_32 },
    { 0xbe2fa469, SUC_lexicon_tags_15 },
    { 0x9c8fa469, SUC_lexicon_tags_3 },
    { 0xb98fa46b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34cfa46d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cfa47a, SUC_lexicon_tags_48 },
    { 0xe52fa47a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fa481, SUC_lexicon_tags_34 },
    { 0xd96fa481, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x112fa484, SUC_lexicon_tags_15 },
    { 0x7ccfa485, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x456fa487, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6fa48d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc7efa48f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8efa497, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cfa49a, SUC_lexicon_tags_1 },
    { 0xde0fa49b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972fa49f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbf2fa4a1, SUC_lexicon_tags_33 },
    { 0xe8fa4a2, SUC_lexicon_tags_295 },
    { 0x1eefa4a1, SUC_lexicon_tags_33 },
    { 0xc98fa4a4, SUC_lexicon_tags_12 },
    { 0x4d8fa4a4, SUC_lexicon_tags_28 },
    { 0x65afa4a2, SUC_lexicon_tags_39 },
    { 0xc32fa4a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02fa4aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c0fa4ac, SUC_lexicon_tags_1 },
    { 0xa9efa4ad, SUC_lexicon_tags_25 },
    { 0xe28fa4ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x584fa4b1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x57afa4b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62fa4b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa48fa4b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x314fa4bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10efa4bf, SUC_lexicon_tags_27 },
    { 0x21cfa4c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf20fa4c3, SUC_lexicon_tags_46 },
    { 0x39afa4c3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x96efa4c7, SUC_lexicon_tags_12 },
    { 0x32efa4c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x706fa4ca, SUC_lexicon_tags_34 },
    { 0xc22fa4ca, SUC_lexicon_tags_0 },
    { 0xbc2fa4cc, SUC_lexicon_tags_27 },
    { 0xe8fa4ca, SUC_lexicon_tags_17 },
    { 0x836fa4ce, SUC_lexicon_tags_70 },
    { 0x2cefa4cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf04fa4d2, SUC_lexicon_tags_26 },
    { 0xba2fa4d2, SUC_lexicon_tags_2 },
    { 0xd60fa4d2, SUC_lexicon_tags_7 },
    { 0xe3afa4d5, SUC_lexicon_tags_37 },
    { 0x3a2fa4d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x376fa4d8, SUC_lexicon_tags_15 },
    { 0xd7efa4d8, SUC_lexicon_tags_37 },
    { 0x8fefa4da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58efa4de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688fa4e5, SUC_lexicon_tags_10 },
    { 0x806fa4e6, SUC_lexicon_tags_49 },
    { 0xf42fa4e7, SUC_lexicon_tags_63 },
    { 0x256fa4e7, SUC_lexicon_tags_1 },
    { 0x232fa4e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0fa4ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3efa4f0, SUC_lexicon_tags_1 },
    { 0x41efa4f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6fa4f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cafa4fa, SUC_lexicon_tags_30 },
    { 0xa0afa4fa, SUC_lexicon_tags_4 },
    { 0x8b6fa4fc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cfa4ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x104fa502, SUC_lexicon_tags_13 },
    { 0x2afa502, SUC_lexicon_tags_28 },
    { 0x5fcfa503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40fa509, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf16fa50b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35efa50d, SUC_lexicon_tags_12 },
    { 0x4aafa50e, SUC_lexicon_tags_19 },
    { 0xe44fa50e, SUC_lexicon_tags_25 },
    { 0x3f6fa510, SUC_lexicon_tags_46 },
    { 0x3c2fa510, SUC_lexicon_tags_14 },
    { 0x26cfa512, SUC_lexicon_tags_15 },
    { 0x1cfa512, SUC_lexicon_tags_75 },
    { 0x690fa514, SUC_lexicon_tags_25 },
    { 0x2bcfa513, SUC_lexicon_tags_12 },
    { 0x688fa513, SUC_lexicon_tags_60 },
    { 0x4e8fa517, SUC_lexicon_tags_7 },
    { 0x57efa514, SUC_lexicon_tags_23 },
    { 0xda8fa517, SUC_lexicon_tags_84 },
    { 0x606fa51a, SUC_lexicon_tags_39 },
    { 0xd98fa518, SUC_lexicon_tags_25 },
    { 0x5c0fa50d, SUC_lexicon_tags_9 },
    { 0xdc2fa51d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb74fa51f, SUC_lexicon_tags_55 },
    { 0xe2cfa51f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452fa527, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404fa52d, SUC_lexicon_tags_25 },
    { 0x178fa52e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0fa533, SUC_lexicon_tags_45 },
    { 0xd2cfa533, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38fa538, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40afa53e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acfa541, SUC_lexicon_tags_63 },
    { 0x29cfa542, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa28fa544, SUC_lexicon_tags_49 },
    { 0xb48fa545, SUC_lexicon_tags_33 },
    { 0xf40fa546, SUC_lexicon_tags_1 },
    { 0xc72fa546, SUC_lexicon_tags_7 },
    { 0x532fa545, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x906fa54b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb68fa54e, SUC_lexicon_tags_29 },
    { 0xf4cfa54e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x438fa552, SUC_lexicon_tags_31 },
    { 0xd12fa553, SUC_lexicon_tags_87 },
    { 0x748fa554, SUC_lexicon_tags_47 },
    { 0xb04fa554, SUC_lexicon_tags_37 },
    { 0x5d8fa553, SUC_lexicon_tags_9 },
    { 0x640fa557, SUC_lexicon_tags_11 },
    { 0xc96fa555, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e2fa55a, SUC_lexicon_tags_30 },
    { 0x2a2fa55b, SUC_lexicon_tags_37 },
    { 0xb14fa55a, SUC_lexicon_tags_25 },
    { 0x9b8fa55d, SUC_lexicon_tags_12 },
    { 0x90efa55c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x692fa568, SUC_lexicon_tags_27 },
    { 0x5dafa569, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0afa56e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f0fa570, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746fa577, SUC_lexicon_tags_48 },
    { 0x57efa578, SUC_lexicon_tags_87 },
    { 0x562fa579, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a2fa57c, SUC_lexicon_tags_36 },
    { 0xd70fa57d, SUC_lexicon_tags_2 },
    { 0x854fa57d, SUC_lexicon_tags_25 },
    { 0x252fa57f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16fa582, SUC_lexicon_tags_11 },
    { 0xd12fa582, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80afa587, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cfa58e, SUC_lexicon_tags_5 },
    { 0xab8fa58f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93cfa591, SUC_lexicon_tags_25 },
    { 0x4d8fa591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x802fa595, SUC_lexicon_tags_11 },
    { 0x3a8fa596, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x81afa599, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220fa59d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4fa5a3, SUC_lexicon_tags_12 },
    { 0xcfa5a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6fa5b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77afa5b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f6fa5b5, SUC_lexicon_tags_7 },
    { 0x590fa5b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x158fa5b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x876fa5ba, SUC_lexicon_tags_25 },
    { 0x1ccfa5bb, SUC_lexicon_tags_45 },
    { 0x7bafa5bc, SUC_lexicon_tags_37 },
    { 0x8fa5bc, SUC_lexicon_tags_82 },
    { 0xfa6fa5bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1f4fa5c0, SUC_lexicon_tags_92 },
    { 0x2f4fa5c0, SUC_lexicon_tags_238 },
    { 0x6e6fa5c0, SUC_lexicon_tags_39 },
    { 0xf20fa5c0, SUC_lexicon_tags_11 },
    { 0x9cafa5c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93cfa5c6, SUC_lexicon_tags_68 },
    { 0x578fa5c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x300fa5ca, SUC_lexicon_tags_49 },
    { 0xaafa5ca, SUC_lexicon_tags_25 },
    { 0x388fa5cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176fa5d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0fa5d8, SUC_lexicon_tags_25 },
    { 0x3c6fa5d8, SUC_lexicon_tags_11 },
    { 0xce0fa5da, SUC_lexicon_tags_0 },
    { 0xec4fa5db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc48fa5df, SUC_lexicon_tags_7 },
    { 0x9ccfa5df, SUC_lexicon_tags_0 },
    { 0x740fa5e0, SUC_lexicon_tags_12 },
    { 0xe30fa5e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x754fa5e4, SUC_lexicon_tags_0 },
    { 0xcc2fa5e4, SUC_lexicon_tags_7 },
    { 0x3cefa5e5, SUC_lexicon_tags_47 },
    { 0xdbafa5e6, SUC_lexicon_tags_9 },
    { 0xacafa5e5, SUC_lexicon_tags_2 },
    { 0x24afa5e9, SUC_lexicon_tags_53 },
    { 0x1f2fa5ea, SUC_lexicon_tags_17 },
    { 0xc0afa5e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaccfa5ee, SUC_lexicon_tags_11 },
    { 0x204fa5ef, SUC_lexicon_tags_11 },
    { 0x2f4fa5ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x702fa5f4, SUC_lexicon_tags_8 },
    { 0xe2fa5f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79cfa5fd, SUC_lexicon_tags_23 },
    { 0x586fa5fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884fa606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0fa60c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x570fa60f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0fa616, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72fa61e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cefa621, SUC_lexicon_tags_19 },
    { 0x914fa622, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5ccfa624, SUC_lexicon_tags_2 },
    { 0x702fa624, SUC_lexicon_tags_19 },
    { 0xeb4fa624, SUC_lexicon_tags_35 },
    { 0x428fa626, SUC_lexicon_tags_46 },
    { 0xdc0fa625, SUC_lexicon_tags_7 },
    { 0xed6fa629, SUC_lexicon_tags_38 },
    { 0xb4cfa62a, SUC_lexicon_tags_15 },
    { 0xf7efa624, SUC_lexicon_tags_34 },
    { 0x6c6fa629, SUC_lexicon_tags_37 },
    { 0xabcfa62d, SUC_lexicon_tags_17 },
    { 0xcc4fa629, SUC_lexicon_tags_25 },
    { 0xce4fa628, SUC_lexicon_tags_12 },
    { 0xf78fa630, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a4fa632, SUC_lexicon_tags_1 },
    { 0x5efa633, SUC_lexicon_tags_25 },
    { 0xeeefa632, SUC_lexicon_tags_17 },
    { 0x316fa632, SUC_lexicon_tags_10 },
    { 0x6fafa636, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa46fa639, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe00fa63b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14fa63f, SUC_lexicon_tags_28 },
    { 0xb42fa640, SUC_lexicon_tags_28 },
    { 0x5b8fa641, SUC_lexicon_tags_60 },
    { 0x784fa642, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcfa645, SUC_lexicon_tags_12 },
    { 0x14fa646, SUC_lexicon_tags_25 },
    { 0x3bafa645, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa64f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd86fa651, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36fa654, SUC_lexicon_tags_45 },
    { 0x82afa654, SUC_lexicon_tags_6 },
    { 0x12efa656, SUC_lexicon_tags_60 },
    { 0xa40fa656, SUC_lexicon_tags_17 },
    { 0x2b8fa656, SUC_lexicon_tags_17 },
    { 0x25efa657, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60fa65d, SUC_lexicon_tags_14 },
    { 0xccafa65e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba6fa660, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42fa668, SUC_lexicon_tags_11 },
    { 0xad2fa668, SUC_lexicon_tags_16 },
    { 0x252fa668, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0afa670, SUC_lexicon_tags_37 },
    { 0x65afa671, SUC_lexicon_tags_55 },
    { 0xde6fa672, SUC_lexicon_tags_68 },
    { 0x81cfa673, SUC_lexicon_tags_27 },
    { 0x124fa674, SUC_lexicon_tags_32 },
    { 0xb00fa672, SUC_lexicon_tags_63 },
    { 0xb08fa676, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x198fa679, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x586fa67b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x840fa67e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90efa680, SUC_lexicon_tags_27 },
    { 0x31cfa680, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x802fa683, SUC_lexicon_tags_33 },
    { 0x6bcfa684, SUC_lexicon_tags_2 },
    { 0x2c6fa685, SUC_lexicon_tags_39 },
    { 0x1f4fa686, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3e6fa688, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88afa68d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4fa691, SUC_lexicon_tags_7 },
    { 0x9b0fa691, SUC_lexicon_tags_2 },
    { 0x172fa692, SUC_lexicon_tags_136 },
    { 0xd56fa694, SUC_lexicon_tags_25 },
    { 0xd7cfa695, SUC_lexicon_tags_50 },
    { 0x43afa691, SUC_lexicon_tags_31 },
    { 0x5cafa692, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24fa69a, SUC_lexicon_tags_85 },
    { 0x784fa69b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8fa69e, SUC_lexicon_tags_37 },
    { 0x65cfa69f, SUC_lexicon_tags_0 },
    { 0x3dcfa6a0, SUC_lexicon_tags_1 },
    { 0x210fa6a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc04fa6a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52fa6a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xefefa6ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x776fa6ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0fa6b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed2fa6b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe76fa6b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x610fa6bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6fa6bf, SUC_lexicon_tags_27 },
    { 0xef8fa6c0, SUC_lexicon_tags_47 },
    { 0xa98fa6c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x522fa6cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214fa6d3, SUC_lexicon_tags_156 },
    { 0xc06fa6d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79cfa6d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb8fa6db, SUC_lexicon_tags_17 },
    { 0x606fa6db, SUC_lexicon_tags_23 },
    { 0xbc8fa6dd, SUC_lexicon_tags_2 },
    { 0x7dafa6de, SUC_lexicon_tags_66 },
    { 0x194fa6df, SUC_lexicon_tags_9 },
    { 0xe0afa6e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65afa6e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69afa6ea, SUC_lexicon_tags_31 },
    { 0x7a6fa6eb, SUC_lexicon_tags_7 },
    { 0x9d6fa6eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xac8fa6ef, SUC_lexicon_tags_25 },
    { 0x906fa6f0, SUC_lexicon_tags_87 },
    { 0x45efa6f1, SUC_lexicon_tags_10 },
    { 0xf0fa6f2, SUC_lexicon_tags_68 },
    { 0x44fa6f3, SUC_lexicon_tags_8 },
    { 0xdf6fa6f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbafa6f6, SUC_lexicon_tags_87 },
    { 0x740fa6f7, SUC_lexicon_tags_60 },
    { 0x96efa6f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x284fa6fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25cfa6fc, SUC_lexicon_tags_13 },
    { 0x2dcfa6fc, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66fa703, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96afa707, SUC_lexicon_tags_19 },
    { 0x2d6fa708, SUC_lexicon_tags_27 },
    { 0x10afa708, SUC_lexicon_tags_12 },
    { 0xc90fa709, SUC_lexicon_tags_30 },
    { 0x9c8fa70b, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaacfa70f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaefa71b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa26fa71f, SUC_lexicon_tags_9 },
    { 0x3e6fa720, SUC_lexicon_tags_0 },
    { 0xbbafa720, SUC_lexicon_tags_9 },
    { 0x94cfa720, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x444fa726, SUC_lexicon_tags_2 },
    { 0x19cfa727, SUC_lexicon_tags_0 },
    { 0xce0fa728, SUC_lexicon_tags_39 },
    { 0x354fa728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x246fa72b, SUC_lexicon_tags_17 },
    { 0x6c6fa72b, SUC_lexicon_tags_6 },
    { 0xb4efa72c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d2fa72f, SUC_lexicon_tags_37 },
    { 0xc36fa730, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874fa734, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x998fa736, SUC_lexicon_tags_37 },
    { 0x30efa736, SUC_lexicon_tags_25 },
    { 0x7b4fa738, SUC_lexicon_tags_26 },
    { 0xe08fa738, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0fa73d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fa741, SUC_lexicon_tags_16 },
    { 0x76afa742, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44efa744, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71afa74b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x60efa74e, SUC_lexicon_tags_2 },
    { 0xd5cfa74f, SUC_lexicon_tags_18 },
    { 0xdaafa750, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x238fa753, SUC_lexicon_tags_10 },
    { 0x4fcfa754, SUC_lexicon_tags_37 },
    { 0x4befa755, SUC_lexicon_tags_2 },
    { 0x6fefa756, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106fa765, SUC_lexicon_tags_30 },
    { 0xf8cfa765, SUC_lexicon_tags_2 },
    { 0xd74fa767, SUC_lexicon_tags_9 },
    { 0x31cfa767, SUC_lexicon_tags_2 },
    { 0xeecfa767, SUC_lexicon_tags_17 },
    { 0xf86fa76a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8fa772, SUC_lexicon_tags_1 },
    { 0x56efa773, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8afa778, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x328fa781, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7d0fa783, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18fa787, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8fa78a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x94afa78c, SUC_lexicon_tags_23 },
    { 0xd06fa78d, SUC_lexicon_tags_47 },
    { 0xb48fa78e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00fa791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7defa793, SUC_lexicon_tags_7 },
    { 0x4defa793, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8afa796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0fa799, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cfa79d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x958fa7a1, SUC_lexicon_tags_28 },
    { 0x306fa7a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54fa7a9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cfa7ac, SUC_lexicon_tags_17 },
    { 0xf70fa7ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82fa7b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8fa7bf, SUC_lexicon_tags_24 },
    { 0x34cfa7c0, SUC_lexicon_tags_37 },
    { 0x2fefa7c1, SUC_lexicon_tags_47 },
    { 0x1aefa7c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x606fa7d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aafa7d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534fa7da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x712fa7dd, SUC_lexicon_tags_3 },
    { 0xc20fa7de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eafa7e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab4fa7e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x884fa7e6, SUC_lexicon_tags_2 },
    { 0x1a0fa7e7, SUC_lexicon_tags_1 },
    { 0x7c0fa7e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa60fa7ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a0fa7ee, SUC_lexicon_tags_47 },
    { 0x194fa7ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x246fa7f2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb6cfa7f4, SUC_lexicon_tags_46 },
    { 0xdbcfa7f4, SUC_lexicon_tags_15 },
    { 0xe94fa7f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2afa7f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf28fa7fb, SUC_lexicon_tags_47 },
    { 0x2dafa7fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f8fa803, SUC_lexicon_tags_12 },
    { 0xe90fa804, SUC_lexicon_tags_11 },
    { 0xeeefa805, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x1a4fa807, SUC_lexicon_tags_8 },
    { 0x708fa807, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5fafa80a, SUC_lexicon_tags_27 },
    { 0x1e2fa80a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4fa819, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xce6fa81b, SUC_lexicon_tags_47 },
    { 0x2afa81c, SUC_lexicon_tags_61 },
    { 0x93efa81d, SUC_lexicon_tags_39 },
    { 0x196fa81c, SUC_lexicon_tags_25 },
    { 0x660fa81f, SUC_lexicon_tags_14 },
    { 0x97cfa81c, SUC_lexicon_tags_7 },
    { 0x208fa81f, SUC_lexicon_tags_37 },
    { 0x8a2fa822, SUC_lexicon_tags_1 },
    { 0x60efa81e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe70fa825, SUC_lexicon_tags_11 },
    { 0x2a0fa826, SUC_lexicon_tags_43 },
    { 0x986fa825, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6fa82a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf6fa82c, SUC_lexicon_tags_79 },
    { 0x5d6fa82d, SUC_lexicon_tags_2 },
    { 0xd32fa82e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18fa832, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0fa835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x568fa837, SUC_lexicon_tags_25 },
    { 0xf4efa838, SUC_lexicon_tags_79 },
    { 0xa9afa838, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fa83f, SUC_lexicon_tags_11 },
    { 0x6bafa840, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x114fa842, SUC_lexicon_tags_34 },
    { 0x3defa842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf88fa846, SUC_lexicon_tags_2 },
    { 0xd5efa847, SUC_lexicon_tags_1 },
    { 0x9e2fa848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7aefa84a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf6cfa84c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0fa84f, SUC_lexicon_tags_42 },
    { 0x462fa84f, SUC_lexicon_tags_15 },
    { 0xd7afa850, SUC_lexicon_tags_37 },
    { 0x75efa852, SUC_lexicon_tags_49 },
    { 0xc64fa853, SUC_lexicon_tags_7 },
    { 0xff2fa854, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x668fa856, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x638fa859, SUC_lexicon_tags_47 },
    { 0x37afa85a, SUC_lexicon_tags_47 },
    { 0x1a2fa85b, SUC_lexicon_tags_21 },
    { 0xbd6fa85b, SUC_lexicon_tags_49 },
    { 0x568fa85b, SUC_lexicon_tags_27 },
    { 0xc6efa85e, SUC_lexicon_tags_63 },
    { 0x66cfa85b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x754fa861, SUC_lexicon_tags_23 },
    { 0xae4fa862, SUC_lexicon_tags_12 },
    { 0xcf2fa861, SUC_lexicon_tags_7 },
    { 0x18afa864, SUC_lexicon_tags_1 },
    { 0x22fa861, SUC_lexicon_tags_11 },
    { 0xd4afa866, SUC_lexicon_tags_29 },
    { 0xea8fa866, SUC_lexicon_tags_30 },
    { 0x6ccfa868, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fa86c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0fa870, SUC_lexicon_tags_6 },
    { 0x67cfa870, SUC_lexicon_tags_43 },
    { 0xf0fa871, SUC_lexicon_tags_11 },
    { 0x646fa870, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980fa878, SUC_lexicon_tags_12 },
    { 0x878fa879, SUC_lexicon_tags_27 },
    { 0x872fa87a, SUC_lexicon_tags_23 },
    { 0x52cfa87b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cefa887, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36fa88a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740fa890, SUC_lexicon_tags_11 },
    { 0xa8cfa891, SUC_lexicon_tags_60 },
    { 0xe0afa891, SUC_lexicon_tags_28 },
    { 0x2a6fa893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c0fa895, SUC_lexicon_tags_25 },
    { 0xbd2fa896, SUC_lexicon_tags_66 },
    { 0x350fa896, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x290fa899, SUC_lexicon_tags_7 },
    { 0x656fa89a, SUC_lexicon_tags_55 },
    { 0xfa8fa89b, SUC_lexicon_tags_0 },
    { 0xddefa89c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb4fa89e, SUC_lexicon_tags_7 },
    { 0x3c6fa89f, SUC_lexicon_tags_7 },
    { 0xc36fa89e, SUC_lexicon_tags_37 },
    { 0x772fa8a1, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24fa8a4, SUC_lexicon_tags_63 },
    { 0xa74fa8a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbfefa8a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x884fa8aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdacfa8ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248fa8b1, SUC_lexicon_tags_37 },
    { 0xbfcfa8b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x772fa8b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474fa8bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2fa8c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6fa8c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf96fa8c6, SUC_lexicon_tags_38 },
    { 0xeb8fa8c6, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42fa8cc, SUC_lexicon_tags_6 },
    { 0x794fa8cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfccfa8cf, SUC_lexicon_tags_48 },
    { 0xf68fa8d0, SUC_lexicon_tags_17 },
    { 0x80afa8d1, SUC_lexicon_tags_25 },
    { 0xbc6fa8d2, SUC_lexicon_tags_23 },
    { 0x476fa8d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb56fa8d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33afa8d8, SUC_lexicon_tags_1 },
    { 0x950fa8d8, SUC_lexicon_tags_43 },
    { 0xf28fa8da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x692fa8dd, SUC_lexicon_tags_15 },
    { 0x94fa8dd, SUC_lexicon_tags_12 },
    { 0x780fa8df, SUC_lexicon_tags_15 },
    { 0x6eafa8dd, SUC_lexicon_tags_47 },
    { 0xbccfa8e1, SUC_lexicon_tags_56 },
    { 0xc94fa8dd, SUC_lexicon_tags_17 },
    { 0xbecfa8e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54afa8e8, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x32cfa8ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaefa8f1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xccafa8f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe04fa8f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc22fa900, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2fa903, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d8fa905, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70efa909, SUC_lexicon_tags_48 },
    { 0xb36fa90a, SUC_lexicon_tags_56 },
    { 0xf42fa90b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdefa90f, SUC_lexicon_tags_38 },
    { 0x954fa90f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadafa915, SUC_lexicon_tags_14 },
    { 0xcbcfa916, SUC_lexicon_tags_10 },
    { 0x30afa916, SUC_lexicon_tags_48 },
    { 0x536fa916, SUC_lexicon_tags_1 },
    { 0xf96fa917, SUC_lexicon_tags_9 },
    { 0x4dafa91a, SUC_lexicon_tags_47 },
    { 0x88efa91b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf28fa922, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdaafa924, SUC_lexicon_tags_17 },
    { 0x1aefa924, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2fa928, SUC_lexicon_tags_23 },
    { 0x64fa929, SUC_lexicon_tags_79 },
    { 0x738fa92a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf68fa92c, SUC_lexicon_tags_10 },
    { 0x840fa92c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f4fa92f, SUC_lexicon_tags_66 },
    { 0x1e4fa930, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc38fa932, SUC_lexicon_tags_15 },
    { 0x924fa933, SUC_lexicon_tags_9 },
    { 0xf2cfa934, SUC_lexicon_tags_7 },
    { 0x856fa935, SUC_lexicon_tags_9 },
    { 0x66fa936, SUC_lexicon_tags_31 },
    { 0xafa936, SUC_lexicon_tags_15 },
    { 0x4f2fa937, SUC_lexicon_tags_17 },
    { 0xeacfa939, SUC_lexicon_tags_12 },
    { 0x5b0fa937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa0fa93c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xefcfa93e, SUC_lexicon_tags_28 },
    { 0x47afa93e, SUC_lexicon_tags_87 },
    { 0x960fa940, SUC_lexicon_tags_37 },
    { 0x3efa93e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e4fa946, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x9a8fa948, SUC_lexicon_tags_15 },
    { 0x4d8fa948, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xf1efa94b, SUC_lexicon_tags_11 },
    { 0x3cfa94c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00fa950, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd8afa952, SUC_lexicon_tags_9 },
    { 0xc7efa952, SUC_lexicon_tags_14 },
    { 0x592fa954, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316fa95a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1afa95e, SUC_lexicon_tags_11 },
    { 0xb80fa95e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x15efa962, SUC_lexicon_tags_84 },
    { 0xfdefa963, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2afa967, SUC_lexicon_tags_21 },
    { 0xf74fa968, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x588fa96b, SUC_lexicon_tags_65 },
    { 0xb48fa96c, SUC_lexicon_tags_2 },
    { 0x528fa96c, SUC_lexicon_tags_65 },
    { 0xa10fa96e, SUC_lexicon_tags_64 },
    { 0x1c6fa96d, SUC_lexicon_tags_11 },
    { 0x6dcfa96d, SUC_lexicon_tags_36 },
    { 0xf4fa970, SUC_lexicon_tags_95 },
    { 0xe3cfa96c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x496fa975, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8aefa977, SUC_lexicon_tags_22 },
    { 0xb04fa978, SUC_lexicon_tags_7 },
    { 0x6b4fa979, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x21afa97c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x238fa97e, SUC_lexicon_tags_0 },
    { 0x344fa97f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x876fa981, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14fa987, SUC_lexicon_tags_11 },
    { 0x3c8fa988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3eefa98a, SUC_lexicon_tags_43 },
    { 0x94afa98a, SUC_lexicon_tags_8 },
    { 0xedcfa98c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58efa994, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x982fa997, SUC_lexicon_tags_38 },
    { 0x2f4fa998, SUC_lexicon_tags_17 },
    { 0x91cfa997, SUC_lexicon_tags_11 },
    { 0x6e2fa99a, SUC_lexicon_tags_84 },
    { 0xfd8fa99b, SUC_lexicon_tags_28 },
    { 0x266fa997, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bafa9a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe16fa9a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x34afa9a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xac4fa9aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5efa9ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcafa9b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86fa9b5, SUC_lexicon_tags_37 },
    { 0x3c6fa9b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8fa9c3, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0xe6efa9c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2fa9d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xae4fa9d7, SUC_lexicon_tags_23 },
    { 0xe5cfa9d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccfa9db, SUC_lexicon_tags_27 },
    { 0xbacfa9dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed0fa9de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8fa9e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe6fa9e5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc42fa9e7, SUC_lexicon_tags_64 },
    { 0x85cfa9e8, SUC_lexicon_tags_5 },
    { 0x2e0fa9e9, SUC_lexicon_tags_23 },
    { 0xd04fa9ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4fa9ef, SUC_lexicon_tags_56 },
    { 0xd66fa9ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x808fa9f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4fa9f8, SUC_lexicon_tags_31 },
    { 0xcf0fa9f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8faa03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedefaa09, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x620faa0b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbbafaa0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6faa11, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd7cfaa13, SUC_lexicon_tags_33 },
    { 0xc3efaa14, SUC_lexicon_tags_71 },
    { 0x91cfaa15, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4e0faa17, SUC_lexicon_tags_65 },
    { 0xe36faa17, SUC_lexicon_tags_55 },
    { 0x4f8faa18, SUC_lexicon_tags_38 },
    { 0x20cfaa1a, SUC_lexicon_tags_17 },
    { 0xa1afaa1a, SUC_lexicon_tags_1 },
    { 0x7ccfaa1c, SUC_lexicon_tags_7 },
    { 0xc94faa1b, SUC_lexicon_tags_1 },
    { 0xbb0faa1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d4faa20, SUC_lexicon_tags_11 },
    { 0x33cfaa21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f0faa23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cfaa2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cafaa2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc76faa2e, SUC_lexicon_tags_37 },
    { 0xcfafaa2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x904faa31, SUC_lexicon_tags_17 },
    { 0xa9cfaa32, SUC_lexicon_tags_86 },
    { 0x412faa33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xedcfaa35, SUC_lexicon_tags_45 },
    { 0x72efaa35, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348faa3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x432faa3e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4faa42, SUC_lexicon_tags_15 },
    { 0x8b6faa42, SUC_lexicon_tags_17 },
    { 0xa6faa43, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28efaa4a, SUC_lexicon_tags_1 },
    { 0x440faa4b, SUC_lexicon_tags_15 },
    { 0x40afaa4c, SUC_lexicon_tags_12 },
    { 0x370faa4d, SUC_lexicon_tags_49 },
    { 0xfc2faa4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76faa54, SUC_lexicon_tags_37 },
    { 0x58cfaa55, SUC_lexicon_tags_47 },
    { 0xdf8faa56, SUC_lexicon_tags_25 },
    { 0x97efaa57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x292faa59, SUC_lexicon_tags_12 },
    { 0xf24faa59, SUC_lexicon_tags_1 },
    { 0x102faa5b, SUC_lexicon_tags_4 },
    { 0x9a8faa5a, SUC_lexicon_tags_46 },
    { 0x1e4faa5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ecfaa5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf12faa61, SUC_lexicon_tags_35 },
    { 0xecefaa61, SUC_lexicon_tags_12 },
    { 0x658faa61, SUC_lexicon_tags_82 },
    { 0x7e2faa62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38faa68, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x79afaa6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7eafaa6d, SUC_lexicon_tags_15 },
    { 0x99afaa6e, SUC_lexicon_tags_61 },
    { 0x63efaa6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x760faa75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94cfaa78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf42faa7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22afaa83, SUC_lexicon_tags_14 },
    { 0x368faa84, SUC_lexicon_tags_55 },
    { 0xac4faa85, SUC_lexicon_tags_11 },
    { 0xb0efaa86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb24faa88, SUC_lexicon_tags_11 },
    { 0xe12faa89, SUC_lexicon_tags_120 },
    { 0xa62faa8a, SUC_lexicon_tags_33 },
    { 0x576faa8b, SUC_lexicon_tags_37 },
    { 0x2efaa8c, SUC_lexicon_tags_2 },
    { 0x7b4faa8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5cfaa8f, SUC_lexicon_tags_29 },
    { 0x33afaa90, SUC_lexicon_tags_16 },
    { 0x2cafaa91, SUC_lexicon_tags_39 },
    { 0x494faa92, SUC_lexicon_tags_11 },
    { 0x92afaa92, SUC_lexicon_tags_12 },
    { 0x6f4faa92, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x58afaa96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12afaa99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf26faa9b, SUC_lexicon_tags_25 },
    { 0x86cfaa9c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x71cfaa9f, SUC_lexicon_tags_17 },
    { 0xdc6faa9f, SUC_lexicon_tags_3 },
    { 0xe56faaa0, SUC_lexicon_tags_37 },
    { 0x6e6faaa2, SUC_lexicon_tags_68 },
    { 0x1efaa9f, SUC_lexicon_tags_17 },
    { 0x204faaa4, SUC_lexicon_tags_39 },
    { 0xfe0faaa4, SUC_lexicon_tags_15 },
    { 0xc5afaaa6, SUC_lexicon_tags_77 },
    { 0x442faaa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0efaaab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3efaaae, SUC_lexicon_tags_2 },
    { 0x368faaae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb98faab5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x12faab8, SUC_lexicon_tags_37 },
    { 0xd1efaab8, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xbe8faabb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x228faabd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6faac0, SUC_lexicon_tags_49 },
    { 0xcf0faac0, SUC_lexicon_tags_18 },
    { 0x25cfaac0, SUC_lexicon_tags_37 },
    { 0x1dafaac0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4faac7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5acfaacc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb9efaace, SUC_lexicon_tags_6 },
    { 0x650faace, SUC_lexicon_tags_31 },
    { 0x6d6faad0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa90faad2, SUC_lexicon_tags_38 },
    { 0xa6faad3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x962faad5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x57cfaad7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300faadb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16faae1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x490faae4, SUC_lexicon_tags_12 },
    { 0x1cafaae4, SUC_lexicon_tags_69 },
    { 0x52faae4, SUC_lexicon_tags_38 },
    { 0xae0faae4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eafaaea, SUC_lexicon_tags_106 },
    { 0xfaafaaea, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44afaaf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe0faaf2, SUC_lexicon_tags_7 },
    { 0x18afaaf3, SUC_lexicon_tags_34 },
    { 0x476faaf2, SUC_lexicon_tags_43 },
    { 0x6cfaaf5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc9efaaf7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x362faafb, SUC_lexicon_tags_0 },
    { 0xdafaafb, SUC_lexicon_tags_1 },
    { 0x4dafaafb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2defab00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06fab08, SUC_lexicon_tags_1 },
    { 0x858fab08, SUC_lexicon_tags_12 },
    { 0x162fab0a, SUC_lexicon_tags_23 },
    { 0x466fab0b, SUC_lexicon_tags_25 },
    { 0xc7cfab0b, SUC_lexicon_tags_49 },
    { 0x220fab09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed2fab0f, SUC_lexicon_tags_17 },
    { 0x286fab10, SUC_lexicon_tags_37 },
    { 0x1fcfab11, SUC_lexicon_tags_34 },
    { 0xba2fab12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88fab18, SUC_lexicon_tags_109 },
    { 0xe68fab19, SUC_lexicon_tags_19 },
    { 0xf68fab1a, SUC_lexicon_tags_12 },
    { 0x34cfab1a, SUC_lexicon_tags_2 },
    { 0xf9cfab1b, SUC_lexicon_tags_47 },
    { 0x3bcfab1b, SUC_lexicon_tags_12 },
    { 0x6ecfab1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8fab23, SUC_lexicon_tags_2 },
    { 0x912fab24, SUC_lexicon_tags_208 },
    { 0xbdefab25, SUC_lexicon_tags_23 },
    { 0x474fab25, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8cafab28, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0fab2d, SUC_lexicon_tags_60 },
    { 0xb3afab2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2fab36, SUC_lexicon_tags_27 },
    { 0xee0fab37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6fab3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79afab3f, SUC_lexicon_tags_7 },
    { 0x2befab40, SUC_lexicon_tags_45 },
    { 0x656fab41, SUC_lexicon_tags_13 },
    { 0xfb2fab42, SUC_lexicon_tags_11 },
    { 0xbd6fab42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58fab46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954fab4a, SUC_lexicon_tags_1 },
    { 0x3eefab4a, SUC_lexicon_tags_43 },
    { 0x7afab4c, SUC_lexicon_tags_11 },
    { 0xe2cfab4d, SUC_lexicon_tags_25 },
    { 0xf36fab4e, SUC_lexicon_tags_49 },
    { 0x584fab4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x832fab52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81afab54, SUC_lexicon_tags_23 },
    { 0xe80fab55, SUC_lexicon_tags_19 },
    { 0x5a2fab56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4fab59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2fab5c, SUC_lexicon_tags_42 },
    { 0xe8afab5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fab61, SUC_lexicon_tags_6 },
    { 0xb18fab62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaccfab66, SUC_lexicon_tags_17 },
    { 0x624fab67, SUC_lexicon_tags_70 },
    { 0xbcafab67, SUC_lexicon_tags_2 },
    { 0x472fab68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x478fab6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350fab70, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8fab78, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdafab7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x194fab7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92cfab80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x738fab83, SUC_lexicon_tags_184 },
    { 0x878fab84, SUC_lexicon_tags_14 },
    { 0x59efab84, SUC_lexicon_tags_15 },
    { 0xa36fab84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x610fab88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148fab8c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cfab93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe0fab95, SUC_lexicon_tags_11 },
    { 0xb6efab96, SUC_lexicon_tags_43 },
    { 0x190fab97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86afab99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72fab9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x61cfaba0, SUC_lexicon_tags_7 },
    { 0xc28faba0, SUC_lexicon_tags_25 },
    { 0x1eefaba2, SUC_lexicon_tags_1 },
    { 0xf60faba2, SUC_lexicon_tags_68 },
    { 0x3cefaba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa32faba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0afaba9, SUC_lexicon_tags_17 },
    { 0xa76fabaa, SUC_lexicon_tags_12 },
    { 0x9c2fabab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x594fabad, SUC_lexicon_tags_37 },
    { 0x550fabae, SUC_lexicon_tags_11 },
    { 0xd24fabaf, SUC_lexicon_tags_80 },
    { 0x3d2fabad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63cfabb3, SUC_lexicon_tags_47 },
    { 0x460fabb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x54cfabb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff8fabb9, SUC_lexicon_tags_18 },
    { 0x140fabba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd98fabbc, SUC_lexicon_tags_65 },
    { 0xaecfabbd, SUC_lexicon_tags_30 },
    { 0x990fabbc, SUC_lexicon_tags_15 },
    { 0xe5afabbf, SUC_lexicon_tags_2 },
    { 0x50cfabc0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eefabc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cfabc7, SUC_lexicon_tags_25 },
    { 0xac6fabc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa1cfabca, SUC_lexicon_tags_25 },
    { 0x54efabca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4fabcf, SUC_lexicon_tags_27 },
    { 0x8f8fabd0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x336fabd2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5afabd5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x10afabd8, SUC_lexicon_tags_62 },
    { 0x6b6fabd9, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x56efabdb, SUC_lexicon_tags_25 },
    { 0x63cfabdc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cfabe3, SUC_lexicon_tags_25 },
    { 0xaf2fabe3, SUC_lexicon_tags_105 },
    { 0x1b4fabe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcfabf2, SUC_lexicon_tags_7 },
    { 0x1aefabf3, SUC_lexicon_tags_43 },
    { 0xaa2fabf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2fabf8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x84afabfa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x48cfabfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x898fac00, SUC_lexicon_tags_46 },
    { 0xd20fac01, SUC_lexicon_tags_11 },
    { 0x49afac02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256fac0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6fac0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24fac15, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x22fac17, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54fac1c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404fac20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x566fac22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x424fac25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe4fac27, SUC_lexicon_tags_21 },
    { 0x8d8fac27, SUC_lexicon_tags_29 },
    { 0x742fac29, SUC_lexicon_tags_11 },
    { 0x580fac2a, SUC_lexicon_tags_17 },
    { 0xc4cfac2b, SUC_lexicon_tags_37 },
    { 0x8c6fac2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4fac2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80fac35, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcfac39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22fac3d, SUC_lexicon_tags_11 },
    { 0x7b8fac3e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf82fac40, SUC_lexicon_tags_11 },
    { 0xd70fac41, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa9afac43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x322fac46, SUC_lexicon_tags_3 },
    { 0x186fac46, SUC_lexicon_tags_12 },
    { 0x3e6fac48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x79efac4a, SUC_lexicon_tags_37 },
    { 0xc66fac4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0afac4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0efac52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fac55, SUC_lexicon_tags_19 },
    { 0x5a6fac56, SUC_lexicon_tags_38 },
    { 0xf2efac57, SUC_lexicon_tags_37 },
    { 0xe2efac57, SUC_lexicon_tags_7 },
    { 0xcbefac58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0fac5c, SUC_lexicon_tags_11 },
    { 0xa7cfac5d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x38cfac5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86fac62, SUC_lexicon_tags_25 },
    { 0xc2fac63, SUC_lexicon_tags_27 },
    { 0xd3efac64, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4fac67, SUC_lexicon_tags_27 },
    { 0xb94fac67, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x59cfac6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0afac6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa78fac6f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x944fac71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67efac76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952fac7e, SUC_lexicon_tags_19 },
    { 0x20cfac7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30fac85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd1cfac87, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00fac8a, SUC_lexicon_tags_30 },
    { 0xc46fac8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cfac90, SUC_lexicon_tags_17 },
    { 0x3a6fac90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0afac97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a0fac9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x712fac9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78faca1, SUC_lexicon_tags_17 },
    { 0x202faca2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x238faca5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcfaca8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x998facab, SUC_lexicon_tags_26 },
    { 0x54afacac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe66facae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cfacb2, SUC_lexicon_tags_82 },
    { 0xac0facb2, SUC_lexicon_tags_11 },
    { 0x23cfacb4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65efacb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd42facba, SUC_lexicon_tags_49 },
    { 0xcc6facba, SUC_lexicon_tags_0 },
    { 0x8e6facbc, SUC_lexicon_tags_12 },
    { 0xdd6facbd, SUC_lexicon_tags_65 },
    { 0xf1afacbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x86afacc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa24facca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14faccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc10faccf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb6facd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c4facd4, SUC_lexicon_tags_63 },
    { 0x414facd5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x848facd7, SUC_lexicon_tags_44 },
    { 0xa42facd8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cfacdb, SUC_lexicon_tags_37 },
    { 0x2e0facdc, SUC_lexicon_tags_27 },
    { 0x594facdc, SUC_lexicon_tags_15 },
    { 0xf0facde, SUC_lexicon_tags_44 },
    { 0xd2efacdd, SUC_lexicon_tags_12 },
    { 0x708face0, SUC_lexicon_tags_25 },
    { 0xff6face0, SUC_lexicon_tags_2 },
    { 0x354facdf, SUC_lexicon_tags_47 },
    { 0x67eface3, SUC_lexicon_tags_37 },
    { 0xc5aface4, SUC_lexicon_tags_38 },
    { 0x58eface3, SUC_lexicon_tags_49 },
    { 0x9eface2, SUC_lexicon_tags_27 },
    { 0xceefacdc, SUC_lexicon_tags_17 },
    { 0x8faface8, SUC_lexicon_tags_33 },
    { 0xf5cface9, SUC_lexicon_tags_65 },
    { 0xfaaface9, SUC_lexicon_tags_7 },
    { 0x6e8facea, SUC_lexicon_tags_25 },
    { 0xcc0faceb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72afacf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e8facf2, SUC_lexicon_tags_28 },
    { 0x9eefacf2, SUC_lexicon_tags_5 },
    { 0xdc4facf3, SUC_lexicon_tags_0 },
    { 0xb36facf5, SUC_lexicon_tags_6 },
    { 0x75efacf5, SUC_lexicon_tags_25 },
    { 0xb72facf7, SUC_lexicon_tags_21 },
    { 0xce2facf8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79afacfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98facfe, SUC_lexicon_tags_2 },
    { 0xc4facff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4fad02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd6fad04, SUC_lexicon_tags_11 },
    { 0xd02fad04, SUC_lexicon_tags_68 },
    { 0xcb8fad06, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32afad10, SUC_lexicon_tags_14 },
    { 0xa64fad11, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4fad17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18fad1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaafad20, SUC_lexicon_tags_25 },
    { 0x982fad21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12efad23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4fad27, SUC_lexicon_tags_17 },
    { 0x294fad27, SUC_lexicon_tags_13 },
    { 0xfecfad29, SUC_lexicon_tags_113 },
    { 0x612fad2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34fad31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba8fad33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2fad37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4fad40, SUC_lexicon_tags_15 },
    { 0x216fad41, SUC_lexicon_tags_46 },
    { 0x856fad40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aafad47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x936fad4a, SUC_lexicon_tags_13 },
    { 0x9b8fad4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d8fad4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x148fad51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6fad55, SUC_lexicon_tags_47 },
    { 0x670fad55, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8afad59, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbd8fad5b, SUC_lexicon_tags_34 },
    { 0xf8fad5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2fad61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462fad6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x51afad6d, SUC_lexicon_tags_2 },
    { 0xe00fad6d, SUC_lexicon_tags_23 },
    { 0x52fad6f, SUC_lexicon_tags_14 },
    { 0xf98fad6f, SUC_lexicon_tags_37 },
    { 0xf84fad71, SUC_lexicon_tags_37 },
    { 0x4defad70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fad77, SUC_lexicon_tags_17 },
    { 0x97afad77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x118fad7b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x59afad7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe08fad81, SUC_lexicon_tags_15 },
    { 0x536fad82, SUC_lexicon_tags_57 },
    { 0xac2fad83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2fad89, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7efad8c, SUC_lexicon_tags_31 },
    { 0xcb6fad8d, SUC_lexicon_tags_11 },
    { 0xd32fad8c, SUC_lexicon_tags_23 },
    { 0x142fad8f, SUC_lexicon_tags_17 },
    { 0xb8cfad90, SUC_lexicon_tags_25 },
    { 0x766fad8d, SUC_lexicon_tags_15 },
    { 0xfdafad8f, SUC_lexicon_tags_27 },
    { 0x912fad8d, SUC_lexicon_tags_43 },
    { 0xf84fad8e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6fad99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72fad9c, SUC_lexicon_tags_39 },
    { 0xf2efad9d, SUC_lexicon_tags_57 },
    { 0x860fad9c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164fada2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcbafada4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad6fada6, SUC_lexicon_tags_37 },
    { 0x2e8fada7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2fadae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefadb5, SUC_lexicon_tags_11 },
    { 0xe4fadb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcafadb9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74fadbf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xacafadc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476fadca, SUC_lexicon_tags_23 },
    { 0x2a4fadca, SUC_lexicon_tags_45 },
    { 0x586fadcc, SUC_lexicon_tags_11 },
    { 0x8a0fadcc, SUC_lexicon_tags_57 },
    { 0xd8afadce, SUC_lexicon_tags_4 },
    { 0x5a6fadcc, SUC_lexicon_tags_4 },
    { 0xf20fadd0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4e2fadd2, SUC_lexicon_tags_11 },
    { 0xc12fadd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7c2fadd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdefadd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe4fadda, SUC_lexicon_tags_19 },
    { 0x9d0fadda, SUC_lexicon_tags_12 },
    { 0x6f8faddc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcafade0, SUC_lexicon_tags_38 },
    { 0xf3efade0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf4fade3, SUC_lexicon_tags_30 },
    { 0xc6fade4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd28fade7, SUC_lexicon_tags_2 },
    { 0x200fade8, SUC_lexicon_tags_33 },
    { 0x8d4fade8, SUC_lexicon_tags_25 },
    { 0xf5afade9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43afaded, SUC_lexicon_tags_3 },
    { 0x144fadee, SUC_lexicon_tags_17 },
    { 0x102fadef, SUC_lexicon_tags_6 },
    { 0x35efaded, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cefadf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62fadf7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabefadfb, SUC_lexicon_tags_11 },
    { 0xaa4fadfc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e4fae01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab2fae03, SUC_lexicon_tags_82 },
    { 0x266fae04, SUC_lexicon_tags_14 },
    { 0x3b0fae05, SUC_lexicon_tags_10 },
    { 0x4b0fae05, SUC_lexicon_tags_14 },
    { 0xe46fae06, SUC_lexicon_tags_17 },
    { 0xbe4fae05, SUC_lexicon_tags_6 },
    { 0xa22fae06, SUC_lexicon_tags_61 },
    { 0xed0fae0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10fae0e, SUC_lexicon_tags_35 },
    { 0xe16fae0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x202fae12, SUC_lexicon_tags_11 },
    { 0x5ccfae13, SUC_lexicon_tags_8 },
    { 0xf98fae14, SUC_lexicon_tags_0 },
    { 0x6cafae15, SUC_lexicon_tags_11 },
    { 0xe70fae15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3efae1c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2fae1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4aafae21, SUC_lexicon_tags_3 },
    { 0x5eafae21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38fae25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x696fae27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc4afae29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cfae2c, SUC_lexicon_tags_65 },
    { 0xd52fae2c, SUC_lexicon_tags_17 },
    { 0x550fae2d, SUC_lexicon_tags_28 },
    { 0xbbcfae2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe92fae31, SUC_lexicon_tags_17 },
    { 0x53efae31, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x88cfae35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee2fae37, SUC_lexicon_tags_12 },
    { 0x74efae38, SUC_lexicon_tags_65 },
    { 0xbcefae37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8fae3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4fae45, SUC_lexicon_tags_13 },
    { 0xa72fae46, SUC_lexicon_tags_33 },
    { 0x822fae46, SUC_lexicon_tags_7 },
    { 0x4e0fae45, SUC_lexicon_tags_11 },
    { 0x27efae49, SUC_lexicon_tags_23 },
    { 0x310fae46, SUC_lexicon_tags_14 },
    { 0xdccfae4b, SUC_lexicon_tags_174 },
    { 0xb48fae4c, SUC_lexicon_tags_47 },
    { 0xc16fae4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dafae50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x286fae52, SUC_lexicon_tags_13 },
    { 0xcfefae52, SUC_lexicon_tags_35 },
    { 0xb88fae54, SUC_lexicon_tags_15 },
    { 0xcf8fae54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa36fae57, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xd4cfae59, SUC_lexicon_tags_1 },
    { 0x972fae5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66fae5d, SUC_lexicon_tags_38 },
    { 0xd40fae5d, SUC_lexicon_tags_87 },
    { 0xd52fae5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4fae63, SUC_lexicon_tags_60 },
    { 0xd98fae64, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8c4fae66, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eafae72, SUC_lexicon_tags_1 },
    { 0x14efae73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54afae75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0fae7b, SUC_lexicon_tags_43 },
    { 0xd84fae7c, SUC_lexicon_tags_1 },
    { 0x694fae7b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2efae80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58fae83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cfae8c, SUC_lexicon_tags_2 },
    { 0xd64fae8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x282fae8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82afae91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6efae93, SUC_lexicon_tags_37 },
    { 0x9acfae93, SUC_lexicon_tags_1 },
    { 0xc5efae93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3efae98, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8afaea2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0afaea4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x92afaea6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x738faea9, SUC_lexicon_tags_0 },
    { 0xd90faeaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4efaead, SUC_lexicon_tags_63 },
    { 0x6fafaeae, SUC_lexicon_tags_11 },
    { 0xfbcfaeae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2faeb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4faeb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380faeba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdf2faebc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856faec0, SUC_lexicon_tags_37 },
    { 0xc40faec1, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x39cfaec3, SUC_lexicon_tags_11 },
    { 0xc4afaec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cefaec8, SUC_lexicon_tags_1 },
    { 0x5defaec9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75efaecb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcfaece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7afaed0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9befaed2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff0faed4, SUC_lexicon_tags_17 },
    { 0x8efaed5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2faed8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90faedc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26faee3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58faee9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70faef0, SUC_lexicon_tags_37 },
    { 0x174faef0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x292faef4, SUC_lexicon_tags_60 },
    { 0x598faef4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0faef8, SUC_lexicon_tags_17 },
    { 0x504faef8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x946faefb, SUC_lexicon_tags_15 },
    { 0x13cfaefc, SUC_lexicon_tags_8 },
    { 0x37efaefd, SUC_lexicon_tags_2 },
    { 0x17efaefe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8faf03, SUC_lexicon_tags_3 },
    { 0x950faf04, SUC_lexicon_tags_65 },
    { 0xb54faf05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11afaf07, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52faf0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0faf12, SUC_lexicon_tags_63 },
    { 0xbccfaf13, SUC_lexicon_tags_11 },
    { 0xc1cfaf12, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c8faf19, SUC_lexicon_tags_4 },
    { 0xbf0faf1a, SUC_lexicon_tags_48 },
    { 0x288faf1a, SUC_lexicon_tags_32 },
    { 0x35efaf1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cfaf20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0faf23, SUC_lexicon_tags_87 },
    { 0x60afaf23, SUC_lexicon_tags_47 },
    { 0x99cfaf23, SUC_lexicon_tags_43 },
    { 0x840faf25, SUC_lexicon_tags_38 },
    { 0xe86faf27, SUC_lexicon_tags_25 },
    { 0x4e8faf25, SUC_lexicon_tags_7 },
    { 0xd5afaf25, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x194faf2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8faf2e, SUC_lexicon_tags_47 },
    { 0x154faf2f, SUC_lexicon_tags_55 },
    { 0xc86faf2f, SUC_lexicon_tags_23 },
    { 0x416faf30, SUC_lexicon_tags_7 },
    { 0xa92faf32, SUC_lexicon_tags_7 },
    { 0x380faf33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9efaf36, SUC_lexicon_tags_10 },
    { 0x1fafaf36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x82efaf39, SUC_lexicon_tags_28 },
    { 0x466faf3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbafaf3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x734faf3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2faf43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4faf47, SUC_lexicon_tags_37 },
    { 0xb88faf48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2faf4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c0faf55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8faf59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8faf5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf06faf60, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dafaf64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88cfaf66, SUC_lexicon_tags_12 },
    { 0x258faf67, SUC_lexicon_tags_40 },
    { 0xbd6faf68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x294faf6a, SUC_lexicon_tags_644 },
    { 0, NULL },
    { 0xbd0faf6c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x502faf6e, SUC_lexicon_tags_25 },
    { 0x708faf6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8faf72, SUC_lexicon_tags_1 },
    { 0xb5cfaf73, SUC_lexicon_tags_14 },
    { 0xe70faf74, SUC_lexicon_tags_32 },
    { 0x7fcfaf75, SUC_lexicon_tags_15 },
    { 0xa0cfaf75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x724faf79, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50faf81, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x406faf83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x560faf85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeafaf8b, SUC_lexicon_tags_27 },
    { 0xec6faf8c, SUC_lexicon_tags_28 },
    { 0xbdafaf8d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbe6faf8f, SUC_lexicon_tags_2 },
    { 0x472faf90, SUC_lexicon_tags_19 },
    { 0x5eafaf8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38faf95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x758faf97, SUC_lexicon_tags_25 },
    { 0xbb4faf97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8faf9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcfafa1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf60fafa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeafafa6, SUC_lexicon_tags_4 },
    { 0xebcfafa7, SUC_lexicon_tags_56 },
    { 0x494fafa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4fafad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aafafb1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3cafafb3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00fafb6, SUC_lexicon_tags_7 },
    { 0x1c0fafb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca8fafb9, SUC_lexicon_tags_33 },
    { 0xe72fafb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5efafbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a8fafbf, SUC_lexicon_tags_49 },
    { 0x72fafbf, SUC_lexicon_tags_47 },
    { 0x69efafbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x456fafc3, SUC_lexicon_tags_25 },
    { 0x216fafc4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x834fafc7, SUC_lexicon_tags_34 },
    { 0x942fafc8, SUC_lexicon_tags_6 },
    { 0x886fafc9, SUC_lexicon_tags_41 },
    { 0x1aafafca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97afafcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19afafce, SUC_lexicon_tags_43 },
    { 0xfeafafcf, SUC_lexicon_tags_296 },
    { 0x52cfafce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x138fafd2, SUC_lexicon_tags_11 },
    { 0xfacfafd3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1eefafd5, SUC_lexicon_tags_2 },
    { 0x334fafd5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x658fafd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19efafdc, SUC_lexicon_tags_87 },
    { 0x71efafdc, SUC_lexicon_tags_47 },
    { 0xdfcfafde, SUC_lexicon_tags_1 },
    { 0xd5efafdf, SUC_lexicon_tags_11 },
    { 0xbefafe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc94fafe5, SUC_lexicon_tags_37 },
    { 0x6d8fafe6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4defafe9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff6fafeb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a6fafef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b4faff1, SUC_lexicon_tags_15 },
    { 0x26faff1, SUC_lexicon_tags_31 },
    { 0x846faff2, SUC_lexicon_tags_17 },
    { 0xfecfaff4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7defaff6, SUC_lexicon_tags_48 },
    { 0xb50faff7, SUC_lexicon_tags_60 },
    { 0xccfaff8, SUC_lexicon_tags_47 },
    { 0x712faff8, SUC_lexicon_tags_33 },
    { 0xa3afaff8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2faffe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x184fb000, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xdd2fb002, SUC_lexicon_tags_17 },
    { 0xaf8fb003, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78afb005, SUC_lexicon_tags_28 },
    { 0xe60fb006, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1afb00c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x598fb00e, SUC_lexicon_tags_40 },
    { 0xbbcfb00e, SUC_lexicon_tags_0 },
    { 0x2d8fb010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x426fb013, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4fb01e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5bafb020, SUC_lexicon_tags_25 },
    { 0x9eafb021, SUC_lexicon_tags_61 },
    { 0xb32fb021, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54fb025, SUC_lexicon_tags_9 },
    { 0x5cfb026, SUC_lexicon_tags_15 },
    { 0x5f2fb026, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbb2fb029, SUC_lexicon_tags_48 },
    { 0xa08fb02a, SUC_lexicon_tags_11 },
    { 0x3e2fb02b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cfb031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a4fb035, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcfb038, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4fb03c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x616fb03f, SUC_lexicon_tags_2 },
    { 0x4c6fb03f, SUC_lexicon_tags_1 },
    { 0x710fb03f, SUC_lexicon_tags_36 },
    { 0x78afb042, SUC_lexicon_tags_37 },
    { 0xcecfb043, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x384fb04c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6fb053, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24fb056, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x242fb058, SUC_lexicon_tags_17 },
    { 0x83efb059, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc00fb05f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeecfb061, SUC_lexicon_tags_45 },
    { 0x6befb062, SUC_lexicon_tags_17 },
    { 0x3fefb063, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbefb069, SUC_lexicon_tags_59 },
    { 0x62efb06a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccfb073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22cfb076, SUC_lexicon_tags_7 },
    { 0x4bcfb076, SUC_lexicon_tags_37 },
    { 0x23cfb076, SUC_lexicon_tags_1 },
    { 0xdcfb079, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fafb07e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x810fb080, SUC_lexicon_tags_15 },
    { 0x66afb080, SUC_lexicon_tags_68 },
    { 0x8f8fb082, SUC_lexicon_tags_65 },
    { 0x56fb082, SUC_lexicon_tags_11 },
    { 0x1eafb081, SUC_lexicon_tags_15 },
    { 0xb70fb081, SUC_lexicon_tags_15 },
    { 0x5c8fb086, SUC_lexicon_tags_11 },
    { 0x3a4fb087, SUC_lexicon_tags_63 },
    { 0xbcefb088, SUC_lexicon_tags_47 },
    { 0x650fb080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd26fb08c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cfb08f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fefb092, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x61efb094, SUC_lexicon_tags_25 },
    { 0x5fcfb095, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbfefb097, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c2fb099, SUC_lexicon_tags_47 },
    { 0x2c8fb09a, SUC_lexicon_tags_11 },
    { 0xae6fb099, SUC_lexicon_tags_17 },
    { 0xdd6fb099, SUC_lexicon_tags_13 },
    { 0x586fb099, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0fb0a1, SUC_lexicon_tags_15 },
    { 0xb98fb0a1, SUC_lexicon_tags_25 },
    { 0xad8fb0a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172fb0aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944fb0af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x29cfb0b2, SUC_lexicon_tags_45 },
    { 0x596fb0b3, SUC_lexicon_tags_12 },
    { 0xdcafb0b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2fb0b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26efb0b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa56fb0bb, SUC_lexicon_tags_39 },
    { 0xe3efb0bc, SUC_lexicon_tags_39 },
    { 0xffcfb0bd, SUC_lexicon_tags_14 },
    { 0x87cfb0be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x368fb0c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x524fb0c5, SUC_lexicon_tags_2 },
    { 0x3a0fb0c5, SUC_lexicon_tags_0 },
    { 0xcacfb0c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d8fb0c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4afb0cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1afb0ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4cfb0d2, SUC_lexicon_tags_15 },
    { 0x6a2fb0d3, SUC_lexicon_tags_68 },
    { 0x916fb0d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf34fb0d6, SUC_lexicon_tags_11 },
    { 0xf38fb0d7, SUC_lexicon_tags_37 },
    { 0xcf6fb0d8, SUC_lexicon_tags_1 },
    { 0xc82fb0d9, SUC_lexicon_tags_7 },
    { 0x7cefb0da, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x44afb0dc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812fb0ed, SUC_lexicon_tags_25 },
    { 0xb6afb0ee, SUC_lexicon_tags_16 },
    { 0xd68fb0ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefb0f9, SUC_lexicon_tags_2 },
    { 0xf9cfb0f9, SUC_lexicon_tags_31 },
    { 0xb42fb0fa, SUC_lexicon_tags_15 },
    { 0xa70fb0fc, SUC_lexicon_tags_17 },
    { 0x54cfb0fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa00fb104, SUC_lexicon_tags_0 },
    { 0xd8efb104, SUC_lexicon_tags_16 },
    { 0x9cfb106, SUC_lexicon_tags_14 },
    { 0x64efb105, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec0fb109, SUC_lexicon_tags_9 },
    { 0x186fb109, SUC_lexicon_tags_13 },
    { 0x8c6fb109, SUC_lexicon_tags_25 },
    { 0x58afb10b, SUC_lexicon_tags_11 },
    { 0x94fb10d, SUC_lexicon_tags_14 },
    { 0xfdefb10a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x480fb110, SUC_lexicon_tags_19 },
    { 0x3fefb111, SUC_lexicon_tags_11 },
    { 0xb9efb111, SUC_lexicon_tags_7 },
    { 0x51cfb112, SUC_lexicon_tags_7 },
    { 0x9cafb110, SUC_lexicon_tags_2 },
    { 0x756fb114, SUC_lexicon_tags_47 },
    { 0xdb4fb116, SUC_lexicon_tags_15 },
    { 0xc1afb117, SUC_lexicon_tags_29 },
    { 0xf16fb115, SUC_lexicon_tags_87 },
    { 0xb96fb119, SUC_lexicon_tags_15 },
    { 0xf18fb11a, SUC_lexicon_tags_11 },
    { 0xe84fb11a, SUC_lexicon_tags_17 },
    { 0xdf8fb11c, SUC_lexicon_tags_1299 },
    { 0xc16fb11d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4fefb11f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fb122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x386fb125, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfb127, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350fb12b, SUC_lexicon_tags_12 },
    { 0x978fb12c, SUC_lexicon_tags_37 },
    { 0x3defb12d, SUC_lexicon_tags_1 },
    { 0x422fb12d, SUC_lexicon_tags_15 },
    { 0x320fb12d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x726fb131, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6fb137, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8fb13b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63afb13e, SUC_lexicon_tags_15 },
    { 0x4c2fb13f, SUC_lexicon_tags_17 },
    { 0xd72fb13e, SUC_lexicon_tags_37 },
    { 0xbeefb13f, SUC_lexicon_tags_14 },
    { 0xff0fb142, SUC_lexicon_tags_25 },
    { 0x770fb141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7cfb145, SUC_lexicon_tags_63 },
    { 0xe08fb146, SUC_lexicon_tags_25 },
    { 0xa74fb145, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbafb14a, SUC_lexicon_tags_15 },
    { 0x9d8fb14b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456fb151, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x868fb153, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd24fb155, SUC_lexicon_tags_1 },
    { 0x9eefb156, SUC_lexicon_tags_39 },
    { 0x536fb157, SUC_lexicon_tags_55 },
    { 0x8fcfb158, SUC_lexicon_tags_21 },
    { 0xd0cfb159, SUC_lexicon_tags_37 },
    { 0x22efb158, SUC_lexicon_tags_38 },
    { 0x462fb157, SUC_lexicon_tags_38 },
    { 0xcdafb157, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6bafb15e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb80fb16b, SUC_lexicon_tags_17 },
    { 0xd4efb16c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbeafb16e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x976fb170, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc24fb176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694fb17a, SUC_lexicon_tags_30 },
    { 0xb8efb17b, SUC_lexicon_tags_17 },
    { 0xf20fb17c, SUC_lexicon_tags_11 },
    { 0x866fb17c, SUC_lexicon_tags_11 },
    { 0x7cfb17e, SUC_lexicon_tags_35 },
    { 0xa36fb17f, SUC_lexicon_tags_47 },
    { 0x292fb17e, SUC_lexicon_tags_17 },
    { 0xe9efb181, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x156fb184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x806fb187, SUC_lexicon_tags_17 },
    { 0xa2afb187, SUC_lexicon_tags_2 },
    { 0x234fb189, SUC_lexicon_tags_22 },
    { 0xd34fb18a, SUC_lexicon_tags_47 },
    { 0x99efb18b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x756fb18d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe04fb18f, SUC_lexicon_tags_47 },
    { 0xfc8fb190, SUC_lexicon_tags_1 },
    { 0x114fb191, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60fb196, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd4cfb198, SUC_lexicon_tags_37 },
    { 0x854fb199, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9acfb19b, SUC_lexicon_tags_11 },
    { 0xd82fb19b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4fb19f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccfb1a3, SUC_lexicon_tags_25 },
    { 0xe8fb1a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf76fb1a6, SUC_lexicon_tags_25 },
    { 0xccafb1a7, SUC_lexicon_tags_2 },
    { 0x9f6fb1a8, SUC_lexicon_tags_14 },
    { 0x5efb1a8, SUC_lexicon_tags_1 },
    { 0x392fb1aa, SUC_lexicon_tags_10 },
    { 0x2ccfb1ab, SUC_lexicon_tags_63 },
    { 0xabafb1aa, SUC_lexicon_tags_29 },
    { 0x222fb1ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388fb1b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aafb1ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd48fb1bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x760fb1be, SUC_lexicon_tags_28 },
    { 0x6bcfb1bf, SUC_lexicon_tags_11 },
    { 0x732fb1be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aafb1c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22fb1ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xefefb1d1, SUC_lexicon_tags_53 },
    { 0xf86fb1d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5cfb1d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d2fb1d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x666fb1d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x762fb1db, SUC_lexicon_tags_4 },
    { 0xea6fb1dc, SUC_lexicon_tags_38 },
    { 0x502fb1dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x854fb1df, SUC_lexicon_tags_47 },
    { 0x63efb1df, SUC_lexicon_tags_1 },
    { 0x35cfb1e1, SUC_lexicon_tags_2 },
    { 0x496fb1e1, SUC_lexicon_tags_61 },
    { 0x712fb1e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3bcfb1e5, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xa8efb1e7, SUC_lexicon_tags_17 },
    { 0x222fb1e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cfb1f2, SUC_lexicon_tags_38 },
    { 0x2f6fb1f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50fb1fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0fb201, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x996fb204, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x89efb206, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc00fb209, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbafb210, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x7d0fb212, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20fb216, SUC_lexicon_tags_17 },
    { 0xfd8fb216, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0fb21a, SUC_lexicon_tags_27 },
    { 0xe2cfb21b, SUC_lexicon_tags_17 },
    { 0xe4fb21b, SUC_lexicon_tags_2 },
    { 0x3cfb21d, SUC_lexicon_tags_26 },
    { 0xa58fb21d, SUC_lexicon_tags_14 },
    { 0xfd0fb21f, SUC_lexicon_tags_7 },
    { 0xdf6fb220, SUC_lexicon_tags_65 },
    { 0xdf0fb21d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8fb227, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcfb22c, SUC_lexicon_tags_63 },
    { 0x9b4fb22d, SUC_lexicon_tags_11 },
    { 0xa68fb22c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0efb231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x780fb235, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0fb241, SUC_lexicon_tags_25 },
    { 0xf76fb242, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x58cfb245, SUC_lexicon_tags_0 },
    { 0xe4efb245, SUC_lexicon_tags_9 },
    { 0x778fb246, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87efb24e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x838fb250, SUC_lexicon_tags_47 },
    { 0x7c2fb251, SUC_lexicon_tags_49 },
    { 0x1e2fb251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb256, SUC_lexicon_tags_299 },
    { 0x3ecfb257, SUC_lexicon_tags_47 },
    { 0xe0efb257, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45afb25c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56efb25e, SUC_lexicon_tags_14 },
    { 0xcdefb25f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbbefb261, SUC_lexicon_tags_11 },
    { 0xd52fb261, SUC_lexicon_tags_1 },
    { 0xf6fb263, SUC_lexicon_tags_16 },
    { 0xc32fb264, SUC_lexicon_tags_63 },
    { 0x382fb262, SUC_lexicon_tags_7 },
    { 0xe46fb266, SUC_lexicon_tags_45 },
    { 0x128fb267, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e8fb26a, SUC_lexicon_tags_7 },
    { 0xa40fb26b, SUC_lexicon_tags_37 },
    { 0x8f8fb26c, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0fb270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76fb276, SUC_lexicon_tags_2 },
    { 0x29afb277, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x118fb279, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44fb27d, SUC_lexicon_tags_37 },
    { 0xd52fb27e, SUC_lexicon_tags_7 },
    { 0xf08fb27e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2fb283, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5efb28f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcfb294, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868fb298, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xa80fb29b, SUC_lexicon_tags_8 },
    { 0x8b6fb29c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98afb29f, SUC_lexicon_tags_11 },
    { 0xaeefb29f, SUC_lexicon_tags_11 },
    { 0x3f0fb2a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6afb2a5, SUC_lexicon_tags_27 },
    { 0xa26fb2a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefefb2ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd3cfb2ad, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fafb2b2, SUC_lexicon_tags_27 },
    { 0xa06fb2b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cfb2bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac4fb2c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8fb2c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6fb2ca, SUC_lexicon_tags_15 },
    { 0x82cfb2cb, SUC_lexicon_tags_37 },
    { 0x722fb2cb, SUC_lexicon_tags_12 },
    { 0xeeefb2cc, SUC_lexicon_tags_27 },
    { 0xe94fb2ce, SUC_lexicon_tags_1 },
    { 0x388fb2ca, SUC_lexicon_tags_15 },
    { 0x9b2fb2d0, SUC_lexicon_tags_65 },
    { 0xe86fb2d0, SUC_lexicon_tags_0 },
    { 0xf94fb2d1, SUC_lexicon_tags_14 },
    { 0x7acfb2d3, SUC_lexicon_tags_47 },
    { 0xca4fb2d4, SUC_lexicon_tags_37 },
    { 0xe88fb2d5, SUC_lexicon_tags_19 },
    { 0xf94fb2d6, SUC_lexicon_tags_23 },
    { 0x900fb2d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586fb2dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x508fb2df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f2fb2e1, SUC_lexicon_tags_8 },
    { 0x4c0fb2e1, SUC_lexicon_tags_80 },
    { 0xf7efb2e2, SUC_lexicon_tags_7 },
    { 0xdcefb2e4, SUC_lexicon_tags_26 },
    { 0x304fb2e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x348fb2e8, SUC_lexicon_tags_26 },
    { 0x654fb2e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9acfb2ec, SUC_lexicon_tags_25 },
    { 0xee0fb2ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38fb2f2, SUC_lexicon_tags_37 },
    { 0x130fb2f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9efb2f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88afb2fd, SUC_lexicon_tags_7 },
    { 0x616fb2fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x562fb300, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fafb303, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x234fb305, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x908fb307, SUC_lexicon_tags_19 },
    { 0xe7efb307, SUC_lexicon_tags_69 },
    { 0x94fb309, SUC_lexicon_tags_85 },
    { 0x41afb30a, SUC_lexicon_tags_129 },
    { 0x760fb30b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x13afb30e, SUC_lexicon_tags_11 },
    { 0x920fb30f, SUC_lexicon_tags_27 },
    { 0xa48fb310, SUC_lexicon_tags_198 },
    { 0x250fb30f, SUC_lexicon_tags_29 },
    { 0x94cfb30e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x792fb314, SUC_lexicon_tags_63 },
    { 0x1f4fb314, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb0afb317, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x104fb31c, SUC_lexicon_tags_7 },
    { 0x306fb31c, SUC_lexicon_tags_30 },
    { 0x74afb31e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x558fb320, SUC_lexicon_tags_23 },
    { 0x832fb321, SUC_lexicon_tags_37 },
    { 0x91afb322, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff2fb324, SUC_lexicon_tags_29 },
    { 0x3cfb325, SUC_lexicon_tags_7 },
    { 0x814fb325, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372fb32b, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e2fb32e, SUC_lexicon_tags_2 },
    { 0x820fb32e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xefafb331, SUC_lexicon_tags_8 },
    { 0x2e0fb332, SUC_lexicon_tags_2 },
    { 0xb08fb333, SUC_lexicon_tags_1 },
    { 0xf5cfb333, SUC_lexicon_tags_16 },
    { 0x4bafb335, SUC_lexicon_tags_45 },
    { 0xd86fb336, SUC_lexicon_tags_107 },
    { 0xb84fb331, SUC_lexicon_tags_38 },
    { 0x55efb332, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xccafb33b, SUC_lexicon_tags_3 },
    { 0xc40fb33c, SUC_lexicon_tags_47 },
    { 0x1befb33c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fafb345, SUC_lexicon_tags_66 },
    { 0x248fb345, SUC_lexicon_tags_12 },
    { 0x29afb347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fafb34c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00fb351, SUC_lexicon_tags_12 },
    { 0xc84fb351, SUC_lexicon_tags_25 },
    { 0x788fb353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x324fb355, SUC_lexicon_tags_9 },
    { 0x18fb356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41cfb358, SUC_lexicon_tags_9 },
    { 0x30fb359, SUC_lexicon_tags_25 },
    { 0x834fb358, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa8fb35e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc24fb360, SUC_lexicon_tags_65 },
    { 0x19efb361, SUC_lexicon_tags_37 },
    { 0x394fb362, SUC_lexicon_tags_17 },
    { 0x502fb362, SUC_lexicon_tags_0 },
    { 0x90cfb364, SUC_lexicon_tags_25 },
    { 0x2a8fb365, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0fb368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x664fb36b, SUC_lexicon_tags_12 },
    { 0xecfb36b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x814fb36f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd14fb374, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc34fb376, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13cfb37a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37efb37d, SUC_lexicon_tags_77 },
    { 0x214fb37e, SUC_lexicon_tags_31 },
    { 0x294fb37e, SUC_lexicon_tags_31 },
    { 0xa92fb380, SUC_lexicon_tags_1 },
    { 0xcc6fb37e, SUC_lexicon_tags_45 },
    { 0xcb4fb381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea6fb387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eefb393, SUC_lexicon_tags_12 },
    { 0xaa0fb394, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7afb398, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc24fb39a, SUC_lexicon_tags_30 },
    { 0x4b8fb39b, SUC_lexicon_tags_11 },
    { 0x35cfb39c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fb3a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6fb3a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8ccfb3a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3efb3af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8fb3b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x48afb3b4, SUC_lexicon_tags_14 },
    { 0x872fb3b5, SUC_lexicon_tags_26 },
    { 0x2b6fb3b6, SUC_lexicon_tags_6 },
    { 0xbdcfb3b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x712fb3ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd12fb3bc, SUC_lexicon_tags_37 },
    { 0x9c4fb3bd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4c0fb3bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e0fb3c3, SUC_lexicon_tags_63 },
    { 0x346fb3c3, SUC_lexicon_tags_16 },
    { 0x156fb3c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cfb3c8, SUC_lexicon_tags_1 },
    { 0x2aefb3c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0cfb3cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8fb3ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64fb3d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29efb3d2, SUC_lexicon_tags_69 },
    { 0xf7cfb3d2, SUC_lexicon_tags_30 },
    { 0x950fb3d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdefb3e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fb3e7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafb3eb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x64efb3ee, SUC_lexicon_tags_17 },
    { 0xca2fb3ef, SUC_lexicon_tags_56 },
    { 0x99cfb3f0, SUC_lexicon_tags_63 },
    { 0xebefb3f1, SUC_lexicon_tags_7 },
    { 0x4cfb3f2, SUC_lexicon_tags_0 },
    { 0xdeafb3f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98cfb3f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6fb3f9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x260fb3fc, SUC_lexicon_tags_1 },
    { 0xfa4fb3fd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8fb402, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0fb406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efb40e, SUC_lexicon_tags_1 },
    { 0x51cfb40f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90fb41a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x848fb41c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x828fb41f, SUC_lexicon_tags_37 },
    { 0x52fb41f, SUC_lexicon_tags_15 },
    { 0xc7efb41f, SUC_lexicon_tags_21 },
    { 0x5d2fb41f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28efb428, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedefb42e, SUC_lexicon_tags_1 },
    { 0x15afb42f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbccfb431, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60cfb433, SUC_lexicon_tags_37 },
    { 0xacfb434, SUC_lexicon_tags_14 },
    { 0xfe8fb435, SUC_lexicon_tags_43 },
    { 0xb34fb435, SUC_lexicon_tags_79 },
    { 0xceefb437, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2fb43a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x11afb43d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecfb441, SUC_lexicon_tags_19 },
    { 0xe62fb442, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96efb444, SUC_lexicon_tags_13 },
    { 0x89efb444, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20efb44a, SUC_lexicon_tags_30 },
    { 0x9b6fb44b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xab2fb44d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12efb455, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x844fb458, SUC_lexicon_tags_37 },
    { 0x8e2fb459, SUC_lexicon_tags_23 },
    { 0xcd6fb459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeafb45e, SUC_lexicon_tags_11 },
    { 0xdf0fb45f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd92fb469, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74cfb46b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b4fb46d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4efb470, SUC_lexicon_tags_12 },
    { 0xe24fb470, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc40fb473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x592fb476, SUC_lexicon_tags_4 },
    { 0xb64fb477, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedefb479, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c0fb482, SUC_lexicon_tags_27 },
    { 0x2efb482, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94fb488, SUC_lexicon_tags_63 },
    { 0xf40fb488, SUC_lexicon_tags_136 },
    { 0x2f8fb48a, SUC_lexicon_tags_15 },
    { 0xaf4fb48b, SUC_lexicon_tags_45 },
    { 0x8dafb48c, SUC_lexicon_tags_2 },
    { 0x1e0fb48a, SUC_lexicon_tags_12 },
    { 0xb6fb48e, SUC_lexicon_tags_31 },
    { 0x450fb489, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeafb493, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cefb495, SUC_lexicon_tags_17 },
    { 0xd60fb496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecfb499, SUC_lexicon_tags_26 },
    { 0xd92fb499, SUC_lexicon_tags_25 },
    { 0xf98fb49b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30efb49e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60fb4a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57cfb4a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6efb4ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28cfb4af, SUC_lexicon_tags_26 },
    { 0xd02fb4b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ecfb4b3, SUC_lexicon_tags_87 },
    { 0xdefb4b4, SUC_lexicon_tags_49 },
    { 0x228fb4b5, SUC_lexicon_tags_43 },
    { 0xb08fb4b5, SUC_lexicon_tags_2 },
    { 0x6a4fb4b6, SUC_lexicon_tags_15 },
    { 0xdb8fb4b8, SUC_lexicon_tags_111 },
    { 0xca8fb4b9, SUC_lexicon_tags_38 },
    { 0x928fb4ba, SUC_lexicon_tags_37 },
    { 0xb8fb4b7, SUC_lexicon_tags_8 },
    { 0x8efb4bc, SUC_lexicon_tags_30 },
    { 0x42fb4bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590fb4c1, SUC_lexicon_tags_37 },
    { 0x4defb4c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58efb4c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeefb4cb, SUC_lexicon_tags_17 },
    { 0xbc6fb4cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404fb4d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x254fb4d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x976fb4d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aefb4da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fb4e0, SUC_lexicon_tags_37 },
    { 0xe20fb4e1, SUC_lexicon_tags_37 },
    { 0x2e4fb4e0, SUC_lexicon_tags_1 },
    { 0xf18fb4e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5cafb4e5, SUC_lexicon_tags_33 },
    { 0x3c8fb4e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4afb4e8, SUC_lexicon_tags_47 },
    { 0x1cefb4e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42fb4ed, SUC_lexicon_tags_15 },
    { 0xba6fb4ee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fb4f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0efb4f5, SUC_lexicon_tags_25 },
    { 0xc88fb4f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe5efb4f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2fb4fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fb501, SUC_lexicon_tags_25 },
    { 0xdb8fb501, SUC_lexicon_tags_15 },
    { 0x29afb503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x638fb505, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcb0fb507, SUC_lexicon_tags_11 },
    { 0xb12fb508, SUC_lexicon_tags_82 },
    { 0x6cafb508, SUC_lexicon_tags_25 },
    { 0x3fefb508, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134fb50f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x114fb511, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6fb515, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902fb519, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdd8fb51b, SUC_lexicon_tags_10 },
    { 0xceafb51c, SUC_lexicon_tags_1 },
    { 0xe5efb51d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44fb524, SUC_lexicon_tags_16 },
    { 0xd14fb524, SUC_lexicon_tags_14 },
    { 0x72fb524, SUC_lexicon_tags_63 },
    { 0xbb8fb524, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10fb52c, SUC_lexicon_tags_10 },
    { 0xd48fb52d, SUC_lexicon_tags_14 },
    { 0x602fb52e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff2fb530, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32fb534, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68fb53b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x124fb53e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8defb541, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc6fb544, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x57cfb547, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cfb54b, SUC_lexicon_tags_55 },
    { 0x226fb54b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfb54f, SUC_lexicon_tags_151 },
    { 0xf4fb550, SUC_lexicon_tags_17 },
    { 0xc84fb551, SUC_lexicon_tags_10 },
    { 0x8a8fb552, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fb555, SUC_lexicon_tags_37 },
    { 0x53cfb556, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86fb55b, SUC_lexicon_tags_30 },
    { 0x398fb55b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fefb55f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68cfb561, SUC_lexicon_tags_12 },
    { 0xb42fb562, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cfb567, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16fb56b, SUC_lexicon_tags_47 },
    { 0xa5efb56c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xffcfb56e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46fb572, SUC_lexicon_tags_33 },
    { 0x148fb573, SUC_lexicon_tags_8 },
    { 0xd00fb574, SUC_lexicon_tags_13 },
    { 0xe98fb575, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cfb578, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe74fb57a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafb581, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878fb585, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc06fb589, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa54fb58c, SUC_lexicon_tags_2 },
    { 0xa92fb58d, SUC_lexicon_tags_31 },
    { 0xe7afb58e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3acfb591, SUC_lexicon_tags_2 },
    { 0x33afb592, SUC_lexicon_tags_37 },
    { 0x17efb592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58fb597, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6fb59c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcfb59f, SUC_lexicon_tags_14 },
    { 0x2ccfb5a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cfb5a7, SUC_lexicon_tags_12 },
    { 0x11afb5a8, SUC_lexicon_tags_25 },
    { 0x7c8fb5a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x432fb5ac, SUC_lexicon_tags_12 },
    { 0xdb4fb5ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fb5b5, SUC_lexicon_tags_6 },
    { 0x380fb5b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x694fb5bc, SUC_lexicon_tags_15 },
    { 0x310fb5bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35cfb5c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x568fb5c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294fb5c6, SUC_lexicon_tags_43 },
    { 0x9fafb5c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf42fb5c9, SUC_lexicon_tags_63 },
    { 0x366fb5ca, SUC_lexicon_tags_2 },
    { 0xd6efb5ca, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x940fb5cd, SUC_lexicon_tags_15 },
    { 0x3dafb5ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f4fb5d0, SUC_lexicon_tags_11 },
    { 0x812fb5d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2fb5d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acfb5dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1b4fb5de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8efb5e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x254fb5e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0afb5e7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf0efb5e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x128fb5ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcdefb5ef, SUC_lexicon_tags_33 },
    { 0xfcefb5f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94fb5f8, SUC_lexicon_tags_14 },
    { 0x4c0fb5f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fb5fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0fb5ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68fb602, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1efb607, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x164fb60a, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54fb60f, SUC_lexicon_tags_85 },
    { 0xcfcfb60f, SUC_lexicon_tags_11 },
    { 0xa6cfb611, SUC_lexicon_tags_25 },
    { 0xa4fb60f, SUC_lexicon_tags_14 },
    { 0xd44fb612, SUC_lexicon_tags_25 },
    { 0x38afb611, SUC_lexicon_tags_14 },
    { 0x2befb613, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6fb619, SUC_lexicon_tags_46 },
    { 0xad6fb61a, SUC_lexicon_tags_15 },
    { 0x348fb61a, SUC_lexicon_tags_17 },
    { 0x302fb61c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32fb622, SUC_lexicon_tags_15 },
    { 0xc30fb623, SUC_lexicon_tags_38 },
    { 0x84cfb624, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654fb62c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcfb631, SUC_lexicon_tags_15 },
    { 0x4c6fb631, SUC_lexicon_tags_2 },
    { 0x8dafb631, SUC_lexicon_tags_37 },
    { 0x1e2fb631, SUC_lexicon_tags_17 },
    { 0xa5afb635, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4defb637, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536fb640, SUC_lexicon_tags_38 },
    { 0xd8efb640, SUC_lexicon_tags_17 },
    { 0x930fb642, SUC_lexicon_tags_60 },
    { 0xbeafb643, SUC_lexicon_tags_17 },
    { 0xf3afb644, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fb647, SUC_lexicon_tags_16 },
    { 0xb2afb648, SUC_lexicon_tags_18 },
    { 0x77cfb648, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe84fb64e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc86fb650, SUC_lexicon_tags_17 },
    { 0x7defb651, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316fb658, SUC_lexicon_tags_32 },
    { 0xfdafb659, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6fb65c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87efb660, SUC_lexicon_tags_38 },
    { 0xd7cfb661, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21afb669, SUC_lexicon_tags_17 },
    { 0xbacfb66a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x700fb66d, SUC_lexicon_tags_45 },
    { 0x494fb66d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fb675, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x566fb677, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x808fb67b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a4fb67d, SUC_lexicon_tags_37 },
    { 0x6f2fb67d, SUC_lexicon_tags_45 },
    { 0x598fb67f, SUC_lexicon_tags_27 },
    { 0x7aafb680, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x110fb683, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434fb687, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132fb68d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0fb690, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cfb694, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cfb697, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8afb699, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x876fb69b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20fb69e, SUC_lexicon_tags_85 },
    { 0xc06fb69f, SUC_lexicon_tags_45 },
    { 0x368fb6a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32fb6a6, SUC_lexicon_tags_40 },
    { 0x8c0fb6a7, SUC_lexicon_tags_17 },
    { 0xbc8fb6a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cafb6ab, SUC_lexicon_tags_25 },
    { 0xa66fb6ab, SUC_lexicon_tags_0 },
    { 0x58cfb6ad, SUC_lexicon_tags_15 },
    { 0xa46fb6ab, SUC_lexicon_tags_25 },
    { 0xc88fb6ab, SUC_lexicon_tags_7 },
    { 0x91cfb6ac, SUC_lexicon_tags_2 },
    { 0x984fb6b1, SUC_lexicon_tags_0 },
    { 0xb80fb6ae, SUC_lexicon_tags_9 },
    { 0x214fb6b3, SUC_lexicon_tags_37 },
    { 0x4f2fb6b4, SUC_lexicon_tags_35 },
    { 0xf3efb6b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x742fb6bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cefb6bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4afb6c2, SUC_lexicon_tags_17 },
    { 0x722fb6c3, SUC_lexicon_tags_37 },
    { 0xe66fb6c4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xacafb6c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ccfb6ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0fb6d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96afb6d5, SUC_lexicon_tags_25 },
    { 0xd68fb6d5, SUC_lexicon_tags_17 },
    { 0x174fb6d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3efb6dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfefb6e0, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0x708fb6e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cefb6e8, SUC_lexicon_tags_19 },
    { 0x1e2fb6e8, SUC_lexicon_tags_0 },
    { 0x6b4fb6ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x34afb6ec, SUC_lexicon_tags_1 },
    { 0xc80fb6ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x400fb6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a2fb6fd, SUC_lexicon_tags_30 },
    { 0xf76fb6fe, SUC_lexicon_tags_11 },
    { 0xefafb6ff, SUC_lexicon_tags_31 },
    { 0xa6efb6ff, SUC_lexicon_tags_7 },
    { 0xfdefb701, SUC_lexicon_tags_11 },
    { 0x4b6fb701, SUC_lexicon_tags_21 },
    { 0xc32fb703, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30fb706, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4fb70f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x650fb711, SUC_lexicon_tags_31 },
    { 0x58efb711, SUC_lexicon_tags_11 },
    { 0x19efb711, SUC_lexicon_tags_25 },
    { 0xa1efb712, SUC_lexicon_tags_15 },
    { 0x356fb713, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xef2fb717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4fb71d, SUC_lexicon_tags_1 },
    { 0x17efb71e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42fb722, SUC_lexicon_tags_31 },
    { 0x9e0fb722, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfb726, SUC_lexicon_tags_12 },
    { 0xc0cfb727, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd74fb729, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6fb72c, SUC_lexicon_tags_2 },
    { 0x9aafb72d, SUC_lexicon_tags_25 },
    { 0x868fb72d, SUC_lexicon_tags_61 },
    { 0x60fb72f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x292fb731, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aefb737, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcfb73a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x304fb73c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xaa6fb73e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940fb743, SUC_lexicon_tags_28 },
    { 0xa30fb744, SUC_lexicon_tags_43 },
    { 0x146fb745, SUC_lexicon_tags_2 },
    { 0x48efb746, SUC_lexicon_tags_12 },
    { 0x25afb743, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0fb74e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc76fb750, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cfb754, SUC_lexicon_tags_5 },
    { 0x76fb755, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x818fb758, SUC_lexicon_tags_17 },
    { 0xa6cfb759, SUC_lexicon_tags_16 },
    { 0xfc6fb75a, SUC_lexicon_tags_45 },
    { 0xbe4fb758, SUC_lexicon_tags_11 },
    { 0x8c0fb75c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0cfb75e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fb761, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cfb765, SUC_lexicon_tags_583 },
    { 0xaeafb765, SUC_lexicon_tags_11 },
    { 0x56cfb767, SUC_lexicon_tags_63 },
    { 0x68fb768, SUC_lexicon_tags_2 },
    { 0xa44fb767, SUC_lexicon_tags_23 },
    { 0x616fb76a, SUC_lexicon_tags_8 },
    { 0x24afb76b, SUC_lexicon_tags_16 },
    { 0x52fb76a, SUC_lexicon_tags_37 },
    { 0x83efb76c, SUC_lexicon_tags_80 },
    { 0x98efb76d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93afb772, SUC_lexicon_tags_37 },
    { 0x45cfb772, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cfb778, SUC_lexicon_tags_12 },
    { 0x6aafb779, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf26fb77d, SUC_lexicon_tags_11 },
    { 0x4a6fb77e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x810fb780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f0fb782, SUC_lexicon_tags_17 },
    { 0x92cfb783, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6fb786, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x212fb789, SUC_lexicon_tags_17 },
    { 0x66cfb789, SUC_lexicon_tags_87 },
    { 0x80afb78a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe28fb78d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x448fb790, SUC_lexicon_tags_2 },
    { 0x634fb791, SUC_lexicon_tags_27 },
    { 0x774fb791, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6eafb794, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a0fb796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67afb798, SUC_lexicon_tags_2 },
    { 0x522fb799, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11efb79d, SUC_lexicon_tags_1 },
    { 0xc94fb79e, SUC_lexicon_tags_17 },
    { 0xed8fb79e, SUC_lexicon_tags_208 },
    { 0x972fb7a0, SUC_lexicon_tags_28 },
    { 0x8e0fb79e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x320fb7a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92efb7a6, SUC_lexicon_tags_66 },
    { 0xe84fb7a7, SUC_lexicon_tags_8 },
    { 0x3bafb7a7, SUC_lexicon_tags_11 },
    { 0x9cafb7a9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xfd8fb7ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5efb7ad, SUC_lexicon_tags_2 },
    { 0x754fb7ae, SUC_lexicon_tags_38 },
    { 0x1f4fb7ae, SUC_lexicon_tags_84 },
    { 0x1fefb7b0, SUC_lexicon_tags_49 },
    { 0x802fb7b0, SUC_lexicon_tags_1 },
    { 0x248fb7b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0fb7b6, SUC_lexicon_tags_310 },
    { 0x478fb7b7, SUC_lexicon_tags_39 },
    { 0xf1efb7b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x56fb7bb, SUC_lexicon_tags_7 },
    { 0xc4afb7bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fb7c0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fb7c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5efb7c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30fb7c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4afb7cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x474fb7d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2fb7d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4fb7d8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa04fb7db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x832fb7de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x802fb7e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cafb7ea, SUC_lexicon_tags_29 },
    { 0xcbafb7ea, SUC_lexicon_tags_57 },
    { 0x270fb7ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112fb7f3, SUC_lexicon_tags_38 },
    { 0x582fb7f4, SUC_lexicon_tags_37 },
    { 0x5e0fb7f5, SUC_lexicon_tags_33 },
    { 0x32cfb7f6, SUC_lexicon_tags_7 },
    { 0x74efb7f7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x916fb7fa, SUC_lexicon_tags_7 },
    { 0xe4efb7fb, SUC_lexicon_tags_63 },
    { 0x262fb7fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x984fb801, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x586fb806, SUC_lexicon_tags_17 },
    { 0xad4fb806, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0afb80a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fafb80c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fefb810, SUC_lexicon_tags_15 },
    { 0x9f4fb810, SUC_lexicon_tags_12 },
    { 0x7aafb812, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c2fb815, SUC_lexicon_tags_7 },
    { 0x980fb815, SUC_lexicon_tags_15 },
    { 0xcdefb817, SUC_lexicon_tags_25 },
    { 0x9b0fb818, SUC_lexicon_tags_15 },
    { 0xeaefb815, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6afb81b, SUC_lexicon_tags_47 },
    { 0x22fb81c, SUC_lexicon_tags_15 },
    { 0x8d8fb81d, SUC_lexicon_tags_37 },
    { 0x5b8fb81e, SUC_lexicon_tags_30 },
    { 0xe22fb81f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb4fb821, SUC_lexicon_tags_37 },
    { 0xe2cfb822, SUC_lexicon_tags_158 },
    { 0xb54fb822, SUC_lexicon_tags_13 },
    { 0x4b0fb824, SUC_lexicon_tags_34 },
    { 0x242fb825, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3efb829, SUC_lexicon_tags_37 },
    { 0xd20fb82a, SUC_lexicon_tags_106 },
    { 0x99efb82b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcfb831, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912fb835, SUC_lexicon_tags_0 },
    { 0x446fb836, SUC_lexicon_tags_37 },
    { 0x128fb836, SUC_lexicon_tags_47 },
    { 0xaefb838, SUC_lexicon_tags_16 },
    { 0x12afb836, SUC_lexicon_tags_11 },
    { 0xf52fb839, SUC_lexicon_tags_33 },
    { 0x1aafb83b, SUC_lexicon_tags_14 },
    { 0xdc0fb83b, SUC_lexicon_tags_25 },
    { 0x3b0fb836, SUC_lexicon_tags_57 },
    { 0x4ecfb837, SUC_lexicon_tags_6 },
    { 0x6b8fb83f, SUC_lexicon_tags_3 },
    { 0x3bafb840, SUC_lexicon_tags_9 },
    { 0x624fb83f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x156fb843, SUC_lexicon_tags_7 },
    { 0x56cfb844, SUC_lexicon_tags_38 },
    { 0xb32fb845, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcacfb847, SUC_lexicon_tags_11 },
    { 0x168fb848, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0x28fb84a, SUC_lexicon_tags_47 },
    { 0x2afb84b, SUC_lexicon_tags_11 },
    { 0x2a8fb84a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcfb850, SUC_lexicon_tags_12 },
    { 0xe52fb851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba6fb853, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900fb85a, SUC_lexicon_tags_14 },
    { 0x18cfb85b, SUC_lexicon_tags_14 },
    { 0x84afb85a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c6fb85e, SUC_lexicon_tags_17 },
    { 0x912fb85f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7fefb861, SUC_lexicon_tags_8 },
    { 0x802fb861, SUC_lexicon_tags_17 },
    { 0xe7afb863, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900fb867, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16efb869, SUC_lexicon_tags_70 },
    { 0xa56fb86a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfefb86d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7efb870, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240fb87b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2befb87f, SUC_lexicon_tags_7 },
    { 0x728fb880, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fafb885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72fb888, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52cfb88a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97efb88f, SUC_lexicon_tags_98 },
    { 0x6c4fb890, SUC_lexicon_tags_17 },
    { 0x66fb891, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0fb895, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xddafb898, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12fb8a5, SUC_lexicon_tags_6 },
    { 0x14fb8a6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd16fb8a9, SUC_lexicon_tags_5 },
    { 0xa5efb8a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeb6fb8ac, SUC_lexicon_tags_25 },
    { 0x834fb8ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4fb8b2, SUC_lexicon_tags_17 },
    { 0x9efb8b2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926fb8b8, SUC_lexicon_tags_17 },
    { 0xe40fb8b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x302fb8bb, SUC_lexicon_tags_17 },
    { 0x7defb8bc, SUC_lexicon_tags_77 },
    { 0x2e0fb8bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe64fb8c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaafb8cb, SUC_lexicon_tags_47 },
    { 0x924fb8cb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cfb8d1, SUC_lexicon_tags_8 },
    { 0xdd4fb8d2, SUC_lexicon_tags_12 },
    { 0x826fb8d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaefb8d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec4fb8db, SUC_lexicon_tags_38 },
    { 0xfcfb8dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14efb8e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x726fb8ea, SUC_lexicon_tags_11 },
    { 0x492fb8ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4efb8f3, SUC_lexicon_tags_0 },
    { 0xd20fb8f3, SUC_lexicon_tags_87 },
    { 0x2a0fb8f3, SUC_lexicon_tags_11 },
    { 0x386fb8f6, SUC_lexicon_tags_48 },
    { 0xb8cfb8f7, SUC_lexicon_tags_60 },
    { 0xd5afb8f7, SUC_lexicon_tags_43 },
    { 0x3aafb8f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x846fb8fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0fb901, SUC_lexicon_tags_30 },
    { 0x474fb901, SUC_lexicon_tags_2 },
    { 0x184fb903, SUC_lexicon_tags_23 },
    { 0x594fb903, SUC_lexicon_tags_8 },
    { 0x2ecfb905, SUC_lexicon_tags_23 },
    { 0xc1cfb906, SUC_lexicon_tags_0 },
    { 0x40afb906, SUC_lexicon_tags_1 },
    { 0x30efb908, SUC_lexicon_tags_1 },
    { 0xc18fb909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffefb90b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde6fb90f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72fb914, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68fb918, SUC_lexicon_tags_15 },
    { 0xe48fb919, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0fb925, SUC_lexicon_tags_25 },
    { 0x4d8fb926, SUC_lexicon_tags_37 },
    { 0x5f4fb927, SUC_lexicon_tags_2 },
    { 0x502fb928, SUC_lexicon_tags_60 },
    { 0xb36fb927, SUC_lexicon_tags_45 },
    { 0xe22fb92a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cafb92c, SUC_lexicon_tags_2 },
    { 0xaf8fb92c, SUC_lexicon_tags_6 },
    { 0x746fb92e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc60fb931, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x902fb938, SUC_lexicon_tags_25 },
    { 0x9e6fb939, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x520fb93c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe6afb93e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26fb941, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8fb944, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6fb948, SUC_lexicon_tags_11 },
    { 0xa8efb948, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eafb94f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe56fb951, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78fb954, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396fb959, SUC_lexicon_tags_30 },
    { 0x11efb95a, SUC_lexicon_tags_9 },
    { 0x556fb95a, SUC_lexicon_tags_27 },
    { 0x496fb95c, SUC_lexicon_tags_7 },
    { 0xd74fb95b, SUC_lexicon_tags_27 },
    { 0x774fb95a, SUC_lexicon_tags_1716 },
    { 0, NULL },
    { 0, NULL },
    { 0x854fb961, SUC_lexicon_tags_60 },
    { 0xfefb961, SUC_lexicon_tags_103 },
    { 0x8d0fb963, SUC_lexicon_tags_8 },
    { 0xb36fb964, SUC_lexicon_tags_10 },
    { 0xb14fb961, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecfb969, SUC_lexicon_tags_11 },
    { 0x998fb969, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xf20fb96c, SUC_lexicon_tags_17 },
    { 0xc9efb96d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44afb972, SUC_lexicon_tags_37 },
    { 0x4e0fb973, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd66fb976, SUC_lexicon_tags_47 },
    { 0x846fb977, SUC_lexicon_tags_25 },
    { 0x164fb978, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8fb980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4fb982, SUC_lexicon_tags_55 },
    { 0x446fb983, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eafb987, SUC_lexicon_tags_11 },
    { 0xea4fb988, SUC_lexicon_tags_12 },
    { 0xb40fb989, SUC_lexicon_tags_11 },
    { 0x334fb98a, SUC_lexicon_tags_45 },
    { 0xaf4fb98a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfb98e, SUC_lexicon_tags_14 },
    { 0x9eafb98e, SUC_lexicon_tags_25 },
    { 0x824fb98f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fb996, SUC_lexicon_tags_31 },
    { 0x92afb997, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xebefb99a, SUC_lexicon_tags_9 },
    { 0xb78fb99b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa52fb99e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4fb9a5, SUC_lexicon_tags_15 },
    { 0x140fb9a5, SUC_lexicon_tags_11 },
    { 0x8b6fb9a7, SUC_lexicon_tags_15 },
    { 0x7dafb9a8, SUC_lexicon_tags_4 },
    { 0x980fb9a6, SUC_lexicon_tags_64 },
    { 0x348fb9aa, SUC_lexicon_tags_17 },
    { 0x5b0fb9aa, SUC_lexicon_tags_94 },
    { 0x99cfb9ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccafb9b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94fb9b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2afb9b6, SUC_lexicon_tags_17 },
    { 0x3f8fb9b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d2fb9ba, SUC_lexicon_tags_37 },
    { 0xb42fb9bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f0fb9bd, SUC_lexicon_tags_30 },
    { 0xc48fb9bd, SUC_lexicon_tags_1 },
    { 0xfecfb9bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35afb9c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddafb9c3, SUC_lexicon_tags_15 },
    { 0xb5cfb9c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cefb9c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a2fb9ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0fb9cd, SUC_lexicon_tags_0 },
    { 0x878fb9cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8cfb9d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4fb9d7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1fafb9d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dcfb9db, SUC_lexicon_tags_37 },
    { 0x74fb9db, SUC_lexicon_tags_11 },
    { 0xc8fb9dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fb9e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46efb9e7, SUC_lexicon_tags_46 },
    { 0x1d0fb9e8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48fb9ec, SUC_lexicon_tags_33 },
    { 0xd3efb9ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x720fb9ef, SUC_lexicon_tags_15 },
    { 0xb70fb9f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250fb9f4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xdd4fb9f6, SUC_lexicon_tags_99 },
    { 0x47efb9f7, SUC_lexicon_tags_49 },
    { 0xdf2fb9f6, SUC_lexicon_tags_17 },
    { 0xe5afb9f8, SUC_lexicon_tags_12 },
    { 0xf6cfb9fa, SUC_lexicon_tags_63 },
    { 0x660fb9fa, SUC_lexicon_tags_37 },
    { 0x23afb9fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2acfba02, SUC_lexicon_tags_25 },
    { 0x372fba03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fba0c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32fba13, SUC_lexicon_tags_37 },
    { 0x810fba14, SUC_lexicon_tags_37 },
    { 0xc10fba15, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x186fba17, SUC_lexicon_tags_29 },
    { 0xd0fba17, SUC_lexicon_tags_15 },
    { 0xf42fba19, SUC_lexicon_tags_15 },
    { 0x728fba18, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04fba21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fba26, SUC_lexicon_tags_1 },
    { 0xac8fba27, SUC_lexicon_tags_15 },
    { 0xf74fba28, SUC_lexicon_tags_12 },
    { 0x480fba29, SUC_lexicon_tags_10 },
    { 0x4e8fba2a, SUC_lexicon_tags_19 },
    { 0x4c6fba28, SUC_lexicon_tags_37 },
    { 0x1b6fba2c, SUC_lexicon_tags_84 },
    { 0xe30fba2c, SUC_lexicon_tags_11 },
    { 0x70fba2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52fba33, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fafba38, SUC_lexicon_tags_2 },
    { 0xa8cfba38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64efba3b, SUC_lexicon_tags_11 },
    { 0xfe6fba3c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xe0cfba3e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02fba42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf48fba44, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x828fba49, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb46fba4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98efba4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290fba52, SUC_lexicon_tags_15 },
    { 0xffcfba52, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6fba58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd42fba5d, SUC_lexicon_tags_29 },
    { 0x358fba5d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x90afba60, SUC_lexicon_tags_12 },
    { 0xfb6fba61, SUC_lexicon_tags_30 },
    { 0xb2fba61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52fba69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6fba6e, SUC_lexicon_tags_2 },
    { 0xa50fba6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x70fba71, SUC_lexicon_tags_47 },
    { 0x846fba72, SUC_lexicon_tags_11 },
    { 0x740fba71, SUC_lexicon_tags_23 },
    { 0x96efba74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x534fba76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4fba78, SUC_lexicon_tags_0 },
    { 0x6a2fba79, SUC_lexicon_tags_47 },
    { 0x28cfba79, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x74fba7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884fba84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cfba89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97cfba8e, SUC_lexicon_tags_17 },
    { 0x5ccfba8f, SUC_lexicon_tags_1 },
    { 0x17afba8f, SUC_lexicon_tags_27 },
    { 0x166fba91, SUC_lexicon_tags_27 },
    { 0x31afba92, SUC_lexicon_tags_37 },
    { 0x216fba90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50fba96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cfba9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd14fba9c, SUC_lexicon_tags_0 },
    { 0xee0fba9d, SUC_lexicon_tags_5 },
    { 0x7b2fba9d, SUC_lexicon_tags_12 },
    { 0xc5afba9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68fbaa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e0fbaa4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x43afbaa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894fbab3, SUC_lexicon_tags_84 },
    { 0x758fbab4, SUC_lexicon_tags_32 },
    { 0x760fbab5, SUC_lexicon_tags_1 },
    { 0x7afbab5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386fbaba, SUC_lexicon_tags_11 },
    { 0xa6fbaba, SUC_lexicon_tags_27 },
    { 0x434fbaba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x262fbabe, SUC_lexicon_tags_30 },
    { 0x990fbabe, SUC_lexicon_tags_30 },
    { 0xe16fbabf, SUC_lexicon_tags_31 },
    { 0xd0afbac1, SUC_lexicon_tags_25 },
    { 0xe46fbac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2fbac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65cfbac7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x92efbac9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa72fbacc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29afbacf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62efbad2, SUC_lexicon_tags_108 },
    { 0xaa2fbad3, SUC_lexicon_tags_45 },
    { 0xe82fbad3, SUC_lexicon_tags_17 },
    { 0x2f2fbad5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274fbad9, SUC_lexicon_tags_2 },
    { 0x67afbad9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b0fbadc, SUC_lexicon_tags_1 },
    { 0xf4efbadd, SUC_lexicon_tags_12 },
    { 0x702fbade, SUC_lexicon_tags_66 },
    { 0xe96fbade, SUC_lexicon_tags_37 },
    { 0xe1afbae0, SUC_lexicon_tags_37 },
    { 0x6f6fbadc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7afbae3, SUC_lexicon_tags_17 },
    { 0xba8fbae3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfeafbae6, SUC_lexicon_tags_37 },
    { 0xe2fbae7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46afbaeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cfbaee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258fbaf7, SUC_lexicon_tags_4 },
    { 0x79afbaf8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x938fbafb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x180fbafd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8fbb07, SUC_lexicon_tags_12 },
    { 0x3efbb08, SUC_lexicon_tags_1 },
    { 0xc6efbb09, SUC_lexicon_tags_2 },
    { 0x892fbb09, SUC_lexicon_tags_1 },
    { 0x8c6fbb0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x402fbb0e, SUC_lexicon_tags_25 },
    { 0xdccfbb0f, SUC_lexicon_tags_30 },
    { 0x5c8fbb0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92fbb13, SUC_lexicon_tags_27 },
    { 0x41afbb14, SUC_lexicon_tags_95 },
    { 0x604fbb14, SUC_lexicon_tags_37 },
    { 0x2d4fbb16, SUC_lexicon_tags_23 },
    { 0x2f4fbb17, SUC_lexicon_tags_32 },
    { 0x9eafbb17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa94fbb1a, SUC_lexicon_tags_33 },
    { 0x220fbb1b, SUC_lexicon_tags_32 },
    { 0x754fbb1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf22fbb20, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x130fbb23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefbb25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc44fbb27, SUC_lexicon_tags_63 },
    { 0x64fbb28, SUC_lexicon_tags_18 },
    { 0x7fcfbb29, SUC_lexicon_tags_207 },
    { 0x940fbb2a, SUC_lexicon_tags_37 },
    { 0x35efbb29, SUC_lexicon_tags_31 },
    { 0x9d2fbb2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4fbb35, SUC_lexicon_tags_25 },
    { 0xa62fbb36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0fbb39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0fbb41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4fbb44, SUC_lexicon_tags_31 },
    { 0xcaefbb45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x57afbb48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2efbb4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4fbb4f, SUC_lexicon_tags_37 },
    { 0x530fbb50, SUC_lexicon_tags_29 },
    { 0x6eafbb51, SUC_lexicon_tags_11 },
    { 0xa92fbb52, SUC_lexicon_tags_12 },
    { 0xed4fbb51, SUC_lexicon_tags_15 },
    { 0x65cfbb54, SUC_lexicon_tags_630 },
    { 0x6fbb55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc9cfbb57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x852fbb59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b4fbb5d, SUC_lexicon_tags_15 },
    { 0xbb2fbb5e, SUC_lexicon_tags_60 },
    { 0x806fbb5f, SUC_lexicon_tags_13 },
    { 0x58afbb5e, SUC_lexicon_tags_28 },
    { 0x7befbb61, SUC_lexicon_tags_23 },
    { 0x70afbb5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce0fbb67, SUC_lexicon_tags_23 },
    { 0xb54fbb67, SUC_lexicon_tags_54 },
    { 0x334fbb69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8fbb70, SUC_lexicon_tags_46 },
    { 0xd0cfbb70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3cafbb73, SUC_lexicon_tags_16 },
    { 0xca0fbb73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eafbb77, SUC_lexicon_tags_16 },
    { 0x154fbb78, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2d0fbb7a, SUC_lexicon_tags_3 },
    { 0xf14fbb7a, SUC_lexicon_tags_11 },
    { 0x6ecfbb7c, SUC_lexicon_tags_43 },
    { 0xfd4fbb7c, SUC_lexicon_tags_6 },
    { 0xadafbb7d, SUC_lexicon_tags_15 },
    { 0xcc8fbb7a, SUC_lexicon_tags_11 },
    { 0xb42fbb80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cafbb87, SUC_lexicon_tags_8 },
    { 0xfbb87, SUC_lexicon_tags_25 },
    { 0x246fbb87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56fbb8c, SUC_lexicon_tags_2 },
    { 0x202fbb8d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226fbb91, SUC_lexicon_tags_15 },
    { 0x2c8fbb92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8afbb96, SUC_lexicon_tags_12 },
    { 0x826fbb97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x504fbb99, SUC_lexicon_tags_25 },
    { 0x9c2fbb99, SUC_lexicon_tags_26 },
    { 0xac8fbb9b, SUC_lexicon_tags_37 },
    { 0x51afbb9b, SUC_lexicon_tags_14 },
    { 0x37afbb9b, SUC_lexicon_tags_60 },
    { 0x5d8fbb9b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92fbba3, SUC_lexicon_tags_17 },
    { 0x198fbba4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4fbbab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe24fbbad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0fbbb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cafbbb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc48fbbba, SUC_lexicon_tags_15 },
    { 0xb2afbbbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0fbbbf, SUC_lexicon_tags_1 },
    { 0x36cfbbc0, SUC_lexicon_tags_15 },
    { 0x30cfbbbf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x754fbbc4, SUC_lexicon_tags_30 },
    { 0xa06fbbc5, SUC_lexicon_tags_45 },
    { 0x652fbbc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242fbbca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cafbbce, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cfbbd3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49afbbdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472fbbdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcfbbea, SUC_lexicon_tags_87 },
    { 0x156fbbeb, SUC_lexicon_tags_43 },
    { 0x748fbbec, SUC_lexicon_tags_37 },
    { 0xb30fbbed, SUC_lexicon_tags_27 },
    { 0x18fbbee, SUC_lexicon_tags_46 },
    { 0x886fbbef, SUC_lexicon_tags_25 },
    { 0x742fbbf0, SUC_lexicon_tags_63 },
    { 0xfd4fbbf1, SUC_lexicon_tags_49 },
    { 0x8c6fbbec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd1afbbf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0fbbf8, SUC_lexicon_tags_7 },
    { 0xb46fbbf9, SUC_lexicon_tags_11 },
    { 0xdccfbbfa, SUC_lexicon_tags_11 },
    { 0xfcefbbfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fbc03, SUC_lexicon_tags_6 },
    { 0xdb6fbc04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69cfbc06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x330fbc08, SUC_lexicon_tags_14 },
    { 0x7f6fbc08, SUC_lexicon_tags_11 },
    { 0x9a4fbc0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fbc0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x602fbc0f, SUC_lexicon_tags_33 },
    { 0x624fbc10, SUC_lexicon_tags_48 },
    { 0xedafbc0f, SUC_lexicon_tags_8 },
    { 0x322fbc12, SUC_lexicon_tags_15 },
    { 0x480fbc10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8efbc16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x88fbc18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3afbc1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fafbc1c, SUC_lexicon_tags_15 },
    { 0x18cfbc1d, SUC_lexicon_tags_25 },
    { 0x560fbc1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfbc22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4eefbc24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeefbc27, SUC_lexicon_tags_7 },
    { 0x10cfbc28, SUC_lexicon_tags_10 },
    { 0xa82fbc29, SUC_lexicon_tags_68 },
    { 0x55cfbc2a, SUC_lexicon_tags_17 },
    { 0x7befbc2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236fbc32, SUC_lexicon_tags_15 },
    { 0xc32fbc32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefbc36, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2fbc3a, SUC_lexicon_tags_7 },
    { 0x46fbc3b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8fbc42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1afbc44, SUC_lexicon_tags_63 },
    { 0xbd8fbc45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88afbc47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x268fbc49, SUC_lexicon_tags_21 },
    { 0xab6fbc4a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4fbc4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeeefbc51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584fbc57, SUC_lexicon_tags_0 },
    { 0x9f4fbc58, SUC_lexicon_tags_12 },
    { 0x1cefbc58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x496fbc5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cfbc69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fbc6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1afbc6e, SUC_lexicon_tags_1 },
    { 0x628fbc6e, SUC_lexicon_tags_7 },
    { 0x714fbc70, SUC_lexicon_tags_11 },
    { 0xee0fbc6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf74fbc74, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04fbc78, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cfbc7e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66cfbc87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x906fbc8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x92fbc8c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8efbc90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa36fbc92, SUC_lexicon_tags_14 },
    { 0xb92fbc93, SUC_lexicon_tags_17 },
    { 0xc4cfbc94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbafbc97, SUC_lexicon_tags_25 },
    { 0x52cfbc98, SUC_lexicon_tags_37 },
    { 0x96cfbc99, SUC_lexicon_tags_28 },
    { 0x336fbc9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fbca1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7bafbca3, SUC_lexicon_tags_45 },
    { 0xb2efbca4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236fbca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5cfbcab, SUC_lexicon_tags_6 },
    { 0xf94fbcac, SUC_lexicon_tags_32 },
    { 0xd60fbcab, SUC_lexicon_tags_4 },
    { 0x8acfbcae, SUC_lexicon_tags_25 },
    { 0xbb8fbcad, SUC_lexicon_tags_11 },
    { 0x646fbcae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94cfbcb2, SUC_lexicon_tags_7 },
    { 0xad8fbcb3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fbcb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b6fbcb8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6fbcbe, SUC_lexicon_tags_17 },
    { 0xeb4fbcbf, SUC_lexicon_tags_37 },
    { 0xe98fbcc0, SUC_lexicon_tags_1 },
    { 0x30efbcbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60efbcc4, SUC_lexicon_tags_47 },
    { 0xd1efbcc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82fbcc7, SUC_lexicon_tags_37 },
    { 0x20afbcc7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xda4fbcca, SUC_lexicon_tags_7 },
    { 0xa54fbccb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a6fbcce, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x300fbcd1, SUC_lexicon_tags_63 },
    { 0x40fbcd2, SUC_lexicon_tags_7 },
    { 0x664fbcd3, SUC_lexicon_tags_85 },
    { 0xab0fbcd4, SUC_lexicon_tags_9 },
    { 0xcf2fbcd5, SUC_lexicon_tags_38 },
    { 0xbf4fbcd4, SUC_lexicon_tags_2 },
    { 0xec0fbcd7, SUC_lexicon_tags_23 },
    { 0x856fbcd2, SUC_lexicon_tags_17 },
    { 0xf58fbcd7, SUC_lexicon_tags_22 },
    { 0x146fbcd9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0afbcde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41efbce0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8fbce4, SUC_lexicon_tags_17 },
    { 0x5b2fbce5, SUC_lexicon_tags_1 },
    { 0x826fbce4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41cfbce8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40afbcf6, SUC_lexicon_tags_1 },
    { 0xc8cfbcf6, SUC_lexicon_tags_37 },
    { 0x940fbcf8, SUC_lexicon_tags_47 },
    { 0xa02fbcf6, SUC_lexicon_tags_0 },
    { 0xd7cfbcfa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8fbcfe, SUC_lexicon_tags_2 },
    { 0x32cfbcff, SUC_lexicon_tags_15 },
    { 0xc48fbd00, SUC_lexicon_tags_32 },
    { 0xa34fbd01, SUC_lexicon_tags_66 },
    { 0xad0fbd02, SUC_lexicon_tags_47 },
    { 0xb02fbd00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x404fbd06, SUC_lexicon_tags_33 },
    { 0x40cfbd06, SUC_lexicon_tags_47 },
    { 0x156fbd06, SUC_lexicon_tags_17 },
    { 0xd8afbd08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0fbd0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82afbd10, SUC_lexicon_tags_12 },
    { 0x3fafbd11, SUC_lexicon_tags_37 },
    { 0x4bafbd10, SUC_lexicon_tags_8 },
    { 0xcacfbd12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa56fbd16, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8fbd1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17efbd1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6afbd20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ccfbd22, SUC_lexicon_tags_17 },
    { 0xa68fbd23, SUC_lexicon_tags_17 },
    { 0x36afbd24, SUC_lexicon_tags_25 },
    { 0x98efbd25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fafbd27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98fbd2a, SUC_lexicon_tags_792 },
    { 0x4dcfbd2a, SUC_lexicon_tags_81 },
    { 0xa94fbd2a, SUC_lexicon_tags_40 },
    { 0xd22fbd2d, SUC_lexicon_tags_77 },
    { 0x740fbd2d, SUC_lexicon_tags_37 },
    { 0x4b0fbd2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ccfbd31, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cfbd36, SUC_lexicon_tags_9 },
    { 0x9a6fbd37, SUC_lexicon_tags_63 },
    { 0x11afbd38, SUC_lexicon_tags_2 },
    { 0xc7afbd38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x748fbd3c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcfbd40, SUC_lexicon_tags_15 },
    { 0xea6fbd41, SUC_lexicon_tags_14 },
    { 0x668fbd42, SUC_lexicon_tags_29 },
    { 0x76afbd43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa66fbd48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd64fbd4a, SUC_lexicon_tags_25 },
    { 0xeacfbd4b, SUC_lexicon_tags_9 },
    { 0xed0fbd4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfbd4f, SUC_lexicon_tags_17 },
    { 0xf68fbd50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfafbd55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x234fbd57, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4b2fbd59, SUC_lexicon_tags_1 },
    { 0xbeefbd59, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40cfbd62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x288fbd6b, SUC_lexicon_tags_43 },
    { 0x8d0fbd6c, SUC_lexicon_tags_45 },
    { 0x136fbd6d, SUC_lexicon_tags_17 },
    { 0xe18fbd6c, SUC_lexicon_tags_32 },
    { 0xc44fbd6c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x746fbd74, SUC_lexicon_tags_38 },
    { 0x74efbd74, SUC_lexicon_tags_1 },
    { 0x4e8fbd74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68fbd78, SUC_lexicon_tags_56 },
    { 0xc84fbd78, SUC_lexicon_tags_1 },
    { 0xbd0fbd7a, SUC_lexicon_tags_23 },
    { 0x77afbd7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4fbd7f, SUC_lexicon_tags_27 },
    { 0x754fbd80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x102fbd82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x62efbd84, SUC_lexicon_tags_9 },
    { 0x166fbd85, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe2fbd87, SUC_lexicon_tags_37 },
    { 0xcf4fbd88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70fbd92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43afbd94, SUC_lexicon_tags_7 },
    { 0x1c6fbd95, SUC_lexicon_tags_37 },
    { 0xa4cfbd96, SUC_lexicon_tags_2 },
    { 0x13afbd96, SUC_lexicon_tags_15 },
    { 0xa02fbd94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9cfbd9a, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94fbd9f, SUC_lexicon_tags_7 },
    { 0x17cfbd9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf92fbda2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4fbda8, SUC_lexicon_tags_15 },
    { 0x94fbda9, SUC_lexicon_tags_15 },
    { 0xca0fbda9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74fbdb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fbdba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedefbdc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6efbdc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ccfbdc5, SUC_lexicon_tags_14 },
    { 0xbb2fbdc6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85afbdcb, SUC_lexicon_tags_10 },
    { 0x594fbdcb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2dcfbdce, SUC_lexicon_tags_1 },
    { 0x674fbdcf, SUC_lexicon_tags_17 },
    { 0xd84fbdce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cfbdd4, SUC_lexicon_tags_66 },
    { 0x84cfbdd5, SUC_lexicon_tags_25 },
    { 0xec8fbdd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54fbddb, SUC_lexicon_tags_15 },
    { 0xd4efbddb, SUC_lexicon_tags_14 },
    { 0xfa0fbddb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x39efbde0, SUC_lexicon_tags_16 },
    { 0x52cfbde0, SUC_lexicon_tags_0 },
    { 0x148fbde1, SUC_lexicon_tags_32 },
    { 0xb40fbde2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea2fbdeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aafbdf0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x74afbdf2, SUC_lexicon_tags_2 },
    { 0x772fbdf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6fbdf5, SUC_lexicon_tags_13 },
    { 0x550fbdf5, SUC_lexicon_tags_23 },
    { 0xd16fbdf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478fbdfd, SUC_lexicon_tags_47 },
    { 0x944fbdfe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd98fbe00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf26fbe02, SUC_lexicon_tags_49 },
    { 0xd1cfbe03, SUC_lexicon_tags_17 },
    { 0xa78fbe04, SUC_lexicon_tags_27 },
    { 0x9c8fbe05, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe9efbe07, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bafbe0d, SUC_lexicon_tags_15 },
    { 0xda0fbe0d, SUC_lexicon_tags_1 },
    { 0x7fefbe0f, SUC_lexicon_tags_11 },
    { 0xdcafbe0f, SUC_lexicon_tags_15 },
    { 0xb56fbe10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42fbe16, SUC_lexicon_tags_41 },
    { 0x568fbe17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfbe1e, SUC_lexicon_tags_2 },
    { 0xd8cfbe1f, SUC_lexicon_tags_45 },
    { 0xf3cfbe20, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x718fbe23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37afbe26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x996fbe28, SUC_lexicon_tags_11 },
    { 0x164fbe29, SUC_lexicon_tags_17 },
    { 0x99afbe28, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbe2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fefbe30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe46fbe32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8afbe3a, SUC_lexicon_tags_25 },
    { 0x38efbe3a, SUC_lexicon_tags_30 },
    { 0xae4fbe3a, SUC_lexicon_tags_19 },
    { 0xcf8fbe3d, SUC_lexicon_tags_31 },
    { 0xc34fbe3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7befbe43, SUC_lexicon_tags_35 },
    { 0x280fbe44, SUC_lexicon_tags_6 },
    { 0x10afbe45, SUC_lexicon_tags_92 },
    { 0x7c2fbe45, SUC_lexicon_tags_105 },
    { 0x434fbe47, SUC_lexicon_tags_37 },
    { 0x620fbe47, SUC_lexicon_tags_27 },
    { 0xd0cfbe49, SUC_lexicon_tags_15 },
    { 0xc6cfbe48, SUC_lexicon_tags_47 },
    { 0x8fefbe44, SUC_lexicon_tags_74 },
    { 0x9cfbe4c, SUC_lexicon_tags_0 },
    { 0x25afbe4d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8defbe51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5cfbe53, SUC_lexicon_tags_6 },
    { 0x8d6fbe54, SUC_lexicon_tags_6 },
    { 0x73cfbe55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x182fbe59, SUC_lexicon_tags_11 },
    { 0xdcfbe5a, SUC_lexicon_tags_197 },
    { 0xc6efbe5b, SUC_lexicon_tags_61 },
    { 0x63afbe5a, SUC_lexicon_tags_26 },
    { 0xc0fbe5d, SUC_lexicon_tags_7 },
    { 0x246fbe5d, SUC_lexicon_tags_19 },
    { 0xff4fbe5f, SUC_lexicon_tags_66 },
    { 0xb78fbe60, SUC_lexicon_tags_26 },
    { 0xe4cfbe61, SUC_lexicon_tags_46 },
    { 0xfc4fbe61, SUC_lexicon_tags_12 },
    { 0x1fcfbe61, SUC_lexicon_tags_39 },
    { 0x8b0fbe60, SUC_lexicon_tags_65 },
    { 0x27afbe5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5f2fbe67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3afbe69, SUC_lexicon_tags_37 },
    { 0x63cfbe6a, SUC_lexicon_tags_15 },
    { 0xc52fbe6b, SUC_lexicon_tags_45 },
    { 0x89afbe6b, SUC_lexicon_tags_1 },
    { 0xb2fbe6d, SUC_lexicon_tags_19 },
    { 0x7fcfbe6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fefbe72, SUC_lexicon_tags_60 },
    { 0x704fbe73, SUC_lexicon_tags_21 },
    { 0x11cfbe73, SUC_lexicon_tags_60 },
    { 0x2fafbe74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6efbe7a, SUC_lexicon_tags_47 },
    { 0x8cefbe7b, SUC_lexicon_tags_45 },
    { 0x2e6fbe7c, SUC_lexicon_tags_12 },
    { 0xd4afbe7d, SUC_lexicon_tags_60 },
    { 0x14afbe7b, SUC_lexicon_tags_39 },
    { 0x6c8fbe7f, SUC_lexicon_tags_11 },
    { 0x628fbe80, SUC_lexicon_tags_60 },
    { 0xa0fbe7d, SUC_lexicon_tags_34 },
    { 0x996fbe82, SUC_lexicon_tags_17 },
    { 0xffcfbe83, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x696fbe85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cfbe8f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184fbe93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60afbe97, SUC_lexicon_tags_18 },
    { 0x53cfbe98, SUC_lexicon_tags_37 },
    { 0x1befbe97, SUC_lexicon_tags_19 },
    { 0x8e0fbe99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x900fbea1, SUC_lexicon_tags_30 },
    { 0x468fbea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd34fbea5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cfbea8, SUC_lexicon_tags_88 },
    { 0xd20fbea9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fafbeab, SUC_lexicon_tags_11 },
    { 0x384fbeab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa88fbeae, SUC_lexicon_tags_29 },
    { 0x8ecfbeae, SUC_lexicon_tags_30 },
    { 0x8ccfbeb0, SUC_lexicon_tags_17 },
    { 0x5f0fbeb0, SUC_lexicon_tags_30 },
    { 0x7cafbeb2, SUC_lexicon_tags_7 },
    { 0x92cfbeb1, SUC_lexicon_tags_15 },
    { 0x966fbeb4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88fbeb9, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8fbebc, SUC_lexicon_tags_17 },
    { 0x424fbebd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcfbec1, SUC_lexicon_tags_29 },
    { 0x7efbec2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x420fbec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x794fbec6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8afbece, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc86fbed3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x322fbed5, SUC_lexicon_tags_65 },
    { 0x5c6fbed6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x540fbed9, SUC_lexicon_tags_12 },
    { 0x75afbeda, SUC_lexicon_tags_85 },
    { 0x8d4fbedb, SUC_lexicon_tags_9 },
    { 0x766fbedc, SUC_lexicon_tags_71 },
    { 0xedcfbedd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8fbee0, SUC_lexicon_tags_37 },
    { 0x85efbee1, SUC_lexicon_tags_12 },
    { 0x4bafbee0, SUC_lexicon_tags_15 },
    { 0xe5cfbee1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb04fbee5, SUC_lexicon_tags_25 },
    { 0x640fbee5, SUC_lexicon_tags_1 },
    { 0x47efbee7, SUC_lexicon_tags_7 },
    { 0x6fefbee6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x466fbeea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x968fbef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dafbef3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cafbef6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afbefb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cfbefe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x366fbf00, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xb50fbf02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa32fbf05, SUC_lexicon_tags_46 },
    { 0xac4fbf05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2efbf09, SUC_lexicon_tags_60 },
    { 0x274fbf0a, SUC_lexicon_tags_14 },
    { 0x280fbf0b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x648fbf0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c4fbf12, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430fbf16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1efbf19, SUC_lexicon_tags_30 },
    { 0x3f4fbf1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fafbf23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fbf2a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fafbf2e, SUC_lexicon_tags_1259 },
    { 0, NULL },
    { 0xbc4fbf30, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6fbf34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x646fbf36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6acfbf38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8fbf3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xad2fbf3e, SUC_lexicon_tags_37 },
    { 0xfd8fbf3f, SUC_lexicon_tags_9 },
    { 0xd7afbf3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x576fbf44, SUC_lexicon_tags_74 },
    { 0xfa6fbf44, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938fbf4a, SUC_lexicon_tags_37 },
    { 0x7c4fbf4a, SUC_lexicon_tags_33 },
    { 0x5dcfbf4c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd04fbf50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15efbf54, SUC_lexicon_tags_32 },
    { 0xa08fbf54, SUC_lexicon_tags_21 },
    { 0x5fefbf56, SUC_lexicon_tags_1321 },
    { 0x95cfbf57, SUC_lexicon_tags_0 },
    { 0xfc2fbf58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0fbf5d, SUC_lexicon_tags_25 },
    { 0xefefbf5e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174fbf63, SUC_lexicon_tags_17 },
    { 0xdd2fbf63, SUC_lexicon_tags_91 },
    { 0x3aafbf63, SUC_lexicon_tags_19 },
    { 0x9fcfbf66, SUC_lexicon_tags_2 },
    { 0xc18fbf67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x40efbf6a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3c4fbf6c, SUC_lexicon_tags_38 },
    { 0xdc6fbf6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0fbf71, SUC_lexicon_tags_2 },
    { 0x8cafbf72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1d0fbf74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e0fbf7f, SUC_lexicon_tags_1 },
    { 0xffafbf80, SUC_lexicon_tags_11 },
    { 0x29afbf81, SUC_lexicon_tags_10 },
    { 0xe88fbf82, SUC_lexicon_tags_7 },
    { 0x994fbf83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc36fbf85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26afbf87, SUC_lexicon_tags_28 },
    { 0x4d0fbf88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfbf8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb5efbf8e, SUC_lexicon_tags_68 },
    { 0x2c8fbf8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376fbf93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cfbf97, SUC_lexicon_tags_17 },
    { 0xbb2fbf97, SUC_lexicon_tags_13 },
    { 0xd4fbf99, SUC_lexicon_tags_10 },
    { 0x778fbf9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x11afbf9c, SUC_lexicon_tags_55 },
    { 0x31afbf9d, SUC_lexicon_tags_1 },
    { 0xed0fbf9e, SUC_lexicon_tags_6 },
    { 0x596fbf9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8fbfa2, SUC_lexicon_tags_66 },
    { 0x56afbfa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x938fbfa6, SUC_lexicon_tags_16 },
    { 0x850fbfa7, SUC_lexicon_tags_8 },
    { 0x478fbfa8, SUC_lexicon_tags_1 },
    { 0x80afbfa9, SUC_lexicon_tags_230 },
    { 0x552fbfa9, SUC_lexicon_tags_7 },
    { 0xe74fbfab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c4fbfad, SUC_lexicon_tags_37 },
    { 0x3aafbfae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f6fbfb0, SUC_lexicon_tags_44 },
    { 0x9fafbfb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdd8fbfb3, SUC_lexicon_tags_5 },
    { 0x50cfbfb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x734fbfb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbefbfb9, SUC_lexicon_tags_33 },
    { 0xa82fbfb9, SUC_lexicon_tags_14 },
    { 0xefafbfbb, SUC_lexicon_tags_14 },
    { 0x28fbfbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34afbfbe, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0x1a8fbfc0, SUC_lexicon_tags_2 },
    { 0x88fbfc1, SUC_lexicon_tags_43 },
    { 0xd5afbfc2, SUC_lexicon_tags_64 },
    { 0xa70fbfc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8befbfc9, SUC_lexicon_tags_60 },
    { 0x728fbfca, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2fbfd3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8fbfda, SUC_lexicon_tags_11 },
    { 0xae4fbfdb, SUC_lexicon_tags_21 },
    { 0x660fbfdc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894fbfe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4fbfe7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9efbfe9, SUC_lexicon_tags_17 },
    { 0x248fbfea, SUC_lexicon_tags_33 },
    { 0x2cefbfeb, SUC_lexicon_tags_46 },
    { 0x680fbfec, SUC_lexicon_tags_8 },
    { 0x168fbfeb, SUC_lexicon_tags_47 },
    { 0x692fbfee, SUC_lexicon_tags_17 },
    { 0x4fefbfeb, SUC_lexicon_tags_32 },
    { 0xcc8fbfeb, SUC_lexicon_tags_32 },
    { 0x3e8fbff0, SUC_lexicon_tags_49 },
    { 0xbacfbfe9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fbffc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc88fc000, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x46efc003, SUC_lexicon_tags_2 },
    { 0xac0fc004, SUC_lexicon_tags_64 },
    { 0xcd4fc003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86fc007, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x610fc00a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aefc00f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x858fc011, SUC_lexicon_tags_46 },
    { 0x7e6fc012, SUC_lexicon_tags_2 },
    { 0xe7afc013, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7f2fc015, SUC_lexicon_tags_215 },
    { 0x336fc016, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcfc01e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06fc022, SUC_lexicon_tags_37 },
    { 0xef6fc022, SUC_lexicon_tags_37 },
    { 0x8d4fc024, SUC_lexicon_tags_7 },
    { 0x1e2fc025, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecafc02f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x75efc032, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9cfc034, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x490fc036, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x626fc038, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa68fc03a, SUC_lexicon_tags_12 },
    { 0xc1cfc03b, SUC_lexicon_tags_37 },
    { 0x908fc03b, SUC_lexicon_tags_19 },
    { 0xe44fc03d, SUC_lexicon_tags_7 },
    { 0xeb8fc03b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6fc042, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x58efc044, SUC_lexicon_tags_0 },
    { 0x82fc045, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4fc048, SUC_lexicon_tags_37 },
    { 0x85afc049, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x408fc04e, SUC_lexicon_tags_0 },
    { 0xc86fc04f, SUC_lexicon_tags_0 },
    { 0x456fc04f, SUC_lexicon_tags_45 },
    { 0x14cfc051, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x200fc053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19cfc057, SUC_lexicon_tags_11 },
    { 0xd42fc058, SUC_lexicon_tags_25 },
    { 0x14afc059, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91cfc05b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6afc05d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12cfc062, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc82fc064, SUC_lexicon_tags_8 },
    { 0xcd0fc065, SUC_lexicon_tags_7 },
    { 0x20efc064, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0efc06a, SUC_lexicon_tags_27 },
    { 0x742fc06b, SUC_lexicon_tags_37 },
    { 0xcc6fc06c, SUC_lexicon_tags_4 },
    { 0x23cfc06c, SUC_lexicon_tags_12 },
    { 0xce2fc06d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c0fc075, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xedafc078, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x274fc07a, SUC_lexicon_tags_25 },
    { 0x130fc07b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb62fc07e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4fc083, SUC_lexicon_tags_37 },
    { 0x2f6fc084, SUC_lexicon_tags_63 },
    { 0x41efc085, SUC_lexicon_tags_14 },
    { 0x9e2fc083, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cfc08d, SUC_lexicon_tags_23 },
    { 0x566fc08e, SUC_lexicon_tags_29 },
    { 0x956fc08f, SUC_lexicon_tags_33 },
    { 0xed0fc090, SUC_lexicon_tags_14 },
    { 0x24efc091, SUC_lexicon_tags_19 },
    { 0x4a8fc092, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x104fc095, SUC_lexicon_tags_2 },
    { 0x6a6fc096, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd54fc098, SUC_lexicon_tags_12 },
    { 0xafefc098, SUC_lexicon_tags_17 },
    { 0x4f4fc09a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476fc09f, SUC_lexicon_tags_39 },
    { 0x1f4fc0a0, SUC_lexicon_tags_3 },
    { 0x6b0fc0a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d0fc0a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb38fc0a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132fc0a9, SUC_lexicon_tags_29 },
    { 0x3b0fc0aa, SUC_lexicon_tags_17 },
    { 0x7d8fc0ab, SUC_lexicon_tags_1 },
    { 0x428fc0ac, SUC_lexicon_tags_64 },
    { 0x936fc0a9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbeafc0af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cafc0b3, SUC_lexicon_tags_13 },
    { 0x87efc0b4, SUC_lexicon_tags_46 },
    { 0x8f6fc0b5, SUC_lexicon_tags_7 },
    { 0x1cafc0b6, SUC_lexicon_tags_4 },
    { 0x94afc0b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58fc0bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aefc0c0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3afc0c4, SUC_lexicon_tags_13 },
    { 0x92fc0c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1cfc0c7, SUC_lexicon_tags_11 },
    { 0x3e2fc0c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84cfc0ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe36fc0cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954fc0d4, SUC_lexicon_tags_11 },
    { 0x336fc0d5, SUC_lexicon_tags_87 },
    { 0xdfefc0d6, SUC_lexicon_tags_8 },
    { 0x13afc0d4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cfc0e1, SUC_lexicon_tags_15 },
    { 0x990fc0e2, SUC_lexicon_tags_17 },
    { 0x2bafc0e3, SUC_lexicon_tags_12 },
    { 0x6f8fc0e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cfc0e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88fc0eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7efc0f2, SUC_lexicon_tags_30 },
    { 0xf98fc0f3, SUC_lexicon_tags_46 },
    { 0x102fc0f2, SUC_lexicon_tags_7 },
    { 0xc3efc0f2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcfc0f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cfc0fa, SUC_lexicon_tags_48 },
    { 0xa64fc0fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb0fc0fe, SUC_lexicon_tags_32 },
    { 0x9f2fc0fe, SUC_lexicon_tags_37 },
    { 0x9b8fc100, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08fc106, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cfc109, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fcfc10b, SUC_lexicon_tags_43 },
    { 0xe76fc10c, SUC_lexicon_tags_14 },
    { 0xfc10d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afc111, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x140fc114, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8efc117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2fc11b, SUC_lexicon_tags_178 },
    { 0xc4afc11b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x684fc11f, SUC_lexicon_tags_15 },
    { 0x3bcfc120, SUC_lexicon_tags_14 },
    { 0xa7efc121, SUC_lexicon_tags_33 },
    { 0x1b2fc120, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37afc12b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32afc133, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44afc139, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310fc13d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe00fc147, SUC_lexicon_tags_43 },
    { 0x270fc148, SUC_lexicon_tags_10 },
    { 0x6eefc149, SUC_lexicon_tags_103 },
    { 0xe28fc147, SUC_lexicon_tags_82 },
    { 0xfa6fc149, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efc14e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x410fc150, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0fc154, SUC_lexicon_tags_15 },
    { 0xde0fc154, SUC_lexicon_tags_1 },
    { 0x3f8fc156, SUC_lexicon_tags_0 },
    { 0x7a4fc155, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99afc159, SUC_lexicon_tags_28 },
    { 0xf46fc15a, SUC_lexicon_tags_4 },
    { 0x9aefc15b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372fc15f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6fc163, SUC_lexicon_tags_2 },
    { 0x650fc164, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efc168, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x926fc16c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63efc175, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x416fc177, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa24fc17a, SUC_lexicon_tags_14 },
    { 0x1cafc17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162fc182, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16efc189, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x904fc18b, SUC_lexicon_tags_63 },
    { 0x53efc18b, SUC_lexicon_tags_37 },
    { 0xda0fc18d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb62fc18f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f6fc191, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x350fc193, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a4fc195, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0fc199, SUC_lexicon_tags_65 },
    { 0xe96fc19a, SUC_lexicon_tags_11 },
    { 0x692fc19a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec4fc19d, SUC_lexicon_tags_8 },
    { 0xacefc19e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5efc1a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x988fc1a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe80fc1ae, SUC_lexicon_tags_25 },
    { 0x7aafc1ae, SUC_lexicon_tags_25 },
    { 0x32afc1b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe64fc1b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6fc1b8, SUC_lexicon_tags_11 },
    { 0x654fc1b9, SUC_lexicon_tags_12 },
    { 0xe8afc1b8, SUC_lexicon_tags_7 },
    { 0x23afc1bb, SUC_lexicon_tags_31 },
    { 0x126fc1bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4fc1c0, SUC_lexicon_tags_26 },
    { 0xf1efc1c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x296fc1c4, SUC_lexicon_tags_11 },
    { 0xaecfc1c5, SUC_lexicon_tags_2 },
    { 0x9a2fc1c5, SUC_lexicon_tags_6 },
    { 0x8b4fc1c7, SUC_lexicon_tags_11 },
    { 0x26cfc1c8, SUC_lexicon_tags_27 },
    { 0x654fc1c9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9dafc1cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fc1d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa78fc1d2, SUC_lexicon_tags_29 },
    { 0x6f4fc1d2, SUC_lexicon_tags_19 },
    { 0xe4afc1d4, SUC_lexicon_tags_49 },
    { 0x80fc1d2, SUC_lexicon_tags_37 },
    { 0x964fc1d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91afc1de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3eafc1e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70fc1e3, SUC_lexicon_tags_14 },
    { 0x7e6fc1e4, SUC_lexicon_tags_15 },
    { 0x802fc1e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6a8fc1e7, SUC_lexicon_tags_253 },
    { 0xad2fc1e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf76fc1ea, SUC_lexicon_tags_34 },
    { 0xe4efc1ea, SUC_lexicon_tags_15 },
    { 0x4d4fc1eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x924fc1ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65afc1f2, SUC_lexicon_tags_47 },
    { 0xa72fc1f3, SUC_lexicon_tags_30 },
    { 0xbfcfc1f4, SUC_lexicon_tags_25 },
    { 0x328fc1f5, SUC_lexicon_tags_15 },
    { 0xdc0fc1f3, SUC_lexicon_tags_37 },
    { 0x6f8fc1f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2efc1fd, SUC_lexicon_tags_1221 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6fc200, SUC_lexicon_tags_4 },
    { 0x2fcfc201, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa62fc203, SUC_lexicon_tags_6 },
    { 0x2a4fc203, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0fc20d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0fc211, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x302fc213, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f8fc215, SUC_lexicon_tags_17 },
    { 0xd04fc216, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838fc228, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cfc22d, SUC_lexicon_tags_25 },
    { 0xd7efc22d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecfc232, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b8fc234, SUC_lexicon_tags_34 },
    { 0x63cfc234, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x366fc237, SUC_lexicon_tags_2 },
    { 0xa2afc238, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58fc240, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x848fc245, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf98fc247, SUC_lexicon_tags_25 },
    { 0xea2fc248, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918fc24c, SUC_lexicon_tags_92 },
    { 0x3b6fc24c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552fc258, SUC_lexicon_tags_47 },
    { 0x510fc259, SUC_lexicon_tags_7 },
    { 0x93afc25a, SUC_lexicon_tags_17 },
    { 0xa34fc258, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3efc25f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95cfc261, SUC_lexicon_tags_33 },
    { 0xe40fc262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x642fc265, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x642fc267, SUC_lexicon_tags_60 },
    { 0x182fc268, SUC_lexicon_tags_45 },
    { 0x7d4fc269, SUC_lexicon_tags_47 },
    { 0xbe0fc269, SUC_lexicon_tags_17 },
    { 0x256fc26b, SUC_lexicon_tags_7 },
    { 0xdbefc26b, SUC_lexicon_tags_37 },
    { 0x898fc269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64afc270, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91cfc272, SUC_lexicon_tags_31 },
    { 0xd34fc272, SUC_lexicon_tags_11 },
    { 0xee8fc274, SUC_lexicon_tags_11 },
    { 0x7aafc275, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70afc279, SUC_lexicon_tags_1 },
    { 0xf14fc27a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d4fc27c, SUC_lexicon_tags_1 },
    { 0x3a8fc27d, SUC_lexicon_tags_48 },
    { 0x990fc27d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c2fc280, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388fc285, SUC_lexicon_tags_8 },
    { 0xc6cfc285, SUC_lexicon_tags_2 },
    { 0xd0cfc286, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x654fc289, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe8afc28b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8fc293, SUC_lexicon_tags_37 },
    { 0x1fafc294, SUC_lexicon_tags_37 },
    { 0x224fc294, SUC_lexicon_tags_37 },
    { 0x20fc296, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeefc29d, SUC_lexicon_tags_68 },
    { 0x2d8fc29d, SUC_lexicon_tags_87 },
    { 0x2e6fc29f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x822fc2a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36cfc2a3, SUC_lexicon_tags_11 },
    { 0x148fc2a4, SUC_lexicon_tags_11 },
    { 0x740fc2a4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x834fc2a8, SUC_lexicon_tags_27 },
    { 0x762fc2a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5cafc2ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54efc2ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cefc2b2, SUC_lexicon_tags_31 },
    { 0x336fc2b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34fc2b7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bafc2bd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x7e4fc2bf, SUC_lexicon_tags_23 },
    { 0x818fc2c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa42fc2c3, SUC_lexicon_tags_33 },
    { 0xfe0fc2c4, SUC_lexicon_tags_66 },
    { 0x31afc2c5, SUC_lexicon_tags_12 },
    { 0x436fc2c5, SUC_lexicon_tags_17 },
    { 0x984fc2c5, SUC_lexicon_tags_15 },
    { 0xb4efc2c6, SUC_lexicon_tags_0 },
    { 0x334fc2c4, SUC_lexicon_tags_26 },
    { 0x78fc2ca, SUC_lexicon_tags_5 },
    { 0xadafc2cb, SUC_lexicon_tags_0 },
    { 0x54afc2ca, SUC_lexicon_tags_43 },
    { 0xabcfc2cd, SUC_lexicon_tags_45 },
    { 0xb3efc2ce, SUC_lexicon_tags_11 },
    { 0xdc4fc2ca, SUC_lexicon_tags_30 },
    { 0xa02fc2d0, SUC_lexicon_tags_37 },
    { 0x80fc2d1, SUC_lexicon_tags_11 },
    { 0x77cfc2d2, SUC_lexicon_tags_23 },
    { 0x7f2fc2d3, SUC_lexicon_tags_103 },
    { 0xb54fc2d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330fc2d8, SUC_lexicon_tags_731 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecfc2e1, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cfc2e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8fc2e8, SUC_lexicon_tags_14 },
    { 0x728fc2e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe06fc2ec, SUC_lexicon_tags_0 },
    { 0x3e2fc2ed, SUC_lexicon_tags_65 },
    { 0xfcefc2ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd88fc2f6, SUC_lexicon_tags_15 },
    { 0x6c0fc2f6, SUC_lexicon_tags_22 },
    { 0xd12fc2f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36fc2fc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x35afc2fe, SUC_lexicon_tags_49 },
    { 0x1bcfc2fe, SUC_lexicon_tags_46 },
    { 0xa44fc300, SUC_lexicon_tags_17 },
    { 0xad6fc300, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cafc309, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd00fc30c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8efc30e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7defc313, SUC_lexicon_tags_33 },
    { 0x76cfc314, SUC_lexicon_tags_15 },
    { 0x4c6fc315, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80fc317, SUC_lexicon_tags_4 },
    { 0xaf0fc318, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5efc31b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdf0fc31d, SUC_lexicon_tags_37 },
    { 0x69efc31d, SUC_lexicon_tags_57 },
    { 0x1ecfc31d, SUC_lexicon_tags_25 },
    { 0x802fc31d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fc324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2fc32c, SUC_lexicon_tags_68 },
    { 0xb7efc32d, SUC_lexicon_tags_25 },
    { 0xf4efc32d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x416fc331, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2dafc333, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cfc338, SUC_lexicon_tags_7 },
    { 0x3e6fc338, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fc33c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafc344, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x980fc34b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aefc351, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fcfc353, SUC_lexicon_tags_27 },
    { 0xa08fc354, SUC_lexicon_tags_38 },
    { 0x946fc354, SUC_lexicon_tags_25 },
    { 0x65efc356, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90fc35a, SUC_lexicon_tags_6 },
    { 0xf34fc35b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe04fc35d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x700fc35f, SUC_lexicon_tags_29 },
    { 0x5fefc35f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cfc364, SUC_lexicon_tags_40 },
    { 0x5fefc365, SUC_lexicon_tags_2 },
    { 0xf42fc365, SUC_lexicon_tags_45 },
    { 0xe34fc367, SUC_lexicon_tags_37 },
    { 0x2f8fc368, SUC_lexicon_tags_11 },
    { 0xb0efc368, SUC_lexicon_tags_14 },
    { 0xcc0fc36a, SUC_lexicon_tags_25 },
    { 0xfc2fc36b, SUC_lexicon_tags_19 },
    { 0xdaefc36b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfa8fc36e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bafc372, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874fc379, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78fc37b, SUC_lexicon_tags_25 },
    { 0x364fc37b, SUC_lexicon_tags_1 },
    { 0x73efc37c, SUC_lexicon_tags_87 },
    { 0x4fafc37d, SUC_lexicon_tags_12 },
    { 0xe62fc37e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6fc386, SUC_lexicon_tags_25 },
    { 0xa7cfc387, SUC_lexicon_tags_33 },
    { 0xabefc387, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684fc38c, SUC_lexicon_tags_31 },
    { 0x830fc38d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960fc392, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x76efc395, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x434fc397, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb1cfc399, SUC_lexicon_tags_27 },
    { 0x32afc39a, SUC_lexicon_tags_17 },
    { 0x2acfc39a, SUC_lexicon_tags_11 },
    { 0x290fc39c, SUC_lexicon_tags_2 },
    { 0x68cfc39c, SUC_lexicon_tags_15 },
    { 0x21cfc399, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0fc3a3, SUC_lexicon_tags_15 },
    { 0x144fc3a4, SUC_lexicon_tags_15 },
    { 0xf92fc3a5, SUC_lexicon_tags_27 },
    { 0x462fc3a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aafc3ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156fc3b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf02fc3b4, SUC_lexicon_tags_14 },
    { 0x8defc3b5, SUC_lexicon_tags_17 },
    { 0xc4afc3b5, SUC_lexicon_tags_1 },
    { 0x932fc3b7, SUC_lexicon_tags_38 },
    { 0x4c6fc3b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2fc3bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb6afc3be, SUC_lexicon_tags_12 },
    { 0x1c6fc3bf, SUC_lexicon_tags_45 },
    { 0xdcfc3bf, SUC_lexicon_tags_0 },
    { 0xddefc3c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3efc3c5, SUC_lexicon_tags_11 },
    { 0xea2fc3c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0fc3ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70fc3ce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfa2fc3d0, SUC_lexicon_tags_47 },
    { 0x304fc3d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea6fc3d3, SUC_lexicon_tags_33 },
    { 0x874fc3d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x814fc3da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30fc3dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4afc3e1, SUC_lexicon_tags_27 },
    { 0xf9cfc3e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaecfc3e5, SUC_lexicon_tags_27 },
    { 0x792fc3e6, SUC_lexicon_tags_17 },
    { 0x7e0fc3e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x834fc3ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x61afc3ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2fc3ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x134fc3f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bcfc3f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26afc3fb, SUC_lexicon_tags_2 },
    { 0xd74fc3fc, SUC_lexicon_tags_25 },
    { 0x852fc3fc, SUC_lexicon_tags_11 },
    { 0x3c2fc3fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a2fc400, SUC_lexicon_tags_38 },
    { 0x6c6fc401, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0afc406, SUC_lexicon_tags_2 },
    { 0xe16fc407, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d6fc409, SUC_lexicon_tags_17 },
    { 0x9b2fc40a, SUC_lexicon_tags_7 },
    { 0xaf8fc40a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x328fc40e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xecafc411, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596fc418, SUC_lexicon_tags_1 },
    { 0xc96fc419, SUC_lexicon_tags_11 },
    { 0xe02fc41a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a4fc41d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x266fc41f, SUC_lexicon_tags_1 },
    { 0xc08fc41f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1dafc422, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52fc426, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x534fc429, SUC_lexicon_tags_43 },
    { 0x1cefc429, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1defc42e, SUC_lexicon_tags_48 },
    { 0xafefc42e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x37afc431, SUC_lexicon_tags_35 },
    { 0xc66fc431, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50fc440, SUC_lexicon_tags_17 },
    { 0xe36fc441, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40fc445, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706fc454, SUC_lexicon_tags_25 },
    { 0x1a4fc455, SUC_lexicon_tags_19 },
    { 0x49cfc456, SUC_lexicon_tags_17 },
    { 0xc74fc457, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x666fc459, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8fc460, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x384fc462, SUC_lexicon_tags_37 },
    { 0xa12fc463, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb92fc465, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc6fc468, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5e0fc46a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cfc470, SUC_lexicon_tags_2 },
    { 0x768fc470, SUC_lexicon_tags_37 },
    { 0x4b4fc472, SUC_lexicon_tags_17 },
    { 0x1b0fc471, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80cfc476, SUC_lexicon_tags_17 },
    { 0x140fc476, SUC_lexicon_tags_17 },
    { 0xd78fc478, SUC_lexicon_tags_11 },
    { 0x1cefc478, SUC_lexicon_tags_19 },
    { 0x7c8fc47a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0fc47f, SUC_lexicon_tags_23 },
    { 0xd48fc480, SUC_lexicon_tags_27 },
    { 0x9ccfc480, SUC_lexicon_tags_231 },
    { 0xcb4fc481, SUC_lexicon_tags_6 },
    { 0x240fc483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e0fc48b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf08fc48e, SUC_lexicon_tags_0 },
    { 0x450fc48f, SUC_lexicon_tags_15 },
    { 0x81cfc490, SUC_lexicon_tags_1 },
    { 0x86fc491, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x27efc494, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382fc49b, SUC_lexicon_tags_229 },
    { 0x444fc49b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cfc49f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe1efc4a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e6fc4a5, SUC_lexicon_tags_2 },
    { 0xee2fc4a5, SUC_lexicon_tags_12 },
    { 0xbfafc4a7, SUC_lexicon_tags_46 },
    { 0xfe8fc4a6, SUC_lexicon_tags_48 },
    { 0xc9efc4a5, SUC_lexicon_tags_17 },
    { 0x734fc4aa, SUC_lexicon_tags_37 },
    { 0xa2afc4a5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfa8fc4ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecfc4b2, SUC_lexicon_tags_23 },
    { 0xd32fc4b3, SUC_lexicon_tags_17 },
    { 0xa72fc4b4, SUC_lexicon_tags_21 },
    { 0x1d6fc4b5, SUC_lexicon_tags_43 },
    { 0xb56fc4b5, SUC_lexicon_tags_8 },
    { 0xbc8fc4b7, SUC_lexicon_tags_15 },
    { 0x7defc4b8, SUC_lexicon_tags_12 },
    { 0xc50fc4b9, SUC_lexicon_tags_17 },
    { 0x1efc4ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0fc4c2, SUC_lexicon_tags_1 },
    { 0x67afc4c3, SUC_lexicon_tags_37 },
    { 0x504fc4c2, SUC_lexicon_tags_31 },
    { 0xec0fc4c5, SUC_lexicon_tags_33 },
    { 0x3c8fc4c6, SUC_lexicon_tags_37 },
    { 0x8f4fc4c7, SUC_lexicon_tags_12 },
    { 0x41efc4c6, SUC_lexicon_tags_15 },
    { 0x2acfc4c9, SUC_lexicon_tags_31 },
    { 0x5defc4c9, SUC_lexicon_tags_28 },
    { 0x646fc4cb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3ecfc4cd, SUC_lexicon_tags_40 },
    { 0x89afc4cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c2fc4d3, SUC_lexicon_tags_17 },
    { 0x696fc4d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83efc4d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x206fc4d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a0fc4dd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbafc4e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492fc4e5, SUC_lexicon_tags_8 },
    { 0xf02fc4e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd94fc4eb, SUC_lexicon_tags_12 },
    { 0xcf4fc4eb, SUC_lexicon_tags_11 },
    { 0xc88fc4ed, SUC_lexicon_tags_11 },
    { 0x4a6fc4ed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472fc4f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x170fc4f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcfc4f9, SUC_lexicon_tags_17 },
    { 0xcfefc4fa, SUC_lexicon_tags_30 },
    { 0x3aafc4fb, SUC_lexicon_tags_9 },
    { 0x14afc4fa, SUC_lexicon_tags_158 },
    { 0xdcfc4fd, SUC_lexicon_tags_19 },
    { 0xbbefc4fe, SUC_lexicon_tags_15 },
    { 0xe38fc4ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf50fc501, SUC_lexicon_tags_25 },
    { 0x7fafc502, SUC_lexicon_tags_66 },
    { 0xdbefc503, SUC_lexicon_tags_38 },
    { 0xf1afc504, SUC_lexicon_tags_17 },
    { 0xca2fc502, SUC_lexicon_tags_29 },
    { 0x92afc506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39cfc508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58afc50a, SUC_lexicon_tags_85 },
    { 0xc5cfc50a, SUC_lexicon_tags_15 },
    { 0x9b4fc50c, SUC_lexicon_tags_35 },
    { 0x26fc50d, SUC_lexicon_tags_12 },
    { 0x98fc50e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cefc511, SUC_lexicon_tags_29 },
    { 0xaaefc511, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x740fc514, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330fc519, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x884fc51c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3eefc51e, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xa4fc520, SUC_lexicon_tags_12 },
    { 0xa70fc521, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fefc527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fc52c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eefc530, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38fc532, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90afc538, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x460fc53a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf3afc53c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeefc542, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676fc547, SUC_lexicon_tags_12 },
    { 0xf24fc548, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda6fc54d, SUC_lexicon_tags_11 },
    { 0x942fc54e, SUC_lexicon_tags_74 },
    { 0x5b6fc54f, SUC_lexicon_tags_14 },
    { 0x450fc54f, SUC_lexicon_tags_1 },
    { 0x378fc54f, SUC_lexicon_tags_26 },
    { 0xb90fc550, SUC_lexicon_tags_25 },
    { 0xa4fc550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x416fc555, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x638fc557, SUC_lexicon_tags_16 },
    { 0xffefc558, SUC_lexicon_tags_1 },
    { 0x352fc559, SUC_lexicon_tags_23 },
    { 0x112fc55a, SUC_lexicon_tags_7 },
    { 0xb72fc55b, SUC_lexicon_tags_49 },
    { 0x50fc55b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26afc560, SUC_lexicon_tags_15 },
    { 0xa68fc561, SUC_lexicon_tags_7 },
    { 0x562fc561, SUC_lexicon_tags_1 },
    { 0x38efc562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e8fc566, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20fc56a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfc571, SUC_lexicon_tags_7 },
    { 0xdd8fc572, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06fc575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4fc578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fefc57a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x824fc57c, SUC_lexicon_tags_17 },
    { 0x41efc57c, SUC_lexicon_tags_37 },
    { 0xe7cfc57e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aefc584, SUC_lexicon_tags_46 },
    { 0xc42fc584, SUC_lexicon_tags_32 },
    { 0xf96fc584, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11afc58d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x808fc590, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x694fc593, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce4fc595, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc4fc597, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afc5a0, SUC_lexicon_tags_57 },
    { 0x60efc5a1, SUC_lexicon_tags_0 },
    { 0x2a0fc5a2, SUC_lexicon_tags_14 },
    { 0x9e4fc5a3, SUC_lexicon_tags_13 },
    { 0x1cafc5a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc10fc5a6, SUC_lexicon_tags_17 },
    { 0x4c0fc5a7, SUC_lexicon_tags_10 },
    { 0x804fc5a7, SUC_lexicon_tags_25 },
    { 0xeb2fc5a8, SUC_lexicon_tags_72 },
    { 0x1c0fc5aa, SUC_lexicon_tags_15 },
    { 0xb2fc5a6, SUC_lexicon_tags_8 },
    { 0x990fc5ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x320fc5ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc04fc5b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecefc5b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2fc5ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8fc5bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0fc5c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c4fc5c5, SUC_lexicon_tags_82 },
    { 0xd44fc5c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x486fc5cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x268fc5ce, SUC_lexicon_tags_29 },
    { 0x490fc5cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6fc5d8, SUC_lexicon_tags_49 },
    { 0xecafc5d8, SUC_lexicon_tags_47 },
    { 0x876fc5da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf68fc5dc, SUC_lexicon_tags_7 },
    { 0xa0cfc5dd, SUC_lexicon_tags_29 },
    { 0x74afc5de, SUC_lexicon_tags_1 },
    { 0x67efc5de, SUC_lexicon_tags_44 },
    { 0x570fc5dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64fc5e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd8fc5e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4fc5e7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x324fc5e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdefc5ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2fc5f1, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x586fc5f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52fc5f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fc5fb, SUC_lexicon_tags_23 },
    { 0xaf0fc5fb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1acfc5fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x222fc604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeefc608, SUC_lexicon_tags_25 },
    { 0x150fc609, SUC_lexicon_tags_8 },
    { 0xc88fc60a, SUC_lexicon_tags_48 },
    { 0x746fc60b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34cfc60e, SUC_lexicon_tags_61 },
    { 0x5c6fc60f, SUC_lexicon_tags_284 },
    { 0, NULL },
    { 0x60cfc611, SUC_lexicon_tags_7 },
    { 0x5b8fc612, SUC_lexicon_tags_6 },
    { 0x82fc613, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772fc619, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa8fc61b, SUC_lexicon_tags_25 },
    { 0xaf4fc61b, SUC_lexicon_tags_25 },
    { 0x4d4fc61c, SUC_lexicon_tags_1 },
    { 0xedefc61b, SUC_lexicon_tags_27 },
    { 0x3aafc61c, SUC_lexicon_tags_61 },
    { 0xa08fc61f, SUC_lexicon_tags_38 },
    { 0xe7cfc61b, SUC_lexicon_tags_14 },
    { 0x418fc620, SUC_lexicon_tags_19 },
    { 0x970fc622, SUC_lexicon_tags_61 },
    { 0xf3cfc624, SUC_lexicon_tags_15 },
    { 0x2efc625, SUC_lexicon_tags_29 },
    { 0xacefc61e, SUC_lexicon_tags_25 },
    { 0x624fc627, SUC_lexicon_tags_30 },
    { 0xcf4fc628, SUC_lexicon_tags_43 },
    { 0x162fc629, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754fc62d, SUC_lexicon_tags_37 },
    { 0xbdafc62d, SUC_lexicon_tags_30 },
    { 0x492fc62e, SUC_lexicon_tags_8 },
    { 0xb18fc62e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x16efc632, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc34fc634, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4defc638, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x194fc63a, SUC_lexicon_tags_70 },
    { 0xef8fc63b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7afc63f, SUC_lexicon_tags_15 },
    { 0xd12fc640, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d4fc642, SUC_lexicon_tags_39 },
    { 0xaecfc643, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x25cfc645, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68efc649, SUC_lexicon_tags_15 },
    { 0xbaafc64a, SUC_lexicon_tags_15 },
    { 0x88fc64b, SUC_lexicon_tags_0 },
    { 0x60afc64b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10efc64f, SUC_lexicon_tags_118 },
    { 0xeacfc64f, SUC_lexicon_tags_9 },
    { 0x800fc651, SUC_lexicon_tags_301 },
    { 0x4ecfc652, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666fc657, SUC_lexicon_tags_15 },
    { 0xbccfc657, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x462fc65a, SUC_lexicon_tags_48 },
    { 0x422fc65b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa62fc65f, SUC_lexicon_tags_0 },
    { 0x676fc660, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf2efc662, SUC_lexicon_tags_14 },
    { 0x374fc663, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbccfc665, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x468fc668, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596fc66c, SUC_lexicon_tags_84 },
    { 0xb46fc66c, SUC_lexicon_tags_28 },
    { 0x126fc66e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfc671, SUC_lexicon_tags_60 },
    { 0xe40fc671, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb90fc674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa8fc676, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb04fc679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4fc67c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x338fc67f, SUC_lexicon_tags_39 },
    { 0x258fc680, SUC_lexicon_tags_2 },
    { 0x19afc67f, SUC_lexicon_tags_13 },
    { 0x5e0fc682, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc20fc687, SUC_lexicon_tags_27 },
    { 0x158fc687, SUC_lexicon_tags_105 },
    { 0x7f2fc688, SUC_lexicon_tags_9 },
    { 0x346fc68a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14fc68e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2fc692, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1defc694, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcfc699, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x81afc69b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f8fc69d, SUC_lexicon_tags_17 },
    { 0xdc2fc69e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee6fc6a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x806fc6a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316fc6ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x666fc6b1, SUC_lexicon_tags_27 },
    { 0xde6fc6b2, SUC_lexicon_tags_48 },
    { 0x20cfc6b2, SUC_lexicon_tags_27 },
    { 0xeb8fc6b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12fc6b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dafc6bc, SUC_lexicon_tags_28 },
    { 0x1eefc6bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa84fc6c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x262fc6c4, SUC_lexicon_tags_47 },
    { 0xfcefc6c4, SUC_lexicon_tags_7 },
    { 0x482fc6c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4fc6c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x814fc6cb, SUC_lexicon_tags_1 },
    { 0xabefc6cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1efc6cf, SUC_lexicon_tags_1 },
    { 0x4a4fc6cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x658fc6d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf14fc6e6, SUC_lexicon_tags_13 },
    { 0xf5cfc6e7, SUC_lexicon_tags_15 },
    { 0x3f4fc6e8, SUC_lexicon_tags_12 },
    { 0xc22fc6e7, SUC_lexicon_tags_68 },
    { 0x84afc6ea, SUC_lexicon_tags_1 },
    { 0xfa6fc6ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c0fc6ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x976fc6ef, SUC_lexicon_tags_11 },
    { 0xf3efc6f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0fc6f4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efc6fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76fc6fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc705, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cfc709, SUC_lexicon_tags_27 },
    { 0xf52fc70a, SUC_lexicon_tags_1 },
    { 0x630fc709, SUC_lexicon_tags_60 },
    { 0x17afc70a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70afc712, SUC_lexicon_tags_30 },
    { 0x45efc713, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x792fc715, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x422fc718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0fc71e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93efc727, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x180fc72a, SUC_lexicon_tags_179 },
    { 0x430fc72b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38fc72e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf66fc731, SUC_lexicon_tags_12 },
    { 0xbf0fc732, SUC_lexicon_tags_17 },
    { 0x2a8fc732, SUC_lexicon_tags_2 },
    { 0xc40fc734, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x822fc737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a8fc739, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x670fc73c, SUC_lexicon_tags_21 },
    { 0x1fcfc73d, SUC_lexicon_tags_7 },
    { 0xf9efc73e, SUC_lexicon_tags_9 },
    { 0x200fc73e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f0fc741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe88fc749, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592fc755, SUC_lexicon_tags_7 },
    { 0x1b0fc756, SUC_lexicon_tags_23 },
    { 0x61efc757, SUC_lexicon_tags_1 },
    { 0x920fc755, SUC_lexicon_tags_2 },
    { 0xf92fc759, SUC_lexicon_tags_56 },
    { 0x482fc75a, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xb20fc75c, SUC_lexicon_tags_68 },
    { 0x4acfc75c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffafc764, SUC_lexicon_tags_28 },
    { 0x1fcfc765, SUC_lexicon_tags_1 },
    { 0xa9cfc766, SUC_lexicon_tags_33 },
    { 0xc7cfc767, SUC_lexicon_tags_0 },
    { 0xfe4fc768, SUC_lexicon_tags_92 },
    { 0x452fc767, SUC_lexicon_tags_11 },
    { 0x2d2fc767, SUC_lexicon_tags_2 },
    { 0xa66fc76b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc7efc76d, SUC_lexicon_tags_38 },
    { 0xf00fc76e, SUC_lexicon_tags_34 },
    { 0x91efc76e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaafc772, SUC_lexicon_tags_12 },
    { 0x34cfc773, SUC_lexicon_tags_2 },
    { 0x4b0fc774, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cfc778, SUC_lexicon_tags_38 },
    { 0xb52fc779, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28cfc77b, SUC_lexicon_tags_25 },
    { 0xf40fc77c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa12fc77f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xadcfc782, SUC_lexicon_tags_6 },
    { 0x65efc782, SUC_lexicon_tags_15 },
    { 0xfbcfc783, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496fc791, SUC_lexicon_tags_17 },
    { 0x9a2fc792, SUC_lexicon_tags_14 },
    { 0x90cfc793, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x72cfc795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0efc797, SUC_lexicon_tags_60 },
    { 0xc4afc798, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd5efc79a, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0x36afc79c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa84fc79f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6fc7a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16afc7a7, SUC_lexicon_tags_15 },
    { 0xd8efc7a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeaafc7aa, SUC_lexicon_tags_34 },
    { 0xeaafc7aa, SUC_lexicon_tags_0 },
    { 0xd94fc7aa, SUC_lexicon_tags_42 },
    { 0xcd8fc7ad, SUC_lexicon_tags_2 },
    { 0xaf8fc7ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1efc7b1, SUC_lexicon_tags_46 },
    { 0x1b4fc7b2, SUC_lexicon_tags_17 },
    { 0xf62fc7b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0cfc7b5, SUC_lexicon_tags_7 },
    { 0x460fc7b6, SUC_lexicon_tags_45 },
    { 0x2e8fc7b6, SUC_lexicon_tags_1 },
    { 0x22cfc7b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fc7bc, SUC_lexicon_tags_68 },
    { 0x454fc7bd, SUC_lexicon_tags_33 },
    { 0x442fc7be, SUC_lexicon_tags_7 },
    { 0x2d6fc7bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76fc7c3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94fc7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a4fc7c9, SUC_lexicon_tags_8 },
    { 0x642fc7ca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a2fc7cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876fc7d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aafc7d8, SUC_lexicon_tags_2 },
    { 0x94efc7d8, SUC_lexicon_tags_17 },
    { 0x6c4fc7da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100fc7de, SUC_lexicon_tags_17 },
    { 0xfe6fc7df, SUC_lexicon_tags_14 },
    { 0xb36fc7de, SUC_lexicon_tags_11 },
    { 0xbf0fc7e1, SUC_lexicon_tags_60 },
    { 0x956fc7e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32fc7e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78afc7ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x752fc7f1, SUC_lexicon_tags_2 },
    { 0x26cfc7f2, SUC_lexicon_tags_15 },
    { 0x4e4fc7f3, SUC_lexicon_tags_74 },
    { 0xb16fc7f3, SUC_lexicon_tags_17 },
    { 0x40efc7f3, SUC_lexicon_tags_32 },
    { 0xd9afc7f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc2fc7fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xabcfc7ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd14fc801, SUC_lexicon_tags_63 },
    { 0xc62fc802, SUC_lexicon_tags_17 },
    { 0xcc2fc802, SUC_lexicon_tags_25 },
    { 0xc26fc804, SUC_lexicon_tags_33 },
    { 0x42efc804, SUC_lexicon_tags_17 },
    { 0x14afc803, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa68fc809, SUC_lexicon_tags_15 },
    { 0xcafc80a, SUC_lexicon_tags_120 },
    { 0xa4efc80b, SUC_lexicon_tags_37 },
    { 0x4aefc80b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32fc812, SUC_lexicon_tags_7 },
    { 0x390fc813, SUC_lexicon_tags_17 },
    { 0xf88fc814, SUC_lexicon_tags_27 },
    { 0x7fefc815, SUC_lexicon_tags_17 },
    { 0x634fc815, SUC_lexicon_tags_46 },
    { 0xab4fc817, SUC_lexicon_tags_11 },
    { 0x7eafc818, SUC_lexicon_tags_15 },
    { 0xd66fc819, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4afc81c, SUC_lexicon_tags_11 },
    { 0xb9afc81c, SUC_lexicon_tags_61 },
    { 0xd10fc81e, SUC_lexicon_tags_2 },
    { 0x51efc81f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8eafc821, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x804fc823, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168fc82c, SUC_lexicon_tags_31 },
    { 0x3c0fc82d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x938fc830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcfc839, SUC_lexicon_tags_11 },
    { 0x3f0fc83a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x898fc83c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cefc83f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11cfc843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0cfc845, SUC_lexicon_tags_11 },
    { 0x464fc845, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe38fc848, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf10fc84a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610fc84e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6fc852, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaaefc854, SUC_lexicon_tags_17 },
    { 0xf2fc855, SUC_lexicon_tags_60 },
    { 0xc34fc856, SUC_lexicon_tags_12 },
    { 0x51cfc856, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6efc85a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00fc85f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ccfc862, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5c8fc864, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610fc868, SUC_lexicon_tags_27 },
    { 0xc2cfc868, SUC_lexicon_tags_17 },
    { 0xc0efc868, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fc870, SUC_lexicon_tags_68 },
    { 0x180fc870, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xabafc874, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x440fc876, SUC_lexicon_tags_13 },
    { 0xf18fc877, SUC_lexicon_tags_30 },
    { 0x18fc877, SUC_lexicon_tags_26 },
    { 0x424fc877, SUC_lexicon_tags_69 },
    { 0xdeefc87a, SUC_lexicon_tags_23 },
    { 0x12fc87b, SUC_lexicon_tags_15 },
    { 0x394fc87b, SUC_lexicon_tags_26 },
    { 0xfcfc87d, SUC_lexicon_tags_35 },
    { 0x2e2fc87d, SUC_lexicon_tags_2 },
    { 0x2f0fc87e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa40fc882, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7aefc884, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce4fc888, SUC_lexicon_tags_11 },
    { 0x924fc888, SUC_lexicon_tags_19 },
    { 0xcc0fc889, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xab4fc88d, SUC_lexicon_tags_11 },
    { 0x6defc88d, SUC_lexicon_tags_17 },
    { 0xbccfc88f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8fc898, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432fc89c, SUC_lexicon_tags_23 },
    { 0xa00fc89d, SUC_lexicon_tags_27 },
    { 0x39afc89e, SUC_lexicon_tags_12 },
    { 0x8dafc89f, SUC_lexicon_tags_4 },
    { 0xca8fc89f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x600fc8a2, SUC_lexicon_tags_17 },
    { 0x71cfc8a3, SUC_lexicon_tags_19 },
    { 0x9bcfc8a4, SUC_lexicon_tags_25 },
    { 0x364fc8a5, SUC_lexicon_tags_27 },
    { 0x57afc8a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18fc8a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfd8fc8ab, SUC_lexicon_tags_37 },
    { 0x1c4fc8ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x658fc8ae, SUC_lexicon_tags_39 },
    { 0x518fc8ae, SUC_lexicon_tags_15 },
    { 0x7aefc8b0, SUC_lexicon_tags_77 },
    { 0x91afc8b1, SUC_lexicon_tags_11 },
    { 0x776fc8b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x576fc8b4, SUC_lexicon_tags_40 },
    { 0x9ecfc8b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2fc8b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a8fc8bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4fc8c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92fc8cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69afc8d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78fc8d9, SUC_lexicon_tags_7 },
    { 0x89efc8da, SUC_lexicon_tags_55 },
    { 0x10efc8db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe4fc8de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x224fc8e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x660fc8e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7befc8e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc06fc8e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28efc8f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x862fc8f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe16fc8f5, SUC_lexicon_tags_54 },
    { 0xe46fc8f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e2fc8fd, SUC_lexicon_tags_79 },
    { 0x1d8fc8fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb26fc900, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xca6fc902, SUC_lexicon_tags_484 },
    { 0xce0fc903, SUC_lexicon_tags_26 },
    { 0xb64fc904, SUC_lexicon_tags_25 },
    { 0xd38fc905, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe94fc908, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fc90b, SUC_lexicon_tags_26 },
    { 0xadcfc90b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcafc910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfc914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x766fc918, SUC_lexicon_tags_23 },
    { 0x500fc919, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fc922, SUC_lexicon_tags_0 },
    { 0x2fcfc922, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e6fc92a, SUC_lexicon_tags_0 },
    { 0xca2fc92b, SUC_lexicon_tags_49 },
    { 0xd60fc92c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe88fc92e, SUC_lexicon_tags_4 },
    { 0x88efc92f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x368fc931, SUC_lexicon_tags_25 },
    { 0x992fc931, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3befc934, SUC_lexicon_tags_49 },
    { 0x8a6fc935, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x858fc938, SUC_lexicon_tags_1 },
    { 0x272fc939, SUC_lexicon_tags_60 },
    { 0x784fc938, SUC_lexicon_tags_23 },
    { 0x6c6fc93b, SUC_lexicon_tags_45 },
    { 0x884fc939, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93afc940, SUC_lexicon_tags_37 },
    { 0x75efc940, SUC_lexicon_tags_11 },
    { 0xaa0fc941, SUC_lexicon_tags_9 },
    { 0x586fc943, SUC_lexicon_tags_3 },
    { 0xca4fc944, SUC_lexicon_tags_25 },
    { 0xbeafc945, SUC_lexicon_tags_25 },
    { 0xc1afc940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x72efc949, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf74fc94b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474fc956, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedcfc95e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x902fc960, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63efc964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fafc967, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39efc96c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0fc971, SUC_lexicon_tags_19 },
    { 0x8c8fc972, SUC_lexicon_tags_11 },
    { 0xb36fc973, SUC_lexicon_tags_1 },
    { 0xc9efc973, SUC_lexicon_tags_11 },
    { 0xc10fc974, SUC_lexicon_tags_12 },
    { 0x67efc976, SUC_lexicon_tags_58 },
    { 0x378fc974, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546fc97d, SUC_lexicon_tags_6 },
    { 0x6bcfc97d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x462fc980, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x96efc982, SUC_lexicon_tags_27 },
    { 0xaf6fc982, SUC_lexicon_tags_25 },
    { 0xe2fc984, SUC_lexicon_tags_23 },
    { 0x270fc985, SUC_lexicon_tags_17 },
    { 0xf62fc986, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74fc98a, SUC_lexicon_tags_12 },
    { 0xf74fc98b, SUC_lexicon_tags_55 },
    { 0x7e6fc98c, SUC_lexicon_tags_9 },
    { 0xa72fc98c, SUC_lexicon_tags_8 },
    { 0xbb8fc98b, SUC_lexicon_tags_37 },
    { 0x8f2fc98c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacafc996, SUC_lexicon_tags_42 },
    { 0x834fc996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x402fc999, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89afc99f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754fc9a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0fc9a8, SUC_lexicon_tags_6 },
    { 0x9c6fc9a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54efc9ab, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xd08fc9ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48efc9af, SUC_lexicon_tags_4 },
    { 0xea6fc9b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fafc9ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64cfc9bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaefc9bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382fc9c5, SUC_lexicon_tags_15 },
    { 0xd8fc9c6, SUC_lexicon_tags_11 },
    { 0x6b0fc9c7, SUC_lexicon_tags_15 },
    { 0xb36fc9c8, SUC_lexicon_tags_12 },
    { 0xd9afc9c9, SUC_lexicon_tags_11 },
    { 0xbe4fc9c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x680fc9cc, SUC_lexicon_tags_23 },
    { 0x126fc9cd, SUC_lexicon_tags_55 },
    { 0xcf4fc9cd, SUC_lexicon_tags_32 },
    { 0x814fc9cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a6fc9d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4fc9d5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700fc9da, SUC_lexicon_tags_27 },
    { 0x180fc9db, SUC_lexicon_tags_2 },
    { 0x85cfc9dc, SUC_lexicon_tags_14 },
    { 0xb92fc9dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0fc9e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf2fc9e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbefc9e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682fc9ed, SUC_lexicon_tags_30 },
    { 0xd08fc9ee, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56fc9f4, SUC_lexicon_tags_17 },
    { 0xf38fc9f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27afc9f7, SUC_lexicon_tags_7 },
    { 0x296fc9f7, SUC_lexicon_tags_15 },
    { 0x47efc9f9, SUC_lexicon_tags_11 },
    { 0xcb4fc9fa, SUC_lexicon_tags_14 },
    { 0x66cfc9fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84afca00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eafca03, SUC_lexicon_tags_35 },
    { 0x4ecfca04, SUC_lexicon_tags_6 },
    { 0x396fca04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526fca09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36fca0c, SUC_lexicon_tags_7 },
    { 0xd4cfca0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x944fca12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24efca17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36fca19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd8fca1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4fca1e, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c8fca27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5aefca29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67efca2d, SUC_lexicon_tags_2 },
    { 0x7afca2e, SUC_lexicon_tags_25 },
    { 0x720fca2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd92fca32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22fca35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1befca38, SUC_lexicon_tags_1 },
    { 0xa90fca39, SUC_lexicon_tags_11 },
    { 0x84afca38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x770fca3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x144fca40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4fca4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7cfca4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25afca4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5afca52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bafca54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758fca58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdeafca5a, SUC_lexicon_tags_63 },
    { 0xbf0fca5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x566fca5e, SUC_lexicon_tags_25 },
    { 0x1f0fca5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4fca63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d0fca65, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa7afca67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x98afca69, SUC_lexicon_tags_25 },
    { 0xe84fca6a, SUC_lexicon_tags_23 },
    { 0xfa2fca6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x666fca6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x648fca70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0fca75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40afca7a, SUC_lexicon_tags_14 },
    { 0x20cfca7b, SUC_lexicon_tags_7 },
    { 0xff4fca7c, SUC_lexicon_tags_37 },
    { 0x54cfca7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3cfca7f, SUC_lexicon_tags_38 },
    { 0x190fca80, SUC_lexicon_tags_14 },
    { 0xd18fca81, SUC_lexicon_tags_12 },
    { 0xb70fca81, SUC_lexicon_tags_2 },
    { 0x932fca81, SUC_lexicon_tags_30 },
    { 0xd7afca84, SUC_lexicon_tags_12 },
    { 0x818fca85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3efca8b, SUC_lexicon_tags_17 },
    { 0x4cfca8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc72fca90, SUC_lexicon_tags_12 },
    { 0xc8fca91, SUC_lexicon_tags_15 },
    { 0x264fca92, SUC_lexicon_tags_11 },
    { 0xbaefca93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb2fca95, SUC_lexicon_tags_37 },
    { 0x262fca95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fca99, SUC_lexicon_tags_29 },
    { 0xa36fca9a, SUC_lexicon_tags_63 },
    { 0x98fca9a, SUC_lexicon_tags_7 },
    { 0xb38fca99, SUC_lexicon_tags_0 },
    { 0x64efca9d, SUC_lexicon_tags_15 },
    { 0x180fca9e, SUC_lexicon_tags_45 },
    { 0x7e8fca9b, SUC_lexicon_tags_38 },
    { 0x626fca9c, SUC_lexicon_tags_2 },
    { 0x2b4fca99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4fcaa7, SUC_lexicon_tags_33 },
    { 0x800fcaa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc6fcaaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8fcaae, SUC_lexicon_tags_11 },
    { 0xf04fcaae, SUC_lexicon_tags_25 },
    { 0x268fcaaf, SUC_lexicon_tags_15 },
    { 0xd10fcaae, SUC_lexicon_tags_19 },
    { 0xeeefcab2, SUC_lexicon_tags_23 },
    { 0xb4cfcab0, SUC_lexicon_tags_45 },
    { 0xafcab4, SUC_lexicon_tags_25 },
    { 0x610fcab5, SUC_lexicon_tags_29 },
    { 0x5aefcab6, SUC_lexicon_tags_15 },
    { 0x5e8fcab7, SUC_lexicon_tags_29 },
    { 0x2a8fcab8, SUC_lexicon_tags_37 },
    { 0xfb0fcab0, SUC_lexicon_tags_14 },
    { 0x496fcab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2efcabd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x210fcabf, SUC_lexicon_tags_37 },
    { 0x7aefcac0, SUC_lexicon_tags_37 },
    { 0xfd0fcac0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568fcac5, SUC_lexicon_tags_56 },
    { 0x39afcac6, SUC_lexicon_tags_1 },
    { 0xd7afcac5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x192fcac9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63afcacc, SUC_lexicon_tags_63 },
    { 0x710fcacd, SUC_lexicon_tags_87 },
    { 0xbb6fcace, SUC_lexicon_tags_25 },
    { 0xda2fcace, SUC_lexicon_tags_43 },
    { 0xeeefcad0, SUC_lexicon_tags_11 },
    { 0x1aafcacf, SUC_lexicon_tags_17 },
    { 0x41efcad2, SUC_lexicon_tags_11 },
    { 0x6d2fcad3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0fcad6, SUC_lexicon_tags_11 },
    { 0xa84fcad7, SUC_lexicon_tags_16 },
    { 0x1eafcad8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6fcadb, SUC_lexicon_tags_12 },
    { 0x90efcadc, SUC_lexicon_tags_34 },
    { 0xc8fcadb, SUC_lexicon_tags_11 },
    { 0x24afcade, SUC_lexicon_tags_19 },
    { 0x6efcadf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1efcae6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb52fcae8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdf6fcaea, SUC_lexicon_tags_37 },
    { 0xb48fcaeb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83cfcaed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87cfcaf6, SUC_lexicon_tags_1 },
    { 0x4ecfcaf7, SUC_lexicon_tags_39 },
    { 0x50efcaf7, SUC_lexicon_tags_26 },
    { 0x4a0fcaf7, SUC_lexicon_tags_12 },
    { 0x9d4fcafa, SUC_lexicon_tags_27 },
    { 0x602fcaf7, SUC_lexicon_tags_46 },
    { 0xf8efcaf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc4fcafe, SUC_lexicon_tags_84 },
    { 0x8b8fcafe, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefcb02, SUC_lexicon_tags_63 },
    { 0x2acfcb03, SUC_lexicon_tags_1 },
    { 0x672fcb04, SUC_lexicon_tags_11 },
    { 0x686fcb04, SUC_lexicon_tags_27 },
    { 0xaccfcb06, SUC_lexicon_tags_63 },
    { 0x4ccfcb04, SUC_lexicon_tags_32 },
    { 0x9eefcb08, SUC_lexicon_tags_36 },
    { 0xeccfcb09, SUC_lexicon_tags_31 },
    { 0xd56fcb03, SUC_lexicon_tags_11 },
    { 0xbe8fcb0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x724fcb0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73afcb0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x964fcb12, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5c2fcb14, SUC_lexicon_tags_15 },
    { 0x828fcb15, SUC_lexicon_tags_4 },
    { 0x84efcb16, SUC_lexicon_tags_7 },
    { 0x820fcb14, SUC_lexicon_tags_11 },
    { 0x610fcb18, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc46fcb1c, SUC_lexicon_tags_19 },
    { 0x414fcb1d, SUC_lexicon_tags_27 },
    { 0x57cfcb1d, SUC_lexicon_tags_27 },
    { 0x2ccfcb1f, SUC_lexicon_tags_19 },
    { 0x954fcb1f, SUC_lexicon_tags_2 },
    { 0x5befcb1d, SUC_lexicon_tags_2 },
    { 0xa6efcb1f, SUC_lexicon_tags_12 },
    { 0x44afcb23, SUC_lexicon_tags_7 },
    { 0x2cefcb24, SUC_lexicon_tags_0 },
    { 0x126fcb25, SUC_lexicon_tags_14 },
    { 0xe94fcb26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x70cfcb29, SUC_lexicon_tags_29 },
    { 0x1e8fcb2a, SUC_lexicon_tags_11 },
    { 0xa3cfcb2b, SUC_lexicon_tags_30 },
    { 0x39cfcb2c, SUC_lexicon_tags_37 },
    { 0xf16fcb2a, SUC_lexicon_tags_19 },
    { 0x7e4fcb2c, SUC_lexicon_tags_33 },
    { 0x148fcb2f, SUC_lexicon_tags_12 },
    { 0xcb8fcb30, SUC_lexicon_tags_4 },
    { 0x7dcfcb30, SUC_lexicon_tags_17 },
    { 0xd0afcb32, SUC_lexicon_tags_45 },
    { 0xc2fcb33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x180fcb38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2cefcb3a, SUC_lexicon_tags_37 },
    { 0xcaefcb3b, SUC_lexicon_tags_12 },
    { 0xf72fcb3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe90fcb40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae0fcb4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176fcb51, SUC_lexicon_tags_2 },
    { 0xa16fcb52, SUC_lexicon_tags_11 },
    { 0xd2cfcb52, SUC_lexicon_tags_1 },
    { 0xf2fcb52, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x94afcb56, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x436fcb58, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8fcb5b, SUC_lexicon_tags_2 },
    { 0x7b0fcb5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f2fcb5e, SUC_lexicon_tags_63 },
    { 0xa98fcb5f, SUC_lexicon_tags_44 },
    { 0xc32fcb5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x356fcb63, SUC_lexicon_tags_33 },
    { 0x28afcb64, SUC_lexicon_tags_35 },
    { 0xb9afcb63, SUC_lexicon_tags_12 },
    { 0x8e2fcb65, SUC_lexicon_tags_8 },
    { 0x228fcb67, SUC_lexicon_tags_29 },
    { 0x900fcb63, SUC_lexicon_tags_43 },
    { 0x2f4fcb69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54fcb6b, SUC_lexicon_tags_4 },
    { 0x5d6fcb6c, SUC_lexicon_tags_159 },
    { 0x304fcb6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efcb70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa88fcb74, SUC_lexicon_tags_7 },
    { 0x794fcb74, SUC_lexicon_tags_39 },
    { 0x9aefcb76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56fcb7d, SUC_lexicon_tags_28 },
    { 0x79afcb7d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cfcb81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x280fcb83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb84fcb87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x166fcb89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47efcb92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3afcb94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8fcb97, SUC_lexicon_tags_15 },
    { 0x7f4fcb97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aefcba6, SUC_lexicon_tags_92 },
    { 0xcd0fcba6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x434fcbaa, SUC_lexicon_tags_2 },
    { 0x5d2fcbab, SUC_lexicon_tags_25 },
    { 0xe78fcbac, SUC_lexicon_tags_6 },
    { 0xed4fcbad, SUC_lexicon_tags_31 },
    { 0x464fcbae, SUC_lexicon_tags_17 },
    { 0xc22fcbaf, SUC_lexicon_tags_9 },
    { 0xd72fcbb0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4afcbb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94efcbb6, SUC_lexicon_tags_25 },
    { 0xedafcbb6, SUC_lexicon_tags_7 },
    { 0xd00fcbb7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23efcbbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8fcbc2, SUC_lexicon_tags_37 },
    { 0xceafcbc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4fcbc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5befcbc8, SUC_lexicon_tags_15 },
    { 0x998fcbc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2afcbcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83efcbd6, SUC_lexicon_tags_37 },
    { 0x384fcbd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdeefcbd9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x182fcbdc, SUC_lexicon_tags_33 },
    { 0xefefcbdd, SUC_lexicon_tags_45 },
    { 0xa48fcbdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecfcbe5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa26fcbe8, SUC_lexicon_tags_25 },
    { 0x550fcbe9, SUC_lexicon_tags_1 },
    { 0x540fcbea, SUC_lexicon_tags_12 },
    { 0xf52fcbea, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xbbafcbed, SUC_lexicon_tags_47 },
    { 0x8f4fcbee, SUC_lexicon_tags_33 },
    { 0xdf8fcbef, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4efcbf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc82fcbf8, SUC_lexicon_tags_11 },
    { 0xa08fcbf9, SUC_lexicon_tags_11 },
    { 0xe94fcbfa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x782fcbfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c2fcc00, SUC_lexicon_tags_63 },
    { 0x5defcc00, SUC_lexicon_tags_63 },
    { 0x120fcc02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf0fcc04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb18fcc0c, SUC_lexicon_tags_17 },
    { 0x90fcc0d, SUC_lexicon_tags_37 },
    { 0xe36fcc0e, SUC_lexicon_tags_77 },
    { 0x5b8fcc0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8fcc12, SUC_lexicon_tags_0 },
    { 0x2d0fcc12, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdcfcc15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x804fcc18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24afcc1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x88cfcc1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6afcc1f, SUC_lexicon_tags_40 },
    { 0xa32fcc1f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc92fcc22, SUC_lexicon_tags_49 },
    { 0xdd4fcc23, SUC_lexicon_tags_37 },
    { 0xeccfcc23, SUC_lexicon_tags_9 },
    { 0x3d2fcc25, SUC_lexicon_tags_55 },
    { 0xcb8fcc26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76efcc2d, SUC_lexicon_tags_32 },
    { 0xcd6fcc2e, SUC_lexicon_tags_14 },
    { 0x638fcc2f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4fcc33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x566fcc36, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fefcc3b, SUC_lexicon_tags_40 },
    { 0xba0fcc3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf06fcc41, SUC_lexicon_tags_25 },
    { 0xc26fcc42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x764fcc45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb80fcc48, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x5befcc4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe8fcc4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16fcc51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46efcc54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd20fcc5a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e2fcc5f, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0cfcc63, SUC_lexicon_tags_12 },
    { 0x36afcc63, SUC_lexicon_tags_1 },
    { 0xabafcc64, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd5afcc67, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6fcc71, SUC_lexicon_tags_0 },
    { 0x472fcc72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcfcc79, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x840fcc7d, SUC_lexicon_tags_17 },
    { 0xebcfcc7e, SUC_lexicon_tags_15 },
    { 0x7dafcc7f, SUC_lexicon_tags_17 },
    { 0x118fcc7f, SUC_lexicon_tags_37 },
    { 0x29efcc7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98fcc87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf54fcc89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcc8e, SUC_lexicon_tags_985 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cefcc96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x168fcc98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb72fcc9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x162fcca0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x184fcca2, SUC_lexicon_tags_38 },
    { 0xddefcca2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x516fcca5, SUC_lexicon_tags_17 },
    { 0x37cfcca6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecafccaa, SUC_lexicon_tags_7 },
    { 0x506fccaa, SUC_lexicon_tags_12 },
    { 0x37afccac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f2fccae, SUC_lexicon_tags_11 },
    { 0x580fccae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6efccb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x298fccb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x886fccb5, SUC_lexicon_tags_25 },
    { 0x600fccb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x242fccbd, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xa0afccbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xceefccc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0fccc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa4fccc9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6fccd1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x890fccd3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22efccd8, SUC_lexicon_tags_7 },
    { 0xb5afccd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x100fccdb, SUC_lexicon_tags_55 },
    { 0xb90fccdc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cfcce9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x56cfccec, SUC_lexicon_tags_46 },
    { 0x94fcced, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x200fccef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10fccf4, SUC_lexicon_tags_17 },
    { 0xb9efccf5, SUC_lexicon_tags_15 },
    { 0x42afccf6, SUC_lexicon_tags_2 },
    { 0xa94fccf5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa8cfccf9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fefccfc, SUC_lexicon_tags_7 },
    { 0x8defccfc, SUC_lexicon_tags_25 },
    { 0x608fccfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cfcd04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652fcd0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578fcd0e, SUC_lexicon_tags_17 },
    { 0x4cefcd0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8fcd12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x708fcd19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456fcd27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1afcd2a, SUC_lexicon_tags_19 },
    { 0x614fcd2b, SUC_lexicon_tags_1 },
    { 0xb3efcd2b, SUC_lexicon_tags_8 },
    { 0x45afcd2d, SUC_lexicon_tags_28 },
    { 0x80efcd2b, SUC_lexicon_tags_17 },
    { 0x8ccfcd2f, SUC_lexicon_tags_108 },
    { 0x974fcd2f, SUC_lexicon_tags_4 },
    { 0x336fcd31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12fcd36, SUC_lexicon_tags_19 },
    { 0xd6efcd37, SUC_lexicon_tags_16 },
    { 0x4bafcd38, SUC_lexicon_tags_27 },
    { 0xa1afcd38, SUC_lexicon_tags_11 },
    { 0x1d6fcd37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x11efcd3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27cfcd43, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x13cfcd46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddefcd48, SUC_lexicon_tags_38 },
    { 0x3d2fcd48, SUC_lexicon_tags_10 },
    { 0xe9efcd4a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x94fcd4c, SUC_lexicon_tags_47 },
    { 0xe10fcd4c, SUC_lexicon_tags_11 },
    { 0xa1afcd4d, SUC_lexicon_tags_63 },
    { 0xb88fcd4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560fcd54, SUC_lexicon_tags_25 },
    { 0x804fcd55, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x324fcd5a, SUC_lexicon_tags_12 },
    { 0x61cfcd5b, SUC_lexicon_tags_414 },
    { 0x704fcd5c, SUC_lexicon_tags_4 },
    { 0x36efcd5b, SUC_lexicon_tags_14 },
    { 0x514fcd5e, SUC_lexicon_tags_7 },
    { 0xcfefcd5a, SUC_lexicon_tags_26 },
    { 0x9b6fcd60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4afcd64, SUC_lexicon_tags_42 },
    { 0xe70fcd64, SUC_lexicon_tags_43 },
    { 0xd44fcd66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fcd6a, SUC_lexicon_tags_15 },
    { 0xf48fcd6b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792fcd71, SUC_lexicon_tags_17 },
    { 0x1fefcd71, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xed8fcd74, SUC_lexicon_tags_48 },
    { 0xeefcd75, SUC_lexicon_tags_12 },
    { 0x66efcd74, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70afcd7a, SUC_lexicon_tags_0 },
    { 0x2a2fcd7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850fcd83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x602fcd88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13efcd8b, SUC_lexicon_tags_178 },
    { 0x94efcd8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0fcd8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf90fcd92, SUC_lexicon_tags_7 },
    { 0xdc0fcd93, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62efcd99, SUC_lexicon_tags_61 },
    { 0x38fcd9a, SUC_lexicon_tags_26 },
    { 0x2b0fcd9b, SUC_lexicon_tags_25 },
    { 0xba4fcd9c, SUC_lexicon_tags_23 },
    { 0x49afcd9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd28fcd9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe16fcda1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86fcda5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1cfcda7, SUC_lexicon_tags_15 },
    { 0xb96fcda7, SUC_lexicon_tags_63 },
    { 0x700fcda8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf46fcdb0, SUC_lexicon_tags_7 },
    { 0x282fcdb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58fcdb5, SUC_lexicon_tags_46 },
    { 0xb54fcdb5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcfcdb9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x116fcdbc, SUC_lexicon_tags_12 },
    { 0xd68fcdbd, SUC_lexicon_tags_1 },
    { 0x592fcdbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acfcdc4, SUC_lexicon_tags_31 },
    { 0xe6afcdc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb04fcdc7, SUC_lexicon_tags_17 },
    { 0xe00fcdc8, SUC_lexicon_tags_30 },
    { 0x34efcdc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76fcdcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf36fcdd1, SUC_lexicon_tags_87 },
    { 0x938fcdd2, SUC_lexicon_tags_11 },
    { 0x592fcdd3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2efcdd8, SUC_lexicon_tags_15 },
    { 0xa8fcdd9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc68fcde1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24fcde7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4fcdee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x278fcdf0, SUC_lexicon_tags_49 },
    { 0x958fcdf1, SUC_lexicon_tags_7 },
    { 0x3efcdf2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x786fcdf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43afcdf8, SUC_lexicon_tags_25 },
    { 0xc0cfcdf9, SUC_lexicon_tags_47 },
    { 0xf38fcdfa, SUC_lexicon_tags_66 },
    { 0xa56fcdfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6afce03, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9afce07, SUC_lexicon_tags_35 },
    { 0x3aafce08, SUC_lexicon_tags_64 },
    { 0x846fce09, SUC_lexicon_tags_1 },
    { 0x67cfce07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4fce0e, SUC_lexicon_tags_25 },
    { 0xbaafce0f, SUC_lexicon_tags_60 },
    { 0x902fce0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494fce16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6fefce18, SUC_lexicon_tags_25 },
    { 0x54efce18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa06fce1b, SUC_lexicon_tags_31 },
    { 0x686fce1c, SUC_lexicon_tags_28 },
    { 0x14fce1c, SUC_lexicon_tags_29 },
    { 0x38efce1e, SUC_lexicon_tags_8 },
    { 0x378fce1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fce26, SUC_lexicon_tags_108 },
    { 0xef4fce27, SUC_lexicon_tags_38 },
    { 0x616fce28, SUC_lexicon_tags_12 },
    { 0x860fce29, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x256fce2b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaf6fce2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfce30, SUC_lexicon_tags_61 },
    { 0x824fce30, SUC_lexicon_tags_37 },
    { 0x8d4fce32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c0fce3a, SUC_lexicon_tags_12 },
    { 0xa4afce3b, SUC_lexicon_tags_19 },
    { 0xa24fce3c, SUC_lexicon_tags_12 },
    { 0xc5cfce3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fce41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cafce45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622fce4d, SUC_lexicon_tags_1 },
    { 0xe8afce4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6fce53, SUC_lexicon_tags_0 },
    { 0x942fce53, SUC_lexicon_tags_29 },
    { 0xc62fce55, SUC_lexicon_tags_10 },
    { 0x3b0fce54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5afce58, SUC_lexicon_tags_1102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334fce5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cfce63, SUC_lexicon_tags_12 },
    { 0xdd6fce63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x698fce67, SUC_lexicon_tags_25 },
    { 0x9e4fce68, SUC_lexicon_tags_4 },
    { 0xa6efce67, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7a8fce6b, SUC_lexicon_tags_60 },
    { 0x9c4fce6c, SUC_lexicon_tags_37 },
    { 0xcd0fce6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb28fce74, SUC_lexicon_tags_7 },
    { 0xd30fce75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122fce79, SUC_lexicon_tags_63 },
    { 0xb36fce7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42fce7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33afce83, SUC_lexicon_tags_12 },
    { 0xceafce84, SUC_lexicon_tags_15 },
    { 0xb16fce85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c4fce89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62afce90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4afce93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x762fce96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0fce99, SUC_lexicon_tags_11 },
    { 0x84afce9a, SUC_lexicon_tags_62 },
    { 0xa5afce9a, SUC_lexicon_tags_17 },
    { 0xdefce9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfeafce9e, SUC_lexicon_tags_47 },
    { 0xde8fce9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9efcea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8cfcea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcefcea9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e0fceac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd4fceae, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6fceb2, SUC_lexicon_tags_28 },
    { 0x812fceb3, SUC_lexicon_tags_26 },
    { 0xae8fceb3, SUC_lexicon_tags_7 },
    { 0x108fceb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20afceb8, SUC_lexicon_tags_84 },
    { 0x490fceb9, SUC_lexicon_tags_28 },
    { 0x25afceb9, SUC_lexicon_tags_11 },
    { 0xc4afcebb, SUC_lexicon_tags_56 },
    { 0xc02fceb9, SUC_lexicon_tags_27 },
    { 0x2b0fcebd, SUC_lexicon_tags_17 },
    { 0x248fcebd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd60fcec0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4fcec3, SUC_lexicon_tags_6 },
    { 0x544fcec3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x210fcec7, SUC_lexicon_tags_15 },
    { 0x9e2fcec8, SUC_lexicon_tags_1 },
    { 0xc8afcec9, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0x5aefcecb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74fced0, SUC_lexicon_tags_1 },
    { 0xd2efced0, SUC_lexicon_tags_33 },
    { 0x9e4fced2, SUC_lexicon_tags_16 },
    { 0xe6efced2, SUC_lexicon_tags_33 },
    { 0xef2fced4, SUC_lexicon_tags_23 },
    { 0x2d6fced5, SUC_lexicon_tags_47 },
    { 0xdccfced2, SUC_lexicon_tags_25 },
    { 0x574fced7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1e8fced9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240fcee0, SUC_lexicon_tags_47 },
    { 0xefafcee1, SUC_lexicon_tags_1 },
    { 0x6aefcee2, SUC_lexicon_tags_37 },
    { 0x1a6fcee1, SUC_lexicon_tags_48 },
    { 0xd5efcee4, SUC_lexicon_tags_25 },
    { 0xab0fcee5, SUC_lexicon_tags_17 },
    { 0xb72fcee5, SUC_lexicon_tags_17 },
    { 0x4fafcee7, SUC_lexicon_tags_29 },
    { 0x3acfcee6, SUC_lexicon_tags_66 },
    { 0xa8cfcee9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x958fceeb, SUC_lexicon_tags_7 },
    { 0x2e2fceeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd6fceee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x476fcef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5eafcef2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94cfcef6, SUC_lexicon_tags_11 },
    { 0x490fcef6, SUC_lexicon_tags_38 },
    { 0xb18fcef8, SUC_lexicon_tags_28 },
    { 0x1defcef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ecfcefb, SUC_lexicon_tags_25 },
    { 0xe70fcefc, SUC_lexicon_tags_11 },
    { 0x72afcefb, SUC_lexicon_tags_17 },
    { 0xcc2fcefe, SUC_lexicon_tags_22 },
    { 0x96cfcefe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76fcf07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x552fcf0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6fcf11, SUC_lexicon_tags_25 },
    { 0x6b0fcf11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2fcf18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcfcf1b, SUC_lexicon_tags_1 },
    { 0x60efcf1b, SUC_lexicon_tags_4 },
    { 0x4cafcf1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x222fcf1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x846fcf21, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcfcf25, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3befcf28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3efcf2d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefcf30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fefcf34, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc56fcf36, SUC_lexicon_tags_1 },
    { 0xa8fcf36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ecfcf43, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf48fcf4b, SUC_lexicon_tags_17 },
    { 0x606fcf4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63cfcf4e, SUC_lexicon_tags_37 },
    { 0x88cfcf4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a4fcf51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43afcf54, SUC_lexicon_tags_43 },
    { 0x26efcf54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fcf59, SUC_lexicon_tags_15 },
    { 0x352fcf5a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76efcf68, SUC_lexicon_tags_11 },
    { 0x43efcf69, SUC_lexicon_tags_11 },
    { 0x44efcf6a, SUC_lexicon_tags_43 },
    { 0x9e6fcf6b, SUC_lexicon_tags_40 },
    { 0x2e0fcf6b, SUC_lexicon_tags_60 },
    { 0x608fcf6d, SUC_lexicon_tags_6 },
    { 0xea0fcf6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x70cfcf70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59cfcf75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21afcf77, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6fcf7b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdbefcf7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2efcf80, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x368fcf82, SUC_lexicon_tags_15 },
    { 0xa52fcf83, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfcf88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x84cfcf8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fcf8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fcf90, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bafcf95, SUC_lexicon_tags_25 },
    { 0x894fcf95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90cfcf99, SUC_lexicon_tags_37 },
    { 0xed4fcf9a, SUC_lexicon_tags_17 },
    { 0xddafcf9b, SUC_lexicon_tags_17 },
    { 0xc0efcf9a, SUC_lexicon_tags_14 },
    { 0x722fcf9d, SUC_lexicon_tags_27 },
    { 0x4b4fcf9e, SUC_lexicon_tags_47 },
    { 0x61efcf9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x498fcfa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cfcfa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x268fcfa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80cfcfaa, SUC_lexicon_tags_203 },
    { 0x738fcfab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x328fcfad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cfcfb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734fcfb5, SUC_lexicon_tags_87 },
    { 0xf68fcfb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x66afcfb8, SUC_lexicon_tags_2 },
    { 0x51efcfb9, SUC_lexicon_tags_7 },
    { 0x318fcfb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x690fcfbc, SUC_lexicon_tags_170 },
    { 0x98efcfbc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fcfc2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0afcfc8, SUC_lexicon_tags_2 },
    { 0x68efcfc9, SUC_lexicon_tags_11 },
    { 0x47efcfca, SUC_lexicon_tags_68 },
    { 0x5afcfca, SUC_lexicon_tags_11 },
    { 0xfb4fcfcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2fcfcf, SUC_lexicon_tags_37 },
    { 0x79efcfd0, SUC_lexicon_tags_38 },
    { 0x5b2fcfd1, SUC_lexicon_tags_2 },
    { 0xf5afcfd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x620fcfd8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x79cfcfda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x506fcfdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x648fcfde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8fcfe1, SUC_lexicon_tags_11 },
    { 0xd3efcfe2, SUC_lexicon_tags_37 },
    { 0x6e4fcfe3, SUC_lexicon_tags_1 },
    { 0xceafcfe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf0fcfe6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47afcfe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c6fcfed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2afcff0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x584fcff4, SUC_lexicon_tags_65 },
    { 0x2f6fcff5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x302fcff7, SUC_lexicon_tags_63 },
    { 0x3fcfcff7, SUC_lexicon_tags_1 },
    { 0x210fcff9, SUC_lexicon_tags_49 },
    { 0xb52fcffa, SUC_lexicon_tags_27 },
    { 0x4cefcffa, SUC_lexicon_tags_83 },
    { 0x7e4fcffc, SUC_lexicon_tags_6 },
    { 0xd44fcffd, SUC_lexicon_tags_85 },
    { 0xe5efcffe, SUC_lexicon_tags_60 },
    { 0x9c0fcffc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbe2fd001, SUC_lexicon_tags_16 },
    { 0x874fd002, SUC_lexicon_tags_2 },
    { 0xe02fd003, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeeefd005, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xebcfd007, SUC_lexicon_tags_17 },
    { 0x36efd007, SUC_lexicon_tags_7 },
    { 0x61cfd007, SUC_lexicon_tags_33 },
    { 0x380fd007, SUC_lexicon_tags_8 },
    { 0x782fd00b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x208fd00e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6afd011, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdeafd013, SUC_lexicon_tags_25 },
    { 0xcb4fd013, SUC_lexicon_tags_17 },
    { 0xb74fd015, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cfd019, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0afd01e, SUC_lexicon_tags_38 },
    { 0x6f0fd01f, SUC_lexicon_tags_17 },
    { 0x9b8fd01f, SUC_lexicon_tags_63 },
    { 0x83efd01f, SUC_lexicon_tags_8 },
    { 0xf4efd01e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf18fd024, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38fd028, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16fd02b, SUC_lexicon_tags_26 },
    { 0x336fd02b, SUC_lexicon_tags_0 },
    { 0x7d6fd02d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62fd032, SUC_lexicon_tags_66 },
    { 0x1fafd032, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54cfd036, SUC_lexicon_tags_9 },
    { 0x65cfd036, SUC_lexicon_tags_15 },
    { 0x86afd036, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378fd03f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5cfd041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8fd044, SUC_lexicon_tags_30 },
    { 0xcbafd045, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x908fd049, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfc0fd04b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x39afd04d, SUC_lexicon_tags_60 },
    { 0x9f6fd04e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8fd054, SUC_lexicon_tags_6 },
    { 0xb08fd055, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2fd058, SUC_lexicon_tags_25 },
    { 0xf8cfd059, SUC_lexicon_tags_17 },
    { 0xe34fd05a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x106fd05c, SUC_lexicon_tags_17 },
    { 0x898fd05d, SUC_lexicon_tags_33 },
    { 0x4ecfd05e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x890fd060, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaeafd062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6fd065, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x818fd076, SUC_lexicon_tags_68 },
    { 0xa42fd076, SUC_lexicon_tags_27 },
    { 0x3f2fd078, SUC_lexicon_tags_28 },
    { 0xb9cfd076, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20cfd07b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcfd080, SUC_lexicon_tags_25 },
    { 0xe82fd081, SUC_lexicon_tags_27 },
    { 0xa20fd082, SUC_lexicon_tags_7 },
    { 0x6eefd082, SUC_lexicon_tags_17 },
    { 0xa04fd084, SUC_lexicon_tags_12 },
    { 0x458fd084, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fd08c, SUC_lexicon_tags_29 },
    { 0xc18fd08d, SUC_lexicon_tags_53 },
    { 0x2d2fd08d, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8efd094, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6fd09a, SUC_lexicon_tags_15 },
    { 0x586fd09b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddefd0a0, SUC_lexicon_tags_7 },
    { 0x136fd0a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x506fd0a4, SUC_lexicon_tags_7 },
    { 0x28efd0a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdafd0aa, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890fd0af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380fd0b4, SUC_lexicon_tags_49 },
    { 0xa4cfd0b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc30fd0b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x514fd0ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x982fd0bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fd0c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x792fd0c2, SUC_lexicon_tags_38 },
    { 0x244fd0c2, SUC_lexicon_tags_4 },
    { 0x6aafd0c3, SUC_lexicon_tags_1 },
    { 0xca4fd0c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292fd0c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08fd0ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac4fd0d0, SUC_lexicon_tags_63 },
    { 0xcf8fd0d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38cfd0d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8fd0dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38cfd0df, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x27cfd0e1, SUC_lexicon_tags_37 },
    { 0x70efd0e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2fd0e6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x374fd0e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4fd0ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8fd0ef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10fd0f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfd0fa, SUC_lexicon_tags_14 },
    { 0x904fd0fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb86fd0ff, SUC_lexicon_tags_37 },
    { 0x5dcfd100, SUC_lexicon_tags_37 },
    { 0xa6fd0ff, SUC_lexicon_tags_37 },
    { 0x26afd102, SUC_lexicon_tags_17 },
    { 0x21afd100, SUC_lexicon_tags_11 },
    { 0x674fd104, SUC_lexicon_tags_37 },
    { 0xcecfd105, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x62afd108, SUC_lexicon_tags_11 },
    { 0x1afd109, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc18fd10b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4fd115, SUC_lexicon_tags_27 },
    { 0xa92fd116, SUC_lexicon_tags_33 },
    { 0xe44fd117, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cfd11a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4fd11e, SUC_lexicon_tags_4 },
    { 0xddefd11f, SUC_lexicon_tags_37 },
    { 0xa2cfd11f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18fd127, SUC_lexicon_tags_14 },
    { 0x40afd127, SUC_lexicon_tags_30 },
    { 0x2defd129, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xae8fd12c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06fd131, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2fd135, SUC_lexicon_tags_14 },
    { 0x40efd136, SUC_lexicon_tags_13 },
    { 0x798fd137, SUC_lexicon_tags_45 },
    { 0x498fd138, SUC_lexicon_tags_15 },
    { 0xb58fd138, SUC_lexicon_tags_9 },
    { 0x534fd138, SUC_lexicon_tags_11 },
    { 0x2e2fd13b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdacfd13d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60fd14b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cfd158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8fd15d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc90fd164, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xca8fd167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6fd16b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23cfd16d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x54afd16f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf96fd171, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972fd177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884fd17f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d2fd185, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x606fd188, SUC_lexicon_tags_14 },
    { 0xe32fd188, SUC_lexicon_tags_26 },
    { 0xafafd18a, SUC_lexicon_tags_28 },
    { 0x7dafd18a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2fd18e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c4fd19d, SUC_lexicon_tags_4 },
    { 0x9d4fd19e, SUC_lexicon_tags_12 },
    { 0xcbafd19f, SUC_lexicon_tags_65 },
    { 0x328fd19f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52fd1a7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6cfd1aa, SUC_lexicon_tags_17 },
    { 0xccfd1ab, SUC_lexicon_tags_43 },
    { 0x80afd1ac, SUC_lexicon_tags_29 },
    { 0xeb4fd1ad, SUC_lexicon_tags_4 },
    { 0x2d4fd1ad, SUC_lexicon_tags_15 },
    { 0xc14fd1ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x27afd1b2, SUC_lexicon_tags_21 },
    { 0xb42fd1b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17efd1b5, SUC_lexicon_tags_11 },
    { 0xbfafd1b5, SUC_lexicon_tags_55 },
    { 0x1e2fd1b6, SUC_lexicon_tags_4 },
    { 0xf88fd1b8, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd1bf, SUC_lexicon_tags_10 },
    { 0xb18fd1bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83efd1c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4fd1c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfecfd1c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bafd1d3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1c4fd1d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x328fd1d7, SUC_lexicon_tags_2 },
    { 0x26fd1d8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2fd1df, SUC_lexicon_tags_11 },
    { 0xef4fd1e0, SUC_lexicon_tags_10 },
    { 0xb18fd1e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x850fd1e3, SUC_lexicon_tags_8 },
    { 0xe04fd1e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x774fd1e7, SUC_lexicon_tags_17 },
    { 0xfdafd1e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce2fd1ea, SUC_lexicon_tags_15 },
    { 0xf62fd1ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x834fd1ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7f4fd1f0, SUC_lexicon_tags_12 },
    { 0xddafd1f1, SUC_lexicon_tags_12 },
    { 0x294fd1f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e2fd1f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136fd1f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9efd1ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x864fd201, SUC_lexicon_tags_2 },
    { 0x77efd201, SUC_lexicon_tags_7 },
    { 0x85afd203, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddafd209, SUC_lexicon_tags_35 },
    { 0x7e6fd20a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x734fd211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x426fd213, SUC_lexicon_tags_9 },
    { 0x8defd213, SUC_lexicon_tags_17 },
    { 0xafd214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2afd218, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2f8fd21a, SUC_lexicon_tags_30 },
    { 0x184fd21b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa90fd21d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4a0fd21f, SUC_lexicon_tags_38 },
    { 0x58fd21f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2fd227, SUC_lexicon_tags_49 },
    { 0x814fd227, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6cfd22a, SUC_lexicon_tags_11 },
    { 0xbfcfd22a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x624fd22f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd48fd232, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650fd236, SUC_lexicon_tags_9 },
    { 0xb72fd236, SUC_lexicon_tags_15 },
    { 0xbcefd238, SUC_lexicon_tags_14 },
    { 0x14efd239, SUC_lexicon_tags_43 },
    { 0x258fd23a, SUC_lexicon_tags_33 },
    { 0xf24fd236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce2fd23d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa2fd23f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x642fd241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x838fd243, SUC_lexicon_tags_37 },
    { 0xa40fd243, SUC_lexicon_tags_4 },
    { 0xc32fd243, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c0fd250, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa2fd252, SUC_lexicon_tags_26 },
    { 0x41cfd253, SUC_lexicon_tags_1 },
    { 0x20afd253, SUC_lexicon_tags_36 },
    { 0x268fd255, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x166fd257, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f2fd259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5dcfd25b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2fd263, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf60fd266, SUC_lexicon_tags_2 },
    { 0x86cfd267, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cfd26a, SUC_lexicon_tags_45 },
    { 0x4bcfd26a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5cefd26d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dafd270, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cfd273, SUC_lexicon_tags_2 },
    { 0xcd6fd273, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeaefd276, SUC_lexicon_tags_12 },
    { 0x876fd276, SUC_lexicon_tags_12 },
    { 0xbe2fd278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd08fd27b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x666fd27e, SUC_lexicon_tags_5 },
    { 0x226fd27e, SUC_lexicon_tags_16 },
    { 0x68afd280, SUC_lexicon_tags_47 },
    { 0x93efd280, SUC_lexicon_tags_8 },
    { 0xf64fd281, SUC_lexicon_tags_63 },
    { 0x74efd283, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6fd287, SUC_lexicon_tags_49 },
    { 0xe6efd288, SUC_lexicon_tags_55 },
    { 0x1e4fd289, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x938fd28c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748fd292, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x358fd294, SUC_lexicon_tags_45 },
    { 0x98fd294, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6fd29a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0fd29d, SUC_lexicon_tags_2 },
    { 0x578fd29e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3a8fd2a0, SUC_lexicon_tags_1 },
    { 0xbeefd2a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb0fd2a3, SUC_lexicon_tags_15 },
    { 0x300fd2a4, SUC_lexicon_tags_17 },
    { 0x1a8fd2a3, SUC_lexicon_tags_17 },
    { 0x77afd2a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fd2ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc8fd2b2, SUC_lexicon_tags_15 },
    { 0x6fd2b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd00fd2bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e4fd2bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebefd2c0, SUC_lexicon_tags_45 },
    { 0x328fd2c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf04fd2c3, SUC_lexicon_tags_1 },
    { 0x818fd2c3, SUC_lexicon_tags_7 },
    { 0x524fd2c3, SUC_lexicon_tags_55 },
    { 0x772fd2c5, SUC_lexicon_tags_7 },
    { 0x88cfd2c7, SUC_lexicon_tags_25 },
    { 0x6dafd2c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4acfd2cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x948fd2cf, SUC_lexicon_tags_2 },
    { 0x5f6fd2d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18fd2d4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf20fd2d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x278fd2da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd84fd2dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64cfd2e5, SUC_lexicon_tags_28 },
    { 0x2c0fd2e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x522fd2e8, SUC_lexicon_tags_19 },
    { 0xc26fd2e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8fd2f2, SUC_lexicon_tags_11 },
    { 0x7dcfd2f3, SUC_lexicon_tags_27 },
    { 0xe22fd2f2, SUC_lexicon_tags_23 },
    { 0xdc6fd2f5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96fd2fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7e4fd2fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3defd304, SUC_lexicon_tags_55 },
    { 0x4eefd304, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11efd309, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ccfd30b, SUC_lexicon_tags_70 },
    { 0x858fd30c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0fd313, SUC_lexicon_tags_0 },
    { 0x2fefd314, SUC_lexicon_tags_63 },
    { 0xfd4fd315, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa2fd321, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7efd323, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xebcfd325, SUC_lexicon_tags_0 },
    { 0x45cfd326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5eefd328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2fd32e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cfd333, SUC_lexicon_tags_6 },
    { 0x89cfd333, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6c4fd336, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1cfd338, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1cfd33a, SUC_lexicon_tags_40 },
    { 0xb8cfd33b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde0fd33d, SUC_lexicon_tags_25 },
    { 0x69cfd33e, SUC_lexicon_tags_29 },
    { 0x29afd33f, SUC_lexicon_tags_25 },
    { 0xfb2fd33e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x140fd342, SUC_lexicon_tags_19 },
    { 0xe28fd343, SUC_lexicon_tags_7 },
    { 0xbe6fd342, SUC_lexicon_tags_66 },
    { 0x8d8fd342, SUC_lexicon_tags_37 },
    { 0x146fd344, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b0fd348, SUC_lexicon_tags_26 },
    { 0x24fd348, SUC_lexicon_tags_0 },
    { 0x312fd349, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcfd34d, SUC_lexicon_tags_65 },
    { 0x682fd34e, SUC_lexicon_tags_37 },
    { 0x982fd34e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4fd35d, SUC_lexicon_tags_12 },
    { 0xe32fd35d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x460fd365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99afd369, SUC_lexicon_tags_17 },
    { 0xe1afd369, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe98fd36c, SUC_lexicon_tags_1192 },
    { 0, NULL },
    { 0, NULL },
    { 0x290fd36f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa02fd371, SUC_lexicon_tags_17 },
    { 0xbc4fd371, SUC_lexicon_tags_5 },
    { 0x556fd373, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6fd37a, SUC_lexicon_tags_87 },
    { 0xdc6fd37b, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672fd381, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382fd38b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe20fd38d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4a8fd38f, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x78efd391, SUC_lexicon_tags_43 },
    { 0xc2cfd391, SUC_lexicon_tags_2 },
    { 0xe12fd393, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe6fd395, SUC_lexicon_tags_26 },
    { 0xb76fd396, SUC_lexicon_tags_28 },
    { 0xfc8fd395, SUC_lexicon_tags_17 },
    { 0x332fd397, SUC_lexicon_tags_15 },
    { 0x6e8fd397, SUC_lexicon_tags_45 },
    { 0xc22fd39a, SUC_lexicon_tags_11 },
    { 0xdb6fd39b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b2fd39d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x858fd3a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfd3b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x882fd3b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2fd3bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3cfd3bf, SUC_lexicon_tags_9 },
    { 0x826fd3bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686fd3c6, SUC_lexicon_tags_16 },
    { 0xaaafd3c6, SUC_lexicon_tags_15 },
    { 0xd1efd3c8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8fd3cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55efd3d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a6fd3d8, SUC_lexicon_tags_17 },
    { 0xd84fd3d9, SUC_lexicon_tags_49 },
    { 0x52efd3da, SUC_lexicon_tags_12 },
    { 0x64cfd3da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd4fd3dd, SUC_lexicon_tags_47 },
    { 0x32afd3de, SUC_lexicon_tags_37 },
    { 0xe5cfd3dd, SUC_lexicon_tags_35 },
    { 0x176fd3e0, SUC_lexicon_tags_12 },
    { 0x4e8fd3e1, SUC_lexicon_tags_39 },
    { 0xf8efd3e1, SUC_lexicon_tags_87 },
    { 0x6eefd3de, SUC_lexicon_tags_11 },
    { 0xbcafd3e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9acfd3e6, SUC_lexicon_tags_15 },
    { 0xed6fd3e7, SUC_lexicon_tags_15 },
    { 0xa18fd3e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cfd3ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x642fd3ef, SUC_lexicon_tags_2 },
    { 0x20cfd3f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cfd3f3, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x84afd3f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafd3fc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4aefd3fe, SUC_lexicon_tags_33 },
    { 0x548fd3ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acfd403, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeaafd405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cfd40c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6fd40f, SUC_lexicon_tags_45 },
    { 0x27efd410, SUC_lexicon_tags_30 },
    { 0x5ccfd411, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x2d8fd413, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6fd415, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x860fd417, SUC_lexicon_tags_23 },
    { 0x86fd417, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24fd41e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32cfd423, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a2fd425, SUC_lexicon_tags_0 },
    { 0xa2afd426, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x692fd429, SUC_lexicon_tags_2 },
    { 0xb92fd429, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92fd432, SUC_lexicon_tags_56 },
    { 0x676fd433, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37efd435, SUC_lexicon_tags_42 },
    { 0xa60fd436, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8fd43e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9cfd441, SUC_lexicon_tags_12 },
    { 0x974fd442, SUC_lexicon_tags_15 },
    { 0x554fd442, SUC_lexicon_tags_49 },
    { 0xbe6fd444, SUC_lexicon_tags_42 },
    { 0x82fd445, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8afd449, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1efd44e, SUC_lexicon_tags_79 },
    { 0x7a6fd44f, SUC_lexicon_tags_53 },
    { 0xdb2fd450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd455, SUC_lexicon_tags_23 },
    { 0x35efd456, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xccefd458, SUC_lexicon_tags_28 },
    { 0xbe4fd459, SUC_lexicon_tags_7 },
    { 0x426fd459, SUC_lexicon_tags_25 },
    { 0x37efd45a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbacfd45e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc0fd460, SUC_lexicon_tags_17 },
    { 0x3befd461, SUC_lexicon_tags_1 },
    { 0x83cfd462, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17efd468, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x576fd46a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4fd46e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cfd472, SUC_lexicon_tags_19 },
    { 0x1dafd473, SUC_lexicon_tags_15 },
    { 0x702fd474, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430fd479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96fd47c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x876fd47f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70efd483, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcefd487, SUC_lexicon_tags_17 },
    { 0x1ccfd488, SUC_lexicon_tags_37 },
    { 0xfdafd489, SUC_lexicon_tags_25 },
    { 0xef4fd489, SUC_lexicon_tags_37 },
    { 0x99efd48b, SUC_lexicon_tags_19 },
    { 0x548fd48c, SUC_lexicon_tags_47 },
    { 0x7f4fd488, SUC_lexicon_tags_21 },
    { 0xb04fd489, SUC_lexicon_tags_66 },
    { 0x9f4fd48f, SUC_lexicon_tags_47 },
    { 0x7dcfd48e, SUC_lexicon_tags_34 },
    { 0x152fd491, SUC_lexicon_tags_28 },
    { 0x786fd490, SUC_lexicon_tags_51 },
    { 0xd28fd493, SUC_lexicon_tags_37 },
    { 0x68efd488, SUC_lexicon_tags_2 },
    { 0xf58fd492, SUC_lexicon_tags_46 },
    { 0x926fd494, SUC_lexicon_tags_59 },
    { 0x38efd497, SUC_lexicon_tags_66 },
    { 0x936fd48b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafd49b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x14afd49e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x440fd4a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x202fd4a4, SUC_lexicon_tags_12 },
    { 0x63efd4a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd0fd4a7, SUC_lexicon_tags_17 },
    { 0x236fd4a8, SUC_lexicon_tags_15 },
    { 0xa52fd4a8, SUC_lexicon_tags_47 },
    { 0xe92fd4aa, SUC_lexicon_tags_87 },
    { 0xac0fd4aa, SUC_lexicon_tags_123 },
    { 0x608fd4ac, SUC_lexicon_tags_12 },
    { 0x28fd4a8, SUC_lexicon_tags_17 },
    { 0x994fd4aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xecfd4b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2fd4b5, SUC_lexicon_tags_37 },
    { 0x6cfd4b6, SUC_lexicon_tags_27 },
    { 0xfa2fd4b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x91efd4b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc6fd4bc, SUC_lexicon_tags_33 },
    { 0x2e0fd4bc, SUC_lexicon_tags_60 },
    { 0x192fd4be, SUC_lexicon_tags_11 },
    { 0xb9afd4bd, SUC_lexicon_tags_15 },
    { 0xf10fd4bd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aefd4c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04fd4d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xacafd4d3, SUC_lexicon_tags_940 },
    { 0xd22fd4d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c6fd4d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd46fd4d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x388fd4db, SUC_lexicon_tags_17 },
    { 0x532fd4dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8efd4e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aafd4e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x672fd4e9, SUC_lexicon_tags_12 },
    { 0x930fd4ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9afd4f1, SUC_lexicon_tags_12 },
    { 0x7e2fd4f2, SUC_lexicon_tags_17 },
    { 0x5a8fd4f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x598fd4f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9cfd4fb, SUC_lexicon_tags_65 },
    { 0x674fd4fc, SUC_lexicon_tags_9 },
    { 0xe12fd4fd, SUC_lexicon_tags_13 },
    { 0x806fd4fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd24fd501, SUC_lexicon_tags_1 },
    { 0x196fd502, SUC_lexicon_tags_11 },
    { 0x80efd503, SUC_lexicon_tags_5 },
    { 0x98fd504, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaefd50b, SUC_lexicon_tags_19 },
    { 0xb2cfd50c, SUC_lexicon_tags_19 },
    { 0x184fd50d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x64efd50f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a8fd513, SUC_lexicon_tags_23 },
    { 0xc98fd514, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4f6fd516, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa56fd518, SUC_lexicon_tags_37 },
    { 0x9b8fd519, SUC_lexicon_tags_113 },
    { 0xd94fd51a, SUC_lexicon_tags_32 },
    { 0x3a2fd51a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cefd51e, SUC_lexicon_tags_31 },
    { 0xc44fd51e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x888fd526, SUC_lexicon_tags_150 },
    { 0x9a8fd527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfd52b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cefd52f, SUC_lexicon_tags_60 },
    { 0xcc4fd52f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dcfd536, SUC_lexicon_tags_33 },
    { 0x9acfd537, SUC_lexicon_tags_25 },
    { 0xaf4fd538, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x22afd53a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d6fd53c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0fd541, SUC_lexicon_tags_46 },
    { 0xda6fd542, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2fd547, SUC_lexicon_tags_15 },
    { 0xd4efd548, SUC_lexicon_tags_1 },
    { 0xd8fd549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa2fd54b, SUC_lexicon_tags_19 },
    { 0x548fd54c, SUC_lexicon_tags_2 },
    { 0x442fd54c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2efd551, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cfd557, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630fd55e, SUC_lexicon_tags_15 },
    { 0x320fd55f, SUC_lexicon_tags_29 },
    { 0x53afd55f, SUC_lexicon_tags_15 },
    { 0x394fd561, SUC_lexicon_tags_37 },
    { 0x218fd55e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa5efd564, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf28fd566, SUC_lexicon_tags_30 },
    { 0x522fd566, SUC_lexicon_tags_15 },
    { 0x288fd568, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd80fd56c, SUC_lexicon_tags_17 },
    { 0xb02fd56d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa50fd570, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x334fd572, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x22afd574, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x44afd577, SUC_lexicon_tags_25 },
    { 0x29efd578, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfefd57a, SUC_lexicon_tags_16 },
    { 0x640fd57b, SUC_lexicon_tags_2 },
    { 0x420fd57b, SUC_lexicon_tags_23 },
    { 0x368fd57d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbbefd57f, SUC_lexicon_tags_17 },
    { 0xd24fd57f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0fd583, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xac6fd586, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f4fd588, SUC_lexicon_tags_60 },
    { 0xcb8fd588, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30afd58b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdefd591, SUC_lexicon_tags_47 },
    { 0x134fd591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb80fd594, SUC_lexicon_tags_37 },
    { 0x6befd595, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6fd59a, SUC_lexicon_tags_28 },
    { 0x828fd59a, SUC_lexicon_tags_17 },
    { 0x880fd59c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44fd5a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d0fd5a8, SUC_lexicon_tags_56 },
    { 0xf8fd5a9, SUC_lexicon_tags_33 },
    { 0xd4afd5aa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2aafd5ac, SUC_lexicon_tags_47 },
    { 0x6c8fd5ac, SUC_lexicon_tags_1 },
    { 0x6dcfd5ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7b4fd5b0, SUC_lexicon_tags_106 },
    { 0x80cfd5b0, SUC_lexicon_tags_7 },
    { 0x5c8fd5b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76afd5b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x588fd5b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25afd5ba, SUC_lexicon_tags_33 },
    { 0x7cefd5bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c2fd5c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc84fd5c3, SUC_lexicon_tags_7 },
    { 0xbcfd5c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e6fd5c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38afd5c9, SUC_lexicon_tags_11 },
    { 0x814fd5ca, SUC_lexicon_tags_15 },
    { 0x15cfd5ca, SUC_lexicon_tags_8 },
    { 0x752fd5cc, SUC_lexicon_tags_45 },
    { 0xd48fd5cc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66efd5d1, SUC_lexicon_tags_15 },
    { 0x13efd5d2, SUC_lexicon_tags_45 },
    { 0x342fd5d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cfd5d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93cfd5da, SUC_lexicon_tags_1 },
    { 0x8b4fd5db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x972fd5dd, SUC_lexicon_tags_37 },
    { 0x3fcfd5de, SUC_lexicon_tags_11 },
    { 0xac6fd5df, SUC_lexicon_tags_2 },
    { 0x28cfd5df, SUC_lexicon_tags_7 },
    { 0x86afd5de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94afd5e3, SUC_lexicon_tags_27 },
    { 0xf00fd5e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eafd5e8, SUC_lexicon_tags_34 },
    { 0xadafd5e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf12fd5eb, SUC_lexicon_tags_17 },
    { 0x94fd5eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56fd5f0, SUC_lexicon_tags_9 },
    { 0x24cfd5f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2fd5f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4fd5fa, SUC_lexicon_tags_25 },
    { 0x4afd5fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbafd602, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe24fd607, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb6fd609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fd60d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cfd612, SUC_lexicon_tags_14 },
    { 0x116fd613, SUC_lexicon_tags_16 },
    { 0xebcfd612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x224fd619, SUC_lexicon_tags_47 },
    { 0x360fd61a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6fd622, SUC_lexicon_tags_12 },
    { 0x97efd623, SUC_lexicon_tags_43 },
    { 0x87afd624, SUC_lexicon_tags_11 },
    { 0x194fd624, SUC_lexicon_tags_7 },
    { 0x71afd624, SUC_lexicon_tags_74 },
    { 0x522fd627, SUC_lexicon_tags_11 },
    { 0x7dafd628, SUC_lexicon_tags_7 },
    { 0x858fd627, SUC_lexicon_tags_33 },
    { 0x1aafd625, SUC_lexicon_tags_25 },
    { 0x476fd62b, SUC_lexicon_tags_11 },
    { 0xa98fd62c, SUC_lexicon_tags_11 },
    { 0x19efd62d, SUC_lexicon_tags_252 },
    { 0xe36fd62d, SUC_lexicon_tags_37 },
    { 0x654fd62c, SUC_lexicon_tags_47 },
    { 0x2f0fd630, SUC_lexicon_tags_11 },
    { 0x502fd631, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cefd634, SUC_lexicon_tags_17 },
    { 0x48fd635, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d8fd637, SUC_lexicon_tags_7 },
    { 0xbf6fd637, SUC_lexicon_tags_1 },
    { 0x3cefd638, SUC_lexicon_tags_14 },
    { 0x744fd63a, SUC_lexicon_tags_116 },
    { 0x862fd63b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x426fd641, SUC_lexicon_tags_45 },
    { 0x126fd641, SUC_lexicon_tags_38 },
    { 0x18fd643, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x42cfd646, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aefd64b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84efd64f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc06fd651, SUC_lexicon_tags_9 },
    { 0x6d4fd652, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0cfd658, SUC_lexicon_tags_0 },
    { 0xd3afd659, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb24fd65f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x954fd662, SUC_lexicon_tags_96 },
    { 0x3b4fd663, SUC_lexicon_tags_47 },
    { 0xd22fd664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e8fd666, SUC_lexicon_tags_46 },
    { 0xe70fd667, SUC_lexicon_tags_0 },
    { 0x3acfd668, SUC_lexicon_tags_6 },
    { 0x2bcfd666, SUC_lexicon_tags_25 },
    { 0xdccfd66a, SUC_lexicon_tags_17 },
    { 0xdcafd66a, SUC_lexicon_tags_14 },
    { 0x7e6fd66c, SUC_lexicon_tags_16 },
    { 0x2dafd66d, SUC_lexicon_tags_15 },
    { 0x90cfd66a, SUC_lexicon_tags_17 },
    { 0x7c2fd66f, SUC_lexicon_tags_60 },
    { 0x32cfd66f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8b2fd672, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x162fd674, SUC_lexicon_tags_32 },
    { 0xffefd675, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0efd679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8efd67e, SUC_lexicon_tags_11 },
    { 0x6e2fd67e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7efd681, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceafd685, SUC_lexicon_tags_15 },
    { 0xe2afd686, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798fd68a, SUC_lexicon_tags_11 },
    { 0x3b6fd68b, SUC_lexicon_tags_14 },
    { 0xf12fd68a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cfd691, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1afd695, SUC_lexicon_tags_25 },
    { 0x41efd696, SUC_lexicon_tags_25 },
    { 0xe74fd697, SUC_lexicon_tags_0 },
    { 0xaa2fd698, SUC_lexicon_tags_33 },
    { 0xeb8fd699, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8fd6a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb20fd6a2, SUC_lexicon_tags_17 },
    { 0x5cefd6a3, SUC_lexicon_tags_38 },
    { 0x388fd6a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fd6ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x508fd6ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1defd6b2, SUC_lexicon_tags_65 },
    { 0xe72fd6b2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0afd6b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf8fd6ba, SUC_lexicon_tags_15 },
    { 0xab0fd6bb, SUC_lexicon_tags_17 },
    { 0x1dcfd6bc, SUC_lexicon_tags_1 },
    { 0x748fd6bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894fd6c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16fd6c5, SUC_lexicon_tags_43 },
    { 0xb3efd6c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x26efd6c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd58fd6cb, SUC_lexicon_tags_47 },
    { 0xe3afd6cc, SUC_lexicon_tags_25 },
    { 0x4a0fd6cd, SUC_lexicon_tags_47 },
    { 0xbacfd6cd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fd6d8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4fd6e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x32efd6e5, SUC_lexicon_tags_15 },
    { 0xa76fd6e6, SUC_lexicon_tags_7 },
    { 0xc84fd6e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed0fd6e9, SUC_lexicon_tags_1 },
    { 0x108fd6ea, SUC_lexicon_tags_23 },
    { 0xc52fd6eb, SUC_lexicon_tags_2 },
    { 0xf6afd6e9, SUC_lexicon_tags_1 },
    { 0x458fd6ed, SUC_lexicon_tags_39 },
    { 0x61cfd6eb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd6cfd6f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1efd6f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa9cfd6f6, SUC_lexicon_tags_25 },
    { 0xff0fd6f7, SUC_lexicon_tags_25 },
    { 0x782fd6f7, SUC_lexicon_tags_6 },
    { 0xb0fd6f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8fd6fd, SUC_lexicon_tags_15 },
    { 0xfeefd6fe, SUC_lexicon_tags_37 },
    { 0x98afd6ff, SUC_lexicon_tags_136 },
    { 0xe3efd700, SUC_lexicon_tags_31 },
    { 0x188fd6fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x206fd703, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10cfd705, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa6fd707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x378fd709, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcfd710, SUC_lexicon_tags_1 },
    { 0x330fd711, SUC_lexicon_tags_27 },
    { 0xb26fd711, SUC_lexicon_tags_11 },
    { 0x308fd713, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40fd715, SUC_lexicon_tags_43 },
    { 0x352fd716, SUC_lexicon_tags_27 },
    { 0xf0afd717, SUC_lexicon_tags_3 },
    { 0x570fd718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78fd71b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x118fd71d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x318fd71f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xb1cfd721, SUC_lexicon_tags_72 },
    { 0xbacfd722, SUC_lexicon_tags_2 },
    { 0xd44fd723, SUC_lexicon_tags_65 },
    { 0x518fd724, SUC_lexicon_tags_11 },
    { 0x38efd722, SUC_lexicon_tags_46 },
    { 0xb74fd726, SUC_lexicon_tags_7 },
    { 0x2fefd727, SUC_lexicon_tags_138 },
    { 0xdfafd728, SUC_lexicon_tags_25 },
    { 0x6b4fd729, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50fd72b, SUC_lexicon_tags_45 },
    { 0xb68fd72c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fd731, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150fd736, SUC_lexicon_tags_12 },
    { 0x31cfd737, SUC_lexicon_tags_30 },
    { 0x178fd738, SUC_lexicon_tags_15 },
    { 0x7b4fd738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2cfd740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34efd745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cfd749, SUC_lexicon_tags_65 },
    { 0x188fd74a, SUC_lexicon_tags_12 },
    { 0x34fd74b, SUC_lexicon_tags_23 },
    { 0x34fd749, SUC_lexicon_tags_49 },
    { 0xfc4fd74c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7afd751, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b8fd753, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x608fd756, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0fd75b, SUC_lexicon_tags_11 },
    { 0x33afd75b, SUC_lexicon_tags_17 },
    { 0x22afd75d, SUC_lexicon_tags_11 },
    { 0xc22fd75e, SUC_lexicon_tags_0 },
    { 0x3d2fd75f, SUC_lexicon_tags_79 },
    { 0xc4afd75c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa92fd764, SUC_lexicon_tags_37 },
    { 0xd6fd765, SUC_lexicon_tags_40 },
    { 0x7befd764, SUC_lexicon_tags_16 },
    { 0xc96fd765, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd4fd769, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184fd76f, SUC_lexicon_tags_7 },
    { 0x7ecfd770, SUC_lexicon_tags_1 },
    { 0x40cfd771, SUC_lexicon_tags_37 },
    { 0x98cfd772, SUC_lexicon_tags_49 },
    { 0xfcafd770, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x566fd775, SUC_lexicon_tags_27 },
    { 0x9fcfd776, SUC_lexicon_tags_12 },
    { 0xb28fd777, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x890fd77a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16fd77e, SUC_lexicon_tags_7 },
    { 0xfcafd77e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2befd785, SUC_lexicon_tags_27 },
    { 0xaf2fd786, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956fd78b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450fd794, SUC_lexicon_tags_7 },
    { 0x612fd795, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xce8fd798, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdc2fd79a, SUC_lexicon_tags_14 },
    { 0xef0fd79a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0fd79e, SUC_lexicon_tags_7 },
    { 0x2f0fd79e, SUC_lexicon_tags_1 },
    { 0x1eafd7a0, SUC_lexicon_tags_11 },
    { 0xc7cfd7a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x936fd7a4, SUC_lexicon_tags_49 },
    { 0xd6efd7a4, SUC_lexicon_tags_49 },
    { 0xde6fd7a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfb0fd7a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0fd7ad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63efd7b1, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x6c4fd7b3, SUC_lexicon_tags_45 },
    { 0x2f8fd7b3, SUC_lexicon_tags_37 },
    { 0x648fd7b3, SUC_lexicon_tags_26 },
    { 0xec6fd7b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312fd7bd, SUC_lexicon_tags_25 },
    { 0x8eafd7be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e2fd7c0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x66efd7c2, SUC_lexicon_tags_2 },
    { 0xfa2fd7c3, SUC_lexicon_tags_86 },
    { 0xf0cfd7c3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x852fd7c7, SUC_lexicon_tags_31 },
    { 0x358fd7c8, SUC_lexicon_tags_66 },
    { 0x62fd7c7, SUC_lexicon_tags_27 },
    { 0xae0fd7ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60fd7d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbfcfd7d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cfd7da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4fd7df, SUC_lexicon_tags_25 },
    { 0x288fd7e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa16fd7e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4fd7e5, SUC_lexicon_tags_8 },
    { 0x3b6fd7e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa02fd7e8, SUC_lexicon_tags_39 },
    { 0x3d8fd7e9, SUC_lexicon_tags_17 },
    { 0x718fd7e9, SUC_lexicon_tags_37 },
    { 0xae6fd7eb, SUC_lexicon_tags_2 },
    { 0x772fd7ea, SUC_lexicon_tags_1 },
    { 0x2dcfd7ed, SUC_lexicon_tags_60 },
    { 0xf76fd7eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2efd7f0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2fd7f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3efd7f6, SUC_lexicon_tags_12 },
    { 0x20fd7f7, SUC_lexicon_tags_7 },
    { 0x2c4fd7f7, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e2fd7fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23cfd805, SUC_lexicon_tags_14 },
    { 0x758fd806, SUC_lexicon_tags_17 },
    { 0xe04fd807, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8c2fd809, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28fd80b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3afd80d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9efd810, SUC_lexicon_tags_33 },
    { 0xc1cfd811, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x53cfd813, SUC_lexicon_tags_1 },
    { 0x38cfd813, SUC_lexicon_tags_19 },
    { 0xd64fd814, SUC_lexicon_tags_11 },
    { 0xffafd815, SUC_lexicon_tags_2 },
    { 0xb5afd815, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x27efd81a, SUC_lexicon_tags_12 },
    { 0x2d8fd81b, SUC_lexicon_tags_23 },
    { 0x9cafd81c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa60fd81e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x952fd820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd78fd823, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x812fd825, SUC_lexicon_tags_15 },
    { 0xd98fd826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3efd82b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x148fd82d, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfacfd835, SUC_lexicon_tags_34 },
    { 0xf60fd836, SUC_lexicon_tags_38 },
    { 0x68afd837, SUC_lexicon_tags_49 },
    { 0x9e8fd838, SUC_lexicon_tags_17 },
    { 0x9a2fd837, SUC_lexicon_tags_34 },
    { 0xcaefd83a, SUC_lexicon_tags_6 },
    { 0xc96fd83b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee8fd840, SUC_lexicon_tags_1 },
    { 0x930fd841, SUC_lexicon_tags_14 },
    { 0x746fd842, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cfd845, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xee0fd848, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5f0fd84a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e2fd84c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd68fd84f, SUC_lexicon_tags_12 },
    { 0x8cafd850, SUC_lexicon_tags_15 },
    { 0x5c2fd851, SUC_lexicon_tags_10 },
    { 0xbb8fd852, SUC_lexicon_tags_15 },
    { 0x784fd853, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcfd857, SUC_lexicon_tags_27 },
    { 0x260fd858, SUC_lexicon_tags_15 },
    { 0x9b6fd859, SUC_lexicon_tags_46 },
    { 0xccfd859, SUC_lexicon_tags_11 },
    { 0x5e4fd858, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56fd85f, SUC_lexicon_tags_11 },
    { 0x838fd860, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b6fd862, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x184fd867, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30fd86d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14cfd870, SUC_lexicon_tags_68 },
    { 0x3fcfd871, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6efd873, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe08fd875, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf44fd878, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292fd87c, SUC_lexicon_tags_26 },
    { 0xa82fd87d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x936fd87f, SUC_lexicon_tags_31 },
    { 0x9c2fd880, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x216fd883, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4befd885, SUC_lexicon_tags_4 },
    { 0x4dafd886, SUC_lexicon_tags_25 },
    { 0x5a6fd886, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xd08fd889, SUC_lexicon_tags_29 },
    { 0x6c2fd88a, SUC_lexicon_tags_12 },
    { 0x6acfd88b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x940fd88f, SUC_lexicon_tags_38 },
    { 0x6e6fd890, SUC_lexicon_tags_17 },
    { 0x192fd88f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cfd896, SUC_lexicon_tags_64 },
    { 0x5acfd897, SUC_lexicon_tags_31 },
    { 0xc18fd897, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58fd89b, SUC_lexicon_tags_6 },
    { 0x394fd89b, SUC_lexicon_tags_17 },
    { 0xd90fd89d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f4fd89f, SUC_lexicon_tags_39 },
    { 0xed2fd89f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23cfd8a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00fd8a5, SUC_lexicon_tags_12 },
    { 0xdccfd8a6, SUC_lexicon_tags_17 },
    { 0x87cfd8a7, SUC_lexicon_tags_69 },
    { 0xd0fd8a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84fd8ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x67cfd8ad, SUC_lexicon_tags_2 },
    { 0xbccfd8ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82fd8b1, SUC_lexicon_tags_11 },
    { 0xb02fd8b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaeefd8b4, SUC_lexicon_tags_11 },
    { 0xcd4fd8b5, SUC_lexicon_tags_17 },
    { 0xbdefd8b6, SUC_lexicon_tags_19 },
    { 0x7afd8b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6fd8bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8fd8be, SUC_lexicon_tags_11 },
    { 0x348fd8bf, SUC_lexicon_tags_14 },
    { 0xae0fd8c0, SUC_lexicon_tags_11 },
    { 0xa0afd8c1, SUC_lexicon_tags_19 },
    { 0x602fd8c1, SUC_lexicon_tags_35 },
    { 0x76fd8c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe8fd8c5, SUC_lexicon_tags_17 },
    { 0x450fd8c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaefd8ce, SUC_lexicon_tags_33 },
    { 0xeefd8cf, SUC_lexicon_tags_80 },
    { 0x572fd8d0, SUC_lexicon_tags_12 },
    { 0x7a8fd8cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c8fd8d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53afd8d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02fd8db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60afd8df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96afd8e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc08fd8e3, SUC_lexicon_tags_21 },
    { 0xdc2fd8e4, SUC_lexicon_tags_57 },
    { 0xfb8fd8e5, SUC_lexicon_tags_25 },
    { 0x75cfd8e6, SUC_lexicon_tags_11 },
    { 0xbc2fd8e3, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6cfd8ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49afd8f0, SUC_lexicon_tags_25 },
    { 0x2a4fd8f1, SUC_lexicon_tags_25 },
    { 0xc12fd8f2, SUC_lexicon_tags_35 },
    { 0x986fd8f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe4fd8f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cafd8f8, SUC_lexicon_tags_103 },
    { 0xa1efd8f8, SUC_lexicon_tags_7 },
    { 0xc80fd8f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x260fd8fc, SUC_lexicon_tags_14 },
    { 0x8aefd8fd, SUC_lexicon_tags_14 },
    { 0x1dafd8fe, SUC_lexicon_tags_43 },
    { 0x1efd8ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x474fd902, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6fd905, SUC_lexicon_tags_25 },
    { 0x762fd906, SUC_lexicon_tags_10 },
    { 0xaeefd905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59afd90a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52efd90e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0fd912, SUC_lexicon_tags_569 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cfd91a, SUC_lexicon_tags_38 },
    { 0xb22fd91b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4cfd91f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x302fd921, SUC_lexicon_tags_48 },
    { 0x47efd922, SUC_lexicon_tags_28 },
    { 0x7a6fd923, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x612fd926, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21afd929, SUC_lexicon_tags_47 },
    { 0xec4fd929, SUC_lexicon_tags_67 },
    { 0x178fd92b, SUC_lexicon_tags_37 },
    { 0xe56fd92a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x2e6fd92e, SUC_lexicon_tags_11 },
    { 0x362fd92f, SUC_lexicon_tags_14 },
    { 0x24fd930, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x740fd932, SUC_lexicon_tags_43 },
    { 0xd7cfd933, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8fd939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3cfd93c, SUC_lexicon_tags_2 },
    { 0x464fd93d, SUC_lexicon_tags_19 },
    { 0xa88fd93e, SUC_lexicon_tags_81 },
    { 0xa62fd93f, SUC_lexicon_tags_9 },
    { 0xbaefd93f, SUC_lexicon_tags_37 },
    { 0x80afd941, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282fd948, SUC_lexicon_tags_37 },
    { 0xc32fd948, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x19cfd94b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31afd94f, SUC_lexicon_tags_19 },
    { 0xf24fd94f, SUC_lexicon_tags_30 },
    { 0x304fd951, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bafd955, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470fd95c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1defd95f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf04fd963, SUC_lexicon_tags_0 },
    { 0x534fd964, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeafd96c, SUC_lexicon_tags_28 },
    { 0xabefd96d, SUC_lexicon_tags_7 },
    { 0xe3efd96e, SUC_lexicon_tags_12 },
    { 0x190fd96f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1aafd971, SUC_lexicon_tags_2 },
    { 0x8c4fd972, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8fd979, SUC_lexicon_tags_25 },
    { 0x430fd979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe04fd97c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x98afd97f, SUC_lexicon_tags_10 },
    { 0x2c4fd980, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0fd983, SUC_lexicon_tags_25 },
    { 0xe26fd984, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cfd987, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0fd98e, SUC_lexicon_tags_1 },
    { 0xf0cfd98f, SUC_lexicon_tags_87 },
    { 0xcaefd98f, SUC_lexicon_tags_2 },
    { 0xf48fd991, SUC_lexicon_tags_17 },
    { 0x1acfd991, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x452fd994, SUC_lexicon_tags_3 },
    { 0xd3cfd995, SUC_lexicon_tags_33 },
    { 0x4c6fd995, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506fd99a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc4efd99c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x36fd99e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc20fd9a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1c2fd9a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc68fd9a6, SUC_lexicon_tags_11 },
    { 0x7c0fd9a7, SUC_lexicon_tags_12 },
    { 0xb2afd9a8, SUC_lexicon_tags_0 },
    { 0xf28fd9a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c0fd9ac, SUC_lexicon_tags_1 },
    { 0x32fd9ac, SUC_lexicon_tags_60 },
    { 0x216fd9ae, SUC_lexicon_tags_32 },
    { 0xd7afd9af, SUC_lexicon_tags_23 },
    { 0xcacfd9ac, SUC_lexicon_tags_57 },
    { 0x2dafd9af, SUC_lexicon_tags_17 },
    { 0xd9cfd9b1, SUC_lexicon_tags_2 },
    { 0x14afd9ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e8fd9b8, SUC_lexicon_tags_30 },
    { 0xc8efd9b9, SUC_lexicon_tags_15 },
    { 0xcd4fd9b8, SUC_lexicon_tags_25 },
    { 0x710fd9b9, SUC_lexicon_tags_7 },
    { 0x40efd9bc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec0fd9c0, SUC_lexicon_tags_60 },
    { 0xf76fd9c1, SUC_lexicon_tags_33 },
    { 0xc02fd9c1, SUC_lexicon_tags_11 },
    { 0x702fd9c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ecfd9c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fd9cc, SUC_lexicon_tags_39 },
    { 0x208fd9cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaefd9d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81efd9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9cfd9dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0efd9e5, SUC_lexicon_tags_35 },
    { 0x5e4fd9e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x858fd9e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd02fd9ea, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb0cfd9ec, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb32fd9f8, SUC_lexicon_tags_19 },
    { 0x528fd9f8, SUC_lexicon_tags_65 },
    { 0x810fd9fa, SUC_lexicon_tags_19 },
    { 0xc00fd9fb, SUC_lexicon_tags_1 },
    { 0xd1cfd9fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda4fd9ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1acfda01, SUC_lexicon_tags_56 },
    { 0xf8afda01, SUC_lexicon_tags_15 },
    { 0x846fda01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2cfda05, SUC_lexicon_tags_27 },
    { 0xa26fda06, SUC_lexicon_tags_95 },
    { 0x372fda06, SUC_lexicon_tags_12 },
    { 0xb6afda05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x948fda0b, SUC_lexicon_tags_2 },
    { 0xf46fda0b, SUC_lexicon_tags_28 },
    { 0xf0efda0b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x906fda0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfafda12, SUC_lexicon_tags_24 },
    { 0x328fda12, SUC_lexicon_tags_2 },
    { 0x656fda13, SUC_lexicon_tags_14 },
    { 0xb34fda15, SUC_lexicon_tags_4 },
    { 0x2aafda16, SUC_lexicon_tags_15 },
    { 0x76afda16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6fda1b, SUC_lexicon_tags_35 },
    { 0xa7afda1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x808fda1f, SUC_lexicon_tags_7 },
    { 0xecafda1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4fda25, SUC_lexicon_tags_12 },
    { 0x9dafda25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9befda29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x166fda2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac8fda2f, SUC_lexicon_tags_2 },
    { 0xb1cfda30, SUC_lexicon_tags_2 },
    { 0xfefda30, SUC_lexicon_tags_11 },
    { 0xd0afda32, SUC_lexicon_tags_30 },
    { 0xe6cfda33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x720fda36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xadafda38, SUC_lexicon_tags_1 },
    { 0xe14fda39, SUC_lexicon_tags_19 },
    { 0x1fefda3a, SUC_lexicon_tags_9 },
    { 0xdfefda39, SUC_lexicon_tags_15 },
    { 0xe68fda3c, SUC_lexicon_tags_7 },
    { 0xe02fda3d, SUC_lexicon_tags_11 },
    { 0x2d6fda39, SUC_lexicon_tags_11 },
    { 0xe8fda3f, SUC_lexicon_tags_65 },
    { 0xa6efda40, SUC_lexicon_tags_282 },
    { 0xc02fda40, SUC_lexicon_tags_25 },
    { 0x91afda3f, SUC_lexicon_tags_12 },
    { 0x3b0fda41, SUC_lexicon_tags_8 },
    { 0xb22fda44, SUC_lexicon_tags_12 },
    { 0x36efda41, SUC_lexicon_tags_192 },
    { 0x4fefda46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fda49, SUC_lexicon_tags_129 },
    { 0x40efda49, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf02fda4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe98fda4e, SUC_lexicon_tags_30 },
    { 0xa24fda4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x366fda53, SUC_lexicon_tags_17 },
    { 0x8f8fda53, SUC_lexicon_tags_2 },
    { 0x912fda55, SUC_lexicon_tags_27 },
    { 0x1ecfda54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc84fda59, SUC_lexicon_tags_15 },
    { 0x96afda5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298fda5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8fda61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58fda66, SUC_lexicon_tags_7 },
    { 0x6e6fda66, SUC_lexicon_tags_30 },
    { 0x7e0fda67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe40fda6a, SUC_lexicon_tags_32 },
    { 0x304fda6a, SUC_lexicon_tags_55 },
    { 0x60cfda6b, SUC_lexicon_tags_18 },
    { 0xabafda6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cefda71, SUC_lexicon_tags_2 },
    { 0x296fda71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x110fda74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7befda76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0fda79, SUC_lexicon_tags_91 },
    { 0x7fefda7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78fda7d, SUC_lexicon_tags_37 },
    { 0x9a0fda7e, SUC_lexicon_tags_6 },
    { 0xef4fda7f, SUC_lexicon_tags_10 },
    { 0xed4fda80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x890fda82, SUC_lexicon_tags_37 },
    { 0x158fda82, SUC_lexicon_tags_12 },
    { 0x1c0fda84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbaefda86, SUC_lexicon_tags_82 },
    { 0xcc4fda87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x478fda8a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4afda8e, SUC_lexicon_tags_57 },
    { 0xc8cfda8f, SUC_lexicon_tags_25 },
    { 0xadefda90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456fda95, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x420fda9a, SUC_lexicon_tags_0 },
    { 0xf48fda9a, SUC_lexicon_tags_37 },
    { 0xefda9c, SUC_lexicon_tags_37 },
    { 0xe78fda9a, SUC_lexicon_tags_17 },
    { 0xafda9c, SUC_lexicon_tags_10 },
    { 0x4f4fda9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138fdaa3, SUC_lexicon_tags_11 },
    { 0x5b4fdaa4, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xd6afdaa6, SUC_lexicon_tags_84 },
    { 0xc60fdaa7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecfdaab, SUC_lexicon_tags_1 },
    { 0xa74fdaac, SUC_lexicon_tags_15 },
    { 0xb74fdaac, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x862fdaaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9afdab1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2fdab6, SUC_lexicon_tags_17 },
    { 0xcefdab7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4fdabe, SUC_lexicon_tags_28 },
    { 0x572fdabe, SUC_lexicon_tags_47 },
    { 0x8dafdabe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518fdac5, SUC_lexicon_tags_4 },
    { 0xe92fdac6, SUC_lexicon_tags_25 },
    { 0xa6fdac6, SUC_lexicon_tags_7 },
    { 0x452fdac6, SUC_lexicon_tags_19 },
    { 0x6a0fdac9, SUC_lexicon_tags_43 },
    { 0xd82fdaca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6fdacd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632fdada, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeeafdadc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8fdadf, SUC_lexicon_tags_1 },
    { 0x862fdadf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33cfdae2, SUC_lexicon_tags_17 },
    { 0x912fdae3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44efdaf2, SUC_lexicon_tags_47 },
    { 0x702fdaf3, SUC_lexicon_tags_13 },
    { 0xb80fdaf4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72cfdafb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x578fdafe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94fdb01, SUC_lexicon_tags_17 },
    { 0x132fdb01, SUC_lexicon_tags_35 },
    { 0xd14fdb03, SUC_lexicon_tags_38 },
    { 0x75efdb03, SUC_lexicon_tags_15 },
    { 0x67afdb05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa98fdb07, SUC_lexicon_tags_33 },
    { 0xc6fdb08, SUC_lexicon_tags_77 },
    { 0xcb2fdb09, SUC_lexicon_tags_11 },
    { 0x2eafdb0a, SUC_lexicon_tags_17 },
    { 0xb82fdb0b, SUC_lexicon_tags_151 },
    { 0xdecfdb0c, SUC_lexicon_tags_2 },
    { 0x596fdb0b, SUC_lexicon_tags_27 },
    { 0xc4efdb0e, SUC_lexicon_tags_62 },
    { 0xaacfdb0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4eafdb11, SUC_lexicon_tags_37 },
    { 0x83efdb11, SUC_lexicon_tags_28 },
    { 0x16cfdb11, SUC_lexicon_tags_79 },
    { 0x74cfdb14, SUC_lexicon_tags_11 },
    { 0xf5afdb15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2fdb1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2fdb1c, SUC_lexicon_tags_17 },
    { 0x7f6fdb1c, SUC_lexicon_tags_11 },
    { 0x746fdb1e, SUC_lexicon_tags_25 },
    { 0x9b6fdb1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec0fdb22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcf8fdb24, SUC_lexicon_tags_71 },
    { 0xfd4fdb25, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xf1afdb27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x822fdb2d, SUC_lexicon_tags_11 },
    { 0x496fdb2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dafdb30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x296fdb33, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbacfdb35, SUC_lexicon_tags_4 },
    { 0xfc2fdb35, SUC_lexicon_tags_17 },
    { 0x2a6fdb37, SUC_lexicon_tags_1 },
    { 0x2fcfdb38, SUC_lexicon_tags_28 },
    { 0x194fdb38, SUC_lexicon_tags_30 },
    { 0xf2fdb36, SUC_lexicon_tags_11 },
    { 0x91afdb36, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8fdb3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccfdb42, SUC_lexicon_tags_82 },
    { 0x440fdb43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x216fdb46, SUC_lexicon_tags_53 },
    { 0x194fdb46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cfdb4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x954fdb54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a0fdb56, SUC_lexicon_tags_2 },
    { 0xf3efdb56, SUC_lexicon_tags_1 },
    { 0xe20fdb57, SUC_lexicon_tags_30 },
    { 0xc1cfdb57, SUC_lexicon_tags_15 },
    { 0x6d4fdb5a, SUC_lexicon_tags_63 },
    { 0x3d8fdb58, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bafdb63, SUC_lexicon_tags_15 },
    { 0x316fdb63, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cfdb6a, SUC_lexicon_tags_23 },
    { 0x26cfdb6b, SUC_lexicon_tags_56 },
    { 0xaa4fdb6c, SUC_lexicon_tags_19 },
    { 0xcdafdb6d, SUC_lexicon_tags_25 },
    { 0xb66fdb6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd12fdb70, SUC_lexicon_tags_9 },
    { 0x3a4fdb71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47afdb74, SUC_lexicon_tags_6 },
    { 0x9f0fdb75, SUC_lexicon_tags_4 },
    { 0xe46fdb76, SUC_lexicon_tags_37 },
    { 0xf4afdb76, SUC_lexicon_tags_19 },
    { 0x1efdb78, SUC_lexicon_tags_25 },
    { 0x104fdb78, SUC_lexicon_tags_29 },
    { 0x44cfdb7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc6cfdb7c, SUC_lexicon_tags_19 },
    { 0xf2cfdb7d, SUC_lexicon_tags_544 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdb80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fafdb84, SUC_lexicon_tags_15 },
    { 0xb30fdb85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cfdb88, SUC_lexicon_tags_77 },
    { 0xe4afdb89, SUC_lexicon_tags_49 },
    { 0x1f2fdb89, SUC_lexicon_tags_7 },
    { 0x8fdb8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29efdb8d, SUC_lexicon_tags_17 },
    { 0x812fdb8d, SUC_lexicon_tags_27 },
    { 0xc2cfdb8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe00fdb91, SUC_lexicon_tags_17 },
    { 0x76cfdb92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x622fdb9b, SUC_lexicon_tags_0 },
    { 0x1afdb9c, SUC_lexicon_tags_25 },
    { 0xa60fdb9d, SUC_lexicon_tags_19 },
    { 0xc12fdb9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e6fdba0, SUC_lexicon_tags_66 },
    { 0xd04fdba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe20fdba3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70fdba6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x614fdba9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50fdbb0, SUC_lexicon_tags_7 },
    { 0xcb4fdbb1, SUC_lexicon_tags_10 },
    { 0x286fdbb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8fdbb5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf96fdbb7, SUC_lexicon_tags_38 },
    { 0xad6fdbb8, SUC_lexicon_tags_60 },
    { 0xd32fdbb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x14efdbbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb36fdbbf, SUC_lexicon_tags_6 },
    { 0xd6afdbc0, SUC_lexicon_tags_2 },
    { 0xd50fdbbf, SUC_lexicon_tags_30 },
    { 0xafdbbf, SUC_lexicon_tags_33 },
    { 0xb3efdbc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4fdbc9, SUC_lexicon_tags_47 },
    { 0x56afdbca, SUC_lexicon_tags_9 },
    { 0xf46fdbc9, SUC_lexicon_tags_29 },
    { 0x372fdbcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8cfdbce, SUC_lexicon_tags_11 },
    { 0x328fdbcf, SUC_lexicon_tags_17 },
    { 0x214fdbd0, SUC_lexicon_tags_43 },
    { 0x8f8fdbd1, SUC_lexicon_tags_11 },
    { 0xf9cfdbd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc6fdbd4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbb6fdbd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770fdbdd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfdbe1, SUC_lexicon_tags_11 },
    { 0x4cefdbe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x240fdbe9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91cfdbeb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x930fdbed, SUC_lexicon_tags_1 },
    { 0x174fdbed, SUC_lexicon_tags_33 },
    { 0x1eafdbee, SUC_lexicon_tags_2 },
    { 0x4c8fdbf0, SUC_lexicon_tags_13 },
    { 0x720fdbf1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2efdbf4, SUC_lexicon_tags_47 },
    { 0x800fdbf5, SUC_lexicon_tags_60 },
    { 0xf80fdbf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16fdbf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12fdbfc, SUC_lexicon_tags_11 },
    { 0x2efdbfc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70fdc01, SUC_lexicon_tags_6 },
    { 0xf70fdc01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbefdc05, SUC_lexicon_tags_1 },
    { 0x4b0fdc06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156fdc0b, SUC_lexicon_tags_8 },
    { 0x1defdc0c, SUC_lexicon_tags_23 },
    { 0x9bcfdc0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2fdc13, SUC_lexicon_tags_60 },
    { 0x6e2fdc13, SUC_lexicon_tags_11 },
    { 0x772fdc14, SUC_lexicon_tags_9 },
    { 0xf84fdc13, SUC_lexicon_tags_66 },
    { 0xe6efdc15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc9efdc19, SUC_lexicon_tags_25 },
    { 0xa26fdc19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a2fdc1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a2fdc1e, SUC_lexicon_tags_26 },
    { 0x21cfdc1e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6eefdc21, SUC_lexicon_tags_11 },
    { 0x932fdc22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa58fdc24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158fdc29, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258fdc30, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92efdc35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a2fdc37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x568fdc39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe48fdc3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed4fdc3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea6fdc41, SUC_lexicon_tags_61 },
    { 0xfc8fdc42, SUC_lexicon_tags_87 },
    { 0x33afdc43, SUC_lexicon_tags_27 },
    { 0x996fdc41, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x104fdc47, SUC_lexicon_tags_19 },
    { 0x5d0fdc48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfc8fdc4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c6fdc4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcfdc50, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd84fdc52, SUC_lexicon_tags_13 },
    { 0xe8fdc53, SUC_lexicon_tags_17 },
    { 0x660fdc54, SUC_lexicon_tags_1 },
    { 0x7a4fdc52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4fdc59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x506fdc5d, SUC_lexicon_tags_33 },
    { 0x752fdc5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1acfdc61, SUC_lexicon_tags_48 },
    { 0x1d8fdc61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6fdc66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fafdc6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c8fdc6f, SUC_lexicon_tags_29 },
    { 0xa6fdc6f, SUC_lexicon_tags_68 },
    { 0x466fdc70, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x120fdc75, SUC_lexicon_tags_48 },
    { 0x678fdc76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684fdc7b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8fdc83, SUC_lexicon_tags_40 },
    { 0xa6fdc83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e8fdc87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4fdc8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbefdc91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6fdc97, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x668fdc9a, SUC_lexicon_tags_8 },
    { 0x1afdc9b, SUC_lexicon_tags_33 },
    { 0x4fefdc9c, SUC_lexicon_tags_37 },
    { 0xb2efdc9a, SUC_lexicon_tags_17 },
    { 0xfe4fdc9e, SUC_lexicon_tags_10 },
    { 0x900fdc9f, SUC_lexicon_tags_3 },
    { 0xbfcfdca0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8fdca7, SUC_lexicon_tags_107 },
    { 0xcb6fdca7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf24fdcad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe94fdcaf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cafdcb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd3cfdcb4, SUC_lexicon_tags_37 },
    { 0x908fdcb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa2fdcb7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49afdcc1, SUC_lexicon_tags_30 },
    { 0x12fdcc2, SUC_lexicon_tags_4 },
    { 0x616fdcc3, SUC_lexicon_tags_37 },
    { 0x480fdcc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb82fdcc6, SUC_lexicon_tags_11 },
    { 0xb8fdcc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4afdcd0, SUC_lexicon_tags_17 },
    { 0x49efdcd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a8fdcd3, SUC_lexicon_tags_38 },
    { 0x1c4fdcd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930fdcd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62fdcdc, SUC_lexicon_tags_49 },
    { 0x37efdcdd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x720fdcdf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x416fdce2, SUC_lexicon_tags_25 },
    { 0x1f8fdce2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec2fdce5, SUC_lexicon_tags_2 },
    { 0x144fdce5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x138fdce9, SUC_lexicon_tags_0 },
    { 0x1c4fdce9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x924fdced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7afdcf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24efdcf5, SUC_lexicon_tags_2 },
    { 0x598fdcf5, SUC_lexicon_tags_87 },
    { 0x7bcfdcf7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8fdcfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcfdcfe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4fdd02, SUC_lexicon_tags_25 },
    { 0xf0efdd02, SUC_lexicon_tags_26 },
    { 0x486fdd04, SUC_lexicon_tags_0 },
    { 0x128fdd05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x312fdd08, SUC_lexicon_tags_55 },
    { 0x38efdd09, SUC_lexicon_tags_37 },
    { 0xa4cfdd0a, SUC_lexicon_tags_17 },
    { 0x964fdd0b, SUC_lexicon_tags_7 },
    { 0x4c4fdd09, SUC_lexicon_tags_17 },
    { 0xb3efdd08, SUC_lexicon_tags_9 },
    { 0xf0afdd0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x688fdd12, SUC_lexicon_tags_49 },
    { 0xff0fdd13, SUC_lexicon_tags_2 },
    { 0xddcfdd14, SUC_lexicon_tags_65 },
    { 0x24efdd14, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x610fdd17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9afdd1f, SUC_lexicon_tags_25 },
    { 0xd2cfdd20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2fdd25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb8fdd27, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x354fdd2a, SUC_lexicon_tags_17 },
    { 0xeaefdd2b, SUC_lexicon_tags_63 },
    { 0x48cfdd2c, SUC_lexicon_tags_15 },
    { 0xf98fdd2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e6fdd30, SUC_lexicon_tags_39 },
    { 0x406fdd31, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2fdd38, SUC_lexicon_tags_14 },
    { 0x24fdd39, SUC_lexicon_tags_5 },
    { 0x492fdd3a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x356fdd3c, SUC_lexicon_tags_14 },
    { 0x6c2fdd3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcfdd41, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcfdd49, SUC_lexicon_tags_47 },
    { 0x364fdd4a, SUC_lexicon_tags_15 },
    { 0x6acfdd4b, SUC_lexicon_tags_0 },
    { 0xc04fdd4a, SUC_lexicon_tags_45 },
    { 0xb04fdd4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434fdd53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fdd5b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cfdd60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x55cfdd63, SUC_lexicon_tags_28 },
    { 0x14efdd64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4fdd69, SUC_lexicon_tags_27 },
    { 0x3c2fdd69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83afdd6e, SUC_lexicon_tags_27 },
    { 0xb10fdd6f, SUC_lexicon_tags_27 },
    { 0x7b8fdd6e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aefdd75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x338fdd77, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4fdd7d, SUC_lexicon_tags_11 },
    { 0x194fdd7e, SUC_lexicon_tags_15 },
    { 0xabefdd7f, SUC_lexicon_tags_47 },
    { 0x81cfdd80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718fdd87, SUC_lexicon_tags_38 },
    { 0xa42fdd88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeefdd8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0efdd91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cfdd94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fafdd96, SUC_lexicon_tags_47 },
    { 0x244fdd97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a0fdd99, SUC_lexicon_tags_7 },
    { 0x53cfdd99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8fdda0, SUC_lexicon_tags_84 },
    { 0x76efdda1, SUC_lexicon_tags_77 },
    { 0x7befdda0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cfdda7, SUC_lexicon_tags_37 },
    { 0x478fdda8, SUC_lexicon_tags_7 },
    { 0x262fdda9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x588fddac, SUC_lexicon_tags_61 },
    { 0x17efddad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa06fddaf, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x652fddb3, SUC_lexicon_tags_49 },
    { 0x8a0fddb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf8afddb6, SUC_lexicon_tags_0 },
    { 0x4eefddb6, SUC_lexicon_tags_27 },
    { 0x7e0fddb8, SUC_lexicon_tags_1194 },
    { 0xd0fddb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e0fddc1, SUC_lexicon_tags_15 },
    { 0xcd8fddc2, SUC_lexicon_tags_55 },
    { 0x6b6fddc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64fddc8, SUC_lexicon_tags_45 },
    { 0x286fddc9, SUC_lexicon_tags_2 },
    { 0x4acfddc9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe18fddcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2fddd5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x222fddd8, SUC_lexicon_tags_17 },
    { 0xcc8fddd8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16fdde1, SUC_lexicon_tags_66 },
    { 0x46efdde1, SUC_lexicon_tags_8 },
    { 0xa70fdde2, SUC_lexicon_tags_65 },
    { 0xb70fdde3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7eefdde6, SUC_lexicon_tags_1 },
    { 0x940fdde7, SUC_lexicon_tags_63 },
    { 0x8afdde8, SUC_lexicon_tags_7 },
    { 0xc2cfdde9, SUC_lexicon_tags_28 },
    { 0xe94fddea, SUC_lexicon_tags_0 },
    { 0xa90fddeb, SUC_lexicon_tags_43 },
    { 0x2befddea, SUC_lexicon_tags_18 },
    { 0xa1afdded, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3afddf1, SUC_lexicon_tags_25 },
    { 0x23efddf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf86fddf4, SUC_lexicon_tags_19 },
    { 0x876fddf5, SUC_lexicon_tags_1 },
    { 0x7a8fddf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcfddfb, SUC_lexicon_tags_28 },
    { 0x742fddfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x122fddff, SUC_lexicon_tags_17 },
    { 0xcafddff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4efde05, SUC_lexicon_tags_0 },
    { 0x3f4fde06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf8afde08, SUC_lexicon_tags_71 },
    { 0x4fde09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72fde0f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cfde12, SUC_lexicon_tags_1 },
    { 0x76cfde13, SUC_lexicon_tags_26 },
    { 0xb6cfde14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7a8fde16, SUC_lexicon_tags_27 },
    { 0xb86fde16, SUC_lexicon_tags_46 },
    { 0xc0efde18, SUC_lexicon_tags_12 },
    { 0x2a4fde19, SUC_lexicon_tags_25 },
    { 0x6fcfde19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342fde1e, SUC_lexicon_tags_0 },
    { 0xc5cfde1f, SUC_lexicon_tags_1 },
    { 0x384fde1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefde23, SUC_lexicon_tags_25 },
    { 0xfacfde24, SUC_lexicon_tags_0 },
    { 0xaefde24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc38fde27, SUC_lexicon_tags_37 },
    { 0x510fde28, SUC_lexicon_tags_37 },
    { 0x718fde29, SUC_lexicon_tags_12 },
    { 0x25cfde2a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x268fde2e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6fde31, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494fde36, SUC_lexicon_tags_22 },
    { 0x4b8fde37, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86efde42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774fde48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826fde4d, SUC_lexicon_tags_27 },
    { 0x728fde4e, SUC_lexicon_tags_17 },
    { 0x79cfde4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccfde51, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x50afde53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x66afde56, SUC_lexicon_tags_27 },
    { 0x508fde56, SUC_lexicon_tags_2 },
    { 0x4befde57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3eefde5a, SUC_lexicon_tags_31 },
    { 0xa9cfde5b, SUC_lexicon_tags_47 },
    { 0xa2cfde5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20efde60, SUC_lexicon_tags_26 },
    { 0x8f2fde61, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3efde65, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaafde69, SUC_lexicon_tags_7 },
    { 0xaf4fde6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f2fde6c, SUC_lexicon_tags_12 },
    { 0xbecfde6d, SUC_lexicon_tags_45 },
    { 0x6e4fde6d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x85cfde71, SUC_lexicon_tags_25 },
    { 0x916fde72, SUC_lexicon_tags_12 },
    { 0x442fde73, SUC_lexicon_tags_17 },
    { 0x2aefde74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95afde7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcfde7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe50fde82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35afde86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x494fde88, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcfde8c, SUC_lexicon_tags_17 },
    { 0xcc4fde8d, SUC_lexicon_tags_9 },
    { 0xdc6fde8e, SUC_lexicon_tags_47 },
    { 0xba4fde8f, SUC_lexicon_tags_7 },
    { 0x600fde90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcfde93, SUC_lexicon_tags_32 },
    { 0x36cfde93, SUC_lexicon_tags_98 },
    { 0x660fde94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfde98, SUC_lexicon_tags_40 },
    { 0x880fde99, SUC_lexicon_tags_1 },
    { 0xffcfde9a, SUC_lexicon_tags_6 },
    { 0xa2fde9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fafdea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe26fdea3, SUC_lexicon_tags_21 },
    { 0x3e4fdea4, SUC_lexicon_tags_2 },
    { 0x8fdea5, SUC_lexicon_tags_26 },
    { 0x7eafdea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89afdea8, SUC_lexicon_tags_66 },
    { 0xbaafdea9, SUC_lexicon_tags_1 },
    { 0x846fdea9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910fdeae, SUC_lexicon_tags_45 },
    { 0xc16fdeae, SUC_lexicon_tags_63 },
    { 0x9a6fdeae, SUC_lexicon_tags_57 },
    { 0x38efdeb0, SUC_lexicon_tags_27 },
    { 0x5b8fdeb2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6fdeb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cfdebc, SUC_lexicon_tags_47 },
    { 0xa5afdebd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfb2fdebf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefded0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xefcfded2, SUC_lexicon_tags_215 },
    { 0x7f0fded3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20fded6, SUC_lexicon_tags_17 },
    { 0xf68fded7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49efdedf, SUC_lexicon_tags_12 },
    { 0x3e0fdedf, SUC_lexicon_tags_7 },
    { 0x836fdee1, SUC_lexicon_tags_25 },
    { 0xdc4fdee2, SUC_lexicon_tags_1 },
    { 0x6a0fdee2, SUC_lexicon_tags_11 },
    { 0xbe6fdee4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4fdee7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbacfdeea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75efdeed, SUC_lexicon_tags_15 },
    { 0x486fdeed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa36fdef0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2afdef8, SUC_lexicon_tags_13 },
    { 0x46efdef9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3e8fdefb, SUC_lexicon_tags_11 },
    { 0xc48fdefc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758fdf01, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44fdf07, SUC_lexicon_tags_39 },
    { 0x416fdf08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740fdf0c, SUC_lexicon_tags_68 },
    { 0xeb6fdf0d, SUC_lexicon_tags_12 },
    { 0x19afdf0c, SUC_lexicon_tags_49 },
    { 0x600fdf0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x890fdf12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0fdf17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x320fdf19, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x47cfdf1b, SUC_lexicon_tags_37 },
    { 0x7eefdf1b, SUC_lexicon_tags_1 },
    { 0x342fdf1d, SUC_lexicon_tags_3 },
    { 0xb1afdf1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x304fdf21, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xae6fdf23, SUC_lexicon_tags_9 },
    { 0xf10fdf24, SUC_lexicon_tags_1220 },
    { 0x1c8fdf25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x726fdf27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d0fdf29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1efdf2c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa68fdf30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820fdf36, SUC_lexicon_tags_30 },
    { 0xee8fdf37, SUC_lexicon_tags_37 },
    { 0x85cfdf36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9b4fdf3a, SUC_lexicon_tags_11 },
    { 0xeb6fdf3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e4fdf3e, SUC_lexicon_tags_15 },
    { 0xb6fdf3e, SUC_lexicon_tags_8 },
    { 0x988fdf40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0fdf44, SUC_lexicon_tags_15 },
    { 0xe64fdf45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x536fdf47, SUC_lexicon_tags_10 },
    { 0xe74fdf47, SUC_lexicon_tags_33 },
    { 0xed2fdf49, SUC_lexicon_tags_15 },
    { 0xe34fdf4a, SUC_lexicon_tags_4 },
    { 0x32cfdf4a, SUC_lexicon_tags_25 },
    { 0x554fdf4a, SUC_lexicon_tags_7 },
    { 0x4c4fdf4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44cfdf54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33efdf5c, SUC_lexicon_tags_14 },
    { 0x90fdf5d, SUC_lexicon_tags_7 },
    { 0x8d4fdf5d, SUC_lexicon_tags_27 },
    { 0x1fafdf5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x908fdf62, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdefdf68, SUC_lexicon_tags_39 },
    { 0xcbcfdf69, SUC_lexicon_tags_75 },
    { 0x9dafdf6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25efdf6c, SUC_lexicon_tags_37 },
    { 0x8e6fdf6d, SUC_lexicon_tags_37 },
    { 0x1a0fdf6c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4fdf7e, SUC_lexicon_tags_77 },
    { 0x402fdf7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efdf82, SUC_lexicon_tags_34 },
    { 0x1fcfdf83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaefdf87, SUC_lexicon_tags_49 },
    { 0xb54fdf88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0fdf8b, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0x112fdf8e, SUC_lexicon_tags_9 },
    { 0x57efdf8f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cfdf92, SUC_lexicon_tags_6 },
    { 0xdc4fdf93, SUC_lexicon_tags_17 },
    { 0x422fdf94, SUC_lexicon_tags_23 },
    { 0xd1efdf95, SUC_lexicon_tags_17 },
    { 0xd1cfdf96, SUC_lexicon_tags_22 },
    { 0x3dafdf96, SUC_lexicon_tags_37 },
    { 0xe14fdf98, SUC_lexicon_tags_25 },
    { 0x3a4fdf97, SUC_lexicon_tags_1 },
    { 0xcc4fdf9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2dafdf9c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168fdfa0, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x6bafdfa2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7efdfa8, SUC_lexicon_tags_7 },
    { 0x740fdfa9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5efdfac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x210fdfaf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25afdfb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8afdfbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6fdfc0, SUC_lexicon_tags_7 },
    { 0x658fdfc1, SUC_lexicon_tags_12 },
    { 0xc08fdfc0, SUC_lexicon_tags_11 },
    { 0xd3efdfc3, SUC_lexicon_tags_37 },
    { 0xdeafdfc4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cfdfc7, SUC_lexicon_tags_7 },
    { 0x118fdfc8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afdfcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x170fdfcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x206fdfd2, SUC_lexicon_tags_113 },
    { 0x202fdfd3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe28fdfd7, SUC_lexicon_tags_14 },
    { 0x5cafdfd7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd98fdfda, SUC_lexicon_tags_17 },
    { 0xc76fdfda, SUC_lexicon_tags_30 },
    { 0xe02fdfdb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe8fdfde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x538fdfe0, SUC_lexicon_tags_19 },
    { 0xc42fdfe1, SUC_lexicon_tags_15 },
    { 0x2f4fdfe2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2fdfe5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ecfdfe7, SUC_lexicon_tags_105 },
    { 0x7b4fdfe7, SUC_lexicon_tags_15 },
    { 0x6e4fdfe8, SUC_lexicon_tags_17 },
    { 0x14cfdfe9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x824fdfed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4fdff1, SUC_lexicon_tags_77 },
    { 0x764fdff1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb86fdff4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cefdffb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778fe001, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecafe005, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd64fe00a, SUC_lexicon_tags_19 },
    { 0x9d8fe00b, SUC_lexicon_tags_3 },
    { 0x4fcfe00b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc60fe00e, SUC_lexicon_tags_15 },
    { 0xe10fe00e, SUC_lexicon_tags_37 },
    { 0xbf6fe010, SUC_lexicon_tags_38 },
    { 0xbaafe00f, SUC_lexicon_tags_37 },
    { 0xd78fe012, SUC_lexicon_tags_28 },
    { 0x5dcfe012, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fe01e, SUC_lexicon_tags_35 },
    { 0x8b2fe01f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2fe022, SUC_lexicon_tags_47 },
    { 0xd0fe023, SUC_lexicon_tags_19 },
    { 0x5c0fe024, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114fe02a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a6fe030, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe82fe033, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4befe035, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2fe038, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7a8fe03a, SUC_lexicon_tags_30 },
    { 0x3aafe03b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acfe040, SUC_lexicon_tags_19 },
    { 0xdcafe040, SUC_lexicon_tags_77 },
    { 0xe02fe042, SUC_lexicon_tags_68 },
    { 0xbbcfe043, SUC_lexicon_tags_8 },
    { 0xe88fe041, SUC_lexicon_tags_12 },
    { 0x196fe045, SUC_lexicon_tags_23 },
    { 0x3e2fe046, SUC_lexicon_tags_47 },
    { 0x490fe046, SUC_lexicon_tags_1 },
    { 0x6b8fe048, SUC_lexicon_tags_47 },
    { 0x51cfe042, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c6fe04b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6fe04e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96efe054, SUC_lexicon_tags_9 },
    { 0x322fe054, SUC_lexicon_tags_9 },
    { 0x8cafe056, SUC_lexicon_tags_12 },
    { 0x846fe055, SUC_lexicon_tags_37 },
    { 0x7e4fe057, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c4fe05a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2fe060, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122fe066, SUC_lexicon_tags_8 },
    { 0x998fe066, SUC_lexicon_tags_25 },
    { 0x380fe068, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0fe06c, SUC_lexicon_tags_31 },
    { 0xddafe06d, SUC_lexicon_tags_29 },
    { 0xcf0fe06e, SUC_lexicon_tags_18 },
    { 0x888fe06f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x506fe071, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x45efe074, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e0fe076, SUC_lexicon_tags_27 },
    { 0x1c6fe076, SUC_lexicon_tags_37 },
    { 0x78fe078, SUC_lexicon_tags_7 },
    { 0x2d2fe078, SUC_lexicon_tags_27 },
    { 0xd12fe077, SUC_lexicon_tags_48 },
    { 0x68fe07b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x726fe083, SUC_lexicon_tags_37 },
    { 0x2b6fe084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfccfe088, SUC_lexicon_tags_49 },
    { 0x7befe088, SUC_lexicon_tags_17 },
    { 0x682fe08a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe0fe08c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13efe091, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2fe097, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x51cfe099, SUC_lexicon_tags_12 },
    { 0x5e8fe09a, SUC_lexicon_tags_37 },
    { 0x90fe09b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4fe0a3, SUC_lexicon_tags_33 },
    { 0xb54fe0a3, SUC_lexicon_tags_8 },
    { 0x394fe0a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0fe0aa, SUC_lexicon_tags_23 },
    { 0x9d4fe0ab, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aafe0b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xed0fe0b3, SUC_lexicon_tags_29 },
    { 0x56efe0b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf04fe0b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc82fe0b8, SUC_lexicon_tags_25 },
    { 0x9fafe0b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1efe0c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfe0c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcfe0ca, SUC_lexicon_tags_11 },
    { 0xef8fe0cb, SUC_lexicon_tags_38 },
    { 0xdfefe0cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x93efe0cf, SUC_lexicon_tags_47 },
    { 0x1b2fe0d0, SUC_lexicon_tags_17 },
    { 0x71efe0cf, SUC_lexicon_tags_6 },
    { 0x282fe0d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33cfe0d8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb10fe0da, SUC_lexicon_tags_1 },
    { 0xe2fe0db, SUC_lexicon_tags_17 },
    { 0x880fe0dc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe20fe0e0, SUC_lexicon_tags_12 },
    { 0xad4fe0e1, SUC_lexicon_tags_37 },
    { 0x630fe0e0, SUC_lexicon_tags_15 },
    { 0xd42fe0e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2fe0e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd8fe0ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x466fe0f1, SUC_lexicon_tags_19 },
    { 0xa3cfe0f2, SUC_lexicon_tags_21 },
    { 0x4e2fe0f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd20fe0f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dafe0f7, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xae4fe0fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3afe0fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cfe101, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fe105, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2afe108, SUC_lexicon_tags_17 },
    { 0x3c2fe109, SUC_lexicon_tags_28 },
    { 0xa78fe10a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce2fe10e, SUC_lexicon_tags_0 },
    { 0x436fe10f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9e8fe111, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eefe114, SUC_lexicon_tags_26 },
    { 0x166fe114, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda2fe117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x748fe11a, SUC_lexicon_tags_19 },
    { 0xc9efe11b, SUC_lexicon_tags_35 },
    { 0x5befe11a, SUC_lexicon_tags_35 },
    { 0xf7afe11d, SUC_lexicon_tags_8 },
    { 0xcf2fe11d, SUC_lexicon_tags_16 },
    { 0x452fe11f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa88fe122, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0efe125, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd40fe129, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x13efe12b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2fe12e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x292fe136, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x628fe138, SUC_lexicon_tags_38 },
    { 0x514fe139, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfe141, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x20afe144, SUC_lexicon_tags_63 },
    { 0xe94fe145, SUC_lexicon_tags_12 },
    { 0xb58fe145, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x308fe148, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2afe14e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fefe150, SUC_lexicon_tags_31 },
    { 0x4b8fe151, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2fe157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ecfe15e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba0fe160, SUC_lexicon_tags_37 },
    { 0xd2cfe161, SUC_lexicon_tags_12 },
    { 0x778fe161, SUC_lexicon_tags_33 },
    { 0xae2fe162, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6fe167, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6fe16a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x342fe16c, SUC_lexicon_tags_6 },
    { 0xeeefe16d, SUC_lexicon_tags_37 },
    { 0x83afe16c, SUC_lexicon_tags_27 },
    { 0xbf0fe16f, SUC_lexicon_tags_27 },
    { 0x7b8fe170, SUC_lexicon_tags_1 },
    { 0xde4fe170, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23efe173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x206fe176, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x1a0fe178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x746fe17a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550fe17f, SUC_lexicon_tags_12 },
    { 0xf4afe17f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37efe189, SUC_lexicon_tags_25 },
    { 0xd0afe18a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc54fe18d, SUC_lexicon_tags_14 },
    { 0x624fe18e, SUC_lexicon_tags_43 },
    { 0xd3afe18f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x740fe191, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26fe193, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3cefe195, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a2fe19b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6fe19f, SUC_lexicon_tags_25 },
    { 0x2c6fe1a0, SUC_lexicon_tags_25 },
    { 0xc8cfe1a0, SUC_lexicon_tags_27 },
    { 0x6acfe1a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92fe1a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54fe1ad, SUC_lexicon_tags_79 },
    { 0x780fe1ad, SUC_lexicon_tags_60 },
    { 0x684fe1af, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24efe1b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc52fe1b5, SUC_lexicon_tags_2 },
    { 0xbcfe1b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e2fe1ba, SUC_lexicon_tags_12 },
    { 0xd2cfe1ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44cfe1bd, SUC_lexicon_tags_12 },
    { 0x688fe1be, SUC_lexicon_tags_33 },
    { 0x1b2fe1bd, SUC_lexicon_tags_25 },
    { 0x9fcfe1c0, SUC_lexicon_tags_84 },
    { 0x6d4fe1c1, SUC_lexicon_tags_40 },
    { 0x12fe1c2, SUC_lexicon_tags_37 },
    { 0xb1afe1be, SUC_lexicon_tags_37 },
    { 0xe10fe1bd, SUC_lexicon_tags_25 },
    { 0x182fe1c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78afe1c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x998fe1cb, SUC_lexicon_tags_37 },
    { 0xb6afe1cc, SUC_lexicon_tags_60 },
    { 0xc1cfe1cc, SUC_lexicon_tags_15 },
    { 0x4fefe1ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99afe1dc, SUC_lexicon_tags_1 },
    { 0xf5efe1dd, SUC_lexicon_tags_7 },
    { 0x70efe1dd, SUC_lexicon_tags_17 },
    { 0x854fe1df, SUC_lexicon_tags_25 },
    { 0xc42fe1df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x794fe1e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0fe1e7, SUC_lexicon_tags_37 },
    { 0x7befe1e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xebefe1ea, SUC_lexicon_tags_57 },
    { 0x2ccfe1eb, SUC_lexicon_tags_8 },
    { 0x802fe1ec, SUC_lexicon_tags_19 },
    { 0xfafe1ea, SUC_lexicon_tags_2 },
    { 0x41afe1ee, SUC_lexicon_tags_12 },
    { 0x804fe1ef, SUC_lexicon_tags_23 },
    { 0x39cfe1ef, SUC_lexicon_tags_13 },
    { 0xa7afe1f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc2fe1f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1afe1fc, SUC_lexicon_tags_87 },
    { 0xe02fe1fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73cfe200, SUC_lexicon_tags_12 },
    { 0x9d8fe201, SUC_lexicon_tags_38 },
    { 0xdd6fe200, SUC_lexicon_tags_4 },
    { 0xafcfe203, SUC_lexicon_tags_68 },
    { 0x722fe203, SUC_lexicon_tags_17 },
    { 0x376fe205, SUC_lexicon_tags_47 },
    { 0xd4cfe205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x960fe20a, SUC_lexicon_tags_0 },
    { 0x500fe20b, SUC_lexicon_tags_11 },
    { 0xcb2fe20b, SUC_lexicon_tags_43 },
    { 0xd0afe20a, SUC_lexicon_tags_15 },
    { 0x9bcfe20b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb5efe210, SUC_lexicon_tags_48 },
    { 0x444fe210, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x864fe214, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x746fe216, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfbcfe218, SUC_lexicon_tags_47 },
    { 0xb98fe218, SUC_lexicon_tags_19 },
    { 0x97cfe219, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218fe220, SUC_lexicon_tags_7 },
    { 0x2efe220, SUC_lexicon_tags_47 },
    { 0x278fe221, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7efe225, SUC_lexicon_tags_11 },
    { 0xdcafe225, SUC_lexicon_tags_7 },
    { 0x296fe227, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x510fe229, SUC_lexicon_tags_9 },
    { 0xf54fe22a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd20fe22c, SUC_lexicon_tags_15 },
    { 0xd38fe22d, SUC_lexicon_tags_7 },
    { 0x3e4fe22e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0afe233, SUC_lexicon_tags_14 },
    { 0xdcafe233, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x686fe237, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb04fe23b, SUC_lexicon_tags_27 },
    { 0xe3afe23c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x56efe23e, SUC_lexicon_tags_105 },
    { 0xb84fe23f, SUC_lexicon_tags_23 },
    { 0xb2fe23e, SUC_lexicon_tags_60 },
    { 0xf34fe23e, SUC_lexicon_tags_34 },
    { 0xafe23e, SUC_lexicon_tags_19 },
    { 0x79efe23e, SUC_lexicon_tags_0 },
    { 0x728fe240, SUC_lexicon_tags_37 },
    { 0x232fe245, SUC_lexicon_tags_32 },
    { 0xb5afe246, SUC_lexicon_tags_60 },
    { 0xf44fe246, SUC_lexicon_tags_8 },
    { 0x454fe248, SUC_lexicon_tags_11 },
    { 0x6eafe245, SUC_lexicon_tags_49 },
    { 0x424fe23e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd66fe24c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd44fe252, SUC_lexicon_tags_14 },
    { 0x8c4fe253, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3c4fe255, SUC_lexicon_tags_15 },
    { 0xf1afe256, SUC_lexicon_tags_50 },
    { 0xe9afe256, SUC_lexicon_tags_11 },
    { 0xeb0fe256, SUC_lexicon_tags_21 },
    { 0xb84fe255, SUC_lexicon_tags_25 },
    { 0x544fe25a, SUC_lexicon_tags_61 },
    { 0x194fe258, SUC_lexicon_tags_9 },
    { 0x43afe25c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd24fe261, SUC_lexicon_tags_17 },
    { 0x826fe262, SUC_lexicon_tags_1217 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fe266, SUC_lexicon_tags_9 },
    { 0x72afe267, SUC_lexicon_tags_1 },
    { 0x176fe268, SUC_lexicon_tags_12 },
    { 0x2efe268, SUC_lexicon_tags_43 },
    { 0x542fe268, SUC_lexicon_tags_31 },
    { 0x56efe268, SUC_lexicon_tags_28 },
    { 0xa5afe26c, SUC_lexicon_tags_47 },
    { 0x842fe26c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50fe274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a4fe276, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bcfe27a, SUC_lexicon_tags_37 },
    { 0xf92fe27a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536fe281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6fe286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3efe288, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebefe28d, SUC_lexicon_tags_17 },
    { 0xc64fe28e, SUC_lexicon_tags_15 },
    { 0xed8fe28f, SUC_lexicon_tags_17 },
    { 0xa8afe28d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefafe296, SUC_lexicon_tags_19 },
    { 0x374fe296, SUC_lexicon_tags_31 },
    { 0x654fe298, SUC_lexicon_tags_23 },
    { 0xc00fe299, SUC_lexicon_tags_63 },
    { 0x27cfe29a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6fe29e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534fe2a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02fe2a7, SUC_lexicon_tags_47 },
    { 0xee0fe2a8, SUC_lexicon_tags_9 },
    { 0x862fe2a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2fe2ae, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bafe2b6, SUC_lexicon_tags_17 },
    { 0x8fcfe2b7, SUC_lexicon_tags_15 },
    { 0xceafe2b6, SUC_lexicon_tags_37 },
    { 0xc22fe2b9, SUC_lexicon_tags_37 },
    { 0x5fcfe2ba, SUC_lexicon_tags_2 },
    { 0x290fe2b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x83afe2bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7afe2c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fe2c4, SUC_lexicon_tags_11 },
    { 0xf10fe2c5, SUC_lexicon_tags_9 },
    { 0xc06fe2c6, SUC_lexicon_tags_38 },
    { 0xe6efe2c5, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4afe2cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490fe2cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1afe2d5, SUC_lexicon_tags_25 },
    { 0x23efe2d5, SUC_lexicon_tags_37 },
    { 0x4dafe2d7, SUC_lexicon_tags_25 },
    { 0xbccfe2d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x826fe2dc, SUC_lexicon_tags_33 },
    { 0xdacfe2dc, SUC_lexicon_tags_15 },
    { 0x56afe2dc, SUC_lexicon_tags_25 },
    { 0xb72fe2df, SUC_lexicon_tags_11 },
    { 0xa1efe2e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16fe2e9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3cefe2eb, SUC_lexicon_tags_17 },
    { 0x6b0fe2ec, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x4fe2ee, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2efe2f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590fe2f7, SUC_lexicon_tags_30 },
    { 0xceafe2f8, SUC_lexicon_tags_7 },
    { 0xee4fe2f8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2fe2fc, SUC_lexicon_tags_25 },
    { 0x512fe2fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4fe2ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd62fe306, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddafe308, SUC_lexicon_tags_38 },
    { 0xb7efe308, SUC_lexicon_tags_69 },
    { 0x8bcfe30a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cfe30e, SUC_lexicon_tags_1 },
    { 0xc48fe30f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18fe313, SUC_lexicon_tags_37 },
    { 0x756fe314, SUC_lexicon_tags_37 },
    { 0x630fe313, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44efe319, SUC_lexicon_tags_11 },
    { 0xd4afe31a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fe31e, SUC_lexicon_tags_44 },
    { 0xb1efe31f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xddcfe322, SUC_lexicon_tags_30 },
    { 0x8c6fe323, SUC_lexicon_tags_31 },
    { 0x90efe324, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed8fe326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9afe329, SUC_lexicon_tags_7 },
    { 0x4fafe329, SUC_lexicon_tags_14 },
    { 0x20efe32a, SUC_lexicon_tags_19 },
    { 0xc78fe32c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf68fe32e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x224fe331, SUC_lexicon_tags_33 },
    { 0x68fe332, SUC_lexicon_tags_91 },
    { 0xa36fe333, SUC_lexicon_tags_47 },
    { 0x788fe332, SUC_lexicon_tags_37 },
    { 0xc50fe335, SUC_lexicon_tags_19 },
    { 0x13efe336, SUC_lexicon_tags_15 },
    { 0x22afe337, SUC_lexicon_tags_2 },
    { 0xb4afe335, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b4fe33a, SUC_lexicon_tags_25 },
    { 0xb72fe33b, SUC_lexicon_tags_10 },
    { 0xaaafe33b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1d6fe33e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x326fe341, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd6fe343, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9cfe345, SUC_lexicon_tags_49 },
    { 0xe10fe345, SUC_lexicon_tags_7 },
    { 0xb6fe347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02fe34c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35efe350, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2fe353, SUC_lexicon_tags_21 },
    { 0xd78fe353, SUC_lexicon_tags_9 },
    { 0x88cfe353, SUC_lexicon_tags_65 },
    { 0xf66fe356, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x898fe358, SUC_lexicon_tags_7 },
    { 0x83efe359, SUC_lexicon_tags_32 },
    { 0xf90fe35a, SUC_lexicon_tags_15 },
    { 0xbfcfe35a, SUC_lexicon_tags_63 },
    { 0xc84fe358, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5efe35e, SUC_lexicon_tags_12 },
    { 0x9c6fe35f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88cfe361, SUC_lexicon_tags_37 },
    { 0x87afe362, SUC_lexicon_tags_30 },
    { 0xc5cfe361, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0fe36b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb08fe36e, SUC_lexicon_tags_29 },
    { 0x5e0fe36e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91afe376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9efe37c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7cfe37e, SUC_lexicon_tags_16 },
    { 0xe70fe37e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574fe383, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfafe387, SUC_lexicon_tags_18 },
    { 0x820fe387, SUC_lexicon_tags_55 },
    { 0x5cafe388, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xef6fe38b, SUC_lexicon_tags_37 },
    { 0xf6cfe38c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdccfe391, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd56fe393, SUC_lexicon_tags_2 },
    { 0xd3afe394, SUC_lexicon_tags_25 },
    { 0xbb0fe395, SUC_lexicon_tags_16 },
    { 0x936fe395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cfe39a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cafe39e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x874fe3a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9dafe3a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfafe3a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cfe3aa, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0xa60fe3ac, SUC_lexicon_tags_12 },
    { 0x5b4fe3ac, SUC_lexicon_tags_26 },
    { 0x256fe3ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x838fe3b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x73cfe3b2, SUC_lexicon_tags_15 },
    { 0xb5efe3b3, SUC_lexicon_tags_37 },
    { 0x444fe3b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2fe3bb, SUC_lexicon_tags_25 },
    { 0x8dafe3bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf42fe3be, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78fe3c2, SUC_lexicon_tags_33 },
    { 0x30fe3c2, SUC_lexicon_tags_0 },
    { 0xd58fe3c3, SUC_lexicon_tags_12 },
    { 0x3ecfe3c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2fe3c8, SUC_lexicon_tags_14 },
    { 0x328fe3c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158fe3ce, SUC_lexicon_tags_597 },
    { 0x51cfe3cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccfe3d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20fe3d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3afe3d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68efe3dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78fe3e2, SUC_lexicon_tags_23 },
    { 0xf74fe3e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x676fe3e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x184fe3e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfbcfe3e9, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49efe3ed, SUC_lexicon_tags_65 },
    { 0xcc8fe3ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60afe3f1, SUC_lexicon_tags_2 },
    { 0xeb6fe3f1, SUC_lexicon_tags_1 },
    { 0xa8fe3f3, SUC_lexicon_tags_11 },
    { 0x958fe3f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2afe3f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cafe3fd, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0x94efe400, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9afe403, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x180fe406, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0fe408, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4fe410, SUC_lexicon_tags_9 },
    { 0xb5cfe410, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc4fe416, SUC_lexicon_tags_10 },
    { 0xddefe417, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fe41c, SUC_lexicon_tags_11 },
    { 0x49efe41d, SUC_lexicon_tags_37 },
    { 0xec0fe41e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26afe421, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100fe429, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc44fe42d, SUC_lexicon_tags_2 },
    { 0x49afe42e, SUC_lexicon_tags_151 },
    { 0x1dcfe42e, SUC_lexicon_tags_7 },
    { 0xb46fe430, SUC_lexicon_tags_17 },
    { 0xf62fe42f, SUC_lexicon_tags_26 },
    { 0x476fe432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0fe439, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58cfe43f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa44fe441, SUC_lexicon_tags_17 },
    { 0xb0afe442, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2fe446, SUC_lexicon_tags_27 },
    { 0xa04fe446, SUC_lexicon_tags_134 },
    { 0x82fe446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6fe44c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe450, SUC_lexicon_tags_37 },
    { 0x126fe451, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf0fe453, SUC_lexicon_tags_23 },
    { 0xbe4fe454, SUC_lexicon_tags_47 },
    { 0x6e8fe455, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x74afe457, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x44afe459, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x626fe45b, SUC_lexicon_tags_25 },
    { 0x2c2fe45c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe62fe45f, SUC_lexicon_tags_1 },
    { 0x71afe460, SUC_lexicon_tags_194 },
    { 0x44fe460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc14fe463, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0fe467, SUC_lexicon_tags_1 },
    { 0xce0fe468, SUC_lexicon_tags_11 },
    { 0xabcfe468, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcb2fe46b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356fe46f, SUC_lexicon_tags_16 },
    { 0xa04fe46f, SUC_lexicon_tags_37 },
    { 0x428fe471, SUC_lexicon_tags_26 },
    { 0x7dcfe46f, SUC_lexicon_tags_10 },
    { 0x8bafe473, SUC_lexicon_tags_46 },
    { 0xa02fe474, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe4efe476, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60efe480, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xccafe482, SUC_lexicon_tags_47 },
    { 0xd0efe483, SUC_lexicon_tags_25 },
    { 0x82cfe484, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdd8fe486, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2fe48b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6fe48e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fe494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90fe497, SUC_lexicon_tags_68 },
    { 0x2ecfe498, SUC_lexicon_tags_46 },
    { 0xc62fe497, SUC_lexicon_tags_12 },
    { 0x448fe499, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9efe49d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x664fe49f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa30fe4a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5defe4a7, SUC_lexicon_tags_27 },
    { 0x2befe4a8, SUC_lexicon_tags_558 },
    { 0x146fe4a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44efe4ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59afe4ae, SUC_lexicon_tags_11 },
    { 0x142fe4af, SUC_lexicon_tags_25 },
    { 0x2a2fe4af, SUC_lexicon_tags_34 },
    { 0xbc6fe4ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb64fe4b3, SUC_lexicon_tags_74 },
    { 0x73cfe4b4, SUC_lexicon_tags_60 },
    { 0xfb6fe4b5, SUC_lexicon_tags_11 },
    { 0x5e2fe4b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cfe4b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eafe4c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x38afe4cb, SUC_lexicon_tags_136 },
    { 0x2acfe4cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbafe4d2, SUC_lexicon_tags_19 },
    { 0x858fe4d3, SUC_lexicon_tags_37 },
    { 0x37cfe4d4, SUC_lexicon_tags_37 },
    { 0x3b6fe4d5, SUC_lexicon_tags_30 },
    { 0xb90fe4d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2cfe4d9, SUC_lexicon_tags_15 },
    { 0x70cfe4da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4fe4de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa82fe4e0, SUC_lexicon_tags_133 },
    { 0x6eafe4e0, SUC_lexicon_tags_25 },
    { 0xaa0fe4e0, SUC_lexicon_tags_29 },
    { 0xb2afe4e3, SUC_lexicon_tags_25 },
    { 0x2a0fe4e4, SUC_lexicon_tags_11 },
    { 0x47efe4e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x892fe4e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd68fe4ee, SUC_lexicon_tags_17 },
    { 0xbaafe4ef, SUC_lexicon_tags_7 },
    { 0x6ccfe4ef, SUC_lexicon_tags_14 },
    { 0x3d8fe4f0, SUC_lexicon_tags_1 },
    { 0xea4fe4f0, SUC_lexicon_tags_25 },
    { 0x900fe4f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x39efe4f6, SUC_lexicon_tags_1 },
    { 0xb96fe4f7, SUC_lexicon_tags_12 },
    { 0x47efe4f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfefe4fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0fe4fe, SUC_lexicon_tags_38 },
    { 0xc64fe4ff, SUC_lexicon_tags_11 },
    { 0x544fe4fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaafe502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c2fe50a, SUC_lexicon_tags_17 },
    { 0xcd6fe50a, SUC_lexicon_tags_37 },
    { 0x198fe50c, SUC_lexicon_tags_28 },
    { 0x384fe50c, SUC_lexicon_tags_7 },
    { 0x9defe50e, SUC_lexicon_tags_47 },
    { 0xd78fe50f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x42efe512, SUC_lexicon_tags_25 },
    { 0x8defe513, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x138fe515, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x436fe518, SUC_lexicon_tags_33 },
    { 0xb3afe519, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e2fe51c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fe520, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2fe523, SUC_lexicon_tags_44 },
    { 0x13cfe524, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ecfe528, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x352fe52a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x220fe52e, SUC_lexicon_tags_47 },
    { 0x88afe52e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4cfe532, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcbefe534, SUC_lexicon_tags_63 },
    { 0xbb4fe535, SUC_lexicon_tags_37 },
    { 0x102fe536, SUC_lexicon_tags_37 },
    { 0x4cefe536, SUC_lexicon_tags_37 },
    { 0x290fe537, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccafe53c, SUC_lexicon_tags_7 },
    { 0xb70fe53d, SUC_lexicon_tags_7 },
    { 0x4c0fe53c, SUC_lexicon_tags_38 },
    { 0x92cfe53f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d6fe541, SUC_lexicon_tags_38 },
    { 0x8a8fe541, SUC_lexicon_tags_1 },
    { 0x4f2fe543, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb0fe545, SUC_lexicon_tags_65 },
    { 0x3d2fe546, SUC_lexicon_tags_11 },
    { 0x99cfe547, SUC_lexicon_tags_7 },
    { 0xbaafe547, SUC_lexicon_tags_31 },
    { 0x34efe549, SUC_lexicon_tags_28 },
    { 0x81efe547, SUC_lexicon_tags_60 },
    { 0x9aafe549, SUC_lexicon_tags_17 },
    { 0x69efe54a, SUC_lexicon_tags_37 },
    { 0x636fe54d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fe551, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612fe555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x748fe558, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x2dcfe55a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x734fe55d, SUC_lexicon_tags_25 },
    { 0x2f0fe55d, SUC_lexicon_tags_37 },
    { 0x202fe55f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61efe563, SUC_lexicon_tags_7 },
    { 0xfb2fe563, SUC_lexicon_tags_1 },
    { 0xf5cfe565, SUC_lexicon_tags_49 },
    { 0xa48fe566, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x404fe568, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefefe571, SUC_lexicon_tags_30 },
    { 0xccafe572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4fe575, SUC_lexicon_tags_17 },
    { 0xdeafe576, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5efe57a, SUC_lexicon_tags_25 },
    { 0x904fe57a, SUC_lexicon_tags_43 },
    { 0x3befe57b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38fe57e, SUC_lexicon_tags_26 },
    { 0x28efe57f, SUC_lexicon_tags_255 },
    { 0x57cfe57f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x804fe582, SUC_lexicon_tags_31 },
    { 0xa5cfe583, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf58fe585, SUC_lexicon_tags_9 },
    { 0x45efe586, SUC_lexicon_tags_63 },
    { 0xfcfe587, SUC_lexicon_tags_31 },
    { 0xa9afe587, SUC_lexicon_tags_69 },
    { 0xd68fe589, SUC_lexicon_tags_20 },
    { 0x4bcfe58a, SUC_lexicon_tags_11 },
    { 0x5b8fe588, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fe58e, SUC_lexicon_tags_49 },
    { 0x4e4fe58e, SUC_lexicon_tags_17 },
    { 0xd90fe590, SUC_lexicon_tags_1 },
    { 0x76fe591, SUC_lexicon_tags_1 },
    { 0x39efe592, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71efe594, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4fe59a, SUC_lexicon_tags_15 },
    { 0x330fe59b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4fe59e, SUC_lexicon_tags_10 },
    { 0x75afe59f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b2fe5a1, SUC_lexicon_tags_165 },
    { 0x2aefe5a1, SUC_lexicon_tags_49 },
    { 0xc2fe5a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x868fe5ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x88efe5ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6fe5b1, SUC_lexicon_tags_11 },
    { 0xa2efe5b2, SUC_lexicon_tags_2 },
    { 0x5f4fe5b3, SUC_lexicon_tags_35 },
    { 0x38efe5b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabafe5b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfaefe5ba, SUC_lexicon_tags_32 },
    { 0xfe2fe5ba, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3afe5c2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fafe5c5, SUC_lexicon_tags_38 },
    { 0x42fe5c5, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x14cfe5c8, SUC_lexicon_tags_8 },
    { 0x88efe5c9, SUC_lexicon_tags_28 },
    { 0xb36fe5ca, SUC_lexicon_tags_14 },
    { 0x19cfe5cb, SUC_lexicon_tags_49 },
    { 0xfc0fe5cc, SUC_lexicon_tags_25 },
    { 0x752fe5cb, SUC_lexicon_tags_11 },
    { 0x2f6fe5ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20efe5d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710fe5e1, SUC_lexicon_tags_11 },
    { 0x584fe5e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x660fe5e4, SUC_lexicon_tags_17 },
    { 0x622fe5e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0fe5e8, SUC_lexicon_tags_33 },
    { 0xcd8fe5e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccfe5eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878fe5f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6afe5f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cfe5f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18fe5fd, SUC_lexicon_tags_37 },
    { 0x2b8fe5fe, SUC_lexicon_tags_1 },
    { 0xef6fe5ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x142fe602, SUC_lexicon_tags_17 },
    { 0x3c8fe603, SUC_lexicon_tags_30 },
    { 0x9b2fe604, SUC_lexicon_tags_14 },
    { 0x8a8fe605, SUC_lexicon_tags_11 },
    { 0x790fe603, SUC_lexicon_tags_29 },
    { 0x22efe606, SUC_lexicon_tags_8 },
    { 0xea0fe603, SUC_lexicon_tags_4 },
    { 0x12cfe609, SUC_lexicon_tags_9 },
    { 0x994fe60a, SUC_lexicon_tags_2 },
    { 0x54afe60a, SUC_lexicon_tags_37 },
    { 0x1e4fe60a, SUC_lexicon_tags_47 },
    { 0x746fe60d, SUC_lexicon_tags_25 },
    { 0xc0efe602, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafe611, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb88fe613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4efe615, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6defe618, SUC_lexicon_tags_37 },
    { 0x644fe619, SUC_lexicon_tags_26 },
    { 0x52fe618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6fe61e, SUC_lexicon_tags_21 },
    { 0x69cfe61f, SUC_lexicon_tags_17 },
    { 0xecafe61e, SUC_lexicon_tags_11 },
    { 0x808fe621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c0fe623, SUC_lexicon_tags_43 },
    { 0x53cfe623, SUC_lexicon_tags_31 },
    { 0x5d0fe623, SUC_lexicon_tags_47 },
    { 0x386fe626, SUC_lexicon_tags_1 },
    { 0x786fe627, SUC_lexicon_tags_11 },
    { 0x928fe627, SUC_lexicon_tags_6 },
    { 0x112fe626, SUC_lexicon_tags_11 },
    { 0xb22fe62a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cfe631, SUC_lexicon_tags_19 },
    { 0x744fe632, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb46fe634, SUC_lexicon_tags_12 },
    { 0xdfefe635, SUC_lexicon_tags_31 },
    { 0x436fe635, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x95afe638, SUC_lexicon_tags_43 },
    { 0x804fe639, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x876fe641, SUC_lexicon_tags_11 },
    { 0x588fe641, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294fe64c, SUC_lexicon_tags_31 },
    { 0x6fefe64d, SUC_lexicon_tags_1 },
    { 0xb04fe64e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa4afe650, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x67cfe653, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a4fe655, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97efe658, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48fe661, SUC_lexicon_tags_45 },
    { 0x93efe662, SUC_lexicon_tags_19 },
    { 0xb40fe663, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaefe666, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f0fe66a, SUC_lexicon_tags_79 },
    { 0x7cfe66a, SUC_lexicon_tags_6 },
    { 0x404fe66c, SUC_lexicon_tags_11 },
    { 0xd7afe66d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fe672, SUC_lexicon_tags_25 },
    { 0x69efe672, SUC_lexicon_tags_16 },
    { 0xb08fe673, SUC_lexicon_tags_48 },
    { 0xb9cfe675, SUC_lexicon_tags_14 },
    { 0x8fe676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4defe679, SUC_lexicon_tags_3 },
    { 0xdf2fe679, SUC_lexicon_tags_34 },
    { 0x46afe67a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd0fe67d, SUC_lexicon_tags_2 },
    { 0x756fe67e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8fe686, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x802fe688, SUC_lexicon_tags_11 },
    { 0x7d2fe689, SUC_lexicon_tags_1 },
    { 0xb88fe68a, SUC_lexicon_tags_77 },
    { 0x8ecfe689, SUC_lexicon_tags_12 },
    { 0xaafe68a, SUC_lexicon_tags_15 },
    { 0xc94fe68c, SUC_lexicon_tags_77 },
    { 0x29afe68e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9efe693, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc98fe696, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x274fe69e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d4fe6a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x738fe6a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0fe6a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aafe6ac, SUC_lexicon_tags_15 },
    { 0x8bafe6ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x140fe6b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf08fe6b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa14fe6b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf42fe6b8, SUC_lexicon_tags_14 },
    { 0x5c6fe6b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500fe6bd, SUC_lexicon_tags_11 },
    { 0xe46fe6be, SUC_lexicon_tags_43 },
    { 0xd0fe6be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x672fe6c2, SUC_lexicon_tags_9 },
    { 0x3d8fe6c3, SUC_lexicon_tags_15 },
    { 0x8a6fe6c4, SUC_lexicon_tags_23 },
    { 0xfacfe6c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0fe6c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336fe6cc, SUC_lexicon_tags_15 },
    { 0x4b2fe6cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x666fe6cf, SUC_lexicon_tags_17 },
    { 0x390fe6d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8fe6d3, SUC_lexicon_tags_46 },
    { 0xfb6fe6d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38cfe6de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e4fe6e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd2fe6e3, SUC_lexicon_tags_26 },
    { 0x278fe6e4, SUC_lexicon_tags_12 },
    { 0x152fe6e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374fe6eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fafe6ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eafe6f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xab2fe6f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc2fe6f5, SUC_lexicon_tags_87 },
    { 0xec2fe6f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7b0fe6f8, SUC_lexicon_tags_38 },
    { 0x580fe6f9, SUC_lexicon_tags_1 },
    { 0xb86fe6fa, SUC_lexicon_tags_0 },
    { 0x648fe6fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcfe6ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x772fe705, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e0fe708, SUC_lexicon_tags_1 },
    { 0xe5efe709, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83afe70b, SUC_lexicon_tags_4 },
    { 0xeb0fe70c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970fe710, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83afe716, SUC_lexicon_tags_38 },
    { 0x4c6fe717, SUC_lexicon_tags_37 },
    { 0x12fe718, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x540fe71c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6fe727, SUC_lexicon_tags_7 },
    { 0x40efe728, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x746fe72a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8fe732, SUC_lexicon_tags_0 },
    { 0x5a2fe733, SUC_lexicon_tags_38 },
    { 0xc58fe734, SUC_lexicon_tags_27 },
    { 0x230fe734, SUC_lexicon_tags_38 },
    { 0x730fe736, SUC_lexicon_tags_55 },
    { 0xc36fe736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac0fe739, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x290fe73b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6fe73e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe54fe740, SUC_lexicon_tags_38 },
    { 0xd72fe741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x718fe743, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x28fe745, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10fe747, SUC_lexicon_tags_17 },
    { 0x9d2fe747, SUC_lexicon_tags_11 },
    { 0x324fe749, SUC_lexicon_tags_10 },
    { 0xce8fe74a, SUC_lexicon_tags_1 },
    { 0xc20fe74a, SUC_lexicon_tags_83 },
    { 0xc3cfe74c, SUC_lexicon_tags_19 },
    { 0x4d4fe74a, SUC_lexicon_tags_38 },
    { 0x642fe74e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x624fe751, SUC_lexicon_tags_14 },
    { 0xf24fe751, SUC_lexicon_tags_65 },
    { 0xcb8fe753, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86afe758, SUC_lexicon_tags_38 },
    { 0x84cfe758, SUC_lexicon_tags_38 },
    { 0x176fe759, SUC_lexicon_tags_37 },
    { 0x8ecfe75b, SUC_lexicon_tags_7 },
    { 0xc36fe75c, SUC_lexicon_tags_11 },
    { 0x526fe75d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed8fe760, SUC_lexicon_tags_17 },
    { 0x4eafe760, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadafe766, SUC_lexicon_tags_8 },
    { 0xe7cfe767, SUC_lexicon_tags_17 },
    { 0x302fe768, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb70fe76a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfefe76c, SUC_lexicon_tags_37 },
    { 0xbc4fe76d, SUC_lexicon_tags_2 },
    { 0xcc6fe76e, SUC_lexicon_tags_140 },
    { 0x550fe76f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x73cfe772, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe777, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba0fe77c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf74fe77e, SUC_lexicon_tags_11 },
    { 0xfaefe77f, SUC_lexicon_tags_37 },
    { 0x888fe780, SUC_lexicon_tags_68 },
    { 0x64efe781, SUC_lexicon_tags_28 },
    { 0x3e8fe77f, SUC_lexicon_tags_63 },
    { 0xd5efe783, SUC_lexicon_tags_17 },
    { 0xa22fe784, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f8fe786, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xab0fe788, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc64fe78b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2defe78e, SUC_lexicon_tags_14 },
    { 0x96afe78f, SUC_lexicon_tags_14 },
    { 0x1bcfe78e, SUC_lexicon_tags_4 },
    { 0xdaafe78f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b2fe793, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe82fe79b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x240fe79d, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8fe7a7, SUC_lexicon_tags_30 },
    { 0x3befe7a7, SUC_lexicon_tags_15 },
    { 0x96cfe7a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x452fe7ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbcfe7af, SUC_lexicon_tags_47 },
    { 0x62cfe7b0, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x774fe7b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37efe7bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x364fe7bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x260fe7c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1fcfe7c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x308fe7c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x482fe7cd, SUC_lexicon_tags_47 },
    { 0xeaafe7ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x426fe7d0, SUC_lexicon_tags_40 },
    { 0xce2fe7d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4fe7da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x698fe7dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98fe7e0, SUC_lexicon_tags_37 },
    { 0x110fe7e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd6fe7e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8fe7e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270fe7f0, SUC_lexicon_tags_17 },
    { 0x6fafe7f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8fe7f5, SUC_lexicon_tags_47 },
    { 0x224fe7f6, SUC_lexicon_tags_8 },
    { 0xd42fe7f7, SUC_lexicon_tags_17 },
    { 0x1eefe7f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bafe7fb, SUC_lexicon_tags_10 },
    { 0x710fe7fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac2fe7fe, SUC_lexicon_tags_7 },
    { 0xe90fe7fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb60fe801, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72fe803, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3cfe80b, SUC_lexicon_tags_2 },
    { 0xbbafe80c, SUC_lexicon_tags_2 },
    { 0x438fe80d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x658fe80f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x768fe812, SUC_lexicon_tags_19 },
    { 0x54fe813, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8bcfe815, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6afe81c, SUC_lexicon_tags_3 },
    { 0x9cefe81d, SUC_lexicon_tags_11 },
    { 0x394fe81d, SUC_lexicon_tags_43 },
    { 0xc96fe81c, SUC_lexicon_tags_30 },
    { 0x9bcfe81f, SUC_lexicon_tags_8 },
    { 0x462fe821, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6fe826, SUC_lexicon_tags_1 },
    { 0xf2cfe827, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x98efe82a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfdcfe82c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5f4fe82e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d8fe830, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c6fe832, SUC_lexicon_tags_25 },
    { 0x56fe833, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc60fe835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42afe83c, SUC_lexicon_tags_151 },
    { 0x99cfe83d, SUC_lexicon_tags_7 },
    { 0x574fe83d, SUC_lexicon_tags_31 },
    { 0xd94fe83e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4fe842, SUC_lexicon_tags_27 },
    { 0xa3efe843, SUC_lexicon_tags_17 },
    { 0xd60fe842, SUC_lexicon_tags_37 },
    { 0x66afe844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38afe84e, SUC_lexicon_tags_25 },
    { 0x3b6fe84f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd08fe853, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbecfe856, SUC_lexicon_tags_7 },
    { 0xbefe856, SUC_lexicon_tags_11 },
    { 0xc46fe858, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c4fe85c, SUC_lexicon_tags_16 },
    { 0x42cfe85c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17cfe85f, SUC_lexicon_tags_42 },
    { 0x93efe860, SUC_lexicon_tags_28 },
    { 0x7bcfe860, SUC_lexicon_tags_6 },
    { 0xaf4fe860, SUC_lexicon_tags_2 },
    { 0xd92fe863, SUC_lexicon_tags_11 },
    { 0x53afe864, SUC_lexicon_tags_17 },
    { 0xbfafe865, SUC_lexicon_tags_15 },
    { 0x5f4fe866, SUC_lexicon_tags_10 },
    { 0xefefe867, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa92fe86a, SUC_lexicon_tags_11 },
    { 0xe14fe86a, SUC_lexicon_tags_47 },
    { 0x5bcfe86b, SUC_lexicon_tags_7 },
    { 0xd42fe86c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdecfe86f, SUC_lexicon_tags_29 },
    { 0x4b2fe86f, SUC_lexicon_tags_30 },
    { 0x9e0fe86f, SUC_lexicon_tags_48 },
    { 0x37efe86f, SUC_lexicon_tags_39 },
    { 0x7e0fe871, SUC_lexicon_tags_63 },
    { 0xfafe874, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xabafe876, SUC_lexicon_tags_53 },
    { 0xf52fe877, SUC_lexicon_tags_12 },
    { 0xcfe878, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aafe87c, SUC_lexicon_tags_17 },
    { 0x11efe87d, SUC_lexicon_tags_15 },
    { 0x72efe87e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xceafe880, SUC_lexicon_tags_49 },
    { 0xa60fe880, SUC_lexicon_tags_12 },
    { 0xe14fe882, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x236fe887, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x70fe889, SUC_lexicon_tags_29 },
    { 0x52cfe889, SUC_lexicon_tags_127 },
    { 0x28efe88b, SUC_lexicon_tags_10 },
    { 0x9c2fe88c, SUC_lexicon_tags_18 },
    { 0xb4fe88d, SUC_lexicon_tags_38 },
    { 0xd9efe88d, SUC_lexicon_tags_33 },
    { 0xfcfe88e, SUC_lexicon_tags_1 },
    { 0x4c0fe88d, SUC_lexicon_tags_25 },
    { 0x65cfe88c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53afe894, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a8fe898, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89efe89f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32afe8a3, SUC_lexicon_tags_17 },
    { 0x748fe8a4, SUC_lexicon_tags_17 },
    { 0xabefe8a3, SUC_lexicon_tags_15 },
    { 0xe9afe8a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3afe8a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fefe8ad, SUC_lexicon_tags_25 },
    { 0x5a0fe8ad, SUC_lexicon_tags_15 },
    { 0x8cefe8ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61afe8b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4c8fe8b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8fe8ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7afe8bd, SUC_lexicon_tags_27 },
    { 0x790fe8be, SUC_lexicon_tags_64 },
    { 0x83efe8bf, SUC_lexicon_tags_31 },
    { 0x178fe8bd, SUC_lexicon_tags_1 },
    { 0xf5afe8bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cfe8c7, SUC_lexicon_tags_38 },
    { 0x9f2fe8c7, SUC_lexicon_tags_6 },
    { 0x588fe8c9, SUC_lexicon_tags_7 },
    { 0x59efe8c8, SUC_lexicon_tags_37 },
    { 0xa2efe8cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc28fe8ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc44fe8d0, SUC_lexicon_tags_12 },
    { 0xdcefe8d0, SUC_lexicon_tags_7 },
    { 0x824fe8d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6fe8d6, SUC_lexicon_tags_2 },
    { 0x440fe8d6, SUC_lexicon_tags_2 },
    { 0x4fe8d6, SUC_lexicon_tags_43 },
    { 0x7c6fe8d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aafe8dd, SUC_lexicon_tags_16 },
    { 0x108fe8de, SUC_lexicon_tags_17 },
    { 0x2ecfe8dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf4fe8e1, SUC_lexicon_tags_49 },
    { 0x936fe8e1, SUC_lexicon_tags_12 },
    { 0x65efe8e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x22afe8e6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x10fe8e8, SUC_lexicon_tags_19 },
    { 0xf8afe8e9, SUC_lexicon_tags_12 },
    { 0x808fe8ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd92fe8ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4defe8ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2fe8f2, SUC_lexicon_tags_63 },
    { 0x708fe8f3, SUC_lexicon_tags_1 },
    { 0x64efe8f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30fe8f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33efe8fa, SUC_lexicon_tags_49 },
    { 0x4cafe8fb, SUC_lexicon_tags_37 },
    { 0x7befe8fc, SUC_lexicon_tags_27 },
    { 0x206fe8fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4fe901, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754fe906, SUC_lexicon_tags_26 },
    { 0x4f0fe907, SUC_lexicon_tags_37 },
    { 0xb64fe907, SUC_lexicon_tags_46 },
    { 0x4a8fe908, SUC_lexicon_tags_6 },
    { 0x3ccfe907, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9efe90d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fefe911, SUC_lexicon_tags_82 },
    { 0xa82fe911, SUC_lexicon_tags_117 },
    { 0xf18fe913, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x752fe915, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7efe917, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f0fe919, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x550fe91b, SUC_lexicon_tags_15 },
    { 0x4fcfe91b, SUC_lexicon_tags_60 },
    { 0xba0fe91d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a4fe926, SUC_lexicon_tags_25 },
    { 0xc4fe927, SUC_lexicon_tags_17 },
    { 0x680fe928, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91cfe92b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10fe92f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa36fe931, SUC_lexicon_tags_336 },
    { 0x520fe932, SUC_lexicon_tags_37 },
    { 0xe50fe933, SUC_lexicon_tags_37 },
    { 0x2eefe933, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x918fe936, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba6fe938, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x14efe93b, SUC_lexicon_tags_63 },
    { 0x356fe93c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefe944, SUC_lexicon_tags_0 },
    { 0x2d4fe944, SUC_lexicon_tags_45 },
    { 0xd34fe944, SUC_lexicon_tags_64 },
    { 0xcaefe944, SUC_lexicon_tags_25 },
    { 0x238fe944, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cafe94b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cfe94f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x116fe952, SUC_lexicon_tags_66 },
    { 0x7cefe953, SUC_lexicon_tags_60 },
    { 0x346fe952, SUC_lexicon_tags_37 },
    { 0x510fe955, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54fe95b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404fe95f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13efe962, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x28afe965, SUC_lexicon_tags_60 },
    { 0x782fe965, SUC_lexicon_tags_37 },
    { 0x5acfe965, SUC_lexicon_tags_37 },
    { 0x32fe968, SUC_lexicon_tags_17 },
    { 0xaa8fe968, SUC_lexicon_tags_2 },
    { 0x724fe96a, SUC_lexicon_tags_27 },
    { 0x826fe96b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52fe96f, SUC_lexicon_tags_12 },
    { 0xd34fe970, SUC_lexicon_tags_1 },
    { 0x52fe970, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2fe973, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3b8fe975, SUC_lexicon_tags_9 },
    { 0x102fe976, SUC_lexicon_tags_0 },
    { 0xcecfe975, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34fe97a, SUC_lexicon_tags_1 },
    { 0xcccfe97b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fe97f, SUC_lexicon_tags_14 },
    { 0x66cfe97f, SUC_lexicon_tags_39 },
    { 0xcbefe97f, SUC_lexicon_tags_60 },
    { 0x404fe97f, SUC_lexicon_tags_12 },
    { 0x272fe983, SUC_lexicon_tags_37 },
    { 0x6ecfe984, SUC_lexicon_tags_39 },
    { 0xab2fe980, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a4fe987, SUC_lexicon_tags_91 },
    { 0x980fe988, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21cfe98c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4fe98f, SUC_lexicon_tags_46 },
    { 0xa0afe98f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x176fe992, SUC_lexicon_tags_15 },
    { 0x5f0fe993, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb94fe995, SUC_lexicon_tags_25 },
    { 0x90afe996, SUC_lexicon_tags_4 },
    { 0x6e2fe997, SUC_lexicon_tags_1 },
    { 0x1cefe998, SUC_lexicon_tags_1 },
    { 0xc8cfe997, SUC_lexicon_tags_43 },
    { 0xd4cfe99a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa02fe9a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30fe9a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x530fe9a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e6fe9a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x464fe9ac, SUC_lexicon_tags_25 },
    { 0x2f8fe9ac, SUC_lexicon_tags_1 },
    { 0xbe0fe9ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x58afe9b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9acfe9b3, SUC_lexicon_tags_15 },
    { 0xd7cfe9b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fe9ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8acfe9c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1cfe9c9, SUC_lexicon_tags_16 },
    { 0x24cfe9ca, SUC_lexicon_tags_336 },
    { 0xfaefe9c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x562fe9ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78fe9d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9afe9d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdb6fe9da, SUC_lexicon_tags_46 },
    { 0x8dafe9da, SUC_lexicon_tags_44 },
    { 0xcd0fe9db, SUC_lexicon_tags_21 },
    { 0xad6fe9da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42afe9e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8cfe9eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91afe9f1, SUC_lexicon_tags_15 },
    { 0xea0fe9f2, SUC_lexicon_tags_21 },
    { 0x74cfe9f2, SUC_lexicon_tags_101 },
    { 0x110fe9f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x282fe9f8, SUC_lexicon_tags_43 },
    { 0xfa0fe9f8, SUC_lexicon_tags_23 },
    { 0xb32fe9fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe76fe9fc, SUC_lexicon_tags_138 },
    { 0x56fe9fc, SUC_lexicon_tags_2 },
    { 0x174fe9fe, SUC_lexicon_tags_28 },
    { 0xba4fe9ff, SUC_lexicon_tags_23 },
    { 0x278fea00, SUC_lexicon_tags_11 },
    { 0xcc2fe9ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x534fea05, SUC_lexicon_tags_11 },
    { 0xfb6fea05, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4fea09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb60fea0c, SUC_lexicon_tags_38 },
    { 0x70cfea0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578fea11, SUC_lexicon_tags_2 },
    { 0xf50fea11, SUC_lexicon_tags_25 },
    { 0x6d4fea12, SUC_lexicon_tags_5 },
    { 0x6f6fea14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53efea16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36efea1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90afea22, SUC_lexicon_tags_17 },
    { 0xb12fea22, SUC_lexicon_tags_11 },
    { 0x58afea24, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbefea29, SUC_lexicon_tags_17 },
    { 0x3afea2a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cfea2f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb70fea31, SUC_lexicon_tags_37 },
    { 0x4c4fea32, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x664fea34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ccfea37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0fea3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x33afea3c, SUC_lexicon_tags_4 },
    { 0x17cfea3d, SUC_lexicon_tags_68 },
    { 0xe3efea3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x84cfea41, SUC_lexicon_tags_33 },
    { 0x70afea42, SUC_lexicon_tags_37 },
    { 0x76cfea42, SUC_lexicon_tags_37 },
    { 0xba8fea43, SUC_lexicon_tags_1 },
    { 0x798fea45, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb4efea47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5defea4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20fea4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a6fea50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x92fea52, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x922fea55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc74fea57, SUC_lexicon_tags_16 },
    { 0xaccfea58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ecfea5a, SUC_lexicon_tags_64 },
    { 0x950fea5a, SUC_lexicon_tags_77 },
    { 0x862fea5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe60fea60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x210fea62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfea65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6fea6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbefea71, SUC_lexicon_tags_15 },
    { 0xebcfea72, SUC_lexicon_tags_9 },
    { 0x578fea73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf26fea76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33afea78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cfea7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc32fea7f, SUC_lexicon_tags_66 },
    { 0x9d6fea80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39efea89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce0fea8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a0fea8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x820fea90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54fea92, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xb42fea94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6fea98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x838fea9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12feaa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x626feaa4, SUC_lexicon_tags_37 },
    { 0x582feaa4, SUC_lexicon_tags_27 },
    { 0xf9efeaa5, SUC_lexicon_tags_37 },
    { 0xec6feaa6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabafeaab, SUC_lexicon_tags_6 },
    { 0xfcefeaac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb26feabb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4d0feabd, SUC_lexicon_tags_37 },
    { 0xf5cfeabd, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xdf2feac0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296feac6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80feac8, SUC_lexicon_tags_2 },
    { 0x4d2feac9, SUC_lexicon_tags_47 },
    { 0xc50feaca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecfeace, SUC_lexicon_tags_17 },
    { 0x89afeacf, SUC_lexicon_tags_25 },
    { 0xfb4feacf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa24fead2, SUC_lexicon_tags_11 },
    { 0xa10fead3, SUC_lexicon_tags_346 },
    { 0, NULL },
    { 0xc10fead5, SUC_lexicon_tags_30 },
    { 0xa4afead6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x512fead9, SUC_lexicon_tags_25 },
    { 0x120fead9, SUC_lexicon_tags_4 },
    { 0xdc2feadb, SUC_lexicon_tags_8 },
    { 0xd5afead9, SUC_lexicon_tags_46 },
    { 0x12afeadd, SUC_lexicon_tags_49 },
    { 0xdbafeade, SUC_lexicon_tags_79 },
    { 0x4ecfeadf, SUC_lexicon_tags_7 },
    { 0x9fcfeae0, SUC_lexicon_tags_2 },
    { 0x52cfeae1, SUC_lexicon_tags_46 },
    { 0x4ccfeae2, SUC_lexicon_tags_36 },
    { 0x17cfeae3, SUC_lexicon_tags_63 },
    { 0xcdafeae4, SUC_lexicon_tags_17 },
    { 0x4cafeae1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6feae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca8feaed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1efeaef, SUC_lexicon_tags_7 },
    { 0x376feaef, SUC_lexicon_tags_7 },
    { 0xb46feaf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2cfeaf4, SUC_lexicon_tags_11 },
    { 0xf5afeaf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x404feaf7, SUC_lexicon_tags_2 },
    { 0x1b2feaf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16feafb, SUC_lexicon_tags_7 },
    { 0x798feafc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a4feafe, SUC_lexicon_tags_0 },
    { 0x1f2feaff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf00feb08, SUC_lexicon_tags_14 },
    { 0x41efeb08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x406feb0b, SUC_lexicon_tags_68 },
    { 0x840feb0b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb40feb10, SUC_lexicon_tags_17 },
    { 0x78efeb11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x288feb14, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91efeb18, SUC_lexicon_tags_1 },
    { 0x46efeb19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb22feb1d, SUC_lexicon_tags_2 },
    { 0x95efeb1d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24feb21, SUC_lexicon_tags_21 },
    { 0x842feb22, SUC_lexicon_tags_16 },
    { 0xa28feb23, SUC_lexicon_tags_37 },
    { 0x6b2feb24, SUC_lexicon_tags_39 },
    { 0x47afeb23, SUC_lexicon_tags_1 },
    { 0x1ecfeb26, SUC_lexicon_tags_17 },
    { 0xa46feb27, SUC_lexicon_tags_17 },
    { 0x2e4feb23, SUC_lexicon_tags_38 },
    { 0x194feb25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4cfeb2b, SUC_lexicon_tags_37 },
    { 0x5f4feb2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532feb34, SUC_lexicon_tags_37 },
    { 0x5a0feb35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x628feb38, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x7aafeb3a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x684feb42, SUC_lexicon_tags_30 },
    { 0x300feb43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8cfeb48, SUC_lexicon_tags_0 },
    { 0xe6feb48, SUC_lexicon_tags_37 },
    { 0x1fcfeb49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250feb4e, SUC_lexicon_tags_29 },
    { 0x44efeb4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x49efeb51, SUC_lexicon_tags_3 },
    { 0x404feb51, SUC_lexicon_tags_37 },
    { 0x7efeb52, SUC_lexicon_tags_45 },
    { 0x12afeb54, SUC_lexicon_tags_11 },
    { 0x226feb54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc24feb57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x31cfeb59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36feb5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cfeb5f, SUC_lexicon_tags_14 },
    { 0x694feb5f, SUC_lexicon_tags_7 },
    { 0x1d8feb61, SUC_lexicon_tags_14 },
    { 0xe1afeb61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2feb67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdefeb70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe96feb74, SUC_lexicon_tags_2 },
    { 0x530feb75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65cfeb77, SUC_lexicon_tags_17 },
    { 0xf46feb77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2cfeb7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d6feb89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x248feb8d, SUC_lexicon_tags_10 },
    { 0x1c0feb8e, SUC_lexicon_tags_15 },
    { 0xa16feb8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbafeb94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x388feb98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd8feb9c, SUC_lexicon_tags_37 },
    { 0x9e4feb9d, SUC_lexicon_tags_11 },
    { 0xf94feb9e, SUC_lexicon_tags_11 },
    { 0x9d6feb9c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112feba3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x422feba6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe0feba9, SUC_lexicon_tags_66 },
    { 0x1fafebaa, SUC_lexicon_tags_80 },
    { 0x1cafebab, SUC_lexicon_tags_27 },
    { 0x5ecfebac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0febaf, SUC_lexicon_tags_25 },
    { 0x460febb0, SUC_lexicon_tags_15 },
    { 0xd0febb0, SUC_lexicon_tags_2 },
    { 0xed4febb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd88febb4, SUC_lexicon_tags_4 },
    { 0x2afebb4, SUC_lexicon_tags_1 },
    { 0x7e4febb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27efebb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f4febba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x376febbc, SUC_lexicon_tags_2 },
    { 0x726febbd, SUC_lexicon_tags_1234 },
    { 0x4b4febbe, SUC_lexicon_tags_1 },
    { 0x32afebbf, SUC_lexicon_tags_11 },
    { 0x8e2febc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x500febc4, SUC_lexicon_tags_33 },
    { 0xea6febc5, SUC_lexicon_tags_28 },
    { 0x358febc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcfebcb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3afebcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a0febd1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaefebd8, SUC_lexicon_tags_38 },
    { 0x4a6febd9, SUC_lexicon_tags_25 },
    { 0x91afebd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbcafebdc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cfebdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd50febe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dafebe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bcfebeb, SUC_lexicon_tags_25 },
    { 0x394febeb, SUC_lexicon_tags_2 },
    { 0xf1efebed, SUC_lexicon_tags_31 },
    { 0x2cfebee, SUC_lexicon_tags_0 },
    { 0xeb8febef, SUC_lexicon_tags_29 },
    { 0x730febf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90efebf2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x86cfebf5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8febfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x452fec01, SUC_lexicon_tags_3 },
    { 0x3b4fec02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74afec0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70fec14, SUC_lexicon_tags_17 },
    { 0xe6afec15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76fec17, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x51efec19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65afec20, SUC_lexicon_tags_0 },
    { 0x5cefec21, SUC_lexicon_tags_7 },
    { 0xe0cfec21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11afec28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4efec30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fec35, SUC_lexicon_tags_6 },
    { 0x4e8fec36, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xadefec38, SUC_lexicon_tags_36 },
    { 0x16cfec38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36efec3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe12fec41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf90fec43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x690fec46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972fec4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe60fec4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0fec53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c4fec55, SUC_lexicon_tags_12 },
    { 0xe0fec55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb14fec58, SUC_lexicon_tags_16 },
    { 0xa7cfec59, SUC_lexicon_tags_27 },
    { 0x496fec59, SUC_lexicon_tags_17 },
    { 0x4f4fec58, SUC_lexicon_tags_65 },
    { 0x848fec58, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3b4fec5e, SUC_lexicon_tags_14 },
    { 0xf82fec5e, SUC_lexicon_tags_46 },
    { 0x1c4fec60, SUC_lexicon_tags_39 },
    { 0xe8cfec5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x820fec65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612fec69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46afec6b, SUC_lexicon_tags_43 },
    { 0xeb0fec6b, SUC_lexicon_tags_17 },
    { 0xca4fec6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83cfec6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x970fec71, SUC_lexicon_tags_36 },
    { 0x252fec72, SUC_lexicon_tags_27 },
    { 0x5bafec73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfec76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa30fec78, SUC_lexicon_tags_63 },
    { 0x300fec79, SUC_lexicon_tags_1 },
    { 0x29efec7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x526fec7c, SUC_lexicon_tags_38 },
    { 0xccfec7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89efec80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x528fec83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x320fec8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1cfec8d, SUC_lexicon_tags_11 },
    { 0xaa6fec8d, SUC_lexicon_tags_12 },
    { 0xb9cfec8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85afec96, SUC_lexicon_tags_156 },
    { 0x61cfec97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36fec9a, SUC_lexicon_tags_16 },
    { 0xa70fec9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29afeca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ccfeca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0efecad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x288fecaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7afecba, SUC_lexicon_tags_103 },
    { 0x3d2fecbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4fecbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24feccc, SUC_lexicon_tags_68 },
    { 0x736feccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9acfecd1, SUC_lexicon_tags_15 },
    { 0xaa4fecd2, SUC_lexicon_tags_7 },
    { 0x68efecd1, SUC_lexicon_tags_7 },
    { 0x2a0fecd3, SUC_lexicon_tags_51 },
    { 0xc2afecd2, SUC_lexicon_tags_38 },
    { 0xb80fecd6, SUC_lexicon_tags_17 },
    { 0xc0afecd7, SUC_lexicon_tags_224 },
    { 0x982fecd4, SUC_lexicon_tags_49 },
    { 0x88efecd9, SUC_lexicon_tags_12 },
    { 0x2a0fecda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36efece2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcfece6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x904fecee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf38fecf4, SUC_lexicon_tags_37 },
    { 0x5aefecf4, SUC_lexicon_tags_14 },
    { 0xd38fecf5, SUC_lexicon_tags_19 },
    { 0xb12fecf7, SUC_lexicon_tags_26 },
    { 0xf3afecf6, SUC_lexicon_tags_15 },
    { 0x38cfecf6, SUC_lexicon_tags_10 },
    { 0xedcfecf8, SUC_lexicon_tags_37 },
    { 0x1defecfb, SUC_lexicon_tags_30 },
    { 0xb1efecfc, SUC_lexicon_tags_29 },
    { 0x6a8fecf5, SUC_lexicon_tags_17 },
    { 0x30fecfe, SUC_lexicon_tags_43 },
    { 0x37afecfe, SUC_lexicon_tags_75 },
    { 0xe2afecff, SUC_lexicon_tags_25 },
    { 0xb30fecfe, SUC_lexicon_tags_17 },
    { 0xaa0fecfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76afed04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5efed09, SUC_lexicon_tags_8 },
    { 0xb5efed0a, SUC_lexicon_tags_15 },
    { 0x426fed0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x280fed0f, SUC_lexicon_tags_37 },
    { 0x938fed10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4fed13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74fed17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0fed1e, SUC_lexicon_tags_37 },
    { 0x144fed1f, SUC_lexicon_tags_56 },
    { 0x6dafed20, SUC_lexicon_tags_82 },
    { 0xf4afed21, SUC_lexicon_tags_66 },
    { 0x620fed20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10fed27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcfed32, SUC_lexicon_tags_47 },
    { 0x91efed33, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4f6fed35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dcfed39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x852fed3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x80afed3e, SUC_lexicon_tags_25 },
    { 0xdfafed3f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6c0fed41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x664fed45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0cfed51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f6fed53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe06fed55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cafed59, SUC_lexicon_tags_26 },
    { 0x2ccfed5a, SUC_lexicon_tags_7 },
    { 0xb70fed5b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb0fed5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc96fed5f, SUC_lexicon_tags_12 },
    { 0xb30fed60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8fed63, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x22efed65, SUC_lexicon_tags_38 },
    { 0xc92fed65, SUC_lexicon_tags_12 },
    { 0xaecfed66, SUC_lexicon_tags_15 },
    { 0x590fed67, SUC_lexicon_tags_1 },
    { 0x706fed69, SUC_lexicon_tags_53 },
    { 0xba6fed6a, SUC_lexicon_tags_11 },
    { 0x6d0fed6b, SUC_lexicon_tags_17 },
    { 0xf94fed6b, SUC_lexicon_tags_25 },
    { 0x9e8fed6d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecfed72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c6fed76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd96fed78, SUC_lexicon_tags_11 },
    { 0x24efed78, SUC_lexicon_tags_21 },
    { 0xd5cfed7a, SUC_lexicon_tags_2 },
    { 0xacfed7b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654fed7f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x560fed82, SUC_lexicon_tags_7 },
    { 0xfdafed83, SUC_lexicon_tags_17 },
    { 0x146fed82, SUC_lexicon_tags_2 },
    { 0x686fed85, SUC_lexicon_tags_3 },
    { 0x4bafed86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99afed8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe26fed8c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x646fed8e, SUC_lexicon_tags_15 },
    { 0x58afed8f, SUC_lexicon_tags_18 },
    { 0x7fefed90, SUC_lexicon_tags_2 },
    { 0xbeefed90, SUC_lexicon_tags_15 },
    { 0x6cefed90, SUC_lexicon_tags_23 },
    { 0x180fed93, SUC_lexicon_tags_29 },
    { 0xd50fed91, SUC_lexicon_tags_1 },
    { 0x16efed95, SUC_lexicon_tags_2 },
    { 0x9d4fed90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x784fed98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x392fed9b, SUC_lexicon_tags_68 },
    { 0x34fed9c, SUC_lexicon_tags_63 },
    { 0x4d8fed9d, SUC_lexicon_tags_46 },
    { 0x15cfed9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14feda2, SUC_lexicon_tags_7 },
    { 0xb92feda2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x66cfeda5, SUC_lexicon_tags_31 },
    { 0x15efeda6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738fedae, SUC_lexicon_tags_47 },
    { 0x2a6fedae, SUC_lexicon_tags_12 },
    { 0x6f0fedae, SUC_lexicon_tags_37 },
    { 0x618fedb1, SUC_lexicon_tags_1 },
    { 0xb48fedb1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fedb5, SUC_lexicon_tags_34 },
    { 0x65cfedb6, SUC_lexicon_tags_8 },
    { 0x5f2fedb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610fedc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fedc4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x846fedc6, SUC_lexicon_tags_11 },
    { 0x968fedc7, SUC_lexicon_tags_14 },
    { 0x7f6fedc7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xce0fedca, SUC_lexicon_tags_11 },
    { 0xdfcfedca, SUC_lexicon_tags_26 },
    { 0x2fefedcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fafedce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0fedd2, SUC_lexicon_tags_7 },
    { 0x558fedd3, SUC_lexicon_tags_37 },
    { 0x144fedd4, SUC_lexicon_tags_37 },
    { 0xd82fedd5, SUC_lexicon_tags_15 },
    { 0x96afedd4, SUC_lexicon_tags_0 },
    { 0xf92fedd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5cfedd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58fede1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fede4, SUC_lexicon_tags_37 },
    { 0xa6fede5, SUC_lexicon_tags_47 },
    { 0x838fede6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbccfeded, SUC_lexicon_tags_8 },
    { 0x73efeded, SUC_lexicon_tags_2 },
    { 0x9eefedef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd02fedf1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89efedf6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa20fedf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cfedfd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfee00, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c6fee04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x890fee06, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa50fee08, SUC_lexicon_tags_11 },
    { 0x9f6fee08, SUC_lexicon_tags_34 },
    { 0x528fee09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd32fee0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732fee10, SUC_lexicon_tags_17 },
    { 0xc94fee11, SUC_lexicon_tags_46 },
    { 0x61afee12, SUC_lexicon_tags_23 },
    { 0xd5afee10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x940fee16, SUC_lexicon_tags_16 },
    { 0x86fee16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb06fee20, SUC_lexicon_tags_14 },
    { 0xed8fee21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x978fee23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3efee29, SUC_lexicon_tags_1 },
    { 0xff2fee2a, SUC_lexicon_tags_29 },
    { 0xe52fee2b, SUC_lexicon_tags_17 },
    { 0x620fee29, SUC_lexicon_tags_2 },
    { 0x3e6fee29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a2fee31, SUC_lexicon_tags_29 },
    { 0x614fee32, SUC_lexicon_tags_2 },
    { 0x9fcfee31, SUC_lexicon_tags_37 },
    { 0x7f2fee34, SUC_lexicon_tags_17 },
    { 0x444fee35, SUC_lexicon_tags_58 },
    { 0xc4afee36, SUC_lexicon_tags_27 },
    { 0x886fee37, SUC_lexicon_tags_25 },
    { 0xa98fee36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fee3f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc0fee49, SUC_lexicon_tags_11 },
    { 0x260fee4a, SUC_lexicon_tags_28 },
    { 0x3f4fee4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1befee4e, SUC_lexicon_tags_12 },
    { 0x1cafee4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6fee53, SUC_lexicon_tags_26 },
    { 0x21cfee54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66efee5b, SUC_lexicon_tags_17 },
    { 0x406fee5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x380fee65, SUC_lexicon_tags_47 },
    { 0x36efee65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0fee6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71afee6c, SUC_lexicon_tags_15 },
    { 0xeaafee6d, SUC_lexicon_tags_11 },
    { 0x99cfee6c, SUC_lexicon_tags_37 },
    { 0x9bcfee6d, SUC_lexicon_tags_25 },
    { 0x3afee70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef2fee72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418fee76, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb60fee7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7afee7d, SUC_lexicon_tags_15 },
    { 0xb3afee7d, SUC_lexicon_tags_70 },
    { 0x40afee7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbcfee81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fcfee84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84efee86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa4cfee88, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30fee8b, SUC_lexicon_tags_15 },
    { 0x83cfee8c, SUC_lexicon_tags_21 },
    { 0x9aefee8d, SUC_lexicon_tags_28 },
    { 0x8bcfee8c, SUC_lexicon_tags_7 },
    { 0xfa0fee8d, SUC_lexicon_tags_38 },
    { 0x42afee8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdefee92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70fee99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf8fee9b, SUC_lexicon_tags_60 },
    { 0x9e6fee9b, SUC_lexicon_tags_15 },
    { 0xb3afee9b, SUC_lexicon_tags_0 },
    { 0x17afee9d, SUC_lexicon_tags_45 },
    { 0xc5cfee9e, SUC_lexicon_tags_0 },
    { 0x62cfeea0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4cfeea4, SUC_lexicon_tags_2 },
    { 0xd8feea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x100feea7, SUC_lexicon_tags_19 },
    { 0xe0afeea7, SUC_lexicon_tags_8 },
    { 0xcfefeea8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37afeeb0, SUC_lexicon_tags_27 },
    { 0x70cfeeb1, SUC_lexicon_tags_1 },
    { 0x30afeeb1, SUC_lexicon_tags_15 },
    { 0x870feeb0, SUC_lexicon_tags_83 },
    { 0xe40feeb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eafeeba, SUC_lexicon_tags_9 },
    { 0x2a6feebb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x668feebf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4feeca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930feece, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132feed7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x752feed9, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x458feedb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8feede, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22efeee0, SUC_lexicon_tags_26 },
    { 0xa00feee0, SUC_lexicon_tags_17 },
    { 0x49afeee1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74feee6, SUC_lexicon_tags_6 },
    { 0xd3afeee6, SUC_lexicon_tags_15 },
    { 0x2c2feee8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58feeed, SUC_lexicon_tags_56 },
    { 0xb56feeee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcfeef3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x358feef5, SUC_lexicon_tags_19 },
    { 0x3c2feef6, SUC_lexicon_tags_7 },
    { 0xeb0feef5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76feefa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x574feefd, SUC_lexicon_tags_29 },
    { 0xc00feefe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fef01, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12fef05, SUC_lexicon_tags_32 },
    { 0x7e0fef06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58fef0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee6fef0c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2fef0f, SUC_lexicon_tags_17 },
    { 0x13cfef10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x860fef14, SUC_lexicon_tags_33 },
    { 0xc18fef15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4fef17, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce0fef19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6fef24, SUC_lexicon_tags_29 },
    { 0x68fef25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398fef29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850fef31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fef34, SUC_lexicon_tags_55 },
    { 0x2befef35, SUC_lexicon_tags_37 },
    { 0xf50fef36, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2a0fef38, SUC_lexicon_tags_7 },
    { 0xbbafef39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x214fef3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6d8fef3e, SUC_lexicon_tags_68 },
    { 0xd74fef3f, SUC_lexicon_tags_1 },
    { 0x6c2fef3e, SUC_lexicon_tags_34 },
    { 0x896fef3f, SUC_lexicon_tags_15 },
    { 0x1a0fef42, SUC_lexicon_tags_107 },
    { 0xabafef42, SUC_lexicon_tags_25 },
    { 0x614fef44, SUC_lexicon_tags_7 },
    { 0x60efef44, SUC_lexicon_tags_7 },
    { 0x3b4fef44, SUC_lexicon_tags_23 },
    { 0x756fef47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b8fef49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fef4c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4fafef4e, SUC_lexicon_tags_19 },
    { 0x1d6fef4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a6fef51, SUC_lexicon_tags_25 },
    { 0x11cfef51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x700fef55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e0fef58, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6fef60, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572fef64, SUC_lexicon_tags_113 },
    { 0x4fafef65, SUC_lexicon_tags_7 },
    { 0x94cfef66, SUC_lexicon_tags_1 },
    { 0x32efef67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76efef69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xff4fef6c, SUC_lexicon_tags_26 },
    { 0x1a6fef6c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd06fef6f, SUC_lexicon_tags_60 },
    { 0xedcfef6f, SUC_lexicon_tags_25 },
    { 0xb6fef71, SUC_lexicon_tags_27 },
    { 0x6fcfef71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb88fef75, SUC_lexicon_tags_25 },
    { 0xaaafef75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb2fef78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92fef7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d4fef7d, SUC_lexicon_tags_2 },
    { 0x2acfef7e, SUC_lexicon_tags_21 },
    { 0x8cafef7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b4fef81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x20cfef84, SUC_lexicon_tags_0 },
    { 0x84afef84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf02fef88, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x388fef8a, SUC_lexicon_tags_47 },
    { 0xa56fef8b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xcaafef8d, SUC_lexicon_tags_17 },
    { 0x28cfef8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efef93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250fef99, SUC_lexicon_tags_84 },
    { 0x794fef9a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98fef9e, SUC_lexicon_tags_45 },
    { 0x456fef9f, SUC_lexicon_tags_7 },
    { 0xf42fef9f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddcfefaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7defefad, SUC_lexicon_tags_2 },
    { 0x562fefad, SUC_lexicon_tags_68 },
    { 0xdfafefad, SUC_lexicon_tags_25 },
    { 0x7cefefb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69afefb2, SUC_lexicon_tags_37 },
    { 0xf76fefb2, SUC_lexicon_tags_8 },
    { 0x652fefb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe88fefb6, SUC_lexicon_tags_7 },
    { 0x946fefb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefcfefba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0fefc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1cfefc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34fefc7, SUC_lexicon_tags_43 },
    { 0xee0fefc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf38fefca, SUC_lexicon_tags_43 },
    { 0x86cfefcb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x494fefcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x226fefd5, SUC_lexicon_tags_6 },
    { 0x5e4fefd6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf56fefd8, SUC_lexicon_tags_54 },
    { 0x3c2fefd9, SUC_lexicon_tags_1 },
    { 0x67efefda, SUC_lexicon_tags_17 },
    { 0x8b8fefdb, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c2fefe0, SUC_lexicon_tags_0 },
    { 0xedefefe1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cfefe5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23afeff3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654feff9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0feffe, SUC_lexicon_tags_2 },
    { 0x9b2fefff, SUC_lexicon_tags_0 },
    { 0xc62ff000, SUC_lexicon_tags_11 },
    { 0x5f4ff001, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x914ff003, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d6ff005, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x290ff008, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd98ff011, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6caff013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdbcff015, SUC_lexicon_tags_33 },
    { 0x6aeff016, SUC_lexicon_tags_1 },
    { 0xe14ff017, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0ff01a, SUC_lexicon_tags_30 },
    { 0x786ff01a, SUC_lexicon_tags_4 },
    { 0x6f0ff01c, SUC_lexicon_tags_35 },
    { 0x1eeff01d, SUC_lexicon_tags_17 },
    { 0x3d4ff01e, SUC_lexicon_tags_9 },
    { 0xb02ff01d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x732ff022, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf44ff027, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20cff029, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feff030, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdd0ff032, SUC_lexicon_tags_7 },
    { 0x410ff032, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x938ff035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x292ff037, SUC_lexicon_tags_7 },
    { 0x48aff038, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa26ff03a, SUC_lexicon_tags_33 },
    { 0x97cff03a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x30ff03d, SUC_lexicon_tags_13 },
    { 0x5e6ff03e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0ff044, SUC_lexicon_tags_35 },
    { 0xbceff044, SUC_lexicon_tags_42 },
    { 0x310ff046, SUC_lexicon_tags_45 },
    { 0x35aff044, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2ff04b, SUC_lexicon_tags_46 },
    { 0x486ff04c, SUC_lexicon_tags_53 },
    { 0xa00ff04c, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x96eff04f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c6ff052, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x288ff054, SUC_lexicon_tags_14 },
    { 0x880ff054, SUC_lexicon_tags_15 },
    { 0x1a6ff055, SUC_lexicon_tags_37 },
    { 0xa3eff057, SUC_lexicon_tags_14 },
    { 0x63cff058, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5faff05c, SUC_lexicon_tags_2 },
    { 0x3b8ff05c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63cff05f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x214ff061, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x194ff065, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9cff067, SUC_lexicon_tags_113 },
    { 0xcdeff067, SUC_lexicon_tags_1 },
    { 0xc16ff068, SUC_lexicon_tags_17 },
    { 0x882ff069, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf86ff06d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x376ff06f, SUC_lexicon_tags_39 },
    { 0xe1cff070, SUC_lexicon_tags_14 },
    { 0x7e4ff070, SUC_lexicon_tags_25 },
    { 0x3deff071, SUC_lexicon_tags_15 },
    { 0xdfeff070, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7baff075, SUC_lexicon_tags_15 },
    { 0x302ff075, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78eff07a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c0ff08b, SUC_lexicon_tags_6 },
    { 0x608ff08c, SUC_lexicon_tags_19 },
    { 0xf30ff08d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5a2ff08f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15cff095, SUC_lexicon_tags_6 },
    { 0x516ff095, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed4ff09a, SUC_lexicon_tags_26 },
    { 0x5d0ff09a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x776ff09d, SUC_lexicon_tags_208 },
    { 0x418ff09d, SUC_lexicon_tags_47 },
    { 0x784ff09f, SUC_lexicon_tags_11 },
    { 0x96cff0a0, SUC_lexicon_tags_55 },
    { 0xca2ff0a0, SUC_lexicon_tags_3 },
    { 0xf88ff09f, SUC_lexicon_tags_17 },
    { 0x54aff0a0, SUC_lexicon_tags_30 },
    { 0xf64ff0a4, SUC_lexicon_tags_228 },
    { 0x50cff0a5, SUC_lexicon_tags_15 },
    { 0x6a4ff0a6, SUC_lexicon_tags_17 },
    { 0xde2ff0a7, SUC_lexicon_tags_17 },
    { 0x408ff09f, SUC_lexicon_tags_15 },
    { 0x46ff0a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x616ff0ac, SUC_lexicon_tags_1 },
    { 0xc5aff0ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa40ff0af, SUC_lexicon_tags_27 },
    { 0x3f8ff0b0, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4ff0b9, SUC_lexicon_tags_25 },
    { 0xf82ff0b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718ff0c4, SUC_lexicon_tags_21 },
    { 0xe4ff0c5, SUC_lexicon_tags_63 },
    { 0x748ff0c5, SUC_lexicon_tags_25 },
    { 0x15cff0c7, SUC_lexicon_tags_39 },
    { 0x844ff0c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70cff0ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9aff0cc, SUC_lexicon_tags_27 },
    { 0x4ff0cc, SUC_lexicon_tags_37 },
    { 0xdc2ff0ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0ff0d1, SUC_lexicon_tags_65 },
    { 0x17aff0d1, SUC_lexicon_tags_4 },
    { 0xf14ff0d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x850ff0d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x240ff0d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ff0dc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x980ff0df, SUC_lexicon_tags_11 },
    { 0x84aff0e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x724ff0e5, SUC_lexicon_tags_0 },
    { 0xba4ff0e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632ff0ea, SUC_lexicon_tags_7 },
    { 0x500ff0ea, SUC_lexicon_tags_14 },
    { 0xe0eff0ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x382ff0f2, SUC_lexicon_tags_26 },
    { 0x230ff0f3, SUC_lexicon_tags_28 },
    { 0xbbeff0f4, SUC_lexicon_tags_0 },
    { 0xd4cff0f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aff0f8, SUC_lexicon_tags_7 },
    { 0xa92ff0f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x992ff0fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2ff0ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x116ff104, SUC_lexicon_tags_11 },
    { 0xb62ff104, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30ff109, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31eff10b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8cff10e, SUC_lexicon_tags_39 },
    { 0x4e6ff10f, SUC_lexicon_tags_11 },
    { 0xbacff110, SUC_lexicon_tags_46 },
    { 0x7c6ff10f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe58ff114, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2ff117, SUC_lexicon_tags_12 },
    { 0x93aff118, SUC_lexicon_tags_37 },
    { 0x4a0ff119, SUC_lexicon_tags_26 },
    { 0x4caff11a, SUC_lexicon_tags_17 },
    { 0x216ff11b, SUC_lexicon_tags_26 },
    { 0xc86ff11b, SUC_lexicon_tags_11 },
    { 0x372ff11c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x432ff120, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfecff124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x924ff126, SUC_lexicon_tags_16 },
    { 0xe78ff127, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36cff12e, SUC_lexicon_tags_15 },
    { 0xe9aff12e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44eff131, SUC_lexicon_tags_15 },
    { 0x814ff132, SUC_lexicon_tags_12 },
    { 0x8a8ff133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06ff136, SUC_lexicon_tags_31 },
    { 0x94eff136, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x462ff13f, SUC_lexicon_tags_60 },
    { 0x2d8ff140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3aff142, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x196ff145, SUC_lexicon_tags_60 },
    { 0x180ff146, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92ff149, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x884ff14e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x22ff150, SUC_lexicon_tags_25 },
    { 0xa6cff151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dcff157, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xabeff159, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92ff161, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x658ff164, SUC_lexicon_tags_49 },
    { 0x6fcff164, SUC_lexicon_tags_38 },
    { 0x91aff166, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7acff16c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x9b8ff16e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0eff173, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e4ff176, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e2ff179, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d0ff17b, SUC_lexicon_tags_19 },
    { 0x80ff17b, SUC_lexicon_tags_15 },
    { 0x6b0ff17d, SUC_lexicon_tags_43 },
    { 0xfe0ff17d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd84ff186, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5faff189, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0eff18c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc2ff18e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4aff194, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc80ff196, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x130ff198, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3baff19f, SUC_lexicon_tags_1 },
    { 0x37aff1a0, SUC_lexicon_tags_1 },
    { 0x3c6ff1a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf22ff1a4, SUC_lexicon_tags_11 },
    { 0xcaeff1a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ff1aa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x132ff1ac, SUC_lexicon_tags_1 },
    { 0xb8cff1ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x568ff1b5, SUC_lexicon_tags_92 },
    { 0x560ff1b6, SUC_lexicon_tags_40 },
    { 0x470ff1b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcff1b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256ff1be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x424ff1c5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0eff1d5, SUC_lexicon_tags_2 },
    { 0x106ff1d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x562ff1db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x43aff1de, SUC_lexicon_tags_17 },
    { 0x524ff1df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212ff1e3, SUC_lexicon_tags_19 },
    { 0xeecff1e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5aaff1e6, SUC_lexicon_tags_12 },
    { 0x1c4ff1e7, SUC_lexicon_tags_37 },
    { 0xb94ff1e8, SUC_lexicon_tags_26 },
    { 0xff2ff1e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99cff1eb, SUC_lexicon_tags_63 },
    { 0x7baff1eb, SUC_lexicon_tags_38 },
    { 0xef0ff1ed, SUC_lexicon_tags_35 },
    { 0xad8ff1eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x752ff1f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xba4ff1f3, SUC_lexicon_tags_16 },
    { 0xd90ff1f3, SUC_lexicon_tags_27 },
    { 0xaf4ff1f5, SUC_lexicon_tags_37 },
    { 0xe9cff1f6, SUC_lexicon_tags_31 },
    { 0x62ff1f6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x204ff1fa, SUC_lexicon_tags_47 },
    { 0x52eff1fb, SUC_lexicon_tags_17 },
    { 0x250ff1fb, SUC_lexicon_tags_15 },
    { 0x9d0ff1fa, SUC_lexicon_tags_0 },
    { 0x1f6ff1fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x368ff200, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78cff205, SUC_lexicon_tags_7 },
    { 0xf3aff205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x612ff208, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x450ff20a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54cff20c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23aff20e, SUC_lexicon_tags_49 },
    { 0x9b0ff20f, SUC_lexicon_tags_47 },
    { 0x252ff210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4ff213, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aff217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c2ff219, SUC_lexicon_tags_25 },
    { 0x7b0ff21a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x706ff21e, SUC_lexicon_tags_33 },
    { 0xdeff21f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x66cff221, SUC_lexicon_tags_5 },
    { 0xfaaff222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a0ff22a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6ff22e, SUC_lexicon_tags_6 },
    { 0xfecff22f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1acff233, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4ff237, SUC_lexicon_tags_79 },
    { 0xfbcff238, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a2ff23a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8cff23e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2ff243, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ff24b, SUC_lexicon_tags_2 },
    { 0xfaaff24c, SUC_lexicon_tags_17 },
    { 0xcf4ff24c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21aff250, SUC_lexicon_tags_201 },
    { 0xc70ff250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ff255, SUC_lexicon_tags_11 },
    { 0xdd0ff255, SUC_lexicon_tags_13 },
    { 0x68eff256, SUC_lexicon_tags_12 },
    { 0x578ff258, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb6cff25a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714ff25e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe88ff261, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2eff264, SUC_lexicon_tags_68 },
    { 0xdbaff265, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x468ff267, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x620ff269, SUC_lexicon_tags_9 },
    { 0x684ff269, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeaff270, SUC_lexicon_tags_37 },
    { 0xc88ff270, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x766ff273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91aff279, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x77aff27b, SUC_lexicon_tags_15 },
    { 0x2d0ff27c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2ff281, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xce0ff284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa6ff288, SUC_lexicon_tags_498 },
    { 0, NULL },
    { 0x8cff28a, SUC_lexicon_tags_25 },
    { 0x6aff28b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9caff290, SUC_lexicon_tags_82 },
    { 0x54eff291, SUC_lexicon_tags_126 },
    { 0x3f8ff292, SUC_lexicon_tags_1 },
    { 0xd5cff293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c8ff29a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ff29e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6eff2a3, SUC_lexicon_tags_11 },
    { 0xa8aff2a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ff2aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91aff2b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31aff2b7, SUC_lexicon_tags_25 },
    { 0x9b4ff2b7, SUC_lexicon_tags_26 },
    { 0x856ff2b9, SUC_lexicon_tags_48 },
    { 0x3deff2b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x192ff2bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddcff2be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9aff2c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65eff2ca, SUC_lexicon_tags_17 },
    { 0xedaff2ca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x49cff2ce, SUC_lexicon_tags_11 },
    { 0xbd0ff2ce, SUC_lexicon_tags_25 },
    { 0x97aff2d0, SUC_lexicon_tags_127 },
    { 0xe40ff2d1, SUC_lexicon_tags_17 },
    { 0x48aff2d2, SUC_lexicon_tags_16 },
    { 0xcb0ff2cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc62ff2d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x962ff2da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x262ff2dd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4eff2e1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14ff2e5, SUC_lexicon_tags_17 },
    { 0xb2cff2e6, SUC_lexicon_tags_15 },
    { 0xe20ff2e7, SUC_lexicon_tags_7 },
    { 0x5bcff2e7, SUC_lexicon_tags_25 },
    { 0x33aff2e9, SUC_lexicon_tags_33 },
    { 0x7deff2ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x302ff2f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48ff2f5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x322ff2f9, SUC_lexicon_tags_43 },
    { 0xe0ff2f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x60cff2fd, SUC_lexicon_tags_11 },
    { 0xfa2ff2fe, SUC_lexicon_tags_7 },
    { 0xc60ff2fe, SUC_lexicon_tags_15 },
    { 0x330ff300, SUC_lexicon_tags_37 },
    { 0x2a8ff301, SUC_lexicon_tags_14 },
    { 0xac4ff300, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aeff308, SUC_lexicon_tags_25 },
    { 0xd0ff309, SUC_lexicon_tags_77 },
    { 0x5b6ff30a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x842ff30d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x878ff30f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eff314, SUC_lexicon_tags_37 },
    { 0x144ff314, SUC_lexicon_tags_31 },
    { 0xff4ff315, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c4ff319, SUC_lexicon_tags_9 },
    { 0x852ff31a, SUC_lexicon_tags_60 },
    { 0x454ff31b, SUC_lexicon_tags_6 },
    { 0x938ff31c, SUC_lexicon_tags_2 },
    { 0xff4ff31d, SUC_lexicon_tags_45 },
    { 0xc2eff31e, SUC_lexicon_tags_25 },
    { 0xc32ff31d, SUC_lexicon_tags_45 },
    { 0x590ff31d, SUC_lexicon_tags_45 },
    { 0x854ff321, SUC_lexicon_tags_9 },
    { 0x4eeff322, SUC_lexicon_tags_39 },
    { 0x586ff322, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdecff325, SUC_lexicon_tags_33 },
    { 0xd5eff326, SUC_lexicon_tags_63 },
    { 0x7e8ff327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x828ff32a, SUC_lexicon_tags_17 },
    { 0xc8aff32b, SUC_lexicon_tags_17 },
    { 0x708ff32c, SUC_lexicon_tags_37 },
    { 0x812ff32d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ff331, SUC_lexicon_tags_17 },
    { 0xb24ff331, SUC_lexicon_tags_1 },
    { 0x112ff333, SUC_lexicon_tags_7 },
    { 0xa0cff334, SUC_lexicon_tags_12 },
    { 0xcaaff335, SUC_lexicon_tags_60 },
    { 0xe92ff336, SUC_lexicon_tags_43 },
    { 0x80eff337, SUC_lexicon_tags_12 },
    { 0xabcff338, SUC_lexicon_tags_63 },
    { 0x7cff339, SUC_lexicon_tags_39 },
    { 0xec2ff33a, SUC_lexicon_tags_197 },
    { 0x912ff33b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc4ff341, SUC_lexicon_tags_37 },
    { 0x85eff342, SUC_lexicon_tags_39 },
    { 0xb4ff343, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20aff346, SUC_lexicon_tags_11 },
    { 0xadeff346, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc70ff349, SUC_lexicon_tags_17 },
    { 0xae2ff34a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x372ff34d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa3eff34f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc30ff351, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70ff356, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf80ff359, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23aff35d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x404ff368, SUC_lexicon_tags_15 },
    { 0x2f2ff369, SUC_lexicon_tags_27 },
    { 0xb88ff36a, SUC_lexicon_tags_5 },
    { 0xd40ff36a, SUC_lexicon_tags_17 },
    { 0xc8eff36b, SUC_lexicon_tags_68 },
    { 0xf4cff36b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872ff371, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab8ff373, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4eff375, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ff37c, SUC_lexicon_tags_17 },
    { 0x5eff37d, SUC_lexicon_tags_60 },
    { 0x92aff37d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xca0ff380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacaff382, SUC_lexicon_tags_25 },
    { 0x13aff383, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x932ff387, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4cff389, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x698ff38b, SUC_lexicon_tags_17 },
    { 0xdc8ff38c, SUC_lexicon_tags_69 },
    { 0x75aff38c, SUC_lexicon_tags_17 },
    { 0xbd0ff38b, SUC_lexicon_tags_12 },
    { 0x19aff38d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc08ff394, SUC_lexicon_tags_17 },
    { 0x7ecff395, SUC_lexicon_tags_37 },
    { 0xd8ff394, SUC_lexicon_tags_1 },
    { 0x4b0ff394, SUC_lexicon_tags_7 },
    { 0xdf2ff395, SUC_lexicon_tags_151 },
    { 0x600ff399, SUC_lexicon_tags_25 },
    { 0xb26ff39a, SUC_lexicon_tags_14 },
    { 0x9baff39a, SUC_lexicon_tags_60 },
    { 0x970ff395, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x448ff3a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf2ff3a4, SUC_lexicon_tags_21 },
    { 0x6c6ff3a5, SUC_lexicon_tags_87 },
    { 0xdaff3a5, SUC_lexicon_tags_2 },
    { 0x198ff3a7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cff3ac, SUC_lexicon_tags_32 },
    { 0xd7eff3ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd48ff3b2, SUC_lexicon_tags_23 },
    { 0xd9eff3b3, SUC_lexicon_tags_29 },
    { 0x9dcff3b4, SUC_lexicon_tags_15 },
    { 0x65eff3b5, SUC_lexicon_tags_37 },
    { 0x91cff3b6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x39eff3b8, SUC_lexicon_tags_6 },
    { 0x5c2ff3b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e4ff3bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x576ff3be, SUC_lexicon_tags_17 },
    { 0x848ff3bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5aff3c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52eff3c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65eff3c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcff3d1, SUC_lexicon_tags_48 },
    { 0xb2aff3d1, SUC_lexicon_tags_2 },
    { 0x960ff3d2, SUC_lexicon_tags_146 },
    { 0x3faff3d4, SUC_lexicon_tags_1 },
    { 0xadaff3d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb20ff3d8, SUC_lexicon_tags_64 },
    { 0x8caff3d9, SUC_lexicon_tags_19 },
    { 0xdb6ff3d8, SUC_lexicon_tags_23 },
    { 0x300ff3da, SUC_lexicon_tags_9 },
    { 0x44eff3dc, SUC_lexicon_tags_2 },
    { 0x76eff3dd, SUC_lexicon_tags_35 },
    { 0x326ff3dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb9aff3e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc78ff3e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a8ff3e4, SUC_lexicon_tags_19 },
    { 0xf56ff3e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe52ff3e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45cff3eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7aff3ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd40ff3f0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb24ff3f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3deff3f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8eaff3fb, SUC_lexicon_tags_30 },
    { 0x2c0ff3fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170ff403, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x11aff405, SUC_lexicon_tags_15 },
    { 0x6f4ff406, SUC_lexicon_tags_15 },
    { 0x206ff407, SUC_lexicon_tags_1 },
    { 0x96eff405, SUC_lexicon_tags_37 },
    { 0x38cff409, SUC_lexicon_tags_25 },
    { 0xee0ff409, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x778ff40c, SUC_lexicon_tags_28 },
    { 0xc9cff40c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4ff413, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x42aff416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbacff418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26eff41d, SUC_lexicon_tags_47 },
    { 0xbf2ff41e, SUC_lexicon_tags_29 },
    { 0x704ff41e, SUC_lexicon_tags_16 },
    { 0xce2ff41d, SUC_lexicon_tags_72 },
    { 0x9c6ff41d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda4ff425, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0ff428, SUC_lexicon_tags_11 },
    { 0xd50ff429, SUC_lexicon_tags_25 },
    { 0x8feff42a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8dcff42c, SUC_lexicon_tags_7 },
    { 0x7c0ff42d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x854ff430, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x926ff432, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836ff43f, SUC_lexicon_tags_7 },
    { 0xc98ff440, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2eff442, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cff447, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87aff449, SUC_lexicon_tags_17 },
    { 0x28cff44a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfa6ff44c, SUC_lexicon_tags_0 },
    { 0x46eff44d, SUC_lexicon_tags_45 },
    { 0x90ff44e, SUC_lexicon_tags_148 },
    { 0x984ff44d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa0ff454, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafcff459, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x126ff45c, SUC_lexicon_tags_47 },
    { 0x61eff45d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2ecff45f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd90ff46a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45cff46d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x532ff474, SUC_lexicon_tags_12 },
    { 0x27cff475, SUC_lexicon_tags_29 },
    { 0x9aff476, SUC_lexicon_tags_1 },
    { 0x38cff474, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x206ff47e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83cff483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad4ff485, SUC_lexicon_tags_33 },
    { 0xd62ff486, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x746ff488, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a6ff48b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x546ff48f, SUC_lexicon_tags_4 },
    { 0x950ff490, SUC_lexicon_tags_11 },
    { 0x65cff48f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb4ff493, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950ff498, SUC_lexicon_tags_31 },
    { 0x1e4ff499, SUC_lexicon_tags_8 },
    { 0xd26ff49a, SUC_lexicon_tags_33 },
    { 0xa3eff49b, SUC_lexicon_tags_11 },
    { 0xd1cff499, SUC_lexicon_tags_30 },
    { 0x29aff49d, SUC_lexicon_tags_45 },
    { 0xb3aff49b, SUC_lexicon_tags_11 },
    { 0x2eaff49f, SUC_lexicon_tags_24 },
    { 0xf4eff4a0, SUC_lexicon_tags_40 },
    { 0xa34ff4a0, SUC_lexicon_tags_33 },
    { 0x466ff49c, SUC_lexicon_tags_47 },
    { 0x38aff4a2, SUC_lexicon_tags_9 },
    { 0xf00ff49f, SUC_lexicon_tags_15 },
    { 0x488ff4a5, SUC_lexicon_tags_1 },
    { 0x14ff4a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3eff4a9, SUC_lexicon_tags_11 },
    { 0xca8ff4a9, SUC_lexicon_tags_17 },
    { 0xd1aff4ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x302ff4ae, SUC_lexicon_tags_23 },
    { 0x672ff4ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8ff4b4, SUC_lexicon_tags_17 },
    { 0x1c4ff4b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x18cff4b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4ff4ba, SUC_lexicon_tags_27 },
    { 0x9a0ff4bb, SUC_lexicon_tags_1 },
    { 0x50ff4bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8ff4c0, SUC_lexicon_tags_17 },
    { 0xc40ff4c1, SUC_lexicon_tags_10 },
    { 0xb0cff4c2, SUC_lexicon_tags_17 },
    { 0xcaff4c3, SUC_lexicon_tags_5 },
    { 0x25eff4c3, SUC_lexicon_tags_27 },
    { 0xd52ff4c1, SUC_lexicon_tags_60 },
    { 0xb4aff4c0, SUC_lexicon_tags_12 },
    { 0xd26ff4c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe00ff4c9, SUC_lexicon_tags_15 },
    { 0x22aff4c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c8ff4ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eff4d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2ccff4d9, SUC_lexicon_tags_25 },
    { 0x3f4ff4d9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf9eff4dc, SUC_lexicon_tags_1 },
    { 0x18eff4dc, SUC_lexicon_tags_7 },
    { 0xe0aff4dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23eff4e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36cff4e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca0ff4e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5caff4e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4cff4ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9feff4ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6bcff4f0, SUC_lexicon_tags_14 },
    { 0xd36ff4f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3faff4f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf7eff4f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac4ff4f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef8ff4fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34cff4ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x56aff501, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34ff509, SUC_lexicon_tags_37 },
    { 0x8a6ff50a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994ff50f, SUC_lexicon_tags_47 },
    { 0xd96ff510, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf98ff512, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e8ff517, SUC_lexicon_tags_3 },
    { 0xcc4ff518, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4ff51c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50ff523, SUC_lexicon_tags_37 },
    { 0xeb6ff524, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e6ff52b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb86ff52e, SUC_lexicon_tags_9 },
    { 0xf66ff52f, SUC_lexicon_tags_55 },
    { 0xf6ff52e, SUC_lexicon_tags_42 },
    { 0xb8aff52f, SUC_lexicon_tags_2 },
    { 0xbfeff532, SUC_lexicon_tags_26 },
    { 0x29aff530, SUC_lexicon_tags_34 },
    { 0x458ff534, SUC_lexicon_tags_82 },
    { 0x8b6ff532, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77aff53d, SUC_lexicon_tags_68 },
    { 0x210ff53e, SUC_lexicon_tags_1 },
    { 0x8dcff53f, SUC_lexicon_tags_43 },
    { 0x6b8ff53d, SUC_lexicon_tags_70 },
    { 0x284ff53d, SUC_lexicon_tags_15 },
    { 0xb0aff53d, SUC_lexicon_tags_25 },
    { 0x130ff53e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2ff546, SUC_lexicon_tags_17 },
    { 0x75aff546, SUC_lexicon_tags_0 },
    { 0x9f2ff546, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c4ff54f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x588ff553, SUC_lexicon_tags_23 },
    { 0x8eff554, SUC_lexicon_tags_49 },
    { 0x76cff555, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf68ff558, SUC_lexicon_tags_26 },
    { 0xcceff558, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aaff55d, SUC_lexicon_tags_35 },
    { 0x5d2ff55e, SUC_lexicon_tags_1 },
    { 0xa34ff55f, SUC_lexicon_tags_11 },
    { 0x2c4ff55f, SUC_lexicon_tags_69 },
    { 0x2f2ff55f, SUC_lexicon_tags_45 },
    { 0xa2ff562, SUC_lexicon_tags_33 },
    { 0x582ff55f, SUC_lexicon_tags_12 },
    { 0xbbeff560, SUC_lexicon_tags_17 },
    { 0x4f8ff564, SUC_lexicon_tags_2 },
    { 0x962ff562, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xce4ff568, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe0ff56b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa54ff56d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cff573, SUC_lexicon_tags_17 },
    { 0xa6aff574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6cff578, SUC_lexicon_tags_47 },
    { 0x8e2ff579, SUC_lexicon_tags_1 },
    { 0x4feff578, SUC_lexicon_tags_35 },
    { 0xdeaff57b, SUC_lexicon_tags_48 },
    { 0xe50ff57c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaa2ff57e, SUC_lexicon_tags_11 },
    { 0xc12ff57f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4cff581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ff584, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c8ff586, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8cff588, SUC_lexicon_tags_63 },
    { 0x3bcff588, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6ccff58b, SUC_lexicon_tags_60 },
    { 0xa58ff58c, SUC_lexicon_tags_6 },
    { 0x76aff58d, SUC_lexicon_tags_151 },
    { 0x708ff58e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x270ff590, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd90ff592, SUC_lexicon_tags_9 },
    { 0x552ff592, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd86ff595, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x754ff597, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb48ff59b, SUC_lexicon_tags_15 },
    { 0x2e4ff59c, SUC_lexicon_tags_1 },
    { 0x60cff59b, SUC_lexicon_tags_25 },
    { 0x952ff59c, SUC_lexicon_tags_38 },
    { 0x596ff59c, SUC_lexicon_tags_45 },
    { 0xc14ff5a0, SUC_lexicon_tags_0 },
    { 0xa8cff5a1, SUC_lexicon_tags_33 },
    { 0x44aff5a2, SUC_lexicon_tags_29 },
    { 0x56aff5a0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ff5a6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x658ff5a8, SUC_lexicon_tags_26 },
    { 0x22cff5a9, SUC_lexicon_tags_11 },
    { 0xa96ff5aa, SUC_lexicon_tags_60 },
    { 0xfa6ff5a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c8ff5b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa74ff5b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e6ff5b8, SUC_lexicon_tags_3 },
    { 0x44ff5b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8ff5be, SUC_lexicon_tags_25 },
    { 0x3e4ff5be, SUC_lexicon_tags_30 },
    { 0xd36ff5bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906ff5c6, SUC_lexicon_tags_29 },
    { 0xdf0ff5c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe2ff5c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x900ff5cb, SUC_lexicon_tags_68 },
    { 0x3feff5cb, SUC_lexicon_tags_26 },
    { 0xd36ff5cb, SUC_lexicon_tags_151 },
    { 0xbe4ff5cb, SUC_lexicon_tags_29 },
    { 0xc64ff5cf, SUC_lexicon_tags_28 },
    { 0xd9aff5ce, SUC_lexicon_tags_5 },
    { 0x8beff5cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x600ff5d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44cff5d5, SUC_lexicon_tags_27 },
    { 0xf32ff5d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x122ff5da, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46ff5e3, SUC_lexicon_tags_13 },
    { 0xb66ff5e4, SUC_lexicon_tags_108 },
    { 0x16cff5e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a8ff5e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6ff5ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa42ff5f5, SUC_lexicon_tags_17 },
    { 0x7b0ff5f5, SUC_lexicon_tags_87 },
    { 0x468ff5f7, SUC_lexicon_tags_25 },
    { 0xaeff5f8, SUC_lexicon_tags_39 },
    { 0xe54ff5f9, SUC_lexicon_tags_35 },
    { 0x970ff5fa, SUC_lexicon_tags_8 },
    { 0x91cff5fa, SUC_lexicon_tags_87 },
    { 0x8acff5fc, SUC_lexicon_tags_17 },
    { 0x6ccff5fd, SUC_lexicon_tags_11 },
    { 0x29aff5fe, SUC_lexicon_tags_21 },
    { 0x3fcff5f7, SUC_lexicon_tags_10 },
    { 0x5e2ff5f6, SUC_lexicon_tags_39 },
    { 0xcd4ff5fc, SUC_lexicon_tags_17 },
    { 0xf1aff602, SUC_lexicon_tags_6 },
    { 0x77eff603, SUC_lexicon_tags_65 },
    { 0x702ff604, SUC_lexicon_tags_14 },
    { 0xb6ff603, SUC_lexicon_tags_33 },
    { 0x848ff604, SUC_lexicon_tags_63 },
    { 0xbc6ff607, SUC_lexicon_tags_25 },
    { 0xaccff604, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdeff60a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aff614, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x824ff617, SUC_lexicon_tags_1 },
    { 0x786ff617, SUC_lexicon_tags_29 },
    { 0xd9eff618, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x72cff61b, SUC_lexicon_tags_1 },
    { 0xf76ff61c, SUC_lexicon_tags_26 },
    { 0x38eff61d, SUC_lexicon_tags_15 },
    { 0x33aff61d, SUC_lexicon_tags_12 },
    { 0xd02ff61e, SUC_lexicon_tags_2 },
    { 0x63eff620, SUC_lexicon_tags_25 },
    { 0x3e6ff620, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xceaff623, SUC_lexicon_tags_26 },
    { 0x2f8ff624, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbd0ff626, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aff629, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f0ff62b, SUC_lexicon_tags_47 },
    { 0xabcff62c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfacff62f, SUC_lexicon_tags_107 },
    { 0xd66ff630, SUC_lexicon_tags_1 },
    { 0x85eff630, SUC_lexicon_tags_43 },
    { 0xfd8ff632, SUC_lexicon_tags_1 },
    { 0xb24ff632, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77aff637, SUC_lexicon_tags_37 },
    { 0x102ff638, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0ff640, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a8ff648, SUC_lexicon_tags_11 },
    { 0xbecff649, SUC_lexicon_tags_17 },
    { 0xafcff64a, SUC_lexicon_tags_47 },
    { 0x716ff648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49aff64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678ff651, SUC_lexicon_tags_118 },
    { 0xf08ff652, SUC_lexicon_tags_15 },
    { 0xc3cff652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x686ff65a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ff65d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aaff665, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bcff667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x940ff669, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x558ff66b, SUC_lexicon_tags_12 },
    { 0xcdaff66c, SUC_lexicon_tags_27 },
    { 0x656ff66c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x548ff66f, SUC_lexicon_tags_17 },
    { 0x52eff670, SUC_lexicon_tags_17 },
    { 0xad0ff66f, SUC_lexicon_tags_77 },
    { 0x8cff672, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3aff674, SUC_lexicon_tags_25 },
    { 0x9aaff675, SUC_lexicon_tags_9 },
    { 0x966ff675, SUC_lexicon_tags_17 },
    { 0xa5aff677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x952ff679, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeeff67d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2ff680, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x674ff685, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa16ff687, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2aff689, SUC_lexicon_tags_15 },
    { 0x568ff68a, SUC_lexicon_tags_23 },
    { 0x71cff68b, SUC_lexicon_tags_12 },
    { 0x256ff68c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc62ff68f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8ff693, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x586ff695, SUC_lexicon_tags_46 },
    { 0xc0ff695, SUC_lexicon_tags_12 },
    { 0x362ff696, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae2ff69b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2ff69e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcff6a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x826ff6a7, SUC_lexicon_tags_17 },
    { 0x808ff6a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa44ff6ab, SUC_lexicon_tags_19 },
    { 0x4aff6ac, SUC_lexicon_tags_47 },
    { 0x47cff6ad, SUC_lexicon_tags_37 },
    { 0x6aff6ad, SUC_lexicon_tags_11 },
    { 0x3aff6af, SUC_lexicon_tags_27 },
    { 0xc52ff6af, SUC_lexicon_tags_11 },
    { 0x7aeff6b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd22ff6b4, SUC_lexicon_tags_31 },
    { 0x70eff6b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69eff6ba, SUC_lexicon_tags_65 },
    { 0x1cff6bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68aff6bf, SUC_lexicon_tags_25 },
    { 0xffeff6c0, SUC_lexicon_tags_31 },
    { 0x7b4ff6c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14ff6c3, SUC_lexicon_tags_124 },
    { 0xfbaff6c4, SUC_lexicon_tags_136 },
    { 0xf90ff6c5, SUC_lexicon_tags_6 },
    { 0x20ff6c5, SUC_lexicon_tags_68 },
    { 0xa3aff6c7, SUC_lexicon_tags_8 },
    { 0x21cff6c8, SUC_lexicon_tags_25 },
    { 0x638ff6c8, SUC_lexicon_tags_2 },
    { 0xe2aff6c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xec8ff6cd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2eff6d1, SUC_lexicon_tags_1 },
    { 0xf54ff6d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x88eff6d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6aff6d7, SUC_lexicon_tags_30 },
    { 0x8c0ff6d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cff6df, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x970ff6e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x32ff6e3, SUC_lexicon_tags_68 },
    { 0x976ff6e4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x68eff6e6, SUC_lexicon_tags_28 },
    { 0xbf4ff6e6, SUC_lexicon_tags_27 },
    { 0x802ff6e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xed8ff6ea, SUC_lexicon_tags_41 },
    { 0xde4ff6eb, SUC_lexicon_tags_39 },
    { 0xff4ff6ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x600ff6f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcff6f6, SUC_lexicon_tags_34 },
    { 0xdf8ff6f6, SUC_lexicon_tags_12 },
    { 0x360ff6f6, SUC_lexicon_tags_2 },
    { 0xb8cff6f8, SUC_lexicon_tags_17 },
    { 0x5baff6fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x832ff6fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x504ff702, SUC_lexicon_tags_11 },
    { 0xa84ff703, SUC_lexicon_tags_2 },
    { 0x2aaff703, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xff6ff707, SUC_lexicon_tags_15 },
    { 0xc42ff708, SUC_lexicon_tags_30 },
    { 0xbaeff709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3cff70b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2beff712, SUC_lexicon_tags_7 },
    { 0xf4eff713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46aff716, SUC_lexicon_tags_96 },
    { 0x986ff717, SUC_lexicon_tags_479 },
    { 0x926ff717, SUC_lexicon_tags_47 },
    { 0xf66ff719, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6beff71f, SUC_lexicon_tags_68 },
    { 0xddcff720, SUC_lexicon_tags_45 },
    { 0xdb6ff720, SUC_lexicon_tags_15 },
    { 0x406ff722, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf64ff72e, SUC_lexicon_tags_8 },
    { 0xaa2ff72f, SUC_lexicon_tags_14 },
    { 0x498ff730, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816ff734, SUC_lexicon_tags_11 },
    { 0x59aff735, SUC_lexicon_tags_15 },
    { 0x83eff736, SUC_lexicon_tags_31 },
    { 0x162ff737, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0cff73a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc4ff73e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x430ff740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86cff745, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaeff74b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb66ff752, SUC_lexicon_tags_23 },
    { 0x948ff753, SUC_lexicon_tags_17 },
    { 0xcb2ff753, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x166ff756, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x272ff758, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51aff75c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41eff767, SUC_lexicon_tags_25 },
    { 0xdb6ff768, SUC_lexicon_tags_31 },
    { 0x4f0ff767, SUC_lexicon_tags_17 },
    { 0xd62ff76a, SUC_lexicon_tags_2 },
    { 0xccaff767, SUC_lexicon_tags_37 },
    { 0x1eaff767, SUC_lexicon_tags_7 },
    { 0xee8ff76c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c6ff76f, SUC_lexicon_tags_37 },
    { 0x486ff770, SUC_lexicon_tags_12 },
    { 0xff8ff770, SUC_lexicon_tags_39 },
    { 0x546ff771, SUC_lexicon_tags_21 },
    { 0x37cff76f, SUC_lexicon_tags_17 },
    { 0xab2ff774, SUC_lexicon_tags_53 },
    { 0xdaff775, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac2ff77b, SUC_lexicon_tags_14 },
    { 0xfccff77b, SUC_lexicon_tags_37 },
    { 0x8e2ff77d, SUC_lexicon_tags_31 },
    { 0xb9cff77d, SUC_lexicon_tags_15 },
    { 0x23cff77f, SUC_lexicon_tags_93 },
    { 0xc00ff77c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554ff786, SUC_lexicon_tags_2 },
    { 0x236ff787, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ff78c, SUC_lexicon_tags_6 },
    { 0xf54ff78d, SUC_lexicon_tags_11 },
    { 0x65aff78e, SUC_lexicon_tags_47 },
    { 0xbaaff78f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e0ff793, SUC_lexicon_tags_25 },
    { 0x4c6ff794, SUC_lexicon_tags_87 },
    { 0xa00ff794, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91eff79a, SUC_lexicon_tags_11 },
    { 0x1f0ff79b, SUC_lexicon_tags_15 },
    { 0x4ccff79b, SUC_lexicon_tags_12 },
    { 0x634ff79d, SUC_lexicon_tags_2 },
    { 0x67eff79d, SUC_lexicon_tags_60 },
    { 0x7f8ff79f, SUC_lexicon_tags_25 },
    { 0x13cff79f, SUC_lexicon_tags_15 },
    { 0xe7eff7a1, SUC_lexicon_tags_26 },
    { 0x37eff7a2, SUC_lexicon_tags_0 },
    { 0x326ff7a2, SUC_lexicon_tags_88 },
    { 0x4acff7a4, SUC_lexicon_tags_15 },
    { 0x9beff7a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27eff7a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xec4ff7ac, SUC_lexicon_tags_10 },
    { 0x792ff7ad, SUC_lexicon_tags_17 },
    { 0x6e6ff7ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe48ff7b2, SUC_lexicon_tags_10 },
    { 0xee4ff7b3, SUC_lexicon_tags_33 },
    { 0x97aff7b4, SUC_lexicon_tags_60 },
    { 0x62aff7b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8ff7b8, SUC_lexicon_tags_11 },
    { 0x132ff7b9, SUC_lexicon_tags_16 },
    { 0x8deff7ba, SUC_lexicon_tags_15 },
    { 0x83cff7bb, SUC_lexicon_tags_12 },
    { 0x8d6ff7bc, SUC_lexicon_tags_77 },
    { 0x36ff7bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca4ff7c0, SUC_lexicon_tags_14 },
    { 0xa2eff7c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4ff7ca, SUC_lexicon_tags_75 },
    { 0x508ff7cb, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cff7d0, SUC_lexicon_tags_17 },
    { 0xd82ff7d1, SUC_lexicon_tags_33 },
    { 0xcfcff7d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356ff7d7, SUC_lexicon_tags_45 },
    { 0x9ccff7d7, SUC_lexicon_tags_37 },
    { 0x3f4ff7d7, SUC_lexicon_tags_1 },
    { 0x3a6ff7da, SUC_lexicon_tags_15 },
    { 0x188ff7db, SUC_lexicon_tags_19 },
    { 0x676ff7db, SUC_lexicon_tags_36 },
    { 0xd02ff7dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50ff7e3, SUC_lexicon_tags_1 },
    { 0xe90ff7e3, SUC_lexicon_tags_29 },
    { 0xeaff7e4, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x540ff7e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce6ff7e9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x88eff7eb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc84ff7ed, SUC_lexicon_tags_11 },
    { 0x9d8ff7ee, SUC_lexicon_tags_14 },
    { 0x6c4ff7ef, SUC_lexicon_tags_2 },
    { 0xed6ff7f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaff7f4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x912ff7f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x994ff7f9, SUC_lexicon_tags_14 },
    { 0xd1cff7fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58eff7ff, SUC_lexicon_tags_38 },
    { 0x8daff800, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270ff804, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654ff808, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8ff812, SUC_lexicon_tags_23 },
    { 0x326ff812, SUC_lexicon_tags_1 },
    { 0x6eeff813, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeceff819, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70ff81d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddaff81f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcff821, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16ff824, SUC_lexicon_tags_1 },
    { 0x466ff825, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbaff82a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114ff82e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234ff837, SUC_lexicon_tags_15 },
    { 0x860ff838, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64eff83d, SUC_lexicon_tags_26 },
    { 0x4f8ff83e, SUC_lexicon_tags_7 },
    { 0xf0cff83f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ecff841, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe44ff845, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcff84c, SUC_lexicon_tags_79 },
    { 0x2caff84d, SUC_lexicon_tags_7 },
    { 0x2c8ff84e, SUC_lexicon_tags_28 },
    { 0x4f4ff84d, SUC_lexicon_tags_9 },
    { 0xd92ff850, SUC_lexicon_tags_15 },
    { 0xd4ff851, SUC_lexicon_tags_27 },
    { 0x512ff84c, SUC_lexicon_tags_37 },
    { 0xdb2ff84c, SUC_lexicon_tags_47 },
    { 0xa02ff853, SUC_lexicon_tags_14 },
    { 0xfa0ff855, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa64ff859, SUC_lexicon_tags_7 },
    { 0x88eff85a, SUC_lexicon_tags_68 },
    { 0x9a6ff85b, SUC_lexicon_tags_38 },
    { 0xed8ff85b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17aff85e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a8ff861, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3acff868, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaeff870, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234ff876, SUC_lexicon_tags_666 },
    { 0, NULL },
    { 0x3bcff878, SUC_lexicon_tags_12 },
    { 0x20eff879, SUC_lexicon_tags_27 },
    { 0x1a4ff879, SUC_lexicon_tags_15 },
    { 0x54eff87b, SUC_lexicon_tags_17 },
    { 0xbb2ff87b, SUC_lexicon_tags_27 },
    { 0x508ff87d, SUC_lexicon_tags_37 },
    { 0x898ff87e, SUC_lexicon_tags_8 },
    { 0xe00ff87e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x894ff882, SUC_lexicon_tags_1 },
    { 0x7c8ff883, SUC_lexicon_tags_11 },
    { 0xcaff883, SUC_lexicon_tags_33 },
    { 0xef6ff884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74ff888, SUC_lexicon_tags_25 },
    { 0x88ff888, SUC_lexicon_tags_43 },
    { 0xbcff88a, SUC_lexicon_tags_0 },
    { 0xeeeff888, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72ff891, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf40ff896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94ff899, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdcff89b, SUC_lexicon_tags_37 },
    { 0xbdaff89c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x702ff89f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0ff8a5, SUC_lexicon_tags_4 },
    { 0xfeeff8a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x336ff8a8, SUC_lexicon_tags_1 },
    { 0xb66ff8a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3cff8ab, SUC_lexicon_tags_4 },
    { 0xd24ff8ac, SUC_lexicon_tags_37 },
    { 0xcf4ff8ad, SUC_lexicon_tags_48 },
    { 0x890ff8ad, SUC_lexicon_tags_13 },
    { 0x9e8ff8af, SUC_lexicon_tags_12 },
    { 0x1a6ff8ad, SUC_lexicon_tags_25 },
    { 0x2beff8b1, SUC_lexicon_tags_37 },
    { 0xc9eff8b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6ff8b6, SUC_lexicon_tags_33 },
    { 0xb26ff8b6, SUC_lexicon_tags_0 },
    { 0x6cff8b7, SUC_lexicon_tags_7 },
    { 0xb16ff8b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x426ff8bb, SUC_lexicon_tags_63 },
    { 0x1fcff8bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd40ff8bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x284ff8c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cff8c8, SUC_lexicon_tags_47 },
    { 0x10aff8c8, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374ff8ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2feff8d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4ff8d9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x218ff8db, SUC_lexicon_tags_11 },
    { 0xbceff8dc, SUC_lexicon_tags_11 },
    { 0xaf2ff8db, SUC_lexicon_tags_15 },
    { 0x74cff8de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc0ff8e1, SUC_lexicon_tags_14 },
    { 0x3d8ff8e2, SUC_lexicon_tags_29 },
    { 0xcf4ff8e3, SUC_lexicon_tags_1 },
    { 0xca8ff8e1, SUC_lexicon_tags_7 },
    { 0xc64ff8e5, SUC_lexicon_tags_37 },
    { 0xc32ff8e5, SUC_lexicon_tags_5 },
    { 0x650ff8e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16ff8ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x96cff8ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x578ff8f0, SUC_lexicon_tags_8 },
    { 0xd1eff8f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30ff8f3, SUC_lexicon_tags_0 },
    { 0x724ff8f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56ff8f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0ff8fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2feff904, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4ff908, SUC_lexicon_tags_37 },
    { 0xe58ff909, SUC_lexicon_tags_17 },
    { 0x37aff90a, SUC_lexicon_tags_12 },
    { 0xa5aff908, SUC_lexicon_tags_1 },
    { 0x8c0ff90c, SUC_lexicon_tags_77 },
    { 0xbd6ff90d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6ff910, SUC_lexicon_tags_32 },
    { 0x46aff910, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xeeff913, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872ff91a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x348ff91e, SUC_lexicon_tags_35 },
    { 0xd4ff91e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cff92a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8feff92e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5beff933, SUC_lexicon_tags_37 },
    { 0xe62ff933, SUC_lexicon_tags_31 },
    { 0x228ff935, SUC_lexicon_tags_25 },
    { 0x564ff935, SUC_lexicon_tags_12 },
    { 0x27aff937, SUC_lexicon_tags_2 },
    { 0x5acff934, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5daff93b, SUC_lexicon_tags_12 },
    { 0x110ff93c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c6ff93f, SUC_lexicon_tags_6 },
    { 0x704ff940, SUC_lexicon_tags_48 },
    { 0xab2ff941, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x35cff943, SUC_lexicon_tags_27 },
    { 0xb8ff943, SUC_lexicon_tags_6 },
    { 0x444ff945, SUC_lexicon_tags_25 },
    { 0x6d4ff945, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff94d, SUC_lexicon_tags_34 },
    { 0x34cff94d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53aff957, SUC_lexicon_tags_6 },
    { 0xb20ff957, SUC_lexicon_tags_26 },
    { 0x550ff957, SUC_lexicon_tags_25 },
    { 0x288ff95a, SUC_lexicon_tags_33 },
    { 0x57aff958, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43eff963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc0ff969, SUC_lexicon_tags_47 },
    { 0x704ff969, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc42ff96f, SUC_lexicon_tags_17 },
    { 0x974ff970, SUC_lexicon_tags_15 },
    { 0xf1aff971, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0eff973, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8ff976, SUC_lexicon_tags_37 },
    { 0x5acff976, SUC_lexicon_tags_7 },
    { 0x48ff977, SUC_lexicon_tags_4 },
    { 0xc14ff978, SUC_lexicon_tags_14 },
    { 0xbc6ff978, SUC_lexicon_tags_14 },
    { 0xd16ff97b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeff980, SUC_lexicon_tags_15 },
    { 0xe7cff981, SUC_lexicon_tags_21 },
    { 0x160ff981, SUC_lexicon_tags_60 },
    { 0x344ff980, SUC_lexicon_tags_2 },
    { 0x1f0ff983, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234ff98b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x422ff98d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae4ff98f, SUC_lexicon_tags_11 },
    { 0xac2ff98f, SUC_lexicon_tags_54 },
    { 0x5e4ff98f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x370ff996, SUC_lexicon_tags_24 },
    { 0x66ff997, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x424ff99a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x506ff99c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b8ff99e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x394ff9a2, SUC_lexicon_tags_37 },
    { 0xcb2ff9a2, SUC_lexicon_tags_0 },
    { 0xca0ff9a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7daff9a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ff9ab, SUC_lexicon_tags_17 },
    { 0xb5cff9ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4aff9ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaaff9b0, SUC_lexicon_tags_6 },
    { 0x92aff9b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x682ff9b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x160ff9b9, SUC_lexicon_tags_44 },
    { 0x7f0ff9ba, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x384ff9bc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff9bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xda2ff9c1, SUC_lexicon_tags_11 },
    { 0x482ff9c2, SUC_lexicon_tags_46 },
    { 0x182ff9c3, SUC_lexicon_tags_17 },
    { 0x68cff9c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x97eff9c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf00ff9ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7aff9d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5baff9d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1cff9d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cff9dd, SUC_lexicon_tags_33 },
    { 0xdb6ff9de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe46ff9e3, SUC_lexicon_tags_37 },
    { 0x2a8ff9e4, SUC_lexicon_tags_25 },
    { 0x788ff9e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aaff9e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc2ff9ee, SUC_lexicon_tags_37 },
    { 0xbaaff9ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6eff9f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf8ff9f8, SUC_lexicon_tags_47 },
    { 0x6b2ff9f9, SUC_lexicon_tags_48 },
    { 0xd1cff9fa, SUC_lexicon_tags_27 },
    { 0x3aeff9fa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb70ff9fd, SUC_lexicon_tags_39 },
    { 0xe24ff9fe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd6effa00, SUC_lexicon_tags_25 },
    { 0xb4ffa01, SUC_lexicon_tags_11 },
    { 0x5affa02, SUC_lexicon_tags_11 },
    { 0xbcffa03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe8ffa0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x344ffa0f, SUC_lexicon_tags_17 },
    { 0xe4cffa10, SUC_lexicon_tags_37 },
    { 0x828ffa0f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xe28ffa13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb4ffa15, SUC_lexicon_tags_11 },
    { 0x34ffa16, SUC_lexicon_tags_11 },
    { 0xd20ffa17, SUC_lexicon_tags_11 },
    { 0xbaffa16, SUC_lexicon_tags_37 },
    { 0x2e2ffa19, SUC_lexicon_tags_56 },
    { 0xaecffa1a, SUC_lexicon_tags_1 },
    { 0x34affa1a, SUC_lexicon_tags_63 },
    { 0x82ffa1b, SUC_lexicon_tags_11 },
    { 0x7f2ffa1b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6acffa22, SUC_lexicon_tags_16 },
    { 0x8f8ffa22, SUC_lexicon_tags_1 },
    { 0x978ffa24, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb82ffa27, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5effa2d, SUC_lexicon_tags_60 },
    { 0xbecffa2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6ffa31, SUC_lexicon_tags_17 },
    { 0xb4effa32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x18ffa34, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78ffa3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e8ffa41, SUC_lexicon_tags_30 },
    { 0x2beffa42, SUC_lexicon_tags_11 },
    { 0xbceffa42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x642ffa45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf68ffa47, SUC_lexicon_tags_6 },
    { 0xd70ffa47, SUC_lexicon_tags_1 },
    { 0x2b0ffa49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c8ffa4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd00ffa50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98cffa58, SUC_lexicon_tags_55 },
    { 0x108ffa58, SUC_lexicon_tags_15 },
    { 0xb62ffa5a, SUC_lexicon_tags_66 },
    { 0x688ffa5b, SUC_lexicon_tags_49 },
    { 0xf6effa5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaaffa64, SUC_lexicon_tags_43 },
    { 0xd8ffa65, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0ffa68, SUC_lexicon_tags_38 },
    { 0xb66ffa69, SUC_lexicon_tags_9 },
    { 0x47cffa6a, SUC_lexicon_tags_11 },
    { 0x964ffa6b, SUC_lexicon_tags_6 },
    { 0xa84ffa6c, SUC_lexicon_tags_23 },
    { 0xb5effa6d, SUC_lexicon_tags_4 },
    { 0x85cffa6d, SUC_lexicon_tags_12 },
    { 0xa66ffa6f, SUC_lexicon_tags_39 },
    { 0x7ecffa6a, SUC_lexicon_tags_12 },
    { 0x456ffa6b, SUC_lexicon_tags_25 },
    { 0xe72ffa72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x422ffa78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9affa7a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x450ffa7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e2ffa84, SUC_lexicon_tags_31 },
    { 0x3b6ffa84, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf30ffa87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4ffa92, SUC_lexicon_tags_37 },
    { 0x7bcffa93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xde0ffa95, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c6ffa99, SUC_lexicon_tags_27 },
    { 0x5d4ffa9a, SUC_lexicon_tags_49 },
    { 0xae8ffa9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4ffa9e, SUC_lexicon_tags_1 },
    { 0x416ffa9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6ffaa2, SUC_lexicon_tags_37 },
    { 0xe0effaa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfecffaa6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a0ffaa9, SUC_lexicon_tags_0 },
    { 0x186ffaa9, SUC_lexicon_tags_17 },
    { 0x9cffaa9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5e8ffaad, SUC_lexicon_tags_4 },
    { 0xac6ffaad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36ffab1, SUC_lexicon_tags_12 },
    { 0xdb8ffab2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc36ffab5, SUC_lexicon_tags_70 },
    { 0x968ffab5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc10ffab8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1cffabb, SUC_lexicon_tags_12 },
    { 0xdd4ffabb, SUC_lexicon_tags_17 },
    { 0xaa6ffabd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x792ffabf, SUC_lexicon_tags_60 },
    { 0xf86ffac0, SUC_lexicon_tags_37 },
    { 0x3f8ffac1, SUC_lexicon_tags_7 },
    { 0xd7effabf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x394ffac4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdecffacb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca6ffacf, SUC_lexicon_tags_19 },
    { 0x4a2ffad0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc02ffad4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84cffad8, SUC_lexicon_tags_15 },
    { 0xbaffad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffcffadc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x572ffadf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x748ffae3, SUC_lexicon_tags_3 },
    { 0x4b4ffae4, SUC_lexicon_tags_11 },
    { 0x1e4ffae4, SUC_lexicon_tags_64 },
    { 0x8d8ffae6, SUC_lexicon_tags_25 },
    { 0xce0ffae4, SUC_lexicon_tags_16 },
    { 0xc28ffae8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24effaec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372ffaf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd06ffaf6, SUC_lexicon_tags_6 },
    { 0xb0cffaf7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x876ffafa, SUC_lexicon_tags_2 },
    { 0xc5cffafa, SUC_lexicon_tags_25 },
    { 0xca2ffafa, SUC_lexicon_tags_19 },
    { 0x90effafa, SUC_lexicon_tags_1568 },
    { 0, NULL },
    { 0x16ffaff, SUC_lexicon_tags_37 },
    { 0x1a6ffb00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x962ffb04, SUC_lexicon_tags_10 },
    { 0x18ffb05, SUC_lexicon_tags_11 },
    { 0xc6cffb06, SUC_lexicon_tags_1 },
    { 0x614ffb07, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x978ffb0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xceaffb0e, SUC_lexicon_tags_39 },
    { 0xb5effb0f, SUC_lexicon_tags_104 },
    { 0xb40ffb10, SUC_lexicon_tags_15 },
    { 0x500ffb0f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda2ffb1a, SUC_lexicon_tags_1 },
    { 0x9f4ffb1b, SUC_lexicon_tags_10 },
    { 0x73effb1c, SUC_lexicon_tags_25 },
    { 0xba6ffb1d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91cffb22, SUC_lexicon_tags_17 },
    { 0x948ffb23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeceffb29, SUC_lexicon_tags_8 },
    { 0xaa2ffb2a, SUC_lexicon_tags_0 },
    { 0x710ffb2b, SUC_lexicon_tags_64 },
    { 0xd42ffb2c, SUC_lexicon_tags_60 },
    { 0xb16ffb29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x398ffb31, SUC_lexicon_tags_11 },
    { 0xbb8ffb32, SUC_lexicon_tags_0 },
    { 0x14cffb33, SUC_lexicon_tags_28 },
    { 0x5cffb34, SUC_lexicon_tags_16 },
    { 0xbcaffb35, SUC_lexicon_tags_56 },
    { 0xdfcffb31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6cffb39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x78effb3b, SUC_lexicon_tags_54 },
    { 0x73cffb3c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb66ffb3e, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x59cffb40, SUC_lexicon_tags_35 },
    { 0xd8effb40, SUC_lexicon_tags_63 },
    { 0xee2ffb42, SUC_lexicon_tags_1 },
    { 0xb70ffb42, SUC_lexicon_tags_56 },
    { 0x312ffb44, SUC_lexicon_tags_12 },
    { 0x7b0ffb43, SUC_lexicon_tags_14 },
    { 0x7a8ffb45, SUC_lexicon_tags_12 },
    { 0xecffb42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0ffb4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2ffb4e, SUC_lexicon_tags_15 },
    { 0xed2ffb4f, SUC_lexicon_tags_2 },
    { 0xbecffb50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x448ffb52, SUC_lexicon_tags_68 },
    { 0xddaffb52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8ffb55, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x768ffb62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd86ffb69, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10cffb6b, SUC_lexicon_tags_43 },
    { 0x272ffb6c, SUC_lexicon_tags_17 },
    { 0x4affb6d, SUC_lexicon_tags_60 },
    { 0x334ffb6e, SUC_lexicon_tags_8 },
    { 0xf5affb6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x236ffb71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0ffb74, SUC_lexicon_tags_14 },
    { 0xb28ffb74, SUC_lexicon_tags_17 },
    { 0x4bcffb76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ffb7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854ffb8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73affb8f, SUC_lexicon_tags_17 },
    { 0x97affb8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaaffb95, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95effb99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x112ffb9b, SUC_lexicon_tags_37 },
    { 0x48effb9c, SUC_lexicon_tags_55 },
    { 0xe3effb9d, SUC_lexicon_tags_26 },
    { 0xf74ffb9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x61cffba0, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33effba7, SUC_lexicon_tags_38 },
    { 0xdb2ffba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafeffbb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x246ffbb8, SUC_lexicon_tags_37 },
    { 0xd8ffbb8, SUC_lexicon_tags_1 },
    { 0x91affbba, SUC_lexicon_tags_15 },
    { 0x3effbbb, SUC_lexicon_tags_48 },
    { 0x914ffbb9, SUC_lexicon_tags_10 },
    { 0xceffbbd, SUC_lexicon_tags_12 },
    { 0xb8effbbe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef0ffbc9, SUC_lexicon_tags_7 },
    { 0x590ffbc9, SUC_lexicon_tags_13 },
    { 0x8a2ffbcb, SUC_lexicon_tags_9 },
    { 0xa12ffbcc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5caffbd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x922ffbd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28cffbda, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x572ffbdd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76cffbe2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xecaffbe4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faffbee, SUC_lexicon_tags_17 },
    { 0x174ffbef, SUC_lexicon_tags_2 },
    { 0xb1cffbf0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5a4ffbf2, SUC_lexicon_tags_1 },
    { 0x9ceffbf2, SUC_lexicon_tags_38 },
    { 0x59affbf4, SUC_lexicon_tags_37 },
    { 0x2f2ffbf5, SUC_lexicon_tags_33 },
    { 0xe86ffbf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4ffbf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cffbff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee6ffc03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x382ffc05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e6ffc09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2ffc10, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe46ffc12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aaffc15, SUC_lexicon_tags_37 },
    { 0x186ffc15, SUC_lexicon_tags_113 },
    { 0xc0effc17, SUC_lexicon_tags_14 },
    { 0xd18ffc18, SUC_lexicon_tags_17 },
    { 0x9c4ffc19, SUC_lexicon_tags_1 },
    { 0xc5affc18, SUC_lexicon_tags_108 },
    { 0x42cffc1b, SUC_lexicon_tags_26 },
    { 0xca0ffc1c, SUC_lexicon_tags_63 },
    { 0x9beffc1d, SUC_lexicon_tags_27 },
    { 0x86affc1e, SUC_lexicon_tags_31 },
    { 0x5ceffc1e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16cffc2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x996ffc31, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa8ffc34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78ffc38, SUC_lexicon_tags_201 },
    { 0x5a2ffc38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x666ffc3c, SUC_lexicon_tags_14 },
    { 0x420ffc3d, SUC_lexicon_tags_0 },
    { 0x180ffc3c, SUC_lexicon_tags_12 },
    { 0x398ffc3f, SUC_lexicon_tags_1 },
    { 0xea8ffc3f, SUC_lexicon_tags_11 },
    { 0x69affc3f, SUC_lexicon_tags_0 },
    { 0xf00ffc42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96effc4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354ffc50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x14cffc52, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x294ffc55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd0ffc57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ceffc5b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x400ffc5e, SUC_lexicon_tags_11 },
    { 0x4b2ffc5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96affc61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ceffc65, SUC_lexicon_tags_11 },
    { 0xcdeffc66, SUC_lexicon_tags_33 },
    { 0xa74ffc67, SUC_lexicon_tags_2 },
    { 0x5b0ffc68, SUC_lexicon_tags_8 },
    { 0xf34ffc69, SUC_lexicon_tags_60 },
    { 0xd0effc67, SUC_lexicon_tags_57 },
    { 0x4dcffc6b, SUC_lexicon_tags_12 },
    { 0x84cffc66, SUC_lexicon_tags_47 },
    { 0x4b2ffc6d, SUC_lexicon_tags_6 },
    { 0xe42ffc68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78affc74, SUC_lexicon_tags_25 },
    { 0x7fcffc74, SUC_lexicon_tags_87 },
    { 0x31affc75, SUC_lexicon_tags_11 },
    { 0x7b8ffc76, SUC_lexicon_tags_25 },
    { 0xa16ffc78, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4affc7e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4deffc84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9affc88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadcffc8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f8ffc96, SUC_lexicon_tags_37 },
    { 0x840ffc97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x146ffc9a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x484ffc9d, SUC_lexicon_tags_23 },
    { 0xd1cffc9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6affca0, SUC_lexicon_tags_47 },
    { 0x68ffca0, SUC_lexicon_tags_92 },
    { 0x2a8ffca2, SUC_lexicon_tags_11 },
    { 0x8fcffca3, SUC_lexicon_tags_45 },
    { 0x322ffca4, SUC_lexicon_tags_55 },
    { 0x4b0ffca5, SUC_lexicon_tags_37 },
    { 0x7a4ffca6, SUC_lexicon_tags_15 },
    { 0xd36ffca7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774ffcae, SUC_lexicon_tags_79 },
    { 0xa32ffcaf, SUC_lexicon_tags_11 },
    { 0x354ffcaf, SUC_lexicon_tags_37 },
    { 0x45affcb0, SUC_lexicon_tags_9 },
    { 0xf60ffcaf, SUC_lexicon_tags_14 },
    { 0x3baffcb3, SUC_lexicon_tags_17 },
    { 0xf0effcb3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe84ffcb6, SUC_lexicon_tags_42 },
    { 0xf58ffcb7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ecffcbc, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6effcbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x906ffcc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8effcc6, SUC_lexicon_tags_1230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1cffcca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x856ffccc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124ffcd1, SUC_lexicon_tags_2 },
    { 0x3e4ffcd2, SUC_lexicon_tags_7 },
    { 0xd64ffcd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcffcd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4ffcdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf10ffcdd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc4ffcdf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe6ffce5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30ffce8, SUC_lexicon_tags_47 },
    { 0x3ccffce8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2ffcee, SUC_lexicon_tags_26 },
    { 0x2caffcef, SUC_lexicon_tags_38 },
    { 0x7dcffcf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40ffcf4, SUC_lexicon_tags_23 },
    { 0x2b0ffcf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e8ffcf7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa48ffcf9, SUC_lexicon_tags_9 },
    { 0xe6cffcf9, SUC_lexicon_tags_17 },
    { 0xc22ffcfb, SUC_lexicon_tags_45 },
    { 0xbd6ffcfb, SUC_lexicon_tags_8 },
    { 0x1faffcfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592ffd05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x354ffd07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13affd0d, SUC_lexicon_tags_11 },
    { 0xc5cffd0e, SUC_lexicon_tags_31 },
    { 0x93affd0f, SUC_lexicon_tags_12 },
    { 0x27effd0f, SUC_lexicon_tags_12 },
    { 0x2d2ffd0f, SUC_lexicon_tags_11 },
    { 0xcceffd0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x976ffd14, SUC_lexicon_tags_15 },
    { 0xd5affd15, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d6ffd1a, SUC_lexicon_tags_11 },
    { 0x1b0ffd1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aaffd23, SUC_lexicon_tags_19 },
    { 0x616ffd24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2ffd2b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x700ffd30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdacffd36, SUC_lexicon_tags_11 },
    { 0x3fcffd37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cffd3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31effd3e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf50ffd40, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x170ffd44, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xac8ffd46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x788ffd48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0ffd4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e6ffd4e, SUC_lexicon_tags_9 },
    { 0x62affd4f, SUC_lexicon_tags_37 },
    { 0x23cffd4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4affd54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb88ffd5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a6ffd63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4affd66, SUC_lexicon_tags_53 },
    { 0xeccffd67, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0ffd72, SUC_lexicon_tags_63 },
    { 0x16effd73, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0ffd78, SUC_lexicon_tags_7 },
    { 0x378ffd79, SUC_lexicon_tags_7 },
    { 0x382ffd7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974ffd7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ffd83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e4ffd86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x866ffd88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x526ffd8c, SUC_lexicon_tags_359 },
    { 0xe44ffd8d, SUC_lexicon_tags_45 },
    { 0x250ffd8d, SUC_lexicon_tags_15 },
    { 0x46effd8f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4affd93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ecffd98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8ffd9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacaffda5, SUC_lexicon_tags_0 },
    { 0x510ffda5, SUC_lexicon_tags_15 },
    { 0x81cffda7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbb0ffda9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa14ffdac, SUC_lexicon_tags_46 },
    { 0x9aaffdad, SUC_lexicon_tags_165 },
    { 0xd98ffdae, SUC_lexicon_tags_0 },
    { 0xf84ffdaf, SUC_lexicon_tags_7 },
    { 0x7caffdae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x332ffdb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x532ffdb4, SUC_lexicon_tags_15 },
    { 0x96ffdb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0ffdba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdcffdbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccffdbf, SUC_lexicon_tags_10 },
    { 0xe94ffdbf, SUC_lexicon_tags_56 },
    { 0xc10ffdc1, SUC_lexicon_tags_1 },
    { 0x4dcffdc2, SUC_lexicon_tags_12 },
    { 0xc62ffdc3, SUC_lexicon_tags_39 },
    { 0x586ffdc2, SUC_lexicon_tags_2 },
    { 0xf46ffdc5, SUC_lexicon_tags_25 },
    { 0x20cffdc6, SUC_lexicon_tags_12 },
    { 0x9ecffdc7, SUC_lexicon_tags_18 },
    { 0x2eaffdc8, SUC_lexicon_tags_15 },
    { 0xb34ffdbf, SUC_lexicon_tags_68 },
    { 0x466ffdbf, SUC_lexicon_tags_11 },
    { 0x5b0ffdcb, SUC_lexicon_tags_27 },
    { 0x674ffdcc, SUC_lexicon_tags_65 },
    { 0xbd6ffdcb, SUC_lexicon_tags_19 },
    { 0x80effdcd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36ffdd4, SUC_lexicon_tags_1 },
    { 0x9d6ffdd4, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xacaffdd7, SUC_lexicon_tags_7 },
    { 0xceaffdd8, SUC_lexicon_tags_37 },
    { 0x9ccffdd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf30ffddb, SUC_lexicon_tags_9 },
    { 0x560ffddc, SUC_lexicon_tags_29 },
    { 0xd88ffddb, SUC_lexicon_tags_50 },
    { 0xb46ffddb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6affde0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x794ffde2, SUC_lexicon_tags_17 },
    { 0x674ffde2, SUC_lexicon_tags_61 },
    { 0x494ffde4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd6affde6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee8ffde9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2caffdeb, SUC_lexicon_tags_17 },
    { 0x512ffdec, SUC_lexicon_tags_29 },
    { 0xea6ffded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x660ffdf2, SUC_lexicon_tags_11 },
    { 0x7e4ffdf3, SUC_lexicon_tags_12 },
    { 0x11effdf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76ffdfb, SUC_lexicon_tags_11 },
    { 0x416ffdfc, SUC_lexicon_tags_5 },
    { 0x834ffdfd, SUC_lexicon_tags_11 },
    { 0xba6ffdfe, SUC_lexicon_tags_27 },
    { 0xb42ffdfc, SUC_lexicon_tags_43 },
    { 0xe14ffe00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x866ffe0f, SUC_lexicon_tags_14 },
    { 0x6f4ffe10, SUC_lexicon_tags_12 },
    { 0xeecffe10, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x646ffe15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ccffe1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x740ffe20, SUC_lexicon_tags_38 },
    { 0x950ffe20, SUC_lexicon_tags_57 },
    { 0x87cffe22, SUC_lexicon_tags_37 },
    { 0xb86ffe22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe56ffe26, SUC_lexicon_tags_45 },
    { 0x9aeffe27, SUC_lexicon_tags_25 },
    { 0x1b8ffe28, SUC_lexicon_tags_16 },
    { 0x240ffe29, SUC_lexicon_tags_1 },
    { 0xa9cffe2a, SUC_lexicon_tags_37 },
    { 0xffe27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x290ffe2d, SUC_lexicon_tags_7 },
    { 0x70ffe2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8ffe33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8ffe36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86effe38, SUC_lexicon_tags_160 },
    { 0xf94ffe38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xacffe3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x442ffe3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aaffe44, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6ffe47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba8ffe4b, SUC_lexicon_tags_56 },
    { 0x428ffe4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c0ffe4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f6ffe50, SUC_lexicon_tags_11 },
    { 0xcd0ffe51, SUC_lexicon_tags_26 },
    { 0x252ffe52, SUC_lexicon_tags_151 },
    { 0xa4ffe51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c2ffe58, SUC_lexicon_tags_17 },
    { 0x1cffe59, SUC_lexicon_tags_19 },
    { 0x9b2ffe58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfdaffe5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef0ffe5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x298ffe60, SUC_lexicon_tags_17 },
    { 0x442ffe61, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x28effe64, SUC_lexicon_tags_2 },
    { 0x66cffe64, SUC_lexicon_tags_33 },
    { 0xa8cffe66, SUC_lexicon_tags_11 },
    { 0x906ffe66, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4ffe6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe40ffe6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x692ffe71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ffe75, SUC_lexicon_tags_27 },
    { 0x958ffe75, SUC_lexicon_tags_37 },
    { 0x4feffe77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfeffe7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd98ffe7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeeffe81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba6ffe85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb02ffe88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f8ffe8a, SUC_lexicon_tags_27 },
    { 0x89effe8a, SUC_lexicon_tags_37 },
    { 0xb98ffe8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50effe92, SUC_lexicon_tags_208 },
    { 0xa0effe92, SUC_lexicon_tags_1 },
    { 0x9b4ffe92, SUC_lexicon_tags_7 },
    { 0x544ffe93, SUC_lexicon_tags_32 },
    { 0x6b8ffe96, SUC_lexicon_tags_14 },
    { 0x44ffe93, SUC_lexicon_tags_14 },
    { 0x75affe98, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aeffe9f, SUC_lexicon_tags_57 },
    { 0x4c2ffe9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x324ffea2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11effea7, SUC_lexicon_tags_37 },
    { 0x75cffea8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c4ffeab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a8ffead, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc68ffeaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x684ffeb1, SUC_lexicon_tags_87 },
    { 0x23cffeb2, SUC_lexicon_tags_2 },
    { 0xd66ffeb3, SUC_lexicon_tags_37 },
    { 0x56cffeb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376ffebd, SUC_lexicon_tags_15 },
    { 0x3a6ffebe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c4ffec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9caffec4, SUC_lexicon_tags_2 },
    { 0x2bcffec5, SUC_lexicon_tags_10 },
    { 0x7b8ffec4, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x76effec9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x114ffecb, SUC_lexicon_tags_25 },
    { 0xb2affecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aeffecf, SUC_lexicon_tags_37 },
    { 0x778ffed0, SUC_lexicon_tags_26 },
    { 0xc36ffed0, SUC_lexicon_tags_60 },
    { 0x418ffecf, SUC_lexicon_tags_34 },
    { 0x478ffed3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcaaffed5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14ffed8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b8ffee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x886ffee3, SUC_lexicon_tags_19 },
    { 0x146ffee4, SUC_lexicon_tags_87 },
    { 0x38ffee5, SUC_lexicon_tags_27 },
    { 0xc16ffee4, SUC_lexicon_tags_17 },
    { 0x638ffee3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2cffeea, SUC_lexicon_tags_68 },
    { 0x4ffeeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf58ffeed, SUC_lexicon_tags_17 },
    { 0xa46ffeee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4ffef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa80ffef5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7effef9, SUC_lexicon_tags_11 },
    { 0xeaffef9, SUC_lexicon_tags_25 },
    { 0xa88ffefb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc4ffefd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4efff00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa48fff04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2fff0a, SUC_lexicon_tags_1 },
    { 0x13cfff0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb32fff0d, SUC_lexicon_tags_12 },
    { 0xeacfff0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7cfff10, SUC_lexicon_tags_23 },
    { 0xf3cfff11, SUC_lexicon_tags_11 },
    { 0x308fff12, SUC_lexicon_tags_535 },
    { 0x990fff12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x83cfff15, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72fff19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x404fff1b, SUC_lexicon_tags_63 },
    { 0xe62fff1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x376fff1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78fff26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfff2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd6efff30, SUC_lexicon_tags_14 },
    { 0xa8efff31, SUC_lexicon_tags_32 },
    { 0x578fff32, SUC_lexicon_tags_48 },
    { 0x234fff32, SUC_lexicon_tags_6 },
    { 0x2afff34, SUC_lexicon_tags_19 },
    { 0x820fff35, SUC_lexicon_tags_47 },
    { 0x53afff36, SUC_lexicon_tags_2 },
    { 0x5dafff32, SUC_lexicon_tags_37 },
    { 0x972fff31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x596fff3a, SUC_lexicon_tags_7 },
    { 0xf42fff3b, SUC_lexicon_tags_66 },
    { 0x84fff3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6efff40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cfff45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60cfff47, SUC_lexicon_tags_9 },
    { 0x686fff48, SUC_lexicon_tags_25 },
    { 0xa4afff47, SUC_lexicon_tags_14 },
    { 0x720fff49, SUC_lexicon_tags_17 },
    { 0xeaefff4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeaefff4d, SUC_lexicon_tags_92 },
    { 0x3fafff4d, SUC_lexicon_tags_55 },
    { 0x52cfff4d, SUC_lexicon_tags_25 },
    { 0x49cfff50, SUC_lexicon_tags_13 },
    { 0x380fff50, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x830fff53, SUC_lexicon_tags_30 },
    { 0x572fff54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80cfff56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x372fff58, SUC_lexicon_tags_1 },
    { 0x88afff59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1defff5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x296fff60, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80efff65, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xa98fff68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39cfff6f, SUC_lexicon_tags_25 },
    { 0xdf4fff70, SUC_lexicon_tags_35 },
    { 0x50cfff71, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6fff78, SUC_lexicon_tags_23 },
    { 0xf7cfff79, SUC_lexicon_tags_12 },
    { 0xe52fff7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf82fff7e, SUC_lexicon_tags_2 },
    { 0x42cfff7e, SUC_lexicon_tags_12 },
    { 0x4b2fff80, SUC_lexicon_tags_25 },
    { 0x76fff81, SUC_lexicon_tags_17 },
    { 0xa02fff82, SUC_lexicon_tags_39 },
    { 0x64fff83, SUC_lexicon_tags_15 },
    { 0x698fff84, SUC_lexicon_tags_11 },
    { 0x19efff80, SUC_lexicon_tags_0 },
    { 0xc7efff82, SUC_lexicon_tags_42 },
    { 0x6a6fff84, SUC_lexicon_tags_45 },
    { 0x1bcfff7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x426fff8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fff8e, SUC_lexicon_tags_7 },
    { 0xa52fff8f, SUC_lexicon_tags_2 },
    { 0x654fff8f, SUC_lexicon_tags_75 },
    { 0xf02fff91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6fff96, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36fff9a, SUC_lexicon_tags_14 },
    { 0x576fff9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0fff9f, SUC_lexicon_tags_8 },
    { 0xca4fffa0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x27afffa3, SUC_lexicon_tags_2 },
    { 0x634fffa4, SUC_lexicon_tags_14 },
    { 0xf34fffa5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79afffac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbccfffae, SUC_lexicon_tags_7 },
    { 0xf6cfffae, SUC_lexicon_tags_1 },
    { 0x316fffb0, SUC_lexicon_tags_12 },
    { 0xe06fffb1, SUC_lexicon_tags_30 },
    { 0x658fffb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52fffb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d0fffb7, SUC_lexicon_tags_1 },
    { 0x3befffb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eefffbc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8fffc4, SUC_lexicon_tags_17 },
    { 0xc8efffc5, SUC_lexicon_tags_27 },
    { 0xe9cfffc6, SUC_lexicon_tags_17 },
    { 0x1f4fffc7, SUC_lexicon_tags_30 },
    { 0x750fffc5, SUC_lexicon_tags_0 },
    { 0xf90fffc9, SUC_lexicon_tags_37 },
    { 0xb7cfffc9, SUC_lexicon_tags_1 },
    { 0xa60fffcb, SUC_lexicon_tags_2 },
    { 0xc14fffcc, SUC_lexicon_tags_63 },
    { 0xda8fffcc, SUC_lexicon_tags_25 },
    { 0xcafffcd, SUC_lexicon_tags_199 },
    { 0xf0fffce, SUC_lexicon_tags_26 },
    { 0x7e4fffc8, SUC_lexicon_tags_2 },
    { 0x9e6fffc5, SUC_lexicon_tags_31 },
    { 0x9f6fffd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afffd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85afffdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efffde, SUC_lexicon_tags_2 },
    { 0xaf2fffde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaafffe3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6cfffe6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1cfffe9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x326fffee, SUC_lexicon_tags_17 },
    { 0x1efffef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47effff6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x914ffff8, SUC_lexicon_tags_23 },
    { 0x4c2ffff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96affffb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeccffffe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a900003, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9100005, SUC_lexicon_tags_45 },
    { 0x4bd00006, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d700009, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2350000d, SUC_lexicon_tags_42 },
    { 0xc90000e, SUC_lexicon_tags_7 },
    { 0xd30000f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80900016, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc30001c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea300026, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca10002c, SUC_lexicon_tags_25 },
    { 0x6110002c, SUC_lexicon_tags_47 },
    { 0x2cd0002e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41500034, SUC_lexicon_tags_37 },
    { 0x8bf00035, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c300039, SUC_lexicon_tags_14 },
    { 0x4a700039, SUC_lexicon_tags_70 },
    { 0xe4900039, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd00040, SUC_lexicon_tags_38 },
    { 0x52f00041, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed00047, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5a100049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e90004c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x15d0004e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x81100050, SUC_lexicon_tags_61 },
    { 0x57b00051, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10100057, SUC_lexicon_tags_1 },
    { 0x37b00058, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9270005b, SUC_lexicon_tags_37 },
    { 0x3550005c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb470005f, SUC_lexicon_tags_29 },
    { 0xc8f0005f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d900063, SUC_lexicon_tags_2 },
    { 0xa300064, SUC_lexicon_tags_31 },
    { 0x8a500065, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf0006a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b0006e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27500070, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x83100072, SUC_lexicon_tags_11 },
    { 0xe5300073, SUC_lexicon_tags_65 },
    { 0x62d00074, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x88500076, SUC_lexicon_tags_15 },
    { 0x700077, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d0007b, SUC_lexicon_tags_40 },
    { 0x5bd0007b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb00085, SUC_lexicon_tags_15 },
    { 0x42100086, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8300088, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3470008a, SUC_lexicon_tags_12 },
    { 0x470008b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a50008d, SUC_lexicon_tags_9 },
    { 0xa130008e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x19500090, SUC_lexicon_tags_43 },
    { 0x9c300091, SUC_lexicon_tags_12 },
    { 0xb1500092, SUC_lexicon_tags_63 },
    { 0x70900091, SUC_lexicon_tags_45 },
    { 0xc0500094, SUC_lexicon_tags_7 },
    { 0x21700090, SUC_lexicon_tags_43 },
    { 0x95900096, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f900098, SUC_lexicon_tags_15 },
    { 0x5a100098, SUC_lexicon_tags_70 },
    { 0x2630009a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe930009c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2130009f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3000a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d000a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725000ac, SUC_lexicon_tags_85 },
    { 0x417000ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa7b000af, SUC_lexicon_tags_282 },
    { 0xa2f000af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df000b3, SUC_lexicon_tags_31 },
    { 0xaf7000b4, SUC_lexicon_tags_55 },
    { 0x29f000b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47000bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63000c1, SUC_lexicon_tags_47 },
    { 0x131000c2, SUC_lexicon_tags_87 },
    { 0xb77000c3, SUC_lexicon_tags_19 },
    { 0x82f000c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf000c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf000cf, SUC_lexicon_tags_15 },
    { 0x9dd000d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1000d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b000d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f000dc, SUC_lexicon_tags_2 },
    { 0x271000dd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631000e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1000e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59000ea, SUC_lexicon_tags_5 },
    { 0x461000eb, SUC_lexicon_tags_2 },
    { 0x689000ec, SUC_lexicon_tags_17 },
    { 0xcb000ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f000f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399000f7, SUC_lexicon_tags_19 },
    { 0x411000f7, SUC_lexicon_tags_37 },
    { 0xf8d000f7, SUC_lexicon_tags_37 },
    { 0xba5000fa, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x425000fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40700100, SUC_lexicon_tags_15 },
    { 0x74f00100, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x58500104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f00108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccb0010a, SUC_lexicon_tags_61 },
    { 0x1910010b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a70010d, SUC_lexicon_tags_60 },
    { 0xeef0010d, SUC_lexicon_tags_2 },
    { 0xd490010f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5c700111, SUC_lexicon_tags_9 },
    { 0xfcf00111, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc1100114, SUC_lexicon_tags_33 },
    { 0x23b00115, SUC_lexicon_tags_25 },
    { 0x7700116, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x82500118, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7650011a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd0011f, SUC_lexicon_tags_11 },
    { 0x1d90011f, SUC_lexicon_tags_0 },
    { 0xba700120, SUC_lexicon_tags_188 },
    { 0xc3300122, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb00127, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a500129, SUC_lexicon_tags_17 },
    { 0x67d0012a, SUC_lexicon_tags_7 },
    { 0xd170012a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e90012d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x68500130, SUC_lexicon_tags_30 },
    { 0x49d00130, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f00135, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc570013a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6590013c, SUC_lexicon_tags_8 },
    { 0xd5d0013d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xda50013f, SUC_lexicon_tags_25 },
    { 0xef90013f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcef00142, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde100147, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8270014d, SUC_lexicon_tags_11 },
    { 0x80d0014d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ff00150, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7700154, SUC_lexicon_tags_37 },
    { 0x7dd00154, SUC_lexicon_tags_4 },
    { 0x9a300155, SUC_lexicon_tags_42 },
    { 0x44100154, SUC_lexicon_tags_114 },
    { 0xf0700158, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f0015c, SUC_lexicon_tags_1 },
    { 0xf190015d, SUC_lexicon_tags_12 },
    { 0xeb0015c, SUC_lexicon_tags_82 },
    { 0x3ef0015f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f00165, SUC_lexicon_tags_1 },
    { 0x55500166, SUC_lexicon_tags_8 },
    { 0x15700167, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f0016d, SUC_lexicon_tags_2 },
    { 0xf810016e, SUC_lexicon_tags_17 },
    { 0x8ef0016e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b100176, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d00179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe030017b, SUC_lexicon_tags_2 },
    { 0xfaf0017b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b500180, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84d00182, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fd00184, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb00187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27500189, SUC_lexicon_tags_33 },
    { 0x7930018a, SUC_lexicon_tags_2 },
    { 0x4710018b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9750018d, SUC_lexicon_tags_9 },
    { 0x5270018d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62f00190, SUC_lexicon_tags_49 },
    { 0x7e500191, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7eb00193, SUC_lexicon_tags_7 },
    { 0x3cb00194, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29300198, SUC_lexicon_tags_17 },
    { 0x50300199, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b0019e, SUC_lexicon_tags_23 },
    { 0x50d0019f, SUC_lexicon_tags_46 },
    { 0x265001a0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x793001a2, SUC_lexicon_tags_2 },
    { 0x6af001a3, SUC_lexicon_tags_25 },
    { 0x5bd001a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9001a7, SUC_lexicon_tags_54 },
    { 0x8ed001a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x34b001aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9001ad, SUC_lexicon_tags_11 },
    { 0x445001ae, SUC_lexicon_tags_37 },
    { 0xda9001af, SUC_lexicon_tags_44 },
    { 0xbcd001ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9001b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf89001b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe3001b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x909001ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad001bd, SUC_lexicon_tags_7 },
    { 0xf55001bd, SUC_lexicon_tags_15 },
    { 0x1ef001be, SUC_lexicon_tags_6 },
    { 0x4d001be, SUC_lexicon_tags_4 },
    { 0x707001c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd47001c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb001ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b001d0, SUC_lexicon_tags_17 },
    { 0x8fb001d1, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x2ab001d3, SUC_lexicon_tags_25 },
    { 0xc7001d4, SUC_lexicon_tags_7 },
    { 0x41b001d3, SUC_lexicon_tags_15 },
    { 0x8e5001d6, SUC_lexicon_tags_1 },
    { 0x55d001d7, SUC_lexicon_tags_11 },
    { 0x453001d8, SUC_lexicon_tags_26 },
    { 0x7d7001d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ad001db, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xb4d001dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393001e3, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11001e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445001eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243001f0, SUC_lexicon_tags_14 },
    { 0x359001f0, SUC_lexicon_tags_11 },
    { 0x5c3001f1, SUC_lexicon_tags_25 },
    { 0x833001f3, SUC_lexicon_tags_6 },
    { 0x5fd001f4, SUC_lexicon_tags_17 },
    { 0x86b001f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1001ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f00202, SUC_lexicon_tags_29 },
    { 0xe2b00202, SUC_lexicon_tags_17 },
    { 0x42500202, SUC_lexicon_tags_11 },
    { 0xdf500205, SUC_lexicon_tags_60 },
    { 0x9300206, SUC_lexicon_tags_15 },
    { 0xb2100205, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590020e, SUC_lexicon_tags_7 },
    { 0x6570020e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13500211, SUC_lexicon_tags_49 },
    { 0x2d300211, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x99900215, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd4700217, SUC_lexicon_tags_15 },
    { 0x2ab00218, SUC_lexicon_tags_79 },
    { 0x55b00218, SUC_lexicon_tags_7 },
    { 0x88f0021a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5430021d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e50021f, SUC_lexicon_tags_11 },
    { 0xc1b00220, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f00226, SUC_lexicon_tags_27 },
    { 0xdbd00227, SUC_lexicon_tags_13 },
    { 0x66100227, SUC_lexicon_tags_46 },
    { 0x2ef00226, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf470022c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd100231, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd00235, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7500239, SUC_lexicon_tags_30 },
    { 0xb950023a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e30023d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4450024c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf50024f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4b300251, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77900255, SUC_lexicon_tags_46 },
    { 0x76900256, SUC_lexicon_tags_15 },
    { 0x40f00257, SUC_lexicon_tags_47 },
    { 0xf8f00255, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xc10025b, SUC_lexicon_tags_42 },
    { 0x6450025c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e30025f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f00262, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf0700264, SUC_lexicon_tags_14 },
    { 0x64f00265, SUC_lexicon_tags_37 },
    { 0xc8300264, SUC_lexicon_tags_25 },
    { 0xb6500264, SUC_lexicon_tags_60 },
    { 0xd0500266, SUC_lexicon_tags_27 },
    { 0x98b00264, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3f50026b, SUC_lexicon_tags_1 },
    { 0xc1b0026b, SUC_lexicon_tags_17 },
    { 0xaf30026d, SUC_lexicon_tags_33 },
    { 0x97d0026e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f00273, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6100276, SUC_lexicon_tags_7 },
    { 0xd3100276, SUC_lexicon_tags_2 },
    { 0xbe300276, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c90027a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85500282, SUC_lexicon_tags_12 },
    { 0xf5f00282, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5d00285, SUC_lexicon_tags_19 },
    { 0xf2100286, SUC_lexicon_tags_63 },
    { 0x17500287, SUC_lexicon_tags_107 },
    { 0x38100286, SUC_lexicon_tags_49 },
    { 0x8c500289, SUC_lexicon_tags_11 },
    { 0x1310028a, SUC_lexicon_tags_75 },
    { 0xc110028b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a10028d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xfe50028f, SUC_lexicon_tags_37 },
    { 0xaa500290, SUC_lexicon_tags_25 },
    { 0x32f0028f, SUC_lexicon_tags_12 },
    { 0x84f00292, SUC_lexicon_tags_68 },
    { 0xb700293, SUC_lexicon_tags_27 },
    { 0x2e900291, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3a100296, SUC_lexicon_tags_843 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a900299, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf0029c, SUC_lexicon_tags_25 },
    { 0x6450029c, SUC_lexicon_tags_65 },
    { 0x19b0029e, SUC_lexicon_tags_48 },
    { 0x1b90029f, SUC_lexicon_tags_33 },
    { 0xb9b002a0, SUC_lexicon_tags_39 },
    { 0x877002a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfc3002a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b002aa, SUC_lexicon_tags_12 },
    { 0x377002aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279002b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1002bb, SUC_lexicon_tags_33 },
    { 0xd3d002bc, SUC_lexicon_tags_3 },
    { 0x8fb002bd, SUC_lexicon_tags_37 },
    { 0x685002bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x389002c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797002c5, SUC_lexicon_tags_17 },
    { 0xdd5002c5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd41002c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7002cd, SUC_lexicon_tags_37 },
    { 0x72d002ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125002d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x561002d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdbd002d8, SUC_lexicon_tags_25 },
    { 0x2f5002d9, SUC_lexicon_tags_7 },
    { 0x89d002da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1002e0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4d9002e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa63002e4, SUC_lexicon_tags_31 },
    { 0x2c1002e4, SUC_lexicon_tags_87 },
    { 0x913002e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743002ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x989002f1, SUC_lexicon_tags_6 },
    { 0x183002f1, SUC_lexicon_tags_32 },
    { 0x8bb002f2, SUC_lexicon_tags_45 },
    { 0xeb7002f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xef9002f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313002fc, SUC_lexicon_tags_25 },
    { 0x80f002fd, SUC_lexicon_tags_23 },
    { 0x2c1002fd, SUC_lexicon_tags_27 },
    { 0x167002fe, SUC_lexicon_tags_6 },
    { 0xce7002fd, SUC_lexicon_tags_45 },
    { 0x8900301, SUC_lexicon_tags_15 },
    { 0xa1100302, SUC_lexicon_tags_9 },
    { 0xff500303, SUC_lexicon_tags_37 },
    { 0xef700303, SUC_lexicon_tags_7 },
    { 0xead00305, SUC_lexicon_tags_12 },
    { 0x60300304, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x99f00308, SUC_lexicon_tags_43 },
    { 0xdc100308, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1610030e, SUC_lexicon_tags_47 },
    { 0x1430030f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e100312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c300317, SUC_lexicon_tags_27 },
    { 0x95700317, SUC_lexicon_tags_32 },
    { 0x17d00319, SUC_lexicon_tags_17 },
    { 0xf170031a, SUC_lexicon_tags_11 },
    { 0x67f0031b, SUC_lexicon_tags_23 },
    { 0xf470031b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4c30031e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b00325, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b00328, SUC_lexicon_tags_2 },
    { 0x9e100329, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d0032c, SUC_lexicon_tags_11 },
    { 0x2ad0032d, SUC_lexicon_tags_11 },
    { 0x55f0032e, SUC_lexicon_tags_7 },
    { 0x7bf0032f, SUC_lexicon_tags_12 },
    { 0xbd100330, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1900333, SUC_lexicon_tags_17 },
    { 0x5d100334, SUC_lexicon_tags_7 },
    { 0x2c300335, SUC_lexicon_tags_11 },
    { 0x3ed00336, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf0033a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f90033e, SUC_lexicon_tags_61 },
    { 0x20b0033e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b00342, SUC_lexicon_tags_17 },
    { 0x17f00343, SUC_lexicon_tags_12 },
    { 0x38900344, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x23900346, SUC_lexicon_tags_19 },
    { 0xf5100347, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4470034a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb0034e, SUC_lexicon_tags_47 },
    { 0x94b0034e, SUC_lexicon_tags_23 },
    { 0xf8500350, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x32900353, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf00355, SUC_lexicon_tags_12 },
    { 0xa700355, SUC_lexicon_tags_15 },
    { 0xad900357, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3b900359, SUC_lexicon_tags_8 },
    { 0x89100359, SUC_lexicon_tags_2 },
    { 0x1990035b, SUC_lexicon_tags_15 },
    { 0x2ed0035c, SUC_lexicon_tags_6 },
    { 0xf1300359, SUC_lexicon_tags_29 },
    { 0x1f70035c, SUC_lexicon_tags_295 },
    { 0xff10035f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf00363, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x80500366, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26100369, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd70036b, SUC_lexicon_tags_29 },
    { 0x99f0036b, SUC_lexicon_tags_6 },
    { 0x6330036b, SUC_lexicon_tags_18 },
    { 0x15f0036d, SUC_lexicon_tags_25 },
    { 0x80d0036d, SUC_lexicon_tags_39 },
    { 0xf010036f, SUC_lexicon_tags_47 },
    { 0xc170036d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f700374, SUC_lexicon_tags_17 },
    { 0x4c700374, SUC_lexicon_tags_35 },
    { 0xa0900374, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18500379, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af0037c, SUC_lexicon_tags_17 },
    { 0xb450037d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb00381, SUC_lexicon_tags_25 },
    { 0xaa300382, SUC_lexicon_tags_11 },
    { 0xa8100383, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad00387, SUC_lexicon_tags_17 },
    { 0xf2300388, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb0038c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3300394, SUC_lexicon_tags_11 },
    { 0x19d00395, SUC_lexicon_tags_32 },
    { 0x32700396, SUC_lexicon_tags_17 },
    { 0xd4f00397, SUC_lexicon_tags_17 },
    { 0xa9500398, SUC_lexicon_tags_17 },
    { 0x49100399, SUC_lexicon_tags_14 },
    { 0x9fd00399, SUC_lexicon_tags_6 },
    { 0x7fd0039b, SUC_lexicon_tags_63 },
    { 0xfa30039b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f0039f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd7d003a1, SUC_lexicon_tags_12 },
    { 0x33f003a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57003a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4e9003a9, SUC_lexicon_tags_23 },
    { 0xc35003aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5003b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1003b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5003bd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xff7003bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75b003c1, SUC_lexicon_tags_14 },
    { 0x741003c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1f003c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f003c9, SUC_lexicon_tags_10 },
    { 0x84b003ca, SUC_lexicon_tags_7 },
    { 0x1b9003cb, SUC_lexicon_tags_2 },
    { 0x60b003cc, SUC_lexicon_tags_33 },
    { 0xcc3003cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b003d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97003d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf003dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9003e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f003e3, SUC_lexicon_tags_59 },
    { 0x2c7003e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d003e7, SUC_lexicon_tags_66 },
    { 0xe65003e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557003ec, SUC_lexicon_tags_31 },
    { 0xdb9003ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f003f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x719003f3, SUC_lexicon_tags_30 },
    { 0x71f003f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521003f9, SUC_lexicon_tags_26 },
    { 0xb51003f9, SUC_lexicon_tags_14 },
    { 0xb69003fb, SUC_lexicon_tags_25 },
    { 0x463003fc, SUC_lexicon_tags_7 },
    { 0x835003fc, SUC_lexicon_tags_8 },
    { 0x31d003fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xffb00400, SUC_lexicon_tags_15 },
    { 0xde300401, SUC_lexicon_tags_19 },
    { 0xa0500402, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86300404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3900406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f90040d, SUC_lexicon_tags_7 },
    { 0xd7d0040e, SUC_lexicon_tags_38 },
    { 0x6d10040d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5db00411, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50300416, SUC_lexicon_tags_11 },
    { 0xbc300416, SUC_lexicon_tags_23 },
    { 0xa6900417, SUC_lexicon_tags_25 },
    { 0x67f00419, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd700420, SUC_lexicon_tags_47 },
    { 0x9100421, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb00425, SUC_lexicon_tags_11 },
    { 0x82f00425, SUC_lexicon_tags_44 },
    { 0x2fb00426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad00429, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b90042f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2f00431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f00436, SUC_lexicon_tags_46 },
    { 0xc300436, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df0043a, SUC_lexicon_tags_45 },
    { 0xb030043b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb810043d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa7d0043f, SUC_lexicon_tags_56 },
    { 0xedb00440, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b00442, SUC_lexicon_tags_17 },
    { 0x64500443, SUC_lexicon_tags_27 },
    { 0x1e300442, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5910044b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4430044d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae300451, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d500455, SUC_lexicon_tags_1 },
    { 0xbb100456, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b0045b, SUC_lexicon_tags_2 },
    { 0x7490045c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4700463, SUC_lexicon_tags_37 },
    { 0xe3900463, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x47500466, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x82f00468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfef0046a, SUC_lexicon_tags_47 },
    { 0xfd90046b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc390046e, SUC_lexicon_tags_11 },
    { 0xbf30046f, SUC_lexicon_tags_15 },
    { 0xf3100470, SUC_lexicon_tags_45 },
    { 0xea70046e, SUC_lexicon_tags_7 },
    { 0xc500472, SUC_lexicon_tags_2 },
    { 0x90d00472, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5100476, SUC_lexicon_tags_0 },
    { 0x17b00477, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b00480, SUC_lexicon_tags_1 },
    { 0x7d700480, SUC_lexicon_tags_25 },
    { 0x8db00482, SUC_lexicon_tags_10 },
    { 0xff500480, SUC_lexicon_tags_3 },
    { 0x41100484, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e700486, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6d00488, SUC_lexicon_tags_23 },
    { 0x54b00489, SUC_lexicon_tags_17 },
    { 0xb2f0048a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcb0048c, SUC_lexicon_tags_42 },
    { 0xd1b0048d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89900497, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3d00499, SUC_lexicon_tags_15 },
    { 0x46d0049a, SUC_lexicon_tags_37 },
    { 0x36f0049b, SUC_lexicon_tags_37 },
    { 0xc0d0049b, SUC_lexicon_tags_14 },
    { 0x1bb0049d, SUC_lexicon_tags_14 },
    { 0x1310049b, SUC_lexicon_tags_2 },
    { 0xc670049d, SUC_lexicon_tags_1 },
    { 0x5330049d, SUC_lexicon_tags_7 },
    { 0xc37004a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d004a8, SUC_lexicon_tags_63 },
    { 0x2cd004a9, SUC_lexicon_tags_113 },
    { 0xd1d004a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d7004ac, SUC_lexicon_tags_13 },
    { 0xe8d004ac, SUC_lexicon_tags_11 },
    { 0x973004ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f1004b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b004b3, SUC_lexicon_tags_68 },
    { 0xbfb004b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf004b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x441004bc, SUC_lexicon_tags_27 },
    { 0xaab004bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89004c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x421004c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83004c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x917004c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5004cf, SUC_lexicon_tags_43 },
    { 0xf23004d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb004d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59004d9, SUC_lexicon_tags_37 },
    { 0xf15004d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x177004dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc09004de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x47b004e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79004e4, SUC_lexicon_tags_33 },
    { 0x589004e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd67004e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1004ed, SUC_lexicon_tags_17 },
    { 0x74b004ee, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaaf004f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1004f4, SUC_lexicon_tags_9 },
    { 0x59b004f4, SUC_lexicon_tags_15 },
    { 0xac5004f4, SUC_lexicon_tags_31 },
    { 0x48b004f4, SUC_lexicon_tags_27 },
    { 0x3c7004f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x857004fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215004ff, SUC_lexicon_tags_65 },
    { 0x30d004ff, SUC_lexicon_tags_37 },
    { 0xec700501, SUC_lexicon_tags_25 },
    { 0xb0700501, SUC_lexicon_tags_49 },
    { 0x91700503, SUC_lexicon_tags_38 },
    { 0xa0f00504, SUC_lexicon_tags_34 },
    { 0xf3500505, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f00510, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9b00512, SUC_lexicon_tags_31 },
    { 0xb0b00512, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f00516, SUC_lexicon_tags_21 },
    { 0x87f00517, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26700519, SUC_lexicon_tags_12 },
    { 0xc790051a, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x54d0051c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9950051f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23100521, SUC_lexicon_tags_14 },
    { 0xd2f00521, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca300524, SUC_lexicon_tags_1 },
    { 0x22700524, SUC_lexicon_tags_15 },
    { 0x92300526, SUC_lexicon_tags_12 },
    { 0x7b300527, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd0052a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9410052e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xadb00530, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b00533, SUC_lexicon_tags_1 },
    { 0x39900534, SUC_lexicon_tags_27 },
    { 0xecf00533, SUC_lexicon_tags_7 },
    { 0x53500536, SUC_lexicon_tags_15 },
    { 0x74d00534, SUC_lexicon_tags_17 },
    { 0x6f00536, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea50053c, SUC_lexicon_tags_648 },
    { 0x80d0053c, SUC_lexicon_tags_12 },
    { 0xb2d0053e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad00541, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f00547, SUC_lexicon_tags_1 },
    { 0xeaf00548, SUC_lexicon_tags_15 },
    { 0xee900547, SUC_lexicon_tags_27 },
    { 0x86b0054a, SUC_lexicon_tags_39 },
    { 0x8f500548, SUC_lexicon_tags_22 },
    { 0x300547, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc4d0054e, SUC_lexicon_tags_7 },
    { 0x5e10054f, SUC_lexicon_tags_2 },
    { 0xee30054f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5100552, SUC_lexicon_tags_2 },
    { 0x18900553, SUC_lexicon_tags_2 },
    { 0x9700554, SUC_lexicon_tags_37 },
    { 0xbfd00553, SUC_lexicon_tags_12 },
    { 0xfff00552, SUC_lexicon_tags_4 },
    { 0xcb900557, SUC_lexicon_tags_66 },
    { 0xa6900554, SUC_lexicon_tags_14 },
    { 0x7a100552, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6150055d, SUC_lexicon_tags_1 },
    { 0xc90055d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60700560, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8700565, SUC_lexicon_tags_11 },
    { 0x1ad00565, SUC_lexicon_tags_47 },
    { 0x4700567, SUC_lexicon_tags_19 },
    { 0xd3300568, SUC_lexicon_tags_25 },
    { 0xf8900568, SUC_lexicon_tags_33 },
    { 0xbed00565, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b00572, SUC_lexicon_tags_25 },
    { 0xd8500572, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f00576, SUC_lexicon_tags_11 },
    { 0xfb700576, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb0057c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67300581, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96300583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff300589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f00590, SUC_lexicon_tags_7 },
    { 0x8ff00591, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47900593, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf500595, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77100599, SUC_lexicon_tags_11 },
    { 0xb390059a, SUC_lexicon_tags_1 },
    { 0xc5f0059b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed005a1, SUC_lexicon_tags_0 },
    { 0xb25005a2, SUC_lexicon_tags_23 },
    { 0x575005a3, SUC_lexicon_tags_17 },
    { 0x69f005a3, SUC_lexicon_tags_37 },
    { 0x8ed005a4, SUC_lexicon_tags_46 },
    { 0xd21005a1, SUC_lexicon_tags_37 },
    { 0x72b005a7, SUC_lexicon_tags_33 },
    { 0x561005a7, SUC_lexicon_tags_19 },
    { 0x331005a3, SUC_lexicon_tags_0 },
    { 0xcd3005aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1005b2, SUC_lexicon_tags_7 },
    { 0xe47005b2, SUC_lexicon_tags_87 },
    { 0xb77005b3, SUC_lexicon_tags_15 },
    { 0x7f3005b5, SUC_lexicon_tags_37 },
    { 0x3c9005b6, SUC_lexicon_tags_12 },
    { 0x585005b6, SUC_lexicon_tags_0 },
    { 0x753005b8, SUC_lexicon_tags_31 },
    { 0xcd5005b9, SUC_lexicon_tags_47 },
    { 0x7a3005b4, SUC_lexicon_tags_84 },
    { 0x5d5005bb, SUC_lexicon_tags_79 },
    { 0x653005bc, SUC_lexicon_tags_11 },
    { 0xd01005b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21005c7, SUC_lexicon_tags_17 },
    { 0x17005c7, SUC_lexicon_tags_30 },
    { 0x91d005c7, SUC_lexicon_tags_38 },
    { 0xfaf005ca, SUC_lexicon_tags_12 },
    { 0xfb5005cb, SUC_lexicon_tags_11 },
    { 0x5c5005c9, SUC_lexicon_tags_15 },
    { 0xd33005cd, SUC_lexicon_tags_37 },
    { 0x1a3005ce, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437005d3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x985005d5, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71005d8, SUC_lexicon_tags_14 },
    { 0xd09005d9, SUC_lexicon_tags_7 },
    { 0x787005da, SUC_lexicon_tags_26 },
    { 0x9af005db, SUC_lexicon_tags_1 },
    { 0x90b005dc, SUC_lexicon_tags_11 },
    { 0xd2d005dc, SUC_lexicon_tags_12 },
    { 0xfaf005de, SUC_lexicon_tags_25 },
    { 0xbdb005df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe99005e1, SUC_lexicon_tags_33 },
    { 0x333005e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653005e8, SUC_lexicon_tags_17 },
    { 0x2a9005e9, SUC_lexicon_tags_11 },
    { 0xeef005ea, SUC_lexicon_tags_68 },
    { 0x853005ea, SUC_lexicon_tags_16 },
    { 0x817005ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397005f1, SUC_lexicon_tags_87 },
    { 0x243005f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96d005f4, SUC_lexicon_tags_50 },
    { 0xacd005f4, SUC_lexicon_tags_14 },
    { 0xad7005f6, SUC_lexicon_tags_11 },
    { 0x62b005f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991005fb, SUC_lexicon_tags_37 },
    { 0xa9005fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43005ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde500603, SUC_lexicon_tags_25 },
    { 0xcb500604, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32100606, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60300608, SUC_lexicon_tags_24 },
    { 0x98500609, SUC_lexicon_tags_27 },
    { 0x57d0060a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350060f, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9300612, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d300615, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61500619, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf490061c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f0061f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b00626, SUC_lexicon_tags_12 },
    { 0x8500627, SUC_lexicon_tags_47 },
    { 0x3db00628, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6070062a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf230062d, SUC_lexicon_tags_13 },
    { 0x57f0062e, SUC_lexicon_tags_9 },
    { 0x3f10062d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a500632, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f00636, SUC_lexicon_tags_12 },
    { 0xa3b00637, SUC_lexicon_tags_37 },
    { 0xe4900636, SUC_lexicon_tags_17 },
    { 0x65900636, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d0063e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2700640, SUC_lexicon_tags_6 },
    { 0x77100640, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a500645, SUC_lexicon_tags_25 },
    { 0x85b00646, SUC_lexicon_tags_29 },
    { 0xe1100647, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b0064f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d00655, SUC_lexicon_tags_7 },
    { 0x39100656, SUC_lexicon_tags_11 },
    { 0x10d00655, SUC_lexicon_tags_37 },
    { 0x13900658, SUC_lexicon_tags_39 },
    { 0x41f00659, SUC_lexicon_tags_10 },
    { 0xe6b0065a, SUC_lexicon_tags_1 },
    { 0xc250065a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58f0065d, SUC_lexicon_tags_38 },
    { 0xad50065d, SUC_lexicon_tags_2 },
    { 0x7930065f, SUC_lexicon_tags_37 },
    { 0x2f90065f, SUC_lexicon_tags_25 },
    { 0xc700660, SUC_lexicon_tags_11 },
    { 0x650065f, SUC_lexicon_tags_43 },
    { 0xb7900660, SUC_lexicon_tags_37 },
    { 0x88f00664, SUC_lexicon_tags_63 },
    { 0xc3100665, SUC_lexicon_tags_1 },
    { 0x66100666, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c70066a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62d0066c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a700670, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4500674, SUC_lexicon_tags_8 },
    { 0x1ff00675, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x44900678, SUC_lexicon_tags_1 },
    { 0xd3d00679, SUC_lexicon_tags_25 },
    { 0x9530067a, SUC_lexicon_tags_55 },
    { 0xc890067b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15f0067d, SUC_lexicon_tags_42 },
    { 0x70b0067e, SUC_lexicon_tags_16 },
    { 0xd0b0067e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b00685, SUC_lexicon_tags_11 },
    { 0x3500686, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5870068d, SUC_lexicon_tags_4 },
    { 0x71d0068e, SUC_lexicon_tags_15 },
    { 0x4670068f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3300691, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d00695, SUC_lexicon_tags_33 },
    { 0x51d00696, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb0069a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6030069c, SUC_lexicon_tags_26 },
    { 0x3a30069d, SUC_lexicon_tags_38 },
    { 0xded0069e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb006a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf77006a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd11006a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x809006a8, SUC_lexicon_tags_33 },
    { 0x7e7006a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x51006ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb006b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d006b6, SUC_lexicon_tags_2 },
    { 0x6e5006b7, SUC_lexicon_tags_9 },
    { 0x6f7006b8, SUC_lexicon_tags_37 },
    { 0x35b006b9, SUC_lexicon_tags_6 },
    { 0x107006ba, SUC_lexicon_tags_37 },
    { 0x91006bb, SUC_lexicon_tags_11 },
    { 0xffd006bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x387006be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab006c5, SUC_lexicon_tags_27 },
    { 0x14b006c6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71006ca, SUC_lexicon_tags_21 },
    { 0xa4d006cb, SUC_lexicon_tags_46 },
    { 0xc75006cc, SUC_lexicon_tags_19 },
    { 0xff9006cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3006d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d006d8, SUC_lexicon_tags_2 },
    { 0x1f3006d9, SUC_lexicon_tags_11 },
    { 0xc45006d9, SUC_lexicon_tags_65 },
    { 0x729006d9, SUC_lexicon_tags_37 },
    { 0x53f006dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x179006de, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe27006e0, SUC_lexicon_tags_17 },
    { 0x785006e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217006e7, SUC_lexicon_tags_38 },
    { 0x3e9006e8, SUC_lexicon_tags_28 },
    { 0xb7d006e8, SUC_lexicon_tags_2 },
    { 0xaa1006e7, SUC_lexicon_tags_1 },
    { 0x2ef006eb, SUC_lexicon_tags_17 },
    { 0x173006eb, SUC_lexicon_tags_37 },
    { 0x7ab006ed, SUC_lexicon_tags_21 },
    { 0x1a9006ed, SUC_lexicon_tags_23 },
    { 0xf61006e8, SUC_lexicon_tags_31 },
    { 0x443006ea, SUC_lexicon_tags_15 },
    { 0x931006ee, SUC_lexicon_tags_17 },
    { 0x95b006eb, SUC_lexicon_tags_27 },
    { 0xd39006eb, SUC_lexicon_tags_14 },
    { 0xc7b006f4, SUC_lexicon_tags_9 },
    { 0xcf5006f5, SUC_lexicon_tags_7 },
    { 0x399006eb, SUC_lexicon_tags_12 },
    { 0x121006e7, SUC_lexicon_tags_11 },
    { 0xc1006f8, SUC_lexicon_tags_127 },
    { 0xb29006f9, SUC_lexicon_tags_1 },
    { 0xa2b006fa, SUC_lexicon_tags_17 },
    { 0x623006eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x797006fe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b00701, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a700703, SUC_lexicon_tags_197 },
    { 0x5fb00704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f500708, SUC_lexicon_tags_2 },
    { 0xcaf00708, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb410070c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b00710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc700715, SUC_lexicon_tags_15 },
    { 0xa7700715, SUC_lexicon_tags_25 },
    { 0x88300717, SUC_lexicon_tags_27 },
    { 0x5fd00718, SUC_lexicon_tags_61 },
    { 0x1c700719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f0071c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd250071e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d0072a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb350072d, SUC_lexicon_tags_61 },
    { 0x1ff0072e, SUC_lexicon_tags_5 },
    { 0x67b0072f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68700736, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded00740, SUC_lexicon_tags_63 },
    { 0xd2f00740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86300743, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce70074b, SUC_lexicon_tags_43 },
    { 0x26b0074b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b90074e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e900754, SUC_lexicon_tags_61 },
    { 0x44100754, SUC_lexicon_tags_11 },
    { 0x37700756, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230075c, SUC_lexicon_tags_37 },
    { 0xa110075c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4a50075f, SUC_lexicon_tags_85 },
    { 0x65d0075f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1f100762, SUC_lexicon_tags_37 },
    { 0x85100763, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x56900765, SUC_lexicon_tags_48 },
    { 0x33b00766, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9bd00768, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a50076c, SUC_lexicon_tags_64 },
    { 0xc6b0076d, SUC_lexicon_tags_0 },
    { 0x9bb0076e, SUC_lexicon_tags_1 },
    { 0x5870076f, SUC_lexicon_tags_11 },
    { 0xab10076f, SUC_lexicon_tags_7 },
    { 0x3cd0076c, SUC_lexicon_tags_37 },
    { 0xd9b00772, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc010077d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x86300780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76f00782, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x66f00784, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2700788, SUC_lexicon_tags_25 },
    { 0x95300788, SUC_lexicon_tags_37 },
    { 0x21f00788, SUC_lexicon_tags_7 },
    { 0xff0078b, SUC_lexicon_tags_32 },
    { 0xfb70078c, SUC_lexicon_tags_47 },
    { 0x3990078c, SUC_lexicon_tags_60 },
    { 0x1af00789, SUC_lexicon_tags_46 },
    { 0x7c500788, SUC_lexicon_tags_1 },
    { 0x90d00790, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0xa2d00792, SUC_lexicon_tags_25 },
    { 0x4e500792, SUC_lexicon_tags_1 },
    { 0xc4300794, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc3300796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39d00798, SUC_lexicon_tags_25 },
    { 0x9bb00799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff0079e, SUC_lexicon_tags_112 },
    { 0x11f0079e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd007a2, SUC_lexicon_tags_15 },
    { 0xee9007a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1007a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf65007a9, SUC_lexicon_tags_33 },
    { 0x35f007aa, SUC_lexicon_tags_19 },
    { 0x875007ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b007ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9007b5, SUC_lexicon_tags_17 },
    { 0xa57007b5, SUC_lexicon_tags_25 },
    { 0x797007b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a3007b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d007bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b007c0, SUC_lexicon_tags_9 },
    { 0x9db007c1, SUC_lexicon_tags_37 },
    { 0xcd5007c2, SUC_lexicon_tags_2 },
    { 0x789007c0, SUC_lexicon_tags_17 },
    { 0x483007c3, SUC_lexicon_tags_60 },
    { 0x8a9007c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x351007c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43007ca, SUC_lexicon_tags_25 },
    { 0xb33007cb, SUC_lexicon_tags_49 },
    { 0x5bb007cc, SUC_lexicon_tags_30 },
    { 0xbb3007cd, SUC_lexicon_tags_63 },
    { 0x65d007cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883007d2, SUC_lexicon_tags_25 },
    { 0x367007d3, SUC_lexicon_tags_7 },
    { 0x1f007d3, SUC_lexicon_tags_25 },
    { 0xc39007d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27007de, SUC_lexicon_tags_12 },
    { 0xe03007df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5007e2, SUC_lexicon_tags_17 },
    { 0x8e1007e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1007e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ff007e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x293007eb, SUC_lexicon_tags_25 },
    { 0xeaf007eb, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xc3f007ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe57007f0, SUC_lexicon_tags_10 },
    { 0x221007f1, SUC_lexicon_tags_1 },
    { 0x6a1007f0, SUC_lexicon_tags_17 },
    { 0xd99007f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53b007f5, SUC_lexicon_tags_91 },
    { 0x19d007f5, SUC_lexicon_tags_38 },
    { 0x897007f7, SUC_lexicon_tags_13 },
    { 0x211007f7, SUC_lexicon_tags_63 },
    { 0x681007f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5007fd, SUC_lexicon_tags_38 },
    { 0xd0b007fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17f00800, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b00803, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e300807, SUC_lexicon_tags_63 },
    { 0xe7f00807, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d0080e, SUC_lexicon_tags_2 },
    { 0x8170080f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9e300811, SUC_lexicon_tags_59 },
    { 0xdc300812, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x81100815, SUC_lexicon_tags_11 },
    { 0x8300815, SUC_lexicon_tags_2 },
    { 0xc900817, SUC_lexicon_tags_7 },
    { 0xb1500817, SUC_lexicon_tags_4 },
    { 0xb900819, SUC_lexicon_tags_2 },
    { 0x4d90081a, SUC_lexicon_tags_1 },
    { 0xd3700819, SUC_lexicon_tags_23 },
    { 0xcf10081a, SUC_lexicon_tags_17 },
    { 0x730081d, SUC_lexicon_tags_12 },
    { 0x58d0081c, SUC_lexicon_tags_68 },
    { 0xf70081f, SUC_lexicon_tags_17 },
    { 0xb9900820, SUC_lexicon_tags_229 },
    { 0xa1500820, SUC_lexicon_tags_25 },
    { 0x7530081f, SUC_lexicon_tags_11 },
    { 0xd4300820, SUC_lexicon_tags_2 },
    { 0x8e300824, SUC_lexicon_tags_37 },
    { 0x6a700824, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f0082c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9150082e, SUC_lexicon_tags_12 },
    { 0xa0d0082f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc8300831, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd00836, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f0083a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df0083d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6890083f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19500842, SUC_lexicon_tags_7 },
    { 0x34300842, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74500848, SUC_lexicon_tags_1 },
    { 0xf0300848, SUC_lexicon_tags_40 },
    { 0x8410084a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1110084c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec700851, SUC_lexicon_tags_19 },
    { 0x1cf00851, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b00855, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e100859, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70085c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f500860, SUC_lexicon_tags_84 },
    { 0x5e700861, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49900865, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad10086a, SUC_lexicon_tags_26 },
    { 0x47d0086a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76b0086d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8cb0086f, SUC_lexicon_tags_19 },
    { 0x6a500870, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1cb00872, SUC_lexicon_tags_29 },
    { 0xb2500873, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f00876, SUC_lexicon_tags_0 },
    { 0xb7900877, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1b00879, SUC_lexicon_tags_2 },
    { 0x1bf0087a, SUC_lexicon_tags_1 },
    { 0x40d0087b, SUC_lexicon_tags_7 },
    { 0xa2d0087c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xac70087e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43700882, SUC_lexicon_tags_37 },
    { 0x73300883, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b00887, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc0700889, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c50088b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f10088e, SUC_lexicon_tags_14 },
    { 0xe010088e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5900891, SUC_lexicon_tags_11 },
    { 0x40f00892, SUC_lexicon_tags_15 },
    { 0xbb00893, SUC_lexicon_tags_17 },
    { 0xbf00893, SUC_lexicon_tags_57 },
    { 0x29700895, SUC_lexicon_tags_33 },
    { 0x4fd00895, SUC_lexicon_tags_33 },
    { 0x4e500896, SUC_lexicon_tags_3 },
    { 0x11b00898, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed0089d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1610089f, SUC_lexicon_tags_33 },
    { 0xa9d008a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67008a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231008a9, SUC_lexicon_tags_87 },
    { 0xf9b008a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xecb008ac, SUC_lexicon_tags_37 },
    { 0xa55008ad, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x65008af, SUC_lexicon_tags_26 },
    { 0x449008af, SUC_lexicon_tags_47 },
    { 0x731008b1, SUC_lexicon_tags_2 },
    { 0x8f3008b2, SUC_lexicon_tags_12 },
    { 0xef008b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9008b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9008bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x903008bd, SUC_lexicon_tags_4 },
    { 0x9e1008be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f008c2, SUC_lexicon_tags_12 },
    { 0x919008c3, SUC_lexicon_tags_8 },
    { 0x461008c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1008c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f008cc, SUC_lexicon_tags_15 },
    { 0xc23008cd, SUC_lexicon_tags_25 },
    { 0xf71008cd, SUC_lexicon_tags_12 },
    { 0x3db008cd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7008d2, SUC_lexicon_tags_26 },
    { 0x333008d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9008d6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f008db, SUC_lexicon_tags_39 },
    { 0x93b008db, SUC_lexicon_tags_8 },
    { 0xf3d008dd, SUC_lexicon_tags_55 },
    { 0xc53008db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x97d008e0, SUC_lexicon_tags_37 },
    { 0x6ad008e1, SUC_lexicon_tags_1 },
    { 0x5db008e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89008e5, SUC_lexicon_tags_39 },
    { 0xf33008e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3008eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x389008ed, SUC_lexicon_tags_25 },
    { 0x475008ee, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f008f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x677008f4, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17008f9, SUC_lexicon_tags_49 },
    { 0x643008f9, SUC_lexicon_tags_68 },
    { 0xcdb008f9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11008fe, SUC_lexicon_tags_15 },
    { 0x5b3008fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd1900901, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x100907, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd490090e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61500912, SUC_lexicon_tags_0 },
    { 0x9700912, SUC_lexicon_tags_39 },
    { 0x69f00914, SUC_lexicon_tags_15 },
    { 0xe5700914, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa90091d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89500921, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6900926, SUC_lexicon_tags_15 },
    { 0x24900927, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa730092b, SUC_lexicon_tags_12 },
    { 0x4b10092c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb550092f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb300932, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb00939, SUC_lexicon_tags_7 },
    { 0xab30093a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b50093c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62900942, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7300944, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xea300947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa670094c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df00952, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x39d00954, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74900956, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f10095a, SUC_lexicon_tags_8 },
    { 0xed50095b, SUC_lexicon_tags_12 },
    { 0x9930095c, SUC_lexicon_tags_43 },
    { 0xec10095d, SUC_lexicon_tags_43 },
    { 0x3630095e, SUC_lexicon_tags_12 },
    { 0x51f0095d, SUC_lexicon_tags_25 },
    { 0x8670095f, SUC_lexicon_tags_23 },
    { 0x5bb00961, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x42100964, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd8100966, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db0096b, SUC_lexicon_tags_2 },
    { 0x71b0096c, SUC_lexicon_tags_25 },
    { 0x1df0096d, SUC_lexicon_tags_1 },
    { 0x3cd0096b, SUC_lexicon_tags_37 },
    { 0x39b0096f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf00972, SUC_lexicon_tags_21 },
    { 0xfcf00972, SUC_lexicon_tags_18 },
    { 0xe2100974, SUC_lexicon_tags_29 },
    { 0x77f00974, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89f00977, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7f00979, SUC_lexicon_tags_1 },
    { 0x25f0097a, SUC_lexicon_tags_27 },
    { 0xe7100979, SUC_lexicon_tags_30 },
    { 0x7610097c, SUC_lexicon_tags_37 },
    { 0xd0b0097d, SUC_lexicon_tags_11 },
    { 0x8dd0097e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbd500980, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf00983, SUC_lexicon_tags_29 },
    { 0x64d00983, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77900986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74300988, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8300991, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d00994, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b0099a, SUC_lexicon_tags_7 },
    { 0x75f0099a, SUC_lexicon_tags_45 },
    { 0xa90099c, SUC_lexicon_tags_14 },
    { 0x710099b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6010099f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x5009a2, SUC_lexicon_tags_15 },
    { 0x961009a3, SUC_lexicon_tags_87 },
    { 0x4fb009a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f009a7, SUC_lexicon_tags_38 },
    { 0x68d009a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1009ab, SUC_lexicon_tags_35 },
    { 0xf69009ac, SUC_lexicon_tags_45 },
    { 0x9b9009ac, SUC_lexicon_tags_65 },
    { 0x6cd009ab, SUC_lexicon_tags_19 },
    { 0x6dd009af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45009b2, SUC_lexicon_tags_2 },
    { 0x219009b2, SUC_lexicon_tags_13 },
    { 0x7f5009b4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x531009b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47009bd, SUC_lexicon_tags_11 },
    { 0xf03009be, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809009c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29009c7, SUC_lexicon_tags_25 },
    { 0xc7009c7, SUC_lexicon_tags_32 },
    { 0x709009c9, SUC_lexicon_tags_37 },
    { 0x9f1009ca, SUC_lexicon_tags_11 },
    { 0xc21009ca, SUC_lexicon_tags_55 },
    { 0x2e9009ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf75009ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd009d1, SUC_lexicon_tags_11 },
    { 0x615009d2, SUC_lexicon_tags_1 },
    { 0x7d1009d1, SUC_lexicon_tags_43 },
    { 0x4c7009d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf69009d6, SUC_lexicon_tags_14 },
    { 0xe43009d6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd009db, SUC_lexicon_tags_35 },
    { 0xe87009dc, SUC_lexicon_tags_29 },
    { 0xd0d009dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1009e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3009e4, SUC_lexicon_tags_47 },
    { 0x7f5009e4, SUC_lexicon_tags_39 },
    { 0xbd009e6, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7009ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1009ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93009f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf009fb, SUC_lexicon_tags_7 },
    { 0x13d009fc, SUC_lexicon_tags_17 },
    { 0x4d1009fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88300a05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6100a0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc300a11, SUC_lexicon_tags_11 },
    { 0xfed00a12, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23300a16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb1100a18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39900a1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb00a1d, SUC_lexicon_tags_39 },
    { 0xe9f00a1e, SUC_lexicon_tags_12 },
    { 0x6c900a1f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b00a27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92300a2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86500a2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70f00a30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a100a34, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b900a37, SUC_lexicon_tags_49 },
    { 0xc5100a38, SUC_lexicon_tags_33 },
    { 0xcf300a39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92700a3d, SUC_lexicon_tags_11 },
    { 0x35b00a3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7100a41, SUC_lexicon_tags_26 },
    { 0x65700a42, SUC_lexicon_tags_17 },
    { 0x4f300a43, SUC_lexicon_tags_68 },
    { 0x76700a44, SUC_lexicon_tags_17 },
    { 0xc2d00a45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc900a48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40100a4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b00a4f, SUC_lexicon_tags_2 },
    { 0x15d00a4f, SUC_lexicon_tags_38 },
    { 0xbd900a51, SUC_lexicon_tags_60 },
    { 0xcd700a52, SUC_lexicon_tags_27 },
    { 0x3c300a53, SUC_lexicon_tags_37 },
    { 0x9fb00a54, SUC_lexicon_tags_27 },
    { 0x1d300a54, SUC_lexicon_tags_37 },
    { 0xfeb00a56, SUC_lexicon_tags_527 },
    { 0xdcd00a56, SUC_lexicon_tags_0 },
    { 0xbb300a4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x62f00a5a, SUC_lexicon_tags_17 },
    { 0xba300a5a, SUC_lexicon_tags_27 },
    { 0x4a700a5c, SUC_lexicon_tags_12 },
    { 0x2bb00a5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd00a60, SUC_lexicon_tags_15 },
    { 0x4f700a60, SUC_lexicon_tags_7 },
    { 0x94900a62, SUC_lexicon_tags_21 },
    { 0x7cd00a63, SUC_lexicon_tags_1 },
    { 0x98300a64, SUC_lexicon_tags_16 },
    { 0xb0900a62, SUC_lexicon_tags_25 },
    { 0xc5300a64, SUC_lexicon_tags_17 },
    { 0x72500a67, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b00a6e, SUC_lexicon_tags_28 },
    { 0x16300a6f, SUC_lexicon_tags_45 },
    { 0x74f00a70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d00a73, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xcb900a75, SUC_lexicon_tags_2 },
    { 0xf3700a76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e900a7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58f00a7d, SUC_lexicon_tags_1 },
    { 0xe0100a7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e900a86, SUC_lexicon_tags_7 },
    { 0xabd00a87, SUC_lexicon_tags_49 },
    { 0xa5b00a87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40d00a8a, SUC_lexicon_tags_11 },
    { 0x27300a8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9300a8d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x19900a8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5300a91, SUC_lexicon_tags_17 },
    { 0x22500a92, SUC_lexicon_tags_37 },
    { 0xba900a93, SUC_lexicon_tags_6 },
    { 0xef100a92, SUC_lexicon_tags_19 },
    { 0xb2100a95, SUC_lexicon_tags_19 },
    { 0xf7b00a96, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x19500a98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b00a9e, SUC_lexicon_tags_38 },
    { 0x72900a9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb00aa1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x78700aa3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf00aa9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81700aad, SUC_lexicon_tags_12 },
    { 0xa7500aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf100ab3, SUC_lexicon_tags_37 },
    { 0xfb500ab4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92d00ab6, SUC_lexicon_tags_16 },
    { 0xc0300ab6, SUC_lexicon_tags_47 },
    { 0x9e100ab7, SUC_lexicon_tags_35 },
    { 0xacd00ab9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef500ac1, SUC_lexicon_tags_32 },
    { 0xdc500ac2, SUC_lexicon_tags_17 },
    { 0x60100ac3, SUC_lexicon_tags_12 },
    { 0x96b00ac4, SUC_lexicon_tags_15 },
    { 0xb2300ac5, SUC_lexicon_tags_15 },
    { 0x3d100ac6, SUC_lexicon_tags_47 },
    { 0x49b00ac4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dd00ac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27500ad7, SUC_lexicon_tags_15 },
    { 0x26b00ad7, SUC_lexicon_tags_2 },
    { 0xe4b00ad7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad00ae2, SUC_lexicon_tags_7 },
    { 0xc1300ae3, SUC_lexicon_tags_1 },
    { 0x3d500ae4, SUC_lexicon_tags_21 },
    { 0x6e100ae2, SUC_lexicon_tags_19 },
    { 0xa5b00ae4, SUC_lexicon_tags_49 },
    { 0x4df00ae3, SUC_lexicon_tags_82 },
    { 0x73f00ae6, SUC_lexicon_tags_60 },
    { 0x43d00ae9, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9500af1, SUC_lexicon_tags_25 },
    { 0x67b00af2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x14f00af4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf00af7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbf900af9, SUC_lexicon_tags_37 },
    { 0x8c500af9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f500afc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f00aff, SUC_lexicon_tags_3 },
    { 0x84700b00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfb100b02, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x98700b04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf00b08, SUC_lexicon_tags_97 },
    { 0x87700b08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b300b19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d00b20, SUC_lexicon_tags_37 },
    { 0x6100b21, SUC_lexicon_tags_1 },
    { 0x1ad00b22, SUC_lexicon_tags_19 },
    { 0x9c700b22, SUC_lexicon_tags_11 },
    { 0x87f00b24, SUC_lexicon_tags_48 },
    { 0xb1100b25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8300b2a, SUC_lexicon_tags_282 },
    { 0xfbb00b2b, SUC_lexicon_tags_25 },
    { 0xa2100b2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe900b30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe900b33, SUC_lexicon_tags_2 },
    { 0x64100b34, SUC_lexicon_tags_6 },
    { 0xb9b00b35, SUC_lexicon_tags_15 },
    { 0x5c500b34, SUC_lexicon_tags_25 },
    { 0x99900b37, SUC_lexicon_tags_38 },
    { 0xfab00b34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f500b3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85900b41, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1e300b43, SUC_lexicon_tags_37 },
    { 0xec900b43, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a900b48, SUC_lexicon_tags_11 },
    { 0xcf500b49, SUC_lexicon_tags_11 },
    { 0xae900b4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb00b4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef500b4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x69100b52, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1700b56, SUC_lexicon_tags_14 },
    { 0x5dd00b56, SUC_lexicon_tags_68 },
    { 0xacf00b56, SUC_lexicon_tags_70 },
    { 0x67100b58, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x73500b5b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d00b5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x66500b61, SUC_lexicon_tags_17 },
    { 0xd100b62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd300b64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x47300b67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef500b6c, SUC_lexicon_tags_10 },
    { 0xaa500b6d, SUC_lexicon_tags_0 },
    { 0xcb500b6e, SUC_lexicon_tags_1 },
    { 0x25b00b6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72f00b71, SUC_lexicon_tags_10 },
    { 0xb8b00b72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d100b76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d100b7f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35900b83, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd0900b85, SUC_lexicon_tags_38 },
    { 0xbaf00b85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa1300b88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1f00b8a, SUC_lexicon_tags_66 },
    { 0x8b900b8b, SUC_lexicon_tags_12 },
    { 0x44f00b8c, SUC_lexicon_tags_2 },
    { 0x77700b8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d00b96, SUC_lexicon_tags_23 },
    { 0xc0500b97, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0x71b00b99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc100b9b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b00b9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb00ba1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34100ba5, SUC_lexicon_tags_33 },
    { 0x27b00ba6, SUC_lexicon_tags_37 },
    { 0xec500ba7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f00bac, SUC_lexicon_tags_37 },
    { 0x5bb00bac, SUC_lexicon_tags_35 },
    { 0x56100bae, SUC_lexicon_tags_27 },
    { 0x83f00bae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57100bb1, SUC_lexicon_tags_25 },
    { 0xb0d00bb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f00bb9, SUC_lexicon_tags_17 },
    { 0x16b00bba, SUC_lexicon_tags_0 },
    { 0x6100bba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc500bc0, SUC_lexicon_tags_30 },
    { 0x1e100bc1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe100bc4, SUC_lexicon_tags_66 },
    { 0xffb00bc5, SUC_lexicon_tags_17 },
    { 0xe0300bc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90900bcd, SUC_lexicon_tags_21 },
    { 0xfcd00bcd, SUC_lexicon_tags_25 },
    { 0xd2d00bcf, SUC_lexicon_tags_12 },
    { 0xf0d00bd0, SUC_lexicon_tags_38 },
    { 0x9d900bd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65100bd5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29100bda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb00bdf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c300be4, SUC_lexicon_tags_17 },
    { 0x2f00be5, SUC_lexicon_tags_2 },
    { 0x53900be5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80300be8, SUC_lexicon_tags_25 },
    { 0x95100be9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1300bec, SUC_lexicon_tags_47 },
    { 0x23300bed, SUC_lexicon_tags_23 },
    { 0x3a300bed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d00bf1, SUC_lexicon_tags_15 },
    { 0x6e300bf1, SUC_lexicon_tags_25 },
    { 0xae300bf3, SUC_lexicon_tags_47 },
    { 0x16700bf3, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x64900bf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4900c03, SUC_lexicon_tags_6 },
    { 0xedb00c04, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc5500c06, SUC_lexicon_tags_1 },
    { 0xd5300c06, SUC_lexicon_tags_0 },
    { 0xd4300c08, SUC_lexicon_tags_728 },
    { 0x80700c09, SUC_lexicon_tags_0 },
    { 0xfe500c0a, SUC_lexicon_tags_16 },
    { 0x63f00c09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91100c1a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x90500c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79100c20, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x15f00c22, SUC_lexicon_tags_11 },
    { 0xccf00c23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf9d00c25, SUC_lexicon_tags_1 },
    { 0x69300c26, SUC_lexicon_tags_17 },
    { 0x4a100c26, SUC_lexicon_tags_92 },
    { 0x46f00c28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7300c2c, SUC_lexicon_tags_39 },
    { 0x90900c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f00c33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36900c35, SUC_lexicon_tags_12 },
    { 0xe9d00c35, SUC_lexicon_tags_2 },
    { 0x91f00c35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2500c39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb300c3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x57100c42, SUC_lexicon_tags_37 },
    { 0xa0d00c42, SUC_lexicon_tags_7 },
    { 0xa2d00c44, SUC_lexicon_tags_188 },
    { 0x4f300c45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4eb00c47, SUC_lexicon_tags_7 },
    { 0x91300c48, SUC_lexicon_tags_37 },
    { 0xbe900c48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d00c4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xed900c4e, SUC_lexicon_tags_12 },
    { 0x33300c4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32500c54, SUC_lexicon_tags_49 },
    { 0xc9100c54, SUC_lexicon_tags_60 },
    { 0xac900c56, SUC_lexicon_tags_25 },
    { 0x7df00c57, SUC_lexicon_tags_38 },
    { 0xb2700c54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc4700c5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b100c5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fd00c5f, SUC_lexicon_tags_35 },
    { 0x4e900c5f, SUC_lexicon_tags_15 },
    { 0xd0300c60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8700c69, SUC_lexicon_tags_29 },
    { 0xf7f00c6a, SUC_lexicon_tags_20 },
    { 0x52100c6b, SUC_lexicon_tags_11 },
    { 0x27300c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9900c6f, SUC_lexicon_tags_11 },
    { 0x2300c70, SUC_lexicon_tags_2 },
    { 0xc6900c70, SUC_lexicon_tags_60 },
    { 0x44d00c72, SUC_lexicon_tags_68 },
    { 0xcf500c73, SUC_lexicon_tags_49 },
    { 0x4a900c70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4e500c76, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f500c7d, SUC_lexicon_tags_1 },
    { 0x27700c7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10100c86, SUC_lexicon_tags_17 },
    { 0x4500c87, SUC_lexicon_tags_46 },
    { 0xa6b00c88, SUC_lexicon_tags_34 },
    { 0x6e100c87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98500c91, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a500c95, SUC_lexicon_tags_81 },
    { 0x6f700c96, SUC_lexicon_tags_4 },
    { 0x4e100c95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d100c99, SUC_lexicon_tags_12 },
    { 0x11500c9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26900caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f900cad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c500caf, SUC_lexicon_tags_1 },
    { 0x16700caf, SUC_lexicon_tags_49 },
    { 0xa6900cb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xea900cb3, SUC_lexicon_tags_21 },
    { 0x43b00cb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d00cb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd00cc2, SUC_lexicon_tags_15 },
    { 0x6f900cc3, SUC_lexicon_tags_12 },
    { 0x42d00cc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2500cc7, SUC_lexicon_tags_14 },
    { 0xf5500cc8, SUC_lexicon_tags_187 },
    { 0x61700cc9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e300cd1, SUC_lexicon_tags_44 },
    { 0x74500cd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c900cd4, SUC_lexicon_tags_12 },
    { 0xb0500cd4, SUC_lexicon_tags_49 },
    { 0xc4300cd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0100cdb, SUC_lexicon_tags_2 },
    { 0x47f00cdb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53300ce0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95700ce4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cf00ce6, SUC_lexicon_tags_38 },
    { 0x4d700ce6, SUC_lexicon_tags_15 },
    { 0x61900ce8, SUC_lexicon_tags_48 },
    { 0xbbb00ce7, SUC_lexicon_tags_49 },
    { 0xe4b00cea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf9b00cec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d00cef, SUC_lexicon_tags_6 },
    { 0x99700cf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8100cf2, SUC_lexicon_tags_65 },
    { 0x38d00cf3, SUC_lexicon_tags_38 },
    { 0x35700cf4, SUC_lexicon_tags_21 },
    { 0x84300cf4, SUC_lexicon_tags_37 },
    { 0x22f00cf6, SUC_lexicon_tags_15 },
    { 0x18900cf4, SUC_lexicon_tags_31 },
    { 0x9ff00cf7, SUC_lexicon_tags_44 },
    { 0x17500cf9, SUC_lexicon_tags_9 },
    { 0xdff00cfa, SUC_lexicon_tags_11 },
    { 0x48500cf9, SUC_lexicon_tags_17 },
    { 0x49d00cfc, SUC_lexicon_tags_25 },
    { 0x70700cf8, SUC_lexicon_tags_24 },
    { 0x8f900cfd, SUC_lexicon_tags_87 },
    { 0x37500cf5, SUC_lexicon_tags_15 },
    { 0x49900d00, SUC_lexicon_tags_68 },
    { 0xd7100d01, SUC_lexicon_tags_27 },
    { 0x2900cf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f00d09, SUC_lexicon_tags_46 },
    { 0xa0b00d0a, SUC_lexicon_tags_7 },
    { 0x78b00d0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa6b00d0d, SUC_lexicon_tags_16 },
    { 0x34b00d0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4500d12, SUC_lexicon_tags_19 },
    { 0xa3100d13, SUC_lexicon_tags_37 },
    { 0x92f00d13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15300d16, SUC_lexicon_tags_63 },
    { 0x72300d17, SUC_lexicon_tags_7 },
    { 0x6e100d17, SUC_lexicon_tags_1 },
    { 0x74900d19, SUC_lexicon_tags_37 },
    { 0xc1900d18, SUC_lexicon_tags_25 },
    { 0xfb900d1b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe1d00d1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b100d1f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf00d26, SUC_lexicon_tags_45 },
    { 0x62b00d27, SUC_lexicon_tags_2 },
    { 0x56900d26, SUC_lexicon_tags_62 },
    { 0x8e100d27, SUC_lexicon_tags_26 },
    { 0x6fd00d26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b00d2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e900d30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdab00d32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72500d36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a700d3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b00d44, SUC_lexicon_tags_47 },
    { 0x24900d45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb300d47, SUC_lexicon_tags_17 },
    { 0x73700d48, SUC_lexicon_tags_17 },
    { 0x97500d49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4500d51, SUC_lexicon_tags_55 },
    { 0x20300d52, SUC_lexicon_tags_3 },
    { 0x75900d53, SUC_lexicon_tags_37 },
    { 0xbeb00d54, SUC_lexicon_tags_12 },
    { 0xef700d51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55500d5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e100d5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c100d65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4f00d67, SUC_lexicon_tags_47 },
    { 0x21f00d68, SUC_lexicon_tags_17 },
    { 0x18500d67, SUC_lexicon_tags_47 },
    { 0x22f00d69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x59500d6d, SUC_lexicon_tags_17 },
    { 0xf1500d6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f00d72, SUC_lexicon_tags_11 },
    { 0x4c100d72, SUC_lexicon_tags_0 },
    { 0x1a100d74, SUC_lexicon_tags_7 },
    { 0xc8b00d74, SUC_lexicon_tags_12 },
    { 0x51500d73, SUC_lexicon_tags_56 },
    { 0x3c500d75, SUC_lexicon_tags_37 },
    { 0x7f500d74, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9eb00d7a, SUC_lexicon_tags_85 },
    { 0xe9f00d7b, SUC_lexicon_tags_33 },
    { 0x9300d7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac500d82, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92700d89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde300d8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfe300d8e, SUC_lexicon_tags_11 },
    { 0x83b00d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44500d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x300d97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d00d99, SUC_lexicon_tags_68 },
    { 0x87100d9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcb00d9c, SUC_lexicon_tags_25 },
    { 0x57d00d9c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x52900da0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe7900da2, SUC_lexicon_tags_32 },
    { 0x5a900da3, SUC_lexicon_tags_7 },
    { 0x56300da3, SUC_lexicon_tags_37 },
    { 0xd4b00da2, SUC_lexicon_tags_65 },
    { 0xd4700da3, SUC_lexicon_tags_61 },
    { 0x1d100da7, SUC_lexicon_tags_14 },
    { 0xf1d00da8, SUC_lexicon_tags_2 },
    { 0x2d500da9, SUC_lexicon_tags_2 },
    { 0x19b00da8, SUC_lexicon_tags_45 },
    { 0x22d00da4, SUC_lexicon_tags_25 },
    { 0x43900dac, SUC_lexicon_tags_8 },
    { 0x5f100dad, SUC_lexicon_tags_12 },
    { 0xd100dae, SUC_lexicon_tags_45 },
    { 0xfef00dac, SUC_lexicon_tags_26 },
    { 0x88700db0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4900db3, SUC_lexicon_tags_38 },
    { 0x6e700db4, SUC_lexicon_tags_12 },
    { 0x8d500db5, SUC_lexicon_tags_14 },
    { 0x28700db6, SUC_lexicon_tags_6 },
    { 0xbb00db5, SUC_lexicon_tags_37 },
    { 0xe6b00db3, SUC_lexicon_tags_55 },
    { 0xf100db7, SUC_lexicon_tags_26 },
    { 0x6100dba, SUC_lexicon_tags_25 },
    { 0xdc300dba, SUC_lexicon_tags_11 },
    { 0x8f300dbb, SUC_lexicon_tags_44 },
    { 0x4f500dba, SUC_lexicon_tags_57 },
    { 0x44d00db9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95700dc0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a300dc4, SUC_lexicon_tags_37 },
    { 0x94900dc5, SUC_lexicon_tags_12 },
    { 0x7bf00dc6, SUC_lexicon_tags_19 },
    { 0x63900dc7, SUC_lexicon_tags_12 },
    { 0x85500dc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db00dcb, SUC_lexicon_tags_13 },
    { 0x40d00dcb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b700dd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43300dd2, SUC_lexicon_tags_26 },
    { 0x87f00dd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28500dd5, SUC_lexicon_tags_33 },
    { 0xf4900dd6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0300ddb, SUC_lexicon_tags_15 },
    { 0xc4100ddb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29500de0, SUC_lexicon_tags_33 },
    { 0x93500de1, SUC_lexicon_tags_11 },
    { 0x21100de1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc4b00de4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb00de8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd00deb, SUC_lexicon_tags_77 },
    { 0x6bf00dec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb00def, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab00df2, SUC_lexicon_tags_17 },
    { 0xe0d00df3, SUC_lexicon_tags_25 },
    { 0x4d100df4, SUC_lexicon_tags_12 },
    { 0x44900df2, SUC_lexicon_tags_37 },
    { 0x74d00df5, SUC_lexicon_tags_11 },
    { 0xe8b00df2, SUC_lexicon_tags_25 },
    { 0xfdd00df3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65500dfb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1100dfe, SUC_lexicon_tags_113 },
    { 0xb8300dfe, SUC_lexicon_tags_10 },
    { 0xef500dff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38900e02, SUC_lexicon_tags_11 },
    { 0x37700e03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a500e0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40b00e0d, SUC_lexicon_tags_63 },
    { 0x61f00e0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d00e12, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b00e15, SUC_lexicon_tags_2 },
    { 0xd4100e16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11500e18, SUC_lexicon_tags_25 },
    { 0xc7f00e18, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f700e1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2900e20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0100e26, SUC_lexicon_tags_37 },
    { 0x52700e26, SUC_lexicon_tags_10 },
    { 0x25d00e28, SUC_lexicon_tags_25 },
    { 0x93500e26, SUC_lexicon_tags_29 },
    { 0x89f00e2a, SUC_lexicon_tags_37 },
    { 0xbdd00e2b, SUC_lexicon_tags_30 },
    { 0x3a500e2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80100e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1300e31, SUC_lexicon_tags_12 },
    { 0x1dd00e31, SUC_lexicon_tags_23 },
    { 0xa1300e33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e700e37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4300e39, SUC_lexicon_tags_3 },
    { 0xaf00e3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25300e3e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb00e42, SUC_lexicon_tags_12 },
    { 0x9f300e42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded00e47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f00e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb00e56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e300e5a, SUC_lexicon_tags_15 },
    { 0xb7900e5a, SUC_lexicon_tags_1 },
    { 0xa0700e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2700e5e, SUC_lexicon_tags_12 },
    { 0xf5500e5f, SUC_lexicon_tags_1 },
    { 0x4c500e60, SUC_lexicon_tags_26 },
    { 0x71d00e61, SUC_lexicon_tags_15 },
    { 0x95500e62, SUC_lexicon_tags_15 },
    { 0x7b300e63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9a100e65, SUC_lexicon_tags_68 },
    { 0xa4b00e66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e700e68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b500e6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8300e6e, SUC_lexicon_tags_127 },
    { 0xc4500e6f, SUC_lexicon_tags_47 },
    { 0xdbf00e70, SUC_lexicon_tags_57 },
    { 0x16500e70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab100e75, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9cb00e77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x32900e7a, SUC_lexicon_tags_12 },
    { 0x7af00e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8300e7e, SUC_lexicon_tags_25 },
    { 0x9bf00e7f, SUC_lexicon_tags_15 },
    { 0x61900e80, SUC_lexicon_tags_12 },
    { 0xbeb00e81, SUC_lexicon_tags_47 },
    { 0xa7100e7f, SUC_lexicon_tags_1 },
    { 0xbbb00e83, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d00e89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x51b00e8b, SUC_lexicon_tags_9 },
    { 0x39f00e8c, SUC_lexicon_tags_37 },
    { 0xdb900e8d, SUC_lexicon_tags_77 },
    { 0x78300e8e, SUC_lexicon_tags_2 },
    { 0x25100e8d, SUC_lexicon_tags_32 },
    { 0x7f00e8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e900e93, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5f100e95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d00e99, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0700e9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x12300e9f, SUC_lexicon_tags_7 },
    { 0x69100ea0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67500ea2, SUC_lexicon_tags_14 },
    { 0x27f00ea3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3af00ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabf00ea7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f100eaa, SUC_lexicon_tags_113 },
    { 0x71900eab, SUC_lexicon_tags_25 },
    { 0x30f00eac, SUC_lexicon_tags_63 },
    { 0x6eb00ead, SUC_lexicon_tags_87 },
    { 0xaf100eac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3300eb0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b00eb3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f00ec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b00ec4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f100ec7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcf00ec9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xccd00ecb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x76100ece, SUC_lexicon_tags_8 },
    { 0xbad00ece, SUC_lexicon_tags_92 },
    { 0x61500ed0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f300ed3, SUC_lexicon_tags_68 },
    { 0x6d100ed3, SUC_lexicon_tags_17 },
    { 0x25300ed5, SUC_lexicon_tags_38 },
    { 0x1d100ed6, SUC_lexicon_tags_6 },
    { 0x58100ed7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd00edc, SUC_lexicon_tags_11 },
    { 0x8c900edd, SUC_lexicon_tags_25 },
    { 0xae700edd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8900ee1, SUC_lexicon_tags_117 },
    { 0xdab00ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab900ee8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12300eec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f00eef, SUC_lexicon_tags_9 },
    { 0x37100ef0, SUC_lexicon_tags_25 },
    { 0x1500ef0, SUC_lexicon_tags_37 },
    { 0x19900ef0, SUC_lexicon_tags_10 },
    { 0xd1500ef3, SUC_lexicon_tags_17 },
    { 0xb1300ef3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaad00ef6, SUC_lexicon_tags_31 },
    { 0x1a100ef6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa100ef9, SUC_lexicon_tags_17 },
    { 0x30100ef9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb00efd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f700f00, SUC_lexicon_tags_0 },
    { 0x17900f00, SUC_lexicon_tags_15 },
    { 0x1fd00f02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf00f05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88100f0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xead00f0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3900f0f, SUC_lexicon_tags_7 },
    { 0xaf500f0f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x81d00f12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x11500f14, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb00f19, SUC_lexicon_tags_6 },
    { 0x21f00f1a, SUC_lexicon_tags_2 },
    { 0x61100f1a, SUC_lexicon_tags_37 },
    { 0x64300f1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74500f1e, SUC_lexicon_tags_14 },
    { 0xe3700f1e, SUC_lexicon_tags_15 },
    { 0x15b00f20, SUC_lexicon_tags_17 },
    { 0x7a100f21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3300f2a, SUC_lexicon_tags_23 },
    { 0xedd00f2b, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf500f30, SUC_lexicon_tags_25 },
    { 0x5b700f31, SUC_lexicon_tags_25 },
    { 0xa7700f32, SUC_lexicon_tags_28 },
    { 0x9ef00f33, SUC_lexicon_tags_0 },
    { 0x69f00f34, SUC_lexicon_tags_2 },
    { 0xfb00f35, SUC_lexicon_tags_11 },
    { 0x76300f36, SUC_lexicon_tags_11 },
    { 0x18700f36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x67500f39, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x13100f3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1700f40, SUC_lexicon_tags_27 },
    { 0x9b700f41, SUC_lexicon_tags_5 },
    { 0xd5b00f41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63100f46, SUC_lexicon_tags_15 },
    { 0xd4f00f46, SUC_lexicon_tags_26 },
    { 0xac100f48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab00f4b, SUC_lexicon_tags_12 },
    { 0x7d500f4c, SUC_lexicon_tags_63 },
    { 0xec900f4c, SUC_lexicon_tags_28 },
    { 0xd3f00f4d, SUC_lexicon_tags_2 },
    { 0xe4900f4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x88500f51, SUC_lexicon_tags_1 },
    { 0x6e900f52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b00f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69900f5f, SUC_lexicon_tags_12 },
    { 0xaad00f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61900f65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d00f68, SUC_lexicon_tags_70 },
    { 0xfa500f69, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xde900f6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64700f6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a300f71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde300f75, SUC_lexicon_tags_69 },
    { 0x2b500f76, SUC_lexicon_tags_25 },
    { 0x4df00f77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb00f7b, SUC_lexicon_tags_46 },
    { 0xe2d00f7c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xf3900f7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f00f81, SUC_lexicon_tags_37 },
    { 0xf9300f81, SUC_lexicon_tags_11 },
    { 0xcf700f83, SUC_lexicon_tags_14 },
    { 0xfb300f84, SUC_lexicon_tags_54 },
    { 0x28f00f81, SUC_lexicon_tags_26 },
    { 0x6f900f86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f00f89, SUC_lexicon_tags_16 },
    { 0x1bd00f8a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7b900f8c, SUC_lexicon_tags_11 },
    { 0x24700f8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b00f91, SUC_lexicon_tags_262 },
    { 0x84900f92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f00f9a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b00f9d, SUC_lexicon_tags_59 },
    { 0x68500f9e, SUC_lexicon_tags_37 },
    { 0x70f00f9f, SUC_lexicon_tags_23 },
    { 0xced00fa0, SUC_lexicon_tags_0 },
    { 0x2ed00fa0, SUC_lexicon_tags_2 },
    { 0xa4500f9f, SUC_lexicon_tags_79 },
    { 0x67300fa3, SUC_lexicon_tags_25 },
    { 0x39f00fa4, SUC_lexicon_tags_6 },
    { 0x74b00fa4, SUC_lexicon_tags_1 },
    { 0x1a300fa4, SUC_lexicon_tags_1 },
    { 0xac100fa3, SUC_lexicon_tags_49 },
    { 0xc7100fa3, SUC_lexicon_tags_17 },
    { 0x22d00fa9, SUC_lexicon_tags_43 },
    { 0x78f00fa9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f300fad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff00fb4, SUC_lexicon_tags_2 },
    { 0xa8f00fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39700fb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x27100fbc, SUC_lexicon_tags_7 },
    { 0x8cf00fbd, SUC_lexicon_tags_45 },
    { 0x61100fbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c900fc6, SUC_lexicon_tags_37 },
    { 0xd8100fc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12100fcb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef00fce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda100fd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9500fd5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b00fdd, SUC_lexicon_tags_1662 },
    { 0x8d500fde, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd500fe3, SUC_lexicon_tags_66 },
    { 0xd2700fe4, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2100fe7, SUC_lexicon_tags_35 },
    { 0x48f00fe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44900feb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x31900fed, SUC_lexicon_tags_17 },
    { 0xb2300fed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8d00ff0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f00ff3, SUC_lexicon_tags_28 },
    { 0x93b00ff4, SUC_lexicon_tags_108 },
    { 0x94f00ff5, SUC_lexicon_tags_37 },
    { 0xec300ff6, SUC_lexicon_tags_23 },
    { 0x69d00ff6, SUC_lexicon_tags_37 },
    { 0x22f00ff7, SUC_lexicon_tags_11 },
    { 0x20d00ff5, SUC_lexicon_tags_1 },
    { 0x53d00ffa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58500ffc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x63b00ffe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d01003, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0100b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac30100e, SUC_lexicon_tags_26 },
    { 0x1b30100f, SUC_lexicon_tags_37 },
    { 0x47901010, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b01015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5901017, SUC_lexicon_tags_17 },
    { 0x45d01018, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2610101c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16701021, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb01023, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7901025, SUC_lexicon_tags_6 },
    { 0x39501025, SUC_lexicon_tags_12 },
    { 0xd8b01027, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f0102c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb430102e, SUC_lexicon_tags_2 },
    { 0x61f0102f, SUC_lexicon_tags_37 },
    { 0x4b301030, SUC_lexicon_tags_25 },
    { 0x54101031, SUC_lexicon_tags_14 },
    { 0x8b01032, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b901035, SUC_lexicon_tags_12 },
    { 0x30f01036, SUC_lexicon_tags_47 },
    { 0x78f01037, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f0103a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5a10103c, SUC_lexicon_tags_17 },
    { 0x8210103d, SUC_lexicon_tags_3 },
    { 0xbad0103d, SUC_lexicon_tags_11 },
    { 0x87b0103e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8701043, SUC_lexicon_tags_26 },
    { 0x64501043, SUC_lexicon_tags_28 },
    { 0x48b01045, SUC_lexicon_tags_15 },
    { 0x70f01046, SUC_lexicon_tags_8 },
    { 0xb5d01047, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62d01049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80901050, SUC_lexicon_tags_273 },
    { 0xfb701050, SUC_lexicon_tags_1 },
    { 0xe1b01052, SUC_lexicon_tags_17 },
    { 0x80f01053, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3301057, SUC_lexicon_tags_26 },
    { 0xb5701058, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa450105c, SUC_lexicon_tags_2 },
    { 0x23b0105d, SUC_lexicon_tags_29 },
    { 0x5e10105d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xced01061, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a901064, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e901066, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad0106c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd50106e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce901074, SUC_lexicon_tags_11 },
    { 0xfa301075, SUC_lexicon_tags_12 },
    { 0x74901075, SUC_lexicon_tags_7 },
    { 0x49901075, SUC_lexicon_tags_31 },
    { 0xc4101078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d0107c, SUC_lexicon_tags_47 },
    { 0x9230107d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53301081, SUC_lexicon_tags_21 },
    { 0x3b701082, SUC_lexicon_tags_37 },
    { 0x94901082, SUC_lexicon_tags_47 },
    { 0x81501083, SUC_lexicon_tags_12 },
    { 0xa0f01084, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd650108b, SUC_lexicon_tags_31 },
    { 0xa3d0108b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61501091, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90701096, SUC_lexicon_tags_16 },
    { 0xdd301097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d0109d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa10109f, SUC_lexicon_tags_43 },
    { 0x3710109f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x70f010a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf13010a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe25010a6, SUC_lexicon_tags_7 },
    { 0xc7b010a7, SUC_lexicon_tags_7 },
    { 0xb6f010a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d010ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d010b5, SUC_lexicon_tags_1 },
    { 0xdcb010b6, SUC_lexicon_tags_11 },
    { 0x8db010b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x377010b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2d010bb, SUC_lexicon_tags_55 },
    { 0xecd010bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b010bf, SUC_lexicon_tags_70 },
    { 0xc5d010c0, SUC_lexicon_tags_12 },
    { 0x7010c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7010c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ed010c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe89010c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5010cc, SUC_lexicon_tags_31 },
    { 0xf3f010cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177010d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x509010db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703010e2, SUC_lexicon_tags_39 },
    { 0x669010e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96b010e5, SUC_lexicon_tags_21 },
    { 0xe23010e5, SUC_lexicon_tags_7 },
    { 0xeef010e6, SUC_lexicon_tags_47 },
    { 0xbc9010e8, SUC_lexicon_tags_1 },
    { 0xb87010e9, SUC_lexicon_tags_11 },
    { 0x24b010ea, SUC_lexicon_tags_33 },
    { 0xd73010eb, SUC_lexicon_tags_43 },
    { 0xbc3010ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303010f3, SUC_lexicon_tags_136 },
    { 0x5d7010f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fd010f6, SUC_lexicon_tags_23 },
    { 0x585010f6, SUC_lexicon_tags_69 },
    { 0xacd010f6, SUC_lexicon_tags_77 },
    { 0x8ad010f9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xec3010fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae101104, SUC_lexicon_tags_60 },
    { 0xd7101105, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4901107, SUC_lexicon_tags_14 },
    { 0xe1701108, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8310110d, SUC_lexicon_tags_29 },
    { 0x2790110e, SUC_lexicon_tags_49 },
    { 0x1670110f, SUC_lexicon_tags_112 },
    { 0x69b01110, SUC_lexicon_tags_11 },
    { 0xd1301111, SUC_lexicon_tags_35 },
    { 0x50501111, SUC_lexicon_tags_60 },
    { 0x87701112, SUC_lexicon_tags_11 },
    { 0xcdf01114, SUC_lexicon_tags_43 },
    { 0x55d01115, SUC_lexicon_tags_14 },
    { 0x61b01116, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e50111b, SUC_lexicon_tags_14 },
    { 0x2850111b, SUC_lexicon_tags_37 },
    { 0xb9b0111c, SUC_lexicon_tags_105 },
    { 0xa6d0111c, SUC_lexicon_tags_25 },
    { 0xf550111f, SUC_lexicon_tags_47 },
    { 0x3090111f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd501126, SUC_lexicon_tags_33 },
    { 0xc2901127, SUC_lexicon_tags_7 },
    { 0x66701128, SUC_lexicon_tags_63 },
    { 0x36301129, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f0112c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf50112f, SUC_lexicon_tags_6 },
    { 0x64701130, SUC_lexicon_tags_37 },
    { 0x45101130, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab101133, SUC_lexicon_tags_68 },
    { 0x71101134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d01137, SUC_lexicon_tags_65 },
    { 0x4b01137, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x85d0113a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf01141, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3270114c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5310114e, SUC_lexicon_tags_7 },
    { 0xafd0114e, SUC_lexicon_tags_46 },
    { 0x92901150, SUC_lexicon_tags_55 },
    { 0x9f901150, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5701154, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb6b01156, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6190115e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf01168, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b0116b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b0116e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d01171, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7c301173, SUC_lexicon_tags_25 },
    { 0xd4301173, SUC_lexicon_tags_38 },
    { 0x6d101175, SUC_lexicon_tags_25 },
    { 0xed101176, SUC_lexicon_tags_23 },
    { 0xa0301177, SUC_lexicon_tags_45 },
    { 0x5e701175, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29501180, SUC_lexicon_tags_37 },
    { 0x4bd01180, SUC_lexicon_tags_7 },
    { 0x2101180, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb701184, SUC_lexicon_tags_54 },
    { 0x26d01185, SUC_lexicon_tags_8 },
    { 0xfef01186, SUC_lexicon_tags_0 },
    { 0xc8101186, SUC_lexicon_tags_2 },
    { 0x26301188, SUC_lexicon_tags_15 },
    { 0x99b01189, SUC_lexicon_tags_12 },
    { 0x99301186, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d0118f, SUC_lexicon_tags_19 },
    { 0xc1101190, SUC_lexicon_tags_38 },
    { 0x37701191, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49701194, SUC_lexicon_tags_12 },
    { 0x20701195, SUC_lexicon_tags_43 },
    { 0x13f01196, SUC_lexicon_tags_47 },
    { 0xb3501194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57011a0, SUC_lexicon_tags_46 },
    { 0x7b5011a1, SUC_lexicon_tags_60 },
    { 0xdc3011a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4d011a4, SUC_lexicon_tags_47 },
    { 0xfa9011a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b011a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x253011ab, SUC_lexicon_tags_15 },
    { 0x9b9011ac, SUC_lexicon_tags_25 },
    { 0xa61011ad, SUC_lexicon_tags_27 },
    { 0x6e1011ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11011b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad011b8, SUC_lexicon_tags_23 },
    { 0x32f011b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05011bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc07011bf, SUC_lexicon_tags_32 },
    { 0x25b011c0, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1011c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447011ce, SUC_lexicon_tags_17 },
    { 0x2a1011cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9bb011d1, SUC_lexicon_tags_7 },
    { 0x327011d2, SUC_lexicon_tags_25 },
    { 0x917011d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09011d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b011da, SUC_lexicon_tags_15 },
    { 0x35011db, SUC_lexicon_tags_17 },
    { 0x4db011da, SUC_lexicon_tags_146 },
    { 0x947011da, SUC_lexicon_tags_27 },
    { 0xb5f011de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127011e6, SUC_lexicon_tags_1 },
    { 0xc3d011e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9011ed, SUC_lexicon_tags_30 },
    { 0x563011ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95011f2, SUC_lexicon_tags_47 },
    { 0xf3011f3, SUC_lexicon_tags_59 },
    { 0x599011f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1011ff, SUC_lexicon_tags_37 },
    { 0xc7501200, SUC_lexicon_tags_14 },
    { 0x73901200, SUC_lexicon_tags_3 },
    { 0x27f01202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5d01204, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f01208, SUC_lexicon_tags_8 },
    { 0xbe501209, SUC_lexicon_tags_69 },
    { 0x2150120a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe030120c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61501215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63f01217, SUC_lexicon_tags_25 },
    { 0x6df01217, SUC_lexicon_tags_2 },
    { 0xa5d01219, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d0121c, SUC_lexicon_tags_63 },
    { 0xcd10121c, SUC_lexicon_tags_7 },
    { 0xac30121e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb01222, SUC_lexicon_tags_4 },
    { 0x75301222, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b01226, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ed01228, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e50122c, SUC_lexicon_tags_8 },
    { 0xdcf0122d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a90122f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5dd01231, SUC_lexicon_tags_37 },
    { 0x9ed01231, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89501234, SUC_lexicon_tags_57 },
    { 0xba701235, SUC_lexicon_tags_25 },
    { 0x59301236, SUC_lexicon_tags_11 },
    { 0x26101236, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b0123d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15301241, SUC_lexicon_tags_53 },
    { 0x8a501241, SUC_lexicon_tags_11 },
    { 0x4701243, SUC_lexicon_tags_35 },
    { 0xe501243, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1501246, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9750124a, SUC_lexicon_tags_1 },
    { 0x9ad0124a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd01250, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x86101252, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd301257, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2670125c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf330125e, SUC_lexicon_tags_60 },
    { 0x8a30125f, SUC_lexicon_tags_10 },
    { 0x5bf0125e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ed01262, SUC_lexicon_tags_10 },
    { 0xbdb01263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf01266, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa101269, SUC_lexicon_tags_55 },
    { 0x84b0126a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd370126c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf950126e, SUC_lexicon_tags_82 },
    { 0x8f70126e, SUC_lexicon_tags_49 },
    { 0x3e301270, SUC_lexicon_tags_87 },
    { 0xde301270, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0x21301274, SUC_lexicon_tags_77 },
    { 0xbb01275, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e901278, SUC_lexicon_tags_63 },
    { 0x7901278, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf870127b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7550127e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x44d01280, SUC_lexicon_tags_12 },
    { 0x5bf01281, SUC_lexicon_tags_48 },
    { 0x98f01282, SUC_lexicon_tags_1 },
    { 0xeeb01280, SUC_lexicon_tags_37 },
    { 0x29f01284, SUC_lexicon_tags_25 },
    { 0xf0901285, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3830128b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa430128e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79b01290, SUC_lexicon_tags_61 },
    { 0x93701291, SUC_lexicon_tags_31 },
    { 0xa5d01291, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d50129a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe950129e, SUC_lexicon_tags_12 },
    { 0xb890129f, SUC_lexicon_tags_27 },
    { 0x42b0129f, SUC_lexicon_tags_16 },
    { 0x3012a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9012a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc03012a6, SUC_lexicon_tags_12 },
    { 0x63d012a7, SUC_lexicon_tags_11 },
    { 0x357012a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda3012aa, SUC_lexicon_tags_2 },
    { 0x83b012aa, SUC_lexicon_tags_15 },
    { 0x337012ab, SUC_lexicon_tags_17 },
    { 0xa41012ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333012b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cd012b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7012b6, SUC_lexicon_tags_11 },
    { 0x253012b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd012bb, SUC_lexicon_tags_7 },
    { 0x25012bb, SUC_lexicon_tags_7 },
    { 0xfe3012bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x26d012bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939012c7, SUC_lexicon_tags_38 },
    { 0xdb012c8, SUC_lexicon_tags_7 },
    { 0xd15012c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc87012cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3012ce, SUC_lexicon_tags_25 },
    { 0x6c3012cf, SUC_lexicon_tags_9 },
    { 0x9dd012d0, SUC_lexicon_tags_11 },
    { 0xa01012d1, SUC_lexicon_tags_33 },
    { 0xb1d012d1, SUC_lexicon_tags_9 },
    { 0x791012ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1fb012d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d012dd, SUC_lexicon_tags_0 },
    { 0x417012dd, SUC_lexicon_tags_17 },
    { 0x665012df, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd69012e1, SUC_lexicon_tags_43 },
    { 0x559012e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee1012e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21012e9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x51d012eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791012f0, SUC_lexicon_tags_25 },
    { 0xa9d012f1, SUC_lexicon_tags_1 },
    { 0x3a7012f1, SUC_lexicon_tags_17 },
    { 0x3c5012f2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f012f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a1012f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a301300, SUC_lexicon_tags_14 },
    { 0x72701301, SUC_lexicon_tags_158 },
    { 0x54b01302, SUC_lexicon_tags_55 },
    { 0x73701302, SUC_lexicon_tags_7 },
    { 0x68901304, SUC_lexicon_tags_2 },
    { 0x58d01304, SUC_lexicon_tags_79 },
    { 0x75101306, SUC_lexicon_tags_61 },
    { 0xd3901306, SUC_lexicon_tags_25 },
    { 0xb4301304, SUC_lexicon_tags_15 },
    { 0x10701309, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f0130c, SUC_lexicon_tags_0 },
    { 0xdd90130c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x30501310, SUC_lexicon_tags_27 },
    { 0xdaf01311, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16101316, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf930131a, SUC_lexicon_tags_45 },
    { 0xf310131b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97b0131d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5501323, SUC_lexicon_tags_34 },
    { 0x25f01324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72701328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d0132b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91501330, SUC_lexicon_tags_11 },
    { 0xa1301331, SUC_lexicon_tags_19 },
    { 0xbe701332, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d01336, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b901338, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e50133a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7d10133c, SUC_lexicon_tags_11 },
    { 0x31d0133d, SUC_lexicon_tags_15 },
    { 0xfbf0133e, SUC_lexicon_tags_1 },
    { 0xbf10133f, SUC_lexicon_tags_33 },
    { 0x51301340, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12301343, SUC_lexicon_tags_56 },
    { 0xad301344, SUC_lexicon_tags_2 },
    { 0xea101344, SUC_lexicon_tags_14 },
    { 0x1a101346, SUC_lexicon_tags_56 },
    { 0xfe101343, SUC_lexicon_tags_37 },
    { 0x88f01344, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a30134b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaef0134d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a50134f, SUC_lexicon_tags_84 },
    { 0xcc301350, SUC_lexicon_tags_34 },
    { 0x7d501351, SUC_lexicon_tags_17 },
    { 0x38b01352, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c901358, SUC_lexicon_tags_22 },
    { 0x9701359, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2090135b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b301368, SUC_lexicon_tags_33 },
    { 0xfcb01369, SUC_lexicon_tags_2 },
    { 0x4ff01368, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77b0136c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd990136e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3501375, SUC_lexicon_tags_83 },
    { 0xb2b01376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1701378, SUC_lexicon_tags_343 },
    { 0x7c701379, SUC_lexicon_tags_124 },
    { 0xde30137a, SUC_lexicon_tags_1 },
    { 0xb6b0137a, SUC_lexicon_tags_43 },
    { 0x3d10137a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf690137f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a701382, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f01385, SUC_lexicon_tags_2 },
    { 0x28d01386, SUC_lexicon_tags_31 },
    { 0x4cf01387, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3df01389, SUC_lexicon_tags_47 },
    { 0xa710138a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe690138f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5501393, SUC_lexicon_tags_6 },
    { 0x3a101393, SUC_lexicon_tags_31 },
    { 0x1d101395, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf5101397, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8a701399, SUC_lexicon_tags_16 },
    { 0x57901399, SUC_lexicon_tags_17 },
    { 0x2d90139b, SUC_lexicon_tags_15 },
    { 0x4090139c, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x8150139e, SUC_lexicon_tags_38 },
    { 0x11d0139f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x123013a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971013a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1013af, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb9d013b1, SUC_lexicon_tags_26 },
    { 0xf9f013b2, SUC_lexicon_tags_63 },
    { 0xbe3013b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe49013b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc15013b7, SUC_lexicon_tags_1 },
    { 0x33b013b7, SUC_lexicon_tags_4 },
    { 0x2dd013b9, SUC_lexicon_tags_25 },
    { 0xad9013b9, SUC_lexicon_tags_12 },
    { 0xe95013bb, SUC_lexicon_tags_39 },
    { 0x75f013b9, SUC_lexicon_tags_65 },
    { 0xf37013b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x881013bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb51013c1, SUC_lexicon_tags_15 },
    { 0xffd013c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc45013c4, SUC_lexicon_tags_15 },
    { 0x503013c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89013c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x641013cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf013ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3013d2, SUC_lexicon_tags_60 },
    { 0xbc7013d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x231013d5, SUC_lexicon_tags_27 },
    { 0x265013d6, SUC_lexicon_tags_68 },
    { 0x7e1013d7, SUC_lexicon_tags_8 },
    { 0x457013d8, SUC_lexicon_tags_57 },
    { 0x6e1013d7, SUC_lexicon_tags_0 },
    { 0x483013d7, SUC_lexicon_tags_11 },
    { 0xf99013da, SUC_lexicon_tags_25 },
    { 0x517013dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed013e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef013e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19013e7, SUC_lexicon_tags_31 },
    { 0xfed013e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1bf013ea, SUC_lexicon_tags_6 },
    { 0xc1f013ea, SUC_lexicon_tags_37 },
    { 0x613013eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b013f0, SUC_lexicon_tags_17 },
    { 0x43b013f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7013f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807013fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f013fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17701400, SUC_lexicon_tags_0 },
    { 0xe2f01401, SUC_lexicon_tags_6 },
    { 0xe4501400, SUC_lexicon_tags_10 },
    { 0xd5101403, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b01406, SUC_lexicon_tags_38 },
    { 0x69d01407, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70140a, SUC_lexicon_tags_37 },
    { 0xc990140a, SUC_lexicon_tags_0 },
    { 0x23d0140a, SUC_lexicon_tags_37 },
    { 0x4f10140d, SUC_lexicon_tags_84 },
    { 0x3750140e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2901413, SUC_lexicon_tags_2 },
    { 0xa2701414, SUC_lexicon_tags_1 },
    { 0x78701415, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37b01417, SUC_lexicon_tags_27 },
    { 0xedd01418, SUC_lexicon_tags_39 },
    { 0x58701418, SUC_lexicon_tags_14 },
    { 0xdcf0141a, SUC_lexicon_tags_27 },
    { 0xe250141a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8190141d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18b0141f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5501422, SUC_lexicon_tags_1 },
    { 0xf4701423, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x34d01425, SUC_lexicon_tags_19 },
    { 0x4a701425, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6350142a, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910142f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a701436, SUC_lexicon_tags_25 },
    { 0xe9b01436, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3690143d, SUC_lexicon_tags_656 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb01440, SUC_lexicon_tags_12 },
    { 0x89301440, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf2b01443, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf190144a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf50144c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf0144f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf301454, SUC_lexicon_tags_12 },
    { 0xd6301455, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef0145a, SUC_lexicon_tags_31 },
    { 0x2d90145b, SUC_lexicon_tags_15 },
    { 0xc7d0145c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeed0145e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3101461, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70901463, SUC_lexicon_tags_38 },
    { 0x5cd01463, SUC_lexicon_tags_60 },
    { 0x84701463, SUC_lexicon_tags_21 },
    { 0x8d901466, SUC_lexicon_tags_15 },
    { 0xe4101467, SUC_lexicon_tags_0 },
    { 0x7e101468, SUC_lexicon_tags_32 },
    { 0xd9901469, SUC_lexicon_tags_47 },
    { 0xc190146a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd230146e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b01474, SUC_lexicon_tags_60 },
    { 0x6a901475, SUC_lexicon_tags_14 },
    { 0x76b01474, SUC_lexicon_tags_2 },
    { 0x7a901476, SUC_lexicon_tags_37 },
    { 0x91901478, SUC_lexicon_tags_17 },
    { 0xe301476, SUC_lexicon_tags_14 },
    { 0x23d0147a, SUC_lexicon_tags_23 },
    { 0xfd30147b, SUC_lexicon_tags_27 },
    { 0x3f10147c, SUC_lexicon_tags_25 },
    { 0x5dd0147a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x67d0147f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9cb01481, SUC_lexicon_tags_45 },
    { 0xf8701482, SUC_lexicon_tags_12 },
    { 0xb6501482, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef01487, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcab01489, SUC_lexicon_tags_162 },
    { 0x8550148a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f50148c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfe90148e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3701490, SUC_lexicon_tags_15 },
    { 0xaf101491, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c101494, SUC_lexicon_tags_17 },
    { 0x63501494, SUC_lexicon_tags_11 },
    { 0xe1701495, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f0149d, SUC_lexicon_tags_19 },
    { 0x6fb0149e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb77014a0, SUC_lexicon_tags_44 },
    { 0xe73014a1, SUC_lexicon_tags_2 },
    { 0x3d9014a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6c3014a4, SUC_lexicon_tags_11 },
    { 0x863014a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb45014a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x783014ab, SUC_lexicon_tags_45 },
    { 0x8b014ac, SUC_lexicon_tags_25 },
    { 0xcc9014ab, SUC_lexicon_tags_17 },
    { 0xed7014ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9014b2, SUC_lexicon_tags_17 },
    { 0x93014b2, SUC_lexicon_tags_115 },
    { 0x439014b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f014b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93014bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b014bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7014c4, SUC_lexicon_tags_15 },
    { 0xe11014c5, SUC_lexicon_tags_28 },
    { 0x51f014c6, SUC_lexicon_tags_179 },
    { 0xebb014c7, SUC_lexicon_tags_1 },
    { 0x839014c8, SUC_lexicon_tags_11 },
    { 0xbb3014c7, SUC_lexicon_tags_138 },
    { 0xb1014c9, SUC_lexicon_tags_25 },
    { 0xa29014c9, SUC_lexicon_tags_28 },
    { 0xc11014cc, SUC_lexicon_tags_10 },
    { 0x6ad014cd, SUC_lexicon_tags_320 },
    { 0xdb7014cd, SUC_lexicon_tags_11 },
    { 0xd9b014ce, SUC_lexicon_tags_13 },
    { 0x7e7014d0, SUC_lexicon_tags_14 },
    { 0x683014cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b014d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xced014d6, SUC_lexicon_tags_25 },
    { 0x307014d6, SUC_lexicon_tags_14 },
    { 0xc31014d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd014dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d014df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x375014e1, SUC_lexicon_tags_23 },
    { 0x847014e2, SUC_lexicon_tags_7 },
    { 0x911014e2, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1014e6, SUC_lexicon_tags_7 },
    { 0x2fd014e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c3014e9, SUC_lexicon_tags_43 },
    { 0x6f014ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x453014ec, SUC_lexicon_tags_43 },
    { 0x88d014ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21014f1, SUC_lexicon_tags_84 },
    { 0xd21014f2, SUC_lexicon_tags_25 },
    { 0xf8b014f1, SUC_lexicon_tags_39 },
    { 0xac7014f3, SUC_lexicon_tags_77 },
    { 0x89d014f5, SUC_lexicon_tags_47 },
    { 0x8e1014f6, SUC_lexicon_tags_40 },
    { 0x361014f6, SUC_lexicon_tags_9 },
    { 0xf1b014f8, SUC_lexicon_tags_37 },
    { 0xb73014f9, SUC_lexicon_tags_37 },
    { 0xbb9014f5, SUC_lexicon_tags_20 },
    { 0x727014fb, SUC_lexicon_tags_62 },
    { 0xaa1014fc, SUC_lexicon_tags_17 },
    { 0xcab014fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb01505, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb1501507, SUC_lexicon_tags_2 },
    { 0x56901508, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8190150c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7d0150e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x13701510, SUC_lexicon_tags_25 },
    { 0xabd01511, SUC_lexicon_tags_17 },
    { 0xffd01512, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x60101514, SUC_lexicon_tags_2 },
    { 0x77301514, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x71b01517, SUC_lexicon_tags_37 },
    { 0x4af01518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf0151d, SUC_lexicon_tags_49 },
    { 0x9e10151e, SUC_lexicon_tags_6 },
    { 0x6bf0151d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4d01521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ab01523, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4901525, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d01528, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8770152a, SUC_lexicon_tags_2 },
    { 0xd430152b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc930152e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7701532, SUC_lexicon_tags_25 },
    { 0xb2501533, SUC_lexicon_tags_25 },
    { 0xf4101534, SUC_lexicon_tags_63 },
    { 0x18701535, SUC_lexicon_tags_11 },
    { 0x97101534, SUC_lexicon_tags_19 },
    { 0x81501534, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x54b01539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc10154b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d01551, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4f301553, SUC_lexicon_tags_11 },
    { 0x15b01554, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd701557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb101559, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f10155c, SUC_lexicon_tags_1 },
    { 0x4df0155c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5901561, SUC_lexicon_tags_25 },
    { 0xadd01562, SUC_lexicon_tags_7 },
    { 0x46101563, SUC_lexicon_tags_2 },
    { 0xb2d01561, SUC_lexicon_tags_38 },
    { 0xa9f01565, SUC_lexicon_tags_15 },
    { 0xeb301565, SUC_lexicon_tags_14 },
    { 0xaf301565, SUC_lexicon_tags_12 },
    { 0x7c101568, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3710156b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d901575, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b01579, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4610157b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a50157e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c701581, SUC_lexicon_tags_1 },
    { 0x83301581, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x98101585, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x87301587, SUC_lexicon_tags_9 },
    { 0x7f301588, SUC_lexicon_tags_17 },
    { 0x5e501589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecf0158b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd0158f, SUC_lexicon_tags_37 },
    { 0x9790158f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36901595, SUC_lexicon_tags_16 },
    { 0x1f101595, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10301599, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb430159c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc950159f, SUC_lexicon_tags_12 },
    { 0x695015a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf79015a2, SUC_lexicon_tags_37 },
    { 0x737015a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf015ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945015b2, SUC_lexicon_tags_11 },
    { 0xc45015b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9015b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb015bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b015c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x981015c2, SUC_lexicon_tags_35 },
    { 0xc1f015c3, SUC_lexicon_tags_34 },
    { 0x97d015c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b015c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63015cb, SUC_lexicon_tags_23 },
    { 0x79b015cc, SUC_lexicon_tags_6 },
    { 0x9f9015cc, SUC_lexicon_tags_14 },
    { 0x161015ce, SUC_lexicon_tags_25 },
    { 0xee5015cf, SUC_lexicon_tags_31 },
    { 0xcbd015d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01015d6, SUC_lexicon_tags_33 },
    { 0x49015d7, SUC_lexicon_tags_29 },
    { 0x8e3015d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f015dc, SUC_lexicon_tags_23 },
    { 0xed9015dd, SUC_lexicon_tags_43 },
    { 0x19d015de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13015e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881015e8, SUC_lexicon_tags_1 },
    { 0xd37015e8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1015ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab015ef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x861015f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d015f6, SUC_lexicon_tags_37 },
    { 0x353015f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761015fb, SUC_lexicon_tags_32 },
    { 0x4e1015fc, SUC_lexicon_tags_25 },
    { 0x645015fd, SUC_lexicon_tags_65 },
    { 0xe9b015fe, SUC_lexicon_tags_60 },
    { 0xbe5015ff, SUC_lexicon_tags_49 },
    { 0xbe701600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d301605, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9d01607, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a10160a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d01611, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9701614, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b701617, SUC_lexicon_tags_45 },
    { 0x6701617, SUC_lexicon_tags_1 },
    { 0x40f01618, SUC_lexicon_tags_11 },
    { 0xe2101618, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x390161d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7f0161f, SUC_lexicon_tags_17 },
    { 0x82d01620, SUC_lexicon_tags_7 },
    { 0x26b01620, SUC_lexicon_tags_12 },
    { 0xfcb01620, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5b01624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6901628, SUC_lexicon_tags_25 },
    { 0xd4901629, SUC_lexicon_tags_37 },
    { 0xb9b0162a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f10162d, SUC_lexicon_tags_37 },
    { 0x8f70162e, SUC_lexicon_tags_146 },
    { 0x8af0162d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d01632, SUC_lexicon_tags_17 },
    { 0xc6d01632, SUC_lexicon_tags_37 },
    { 0x61b01634, SUC_lexicon_tags_11 },
    { 0x2d01635, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f01639, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d301642, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd01645, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2230164c, SUC_lexicon_tags_25 },
    { 0xd730164c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff01650, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb0101652, SUC_lexicon_tags_2 },
    { 0xc3f01652, SUC_lexicon_tags_66 },
    { 0xeb701653, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40f01656, SUC_lexicon_tags_65 },
    { 0x92f01657, SUC_lexicon_tags_25 },
    { 0xcd101658, SUC_lexicon_tags_12 },
    { 0x68301656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf10165c, SUC_lexicon_tags_37 },
    { 0x6810165d, SUC_lexicon_tags_49 },
    { 0x68f0165e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f01663, SUC_lexicon_tags_26 },
    { 0xad701664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d01669, SUC_lexicon_tags_7 },
    { 0x3790166a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa990166e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49301672, SUC_lexicon_tags_38 },
    { 0x28f01673, SUC_lexicon_tags_7 },
    { 0x45b01674, SUC_lexicon_tags_25 },
    { 0x7b701675, SUC_lexicon_tags_1 },
    { 0xe9b01675, SUC_lexicon_tags_82 },
    { 0xe3d01672, SUC_lexicon_tags_7 },
    { 0xc2901678, SUC_lexicon_tags_37 },
    { 0xdad01672, SUC_lexicon_tags_1 },
    { 0x37b0167a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ad0167c, SUC_lexicon_tags_1 },
    { 0xe70167d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6901681, SUC_lexicon_tags_37 },
    { 0x6cf01681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d301685, SUC_lexicon_tags_37 },
    { 0x84b01686, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf01689, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb0168c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2ff0168e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4901691, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b01695, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6301699, SUC_lexicon_tags_8 },
    { 0x4e10169a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d016a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1016a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31016a9, SUC_lexicon_tags_63 },
    { 0xd57016aa, SUC_lexicon_tags_37 },
    { 0xf35016ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21016af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7f016b1, SUC_lexicon_tags_47 },
    { 0xb6f016b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269016b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725016c0, SUC_lexicon_tags_53 },
    { 0xabb016c1, SUC_lexicon_tags_37 },
    { 0x63f016c2, SUC_lexicon_tags_27 },
    { 0xc2f016c2, SUC_lexicon_tags_60 },
    { 0xb7016c4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xfdf016c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b016cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d016ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x253016d1, SUC_lexicon_tags_21 },
    { 0x983016d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed016d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda7016d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345016dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f016df, SUC_lexicon_tags_12 },
    { 0xb8d016df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x65016e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed016e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f016e8, SUC_lexicon_tags_48 },
    { 0xa11016e9, SUC_lexicon_tags_26 },
    { 0xeb3016ea, SUC_lexicon_tags_1 },
    { 0x841016e9, SUC_lexicon_tags_30 },
    { 0x1a9016ec, SUC_lexicon_tags_17 },
    { 0x59f016ed, SUC_lexicon_tags_40 },
    { 0x649016ee, SUC_lexicon_tags_28 },
    { 0x2fd016ed, SUC_lexicon_tags_5 },
    { 0x119016ea, SUC_lexicon_tags_12 },
    { 0x9b3016f1, SUC_lexicon_tags_30 },
    { 0x90b016eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f016f7, SUC_lexicon_tags_17 },
    { 0xf07016f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ed016fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e9016fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd99016fe, SUC_lexicon_tags_15 },
    { 0x8d5016ff, SUC_lexicon_tags_15 },
    { 0x3d9016ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab01707, SUC_lexicon_tags_17 },
    { 0x67301707, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcaf0170a, SUC_lexicon_tags_2 },
    { 0xa1b0170a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xab70170e, SUC_lexicon_tags_1 },
    { 0xfc30170f, SUC_lexicon_tags_37 },
    { 0x4c30170e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x49901712, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82101714, SUC_lexicon_tags_54 },
    { 0x8df01714, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4b01717, SUC_lexicon_tags_12 },
    { 0x8b01718, SUC_lexicon_tags_39 },
    { 0xf901718, SUC_lexicon_tags_2 },
    { 0xf8101719, SUC_lexicon_tags_11 },
    { 0xe430171b, SUC_lexicon_tags_23 },
    { 0xfc90171b, SUC_lexicon_tags_16 },
    { 0x8570171d, SUC_lexicon_tags_1 },
    { 0x95501717, SUC_lexicon_tags_19 },
    { 0x2b701719, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf7301721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86501724, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33101727, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca701731, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f01734, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d01737, SUC_lexicon_tags_17 },
    { 0xd901737, SUC_lexicon_tags_25 },
    { 0x11b01737, SUC_lexicon_tags_47 },
    { 0x67f01737, SUC_lexicon_tags_11 },
    { 0x2530173a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xec90173d, SUC_lexicon_tags_12 },
    { 0xfa10173d, SUC_lexicon_tags_3 },
    { 0x4f90173d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f01743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf701746, SUC_lexicon_tags_31 },
    { 0x78301747, SUC_lexicon_tags_48 },
    { 0x73d01748, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa850174f, SUC_lexicon_tags_19 },
    { 0x28101750, SUC_lexicon_tags_28 },
    { 0xed301751, SUC_lexicon_tags_17 },
    { 0xff101751, SUC_lexicon_tags_6 },
    { 0x29d01750, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d0175a, SUC_lexicon_tags_17 },
    { 0x930175b, SUC_lexicon_tags_66 },
    { 0x8650175c, SUC_lexicon_tags_12 },
    { 0x3cf0175d, SUC_lexicon_tags_1 },
    { 0xe5f0175e, SUC_lexicon_tags_16 },
    { 0x2f10175e, SUC_lexicon_tags_82 },
    { 0x59f01760, SUC_lexicon_tags_60 },
    { 0x330175c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e701764, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0301772, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b01775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34901778, SUC_lexicon_tags_27 },
    { 0xd7d01779, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4b0177b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f01780, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x40101782, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x81b01784, SUC_lexicon_tags_40 },
    { 0x65b01785, SUC_lexicon_tags_17 },
    { 0xf2901786, SUC_lexicon_tags_33 },
    { 0x3f301787, SUC_lexicon_tags_14 },
    { 0x1a501787, SUC_lexicon_tags_12 },
    { 0xdc701787, SUC_lexicon_tags_7 },
    { 0x9df0178a, SUC_lexicon_tags_49 },
    { 0xdf70178b, SUC_lexicon_tags_17 },
    { 0x59d01786, SUC_lexicon_tags_40 },
    { 0x4b0178d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdc90178f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5701792, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e501794, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9301799, SUC_lexicon_tags_45 },
    { 0xdb70179a, SUC_lexicon_tags_37 },
    { 0xc6d0179b, SUC_lexicon_tags_17 },
    { 0x6670179c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5017a0, SUC_lexicon_tags_34 },
    { 0xe13017a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x121017a3, SUC_lexicon_tags_11 },
    { 0x171017a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5017a9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x509017ac, SUC_lexicon_tags_26 },
    { 0xaaf017ad, SUC_lexicon_tags_12 },
    { 0xac3017ae, SUC_lexicon_tags_61 },
    { 0xe0d017af, SUC_lexicon_tags_80 },
    { 0x62f017ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1017b4, SUC_lexicon_tags_19 },
    { 0xa59017b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289017bb, SUC_lexicon_tags_84 },
    { 0xa83017bc, SUC_lexicon_tags_222 },
    { 0, NULL },
    { 0xc65017be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ab017c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97f017c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07017c5, SUC_lexicon_tags_31 },
    { 0xff017c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9f017c8, SUC_lexicon_tags_1 },
    { 0xf1d017c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b017cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd017d1, SUC_lexicon_tags_7 },
    { 0xe4f017d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1017d5, SUC_lexicon_tags_12 },
    { 0x689017d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x707017d8, SUC_lexicon_tags_38 },
    { 0x145017d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d017dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5017e3, SUC_lexicon_tags_47 },
    { 0x475017e3, SUC_lexicon_tags_15 },
    { 0x27b017e5, SUC_lexicon_tags_25 },
    { 0xf83017e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x419017e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x267017ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75017ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bf017f1, SUC_lexicon_tags_2 },
    { 0x227017f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db017f9, SUC_lexicon_tags_55 },
    { 0x48d017fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x92b017fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf37017fe, SUC_lexicon_tags_43 },
    { 0xc4b017ff, SUC_lexicon_tags_12 },
    { 0x341017ff, SUC_lexicon_tags_84 },
    { 0x8eb01801, SUC_lexicon_tags_60 },
    { 0x56b017fe, SUC_lexicon_tags_87 },
    { 0x9901802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71b01805, SUC_lexicon_tags_107 },
    { 0xf5f01806, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b01809, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb01823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0182a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b0182d, SUC_lexicon_tags_65 },
    { 0xc750182d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe701834, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f0183a, SUC_lexicon_tags_1 },
    { 0x4710183b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb0183e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf101843, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c90184c, SUC_lexicon_tags_1 },
    { 0xec90184c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60f0184f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0901857, SUC_lexicon_tags_49 },
    { 0x7fb01858, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c10185c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96f0185e, SUC_lexicon_tags_47 },
    { 0x7bf0185f, SUC_lexicon_tags_12 },
    { 0x7190185e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x52b01862, SUC_lexicon_tags_23 },
    { 0x89f01862, SUC_lexicon_tags_17 },
    { 0x84701862, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7701866, SUC_lexicon_tags_33 },
    { 0xc2701867, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd0186d, SUC_lexicon_tags_13 },
    { 0xe6f0186d, SUC_lexicon_tags_1 },
    { 0xb1b0186f, SUC_lexicon_tags_4 },
    { 0x8701870, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb901874, SUC_lexicon_tags_15 },
    { 0xde901875, SUC_lexicon_tags_16 },
    { 0xfc501876, SUC_lexicon_tags_17 },
    { 0x3b101877, SUC_lexicon_tags_17 },
    { 0x5ff01877, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd0187c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4301881, SUC_lexicon_tags_28 },
    { 0xb2701881, SUC_lexicon_tags_37 },
    { 0x96701883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24101886, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec30188b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1950188e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5501893, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49b01895, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8830189c, SUC_lexicon_tags_7 },
    { 0x1290189d, SUC_lexicon_tags_11 },
    { 0x9130189d, SUC_lexicon_tags_19 },
    { 0xdd90189f, SUC_lexicon_tags_12 },
    { 0x61f018a0, SUC_lexicon_tags_19 },
    { 0x599018a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d018a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3018ad, SUC_lexicon_tags_23 },
    { 0x923018ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b7018b0, SUC_lexicon_tags_65 },
    { 0xe39018b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe95018b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef018b7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87018bb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537018c3, SUC_lexicon_tags_2 },
    { 0x19018c3, SUC_lexicon_tags_27 },
    { 0x645018c5, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7018ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb018cd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97018e6, SUC_lexicon_tags_38 },
    { 0x7f5018e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15018ea, SUC_lexicon_tags_26 },
    { 0x173018ea, SUC_lexicon_tags_25 },
    { 0xc93018ec, SUC_lexicon_tags_55 },
    { 0xee1018ed, SUC_lexicon_tags_32 },
    { 0xb2b018ee, SUC_lexicon_tags_11 },
    { 0x35018ef, SUC_lexicon_tags_37 },
    { 0x9d7018f0, SUC_lexicon_tags_6 },
    { 0xa6b018ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7018f6, SUC_lexicon_tags_66 },
    { 0x4b1018f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5018fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57f018fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa501902, SUC_lexicon_tags_17 },
    { 0x95501903, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ed01905, SUC_lexicon_tags_38 },
    { 0x73701906, SUC_lexicon_tags_1 },
    { 0xea101907, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2f01909, SUC_lexicon_tags_12 },
    { 0xa50190a, SUC_lexicon_tags_2 },
    { 0xda901909, SUC_lexicon_tags_37 },
    { 0x9ab01909, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d01916, SUC_lexicon_tags_27 },
    { 0x25501916, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb370191d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51701921, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf01927, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24f01929, SUC_lexicon_tags_14 },
    { 0x4ef01929, SUC_lexicon_tags_1 },
    { 0x95d0192b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6270192e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a501931, SUC_lexicon_tags_38 },
    { 0x51f01932, SUC_lexicon_tags_11 },
    { 0x6ff01933, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db01936, SUC_lexicon_tags_40 },
    { 0x98f01936, SUC_lexicon_tags_46 },
    { 0xf1101938, SUC_lexicon_tags_12 },
    { 0x38501939, SUC_lexicon_tags_11 },
    { 0xb2301938, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9110193c, SUC_lexicon_tags_15 },
    { 0xb6f0193c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7701940, SUC_lexicon_tags_25 },
    { 0x38701941, SUC_lexicon_tags_65 },
    { 0xd5101941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7eb01944, SUC_lexicon_tags_27 },
    { 0x70501945, SUC_lexicon_tags_17 },
    { 0xf1901946, SUC_lexicon_tags_11 },
    { 0x6b101946, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b101949, SUC_lexicon_tags_4 },
    { 0xadf0194a, SUC_lexicon_tags_10 },
    { 0xed0194a, SUC_lexicon_tags_7 },
    { 0x6f30194c, SUC_lexicon_tags_38 },
    { 0xfcf0194d, SUC_lexicon_tags_23 },
    { 0x5950194c, SUC_lexicon_tags_77 },
    { 0xd2b0194e, SUC_lexicon_tags_48 },
    { 0xe4d0194d, SUC_lexicon_tags_11 },
    { 0x2101949, SUC_lexicon_tags_27 },
    { 0x90d01952, SUC_lexicon_tags_30 },
    { 0xb9d01952, SUC_lexicon_tags_47 },
    { 0x1230194e, SUC_lexicon_tags_11 },
    { 0xc9101955, SUC_lexicon_tags_38 },
    { 0x70d01956, SUC_lexicon_tags_33 },
    { 0xc8301950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37f01959, SUC_lexicon_tags_11 },
    { 0xfbb0195a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa2f0195c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd0195f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd01964, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6f01966, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2901968, SUC_lexicon_tags_17 },
    { 0x38701968, SUC_lexicon_tags_15 },
    { 0x2350196a, SUC_lexicon_tags_28 },
    { 0xcf0196b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40f0196d, SUC_lexicon_tags_25 },
    { 0xa8f0196d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9101971, SUC_lexicon_tags_63 },
    { 0x87d01972, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49501978, SUC_lexicon_tags_101 },
    { 0x4e701979, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf30197b, SUC_lexicon_tags_54 },
    { 0x6bd0197c, SUC_lexicon_tags_46 },
    { 0xdd10197c, SUC_lexicon_tags_112 },
    { 0x6590197d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xebb01980, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d01983, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa901992, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2501994, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6210199a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c70199c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0d0199e, SUC_lexicon_tags_3 },
    { 0x5370199f, SUC_lexicon_tags_7 },
    { 0xfcf0199e, SUC_lexicon_tags_37 },
    { 0xbc1019a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3019a4, SUC_lexicon_tags_8 },
    { 0x267019a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9019a8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd57019aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd09019ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d019b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787019c4, SUC_lexicon_tags_63 },
    { 0xa13019c5, SUC_lexicon_tags_12 },
    { 0xa2d019c6, SUC_lexicon_tags_26 },
    { 0x18f019c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471019d0, SUC_lexicon_tags_57 },
    { 0xbf7019d0, SUC_lexicon_tags_37 },
    { 0x9e7019d2, SUC_lexicon_tags_19 },
    { 0xa39019d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d019d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x435019db, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x107019dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05019e0, SUC_lexicon_tags_37 },
    { 0x23d019e0, SUC_lexicon_tags_25 },
    { 0xe27019e0, SUC_lexicon_tags_2 },
    { 0x683019e3, SUC_lexicon_tags_0 },
    { 0x3d7019e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x87019e6, SUC_lexicon_tags_12 },
    { 0xfad019e6, SUC_lexicon_tags_37 },
    { 0xf69019e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee5019ea, SUC_lexicon_tags_25 },
    { 0xe5d019eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5019f2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7019f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573019f9, SUC_lexicon_tags_25 },
    { 0x595019fa, SUC_lexicon_tags_49 },
    { 0xe8f019fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4d019fd, SUC_lexicon_tags_15 },
    { 0xbdd019fe, SUC_lexicon_tags_27 },
    { 0x1eb019ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb01a03, SUC_lexicon_tags_25 },
    { 0xc2f01a04, SUC_lexicon_tags_6 },
    { 0x70901a05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34501a07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3101a09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90901a0e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf301a10, SUC_lexicon_tags_12 },
    { 0xd6d01a10, SUC_lexicon_tags_44 },
    { 0xdfd01a11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8501a14, SUC_lexicon_tags_32 },
    { 0x52f01a15, SUC_lexicon_tags_37 },
    { 0xbcd01a16, SUC_lexicon_tags_2 },
    { 0xc9901a17, SUC_lexicon_tags_72 },
    { 0xadf01a18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4ed01a1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7101a1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17101a1f, SUC_lexicon_tags_0 },
    { 0xe2101a20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3701a22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad01a27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4f01a29, SUC_lexicon_tags_12 },
    { 0x82301a2a, SUC_lexicon_tags_12 },
    { 0x70501a2a, SUC_lexicon_tags_7 },
    { 0xd4701a2c, SUC_lexicon_tags_2 },
    { 0x87901a2d, SUC_lexicon_tags_25 },
    { 0x19701a2e, SUC_lexicon_tags_30 },
    { 0x95101a2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d01a33, SUC_lexicon_tags_23 },
    { 0xd3d01a34, SUC_lexicon_tags_38 },
    { 0x8cf01a35, SUC_lexicon_tags_25 },
    { 0xe1301a36, SUC_lexicon_tags_7 },
    { 0x10f01a37, SUC_lexicon_tags_32 },
    { 0xe1701a37, SUC_lexicon_tags_5 },
    { 0xe1f01a35, SUC_lexicon_tags_0 },
    { 0xb6d01a3a, SUC_lexicon_tags_37 },
    { 0xa2d01a3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3501a41, SUC_lexicon_tags_2 },
    { 0x60101a42, SUC_lexicon_tags_26 },
    { 0x19101a43, SUC_lexicon_tags_38 },
    { 0x75301a43, SUC_lexicon_tags_12 },
    { 0xe101a41, SUC_lexicon_tags_198 },
    { 0x50701a46, SUC_lexicon_tags_43 },
    { 0x7101a46, SUC_lexicon_tags_7 },
    { 0xd1701a48, SUC_lexicon_tags_1 },
    { 0xca901a47, SUC_lexicon_tags_1 },
    { 0xe4701a4a, SUC_lexicon_tags_45 },
    { 0x3b501a49, SUC_lexicon_tags_7 },
    { 0xc3101a4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34f01a4e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4b901a50, SUC_lexicon_tags_35 },
    { 0xa901a51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xea901a53, SUC_lexicon_tags_11 },
    { 0x38f01a53, SUC_lexicon_tags_3 },
    { 0x29301a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd01a5d, SUC_lexicon_tags_15 },
    { 0x87f01a5d, SUC_lexicon_tags_7 },
    { 0xdb901a5f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad01a67, SUC_lexicon_tags_66 },
    { 0x5c701a67, SUC_lexicon_tags_39 },
    { 0xd6b01a69, SUC_lexicon_tags_11 },
    { 0xca101a67, SUC_lexicon_tags_25 },
    { 0x23901a69, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f01a6f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97901a74, SUC_lexicon_tags_7 },
    { 0xe3901a75, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x64701a78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf01a7d, SUC_lexicon_tags_49 },
    { 0x42901a7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb01a82, SUC_lexicon_tags_12 },
    { 0xcc301a83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f01a86, SUC_lexicon_tags_8 },
    { 0xcc301a87, SUC_lexicon_tags_11 },
    { 0x21d01a87, SUC_lexicon_tags_22 },
    { 0xab501a89, SUC_lexicon_tags_40 },
    { 0x21101a87, SUC_lexicon_tags_35 },
    { 0x48701a8b, SUC_lexicon_tags_37 },
    { 0xf8501a8a, SUC_lexicon_tags_87 },
    { 0x18101a8b, SUC_lexicon_tags_25 },
    { 0xa1901a8e, SUC_lexicon_tags_15 },
    { 0x76301a8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef01a92, SUC_lexicon_tags_37 },
    { 0x38101a93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5d01a95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f01a98, SUC_lexicon_tags_14 },
    { 0xc2301a98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf01a9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb01aa0, SUC_lexicon_tags_10 },
    { 0x73b01aa1, SUC_lexicon_tags_31 },
    { 0x61b01aa2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef01aa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5301aa7, SUC_lexicon_tags_27 },
    { 0xed701aa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e301ab5, SUC_lexicon_tags_1 },
    { 0xff501ab5, SUC_lexicon_tags_33 },
    { 0x7cf01ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc901aba, SUC_lexicon_tags_65 },
    { 0xa1901aba, SUC_lexicon_tags_66 },
    { 0x68501abc, SUC_lexicon_tags_15 },
    { 0xd7b01abb, SUC_lexicon_tags_49 },
    { 0x2e901abe, SUC_lexicon_tags_12 },
    { 0xd1501abc, SUC_lexicon_tags_49 },
    { 0x97901ac0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f01ac4, SUC_lexicon_tags_45 },
    { 0xf4701ac5, SUC_lexicon_tags_0 },
    { 0x17701ac6, SUC_lexicon_tags_49 },
    { 0x28501ac5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a301ac9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f01acd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67301ad3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab01ad8, SUC_lexicon_tags_23 },
    { 0xf7301ad9, SUC_lexicon_tags_38 },
    { 0xf7f01ad8, SUC_lexicon_tags_37 },
    { 0x32701ada, SUC_lexicon_tags_45 },
    { 0x1a101adc, SUC_lexicon_tags_45 },
    { 0x8a701add, SUC_lexicon_tags_23 },
    { 0xe501adb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2701ae1, SUC_lexicon_tags_2 },
    { 0x61b01ae1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df01ae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1501aee, SUC_lexicon_tags_66 },
    { 0x74f01aef, SUC_lexicon_tags_63 },
    { 0xf3f01af0, SUC_lexicon_tags_4 },
    { 0x16101aef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c301af4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57901af7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62301af9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfb301afb, SUC_lexicon_tags_29 },
    { 0x8ab01afc, SUC_lexicon_tags_68 },
    { 0xdf501afb, SUC_lexicon_tags_0 },
    { 0x38901afc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9ef01b00, SUC_lexicon_tags_17 },
    { 0xde301b01, SUC_lexicon_tags_37 },
    { 0x88d01b01, SUC_lexicon_tags_17 },
    { 0x59901b01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74101b09, SUC_lexicon_tags_1 },
    { 0xbcf01b0a, SUC_lexicon_tags_14 },
    { 0xafd01b09, SUC_lexicon_tags_27 },
    { 0x82701b0c, SUC_lexicon_tags_27 },
    { 0x49501b0d, SUC_lexicon_tags_17 },
    { 0x22f01b09, SUC_lexicon_tags_49 },
    { 0xe4f01b09, SUC_lexicon_tags_2 },
    { 0xef901b10, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d01b13, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x82701b15, SUC_lexicon_tags_17 },
    { 0xd3101b16, SUC_lexicon_tags_50 },
    { 0xc3f01b15, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf01b1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda501b1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c101b22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d01b24, SUC_lexicon_tags_11 },
    { 0x99b01b25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab01b28, SUC_lexicon_tags_11 },
    { 0x8701b28, SUC_lexicon_tags_7 },
    { 0x91701b2a, SUC_lexicon_tags_3 },
    { 0x7b01b2b, SUC_lexicon_tags_37 },
    { 0xd701b2c, SUC_lexicon_tags_11 },
    { 0xba301b2a, SUC_lexicon_tags_15 },
    { 0x9c501b2e, SUC_lexicon_tags_17 },
    { 0x87d01b29, SUC_lexicon_tags_37 },
    { 0x98501b2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87901b32, SUC_lexicon_tags_24 },
    { 0xa5b01b33, SUC_lexicon_tags_45 },
    { 0xc5901b34, SUC_lexicon_tags_29 },
    { 0x7e101b33, SUC_lexicon_tags_28 },
    { 0x8c701b33, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13501b3a, SUC_lexicon_tags_7 },
    { 0xaa301b3b, SUC_lexicon_tags_33 },
    { 0x84101b3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd3f01b3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x78901b41, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0501b44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56701b46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2101b48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf301b4b, SUC_lexicon_tags_11 },
    { 0xf4f01b4c, SUC_lexicon_tags_38 },
    { 0xff901b4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeed01b4f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb01b52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78f01b54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x19d01b56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e101b5a, SUC_lexicon_tags_252 },
    { 0x1e501b5a, SUC_lexicon_tags_11 },
    { 0x30d01b5c, SUC_lexicon_tags_23 },
    { 0xc5101b5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb01b65, SUC_lexicon_tags_170 },
    { 0x80b01b65, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb301b69, SUC_lexicon_tags_47 },
    { 0x77701b69, SUC_lexicon_tags_1 },
    { 0x5cf01b69, SUC_lexicon_tags_26 },
    { 0xc5b01b6c, SUC_lexicon_tags_25 },
    { 0x6ed01b6c, SUC_lexicon_tags_138 },
    { 0x11901b6e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd101b74, SUC_lexicon_tags_45 },
    { 0xe2501b75, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa101b7a, SUC_lexicon_tags_82 },
    { 0x23101b7b, SUC_lexicon_tags_7 },
    { 0xb0901b7c, SUC_lexicon_tags_2 },
    { 0xe3301b7d, SUC_lexicon_tags_26 },
    { 0x5cb01b7d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b01b87, SUC_lexicon_tags_63 },
    { 0x12f01b88, SUC_lexicon_tags_15 },
    { 0x44b01b89, SUC_lexicon_tags_29 },
    { 0xc5f01b8a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d01b9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3701ba0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0701ba5, SUC_lexicon_tags_56 },
    { 0x8b701ba6, SUC_lexicon_tags_37 },
    { 0x60101ba6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35101ba9, SUC_lexicon_tags_37 },
    { 0x47301ba9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xfff01bac, SUC_lexicon_tags_0 },
    { 0x98901bac, SUC_lexicon_tags_12 },
    { 0x7c101bad, SUC_lexicon_tags_25 },
    { 0x25b01baf, SUC_lexicon_tags_37 },
    { 0xb2f01bb0, SUC_lexicon_tags_45 },
    { 0x72701bb1, SUC_lexicon_tags_37 },
    { 0xf1d01bb2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6dd01bb4, SUC_lexicon_tags_39 },
    { 0xb7501bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26101bb8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d301bbb, SUC_lexicon_tags_37 },
    { 0xbd501bbb, SUC_lexicon_tags_25 },
    { 0x45d01bbd, SUC_lexicon_tags_4 },
    { 0xc8d01bbb, SUC_lexicon_tags_11 },
    { 0x50101bbf, SUC_lexicon_tags_2 },
    { 0xe4501bc0, SUC_lexicon_tags_60 },
    { 0x8c301bc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f01bc5, SUC_lexicon_tags_63 },
    { 0x32b01bc6, SUC_lexicon_tags_26 },
    { 0x39701bc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57301bc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44101bce, SUC_lexicon_tags_8 },
    { 0x75b01bcf, SUC_lexicon_tags_17 },
    { 0x4cd01bd0, SUC_lexicon_tags_13 },
    { 0xb3f01bd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d901be1, SUC_lexicon_tags_2 },
    { 0x5df01be2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x27501be4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd01be9, SUC_lexicon_tags_17 },
    { 0x36301bea, SUC_lexicon_tags_6 },
    { 0xf4d01beb, SUC_lexicon_tags_11 },
    { 0x74501beb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa3b01bee, SUC_lexicon_tags_22 },
    { 0xe7d01bee, SUC_lexicon_tags_17 },
    { 0xead01bee, SUC_lexicon_tags_23 },
    { 0xef301bf1, SUC_lexicon_tags_11 },
    { 0x69501bef, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8501bf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb01bf8, SUC_lexicon_tags_15 },
    { 0x1c501bf8, SUC_lexicon_tags_7 },
    { 0x77d01bfa, SUC_lexicon_tags_7 },
    { 0x54901bfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a101c01, SUC_lexicon_tags_17 },
    { 0x1c901c01, SUC_lexicon_tags_43 },
    { 0xec301c03, SUC_lexicon_tags_38 },
    { 0x97b01c04, SUC_lexicon_tags_37 },
    { 0x4b301c04, SUC_lexicon_tags_27 },
    { 0xc4901c06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e101c12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19501c17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f01c1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df01c21, SUC_lexicon_tags_12 },
    { 0xc0501c22, SUC_lexicon_tags_37 },
    { 0xd7301c23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa6f01c25, SUC_lexicon_tags_60 },
    { 0x38101c26, SUC_lexicon_tags_37 },
    { 0xcd301c27, SUC_lexicon_tags_7 },
    { 0xbaf01c25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x18901c2b, SUC_lexicon_tags_31 },
    { 0x6ed01c2c, SUC_lexicon_tags_19 },
    { 0x2d501c2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5101c2f, SUC_lexicon_tags_11 },
    { 0x29901c30, SUC_lexicon_tags_14 },
    { 0x86f01c31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b01c35, SUC_lexicon_tags_33 },
    { 0xff01c35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f01c3b, SUC_lexicon_tags_47 },
    { 0xdd101c3c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad301c41, SUC_lexicon_tags_7 },
    { 0xba901c42, SUC_lexicon_tags_47 },
    { 0x8f101c43, SUC_lexicon_tags_23 },
    { 0x9b701c42, SUC_lexicon_tags_16 },
    { 0x6e101c43, SUC_lexicon_tags_53 },
    { 0x3ab01c44, SUC_lexicon_tags_37 },
    { 0x2dd01c47, SUC_lexicon_tags_37 },
    { 0xe4d01c41, SUC_lexicon_tags_14 },
    { 0xf1101c43, SUC_lexicon_tags_37 },
    { 0x7cd01c4a, SUC_lexicon_tags_14 },
    { 0xfdb01c42, SUC_lexicon_tags_15 },
    { 0x8b701c4a, SUC_lexicon_tags_12 },
    { 0x3c301c49, SUC_lexicon_tags_19 },
    { 0xb1501c4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35b01c50, SUC_lexicon_tags_15 },
    { 0xbbb01c51, SUC_lexicon_tags_17 },
    { 0x12901c50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8901c57, SUC_lexicon_tags_14 },
    { 0x65701c58, SUC_lexicon_tags_738 },
    { 0x6f01c57, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6a701c5b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44701c61, SUC_lexicon_tags_12 },
    { 0x3ed01c62, SUC_lexicon_tags_39 },
    { 0xfaf01c63, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f01c6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d01c6f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4701c71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x84701c73, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d901c77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x28301c79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9501c7e, SUC_lexicon_tags_451 },
    { 0xd1501c7f, SUC_lexicon_tags_33 },
    { 0x7e301c80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7301c82, SUC_lexicon_tags_1 },
    { 0x62901c83, SUC_lexicon_tags_0 },
    { 0xc4b01c82, SUC_lexicon_tags_25 },
    { 0xb901c82, SUC_lexicon_tags_26 },
    { 0xa0101c86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26501c89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c101c90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab01c94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10301c9a, SUC_lexicon_tags_27 },
    { 0xaf101c9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23501c9d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27901ca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf501ca4, SUC_lexicon_tags_17 },
    { 0x3d01ca4, SUC_lexicon_tags_6 },
    { 0xa101ca5, SUC_lexicon_tags_40 },
    { 0xdaf01ca5, SUC_lexicon_tags_37 },
    { 0xccd01ca8, SUC_lexicon_tags_103 },
    { 0x14901ca9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1701cad, SUC_lexicon_tags_17 },
    { 0x101cad, SUC_lexicon_tags_47 },
    { 0x7bf01cad, SUC_lexicon_tags_15 },
    { 0xc5901cae, SUC_lexicon_tags_10 },
    { 0x2f701cb1, SUC_lexicon_tags_77 },
    { 0x74301cb2, SUC_lexicon_tags_13 },
    { 0x61d01caf, SUC_lexicon_tags_60 },
    { 0xccb01cb4, SUC_lexicon_tags_36 },
    { 0x48501cb4, SUC_lexicon_tags_63 },
    { 0x25101cb5, SUC_lexicon_tags_15 },
    { 0xedf01cb7, SUC_lexicon_tags_6 },
    { 0x87301cb0, SUC_lexicon_tags_12 },
    { 0xbbd01cb2, SUC_lexicon_tags_58 },
    { 0xfbf01cba, SUC_lexicon_tags_20 },
    { 0xb0d01caf, SUC_lexicon_tags_31 },
    { 0x28501cbc, SUC_lexicon_tags_65 },
    { 0xd4d01cbc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd5701cbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9701cc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d01cc6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x73101cc8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7d701cca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4101cd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b01cd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb501cdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54301cdd, SUC_lexicon_tags_55 },
    { 0x59301cde, SUC_lexicon_tags_43 },
    { 0xf4701cdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d701ce3, SUC_lexicon_tags_28 },
    { 0x1a701ce4, SUC_lexicon_tags_124 },
    { 0xf0d01ce3, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65101ce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c301cec, SUC_lexicon_tags_19 },
    { 0xd8301cec, SUC_lexicon_tags_27 },
    { 0x6d901cec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff01cf3, SUC_lexicon_tags_12 },
    { 0x21d01cf4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f01cfa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b01d07, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f01d0b, SUC_lexicon_tags_12 },
    { 0xb6b01d0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4701d0e, SUC_lexicon_tags_30 },
    { 0x33501d0f, SUC_lexicon_tags_11 },
    { 0xfe301d0e, SUC_lexicon_tags_38 },
    { 0x5b701d0f, SUC_lexicon_tags_3 },
    { 0x3cf01d0e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x83f01d14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x49f01d16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x47101d19, SUC_lexicon_tags_77 },
    { 0xf7501d1a, SUC_lexicon_tags_13 },
    { 0x41d01d19, SUC_lexicon_tags_49 },
    { 0x38f01d1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6d01d1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdb01d20, SUC_lexicon_tags_45 },
    { 0x2301d20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf9101d23, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x83901d25, SUC_lexicon_tags_29 },
    { 0x53901d26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b01d29, SUC_lexicon_tags_24 },
    { 0x92b01d29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29701d2d, SUC_lexicon_tags_2 },
    { 0xb7701d2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a301d31, SUC_lexicon_tags_1 },
    { 0x22901d31, SUC_lexicon_tags_19 },
    { 0xa1101d33, SUC_lexicon_tags_30 },
    { 0x68b01d34, SUC_lexicon_tags_74 },
    { 0xb2101d34, SUC_lexicon_tags_45 },
    { 0xd4101d35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b01d39, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3bf01d3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48301d3e, SUC_lexicon_tags_80 },
    { 0x5bb01d3e, SUC_lexicon_tags_45 },
    { 0x47301d40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d701d46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff01d49, SUC_lexicon_tags_8 },
    { 0x87301d49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3b501d4c, SUC_lexicon_tags_12 },
    { 0xa3b01d4d, SUC_lexicon_tags_48 },
    { 0x63f01d4e, SUC_lexicon_tags_37 },
    { 0x74301d4f, SUC_lexicon_tags_15 },
    { 0x2b01d4d, SUC_lexicon_tags_63 },
    { 0x34901d4e, SUC_lexicon_tags_12 },
    { 0xb4101d52, SUC_lexicon_tags_11 },
    { 0xa5701d50, SUC_lexicon_tags_45 },
    { 0xa6301d54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95301d59, SUC_lexicon_tags_253 },
    { 0x31901d5a, SUC_lexicon_tags_1 },
    { 0xe7d01d5b, SUC_lexicon_tags_11 },
    { 0x3e501d59, SUC_lexicon_tags_7 },
    { 0x79501d5c, SUC_lexicon_tags_25 },
    { 0x7f901d5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d01d61, SUC_lexicon_tags_28 },
    { 0x15901d62, SUC_lexicon_tags_340 },
    { 0x4b901d63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56d01d65, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x98701d67, SUC_lexicon_tags_1 },
    { 0xd7f01d68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1701d6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33101d71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8301d75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0101d78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc701d7e, SUC_lexicon_tags_23 },
    { 0x32f01d7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b01d82, SUC_lexicon_tags_25 },
    { 0x76901d82, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8901d88, SUC_lexicon_tags_60 },
    { 0x3d101d89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f501d8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82301d8f, SUC_lexicon_tags_26 },
    { 0x2bd01d90, SUC_lexicon_tags_7 },
    { 0x9f01d91, SUC_lexicon_tags_65 },
    { 0x97901d92, SUC_lexicon_tags_70 },
    { 0x53901d93, SUC_lexicon_tags_14 },
    { 0xf3d01d8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0d01d96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf701d99, SUC_lexicon_tags_79 },
    { 0x93901d99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4101d9c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa5901d9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47701da2, SUC_lexicon_tags_8 },
    { 0x76501da2, SUC_lexicon_tags_41 },
    { 0x1bb01da4, SUC_lexicon_tags_48 },
    { 0xa0b01da5, SUC_lexicon_tags_27 },
    { 0x66701da3, SUC_lexicon_tags_11 },
    { 0x5bf01da4, SUC_lexicon_tags_3 },
    { 0x36901da8, SUC_lexicon_tags_1 },
    { 0xefd01da9, SUC_lexicon_tags_1 },
    { 0x68501daa, SUC_lexicon_tags_63 },
    { 0xbf701da8, SUC_lexicon_tags_38 },
    { 0xbd501dac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b01db0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90301db2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b01db5, SUC_lexicon_tags_1 },
    { 0x8b701db5, SUC_lexicon_tags_37 },
    { 0x7a901db5, SUC_lexicon_tags_66 },
    { 0x6d101db7, SUC_lexicon_tags_87 },
    { 0x52301db9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c301dbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6901dc2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d01dc7, SUC_lexicon_tags_8 },
    { 0xe6b01dc8, SUC_lexicon_tags_25 },
    { 0x67f01dc9, SUC_lexicon_tags_37 },
    { 0xf8101dc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x66d01dcc, SUC_lexicon_tags_47 },
    { 0x85101dcd, SUC_lexicon_tags_8 },
    { 0x32901dcc, SUC_lexicon_tags_15 },
    { 0x24f01dce, SUC_lexicon_tags_17 },
    { 0x17901dd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7701dd5, SUC_lexicon_tags_37 },
    { 0xccf01dd6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c501dd9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f501ddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1301de1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b01de8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d701dee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee101df1, SUC_lexicon_tags_37 },
    { 0x11501df1, SUC_lexicon_tags_26 },
    { 0x75d01df1, SUC_lexicon_tags_17 },
    { 0x25501df4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad701df7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15701dfc, SUC_lexicon_tags_4 },
    { 0x9df01dfd, SUC_lexicon_tags_233 },
    { 0x9e301dfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b01e01, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf901e04, SUC_lexicon_tags_12 },
    { 0xfc301e05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d701e0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9f01e0c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2101e0f, SUC_lexicon_tags_37 },
    { 0x2b301e10, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7e701e12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x60301e14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xad301e16, SUC_lexicon_tags_49 },
    { 0xb0901e16, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8501e1b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3501e1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4f701e20, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8ad01e22, SUC_lexicon_tags_26 },
    { 0x31b01e23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90901e27, SUC_lexicon_tags_601 },
    { 0x35301e28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e901e2c, SUC_lexicon_tags_17 },
    { 0x73b01e2d, SUC_lexicon_tags_7 },
    { 0xe4b01e2e, SUC_lexicon_tags_11 },
    { 0xf0901e2f, SUC_lexicon_tags_39 },
    { 0xcf501e30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3101e32, SUC_lexicon_tags_2 },
    { 0x9a301e32, SUC_lexicon_tags_21 },
    { 0x55f01e33, SUC_lexicon_tags_28 },
    { 0x82b01e35, SUC_lexicon_tags_0 },
    { 0xdf101e33, SUC_lexicon_tags_37 },
    { 0xe1901e37, SUC_lexicon_tags_27 },
    { 0xddd01e38, SUC_lexicon_tags_15 },
    { 0x9701e33, SUC_lexicon_tags_0 },
    { 0x3301e3a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x27101e3d, SUC_lexicon_tags_49 },
    { 0xafb01e3e, SUC_lexicon_tags_37 },
    { 0x98301e3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6301e42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a101e46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe301e49, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xad01e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b01e52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57301e56, SUC_lexicon_tags_11 },
    { 0xe1901e56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7101e59, SUC_lexicon_tags_45 },
    { 0x61501e5a, SUC_lexicon_tags_63 },
    { 0x68101e5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d01e5e, SUC_lexicon_tags_37 },
    { 0xca101e5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e901e61, SUC_lexicon_tags_46 },
    { 0xa501e62, SUC_lexicon_tags_70 },
    { 0xe2301e63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0501e65, SUC_lexicon_tags_23 },
    { 0xcc701e66, SUC_lexicon_tags_25 },
    { 0xad701e66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61701e6c, SUC_lexicon_tags_25 },
    { 0xecb01e6c, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0x86701e6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1301e71, SUC_lexicon_tags_11 },
    { 0x42101e72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd01e76, SUC_lexicon_tags_63 },
    { 0xe1701e77, SUC_lexicon_tags_2 },
    { 0x20501e78, SUC_lexicon_tags_47 },
    { 0x69d01e79, SUC_lexicon_tags_27 },
    { 0x39301e79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18301e7d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e501e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43501e86, SUC_lexicon_tags_43 },
    { 0x59d01e87, SUC_lexicon_tags_15 },
    { 0xccd01e88, SUC_lexicon_tags_25 },
    { 0x93901e89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b01e90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86501e97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23101e99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd01e9f, SUC_lexicon_tags_60 },
    { 0x74b01ea0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc301ea2, SUC_lexicon_tags_30 },
    { 0x3301ea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e301ea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40f01ea9, SUC_lexicon_tags_37 },
    { 0x7e701ea9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc501eb1, SUC_lexicon_tags_49 },
    { 0x41901eb2, SUC_lexicon_tags_15 },
    { 0xb5101eb3, SUC_lexicon_tags_38 },
    { 0x40701eb1, SUC_lexicon_tags_124 },
    { 0xe3d01eb2, SUC_lexicon_tags_27 },
    { 0x77501eb4, SUC_lexicon_tags_27 },
    { 0x1c501eb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f01ebb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9501ebd, SUC_lexicon_tags_4 },
    { 0x22301ebe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3501ec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10901ec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76901eca, SUC_lexicon_tags_28 },
    { 0x36f01eca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4701ed2, SUC_lexicon_tags_17 },
    { 0xa4d01ed3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c301ed6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12101eda, SUC_lexicon_tags_37 },
    { 0x6d701edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b01ee0, SUC_lexicon_tags_47 },
    { 0x12b01ee1, SUC_lexicon_tags_12 },
    { 0x49301ee2, SUC_lexicon_tags_29 },
    { 0xa501ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cb01ee5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe101ee7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35501eec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4bf01eee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b01ef1, SUC_lexicon_tags_19 },
    { 0xa2301ef2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xaef01ef4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36101ef6, SUC_lexicon_tags_1 },
    { 0x44301ef7, SUC_lexicon_tags_8 },
    { 0xb8701ef8, SUC_lexicon_tags_1 },
    { 0x26901ef6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd4301efb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0301efe, SUC_lexicon_tags_25 },
    { 0x26d01eff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f01f02, SUC_lexicon_tags_11 },
    { 0x5dd01f03, SUC_lexicon_tags_38 },
    { 0x1bb01f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d01f08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc101f0c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77501f11, SUC_lexicon_tags_12 },
    { 0x17301f12, SUC_lexicon_tags_23 },
    { 0xc1f01f12, SUC_lexicon_tags_8 },
    { 0x85501f12, SUC_lexicon_tags_7 },
    { 0xcc101f15, SUC_lexicon_tags_1 },
    { 0xe0b01f12, SUC_lexicon_tags_27 },
    { 0xac901f17, SUC_lexicon_tags_10 },
    { 0xaaf01f16, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7301f1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45901f22, SUC_lexicon_tags_15 },
    { 0xa0f01f23, SUC_lexicon_tags_54 },
    { 0x49d01f24, SUC_lexicon_tags_17 },
    { 0xc5101f25, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74701f32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb301f35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9d01f37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf301f3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bb01f3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c301f41, SUC_lexicon_tags_15 },
    { 0xcf01f42, SUC_lexicon_tags_11 },
    { 0xfa501f43, SUC_lexicon_tags_91 },
    { 0xe9101f42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xef101f46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77701f48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4c901f4a, SUC_lexicon_tags_45 },
    { 0xe4501f4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e901f4e, SUC_lexicon_tags_1 },
    { 0x4f901f4f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0501f55, SUC_lexicon_tags_7 },
    { 0xb1301f56, SUC_lexicon_tags_29 },
    { 0x3cf01f56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74b01f59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f01f5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce501f61, SUC_lexicon_tags_37 },
    { 0xccb01f61, SUC_lexicon_tags_25 },
    { 0x65d01f61, SUC_lexicon_tags_4 },
    { 0xf6501f64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x56901f67, SUC_lexicon_tags_17 },
    { 0x9ad01f68, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20701f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec101f75, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe1b01f77, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b01f7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x13f01f7e, SUC_lexicon_tags_7 },
    { 0x62501f7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b01f84, SUC_lexicon_tags_71 },
    { 0x65501f84, SUC_lexicon_tags_63 },
    { 0x71f01f86, SUC_lexicon_tags_60 },
    { 0x4b301f86, SUC_lexicon_tags_31 },
    { 0x20d01f87, SUC_lexicon_tags_7 },
    { 0xe2501f89, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f01f8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54501f90, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58301f92, SUC_lexicon_tags_27 },
    { 0x80b01f93, SUC_lexicon_tags_17 },
    { 0x65901f93, SUC_lexicon_tags_11 },
    { 0x4b01f95, SUC_lexicon_tags_5 },
    { 0x17101f95, SUC_lexicon_tags_33 },
    { 0x4b501f97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01f9a, SUC_lexicon_tags_12 },
    { 0x9f901f9a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c501f9f, SUC_lexicon_tags_11 },
    { 0xfc501fa0, SUC_lexicon_tags_45 },
    { 0xdcd01fa1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde101fa3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x30101fa5, SUC_lexicon_tags_60 },
    { 0x8ab01fa5, SUC_lexicon_tags_12 },
    { 0x2c101fa7, SUC_lexicon_tags_3 },
    { 0x76101fa8, SUC_lexicon_tags_6 },
    { 0x1f901fa9, SUC_lexicon_tags_28 },
    { 0xa1f01fa6, SUC_lexicon_tags_9 },
    { 0x61301faa, SUC_lexicon_tags_49 },
    { 0xcd901fa8, SUC_lexicon_tags_15 },
    { 0xd7b01fa9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3501fb2, SUC_lexicon_tags_21 },
    { 0x60d01fb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc0f01fb5, SUC_lexicon_tags_14 },
    { 0x83101fb5, SUC_lexicon_tags_12 },
    { 0x82901fb7, SUC_lexicon_tags_25 },
    { 0x2e301fb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1301fbc, SUC_lexicon_tags_17 },
    { 0xfbb01fbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3101fc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a701fc3, SUC_lexicon_tags_25 },
    { 0x5c501fc4, SUC_lexicon_tags_11 },
    { 0x57b01fc4, SUC_lexicon_tags_16 },
    { 0x37101fc5, SUC_lexicon_tags_2 },
    { 0xf2101fc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x46101fca, SUC_lexicon_tags_15 },
    { 0x12901fcb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd701fcd, SUC_lexicon_tags_14 },
    { 0xcf701fcd, SUC_lexicon_tags_45 },
    { 0x7d901fce, SUC_lexicon_tags_25 },
    { 0x80f01fd0, SUC_lexicon_tags_25 },
    { 0x76f01fd0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c901fd4, SUC_lexicon_tags_62 },
    { 0xd301fd5, SUC_lexicon_tags_37 },
    { 0xf0701fd5, SUC_lexicon_tags_14 },
    { 0x12501fd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3301fdd, SUC_lexicon_tags_14 },
    { 0x80701fde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a301fe3, SUC_lexicon_tags_31 },
    { 0xfc501fe4, SUC_lexicon_tags_26 },
    { 0xe9101fe4, SUC_lexicon_tags_33 },
    { 0x67901fe6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9301fe9, SUC_lexicon_tags_124 },
    { 0x47701fe9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4a701fec, SUC_lexicon_tags_14 },
    { 0x4301fed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d701ff0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd01ff3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27901ff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27101ffb, SUC_lexicon_tags_1 },
    { 0x9b901ffb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b02000, SUC_lexicon_tags_47 },
    { 0x63d02001, SUC_lexicon_tags_7 },
    { 0x4d702002, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd902005, SUC_lexicon_tags_17 },
    { 0x39d02006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9790200b, SUC_lexicon_tags_37 },
    { 0x130200c, SUC_lexicon_tags_27 },
    { 0x28f0200d, SUC_lexicon_tags_48 },
    { 0xe550200e, SUC_lexicon_tags_15 },
    { 0x9070200e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1502015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1102018, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf0201c, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b902020, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbb02022, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd4702024, SUC_lexicon_tags_55 },
    { 0x75b02025, SUC_lexicon_tags_31 },
    { 0x31102026, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7230202f, SUC_lexicon_tags_9 },
    { 0xad702030, SUC_lexicon_tags_27 },
    { 0xa290202f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab02035, SUC_lexicon_tags_14 },
    { 0x55502036, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b0203c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c70203f, SUC_lexicon_tags_17 },
    { 0x82302040, SUC_lexicon_tags_9 },
    { 0xa5302041, SUC_lexicon_tags_165 },
    { 0x34302041, SUC_lexicon_tags_44 },
    { 0x1102040, SUC_lexicon_tags_17 },
    { 0xc1102044, SUC_lexicon_tags_31 },
    { 0xa1d02044, SUC_lexicon_tags_43 },
    { 0xeb702046, SUC_lexicon_tags_37 },
    { 0x3bf02046, SUC_lexicon_tags_68 },
    { 0xe6502041, SUC_lexicon_tags_15 },
    { 0x95b02040, SUC_lexicon_tags_27 },
    { 0x35502047, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd370204d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc890204f, SUC_lexicon_tags_17 },
    { 0x15302050, SUC_lexicon_tags_30 },
    { 0xf6d02051, SUC_lexicon_tags_45 },
    { 0x81f02052, SUC_lexicon_tags_0 },
    { 0x1902051, SUC_lexicon_tags_37 },
    { 0xd6902050, SUC_lexicon_tags_17 },
    { 0xb1302055, SUC_lexicon_tags_7 },
    { 0x19d02056, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x92302058, SUC_lexicon_tags_25 },
    { 0x1eb02059, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd10205f, SUC_lexicon_tags_11 },
    { 0x6110205f, SUC_lexicon_tags_47 },
    { 0x2b02060, SUC_lexicon_tags_45 },
    { 0xf902062, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed02069, SUC_lexicon_tags_16 },
    { 0xfe90206a, SUC_lexicon_tags_1 },
    { 0x3a30206b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe470206e, SUC_lexicon_tags_8 },
    { 0xb30206f, SUC_lexicon_tags_19 },
    { 0x5bf0206f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99702072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94702076, SUC_lexicon_tags_29 },
    { 0xcdf02077, SUC_lexicon_tags_12 },
    { 0xf6102078, SUC_lexicon_tags_37 },
    { 0xe0f02079, SUC_lexicon_tags_17 },
    { 0xd1302078, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d0207e, SUC_lexicon_tags_7 },
    { 0xcb90207f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x83702081, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2502084, SUC_lexicon_tags_19 },
    { 0xdbb02085, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30208a, SUC_lexicon_tags_2 },
    { 0x6170208b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd02092, SUC_lexicon_tags_12 },
    { 0x94502092, SUC_lexicon_tags_28 },
    { 0x64b02094, SUC_lexicon_tags_37 },
    { 0x28b02092, SUC_lexicon_tags_7 },
    { 0xd8d02096, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8502098, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f50209f, SUC_lexicon_tags_2 },
    { 0xa1b020a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf020a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9020a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955020ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3020b1, SUC_lexicon_tags_14 },
    { 0x4d5020b1, SUC_lexicon_tags_154 },
    { 0x329020b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1020bc, SUC_lexicon_tags_37 },
    { 0x8b9020bd, SUC_lexicon_tags_11 },
    { 0x841020be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bd020c0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d020ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609020d2, SUC_lexicon_tags_14 },
    { 0xdf3020d3, SUC_lexicon_tags_17 },
    { 0xe43020d2, SUC_lexicon_tags_47 },
    { 0xf29020d2, SUC_lexicon_tags_31 },
    { 0xa87020d3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b020da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257020e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfeb020e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd020e5, SUC_lexicon_tags_37 },
    { 0x881020e5, SUC_lexicon_tags_49 },
    { 0x81d020e7, SUC_lexicon_tags_60 },
    { 0xf3020e5, SUC_lexicon_tags_37 },
    { 0x27b020e9, SUC_lexicon_tags_19 },
    { 0x73d020ea, SUC_lexicon_tags_12 },
    { 0x123020ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d020ef, SUC_lexicon_tags_17 },
    { 0x74d020f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725020f8, SUC_lexicon_tags_43 },
    { 0xaf9020f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95020fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d02105, SUC_lexicon_tags_37 },
    { 0xbad02106, SUC_lexicon_tags_4 },
    { 0x5fb02107, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ab02109, SUC_lexicon_tags_15 },
    { 0x5010210a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6d0210c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b50210e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0702111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb02116, SUC_lexicon_tags_37 },
    { 0x91b02116, SUC_lexicon_tags_2 },
    { 0x19d02118, SUC_lexicon_tags_15 },
    { 0xf1f02118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f0211d, SUC_lexicon_tags_2 },
    { 0x1df0211e, SUC_lexicon_tags_35 },
    { 0xa130211f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d702121, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff02124, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd102129, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf50212b, SUC_lexicon_tags_15 },
    { 0x5b50212b, SUC_lexicon_tags_46 },
    { 0x1df0212d, SUC_lexicon_tags_191 },
    { 0x8c90212e, SUC_lexicon_tags_3 },
    { 0xa70212f, SUC_lexicon_tags_12 },
    { 0x3590212f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x15502133, SUC_lexicon_tags_15 },
    { 0x82302134, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8702137, SUC_lexicon_tags_12 },
    { 0xd4702138, SUC_lexicon_tags_15 },
    { 0x88d02137, SUC_lexicon_tags_37 },
    { 0xc1d0213a, SUC_lexicon_tags_15 },
    { 0x2b10213a, SUC_lexicon_tags_1 },
    { 0xce90213b, SUC_lexicon_tags_17 },
    { 0xd0213d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb502140, SUC_lexicon_tags_33 },
    { 0xafb02140, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd702146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6502148, SUC_lexicon_tags_34 },
    { 0xe702149, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc30214d, SUC_lexicon_tags_7 },
    { 0x53f0214e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a702150, SUC_lexicon_tags_37 },
    { 0xca502151, SUC_lexicon_tags_29 },
    { 0x6df02151, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0x15d02154, SUC_lexicon_tags_63 },
    { 0x14502155, SUC_lexicon_tags_63 },
    { 0xb3902155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b02159, SUC_lexicon_tags_18 },
    { 0x7d702159, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb50215c, SUC_lexicon_tags_63 },
    { 0x44d0215c, SUC_lexicon_tags_49 },
    { 0x9f0215e, SUC_lexicon_tags_16 },
    { 0xf6d0215e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12502163, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8902166, SUC_lexicon_tags_12 },
    { 0xeb302167, SUC_lexicon_tags_15 },
    { 0x44702168, SUC_lexicon_tags_34 },
    { 0xa102168, SUC_lexicon_tags_8 },
    { 0xc1502168, SUC_lexicon_tags_11 },
    { 0x81902168, SUC_lexicon_tags_37 },
    { 0x6530216c, SUC_lexicon_tags_1 },
    { 0xb2d0216d, SUC_lexicon_tags_31 },
    { 0xb290216d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd02171, SUC_lexicon_tags_2 },
    { 0xb8d02172, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf102174, SUC_lexicon_tags_584 },
    { 0, NULL },
    { 0xde502176, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8af02178, SUC_lexicon_tags_0 },
    { 0x1f02179, SUC_lexicon_tags_60 },
    { 0x45502179, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b0217e, SUC_lexicon_tags_25 },
    { 0x9890217e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x58b02181, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92302185, SUC_lexicon_tags_0 },
    { 0xc4302186, SUC_lexicon_tags_12 },
    { 0x6af02187, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa2702189, SUC_lexicon_tags_54 },
    { 0x4610218a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50702190, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f02199, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2a50219b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb490219d, SUC_lexicon_tags_26 },
    { 0x7690219e, SUC_lexicon_tags_60 },
    { 0x9d30219f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511021a3, SUC_lexicon_tags_160 },
    { 0xf7b021a4, SUC_lexicon_tags_43 },
    { 0xbed021a5, SUC_lexicon_tags_2 },
    { 0x56b021a6, SUC_lexicon_tags_27 },
    { 0xabb021a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd021aa, SUC_lexicon_tags_25 },
    { 0x445021ab, SUC_lexicon_tags_15 },
    { 0xfe5021aa, SUC_lexicon_tags_46 },
    { 0x3ab021aa, SUC_lexicon_tags_4 },
    { 0xc63021ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x587021b1, SUC_lexicon_tags_72 },
    { 0x765021b2, SUC_lexicon_tags_17 },
    { 0x179021b2, SUC_lexicon_tags_28 },
    { 0x8ab021b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165021b8, SUC_lexicon_tags_18 },
    { 0x7cf021b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3021c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6ff021c3, SUC_lexicon_tags_8 },
    { 0xacb021c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47021cc, SUC_lexicon_tags_7 },
    { 0xc4f021cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb87021cf, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xe09021d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x125021d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101021d7, SUC_lexicon_tags_15 },
    { 0x6ef021d8, SUC_lexicon_tags_1 },
    { 0xe3f021d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b3021db, SUC_lexicon_tags_14 },
    { 0xfd021dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x187021de, SUC_lexicon_tags_23 },
    { 0xe47021df, SUC_lexicon_tags_17 },
    { 0x10d021e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x489021e2, SUC_lexicon_tags_48 },
    { 0x163021e3, SUC_lexicon_tags_37 },
    { 0xc69021e3, SUC_lexicon_tags_25 },
    { 0xe81021e4, SUC_lexicon_tags_39 },
    { 0x393021e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631021eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef021f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x369021f4, SUC_lexicon_tags_60 },
    { 0x363021f5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaf9021f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3021fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5702201, SUC_lexicon_tags_12 },
    { 0xe4502202, SUC_lexicon_tags_1 },
    { 0x2bf02202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8902205, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5102207, SUC_lexicon_tags_25 },
    { 0xffb02207, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d0220d, SUC_lexicon_tags_7 },
    { 0xfab0220d, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0xb3702210, SUC_lexicon_tags_23 },
    { 0xa9502211, SUC_lexicon_tags_35 },
    { 0x8bb02212, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13902217, SUC_lexicon_tags_27 },
    { 0xa902217, SUC_lexicon_tags_15 },
    { 0xe7902219, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3302222, SUC_lexicon_tags_23 },
    { 0x2b502223, SUC_lexicon_tags_0 },
    { 0x25b02224, SUC_lexicon_tags_11 },
    { 0x12d02225, SUC_lexicon_tags_19 },
    { 0x68502225, SUC_lexicon_tags_2 },
    { 0xea902227, SUC_lexicon_tags_17 },
    { 0xe0f02228, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff30222d, SUC_lexicon_tags_27 },
    { 0xf9d0222d, SUC_lexicon_tags_56 },
    { 0xd630222e, SUC_lexicon_tags_26 },
    { 0x48b0222f, SUC_lexicon_tags_9 },
    { 0xf2102231, SUC_lexicon_tags_82 },
    { 0xf7b02231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5302237, SUC_lexicon_tags_7 },
    { 0xf1f02238, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1210223c, SUC_lexicon_tags_15 },
    { 0x7110223d, SUC_lexicon_tags_1 },
    { 0x4ef0223d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb02243, SUC_lexicon_tags_2 },
    { 0x13b02244, SUC_lexicon_tags_39 },
    { 0x9b902244, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4e902247, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a502249, SUC_lexicon_tags_85 },
    { 0x4130224a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb10224d, SUC_lexicon_tags_56 },
    { 0x2270224d, SUC_lexicon_tags_37 },
    { 0x5bf0224f, SUC_lexicon_tags_14 },
    { 0xddd02250, SUC_lexicon_tags_17 },
    { 0x3170224d, SUC_lexicon_tags_45 },
    { 0xe1702252, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe702255, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf0225a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92902262, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb9902264, SUC_lexicon_tags_229 },
    { 0x3102265, SUC_lexicon_tags_12 },
    { 0x3e702265, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x14102268, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f0226b, SUC_lexicon_tags_41 },
    { 0xebb0226c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc5d0226e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f02271, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x45b02273, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe590227b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8250227e, SUC_lexicon_tags_37 },
    { 0xbc70227f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99502286, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c30228b, SUC_lexicon_tags_62 },
    { 0x1c50228b, SUC_lexicon_tags_12 },
    { 0x8910228d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x97f0228f, SUC_lexicon_tags_27 },
    { 0xbfb02290, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0702295, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b02299, SUC_lexicon_tags_37 },
    { 0x6050229a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9ab0229c, SUC_lexicon_tags_55 },
    { 0x7710229d, SUC_lexicon_tags_17 },
    { 0xd0229e, SUC_lexicon_tags_27 },
    { 0xc7d0229f, SUC_lexicon_tags_63 },
    { 0xcdf0229d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59022a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5022a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b022b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb95022b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe7022b5, SUC_lexicon_tags_140 },
    { 0x27f022b6, SUC_lexicon_tags_25 },
    { 0xd0d022b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5022bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34d022bd, SUC_lexicon_tags_0 },
    { 0xead022bd, SUC_lexicon_tags_47 },
    { 0x49b022be, SUC_lexicon_tags_87 },
    { 0x8cb022c0, SUC_lexicon_tags_77 },
    { 0x4f7022c1, SUC_lexicon_tags_12 },
    { 0x4ad022c1, SUC_lexicon_tags_7 },
    { 0x121022c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x149022c6, SUC_lexicon_tags_14 },
    { 0x187022c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73022cd, SUC_lexicon_tags_12 },
    { 0x2b3022cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9d022d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929022d8, SUC_lexicon_tags_47 },
    { 0xe7f022d8, SUC_lexicon_tags_30 },
    { 0x309022da, SUC_lexicon_tags_25 },
    { 0x261022da, SUC_lexicon_tags_47 },
    { 0xdb022d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef022e5, SUC_lexicon_tags_0 },
    { 0xcdd022e6, SUC_lexicon_tags_26 },
    { 0x103022e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17022ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9022f2, SUC_lexicon_tags_10 },
    { 0x103022f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb25022f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ad022f7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55022fa, SUC_lexicon_tags_47 },
    { 0xc59022fb, SUC_lexicon_tags_7 },
    { 0xd37022fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b022ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3902301, SUC_lexicon_tags_101 },
    { 0x30902301, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0, NULL },
    { 0x8902305, SUC_lexicon_tags_37 },
    { 0x4b102306, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7190230a, SUC_lexicon_tags_26 },
    { 0xe050230a, SUC_lexicon_tags_2 },
    { 0xd70230c, SUC_lexicon_tags_38 },
    { 0xf790230d, SUC_lexicon_tags_25 },
    { 0x31d0230e, SUC_lexicon_tags_17 },
    { 0x7c90230e, SUC_lexicon_tags_25 },
    { 0x8fd0230d, SUC_lexicon_tags_0 },
    { 0xdc902311, SUC_lexicon_tags_33 },
    { 0xbe30230a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d02317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d0231a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdd10231c, SUC_lexicon_tags_25 },
    { 0x25f0231d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd02323, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d0232c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb50232f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe1f02331, SUC_lexicon_tags_12 },
    { 0x4902331, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b702335, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad02338, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa670233c, SUC_lexicon_tags_25 },
    { 0x4cf0233c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xded0233f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d102342, SUC_lexicon_tags_24 },
    { 0x75902342, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3bd02345, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf02351, SUC_lexicon_tags_11 },
    { 0xf0302352, SUC_lexicon_tags_54 },
    { 0xeaf02353, SUC_lexicon_tags_8 },
    { 0xe7f02354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e702357, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34302359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50235f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45902365, SUC_lexicon_tags_63 },
    { 0xfc902366, SUC_lexicon_tags_107 },
    { 0xe3f02367, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d502369, SUC_lexicon_tags_25 },
    { 0x23f0236a, SUC_lexicon_tags_15 },
    { 0xbc50236a, SUC_lexicon_tags_6 },
    { 0xcef0236a, SUC_lexicon_tags_57 },
    { 0xcaf0236d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6150236f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f02373, SUC_lexicon_tags_15 },
    { 0x28d02374, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f02377, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f0237a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d0237d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d02381, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9b02383, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa102388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf070238a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7250238e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b02393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb02395, SUC_lexicon_tags_32 },
    { 0x80302396, SUC_lexicon_tags_37 },
    { 0xf4502395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x410239b, SUC_lexicon_tags_11 },
    { 0x50b0239b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa50239e, SUC_lexicon_tags_47 },
    { 0x6bf0239e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x91d023a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1023a5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d023aa, SUC_lexicon_tags_39 },
    { 0x59023ab, SUC_lexicon_tags_8 },
    { 0x7ff023ac, SUC_lexicon_tags_19 },
    { 0x4bf023ad, SUC_lexicon_tags_46 },
    { 0xced023ad, SUC_lexicon_tags_25 },
    { 0xaef023ad, SUC_lexicon_tags_33 },
    { 0xaa3023ae, SUC_lexicon_tags_0 },
    { 0xd03023b1, SUC_lexicon_tags_15 },
    { 0x835023b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131023bd, SUC_lexicon_tags_4 },
    { 0xf3d023be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0b023c0, SUC_lexicon_tags_37 },
    { 0x145023c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad023c4, SUC_lexicon_tags_15 },
    { 0x3d3023c5, SUC_lexicon_tags_64 },
    { 0x169023c6, SUC_lexicon_tags_68 },
    { 0x9f3023c5, SUC_lexicon_tags_33 },
    { 0x3c1023c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f9023ca, SUC_lexicon_tags_45 },
    { 0x60f023ca, SUC_lexicon_tags_30 },
    { 0x4ff023cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31023d1, SUC_lexicon_tags_21 },
    { 0x255023d1, SUC_lexicon_tags_12 },
    { 0xd0f023d3, SUC_lexicon_tags_87 },
    { 0x5ed023d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d023d8, SUC_lexicon_tags_21 },
    { 0xb77023d9, SUC_lexicon_tags_14 },
    { 0x43b023d8, SUC_lexicon_tags_31 },
    { 0xb7b023da, SUC_lexicon_tags_17 },
    { 0x247023dc, SUC_lexicon_tags_1 },
    { 0xa1f023dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f023e2, SUC_lexicon_tags_45 },
    { 0x939023e2, SUC_lexicon_tags_7 },
    { 0x4e9023e4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f023e8, SUC_lexicon_tags_390 },
    { 0xd43023e9, SUC_lexicon_tags_19 },
    { 0x8eb023e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x799023ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b023ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x159023f2, SUC_lexicon_tags_14 },
    { 0x1ab023f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7023f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81b023fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfd9023fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbd1023ff, SUC_lexicon_tags_15 },
    { 0x1b902400, SUC_lexicon_tags_39 },
    { 0x5ab02400, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x58702404, SUC_lexicon_tags_25 },
    { 0x5702405, SUC_lexicon_tags_27 },
    { 0xe1302406, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e70240b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1730240d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2fb0240f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff902413, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9710241e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d302421, SUC_lexicon_tags_15 },
    { 0xd5d02422, SUC_lexicon_tags_49 },
    { 0xe702423, SUC_lexicon_tags_30 },
    { 0x74f02423, SUC_lexicon_tags_37 },
    { 0x88102424, SUC_lexicon_tags_70 },
    { 0x4cf02424, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4502428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5610242a, SUC_lexicon_tags_13 },
    { 0x5ab0242a, SUC_lexicon_tags_1 },
    { 0x1f0242c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x41f0242e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8502430, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b02434, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96102439, SUC_lexicon_tags_14 },
    { 0x1330243a, SUC_lexicon_tags_62 },
    { 0x48b0243b, SUC_lexicon_tags_37 },
    { 0xb170243c, SUC_lexicon_tags_21 },
    { 0x5570243d, SUC_lexicon_tags_33 },
    { 0xb0243d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f02446, SUC_lexicon_tags_30 },
    { 0xecb02447, SUC_lexicon_tags_14 },
    { 0x8f702448, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a102450, SUC_lexicon_tags_47 },
    { 0x9502451, SUC_lexicon_tags_29 },
    { 0x58902452, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef502455, SUC_lexicon_tags_87 },
    { 0xc702456, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d702458, SUC_lexicon_tags_11 },
    { 0x50102459, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0245c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d0245f, SUC_lexicon_tags_27 },
    { 0x5730245f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94702464, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f902468, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87d0246a, SUC_lexicon_tags_37 },
    { 0x9110246b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd0246e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x91f02470, SUC_lexicon_tags_49 },
    { 0x16f02471, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c302478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6170247c, SUC_lexicon_tags_1 },
    { 0x19b0247d, SUC_lexicon_tags_14 },
    { 0x8cb0247c, SUC_lexicon_tags_17 },
    { 0x4010247f, SUC_lexicon_tags_5 },
    { 0x7a102480, SUC_lexicon_tags_46 },
    { 0x3f302481, SUC_lexicon_tags_38 },
    { 0x7ff02482, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd170248b, SUC_lexicon_tags_15 },
    { 0xfeb0248c, SUC_lexicon_tags_65 },
    { 0x9930248d, SUC_lexicon_tags_25 },
    { 0x2ed0248e, SUC_lexicon_tags_37 },
    { 0xd810248f, SUC_lexicon_tags_21 },
    { 0x8a50248c, SUC_lexicon_tags_4 },
    { 0x2a90248c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe8b02493, SUC_lexicon_tags_0 },
    { 0x60102494, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e102497, SUC_lexicon_tags_37 },
    { 0x89f02497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2950249c, SUC_lexicon_tags_79 },
    { 0xa9f0249d, SUC_lexicon_tags_84 },
    { 0xf930249e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37024a1, SUC_lexicon_tags_15 },
    { 0xb15024a2, SUC_lexicon_tags_15 },
    { 0xd73024a3, SUC_lexicon_tags_0 },
    { 0xced024a4, SUC_lexicon_tags_7 },
    { 0x653024a3, SUC_lexicon_tags_17 },
    { 0xb9024a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b024a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3024ac, SUC_lexicon_tags_19 },
    { 0xac3024ad, SUC_lexicon_tags_2 },
    { 0xc77024ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b024b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d024b6, SUC_lexicon_tags_45 },
    { 0x8ad024b7, SUC_lexicon_tags_25 },
    { 0xbef024b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb024be, SUC_lexicon_tags_2 },
    { 0x677024bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb19024c1, SUC_lexicon_tags_31 },
    { 0x129024c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf024c5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625024ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19b024cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf024d1, SUC_lexicon_tags_66 },
    { 0xb5f024d2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd35024d4, SUC_lexicon_tags_27 },
    { 0xf63024d5, SUC_lexicon_tags_25 },
    { 0x77d024d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x853024d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f024db, SUC_lexicon_tags_46 },
    { 0x5e5024dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x93b024de, SUC_lexicon_tags_23 },
    { 0xed7024df, SUC_lexicon_tags_4 },
    { 0xe6d024de, SUC_lexicon_tags_8 },
    { 0xb67024e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935024e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x941024e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b024eb, SUC_lexicon_tags_15 },
    { 0x253024eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07024f2, SUC_lexicon_tags_60 },
    { 0x329024f3, SUC_lexicon_tags_11 },
    { 0x55d024f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xde9024f6, SUC_lexicon_tags_11 },
    { 0xc41024f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1024fa, SUC_lexicon_tags_15 },
    { 0xee5024fa, SUC_lexicon_tags_11 },
    { 0x8f024fa, SUC_lexicon_tags_63 },
    { 0xe6b024fd, SUC_lexicon_tags_63 },
    { 0x333024fc, SUC_lexicon_tags_1 },
    { 0xf95024ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x54502501, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcd02503, SUC_lexicon_tags_35 },
    { 0x7fd02503, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79d02506, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4610250c, SUC_lexicon_tags_63 },
    { 0x35f0250d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2b0250f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94d02511, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e102518, SUC_lexicon_tags_12 },
    { 0x62902518, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda90251c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22702524, SUC_lexicon_tags_10 },
    { 0x5bd02524, SUC_lexicon_tags_45 },
    { 0x45702526, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330252b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7702530, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x12102532, SUC_lexicon_tags_17 },
    { 0xaeb02532, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb9102535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45f02537, SUC_lexicon_tags_2 },
    { 0xe0102538, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3290253b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c90253d, SUC_lexicon_tags_46 },
    { 0x8270253e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77902542, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x77302544, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x74b02546, SUC_lexicon_tags_15 },
    { 0xc3d02547, SUC_lexicon_tags_25 },
    { 0x1502548, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa010254a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb090254f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d02555, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa9f02557, SUC_lexicon_tags_25 },
    { 0xf2302557, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3af0255b, SUC_lexicon_tags_25 },
    { 0xde50255c, SUC_lexicon_tags_62 },
    { 0x650255c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0255f, SUC_lexicon_tags_77 },
    { 0xa6902560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd02566, SUC_lexicon_tags_1 },
    { 0xb5d02567, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xefb02569, SUC_lexicon_tags_33 },
    { 0x1070256a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b50256d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5030256f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93902573, SUC_lexicon_tags_2 },
    { 0xca502574, SUC_lexicon_tags_8 },
    { 0x7c902573, SUC_lexicon_tags_37 },
    { 0x54702573, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5b02578, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc390257c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24b0257e, SUC_lexicon_tags_61 },
    { 0x81f0257f, SUC_lexicon_tags_16 },
    { 0xfad02580, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd702582, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf902586, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfe102588, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2650258a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5702590, SUC_lexicon_tags_19 },
    { 0xf9302590, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x76302593, SUC_lexicon_tags_7 },
    { 0x76102594, SUC_lexicon_tags_1 },
    { 0xa5902595, SUC_lexicon_tags_23 },
    { 0xcd902596, SUC_lexicon_tags_33 },
    { 0xbf902596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe310259a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f0259f, SUC_lexicon_tags_1 },
    { 0x8a5025a0, SUC_lexicon_tags_38 },
    { 0xf5025a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31025a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603025aa, SUC_lexicon_tags_70 },
    { 0x571025aa, SUC_lexicon_tags_39 },
    { 0x633025ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa6f025ae, SUC_lexicon_tags_9 },
    { 0x2e5025af, SUC_lexicon_tags_37 },
    { 0xaf3025b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x167025b2, SUC_lexicon_tags_12 },
    { 0xe61025b2, SUC_lexicon_tags_27 },
    { 0xe91025b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335025bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1025be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cd025c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x815025c2, SUC_lexicon_tags_47 },
    { 0xb95025c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1025c6, SUC_lexicon_tags_9 },
    { 0x24b025c6, SUC_lexicon_tags_2 },
    { 0xb4f025c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3025cb, SUC_lexicon_tags_1 },
    { 0xc61025cb, SUC_lexicon_tags_1091 },
    { 0, NULL },
    { 0x4a7025ce, SUC_lexicon_tags_1 },
    { 0x221025ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5025d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x513025d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1025db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9025dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x561025df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7025e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5025e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43025e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b025f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d025f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x845025f5, SUC_lexicon_tags_8 },
    { 0x8e1025f5, SUC_lexicon_tags_10 },
    { 0x569025f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x155025fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x535025fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f5025ff, SUC_lexicon_tags_17 },
    { 0xd25025ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x79902603, SUC_lexicon_tags_61 },
    { 0x96302603, SUC_lexicon_tags_60 },
    { 0x13f02605, SUC_lexicon_tags_11 },
    { 0xae902605, SUC_lexicon_tags_43 },
    { 0x8f502605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9050260c, SUC_lexicon_tags_37 },
    { 0x79d0260d, SUC_lexicon_tags_17 },
    { 0x190260c, SUC_lexicon_tags_7 },
    { 0x4f10260f, SUC_lexicon_tags_2 },
    { 0xb5902610, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62502612, SUC_lexicon_tags_3 },
    { 0xb9f02612, SUC_lexicon_tags_87 },
    { 0x29502614, SUC_lexicon_tags_33 },
    { 0xb6302615, SUC_lexicon_tags_35 },
    { 0x76b02614, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd502619, SUC_lexicon_tags_31 },
    { 0xad0261a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9950261f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b02625, SUC_lexicon_tags_18 },
    { 0xd2b02625, SUC_lexicon_tags_11 },
    { 0xfd702627, SUC_lexicon_tags_31 },
    { 0x12b02628, SUC_lexicon_tags_0 },
    { 0xe6b02629, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce902633, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3d302635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b70263a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4702640, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd02644, SUC_lexicon_tags_45 },
    { 0x8eb02644, SUC_lexicon_tags_27 },
    { 0x8dd02646, SUC_lexicon_tags_11 },
    { 0x98902647, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d30264b, SUC_lexicon_tags_37 },
    { 0x37b0264c, SUC_lexicon_tags_7 },
    { 0xe1b0264d, SUC_lexicon_tags_29 },
    { 0x9890264d, SUC_lexicon_tags_55 },
    { 0x2810264f, SUC_lexicon_tags_11 },
    { 0x3250264b, SUC_lexicon_tags_37 },
    { 0xf5f0264d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe6b02653, SUC_lexicon_tags_2 },
    { 0x56502653, SUC_lexicon_tags_104 },
    { 0x76b02654, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d0265e, SUC_lexicon_tags_0 },
    { 0x9d70265f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2702663, SUC_lexicon_tags_84 },
    { 0xb2902664, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6502667, SUC_lexicon_tags_37 },
    { 0xab102667, SUC_lexicon_tags_11 },
    { 0xa2302667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d02671, SUC_lexicon_tags_11 },
    { 0xb4502672, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b902674, SUC_lexicon_tags_57 },
    { 0x3a102674, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc590267b, SUC_lexicon_tags_12 },
    { 0x3e10267c, SUC_lexicon_tags_37 },
    { 0x5af0267b, SUC_lexicon_tags_28 },
    { 0x6790267c, SUC_lexicon_tags_17 },
    { 0x550267f, SUC_lexicon_tags_12 },
    { 0xb102680, SUC_lexicon_tags_25 },
    { 0x8f02681, SUC_lexicon_tags_63 },
    { 0x7350267e, SUC_lexicon_tags_19 },
    { 0xb850267c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa902685, SUC_lexicon_tags_46 },
    { 0x56d02685, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeed02688, SUC_lexicon_tags_1 },
    { 0xeeb02688, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4f70268b, SUC_lexicon_tags_1 },
    { 0xb090268c, SUC_lexicon_tags_37 },
    { 0xa330268d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaf50268f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb302693, SUC_lexicon_tags_31 },
    { 0x42b02693, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x502699, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2710269b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb10269d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9026a1, SUC_lexicon_tags_47 },
    { 0x38d026a1, SUC_lexicon_tags_299 },
    { 0x3bb026a3, SUC_lexicon_tags_92 },
    { 0x765026a4, SUC_lexicon_tags_15 },
    { 0x263026a3, SUC_lexicon_tags_4 },
    { 0x99b026a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b026ac, SUC_lexicon_tags_4 },
    { 0x871026ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d026b6, SUC_lexicon_tags_91 },
    { 0x3df026b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3026ba, SUC_lexicon_tags_25 },
    { 0xbf7026bb, SUC_lexicon_tags_8 },
    { 0xd9b026bc, SUC_lexicon_tags_15 },
    { 0xf9f026bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5026c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7f026c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59026c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x899026c8, SUC_lexicon_tags_21 },
    { 0x6e1026c9, SUC_lexicon_tags_12 },
    { 0x1a9026ca, SUC_lexicon_tags_63 },
    { 0x571026cb, SUC_lexicon_tags_15 },
    { 0x37b026cb, SUC_lexicon_tags_33 },
    { 0x173026cd, SUC_lexicon_tags_15 },
    { 0x221026cd, SUC_lexicon_tags_39 },
    { 0xcf026cf, SUC_lexicon_tags_30 },
    { 0xc81026cf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f026d3, SUC_lexicon_tags_74 },
    { 0x86d026d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbe3026d6, SUC_lexicon_tags_15 },
    { 0xb37026d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x383026da, SUC_lexicon_tags_11 },
    { 0x163026db, SUC_lexicon_tags_2 },
    { 0xe0f026da, SUC_lexicon_tags_27 },
    { 0xa7d026db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd026df, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647026e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96d026ea, SUC_lexicon_tags_2 },
    { 0x989026ea, SUC_lexicon_tags_35 },
    { 0x2ff026ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9026f1, SUC_lexicon_tags_25 },
    { 0x633026f1, SUC_lexicon_tags_25 },
    { 0xe79026f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f026f8, SUC_lexicon_tags_17 },
    { 0xa7d026f9, SUC_lexicon_tags_16 },
    { 0xa8d026fa, SUC_lexicon_tags_47 },
    { 0x3026fb, SUC_lexicon_tags_11 },
    { 0xa35026fc, SUC_lexicon_tags_31 },
    { 0xb3b026fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5902707, SUC_lexicon_tags_12 },
    { 0xad102708, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d0270b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb0270e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff02716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90271a, SUC_lexicon_tags_15 },
    { 0xfd90271a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad0271e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0d02720, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b02723, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd02728, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f02730, SUC_lexicon_tags_59 },
    { 0x99f02731, SUC_lexicon_tags_11 },
    { 0xcc302732, SUC_lexicon_tags_11 },
    { 0x86902733, SUC_lexicon_tags_23 },
    { 0x93d02733, SUC_lexicon_tags_47 },
    { 0x8af02734, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e30273e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e902747, SUC_lexicon_tags_19 },
    { 0x83702747, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbcd0274a, SUC_lexicon_tags_47 },
    { 0xfe10274a, SUC_lexicon_tags_40 },
    { 0x53d0274c, SUC_lexicon_tags_733 },
    { 0x9450274d, SUC_lexicon_tags_47 },
    { 0x1d50274e, SUC_lexicon_tags_13 },
    { 0x3790274e, SUC_lexicon_tags_12 },
    { 0xc5f02750, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56102752, SUC_lexicon_tags_80 },
    { 0xe8f02753, SUC_lexicon_tags_29 },
    { 0x7502754, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7302757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1790275c, SUC_lexicon_tags_28 },
    { 0xef0275c, SUC_lexicon_tags_32 },
    { 0xac50275e, SUC_lexicon_tags_82 },
    { 0x210275d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d02764, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1702767, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d0276a, SUC_lexicon_tags_14 },
    { 0x4590276b, SUC_lexicon_tags_33 },
    { 0xff50276b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfb30276e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58902773, SUC_lexicon_tags_47 },
    { 0xca502774, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf102778, SUC_lexicon_tags_25 },
    { 0xab102779, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf970277c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f02781, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d02785, SUC_lexicon_tags_6 },
    { 0x37702786, SUC_lexicon_tags_23 },
    { 0x6bd02786, SUC_lexicon_tags_17 },
    { 0x45b02785, SUC_lexicon_tags_2 },
    { 0xef02789, SUC_lexicon_tags_37 },
    { 0x8970278a, SUC_lexicon_tags_53 },
    { 0x9b502786, SUC_lexicon_tags_15 },
    { 0x2c90278c, SUC_lexicon_tags_57 },
    { 0x5b50278d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b02790, SUC_lexicon_tags_66 },
    { 0x94502791, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f02799, SUC_lexicon_tags_33 },
    { 0x5850279a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc350279c, SUC_lexicon_tags_10 },
    { 0x3f50279d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc930279f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7027a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x445027a4, SUC_lexicon_tags_23 },
    { 0xd11027a5, SUC_lexicon_tags_21 },
    { 0xdff027a6, SUC_lexicon_tags_45 },
    { 0x415027a7, SUC_lexicon_tags_12 },
    { 0xf5d027a6, SUC_lexicon_tags_47 },
    { 0x6bb027a6, SUC_lexicon_tags_49 },
    { 0x905027a7, SUC_lexicon_tags_2 },
    { 0x4d027ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3027b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71b027b9, SUC_lexicon_tags_37 },
    { 0x7ef027ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe5027bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397027c0, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0x2c9027c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49027c6, SUC_lexicon_tags_92 },
    { 0x6b3027c7, SUC_lexicon_tags_11 },
    { 0xb39027c7, SUC_lexicon_tags_8 },
    { 0xd25027c8, SUC_lexicon_tags_4 },
    { 0x7b9027ca, SUC_lexicon_tags_47 },
    { 0x709027c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe43027cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251027d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x375027d6, SUC_lexicon_tags_17 },
    { 0x44b027d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd027dd, SUC_lexicon_tags_17 },
    { 0xea9027de, SUC_lexicon_tags_14 },
    { 0xbef027de, SUC_lexicon_tags_1 },
    { 0xbbf027e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99027e5, SUC_lexicon_tags_13 },
    { 0x8c7027e5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x353027e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x85d027ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ef027ec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x931027ef, SUC_lexicon_tags_43 },
    { 0x221027f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3027f5, SUC_lexicon_tags_25 },
    { 0xbd7027f6, SUC_lexicon_tags_37 },
    { 0xfe3027f6, SUC_lexicon_tags_14 },
    { 0xcbb027f5, SUC_lexicon_tags_39 },
    { 0x7bb027f8, SUC_lexicon_tags_6 },
    { 0xf63027f8, SUC_lexicon_tags_9 },
    { 0xe71027fb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x471027fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bf027ff, SUC_lexicon_tags_178 },
    { 0x82d027ff, SUC_lexicon_tags_27 },
    { 0x72302800, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71502806, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79d02808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5b0280a, SUC_lexicon_tags_47 },
    { 0xd010280b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb3f0280d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66b0280f, SUC_lexicon_tags_30 },
    { 0x1a502810, SUC_lexicon_tags_33 },
    { 0xc7102811, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8302813, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35902818, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf50281f, SUC_lexicon_tags_32 },
    { 0x9db02820, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89502826, SUC_lexicon_tags_29 },
    { 0xadf02827, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbd502829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1690282c, SUC_lexicon_tags_12 },
    { 0x5390282c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70702832, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b302835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41502838, SUC_lexicon_tags_7 },
    { 0xb5102839, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78f0283b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26b0283d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e702840, SUC_lexicon_tags_25 },
    { 0x46302841, SUC_lexicon_tags_48 },
    { 0x91102841, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3902844, SUC_lexicon_tags_7 },
    { 0x50b02844, SUC_lexicon_tags_1 },
    { 0xb6902846, SUC_lexicon_tags_66 },
    { 0xb9f02844, SUC_lexicon_tags_1 },
    { 0xa4502845, SUC_lexicon_tags_17 },
    { 0xb5902849, SUC_lexicon_tags_0 },
    { 0x39b0284a, SUC_lexicon_tags_9 },
    { 0xa1f0284a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf0284e, SUC_lexicon_tags_1 },
    { 0xbfd0284f, SUC_lexicon_tags_7 },
    { 0x9c302850, SUC_lexicon_tags_11 },
    { 0x13102851, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1302854, SUC_lexicon_tags_10 },
    { 0x6af02855, SUC_lexicon_tags_27 },
    { 0xec902856, SUC_lexicon_tags_8 },
    { 0x13702856, SUC_lexicon_tags_31 },
    { 0x9db02857, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a50285b, SUC_lexicon_tags_37 },
    { 0xd510285c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f02864, SUC_lexicon_tags_1 },
    { 0x55502865, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f02868, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b0286b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a90286d, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d02871, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35102879, SUC_lexicon_tags_12 },
    { 0x4110287a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6190287d, SUC_lexicon_tags_12 },
    { 0xfb10287e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x20b02880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9702882, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d702885, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36302887, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8430288c, SUC_lexicon_tags_25 },
    { 0xb790288c, SUC_lexicon_tags_1 },
    { 0x20d0288e, SUC_lexicon_tags_31 },
    { 0xe730288f, SUC_lexicon_tags_7 },
    { 0x7df0288d, SUC_lexicon_tags_30 },
    { 0x1c50288c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5502893, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66b02895, SUC_lexicon_tags_56 },
    { 0x65102896, SUC_lexicon_tags_2 },
    { 0x53f02895, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x75702899, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4570289b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7630289d, SUC_lexicon_tags_4 },
    { 0xfff0289e, SUC_lexicon_tags_17 },
    { 0xb290289f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf69028a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x321028a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x577028a5, SUC_lexicon_tags_43 },
    { 0x1b028a6, SUC_lexicon_tags_37 },
    { 0xd03028a7, SUC_lexicon_tags_25 },
    { 0xe3b028a5, SUC_lexicon_tags_27 },
    { 0x99028a8, SUC_lexicon_tags_27 },
    { 0xabf028aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b028af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11028b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29028bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7028c0, SUC_lexicon_tags_60 },
    { 0x17d028c1, SUC_lexicon_tags_2 },
    { 0x615028c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x173028c5, SUC_lexicon_tags_25 },
    { 0x303028c5, SUC_lexicon_tags_14 },
    { 0xa49028c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf1028c9, SUC_lexicon_tags_45 },
    { 0x7e9028ca, SUC_lexicon_tags_7 },
    { 0x4d3028ca, SUC_lexicon_tags_35 },
    { 0x883028cc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa4b028ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1028d2, SUC_lexicon_tags_60 },
    { 0x9ed028d3, SUC_lexicon_tags_30 },
    { 0x265028d3, SUC_lexicon_tags_38 },
    { 0x233028d5, SUC_lexicon_tags_7 },
    { 0x9cf028d5, SUC_lexicon_tags_15 },
    { 0x6ab028d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x231028da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f028dd, SUC_lexicon_tags_37 },
    { 0xe23028de, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d028e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b7028e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801028e8, SUC_lexicon_tags_8 },
    { 0xaeb028e9, SUC_lexicon_tags_44 },
    { 0x293028ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9028f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x969028f3, SUC_lexicon_tags_82 },
    { 0x293028f4, SUC_lexicon_tags_5 },
    { 0xb5028f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d028f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b028fe, SUC_lexicon_tags_11 },
    { 0xaef028ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad02903, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x27502906, SUC_lexicon_tags_49 },
    { 0x77302907, SUC_lexicon_tags_4 },
    { 0x7ab02908, SUC_lexicon_tags_4 },
    { 0x75102909, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc030290b, SUC_lexicon_tags_2 },
    { 0xb530290b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93302910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25702912, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30102917, SUC_lexicon_tags_61 },
    { 0xde302918, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1030291a, SUC_lexicon_tags_33 },
    { 0xa9f0291b, SUC_lexicon_tags_49 },
    { 0x1730291a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12b0291e, SUC_lexicon_tags_43 },
    { 0x9f90291f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3502925, SUC_lexicon_tags_33 },
    { 0xdcd02925, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f902931, SUC_lexicon_tags_9 },
    { 0xc4502931, SUC_lexicon_tags_32 },
    { 0xb6702931, SUC_lexicon_tags_27 },
    { 0x94102934, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b02939, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1910293b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b02942, SUC_lexicon_tags_26 },
    { 0xe5102943, SUC_lexicon_tags_55 },
    { 0xb9902944, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f02947, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b502949, SUC_lexicon_tags_11 },
    { 0x9b0294a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95f0294c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2102950, SUC_lexicon_tags_43 },
    { 0xc4d02951, SUC_lexicon_tags_25 },
    { 0xfa302952, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82b02954, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9502957, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f02960, SUC_lexicon_tags_11 },
    { 0xd6502961, SUC_lexicon_tags_47 },
    { 0x32d02961, SUC_lexicon_tags_23 },
    { 0xb6502963, SUC_lexicon_tags_47 },
    { 0xea902962, SUC_lexicon_tags_37 },
    { 0x2d702964, SUC_lexicon_tags_35 },
    { 0x13b02966, SUC_lexicon_tags_7 },
    { 0xc3902967, SUC_lexicon_tags_49 },
    { 0xa1902968, SUC_lexicon_tags_8 },
    { 0x82f02967, SUC_lexicon_tags_45 },
    { 0x7210296a, SUC_lexicon_tags_9 },
    { 0x1350296b, SUC_lexicon_tags_15 },
    { 0x8e30296b, SUC_lexicon_tags_65 },
    { 0x3d30296d, SUC_lexicon_tags_1 },
    { 0xb650296e, SUC_lexicon_tags_11 },
    { 0x6570296d, SUC_lexicon_tags_15 },
    { 0x7702970, SUC_lexicon_tags_17 },
    { 0x5f02971, SUC_lexicon_tags_14 },
    { 0x63502968, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d02975, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1702978, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1af0297a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d0297f, SUC_lexicon_tags_19 },
    { 0xf910297f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24302984, SUC_lexicon_tags_65 },
    { 0x90702985, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf02988, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc010298c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d502990, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0302996, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe450299b, SUC_lexicon_tags_37 },
    { 0x7950299c, SUC_lexicon_tags_13 },
    { 0x4a70299d, SUC_lexicon_tags_17 },
    { 0xc370299d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x183029a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5029a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f029a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x335029a8, SUC_lexicon_tags_35 },
    { 0xa1d029a9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b029b2, SUC_lexicon_tags_1087 },
    { 0xecd029b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70f029b5, SUC_lexicon_tags_11 },
    { 0xa3029b5, SUC_lexicon_tags_63 },
    { 0xb71029b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09029bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x157029be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3029c1, SUC_lexicon_tags_30 },
    { 0xe43029c2, SUC_lexicon_tags_16 },
    { 0xf17029c3, SUC_lexicon_tags_113 },
    { 0x28b029c4, SUC_lexicon_tags_48 },
    { 0x89f029c5, SUC_lexicon_tags_25 },
    { 0xd83029c6, SUC_lexicon_tags_17 },
    { 0xca3029c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19029cb, SUC_lexicon_tags_49 },
    { 0x413029cc, SUC_lexicon_tags_1 },
    { 0xcdb029cc, SUC_lexicon_tags_12 },
    { 0xf75029cb, SUC_lexicon_tags_27 },
    { 0xcd3029cf, SUC_lexicon_tags_61 },
    { 0xcc5029cc, SUC_lexicon_tags_46 },
    { 0xc43029d1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa8b029d3, SUC_lexicon_tags_17 },
    { 0x4f1029d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d029db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109029e2, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1029e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x699029ea, SUC_lexicon_tags_30 },
    { 0x3e7029ea, SUC_lexicon_tags_7 },
    { 0xf83029ec, SUC_lexicon_tags_37 },
    { 0x11029ec, SUC_lexicon_tags_48 },
    { 0x89d029ee, SUC_lexicon_tags_34 },
    { 0x8d9029ee, SUC_lexicon_tags_37 },
    { 0x5bb029eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc23029f2, SUC_lexicon_tags_37 },
    { 0xe35029f2, SUC_lexicon_tags_27 },
    { 0x953029f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x537029f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1029f9, SUC_lexicon_tags_1402 },
    { 0, NULL },
    { 0x9e3029fb, SUC_lexicon_tags_11 },
    { 0x5ed029fc, SUC_lexicon_tags_37 },
    { 0x5eb029fb, SUC_lexicon_tags_17 },
    { 0x787029fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d02a03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9102a05, SUC_lexicon_tags_17 },
    { 0xf9702a06, SUC_lexicon_tags_45 },
    { 0x66102a07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1902a0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb302a0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17f02a11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x48502a13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87502a15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a502a18, SUC_lexicon_tags_37 },
    { 0xa7102a19, SUC_lexicon_tags_17 },
    { 0xcf302a1a, SUC_lexicon_tags_12 },
    { 0x1c302a1b, SUC_lexicon_tags_29 },
    { 0xaf102a1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9502a1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd02a24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21302a2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f02a32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8b102a34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d302a36, SUC_lexicon_tags_15 },
    { 0xdd902a37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba102a3b, SUC_lexicon_tags_66 },
    { 0xfe102a3c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d702a3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b02a42, SUC_lexicon_tags_15 },
    { 0x19f02a43, SUC_lexicon_tags_15 },
    { 0x15b02a42, SUC_lexicon_tags_2 },
    { 0x32702a45, SUC_lexicon_tags_38 },
    { 0xf6502a45, SUC_lexicon_tags_46 },
    { 0x34b02a47, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83902a4f, SUC_lexicon_tags_26 },
    { 0x2502a4f, SUC_lexicon_tags_37 },
    { 0xdf902a51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f02a5a, SUC_lexicon_tags_29 },
    { 0x32102a5b, SUC_lexicon_tags_48 },
    { 0xde902a5c, SUC_lexicon_tags_178 },
    { 0x6102a5d, SUC_lexicon_tags_48 },
    { 0x52702a5c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaed02a60, SUC_lexicon_tags_38 },
    { 0x6a302a61, SUC_lexicon_tags_11 },
    { 0xe7f02a62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa702a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96f02a66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed702a6b, SUC_lexicon_tags_87 },
    { 0x4dd02a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42502a6e, SUC_lexicon_tags_15 },
    { 0xe7302a6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e302a72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e702a74, SUC_lexicon_tags_11 },
    { 0x90702a75, SUC_lexicon_tags_33 },
    { 0xc7302a76, SUC_lexicon_tags_14 },
    { 0x13f02a76, SUC_lexicon_tags_11 },
    { 0xc0102a78, SUC_lexicon_tags_37 },
    { 0x30502a78, SUC_lexicon_tags_17 },
    { 0x42702a77, SUC_lexicon_tags_10 },
    { 0xea102a75, SUC_lexicon_tags_47 },
    { 0x8dd02a7c, SUC_lexicon_tags_25 },
    { 0x7e102a7d, SUC_lexicon_tags_17 },
    { 0xea902a7d, SUC_lexicon_tags_47 },
    { 0xe8102a7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5702a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce302a87, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf02a8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91302a8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2502a8f, SUC_lexicon_tags_30 },
    { 0x68502a8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b902a95, SUC_lexicon_tags_11 },
    { 0x44302a96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7702a98, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f02aa0, SUC_lexicon_tags_11 },
    { 0x9c502aa1, SUC_lexicon_tags_37 },
    { 0x71f02aa0, SUC_lexicon_tags_43 },
    { 0x8a702aa1, SUC_lexicon_tags_17 },
    { 0x4e302aa2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdf102aa6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7e902aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3502aab, SUC_lexicon_tags_8 },
    { 0xd4902aac, SUC_lexicon_tags_91 },
    { 0x11b02aad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97302ab5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf302abd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2db02abf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x43702ac1, SUC_lexicon_tags_1 },
    { 0xe3302ac2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xeed02ac4, SUC_lexicon_tags_17 },
    { 0x16102ac5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b02ac9, SUC_lexicon_tags_29 },
    { 0x73302aca, SUC_lexicon_tags_64 },
    { 0xdaf02acb, SUC_lexicon_tags_5 },
    { 0x87d02aca, SUC_lexicon_tags_3 },
    { 0x37702acc, SUC_lexicon_tags_12 },
    { 0xc3d02ace, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8302ad0, SUC_lexicon_tags_7 },
    { 0x53502ad1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf02ad4, SUC_lexicon_tags_16 },
    { 0xeb702ad5, SUC_lexicon_tags_7 },
    { 0x97102ad6, SUC_lexicon_tags_2 },
    { 0x84f02ad5, SUC_lexicon_tags_17 },
    { 0x8fd02ad8, SUC_lexicon_tags_25 },
    { 0x34102ad9, SUC_lexicon_tags_14 },
    { 0xe5902ad9, SUC_lexicon_tags_17 },
    { 0x8d302ad7, SUC_lexicon_tags_6 },
    { 0xf4902adc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b02ae1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde302ae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7902ae7, SUC_lexicon_tags_20 },
    { 0xf7b02ae8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d102aeb, SUC_lexicon_tags_19 },
    { 0x22702aec, SUC_lexicon_tags_66 },
    { 0xc0102aed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xebb02aef, SUC_lexicon_tags_11 },
    { 0x35d02af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41b02af2, SUC_lexicon_tags_17 },
    { 0x74102af3, SUC_lexicon_tags_11 },
    { 0x52502af4, SUC_lexicon_tags_35 },
    { 0xad02af3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d502af7, SUC_lexicon_tags_15 },
    { 0x2d902af8, SUC_lexicon_tags_1 },
    { 0x95702af8, SUC_lexicon_tags_11 },
    { 0xf5502af9, SUC_lexicon_tags_11 },
    { 0x86902afb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1302aff, SUC_lexicon_tags_14 },
    { 0x5db02b00, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x34302b03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90702b06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f02b09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd02b0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1b02b10, SUC_lexicon_tags_23 },
    { 0x63702b10, SUC_lexicon_tags_7 },
    { 0x1fd02b12, SUC_lexicon_tags_0 },
    { 0x73702b11, SUC_lexicon_tags_15 },
    { 0x65d02b14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b502b17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f02b1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8a702b1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27502b21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff02b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83d02b2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42902b31, SUC_lexicon_tags_17 },
    { 0xedb02b32, SUC_lexicon_tags_47 },
    { 0x54702b31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef02b36, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xdc502b38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x74f02b3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c302b3e, SUC_lexicon_tags_70 },
    { 0x2a102b3f, SUC_lexicon_tags_30 },
    { 0xddf02b3f, SUC_lexicon_tags_2 },
    { 0xce502b41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d02b44, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17702b49, SUC_lexicon_tags_1 },
    { 0x7cb02b49, SUC_lexicon_tags_12 },
    { 0x29702b4b, SUC_lexicon_tags_37 },
    { 0x64702b4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8902b52, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x39102b55, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xcaf02b57, SUC_lexicon_tags_11 },
    { 0xc9d02b58, SUC_lexicon_tags_8 },
    { 0xd5f02b59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d02b5c, SUC_lexicon_tags_8 },
    { 0x9702b5d, SUC_lexicon_tags_37 },
    { 0x98d02b5c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x68f02b60, SUC_lexicon_tags_25 },
    { 0xce302b61, SUC_lexicon_tags_8 },
    { 0x19302b60, SUC_lexicon_tags_27 },
    { 0xfed02b61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50102b65, SUC_lexicon_tags_37 },
    { 0xcbf02b65, SUC_lexicon_tags_15 },
    { 0xbc702b65, SUC_lexicon_tags_31 },
    { 0x1b902b67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b02b6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93902b6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1902b71, SUC_lexicon_tags_15 },
    { 0x56102b72, SUC_lexicon_tags_28 },
    { 0xc502b73, SUC_lexicon_tags_11 },
    { 0x70302b73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d902b78, SUC_lexicon_tags_25 },
    { 0xe7302b78, SUC_lexicon_tags_14 },
    { 0x29f02b7a, SUC_lexicon_tags_11 },
    { 0x68502b7a, SUC_lexicon_tags_12 },
    { 0x9102b7c, SUC_lexicon_tags_16 },
    { 0x77302b7a, SUC_lexicon_tags_12 },
    { 0x4bb02b7e, SUC_lexicon_tags_43 },
    { 0xd1d02b7f, SUC_lexicon_tags_11 },
    { 0x8df02b7f, SUC_lexicon_tags_47 },
    { 0xaf102b81, SUC_lexicon_tags_15 },
    { 0x24302b82, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d02b87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb02b8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad902b8f, SUC_lexicon_tags_12 },
    { 0x78302b8f, SUC_lexicon_tags_9 },
    { 0xe5f02b91, SUC_lexicon_tags_38 },
    { 0x31f02b92, SUC_lexicon_tags_37 },
    { 0xd5502b90, SUC_lexicon_tags_19 },
    { 0xbbf02b90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb02b97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82702b9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcd302b9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41702ba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34b02ba4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b02ba7, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f02bad, SUC_lexicon_tags_25 },
    { 0x7d502bad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a302bb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabb02bb3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d02bb7, SUC_lexicon_tags_25 },
    { 0x24f02bb8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x56902bbb, SUC_lexicon_tags_15 },
    { 0x7c102bbc, SUC_lexicon_tags_12 },
    { 0xb7b02bbc, SUC_lexicon_tags_14 },
    { 0x8702bbe, SUC_lexicon_tags_19 },
    { 0x9d302bbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc102bc5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf02bce, SUC_lexicon_tags_19 },
    { 0xea702bcf, SUC_lexicon_tags_25 },
    { 0xfcb02bd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48d02bd2, SUC_lexicon_tags_25 },
    { 0x4bd02bd3, SUC_lexicon_tags_25 },
    { 0xb8502bd4, SUC_lexicon_tags_37 },
    { 0x80902bd3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd02bd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9102bdc, SUC_lexicon_tags_21 },
    { 0x6c902bdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88702be0, SUC_lexicon_tags_1 },
    { 0x66102be1, SUC_lexicon_tags_12 },
    { 0x1102be1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59302be4, SUC_lexicon_tags_0 },
    { 0xb0702be5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f02be9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44d02beb, SUC_lexicon_tags_37 },
    { 0xf2902bec, SUC_lexicon_tags_2 },
    { 0x8d502bed, SUC_lexicon_tags_1061 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d02bf0, SUC_lexicon_tags_81 },
    { 0xc8102bf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58702bf5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50702bf9, SUC_lexicon_tags_28 },
    { 0xaff02bf9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1302bfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b02c00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x20902c02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7702c09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1902c0b, SUC_lexicon_tags_17 },
    { 0x63502c0c, SUC_lexicon_tags_23 },
    { 0x7a902c0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd02c11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2f02c13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf02c17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb02c1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x902c1c, SUC_lexicon_tags_17 },
    { 0x88302c1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc102c24, SUC_lexicon_tags_17 },
    { 0xf0d02c25, SUC_lexicon_tags_34 },
    { 0xf4302c26, SUC_lexicon_tags_8 },
    { 0xcc102c26, SUC_lexicon_tags_7 },
    { 0xabd02c27, SUC_lexicon_tags_15 },
    { 0xa1b02c29, SUC_lexicon_tags_35 },
    { 0x63f02c28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3902c2d, SUC_lexicon_tags_37 },
    { 0xa0f02c2e, SUC_lexicon_tags_18 },
    { 0xd8b02c2e, SUC_lexicon_tags_30 },
    { 0x98302c2e, SUC_lexicon_tags_21 },
    { 0x1b02c31, SUC_lexicon_tags_2 },
    { 0x22302c31, SUC_lexicon_tags_10 },
    { 0x9c702c2e, SUC_lexicon_tags_46 },
    { 0xf702c33, SUC_lexicon_tags_11 },
    { 0x68902c35, SUC_lexicon_tags_65 },
    { 0xb3302c35, SUC_lexicon_tags_11 },
    { 0x35302c35, SUC_lexicon_tags_57 },
    { 0x94102c30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99502c3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa102c42, SUC_lexicon_tags_29 },
    { 0x6ed02c42, SUC_lexicon_tags_22 },
    { 0x9a902c42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5102c49, SUC_lexicon_tags_57 },
    { 0xd8502c49, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb02c4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8902c51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6102c53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5cb02c55, SUC_lexicon_tags_10 },
    { 0x37d02c55, SUC_lexicon_tags_31 },
    { 0x1d502c55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb02c5a, SUC_lexicon_tags_25 },
    { 0x2ad02c5a, SUC_lexicon_tags_12 },
    { 0xebf02c5c, SUC_lexicon_tags_0 },
    { 0x19f02c5a, SUC_lexicon_tags_17 },
    { 0xa9302c5d, SUC_lexicon_tags_48 },
    { 0x13102c5a, SUC_lexicon_tags_1 },
    { 0x45502c60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7102c68, SUC_lexicon_tags_26 },
    { 0xa9502c69, SUC_lexicon_tags_56 },
    { 0xad302c69, SUC_lexicon_tags_15 },
    { 0xd702c6a, SUC_lexicon_tags_39 },
    { 0xd2902c6c, SUC_lexicon_tags_7 },
    { 0x69302c6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66702c6f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x26d02c71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22902c79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32902c7f, SUC_lexicon_tags_63 },
    { 0xbe502c80, SUC_lexicon_tags_49 },
    { 0xb3702c81, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd3702c83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d902c86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27302c8b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc7702c8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f02c91, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6902c96, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df02c9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92702c9f, SUC_lexicon_tags_6 },
    { 0xe0902c9f, SUC_lexicon_tags_34 },
    { 0xa2f02ca1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb102caa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f902caf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8702cb3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8102cb6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4302cbd, SUC_lexicon_tags_11 },
    { 0x5502cbe, SUC_lexicon_tags_31 },
    { 0x1df02cbf, SUC_lexicon_tags_47 },
    { 0x94102cbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x66d02cc2, SUC_lexicon_tags_14 },
    { 0xa6102cc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f02cc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1102cc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40b02ccb, SUC_lexicon_tags_37 },
    { 0x1e102ccc, SUC_lexicon_tags_29 },
    { 0x39702ccb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15902ccf, SUC_lexicon_tags_9 },
    { 0xc7302cd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1902cd2, SUC_lexicon_tags_17 },
    { 0x4dd02cd3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24902cd7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab02cda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee302cdc, SUC_lexicon_tags_45 },
    { 0x9df02cdd, SUC_lexicon_tags_84 },
    { 0x15502cdc, SUC_lexicon_tags_21 },
    { 0x74b02cdf, SUC_lexicon_tags_32 },
    { 0xd7102ce0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7af02ce2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb02ce7, SUC_lexicon_tags_47 },
    { 0x5f502ce7, SUC_lexicon_tags_2 },
    { 0x26702ce9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1502ced, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b02cf0, SUC_lexicon_tags_17 },
    { 0xf9302cf0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f02cf4, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f02cf7, SUC_lexicon_tags_14 },
    { 0xdad02cf8, SUC_lexicon_tags_25 },
    { 0x7b702cf8, SUC_lexicon_tags_25 },
    { 0xfcf02cfa, SUC_lexicon_tags_11 },
    { 0xb2702cf8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb02d03, SUC_lexicon_tags_63 },
    { 0x98902d03, SUC_lexicon_tags_15 },
    { 0x9e902d05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c902d08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26f02d0a, SUC_lexicon_tags_12 },
    { 0xe2f02d0a, SUC_lexicon_tags_23 },
    { 0x56302d0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xadf02d0e, SUC_lexicon_tags_10 },
    { 0x19302d0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4502d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4f02d15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45102d17, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b02d1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe302d21, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f02d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33302d28, SUC_lexicon_tags_37 },
    { 0x75102d28, SUC_lexicon_tags_7 },
    { 0x5fd02d28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x79d02d2c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85702d33, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x10302d35, SUC_lexicon_tags_17 },
    { 0x7d502d35, SUC_lexicon_tags_27 },
    { 0xde302d37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c302d3c, SUC_lexicon_tags_0 },
    { 0xd9b02d3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf02d41, SUC_lexicon_tags_27 },
    { 0xf6b02d42, SUC_lexicon_tags_7 },
    { 0x9f102d42, SUC_lexicon_tags_6 },
    { 0x58b02d42, SUC_lexicon_tags_25 },
    { 0xa0502d45, SUC_lexicon_tags_26 },
    { 0x9a302d45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6302d48, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x68702d4a, SUC_lexicon_tags_14 },
    { 0x63702d4a, SUC_lexicon_tags_14 },
    { 0x45902d4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5f02d4e, SUC_lexicon_tags_60 },
    { 0xe4302d4f, SUC_lexicon_tags_15 },
    { 0xbe902d50, SUC_lexicon_tags_23 },
    { 0x53102d51, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57902d59, SUC_lexicon_tags_37 },
    { 0x13102d5a, SUC_lexicon_tags_19 },
    { 0x51102d5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb02d5e, SUC_lexicon_tags_63 },
    { 0xe1502d5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d02d65, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7102d67, SUC_lexicon_tags_29 },
    { 0xfb102d68, SUC_lexicon_tags_25 },
    { 0xb9502d69, SUC_lexicon_tags_25 },
    { 0xf0d02d68, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1902d74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf02d79, SUC_lexicon_tags_25 },
    { 0xf3702d79, SUC_lexicon_tags_17 },
    { 0x6af02d7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64502d7d, SUC_lexicon_tags_23 },
    { 0x48b02d7e, SUC_lexicon_tags_32 },
    { 0x102d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c502d85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98502d87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6702d8f, SUC_lexicon_tags_35 },
    { 0xd9302d90, SUC_lexicon_tags_27 },
    { 0x2d302d90, SUC_lexicon_tags_25 },
    { 0x50302d8f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3502d95, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f02d99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6dd02d9b, SUC_lexicon_tags_2 },
    { 0x92302d9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19102da2, SUC_lexicon_tags_27 },
    { 0x99702da2, SUC_lexicon_tags_55 },
    { 0xc502da2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9d02da6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5702dac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9502db0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0702db2, SUC_lexicon_tags_63 },
    { 0x43302db3, SUC_lexicon_tags_14 },
    { 0xb3902db4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0502db6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd02db9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a702dbc, SUC_lexicon_tags_60 },
    { 0xb7502dbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49902dc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4302dc5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x57d02dc7, SUC_lexicon_tags_25 },
    { 0x16102dc8, SUC_lexicon_tags_146 },
    { 0x9bb02dc9, SUC_lexicon_tags_17 },
    { 0xc2d02dca, SUC_lexicon_tags_0 },
    { 0x4a902dcb, SUC_lexicon_tags_31 },
    { 0x3c502dc7, SUC_lexicon_tags_43 },
    { 0xd3702dcd, SUC_lexicon_tags_10 },
    { 0x53302dce, SUC_lexicon_tags_1 },
    { 0x97102dcd, SUC_lexicon_tags_14 },
    { 0x91302dd0, SUC_lexicon_tags_16 },
    { 0x1f02dd1, SUC_lexicon_tags_124 },
    { 0xa0902dd2, SUC_lexicon_tags_60 },
    { 0xd2b02dd3, SUC_lexicon_tags_39 },
    { 0x90d02dd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f02dd8, SUC_lexicon_tags_29 },
    { 0x25b02dd9, SUC_lexicon_tags_17 },
    { 0x1b102dda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc102ddc, SUC_lexicon_tags_29 },
    { 0x36502ddd, SUC_lexicon_tags_10 },
    { 0x5df02ddd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f02de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b302de3, SUC_lexicon_tags_1 },
    { 0xbed02de4, SUC_lexicon_tags_28 },
    { 0x6e102de5, SUC_lexicon_tags_6 },
    { 0xca702de5, SUC_lexicon_tags_42 },
    { 0xb7f02de4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee302def, SUC_lexicon_tags_14 },
    { 0xaf902df0, SUC_lexicon_tags_31 },
    { 0xecf02df1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x11502df3, SUC_lexicon_tags_9 },
    { 0xacd02df4, SUC_lexicon_tags_23 },
    { 0x3af02df5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba702dfc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b02e00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda702e08, SUC_lexicon_tags_31 },
    { 0xfb102e09, SUC_lexicon_tags_2 },
    { 0xa5b02e09, SUC_lexicon_tags_23 },
    { 0x4f702e0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90902e0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97502e11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95102e13, SUC_lexicon_tags_14 },
    { 0x14102e14, SUC_lexicon_tags_87 },
    { 0x78f02e15, SUC_lexicon_tags_11 },
    { 0xac902e13, SUC_lexicon_tags_31 },
    { 0x80302e14, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60702e1c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f02e1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d02e22, SUC_lexicon_tags_15 },
    { 0x7f02e23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e502e25, SUC_lexicon_tags_66 },
    { 0x8d02e26, SUC_lexicon_tags_1 },
    { 0xef902e27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb02e2c, SUC_lexicon_tags_37 },
    { 0xaa902e2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb6502e2f, SUC_lexicon_tags_4 },
    { 0x18702e30, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x84502e33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x85702e35, SUC_lexicon_tags_47 },
    { 0xdd902e36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9102e38, SUC_lexicon_tags_1 },
    { 0xd8b02e39, SUC_lexicon_tags_64 },
    { 0x47f02e38, SUC_lexicon_tags_1 },
    { 0xa4302e38, SUC_lexicon_tags_17 },
    { 0x25702e3a, SUC_lexicon_tags_30 },
    { 0xc7f02e3d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb02e41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e302e4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65302e50, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d02e53, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b02e57, SUC_lexicon_tags_30 },
    { 0x9ef02e58, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f02e5b, SUC_lexicon_tags_85 },
    { 0x66702e5c, SUC_lexicon_tags_26 },
    { 0xfdf02e5c, SUC_lexicon_tags_12 },
    { 0xee102e5e, SUC_lexicon_tags_146 },
    { 0xe3302e5f, SUC_lexicon_tags_60 },
    { 0x6d02e5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31102e62, SUC_lexicon_tags_21 },
    { 0x79102e63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0f02e65, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x15302e67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87b02e69, SUC_lexicon_tags_11 },
    { 0x81902e6a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53902e6e, SUC_lexicon_tags_56 },
    { 0xa2502e6e, SUC_lexicon_tags_37 },
    { 0x7ef02e6e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4102e73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e302e76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb102e79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b702e7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b02e7f, SUC_lexicon_tags_29 },
    { 0xdd02e80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb02e84, SUC_lexicon_tags_43 },
    { 0x3c302e84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd0102e87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d02e8d, SUC_lexicon_tags_2 },
    { 0xdf902e8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f02e91, SUC_lexicon_tags_6 },
    { 0xca102e91, SUC_lexicon_tags_19 },
    { 0xd6302e93, SUC_lexicon_tags_198 },
    { 0x46f02e94, SUC_lexicon_tags_15 },
    { 0xf8502e95, SUC_lexicon_tags_15 },
    { 0xaeb02e96, SUC_lexicon_tags_17 },
    { 0xb6302e97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0102e9a, SUC_lexicon_tags_7 },
    { 0xa5b02e9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14902e9f, SUC_lexicon_tags_87 },
    { 0xe3502ea0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xddd02ea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8302ea6, SUC_lexicon_tags_198 },
    { 0x6fb02ea6, SUC_lexicon_tags_29 },
    { 0xec702ea8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84102eaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9cd02eac, SUC_lexicon_tags_11 },
    { 0x21102ead, SUC_lexicon_tags_0 },
    { 0x37f02eae, SUC_lexicon_tags_15 },
    { 0x5bb02ead, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfc302eb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3102eb4, SUC_lexicon_tags_14 },
    { 0xe3102eb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5702eb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa502ebb, SUC_lexicon_tags_48 },
    { 0x68f02ebc, SUC_lexicon_tags_11 },
    { 0x71902ebb, SUC_lexicon_tags_25 },
    { 0x54d02ebe, SUC_lexicon_tags_11 },
    { 0xbdf02ebf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f02ec7, SUC_lexicon_tags_28 },
    { 0xd0702ec8, SUC_lexicon_tags_1 },
    { 0x25b02ec9, SUC_lexicon_tags_39 },
    { 0xf3f02eca, SUC_lexicon_tags_600 },
    { 0x35f02eca, SUC_lexicon_tags_55 },
    { 0xaaf02ecc, SUC_lexicon_tags_7 },
    { 0x88702ecd, SUC_lexicon_tags_37 },
    { 0x13702ecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb02ed7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4302ed9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c502edf, SUC_lexicon_tags_30 },
    { 0x22302ee0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a302ee3, SUC_lexicon_tags_1 },
    { 0xd9302ee3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3902ee6, SUC_lexicon_tags_49 },
    { 0x38702ee6, SUC_lexicon_tags_47 },
    { 0x3d302ee8, SUC_lexicon_tags_29 },
    { 0x9f902ee6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0b02eeb, SUC_lexicon_tags_103 },
    { 0x31302eeb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d02ef0, SUC_lexicon_tags_1186 },
    { 0x4b502ef1, SUC_lexicon_tags_37 },
    { 0x6f302ef2, SUC_lexicon_tags_26 },
    { 0x64302ef3, SUC_lexicon_tags_38 },
    { 0xd2d02ef1, SUC_lexicon_tags_65 },
    { 0xe7502ef5, SUC_lexicon_tags_2 },
    { 0x52302ef5, SUC_lexicon_tags_27 },
    { 0x8e502ef6, SUC_lexicon_tags_17 },
    { 0x19702ef5, SUC_lexicon_tags_25 },
    { 0xa7b02ef1, SUC_lexicon_tags_14 },
    { 0x12d02efa, SUC_lexicon_tags_95 },
    { 0xbd02ef5, SUC_lexicon_tags_2 },
    { 0x5ad02ef4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46902f00, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x98102f02, SUC_lexicon_tags_25 },
    { 0xbed02f03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84102f05, SUC_lexicon_tags_45 },
    { 0x43702f06, SUC_lexicon_tags_87 },
    { 0xc0902f07, SUC_lexicon_tags_265 },
    { 0xb3502f08, SUC_lexicon_tags_32 },
    { 0x53f02f08, SUC_lexicon_tags_17 },
    { 0xebf02f0a, SUC_lexicon_tags_11 },
    { 0xc1f02f0b, SUC_lexicon_tags_40 },
    { 0xa3102f0a, SUC_lexicon_tags_57 },
    { 0x2a502f0d, SUC_lexicon_tags_2 },
    { 0x2c102f0d, SUC_lexicon_tags_2 },
    { 0xaa502f0e, SUC_lexicon_tags_17 },
    { 0x8d02f10, SUC_lexicon_tags_15 },
    { 0xa0f02f10, SUC_lexicon_tags_29 },
    { 0x77102f10, SUC_lexicon_tags_6 },
    { 0xab902f0f, SUC_lexicon_tags_55 },
    { 0x4f302f0e, SUC_lexicon_tags_87 },
    { 0x6bb02f15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1db02f17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd502f1a, SUC_lexicon_tags_43 },
    { 0x95902f1b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda702f22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b02f27, SUC_lexicon_tags_8 },
    { 0x71d02f27, SUC_lexicon_tags_21 },
    { 0xcf502f29, SUC_lexicon_tags_33 },
    { 0x32902f2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d902f2d, SUC_lexicon_tags_17 },
    { 0xd3502f2e, SUC_lexicon_tags_56 },
    { 0xc8b02f2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f502f36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18902f38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ff02f3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7502f3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c302f3f, SUC_lexicon_tags_17 },
    { 0x22702f40, SUC_lexicon_tags_21 },
    { 0x90302f40, SUC_lexicon_tags_7 },
    { 0x93502f42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xddb02f44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x16702f47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab02f4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df02f4f, SUC_lexicon_tags_79 },
    { 0x2302f4f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb8302f52, SUC_lexicon_tags_12 },
    { 0x27902f52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb02f57, SUC_lexicon_tags_37 },
    { 0x3e902f58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbf902f5a, SUC_lexicon_tags_56 },
    { 0xe7902f5b, SUC_lexicon_tags_38 },
    { 0x84702f5c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x26302f5f, SUC_lexicon_tags_33 },
    { 0x25902f60, SUC_lexicon_tags_38 },
    { 0xafb02f61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfb02f63, SUC_lexicon_tags_39 },
    { 0x2f302f63, SUC_lexicon_tags_12 },
    { 0x78502f64, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90502f69, SUC_lexicon_tags_12 },
    { 0x44702f6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad502f6d, SUC_lexicon_tags_6 },
    { 0xa7f02f6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48502f73, SUC_lexicon_tags_27 },
    { 0xd6f02f74, SUC_lexicon_tags_56 },
    { 0x42902f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f02f80, SUC_lexicon_tags_250 },
    { 0x3c902f81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x66b02f83, SUC_lexicon_tags_15 },
    { 0x4af02f84, SUC_lexicon_tags_63 },
    { 0xee702f84, SUC_lexicon_tags_12 },
    { 0xaa902f86, SUC_lexicon_tags_27 },
    { 0xaa502f85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70902f89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14102f8f, SUC_lexicon_tags_17 },
    { 0x71102f90, SUC_lexicon_tags_27 },
    { 0x7b502f90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea302f9c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4302f9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb8702fa1, SUC_lexicon_tags_27 },
    { 0xe0102fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7702fa5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf302fa8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54f02faa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea702fb2, SUC_lexicon_tags_39 },
    { 0x5dd02fb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f02fb9, SUC_lexicon_tags_56 },
    { 0x44102fba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1302fbe, SUC_lexicon_tags_28 },
    { 0x9902fbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3502fc2, SUC_lexicon_tags_12 },
    { 0x19d02fc3, SUC_lexicon_tags_2 },
    { 0x6d02fc4, SUC_lexicon_tags_84 },
    { 0x7a902fc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b02fcb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d702fd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78902fd3, SUC_lexicon_tags_11 },
    { 0xfb02fd4, SUC_lexicon_tags_21 },
    { 0x10f02fd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcb502fd7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d102fd9, SUC_lexicon_tags_15 },
    { 0xafd02fd9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde502fdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3102fe2, SUC_lexicon_tags_43 },
    { 0xe9b02fe3, SUC_lexicon_tags_0 },
    { 0xe5502fe4, SUC_lexicon_tags_12 },
    { 0x87902fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b02fe8, SUC_lexicon_tags_14 },
    { 0x60302fe8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c302feb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c102fee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e702ff4, SUC_lexicon_tags_23 },
    { 0x42302ff4, SUC_lexicon_tags_15 },
    { 0x9e302ff6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0902ffb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80902ffe, SUC_lexicon_tags_27 },
    { 0xf5b02ffe, SUC_lexicon_tags_21 },
    { 0x7ad02fff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d903006, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b903010, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b503013, SUC_lexicon_tags_68 },
    { 0xf0903014, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7250301b, SUC_lexicon_tags_22 },
    { 0xfbb0301b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74103020, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xead03022, SUC_lexicon_tags_68 },
    { 0x28d03023, SUC_lexicon_tags_11 },
    { 0x7d903023, SUC_lexicon_tags_61 },
    { 0x52f03025, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef103027, SUC_lexicon_tags_23 },
    { 0xeb103028, SUC_lexicon_tags_17 },
    { 0x81f03029, SUC_lexicon_tags_25 },
    { 0x9630302a, SUC_lexicon_tags_63 },
    { 0x4490302b, SUC_lexicon_tags_26 },
    { 0xa270302a, SUC_lexicon_tags_37 },
    { 0xa5b0302d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff303031, SUC_lexicon_tags_27 },
    { 0xfbd03032, SUC_lexicon_tags_138 },
    { 0x95b03033, SUC_lexicon_tags_26 },
    { 0xe3d03034, SUC_lexicon_tags_37 },
    { 0xfbb03035, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690303b, SUC_lexicon_tags_46 },
    { 0xa510303b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0303f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbbf03041, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb6703043, SUC_lexicon_tags_1 },
    { 0x91d03043, SUC_lexicon_tags_34 },
    { 0xee903044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8050304a, SUC_lexicon_tags_12 },
    { 0x7910304a, SUC_lexicon_tags_37 },
    { 0xd010304b, SUC_lexicon_tags_28 },
    { 0x28d0304a, SUC_lexicon_tags_19 },
    { 0x6690304e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49503055, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b03058, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b90305c, SUC_lexicon_tags_9 },
    { 0xf3d0305d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd03060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0103062, SUC_lexicon_tags_2 },
    { 0x15503062, SUC_lexicon_tags_14 },
    { 0x94503062, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d03067, SUC_lexicon_tags_23 },
    { 0x68103068, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b0306b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf503074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5903077, SUC_lexicon_tags_17 },
    { 0x66d03077, SUC_lexicon_tags_7 },
    { 0xeb503079, SUC_lexicon_tags_37 },
    { 0xf3303079, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f0307e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f03088, SUC_lexicon_tags_2 },
    { 0x8e103089, SUC_lexicon_tags_45 },
    { 0x7d0308a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8890308c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d03090, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8503093, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f03099, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4390309b, SUC_lexicon_tags_2 },
    { 0xca90309c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdb30309e, SUC_lexicon_tags_48 },
    { 0x9e30309e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d030a4, SUC_lexicon_tags_13 },
    { 0xa5f030a5, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27030aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5f030ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbab030ae, SUC_lexicon_tags_12 },
    { 0xfa7030ae, SUC_lexicon_tags_37 },
    { 0x8af030af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73030b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf7030b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03030b8, SUC_lexicon_tags_11 },
    { 0xe6b030b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61f030bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x369030be, SUC_lexicon_tags_17 },
    { 0x3e1030bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b030c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3030c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b030cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x127030ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd030d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5030d7, SUC_lexicon_tags_31 },
    { 0xfc7030d8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b030de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8b030e0, SUC_lexicon_tags_19 },
    { 0xa25030e1, SUC_lexicon_tags_7 },
    { 0xbcf030e2, SUC_lexicon_tags_9 },
    { 0xc5030e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861030e7, SUC_lexicon_tags_33 },
    { 0x4c7030e8, SUC_lexicon_tags_11 },
    { 0x139030e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3030ed, SUC_lexicon_tags_47 },
    { 0x61b030ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d030f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf29030f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9030fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b03101, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb03105, SUC_lexicon_tags_235 },
    { 0xb2303105, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f903109, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5330310d, SUC_lexicon_tags_37 },
    { 0xacf0310d, SUC_lexicon_tags_11 },
    { 0x2510310f, SUC_lexicon_tags_27 },
    { 0x9af0310d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7703112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8903114, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f03119, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b03122, SUC_lexicon_tags_45 },
    { 0x8e903123, SUC_lexicon_tags_38 },
    { 0x77703122, SUC_lexicon_tags_33 },
    { 0x51903122, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xadf03127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd450312a, SUC_lexicon_tags_21 },
    { 0x2b70312b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c10312d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d703132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5503137, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4750313e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf303149, SUC_lexicon_tags_25 },
    { 0x8450314a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab03151, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b03154, SUC_lexicon_tags_2 },
    { 0x4b03155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9303157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb0315e, SUC_lexicon_tags_27 },
    { 0x14f0315f, SUC_lexicon_tags_2 },
    { 0x55d03160, SUC_lexicon_tags_37 },
    { 0x9ef03161, SUC_lexicon_tags_7 },
    { 0xdb703162, SUC_lexicon_tags_19 },
    { 0xb4d03162, SUC_lexicon_tags_11 },
    { 0xcd103164, SUC_lexicon_tags_15 },
    { 0xdc903162, SUC_lexicon_tags_45 },
    { 0xf8503165, SUC_lexicon_tags_12 },
    { 0x50d03167, SUC_lexicon_tags_19 },
    { 0xc430315e, SUC_lexicon_tags_10 },
    { 0xc6d03169, SUC_lexicon_tags_2 },
    { 0x3410315f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde903170, SUC_lexicon_tags_0 },
    { 0xd03171, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f03174, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x903177, SUC_lexicon_tags_49 },
    { 0xbcb03178, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe0f0317a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4350317c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76903189, SUC_lexicon_tags_39 },
    { 0x6430318a, SUC_lexicon_tags_37 },
    { 0x2e50318b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df0318e, SUC_lexicon_tags_11 },
    { 0x3270318f, SUC_lexicon_tags_45 },
    { 0x7c703190, SUC_lexicon_tags_38 },
    { 0xe7b03191, SUC_lexicon_tags_17 },
    { 0x11f0318e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd1b03194, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x11703196, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb0319c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b031a0, SUC_lexicon_tags_11 },
    { 0xd09031a0, SUC_lexicon_tags_4 },
    { 0x6a9031a2, SUC_lexicon_tags_49 },
    { 0x94d031a3, SUC_lexicon_tags_9 },
    { 0xf05031a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f031a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b031ac, SUC_lexicon_tags_43 },
    { 0x5e7031ac, SUC_lexicon_tags_17 },
    { 0x999031ae, SUC_lexicon_tags_11 },
    { 0x765031ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b031b3, SUC_lexicon_tags_33 },
    { 0x715031b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1b031b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d031b9, SUC_lexicon_tags_39 },
    { 0xd4b031b9, SUC_lexicon_tags_17 },
    { 0x5e9031b9, SUC_lexicon_tags_8 },
    { 0x391031bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265031c2, SUC_lexicon_tags_10 },
    { 0x8bd031c3, SUC_lexicon_tags_7 },
    { 0x3f3031c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31031c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x783031c8, SUC_lexicon_tags_11 },
    { 0x847031c9, SUC_lexicon_tags_11 },
    { 0x1af031c8, SUC_lexicon_tags_1 },
    { 0xb7f031c9, SUC_lexicon_tags_6 },
    { 0xfc5031cb, SUC_lexicon_tags_17 },
    { 0xe43031cd, SUC_lexicon_tags_12 },
    { 0x531031ce, SUC_lexicon_tags_1 },
    { 0x609031ce, SUC_lexicon_tags_29 },
    { 0xdd9031cf, SUC_lexicon_tags_47 },
    { 0x2ed031d1, SUC_lexicon_tags_131 },
    { 0xe5d031d1, SUC_lexicon_tags_14 },
    { 0x1db031d0, SUC_lexicon_tags_46 },
    { 0x6cd031d3, SUC_lexicon_tags_37 },
    { 0xc73031ce, SUC_lexicon_tags_15 },
    { 0x60f031d3, SUC_lexicon_tags_12 },
    { 0x95031d6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b031dd, SUC_lexicon_tags_63 },
    { 0x32b031de, SUC_lexicon_tags_37 },
    { 0x9c7031df, SUC_lexicon_tags_1 },
    { 0x40d031e0, SUC_lexicon_tags_2 },
    { 0xf17031e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b031ec, SUC_lexicon_tags_39 },
    { 0x8b031ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x887031ef, SUC_lexicon_tags_45 },
    { 0xc45031f0, SUC_lexicon_tags_63 },
    { 0xa61031f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55031f4, SUC_lexicon_tags_33 },
    { 0xc23031f4, SUC_lexicon_tags_28 },
    { 0x487031f5, SUC_lexicon_tags_42 },
    { 0xf3f031f5, SUC_lexicon_tags_11 },
    { 0x7a1031f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd35031fa, SUC_lexicon_tags_47 },
    { 0x2d9031fb, SUC_lexicon_tags_12 },
    { 0xa8d031fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab031ff, SUC_lexicon_tags_37 },
    { 0x90d031ff, SUC_lexicon_tags_7 },
    { 0x22d03201, SUC_lexicon_tags_17 },
    { 0xb8f03200, SUC_lexicon_tags_66 },
    { 0x4ad03201, SUC_lexicon_tags_12 },
    { 0xae303201, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xde703206, SUC_lexicon_tags_37 },
    { 0xbb903207, SUC_lexicon_tags_29 },
    { 0x7e703206, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5b0320a, SUC_lexicon_tags_11 },
    { 0x9410320b, SUC_lexicon_tags_33 },
    { 0xffd0320c, SUC_lexicon_tags_15 },
    { 0xd150320d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e903210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79303212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58903215, SUC_lexicon_tags_2 },
    { 0xa7f03216, SUC_lexicon_tags_37 },
    { 0x71703217, SUC_lexicon_tags_17 },
    { 0xdff03216, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d0321d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb03220, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18303225, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b03228, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa930322b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9f0322d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1103231, SUC_lexicon_tags_27 },
    { 0xe2103232, SUC_lexicon_tags_12 },
    { 0x25b03233, SUC_lexicon_tags_19 },
    { 0x48103234, SUC_lexicon_tags_77 },
    { 0x4f503235, SUC_lexicon_tags_25 },
    { 0x60103236, SUC_lexicon_tags_12 },
    { 0xe9503237, SUC_lexicon_tags_7 },
    { 0x6a703232, SUC_lexicon_tags_11 },
    { 0xc503239, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe70323e, SUC_lexicon_tags_2 },
    { 0x30d0323f, SUC_lexicon_tags_66 },
    { 0xad903240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8503242, SUC_lexicon_tags_11 },
    { 0xa5703243, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6950324d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52303252, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b503257, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77b03259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b50325d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa503263, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b03266, SUC_lexicon_tags_2 },
    { 0x46f03267, SUC_lexicon_tags_14 },
    { 0x55b03268, SUC_lexicon_tags_8 },
    { 0x5a903268, SUC_lexicon_tags_98 },
    { 0xcd70326a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cd0326c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xec50326e, SUC_lexicon_tags_6 },
    { 0x9c70326e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28903273, SUC_lexicon_tags_38 },
    { 0x2f703273, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98503276, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4703279, SUC_lexicon_tags_23 },
    { 0x7bd0327a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebb0327c, SUC_lexicon_tags_12 },
    { 0xc4f0327c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5103281, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x96703283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a70328b, SUC_lexicon_tags_19 },
    { 0x8010328b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd810328e, SUC_lexicon_tags_7 },
    { 0xe610328f, SUC_lexicon_tags_17 },
    { 0x5750328e, SUC_lexicon_tags_23 },
    { 0xb0b03291, SUC_lexicon_tags_1 },
    { 0x33303291, SUC_lexicon_tags_65 },
    { 0xa1903293, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6303295, SUC_lexicon_tags_68 },
    { 0x3bd03296, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeef03298, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d0329c, SUC_lexicon_tags_38 },
    { 0x2f10329d, SUC_lexicon_tags_17 },
    { 0xfc10329d, SUC_lexicon_tags_17 },
    { 0xf970329c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa6b032a1, SUC_lexicon_tags_7 },
    { 0x9c7032a2, SUC_lexicon_tags_30 },
    { 0x281032a3, SUC_lexicon_tags_11 },
    { 0x799032a4, SUC_lexicon_tags_23 },
    { 0xe93032a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5032a8, SUC_lexicon_tags_2 },
    { 0xb23032a9, SUC_lexicon_tags_25 },
    { 0x609032a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7032ad, SUC_lexicon_tags_12 },
    { 0x987032ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9032b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcf032b3, SUC_lexicon_tags_1 },
    { 0x69d032b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925032b8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b032bb, SUC_lexicon_tags_47 },
    { 0xc9f032bc, SUC_lexicon_tags_6 },
    { 0xc29032bc, SUC_lexicon_tags_45 },
    { 0x617032be, SUC_lexicon_tags_23 },
    { 0x1e3032bf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x291032c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67032c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903032ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77032d4, SUC_lexicon_tags_6 },
    { 0xc01032d5, SUC_lexicon_tags_29 },
    { 0xd1d032d6, SUC_lexicon_tags_48 },
    { 0xc25032d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb032da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fd032dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x771032de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f032e2, SUC_lexicon_tags_26 },
    { 0xb5f032e3, SUC_lexicon_tags_0 },
    { 0x1c1032e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b032e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x699032ea, SUC_lexicon_tags_17 },
    { 0xfc1032ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe1b032ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a9032ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b032fc, SUC_lexicon_tags_87 },
    { 0xe2d032fd, SUC_lexicon_tags_28 },
    { 0xd1032fe, SUC_lexicon_tags_1 },
    { 0x905032fe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4110330a, SUC_lexicon_tags_1 },
    { 0x73b0330a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9770330d, SUC_lexicon_tags_70 },
    { 0xf490330e, SUC_lexicon_tags_33 },
    { 0xc110330d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f03313, SUC_lexicon_tags_37 },
    { 0xefd03313, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf903316, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fd03318, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f0331b, SUC_lexicon_tags_37 },
    { 0xc990331c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3710331f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd03324, SUC_lexicon_tags_2 },
    { 0xde903325, SUC_lexicon_tags_77 },
    { 0xd903326, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4490332a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b30332e, SUC_lexicon_tags_11 },
    { 0x9dd0332e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6d103331, SUC_lexicon_tags_6 },
    { 0x6f503332, SUC_lexicon_tags_37 },
    { 0xa2d03332, SUC_lexicon_tags_7 },
    { 0xaab03334, SUC_lexicon_tags_65 },
    { 0xb1503334, SUC_lexicon_tags_25 },
    { 0x75b03336, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x95303339, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90333d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7903341, SUC_lexicon_tags_48 },
    { 0x69103342, SUC_lexicon_tags_17 },
    { 0x42703343, SUC_lexicon_tags_25 },
    { 0x3c303343, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdb303346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23703348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d0334c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf50334f, SUC_lexicon_tags_11 },
    { 0xaf03350, SUC_lexicon_tags_19 },
    { 0x76903350, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6703356, SUC_lexicon_tags_14 },
    { 0x36f03356, SUC_lexicon_tags_47 },
    { 0x19f03358, SUC_lexicon_tags_7 },
    { 0x1db03358, SUC_lexicon_tags_45 },
    { 0x43d03359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1630335c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf03361, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f503365, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2703368, SUC_lexicon_tags_49 },
    { 0xb6d03368, SUC_lexicon_tags_17 },
    { 0xc850336a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb0336f, SUC_lexicon_tags_46 },
    { 0x3170336f, SUC_lexicon_tags_30 },
    { 0x92b03370, SUC_lexicon_tags_47 },
    { 0xeef03372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b03375, SUC_lexicon_tags_68 },
    { 0xc8b03376, SUC_lexicon_tags_43 },
    { 0xa4903377, SUC_lexicon_tags_37 },
    { 0x4bb03378, SUC_lexicon_tags_2 },
    { 0xa1703378, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3290337c, SUC_lexicon_tags_26 },
    { 0x9ed0337d, SUC_lexicon_tags_2 },
    { 0x5210337c, SUC_lexicon_tags_1 },
    { 0x5af0337e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeb03381, SUC_lexicon_tags_199 },
    { 0x69f03382, SUC_lexicon_tags_4 },
    { 0xacf03383, SUC_lexicon_tags_49 },
    { 0xb3103384, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf703388, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10338b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40103390, SUC_lexicon_tags_14 },
    { 0xefd03391, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3fb03393, SUC_lexicon_tags_49 },
    { 0x78f03394, SUC_lexicon_tags_23 },
    { 0x81703395, SUC_lexicon_tags_15 },
    { 0x25f03393, SUC_lexicon_tags_15 },
    { 0x5f03395, SUC_lexicon_tags_37 },
    { 0x52503395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe30339b, SUC_lexicon_tags_61 },
    { 0x7150339c, SUC_lexicon_tags_28 },
    { 0xccb0339c, SUC_lexicon_tags_37 },
    { 0x7f10339e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x465033a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x805033a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d033a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x79033aa, SUC_lexicon_tags_47 },
    { 0x2a3033ab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5033af, SUC_lexicon_tags_8 },
    { 0x8b3033af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5033b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f033bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b033bf, SUC_lexicon_tags_15 },
    { 0x479033bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f033c3, SUC_lexicon_tags_60 },
    { 0x865033c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x861033c6, SUC_lexicon_tags_25 },
    { 0x503033c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6af033c9, SUC_lexicon_tags_2 },
    { 0x517033ca, SUC_lexicon_tags_24 },
    { 0xeed033cb, SUC_lexicon_tags_12 },
    { 0xcad033cb, SUC_lexicon_tags_12 },
    { 0xa07033cd, SUC_lexicon_tags_56 },
    { 0x22f033cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179033d3, SUC_lexicon_tags_31 },
    { 0xf01033d3, SUC_lexicon_tags_4 },
    { 0x8f1033d4, SUC_lexicon_tags_1 },
    { 0xe2d033d5, SUC_lexicon_tags_133 },
    { 0x95d033d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf033dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d033e5, SUC_lexicon_tags_17 },
    { 0xaa9033e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab033eb, SUC_lexicon_tags_263 },
    { 0xd7033eb, SUC_lexicon_tags_10 },
    { 0x723033ed, SUC_lexicon_tags_31 },
    { 0xaa3033ee, SUC_lexicon_tags_34 },
    { 0x8c9033ef, SUC_lexicon_tags_13 },
    { 0xc1f033ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2b033f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef033f6, SUC_lexicon_tags_7 },
    { 0x963033f6, SUC_lexicon_tags_43 },
    { 0x3c3033f8, SUC_lexicon_tags_17 },
    { 0x1eb033f9, SUC_lexicon_tags_60 },
    { 0x1b1033fa, SUC_lexicon_tags_13 },
    { 0xd8b033fb, SUC_lexicon_tags_17 },
    { 0xa71033fc, SUC_lexicon_tags_3 },
    { 0x181033fb, SUC_lexicon_tags_19 },
    { 0x7ab033fb, SUC_lexicon_tags_46 },
    { 0x8a7033ff, SUC_lexicon_tags_12 },
    { 0xfe5033f7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb03406, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f0340b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbb0340d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b903415, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3703418, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe570341f, SUC_lexicon_tags_49 },
    { 0xaf03420, SUC_lexicon_tags_6 },
    { 0xed03421, SUC_lexicon_tags_28 },
    { 0xd3b03421, SUC_lexicon_tags_26 },
    { 0xbf503420, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x92f03425, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb650342b, SUC_lexicon_tags_47 },
    { 0xb750342c, SUC_lexicon_tags_17 },
    { 0xbf90342b, SUC_lexicon_tags_25 },
    { 0xaab0342e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5cf03430, SUC_lexicon_tags_7 },
    { 0x3a503431, SUC_lexicon_tags_47 },
    { 0xd1103432, SUC_lexicon_tags_86 },
    { 0x22103433, SUC_lexicon_tags_42 },
    { 0xf5703434, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2703439, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b70343c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd303442, SUC_lexicon_tags_7 },
    { 0xadf03443, SUC_lexicon_tags_46 },
    { 0x46703444, SUC_lexicon_tags_17 },
    { 0x51303444, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x30903448, SUC_lexicon_tags_11 },
    { 0xc0b03449, SUC_lexicon_tags_65 },
    { 0x2370344a, SUC_lexicon_tags_37 },
    { 0x7d10344b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x830344d, SUC_lexicon_tags_12 },
    { 0xddb0344e, SUC_lexicon_tags_45 },
    { 0x4810344f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27103453, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f503456, SUC_lexicon_tags_60 },
    { 0x78703457, SUC_lexicon_tags_7 },
    { 0x7d303457, SUC_lexicon_tags_7 },
    { 0x6bd03459, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae10345e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb303466, SUC_lexicon_tags_33 },
    { 0x62b03467, SUC_lexicon_tags_15 },
    { 0x58503467, SUC_lexicon_tags_12 },
    { 0xb5903466, SUC_lexicon_tags_1 },
    { 0x6530346a, SUC_lexicon_tags_11 },
    { 0xd2f0346b, SUC_lexicon_tags_37 },
    { 0x8df03466, SUC_lexicon_tags_46 },
    { 0x47b0346c, SUC_lexicon_tags_30 },
    { 0xb8d0346e, SUC_lexicon_tags_25 },
    { 0xb370346e, SUC_lexicon_tags_48 },
    { 0xf50346d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad03478, SUC_lexicon_tags_29 },
    { 0xe7303479, SUC_lexicon_tags_37 },
    { 0xa110347a, SUC_lexicon_tags_47 },
    { 0xf010347a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xda50347e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab703480, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7903484, SUC_lexicon_tags_11 },
    { 0x1d103485, SUC_lexicon_tags_15 },
    { 0x8703486, SUC_lexicon_tags_7 },
    { 0x46303487, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb910348e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ff03490, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33103493, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d034a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x487034a4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x233034a7, SUC_lexicon_tags_11 },
    { 0x257034a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x799034ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965034b0, SUC_lexicon_tags_10 },
    { 0xab7034b1, SUC_lexicon_tags_12 },
    { 0xa3f034b1, SUC_lexicon_tags_1 },
    { 0x887034b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb034b8, SUC_lexicon_tags_37 },
    { 0x6b5034b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73034bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d034be, SUC_lexicon_tags_3 },
    { 0x8fd034be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x191034c1, SUC_lexicon_tags_8 },
    { 0x9b5034c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x427034c4, SUC_lexicon_tags_15 },
    { 0x31d034c5, SUC_lexicon_tags_41 },
    { 0xae7034c5, SUC_lexicon_tags_47 },
    { 0xa39034c7, SUC_lexicon_tags_11 },
    { 0xa7d034c7, SUC_lexicon_tags_28 },
    { 0x457034c9, SUC_lexicon_tags_37 },
    { 0x2ff034ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3034cd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb5034cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x41d034d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7034d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5034da, SUC_lexicon_tags_37 },
    { 0xf9034db, SUC_lexicon_tags_6 },
    { 0xdc9034db, SUC_lexicon_tags_12 },
    { 0x247034dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3034e1, SUC_lexicon_tags_2 },
    { 0x181034e2, SUC_lexicon_tags_63 },
    { 0x7b5034e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03034e6, SUC_lexicon_tags_14 },
    { 0x22b034e7, SUC_lexicon_tags_25 },
    { 0xa0f034e6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6db034ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9034ee, SUC_lexicon_tags_37 },
    { 0xdf1034ef, SUC_lexicon_tags_25 },
    { 0xa53034f0, SUC_lexicon_tags_37 },
    { 0xd2d034f0, SUC_lexicon_tags_17 },
    { 0xdfb034f2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd034f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5034f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4d034fa, SUC_lexicon_tags_38 },
    { 0x48d034fa, SUC_lexicon_tags_316 },
    { 0, NULL },
    { 0, NULL },
    { 0x627034fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea303502, SUC_lexicon_tags_87 },
    { 0xec103503, SUC_lexicon_tags_7 },
    { 0xc6703504, SUC_lexicon_tags_27 },
    { 0x5a903505, SUC_lexicon_tags_60 },
    { 0x2fb03504, SUC_lexicon_tags_77 },
    { 0xe6103503, SUC_lexicon_tags_3 },
    { 0xa8503507, SUC_lexicon_tags_17 },
    { 0xa9d03509, SUC_lexicon_tags_45 },
    { 0x5f903507, SUC_lexicon_tags_10 },
    { 0x7f50350b, SUC_lexicon_tags_37 },
    { 0x9790350c, SUC_lexicon_tags_37 },
    { 0xa6b03508, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad90350f, SUC_lexicon_tags_27 },
    { 0xe4b03510, SUC_lexicon_tags_0 },
    { 0x2a70350f, SUC_lexicon_tags_63 },
    { 0xa2303512, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1cf03514, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc103518, SUC_lexicon_tags_56 },
    { 0x9e103519, SUC_lexicon_tags_33 },
    { 0x80f03519, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f0351d, SUC_lexicon_tags_70 },
    { 0xa950351d, SUC_lexicon_tags_39 },
    { 0x9e90351f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c903526, SUC_lexicon_tags_1 },
    { 0x62103526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6503529, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16903535, SUC_lexicon_tags_46 },
    { 0x26d03535, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x86703539, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15b0353b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6303544, SUC_lexicon_tags_3 },
    { 0xf3103545, SUC_lexicon_tags_25 },
    { 0x37d03546, SUC_lexicon_tags_7 },
    { 0x73103545, SUC_lexicon_tags_17 },
    { 0x50303548, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2090354a, SUC_lexicon_tags_123 },
    { 0xebf0354b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97903555, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x45703558, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d0355c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d0355f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4503561, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa3d03563, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68103566, SUC_lexicon_tags_19 },
    { 0xd3503567, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9b03569, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98103571, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3103573, SUC_lexicon_tags_66 },
    { 0x41b03573, SUC_lexicon_tags_22 },
    { 0xfb03575, SUC_lexicon_tags_17 },
    { 0xf7303575, SUC_lexicon_tags_48 },
    { 0xa5703577, SUC_lexicon_tags_43 },
    { 0x75f03577, SUC_lexicon_tags_29 },
    { 0xc1703579, SUC_lexicon_tags_14 },
    { 0x3610357a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xed10357c, SUC_lexicon_tags_84 },
    { 0x7250357d, SUC_lexicon_tags_45 },
    { 0xef0357c, SUC_lexicon_tags_7 },
    { 0x56b0357f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50903586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d103588, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd0358e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff03593, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb903597, SUC_lexicon_tags_1 },
    { 0xd903598, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b0359b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda50359d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61035a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f035a3, SUC_lexicon_tags_1 },
    { 0x333035a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9035ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265035b1, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395035b8, SUC_lexicon_tags_0 },
    { 0x537035b9, SUC_lexicon_tags_37 },
    { 0x67035b8, SUC_lexicon_tags_53 },
    { 0xa7035bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83d035bd, SUC_lexicon_tags_17 },
    { 0x14d035bd, SUC_lexicon_tags_37 },
    { 0x5c1035bd, SUC_lexicon_tags_0 },
    { 0xa35035c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e3035c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1035cb, SUC_lexicon_tags_2 },
    { 0xa8d035cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335035d2, SUC_lexicon_tags_25 },
    { 0xf65035d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x893035d5, SUC_lexicon_tags_7 },
    { 0x7cb035d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fb035d8, SUC_lexicon_tags_43 },
    { 0x55035d9, SUC_lexicon_tags_33 },
    { 0x5ed035d9, SUC_lexicon_tags_57 },
    { 0xfdb035db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7035df, SUC_lexicon_tags_1 },
    { 0xc83035df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43035ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f035f0, SUC_lexicon_tags_17 },
    { 0x917035f1, SUC_lexicon_tags_38 },
    { 0x9df035f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7035f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d7035f7, SUC_lexicon_tags_32 },
    { 0x393035f8, SUC_lexicon_tags_32 },
    { 0xb39035f9, SUC_lexicon_tags_23 },
    { 0x59035fa, SUC_lexicon_tags_87 },
    { 0x95b035f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf53035fd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37503601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4103603, SUC_lexicon_tags_18 },
    { 0x6a303604, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9303606, SUC_lexicon_tags_8 },
    { 0x33503607, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe030360b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb703610, SUC_lexicon_tags_46 },
    { 0x54903611, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x62703614, SUC_lexicon_tags_96 },
    { 0x73103615, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f30361b, SUC_lexicon_tags_84 },
    { 0xd0361b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc10361f, SUC_lexicon_tags_66 },
    { 0x29503620, SUC_lexicon_tags_14 },
    { 0xb1503620, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e90362d, SUC_lexicon_tags_2 },
    { 0x61b0362d, SUC_lexicon_tags_43 },
    { 0x4150362e, SUC_lexicon_tags_63 },
    { 0x9910362f, SUC_lexicon_tags_37 },
    { 0x85d03630, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d03635, SUC_lexicon_tags_10 },
    { 0x96303636, SUC_lexicon_tags_27 },
    { 0x55703636, SUC_lexicon_tags_17 },
    { 0xd9b03637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c70363d, SUC_lexicon_tags_31 },
    { 0xf90363e, SUC_lexicon_tags_17 },
    { 0xa5f0363f, SUC_lexicon_tags_37 },
    { 0x7a103640, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd903644, SUC_lexicon_tags_28 },
    { 0x53903644, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f503648, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a30364b, SUC_lexicon_tags_0 },
    { 0x3450364c, SUC_lexicon_tags_1 },
    { 0xf070364d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf370364f, SUC_lexicon_tags_17 },
    { 0x47303650, SUC_lexicon_tags_25 },
    { 0x3e103651, SUC_lexicon_tags_12 },
    { 0x2d703652, SUC_lexicon_tags_118 },
    { 0xf3903653, SUC_lexicon_tags_60 },
    { 0x5ab03653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9670365b, SUC_lexicon_tags_47 },
    { 0x5290365c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82903660, SUC_lexicon_tags_11 },
    { 0xc9103660, SUC_lexicon_tags_2 },
    { 0x6ab03660, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7903665, SUC_lexicon_tags_7 },
    { 0x9df03665, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xa8f03668, SUC_lexicon_tags_12 },
    { 0x17f03668, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f0366e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e303671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee103674, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xea903676, SUC_lexicon_tags_50 },
    { 0xeb703677, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f0367c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b0367f, SUC_lexicon_tags_3 },
    { 0xa1703680, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75703684, SUC_lexicon_tags_40 },
    { 0x7c703685, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd03688, SUC_lexicon_tags_0 },
    { 0x53d03688, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf0368e, SUC_lexicon_tags_26 },
    { 0x9f10368f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25903691, SUC_lexicon_tags_19 },
    { 0xd9b03692, SUC_lexicon_tags_12 },
    { 0x18d03691, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66503695, SUC_lexicon_tags_4 },
    { 0x5f903696, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd0369d, SUC_lexicon_tags_63 },
    { 0xf90369d, SUC_lexicon_tags_26 },
    { 0xfd0369f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x8d1036a1, SUC_lexicon_tags_3 },
    { 0xbc1036a2, SUC_lexicon_tags_2 },
    { 0xc29036a3, SUC_lexicon_tags_11 },
    { 0x2bb036a4, SUC_lexicon_tags_11 },
    { 0xf83036a1, SUC_lexicon_tags_1 },
    { 0xed9036a4, SUC_lexicon_tags_9 },
    { 0xad7036a7, SUC_lexicon_tags_37 },
    { 0xf1036a4, SUC_lexicon_tags_11 },
    { 0x9ad036a8, SUC_lexicon_tags_29 },
    { 0x371036a5, SUC_lexicon_tags_37 },
    { 0xc9d036ab, SUC_lexicon_tags_15 },
    { 0xa6d036ac, SUC_lexicon_tags_17 },
    { 0xef5036a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee5036af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1cd036b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541036b5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x435036b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x721036b9, SUC_lexicon_tags_12 },
    { 0x595036b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29036bf, SUC_lexicon_tags_27 },
    { 0xd3d036c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d036c3, SUC_lexicon_tags_11 },
    { 0x14d036c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b036c8, SUC_lexicon_tags_27 },
    { 0x58d036c9, SUC_lexicon_tags_40 },
    { 0xe9d036ca, SUC_lexicon_tags_2 },
    { 0xc37036ca, SUC_lexicon_tags_2 },
    { 0x905036cb, SUC_lexicon_tags_68 },
    { 0x4a1036cd, SUC_lexicon_tags_26 },
    { 0x437036ce, SUC_lexicon_tags_15 },
    { 0xadf036cf, SUC_lexicon_tags_27 },
    { 0xb95036cf, SUC_lexicon_tags_4 },
    { 0x74d036d0, SUC_lexicon_tags_23 },
    { 0x687036d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef036d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91036de, SUC_lexicon_tags_49 },
    { 0x819036df, SUC_lexicon_tags_12 },
    { 0xa33036de, SUC_lexicon_tags_1 },
    { 0x81036e1, SUC_lexicon_tags_7 },
    { 0x5a9036e2, SUC_lexicon_tags_34 },
    { 0x4d036e3, SUC_lexicon_tags_47 },
    { 0xcf1036e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdc7036e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7036ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7036f1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5036f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcbd036f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad036fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b03705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe70370a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f30370e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x22d03710, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45f03712, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9d903714, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7703719, SUC_lexicon_tags_29 },
    { 0x1b10371a, SUC_lexicon_tags_37 },
    { 0xf5b0371b, SUC_lexicon_tags_63 },
    { 0xf2d0371b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd390371f, SUC_lexicon_tags_12 },
    { 0xbf303720, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d03726, SUC_lexicon_tags_31 },
    { 0x2ab03727, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f0372b, SUC_lexicon_tags_15 },
    { 0x7fd0372b, SUC_lexicon_tags_12 },
    { 0x83b0372b, SUC_lexicon_tags_15 },
    { 0x7810372d, SUC_lexicon_tags_69 },
    { 0x370372f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11703735, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810373c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7770373f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4303743, SUC_lexicon_tags_1 },
    { 0xd4103744, SUC_lexicon_tags_7 },
    { 0x3ef03745, SUC_lexicon_tags_12 },
    { 0xc3503745, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f03749, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4030374b, SUC_lexicon_tags_45 },
    { 0x4d70374c, SUC_lexicon_tags_11 },
    { 0xfcb0374b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x9030374f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d03757, SUC_lexicon_tags_21 },
    { 0x51103757, SUC_lexicon_tags_43 },
    { 0x51303759, SUC_lexicon_tags_16 },
    { 0x10b03759, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d03766, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9f903768, SUC_lexicon_tags_15 },
    { 0x84503768, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8610376d, SUC_lexicon_tags_6 },
    { 0xa6f0376d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5903771, SUC_lexicon_tags_15 },
    { 0x9f303772, SUC_lexicon_tags_25 },
    { 0xd2303773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6103775, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7503779, SUC_lexicon_tags_11 },
    { 0x15d0377a, SUC_lexicon_tags_37 },
    { 0x6cf0377b, SUC_lexicon_tags_25 },
    { 0xdcf0377a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc303780, SUC_lexicon_tags_85 },
    { 0x9f503781, SUC_lexicon_tags_2 },
    { 0xcad03780, SUC_lexicon_tags_37 },
    { 0x6a903783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb03786, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7303793, SUC_lexicon_tags_60 },
    { 0x5a303794, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9303796, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd03798, SUC_lexicon_tags_7 },
    { 0x4c303798, SUC_lexicon_tags_11 },
    { 0x8bf03799, SUC_lexicon_tags_47 },
    { 0x4ef0379b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9130379f, SUC_lexicon_tags_47 },
    { 0xf85037a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a3037a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7037a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71037b0, SUC_lexicon_tags_39 },
    { 0xbd7037b1, SUC_lexicon_tags_2 },
    { 0xc7037b1, SUC_lexicon_tags_46 },
    { 0x6db037b3, SUC_lexicon_tags_69 },
    { 0xe3f037b4, SUC_lexicon_tags_31 },
    { 0x5e5037b5, SUC_lexicon_tags_11 },
    { 0xdf1037b5, SUC_lexicon_tags_25 },
    { 0x63f037b7, SUC_lexicon_tags_65 },
    { 0x385037b7, SUC_lexicon_tags_15 },
    { 0x275037b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f037c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0b037c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d037c5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb037cd, SUC_lexicon_tags_6 },
    { 0xd5f037ce, SUC_lexicon_tags_47 },
    { 0xc53037ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b037d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33037dc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x695037de, SUC_lexicon_tags_1 },
    { 0xbd1037de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905037e3, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0x443037e5, SUC_lexicon_tags_23 },
    { 0x5ed037e6, SUC_lexicon_tags_37 },
    { 0xa07037e7, SUC_lexicon_tags_6 },
    { 0x33b037e8, SUC_lexicon_tags_12 },
    { 0xdbb037e9, SUC_lexicon_tags_17 },
    { 0xd75037e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb037ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955037f1, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d037f4, SUC_lexicon_tags_894 },
    { 0xc57037f5, SUC_lexicon_tags_37 },
    { 0xc2b037f5, SUC_lexicon_tags_63 },
    { 0x9b1037f5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbe9037f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf037ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2903802, SUC_lexicon_tags_7 },
    { 0x26b03803, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f03805, SUC_lexicon_tags_38 },
    { 0x103806, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf0380a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7570380c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b0380f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x14503812, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbd903814, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48703818, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9190381b, SUC_lexicon_tags_15 },
    { 0xe1d0381b, SUC_lexicon_tags_11 },
    { 0x67f0381d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48703821, SUC_lexicon_tags_46 },
    { 0x49303822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49703824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb303828, SUC_lexicon_tags_29 },
    { 0x25f03828, SUC_lexicon_tags_2 },
    { 0x11303828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28f0382c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0303833, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b03836, SUC_lexicon_tags_15 },
    { 0xe1f03837, SUC_lexicon_tags_11 },
    { 0x5cb03838, SUC_lexicon_tags_114 },
    { 0xd6b03839, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8db0383b, SUC_lexicon_tags_2 },
    { 0x7930383b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b0383f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d03843, SUC_lexicon_tags_15 },
    { 0x64503843, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0103847, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a30384a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8103852, SUC_lexicon_tags_117 },
    { 0x44303853, SUC_lexicon_tags_27 },
    { 0x69b03852, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd7303856, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe470385a, SUC_lexicon_tags_21 },
    { 0xb230385a, SUC_lexicon_tags_26 },
    { 0xa270385c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3503861, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd03865, SUC_lexicon_tags_7 },
    { 0xb903866, SUC_lexicon_tags_45 },
    { 0x7c903867, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc1503869, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d0386e, SUC_lexicon_tags_17 },
    { 0x6cd0386f, SUC_lexicon_tags_33 },
    { 0x54f0386f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d03875, SUC_lexicon_tags_32 },
    { 0x67b03875, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0387a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc103884, SUC_lexicon_tags_27 },
    { 0x78b03884, SUC_lexicon_tags_25 },
    { 0x6503885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b0388a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7330388c, SUC_lexicon_tags_15 },
    { 0xe5f0388d, SUC_lexicon_tags_30 },
    { 0xfb30388e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab03892, SUC_lexicon_tags_27 },
    { 0x30f03892, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7503897, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe6d03899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1cf0389b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaaf0389d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f038a0, SUC_lexicon_tags_33 },
    { 0x40d038a1, SUC_lexicon_tags_43 },
    { 0x163038a2, SUC_lexicon_tags_11 },
    { 0xb15038a1, SUC_lexicon_tags_8 },
    { 0xf73038a2, SUC_lexicon_tags_25 },
    { 0x531038a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f038ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d038b1, SUC_lexicon_tags_49 },
    { 0xeaf038b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaa7038b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3038ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcd5038bc, SUC_lexicon_tags_17 },
    { 0xcd1038bd, SUC_lexicon_tags_9 },
    { 0x4f5038be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743038c9, SUC_lexicon_tags_27 },
    { 0x7ff038ca, SUC_lexicon_tags_1 },
    { 0x43b038cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d038cf, SUC_lexicon_tags_6 },
    { 0x20f038cf, SUC_lexicon_tags_26 },
    { 0xf47038d1, SUC_lexicon_tags_25 },
    { 0x343038d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169038d7, SUC_lexicon_tags_30 },
    { 0xd9f038d8, SUC_lexicon_tags_17 },
    { 0xf8b038d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f038dc, SUC_lexicon_tags_11 },
    { 0x3f3038dc, SUC_lexicon_tags_30 },
    { 0x9ef038de, SUC_lexicon_tags_37 },
    { 0xb15038de, SUC_lexicon_tags_86 },
    { 0x9f1038df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5df038e2, SUC_lexicon_tags_37 },
    { 0x169038e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x335038e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xff5038e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f038eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59038ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe29038f0, SUC_lexicon_tags_49 },
    { 0xdcb038f1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7038f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71038fd, SUC_lexicon_tags_15 },
    { 0xc03038fe, SUC_lexicon_tags_0 },
    { 0x8bb038ff, SUC_lexicon_tags_47 },
    { 0x62703900, SUC_lexicon_tags_37 },
    { 0x4bb03901, SUC_lexicon_tags_37 },
    { 0xb6503902, SUC_lexicon_tags_25 },
    { 0xa903901, SUC_lexicon_tags_15 },
    { 0x9503904, SUC_lexicon_tags_118 },
    { 0xf3703905, SUC_lexicon_tags_25 },
    { 0xbf038ff, SUC_lexicon_tags_30 },
    { 0x15b038fd, SUC_lexicon_tags_12 },
    { 0x63903904, SUC_lexicon_tags_29 },
    { 0xde903906, SUC_lexicon_tags_57 },
    { 0xc1038fe, SUC_lexicon_tags_0 },
    { 0x525038fe, SUC_lexicon_tags_13 },
    { 0xfcb0390c, SUC_lexicon_tags_37 },
    { 0xf210390c, SUC_lexicon_tags_7 },
    { 0x3870390e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9b03910, SUC_lexicon_tags_11 },
    { 0x59703911, SUC_lexicon_tags_46 },
    { 0x4ff03912, SUC_lexicon_tags_27 },
    { 0xb8d03912, SUC_lexicon_tags_16 },
    { 0x3d503913, SUC_lexicon_tags_7 },
    { 0xde903911, SUC_lexicon_tags_84 },
    { 0x1a703916, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5790391a, SUC_lexicon_tags_17 },
    { 0x5850391b, SUC_lexicon_tags_47 },
    { 0x3ab0391c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f03922, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49903924, SUC_lexicon_tags_7 },
    { 0xf8d03924, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf0392a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57f0392c, SUC_lexicon_tags_8 },
    { 0x28d0392d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f03930, SUC_lexicon_tags_17 },
    { 0xa8903931, SUC_lexicon_tags_14 },
    { 0xdb503932, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x95303935, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76d03937, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4c103939, SUC_lexicon_tags_19 },
    { 0x45b0393a, SUC_lexicon_tags_14 },
    { 0x8bb0393a, SUC_lexicon_tags_17 },
    { 0x6130393c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a303942, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab03947, SUC_lexicon_tags_343 },
    { 0xb1f03948, SUC_lexicon_tags_7 },
    { 0x4df03949, SUC_lexicon_tags_37 },
    { 0x6250394a, SUC_lexicon_tags_6 },
    { 0x8510394a, SUC_lexicon_tags_12 },
    { 0x8390394b, SUC_lexicon_tags_23 },
    { 0x7030394d, SUC_lexicon_tags_68 },
    { 0xd9b0394d, SUC_lexicon_tags_25 },
    { 0x87d0394f, SUC_lexicon_tags_14 },
    { 0xa1d0394d, SUC_lexicon_tags_12 },
    { 0x96903951, SUC_lexicon_tags_14 },
    { 0x9fb03952, SUC_lexicon_tags_37 },
    { 0xd5f03953, SUC_lexicon_tags_17 },
    { 0x80503953, SUC_lexicon_tags_68 },
    { 0xa310394d, SUC_lexicon_tags_14 },
    { 0x78b03956, SUC_lexicon_tags_37 },
    { 0xf4b03957, SUC_lexicon_tags_66 },
    { 0xf1f03958, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd70395c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a303964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f103968, SUC_lexicon_tags_17 },
    { 0xa9103969, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb0396c, SUC_lexicon_tags_1 },
    { 0x2730396d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1230396f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d703973, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f03978, SUC_lexicon_tags_9 },
    { 0xc3703979, SUC_lexicon_tags_47 },
    { 0x97503978, SUC_lexicon_tags_35 },
    { 0x3dd03978, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a503980, SUC_lexicon_tags_25 },
    { 0x85f03980, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb03986, SUC_lexicon_tags_17 },
    { 0x6c903987, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d0398b, SUC_lexicon_tags_25 },
    { 0x7490398c, SUC_lexicon_tags_31 },
    { 0x1af0398d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d03990, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1703996, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6750399c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d039a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73039a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357039ac, SUC_lexicon_tags_11 },
    { 0xaeb039ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f039b1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f039b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f1039b6, SUC_lexicon_tags_26 },
    { 0xbff039b6, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xaf039b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e5039bb, SUC_lexicon_tags_43 },
    { 0xfdf039bc, SUC_lexicon_tags_27 },
    { 0x111039bd, SUC_lexicon_tags_15 },
    { 0x295039bc, SUC_lexicon_tags_9 },
    { 0xc23039bf, SUC_lexicon_tags_12 },
    { 0xbe1039c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345039c5, SUC_lexicon_tags_7 },
    { 0xa4b039c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f039c8, SUC_lexicon_tags_27 },
    { 0x207039c9, SUC_lexicon_tags_63 },
    { 0x58d039ca, SUC_lexicon_tags_0 },
    { 0x213039cb, SUC_lexicon_tags_25 },
    { 0x53d039cb, SUC_lexicon_tags_25 },
    { 0x739039cb, SUC_lexicon_tags_17 },
    { 0xdd1039cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb039d1, SUC_lexicon_tags_48 },
    { 0x807039d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af039dd, SUC_lexicon_tags_60 },
    { 0xcd9039de, SUC_lexicon_tags_17 },
    { 0x529039df, SUC_lexicon_tags_25 },
    { 0x4d1039e0, SUC_lexicon_tags_1 },
    { 0x6d9039dd, SUC_lexicon_tags_7 },
    { 0x45039de, SUC_lexicon_tags_77 },
    { 0x529039df, SUC_lexicon_tags_14 },
    { 0xad5039e3, SUC_lexicon_tags_38 },
    { 0x155039e5, SUC_lexicon_tags_11 },
    { 0x2db039e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5039ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727039f4, SUC_lexicon_tags_2 },
    { 0x3bb039f4, SUC_lexicon_tags_65 },
    { 0x42b039f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79d039f8, SUC_lexicon_tags_23 },
    { 0xe9039f9, SUC_lexicon_tags_6 },
    { 0x19d039f8, SUC_lexicon_tags_25 },
    { 0xd0f039fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6f039fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a503a03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37903a06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x53503a09, SUC_lexicon_tags_77 },
    { 0xcd903a0a, SUC_lexicon_tags_60 },
    { 0x1b703a09, SUC_lexicon_tags_15 },
    { 0xb0903a0c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4703a10, SUC_lexicon_tags_87 },
    { 0xf1b03a11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed03a15, SUC_lexicon_tags_43 },
    { 0xe1d03a15, SUC_lexicon_tags_9 },
    { 0x17d03a16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d303a1d, SUC_lexicon_tags_17 },
    { 0x1ed03a1d, SUC_lexicon_tags_19 },
    { 0xed303a1e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x27703a21, SUC_lexicon_tags_66 },
    { 0xa9703a22, SUC_lexicon_tags_11 },
    { 0x83303a22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1303a2b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7703a2f, SUC_lexicon_tags_21 },
    { 0xc5703a30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57103a33, SUC_lexicon_tags_0 },
    { 0x7c703a34, SUC_lexicon_tags_896 },
    { 0x84b03a35, SUC_lexicon_tags_9 },
    { 0x47703a35, SUC_lexicon_tags_38 },
    { 0xd3903a37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b703a3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab103a40, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c103a44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae703a48, SUC_lexicon_tags_37 },
    { 0xce703a48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x83503a4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb03a51, SUC_lexicon_tags_201 },
    { 0xc2303a51, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d703a58, SUC_lexicon_tags_15 },
    { 0xe0303a58, SUC_lexicon_tags_37 },
    { 0x42503a5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d903a64, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c703a6a, SUC_lexicon_tags_2 },
    { 0x2cb03a6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b903a6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3103a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71503a76, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81903a7a, SUC_lexicon_tags_16 },
    { 0x9f503a7a, SUC_lexicon_tags_3 },
    { 0x23903a7b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5303a85, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d03a88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a8f, SUC_lexicon_tags_26 },
    { 0x78303a8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4103a92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfd03a94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf503a97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x73703a99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b03a9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf703aa2, SUC_lexicon_tags_2 },
    { 0x83503aa2, SUC_lexicon_tags_2 },
    { 0x2eb03aa4, SUC_lexicon_tags_14 },
    { 0x9e903aa5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd03aa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37703aac, SUC_lexicon_tags_25 },
    { 0xf1b03aad, SUC_lexicon_tags_66 },
    { 0x64703aae, SUC_lexicon_tags_47 },
    { 0x8dd03aad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6703ab2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3b303ab4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20903abc, SUC_lexicon_tags_114 },
    { 0xf6f03abd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37503ac2, SUC_lexicon_tags_37 },
    { 0xc2103ac3, SUC_lexicon_tags_25 },
    { 0xe5f03ac4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88103ac6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf03acb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbaf03acd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef703ad3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34103ad5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf03add, SUC_lexicon_tags_65 },
    { 0xd9103ade, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca903ae8, SUC_lexicon_tags_63 },
    { 0x94703ae9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a503aeb, SUC_lexicon_tags_37 },
    { 0xd0703aeb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b703aef, SUC_lexicon_tags_2 },
    { 0x4e703af0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7f03af2, SUC_lexicon_tags_13 },
    { 0xfed03af3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17d03af5, SUC_lexicon_tags_17 },
    { 0x28d03af6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xaaf03af8, SUC_lexicon_tags_15 },
    { 0x7703af8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31503afc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d503aff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5903b01, SUC_lexicon_tags_8 },
    { 0x74f03b01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xed103b04, SUC_lexicon_tags_7 },
    { 0xfa903b05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd103b07, SUC_lexicon_tags_2 },
    { 0xf503b08, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b503b12, SUC_lexicon_tags_66 },
    { 0x75503b13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48703b16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83d03b18, SUC_lexicon_tags_37 },
    { 0x46b03b18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf103b1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15f03b1d, SUC_lexicon_tags_6 },
    { 0x97b03b1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48703b22, SUC_lexicon_tags_9 },
    { 0xb103b23, SUC_lexicon_tags_1 },
    { 0x51d03b22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f03b2a, SUC_lexicon_tags_45 },
    { 0x57703b2b, SUC_lexicon_tags_66 },
    { 0xd5703b2b, SUC_lexicon_tags_1 },
    { 0xa4503b2d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf0f03b2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbb03b31, SUC_lexicon_tags_37 },
    { 0xc7103b32, SUC_lexicon_tags_38 },
    { 0x7b03b32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8303b3a, SUC_lexicon_tags_0 },
    { 0x49103b3b, SUC_lexicon_tags_15 },
    { 0x33303b3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b03b4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0703b4d, SUC_lexicon_tags_37 },
    { 0x8bf03b4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d03b53, SUC_lexicon_tags_37 },
    { 0x5dd03b54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bf03b56, SUC_lexicon_tags_26 },
    { 0xe8d03b57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11303b59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb503b5c, SUC_lexicon_tags_12 },
    { 0x5ab03b5d, SUC_lexicon_tags_60 },
    { 0x35f03b5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x89d03b60, SUC_lexicon_tags_25 },
    { 0xf8303b60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc0903b63, SUC_lexicon_tags_37 },
    { 0x47303b63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2903b67, SUC_lexicon_tags_23 },
    { 0x4a303b67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503b6c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f03b6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25703b73, SUC_lexicon_tags_2 },
    { 0xb7d03b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d03b7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf03b83, SUC_lexicon_tags_136 },
    { 0xe5103b84, SUC_lexicon_tags_13 },
    { 0x82b03b83, SUC_lexicon_tags_1 },
    { 0x63303b83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51703b8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f703b8c, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc303b94, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x19b03b96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4303b98, SUC_lexicon_tags_17 },
    { 0x8e903b98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x83303b9c, SUC_lexicon_tags_60 },
    { 0xb7103b9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x84f03b9f, SUC_lexicon_tags_33 },
    { 0x32103ba0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b03ba4, SUC_lexicon_tags_12 },
    { 0x13903ba5, SUC_lexicon_tags_15 },
    { 0x4f303ba4, SUC_lexicon_tags_32 },
    { 0x15703ba4, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb03baa, SUC_lexicon_tags_63 },
    { 0xba903bab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a903bad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f703bb7, SUC_lexicon_tags_11 },
    { 0xca703bb8, SUC_lexicon_tags_12 },
    { 0x8bb03bb9, SUC_lexicon_tags_39 },
    { 0x6eb03bb8, SUC_lexicon_tags_0 },
    { 0x90703bb7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce103bc0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa3d03bc2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x99703bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd03bc8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52103bcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d03bd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa503bda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f03bde, SUC_lexicon_tags_12 },
    { 0x17303bde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18b03be1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x93503be4, SUC_lexicon_tags_33 },
    { 0x92903be5, SUC_lexicon_tags_13 },
    { 0x26303be6, SUC_lexicon_tags_37 },
    { 0x4cf03be6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b03be9, SUC_lexicon_tags_55 },
    { 0x47503be9, SUC_lexicon_tags_11 },
    { 0x6903beb, SUC_lexicon_tags_0 },
    { 0xa103bea, SUC_lexicon_tags_11 },
    { 0xc1703bed, SUC_lexicon_tags_11 },
    { 0xf0d03bec, SUC_lexicon_tags_16 },
    { 0xa2d03bed, SUC_lexicon_tags_17 },
    { 0x41d03bf0, SUC_lexicon_tags_1 },
    { 0x59503bee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec503bf4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8503bf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9d03bfa, SUC_lexicon_tags_23 },
    { 0xe6303bfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d03bfe, SUC_lexicon_tags_8 },
    { 0x87703bff, SUC_lexicon_tags_0 },
    { 0xb3503bfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2703c04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb03c07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x82503c0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95b03c0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9703c0e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1303c16, SUC_lexicon_tags_11 },
    { 0xd9903c16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78303c1b, SUC_lexicon_tags_37 },
    { 0xe703c1c, SUC_lexicon_tags_44 },
    { 0x35103c1c, SUC_lexicon_tags_15 },
    { 0xa3103c1e, SUC_lexicon_tags_15 },
    { 0xc703c1f, SUC_lexicon_tags_0 },
    { 0x62d03c20, SUC_lexicon_tags_17 },
    { 0xb2303c21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03c24, SUC_lexicon_tags_35 },
    { 0xbe903c24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903c2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f03c2e, SUC_lexicon_tags_31 },
    { 0xbad03c2f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35303c34, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4903c37, SUC_lexicon_tags_2 },
    { 0x54703c38, SUC_lexicon_tags_11 },
    { 0x3e703c38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69d03c3b, SUC_lexicon_tags_15 },
    { 0x42f03c3b, SUC_lexicon_tags_17 },
    { 0x8c303c3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec103c41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f03c50, SUC_lexicon_tags_30 },
    { 0xa5103c50, SUC_lexicon_tags_7 },
    { 0xe3903c52, SUC_lexicon_tags_1 },
    { 0x90f03c53, SUC_lexicon_tags_38 },
    { 0x62903c50, SUC_lexicon_tags_6 },
    { 0x50903c55, SUC_lexicon_tags_45 },
    { 0xcff03c56, SUC_lexicon_tags_23 },
    { 0x60303c55, SUC_lexicon_tags_17 },
    { 0x64103c58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e303c5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9503c5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99b03c60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8903c65, SUC_lexicon_tags_33 },
    { 0x2e303c66, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe503c69, SUC_lexicon_tags_978 },
    { 0xff03c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca103c6c, SUC_lexicon_tags_1 },
    { 0xf6103c6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed03c71, SUC_lexicon_tags_1 },
    { 0xd503c71, SUC_lexicon_tags_25 },
    { 0xc4703c73, SUC_lexicon_tags_12 },
    { 0xb103c74, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x17303c76, SUC_lexicon_tags_11 },
    { 0xa9703c76, SUC_lexicon_tags_14 },
    { 0x85503c78, SUC_lexicon_tags_15 },
    { 0xbb903c77, SUC_lexicon_tags_48 },
    { 0xae703c78, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f303c81, SUC_lexicon_tags_9 },
    { 0x8f703c81, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xedf03c84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4903c87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cf03c89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb03c8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x45303c91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95303c93, SUC_lexicon_tags_6 },
    { 0xdd703c94, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa303c9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0103ca0, SUC_lexicon_tags_49 },
    { 0x68903ca1, SUC_lexicon_tags_263 },
    { 0xe1b03ca2, SUC_lexicon_tags_19 },
    { 0xa2903ca1, SUC_lexicon_tags_12 },
    { 0x12303ca4, SUC_lexicon_tags_12 },
    { 0x8703ca4, SUC_lexicon_tags_23 },
    { 0xd2b03ca6, SUC_lexicon_tags_14 },
    { 0xc8103ca4, SUC_lexicon_tags_25 },
    { 0x15f03ca0, SUC_lexicon_tags_11 },
    { 0x59703ca9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a903cac, SUC_lexicon_tags_3 },
    { 0x83503cad, SUC_lexicon_tags_100 },
    { 0xa7503cae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72903cb5, SUC_lexicon_tags_7 },
    { 0x70f03cb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36f03cb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15703cba, SUC_lexicon_tags_25 },
    { 0x98703cba, SUC_lexicon_tags_33 },
    { 0x31d03cbc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x60103cbe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x46703cc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4b03cc3, SUC_lexicon_tags_37 },
    { 0x64903cc4, SUC_lexicon_tags_1 },
    { 0x72903cc3, SUC_lexicon_tags_47 },
    { 0x86303cc5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a903cc9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0903ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb703cce, SUC_lexicon_tags_7 },
    { 0xc503ccf, SUC_lexicon_tags_17 },
    { 0xaef03cd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c903cd2, SUC_lexicon_tags_151 },
    { 0x8af03cd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd03cd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54903cd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb03cdd, SUC_lexicon_tags_28 },
    { 0xe6903cde, SUC_lexicon_tags_1 },
    { 0x84503cdf, SUC_lexicon_tags_28 },
    { 0x2b703ce0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56903ced, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e503cf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44103cf3, SUC_lexicon_tags_47 },
    { 0xf6d03cf4, SUC_lexicon_tags_2 },
    { 0xa8f03cf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f303cfa, SUC_lexicon_tags_7 },
    { 0xf7303cfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18d03cfd, SUC_lexicon_tags_17 },
    { 0x76f03cfe, SUC_lexicon_tags_12 },
    { 0x3c303cff, SUC_lexicon_tags_82 },
    { 0x79d03cff, SUC_lexicon_tags_37 },
    { 0xdcd03d01, SUC_lexicon_tags_8 },
    { 0x6bf03d02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b03d06, SUC_lexicon_tags_12 },
    { 0x80303d07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7903d0a, SUC_lexicon_tags_7 },
    { 0x5b103d0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf503d0f, SUC_lexicon_tags_7 },
    { 0xadf03d10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x78f03d12, SUC_lexicon_tags_37 },
    { 0xd0303d13, SUC_lexicon_tags_17 },
    { 0x14303d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf903d16, SUC_lexicon_tags_1 },
    { 0xcf903d16, SUC_lexicon_tags_2 },
    { 0x5903d16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b03d1b, SUC_lexicon_tags_7 },
    { 0xa7d03d1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80103d1e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xec503d20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef03d22, SUC_lexicon_tags_37 },
    { 0x9b903d22, SUC_lexicon_tags_1 },
    { 0xecd03d23, SUC_lexicon_tags_17 },
    { 0xccd03d23, SUC_lexicon_tags_33 },
    { 0x80f03d24, SUC_lexicon_tags_37 },
    { 0x22103d27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b503d2a, SUC_lexicon_tags_15 },
    { 0x2af03d2b, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db03d2f, SUC_lexicon_tags_37 },
    { 0xab303d2f, SUC_lexicon_tags_11 },
    { 0xe8703d2f, SUC_lexicon_tags_47 },
    { 0x79b03d32, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1503d3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d103d3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xddb03d3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7703d43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d903d46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14103d4c, SUC_lexicon_tags_16 },
    { 0xfb503d4d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d03d52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b03d56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x64303d58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b303d5b, SUC_lexicon_tags_7 },
    { 0x7f03d5c, SUC_lexicon_tags_19 },
    { 0xde303d5b, SUC_lexicon_tags_37 },
    { 0x9e103d5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86703d64, SUC_lexicon_tags_25 },
    { 0x94303d64, SUC_lexicon_tags_11 },
    { 0x2c903d65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdab03d68, SUC_lexicon_tags_15 },
    { 0x13503d69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5703d6b, SUC_lexicon_tags_19 },
    { 0xf4903d6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0303d6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67903d71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf703d73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f03d76, SUC_lexicon_tags_27 },
    { 0xd0b03d77, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd103d7d, SUC_lexicon_tags_48 },
    { 0x59503d7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4503d81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f03d85, SUC_lexicon_tags_77 },
    { 0x25b03d85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc903d89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b03d8e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b03d95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9dd03d97, SUC_lexicon_tags_129 },
    { 0xd6703d97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54d03d9a, SUC_lexicon_tags_8 },
    { 0xf9903d9b, SUC_lexicon_tags_14 },
    { 0xf5103d9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73b03d9e, SUC_lexicon_tags_33 },
    { 0x39103d9f, SUC_lexicon_tags_38 },
    { 0xe7503d9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c303da3, SUC_lexicon_tags_1 },
    { 0x54103da4, SUC_lexicon_tags_20 },
    { 0xf1103da3, SUC_lexicon_tags_2 },
    { 0x3ff03da5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f03dab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd703dad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71503db3, SUC_lexicon_tags_2 },
    { 0x88703db3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b03db7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x45503db9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c503dc0, SUC_lexicon_tags_7 },
    { 0x72903dc1, SUC_lexicon_tags_26 },
    { 0xd6903dc0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd4303dc4, SUC_lexicon_tags_11 },
    { 0x3a303dc4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6ab03dc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x41703dca, SUC_lexicon_tags_11 },
    { 0xf3303dcb, SUC_lexicon_tags_15 },
    { 0x3ef03dcc, SUC_lexicon_tags_27 },
    { 0xa7303dcb, SUC_lexicon_tags_6 },
    { 0xead03dce, SUC_lexicon_tags_27 },
    { 0xde103dcf, SUC_lexicon_tags_7 },
    { 0x21f03dcd, SUC_lexicon_tags_37 },
    { 0x62703dd1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a303dd5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e703dd7, SUC_lexicon_tags_37 },
    { 0x54303dd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45303ddb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0303ddf, SUC_lexicon_tags_37 },
    { 0xb5503de0, SUC_lexicon_tags_4 },
    { 0x66f03de0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b03de4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48103de7, SUC_lexicon_tags_17 },
    { 0xfd103de7, SUC_lexicon_tags_11 },
    { 0x37703de9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb903ded, SUC_lexicon_tags_15 },
    { 0x12b03dee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3503df0, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x63903df2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b03df5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d03df9, SUC_lexicon_tags_26 },
    { 0x3d103dfa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45703e04, SUC_lexicon_tags_8 },
    { 0x57903e04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb03e0b, SUC_lexicon_tags_1 },
    { 0x4c903e0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf03e11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb03e17, SUC_lexicon_tags_30 },
    { 0x34f03e18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc503e1d, SUC_lexicon_tags_15 },
    { 0x96903e1d, SUC_lexicon_tags_11 },
    { 0xee903e1f, SUC_lexicon_tags_25 },
    { 0x4e903e20, SUC_lexicon_tags_27 },
    { 0xc2303e21, SUC_lexicon_tags_37 },
    { 0xab903e22, SUC_lexicon_tags_12 },
    { 0x4ef03e22, SUC_lexicon_tags_25 },
    { 0x9f303e22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb03e26, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc903e2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b03e2f, SUC_lexicon_tags_39 },
    { 0x68703e2f, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0303e33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32903e3a, SUC_lexicon_tags_1 },
    { 0xe3903e3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a303e42, SUC_lexicon_tags_10 },
    { 0x91f03e43, SUC_lexicon_tags_1 },
    { 0x34103e44, SUC_lexicon_tags_57 },
    { 0xd9103e43, SUC_lexicon_tags_12 },
    { 0x90703e46, SUC_lexicon_tags_7 },
    { 0xa3503e47, SUC_lexicon_tags_11 },
    { 0xb7b03e47, SUC_lexicon_tags_47 },
    { 0xe4d03e42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91503e4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6503e52, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x37103e54, SUC_lexicon_tags_2 },
    { 0xf0303e55, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8c903e57, SUC_lexicon_tags_22 },
    { 0x47103e58, SUC_lexicon_tags_27 },
    { 0xe9303e59, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2fd03e5b, SUC_lexicon_tags_11 },
    { 0x42f03e5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92503e5e, SUC_lexicon_tags_48 },
    { 0x55d03e5f, SUC_lexicon_tags_11 },
    { 0xe1903e60, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3103e67, SUC_lexicon_tags_47 },
    { 0x7bf03e67, SUC_lexicon_tags_43 },
    { 0x87303e68, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc503e6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff103e70, SUC_lexicon_tags_37 },
    { 0x34903e71, SUC_lexicon_tags_7 },
    { 0xf8903e70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97103e74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0103e77, SUC_lexicon_tags_11 },
    { 0x8dd03e77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60503e7a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d03e7e, SUC_lexicon_tags_19 },
    { 0xb2b03e7f, SUC_lexicon_tags_12 },
    { 0x13703e7f, SUC_lexicon_tags_15 },
    { 0x2d103e7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a503e8b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39303e91, SUC_lexicon_tags_3 },
    { 0x3d503e92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43103e97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73103e99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77903e9e, SUC_lexicon_tags_32 },
    { 0x31503e9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41d03ea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10d03ea3, SUC_lexicon_tags_9 },
    { 0xc0703ea4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd03ea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4103eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7af03ead, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f03eb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf103eb4, SUC_lexicon_tags_46 },
    { 0xac703eb5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x18d03eb7, SUC_lexicon_tags_17 },
    { 0xb3b03eb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf4503eba, SUC_lexicon_tags_28 },
    { 0x10503eba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8503ec0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xed503ec3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d03ec6, SUC_lexicon_tags_19 },
    { 0xa6103ec7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf103eca, SUC_lexicon_tags_1 },
    { 0x5ad03ecb, SUC_lexicon_tags_7 },
    { 0xc4703ecb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39903ed1, SUC_lexicon_tags_32 },
    { 0x10703ed2, SUC_lexicon_tags_15 },
    { 0x14d03ed2, SUC_lexicon_tags_8 },
    { 0x12f03ed1, SUC_lexicon_tags_37 },
    { 0x7a103ed2, SUC_lexicon_tags_37 },
    { 0xd3903ed6, SUC_lexicon_tags_25 },
    { 0x41703ed6, SUC_lexicon_tags_60 },
    { 0xf9b03ed8, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xb6703eda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61303edf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70503ee1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe3f03ee3, SUC_lexicon_tags_14 },
    { 0x66b03ee4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b03ee8, SUC_lexicon_tags_37 },
    { 0x9103ee9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f03eec, SUC_lexicon_tags_47 },
    { 0x1fb03eed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e103ef3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb03efd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x62903f00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb03f0d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x78703f0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb703f11, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9b103f13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd03f16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65903f1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa503f20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7503f25, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d03f29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb6f03f2b, SUC_lexicon_tags_29 },
    { 0x4303f2c, SUC_lexicon_tags_87 },
    { 0xedf03f2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85303f31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60903f35, SUC_lexicon_tags_7 },
    { 0x20303f35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x81f03f38, SUC_lexicon_tags_19 },
    { 0x90f03f39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5503f3c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbaf03f3e, SUC_lexicon_tags_12 },
    { 0xe1b03f3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39503f51, SUC_lexicon_tags_23 },
    { 0xf5303f52, SUC_lexicon_tags_11 },
    { 0x89d03f52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55103f57, SUC_lexicon_tags_10 },
    { 0xbbf03f58, SUC_lexicon_tags_7 },
    { 0x13d03f59, SUC_lexicon_tags_25 },
    { 0x8ff03f5a, SUC_lexicon_tags_49 },
    { 0xffd03f57, SUC_lexicon_tags_14 },
    { 0x71d03f5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5503f5e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b03f62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4103f66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf03f6f, SUC_lexicon_tags_84 },
    { 0x2cd03f6f, SUC_lexicon_tags_37 },
    { 0x3f703f71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde703f77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcdf03f79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60503f7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed103f82, SUC_lexicon_tags_17 },
    { 0xead03f82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf303f86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6f03f88, SUC_lexicon_tags_79 },
    { 0x50103f89, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24303f8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x65d03f8f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x13303f91, SUC_lexicon_tags_15 },
    { 0xeaf03f92, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4703f96, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x21f03f98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54f03f9a, SUC_lexicon_tags_2 },
    { 0xa4703f9b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3e103f9d, SUC_lexicon_tags_11 },
    { 0xa6903f9e, SUC_lexicon_tags_31 },
    { 0xfdd03f9f, SUC_lexicon_tags_33 },
    { 0xc8f03fa0, SUC_lexicon_tags_28 },
    { 0xd9103fa1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22103fa6, SUC_lexicon_tags_37 },
    { 0xae103fa6, SUC_lexicon_tags_49 },
    { 0xb4f03fa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f103fac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f03fb1, SUC_lexicon_tags_33 },
    { 0x5cb03fb2, SUC_lexicon_tags_55 },
    { 0x58d03fb3, SUC_lexicon_tags_47 },
    { 0x4d303fb1, SUC_lexicon_tags_37 },
    { 0x3e903fb3, SUC_lexicon_tags_2 },
    { 0x54b03fb6, SUC_lexicon_tags_1 },
    { 0x7f503fb7, SUC_lexicon_tags_25 },
    { 0x50f03fb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d03fbd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x3f03fbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73103fc4, SUC_lexicon_tags_47 },
    { 0x12b03fc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8303fc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f03fcc, SUC_lexicon_tags_2 },
    { 0xa1103fcd, SUC_lexicon_tags_15 },
    { 0xa1103fcc, SUC_lexicon_tags_15 },
    { 0x61b03fcd, SUC_lexicon_tags_7 },
    { 0x9b703fce, SUC_lexicon_tags_1 },
    { 0xdff03fd1, SUC_lexicon_tags_33 },
    { 0x4d903fd0, SUC_lexicon_tags_19 },
    { 0xea503fcd, SUC_lexicon_tags_7 },
    { 0xe0503fd1, SUC_lexicon_tags_17 },
    { 0xa703fd5, SUC_lexicon_tags_48 },
    { 0x32b03fd3, SUC_lexicon_tags_23 },
    { 0x46103fd7, SUC_lexicon_tags_4 },
    { 0x30f03fd7, SUC_lexicon_tags_34 },
    { 0x6cd03fd8, SUC_lexicon_tags_1 },
    { 0x73703fd7, SUC_lexicon_tags_10 },
    { 0x6bb03fdb, SUC_lexicon_tags_2 },
    { 0x2b103fdc, SUC_lexicon_tags_37 },
    { 0x41b03fd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6bb03fdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e503fe4, SUC_lexicon_tags_35 },
    { 0x5f103fe4, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x40103fe8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x45903fea, SUC_lexicon_tags_15 },
    { 0xf9f03fea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd03fee, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7903ff2, SUC_lexicon_tags_17 },
    { 0xc5503ff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c303ff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31903fff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88104005, SUC_lexicon_tags_17 },
    { 0xb1b04005, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f04009, SUC_lexicon_tags_11 },
    { 0x49f0400a, SUC_lexicon_tags_2 },
    { 0x75504009, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x5c90400d, SUC_lexicon_tags_65 },
    { 0x2510400e, SUC_lexicon_tags_2 },
    { 0xaab0400f, SUC_lexicon_tags_25 },
    { 0xb0b04010, SUC_lexicon_tags_47 },
    { 0x81f0400d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8770401c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa990401f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7504021, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd04029, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xae50402c, SUC_lexicon_tags_27 },
    { 0x4070402d, SUC_lexicon_tags_25 },
    { 0x10b0402e, SUC_lexicon_tags_37 },
    { 0x310402f, SUC_lexicon_tags_25 },
    { 0x5470402f, SUC_lexicon_tags_17 },
    { 0x62f04031, SUC_lexicon_tags_15 },
    { 0xa8d04032, SUC_lexicon_tags_15 },
    { 0x88504032, SUC_lexicon_tags_11 },
    { 0x75104034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7130403e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd1104040, SUC_lexicon_tags_10 },
    { 0xe6f04041, SUC_lexicon_tags_25 },
    { 0xc4504042, SUC_lexicon_tags_2 },
    { 0x89904043, SUC_lexicon_tags_11 },
    { 0xead04044, SUC_lexicon_tags_12 },
    { 0x60104044, SUC_lexicon_tags_27 },
    { 0xe2104042, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbdd04048, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb70404b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4b10404d, SUC_lexicon_tags_17 },
    { 0x1490404d, SUC_lexicon_tags_63 },
    { 0x7270404f, SUC_lexicon_tags_0 },
    { 0xe2b0404f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd704052, SUC_lexicon_tags_45 },
    { 0x49904052, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x31104056, SUC_lexicon_tags_17 },
    { 0xee704057, SUC_lexicon_tags_27 },
    { 0xa5d04057, SUC_lexicon_tags_35 },
    { 0x40b04059, SUC_lexicon_tags_49 },
    { 0xe310405a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63704061, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1704063, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0xde304066, SUC_lexicon_tags_4 },
    { 0xdb904067, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c90406f, SUC_lexicon_tags_33 },
    { 0x74d04070, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce504074, SUC_lexicon_tags_6 },
    { 0x27904074, SUC_lexicon_tags_8 },
    { 0x86904076, SUC_lexicon_tags_2 },
    { 0xbfd04076, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf04084, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2050408f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37304099, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2210409c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed90409e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5d1040a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef040a3, SUC_lexicon_tags_317 },
    { 0xf07040a4, SUC_lexicon_tags_25 },
    { 0x60b040a3, SUC_lexicon_tags_1 },
    { 0x99b040a4, SUC_lexicon_tags_1504 },
    { 0xd1f040a4, SUC_lexicon_tags_0 },
    { 0x4d040a5, SUC_lexicon_tags_37 },
    { 0x8e5040a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311040ad, SUC_lexicon_tags_30 },
    { 0xfa5040ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953040b2, SUC_lexicon_tags_12 },
    { 0x3d3040b3, SUC_lexicon_tags_37 },
    { 0x701040b3, SUC_lexicon_tags_2 },
    { 0xd49040b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cb040b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943040c0, SUC_lexicon_tags_7 },
    { 0x615040c1, SUC_lexicon_tags_30 },
    { 0x5c1040c2, SUC_lexicon_tags_38 },
    { 0xe27040c0, SUC_lexicon_tags_17 },
    { 0xde7040c4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfbf040c6, SUC_lexicon_tags_2 },
    { 0xb2f040c6, SUC_lexicon_tags_25 },
    { 0x8bb040c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4f1040ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22b040cc, SUC_lexicon_tags_1 },
    { 0x4c5040cd, SUC_lexicon_tags_11 },
    { 0x607040ce, SUC_lexicon_tags_37 },
    { 0xce5040cf, SUC_lexicon_tags_15 },
    { 0xe1040cf, SUC_lexicon_tags_17 },
    { 0x971040d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f040d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1040de, SUC_lexicon_tags_23 },
    { 0xa15040df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849040e4, SUC_lexicon_tags_14 },
    { 0x6af040e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x371040e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x455040eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f040f0, SUC_lexicon_tags_43 },
    { 0xbe3040f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d040f4, SUC_lexicon_tags_420 },
    { 0x569040f5, SUC_lexicon_tags_0 },
    { 0x89d040f6, SUC_lexicon_tags_8 },
    { 0xaad040f7, SUC_lexicon_tags_17 },
    { 0x34b040f7, SUC_lexicon_tags_60 },
    { 0xc07040f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15040fd, SUC_lexicon_tags_10 },
    { 0x417040fd, SUC_lexicon_tags_19 },
    { 0x47f040ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb04102, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd1504104, SUC_lexicon_tags_11 },
    { 0x32904105, SUC_lexicon_tags_6 },
    { 0xb3904106, SUC_lexicon_tags_11 },
    { 0x42504107, SUC_lexicon_tags_16 },
    { 0x68f04108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a70410a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa590410e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20904114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc504125, SUC_lexicon_tags_5 },
    { 0x5eb04125, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6904129, SUC_lexicon_tags_29 },
    { 0x3eb04129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad04130, SUC_lexicon_tags_25 },
    { 0xb0304131, SUC_lexicon_tags_2 },
    { 0xcd104131, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48904134, SUC_lexicon_tags_12 },
    { 0xfc304134, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2dd04137, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0413a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f0413d, SUC_lexicon_tags_127 },
    { 0x9750413e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c704141, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6c504143, SUC_lexicon_tags_14 },
    { 0x16d04143, SUC_lexicon_tags_65 },
    { 0x61d04143, SUC_lexicon_tags_12 },
    { 0xe4b04146, SUC_lexicon_tags_1 },
    { 0xa1d04147, SUC_lexicon_tags_29 },
    { 0x6504143, SUC_lexicon_tags_31 },
    { 0xa2f04149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd490414b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10414e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db04154, SUC_lexicon_tags_2 },
    { 0xc2904155, SUC_lexicon_tags_7 },
    { 0x5f04156, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd0415a, SUC_lexicon_tags_47 },
    { 0xe0f0415b, SUC_lexicon_tags_68 },
    { 0xf090415c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd30415e, SUC_lexicon_tags_7 },
    { 0x9970415e, SUC_lexicon_tags_31 },
    { 0xdd90415f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d104162, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda504164, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c104166, SUC_lexicon_tags_12 },
    { 0xd9b04167, SUC_lexicon_tags_43 },
    { 0x7fb04168, SUC_lexicon_tags_31 },
    { 0x5a504166, SUC_lexicon_tags_23 },
    { 0xabf04168, SUC_lexicon_tags_27 },
    { 0x3290416b, SUC_lexicon_tags_27 },
    { 0xd70416c, SUC_lexicon_tags_47 },
    { 0x7090416c, SUC_lexicon_tags_34 },
    { 0x2990416e, SUC_lexicon_tags_27 },
    { 0x46f0416f, SUC_lexicon_tags_15 },
    { 0x9010416f, SUC_lexicon_tags_48 },
    { 0x23504171, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d04179, SUC_lexicon_tags_25 },
    { 0x3bf0417a, SUC_lexicon_tags_22 },
    { 0x48904179, SUC_lexicon_tags_12 },
    { 0x1f50417c, SUC_lexicon_tags_126 },
    { 0x1db0417d, SUC_lexicon_tags_12 },
    { 0x62f0417e, SUC_lexicon_tags_17 },
    { 0x5410417b, SUC_lexicon_tags_2 },
    { 0x36104179, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0304186, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f0418e, SUC_lexicon_tags_0 },
    { 0x9630418f, SUC_lexicon_tags_11 },
    { 0xbcb04190, SUC_lexicon_tags_15 },
    { 0x330418f, SUC_lexicon_tags_84 },
    { 0xab304190, SUC_lexicon_tags_37 },
    { 0xc9f04191, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b0419e, SUC_lexicon_tags_45 },
    { 0xb30419f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5041a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b041a7, SUC_lexicon_tags_32 },
    { 0xe39041a8, SUC_lexicon_tags_11 },
    { 0x12f041a7, SUC_lexicon_tags_11 },
    { 0x9ff041aa, SUC_lexicon_tags_29 },
    { 0xd97041a8, SUC_lexicon_tags_6 },
    { 0x459041ac, SUC_lexicon_tags_25 },
    { 0xc21041ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ed041af, SUC_lexicon_tags_1 },
    { 0x427041af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb041b4, SUC_lexicon_tags_7 },
    { 0x79d041b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcf041b7, SUC_lexicon_tags_37 },
    { 0x1d3041b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed041bc, SUC_lexicon_tags_15 },
    { 0xcb5041bd, SUC_lexicon_tags_26 },
    { 0x719041bd, SUC_lexicon_tags_47 },
    { 0x8c7041bd, SUC_lexicon_tags_45 },
    { 0x757041c0, SUC_lexicon_tags_13 },
    { 0x611041c1, SUC_lexicon_tags_4 },
    { 0x31d041c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x451041c4, SUC_lexicon_tags_17 },
    { 0x203041c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23041ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa89041cc, SUC_lexicon_tags_1 },
    { 0x92f041cd, SUC_lexicon_tags_11 },
    { 0x983041cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71f041d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed041d7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1041e0, SUC_lexicon_tags_38 },
    { 0xf8d041e1, SUC_lexicon_tags_25 },
    { 0x1e3041e2, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d041e6, SUC_lexicon_tags_46 },
    { 0xb63041e7, SUC_lexicon_tags_14 },
    { 0xaeb041e7, SUC_lexicon_tags_16 },
    { 0x9e3041e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7041ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9a7041ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0b041f1, SUC_lexicon_tags_15 },
    { 0x811041f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68f041f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd041f7, SUC_lexicon_tags_45 },
    { 0x32b041f8, SUC_lexicon_tags_30 },
    { 0x3b1041f7, SUC_lexicon_tags_8 },
    { 0xdb041fa, SUC_lexicon_tags_11 },
    { 0xaa7041fb, SUC_lexicon_tags_17 },
    { 0x431041fc, SUC_lexicon_tags_17 },
    { 0x67041fd, SUC_lexicon_tags_113 },
    { 0x735041fb, SUC_lexicon_tags_37 },
    { 0x2bf041fc, SUC_lexicon_tags_19 },
    { 0x58504200, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f04203, SUC_lexicon_tags_15 },
    { 0xf5704204, SUC_lexicon_tags_92 },
    { 0x4fd04205, SUC_lexicon_tags_37 },
    { 0xf7904204, SUC_lexicon_tags_33 },
    { 0x4f04207, SUC_lexicon_tags_38 },
    { 0x52504207, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd0420c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf04212, SUC_lexicon_tags_12 },
    { 0x95304213, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb04218, SUC_lexicon_tags_30 },
    { 0x304219, SUC_lexicon_tags_42 },
    { 0x2cb04218, SUC_lexicon_tags_37 },
    { 0x8af04218, SUC_lexicon_tags_2 },
    { 0x77704219, SUC_lexicon_tags_1 },
    { 0xde10421d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xff904220, SUC_lexicon_tags_63 },
    { 0xf0704221, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63104225, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f04227, SUC_lexicon_tags_17 },
    { 0x27104228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b0422e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81104232, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11704236, SUC_lexicon_tags_45 },
    { 0x1e904237, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5190423a, SUC_lexicon_tags_37 },
    { 0x4c10423b, SUC_lexicon_tags_2 },
    { 0xa570423c, SUC_lexicon_tags_15 },
    { 0x3a50423c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3bb0423f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90504241, SUC_lexicon_tags_95 },
    { 0x504241, SUC_lexicon_tags_30 },
    { 0x85d04243, SUC_lexicon_tags_17 },
    { 0x10504244, SUC_lexicon_tags_31 },
    { 0xc6f04243, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a904247, SUC_lexicon_tags_15 },
    { 0xf4304248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e90424c, SUC_lexicon_tags_25 },
    { 0xb870424d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3d10424f, SUC_lexicon_tags_37 },
    { 0xa570424f, SUC_lexicon_tags_37 },
    { 0xd750424f, SUC_lexicon_tags_2 },
    { 0x8e704252, SUC_lexicon_tags_23 },
    { 0x42504253, SUC_lexicon_tags_70 },
    { 0x8f104253, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f104256, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0425c, SUC_lexicon_tags_30 },
    { 0xaad0425c, SUC_lexicon_tags_43 },
    { 0xf590425e, SUC_lexicon_tags_39 },
    { 0x1d90425d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16304263, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f0426b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b0426f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xda904272, SUC_lexicon_tags_37 },
    { 0x80d04273, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f904278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f04280, SUC_lexicon_tags_6 },
    { 0xe1f04280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f04288, SUC_lexicon_tags_19 },
    { 0x61304288, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa910428c, SUC_lexicon_tags_29 },
    { 0x89f0428d, SUC_lexicon_tags_2 },
    { 0xcf70428d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96904298, SUC_lexicon_tags_31 },
    { 0x7af04299, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b70429d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f042a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd67042a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbc7042a4, SUC_lexicon_tags_1 },
    { 0x6ed042a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x787042a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb042ae, SUC_lexicon_tags_39 },
    { 0x80f042af, SUC_lexicon_tags_63 },
    { 0xd59042af, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1042b5, SUC_lexicon_tags_66 },
    { 0xa35042b5, SUC_lexicon_tags_7 },
    { 0xb73042b5, SUC_lexicon_tags_35 },
    { 0xc2d042b5, SUC_lexicon_tags_14 },
    { 0x1df042b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7042bd, SUC_lexicon_tags_11 },
    { 0xd43042be, SUC_lexicon_tags_60 },
    { 0xe1b042be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509042c3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x371042c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55042d0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x811042d2, SUC_lexicon_tags_25 },
    { 0x487042d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x55b042d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff042d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ef042db, SUC_lexicon_tags_43 },
    { 0x875042dc, SUC_lexicon_tags_43 },
    { 0x17042dd, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d042e1, SUC_lexicon_tags_542 },
    { 0x7db042e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7042e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d042eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9d042ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x767042ef, SUC_lexicon_tags_14 },
    { 0xc43042ef, SUC_lexicon_tags_14 },
    { 0x903042ef, SUC_lexicon_tags_29 },
    { 0xac7042f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x713042f5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5042fa, SUC_lexicon_tags_15 },
    { 0x273042fb, SUC_lexicon_tags_45 },
    { 0x165042fc, SUC_lexicon_tags_39 },
    { 0x99b042fc, SUC_lexicon_tags_1 },
    { 0x2b1042fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef504300, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x2904303, SUC_lexicon_tags_21 },
    { 0xccb04304, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a90430a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f50430d, SUC_lexicon_tags_45 },
    { 0x5410430e, SUC_lexicon_tags_1 },
    { 0x1b0430d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a304316, SUC_lexicon_tags_27 },
    { 0x1af04317, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd0431a, SUC_lexicon_tags_2 },
    { 0x1390431b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cf0431d, SUC_lexicon_tags_11 },
    { 0xcab0431e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f04322, SUC_lexicon_tags_19 },
    { 0x9c704323, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f04326, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x98704328, SUC_lexicon_tags_77 },
    { 0x60104329, SUC_lexicon_tags_12 },
    { 0xb8f04329, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf70432e, SUC_lexicon_tags_49 },
    { 0xe330432f, SUC_lexicon_tags_48 },
    { 0x490432e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88704334, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8304336, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65904338, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b04340, SUC_lexicon_tags_43 },
    { 0x47504341, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ad04343, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed04346, SUC_lexicon_tags_17 },
    { 0xd2904347, SUC_lexicon_tags_65 },
    { 0x78304348, SUC_lexicon_tags_31 },
    { 0x17d04349, SUC_lexicon_tags_25 },
    { 0x5350434a, SUC_lexicon_tags_11 },
    { 0x5304346, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec70434d, SUC_lexicon_tags_47 },
    { 0x1c30434e, SUC_lexicon_tags_15 },
    { 0xb630434f, SUC_lexicon_tags_47 },
    { 0x9eb0434f, SUC_lexicon_tags_63 },
    { 0xe7104351, SUC_lexicon_tags_12 },
    { 0x1030434f, SUC_lexicon_tags_37 },
    { 0x56504352, SUC_lexicon_tags_63 },
    { 0xda704350, SUC_lexicon_tags_17 },
    { 0xb1504355, SUC_lexicon_tags_1 },
    { 0x2b904356, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d504358, SUC_lexicon_tags_0 },
    { 0x48b04359, SUC_lexicon_tags_27 },
    { 0xc1904358, SUC_lexicon_tags_7 },
    { 0xd2b0435b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe090435e, SUC_lexicon_tags_44 },
    { 0xe890435f, SUC_lexicon_tags_27 },
    { 0x3830435f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc904363, SUC_lexicon_tags_27 },
    { 0x9a704364, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf10436e, SUC_lexicon_tags_37 },
    { 0x7c50436f, SUC_lexicon_tags_82 },
    { 0xbb104370, SUC_lexicon_tags_41 },
    { 0xb7f04371, SUC_lexicon_tags_38 },
    { 0xd4b04372, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14504378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50b0437a, SUC_lexicon_tags_66 },
    { 0x30f0437b, SUC_lexicon_tags_13 },
    { 0xfef0437c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd510437e, SUC_lexicon_tags_45 },
    { 0xc310437e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0704384, SUC_lexicon_tags_37 },
    { 0x24904385, SUC_lexicon_tags_161 },
    { 0xb9d04386, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabd04388, SUC_lexicon_tags_58 },
    { 0x24704389, SUC_lexicon_tags_5 },
    { 0xf6304389, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c10438e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1f904390, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1a104392, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a304398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0439c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa350439f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x633043a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d043a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af043a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899043b5, SUC_lexicon_tags_60 },
    { 0x7f7043b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f043b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719043be, SUC_lexicon_tags_23 },
    { 0x2f9043be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f043c2, SUC_lexicon_tags_7 },
    { 0x937043c3, SUC_lexicon_tags_15 },
    { 0x381043c4, SUC_lexicon_tags_1 },
    { 0xa8f043c5, SUC_lexicon_tags_17 },
    { 0x967043c4, SUC_lexicon_tags_9 },
    { 0x24d043c7, SUC_lexicon_tags_95 },
    { 0x635043c5, SUC_lexicon_tags_12 },
    { 0x3a1043c2, SUC_lexicon_tags_148 },
    { 0x511043c5, SUC_lexicon_tags_7 },
    { 0xd03043c9, SUC_lexicon_tags_19 },
    { 0x373043cc, SUC_lexicon_tags_6 },
    { 0xda1043cd, SUC_lexicon_tags_6 },
    { 0xb5f043cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539043d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7043dc, SUC_lexicon_tags_38 },
    { 0xbc9043dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89043e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77043e3, SUC_lexicon_tags_17 },
    { 0xe45043e3, SUC_lexicon_tags_11 },
    { 0xae3043e5, SUC_lexicon_tags_35 },
    { 0xc21043e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d043e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9043ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x389043ef, SUC_lexicon_tags_116 },
    { 0xd33043f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x567043f3, SUC_lexicon_tags_5 },
    { 0xec5043f4, SUC_lexicon_tags_13 },
    { 0xb89043f5, SUC_lexicon_tags_43 },
    { 0x5c1043f6, SUC_lexicon_tags_11 },
    { 0xec3043f7, SUC_lexicon_tags_15 },
    { 0x7b5043f6, SUC_lexicon_tags_106 },
    { 0x77b043f9, SUC_lexicon_tags_26 },
    { 0xf3b043f8, SUC_lexicon_tags_37 },
    { 0x1d3043fb, SUC_lexicon_tags_27 },
    { 0xced043fc, SUC_lexicon_tags_45 },
    { 0x89b043f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x27d043ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d504405, SUC_lexicon_tags_32 },
    { 0x20504406, SUC_lexicon_tags_11 },
    { 0xbf904407, SUC_lexicon_tags_62 },
    { 0x18104405, SUC_lexicon_tags_1 },
    { 0x78b04407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc550440f, SUC_lexicon_tags_1364 },
    { 0, NULL },
    { 0x16504411, SUC_lexicon_tags_11 },
    { 0x4db04411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55904414, SUC_lexicon_tags_17 },
    { 0xb4504414, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45904418, SUC_lexicon_tags_1 },
    { 0xb4f04419, SUC_lexicon_tags_0 },
    { 0xabb04419, SUC_lexicon_tags_11 },
    { 0x1930441b, SUC_lexicon_tags_25 },
    { 0xa310441a, SUC_lexicon_tags_32 },
    { 0x3210441d, SUC_lexicon_tags_27 },
    { 0x43d0441e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x104421, SUC_lexicon_tags_1 },
    { 0xd9504421, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b04426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89504428, SUC_lexicon_tags_32 },
    { 0xead04429, SUC_lexicon_tags_1478 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d0442e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1904437, SUC_lexicon_tags_11 },
    { 0xfb304438, SUC_lexicon_tags_12 },
    { 0xfd904438, SUC_lexicon_tags_15 },
    { 0x40d04438, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf10443c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4704440, SUC_lexicon_tags_14 },
    { 0x73504440, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd04445, SUC_lexicon_tags_31 },
    { 0xcd504445, SUC_lexicon_tags_27 },
    { 0xa5b04447, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1070444c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41704450, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xea304452, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4504457, SUC_lexicon_tags_11 },
    { 0x16b04457, SUC_lexicon_tags_8 },
    { 0xe8d04459, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d30445b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf4b0445d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55904462, SUC_lexicon_tags_5 },
    { 0x34304462, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x51904466, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e304468, SUC_lexicon_tags_21 },
    { 0x2c504468, SUC_lexicon_tags_43 },
    { 0x1fd0446a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f0446e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb504470, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e904476, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6850447d, SUC_lexicon_tags_84 },
    { 0xd50447e, SUC_lexicon_tags_17 },
    { 0xea10447f, SUC_lexicon_tags_4 },
    { 0x1530447f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f04483, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf104485, SUC_lexicon_tags_6 },
    { 0x63104485, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4d304488, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf550448a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0448d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe704491, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12704496, SUC_lexicon_tags_15 },
    { 0x2d04496, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87304499, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23b0449b, SUC_lexicon_tags_29 },
    { 0xa50449c, SUC_lexicon_tags_12 },
    { 0x81f0449d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4510449f, SUC_lexicon_tags_14 },
    { 0xdb3044a0, SUC_lexicon_tags_37 },
    { 0x1a5044a0, SUC_lexicon_tags_13 },
    { 0x750449f, SUC_lexicon_tags_1 },
    { 0xd01044a1, SUC_lexicon_tags_11 },
    { 0xeb10449f, SUC_lexicon_tags_2 },
    { 0x2eb044a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xadf044a7, SUC_lexicon_tags_11 },
    { 0xbd5044a8, SUC_lexicon_tags_37 },
    { 0x1bd044a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf37044ab, SUC_lexicon_tags_38 },
    { 0x63b044ac, SUC_lexicon_tags_27 },
    { 0x77d044ac, SUC_lexicon_tags_2 },
    { 0x38b044ac, SUC_lexicon_tags_19 },
    { 0x2eb044ac, SUC_lexicon_tags_1 },
    { 0x953044ad, SUC_lexicon_tags_0 },
    { 0xe07044b1, SUC_lexicon_tags_1 },
    { 0xa09044b2, SUC_lexicon_tags_28 },
    { 0x71f044b3, SUC_lexicon_tags_17 },
    { 0x64f044b4, SUC_lexicon_tags_12 },
    { 0x4b5044b2, SUC_lexicon_tags_63 },
    { 0x235044b6, SUC_lexicon_tags_19 },
    { 0x9f9044b4, SUC_lexicon_tags_30 },
    { 0xb8d044b8, SUC_lexicon_tags_25 },
    { 0xdd9044b2, SUC_lexicon_tags_91 },
    { 0xd61044ba, SUC_lexicon_tags_12 },
    { 0x9df044b7, SUC_lexicon_tags_12 },
    { 0xf69044ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8b044be, SUC_lexicon_tags_14 },
    { 0x1cf044bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23044c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07044c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x815044c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11044cd, SUC_lexicon_tags_2 },
    { 0xab3044cd, SUC_lexicon_tags_11 },
    { 0x4ad044cf, SUC_lexicon_tags_12 },
    { 0xffb044d0, SUC_lexicon_tags_45 },
    { 0x79044ce, SUC_lexicon_tags_49 },
    { 0xb1044d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3044d7, SUC_lexicon_tags_11 },
    { 0x5c3044d8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23044dd, SUC_lexicon_tags_17 },
    { 0x3b5044de, SUC_lexicon_tags_47 },
    { 0xe3f044df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cf044e1, SUC_lexicon_tags_26 },
    { 0x67044e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3044ec, SUC_lexicon_tags_1 },
    { 0xc13044ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd044f0, SUC_lexicon_tags_27 },
    { 0x81f044f1, SUC_lexicon_tags_9 },
    { 0x855044f1, SUC_lexicon_tags_45 },
    { 0xe09044f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b044f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc7d044fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1044ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f04504, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38104507, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71504509, SUC_lexicon_tags_1 },
    { 0x810450a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf090450c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0504517, SUC_lexicon_tags_1 },
    { 0x4a104518, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b0451e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1504521, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46504525, SUC_lexicon_tags_14 },
    { 0xcff04525, SUC_lexicon_tags_30 },
    { 0xc8504526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3890452a, SUC_lexicon_tags_19 },
    { 0xfd0452b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4904530, SUC_lexicon_tags_2 },
    { 0x45b04531, SUC_lexicon_tags_38 },
    { 0xc1904531, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x35704535, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe9904537, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb0453b, SUC_lexicon_tags_49 },
    { 0xc690453c, SUC_lexicon_tags_8 },
    { 0xe150453d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b04540, SUC_lexicon_tags_34 },
    { 0x9cf04540, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x7cf04543, SUC_lexicon_tags_15 },
    { 0x13904544, SUC_lexicon_tags_11 },
    { 0xc4f04545, SUC_lexicon_tags_29 },
    { 0x5504546, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe210454a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9b0454c, SUC_lexicon_tags_7 },
    { 0x7790454c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d0454f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83904552, SUC_lexicon_tags_30 },
    { 0xafb04553, SUC_lexicon_tags_2 },
    { 0x10504554, SUC_lexicon_tags_7 },
    { 0xe1704552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55b04557, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e30455c, SUC_lexicon_tags_60 },
    { 0x48d0455c, SUC_lexicon_tags_15 },
    { 0x7bf0455e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd04561, SUC_lexicon_tags_38 },
    { 0xfc704562, SUC_lexicon_tags_9 },
    { 0xff104563, SUC_lexicon_tags_14 },
    { 0x98f04564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ef04566, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0504569, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa010456b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c90456e, SUC_lexicon_tags_230 },
    { 0x78b0456f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f104571, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b04575, SUC_lexicon_tags_1 },
    { 0x93504576, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f04579, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7530457f, SUC_lexicon_tags_63 },
    { 0x5bf04580, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b04585, SUC_lexicon_tags_76 },
    { 0x9dd04586, SUC_lexicon_tags_3 },
    { 0x27304586, SUC_lexicon_tags_4 },
    { 0x95904588, SUC_lexicon_tags_25 },
    { 0xa2b04589, SUC_lexicon_tags_14 },
    { 0x19f0458a, SUC_lexicon_tags_14 },
    { 0x4130458b, SUC_lexicon_tags_49 },
    { 0x2bf0458a, SUC_lexicon_tags_1 },
    { 0xe110458a, SUC_lexicon_tags_11 },
    { 0x3350458e, SUC_lexicon_tags_17 },
    { 0xb7d0458e, SUC_lexicon_tags_12 },
    { 0xae304589, SUC_lexicon_tags_4 },
    { 0xd5f0458b, SUC_lexicon_tags_7 },
    { 0xf510458f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec504595, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65045a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x907045a3, SUC_lexicon_tags_25 },
    { 0x50f045a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f045a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x519045a9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f045af, SUC_lexicon_tags_11 },
    { 0x87045b0, SUC_lexicon_tags_0 },
    { 0x52b045b1, SUC_lexicon_tags_23 },
    { 0x8b9045b2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841045be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf045c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde3045c3, SUC_lexicon_tags_47 },
    { 0xc33045c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x541045c7, SUC_lexicon_tags_15 },
    { 0xc77045c8, SUC_lexicon_tags_7 },
    { 0x3cb045c7, SUC_lexicon_tags_1 },
    { 0x9a5045ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5045cd, SUC_lexicon_tags_27 },
    { 0x50b045cd, SUC_lexicon_tags_56 },
    { 0x4b9045cf, SUC_lexicon_tags_60 },
    { 0xfb045d0, SUC_lexicon_tags_13 },
    { 0x6b3045d1, SUC_lexicon_tags_12 },
    { 0x87045d1, SUC_lexicon_tags_17 },
    { 0xfbd045d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94d045d5, SUC_lexicon_tags_1 },
    { 0x4cb045d5, SUC_lexicon_tags_14 },
    { 0xb7d045d6, SUC_lexicon_tags_37 },
    { 0xc83045d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x383045db, SUC_lexicon_tags_25 },
    { 0x25f045dc, SUC_lexicon_tags_37 },
    { 0xe65045dc, SUC_lexicon_tags_25 },
    { 0x7eb045de, SUC_lexicon_tags_77 },
    { 0xc5f045df, SUC_lexicon_tags_41 },
    { 0xb31045dc, SUC_lexicon_tags_588 },
    { 0, NULL },
    { 0x27f045e2, SUC_lexicon_tags_145 },
    { 0xceb045e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9045e7, SUC_lexicon_tags_47 },
    { 0x111045e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa81045ea, SUC_lexicon_tags_23 },
    { 0x1f5045eb, SUC_lexicon_tags_19 },
    { 0x667045ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb045f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe27045f3, SUC_lexicon_tags_1 },
    { 0x8b9045f4, SUC_lexicon_tags_11 },
    { 0x6b3045f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43045f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31045fc, SUC_lexicon_tags_7 },
    { 0x9e1045fd, SUC_lexicon_tags_25 },
    { 0x591045fc, SUC_lexicon_tags_15 },
    { 0xf73045ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8504602, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d04607, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0460a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5850460d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96304615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac104618, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a10461d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2c70461f, SUC_lexicon_tags_17 },
    { 0xd7b0461f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24f04622, SUC_lexicon_tags_7 },
    { 0x69704623, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x61d04625, SUC_lexicon_tags_11 },
    { 0x5ab04625, SUC_lexicon_tags_1 },
    { 0xffd04627, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c70462d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24304634, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f504636, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35104638, SUC_lexicon_tags_25 },
    { 0xa6504638, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b0463d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4a50463f, SUC_lexicon_tags_7 },
    { 0x81b04640, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6b304642, SUC_lexicon_tags_2 },
    { 0xdfd04642, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39504645, SUC_lexicon_tags_87 },
    { 0xd3104646, SUC_lexicon_tags_0 },
    { 0x704647, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef30464c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec90464f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3104652, SUC_lexicon_tags_31 },
    { 0x4d04653, SUC_lexicon_tags_25 },
    { 0xa504654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b04659, SUC_lexicon_tags_37 },
    { 0x1504659, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf0465e, SUC_lexicon_tags_11 },
    { 0xfef0465f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34504663, SUC_lexicon_tags_17 },
    { 0xfa504664, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d10466a, SUC_lexicon_tags_2 },
    { 0x2550466a, SUC_lexicon_tags_23 },
    { 0xd9d0466c, SUC_lexicon_tags_47 },
    { 0xf570466d, SUC_lexicon_tags_7 },
    { 0xcd50466e, SUC_lexicon_tags_35 },
    { 0x4df0466f, SUC_lexicon_tags_2 },
    { 0xf5104670, SUC_lexicon_tags_7 },
    { 0xc5f04671, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d504674, SUC_lexicon_tags_37 },
    { 0xcdb04675, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b04678, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6c90467a, SUC_lexicon_tags_44 },
    { 0x2730467b, SUC_lexicon_tags_14 },
    { 0x3770467a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39304683, SUC_lexicon_tags_33 },
    { 0xf3304683, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57304688, SUC_lexicon_tags_31 },
    { 0x8f504688, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd4f0468b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0468e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf5d04690, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d04694, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x904696, SUC_lexicon_tags_37 },
    { 0x39304697, SUC_lexicon_tags_12 },
    { 0x6f904697, SUC_lexicon_tags_15 },
    { 0x34d04697, SUC_lexicon_tags_37 },
    { 0x55b0469a, SUC_lexicon_tags_7 },
    { 0x4504698, SUC_lexicon_tags_45 },
    { 0x73904697, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x40d0469e, SUC_lexicon_tags_66 },
    { 0x6990469f, SUC_lexicon_tags_23 },
    { 0x921046a0, SUC_lexicon_tags_30 },
    { 0x6a50469f, SUC_lexicon_tags_138 },
    { 0x9810469f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x10b046a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639046aa, SUC_lexicon_tags_23 },
    { 0xfbb046aa, SUC_lexicon_tags_37 },
    { 0x235046aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8f5046ae, SUC_lexicon_tags_8 },
    { 0xba9046ae, SUC_lexicon_tags_15 },
    { 0xbfb046b0, SUC_lexicon_tags_17 },
    { 0x1ed046b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x837046b3, SUC_lexicon_tags_13 },
    { 0xfb1046b4, SUC_lexicon_tags_11 },
    { 0xc5b046b5, SUC_lexicon_tags_17 },
    { 0xa07046b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x86d046b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7046bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7046be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93046c1, SUC_lexicon_tags_23 },
    { 0x731046c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xec1046c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83046cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f046cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4db046d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95046d8, SUC_lexicon_tags_2 },
    { 0x683046d9, SUC_lexicon_tags_17 },
    { 0x2b7046da, SUC_lexicon_tags_8 },
    { 0x927046db, SUC_lexicon_tags_0 },
    { 0x37046db, SUC_lexicon_tags_12 },
    { 0xbef046da, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa5046df, SUC_lexicon_tags_11 },
    { 0x527046df, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405046e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b7046ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7046f0, SUC_lexicon_tags_27 },
    { 0x715046f0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdab046f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941046f9, SUC_lexicon_tags_3 },
    { 0xe71046fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21046ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94f04701, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x45504704, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd704709, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9990470c, SUC_lexicon_tags_25 },
    { 0x39d0470c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67104711, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7fd04713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff0471b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6ff0471d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b04721, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x32904724, SUC_lexicon_tags_66 },
    { 0x89f04725, SUC_lexicon_tags_11 },
    { 0x5304726, SUC_lexicon_tags_44 },
    { 0x55304726, SUC_lexicon_tags_1 },
    { 0xa7504727, SUC_lexicon_tags_25 },
    { 0x23f04728, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb270472b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d70472d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d04731, SUC_lexicon_tags_87 },
    { 0x9d04732, SUC_lexicon_tags_12 },
    { 0x49704732, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf04736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd04740, SUC_lexicon_tags_11 },
    { 0x6cb04741, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1704744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a504747, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d0474a, SUC_lexicon_tags_12 },
    { 0xb810474b, SUC_lexicon_tags_63 },
    { 0x29b0474c, SUC_lexicon_tags_25 },
    { 0x8370474d, SUC_lexicon_tags_62 },
    { 0xcc10474e, SUC_lexicon_tags_12 },
    { 0xa490474f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4904757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6830475b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd0475e, SUC_lexicon_tags_19 },
    { 0xa570475f, SUC_lexicon_tags_6 },
    { 0x3b50475e, SUC_lexicon_tags_35 },
    { 0xe7b04761, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc704768, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8030476b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b04773, SUC_lexicon_tags_1 },
    { 0x95504774, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1470477c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc450477f, SUC_lexicon_tags_8 },
    { 0xd8904780, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57304782, SUC_lexicon_tags_35 },
    { 0xc0f04783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x950478a, SUC_lexicon_tags_46 },
    { 0xc5d0478b, SUC_lexicon_tags_2 },
    { 0x1a30478b, SUC_lexicon_tags_47 },
    { 0x9150478b, SUC_lexicon_tags_26 },
    { 0x60b0478b, SUC_lexicon_tags_29 },
    { 0xb350478f, SUC_lexicon_tags_21 },
    { 0xf310478d, SUC_lexicon_tags_60 },
    { 0x3cd0478a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x93b04793, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5f04795, SUC_lexicon_tags_25 },
    { 0xc0f04796, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b04799, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa990479f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3047a3, SUC_lexicon_tags_7 },
    { 0x47047a4, SUC_lexicon_tags_43 },
    { 0x2c1047a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b9047a7, SUC_lexicon_tags_0 },
    { 0xb2d047a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9047b0, SUC_lexicon_tags_17 },
    { 0xd73047b1, SUC_lexicon_tags_15 },
    { 0x6ad047b2, SUC_lexicon_tags_12 },
    { 0xc2f047b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab047b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaf047b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13047ba, SUC_lexicon_tags_2 },
    { 0xc6f047ba, SUC_lexicon_tags_17 },
    { 0xc2b047bc, SUC_lexicon_tags_27 },
    { 0xa93047bd, SUC_lexicon_tags_7 },
    { 0x227047be, SUC_lexicon_tags_46 },
    { 0xc6b047bf, SUC_lexicon_tags_2 },
    { 0xf01047be, SUC_lexicon_tags_45 },
    { 0xcb9047c1, SUC_lexicon_tags_37 },
    { 0xdf3047c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17047c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09047c8, SUC_lexicon_tags_37 },
    { 0x54f047c9, SUC_lexicon_tags_47 },
    { 0xb4b047c9, SUC_lexicon_tags_11 },
    { 0x141047cb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x101047cd, SUC_lexicon_tags_7 },
    { 0x9ed047cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x717047d1, SUC_lexicon_tags_12 },
    { 0x139047d2, SUC_lexicon_tags_27 },
    { 0xf7b047d3, SUC_lexicon_tags_14 },
    { 0xb2b047d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x409047d7, SUC_lexicon_tags_30 },
    { 0xe0d047d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9047db, SUC_lexicon_tags_28 },
    { 0x7c1047db, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7047e0, SUC_lexicon_tags_17 },
    { 0x531047e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5047e9, SUC_lexicon_tags_26 },
    { 0x22d047ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe59047ec, SUC_lexicon_tags_105 },
    { 0x143047ed, SUC_lexicon_tags_66 },
    { 0xe53047ee, SUC_lexicon_tags_91 },
    { 0x221047ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46b047f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f047f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23047f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb9d047fb, SUC_lexicon_tags_79 },
    { 0xac9047fc, SUC_lexicon_tags_37 },
    { 0xcf5047fd, SUC_lexicon_tags_87 },
    { 0x9e3047fc, SUC_lexicon_tags_79 },
    { 0xdb5047fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d304801, SUC_lexicon_tags_2 },
    { 0x45304802, SUC_lexicon_tags_17 },
    { 0xbd704803, SUC_lexicon_tags_56 },
    { 0x66f04804, SUC_lexicon_tags_28 },
    { 0x4cb04804, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3150480a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e904810, SUC_lexicon_tags_56 },
    { 0x7504811, SUC_lexicon_tags_12 },
    { 0xecf04810, SUC_lexicon_tags_4 },
    { 0x22104812, SUC_lexicon_tags_2 },
    { 0xa8704814, SUC_lexicon_tags_17 },
    { 0xb1b04815, SUC_lexicon_tags_1 },
    { 0xa1904815, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x39104819, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59704820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b04823, SUC_lexicon_tags_45 },
    { 0x15b04824, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43904826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb04829, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9050482c, SUC_lexicon_tags_3 },
    { 0xd5b0482d, SUC_lexicon_tags_30 },
    { 0x5710482e, SUC_lexicon_tags_235 },
    { 0xbd50482f, SUC_lexicon_tags_21 },
    { 0x3710482d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb04833, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f04838, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8190483b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0483f, SUC_lexicon_tags_13 },
    { 0xab504840, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x97104843, SUC_lexicon_tags_12 },
    { 0xe6904844, SUC_lexicon_tags_47 },
    { 0x62904845, SUC_lexicon_tags_12 },
    { 0x11104844, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30484a, SUC_lexicon_tags_12 },
    { 0xe830484b, SUC_lexicon_tags_17 },
    { 0xd410484c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d04851, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7304856, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4104859, SUC_lexicon_tags_27 },
    { 0x35f0485a, SUC_lexicon_tags_11 },
    { 0xfa90485a, SUC_lexicon_tags_43 },
    { 0xbb0485b, SUC_lexicon_tags_37 },
    { 0x87d0485d, SUC_lexicon_tags_25 },
    { 0xaff0485a, SUC_lexicon_tags_0 },
    { 0x9fb0485b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f04863, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11904866, SUC_lexicon_tags_6 },
    { 0x3bf04866, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x72904869, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3590486b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6a30486d, SUC_lexicon_tags_12 },
    { 0x4f70486e, SUC_lexicon_tags_7 },
    { 0xf330486f, SUC_lexicon_tags_1 },
    { 0x82d0486d, SUC_lexicon_tags_25 },
    { 0x6e30486e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f04877, SUC_lexicon_tags_49 },
    { 0x94104877, SUC_lexicon_tags_26 },
    { 0x68504879, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b0487d, SUC_lexicon_tags_14 },
    { 0x5550487e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35304882, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0104885, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4170488a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e30488c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf104890, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8d904892, SUC_lexicon_tags_37 },
    { 0x33b04893, SUC_lexicon_tags_15 },
    { 0x6f904894, SUC_lexicon_tags_11 },
    { 0x9a904895, SUC_lexicon_tags_70 },
    { 0x48f04895, SUC_lexicon_tags_12 },
    { 0x13504897, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e904899, SUC_lexicon_tags_25 },
    { 0x1b104899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x990489c, SUC_lexicon_tags_2 },
    { 0x9eb0489d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6390489f, SUC_lexicon_tags_11 },
    { 0x59048a0, SUC_lexicon_tags_6 },
    { 0xb130489f, SUC_lexicon_tags_29 },
    { 0xeff048a2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x589048a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf048ab, SUC_lexicon_tags_7 },
    { 0x625048ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949048b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313048b9, SUC_lexicon_tags_47 },
    { 0x2f7048b9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7cb048bc, SUC_lexicon_tags_11 },
    { 0x21b048bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1048c4, SUC_lexicon_tags_46 },
    { 0xedb048c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f048cb, SUC_lexicon_tags_2 },
    { 0x31048cb, SUC_lexicon_tags_10 },
    { 0x4b9048cd, SUC_lexicon_tags_63 },
    { 0x3e7048cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d048d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b048d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5048d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b048db, SUC_lexicon_tags_1 },
    { 0xa95048dc, SUC_lexicon_tags_45 },
    { 0x8a5048dc, SUC_lexicon_tags_30 },
    { 0x739048dd, SUC_lexicon_tags_4 },
    { 0xb69048db, SUC_lexicon_tags_25 },
    { 0x5f3048e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1048e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65048e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69048ec, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df048f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb57048f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133048fb, SUC_lexicon_tags_1 },
    { 0xd97048fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x811048ff, SUC_lexicon_tags_45 },
    { 0x12104900, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16904902, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b04905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a904909, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0xe5b0490b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37904910, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f704912, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe504914, SUC_lexicon_tags_25 },
    { 0xb3b04915, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeb504917, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f0491a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d04922, SUC_lexicon_tags_17 },
    { 0xe1104923, SUC_lexicon_tags_31 },
    { 0x3f704923, SUC_lexicon_tags_14 },
    { 0x32704922, SUC_lexicon_tags_23 },
    { 0xff304923, SUC_lexicon_tags_0 },
    { 0xf9104927, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c70492c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82704930, SUC_lexicon_tags_1 },
    { 0x7a304931, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0304933, SUC_lexicon_tags_164 },
    { 0, NULL },
    { 0xcc304935, SUC_lexicon_tags_1 },
    { 0x27f04936, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0904939, SUC_lexicon_tags_50 },
    { 0x8af0493a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeeb0493c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d30493f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce104945, SUC_lexicon_tags_26 },
    { 0xafb04946, SUC_lexicon_tags_0 },
    { 0x24b04947, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe810494a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdb0494c, SUC_lexicon_tags_25 },
    { 0xa9f0494d, SUC_lexicon_tags_60 },
    { 0xd0f0494e, SUC_lexicon_tags_47 },
    { 0xd4f0494f, SUC_lexicon_tags_12 },
    { 0x6e50494e, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0xff104952, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0904958, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe410495e, SUC_lexicon_tags_7 },
    { 0xc990495f, SUC_lexicon_tags_11 },
    { 0xa210495e, SUC_lexicon_tags_7 },
    { 0xbb704961, SUC_lexicon_tags_2 },
    { 0xd4f04962, SUC_lexicon_tags_14 },
    { 0xeeb04962, SUC_lexicon_tags_30 },
    { 0x7d04964, SUC_lexicon_tags_23 },
    { 0xf3504961, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6104967, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6b04969, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8030496f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff04973, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb904975, SUC_lexicon_tags_3 },
    { 0x13704976, SUC_lexicon_tags_25 },
    { 0x97504977, SUC_lexicon_tags_1 },
    { 0x3c304978, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f10497c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdc50497e, SUC_lexicon_tags_14 },
    { 0x2c10497f, SUC_lexicon_tags_19 },
    { 0xf7104980, SUC_lexicon_tags_37 },
    { 0xe1d0497e, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd04984, SUC_lexicon_tags_17 },
    { 0x2ed04985, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d704987, SUC_lexicon_tags_6 },
    { 0x89904988, SUC_lexicon_tags_32 },
    { 0xdf304988, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97504991, SUC_lexicon_tags_15 },
    { 0x5e304991, SUC_lexicon_tags_11 },
    { 0x8d704992, SUC_lexicon_tags_10 },
    { 0x6bd04993, SUC_lexicon_tags_49 },
    { 0xf104991, SUC_lexicon_tags_31 },
    { 0x6b704996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f04999, SUC_lexicon_tags_6 },
    { 0xd570499a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32d0499c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c30499f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915049a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99049a6, SUC_lexicon_tags_19 },
    { 0x16b049a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41049ad, SUC_lexicon_tags_15 },
    { 0xbb5049ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1049b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x471049b4, SUC_lexicon_tags_7 },
    { 0x14b049b5, SUC_lexicon_tags_65 },
    { 0xf9049b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85049b9, SUC_lexicon_tags_3 },
    { 0xcf1049b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x283049bd, SUC_lexicon_tags_4 },
    { 0xf89049be, SUC_lexicon_tags_87 },
    { 0x37d049be, SUC_lexicon_tags_23 },
    { 0x3d1049c0, SUC_lexicon_tags_37 },
    { 0x54f049c0, SUC_lexicon_tags_11 },
    { 0x805049c2, SUC_lexicon_tags_170 },
    { 0x669049c2, SUC_lexicon_tags_11 },
    { 0x109049c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7049c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad5049c9, SUC_lexicon_tags_68 },
    { 0x6fb049ca, SUC_lexicon_tags_17 },
    { 0x49f049cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9049ce, SUC_lexicon_tags_15 },
    { 0x161049cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x105049d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc33049d4, SUC_lexicon_tags_57 },
    { 0x4ef049d4, SUC_lexicon_tags_12 },
    { 0xd5d049d6, SUC_lexicon_tags_17 },
    { 0x867049d7, SUC_lexicon_tags_0 },
    { 0xbd7049d4, SUC_lexicon_tags_48 },
    { 0x2eb049d9, SUC_lexicon_tags_25 },
    { 0x9b7049da, SUC_lexicon_tags_8 },
    { 0xa49049d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b049df, SUC_lexicon_tags_68 },
    { 0x8dd049e0, SUC_lexicon_tags_7 },
    { 0x549049df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3049e4, SUC_lexicon_tags_17 },
    { 0xef9049e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383049ea, SUC_lexicon_tags_6 },
    { 0x289049eb, SUC_lexicon_tags_31 },
    { 0x65d049eb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7049f2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x779049f5, SUC_lexicon_tags_17 },
    { 0xb63049f6, SUC_lexicon_tags_7 },
    { 0x9a5049f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62b049f9, SUC_lexicon_tags_7 },
    { 0x129049f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd049fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d049ff, SUC_lexicon_tags_47 },
    { 0xa1504a00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef04a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8304a08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c904a0b, SUC_lexicon_tags_2 },
    { 0xf2504a0c, SUC_lexicon_tags_70 },
    { 0x61904a0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40b04a0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b04a12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcbd04a14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b04a1f, SUC_lexicon_tags_14 },
    { 0x49704a20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b04a23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf504a28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad04a2b, SUC_lexicon_tags_2 },
    { 0x5a904a2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f04a30, SUC_lexicon_tags_160 },
    { 0x40104a31, SUC_lexicon_tags_11 },
    { 0x3cd04a30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48b04a34, SUC_lexicon_tags_12 },
    { 0x21104a35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a304a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf04a3c, SUC_lexicon_tags_14 },
    { 0xb4504a3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b04a41, SUC_lexicon_tags_17 },
    { 0x2c704a42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e304a46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8304a48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c904a4b, SUC_lexicon_tags_57 },
    { 0x96104a4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf304a4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d04a54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd04a57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f04a5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d04a60, SUC_lexicon_tags_10 },
    { 0xbfb04a60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f04a64, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6b304a66, SUC_lexicon_tags_0 },
    { 0xf8304a66, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f04a6a, SUC_lexicon_tags_27 },
    { 0xfb704a6a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9104a77, SUC_lexicon_tags_12 },
    { 0xf504a78, SUC_lexicon_tags_19 },
    { 0x19f04a78, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x27504a7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7c304a7e, SUC_lexicon_tags_17 },
    { 0xff504a7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e704a84, SUC_lexicon_tags_1 },
    { 0x69104a85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe304a8d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d304a90, SUC_lexicon_tags_42 },
    { 0x5a304a91, SUC_lexicon_tags_87 },
    { 0x50104a92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1704a96, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b04a9a, SUC_lexicon_tags_7 },
    { 0xcc304a9b, SUC_lexicon_tags_25 },
    { 0x81d04a9b, SUC_lexicon_tags_11 },
    { 0x4bd04a9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56304aa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59704aa3, SUC_lexicon_tags_100 },
    { 0x54d04aa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4304aab, SUC_lexicon_tags_9 },
    { 0x2104aac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f904aae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3ed04ab0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b04ab7, SUC_lexicon_tags_43 },
    { 0x30904ab8, SUC_lexicon_tags_47 },
    { 0xdff04ab9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3bd04abb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x83704abd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x24304ac0, SUC_lexicon_tags_28 },
    { 0x77704ac1, SUC_lexicon_tags_17 },
    { 0xff04ac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d04ac5, SUC_lexicon_tags_17 },
    { 0x16b04ac6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5304ac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8eb04aca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdef04acc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6704ad2, SUC_lexicon_tags_68 },
    { 0x5a304ad2, SUC_lexicon_tags_65 },
    { 0xe5d04ad3, SUC_lexicon_tags_17 },
    { 0xe1504ad3, SUC_lexicon_tags_100 },
    { 0xbbb04ad6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0104ad9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x47104adb, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b04ade, SUC_lexicon_tags_47 },
    { 0x62304adf, SUC_lexicon_tags_37 },
    { 0xa1f04ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bb04ae2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x21704ae5, SUC_lexicon_tags_37 },
    { 0x2c304ae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa504aea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1b04aec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf504af0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d04af4, SUC_lexicon_tags_11 },
    { 0x9eb04af4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f04afb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6cb04afd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2904b02, SUC_lexicon_tags_21 },
    { 0xed04b03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15904b0c, SUC_lexicon_tags_21 },
    { 0x98904b0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d04b10, SUC_lexicon_tags_11 },
    { 0x6bf04b11, SUC_lexicon_tags_1 },
    { 0x4b304b12, SUC_lexicon_tags_14 },
    { 0xc4f04b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f04b17, SUC_lexicon_tags_1 },
    { 0x8cb04b17, SUC_lexicon_tags_63 },
    { 0x92104b17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b04b1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24704b22, SUC_lexicon_tags_31 },
    { 0x2d304b23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f04b26, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa504b2b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d04b30, SUC_lexicon_tags_108 },
    { 0xc4d04b30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f504b38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29904b3a, SUC_lexicon_tags_65 },
    { 0x11304b3b, SUC_lexicon_tags_15 },
    { 0xe304b3c, SUC_lexicon_tags_82 },
    { 0x39904b3d, SUC_lexicon_tags_32 },
    { 0x3904b3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6f304b40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f904b43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5304b45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd04b48, SUC_lexicon_tags_28 },
    { 0xa4504b48, SUC_lexicon_tags_4 },
    { 0x37704b49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4504b4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xff104b52, SUC_lexicon_tags_12 },
    { 0xf9704b52, SUC_lexicon_tags_9 },
    { 0x5d304b53, SUC_lexicon_tags_15 },
    { 0xd2504b55, SUC_lexicon_tags_25 },
    { 0x12904b54, SUC_lexicon_tags_4 },
    { 0x44d04b53, SUC_lexicon_tags_17 },
    { 0x1f504b56, SUC_lexicon_tags_107 },
    { 0xcf904b59, SUC_lexicon_tags_38 },
    { 0xa5b04b52, SUC_lexicon_tags_12 },
    { 0x87104b58, SUC_lexicon_tags_45 },
    { 0xeb904b5c, SUC_lexicon_tags_32 },
    { 0xc9704b5d, SUC_lexicon_tags_44 },
    { 0x58104b55, SUC_lexicon_tags_2 },
    { 0x4ff04b5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36504b64, SUC_lexicon_tags_37 },
    { 0x79104b65, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x83304b67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x70304b69, SUC_lexicon_tags_43 },
    { 0x93904b6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1504b72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x77904b74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x16504b77, SUC_lexicon_tags_37 },
    { 0x3e704b78, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf7d04b7a, SUC_lexicon_tags_12 },
    { 0xd7904b7b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4104b82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f104b88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73d04b8a, SUC_lexicon_tags_16 },
    { 0xfdd04b8b, SUC_lexicon_tags_37 },
    { 0x2cf04b8c, SUC_lexicon_tags_9 },
    { 0x19504b8c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f704b90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x39504b92, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdc104b94, SUC_lexicon_tags_11 },
    { 0xcd04b94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25504b99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d504b9c, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0xa4304b9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x30d04ba0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ed04ba2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b04ba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55f04ba6, SUC_lexicon_tags_19 },
    { 0x8eb04ba7, SUC_lexicon_tags_17 },
    { 0x1a704ba8, SUC_lexicon_tags_600 },
    { 0x6ff04ba9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d904bb3, SUC_lexicon_tags_1 },
    { 0xdcb04bb3, SUC_lexicon_tags_47 },
    { 0x7ed04bb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd904bb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef104bbd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xff304bbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67904bc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a104bc5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x99304bc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45304bd2, SUC_lexicon_tags_26 },
    { 0x82504bd3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7d904bd5, SUC_lexicon_tags_23 },
    { 0xd1304bd6, SUC_lexicon_tags_1 },
    { 0xdcb04bd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b04bda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45904bdc, SUC_lexicon_tags_32 },
    { 0x27504bdc, SUC_lexicon_tags_38 },
    { 0x79d04bde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x12304be0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdcf04be2, SUC_lexicon_tags_36 },
    { 0x2c504be3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6704be8, SUC_lexicon_tags_14 },
    { 0xa2904be9, SUC_lexicon_tags_27 },
    { 0x8bb04bea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x75504bed, SUC_lexicon_tags_7 },
    { 0x73904bed, SUC_lexicon_tags_4 },
    { 0x9704bef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf04bf2, SUC_lexicon_tags_9 },
    { 0xb7104bf2, SUC_lexicon_tags_8 },
    { 0xa9104bf3, SUC_lexicon_tags_11 },
    { 0x20f04bf4, SUC_lexicon_tags_37 },
    { 0x2f504bf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f04bf9, SUC_lexicon_tags_26 },
    { 0xad04bfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7104bfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfd04bff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37d04c01, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d04c04, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe2b04c06, SUC_lexicon_tags_87 },
    { 0x51904c06, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x58504c09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b04c11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3504c14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb904c17, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8b704c19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55704c22, SUC_lexicon_tags_27 },
    { 0x38504c22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f04c2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39104c33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3704c35, SUC_lexicon_tags_1 },
    { 0xf304c36, SUC_lexicon_tags_21 },
    { 0x21104c37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18304c39, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x31304c3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d04c3e, SUC_lexicon_tags_7 },
    { 0x3e904c3e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd04c42, SUC_lexicon_tags_37 },
    { 0x80b04c42, SUC_lexicon_tags_13 },
    { 0x1d904c44, SUC_lexicon_tags_11 },
    { 0x20104c44, SUC_lexicon_tags_26 },
    { 0x18304c44, SUC_lexicon_tags_38 },
    { 0x6ed04c46, SUC_lexicon_tags_7 },
    { 0x62704c48, SUC_lexicon_tags_11 },
    { 0x8ff04c49, SUC_lexicon_tags_17 },
    { 0x76f04c48, SUC_lexicon_tags_40 },
    { 0xd0d04c4a, SUC_lexicon_tags_25 },
    { 0x2af04c49, SUC_lexicon_tags_26 },
    { 0x19704c4d, SUC_lexicon_tags_1 },
    { 0xe8504c4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfb704c50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd504c53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a504c58, SUC_lexicon_tags_29 },
    { 0x37304c59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63304c5e, SUC_lexicon_tags_10 },
    { 0xeaf04c5f, SUC_lexicon_tags_17 },
    { 0x97504c60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b04c64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b04c68, SUC_lexicon_tags_2 },
    { 0x87d04c68, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f904c6f, SUC_lexicon_tags_10 },
    { 0xe8104c6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8904c73, SUC_lexicon_tags_25 },
    { 0x4f104c73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x77f04c76, SUC_lexicon_tags_17 },
    { 0x6ab04c76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa304c79, SUC_lexicon_tags_5 },
    { 0x80504c7a, SUC_lexicon_tags_47 },
    { 0x7ed04c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58504c7f, SUC_lexicon_tags_76 },
    { 0xea504c7f, SUC_lexicon_tags_37 },
    { 0x96904c81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42304c83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2504c8c, SUC_lexicon_tags_27 },
    { 0x7d04c8c, SUC_lexicon_tags_28 },
    { 0xb5904c8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa104c92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b04c94, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2dd04c96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f904c9d, SUC_lexicon_tags_38 },
    { 0x2df04c9e, SUC_lexicon_tags_6 },
    { 0x6f704c9f, SUC_lexicon_tags_15 },
    { 0xc1b04ca0, SUC_lexicon_tags_80 },
    { 0xa0304ca1, SUC_lexicon_tags_37 },
    { 0xac104ca2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x20904ca4, SUC_lexicon_tags_11 },
    { 0xcb504ca5, SUC_lexicon_tags_25 },
    { 0xfe504ca6, SUC_lexicon_tags_7 },
    { 0xcb04ca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9104cac, SUC_lexicon_tags_25 },
    { 0x7c904cad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d04cb9, SUC_lexicon_tags_63 },
    { 0xf4904cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51104cbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83f04cbe, SUC_lexicon_tags_33 },
    { 0xf8904cbf, SUC_lexicon_tags_19 },
    { 0xbd104cbf, SUC_lexicon_tags_11 },
    { 0x54904cc1, SUC_lexicon_tags_33 },
    { 0xc1b04cc2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d04ccd, SUC_lexicon_tags_37 },
    { 0xfa904ccd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b704cd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76904cd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a704cdb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf7f04cdd, SUC_lexicon_tags_14 },
    { 0x7b504cde, SUC_lexicon_tags_1 },
    { 0x35d04cde, SUC_lexicon_tags_43 },
    { 0xde104cdf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31304ce6, SUC_lexicon_tags_2 },
    { 0x51904ce7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b904ceb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df04cf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f104cf6, SUC_lexicon_tags_43 },
    { 0x81104cf6, SUC_lexicon_tags_56 },
    { 0x79904cf8, SUC_lexicon_tags_11 },
    { 0x1c904cf9, SUC_lexicon_tags_2 },
    { 0x65704cfa, SUC_lexicon_tags_27 },
    { 0x23d04cfb, SUC_lexicon_tags_25 },
    { 0x12f04cfc, SUC_lexicon_tags_8 },
    { 0xe3d04cfd, SUC_lexicon_tags_0 },
    { 0xda904cfd, SUC_lexicon_tags_12 },
    { 0x60504cf7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58104d07, SUC_lexicon_tags_14 },
    { 0x68104d08, SUC_lexicon_tags_38 },
    { 0xd5d04d08, SUC_lexicon_tags_25 },
    { 0xc1904d0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f04d0e, SUC_lexicon_tags_114 },
    { 0x3fb04d0e, SUC_lexicon_tags_10 },
    { 0x38904d10, SUC_lexicon_tags_6 },
    { 0xc0904d10, SUC_lexicon_tags_19 },
    { 0x48d04d12, SUC_lexicon_tags_10 },
    { 0x33104d13, SUC_lexicon_tags_11 },
    { 0x78f04d14, SUC_lexicon_tags_15 },
    { 0x43904d14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c704d17, SUC_lexicon_tags_38 },
    { 0xd6d04d18, SUC_lexicon_tags_21 },
    { 0x4e904d19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90104d1d, SUC_lexicon_tags_46 },
    { 0xda904d1e, SUC_lexicon_tags_951 },
    { 0x7ab04d1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f04d22, SUC_lexicon_tags_38 },
    { 0xfaf04d22, SUC_lexicon_tags_82 },
    { 0x9d904d24, SUC_lexicon_tags_15 },
    { 0xdcb04d25, SUC_lexicon_tags_25 },
    { 0xa5304d26, SUC_lexicon_tags_19 },
    { 0x43304d26, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad304d2b, SUC_lexicon_tags_350 },
    { 0x14f04d2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1904d2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f04d33, SUC_lexicon_tags_32 },
    { 0x96704d34, SUC_lexicon_tags_38 },
    { 0x11f04d33, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x78d04d37, SUC_lexicon_tags_31 },
    { 0xfa904d37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7704d3b, SUC_lexicon_tags_2 },
    { 0x8a704d3b, SUC_lexicon_tags_47 },
    { 0x4c904d3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x91104d3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5704d42, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x1c104d44, SUC_lexicon_tags_23 },
    { 0x8ef04d45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9dd04d47, SUC_lexicon_tags_9 },
    { 0x4a904d48, SUC_lexicon_tags_60 },
    { 0x74704d47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3904d4d, SUC_lexicon_tags_83 },
    { 0xb7704d4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x88304d50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa304d55, SUC_lexicon_tags_25 },
    { 0xaf904d55, SUC_lexicon_tags_61 },
    { 0xbf704d57, SUC_lexicon_tags_0 },
    { 0xea904d58, SUC_lexicon_tags_12 },
    { 0x3bb04d57, SUC_lexicon_tags_9 },
    { 0xc7304d5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa6104d5c, SUC_lexicon_tags_17 },
    { 0xfb304d5d, SUC_lexicon_tags_14 },
    { 0xc3f04d5e, SUC_lexicon_tags_27 },
    { 0xb9f04d5f, SUC_lexicon_tags_38 },
    { 0x86304d5e, SUC_lexicon_tags_11 },
    { 0x6a304d61, SUC_lexicon_tags_25 },
    { 0x25304d62, SUC_lexicon_tags_68 },
    { 0x68b04d63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8304d66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93704d6b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3a504d6d, SUC_lexicon_tags_1 },
    { 0xe8f04d6e, SUC_lexicon_tags_33 },
    { 0xedb04d6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf504d71, SUC_lexicon_tags_39 },
    { 0xd2b04d72, SUC_lexicon_tags_37 },
    { 0x99b04d73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2704d77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce304d7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b04d81, SUC_lexicon_tags_23 },
    { 0xbff04d81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c904d86, SUC_lexicon_tags_15 },
    { 0x2b904d87, SUC_lexicon_tags_12 },
    { 0xfa104d87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d304d8b, SUC_lexicon_tags_27 },
    { 0xd3704d8c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb04d90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7904d93, SUC_lexicon_tags_15 },
    { 0x96f04d94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cf04d96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c504d99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f904d9b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6fb04d9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b04da1, SUC_lexicon_tags_62 },
    { 0x6a104da1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52904da5, SUC_lexicon_tags_37 },
    { 0x4a904da6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d04da9, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25304dae, SUC_lexicon_tags_1 },
    { 0xb9d04dae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46104db5, SUC_lexicon_tags_25 },
    { 0x4d904db6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d04dba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0904dbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x33904dbf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74504dc3, SUC_lexicon_tags_15 },
    { 0x4d304dc4, SUC_lexicon_tags_60 },
    { 0x20704dc3, SUC_lexicon_tags_60 },
    { 0xb5f04dc6, SUC_lexicon_tags_516 },
    { 0xf4b04dc7, SUC_lexicon_tags_17 },
    { 0xae304dc7, SUC_lexicon_tags_17 },
    { 0xb104dc6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x86704dcb, SUC_lexicon_tags_66 },
    { 0xbcd04dcc, SUC_lexicon_tags_38 },
    { 0xb4904dcc, SUC_lexicon_tags_23 },
    { 0xfcd04dcc, SUC_lexicon_tags_37 },
    { 0x8a704dce, SUC_lexicon_tags_31 },
    { 0xd9304dd0, SUC_lexicon_tags_7 },
    { 0x48904dcb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d04dd4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54504dda, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab504de0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8104de4, SUC_lexicon_tags_17 },
    { 0x7b504de5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd704deb, SUC_lexicon_tags_19 },
    { 0xaf904dec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d04df2, SUC_lexicon_tags_17 },
    { 0xbb104df2, SUC_lexicon_tags_82 },
    { 0x4ed04df2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f104df9, SUC_lexicon_tags_1 },
    { 0x74904df9, SUC_lexicon_tags_5 },
    { 0x23904df9, SUC_lexicon_tags_61 },
    { 0x1d04dfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5f04dfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d304e07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x86b04e09, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2104e0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd04e11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3504e14, SUC_lexicon_tags_514 },
    { 0xaf704e15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52104e18, SUC_lexicon_tags_15 },
    { 0xd8104e18, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xded04e1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d04e22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x55904e25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df04e2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x18304e2d, SUC_lexicon_tags_10 },
    { 0xf6f04e2e, SUC_lexicon_tags_38 },
    { 0xa7d04e2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b304e31, SUC_lexicon_tags_2 },
    { 0x62504e32, SUC_lexicon_tags_29 },
    { 0x5cd04e33, SUC_lexicon_tags_1 },
    { 0x13504e32, SUC_lexicon_tags_673 },
    { 0x1eb04e33, SUC_lexicon_tags_15 },
    { 0x25304e36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab04e39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e504e3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef304e3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf04e43, SUC_lexicon_tags_33 },
    { 0x89504e44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21504e4d, SUC_lexicon_tags_45 },
    { 0x4a704e4e, SUC_lexicon_tags_39 },
    { 0x7ad04e4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8704e52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf904e57, SUC_lexicon_tags_38 },
    { 0x21d04e58, SUC_lexicon_tags_25 },
    { 0xca504e58, SUC_lexicon_tags_12 },
    { 0xd7f04e58, SUC_lexicon_tags_68 },
    { 0xb7d04e5b, SUC_lexicon_tags_49 },
    { 0x2ab04e5c, SUC_lexicon_tags_37 },
    { 0x92704e58, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x94904e60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45304e64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2304e67, SUC_lexicon_tags_65 },
    { 0x904e68, SUC_lexicon_tags_7 },
    { 0xd2d04e69, SUC_lexicon_tags_11 },
    { 0xf1304e6a, SUC_lexicon_tags_14 },
    { 0x71704e6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72104e6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7904e70, SUC_lexicon_tags_15 },
    { 0xbbd04e71, SUC_lexicon_tags_49 },
    { 0xd3304e71, SUC_lexicon_tags_16 },
    { 0xe7704e73, SUC_lexicon_tags_6 },
    { 0x35f04e74, SUC_lexicon_tags_25 },
    { 0x1c704e71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f04e78, SUC_lexicon_tags_42 },
    { 0x6c104e78, SUC_lexicon_tags_39 },
    { 0x1c904e7a, SUC_lexicon_tags_31 },
    { 0x17904e7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3104e7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d04e80, SUC_lexicon_tags_14 },
    { 0xe4704e81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7104e89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd04e8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b04e93, SUC_lexicon_tags_11 },
    { 0x2304e94, SUC_lexicon_tags_1 },
    { 0x91104e93, SUC_lexicon_tags_11 },
    { 0x8db04e96, SUC_lexicon_tags_19 },
    { 0x4e304e96, SUC_lexicon_tags_14 },
    { 0x97104e98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x69d04e9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4304e9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd04ea1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a104ea5, SUC_lexicon_tags_25 },
    { 0x61504ea6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1d04ea8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f04eac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0904eaf, SUC_lexicon_tags_12 },
    { 0xe7704eaf, SUC_lexicon_tags_2 },
    { 0x1d704eb1, SUC_lexicon_tags_2 },
    { 0xe9504eb0, SUC_lexicon_tags_30 },
    { 0x60904eb3, SUC_lexicon_tags_8 },
    { 0x27304eb4, SUC_lexicon_tags_0 },
    { 0x7a504eb1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe3b04eb7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf4304eb9, SUC_lexicon_tags_47 },
    { 0x62304eba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5104ebf, SUC_lexicon_tags_1 },
    { 0x8304ebf, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xba704ec2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f704ec6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b04eca, SUC_lexicon_tags_68 },
    { 0x4cb04ecb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8bb04ecd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17504ecf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x77904ed2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39304ed5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0304ed9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31504ede, SUC_lexicon_tags_38 },
    { 0x8b104edf, SUC_lexicon_tags_0 },
    { 0xcdd04ee0, SUC_lexicon_tags_42 },
    { 0xa2104ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e104ee7, SUC_lexicon_tags_30 },
    { 0x3b304ee8, SUC_lexicon_tags_60 },
    { 0xdef04ee8, SUC_lexicon_tags_13 },
    { 0x36d04ee7, SUC_lexicon_tags_25 },
    { 0xeeb04eeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87104eed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed904ef6, SUC_lexicon_tags_17 },
    { 0xc1104ef6, SUC_lexicon_tags_12 },
    { 0x7f304ef6, SUC_lexicon_tags_25 },
    { 0x7704ef9, SUC_lexicon_tags_12 },
    { 0x39b04ef9, SUC_lexicon_tags_30 },
    { 0x90904ef9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d04f00, SUC_lexicon_tags_15 },
    { 0x2b304f01, SUC_lexicon_tags_0 },
    { 0xa4304f02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8504f05, SUC_lexicon_tags_13 },
    { 0x27f04f06, SUC_lexicon_tags_8 },
    { 0xe8704f07, SUC_lexicon_tags_12 },
    { 0xe1104f05, SUC_lexicon_tags_87 },
    { 0x8a704f05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19104f0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39304f12, SUC_lexicon_tags_36 },
    { 0xb6104f13, SUC_lexicon_tags_12 },
    { 0x3e704f13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24104f19, SUC_lexicon_tags_27 },
    { 0x33304f1a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcc504f1c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b04f20, SUC_lexicon_tags_14 },
    { 0xf4504f21, SUC_lexicon_tags_7 },
    { 0x59504f21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d304f24, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x35904f26, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5e104f28, SUC_lexicon_tags_14 },
    { 0xbc304f29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0704f2b, SUC_lexicon_tags_11 },
    { 0x56704f2c, SUC_lexicon_tags_13 },
    { 0xbc704f2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0704f30, SUC_lexicon_tags_37 },
    { 0x7a304f31, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x18704f33, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f04f42, SUC_lexicon_tags_25 },
    { 0x3bf04f43, SUC_lexicon_tags_7 },
    { 0x20104f44, SUC_lexicon_tags_30 },
    { 0xb6f04f45, SUC_lexicon_tags_14 },
    { 0xf7104f46, SUC_lexicon_tags_29 },
    { 0xb4704f46, SUC_lexicon_tags_16 },
    { 0xc6304f45, SUC_lexicon_tags_8 },
    { 0xef04f47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c104f4b, SUC_lexicon_tags_66 },
    { 0x8f904f4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af04f4f, SUC_lexicon_tags_65 },
    { 0x48504f50, SUC_lexicon_tags_19 },
    { 0x6ef04f50, SUC_lexicon_tags_9 },
    { 0xc6b04f52, SUC_lexicon_tags_37 },
    { 0x3d04f51, SUC_lexicon_tags_250 },
    { 0xc1304f54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea504f5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8904f61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a904f63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59904f65, SUC_lexicon_tags_38 },
    { 0xffd04f66, SUC_lexicon_tags_1 },
    { 0x11104f67, SUC_lexicon_tags_11 },
    { 0xe0704f67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4304f6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6704f70, SUC_lexicon_tags_2 },
    { 0x48104f70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd04f74, SUC_lexicon_tags_7 },
    { 0x48104f75, SUC_lexicon_tags_8 },
    { 0x5f04f75, SUC_lexicon_tags_49 },
    { 0x13304f77, SUC_lexicon_tags_14 },
    { 0x34304f76, SUC_lexicon_tags_13 },
    { 0x33704f79, SUC_lexicon_tags_17 },
    { 0x66304f79, SUC_lexicon_tags_0 },
    { 0x52d04f7b, SUC_lexicon_tags_6 },
    { 0x15704f7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf504f7f, SUC_lexicon_tags_47 },
    { 0x18904f7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x46d04f82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85104f84, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x44104f87, SUC_lexicon_tags_17 },
    { 0xf5304f88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bf04f8a, SUC_lexicon_tags_3 },
    { 0xcdb04f8b, SUC_lexicon_tags_11 },
    { 0xcdf04f8a, SUC_lexicon_tags_72 },
    { 0xcdb04f8d, SUC_lexicon_tags_25 },
    { 0x8b704f8e, SUC_lexicon_tags_15 },
    { 0x9bb04f8e, SUC_lexicon_tags_0 },
    { 0xbb704f90, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8304f93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85504f99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81b04f9b, SUC_lexicon_tags_37 },
    { 0x7e704f9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa2f04f9e, SUC_lexicon_tags_46 },
    { 0x36d04f9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8904fa1, SUC_lexicon_tags_80 },
    { 0x6cf04fa1, SUC_lexicon_tags_38 },
    { 0xea104fa3, SUC_lexicon_tags_19 },
    { 0xc6704fa4, SUC_lexicon_tags_43 },
    { 0x4d504fa5, SUC_lexicon_tags_27 },
    { 0x51b04fa5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3dd04fa8, SUC_lexicon_tags_12 },
    { 0x59104fa9, SUC_lexicon_tags_11 },
    { 0x7a304faa, SUC_lexicon_tags_43 },
    { 0xca704fab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe7304fad, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x17f04faf, SUC_lexicon_tags_2 },
    { 0xf1d04fb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3504fb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13704fb5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c304fb8, SUC_lexicon_tags_25 },
    { 0x69504fb9, SUC_lexicon_tags_37 },
    { 0x6e704fb8, SUC_lexicon_tags_15 },
    { 0x90304fbb, SUC_lexicon_tags_14 },
    { 0xab104fbb, SUC_lexicon_tags_63 },
    { 0xdcd04fb9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x42d04fbf, SUC_lexicon_tags_8 },
    { 0x66f04fc0, SUC_lexicon_tags_25 },
    { 0xf9d04fc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b04fc6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa6f04fc8, SUC_lexicon_tags_61 },
    { 0x2e904fc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6f04fcb, SUC_lexicon_tags_62 },
    { 0x56104fcc, SUC_lexicon_tags_91 },
    { 0x61104fcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7104fd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6304fd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b04fdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c904fe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0d04fea, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x86304fec, SUC_lexicon_tags_12 },
    { 0x23104fed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a304ff2, SUC_lexicon_tags_7 },
    { 0x9b304ff3, SUC_lexicon_tags_25 },
    { 0x78904ff2, SUC_lexicon_tags_25 },
    { 0x30104ff5, SUC_lexicon_tags_29 },
    { 0xa4104ff6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf04ffa, SUC_lexicon_tags_33 },
    { 0x2bd04ffa, SUC_lexicon_tags_228 },
    { 0x97904ffc, SUC_lexicon_tags_7 },
    { 0xee704ffd, SUC_lexicon_tags_2 },
    { 0x93b04ffe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4905003, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2305005, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2270500b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2305010, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa8505012, SUC_lexicon_tags_23 },
    { 0xbe105012, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8910501a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8570501c, SUC_lexicon_tags_38 },
    { 0x1dd0501d, SUC_lexicon_tags_2 },
    { 0xb190501c, SUC_lexicon_tags_42 },
    { 0x330501d, SUC_lexicon_tags_37 },
    { 0x34505020, SUC_lexicon_tags_46 },
    { 0xff30501e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xbf105023, SUC_lexicon_tags_10 },
    { 0x2cb05024, SUC_lexicon_tags_47 },
    { 0xdd105025, SUC_lexicon_tags_9 },
    { 0xefd05026, SUC_lexicon_tags_12 },
    { 0x4f905026, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2905029, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b0502c, SUC_lexicon_tags_68 },
    { 0xceb0502c, SUC_lexicon_tags_45 },
    { 0x7cd0502e, SUC_lexicon_tags_11 },
    { 0xf1d0502f, SUC_lexicon_tags_1 },
    { 0x5910502c, SUC_lexicon_tags_6 },
    { 0x6010502c, SUC_lexicon_tags_27 },
    { 0xdaf05032, SUC_lexicon_tags_1 },
    { 0xe050502d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f30503a, SUC_lexicon_tags_47 },
    { 0x6b50503a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb990503d, SUC_lexicon_tags_7 },
    { 0xd410503e, SUC_lexicon_tags_1 },
    { 0xb330503d, SUC_lexicon_tags_0 },
    { 0x97505040, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xa2d05042, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xde705045, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e30504c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b0504e, SUC_lexicon_tags_26 },
    { 0x34f0504f, SUC_lexicon_tags_49 },
    { 0xda505050, SUC_lexicon_tags_57 },
    { 0x83105051, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f05054, SUC_lexicon_tags_14 },
    { 0x5fd05055, SUC_lexicon_tags_6 },
    { 0x2c905055, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1905058, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f10505a, SUC_lexicon_tags_15 },
    { 0x7250505b, SUC_lexicon_tags_1 },
    { 0x71f0505a, SUC_lexicon_tags_136 },
    { 0xd830505d, SUC_lexicon_tags_2 },
    { 0xe770505e, SUC_lexicon_tags_37 },
    { 0x2bb0505e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd05064, SUC_lexicon_tags_70 },
    { 0x93d05065, SUC_lexicon_tags_6 },
    { 0xcbb05064, SUC_lexicon_tags_3 },
    { 0x76b05066, SUC_lexicon_tags_108 },
    { 0xdf505064, SUC_lexicon_tags_37 },
    { 0xdd105069, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf50506c, SUC_lexicon_tags_12 },
    { 0xab10506c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc9d0506f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4e105071, SUC_lexicon_tags_1 },
    { 0x5a305071, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60305075, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27f05077, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65f05079, SUC_lexicon_tags_118 },
    { 0x2db05079, SUC_lexicon_tags_56 },
    { 0xa0d0507b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5070507d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1305080, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf05088, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7650508a, SUC_lexicon_tags_28 },
    { 0x4170508b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb690508d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b05093, SUC_lexicon_tags_11 },
    { 0x69505094, SUC_lexicon_tags_83 },
    { 0x6e305095, SUC_lexicon_tags_37 },
    { 0x5ed05096, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc305098, SUC_lexicon_tags_4 },
    { 0x31f05099, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf350509b, SUC_lexicon_tags_43 },
    { 0xf590509b, SUC_lexicon_tags_63 },
    { 0x5df0509d, SUC_lexicon_tags_0 },
    { 0x3af0509b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09050a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27f050a3, SUC_lexicon_tags_31 },
    { 0x19d050a3, SUC_lexicon_tags_17 },
    { 0x86b050a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d050a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b050ac, SUC_lexicon_tags_63 },
    { 0xe79050ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa63050af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f050b7, SUC_lexicon_tags_8 },
    { 0x3b9050b8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd050bc, SUC_lexicon_tags_7 },
    { 0xa01050bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x959050bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3050c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb050d2, SUC_lexicon_tags_15 },
    { 0xdff050d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf050d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6f050d8, SUC_lexicon_tags_11 },
    { 0x82d050d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9050dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x515050e0, SUC_lexicon_tags_38 },
    { 0xa0f050e1, SUC_lexicon_tags_14 },
    { 0x649050e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5050e5, SUC_lexicon_tags_37 },
    { 0x31d050e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb050eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b050f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445050f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb5050f9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5050fd, SUC_lexicon_tags_47 },
    { 0x2af050fd, SUC_lexicon_tags_37 },
    { 0x9b5050fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd505101, SUC_lexicon_tags_47 },
    { 0x67905102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc905106, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d05109, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x810510b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9a10510d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd05111, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b05114, SUC_lexicon_tags_9 },
    { 0x27705114, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8cb05117, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x79705119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c70511c, SUC_lexicon_tags_8 },
    { 0xee90511d, SUC_lexicon_tags_25 },
    { 0x8e50511c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f05123, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xee305125, SUC_lexicon_tags_12 },
    { 0xfbd05126, SUC_lexicon_tags_11 },
    { 0xa3705127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4105129, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab05130, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f05135, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0f05137, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f0513b, SUC_lexicon_tags_17 },
    { 0xfff0513c, SUC_lexicon_tags_27 },
    { 0x9950513c, SUC_lexicon_tags_34 },
    { 0x6690513e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2905140, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x11f05142, SUC_lexicon_tags_15 },
    { 0xc8b05142, SUC_lexicon_tags_39 },
    { 0x30705144, SUC_lexicon_tags_7 },
    { 0x99d05142, SUC_lexicon_tags_1 },
    { 0x42505146, SUC_lexicon_tags_1 },
    { 0xa0105142, SUC_lexicon_tags_179 },
    { 0x88505148, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc470514a, SUC_lexicon_tags_66 },
    { 0x1590514b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f0514e, SUC_lexicon_tags_26 },
    { 0x5fb0514f, SUC_lexicon_tags_17 },
    { 0x6cb0514f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb05156, SUC_lexicon_tags_30 },
    { 0x6c905157, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x24705159, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd670515b, SUC_lexicon_tags_15 },
    { 0xb910515b, SUC_lexicon_tags_29 },
    { 0x4f50515d, SUC_lexicon_tags_23 },
    { 0x4b90515d, SUC_lexicon_tags_15 },
    { 0xded0515f, SUC_lexicon_tags_63 },
    { 0xd710515e, SUC_lexicon_tags_15 },
    { 0x67705161, SUC_lexicon_tags_37 },
    { 0x2eb0515d, SUC_lexicon_tags_1 },
    { 0xf7305160, SUC_lexicon_tags_7 },
    { 0x6705163, SUC_lexicon_tags_63 },
    { 0x60f0515f, SUC_lexicon_tags_0 },
    { 0xf2f05166, SUC_lexicon_tags_1 },
    { 0xe5105167, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xef105169, SUC_lexicon_tags_34 },
    { 0x2d50516a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4910516f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2505171, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d05176, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x770517b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc390517f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff05182, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0x8b905184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d305186, SUC_lexicon_tags_15 },
    { 0x2a905187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd990518b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc30518d, SUC_lexicon_tags_25 },
    { 0x8210518d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c905193, SUC_lexicon_tags_2 },
    { 0x51305194, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d05197, SUC_lexicon_tags_7 },
    { 0x35505197, SUC_lexicon_tags_27 },
    { 0x7c905198, SUC_lexicon_tags_82 },
    { 0xa5b05199, SUC_lexicon_tags_5 },
    { 0x53905199, SUC_lexicon_tags_12 },
    { 0x5550519c, SUC_lexicon_tags_7 },
    { 0xdff0519c, SUC_lexicon_tags_2 },
    { 0x4250519c, SUC_lexicon_tags_0 },
    { 0x1a905198, SUC_lexicon_tags_7 },
    { 0xadb051a0, SUC_lexicon_tags_1 },
    { 0x907051a0, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1051a4, SUC_lexicon_tags_23 },
    { 0xa3f051a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x569051a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f051ab, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4d9051ad, SUC_lexicon_tags_26 },
    { 0x837051ae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x339051b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d051bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x167051be, SUC_lexicon_tags_49 },
    { 0x4f7051bf, SUC_lexicon_tags_11 },
    { 0xaaf051bf, SUC_lexicon_tags_17 },
    { 0x62d051be, SUC_lexicon_tags_16 },
    { 0xe8f051c2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc87051c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x527051c7, SUC_lexicon_tags_33 },
    { 0xd73051c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d051cf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x521051d1, SUC_lexicon_tags_9 },
    { 0xc69051d2, SUC_lexicon_tags_15 },
    { 0xd2d051d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d051da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x181051dc, SUC_lexicon_tags_43 },
    { 0xd1f051dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1051e0, SUC_lexicon_tags_31 },
    { 0x513051e0, SUC_lexicon_tags_17 },
    { 0xba3051e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d051e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc27051e8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63051ef, SUC_lexicon_tags_7 },
    { 0x63f051f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41051fd, SUC_lexicon_tags_27 },
    { 0x737051fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94905201, SUC_lexicon_tags_10 },
    { 0xe7b05201, SUC_lexicon_tags_38 },
    { 0x4c305203, SUC_lexicon_tags_33 },
    { 0x90f05203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52505206, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd0520f, SUC_lexicon_tags_23 },
    { 0xf0d0520f, SUC_lexicon_tags_1 },
    { 0x47d05211, SUC_lexicon_tags_37 },
    { 0xb4d05212, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45905214, SUC_lexicon_tags_38 },
    { 0x5bb05215, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0305217, SUC_lexicon_tags_25 },
    { 0x79f05217, SUC_lexicon_tags_32 },
    { 0xbc505219, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb310521b, SUC_lexicon_tags_23 },
    { 0x730521c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97f0521e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9305220, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1305223, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f05226, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf150522c, SUC_lexicon_tags_63 },
    { 0x2f30522d, SUC_lexicon_tags_15 },
    { 0x3e90522c, SUC_lexicon_tags_17 },
    { 0xeb50522d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x24305231, SUC_lexicon_tags_47 },
    { 0x2b905232, SUC_lexicon_tags_8 },
    { 0xf5f05232, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a905235, SUC_lexicon_tags_25 },
    { 0x69105236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86505239, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f05241, SUC_lexicon_tags_2 },
    { 0x49305242, SUC_lexicon_tags_43 },
    { 0xbfb05243, SUC_lexicon_tags_17 },
    { 0x93105244, SUC_lexicon_tags_25 },
    { 0x105244, SUC_lexicon_tags_15 },
    { 0xb5d05246, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae50524a, SUC_lexicon_tags_38 },
    { 0x3210524b, SUC_lexicon_tags_23 },
    { 0xd710524b, SUC_lexicon_tags_8 },
    { 0x4f10524c, SUC_lexicon_tags_14 },
    { 0x48f0524e, SUC_lexicon_tags_37 },
    { 0x4650524b, SUC_lexicon_tags_7 },
    { 0x2570524b, SUC_lexicon_tags_30 },
    { 0x4ed0524c, SUC_lexicon_tags_4 },
    { 0x96f05252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3b05254, SUC_lexicon_tags_15 },
    { 0xccf05254, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f0525c, SUC_lexicon_tags_6 },
    { 0xda90525d, SUC_lexicon_tags_26 },
    { 0x510525e, SUC_lexicon_tags_37 },
    { 0xe830525f, SUC_lexicon_tags_2 },
    { 0xfa90525f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1105262, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86105265, SUC_lexicon_tags_7 },
    { 0xa2d05266, SUC_lexicon_tags_11 },
    { 0x8b905267, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0526a, SUC_lexicon_tags_7 },
    { 0x91d0526a, SUC_lexicon_tags_32 },
    { 0xc590526a, SUC_lexicon_tags_15 },
    { 0xe050526d, SUC_lexicon_tags_2 },
    { 0x2610526d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f70527b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5290527d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xce50527f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x25905281, SUC_lexicon_tags_13 },
    { 0x47905281, SUC_lexicon_tags_33 },
    { 0x67505281, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f105286, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcc705288, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x430528a, SUC_lexicon_tags_38 },
    { 0xaf90528a, SUC_lexicon_tags_29 },
    { 0x9e50528c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28f0528e, SUC_lexicon_tags_14 },
    { 0x1fd0528f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b05295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb705297, SUC_lexicon_tags_38 },
    { 0xf3f05298, SUC_lexicon_tags_15 },
    { 0x26105299, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad50529e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x735052a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab052a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafb052a7, SUC_lexicon_tags_26 },
    { 0x307052a8, SUC_lexicon_tags_47 },
    { 0xc89052a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55052ad, SUC_lexicon_tags_583 },
    { 0x179052ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3052b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x841052b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xec3052b6, SUC_lexicon_tags_27 },
    { 0x25d052b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29052ba, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x40f052bc, SUC_lexicon_tags_17 },
    { 0x4a7052bc, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97052c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7052c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f052ca, SUC_lexicon_tags_42 },
    { 0xf3b052cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9052d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9df052d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d3052d6, SUC_lexicon_tags_61 },
    { 0x75d052d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e3052d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83052e4, SUC_lexicon_tags_37 },
    { 0x92b052e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f052ea, SUC_lexicon_tags_25 },
    { 0xaa5052eb, SUC_lexicon_tags_48 },
    { 0x9c3052ec, SUC_lexicon_tags_1 },
    { 0xb97052eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb052f0, SUC_lexicon_tags_62 },
    { 0x21052f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671052f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7052f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb69052fb, SUC_lexicon_tags_14 },
    { 0x37f052fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4b052fe, SUC_lexicon_tags_860 },
    { 0, NULL },
    { 0x4c505300, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b105305, SUC_lexicon_tags_63 },
    { 0xaa905306, SUC_lexicon_tags_0 },
    { 0x17f05306, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b0530a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c10530c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1210530f, SUC_lexicon_tags_38 },
    { 0xaa905310, SUC_lexicon_tags_43 },
    { 0xd2f05311, SUC_lexicon_tags_37 },
    { 0x8f705312, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72905316, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f0531b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d05320, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe905322, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41105328, SUC_lexicon_tags_69 },
    { 0x93505328, SUC_lexicon_tags_27 },
    { 0x22d0532a, SUC_lexicon_tags_33 },
    { 0x59f0532a, SUC_lexicon_tags_12 },
    { 0x9530532c, SUC_lexicon_tags_15 },
    { 0xa950532a, SUC_lexicon_tags_19 },
    { 0xcff0532e, SUC_lexicon_tags_37 },
    { 0xcd905328, SUC_lexicon_tags_57 },
    { 0x50b05330, SUC_lexicon_tags_2 },
    { 0x85d05331, SUC_lexicon_tags_34 },
    { 0xda905329, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b05335, SUC_lexicon_tags_44 },
    { 0x97105336, SUC_lexicon_tags_28 },
    { 0xea105337, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5490533a, SUC_lexicon_tags_1 },
    { 0xb370533b, SUC_lexicon_tags_14 },
    { 0x24f0533a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29705340, SUC_lexicon_tags_13 },
    { 0x97f05340, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9905344, SUC_lexicon_tags_15 },
    { 0x30105345, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x50d05347, SUC_lexicon_tags_15 },
    { 0x52905348, SUC_lexicon_tags_15 },
    { 0x60305349, SUC_lexicon_tags_27 },
    { 0x71b0534a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f05350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96305357, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9590535a, SUC_lexicon_tags_25 },
    { 0xa870535b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b705360, SUC_lexicon_tags_25 },
    { 0x39d05361, SUC_lexicon_tags_45 },
    { 0xf3105361, SUC_lexicon_tags_15 },
    { 0x1c105360, SUC_lexicon_tags_17 },
    { 0x6bd05364, SUC_lexicon_tags_27 },
    { 0x6a505365, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcbb05367, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40905369, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad0536d, SUC_lexicon_tags_15 },
    { 0xd8d0536d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x98705370, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x9d305372, SUC_lexicon_tags_27 },
    { 0x29305372, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f0537e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d05382, SUC_lexicon_tags_37 },
    { 0x7a305383, SUC_lexicon_tags_37 },
    { 0x54f05382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf50538c, SUC_lexicon_tags_37 },
    { 0x6d70538d, SUC_lexicon_tags_33 },
    { 0x1470538d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42905392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af05396, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x22905398, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af0539b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2690539d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2650539f, SUC_lexicon_tags_60 },
    { 0x4c30539f, SUC_lexicon_tags_27 },
    { 0xf9d053a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265053a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x255053a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fd053a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9053b3, SUC_lexicon_tags_8 },
    { 0xca9053b4, SUC_lexicon_tags_17 },
    { 0x5bb053b4, SUC_lexicon_tags_47 },
    { 0x305053b6, SUC_lexicon_tags_13 },
    { 0xb6b053b3, SUC_lexicon_tags_17 },
    { 0x2a7053b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa11053ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d053be, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x61d053c0, SUC_lexicon_tags_49 },
    { 0x335053c1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xdd053c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57053c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb75053c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db053cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x907053cf, SUC_lexicon_tags_23 },
    { 0xd053d0, SUC_lexicon_tags_26 },
    { 0x451053d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc2f053d3, SUC_lexicon_tags_45 },
    { 0xffd053d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf09053d6, SUC_lexicon_tags_60 },
    { 0x2f053d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc33053d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83053dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5053e0, SUC_lexicon_tags_15 },
    { 0x809053e1, SUC_lexicon_tags_2 },
    { 0x8f053e2, SUC_lexicon_tags_28 },
    { 0x17f053e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9053e8, SUC_lexicon_tags_21 },
    { 0x4b053e9, SUC_lexicon_tags_35 },
    { 0x209053ea, SUC_lexicon_tags_87 },
    { 0x425053ea, SUC_lexicon_tags_25 },
    { 0xf9b053ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x581053ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7053f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53053f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1053fc, SUC_lexicon_tags_37 },
    { 0x471053fd, SUC_lexicon_tags_65 },
    { 0xdaf053fc, SUC_lexicon_tags_7 },
    { 0x121053fc, SUC_lexicon_tags_65 },
    { 0x54105400, SUC_lexicon_tags_27 },
    { 0x56f05400, SUC_lexicon_tags_0 },
    { 0xa4f05402, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b05405, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85905407, SUC_lexicon_tags_87 },
    { 0xb105408, SUC_lexicon_tags_129 },
    { 0x8bd05409, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1050540b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a905410, SUC_lexicon_tags_17 },
    { 0x44305411, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d05414, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1905416, SUC_lexicon_tags_15 },
    { 0x49105417, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2505419, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1150541c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b05421, SUC_lexicon_tags_49 },
    { 0x3d705422, SUC_lexicon_tags_7 },
    { 0xbcd05423, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0542d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5130542f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9f05431, SUC_lexicon_tags_25 },
    { 0x6705432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57b05434, SUC_lexicon_tags_37 },
    { 0xc3905434, SUC_lexicon_tags_2 },
    { 0xe5f05436, SUC_lexicon_tags_37 },
    { 0x3705437, SUC_lexicon_tags_57 },
    { 0x6dd05437, SUC_lexicon_tags_0 },
    { 0x16505437, SUC_lexicon_tags_6 },
    { 0x77f05435, SUC_lexicon_tags_2 },
    { 0xd170543b, SUC_lexicon_tags_27 },
    { 0x7170543b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11705447, SUC_lexicon_tags_49 },
    { 0xc7705448, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f0544d, SUC_lexicon_tags_11 },
    { 0x21d0544e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44f05450, SUC_lexicon_tags_49 },
    { 0xaf105451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25505453, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99905455, SUC_lexicon_tags_7 },
    { 0x6f705455, SUC_lexicon_tags_14 },
    { 0xe2f05457, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8050545f, SUC_lexicon_tags_45 },
    { 0xe7d05460, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0705463, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6305467, SUC_lexicon_tags_1 },
    { 0x86905467, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a70546d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x57905470, SUC_lexicon_tags_38 },
    { 0x9ff05471, SUC_lexicon_tags_47 },
    { 0x2f905471, SUC_lexicon_tags_8 },
    { 0xdb305473, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd05476, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x50f05478, SUC_lexicon_tags_11 },
    { 0xa5b05479, SUC_lexicon_tags_32 },
    { 0x7ef05479, SUC_lexicon_tags_37 },
    { 0xd7f0547b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8905483, SUC_lexicon_tags_15 },
    { 0x9bf05484, SUC_lexicon_tags_7 },
    { 0x4c305485, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f05488, SUC_lexicon_tags_6 },
    { 0xecf05489, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfab0548b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d0549e, SUC_lexicon_tags_45 },
    { 0x9190549f, SUC_lexicon_tags_92 },
    { 0xc010549f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9054a4, SUC_lexicon_tags_26 },
    { 0x599054a5, SUC_lexicon_tags_15 },
    { 0x989054a6, SUC_lexicon_tags_336 },
    { 0x50d054a7, SUC_lexicon_tags_9 },
    { 0xe4f054a5, SUC_lexicon_tags_4 },
    { 0xb89054a9, SUC_lexicon_tags_43 },
    { 0x49f054aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x221054ad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8c3054af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd054b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd054b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb17054b9, SUC_lexicon_tags_15 },
    { 0xd0f054b9, SUC_lexicon_tags_19 },
    { 0x10d054ba, SUC_lexicon_tags_84 },
    { 0x2cd054ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7dd054be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d054c1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x23b054c3, SUC_lexicon_tags_56 },
    { 0x8d5054c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x307054c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1054cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xec7054cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363054d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5054d8, SUC_lexicon_tags_39 },
    { 0xdcf054d9, SUC_lexicon_tags_45 },
    { 0x88b054da, SUC_lexicon_tags_45 },
    { 0x633054da, SUC_lexicon_tags_30 },
    { 0xc15054dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5054df, SUC_lexicon_tags_17 },
    { 0x501054df, SUC_lexicon_tags_17 },
    { 0x71f054e0, SUC_lexicon_tags_17 },
    { 0x325054e2, SUC_lexicon_tags_0 },
    { 0x65d054e3, SUC_lexicon_tags_63 },
    { 0x207054e1, SUC_lexicon_tags_2 },
    { 0x35f054e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f054ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95054ef, SUC_lexicon_tags_13 },
    { 0x961054f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf4b054f2, SUC_lexicon_tags_37 },
    { 0xe25054f3, SUC_lexicon_tags_17 },
    { 0xd91054f4, SUC_lexicon_tags_15 },
    { 0x3af054f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef054f9, SUC_lexicon_tags_88 },
    { 0x91f054fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf7054fc, SUC_lexicon_tags_11 },
    { 0x57054fd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe5d054ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a70550d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e505510, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74905514, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8210551e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x89905521, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43305523, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f05526, SUC_lexicon_tags_23 },
    { 0x58905526, SUC_lexicon_tags_7 },
    { 0xd4305526, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb0552c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8505531, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc305535, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b0553a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6df0553c, SUC_lexicon_tags_9 },
    { 0x5b90553c, SUC_lexicon_tags_1 },
    { 0x4cb0553e, SUC_lexicon_tags_37 },
    { 0x3a10553e, SUC_lexicon_tags_1 },
    { 0xf705540, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a305542, SUC_lexicon_tags_25 },
    { 0x31305542, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd3105545, SUC_lexicon_tags_27 },
    { 0xd6d05545, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x42505548, SUC_lexicon_tags_15 },
    { 0x2f905549, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb0554c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab905551, SUC_lexicon_tags_37 },
    { 0x97305551, SUC_lexicon_tags_17 },
    { 0x46705553, SUC_lexicon_tags_19 },
    { 0xc8b05554, SUC_lexicon_tags_17 },
    { 0xef05551, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30555a, SUC_lexicon_tags_2 },
    { 0xfc70555b, SUC_lexicon_tags_85 },
    { 0xc690555c, SUC_lexicon_tags_37 },
    { 0x6d0555a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4130555f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f05564, SUC_lexicon_tags_171 },
    { 0x66505565, SUC_lexicon_tags_6 },
    { 0x59505566, SUC_lexicon_tags_7 },
    { 0x1ad05566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9470556a, SUC_lexicon_tags_12 },
    { 0x6510556b, SUC_lexicon_tags_1 },
    { 0x6d50556b, SUC_lexicon_tags_15 },
    { 0x4070556d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75705573, SUC_lexicon_tags_17 },
    { 0xd1905574, SUC_lexicon_tags_9 },
    { 0x36305575, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d05578, SUC_lexicon_tags_38 },
    { 0x61305578, SUC_lexicon_tags_2 },
    { 0xdef05579, SUC_lexicon_tags_62 },
    { 0x9310557a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a90557f, SUC_lexicon_tags_25 },
    { 0xd1b05580, SUC_lexicon_tags_11 },
    { 0xaff05581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd05584, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d05587, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf050558d, SUC_lexicon_tags_68 },
    { 0xecb0558e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f05594, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70105596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7055a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d055a5, SUC_lexicon_tags_17 },
    { 0xc0f055a6, SUC_lexicon_tags_7 },
    { 0xabf055a6, SUC_lexicon_tags_46 },
    { 0x863055a5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb055b7, SUC_lexicon_tags_107 },
    { 0x149055b8, SUC_lexicon_tags_2 },
    { 0x257055b8, SUC_lexicon_tags_17 },
    { 0xb29055b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x101055bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5055c0, SUC_lexicon_tags_45 },
    { 0x4fb055c1, SUC_lexicon_tags_12 },
    { 0x60b055c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d055c6, SUC_lexicon_tags_2 },
    { 0xe63055c7, SUC_lexicon_tags_14 },
    { 0x315055c7, SUC_lexicon_tags_45 },
    { 0x8a7055c6, SUC_lexicon_tags_12 },
    { 0xd3b055ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x335055cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9bb055ce, SUC_lexicon_tags_17 },
    { 0xfd3055cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f3055d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917055d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccf055d7, SUC_lexicon_tags_0 },
    { 0x585055d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f055dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5055e0, SUC_lexicon_tags_37 },
    { 0xe69055e1, SUC_lexicon_tags_2 },
    { 0xa3f055e2, SUC_lexicon_tags_9 },
    { 0x193055e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b055e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205055ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f055f1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe5055f3, SUC_lexicon_tags_43 },
    { 0xaa5055f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3ed055f6, SUC_lexicon_tags_12 },
    { 0x741055f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3055fa, SUC_lexicon_tags_46 },
    { 0x3c7055fb, SUC_lexicon_tags_7 },
    { 0xc5f055fc, SUC_lexicon_tags_25 },
    { 0xba7055fb, SUC_lexicon_tags_25 },
    { 0xf39055fd, SUC_lexicon_tags_7 },
    { 0x723055ff, SUC_lexicon_tags_14 },
    { 0x62105600, SUC_lexicon_tags_15 },
    { 0x227055ff, SUC_lexicon_tags_25 },
    { 0xb305602, SUC_lexicon_tags_11 },
    { 0x3e1055ff, SUC_lexicon_tags_31 },
    { 0xd0705604, SUC_lexicon_tags_14 },
    { 0x4ff05605, SUC_lexicon_tags_25 },
    { 0xb2105606, SUC_lexicon_tags_55 },
    { 0x98305606, SUC_lexicon_tags_14 },
    { 0x19705602, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31d0560a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8310560c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x470560f, SUC_lexicon_tags_8 },
    { 0x7090560f, SUC_lexicon_tags_17 },
    { 0x53705610, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19905613, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53f05615, SUC_lexicon_tags_61 },
    { 0x10105616, SUC_lexicon_tags_77 },
    { 0x73b05617, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd0561b, SUC_lexicon_tags_850 },
    { 0x4c70561c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4905620, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1905622, SUC_lexicon_tags_17 },
    { 0x8c905623, SUC_lexicon_tags_63 },
    { 0x52d05623, SUC_lexicon_tags_7 },
    { 0x40705625, SUC_lexicon_tags_1 },
    { 0x47505626, SUC_lexicon_tags_37 },
    { 0xda305626, SUC_lexicon_tags_9 },
    { 0xb3d05627, SUC_lexicon_tags_49 },
    { 0x7c905628, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1105630, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35505638, SUC_lexicon_tags_17 },
    { 0xca505639, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b50563b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf190563e, SUC_lexicon_tags_17 },
    { 0xe0d0563f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1705641, SUC_lexicon_tags_92 },
    { 0xfe505642, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe105645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11905647, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2610564c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b90564e, SUC_lexicon_tags_43 },
    { 0xbd10564f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb9d05651, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x41905654, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf10565c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69505662, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66505664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe105666, SUC_lexicon_tags_37 },
    { 0x42905666, SUC_lexicon_tags_43 },
    { 0xafd05668, SUC_lexicon_tags_25 },
    { 0x57f05669, SUC_lexicon_tags_2 },
    { 0x8c705669, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47b0566c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b0566f, SUC_lexicon_tags_49 },
    { 0xd2d0566f, SUC_lexicon_tags_1 },
    { 0xfd105671, SUC_lexicon_tags_11 },
    { 0xcb305670, SUC_lexicon_tags_25 },
    { 0x6d0566f, SUC_lexicon_tags_79 },
    { 0x39905671, SUC_lexicon_tags_7 },
    { 0x1b05671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf905678, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0d0567a, SUC_lexicon_tags_17 },
    { 0xa3f0567a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b0567f, SUC_lexicon_tags_12 },
    { 0xa1b0567f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda705685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb705687, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb0568c, SUC_lexicon_tags_14 },
    { 0xd370568c, SUC_lexicon_tags_2 },
    { 0x27d0568e, SUC_lexicon_tags_11 },
    { 0xcc30568f, SUC_lexicon_tags_11 },
    { 0xf1d05690, SUC_lexicon_tags_47 },
    { 0xe505691, SUC_lexicon_tags_1 },
    { 0x26505691, SUC_lexicon_tags_1 },
    { 0xdf905693, SUC_lexicon_tags_0 },
    { 0x1530568c, SUC_lexicon_tags_31 },
    { 0xa4305695, SUC_lexicon_tags_17 },
    { 0x97305695, SUC_lexicon_tags_28 },
    { 0xfdd05697, SUC_lexicon_tags_67 },
    { 0x4ef05692, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe090569b, SUC_lexicon_tags_12 },
    { 0x4cb0569c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87056a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41056a8, SUC_lexicon_tags_19 },
    { 0xebd056a9, SUC_lexicon_tags_30 },
    { 0xf29056aa, SUC_lexicon_tags_7 },
    { 0x3d5056aa, SUC_lexicon_tags_15 },
    { 0xd6d056ac, SUC_lexicon_tags_17 },
    { 0x953056a9, SUC_lexicon_tags_46 },
    { 0x3c5056ae, SUC_lexicon_tags_56 },
    { 0xadf056a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3056b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd25056b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6bb056b7, SUC_lexicon_tags_7 },
    { 0x91056b8, SUC_lexicon_tags_27 },
    { 0x2bb056b9, SUC_lexicon_tags_15 },
    { 0x911056ba, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf056bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07056c9, SUC_lexicon_tags_12 },
    { 0xf4d056ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdd056cc, SUC_lexicon_tags_17 },
    { 0xd61056cd, SUC_lexicon_tags_1 },
    { 0x11b056cc, SUC_lexicon_tags_37 },
    { 0x1d5056cf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1056d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9056d5, SUC_lexicon_tags_37 },
    { 0x57056d6, SUC_lexicon_tags_25 },
    { 0x2cd056d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f056db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7056df, SUC_lexicon_tags_9 },
    { 0xc5056e0, SUC_lexicon_tags_16 },
    { 0xb1b056df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295056e5, SUC_lexicon_tags_25 },
    { 0x57b056e6, SUC_lexicon_tags_29 },
    { 0x927056e7, SUC_lexicon_tags_63 },
    { 0x361056e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfff056ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d056ee, SUC_lexicon_tags_37 },
    { 0x31b056ef, SUC_lexicon_tags_11 },
    { 0xe01056f0, SUC_lexicon_tags_14 },
    { 0x647056f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9056f7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x185056f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c105700, SUC_lexicon_tags_11 },
    { 0x5b105700, SUC_lexicon_tags_49 },
    { 0xcf305702, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f05706, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f05709, SUC_lexicon_tags_12 },
    { 0x85f0570a, SUC_lexicon_tags_37 },
    { 0x2f05709, SUC_lexicon_tags_30 },
    { 0x46d05709, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17705717, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10571b, SUC_lexicon_tags_9 },
    { 0x3fd0571c, SUC_lexicon_tags_25 },
    { 0x64b0571d, SUC_lexicon_tags_11 },
    { 0x79b0571e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3105722, SUC_lexicon_tags_59 },
    { 0x89b05723, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac905727, SUC_lexicon_tags_38 },
    { 0x99d05728, SUC_lexicon_tags_37 },
    { 0x88305729, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1090572e, SUC_lexicon_tags_15 },
    { 0xc870572e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb05738, SUC_lexicon_tags_31 },
    { 0xaf905738, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd410573b, SUC_lexicon_tags_38 },
    { 0xccb0573c, SUC_lexicon_tags_28 },
    { 0xaf70573d, SUC_lexicon_tags_25 },
    { 0x8db0573c, SUC_lexicon_tags_6 },
    { 0xe3f0573d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85f05741, SUC_lexicon_tags_65 },
    { 0x86305741, SUC_lexicon_tags_49 },
    { 0x54b05741, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26705746, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57b05748, SUC_lexicon_tags_49 },
    { 0x82f05749, SUC_lexicon_tags_1 },
    { 0xff105749, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd05752, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x94105754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc705757, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa130575d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb05761, SUC_lexicon_tags_2 },
    { 0xe0d05762, SUC_lexicon_tags_45 },
    { 0x92705763, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b05766, SUC_lexicon_tags_17 },
    { 0x83905766, SUC_lexicon_tags_60 },
    { 0x31305766, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad30576c, SUC_lexicon_tags_43 },
    { 0x8970576d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2505771, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68f05773, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbff05775, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8f05777, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f0577b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf370577d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16705784, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1250578c, SUC_lexicon_tags_25 },
    { 0xea30578d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3f0578f, SUC_lexicon_tags_31 },
    { 0x88105790, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2505792, SUC_lexicon_tags_27 },
    { 0x1d705792, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8f05795, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9fb05797, SUC_lexicon_tags_11 },
    { 0x70b05798, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bd0579a, SUC_lexicon_tags_27 },
    { 0xe510579b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4490579e, SUC_lexicon_tags_9 },
    { 0x4a50579e, SUC_lexicon_tags_17 },
    { 0x9cd057a0, SUC_lexicon_tags_2 },
    { 0x7370579f, SUC_lexicon_tags_27 },
    { 0xf95057a2, SUC_lexicon_tags_15 },
    { 0x1490579f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x885057a5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7057ae, SUC_lexicon_tags_25 },
    { 0x953057af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d057b3, SUC_lexicon_tags_15 },
    { 0x845057b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x48b057b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46f057b8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1057be, SUC_lexicon_tags_45 },
    { 0xdc5057be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5057c4, SUC_lexicon_tags_37 },
    { 0x2bf057c4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19057c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadd057ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf057ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df057d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b057d7, SUC_lexicon_tags_65 },
    { 0xb63057d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe1057da, SUC_lexicon_tags_12 },
    { 0xeeb057db, SUC_lexicon_tags_11 },
    { 0x86b057dc, SUC_lexicon_tags_1 },
    { 0x9f5057dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x867057e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9f057e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3057e8, SUC_lexicon_tags_64 },
    { 0xae1057e9, SUC_lexicon_tags_60 },
    { 0x211057ea, SUC_lexicon_tags_47 },
    { 0xb71057ea, SUC_lexicon_tags_15 },
    { 0xd6d057eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75057ef, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95057f4, SUC_lexicon_tags_92 },
    { 0x8c7057f5, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xcd9057f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x68d057f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb057fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x537057ff, SUC_lexicon_tags_27 },
    { 0x8bb05800, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d305802, SUC_lexicon_tags_28 },
    { 0xc5105802, SUC_lexicon_tags_13 },
    { 0x31f05804, SUC_lexicon_tags_23 },
    { 0x4af05804, SUC_lexicon_tags_15 },
    { 0x3b705806, SUC_lexicon_tags_37 },
    { 0xf3305805, SUC_lexicon_tags_12 },
    { 0xc505808, SUC_lexicon_tags_37 },
    { 0x9eb05808, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d0580c, SUC_lexicon_tags_45 },
    { 0xf530580d, SUC_lexicon_tags_37 },
    { 0x9cd0580d, SUC_lexicon_tags_7 },
    { 0x3c90580f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf05812, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf105818, SUC_lexicon_tags_7 },
    { 0x54f05819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2430581b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b05821, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3fd05823, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b305825, SUC_lexicon_tags_43 },
    { 0xf1705826, SUC_lexicon_tags_105 },
    { 0x8ad05827, SUC_lexicon_tags_8 },
    { 0xca905828, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1950582c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f05830, SUC_lexicon_tags_37 },
    { 0xac505831, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a105835, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30d05837, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b05841, SUC_lexicon_tags_49 },
    { 0xd4505842, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe9905844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf05847, SUC_lexicon_tags_17 },
    { 0xfc305848, SUC_lexicon_tags_12 },
    { 0x73305847, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7b0584b, SUC_lexicon_tags_8 },
    { 0x7ef0584b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf310584e, SUC_lexicon_tags_2 },
    { 0x7210584e, SUC_lexicon_tags_32 },
    { 0xcaf05850, SUC_lexicon_tags_27 },
    { 0xfa905851, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb05855, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3990585b, SUC_lexicon_tags_43 },
    { 0xd190585c, SUC_lexicon_tags_1 },
    { 0xb710585d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab05860, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e705864, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb05868, SUC_lexicon_tags_23 },
    { 0xcd305869, SUC_lexicon_tags_30 },
    { 0xc8f0586a, SUC_lexicon_tags_35 },
    { 0xe7b0586a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d105874, SUC_lexicon_tags_11 },
    { 0x6c705874, SUC_lexicon_tags_2 },
    { 0xef05876, SUC_lexicon_tags_27 },
    { 0xb9305877, SUC_lexicon_tags_60 },
    { 0x8fd05878, SUC_lexicon_tags_14 },
    { 0xf5b05874, SUC_lexicon_tags_61 },
    { 0xfb10587a, SUC_lexicon_tags_25 },
    { 0x1710587a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d05880, SUC_lexicon_tags_68 },
    { 0xa4705881, SUC_lexicon_tags_14 },
    { 0x62105882, SUC_lexicon_tags_17 },
    { 0x90305882, SUC_lexicon_tags_37 },
    { 0xaf105881, SUC_lexicon_tags_25 },
    { 0xa3b05885, SUC_lexicon_tags_23 },
    { 0x2df05886, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc505889, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6390588d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb05897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd0589f, SUC_lexicon_tags_1 },
    { 0xa6d058a0, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b058a3, SUC_lexicon_tags_55 },
    { 0xa8d058a4, SUC_lexicon_tags_28 },
    { 0xdc1058a4, SUC_lexicon_tags_10 },
    { 0x1b3058a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489058ab, SUC_lexicon_tags_152 },
    { 0xc99058ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7058af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801058bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe85058bf, SUC_lexicon_tags_23 },
    { 0xafd058bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39b058c2, SUC_lexicon_tags_6 },
    { 0x431058c3, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf058c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x845058c8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed058cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d058d1, SUC_lexicon_tags_61 },
    { 0xf7058d2, SUC_lexicon_tags_15 },
    { 0x1fb058d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5058d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x751058da, SUC_lexicon_tags_11 },
    { 0x293058da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb058e0, SUC_lexicon_tags_33 },
    { 0xd93058e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f5058e3, SUC_lexicon_tags_44 },
    { 0x9d7058e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39058eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3058f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935058f4, SUC_lexicon_tags_49 },
    { 0x80d058f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe59058f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19058fb, SUC_lexicon_tags_15 },
    { 0xa81058fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5c1058fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8905900, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71105902, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d05906, SUC_lexicon_tags_1 },
    { 0x9e305907, SUC_lexicon_tags_17 },
    { 0xb305908, SUC_lexicon_tags_60 },
    { 0xe4905908, SUC_lexicon_tags_25 },
    { 0x21f0590a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x390590c, SUC_lexicon_tags_47 },
    { 0xf910590d, SUC_lexicon_tags_66 },
    { 0xddd0590d, SUC_lexicon_tags_25 },
    { 0x1bb0590c, SUC_lexicon_tags_61 },
    { 0x1d10590c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d05913, SUC_lexicon_tags_7 },
    { 0xc9705914, SUC_lexicon_tags_37 },
    { 0x67105915, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d05918, SUC_lexicon_tags_1 },
    { 0x2d705919, SUC_lexicon_tags_3 },
    { 0xf350591a, SUC_lexicon_tags_47 },
    { 0x4ed0591b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd0591e, SUC_lexicon_tags_11 },
    { 0x8d30591e, SUC_lexicon_tags_60 },
    { 0xb350591e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x25305922, SUC_lexicon_tags_0 },
    { 0x83f05923, SUC_lexicon_tags_7 },
    { 0x12f05923, SUC_lexicon_tags_11 },
    { 0xa105925, SUC_lexicon_tags_17 },
    { 0x66105926, SUC_lexicon_tags_74 },
    { 0x47d05927, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b30592e, SUC_lexicon_tags_11 },
    { 0x4190592f, SUC_lexicon_tags_17 },
    { 0x1af05930, SUC_lexicon_tags_0 },
    { 0x94505931, SUC_lexicon_tags_174 },
    { 0xd6705932, SUC_lexicon_tags_87 },
    { 0x9a905933, SUC_lexicon_tags_14 },
    { 0xb3d0592e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f05937, SUC_lexicon_tags_48 },
    { 0x63905937, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3130593a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6970593d, SUC_lexicon_tags_1 },
    { 0xcef0593e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2705942, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d05945, SUC_lexicon_tags_7 },
    { 0xc4705946, SUC_lexicon_tags_45 },
    { 0xb105945, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa9105949, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b0594e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x31d05950, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x3fd05952, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d05955, SUC_lexicon_tags_11 },
    { 0x4d305956, SUC_lexicon_tags_1 },
    { 0xf1b05957, SUC_lexicon_tags_27 },
    { 0x28b05955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50595c, SUC_lexicon_tags_2 },
    { 0x9990595d, SUC_lexicon_tags_17 },
    { 0x74b0595e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd05961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e305965, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86905969, SUC_lexicon_tags_7 },
    { 0x9950596a, SUC_lexicon_tags_55 },
    { 0xf650596a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2d0596d, SUC_lexicon_tags_14 },
    { 0x830596e, SUC_lexicon_tags_49 },
    { 0x34b0596f, SUC_lexicon_tags_45 },
    { 0x40f0596f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x50705972, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x11305974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e305977, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5630597b, SUC_lexicon_tags_39 },
    { 0xe970597c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0597f, SUC_lexicon_tags_15 },
    { 0x88305980, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f05982, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcad05984, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3105990, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b05995, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xab505997, SUC_lexicon_tags_15 },
    { 0xd4f05998, SUC_lexicon_tags_18 },
    { 0x9a105997, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8470599c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237059a0, SUC_lexicon_tags_37 },
    { 0x6ab059a1, SUC_lexicon_tags_25 },
    { 0x5a3059a2, SUC_lexicon_tags_28 },
    { 0x40d059a2, SUC_lexicon_tags_1 },
    { 0x5d059a2, SUC_lexicon_tags_29 },
    { 0xa1d059a5, SUC_lexicon_tags_49 },
    { 0x237059a6, SUC_lexicon_tags_15 },
    { 0x873059a2, SUC_lexicon_tags_7 },
    { 0xc15059a8, SUC_lexicon_tags_7 },
    { 0x71b059a9, SUC_lexicon_tags_37 },
    { 0x1bd059a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c5059ac, SUC_lexicon_tags_49 },
    { 0x51b059ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x97f059af, SUC_lexicon_tags_25 },
    { 0xe9b059b0, SUC_lexicon_tags_12 },
    { 0x18f059b1, SUC_lexicon_tags_17 },
    { 0x6e3059af, SUC_lexicon_tags_174 },
    { 0xfb7059b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe65059b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9059bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b059be, SUC_lexicon_tags_25 },
    { 0x11059bf, SUC_lexicon_tags_107 },
    { 0x33059bf, SUC_lexicon_tags_9 },
    { 0xae5059c1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b059c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf59059c6, SUC_lexicon_tags_17 },
    { 0xc87059c6, SUC_lexicon_tags_15 },
    { 0x5f5059c7, SUC_lexicon_tags_37 },
    { 0xca1059c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f059d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3059d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5eb059da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b059dd, SUC_lexicon_tags_11 },
    { 0x46d059de, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5059e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5059e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7059eb, SUC_lexicon_tags_17 },
    { 0x241059ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc85059ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab059f1, SUC_lexicon_tags_1 },
    { 0x131059f1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8f9059f4, SUC_lexicon_tags_0 },
    { 0xba7059f5, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0x99059f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2905a01, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c905a05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3705a09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9705a0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x36905a0e, SUC_lexicon_tags_17 },
    { 0x3bd05a0f, SUC_lexicon_tags_33 },
    { 0xa5705a0e, SUC_lexicon_tags_37 },
    { 0x7e905a0e, SUC_lexicon_tags_2 },
    { 0x13505a12, SUC_lexicon_tags_21 },
    { 0x82105a13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc505a15, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2e705a17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f05a19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5105a1b, SUC_lexicon_tags_10 },
    { 0x3b905a1c, SUC_lexicon_tags_39 },
    { 0xc7305a1d, SUC_lexicon_tags_1 },
    { 0x9b105a1e, SUC_lexicon_tags_37 },
    { 0x20905a1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd05a22, SUC_lexicon_tags_2 },
    { 0x4ed05a23, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f05a28, SUC_lexicon_tags_29 },
    { 0xe1305a29, SUC_lexicon_tags_26 },
    { 0x21705a2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc705a2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7305a2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab05a33, SUC_lexicon_tags_43 },
    { 0x48705a33, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad05a40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f05a44, SUC_lexicon_tags_29 },
    { 0x6d305a45, SUC_lexicon_tags_21 },
    { 0x68305a46, SUC_lexicon_tags_60 },
    { 0xba905a47, SUC_lexicon_tags_6 },
    { 0x73105a48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92b05a4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b05a4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8905a52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36705a56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30705a5a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7705a5d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d05a61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6f05a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5b05a65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31105a67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2305a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc305a72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa105a75, SUC_lexicon_tags_58 },
    { 0xa7105a76, SUC_lexicon_tags_66 },
    { 0x60905a77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4105a79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d05a7c, SUC_lexicon_tags_25 },
    { 0x3f105a7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64905a7f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xea705a81, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84105a86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b105a8b, SUC_lexicon_tags_57 },
    { 0x56705a8c, SUC_lexicon_tags_59 },
    { 0xaa705a8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f05a90, SUC_lexicon_tags_60 },
    { 0x71305a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4b05a93, SUC_lexicon_tags_12 },
    { 0xab105a94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7505a97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd05a9a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x23905a9c, SUC_lexicon_tags_11 },
    { 0xce305a9d, SUC_lexicon_tags_38 },
    { 0x8bf05a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d05aa2, SUC_lexicon_tags_47 },
    { 0x21305aa3, SUC_lexicon_tags_29 },
    { 0xab905aa3, SUC_lexicon_tags_9 },
    { 0x5b705aa2, SUC_lexicon_tags_23 },
    { 0xa1d05aa6, SUC_lexicon_tags_79 },
    { 0x78705aa2, SUC_lexicon_tags_63 },
    { 0xf0105aa8, SUC_lexicon_tags_33 },
    { 0x71b05aa9, SUC_lexicon_tags_43 },
    { 0x80b05aa8, SUC_lexicon_tags_10 },
    { 0x7f905aab, SUC_lexicon_tags_47 },
    { 0x54105aac, SUC_lexicon_tags_47 },
    { 0x9ed05aac, SUC_lexicon_tags_11 },
    { 0xc5305aaa, SUC_lexicon_tags_37 },
    { 0x4b905aaf, SUC_lexicon_tags_11 },
    { 0x1fd05aa9, SUC_lexicon_tags_63 },
    { 0xa3f05aa7, SUC_lexicon_tags_23 },
    { 0x20505aaf, SUC_lexicon_tags_37 },
    { 0x69905ab3, SUC_lexicon_tags_30 },
    { 0xa2905ab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5305ab6, SUC_lexicon_tags_22 },
    { 0x9b105ab6, SUC_lexicon_tags_9 },
    { 0x4ad05ab8, SUC_lexicon_tags_1 },
    { 0x3af05ab9, SUC_lexicon_tags_5 },
    { 0x97305aba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5f05abc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e105ac0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47305ac2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfef05ac4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9905acf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42105ad2, SUC_lexicon_tags_25 },
    { 0x1e905ad2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23305ad6, SUC_lexicon_tags_14 },
    { 0x3eb05ad7, SUC_lexicon_tags_11 },
    { 0x9ad05ad7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf905ade, SUC_lexicon_tags_11 },
    { 0x26b05adf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f05ae7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45105aec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef05af0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6705af4, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a105afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19105b01, SUC_lexicon_tags_30 },
    { 0x6d105b02, SUC_lexicon_tags_37 },
    { 0x1105b01, SUC_lexicon_tags_55 },
    { 0x82b05b03, SUC_lexicon_tags_2 },
    { 0xba305b03, SUC_lexicon_tags_7 },
    { 0xd9705b01, SUC_lexicon_tags_30 },
    { 0x55b05b06, SUC_lexicon_tags_43 },
    { 0x8df05b05, SUC_lexicon_tags_15 },
    { 0xc105b06, SUC_lexicon_tags_45 },
    { 0x7b05b0a, SUC_lexicon_tags_9 },
    { 0x69705b0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef05b0f, SUC_lexicon_tags_49 },
    { 0x84505b0f, SUC_lexicon_tags_0 },
    { 0x13305b10, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb05b16, SUC_lexicon_tags_56 },
    { 0x89705b16, SUC_lexicon_tags_11 },
    { 0x3d305b18, SUC_lexicon_tags_2 },
    { 0x20105b19, SUC_lexicon_tags_55 },
    { 0x57305b19, SUC_lexicon_tags_10 },
    { 0xe1f05b1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93505b20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83705b25, SUC_lexicon_tags_10 },
    { 0xca705b26, SUC_lexicon_tags_37 },
    { 0x3df05b26, SUC_lexicon_tags_25 },
    { 0xb4f05b25, SUC_lexicon_tags_37 },
    { 0x63b05b29, SUC_lexicon_tags_15 },
    { 0x4fb05b27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9705b2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d505b2e, SUC_lexicon_tags_28 },
    { 0x1b05b2f, SUC_lexicon_tags_62 },
    { 0xcbd05b30, SUC_lexicon_tags_23 },
    { 0xe9105b31, SUC_lexicon_tags_37 },
    { 0x1705b31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7505b39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfd305b3b, SUC_lexicon_tags_25 },
    { 0xcf05b3c, SUC_lexicon_tags_37 },
    { 0x4bd05b3d, SUC_lexicon_tags_8 },
    { 0x9db05b3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d905b43, SUC_lexicon_tags_437 },
    { 0x62d05b44, SUC_lexicon_tags_7 },
    { 0x1a905b45, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xdcd05b47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6f05b49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d05b58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69705b5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf905b5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86105b5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d05b63, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57105b69, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c705b6c, SUC_lexicon_tags_29 },
    { 0x44705b6d, SUC_lexicon_tags_28 },
    { 0xf3105b6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59105b71, SUC_lexicon_tags_1 },
    { 0x27d05b71, SUC_lexicon_tags_29 },
    { 0x40305b73, SUC_lexicon_tags_28 },
    { 0xbe505b74, SUC_lexicon_tags_9 },
    { 0x1a905b75, SUC_lexicon_tags_30 },
    { 0xd5905b76, SUC_lexicon_tags_7 },
    { 0xb1905b74, SUC_lexicon_tags_17 },
    { 0xc705b78, SUC_lexicon_tags_7 },
    { 0x77305b75, SUC_lexicon_tags_15 },
    { 0x61705b76, SUC_lexicon_tags_2 },
    { 0x18f05b78, SUC_lexicon_tags_37 },
    { 0x7ef05b7c, SUC_lexicon_tags_27 },
    { 0x2cf05b78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b05b85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c705b8a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb05b90, SUC_lexicon_tags_16 },
    { 0xae905b90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb705b96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55305b9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9305b9c, SUC_lexicon_tags_38 },
    { 0xb505b9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c305baa, SUC_lexicon_tags_2 },
    { 0x4b05baa, SUC_lexicon_tags_1382 },
    { 0xa3105baa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24705bb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7105bb5, SUC_lexicon_tags_12 },
    { 0x29105bb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf705bb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37305bbd, SUC_lexicon_tags_28 },
    { 0xba105bbe, SUC_lexicon_tags_19 },
    { 0xb9f05bbf, SUC_lexicon_tags_1 },
    { 0x4ef05bc0, SUC_lexicon_tags_14 },
    { 0x5dd05bc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad05bc4, SUC_lexicon_tags_19 },
    { 0x31d05bc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b905bcb, SUC_lexicon_tags_53 },
    { 0x7c505bcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde305bd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ff05bd3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d05bd8, SUC_lexicon_tags_84 },
    { 0x1ed05bd8, SUC_lexicon_tags_12 },
    { 0x8e305bda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6505bde, SUC_lexicon_tags_35 },
    { 0x53305bdf, SUC_lexicon_tags_7 },
    { 0x1df05bde, SUC_lexicon_tags_13 },
    { 0x305be0, SUC_lexicon_tags_1 },
    { 0x83905bde, SUC_lexicon_tags_37 },
    { 0xf9d05be3, SUC_lexicon_tags_61 },
    { 0xdb505be1, SUC_lexicon_tags_27 },
    { 0x6105bde, SUC_lexicon_tags_14 },
    { 0xd4905be6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb05bec, SUC_lexicon_tags_37 },
    { 0xe1105bed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6905bf1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf05bf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ef05bf6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69705bfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d505bff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6705c01, SUC_lexicon_tags_26 },
    { 0x6fb05c01, SUC_lexicon_tags_35 },
    { 0x72d05c03, SUC_lexicon_tags_44 },
    { 0xb7105c03, SUC_lexicon_tags_12 },
    { 0x93b05c04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10305c0a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b05c0f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b05c12, SUC_lexicon_tags_60 },
    { 0x4df05c13, SUC_lexicon_tags_37 },
    { 0x22905c14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb05c18, SUC_lexicon_tags_7 },
    { 0xa9305c18, SUC_lexicon_tags_63 },
    { 0xe4d05c19, SUC_lexicon_tags_16 },
    { 0x1e705c1b, SUC_lexicon_tags_47 },
    { 0xd4505c1b, SUC_lexicon_tags_98 },
    { 0x63905c1b, SUC_lexicon_tags_35 },
    { 0x8305c1b, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xa3305c20, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb705c23, SUC_lexicon_tags_16 },
    { 0xeb05c23, SUC_lexicon_tags_4 },
    { 0xbe705c24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee905c2c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xad905c2f, SUC_lexicon_tags_11 },
    { 0xae305c2f, SUC_lexicon_tags_15 },
    { 0x5305c2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28705c33, SUC_lexicon_tags_138 },
    { 0x4cd05c34, SUC_lexicon_tags_28 },
    { 0x64f05c35, SUC_lexicon_tags_9 },
    { 0xd7505c36, SUC_lexicon_tags_72 },
    { 0xe2905c34, SUC_lexicon_tags_2 },
    { 0x76905c37, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf05c3c, SUC_lexicon_tags_17 },
    { 0x50905c3d, SUC_lexicon_tags_17 },
    { 0xa3f05c3e, SUC_lexicon_tags_619 },
    { 0xa4905c3c, SUC_lexicon_tags_79 },
    { 0xe2f05c3d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x85105c42, SUC_lexicon_tags_37 },
    { 0xa4505c42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x49105c45, SUC_lexicon_tags_82 },
    { 0xe2705c46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e105c49, SUC_lexicon_tags_29 },
    { 0x26105c49, SUC_lexicon_tags_25 },
    { 0x1df05c4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ef05c4d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d505c51, SUC_lexicon_tags_25 },
    { 0x75905c52, SUC_lexicon_tags_46 },
    { 0xcb505c53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60505c57, SUC_lexicon_tags_1 },
    { 0xb9d05c58, SUC_lexicon_tags_7 },
    { 0x48f05c58, SUC_lexicon_tags_23 },
    { 0x11b05c59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d05c5d, SUC_lexicon_tags_26 },
    { 0x7e305c5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54305c61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a905c64, SUC_lexicon_tags_69 },
    { 0xfb05c64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96305c68, SUC_lexicon_tags_9 },
    { 0xd0b05c69, SUC_lexicon_tags_0 },
    { 0x76b05c6a, SUC_lexicon_tags_8 },
    { 0xd0305c6a, SUC_lexicon_tags_56 },
    { 0x84b05c6b, SUC_lexicon_tags_25 },
    { 0x5a305c6a, SUC_lexicon_tags_65 },
    { 0x3b705c6e, SUC_lexicon_tags_47 },
    { 0x37105c6f, SUC_lexicon_tags_12 },
    { 0xfc105c69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b05c73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7305c7c, SUC_lexicon_tags_26 },
    { 0xc3b05c7d, SUC_lexicon_tags_14 },
    { 0x6b505c7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84505c84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f05c8a, SUC_lexicon_tags_11 },
    { 0xbab05c8a, SUC_lexicon_tags_47 },
    { 0xbdf05c8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x55105c8e, SUC_lexicon_tags_0 },
    { 0x6cf05c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6105c91, SUC_lexicon_tags_11 },
    { 0xb2705c92, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa6505c94, SUC_lexicon_tags_48 },
    { 0xd3905c95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22105c9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77f05c9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f05c9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0d05ca1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7105ca4, SUC_lexicon_tags_31 },
    { 0xb7b05ca5, SUC_lexicon_tags_9 },
    { 0x9e305ca5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d05ca8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa9d05caa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4f505cac, SUC_lexicon_tags_43 },
    { 0xcdf05cac, SUC_lexicon_tags_6 },
    { 0x9fd05cad, SUC_lexicon_tags_34 },
    { 0x87505cad, SUC_lexicon_tags_38 },
    { 0x51d05cb0, SUC_lexicon_tags_7 },
    { 0x38705cac, SUC_lexicon_tags_7 },
    { 0x20505cb2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x85105cb5, SUC_lexicon_tags_37 },
    { 0xd6705cb5, SUC_lexicon_tags_12 },
    { 0xde905cb7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d05cbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7705cbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde905cc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e505cc8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d05ccb, SUC_lexicon_tags_37 },
    { 0xc1705ccb, SUC_lexicon_tags_7 },
    { 0x37d05ccd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b05cd0, SUC_lexicon_tags_11 },
    { 0x23305cd0, SUC_lexicon_tags_295 },
    { 0x69105cd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2305cd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b05cd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2105cd9, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d05cdd, SUC_lexicon_tags_66 },
    { 0x8c505cde, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcef05ce0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d05ce4, SUC_lexicon_tags_33 },
    { 0x39b05ce5, SUC_lexicon_tags_23 },
    { 0x75105ce6, SUC_lexicon_tags_12 },
    { 0xe6305ce5, SUC_lexicon_tags_4 },
    { 0x70305ce8, SUC_lexicon_tags_30 },
    { 0x77505ce9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef05ced, SUC_lexicon_tags_15 },
    { 0xcbf05ced, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81905cf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f05cf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2505cf8, SUC_lexicon_tags_386 },
    { 0, NULL },
    { 0xabb05cfa, SUC_lexicon_tags_8 },
    { 0xcb305cfa, SUC_lexicon_tags_61 },
    { 0xa0d05cfb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0705cff, SUC_lexicon_tags_12 },
    { 0x1e905d00, SUC_lexicon_tags_17 },
    { 0xd8305d01, SUC_lexicon_tags_57 },
    { 0x15505d00, SUC_lexicon_tags_37 },
    { 0x3d905d03, SUC_lexicon_tags_107 },
    { 0x43d05d03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd05d06, SUC_lexicon_tags_29 },
    { 0x99b05d07, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x29305d09, SUC_lexicon_tags_26 },
    { 0xf6f05d0a, SUC_lexicon_tags_47 },
    { 0x34f05d0b, SUC_lexicon_tags_10 },
    { 0xca905d0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0b05d0e, SUC_lexicon_tags_2 },
    { 0xb6905d0f, SUC_lexicon_tags_45 },
    { 0x9505d10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4b05d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b05d17, SUC_lexicon_tags_0 },
    { 0x2e705d18, SUC_lexicon_tags_37 },
    { 0xd3905d19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d705d1d, SUC_lexicon_tags_15 },
    { 0x1af05d1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf05d22, SUC_lexicon_tags_24 },
    { 0x8105d23, SUC_lexicon_tags_65 },
    { 0xefb05d23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f305d26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1eb05d28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8905d2a, SUC_lexicon_tags_9 },
    { 0x40105d2b, SUC_lexicon_tags_12 },
    { 0x46305d2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c505d2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55305d30, SUC_lexicon_tags_23 },
    { 0x56b05d31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a705d36, SUC_lexicon_tags_37 },
    { 0x94d05d37, SUC_lexicon_tags_0 },
    { 0x7d905d37, SUC_lexicon_tags_9 },
    { 0x93505d39, SUC_lexicon_tags_45 },
    { 0x31905d3a, SUC_lexicon_tags_13 },
    { 0xbd05d39, SUC_lexicon_tags_60 },
    { 0xc6d05d3c, SUC_lexicon_tags_27 },
    { 0x1ef05d3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x95b05d3f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d05d45, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b05d48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1905d4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b05d4d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1905d4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4df05d51, SUC_lexicon_tags_2 },
    { 0xd9305d51, SUC_lexicon_tags_1 },
    { 0xd6b05d53, SUC_lexicon_tags_29 },
    { 0xbfd05d54, SUC_lexicon_tags_2 },
    { 0xec305d54, SUC_lexicon_tags_2 },
    { 0x7dd05d55, SUC_lexicon_tags_29 },
    { 0x9e505d57, SUC_lexicon_tags_2 },
    { 0x11305d54, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xed505d5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af05d5e, SUC_lexicon_tags_1 },
    { 0xe7f05d5f, SUC_lexicon_tags_38 },
    { 0x5d105d60, SUC_lexicon_tags_11 },
    { 0xfdb05d60, SUC_lexicon_tags_45 },
    { 0x94d05d60, SUC_lexicon_tags_17 },
    { 0x2eb05d63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7f05d65, SUC_lexicon_tags_18 },
    { 0xe2305d65, SUC_lexicon_tags_25 },
    { 0xe3f05d65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e705d6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe0905d6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f05d70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a305d73, SUC_lexicon_tags_23 },
    { 0x6df05d73, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f05d79, SUC_lexicon_tags_4 },
    { 0x59905d79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff05d7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44105d82, SUC_lexicon_tags_17 },
    { 0x5305d82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1905d8c, SUC_lexicon_tags_23 },
    { 0x5cd05d8d, SUC_lexicon_tags_25 },
    { 0x55105d8e, SUC_lexicon_tags_43 },
    { 0xda705d8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a505d98, SUC_lexicon_tags_15 },
    { 0x3d905d98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3305d9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f05d9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e305da1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb905da4, SUC_lexicon_tags_2 },
    { 0x1f105da4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d05da9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x97905dac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98305dae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4305db4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb05db7, SUC_lexicon_tags_26 },
    { 0xd6105db7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2905dbc, SUC_lexicon_tags_1 },
    { 0xb6b05dbd, SUC_lexicon_tags_46 },
    { 0x2ab05dbd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8705dc1, SUC_lexicon_tags_31 },
    { 0x33505dc2, SUC_lexicon_tags_12 },
    { 0xb2b05dc3, SUC_lexicon_tags_15 },
    { 0xfb505dc4, SUC_lexicon_tags_37 },
    { 0xe0b05dc5, SUC_lexicon_tags_2 },
    { 0x71105dc6, SUC_lexicon_tags_25 },
    { 0x3e705dc4, SUC_lexicon_tags_0 },
    { 0xfcf05dc8, SUC_lexicon_tags_49 },
    { 0x6df05dc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51905dcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b05dce, SUC_lexicon_tags_179 },
    { 0x4cb05dce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf905dd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc905dd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21705ddd, SUC_lexicon_tags_14 },
    { 0xb9305dde, SUC_lexicon_tags_0 },
    { 0x5b05ddf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e705de4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8905deb, SUC_lexicon_tags_7 },
    { 0x2f705dec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce705df3, SUC_lexicon_tags_25 },
    { 0xa0905df4, SUC_lexicon_tags_23 },
    { 0xf505df5, SUC_lexicon_tags_7 },
    { 0x33f05df3, SUC_lexicon_tags_33 },
    { 0xfff05df7, SUC_lexicon_tags_38 },
    { 0x9ed05df8, SUC_lexicon_tags_10 },
    { 0x21305df6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b05dfc, SUC_lexicon_tags_27 },
    { 0x1f05dfd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72305e01, SUC_lexicon_tags_79 },
    { 0xea905e01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b05e0a, SUC_lexicon_tags_11 },
    { 0x8fb05e0a, SUC_lexicon_tags_16 },
    { 0xfe705e0a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5105e17, SUC_lexicon_tags_74 },
    { 0x49f05e17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63505e20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e305e23, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0xc2305e25, SUC_lexicon_tags_15 },
    { 0xccd05e26, SUC_lexicon_tags_41 },
    { 0xa2b05e26, SUC_lexicon_tags_14 },
    { 0xaf905e25, SUC_lexicon_tags_38 },
    { 0x78905e29, SUC_lexicon_tags_21 },
    { 0x11d05e2a, SUC_lexicon_tags_7 },
    { 0xced05e27, SUC_lexicon_tags_9 },
    { 0xfdf05e2c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaed05e2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7cd05e30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8f505e32, SUC_lexicon_tags_35 },
    { 0x76705e33, SUC_lexicon_tags_15 },
    { 0xa7f05e34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef705e39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69f05e3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f905e49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x21d05e4b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7905e4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1105e51, SUC_lexicon_tags_25 },
    { 0x43d05e52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64705e58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7905e5a, SUC_lexicon_tags_11 },
    { 0x7d05e5b, SUC_lexicon_tags_43 },
    { 0xbad05e5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87f05e5e, SUC_lexicon_tags_23 },
    { 0x6ed05e5e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3ed05e61, SUC_lexicon_tags_7 },
    { 0x7f705e62, SUC_lexicon_tags_12 },
    { 0x66b05e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5505e65, SUC_lexicon_tags_17 },
    { 0x22b05e66, SUC_lexicon_tags_17 },
    { 0xc4f05e65, SUC_lexicon_tags_55 },
    { 0x1fd05e65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d05e6b, SUC_lexicon_tags_47 },
    { 0xf4705e6b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c305e73, SUC_lexicon_tags_0 },
    { 0x36b05e73, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b05e7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6705e7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d05e80, SUC_lexicon_tags_11 },
    { 0x91b05e81, SUC_lexicon_tags_66 },
    { 0x92b05e80, SUC_lexicon_tags_6 },
    { 0x58505e83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57b05e85, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x87705e88, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e105e8b, SUC_lexicon_tags_11 },
    { 0x2b105e8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd05e8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10705e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb05ea1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa7905ea3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b505ea7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31705ea9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x59905eab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb505eae, SUC_lexicon_tags_77 },
    { 0x64d05eae, SUC_lexicon_tags_10 },
    { 0xceb05eae, SUC_lexicon_tags_17 },
    { 0x4ff05eb1, SUC_lexicon_tags_45 },
    { 0xb4705eb1, SUC_lexicon_tags_127 },
    { 0x7ed05eaf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa9f05eb5, SUC_lexicon_tags_37 },
    { 0xc8505eb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f05ebe, SUC_lexicon_tags_30 },
    { 0xcc305ebe, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb05ec4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2505ece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52305ed1, SUC_lexicon_tags_33 },
    { 0x1a905ed2, SUC_lexicon_tags_2 },
    { 0xdc905ed3, SUC_lexicon_tags_4 },
    { 0xc8d05ed2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c105ede, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48b05ee0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc7b05ee2, SUC_lexicon_tags_47 },
    { 0x5e305ee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f05ee7, SUC_lexicon_tags_45 },
    { 0x69d05ee8, SUC_lexicon_tags_56 },
    { 0xceb05ee9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x76705eeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3705eee, SUC_lexicon_tags_1 },
    { 0x19b05eee, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a705ef4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88305ef9, SUC_lexicon_tags_6 },
    { 0xbbf05ef9, SUC_lexicon_tags_7 },
    { 0xa8305efb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf05f04, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf105f08, SUC_lexicon_tags_28 },
    { 0x8ed05f09, SUC_lexicon_tags_27 },
    { 0xfd05f09, SUC_lexicon_tags_1 },
    { 0x8cd05f0b, SUC_lexicon_tags_17 },
    { 0x74105f0c, SUC_lexicon_tags_1 },
    { 0x6b105f0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83705f0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d05f13, SUC_lexicon_tags_28 },
    { 0xb0105f14, SUC_lexicon_tags_1 },
    { 0x69905f15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b05f18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4505f1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8105f22, SUC_lexicon_tags_37 },
    { 0xf0305f23, SUC_lexicon_tags_11 },
    { 0x2505f22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x96705f27, SUC_lexicon_tags_21 },
    { 0xa6705f27, SUC_lexicon_tags_12 },
    { 0xd905f29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x35905f2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4b05f2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9105f2f, SUC_lexicon_tags_4 },
    { 0x7ab05f30, SUC_lexicon_tags_2 },
    { 0xd9d05f31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23f05f33, SUC_lexicon_tags_8 },
    { 0x27305f34, SUC_lexicon_tags_17 },
    { 0x81105f35, SUC_lexicon_tags_11 },
    { 0x1dd05f36, SUC_lexicon_tags_46 },
    { 0xd1705f37, SUC_lexicon_tags_25 },
    { 0xcf05f36, SUC_lexicon_tags_25 },
    { 0xb0505f39, SUC_lexicon_tags_43 },
    { 0xe2905f3a, SUC_lexicon_tags_11 },
    { 0xbb105f33, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd05f3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21505f40, SUC_lexicon_tags_32 },
    { 0x54f05f41, SUC_lexicon_tags_27 },
    { 0xbf705f40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d05f46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf05f49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7505f4b, SUC_lexicon_tags_87 },
    { 0x29305f4c, SUC_lexicon_tags_38 },
    { 0xaef05f4c, SUC_lexicon_tags_7 },
    { 0x69705f4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d05f52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde705f56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b05f5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75905f5f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c705f62, SUC_lexicon_tags_1 },
    { 0x24b05f63, SUC_lexicon_tags_57 },
    { 0x53d05f63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x67105f67, SUC_lexicon_tags_11 },
    { 0x50505f68, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b05f6c, SUC_lexicon_tags_45 },
    { 0xfd305f6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5305f70, SUC_lexicon_tags_32 },
    { 0xa9705f70, SUC_lexicon_tags_2 },
    { 0x61305f72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6705f74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf305f77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17905f79, SUC_lexicon_tags_2 },
    { 0x3bd05f7a, SUC_lexicon_tags_54 },
    { 0xac305f79, SUC_lexicon_tags_17 },
    { 0xcad05f7a, SUC_lexicon_tags_45 },
    { 0x53d05f7a, SUC_lexicon_tags_56 },
    { 0xb6105f7e, SUC_lexicon_tags_12 },
    { 0x73b05f7f, SUC_lexicon_tags_11 },
    { 0xe8705f80, SUC_lexicon_tags_49 },
    { 0xbe305f81, SUC_lexicon_tags_8 },
    { 0x5ab05f82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcb05f84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18305f87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4e505f89, SUC_lexicon_tags_7 },
    { 0x4505f8a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf705f8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d05f95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b905f97, SUC_lexicon_tags_12 },
    { 0xa105f97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4705f9b, SUC_lexicon_tags_138 },
    { 0x4f705f9b, SUC_lexicon_tags_82 },
    { 0x33105f9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4dd05f9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e105faa, SUC_lexicon_tags_37 },
    { 0xb3b05faa, SUC_lexicon_tags_12 },
    { 0xc1b05fab, SUC_lexicon_tags_15 },
    { 0x72305fac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc1b05faf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f705fb2, SUC_lexicon_tags_9 },
    { 0x45105fb3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x57f05fb5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd705fb9, SUC_lexicon_tags_63 },
    { 0xc3905fb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b05fbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1105fbf, SUC_lexicon_tags_15 },
    { 0x5f505fc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df05fc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf105fcb, SUC_lexicon_tags_33 },
    { 0xfd905fcb, SUC_lexicon_tags_66 },
    { 0x9305fcd, SUC_lexicon_tags_6 },
    { 0xacf05fce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd105fd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86505fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed105fd8, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xea505fda, SUC_lexicon_tags_49 },
    { 0x6705fda, SUC_lexicon_tags_12 },
    { 0x5bb05fdc, SUC_lexicon_tags_2 },
    { 0x37d05fdd, SUC_lexicon_tags_8 },
    { 0xc1f05fde, SUC_lexicon_tags_63 },
    { 0xcb505fda, SUC_lexicon_tags_25 },
    { 0xadd05fe0, SUC_lexicon_tags_27 },
    { 0xc7905fe1, SUC_lexicon_tags_22 },
    { 0xa7305fe0, SUC_lexicon_tags_28 },
    { 0xa6f05fdb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d305fe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce505fee, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed05ff1, SUC_lexicon_tags_27 },
    { 0x3e105ff2, SUC_lexicon_tags_37 },
    { 0x8a905ff3, SUC_lexicon_tags_17 },
    { 0xa5d05ff3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25b05ff6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa305ffa, SUC_lexicon_tags_7 },
    { 0xb3505ffb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94506001, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb06004, SUC_lexicon_tags_45 },
    { 0x76706004, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5f06007, SUC_lexicon_tags_68 },
    { 0x23f06008, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c70600b, SUC_lexicon_tags_38 },
    { 0xfc90600c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9990600e, SUC_lexicon_tags_15 },
    { 0x1590600f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1306011, SUC_lexicon_tags_6 },
    { 0x40506012, SUC_lexicon_tags_15 },
    { 0xb1906013, SUC_lexicon_tags_37 },
    { 0x1af06014, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d06019, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5b0601b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4906025, SUC_lexicon_tags_75 },
    { 0x93d06026, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc650602a, SUC_lexicon_tags_15 },
    { 0xb50602a, SUC_lexicon_tags_1 },
    { 0x1250602c, SUC_lexicon_tags_12 },
    { 0x1d90602c, SUC_lexicon_tags_165 },
    { 0x85f0602e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5306030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f06033, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x57306035, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c106039, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9890603b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf710603d, SUC_lexicon_tags_37 },
    { 0x9350603e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f06042, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb9f06044, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c906046, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee906048, SUC_lexicon_tags_43 },
    { 0xaf06048, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xe5f0604b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xb430604e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x75106050, SUC_lexicon_tags_26 },
    { 0x7cd06051, SUC_lexicon_tags_9 },
    { 0x40306052, SUC_lexicon_tags_18 },
    { 0xe6506053, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb4306055, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d06058, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5410605e, SUC_lexicon_tags_17 },
    { 0xac10605e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf0d06061, SUC_lexicon_tags_27 },
    { 0x4b06062, SUC_lexicon_tags_1 },
    { 0x2cd06061, SUC_lexicon_tags_1 },
    { 0x13506061, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d06067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x610606d, SUC_lexicon_tags_192 },
    { 0x8b10606e, SUC_lexicon_tags_30 },
    { 0x1690606f, SUC_lexicon_tags_38 },
    { 0x9070606d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d06077, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf270607c, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf06080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b06088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8710608b, SUC_lexicon_tags_13 },
    { 0x96d0608c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc450608e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b06091, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b506099, SUC_lexicon_tags_105 },
    { 0x7490609a, SUC_lexicon_tags_25 },
    { 0xd4f0609a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfad0609d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf060a1, SUC_lexicon_tags_15 },
    { 0xafd060a1, SUC_lexicon_tags_17 },
    { 0x461060a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05060ac, SUC_lexicon_tags_31 },
    { 0x93b060ad, SUC_lexicon_tags_26 },
    { 0xf7d060ad, SUC_lexicon_tags_29 },
    { 0x92b060af, SUC_lexicon_tags_11 },
    { 0x7a5060b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491060b6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6f1060b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d060bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86f060be, SUC_lexicon_tags_0 },
    { 0x3c1060bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffb060c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x121060c3, SUC_lexicon_tags_37 },
    { 0x30b060c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7060c8, SUC_lexicon_tags_37 },
    { 0xbbf060c9, SUC_lexicon_tags_33 },
    { 0x213060ca, SUC_lexicon_tags_33 },
    { 0xc71060cb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5bd060cd, SUC_lexicon_tags_1 },
    { 0xa73060cd, SUC_lexicon_tags_14 },
    { 0x8bf060ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d060d2, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d060d5, SUC_lexicon_tags_47 },
    { 0x391060d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85060da, SUC_lexicon_tags_23 },
    { 0x221060db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f060de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb53060e0, SUC_lexicon_tags_14 },
    { 0xb45060e1, SUC_lexicon_tags_37 },
    { 0x5c7060e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61060e8, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b060eb, SUC_lexicon_tags_47 },
    { 0xd53060ec, SUC_lexicon_tags_23 },
    { 0xb15060ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd5060ef, SUC_lexicon_tags_55 },
    { 0x54b060f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673060f5, SUC_lexicon_tags_15 },
    { 0x433060f5, SUC_lexicon_tags_4 },
    { 0xb3060f6, SUC_lexicon_tags_17 },
    { 0x43b060f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x237060fa, SUC_lexicon_tags_2 },
    { 0x7c3060fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x299060fd, SUC_lexicon_tags_31 },
    { 0x4e7060fd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f06101, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58306105, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x79506108, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53106111, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20306115, SUC_lexicon_tags_35 },
    { 0x78506116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b0611b, SUC_lexicon_tags_14 },
    { 0x3470611c, SUC_lexicon_tags_2 },
    { 0x96b0611d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb06126, SUC_lexicon_tags_37 },
    { 0xf9306126, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b706129, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a30612c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3870612f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b906131, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40d06133, SUC_lexicon_tags_39 },
    { 0x706133, SUC_lexicon_tags_25 },
    { 0x39306134, SUC_lexicon_tags_11 },
    { 0x36306135, SUC_lexicon_tags_25 },
    { 0xdf906136, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6790613e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12106142, SUC_lexicon_tags_8 },
    { 0x67f06143, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x35706145, SUC_lexicon_tags_14 },
    { 0x6bd06146, SUC_lexicon_tags_33 },
    { 0x11d06145, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd0614a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82306153, SUC_lexicon_tags_26 },
    { 0x9e506153, SUC_lexicon_tags_12 },
    { 0xf506155, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad706158, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd0615b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27106160, SUC_lexicon_tags_468 },
    { 0x38906161, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcd106163, SUC_lexicon_tags_38 },
    { 0x74506164, SUC_lexicon_tags_7 },
    { 0x4e706163, SUC_lexicon_tags_37 },
    { 0x81906163, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e306169, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1050616d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f906175, SUC_lexicon_tags_37 },
    { 0x47106175, SUC_lexicon_tags_17 },
    { 0x42d06176, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5690617c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f30617f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d06182, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xceb06184, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde906189, SUC_lexicon_tags_7 },
    { 0xea50618a, SUC_lexicon_tags_1 },
    { 0xb3b0618a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50618e, SUC_lexicon_tags_49 },
    { 0xeff0618f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82906191, SUC_lexicon_tags_8 },
    { 0xa5106192, SUC_lexicon_tags_32 },
    { 0x2b506193, SUC_lexicon_tags_11 },
    { 0xcf706193, SUC_lexicon_tags_11 },
    { 0x47706191, SUC_lexicon_tags_0 },
    { 0x9a706196, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21d06198, SUC_lexicon_tags_2 },
    { 0x87306198, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe350619e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x505061a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f061a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd43061a9, SUC_lexicon_tags_37 },
    { 0xa81061aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x441061ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f061af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9061b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd061b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x299061b8, SUC_lexicon_tags_4 },
    { 0x965061b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb01061bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x407061be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1061c2, SUC_lexicon_tags_37 },
    { 0x8a7061c3, SUC_lexicon_tags_87 },
    { 0x2c1061c2, SUC_lexicon_tags_8 },
    { 0x577061c5, SUC_lexicon_tags_15 },
    { 0xb47061c6, SUC_lexicon_tags_2 },
    { 0xd37061c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1061ca, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd061d5, SUC_lexicon_tags_39 },
    { 0xfed061d5, SUC_lexicon_tags_66 },
    { 0xeed061d6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9061da, SUC_lexicon_tags_39 },
    { 0x7c1061da, SUC_lexicon_tags_92 },
    { 0xadf061dc, SUC_lexicon_tags_1 },
    { 0x56f061da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x115061e0, SUC_lexicon_tags_25 },
    { 0x677061e1, SUC_lexicon_tags_23 },
    { 0x2e9061e2, SUC_lexicon_tags_1 },
    { 0x765061e3, SUC_lexicon_tags_30 },
    { 0x9b9061e4, SUC_lexicon_tags_37 },
    { 0x59d061e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b061ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa01061ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x473061ee, SUC_lexicon_tags_21 },
    { 0xad3061ef, SUC_lexicon_tags_82 },
    { 0xff1061f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb49061f3, SUC_lexicon_tags_12 },
    { 0xf45061f4, SUC_lexicon_tags_11 },
    { 0x523061f4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff061f9, SUC_lexicon_tags_7 },
    { 0x4c7061fa, SUC_lexicon_tags_11 },
    { 0x825061fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b061ff, SUC_lexicon_tags_11 },
    { 0xb0d06200, SUC_lexicon_tags_26 },
    { 0xde906200, SUC_lexicon_tags_0 },
    { 0x9e106200, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5906206, SUC_lexicon_tags_25 },
    { 0xea106206, SUC_lexicon_tags_12 },
    { 0x9b906207, SUC_lexicon_tags_23 },
    { 0xffb06209, SUC_lexicon_tags_45 },
    { 0xd8d06207, SUC_lexicon_tags_7 },
    { 0xfe506207, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd390620d, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d06210, SUC_lexicon_tags_12 },
    { 0xe0706211, SUC_lexicon_tags_25 },
    { 0x53906212, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d06216, SUC_lexicon_tags_98 },
    { 0x76706217, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0b06219, SUC_lexicon_tags_55 },
    { 0x5c50621a, SUC_lexicon_tags_63 },
    { 0xb450621b, SUC_lexicon_tags_1 },
    { 0x6fb0621b, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f906223, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x51f06225, SUC_lexicon_tags_55 },
    { 0x27506226, SUC_lexicon_tags_31 },
    { 0x26906227, SUC_lexicon_tags_8 },
    { 0x54106228, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf250622a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b10622d, SUC_lexicon_tags_21 },
    { 0x5d10622d, SUC_lexicon_tags_87 },
    { 0x7a70622f, SUC_lexicon_tags_63 },
    { 0xd110622f, SUC_lexicon_tags_31 },
    { 0x4506231, SUC_lexicon_tags_7 },
    { 0x12b06232, SUC_lexicon_tags_17 },
    { 0x6a30622e, SUC_lexicon_tags_7 },
    { 0xd4b06234, SUC_lexicon_tags_37 },
    { 0x92306235, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7df06237, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7106239, SUC_lexicon_tags_11 },
    { 0xd1d0623a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b06240, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f506242, SUC_lexicon_tags_46 },
    { 0x53f06242, SUC_lexicon_tags_11 },
    { 0x16106244, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b06247, SUC_lexicon_tags_61 },
    { 0xb4506247, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa650624a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e90624d, SUC_lexicon_tags_28 },
    { 0x80b0624e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x30706250, SUC_lexicon_tags_11 },
    { 0x24706250, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f06256, SUC_lexicon_tags_25 },
    { 0xfc906257, SUC_lexicon_tags_60 },
    { 0xa0306258, SUC_lexicon_tags_47 },
    { 0x93506259, SUC_lexicon_tags_3 },
    { 0x47106259, SUC_lexicon_tags_25 },
    { 0xfaf06259, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd810625e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6906261, SUC_lexicon_tags_4 },
    { 0xc1706262, SUC_lexicon_tags_7 },
    { 0x4fd06263, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf306265, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x86906267, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4fb06269, SUC_lexicon_tags_0 },
    { 0x4390626a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e306270, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50627e, SUC_lexicon_tags_17 },
    { 0xabb0627e, SUC_lexicon_tags_25 },
    { 0xa5506280, SUC_lexicon_tags_60 },
    { 0xcc70627f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13706285, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3906288, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b0628e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6106291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f306296, SUC_lexicon_tags_43 },
    { 0x19b06296, SUC_lexicon_tags_11 },
    { 0xebb06298, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2650629a, SUC_lexicon_tags_15 },
    { 0xb310629b, SUC_lexicon_tags_661 },
    { 0x7010629c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead062a1, SUC_lexicon_tags_11 },
    { 0x1e3062a2, SUC_lexicon_tags_47 },
    { 0x42d062a1, SUC_lexicon_tags_12 },
    { 0x59d062a2, SUC_lexicon_tags_17 },
    { 0x48b062a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517062ae, SUC_lexicon_tags_49 },
    { 0xc87062af, SUC_lexicon_tags_287 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d062b2, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795062ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543062c1, SUC_lexicon_tags_19 },
    { 0x55062c2, SUC_lexicon_tags_15 },
    { 0xc73062c2, SUC_lexicon_tags_1 },
    { 0xab7062c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b062c8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5062cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d062ce, SUC_lexicon_tags_11 },
    { 0x7bf062cf, SUC_lexicon_tags_43 },
    { 0xae3062d0, SUC_lexicon_tags_27 },
    { 0xd2d062ce, SUC_lexicon_tags_15 },
    { 0x84d062cf, SUC_lexicon_tags_12 },
    { 0xbed062d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3062d5, SUC_lexicon_tags_43 },
    { 0xb0b062d6, SUC_lexicon_tags_127 },
    { 0x9dd062d7, SUC_lexicon_tags_23 },
    { 0xbb7062d8, SUC_lexicon_tags_27 },
    { 0x247062d8, SUC_lexicon_tags_1 },
    { 0xd6d062da, SUC_lexicon_tags_7 },
    { 0x889062db, SUC_lexicon_tags_29 },
    { 0x9c7062d6, SUC_lexicon_tags_25 },
    { 0x31b062dd, SUC_lexicon_tags_11 },
    { 0x85062de, SUC_lexicon_tags_31 },
    { 0x5fb062d6, SUC_lexicon_tags_37 },
    { 0x749062e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67062e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x733062e8, SUC_lexicon_tags_19 },
    { 0x2a5062e8, SUC_lexicon_tags_24 },
    { 0xcb1062e8, SUC_lexicon_tags_68 },
    { 0xc8d062eb, SUC_lexicon_tags_30 },
    { 0x349062ec, SUC_lexicon_tags_0 },
    { 0x855062ed, SUC_lexicon_tags_25 },
    { 0xf55062ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b062f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b062f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869062f9, SUC_lexicon_tags_6 },
    { 0x565062fa, SUC_lexicon_tags_45 },
    { 0xe25062fb, SUC_lexicon_tags_31 },
    { 0x21d062fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49506300, SUC_lexicon_tags_23 },
    { 0xc8706301, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f06304, SUC_lexicon_tags_3 },
    { 0x99f06305, SUC_lexicon_tags_17 },
    { 0x35106305, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0506309, SUC_lexicon_tags_37 },
    { 0x106309, SUC_lexicon_tags_1 },
    { 0x8870630b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5330630e, SUC_lexicon_tags_12 },
    { 0x6c50630f, SUC_lexicon_tags_35 },
    { 0x6f30630f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe306312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f506318, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8890631b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9db0631d, SUC_lexicon_tags_15 },
    { 0xccf0631e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf06321, SUC_lexicon_tags_1480 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f06324, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfb706326, SUC_lexicon_tags_11 },
    { 0xda506326, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a10632a, SUC_lexicon_tags_12 },
    { 0x6ff0632b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4ef0632d, SUC_lexicon_tags_19 },
    { 0xba30632d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33906331, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5f06333, SUC_lexicon_tags_14 },
    { 0xe0706334, SUC_lexicon_tags_77 },
    { 0xd8d06334, SUC_lexicon_tags_23 },
    { 0x9ab06333, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d0633b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3090633d, SUC_lexicon_tags_29 },
    { 0x1290633d, SUC_lexicon_tags_37 },
    { 0xfd90633d, SUC_lexicon_tags_30 },
    { 0x23b0633f, SUC_lexicon_tags_12 },
    { 0x13d06340, SUC_lexicon_tags_15 },
    { 0x1e106342, SUC_lexicon_tags_15 },
    { 0xf2d06340, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28706347, SUC_lexicon_tags_9 },
    { 0x5c106348, SUC_lexicon_tags_25 },
    { 0x65b06347, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb410634f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcbd06351, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df06355, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2950635e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3906360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd06362, SUC_lexicon_tags_15 },
    { 0x6d506362, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x55506366, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1150636b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa430636f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x88106371, SUC_lexicon_tags_63 },
    { 0xb0f06371, SUC_lexicon_tags_7 },
    { 0xc906371, SUC_lexicon_tags_33 },
    { 0x38f06374, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cd06376, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1e906378, SUC_lexicon_tags_37 },
    { 0x67906378, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc210637b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b0637f, SUC_lexicon_tags_17 },
    { 0x5e506380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3106385, SUC_lexicon_tags_37 },
    { 0xabf06385, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25706388, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb706393, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb06399, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1063a0, SUC_lexicon_tags_25 },
    { 0x5d1063a1, SUC_lexicon_tags_63 },
    { 0x869063a2, SUC_lexicon_tags_25 },
    { 0x709063a2, SUC_lexicon_tags_17 },
    { 0xa45063a1, SUC_lexicon_tags_26 },
    { 0xed3063a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x601063a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113063b1, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55063b5, SUC_lexicon_tags_14 },
    { 0xf25063b6, SUC_lexicon_tags_25 },
    { 0x9b3063b6, SUC_lexicon_tags_12 },
    { 0x42b063b6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b063bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1063c3, SUC_lexicon_tags_2 },
    { 0xea3063c4, SUC_lexicon_tags_21 },
    { 0xbf5063c5, SUC_lexicon_tags_37 },
    { 0xafb063c5, SUC_lexicon_tags_15 },
    { 0xebf063c6, SUC_lexicon_tags_3 },
    { 0x827063c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73063cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9063cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31063d2, SUC_lexicon_tags_60 },
    { 0x5f9063d3, SUC_lexicon_tags_9 },
    { 0x1d3063d2, SUC_lexicon_tags_38 },
    { 0xb81063d5, SUC_lexicon_tags_7 },
    { 0x7eb063d6, SUC_lexicon_tags_47 },
    { 0xa87063d6, SUC_lexicon_tags_12 },
    { 0x89d063d8, SUC_lexicon_tags_15 },
    { 0x71f063d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13063dc, SUC_lexicon_tags_9 },
    { 0xc59063dd, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9063e0, SUC_lexicon_tags_2 },
    { 0xe9d063e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b063eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd063ef, SUC_lexicon_tags_2 },
    { 0xd0d063ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff063f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x461063f5, SUC_lexicon_tags_24 },
    { 0xabb063f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1063f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749063fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6506404, SUC_lexicon_tags_37 },
    { 0x98906405, SUC_lexicon_tags_17 },
    { 0xf4306404, SUC_lexicon_tags_1 },
    { 0x66706407, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9d06409, SUC_lexicon_tags_60 },
    { 0x5e106409, SUC_lexicon_tags_3 },
    { 0xa7106409, SUC_lexicon_tags_19 },
    { 0xd510640b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd2b0640e, SUC_lexicon_tags_11 },
    { 0xcaf0640f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab06412, SUC_lexicon_tags_0 },
    { 0xf1306412, SUC_lexicon_tags_19 },
    { 0xaf06414, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc906418, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4450641d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f06423, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab06426, SUC_lexicon_tags_15 },
    { 0x5dd06427, SUC_lexicon_tags_2 },
    { 0x60906426, SUC_lexicon_tags_23 },
    { 0x6c706427, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x970642b, SUC_lexicon_tags_37 },
    { 0xe2d0642b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65d0642e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51906433, SUC_lexicon_tags_29 },
    { 0x14306434, SUC_lexicon_tags_23 },
    { 0xc9906435, SUC_lexicon_tags_63 },
    { 0xd1706435, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe706438, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x9610643b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e506440, SUC_lexicon_tags_6 },
    { 0xc7906441, SUC_lexicon_tags_19 },
    { 0x5fb06442, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed06447, SUC_lexicon_tags_44 },
    { 0x78906447, SUC_lexicon_tags_37 },
    { 0x4306449, SUC_lexicon_tags_7 },
    { 0x55906449, SUC_lexicon_tags_17 },
    { 0x14b0644b, SUC_lexicon_tags_8 },
    { 0x590644a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4850644e, SUC_lexicon_tags_46 },
    { 0x20d0644f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7706452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61706454, SUC_lexicon_tags_127 },
    { 0x58906455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78b06457, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc906468, SUC_lexicon_tags_57 },
    { 0x76506469, SUC_lexicon_tags_15 },
    { 0x16106469, SUC_lexicon_tags_14 },
    { 0xed0646b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a90646d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x86f0646f, SUC_lexicon_tags_2 },
    { 0x8d306470, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70106474, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf06478, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8770647c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c706484, SUC_lexicon_tags_92 },
    { 0x64106485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f506487, SUC_lexicon_tags_15 },
    { 0x2b106488, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x170648a, SUC_lexicon_tags_39 },
    { 0x70f0648b, SUC_lexicon_tags_46 },
    { 0x54b0648b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e30648f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc706491, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d06496, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6130649a, SUC_lexicon_tags_7 },
    { 0x84f0649a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb90649d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd90649f, SUC_lexicon_tags_3 },
    { 0xa07064a0, SUC_lexicon_tags_88 },
    { 0x17f0649f, SUC_lexicon_tags_15 },
    { 0xbeb064a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5064a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x251064aa, SUC_lexicon_tags_25 },
    { 0x2ad064ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163064af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5064b9, SUC_lexicon_tags_56 },
    { 0x7dd064ba, SUC_lexicon_tags_37 },
    { 0x4d064ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135064c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe7f064c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9064cd, SUC_lexicon_tags_1 },
    { 0xccf064ce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d064d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b7064d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35064d9, SUC_lexicon_tags_74 },
    { 0x8af064da, SUC_lexicon_tags_2 },
    { 0x72d064da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9064df, SUC_lexicon_tags_0 },
    { 0xfab064df, SUC_lexicon_tags_9 },
    { 0xe5064df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33064e4, SUC_lexicon_tags_17 },
    { 0x82b064e5, SUC_lexicon_tags_10 },
    { 0x431064e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1064ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc81064f0, SUC_lexicon_tags_49 },
    { 0x90b064f1, SUC_lexicon_tags_14 },
    { 0x87f064f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97064f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xadd064f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7064fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead06503, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8706509, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5490650b, SUC_lexicon_tags_17 },
    { 0x3af0650b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf110650e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9906512, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc506517, SUC_lexicon_tags_28 },
    { 0x37b06517, SUC_lexicon_tags_39 },
    { 0x14306519, SUC_lexicon_tags_12 },
    { 0xc9706519, SUC_lexicon_tags_481 },
    { 0x4b0651b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d70651e, SUC_lexicon_tags_37 },
    { 0xa390651f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fb06521, SUC_lexicon_tags_37 },
    { 0x8a906522, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b06526, SUC_lexicon_tags_85 },
    { 0xd0f06526, SUC_lexicon_tags_14 },
    { 0xb306526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50652c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd06530, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x18d06532, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa430653a, SUC_lexicon_tags_15 },
    { 0x8810653a, SUC_lexicon_tags_37 },
    { 0x5f30653c, SUC_lexicon_tags_17 },
    { 0xe870653d, SUC_lexicon_tags_2 },
    { 0xbb50653e, SUC_lexicon_tags_55 },
    { 0x7630653d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30f06541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea906544, SUC_lexicon_tags_25 },
    { 0x17906545, SUC_lexicon_tags_17 },
    { 0x1106546, SUC_lexicon_tags_37 },
    { 0xe506544, SUC_lexicon_tags_2 },
    { 0x99306548, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e90654e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38306552, SUC_lexicon_tags_63 },
    { 0xddf06552, SUC_lexicon_tags_2 },
    { 0xb0d06554, SUC_lexicon_tags_74 },
    { 0x65706554, SUC_lexicon_tags_5 },
    { 0xe2106555, SUC_lexicon_tags_32 },
    { 0xc7106557, SUC_lexicon_tags_30 },
    { 0x54506554, SUC_lexicon_tags_29 },
    { 0x8e506559, SUC_lexicon_tags_25 },
    { 0xbcd06553, SUC_lexicon_tags_5 },
    { 0x4090655b, SUC_lexicon_tags_25 },
    { 0x86f0655b, SUC_lexicon_tags_26 },
    { 0xad30655b, SUC_lexicon_tags_1 },
    { 0x4b0655e, SUC_lexicon_tags_19 },
    { 0xd30655e, SUC_lexicon_tags_7 },
    { 0x90906559, SUC_lexicon_tags_23 },
    { 0xe6d06561, SUC_lexicon_tags_37 },
    { 0xb1706561, SUC_lexicon_tags_11 },
    { 0x2bf06563, SUC_lexicon_tags_12 },
    { 0xc7306562, SUC_lexicon_tags_274 },
    { 0x54f0655c, SUC_lexicon_tags_37 },
    { 0xcdd06566, SUC_lexicon_tags_37 },
    { 0x61d06559, SUC_lexicon_tags_14 },
    { 0xaa306559, SUC_lexicon_tags_198 },
    { 0x3c306569, SUC_lexicon_tags_38 },
    { 0x81b06569, SUC_lexicon_tags_25 },
    { 0xb030656b, SUC_lexicon_tags_46 },
    { 0x74d06569, SUC_lexicon_tags_25 },
    { 0x6710656b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb06570, SUC_lexicon_tags_12 },
    { 0xce506570, SUC_lexicon_tags_25 },
    { 0x3f706572, SUC_lexicon_tags_4 },
    { 0x57306573, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b106576, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9306579, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7510657c, SUC_lexicon_tags_2 },
    { 0x4d10657d, SUC_lexicon_tags_1 },
    { 0x430657d, SUC_lexicon_tags_17 },
    { 0xa430657d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a306582, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xe8906584, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x35506586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6410658f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0306593, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf610659b, SUC_lexicon_tags_47 },
    { 0x6a70659c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2610659f, SUC_lexicon_tags_47 },
    { 0x9af065a0, SUC_lexicon_tags_27 },
    { 0xa1065a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x805065a4, SUC_lexicon_tags_12 },
    { 0x4f1065a5, SUC_lexicon_tags_55 },
    { 0x255065a6, SUC_lexicon_tags_29 },
    { 0xef1065a7, SUC_lexicon_tags_1 },
    { 0x81065a8, SUC_lexicon_tags_29 },
    { 0xa3f065a9, SUC_lexicon_tags_19 },
    { 0xfd9065a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28f065ac, SUC_lexicon_tags_6 },
    { 0x319065ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb4d065af, SUC_lexicon_tags_14 },
    { 0x5a7065b0, SUC_lexicon_tags_19 },
    { 0xc0d065b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7065b4, SUC_lexicon_tags_37 },
    { 0x839065b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9065bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x835065c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd95065c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa33065c4, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x449065c6, SUC_lexicon_tags_12 },
    { 0x577065c7, SUC_lexicon_tags_33 },
    { 0x3e9065c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d065cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad7065ce, SUC_lexicon_tags_17 },
    { 0x621065cf, SUC_lexicon_tags_0 },
    { 0xb09065d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979065d4, SUC_lexicon_tags_12 },
    { 0xc1f065d5, SUC_lexicon_tags_17 },
    { 0x917065d4, SUC_lexicon_tags_6 },
    { 0xe93065d7, SUC_lexicon_tags_56 },
    { 0xfb1065d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b065dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8f7065df, SUC_lexicon_tags_35 },
    { 0xe3d065e0, SUC_lexicon_tags_11 },
    { 0xfe7065e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1dd065e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1065e6, SUC_lexicon_tags_63 },
    { 0x637065e7, SUC_lexicon_tags_55 },
    { 0x583065e7, SUC_lexicon_tags_65 },
    { 0x361065e9, SUC_lexicon_tags_35 },
    { 0x915065ea, SUC_lexicon_tags_28 },
    { 0xcc9065eb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb39065ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5065ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x46f065f1, SUC_lexicon_tags_7 },
    { 0x5f1065f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x797065f4, SUC_lexicon_tags_0 },
    { 0x5b7065f4, SUC_lexicon_tags_14 },
    { 0x3fb065f6, SUC_lexicon_tags_21 },
    { 0x6a3065f7, SUC_lexicon_tags_27 },
    { 0x8a1065f5, SUC_lexicon_tags_19 },
    { 0x2e5065f9, SUC_lexicon_tags_37 },
    { 0xbef065fa, SUC_lexicon_tags_19 },
    { 0x9b3065fa, SUC_lexicon_tags_10 },
    { 0x721065f8, SUC_lexicon_tags_13 },
    { 0x523065fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c706600, SUC_lexicon_tags_46 },
    { 0x2b906601, SUC_lexicon_tags_17 },
    { 0x5906602, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15706605, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb06609, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f0660d, SUC_lexicon_tags_10 },
    { 0xf070660e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf06613, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef06616, SUC_lexicon_tags_37 },
    { 0xb8906616, SUC_lexicon_tags_2 },
    { 0x46306617, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1010661a, SUC_lexicon_tags_12 },
    { 0x3d90661a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4906620, SUC_lexicon_tags_2 },
    { 0x7ef06621, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8706623, SUC_lexicon_tags_87 },
    { 0xa906623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2b06626, SUC_lexicon_tags_12 },
    { 0x5f506627, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8870662b, SUC_lexicon_tags_37 },
    { 0xcb50662c, SUC_lexicon_tags_14 },
    { 0xae10662d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff90662f, SUC_lexicon_tags_65 },
    { 0x68306630, SUC_lexicon_tags_8 },
    { 0xcfd06631, SUC_lexicon_tags_14 },
    { 0x93906632, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88506638, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a10663c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47706640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b06644, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d06647, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8ef06649, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f90664d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd270664f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d306651, SUC_lexicon_tags_45 },
    { 0xfe106651, SUC_lexicon_tags_15 },
    { 0x37506653, SUC_lexicon_tags_15 },
    { 0xdd306654, SUC_lexicon_tags_43 },
    { 0xa4d06654, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d0665c, SUC_lexicon_tags_25 },
    { 0x7b90665d, SUC_lexicon_tags_17 },
    { 0xdbb0665e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x45906660, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde906664, SUC_lexicon_tags_37 },
    { 0x91306664, SUC_lexicon_tags_49 },
    { 0x86306666, SUC_lexicon_tags_11 },
    { 0xf0d06667, SUC_lexicon_tags_37 },
    { 0x12506668, SUC_lexicon_tags_47 },
    { 0xe9d06669, SUC_lexicon_tags_25 },
    { 0x71f06669, SUC_lexicon_tags_25 },
    { 0x29f06666, SUC_lexicon_tags_45 },
    { 0x96306667, SUC_lexicon_tags_49 },
    { 0xa5706666, SUC_lexicon_tags_28 },
    { 0x6930666e, SUC_lexicon_tags_27 },
    { 0xaa30666f, SUC_lexicon_tags_79 },
    { 0xe3906668, SUC_lexicon_tags_47 },
    { 0x5e70666b, SUC_lexicon_tags_55 },
    { 0x39f06672, SUC_lexicon_tags_37 },
    { 0x1e706672, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6170667a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x30667c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc150667e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c706682, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad506686, SUC_lexicon_tags_39 },
    { 0x93506687, SUC_lexicon_tags_1 },
    { 0x10b06688, SUC_lexicon_tags_37 },
    { 0x31706687, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2d0668b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4250668d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c706691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11306696, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfc106698, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe7b0669a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc230669d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b066a4, SUC_lexicon_tags_11 },
    { 0xa1b066a5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d066a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33b066ab, SUC_lexicon_tags_39 },
    { 0x8ab066ac, SUC_lexicon_tags_39 },
    { 0x2fd066ad, SUC_lexicon_tags_25 },
    { 0xf4d066ad, SUC_lexicon_tags_7 },
    { 0x86d066af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x94b066b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x715066b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedb066b5, SUC_lexicon_tags_12 },
    { 0x49d066b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd95066b8, SUC_lexicon_tags_29 },
    { 0x3c1066b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x591066bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad1066bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xba3066bf, SUC_lexicon_tags_7 },
    { 0x2c7066c0, SUC_lexicon_tags_11 },
    { 0x381066bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff066c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x12f066c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe25066c8, SUC_lexicon_tags_82 },
    { 0x2dd066c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe1f066cb, SUC_lexicon_tags_1 },
    { 0xbb7066cb, SUC_lexicon_tags_43 },
    { 0x7ad066cd, SUC_lexicon_tags_29 },
    { 0xd9d066ce, SUC_lexicon_tags_1 },
    { 0xf85066cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f066d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d066db, SUC_lexicon_tags_14 },
    { 0x1f1066dc, SUC_lexicon_tags_12 },
    { 0xcf3066dd, SUC_lexicon_tags_3 },
    { 0x395066de, SUC_lexicon_tags_2 },
    { 0xbb5066de, SUC_lexicon_tags_17 },
    { 0x1ff066e0, SUC_lexicon_tags_2 },
    { 0x459066e1, SUC_lexicon_tags_30 },
    { 0xbf1066e0, SUC_lexicon_tags_33 },
    { 0xc39066dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b066ea, SUC_lexicon_tags_6 },
    { 0x847066eb, SUC_lexicon_tags_38 },
    { 0x27b066ec, SUC_lexicon_tags_31 },
    { 0x317066ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3066f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97066f5, SUC_lexicon_tags_37 },
    { 0x3e3066f6, SUC_lexicon_tags_28 },
    { 0x825066f6, SUC_lexicon_tags_11 },
    { 0x8e7066f8, SUC_lexicon_tags_37 },
    { 0xbfd066f7, SUC_lexicon_tags_61 },
    { 0x323066f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed106700, SUC_lexicon_tags_46 },
    { 0xe3706700, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b106707, SUC_lexicon_tags_63 },
    { 0x13d06707, SUC_lexicon_tags_28 },
    { 0xedd06708, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xd230670b, SUC_lexicon_tags_27 },
    { 0x7eb0670b, SUC_lexicon_tags_43 },
    { 0xf2d0670b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e106713, SUC_lexicon_tags_37 },
    { 0xedb06714, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1830671e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7cf06720, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4306724, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab06727, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2190672a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdcd0672c, SUC_lexicon_tags_62 },
    { 0xa690672c, SUC_lexicon_tags_11 },
    { 0x1630672e, SUC_lexicon_tags_45 },
    { 0xa490672f, SUC_lexicon_tags_12 },
    { 0x69706730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f06733, SUC_lexicon_tags_37 },
    { 0x65d06734, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f906736, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf506739, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5590673b, SUC_lexicon_tags_60 },
    { 0x4c50673b, SUC_lexicon_tags_15 },
    { 0x59d0673d, SUC_lexicon_tags_47 },
    { 0x3f50673c, SUC_lexicon_tags_11 },
    { 0xa8f0673d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b06742, SUC_lexicon_tags_37 },
    { 0x76706743, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27306745, SUC_lexicon_tags_31 },
    { 0xb6506746, SUC_lexicon_tags_10 },
    { 0x7106745, SUC_lexicon_tags_84 },
    { 0xa1506747, SUC_lexicon_tags_45 },
    { 0xe3706748, SUC_lexicon_tags_38 },
    { 0x36706749, SUC_lexicon_tags_3 },
    { 0xf9f0674b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7106750, SUC_lexicon_tags_11 },
    { 0xa5d06751, SUC_lexicon_tags_2 },
    { 0xa7506752, SUC_lexicon_tags_43 },
    { 0xf506752, SUC_lexicon_tags_35 },
    { 0x2a706754, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a306756, SUC_lexicon_tags_408 },
    { 0xa6b06757, SUC_lexicon_tags_15 },
    { 0xad06757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d0675b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b10675e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc706763, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf06768, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa630676a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa010676d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf90676f, SUC_lexicon_tags_15 },
    { 0x9cf06770, SUC_lexicon_tags_29 },
    { 0x3d30676f, SUC_lexicon_tags_17 },
    { 0x8f506770, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeaf06774, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910677a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f06784, SUC_lexicon_tags_7 },
    { 0xe4106785, SUC_lexicon_tags_1 },
    { 0xf3506784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff0678c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f0678f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb06792, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a506794, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x62106796, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6370679b, SUC_lexicon_tags_451 },
    { 0, NULL },
    { 0, NULL },
    { 0x4650679e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb7067a0, SUC_lexicon_tags_47 },
    { 0x18f067a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1067a8, SUC_lexicon_tags_37 },
    { 0x7cf067a9, SUC_lexicon_tags_32 },
    { 0xd91067a9, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0xf43067ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb91067ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef067b2, SUC_lexicon_tags_11 },
    { 0x8b7067b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x165067b5, SUC_lexicon_tags_12 },
    { 0x429067b6, SUC_lexicon_tags_14 },
    { 0x359067b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x373067b9, SUC_lexicon_tags_15 },
    { 0xeb1067b9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8d3067bc, SUC_lexicon_tags_15 },
    { 0x373067bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5067bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7067c6, SUC_lexicon_tags_0 },
    { 0x383067c6, SUC_lexicon_tags_8 },
    { 0x1a7067c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x751067cb, SUC_lexicon_tags_17 },
    { 0xfc9067cb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5067d3, SUC_lexicon_tags_0 },
    { 0x419067d4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23067d7, SUC_lexicon_tags_28 },
    { 0x909067d7, SUC_lexicon_tags_30 },
    { 0xdf3067d9, SUC_lexicon_tags_60 },
    { 0xfe3067da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b067dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7067e0, SUC_lexicon_tags_56 },
    { 0x623067e1, SUC_lexicon_tags_74 },
    { 0x5eb067e1, SUC_lexicon_tags_2 },
    { 0xa1067e3, SUC_lexicon_tags_8 },
    { 0x2c7067e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x169067e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x649067e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b1067eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x755067ed, SUC_lexicon_tags_38 },
    { 0x7c1067ee, SUC_lexicon_tags_114 },
    { 0x7d067ef, SUC_lexicon_tags_26 },
    { 0xd1f067ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69067f4, SUC_lexicon_tags_702 },
    { 0x6e7067f5, SUC_lexicon_tags_60 },
    { 0x3c3067f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x925067f9, SUC_lexicon_tags_26 },
    { 0x17f067f9, SUC_lexicon_tags_45 },
    { 0xeb7067fa, SUC_lexicon_tags_23 },
    { 0x325067fc, SUC_lexicon_tags_38 },
    { 0x1bd067fb, SUC_lexicon_tags_17 },
    { 0xcef067fe, SUC_lexicon_tags_17 },
    { 0xd3d067ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfef06801, SUC_lexicon_tags_53 },
    { 0x106801, SUC_lexicon_tags_11 },
    { 0xd0706801, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b06806, SUC_lexicon_tags_7 },
    { 0x5b306807, SUC_lexicon_tags_7 },
    { 0xe706807, SUC_lexicon_tags_12 },
    { 0xa0506809, SUC_lexicon_tags_33 },
    { 0xafb06808, SUC_lexicon_tags_33 },
    { 0xd910680a, SUC_lexicon_tags_1 },
    { 0xc1306808, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc390680f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94706813, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xc5706815, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d106818, SUC_lexicon_tags_26 },
    { 0xe7306819, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a90681b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa706820, SUC_lexicon_tags_56 },
    { 0xc1506820, SUC_lexicon_tags_59 },
    { 0xa7f06822, SUC_lexicon_tags_1 },
    { 0xfaf06822, SUC_lexicon_tags_31 },
    { 0xaff06824, SUC_lexicon_tags_19 },
    { 0xe3d06823, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b06827, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xad506829, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc850682c, SUC_lexicon_tags_23 },
    { 0x8d30682d, SUC_lexicon_tags_16 },
    { 0xd350682e, SUC_lexicon_tags_17 },
    { 0x19d0682d, SUC_lexicon_tags_7 },
    { 0x19b06830, SUC_lexicon_tags_61 },
    { 0x99d06830, SUC_lexicon_tags_37 },
    { 0x1dd06831, SUC_lexicon_tags_25 },
    { 0x68f06833, SUC_lexicon_tags_31 },
    { 0x9cd06834, SUC_lexicon_tags_68 },
    { 0x7a706834, SUC_lexicon_tags_12 },
    { 0xfe506830, SUC_lexicon_tags_11 },
    { 0xe0706836, SUC_lexicon_tags_0 },
    { 0xe510682f, SUC_lexicon_tags_45 },
    { 0x82b06839, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe290683c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b06840, SUC_lexicon_tags_18 },
    { 0x67b06841, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39d06843, SUC_lexicon_tags_37 },
    { 0xdc506844, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18706846, SUC_lexicon_tags_11 },
    { 0x7d306847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f106849, SUC_lexicon_tags_43 },
    { 0x1f70684a, SUC_lexicon_tags_25 },
    { 0xebd0684b, SUC_lexicon_tags_25 },
    { 0xf390684b, SUC_lexicon_tags_8 },
    { 0x8c90684d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfc70684f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b906857, SUC_lexicon_tags_15 },
    { 0x9a306858, SUC_lexicon_tags_26 },
    { 0xca306859, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb706860, SUC_lexicon_tags_3 },
    { 0x42f06861, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d06867, SUC_lexicon_tags_59 },
    { 0x4ff06867, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb30686a, SUC_lexicon_tags_47 },
    { 0x13b0686b, SUC_lexicon_tags_61 },
    { 0xebf0686c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7906872, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed06879, SUC_lexicon_tags_28 },
    { 0x1bb0687a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d0687d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b06881, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9706884, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b50688c, SUC_lexicon_tags_7 },
    { 0x7d70688c, SUC_lexicon_tags_25 },
    { 0xcbd0688e, SUC_lexicon_tags_87 },
    { 0x3990688e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8906893, SUC_lexicon_tags_15 },
    { 0xa8d06893, SUC_lexicon_tags_130 },
    { 0xc0306895, SUC_lexicon_tags_25 },
    { 0x6b906895, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b0689a, SUC_lexicon_tags_2 },
    { 0x3130689b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0d0689d, SUC_lexicon_tags_14 },
    { 0x72b0689d, SUC_lexicon_tags_82 },
    { 0xd2f0689d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917068a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5068a6, SUC_lexicon_tags_37 },
    { 0x643068a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb068aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b068ae, SUC_lexicon_tags_11 },
    { 0x27f068af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d068b2, SUC_lexicon_tags_12 },
    { 0xced068b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3c1068b5, SUC_lexicon_tags_2 },
    { 0xaed068b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a3068b8, SUC_lexicon_tags_37 },
    { 0xd37068b9, SUC_lexicon_tags_38 },
    { 0xcc1068ba, SUC_lexicon_tags_7 },
    { 0x96b068bb, SUC_lexicon_tags_8 },
    { 0x6b5068bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67068c0, SUC_lexicon_tags_25 },
    { 0x537068c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x105068c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f068ca, SUC_lexicon_tags_25 },
    { 0xf01068cb, SUC_lexicon_tags_28 },
    { 0x3bb068cb, SUC_lexicon_tags_39 },
    { 0xc2b068cb, SUC_lexicon_tags_37 },
    { 0x9bb068ce, SUC_lexicon_tags_12 },
    { 0x3bb068ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f068dd, SUC_lexicon_tags_2 },
    { 0xf05068dd, SUC_lexicon_tags_1 },
    { 0x4dd068de, SUC_lexicon_tags_1 },
    { 0xbe9068e0, SUC_lexicon_tags_5 },
    { 0x46f068e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddb068e3, SUC_lexicon_tags_57 },
    { 0x657068e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777068ea, SUC_lexicon_tags_92 },
    { 0x65068eb, SUC_lexicon_tags_28 },
    { 0xf25068ec, SUC_lexicon_tags_47 },
    { 0x30f068ec, SUC_lexicon_tags_2 },
    { 0xfd9068ee, SUC_lexicon_tags_57 },
    { 0x647068ef, SUC_lexicon_tags_62 },
    { 0x421068f0, SUC_lexicon_tags_84 },
    { 0x511068ed, SUC_lexicon_tags_37 },
    { 0x361068f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d7068f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15068fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43068fe, SUC_lexicon_tags_28 },
    { 0xe85068ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4706904, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x22906906, SUC_lexicon_tags_17 },
    { 0x20f06907, SUC_lexicon_tags_31 },
    { 0x55306908, SUC_lexicon_tags_48 },
    { 0xa0f06909, SUC_lexicon_tags_46 },
    { 0x8550690a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2950690e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a506911, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1106914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5906917, SUC_lexicon_tags_33 },
    { 0x26f06917, SUC_lexicon_tags_77 },
    { 0x29506919, SUC_lexicon_tags_47 },
    { 0x26f0691a, SUC_lexicon_tags_57 },
    { 0x42d06918, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f0691e, SUC_lexicon_tags_14 },
    { 0xf790691f, SUC_lexicon_tags_33 },
    { 0x71706920, SUC_lexicon_tags_47 },
    { 0x8010691e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70506926, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b0692a, SUC_lexicon_tags_47 },
    { 0x9cd0692a, SUC_lexicon_tags_13 },
    { 0x59d0692c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd506930, SUC_lexicon_tags_12 },
    { 0x1b906930, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1706934, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x99106936, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4690693a, SUC_lexicon_tags_45 },
    { 0xf850693b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba30693d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b06940, SUC_lexicon_tags_2 },
    { 0x9d906941, SUC_lexicon_tags_82 },
    { 0x66d06942, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda506944, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c106948, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d0694d, SUC_lexicon_tags_0 },
    { 0xef30694d, SUC_lexicon_tags_61 },
    { 0xfa50694d, SUC_lexicon_tags_19 },
    { 0x9d106950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d06953, SUC_lexicon_tags_17 },
    { 0x77106954, SUC_lexicon_tags_91 },
    { 0xcfb06955, SUC_lexicon_tags_15 },
    { 0x60906956, SUC_lexicon_tags_12 },
    { 0x96d06957, SUC_lexicon_tags_7 },
    { 0x72306956, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f0695f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a306961, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7106963, SUC_lexicon_tags_28 },
    { 0x5fd06963, SUC_lexicon_tags_27 },
    { 0x1f106965, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa270696b, SUC_lexicon_tags_7 },
    { 0x80d0696c, SUC_lexicon_tags_28 },
    { 0xae70696b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6130696f, SUC_lexicon_tags_37 },
    { 0xa2306970, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2dd06972, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf06979, SUC_lexicon_tags_12 },
    { 0x8b10697a, SUC_lexicon_tags_66 },
    { 0xa1f0697b, SUC_lexicon_tags_25 },
    { 0xe8b0697b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bf0697e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x17506981, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b06986, SUC_lexicon_tags_11 },
    { 0xf7906987, SUC_lexicon_tags_12 },
    { 0x94f06988, SUC_lexicon_tags_25 },
    { 0xd2906986, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5706990, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8c706992, SUC_lexicon_tags_38 },
    { 0x97706993, SUC_lexicon_tags_351 },
    { 0x1bd06994, SUC_lexicon_tags_1 },
    { 0x23706995, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe506997, SUC_lexicon_tags_16 },
    { 0x86306998, SUC_lexicon_tags_39 },
    { 0x5e906999, SUC_lexicon_tags_2 },
    { 0x15f0699a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb0699e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d069a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d069ae, SUC_lexicon_tags_32 },
    { 0x165069af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3069b2, SUC_lexicon_tags_1 },
    { 0x1cd069b2, SUC_lexicon_tags_91 },
    { 0x295069b4, SUC_lexicon_tags_2 },
    { 0xbd5069b4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff069b9, SUC_lexicon_tags_49 },
    { 0xa47069ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771069c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d069c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b069c7, SUC_lexicon_tags_60 },
    { 0x9cd069c7, SUC_lexicon_tags_77 },
    { 0xacf069c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f069cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa51069d2, SUC_lexicon_tags_8 },
    { 0x995069d3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3a1069d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7069dd, SUC_lexicon_tags_43 },
    { 0x88b069dd, SUC_lexicon_tags_16 },
    { 0x419069de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x903069e2, SUC_lexicon_tags_14 },
    { 0x18d069e3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207069e8, SUC_lexicon_tags_2 },
    { 0x193069e9, SUC_lexicon_tags_7 },
    { 0xcdd069e9, SUC_lexicon_tags_33 },
    { 0x951069e9, SUC_lexicon_tags_30 },
    { 0x7ab069ec, SUC_lexicon_tags_25 },
    { 0x865069eb, SUC_lexicon_tags_26 },
    { 0xded069ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283069f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d069f9, SUC_lexicon_tags_7 },
    { 0xd8d069fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63069fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69106a01, SUC_lexicon_tags_18 },
    { 0x1ef06a02, SUC_lexicon_tags_15 },
    { 0x16306a02, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce706a07, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f506a0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3106a0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb7506a10, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x19306a13, SUC_lexicon_tags_31 },
    { 0x1f506a14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d706a19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf06a1c, SUC_lexicon_tags_30 },
    { 0x6e306a1c, SUC_lexicon_tags_15 },
    { 0x11d06a1e, SUC_lexicon_tags_15 },
    { 0xdbf06a1e, SUC_lexicon_tags_9 },
    { 0xa2906a1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x22506a22, SUC_lexicon_tags_2 },
    { 0xaa506a22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d06a27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36906a2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed06a33, SUC_lexicon_tags_15 },
    { 0x5ad06a34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19906a37, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6d106a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd506a3e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef706a44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8106a46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x52106a48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b06a4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfc706a4e, SUC_lexicon_tags_1 },
    { 0xf5706a4e, SUC_lexicon_tags_0 },
    { 0x4bf06a50, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83106a57, SUC_lexicon_tags_0 },
    { 0xb7d06a57, SUC_lexicon_tags_25 },
    { 0x3b706a59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63306a5d, SUC_lexicon_tags_37 },
    { 0xf3106a5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x97906a60, SUC_lexicon_tags_33 },
    { 0xe5106a60, SUC_lexicon_tags_55 },
    { 0xed06a60, SUC_lexicon_tags_48 },
    { 0xdc506a63, SUC_lexicon_tags_35 },
    { 0xbd706a64, SUC_lexicon_tags_25 },
    { 0x8a706a65, SUC_lexicon_tags_10 },
    { 0x7bf06a66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d06a6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3306a6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x55d06a71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb06a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78706a77, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb06a7b, SUC_lexicon_tags_17 },
    { 0xbe106a7c, SUC_lexicon_tags_11 },
    { 0x9ff06a7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b06a82, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81706a88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f706a8d, SUC_lexicon_tags_12 },
    { 0x45106a8e, SUC_lexicon_tags_28 },
    { 0xd8706a8d, SUC_lexicon_tags_8 },
    { 0x83306a8d, SUC_lexicon_tags_26 },
    { 0xb4f06a91, SUC_lexicon_tags_11 },
    { 0xde506a92, SUC_lexicon_tags_25 },
    { 0x31d06a92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b06a9c, SUC_lexicon_tags_60 },
    { 0x91b06a9d, SUC_lexicon_tags_7 },
    { 0x55d06a9e, SUC_lexicon_tags_49 },
    { 0x68506a9f, SUC_lexicon_tags_7 },
    { 0x6d706aa0, SUC_lexicon_tags_47 },
    { 0x42b06aa1, SUC_lexicon_tags_14 },
    { 0x18706aa2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6306aa6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x68b06aa8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3506aaa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5306aad, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d06ab6, SUC_lexicon_tags_25 },
    { 0xcaf06ab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44306abb, SUC_lexicon_tags_23 },
    { 0xe0d06abb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e306abe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e906ac0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b306ac3, SUC_lexicon_tags_6 },
    { 0xe3506ac3, SUC_lexicon_tags_12 },
    { 0x41b06ac5, SUC_lexicon_tags_25 },
    { 0xe8306ac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b306ac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db06acb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31906ace, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84506ad0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6ef06ad2, SUC_lexicon_tags_12 },
    { 0xe4d06ad3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xac706ad5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc9d06ad7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0306ada, SUC_lexicon_tags_68 },
    { 0xe8d06ada, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78b06add, SUC_lexicon_tags_33 },
    { 0x74306ade, SUC_lexicon_tags_27 },
    { 0x22f06adf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b106ae3, SUC_lexicon_tags_7 },
    { 0x58b06ae4, SUC_lexicon_tags_37 },
    { 0x1ff06ae3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33306aec, SUC_lexicon_tags_38 },
    { 0xabd06aed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3306aef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f06af6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xca706af8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b06b00, SUC_lexicon_tags_57 },
    { 0xc3706b01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cd06b03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x81d06b05, SUC_lexicon_tags_838 },
    { 0x29d06b05, SUC_lexicon_tags_14 },
    { 0xd3f06b06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6106b09, SUC_lexicon_tags_7 },
    { 0x8706b0a, SUC_lexicon_tags_15 },
    { 0x27d06b09, SUC_lexicon_tags_1 },
    { 0x99906b0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc906b0e, SUC_lexicon_tags_17 },
    { 0x75506b0f, SUC_lexicon_tags_37 },
    { 0x47906b0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b06b13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8706b19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d06b1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd06b20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f06b2c, SUC_lexicon_tags_23 },
    { 0xd506b2d, SUC_lexicon_tags_11 },
    { 0x5e306b2e, SUC_lexicon_tags_23 },
    { 0xf9506b2d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14506b33, SUC_lexicon_tags_12 },
    { 0xd0706b34, SUC_lexicon_tags_12 },
    { 0x3cb06b34, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df06b38, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef06b3c, SUC_lexicon_tags_55 },
    { 0xa5b06b3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78306b42, SUC_lexicon_tags_7 },
    { 0xd6106b42, SUC_lexicon_tags_7 },
    { 0x34f06b44, SUC_lexicon_tags_19 },
    { 0x8db06b45, SUC_lexicon_tags_11 },
    { 0x95706b42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77106b4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a906b4f, SUC_lexicon_tags_1 },
    { 0xa7f06b50, SUC_lexicon_tags_3 },
    { 0xe6506b51, SUC_lexicon_tags_11 },
    { 0xf7506b4f, SUC_lexicon_tags_65 },
    { 0xbc706b4f, SUC_lexicon_tags_59 },
    { 0x4a706b52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f06b57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17b06b59, SUC_lexicon_tags_17 },
    { 0x4b706b5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12706b60, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb06b65, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb306b68, SUC_lexicon_tags_49 },
    { 0xe6f06b69, SUC_lexicon_tags_80 },
    { 0x11306b6a, SUC_lexicon_tags_56 },
    { 0xbf506b6b, SUC_lexicon_tags_2 },
    { 0xee506b6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d06b6f, SUC_lexicon_tags_12 },
    { 0xbc506b70, SUC_lexicon_tags_37 },
    { 0xf3106b70, SUC_lexicon_tags_24 },
    { 0xcfb06b72, SUC_lexicon_tags_15 },
    { 0xc0306b73, SUC_lexicon_tags_37 },
    { 0xe8b06b74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf06b77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85506b7b, SUC_lexicon_tags_15 },
    { 0x95f06b7c, SUC_lexicon_tags_33 },
    { 0x2ff06b7d, SUC_lexicon_tags_19 },
    { 0x62b06b7e, SUC_lexicon_tags_91 },
    { 0xba706b7f, SUC_lexicon_tags_16 },
    { 0xfa706b80, SUC_lexicon_tags_0 },
    { 0x3e306b81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1106b84, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a506b88, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd06b8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4506b94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d06b9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24706ba0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb06ba6, SUC_lexicon_tags_37 },
    { 0xc2f06ba7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafb06ba9, SUC_lexicon_tags_2 },
    { 0x9106baa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f06bae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34906bb4, SUC_lexicon_tags_8 },
    { 0xf1906bb5, SUC_lexicon_tags_1 },
    { 0xc1b06bb5, SUC_lexicon_tags_15 },
    { 0x14d06bb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6af06bb9, SUC_lexicon_tags_25 },
    { 0xdc506bba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2306bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69106bc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b06bc6, SUC_lexicon_tags_11 },
    { 0x7c506bc7, SUC_lexicon_tags_37 },
    { 0xdf106bc7, SUC_lexicon_tags_27 },
    { 0xe2706bc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedd06bcb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x83306bcd, SUC_lexicon_tags_15 },
    { 0xccb06bcd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa8706bd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75f06bd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6306bd4, SUC_lexicon_tags_16 },
    { 0x9c306bd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0306bd8, SUC_lexicon_tags_64 },
    { 0xdb706bd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d06be1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a106beb, SUC_lexicon_tags_55 },
    { 0x69b06bec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98906bee, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22706bf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e706bf4, SUC_lexicon_tags_21 },
    { 0x83106bf5, SUC_lexicon_tags_1 },
    { 0xdaf06bf6, SUC_lexicon_tags_11 },
    { 0x6e506bf7, SUC_lexicon_tags_11 },
    { 0x90706bf8, SUC_lexicon_tags_47 },
    { 0x1dd06bf9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x69106bfc, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0xff106bff, SUC_lexicon_tags_49 },
    { 0x83106c00, SUC_lexicon_tags_11 },
    { 0xe1d06c00, SUC_lexicon_tags_45 },
    { 0x70306c02, SUC_lexicon_tags_87 },
    { 0x93706c03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef06c06, SUC_lexicon_tags_30 },
    { 0x45106c07, SUC_lexicon_tags_25 },
    { 0xd5d06c07, SUC_lexicon_tags_35 },
    { 0xf8106c09, SUC_lexicon_tags_65 },
    { 0xf7b06c08, SUC_lexicon_tags_1 },
    { 0xddf06c09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf06c0e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1ff06c10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67b06c12, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe9706c14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17906c1a, SUC_lexicon_tags_2 },
    { 0x43906c1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bb06c1d, SUC_lexicon_tags_11 },
    { 0xe4306c1e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x23d06c20, SUC_lexicon_tags_37 },
    { 0x38b06c21, SUC_lexicon_tags_23 },
    { 0x6e306c22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71506c27, SUC_lexicon_tags_12 },
    { 0x49306c28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d06c2b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x62706c2d, SUC_lexicon_tags_17 },
    { 0xc3306c2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31506c31, SUC_lexicon_tags_44 },
    { 0x74706c32, SUC_lexicon_tags_17 },
    { 0xa8506c33, SUC_lexicon_tags_40 },
    { 0xe2d06c34, SUC_lexicon_tags_10 },
    { 0xe6d06c35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a906c38, SUC_lexicon_tags_68 },
    { 0x97306c38, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2e706c3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b06c45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7506c48, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc706c4b, SUC_lexicon_tags_23 },
    { 0xb2106c4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb106c4f, SUC_lexicon_tags_692 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58506c57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6506c59, SUC_lexicon_tags_19 },
    { 0xd6d06c5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5706c5e, SUC_lexicon_tags_29 },
    { 0x1d706c5f, SUC_lexicon_tags_23 },
    { 0xba906c60, SUC_lexicon_tags_47 },
    { 0x45106c60, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d706c64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a706c66, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43906c6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34106c6f, SUC_lexicon_tags_39 },
    { 0x88b06c70, SUC_lexicon_tags_14 },
    { 0xb3506c6f, SUC_lexicon_tags_2 },
    { 0x9c506c72, SUC_lexicon_tags_15 },
    { 0xa2506c6f, SUC_lexicon_tags_15 },
    { 0x80b06c70, SUC_lexicon_tags_25 },
    { 0xe7106c75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87f06c77, SUC_lexicon_tags_15 },
    { 0x6a306c78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4306c7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3706c7c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe8106c7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6306c81, SUC_lexicon_tags_12 },
    { 0xf4906c82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f06c85, SUC_lexicon_tags_45 },
    { 0xae306c86, SUC_lexicon_tags_33 },
    { 0xf8d06c86, SUC_lexicon_tags_1 },
    { 0x8d106c88, SUC_lexicon_tags_7 },
    { 0x92906c89, SUC_lexicon_tags_6 },
    { 0xcff06c8a, SUC_lexicon_tags_25 },
    { 0x6d906c8b, SUC_lexicon_tags_43 },
    { 0xa7306c86, SUC_lexicon_tags_12 },
    { 0x1e506c8d, SUC_lexicon_tags_12 },
    { 0x47f06c8d, SUC_lexicon_tags_45 },
    { 0x65b06c8f, SUC_lexicon_tags_307 },
    { 0x10506c90, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbd506c92, SUC_lexicon_tags_11 },
    { 0x36b06c93, SUC_lexicon_tags_23 },
    { 0xab906c92, SUC_lexicon_tags_11 },
    { 0xc3d06c95, SUC_lexicon_tags_15 },
    { 0x33906c94, SUC_lexicon_tags_15 },
    { 0xe9306c92, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x47d06c99, SUC_lexicon_tags_1 },
    { 0x11d06c9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7f306c9c, SUC_lexicon_tags_25 },
    { 0xfe706c9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56906ca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x20306ca7, SUC_lexicon_tags_11 },
    { 0xc2f06ca8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff506cae, SUC_lexicon_tags_49 },
    { 0x7af06caf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b106cb3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3706cb6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd06cbd, SUC_lexicon_tags_1 },
    { 0x1a106cbe, SUC_lexicon_tags_27 },
    { 0xf5f06cbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd906cc3, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x31306cc6, SUC_lexicon_tags_14 },
    { 0x31f06cc6, SUC_lexicon_tags_17 },
    { 0x22b06cc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa706ccc, SUC_lexicon_tags_72 },
    { 0x18506ccd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x53706cd0, SUC_lexicon_tags_11 },
    { 0x67906cd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac506cd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe706cd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84906cd9, SUC_lexicon_tags_79 },
    { 0x92306cda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7506cdd, SUC_lexicon_tags_12 },
    { 0x84f06cde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29706ce0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b06ce3, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0706ce9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc7906ceb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7bb06ced, SUC_lexicon_tags_2 },
    { 0x1f706cee, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb106cf3, SUC_lexicon_tags_25 },
    { 0xe5906cf3, SUC_lexicon_tags_33 },
    { 0x87d06cf5, SUC_lexicon_tags_2 },
    { 0x9d106cf6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed06cfd, SUC_lexicon_tags_12 },
    { 0x75506cfd, SUC_lexicon_tags_17 },
    { 0x2ff06cfd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82706d07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff106d0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd06d0e, SUC_lexicon_tags_37 },
    { 0xd2b06d0f, SUC_lexicon_tags_46 },
    { 0x13506d0f, SUC_lexicon_tags_2 },
    { 0xbab06d0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c906d18, SUC_lexicon_tags_1 },
    { 0x14306d18, SUC_lexicon_tags_2 },
    { 0xb2b06d1a, SUC_lexicon_tags_60 },
    { 0x5f706d1a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c106d1e, SUC_lexicon_tags_10 },
    { 0x37106d1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8af06d21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d06d26, SUC_lexicon_tags_12 },
    { 0x44306d27, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26106d29, SUC_lexicon_tags_12 },
    { 0x6a906d2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacf06d2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d306d33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60f06d35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44306d37, SUC_lexicon_tags_1 },
    { 0xdab06d37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xead06d3a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90106d3c, SUC_lexicon_tags_47 },
    { 0x4fb06d3d, SUC_lexicon_tags_0 },
    { 0x60d06d3d, SUC_lexicon_tags_14 },
    { 0x17306d3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8f906d41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c906d45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e506d48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65506d4c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b06d54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4b306d56, SUC_lexicon_tags_7 },
    { 0xac506d56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d06d5e, SUC_lexicon_tags_0 },
    { 0xa3706d5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd06d62, SUC_lexicon_tags_23 },
    { 0x2f06d63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x98906d66, SUC_lexicon_tags_11 },
    { 0x28b06d67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f06d6a, SUC_lexicon_tags_2 },
    { 0x86106d6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x21706d6e, SUC_lexicon_tags_45 },
    { 0x46f06d6f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf06d72, SUC_lexicon_tags_17 },
    { 0x87506d73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0106d78, SUC_lexicon_tags_63 },
    { 0xcdd06d79, SUC_lexicon_tags_1 },
    { 0xb2b06d7a, SUC_lexicon_tags_27 },
    { 0xedb06d78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcb506d7d, SUC_lexicon_tags_45 },
    { 0x55906d7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe506d83, SUC_lexicon_tags_25 },
    { 0x31b06d83, SUC_lexicon_tags_15 },
    { 0x54306d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4d06d87, SUC_lexicon_tags_65 },
    { 0xa8306d87, SUC_lexicon_tags_14 },
    { 0xe506d87, SUC_lexicon_tags_15 },
    { 0xfa106d8a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x13706d8c, SUC_lexicon_tags_33 },
    { 0x1d106d8d, SUC_lexicon_tags_21 },
    { 0xc1306d8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db06d91, SUC_lexicon_tags_1 },
    { 0xce306d91, SUC_lexicon_tags_88 },
    { 0x17b06d93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41506d98, SUC_lexicon_tags_11 },
    { 0x87f06d99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b06d9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b06d9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b06da2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b106da5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b06daa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x44906dad, SUC_lexicon_tags_7 },
    { 0x97306dae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff06db2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5d06db4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbdf06db6, SUC_lexicon_tags_11 },
    { 0x61306db7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc706dba, SUC_lexicon_tags_6 },
    { 0xb6506dbb, SUC_lexicon_tags_37 },
    { 0x18f06dbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58306dbe, SUC_lexicon_tags_28 },
    { 0x4f306dbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99506dc8, SUC_lexicon_tags_0 },
    { 0xd5306dc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5f06dcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f306dcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6db06dcf, SUC_lexicon_tags_33 },
    { 0x9bb06dd0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d06dd4, SUC_lexicon_tags_19 },
    { 0xf6306dd5, SUC_lexicon_tags_25 },
    { 0x97506dd6, SUC_lexicon_tags_15 },
    { 0xc2f06dd4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e106ddc, SUC_lexicon_tags_0 },
    { 0xf5106ddd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x11f06ddf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d906de2, SUC_lexicon_tags_15 },
    { 0xa5706de3, SUC_lexicon_tags_26 },
    { 0x1c906de4, SUC_lexicon_tags_45 },
    { 0x22706de5, SUC_lexicon_tags_17 },
    { 0x47506de6, SUC_lexicon_tags_15 },
    { 0x32d06de7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x79706dea, SUC_lexicon_tags_11 },
    { 0x98d06deb, SUC_lexicon_tags_16 },
    { 0x97b06deb, SUC_lexicon_tags_124 },
    { 0xf9906ded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea706df1, SUC_lexicon_tags_27 },
    { 0xb5506df1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff06df5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92106dfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3706e03, SUC_lexicon_tags_40 },
    { 0xe8706e03, SUC_lexicon_tags_60 },
    { 0x1bd06e04, SUC_lexicon_tags_47 },
    { 0xb0106e03, SUC_lexicon_tags_26 },
    { 0xbfd06e07, SUC_lexicon_tags_19 },
    { 0xdeb06e07, SUC_lexicon_tags_23 },
    { 0xb8906e09, SUC_lexicon_tags_1 },
    { 0xd3306e0a, SUC_lexicon_tags_9 },
    { 0x10106e09, SUC_lexicon_tags_17 },
    { 0x2e906e0b, SUC_lexicon_tags_45 },
    { 0x67906e0b, SUC_lexicon_tags_63 },
    { 0xf2f06e0e, SUC_lexicon_tags_11 },
    { 0xe2b06e0c, SUC_lexicon_tags_77 },
    { 0x98106e10, SUC_lexicon_tags_63 },
    { 0x81f06e11, SUC_lexicon_tags_47 },
    { 0xf9906e12, SUC_lexicon_tags_15 },
    { 0x7b306e13, SUC_lexicon_tags_47 },
    { 0x60306e14, SUC_lexicon_tags_104 },
    { 0xce306e15, SUC_lexicon_tags_31 },
    { 0x98106e13, SUC_lexicon_tags_7 },
    { 0x47306e07, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d06e1c, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x40706e1f, SUC_lexicon_tags_37 },
    { 0x37f06e20, SUC_lexicon_tags_62 },
    { 0x29306e21, SUC_lexicon_tags_15 },
    { 0xf2b06e22, SUC_lexicon_tags_106 },
    { 0x10d06e22, SUC_lexicon_tags_45 },
    { 0x6cb06e21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85706e2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6306e33, SUC_lexicon_tags_16 },
    { 0x76b06e33, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f06e3a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9a106e3c, SUC_lexicon_tags_1 },
    { 0xf2706e3d, SUC_lexicon_tags_34 },
    { 0x3ef06e3e, SUC_lexicon_tags_17 },
    { 0x7e106e3f, SUC_lexicon_tags_19 },
    { 0xadf06e40, SUC_lexicon_tags_60 },
    { 0x8d706e3e, SUC_lexicon_tags_2 },
    { 0x80d06e3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x106e46, SUC_lexicon_tags_47 },
    { 0x62906e47, SUC_lexicon_tags_30 },
    { 0xd9d06e46, SUC_lexicon_tags_43 },
    { 0x37706e49, SUC_lexicon_tags_2 },
    { 0x51106e4a, SUC_lexicon_tags_7 },
    { 0xa6d06e4b, SUC_lexicon_tags_1 },
    { 0x10106e4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb06e4f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa2106e51, SUC_lexicon_tags_37 },
    { 0xf8106e52, SUC_lexicon_tags_33 },
    { 0xc6d06e53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d06e56, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xeb906e58, SUC_lexicon_tags_1 },
    { 0x98506e58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e306e64, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc9106e66, SUC_lexicon_tags_25 },
    { 0x75f06e66, SUC_lexicon_tags_9 },
    { 0xec706e68, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3cd06e6a, SUC_lexicon_tags_11 },
    { 0x3f706e6b, SUC_lexicon_tags_79 },
    { 0x21d06e6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40506e6e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb906e73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd1b06e75, SUC_lexicon_tags_37 },
    { 0x94106e76, SUC_lexicon_tags_25 },
    { 0x28506e75, SUC_lexicon_tags_49 },
    { 0x6d106e78, SUC_lexicon_tags_33 },
    { 0x74906e76, SUC_lexicon_tags_17 },
    { 0x3bf06e76, SUC_lexicon_tags_37 },
    { 0x5c506e7a, SUC_lexicon_tags_11 },
    { 0x13f06e75, SUC_lexicon_tags_27 },
    { 0x90d06e7d, SUC_lexicon_tags_17 },
    { 0x24f06e7e, SUC_lexicon_tags_31 },
    { 0x53306e7f, SUC_lexicon_tags_15 },
    { 0x7706e7e, SUC_lexicon_tags_11 },
    { 0xcf06e81, SUC_lexicon_tags_29 },
    { 0x3bd06e80, SUC_lexicon_tags_47 },
    { 0xa4706e81, SUC_lexicon_tags_37 },
    { 0x49106e83, SUC_lexicon_tags_47 },
    { 0x7706e85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d06e88, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x93106e8a, SUC_lexicon_tags_25 },
    { 0x35106e8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb06e8f, SUC_lexicon_tags_37 },
    { 0x12906e8f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x63706e92, SUC_lexicon_tags_31 },
    { 0x21b06e93, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f306e97, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc5506e99, SUC_lexicon_tags_0 },
    { 0x38306e9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d06e9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54506ea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22306eaf, SUC_lexicon_tags_19 },
    { 0xecb06eaf, SUC_lexicon_tags_82 },
    { 0x1df06eb1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf7906eb3, SUC_lexicon_tags_9 },
    { 0xd0106eb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa8f06eb6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa506ebc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x45306ebe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x37106ec0, SUC_lexicon_tags_38 },
    { 0xa4906ec0, SUC_lexicon_tags_37 },
    { 0xbbb06ec0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d06ec6, SUC_lexicon_tags_27 },
    { 0xc9306ec7, SUC_lexicon_tags_113 },
    { 0x38706ec7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfaf06eca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee906ed0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5906ed3, SUC_lexicon_tags_14 },
    { 0xa6906ed4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59306eda, SUC_lexicon_tags_0 },
    { 0xa2106eda, SUC_lexicon_tags_15 },
    { 0xd706edc, SUC_lexicon_tags_9 },
    { 0x5fb06edd, SUC_lexicon_tags_49 },
    { 0x38306edc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd06ee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8d06eea, SUC_lexicon_tags_12 },
    { 0x9bd06eea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bb06eed, SUC_lexicon_tags_23 },
    { 0xa3506eee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ff06ef0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b06ef5, SUC_lexicon_tags_11 },
    { 0x7b306ef5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x45906ef8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13506efe, SUC_lexicon_tags_12 },
    { 0xaff06eff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b06f02, SUC_lexicon_tags_35 },
    { 0x77506f02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d506f09, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x29906f0b, SUC_lexicon_tags_148 },
    { 0xaa106f0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd06f10, SUC_lexicon_tags_15 },
    { 0x37106f11, SUC_lexicon_tags_4 },
    { 0x4a706f11, SUC_lexicon_tags_0 },
    { 0x69f06f10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41f06f15, SUC_lexicon_tags_31 },
    { 0x96706f16, SUC_lexicon_tags_25 },
    { 0xbf506f15, SUC_lexicon_tags_2 },
    { 0xab706f18, SUC_lexicon_tags_8 },
    { 0x58f06f19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36506f1b, SUC_lexicon_tags_23 },
    { 0xdb306f1c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d06f21, SUC_lexicon_tags_32 },
    { 0x1fd06f21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd06f2a, SUC_lexicon_tags_15 },
    { 0xce106f2b, SUC_lexicon_tags_12 },
    { 0xe4f06f2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d06f2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcf06f31, SUC_lexicon_tags_17 },
    { 0x70706f31, SUC_lexicon_tags_7 },
    { 0x25706f33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d06f38, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xec306f3b, SUC_lexicon_tags_56 },
    { 0x45f06f3b, SUC_lexicon_tags_35 },
    { 0x61306f3c, SUC_lexicon_tags_11 },
    { 0xd4506f3c, SUC_lexicon_tags_60 },
    { 0x65706f3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b06f46, SUC_lexicon_tags_18 },
    { 0xc9d06f46, SUC_lexicon_tags_37 },
    { 0xf9d06f48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec906f4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f06f4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f306f52, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb06f55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab06f58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84d06f5a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f06f5d, SUC_lexicon_tags_17 },
    { 0xdc506f5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b06f60, SUC_lexicon_tags_12 },
    { 0x90306f60, SUC_lexicon_tags_28 },
    { 0xb8d06f61, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced06f68, SUC_lexicon_tags_25 },
    { 0x2a506f69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38506f6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e506f71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f06f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18306f77, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b06f7b, SUC_lexicon_tags_19 },
    { 0x77106f7b, SUC_lexicon_tags_33 },
    { 0x83d06f7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4906f7f, SUC_lexicon_tags_15 },
    { 0x7a506f80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39906f82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae706f84, SUC_lexicon_tags_37 },
    { 0xa2106f84, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9506f8b, SUC_lexicon_tags_27 },
    { 0xe4706f8b, SUC_lexicon_tags_43 },
    { 0x80106f8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4af06f8f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34506f95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x17906f97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b06f9e, SUC_lexicon_tags_39 },
    { 0x89306f9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2306fa1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1106fa4, SUC_lexicon_tags_32 },
    { 0xab06fa5, SUC_lexicon_tags_25 },
    { 0x2a906fa6, SUC_lexicon_tags_25 },
    { 0x3906fa6, SUC_lexicon_tags_14 },
    { 0xb0906fa8, SUC_lexicon_tags_2 },
    { 0xae106fa9, SUC_lexicon_tags_29 },
    { 0xfc306fa9, SUC_lexicon_tags_15 },
    { 0xf4106fa6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7b06fad, SUC_lexicon_tags_38 },
    { 0x90b06fad, SUC_lexicon_tags_37 },
    { 0x13706faf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df06fb2, SUC_lexicon_tags_2 },
    { 0x3e506fb3, SUC_lexicon_tags_35 },
    { 0xa2906fb3, SUC_lexicon_tags_12 },
    { 0x88106fb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b06fbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21b06fbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52506fc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce306fc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12306fca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x71f06fcc, SUC_lexicon_tags_15 },
    { 0xd2106fcc, SUC_lexicon_tags_17 },
    { 0x23b06fce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7106fd1, SUC_lexicon_tags_0 },
    { 0x63906fd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef306fd6, SUC_lexicon_tags_3 },
    { 0x4fb06fd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82906fdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d06fde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d06fe1, SUC_lexicon_tags_37 },
    { 0x60306fe1, SUC_lexicon_tags_17 },
    { 0x6c906fe3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb706fe7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf06ff0, SUC_lexicon_tags_38 },
    { 0x3bb06ff1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb506ff4, SUC_lexicon_tags_17 },
    { 0xc6706ff4, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0, NULL },
    { 0x28906ff8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1b06ffa, SUC_lexicon_tags_2 },
    { 0xe7f06ffb, SUC_lexicon_tags_11 },
    { 0xb4306ffc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b07001, SUC_lexicon_tags_68 },
    { 0x8a507001, SUC_lexicon_tags_17 },
    { 0x16f07002, SUC_lexicon_tags_17 },
    { 0xe4707004, SUC_lexicon_tags_17 },
    { 0x8bd07005, SUC_lexicon_tags_1 },
    { 0xeef07006, SUC_lexicon_tags_47 },
    { 0x82d07002, SUC_lexicon_tags_168 },
    { 0x3bf07008, SUC_lexicon_tags_17 },
    { 0xba507009, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc830700c, SUC_lexicon_tags_21 },
    { 0x38f0700d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e107011, SUC_lexicon_tags_37 },
    { 0x83107011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3307014, SUC_lexicon_tags_15 },
    { 0x75907015, SUC_lexicon_tags_12 },
    { 0xbd507016, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1730701a, SUC_lexicon_tags_37 },
    { 0x1e10701b, SUC_lexicon_tags_7 },
    { 0xf8f0701c, SUC_lexicon_tags_31 },
    { 0xac70701c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4370701f, SUC_lexicon_tags_2 },
    { 0x70507020, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb907022, SUC_lexicon_tags_12 },
    { 0x6b307023, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36707025, SUC_lexicon_tags_38 },
    { 0x32b07026, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xccf07028, SUC_lexicon_tags_11 },
    { 0xe4b07028, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb0702c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b0702e, SUC_lexicon_tags_31 },
    { 0xb190702f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe7107031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb707033, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x94707035, SUC_lexicon_tags_1 },
    { 0xff707036, SUC_lexicon_tags_12 },
    { 0x6c107037, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d0703c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb07040, SUC_lexicon_tags_37 },
    { 0xb8d07041, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f0704c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d07050, SUC_lexicon_tags_12 },
    { 0x43507051, SUC_lexicon_tags_34 },
    { 0xbb907052, SUC_lexicon_tags_37 },
    { 0xda907053, SUC_lexicon_tags_32 },
    { 0x3b507054, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df07059, SUC_lexicon_tags_31 },
    { 0xea50705a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a30705c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd090705e, SUC_lexicon_tags_10 },
    { 0xc030705f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f07063, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6307068, SUC_lexicon_tags_3 },
    { 0x14907069, SUC_lexicon_tags_49 },
    { 0xd790706a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4630706c, SUC_lexicon_tags_17 },
    { 0x3b50706d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd307073, SUC_lexicon_tags_31 },
    { 0xce107073, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x61d07076, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d30707a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6d0707c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a90707e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d307084, SUC_lexicon_tags_11 },
    { 0x81f07085, SUC_lexicon_tags_2 },
    { 0xcf707084, SUC_lexicon_tags_1 },
    { 0x35307087, SUC_lexicon_tags_1 },
    { 0x80307088, SUC_lexicon_tags_11 },
    { 0x39107086, SUC_lexicon_tags_7 },
    { 0x7290708a, SUC_lexicon_tags_45 },
    { 0x5090708a, SUC_lexicon_tags_15 },
    { 0xee10708c, SUC_lexicon_tags_9 },
    { 0x1790708d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71f0708f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xabf07091, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57307094, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b07097, SUC_lexicon_tags_12 },
    { 0x31507098, SUC_lexicon_tags_1 },
    { 0xb9107099, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeed0709b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb850709e, SUC_lexicon_tags_17 },
    { 0x34d0709f, SUC_lexicon_tags_37 },
    { 0xc8d0709f, SUC_lexicon_tags_11 },
    { 0x51d070a1, SUC_lexicon_tags_19 },
    { 0xbb7070a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd070a6, SUC_lexicon_tags_15 },
    { 0x30f070a6, SUC_lexicon_tags_38 },
    { 0x41d070a8, SUC_lexicon_tags_27 },
    { 0xba9070a6, SUC_lexicon_tags_11 },
    { 0x489070aa, SUC_lexicon_tags_1269 },
    { 0x3fb070a7, SUC_lexicon_tags_27 },
    { 0xcd9070ac, SUC_lexicon_tags_45 },
    { 0xbcf070ad, SUC_lexicon_tags_127 },
    { 0x95f070aa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb070b3, SUC_lexicon_tags_103 },
    { 0x9c3070b3, SUC_lexicon_tags_15 },
    { 0x49070b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x491070b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9070bb, SUC_lexicon_tags_49 },
    { 0x9eb070bb, SUC_lexicon_tags_1 },
    { 0x273070bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1070c0, SUC_lexicon_tags_37 },
    { 0xe7d070c1, SUC_lexicon_tags_33 },
    { 0x8e5070c2, SUC_lexicon_tags_2 },
    { 0xaeb070c3, SUC_lexicon_tags_14 },
    { 0x597070c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5070c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5070cb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x33f070cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5070d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x807070d7, SUC_lexicon_tags_6 },
    { 0x999070d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x803070da, SUC_lexicon_tags_4 },
    { 0x13b070db, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x465070de, SUC_lexicon_tags_7 },
    { 0x937070df, SUC_lexicon_tags_27 },
    { 0x61d070de, SUC_lexicon_tags_60 },
    { 0xa3d070e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf070e6, SUC_lexicon_tags_23 },
    { 0xb27070e6, SUC_lexicon_tags_47 },
    { 0xe03070e8, SUC_lexicon_tags_7 },
    { 0x73f070e9, SUC_lexicon_tags_30 },
    { 0xa37070ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x457070ed, SUC_lexicon_tags_12 },
    { 0x405070ed, SUC_lexicon_tags_38 },
    { 0x2c9070ed, SUC_lexicon_tags_12 },
    { 0x293070ef, SUC_lexicon_tags_25 },
    { 0x8f070f1, SUC_lexicon_tags_17 },
    { 0xa6d070f2, SUC_lexicon_tags_17 },
    { 0xcf070ee, SUC_lexicon_tags_7 },
    { 0xfcd070f4, SUC_lexicon_tags_7 },
    { 0x3b5070ed, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x6e7070f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897070ff, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b307102, SUC_lexicon_tags_29 },
    { 0xad907103, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8230710b, SUC_lexicon_tags_63 },
    { 0xc710710b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e107110, SUC_lexicon_tags_1 },
    { 0x82d07111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7907113, SUC_lexicon_tags_15 },
    { 0x66907114, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb0711d, SUC_lexicon_tags_1380 },
    { 0, NULL },
    { 0xc990711f, SUC_lexicon_tags_55 },
    { 0xcdd07120, SUC_lexicon_tags_15 },
    { 0xbf07120, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d07129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff0712c, SUC_lexicon_tags_17 },
    { 0xe70712d, SUC_lexicon_tags_4 },
    { 0x54d0712e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad307132, SUC_lexicon_tags_26 },
    { 0xae707133, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f07136, SUC_lexicon_tags_66 },
    { 0xad507137, SUC_lexicon_tags_2 },
    { 0xdfb07137, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc730713c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a707140, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5107143, SUC_lexicon_tags_37 },
    { 0xe4507144, SUC_lexicon_tags_15 },
    { 0x9d307144, SUC_lexicon_tags_32 },
    { 0x9f107146, SUC_lexicon_tags_27 },
    { 0x6ad07147, SUC_lexicon_tags_25 },
    { 0x12f07148, SUC_lexicon_tags_34 },
    { 0x9307149, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa030714c, SUC_lexicon_tags_14 },
    { 0x4d0714d, SUC_lexicon_tags_11 },
    { 0x1d70714e, SUC_lexicon_tags_2 },
    { 0x9a70714d, SUC_lexicon_tags_1 },
    { 0xf970714f, SUC_lexicon_tags_9 },
    { 0x5150714c, SUC_lexicon_tags_12 },
    { 0xb0707151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb07156, SUC_lexicon_tags_25 },
    { 0x47507157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3510715d, SUC_lexicon_tags_31 },
    { 0xd2f0715e, SUC_lexicon_tags_17 },
    { 0x4d30715d, SUC_lexicon_tags_38 },
    { 0x3bd07160, SUC_lexicon_tags_3 },
    { 0xf4b07160, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5307167, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd590716c, SUC_lexicon_tags_31 },
    { 0x1610716d, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b07171, SUC_lexicon_tags_13 },
    { 0x72107172, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x1507174, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3107179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1010717e, SUC_lexicon_tags_6 },
    { 0x4390717f, SUC_lexicon_tags_1 },
    { 0xc430717e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9707182, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x44907184, SUC_lexicon_tags_39 },
    { 0x89d07185, SUC_lexicon_tags_12 },
    { 0x59907185, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e10718b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f0718e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb07196, SUC_lexicon_tags_32 },
    { 0x17707196, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3307199, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405071a2, SUC_lexicon_tags_25 },
    { 0xb11071a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad071a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2b071ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x197071ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75071b0, SUC_lexicon_tags_23 },
    { 0x967071b1, SUC_lexicon_tags_47 },
    { 0xdb5071b2, SUC_lexicon_tags_833 },
    { 0x305071b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x89071b6, SUC_lexicon_tags_7 },
    { 0x12f071b6, SUC_lexicon_tags_39 },
    { 0x3c1071b8, SUC_lexicon_tags_33 },
    { 0x417071b9, SUC_lexicon_tags_2 },
    { 0x3d1071b9, SUC_lexicon_tags_9 },
    { 0xaf1071ba, SUC_lexicon_tags_17 },
    { 0x2b1071bc, SUC_lexicon_tags_15 },
    { 0xcb9071bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d071c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95071c3, SUC_lexicon_tags_17 },
    { 0x3d071c4, SUC_lexicon_tags_60 },
    { 0xde1071c5, SUC_lexicon_tags_1 },
    { 0x13d071c6, SUC_lexicon_tags_55 },
    { 0x869071c6, SUC_lexicon_tags_7 },
    { 0xbcb071c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d071cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x859071ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285071d4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833071d8, SUC_lexicon_tags_26 },
    { 0x1cf071d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1071de, SUC_lexicon_tags_125 },
    { 0x695071df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc81071e1, SUC_lexicon_tags_37 },
    { 0x66b071e2, SUC_lexicon_tags_25 },
    { 0xdf5071e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcd1071e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d071e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7071ea, SUC_lexicon_tags_45 },
    { 0xd3f071eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5071ee, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa41071f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99071f6, SUC_lexicon_tags_37 },
    { 0x2ed071f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb71071f9, SUC_lexicon_tags_28 },
    { 0xc7071f9, SUC_lexicon_tags_46 },
    { 0x925071fb, SUC_lexicon_tags_25 },
    { 0x7e5071fc, SUC_lexicon_tags_79 },
    { 0x905071fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f07200, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf07203, SUC_lexicon_tags_25 },
    { 0x45907203, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19707206, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f907209, SUC_lexicon_tags_26 },
    { 0xd950720a, SUC_lexicon_tags_37 },
    { 0x8330720b, SUC_lexicon_tags_12 },
    { 0xbf90720b, SUC_lexicon_tags_0 },
    { 0x6f507209, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8307210, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99707222, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f07225, SUC_lexicon_tags_7 },
    { 0x6fb07225, SUC_lexicon_tags_2 },
    { 0x92707227, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0722a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150722e, SUC_lexicon_tags_44 },
    { 0xaf50722f, SUC_lexicon_tags_25 },
    { 0xa310722f, SUC_lexicon_tags_31 },
    { 0x4530722f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x27907234, SUC_lexicon_tags_80 },
    { 0xe4f07235, SUC_lexicon_tags_1 },
    { 0x45107236, SUC_lexicon_tags_38 },
    { 0x58307236, SUC_lexicon_tags_15 },
    { 0xcb07234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c90723e, SUC_lexicon_tags_5 },
    { 0x5050723e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad07242, SUC_lexicon_tags_28 },
    { 0x80b07242, SUC_lexicon_tags_1 },
    { 0x2ab07242, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa390724a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d90724d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab07252, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db07257, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2230725c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4250725e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x64307260, SUC_lexicon_tags_17 },
    { 0x75907260, SUC_lexicon_tags_12 },
    { 0x86b07262, SUC_lexicon_tags_113 },
    { 0xb5b07262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9307267, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6810726c, SUC_lexicon_tags_36 },
    { 0xb990726d, SUC_lexicon_tags_47 },
    { 0xd6d0726e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc6307270, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x64107272, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x17307275, SUC_lexicon_tags_17 },
    { 0xcf907276, SUC_lexicon_tags_60 },
    { 0xc3507277, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61707279, SUC_lexicon_tags_1 },
    { 0x94f07279, SUC_lexicon_tags_26 },
    { 0x2770727b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d0727e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd507280, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbe307282, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9330728c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c107292, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c907295, SUC_lexicon_tags_39 },
    { 0x7d507295, SUC_lexicon_tags_12 },
    { 0x3ed07296, SUC_lexicon_tags_1 },
    { 0x3e107298, SUC_lexicon_tags_109 },
    { 0xd0907299, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5072a2, SUC_lexicon_tags_25 },
    { 0x589072a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df072a6, SUC_lexicon_tags_30 },
    { 0x403072a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4f072a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92b072ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f072b2, SUC_lexicon_tags_25 },
    { 0xd4d072b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1072b8, SUC_lexicon_tags_1 },
    { 0x371072b9, SUC_lexicon_tags_47 },
    { 0x94b072ba, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d072be, SUC_lexicon_tags_0 },
    { 0x933072bf, SUC_lexicon_tags_17 },
    { 0xaab072be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397072c4, SUC_lexicon_tags_22 },
    { 0x40b072c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d072ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f072d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x461072d3, SUC_lexicon_tags_21 },
    { 0xd3f072d4, SUC_lexicon_tags_0 },
    { 0x85b072d4, SUC_lexicon_tags_19 },
    { 0xf45072d4, SUC_lexicon_tags_8 },
    { 0x659072d7, SUC_lexicon_tags_14 },
    { 0x4c3072d6, SUC_lexicon_tags_47 },
    { 0x15f072d6, SUC_lexicon_tags_2 },
    { 0x8e7072d9, SUC_lexicon_tags_23 },
    { 0xf07072da, SUC_lexicon_tags_11 },
    { 0x87072d3, SUC_lexicon_tags_2 },
    { 0x95072dd, SUC_lexicon_tags_38 },
    { 0x659072de, SUC_lexicon_tags_87 },
    { 0xc7f072df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xab7072e1, SUC_lexicon_tags_37 },
    { 0xb6b072e2, SUC_lexicon_tags_65 },
    { 0x31d072e3, SUC_lexicon_tags_7 },
    { 0x783072e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f072eb, SUC_lexicon_tags_37 },
    { 0xf9b072ec, SUC_lexicon_tags_7 },
    { 0xf79072ec, SUC_lexicon_tags_88 },
    { 0xcfd072ec, SUC_lexicon_tags_2 },
    { 0x9cf072eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47072f2, SUC_lexicon_tags_17 },
    { 0x135072f3, SUC_lexicon_tags_14 },
    { 0xfa9072f2, SUC_lexicon_tags_2 },
    { 0x7db072f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3072fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19072fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9072ff, SUC_lexicon_tags_47 },
    { 0x1a507300, SUC_lexicon_tags_15 },
    { 0xeab07300, SUC_lexicon_tags_38 },
    { 0xdb107300, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xadf07304, SUC_lexicon_tags_0 },
    { 0x87b07305, SUC_lexicon_tags_33 },
    { 0x34707306, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9430730a, SUC_lexicon_tags_9 },
    { 0x9850730a, SUC_lexicon_tags_3 },
    { 0x9d10730c, SUC_lexicon_tags_35 },
    { 0x7e90730b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x46307310, SUC_lexicon_tags_8 },
    { 0xd9107311, SUC_lexicon_tags_38 },
    { 0x7ab07312, SUC_lexicon_tags_3 },
    { 0x3e907313, SUC_lexicon_tags_12 },
    { 0x1ab07313, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca50731a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4570731c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf70731f, SUC_lexicon_tags_88 },
    { 0x32707320, SUC_lexicon_tags_19 },
    { 0xb3907320, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5107324, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb907329, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6907331, SUC_lexicon_tags_37 },
    { 0xb8b07332, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xc3307334, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b107338, SUC_lexicon_tags_0 },
    { 0x42f07338, SUC_lexicon_tags_11 },
    { 0x96b0733a, SUC_lexicon_tags_136 },
    { 0xc350733b, SUC_lexicon_tags_39 },
    { 0xeaf0733b, SUC_lexicon_tags_4 },
    { 0xd750733c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5707342, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1307349, SUC_lexicon_tags_33 },
    { 0x3a30734a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f0734d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xddd0734f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe7507351, SUC_lexicon_tags_7 },
    { 0x69907352, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x90507354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e707357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac90735a, SUC_lexicon_tags_0 },
    { 0x7d30735b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca50735e, SUC_lexicon_tags_23 },
    { 0x2230735f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8507365, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d70736e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5707372, SUC_lexicon_tags_15 },
    { 0x5ab07372, SUC_lexicon_tags_1 },
    { 0x53b07374, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f0737d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e10737f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x75907382, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b07386, SUC_lexicon_tags_14 },
    { 0x60b07386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9430738a, SUC_lexicon_tags_82 },
    { 0xa0d0738b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x21f0738d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b07392, SUC_lexicon_tags_11 },
    { 0xa7507392, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb507395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb07398, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd0739d, SUC_lexicon_tags_17 },
    { 0x58d0739e, SUC_lexicon_tags_11 },
    { 0x2c90739f, SUC_lexicon_tags_11 },
    { 0x727073a0, SUC_lexicon_tags_1 },
    { 0x890739f, SUC_lexicon_tags_47 },
    { 0x8b073a1, SUC_lexicon_tags_2 },
    { 0x47073a3, SUC_lexicon_tags_25 },
    { 0xcab073a1, SUC_lexicon_tags_9 },
    { 0xc05073a5, SUC_lexicon_tags_33 },
    { 0x863073a5, SUC_lexicon_tags_14 },
    { 0xfe5073a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3073ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x121073ad, SUC_lexicon_tags_28 },
    { 0xaef073ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df073b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2af073b3, SUC_lexicon_tags_25 },
    { 0x3a7073b3, SUC_lexicon_tags_7 },
    { 0xdeb073b5, SUC_lexicon_tags_32 },
    { 0xdd073b4, SUC_lexicon_tags_15 },
    { 0xbd073b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3b073b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ab073bb, SUC_lexicon_tags_1 },
    { 0x525073bc, SUC_lexicon_tags_25 },
    { 0x1f5073bd, SUC_lexicon_tags_11 },
    { 0xe77073be, SUC_lexicon_tags_26 },
    { 0xfcb073bf, SUC_lexicon_tags_4 },
    { 0x303073bf, SUC_lexicon_tags_47 },
    { 0xa5d073c1, SUC_lexicon_tags_12 },
    { 0x683073c2, SUC_lexicon_tags_11 },
    { 0x25d073c3, SUC_lexicon_tags_12 },
    { 0xf43073bd, SUC_lexicon_tags_198 },
    { 0x22d073c5, SUC_lexicon_tags_299 },
    { 0x727073be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9073cd, SUC_lexicon_tags_56 },
    { 0x4c1073cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5073d3, SUC_lexicon_tags_1 },
    { 0x14f073d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73073d8, SUC_lexicon_tags_25 },
    { 0x209073d8, SUC_lexicon_tags_45 },
    { 0x6b3073da, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x259073dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e3073df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37073e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf073e7, SUC_lexicon_tags_19 },
    { 0xc47073e8, SUC_lexicon_tags_68 },
    { 0x83b073e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7073ec, SUC_lexicon_tags_17 },
    { 0x149073ed, SUC_lexicon_tags_11 },
    { 0x277073ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53073f2, SUC_lexicon_tags_11 },
    { 0xab073f2, SUC_lexicon_tags_2 },
    { 0xa7f073f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xff9073f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b073fd, SUC_lexicon_tags_31 },
    { 0x46b073fe, SUC_lexicon_tags_37 },
    { 0xe57073fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x41307402, SUC_lexicon_tags_8 },
    { 0x57307402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f0740a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff30740c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb107410, SUC_lexicon_tags_1 },
    { 0x88507410, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9b07413, SUC_lexicon_tags_14 },
    { 0x74d07413, SUC_lexicon_tags_1 },
    { 0x69707415, SUC_lexicon_tags_37 },
    { 0x7d907416, SUC_lexicon_tags_8 },
    { 0x64b07417, SUC_lexicon_tags_15 },
    { 0xf3107415, SUC_lexicon_tags_32 },
    { 0x61b07418, SUC_lexicon_tags_2 },
    { 0x11907418, SUC_lexicon_tags_24 },
    { 0x7870741b, SUC_lexicon_tags_28 },
    { 0x8e10741c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35107421, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1bb07423, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xddb07425, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af07429, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6850742d, SUC_lexicon_tags_19 },
    { 0x5530742d, SUC_lexicon_tags_23 },
    { 0x1e50742f, SUC_lexicon_tags_77 },
    { 0xc810742f, SUC_lexicon_tags_17 },
    { 0x53b0742d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b907433, SUC_lexicon_tags_27 },
    { 0x6ed07434, SUC_lexicon_tags_63 },
    { 0x27f07434, SUC_lexicon_tags_9 },
    { 0xb0707433, SUC_lexicon_tags_17 },
    { 0x4a907433, SUC_lexicon_tags_60 },
    { 0x8d07435, SUC_lexicon_tags_25 },
    { 0x3d907439, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe630743b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4890743d, SUC_lexicon_tags_1 },
    { 0xc0b0743e, SUC_lexicon_tags_27 },
    { 0x1010743e, SUC_lexicon_tags_33 },
    { 0x3bf0743f, SUC_lexicon_tags_32 },
    { 0xf1907441, SUC_lexicon_tags_10 },
    { 0x8e107442, SUC_lexicon_tags_38 },
    { 0x91507443, SUC_lexicon_tags_10 },
    { 0x2f907444, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f0744c, SUC_lexicon_tags_47 },
    { 0xee30744d, SUC_lexicon_tags_27 },
    { 0xe390744e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38907450, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac507459, SUC_lexicon_tags_2 },
    { 0xaf50745a, SUC_lexicon_tags_0 },
    { 0xb0745a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b07461, SUC_lexicon_tags_37 },
    { 0x5907462, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53507466, SUC_lexicon_tags_15 },
    { 0x85d07466, SUC_lexicon_tags_37 },
    { 0x3df07468, SUC_lexicon_tags_4 },
    { 0xf5d07468, SUC_lexicon_tags_13 },
    { 0x5430746a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f07471, SUC_lexicon_tags_25 },
    { 0xbf307471, SUC_lexicon_tags_9 },
    { 0x6d07471, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f0747e, SUC_lexicon_tags_81 },
    { 0x6eb0747f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98107481, SUC_lexicon_tags_17 },
    { 0x98d07482, SUC_lexicon_tags_37 },
    { 0x63307483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34707487, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc670748d, SUC_lexicon_tags_14 },
    { 0x5a10748e, SUC_lexicon_tags_9 },
    { 0x6bd0748f, SUC_lexicon_tags_57 },
    { 0x52d0748f, SUC_lexicon_tags_60 },
    { 0xa90748d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0307494, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x43707496, SUC_lexicon_tags_60 },
    { 0xba707497, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d0749a, SUC_lexicon_tags_26 },
    { 0xf1b0749b, SUC_lexicon_tags_23 },
    { 0xddf0749c, SUC_lexicon_tags_29 },
    { 0x8550749c, SUC_lexicon_tags_36 },
    { 0xd3f0749e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15074a4, SUC_lexicon_tags_17 },
    { 0xe05074a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1074a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x139074ab, SUC_lexicon_tags_2 },
    { 0x7d5074ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77b074ae, SUC_lexicon_tags_15 },
    { 0xa11074af, SUC_lexicon_tags_11 },
    { 0x6d9074af, SUC_lexicon_tags_8 },
    { 0xd99074af, SUC_lexicon_tags_27 },
    { 0xc2f074ae, SUC_lexicon_tags_136 },
    { 0xe37074b3, SUC_lexicon_tags_1 },
    { 0x711074b4, SUC_lexicon_tags_12 },
    { 0x243074b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325074ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b074c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb49074c7, SUC_lexicon_tags_9 },
    { 0xa75074c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f074cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71074d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d074d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd074d6, SUC_lexicon_tags_60 },
    { 0x18d074d7, SUC_lexicon_tags_15 },
    { 0x2e3074d7, SUC_lexicon_tags_27 },
    { 0x47f074d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x737074db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693074e0, SUC_lexicon_tags_60 },
    { 0x6a3074e1, SUC_lexicon_tags_11 },
    { 0x7c9074e2, SUC_lexicon_tags_68 },
    { 0x3d3074e3, SUC_lexicon_tags_1 },
    { 0xe9d074e4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xab5074e6, SUC_lexicon_tags_11 },
    { 0xe97074e6, SUC_lexicon_tags_31 },
    { 0x8c9074e6, SUC_lexicon_tags_25 },
    { 0x99f074e9, SUC_lexicon_tags_107 },
    { 0xa91074e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d074f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x693074f7, SUC_lexicon_tags_11 },
    { 0x997074f8, SUC_lexicon_tags_25 },
    { 0x9c9074f8, SUC_lexicon_tags_15 },
    { 0x48b074f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d074fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b07501, SUC_lexicon_tags_7 },
    { 0xc3307502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57907504, SUC_lexicon_tags_43 },
    { 0x31507504, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26707507, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0707509, SUC_lexicon_tags_43 },
    { 0xf1107509, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76307512, SUC_lexicon_tags_11 },
    { 0xd5d07512, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe107515, SUC_lexicon_tags_39 },
    { 0x4fd07515, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29907518, SUC_lexicon_tags_12 },
    { 0xe3d07518, SUC_lexicon_tags_11 },
    { 0xc5f0751a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d0751e, SUC_lexicon_tags_26 },
    { 0xbc30751f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf07524, SUC_lexicon_tags_25 },
    { 0xbe107525, SUC_lexicon_tags_11 },
    { 0x73307524, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3430752c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0307530, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x18f07532, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b07538, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1690753a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8ab0753c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f50753e, SUC_lexicon_tags_11 },
    { 0x56f0753e, SUC_lexicon_tags_32 },
    { 0xf8f0753e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79907542, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb0107544, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10754b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba707551, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x2ed07553, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x707555, SUC_lexicon_tags_15 },
    { 0x53907556, SUC_lexicon_tags_63 },
    { 0x8c107557, SUC_lexicon_tags_12 },
    { 0x13907558, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f0755b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40507561, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8870756e, SUC_lexicon_tags_47 },
    { 0x87d0756f, SUC_lexicon_tags_654 },
    { 0, NULL },
    { 0x7a107571, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf7307573, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71907579, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a907583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10507585, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfc907587, SUC_lexicon_tags_12 },
    { 0x6eb07587, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3b90758a, SUC_lexicon_tags_14 },
    { 0x95d0758b, SUC_lexicon_tags_26 },
    { 0xebb0758a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad07591, SUC_lexicon_tags_35 },
    { 0x7f707592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d107595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c307597, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d0759a, SUC_lexicon_tags_82 },
    { 0xc70759b, SUC_lexicon_tags_6 },
    { 0xf150759c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177075a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x137075a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39075a7, SUC_lexicon_tags_2 },
    { 0x79d075a8, SUC_lexicon_tags_43 },
    { 0x1ff075a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29d075ab, SUC_lexicon_tags_28 },
    { 0x70f075ac, SUC_lexicon_tags_9 },
    { 0x437075ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983075b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x853075b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605075bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f075c1, SUC_lexicon_tags_7 },
    { 0x4fd075c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b075c5, SUC_lexicon_tags_12 },
    { 0xa4b075c5, SUC_lexicon_tags_11 },
    { 0x9cf075c7, SUC_lexicon_tags_17 },
    { 0x94d075c6, SUC_lexicon_tags_17 },
    { 0x773075c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7075cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05075d7, SUC_lexicon_tags_5 },
    { 0xf07075d8, SUC_lexicon_tags_0 },
    { 0x3c5075d9, SUC_lexicon_tags_7 },
    { 0xc7075da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f075dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1075e0, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db075e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad075e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca1075eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121075f0, SUC_lexicon_tags_23 },
    { 0x3e5075f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b075f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767075fc, SUC_lexicon_tags_11 },
    { 0x507075fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9907600, SUC_lexicon_tags_27 },
    { 0xe3b07601, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db07607, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b0760f, SUC_lexicon_tags_9 },
    { 0x60707610, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae307614, SUC_lexicon_tags_11 },
    { 0xbbf07615, SUC_lexicon_tags_19 },
    { 0xa0b07614, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe990761a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2990761c, SUC_lexicon_tags_15 },
    { 0x63f0761c, SUC_lexicon_tags_11 },
    { 0xed50761d, SUC_lexicon_tags_45 },
    { 0xaa90761f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b07622, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22907625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1507627, SUC_lexicon_tags_65 },
    { 0x4bf07628, SUC_lexicon_tags_0 },
    { 0xa4b07627, SUC_lexicon_tags_84 },
    { 0x57d0762a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f30762d, SUC_lexicon_tags_11 },
    { 0x55d0762d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96707638, SUC_lexicon_tags_11 },
    { 0x76707639, SUC_lexicon_tags_38 },
    { 0xa5b07638, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9590763c, SUC_lexicon_tags_15 },
    { 0xd830763d, SUC_lexicon_tags_4 },
    { 0x3350763e, SUC_lexicon_tags_7 },
    { 0xedb0763f, SUC_lexicon_tags_63 },
    { 0xc7d07640, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd07643, SUC_lexicon_tags_1 },
    { 0x43d07644, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d07647, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1c907649, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2107650, SUC_lexicon_tags_44 },
    { 0xb1d07651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe707659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x190765e, SUC_lexicon_tags_7 },
    { 0x13f0765f, SUC_lexicon_tags_63 },
    { 0xeb30765f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x61307662, SUC_lexicon_tags_47 },
    { 0xef907662, SUC_lexicon_tags_140 },
    { 0x58507662, SUC_lexicon_tags_68 },
    { 0x10307663, SUC_lexicon_tags_2 },
    { 0xe6b07666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1507668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd07670, SUC_lexicon_tags_38 },
    { 0xf3f07670, SUC_lexicon_tags_7 },
    { 0xfcb07672, SUC_lexicon_tags_60 },
    { 0xc2107672, SUC_lexicon_tags_15 },
    { 0x6d707672, SUC_lexicon_tags_97 },
    { 0x91b07673, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf07679, SUC_lexicon_tags_17 },
    { 0x7b07679, SUC_lexicon_tags_25 },
    { 0xb10767b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd0767f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d07684, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a907686, SUC_lexicon_tags_14 },
    { 0x91d07687, SUC_lexicon_tags_34 },
    { 0xadd07688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73f0768a, SUC_lexicon_tags_37 },
    { 0x9350768b, SUC_lexicon_tags_0 },
    { 0x7530768c, SUC_lexicon_tags_32 },
    { 0xd90768d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe0d0768f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4507691, SUC_lexicon_tags_17 },
    { 0x28d07692, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcbd07694, SUC_lexicon_tags_17 },
    { 0x6c307695, SUC_lexicon_tags_35 },
    { 0xd6707696, SUC_lexicon_tags_14 },
    { 0x1f507696, SUC_lexicon_tags_43 },
    { 0xebd07696, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7f0769a, SUC_lexicon_tags_1 },
    { 0x86d0769b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9eb0769d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f076a0, SUC_lexicon_tags_37 },
    { 0x29f076a1, SUC_lexicon_tags_3 },
    { 0xe1f076a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271076a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x153076ab, SUC_lexicon_tags_14 },
    { 0xb8b076ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb076b1, SUC_lexicon_tags_11 },
    { 0x4d9076b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57076b6, SUC_lexicon_tags_165 },
    { 0xf07076b6, SUC_lexicon_tags_15 },
    { 0x91076b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85076bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x287076bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423076c4, SUC_lexicon_tags_11 },
    { 0xf6f076c5, SUC_lexicon_tags_26 },
    { 0x589076c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737076ca, SUC_lexicon_tags_11 },
    { 0x8bf076cb, SUC_lexicon_tags_37 },
    { 0xd2b076cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdb9076ce, SUC_lexicon_tags_11 },
    { 0x689076cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9076d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851076df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f076e2, SUC_lexicon_tags_2 },
    { 0x459076e3, SUC_lexicon_tags_45 },
    { 0xb7b076e2, SUC_lexicon_tags_11 },
    { 0x985076e5, SUC_lexicon_tags_27 },
    { 0x441076e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7076e9, SUC_lexicon_tags_8 },
    { 0xc39076e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699076ee, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x475076f0, SUC_lexicon_tags_48 },
    { 0x913076f1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d076f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3076fd, SUC_lexicon_tags_27 },
    { 0x6bb076fd, SUC_lexicon_tags_37 },
    { 0x5f3076ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce507704, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x79d07706, SUC_lexicon_tags_1 },
    { 0xd4107707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa110770b, SUC_lexicon_tags_15 },
    { 0x4e10770c, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x5810770f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b07715, SUC_lexicon_tags_47 },
    { 0x49107716, SUC_lexicon_tags_27 },
    { 0x21b07715, SUC_lexicon_tags_38 },
    { 0xaf107717, SUC_lexicon_tags_28 },
    { 0xc4507719, SUC_lexicon_tags_25 },
    { 0x25d07719, SUC_lexicon_tags_44 },
    { 0x92d07719, SUC_lexicon_tags_63 },
    { 0x8610771c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29907720, SUC_lexicon_tags_28 },
    { 0x9bf07721, SUC_lexicon_tags_7 },
    { 0x77307720, SUC_lexicon_tags_34 },
    { 0x3d307720, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e70772b, SUC_lexicon_tags_84 },
    { 0xfcf0772c, SUC_lexicon_tags_25 },
    { 0x97f0772d, SUC_lexicon_tags_7 },
    { 0xd4d0772c, SUC_lexicon_tags_116 },
    { 0xcbb0772f, SUC_lexicon_tags_43 },
    { 0x61107730, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x26307732, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x36907735, SUC_lexicon_tags_70 },
    { 0xe8f07736, SUC_lexicon_tags_15 },
    { 0xab107737, SUC_lexicon_tags_27 },
    { 0xaed07737, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f0773f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x72f07741, SUC_lexicon_tags_56 },
    { 0xa4507742, SUC_lexicon_tags_61 },
    { 0x53d07742, SUC_lexicon_tags_43 },
    { 0x86f07744, SUC_lexicon_tags_6 },
    { 0x6107742, SUC_lexicon_tags_36 },
    { 0xcab07742, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb07749, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd0775e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x22107761, SUC_lexicon_tags_39 },
    { 0xcb307762, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb07768, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d0776b, SUC_lexicon_tags_77 },
    { 0x9770776c, SUC_lexicon_tags_2 },
    { 0xbed0776d, SUC_lexicon_tags_17 },
    { 0x2eb0776d, SUC_lexicon_tags_2 },
    { 0x47f0776d, SUC_lexicon_tags_12 },
    { 0x5d507770, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded07774, SUC_lexicon_tags_45 },
    { 0xd6f07774, SUC_lexicon_tags_17 },
    { 0x96b07776, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7907778, SUC_lexicon_tags_11 },
    { 0x15707779, SUC_lexicon_tags_65 },
    { 0x1190777a, SUC_lexicon_tags_11 },
    { 0xe410777a, SUC_lexicon_tags_17 },
    { 0xc350777b, SUC_lexicon_tags_25 },
    { 0x7310777d, SUC_lexicon_tags_15 },
    { 0xae30777d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x95107780, SUC_lexicon_tags_15 },
    { 0x99907781, SUC_lexicon_tags_12 },
    { 0x60f07780, SUC_lexicon_tags_63 },
    { 0xc5b07782, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79507788, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f0778b, SUC_lexicon_tags_65 },
    { 0xbd30778c, SUC_lexicon_tags_56 },
    { 0x50b0778c, SUC_lexicon_tags_25 },
    { 0x9050778e, SUC_lexicon_tags_10 },
    { 0xf4f0778e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a507793, SUC_lexicon_tags_9 },
    { 0x85307794, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d07798, SUC_lexicon_tags_80 },
    { 0x9b07799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf790779d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17f0779f, SUC_lexicon_tags_26 },
    { 0xcf90779f, SUC_lexicon_tags_55 },
    { 0xf17077a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x393077a3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725077a8, SUC_lexicon_tags_11 },
    { 0x783077a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407077b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b077b3, SUC_lexicon_tags_2 },
    { 0xf9b077b3, SUC_lexicon_tags_22 },
    { 0x837077b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67077bc, SUC_lexicon_tags_12 },
    { 0xd3f077bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953077c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab077c3, SUC_lexicon_tags_47 },
    { 0x3e5077c3, SUC_lexicon_tags_40 },
    { 0x971077c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05077c8, SUC_lexicon_tags_31 },
    { 0x3fd077c9, SUC_lexicon_tags_1 },
    { 0x177077ca, SUC_lexicon_tags_38 },
    { 0x2cb077cb, SUC_lexicon_tags_23 },
    { 0xae5077c8, SUC_lexicon_tags_63 },
    { 0x1a1077cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x65d077cf, SUC_lexicon_tags_91 },
    { 0x81b077d0, SUC_lexicon_tags_32 },
    { 0xa9f077d1, SUC_lexicon_tags_11 },
    { 0x575077d0, SUC_lexicon_tags_11 },
    { 0x907077d0, SUC_lexicon_tags_15 },
    { 0x9c1077d3, SUC_lexicon_tags_15 },
    { 0xd6f077d5, SUC_lexicon_tags_25 },
    { 0xe01077d6, SUC_lexicon_tags_45 },
    { 0xb7b077d7, SUC_lexicon_tags_66 },
    { 0xef077d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1077e3, SUC_lexicon_tags_2 },
    { 0x99d077e3, SUC_lexicon_tags_17 },
    { 0xcdf077e3, SUC_lexicon_tags_44 },
    { 0xeed077e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189077ea, SUC_lexicon_tags_16 },
    { 0xf75077eb, SUC_lexicon_tags_82 },
    { 0x471077ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x195077ee, SUC_lexicon_tags_0 },
    { 0x5fb077ee, SUC_lexicon_tags_17 },
    { 0xaff077f0, SUC_lexicon_tags_27 },
    { 0x175077f1, SUC_lexicon_tags_31 },
    { 0x2d1077f0, SUC_lexicon_tags_9 },
    { 0xd6d077f1, SUC_lexicon_tags_2 },
    { 0xd55077ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84b077f6, SUC_lexicon_tags_15 },
    { 0xd47077f6, SUC_lexicon_tags_28 },
    { 0x9cd077f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107800, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf07803, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb507807, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb590780b, SUC_lexicon_tags_19 },
    { 0x7130780c, SUC_lexicon_tags_37 },
    { 0xf250780d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd50780f, SUC_lexicon_tags_47 },
    { 0x2c107810, SUC_lexicon_tags_25 },
    { 0xdf107811, SUC_lexicon_tags_12 },
    { 0x51507812, SUC_lexicon_tags_15 },
    { 0xe4707812, SUC_lexicon_tags_37 },
    { 0x10f07812, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd0d07816, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x9ad07818, SUC_lexicon_tags_29 },
    { 0x8a307819, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1907822, SUC_lexicon_tags_26 },
    { 0xdc507823, SUC_lexicon_tags_14 },
    { 0x8b07823, SUC_lexicon_tags_160 },
    { 0xc8f07825, SUC_lexicon_tags_16 },
    { 0xda307826, SUC_lexicon_tags_25 },
    { 0x25507823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85107829, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d0782d, SUC_lexicon_tags_1 },
    { 0x8750782e, SUC_lexicon_tags_47 },
    { 0x7ed0782f, SUC_lexicon_tags_60 },
    { 0xea70782e, SUC_lexicon_tags_11 },
    { 0xfc907831, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d30783d, SUC_lexicon_tags_28 },
    { 0x4ff0783e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaf07840, SUC_lexicon_tags_37 },
    { 0x63507841, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9907844, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee70784a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10784e, SUC_lexicon_tags_37 },
    { 0x27f0784e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6907851, SUC_lexicon_tags_25 },
    { 0xe9507851, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8307855, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x670785a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad307861, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2107866, SUC_lexicon_tags_49 },
    { 0xbf307867, SUC_lexicon_tags_28 },
    { 0x28b07866, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1310786d, SUC_lexicon_tags_2 },
    { 0x5eb0786e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7307871, SUC_lexicon_tags_9 },
    { 0xadd07871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b707874, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d07877, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab10787b, SUC_lexicon_tags_33 },
    { 0x24b0787c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba507880, SUC_lexicon_tags_6 },
    { 0xfb907881, SUC_lexicon_tags_8 },
    { 0x9e107881, SUC_lexicon_tags_3 },
    { 0x62f07881, SUC_lexicon_tags_192 },
    { 0x2cb07883, SUC_lexicon_tags_33 },
    { 0x4d307885, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x14107887, SUC_lexicon_tags_37 },
    { 0xf307888, SUC_lexicon_tags_37 },
    { 0x8ff07889, SUC_lexicon_tags_12 },
    { 0x73507889, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc710788e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87707893, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa270789b, SUC_lexicon_tags_2 },
    { 0x6d90789c, SUC_lexicon_tags_87 },
    { 0xb150789d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b078a2, SUC_lexicon_tags_138 },
    { 0xf11078a2, SUC_lexicon_tags_11 },
    { 0xd8b078a2, SUC_lexicon_tags_38 },
    { 0x11f078a3, SUC_lexicon_tags_15 },
    { 0xb0f078a6, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x4b1078a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf7078aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf078ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e3078af, SUC_lexicon_tags_16 },
    { 0x9ed078b0, SUC_lexicon_tags_25 },
    { 0x421078b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x483078b3, SUC_lexicon_tags_8 },
    { 0x37078b4, SUC_lexicon_tags_7 },
    { 0xa61078b4, SUC_lexicon_tags_23 },
    { 0xe9d078b6, SUC_lexicon_tags_23 },
    { 0xd91078b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd078bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633078c2, SUC_lexicon_tags_1 },
    { 0x40f078c3, SUC_lexicon_tags_8 },
    { 0x525078c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x743078c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69078c9, SUC_lexicon_tags_27 },
    { 0x43b078ca, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0x567078cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449078d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53078d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x84b078d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91078e1, SUC_lexicon_tags_11 },
    { 0x3bb078e2, SUC_lexicon_tags_19 },
    { 0xd55078e3, SUC_lexicon_tags_19 },
    { 0x183078e4, SUC_lexicon_tags_14 },
    { 0x6c9078e5, SUC_lexicon_tags_7 },
    { 0x7f7078e6, SUC_lexicon_tags_38 },
    { 0x8bd078e3, SUC_lexicon_tags_56 },
    { 0xa3d078e8, SUC_lexicon_tags_17 },
    { 0xe0d078e9, SUC_lexicon_tags_3 },
    { 0xac5078e8, SUC_lexicon_tags_98 },
    { 0x2ef078ea, SUC_lexicon_tags_87 },
    { 0x8a9078ec, SUC_lexicon_tags_1 },
    { 0x78f078e2, SUC_lexicon_tags_113 },
    { 0x169078ec, SUC_lexicon_tags_11 },
    { 0x70b078ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21078f3, SUC_lexicon_tags_13 },
    { 0x18b078f3, SUC_lexicon_tags_33 },
    { 0xdff078f5, SUC_lexicon_tags_2 },
    { 0x96d078f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x421078f9, SUC_lexicon_tags_1 },
    { 0x39d078f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25d078fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8f1078fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x77907900, SUC_lexicon_tags_23 },
    { 0xf07901, SUC_lexicon_tags_17 },
    { 0x1f907901, SUC_lexicon_tags_0 },
    { 0x38907900, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23907905, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f307909, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f0790c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ad0790e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d07913, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8f507915, SUC_lexicon_tags_15 },
    { 0x5e707915, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c30791d, SUC_lexicon_tags_12 },
    { 0x8110791e, SUC_lexicon_tags_39 },
    { 0x3f50791f, SUC_lexicon_tags_25 },
    { 0x9f307920, SUC_lexicon_tags_29 },
    { 0xb1f07921, SUC_lexicon_tags_9 },
    { 0xe450791e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7d07924, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf5907926, SUC_lexicon_tags_1 },
    { 0xc8f07927, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x43f07929, SUC_lexicon_tags_45 },
    { 0xf170792a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e70792d, SUC_lexicon_tags_29 },
    { 0x950792e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa3707930, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40107936, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8550793b, SUC_lexicon_tags_63 },
    { 0x78d0793b, SUC_lexicon_tags_28 },
    { 0x90b0793d, SUC_lexicon_tags_25 },
    { 0xb7f0793b, SUC_lexicon_tags_70 },
    { 0x9d90793f, SUC_lexicon_tags_15 },
    { 0x50107940, SUC_lexicon_tags_33 },
    { 0xa0307940, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff0794a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4690794d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2070794f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb07951, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d07956, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x4107958, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d0795e, SUC_lexicon_tags_7 },
    { 0x3250795f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f07964, SUC_lexicon_tags_28 },
    { 0xb5907964, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x99b07967, SUC_lexicon_tags_18 },
    { 0xc6d07967, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b0796f, SUC_lexicon_tags_87 },
    { 0xf570796f, SUC_lexicon_tags_13 },
    { 0x1f507970, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa307979, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f0797c, SUC_lexicon_tags_37 },
    { 0x1610797c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34d0797f, SUC_lexicon_tags_66 },
    { 0x3ef07980, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36507987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb907990, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30307993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d70799c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d0799f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f079a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7079a7, SUC_lexicon_tags_42 },
    { 0x909079a8, SUC_lexicon_tags_91 },
    { 0x227079a9, SUC_lexicon_tags_26 },
    { 0x559079aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf3f079ac, SUC_lexicon_tags_8 },
    { 0xb69079ad, SUC_lexicon_tags_14 },
    { 0xb21079ae, SUC_lexicon_tags_17 },
    { 0xd15079af, SUC_lexicon_tags_5 },
    { 0x9bf079af, SUC_lexicon_tags_2 },
    { 0x8e7079ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdcb079b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d079b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd079bc, SUC_lexicon_tags_87 },
    { 0x43f079bd, SUC_lexicon_tags_15 },
    { 0x4dd079be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d079c2, SUC_lexicon_tags_11 },
    { 0x4f9079c3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287079ca, SUC_lexicon_tags_14 },
    { 0x5a9079cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x721079ce, SUC_lexicon_tags_25 },
    { 0x903079ce, SUC_lexicon_tags_37 },
    { 0xd11079cf, SUC_lexicon_tags_27 },
    { 0xb5f079d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x967079d4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1079d8, SUC_lexicon_tags_7 },
    { 0xf43079d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa93079db, SUC_lexicon_tags_37 },
    { 0xb37079dc, SUC_lexicon_tags_23 },
    { 0x93079dd, SUC_lexicon_tags_14 },
    { 0x2a1079de, SUC_lexicon_tags_112 },
    { 0x637079df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f079e5, SUC_lexicon_tags_37 },
    { 0x9b9079e6, SUC_lexicon_tags_7 },
    { 0xcf7079e7, SUC_lexicon_tags_15 },
    { 0x169079e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943079ed, SUC_lexicon_tags_7 },
    { 0xc51079ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11079f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x647079f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb17079f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7079fe, SUC_lexicon_tags_9 },
    { 0xb3079ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x83907a01, SUC_lexicon_tags_15 },
    { 0x15b07a01, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff07a06, SUC_lexicon_tags_17 },
    { 0xa5707a06, SUC_lexicon_tags_19 },
    { 0x8507a08, SUC_lexicon_tags_6 },
    { 0xf1d07a06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34107a0c, SUC_lexicon_tags_26 },
    { 0xb0507a0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f07a13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x95507a16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5907a18, SUC_lexicon_tags_12 },
    { 0x37107a18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b07a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11707a21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa107a23, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b07a27, SUC_lexicon_tags_17 },
    { 0xa9b07a28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4307a2a, SUC_lexicon_tags_27 },
    { 0x9d507a2b, SUC_lexicon_tags_68 },
    { 0x51507a2c, SUC_lexicon_tags_2 },
    { 0x88107a2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9507a2f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f07a33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x13307a36, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x46907a39, SUC_lexicon_tags_140 },
    { 0xd0907a3a, SUC_lexicon_tags_129 },
    { 0x8707a39, SUC_lexicon_tags_11 },
    { 0x2dd07a3c, SUC_lexicon_tags_11 },
    { 0x1107a39, SUC_lexicon_tags_33 },
    { 0x73b07a3a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d07a43, SUC_lexicon_tags_9 },
    { 0xc7b07a44, SUC_lexicon_tags_21 },
    { 0x7b907a44, SUC_lexicon_tags_33 },
    { 0xbd907a46, SUC_lexicon_tags_47 },
    { 0xaf507a47, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b07a4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b07a51, SUC_lexicon_tags_57 },
    { 0x1df07a51, SUC_lexicon_tags_49 },
    { 0x94107a51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1d07a55, SUC_lexicon_tags_25 },
    { 0x53107a56, SUC_lexicon_tags_17 },
    { 0xef507a57, SUC_lexicon_tags_60 },
    { 0x23107a58, SUC_lexicon_tags_43 },
    { 0xd0f07a57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x91507a5c, SUC_lexicon_tags_49 },
    { 0x82907a5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x54907a60, SUC_lexicon_tags_55 },
    { 0x9d107a60, SUC_lexicon_tags_11 },
    { 0x1af07a62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa8707a64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4707a6d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f07a71, SUC_lexicon_tags_42 },
    { 0x9307a72, SUC_lexicon_tags_15 },
    { 0x21907a72, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed07a77, SUC_lexicon_tags_12 },
    { 0xc0d07a78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf307a7b, SUC_lexicon_tags_34 },
    { 0x1a907a7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd07a80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fb07a82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb107a8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf107a8c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef907a93, SUC_lexicon_tags_2 },
    { 0x51707a93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0907a97, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60507a9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd07aa2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36707aa8, SUC_lexicon_tags_9 },
    { 0xd4b07aa9, SUC_lexicon_tags_26 },
    { 0x18b07aa9, SUC_lexicon_tags_25 },
    { 0x3e307aa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd907ab0, SUC_lexicon_tags_14 },
    { 0x5f07ab0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78107ab3, SUC_lexicon_tags_31 },
    { 0x43507ab4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb507ab8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3707abb, SUC_lexicon_tags_15 },
    { 0x87d07abb, SUC_lexicon_tags_37 },
    { 0xf1507abc, SUC_lexicon_tags_7 },
    { 0x7cb07abe, SUC_lexicon_tags_43 },
    { 0x5af07abe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a507ac2, SUC_lexicon_tags_11 },
    { 0x39907ac3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44507ac6, SUC_lexicon_tags_82 },
    { 0xcab07ac7, SUC_lexicon_tags_0 },
    { 0x57d07ac8, SUC_lexicon_tags_31 },
    { 0x9c507ac8, SUC_lexicon_tags_33 },
    { 0xa3707aca, SUC_lexicon_tags_37 },
    { 0xded07aca, SUC_lexicon_tags_7 },
    { 0x90b07acc, SUC_lexicon_tags_35 },
    { 0x4b07acd, SUC_lexicon_tags_82 },
    { 0xae707aca, SUC_lexicon_tags_38 },
    { 0xf9f07ace, SUC_lexicon_tags_17 },
    { 0x9d907acc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdd07ad2, SUC_lexicon_tags_19 },
    { 0xec707ad3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f07ad7, SUC_lexicon_tags_12 },
    { 0x28f07ad8, SUC_lexicon_tags_7 },
    { 0x5a107ad9, SUC_lexicon_tags_2 },
    { 0x2af07ada, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a507add, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x20107adf, SUC_lexicon_tags_11 },
    { 0x19507adf, SUC_lexicon_tags_37 },
    { 0x3bf07ae1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d07ae5, SUC_lexicon_tags_11 },
    { 0x21d07ae5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x59507ae8, SUC_lexicon_tags_15 },
    { 0xc5b07ae9, SUC_lexicon_tags_15 },
    { 0xe2107aea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3907af3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f07af7, SUC_lexicon_tags_12 },
    { 0x7d07af8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee107afc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a107aff, SUC_lexicon_tags_8 },
    { 0x9bd07aff, SUC_lexicon_tags_17 },
    { 0xd3f07b00, SUC_lexicon_tags_1 },
    { 0x51707b02, SUC_lexicon_tags_27 },
    { 0xa3b07b03, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98507b09, SUC_lexicon_tags_46 },
    { 0xce907b09, SUC_lexicon_tags_55 },
    { 0xc307b0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f07b0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d07b14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f107b1e, SUC_lexicon_tags_17 },
    { 0x32907b1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b07b22, SUC_lexicon_tags_25 },
    { 0x1e507b22, SUC_lexicon_tags_23 },
    { 0xeb307b24, SUC_lexicon_tags_11 },
    { 0x62b07b25, SUC_lexicon_tags_113 },
    { 0x43f07b25, SUC_lexicon_tags_27 },
    { 0x10107b24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab07b2c, SUC_lexicon_tags_25 },
    { 0xd9d07b2d, SUC_lexicon_tags_17 },
    { 0x2d107b2c, SUC_lexicon_tags_2 },
    { 0x10d07b2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b307b31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b07b35, SUC_lexicon_tags_63 },
    { 0xe9107b35, SUC_lexicon_tags_6 },
    { 0x9f07b36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7507b3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed307b47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c107b53, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d07b56, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95107b5f, SUC_lexicon_tags_12 },
    { 0xa707b60, SUC_lexicon_tags_31 },
    { 0x4e707b61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93f07b63, SUC_lexicon_tags_61 },
    { 0xc5707b63, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20307b68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707b71, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb07b74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb2307b76, SUC_lexicon_tags_23 },
    { 0x37107b77, SUC_lexicon_tags_198 },
    { 0x28307b78, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f07b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6707b80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65b07b82, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95707b87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c507b8a, SUC_lexicon_tags_12 },
    { 0x41507b8a, SUC_lexicon_tags_6 },
    { 0x55307b8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfb07b8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3307b91, SUC_lexicon_tags_45 },
    { 0x18907b91, SUC_lexicon_tags_1 },
    { 0xc0307b91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8907b96, SUC_lexicon_tags_27 },
    { 0x81f07b96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d07b9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8507b9d, SUC_lexicon_tags_37 },
    { 0x41f07b9e, SUC_lexicon_tags_19 },
    { 0xd9507b9e, SUC_lexicon_tags_17 },
    { 0xc9b07ba0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbb07ba2, SUC_lexicon_tags_700 },
    { 0x83307ba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3107ba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d07ba8, SUC_lexicon_tags_15 },
    { 0x7a707ba9, SUC_lexicon_tags_15 },
    { 0x73907baa, SUC_lexicon_tags_37 },
    { 0xaf707bab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf0d07bad, SUC_lexicon_tags_37 },
    { 0x17307bad, SUC_lexicon_tags_25 },
    { 0x9cd07bae, SUC_lexicon_tags_38 },
    { 0x57907bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f07bb4, SUC_lexicon_tags_7 },
    { 0x94907bb5, SUC_lexicon_tags_17 },
    { 0x96907bb5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x89307bb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d07bbb, SUC_lexicon_tags_60 },
    { 0x98f07bbb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd07bbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6507bc8, SUC_lexicon_tags_47 },
    { 0xf7b07bc8, SUC_lexicon_tags_22 },
    { 0x5cd07bca, SUC_lexicon_tags_11 },
    { 0xd6707bc9, SUC_lexicon_tags_82 },
    { 0x48907bc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8907bd1, SUC_lexicon_tags_31 },
    { 0x3a107bd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9907bdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d07be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f07be9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78907bec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x68907bef, SUC_lexicon_tags_17 },
    { 0x85b07bf0, SUC_lexicon_tags_37 },
    { 0xf4d07bf1, SUC_lexicon_tags_25 },
    { 0xf8f07bf2, SUC_lexicon_tags_15 },
    { 0xf2907bf3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d07bf6, SUC_lexicon_tags_20 },
    { 0x7b907bf6, SUC_lexicon_tags_1 },
    { 0x9fd07bf7, SUC_lexicon_tags_92 },
    { 0x66107bf9, SUC_lexicon_tags_7 },
    { 0x5f107bfa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d07bff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22307c13, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d07c17, SUC_lexicon_tags_15 },
    { 0x30f07c17, SUC_lexicon_tags_15 },
    { 0x4507c17, SUC_lexicon_tags_38 },
    { 0x55707c1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb07c28, SUC_lexicon_tags_6 },
    { 0xbe307c29, SUC_lexicon_tags_23 },
    { 0x57107c2a, SUC_lexicon_tags_37 },
    { 0x60307c29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30307c30, SUC_lexicon_tags_82 },
    { 0x57307c30, SUC_lexicon_tags_11 },
    { 0x15307c30, SUC_lexicon_tags_48 },
    { 0xb7107c30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58507c37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xff907c3a, SUC_lexicon_tags_11 },
    { 0x81d07c3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff07c3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69907c47, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed07c4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7507c4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d07c50, SUC_lexicon_tags_2 },
    { 0xeb07c51, SUC_lexicon_tags_9 },
    { 0xa8107c51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a507c5c, SUC_lexicon_tags_11 },
    { 0xc8307c5d, SUC_lexicon_tags_38 },
    { 0xe8f07c5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcb07c60, SUC_lexicon_tags_65 },
    { 0x45d07c61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd907c69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7307c6c, SUC_lexicon_tags_26 },
    { 0xca907c6d, SUC_lexicon_tags_23 },
    { 0x7dd07c6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8107c71, SUC_lexicon_tags_11 },
    { 0x9a307c72, SUC_lexicon_tags_14 },
    { 0x24907c71, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61707c77, SUC_lexicon_tags_1 },
    { 0x42f07c77, SUC_lexicon_tags_27 },
    { 0xaed07c79, SUC_lexicon_tags_37 },
    { 0xec307c79, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93707c7f, SUC_lexicon_tags_2 },
    { 0x45907c80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2b07c82, SUC_lexicon_tags_423 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf907c88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a707c8c, SUC_lexicon_tags_11 },
    { 0x5bf07c8c, SUC_lexicon_tags_1 },
    { 0x51f07c8d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83907c92, SUC_lexicon_tags_25 },
    { 0xa1707c93, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda307c9c, SUC_lexicon_tags_12 },
    { 0x93507c9d, SUC_lexicon_tags_61 },
    { 0x91307c9c, SUC_lexicon_tags_17 },
    { 0x7b07c9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x53d07ca1, SUC_lexicon_tags_37 },
    { 0xa7b07ca1, SUC_lexicon_tags_1 },
    { 0xe1b07ca2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab07ca8, SUC_lexicon_tags_7 },
    { 0x48507ca9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6707cab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f707caf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd07cb2, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0107cb5, SUC_lexicon_tags_12 },
    { 0xe3707cb5, SUC_lexicon_tags_37 },
    { 0xb1707cb6, SUC_lexicon_tags_37 },
    { 0xaf707cb8, SUC_lexicon_tags_156 },
    { 0xb4f07cb8, SUC_lexicon_tags_66 },
    { 0x4e907cb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6107cbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x51307cbf, SUC_lexicon_tags_15 },
    { 0xa2d07cc0, SUC_lexicon_tags_2 },
    { 0x42507cc0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfa107cc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39107cca, SUC_lexicon_tags_11 },
    { 0xe9d07cca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd07cd4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f07cda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa107ce2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd07ce7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb07cf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d07cf4, SUC_lexicon_tags_37 },
    { 0xc8507cf5, SUC_lexicon_tags_27 },
    { 0x5db07cf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9707cfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x49707cfe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd107d02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b07d04, SUC_lexicon_tags_11 },
    { 0xb7907d05, SUC_lexicon_tags_29 },
    { 0x1ad07d05, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd107d11, SUC_lexicon_tags_39 },
    { 0xbfd07d11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f707d15, SUC_lexicon_tags_1 },
    { 0xe2d07d16, SUC_lexicon_tags_9 },
    { 0xbf907d16, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdfb07d19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f707d1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b307d20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd07d24, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b07d27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd07d2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb07d36, SUC_lexicon_tags_12 },
    { 0x5f707d36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f07d3a, SUC_lexicon_tags_15 },
    { 0xdc307d3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df07d3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff07d44, SUC_lexicon_tags_11 },
    { 0xcf507d45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82707d47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f07d4d, SUC_lexicon_tags_30 },
    { 0x52907d4d, SUC_lexicon_tags_146 },
    { 0x4c907d4e, SUC_lexicon_tags_27 },
    { 0xc1d07d50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x70507d52, SUC_lexicon_tags_1 },
    { 0x99507d53, SUC_lexicon_tags_26 },
    { 0x57f07d54, SUC_lexicon_tags_12 },
    { 0x67d07d55, SUC_lexicon_tags_45 },
    { 0xf0307d56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ad07d58, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e907d5d, SUC_lexicon_tags_37 },
    { 0xbcf07d5d, SUC_lexicon_tags_23 },
    { 0xa0107d5f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x42707d61, SUC_lexicon_tags_66 },
    { 0x49907d62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeb707d64, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a107d6a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a707d6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f07d74, SUC_lexicon_tags_47 },
    { 0x29f07d75, SUC_lexicon_tags_25 },
    { 0x2d507d74, SUC_lexicon_tags_7 },
    { 0x22307d77, SUC_lexicon_tags_15 },
    { 0x93307d77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd07d88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea707d8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7df07d8e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x39507d90, SUC_lexicon_tags_49 },
    { 0x87507d90, SUC_lexicon_tags_37 },
    { 0xc1107d92, SUC_lexicon_tags_87 },
    { 0x96307d93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc0707d95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x82107d98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76707d9d, SUC_lexicon_tags_31 },
    { 0x6b507d9e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5507da1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60107da5, SUC_lexicon_tags_1 },
    { 0x33f07da6, SUC_lexicon_tags_1 },
    { 0x41307da7, SUC_lexicon_tags_12 },
    { 0x39507da8, SUC_lexicon_tags_8 },
    { 0x63b07da9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b07daf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50307db3, SUC_lexicon_tags_48 },
    { 0x80b07db4, SUC_lexicon_tags_25 },
    { 0xf307db5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8707db7, SUC_lexicon_tags_33 },
    { 0x42d07db8, SUC_lexicon_tags_65 },
    { 0xb7707db9, SUC_lexicon_tags_47 },
    { 0xcd907dba, SUC_lexicon_tags_19 },
    { 0x1e307dbb, SUC_lexicon_tags_37 },
    { 0x37307db8, SUC_lexicon_tags_12 },
    { 0xa2b07dbd, SUC_lexicon_tags_60 },
    { 0x31707dbd, SUC_lexicon_tags_7 },
    { 0x2ff07dbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90907dc5, SUC_lexicon_tags_38 },
    { 0xc7507dc6, SUC_lexicon_tags_8 },
    { 0xeef07dc7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8eb07dc9, SUC_lexicon_tags_1 },
    { 0xe8b07dc9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf07dce, SUC_lexicon_tags_11 },
    { 0xfeb07dcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a707dd1, SUC_lexicon_tags_2 },
    { 0xc1707dd1, SUC_lexicon_tags_2 },
    { 0x95f07dd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f07dd6, SUC_lexicon_tags_31 },
    { 0xe5907dd7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca907ddb, SUC_lexicon_tags_14 },
    { 0x1d307ddc, SUC_lexicon_tags_17 },
    { 0x5c507ddb, SUC_lexicon_tags_17 },
    { 0x19107dde, SUC_lexicon_tags_57 },
    { 0xabb07ddc, SUC_lexicon_tags_29 },
    { 0xa0707de0, SUC_lexicon_tags_33 },
    { 0x82907de1, SUC_lexicon_tags_33 },
    { 0xed507de2, SUC_lexicon_tags_45 },
    { 0xb2d07de3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5507de9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8707df2, SUC_lexicon_tags_2 },
    { 0x24b07df3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd07df6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e907dfa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7e307dfc, SUC_lexicon_tags_60 },
    { 0xd8d07dfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b07e01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7107e04, SUC_lexicon_tags_29 },
    { 0xbbd07e05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb07e0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0d07e0d, SUC_lexicon_tags_7 },
    { 0xed707e0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d07e12, SUC_lexicon_tags_31 },
    { 0x15f07e13, SUC_lexicon_tags_49 },
    { 0xeef07e14, SUC_lexicon_tags_12 },
    { 0xa6507e15, SUC_lexicon_tags_12 },
    { 0x12707e15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b507e1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9107e1e, SUC_lexicon_tags_16 },
    { 0x91307e1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d07e23, SUC_lexicon_tags_0 },
    { 0x52107e24, SUC_lexicon_tags_7 },
    { 0x6b707e25, SUC_lexicon_tags_6 },
    { 0x29707e26, SUC_lexicon_tags_1 },
    { 0xe6307e24, SUC_lexicon_tags_7 },
    { 0x40d07e25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d07e2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd07e2f, SUC_lexicon_tags_7 },
    { 0x1e907e30, SUC_lexicon_tags_11 },
    { 0x99907e31, SUC_lexicon_tags_61 },
    { 0x44907e32, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6107e38, SUC_lexicon_tags_74 },
    { 0x58b07e38, SUC_lexicon_tags_12 },
    { 0x36f07e3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcf107e3c, SUC_lexicon_tags_63 },
    { 0x69f07e3d, SUC_lexicon_tags_21 },
    { 0x4cf07e3e, SUC_lexicon_tags_31 },
    { 0xb2307e3f, SUC_lexicon_tags_39 },
    { 0xe1107e3e, SUC_lexicon_tags_68 },
    { 0x72907e41, SUC_lexicon_tags_29 },
    { 0xf1907e41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda507e45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe9b07e47, SUC_lexicon_tags_38 },
    { 0xe1307e47, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf307e4b, SUC_lexicon_tags_92 },
    { 0x107e4c, SUC_lexicon_tags_49 },
    { 0xda907e4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc107e51, SUC_lexicon_tags_43 },
    { 0x95307e51, SUC_lexicon_tags_37 },
    { 0x74907e52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84107e55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4507e58, SUC_lexicon_tags_23 },
    { 0x49307e58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0307e5b, SUC_lexicon_tags_16 },
    { 0x93f07e5c, SUC_lexicon_tags_14 },
    { 0x4b107e5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f707e61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x33b07e63, SUC_lexicon_tags_11 },
    { 0x7b907e64, SUC_lexicon_tags_49 },
    { 0xe2907e65, SUC_lexicon_tags_33 },
    { 0x41107e66, SUC_lexicon_tags_25 },
    { 0x31d07e63, SUC_lexicon_tags_57 },
    { 0x38107e68, SUC_lexicon_tags_17 },
    { 0x2e707e69, SUC_lexicon_tags_2 },
    { 0xdcb07e6a, SUC_lexicon_tags_17 },
    { 0x5f707e68, SUC_lexicon_tags_63 },
    { 0x78907e6a, SUC_lexicon_tags_15 },
    { 0xf4107e6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x25707e70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d07e73, SUC_lexicon_tags_3 },
    { 0xa6f07e73, SUC_lexicon_tags_31 },
    { 0x65907e75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7707e78, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22707e7d, SUC_lexicon_tags_65 },
    { 0x62b07e7d, SUC_lexicon_tags_16 },
    { 0x4d307e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22907e83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd307e86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad07e8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f07e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7507e93, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef07e96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44d07e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21107e9c, SUC_lexicon_tags_15 },
    { 0x3bf07e9c, SUC_lexicon_tags_7 },
    { 0x39d07e9d, SUC_lexicon_tags_37 },
    { 0x59907e9f, SUC_lexicon_tags_33 },
    { 0xfd907e9e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a307ea4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b707ea6, SUC_lexicon_tags_27 },
    { 0x9bf07ea7, SUC_lexicon_tags_33 },
    { 0xb0707ea6, SUC_lexicon_tags_8 },
    { 0x60307ea9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d07eb3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d07eb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36307eba, SUC_lexicon_tags_902 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d07ebd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x17307ebf, SUC_lexicon_tags_9 },
    { 0x9307ec0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77107ec4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc5b07ec6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f07eca, SUC_lexicon_tags_207 },
    { 0x24d07ecb, SUC_lexicon_tags_3 },
    { 0x1d107ecb, SUC_lexicon_tags_37 },
    { 0x21507ecd, SUC_lexicon_tags_30 },
    { 0x87107ecc, SUC_lexicon_tags_11 },
    { 0x22107ecf, SUC_lexicon_tags_15 },
    { 0x15507ecf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76307ed4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef07ed9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d07ede, SUC_lexicon_tags_87 },
    { 0x51107edf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d307ee4, SUC_lexicon_tags_17 },
    { 0x2707ee5, SUC_lexicon_tags_25 },
    { 0x3e107ee6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3907ee9, SUC_lexicon_tags_2 },
    { 0x50307eea, SUC_lexicon_tags_23 },
    { 0xbab07eeb, SUC_lexicon_tags_38 },
    { 0x73907eeb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ef07eee, SUC_lexicon_tags_23 },
    { 0x2f507eef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa507ef3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd507ef6, SUC_lexicon_tags_81 },
    { 0xb1907ef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c307ef9, SUC_lexicon_tags_15 },
    { 0x2f507efa, SUC_lexicon_tags_66 },
    { 0xa6707efa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b07efe, SUC_lexicon_tags_0 },
    { 0x24507efe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec507f02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd07f09, SUC_lexicon_tags_65 },
    { 0x92307f0a, SUC_lexicon_tags_11 },
    { 0xe3b07f0a, SUC_lexicon_tags_11 },
    { 0x75507f0a, SUC_lexicon_tags_11 },
    { 0x6f907f09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f07f13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38d07f15, SUC_lexicon_tags_14 },
    { 0x66b07f16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd707f1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe507f1e, SUC_lexicon_tags_56 },
    { 0x7af07f1e, SUC_lexicon_tags_7 },
    { 0xd3b07f20, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf707f23, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcf07f25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa1507f27, SUC_lexicon_tags_32 },
    { 0xb8d07f27, SUC_lexicon_tags_15 },
    { 0x98707f29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa7907f2b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f07f2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f307f31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd07f36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b07f3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xae107f3e, SUC_lexicon_tags_37 },
    { 0x75f07f3e, SUC_lexicon_tags_26 },
    { 0x21707f3e, SUC_lexicon_tags_11 },
    { 0xd1107f40, SUC_lexicon_tags_4 },
    { 0x2ff07f42, SUC_lexicon_tags_25 },
    { 0x1df07f43, SUC_lexicon_tags_3 },
    { 0xfd307f44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d07f47, SUC_lexicon_tags_2 },
    { 0x42b07f47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1507f4a, SUC_lexicon_tags_14 },
    { 0x60d07f4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a507f4e, SUC_lexicon_tags_30 },
    { 0x9d507f4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17307f51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f07f54, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb07f59, SUC_lexicon_tags_1 },
    { 0x59507f59, SUC_lexicon_tags_1 },
    { 0xf7b07f5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a707f5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21307f61, SUC_lexicon_tags_46 },
    { 0xf0707f62, SUC_lexicon_tags_25 },
    { 0x45f07f63, SUC_lexicon_tags_17 },
    { 0xa7d07f64, SUC_lexicon_tags_7 },
    { 0x34d07f61, SUC_lexicon_tags_31 },
    { 0xcc307f64, SUC_lexicon_tags_46 },
    { 0x7fb07f63, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xab307f69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cb07f6b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x89f07f6d, SUC_lexicon_tags_28 },
    { 0xc907f6e, SUC_lexicon_tags_17 },
    { 0x9b07f6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81307f75, SUC_lexicon_tags_17 },
    { 0x9b907f76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0107f79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb307f7b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb3507f7d, SUC_lexicon_tags_34 },
    { 0xd8d07f7d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f07f83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf7907f85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b07f8a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb07f9b, SUC_lexicon_tags_9 },
    { 0xc5307f9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0707f9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96707fa2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d07fa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x68707fa8, SUC_lexicon_tags_33 },
    { 0x94907fa8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29907fb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb07fb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x65707fba, SUC_lexicon_tags_63 },
    { 0xf8d07fba, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1cf07fbd, SUC_lexicon_tags_27 },
    { 0x3cd07fbd, SUC_lexicon_tags_25 },
    { 0xfd507fbd, SUC_lexicon_tags_15 },
    { 0xafb07fc0, SUC_lexicon_tags_11 },
    { 0xab107fc1, SUC_lexicon_tags_2 },
    { 0xb0b07fc2, SUC_lexicon_tags_30 },
    { 0x81907fbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56507fc5, SUC_lexicon_tags_7 },
    { 0xbfd07fc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x54507fc8, SUC_lexicon_tags_37 },
    { 0x87907fc9, SUC_lexicon_tags_60 },
    { 0x3af07fc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b507fce, SUC_lexicon_tags_14 },
    { 0x7a507fcf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xacb07fd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8507fd6, SUC_lexicon_tags_17 },
    { 0x8107fd7, SUC_lexicon_tags_21 },
    { 0x40707fd8, SUC_lexicon_tags_11 },
    { 0xa3307fd9, SUC_lexicon_tags_10 },
    { 0xd5507fd7, SUC_lexicon_tags_11 },
    { 0x58b07fdb, SUC_lexicon_tags_3 },
    { 0xc3907fdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57707fe0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd0b07fe2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d07fe7, SUC_lexicon_tags_39 },
    { 0xaef07fe8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25507fec, SUC_lexicon_tags_1 },
    { 0xa9507fed, SUC_lexicon_tags_372 },
    { 0xce507fed, SUC_lexicon_tags_33 },
    { 0x27507fee, SUC_lexicon_tags_1 },
    { 0xbf507ff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9307ff5, SUC_lexicon_tags_10 },
    { 0x69307ff6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xedd07ff8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2507ffa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d308000, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74308002, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5ed08004, SUC_lexicon_tags_25 },
    { 0x8fb08004, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f0800a, SUC_lexicon_tags_7 },
    { 0x8dd0800b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1670800d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x5250800f, SUC_lexicon_tags_6 },
    { 0xe90800f, SUC_lexicon_tags_35 },
    { 0xcd08011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c308013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34308015, SUC_lexicon_tags_43 },
    { 0xa7508016, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4e708018, SUC_lexicon_tags_1 },
    { 0xd0f08018, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d08021, SUC_lexicon_tags_17 },
    { 0x76308022, SUC_lexicon_tags_11 },
    { 0x64908023, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6308026, SUC_lexicon_tags_2 },
    { 0xdb108026, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb908029, SUC_lexicon_tags_17 },
    { 0x5f70802a, SUC_lexicon_tags_38 },
    { 0xc9f0802b, SUC_lexicon_tags_26 },
    { 0xaaf08029, SUC_lexicon_tags_39 },
    { 0xcf50802a, SUC_lexicon_tags_15 },
    { 0x4290802d, SUC_lexicon_tags_1 },
    { 0x8250802c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8b08031, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9108035, SUC_lexicon_tags_27 },
    { 0xe3708036, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f08039, SUC_lexicon_tags_25 },
    { 0x9af0803a, SUC_lexicon_tags_25 },
    { 0x1ed0803b, SUC_lexicon_tags_21 },
    { 0x5810803c, SUC_lexicon_tags_25 },
    { 0x5cf0803d, SUC_lexicon_tags_17 },
    { 0x13708039, SUC_lexicon_tags_61 },
    { 0x7250803f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4308043, SUC_lexicon_tags_7 },
    { 0xfdb08044, SUC_lexicon_tags_2 },
    { 0xe7508043, SUC_lexicon_tags_25 },
    { 0x56f08045, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0908049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd230804d, SUC_lexicon_tags_1 },
    { 0x5d0804e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16508050, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72108053, SUC_lexicon_tags_17 },
    { 0x2b508054, SUC_lexicon_tags_6 },
    { 0xab08053, SUC_lexicon_tags_11 },
    { 0x14708056, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56908061, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x3bb08063, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f08069, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1230806c, SUC_lexicon_tags_197 },
    { 0xdd70806d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd708071, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70508073, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e108075, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2508078, SUC_lexicon_tags_37 },
    { 0xe8708078, SUC_lexicon_tags_27 },
    { 0x90f0807a, SUC_lexicon_tags_11 },
    { 0x5f70807b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76908080, SUC_lexicon_tags_15 },
    { 0x6fd08081, SUC_lexicon_tags_11 },
    { 0x2e508081, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc5108084, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b08087, SUC_lexicon_tags_59 },
    { 0xcd308088, SUC_lexicon_tags_19 },
    { 0xdb108089, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d0808d, SUC_lexicon_tags_33 },
    { 0xa690808e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10108090, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x94108093, SUC_lexicon_tags_11 },
    { 0x7dd08094, SUC_lexicon_tags_43 },
    { 0xd2108095, SUC_lexicon_tags_66 },
    { 0xeef08096, SUC_lexicon_tags_21 },
    { 0xeb08097, SUC_lexicon_tags_12 },
    { 0xff308097, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9930809a, SUC_lexicon_tags_68 },
    { 0x9650809a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55080a0, SUC_lexicon_tags_1 },
    { 0x1cb080a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x807080a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d080a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255080ae, SUC_lexicon_tags_28 },
    { 0x3d9080af, SUC_lexicon_tags_2 },
    { 0x8f5080b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51080b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8d080b6, SUC_lexicon_tags_11 },
    { 0x145080b7, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f080ba, SUC_lexicon_tags_15 },
    { 0x5a9080ba, SUC_lexicon_tags_0 },
    { 0x213080bb, SUC_lexicon_tags_7 },
    { 0x12d080bd, SUC_lexicon_tags_11 },
    { 0x919080be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d9080c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x203080c2, SUC_lexicon_tags_38 },
    { 0x59d080c3, SUC_lexicon_tags_17 },
    { 0xf7d080c4, SUC_lexicon_tags_1 },
    { 0xd23080c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391080ce, SUC_lexicon_tags_7 },
    { 0xb2b080cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad080d2, SUC_lexicon_tags_12 },
    { 0x477080d3, SUC_lexicon_tags_27 },
    { 0xbef080d4, SUC_lexicon_tags_28 },
    { 0xdb7080d3, SUC_lexicon_tags_35 },
    { 0x14b080d6, SUC_lexicon_tags_26 },
    { 0xcf1080d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651080db, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xefb080dd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59080e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67d080e2, SUC_lexicon_tags_64 },
    { 0x347080e3, SUC_lexicon_tags_9 },
    { 0x473080e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53080ec, SUC_lexicon_tags_11 },
    { 0xcaf080ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b080f0, SUC_lexicon_tags_37 },
    { 0x3a3080f0, SUC_lexicon_tags_29 },
    { 0x67f080f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af080f6, SUC_lexicon_tags_1 },
    { 0x79d080f6, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1080fb, SUC_lexicon_tags_23 },
    { 0xa01080fc, SUC_lexicon_tags_84 },
    { 0x50d080fd, SUC_lexicon_tags_87 },
    { 0x76d080fe, SUC_lexicon_tags_2 },
    { 0xaf5080ff, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4308105, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d08112, SUC_lexicon_tags_47 },
    { 0x40308112, SUC_lexicon_tags_77 },
    { 0x5bb08114, SUC_lexicon_tags_31 },
    { 0x97508115, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb8508117, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b0811a, SUC_lexicon_tags_17 },
    { 0x3d10811b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x110811d, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d08120, SUC_lexicon_tags_45 },
    { 0x85108121, SUC_lexicon_tags_55 },
    { 0x69d08122, SUC_lexicon_tags_15 },
    { 0xbe308121, SUC_lexicon_tags_255 },
    { 0x10f08124, SUC_lexicon_tags_47 },
    { 0x69b08124, SUC_lexicon_tags_68 },
    { 0xa2b08120, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae708128, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f0812c, SUC_lexicon_tags_25 },
    { 0xa650812c, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x64d0812f, SUC_lexicon_tags_6 },
    { 0xf08130, SUC_lexicon_tags_1 },
    { 0xb3d08131, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcf308133, SUC_lexicon_tags_47 },
    { 0xd8908133, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f08137, SUC_lexicon_tags_60 },
    { 0x30708137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x410813a, SUC_lexicon_tags_1 },
    { 0x7150813b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb08140, SUC_lexicon_tags_25 },
    { 0x4dd08141, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd08147, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf90814d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1108151, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x41908154, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x26908157, SUC_lexicon_tags_82 },
    { 0x81308157, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd08161, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1308163, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe10816a, SUC_lexicon_tags_118 },
    { 0x53d0816a, SUC_lexicon_tags_22 },
    { 0xb7f0816a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b70816e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfdb08170, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2308172, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd08177, SUC_lexicon_tags_17 },
    { 0xfa308178, SUC_lexicon_tags_106 },
    { 0x7908178, SUC_lexicon_tags_11 },
    { 0xe70817a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd308180, SUC_lexicon_tags_37 },
    { 0x77d08181, SUC_lexicon_tags_17 },
    { 0x17f08180, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3fb08184, SUC_lexicon_tags_0 },
    { 0xd3708185, SUC_lexicon_tags_8 },
    { 0x38508186, SUC_lexicon_tags_11 },
    { 0x70b08186, SUC_lexicon_tags_761 },
    { 0x53108185, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f0818d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e508194, SUC_lexicon_tags_7 },
    { 0xc8908195, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23308197, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5fd08199, SUC_lexicon_tags_43 },
    { 0xa8d0819a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x230819c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x8870819e, SUC_lexicon_tags_37 },
    { 0x4990819f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2b081a1, SUC_lexicon_tags_17 },
    { 0xe7d081a2, SUC_lexicon_tags_31 },
    { 0xd4f081a3, SUC_lexicon_tags_37 },
    { 0x783081a4, SUC_lexicon_tags_1 },
    { 0x49f081a4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1081aa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5f1081ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f081af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d081b2, SUC_lexicon_tags_0 },
    { 0x599081b3, SUC_lexicon_tags_37 },
    { 0xa33081b4, SUC_lexicon_tags_0 },
    { 0xb91081b3, SUC_lexicon_tags_65 },
    { 0xd3b081b3, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xa71081b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x11081bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675081c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473081c4, SUC_lexicon_tags_11 },
    { 0xe1b081c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9081cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849081d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71081d2, SUC_lexicon_tags_38 },
    { 0x605081d3, SUC_lexicon_tags_9 },
    { 0x897081d3, SUC_lexicon_tags_27 },
    { 0xaf7081d3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7081e0, SUC_lexicon_tags_19 },
    { 0x7a9081e0, SUC_lexicon_tags_11 },
    { 0xdf9081e2, SUC_lexicon_tags_11 },
    { 0xe2d081e1, SUC_lexicon_tags_65 },
    { 0x23b081e1, SUC_lexicon_tags_6 },
    { 0x6ff081e5, SUC_lexicon_tags_1 },
    { 0xadb081e6, SUC_lexicon_tags_25 },
    { 0x901081e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2af081e9, SUC_lexicon_tags_15 },
    { 0xd79081ea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357081f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3081f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb7081f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbef081f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d081fb, SUC_lexicon_tags_14 },
    { 0xcf081fb, SUC_lexicon_tags_28 },
    { 0xb47081fb, SUC_lexicon_tags_28 },
    { 0xe69081fc, SUC_lexicon_tags_7 },
    { 0x8d081fe, SUC_lexicon_tags_10 },
    { 0x7908200, SUC_lexicon_tags_25 },
    { 0xf9f08201, SUC_lexicon_tags_45 },
    { 0x3df08202, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x96b08204, SUC_lexicon_tags_2 },
    { 0xd7908205, SUC_lexicon_tags_37 },
    { 0xd6508205, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa508209, SUC_lexicon_tags_87 },
    { 0x4b108209, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29108210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff0821b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb10821d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7e50821f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b08223, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5508229, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab10822b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1610822e, SUC_lexicon_tags_19 },
    { 0x6bb0822f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xe1708231, SUC_lexicon_tags_29 },
    { 0x23308232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f08235, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc930823f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5ad08241, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1508245, SUC_lexicon_tags_8 },
    { 0xd0108246, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3250824e, SUC_lexicon_tags_1 },
    { 0x3b70824f, SUC_lexicon_tags_11 },
    { 0x9850824f, SUC_lexicon_tags_27 },
    { 0xdfb0824f, SUC_lexicon_tags_32 },
    { 0xd8508251, SUC_lexicon_tags_31 },
    { 0x38108253, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x88b08255, SUC_lexicon_tags_1 },
    { 0x3df08256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b308258, SUC_lexicon_tags_15 },
    { 0x45508259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa950825b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2708260, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed08266, SUC_lexicon_tags_11 },
    { 0x51708266, SUC_lexicon_tags_15 },
    { 0x59b08268, SUC_lexicon_tags_37 },
    { 0xe0708267, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9510826f, SUC_lexicon_tags_45 },
    { 0xf030826f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b08274, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x49108277, SUC_lexicon_tags_25 },
    { 0x9a508278, SUC_lexicon_tags_49 },
    { 0x36f08278, SUC_lexicon_tags_23 },
    { 0xb550827a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2650827c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8108281, SUC_lexicon_tags_8 },
    { 0x71f08281, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d08288, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4590828a, SUC_lexicon_tags_33 },
    { 0x9bf0828a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e70828d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac10828f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d08298, SUC_lexicon_tags_39 },
    { 0x5d908298, SUC_lexicon_tags_25 },
    { 0xb790829a, SUC_lexicon_tags_15 },
    { 0x77b0829a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc30829d, SUC_lexicon_tags_17 },
    { 0x2bd0829e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x839082a1, SUC_lexicon_tags_12 },
    { 0x5a9082a2, SUC_lexicon_tags_2 },
    { 0xbcb082a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7082a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbdd082a8, SUC_lexicon_tags_26 },
    { 0xd43082a9, SUC_lexicon_tags_8 },
    { 0xa13082aa, SUC_lexicon_tags_10 },
    { 0x19b082a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79f082ad, SUC_lexicon_tags_84 },
    { 0x6d7082ae, SUC_lexicon_tags_12 },
    { 0x89f082af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59082b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87d082b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637082bb, SUC_lexicon_tags_1 },
    { 0x4a3082bc, SUC_lexicon_tags_200 },
    { 0x197082bd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41082c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x919082c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea3082c8, SUC_lexicon_tags_12 },
    { 0x611082c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x959082cb, SUC_lexicon_tags_14 },
    { 0x3d082cc, SUC_lexicon_tags_11 },
    { 0xdcb082cd, SUC_lexicon_tags_33 },
    { 0xff1082ce, SUC_lexicon_tags_12 },
    { 0x8a5082cf, SUC_lexicon_tags_19 },
    { 0xc71082d0, SUC_lexicon_tags_29 },
    { 0x4e5082cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9082d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc39082d7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39082da, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383082e0, SUC_lexicon_tags_68 },
    { 0x533082e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x99b082e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x945082e5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9082ee, SUC_lexicon_tags_1 },
    { 0xfcf082ef, SUC_lexicon_tags_27 },
    { 0x275082f0, SUC_lexicon_tags_26 },
    { 0x103082ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad082f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715082fc, SUC_lexicon_tags_95 },
    { 0x1f9082fd, SUC_lexicon_tags_11 },
    { 0x73f082fe, SUC_lexicon_tags_68 },
    { 0x773082fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a908303, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfb308305, SUC_lexicon_tags_27 },
    { 0xfeb08306, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b08309, SUC_lexicon_tags_2 },
    { 0x9b10830a, SUC_lexicon_tags_23 },
    { 0x1c10830b, SUC_lexicon_tags_26 },
    { 0x9770830a, SUC_lexicon_tags_19 },
    { 0x7a70830a, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18908313, SUC_lexicon_tags_12 },
    { 0xb2708314, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe050831b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6508327, SUC_lexicon_tags_37 },
    { 0xc508328, SUC_lexicon_tags_17 },
    { 0xc6f08329, SUC_lexicon_tags_2 },
    { 0x5670832a, SUC_lexicon_tags_17 },
    { 0x87b08328, SUC_lexicon_tags_49 },
    { 0x1ad0832a, SUC_lexicon_tags_25 },
    { 0xad50832d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd08330, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd508335, SUC_lexicon_tags_4 },
    { 0xe0908336, SUC_lexicon_tags_19 },
    { 0x7eb08337, SUC_lexicon_tags_38 },
    { 0xe8d08336, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b90833c, SUC_lexicon_tags_66 },
    { 0xc50833d, SUC_lexicon_tags_17 },
    { 0x7830833e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d08342, SUC_lexicon_tags_43 },
    { 0x1fb08342, SUC_lexicon_tags_12 },
    { 0xa108343, SUC_lexicon_tags_17 },
    { 0x58b08342, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaad08347, SUC_lexicon_tags_12 },
    { 0xa8308348, SUC_lexicon_tags_25 },
    { 0xed08349, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3308351, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6eb08353, SUC_lexicon_tags_8 },
    { 0xf8d08354, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b08359, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb50835c, SUC_lexicon_tags_37 },
    { 0x1d10835d, SUC_lexicon_tags_14 },
    { 0xab70835e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81508362, SUC_lexicon_tags_11 },
    { 0x3cb08362, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f708367, SUC_lexicon_tags_7 },
    { 0x15b08368, SUC_lexicon_tags_17 },
    { 0x6308368, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f70836d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12108371, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdcb08373, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x50b08375, SUC_lexicon_tags_87 },
    { 0x88708376, SUC_lexicon_tags_17 },
    { 0x3ab08377, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b0837d, SUC_lexicon_tags_33 },
    { 0x2b30837e, SUC_lexicon_tags_48 },
    { 0xe5b0837f, SUC_lexicon_tags_9 },
    { 0x5d10837d, SUC_lexicon_tags_63 },
    { 0x6ff08381, SUC_lexicon_tags_23 },
    { 0x26d0837d, SUC_lexicon_tags_25 },
    { 0x4f08382, SUC_lexicon_tags_7 },
    { 0x53308384, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f08388, SUC_lexicon_tags_1 },
    { 0x22308389, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b50838b, SUC_lexicon_tags_1 },
    { 0x5250838c, SUC_lexicon_tags_25 },
    { 0x6050838d, SUC_lexicon_tags_12 },
    { 0x2a90838c, SUC_lexicon_tags_11 },
    { 0xb9f0838f, SUC_lexicon_tags_6 },
    { 0x11908390, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad08394, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff08397, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff70839b, SUC_lexicon_tags_40 },
    { 0x3af0839c, SUC_lexicon_tags_43 },
    { 0xfd90839c, SUC_lexicon_tags_49 },
    { 0x3970839c, SUC_lexicon_tags_1 },
    { 0xcd90839f, SUC_lexicon_tags_7 },
    { 0xea10839e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637083a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261083ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d083b0, SUC_lexicon_tags_59 },
    { 0xa43083b1, SUC_lexicon_tags_80 },
    { 0x7d083b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b083b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdd083b9, SUC_lexicon_tags_14 },
    { 0x345083ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817083be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x219083c0, SUC_lexicon_tags_7 },
    { 0xc63083c0, SUC_lexicon_tags_47 },
    { 0x981083c2, SUC_lexicon_tags_2 },
    { 0x49d083c3, SUC_lexicon_tags_45 },
    { 0x1d5083c4, SUC_lexicon_tags_61 },
    { 0x90d083c2, SUC_lexicon_tags_63 },
    { 0x341083c1, SUC_lexicon_tags_2 },
    { 0xf8f083c7, SUC_lexicon_tags_27 },
    { 0x58f083c8, SUC_lexicon_tags_11 },
    { 0xc3f083c2, SUC_lexicon_tags_49 },
    { 0xd5f083ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3083d0, SUC_lexicon_tags_7 },
    { 0x72f083d0, SUC_lexicon_tags_15 },
    { 0xb0b083d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf083d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe1f083d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01083dc, SUC_lexicon_tags_38 },
    { 0x3a3083dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51b083df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd29083e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b083e4, SUC_lexicon_tags_45 },
    { 0x483083e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1083e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9083ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119083f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f1083f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285083fa, SUC_lexicon_tags_70 },
    { 0x923083fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90308400, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x43f08402, SUC_lexicon_tags_30 },
    { 0x2fb08403, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa308405, SUC_lexicon_tags_2 },
    { 0x52108406, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc50840b, SUC_lexicon_tags_23 },
    { 0x7850840c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f08415, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74108419, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a50841f, SUC_lexicon_tags_16 },
    { 0xc7708420, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6308427, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xb590842a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x34d0842c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e10842f, SUC_lexicon_tags_65 },
    { 0x78708430, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x17508432, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3470843a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a308440, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d08446, SUC_lexicon_tags_46 },
    { 0x10f08446, SUC_lexicon_tags_25 },
    { 0x29b08448, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f0844f, SUC_lexicon_tags_11 },
    { 0x8a90844f, SUC_lexicon_tags_0 },
    { 0xe1b08451, SUC_lexicon_tags_49 },
    { 0x91308450, SUC_lexicon_tags_14 },
    { 0x51308450, SUC_lexicon_tags_37 },
    { 0x48b08451, SUC_lexicon_tags_43 },
    { 0xd4b08452, SUC_lexicon_tags_2 },
    { 0x33f08455, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31508459, SUC_lexicon_tags_19 },
    { 0xfa70845a, SUC_lexicon_tags_17 },
    { 0x8c30845b, SUC_lexicon_tags_58 },
    { 0x650845a, SUC_lexicon_tags_25 },
    { 0x7e90845c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc508460, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda508465, SUC_lexicon_tags_37 },
    { 0xa1108465, SUC_lexicon_tags_0 },
    { 0xa7308467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7708469, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f0846d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43108475, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xced08478, SUC_lexicon_tags_38 },
    { 0xc2108479, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b08483, SUC_lexicon_tags_7 },
    { 0xb3b08484, SUC_lexicon_tags_45 },
    { 0x66b08485, SUC_lexicon_tags_45 },
    { 0x7b508486, SUC_lexicon_tags_31 },
    { 0xe9f08487, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d0848a, SUC_lexicon_tags_15 },
    { 0x8f30848b, SUC_lexicon_tags_11 },
    { 0xffd0848b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d90848f, SUC_lexicon_tags_47 },
    { 0xbff08490, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e508493, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f0849b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e30849e, SUC_lexicon_tags_37 },
    { 0xfb10849f, SUC_lexicon_tags_26 },
    { 0xe430849f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x773084a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387084a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d084a9, SUC_lexicon_tags_23 },
    { 0x4b084aa, SUC_lexicon_tags_1 },
    { 0xe51084aa, SUC_lexicon_tags_39 },
    { 0xa61084ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdd084ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f084b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99b084b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x945084b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa83084b8, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237084bc, SUC_lexicon_tags_104 },
    { 0xab3084bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x76d084bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5084c2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x377084c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x393084c6, SUC_lexicon_tags_37 },
    { 0x8b1084c7, SUC_lexicon_tags_33 },
    { 0x4a7084c7, SUC_lexicon_tags_61 },
    { 0xa99084c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb084cd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09084d8, SUC_lexicon_tags_144 },
    { 0xb89084d9, SUC_lexicon_tags_12 },
    { 0xdcb084da, SUC_lexicon_tags_18 },
    { 0xc07084da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729084e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32f084e2, SUC_lexicon_tags_2 },
    { 0x1d3084e3, SUC_lexicon_tags_45 },
    { 0x30d084e4, SUC_lexicon_tags_39 },
    { 0x56d084e5, SUC_lexicon_tags_12 },
    { 0x2c3084e6, SUC_lexicon_tags_15 },
    { 0x139084e7, SUC_lexicon_tags_37 },
    { 0x981084e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb7084ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575084f4, SUC_lexicon_tags_11 },
    { 0xcf9084f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31084fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05084fe, SUC_lexicon_tags_17 },
    { 0x7b1084ff, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17508505, SUC_lexicon_tags_49 },
    { 0x4d508506, SUC_lexicon_tags_6 },
    { 0x1f508507, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0850a, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0xe910850d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a108518, SUC_lexicon_tags_27 },
    { 0xb3508518, SUC_lexicon_tags_1 },
    { 0x43d0851a, SUC_lexicon_tags_37 },
    { 0xb790851a, SUC_lexicon_tags_11 },
    { 0xb2d0851b, SUC_lexicon_tags_26 },
    { 0xd9108519, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8130851f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ed08521, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xde308524, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff908527, SUC_lexicon_tags_3 },
    { 0x20308528, SUC_lexicon_tags_12 },
    { 0x84308528, SUC_lexicon_tags_60 },
    { 0xc170852a, SUC_lexicon_tags_15 },
    { 0x7cd0852b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84708531, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b08534, SUC_lexicon_tags_84 },
    { 0xc0308535, SUC_lexicon_tags_0 },
    { 0x23508536, SUC_lexicon_tags_12 },
    { 0x57508534, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3910853a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb08547, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf70854a, SUC_lexicon_tags_29 },
    { 0xeb30854a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd0854f, SUC_lexicon_tags_31 },
    { 0x3b30854f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f08555, SUC_lexicon_tags_12 },
    { 0x7e908556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc908558, SUC_lexicon_tags_2 },
    { 0x5fd08558, SUC_lexicon_tags_25 },
    { 0xbeb0855a, SUC_lexicon_tags_60 },
    { 0xc70855a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c90855d, SUC_lexicon_tags_21 },
    { 0xd590855d, SUC_lexicon_tags_28 },
    { 0x2990855f, SUC_lexicon_tags_11 },
    { 0xb1508560, SUC_lexicon_tags_12 },
    { 0xef908561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f08564, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b0856b, SUC_lexicon_tags_1 },
    { 0xb10856b, SUC_lexicon_tags_17 },
    { 0xc670856d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3108570, SUC_lexicon_tags_37 },
    { 0x66f08571, SUC_lexicon_tags_15 },
    { 0xdfd08572, SUC_lexicon_tags_14 },
    { 0x7fb08573, SUC_lexicon_tags_25 },
    { 0xe0d08574, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x37108577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58308579, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1010857b, SUC_lexicon_tags_2 },
    { 0x8210857b, SUC_lexicon_tags_31 },
    { 0xa870857d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce908581, SUC_lexicon_tags_25 },
    { 0xa6908582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8108585, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a908587, SUC_lexicon_tags_26 },
    { 0x47108588, SUC_lexicon_tags_15 },
    { 0xab908589, SUC_lexicon_tags_29 },
    { 0xcc908589, SUC_lexicon_tags_852 },
    { 0x95f0858b, SUC_lexicon_tags_38 },
    { 0xaf50858c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb830858e, SUC_lexicon_tags_85 },
    { 0x9970858f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd08594, SUC_lexicon_tags_33 },
    { 0x36d08595, SUC_lexicon_tags_15 },
    { 0x64908596, SUC_lexicon_tags_43 },
    { 0x1e708594, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6a108599, SUC_lexicon_tags_43 },
    { 0xd570859a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1070859e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525085a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f085a9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03085af, SUC_lexicon_tags_49 },
    { 0x97b085b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd085b5, SUC_lexicon_tags_0 },
    { 0x461085b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f085bf, SUC_lexicon_tags_25 },
    { 0x213085c0, SUC_lexicon_tags_53 },
    { 0xc5d085c1, SUC_lexicon_tags_12 },
    { 0x85d085c2, SUC_lexicon_tags_108 },
    { 0xcdd085c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df085c6, SUC_lexicon_tags_38 },
    { 0x95d085c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x839085c9, SUC_lexicon_tags_2 },
    { 0xc9d085c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81f085cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x595085ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5085d2, SUC_lexicon_tags_43 },
    { 0xe3085d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdbb085d5, SUC_lexicon_tags_1372 },
    { 0, NULL },
    { 0x769085d7, SUC_lexicon_tags_11 },
    { 0x871085d8, SUC_lexicon_tags_11 },
    { 0xabf085d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85085e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd71085e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19085eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7085ee, SUC_lexicon_tags_12 },
    { 0x24d085ef, SUC_lexicon_tags_5 },
    { 0x7af085f0, SUC_lexicon_tags_38 },
    { 0x1eb085f1, SUC_lexicon_tags_26 },
    { 0x417085f2, SUC_lexicon_tags_92 },
    { 0x5e7085f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263085f8, SUC_lexicon_tags_31 },
    { 0xc53085f9, SUC_lexicon_tags_49 },
    { 0x7c3085fa, SUC_lexicon_tags_8 },
    { 0x7e5085fa, SUC_lexicon_tags_56 },
    { 0x7d085f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e3085fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb08602, SUC_lexicon_tags_91 },
    { 0x62308603, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6708607, SUC_lexicon_tags_32 },
    { 0x7d108607, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcef0860a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd70860d, SUC_lexicon_tags_45 },
    { 0x6650860e, SUC_lexicon_tags_7 },
    { 0x26b0860e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb08613, SUC_lexicon_tags_12 },
    { 0x45d08614, SUC_lexicon_tags_10 },
    { 0x52508614, SUC_lexicon_tags_12 },
    { 0xfc508614, SUC_lexicon_tags_4 },
    { 0xbb308617, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xea708619, SUC_lexicon_tags_11 },
    { 0x9190861a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6890861f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd08628, SUC_lexicon_tags_25 },
    { 0xd7308629, SUC_lexicon_tags_2 },
    { 0x4a70862a, SUC_lexicon_tags_2 },
    { 0x6550862b, SUC_lexicon_tags_14 },
    { 0xa830862c, SUC_lexicon_tags_43 },
    { 0xe530862d, SUC_lexicon_tags_47 },
    { 0x2230862e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef108631, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd708636, SUC_lexicon_tags_52 },
    { 0x5e908637, SUC_lexicon_tags_37 },
    { 0xe7908638, SUC_lexicon_tags_4 },
    { 0x4c108639, SUC_lexicon_tags_26 },
    { 0xeb908638, SUC_lexicon_tags_27 },
    { 0xd110863b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a90863f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3bf08641, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x76108644, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17f08646, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4308649, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d90864c, SUC_lexicon_tags_46 },
    { 0xb070864d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1e70864f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9708651, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6d508653, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cf08655, SUC_lexicon_tags_25 },
    { 0xe8108655, SUC_lexicon_tags_58 },
    { 0xd108657, SUC_lexicon_tags_15 },
    { 0x8c308658, SUC_lexicon_tags_57 },
    { 0x92f08659, SUC_lexicon_tags_23 },
    { 0x15308656, SUC_lexicon_tags_37 },
    { 0x9b30865b, SUC_lexicon_tags_26 },
    { 0x18708656, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x58d0865e, SUC_lexicon_tags_23 },
    { 0x35d0865f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36508661, SUC_lexicon_tags_25 },
    { 0x74b08662, SUC_lexicon_tags_49 },
    { 0xe9308662, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d08667, SUC_lexicon_tags_17 },
    { 0xa4108668, SUC_lexicon_tags_54 },
    { 0x66908669, SUC_lexicon_tags_30 },
    { 0xab0866a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6630866c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb610867a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb30867c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35908684, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9508686, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b10868d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9708691, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e308697, SUC_lexicon_tags_61 },
    { 0x7f308697, SUC_lexicon_tags_15 },
    { 0xf1108697, SUC_lexicon_tags_33 },
    { 0x68d08697, SUC_lexicon_tags_0 },
    { 0x5b10869b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3f0869d, SUC_lexicon_tags_37 },
    { 0xb790869e, SUC_lexicon_tags_17 },
    { 0xa570869d, SUC_lexicon_tags_12 },
    { 0x9e5086a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81d086a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x501086a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d086a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe85086aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b086af, SUC_lexicon_tags_68 },
    { 0xdd9086b0, SUC_lexicon_tags_82 },
    { 0xfbb086af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71086b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805086bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d086c2, SUC_lexicon_tags_45 },
    { 0xd99086c2, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xb45086c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b086c8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d086cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5086d1, SUC_lexicon_tags_27 },
    { 0x3bd086d1, SUC_lexicon_tags_7 },
    { 0x35d086d3, SUC_lexicon_tags_7 },
    { 0x625086d4, SUC_lexicon_tags_27 },
    { 0xe65086d5, SUC_lexicon_tags_17 },
    { 0xa11086d1, SUC_lexicon_tags_34 },
    { 0xc5b086d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6f9086d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x441086db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad086de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b086e1, SUC_lexicon_tags_12 },
    { 0x313086e2, SUC_lexicon_tags_12 },
    { 0xa87086e3, SUC_lexicon_tags_11 },
    { 0x88f086e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129086e8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x785086ea, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503086ee, SUC_lexicon_tags_54 },
    { 0x8cd086ef, SUC_lexicon_tags_7 },
    { 0xdff086f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe77086f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6d086f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x711086f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5f086f9, SUC_lexicon_tags_25 },
    { 0x537086fa, SUC_lexicon_tags_17 },
    { 0xd57086fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5508701, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93708706, SUC_lexicon_tags_83 },
    { 0xc4b08706, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x950870a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8070870d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27b0870f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d08713, SUC_lexicon_tags_23 },
    { 0x90f08713, SUC_lexicon_tags_23 },
    { 0x12308715, SUC_lexicon_tags_2 },
    { 0x7d708716, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd10871b, SUC_lexicon_tags_7 },
    { 0xcdf0871b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8190871f, SUC_lexicon_tags_37 },
    { 0xd6d0871f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7508722, SUC_lexicon_tags_2 },
    { 0xa2508723, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2708728, SUC_lexicon_tags_37 },
    { 0x60908729, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd950872c, SUC_lexicon_tags_37 },
    { 0x5b30872d, SUC_lexicon_tags_1 },
    { 0x4e70872c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaff08730, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e308736, SUC_lexicon_tags_37 },
    { 0x96708737, SUC_lexicon_tags_45 },
    { 0xef708738, SUC_lexicon_tags_29 },
    { 0x3ff08739, SUC_lexicon_tags_14 },
    { 0x5750873a, SUC_lexicon_tags_63 },
    { 0x8cd0873b, SUC_lexicon_tags_179 },
    { 0x9090873b, SUC_lexicon_tags_6 },
    { 0x8b50873d, SUC_lexicon_tags_17 },
    { 0xd5f0873c, SUC_lexicon_tags_23 },
    { 0x9d108736, SUC_lexicon_tags_22 },
    { 0x9390873e, SUC_lexicon_tags_11 },
    { 0x48f0873f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b908744, SUC_lexicon_tags_15 },
    { 0x5bd08745, SUC_lexicon_tags_15 },
    { 0xe0b08746, SUC_lexicon_tags_11 },
    { 0xf0908747, SUC_lexicon_tags_27 },
    { 0x17108748, SUC_lexicon_tags_12 },
    { 0x40908749, SUC_lexicon_tags_1 },
    { 0x95908749, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9510874d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b08751, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x62908753, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58508755, SUC_lexicon_tags_12 },
    { 0x83708755, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e708759, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed30875b, SUC_lexicon_tags_4 },
    { 0xf950875c, SUC_lexicon_tags_25 },
    { 0x12b0875d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d08760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43508762, SUC_lexicon_tags_37 },
    { 0x6dd08763, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0508766, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea508769, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb508771, SUC_lexicon_tags_10 },
    { 0xbbd08772, SUC_lexicon_tags_68 },
    { 0xe2f08773, SUC_lexicon_tags_30 },
    { 0xfe508772, SUC_lexicon_tags_37 },
    { 0xb4108775, SUC_lexicon_tags_23 },
    { 0xc8908776, SUC_lexicon_tags_47 },
    { 0xe5b08777, SUC_lexicon_tags_15 },
    { 0x31508778, SUC_lexicon_tags_17 },
    { 0x96308779, SUC_lexicon_tags_224 },
    { 0x95908777, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4690877f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bf08781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5108784, SUC_lexicon_tags_25 },
    { 0x5e308785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d08788, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd0878c, SUC_lexicon_tags_14 },
    { 0xfb70878d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda708791, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6108793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9508796, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x49108799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4250879c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1087a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57087ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5087b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9087b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b087ba, SUC_lexicon_tags_15 },
    { 0x405087bb, SUC_lexicon_tags_38 },
    { 0x4ff087bc, SUC_lexicon_tags_31 },
    { 0x697087bc, SUC_lexicon_tags_16 },
    { 0xc13087bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7087c2, SUC_lexicon_tags_25 },
    { 0x397087c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7087c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e7087c8, SUC_lexicon_tags_1 },
    { 0x1f3087c9, SUC_lexicon_tags_25 },
    { 0x951087c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9087ce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f087d1, SUC_lexicon_tags_15 },
    { 0xb1087d2, SUC_lexicon_tags_17 },
    { 0x59087d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x507087d5, SUC_lexicon_tags_2 },
    { 0x1cf087d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd087d8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d087e0, SUC_lexicon_tags_11 },
    { 0x455087e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x69b087e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf087e6, SUC_lexicon_tags_8 },
    { 0xd43087e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb69087e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b087ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7087f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26f087f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbff087f4, SUC_lexicon_tags_28 },
    { 0x779087f5, SUC_lexicon_tags_11 },
    { 0xe85087f6, SUC_lexicon_tags_25 },
    { 0xfe3087f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd087fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x695087fc, SUC_lexicon_tags_25 },
    { 0xa2d087fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a7087ff, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x27108801, SUC_lexicon_tags_37 },
    { 0x5908801, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d08808, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4590880a, SUC_lexicon_tags_19 },
    { 0x2670880b, SUC_lexicon_tags_12 },
    { 0xc030880a, SUC_lexicon_tags_48 },
    { 0x7230880b, SUC_lexicon_tags_87 },
    { 0xe6f0880c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5dd08810, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x33708812, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf08815, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d708817, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0881c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79d0881e, SUC_lexicon_tags_15 },
    { 0xff70881e, SUC_lexicon_tags_11 },
    { 0xae308820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf08823, SUC_lexicon_tags_25 },
    { 0x4db08824, SUC_lexicon_tags_31 },
    { 0x7f708824, SUC_lexicon_tags_17 },
    { 0xbc908826, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xca508828, SUC_lexicon_tags_2 },
    { 0x2df08829, SUC_lexicon_tags_16 },
    { 0xf2508829, SUC_lexicon_tags_63 },
    { 0xa050882b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd0882e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf508831, SUC_lexicon_tags_14 },
    { 0xfd708831, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b08837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ad08839, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23708840, SUC_lexicon_tags_2 },
    { 0x8cf08840, SUC_lexicon_tags_11 },
    { 0x12508842, SUC_lexicon_tags_0 },
    { 0x8ad08841, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d08846, SUC_lexicon_tags_30 },
    { 0x63b08846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa910884c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd08852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6708856, SUC_lexicon_tags_35 },
    { 0x9e108856, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4890885a, SUC_lexicon_tags_17 },
    { 0xd5d0885a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdbd0885d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f308862, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d08866, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d308868, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b0886d, SUC_lexicon_tags_23 },
    { 0xcc10886e, SUC_lexicon_tags_12 },
    { 0x510886e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53f08871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b508875, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d0887c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe508887, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd570888a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8750888c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8410888e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92108890, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81b08892, SUC_lexicon_tags_10 },
    { 0x7e708893, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b08897, SUC_lexicon_tags_43 },
    { 0x79108898, SUC_lexicon_tags_33 },
    { 0xda308899, SUC_lexicon_tags_25 },
    { 0xdc708899, SUC_lexicon_tags_63 },
    { 0x39d08899, SUC_lexicon_tags_43 },
    { 0xc6508899, SUC_lexicon_tags_1 },
    { 0x6770889c, SUC_lexicon_tags_29 },
    { 0xf610889a, SUC_lexicon_tags_11 },
    { 0xda30889d, SUC_lexicon_tags_19 },
    { 0xa1b08898, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf088a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd088a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x17088a9, SUC_lexicon_tags_2 },
    { 0xebb088aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb088ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed088b0, SUC_lexicon_tags_79 },
    { 0xdd7088b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97088b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed088bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563088c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4eb088c4, SUC_lexicon_tags_28 },
    { 0xb088c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69088c7, SUC_lexicon_tags_2 },
    { 0x2db088c8, SUC_lexicon_tags_17 },
    { 0xda3088c7, SUC_lexicon_tags_9 },
    { 0x731088c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xad7088cc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5088d1, SUC_lexicon_tags_37 },
    { 0x1d9088d2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85088d8, SUC_lexicon_tags_16 },
    { 0xff7088d9, SUC_lexicon_tags_37 },
    { 0x1d5088da, SUC_lexicon_tags_27 },
    { 0xd77088d8, SUC_lexicon_tags_28 },
    { 0x111088d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a1088de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61f088e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13088e5, SUC_lexicon_tags_14 },
    { 0x169088e6, SUC_lexicon_tags_3 },
    { 0x8a3088e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb088ea, SUC_lexicon_tags_23 },
    { 0xa95088eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b088ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b088f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x919088f6, SUC_lexicon_tags_12 },
    { 0xa63088f6, SUC_lexicon_tags_2 },
    { 0xe1088f8, SUC_lexicon_tags_32 },
    { 0xb81088f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5088fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e908903, SUC_lexicon_tags_30 },
    { 0x70108904, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x48b08906, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd08909, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5290890b, SUC_lexicon_tags_0 },
    { 0x2b70890c, SUC_lexicon_tags_15 },
    { 0x6fb0890c, SUC_lexicon_tags_25 },
    { 0x4d70890e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af08912, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23108914, SUC_lexicon_tags_66 },
    { 0xb4d08915, SUC_lexicon_tags_17 },
    { 0x2d908914, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4970891b, SUC_lexicon_tags_87 },
    { 0x54f0891c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf08921, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0508925, SUC_lexicon_tags_11 },
    { 0x5a508926, SUC_lexicon_tags_25 },
    { 0xdc108927, SUC_lexicon_tags_12 },
    { 0x71708927, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d0892f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xae108931, SUC_lexicon_tags_30 },
    { 0x53508931, SUC_lexicon_tags_17 },
    { 0x51308932, SUC_lexicon_tags_33 },
    { 0x7c308934, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f0893a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e30893d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e50893f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f08942, SUC_lexicon_tags_0 },
    { 0x1a708943, SUC_lexicon_tags_65 },
    { 0x69908943, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda908948, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x110894e, SUC_lexicon_tags_25 },
    { 0xa390894f, SUC_lexicon_tags_11 },
    { 0x42708950, SUC_lexicon_tags_27 },
    { 0xaf50894e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c508955, SUC_lexicon_tags_47 },
    { 0x88108955, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd0895a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed0895d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b308961, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc5508963, SUC_lexicon_tags_38 },
    { 0xa7d08964, SUC_lexicon_tags_23 },
    { 0xe8108964, SUC_lexicon_tags_38 },
    { 0xc3f08966, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4590896c, SUC_lexicon_tags_49 },
    { 0x8e90896d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4108970, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad0897b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5908980, SUC_lexicon_tags_26 },
    { 0x33f08980, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc708983, SUC_lexicon_tags_37 },
    { 0x60508984, SUC_lexicon_tags_40 },
    { 0x47b08985, SUC_lexicon_tags_15 },
    { 0xad708985, SUC_lexicon_tags_17 },
    { 0xb5108985, SUC_lexicon_tags_160 },
    { 0x77d08988, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa990898a, SUC_lexicon_tags_57 },
    { 0x330898a, SUC_lexicon_tags_7 },
    { 0x2590898c, SUC_lexicon_tags_43 },
    { 0x20d0898d, SUC_lexicon_tags_17 },
    { 0x2210898e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb508996, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b0899a, SUC_lexicon_tags_49 },
    { 0x2f10899a, SUC_lexicon_tags_12 },
    { 0x2890899a, SUC_lexicon_tags_30 },
    { 0x8690899d, SUC_lexicon_tags_25 },
    { 0xe990899d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497089a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3089a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b089ac, SUC_lexicon_tags_12 },
    { 0xc19089ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf5089af, SUC_lexicon_tags_17 },
    { 0xfe7089b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef089b6, SUC_lexicon_tags_26 },
    { 0x351089b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc45089b9, SUC_lexicon_tags_27 },
    { 0x6f9089ba, SUC_lexicon_tags_32 },
    { 0x677089ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b089be, SUC_lexicon_tags_17 },
    { 0xd05089be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115089c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473089c9, SUC_lexicon_tags_12 },
    { 0x515089c9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab089cf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b089d3, SUC_lexicon_tags_27 },
    { 0xc8b089d4, SUC_lexicon_tags_28 },
    { 0x43d089d4, SUC_lexicon_tags_10 },
    { 0xc33089d6, SUC_lexicon_tags_29 },
    { 0xf79089d6, SUC_lexicon_tags_1 },
    { 0xaf9089d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc3089da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5089de, SUC_lexicon_tags_25 },
    { 0xe9f089df, SUC_lexicon_tags_17 },
    { 0xe85089e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea5089e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7b089e4, SUC_lexicon_tags_38 },
    { 0xb7089e5, SUC_lexicon_tags_43 },
    { 0xbdb089e4, SUC_lexicon_tags_15 },
    { 0xfb9089e5, SUC_lexicon_tags_31 },
    { 0x465089e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5089eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x223089ed, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x50d089ef, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99089f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d089fb, SUC_lexicon_tags_46 },
    { 0x6bb089fc, SUC_lexicon_tags_68 },
    { 0x993089fc, SUC_lexicon_tags_21 },
    { 0x269089fe, SUC_lexicon_tags_71 },
    { 0x259089ff, SUC_lexicon_tags_25 },
    { 0x9c1089ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9ab08a02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f908a05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28d08a07, SUC_lexicon_tags_12 },
    { 0x8a508a07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9708a0c, SUC_lexicon_tags_82 },
    { 0xe4908a0d, SUC_lexicon_tags_17 },
    { 0x76f08a0e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5c908a10, SUC_lexicon_tags_14 },
    { 0xa2d08a10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f08a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e908a19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52108a1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d08a1f, SUC_lexicon_tags_38 },
    { 0x37108a1f, SUC_lexicon_tags_26 },
    { 0x5b08a21, SUC_lexicon_tags_38 },
    { 0x87f08a22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd08a26, SUC_lexicon_tags_49 },
    { 0x32308a27, SUC_lexicon_tags_23 },
    { 0xe5108a27, SUC_lexicon_tags_2 },
    { 0x96f08a29, SUC_lexicon_tags_25 },
    { 0x56908a29, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf08a2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57108a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91908a3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e708a43, SUC_lexicon_tags_12 },
    { 0xf1908a44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ef08a46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf08a4c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5308a4e, SUC_lexicon_tags_39 },
    { 0xb2508a4e, SUC_lexicon_tags_49 },
    { 0x9d08a50, SUC_lexicon_tags_23 },
    { 0xcb708a51, SUC_lexicon_tags_57 },
    { 0x2108a51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9b08a54, SUC_lexicon_tags_13 },
    { 0xf0f08a55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb508a57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d08a5a, SUC_lexicon_tags_27 },
    { 0x8508a5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af08a5e, SUC_lexicon_tags_11 },
    { 0xfcf08a5f, SUC_lexicon_tags_46 },
    { 0xa4b08a5f, SUC_lexicon_tags_31 },
    { 0x32d08a61, SUC_lexicon_tags_3 },
    { 0x68708a62, SUC_lexicon_tags_43 },
    { 0x12308a63, SUC_lexicon_tags_2 },
    { 0xf6708a63, SUC_lexicon_tags_60 },
    { 0xd0308a65, SUC_lexicon_tags_12 },
    { 0x5f108a65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed508a6b, SUC_lexicon_tags_17 },
    { 0xf9308a6c, SUC_lexicon_tags_45 },
    { 0x34f08a6c, SUC_lexicon_tags_2 },
    { 0xd9908a6d, SUC_lexicon_tags_25 },
    { 0x80908a6f, SUC_lexicon_tags_17 },
    { 0xd2f08a6e, SUC_lexicon_tags_9 },
    { 0x32d08a71, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc708a76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bf08a78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28108a7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8108a7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8d508a7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6308a82, SUC_lexicon_tags_15 },
    { 0xe0108a82, SUC_lexicon_tags_12 },
    { 0xf5108a84, SUC_lexicon_tags_6 },
    { 0xf8308a85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1508a8a, SUC_lexicon_tags_15 },
    { 0x74308a8b, SUC_lexicon_tags_15 },
    { 0x1ff08a8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8908a8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b708a96, SUC_lexicon_tags_45 },
    { 0xef908a96, SUC_lexicon_tags_17 },
    { 0xb4f08a97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3508a9c, SUC_lexicon_tags_37 },
    { 0x53508a9d, SUC_lexicon_tags_23 },
    { 0xd5d08a9c, SUC_lexicon_tags_17 },
    { 0x86708a9e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73708aa6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63d08aa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49d08aaa, SUC_lexicon_tags_45 },
    { 0x35308aab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaaf08aad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27708aaf, SUC_lexicon_tags_37 },
    { 0xd5708aaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda908ab2, SUC_lexicon_tags_17 },
    { 0x9e308ab3, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x5cb08ab5, SUC_lexicon_tags_17 },
    { 0xacf08ab5, SUC_lexicon_tags_83 },
    { 0x80d08ab5, SUC_lexicon_tags_81 },
    { 0x56908ab8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53708aba, SUC_lexicon_tags_39 },
    { 0x12f08abb, SUC_lexicon_tags_37 },
    { 0xe5d08abc, SUC_lexicon_tags_45 },
    { 0x19908abd, SUC_lexicon_tags_33 },
    { 0xb708abe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7308ac0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f08ac3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21708ac5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x26908ac8, SUC_lexicon_tags_25 },
    { 0x1df08ac9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e908acb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xac308acd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7308ad1, SUC_lexicon_tags_25 },
    { 0xf2708ad2, SUC_lexicon_tags_6 },
    { 0x2fb08ad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f108ad7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b708adb, SUC_lexicon_tags_87 },
    { 0x98d08adc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb7f08ade, SUC_lexicon_tags_25 },
    { 0x61b08adf, SUC_lexicon_tags_45 },
    { 0x44f08ae0, SUC_lexicon_tags_45 },
    { 0x6f908adf, SUC_lexicon_tags_16 },
    { 0xc508ae1, SUC_lexicon_tags_47 },
    { 0x44b08ae3, SUC_lexicon_tags_2 },
    { 0x9bf08ae4, SUC_lexicon_tags_2 },
    { 0xf908ae5, SUC_lexicon_tags_37 },
    { 0x69108ae5, SUC_lexicon_tags_15 },
    { 0x32908ae4, SUC_lexicon_tags_28 },
    { 0x13d08ae4, SUC_lexicon_tags_11 },
    { 0xb9d08ae9, SUC_lexicon_tags_275 },
    { 0x73108aea, SUC_lexicon_tags_31 },
    { 0x9a708aeb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x51708aed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9a108aef, SUC_lexicon_tags_27 },
    { 0xbb508aef, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x96508af2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b08af6, SUC_lexicon_tags_12 },
    { 0xf5f08af6, SUC_lexicon_tags_45 },
    { 0xfdd08af8, SUC_lexicon_tags_19 },
    { 0x7c308af9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1508b00, SUC_lexicon_tags_9 },
    { 0x9ff08b01, SUC_lexicon_tags_0 },
    { 0x97108b01, SUC_lexicon_tags_38 },
    { 0x59508b03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6d08b05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda308b0b, SUC_lexicon_tags_2 },
    { 0x47908b0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc908b11, SUC_lexicon_tags_23 },
    { 0x1b508b12, SUC_lexicon_tags_17 },
    { 0x36f08b12, SUC_lexicon_tags_12 },
    { 0x7708b14, SUC_lexicon_tags_31 },
    { 0xbd908b14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d108b19, SUC_lexicon_tags_8 },
    { 0x47108b1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d508b20, SUC_lexicon_tags_13 },
    { 0xbd508b20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0908b24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad08b27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94708b2d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x3ef08b2f, SUC_lexicon_tags_7 },
    { 0xcab08b30, SUC_lexicon_tags_37 },
    { 0xf0908b31, SUC_lexicon_tags_17 },
    { 0x9c108b2f, SUC_lexicon_tags_49 },
    { 0x108b32, SUC_lexicon_tags_45 },
    { 0x6d108b34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xff108b37, SUC_lexicon_tags_26 },
    { 0x7fb08b38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4b08b3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d08b40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74908b45, SUC_lexicon_tags_1 },
    { 0xc9308b45, SUC_lexicon_tags_14 },
    { 0xe1b08b46, SUC_lexicon_tags_11 },
    { 0xe9b08b46, SUC_lexicon_tags_63 },
    { 0x8ef08b48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb08b4e, SUC_lexicon_tags_10 },
    { 0x92908b4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x99f08b51, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b08b5b, SUC_lexicon_tags_37 },
    { 0xb9308b5c, SUC_lexicon_tags_7 },
    { 0xe5b08b5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe908b60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x50508b63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18508b65, SUC_lexicon_tags_17 },
    { 0x24f08b66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9508b68, SUC_lexicon_tags_505 },
    { 0, NULL },
    { 0x9db08b6a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f08b6d, SUC_lexicon_tags_1 },
    { 0x8e908b6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad08b72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xff08b75, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b08b7a, SUC_lexicon_tags_2 },
    { 0xef508b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d08b7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0508b81, SUC_lexicon_tags_25 },
    { 0xcab08b81, SUC_lexicon_tags_63 },
    { 0xa9708b81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d08b87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61308b89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d08b8c, SUC_lexicon_tags_83 },
    { 0x98108b8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6708b8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef08b98, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4308b9c, SUC_lexicon_tags_23 },
    { 0xe9308b9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48308ba2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d08ba6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91308baa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6708bb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11508bb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33508bbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf908bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83708bc5, SUC_lexicon_tags_10 },
    { 0x6ff08bc5, SUC_lexicon_tags_45 },
    { 0x9f308bc6, SUC_lexicon_tags_29 },
    { 0xa6108bc8, SUC_lexicon_tags_35 },
    { 0x68d08bc9, SUC_lexicon_tags_25 },
    { 0x32d08bc9, SUC_lexicon_tags_27 },
    { 0x10308bcb, SUC_lexicon_tags_13 },
    { 0xb7308bcc, SUC_lexicon_tags_17 },
    { 0x7ad08bca, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x75108bcf, SUC_lexicon_tags_30 },
    { 0x21308bd0, SUC_lexicon_tags_88 },
    { 0x78d08bd1, SUC_lexicon_tags_43 },
    { 0xa908bd2, SUC_lexicon_tags_0 },
    { 0x99708bd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7308bd5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4708bde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d08be2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d08be5, SUC_lexicon_tags_25 },
    { 0xac508be5, SUC_lexicon_tags_87 },
    { 0x2e708be6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cd08be9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80308beb, SUC_lexicon_tags_32 },
    { 0xb2908bec, SUC_lexicon_tags_7 },
    { 0xf3108bec, SUC_lexicon_tags_63 },
    { 0xcfb08bed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1708bf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcb08bf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb08bfa, SUC_lexicon_tags_87 },
    { 0x5f908bfb, SUC_lexicon_tags_7 },
    { 0x35d08bfb, SUC_lexicon_tags_463 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d908c00, SUC_lexicon_tags_37 },
    { 0x4b08c01, SUC_lexicon_tags_6 },
    { 0xc8708c02, SUC_lexicon_tags_39 },
    { 0x21908c03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae108c05, SUC_lexicon_tags_14 },
    { 0x85508c06, SUC_lexicon_tags_37 },
    { 0xb6f08c06, SUC_lexicon_tags_9 },
    { 0x38908c07, SUC_lexicon_tags_55 },
    { 0xe9508c05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd08c0c, SUC_lexicon_tags_17 },
    { 0xb6108c0c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d708c0f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x30908c12, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f508c15, SUC_lexicon_tags_11 },
    { 0xb3f08c16, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c108c19, SUC_lexicon_tags_2 },
    { 0xb8d08c19, SUC_lexicon_tags_8 },
    { 0x88f08c1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab08c1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80708c27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40708c29, SUC_lexicon_tags_45 },
    { 0x32d08c2a, SUC_lexicon_tags_32 },
    { 0xf3708c2a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa708c30, SUC_lexicon_tags_7 },
    { 0xb908c30, SUC_lexicon_tags_46 },
    { 0xf2f08c32, SUC_lexicon_tags_25 },
    { 0x39708c33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d08c36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63108c3a, SUC_lexicon_tags_63 },
    { 0xcc108c3b, SUC_lexicon_tags_65 },
    { 0x6cf08c3b, SUC_lexicon_tags_17 },
    { 0x66f08c3c, SUC_lexicon_tags_26 },
    { 0x4a908c3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e708c43, SUC_lexicon_tags_72 },
    { 0xea908c44, SUC_lexicon_tags_17 },
    { 0xb3908c45, SUC_lexicon_tags_50 },
    { 0xd1708c43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83f08c48, SUC_lexicon_tags_30 },
    { 0xc9f08c49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29908c4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d308c53, SUC_lexicon_tags_1 },
    { 0xb3708c54, SUC_lexicon_tags_27 },
    { 0x95f08c55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72d08c57, SUC_lexicon_tags_26 },
    { 0xed708c57, SUC_lexicon_tags_9 },
    { 0xdf508c59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95708c63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x60508c65, SUC_lexicon_tags_7 },
    { 0xa6908c66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab08c6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6708c73, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x96f08c75, SUC_lexicon_tags_25 },
    { 0xe3108c75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f08c7b, SUC_lexicon_tags_29 },
    { 0x70d08c7c, SUC_lexicon_tags_30 },
    { 0xc7508c7d, SUC_lexicon_tags_15 },
    { 0x33b08c7e, SUC_lexicon_tags_37 },
    { 0xdb08c7f, SUC_lexicon_tags_43 },
    { 0xff908c80, SUC_lexicon_tags_11 },
    { 0x4d908c81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98308c89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94308c8d, SUC_lexicon_tags_17 },
    { 0xf0308c8e, SUC_lexicon_tags_1 },
    { 0x3c908c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30708c91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f708c94, SUC_lexicon_tags_12 },
    { 0x4db08c94, SUC_lexicon_tags_25 },
    { 0x85708c96, SUC_lexicon_tags_17 },
    { 0xd1908c97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef08c9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x65708ca1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x97508ca3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53508ca6, SUC_lexicon_tags_37 },
    { 0xc0b08ca6, SUC_lexicon_tags_65 },
    { 0x86908ca6, SUC_lexicon_tags_17 },
    { 0xce508ca8, SUC_lexicon_tags_1 },
    { 0x25d08caa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8b508cac, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x90908cae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x30508cb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ab08cb2, SUC_lexicon_tags_23 },
    { 0xfc908cb3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3bd08cb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe2508cb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7a708cb9, SUC_lexicon_tags_16 },
    { 0x52d08cb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa108cbd, SUC_lexicon_tags_9 },
    { 0x60508cbe, SUC_lexicon_tags_10 },
    { 0xc5d08cbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb108cc1, SUC_lexicon_tags_79 },
    { 0x76308cc1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x47f08cc4, SUC_lexicon_tags_25 },
    { 0x8df08cc4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x62508cc7, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x2e508cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53708ccb, SUC_lexicon_tags_29 },
    { 0xb2108ccb, SUC_lexicon_tags_30 },
    { 0x7e708ccd, SUC_lexicon_tags_37 },
    { 0xbf08cce, SUC_lexicon_tags_4 },
    { 0xfd108ccf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e908cd8, SUC_lexicon_tags_8 },
    { 0x3f508cd9, SUC_lexicon_tags_7 },
    { 0x72108cd9, SUC_lexicon_tags_2 },
    { 0x108cda, SUC_lexicon_tags_47 },
    { 0x9d708cdc, SUC_lexicon_tags_17 },
    { 0xbe708cdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d08ce0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x21508ce2, SUC_lexicon_tags_55 },
    { 0x76f08ce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefd08ce5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3508cec, SUC_lexicon_tags_25 },
    { 0xf9508ced, SUC_lexicon_tags_15 },
    { 0xe7508cee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xca108cf0, SUC_lexicon_tags_37 },
    { 0xc508cf1, SUC_lexicon_tags_7 },
    { 0x39908cf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a308cfb, SUC_lexicon_tags_16 },
    { 0x29508cfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7908d02, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x39b08d04, SUC_lexicon_tags_45 },
    { 0xb9d08d05, SUC_lexicon_tags_17 },
    { 0xe7708d06, SUC_lexicon_tags_46 },
    { 0x52908d07, SUC_lexicon_tags_11 },
    { 0xaf508d08, SUC_lexicon_tags_37 },
    { 0x24f08d09, SUC_lexicon_tags_37 },
    { 0xadb08d0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6108d0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9908d12, SUC_lexicon_tags_25 },
    { 0xe1b08d13, SUC_lexicon_tags_15 },
    { 0xcab08d12, SUC_lexicon_tags_66 },
    { 0x16f08d15, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf7908d17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0708d19, SUC_lexicon_tags_37 },
    { 0xd1d08d1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfcf08d1c, SUC_lexicon_tags_43 },
    { 0x1b908d1d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf08d21, SUC_lexicon_tags_1 },
    { 0xeeb08d22, SUC_lexicon_tags_17 },
    { 0x61908d22, SUC_lexicon_tags_2 },
    { 0xebb08d23, SUC_lexicon_tags_4 },
    { 0x25508d25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33308d28, SUC_lexicon_tags_7 },
    { 0x1b508d29, SUC_lexicon_tags_25 },
    { 0xc8b08d2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ed08d2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40108d2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ef08d30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce308d33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b08d36, SUC_lexicon_tags_55 },
    { 0xbb308d37, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x92f08d39, SUC_lexicon_tags_37 },
    { 0x64908d3a, SUC_lexicon_tags_60 },
    { 0xbb108d3a, SUC_lexicon_tags_25 },
    { 0xa4108d3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4cf08d3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a908d41, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4a708d43, SUC_lexicon_tags_10 },
    { 0xac508d44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b08d4b, SUC_lexicon_tags_27 },
    { 0x68708d4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x30708d4f, SUC_lexicon_tags_2 },
    { 0x88d08d50, SUC_lexicon_tags_15 },
    { 0xb708d51, SUC_lexicon_tags_7 },
    { 0x4b708d51, SUC_lexicon_tags_28 },
    { 0x8f708d52, SUC_lexicon_tags_92 },
    { 0x5e308d51, SUC_lexicon_tags_17 },
    { 0x50508d55, SUC_lexicon_tags_22 },
    { 0x17d08d54, SUC_lexicon_tags_84 },
    { 0x88108d57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfc508d59, SUC_lexicon_tags_12 },
    { 0xf3b08d5a, SUC_lexicon_tags_30 },
    { 0xf508d59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec108d63, SUC_lexicon_tags_60 },
    { 0x4cf08d63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7df08d66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37308d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73108d6b, SUC_lexicon_tags_27 },
    { 0xc0108d6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96308d6e, SUC_lexicon_tags_17 },
    { 0x9ff08d6e, SUC_lexicon_tags_16 },
    { 0x6dd08d6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5108d72, SUC_lexicon_tags_0 },
    { 0xd7f08d72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa908d77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5908d79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd5b08d7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b08d7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b08d8b, SUC_lexicon_tags_47 },
    { 0xfb308d8c, SUC_lexicon_tags_60 },
    { 0x65d08d8b, SUC_lexicon_tags_11 },
    { 0xd6f08d8c, SUC_lexicon_tags_87 },
    { 0x73908d8d, SUC_lexicon_tags_1 },
    { 0x60d08d90, SUC_lexicon_tags_45 },
    { 0xf3f08d91, SUC_lexicon_tags_4 },
    { 0x87d08d92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5508d96, SUC_lexicon_tags_14 },
    { 0x50108d96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57708da1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x16d08da3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12508daa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xaa308dac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7308db1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4308db3, SUC_lexicon_tags_45 },
    { 0x8e308db3, SUC_lexicon_tags_57 },
    { 0x5a108db3, SUC_lexicon_tags_56 },
    { 0x26708db6, SUC_lexicon_tags_14 },
    { 0x27f08db4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff08dba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab08dbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb08dc5, SUC_lexicon_tags_15 },
    { 0x16708dc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf08dcc, SUC_lexicon_tags_47 },
    { 0x4e108dcd, SUC_lexicon_tags_47 },
    { 0xf4b08dcd, SUC_lexicon_tags_17 },
    { 0x98108dcd, SUC_lexicon_tags_87 },
    { 0x2df08dcf, SUC_lexicon_tags_27 },
    { 0x37f08dcf, SUC_lexicon_tags_17 },
    { 0xe6508dcc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88908dd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x69108dd8, SUC_lexicon_tags_252 },
    { 0xca708dd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c108ddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7108de0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72308de7, SUC_lexicon_tags_63 },
    { 0x53908de8, SUC_lexicon_tags_47 },
    { 0x8f308de9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad08dec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0708df3, SUC_lexicon_tags_32 },
    { 0xaed08df3, SUC_lexicon_tags_4 },
    { 0x9d908df5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11308df8, SUC_lexicon_tags_1 },
    { 0x58908df9, SUC_lexicon_tags_11 },
    { 0xf508dfa, SUC_lexicon_tags_9 },
    { 0x79f08dfa, SUC_lexicon_tags_25 },
    { 0xa7b08dfc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72508dff, SUC_lexicon_tags_25 },
    { 0x68b08dff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8708e04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1d08e06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9308e13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f08e19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb08e1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8508e24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc108e27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f08e2b, SUC_lexicon_tags_17 },
    { 0x38f08e2b, SUC_lexicon_tags_27 },
    { 0xbb108e2d, SUC_lexicon_tags_11 },
    { 0x4eb08e2e, SUC_lexicon_tags_25 },
    { 0xac508e2e, SUC_lexicon_tags_12 },
    { 0x83908e2e, SUC_lexicon_tags_30 },
    { 0x64b08e2f, SUC_lexicon_tags_1 },
    { 0xaf508e32, SUC_lexicon_tags_1 },
    { 0xa1d08e2b, SUC_lexicon_tags_21 },
    { 0x3b508e34, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x21b08e36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cd08e38, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f08e3e, SUC_lexicon_tags_37 },
    { 0x4d508e3f, SUC_lexicon_tags_63 },
    { 0x83908e40, SUC_lexicon_tags_0 },
    { 0xc9d08e41, SUC_lexicon_tags_30 },
    { 0xfbb08e42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x52308e44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8508e4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51d08e4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85108e50, SUC_lexicon_tags_25 },
    { 0x99108e51, SUC_lexicon_tags_17 },
    { 0x6bf08e52, SUC_lexicon_tags_63 },
    { 0x64708e53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x68708e55, SUC_lexicon_tags_17 },
    { 0x49b08e56, SUC_lexicon_tags_10 },
    { 0x8bd08e57, SUC_lexicon_tags_29 },
    { 0xb5108e58, SUC_lexicon_tags_7 },
    { 0xff308e58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea308e5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7d08e5d, SUC_lexicon_tags_87 },
    { 0x96d08e5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77b08e60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f08e63, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7108e6b, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0x19708e6e, SUC_lexicon_tags_37 },
    { 0x40908e6f, SUC_lexicon_tags_25 },
    { 0x58d08e70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b308e73, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x43508e75, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54508e7b, SUC_lexicon_tags_2 },
    { 0x1b708e7c, SUC_lexicon_tags_11 },
    { 0xa2308e7d, SUC_lexicon_tags_20 },
    { 0x6cf08e7e, SUC_lexicon_tags_15 },
    { 0xeaf08e7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d08e82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf08e85, SUC_lexicon_tags_11 },
    { 0x57108e85, SUC_lexicon_tags_10 },
    { 0xd3108e87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd08e8e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xda708e90, SUC_lexicon_tags_160 },
    { 0x46108e91, SUC_lexicon_tags_38 },
    { 0x6b108e92, SUC_lexicon_tags_49 },
    { 0x4b708e92, SUC_lexicon_tags_11 },
    { 0xa6908e93, SUC_lexicon_tags_47 },
    { 0xad08e93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc308e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1308e9a, SUC_lexicon_tags_27 },
    { 0x34308e9a, SUC_lexicon_tags_23 },
    { 0xc6b08e9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28108ea2, SUC_lexicon_tags_27 },
    { 0xa6b08ea3, SUC_lexicon_tags_68 },
    { 0x56108ea4, SUC_lexicon_tags_14 },
    { 0x99108ea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f908ea8, SUC_lexicon_tags_14 },
    { 0xf908ea8, SUC_lexicon_tags_15 },
    { 0x1db08ea9, SUC_lexicon_tags_26 },
    { 0x10708eab, SUC_lexicon_tags_33 },
    { 0xa6308eac, SUC_lexicon_tags_12 },
    { 0x8d908ead, SUC_lexicon_tags_62 },
    { 0x82908eab, SUC_lexicon_tags_7 },
    { 0x4c108ead, SUC_lexicon_tags_38 },
    { 0xb1f08eb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90708eb4, SUC_lexicon_tags_2 },
    { 0x9b308eb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21108ebb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d08ebe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb08ec3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x26908ec5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6d08ec7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a708eca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25908ecd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x47508ed0, SUC_lexicon_tags_34 },
    { 0x94f08ed0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f08ed6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb308ed9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44308edb, SUC_lexicon_tags_14 },
    { 0x34d08edb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe108edf, SUC_lexicon_tags_7 },
    { 0x86308edf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d08ee3, SUC_lexicon_tags_26 },
    { 0x9fb08ee4, SUC_lexicon_tags_37 },
    { 0x11508ee5, SUC_lexicon_tags_8 },
    { 0x3df08ee6, SUC_lexicon_tags_14 },
    { 0xccf08ee5, SUC_lexicon_tags_31 },
    { 0xc1d08ee8, SUC_lexicon_tags_33 },
    { 0xe6308ee8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x88708eec, SUC_lexicon_tags_1 },
    { 0xf3308eec, SUC_lexicon_tags_63 },
    { 0x71108eee, SUC_lexicon_tags_37 },
    { 0x54d08eef, SUC_lexicon_tags_80 },
    { 0x41f08ef0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e708ef3, SUC_lexicon_tags_21 },
    { 0xd3f08ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87108ef8, SUC_lexicon_tags_23 },
    { 0xb6f08ef8, SUC_lexicon_tags_27 },
    { 0x7ef08ef9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef08efe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf08f04, SUC_lexicon_tags_10 },
    { 0x6c508f04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39908f0a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3cd08f0c, SUC_lexicon_tags_17 },
    { 0x7fd08f0d, SUC_lexicon_tags_45 },
    { 0xc9708f0e, SUC_lexicon_tags_38 },
    { 0x3d08f0d, SUC_lexicon_tags_15 },
    { 0xc3108f10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a708f16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8a708f18, SUC_lexicon_tags_30 },
    { 0x28708f19, SUC_lexicon_tags_9 },
    { 0x91f08f19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae708f2a, SUC_lexicon_tags_29 },
    { 0x97f08f2b, SUC_lexicon_tags_17 },
    { 0x59d08f2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x50708f2f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1708f37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53308f3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a508f3e, SUC_lexicon_tags_14 },
    { 0xb8108f3f, SUC_lexicon_tags_17 },
    { 0x7db08f40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9108f47, SUC_lexicon_tags_1 },
    { 0x99d08f48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f08f4b, SUC_lexicon_tags_25 },
    { 0x3a308f4c, SUC_lexicon_tags_25 },
    { 0x6508f4d, SUC_lexicon_tags_33 },
    { 0x5d708f4d, SUC_lexicon_tags_47 },
    { 0x99908f4d, SUC_lexicon_tags_1 },
    { 0xb8908f4c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x89308f53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4708f55, SUC_lexicon_tags_2 },
    { 0x7a308f55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16108f58, SUC_lexicon_tags_60 },
    { 0xd8b08f58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1108f5c, SUC_lexicon_tags_2 },
    { 0xe9708f5c, SUC_lexicon_tags_16 },
    { 0x26708f5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b908f61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48908f67, SUC_lexicon_tags_30 },
    { 0x9c308f68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9d908f6a, SUC_lexicon_tags_25 },
    { 0x2e308f6a, SUC_lexicon_tags_27 },
    { 0x1a508f6c, SUC_lexicon_tags_1 },
    { 0x7ff08f6d, SUC_lexicon_tags_151 },
    { 0x39b08f6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2908f73, SUC_lexicon_tags_47 },
    { 0x22d08f74, SUC_lexicon_tags_27 },
    { 0xbe908f74, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d08f78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b08f7b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5708f81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f08f85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b708f89, SUC_lexicon_tags_15 },
    { 0xed908f8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f508f8c, SUC_lexicon_tags_11 },
    { 0x60308f8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b508f8f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f08f92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2108f95, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6908f9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9308f9e, SUC_lexicon_tags_7 },
    { 0xd2508f9f, SUC_lexicon_tags_201 },
    { 0xcfd08fa0, SUC_lexicon_tags_13 },
    { 0x81308fa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f08fa5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f08fa8, SUC_lexicon_tags_17 },
    { 0x2108fa9, SUC_lexicon_tags_1 },
    { 0x28d08fa9, SUC_lexicon_tags_9 },
    { 0xe7108fa8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbb108fad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d708fb5, SUC_lexicon_tags_39 },
    { 0x1bf08fb6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95308fbe, SUC_lexicon_tags_0 },
    { 0xb0908fbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd08fc4, SUC_lexicon_tags_6 },
    { 0xe4b08fc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x14908fc7, SUC_lexicon_tags_15 },
    { 0x9e508fc8, SUC_lexicon_tags_2 },
    { 0x45d08fc8, SUC_lexicon_tags_11 },
    { 0xe2f08fca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d108fcc, SUC_lexicon_tags_12 },
    { 0xf9f08fcd, SUC_lexicon_tags_39 },
    { 0xc5508fce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd108fd1, SUC_lexicon_tags_1 },
    { 0xbf708fd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f908fd9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d08fdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7508fe3, SUC_lexicon_tags_16 },
    { 0x27f08fe4, SUC_lexicon_tags_87 },
    { 0x26f08fe5, SUC_lexicon_tags_207 },
    { 0xedb08fe6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b08fe9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f08fee, SUC_lexicon_tags_1 },
    { 0x2d908fef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1908ff1, SUC_lexicon_tags_14 },
    { 0xa8708ff1, SUC_lexicon_tags_25 },
    { 0x35708ff3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcd08ff5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2308ff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53708ffb, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x34d08ffd, SUC_lexicon_tags_70 },
    { 0x4a508ffd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa109001, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7509003, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0f09005, SUC_lexicon_tags_17 },
    { 0x2c309006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2530900a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd0900f, SUC_lexicon_tags_31 },
    { 0x7ff09010, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f709012, SUC_lexicon_tags_49 },
    { 0x67109013, SUC_lexicon_tags_25 },
    { 0x13909012, SUC_lexicon_tags_7 },
    { 0xda309015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7109017, SUC_lexicon_tags_17 },
    { 0x39f09017, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb0901b, SUC_lexicon_tags_25 },
    { 0x3b50901c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd810901e, SUC_lexicon_tags_1 },
    { 0xfa90901f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f09027, SUC_lexicon_tags_27 },
    { 0x7c909027, SUC_lexicon_tags_7 },
    { 0xdd309029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b50902c, SUC_lexicon_tags_25 },
    { 0xf790902d, SUC_lexicon_tags_10 },
    { 0xa4b0902c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7909032, SUC_lexicon_tags_2 },
    { 0x15309033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63109038, SUC_lexicon_tags_34 },
    { 0x40109039, SUC_lexicon_tags_65 },
    { 0xd0f0903a, SUC_lexicon_tags_12 },
    { 0x1750903a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b09040, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd09044, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b30904a, SUC_lexicon_tags_7 },
    { 0xcb50904b, SUC_lexicon_tags_25 },
    { 0x6030904b, SUC_lexicon_tags_7 },
    { 0xd5f0904d, SUC_lexicon_tags_56 },
    { 0x92b0904c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xc0909050, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b09056, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3d09058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60d0905a, SUC_lexicon_tags_32 },
    { 0x3bd0905b, SUC_lexicon_tags_11 },
    { 0xc70905c, SUC_lexicon_tags_30 },
    { 0xca30905b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f09060, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f09064, SUC_lexicon_tags_0 },
    { 0xdd909065, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x14709068, SUC_lexicon_tags_15 },
    { 0xe509069, SUC_lexicon_tags_28 },
    { 0xe8f09069, SUC_lexicon_tags_26 },
    { 0xf5b0906b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeeb0906d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x82909070, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4fb09072, SUC_lexicon_tags_2 },
    { 0x77309073, SUC_lexicon_tags_37 },
    { 0x5b109074, SUC_lexicon_tags_11 },
    { 0x66309075, SUC_lexicon_tags_43 },
    { 0xd6f09072, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc90907b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xbeb0907d, SUC_lexicon_tags_11 },
    { 0xe570907d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3509083, SUC_lexicon_tags_11 },
    { 0x1c909084, SUC_lexicon_tags_17 },
    { 0xa8509083, SUC_lexicon_tags_23 },
    { 0xf6b09086, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x35509089, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65509090, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93709094, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17309096, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5370909d, SUC_lexicon_tags_25 },
    { 0x12f0909e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d090a1, SUC_lexicon_tags_12 },
    { 0xe5090a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf5090a4, SUC_lexicon_tags_60 },
    { 0xb3090a4, SUC_lexicon_tags_69 },
    { 0x859090a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x235090a9, SUC_lexicon_tags_47 },
    { 0x277090aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1090ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc6f090b0, SUC_lexicon_tags_29 },
    { 0x7b7090b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc3090b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3f090b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f090b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1090bc, SUC_lexicon_tags_391 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41090bf, SUC_lexicon_tags_37 },
    { 0x9f1090bf, SUC_lexicon_tags_0 },
    { 0xcb1090c1, SUC_lexicon_tags_0 },
    { 0xa35090c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f090c5, SUC_lexicon_tags_30 },
    { 0xba3090c6, SUC_lexicon_tags_25 },
    { 0x3090c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc25090c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x791090cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b090d2, SUC_lexicon_tags_27 },
    { 0xf83090d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x463090d6, SUC_lexicon_tags_48 },
    { 0xc1f090d6, SUC_lexicon_tags_23 },
    { 0xa87090d7, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0x5fb090da, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8f7090dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37090df, SUC_lexicon_tags_53 },
    { 0xdfb090e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd090e3, SUC_lexicon_tags_37 },
    { 0xecf090e4, SUC_lexicon_tags_17 },
    { 0x6e5090e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb65090e7, SUC_lexicon_tags_48 },
    { 0x9a5090e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b090ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xad5090f0, SUC_lexicon_tags_8 },
    { 0x745090f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7090f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d1090f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3090f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad090fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b09103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46509105, SUC_lexicon_tags_27 },
    { 0x3d309105, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7509109, SUC_lexicon_tags_2 },
    { 0xd0109109, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x950910d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6250910f, SUC_lexicon_tags_15 },
    { 0x56f09110, SUC_lexicon_tags_124 },
    { 0xfc309111, SUC_lexicon_tags_0 },
    { 0x5e709111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b09115, SUC_lexicon_tags_1 },
    { 0x2b709115, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3cb09118, SUC_lexicon_tags_3 },
    { 0xdbd09118, SUC_lexicon_tags_60 },
    { 0xe1f09118, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe050911c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32909122, SUC_lexicon_tags_37 },
    { 0x50b09122, SUC_lexicon_tags_0 },
    { 0x5c709124, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x47b09126, SUC_lexicon_tags_2 },
    { 0x50b09127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd0912c, SUC_lexicon_tags_1 },
    { 0xe4f0912d, SUC_lexicon_tags_15 },
    { 0x7230912c, SUC_lexicon_tags_87 },
    { 0x5050912f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3309138, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b10913b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d09141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d0914a, SUC_lexicon_tags_6 },
    { 0x8ab0914b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d09150, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2eb09152, SUC_lexicon_tags_4 },
    { 0x21709153, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4309155, SUC_lexicon_tags_52 },
    { 0x81109156, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f09159, SUC_lexicon_tags_54 },
    { 0x3590915a, SUC_lexicon_tags_38 },
    { 0xb6f09159, SUC_lexicon_tags_11 },
    { 0x5630915b, SUC_lexicon_tags_37 },
    { 0x170915d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x40709160, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe4109162, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b09166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fb09168, SUC_lexicon_tags_11 },
    { 0x48109168, SUC_lexicon_tags_2 },
    { 0x4510916a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb210916d, SUC_lexicon_tags_15 },
    { 0x2110916e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff309172, SUC_lexicon_tags_14 },
    { 0x5d509172, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0917a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b0917d, SUC_lexicon_tags_7 },
    { 0x6870917e, SUC_lexicon_tags_47 },
    { 0xba30917f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba509188, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b0918b, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xf150918d, SUC_lexicon_tags_63 },
    { 0x86d0918e, SUC_lexicon_tags_17 },
    { 0x2930918f, SUC_lexicon_tags_12 },
    { 0x7210918f, SUC_lexicon_tags_27 },
    { 0xf9d09191, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63b09193, SUC_lexicon_tags_37 },
    { 0x5fb09193, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9f909196, SUC_lexicon_tags_6 },
    { 0x65309197, SUC_lexicon_tags_55 },
    { 0xba309198, SUC_lexicon_tags_15 },
    { 0xcdd09199, SUC_lexicon_tags_14 },
    { 0xb350919a, SUC_lexicon_tags_29 },
    { 0x7cd0919b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d0919f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9091a2, SUC_lexicon_tags_14 },
    { 0xa7f091a3, SUC_lexicon_tags_30 },
    { 0x797091a2, SUC_lexicon_tags_25 },
    { 0x555091a4, SUC_lexicon_tags_38 },
    { 0x8d3091a6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b091a9, SUC_lexicon_tags_17 },
    { 0xd6b091a9, SUC_lexicon_tags_11 },
    { 0xf05091aa, SUC_lexicon_tags_15 },
    { 0x6a7091aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x279091af, SUC_lexicon_tags_57 },
    { 0xd5d091b0, SUC_lexicon_tags_7 },
    { 0xb85091af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67091b4, SUC_lexicon_tags_15 },
    { 0x3d5091b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x457091b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531091bc, SUC_lexicon_tags_65 },
    { 0x13f091bd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x249091bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6c9091c1, SUC_lexicon_tags_4 },
    { 0xe79091c1, SUC_lexicon_tags_37 },
    { 0x977091c3, SUC_lexicon_tags_15 },
    { 0x1c3091c4, SUC_lexicon_tags_12 },
    { 0x291091c5, SUC_lexicon_tags_2 },
    { 0x54f091c5, SUC_lexicon_tags_56 },
    { 0x733091c7, SUC_lexicon_tags_17 },
    { 0x2d1091c8, SUC_lexicon_tags_15 },
    { 0xf3b091c8, SUC_lexicon_tags_1 },
    { 0x55d091c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703091d6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1091d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb091dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69091e7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8fb091e9, SUC_lexicon_tags_11 },
    { 0x13091ea, SUC_lexicon_tags_17 },
    { 0x4d3091eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcab091ed, SUC_lexicon_tags_19 },
    { 0x93d091ed, SUC_lexicon_tags_47 },
    { 0x97b091ee, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f091f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x463091fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db091ff, SUC_lexicon_tags_25 },
    { 0x31f091ff, SUC_lexicon_tags_27 },
    { 0x9e109201, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f309204, SUC_lexicon_tags_70 },
    { 0xc9b09205, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3909207, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x21709209, SUC_lexicon_tags_25 },
    { 0xcd0920a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd0920f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b09213, SUC_lexicon_tags_0 },
    { 0xeed09213, SUC_lexicon_tags_26 },
    { 0xfaf09213, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d90921a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae109225, SUC_lexicon_tags_7 },
    { 0xd5709226, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc909228, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x650922a, SUC_lexicon_tags_25 },
    { 0x5c50922b, SUC_lexicon_tags_23 },
    { 0x75b0922c, SUC_lexicon_tags_19 },
    { 0x86f0922c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1750922f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0909231, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db09234, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x44709237, SUC_lexicon_tags_12 },
    { 0xfe309238, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6a30923a, SUC_lexicon_tags_16 },
    { 0xaed0923b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb309240, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4309242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c909247, SUC_lexicon_tags_25 },
    { 0x56909248, SUC_lexicon_tags_1 },
    { 0x62309248, SUC_lexicon_tags_45 },
    { 0x12b0924a, SUC_lexicon_tags_40 },
    { 0x6c509248, SUC_lexicon_tags_14 },
    { 0xeb709249, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d0924f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f09252, SUC_lexicon_tags_47 },
    { 0x47b09253, SUC_lexicon_tags_7 },
    { 0x78b09254, SUC_lexicon_tags_21 },
    { 0xe5b09255, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1109257, SUC_lexicon_tags_47 },
    { 0x93709257, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4630925c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7f0925e, SUC_lexicon_tags_11 },
    { 0x8790925f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f09265, SUC_lexicon_tags_16 },
    { 0xa5309266, SUC_lexicon_tags_4 },
    { 0x11f09266, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa730926a, SUC_lexicon_tags_26 },
    { 0x53b0926b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1830926d, SUC_lexicon_tags_15 },
    { 0x2150926e, SUC_lexicon_tags_2 },
    { 0x36b0926f, SUC_lexicon_tags_39 },
    { 0xd4d09270, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x309273, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2309279, SUC_lexicon_tags_15 },
    { 0x4590927a, SUC_lexicon_tags_1 },
    { 0x4f0927a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e709280, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a909282, SUC_lexicon_tags_62 },
    { 0x7a509282, SUC_lexicon_tags_11 },
    { 0x9909282, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8850928b, SUC_lexicon_tags_11 },
    { 0x8970928b, SUC_lexicon_tags_15 },
    { 0x94b0928b, SUC_lexicon_tags_31 },
    { 0x4ef0928e, SUC_lexicon_tags_63 },
    { 0xb390928f, SUC_lexicon_tags_10 },
    { 0x350928c, SUC_lexicon_tags_45 },
    { 0xca709291, SUC_lexicon_tags_7 },
    { 0x59f09292, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac109296, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b0929c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed092a1, SUC_lexicon_tags_61 },
    { 0x5a1092a1, SUC_lexicon_tags_29 },
    { 0xfd3092a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x471092a6, SUC_lexicon_tags_12 },
    { 0xd23092a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429092b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f092b7, SUC_lexicon_tags_63 },
    { 0xe07092b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed092bb, SUC_lexicon_tags_23 },
    { 0x1f5092bc, SUC_lexicon_tags_26 },
    { 0x9bd092bd, SUC_lexicon_tags_42 },
    { 0x9a5092be, SUC_lexicon_tags_49 },
    { 0x1b092bc, SUC_lexicon_tags_17 },
    { 0x179092bc, SUC_lexicon_tags_7 },
    { 0x167092bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f092c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3092cb, SUC_lexicon_tags_37 },
    { 0xfef092cc, SUC_lexicon_tags_26 },
    { 0xc8f092cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39092d0, SUC_lexicon_tags_17 },
    { 0xdd3092d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbe7092d3, SUC_lexicon_tags_15 },
    { 0x2bd092d4, SUC_lexicon_tags_29 },
    { 0x887092d5, SUC_lexicon_tags_15 },
    { 0xed5092d3, SUC_lexicon_tags_15 },
    { 0x79f092d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31092da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407092de, SUC_lexicon_tags_37 },
    { 0xe21092df, SUC_lexicon_tags_7 },
    { 0x7a9092df, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b092e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331092ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x309092ec, SUC_lexicon_tags_11 },
    { 0xa95092ec, SUC_lexicon_tags_7 },
    { 0xd0f092ee, SUC_lexicon_tags_2 },
    { 0xa93092ec, SUC_lexicon_tags_7 },
    { 0xc8d092ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc73092f2, SUC_lexicon_tags_25 },
    { 0x51d092f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x259092f5, SUC_lexicon_tags_15 },
    { 0x5b1092f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6d092f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219092ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d09302, SUC_lexicon_tags_9 },
    { 0x2c109303, SUC_lexicon_tags_11 },
    { 0xc9909302, SUC_lexicon_tags_37 },
    { 0x1ff09305, SUC_lexicon_tags_6 },
    { 0x84309304, SUC_lexicon_tags_25 },
    { 0x15b09306, SUC_lexicon_tags_2 },
    { 0x25309308, SUC_lexicon_tags_1 },
    { 0x9df09309, SUC_lexicon_tags_31 },
    { 0x7dd0930a, SUC_lexicon_tags_14 },
    { 0x77509309, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a10930e, SUC_lexicon_tags_68 },
    { 0x6bb0930f, SUC_lexicon_tags_1 },
    { 0x76909310, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3709315, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d09319, SUC_lexicon_tags_11 },
    { 0x58d0931a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x34d0931c, SUC_lexicon_tags_6 },
    { 0x1790931d, SUC_lexicon_tags_37 },
    { 0x7b0931e, SUC_lexicon_tags_23 },
    { 0x7df0931f, SUC_lexicon_tags_41 },
    { 0xd9909320, SUC_lexicon_tags_14 },
    { 0xc2909320, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db09325, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb4509327, SUC_lexicon_tags_47 },
    { 0xaa509327, SUC_lexicon_tags_37 },
    { 0xe6f09329, SUC_lexicon_tags_60 },
    { 0xf3f09329, SUC_lexicon_tags_37 },
    { 0xb9b0932b, SUC_lexicon_tags_25 },
    { 0x4770932b, SUC_lexicon_tags_25 },
    { 0x5530932b, SUC_lexicon_tags_6 },
    { 0x9e10932e, SUC_lexicon_tags_39 },
    { 0xdff0932f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27909333, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb5509335, SUC_lexicon_tags_25 },
    { 0x34b09336, SUC_lexicon_tags_25 },
    { 0xd09337, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6330933a, SUC_lexicon_tags_11 },
    { 0xa9b0933a, SUC_lexicon_tags_40 },
    { 0x2fb0933b, SUC_lexicon_tags_56 },
    { 0x10d0933d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x6870933f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78709346, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a109348, SUC_lexicon_tags_38 },
    { 0x1309349, SUC_lexicon_tags_19 },
    { 0xd8f0934a, SUC_lexicon_tags_12 },
    { 0xe4909349, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d0934e, SUC_lexicon_tags_23 },
    { 0x5590934f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f09352, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9b109354, SUC_lexicon_tags_49 },
    { 0x98709355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09359, SUC_lexicon_tags_37 },
    { 0xc3309359, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b09362, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d09365, SUC_lexicon_tags_37 },
    { 0xcc309366, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda30936b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a90936f, SUC_lexicon_tags_24 },
    { 0x44309370, SUC_lexicon_tags_45 },
    { 0x70d09371, SUC_lexicon_tags_37 },
    { 0x3c509372, SUC_lexicon_tags_7 },
    { 0x8a509373, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2709375, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d0937c, SUC_lexicon_tags_11 },
    { 0xccf0937d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf09383, SUC_lexicon_tags_224 },
    { 0x3d909383, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9ef09386, SUC_lexicon_tags_11 },
    { 0x93509386, SUC_lexicon_tags_75 },
    { 0xd0309386, SUC_lexicon_tags_11 },
    { 0x6ab09389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce10938e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32909390, SUC_lexicon_tags_11 },
    { 0xac109391, SUC_lexicon_tags_37 },
    { 0x12d09392, SUC_lexicon_tags_23 },
    { 0xa9309393, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf09397, SUC_lexicon_tags_47 },
    { 0x5d509398, SUC_lexicon_tags_37 },
    { 0x6ff09399, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc250939b, SUC_lexicon_tags_1 },
    { 0x2cb0939b, SUC_lexicon_tags_13 },
    { 0x4990939d, SUC_lexicon_tags_0 },
    { 0x350939e, SUC_lexicon_tags_12 },
    { 0xc5f0939d, SUC_lexicon_tags_0 },
    { 0x313093a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff093a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb093aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117093af, SUC_lexicon_tags_47 },
    { 0xf11093af, SUC_lexicon_tags_12 },
    { 0x365093af, SUC_lexicon_tags_17 },
    { 0xcf1093b0, SUC_lexicon_tags_31 },
    { 0xf0b093b1, SUC_lexicon_tags_113 },
    { 0xa23093b3, SUC_lexicon_tags_1 },
    { 0xf77093b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb093b8, SUC_lexicon_tags_37 },
    { 0x44f093b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837093bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb093c1, SUC_lexicon_tags_133 },
    { 0xe01093c1, SUC_lexicon_tags_25 },
    { 0xf97093c1, SUC_lexicon_tags_17 },
    { 0x107093c4, SUC_lexicon_tags_25 },
    { 0x33d093c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b093c8, SUC_lexicon_tags_11 },
    { 0x439093c8, SUC_lexicon_tags_25 },
    { 0x92f093ca, SUC_lexicon_tags_74 },
    { 0x88d093ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef093ce, SUC_lexicon_tags_1 },
    { 0x4c9093ce, SUC_lexicon_tags_47 },
    { 0x74f093d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9093d6, SUC_lexicon_tags_63 },
    { 0x927093d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5093db, SUC_lexicon_tags_45 },
    { 0x5c1093dc, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xd13093de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57f093e0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe81093e2, SUC_lexicon_tags_17 },
    { 0xf7f093e2, SUC_lexicon_tags_2 },
    { 0xcf1093e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9093ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda9093f1, SUC_lexicon_tags_124 },
    { 0xc3b093f2, SUC_lexicon_tags_15 },
    { 0x737093f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaef093f5, SUC_lexicon_tags_47 },
    { 0xa0f093f5, SUC_lexicon_tags_49 },
    { 0x7dd093f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73f093fa, SUC_lexicon_tags_23 },
    { 0x96b093fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x555093fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x265093ff, SUC_lexicon_tags_1 },
    { 0x88709400, SUC_lexicon_tags_8 },
    { 0xe7909401, SUC_lexicon_tags_14 },
    { 0x4fd09401, SUC_lexicon_tags_34 },
    { 0x64309403, SUC_lexicon_tags_11 },
    { 0xee909401, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb09407, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b0940b, SUC_lexicon_tags_7 },
    { 0xa590940b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2f50940e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3709414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13509418, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9990941a, SUC_lexicon_tags_4 },
    { 0x7cd0941b, SUC_lexicon_tags_27 },
    { 0x2250941c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1230941e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21109427, SUC_lexicon_tags_1 },
    { 0x15d09428, SUC_lexicon_tags_14 },
    { 0x32709429, SUC_lexicon_tags_32 },
    { 0xb0309429, SUC_lexicon_tags_15 },
    { 0x8650942a, SUC_lexicon_tags_0 },
    { 0x510942c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f0942f, SUC_lexicon_tags_17 },
    { 0xa190942f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7909434, SUC_lexicon_tags_17 },
    { 0x2c509435, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9910943c, SUC_lexicon_tags_15 },
    { 0xcb70943d, SUC_lexicon_tags_11 },
    { 0xbb50943e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7109440, SUC_lexicon_tags_25 },
    { 0xdd09441, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e109445, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8109447, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x78d09449, SUC_lexicon_tags_33 },
    { 0x73d09449, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e109452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d709457, SUC_lexicon_tags_7 },
    { 0xaf709458, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4310945b, SUC_lexicon_tags_12 },
    { 0x7f10945b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2210945f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0309462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x909464, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb4309466, SUC_lexicon_tags_17 },
    { 0x89509466, SUC_lexicon_tags_61 },
    { 0x6e909468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59d0946a, SUC_lexicon_tags_11 },
    { 0xdaf0946a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a70946f, SUC_lexicon_tags_2 },
    { 0xa8109470, SUC_lexicon_tags_16 },
    { 0x10f0946f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21909477, SUC_lexicon_tags_47 },
    { 0x9fb09478, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf30947f, SUC_lexicon_tags_37 },
    { 0x4c909480, SUC_lexicon_tags_6 },
    { 0xf2309481, SUC_lexicon_tags_11 },
    { 0xbe309482, SUC_lexicon_tags_7 },
    { 0x33b09483, SUC_lexicon_tags_28 },
    { 0x49309482, SUC_lexicon_tags_29 },
    { 0x72509485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7109489, SUC_lexicon_tags_37 },
    { 0x9d10948a, SUC_lexicon_tags_15 },
    { 0xdd90948a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a90948e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66309492, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f30949a, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f50949e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9094a1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xecf094a3, SUC_lexicon_tags_12 },
    { 0x989094a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc9b094a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9094aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f094b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd094b7, SUC_lexicon_tags_17 },
    { 0x541094b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x231094ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b094bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d094c2, SUC_lexicon_tags_26 },
    { 0xf27094c3, SUC_lexicon_tags_21 },
    { 0x7cf094c4, SUC_lexicon_tags_23 },
    { 0xa5d094c4, SUC_lexicon_tags_25 },
    { 0xb5d094c6, SUC_lexicon_tags_45 },
    { 0xd9d094c3, SUC_lexicon_tags_21 },
    { 0x573094c8, SUC_lexicon_tags_27 },
    { 0xd87094c4, SUC_lexicon_tags_26 },
    { 0x10f094ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7b094cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1094d0, SUC_lexicon_tags_11 },
    { 0x17f094d1, SUC_lexicon_tags_27 },
    { 0x74b094d0, SUC_lexicon_tags_38 },
    { 0xdad094d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7094d6, SUC_lexicon_tags_33 },
    { 0xf3094d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41b094d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x449094dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f094df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x86b094e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59094e8, SUC_lexicon_tags_29 },
    { 0x3d7094e8, SUC_lexicon_tags_37 },
    { 0x825094ea, SUC_lexicon_tags_1 },
    { 0x64d094ea, SUC_lexicon_tags_2 },
    { 0xa37094ec, SUC_lexicon_tags_25 },
    { 0xca9094ec, SUC_lexicon_tags_10 },
    { 0x2f3094eb, SUC_lexicon_tags_30 },
    { 0x9f7094ea, SUC_lexicon_tags_49 },
    { 0x443094ea, SUC_lexicon_tags_15 },
    { 0xaf094f1, SUC_lexicon_tags_6 },
    { 0xd3f094f1, SUC_lexicon_tags_11 },
    { 0x62d094f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab094f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451094fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b09500, SUC_lexicon_tags_39 },
    { 0x2d509501, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x6c509503, SUC_lexicon_tags_17 },
    { 0x5ab09504, SUC_lexicon_tags_1 },
    { 0xc8f09504, SUC_lexicon_tags_32 },
    { 0x54b09506, SUC_lexicon_tags_60 },
    { 0xdc909507, SUC_lexicon_tags_2 },
    { 0xbbf09508, SUC_lexicon_tags_1 },
    { 0xc2b09509, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb470950b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf0950f, SUC_lexicon_tags_10 },
    { 0x50f0950f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67709513, SUC_lexicon_tags_43 },
    { 0x7c709513, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d09518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d10951b, SUC_lexicon_tags_10 },
    { 0x2510951b, SUC_lexicon_tags_7 },
    { 0xaa30951d, SUC_lexicon_tags_48 },
    { 0x1a90951e, SUC_lexicon_tags_38 },
    { 0xf7d0951f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad09522, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d09525, SUC_lexicon_tags_17 },
    { 0x52d09526, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcbd09528, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f0952c, SUC_lexicon_tags_11 },
    { 0x1b0952c, SUC_lexicon_tags_15 },
    { 0x8510952e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1509531, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2309535, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6909538, SUC_lexicon_tags_107 },
    { 0x38909539, SUC_lexicon_tags_79 },
    { 0x1b90953a, SUC_lexicon_tags_15 },
    { 0x34b09538, SUC_lexicon_tags_2 },
    { 0x33f0953a, SUC_lexicon_tags_10 },
    { 0xac70953d, SUC_lexicon_tags_37 },
    { 0xc5f0953e, SUC_lexicon_tags_62 },
    { 0x7130953c, SUC_lexicon_tags_17 },
    { 0x9a909540, SUC_lexicon_tags_3 },
    { 0x9e509541, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88f09543, SUC_lexicon_tags_1278 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b09546, SUC_lexicon_tags_28 },
    { 0x7b309547, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d0954b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7950954d, SUC_lexicon_tags_12 },
    { 0xf6b0954e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9709553, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb09557, SUC_lexicon_tags_23 },
    { 0x66b09558, SUC_lexicon_tags_17 },
    { 0xbfd09557, SUC_lexicon_tags_17 },
    { 0x7d0955a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b0955e, SUC_lexicon_tags_15 },
    { 0x3e70955f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd09564, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98509568, SUC_lexicon_tags_28 },
    { 0xccf09569, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1290956b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9970956e, SUC_lexicon_tags_12 },
    { 0x1ed0956f, SUC_lexicon_tags_17 },
    { 0xd7109570, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40709578, SUC_lexicon_tags_25 },
    { 0x78309579, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f0957d, SUC_lexicon_tags_38 },
    { 0x8cf0957e, SUC_lexicon_tags_61 },
    { 0xd290957f, SUC_lexicon_tags_25 },
    { 0x28709580, SUC_lexicon_tags_6 },
    { 0x16509580, SUC_lexicon_tags_47 },
    { 0x36709582, SUC_lexicon_tags_31 },
    { 0xff70957e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x32909585, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb510958f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d0959d, SUC_lexicon_tags_27 },
    { 0x4ff0959e, SUC_lexicon_tags_10 },
    { 0x4eb0959e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad095a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd095a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x907095ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1095b0, SUC_lexicon_tags_25 },
    { 0x9fb095b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3095b5, SUC_lexicon_tags_11 },
    { 0xe09095b6, SUC_lexicon_tags_140 },
    { 0x21b095b7, SUC_lexicon_tags_201 },
    { 0x953095b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9dd095ba, SUC_lexicon_tags_1 },
    { 0x551095bb, SUC_lexicon_tags_28 },
    { 0xedd095bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465095c1, SUC_lexicon_tags_37 },
    { 0xb73095c1, SUC_lexicon_tags_14 },
    { 0x24b095c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f095c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x99095ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225095cf, SUC_lexicon_tags_19 },
    { 0x223095cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5095d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x721095d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x301095d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47095db, SUC_lexicon_tags_13 },
    { 0x565095dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf095df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad095e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab095e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b095ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a7095ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129095f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d095f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd095fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x233095fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e3095ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd09602, SUC_lexicon_tags_1 },
    { 0x7c909603, SUC_lexicon_tags_14 },
    { 0x65309604, SUC_lexicon_tags_28 },
    { 0x88f09603, SUC_lexicon_tags_26 },
    { 0x76509606, SUC_lexicon_tags_17 },
    { 0x3f909607, SUC_lexicon_tags_12 },
    { 0x5db09608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b0960c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f09610, SUC_lexicon_tags_25 },
    { 0xb309610, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c709614, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb09619, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9510961b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x51d0961d, SUC_lexicon_tags_2 },
    { 0xedd0961e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4709622, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d109628, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bf0962a, SUC_lexicon_tags_57 },
    { 0xb550962b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb250962e, SUC_lexicon_tags_11 },
    { 0x8ab0962f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9f09631, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3af09634, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x9ef09636, SUC_lexicon_tags_14 },
    { 0x1f309637, SUC_lexicon_tags_2 },
    { 0xae309637, SUC_lexicon_tags_2 },
    { 0xd0709638, SUC_lexicon_tags_43 },
    { 0xd090963a, SUC_lexicon_tags_11 },
    { 0x5d90963b, SUC_lexicon_tags_68 },
    { 0x13d09638, SUC_lexicon_tags_27 },
    { 0x3630963d, SUC_lexicon_tags_2 },
    { 0xd5d0963e, SUC_lexicon_tags_1 },
    { 0xd030963f, SUC_lexicon_tags_15 },
    { 0x31709640, SUC_lexicon_tags_39 },
    { 0x87d0963e, SUC_lexicon_tags_29 },
    { 0xeab09642, SUC_lexicon_tags_23 },
    { 0xd0509642, SUC_lexicon_tags_85 },
    { 0x9e509644, SUC_lexicon_tags_12 },
    { 0xaad09645, SUC_lexicon_tags_25 },
    { 0x15b0963b, SUC_lexicon_tags_33 },
    { 0x87f09647, SUC_lexicon_tags_2 },
    { 0x31109647, SUC_lexicon_tags_2 },
    { 0x1ad0963c, SUC_lexicon_tags_3 },
    { 0x2e50963e, SUC_lexicon_tags_1 },
    { 0x6d10963e, SUC_lexicon_tags_47 },
    { 0x1810964c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c509650, SUC_lexicon_tags_38 },
    { 0x76709651, SUC_lexicon_tags_47 },
    { 0xdff09652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde909658, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b0965f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc309662, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9709669, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46b0966b, SUC_lexicon_tags_43 },
    { 0x4ef0966c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8190966e, SUC_lexicon_tags_33 },
    { 0xfc90966e, SUC_lexicon_tags_30 },
    { 0xdc309670, SUC_lexicon_tags_15 },
    { 0x5a509671, SUC_lexicon_tags_4 },
    { 0x7b109670, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d09676, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb0967a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b09680, SUC_lexicon_tags_34 },
    { 0xf3109681, SUC_lexicon_tags_18 },
    { 0x86509682, SUC_lexicon_tags_15 },
    { 0x71b09682, SUC_lexicon_tags_30 },
    { 0xbf909684, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc109686, SUC_lexicon_tags_37 },
    { 0x68f09687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1df09689, SUC_lexicon_tags_21 },
    { 0x3770968a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3db0968c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb50968e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbab09690, SUC_lexicon_tags_63 },
    { 0x39f09691, SUC_lexicon_tags_66 },
    { 0x70109690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0509698, SUC_lexicon_tags_15 },
    { 0xd8d09699, SUC_lexicon_tags_39 },
    { 0x68d0969a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e90969c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15d0969e, SUC_lexicon_tags_38 },
    { 0x5050969f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x617096a1, SUC_lexicon_tags_2 },
    { 0xe33096a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b3096a4, SUC_lexicon_tags_15 },
    { 0x819096a4, SUC_lexicon_tags_11 },
    { 0x87b096a6, SUC_lexicon_tags_85 },
    { 0x333096a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3096a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927096b2, SUC_lexicon_tags_66 },
    { 0xeb1096b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93096b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9096bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x243096be, SUC_lexicon_tags_1 },
    { 0x403096bf, SUC_lexicon_tags_14 },
    { 0x2ad096c0, SUC_lexicon_tags_13 },
    { 0x4cf096c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1096c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53096cd, SUC_lexicon_tags_37 },
    { 0x2b5096ce, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd096d1, SUC_lexicon_tags_16 },
    { 0xdb9096d2, SUC_lexicon_tags_37 },
    { 0x957096d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb096d6, SUC_lexicon_tags_17 },
    { 0xe05096d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f096da, SUC_lexicon_tags_33 },
    { 0x40f096db, SUC_lexicon_tags_25 },
    { 0x3e9096dc, SUC_lexicon_tags_0 },
    { 0x109096dc, SUC_lexicon_tags_44 },
    { 0x123096da, SUC_lexicon_tags_12 },
    { 0x4b1096df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61096e4, SUC_lexicon_tags_37 },
    { 0xd11096e4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7096e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca1096ea, SUC_lexicon_tags_12 },
    { 0xf71096eb, SUC_lexicon_tags_25 },
    { 0x9f7096ec, SUC_lexicon_tags_66 },
    { 0xf7f096ed, SUC_lexicon_tags_11 },
    { 0x4b9096ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5096f2, SUC_lexicon_tags_25 },
    { 0x73d096f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b096f6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d096f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319096ff, SUC_lexicon_tags_1 },
    { 0x5af09700, SUC_lexicon_tags_25 },
    { 0xeb309701, SUC_lexicon_tags_6 },
    { 0xbfd09702, SUC_lexicon_tags_82 },
    { 0x26509703, SUC_lexicon_tags_2 },
    { 0x67909700, SUC_lexicon_tags_45 },
    { 0x2cd096ff, SUC_lexicon_tags_66 },
    { 0x43509706, SUC_lexicon_tags_7 },
    { 0x6ab09703, SUC_lexicon_tags_92 },
    { 0x51d09701, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2850970b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f0970e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d509710, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b09718, SUC_lexicon_tags_1 },
    { 0x79309718, SUC_lexicon_tags_11 },
    { 0xf8b0971a, SUC_lexicon_tags_45 },
    { 0xa70971b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x2170971d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf709722, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x81b09724, SUC_lexicon_tags_9 },
    { 0x61309725, SUC_lexicon_tags_25 },
    { 0x3e109724, SUC_lexicon_tags_2 },
    { 0x1e709727, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4590972d, SUC_lexicon_tags_68 },
    { 0xb30972d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb09731, SUC_lexicon_tags_25 },
    { 0x7a909732, SUC_lexicon_tags_5 },
    { 0xa1b09733, SUC_lexicon_tags_25 },
    { 0xc8109734, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf909738, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf3b0973a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5030973d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83509748, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x250974c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0974f, SUC_lexicon_tags_11 },
    { 0x84d09750, SUC_lexicon_tags_24 },
    { 0x8030974f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf909757, SUC_lexicon_tags_15 },
    { 0x13109758, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1eb0975a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa90975f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xad509761, SUC_lexicon_tags_12 },
    { 0x2bb09762, SUC_lexicon_tags_17 },
    { 0x32309763, SUC_lexicon_tags_38 },
    { 0xd9509764, SUC_lexicon_tags_17 },
    { 0x90709765, SUC_lexicon_tags_17 },
    { 0xa6109762, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27509768, SUC_lexicon_tags_91 },
    { 0x9df09769, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7cf0976b, SUC_lexicon_tags_11 },
    { 0x2050976c, SUC_lexicon_tags_30 },
    { 0x8ab0976d, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0x990976f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57909775, SUC_lexicon_tags_37 },
    { 0x39509775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef0977c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d09780, SUC_lexicon_tags_15 },
    { 0xb109781, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2509786, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5109789, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1510978b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda90978e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c909790, SUC_lexicon_tags_25 },
    { 0x78309791, SUC_lexicon_tags_25 },
    { 0xd9709792, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f09795, SUC_lexicon_tags_40 },
    { 0x6109796, SUC_lexicon_tags_46 },
    { 0x57909797, SUC_lexicon_tags_31 },
    { 0xebd09798, SUC_lexicon_tags_63 },
    { 0x2a709798, SUC_lexicon_tags_28 },
    { 0xb410979a, SUC_lexicon_tags_2 },
    { 0xb710979b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x53d0979d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911097a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53097a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3097ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f097b0, SUC_lexicon_tags_37 },
    { 0xc7d097b1, SUC_lexicon_tags_33 },
    { 0xb3f097b0, SUC_lexicon_tags_47 },
    { 0xd27097b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3ab097b5, SUC_lexicon_tags_12 },
    { 0xb17097b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f097ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b097be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacf097c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d097c3, SUC_lexicon_tags_25 },
    { 0xf95097c4, SUC_lexicon_tags_42 },
    { 0xc8f097c5, SUC_lexicon_tags_7 },
    { 0x5e5097c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff3097c8, SUC_lexicon_tags_27 },
    { 0xb3d097c9, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b097cd, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd097d1, SUC_lexicon_tags_15 },
    { 0xa3d097d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1097d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef097d9, SUC_lexicon_tags_9 },
    { 0x1f097da, SUC_lexicon_tags_7 },
    { 0x805097db, SUC_lexicon_tags_12 },
    { 0x8af097da, SUC_lexicon_tags_47 },
    { 0x4d7097dd, SUC_lexicon_tags_7 },
    { 0xb73097de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89097e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x951097e9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f097f0, SUC_lexicon_tags_7 },
    { 0x789097f1, SUC_lexicon_tags_45 },
    { 0x3d3097f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707097f6, SUC_lexicon_tags_29 },
    { 0x4f7097f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199097fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9097fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b09803, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd0980a, SUC_lexicon_tags_7 },
    { 0xe830980b, SUC_lexicon_tags_87 },
    { 0x3270980c, SUC_lexicon_tags_42 },
    { 0x370980a, SUC_lexicon_tags_116 },
    { 0x1b10980e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb509811, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b09816, SUC_lexicon_tags_27 },
    { 0x3d309817, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x81b09819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f0981e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b09821, SUC_lexicon_tags_30 },
    { 0x89f09822, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3109824, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x72309826, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d0982c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1990982f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9df09831, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44109833, SUC_lexicon_tags_63 },
    { 0xcc509834, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b09844, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7909846, SUC_lexicon_tags_10 },
    { 0xbb09847, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f50984a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5250984e, SUC_lexicon_tags_7 },
    { 0xecd0984e, SUC_lexicon_tags_66 },
    { 0xadb0984f, SUC_lexicon_tags_6 },
    { 0x26d09851, SUC_lexicon_tags_106 },
    { 0xc1b09852, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb509855, SUC_lexicon_tags_6 },
    { 0x68509856, SUC_lexicon_tags_62 },
    { 0xf2109857, SUC_lexicon_tags_31 },
    { 0xb8509856, SUC_lexicon_tags_37 },
    { 0xd2509855, SUC_lexicon_tags_25 },
    { 0xa8d0985a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b10985e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d09865, SUC_lexicon_tags_49 },
    { 0x3b909866, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4070986b, SUC_lexicon_tags_10 },
    { 0xcb10986c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72109871, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c30987a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd390987e, SUC_lexicon_tags_31 },
    { 0x7c10987f, SUC_lexicon_tags_63 },
    { 0xe3f09880, SUC_lexicon_tags_25 },
    { 0xb9f0987f, SUC_lexicon_tags_6 },
    { 0xbf09880, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db09889, SUC_lexicon_tags_12 },
    { 0xf5909889, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0f0988c, SUC_lexicon_tags_7 },
    { 0x6410988c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x970988f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa3509891, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850989f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745098a3, SUC_lexicon_tags_23 },
    { 0xbaf098a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef098a9, SUC_lexicon_tags_17 },
    { 0x72d098aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9098b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x87f098b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf098b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7098ba, SUC_lexicon_tags_9 },
    { 0x841098bb, SUC_lexicon_tags_25 },
    { 0xedb098bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65098c0, SUC_lexicon_tags_33 },
    { 0x62d098c1, SUC_lexicon_tags_15 },
    { 0x1e9098c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225098c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af098cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x313098cd, SUC_lexicon_tags_23 },
    { 0xebb098cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df098d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6d098d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34d098d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91098d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7098e0, SUC_lexicon_tags_25 },
    { 0xa3f098e1, SUC_lexicon_tags_15 },
    { 0xd95098e1, SUC_lexicon_tags_11 },
    { 0x19098e0, SUC_lexicon_tags_17 },
    { 0xbfb098e4, SUC_lexicon_tags_15 },
    { 0xed5098e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d098e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5098f0, SUC_lexicon_tags_39 },
    { 0x8fb098f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89098f5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdc9098f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57098fa, SUC_lexicon_tags_8 },
    { 0x779098fb, SUC_lexicon_tags_158 },
    { 0x9bf098fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d09900, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8909904, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb09907, SUC_lexicon_tags_2 },
    { 0x63f09908, SUC_lexicon_tags_37 },
    { 0xa3109907, SUC_lexicon_tags_2 },
    { 0xdd70990a, SUC_lexicon_tags_1 },
    { 0xb310990b, SUC_lexicon_tags_60 },
    { 0xdb30990a, SUC_lexicon_tags_47 },
    { 0x21109909, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5d0990f, SUC_lexicon_tags_16 },
    { 0x37909910, SUC_lexicon_tags_37 },
    { 0x6f709911, SUC_lexicon_tags_54 },
    { 0xc1309911, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b09919, SUC_lexicon_tags_2 },
    { 0xc7b0991a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5d10991c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509921, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75d09923, SUC_lexicon_tags_11 },
    { 0x85709924, SUC_lexicon_tags_11 },
    { 0x7f709925, SUC_lexicon_tags_1 },
    { 0x79909926, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3509929, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb09930, SUC_lexicon_tags_43 },
    { 0x4ed09931, SUC_lexicon_tags_0 },
    { 0x1a509932, SUC_lexicon_tags_11 },
    { 0x1f109933, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf09936, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b30993a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca709943, SUC_lexicon_tags_2 },
    { 0xa4109944, SUC_lexicon_tags_47 },
    { 0xe1b09943, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40709947, SUC_lexicon_tags_11 },
    { 0xd6309948, SUC_lexicon_tags_60 },
    { 0xb2109948, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb250994b, SUC_lexicon_tags_0 },
    { 0x1050994c, SUC_lexicon_tags_19 },
    { 0xaa70994d, SUC_lexicon_tags_38 },
    { 0xf1d0994e, SUC_lexicon_tags_84 },
    { 0x3d0994e, SUC_lexicon_tags_84 },
    { 0xc050994d, SUC_lexicon_tags_11 },
    { 0xb1f09950, SUC_lexicon_tags_32 },
    { 0xccd09952, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4d909954, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x77f09956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230995d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8909960, SUC_lexicon_tags_15 },
    { 0x6309960, SUC_lexicon_tags_33 },
    { 0xf3109961, SUC_lexicon_tags_55 },
    { 0xa0509960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa810996a, SUC_lexicon_tags_31 },
    { 0x2d70996b, SUC_lexicon_tags_8 },
    { 0x9690996c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5109974, SUC_lexicon_tags_12 },
    { 0x6709974, SUC_lexicon_tags_26 },
    { 0x77309976, SUC_lexicon_tags_7 },
    { 0x7a509977, SUC_lexicon_tags_2 },
    { 0x27709977, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2270997a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd570997f, SUC_lexicon_tags_1 },
    { 0x77309980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d09986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8490998c, SUC_lexicon_tags_15 },
    { 0x51d0998c, SUC_lexicon_tags_49 },
    { 0xdcd0998c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c709994, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x91509996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf109999, SUC_lexicon_tags_83 },
    { 0x49109999, SUC_lexicon_tags_49 },
    { 0xe50999a, SUC_lexicon_tags_13 },
    { 0x4170999c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc90999e, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35099a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf23099a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x535099a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb7099ab, SUC_lexicon_tags_57 },
    { 0xda9099ab, SUC_lexicon_tags_79 },
    { 0xab7099ad, SUC_lexicon_tags_12 },
    { 0x4ff099ad, SUC_lexicon_tags_15 },
    { 0x5a7099af, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b099b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b099b9, SUC_lexicon_tags_1 },
    { 0x2c3099ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d099bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcff099bf, SUC_lexicon_tags_37 },
    { 0x329099bf, SUC_lexicon_tags_13 },
    { 0xeb9099c0, SUC_lexicon_tags_16 },
    { 0x213099c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d099c7, SUC_lexicon_tags_12 },
    { 0x6db099c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7f099ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5099cf, SUC_lexicon_tags_25 },
    { 0x711099d0, SUC_lexicon_tags_1 },
    { 0x66b099d1, SUC_lexicon_tags_19 },
    { 0xde3099d2, SUC_lexicon_tags_23 },
    { 0x85099d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb099d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469099db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x225099de, SUC_lexicon_tags_1 },
    { 0xb31099de, SUC_lexicon_tags_29 },
    { 0xafb099df, SUC_lexicon_tags_43 },
    { 0x6c3099e0, SUC_lexicon_tags_39 },
    { 0xc1f099de, SUC_lexicon_tags_151 },
    { 0x2c5099de, SUC_lexicon_tags_45 },
    { 0xa81099e1, SUC_lexicon_tags_30 },
    { 0xaa9099e3, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0xc87099e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9099eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x871099ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e3099ef, SUC_lexicon_tags_25 },
    { 0x163099f0, SUC_lexicon_tags_3 },
    { 0x419099f0, SUC_lexicon_tags_15 },
    { 0xb5f099f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf19099f4, SUC_lexicon_tags_12 },
    { 0xb81099f4, SUC_lexicon_tags_55 },
    { 0x7cd099f6, SUC_lexicon_tags_45 },
    { 0x85b099f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e7099f9, SUC_lexicon_tags_22 },
    { 0xd25099fa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd509a00, SUC_lexicon_tags_25 },
    { 0x87d09a01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cd09a03, SUC_lexicon_tags_37 },
    { 0x97f09a04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df09a08, SUC_lexicon_tags_21 },
    { 0xd2509a09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b09a0c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98509a12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb09a15, SUC_lexicon_tags_11 },
    { 0x8f709a15, SUC_lexicon_tags_15 },
    { 0x3eb09a17, SUC_lexicon_tags_1178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf709a1b, SUC_lexicon_tags_15 },
    { 0x8ab09a1c, SUC_lexicon_tags_45 },
    { 0xa3d09a1d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db09a20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f09a27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69509a29, SUC_lexicon_tags_19 },
    { 0x49b09a29, SUC_lexicon_tags_25 },
    { 0x5909a2b, SUC_lexicon_tags_85 },
    { 0x43309a2c, SUC_lexicon_tags_17 },
    { 0x2f909a2b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb4909a2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf09a32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62509a37, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8e909a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91d09a3b, SUC_lexicon_tags_34 },
    { 0x47f09a3b, SUC_lexicon_tags_11 },
    { 0xf7509a3d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f09a41, SUC_lexicon_tags_4 },
    { 0xccf09a42, SUC_lexicon_tags_37 },
    { 0x7f709a43, SUC_lexicon_tags_140 },
    { 0x92b09a43, SUC_lexicon_tags_17 },
    { 0x62909a45, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5109a49, SUC_lexicon_tags_17 },
    { 0x16d09a49, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4af09a4c, SUC_lexicon_tags_1 },
    { 0x55b09a4d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb709a4f, SUC_lexicon_tags_14 },
    { 0xd1b09a50, SUC_lexicon_tags_1 },
    { 0xd1f09a51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb09a54, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc5909a56, SUC_lexicon_tags_33 },
    { 0x2a709a56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7109a59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4af09a5b, SUC_lexicon_tags_1 },
    { 0xf3709a5c, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xba509a5f, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xda909a62, SUC_lexicon_tags_30 },
    { 0x11309a63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x86f09a65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6909a69, SUC_lexicon_tags_25 },
    { 0xbf09a69, SUC_lexicon_tags_47 },
    { 0xef709a6b, SUC_lexicon_tags_38 },
    { 0x5e709a6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50709a6e, SUC_lexicon_tags_11 },
    { 0xdd509a6e, SUC_lexicon_tags_60 },
    { 0xa309a70, SUC_lexicon_tags_30 },
    { 0x11309a6f, SUC_lexicon_tags_68 },
    { 0x86b09a72, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x70109a74, SUC_lexicon_tags_25 },
    { 0xec309a74, SUC_lexicon_tags_11 },
    { 0x4a909a76, SUC_lexicon_tags_30 },
    { 0x6c309a76, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3f509a79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3909a7e, SUC_lexicon_tags_49 },
    { 0xf5909a7f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x24309a82, SUC_lexicon_tags_12 },
    { 0x63d09a83, SUC_lexicon_tags_23 },
    { 0x1a709a84, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f09a88, SUC_lexicon_tags_9 },
    { 0xd6509a89, SUC_lexicon_tags_0 },
    { 0xeaf09a8a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4f309a8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb909a8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d109a91, SUC_lexicon_tags_34 },
    { 0x52709a92, SUC_lexicon_tags_37 },
    { 0x2c109a93, SUC_lexicon_tags_66 },
    { 0x1b09a91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1909a96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46b09a98, SUC_lexicon_tags_14 },
    { 0xc0309a98, SUC_lexicon_tags_2 },
    { 0x4a309a99, SUC_lexicon_tags_12 },
    { 0x79709a9a, SUC_lexicon_tags_92 },
    { 0x29909a9c, SUC_lexicon_tags_11 },
    { 0x6c709a9a, SUC_lexicon_tags_37 },
    { 0x27509a9b, SUC_lexicon_tags_0 },
    { 0xd0909a9f, SUC_lexicon_tags_14 },
    { 0x51509aa0, SUC_lexicon_tags_11 },
    { 0xac709a98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d909aa8, SUC_lexicon_tags_0 },
    { 0x4a309aa8, SUC_lexicon_tags_45 },
    { 0xe4309aaa, SUC_lexicon_tags_8 },
    { 0x14f09aa8, SUC_lexicon_tags_46 },
    { 0x87709aac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d09ab7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a709abd, SUC_lexicon_tags_68 },
    { 0x4d109abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacd09ac0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d09ac3, SUC_lexicon_tags_9 },
    { 0xd7d09ac3, SUC_lexicon_tags_47 },
    { 0x1b909ac5, SUC_lexicon_tags_7 },
    { 0x8909ac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab09ac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d09acb, SUC_lexicon_tags_29 },
    { 0x91309acc, SUC_lexicon_tags_7 },
    { 0xdad09acd, SUC_lexicon_tags_1 },
    { 0x74709acd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa9909ad0, SUC_lexicon_tags_2 },
    { 0x9909ad0, SUC_lexicon_tags_15 },
    { 0x22109ad0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23509ad8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41509ade, SUC_lexicon_tags_25 },
    { 0x94f09ade, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb09ae4, SUC_lexicon_tags_7 },
    { 0x7f09ae4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b09aea, SUC_lexicon_tags_12 },
    { 0x9c109aeb, SUC_lexicon_tags_21 },
    { 0x92f09aec, SUC_lexicon_tags_25 },
    { 0x5df09aed, SUC_lexicon_tags_31 },
    { 0xc9509aed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc1709af0, SUC_lexicon_tags_12 },
    { 0x13109af0, SUC_lexicon_tags_12 },
    { 0x67509af0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6509af7, SUC_lexicon_tags_27 },
    { 0x8bb09af8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f09afb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c309afd, SUC_lexicon_tags_82 },
    { 0x51b09afe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8909b02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81909b06, SUC_lexicon_tags_37 },
    { 0xcaf09b07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98309b0b, SUC_lexicon_tags_39 },
    { 0xdfb09b0c, SUC_lexicon_tags_11 },
    { 0xe5909b0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d09b13, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xfa709b15, SUC_lexicon_tags_11 },
    { 0xa8f09b15, SUC_lexicon_tags_43 },
    { 0xfef09b15, SUC_lexicon_tags_43 },
    { 0xc5909b18, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xff509b1b, SUC_lexicon_tags_2 },
    { 0x23b09b1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b09b20, SUC_lexicon_tags_26 },
    { 0xecb09b20, SUC_lexicon_tags_2 },
    { 0x57909b20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31b09b24, SUC_lexicon_tags_57 },
    { 0x59b09b25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c509b28, SUC_lexicon_tags_7 },
    { 0xccd09b29, SUC_lexicon_tags_31 },
    { 0xdeb09b2a, SUC_lexicon_tags_23 },
    { 0xa0d09b29, SUC_lexicon_tags_16 },
    { 0x16109b2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d09b3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b09b3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f09b40, SUC_lexicon_tags_33 },
    { 0xf09b41, SUC_lexicon_tags_11 },
    { 0x7eb09b40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b909b44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d109b49, SUC_lexicon_tags_63 },
    { 0x3d109b4a, SUC_lexicon_tags_37 },
    { 0x73b09b4b, SUC_lexicon_tags_25 },
    { 0xb709b4c, SUC_lexicon_tags_15 },
    { 0xb1909b4d, SUC_lexicon_tags_15 },
    { 0x18309b4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a909b50, SUC_lexicon_tags_7 },
    { 0x23309b51, SUC_lexicon_tags_27 },
    { 0x86109b50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49309b54, SUC_lexicon_tags_46 },
    { 0x3c909b55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df09b5d, SUC_lexicon_tags_17 },
    { 0x25d09b5e, SUC_lexicon_tags_15 },
    { 0x31109b5f, SUC_lexicon_tags_25 },
    { 0x5f09b5e, SUC_lexicon_tags_2 },
    { 0xd509b60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50709b64, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb09b67, SUC_lexicon_tags_2 },
    { 0xff09b67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a309b6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x85909b6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e909b71, SUC_lexicon_tags_53 },
    { 0xd3909b71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b09b79, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb09b7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5109b7e, SUC_lexicon_tags_39 },
    { 0x92309b7f, SUC_lexicon_tags_3 },
    { 0x55109b80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f09b84, SUC_lexicon_tags_42 },
    { 0x78709b85, SUC_lexicon_tags_28 },
    { 0xd1b09b86, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d09b89, SUC_lexicon_tags_11 },
    { 0x67d09b8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xae109b8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b509b90, SUC_lexicon_tags_49 },
    { 0xe709b90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f09b98, SUC_lexicon_tags_2 },
    { 0xfd109b99, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b09b9e, SUC_lexicon_tags_0 },
    { 0x7d909b9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78509ba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f709ba6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84509ba8, SUC_lexicon_tags_23 },
    { 0x20f09ba9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31709bad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6109bb0, SUC_lexicon_tags_25 },
    { 0x23909bb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0909bb7, SUC_lexicon_tags_8 },
    { 0xef709bb7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f09bbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f09bc4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d709bcb, SUC_lexicon_tags_31 },
    { 0xc7509bcc, SUC_lexicon_tags_6 },
    { 0x19f09bcc, SUC_lexicon_tags_23 },
    { 0x45309bcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf09bd9, SUC_lexicon_tags_19 },
    { 0xd8709bd9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f09bde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6309be1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3e309be3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d09be7, SUC_lexicon_tags_100 },
    { 0x2f09be8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x60d09bea, SUC_lexicon_tags_1 },
    { 0xe0f09bea, SUC_lexicon_tags_17 },
    { 0x84309bec, SUC_lexicon_tags_45 },
    { 0x19d09bec, SUC_lexicon_tags_15 },
    { 0x78109bee, SUC_lexicon_tags_7 },
    { 0x55b09bef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb1709bf1, SUC_lexicon_tags_11 },
    { 0xdad09bf2, SUC_lexicon_tags_45 },
    { 0x25509bf1, SUC_lexicon_tags_37 },
    { 0xcaf09bf2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4eb09bf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e909bf9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94709bfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9309c0a, SUC_lexicon_tags_1181 },
    { 0xfc309c0b, SUC_lexicon_tags_37 },
    { 0xb309c0c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d09c12, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60309c16, SUC_lexicon_tags_8 },
    { 0xb1909c17, SUC_lexicon_tags_11 },
    { 0x9c309c18, SUC_lexicon_tags_37 },
    { 0xae709c16, SUC_lexicon_tags_60 },
    { 0x59109c1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46709c1e, SUC_lexicon_tags_15 },
    { 0xa6709c1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f09c25, SUC_lexicon_tags_18 },
    { 0xeab09c25, SUC_lexicon_tags_11 },
    { 0x34f09c25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65109c29, SUC_lexicon_tags_65 },
    { 0x63f09c2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78909c2d, SUC_lexicon_tags_29 },
    { 0x66709c2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3509c30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4309c33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0b09c35, SUC_lexicon_tags_17 },
    { 0x70f09c36, SUC_lexicon_tags_10 },
    { 0x88109c35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5309c39, SUC_lexicon_tags_37 },
    { 0xb9f09c39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29109c3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8909c3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f09c41, SUC_lexicon_tags_6 },
    { 0x5fd09c42, SUC_lexicon_tags_45 },
    { 0x72709c42, SUC_lexicon_tags_15 },
    { 0x97509c44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3309c48, SUC_lexicon_tags_11 },
    { 0x61709c48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54509c4c, SUC_lexicon_tags_2 },
    { 0xee309c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb909c4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d09c53, SUC_lexicon_tags_47 },
    { 0xbad09c54, SUC_lexicon_tags_27 },
    { 0x9309c54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2909c57, SUC_lexicon_tags_37 },
    { 0xb8109c57, SUC_lexicon_tags_25 },
    { 0x41509c59, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1909c5c, SUC_lexicon_tags_5 },
    { 0x6b109c5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f09c62, SUC_lexicon_tags_37 },
    { 0x5e709c63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8909c65, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c709c6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf509c74, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd309c78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6509c7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d09c7f, SUC_lexicon_tags_60 },
    { 0xe109c80, SUC_lexicon_tags_61 },
    { 0x97709c80, SUC_lexicon_tags_61 },
    { 0xc8b09c81, SUC_lexicon_tags_63 },
    { 0x4109c83, SUC_lexicon_tags_23 },
    { 0x6a709c81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a709c86, SUC_lexicon_tags_8 },
    { 0x2bb09c87, SUC_lexicon_tags_17 },
    { 0xeb709c87, SUC_lexicon_tags_37 },
    { 0xd4309c86, SUC_lexicon_tags_25 },
    { 0x51509c87, SUC_lexicon_tags_1 },
    { 0xf3709c8b, SUC_lexicon_tags_107 },
    { 0xc3109c8c, SUC_lexicon_tags_65 },
    { 0x60309c8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x78909c90, SUC_lexicon_tags_7 },
    { 0xe7d09c90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d09c9a, SUC_lexicon_tags_9 },
    { 0xec309c9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f09ca0, SUC_lexicon_tags_37 },
    { 0x6d709ca1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f09ca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5709ca6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbf09ca8, SUC_lexicon_tags_61 },
    { 0xc8d09ca9, SUC_lexicon_tags_8 },
    { 0xd2b09caa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf4109cac, SUC_lexicon_tags_103 },
    { 0xbff09cac, SUC_lexicon_tags_17 },
    { 0x13309cae, SUC_lexicon_tags_26 },
    { 0x4a109cac, SUC_lexicon_tags_27 },
    { 0x7d309caf, SUC_lexicon_tags_13 },
    { 0x33109cac, SUC_lexicon_tags_6 },
    { 0xebf09cb2, SUC_lexicon_tags_31 },
    { 0xea309cb3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1b109cb5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f09cbd, SUC_lexicon_tags_49 },
    { 0x96709cbd, SUC_lexicon_tags_91 },
    { 0x66f09cbf, SUC_lexicon_tags_17 },
    { 0x6e109cc0, SUC_lexicon_tags_17 },
    { 0xf4709cbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84509cc3, SUC_lexicon_tags_6 },
    { 0xcd709cc3, SUC_lexicon_tags_7 },
    { 0x68709cc5, SUC_lexicon_tags_33 },
    { 0x73b09cc6, SUC_lexicon_tags_25 },
    { 0x71309cc7, SUC_lexicon_tags_11 },
    { 0x1e509cc6, SUC_lexicon_tags_27 },
    { 0x32509cc4, SUC_lexicon_tags_76 },
    { 0x11909cca, SUC_lexicon_tags_63 },
    { 0x78509cc7, SUC_lexicon_tags_47 },
    { 0x8bf09cc9, SUC_lexicon_tags_63 },
    { 0x4eb09cc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b09ccf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d909cd2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41909cd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x98d09cd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d309cda, SUC_lexicon_tags_1 },
    { 0xd7309cdb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1af09cdd, SUC_lexicon_tags_1 },
    { 0x54709cde, SUC_lexicon_tags_31 },
    { 0xfc509cde, SUC_lexicon_tags_14 },
    { 0x3709ce0, SUC_lexicon_tags_47 },
    { 0x5cb09ce1, SUC_lexicon_tags_1 },
    { 0xcf509ce1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x88909ce4, SUC_lexicon_tags_23 },
    { 0x96909ce5, SUC_lexicon_tags_7 },
    { 0x9309ce6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d09ceb, SUC_lexicon_tags_37 },
    { 0x80909cec, SUC_lexicon_tags_54 },
    { 0x1a909ced, SUC_lexicon_tags_10 },
    { 0x23709cec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28309cf4, SUC_lexicon_tags_70 },
    { 0x6fb09cf5, SUC_lexicon_tags_1 },
    { 0x6509cf6, SUC_lexicon_tags_2 },
    { 0x9f309cf6, SUC_lexicon_tags_15 },
    { 0x95f09cf5, SUC_lexicon_tags_2 },
    { 0xd309cf9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23109d01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x48109d03, SUC_lexicon_tags_57 },
    { 0xe3909d03, SUC_lexicon_tags_15 },
    { 0x2fd09d04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40b09d07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7309d0b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf9709d0d, SUC_lexicon_tags_12 },
    { 0x99d09d0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3709d12, SUC_lexicon_tags_45 },
    { 0xce109d13, SUC_lexicon_tags_60 },
    { 0xde909d14, SUC_lexicon_tags_28 },
    { 0xd4b09d15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30109d20, SUC_lexicon_tags_14 },
    { 0xe9d09d21, SUC_lexicon_tags_16 },
    { 0x36309d20, SUC_lexicon_tags_37 },
    { 0x8f309d23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb909d2b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe3309d2d, SUC_lexicon_tags_47 },
    { 0x1df09d2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3909d35, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f09d39, SUC_lexicon_tags_1 },
    { 0xcef09d3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c109d43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d09d46, SUC_lexicon_tags_19 },
    { 0xfc109d46, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xeef09d49, SUC_lexicon_tags_8 },
    { 0x7a909d49, SUC_lexicon_tags_42 },
    { 0x16509d4b, SUC_lexicon_tags_11 },
    { 0x41f09d4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75109d4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3109d54, SUC_lexicon_tags_34 },
    { 0x77909d54, SUC_lexicon_tags_6 },
    { 0x72309d56, SUC_lexicon_tags_14 },
    { 0x90d09d57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a309d5a, SUC_lexicon_tags_27 },
    { 0xbef09d5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57109d5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x40509d62, SUC_lexicon_tags_12 },
    { 0x77f09d63, SUC_lexicon_tags_1 },
    { 0xabb09d63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3d09d66, SUC_lexicon_tags_9 },
    { 0x87709d67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a709d69, SUC_lexicon_tags_15 },
    { 0x11309d6a, SUC_lexicon_tags_16 },
    { 0xd3109d6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb7509d6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b09d70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bd09d72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cf09d74, SUC_lexicon_tags_0 },
    { 0x3bf09d75, SUC_lexicon_tags_25 },
    { 0x24d09d76, SUC_lexicon_tags_23 },
    { 0xccb09d74, SUC_lexicon_tags_0 },
    { 0xa8109d78, SUC_lexicon_tags_17 },
    { 0x40309d79, SUC_lexicon_tags_1 },
    { 0xd1509d79, SUC_lexicon_tags_1 },
    { 0xa9909d78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70b09d7d, SUC_lexicon_tags_30 },
    { 0x60d09d7e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b09d81, SUC_lexicon_tags_17 },
    { 0xbb109d81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd09d89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f509d8b, SUC_lexicon_tags_0 },
    { 0x6c909d8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d09d8f, SUC_lexicon_tags_7 },
    { 0xe0309d90, SUC_lexicon_tags_14 },
    { 0x27909d90, SUC_lexicon_tags_10 },
    { 0x25d09d8f, SUC_lexicon_tags_17 },
    { 0xb8909d93, SUC_lexicon_tags_23 },
    { 0xc2509d94, SUC_lexicon_tags_47 },
    { 0xd109d95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78d09d97, SUC_lexicon_tags_25 },
    { 0x5a109d98, SUC_lexicon_tags_28 },
    { 0xab309d99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x73509d9c, SUC_lexicon_tags_38 },
    { 0xfc709d9d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x29b09d9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e509da3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb09da8, SUC_lexicon_tags_37 },
    { 0x8e709da9, SUC_lexicon_tags_17 },
    { 0xf6d09da9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa109dae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf09db2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf09dbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b09dc1, SUC_lexicon_tags_17 },
    { 0xd5709dc1, SUC_lexicon_tags_2 },
    { 0x56909dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdbb09dc5, SUC_lexicon_tags_1 },
    { 0x22909dc6, SUC_lexicon_tags_21 },
    { 0xb2709dc7, SUC_lexicon_tags_1 },
    { 0xaff09dc7, SUC_lexicon_tags_1 },
    { 0x13709dc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97509dcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x65309dd0, SUC_lexicon_tags_25 },
    { 0xa6709dd1, SUC_lexicon_tags_25 },
    { 0xb6509dd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d509dd7, SUC_lexicon_tags_14 },
    { 0x55309dd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac909ddd, SUC_lexicon_tags_39 },
    { 0x85309dde, SUC_lexicon_tags_15 },
    { 0xc6d09ddf, SUC_lexicon_tags_68 },
    { 0xe1109dde, SUC_lexicon_tags_50 },
    { 0xa2509de1, SUC_lexicon_tags_11 },
    { 0x89b09dde, SUC_lexicon_tags_2 },
    { 0x6af09de3, SUC_lexicon_tags_131 },
    { 0x48b09de4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b09de8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf09ded, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5eb09def, SUC_lexicon_tags_92 },
    { 0xdfd09df0, SUC_lexicon_tags_40 },
    { 0xde709df1, SUC_lexicon_tags_37 },
    { 0xb8709df1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4709df8, SUC_lexicon_tags_15 },
    { 0x82309df9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xf309dfb, SUC_lexicon_tags_49 },
    { 0xd7b09dfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3f09dfe, SUC_lexicon_tags_39 },
    { 0x47109dff, SUC_lexicon_tags_11 },
    { 0x41d09e00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13509e03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x29909e06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa9309e08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb09e0e, SUC_lexicon_tags_12 },
    { 0xbff09e0f, SUC_lexicon_tags_28 },
    { 0x4509e10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f09e14, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b09e1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70309e1f, SUC_lexicon_tags_28 },
    { 0x37d09e1f, SUC_lexicon_tags_17 },
    { 0x2fd09e20, SUC_lexicon_tags_1712 },
    { 0, NULL },
    { 0x1c509e23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a109e25, SUC_lexicon_tags_17 },
    { 0x60109e25, SUC_lexicon_tags_2 },
    { 0x9cb09e27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x19f09e29, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb509e2d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8ef09e2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb09e32, SUC_lexicon_tags_25 },
    { 0x80309e33, SUC_lexicon_tags_34 },
    { 0x4f309e34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f09e37, SUC_lexicon_tags_63 },
    { 0xdbb09e38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6109e3b, SUC_lexicon_tags_10 },
    { 0x20109e3c, SUC_lexicon_tags_63 },
    { 0xc9b09e3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceb09e3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45309e41, SUC_lexicon_tags_47 },
    { 0x3b09e42, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x81309e44, SUC_lexicon_tags_1 },
    { 0x9df09e45, SUC_lexicon_tags_2 },
    { 0x6ad09e44, SUC_lexicon_tags_0 },
    { 0xc1f09e47, SUC_lexicon_tags_7 },
    { 0x31509e47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfd09e4a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x68d09e4c, SUC_lexicon_tags_40 },
    { 0x5f109e4d, SUC_lexicon_tags_21 },
    { 0x5ef09e4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78709e50, SUC_lexicon_tags_15 },
    { 0x83509e51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf509e55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d709e5d, SUC_lexicon_tags_39 },
    { 0x26909e5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0309e60, SUC_lexicon_tags_14 },
    { 0xf709e61, SUC_lexicon_tags_33 },
    { 0xfaf09e61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9bf09e64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x74309e67, SUC_lexicon_tags_14 },
    { 0x9eb09e68, SUC_lexicon_tags_34 },
    { 0x98109e68, SUC_lexicon_tags_63 },
    { 0x51f09e6a, SUC_lexicon_tags_2 },
    { 0x26509e6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b09e70, SUC_lexicon_tags_15 },
    { 0xa7909e70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cb09e73, SUC_lexicon_tags_15 },
    { 0xa7f09e73, SUC_lexicon_tags_25 },
    { 0x7709e74, SUC_lexicon_tags_11 },
    { 0xa7909e74, SUC_lexicon_tags_25 },
    { 0x84509e77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68909e7a, SUC_lexicon_tags_62 },
    { 0xead09e7b, SUC_lexicon_tags_23 },
    { 0xf8f09e7a, SUC_lexicon_tags_15 },
    { 0x81109e7b, SUC_lexicon_tags_15 },
    { 0xc4b09e7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d09e82, SUC_lexicon_tags_55 },
    { 0xc5509e82, SUC_lexicon_tags_37 },
    { 0x4a909e84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac709e86, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26509e88, SUC_lexicon_tags_55 },
    { 0x71309e89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af09e8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4909e8e, SUC_lexicon_tags_25 },
    { 0x64f09e8e, SUC_lexicon_tags_37 },
    { 0x29309e8f, SUC_lexicon_tags_15 },
    { 0x2eb09e91, SUC_lexicon_tags_16 },
    { 0x41b09e91, SUC_lexicon_tags_37 },
    { 0xa8309e93, SUC_lexicon_tags_1 },
    { 0xf2109e8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95309e96, SUC_lexicon_tags_11 },
    { 0x98b09e96, SUC_lexicon_tags_23 },
    { 0xf9b09e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13109e9c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d509e9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d09ea9, SUC_lexicon_tags_12 },
    { 0x2e309eaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c109eac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d09eaf, SUC_lexicon_tags_8 },
    { 0x18109eaf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd109eb3, SUC_lexicon_tags_6 },
    { 0x4c309eb4, SUC_lexicon_tags_37 },
    { 0xc6309eb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb09eb8, SUC_lexicon_tags_17 },
    { 0x1c909eb8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f09ebc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22909ebe, SUC_lexicon_tags_7 },
    { 0x96509ebe, SUC_lexicon_tags_107 },
    { 0x95109ec0, SUC_lexicon_tags_14 },
    { 0x21509ec1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3109ec3, SUC_lexicon_tags_68 },
    { 0x27d09ec4, SUC_lexicon_tags_6 },
    { 0x6f09ec4, SUC_lexicon_tags_17 },
    { 0xf2d09ec6, SUC_lexicon_tags_36 },
    { 0x20f09ec7, SUC_lexicon_tags_30 },
    { 0xde909ec8, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb09ecc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7909ecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f709ed3, SUC_lexicon_tags_1 },
    { 0x86b09ed4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x14d09ed6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db09ed9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d09edb, SUC_lexicon_tags_38 },
    { 0x6f509edc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98109ee4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca509ee9, SUC_lexicon_tags_17 },
    { 0x57709eea, SUC_lexicon_tags_23 },
    { 0x8eb09eeb, SUC_lexicon_tags_17 },
    { 0x38f09eec, SUC_lexicon_tags_77 },
    { 0xa2d09eeb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11709ef1, SUC_lexicon_tags_26 },
    { 0x64909ef2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af09ef7, SUC_lexicon_tags_17 },
    { 0x83d09ef8, SUC_lexicon_tags_25 },
    { 0xa4d09ef8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50b09efb, SUC_lexicon_tags_57 },
    { 0xeab09efb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f09f00, SUC_lexicon_tags_46 },
    { 0xecf09f00, SUC_lexicon_tags_38 },
    { 0xe2309f02, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae509f06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3309f08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce509f0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f09f0f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xed309f11, SUC_lexicon_tags_49 },
    { 0x8d709f11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9509f17, SUC_lexicon_tags_114 },
    { 0xf3109f18, SUC_lexicon_tags_49 },
    { 0x7df09f18, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61509f1d, SUC_lexicon_tags_48 },
    { 0x43709f1e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f709f28, SUC_lexicon_tags_2 },
    { 0xad109f29, SUC_lexicon_tags_38 },
    { 0x7fb09f29, SUC_lexicon_tags_37 },
    { 0xbbb09f2b, SUC_lexicon_tags_25 },
    { 0x15509f2c, SUC_lexicon_tags_11 },
    { 0xb7f09f2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32109f30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb09f33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd09f36, SUC_lexicon_tags_14 },
    { 0x30109f36, SUC_lexicon_tags_21 },
    { 0xc5709f38, SUC_lexicon_tags_9 },
    { 0x1309f39, SUC_lexicon_tags_21 },
    { 0x43d09f3a, SUC_lexicon_tags_12 },
    { 0x54109f39, SUC_lexicon_tags_15 },
    { 0x96f09f3c, SUC_lexicon_tags_23 },
    { 0x32709f3d, SUC_lexicon_tags_37 },
    { 0x4e309f3e, SUC_lexicon_tags_9 },
    { 0x26509f3c, SUC_lexicon_tags_29 },
    { 0xdb509f40, SUC_lexicon_tags_37 },
    { 0xb5909f40, SUC_lexicon_tags_17 },
    { 0xcb09f3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35709f44, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf09f4a, SUC_lexicon_tags_39 },
    { 0x4c109f4a, SUC_lexicon_tags_14 },
    { 0x75709f4a, SUC_lexicon_tags_47 },
    { 0xa9f09f4a, SUC_lexicon_tags_15 },
    { 0xecd09f4a, SUC_lexicon_tags_49 },
    { 0x3f909f4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d509f51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f509f55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1109f5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d09f5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9509f64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90309f66, SUC_lexicon_tags_34 },
    { 0xf509f67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f309f6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e309f6e, SUC_lexicon_tags_47 },
    { 0x1f709f6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80909f76, SUC_lexicon_tags_25 },
    { 0x90d09f77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19b09f79, SUC_lexicon_tags_15 },
    { 0xaed09f7a, SUC_lexicon_tags_25 },
    { 0x78f09f7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d09f80, SUC_lexicon_tags_2 },
    { 0xc509f81, SUC_lexicon_tags_2 },
    { 0x7e709f82, SUC_lexicon_tags_21 },
    { 0xe509f83, SUC_lexicon_tags_60 },
    { 0x6ad09f80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7db09f86, SUC_lexicon_tags_1 },
    { 0x65309f87, SUC_lexicon_tags_55 },
    { 0x54d09f86, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f09f90, SUC_lexicon_tags_7 },
    { 0xd8d09f91, SUC_lexicon_tags_7 },
    { 0xb8b09f92, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48909f98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2cd09f9a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa909f9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f09fa3, SUC_lexicon_tags_17 },
    { 0x93b09fa4, SUC_lexicon_tags_29 },
    { 0x72f09fa4, SUC_lexicon_tags_64 },
    { 0xd0309fa6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf09fa9, SUC_lexicon_tags_40 },
    { 0x81109faa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b09fad, SUC_lexicon_tags_15 },
    { 0xbd109fae, SUC_lexicon_tags_15 },
    { 0x7309faf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad09fb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x16909fb9, SUC_lexicon_tags_29 },
    { 0x9d709fba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb09fbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26709fc0, SUC_lexicon_tags_27 },
    { 0xc8509fc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d09fc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb109fcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d09fd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77709fe5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0309fe8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7309fea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef309fec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa3f09fee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x50b09ff0, SUC_lexicon_tags_47 },
    { 0x4dd09ff1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd709ff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef909ff9, SUC_lexicon_tags_1 },
    { 0x83109ffa, SUC_lexicon_tags_4 },
    { 0x84709ff9, SUC_lexicon_tags_47 },
    { 0x10909ffc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab90a006, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f70a009, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9e70a00b, SUC_lexicon_tags_7 },
    { 0x1210a00c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4650a00e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd570a012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fb0a014, SUC_lexicon_tags_7 },
    { 0x6350a015, SUC_lexicon_tags_25 },
    { 0x3110a016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd0a019, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5cb0a01b, SUC_lexicon_tags_45 },
    { 0x24b0a01b, SUC_lexicon_tags_2 },
    { 0x58f0a01d, SUC_lexicon_tags_77 },
    { 0xe530a01d, SUC_lexicon_tags_10 },
    { 0xb270a01b, SUC_lexicon_tags_12 },
    { 0xb6f0a01b, SUC_lexicon_tags_49 },
    { 0x8710a01c, SUC_lexicon_tags_46 },
    { 0xd5f0a022, SUC_lexicon_tags_37 },
    { 0xc250a023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0a02c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db0a033, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2f0a035, SUC_lexicon_tags_11 },
    { 0x9270a035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb550a038, SUC_lexicon_tags_8 },
    { 0x5c30a039, SUC_lexicon_tags_27 },
    { 0x2070a038, SUC_lexicon_tags_12 },
    { 0xdd50a03b, SUC_lexicon_tags_65 },
    { 0xfdf0a03b, SUC_lexicon_tags_57 },
    { 0x9890a03d, SUC_lexicon_tags_19 },
    { 0x1c70a03d, SUC_lexicon_tags_47 },
    { 0xa3d0a03e, SUC_lexicon_tags_15 },
    { 0x2eb0a039, SUC_lexicon_tags_11 },
    { 0x90d0a041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8650a04a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6770a04f, SUC_lexicon_tags_4 },
    { 0x8550a050, SUC_lexicon_tags_45 },
    { 0xb290a051, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7750a055, SUC_lexicon_tags_37 },
    { 0xba70a056, SUC_lexicon_tags_2 },
    { 0xe870a056, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x1db0a059, SUC_lexicon_tags_15 },
    { 0x9750a05a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0a065, SUC_lexicon_tags_17 },
    { 0xcf10a065, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e50a068, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d0a06b, SUC_lexicon_tags_21 },
    { 0x9310a06c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5f0a06e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7970a076, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b0a079, SUC_lexicon_tags_2 },
    { 0x8110a07a, SUC_lexicon_tags_23 },
    { 0x5c10a07b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b0a07e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6210a080, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf530a087, SUC_lexicon_tags_69 },
    { 0x7310a088, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb0a08c, SUC_lexicon_tags_45 },
    { 0x44d0a08d, SUC_lexicon_tags_11 },
    { 0x44d0a08e, SUC_lexicon_tags_25 },
    { 0x4970a08c, SUC_lexicon_tags_37 },
    { 0x8fb0a08f, SUC_lexicon_tags_3 },
    { 0xceb0a091, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcdb0a093, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d0a099, SUC_lexicon_tags_19 },
    { 0x82b0a09a, SUC_lexicon_tags_49 },
    { 0xbbf0a09b, SUC_lexicon_tags_34 },
    { 0xd5b0a09c, SUC_lexicon_tags_63 },
    { 0x97f0a09b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a10a0a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb290a0ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bb0a0ad, SUC_lexicon_tags_37 },
    { 0xe850a0ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf810a0b0, SUC_lexicon_tags_7 },
    { 0x5350a0b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0a0b5, SUC_lexicon_tags_37 },
    { 0x3d70a0b6, SUC_lexicon_tags_23 },
    { 0xf1d0a0b6, SUC_lexicon_tags_34 },
    { 0xaf0a0b8, SUC_lexicon_tags_15 },
    { 0xc8b0a0b8, SUC_lexicon_tags_43 },
    { 0x5d10a0ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa870a0bd, SUC_lexicon_tags_2 },
    { 0xa2d0a0be, SUC_lexicon_tags_4 },
    { 0xc670a0bf, SUC_lexicon_tags_6 },
    { 0xd830a0c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd090a0c3, SUC_lexicon_tags_167 },
    { 0xb290a0c4, SUC_lexicon_tags_84 },
    { 0x7170a0c4, SUC_lexicon_tags_37 },
    { 0x7330a0c4, SUC_lexicon_tags_1 },
    { 0xd010a0c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b0a0ca, SUC_lexicon_tags_49 },
    { 0xeef0a0cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3030a0d1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf630a0d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8350a0d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2bb0a0d9, SUC_lexicon_tags_12 },
    { 0x6330a0da, SUC_lexicon_tags_12 },
    { 0x3fd0a0d9, SUC_lexicon_tags_17 },
    { 0x54d0a0dc, SUC_lexicon_tags_1 },
    { 0x490a0dc, SUC_lexicon_tags_12 },
    { 0x9ed0a0de, SUC_lexicon_tags_0 },
    { 0x2a50a0dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1030a0e3, SUC_lexicon_tags_17 },
    { 0x5e70a0e4, SUC_lexicon_tags_6 },
    { 0xeaf0a0e5, SUC_lexicon_tags_160 },
    { 0x64b0a0e6, SUC_lexicon_tags_37 },
    { 0xb10a0e7, SUC_lexicon_tags_0 },
    { 0x8130a0e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab90a0eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6570a0ef, SUC_lexicon_tags_17 },
    { 0x7dd0a0f0, SUC_lexicon_tags_11 },
    { 0xb130a0f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2970a0f3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb0a0f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe6b0a0f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f10a0fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd530a0ff, SUC_lexicon_tags_37 },
    { 0x7930a100, SUC_lexicon_tags_85 },
    { 0x7fb0a101, SUC_lexicon_tags_26 },
    { 0xcaf0a102, SUC_lexicon_tags_37 },
    { 0x4c90a103, SUC_lexicon_tags_10 },
    { 0xc650a102, SUC_lexicon_tags_31 },
    { 0xe890a102, SUC_lexicon_tags_34 },
    { 0xf110a106, SUC_lexicon_tags_47 },
    { 0x7450a107, SUC_lexicon_tags_437 },
    { 0x5bd0a103, SUC_lexicon_tags_15 },
    { 0x9030a101, SUC_lexicon_tags_37 },
    { 0xce10a104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bb0a10c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4970a10f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced0a115, SUC_lexicon_tags_13 },
    { 0x6c30a116, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb7b0a118, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa270a11b, SUC_lexicon_tags_8 },
    { 0xfa90a11c, SUC_lexicon_tags_27 },
    { 0x27f0a11d, SUC_lexicon_tags_38 },
    { 0x38f0a11c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x11b0a120, SUC_lexicon_tags_1128 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf0a125, SUC_lexicon_tags_45 },
    { 0xc1d0a125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc50a12c, SUC_lexicon_tags_2 },
    { 0x2150a12d, SUC_lexicon_tags_14 },
    { 0x83f0a12e, SUC_lexicon_tags_25 },
    { 0xbf30a12f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe470a133, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0a136, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b0a13f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b70a142, SUC_lexicon_tags_103 },
    { 0xc10a143, SUC_lexicon_tags_6 },
    { 0x8010a144, SUC_lexicon_tags_27 },
    { 0x33b0a145, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x90b0a147, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf550a149, SUC_lexicon_tags_7 },
    { 0x34b0a149, SUC_lexicon_tags_49 },
    { 0x8e90a149, SUC_lexicon_tags_47 },
    { 0x85f0a14c, SUC_lexicon_tags_30 },
    { 0x4a70a14c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f90a152, SUC_lexicon_tags_49 },
    { 0x9810a153, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7430a157, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf530a15a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530a161, SUC_lexicon_tags_12 },
    { 0x4ad0a162, SUC_lexicon_tags_39 },
    { 0x5c10a163, SUC_lexicon_tags_1 },
    { 0x7b70a163, SUC_lexicon_tags_11 },
    { 0x250a165, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f0a168, SUC_lexicon_tags_6 },
    { 0x9d0a169, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d0a16f, SUC_lexicon_tags_0 },
    { 0x85f0a16f, SUC_lexicon_tags_87 },
    { 0xe50a16f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba70a179, SUC_lexicon_tags_61 },
    { 0x8d50a17a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf0a17f, SUC_lexicon_tags_19 },
    { 0xae30a180, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf0a188, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d0a18b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d50a18f, SUC_lexicon_tags_65 },
    { 0xc930a190, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x48b0a192, SUC_lexicon_tags_11 },
    { 0xd7d0a193, SUC_lexicon_tags_23 },
    { 0xf8f0a194, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb50a198, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8590a19a, SUC_lexicon_tags_19 },
    { 0x750a19a, SUC_lexicon_tags_2 },
    { 0x9430a19c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4310a1a0, SUC_lexicon_tags_11 },
    { 0x76b0a1a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f10a1a5, SUC_lexicon_tags_35 },
    { 0x27b0a1a6, SUC_lexicon_tags_37 },
    { 0x3670a1a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb90a1aa, SUC_lexicon_tags_19 },
    { 0x1f0a1ab, SUC_lexicon_tags_13 },
    { 0x4eb0a1ac, SUC_lexicon_tags_16 },
    { 0x110a1ad, SUC_lexicon_tags_0 },
    { 0xff0a1aa, SUC_lexicon_tags_26 },
    { 0x6510a1af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc410a1b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe470a1b7, SUC_lexicon_tags_98 },
    { 0xc970a1b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4870a1bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c30a1bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x79b0a1c1, SUC_lexicon_tags_37 },
    { 0xeef0a1c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa210a1c4, SUC_lexicon_tags_37 },
    { 0xbc70a1c5, SUC_lexicon_tags_21 },
    { 0xdf90a1c4, SUC_lexicon_tags_37 },
    { 0xeb30a1c6, SUC_lexicon_tags_37 },
    { 0x3950a1c8, SUC_lexicon_tags_75 },
    { 0x1770a1c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2870a1cc, SUC_lexicon_tags_33 },
    { 0x5970a1cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe770a1d0, SUC_lexicon_tags_37 },
    { 0x7df0a1d1, SUC_lexicon_tags_1 },
    { 0xc3d0a1d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d0a1d6, SUC_lexicon_tags_15 },
    { 0x6590a1d7, SUC_lexicon_tags_34 },
    { 0x6b10a1d8, SUC_lexicon_tags_220 },
    { 0x99f0a1d9, SUC_lexicon_tags_37 },
    { 0xd90a1da, SUC_lexicon_tags_15 },
    { 0xfd0a1db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab0a1e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc310a1e6, SUC_lexicon_tags_11 },
    { 0xb150a1e7, SUC_lexicon_tags_165 },
    { 0xfe90a1e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b10a1ea, SUC_lexicon_tags_12 },
    { 0x2930a1ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaab0a1ed, SUC_lexicon_tags_61 },
    { 0xc90a1ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d0a1f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa590a1fb, SUC_lexicon_tags_15 },
    { 0x30f0a1fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4650a200, SUC_lexicon_tags_25 },
    { 0x22b0a201, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7510a205, SUC_lexicon_tags_48 },
    { 0xf030a206, SUC_lexicon_tags_39 },
    { 0x3cd0a206, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe50a20c, SUC_lexicon_tags_17 },
    { 0x2fb0a20d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c50a210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4530a219, SUC_lexicon_tags_30 },
    { 0x25d0a21a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b0a223, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf050a226, SUC_lexicon_tags_12 },
    { 0xd9f0a227, SUC_lexicon_tags_15 },
    { 0x5b10a228, SUC_lexicon_tags_11 },
    { 0xfed0a229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f0a22c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbb0a22e, SUC_lexicon_tags_17 },
    { 0x1670a22f, SUC_lexicon_tags_19 },
    { 0x2f70a230, SUC_lexicon_tags_7 },
    { 0xf690a231, SUC_lexicon_tags_37 },
    { 0xaf30a232, SUC_lexicon_tags_7 },
    { 0x6f90a233, SUC_lexicon_tags_21 },
    { 0x3550a232, SUC_lexicon_tags_65 },
    { 0xefd0a235, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x35f0a237, SUC_lexicon_tags_23 },
    { 0xccb0a238, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf710a23a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b50a23d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8730a240, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7210a243, SUC_lexicon_tags_1 },
    { 0xb1f0a243, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f0a247, SUC_lexicon_tags_47 },
    { 0x6ff0a248, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2110a24a, SUC_lexicon_tags_2 },
    { 0xa290a24b, SUC_lexicon_tags_49 },
    { 0xbaf0a24c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe470a253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd370a258, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce70a25f, SUC_lexicon_tags_1 },
    { 0x3190a260, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d0a266, SUC_lexicon_tags_37 },
    { 0x170a267, SUC_lexicon_tags_28 },
    { 0x1810a266, SUC_lexicon_tags_31 },
    { 0xe6d0a266, SUC_lexicon_tags_15 },
    { 0x24d0a26a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d0a26d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb10a273, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb150a27b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd30a27e, SUC_lexicon_tags_37 },
    { 0x8f0a27f, SUC_lexicon_tags_27 },
    { 0x1b90a27f, SUC_lexicon_tags_7 },
    { 0x7650a27e, SUC_lexicon_tags_25 },
    { 0xc490a27f, SUC_lexicon_tags_38 },
    { 0x29d0a27e, SUC_lexicon_tags_18 },
    { 0x3710a280, SUC_lexicon_tags_34 },
    { 0xa850a285, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xca70a287, SUC_lexicon_tags_31 },
    { 0x9090a287, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30a28d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d0a294, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f0a297, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f50a29c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7190a2a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1f0a2a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c30a2a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa610a2a9, SUC_lexicon_tags_15 },
    { 0x1cb0a2aa, SUC_lexicon_tags_37 },
    { 0xad30a2aa, SUC_lexicon_tags_14 },
    { 0xc7d0a2aa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xca30a2ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec50a2b1, SUC_lexicon_tags_8 },
    { 0x93d0a2b2, SUC_lexicon_tags_11 },
    { 0x6e10a2b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23f0a2b5, SUC_lexicon_tags_38 },
    { 0xf7d0a2b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb9f0a2b8, SUC_lexicon_tags_12 },
    { 0x9bd0a2b9, SUC_lexicon_tags_25 },
    { 0x4370a2ba, SUC_lexicon_tags_49 },
    { 0xd2f0a2bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x430a2bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f30a2bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b0a2c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf290a2c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b0a2c7, SUC_lexicon_tags_38 },
    { 0x9650a2c8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe050a2d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b0a2d8, SUC_lexicon_tags_0 },
    { 0x9650a2d9, SUC_lexicon_tags_7 },
    { 0xa010a2d9, SUC_lexicon_tags_37 },
    { 0x4230a2db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d70a2dd, SUC_lexicon_tags_2 },
    { 0x98b0a2de, SUC_lexicon_tags_46 },
    { 0x6f50a2df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd810a2e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf0a2e5, SUC_lexicon_tags_10 },
    { 0xf3d0a2e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad0a2e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d0a2ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc530a2f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf890a2f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc7f0a2f4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x9af0a2f6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf0a2f9, SUC_lexicon_tags_95 },
    { 0xbcb0a2f9, SUC_lexicon_tags_66 },
    { 0x8090a2fa, SUC_lexicon_tags_25 },
    { 0x390a2fc, SUC_lexicon_tags_87 },
    { 0xe410a2f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa770a300, SUC_lexicon_tags_35 },
    { 0xf030a301, SUC_lexicon_tags_27 },
    { 0xd490a302, SUC_lexicon_tags_23 },
    { 0x6df0a303, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f0a307, SUC_lexicon_tags_9 },
    { 0x7b70a307, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad90a30c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b0a30e, SUC_lexicon_tags_37 },
    { 0x4730a30e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9890a319, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b0a31e, SUC_lexicon_tags_30 },
    { 0x2450a31f, SUC_lexicon_tags_7 },
    { 0x56f0a320, SUC_lexicon_tags_11 },
    { 0x2d10a321, SUC_lexicon_tags_25 },
    { 0x4a50a320, SUC_lexicon_tags_17 },
    { 0xaf70a323, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2470a328, SUC_lexicon_tags_1 },
    { 0x3b90a328, SUC_lexicon_tags_661 },
    { 0x8c30a32a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5770a32c, SUC_lexicon_tags_17 },
    { 0x80b0a32c, SUC_lexicon_tags_37 },
    { 0xa1d0a32c, SUC_lexicon_tags_172 },
    { 0x79d0a32f, SUC_lexicon_tags_27 },
    { 0xa270a330, SUC_lexicon_tags_66 },
    { 0xbaf0a32d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5450a335, SUC_lexicon_tags_30 },
    { 0xe270a336, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf170a338, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30a33d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7690a345, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2530a348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad90a34a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd950a34e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b70a354, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3450a356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb0a35a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca70a35e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6190a360, SUC_lexicon_tags_1 },
    { 0xb4b0a361, SUC_lexicon_tags_1 },
    { 0xd450a362, SUC_lexicon_tags_25 },
    { 0x7190a363, SUC_lexicon_tags_25 },
    { 0xdb70a364, SUC_lexicon_tags_7 },
    { 0x6f70a361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4470a368, SUC_lexicon_tags_7 },
    { 0x2bd0a369, SUC_lexicon_tags_1 },
    { 0x950a36a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf310a370, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd250a372, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6bf0a374, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52d0a376, SUC_lexicon_tags_12 },
    { 0x2b10a377, SUC_lexicon_tags_11 },
    { 0x60f0a377, SUC_lexicon_tags_17 },
    { 0x66b0a379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9df0a37b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e90a382, SUC_lexicon_tags_2 },
    { 0xf4b0a382, SUC_lexicon_tags_6 },
    { 0x6d10a384, SUC_lexicon_tags_17 },
    { 0xbfb0a384, SUC_lexicon_tags_1 },
    { 0x7ad0a382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a90a389, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b50a38d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7110a38f, SUC_lexicon_tags_2 },
    { 0x42f0a38f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xce90a393, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e30a395, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b10a39b, SUC_lexicon_tags_45 },
    { 0xba50a39b, SUC_lexicon_tags_1 },
    { 0x5ad0a39d, SUC_lexicon_tags_11 },
    { 0xef30a39e, SUC_lexicon_tags_0 },
    { 0x2f90a39e, SUC_lexicon_tags_26 },
    { 0x8390a3a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5010a3a2, SUC_lexicon_tags_14 },
    { 0x6e50a3a3, SUC_lexicon_tags_7 },
    { 0x7f50a3a2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d0a3ab, SUC_lexicon_tags_2 },
    { 0x29f0a3ac, SUC_lexicon_tags_11 },
    { 0x6610a3ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f0a3b0, SUC_lexicon_tags_26 },
    { 0x9910a3b1, SUC_lexicon_tags_17 },
    { 0xf410a3b1, SUC_lexicon_tags_8 },
    { 0xc230a3b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb0a3b8, SUC_lexicon_tags_23 },
    { 0x3270a3b9, SUC_lexicon_tags_2 },
    { 0x8f70a3ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f0a3c2, SUC_lexicon_tags_11 },
    { 0x56d0a3c3, SUC_lexicon_tags_38 },
    { 0x43f0a3c4, SUC_lexicon_tags_11 },
    { 0xdfd0a3c5, SUC_lexicon_tags_11 },
    { 0x2950a3c6, SUC_lexicon_tags_33 },
    { 0x4b30a3c7, SUC_lexicon_tags_14 },
    { 0xa410a3c6, SUC_lexicon_tags_1 },
    { 0x2ab0a3c7, SUC_lexicon_tags_47 },
    { 0xec70a3c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc730a3cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db0a3cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab0a3d2, SUC_lexicon_tags_103 },
    { 0x9f0a3d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca50a3d9, SUC_lexicon_tags_38 },
    { 0xfb70a3d9, SUC_lexicon_tags_2 },
    { 0x4bd0a3db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b0a3de, SUC_lexicon_tags_19 },
    { 0x69d0a3df, SUC_lexicon_tags_17 },
    { 0x6970a3e0, SUC_lexicon_tags_2 },
    { 0xb230a3df, SUC_lexicon_tags_15 },
    { 0x6210a3e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f70a3e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7190a3eb, SUC_lexicon_tags_2 },
    { 0x2f30a3ec, SUC_lexicon_tags_37 },
    { 0x53f0a3ec, SUC_lexicon_tags_38 },
    { 0x9c30a3eb, SUC_lexicon_tags_4 },
    { 0x7950a3ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd610a3f2, SUC_lexicon_tags_46 },
    { 0xbcb0a3f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb0a3f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cd0a3f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3bf0a3fa, SUC_lexicon_tags_11 },
    { 0x7170a3fa, SUC_lexicon_tags_48 },
    { 0xdd90a3fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a90a3fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d0a401, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4710a403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea30a40c, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x80d0a40e, SUC_lexicon_tags_17 },
    { 0xc7f0a40e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf30a411, SUC_lexicon_tags_9 },
    { 0x2d50a412, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f0a41b, SUC_lexicon_tags_12 },
    { 0x7190a41c, SUC_lexicon_tags_17 },
    { 0xae70a41d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d50a425, SUC_lexicon_tags_66 },
    { 0x3690a425, SUC_lexicon_tags_25 },
    { 0x9870a427, SUC_lexicon_tags_19 },
    { 0x9050a428, SUC_lexicon_tags_2 },
    { 0xc990a428, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b0a42e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe530a431, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce30a433, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x42b0a435, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8470a439, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe970a43b, SUC_lexicon_tags_26 },
    { 0x7450a43c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d0a43f, SUC_lexicon_tags_7 },
    { 0xbd0a43f, SUC_lexicon_tags_37 },
    { 0xa910a440, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73d0a443, SUC_lexicon_tags_1 },
    { 0xd9b0a444, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc770a44b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a30a44e, SUC_lexicon_tags_45 },
    { 0xd870a44f, SUC_lexicon_tags_3 },
    { 0x4df0a44e, SUC_lexicon_tags_88 },
    { 0x30d0a44f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7930a455, SUC_lexicon_tags_7 },
    { 0x3db0a456, SUC_lexicon_tags_14 },
    { 0x9bf0a457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe330a45a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb890a45d, SUC_lexicon_tags_19 },
    { 0xce10a45e, SUC_lexicon_tags_15 },
    { 0xb250a45d, SUC_lexicon_tags_25 },
    { 0x5dd0a460, SUC_lexicon_tags_17 },
    { 0x4310a461, SUC_lexicon_tags_63 },
    { 0xfe70a462, SUC_lexicon_tags_45 },
    { 0xfdf0a461, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1970a465, SUC_lexicon_tags_37 },
    { 0x2430a465, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe2d0a468, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa270a46f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50a473, SUC_lexicon_tags_27 },
    { 0x65d0a474, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4f0a476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6330a482, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b0a486, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf030a48c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f0a495, SUC_lexicon_tags_27 },
    { 0x8a90a496, SUC_lexicon_tags_37 },
    { 0x9750a497, SUC_lexicon_tags_11 },
    { 0xe10a498, SUC_lexicon_tags_11 },
    { 0x6b90a499, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7230a49d, SUC_lexicon_tags_15 },
    { 0xdf70a49e, SUC_lexicon_tags_1 },
    { 0x3e70a49f, SUC_lexicon_tags_3 },
    { 0x6cf0a49f, SUC_lexicon_tags_8 },
    { 0xb510a4a1, SUC_lexicon_tags_2 },
    { 0xb110a4a1, SUC_lexicon_tags_59 },
    { 0xc2f0a4a3, SUC_lexicon_tags_2 },
    { 0xb50a4a4, SUC_lexicon_tags_15 },
    { 0xed70a49e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad0a4a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d0a4ab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa870a4ae, SUC_lexicon_tags_7 },
    { 0xf0f0a4af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd50a4b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc910a4b9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xeb0a4bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2490a4bd, SUC_lexicon_tags_28 },
    { 0xbd30a4be, SUC_lexicon_tags_25 },
    { 0xf650a4be, SUC_lexicon_tags_3 },
    { 0x4e30a4c0, SUC_lexicon_tags_7 },
    { 0x3d70a4c0, SUC_lexicon_tags_91 },
    { 0x89b0a4c0, SUC_lexicon_tags_17 },
    { 0x62f0a4be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1230a4c5, SUC_lexicon_tags_60 },
    { 0xeaf0a4c6, SUC_lexicon_tags_37 },
    { 0x5110a4c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x53f0a4c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe290a4cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf690a4d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7050a4d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8490a4d8, SUC_lexicon_tags_16 },
    { 0xbab0a4d9, SUC_lexicon_tags_1 },
    { 0x7750a4d8, SUC_lexicon_tags_49 },
    { 0x7230a4d8, SUC_lexicon_tags_1 },
    { 0xe110a4db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2890a4e1, SUC_lexicon_tags_12 },
    { 0x43d0a4e1, SUC_lexicon_tags_49 },
    { 0x5f10a4e3, SUC_lexicon_tags_27 },
    { 0xc430a4e4, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2370a4eb, SUC_lexicon_tags_12 },
    { 0x530a4ec, SUC_lexicon_tags_38 },
    { 0x1650a4eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b0a4f1, SUC_lexicon_tags_68 },
    { 0x2b0a4f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf770a4f4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a70a4fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d0a500, SUC_lexicon_tags_2 },
    { 0x7e10a501, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfd70a503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8850a505, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad0a50a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9890a50c, SUC_lexicon_tags_10 },
    { 0x2090a50d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55f0a50f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc750a517, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5890a51b, SUC_lexicon_tags_17 },
    { 0xdb50a51b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc870a520, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7b0a522, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc270a524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc70a52e, SUC_lexicon_tags_1 },
    { 0xe950a52f, SUC_lexicon_tags_14 },
    { 0xbf90a52e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f0a535, SUC_lexicon_tags_27 },
    { 0x53f0a535, SUC_lexicon_tags_12 },
    { 0x4110a537, SUC_lexicon_tags_23 },
    { 0x1d0a537, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f0a53e, SUC_lexicon_tags_47 },
    { 0x4b70a53f, SUC_lexicon_tags_12 },
    { 0xdd70a540, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d0a546, SUC_lexicon_tags_17 },
    { 0x8490a546, SUC_lexicon_tags_45 },
    { 0x330a548, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb0a54d, SUC_lexicon_tags_17 },
    { 0xf90a54e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f0a552, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10a556, SUC_lexicon_tags_45 },
    { 0x4fb0a557, SUC_lexicon_tags_66 },
    { 0x9230a558, SUC_lexicon_tags_17 },
    { 0x7090a556, SUC_lexicon_tags_37 },
    { 0x6d0a55a, SUC_lexicon_tags_0 },
    { 0x3870a558, SUC_lexicon_tags_23 },
    { 0x6db0a556, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0a565, SUC_lexicon_tags_141 },
    { 0x5410a566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae50a568, SUC_lexicon_tags_11 },
    { 0x1610a569, SUC_lexicon_tags_21 },
    { 0xff70a56a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f0a56e, SUC_lexicon_tags_17 },
    { 0x8e30a56f, SUC_lexicon_tags_23 },
    { 0xb070a570, SUC_lexicon_tags_19 },
    { 0x5b90a571, SUC_lexicon_tags_43 },
    { 0x6a50a572, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db0a577, SUC_lexicon_tags_1 },
    { 0x6fd0a578, SUC_lexicon_tags_25 },
    { 0x1af0a579, SUC_lexicon_tags_8 },
    { 0x4730a577, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d0a57d, SUC_lexicon_tags_87 },
    { 0x85f0a57d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e90a582, SUC_lexicon_tags_37 },
    { 0xdb70a582, SUC_lexicon_tags_23 },
    { 0x750a584, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf1b0a586, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4650a592, SUC_lexicon_tags_25 },
    { 0x48d0a592, SUC_lexicon_tags_23 },
    { 0x9a50a594, SUC_lexicon_tags_45 },
    { 0xdeb0a595, SUC_lexicon_tags_7 },
    { 0x8b0a596, SUC_lexicon_tags_17 },
    { 0x25f0a596, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4890a59a, SUC_lexicon_tags_47 },
    { 0x7550a59b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6710a59e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1050a5a1, SUC_lexicon_tags_11 },
    { 0xa930a5a1, SUC_lexicon_tags_21 },
    { 0x3090a5a3, SUC_lexicon_tags_62 },
    { 0x15b0a5a4, SUC_lexicon_tags_54 },
    { 0xb2b0a5a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a90a5a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef50a5a9, SUC_lexicon_tags_1 },
    { 0x4690a5aa, SUC_lexicon_tags_25 },
    { 0xd650a5ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d70a5b0, SUC_lexicon_tags_12 },
    { 0x6e70a5b0, SUC_lexicon_tags_16 },
    { 0xa5b0a5b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf610a5b4, SUC_lexicon_tags_19 },
    { 0x1a10a5b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d90a5b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1970a5bb, SUC_lexicon_tags_138 },
    { 0x7f30a5bb, SUC_lexicon_tags_17 },
    { 0x3770a5bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa470a5bf, SUC_lexicon_tags_43 },
    { 0x70f0a5bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2f0a5c2, SUC_lexicon_tags_15 },
    { 0xdf30a5c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3690a5c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1eb0a5cb, SUC_lexicon_tags_1 },
    { 0x6850a5cb, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0a5d1, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0a5d5, SUC_lexicon_tags_25 },
    { 0x10a5d6, SUC_lexicon_tags_15 },
    { 0x49b0a5d7, SUC_lexicon_tags_4 },
    { 0x60d0a5d5, SUC_lexicon_tags_31 },
    { 0xb450a5d9, SUC_lexicon_tags_7 },
    { 0x36f0a5d7, SUC_lexicon_tags_37 },
    { 0x5050a5db, SUC_lexicon_tags_19 },
    { 0xbd70a5db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2190a5e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3470a5e8, SUC_lexicon_tags_31 },
    { 0xeff0a5e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4410a5ec, SUC_lexicon_tags_282 },
    { 0xcd30a5ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf050a5f1, SUC_lexicon_tags_32 },
    { 0x6310a5f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b0a5f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef0a5f9, SUC_lexicon_tags_37 },
    { 0x5ed0a5fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd0a5fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c10a600, SUC_lexicon_tags_7 },
    { 0xa8f0a601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc550a60b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f10a60e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf310a613, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0a61a, SUC_lexicon_tags_11 },
    { 0xeed0a61b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a10a61e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd10a623, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe990a627, SUC_lexicon_tags_39 },
    { 0xd550a627, SUC_lexicon_tags_37 },
    { 0x55d0a629, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5590a62c, SUC_lexicon_tags_1 },
    { 0xab10a62d, SUC_lexicon_tags_17 },
    { 0x490a62e, SUC_lexicon_tags_11 },
    { 0x7410a62d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x550a631, SUC_lexicon_tags_45 },
    { 0xa830a631, SUC_lexicon_tags_29 },
    { 0x9e30a631, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf010a636, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a10a63a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71b0a63c, SUC_lexicon_tags_37 },
    { 0x8d90a63d, SUC_lexicon_tags_61 },
    { 0xc590a63e, SUC_lexicon_tags_17 },
    { 0xc110a63e, SUC_lexicon_tags_3 },
    { 0xa070a63f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x22d0a642, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2e50a644, SUC_lexicon_tags_12 },
    { 0x47b0a645, SUC_lexicon_tags_18 },
    { 0x8850a646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd0a64b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9390a64e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10d0a650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b10a654, SUC_lexicon_tags_23 },
    { 0xe470a654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57d0a657, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63b0a659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8230a65b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d0a65e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530a663, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa650a66a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d0a66e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd130a671, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0a678, SUC_lexicon_tags_47 },
    { 0xcdf0a679, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe050a67f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7170a686, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec90a689, SUC_lexicon_tags_15 },
    { 0xc590a68a, SUC_lexicon_tags_36 },
    { 0x45b0a68b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a50a692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd730a695, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5590a69a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x30b0a69c, SUC_lexicon_tags_33 },
    { 0x45f0a69c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8050a69f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa90a6a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe690a6a3, SUC_lexicon_tags_19 },
    { 0x6f50a6a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3890a6a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd0a6b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef0a6b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5d0a6b6, SUC_lexicon_tags_47 },
    { 0x2390a6b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f30a6ba, SUC_lexicon_tags_38 },
    { 0x8f30a6bb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f0a6c4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4690a6c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed0a6ca, SUC_lexicon_tags_17 },
    { 0x3b70a6cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb550a6ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xee90a6d0, SUC_lexicon_tags_2 },
    { 0x8b70a6d1, SUC_lexicon_tags_7 },
    { 0x8350a6d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff70a6d9, SUC_lexicon_tags_31 },
    { 0x7c50a6da, SUC_lexicon_tags_17 },
    { 0xf1b0a6db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f0a6e0, SUC_lexicon_tags_64 },
    { 0xdc90a6e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9270a6e3, SUC_lexicon_tags_0 },
    { 0xb630a6e4, SUC_lexicon_tags_29 },
    { 0xdfb0a6e4, SUC_lexicon_tags_7 },
    { 0x5270a6e6, SUC_lexicon_tags_15 },
    { 0x85d0a6e7, SUC_lexicon_tags_27 },
    { 0x5ef0a6e8, SUC_lexicon_tags_2 },
    { 0xe930a6e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa830a6ec, SUC_lexicon_tags_11 },
    { 0x9150a6ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa470a6f0, SUC_lexicon_tags_26 },
    { 0xc650a6f1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb670a6f3, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x3890a6f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x75b0a6f7, SUC_lexicon_tags_3 },
    { 0x1890a6f8, SUC_lexicon_tags_37 },
    { 0xe130a6f9, SUC_lexicon_tags_26 },
    { 0xe3d0a6fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8530a702, SUC_lexicon_tags_140 },
    { 0xdd0a702, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3510a706, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5910a70b, SUC_lexicon_tags_17 },
    { 0x44d0a70c, SUC_lexicon_tags_17 },
    { 0x2270a70d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53d0a70f, SUC_lexicon_tags_15 },
    { 0x71d0a710, SUC_lexicon_tags_7 },
    { 0x50d0a711, SUC_lexicon_tags_42 },
    { 0x3a70a712, SUC_lexicon_tags_37 },
    { 0xb9d0a712, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe250a715, SUC_lexicon_tags_11 },
    { 0x6230a716, SUC_lexicon_tags_11 },
    { 0x6350a715, SUC_lexicon_tags_37 },
    { 0x7010a716, SUC_lexicon_tags_0 },
    { 0x2190a719, SUC_lexicon_tags_12 },
    { 0xd550a715, SUC_lexicon_tags_32 },
    { 0x81d0a718, SUC_lexicon_tags_9 },
    { 0xa930a71c, SUC_lexicon_tags_359 },
    { 0x8c10a71d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d30a723, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9050a725, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd0a72c, SUC_lexicon_tags_35 },
    { 0x670a72c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cf0a72f, SUC_lexicon_tags_11 },
    { 0xd870a72f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x450a733, SUC_lexicon_tags_37 },
    { 0x2b30a734, SUC_lexicon_tags_17 },
    { 0x4ed0a735, SUC_lexicon_tags_9 },
    { 0xdfb0a736, SUC_lexicon_tags_30 },
    { 0xc8f0a734, SUC_lexicon_tags_25 },
    { 0xbf70a738, SUC_lexicon_tags_11 },
    { 0x4730a738, SUC_lexicon_tags_27 },
    { 0x78d0a73a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4770a73c, SUC_lexicon_tags_33 },
    { 0x5010a73c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f0a740, SUC_lexicon_tags_14 },
    { 0xa8d0a741, SUC_lexicon_tags_17 },
    { 0x2890a742, SUC_lexicon_tags_17 },
    { 0xd350a743, SUC_lexicon_tags_8 },
    { 0xc470a740, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x10d0a746, SUC_lexicon_tags_11 },
    { 0x9cf0a747, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9df0a749, SUC_lexicon_tags_60 },
    { 0xc670a74a, SUC_lexicon_tags_7 },
    { 0xd8d0a74b, SUC_lexicon_tags_11 },
    { 0xa670a74c, SUC_lexicon_tags_37 },
    { 0xdf90a74d, SUC_lexicon_tags_1 },
    { 0x1bf0a74a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4430a753, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8850a757, SUC_lexicon_tags_1 },
    { 0x50b0a758, SUC_lexicon_tags_28 },
    { 0x750a759, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaab0a75b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af0a75e, SUC_lexicon_tags_55 },
    { 0x70b0a75f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbfb0a761, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd0a766, SUC_lexicon_tags_43 },
    { 0x25d0a767, SUC_lexicon_tags_26 },
    { 0x86d0a768, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31b0a76a, SUC_lexicon_tags_11 },
    { 0xf9f0a76b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31f0a76d, SUC_lexicon_tags_11 },
    { 0xa110a76d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2470a770, SUC_lexicon_tags_28 },
    { 0x2d10a771, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df0a774, SUC_lexicon_tags_1 },
    { 0x2e30a775, SUC_lexicon_tags_1 },
    { 0x9cb0a775, SUC_lexicon_tags_583 },
    { 0x4bd0a777, SUC_lexicon_tags_79 },
    { 0x32f0a776, SUC_lexicon_tags_14 },
    { 0x5a50a776, SUC_lexicon_tags_37 },
    { 0x92b0a77a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd070a77d, SUC_lexicon_tags_68 },
    { 0xe610a77e, SUC_lexicon_tags_25 },
    { 0xd090a77f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7450a784, SUC_lexicon_tags_5 },
    { 0xfa30a784, SUC_lexicon_tags_87 },
    { 0xea10a786, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x8610a789, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb0a78d, SUC_lexicon_tags_25 },
    { 0xe590a78d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd0a791, SUC_lexicon_tags_12 },
    { 0x4d30a792, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb30a797, SUC_lexicon_tags_17 },
    { 0x930a797, SUC_lexicon_tags_23 },
    { 0x6970a799, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f30a79d, SUC_lexicon_tags_25 },
    { 0x7630a79e, SUC_lexicon_tags_14 },
    { 0x61f0a79f, SUC_lexicon_tags_17 },
    { 0x23b0a7a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa250a7a3, SUC_lexicon_tags_8 },
    { 0xa010a7a4, SUC_lexicon_tags_0 },
    { 0xda50a7a5, SUC_lexicon_tags_16 },
    { 0x7870a7a6, SUC_lexicon_tags_17 },
    { 0x56d0a7a7, SUC_lexicon_tags_4 },
    { 0x1d70a7a7, SUC_lexicon_tags_6 },
    { 0xd130a7a5, SUC_lexicon_tags_37 },
    { 0x2fd0a7aa, SUC_lexicon_tags_9 },
    { 0x6b30a7ab, SUC_lexicon_tags_15 },
    { 0x5f50a7ac, SUC_lexicon_tags_16 },
    { 0x2430a7ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd450a7af, SUC_lexicon_tags_17 },
    { 0x4110a7b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab0a7b5, SUC_lexicon_tags_87 },
    { 0xc130a7b6, SUC_lexicon_tags_15 },
    { 0x2c30a7b7, SUC_lexicon_tags_26 },
    { 0xeb90a7b5, SUC_lexicon_tags_60 },
    { 0xfa90a7b9, SUC_lexicon_tags_35 },
    { 0x1b30a7ba, SUC_lexicon_tags_2 },
    { 0xcfd0a7bb, SUC_lexicon_tags_37 },
    { 0x96d0a7bc, SUC_lexicon_tags_4 },
    { 0x3d90a7bc, SUC_lexicon_tags_77 },
    { 0x1a90a7be, SUC_lexicon_tags_2 },
    { 0xbe10a7bc, SUC_lexicon_tags_6 },
    { 0xa6b0a7bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d0a7c8, SUC_lexicon_tags_25 },
    { 0x48f0a7c9, SUC_lexicon_tags_15 },
    { 0xc150a7c9, SUC_lexicon_tags_12 },
    { 0x7b0a7cb, SUC_lexicon_tags_11 },
    { 0x18b0a7c9, SUC_lexicon_tags_12 },
    { 0x7690a7cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3790a7cf, SUC_lexicon_tags_17 },
    { 0x2310a7cf, SUC_lexicon_tags_25 },
    { 0x70b0a7d1, SUC_lexicon_tags_11 },
    { 0xff50a7d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1850a7d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb0a7d8, SUC_lexicon_tags_11 },
    { 0x2630a7d9, SUC_lexicon_tags_7 },
    { 0xfd70a7da, SUC_lexicon_tags_1 },
    { 0xd9b0a7da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8070a7e1, SUC_lexicon_tags_25 },
    { 0xb7f0a7e2, SUC_lexicon_tags_7 },
    { 0xd050a7e3, SUC_lexicon_tags_30 },
    { 0x9050a7e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18b0a7e6, SUC_lexicon_tags_1 },
    { 0xdb10a7e7, SUC_lexicon_tags_10 },
    { 0x4ff0a7e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92d0a7ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7870a7ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1a30a7ee, SUC_lexicon_tags_3 },
    { 0x47f0a7ee, SUC_lexicon_tags_46 },
    { 0x93f0a7f0, SUC_lexicon_tags_29 },
    { 0xe210a7f1, SUC_lexicon_tags_38 },
    { 0xeb50a7f2, SUC_lexicon_tags_14 },
    { 0xe250a7f3, SUC_lexicon_tags_49 },
    { 0x5e10a7f3, SUC_lexicon_tags_25 },
    { 0x1db0a7f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9290a7f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x190a7fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa950a7fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f0a7ff, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0xe950a801, SUC_lexicon_tags_77 },
    { 0xeff0a802, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f0a805, SUC_lexicon_tags_2 },
    { 0xea30a806, SUC_lexicon_tags_25 },
    { 0x9170a806, SUC_lexicon_tags_1 },
    { 0x9030a808, SUC_lexicon_tags_9 },
    { 0xd2d0a809, SUC_lexicon_tags_38 },
    { 0xe050a80a, SUC_lexicon_tags_98 },
    { 0x9e90a807, SUC_lexicon_tags_198 },
    { 0xed90a807, SUC_lexicon_tags_16 },
    { 0x2b70a80d, SUC_lexicon_tags_14 },
    { 0xdf50a80e, SUC_lexicon_tags_17 },
    { 0x3230a80f, SUC_lexicon_tags_12 },
    { 0x7c50a80f, SUC_lexicon_tags_43 },
    { 0x17d0a810, SUC_lexicon_tags_9 },
    { 0x2630a811, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad0a816, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db0a81a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5670a821, SUC_lexicon_tags_60 },
    { 0x6110a821, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe290a82d, SUC_lexicon_tags_11 },
    { 0x9850a82e, SUC_lexicon_tags_35 },
    { 0xd70a82e, SUC_lexicon_tags_65 },
    { 0x3230a82e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b30a833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa70a836, SUC_lexicon_tags_17 },
    { 0xda70a836, SUC_lexicon_tags_12 },
    { 0x81d0a838, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7af0a83a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3050a83c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a30a83e, SUC_lexicon_tags_79 },
    { 0x97f0a83f, SUC_lexicon_tags_33 },
    { 0xb030a840, SUC_lexicon_tags_2 },
    { 0xac30a841, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xc070a844, SUC_lexicon_tags_25 },
    { 0x9eb0a845, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d0a848, SUC_lexicon_tags_37 },
    { 0xc790a848, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xadd0a84b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8370a84d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe690a850, SUC_lexicon_tags_38 },
    { 0x55f0a851, SUC_lexicon_tags_75 },
    { 0x3150a850, SUC_lexicon_tags_1 },
    { 0xa630a853, SUC_lexicon_tags_37 },
    { 0xee10a853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b0a859, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f0a85d, SUC_lexicon_tags_29 },
    { 0xdb0a85e, SUC_lexicon_tags_17 },
    { 0xf290a85f, SUC_lexicon_tags_25 },
    { 0xcf70a85e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x79f0a862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42f0a864, SUC_lexicon_tags_36 },
    { 0x29f0a865, SUC_lexicon_tags_11 },
    { 0x3bb0a866, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb0a86b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xba90a86e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab0a870, SUC_lexicon_tags_0 },
    { 0xb590a870, SUC_lexicon_tags_27 },
    { 0x6730a872, SUC_lexicon_tags_6 },
    { 0x9410a872, SUC_lexicon_tags_15 },
    { 0xc190a873, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8950a877, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c70a87c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8850a87e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f70a883, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f50a889, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1750a88e, SUC_lexicon_tags_16 },
    { 0x510a88f, SUC_lexicon_tags_25 },
    { 0x56f0a890, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9850a893, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb350a898, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1ff0a89a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfa70a89c, SUC_lexicon_tags_7 },
    { 0x38b0a89c, SUC_lexicon_tags_25 },
    { 0x8a90a89d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde50a8a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd30a8a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1410a8a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad0a8af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d30a8b3, SUC_lexicon_tags_12 },
    { 0x6730a8b3, SUC_lexicon_tags_11 },
    { 0x1bb0a8b5, SUC_lexicon_tags_16 },
    { 0x3b30a8b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a50a8c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa050a8c9, SUC_lexicon_tags_27 },
    { 0xb70a8ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf0a8cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9b0a8cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e70a8d2, SUC_lexicon_tags_37 },
    { 0x290a8d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc350a8d8, SUC_lexicon_tags_21 },
    { 0xa50a8d9, SUC_lexicon_tags_15 },
    { 0x4b10a8da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d0a8de, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd450a8e0, SUC_lexicon_tags_33 },
    { 0x4ff0a8e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4190a8e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1410a8e5, SUC_lexicon_tags_58 },
    { 0xf850a8e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc90a8e9, SUC_lexicon_tags_40 },
    { 0x7850a8ea, SUC_lexicon_tags_7 },
    { 0xc970a8eb, SUC_lexicon_tags_49 },
    { 0xe6b0a8ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa710a8ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9470a8f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf490a8f3, SUC_lexicon_tags_26 },
    { 0x8430a8f3, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xe710a8f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4030a8fc, SUC_lexicon_tags_11 },
    { 0x4710a8fd, SUC_lexicon_tags_12 },
    { 0x55b0a8fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf970a900, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd0a908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d0a90b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6f0a90d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a70a910, SUC_lexicon_tags_1 },
    { 0xe410a910, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3a10a913, SUC_lexicon_tags_31 },
    { 0xd8f0a914, SUC_lexicon_tags_11 },
    { 0x1510a915, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb550a919, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5910a91f, SUC_lexicon_tags_17 },
    { 0x6d10a91f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xeff0a922, SUC_lexicon_tags_15 },
    { 0xbd70a923, SUC_lexicon_tags_1 },
    { 0x5750a923, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6530a926, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd470a930, SUC_lexicon_tags_7 },
    { 0x68b0a931, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae10a935, SUC_lexicon_tags_11 },
    { 0xcd30a936, SUC_lexicon_tags_28 },
    { 0x1bd0a937, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xad0a93a, SUC_lexicon_tags_37 },
    { 0x5b10a93a, SUC_lexicon_tags_37 },
    { 0x6990a93b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd0a940, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc6d0a942, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe7d0a944, SUC_lexicon_tags_38 },
    { 0xf4d0a945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4b0a947, SUC_lexicon_tags_3 },
    { 0x37d0a948, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7950a94c, SUC_lexicon_tags_17 },
    { 0x25b0a94d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa7b0a94f, SUC_lexicon_tags_37 },
    { 0x4030a950, SUC_lexicon_tags_33 },
    { 0x3ab0a950, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb0a954, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19b0a956, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f0a95c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa590a95e, SUC_lexicon_tags_37 },
    { 0xe010a95f, SUC_lexicon_tags_33 },
    { 0x590a95f, SUC_lexicon_tags_32 },
    { 0x2b0a961, SUC_lexicon_tags_47 },
    { 0xee10a960, SUC_lexicon_tags_71 },
    { 0x8a70a963, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe850a96e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e10a975, SUC_lexicon_tags_28 },
    { 0x6210a975, SUC_lexicon_tags_45 },
    { 0x83f0a977, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa650a97d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf0a982, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b0a985, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf570a98a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeef0a98c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10a991, SUC_lexicon_tags_1 },
    { 0x3450a991, SUC_lexicon_tags_19 },
    { 0xb390a993, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5870a995, SUC_lexicon_tags_14 },
    { 0x1a50a996, SUC_lexicon_tags_26 },
    { 0x9790a996, SUC_lexicon_tags_7 },
    { 0xa110a998, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef30a99c, SUC_lexicon_tags_37 },
    { 0x7df0a99c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8430a99f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd0a9a2, SUC_lexicon_tags_31 },
    { 0x8230a9a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa130a9a8, SUC_lexicon_tags_65 },
    { 0x7610a9a9, SUC_lexicon_tags_2 },
    { 0xd590a9aa, SUC_lexicon_tags_11 },
    { 0xbf0a9ab, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d70a9ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f90a9b1, SUC_lexicon_tags_37 },
    { 0xb0d0a9b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf0a9b7, SUC_lexicon_tags_114 },
    { 0xfc10a9b8, SUC_lexicon_tags_0 },
    { 0xa30a9b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60d0a9bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf370a9bd, SUC_lexicon_tags_25 },
    { 0x1bd0a9be, SUC_lexicon_tags_14 },
    { 0x4d30a9be, SUC_lexicon_tags_12 },
    { 0x2010a9be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6310a9c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f0a9c7, SUC_lexicon_tags_25 },
    { 0x650a9c8, SUC_lexicon_tags_2 },
    { 0x3d10a9c9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2610a9cb, SUC_lexicon_tags_83 },
    { 0x7b50a9cb, SUC_lexicon_tags_17 },
    { 0xaa30a9cd, SUC_lexicon_tags_49 },
    { 0x5530a9ce, SUC_lexicon_tags_12 },
    { 0x7cd0a9cf, SUC_lexicon_tags_25 },
    { 0xfbd0a9d0, SUC_lexicon_tags_39 },
    { 0x6ab0a9d1, SUC_lexicon_tags_7 },
    { 0xdb90a9d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e10a9d7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c70a9da, SUC_lexicon_tags_33 },
    { 0xe890a9db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff0a9df, SUC_lexicon_tags_25 },
    { 0x4f10a9e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d30a9e7, SUC_lexicon_tags_0 },
    { 0x6f0a9e7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8570a9ed, SUC_lexicon_tags_12 },
    { 0xcff0a9ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f70a9f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d0a9f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b0aa02, SUC_lexicon_tags_37 },
    { 0x1550aa02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6510aa05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f90aa0c, SUC_lexicon_tags_2 },
    { 0xe4f0aa0d, SUC_lexicon_tags_30 },
    { 0xf210aa0e, SUC_lexicon_tags_47 },
    { 0xa990aa0e, SUC_lexicon_tags_60 },
    { 0x1090aa0f, SUC_lexicon_tags_26 },
    { 0xb5d0aa11, SUC_lexicon_tags_23 },
    { 0xf0d0aa11, SUC_lexicon_tags_37 },
    { 0x9cb0aa13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df0aa17, SUC_lexicon_tags_12 },
    { 0x9fb0aa18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a50aa21, SUC_lexicon_tags_17 },
    { 0xe530aa22, SUC_lexicon_tags_2 },
    { 0x5250aa21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4f0aa25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4790aa29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4510aa2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b0aa30, SUC_lexicon_tags_4 },
    { 0x23d0aa31, SUC_lexicon_tags_37 },
    { 0x3750aa32, SUC_lexicon_tags_2 },
    { 0x3910aa30, SUC_lexicon_tags_37 },
    { 0xa930aa31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c10aa3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a10aa40, SUC_lexicon_tags_17 },
    { 0x1090aa40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5290aa44, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa090aa47, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb7f0aa49, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c10aa51, SUC_lexicon_tags_11 },
    { 0x6c90aa52, SUC_lexicon_tags_16 },
    { 0x7cb0aa52, SUC_lexicon_tags_15 },
    { 0xa1d0aa54, SUC_lexicon_tags_27 },
    { 0x6c30aa55, SUC_lexicon_tags_25 },
    { 0xb370aa56, SUC_lexicon_tags_37 },
    { 0x2550aa57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d0aa5b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf0aa5e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d0aa61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4150aa63, SUC_lexicon_tags_11 },
    { 0xe2f0aa64, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1d0aa66, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1970aa69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7eb0aa6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1930aa6f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2930aa73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb270aa75, SUC_lexicon_tags_2 },
    { 0xc1f0aa76, SUC_lexicon_tags_2 },
    { 0x5130aa75, SUC_lexicon_tags_12 },
    { 0xb570aa77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc510aa7d, SUC_lexicon_tags_1 },
    { 0x6f0aa7e, SUC_lexicon_tags_2 },
    { 0x8810aa7e, SUC_lexicon_tags_14 },
    { 0x8e90aa80, SUC_lexicon_tags_2 },
    { 0x3b90aa80, SUC_lexicon_tags_17 },
    { 0x88b0aa82, SUC_lexicon_tags_53 },
    { 0xd370aa7d, SUC_lexicon_tags_49 },
    { 0xd050aa83, SUC_lexicon_tags_12 },
    { 0x91b0aa85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1610aa87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef0aa8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc150aa8c, SUC_lexicon_tags_37 },
    { 0x2a70aa8c, SUC_lexicon_tags_12 },
    { 0x6ad0aa8e, SUC_lexicon_tags_14 },
    { 0x9750aa8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc770aa91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd70aa96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed30aa9c, SUC_lexicon_tags_13 },
    { 0x1ed0aa9d, SUC_lexicon_tags_0 },
    { 0x8e10aa9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb70aaa1, SUC_lexicon_tags_17 },
    { 0x36b0aaa2, SUC_lexicon_tags_25 },
    { 0x84b0aaa2, SUC_lexicon_tags_37 },
    { 0x3d90aaa1, SUC_lexicon_tags_74 },
    { 0xc310aaa1, SUC_lexicon_tags_13 },
    { 0xb250aaa2, SUC_lexicon_tags_63 },
    { 0xdcf0aaa7, SUC_lexicon_tags_37 },
    { 0x5d90aaa3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af0aaae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d0aab5, SUC_lexicon_tags_17 },
    { 0xbf30aab6, SUC_lexicon_tags_39 },
    { 0xf2d0aab6, SUC_lexicon_tags_37 },
    { 0xa250aab8, SUC_lexicon_tags_25 },
    { 0xeed0aab6, SUC_lexicon_tags_37 },
    { 0xd610aab7, SUC_lexicon_tags_47 },
    { 0x550aab7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b0aac0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2950aac5, SUC_lexicon_tags_11 },
    { 0xe050aac6, SUC_lexicon_tags_2 },
    { 0x4f90aac7, SUC_lexicon_tags_37 },
    { 0xf930aac8, SUC_lexicon_tags_10 },
    { 0x5e90aac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xad50aacc, SUC_lexicon_tags_15 },
    { 0x2130aacd, SUC_lexicon_tags_37 },
    { 0xf050aace, SUC_lexicon_tags_11 },
    { 0x7b10aacd, SUC_lexicon_tags_26 },
    { 0x90d0aacf, SUC_lexicon_tags_15 },
    { 0x95d0aad1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd970aad6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d0aadd, SUC_lexicon_tags_12 },
    { 0x9e70aade, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f0aae2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f0aae5, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb0aae9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db0aaec, SUC_lexicon_tags_68 },
    { 0xc450aaed, SUC_lexicon_tags_27 },
    { 0xd230aaed, SUC_lexicon_tags_29 },
    { 0xe7b0aaee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f0aaf8, SUC_lexicon_tags_12 },
    { 0xe30aaf9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x54f0aafb, SUC_lexicon_tags_46 },
    { 0x7db0aafc, SUC_lexicon_tags_34 },
    { 0x8590aafd, SUC_lexicon_tags_44 },
    { 0xc4d0aafe, SUC_lexicon_tags_33 },
    { 0xbff0aafe, SUC_lexicon_tags_2 },
    { 0x36b0aafe, SUC_lexicon_tags_2 },
    { 0x9ff0aaff, SUC_lexicon_tags_30 },
    { 0xb310aafd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0d0ab04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6470ab0b, SUC_lexicon_tags_7 },
    { 0x1390ab0c, SUC_lexicon_tags_27 },
    { 0xc790ab0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa130ab0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad0ab12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad10ab16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d0ab1b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f0ab1f, SUC_lexicon_tags_758 },
    { 0, NULL },
    { 0x1830ab21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe670ab27, SUC_lexicon_tags_37 },
    { 0xca50ab28, SUC_lexicon_tags_17 },
    { 0x9f10ab28, SUC_lexicon_tags_37 },
    { 0x2910ab27, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d0ab2d, SUC_lexicon_tags_17 },
    { 0x3df0ab2e, SUC_lexicon_tags_11 },
    { 0x2e10ab2f, SUC_lexicon_tags_23 },
    { 0x770ab2e, SUC_lexicon_tags_2 },
    { 0x4a70ab2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e70ab3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d0ab40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fb0ab42, SUC_lexicon_tags_0 },
    { 0xe2b0ab42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7070ab45, SUC_lexicon_tags_12 },
    { 0xa6b0ab46, SUC_lexicon_tags_28 },
    { 0x7750ab47, SUC_lexicon_tags_15 },
    { 0x3b50ab46, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xddd0ab4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb790ab4e, SUC_lexicon_tags_10 },
    { 0x9f10ab4f, SUC_lexicon_tags_11 },
    { 0xe990ab50, SUC_lexicon_tags_17 },
    { 0xe9f0ab50, SUC_lexicon_tags_201 },
    { 0x1750ab50, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f0ab58, SUC_lexicon_tags_0 },
    { 0x970ab58, SUC_lexicon_tags_27 },
    { 0xfd0ab58, SUC_lexicon_tags_25 },
    { 0xecf0ab58, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa790ab65, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb0ab68, SUC_lexicon_tags_18 },
    { 0xba10ab69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f0ab6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa410ab70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa030ab72, SUC_lexicon_tags_1 },
    { 0x8490ab73, SUC_lexicon_tags_32 },
    { 0xaff0ab74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd750ab7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5770ab7e, SUC_lexicon_tags_17 },
    { 0x3350ab7f, SUC_lexicon_tags_30 },
    { 0xf670ab7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef0ab87, SUC_lexicon_tags_25 },
    { 0x8170ab88, SUC_lexicon_tags_29 },
    { 0xd890ab88, SUC_lexicon_tags_48 },
    { 0x2fb0ab8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1690ab8d, SUC_lexicon_tags_11 },
    { 0x4410ab8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc330ab90, SUC_lexicon_tags_2 },
    { 0xefd0ab90, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8150ab93, SUC_lexicon_tags_27 },
    { 0x5390ab94, SUC_lexicon_tags_15 },
    { 0xf810ab95, SUC_lexicon_tags_11 },
    { 0xfc90ab96, SUC_lexicon_tags_23 },
    { 0xeef0ab97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b0ab9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7650ab9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b0aba4, SUC_lexicon_tags_12 },
    { 0x5f10aba5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1690aba9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9830abab, SUC_lexicon_tags_9 },
    { 0xf6d0abab, SUC_lexicon_tags_33 },
    { 0x2cd0abab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddf0abaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8250abb1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8bf0abb3, SUC_lexicon_tags_37 },
    { 0x33b0abb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b50abb8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b0abbf, SUC_lexicon_tags_15 },
    { 0x3cf0abc0, SUC_lexicon_tags_1 },
    { 0x5df0abbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf090abc4, SUC_lexicon_tags_191 },
    { 0xf110abc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd510abc7, SUC_lexicon_tags_8 },
    { 0x2530abc8, SUC_lexicon_tags_0 },
    { 0xb4b0abc7, SUC_lexicon_tags_37 },
    { 0xe7b0abca, SUC_lexicon_tags_9 },
    { 0x6a10abca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf0abd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7030abe1, SUC_lexicon_tags_12 },
    { 0xa1b0abe2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf0abe5, SUC_lexicon_tags_37 },
    { 0x8db0abe6, SUC_lexicon_tags_37 },
    { 0xfbf0abe6, SUC_lexicon_tags_37 },
    { 0xdb0abe7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d0abf3, SUC_lexicon_tags_38 },
    { 0xf770abf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb10abf7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b0abfa, SUC_lexicon_tags_45 },
    { 0xdaf0abfb, SUC_lexicon_tags_19 },
    { 0x7390abfc, SUC_lexicon_tags_23 },
    { 0x8c70abfc, SUC_lexicon_tags_19 },
    { 0x7690abfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa390ac04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7fb0ac06, SUC_lexicon_tags_8 },
    { 0xda10ac06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf90ac09, SUC_lexicon_tags_31 },
    { 0xb50ac0a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x5830ac0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff70ac10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb0ac14, SUC_lexicon_tags_7 },
    { 0xe510ac15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad50ac19, SUC_lexicon_tags_17 },
    { 0xc2b0ac19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd610ac1d, SUC_lexicon_tags_0 },
    { 0xaef0ac1d, SUC_lexicon_tags_1 },
    { 0xdd50ac1e, SUC_lexicon_tags_15 },
    { 0x9a10ac20, SUC_lexicon_tags_45 },
    { 0xf0d0ac20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd510ac23, SUC_lexicon_tags_1 },
    { 0x6a50ac24, SUC_lexicon_tags_37 },
    { 0xdab0ac25, SUC_lexicon_tags_27 },
    { 0x7d10ac26, SUC_lexicon_tags_8 },
    { 0x23d0ac27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd50ac29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8670ac2f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f0ac33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac90ac3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0ac3e, SUC_lexicon_tags_6 },
    { 0x4270ac3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf0ac45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8390ac4d, SUC_lexicon_tags_25 },
    { 0x56d0ac4e, SUC_lexicon_tags_14 },
    { 0x7590ac4f, SUC_lexicon_tags_37 },
    { 0x4550ac4f, SUC_lexicon_tags_218 },
    { 0xde90ac4f, SUC_lexicon_tags_42 },
    { 0x6590ac52, SUC_lexicon_tags_25 },
    { 0x9c10ac53, SUC_lexicon_tags_42 },
    { 0x1d30ac4e, SUC_lexicon_tags_2 },
    { 0xe990ac4d, SUC_lexicon_tags_160 },
    { 0x48f0ac50, SUC_lexicon_tags_23 },
    { 0x5270ac50, SUC_lexicon_tags_15 },
    { 0xc2b0ac58, SUC_lexicon_tags_56 },
    { 0xf210ac59, SUC_lexicon_tags_37 },
    { 0x45d0ac58, SUC_lexicon_tags_9 },
    { 0xcd90ac5a, SUC_lexicon_tags_37 },
    { 0xb7f0ac4f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4390ac61, SUC_lexicon_tags_47 },
    { 0x9a30ac62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb0ac66, SUC_lexicon_tags_12 },
    { 0x6f70ac66, SUC_lexicon_tags_65 },
    { 0xba70ac66, SUC_lexicon_tags_33 },
    { 0xa5f0ac69, SUC_lexicon_tags_13 },
    { 0x10ac69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a50ac6d, SUC_lexicon_tags_12 },
    { 0xd1d0ac6e, SUC_lexicon_tags_0 },
    { 0x810ac6e, SUC_lexicon_tags_25 },
    { 0x9470ac6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5730ac79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b0ac7f, SUC_lexicon_tags_15 },
    { 0xf470ac80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd270ac83, SUC_lexicon_tags_7 },
    { 0xf4d0ac84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f0ac88, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1150ac8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe870ac90, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90ac95, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc130ac98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3670ac9a, SUC_lexicon_tags_37 },
    { 0x7f0ac9b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b0ac9e, SUC_lexicon_tags_15 },
    { 0xf6f0ac9f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa250aca1, SUC_lexicon_tags_79 },
    { 0xf930aca1, SUC_lexicon_tags_25 },
    { 0x86d0aca1, SUC_lexicon_tags_37 },
    { 0x3990aca4, SUC_lexicon_tags_60 },
    { 0x7410aca5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf0aca8, SUC_lexicon_tags_4 },
    { 0xb550aca9, SUC_lexicon_tags_136 },
    { 0xe330aca9, SUC_lexicon_tags_12 },
    { 0xe670acab, SUC_lexicon_tags_17 },
    { 0x81b0acac, SUC_lexicon_tags_17 },
    { 0x8a50acab, SUC_lexicon_tags_15 },
    { 0x5370acad, SUC_lexicon_tags_2 },
    { 0xb490acaf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc30acb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9750acba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c10acc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf210acc4, SUC_lexicon_tags_38 },
    { 0xb230acc4, SUC_lexicon_tags_11 },
    { 0x2670acc6, SUC_lexicon_tags_27 },
    { 0xaa90acc7, SUC_lexicon_tags_11 },
    { 0xaf90acc5, SUC_lexicon_tags_40 },
    { 0xeff0acc9, SUC_lexicon_tags_9 },
    { 0x90f0acca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b0acce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f0acd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd70acd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1270acd6, SUC_lexicon_tags_2 },
    { 0x330acd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff0acdc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc10ace2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa390ace8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc30acea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e50acee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29b0acf0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe570acf4, SUC_lexicon_tags_8 },
    { 0xd870acf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3390acf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x79b0acfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb0acfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a10ad01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa610ad04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65b0ad06, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe5d0ad08, SUC_lexicon_tags_37 },
    { 0xdbb0ad09, SUC_lexicon_tags_47 },
    { 0x52b0ad09, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d0ad0d, SUC_lexicon_tags_32 },
    { 0xe410ad0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f0ad11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc70ad15, SUC_lexicon_tags_28 },
    { 0x9430ad15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccd0ad18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbeb0ad1a, SUC_lexicon_tags_6 },
    { 0xda90ad1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c70ad1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4490ad22, SUC_lexicon_tags_37 },
    { 0x1850ad22, SUC_lexicon_tags_37 },
    { 0x6e30ad24, SUC_lexicon_tags_43 },
    { 0x26f0ad24, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x83f0ad27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5790ad29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5770ad2b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd0ad2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf630ad31, SUC_lexicon_tags_7 },
    { 0xe930ad32, SUC_lexicon_tags_25 },
    { 0x3550ad33, SUC_lexicon_tags_15 },
    { 0xebb0ad34, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b0ad3a, SUC_lexicon_tags_16 },
    { 0x2030ad3b, SUC_lexicon_tags_17 },
    { 0x8f10ad3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xac90ad3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d0ad48, SUC_lexicon_tags_0 },
    { 0x9d0ad49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd490ad4c, SUC_lexicon_tags_11 },
    { 0xcdd0ad4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x590ad4f, SUC_lexicon_tags_15 },
    { 0x1e10ad50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe390ad54, SUC_lexicon_tags_11 },
    { 0x3c30ad54, SUC_lexicon_tags_6 },
    { 0x27b0ad56, SUC_lexicon_tags_28 },
    { 0xae90ad54, SUC_lexicon_tags_25 },
    { 0x2990ad57, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe770ad5a, SUC_lexicon_tags_11 },
    { 0x9650ad5b, SUC_lexicon_tags_25 },
    { 0x5930ad5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd270ad5e, SUC_lexicon_tags_15 },
    { 0x5150ad5e, SUC_lexicon_tags_30 },
    { 0xd90ad60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab50ad62, SUC_lexicon_tags_13 },
    { 0xc110ad63, SUC_lexicon_tags_12 },
    { 0xc250ad64, SUC_lexicon_tags_2 },
    { 0xbcf0ad65, SUC_lexicon_tags_2 },
    { 0xf8d0ad62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa070ad6b, SUC_lexicon_tags_25 },
    { 0x5870ad6b, SUC_lexicon_tags_542 },
    { 0xe530ad6d, SUC_lexicon_tags_17 },
    { 0x51f0ad6e, SUC_lexicon_tags_45 },
    { 0x57f0ad6e, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x5270ad71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d0ad77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a70ad79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1350ad7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb0ad7e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xee30ad80, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6db0ad82, SUC_lexicon_tags_82 },
    { 0x23d0ad83, SUC_lexicon_tags_37 },
    { 0xdb90ad84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xde50ad86, SUC_lexicon_tags_14 },
    { 0x16f0ad86, SUC_lexicon_tags_14 },
    { 0xec30ad87, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b0ad92, SUC_lexicon_tags_65 },
    { 0xad0ad93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc30ad95, SUC_lexicon_tags_14 },
    { 0xe390ad96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6990ad9c, SUC_lexicon_tags_2 },
    { 0xc670ad9d, SUC_lexicon_tags_10 },
    { 0xb670ad9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf490ada8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x13b0adaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc70adaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310adb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff0adb7, SUC_lexicon_tags_0 },
    { 0xb3d0adb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b0adbb, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x76b0adbd, SUC_lexicon_tags_79 },
    { 0x6dd0adbe, SUC_lexicon_tags_21 },
    { 0x44b0adbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd090adc1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc610adc4, SUC_lexicon_tags_15 },
    { 0xc70adc5, SUC_lexicon_tags_39 },
    { 0xed70adc4, SUC_lexicon_tags_12 },
    { 0x9e90adc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530adcb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe530adcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e90add1, SUC_lexicon_tags_32 },
    { 0x3750add2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe430add6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9810adda, SUC_lexicon_tags_9 },
    { 0x4330addb, SUC_lexicon_tags_4 },
    { 0x31f0addc, SUC_lexicon_tags_15 },
    { 0x2830addd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ade1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6dd0ade3, SUC_lexicon_tags_11 },
    { 0x4cb0ade4, SUC_lexicon_tags_28 },
    { 0xeb90ade3, SUC_lexicon_tags_60 },
    { 0x7670ade6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e70ade9, SUC_lexicon_tags_34 },
    { 0x6cf0adea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6990adef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5050adf2, SUC_lexicon_tags_12 },
    { 0x5ff0adf3, SUC_lexicon_tags_8 },
    { 0x2490adf4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f0adf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f0adfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cf0adfe, SUC_lexicon_tags_11 },
    { 0x2670adff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8410ae03, SUC_lexicon_tags_12 },
    { 0x3070ae03, SUC_lexicon_tags_14 },
    { 0xad70ae05, SUC_lexicon_tags_27 },
    { 0x5690ae05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf90ae0a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b0ae0e, SUC_lexicon_tags_17 },
    { 0x6c90ae0f, SUC_lexicon_tags_27 },
    { 0xef10ae10, SUC_lexicon_tags_54 },
    { 0x1210ae0f, SUC_lexicon_tags_60 },
    { 0x7390ae12, SUC_lexicon_tags_25 },
    { 0xcaf0ae11, SUC_lexicon_tags_15 },
    { 0x20f0ae10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc990ae1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf0ae1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb70ae22, SUC_lexicon_tags_2 },
    { 0x9510ae23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc50ae28, SUC_lexicon_tags_9 },
    { 0x8db0ae29, SUC_lexicon_tags_7 },
    { 0x4f70ae2a, SUC_lexicon_tags_17 },
    { 0x3e90ae2b, SUC_lexicon_tags_59 },
    { 0x95b0ae29, SUC_lexicon_tags_0 },
    { 0x3e70ae2d, SUC_lexicon_tags_12 },
    { 0xd50ae2e, SUC_lexicon_tags_1 },
    { 0x75f0ae2f, SUC_lexicon_tags_21 },
    { 0x9790ae30, SUC_lexicon_tags_7 },
    { 0x6d70ae2c, SUC_lexicon_tags_49 },
    { 0xeb90ae32, SUC_lexicon_tags_37 },
    { 0x6830ae29, SUC_lexicon_tags_34 },
    { 0x7a50ae29, SUC_lexicon_tags_6 },
    { 0xcf30ae35, SUC_lexicon_tags_7 },
    { 0xa950ae35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b70ae39, SUC_lexicon_tags_19 },
    { 0xa2b0ae3a, SUC_lexicon_tags_27 },
    { 0xc770ae3b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa7d0ae3d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2250ae41, SUC_lexicon_tags_84 },
    { 0xf0d0ae42, SUC_lexicon_tags_54 },
    { 0xba50ae43, SUC_lexicon_tags_25 },
    { 0xfcb0ae42, SUC_lexicon_tags_1 },
    { 0x7db0ae42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d0ae4a, SUC_lexicon_tags_37 },
    { 0x5f30ae4a, SUC_lexicon_tags_23 },
    { 0x8fd0ae4c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf0ae51, SUC_lexicon_tags_19 },
    { 0x8ef0ae51, SUC_lexicon_tags_28 },
    { 0xae90ae52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d0ae56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa670ae59, SUC_lexicon_tags_45 },
    { 0xb630ae5a, SUC_lexicon_tags_45 },
    { 0x3250ae5b, SUC_lexicon_tags_6 },
    { 0x9850ae5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd890ae5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8d50ae60, SUC_lexicon_tags_38 },
    { 0x6390ae61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b0ae6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b30ae70, SUC_lexicon_tags_29 },
    { 0xbef0ae71, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff0ae78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf10ae7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa290ae84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3af0ae86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7010ae89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc770ae90, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x9770ae92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xced0ae94, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x5e70ae96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd250ae9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd0ae9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d0aea2, SUC_lexicon_tags_17 },
    { 0xa530aea2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f0aea6, SUC_lexicon_tags_45 },
    { 0xc590aea7, SUC_lexicon_tags_12 },
    { 0xbf30aea7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f0aeae, SUC_lexicon_tags_33 },
    { 0x97d0aeaf, SUC_lexicon_tags_31 },
    { 0x1a50aeb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc310aeb4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b0aeb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550aec0, SUC_lexicon_tags_23 },
    { 0x8590aec1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe70aec3, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d30aec8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe750aecd, SUC_lexicon_tags_80 },
    { 0x3c50aece, SUC_lexicon_tags_35 },
    { 0xcd0aece, SUC_lexicon_tags_45 },
    { 0xf9f0aed0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b0aed6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd0aed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6470aedb, SUC_lexicon_tags_23 },
    { 0xbe90aedc, SUC_lexicon_tags_23 },
    { 0xf5f0aedb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ab0aedf, SUC_lexicon_tags_38 },
    { 0xaa50aee0, SUC_lexicon_tags_9 },
    { 0x3af0aee1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9690aee3, SUC_lexicon_tags_12 },
    { 0x58d0aee4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xad90aee6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f0aeec, SUC_lexicon_tags_15 },
    { 0x5570aeec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55b0aeef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d50aef2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c30aef5, SUC_lexicon_tags_14 },
    { 0x60f0aef6, SUC_lexicon_tags_11 },
    { 0xbf70aef6, SUC_lexicon_tags_23 },
    { 0x22d0aef8, SUC_lexicon_tags_17 },
    { 0xc330aef9, SUC_lexicon_tags_45 },
    { 0x8690aefa, SUC_lexicon_tags_43 },
    { 0x5770aefb, SUC_lexicon_tags_12 },
    { 0xf970aefa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef0af02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb50af0a, SUC_lexicon_tags_65 },
    { 0x3f0af0b, SUC_lexicon_tags_23 },
    { 0x1b30af0c, SUC_lexicon_tags_19 },
    { 0xf630af0d, SUC_lexicon_tags_17 },
    { 0x5c10af0c, SUC_lexicon_tags_7 },
    { 0x88b0af0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d0af11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f10af15, SUC_lexicon_tags_10 },
    { 0xb1b0af16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa830af18, SUC_lexicon_tags_7 },
    { 0x20b0af19, SUC_lexicon_tags_40 },
    { 0xf2d0af19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d0af1e, SUC_lexicon_tags_25 },
    { 0xe4b0af1e, SUC_lexicon_tags_45 },
    { 0x2c10af20, SUC_lexicon_tags_11 },
    { 0x8870af21, SUC_lexicon_tags_33 },
    { 0x4d30af22, SUC_lexicon_tags_56 },
    { 0xa530af1e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xaab0af25, SUC_lexicon_tags_10 },
    { 0x1590af25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4750af29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6310af30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f0af33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb0af37, SUC_lexicon_tags_12 },
    { 0xa850af38, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa510af3a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e90af3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc50af3f, SUC_lexicon_tags_33 },
    { 0x7b30af3f, SUC_lexicon_tags_38 },
    { 0x3290af41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2850af44, SUC_lexicon_tags_49 },
    { 0x710af45, SUC_lexicon_tags_7 },
    { 0xa7d0af44, SUC_lexicon_tags_21 },
    { 0x73b0af47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7570af49, SUC_lexicon_tags_7 },
    { 0xcf30af4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe590af4c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f50af4f, SUC_lexicon_tags_60 },
    { 0xea50af4f, SUC_lexicon_tags_7 },
    { 0xc530af51, SUC_lexicon_tags_7 },
    { 0xbd30af52, SUC_lexicon_tags_7 },
    { 0x10d0af53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b0af58, SUC_lexicon_tags_17 },
    { 0xcfb0af58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44f0af5b, SUC_lexicon_tags_63 },
    { 0xd7f0af5b, SUC_lexicon_tags_2 },
    { 0x13b0af5d, SUC_lexicon_tags_48 },
    { 0xf550af5e, SUC_lexicon_tags_15 },
    { 0xebf0af5e, SUC_lexicon_tags_9 },
    { 0x4670af5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc050af63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7b0af65, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x24b0af67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b70af6a, SUC_lexicon_tags_10 },
    { 0x310af6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc070af6d, SUC_lexicon_tags_23 },
    { 0x7690af6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8050af73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b70af75, SUC_lexicon_tags_1 },
    { 0xf210af76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd430af7d, SUC_lexicon_tags_1 },
    { 0x3a70af7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb450af80, SUC_lexicon_tags_17 },
    { 0x12b0af81, SUC_lexicon_tags_13 },
    { 0x2cb0af82, SUC_lexicon_tags_30 },
    { 0xb030af80, SUC_lexicon_tags_37 },
    { 0xcbb0af82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe010af88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6350af8a, SUC_lexicon_tags_15 },
    { 0x6670af8b, SUC_lexicon_tags_30 },
    { 0x7010af8c, SUC_lexicon_tags_72 },
    { 0x22b0af8d, SUC_lexicon_tags_34 },
    { 0x89b0af8e, SUC_lexicon_tags_79 },
    { 0x30d0af8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2dd0af91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5630af93, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x210af96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5170af9a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf390af9d, SUC_lexicon_tags_15 },
    { 0x2170af9e, SUC_lexicon_tags_15 },
    { 0x7c70af9f, SUC_lexicon_tags_63 },
    { 0x13f0af9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b0afa7, SUC_lexicon_tags_60 },
    { 0x4830afa7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf570afaa, SUC_lexicon_tags_11 },
    { 0x8cf0afab, SUC_lexicon_tags_25 },
    { 0xdf0afaa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a10afaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f0afb2, SUC_lexicon_tags_9 },
    { 0x2e30afb2, SUC_lexicon_tags_37 },
    { 0x8d10afb4, SUC_lexicon_tags_47 },
    { 0x6330afb5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f0afbb, SUC_lexicon_tags_68 },
    { 0xcc70afbc, SUC_lexicon_tags_29 },
    { 0xf6b0afbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x550afbf, SUC_lexicon_tags_28 },
    { 0x5dd0afc0, SUC_lexicon_tags_8 },
    { 0xee50afbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe830afc6, SUC_lexicon_tags_2 },
    { 0x6590afc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f0afcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2670afce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa030afd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7050afd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc30afda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad0afde, SUC_lexicon_tags_15 },
    { 0x9f90afde, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd570afe3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70afe6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa530afea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca10afee, SUC_lexicon_tags_48 },
    { 0xa950afef, SUC_lexicon_tags_45 },
    { 0x63d0aff0, SUC_lexicon_tags_18 },
    { 0xc0d0aff1, SUC_lexicon_tags_4 },
    { 0x12b0aff1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a10aff5, SUC_lexicon_tags_92 },
    { 0x8ab0aff5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c90aff9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ed0affb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbff0affd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9030afff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb70b003, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8c70b005, SUC_lexicon_tags_61 },
    { 0xe410b006, SUC_lexicon_tags_11 },
    { 0x50f0b006, SUC_lexicon_tags_11 },
    { 0x4e50b008, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb0b00e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed0b011, SUC_lexicon_tags_77 },
    { 0x1110b011, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2350b018, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4410b01a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x52d0b01c, SUC_lexicon_tags_26 },
    { 0x5e30b01c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3bf0b01f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb770b023, SUC_lexicon_tags_17 },
    { 0xef70b023, SUC_lexicon_tags_63 },
    { 0xa8b0b023, SUC_lexicon_tags_23 },
    { 0x8a90b023, SUC_lexicon_tags_15 },
    { 0x7810b027, SUC_lexicon_tags_23 },
    { 0xa70b028, SUC_lexicon_tags_25 },
    { 0x30d0b028, SUC_lexicon_tags_25 },
    { 0xb550b026, SUC_lexicon_tags_25 },
    { 0x18f0b02b, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7430b02f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f10b039, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e90b03b, SUC_lexicon_tags_48 },
    { 0x6370b03b, SUC_lexicon_tags_4 },
    { 0x8b0b03d, SUC_lexicon_tags_10 },
    { 0xc610b03d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7310b040, SUC_lexicon_tags_25 },
    { 0x8670b041, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9610b044, SUC_lexicon_tags_1 },
    { 0x4d90b045, SUC_lexicon_tags_17 },
    { 0xe870b044, SUC_lexicon_tags_49 },
    { 0xaf50b046, SUC_lexicon_tags_17 },
    { 0x3910b045, SUC_lexicon_tags_4 },
    { 0xef70b044, SUC_lexicon_tags_17 },
    { 0xea30b04a, SUC_lexicon_tags_165 },
    { 0xac50b04a, SUC_lexicon_tags_6 },
    { 0x1850b04a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd430b050, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf90b055, SUC_lexicon_tags_27 },
    { 0xf550b056, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x76d0b058, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1010b05f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b50b065, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2230b068, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb650b06a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c30b06f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4930b075, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x350b07a, SUC_lexicon_tags_37 },
    { 0x4870b07a, SUC_lexicon_tags_87 },
    { 0xed30b07b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4370b07e, SUC_lexicon_tags_49 },
    { 0x17f0b07f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50b085, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb0b088, SUC_lexicon_tags_1 },
    { 0x5490b088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ed0b08b, SUC_lexicon_tags_68 },
    { 0xf850b08c, SUC_lexicon_tags_126 },
    { 0xf690b08c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb290b095, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60f0b097, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf730b09a, SUC_lexicon_tags_63 },
    { 0x8d70b09b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf50b09e, SUC_lexicon_tags_7 },
    { 0x8e30b09f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf070b0a2, SUC_lexicon_tags_33 },
    { 0xd110b0a3, SUC_lexicon_tags_14 },
    { 0x3ab0b0a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1970b0a9, SUC_lexicon_tags_14 },
    { 0xea50b0aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8890b0ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc430b0b3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd10b0b5, SUC_lexicon_tags_38 },
    { 0xbb10b0b6, SUC_lexicon_tags_142 },
    { 0x8b10b0b7, SUC_lexicon_tags_2 },
    { 0xf70b0b7, SUC_lexicon_tags_17 },
    { 0x4b50b0b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0b0bb, SUC_lexicon_tags_15 },
    { 0x87f0b0bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c30b0c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebb0b0c3, SUC_lexicon_tags_37 },
    { 0x8f70b0c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc670b0c7, SUC_lexicon_tags_38 },
    { 0x8af0b0c8, SUC_lexicon_tags_12 },
    { 0xf6b0b0c9, SUC_lexicon_tags_0 },
    { 0x8770b0ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b0b0d1, SUC_lexicon_tags_12 },
    { 0x5870b0d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa270b0d5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd850b0d8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7810b0da, SUC_lexicon_tags_1 },
    { 0xd6b0b0db, SUC_lexicon_tags_9 },
    { 0x50b0b0dc, SUC_lexicon_tags_37 },
    { 0x6f90b0dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd0b0e0, SUC_lexicon_tags_15 },
    { 0x2710b0e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9330b0e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b0b0e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5250b0ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6510b0ec, SUC_lexicon_tags_1 },
    { 0x4890b0ec, SUC_lexicon_tags_47 },
    { 0x3d0b0ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7390b0f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df0b0f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf0b0f7, SUC_lexicon_tags_131 },
    { 0x3d70b0f8, SUC_lexicon_tags_28 },
    { 0x3df0b0f8, SUC_lexicon_tags_1 },
    { 0xf6b0b0f9, SUC_lexicon_tags_1 },
    { 0xc350b0fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1670b0fe, SUC_lexicon_tags_17 },
    { 0x4030b0ff, SUC_lexicon_tags_66 },
    { 0xf2b0b0fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd0b103, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd130b105, SUC_lexicon_tags_27 },
    { 0x5cb0b105, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f0b10b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea30b111, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf590b113, SUC_lexicon_tags_14 },
    { 0x95b0b114, SUC_lexicon_tags_25 },
    { 0x7550b114, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc290b11c, SUC_lexicon_tags_45 },
    { 0xa630b11c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd90b120, SUC_lexicon_tags_107 },
    { 0x1c50b120, SUC_lexicon_tags_230 },
    { 0x6f10b120, SUC_lexicon_tags_8 },
    { 0x28d0b123, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd0b129, SUC_lexicon_tags_11 },
    { 0x4850b129, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xbed0b12c, SUC_lexicon_tags_218 },
    { 0xe3d0b12d, SUC_lexicon_tags_130 },
    { 0x97d0b12e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe590b131, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f0b13b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ed0b13d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d70b144, SUC_lexicon_tags_66 },
    { 0x2790b145, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0b148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a10b14d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab0b151, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f0b154, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb810b159, SUC_lexicon_tags_32 },
    { 0x290b159, SUC_lexicon_tags_2 },
    { 0x94f0b15b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a30b161, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd850b165, SUC_lexicon_tags_27 },
    { 0x1a50b165, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9270b16a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff0b16e, SUC_lexicon_tags_39 },
    { 0xd770b16f, SUC_lexicon_tags_108 },
    { 0x3070b170, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb0b175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d0b178, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db0b17d, SUC_lexicon_tags_47 },
    { 0x4c30b17e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb0b184, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3510b191, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d0b196, SUC_lexicon_tags_594 },
    { 0x9530b197, SUC_lexicon_tags_46 },
    { 0x3ef0b197, SUC_lexicon_tags_12 },
    { 0x8d70b199, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc990b19c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa070b19e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9490b1a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5f0b1a2, SUC_lexicon_tags_15 },
    { 0xd70b1a3, SUC_lexicon_tags_15 },
    { 0x2550b1a2, SUC_lexicon_tags_25 },
    { 0x9f50b1a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x19b0b1a7, SUC_lexicon_tags_38 },
    { 0xebf0b1a8, SUC_lexicon_tags_38 },
    { 0xfa90b1a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed0b1ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b0b1af, SUC_lexicon_tags_1 },
    { 0xbab0b1b0, SUC_lexicon_tags_4 },
    { 0x5eb0b1b1, SUC_lexicon_tags_54 },
    { 0xa0f0b1b2, SUC_lexicon_tags_16 },
    { 0x9f90b1b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x650b1b5, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x6d90b1b7, SUC_lexicon_tags_19 },
    { 0x47b0b1b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e30b1bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7990b1bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e50b1c0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d70b1cb, SUC_lexicon_tags_37 },
    { 0x6f70b1cc, SUC_lexicon_tags_45 },
    { 0x3510b1cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d0b1d0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6df0b1d2, SUC_lexicon_tags_61 },
    { 0xbed0b1d2, SUC_lexicon_tags_7 },
    { 0xbb10b1d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa670b1d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b0b1d9, SUC_lexicon_tags_1 },
    { 0x3cf0b1d9, SUC_lexicon_tags_23 },
    { 0xab50b1db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63d0b1dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed0b1e4, SUC_lexicon_tags_38 },
    { 0x1df0b1e4, SUC_lexicon_tags_2 },
    { 0x3d70b1e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd790b1e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f0b1eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4950b1ed, SUC_lexicon_tags_6 },
    { 0x7a10b1ee, SUC_lexicon_tags_84 },
    { 0xe250b1ed, SUC_lexicon_tags_25 },
    { 0xb510b1f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa810b1f4, SUC_lexicon_tags_11 },
    { 0x32f0b1f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2d0b1f7, SUC_lexicon_tags_68 },
    { 0x1670b1f8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x8290b1fa, SUC_lexicon_tags_12 },
    { 0x2130b1fa, SUC_lexicon_tags_37 },
    { 0x1d30b1fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd0b207, SUC_lexicon_tags_27 },
    { 0x23b0b208, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce50b210, SUC_lexicon_tags_43 },
    { 0x1430b211, SUC_lexicon_tags_64 },
    { 0x3ef0b211, SUC_lexicon_tags_14 },
    { 0x2570b210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb150b216, SUC_lexicon_tags_56 },
    { 0x3870b216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3710b21b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf890b21d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4810b222, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6150b22b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91b0b22d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfc50b22f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4350b231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3450b238, SUC_lexicon_tags_15 },
    { 0xefb0b239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc90b243, SUC_lexicon_tags_14 },
    { 0xd830b244, SUC_lexicon_tags_37 },
    { 0x1d90b245, SUC_lexicon_tags_0 },
    { 0xffd0b246, SUC_lexicon_tags_26 },
    { 0x2490b247, SUC_lexicon_tags_15 },
    { 0x6610b248, SUC_lexicon_tags_14 },
    { 0x9e30b248, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd110b24e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d0b253, SUC_lexicon_tags_7 },
    { 0x9c10b254, SUC_lexicon_tags_16 },
    { 0xc9d0b255, SUC_lexicon_tags_2 },
    { 0x3b90b254, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f0b25c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8810b25e, SUC_lexicon_tags_25 },
    { 0xc010b25f, SUC_lexicon_tags_25 },
    { 0x92f0b260, SUC_lexicon_tags_17 },
    { 0xf4b0b260, SUC_lexicon_tags_13 },
    { 0xd330b260, SUC_lexicon_tags_2 },
    { 0x77d0b263, SUC_lexicon_tags_25 },
    { 0xc290b260, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x2430b266, SUC_lexicon_tags_37 },
    { 0x8350b267, SUC_lexicon_tags_0 },
    { 0x1490b267, SUC_lexicon_tags_1 },
    { 0x25f0b268, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xabb0b26b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd410b26f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea50b271, SUC_lexicon_tags_25 },
    { 0x34f0b272, SUC_lexicon_tags_2 },
    { 0x6490b273, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d0b278, SUC_lexicon_tags_34 },
    { 0xa010b279, SUC_lexicon_tags_7 },
    { 0x6b30b27a, SUC_lexicon_tags_37 },
    { 0xb6d0b27a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b70b27e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd330b281, SUC_lexicon_tags_17 },
    { 0x3650b282, SUC_lexicon_tags_33 },
    { 0x48b0b282, SUC_lexicon_tags_12 },
    { 0xa6b0b284, SUC_lexicon_tags_47 },
    { 0xf5f0b284, SUC_lexicon_tags_49 },
    { 0x3c90b286, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca50b28c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d30b28e, SUC_lexicon_tags_43 },
    { 0x6910b28e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f0b293, SUC_lexicon_tags_60 },
    { 0xf590b294, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb30b298, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b0b29b, SUC_lexicon_tags_17 },
    { 0x11b0b29c, SUC_lexicon_tags_7 },
    { 0xfc10b29c, SUC_lexicon_tags_36 },
    { 0x6850b29e, SUC_lexicon_tags_37 },
    { 0x6d70b29c, SUC_lexicon_tags_40 },
    { 0xa3d0b2a0, SUC_lexicon_tags_25 },
    { 0x89d0b2a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba90b2a5, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb0b2aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e10b2b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f10b2b7, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x5d10b2b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d90b2c1, SUC_lexicon_tags_47 },
    { 0xcd70b2c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7810b2c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd0b2ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe5b0b2cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea30b2d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd690b2d3, SUC_lexicon_tags_15 },
    { 0x4d50b2d4, SUC_lexicon_tags_25 },
    { 0x6650b2d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5f0b2d7, SUC_lexicon_tags_31 },
    { 0x1310b2d7, SUC_lexicon_tags_201 },
    { 0x90d0b2d9, SUC_lexicon_tags_2 },
    { 0x2f30b2d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b0b2df, SUC_lexicon_tags_4 },
    { 0x3710b2df, SUC_lexicon_tags_8 },
    { 0x8f30b2df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe10b2e4, SUC_lexicon_tags_27 },
    { 0x7a90b2e5, SUC_lexicon_tags_37 },
    { 0x5070b2e4, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f0b2ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5590b2f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa870b2f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb670b2fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb690b300, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xccb0b302, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7db0b304, SUC_lexicon_tags_38 },
    { 0x5470b305, SUC_lexicon_tags_33 },
    { 0xc350b306, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe90b308, SUC_lexicon_tags_17 },
    { 0x1250b309, SUC_lexicon_tags_11 },
    { 0x5350b30a, SUC_lexicon_tags_1 },
    { 0xaef0b30a, SUC_lexicon_tags_31 },
    { 0x8f0b30b, SUC_lexicon_tags_13 },
    { 0x3950b30d, SUC_lexicon_tags_12 },
    { 0xc0d0b30d, SUC_lexicon_tags_0 },
    { 0x1870b30d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54f0b311, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d50b319, SUC_lexicon_tags_25 },
    { 0x8d90b31a, SUC_lexicon_tags_33 },
    { 0x2930b31b, SUC_lexicon_tags_30 },
    { 0x6ef0b319, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x510b31e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9db0b320, SUC_lexicon_tags_37 },
    { 0x22f0b321, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5570b323, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b70b32a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb390b32f, SUC_lexicon_tags_37 },
    { 0xb50b32f, SUC_lexicon_tags_14 },
    { 0x5d30b330, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb0b334, SUC_lexicon_tags_17 },
    { 0x4b90b334, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xce50b338, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2c70b33a, SUC_lexicon_tags_17 },
    { 0x2670b33b, SUC_lexicon_tags_12 },
    { 0x7b90b33c, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0b33f, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70b342, SUC_lexicon_tags_7 },
    { 0x3df0b342, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3310b345, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4530b348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef0b34e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d0b352, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa510b355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5810b358, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed0b361, SUC_lexicon_tags_37 },
    { 0x5b10b361, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54b0b364, SUC_lexicon_tags_49 },
    { 0xee10b365, SUC_lexicon_tags_1 },
    { 0x5970b364, SUC_lexicon_tags_0 },
    { 0x53f0b367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed0b36a, SUC_lexicon_tags_14 },
    { 0x5cf0b36a, SUC_lexicon_tags_2 },
    { 0x6990b36a, SUC_lexicon_tags_0 },
    { 0x96b0b36d, SUC_lexicon_tags_17 },
    { 0xaa90b36e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31f0b370, SUC_lexicon_tags_15 },
    { 0x6990b371, SUC_lexicon_tags_14 },
    { 0x7bb0b372, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11b0b374, SUC_lexicon_tags_47 },
    { 0x96f0b375, SUC_lexicon_tags_19 },
    { 0xcd30b375, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8b0b378, SUC_lexicon_tags_12 },
    { 0xbd70b379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b0b37c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe270b37e, SUC_lexicon_tags_19 },
    { 0x71f0b37f, SUC_lexicon_tags_38 },
    { 0xb1b0b37f, SUC_lexicon_tags_37 },
    { 0x9bf0b381, SUC_lexicon_tags_70 },
    { 0xb930b381, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb310b385, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f10b38a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c10b38d, SUC_lexicon_tags_2 },
    { 0xf490b38d, SUC_lexicon_tags_7 },
    { 0x3230b38f, SUC_lexicon_tags_43 },
    { 0x5a70b390, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6bb0b392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb30b394, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5010b396, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b30b39b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1eb0b39d, SUC_lexicon_tags_1 },
    { 0x90f0b39e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f0b3a1, SUC_lexicon_tags_1 },
    { 0x5ab0b3a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70b3a8, SUC_lexicon_tags_37 },
    { 0x81b0b3a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc670b3ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb0b3ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed0b3b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca30b3b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d0b3bb, SUC_lexicon_tags_68 },
    { 0x2c70b3bb, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0b3bf, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b30b3c2, SUC_lexicon_tags_26 },
    { 0xeb90b3c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b0b3c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc450b3c8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe410b3cf, SUC_lexicon_tags_13 },
    { 0xa770b3d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94f0b3d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70b3d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5730b3da, SUC_lexicon_tags_17 },
    { 0x98d0b3db, SUC_lexicon_tags_45 },
    { 0x4f50b3dc, SUC_lexicon_tags_1 },
    { 0xec90b3dc, SUC_lexicon_tags_63 },
    { 0xf0b3da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b0b3e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3470b3e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc6b0b3e8, SUC_lexicon_tags_25 },
    { 0x38b0b3e9, SUC_lexicon_tags_37 },
    { 0xf990b3e9, SUC_lexicon_tags_1 },
    { 0x7530b3e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e70b3ed, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x6110b3ef, SUC_lexicon_tags_25 },
    { 0xdb70b3f0, SUC_lexicon_tags_68 },
    { 0xbb10b3f0, SUC_lexicon_tags_23 },
    { 0x7d10b3f1, SUC_lexicon_tags_25 },
    { 0x8cf0b3f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd0b3f8, SUC_lexicon_tags_11 },
    { 0x2f30b3f9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6870b3fb, SUC_lexicon_tags_45 },
    { 0xb890b3fc, SUC_lexicon_tags_30 },
    { 0x3ad0b3fc, SUC_lexicon_tags_14 },
    { 0x6250b3fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c30b402, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1690b404, SUC_lexicon_tags_13 },
    { 0x3070b405, SUC_lexicon_tags_47 },
    { 0x4d50b406, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe970b408, SUC_lexicon_tags_37 },
    { 0x1bf0b409, SUC_lexicon_tags_37 },
    { 0xbd0b409, SUC_lexicon_tags_2 },
    { 0xc030b40b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd0b410, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a30b412, SUC_lexicon_tags_23 },
    { 0x8110b413, SUC_lexicon_tags_37 },
    { 0x9410b413, SUC_lexicon_tags_14 },
    { 0xfb70b413, SUC_lexicon_tags_65 },
    { 0xe690b415, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd470b418, SUC_lexicon_tags_25 },
    { 0x20f0b418, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4570b41e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d0b421, SUC_lexicon_tags_17 },
    { 0x7510b421, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1670b425, SUC_lexicon_tags_27 },
    { 0xca50b425, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb0b42b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x13d0b42d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc0b0b42f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b436, SUC_lexicon_tags_49 },
    { 0x9c30b437, SUC_lexicon_tags_9 },
    { 0xef30b436, SUC_lexicon_tags_14 },
    { 0x6910b439, SUC_lexicon_tags_11 },
    { 0x7890b438, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c90b43c, SUC_lexicon_tags_47 },
    { 0xd390b43d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x370b43f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30b445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b0b448, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb630b44f, SUC_lexicon_tags_4 },
    { 0xc010b44f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6570b452, SUC_lexicon_tags_7 },
    { 0xcd90b453, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb1d0b455, SUC_lexicon_tags_0 },
    { 0x7710b455, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d90b459, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10b0b45b, SUC_lexicon_tags_7 },
    { 0xe830b45c, SUC_lexicon_tags_12 },
    { 0xa170b45d, SUC_lexicon_tags_37 },
    { 0x17b0b45e, SUC_lexicon_tags_35 },
    { 0x36f0b45e, SUC_lexicon_tags_14 },
    { 0xcfd0b460, SUC_lexicon_tags_1 },
    { 0x42d0b45e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf0b466, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce70b46a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb570b46e, SUC_lexicon_tags_15 },
    { 0xb850b46f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f50b472, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4790b478, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b0b48c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b0b48f, SUC_lexicon_tags_25 },
    { 0x5070b48f, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff0b493, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1610b499, SUC_lexicon_tags_2 },
    { 0xd710b49a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0b4a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d30b4a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4170b4a7, SUC_lexicon_tags_1 },
    { 0x3f0b4a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890b4af, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd0b4b2, SUC_lexicon_tags_53 },
    { 0x5570b4b3, SUC_lexicon_tags_37 },
    { 0x1ff0b4b2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x19f0b4b6, SUC_lexicon_tags_12 },
    { 0xa110b4b6, SUC_lexicon_tags_27 },
    { 0xdd90b4b8, SUC_lexicon_tags_74 },
    { 0x6510b4b7, SUC_lexicon_tags_23 },
    { 0x7f50b4b8, SUC_lexicon_tags_37 },
    { 0x2f0b4bb, SUC_lexicon_tags_17 },
    { 0x53b0b4bb, SUC_lexicon_tags_23 },
    { 0xc630b4bd, SUC_lexicon_tags_17 },
    { 0x8fd0b4bd, SUC_lexicon_tags_18 },
    { 0x970b4be, SUC_lexicon_tags_27 },
    { 0x530b4c0, SUC_lexicon_tags_14 },
    { 0x4510b4bb, SUC_lexicon_tags_25 },
    { 0x2af0b4c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d0b4c8, SUC_lexicon_tags_1 },
    { 0x7d10b4c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc850b4cc, SUC_lexicon_tags_47 },
    { 0x2e90b4cd, SUC_lexicon_tags_0 },
    { 0xad70b4ce, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa90b4d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f0b4e4, SUC_lexicon_tags_2 },
    { 0x9330b4e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1510b4e9, SUC_lexicon_tags_7 },
    { 0x28d0b4ea, SUC_lexicon_tags_25 },
    { 0x7b70b4eb, SUC_lexicon_tags_6 },
    { 0x1510b4ec, SUC_lexicon_tags_37 },
    { 0x5530b4ea, SUC_lexicon_tags_32 },
    { 0x9250b4eb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b0b4f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb110b4f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b0b4fa, SUC_lexicon_tags_25 },
    { 0x850b4fb, SUC_lexicon_tags_12 },
    { 0x87d0b4fc, SUC_lexicon_tags_1 },
    { 0xb9d0b4fd, SUC_lexicon_tags_60 },
    { 0x1070b4fb, SUC_lexicon_tags_25 },
    { 0xf970b4ff, SUC_lexicon_tags_2 },
    { 0xb1b0b500, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2fb0b502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8450b508, SUC_lexicon_tags_79 },
    { 0xaf0b509, SUC_lexicon_tags_55 },
    { 0xbe50b508, SUC_lexicon_tags_25 },
    { 0xf5f0b50b, SUC_lexicon_tags_37 },
    { 0xad0b50b, SUC_lexicon_tags_15 },
    { 0x6330b50b, SUC_lexicon_tags_22 },
    { 0xc070b50e, SUC_lexicon_tags_13 },
    { 0x96f0b50f, SUC_lexicon_tags_39 },
    { 0xa830b510, SUC_lexicon_tags_17 },
    { 0x1a30b50d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc430b525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe130b529, SUC_lexicon_tags_14 },
    { 0xe3d0b52a, SUC_lexicon_tags_17 },
    { 0xc370b52b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf790b534, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73d0b536, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae90b53f, SUC_lexicon_tags_12 },
    { 0xe270b53f, SUC_lexicon_tags_11 },
    { 0xe070b53f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f0b54a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd350b54c, SUC_lexicon_tags_1 },
    { 0x1d30b54d, SUC_lexicon_tags_7 },
    { 0x5fd0b54e, SUC_lexicon_tags_14 },
    { 0xf30b54c, SUC_lexicon_tags_0 },
    { 0x6950b54e, SUC_lexicon_tags_14 },
    { 0xec10b54f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b0b556, SUC_lexicon_tags_7 },
    { 0xd370b557, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e90b55d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff0b562, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf0b565, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c30b56d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7150b56f, SUC_lexicon_tags_7 },
    { 0xdcb0b570, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb70b572, SUC_lexicon_tags_37 },
    { 0x7370b573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7f0b575, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1450b57a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad70b582, SUC_lexicon_tags_9 },
    { 0x5710b583, SUC_lexicon_tags_30 },
    { 0x6a50b584, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f0b588, SUC_lexicon_tags_25 },
    { 0xa330b589, SUC_lexicon_tags_7 },
    { 0xe210b588, SUC_lexicon_tags_29 },
    { 0x5d0b588, SUC_lexicon_tags_27 },
    { 0x3630b58c, SUC_lexicon_tags_37 },
    { 0x8af0b58d, SUC_lexicon_tags_105 },
    { 0x8a70b58e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9510b594, SUC_lexicon_tags_48 },
    { 0x3290b595, SUC_lexicon_tags_25 },
    { 0xdb70b596, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea70b59a, SUC_lexicon_tags_33 },
    { 0xc6f0b59b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d0b5ae, SUC_lexicon_tags_87 },
    { 0xb1d0b5ae, SUC_lexicon_tags_60 },
    { 0x2610b5b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13f0b5b2, SUC_lexicon_tags_2 },
    { 0x27f0b5b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xab30b5b6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150b5c1, SUC_lexicon_tags_38 },
    { 0x2f30b5c2, SUC_lexicon_tags_37 },
    { 0x3710b5c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b0b5c6, SUC_lexicon_tags_5 },
    { 0x2690b5c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x59b0b5c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70b5cd, SUC_lexicon_tags_19 },
    { 0xb0b5ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe570b5d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0b5d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6750b5df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4170b5e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f0b5e6, SUC_lexicon_tags_25 },
    { 0xba90b5e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x57b0b5e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83b0b5eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa650b5ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd670b5ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x26d0b5f1, SUC_lexicon_tags_26 },
    { 0x6b90b5f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3270b5f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9870b5fa, SUC_lexicon_tags_45 },
    { 0x5e50b5fb, SUC_lexicon_tags_11 },
    { 0x3bf0b5fb, SUC_lexicon_tags_23 },
    { 0x9b30b5fc, SUC_lexicon_tags_87 },
    { 0x4590b5fe, SUC_lexicon_tags_2 },
    { 0xadd0b5ff, SUC_lexicon_tags_15 },
    { 0xd70b5ff, SUC_lexicon_tags_7 },
    { 0xb2b0b601, SUC_lexicon_tags_0 },
    { 0x6330b5fc, SUC_lexicon_tags_6 },
    { 0x1b0b600, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2450b605, SUC_lexicon_tags_37 },
    { 0xc570b606, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6230b609, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9b30b60b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b0b60e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd350b610, SUC_lexicon_tags_15 },
    { 0x6650b611, SUC_lexicon_tags_1 },
    { 0xeab0b610, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f0b617, SUC_lexicon_tags_23 },
    { 0x38f0b618, SUC_lexicon_tags_15 },
    { 0x2310b617, SUC_lexicon_tags_7 },
    { 0x52b0b61a, SUC_lexicon_tags_37 },
    { 0xeb50b61b, SUC_lexicon_tags_37 },
    { 0x35f0b61a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc70b620, SUC_lexicon_tags_28 },
    { 0x64b0b621, SUC_lexicon_tags_11 },
    { 0xfad0b622, SUC_lexicon_tags_11 },
    { 0xf50b623, SUC_lexicon_tags_14 },
    { 0xc710b624, SUC_lexicon_tags_56 },
    { 0x3070b625, SUC_lexicon_tags_27 },
    { 0x87d0b625, SUC_lexicon_tags_1 },
    { 0x6eb0b622, SUC_lexicon_tags_30 },
    { 0xbe30b622, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xeab0b62a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6a90b62c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d50b62e, SUC_lexicon_tags_31 },
    { 0xde90b62e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b0b632, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d0b636, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b0b63f, SUC_lexicon_tags_32 },
    { 0x810b640, SUC_lexicon_tags_17 },
    { 0x7730b63f, SUC_lexicon_tags_22 },
    { 0xb0b0b642, SUC_lexicon_tags_140 },
    { 0x1170b640, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4350b647, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb130b649, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf50b64e, SUC_lexicon_tags_2 },
    { 0x7e70b64f, SUC_lexicon_tags_27 },
    { 0xed50b650, SUC_lexicon_tags_37 },
    { 0xaa10b651, SUC_lexicon_tags_17 },
    { 0xf70b651, SUC_lexicon_tags_63 },
    { 0x39d0b651, SUC_lexicon_tags_37 },
    { 0x63f0b654, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4170b656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d0b659, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8750b65e, SUC_lexicon_tags_15 },
    { 0xff50b65f, SUC_lexicon_tags_31 },
    { 0x5690b660, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f0b665, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf350b669, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfeb0b66b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b0b66f, SUC_lexicon_tags_1 },
    { 0x3370b670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f0b674, SUC_lexicon_tags_38 },
    { 0xb770b674, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5070b67b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d90b67d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa550b67f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef0b68d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8d0b68f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3f30b691, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b70b694, SUC_lexicon_tags_2 },
    { 0x42b0b694, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd0b697, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8690b69b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd890b69d, SUC_lexicon_tags_2 },
    { 0xf130b69e, SUC_lexicon_tags_12 },
    { 0x9db0b69f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d0b6a4, SUC_lexicon_tags_12 },
    { 0x6cf0b6a5, SUC_lexicon_tags_2 },
    { 0x31f0b6a6, SUC_lexicon_tags_57 },
    { 0x7150b6a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20d0b6a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb0b6ac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x630b6b0, SUC_lexicon_tags_27 },
    { 0x1cb0b6b1, SUC_lexicon_tags_37 },
    { 0xc7d0b6b2, SUC_lexicon_tags_29 },
    { 0x82f0b6b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4130b6b5, SUC_lexicon_tags_87 },
    { 0x57d0b6b6, SUC_lexicon_tags_1 },
    { 0x9610b6b6, SUC_lexicon_tags_37 },
    { 0x470b6b8, SUC_lexicon_tags_37 },
    { 0xfc10b6b9, SUC_lexicon_tags_11 },
    { 0x3d0b6b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e50b6c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6430b6c8, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0xee30b6ca, SUC_lexicon_tags_7 },
    { 0x50b0b6cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2690b6ce, SUC_lexicon_tags_15 },
    { 0x7f30b6ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe710b6d2, SUC_lexicon_tags_14 },
    { 0x8890b6d3, SUC_lexicon_tags_28 },
    { 0x35f0b6d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0f0b6d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e90b6d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e70b6e1, SUC_lexicon_tags_37 },
    { 0x9810b6e2, SUC_lexicon_tags_23 },
    { 0xc750b6e3, SUC_lexicon_tags_11 },
    { 0xfff0b6e4, SUC_lexicon_tags_7 },
    { 0xfc90b6e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff70b6e7, SUC_lexicon_tags_15 },
    { 0x3b30b6e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b0b6ec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5d90b6ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x96b0b6f0, SUC_lexicon_tags_151 },
    { 0x8a10b6f0, SUC_lexicon_tags_4 },
    { 0xa650b6f2, SUC_lexicon_tags_25 },
    { 0xe510b6f3, SUC_lexicon_tags_2 },
    { 0xcf30b6f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4910b6fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf070b6fd, SUC_lexicon_tags_37 },
    { 0x30b0b6fe, SUC_lexicon_tags_18 },
    { 0xf950b6ff, SUC_lexicon_tags_7 },
    { 0x7170b6ff, SUC_lexicon_tags_12 },
    { 0xe5b0b701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf0b708, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e50b70a, SUC_lexicon_tags_98 },
    { 0x2ff0b70a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc10b70d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9850b712, SUC_lexicon_tags_12 },
    { 0x7dd0b713, SUC_lexicon_tags_12 },
    { 0xc010b714, SUC_lexicon_tags_12 },
    { 0xf930b715, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5e90b717, SUC_lexicon_tags_2 },
    { 0x2830b718, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4db0b71a, SUC_lexicon_tags_26 },
    { 0xaad0b71b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d0b72f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21b0b731, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b0b735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c10b739, SUC_lexicon_tags_45 },
    { 0x2e10b739, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd110b73d, SUC_lexicon_tags_56 },
    { 0xe90b73e, SUC_lexicon_tags_150 },
    { 0x2830b73f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xea30b741, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec70b745, SUC_lexicon_tags_25 },
    { 0xac90b746, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7f70b748, SUC_lexicon_tags_45 },
    { 0xe6d0b748, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x410b74c, SUC_lexicon_tags_9 },
    { 0xb1d0b74d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d0b752, SUC_lexicon_tags_34 },
    { 0xec70b753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfaf0b755, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4210b75a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f0b75d, SUC_lexicon_tags_37 },
    { 0xbd70b75e, SUC_lexicon_tags_19 },
    { 0xdeb0b75f, SUC_lexicon_tags_1 },
    { 0x5d30b760, SUC_lexicon_tags_17 },
    { 0x5d30b75e, SUC_lexicon_tags_1 },
    { 0xe830b762, SUC_lexicon_tags_7 },
    { 0x7270b763, SUC_lexicon_tags_13 },
    { 0x2710b764, SUC_lexicon_tags_11 },
    { 0xdd0b765, SUC_lexicon_tags_15 },
    { 0x1d0b763, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4290b768, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe870b76b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6290b76d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e10b777, SUC_lexicon_tags_23 },
    { 0x2170b778, SUC_lexicon_tags_11 },
    { 0x5290b779, SUC_lexicon_tags_23 },
    { 0xcaf0b77a, SUC_lexicon_tags_37 },
    { 0x3910b77a, SUC_lexicon_tags_2 },
    { 0x7110b77c, SUC_lexicon_tags_15 },
    { 0xd050b77c, SUC_lexicon_tags_1 },
    { 0x2790b77a, SUC_lexicon_tags_68 },
    { 0xcef0b77b, SUC_lexicon_tags_43 },
    { 0x90b77f, SUC_lexicon_tags_13 },
    { 0x4af0b781, SUC_lexicon_tags_45 },
    { 0xcb90b77e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf30b784, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef0b789, SUC_lexicon_tags_17 },
    { 0xe030b78a, SUC_lexicon_tags_11 },
    { 0x4570b78a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d0b78f, SUC_lexicon_tags_37 },
    { 0xf750b78f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f0b796, SUC_lexicon_tags_28 },
    { 0x6310b797, SUC_lexicon_tags_6 },
    { 0xc270b798, SUC_lexicon_tags_9 },
    { 0x5f30b798, SUC_lexicon_tags_47 },
    { 0x9850b798, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970b7a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8410b7a4, SUC_lexicon_tags_25 },
    { 0x410b7a4, SUC_lexicon_tags_45 },
    { 0x19b0b7a4, SUC_lexicon_tags_307 },
    { 0xb090b7a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2530b7ac, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x5610b7ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d0b7b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7750b7b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaeb0b7b6, SUC_lexicon_tags_37 },
    { 0x8b30b7b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec70b7bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xca70b7be, SUC_lexicon_tags_65 },
    { 0x9ef0b7bf, SUC_lexicon_tags_14 },
    { 0xf850b7c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2710b7c4, SUC_lexicon_tags_17 },
    { 0x5cb0b7c5, SUC_lexicon_tags_61 },
    { 0x48d0b7c6, SUC_lexicon_tags_25 },
    { 0x6fb0b7c7, SUC_lexicon_tags_25 },
    { 0xab0b7c5, SUC_lexicon_tags_12 },
    { 0x71f0b7c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa7b0b7cb, SUC_lexicon_tags_536 },
    { 0xe3b0b7cb, SUC_lexicon_tags_63 },
    { 0x5d50b7cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa3d0b7cf, SUC_lexicon_tags_2 },
    { 0x2730b7d0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b50b7d6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7070b7d8, SUC_lexicon_tags_38 },
    { 0xe4d0b7d8, SUC_lexicon_tags_17 },
    { 0xe650b7da, SUC_lexicon_tags_37 },
    { 0xc490b7d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9710b7df, SUC_lexicon_tags_7 },
    { 0x3070b7e0, SUC_lexicon_tags_15 },
    { 0x4fd0b7e1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3c90b7e3, SUC_lexicon_tags_28 },
    { 0x8ad0b7e4, SUC_lexicon_tags_25 },
    { 0x68d0b7e5, SUC_lexicon_tags_27 },
    { 0xe9f0b7e6, SUC_lexicon_tags_37 },
    { 0xe670b7e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5470b7ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d70b7ef, SUC_lexicon_tags_37 },
    { 0x4e10b7ef, SUC_lexicon_tags_19 },
    { 0x2b70b7f0, SUC_lexicon_tags_6 },
    { 0xb030b7f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3870b7f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b50b7fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2630b7fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29d0b800, SUC_lexicon_tags_1 },
    { 0xaeb0b801, SUC_lexicon_tags_74 },
    { 0x9a30b802, SUC_lexicon_tags_60 },
    { 0x8df0b803, SUC_lexicon_tags_282 },
    { 0x1790b802, SUC_lexicon_tags_14 },
    { 0x90b805, SUC_lexicon_tags_29 },
    { 0x30b0b806, SUC_lexicon_tags_11 },
    { 0x1d10b807, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed0b80b, SUC_lexicon_tags_33 },
    { 0x9d50b80b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd030b80f, SUC_lexicon_tags_1 },
    { 0x1f0b80f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4190b812, SUC_lexicon_tags_82 },
    { 0x64b0b813, SUC_lexicon_tags_48 },
    { 0x17f0b812, SUC_lexicon_tags_33 },
    { 0xcfd0b815, SUC_lexicon_tags_31 },
    { 0x95d0b815, SUC_lexicon_tags_43 },
    { 0x3b30b813, SUC_lexicon_tags_37 },
    { 0x58d0b818, SUC_lexicon_tags_15 },
    { 0xbaf0b819, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8370b81b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8670b81d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bd0b81f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a50b826, SUC_lexicon_tags_43 },
    { 0x3930b827, SUC_lexicon_tags_10 },
    { 0x6b90b828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f0b82b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcc50b82d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc90b831, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed0b835, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb0b838, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f30b83d, SUC_lexicon_tags_64 },
    { 0x9ed0b83d, SUC_lexicon_tags_19 },
    { 0x1070b83f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd830b842, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81f0b844, SUC_lexicon_tags_0 },
    { 0x7190b845, SUC_lexicon_tags_28 },
    { 0x3270b845, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x790b848, SUC_lexicon_tags_25 },
    { 0xb6d0b849, SUC_lexicon_tags_21 },
    { 0x5010b849, SUC_lexicon_tags_193 },
    { 0x8050b849, SUC_lexicon_tags_12 },
    { 0x950b84c, SUC_lexicon_tags_26 },
    { 0x65f0b84d, SUC_lexicon_tags_17 },
    { 0x44f0b84c, SUC_lexicon_tags_84 },
    { 0x9930b84f, SUC_lexicon_tags_7 },
    { 0x410b84f, SUC_lexicon_tags_66 },
    { 0xbef0b84a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x49b0b853, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xab0b855, SUC_lexicon_tags_7 },
    { 0xd150b855, SUC_lexicon_tags_29 },
    { 0xc890b857, SUC_lexicon_tags_15 },
    { 0x15b0b858, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8230b85a, SUC_lexicon_tags_49 },
    { 0x4b30b85a, SUC_lexicon_tags_12 },
    { 0xea90b85c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc010b85e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5870b861, SUC_lexicon_tags_0 },
    { 0x2730b862, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3390b867, SUC_lexicon_tags_63 },
    { 0xb110b867, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb630b86c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f0b870, SUC_lexicon_tags_11 },
    { 0xc10b871, SUC_lexicon_tags_15 },
    { 0x99b0b871, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda30b874, SUC_lexicon_tags_49 },
    { 0x1810b875, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x490b878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb070b87a, SUC_lexicon_tags_27 },
    { 0x2090b87b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d0b880, SUC_lexicon_tags_29 },
    { 0x6430b880, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87f0b883, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ef0b885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc510b887, SUC_lexicon_tags_47 },
    { 0x71d0b888, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb0b88c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b0b890, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5590b892, SUC_lexicon_tags_66 },
    { 0x7830b892, SUC_lexicon_tags_26 },
    { 0xc650b892, SUC_lexicon_tags_23 },
    { 0xfc90b895, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xd810b898, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf350b89e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97b0b8a0, SUC_lexicon_tags_11 },
    { 0x3250b8a1, SUC_lexicon_tags_11 },
    { 0x5090b8a0, SUC_lexicon_tags_15 },
    { 0x1d10b8a3, SUC_lexicon_tags_25 },
    { 0x8f70b8a4, SUC_lexicon_tags_7 },
    { 0x3550b8a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4ed0b8a7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db0b8ae, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1650b8b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce90b8bc, SUC_lexicon_tags_1 },
    { 0x39f0b8bc, SUC_lexicon_tags_13 },
    { 0xc010b8be, SUC_lexicon_tags_7 },
    { 0xba30b8bf, SUC_lexicon_tags_25 },
    { 0xc8d0b8be, SUC_lexicon_tags_29 },
    { 0x7bb0b8c0, SUC_lexicon_tags_21 },
    { 0x8710b8c2, SUC_lexicon_tags_25 },
    { 0x1030b8c3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f0b8c6, SUC_lexicon_tags_25 },
    { 0x7490b8c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7df0b8c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43d0b8cb, SUC_lexicon_tags_25 },
    { 0x6c10b8cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b0b8d0, SUC_lexicon_tags_11 },
    { 0x1450b8d1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8670b8d3, SUC_lexicon_tags_57 },
    { 0xd10b8d4, SUC_lexicon_tags_3 },
    { 0xdfb0b8d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bb0b8d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f30b8e0, SUC_lexicon_tags_15 },
    { 0x7f50b8e0, SUC_lexicon_tags_11 },
    { 0x4b70b8e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc990b8e5, SUC_lexicon_tags_27 },
    { 0xcb30b8e6, SUC_lexicon_tags_198 },
    { 0x50b0b8e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d30b8ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x97b0b8ee, SUC_lexicon_tags_1 },
    { 0xb450b8ef, SUC_lexicon_tags_1 },
    { 0xed10b8ee, SUC_lexicon_tags_2 },
    { 0xea10b8f0, SUC_lexicon_tags_2 },
    { 0x7d30b8f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe30b8f4, SUC_lexicon_tags_7 },
    { 0xbc10b8f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f90b8f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4910b8f9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d0b8ff, SUC_lexicon_tags_66 },
    { 0x7f0b8ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6670b902, SUC_lexicon_tags_38 },
    { 0x3150b902, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb0b907, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9c90b909, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd070b90f, SUC_lexicon_tags_38 },
    { 0xe3d0b910, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca10b913, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe310b915, SUC_lexicon_tags_65 },
    { 0xad30b916, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f0b919, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb50b91c, SUC_lexicon_tags_45 },
    { 0x8770b91c, SUC_lexicon_tags_1 },
    { 0x5230b91e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde90b927, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9210b92a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2d0b92c, SUC_lexicon_tags_37 },
    { 0x8e50b92d, SUC_lexicon_tags_37 },
    { 0xbf90b92c, SUC_lexicon_tags_12 },
    { 0xc050b92c, SUC_lexicon_tags_23 },
    { 0xc4f0b930, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b30b933, SUC_lexicon_tags_6 },
    { 0xb0b0b934, SUC_lexicon_tags_5 },
    { 0x98f0b935, SUC_lexicon_tags_0 },
    { 0xd930b933, SUC_lexicon_tags_25 },
    { 0x80d0b937, SUC_lexicon_tags_7 },
    { 0xda10b938, SUC_lexicon_tags_25 },
    { 0x9e50b938, SUC_lexicon_tags_107 },
    { 0x9e70b938, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf410b93c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2850b93e, SUC_lexicon_tags_11 },
    { 0xdb70b93f, SUC_lexicon_tags_17 },
    { 0xad30b93f, SUC_lexicon_tags_45 },
    { 0x9650b93e, SUC_lexicon_tags_7 },
    { 0xb70b941, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f0b951, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe430b954, SUC_lexicon_tags_10 },
    { 0xaaf0b955, SUC_lexicon_tags_0 },
    { 0x4930b956, SUC_lexicon_tags_5 },
    { 0x4b0b957, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a30b95c, SUC_lexicon_tags_13 },
    { 0xf5d0b95c, SUC_lexicon_tags_17 },
    { 0x7a30b95c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79f0b960, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb0b964, SUC_lexicon_tags_17 },
    { 0x73d0b964, SUC_lexicon_tags_37 },
    { 0x74b0b966, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17d0b968, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f30b96d, SUC_lexicon_tags_2 },
    { 0x7f90b96d, SUC_lexicon_tags_8 },
    { 0x29f0b96f, SUC_lexicon_tags_31 },
    { 0x87f0b96f, SUC_lexicon_tags_12 },
    { 0x8e50b96f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeed0b973, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed10b97c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb630b97e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f0b981, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe990b984, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x72f0b986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd270b989, SUC_lexicon_tags_25 },
    { 0x2b10b989, SUC_lexicon_tags_23 },
    { 0x8d50b98a, SUC_lexicon_tags_15 },
    { 0x6b90b98b, SUC_lexicon_tags_15 },
    { 0xac90b98d, SUC_lexicon_tags_37 },
    { 0x3270b98e, SUC_lexicon_tags_0 },
    { 0xb4d0b98b, SUC_lexicon_tags_25 },
    { 0xa7b0b989, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x430b994, SUC_lexicon_tags_17 },
    { 0x3390b995, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab0b999, SUC_lexicon_tags_17 },
    { 0x430b99a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee90b99d, SUC_lexicon_tags_7 },
    { 0xe110b99d, SUC_lexicon_tags_60 },
    { 0x76d0b99f, SUC_lexicon_tags_43 },
    { 0xa1f0b9a0, SUC_lexicon_tags_49 },
    { 0x6590b9a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1930b9a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc650b9a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9850b9aa, SUC_lexicon_tags_55 },
    { 0x32f0b9aa, SUC_lexicon_tags_6 },
    { 0x7e70b9aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f0b9af, SUC_lexicon_tags_2 },
    { 0xaef0b9b0, SUC_lexicon_tags_32 },
    { 0xdcf0b9b1, SUC_lexicon_tags_0 },
    { 0xcfb0b9b2, SUC_lexicon_tags_37 },
    { 0xb590b9b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7730b9ba, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x7e10b9bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5190b9be, SUC_lexicon_tags_14 },
    { 0x2730b9bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed0b9c5, SUC_lexicon_tags_37 },
    { 0x4430b9c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d0b9ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a70b9ce, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1490b9d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b0b9d7, SUC_lexicon_tags_31 },
    { 0x97b0b9d7, SUC_lexicon_tags_33 },
    { 0x9010b9d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x30d0b9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded0b9e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3d70b9e3, SUC_lexicon_tags_17 },
    { 0xfaf0b9e4, SUC_lexicon_tags_26 },
    { 0x1990b9e5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90b9e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe130b9eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d90b9f0, SUC_lexicon_tags_44 },
    { 0x32f0b9f1, SUC_lexicon_tags_15 },
    { 0xda50b9f1, SUC_lexicon_tags_37 },
    { 0x5eb0b9f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb50b9f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b30b9fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9310b9fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b0ba01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe690ba04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8350ba08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af0ba0c, SUC_lexicon_tags_25 },
    { 0x29b0ba0d, SUC_lexicon_tags_38 },
    { 0x1f90ba0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f50ba10, SUC_lexicon_tags_10 },
    { 0xbef0ba11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4130ba13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a10ba16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f0ba19, SUC_lexicon_tags_9 },
    { 0x43d0ba19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a50ba1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2350ba20, SUC_lexicon_tags_4 },
    { 0x3010ba20, SUC_lexicon_tags_25 },
    { 0xed10ba22, SUC_lexicon_tags_14 },
    { 0xee50ba22, SUC_lexicon_tags_42 },
    { 0x8630ba24, SUC_lexicon_tags_34 },
    { 0x5c30ba24, SUC_lexicon_tags_6 },
    { 0xc6d0ba26, SUC_lexicon_tags_31 },
    { 0x8d90ba26, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af0ba2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d90ba2d, SUC_lexicon_tags_27 },
    { 0x6bb0ba2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7010ba31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65f0ba33, SUC_lexicon_tags_27 },
    { 0x2db0ba34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9290ba36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b10ba3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x74d0ba3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfbb0ba3f, SUC_lexicon_tags_11 },
    { 0x7d90ba40, SUC_lexicon_tags_23 },
    { 0xec90ba3f, SUC_lexicon_tags_7 },
    { 0xe2d0ba42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4830ba47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3930ba54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc350ba56, SUC_lexicon_tags_47 },
    { 0x8770ba57, SUC_lexicon_tags_37 },
    { 0x7ef0ba57, SUC_lexicon_tags_33 },
    { 0x66d0ba59, SUC_lexicon_tags_9 },
    { 0xb9d0ba56, SUC_lexicon_tags_11 },
    { 0xaa90ba59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd990ba5d, SUC_lexicon_tags_25 },
    { 0xeef0ba5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfb0ba60, SUC_lexicon_tags_45 },
    { 0x8830ba60, SUC_lexicon_tags_38 },
    { 0xfb30ba60, SUC_lexicon_tags_35 },
    { 0xe030ba63, SUC_lexicon_tags_17 },
    { 0xb1d0ba64, SUC_lexicon_tags_37 },
    { 0x16d0ba65, SUC_lexicon_tags_15 },
    { 0xded0ba65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df0ba6a, SUC_lexicon_tags_15 },
    { 0xc270ba6b, SUC_lexicon_tags_56 },
    { 0x350ba6b, SUC_lexicon_tags_2 },
    { 0x12b0ba6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7590ba73, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x35b0ba75, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f90ba78, SUC_lexicon_tags_17 },
    { 0xeb0ba78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3930ba83, SUC_lexicon_tags_15 },
    { 0xbc90ba84, SUC_lexicon_tags_11 },
    { 0x8330ba85, SUC_lexicon_tags_38 },
    { 0x4a70ba85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb90ba8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e50ba8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2290ba92, SUC_lexicon_tags_27 },
    { 0xf1b0ba92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb490ba95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa30ba97, SUC_lexicon_tags_11 },
    { 0x98b0ba98, SUC_lexicon_tags_19 },
    { 0x38f0ba99, SUC_lexicon_tags_65 },
    { 0x4d70ba99, SUC_lexicon_tags_7 },
    { 0x3dd0ba9b, SUC_lexicon_tags_31 },
    { 0x6b70ba9a, SUC_lexicon_tags_7 },
    { 0xe4b0ba99, SUC_lexicon_tags_12 },
    { 0x7f30ba9e, SUC_lexicon_tags_55 },
    { 0x5350ba9f, SUC_lexicon_tags_2 },
    { 0x570ba9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f0baa3, SUC_lexicon_tags_21 },
    { 0x1250baa4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c90baaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf010baae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2f0bab0, SUC_lexicon_tags_8 },
    { 0x26b0bab1, SUC_lexicon_tags_77 },
    { 0x6750bab1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcef0bab4, SUC_lexicon_tags_8 },
    { 0xc6d0bab5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b30bab9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9670babc, SUC_lexicon_tags_11 },
    { 0x77d0babc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2470bac2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf0bac5, SUC_lexicon_tags_21 },
    { 0x3b10bac6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc630bac9, SUC_lexicon_tags_74 },
    { 0x86d0bac9, SUC_lexicon_tags_14 },
    { 0x66d0bacb, SUC_lexicon_tags_47 },
    { 0x950bacb, SUC_lexicon_tags_15 },
    { 0x5ab0bac9, SUC_lexicon_tags_17 },
    { 0xe4d0bacc, SUC_lexicon_tags_33 },
    { 0x8910bacf, SUC_lexicon_tags_27 },
    { 0xead0bad0, SUC_lexicon_tags_0 },
    { 0x2990bad1, SUC_lexicon_tags_17 },
    { 0x69b0bad2, SUC_lexicon_tags_7 },
    { 0xca70bad0, SUC_lexicon_tags_1 },
    { 0xf770bad4, SUC_lexicon_tags_7 },
    { 0xb810bad5, SUC_lexicon_tags_7 },
    { 0x7250bad5, SUC_lexicon_tags_37 },
    { 0x4810bad7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf590bad9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6290badc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1630bade, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced0bae2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b0bae9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7f30baeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6230baee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7250baf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49f0baf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d0baf8, SUC_lexicon_tags_7 },
    { 0x5630baf9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d0bafc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1f50bafe, SUC_lexicon_tags_21 },
    { 0xed70baff, SUC_lexicon_tags_8 },
    { 0x4770bb00, SUC_lexicon_tags_6 },
    { 0x4f10baff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe990bb03, SUC_lexicon_tags_7 },
    { 0x9a90bb04, SUC_lexicon_tags_25 },
    { 0xae30bb05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3990bb0b, SUC_lexicon_tags_7 },
    { 0x7a30bb0c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x7570bb0f, SUC_lexicon_tags_5 },
    { 0xda30bb10, SUC_lexicon_tags_12 },
    { 0x8930bb11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc70bb13, SUC_lexicon_tags_25 },
    { 0xb790bb14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf0bb1a, SUC_lexicon_tags_6 },
    { 0xd5b0bb1a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xdf30bb1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7090bb22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ff0bb24, SUC_lexicon_tags_25 },
    { 0xd2b0bb25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb690bb2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e50bb2f, SUC_lexicon_tags_30 },
    { 0x50bb30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8d0bb32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe310bb35, SUC_lexicon_tags_55 },
    { 0x87f0bb35, SUC_lexicon_tags_15 },
    { 0xd270bb37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd470bb3c, SUC_lexicon_tags_12 },
    { 0x7e30bb3d, SUC_lexicon_tags_25 },
    { 0x88b0bb3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4790bb40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x490bb43, SUC_lexicon_tags_37 },
    { 0xb4d0bb43, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6fb0bb46, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d0bb49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb030bb4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7370bb50, SUC_lexicon_tags_47 },
    { 0xbeb0bb51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4b0bb53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f0bb56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfc70bb58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f0bb5b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5490bb5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9710bb61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xac90bb64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b0bb67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50bb6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b0bb6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe10bb73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x13f0bb75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0f0bb77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6470bb7e, SUC_lexicon_tags_53 },
    { 0x2350bb7e, SUC_lexicon_tags_10 },
    { 0x3210bb80, SUC_lexicon_tags_87 },
    { 0x9c50bb81, SUC_lexicon_tags_2 },
    { 0x410bb7e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed30bb86, SUC_lexicon_tags_25 },
    { 0x17f0bb87, SUC_lexicon_tags_47 },
    { 0xc190bb86, SUC_lexicon_tags_58 },
    { 0xae70bb88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b10bb8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d0bb8f, SUC_lexicon_tags_7 },
    { 0x2a70bb8f, SUC_lexicon_tags_29 },
    { 0x1270bb90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8b0bb93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd90bb97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d0bba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca10bba5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc010bbaa, SUC_lexicon_tags_1 },
    { 0xc930bbab, SUC_lexicon_tags_25 },
    { 0xb0f0bbac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c10bbae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d0bbb5, SUC_lexicon_tags_14 },
    { 0x2db0bbb5, SUC_lexicon_tags_47 },
    { 0xc130bbb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf850bbb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xded0bbbb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0bbbe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd790bbc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf510bbc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8490bbcf, SUC_lexicon_tags_9 },
    { 0x2570bbd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50bbd3, SUC_lexicon_tags_28 },
    { 0x6690bbd3, SUC_lexicon_tags_27 },
    { 0x6390bbd5, SUC_lexicon_tags_17 },
    { 0x3290bbd6, SUC_lexicon_tags_63 },
    { 0xba50bbd4, SUC_lexicon_tags_37 },
    { 0x7690bbd8, SUC_lexicon_tags_64 },
    { 0xccb0bbd8, SUC_lexicon_tags_15 },
    { 0x2490bbda, SUC_lexicon_tags_11 },
    { 0xacb0bbda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd450bbdf, SUC_lexicon_tags_31 },
    { 0x3690bbdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e90bbe4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf70bbeb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa150bbed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d0bbf2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4870bbf6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2490bbf8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf130bbfb, SUC_lexicon_tags_37 },
    { 0x2690bbfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa70bbfe, SUC_lexicon_tags_393 },
    { 0x570bbff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9470bc0a, SUC_lexicon_tags_12 },
    { 0x4b90bc0b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2d70bc0d, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0x7370bc0f, SUC_lexicon_tags_38 },
    { 0xeff0bc0f, SUC_lexicon_tags_1 },
    { 0xc1b0bc0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d0bc14, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1710bc18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1050bc1c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b0bc20, SUC_lexicon_tags_8 },
    { 0x2b50bc20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f0bc28, SUC_lexicon_tags_37 },
    { 0x34d0bc29, SUC_lexicon_tags_70 },
    { 0x6ef0bc2a, SUC_lexicon_tags_28 },
    { 0x9610bc28, SUC_lexicon_tags_69 },
    { 0xfc30bc2c, SUC_lexicon_tags_11 },
    { 0x2e30bc2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f10bc2f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf050bc33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x16f0bc35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d30bc38, SUC_lexicon_tags_12 },
    { 0xa590bc39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b0bc3d, SUC_lexicon_tags_11 },
    { 0x6930bc3e, SUC_lexicon_tags_28 },
    { 0xefd0bc3f, SUC_lexicon_tags_6 },
    { 0x4910bc40, SUC_lexicon_tags_2 },
    { 0x59f0bc3d, SUC_lexicon_tags_32 },
    { 0x6d50bc42, SUC_lexicon_tags_10 },
    { 0x7e90bc43, SUC_lexicon_tags_23 },
    { 0xf130bc43, SUC_lexicon_tags_30 },
    { 0xa3d0bc3d, SUC_lexicon_tags_39 },
    { 0xaf90bc46, SUC_lexicon_tags_27 },
    { 0x1830bc46, SUC_lexicon_tags_1 },
    { 0x9830bc42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3690bc4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2910bc4f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaef0bc51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48d0bc53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b0bc58, SUC_lexicon_tags_16 },
    { 0x6730bc58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1150bc5d, SUC_lexicon_tags_25 },
    { 0xbc30bc5e, SUC_lexicon_tags_66 },
    { 0xbc10bc5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330bc63, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c70bc67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef30bc6d, SUC_lexicon_tags_6 },
    { 0xcdb0bc6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4890bc73, SUC_lexicon_tags_68 },
    { 0x6f30bc74, SUC_lexicon_tags_37 },
    { 0xcbb0bc75, SUC_lexicon_tags_14 },
    { 0x3f70bc74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb150bc78, SUC_lexicon_tags_46 },
    { 0xa850bc78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf710bc82, SUC_lexicon_tags_44 },
    { 0x4750bc83, SUC_lexicon_tags_19 },
    { 0x9530bc82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8d0bc86, SUC_lexicon_tags_63 },
    { 0xea10bc87, SUC_lexicon_tags_0 },
    { 0x23d0bc88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd890bc8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba90bc90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd030bc92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91b0bc94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb50bc9b, SUC_lexicon_tags_29 },
    { 0xb3f0bc9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b0bc9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3b0bca1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc030bca4, SUC_lexicon_tags_25 },
    { 0xa9f0bca4, SUC_lexicon_tags_15 },
    { 0x3bf0bca4, SUC_lexicon_tags_0 },
    { 0xb5f0bca7, SUC_lexicon_tags_25 },
    { 0xb390bca8, SUC_lexicon_tags_85 },
    { 0xd530bca9, SUC_lexicon_tags_48 },
    { 0xbc10bcaa, SUC_lexicon_tags_28 },
    { 0xfd70bcab, SUC_lexicon_tags_13 },
    { 0xc710bcac, SUC_lexicon_tags_1 },
    { 0x1410bcad, SUC_lexicon_tags_60 },
    { 0xa3d0bcae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7170bcb0, SUC_lexicon_tags_2 },
    { 0x2790bcb1, SUC_lexicon_tags_17 },
    { 0x55b0bcb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b0bcba, SUC_lexicon_tags_60 },
    { 0xbdf0bcbb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6870bcbe, SUC_lexicon_tags_16 },
    { 0xc830bcbe, SUC_lexicon_tags_53 },
    { 0x4430bcbe, SUC_lexicon_tags_37 },
    { 0xead0bcc0, SUC_lexicon_tags_32 },
    { 0x9ed0bcc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f0bcc6, SUC_lexicon_tags_79 },
    { 0x4670bcc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda70bcc9, SUC_lexicon_tags_7 },
    { 0x4410bcc9, SUC_lexicon_tags_11 },
    { 0x1e50bccb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d0bcce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9550bcd1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1730bcdb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe290bce2, SUC_lexicon_tags_37 },
    { 0x75b0bce2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b0bce8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d10bcee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x330bcf6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9b70bcf8, SUC_lexicon_tags_45 },
    { 0xeb0bcf9, SUC_lexicon_tags_38 },
    { 0xeb10bcfa, SUC_lexicon_tags_14 },
    { 0xfd0bcfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa970bcfe, SUC_lexicon_tags_17 },
    { 0x9cf0bcff, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xff70bd02, SUC_lexicon_tags_7 },
    { 0xa770bd03, SUC_lexicon_tags_45 },
    { 0x3b0bd04, SUC_lexicon_tags_38 },
    { 0xb110bd04, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1a90bd07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50bd0a, SUC_lexicon_tags_44 },
    { 0x94b0bd0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c50bd0f, SUC_lexicon_tags_1 },
    { 0x1790bd10, SUC_lexicon_tags_15 },
    { 0xc990bd10, SUC_lexicon_tags_14 },
    { 0xfd0bd11, SUC_lexicon_tags_55 },
    { 0xd970bd11, SUC_lexicon_tags_11 },
    { 0x4d0bd14, SUC_lexicon_tags_37 },
    { 0x3430bd15, SUC_lexicon_tags_7 },
    { 0x8fd0bd16, SUC_lexicon_tags_25 },
    { 0xc6d0bd16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f50bd1c, SUC_lexicon_tags_45 },
    { 0x7a30bd1d, SUC_lexicon_tags_15 },
    { 0x4890bd1e, SUC_lexicon_tags_17 },
    { 0x9d70bd1e, SUC_lexicon_tags_28 },
    { 0x9ff0bd1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f30bd22, SUC_lexicon_tags_11 },
    { 0xd1b0bd23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc570bd27, SUC_lexicon_tags_60 },
    { 0xf510bd28, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe830bd2a, SUC_lexicon_tags_6 },
    { 0x1790bd2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4730bd2d, SUC_lexicon_tags_11 },
    { 0xa270bd2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa970bd32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1410bd36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9310bd3f, SUC_lexicon_tags_68 },
    { 0x68f0bd40, SUC_lexicon_tags_7 },
    { 0x1410bd41, SUC_lexicon_tags_38 },
    { 0xc3d0bd42, SUC_lexicon_tags_2 },
    { 0x19b0bd42, SUC_lexicon_tags_12 },
    { 0x9130bd44, SUC_lexicon_tags_1 },
    { 0x5090bd43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31b0bd47, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d0bd4a, SUC_lexicon_tags_12 },
    { 0x75f0bd4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae70bd4d, SUC_lexicon_tags_7 },
    { 0x8c70bd4e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd0bd52, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf0bd55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa050bd57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650bd60, SUC_lexicon_tags_96 },
    { 0xbc90bd60, SUC_lexicon_tags_39 },
    { 0xf1f0bd62, SUC_lexicon_tags_1 },
    { 0x4130bd63, SUC_lexicon_tags_11 },
    { 0x37d0bd60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9550bd67, SUC_lexicon_tags_6 },
    { 0xabf0bd68, SUC_lexicon_tags_31 },
    { 0x5db0bd67, SUC_lexicon_tags_17 },
    { 0x90f0bd6a, SUC_lexicon_tags_17 },
    { 0x3b30bd6a, SUC_lexicon_tags_64 },
    { 0x9130bd6c, SUC_lexicon_tags_47 },
    { 0x8350bd6b, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f30bd72, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b70bd75, SUC_lexicon_tags_25 },
    { 0xa490bd76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f0bd7a, SUC_lexicon_tags_17 },
    { 0xa250bd7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f0bd86, SUC_lexicon_tags_15 },
    { 0x66f0bd86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3010bd89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5510bd8b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd110bd8e, SUC_lexicon_tags_38 },
    { 0x52f0bd8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf490bd91, SUC_lexicon_tags_11 },
    { 0x55b0bd91, SUC_lexicon_tags_37 },
    { 0x1cd0bd92, SUC_lexicon_tags_17 },
    { 0x7870bd94, SUC_lexicon_tags_8 },
    { 0x3eb0bd95, SUC_lexicon_tags_45 },
    { 0x9050bd93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5930bd9a, SUC_lexicon_tags_31 },
    { 0xdc70bd9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xacb0bd9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb50bd9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb510bda1, SUC_lexicon_tags_11 },
    { 0xd290bda2, SUC_lexicon_tags_27 },
    { 0xc6f0bda3, SUC_lexicon_tags_14 },
    { 0xa730bda3, SUC_lexicon_tags_58 },
    { 0xedd0bda3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff50bdaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c50bdb0, SUC_lexicon_tags_11 },
    { 0x8c90bdb0, SUC_lexicon_tags_37 },
    { 0xfa10bdb2, SUC_lexicon_tags_17 },
    { 0x62f0bdb2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd30bdb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd0bdbb, SUC_lexicon_tags_1 },
    { 0x3950bdbc, SUC_lexicon_tags_9 },
    { 0x2a90bdbd, SUC_lexicon_tags_4 },
    { 0xd150bdbe, SUC_lexicon_tags_17 },
    { 0xe290bdbf, SUC_lexicon_tags_37 },
    { 0xac30bdbb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f0bdc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xecf0bdc5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9350bdc8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f0bdcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf30bdcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe50bdd5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b0bdd9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b0bdde, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc90bde1, SUC_lexicon_tags_65 },
    { 0xb810bde2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e50bde6, SUC_lexicon_tags_7 },
    { 0xabd0bde6, SUC_lexicon_tags_63 },
    { 0x9f70bde6, SUC_lexicon_tags_9 },
    { 0x3bf0bde9, SUC_lexicon_tags_11 },
    { 0xd2d0bdea, SUC_lexicon_tags_47 },
    { 0x7c30bde9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3390bdf6, SUC_lexicon_tags_17 },
    { 0xda50bdf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc650bdfc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6250bdfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f90be02, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x62b0be04, SUC_lexicon_tags_43 },
    { 0x28b0be05, SUC_lexicon_tags_12 },
    { 0xfdb0be06, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf2f0be08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x33b0be0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7950be0d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd050be10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e30be12, SUC_lexicon_tags_1 },
    { 0x5870be13, SUC_lexicon_tags_46 },
    { 0x46f0be14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a30be19, SUC_lexicon_tags_82 },
    { 0x4370be1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b0be1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb490be21, SUC_lexicon_tags_10 },
    { 0x9530be22, SUC_lexicon_tags_2 },
    { 0xd6b0be23, SUC_lexicon_tags_12 },
    { 0xf30be22, SUC_lexicon_tags_25 },
    { 0x7b0be22, SUC_lexicon_tags_82 },
    { 0x9fb0be25, SUC_lexicon_tags_1 },
    { 0x7ff0be26, SUC_lexicon_tags_15 },
    { 0xd110be28, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e10be2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b0be2e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x82b0be30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddf0be32, SUC_lexicon_tags_12 },
    { 0x4330be33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3910be35, SUC_lexicon_tags_64 },
    { 0xf4d0be35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb270be39, SUC_lexicon_tags_95 },
    { 0xca90be3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc870be3c, SUC_lexicon_tags_16 },
    { 0xa8f0be3d, SUC_lexicon_tags_25 },
    { 0x2050be3e, SUC_lexicon_tags_37 },
    { 0x4490be3f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9410be41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b50be49, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6230be4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1550be4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4f0be51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75d0be53, SUC_lexicon_tags_15 },
    { 0xf8f0be53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b50be5b, SUC_lexicon_tags_37 },
    { 0xbdf0be5c, SUC_lexicon_tags_1 },
    { 0x47f0be5d, SUC_lexicon_tags_7 },
    { 0xe090be5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7050be62, SUC_lexicon_tags_38 },
    { 0x1ef0be63, SUC_lexicon_tags_11 },
    { 0xd5f0be64, SUC_lexicon_tags_37 },
    { 0xcd50be63, SUC_lexicon_tags_60 },
    { 0xe9d0be66, SUC_lexicon_tags_85 },
    { 0x9490be67, SUC_lexicon_tags_39 },
    { 0xb3b0be68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0be71, SUC_lexicon_tags_43 },
    { 0x9f70be72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b0be7a, SUC_lexicon_tags_39 },
    { 0x9f10be7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6990be7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8030be82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7970be86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5530be91, SUC_lexicon_tags_45 },
    { 0xc7d0be92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a50be95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8090be97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3130be9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6910be9d, SUC_lexicon_tags_30 },
    { 0x54d0be9d, SUC_lexicon_tags_14 },
    { 0xda10be9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4890bea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d0beae, SUC_lexicon_tags_2 },
    { 0x3c90beae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d0beb2, SUC_lexicon_tags_15 },
    { 0x4830beb3, SUC_lexicon_tags_138 },
    { 0x2df0beb4, SUC_lexicon_tags_39 },
    { 0x7dd0beb5, SUC_lexicon_tags_46 },
    { 0x1990beb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f0bebb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7990bebd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d0bec0, SUC_lexicon_tags_14 },
    { 0x7bf0bec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad0bec5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b0becb, SUC_lexicon_tags_26 },
    { 0x3450becb, SUC_lexicon_tags_107 },
    { 0xd410becb, SUC_lexicon_tags_12 },
    { 0x8670bece, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa3b0bed0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70bed3, SUC_lexicon_tags_9 },
    { 0xceb0bed4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcab0bed6, SUC_lexicon_tags_11 },
    { 0xb070bed7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e70beda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9790bedc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7130bede, SUC_lexicon_tags_45 },
    { 0xc6f0bede, SUC_lexicon_tags_14 },
    { 0xd430bee0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc090bee4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad10beee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a90bef4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4570bef7, SUC_lexicon_tags_2 },
    { 0x86f0bef8, SUC_lexicon_tags_9 },
    { 0x9af0bef7, SUC_lexicon_tags_14 },
    { 0xde50befa, SUC_lexicon_tags_25 },
    { 0x8330befb, SUC_lexicon_tags_63 },
    { 0x2db0befc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a50beff, SUC_lexicon_tags_29 },
    { 0x6890bf00, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfe10bf02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d0bf06, SUC_lexicon_tags_12 },
    { 0x26f0bf07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f0bf0a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bf12, SUC_lexicon_tags_27 },
    { 0xd590bf13, SUC_lexicon_tags_15 },
    { 0x7e50bf14, SUC_lexicon_tags_15 },
    { 0x30bf15, SUC_lexicon_tags_15 },
    { 0x76b0bf16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b10bf1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b50bf1f, SUC_lexicon_tags_21 },
    { 0x1eb0bf20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88d0bf22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa110bf24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc10bf26, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc370bf28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8750bf2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x470bf2e, SUC_lexicon_tags_32 },
    { 0xc610bf2e, SUC_lexicon_tags_25 },
    { 0x56f0bf2e, SUC_lexicon_tags_25 },
    { 0x6690bf2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77f0bf33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f0bf39, SUC_lexicon_tags_60 },
    { 0x17b0bf39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3030bf3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf0bf42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf50bf48, SUC_lexicon_tags_38 },
    { 0x17f0bf48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f70bf4f, SUC_lexicon_tags_43 },
    { 0x7770bf50, SUC_lexicon_tags_13 },
    { 0x8af0bf51, SUC_lexicon_tags_65 },
    { 0x6750bf52, SUC_lexicon_tags_9 },
    { 0xd6f0bf52, SUC_lexicon_tags_26 },
    { 0x1930bf54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde70bf56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56b0bf58, SUC_lexicon_tags_37 },
    { 0x5b50bf59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b90bf5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3350bf64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d0bf68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3e10bf6a, SUC_lexicon_tags_35 },
    { 0xe8f0bf6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb0bf6e, SUC_lexicon_tags_6 },
    { 0xd610bf6f, SUC_lexicon_tags_190 },
    { 0x3590bf70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0bf74, SUC_lexicon_tags_26 },
    { 0x9eb0bf75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed70bf77, SUC_lexicon_tags_14 },
    { 0xd3f0bf78, SUC_lexicon_tags_11 },
    { 0x6570bf79, SUC_lexicon_tags_29 },
    { 0xcbf0bf79, SUC_lexicon_tags_11 },
    { 0xb9d0bf7b, SUC_lexicon_tags_19 },
    { 0xf2d0bf7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb230bf7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff0bf81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef0bf85, SUC_lexicon_tags_63 },
    { 0xd9b0bf85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x750bf88, SUC_lexicon_tags_1 },
    { 0x89f0bf89, SUC_lexicon_tags_214 },
    { 0x6b10bf8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38b0bf8c, SUC_lexicon_tags_17 },
    { 0x96b0bf8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0bf90, SUC_lexicon_tags_0 },
    { 0x89b0bf91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce10bf93, SUC_lexicon_tags_8 },
    { 0xdbb0bf94, SUC_lexicon_tags_47 },
    { 0xefd0bf95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd10bf97, SUC_lexicon_tags_2 },
    { 0x33b0bf98, SUC_lexicon_tags_14 },
    { 0xbf50bf99, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x750bf9c, SUC_lexicon_tags_0 },
    { 0x230bf9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x970bf9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb0bfa2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe170bfa4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe170bfa6, SUC_lexicon_tags_8 },
    { 0xf510bfa7, SUC_lexicon_tags_15 },
    { 0x29d0bfa8, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xb430bfaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x710bfad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b10bfb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f0bfb5, SUC_lexicon_tags_15 },
    { 0xc510bfb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a70bfb9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfcd0bfbb, SUC_lexicon_tags_16 },
    { 0xa250bfbc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6910bfbe, SUC_lexicon_tags_17 },
    { 0x1190bfbf, SUC_lexicon_tags_37 },
    { 0xe090bfbe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f0bfc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f0bfca, SUC_lexicon_tags_60 },
    { 0xdb70bfca, SUC_lexicon_tags_33 },
    { 0x4e10bfca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe190bfce, SUC_lexicon_tags_51 },
    { 0xea70bfcf, SUC_lexicon_tags_38 },
    { 0x8590bfd0, SUC_lexicon_tags_15 },
    { 0xb5d0bfd1, SUC_lexicon_tags_29 },
    { 0x8b70bfcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d0bfd6, SUC_lexicon_tags_17 },
    { 0x99f0bfd6, SUC_lexicon_tags_12 },
    { 0x50b0bfd8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfd0bfda, SUC_lexicon_tags_29 },
    { 0x1b90bfdb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43b0bfdd, SUC_lexicon_tags_1 },
    { 0x5a70bfde, SUC_lexicon_tags_68 },
    { 0x4e50bfdd, SUC_lexicon_tags_47 },
    { 0x94b0bfe0, SUC_lexicon_tags_37 },
    { 0x2790bfe0, SUC_lexicon_tags_63 },
    { 0xf390bfe2, SUC_lexicon_tags_12 },
    { 0xf0f0bfe1, SUC_lexicon_tags_2 },
    { 0x9130bfe1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af0bfe7, SUC_lexicon_tags_25 },
    { 0x4230bfe8, SUC_lexicon_tags_17 },
    { 0xfeb0bfe7, SUC_lexicon_tags_0 },
    { 0x8ef0bfe8, SUC_lexicon_tags_57 },
    { 0xa590bfeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd70bfed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b0bff0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5030bff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd0bffb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad0bffe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3870c006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0c009, SUC_lexicon_tags_11 },
    { 0x3f70c009, SUC_lexicon_tags_15 },
    { 0x2e90c00b, SUC_lexicon_tags_37 },
    { 0x2510c00b, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd0c00f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb0c018, SUC_lexicon_tags_12 },
    { 0x8490c018, SUC_lexicon_tags_11 },
    { 0xaf50c01a, SUC_lexicon_tags_2 },
    { 0xd2d0c01b, SUC_lexicon_tags_12 },
    { 0x69b0c01a, SUC_lexicon_tags_55 },
    { 0xfdf0c01b, SUC_lexicon_tags_83 },
    { 0x7410c018, SUC_lexicon_tags_5 },
    { 0xf910c01f, SUC_lexicon_tags_77 },
    { 0xf050c020, SUC_lexicon_tags_37 },
    { 0x6650c021, SUC_lexicon_tags_1 },
    { 0x47f0c022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc290c025, SUC_lexicon_tags_33 },
    { 0xead0c026, SUC_lexicon_tags_37 },
    { 0x6650c026, SUC_lexicon_tags_25 },
    { 0x9a30c026, SUC_lexicon_tags_17 },
    { 0x7f30c025, SUC_lexicon_tags_17 },
    { 0x5550c028, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f0c02d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcf0c02f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf050c034, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab50c038, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e10c03c, SUC_lexicon_tags_12 },
    { 0x62b0c03c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b0c043, SUC_lexicon_tags_17 },
    { 0xf530c044, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb0c047, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa570c049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f0c04c, SUC_lexicon_tags_37 },
    { 0xd490c04d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc490c051, SUC_lexicon_tags_0 },
    { 0x57f0c051, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd0c056, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10c059, SUC_lexicon_tags_60 },
    { 0x110c05a, SUC_lexicon_tags_38 },
    { 0xe170c059, SUC_lexicon_tags_34 },
    { 0xa0f0c05c, SUC_lexicon_tags_6 },
    { 0x31d0c05d, SUC_lexicon_tags_37 },
    { 0xa810c05e, SUC_lexicon_tags_49 },
    { 0xab90c05f, SUC_lexicon_tags_43 },
    { 0x10b0c059, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b70c063, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e30c065, SUC_lexicon_tags_46 },
    { 0xe7f0c066, SUC_lexicon_tags_43 },
    { 0x8fd0c065, SUC_lexicon_tags_63 },
    { 0x6650c068, SUC_lexicon_tags_27 },
    { 0x6ed0c069, SUC_lexicon_tags_37 },
    { 0xb390c069, SUC_lexicon_tags_45 },
    { 0x77d0c06a, SUC_lexicon_tags_10 },
    { 0xa710c06b, SUC_lexicon_tags_14 },
    { 0x1370c069, SUC_lexicon_tags_9 },
    { 0x4810c06e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f0c071, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb0c079, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5dd0c07b, SUC_lexicon_tags_37 },
    { 0xdb30c07c, SUC_lexicon_tags_1 },
    { 0x1a50c07d, SUC_lexicon_tags_46 },
    { 0xe330c07e, SUC_lexicon_tags_61 },
    { 0x9dd0c07d, SUC_lexicon_tags_15 },
    { 0xc950c080, SUC_lexicon_tags_49 },
    { 0x59b0c081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2990c084, SUC_lexicon_tags_26 },
    { 0x2150c085, SUC_lexicon_tags_79 },
    { 0x6470c086, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfb10c088, SUC_lexicon_tags_4 },
    { 0x1f30c089, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b50c08d, SUC_lexicon_tags_63 },
    { 0xd3b0c08d, SUC_lexicon_tags_7 },
    { 0xe1b0c08f, SUC_lexicon_tags_12 },
    { 0x7e50c08e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530c094, SUC_lexicon_tags_107 },
    { 0x6ff0c095, SUC_lexicon_tags_11 },
    { 0x5c50c096, SUC_lexicon_tags_49 },
    { 0x1a90c097, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf950c09a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc4d0c09c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d90c09e, SUC_lexicon_tags_31 },
    { 0xd3b0c09f, SUC_lexicon_tags_8 },
    { 0xd950c09f, SUC_lexicon_tags_12 },
    { 0x4250c0a1, SUC_lexicon_tags_12 },
    { 0x4f0c0a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d0c0a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c50c0a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2a90c0ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9870c0b0, SUC_lexicon_tags_37 },
    { 0xedd0c0b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7090c0b3, SUC_lexicon_tags_21 },
    { 0xf3d0c0b4, SUC_lexicon_tags_14 },
    { 0x9990c0b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3a30c0b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb50c0b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf70c0bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf0c0c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd0c0cb, SUC_lexicon_tags_30 },
    { 0x4e10c0cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x78d0c0ce, SUC_lexicon_tags_7 },
    { 0x6630c0cf, SUC_lexicon_tags_84 },
    { 0xd090c0d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe1b0c0d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1770c0d4, SUC_lexicon_tags_151 },
    { 0xce50c0d4, SUC_lexicon_tags_1 },
    { 0x29d0c0d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d0c0dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a10c0e0, SUC_lexicon_tags_0 },
    { 0xb450c0e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5290c0e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe090c0ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x66d0c0ec, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x12f0c0ee, SUC_lexicon_tags_15 },
    { 0xec70c0ee, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5150c0f2, SUC_lexicon_tags_14 },
    { 0xa10c0f2, SUC_lexicon_tags_1 },
    { 0x4e10c0f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0c0f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf650c100, SUC_lexicon_tags_37 },
    { 0xeb30c100, SUC_lexicon_tags_7 },
    { 0x5090c102, SUC_lexicon_tags_11 },
    { 0x2f50c103, SUC_lexicon_tags_235 },
    { 0x7e70c104, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb0c108, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70c10d, SUC_lexicon_tags_43 },
    { 0xd310c10e, SUC_lexicon_tags_17 },
    { 0x7fb0c10d, SUC_lexicon_tags_15 },
    { 0x5dd0c110, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f30c112, SUC_lexicon_tags_47 },
    { 0xf850c113, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xac10c115, SUC_lexicon_tags_12 },
    { 0x70f0c115, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5130c119, SUC_lexicon_tags_97 },
    { 0xbd30c11a, SUC_lexicon_tags_1 },
    { 0xd510c11a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b0c11f, SUC_lexicon_tags_38 },
    { 0x2010c120, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xff0c123, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x390c125, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa30c12c, SUC_lexicon_tags_17 },
    { 0x4a30c12d, SUC_lexicon_tags_2 },
    { 0x2110c12e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a10c132, SUC_lexicon_tags_12 },
    { 0xb9d0c133, SUC_lexicon_tags_37 },
    { 0xf050c134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3150c13a, SUC_lexicon_tags_49 },
    { 0x3eb0c13b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2810c13e, SUC_lexicon_tags_30 },
    { 0x7510c13f, SUC_lexicon_tags_15 },
    { 0x890c140, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff0c147, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x34d0c149, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f0c14d, SUC_lexicon_tags_68 },
    { 0x6b10c14d, SUC_lexicon_tags_7 },
    { 0xe590c14d, SUC_lexicon_tags_25 },
    { 0xa130c150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1150c152, SUC_lexicon_tags_15 },
    { 0x1350c153, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe850c155, SUC_lexicon_tags_11 },
    { 0x4d30c156, SUC_lexicon_tags_47 },
    { 0xcf90c157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ad0c159, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b0c15c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6630c15e, SUC_lexicon_tags_7 },
    { 0xf530c15e, SUC_lexicon_tags_37 },
    { 0xabf0c160, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xef50c162, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9350c16a, SUC_lexicon_tags_10 },
    { 0x2930c16b, SUC_lexicon_tags_25 },
    { 0xe990c16c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4010c170, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f70c17c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb110c182, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xb310c185, SUC_lexicon_tags_1 },
    { 0x74d0c186, SUC_lexicon_tags_25 },
    { 0xc5d0c186, SUC_lexicon_tags_39 },
    { 0x9450c188, SUC_lexicon_tags_11 },
    { 0xecb0c189, SUC_lexicon_tags_12 },
    { 0xb930c189, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa30c18d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af0c192, SUC_lexicon_tags_6 },
    { 0xbc30c193, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b50c196, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa30c19c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11f0c19e, SUC_lexicon_tags_6 },
    { 0xb450c19f, SUC_lexicon_tags_2 },
    { 0x1d30c1a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa390c1a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe750c1a7, SUC_lexicon_tags_12 },
    { 0x95d0c1a8, SUC_lexicon_tags_37 },
    { 0x67d0c1a8, SUC_lexicon_tags_35 },
    { 0xdb90c1aa, SUC_lexicon_tags_37 },
    { 0x99b0c1aa, SUC_lexicon_tags_0 },
    { 0x65b0c1ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdd50c1ae, SUC_lexicon_tags_7 },
    { 0x7cf0c1af, SUC_lexicon_tags_27 },
    { 0x3f10c1ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1730c1ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe250c1bf, SUC_lexicon_tags_13 },
    { 0xdf30c1c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f0c1c6, SUC_lexicon_tags_25 },
    { 0xedf0c1c6, SUC_lexicon_tags_12 },
    { 0xe110c1c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2cd0c1ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1490c1ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbe70c1d0, SUC_lexicon_tags_25 },
    { 0x7b50c1d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f50c1d4, SUC_lexicon_tags_17 },
    { 0xf0d0c1d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa10c1da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf570c1e0, SUC_lexicon_tags_12 },
    { 0x80b0c1e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7970c1e4, SUC_lexicon_tags_33 },
    { 0x1e30c1e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9f0c1e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf90c1e9, SUC_lexicon_tags_61 },
    { 0xc5b0c1e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e70c1ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b0c1f3, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf30c205, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f0c209, SUC_lexicon_tags_15 },
    { 0x88d0c209, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f50c20e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb0c211, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d0c214, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa330c217, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd190c21c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd030c224, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb2b0c226, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa090c228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd510c22a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7130c232, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9190c235, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d0c239, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b0c23b, SUC_lexicon_tags_37 },
    { 0x8170c23c, SUC_lexicon_tags_23 },
    { 0x45f0c23d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1730c243, SUC_lexicon_tags_17 },
    { 0x1570c243, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2970c249, SUC_lexicon_tags_0 },
    { 0x9c30c249, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3570c252, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1290c256, SUC_lexicon_tags_45 },
    { 0xf5d0c257, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1710c25a, SUC_lexicon_tags_16 },
    { 0x20b0c25a, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x3dd0c25d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x3950c260, SUC_lexicon_tags_17 },
    { 0x9950c261, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df0c269, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7410c26d, SUC_lexicon_tags_6 },
    { 0x4810c26e, SUC_lexicon_tags_14 },
    { 0x7c50c26d, SUC_lexicon_tags_2 },
    { 0xedf0c26f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7d0c272, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4910c274, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdbf0c276, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd050c278, SUC_lexicon_tags_46 },
    { 0xd270c278, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb0c283, SUC_lexicon_tags_1 },
    { 0x55b0c284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7390c286, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4350c288, SUC_lexicon_tags_25 },
    { 0xe4d0c289, SUC_lexicon_tags_10 },
    { 0xae30c289, SUC_lexicon_tags_23 },
    { 0xdd10c28a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb850c294, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f0c29a, SUC_lexicon_tags_25 },
    { 0x9530c29b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a30c29e, SUC_lexicon_tags_194 },
    { 0xedd0c29e, SUC_lexicon_tags_1 },
    { 0x47f0c2a0, SUC_lexicon_tags_63 },
    { 0xb3b0c2a1, SUC_lexicon_tags_2 },
    { 0x3090c2a2, SUC_lexicon_tags_37 },
    { 0xe0f0c2a3, SUC_lexicon_tags_26 },
    { 0xd4f0c2a4, SUC_lexicon_tags_33 },
    { 0x89d0c2a5, SUC_lexicon_tags_37 },
    { 0xe810c2a1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x3550c2a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e10c2ab, SUC_lexicon_tags_45 },
    { 0x5050c2ab, SUC_lexicon_tags_7 },
    { 0x8af0c2ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc930c2b1, SUC_lexicon_tags_13 },
    { 0x5330c2b2, SUC_lexicon_tags_38 },
    { 0xa8b0c2b2, SUC_lexicon_tags_65 },
    { 0x6830c2b2, SUC_lexicon_tags_43 },
    { 0x4e70c2b5, SUC_lexicon_tags_10 },
    { 0xebf0c2b6, SUC_lexicon_tags_6 },
    { 0xa2b0c2b6, SUC_lexicon_tags_7 },
    { 0x5d90c2b7, SUC_lexicon_tags_60 },
    { 0xedb0c2b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca50c2be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f0c2c5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b0c2c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9870c2cb, SUC_lexicon_tags_28 },
    { 0x9290c2cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cd0c2ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9e70c2d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf0c2d3, SUC_lexicon_tags_1 },
    { 0x67d0c2d4, SUC_lexicon_tags_25 },
    { 0x8350c2d4, SUC_lexicon_tags_7 },
    { 0x9e70c2d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc30c2d8, SUC_lexicon_tags_27 },
    { 0xb0b0c2d8, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa450c2e4, SUC_lexicon_tags_12 },
    { 0xc750c2e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26f0c2e7, SUC_lexicon_tags_7 },
    { 0x4dd0c2e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1570c2ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf190c2ec, SUC_lexicon_tags_48 },
    { 0x630c2ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5990c2ef, SUC_lexicon_tags_16 },
    { 0x64d0c2ef, SUC_lexicon_tags_37 },
    { 0xe490c2f1, SUC_lexicon_tags_14 },
    { 0xea50c2f2, SUC_lexicon_tags_27 },
    { 0xa9d0c2f3, SUC_lexicon_tags_17 },
    { 0x23d0c2f4, SUC_lexicon_tags_43 },
    { 0xd550c2ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b90c2f9, SUC_lexicon_tags_8 },
    { 0xbf70c2f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5950c2fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c90c2ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5950c305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7990c308, SUC_lexicon_tags_30 },
    { 0x4510c309, SUC_lexicon_tags_25 },
    { 0x3cd0c30a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6530c30f, SUC_lexicon_tags_47 },
    { 0xa3d0c30f, SUC_lexicon_tags_25 },
    { 0x43f0c310, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd30c31f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xba10c322, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8070c324, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcbd0c326, SUC_lexicon_tags_26 },
    { 0x51f0c326, SUC_lexicon_tags_17 },
    { 0x2830c327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8170c32e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6670c332, SUC_lexicon_tags_60 },
    { 0x9c70c333, SUC_lexicon_tags_12 },
    { 0x24f0c333, SUC_lexicon_tags_30 },
    { 0xc50c335, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde90c33a, SUC_lexicon_tags_1 },
    { 0x7650c33b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8210c33f, SUC_lexicon_tags_8 },
    { 0x3670c33f, SUC_lexicon_tags_11 },
    { 0xfe50c341, SUC_lexicon_tags_25 },
    { 0x1f90c342, SUC_lexicon_tags_17 },
    { 0xbf50c340, SUC_lexicon_tags_2 },
    { 0xbe10c344, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7710c347, SUC_lexicon_tags_25 },
    { 0x3fd0c347, SUC_lexicon_tags_69 },
    { 0xe750c348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f0c34c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d0c351, SUC_lexicon_tags_37 },
    { 0x2bb0c352, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d0c356, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef0c35e, SUC_lexicon_tags_37 },
    { 0x4550c35f, SUC_lexicon_tags_1 },
    { 0xe290c360, SUC_lexicon_tags_7 },
    { 0x10c35f, SUC_lexicon_tags_17 },
    { 0xb5d0c35e, SUC_lexicon_tags_14 },
    { 0x8290c363, SUC_lexicon_tags_1 },
    { 0xa290c364, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f0c368, SUC_lexicon_tags_15 },
    { 0x22f0c369, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d0c36d, SUC_lexicon_tags_415 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f0c370, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4170c372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc150c374, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c90c378, SUC_lexicon_tags_7 },
    { 0x3af0c378, SUC_lexicon_tags_77 },
    { 0x4810c378, SUC_lexicon_tags_11 },
    { 0xc330c37a, SUC_lexicon_tags_0 },
    { 0x14f0c37c, SUC_lexicon_tags_65 },
    { 0x8750c37d, SUC_lexicon_tags_2 },
    { 0x6b90c37e, SUC_lexicon_tags_11 },
    { 0x5b0c37f, SUC_lexicon_tags_25 },
    { 0xcef0c37f, SUC_lexicon_tags_49 },
    { 0x1890c381, SUC_lexicon_tags_63 },
    { 0x8370c37e, SUC_lexicon_tags_62 },
    { 0xfd30c383, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6450c385, SUC_lexicon_tags_30 },
    { 0x5bf0c386, SUC_lexicon_tags_47 },
    { 0x81d0c387, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa970c38b, SUC_lexicon_tags_30 },
    { 0x2bd0c38b, SUC_lexicon_tags_8 },
    { 0x4a30c38d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b0c390, SUC_lexicon_tags_103 },
    { 0xbe90c391, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb850c393, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f0c399, SUC_lexicon_tags_37 },
    { 0xd0d0c39a, SUC_lexicon_tags_10 },
    { 0x590c39b, SUC_lexicon_tags_12 },
    { 0x8550c39a, SUC_lexicon_tags_11 },
    { 0xe810c39d, SUC_lexicon_tags_11 },
    { 0xd190c39d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3bf0c3a0, SUC_lexicon_tags_43 },
    { 0x6930c3a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf50c3a4, SUC_lexicon_tags_1 },
    { 0x5530c3a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e30c3ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f10c3af, SUC_lexicon_tags_31 },
    { 0xa610c3b0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x79b0c3b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9230c3b4, SUC_lexicon_tags_47 },
    { 0x7fb0c3b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f0c3b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2490c3bd, SUC_lexicon_tags_11 },
    { 0x96f0c3bd, SUC_lexicon_tags_60 },
    { 0xb010c3bf, SUC_lexicon_tags_2 },
    { 0x19b0c3bd, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x73f0c3c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf0c3c6, SUC_lexicon_tags_3 },
    { 0x3c90c3c6, SUC_lexicon_tags_7 },
    { 0x9d70c3c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76d0c3ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f90c3d6, SUC_lexicon_tags_23 },
    { 0xe010c3d6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8f90c3d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d0c3e1, SUC_lexicon_tags_1 },
    { 0x85d0c3e2, SUC_lexicon_tags_2 },
    { 0x81b0c3e2, SUC_lexicon_tags_1 },
    { 0x9e30c3e1, SUC_lexicon_tags_77 },
    { 0xbb0c3e1, SUC_lexicon_tags_38 },
    { 0x210c3e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf770c3e9, SUC_lexicon_tags_17 },
    { 0x7590c3ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f0c3ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb0c3f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d0c3f7, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b0c3fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0c402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2430c405, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc50c40b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5530c40d, SUC_lexicon_tags_7 },
    { 0xc6f0c40e, SUC_lexicon_tags_17 },
    { 0x4590c40f, SUC_lexicon_tags_43 },
    { 0x7c30c40d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3290c417, SUC_lexicon_tags_27 },
    { 0xbff0c417, SUC_lexicon_tags_1 },
    { 0x8e10c417, SUC_lexicon_tags_25 },
    { 0x7f50c419, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f0c41e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70c421, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd10c423, SUC_lexicon_tags_39 },
    { 0x7590c423, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb0c429, SUC_lexicon_tags_48 },
    { 0xe0b0c429, SUC_lexicon_tags_27 },
    { 0xfa90c42a, SUC_lexicon_tags_17 },
    { 0x5050c42b, SUC_lexicon_tags_15 },
    { 0x6d0c42a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f0c433, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7370c438, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad50c43c, SUC_lexicon_tags_33 },
    { 0xc070c43c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3890c43f, SUC_lexicon_tags_43 },
    { 0x7ff0c43f, SUC_lexicon_tags_25 },
    { 0x48d0c441, SUC_lexicon_tags_25 },
    { 0xaa30c441, SUC_lexicon_tags_15 },
    { 0xfd90c443, SUC_lexicon_tags_39 },
    { 0xc6f0c444, SUC_lexicon_tags_11 },
    { 0x25b0c444, SUC_lexicon_tags_37 },
    { 0x9890c442, SUC_lexicon_tags_42 },
    { 0xb290c447, SUC_lexicon_tags_17 },
    { 0x1d70c448, SUC_lexicon_tags_12 },
    { 0x4d10c449, SUC_lexicon_tags_37 },
    { 0x55f0c44a, SUC_lexicon_tags_7 },
    { 0x4ed0c44a, SUC_lexicon_tags_28 },
    { 0x7d10c449, SUC_lexicon_tags_15 },
    { 0x2fd0c44c, SUC_lexicon_tags_1 },
    { 0x50f0c44e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b0c451, SUC_lexicon_tags_2 },
    { 0x59d0c452, SUC_lexicon_tags_53 },
    { 0xd950c451, SUC_lexicon_tags_37 },
    { 0xa010c453, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfff0c456, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe70c458, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fb0c45a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd0c45e, SUC_lexicon_tags_6 },
    { 0x32d0c45f, SUC_lexicon_tags_27 },
    { 0x9850c45f, SUC_lexicon_tags_38 },
    { 0x490c461, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x330c463, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x10d0c465, SUC_lexicon_tags_2 },
    { 0x870c466, SUC_lexicon_tags_15 },
    { 0xd770c465, SUC_lexicon_tags_41 },
    { 0xd950c467, SUC_lexicon_tags_1 },
    { 0x4730c466, SUC_lexicon_tags_66 },
    { 0x9b50c466, SUC_lexicon_tags_29 },
    { 0x67f0c46b, SUC_lexicon_tags_49 },
    { 0x6b10c46a, SUC_lexicon_tags_9 },
    { 0x28f0c466, SUC_lexicon_tags_37 },
    { 0x6d10c46d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d90c472, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7c90c474, SUC_lexicon_tags_17 },
    { 0x4290c475, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8070c478, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6df0c47a, SUC_lexicon_tags_17 },
    { 0x33f0c47b, SUC_lexicon_tags_27 },
    { 0x26f0c47c, SUC_lexicon_tags_62 },
    { 0x9eb0c47b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab0c482, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810c486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd770c48a, SUC_lexicon_tags_7 },
    { 0x5530c48a, SUC_lexicon_tags_15 },
    { 0xfcb0c48c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe130c48f, SUC_lexicon_tags_11 },
    { 0xfad0c48f, SUC_lexicon_tags_7 },
    { 0xdd70c490, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb830c493, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1430c49b, SUC_lexicon_tags_12 },
    { 0x6650c49c, SUC_lexicon_tags_6 },
    { 0x990c49d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f0c4a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b0c4ab, SUC_lexicon_tags_37 },
    { 0x5db0c4ab, SUC_lexicon_tags_19 },
    { 0x1030c4ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3630c4af, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xad10c4b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd610c4b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef10c4b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1510c4ba, SUC_lexicon_tags_38 },
    { 0xe610c4ba, SUC_lexicon_tags_7 },
    { 0xcd10c4bc, SUC_lexicon_tags_1 },
    { 0x8690c4bd, SUC_lexicon_tags_25 },
    { 0xf4d0c4be, SUC_lexicon_tags_47 },
    { 0x9e70c4bf, SUC_lexicon_tags_1 },
    { 0xd9f0c4c0, SUC_lexicon_tags_15 },
    { 0xeb70c4c0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f0c4cc, SUC_lexicon_tags_2 },
    { 0xc90c4cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92f0c4cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d0c4d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1890c4d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb0c4d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb0c4db, SUC_lexicon_tags_37 },
    { 0x4930c4dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f0c4e0, SUC_lexicon_tags_38 },
    { 0x9090c4e1, SUC_lexicon_tags_1 },
    { 0xaad0c4e1, SUC_lexicon_tags_15 },
    { 0x5490c4e0, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6050c4e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20d0c4ea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f50c4ef, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1650c4f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9f0c4f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b0c4f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd0c4fc, SUC_lexicon_tags_7 },
    { 0x97f0c4fd, SUC_lexicon_tags_37 },
    { 0x37b0c4fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f0c505, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d0c509, SUC_lexicon_tags_38 },
    { 0x4610c50a, SUC_lexicon_tags_37 },
    { 0xc30c50b, SUC_lexicon_tags_28 },
    { 0x4d30c50c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb0c510, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc330c512, SUC_lexicon_tags_27 },
    { 0x99f0c513, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf990c523, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x490c527, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb10c52b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd10c52e, SUC_lexicon_tags_14 },
    { 0x9d50c52e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8690c532, SUC_lexicon_tags_30 },
    { 0x830c533, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd10c538, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3770c53d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a10c542, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d0c547, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xced0c54a, SUC_lexicon_tags_48 },
    { 0xff10c54a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7810c54d, SUC_lexicon_tags_2 },
    { 0x4250c54e, SUC_lexicon_tags_11 },
    { 0x6a70c54f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb0c556, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb770c559, SUC_lexicon_tags_8 },
    { 0x1770c55a, SUC_lexicon_tags_17 },
    { 0xfbd0c55b, SUC_lexicon_tags_12 },
    { 0xa550c55c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8e10c55e, SUC_lexicon_tags_45 },
    { 0x1210c55e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7650c562, SUC_lexicon_tags_29 },
    { 0x93d0c562, SUC_lexicon_tags_21 },
    { 0x1ab0c564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3150c567, SUC_lexicon_tags_47 },
    { 0x6cb0c568, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1290c56a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa310c56e, SUC_lexicon_tags_11 },
    { 0x2af0c56f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db0c573, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3750c57c, SUC_lexicon_tags_45 },
    { 0x5190c57d, SUC_lexicon_tags_7 },
    { 0x8310c57e, SUC_lexicon_tags_15 },
    { 0x8370c57f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf30c581, SUC_lexicon_tags_84 },
    { 0x5df0c582, SUC_lexicon_tags_18 },
    { 0xb3d0c581, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9e10c585, SUC_lexicon_tags_48 },
    { 0xe190c585, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d30c588, SUC_lexicon_tags_7 },
    { 0x8d70c589, SUC_lexicon_tags_7 },
    { 0x2150c58a, SUC_lexicon_tags_11 },
    { 0xde10c589, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x73f0c58d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf850c591, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff0c595, SUC_lexicon_tags_16 },
    { 0xc2d0c595, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x110c599, SUC_lexicon_tags_27 },
    { 0x65b0c59a, SUC_lexicon_tags_66 },
    { 0x64b0c59a, SUC_lexicon_tags_9 },
    { 0xc0d0c59a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30c59f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe170c5a4, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xc3d0c5a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d50c5aa, SUC_lexicon_tags_0 },
    { 0x2970c5ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc10c5ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4710c5b0, SUC_lexicon_tags_33 },
    { 0x1970c5b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb30c5b5, SUC_lexicon_tags_10 },
    { 0x370c5b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1730c5bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd7d0c5bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x57f0c5bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf630c5c1, SUC_lexicon_tags_32 },
    { 0xe3d0c5c2, SUC_lexicon_tags_25 },
    { 0x4190c5c3, SUC_lexicon_tags_12 },
    { 0x9150c5c4, SUC_lexicon_tags_0 },
    { 0xd7b0c5c4, SUC_lexicon_tags_179 },
    { 0x3410c5c2, SUC_lexicon_tags_2 },
    { 0x1f90c5c1, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xf270c5ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9130c5cd, SUC_lexicon_tags_69 },
    { 0x3250c5cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a90c5d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd0c5d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe970c5db, SUC_lexicon_tags_6 },
    { 0x46d0c5dc, SUC_lexicon_tags_48 },
    { 0xb7b0c5dd, SUC_lexicon_tags_38 },
    { 0x6090c5de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf0c5e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce10c5e4, SUC_lexicon_tags_1 },
    { 0x1770c5e5, SUC_lexicon_tags_56 },
    { 0xea50c5e6, SUC_lexicon_tags_38 },
    { 0x7ff0c5e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c50c5ed, SUC_lexicon_tags_7 },
    { 0x6dd0c5ee, SUC_lexicon_tags_1219 },
    { 0xe4f0c5ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db0c5f3, SUC_lexicon_tags_0 },
    { 0x5070c5f3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d0c5f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3e10c5f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d0c5fd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7830c600, SUC_lexicon_tags_11 },
    { 0x1c50c601, SUC_lexicon_tags_25 },
    { 0x7550c602, SUC_lexicon_tags_25 },
    { 0x150c603, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb0c606, SUC_lexicon_tags_14 },
    { 0x9410c607, SUC_lexicon_tags_25 },
    { 0xfb0c606, SUC_lexicon_tags_25 },
    { 0xce30c607, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ed0c60b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f0c60f, SUC_lexicon_tags_38 },
    { 0x4410c610, SUC_lexicon_tags_45 },
    { 0x5d10c611, SUC_lexicon_tags_37 },
    { 0x7210c610, SUC_lexicon_tags_0 },
    { 0x9910c60f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f10c618, SUC_lexicon_tags_49 },
    { 0x250c618, SUC_lexicon_tags_27 },
    { 0x56f0c618, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xdef0c61c, SUC_lexicon_tags_1 },
    { 0x6670c61d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4450c620, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb0c624, SUC_lexicon_tags_2 },
    { 0xe8d0c624, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb0c629, SUC_lexicon_tags_11 },
    { 0x57f0c62a, SUC_lexicon_tags_11 },
    { 0xbd10c62a, SUC_lexicon_tags_40 },
    { 0x1050c62c, SUC_lexicon_tags_0 },
    { 0x710c62d, SUC_lexicon_tags_63 },
    { 0x84b0c62e, SUC_lexicon_tags_9 },
    { 0x6970c62f, SUC_lexicon_tags_2 },
    { 0xfb30c62f, SUC_lexicon_tags_14 },
    { 0x9190c630, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd0c635, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc10c637, SUC_lexicon_tags_12 },
    { 0xf670c638, SUC_lexicon_tags_17 },
    { 0x8df0c637, SUC_lexicon_tags_11 },
    { 0x42b0c63a, SUC_lexicon_tags_42 },
    { 0xacd0c63a, SUC_lexicon_tags_2 },
    { 0x7b50c63c, SUC_lexicon_tags_30 },
    { 0xacb0c637, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x55f0c63f, SUC_lexicon_tags_31 },
    { 0xcc50c63f, SUC_lexicon_tags_11 },
    { 0x6f30c641, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d0c646, SUC_lexicon_tags_11 },
    { 0x5130c646, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e90c649, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad0c64e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe70c651, SUC_lexicon_tags_8 },
    { 0x9670c652, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810c656, SUC_lexicon_tags_46 },
    { 0xebb0c656, SUC_lexicon_tags_60 },
    { 0x9ed0c658, SUC_lexicon_tags_29 },
    { 0x8c70c658, SUC_lexicon_tags_38 },
    { 0xb7f0c658, SUC_lexicon_tags_17 },
    { 0x5d10c659, SUC_lexicon_tags_11 },
    { 0x1450c65c, SUC_lexicon_tags_2 },
    { 0x7c10c65a, SUC_lexicon_tags_28 },
    { 0x5bf0c65e, SUC_lexicon_tags_68 },
    { 0x3a90c65f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaff0c661, SUC_lexicon_tags_15 },
    { 0xd5d0c662, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f0c664, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b0c66b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x470c66e, SUC_lexicon_tags_6 },
    { 0x2030c66f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7470c674, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf70c676, SUC_lexicon_tags_37 },
    { 0xe3f0c677, SUC_lexicon_tags_57 },
    { 0xd370c677, SUC_lexicon_tags_45 },
    { 0xf250c679, SUC_lexicon_tags_16 },
    { 0x1410c67a, SUC_lexicon_tags_37 },
    { 0xac70c67b, SUC_lexicon_tags_11 },
    { 0xf10c67b, SUC_lexicon_tags_0 },
    { 0xc130c67d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe970c67f, SUC_lexicon_tags_37 },
    { 0xa010c67f, SUC_lexicon_tags_28 },
    { 0x6ed0c680, SUC_lexicon_tags_43 },
    { 0x6190c680, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc130c684, SUC_lexicon_tags_77 },
    { 0x9470c685, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xa490c688, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc850c68a, SUC_lexicon_tags_49 },
    { 0xbd90c68b, SUC_lexicon_tags_32 },
    { 0x1850c68b, SUC_lexicon_tags_19 },
    { 0xcc90c68d, SUC_lexicon_tags_1 },
    { 0x6c70c68e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f10c690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca10c697, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b30c699, SUC_lexicon_tags_28 },
    { 0x26f0c69a, SUC_lexicon_tags_19 },
    { 0x5670c69b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb0c69e, SUC_lexicon_tags_26 },
    { 0x1c30c69f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f0c6a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc470c6a9, SUC_lexicon_tags_23 },
    { 0xc650c6a9, SUC_lexicon_tags_11 },
    { 0xc690c6a9, SUC_lexicon_tags_0 },
    { 0x8bb0c6aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4470c6b3, SUC_lexicon_tags_47 },
    { 0xe650c6b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2cd0c6b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac70c6b9, SUC_lexicon_tags_15 },
    { 0x8210c6b9, SUC_lexicon_tags_33 },
    { 0x9ed0c6bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff0c6be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb10c6c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa770c6c4, SUC_lexicon_tags_19 },
    { 0x32b0c6c5, SUC_lexicon_tags_11 },
    { 0xc830c6c4, SUC_lexicon_tags_43 },
    { 0xd890c6c7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc650c6ca, SUC_lexicon_tags_2 },
    { 0xb1f0c6ca, SUC_lexicon_tags_4 },
    { 0xeaf0c6ca, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b0c6d4, SUC_lexicon_tags_12 },
    { 0x74d0c6d5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9790c6db, SUC_lexicon_tags_8 },
    { 0x4370c6dc, SUC_lexicon_tags_43 },
    { 0x9910c6dd, SUC_lexicon_tags_7 },
    { 0x1c30c6de, SUC_lexicon_tags_33 },
    { 0x3f50c6dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x450c6e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb10c6e5, SUC_lexicon_tags_17 },
    { 0xdd70c6e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9710c6ec, SUC_lexicon_tags_37 },
    { 0xe550c6ed, SUC_lexicon_tags_11 },
    { 0xdc30c6ee, SUC_lexicon_tags_14 },
    { 0xc6f0c6ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b0c6f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96d0c6f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8050c6f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc170c6fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d90c6fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9210c702, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa910c704, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd50c709, SUC_lexicon_tags_31 },
    { 0x1fd0c70a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f50c712, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db0c716, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2670c718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1030c729, SUC_lexicon_tags_33 },
    { 0x3230c72a, SUC_lexicon_tags_45 },
    { 0xd250c72b, SUC_lexicon_tags_13 },
    { 0x29f0c72c, SUC_lexicon_tags_11 },
    { 0x8a30c72b, SUC_lexicon_tags_37 },
    { 0xc250c72e, SUC_lexicon_tags_65 },
    { 0x690c72f, SUC_lexicon_tags_11 },
    { 0xa730c730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59d0c732, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe2f0c734, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f0c737, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5870c73b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3370c73d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d50c740, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9530c745, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9190c74a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2870c74d, SUC_lexicon_tags_38 },
    { 0x9d90c74e, SUC_lexicon_tags_14 },
    { 0x31f0c74f, SUC_lexicon_tags_12 },
    { 0x3dd0c74f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe70c753, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d0c75a, SUC_lexicon_tags_2 },
    { 0x75d0c75b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfaf0c75d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f10c761, SUC_lexicon_tags_11 },
    { 0x3130c762, SUC_lexicon_tags_49 },
    { 0x23d0c763, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d0c766, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b0c769, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x270c76c, SUC_lexicon_tags_17 },
    { 0x4450c76d, SUC_lexicon_tags_19 },
    { 0x4a70c76e, SUC_lexicon_tags_47 },
    { 0x4cf0c76e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d0c776, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb830c778, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8db0c77a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4ff0c77c, SUC_lexicon_tags_17 },
    { 0x3050c77d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf50c780, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa970c789, SUC_lexicon_tags_38 },
    { 0xbad0c78a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e90c78c, SUC_lexicon_tags_27 },
    { 0xb2b0c78d, SUC_lexicon_tags_4 },
    { 0x4dd0c78d, SUC_lexicon_tags_17 },
    { 0x79b0c78f, SUC_lexicon_tags_12 },
    { 0x1bd0c790, SUC_lexicon_tags_34 },
    { 0x2b50c78c, SUC_lexicon_tags_25 },
    { 0x3570c791, SUC_lexicon_tags_11 },
    { 0x1eb0c793, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0x1090c795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc690c797, SUC_lexicon_tags_11 },
    { 0xf3b0c797, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x28d0c79a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6950c79e, SUC_lexicon_tags_17 },
    { 0xc350c79e, SUC_lexicon_tags_11 },
    { 0xd530c7a0, SUC_lexicon_tags_15 },
    { 0x27d0c79f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a30c7a5, SUC_lexicon_tags_29 },
    { 0x86b0c7a6, SUC_lexicon_tags_27 },
    { 0x3d90c7a7, SUC_lexicon_tags_25 },
    { 0x1530c7a8, SUC_lexicon_tags_82 },
    { 0xb5b0c7a8, SUC_lexicon_tags_15 },
    { 0xbf0c7aa, SUC_lexicon_tags_33 },
    { 0xa630c7aa, SUC_lexicon_tags_7 },
    { 0xeb70c7ac, SUC_lexicon_tags_7 },
    { 0x5a30c7ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb630c7b0, SUC_lexicon_tags_38 },
    { 0x2cd0c7b0, SUC_lexicon_tags_8 },
    { 0xfe90c7b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab0c7b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc050c7ba, SUC_lexicon_tags_17 },
    { 0x1030c7bb, SUC_lexicon_tags_23 },
    { 0x4810c7bc, SUC_lexicon_tags_10 },
    { 0x3f30c7bb, SUC_lexicon_tags_23 },
    { 0x8810c7be, SUC_lexicon_tags_7 },
    { 0xb530c7bf, SUC_lexicon_tags_2 },
    { 0x4330c7bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a30c7c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3370c7c6, SUC_lexicon_tags_7 },
    { 0x27d0c7c7, SUC_lexicon_tags_5 },
    { 0xd050c7c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3210c7ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad50c7ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6210c7d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d50c7d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6310c7d8, SUC_lexicon_tags_13 },
    { 0x1e50c7d9, SUC_lexicon_tags_47 },
    { 0x19b0c7d9, SUC_lexicon_tags_45 },
    { 0x5570c7d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4690c7de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2730c7e3, SUC_lexicon_tags_11 },
    { 0xa170c7e4, SUC_lexicon_tags_15 },
    { 0x3570c7e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d0c7ea, SUC_lexicon_tags_21 },
    { 0xe410c7eb, SUC_lexicon_tags_37 },
    { 0x5c30c7ec, SUC_lexicon_tags_63 },
    { 0xa710c7ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db0c7f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf50c7f6, SUC_lexicon_tags_19 },
    { 0x2490c7f7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9490c7fc, SUC_lexicon_tags_38 },
    { 0xb90c7fd, SUC_lexicon_tags_17 },
    { 0x4a30c7fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d0c803, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f0c806, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c50c809, SUC_lexicon_tags_1 },
    { 0x5f90c80a, SUC_lexicon_tags_1 },
    { 0x3c90c80a, SUC_lexicon_tags_25 },
    { 0x11d0c80c, SUC_lexicon_tags_0 },
    { 0x2510c80d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5030c80f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb530c812, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f0c815, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd0c81c, SUC_lexicon_tags_28 },
    { 0xaab0c81c, SUC_lexicon_tags_17 },
    { 0x81d0c81c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad0c827, SUC_lexicon_tags_27 },
    { 0xfd70c828, SUC_lexicon_tags_45 },
    { 0xb510c828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6eb0c82b, SUC_lexicon_tags_17 },
    { 0xcff0c82c, SUC_lexicon_tags_17 },
    { 0x3430c82c, SUC_lexicon_tags_91 },
    { 0x4f0c82d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5590c835, SUC_lexicon_tags_38 },
    { 0x11d0c836, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd90c83c, SUC_lexicon_tags_106 },
    { 0xf7b0c83d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2310c845, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6450c849, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f0c84c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf0c852, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdeb0c854, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa10c858, SUC_lexicon_tags_7 },
    { 0x3c70c858, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x63b0c85b, SUC_lexicon_tags_26 },
    { 0x22d0c85b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d0c862, SUC_lexicon_tags_47 },
    { 0xad30c862, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xb3b0c865, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb970c868, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda70c86c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8e30c86e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3d50c870, SUC_lexicon_tags_60 },
    { 0xba30c871, SUC_lexicon_tags_1 },
    { 0x42b0c871, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc630c875, SUC_lexicon_tags_61 },
    { 0x2cd0c875, SUC_lexicon_tags_17 },
    { 0xecf0c875, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef0c87d, SUC_lexicon_tags_2 },
    { 0xd350c87e, SUC_lexicon_tags_12 },
    { 0x8870c87f, SUC_lexicon_tags_77 },
    { 0xb9d0c87e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7710c886, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x73d0c888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5910c88e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2350c891, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8490c893, SUC_lexicon_tags_10 },
    { 0x8070c893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x73f0c897, SUC_lexicon_tags_37 },
    { 0xa3f0c898, SUC_lexicon_tags_9 },
    { 0x1c30c899, SUC_lexicon_tags_11 },
    { 0x95b0c89a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6eb0c89c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7950c8a0, SUC_lexicon_tags_11 },
    { 0x8fd0c8a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x150c8a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7190c8a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb290c8ab, SUC_lexicon_tags_244 },
    { 0xcf50c8ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1750c8b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb050c8b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b30c8b4, SUC_lexicon_tags_7 },
    { 0xc5f0c8b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x21b0c8b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41d0c8b9, SUC_lexicon_tags_425 },
    { 0x3590c8ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b0c8bf, SUC_lexicon_tags_12 },
    { 0x90f0c8c0, SUC_lexicon_tags_25 },
    { 0x6930c8c1, SUC_lexicon_tags_11 },
    { 0x4b30c8c0, SUC_lexicon_tags_47 },
    { 0xfab0c8c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb70c8c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3150c8d0, SUC_lexicon_tags_2 },
    { 0x4b50c8d0, SUC_lexicon_tags_25 },
    { 0x2170c8d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b0c8d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e50c8e0, SUC_lexicon_tags_1 },
    { 0x8070c8e0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4810c8e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3270c8e8, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf650c8ee, SUC_lexicon_tags_11 },
    { 0xe6b0c8ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb310c8f2, SUC_lexicon_tags_87 },
    { 0x1270c8f3, SUC_lexicon_tags_25 },
    { 0xfff0c8f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x50c8f7, SUC_lexicon_tags_60 },
    { 0x2530c8f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x49f0c8fa, SUC_lexicon_tags_33 },
    { 0xfbd0c8fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2290c8fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0c903, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b0c90c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab0c911, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3650c913, SUC_lexicon_tags_37 },
    { 0x9990c914, SUC_lexicon_tags_7 },
    { 0x4ad0c915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f0c917, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6cf0c919, SUC_lexicon_tags_37 },
    { 0x3fd0c91a, SUC_lexicon_tags_4 },
    { 0xa510c919, SUC_lexicon_tags_11 },
    { 0x2b50c919, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6330c91f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b0c924, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd50c926, SUC_lexicon_tags_9 },
    { 0xdd70c927, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f0c92b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x12b0c92d, SUC_lexicon_tags_27 },
    { 0xbb30c92d, SUC_lexicon_tags_23 },
    { 0x7f50c92f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90c933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbd0c935, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe470c93b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4150c93f, SUC_lexicon_tags_37 },
    { 0x1ff0c93f, SUC_lexicon_tags_14 },
    { 0x9df0c93f, SUC_lexicon_tags_31 },
    { 0xb7b0c942, SUC_lexicon_tags_43 },
    { 0x670c943, SUC_lexicon_tags_14 },
    { 0x4990c940, SUC_lexicon_tags_33 },
    { 0x8350c945, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d0c949, SUC_lexicon_tags_65 },
    { 0x9b50c94a, SUC_lexicon_tags_38 },
    { 0xf150c94a, SUC_lexicon_tags_1 },
    { 0xcff0c94a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a70c951, SUC_lexicon_tags_34 },
    { 0x6250c952, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b50c954, SUC_lexicon_tags_55 },
    { 0x2cf0c954, SUC_lexicon_tags_33 },
    { 0xc6b0c956, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc170c958, SUC_lexicon_tags_47 },
    { 0x9390c959, SUC_lexicon_tags_15 },
    { 0xda50c959, SUC_lexicon_tags_37 },
    { 0x6a30c95b, SUC_lexicon_tags_33 },
    { 0xa190c95c, SUC_lexicon_tags_1 },
    { 0xa030c959, SUC_lexicon_tags_11 },
    { 0x3710c95e, SUC_lexicon_tags_68 },
    { 0xfa10c95f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e50c962, SUC_lexicon_tags_23 },
    { 0xe6d0c962, SUC_lexicon_tags_26 },
    { 0x1ef0c962, SUC_lexicon_tags_29 },
    { 0xc630c965, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40b0c967, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf250c96d, SUC_lexicon_tags_1 },
    { 0x8250c96e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3130c975, SUC_lexicon_tags_1 },
    { 0x24d0c975, SUC_lexicon_tags_39 },
    { 0x66f0c977, SUC_lexicon_tags_19 },
    { 0x1050c975, SUC_lexicon_tags_37 },
    { 0x6cf0c979, SUC_lexicon_tags_25 },
    { 0x5b10c979, SUC_lexicon_tags_11 },
    { 0xc170c976, SUC_lexicon_tags_17 },
    { 0xee90c97c, SUC_lexicon_tags_27 },
    { 0xb0c97d, SUC_lexicon_tags_28 },
    { 0x8650c97d, SUC_lexicon_tags_25 },
    { 0x5e70c977, SUC_lexicon_tags_14 },
    { 0x6a90c97a, SUC_lexicon_tags_37 },
    { 0x1470c980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6510c986, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b50c989, SUC_lexicon_tags_11 },
    { 0xb350c989, SUC_lexicon_tags_31 },
    { 0x6d50c989, SUC_lexicon_tags_37 },
    { 0xcb10c98b, SUC_lexicon_tags_1 },
    { 0x84b0c98d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d0c991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e30c994, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56b0c996, SUC_lexicon_tags_43 },
    { 0x8b90c996, SUC_lexicon_tags_15 },
    { 0xf910c998, SUC_lexicon_tags_30 },
    { 0xd390c999, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc50c99c, SUC_lexicon_tags_7 },
    { 0xa1f0c99c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10c9a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d0c9a6, SUC_lexicon_tags_5 },
    { 0x51d0c9a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e90c9ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa330c9ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b70c9b1, SUC_lexicon_tags_6 },
    { 0xe050c9b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xed90c9b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b0c9bb, SUC_lexicon_tags_25 },
    { 0x70b0c9bc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa4d0c9be, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x2cd0c9c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xac10c9c3, SUC_lexicon_tags_37 },
    { 0xf7d0c9c4, SUC_lexicon_tags_1 },
    { 0x6cd0c9c5, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xa890c9c7, SUC_lexicon_tags_4 },
    { 0x5e50c9c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9410c9ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ad0c9cc, SUC_lexicon_tags_7 },
    { 0x7cb0c9cc, SUC_lexicon_tags_11 },
    { 0x4b70c9cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5250c9d2, SUC_lexicon_tags_30 },
    { 0x9070c9d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf30c9d5, SUC_lexicon_tags_23 },
    { 0x85b0c9d6, SUC_lexicon_tags_23 },
    { 0xc290c9d7, SUC_lexicon_tags_2 },
    { 0xc0b0c9d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4010c9db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b0c9e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f10c9e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e90c9e6, SUC_lexicon_tags_47 },
    { 0x9db0c9e7, SUC_lexicon_tags_15 },
    { 0x5b50c9e8, SUC_lexicon_tags_25 },
    { 0xd970c9e9, SUC_lexicon_tags_37 },
    { 0xf750c9e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5790c9ec, SUC_lexicon_tags_27 },
    { 0x17b0c9ed, SUC_lexicon_tags_1 },
    { 0xa330c9ee, SUC_lexicon_tags_27 },
    { 0x93b0c9ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af0c9f5, SUC_lexicon_tags_16 },
    { 0x7df0c9f6, SUC_lexicon_tags_11 },
    { 0x3ef0c9f7, SUC_lexicon_tags_25 },
    { 0x8bf0c9f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5490ca00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52b0ca02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f0ca08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5750ca0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4470ca0d, SUC_lexicon_tags_23 },
    { 0x89f0ca0e, SUC_lexicon_tags_17 },
    { 0xb130ca0f, SUC_lexicon_tags_19 },
    { 0xced0ca0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7410ca13, SUC_lexicon_tags_14 },
    { 0x25d0ca13, SUC_lexicon_tags_33 },
    { 0xb410ca15, SUC_lexicon_tags_23 },
    { 0x3370ca16, SUC_lexicon_tags_17 },
    { 0xfc90ca17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc610ca1a, SUC_lexicon_tags_12 },
    { 0xdaf0ca1b, SUC_lexicon_tags_11 },
    { 0x7390ca1c, SUC_lexicon_tags_2 },
    { 0x4010ca1d, SUC_lexicon_tags_45 },
    { 0x3a30ca1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6070ca23, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650ca28, SUC_lexicon_tags_13 },
    { 0xe3b0ca29, SUC_lexicon_tags_27 },
    { 0xe810ca29, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d0ca2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb570ca36, SUC_lexicon_tags_25 },
    { 0xbf10ca36, SUC_lexicon_tags_25 },
    { 0x4f50ca37, SUC_lexicon_tags_60 },
    { 0x2090ca39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38f0ca3b, SUC_lexicon_tags_37 },
    { 0xd370ca3b, SUC_lexicon_tags_7 },
    { 0xef0ca3d, SUC_lexicon_tags_155 },
    { 0x6af0ca3e, SUC_lexicon_tags_31 },
    { 0xb290ca3e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf370ca41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad0ca44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2110ca46, SUC_lexicon_tags_25 },
    { 0x1f0ca46, SUC_lexicon_tags_12 },
    { 0x5790ca48, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc130ca4c, SUC_lexicon_tags_2 },
    { 0x7bf0ca4c, SUC_lexicon_tags_38 },
    { 0xfcb0ca4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d90ca52, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x4af0ca54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee70ca56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d0ca5b, SUC_lexicon_tags_17 },
    { 0x9090ca5b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4310ca60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd0ca63, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d90ca66, SUC_lexicon_tags_91 },
    { 0xab30ca67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1090ca6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac90ca6c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8db0ca6e, SUC_lexicon_tags_29 },
    { 0xca70ca6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf270ca72, SUC_lexicon_tags_7 },
    { 0xa370ca73, SUC_lexicon_tags_2 },
    { 0xdfb0ca74, SUC_lexicon_tags_60 },
    { 0x2330ca74, SUC_lexicon_tags_25 },
    { 0xe570ca76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb10ca7a, SUC_lexicon_tags_0 },
    { 0xe7b0ca7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e90ca81, SUC_lexicon_tags_17 },
    { 0x96b0ca82, SUC_lexicon_tags_60 },
    { 0x87b0ca81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd110ca88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9190ca8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf3b0ca8f, SUC_lexicon_tags_15 },
    { 0x4350ca90, SUC_lexicon_tags_68 },
    { 0x9170ca8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb630ca94, SUC_lexicon_tags_25 },
    { 0x35b0ca94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8970ca97, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d50ca9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70ca9f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6cf0caa1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b0caa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d30caa8, SUC_lexicon_tags_25 },
    { 0x6590caa9, SUC_lexicon_tags_9 },
    { 0xcc90caaa, SUC_lexicon_tags_1 },
    { 0x7330caab, SUC_lexicon_tags_33 },
    { 0x8950caac, SUC_lexicon_tags_45 },
    { 0x4dd0caac, SUC_lexicon_tags_15 },
    { 0x3910caab, SUC_lexicon_tags_17 },
    { 0xb430caaf, SUC_lexicon_tags_12 },
    { 0x6ef0caaf, SUC_lexicon_tags_32 },
    { 0x9310cab1, SUC_lexicon_tags_37 },
    { 0x69f0cab2, SUC_lexicon_tags_6 },
    { 0xfa30cab1, SUC_lexicon_tags_2 },
    { 0xcb10caad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5b0cab6, SUC_lexicon_tags_39 },
    { 0x9930cab6, SUC_lexicon_tags_6 },
    { 0x4ab0cab8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xae70cabb, SUC_lexicon_tags_68 },
    { 0x41b0cabb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b0cac0, SUC_lexicon_tags_7 },
    { 0x8110cac1, SUC_lexicon_tags_37 },
    { 0x18d0cac1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7790cac4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2010cac9, SUC_lexicon_tags_23 },
    { 0xa070caca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d10cacd, SUC_lexicon_tags_9 },
    { 0x5270cace, SUC_lexicon_tags_23 },
    { 0x7d10cacf, SUC_lexicon_tags_49 },
    { 0x9370cad0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f0cad8, SUC_lexicon_tags_6 },
    { 0xda50cad9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f0cadf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f0cae2, SUC_lexicon_tags_15 },
    { 0xbd0cae3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb310cae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2930cae8, SUC_lexicon_tags_14 },
    { 0xacb0cae9, SUC_lexicon_tags_1 },
    { 0x54f0caea, SUC_lexicon_tags_39 },
    { 0xd970cae9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb0caf2, SUC_lexicon_tags_14 },
    { 0x73b0caf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc70caf6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc570cafb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf70caff, SUC_lexicon_tags_7 },
    { 0xdbf0cb00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5d0cb02, SUC_lexicon_tags_25 },
    { 0x1670cb03, SUC_lexicon_tags_11 },
    { 0xd3f0cb04, SUC_lexicon_tags_37 },
    { 0xae10cb05, SUC_lexicon_tags_7 },
    { 0xf390cb05, SUC_lexicon_tags_9 },
    { 0x7f50cb07, SUC_lexicon_tags_1 },
    { 0xe810cb08, SUC_lexicon_tags_37 },
    { 0x8b90cb05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f0cb0c, SUC_lexicon_tags_7 },
    { 0xd670cb0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8930cb10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd0cb13, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d0cb19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef0cb1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5710cb22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6690cb25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4750cb27, SUC_lexicon_tags_43 },
    { 0x8770cb28, SUC_lexicon_tags_11 },
    { 0x5230cb29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1890cb2b, SUC_lexicon_tags_4 },
    { 0xac70cb2b, SUC_lexicon_tags_26 },
    { 0x2290cb2c, SUC_lexicon_tags_25 },
    { 0x7190cb2c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbf50cb30, SUC_lexicon_tags_1 },
    { 0xf8f0cb30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4a90cb33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3970cb3c, SUC_lexicon_tags_11 },
    { 0xb3d0cb3d, SUC_lexicon_tags_92 },
    { 0xb8d0cb3c, SUC_lexicon_tags_17 },
    { 0x3350cb3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7890cb42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7090cb44, SUC_lexicon_tags_23 },
    { 0xe8f0cb45, SUC_lexicon_tags_31 },
    { 0x3190cb46, SUC_lexicon_tags_11 },
    { 0xf970cb45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98b0cb49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f70cb4e, SUC_lexicon_tags_14 },
    { 0x3e90cb4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc850cb51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb70cb54, SUC_lexicon_tags_7 },
    { 0xc670cb54, SUC_lexicon_tags_11 },
    { 0x59f0cb56, SUC_lexicon_tags_47 },
    { 0xa850cb57, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8590cb5a, SUC_lexicon_tags_2 },
    { 0x10cb5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f10cb5e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c90cb61, SUC_lexicon_tags_25 },
    { 0x55b0cb62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5410cb65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd0cb6b, SUC_lexicon_tags_17 },
    { 0xab70cb6b, SUC_lexicon_tags_34 },
    { 0xdf30cb6d, SUC_lexicon_tags_25 },
    { 0xde90cb6e, SUC_lexicon_tags_38 },
    { 0xcf50cb6d, SUC_lexicon_tags_7 },
    { 0xe30cb70, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8090cb75, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x2290cb77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf490cb79, SUC_lexicon_tags_11 },
    { 0xbf50cb79, SUC_lexicon_tags_2 },
    { 0xbe50cb7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf30cb85, SUC_lexicon_tags_25 },
    { 0xefd0cb86, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6330cb89, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3070cb8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8090cb8f, SUC_lexicon_tags_11 },
    { 0xd9b0cb8f, SUC_lexicon_tags_19 },
    { 0xa250cb91, SUC_lexicon_tags_29 },
    { 0xcc70cb91, SUC_lexicon_tags_77 },
    { 0x2570cb92, SUC_lexicon_tags_15 },
    { 0xd8d0cb94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b90cb9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7030cb9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfdf0cba1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7010cba3, SUC_lexicon_tags_6 },
    { 0x3690cba4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36f0cba6, SUC_lexicon_tags_12 },
    { 0xa910cba7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4610cbab, SUC_lexicon_tags_32 },
    { 0xb830cbac, SUC_lexicon_tags_28 },
    { 0x51f0cbac, SUC_lexicon_tags_8 },
    { 0x8750cbae, SUC_lexicon_tags_65 },
    { 0xca50cbab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6d0cbb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f0cbb5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe290cbb7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2110cbb9, SUC_lexicon_tags_37 },
    { 0x73f0cbba, SUC_lexicon_tags_28 },
    { 0x5950cbba, SUC_lexicon_tags_41 },
    { 0xa6d0cbbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd110cbc0, SUC_lexicon_tags_12 },
    { 0x82d0cbc1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed0cbc4, SUC_lexicon_tags_17 },
    { 0xc750cbc5, SUC_lexicon_tags_66 },
    { 0x6030cbc5, SUC_lexicon_tags_17 },
    { 0x3c90cbc7, SUC_lexicon_tags_8 },
    { 0xb2d0cbc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50cbcb, SUC_lexicon_tags_28 },
    { 0xaff0cbcc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd0cbd1, SUC_lexicon_tags_63 },
    { 0x3870cbd2, SUC_lexicon_tags_25 },
    { 0x190cbd3, SUC_lexicon_tags_12 },
    { 0x8730cbd1, SUC_lexicon_tags_29 },
    { 0x5510cbd2, SUC_lexicon_tags_11 },
    { 0x2130cbd6, SUC_lexicon_tags_46 },
    { 0x31b0cbd7, SUC_lexicon_tags_68 },
    { 0x3dd0cbd8, SUC_lexicon_tags_15 },
    { 0xbbb0cbd9, SUC_lexicon_tags_15 },
    { 0xff50cbd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8030cbe3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1410cbe5, SUC_lexicon_tags_23 },
    { 0x92d0cbe6, SUC_lexicon_tags_1 },
    { 0x8a30cbe7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb10cbec, SUC_lexicon_tags_29 },
    { 0x1d30cbed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc590cbef, SUC_lexicon_tags_12 },
    { 0x9210cbf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2430cbf2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfdb0cbf4, SUC_lexicon_tags_10 },
    { 0x92b0cbf5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff50cc04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9430cc06, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f0cc0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e50cc13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d0cc16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb10cc19, SUC_lexicon_tags_38 },
    { 0x92d0cc1a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd0cc1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdeb0cc21, SUC_lexicon_tags_37 },
    { 0x5570cc21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8290cc25, SUC_lexicon_tags_0 },
    { 0x8f70cc26, SUC_lexicon_tags_60 },
    { 0xa270cc27, SUC_lexicon_tags_19 },
    { 0x9830cc26, SUC_lexicon_tags_33 },
    { 0xfef0cc29, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b0cc2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f0cc35, SUC_lexicon_tags_47 },
    { 0x15f0cc35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa70cc3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdd0cc3d, SUC_lexicon_tags_15 },
    { 0xc890cc3d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x82b0cc40, SUC_lexicon_tags_29 },
    { 0x8130cc41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9370cc45, SUC_lexicon_tags_27 },
    { 0xf190cc45, SUC_lexicon_tags_25 },
    { 0x170cc47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c90cc4b, SUC_lexicon_tags_37 },
    { 0x9d50cc4b, SUC_lexicon_tags_12 },
    { 0xdd90cc4b, SUC_lexicon_tags_1 },
    { 0x4550cc4e, SUC_lexicon_tags_48 },
    { 0xd290cc4f, SUC_lexicon_tags_1 },
    { 0xda10cc50, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2430cc52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18b0cc54, SUC_lexicon_tags_37 },
    { 0x22d0cc55, SUC_lexicon_tags_37 },
    { 0x92d0cc54, SUC_lexicon_tags_47 },
    { 0xaff0cc57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xca30cc59, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb0cc5c, SUC_lexicon_tags_522 },
    { 0xae10cc5d, SUC_lexicon_tags_37 },
    { 0x6830cc5d, SUC_lexicon_tags_15 },
    { 0xfa70cc5f, SUC_lexicon_tags_37 },
    { 0x510cc60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa510cc64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bf0cc66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b70cc6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff0cc6f, SUC_lexicon_tags_48 },
    { 0x88b0cc70, SUC_lexicon_tags_49 },
    { 0x8770cc71, SUC_lexicon_tags_27 },
    { 0x4a70cc72, SUC_lexicon_tags_7 },
    { 0xd130cc72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe650cc75, SUC_lexicon_tags_31 },
    { 0xf0cc76, SUC_lexicon_tags_37 },
    { 0x230cc75, SUC_lexicon_tags_30 },
    { 0x82f0cc76, SUC_lexicon_tags_656 },
    { 0xa090cc77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa50cc7b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x3610cc7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c50cc84, SUC_lexicon_tags_37 },
    { 0x4710cc84, SUC_lexicon_tags_21 },
    { 0x5370cc86, SUC_lexicon_tags_2 },
    { 0xd270cc86, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5630cc8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e70cc8f, SUC_lexicon_tags_10 },
    { 0xef10cc8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe6d0cc92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7710cc96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28d0cc98, SUC_lexicon_tags_49 },
    { 0xb2f0cc99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59d0cc9b, SUC_lexicon_tags_37 },
    { 0xb0cc9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81f0cc9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5710cca0, SUC_lexicon_tags_0 },
    { 0x4c90cca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5250cca3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x270cca7, SUC_lexicon_tags_21 },
    { 0xa2b0cca8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf950ccaa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f0ccad, SUC_lexicon_tags_37 },
    { 0x5c30ccad, SUC_lexicon_tags_19 },
    { 0x7790ccad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef0ccb3, SUC_lexicon_tags_315 },
    { 0xfa10ccb3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf430ccb7, SUC_lexicon_tags_29 },
    { 0xec50ccb8, SUC_lexicon_tags_0 },
    { 0xdd10ccb8, SUC_lexicon_tags_7 },
    { 0xcbf0ccba, SUC_lexicon_tags_85 },
    { 0x9350ccbb, SUC_lexicon_tags_37 },
    { 0xc2b0ccbc, SUC_lexicon_tags_27 },
    { 0x5d50ccbd, SUC_lexicon_tags_11 },
    { 0xfad0ccbc, SUC_lexicon_tags_9 },
    { 0xa130ccbf, SUC_lexicon_tags_38 },
    { 0x7150ccc0, SUC_lexicon_tags_7 },
    { 0xcd70ccbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x830ccc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d0ccd1, SUC_lexicon_tags_37 },
    { 0x4570ccd1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa870ccd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc870ccda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f30ccdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5330ccde, SUC_lexicon_tags_38 },
    { 0xa870ccdf, SUC_lexicon_tags_19 },
    { 0x3bb0ccdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2f0cce2, SUC_lexicon_tags_15 },
    { 0x93d0cce3, SUC_lexicon_tags_15 },
    { 0x7610cce4, SUC_lexicon_tags_16 },
    { 0x8cf0cce2, SUC_lexicon_tags_35 },
    { 0xf7d0cce6, SUC_lexicon_tags_14 },
    { 0xe8d0cce7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8990ccec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3890ccee, SUC_lexicon_tags_107 },
    { 0x53b0ccef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3fb0ccf1, SUC_lexicon_tags_66 },
    { 0x8010ccf2, SUC_lexicon_tags_11 },
    { 0x8c50ccf3, SUC_lexicon_tags_12 },
    { 0x8170ccf4, SUC_lexicon_tags_56 },
    { 0x9b50ccf2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf310ccf8, SUC_lexicon_tags_3 },
    { 0x62f0ccf8, SUC_lexicon_tags_129 },
    { 0x9930ccfa, SUC_lexicon_tags_8 },
    { 0x6690ccfb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8810cd01, SUC_lexicon_tags_18 },
    { 0xe90cd02, SUC_lexicon_tags_25 },
    { 0xdcb0cd03, SUC_lexicon_tags_17 },
    { 0xaf50cd01, SUC_lexicon_tags_160 },
    { 0xf530cd05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc050cd09, SUC_lexicon_tags_63 },
    { 0xd0d0cd0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c10cd0d, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd0cd12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe090cd14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f0cd18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c30cd1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97d0cd1c, SUC_lexicon_tags_37 },
    { 0x3d90cd1c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf0cd22, SUC_lexicon_tags_27 },
    { 0x1790cd23, SUC_lexicon_tags_11 },
    { 0x570cd24, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x22f0cd26, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5b90cd28, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed0cd2e, SUC_lexicon_tags_29 },
    { 0x1c30cd2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6750cd34, SUC_lexicon_tags_3 },
    { 0x72f0cd35, SUC_lexicon_tags_32 },
    { 0x5370cd36, SUC_lexicon_tags_26 },
    { 0x66b0cd34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530cd3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8350cd45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70cd4c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa90cd50, SUC_lexicon_tags_38 },
    { 0xb010cd51, SUC_lexicon_tags_17 },
    { 0x2d0cd52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd0cd56, SUC_lexicon_tags_25 },
    { 0x5130cd57, SUC_lexicon_tags_12 },
    { 0xa5b0cd58, SUC_lexicon_tags_13 },
    { 0x1350cd58, SUC_lexicon_tags_23 },
    { 0x5910cd58, SUC_lexicon_tags_37 },
    { 0xcb30cd5a, SUC_lexicon_tags_37 },
    { 0x21d0cd5c, SUC_lexicon_tags_55 },
    { 0xe0d0cd5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad0cd61, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb050cd64, SUC_lexicon_tags_7 },
    { 0x3590cd65, SUC_lexicon_tags_7 },
    { 0x8cd0cd64, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f0cd6c, SUC_lexicon_tags_37 },
    { 0xebd0cd6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed0cd73, SUC_lexicon_tags_37 },
    { 0x4e10cd74, SUC_lexicon_tags_11 },
    { 0x5930cd74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590cd7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b0cd81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f0cd84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2270cd86, SUC_lexicon_tags_37 },
    { 0xde10cd86, SUC_lexicon_tags_4 },
    { 0x31f0cd88, SUC_lexicon_tags_49 },
    { 0xb770cd89, SUC_lexicon_tags_60 },
    { 0xddf0cd87, SUC_lexicon_tags_25 },
    { 0xcfd0cd86, SUC_lexicon_tags_37 },
    { 0x51d0cd8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc10cd8f, SUC_lexicon_tags_45 },
    { 0x4ad0cd90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c90cd99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ed0cd9b, SUC_lexicon_tags_9 },
    { 0xa990cd9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d90cd9f, SUC_lexicon_tags_9 },
    { 0xe9d0cd9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb90cda4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c90cdaa, SUC_lexicon_tags_29 },
    { 0xd8d0cdab, SUC_lexicon_tags_17 },
    { 0x1cd0cdaa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc50cdb0, SUC_lexicon_tags_25 },
    { 0x66f0cdb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8590cdb6, SUC_lexicon_tags_55 },
    { 0x9970cdb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe290cdbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa90cdc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2970cdc5, SUC_lexicon_tags_66 },
    { 0x7d70cdc6, SUC_lexicon_tags_1 },
    { 0x46d0cdc7, SUC_lexicon_tags_17 },
    { 0xdb70cdc8, SUC_lexicon_tags_26 },
    { 0x5c70cdc9, SUC_lexicon_tags_25 },
    { 0x1d30cdca, SUC_lexicon_tags_25 },
    { 0x8cb0cdca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f0cdce, SUC_lexicon_tags_19 },
    { 0xd4f0cdcf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcd10cdd1, SUC_lexicon_tags_30 },
    { 0x57b0cdd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf0cdd7, SUC_lexicon_tags_17 },
    { 0x37d0cdd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd410cde3, SUC_lexicon_tags_21 },
    { 0x690cde4, SUC_lexicon_tags_2 },
    { 0x8990cde5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc730cde7, SUC_lexicon_tags_10 },
    { 0x8190cde8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7110cdea, SUC_lexicon_tags_11 },
    { 0xde90cdeb, SUC_lexicon_tags_15 },
    { 0x1810cdeb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb130cdf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db0cdf3, SUC_lexicon_tags_39 },
    { 0x2850cdf4, SUC_lexicon_tags_15 },
    { 0xbab0cdf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed50cdf9, SUC_lexicon_tags_15 },
    { 0x3290cdf9, SUC_lexicon_tags_29 },
    { 0xf170cdfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c10cdfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe30ce01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df0ce08, SUC_lexicon_tags_47 },
    { 0x2b10ce09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd0ce0d, SUC_lexicon_tags_1 },
    { 0x5bd0ce0e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac30ce12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f0ce15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9130ce19, SUC_lexicon_tags_1 },
    { 0xf730ce19, SUC_lexicon_tags_11 },
    { 0xbd50ce1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a50ce1d, SUC_lexicon_tags_12 },
    { 0x50d0ce1e, SUC_lexicon_tags_60 },
    { 0xe110ce1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ed0ce21, SUC_lexicon_tags_138 },
    { 0xad90ce21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e90ce24, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef0ce29, SUC_lexicon_tags_45 },
    { 0x8590ce29, SUC_lexicon_tags_55 },
    { 0xa5d0ce29, SUC_lexicon_tags_25 },
    { 0x9c30ce2c, SUC_lexicon_tags_37 },
    { 0xc4d0ce2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b0ce31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2f0ce33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d0ce3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7170ce40, SUC_lexicon_tags_7 },
    { 0x5eb0ce41, SUC_lexicon_tags_49 },
    { 0x3890ce42, SUC_lexicon_tags_1 },
    { 0x3230ce41, SUC_lexicon_tags_14 },
    { 0xf850ce41, SUC_lexicon_tags_11 },
    { 0xdf0ce45, SUC_lexicon_tags_15 },
    { 0x79f0ce46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f30ce4a, SUC_lexicon_tags_88 },
    { 0xab70ce4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf0ce4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde30ce52, SUC_lexicon_tags_0 },
    { 0x1cf0ce52, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc50ce55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1f0ce57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb030ce59, SUC_lexicon_tags_4 },
    { 0xf890ce59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd130ce5e, SUC_lexicon_tags_46 },
    { 0xf450ce5e, SUC_lexicon_tags_60 },
    { 0xc630ce60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe10ce63, SUC_lexicon_tags_1 },
    { 0x24d0ce64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9490ce6b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b0ce74, SUC_lexicon_tags_25 },
    { 0xd610ce75, SUC_lexicon_tags_37 },
    { 0xc070ce74, SUC_lexicon_tags_51 },
    { 0xc3f0ce76, SUC_lexicon_tags_7 },
    { 0xdc90ce77, SUC_lexicon_tags_45 },
    { 0xdff0ce79, SUC_lexicon_tags_48 },
    { 0xde10ce77, SUC_lexicon_tags_12 },
    { 0x8ff0ce78, SUC_lexicon_tags_15 },
    { 0x4fd0ce7c, SUC_lexicon_tags_1 },
    { 0xf90ce77, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xe5d0ce7f, SUC_lexicon_tags_45 },
    { 0x8070ce7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4610ce82, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a50ce89, SUC_lexicon_tags_15 },
    { 0x1650ce89, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f0ce8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb0ce90, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb30ce92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x65d0ce94, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6b50ce96, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b70ce9b, SUC_lexicon_tags_11 },
    { 0x3230ce9b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb270ce9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe350cea2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d0cea7, SUC_lexicon_tags_0 },
    { 0x2db0cea8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1070ceaa, SUC_lexicon_tags_35 },
    { 0x48f0ceaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc750ceb0, SUC_lexicon_tags_43 },
    { 0xe090ceb0, SUC_lexicon_tags_14 },
    { 0x5a30ceb2, SUC_lexicon_tags_51 },
    { 0xe30ceb3, SUC_lexicon_tags_46 },
    { 0xf750ceb3, SUC_lexicon_tags_27 },
    { 0x8a10ceb5, SUC_lexicon_tags_25 },
    { 0x4050ceb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f50cec3, SUC_lexicon_tags_31 },
    { 0x9cd0cec4, SUC_lexicon_tags_84 },
    { 0x3090cec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1850cecb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc430ced0, SUC_lexicon_tags_23 },
    { 0xa810ced0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf50ced4, SUC_lexicon_tags_25 },
    { 0x3bf0ced4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9110cedb, SUC_lexicon_tags_12 },
    { 0x3490cedc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa190cede, SUC_lexicon_tags_84 },
    { 0xc10cedf, SUC_lexicon_tags_37 },
    { 0xf410cee0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xac30cee3, SUC_lexicon_tags_1 },
    { 0xa4f0cee3, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd0cee8, SUC_lexicon_tags_12 },
    { 0xc810cee9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f0ceee, SUC_lexicon_tags_28 },
    { 0x110ceef, SUC_lexicon_tags_7 },
    { 0x110cef0, SUC_lexicon_tags_25 },
    { 0x5fd0cef1, SUC_lexicon_tags_12 },
    { 0x4f70ceee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4370cef9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd5b0cefb, SUC_lexicon_tags_37 },
    { 0x2730cefb, SUC_lexicon_tags_14 },
    { 0xb3f0cefb, SUC_lexicon_tags_65 },
    { 0x3cf0cefe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c10cf01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b0cf07, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b0cf0a, SUC_lexicon_tags_3 },
    { 0x77d0cf0b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4df0cf0d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb0cf10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc190cf12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a10cf15, SUC_lexicon_tags_26 },
    { 0x5850cf16, SUC_lexicon_tags_29 },
    { 0x4bd0cf15, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb0cf1a, SUC_lexicon_tags_54 },
    { 0xb9f0cf1b, SUC_lexicon_tags_11 },
    { 0xd850cf1c, SUC_lexicon_tags_25 },
    { 0x1930cf1a, SUC_lexicon_tags_15 },
    { 0x5e10cf1e, SUC_lexicon_tags_25 },
    { 0x8a70cf1f, SUC_lexicon_tags_1 },
    { 0x3230cf1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad10cf23, SUC_lexicon_tags_45 },
    { 0xc4d0cf23, SUC_lexicon_tags_7 },
    { 0xde70cf24, SUC_lexicon_tags_35 },
    { 0xcf70cf25, SUC_lexicon_tags_13 },
    { 0x7eb0cf27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6290cf2a, SUC_lexicon_tags_82 },
    { 0x69d0cf2b, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf70cf30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf0cf35, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7130cf3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5370cf47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc70cf49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1750cf4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ff0cf4f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x690cf54, SUC_lexicon_tags_11 },
    { 0xbe10cf54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc10cf59, SUC_lexicon_tags_14 },
    { 0x1e90cf5a, SUC_lexicon_tags_27 },
    { 0x6e70cf5b, SUC_lexicon_tags_597 },
    { 0xf970cf5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7270cf5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xded0cf60, SUC_lexicon_tags_17 },
    { 0x3fd0cf61, SUC_lexicon_tags_45 },
    { 0xdbb0cf62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a70cf68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab0cf6b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5010cf6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8830cf71, SUC_lexicon_tags_47 },
    { 0x4df0cf71, SUC_lexicon_tags_17 },
    { 0x7330cf73, SUC_lexicon_tags_12 },
    { 0xd850cf73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1550cf79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5f0cf7b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd010cf7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d0cf81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd70cf84, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10cf89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd0cf8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2fd0cf8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1170cf93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d0cf96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7910cf9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef0cf9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed0cfa0, SUC_lexicon_tags_77 },
    { 0x1190cfa0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b0cfa4, SUC_lexicon_tags_19 },
    { 0xc390cfa4, SUC_lexicon_tags_17 },
    { 0x4710cfa6, SUC_lexicon_tags_49 },
    { 0xfbb0cfa7, SUC_lexicon_tags_31 },
    { 0x95f0cfa8, SUC_lexicon_tags_25 },
    { 0x7af0cfa9, SUC_lexicon_tags_12 },
    { 0x5a90cfa7, SUC_lexicon_tags_37 },
    { 0xbb50cfab, SUC_lexicon_tags_2 },
    { 0x5f10cfab, SUC_lexicon_tags_84 },
    { 0xb510cfab, SUC_lexicon_tags_14 },
    { 0x56d0cfae, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x54d0cfb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6150cfb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x78b0cfb4, SUC_lexicon_tags_63 },
    { 0x60d0cfb5, SUC_lexicon_tags_43 },
    { 0xa6d0cfb4, SUC_lexicon_tags_13 },
    { 0x1b70cfb7, SUC_lexicon_tags_9 },
    { 0xf410cfb8, SUC_lexicon_tags_7 },
    { 0xa2d0cfb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a30cfbb, SUC_lexicon_tags_15 },
    { 0xda90cfbc, SUC_lexicon_tags_57 },
    { 0x3150cfbc, SUC_lexicon_tags_25 },
    { 0x5430cfbb, SUC_lexicon_tags_17 },
    { 0x6af0cfbf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6bd0cfc1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f0cfc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b0cfcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f0cfd8, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x5690cfda, SUC_lexicon_tags_15 },
    { 0xc790cfdb, SUC_lexicon_tags_26 },
    { 0xb730cfdc, SUC_lexicon_tags_43 },
    { 0xd3f0cfdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd650cfe1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9030cfe3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad0cfe7, SUC_lexicon_tags_66 },
    { 0x4590cfe8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d0cfee, SUC_lexicon_tags_43 },
    { 0xdbd0cfef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe650cff1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d0cff4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9090cff6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b0cffb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd50cffd, SUC_lexicon_tags_37 },
    { 0xd2d0cffe, SUC_lexicon_tags_47 },
    { 0x79f0cfff, SUC_lexicon_tags_22 },
    { 0x4af0cffd, SUC_lexicon_tags_15 },
    { 0xb10d001, SUC_lexicon_tags_42 },
    { 0xde10d001, SUC_lexicon_tags_15 },
    { 0x3550d003, SUC_lexicon_tags_56 },
    { 0xec10d003, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2030d006, SUC_lexicon_tags_47 },
    { 0x9c50d007, SUC_lexicon_tags_17 },
    { 0x49f0d006, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x78b0d00a, SUC_lexicon_tags_33 },
    { 0x79d0d00b, SUC_lexicon_tags_15 },
    { 0xc90d00a, SUC_lexicon_tags_31 },
    { 0xb770d00d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d0d011, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e30d015, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b0d020, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b10d022, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde70d029, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x810d030, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1570d032, SUC_lexicon_tags_48 },
    { 0x75d0d033, SUC_lexicon_tags_37 },
    { 0xf210d034, SUC_lexicon_tags_36 },
    { 0xfad0d034, SUC_lexicon_tags_14 },
    { 0x9050d034, SUC_lexicon_tags_21 },
    { 0xcff0d037, SUC_lexicon_tags_12 },
    { 0x52d0d037, SUC_lexicon_tags_62 },
    { 0x1390d039, SUC_lexicon_tags_19 },
    { 0x91d0d03a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7270d03e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef0d041, SUC_lexicon_tags_37 },
    { 0x6770d041, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc350d047, SUC_lexicon_tags_45 },
    { 0x2910d048, SUC_lexicon_tags_10 },
    { 0xf70d048, SUC_lexicon_tags_37 },
    { 0x4cd0d04a, SUC_lexicon_tags_12 },
    { 0x4b10d048, SUC_lexicon_tags_60 },
    { 0x550d04a, SUC_lexicon_tags_60 },
    { 0xcc90d04a, SUC_lexicon_tags_25 },
    { 0xf1f0d04e, SUC_lexicon_tags_42 },
    { 0x7ff0d04d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc370d051, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8470d054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd630d057, SUC_lexicon_tags_37 },
    { 0x2d70d058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb0d05e, SUC_lexicon_tags_48 },
    { 0x62d0d05f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b0d065, SUC_lexicon_tags_9 },
    { 0x3610d066, SUC_lexicon_tags_14 },
    { 0xb8d0d067, SUC_lexicon_tags_17 },
    { 0x4370d068, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b0d06b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f10d072, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4510d074, SUC_lexicon_tags_28 },
    { 0x6370d075, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0d079, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe70d07d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4f0d07f, SUC_lexicon_tags_56 },
    { 0xeb50d07f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd90d083, SUC_lexicon_tags_63 },
    { 0x3cb0d083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd70d08a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e10d08f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc50d09d, SUC_lexicon_tags_70 },
    { 0xba10d09d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed10d0a7, SUC_lexicon_tags_27 },
    { 0xb250d0a8, SUC_lexicon_tags_37 },
    { 0xaa30d0a8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf70d0ad, SUC_lexicon_tags_17 },
    { 0x7290d0ae, SUC_lexicon_tags_21 },
    { 0x26f0d0ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3210d0b4, SUC_lexicon_tags_21 },
    { 0x7cf0d0b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdcd0d0b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7950d0b9, SUC_lexicon_tags_0 },
    { 0xb030d0ba, SUC_lexicon_tags_45 },
    { 0xa090d0bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a50d0bf, SUC_lexicon_tags_27 },
    { 0x5750d0c0, SUC_lexicon_tags_48 },
    { 0x19d0d0c0, SUC_lexicon_tags_36 },
    { 0xb070d0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5950d0c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4290d0c8, SUC_lexicon_tags_699 },
    { 0, NULL },
    { 0xf290d0ca, SUC_lexicon_tags_1 },
    { 0x2b30d0cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7530d0cd, SUC_lexicon_tags_37 },
    { 0x2450d0ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf150d0d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8830d0d6, SUC_lexicon_tags_63 },
    { 0xeff0d0d7, SUC_lexicon_tags_17 },
    { 0x370d0d8, SUC_lexicon_tags_14 },
    { 0xab90d0d9, SUC_lexicon_tags_23 },
    { 0x44f0d0d9, SUC_lexicon_tags_2 },
    { 0x3ed0d0d8, SUC_lexicon_tags_47 },
    { 0x6090d0dc, SUC_lexicon_tags_7 },
    { 0x6390d0d7, SUC_lexicon_tags_7 },
    { 0xe10d0da, SUC_lexicon_tags_55 },
    { 0x3a90d0df, SUC_lexicon_tags_15 },
    { 0x8750d0df, SUC_lexicon_tags_45 },
    { 0xe2d0d0e1, SUC_lexicon_tags_61 },
    { 0x3f50d0e2, SUC_lexicon_tags_4 },
    { 0x1550d0e3, SUC_lexicon_tags_3 },
    { 0x3b90d0e3, SUC_lexicon_tags_0 },
    { 0x5770d0e5, SUC_lexicon_tags_19 },
    { 0xf7d0d0db, SUC_lexicon_tags_9 },
    { 0x16f0d0e7, SUC_lexicon_tags_9 },
    { 0x8f90d0e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc250d0ea, SUC_lexicon_tags_19 },
    { 0x4c90d0ea, SUC_lexicon_tags_31 },
    { 0x1a90d0eb, SUC_lexicon_tags_31 },
    { 0xd090d0ea, SUC_lexicon_tags_47 },
    { 0x4ab0d0ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf4b0d0f0, SUC_lexicon_tags_29 },
    { 0x1d30d0f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc30d0f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7110d0f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd750d0f9, SUC_lexicon_tags_12 },
    { 0xccb0d0f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b0d0ff, SUC_lexicon_tags_47 },
    { 0xbd10d0ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d10d102, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f0d10a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb650d10c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xebf0d10e, SUC_lexicon_tags_47 },
    { 0xf090d10e, SUC_lexicon_tags_25 },
    { 0x60d0d10e, SUC_lexicon_tags_47 },
    { 0xad10d111, SUC_lexicon_tags_29 },
    { 0x94b0d112, SUC_lexicon_tags_11 },
    { 0x7690d113, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e50d11b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f0d11e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x4710d121, SUC_lexicon_tags_0 },
    { 0xbd0d122, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x850d125, SUC_lexicon_tags_9 },
    { 0x9e50d126, SUC_lexicon_tags_49 },
    { 0x3ad0d127, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb0d12c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0d131, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa90d133, SUC_lexicon_tags_26 },
    { 0xe6b0d134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2610d136, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5690d13c, SUC_lexicon_tags_0 },
    { 0x56f0d13d, SUC_lexicon_tags_29 },
    { 0xcd10d13c, SUC_lexicon_tags_37 },
    { 0xaf0d13f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9710d143, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb0d14a, SUC_lexicon_tags_31 },
    { 0xba10d14b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd0d14f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1490d15d, SUC_lexicon_tags_4 },
    { 0xd730d15e, SUC_lexicon_tags_17 },
    { 0x31d0d15d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5350d161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5070d163, SUC_lexicon_tags_11 },
    { 0x7990d164, SUC_lexicon_tags_32 },
    { 0x3750d164, SUC_lexicon_tags_17 },
    { 0x9290d166, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8890d169, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3350d16c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f30d174, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x770d177, SUC_lexicon_tags_55 },
    { 0x5c70d177, SUC_lexicon_tags_45 },
    { 0x7bd0d179, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab0d17d, SUC_lexicon_tags_65 },
    { 0x2770d17d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf870d180, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d90d184, SUC_lexicon_tags_7 },
    { 0x1db0d184, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7630d187, SUC_lexicon_tags_37 },
    { 0xc2f0d187, SUC_lexicon_tags_45 },
    { 0x2730d187, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x410d18c, SUC_lexicon_tags_12 },
    { 0x39b0d18c, SUC_lexicon_tags_15 },
    { 0xe5b0d18e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6670d193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9450d196, SUC_lexicon_tags_84 },
    { 0x1a50d197, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9750d199, SUC_lexicon_tags_7 },
    { 0xcbb0d19a, SUC_lexicon_tags_29 },
    { 0x81f0d19a, SUC_lexicon_tags_12 },
    { 0xbed0d19c, SUC_lexicon_tags_25 },
    { 0x60f0d199, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd0d1a0, SUC_lexicon_tags_25 },
    { 0xbaf0d1a0, SUC_lexicon_tags_2 },
    { 0xde70d1a1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d50d1a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0d1a8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce50d1ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x11f0d1ae, SUC_lexicon_tags_1 },
    { 0x7090d1af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd0d1b2, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x2050d1b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb890d1b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9370d1bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc90d1bf, SUC_lexicon_tags_11 },
    { 0x34d0d1bf, SUC_lexicon_tags_12 },
    { 0x5e10d1c1, SUC_lexicon_tags_26 },
    { 0x35b0d1c2, SUC_lexicon_tags_6 },
    { 0x8d50d1bf, SUC_lexicon_tags_17 },
    { 0x3350d1c1, SUC_lexicon_tags_57 },
    { 0xd7d0d1c1, SUC_lexicon_tags_11 },
    { 0x5a50d1c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc30d1cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5030d1cf, SUC_lexicon_tags_15 },
    { 0xdc10d1d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50d1da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9250d1dc, SUC_lexicon_tags_30 },
    { 0xe810d1dd, SUC_lexicon_tags_182 },
    { 0xfe30d1de, SUC_lexicon_tags_29 },
    { 0xe6d0d1df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5110d1e5, SUC_lexicon_tags_25 },
    { 0x5750d1e6, SUC_lexicon_tags_12 },
    { 0x8130d1e6, SUC_lexicon_tags_28 },
    { 0x2470d1e8, SUC_lexicon_tags_37 },
    { 0x9c10d1e9, SUC_lexicon_tags_27 },
    { 0x3a30d1ea, SUC_lexicon_tags_8 },
    { 0x8bb0d1eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbb0d1ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5d30d1ef, SUC_lexicon_tags_12 },
    { 0x4990d1f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0d1f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f0d1f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff0d1fe, SUC_lexicon_tags_30 },
    { 0x56f0d1ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf30d201, SUC_lexicon_tags_23 },
    { 0x4890d201, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f0d20b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb90d20e, SUC_lexicon_tags_32 },
    { 0x2390d20e, SUC_lexicon_tags_26 },
    { 0x4150d20e, SUC_lexicon_tags_30 },
    { 0x91f0d211, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd0d216, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdbb0d218, SUC_lexicon_tags_9 },
    { 0x1570d219, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb0d21c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc150d21e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xafd0d220, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca10d226, SUC_lexicon_tags_1 },
    { 0xf0b0d227, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe730d229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8190d22c, SUC_lexicon_tags_7 },
    { 0xc510d22d, SUC_lexicon_tags_18 },
    { 0x7350d22d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2690d232, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5910d235, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d0d238, SUC_lexicon_tags_91 },
    { 0x46f0d239, SUC_lexicon_tags_49 },
    { 0xdc30d23a, SUC_lexicon_tags_1 },
    { 0x8910d239, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe10d23f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5590d246, SUC_lexicon_tags_23 },
    { 0xcfb0d247, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4ed0d249, SUC_lexicon_tags_24 },
    { 0x2e70d24a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec30d24c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb210d24f, SUC_lexicon_tags_11 },
    { 0x6450d250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe50d252, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6370d256, SUC_lexicon_tags_2 },
    { 0x6630d256, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a90d25e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb90d262, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe010d264, SUC_lexicon_tags_25 },
    { 0x1070d265, SUC_lexicon_tags_15 },
    { 0x8d50d266, SUC_lexicon_tags_27 },
    { 0x9030d265, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb0d26e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe410d272, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x23d0d274, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf0d278, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d0d27b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f0d281, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x33b0d283, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d0d287, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d0d28a, SUC_lexicon_tags_25 },
    { 0x2a10d28a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb810d28e, SUC_lexicon_tags_27 },
    { 0x28f0d28f, SUC_lexicon_tags_37 },
    { 0xedf0d290, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf990d294, SUC_lexicon_tags_43 },
    { 0xebb0d294, SUC_lexicon_tags_46 },
    { 0xf7f0d296, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f50d298, SUC_lexicon_tags_25 },
    { 0x5b30d299, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf630d2a4, SUC_lexicon_tags_33 },
    { 0xc690d2a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7250d2a7, SUC_lexicon_tags_31 },
    { 0xd250d2a8, SUC_lexicon_tags_11 },
    { 0x650d2a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb770d2ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5790d2af, SUC_lexicon_tags_32 },
    { 0xc050d2b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f10d2b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d0d2b8, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb810d2be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4390d2c1, SUC_lexicon_tags_4 },
    { 0xa190d2c2, SUC_lexicon_tags_21 },
    { 0x4e50d2c2, SUC_lexicon_tags_17 },
    { 0x23f0d2c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b90d2c6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6050d2cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3130d2d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a50d2dc, SUC_lexicon_tags_47 },
    { 0xc9f0d2dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8430d2df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2390d2e1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f50d2e4, SUC_lexicon_tags_12 },
    { 0xef10d2e5, SUC_lexicon_tags_60 },
    { 0xf7b0d2e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf950d2e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec10d2ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa150d2f4, SUC_lexicon_tags_15 },
    { 0x8750d2f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c50d2f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d0d2fb, SUC_lexicon_tags_38 },
    { 0xdf90d2fb, SUC_lexicon_tags_11 },
    { 0x16d0d2fd, SUC_lexicon_tags_2 },
    { 0x5530d2fe, SUC_lexicon_tags_1 },
    { 0xa50d2fb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa050d301, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad0d305, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa150d308, SUC_lexicon_tags_41 },
    { 0x2b90d309, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe70d30b, SUC_lexicon_tags_25 },
    { 0x250d30c, SUC_lexicon_tags_49 },
    { 0x2d0d30b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd8d0d30f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4bb0d311, SUC_lexicon_tags_25 },
    { 0xd8d0d311, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b0d315, SUC_lexicon_tags_37 },
    { 0xdc10d316, SUC_lexicon_tags_17 },
    { 0xcd90d317, SUC_lexicon_tags_1 },
    { 0xcd0d318, SUC_lexicon_tags_11 },
    { 0x4170d319, SUC_lexicon_tags_25 },
    { 0x1030d31a, SUC_lexicon_tags_12 },
    { 0x30b0d31b, SUC_lexicon_tags_37 },
    { 0x8bb0d31c, SUC_lexicon_tags_15 },
    { 0x5990d316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6810d31f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1690d322, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf670d334, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb0d337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf510d339, SUC_lexicon_tags_30 },
    { 0x5490d339, SUC_lexicon_tags_29 },
    { 0x4fb0d339, SUC_lexicon_tags_7 },
    { 0xa070d33c, SUC_lexicon_tags_47 },
    { 0xe190d33b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8210d340, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530d347, SUC_lexicon_tags_68 },
    { 0x1870d347, SUC_lexicon_tags_7 },
    { 0xe0b0d349, SUC_lexicon_tags_7 },
    { 0x670d34a, SUC_lexicon_tags_17 },
    { 0x8fb0d34b, SUC_lexicon_tags_25 },
    { 0x9470d348, SUC_lexicon_tags_44 },
    { 0x3790d34d, SUC_lexicon_tags_60 },
    { 0x7dd0d34d, SUC_lexicon_tags_224 },
    { 0x7850d34f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe670d356, SUC_lexicon_tags_45 },
    { 0x210d357, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa290d35a, SUC_lexicon_tags_14 },
    { 0x1a10d35a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf30d35e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda90d363, SUC_lexicon_tags_1 },
    { 0xb770d364, SUC_lexicon_tags_47 },
    { 0x3e70d365, SUC_lexicon_tags_140 },
    { 0x29b0d364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf810d36c, SUC_lexicon_tags_47 },
    { 0xbc30d36d, SUC_lexicon_tags_7 },
    { 0x6150d36d, SUC_lexicon_tags_43 },
    { 0xbb90d36f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb0d373, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b0d376, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc10d380, SUC_lexicon_tags_33 },
    { 0x6690d380, SUC_lexicon_tags_1 },
    { 0x31f0d382, SUC_lexicon_tags_14 },
    { 0xd110d383, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310d392, SUC_lexicon_tags_25 },
    { 0xc430d392, SUC_lexicon_tags_30 },
    { 0xdd90d393, SUC_lexicon_tags_0 },
    { 0xf250d393, SUC_lexicon_tags_4 },
    { 0xdeb0d396, SUC_lexicon_tags_7 },
    { 0x4650d392, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fb0d399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d39f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9330d3a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f0d3a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e70d3a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff30d3b0, SUC_lexicon_tags_51 },
    { 0xc930d3b1, SUC_lexicon_tags_33 },
    { 0x890d3b2, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xb0f0d3b4, SUC_lexicon_tags_63 },
    { 0x77b0d3b5, SUC_lexicon_tags_17 },
    { 0xe5f0d3b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb0d3bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf30d3bd, SUC_lexicon_tags_23 },
    { 0x9d90d3be, SUC_lexicon_tags_2 },
    { 0xb170d3bf, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa790d3c4, SUC_lexicon_tags_37 },
    { 0x3890d3c4, SUC_lexicon_tags_15 },
    { 0x6b0d3c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd0d3ca, SUC_lexicon_tags_10 },
    { 0x2ed0d3ca, SUC_lexicon_tags_55 },
    { 0x16d0d3ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf150d3d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1190d3dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe210d3df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3030d3e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a90d3e6, SUC_lexicon_tags_17 },
    { 0xb890d3e7, SUC_lexicon_tags_11 },
    { 0xe190d3e7, SUC_lexicon_tags_7 },
    { 0x79d0d3e6, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9390d3ee, SUC_lexicon_tags_49 },
    { 0x46d0d3ee, SUC_lexicon_tags_17 },
    { 0xc810d3f0, SUC_lexicon_tags_8 },
    { 0x150d3ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ef0d3f3, SUC_lexicon_tags_26 },
    { 0x34b0d3f4, SUC_lexicon_tags_12 },
    { 0x3130d3f5, SUC_lexicon_tags_12 },
    { 0x9d30d3f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8690d3fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x330d3fd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xc690d400, SUC_lexicon_tags_33 },
    { 0x6650d401, SUC_lexicon_tags_55 },
    { 0x4590d401, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3810d405, SUC_lexicon_tags_27 },
    { 0x9d50d406, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9e70d408, SUC_lexicon_tags_45 },
    { 0x6cf0d408, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b0d40c, SUC_lexicon_tags_15 },
    { 0x5a50d40d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde50d40f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4770d411, SUC_lexicon_tags_37 },
    { 0x17d0d411, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f0d419, SUC_lexicon_tags_15 },
    { 0x2ed0d41a, SUC_lexicon_tags_79 },
    { 0x1230d41b, SUC_lexicon_tags_19 },
    { 0xa550d41c, SUC_lexicon_tags_32 },
    { 0xd710d41b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbff0d41f, SUC_lexicon_tags_13 },
    { 0x2830d41f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cb0d422, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a10d426, SUC_lexicon_tags_11 },
    { 0x7070d427, SUC_lexicon_tags_10 },
    { 0x3d70d427, SUC_lexicon_tags_17 },
    { 0x92d0d429, SUC_lexicon_tags_23 },
    { 0xb230d42a, SUC_lexicon_tags_11 },
    { 0xb990d42a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x18d0d42d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f50d430, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd0d433, SUC_lexicon_tags_12 },
    { 0x99f0d433, SUC_lexicon_tags_1 },
    { 0xa1f0d435, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f90d438, SUC_lexicon_tags_35 },
    { 0x9f90d439, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6290d43b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xded0d43e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf170d442, SUC_lexicon_tags_60 },
    { 0xaa70d442, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd290d447, SUC_lexicon_tags_38 },
    { 0xd490d447, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb750d44b, SUC_lexicon_tags_7 },
    { 0x77b0d44c, SUC_lexicon_tags_32 },
    { 0xb6f0d44d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4e10d44f, SUC_lexicon_tags_11 },
    { 0x6110d44f, SUC_lexicon_tags_60 },
    { 0xe7b0d450, SUC_lexicon_tags_30 },
    { 0x5d90d450, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x4270d454, SUC_lexicon_tags_12 },
    { 0x9f0d455, SUC_lexicon_tags_13 },
    { 0x4810d456, SUC_lexicon_tags_7 },
    { 0x1ad0d457, SUC_lexicon_tags_37 },
    { 0x9630d458, SUC_lexicon_tags_37 },
    { 0xfed0d459, SUC_lexicon_tags_60 },
    { 0xd1d0d458, SUC_lexicon_tags_15 },
    { 0x7b70d455, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f0d463, SUC_lexicon_tags_6 },
    { 0x2550d464, SUC_lexicon_tags_27 },
    { 0x92d0d463, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb0d469, SUC_lexicon_tags_0 },
    { 0x9eb0d46a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1730d46d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5470d472, SUC_lexicon_tags_47 },
    { 0x29d0d473, SUC_lexicon_tags_42 },
    { 0xa170d473, SUC_lexicon_tags_9 },
    { 0x1350d472, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d0d47a, SUC_lexicon_tags_66 },
    { 0xb0b0d47b, SUC_lexicon_tags_25 },
    { 0x7c90d47a, SUC_lexicon_tags_14 },
    { 0xff50d47d, SUC_lexicon_tags_1 },
    { 0xf7d0d47e, SUC_lexicon_tags_14 },
    { 0x17d0d47f, SUC_lexicon_tags_17 },
    { 0xc4b0d47e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b0d484, SUC_lexicon_tags_129 },
    { 0xfdd0d485, SUC_lexicon_tags_25 },
    { 0xbbf0d486, SUC_lexicon_tags_3 },
    { 0xb6b0d487, SUC_lexicon_tags_26 },
    { 0x5830d485, SUC_lexicon_tags_4 },
    { 0xaad0d487, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30d48e, SUC_lexicon_tags_26 },
    { 0x1e30d48f, SUC_lexicon_tags_45 },
    { 0x3530d490, SUC_lexicon_tags_12 },
    { 0xe7b0d491, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d50d494, SUC_lexicon_tags_46 },
    { 0x9c30d495, SUC_lexicon_tags_38 },
    { 0xbf0d496, SUC_lexicon_tags_7 },
    { 0x1070d497, SUC_lexicon_tags_7 },
    { 0x6990d498, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d50d49b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4ab0d49d, SUC_lexicon_tags_15 },
    { 0xe0d0d49d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9450d4a2, SUC_lexicon_tags_11 },
    { 0xd090d4a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7310d4a5, SUC_lexicon_tags_25 },
    { 0x79f0d4a5, SUC_lexicon_tags_11 },
    { 0xbc70d4a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa870d4ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9610d4b0, SUC_lexicon_tags_48 },
    { 0x690d4b1, SUC_lexicon_tags_19 },
    { 0x9cb0d4b2, SUC_lexicon_tags_2 },
    { 0xee10d4b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb0d4b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f0d4bd, SUC_lexicon_tags_47 },
    { 0xebf0d4be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e90d4c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5090d4c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d0d4cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc830d4ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6650d4d1, SUC_lexicon_tags_46 },
    { 0x78d0d4d2, SUC_lexicon_tags_87 },
    { 0xc90d4d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1230d4e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17b0d4e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xda90d4e6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd0d4ec, SUC_lexicon_tags_2 },
    { 0xfd30d4ed, SUC_lexicon_tags_5 },
    { 0xa870d4ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4610d4f0, SUC_lexicon_tags_61 },
    { 0xd5d0d4f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec30d4f6, SUC_lexicon_tags_11 },
    { 0xe9d0d4f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3f0d4f9, SUC_lexicon_tags_19 },
    { 0x5bf0d4fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b90d4fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ad0d4fe, SUC_lexicon_tags_7 },
    { 0xcc50d4ff, SUC_lexicon_tags_35 },
    { 0xef70d500, SUC_lexicon_tags_80 },
    { 0x8550d500, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc130d506, SUC_lexicon_tags_1 },
    { 0x9490d506, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc90d509, SUC_lexicon_tags_32 },
    { 0x4cb0d509, SUC_lexicon_tags_38 },
    { 0xad90d50a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d0d516, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9c90d518, SUC_lexicon_tags_12 },
    { 0x7e50d519, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x19d0d51b, SUC_lexicon_tags_29 },
    { 0xceb0d51c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda90d51f, SUC_lexicon_tags_25 },
    { 0xa330d51f, SUC_lexicon_tags_56 },
    { 0x64b0d521, SUC_lexicon_tags_49 },
    { 0x3a90d522, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f0d52a, SUC_lexicon_tags_33 },
    { 0x74b0d52b, SUC_lexicon_tags_4 },
    { 0x7ab0d52a, SUC_lexicon_tags_33 },
    { 0x5090d52d, SUC_lexicon_tags_17 },
    { 0x9b70d52e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3610d534, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2250d53b, SUC_lexicon_tags_47 },
    { 0x9a10d53c, SUC_lexicon_tags_2 },
    { 0xb010d53c, SUC_lexicon_tags_27 },
    { 0xebd0d53e, SUC_lexicon_tags_3 },
    { 0x5bb0d53f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f0d543, SUC_lexicon_tags_25 },
    { 0x9dd0d543, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdd50d546, SUC_lexicon_tags_23 },
    { 0x2590d547, SUC_lexicon_tags_37 },
    { 0x34b0d548, SUC_lexicon_tags_16 },
    { 0xf030d549, SUC_lexicon_tags_7 },
    { 0x1290d546, SUC_lexicon_tags_61 },
    { 0x77b0d54b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb050d54e, SUC_lexicon_tags_14 },
    { 0x72f0d54f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1530d552, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd0d556, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d0d559, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850d566, SUC_lexicon_tags_1 },
    { 0xb050d567, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a30d569, SUC_lexicon_tags_18 },
    { 0xbdd0d569, SUC_lexicon_tags_14 },
    { 0xf6d0d56b, SUC_lexicon_tags_27 },
    { 0xc70d56b, SUC_lexicon_tags_38 },
    { 0x7070d56b, SUC_lexicon_tags_56 },
    { 0x7330d56a, SUC_lexicon_tags_12 },
    { 0x17b0d56f, SUC_lexicon_tags_9 },
    { 0xd090d56f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd450d572, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed90d577, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1930d579, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa870d57e, SUC_lexicon_tags_37 },
    { 0x1ef0d57f, SUC_lexicon_tags_2 },
    { 0x34f0d580, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5350d586, SUC_lexicon_tags_12 },
    { 0x48b0d587, SUC_lexicon_tags_234 },
    { 0x69d0d588, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d0d590, SUC_lexicon_tags_37 },
    { 0xd310d591, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x64f0d593, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d0d598, SUC_lexicon_tags_65 },
    { 0xb290d599, SUC_lexicon_tags_12 },
    { 0x8030d59a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8c70d59c, SUC_lexicon_tags_1 },
    { 0x9850d59d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d0d5a2, SUC_lexicon_tags_1 },
    { 0x9cf0d5a3, SUC_lexicon_tags_11 },
    { 0xad70d5a3, SUC_lexicon_tags_23 },
    { 0x1ed0d5a5, SUC_lexicon_tags_30 },
    { 0x5870d5a6, SUC_lexicon_tags_25 },
    { 0xef50d5a2, SUC_lexicon_tags_13 },
    { 0xe9d0d5a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd0d5ab, SUC_lexicon_tags_2 },
    { 0x3f50d5ac, SUC_lexicon_tags_27 },
    { 0xf610d5ab, SUC_lexicon_tags_31 },
    { 0x4970d5ac, SUC_lexicon_tags_7 },
    { 0x8f30d5af, SUC_lexicon_tags_11 },
    { 0x2370d5b0, SUC_lexicon_tags_37 },
    { 0xbcd0d5b1, SUC_lexicon_tags_11 },
    { 0xb0f0d5af, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xfc10d5b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a70d5ba, SUC_lexicon_tags_12 },
    { 0x5f50d5bb, SUC_lexicon_tags_37 },
    { 0x6610d5bc, SUC_lexicon_tags_25 },
    { 0xaad0d5bb, SUC_lexicon_tags_25 },
    { 0x8d0d5be, SUC_lexicon_tags_13 },
    { 0x9bb0d5be, SUC_lexicon_tags_27 },
    { 0x1ab0d5c0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8970d5c2, SUC_lexicon_tags_0 },
    { 0x2d0d5c3, SUC_lexicon_tags_31 },
    { 0x23d0d5c4, SUC_lexicon_tags_2 },
    { 0xdb90d5c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3190d5c7, SUC_lexicon_tags_27 },
    { 0x6570d5c7, SUC_lexicon_tags_37 },
    { 0x5a50d5c7, SUC_lexicon_tags_25 },
    { 0x3870d5c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7170d5ce, SUC_lexicon_tags_11 },
    { 0xcf30d5cf, SUC_lexicon_tags_25 },
    { 0xa350d5ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1b0d5d2, SUC_lexicon_tags_61 },
    { 0x6850d5d2, SUC_lexicon_tags_23 },
    { 0x2e90d5d4, SUC_lexicon_tags_17 },
    { 0xc950d5d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71b0d5d7, SUC_lexicon_tags_1 },
    { 0xc950d5d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe630d5db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e30d5e2, SUC_lexicon_tags_7 },
    { 0x9750d5e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4150d5e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd070d5e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6030d5ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1830d5ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b90d5f6, SUC_lexicon_tags_38 },
    { 0x7750d5f6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9690d5f9, SUC_lexicon_tags_43 },
    { 0x5ad0d5fa, SUC_lexicon_tags_25 },
    { 0xd030d5fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb0d5ff, SUC_lexicon_tags_25 },
    { 0x6f0d600, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b0d603, SUC_lexicon_tags_11 },
    { 0x8370d604, SUC_lexicon_tags_25 },
    { 0xdfb0d604, SUC_lexicon_tags_15 },
    { 0xfb90d606, SUC_lexicon_tags_2 },
    { 0xf350d607, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4530d60a, SUC_lexicon_tags_45 },
    { 0xd2b0d60b, SUC_lexicon_tags_1 },
    { 0x8290d60c, SUC_lexicon_tags_37 },
    { 0x1fd0d60b, SUC_lexicon_tags_17 },
    { 0xfdf0d60b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b0d612, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb5b0d614, SUC_lexicon_tags_65 },
    { 0xa590d615, SUC_lexicon_tags_37 },
    { 0x4390d614, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d10d618, SUC_lexicon_tags_37 },
    { 0x9fb0d619, SUC_lexicon_tags_11 },
    { 0xec90d619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3510d61c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f30d61e, SUC_lexicon_tags_79 },
    { 0x1150d61f, SUC_lexicon_tags_37 },
    { 0x4a70d61e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba70d624, SUC_lexicon_tags_65 },
    { 0xf7b0d625, SUC_lexicon_tags_3 },
    { 0x3290d626, SUC_lexicon_tags_12 },
    { 0xab10d626, SUC_lexicon_tags_25 },
    { 0x46f0d624, SUC_lexicon_tags_47 },
    { 0xa7d0d625, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce30d62c, SUC_lexicon_tags_6 },
    { 0x6ab0d62c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3630d630, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5370d633, SUC_lexicon_tags_7 },
    { 0xda50d634, SUC_lexicon_tags_1 },
    { 0x7fd0d633, SUC_lexicon_tags_3 },
    { 0x2ef0d635, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9cb0d638, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf0d63b, SUC_lexicon_tags_37 },
    { 0x950d63b, SUC_lexicon_tags_29 },
    { 0xff10d63d, SUC_lexicon_tags_1 },
    { 0x3dd0d63e, SUC_lexicon_tags_47 },
    { 0xb9b0d63b, SUC_lexicon_tags_20 },
    { 0xc7d0d63e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d0d645, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f70d647, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d0d64b, SUC_lexicon_tags_2 },
    { 0xae10d64c, SUC_lexicon_tags_47 },
    { 0x88d0d64d, SUC_lexicon_tags_37 },
    { 0xb90d64b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a70d653, SUC_lexicon_tags_47 },
    { 0x1e70d654, SUC_lexicon_tags_0 },
    { 0x2e10d655, SUC_lexicon_tags_60 },
    { 0x1cb0d656, SUC_lexicon_tags_61 },
    { 0xb230d656, SUC_lexicon_tags_19 },
    { 0x5a70d658, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9370d65b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc210d65e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb0d666, SUC_lexicon_tags_1 },
    { 0x2e70d667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd30d66a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c30d66e, SUC_lexicon_tags_57 },
    { 0x24f0d66f, SUC_lexicon_tags_31 },
    { 0xacd0d670, SUC_lexicon_tags_1 },
    { 0x2870d670, SUC_lexicon_tags_27 },
    { 0xdcf0d672, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc690d675, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb070d67d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84f0d67f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe30d689, SUC_lexicon_tags_151 },
    { 0x7c30d68a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb0d690, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c50d693, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5d0d695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68d0d697, SUC_lexicon_tags_77 },
    { 0x8a30d698, SUC_lexicon_tags_15 },
    { 0x3670d697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4910d6a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ef0d6a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d0d6a7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfd50d6a9, SUC_lexicon_tags_27 },
    { 0x8a10d6aa, SUC_lexicon_tags_6 },
    { 0xb950d6ab, SUC_lexicon_tags_45 },
    { 0xdf70d6a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d0d6b1, SUC_lexicon_tags_17 },
    { 0xed30d6b2, SUC_lexicon_tags_31 },
    { 0x1190d6b3, SUC_lexicon_tags_12 },
    { 0x7070d6b4, SUC_lexicon_tags_17 },
    { 0x7b30d6b3, SUC_lexicon_tags_68 },
    { 0xd270d6b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1990d6bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f0d6be, SUC_lexicon_tags_54 },
    { 0x5210d6bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d0d6c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b0d6c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5210d6cb, SUC_lexicon_tags_17 },
    { 0xb090d6cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d0d6d6, SUC_lexicon_tags_15 },
    { 0x7ed0d6d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd350d6db, SUC_lexicon_tags_47 },
    { 0x3c30d6db, SUC_lexicon_tags_66 },
    { 0x90f0d6dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd110d6e1, SUC_lexicon_tags_14 },
    { 0x8530d6e2, SUC_lexicon_tags_37 },
    { 0xeeb0d6e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd170d6e7, SUC_lexicon_tags_2 },
    { 0x2190d6e8, SUC_lexicon_tags_12 },
    { 0x1d0d6e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2970d6eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f0d6ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb0d6fc, SUC_lexicon_tags_56 },
    { 0x1170d6fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb230d700, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5510d704, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0d70b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd0d70e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe690d714, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f0d717, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f0d71b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb950d720, SUC_lexicon_tags_60 },
    { 0x4350d721, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3870d728, SUC_lexicon_tags_37 },
    { 0xcbb0d729, SUC_lexicon_tags_7 },
    { 0xe730d729, SUC_lexicon_tags_9 },
    { 0x9990d72a, SUC_lexicon_tags_25 },
    { 0xf70d72c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xde50d72e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d0d732, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed0d736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd390d73a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae10d73d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b90d73f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f0d743, SUC_lexicon_tags_49 },
    { 0x6fb0d744, SUC_lexicon_tags_17 },
    { 0xf0b0d745, SUC_lexicon_tags_2 },
    { 0xbaf0d746, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xed70d748, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3230d74d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a50d74f, SUC_lexicon_tags_255 },
    { 0x5690d74f, SUC_lexicon_tags_19 },
    { 0xe570d750, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd0d755, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb5b0d757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa390d759, SUC_lexicon_tags_3 },
    { 0x8090d75a, SUC_lexicon_tags_45 },
    { 0x28f0d75b, SUC_lexicon_tags_1 },
    { 0x3f10d75b, SUC_lexicon_tags_61 },
    { 0x6850d75d, SUC_lexicon_tags_28 },
    { 0x39f0d75e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3910d763, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a90d766, SUC_lexicon_tags_7 },
    { 0x1990d767, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d0d76a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6f0d76c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3830d772, SUC_lexicon_tags_11 },
    { 0x5db0d773, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3490d775, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4cd0d777, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad0d77a, SUC_lexicon_tags_49 },
    { 0x6870d77b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3770d77f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x17d0d781, SUC_lexicon_tags_6 },
    { 0xf230d781, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd0d787, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b90d78e, SUC_lexicon_tags_14 },
    { 0x8c90d78f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9110d796, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc510d799, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5210d79b, SUC_lexicon_tags_33 },
    { 0x1490d79b, SUC_lexicon_tags_1 },
    { 0x1330d79d, SUC_lexicon_tags_23 },
    { 0xb530d79e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xce70d7a0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x28f0d7a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf850d7a4, SUC_lexicon_tags_10 },
    { 0x8f0d7a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcaf0d7a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4330d7a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe490d7ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9430d7ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9950d7b1, SUC_lexicon_tags_14 },
    { 0xe290d7b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4750d7b6, SUC_lexicon_tags_7 },
    { 0x6710d7b6, SUC_lexicon_tags_9 },
    { 0x28d0d7b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc490d7ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f0d7be, SUC_lexicon_tags_37 },
    { 0xab70d7bf, SUC_lexicon_tags_1 },
    { 0xdf10d7bf, SUC_lexicon_tags_11 },
    { 0x5fb0d7c1, SUC_lexicon_tags_17 },
    { 0xa990d7c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb950d7c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d0d7cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28b0d7cd, SUC_lexicon_tags_37 },
    { 0xd230d7cd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f50d7d1, SUC_lexicon_tags_7 },
    { 0x1990d7d2, SUC_lexicon_tags_13 },
    { 0xbc90d7d3, SUC_lexicon_tags_38 },
    { 0x6b10d7d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe850d7d6, SUC_lexicon_tags_7 },
    { 0x68d0d7d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7970d7db, SUC_lexicon_tags_23 },
    { 0x87d0d7dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d0d7df, SUC_lexicon_tags_18 },
    { 0x9490d7e0, SUC_lexicon_tags_23 },
    { 0x4390d7e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e70d7e3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed0d7e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab0d7eb, SUC_lexicon_tags_62 },
    { 0xd610d7ec, SUC_lexicon_tags_65 },
    { 0xb6b0d7eb, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70d7f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b0d7f4, SUC_lexicon_tags_11 },
    { 0xb4b0d7f5, SUC_lexicon_tags_8 },
    { 0xb4d0d7f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99d0d7f8, SUC_lexicon_tags_193 },
    { 0xba90d7f8, SUC_lexicon_tags_25 },
    { 0xdc30d7f9, SUC_lexicon_tags_11 },
    { 0xcdd0d7f9, SUC_lexicon_tags_8 },
    { 0x5530d7fc, SUC_lexicon_tags_46 },
    { 0xcdf0d7fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9010d806, SUC_lexicon_tags_12 },
    { 0xce50d807, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d0d80a, SUC_lexicon_tags_63 },
    { 0xa470d80b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9f0d80d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d90d811, SUC_lexicon_tags_7 },
    { 0x30d812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6310d818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9350d81b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe90d81e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe50d822, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe870d829, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xe250d82b, SUC_lexicon_tags_2 },
    { 0x1750d82c, SUC_lexicon_tags_25 },
    { 0xfa70d82d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xd570d830, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d0d834, SUC_lexicon_tags_12 },
    { 0xeb90d835, SUC_lexicon_tags_37 },
    { 0xe630d836, SUC_lexicon_tags_29 },
    { 0x86d0d837, SUC_lexicon_tags_2 },
    { 0xb10d838, SUC_lexicon_tags_27 },
    { 0x9ad0d839, SUC_lexicon_tags_23 },
    { 0x2390d839, SUC_lexicon_tags_1 },
    { 0xff70d838, SUC_lexicon_tags_1 },
    { 0x8090d83c, SUC_lexicon_tags_25 },
    { 0xc6f0d839, SUC_lexicon_tags_83 },
    { 0xa9b0d83e, SUC_lexicon_tags_29 },
    { 0xe8d0d83f, SUC_lexicon_tags_45 },
    { 0x670d840, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7d90d842, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d0d847, SUC_lexicon_tags_11 },
    { 0x88b0d848, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f0d850, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e70d852, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f0d855, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b0d859, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d0d864, SUC_lexicon_tags_38 },
    { 0x9af0d865, SUC_lexicon_tags_9 },
    { 0xf790d865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa230d86b, SUC_lexicon_tags_1 },
    { 0x7110d86c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb890d86f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc190d874, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xc3f0d876, SUC_lexicon_tags_21 },
    { 0xb770d877, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2750d879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd510d87d, SUC_lexicon_tags_37 },
    { 0x8a50d87e, SUC_lexicon_tags_17 },
    { 0xd410d87e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad0d882, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb0d885, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed90d889, SUC_lexicon_tags_7 },
    { 0x8150d889, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd4b0d88c, SUC_lexicon_tags_26 },
    { 0xc310d88c, SUC_lexicon_tags_7 },
    { 0xfeb0d88c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbd0d890, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f0d893, SUC_lexicon_tags_17 },
    { 0x8210d893, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1130d897, SUC_lexicon_tags_48 },
    { 0xb730d898, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70d89b, SUC_lexicon_tags_6 },
    { 0xbf50d89b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e90d8a0, SUC_lexicon_tags_39 },
    { 0x2df0d8a1, SUC_lexicon_tags_20 },
    { 0x7e70d8a2, SUC_lexicon_tags_27 },
    { 0x9f50d8a2, SUC_lexicon_tags_47 },
    { 0xd030d8a4, SUC_lexicon_tags_103 },
    { 0x34f0d8a5, SUC_lexicon_tags_37 },
    { 0x1930d8a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6b0d8a8, SUC_lexicon_tags_12 },
    { 0x18d0d8a9, SUC_lexicon_tags_34 },
    { 0x56f0d8a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef0d8b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb0d8b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7810d8b9, SUC_lexicon_tags_11 },
    { 0xea50d8ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe030d8bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b30d8cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae30d8cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9390d8cf, SUC_lexicon_tags_11 },
    { 0xd30d8cf, SUC_lexicon_tags_38 },
    { 0xbbf0d8d1, SUC_lexicon_tags_45 },
    { 0x8730d8d2, SUC_lexicon_tags_37 },
    { 0xe0d0d8d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd330d8d9, SUC_lexicon_tags_37 },
    { 0xa30d8da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x57d0d8dc, SUC_lexicon_tags_37 },
    { 0xb4d0d8dc, SUC_lexicon_tags_47 },
    { 0xc510d8dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1690d8e0, SUC_lexicon_tags_11 },
    { 0xdcf0d8e1, SUC_lexicon_tags_37 },
    { 0xd090d8e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf930d8e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d0d8e8, SUC_lexicon_tags_81 },
    { 0x8110d8e8, SUC_lexicon_tags_43 },
    { 0x98b0d8ea, SUC_lexicon_tags_38 },
    { 0x7030d8ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8470d8ee, SUC_lexicon_tags_16 },
    { 0x40f0d8ef, SUC_lexicon_tags_7 },
    { 0x5e50d8f0, SUC_lexicon_tags_3 },
    { 0x57b0d8ef, SUC_lexicon_tags_14 },
    { 0xc210d8f1, SUC_lexicon_tags_27 },
    { 0x1650d8ef, SUC_lexicon_tags_28 },
    { 0xbc50d8f4, SUC_lexicon_tags_46 },
    { 0xa970d8f2, SUC_lexicon_tags_37 },
    { 0xc010d8f6, SUC_lexicon_tags_144 },
    { 0x4bd0d8f6, SUC_lexicon_tags_2 },
    { 0x1290d8f8, SUC_lexicon_tags_21 },
    { 0x6790d8f8, SUC_lexicon_tags_37 },
    { 0x1550d8f7, SUC_lexicon_tags_2 },
    { 0x4630d8fb, SUC_lexicon_tags_29 },
    { 0x6bb0d8f8, SUC_lexicon_tags_118 },
    { 0xd070d8fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe370d8ff, SUC_lexicon_tags_25 },
    { 0x43d0d900, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b0d904, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4430d907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf10d90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b30d90d, SUC_lexicon_tags_12 },
    { 0x35f0d90e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec30d910, SUC_lexicon_tags_1721 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f0d917, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xed30d919, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97f0d91b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1890d91f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb6b0d921, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c50d923, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af0d926, SUC_lexicon_tags_65 },
    { 0x9d0d926, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3490d92c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbeb0d92e, SUC_lexicon_tags_47 },
    { 0xce70d92f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a30d933, SUC_lexicon_tags_23 },
    { 0xb3b0d933, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb110d938, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb70d93c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10d940, SUC_lexicon_tags_937 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb550d944, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e90d949, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3250d94b, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f0d950, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8d0d952, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbf0d954, SUC_lexicon_tags_39 },
    { 0x3370d955, SUC_lexicon_tags_1 },
    { 0x8790d956, SUC_lexicon_tags_11 },
    { 0xf5f0d957, SUC_lexicon_tags_24 },
    { 0xd2b0d955, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5a10d95a, SUC_lexicon_tags_32 },
    { 0xa0f0d95b, SUC_lexicon_tags_91 },
    { 0xff70d95c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb410d95e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f70d962, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf530d964, SUC_lexicon_tags_26 },
    { 0xc1f0d965, SUC_lexicon_tags_11 },
    { 0x7310d966, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd90d96c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d10d96e, SUC_lexicon_tags_37 },
    { 0x88d0d96f, SUC_lexicon_tags_66 },
    { 0x6f0d96f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9030d975, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7990d97a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa490d97c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d10d981, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5290d985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5f0d987, SUC_lexicon_tags_82 },
    { 0x4310d988, SUC_lexicon_tags_46 },
    { 0xd3d0d988, SUC_lexicon_tags_37 },
    { 0xcd10d989, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea70d98c, SUC_lexicon_tags_25 },
    { 0xb3d0d98c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d0d999, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf250d99c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4430d99f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b0d9a7, SUC_lexicon_tags_14 },
    { 0xa8f0d9a8, SUC_lexicon_tags_0 },
    { 0xad50d9a9, SUC_lexicon_tags_49 },
    { 0x6570d9aa, SUC_lexicon_tags_47 },
    { 0x6170d9a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa690d9ae, SUC_lexicon_tags_30 },
    { 0x8610d9ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd150d9b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2570d9b3, SUC_lexicon_tags_1 },
    { 0xfe10d9b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd990d9bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb0d9c5, SUC_lexicon_tags_0 },
    { 0x5c30d9c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2610d9cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea50d9ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb050d9d5, SUC_lexicon_tags_34 },
    { 0x2f90d9d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1230d9d9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7fd0d9db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x22f0d9dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b0d9e0, SUC_lexicon_tags_65 },
    { 0x2ff0d9e1, SUC_lexicon_tags_11 },
    { 0xef70d9e2, SUC_lexicon_tags_37 },
    { 0x83b0d9e2, SUC_lexicon_tags_30 },
    { 0x58f0d9e3, SUC_lexicon_tags_12 },
    { 0x1cf0d9e5, SUC_lexicon_tags_1 },
    { 0x51b0d9e6, SUC_lexicon_tags_8 },
    { 0x9db0d9e7, SUC_lexicon_tags_7 },
    { 0xe190d9e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f0d9f1, SUC_lexicon_tags_25 },
    { 0xf530d9f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c70d9f9, SUC_lexicon_tags_3 },
    { 0xeb30d9fa, SUC_lexicon_tags_19 },
    { 0x52d0d9fa, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4590da01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2370da04, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6750da06, SUC_lexicon_tags_27 },
    { 0xa890da07, SUC_lexicon_tags_60 },
    { 0x5d70da08, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7970da0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f0da10, SUC_lexicon_tags_40 },
    { 0x4710da10, SUC_lexicon_tags_7 },
    { 0x33f0da11, SUC_lexicon_tags_37 },
    { 0x7b90da13, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc410da16, SUC_lexicon_tags_23 },
    { 0x8e70da16, SUC_lexicon_tags_25 },
    { 0xba10da18, SUC_lexicon_tags_14 },
    { 0xd90da19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f70da1d, SUC_lexicon_tags_23 },
    { 0xc10da1d, SUC_lexicon_tags_23 },
    { 0x8230da1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83f0da21, SUC_lexicon_tags_17 },
    { 0x9170da21, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb90da25, SUC_lexicon_tags_17 },
    { 0x52f0da26, SUC_lexicon_tags_63 },
    { 0xeaf0da27, SUC_lexicon_tags_33 },
    { 0xfeb0da28, SUC_lexicon_tags_0 },
    { 0x8690da28, SUC_lexicon_tags_7 },
    { 0xe190da27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d30da2c, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf70da33, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff10da3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb330da42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa490da44, SUC_lexicon_tags_38 },
    { 0x7b10da45, SUC_lexicon_tags_37 },
    { 0xe650da46, SUC_lexicon_tags_15 },
    { 0x3d50da47, SUC_lexicon_tags_38 },
    { 0x7410da45, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1bb0da4a, SUC_lexicon_tags_47 },
    { 0xe5f0da4b, SUC_lexicon_tags_15 },
    { 0xe310da4c, SUC_lexicon_tags_27 },
    { 0x2c30da4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9370da4f, SUC_lexicon_tags_12 },
    { 0xc450da50, SUC_lexicon_tags_27 },
    { 0x9710da50, SUC_lexicon_tags_0 },
    { 0xa710da4f, SUC_lexicon_tags_20 },
    { 0x3910da52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xae90da55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2cd0da57, SUC_lexicon_tags_37 },
    { 0xc030da57, SUC_lexicon_tags_170 },
    { 0xebf0da57, SUC_lexicon_tags_23 },
    { 0x5e10da57, SUC_lexicon_tags_7 },
    { 0xd010da58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf50da5f, SUC_lexicon_tags_56 },
    { 0xa850da5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6030da62, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xd010da65, SUC_lexicon_tags_19 },
    { 0x5090da66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a50da68, SUC_lexicon_tags_11 },
    { 0x3ab0da69, SUC_lexicon_tags_7 },
    { 0xd350da6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd0da6d, SUC_lexicon_tags_30 },
    { 0xef10da6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x51b0da70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb70da73, SUC_lexicon_tags_1 },
    { 0x9bd0da74, SUC_lexicon_tags_10 },
    { 0x6930da75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0da78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69f0da7a, SUC_lexicon_tags_25 },
    { 0x17f0da7b, SUC_lexicon_tags_46 },
    { 0xdb10da7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc30da7e, SUC_lexicon_tags_48 },
    { 0xde90da7e, SUC_lexicon_tags_25 },
    { 0x3dd0da7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d0da85, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb510da88, SUC_lexicon_tags_22 },
    { 0xb750da89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f0da8c, SUC_lexicon_tags_1 },
    { 0xb170da8c, SUC_lexicon_tags_32 },
    { 0xe530da8e, SUC_lexicon_tags_8 },
    { 0x98f0da8f, SUC_lexicon_tags_37 },
    { 0x21b0da8e, SUC_lexicon_tags_158 },
    { 0xc230da8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd850da95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b0da97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9250da9c, SUC_lexicon_tags_15 },
    { 0xbe90da9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc90daa0, SUC_lexicon_tags_47 },
    { 0x970daa1, SUC_lexicon_tags_15 },
    { 0x1db0daa2, SUC_lexicon_tags_37 },
    { 0x7d50daa3, SUC_lexicon_tags_37 },
    { 0xf90daa4, SUC_lexicon_tags_37 },
    { 0x5530daa5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ad0daa7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd10daaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f0daae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf30dab0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa870dab8, SUC_lexicon_tags_25 },
    { 0x4b70dab9, SUC_lexicon_tags_7 },
    { 0x1ef0dab8, SUC_lexicon_tags_43 },
    { 0x74b0dab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9970dabd, SUC_lexicon_tags_37 },
    { 0xc850dabd, SUC_lexicon_tags_4 },
    { 0x4c30dabf, SUC_lexicon_tags_30 },
    { 0x18d0dac0, SUC_lexicon_tags_37 },
    { 0xe030dac1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb30dac7, SUC_lexicon_tags_15 },
    { 0x74d0dac8, SUC_lexicon_tags_37 },
    { 0x2f70dac7, SUC_lexicon_tags_32 },
    { 0x7f0daca, SUC_lexicon_tags_27 },
    { 0xbad0dacb, SUC_lexicon_tags_64 },
    { 0x8790dac8, SUC_lexicon_tags_16 },
    { 0x6590daca, SUC_lexicon_tags_33 },
    { 0xf170dacb, SUC_lexicon_tags_19 },
    { 0x2eb0dacf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e90dad3, SUC_lexicon_tags_17 },
    { 0x1b0dad3, SUC_lexicon_tags_17 },
    { 0x5450dad4, SUC_lexicon_tags_17 },
    { 0xf070dad5, SUC_lexicon_tags_46 },
    { 0x2670dad7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6230dad9, SUC_lexicon_tags_63 },
    { 0xcf70dada, SUC_lexicon_tags_11 },
    { 0x2590dad9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af0dade, SUC_lexicon_tags_32 },
    { 0x9050dadf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe90dae1, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x7630dae3, SUC_lexicon_tags_9 },
    { 0xecf0dae4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f0dae7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50daea, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb0daed, SUC_lexicon_tags_27 },
    { 0xae90daee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa470daf2, SUC_lexicon_tags_7 },
    { 0x2150daf3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f10daf6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd30daf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2570dafb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1710db03, SUC_lexicon_tags_33 },
    { 0xb0b0db04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6610db07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf070db0b, SUC_lexicon_tags_25 },
    { 0x1dd0db0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdb0db0e, SUC_lexicon_tags_12 },
    { 0x2730db0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d0db13, SUC_lexicon_tags_17 },
    { 0x1fd0db14, SUC_lexicon_tags_11 },
    { 0xcd70db13, SUC_lexicon_tags_11 },
    { 0x6510db14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c70db1b, SUC_lexicon_tags_19 },
    { 0xec90db1c, SUC_lexicon_tags_7 },
    { 0x110db1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x730db1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd610db27, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f90db2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32b0db2e, SUC_lexicon_tags_23 },
    { 0x22d0db2e, SUC_lexicon_tags_11 },
    { 0xe1d0db2e, SUC_lexicon_tags_17 },
    { 0x9af0db31, SUC_lexicon_tags_14 },
    { 0x9b30db31, SUC_lexicon_tags_2 },
    { 0xb690db2f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb0db38, SUC_lexicon_tags_25 },
    { 0x47f0db39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d0db47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2130db4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb170db4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd30db4f, SUC_lexicon_tags_17 },
    { 0x5a10db50, SUC_lexicon_tags_38 },
    { 0x2f10db4f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa9d0db53, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb0db56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed0db5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e10db60, SUC_lexicon_tags_7 },
    { 0x4e10db61, SUC_lexicon_tags_25 },
    { 0xe70db61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3490db64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b0db6a, SUC_lexicon_tags_17 },
    { 0xe830db6a, SUC_lexicon_tags_2 },
    { 0x88f0db6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x290db6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b0db74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dd0db76, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4910db7a, SUC_lexicon_tags_10 },
    { 0x16b0db7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa9d0db7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce90db82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8010db87, SUC_lexicon_tags_16 },
    { 0xc370db88, SUC_lexicon_tags_27 },
    { 0xe730db87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f90db8c, SUC_lexicon_tags_29 },
    { 0xde30db8d, SUC_lexicon_tags_79 },
    { 0x97d0db8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a70db93, SUC_lexicon_tags_25 },
    { 0x5450db93, SUC_lexicon_tags_1 },
    { 0x7570db95, SUC_lexicon_tags_49 },
    { 0x9610db95, SUC_lexicon_tags_2 },
    { 0x6d70db95, SUC_lexicon_tags_17 },
    { 0xb6d0db97, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab0db9e, SUC_lexicon_tags_26 },
    { 0x26d0db9e, SUC_lexicon_tags_5 },
    { 0xbe50db9f, SUC_lexicon_tags_61 },
    { 0x23f0dba0, SUC_lexicon_tags_15 },
    { 0x10f0dba2, SUC_lexicon_tags_12 },
    { 0x8390dba2, SUC_lexicon_tags_31 },
    { 0xb610dba4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd0dba7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6b0dba9, SUC_lexicon_tags_68 },
    { 0xf150dbaa, SUC_lexicon_tags_38 },
    { 0x60d0dbaa, SUC_lexicon_tags_15 },
    { 0x5f10dbac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd290dbae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x590dbb1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3770dbb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed0dbb6, SUC_lexicon_tags_63 },
    { 0xe5f0dbb6, SUC_lexicon_tags_25 },
    { 0xae90dbb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfd0dbba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe70dbbd, SUC_lexicon_tags_7 },
    { 0xc550dbbd, SUC_lexicon_tags_65 },
    { 0xdb0dbbf, SUC_lexicon_tags_26 },
    { 0x8710dbbf, SUC_lexicon_tags_0 },
    { 0x7670dbc1, SUC_lexicon_tags_2 },
    { 0x3d70dbc2, SUC_lexicon_tags_2 },
    { 0xff30dbc2, SUC_lexicon_tags_54 },
    { 0x7a10dbc4, SUC_lexicon_tags_12 },
    { 0x1b50dbc5, SUC_lexicon_tags_49 },
    { 0xedb0dbc6, SUC_lexicon_tags_37 },
    { 0xa690dbc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd0dbcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc290dbcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x710dbd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb30dbdf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1f30dbe1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa830dbe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c30dbec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3810dbf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d0dbf8, SUC_lexicon_tags_1 },
    { 0xfd0dbf9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7d50dbfb, SUC_lexicon_tags_2 },
    { 0x2590dbfb, SUC_lexicon_tags_47 },
    { 0x2ad0dbfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad90dc01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75b0dc03, SUC_lexicon_tags_60 },
    { 0xd850dc03, SUC_lexicon_tags_1 },
    { 0x6d0dc05, SUC_lexicon_tags_0 },
    { 0x4650dc06, SUC_lexicon_tags_19 },
    { 0x7ab0dc03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5410dc0c, SUC_lexicon_tags_7 },
    { 0xe370dc0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3650dc0f, SUC_lexicon_tags_55 },
    { 0x7390dc10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9230dc19, SUC_lexicon_tags_17 },
    { 0x2810dc1a, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x9c30dc1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b0dc1f, SUC_lexicon_tags_15 },
    { 0xd750dc20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf670dc25, SUC_lexicon_tags_2 },
    { 0xca30dc25, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd0dc2a, SUC_lexicon_tags_17 },
    { 0x8e50dc2b, SUC_lexicon_tags_15 },
    { 0x7f70dc2c, SUC_lexicon_tags_37 },
    { 0x8f10dc2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3090dc2f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa990dc32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b0dc36, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6770dc3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50dc3d, SUC_lexicon_tags_29 },
    { 0xaa90dc3e, SUC_lexicon_tags_37 },
    { 0xb150dc3e, SUC_lexicon_tags_0 },
    { 0xee90dc3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbff0dc42, SUC_lexicon_tags_37 },
    { 0x29f0dc43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcbb0dc45, SUC_lexicon_tags_11 },
    { 0x3b10dc45, SUC_lexicon_tags_17 },
    { 0x4af0dc45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc90dc4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6330dc4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb0dc52, SUC_lexicon_tags_32 },
    { 0x13f0dc52, SUC_lexicon_tags_15 },
    { 0xbfb0dc52, SUC_lexicon_tags_14 },
    { 0xb3d0dc55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x230dc57, SUC_lexicon_tags_7 },
    { 0x5370dc57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaf0dc5a, SUC_lexicon_tags_19 },
    { 0xe8d0dc5b, SUC_lexicon_tags_36 },
    { 0x6f50dc5b, SUC_lexicon_tags_12 },
    { 0x7730dc5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe250dc61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd130dc64, SUC_lexicon_tags_37 },
    { 0x3d90dc65, SUC_lexicon_tags_60 },
    { 0xebf0dc64, SUC_lexicon_tags_26 },
    { 0xdaf0dc67, SUC_lexicon_tags_60 },
    { 0x1eb0dc68, SUC_lexicon_tags_29 },
    { 0x1590dc69, SUC_lexicon_tags_7 },
    { 0x8330dc6a, SUC_lexicon_tags_49 },
    { 0x3df0dc6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x55f0dc6d, SUC_lexicon_tags_37 },
    { 0x2370dc6e, SUC_lexicon_tags_25 },
    { 0xe290dc6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe30dc72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc050dc75, SUC_lexicon_tags_4 },
    { 0xedf0dc76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b0dc7a, SUC_lexicon_tags_25 },
    { 0x9c50dc7b, SUC_lexicon_tags_11 },
    { 0x21d0dc7a, SUC_lexicon_tags_12 },
    { 0x6790dc7d, SUC_lexicon_tags_17 },
    { 0x7130dc7b, SUC_lexicon_tags_29 },
    { 0x78d0dc7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9150dc89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb0dc8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf490dc93, SUC_lexicon_tags_104 },
    { 0xfa50dc93, SUC_lexicon_tags_2 },
    { 0x1a10dc95, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b0dc9d, SUC_lexicon_tags_0 },
    { 0x7210dc9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4070dca0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9e50dca2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x87b0dca4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b0dcab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1510dcae, SUC_lexicon_tags_37 },
    { 0x93d0dcaf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d0dcb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6910dcbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x530dcc0, SUC_lexicon_tags_26 },
    { 0xce90dcc1, SUC_lexicon_tags_31 },
    { 0x2770dcc2, SUC_lexicon_tags_37 },
    { 0xb750dcc3, SUC_lexicon_tags_0 },
    { 0x6e70dcc4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe510dcc8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf870dcca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8750dccc, SUC_lexicon_tags_12 },
    { 0xc7b0dccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d10dccf, SUC_lexicon_tags_2 },
    { 0x8a30dccf, SUC_lexicon_tags_29 },
    { 0xa4f0dcd1, SUC_lexicon_tags_7 },
    { 0x970dcd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6f0dcd4, SUC_lexicon_tags_11 },
    { 0x8b50dcd5, SUC_lexicon_tags_30 },
    { 0xf0dcd6, SUC_lexicon_tags_2 },
    { 0x6bd0dcd7, SUC_lexicon_tags_23 },
    { 0x18f0dcd5, SUC_lexicon_tags_25 },
    { 0x1cf0dcd8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef0dcdc, SUC_lexicon_tags_75 },
    { 0x6910dcdc, SUC_lexicon_tags_37 },
    { 0xa390dcde, SUC_lexicon_tags_23 },
    { 0xfe30dcde, SUC_lexicon_tags_7 },
    { 0x1730dcde, SUC_lexicon_tags_46 },
    { 0x5070dcde, SUC_lexicon_tags_49 },
    { 0xaa70dcdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4930dce4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe790dce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0dce9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d10dceb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe870dced, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6910dcf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacf0dcf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2d0dcf6, SUC_lexicon_tags_17 },
    { 0xde70dcf7, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0, NULL },
    { 0x7210dcfa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xebb0dcfc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b90dcff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a10dd07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd90dd0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9430dd12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf670dd15, SUC_lexicon_tags_32 },
    { 0x9250dd16, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d0dd19, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x8110dd1c, SUC_lexicon_tags_37 },
    { 0xd7b0dd1d, SUC_lexicon_tags_77 },
    { 0x8630dd1e, SUC_lexicon_tags_43 },
    { 0x7f10dd1f, SUC_lexicon_tags_25 },
    { 0xf5b0dd20, SUC_lexicon_tags_29 },
    { 0x5370dd1e, SUC_lexicon_tags_82 },
    { 0x1250dd20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc330dd29, SUC_lexicon_tags_1 },
    { 0xd630dd2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa310dd31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd030dd34, SUC_lexicon_tags_37 },
    { 0xcf0dd35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f0dd3b, SUC_lexicon_tags_37 },
    { 0x6450dd3b, SUC_lexicon_tags_63 },
    { 0x9430dd3d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x23b0dd3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4530dd44, SUC_lexicon_tags_12 },
    { 0xbc30dd45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf270dd47, SUC_lexicon_tags_154 },
    { 0x4d70dd48, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb90dd4f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6730dd51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd990dd53, SUC_lexicon_tags_1 },
    { 0x6990dd53, SUC_lexicon_tags_1 },
    { 0xe910dd53, SUC_lexicon_tags_28 },
    { 0x4890dd56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b10dd59, SUC_lexicon_tags_23 },
    { 0x8ed0dd5a, SUC_lexicon_tags_69 },
    { 0x71f0dd5a, SUC_lexicon_tags_37 },
    { 0x11b0dd5b, SUC_lexicon_tags_16 },
    { 0xc010dd5d, SUC_lexicon_tags_28 },
    { 0x1ad0dd5e, SUC_lexicon_tags_7 },
    { 0x3eb0dd5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f70dd61, SUC_lexicon_tags_234 },
    { 0x9490dd62, SUC_lexicon_tags_63 },
    { 0x7e50dd63, SUC_lexicon_tags_15 },
    { 0x5210dd64, SUC_lexicon_tags_55 },
    { 0x9b10dd65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4d0dd67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9830dd6e, SUC_lexicon_tags_198 },
    { 0x1d30dd6f, SUC_lexicon_tags_37 },
    { 0xe4f0dd70, SUC_lexicon_tags_17 },
    { 0x41d0dd71, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5230dd75, SUC_lexicon_tags_0 },
    { 0x3670dd76, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xad70dd79, SUC_lexicon_tags_17 },
    { 0x3890dd7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b0dd7d, SUC_lexicon_tags_21 },
    { 0xfd70dd7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xea90dd81, SUC_lexicon_tags_22 },
    { 0xa30dd82, SUC_lexicon_tags_55 },
    { 0x63d0dd82, SUC_lexicon_tags_38 },
    { 0xf950dd84, SUC_lexicon_tags_37 },
    { 0x3450dd81, SUC_lexicon_tags_14 },
    { 0xc4b0dd86, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e50dd88, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2750dd92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d0dd96, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfc70dd98, SUC_lexicon_tags_15 },
    { 0xb5b0dd99, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd10dd9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2410dd9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf790dda2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf0dda5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf7f0dda7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7250ddab, SUC_lexicon_tags_15 },
    { 0x1d0ddac, SUC_lexicon_tags_27 },
    { 0x19f0ddad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8230ddaf, SUC_lexicon_tags_1 },
    { 0x4790ddaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x53f0ddb2, SUC_lexicon_tags_9 },
    { 0xebf0ddb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf90ddc2, SUC_lexicon_tags_33 },
    { 0x5fd0ddc2, SUC_lexicon_tags_48 },
    { 0x51d0ddc3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b0ddc7, SUC_lexicon_tags_46 },
    { 0xa4f0ddc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b90ddd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f0ddd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc510dddb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb450dde4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3ad0dde6, SUC_lexicon_tags_23 },
    { 0x4030dde6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d0ddec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb250ddf1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db0ddf5, SUC_lexicon_tags_38 },
    { 0x1410ddf6, SUC_lexicon_tags_65 },
    { 0xda90ddf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d0ddfa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7a70ddfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b0ddff, SUC_lexicon_tags_15 },
    { 0x95b0de00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb0de04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1090de06, SUC_lexicon_tags_11 },
    { 0x4450de07, SUC_lexicon_tags_17 },
    { 0xd690de08, SUC_lexicon_tags_68 },
    { 0x6810de08, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a90de12, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf10de15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0de18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8eb0de1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa870de1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b0de21, SUC_lexicon_tags_68 },
    { 0x22b0de21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe10de25, SUC_lexicon_tags_1 },
    { 0x94d0de26, SUC_lexicon_tags_37 },
    { 0xc230de27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb0de2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d0de31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8330de33, SUC_lexicon_tags_17 },
    { 0x32f0de34, SUC_lexicon_tags_39 },
    { 0x6530de34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1190de3e, SUC_lexicon_tags_19 },
    { 0x4b70de3e, SUC_lexicon_tags_63 },
    { 0x3f50de3e, SUC_lexicon_tags_37 },
    { 0x63d0de41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8210de43, SUC_lexicon_tags_37 },
    { 0xa50de44, SUC_lexicon_tags_12 },
    { 0xfa30de45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7fd0de47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d0de4b, SUC_lexicon_tags_7 },
    { 0x5030de4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef10de51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a10de55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73b0de57, SUC_lexicon_tags_33 },
    { 0x2330de58, SUC_lexicon_tags_15 },
    { 0x5bb0de59, SUC_lexicon_tags_69 },
    { 0x3b90de5a, SUC_lexicon_tags_6 },
    { 0x6570de5b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5550de60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7290de62, SUC_lexicon_tags_2 },
    { 0xb250de63, SUC_lexicon_tags_6 },
    { 0xf690de62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa250de69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd390de6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1510de70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b0de76, SUC_lexicon_tags_2 },
    { 0x43f0de77, SUC_lexicon_tags_19 },
    { 0x5c90de76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4650de7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530de7e, SUC_lexicon_tags_25 },
    { 0x4070de7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1ed0de81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc310de83, SUC_lexicon_tags_33 },
    { 0x7090de83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcad0de86, SUC_lexicon_tags_37 },
    { 0xde90de87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d0de8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4910de8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1790de91, SUC_lexicon_tags_11 },
    { 0xed0de92, SUC_lexicon_tags_1 },
    { 0xbbf0de92, SUC_lexicon_tags_57 },
    { 0xdc50de92, SUC_lexicon_tags_14 },
    { 0x21d0de94, SUC_lexicon_tags_68 },
    { 0xa9f0de96, SUC_lexicon_tags_8 },
    { 0x8bf0de96, SUC_lexicon_tags_56 },
    { 0x1ed0de95, SUC_lexicon_tags_11 },
    { 0xa6f0de91, SUC_lexicon_tags_17 },
    { 0x7670de97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f0de9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd390de9e, SUC_lexicon_tags_12 },
    { 0xceb0de9f, SUC_lexicon_tags_70 },
    { 0x8df0dea0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2310dea6, SUC_lexicon_tags_64 },
    { 0xa4f0dea6, SUC_lexicon_tags_37 },
    { 0x37f0dea8, SUC_lexicon_tags_1 },
    { 0xca10dea8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e50deb0, SUC_lexicon_tags_11 },
    { 0xf7f0deb1, SUC_lexicon_tags_16 },
    { 0x5110deb2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e70deb7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d0deba, SUC_lexicon_tags_62 },
    { 0x7ed0debb, SUC_lexicon_tags_60 },
    { 0x530debc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f0dec0, SUC_lexicon_tags_2 },
    { 0x1990dec1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb90dec4, SUC_lexicon_tags_37 },
    { 0x4210dec5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa950dec7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7f0dec9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x310decd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7230decf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2f0ded1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2a10ded3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a70ded9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf0dedd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6790dee1, SUC_lexicon_tags_43 },
    { 0x7450dee2, SUC_lexicon_tags_14 },
    { 0xc10dee3, SUC_lexicon_tags_27 },
    { 0x9490dee4, SUC_lexicon_tags_38 },
    { 0x6930dee2, SUC_lexicon_tags_37 },
    { 0xfb90dee6, SUC_lexicon_tags_11 },
    { 0x6e90dee7, SUC_lexicon_tags_38 },
    { 0xd8b0dee8, SUC_lexicon_tags_30 },
    { 0x670dee5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc890def0, SUC_lexicon_tags_1 },
    { 0xe5d0def0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7df0def3, SUC_lexicon_tags_51 },
    { 0x9f0def4, SUC_lexicon_tags_66 },
    { 0xe890def3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d0def9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc330defb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf30defd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9950deff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5890df01, SUC_lexicon_tags_1 },
    { 0xe6d0df01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd230df04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d0df0e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca70df13, SUC_lexicon_tags_14 },
    { 0xe010df13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe4b0df16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8850df19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7410df1d, SUC_lexicon_tags_49 },
    { 0x9c50df1e, SUC_lexicon_tags_2 },
    { 0x92d0df1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a50df22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4490df26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d0df29, SUC_lexicon_tags_15 },
    { 0x9550df2a, SUC_lexicon_tags_2 },
    { 0x3e30df2b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe890df2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe70df31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d0df36, SUC_lexicon_tags_23 },
    { 0x45b0df37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9eb0df39, SUC_lexicon_tags_39 },
    { 0xea90df39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a30df3d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a50df43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b0df45, SUC_lexicon_tags_37 },
    { 0x2ad0df46, SUC_lexicon_tags_29 },
    { 0x3a70df46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d0df4a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf190df51, SUC_lexicon_tags_37 },
    { 0xa230df51, SUC_lexicon_tags_23 },
    { 0x5070df51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc350df56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd30df59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b0df5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xebf0df5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf250df68, SUC_lexicon_tags_48 },
    { 0x3a90df68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e70df6b, SUC_lexicon_tags_1 },
    { 0x79b0df6c, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xb5d0df6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2050df70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f0df72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2870df7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa6d0df7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9b10df7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e50df80, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd0df88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa690df8c, SUC_lexicon_tags_863 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x550df90, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2410df92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf90df94, SUC_lexicon_tags_103 },
    { 0x7370df94, SUC_lexicon_tags_15 },
    { 0xa110df96, SUC_lexicon_tags_12 },
    { 0xbf30df97, SUC_lexicon_tags_0 },
    { 0x5fd0df98, SUC_lexicon_tags_47 },
    { 0xbf0df99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe330df9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa670df9e, SUC_lexicon_tags_10 },
    { 0x1430df9e, SUC_lexicon_tags_1 },
    { 0x5e10dfa0, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b50dfa3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d0dfa9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c10dfab, SUC_lexicon_tags_37 },
    { 0xf910dfac, SUC_lexicon_tags_27 },
    { 0xf350dfac, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d0dfb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b0dfb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d0dfba, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xba50dfbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4010dfbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b0dfca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf270dfcc, SUC_lexicon_tags_31 },
    { 0xe810dfcd, SUC_lexicon_tags_1 },
    { 0x38d0dfcc, SUC_lexicon_tags_29 },
    { 0x2630dfcf, SUC_lexicon_tags_12 },
    { 0x2dd0dfcf, SUC_lexicon_tags_43 },
    { 0x7090dfd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d0dfd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd0dfdb, SUC_lexicon_tags_15 },
    { 0x2ab0dfdb, SUC_lexicon_tags_7 },
    { 0x570dfdb, SUC_lexicon_tags_55 },
    { 0xb70dfdc, SUC_lexicon_tags_7 },
    { 0xa7d0dfdf, SUC_lexicon_tags_12 },
    { 0xbc30dfdf, SUC_lexicon_tags_63 },
    { 0x3410dfe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf110dfe3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d0dfe9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1990dfef, SUC_lexicon_tags_47 },
    { 0x870dfef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8290dff2, SUC_lexicon_tags_27 },
    { 0x6690dff2, SUC_lexicon_tags_2 },
    { 0xb90dff4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d0dff8, SUC_lexicon_tags_7 },
    { 0xfa90dff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa730dffd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7890e006, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e30e00e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a50e014, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8550e018, SUC_lexicon_tags_33 },
    { 0x570e019, SUC_lexicon_tags_4 },
    { 0xbf50e01a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d0e021, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0e023, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab0e028, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb0e02c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38b0e02e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8230e036, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x88b0e038, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7250e03a, SUC_lexicon_tags_60 },
    { 0x8b50e03b, SUC_lexicon_tags_37 },
    { 0x9410e03b, SUC_lexicon_tags_182 },
    { 0x11b0e03b, SUC_lexicon_tags_37 },
    { 0x1b0e03e, SUC_lexicon_tags_11 },
    { 0x3d90e03e, SUC_lexicon_tags_21 },
    { 0x39b0e040, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fd0e042, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f0e044, SUC_lexicon_tags_1 },
    { 0x56b0e045, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f10e049, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d0e04c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2b0e04e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10e051, SUC_lexicon_tags_0 },
    { 0xb6f0e052, SUC_lexicon_tags_48 },
    { 0xdc90e051, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c50e057, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc750e05c, SUC_lexicon_tags_7 },
    { 0xfff0e05d, SUC_lexicon_tags_12 },
    { 0x9530e05e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc830e061, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf0e064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe10e066, SUC_lexicon_tags_15 },
    { 0xb6b0e066, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc570e06a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec10e06d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8e50e06f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd0e072, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca10e079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59b0e07b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d0e080, SUC_lexicon_tags_25 },
    { 0xaa70e081, SUC_lexicon_tags_12 },
    { 0x2cd0e081, SUC_lexicon_tags_1027 },
    { 0x1970e081, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5750e088, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd010e08a, SUC_lexicon_tags_2 },
    { 0x83f0e08b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b10e08e, SUC_lexicon_tags_1 },
    { 0xe270e08e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5370e092, SUC_lexicon_tags_12 },
    { 0xb4b0e093, SUC_lexicon_tags_25 },
    { 0xfcd0e093, SUC_lexicon_tags_12 },
    { 0x3870e095, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7670e09a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b0e09d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8870e0a2, SUC_lexicon_tags_37 },
    { 0x6ad0e0a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa990e0a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8930e0a9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc810e0ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b50e0ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f0e0b2, SUC_lexicon_tags_44 },
    { 0x5c70e0b3, SUC_lexicon_tags_19 },
    { 0x91d0e0b3, SUC_lexicon_tags_25 },
    { 0xf050e0b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6750e0be, SUC_lexicon_tags_11 },
    { 0x99b0e0be, SUC_lexicon_tags_37 },
    { 0x3f70e0c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9750e0c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf050e0c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4630e0c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab0e0cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdff0e0cf, SUC_lexicon_tags_37 },
    { 0x750e0d0, SUC_lexicon_tags_43 },
    { 0x3930e0d0, SUC_lexicon_tags_6 },
    { 0xcdf0e0d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x98b0e0d4, SUC_lexicon_tags_47 },
    { 0x7550e0d5, SUC_lexicon_tags_40 },
    { 0xd7f0e0d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3150e0d8, SUC_lexicon_tags_15 },
    { 0x78d0e0d8, SUC_lexicon_tags_25 },
    { 0x8170e0d9, SUC_lexicon_tags_37 },
    { 0x9f90e0db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8710e0de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b0e0e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f0e0e7, SUC_lexicon_tags_37 },
    { 0xe190e0e8, SUC_lexicon_tags_29 },
    { 0x73b0e0e9, SUC_lexicon_tags_11 },
    { 0x51b0e0ea, SUC_lexicon_tags_47 },
    { 0x41f0e0e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd830e0ed, SUC_lexicon_tags_62 },
    { 0xbd90e0ee, SUC_lexicon_tags_49 },
    { 0x9af0e0ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9af0e0f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2d0e0f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab0e0f6, SUC_lexicon_tags_7 },
    { 0xf230e0f7, SUC_lexicon_tags_87 },
    { 0xba90e0f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd0e0fb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f0e0ff, SUC_lexicon_tags_14 },
    { 0x7a30e100, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1d90e102, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c70e105, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x82b0e107, SUC_lexicon_tags_37 },
    { 0xc8f0e107, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x76b0e10a, SUC_lexicon_tags_11 },
    { 0x9050e10b, SUC_lexicon_tags_47 },
    { 0xadd0e10c, SUC_lexicon_tags_31 },
    { 0x9b10e10d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3350e110, SUC_lexicon_tags_45 },
    { 0x8db0e111, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d70e113, SUC_lexicon_tags_2 },
    { 0x6a10e113, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3810e122, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3070e128, SUC_lexicon_tags_62 },
    { 0xadf0e129, SUC_lexicon_tags_2 },
    { 0xe710e12a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc70e12d, SUC_lexicon_tags_26 },
    { 0xb30e12e, SUC_lexicon_tags_60 },
    { 0x35f0e12f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d0e133, SUC_lexicon_tags_7 },
    { 0x8910e134, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d0e139, SUC_lexicon_tags_25 },
    { 0x6e30e13a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b50e13c, SUC_lexicon_tags_7 },
    { 0x4e90e13d, SUC_lexicon_tags_49 },
    { 0xa1b0e13d, SUC_lexicon_tags_38 },
    { 0xfd90e13c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c30e147, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb0e14b, SUC_lexicon_tags_59 },
    { 0xf470e14c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d0e150, SUC_lexicon_tags_11 },
    { 0x9830e150, SUC_lexicon_tags_47 },
    { 0x9730e151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceb0e154, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d0e158, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d0e15b, SUC_lexicon_tags_7 },
    { 0xcc10e15c, SUC_lexicon_tags_64 },
    { 0x5370e15d, SUC_lexicon_tags_33 },
    { 0xb530e15d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e30e160, SUC_lexicon_tags_25 },
    { 0xa510e161, SUC_lexicon_tags_26 },
    { 0x7d10e161, SUC_lexicon_tags_46 },
    { 0xefb0e163, SUC_lexicon_tags_11 },
    { 0x9870e163, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9870e16a, SUC_lexicon_tags_17 },
    { 0xf850e16a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb850e171, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x63d0e173, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef30e179, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910e181, SUC_lexicon_tags_12 },
    { 0x490e182, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f70e18a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc510e18d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3430e191, SUC_lexicon_tags_60 },
    { 0x3910e192, SUC_lexicon_tags_25 },
    { 0xb930e193, SUC_lexicon_tags_15 },
    { 0xe710e194, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf0e19a, SUC_lexicon_tags_11 },
    { 0x5e90e19b, SUC_lexicon_tags_15 },
    { 0x90d0e19a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf0e19e, SUC_lexicon_tags_8 },
    { 0xc6f0e19f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc690e1a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xedd0e1a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0e1a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf550e1b0, SUC_lexicon_tags_12 },
    { 0x73f0e1b1, SUC_lexicon_tags_23 },
    { 0xf170e1b1, SUC_lexicon_tags_32 },
    { 0x1750e1b3, SUC_lexicon_tags_37 },
    { 0xf970e1b3, SUC_lexicon_tags_39 },
    { 0x19d0e1b3, SUC_lexicon_tags_27 },
    { 0xdab0e1b6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d0e1bb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x250e1bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8e70e1bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7130e1c1, SUC_lexicon_tags_1 },
    { 0x4390e1c2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd0e1c8, SUC_lexicon_tags_77 },
    { 0x4a90e1c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7c10e1cb, SUC_lexicon_tags_49 },
    { 0x5ab0e1cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5f0e1ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f0e1d1, SUC_lexicon_tags_92 },
    { 0x1b90e1d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc510e1d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5150e1d9, SUC_lexicon_tags_15 },
    { 0xbfb0e1da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3270e1df, SUC_lexicon_tags_1 },
    { 0xc010e1df, SUC_lexicon_tags_25 },
    { 0xae70e1e1, SUC_lexicon_tags_18 },
    { 0x39b0e1df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d0e1ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d0e1ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb570e1f2, SUC_lexicon_tags_77 },
    { 0x4c30e1f3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc30e1f7, SUC_lexicon_tags_138 },
    { 0x9330e1f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8f50e1fa, SUC_lexicon_tags_1 },
    { 0xb850e1fb, SUC_lexicon_tags_15 },
    { 0x4e30e1fc, SUC_lexicon_tags_2 },
    { 0xa2b0e1fd, SUC_lexicon_tags_11 },
    { 0x99f0e1fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb710e201, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2010e207, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7d0e209, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe10e20f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fd0e211, SUC_lexicon_tags_61 },
    { 0xb2d0e212, SUC_lexicon_tags_11 },
    { 0xdbf0e213, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f0e216, SUC_lexicon_tags_12 },
    { 0x1990e216, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc830e21b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b0e220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7910e226, SUC_lexicon_tags_14 },
    { 0x4e90e227, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc670e229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b0e22c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf910e22f, SUC_lexicon_tags_12 },
    { 0xe570e230, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f0e235, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b0e239, SUC_lexicon_tags_15 },
    { 0xcf10e23a, SUC_lexicon_tags_30 },
    { 0x3c90e23b, SUC_lexicon_tags_12 },
    { 0xd0b0e23a, SUC_lexicon_tags_17 },
    { 0x98f0e23d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaef0e23f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1250e245, SUC_lexicon_tags_0 },
    { 0xd290e246, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd0e24a, SUC_lexicon_tags_27 },
    { 0xcdd0e24a, SUC_lexicon_tags_43 },
    { 0xb1d0e24c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd410e24e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b0e253, SUC_lexicon_tags_25 },
    { 0xd10e254, SUC_lexicon_tags_12 },
    { 0xa870e255, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4870e257, SUC_lexicon_tags_28 },
    { 0x28b0e258, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc70e25c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d0e25f, SUC_lexicon_tags_14 },
    { 0x6190e260, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9850e262, SUC_lexicon_tags_1 },
    { 0x3ab0e263, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb30e269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f0e26d, SUC_lexicon_tags_37 },
    { 0xae50e26e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x730e270, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f70e273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc330e278, SUC_lexicon_tags_45 },
    { 0x59b0e279, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb10e280, SUC_lexicon_tags_38 },
    { 0xd170e281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d90e283, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcb30e285, SUC_lexicon_tags_31 },
    { 0x8cb0e286, SUC_lexicon_tags_275 },
    { 0xed50e287, SUC_lexicon_tags_48 },
    { 0xe9b0e287, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d0e28b, SUC_lexicon_tags_19 },
    { 0xc130e28c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf0e28f, SUC_lexicon_tags_12 },
    { 0x2110e290, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa270e292, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1bd0e294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff0e297, SUC_lexicon_tags_7 },
    { 0xfe10e298, SUC_lexicon_tags_82 },
    { 0x3310e298, SUC_lexicon_tags_37 },
    { 0x2d50e298, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f0e2a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf0e2a4, SUC_lexicon_tags_37 },
    { 0x16f0e2a4, SUC_lexicon_tags_9 },
    { 0xfbf0e2a5, SUC_lexicon_tags_12 },
    { 0x1c90e2a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e90e2ac, SUC_lexicon_tags_38 },
    { 0xc490e2ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x970e2af, SUC_lexicon_tags_789 },
    { 0xca70e2af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b0e2b3, SUC_lexicon_tags_40 },
    { 0xe090e2b4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd0e2b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3650e2bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x55b0e2bd, SUC_lexicon_tags_25 },
    { 0xcb10e2bd, SUC_lexicon_tags_28 },
    { 0x66f0e2be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe10e2c3, SUC_lexicon_tags_37 },
    { 0x1f70e2c4, SUC_lexicon_tags_37 },
    { 0xdc50e2c5, SUC_lexicon_tags_43 },
    { 0xff50e2c5, SUC_lexicon_tags_25 },
    { 0x3df0e2c5, SUC_lexicon_tags_7 },
    { 0xfd0e2c4, SUC_lexicon_tags_47 },
    { 0x70d0e2c9, SUC_lexicon_tags_50 },
    { 0xfc10e2c9, SUC_lexicon_tags_7 },
    { 0x8a90e2c9, SUC_lexicon_tags_47 },
    { 0x1550e2c7, SUC_lexicon_tags_7 },
    { 0xc110e2cd, SUC_lexicon_tags_16 },
    { 0xe0b0e2ce, SUC_lexicon_tags_17 },
    { 0xc970e2cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b0e2db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f0e2df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x310e2e1, SUC_lexicon_tags_11 },
    { 0xbb70e2e1, SUC_lexicon_tags_80 },
    { 0x27d0e2e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72d0e2e5, SUC_lexicon_tags_38 },
    { 0x310e2e6, SUC_lexicon_tags_37 },
    { 0x70b0e2e7, SUC_lexicon_tags_2 },
    { 0xfb90e2e8, SUC_lexicon_tags_11 },
    { 0x1630e2e9, SUC_lexicon_tags_31 },
    { 0x6070e2ea, SUC_lexicon_tags_27 },
    { 0x8fb0e2e9, SUC_lexicon_tags_2 },
    { 0xf6b0e2ec, SUC_lexicon_tags_23 },
    { 0x8cf0e2eb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2bb0e2ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf050e2f4, SUC_lexicon_tags_14 },
    { 0x67d0e2f5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f0e2f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf0e2fb, SUC_lexicon_tags_7 },
    { 0x2990e2fc, SUC_lexicon_tags_45 },
    { 0xf610e2fb, SUC_lexicon_tags_25 },
    { 0xe670e2fe, SUC_lexicon_tags_2 },
    { 0x53d0e2fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf190e306, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4070e308, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc770e30b, SUC_lexicon_tags_55 },
    { 0x9df0e30c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f50e310, SUC_lexicon_tags_11 },
    { 0xd3d0e310, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x45d0e313, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc830e316, SUC_lexicon_tags_25 },
    { 0x2210e317, SUC_lexicon_tags_67 },
    { 0xf790e318, SUC_lexicon_tags_42 },
    { 0xaa10e319, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd0e31e, SUC_lexicon_tags_65 },
    { 0xa870e31f, SUC_lexicon_tags_32 },
    { 0xa6d0e31f, SUC_lexicon_tags_19 },
    { 0xb1f0e321, SUC_lexicon_tags_2 },
    { 0xdad0e322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba50e324, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0e32a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e10e32d, SUC_lexicon_tags_14 },
    { 0x8490e32e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc250e331, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb70e336, SUC_lexicon_tags_12 },
    { 0x9930e337, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb0e33f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2810e341, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1130e347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f0e34a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd070e34c, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb0e352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4950e359, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef0e35d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6950e361, SUC_lexicon_tags_47 },
    { 0xc670e362, SUC_lexicon_tags_11 },
    { 0x9f50e363, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6010e36a, SUC_lexicon_tags_7 },
    { 0x9f50e36a, SUC_lexicon_tags_30 },
    { 0xa630e36a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd830e370, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d0e373, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5cf0e375, SUC_lexicon_tags_25 },
    { 0x2ef0e375, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xecd0e378, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb530e37a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9b0e37c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5d0e37e, SUC_lexicon_tags_32 },
    { 0xe2d0e37f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c70e382, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d0e390, SUC_lexicon_tags_0 },
    { 0xb870e391, SUC_lexicon_tags_49 },
    { 0x5430e390, SUC_lexicon_tags_25 },
    { 0x4d30e392, SUC_lexicon_tags_2 },
    { 0x89b0e394, SUC_lexicon_tags_37 },
    { 0xaeb0e394, SUC_lexicon_tags_10 },
    { 0xe9f0e395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb0e39a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cd0e39c, SUC_lexicon_tags_17 },
    { 0x1dd0e39c, SUC_lexicon_tags_37 },
    { 0x430e39c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5790e3a2, SUC_lexicon_tags_47 },
    { 0x4bf0e3a3, SUC_lexicon_tags_31 },
    { 0x83f0e3a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd230e3a6, SUC_lexicon_tags_15 },
    { 0x4130e3a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b0e3ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d0e3b0, SUC_lexicon_tags_8 },
    { 0x9ed0e3b0, SUC_lexicon_tags_15 },
    { 0x1f90e3b1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b0e3b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4270e3ba, SUC_lexicon_tags_45 },
    { 0xe50e3ba, SUC_lexicon_tags_14 },
    { 0x5850e3bc, SUC_lexicon_tags_8 },
    { 0x92b0e3bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2030e3c0, SUC_lexicon_tags_37 },
    { 0xda50e3c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b0e3c5, SUC_lexicon_tags_49 },
    { 0x3cd0e3c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f50e3cb, SUC_lexicon_tags_7 },
    { 0xedf0e3cb, SUC_lexicon_tags_12 },
    { 0x7a10e3cc, SUC_lexicon_tags_2 },
    { 0x1590e3ce, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x8fb0e3d0, SUC_lexicon_tags_12 },
    { 0xc070e3d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf690e3d3, SUC_lexicon_tags_123 },
    { 0x7390e3d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f0e3da, SUC_lexicon_tags_54 },
    { 0xfcd0e3db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8830e3df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x510e3e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43f0e3e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5170e3e6, SUC_lexicon_tags_33 },
    { 0xd630e3e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x390e3eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0d0e3ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad70e3f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ab0e3f2, SUC_lexicon_tags_15 },
    { 0x5a50e3f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc70e3f9, SUC_lexicon_tags_29 },
    { 0x6930e3fa, SUC_lexicon_tags_11 },
    { 0x11b0e3fb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3d50e3fd, SUC_lexicon_tags_15 },
    { 0x9710e3fe, SUC_lexicon_tags_9 },
    { 0x7330e3ff, SUC_lexicon_tags_2 },
    { 0x9c50e400, SUC_lexicon_tags_8 },
    { 0xc770e401, SUC_lexicon_tags_26 },
    { 0xe110e402, SUC_lexicon_tags_21 },
    { 0x4790e403, SUC_lexicon_tags_15 },
    { 0x92b0e3ff, SUC_lexicon_tags_6 },
    { 0xfdd0e403, SUC_lexicon_tags_164 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b0e40c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd0e40f, SUC_lexicon_tags_36 },
    { 0x950e40f, SUC_lexicon_tags_25 },
    { 0x19b0e411, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4930e414, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d0e417, SUC_lexicon_tags_10 },
    { 0x2390e418, SUC_lexicon_tags_14 },
    { 0xc7f0e419, SUC_lexicon_tags_17 },
    { 0x2dd0e41a, SUC_lexicon_tags_7 },
    { 0x2bf0e41b, SUC_lexicon_tags_26 },
    { 0x74f0e41b, SUC_lexicon_tags_12 },
    { 0xd010e41d, SUC_lexicon_tags_11 },
    { 0x4650e418, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2690e420, SUC_lexicon_tags_38 },
    { 0x6c10e421, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa890e427, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce70e42b, SUC_lexicon_tags_37 },
    { 0x3930e42c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d0e43c, SUC_lexicon_tags_9 },
    { 0x6710e43d, SUC_lexicon_tags_26 },
    { 0x9f10e43d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf910e440, SUC_lexicon_tags_17 },
    { 0xd7d0e441, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f0e445, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7810e447, SUC_lexicon_tags_14 },
    { 0x8350e448, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf290e44a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c90e44d, SUC_lexicon_tags_17 },
    { 0xcc90e44d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b0e452, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b0e458, SUC_lexicon_tags_37 },
    { 0x4170e459, SUC_lexicon_tags_23 },
    { 0xb4f0e459, SUC_lexicon_tags_103 },
    { 0x2650e45b, SUC_lexicon_tags_18 },
    { 0x8510e45b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2390e467, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5370e46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a30e470, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff0e473, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa70e477, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1cd0e479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa50e47d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x470e47f, SUC_lexicon_tags_47 },
    { 0x12d0e480, SUC_lexicon_tags_63 },
    { 0x4830e481, SUC_lexicon_tags_39 },
    { 0x1c30e482, SUC_lexicon_tags_2 },
    { 0x6970e481, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x2730e485, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4570e48a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b50e48d, SUC_lexicon_tags_11 },
    { 0x3650e48d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb50e491, SUC_lexicon_tags_49 },
    { 0xc050e492, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a30e494, SUC_lexicon_tags_34 },
    { 0x6f30e495, SUC_lexicon_tags_12 },
    { 0x5b70e496, SUC_lexicon_tags_0 },
    { 0x61f0e496, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc190e49b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe630e4a0, SUC_lexicon_tags_15 },
    { 0xf7d0e4a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd750e4a3, SUC_lexicon_tags_17 },
    { 0xd110e4a4, SUC_lexicon_tags_48 },
    { 0x1a30e4a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb250e4ac, SUC_lexicon_tags_29 },
    { 0x8610e4ad, SUC_lexicon_tags_1 },
    { 0x9970e4ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f0e4b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d0e4b8, SUC_lexicon_tags_37 },
    { 0x4090e4b8, SUC_lexicon_tags_11 },
    { 0xcf70e4b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb130e4bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf0e4c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2d0e4c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44d0e4ca, SUC_lexicon_tags_33 },
    { 0x44b0e4cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf690e4cf, SUC_lexicon_tags_26 },
    { 0xda50e4d0, SUC_lexicon_tags_1063 },
    { 0x7710e4cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1270e4d5, SUC_lexicon_tags_37 },
    { 0x5490e4d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1090e4da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef0e4e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa010e4e3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd850e4e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d10e4e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7d10e4eb, SUC_lexicon_tags_37 },
    { 0xc250e4eb, SUC_lexicon_tags_17 },
    { 0xd990e4eb, SUC_lexicon_tags_37 },
    { 0x9990e4ee, SUC_lexicon_tags_29 },
    { 0xd5f0e4ec, SUC_lexicon_tags_37 },
    { 0x6090e4f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3730e4f4, SUC_lexicon_tags_37 },
    { 0x6df0e4f5, SUC_lexicon_tags_101 },
    { 0xf5d0e4f6, SUC_lexicon_tags_37 },
    { 0xd990e4f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc930e4fc, SUC_lexicon_tags_1 },
    { 0x3790e4fd, SUC_lexicon_tags_17 },
    { 0x6170e4fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d0e501, SUC_lexicon_tags_11 },
    { 0xb270e501, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6dd0e504, SUC_lexicon_tags_17 },
    { 0x9d0e505, SUC_lexicon_tags_15 },
    { 0xd2b0e504, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4f0e508, SUC_lexicon_tags_3 },
    { 0xfc90e509, SUC_lexicon_tags_69 },
    { 0x5a50e50a, SUC_lexicon_tags_906 },
    { 0, NULL },
    { 0, NULL },
    { 0x8710e50d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb530e50f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9890e511, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x810e514, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5f0e516, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd90e51a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d0e51f, SUC_lexicon_tags_10 },
    { 0x9dd0e520, SUC_lexicon_tags_14 },
    { 0x6710e51f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f0e526, SUC_lexicon_tags_25 },
    { 0x3270e526, SUC_lexicon_tags_8 },
    { 0x2070e527, SUC_lexicon_tags_3 },
    { 0x10b0e526, SUC_lexicon_tags_28 },
    { 0xd7b0e52a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d30e52e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad0e533, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b0e539, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b50e53d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca10e541, SUC_lexicon_tags_11 },
    { 0x8430e541, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8570e546, SUC_lexicon_tags_12 },
    { 0x5670e547, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd450e551, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc210e55b, SUC_lexicon_tags_56 },
    { 0x470e55b, SUC_lexicon_tags_8 },
    { 0xe390e55d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19d0e55f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3550e561, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f0e564, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b50e567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc70e56b, SUC_lexicon_tags_182 },
    { 0x6db0e56b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x370e56e, SUC_lexicon_tags_21 },
    { 0xa950e56f, SUC_lexicon_tags_2 },
    { 0xa30e56e, SUC_lexicon_tags_17 },
    { 0x92d0e571, SUC_lexicon_tags_2 },
    { 0xddd0e572, SUC_lexicon_tags_65 },
    { 0xb9b0e56f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5350e576, SUC_lexicon_tags_15 },
    { 0x3dd0e576, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3370e57f, SUC_lexicon_tags_1 },
    { 0x1e10e57f, SUC_lexicon_tags_7 },
    { 0x20f0e581, SUC_lexicon_tags_40 },
    { 0x4c30e582, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f0e588, SUC_lexicon_tags_25 },
    { 0x57d0e589, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d0e58c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9590e590, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf210e593, SUC_lexicon_tags_29 },
    { 0x8e50e594, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf50e599, SUC_lexicon_tags_12 },
    { 0x6970e599, SUC_lexicon_tags_47 },
    { 0x81f0e59b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3510e5a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xacb0e5a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1190e5a8, SUC_lexicon_tags_31 },
    { 0xd90e5a8, SUC_lexicon_tags_12 },
    { 0x8890e5aa, SUC_lexicon_tags_11 },
    { 0xb650e5aa, SUC_lexicon_tags_2 },
    { 0xd7b0e5a8, SUC_lexicon_tags_9 },
    { 0x7050e5ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49d0e5af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1990e5b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a70e5b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x150e5b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd0e5bc, SUC_lexicon_tags_37 },
    { 0xd9b0e5bc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf0e5c2, SUC_lexicon_tags_33 },
    { 0x5d50e5c3, SUC_lexicon_tags_2 },
    { 0xa9d0e5c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x550e5c7, SUC_lexicon_tags_10 },
    { 0x6750e5c8, SUC_lexicon_tags_17 },
    { 0x8630e5c8, SUC_lexicon_tags_4 },
    { 0xd810e5ca, SUC_lexicon_tags_15 },
    { 0xeb30e5c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b0e5cd, SUC_lexicon_tags_37 },
    { 0x7210e5cd, SUC_lexicon_tags_25 },
    { 0x8830e5ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6050e5d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b0e5d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd030e5d9, SUC_lexicon_tags_37 },
    { 0xe8f0e5da, SUC_lexicon_tags_16 },
    { 0x81b0e5db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70e5de, SUC_lexicon_tags_47 },
    { 0xd490e5de, SUC_lexicon_tags_43 },
    { 0x78f0e5e0, SUC_lexicon_tags_37 },
    { 0xa750e5e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd570e5e4, SUC_lexicon_tags_7 },
    { 0x5b50e5e5, SUC_lexicon_tags_19 },
    { 0x6f0e5e4, SUC_lexicon_tags_12 },
    { 0x31b0e5e7, SUC_lexicon_tags_113 },
    { 0x18b0e5e8, SUC_lexicon_tags_17 },
    { 0x5b70e5e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d0e5ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd0e5f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9170e5fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f0e5fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9790e5ff, SUC_lexicon_tags_32 },
    { 0xafb0e600, SUC_lexicon_tags_25 },
    { 0xfd50e5ff, SUC_lexicon_tags_27 },
    { 0x7210e601, SUC_lexicon_tags_4 },
    { 0xa9f0e603, SUC_lexicon_tags_37 },
    { 0xaa10e604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ab0e606, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe50e608, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e30e60a, SUC_lexicon_tags_63 },
    { 0xf4d0e60b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db0e60f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa270e612, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe7d0e614, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b0e617, SUC_lexicon_tags_17 },
    { 0xe630e618, SUC_lexicon_tags_15 },
    { 0xb850e618, SUC_lexicon_tags_9 },
    { 0xd530e618, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x270e61c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2d30e61e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d0e621, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f50e625, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x98b0e627, SUC_lexicon_tags_23 },
    { 0xb090e628, SUC_lexicon_tags_25 },
    { 0x1b90e629, SUC_lexicon_tags_15 },
    { 0xf150e62a, SUC_lexicon_tags_56 },
    { 0x2250e62b, SUC_lexicon_tags_16 },
    { 0xebb0e62c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfaf0e62e, SUC_lexicon_tags_7 },
    { 0x2f10e62f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfdf0e631, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d10e636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa150e639, SUC_lexicon_tags_26 },
    { 0xf1d0e63a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd250e63f, SUC_lexicon_tags_4 },
    { 0xa430e640, SUC_lexicon_tags_11 },
    { 0xbd30e640, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf0e646, SUC_lexicon_tags_0 },
    { 0x9c70e647, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b0e64a, SUC_lexicon_tags_2 },
    { 0xd370e64a, SUC_lexicon_tags_26 },
    { 0x9050e64b, SUC_lexicon_tags_6 },
    { 0x2170e64d, SUC_lexicon_tags_2 },
    { 0xf230e64e, SUC_lexicon_tags_11 },
    { 0x6150e64f, SUC_lexicon_tags_44 },
    { 0xf3f0e64f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1350e652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3070e655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0b0e657, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2270e65a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x250e65c, SUC_lexicon_tags_27 },
    { 0x1d50e65d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3710e660, SUC_lexicon_tags_37 },
    { 0xc490e661, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb190e667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe530e66f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7d0e671, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f0e679, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2750e67d, SUC_lexicon_tags_33 },
    { 0x4750e67e, SUC_lexicon_tags_29 },
    { 0xbd0e67f, SUC_lexicon_tags_37 },
    { 0x5e10e680, SUC_lexicon_tags_7 },
    { 0x8030e680, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f10e686, SUC_lexicon_tags_32 },
    { 0x12b0e687, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd0e68b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4270e692, SUC_lexicon_tags_7 },
    { 0x63b0e693, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f0e696, SUC_lexicon_tags_2 },
    { 0xc630e697, SUC_lexicon_tags_37 },
    { 0x5590e697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f90e69a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff50e69f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf0e6a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb0e6ab, SUC_lexicon_tags_7 },
    { 0xa50e6ab, SUC_lexicon_tags_17 },
    { 0x7eb0e6ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8f0e6af, SUC_lexicon_tags_37 },
    { 0xfeb0e6b0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9630e6b6, SUC_lexicon_tags_47 },
    { 0xa590e6b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc130e6be, SUC_lexicon_tags_25 },
    { 0x20d0e6bf, SUC_lexicon_tags_12 },
    { 0x5810e6be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d0e6c4, SUC_lexicon_tags_23 },
    { 0xf4b0e6c5, SUC_lexicon_tags_19 },
    { 0xda70e6c4, SUC_lexicon_tags_17 },
    { 0xbd30e6c7, SUC_lexicon_tags_25 },
    { 0xfdb0e6c4, SUC_lexicon_tags_25 },
    { 0x6350e6c9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910e6ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6270e6d0, SUC_lexicon_tags_12 },
    { 0x5750e6d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6550e6d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17f0e6d5, SUC_lexicon_tags_23 },
    { 0xf9f0e6d5, SUC_lexicon_tags_25 },
    { 0x1f30e6d6, SUC_lexicon_tags_25 },
    { 0x5910e6d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c70e6da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7890e6df, SUC_lexicon_tags_310 },
    { 0x56f0e6df, SUC_lexicon_tags_17 },
    { 0x5d50e6df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2f0e6e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2390e6e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3330e6eb, SUC_lexicon_tags_63 },
    { 0x2470e6eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3d0e6ee, SUC_lexicon_tags_7 },
    { 0x83b0e6ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb870e6f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed10e6f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9d10e6f5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5930e6fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f0e701, SUC_lexicon_tags_27 },
    { 0xb2f0e701, SUC_lexicon_tags_37 },
    { 0x44b0e703, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd50e705, SUC_lexicon_tags_60 },
    { 0xf490e705, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd030e70a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10e70d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x5d0e70f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x150e711, SUC_lexicon_tags_17 },
    { 0x3d70e712, SUC_lexicon_tags_2 },
    { 0xa650e712, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d70e716, SUC_lexicon_tags_68 },
    { 0x5250e717, SUC_lexicon_tags_25 },
    { 0x95b0e718, SUC_lexicon_tags_38 },
    { 0x4490e719, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b0e71c, SUC_lexicon_tags_11 },
    { 0x7d0e71d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8070e722, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f0e726, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x16d0e728, SUC_lexicon_tags_37 },
    { 0x21d0e729, SUC_lexicon_tags_14 },
    { 0x43d0e72a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9db0e72c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2890e72f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b0e736, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5950e739, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7630e73c, SUC_lexicon_tags_38 },
    { 0x9730e73d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0e742, SUC_lexicon_tags_6 },
    { 0x8a50e743, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb450e746, SUC_lexicon_tags_14 },
    { 0xa570e747, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x6070e749, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdfd0e74b, SUC_lexicon_tags_40 },
    { 0xd90e74c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc970e74f, SUC_lexicon_tags_108 },
    { 0x8cf0e750, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d0e754, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8630e75b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df0e75e, SUC_lexicon_tags_48 },
    { 0x64b0e75f, SUC_lexicon_tags_0 },
    { 0x69d0e760, SUC_lexicon_tags_7 },
    { 0xa530e761, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc230e764, SUC_lexicon_tags_16 },
    { 0x89b0e765, SUC_lexicon_tags_45 },
    { 0x7830e766, SUC_lexicon_tags_60 },
    { 0xb3b0e765, SUC_lexicon_tags_1 },
    { 0xd790e767, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb50e76a, SUC_lexicon_tags_57 },
    { 0x22d0e76a, SUC_lexicon_tags_25 },
    { 0xc270e76c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e70e76f, SUC_lexicon_tags_37 },
    { 0x9c10e770, SUC_lexicon_tags_15 },
    { 0xa270e770, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf50e77d, SUC_lexicon_tags_18 },
    { 0x11b0e77d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd650e780, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b0e78a, SUC_lexicon_tags_14 },
    { 0xf9f0e78b, SUC_lexicon_tags_2 },
    { 0x4950e78a, SUC_lexicon_tags_46 },
    { 0x98b0e78b, SUC_lexicon_tags_17 },
    { 0x6070e78a, SUC_lexicon_tags_31 },
    { 0x36f0e78d, SUC_lexicon_tags_37 },
    { 0x3670e790, SUC_lexicon_tags_37 },
    { 0x2f90e790, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f0e79c, SUC_lexicon_tags_33 },
    { 0x74b0e79d, SUC_lexicon_tags_13 },
    { 0x5290e79d, SUC_lexicon_tags_12 },
    { 0xe190e79f, SUC_lexicon_tags_25 },
    { 0x1bb0e7a0, SUC_lexicon_tags_17 },
    { 0xd090e79f, SUC_lexicon_tags_2 },
    { 0x4ad0e7a2, SUC_lexicon_tags_23 },
    { 0x6990e7a3, SUC_lexicon_tags_23 },
    { 0x6b70e79e, SUC_lexicon_tags_17 },
    { 0xe3f0e79c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce90e7ad, SUC_lexicon_tags_47 },
    { 0x9950e7ad, SUC_lexicon_tags_7 },
    { 0x5210e7ad, SUC_lexicon_tags_25 },
    { 0x50f0e7b0, SUC_lexicon_tags_87 },
    { 0xdfd0e7b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b0e7b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8270e7b8, SUC_lexicon_tags_25 },
    { 0x2210e7b8, SUC_lexicon_tags_37 },
    { 0x1510e7b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b0e7bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7b0e7bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc710e7c1, SUC_lexicon_tags_17 },
    { 0x5410e7c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf370e7c6, SUC_lexicon_tags_33 },
    { 0xc6d0e7c7, SUC_lexicon_tags_70 },
    { 0xc110e7c8, SUC_lexicon_tags_12 },
    { 0xbd10e7c8, SUC_lexicon_tags_2 },
    { 0xa90e7c9, SUC_lexicon_tags_33 },
    { 0x73f0e7cb, SUC_lexicon_tags_46 },
    { 0x4c50e7ca, SUC_lexicon_tags_23 },
    { 0x9250e7cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4490e7d0, SUC_lexicon_tags_33 },
    { 0x80d0e7d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd0e7d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6750e7d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x290e7dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd0e7e0, SUC_lexicon_tags_30 },
    { 0x2e10e7e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e10e7ea, SUC_lexicon_tags_11 },
    { 0x6e70e7eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9470e7ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4590e7ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c90e7f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f0e7f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe770e801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef0e804, SUC_lexicon_tags_17 },
    { 0x6050e804, SUC_lexicon_tags_82 },
    { 0x27b0e806, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9430e809, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d0e80f, SUC_lexicon_tags_23 },
    { 0x1570e810, SUC_lexicon_tags_80 },
    { 0xbdd0e810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb30e815, SUC_lexicon_tags_21 },
    { 0xa870e816, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd0e81a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d0e821, SUC_lexicon_tags_60 },
    { 0x88f0e821, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f70e824, SUC_lexicon_tags_108 },
    { 0xcf50e824, SUC_lexicon_tags_152 },
    { 0xdc90e825, SUC_lexicon_tags_25 },
    { 0x5e50e827, SUC_lexicon_tags_0 },
    { 0x1750e828, SUC_lexicon_tags_12 },
    { 0x6450e829, SUC_lexicon_tags_37 },
    { 0x3d90e82a, SUC_lexicon_tags_14 },
    { 0xd6b0e82a, SUC_lexicon_tags_37 },
    { 0x4df0e82c, SUC_lexicon_tags_31 },
    { 0xc630e82a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f50e831, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f30e834, SUC_lexicon_tags_44 },
    { 0x6dd0e834, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3f0e837, SUC_lexicon_tags_17 },
    { 0xcb50e838, SUC_lexicon_tags_49 },
    { 0x1b10e839, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa910e83b, SUC_lexicon_tags_826 },
    { 0xde90e83c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2110e83e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc30e842, SUC_lexicon_tags_12 },
    { 0x75b0e843, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9810e84c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d70e84e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9510e853, SUC_lexicon_tags_11 },
    { 0x4d50e854, SUC_lexicon_tags_23 },
    { 0x33d0e855, SUC_lexicon_tags_15 },
    { 0xa7d0e856, SUC_lexicon_tags_29 },
    { 0x7e90e855, SUC_lexicon_tags_17 },
    { 0x8fb0e854, SUC_lexicon_tags_60 },
    { 0x3890e859, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb7d0e85b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a30e860, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3550e864, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x22b0e866, SUC_lexicon_tags_27 },
    { 0xddb0e867, SUC_lexicon_tags_31 },
    { 0x4110e867, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf90e86e, SUC_lexicon_tags_35 },
    { 0xfa50e86f, SUC_lexicon_tags_11 },
    { 0xd30e870, SUC_lexicon_tags_43 },
    { 0x47f0e86f, SUC_lexicon_tags_14 },
    { 0x7490e870, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6530e876, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7470e879, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdcf0e87b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba10e880, SUC_lexicon_tags_7 },
    { 0x35f0e880, SUC_lexicon_tags_37 },
    { 0xd350e880, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f0e886, SUC_lexicon_tags_2 },
    { 0x2150e887, SUC_lexicon_tags_10 },
    { 0xffd0e888, SUC_lexicon_tags_25 },
    { 0x5a90e889, SUC_lexicon_tags_23 },
    { 0xe910e889, SUC_lexicon_tags_1 },
    { 0x1a50e88b, SUC_lexicon_tags_17 },
    { 0x3c30e886, SUC_lexicon_tags_27 },
    { 0x5390e889, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89f0e88f, SUC_lexicon_tags_0 },
    { 0xd690e890, SUC_lexicon_tags_31 },
    { 0x3910e88f, SUC_lexicon_tags_37 },
    { 0x3810e892, SUC_lexicon_tags_11 },
    { 0x47f0e893, SUC_lexicon_tags_5 },
    { 0x12d0e892, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d0e89a, SUC_lexicon_tags_0 },
    { 0x9ef0e89b, SUC_lexicon_tags_7 },
    { 0x5130e89c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b0e8a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b0e8a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e70e8ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaef0e8ad, SUC_lexicon_tags_19 },
    { 0x66f0e8ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd690e8b6, SUC_lexicon_tags_63 },
    { 0xf150e8b7, SUC_lexicon_tags_2 },
    { 0xf4f0e8b8, SUC_lexicon_tags_45 },
    { 0xc5f0e8b8, SUC_lexicon_tags_65 },
    { 0x5b10e8b8, SUC_lexicon_tags_87 },
    { 0xdcb0e8bb, SUC_lexicon_tags_27 },
    { 0xbd90e8ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x76f0e8be, SUC_lexicon_tags_63 },
    { 0xc650e8bf, SUC_lexicon_tags_4 },
    { 0x8470e8bf, SUC_lexicon_tags_0 },
    { 0xcd50e8bf, SUC_lexicon_tags_14 },
    { 0x3050e8c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbf70e8c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9d0e8c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa70e8c9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d30e8ce, SUC_lexicon_tags_201 },
    { 0x7750e8ce, SUC_lexicon_tags_48 },
    { 0x6cf0e8ce, SUC_lexicon_tags_1 },
    { 0x4250e8ce, SUC_lexicon_tags_23 },
    { 0x58b0e8d2, SUC_lexicon_tags_2 },
    { 0x7c30e8d2, SUC_lexicon_tags_37 },
    { 0x5a30e8d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc90e8d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x890e8db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa170e8df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd0e8e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb830e8e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1290e8ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7630e8ec, SUC_lexicon_tags_14 },
    { 0xb5b0e8ed, SUC_lexicon_tags_12 },
    { 0x4970e8ed, SUC_lexicon_tags_84 },
    { 0xb9d0e8ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x850e8f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4550e8f7, SUC_lexicon_tags_63 },
    { 0x7790e8f7, SUC_lexicon_tags_21 },
    { 0x6810e8f9, SUC_lexicon_tags_37 },
    { 0x1870e8f8, SUC_lexicon_tags_21 },
    { 0xd9d0e8fb, SUC_lexicon_tags_11 },
    { 0x94f0e8f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdfb0e8fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9170e900, SUC_lexicon_tags_38 },
    { 0xaef0e901, SUC_lexicon_tags_117 },
    { 0x87f0e902, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1190e904, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeef0e906, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6370e908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa030e90a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf590e910, SUC_lexicon_tags_33 },
    { 0x73b0e911, SUC_lexicon_tags_17 },
    { 0x47b0e912, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b70e915, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcdd0e917, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf70e91a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xffb0e91c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca30e920, SUC_lexicon_tags_234 },
    { 0xaa70e921, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0e926, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe70e928, SUC_lexicon_tags_15 },
    { 0x7db0e929, SUC_lexicon_tags_14 },
    { 0x1df0e92a, SUC_lexicon_tags_11 },
    { 0x4970e929, SUC_lexicon_tags_44 },
    { 0x55d0e92c, SUC_lexicon_tags_12 },
    { 0x1c30e92d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e70e938, SUC_lexicon_tags_25 },
    { 0x13f0e938, SUC_lexicon_tags_15 },
    { 0x1370e938, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a30e93f, SUC_lexicon_tags_15 },
    { 0xc5b0e93f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb50e942, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc10e945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb0e948, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa90e94a, SUC_lexicon_tags_6 },
    { 0x6fb0e94b, SUC_lexicon_tags_47 },
    { 0x1410e94c, SUC_lexicon_tags_35 },
    { 0x62b0e94d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d0e951, SUC_lexicon_tags_0 },
    { 0x6630e952, SUC_lexicon_tags_34 },
    { 0x8930e952, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b0e95a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd770e95d, SUC_lexicon_tags_40 },
    { 0xd770e95d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9010e960, SUC_lexicon_tags_25 },
    { 0xe410e961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6850e963, SUC_lexicon_tags_7 },
    { 0x21f0e964, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc10e969, SUC_lexicon_tags_17 },
    { 0x1730e96a, SUC_lexicon_tags_25 },
    { 0xddd0e96b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f10e970, SUC_lexicon_tags_23 },
    { 0xad10e971, SUC_lexicon_tags_43 },
    { 0xbfd0e972, SUC_lexicon_tags_25 },
    { 0x4b50e972, SUC_lexicon_tags_55 },
    { 0xfff0e974, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x51f0e976, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53b0e978, SUC_lexicon_tags_65 },
    { 0x29b0e979, SUC_lexicon_tags_23 },
    { 0x8510e97a, SUC_lexicon_tags_17 },
    { 0x19f0e97b, SUC_lexicon_tags_37 },
    { 0x1d0e97b, SUC_lexicon_tags_37 },
    { 0xd630e97a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d0e980, SUC_lexicon_tags_7 },
    { 0x1af0e980, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd210e983, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d0e987, SUC_lexicon_tags_37 },
    { 0x76b0e988, SUC_lexicon_tags_45 },
    { 0xb3b0e988, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1910e98b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf190e98e, SUC_lexicon_tags_61 },
    { 0x8cf0e98f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf370e993, SUC_lexicon_tags_35 },
    { 0x5730e993, SUC_lexicon_tags_7 },
    { 0xbb90e995, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e50e99f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb10e9ab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2d10e9ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd0e9b1, SUC_lexicon_tags_19 },
    { 0xc6b0e9b2, SUC_lexicon_tags_65 },
    { 0x330e9b3, SUC_lexicon_tags_84 },
    { 0x1710e9b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc710e9b6, SUC_lexicon_tags_17 },
    { 0xae70e9b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a90e9b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2850e9bb, SUC_lexicon_tags_63 },
    { 0x7750e9bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe610e9be, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe10e9c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c70e9c5, SUC_lexicon_tags_33 },
    { 0xa250e9c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ad0e9c8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b30e9ce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f0e9d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b0e9d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa530e9d9, SUC_lexicon_tags_2 },
    { 0xefb0e9d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f0e9e0, SUC_lexicon_tags_97 },
    { 0xb8d0e9e1, SUC_lexicon_tags_0 },
    { 0x4670e9e0, SUC_lexicon_tags_37 },
    { 0xcb10e9e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9110e9e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e90e9ea, SUC_lexicon_tags_16 },
    { 0x8bf0e9eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0e9f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6930e9f5, SUC_lexicon_tags_49 },
    { 0xcbd0e9f6, SUC_lexicon_tags_38 },
    { 0xccb0e9f7, SUC_lexicon_tags_1 },
    { 0x46d0e9f8, SUC_lexicon_tags_34 },
    { 0x8450e9f9, SUC_lexicon_tags_58 },
    { 0xe0f0e9f6, SUC_lexicon_tags_12 },
    { 0xbb50e9f7, SUC_lexicon_tags_16 },
    { 0xb10e9f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99d0e9fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8070ea00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2410ea06, SUC_lexicon_tags_55 },
    { 0x1450ea07, SUC_lexicon_tags_12 },
    { 0xb90ea08, SUC_lexicon_tags_33 },
    { 0xc350ea08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1330ea0c, SUC_lexicon_tags_48 },
    { 0x8eb0ea0d, SUC_lexicon_tags_2 },
    { 0x5350ea0c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f0ea13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db0ea18, SUC_lexicon_tags_7 },
    { 0x3a30ea19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20b0ea1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf50ea1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9230ea24, SUC_lexicon_tags_81 },
    { 0xa170ea25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf410ea27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d0ea29, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf110ea2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb50ea32, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5970ea34, SUC_lexicon_tags_107 },
    { 0x77b0ea35, SUC_lexicon_tags_37 },
    { 0xef0ea35, SUC_lexicon_tags_103 },
    { 0x3730ea35, SUC_lexicon_tags_11 },
    { 0xbd90ea36, SUC_lexicon_tags_19 },
    { 0x3790ea34, SUC_lexicon_tags_12 },
    { 0x80b0ea3a, SUC_lexicon_tags_33 },
    { 0xf9f0ea34, SUC_lexicon_tags_9 },
    { 0xbf70ea39, SUC_lexicon_tags_34 },
    { 0xe650ea38, SUC_lexicon_tags_15 },
    { 0xa5b0ea36, SUC_lexicon_tags_8 },
    { 0x79b0ea3f, SUC_lexicon_tags_28 },
    { 0xb30ea3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd0ea43, SUC_lexicon_tags_28 },
    { 0x9bb0ea44, SUC_lexicon_tags_43 },
    { 0xa770ea43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d0ea53, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x3130ea55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf70ea57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e90ea59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b0ea5e, SUC_lexicon_tags_43 },
    { 0xd30ea5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe530ea61, SUC_lexicon_tags_2 },
    { 0x4e50ea62, SUC_lexicon_tags_15 },
    { 0x49b0ea62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd0ea66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb130ea6d, SUC_lexicon_tags_37 },
    { 0x9010ea6d, SUC_lexicon_tags_37 },
    { 0x2770ea6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb90ea71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f30ea78, SUC_lexicon_tags_1 },
    { 0xde30ea78, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa370ea7b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6470ea7e, SUC_lexicon_tags_55 },
    { 0x9590ea7f, SUC_lexicon_tags_19 },
    { 0x4390ea80, SUC_lexicon_tags_22 },
    { 0x9f30ea81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb0ea84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f0ea87, SUC_lexicon_tags_59 },
    { 0xae70ea87, SUC_lexicon_tags_4 },
    { 0x5030ea88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5770ea8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4d0ea8e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d0ea99, SUC_lexicon_tags_25 },
    { 0x7af0ea99, SUC_lexicon_tags_12 },
    { 0xb690ea9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdef0ea9d, SUC_lexicon_tags_23 },
    { 0x2c90ea9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf30eaa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530eaac, SUC_lexicon_tags_25 },
    { 0xc770eaad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6950eaaf, SUC_lexicon_tags_1 },
    { 0xad70eab0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8230eab2, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6670eab6, SUC_lexicon_tags_37 },
    { 0x8250eab7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x98d0eab9, SUC_lexicon_tags_12 },
    { 0x7d30eaba, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3530eabc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb50eac0, SUC_lexicon_tags_2 },
    { 0xdc10eac1, SUC_lexicon_tags_14 },
    { 0x77d0eac0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3550eac8, SUC_lexicon_tags_11 },
    { 0x10f0eac8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf590eacd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe530ead1, SUC_lexicon_tags_118 },
    { 0xd2f0ead2, SUC_lexicon_tags_47 },
    { 0xe1d0ead3, SUC_lexicon_tags_1 },
    { 0x7550ead4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed0ead7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb10eadc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd550eae3, SUC_lexicon_tags_30 },
    { 0x3d10eae3, SUC_lexicon_tags_14 },
    { 0xd230eae5, SUC_lexicon_tags_25 },
    { 0x3790eae6, SUC_lexicon_tags_421 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d0eaee, SUC_lexicon_tags_11 },
    { 0x2ef0eaef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4d0eaf1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd790eaf3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9310eaf5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xcdb0eaf7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae10eafb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd730eb00, SUC_lexicon_tags_47 },
    { 0xf690eb01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd0eb04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd70eb09, SUC_lexicon_tags_63 },
    { 0xc7b0eb0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30eb10, SUC_lexicon_tags_2 },
    { 0xf290eb11, SUC_lexicon_tags_17 },
    { 0xc50eb12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeef0eb14, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf910eb17, SUC_lexicon_tags_77 },
    { 0x3990eb18, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa2d0eb1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3390eb1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e90eb23, SUC_lexicon_tags_37 },
    { 0xa750eb24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b0eb29, SUC_lexicon_tags_11 },
    { 0x4030eb2a, SUC_lexicon_tags_23 },
    { 0x6010eb2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7330eb2d, SUC_lexicon_tags_11 },
    { 0x8f0eb2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x590eb32, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd990eb38, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8a50eb3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd130eb3d, SUC_lexicon_tags_31 },
    { 0x63d0eb3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5030eb41, SUC_lexicon_tags_44 },
    { 0x5270eb42, SUC_lexicon_tags_11 },
    { 0xd2f0eb43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x29f0eb45, SUC_lexicon_tags_47 },
    { 0x7530eb46, SUC_lexicon_tags_29 },
    { 0xab0eb47, SUC_lexicon_tags_37 },
    { 0xc370eb45, SUC_lexicon_tags_21 },
    { 0x2af0eb48, SUC_lexicon_tags_45 },
    { 0x72b0eb45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16b0eb4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3010eb4e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5db0eb50, SUC_lexicon_tags_15 },
    { 0xbb30eb51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8770eb57, SUC_lexicon_tags_9 },
    { 0xafd0eb58, SUC_lexicon_tags_3 },
    { 0x5330eb58, SUC_lexicon_tags_4 },
    { 0x1ab0eb58, SUC_lexicon_tags_32 },
    { 0x2eb0eb5b, SUC_lexicon_tags_30 },
    { 0x69d0eb57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f0eb5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc390eb64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9f0eb66, SUC_lexicon_tags_30 },
    { 0x8af0eb66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa90eb69, SUC_lexicon_tags_25 },
    { 0xc7b0eb6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb90eb6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7590eb6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c10eb71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d50eb76, SUC_lexicon_tags_25 },
    { 0xe130eb76, SUC_lexicon_tags_12 },
    { 0xcf90eb78, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb0eb7b, SUC_lexicon_tags_6 },
    { 0x7a70eb7b, SUC_lexicon_tags_25 },
    { 0x8890eb7c, SUC_lexicon_tags_29 },
    { 0x29f0eb7e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x2830eb80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa950eb83, SUC_lexicon_tags_25 },
    { 0x2f70eb83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd10eb89, SUC_lexicon_tags_26 },
    { 0xc570eb89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd0eb91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b90eb94, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb50eb99, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7270eb9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5090eb9f, SUC_lexicon_tags_11 },
    { 0x5410eba0, SUC_lexicon_tags_28 },
    { 0x8e70eba0, SUC_lexicon_tags_14 },
    { 0x7d10eba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0ebad, SUC_lexicon_tags_37 },
    { 0x6c50ebae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6f10ebb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2710ebb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b0ebb9, SUC_lexicon_tags_27 },
    { 0xacf0ebba, SUC_lexicon_tags_14 },
    { 0xde10ebbb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3150ebc0, SUC_lexicon_tags_17 },
    { 0xca90ebc1, SUC_lexicon_tags_45 },
    { 0xe890ebc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdef0ebc4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa590ebc6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd830ebcb, SUC_lexicon_tags_49 },
    { 0x53d0ebcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcf0ebce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa570ebd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b0ebdc, SUC_lexicon_tags_16 },
    { 0xd1d0ebdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf830ebe1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69d0ebe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd030ebe6, SUC_lexicon_tags_17 },
    { 0xe090ebe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef0ebeb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f0ebf0, SUC_lexicon_tags_254 },
    { 0xadd0ebf1, SUC_lexicon_tags_15 },
    { 0x3bf0ebf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f0ebf6, SUC_lexicon_tags_17 },
    { 0x3910ebf7, SUC_lexicon_tags_14 },
    { 0x3990ebf8, SUC_lexicon_tags_8 },
    { 0x77f0ebf7, SUC_lexicon_tags_19 },
    { 0x2970ebfa, SUC_lexicon_tags_25 },
    { 0x3810ebf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1510ebfe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd690ec05, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee10ec0a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e90ec0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa550ec10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c30ec15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe9d0ec17, SUC_lexicon_tags_17 },
    { 0x1fd0ec17, SUC_lexicon_tags_27 },
    { 0xcd50ec17, SUC_lexicon_tags_9 },
    { 0x1c50ec1a, SUC_lexicon_tags_7 },
    { 0xe4b0ec1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd130ec1e, SUC_lexicon_tags_25 },
    { 0x7850ec1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd0ec26, SUC_lexicon_tags_15 },
    { 0x28b0ec26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2730ec29, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b70ec2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7770ec31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1110ec35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3390ec39, SUC_lexicon_tags_37 },
    { 0xecb0ec3a, SUC_lexicon_tags_49 },
    { 0x1e50ec39, SUC_lexicon_tags_66 },
    { 0xb070ec3c, SUC_lexicon_tags_15 },
    { 0x1d0ec3c, SUC_lexicon_tags_33 },
    { 0x3a10ec3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf70ec41, SUC_lexicon_tags_47 },
    { 0x1110ec42, SUC_lexicon_tags_25 },
    { 0xc990ec43, SUC_lexicon_tags_37 },
    { 0x3ff0ec43, SUC_lexicon_tags_2 },
    { 0xc2b0ec43, SUC_lexicon_tags_17 },
    { 0x4450ec42, SUC_lexicon_tags_28 },
    { 0x7db0ec47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f30ec49, SUC_lexicon_tags_27 },
    { 0xf770ec4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7250ec50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb0ec52, SUC_lexicon_tags_5 },
    { 0x49b0ec52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd50ec55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e10ec57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc870ec5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3630ec63, SUC_lexicon_tags_47 },
    { 0xbe30ec64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf70ec66, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6650ec68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf170ec6b, SUC_lexicon_tags_30 },
    { 0x4530ec6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9610ec6f, SUC_lexicon_tags_12 },
    { 0xa5d0ec70, SUC_lexicon_tags_84 },
    { 0xdd50ec70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3410ec75, SUC_lexicon_tags_29 },
    { 0x2bf0ec76, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x490ec78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a10ec7a, SUC_lexicon_tags_12 },
    { 0x9c50ec7b, SUC_lexicon_tags_4 },
    { 0x22d0ec7c, SUC_lexicon_tags_0 },
    { 0x16d0ec7a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910ec83, SUC_lexicon_tags_0 },
    { 0x4f70ec83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd070ec86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e30ec8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed0ec95, SUC_lexicon_tags_61 },
    { 0xdab0ec95, SUC_lexicon_tags_48 },
    { 0xb30ec97, SUC_lexicon_tags_47 },
    { 0x5430ec95, SUC_lexicon_tags_14 },
    { 0xb4f0ec98, SUC_lexicon_tags_65 },
    { 0x6e30ec9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5910ec9f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d0eca8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf0ecad, SUC_lexicon_tags_7 },
    { 0x5d0ecae, SUC_lexicon_tags_14 },
    { 0xbc70ecaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa250ecb1, SUC_lexicon_tags_7 },
    { 0xa470ecb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a50ecb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd0ecb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb190ecbc, SUC_lexicon_tags_15 },
    { 0x14b0ecbd, SUC_lexicon_tags_11 },
    { 0x2d30ecbe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e70ecc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6d0ecc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46d0ecc5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xab30ecc7, SUC_lexicon_tags_1 },
    { 0x9b90ecc7, SUC_lexicon_tags_24 },
    { 0xb730ecc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b30eccc, SUC_lexicon_tags_17 },
    { 0xb9b0eccd, SUC_lexicon_tags_15 },
    { 0xc490ecce, SUC_lexicon_tags_15 },
    { 0x1c50eccf, SUC_lexicon_tags_15 },
    { 0x470eccc, SUC_lexicon_tags_56 },
    { 0x3b10ecd0, SUC_lexicon_tags_26 },
    { 0x9970ecd2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f50ecd5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df0ecdd, SUC_lexicon_tags_15 },
    { 0xffb0ecdd, SUC_lexicon_tags_1 },
    { 0x4170ecdf, SUC_lexicon_tags_25 },
    { 0x1490ecdf, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd0ece8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2a90ecea, SUC_lexicon_tags_17 },
    { 0xcf90eceb, SUC_lexicon_tags_11 },
    { 0xe290eceb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe210ecee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af0ecf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b70ecf4, SUC_lexicon_tags_2 },
    { 0xa570ecf4, SUC_lexicon_tags_0 },
    { 0x8730ecf4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1550ecf8, SUC_lexicon_tags_1 },
    { 0xd750ecf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb510ecfc, SUC_lexicon_tags_49 },
    { 0x4f10ecfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5030ed0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27d0ed0c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e50ed10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a70ed15, SUC_lexicon_tags_26 },
    { 0x69f0ed15, SUC_lexicon_tags_32 },
    { 0xb90ed17, SUC_lexicon_tags_37 },
    { 0xa610ed18, SUC_lexicon_tags_23 },
    { 0x8bf0ed15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x46f0ed1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd0ed1f, SUC_lexicon_tags_595 },
    { 0xa890ed20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac10ed22, SUC_lexicon_tags_19 },
    { 0x6eb0ed23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xad50ed25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d0ed28, SUC_lexicon_tags_0 },
    { 0xd4d0ed28, SUC_lexicon_tags_63 },
    { 0x5db0ed2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb70ed2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa110ed3c, SUC_lexicon_tags_27 },
    { 0x5e90ed3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xae30ed40, SUC_lexicon_tags_23 },
    { 0xefd0ed40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3090ed45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d0ed48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0ed4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf970ed51, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x8110ed53, SUC_lexicon_tags_32 },
    { 0x2390ed54, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7e30ed56, SUC_lexicon_tags_38 },
    { 0x5d30ed57, SUC_lexicon_tags_12 },
    { 0x6d10ed58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bb0ed5a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xee30ed5d, SUC_lexicon_tags_3 },
    { 0x7e90ed5e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0ed61, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd0ed64, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f0ed6b, SUC_lexicon_tags_38 },
    { 0x6a50ed6c, SUC_lexicon_tags_10 },
    { 0xf990ed6d, SUC_lexicon_tags_8 },
    { 0x4930ed6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9510ed72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x250ed74, SUC_lexicon_tags_17 },
    { 0x6b50ed75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3250ed7a, SUC_lexicon_tags_9 },
    { 0x9130ed7b, SUC_lexicon_tags_33 },
    { 0xa110ed7c, SUC_lexicon_tags_29 },
    { 0x78b0ed7c, SUC_lexicon_tags_15 },
    { 0x4c10ed7e, SUC_lexicon_tags_25 },
    { 0xd7d0ed7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b0ed83, SUC_lexicon_tags_18 },
    { 0x12d0ed84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89f0ed86, SUC_lexicon_tags_46 },
    { 0xfff0ed86, SUC_lexicon_tags_19 },
    { 0xfeb0ed88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b0ed8d, SUC_lexicon_tags_15 },
    { 0xf7d0ed8d, SUC_lexicon_tags_37 },
    { 0x5130ed8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa990ed95, SUC_lexicon_tags_33 },
    { 0xc610ed95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd010ed99, SUC_lexicon_tags_262 },
    { 0x870ed9a, SUC_lexicon_tags_25 },
    { 0xb450ed9a, SUC_lexicon_tags_47 },
    { 0x4930ed9c, SUC_lexicon_tags_27 },
    { 0x2af0ed9c, SUC_lexicon_tags_8 },
    { 0x96b0ed9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b0eda3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2670eda8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ab0edaa, SUC_lexicon_tags_66 },
    { 0x82d0edab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa510edb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e30edb8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6070edbe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf70edc1, SUC_lexicon_tags_15 },
    { 0x54f0edc2, SUC_lexicon_tags_14 },
    { 0x17d0edc3, SUC_lexicon_tags_46 },
    { 0xeed0edc3, SUC_lexicon_tags_7 },
    { 0xe1b0edc5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x870edc7, SUC_lexicon_tags_31 },
    { 0x30edc7, SUC_lexicon_tags_0 },
    { 0xd870edc9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef0edce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4570edd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe10ede0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe510ede2, SUC_lexicon_tags_28 },
    { 0xec70ede2, SUC_lexicon_tags_17 },
    { 0xf670ede4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe50ede7, SUC_lexicon_tags_19 },
    { 0x9a50ede8, SUC_lexicon_tags_47 },
    { 0xd4b0ede9, SUC_lexicon_tags_2 },
    { 0x78b0ede9, SUC_lexicon_tags_1 },
    { 0xd470edeb, SUC_lexicon_tags_31 },
    { 0xec70edec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3530edf1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x5630edf4, SUC_lexicon_tags_25 },
    { 0x2370edf5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa390edf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf0edfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd90ee02, SUC_lexicon_tags_15 },
    { 0x9130ee03, SUC_lexicon_tags_11 },
    { 0x77f0ee03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8870ee06, SUC_lexicon_tags_34 },
    { 0xae90ee07, SUC_lexicon_tags_28 },
    { 0xfc50ee07, SUC_lexicon_tags_1 },
    { 0x9290ee09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa070ee0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfdd0ee0d, SUC_lexicon_tags_15 },
    { 0xe650ee0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9990ee10, SUC_lexicon_tags_7 },
    { 0xbcb0ee10, SUC_lexicon_tags_13 },
    { 0xef50ee12, SUC_lexicon_tags_47 },
    { 0xa8f0ee13, SUC_lexicon_tags_31 },
    { 0x4b90ee12, SUC_lexicon_tags_14 },
    { 0x2b70ee12, SUC_lexicon_tags_65 },
    { 0x55b0ee16, SUC_lexicon_tags_0 },
    { 0x1530ee17, SUC_lexicon_tags_30 },
    { 0xdbb0ee17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf0ee20, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6130ee25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb0ee28, SUC_lexicon_tags_45 },
    { 0xae70ee29, SUC_lexicon_tags_25 },
    { 0x2bd0ee2a, SUC_lexicon_tags_37 },
    { 0x1eb0ee29, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf350ee37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d50ee39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe990ee3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f10ee3d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5270ee43, SUC_lexicon_tags_30 },
    { 0xfd10ee43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f0ee4a, SUC_lexicon_tags_11 },
    { 0x85b0ee4a, SUC_lexicon_tags_0 },
    { 0x2210ee4c, SUC_lexicon_tags_30 },
    { 0x2fb0ee4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba30ee50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d70ee53, SUC_lexicon_tags_17 },
    { 0xa450ee54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf0ee57, SUC_lexicon_tags_44 },
    { 0xc10ee58, SUC_lexicon_tags_0 },
    { 0x3f0ee58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5610ee5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea70ee5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19b0ee61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa170ee67, SUC_lexicon_tags_33 },
    { 0x8e10ee68, SUC_lexicon_tags_55 },
    { 0xfe90ee69, SUC_lexicon_tags_1102 },
    { 0x82f0ee6a, SUC_lexicon_tags_33 },
    { 0xb270ee67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4270ee6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f0ee72, SUC_lexicon_tags_8 },
    { 0x34f0ee73, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5630ee75, SUC_lexicon_tags_2 },
    { 0x5970ee75, SUC_lexicon_tags_37 },
    { 0x3f0ee76, SUC_lexicon_tags_37 },
    { 0x9310ee75, SUC_lexicon_tags_679 },
    { 0x8690ee78, SUC_lexicon_tags_23 },
    { 0x2450ee76, SUC_lexicon_tags_170 },
    { 0x4bf0ee76, SUC_lexicon_tags_17 },
    { 0xed10ee76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1b0ee7e, SUC_lexicon_tags_11 },
    { 0x24d0ee7f, SUC_lexicon_tags_17 },
    { 0x60d0ee80, SUC_lexicon_tags_26 },
    { 0x7310ee7f, SUC_lexicon_tags_12 },
    { 0xc0b0ee82, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe3d0ee84, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd150ee8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4650ee8e, SUC_lexicon_tags_12 },
    { 0x9eb0ee8f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd210ee93, SUC_lexicon_tags_7 },
    { 0x6b50ee94, SUC_lexicon_tags_129 },
    { 0xdab0ee95, SUC_lexicon_tags_45 },
    { 0x1d10ee95, SUC_lexicon_tags_38 },
    { 0xe810ee97, SUC_lexicon_tags_182 },
    { 0xb570ee98, SUC_lexicon_tags_23 },
    { 0x9470ee98, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f0ee9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b90ee9f, SUC_lexicon_tags_29 },
    { 0x3c70ee9f, SUC_lexicon_tags_25 },
    { 0x2cd0eea0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7250eea3, SUC_lexicon_tags_9 },
    { 0xe950eea4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6330eea6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab0eeaa, SUC_lexicon_tags_33 },
    { 0x5b0eeaa, SUC_lexicon_tags_17 },
    { 0x8830eeac, SUC_lexicon_tags_11 },
    { 0x7cf0eead, SUC_lexicon_tags_37 },
    { 0xfaf0eeae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee90eeb0, SUC_lexicon_tags_8 },
    { 0xe110eeb1, SUC_lexicon_tags_15 },
    { 0x3810eeb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d0eeb6, SUC_lexicon_tags_12 },
    { 0xd4d0eeb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b0eebb, SUC_lexicon_tags_49 },
    { 0x69f0eebc, SUC_lexicon_tags_0 },
    { 0x6190eebd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1950eebf, SUC_lexicon_tags_6 },
    { 0xcd90eebf, SUC_lexicon_tags_19 },
    { 0x6d0eec0, SUC_lexicon_tags_7 },
    { 0x2230eec2, SUC_lexicon_tags_47 },
    { 0x7510eec3, SUC_lexicon_tags_3 },
    { 0x8490eec4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1090eec7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb0eecb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d90eed1, SUC_lexicon_tags_7 },
    { 0xf6f0eed1, SUC_lexicon_tags_15 },
    { 0xafd0eed2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf290eeda, SUC_lexicon_tags_23 },
    { 0x3bf0eedb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8430eedf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d0eee3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd990eee5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b0eee7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6470eee9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb830eef0, SUC_lexicon_tags_12 },
    { 0x9430eef1, SUC_lexicon_tags_17 },
    { 0x4dd0eef1, SUC_lexicon_tags_25 },
    { 0x510eef3, SUC_lexicon_tags_16 },
    { 0x2310eef3, SUC_lexicon_tags_37 },
    { 0x5630eef5, SUC_lexicon_tags_2 },
    { 0x7730eef5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b0eefe, SUC_lexicon_tags_31 },
    { 0x8130eefe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8670ef04, SUC_lexicon_tags_15 },
    { 0xb010ef05, SUC_lexicon_tags_37 },
    { 0x6f90ef06, SUC_lexicon_tags_28 },
    { 0xd6b0ef07, SUC_lexicon_tags_25 },
    { 0x970ef08, SUC_lexicon_tags_38 },
    { 0xfb10ef09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d0ef0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe910ef11, SUC_lexicon_tags_47 },
    { 0x9670ef12, SUC_lexicon_tags_37 },
    { 0x8c10ef12, SUC_lexicon_tags_11 },
    { 0x4f0ef14, SUC_lexicon_tags_55 },
    { 0x8d90ef15, SUC_lexicon_tags_49 },
    { 0xd970ef16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba50ef18, SUC_lexicon_tags_38 },
    { 0xab0ef18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab50ef1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8770ef26, SUC_lexicon_tags_63 },
    { 0xdff0ef26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f0ef2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5d0ef2c, SUC_lexicon_tags_2 },
    { 0xc710ef2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f0ef30, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef50ef3b, SUC_lexicon_tags_62 },
    { 0xfad0ef3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d0ef42, SUC_lexicon_tags_15 },
    { 0x9b50ef42, SUC_lexicon_tags_15 },
    { 0x89f0ef44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cf0ef46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7630ef48, SUC_lexicon_tags_27 },
    { 0xd2d0ef48, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7250ef4e, SUC_lexicon_tags_85 },
    { 0x3350ef4f, SUC_lexicon_tags_2 },
    { 0xbfb0ef50, SUC_lexicon_tags_77 },
    { 0x2d50ef51, SUC_lexicon_tags_62 },
    { 0xb430ef4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d70ef54, SUC_lexicon_tags_29 },
    { 0x8d0ef54, SUC_lexicon_tags_7 },
    { 0x7c70ef56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68f0ef58, SUC_lexicon_tags_25 },
    { 0x7430ef58, SUC_lexicon_tags_37 },
    { 0x8f30ef5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d0ef5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39d0ef5f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9650ef62, SUC_lexicon_tags_0 },
    { 0x1c30ef63, SUC_lexicon_tags_6 },
    { 0x1f0ef62, SUC_lexicon_tags_45 },
    { 0xf010ef65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1070ef6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd270ef6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xecd0ef6e, SUC_lexicon_tags_30 },
    { 0x4f90ef6e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd50ef72, SUC_lexicon_tags_27 },
    { 0x14b0ef73, SUC_lexicon_tags_97 },
    { 0xb230ef74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf0ef77, SUC_lexicon_tags_151 },
    { 0xb050ef77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb0d0ef7a, SUC_lexicon_tags_33 },
    { 0x4850ef7b, SUC_lexicon_tags_30 },
    { 0x8e90ef7c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xb590ef7f, SUC_lexicon_tags_61 },
    { 0x6f0ef7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xad50ef82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5790ef86, SUC_lexicon_tags_38 },
    { 0x3db0ef86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb0ef8b, SUC_lexicon_tags_63 },
    { 0xf550ef8c, SUC_lexicon_tags_62 },
    { 0x4770ef8d, SUC_lexicon_tags_15 },
    { 0x4070ef8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f0ef95, SUC_lexicon_tags_15 },
    { 0x1710ef96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad0ef9a, SUC_lexicon_tags_25 },
    { 0xcaf0ef9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf0efa3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2530efa5, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x7dd0efa7, SUC_lexicon_tags_7 },
    { 0x23f0efa7, SUC_lexicon_tags_37 },
    { 0x1af0efa9, SUC_lexicon_tags_19 },
    { 0x6a10efaa, SUC_lexicon_tags_17 },
    { 0x1ff0efaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f0efba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2310efbc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf0efc3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe070efc6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4450efca, SUC_lexicon_tags_19 },
    { 0x3a70efcb, SUC_lexicon_tags_17 },
    { 0xa50efcc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcfd0efce, SUC_lexicon_tags_2 },
    { 0xa650efcf, SUC_lexicon_tags_31 },
    { 0xa8d0efd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc70efd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe370efd9, SUC_lexicon_tags_11 },
    { 0xf910efda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeeb0efdc, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f0efe3, SUC_lexicon_tags_39 },
    { 0x5c10efe4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2270efe6, SUC_lexicon_tags_21 },
    { 0x7d70efe7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaa90efe9, SUC_lexicon_tags_40 },
    { 0xf070efe9, SUC_lexicon_tags_7 },
    { 0xe3f0efeb, SUC_lexicon_tags_60 },
    { 0xb3d0efec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9110eff2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a90eff5, SUC_lexicon_tags_15 },
    { 0x8f70eff6, SUC_lexicon_tags_37 },
    { 0xeb10eff6, SUC_lexicon_tags_70 },
    { 0xd670eff8, SUC_lexicon_tags_25 },
    { 0x8d10eff9, SUC_lexicon_tags_11 },
    { 0x9b70effa, SUC_lexicon_tags_43 },
    { 0x390effb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd0f00a, SUC_lexicon_tags_0 },
    { 0x2590f00a, SUC_lexicon_tags_33 },
    { 0xfd90f00c, SUC_lexicon_tags_47 },
    { 0x1ef0f00d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac30f017, SUC_lexicon_tags_415 },
    { 0x6f70f018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5290f01b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd850f026, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xec10f029, SUC_lexicon_tags_60 },
    { 0x31d0f02a, SUC_lexicon_tags_60 },
    { 0xd5d0f02a, SUC_lexicon_tags_1 },
    { 0xcb30f02c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8250f02e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd810f034, SUC_lexicon_tags_48 },
    { 0x2790f035, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4250f03c, SUC_lexicon_tags_37 },
    { 0x6b90f03d, SUC_lexicon_tags_6 },
    { 0x4db0f03e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x2170f040, SUC_lexicon_tags_23 },
    { 0xd0b0f041, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2450f043, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1670f047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b0f054, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced0f059, SUC_lexicon_tags_25 },
    { 0xdc90f05a, SUC_lexicon_tags_542 },
    { 0x7470f05b, SUC_lexicon_tags_9 },
    { 0xc30f05b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc870f060, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab0f067, SUC_lexicon_tags_11 },
    { 0x6d50f068, SUC_lexicon_tags_1 },
    { 0x8950f068, SUC_lexicon_tags_25 },
    { 0xd210f06a, SUC_lexicon_tags_66 },
    { 0x15d0f069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb650f06d, SUC_lexicon_tags_26 },
    { 0xddd0f06d, SUC_lexicon_tags_46 },
    { 0x4290f06f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f0f072, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f70f074, SUC_lexicon_tags_37 },
    { 0xafb0f075, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4eb0f077, SUC_lexicon_tags_56 },
    { 0x3e30f078, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb0f07e, SUC_lexicon_tags_66 },
    { 0xdad0f07e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba30f087, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf250f08b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f50f08e, SUC_lexicon_tags_63 },
    { 0xaad0f08e, SUC_lexicon_tags_47 },
    { 0x4310f08e, SUC_lexicon_tags_37 },
    { 0xe890f091, SUC_lexicon_tags_47 },
    { 0xf0d0f091, SUC_lexicon_tags_4 },
    { 0x4c70f093, SUC_lexicon_tags_7 },
    { 0x37f0f08f, SUC_lexicon_tags_5 },
    { 0x610f095, SUC_lexicon_tags_29 },
    { 0xe510f096, SUC_lexicon_tags_17 },
    { 0xf610f097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4210f09b, SUC_lexicon_tags_39 },
    { 0x99b0f09b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f70f0a3, SUC_lexicon_tags_12 },
    { 0x3a70f0a4, SUC_lexicon_tags_17 },
    { 0x1d70f0a4, SUC_lexicon_tags_37 },
    { 0x4850f0a6, SUC_lexicon_tags_34 },
    { 0x7230f0a7, SUC_lexicon_tags_27 },
    { 0x6c70f0a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca30f0ab, SUC_lexicon_tags_7 },
    { 0xd710f0ac, SUC_lexicon_tags_37 },
    { 0x67b0f0ab, SUC_lexicon_tags_70 },
    { 0xa350f0ae, SUC_lexicon_tags_47 },
    { 0xbbf0f0af, SUC_lexicon_tags_0 },
    { 0xc8f0f0b0, SUC_lexicon_tags_8 },
    { 0xbf0f0b0, SUC_lexicon_tags_37 },
    { 0x7730f0b2, SUC_lexicon_tags_25 },
    { 0x9630f0af, SUC_lexicon_tags_37 },
    { 0x5550f0ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8810f0ba, SUC_lexicon_tags_1 },
    { 0xe730f0bb, SUC_lexicon_tags_17 },
    { 0x1df0f0bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe510f0be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf2d0f0c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5f50f0c2, SUC_lexicon_tags_63 },
    { 0x52b0f0c2, SUC_lexicon_tags_49 },
    { 0xe310f0c3, SUC_lexicon_tags_2 },
    { 0xa5d0f0c3, SUC_lexicon_tags_49 },
    { 0xcc50f0c6, SUC_lexicon_tags_25 },
    { 0x4a70f0c7, SUC_lexicon_tags_31 },
    { 0xfd10f0c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c10f0cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ff0f0cd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe90f0cf, SUC_lexicon_tags_45 },
    { 0x6750f0d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d0f0d8, SUC_lexicon_tags_65 },
    { 0xfbf0f0d9, SUC_lexicon_tags_17 },
    { 0x4dd0f0da, SUC_lexicon_tags_4 },
    { 0x7530f0da, SUC_lexicon_tags_2 },
    { 0xdcd0f0dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa30f0df, SUC_lexicon_tags_1 },
    { 0x7790f0e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e30f0e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf370f0e9, SUC_lexicon_tags_29 },
    { 0x8790f0ea, SUC_lexicon_tags_63 },
    { 0xbd70f0e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6850f0ed, SUC_lexicon_tags_37 },
    { 0x3890f0ee, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0xf450f0f0, SUC_lexicon_tags_47 },
    { 0x89f0f0f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe170f0f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8c50f0f5, SUC_lexicon_tags_27 },
    { 0xb830f0f6, SUC_lexicon_tags_1 },
    { 0xb470f0f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50f0fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f0f0ff, SUC_lexicon_tags_33 },
    { 0x66f0f100, SUC_lexicon_tags_23 },
    { 0x13b0f0ff, SUC_lexicon_tags_60 },
    { 0x58f0f102, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b50f104, SUC_lexicon_tags_10 },
    { 0xa590f104, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8b30f107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d0f10b, SUC_lexicon_tags_7 },
    { 0xd490f10b, SUC_lexicon_tags_15 },
    { 0x1190f10d, SUC_lexicon_tags_4 },
    { 0x1210f10d, SUC_lexicon_tags_7 },
    { 0x1c70f10d, SUC_lexicon_tags_43 },
    { 0xf7d0f10f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f0f118, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d0f11c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbb0f11e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e30f128, SUC_lexicon_tags_7 },
    { 0xa8d0f129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f30f12b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaed0f12d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc770f131, SUC_lexicon_tags_10 },
    { 0x2cf0f131, SUC_lexicon_tags_47 },
    { 0x41f0f133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d0f137, SUC_lexicon_tags_37 },
    { 0xa730f138, SUC_lexicon_tags_45 },
    { 0x2c10f137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3350f13c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51f0f13e, SUC_lexicon_tags_17 },
    { 0x4b10f13e, SUC_lexicon_tags_14 },
    { 0x3b70f13f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8230f142, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b0f146, SUC_lexicon_tags_50 },
    { 0x4e50f147, SUC_lexicon_tags_26 },
    { 0x3c70f148, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe230f14a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc550f14d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa10f150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe830f152, SUC_lexicon_tags_47 },
    { 0xb5d0f152, SUC_lexicon_tags_7 },
    { 0xe070f154, SUC_lexicon_tags_103 },
    { 0x3c90f154, SUC_lexicon_tags_2 },
    { 0x8490f156, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd270f15b, SUC_lexicon_tags_11 },
    { 0xe9b0f15c, SUC_lexicon_tags_7 },
    { 0xe570f15d, SUC_lexicon_tags_47 },
    { 0xecf0f15d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6270f161, SUC_lexicon_tags_7 },
    { 0xcdd0f161, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bd0f164, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e50f167, SUC_lexicon_tags_35 },
    { 0x1cf0f168, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9c70f16a, SUC_lexicon_tags_17 },
    { 0xf870f16b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5570f171, SUC_lexicon_tags_0 },
    { 0x44b0f172, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf30f176, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a70f17b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb2f0f17d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f0f188, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf030f18b, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xe410f18d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3430f18f, SUC_lexicon_tags_65 },
    { 0xf790f190, SUC_lexicon_tags_27 },
    { 0x4d90f191, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xac90f193, SUC_lexicon_tags_15 },
    { 0xca70f193, SUC_lexicon_tags_7 },
    { 0xf90f195, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f0f19b, SUC_lexicon_tags_14 },
    { 0x2a90f19c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd70f19e, SUC_lexicon_tags_43 },
    { 0x1430f19f, SUC_lexicon_tags_16 },
    { 0x1190f19f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6410f1a4, SUC_lexicon_tags_28 },
    { 0x7e10f1a4, SUC_lexicon_tags_33 },
    { 0x2990f1a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2850f1a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xed50f1ab, SUC_lexicon_tags_31 },
    { 0xf070f1ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8090f1af, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9230f1b1, SUC_lexicon_tags_25 },
    { 0x8b0f1b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf0f1b5, SUC_lexicon_tags_30 },
    { 0x3e90f1b6, SUC_lexicon_tags_38 },
    { 0x3d50f1b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf930f1ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2910f1c0, SUC_lexicon_tags_4 },
    { 0xe4d0f1c1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5610f1c6, SUC_lexicon_tags_31 },
    { 0xdd90f1c7, SUC_lexicon_tags_25 },
    { 0xd250f1c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc9b0f1ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd070f1cc, SUC_lexicon_tags_47 },
    { 0x52b0f1cd, SUC_lexicon_tags_61 },
    { 0x890f1ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b50f1d3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d0f1d9, SUC_lexicon_tags_14 },
    { 0xe4b0f1da, SUC_lexicon_tags_19 },
    { 0xe030f1da, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf110f1dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfd0f1df, SUC_lexicon_tags_33 },
    { 0xbd50f1df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd10f1e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4950f1e7, SUC_lexicon_tags_25 },
    { 0xfbd0f1e8, SUC_lexicon_tags_11 },
    { 0x470f1e7, SUC_lexicon_tags_37 },
    { 0xd2d0f1ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b0f1ed, SUC_lexicon_tags_87 },
    { 0xb530f1ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe770f1f2, SUC_lexicon_tags_37 },
    { 0x2a90f1f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc490f1f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc370f1fe, SUC_lexicon_tags_11 },
    { 0x5450f1fe, SUC_lexicon_tags_38 },
    { 0xe090f1ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55f0f202, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56d0f204, SUC_lexicon_tags_25 },
    { 0x3710f204, SUC_lexicon_tags_9 },
    { 0xc70f206, SUC_lexicon_tags_37 },
    { 0xa630f207, SUC_lexicon_tags_15 },
    { 0x6410f206, SUC_lexicon_tags_11 },
    { 0xb9b0f207, SUC_lexicon_tags_106 },
    { 0x1ff0f20a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe130f20c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a90f211, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xea50f214, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba50f21a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf0f224, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0f229, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb210f22e, SUC_lexicon_tags_14 },
    { 0xef10f22f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8570f233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe270f236, SUC_lexicon_tags_109 },
    { 0x9f70f236, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf830f23c, SUC_lexicon_tags_17 },
    { 0xbe30f23d, SUC_lexicon_tags_47 },
    { 0x2950f23e, SUC_lexicon_tags_63 },
    { 0x6470f23d, SUC_lexicon_tags_2 },
    { 0x10f0f23f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a90f243, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde90f245, SUC_lexicon_tags_25 },
    { 0x5690f246, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e70f24e, SUC_lexicon_tags_28 },
    { 0x1c70f24f, SUC_lexicon_tags_27 },
    { 0xde10f250, SUC_lexicon_tags_8 },
    { 0x1290f250, SUC_lexicon_tags_19 },
    { 0xd670f252, SUC_lexicon_tags_30 },
    { 0xffd0f253, SUC_lexicon_tags_39 },
    { 0x7a30f253, SUC_lexicon_tags_26 },
    { 0x9ed0f255, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd0f259, SUC_lexicon_tags_8 },
    { 0x1730f259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52d0f25c, SUC_lexicon_tags_25 },
    { 0xd830f25c, SUC_lexicon_tags_19 },
    { 0x5290f25e, SUC_lexicon_tags_12 },
    { 0x4d70f25f, SUC_lexicon_tags_25 },
    { 0x2c70f260, SUC_lexicon_tags_23 },
    { 0xb330f261, SUC_lexicon_tags_2 },
    { 0xe7f0f261, SUC_lexicon_tags_7 },
    { 0x5d30f263, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f0f266, SUC_lexicon_tags_47 },
    { 0x9cb0f266, SUC_lexicon_tags_2 },
    { 0x3b10f267, SUC_lexicon_tags_19 },
    { 0xa390f269, SUC_lexicon_tags_56 },
    { 0x3110f269, SUC_lexicon_tags_7 },
    { 0xa10f26b, SUC_lexicon_tags_17 },
    { 0x71b0f26b, SUC_lexicon_tags_38 },
    { 0x1090f26d, SUC_lexicon_tags_55 },
    { 0x86b0f26e, SUC_lexicon_tags_27 },
    { 0x2fb0f26d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd150f273, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff50f276, SUC_lexicon_tags_14 },
    { 0xb6d0f277, SUC_lexicon_tags_45 },
    { 0xae70f278, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xafd0f27a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1f70f27c, SUC_lexicon_tags_8 },
    { 0xdcf0f27d, SUC_lexicon_tags_1 },
    { 0x42d0f27e, SUC_lexicon_tags_26 },
    { 0xb530f27f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a10f281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b0f285, SUC_lexicon_tags_25 },
    { 0xd5d0f286, SUC_lexicon_tags_60 },
    { 0x6e70f287, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x790f289, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf0f28c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f0f28f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaeb0f291, SUC_lexicon_tags_23 },
    { 0x2ad0f292, SUC_lexicon_tags_25 },
    { 0x23d0f293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb10f297, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc90f29a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac50f29c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b0f2a0, SUC_lexicon_tags_7 },
    { 0x9a30f2a1, SUC_lexicon_tags_60 },
    { 0x7390f2a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9770f2a9, SUC_lexicon_tags_27 },
    { 0x9b90f2aa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3fb0f2ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf70f2b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5010f2b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f0f2b7, SUC_lexicon_tags_17 },
    { 0x4950f2b8, SUC_lexicon_tags_17 },
    { 0xce90f2b7, SUC_lexicon_tags_17 },
    { 0x4ef0f2ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a90f2be, SUC_lexicon_tags_27 },
    { 0xcfd0f2be, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xed70f2c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb0f2c6, SUC_lexicon_tags_61 },
    { 0xc470f2c7, SUC_lexicon_tags_15 },
    { 0x470f2c8, SUC_lexicon_tags_61 },
    { 0xadd0f2c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb50f2cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7610f2d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdb50f2d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f0f2dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed0f2e0, SUC_lexicon_tags_11 },
    { 0xe510f2e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb70f2e3, SUC_lexicon_tags_23 },
    { 0xa790f2e4, SUC_lexicon_tags_11 },
    { 0x1bf0f2e5, SUC_lexicon_tags_47 },
    { 0xf670f2e6, SUC_lexicon_tags_15 },
    { 0x4a30f2e6, SUC_lexicon_tags_37 },
    { 0x5950f2e7, SUC_lexicon_tags_0 },
    { 0x5c10f2e9, SUC_lexicon_tags_30 },
    { 0xf570f2ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb750f2ef, SUC_lexicon_tags_2 },
    { 0x2890f2f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4810f2f2, SUC_lexicon_tags_17 },
    { 0xc2d0f2f2, SUC_lexicon_tags_37 },
    { 0xfbf0f2f2, SUC_lexicon_tags_16 },
    { 0x59d0f2f4, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xd270f2f7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7b10f2f9, SUC_lexicon_tags_3 },
    { 0x8870f2fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10f2fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff0f300, SUC_lexicon_tags_6 },
    { 0xf390f301, SUC_lexicon_tags_7 },
    { 0xb70f302, SUC_lexicon_tags_15 },
    { 0x7d70f302, SUC_lexicon_tags_84 },
    { 0x52f0f304, SUC_lexicon_tags_14 },
    { 0x6ef0f304, SUC_lexicon_tags_39 },
    { 0xd1b0f304, SUC_lexicon_tags_11 },
    { 0x3c30f304, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8450f30a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9790f30c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4870f312, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff90f315, SUC_lexicon_tags_66 },
    { 0x4f90f315, SUC_lexicon_tags_1 },
    { 0x1430f317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd430f31e, SUC_lexicon_tags_15 },
    { 0xba50f31f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2450f324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb490f327, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b0f32d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d0f330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf310f332, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0x9c50f334, SUC_lexicon_tags_2 },
    { 0x38b0f335, SUC_lexicon_tags_11 },
    { 0x6090f336, SUC_lexicon_tags_37 },
    { 0x14f0f335, SUC_lexicon_tags_38 },
    { 0x2650f334, SUC_lexicon_tags_8 },
    { 0xf5f0f339, SUC_lexicon_tags_34 },
    { 0x4830f339, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd0f33d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb0f342, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb0f34a, SUC_lexicon_tags_26 },
    { 0x15d0f34b, SUC_lexicon_tags_1 },
    { 0x1450f34c, SUC_lexicon_tags_2 },
    { 0xf450f34c, SUC_lexicon_tags_26 },
    { 0x5a10f34e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe510f352, SUC_lexicon_tags_37 },
    { 0x5370f353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcd0f355, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6610f358, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f50f35c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab70f35f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6130f361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc470f366, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x67f0f368, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b0f36a, SUC_lexicon_tags_25 },
    { 0xd50f36a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x710f36d, SUC_lexicon_tags_37 },
    { 0x84f0f36e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf790f375, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e30f379, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff0f37d, SUC_lexicon_tags_43 },
    { 0xb330f37e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f0f382, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff0f386, SUC_lexicon_tags_45 },
    { 0x9810f386, SUC_lexicon_tags_45 },
    { 0x48b0f388, SUC_lexicon_tags_65 },
    { 0x8870f389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab0f392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ef0f394, SUC_lexicon_tags_15 },
    { 0xb7b0f395, SUC_lexicon_tags_27 },
    { 0xc350f395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b0f39a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd010f39d, SUC_lexicon_tags_17 },
    { 0x56f0f39e, SUC_lexicon_tags_31 },
    { 0xca90f39f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc70f3a3, SUC_lexicon_tags_68 },
    { 0xac90f3a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4750f3a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4d0f3ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd70f3ad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc490f3b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed70f3b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a10f3b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f0f3bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1570f3bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef0f3c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa70f3c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa410f3c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8350f3cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ef0f3cd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f50f3d0, SUC_lexicon_tags_0 },
    { 0x4e50f3d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1930f3d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c10f3d9, SUC_lexicon_tags_1 },
    { 0xd2f0f3da, SUC_lexicon_tags_17 },
    { 0xc450f3db, SUC_lexicon_tags_5 },
    { 0x76f0f3dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf50f3de, SUC_lexicon_tags_30 },
    { 0xff70f3df, SUC_lexicon_tags_14 },
    { 0x5670f3e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf0f3e5, SUC_lexicon_tags_7 },
    { 0x4df0f3e5, SUC_lexicon_tags_17 },
    { 0x4db0f3e7, SUC_lexicon_tags_14 },
    { 0x4490f3e6, SUC_lexicon_tags_48 },
    { 0xb1d0f3e9, SUC_lexicon_tags_21 },
    { 0x4510f3ea, SUC_lexicon_tags_17 },
    { 0xd630f3e6, SUC_lexicon_tags_17 },
    { 0x2950f3e8, SUC_lexicon_tags_22 },
    { 0xb630f3e8, SUC_lexicon_tags_2 },
    { 0xc870f3ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5b0f3f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f0f3f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1890f3fc, SUC_lexicon_tags_33 },
    { 0xd690f3fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x65f0f3ff, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8730f401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe90f403, SUC_lexicon_tags_25 },
    { 0x2350f404, SUC_lexicon_tags_37 },
    { 0x9130f405, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed50f408, SUC_lexicon_tags_29 },
    { 0x3bd0f409, SUC_lexicon_tags_6 },
    { 0xd9d0f40a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf150f40c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x65b0f40e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccb0f410, SUC_lexicon_tags_11 },
    { 0xbb50f410, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c50f413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d70f416, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b0f41a, SUC_lexicon_tags_1 },
    { 0x3fd0f41b, SUC_lexicon_tags_14 },
    { 0xbdf0f41a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd0f421, SUC_lexicon_tags_597 },
    { 0x1ef0f421, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f30f424, SUC_lexicon_tags_11 },
    { 0x1cd0f424, SUC_lexicon_tags_15 },
    { 0x2930f426, SUC_lexicon_tags_9 },
    { 0x48b0f425, SUC_lexicon_tags_21 },
    { 0xae30f424, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d50f42f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf0f432, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c50f435, SUC_lexicon_tags_31 },
    { 0x7030f436, SUC_lexicon_tags_12 },
    { 0xd550f437, SUC_lexicon_tags_15 },
    { 0x8870f438, SUC_lexicon_tags_2 },
    { 0x77f0f439, SUC_lexicon_tags_65 },
    { 0x1f10f43a, SUC_lexicon_tags_2 },
    { 0x9830f43b, SUC_lexicon_tags_25 },
    { 0xf230f435, SUC_lexicon_tags_17 },
    { 0x2b70f43d, SUC_lexicon_tags_12 },
    { 0x5890f436, SUC_lexicon_tags_25 },
    { 0xd190f43e, SUC_lexicon_tags_77 },
    { 0x40b0f436, SUC_lexicon_tags_9 },
    { 0xc910f43b, SUC_lexicon_tags_9 },
    { 0x6c10f435, SUC_lexicon_tags_12 },
    { 0x9890f439, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b30f446, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc270f448, SUC_lexicon_tags_17 },
    { 0x2c70f449, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f0f44d, SUC_lexicon_tags_27 },
    { 0xb9b0f44d, SUC_lexicon_tags_61 },
    { 0x5dd0f44f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8370f452, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7650f457, SUC_lexicon_tags_37 },
    { 0x53b0f458, SUC_lexicon_tags_2 },
    { 0x8470f457, SUC_lexicon_tags_17 },
    { 0x1690f45a, SUC_lexicon_tags_37 },
    { 0x5e90f458, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f0f45e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe870f461, SUC_lexicon_tags_1 },
    { 0x5c50f462, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe150f465, SUC_lexicon_tags_7 },
    { 0xe9d0f465, SUC_lexicon_tags_12 },
    { 0xa350f466, SUC_lexicon_tags_7 },
    { 0x990f468, SUC_lexicon_tags_87 },
    { 0x31d0f467, SUC_lexicon_tags_45 },
    { 0x2210f469, SUC_lexicon_tags_19 },
    { 0xf270f46a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d30f46f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e10f472, SUC_lexicon_tags_27 },
    { 0x60b0f472, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x88f0f475, SUC_lexicon_tags_12 },
    { 0x83b0f476, SUC_lexicon_tags_38 },
    { 0x1e50f477, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61d0f479, SUC_lexicon_tags_678 },
    { 0x8a90f479, SUC_lexicon_tags_65 },
    { 0xe730f479, SUC_lexicon_tags_23 },
    { 0x9990f47c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68f0f47e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef0f481, SUC_lexicon_tags_19 },
    { 0x9a90f482, SUC_lexicon_tags_11 },
    { 0xef0f483, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xddd0f485, SUC_lexicon_tags_77 },
    { 0x7c10f486, SUC_lexicon_tags_11 },
    { 0x350f487, SUC_lexicon_tags_0 },
    { 0xe710f488, SUC_lexicon_tags_14 },
    { 0xf950f486, SUC_lexicon_tags_27 },
    { 0x8e70f487, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe30f48f, SUC_lexicon_tags_7 },
    { 0x8e90f48f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde10f495, SUC_lexicon_tags_17 },
    { 0x9ab0f496, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82b0f498, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef30f49c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9250f49f, SUC_lexicon_tags_45 },
    { 0x4e50f4a0, SUC_lexicon_tags_87 },
    { 0xd5d0f4a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb250f4a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xadb0f4a5, SUC_lexicon_tags_7 },
    { 0xe8f0f4a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf0f0f4a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f0f4b1, SUC_lexicon_tags_57 },
    { 0x8e70f4b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeef0f4b4, SUC_lexicon_tags_59 },
    { 0xa330f4b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb730f4b8, SUC_lexicon_tags_0 },
    { 0x6fb0f4b9, SUC_lexicon_tags_153 },
    { 0, NULL },
    { 0x1970f4bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd70f4be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b0f4c4, SUC_lexicon_tags_129 },
    { 0x14b0f4c5, SUC_lexicon_tags_10 },
    { 0x8a70f4c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d0f4c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6990f4d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x47b0f4d2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f0f4d6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xae10f4d8, SUC_lexicon_tags_19 },
    { 0x37b0f4d9, SUC_lexicon_tags_11 },
    { 0x29b0f4da, SUC_lexicon_tags_45 },
    { 0x6330f4da, SUC_lexicon_tags_17 },
    { 0x3bd0f4d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8830f4de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf730f4e5, SUC_lexicon_tags_6 },
    { 0x950f4e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b0f4e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d0f4f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2790f4f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f0f4f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe870f4fa, SUC_lexicon_tags_2 },
    { 0x43f0f4fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x930f4fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8710f504, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8490f506, SUC_lexicon_tags_2 },
    { 0xe450f506, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca90f509, SUC_lexicon_tags_106 },
    { 0x4e70f50a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd7d0f50c, SUC_lexicon_tags_10 },
    { 0xc570f50c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa10f510, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb0f519, SUC_lexicon_tags_47 },
    { 0x1510f51a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc210f522, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6a90f524, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb0f527, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab0f52b, SUC_lexicon_tags_10 },
    { 0x2bd0f52c, SUC_lexicon_tags_15 },
    { 0x5f70f52d, SUC_lexicon_tags_13 },
    { 0x8bd0f52d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa30f531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb810f535, SUC_lexicon_tags_11 },
    { 0x5a90f536, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e30f53a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd0f53d, SUC_lexicon_tags_27 },
    { 0x60f0f53e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9f0f540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af0f547, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5850f549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed70f54c, SUC_lexicon_tags_33 },
    { 0xb930f54c, SUC_lexicon_tags_7 },
    { 0xe850f54e, SUC_lexicon_tags_27 },
    { 0x80b0f54c, SUC_lexicon_tags_15 },
    { 0x58d0f54d, SUC_lexicon_tags_25 },
    { 0xfb0f54e, SUC_lexicon_tags_38 },
    { 0xa2f0f551, SUC_lexicon_tags_2 },
    { 0xd70f550, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3450f557, SUC_lexicon_tags_48 },
    { 0x3710f557, SUC_lexicon_tags_17 },
    { 0x770f557, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x79b0f55b, SUC_lexicon_tags_7 },
    { 0x8730f55b, SUC_lexicon_tags_79 },
    { 0x930f55b, SUC_lexicon_tags_17 },
    { 0x5490f55e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d0f561, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5810f564, SUC_lexicon_tags_4 },
    { 0xd7d0f565, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2910f567, SUC_lexicon_tags_11 },
    { 0xa170f568, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4730f56b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1130f56d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x50d0f56f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8370f572, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7df0f574, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f0f577, SUC_lexicon_tags_0 },
    { 0x5610f578, SUC_lexicon_tags_33 },
    { 0xd1f0f579, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb610f57c, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xbc30f57e, SUC_lexicon_tags_63 },
    { 0x5210f57f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbb90f581, SUC_lexicon_tags_17 },
    { 0x7ed0f581, SUC_lexicon_tags_29 },
    { 0x6010f582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d0f588, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfeb0f58a, SUC_lexicon_tags_9 },
    { 0x99f0f58b, SUC_lexicon_tags_62 },
    { 0x7ff0f58c, SUC_lexicon_tags_38 },
    { 0x62b0f58d, SUC_lexicon_tags_12 },
    { 0xce30f58c, SUC_lexicon_tags_37 },
    { 0x4f30f58a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x630f591, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xed90f594, SUC_lexicon_tags_11 },
    { 0xc370f594, SUC_lexicon_tags_53 },
    { 0x29d0f596, SUC_lexicon_tags_72 },
    { 0xf3b0f595, SUC_lexicon_tags_3 },
    { 0x8330f598, SUC_lexicon_tags_22 },
    { 0xddf0f599, SUC_lexicon_tags_7 },
    { 0xc530f595, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7410f5a3, SUC_lexicon_tags_27 },
    { 0x66b0f5a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd710f5a8, SUC_lexicon_tags_45 },
    { 0x74f0f5a9, SUC_lexicon_tags_27 },
    { 0xf670f5aa, SUC_lexicon_tags_14 },
    { 0x67f0f5a9, SUC_lexicon_tags_17 },
    { 0x4010f5ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42b0f5ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x35f0f5b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2f0f5b2, SUC_lexicon_tags_49 },
    { 0x87b0f5b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2190f5b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xecf0f5b8, SUC_lexicon_tags_29 },
    { 0x4b0f5b9, SUC_lexicon_tags_31 },
    { 0x6850f5ba, SUC_lexicon_tags_25 },
    { 0x9db0f5ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb90f5c7, SUC_lexicon_tags_10 },
    { 0xa6f0f5c7, SUC_lexicon_tags_45 },
    { 0xff30f5c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6130f5cc, SUC_lexicon_tags_1 },
    { 0xe950f5cc, SUC_lexicon_tags_32 },
    { 0xe890f5cc, SUC_lexicon_tags_12 },
    { 0xfef0f5ce, SUC_lexicon_tags_7 },
    { 0x8790f5d0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af0f5d4, SUC_lexicon_tags_10 },
    { 0xe110f5d4, SUC_lexicon_tags_47 },
    { 0x3f90f5d4, SUC_lexicon_tags_9 },
    { 0xf210f5d7, SUC_lexicon_tags_37 },
    { 0x9db0f5d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b0f5dc, SUC_lexicon_tags_7 },
    { 0x5cf0f5dd, SUC_lexicon_tags_27 },
    { 0x770f5de, SUC_lexicon_tags_23 },
    { 0x6330f5df, SUC_lexicon_tags_129 },
    { 0xac90f5e0, SUC_lexicon_tags_25 },
    { 0xbd50f5e0, SUC_lexicon_tags_17 },
    { 0x950f5e2, SUC_lexicon_tags_66 },
    { 0x5910f5e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6430f5e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde10f5ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6790f5f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7510f5f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbad0f5f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a50f5fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f5fd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf70f602, SUC_lexicon_tags_0 },
    { 0x6cf0f603, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8170f605, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb990f608, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6670f60b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3530f60d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfa30f60f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc50f613, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4030f615, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d0f618, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb710f623, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa90f627, SUC_lexicon_tags_43 },
    { 0x9890f627, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd50f62a, SUC_lexicon_tags_11 },
    { 0xa3d0f62b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab0f634, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61d0f636, SUC_lexicon_tags_37 },
    { 0x22d0f637, SUC_lexicon_tags_0 },
    { 0x5dd0f638, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d0f63b, SUC_lexicon_tags_55 },
    { 0x4510f63c, SUC_lexicon_tags_1 },
    { 0xdc10f63c, SUC_lexicon_tags_47 },
    { 0xabd0f63e, SUC_lexicon_tags_0 },
    { 0xa110f63f, SUC_lexicon_tags_37 },
    { 0xbb0f63f, SUC_lexicon_tags_25 },
    { 0xf0b0f641, SUC_lexicon_tags_268 },
    { 0x6f0f641, SUC_lexicon_tags_15 },
    { 0xc430f641, SUC_lexicon_tags_39 },
    { 0x5e30f63e, SUC_lexicon_tags_43 },
    { 0x8630f63e, SUC_lexicon_tags_17 },
    { 0xa990f63c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7b0f648, SUC_lexicon_tags_79 },
    { 0x5e70f648, SUC_lexicon_tags_43 },
    { 0xdff0f649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb0f64e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530f653, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd210f655, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a90f658, SUC_lexicon_tags_30 },
    { 0xeb30f659, SUC_lexicon_tags_25 },
    { 0x9270f65a, SUC_lexicon_tags_1 },
    { 0x6110f65a, SUC_lexicon_tags_1 },
    { 0x56f0f65c, SUC_lexicon_tags_25 },
    { 0xb910f65d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d0f661, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe550f663, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d0f666, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5410f668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde30f66a, SUC_lexicon_tags_1434 },
    { 0, NULL },
    { 0xee30f66c, SUC_lexicon_tags_1 },
    { 0xd650f66d, SUC_lexicon_tags_25 },
    { 0x1150f66e, SUC_lexicon_tags_27 },
    { 0x6870f66f, SUC_lexicon_tags_7 },
    { 0x5830f66e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x72d0f672, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c90f675, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2810f67b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4f0f67d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea90f67f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d0f681, SUC_lexicon_tags_37 },
    { 0x24d0f682, SUC_lexicon_tags_25 },
    { 0xbcb0f682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd70f686, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc090f688, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb310f692, SUC_lexicon_tags_19 },
    { 0xe4d0f693, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7310f698, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f0f69c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb30f69e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f10f6a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22b0f6a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc490f6a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb050f6a9, SUC_lexicon_tags_47 },
    { 0x24d0f6a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf990f6ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f6b1, SUC_lexicon_tags_1 },
    { 0x1290f6b2, SUC_lexicon_tags_5 },
    { 0x7e50f6b3, SUC_lexicon_tags_87 },
    { 0xfbb0f6b4, SUC_lexicon_tags_15 },
    { 0xf910f6b5, SUC_lexicon_tags_11 },
    { 0xdff0f6b6, SUC_lexicon_tags_44 },
    { 0x4d0f6b7, SUC_lexicon_tags_16 },
    { 0x7730f6b8, SUC_lexicon_tags_27 },
    { 0x3650f6b7, SUC_lexicon_tags_15 },
    { 0xcab0f6ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f0f6bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd3b0f6bf, SUC_lexicon_tags_19 },
    { 0x15f0f6bf, SUC_lexicon_tags_87 },
    { 0xb5d0f6bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a30f6c8, SUC_lexicon_tags_68 },
    { 0xdcd0f6c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x530f6ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1430f6d9, SUC_lexicon_tags_33 },
    { 0xabb0f6d9, SUC_lexicon_tags_80 },
    { 0x1f10f6db, SUC_lexicon_tags_87 },
    { 0x8030f6dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7830f6e2, SUC_lexicon_tags_37 },
    { 0xc510f6e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd430f6e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7f0f6e7, SUC_lexicon_tags_14 },
    { 0x7950f6e8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x3810f6ea, SUC_lexicon_tags_6 },
    { 0x1710f6eb, SUC_lexicon_tags_27 },
    { 0xcdf0f6ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1470f6ef, SUC_lexicon_tags_11 },
    { 0xdd50f6ef, SUC_lexicon_tags_68 },
    { 0x3ab0f6f1, SUC_lexicon_tags_1 },
    { 0x85f0f6f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc990f6f5, SUC_lexicon_tags_37 },
    { 0x5a50f6f6, SUC_lexicon_tags_43 },
    { 0x2230f6f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee90f6f9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b0f6ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f0f707, SUC_lexicon_tags_11 },
    { 0x8450f708, SUC_lexicon_tags_17 },
    { 0x1bb0f709, SUC_lexicon_tags_60 },
    { 0xff10f707, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9250f70d, SUC_lexicon_tags_30 },
    { 0x3ab0f70e, SUC_lexicon_tags_2 },
    { 0x8510f70f, SUC_lexicon_tags_29 },
    { 0x410f710, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb30f712, SUC_lexicon_tags_38 },
    { 0xc6b0f713, SUC_lexicon_tags_5 },
    { 0x5d10f714, SUC_lexicon_tags_11 },
    { 0xad90f714, SUC_lexicon_tags_60 },
    { 0xced0f712, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1450f719, SUC_lexicon_tags_16 },
    { 0x9950f719, SUC_lexicon_tags_14 },
    { 0xa750f71b, SUC_lexicon_tags_25 },
    { 0xe430f71b, SUC_lexicon_tags_0 },
    { 0x5f70f71d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5990f71f, SUC_lexicon_tags_7 },
    { 0xd90f720, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fb0f722, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1970f726, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5750f72a, SUC_lexicon_tags_37 },
    { 0x6490f72a, SUC_lexicon_tags_37 },
    { 0x43d0f72c, SUC_lexicon_tags_29 },
    { 0xde10f72d, SUC_lexicon_tags_25 },
    { 0xecd0f72e, SUC_lexicon_tags_11 },
    { 0x4bf0f72a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4690f734, SUC_lexicon_tags_27 },
    { 0x7e70f735, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xed90f737, SUC_lexicon_tags_26 },
    { 0x7c90f738, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf770f73b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb450f73d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc790f740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d0f743, SUC_lexicon_tags_0 },
    { 0xd950f744, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b0f748, SUC_lexicon_tags_37 },
    { 0x4cf0f749, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9770f74c, SUC_lexicon_tags_428 },
    { 0xc750f74c, SUC_lexicon_tags_31 },
    { 0x7030f74d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7070f752, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530f757, SUC_lexicon_tags_37 },
    { 0x490f757, SUC_lexicon_tags_14 },
    { 0x3810f759, SUC_lexicon_tags_60 },
    { 0x4770f75a, SUC_lexicon_tags_25 },
    { 0x5630f75a, SUC_lexicon_tags_5 },
    { 0xfb0f759, SUC_lexicon_tags_7 },
    { 0x4ff0f75b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa70f764, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf850f766, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9150f76b, SUC_lexicon_tags_37 },
    { 0x54b0f76b, SUC_lexicon_tags_9 },
    { 0x410f76d, SUC_lexicon_tags_37 },
    { 0x5df0f76e, SUC_lexicon_tags_28 },
    { 0xeb0f76d, SUC_lexicon_tags_21 },
    { 0xb4d0f76e, SUC_lexicon_tags_19 },
    { 0xaa50f771, SUC_lexicon_tags_47 },
    { 0xe810f771, SUC_lexicon_tags_14 },
    { 0x89d0f773, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c30f775, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d0f779, SUC_lexicon_tags_0 },
    { 0x61b0f77a, SUC_lexicon_tags_12 },
    { 0x8890f779, SUC_lexicon_tags_15 },
    { 0x6f50f77a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb470f77e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf30f781, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1dd0f783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b0f787, SUC_lexicon_tags_15 },
    { 0x7a30f787, SUC_lexicon_tags_25 },
    { 0x5810f789, SUC_lexicon_tags_19 },
    { 0x9450f78a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb970f793, SUC_lexicon_tags_15 },
    { 0xf170f794, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef0f797, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6730f79c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2530f79e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab0f7a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf690f7a4, SUC_lexicon_tags_25 },
    { 0x8c90f7a5, SUC_lexicon_tags_25 },
    { 0x3370f7a4, SUC_lexicon_tags_7 },
    { 0x1350f7a7, SUC_lexicon_tags_37 },
    { 0x23f0f7a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0f7af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f0f7b2, SUC_lexicon_tags_33 },
    { 0x2270f7b2, SUC_lexicon_tags_23 },
    { 0x9a10f7b4, SUC_lexicon_tags_37 },
    { 0x9bb0f7b4, SUC_lexicon_tags_17 },
    { 0x5fb0f7b6, SUC_lexicon_tags_573 },
    { 0, NULL },
    { 0x2a70f7b8, SUC_lexicon_tags_30 },
    { 0x1c30f7b9, SUC_lexicon_tags_2 },
    { 0x66d0f7ba, SUC_lexicon_tags_25 },
    { 0x3eb0f7b9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f70f7be, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f0f7c1, SUC_lexicon_tags_25 },
    { 0x9c90f7c2, SUC_lexicon_tags_17 },
    { 0x1990f7c3, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d90f7c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7870f7ca, SUC_lexicon_tags_33 },
    { 0xd830f7cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4f0f7cd, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b0f7d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc330f7d4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa290f7d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd0f7dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af0f7e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c70f7e5, SUC_lexicon_tags_1 },
    { 0x62b0f7e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d30f7e8, SUC_lexicon_tags_38 },
    { 0xba70f7e9, SUC_lexicon_tags_15 },
    { 0x7970f7ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4130f7ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a50f7fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b0f800, SUC_lexicon_tags_12 },
    { 0x2370f801, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd0f805, SUC_lexicon_tags_47 },
    { 0x2110f806, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd10f809, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x810f80b, SUC_lexicon_tags_2 },
    { 0xdab0f80c, SUC_lexicon_tags_12 },
    { 0xf550f80d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe1d0f80f, SUC_lexicon_tags_63 },
    { 0x7cb0f80f, SUC_lexicon_tags_37 },
    { 0x76d0f80f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e30f813, SUC_lexicon_tags_11 },
    { 0x7dd0f814, SUC_lexicon_tags_0 },
    { 0xd970f815, SUC_lexicon_tags_25 },
    { 0x79d0f814, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb50f819, SUC_lexicon_tags_2 },
    { 0xc3b0f81a, SUC_lexicon_tags_0 },
    { 0xb330f819, SUC_lexicon_tags_17 },
    { 0xd30f81c, SUC_lexicon_tags_17 },
    { 0xfab0f81d, SUC_lexicon_tags_7 },
    { 0xad90f81b, SUC_lexicon_tags_27 },
    { 0xc2f0f81b, SUC_lexicon_tags_62 },
    { 0xda30f81e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d70f822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7350f82a, SUC_lexicon_tags_8 },
    { 0xbfb0f82b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf0f831, SUC_lexicon_tags_27 },
    { 0xc450f832, SUC_lexicon_tags_43 },
    { 0x5230f832, SUC_lexicon_tags_17 },
    { 0x6650f833, SUC_lexicon_tags_7 },
    { 0x25d0f835, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7470f837, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a90f83a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa50f83f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d0f846, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb0f849, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4cb0f84b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25d0f84d, SUC_lexicon_tags_61 },
    { 0x6570f84d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc10f850, SUC_lexicon_tags_43 },
    { 0xc4d0f851, SUC_lexicon_tags_0 },
    { 0x4ef0f852, SUC_lexicon_tags_12 },
    { 0x3690f851, SUC_lexicon_tags_27 },
    { 0x4170f853, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc770f856, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed0f85c, SUC_lexicon_tags_2 },
    { 0x53f0f85d, SUC_lexicon_tags_61 },
    { 0xf8f0f85d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf0f860, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f0f865, SUC_lexicon_tags_29 },
    { 0x8fb0f866, SUC_lexicon_tags_49 },
    { 0x13d0f867, SUC_lexicon_tags_11 },
    { 0x10b0f868, SUC_lexicon_tags_19 },
    { 0xf610f867, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a30f86d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab0f871, SUC_lexicon_tags_37 },
    { 0x7170f872, SUC_lexicon_tags_15 },
    { 0x7130f872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3450f877, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb3b0f879, SUC_lexicon_tags_8 },
    { 0x4450f87a, SUC_lexicon_tags_65 },
    { 0x8510f87a, SUC_lexicon_tags_77 },
    { 0xc570f87c, SUC_lexicon_tags_23 },
    { 0x8c50f87d, SUC_lexicon_tags_65 },
    { 0xddf0f87c, SUC_lexicon_tags_49 },
    { 0xeef0f87c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2630f881, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe9f0f883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a70f886, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xca50f888, SUC_lexicon_tags_2 },
    { 0x94b0f889, SUC_lexicon_tags_19 },
    { 0x9870f88a, SUC_lexicon_tags_1654 },
    { 0x430f88b, SUC_lexicon_tags_15 },
    { 0x29f0f88c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff0f893, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e70f896, SUC_lexicon_tags_8 },
    { 0x4650f896, SUC_lexicon_tags_12 },
    { 0xc950f898, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb870f89a, SUC_lexicon_tags_19 },
    { 0xedb0f89b, SUC_lexicon_tags_17 },
    { 0xfc50f89c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa2d0f89e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4810f8a1, SUC_lexicon_tags_62 },
    { 0xa7d0f8a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3810f8a5, SUC_lexicon_tags_61 },
    { 0x6ab0f8a6, SUC_lexicon_tags_12 },
    { 0xdf70f8a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6190f8aa, SUC_lexicon_tags_7 },
    { 0x7230f8ab, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc070f8ad, SUC_lexicon_tags_23 },
    { 0x17b0f8ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2690f8b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6870f8b7, SUC_lexicon_tags_31 },
    { 0xf2b0f8b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8270f8bc, SUC_lexicon_tags_17 },
    { 0x4630f8bd, SUC_lexicon_tags_47 },
    { 0x1370f8be, SUC_lexicon_tags_37 },
    { 0xcf0f8bf, SUC_lexicon_tags_37 },
    { 0xea50f8c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1a30f8c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x67b0f8c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d0f8c7, SUC_lexicon_tags_25 },
    { 0x55b0f8c8, SUC_lexicon_tags_7 },
    { 0x330f8c9, SUC_lexicon_tags_2 },
    { 0xe490f8ca, SUC_lexicon_tags_37 },
    { 0xe730f8c9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xdb70f8cd, SUC_lexicon_tags_11 },
    { 0x83b0f8ce, SUC_lexicon_tags_48 },
    { 0xe150f8cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd330f8d1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd0f8d4, SUC_lexicon_tags_47 },
    { 0x5230f8d5, SUC_lexicon_tags_17 },
    { 0xfeb0f8d6, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d0f8d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb430f8db, SUC_lexicon_tags_37 },
    { 0xedd0f8dc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa450f8de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x910f8e5, SUC_lexicon_tags_1 },
    { 0xbbf0f8e6, SUC_lexicon_tags_27 },
    { 0xb010f8e7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2090f8e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b10f8ee, SUC_lexicon_tags_63 },
    { 0x5330f8ef, SUC_lexicon_tags_15 },
    { 0xa150f8f0, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef0f8f4, SUC_lexicon_tags_15 },
    { 0x3c10f8f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12f0f8f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab50f8fa, SUC_lexicon_tags_2 },
    { 0x1b30f8fa, SUC_lexicon_tags_48 },
    { 0xb950f8fc, SUC_lexicon_tags_47 },
    { 0xa90f8fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2290f903, SUC_lexicon_tags_6 },
    { 0x97f0f904, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8d30f906, SUC_lexicon_tags_10 },
    { 0x9c70f907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1d0f909, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1190f90b, SUC_lexicon_tags_37 },
    { 0xf230f90c, SUC_lexicon_tags_37 },
    { 0x2d90f90c, SUC_lexicon_tags_14 },
    { 0x9a70f90e, SUC_lexicon_tags_68 },
    { 0xfa90f90f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5d0f911, SUC_lexicon_tags_11 },
    { 0x3770f911, SUC_lexicon_tags_17 },
    { 0xe2f0f912, SUC_lexicon_tags_30 },
    { 0xd870f914, SUC_lexicon_tags_27 },
    { 0xa450f913, SUC_lexicon_tags_11 },
    { 0x9ff0f912, SUC_lexicon_tags_37 },
    { 0xb190f917, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe390f91e, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a50f921, SUC_lexicon_tags_39 },
    { 0x2750f922, SUC_lexicon_tags_1 },
    { 0x2bb0f923, SUC_lexicon_tags_12 },
    { 0x7570f924, SUC_lexicon_tags_15 },
    { 0x3330f925, SUC_lexicon_tags_19 },
    { 0xc130f925, SUC_lexicon_tags_32 },
    { 0x4370f927, SUC_lexicon_tags_33 },
    { 0x2750f928, SUC_lexicon_tags_7 },
    { 0x76f0f929, SUC_lexicon_tags_23 },
    { 0x1190f92a, SUC_lexicon_tags_30 },
    { 0xf3d0f924, SUC_lexicon_tags_8 },
    { 0xf630f922, SUC_lexicon_tags_62 },
    { 0x370f92d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xecd0f92f, SUC_lexicon_tags_27 },
    { 0x7dd0f930, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2590f93c, SUC_lexicon_tags_17 },
    { 0xdd10f93d, SUC_lexicon_tags_7 },
    { 0x3c70f93e, SUC_lexicon_tags_25 },
    { 0x8fd0f93d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xac90f941, SUC_lexicon_tags_7 },
    { 0xb430f942, SUC_lexicon_tags_28 },
    { 0xd0f943, SUC_lexicon_tags_7 },
    { 0xc110f943, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf050f949, SUC_lexicon_tags_47 },
    { 0xcef0f94a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8910f94d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3670f954, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d0f958, SUC_lexicon_tags_21 },
    { 0xa990f959, SUC_lexicon_tags_11 },
    { 0x2f0f95a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a90f95c, SUC_lexicon_tags_7 },
    { 0xe2d0f95c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfc30f95f, SUC_lexicon_tags_63 },
    { 0xfa50f960, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2350f962, SUC_lexicon_tags_66 },
    { 0x5af0f963, SUC_lexicon_tags_15 },
    { 0xfcd0f962, SUC_lexicon_tags_37 },
    { 0xdcf0f962, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e30f96c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb690f972, SUC_lexicon_tags_706 },
    { 0xa3d0f972, SUC_lexicon_tags_49 },
    { 0xc830f973, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7250f976, SUC_lexicon_tags_15 },
    { 0x2810f977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5830f97d, SUC_lexicon_tags_37 },
    { 0x6330f97d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8990f983, SUC_lexicon_tags_37 },
    { 0x51b0f983, SUC_lexicon_tags_3 },
    { 0xad90f985, SUC_lexicon_tags_2 },
    { 0xead0f986, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59d0f988, SUC_lexicon_tags_84 },
    { 0x1930f989, SUC_lexicon_tags_10 },
    { 0xc270f988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb50f990, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e70f997, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e90f99b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e10f9a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d0f9a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9170f9ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb770f9ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d70f9b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf0f9b5, SUC_lexicon_tags_7 },
    { 0x8c90f9b6, SUC_lexicon_tags_15 },
    { 0xd6d0f9b6, SUC_lexicon_tags_2 },
    { 0x9950f9b7, SUC_lexicon_tags_11 },
    { 0xb9d0f9b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f0f9be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf270f9c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1570f9c2, SUC_lexicon_tags_1 },
    { 0x8830f9c3, SUC_lexicon_tags_15 },
    { 0xb330f9c4, SUC_lexicon_tags_63 },
    { 0xfab0f9c5, SUC_lexicon_tags_48 },
    { 0xa190f9c4, SUC_lexicon_tags_10 },
    { 0x32f0f9c6, SUC_lexicon_tags_1 },
    { 0x6e90f9c8, SUC_lexicon_tags_39 },
    { 0xa950f9c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9fb0f9cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc850f9d4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9ab0f9d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d0f9da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe350f9e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf0f9e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2270f9e8, SUC_lexicon_tags_37 },
    { 0x5db0f9e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde50f9ec, SUC_lexicon_tags_23 },
    { 0x68b0f9ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3430f9f0, SUC_lexicon_tags_15 },
    { 0x86d0f9f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x450f9f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1430f9f6, SUC_lexicon_tags_1714 },
    { 0xb8f0f9f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc30f9fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96f0f9fc, SUC_lexicon_tags_2 },
    { 0xf4b0f9fc, SUC_lexicon_tags_15 },
    { 0x75f0f9fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac30fa00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d0fa0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe30fa12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8450fa14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd930fa16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb70fa18, SUC_lexicon_tags_30 },
    { 0xc090fa19, SUC_lexicon_tags_30 },
    { 0x93b0fa1a, SUC_lexicon_tags_48 },
    { 0x7330fa19, SUC_lexicon_tags_30 },
    { 0x9bb0fa1c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc790fa21, SUC_lexicon_tags_65 },
    { 0x9490fa22, SUC_lexicon_tags_11 },
    { 0x5030fa23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47f0fa25, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd0fa2a, SUC_lexicon_tags_11 },
    { 0xf9d0fa2b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b0fa2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd0fa32, SUC_lexicon_tags_396 },
    { 0x9e50fa33, SUC_lexicon_tags_7 },
    { 0xad30fa33, SUC_lexicon_tags_37 },
    { 0xa10fa32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa410fa37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f50fa39, SUC_lexicon_tags_2 },
    { 0xc710fa39, SUC_lexicon_tags_11 },
    { 0x66b0fa3b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf90fa41, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7450fa45, SUC_lexicon_tags_103 },
    { 0x22b0fa46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab0fa4a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b0fa4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f0fa54, SUC_lexicon_tags_16 },
    { 0xfeb0fa55, SUC_lexicon_tags_17 },
    { 0x7e90fa55, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc070fa5b, SUC_lexicon_tags_14 },
    { 0x6f70fa5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6790fa65, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3950fa67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd190fa69, SUC_lexicon_tags_0 },
    { 0xdf30fa6a, SUC_lexicon_tags_160 },
    { 0x7530fa69, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f0fa6f, SUC_lexicon_tags_11 },
    { 0x50d0fa6f, SUC_lexicon_tags_32 },
    { 0xda30fa6f, SUC_lexicon_tags_31 },
    { 0xdd90fa72, SUC_lexicon_tags_25 },
    { 0x75f0fa73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d50fa7e, SUC_lexicon_tags_178 },
    { 0xccf0fa7f, SUC_lexicon_tags_33 },
    { 0xf6d0fa80, SUC_lexicon_tags_65 },
    { 0x5550fa81, SUC_lexicon_tags_15 },
    { 0xe790fa80, SUC_lexicon_tags_36 },
    { 0x6e70fa83, SUC_lexicon_tags_18 },
    { 0xa250fa84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5770fa89, SUC_lexicon_tags_15 },
    { 0xecb0fa8a, SUC_lexicon_tags_17 },
    { 0xbe50fa8b, SUC_lexicon_tags_2 },
    { 0x75b0fa8c, SUC_lexicon_tags_1171 },
    { 0x3a30fa8c, SUC_lexicon_tags_1 },
    { 0x4290fa8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d50fa91, SUC_lexicon_tags_114 },
    { 0x2990fa91, SUC_lexicon_tags_25 },
    { 0x2d30fa93, SUC_lexicon_tags_1 },
    { 0xe490fa92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb0fa97, SUC_lexicon_tags_2 },
    { 0xdf50fa98, SUC_lexicon_tags_14 },
    { 0xd310fa99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff0fa9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf830faa3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b50faa8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x35b0faaa, SUC_lexicon_tags_17 },
    { 0xe8f0faaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b50fab0, SUC_lexicon_tags_74 },
    { 0xaf90fab1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd0fab5, SUC_lexicon_tags_19 },
    { 0xd250fab6, SUC_lexicon_tags_17 },
    { 0x4f90fab7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f10faba, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b0fabf, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6710fac1, SUC_lexicon_tags_68 },
    { 0x9450fac2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b0fac5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc150facb, SUC_lexicon_tags_14 },
    { 0xe9d0facc, SUC_lexicon_tags_28 },
    { 0xfab0facd, SUC_lexicon_tags_21 },
    { 0x1470facb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65d0fad0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb150fad5, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb0fad8, SUC_lexicon_tags_37 },
    { 0x350fad9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7390fadb, SUC_lexicon_tags_17 },
    { 0x76b0fadc, SUC_lexicon_tags_23 },
    { 0xc6d0fadc, SUC_lexicon_tags_18 },
    { 0xa830fade, SUC_lexicon_tags_8 },
    { 0xae70fadf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8670fae1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6f0fae3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb770fae5, SUC_lexicon_tags_11 },
    { 0x6d70fae6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb0faea, SUC_lexicon_tags_37 },
    { 0x8a90faea, SUC_lexicon_tags_37 },
    { 0xa50faec, SUC_lexicon_tags_37 },
    { 0x630faed, SUC_lexicon_tags_25 },
    { 0xac30faea, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x2170faf0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3650faf2, SUC_lexicon_tags_11 },
    { 0x8610faf2, SUC_lexicon_tags_60 },
    { 0xec30faf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb0faf7, SUC_lexicon_tags_9 },
    { 0x8c50faf8, SUC_lexicon_tags_1 },
    { 0x9990faf9, SUC_lexicon_tags_16 },
    { 0x4570fafa, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc710fb01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda50fb05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9450fb0a, SUC_lexicon_tags_23 },
    { 0xbbb0fb0b, SUC_lexicon_tags_17 },
    { 0xed30fb0c, SUC_lexicon_tags_7 },
    { 0xff10fb0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7370fb1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1450fb21, SUC_lexicon_tags_87 },
    { 0xe2b0fb21, SUC_lexicon_tags_15 },
    { 0xd770fb23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd0fb26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x970fb2b, SUC_lexicon_tags_38 },
    { 0x50b0fb2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd0fb2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b0fb36, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7bb0fb38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d0fb3b, SUC_lexicon_tags_7 },
    { 0xd270fb3b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e30fb3e, SUC_lexicon_tags_2 },
    { 0xc550fb3f, SUC_lexicon_tags_2 },
    { 0x830fb3f, SUC_lexicon_tags_11 },
    { 0x7370fb41, SUC_lexicon_tags_17 },
    { 0x74f0fb42, SUC_lexicon_tags_1 },
    { 0x68b0fb43, SUC_lexicon_tags_1 },
    { 0x73b0fb44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd950fb4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4910fb4d, SUC_lexicon_tags_11 },
    { 0xad70fb4d, SUC_lexicon_tags_17 },
    { 0xfab0fb4f, SUC_lexicon_tags_12 },
    { 0xd1f0fb4f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1490fb52, SUC_lexicon_tags_60 },
    { 0xd3f0fb53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc590fb55, SUC_lexicon_tags_15 },
    { 0xbf50fb56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed0fb5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd0fb60, SUC_lexicon_tags_37 },
    { 0x79f0fb61, SUC_lexicon_tags_65 },
    { 0xdab0fb62, SUC_lexicon_tags_7 },
    { 0x8ad0fb63, SUC_lexicon_tags_38 },
    { 0xe630fb64, SUC_lexicon_tags_11 },
    { 0x4c90fb65, SUC_lexicon_tags_17 },
    { 0xc6f0fb64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f0fb6b, SUC_lexicon_tags_56 },
    { 0xd210fb6b, SUC_lexicon_tags_689 },
    { 0xd570fb6d, SUC_lexicon_tags_68 },
    { 0x6cb0fb6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fb71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e50fb74, SUC_lexicon_tags_12 },
    { 0x9f70fb75, SUC_lexicon_tags_15 },
    { 0x4810fb76, SUC_lexicon_tags_57 },
    { 0xa950fb77, SUC_lexicon_tags_1 },
    { 0x4090fb77, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x230fb7d, SUC_lexicon_tags_17 },
    { 0x38b0fb7d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa230fb81, SUC_lexicon_tags_68 },
    { 0x8d10fb82, SUC_lexicon_tags_37 },
    { 0xb950fb83, SUC_lexicon_tags_37 },
    { 0xaa50fb84, SUC_lexicon_tags_49 },
    { 0xa750fb83, SUC_lexicon_tags_13 },
    { 0x9370fb86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e30fb88, SUC_lexicon_tags_23 },
    { 0xbdd0fb89, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9df0fb8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d10fb91, SUC_lexicon_tags_15 },
    { 0xa210fb91, SUC_lexicon_tags_15 },
    { 0x23b0fb93, SUC_lexicon_tags_25 },
    { 0x5eb0fb91, SUC_lexicon_tags_77 },
    { 0xec70fb95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f0fb98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b0fb9c, SUC_lexicon_tags_31 },
    { 0x45f0fb9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x730fb9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3b0fba1, SUC_lexicon_tags_2 },
    { 0xd470fba2, SUC_lexicon_tags_14 },
    { 0x5150fba3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe270fba5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x470fbaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4090fbae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd690fbb4, SUC_lexicon_tags_33 },
    { 0x8270fbb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63f0fbb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc450fbba, SUC_lexicon_tags_37 },
    { 0x9df0fbbb, SUC_lexicon_tags_98 },
    { 0xad30fbbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c30fbc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b0fbc4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2550fbc9, SUC_lexicon_tags_28 },
    { 0x11f0fbca, SUC_lexicon_tags_66 },
    { 0xbbb0fbcb, SUC_lexicon_tags_54 },
    { 0xf850fbcc, SUC_lexicon_tags_19 },
    { 0x3950fbcd, SUC_lexicon_tags_10 },
    { 0x62f0fbce, SUC_lexicon_tags_15 },
    { 0x9ab0fbcd, SUC_lexicon_tags_26 },
    { 0x86d0fbca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9930fbd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5090fbd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf230fbd6, SUC_lexicon_tags_25 },
    { 0xa5b0fbd7, SUC_lexicon_tags_11 },
    { 0x530fbd8, SUC_lexicon_tags_37 },
    { 0xbad0fbd7, SUC_lexicon_tags_17 },
    { 0x10b0fbda, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad0fbdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a70fbe0, SUC_lexicon_tags_8 },
    { 0x54b0fbe1, SUC_lexicon_tags_7 },
    { 0x8350fbe2, SUC_lexicon_tags_17 },
    { 0x3970fbe3, SUC_lexicon_tags_8 },
    { 0x2830fbe4, SUC_lexicon_tags_25 },
    { 0xfd30fbe2, SUC_lexicon_tags_7 },
    { 0xfbf0fbe6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3190fbea, SUC_lexicon_tags_63 },
    { 0x4d30fbea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x42b0fbed, SUC_lexicon_tags_37 },
    { 0x4810fbee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4250fbf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef0fbf4, SUC_lexicon_tags_17 },
    { 0xa950fbf5, SUC_lexicon_tags_15 },
    { 0x48f0fbf5, SUC_lexicon_tags_11 },
    { 0x9850fbf7, SUC_lexicon_tags_17 },
    { 0x42b0fbf6, SUC_lexicon_tags_37 },
    { 0xa8b0fbf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf790fbfb, SUC_lexicon_tags_1 },
    { 0xc050fbfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a70fbff, SUC_lexicon_tags_17 },
    { 0x3470fbff, SUC_lexicon_tags_23 },
    { 0xaf70fc00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b10fc03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe150fc07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd910fc0c, SUC_lexicon_tags_23 },
    { 0x830fc0c, SUC_lexicon_tags_11 },
    { 0xf870fc0d, SUC_lexicon_tags_7 },
    { 0x77d0fc0c, SUC_lexicon_tags_7 },
    { 0x99f0fc10, SUC_lexicon_tags_45 },
    { 0xc4f0fc10, SUC_lexicon_tags_17 },
    { 0xee90fc12, SUC_lexicon_tags_45 },
    { 0xc470fc10, SUC_lexicon_tags_47 },
    { 0xbed0fc14, SUC_lexicon_tags_6 },
    { 0x1eb0fc14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb790fc1d, SUC_lexicon_tags_17 },
    { 0xb030fc1e, SUC_lexicon_tags_33 },
    { 0x8ad0fc1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f0fc26, SUC_lexicon_tags_2 },
    { 0xaf90fc27, SUC_lexicon_tags_33 },
    { 0x8330fc26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a10fc2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b0fc2f, SUC_lexicon_tags_2 },
    { 0x61f0fc30, SUC_lexicon_tags_33 },
    { 0xbd30fc30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d0fc35, SUC_lexicon_tags_14 },
    { 0x9390fc36, SUC_lexicon_tags_25 },
    { 0x5a50fc37, SUC_lexicon_tags_19 },
    { 0xd490fc36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa850fc3f, SUC_lexicon_tags_55 },
    { 0x69b0fc3f, SUC_lexicon_tags_26 },
    { 0xc310fc41, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6190fc4b, SUC_lexicon_tags_11 },
    { 0x84d0fc4b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb70fc50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b0fc57, SUC_lexicon_tags_37 },
    { 0x4f90fc57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f50fc5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd0fc62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4010fc6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb0fc72, SUC_lexicon_tags_12 },
    { 0xb6b0fc73, SUC_lexicon_tags_15 },
    { 0xca70fc73, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c10fc78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d0fc81, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d0fc88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa250fc8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf490fc8f, SUC_lexicon_tags_25 },
    { 0x53f0fc90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf150fc95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde30fc98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3410fc9a, SUC_lexicon_tags_7 },
    { 0x5f70fc9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b70fca1, SUC_lexicon_tags_37 },
    { 0xe950fca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3270fca4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f0fca8, SUC_lexicon_tags_37 },
    { 0x1370fca9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1010fcae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd370fcb0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xac50fcb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf50fcb7, SUC_lexicon_tags_43 },
    { 0x3c90fcb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b0fcbb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9170fcc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b0fcc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde10fcd4, SUC_lexicon_tags_0 },
    { 0x6c50fcd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e90fcd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9450fcdb, SUC_lexicon_tags_25 },
    { 0x9830fcdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6d0fcde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f0fce2, SUC_lexicon_tags_17 },
    { 0x17f0fce3, SUC_lexicon_tags_23 },
    { 0x50d0fce3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc10fce6, SUC_lexicon_tags_35 },
    { 0x5050fce7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc210fceb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7030fcef, SUC_lexicon_tags_23 },
    { 0xced0fcf0, SUC_lexicon_tags_19 },
    { 0x7ef0fcf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d0fcf7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f50fcfa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4270fd02, SUC_lexicon_tags_10 },
    { 0xbed0fd03, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1bf0fd05, SUC_lexicon_tags_12 },
    { 0xf370fd05, SUC_lexicon_tags_37 },
    { 0x82d0fd06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb850fd0a, SUC_lexicon_tags_12 },
    { 0x8770fd0b, SUC_lexicon_tags_11 },
    { 0x1150fd0c, SUC_lexicon_tags_11 },
    { 0xe7b0fd0c, SUC_lexicon_tags_729 },
    { 0x7650fd0e, SUC_lexicon_tags_2 },
    { 0x4970fd0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4270fd1b, SUC_lexicon_tags_15 },
    { 0xab50fd1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc010fd25, SUC_lexicon_tags_11 },
    { 0x2b90fd26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3930fd28, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4250fd2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd70fd36, SUC_lexicon_tags_55 },
    { 0xfa50fd37, SUC_lexicon_tags_11 },
    { 0x9850fd36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2570fd3a, SUC_lexicon_tags_2 },
    { 0xd1f0fd3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd0fd3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4630fd41, SUC_lexicon_tags_2 },
    { 0x8670fd42, SUC_lexicon_tags_28 },
    { 0x1f0fd42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1730fd48, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff30fd4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x370fd4e, SUC_lexicon_tags_7 },
    { 0x6d0fd4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f10fd51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d0fd58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x84b0fd5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe730fd5d, SUC_lexicon_tags_14 },
    { 0xe350fd5e, SUC_lexicon_tags_47 },
    { 0xbc30fd5e, SUC_lexicon_tags_17 },
    { 0x45d0fd5d, SUC_lexicon_tags_153 },
    { 0x7570fd5f, SUC_lexicon_tags_17 },
    { 0x4d90fd62, SUC_lexicon_tags_4 },
    { 0x6670fd5d, SUC_lexicon_tags_33 },
    { 0xe890fd64, SUC_lexicon_tags_15 },
    { 0xa3b0fd5d, SUC_lexicon_tags_25 },
    { 0xa4f0fd66, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9190fd6d, SUC_lexicon_tags_6 },
    { 0x2670fd6d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc370fd77, SUC_lexicon_tags_15 },
    { 0xd1b0fd78, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5330fd7b, SUC_lexicon_tags_37 },
    { 0xdf0fd7c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7250fd81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcd0fd83, SUC_lexicon_tags_17 },
    { 0x4370fd84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc90fd86, SUC_lexicon_tags_43 },
    { 0x6d30fd87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf190fd90, SUC_lexicon_tags_11 },
    { 0x9f10fd91, SUC_lexicon_tags_13 },
    { 0xf10fd92, SUC_lexicon_tags_19 },
    { 0x10b0fd93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5150fd96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f70fd98, SUC_lexicon_tags_17 },
    { 0xb670fd99, SUC_lexicon_tags_75 },
    { 0xf5f0fd9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3530fd9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7590fda1, SUC_lexicon_tags_38 },
    { 0xa250fda2, SUC_lexicon_tags_7 },
    { 0xb430fda3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f0fda6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2210fdab, SUC_lexicon_tags_37 },
    { 0x2f0fdac, SUC_lexicon_tags_21 },
    { 0xb4d0fdac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa90fdb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b0fdb4, SUC_lexicon_tags_12 },
    { 0x4dd0fdb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f10fdb9, SUC_lexicon_tags_17 },
    { 0xe110fdba, SUC_lexicon_tags_29 },
    { 0xb950fdbb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2890fdbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x650fdc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc350fdc9, SUC_lexicon_tags_60 },
    { 0x7590fdc9, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xdfb0fdcc, SUC_lexicon_tags_37 },
    { 0x59d0fdcd, SUC_lexicon_tags_33 },
    { 0x8130fdce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc30fdd2, SUC_lexicon_tags_37 },
    { 0x83d0fdd2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db0fdd6, SUC_lexicon_tags_17 },
    { 0x14d0fdd7, SUC_lexicon_tags_37 },
    { 0x5a70fdd8, SUC_lexicon_tags_45 },
    { 0x1210fdd8, SUC_lexicon_tags_68 },
    { 0x3b30fdd6, SUC_lexicon_tags_8 },
    { 0xdaf0fddb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19f0fddd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1390fde0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf750fde6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5770fde8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e70fdec, SUC_lexicon_tags_107 },
    { 0x10d0fded, SUC_lexicon_tags_158 },
    { 0xe870fdee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d0fdf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d0fdfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d90fdfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e70fe00, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1590fe06, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df0fe0a, SUC_lexicon_tags_12 },
    { 0x4f0fe0b, SUC_lexicon_tags_14 },
    { 0x20f0fe0c, SUC_lexicon_tags_0 },
    { 0xf1b0fe0d, SUC_lexicon_tags_2 },
    { 0x7870fe0d, SUC_lexicon_tags_12 },
    { 0xbc90fe0f, SUC_lexicon_tags_6 },
    { 0xe710fe0c, SUC_lexicon_tags_0 },
    { 0x37d0fe11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f0fe17, SUC_lexicon_tags_39 },
    { 0x21b0fe17, SUC_lexicon_tags_19 },
    { 0xe750fe17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb70fe1b, SUC_lexicon_tags_65 },
    { 0x27b0fe1b, SUC_lexicon_tags_3 },
    { 0x9650fe1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd70fe1f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b0fe22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc650fe2c, SUC_lexicon_tags_37 },
    { 0xaed0fe2d, SUC_lexicon_tags_14 },
    { 0x9b70fe2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9bb0fe30, SUC_lexicon_tags_17 },
    { 0x3970fe31, SUC_lexicon_tags_27 },
    { 0xba50fe30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d50fe3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1070fe42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2190fe47, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d0fe4d, SUC_lexicon_tags_63 },
    { 0x57b0fe4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc10fe50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea90fe56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b10fe5a, SUC_lexicon_tags_29 },
    { 0x7750fe5b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc930fe5e, SUC_lexicon_tags_25 },
    { 0x1cf0fe5f, SUC_lexicon_tags_17 },
    { 0xc5b0fe5e, SUC_lexicon_tags_45 },
    { 0x7090fe61, SUC_lexicon_tags_0 },
    { 0x9b0fe60, SUC_lexicon_tags_39 },
    { 0xa8b0fe63, SUC_lexicon_tags_43 },
    { 0x2150fe60, SUC_lexicon_tags_9 },
    { 0x7790fe61, SUC_lexicon_tags_49 },
    { 0x9b0fe62, SUC_lexicon_tags_100 },
    { 0x7550fe67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x88b0fe69, SUC_lexicon_tags_23 },
    { 0x18f0fe6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1250fe75, SUC_lexicon_tags_2 },
    { 0x2770fe76, SUC_lexicon_tags_69 },
    { 0xb2d0fe75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2710fe7a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc70fe80, SUC_lexicon_tags_47 },
    { 0x27d0fe80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9750fe83, SUC_lexicon_tags_605 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7930fe8a, SUC_lexicon_tags_7 },
    { 0xcd10fe8b, SUC_lexicon_tags_7 },
    { 0x2350fe8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5450fe90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b0fe94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb230fe98, SUC_lexicon_tags_12 },
    { 0xb070fe98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x59b0fe9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90fe9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64f0fea1, SUC_lexicon_tags_25 },
    { 0xc6f0fea2, SUC_lexicon_tags_69 },
    { 0x1730fea2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf410fea5, SUC_lexicon_tags_23 },
    { 0xb670fea6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x63f0fea8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f0feac, SUC_lexicon_tags_14 },
    { 0x5e50feac, SUC_lexicon_tags_7 },
    { 0x6db0feae, SUC_lexicon_tags_7 },
    { 0x34b0feaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9350feb2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2850feb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b70feb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6130feb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc530febb, SUC_lexicon_tags_17 },
    { 0x21b0febc, SUC_lexicon_tags_12 },
    { 0x7790febd, SUC_lexicon_tags_9 },
    { 0xba90febe, SUC_lexicon_tags_87 },
    { 0xac10febe, SUC_lexicon_tags_7 },
    { 0x990febe, SUC_lexicon_tags_49 },
    { 0x9150febd, SUC_lexicon_tags_7 },
    { 0x250febf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d0fec6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8290fecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d0fed0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7b50fed2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd610fed5, SUC_lexicon_tags_23 },
    { 0x9530fed6, SUC_lexicon_tags_33 },
    { 0xa0f0fed7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb170feda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec90fee2, SUC_lexicon_tags_9 },
    { 0x7a90fee3, SUC_lexicon_tags_3 },
    { 0xce90fee2, SUC_lexicon_tags_2 },
    { 0x7a90fee5, SUC_lexicon_tags_1 },
    { 0x8d50fee3, SUC_lexicon_tags_68 },
    { 0x4e70fee7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xce10fee9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9610feef, SUC_lexicon_tags_37 },
    { 0xe350feef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5670fef4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6890fef9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e50ff01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d50ff04, SUC_lexicon_tags_7 },
    { 0xd090ff05, SUC_lexicon_tags_1 },
    { 0x8c30ff06, SUC_lexicon_tags_12 },
    { 0xfcb0ff04, SUC_lexicon_tags_57 },
    { 0xf2b0ff04, SUC_lexicon_tags_49 },
    { 0x98b0ff09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x650ff0c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb1f0ff0e, SUC_lexicon_tags_1377 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb30ff12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a70ff15, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa290ff1c, SUC_lexicon_tags_17 },
    { 0x2550ff1c, SUC_lexicon_tags_47 },
    { 0x630ff1e, SUC_lexicon_tags_37 },
    { 0x2c70ff1f, SUC_lexicon_tags_10 },
    { 0x6d0ff20, SUC_lexicon_tags_49 },
    { 0xdff0ff1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f0ff27, SUC_lexicon_tags_48 },
    { 0xbd10ff28, SUC_lexicon_tags_2 },
    { 0x6330ff29, SUC_lexicon_tags_38 },
    { 0x7370ff28, SUC_lexicon_tags_3 },
    { 0x7410ff2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b0ff32, SUC_lexicon_tags_7 },
    { 0xa5b0ff33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x330ff35, SUC_lexicon_tags_17 },
    { 0xac30ff35, SUC_lexicon_tags_22 },
    { 0x13b0ff37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba70ff44, SUC_lexicon_tags_26 },
    { 0x6510ff45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f0ff48, SUC_lexicon_tags_84 },
    { 0x5050ff49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a70ff4b, SUC_lexicon_tags_55 },
    { 0x5170ff4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df0ff4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x53d0ff51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeab0ff53, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8710ff57, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa530ff5c, SUC_lexicon_tags_12 },
    { 0x31d0ff5d, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d0ff60, SUC_lexicon_tags_47 },
    { 0x830ff61, SUC_lexicon_tags_19 },
    { 0xeb90ff61, SUC_lexicon_tags_60 },
    { 0xd7d0ff62, SUC_lexicon_tags_12 },
    { 0xc7b0ff64, SUC_lexicon_tags_25 },
    { 0xf990ff61, SUC_lexicon_tags_14 },
    { 0x3cb0ff61, SUC_lexicon_tags_15 },
    { 0x5990ff67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d0ff6d, SUC_lexicon_tags_60 },
    { 0xdab0ff6e, SUC_lexicon_tags_9 },
    { 0x2e50ff6f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe870ff73, SUC_lexicon_tags_17 },
    { 0xb230ff74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x550ff77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b30ff7a, SUC_lexicon_tags_10 },
    { 0x1870ff7b, SUC_lexicon_tags_235 },
    { 0x7690ff7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9050ff7e, SUC_lexicon_tags_12 },
    { 0x63b0ff7f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf510ff85, SUC_lexicon_tags_25 },
    { 0x6d70ff85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa170ff8a, SUC_lexicon_tags_66 },
    { 0x2510ff8b, SUC_lexicon_tags_49 },
    { 0xf110ff8b, SUC_lexicon_tags_16 },
    { 0x7f70ff8d, SUC_lexicon_tags_25 },
    { 0xb8b0ff8e, SUC_lexicon_tags_6 },
    { 0x750ff8e, SUC_lexicon_tags_43 },
    { 0xd270ff90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3dd0ff92, SUC_lexicon_tags_82 },
    { 0xfd50ff92, SUC_lexicon_tags_7 },
    { 0x7d30ff93, SUC_lexicon_tags_11 },
    { 0x7b0ff95, SUC_lexicon_tags_37 },
    { 0xccb0ff96, SUC_lexicon_tags_0 },
    { 0xacb0ff96, SUC_lexicon_tags_14 },
    { 0x2550ff96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db0ff9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb50ffa1, SUC_lexicon_tags_15 },
    { 0xdc50ffa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f0ffa7, SUC_lexicon_tags_63 },
    { 0x6670ffa8, SUC_lexicon_tags_31 },
    { 0xe030ffa9, SUC_lexicon_tags_11 },
    { 0xf1b0ffaa, SUC_lexicon_tags_12 },
    { 0xe110ffaa, SUC_lexicon_tags_17 },
    { 0x80d0ffac, SUC_lexicon_tags_17 },
    { 0x5210ffad, SUC_lexicon_tags_25 },
    { 0xf330ffae, SUC_lexicon_tags_17 },
    { 0xe270ffae, SUC_lexicon_tags_12 },
    { 0xca90ffb0, SUC_lexicon_tags_65 },
    { 0xef0ffb0, SUC_lexicon_tags_30 },
    { 0x4930ffad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9870ffb5, SUC_lexicon_tags_16 },
    { 0x15b0ffb6, SUC_lexicon_tags_11 },
    { 0x1730ffb7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8ad0ffb9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd450ffc2, SUC_lexicon_tags_106 },
    { 0x40f0ffc3, SUC_lexicon_tags_68 },
    { 0x3250ffc4, SUC_lexicon_tags_25 },
    { 0x8570ffc4, SUC_lexicon_tags_48 },
    { 0x4950ffc6, SUC_lexicon_tags_16 },
    { 0x7af0ffc7, SUC_lexicon_tags_25 },
    { 0xaad0ffc6, SUC_lexicon_tags_2 },
    { 0xf830ffc9, SUC_lexicon_tags_25 },
    { 0x75d0ffca, SUC_lexicon_tags_17 },
    { 0xff90ffca, SUC_lexicon_tags_6 },
    { 0x7d10ffca, SUC_lexicon_tags_56 },
    { 0x5010ffc8, SUC_lexicon_tags_4 },
    { 0x1170ffce, SUC_lexicon_tags_7 },
    { 0xf650ffcf, SUC_lexicon_tags_17 },
    { 0x8a30ffd0, SUC_lexicon_tags_38 },
    { 0x3b50ffcd, SUC_lexicon_tags_21 },
    { 0x3750ffc8, SUC_lexicon_tags_17 },
    { 0xa570ffd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4050ffd8, SUC_lexicon_tags_47 },
    { 0xfa50ffd9, SUC_lexicon_tags_38 },
    { 0xf30ffda, SUC_lexicon_tags_38 },
    { 0x9d50ffda, SUC_lexicon_tags_2 },
    { 0xcb50ffdc, SUC_lexicon_tags_3 },
    { 0x3930ffdd, SUC_lexicon_tags_2 },
    { 0x6f10ffdc, SUC_lexicon_tags_61 },
    { 0x5110ffdc, SUC_lexicon_tags_2 },
    { 0xa4f0ffe0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd10ffe9, SUC_lexicon_tags_45 },
    { 0xd2f0ffe9, SUC_lexicon_tags_148 },
    { 0x62b0ffea, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab0ffee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2390fff2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ef0fff4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf0fff7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8410fff9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x64b0fffb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb3d0fffd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc510005, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72b10007, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d1000a, SUC_lexicon_tags_46 },
    { 0x72f1000a, SUC_lexicon_tags_77 },
    { 0xfcd1000c, SUC_lexicon_tags_1 },
    { 0x5a71000d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98f1000f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x64310012, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7710016, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d1001d, SUC_lexicon_tags_0 },
    { 0xd091001e, SUC_lexicon_tags_23 },
    { 0xc0f1001d, SUC_lexicon_tags_26 },
    { 0x58910020, SUC_lexicon_tags_25 },
    { 0xdad10020, SUC_lexicon_tags_55 },
    { 0x1cd10021, SUC_lexicon_tags_45 },
    { 0x95910023, SUC_lexicon_tags_7 },
    { 0xd2110024, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xca110027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f51002a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe531002f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b10032, SUC_lexicon_tags_8 },
    { 0xa4110033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36d10035, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc510037, SUC_lexicon_tags_47 },
    { 0x73510038, SUC_lexicon_tags_37 },
    { 0xa6910039, SUC_lexicon_tags_47 },
    { 0x29710038, SUC_lexicon_tags_43 },
    { 0x7cb1003b, SUC_lexicon_tags_47 },
    { 0xad51003c, SUC_lexicon_tags_2 },
    { 0x8dd1003c, SUC_lexicon_tags_46 },
    { 0x85d1003e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x68d10040, SUC_lexicon_tags_65 },
    { 0xac510041, SUC_lexicon_tags_45 },
    { 0x6f110042, SUC_lexicon_tags_39 },
    { 0xcb310043, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13910047, SUC_lexicon_tags_15 },
    { 0xddd10048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db1004b, SUC_lexicon_tags_49 },
    { 0x5a91004b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d1004f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd10055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28310057, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d1005c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3110060, SUC_lexicon_tags_37 },
    { 0xee310061, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f10069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f1006e, SUC_lexicon_tags_0 },
    { 0x6ed1006f, SUC_lexicon_tags_37 },
    { 0xdcb1006e, SUC_lexicon_tags_25 },
    { 0x38910071, SUC_lexicon_tags_25 },
    { 0xffd1006f, SUC_lexicon_tags_11 },
    { 0x94910073, SUC_lexicon_tags_27 },
    { 0xa5d10074, SUC_lexicon_tags_17 },
    { 0x6a910074, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89310079, SUC_lexicon_tags_25 },
    { 0x6bd1007a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc51007d, SUC_lexicon_tags_45 },
    { 0x6051007e, SUC_lexicon_tags_25 },
    { 0xcd51007d, SUC_lexicon_tags_25 },
    { 0x91f10080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a110086, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e71008b, SUC_lexicon_tags_47 },
    { 0x9b11008c, SUC_lexicon_tags_49 },
    { 0x8371008d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb10090, SUC_lexicon_tags_12 },
    { 0xae510090, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65110098, SUC_lexicon_tags_17 },
    { 0x31110099, SUC_lexicon_tags_148 },
    { 0xa6f1009a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483100a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x669100a3, SUC_lexicon_tags_11 },
    { 0xe7d100a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x103100a7, SUC_lexicon_tags_47 },
    { 0xc7100a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x625100ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa53100ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23100b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd53100b3, SUC_lexicon_tags_7 },
    { 0xaf5100b3, SUC_lexicon_tags_38 },
    { 0x611100b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505100bb, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35100c3, SUC_lexicon_tags_11 },
    { 0x801100c4, SUC_lexicon_tags_15 },
    { 0xdbf100c5, SUC_lexicon_tags_37 },
    { 0xe9100c6, SUC_lexicon_tags_1 },
    { 0x6e7100c7, SUC_lexicon_tags_14 },
    { 0x519100c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb100cc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d100cf, SUC_lexicon_tags_47 },
    { 0x583100d0, SUC_lexicon_tags_26 },
    { 0x723100d1, SUC_lexicon_tags_47 },
    { 0x75b100d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9100d5, SUC_lexicon_tags_14 },
    { 0xe17100d5, SUC_lexicon_tags_23 },
    { 0xeeb100d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5f5100d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd100dc, SUC_lexicon_tags_37 },
    { 0x6f1100dd, SUC_lexicon_tags_1 },
    { 0x3ad100dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdb100e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x45100e2, SUC_lexicon_tags_57 },
    { 0xa37100e3, SUC_lexicon_tags_0 },
    { 0xcb7100e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945100e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f100ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9100f0, SUC_lexicon_tags_4 },
    { 0xc0b100f1, SUC_lexicon_tags_25 },
    { 0x453100f1, SUC_lexicon_tags_58 },
    { 0xc97100f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d7100f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x633100f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7100fa, SUC_lexicon_tags_25 },
    { 0x3c9100fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7100fe, SUC_lexicon_tags_6 },
    { 0x2bb100fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c310102, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68710108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb1010b, SUC_lexicon_tags_61 },
    { 0xaed1010c, SUC_lexicon_tags_7 },
    { 0x991010d, SUC_lexicon_tags_15 },
    { 0x29b1010d, SUC_lexicon_tags_17 },
    { 0xe911010f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x36710112, SUC_lexicon_tags_48 },
    { 0xd5b10113, SUC_lexicon_tags_12 },
    { 0x31d10113, SUC_lexicon_tags_15 },
    { 0xd7310114, SUC_lexicon_tags_2 },
    { 0x39b10114, SUC_lexicon_tags_11 },
    { 0x53110114, SUC_lexicon_tags_10 },
    { 0x43510118, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f11011a, SUC_lexicon_tags_7 },
    { 0xe0f1011b, SUC_lexicon_tags_74 },
    { 0xaeb1011b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe171011e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf910121, SUC_lexicon_tags_43 },
    { 0x2bd10122, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6310124, SUC_lexicon_tags_12 },
    { 0xf2710125, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x86d10127, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x11012a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6231012c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d1012f, SUC_lexicon_tags_9 },
    { 0xd0d10130, SUC_lexicon_tags_17 },
    { 0x37f10131, SUC_lexicon_tags_28 },
    { 0x54310132, SUC_lexicon_tags_25 },
    { 0x5b110132, SUC_lexicon_tags_37 },
    { 0xe4110130, SUC_lexicon_tags_8 },
    { 0x67d10135, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xffd10137, SUC_lexicon_tags_101 },
    { 0xeb110138, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cf1013a, SUC_lexicon_tags_2 },
    { 0x4fd1013a, SUC_lexicon_tags_31 },
    { 0x7f11013c, SUC_lexicon_tags_11 },
    { 0xc111013d, SUC_lexicon_tags_17 },
    { 0x7351013d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf10143, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b510147, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f1014a, SUC_lexicon_tags_37 },
    { 0xed91014a, SUC_lexicon_tags_19 },
    { 0x751014c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7d1014e, SUC_lexicon_tags_38 },
    { 0x2db1014e, SUC_lexicon_tags_8 },
    { 0xc3d1014f, SUC_lexicon_tags_37 },
    { 0x8c11014e, SUC_lexicon_tags_29 },
    { 0x4b710152, SUC_lexicon_tags_13 },
    { 0x13b10153, SUC_lexicon_tags_7 },
    { 0x12910154, SUC_lexicon_tags_56 },
    { 0x7fb1014f, SUC_lexicon_tags_19 },
    { 0xdf110156, SUC_lexicon_tags_9 },
    { 0xacb1014e, SUC_lexicon_tags_14 },
    { 0x99f10158, SUC_lexicon_tags_12 },
    { 0xe5910159, SUC_lexicon_tags_82 },
    { 0xb8d10159, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c91015d, SUC_lexicon_tags_12 },
    { 0x3ff1015d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b10161, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8510166, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x82310168, SUC_lexicon_tags_37 },
    { 0xf1f10169, SUC_lexicon_tags_2 },
    { 0x99f1016a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1371016c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d710171, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6c110173, SUC_lexicon_tags_1 },
    { 0xe7510174, SUC_lexicon_tags_1 },
    { 0x49f10174, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25f10177, SUC_lexicon_tags_37 },
    { 0xb7510178, SUC_lexicon_tags_17 },
    { 0xf7b10179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeff1017b, SUC_lexicon_tags_17 },
    { 0xc291017c, SUC_lexicon_tags_49 },
    { 0xf171017d, SUC_lexicon_tags_33 },
    { 0x80d1017c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7110181, SUC_lexicon_tags_40 },
    { 0xa2510182, SUC_lexicon_tags_32 },
    { 0x46710181, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf10188, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9651018b, SUC_lexicon_tags_91 },
    { 0x431018c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a910190, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88910196, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf1019a, SUC_lexicon_tags_49 },
    { 0x2a11019b, SUC_lexicon_tags_1 },
    { 0x1291019b, SUC_lexicon_tags_17 },
    { 0x9811019c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x485101a0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7101a4, SUC_lexicon_tags_1 },
    { 0x549101a5, SUC_lexicon_tags_11 },
    { 0x5d1101a6, SUC_lexicon_tags_12 },
    { 0xab101a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3101ac, SUC_lexicon_tags_2 },
    { 0x929101ad, SUC_lexicon_tags_17 },
    { 0x113101ad, SUC_lexicon_tags_27 },
    { 0x8bf101af, SUC_lexicon_tags_2 },
    { 0xda1101b0, SUC_lexicon_tags_91 },
    { 0x455101b1, SUC_lexicon_tags_65 },
    { 0x743101b2, SUC_lexicon_tags_25 },
    { 0xf9d101b2, SUC_lexicon_tags_15 },
    { 0x865101b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1101b9, SUC_lexicon_tags_25 },
    { 0x50d101ba, SUC_lexicon_tags_16 },
    { 0x8a5101bb, SUC_lexicon_tags_25 },
    { 0x2ef101bc, SUC_lexicon_tags_7 },
    { 0x839101bd, SUC_lexicon_tags_11 },
    { 0xc83101bd, SUC_lexicon_tags_0 },
    { 0x919101bf, SUC_lexicon_tags_9 },
    { 0xd83101bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921101c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647101cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7101d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343101d4, SUC_lexicon_tags_15 },
    { 0x1a5101d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf101de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc3101e0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x77f101e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d101e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52b101e8, SUC_lexicon_tags_32 },
    { 0x5fb101e9, SUC_lexicon_tags_16 },
    { 0x225101e9, SUC_lexicon_tags_4 },
    { 0x49b101e8, SUC_lexicon_tags_207 },
    { 0xb01101ec, SUC_lexicon_tags_15 },
    { 0xd51101ed, SUC_lexicon_tags_6 },
    { 0x533101ee, SUC_lexicon_tags_25 },
    { 0xdf3101ec, SUC_lexicon_tags_79 },
    { 0x16f101ee, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9101f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc4d101fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x935101fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x907101ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d10207, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9231020d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xda11020f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d10213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf91021b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a71021d, SUC_lexicon_tags_11 },
    { 0x5771021d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf10221, SUC_lexicon_tags_37 },
    { 0x65d10222, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea310224, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd851022b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x131022d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b10231, SUC_lexicon_tags_35 },
    { 0x40710232, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c310236, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f1023b, SUC_lexicon_tags_64 },
    { 0xb4f1023b, SUC_lexicon_tags_15 },
    { 0x5811023b, SUC_lexicon_tags_14 },
    { 0x4e51023c, SUC_lexicon_tags_11 },
    { 0x51b1023f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e710242, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd10247, SUC_lexicon_tags_17 },
    { 0x53710248, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f71024b, SUC_lexicon_tags_11 },
    { 0x84b1024c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe711024f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10910252, SUC_lexicon_tags_38 },
    { 0x5a110253, SUC_lexicon_tags_47 },
    { 0x45710253, SUC_lexicon_tags_15 },
    { 0x96d10252, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82f10257, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe51025d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3910263, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf451026c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa851026e, SUC_lexicon_tags_75 },
    { 0xae31026f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b10272, SUC_lexicon_tags_12 },
    { 0xf8710273, SUC_lexicon_tags_1 },
    { 0x1df10274, SUC_lexicon_tags_1 },
    { 0xfb510275, SUC_lexicon_tags_15 },
    { 0x25910276, SUC_lexicon_tags_31 },
    { 0xc710276, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3351027d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e510285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d10287, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0d10289, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f1028d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf10292, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9910294, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc710298, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1711029b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xadd1029d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71102a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b102a5, SUC_lexicon_tags_11 },
    { 0x3c7102a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807102ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd1102ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbb5102af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9102b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb102b9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5102bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d102c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d102cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b102d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17d102d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa65102d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x871102db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x921102dd, SUC_lexicon_tags_11 },
    { 0x7cf102de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f9102e0, SUC_lexicon_tags_31 },
    { 0xf3f102e1, SUC_lexicon_tags_25 },
    { 0x1bd102e2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91102e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x199102e8, SUC_lexicon_tags_38 },
    { 0x48b102e8, SUC_lexicon_tags_30 },
    { 0xa51102ea, SUC_lexicon_tags_7 },
    { 0x393102eb, SUC_lexicon_tags_38 },
    { 0xfa3102ec, SUC_lexicon_tags_37 },
    { 0x257102ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x437102ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b102f7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275102fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9102fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4bb10300, SUC_lexicon_tags_15 },
    { 0x9bd10301, SUC_lexicon_tags_17 },
    { 0x52d10302, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a51030b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1030e, SUC_lexicon_tags_63 },
    { 0x5f11030f, SUC_lexicon_tags_32 },
    { 0x5ad10310, SUC_lexicon_tags_55 },
    { 0x2f910311, SUC_lexicon_tags_56 },
    { 0xd7b10311, SUC_lexicon_tags_37 },
    { 0xe3b10313, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21710317, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f10321, SUC_lexicon_tags_17 },
    { 0xc9510322, SUC_lexicon_tags_38 },
    { 0x6ab10322, SUC_lexicon_tags_11 },
    { 0x4c310324, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d310327, SUC_lexicon_tags_25 },
    { 0x26f10328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab1032c, SUC_lexicon_tags_0 },
    { 0xbd71032c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d10331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47310334, SUC_lexicon_tags_37 },
    { 0x15f10334, SUC_lexicon_tags_29 },
    { 0xd1110334, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb1033a, SUC_lexicon_tags_12 },
    { 0xf171033b, SUC_lexicon_tags_43 },
    { 0xd7f1033b, SUC_lexicon_tags_298 },
    { 0xb2b1033d, SUC_lexicon_tags_25 },
    { 0xf4d1033b, SUC_lexicon_tags_25 },
    { 0x12f1033f, SUC_lexicon_tags_37 },
    { 0x6a110340, SUC_lexicon_tags_12 },
    { 0x17b10341, SUC_lexicon_tags_2 },
    { 0xa5b1033f, SUC_lexicon_tags_25 },
    { 0x83d10343, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee110346, SUC_lexicon_tags_62 },
    { 0x2ab10347, SUC_lexicon_tags_60 },
    { 0xd8b10346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf1034b, SUC_lexicon_tags_0 },
    { 0x6ab1034c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb1034f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b10352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68710354, SUC_lexicon_tags_37 },
    { 0x14910355, SUC_lexicon_tags_4 },
    { 0x8fd10356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16310358, SUC_lexicon_tags_289 },
    { 0x9710359, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd1035c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb310361, SUC_lexicon_tags_17 },
    { 0x3f710362, SUC_lexicon_tags_63 },
    { 0x40b10362, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74d10365, SUC_lexicon_tags_31 },
    { 0x75710366, SUC_lexicon_tags_27 },
    { 0xcf310366, SUC_lexicon_tags_32 },
    { 0xa7710365, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef1036b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7751036e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e110373, SUC_lexicon_tags_27 },
    { 0x15910374, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a51037d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb110381, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x21f10383, SUC_lexicon_tags_12 },
    { 0x76910383, SUC_lexicon_tags_37 },
    { 0xb5110385, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351038a, SUC_lexicon_tags_37 },
    { 0x7411038a, SUC_lexicon_tags_11 },
    { 0xacf1038c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b10390, SUC_lexicon_tags_37 },
    { 0xe6510391, SUC_lexicon_tags_0 },
    { 0x66310390, SUC_lexicon_tags_351 },
    { 0x85710391, SUC_lexicon_tags_27 },
    { 0x3db10394, SUC_lexicon_tags_29 },
    { 0x79910395, SUC_lexicon_tags_27 },
    { 0x92110396, SUC_lexicon_tags_0 },
    { 0x61310397, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51039b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe531039f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5103a2, SUC_lexicon_tags_2 },
    { 0x5ff103a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f103a8, SUC_lexicon_tags_1 },
    { 0x5a9103a9, SUC_lexicon_tags_17 },
    { 0xa23103aa, SUC_lexicon_tags_1 },
    { 0x761103a9, SUC_lexicon_tags_6 },
    { 0xe1f103a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5103af, SUC_lexicon_tags_38 },
    { 0x709103b0, SUC_lexicon_tags_43 },
    { 0x753103b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23103b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9103ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d103bd, SUC_lexicon_tags_15 },
    { 0xb55103bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd103c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f103c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d103d3, SUC_lexicon_tags_15 },
    { 0x349103d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef103da, SUC_lexicon_tags_69 },
    { 0xd31103db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdf1103dd, SUC_lexicon_tags_37 },
    { 0x1ab103dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157103e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba5103e5, SUC_lexicon_tags_37 },
    { 0x1ff103e6, SUC_lexicon_tags_30 },
    { 0xcf103e6, SUC_lexicon_tags_47 },
    { 0xd0f103e8, SUC_lexicon_tags_19 },
    { 0x24d103e8, SUC_lexicon_tags_21 },
    { 0x2b9103ea, SUC_lexicon_tags_4 },
    { 0x4d7103eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1103ee, SUC_lexicon_tags_63 },
    { 0x59b103ee, SUC_lexicon_tags_17 },
    { 0x489103f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af103f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53103f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967103fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x949103fc, SUC_lexicon_tags_23 },
    { 0xb57103fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb310400, SUC_lexicon_tags_6 },
    { 0xf9310401, SUC_lexicon_tags_37 },
    { 0xa910401, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f10406, SUC_lexicon_tags_30 },
    { 0x1af10406, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1a910409, SUC_lexicon_tags_26 },
    { 0xb110409, SUC_lexicon_tags_2 },
    { 0xa631040a, SUC_lexicon_tags_25 },
    { 0xca71040c, SUC_lexicon_tags_11 },
    { 0xfff1040c, SUC_lexicon_tags_37 },
    { 0x6511040e, SUC_lexicon_tags_25 },
    { 0x2f91040f, SUC_lexicon_tags_37 },
    { 0xdd910410, SUC_lexicon_tags_11 },
    { 0x50d10411, SUC_lexicon_tags_21 },
    { 0x21b1040c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa310416, SUC_lexicon_tags_11 },
    { 0x27b10417, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f1041b, SUC_lexicon_tags_25 },
    { 0x5c11041b, SUC_lexicon_tags_18 },
    { 0x3cf1041d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5f1041f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78710423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3310425, SUC_lexicon_tags_63 },
    { 0x40510425, SUC_lexicon_tags_25 },
    { 0x2e510426, SUC_lexicon_tags_12 },
    { 0xadf10428, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xef91042b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd10433, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1f10435, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1043f, SUC_lexicon_tags_26 },
    { 0xc671043f, SUC_lexicon_tags_15 },
    { 0xed510441, SUC_lexicon_tags_37 },
    { 0x7b710442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80910444, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37310448, SUC_lexicon_tags_15 },
    { 0xca710448, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x98f1044b, SUC_lexicon_tags_21 },
    { 0x3b51044c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3431044e, SUC_lexicon_tags_17 },
    { 0xd0b1044f, SUC_lexicon_tags_87 },
    { 0xdbf10450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64d10452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f10456, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd391045d, SUC_lexicon_tags_37 },
    { 0xf3b1045e, SUC_lexicon_tags_1 },
    { 0xb7f1045f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2b10461, SUC_lexicon_tags_95 },
    { 0xddf10462, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x5cf10464, SUC_lexicon_tags_45 },
    { 0xe2b10465, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb51046a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4491046d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d10472, SUC_lexicon_tags_41 },
    { 0x9cd10473, SUC_lexicon_tags_27 },
    { 0x2e510474, SUC_lexicon_tags_15 },
    { 0xaab10474, SUC_lexicon_tags_9 },
    { 0xe4910476, SUC_lexicon_tags_63 },
    { 0xcf910474, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32d10479, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e31047c, SUC_lexicon_tags_15 },
    { 0xa2f1047d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc051047f, SUC_lexicon_tags_14 },
    { 0x9351047f, SUC_lexicon_tags_661 },
    { 0xc8910481, SUC_lexicon_tags_29 },
    { 0xb5710482, SUC_lexicon_tags_37 },
    { 0xb3310483, SUC_lexicon_tags_7 },
    { 0xf4f10482, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d10487, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9710489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b51048b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad1048d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25110490, SUC_lexicon_tags_15 },
    { 0xa1510490, SUC_lexicon_tags_10 },
    { 0x8110492, SUC_lexicon_tags_12 },
    { 0xc2710491, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e310495, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62710497, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b71049c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a91049f, SUC_lexicon_tags_15 },
    { 0x2c9104a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9104a4, SUC_lexicon_tags_9 },
    { 0x6af104a5, SUC_lexicon_tags_38 },
    { 0x9d3104a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f104aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe59104ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x129104af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef104b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb89104b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b104b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897104bd, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xa57104bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825104c5, SUC_lexicon_tags_37 },
    { 0x76d104c6, SUC_lexicon_tags_60 },
    { 0x89f104c6, SUC_lexicon_tags_3 },
    { 0x45b104c7, SUC_lexicon_tags_63 },
    { 0x1cf104c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x741104cc, SUC_lexicon_tags_25 },
    { 0x72b104cd, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xcbd104cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x425104d1, SUC_lexicon_tags_7 },
    { 0xbf7104d2, SUC_lexicon_tags_11 },
    { 0x5bf104d3, SUC_lexicon_tags_2 },
    { 0xfeb104d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x293104d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9104db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x253104dd, SUC_lexicon_tags_37 },
    { 0x97b104de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a5104e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35104e3, SUC_lexicon_tags_1 },
    { 0x4bf104e3, SUC_lexicon_tags_2 },
    { 0xb15104e4, SUC_lexicon_tags_1 },
    { 0xe81104e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5104ea, SUC_lexicon_tags_1 },
    { 0x2d5104ea, SUC_lexicon_tags_17 },
    { 0x511104eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7a7104ee, SUC_lexicon_tags_11 },
    { 0xe1b104ee, SUC_lexicon_tags_37 },
    { 0x653104f0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf104f3, SUC_lexicon_tags_19 },
    { 0x661104f4, SUC_lexicon_tags_33 },
    { 0x9f3104f5, SUC_lexicon_tags_26 },
    { 0x8c9104f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x509104f8, SUC_lexicon_tags_15 },
    { 0x211104f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d104fc, SUC_lexicon_tags_25 },
    { 0x229104fd, SUC_lexicon_tags_2 },
    { 0xc87104fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x81910501, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad10504, SUC_lexicon_tags_106 },
    { 0x11f10505, SUC_lexicon_tags_17 },
    { 0xa3710506, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xea910509, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4171050b, SUC_lexicon_tags_48 },
    { 0xf131050c, SUC_lexicon_tags_55 },
    { 0xd151050d, SUC_lexicon_tags_15 },
    { 0x27b1050d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x16310511, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9310515, SUC_lexicon_tags_37 },
    { 0xb7910515, SUC_lexicon_tags_35 },
    { 0x5a710517, SUC_lexicon_tags_1 },
    { 0x6ef10517, SUC_lexicon_tags_2 },
    { 0x16910519, SUC_lexicon_tags_37 },
    { 0xddb1051a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ef1051c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b1051f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2fb10521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fb10523, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x71910526, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f10529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9251052c, SUC_lexicon_tags_15 },
    { 0x4831052d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1710533, SUC_lexicon_tags_68 },
    { 0xaa110533, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd110537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5110539, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b51053d, SUC_lexicon_tags_21 },
    { 0x56d1053e, SUC_lexicon_tags_0 },
    { 0x6491053f, SUC_lexicon_tags_17 },
    { 0x74b1053d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f910543, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b10547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9331054b, SUC_lexicon_tags_46 },
    { 0x59d1054c, SUC_lexicon_tags_1 },
    { 0x7251054d, SUC_lexicon_tags_14 },
    { 0xb151054c, SUC_lexicon_tags_68 },
    { 0xb3b1054c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff710555, SUC_lexicon_tags_12 },
    { 0x88510555, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d10559, SUC_lexicon_tags_23 },
    { 0x1611055a, SUC_lexicon_tags_27 },
    { 0x5d31055b, SUC_lexicon_tags_10 },
    { 0xeb1055c, SUC_lexicon_tags_17 },
    { 0xe0910559, SUC_lexicon_tags_7 },
    { 0xf4f1055e, SUC_lexicon_tags_2 },
    { 0x5491055e, SUC_lexicon_tags_10 },
    { 0xe731055f, SUC_lexicon_tags_79 },
    { 0xf4f10561, SUC_lexicon_tags_37 },
    { 0xf0b10562, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb10566, SUC_lexicon_tags_60 },
    { 0x45110566, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1056a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf1056e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc910572, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x49710574, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3ad10576, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7e910578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d1057b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc5f1057d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d10584, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5910588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20510591, SUC_lexicon_tags_2 },
    { 0x87310592, SUC_lexicon_tags_15 },
    { 0xd8710593, SUC_lexicon_tags_48 },
    { 0x6ff10594, SUC_lexicon_tags_42 },
    { 0x49b10595, SUC_lexicon_tags_37 },
    { 0x41510596, SUC_lexicon_tags_12 },
    { 0x79d10595, SUC_lexicon_tags_14 },
    { 0xdb510598, SUC_lexicon_tags_1 },
    { 0xa7310599, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b1059c, SUC_lexicon_tags_2 },
    { 0xb8d1059d, SUC_lexicon_tags_2 },
    { 0x7c51059e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x937105a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe09105a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49105a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f3105a8, SUC_lexicon_tags_6 },
    { 0x443105a8, SUC_lexicon_tags_31 },
    { 0x6a1105aa, SUC_lexicon_tags_37 },
    { 0x2bd105ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243105af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32f105b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f105b4, SUC_lexicon_tags_17 },
    { 0x7105b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555105bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1105bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f105c3, SUC_lexicon_tags_15 },
    { 0xf17105c4, SUC_lexicon_tags_29 },
    { 0xaff105c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x919105c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd13105ca, SUC_lexicon_tags_27 },
    { 0xc69105ca, SUC_lexicon_tags_34 },
    { 0xc43105ca, SUC_lexicon_tags_1 },
    { 0x2e3105cd, SUC_lexicon_tags_17 },
    { 0x54b105ce, SUC_lexicon_tags_27 },
    { 0x4bd105cd, SUC_lexicon_tags_65 },
    { 0xe7d105d0, SUC_lexicon_tags_7 },
    { 0xc33105d1, SUC_lexicon_tags_33 },
    { 0xaad105d1, SUC_lexicon_tags_15 },
    { 0x907105d3, SUC_lexicon_tags_17 },
    { 0xd09105d3, SUC_lexicon_tags_19 },
    { 0x887105d2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85105d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81105dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199105e4, SUC_lexicon_tags_312 },
    { 0x4f9105e5, SUC_lexicon_tags_38 },
    { 0x5fd105e5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887105ec, SUC_lexicon_tags_2 },
    { 0x995105ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58d105ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b105f3, SUC_lexicon_tags_31 },
    { 0x173105f3, SUC_lexicon_tags_31 },
    { 0xde1105f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f105f8, SUC_lexicon_tags_21 },
    { 0x241105f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227105fd, SUC_lexicon_tags_23 },
    { 0x947105fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd710600, SUC_lexicon_tags_12 },
    { 0xe4d10600, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b310607, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5911060a, SUC_lexicon_tags_7 },
    { 0x5df1060a, SUC_lexicon_tags_23 },
    { 0x6431060c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15710614, SUC_lexicon_tags_0 },
    { 0x2bd10615, SUC_lexicon_tags_43 },
    { 0xaab10615, SUC_lexicon_tags_6 },
    { 0xfb310614, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4511061e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59510620, SUC_lexicon_tags_12 },
    { 0xf2910621, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5e110623, SUC_lexicon_tags_30 },
    { 0x1d10624, SUC_lexicon_tags_28 },
    { 0xb4b10625, SUC_lexicon_tags_11 },
    { 0x33310626, SUC_lexicon_tags_60 },
    { 0xb1f10625, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb551062c, SUC_lexicon_tags_47 },
    { 0x7131062c, SUC_lexicon_tags_30 },
    { 0x2e31062c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33710630, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c110637, SUC_lexicon_tags_49 },
    { 0xb4510638, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e510640, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b310644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7710648, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x8211064b, SUC_lexicon_tags_31 },
    { 0xa831064b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5510654, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5310658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36f1065a, SUC_lexicon_tags_15 },
    { 0xb411065a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3671065d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df10660, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xde110662, SUC_lexicon_tags_32 },
    { 0xb7b10662, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb710668, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5991066a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b1066f, SUC_lexicon_tags_7 },
    { 0x4cd1066f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b10673, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb1067b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf10680, SUC_lexicon_tags_7 },
    { 0xfa110681, SUC_lexicon_tags_39 },
    { 0x94510682, SUC_lexicon_tags_30 },
    { 0xaa910683, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3510687, SUC_lexicon_tags_25 },
    { 0x6b910687, SUC_lexicon_tags_11 },
    { 0xf9910687, SUC_lexicon_tags_4 },
    { 0x94b10688, SUC_lexicon_tags_14 },
    { 0x2bd1068b, SUC_lexicon_tags_1 },
    { 0x6671068c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1eb1068e, SUC_lexicon_tags_10 },
    { 0x4d11068f, SUC_lexicon_tags_2 },
    { 0xf2d1068e, SUC_lexicon_tags_25 },
    { 0x4f910691, SUC_lexicon_tags_17 },
    { 0x13510692, SUC_lexicon_tags_2 },
    { 0x74b10691, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91f10695, SUC_lexicon_tags_17 },
    { 0xeb110695, SUC_lexicon_tags_37 },
    { 0x18f10696, SUC_lexicon_tags_37 },
    { 0x19d10698, SUC_lexicon_tags_17 },
    { 0xdd10695, SUC_lexicon_tags_10 },
    { 0xe451069a, SUC_lexicon_tags_1 },
    { 0x6011069a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4451069e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x907106a1, SUC_lexicon_tags_2 },
    { 0xc35106a2, SUC_lexicon_tags_25 },
    { 0x4b1106a2, SUC_lexicon_tags_82 },
    { 0xe8f106a4, SUC_lexicon_tags_11 },
    { 0x5af106a5, SUC_lexicon_tags_23 },
    { 0x845106a6, SUC_lexicon_tags_12 },
    { 0xdef106a6, SUC_lexicon_tags_1 },
    { 0xf59106a8, SUC_lexicon_tags_8 },
    { 0xe37106a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15106ac, SUC_lexicon_tags_23 },
    { 0x775106ad, SUC_lexicon_tags_2 },
    { 0x1cf106ae, SUC_lexicon_tags_12 },
    { 0x87d106ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41106b2, SUC_lexicon_tags_19 },
    { 0xae9106b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79106b6, SUC_lexicon_tags_38 },
    { 0x2ff106b6, SUC_lexicon_tags_11 },
    { 0x329106b8, SUC_lexicon_tags_25 },
    { 0xfdf106b6, SUC_lexicon_tags_9 },
    { 0xea3106ba, SUC_lexicon_tags_25 },
    { 0xafb106b8, SUC_lexicon_tags_37 },
    { 0x9f106bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fd106be, SUC_lexicon_tags_28 },
    { 0x69f106bf, SUC_lexicon_tags_37 },
    { 0x95106bf, SUC_lexicon_tags_9 },
    { 0xcf3106c1, SUC_lexicon_tags_34 },
    { 0xc05106c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d106c6, SUC_lexicon_tags_66 },
    { 0x419106c7, SUC_lexicon_tags_11 },
    { 0xb1f106c7, SUC_lexicon_tags_47 },
    { 0x1d3106c7, SUC_lexicon_tags_21 },
    { 0xd8f106ca, SUC_lexicon_tags_6 },
    { 0x535106cb, SUC_lexicon_tags_11 },
    { 0x5c3106c6, SUC_lexicon_tags_12 },
    { 0xb9b106cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3106d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5106d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc31106d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed106db, SUC_lexicon_tags_29 },
    { 0xc65106dc, SUC_lexicon_tags_15 },
    { 0xffd106db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5106e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b106e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5106e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9106eb, SUC_lexicon_tags_12 },
    { 0x295106eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5106f7, SUC_lexicon_tags_47 },
    { 0x115106f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc99106fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56310701, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50b10703, SUC_lexicon_tags_15 },
    { 0x66b10703, SUC_lexicon_tags_7 },
    { 0xa7510704, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61710707, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7c310709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6310717, SUC_lexicon_tags_2 },
    { 0x53310718, SUC_lexicon_tags_116 },
    { 0x5d710718, SUC_lexicon_tags_29 },
    { 0x66710718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41910721, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f10730, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x72710733, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x24f10735, SUC_lexicon_tags_12 },
    { 0xad910735, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9311073c, SUC_lexicon_tags_23 },
    { 0xcb71073c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27f1073f, SUC_lexicon_tags_1 },
    { 0x67710740, SUC_lexicon_tags_66 },
    { 0xb1110741, SUC_lexicon_tags_34 },
    { 0xeed10741, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeaf10744, SUC_lexicon_tags_16 },
    { 0xf5110745, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b10752, SUC_lexicon_tags_8 },
    { 0xa2510753, SUC_lexicon_tags_49 },
    { 0xbeb10754, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefb10756, SUC_lexicon_tags_25 },
    { 0x4f310756, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7df10759, SUC_lexicon_tags_39 },
    { 0xd7f10759, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1310762, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b10768, SUC_lexicon_tags_1 },
    { 0x62d10768, SUC_lexicon_tags_140 },
    { 0x1151076a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d10770, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96710775, SUC_lexicon_tags_18 },
    { 0x1c310775, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb10779, SUC_lexicon_tags_12 },
    { 0x7031077a, SUC_lexicon_tags_9 },
    { 0x6f71077a, SUC_lexicon_tags_2 },
    { 0xae71077c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8591077e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf8d10780, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7710783, SUC_lexicon_tags_19 },
    { 0x29310784, SUC_lexicon_tags_7 },
    { 0x55d10784, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d1078f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6110798, SUC_lexicon_tags_12 },
    { 0x47710799, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb71079b, SUC_lexicon_tags_37 },
    { 0x7e71079b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4951079f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd9107a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5107a4, SUC_lexicon_tags_26 },
    { 0x25b107a5, SUC_lexicon_tags_10 },
    { 0x941107a6, SUC_lexicon_tags_38 },
    { 0x727107a5, SUC_lexicon_tags_38 },
    { 0x3c5107a8, SUC_lexicon_tags_19 },
    { 0xb9f107a9, SUC_lexicon_tags_33 },
    { 0xead107aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf107b0, SUC_lexicon_tags_17 },
    { 0x649107b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9c5107b3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b107ba, SUC_lexicon_tags_9 },
    { 0x995107bb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xecb107bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4d107bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f107c2, SUC_lexicon_tags_1152 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d107c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f107c9, SUC_lexicon_tags_84 },
    { 0xe97107ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9107cd, SUC_lexicon_tags_31 },
    { 0x92f107ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937107d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb107d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xba9107da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf61107dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7107df, SUC_lexicon_tags_47 },
    { 0x661107e0, SUC_lexicon_tags_11 },
    { 0xb85107e1, SUC_lexicon_tags_46 },
    { 0x379107e1, SUC_lexicon_tags_17 },
    { 0xf6d107e3, SUC_lexicon_tags_47 },
    { 0x13f107e4, SUC_lexicon_tags_14 },
    { 0xe21107e1, SUC_lexicon_tags_72 },
    { 0x41d107e6, SUC_lexicon_tags_15 },
    { 0x469107e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef107eb, SUC_lexicon_tags_11 },
    { 0x7bd107eb, SUC_lexicon_tags_32 },
    { 0x52b107eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3b107ef, SUC_lexicon_tags_31 },
    { 0x20d107f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb107f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x529107f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x693107fa, SUC_lexicon_tags_65 },
    { 0xb01107fb, SUC_lexicon_tags_45 },
    { 0xd1d107fb, SUC_lexicon_tags_0 },
    { 0x411107fb, SUC_lexicon_tags_31 },
    { 0xfc3107fc, SUC_lexicon_tags_140 },
    { 0xa51107ff, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdfb10801, SUC_lexicon_tags_26 },
    { 0xda310802, SUC_lexicon_tags_16 },
    { 0x8f510803, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb1910805, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76310807, SUC_lexicon_tags_19 },
    { 0x87110808, SUC_lexicon_tags_17 },
    { 0x3310809, SUC_lexicon_tags_17 },
    { 0x14110807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b1080e, SUC_lexicon_tags_11 },
    { 0x7df1080f, SUC_lexicon_tags_1 },
    { 0x1a310810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28510813, SUC_lexicon_tags_42 },
    { 0xecf10814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c710816, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27910818, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f10829, SUC_lexicon_tags_12 },
    { 0x66d1082a, SUC_lexicon_tags_25 },
    { 0xc591082b, SUC_lexicon_tags_68 },
    { 0x3cf1082b, SUC_lexicon_tags_4 },
    { 0xaf31082d, SUC_lexicon_tags_7 },
    { 0xeb51082a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd10831, SUC_lexicon_tags_68 },
    { 0x4ab10832, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x53710835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa110838, SUC_lexicon_tags_46 },
    { 0xf5910839, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d1083d, SUC_lexicon_tags_15 },
    { 0x11d1083e, SUC_lexicon_tags_37 },
    { 0x31083e, SUC_lexicon_tags_19 },
    { 0xde91083d, SUC_lexicon_tags_60 },
    { 0xbcf10841, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf10844, SUC_lexicon_tags_38 },
    { 0xe3f10844, SUC_lexicon_tags_124 },
    { 0xcc710846, SUC_lexicon_tags_37 },
    { 0xd7d10846, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f1084c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9710850, SUC_lexicon_tags_220 },
    { 0x35910851, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5310853, SUC_lexicon_tags_27 },
    { 0x11d10854, SUC_lexicon_tags_21 },
    { 0x1ef10855, SUC_lexicon_tags_0 },
    { 0xd0b10856, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d1085a, SUC_lexicon_tags_45 },
    { 0x3811085a, SUC_lexicon_tags_2 },
    { 0x30f1085c, SUC_lexicon_tags_38 },
    { 0x80b1085c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9310862, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc310866, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2731086a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5191086d, SUC_lexicon_tags_25 },
    { 0xb1b1086d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d10872, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47910878, SUC_lexicon_tags_60 },
    { 0x61310879, SUC_lexicon_tags_49 },
    { 0xf251087a, SUC_lexicon_tags_14 },
    { 0xf751087a, SUC_lexicon_tags_7 },
    { 0x59f1087c, SUC_lexicon_tags_38 },
    { 0x4651087d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3911087f, SUC_lexicon_tags_17 },
    { 0x7eb10880, SUC_lexicon_tags_6 },
    { 0x1211087f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e510883, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3910887, SUC_lexicon_tags_15 },
    { 0xefb10887, SUC_lexicon_tags_17 },
    { 0x40f10887, SUC_lexicon_tags_38 },
    { 0x61310887, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd291088e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb71089c, SUC_lexicon_tags_70 },
    { 0xc111089d, SUC_lexicon_tags_7 },
    { 0xfbb1089d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdbb108a0, SUC_lexicon_tags_25 },
    { 0x91d108a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x123108a3, SUC_lexicon_tags_0 },
    { 0x47108a4, SUC_lexicon_tags_28 },
    { 0x8a9108a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x889108a7, SUC_lexicon_tags_17 },
    { 0x88f108a8, SUC_lexicon_tags_47 },
    { 0xe3f108a9, SUC_lexicon_tags_11 },
    { 0xbad108a9, SUC_lexicon_tags_2 },
    { 0x6bf108ab, SUC_lexicon_tags_8 },
    { 0x9d7108ab, SUC_lexicon_tags_615 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697108b2, SUC_lexicon_tags_39 },
    { 0x9ff108b3, SUC_lexicon_tags_1 },
    { 0x221108b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f1108b6, SUC_lexicon_tags_68 },
    { 0xd3d108b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577108bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x491108c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47108c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39108c9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x525108cb, SUC_lexicon_tags_4 },
    { 0x775108cc, SUC_lexicon_tags_37 },
    { 0xacb108cd, SUC_lexicon_tags_12 },
    { 0x115108ce, SUC_lexicon_tags_25 },
    { 0x1ed108cf, SUC_lexicon_tags_1 },
    { 0x73d108cf, SUC_lexicon_tags_39 },
    { 0x773108cd, SUC_lexicon_tags_37 },
    { 0x83b108ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x833108d4, SUC_lexicon_tags_12 },
    { 0xc7f108d5, SUC_lexicon_tags_37 },
    { 0xa61108d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f108da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31108de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf108e1, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xfab108e3, SUC_lexicon_tags_15 },
    { 0xe03108e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5108e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f108eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9108ef, SUC_lexicon_tags_15 },
    { 0x1e1108f0, SUC_lexicon_tags_0 },
    { 0xdd5108f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4d108f3, SUC_lexicon_tags_25 },
    { 0x171108f4, SUC_lexicon_tags_29 },
    { 0x287108f5, SUC_lexicon_tags_1 },
    { 0x4a3108f6, SUC_lexicon_tags_25 },
    { 0xa7f108f6, SUC_lexicon_tags_11 },
    { 0x4d5108f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad108ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73710904, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe110906, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71090a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f10910, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b10914, SUC_lexicon_tags_0 },
    { 0xaf10914, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f10918, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa451091b, SUC_lexicon_tags_49 },
    { 0xe51091c, SUC_lexicon_tags_7 },
    { 0xc851091d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b910921, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b110924, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf10929, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf831092b, SUC_lexicon_tags_26 },
    { 0x20d1092c, SUC_lexicon_tags_28 },
    { 0x5731092c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf10931, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea510936, SUC_lexicon_tags_10 },
    { 0x8cb10937, SUC_lexicon_tags_15 },
    { 0x93710937, SUC_lexicon_tags_11 },
    { 0x9b710939, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf071093c, SUC_lexicon_tags_15 },
    { 0x4551093d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f110941, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d10949, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1094e, SUC_lexicon_tags_37 },
    { 0x4d1094e, SUC_lexicon_tags_37 },
    { 0x98110950, SUC_lexicon_tags_30 },
    { 0xc9710950, SUC_lexicon_tags_10 },
    { 0x26710952, SUC_lexicon_tags_11 },
    { 0x56f10953, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7510957, SUC_lexicon_tags_1 },
    { 0x78310957, SUC_lexicon_tags_7 },
    { 0x69510959, SUC_lexicon_tags_17 },
    { 0x9411095a, SUC_lexicon_tags_1 },
    { 0x6871095b, SUC_lexicon_tags_37 },
    { 0xcb11095b, SUC_lexicon_tags_17 },
    { 0xf331095c, SUC_lexicon_tags_15 },
    { 0x1fb1095b, SUC_lexicon_tags_48 },
    { 0x8591095b, SUC_lexicon_tags_66 },
    { 0x4ab1095c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94310964, SUC_lexicon_tags_17 },
    { 0x7db10965, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b710967, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec91097c, SUC_lexicon_tags_27 },
    { 0xbe91097d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b51097f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1bf10981, SUC_lexicon_tags_37 },
    { 0x40710982, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c310985, SUC_lexicon_tags_47 },
    { 0xbfd10986, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc531098a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1831098c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b10993, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac31099b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfa31099d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa691099f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x215109a1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417109ac, SUC_lexicon_tags_0 },
    { 0x75b109ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21109b0, SUC_lexicon_tags_65 },
    { 0x467109b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x151109b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713109bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7109bf, SUC_lexicon_tags_25 },
    { 0xdc9109c0, SUC_lexicon_tags_55 },
    { 0x5f1109c0, SUC_lexicon_tags_15 },
    { 0x837109c2, SUC_lexicon_tags_0 },
    { 0x7f7109c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5109c7, SUC_lexicon_tags_2 },
    { 0xb53109c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85109d6, SUC_lexicon_tags_15 },
    { 0xec5109d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x787109d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5109dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165109e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17109eb, SUC_lexicon_tags_88 },
    { 0xf35109ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17109f5, SUC_lexicon_tags_17 },
    { 0xbeb109f6, SUC_lexicon_tags_49 },
    { 0x77b109f7, SUC_lexicon_tags_23 },
    { 0x167109f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2910a03, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60510a0c, SUC_lexicon_tags_11 },
    { 0xd1f10a0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29310a10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6310a15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd10a18, SUC_lexicon_tags_15 },
    { 0xec510a19, SUC_lexicon_tags_1 },
    { 0x49d10a1a, SUC_lexicon_tags_21 },
    { 0xdd110a1b, SUC_lexicon_tags_28 },
    { 0xa3b10a1c, SUC_lexicon_tags_0 },
    { 0xd6d10a1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0b10a1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66510a24, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb310a29, SUC_lexicon_tags_12 },
    { 0x5cd10a2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd10a2e, SUC_lexicon_tags_33 },
    { 0x43f10a2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4dd10a31, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x13510a34, SUC_lexicon_tags_35 },
    { 0x8d510a35, SUC_lexicon_tags_14 },
    { 0x83b10a36, SUC_lexicon_tags_32 },
    { 0x12710a35, SUC_lexicon_tags_12 },
    { 0x1f10a34, SUC_lexicon_tags_25 },
    { 0x1fd10a38, SUC_lexicon_tags_34 },
    { 0x35d10a3a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57910a40, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa9710a42, SUC_lexicon_tags_15 },
    { 0xecd10a43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd10a47, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec710a51, SUC_lexicon_tags_8 },
    { 0xe0710a51, SUC_lexicon_tags_48 },
    { 0x32d10a52, SUC_lexicon_tags_19 },
    { 0xafb10a54, SUC_lexicon_tags_1 },
    { 0xaf510a55, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa1510a57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e110a5b, SUC_lexicon_tags_37 },
    { 0x9d510a5c, SUC_lexicon_tags_32 },
    { 0x4910a5c, SUC_lexicon_tags_17 },
    { 0x93d10a5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d510a61, SUC_lexicon_tags_3 },
    { 0x9e910a62, SUC_lexicon_tags_32 },
    { 0xec510a62, SUC_lexicon_tags_30 },
    { 0x70910a64, SUC_lexicon_tags_37 },
    { 0x9ef10a64, SUC_lexicon_tags_0 },
    { 0x66310a61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b510a69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64710a6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d10a73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42710a77, SUC_lexicon_tags_1 },
    { 0x4e110a77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a510a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e110a80, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb10a8b, SUC_lexicon_tags_2 },
    { 0x61710a8c, SUC_lexicon_tags_8 },
    { 0x67310a8d, SUC_lexicon_tags_17 },
    { 0xfbf10a8b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x82510a91, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9510a96, SUC_lexicon_tags_39 },
    { 0x5e910a97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18110a9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8b310a9d, SUC_lexicon_tags_13 },
    { 0x43d10a9e, SUC_lexicon_tags_1 },
    { 0x4d510a9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf10aa3, SUC_lexicon_tags_2 },
    { 0xb1910aa4, SUC_lexicon_tags_20 },
    { 0x10510aa5, SUC_lexicon_tags_11 },
    { 0x18510aa5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56310aaa, SUC_lexicon_tags_0 },
    { 0xab710aab, SUC_lexicon_tags_25 },
    { 0xbd910aac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d10aaf, SUC_lexicon_tags_30 },
    { 0x93710ab0, SUC_lexicon_tags_5 },
    { 0xaa110ab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c710ab8, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d10abd, SUC_lexicon_tags_12 },
    { 0xa110abd, SUC_lexicon_tags_12 },
    { 0x4d910abf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x21710ac1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcb110ac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd10ac8, SUC_lexicon_tags_25 },
    { 0xfc910ac8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c910ad3, SUC_lexicon_tags_19 },
    { 0x34b10ad4, SUC_lexicon_tags_8 },
    { 0xbc110ad5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1110ad8, SUC_lexicon_tags_79 },
    { 0xe1710ad8, SUC_lexicon_tags_8 },
    { 0xd8710ada, SUC_lexicon_tags_25 },
    { 0xbdf10ada, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x65d10add, SUC_lexicon_tags_151 },
    { 0x91910ade, SUC_lexicon_tags_12 },
    { 0xa3510adf, SUC_lexicon_tags_23 },
    { 0x1a110ae0, SUC_lexicon_tags_37 },
    { 0xf5f10ae1, SUC_lexicon_tags_7 },
    { 0x59110ae2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f710ae5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d10aec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84910aee, SUC_lexicon_tags_25 },
    { 0x9fb10aef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d710af1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92b10af3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84710af7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd910af9, SUC_lexicon_tags_2 },
    { 0xae710afa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98710afe, SUC_lexicon_tags_49 },
    { 0x51f10aff, SUC_lexicon_tags_23 },
    { 0xb6b10aff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc910b03, SUC_lexicon_tags_1 },
    { 0x25310b04, SUC_lexicon_tags_49 },
    { 0x33910b05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a710b0a, SUC_lexicon_tags_17 },
    { 0x90310b0a, SUC_lexicon_tags_9 },
    { 0xadb10b0c, SUC_lexicon_tags_1 },
    { 0x9a910b0c, SUC_lexicon_tags_47 },
    { 0xb5310b0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x57710b10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e910b12, SUC_lexicon_tags_63 },
    { 0x42f10b13, SUC_lexicon_tags_17 },
    { 0x7df10b13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63510b19, SUC_lexicon_tags_19 },
    { 0xf6b10b1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb2f10b1c, SUC_lexicon_tags_60 },
    { 0xadf10b1d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b10b21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70110b27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef10b2a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3710b30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa710b36, SUC_lexicon_tags_25 },
    { 0xccb10b37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d310b3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6bf10b3c, SUC_lexicon_tags_14 },
    { 0x91110b3d, SUC_lexicon_tags_4 },
    { 0xf310b3e, SUC_lexicon_tags_11 },
    { 0x4eb10b3e, SUC_lexicon_tags_43 },
    { 0x5a310b40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3910b44, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c310b47, SUC_lexicon_tags_25 },
    { 0x3910b48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf10b51, SUC_lexicon_tags_15 },
    { 0xa1d10b52, SUC_lexicon_tags_38 },
    { 0x3c710b53, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f710b56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60910b5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d10b5e, SUC_lexicon_tags_25 },
    { 0x82f10b5f, SUC_lexicon_tags_27 },
    { 0x73f10b60, SUC_lexicon_tags_11 },
    { 0x2e710b61, SUC_lexicon_tags_82 },
    { 0x5a110b5e, SUC_lexicon_tags_17 },
    { 0x96510b61, SUC_lexicon_tags_15 },
    { 0xb4910b64, SUC_lexicon_tags_59 },
    { 0x65510b62, SUC_lexicon_tags_15 },
    { 0xd9710b66, SUC_lexicon_tags_17 },
    { 0x23b10b66, SUC_lexicon_tags_37 },
    { 0xcab10b66, SUC_lexicon_tags_25 },
    { 0x40910b65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b510b6b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b10b6e, SUC_lexicon_tags_47 },
    { 0xd9710b6f, SUC_lexicon_tags_2 },
    { 0xa3f10b70, SUC_lexicon_tags_25 },
    { 0x83b10b70, SUC_lexicon_tags_7 },
    { 0xc1d10b72, SUC_lexicon_tags_1 },
    { 0x18910b73, SUC_lexicon_tags_29 },
    { 0xf6310b73, SUC_lexicon_tags_37 },
    { 0xf7310b75, SUC_lexicon_tags_25 },
    { 0x2d510b76, SUC_lexicon_tags_14 },
    { 0xa2510b6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d10b7c, SUC_lexicon_tags_44 },
    { 0x62d10b7d, SUC_lexicon_tags_38 },
    { 0x99910b7d, SUC_lexicon_tags_65 },
    { 0xab110b7e, SUC_lexicon_tags_2 },
    { 0x2f510b80, SUC_lexicon_tags_17 },
    { 0x8a110b7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xab10b84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d10b88, SUC_lexicon_tags_12 },
    { 0x28310b88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45110b8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1710b8f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b10b94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff10b98, SUC_lexicon_tags_25 },
    { 0xd4910b99, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9910b9c, SUC_lexicon_tags_57 },
    { 0x6710b9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d10ba0, SUC_lexicon_tags_1 },
    { 0xcc510ba1, SUC_lexicon_tags_68 },
    { 0x49510ba0, SUC_lexicon_tags_48 },
    { 0xc6d10ba3, SUC_lexicon_tags_12 },
    { 0x1dd10ba3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db10ba8, SUC_lexicon_tags_25 },
    { 0x5c510ba9, SUC_lexicon_tags_43 },
    { 0xcbd10baa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8af10bac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b10baf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51310bb8, SUC_lexicon_tags_7 },
    { 0x7d710bb8, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d10bbc, SUC_lexicon_tags_12 },
    { 0x32d10bbc, SUC_lexicon_tags_17 },
    { 0xb1b10bbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d310bc2, SUC_lexicon_tags_37 },
    { 0x3f910bc2, SUC_lexicon_tags_32 },
    { 0xa7510bc3, SUC_lexicon_tags_2 },
    { 0x23710bc5, SUC_lexicon_tags_14 },
    { 0xb5d10bc5, SUC_lexicon_tags_25 },
    { 0xb310bc3, SUC_lexicon_tags_37 },
    { 0xe7910bc8, SUC_lexicon_tags_66 },
    { 0x5fd10bc4, SUC_lexicon_tags_38 },
    { 0x48510bc4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d10bcf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x93510bd2, SUC_lexicon_tags_7 },
    { 0x19910bd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b10bd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d10bdc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a510be2, SUC_lexicon_tags_33 },
    { 0xbdd10be3, SUC_lexicon_tags_84 },
    { 0x61110be3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x92910be6, SUC_lexicon_tags_33 },
    { 0x28f10be7, SUC_lexicon_tags_11 },
    { 0xe4710be8, SUC_lexicon_tags_63 },
    { 0xddb10be8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb10bee, SUC_lexicon_tags_19 },
    { 0xe310bef, SUC_lexicon_tags_23 },
    { 0x84310bee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b510bf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9910bfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ab10bfc, SUC_lexicon_tags_7 },
    { 0xeb710bfd, SUC_lexicon_tags_48 },
    { 0x95510bfe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7510c01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbef10c03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x99510c06, SUC_lexicon_tags_25 },
    { 0x82510c06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73110c10, SUC_lexicon_tags_8 },
    { 0x5f110c11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28d10c13, SUC_lexicon_tags_12 },
    { 0x19310c14, SUC_lexicon_tags_12 },
    { 0x8f110c15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d10c1b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x54710c1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10b10c1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x13710c21, SUC_lexicon_tags_8 },
    { 0x24b10c21, SUC_lexicon_tags_0 },
    { 0xaf110c21, SUC_lexicon_tags_25 },
    { 0x56d10c22, SUC_lexicon_tags_35 },
    { 0x2e910c25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x31310c28, SUC_lexicon_tags_46 },
    { 0x13310c29, SUC_lexicon_tags_17 },
    { 0xcf110c2a, SUC_lexicon_tags_25 },
    { 0xc1d10c2b, SUC_lexicon_tags_1 },
    { 0x44510c2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71910c36, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6fd10c38, SUC_lexicon_tags_25 },
    { 0x43110c39, SUC_lexicon_tags_28 },
    { 0x7e110c38, SUC_lexicon_tags_25 },
    { 0x65310c3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42510c3d, SUC_lexicon_tags_37 },
    { 0xfe110c3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf710c42, SUC_lexicon_tags_33 },
    { 0x76110c43, SUC_lexicon_tags_17 },
    { 0x32d10c44, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f10c48, SUC_lexicon_tags_45 },
    { 0x310c49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55910c4b, SUC_lexicon_tags_63 },
    { 0xe9f10c4c, SUC_lexicon_tags_10 },
    { 0x3d110c4c, SUC_lexicon_tags_66 },
    { 0xced10c4e, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d10c57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8110c5b, SUC_lexicon_tags_7 },
    { 0xcdb10c5c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf10c60, SUC_lexicon_tags_151 },
    { 0x1dd10c61, SUC_lexicon_tags_15 },
    { 0xa0910c62, SUC_lexicon_tags_0 },
    { 0x3f510c61, SUC_lexicon_tags_14 },
    { 0x3d510c64, SUC_lexicon_tags_25 },
    { 0x46110c65, SUC_lexicon_tags_2 },
    { 0x39110c66, SUC_lexicon_tags_17 },
    { 0xb9b10c66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd10c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d10c73, SUC_lexicon_tags_49 },
    { 0x77910c74, SUC_lexicon_tags_29 },
    { 0x8c510c75, SUC_lexicon_tags_16 },
    { 0xac110c76, SUC_lexicon_tags_27 },
    { 0x8c510c77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b10c7a, SUC_lexicon_tags_33 },
    { 0xc3310c7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd10c81, SUC_lexicon_tags_17 },
    { 0x28710c82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b10c85, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xebb10c87, SUC_lexicon_tags_68 },
    { 0x98710c88, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7a310c8a, SUC_lexicon_tags_1 },
    { 0x6c910c8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55110c8f, SUC_lexicon_tags_2 },
    { 0x18b10c90, SUC_lexicon_tags_34 },
    { 0xb6910c8f, SUC_lexicon_tags_30 },
    { 0x95110c92, SUC_lexicon_tags_7 },
    { 0xc8d10c93, SUC_lexicon_tags_60 },
    { 0x83910c94, SUC_lexicon_tags_56 },
    { 0xee310c94, SUC_lexicon_tags_11 },
    { 0x92710c96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3b10c98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf10c9d, SUC_lexicon_tags_48 },
    { 0xf0510c9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d10ca2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4910ca5, SUC_lexicon_tags_84 },
    { 0xba710ca6, SUC_lexicon_tags_11 },
    { 0x32910ca7, SUC_lexicon_tags_10 },
    { 0x69510ca8, SUC_lexicon_tags_49 },
    { 0xf3d10ca9, SUC_lexicon_tags_35 },
    { 0x90d10ca9, SUC_lexicon_tags_8 },
    { 0x8af10caa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b10caf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x11310cb2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x58510cb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91110cc2, SUC_lexicon_tags_11 },
    { 0x2f310cc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f10cd4, SUC_lexicon_tags_15 },
    { 0xe3f10cd5, SUC_lexicon_tags_77 },
    { 0x44d10cd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f510cd8, SUC_lexicon_tags_15 },
    { 0xe2310cd9, SUC_lexicon_tags_19 },
    { 0x29710cda, SUC_lexicon_tags_118 },
    { 0x6bf10cd8, SUC_lexicon_tags_11 },
    { 0x34b10cdb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfd10cde, SUC_lexicon_tags_19 },
    { 0xdfb10cde, SUC_lexicon_tags_15 },
    { 0xe9510cdf, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0710ce5, SUC_lexicon_tags_46 },
    { 0xe6510ce6, SUC_lexicon_tags_74 },
    { 0x7a110ce7, SUC_lexicon_tags_19 },
    { 0x97110ce5, SUC_lexicon_tags_15 },
    { 0x9ed10ce9, SUC_lexicon_tags_1 },
    { 0x39910ce6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x37110ced, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b10cf0, SUC_lexicon_tags_2 },
    { 0x1f310cf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f10cf5, SUC_lexicon_tags_37 },
    { 0x41710cf6, SUC_lexicon_tags_63 },
    { 0x2510cf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff10cfb, SUC_lexicon_tags_48 },
    { 0x94f10cfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e710cfe, SUC_lexicon_tags_38 },
    { 0x6b510cff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c310d04, SUC_lexicon_tags_63 },
    { 0x5d510d05, SUC_lexicon_tags_2 },
    { 0xe4710d05, SUC_lexicon_tags_1 },
    { 0x53b10d07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58310d09, SUC_lexicon_tags_38 },
    { 0x6b710d0a, SUC_lexicon_tags_11 },
    { 0x3710d0b, SUC_lexicon_tags_37 },
    { 0x1a910d0c, SUC_lexicon_tags_12 },
    { 0x14d10d0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f10d11, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x81110d14, SUC_lexicon_tags_0 },
    { 0xbaf10d15, SUC_lexicon_tags_32 },
    { 0xf9f10d16, SUC_lexicon_tags_14 },
    { 0xc9d10d16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50710d1c, SUC_lexicon_tags_1 },
    { 0x3d10d1c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb0b10d1f, SUC_lexicon_tags_31 },
    { 0x87510d1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee510d29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1910d2b, SUC_lexicon_tags_69 },
    { 0xde510d2c, SUC_lexicon_tags_14 },
    { 0xad110d2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x73510d2f, SUC_lexicon_tags_49 },
    { 0x97710d30, SUC_lexicon_tags_14 },
    { 0x73310d31, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5910d34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b10d39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5310d46, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d510d49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf910d4b, SUC_lexicon_tags_15 },
    { 0x8910d4c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91310d51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd510d54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d10d59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81110d5b, SUC_lexicon_tags_55 },
    { 0x5f910d5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c310d5e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb10d61, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5310d64, SUC_lexicon_tags_2 },
    { 0x24110d65, SUC_lexicon_tags_0 },
    { 0xf8d10d66, SUC_lexicon_tags_25 },
    { 0x2b310d66, SUC_lexicon_tags_31 },
    { 0x3fd10d67, SUC_lexicon_tags_11 },
    { 0xfff10d67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf10d6c, SUC_lexicon_tags_2 },
    { 0x83110d6d, SUC_lexicon_tags_11 },
    { 0x7f310d6e, SUC_lexicon_tags_7 },
    { 0x57d10d6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd8110d71, SUC_lexicon_tags_12 },
    { 0x2510d71, SUC_lexicon_tags_11 },
    { 0x67b10d73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd10d7a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x46710d7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe910d7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b10d87, SUC_lexicon_tags_15 },
    { 0x1d10d88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6c310d8a, SUC_lexicon_tags_25 },
    { 0xd310d8b, SUC_lexicon_tags_107 },
    { 0xd710d8c, SUC_lexicon_tags_60 },
    { 0xd9b10d8c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d10d93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca910d98, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8710da1, SUC_lexicon_tags_37 },
    { 0x30510da2, SUC_lexicon_tags_17 },
    { 0x30f10da2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94710dab, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc4510dad, SUC_lexicon_tags_26 },
    { 0xddb10dad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x79b10db0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae910db4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6b10db6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4b510db8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec310dbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf10dbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87110dc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xea710dc3, SUC_lexicon_tags_15 },
    { 0xc5310dc4, SUC_lexicon_tags_2 },
    { 0x48f10dc5, SUC_lexicon_tags_12 },
    { 0x5310dc6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4c510dc8, SUC_lexicon_tags_65 },
    { 0xbf510dc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c510dcb, SUC_lexicon_tags_15 },
    { 0x6f910dcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1110dd0, SUC_lexicon_tags_2 },
    { 0x5b310dd0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb110dd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55310dd9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79310dde, SUC_lexicon_tags_1 },
    { 0xa3f10dde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x21510de2, SUC_lexicon_tags_25 },
    { 0xb0f10de3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93310de5, SUC_lexicon_tags_68 },
    { 0x15f10de6, SUC_lexicon_tags_15 },
    { 0x7b110de6, SUC_lexicon_tags_378 },
    { 0xa7f10de6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d510df2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b10df7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f10dfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40310e01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f510e03, SUC_lexicon_tags_27 },
    { 0xb6110e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3f10e06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1d10e08, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1110e0c, SUC_lexicon_tags_27 },
    { 0x82310e0d, SUC_lexicon_tags_47 },
    { 0x9f110e0e, SUC_lexicon_tags_25 },
    { 0x17310e0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x61710e11, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f10e18, SUC_lexicon_tags_15 },
    { 0x8f110e18, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef10e1e, SUC_lexicon_tags_82 },
    { 0xe6110e1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7910e22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d10e27, SUC_lexicon_tags_37 },
    { 0x21f10e28, SUC_lexicon_tags_3 },
    { 0x2910e29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1110e2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1110e34, SUC_lexicon_tags_21 },
    { 0x6b910e35, SUC_lexicon_tags_7 },
    { 0x2e710e35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb10e3d, SUC_lexicon_tags_37 },
    { 0x71d10e3e, SUC_lexicon_tags_25 },
    { 0x6cf10e3d, SUC_lexicon_tags_82 },
    { 0xb9f10e40, SUC_lexicon_tags_43 },
    { 0xbe510e40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0310e44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd10e4a, SUC_lexicon_tags_27 },
    { 0xc3b10e4b, SUC_lexicon_tags_14 },
    { 0x82110e4c, SUC_lexicon_tags_23 },
    { 0xf3310e4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40f10e4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f10e54, SUC_lexicon_tags_17 },
    { 0x54510e55, SUC_lexicon_tags_12 },
    { 0x21b10e55, SUC_lexicon_tags_1 },
    { 0xf9910e57, SUC_lexicon_tags_17 },
    { 0xc2b10e54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7310e5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a310e62, SUC_lexicon_tags_0 },
    { 0x1c710e63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ed10e65, SUC_lexicon_tags_28 },
    { 0xd1b10e66, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb9310e68, SUC_lexicon_tags_33 },
    { 0x94f10e69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec710e6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda710e6f, SUC_lexicon_tags_34 },
    { 0xbcb10e70, SUC_lexicon_tags_37 },
    { 0x7710e71, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7e710e73, SUC_lexicon_tags_47 },
    { 0xa4110e74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7910e78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd10e7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c510e7f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2cf10e81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd310e87, SUC_lexicon_tags_37 },
    { 0x99d10e88, SUC_lexicon_tags_25 },
    { 0x44f10e88, SUC_lexicon_tags_4 },
    { 0xa9f10e8a, SUC_lexicon_tags_33 },
    { 0xc4d10e8b, SUC_lexicon_tags_1 },
    { 0x28d10e8c, SUC_lexicon_tags_33 },
    { 0x27110e8a, SUC_lexicon_tags_26 },
    { 0xb6310e8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x76310e91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5310e99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3510e9e, SUC_lexicon_tags_28 },
    { 0xbd110e9f, SUC_lexicon_tags_15 },
    { 0x17510ea0, SUC_lexicon_tags_30 },
    { 0x42b10ea1, SUC_lexicon_tags_40 },
    { 0x1b310ea2, SUC_lexicon_tags_37 },
    { 0x5a110e9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b10ea5, SUC_lexicon_tags_17 },
    { 0xf3d10ea5, SUC_lexicon_tags_2 },
    { 0x19910ea7, SUC_lexicon_tags_37 },
    { 0x73310ea8, SUC_lexicon_tags_12 },
    { 0x4c910ea7, SUC_lexicon_tags_2 },
    { 0x72710eaa, SUC_lexicon_tags_17 },
    { 0xa2b10eab, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xa0910ead, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf8310eaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5d10eb1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d10eb4, SUC_lexicon_tags_45 },
    { 0xc1510eb5, SUC_lexicon_tags_1 },
    { 0x48d10eb6, SUC_lexicon_tags_17 },
    { 0xc9910eb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37310eb9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f10ebd, SUC_lexicon_tags_82 },
    { 0xea310ebd, SUC_lexicon_tags_15 },
    { 0x3310ebf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d10ec9, SUC_lexicon_tags_1 },
    { 0xa8f10ec9, SUC_lexicon_tags_17 },
    { 0x96310ecb, SUC_lexicon_tags_6 },
    { 0x6c710ecc, SUC_lexicon_tags_25 },
    { 0xe9510ecd, SUC_lexicon_tags_27 },
    { 0xa110ecb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76710ed3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61b10ed5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63310edd, SUC_lexicon_tags_6 },
    { 0xf2510edd, SUC_lexicon_tags_29 },
    { 0x37310ede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9910ee5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe1f10ee7, SUC_lexicon_tags_25 },
    { 0xd4910ee8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb310eea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79710eec, SUC_lexicon_tags_43 },
    { 0x21f10eed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29510ef6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x33710ef8, SUC_lexicon_tags_0 },
    { 0xf6110ef8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x32110efc, SUC_lexicon_tags_1 },
    { 0x4df10efd, SUC_lexicon_tags_7 },
    { 0x84b10efd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92910f02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25310f05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb10f0a, SUC_lexicon_tags_0 },
    { 0x7510f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d10f11, SUC_lexicon_tags_37 },
    { 0x3c710f11, SUC_lexicon_tags_43 },
    { 0xb7510f13, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7d10f15, SUC_lexicon_tags_33 },
    { 0x8310f15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a510f18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f310f1a, SUC_lexicon_tags_7 },
    { 0xb4510f1b, SUC_lexicon_tags_25 },
    { 0xc4b10f1c, SUC_lexicon_tags_11 },
    { 0xde510f1d, SUC_lexicon_tags_87 },
    { 0x3dd10f1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b10f26, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa110f2b, SUC_lexicon_tags_37 },
    { 0xf7510f2b, SUC_lexicon_tags_11 },
    { 0x17d10f2d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70510f31, SUC_lexicon_tags_18 },
    { 0x37f10f32, SUC_lexicon_tags_17 },
    { 0x76910f33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88110f35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d310f39, SUC_lexicon_tags_35 },
    { 0x8f910f3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d110f3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34b10f3e, SUC_lexicon_tags_27 },
    { 0x9af10f3f, SUC_lexicon_tags_19 },
    { 0x3ed10f3f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf910f4a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x54710f4d, SUC_lexicon_tags_11 },
    { 0x29910f4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3910f50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d10f54, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x32f10f56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b10f59, SUC_lexicon_tags_17 },
    { 0x82d10f5a, SUC_lexicon_tags_16 },
    { 0xcc710f5b, SUC_lexicon_tags_27 },
    { 0x10510f59, SUC_lexicon_tags_7 },
    { 0xdf510f5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63510f63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d10f66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93b10f68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2910f73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf10f79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8d10f7b, SUC_lexicon_tags_17 },
    { 0xe7910f7c, SUC_lexicon_tags_1 },
    { 0xffd10f7d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f10f82, SUC_lexicon_tags_30 },
    { 0x46f10f83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf10f89, SUC_lexicon_tags_17 },
    { 0xbfb10f8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea710f8e, SUC_lexicon_tags_25 },
    { 0x83d10f8e, SUC_lexicon_tags_35 },
    { 0xb2910f90, SUC_lexicon_tags_6 },
    { 0x4d10f8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd310f95, SUC_lexicon_tags_47 },
    { 0x56710f96, SUC_lexicon_tags_65 },
    { 0x68510f97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61110f9d, SUC_lexicon_tags_12 },
    { 0xe7710f9d, SUC_lexicon_tags_25 },
    { 0xd0310f9d, SUC_lexicon_tags_30 },
    { 0xdf310f9e, SUC_lexicon_tags_61 },
    { 0x54310fa1, SUC_lexicon_tags_11 },
    { 0x6d310fa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf910fa6, SUC_lexicon_tags_37 },
    { 0x43f10fa7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55f10fa9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe510fb4, SUC_lexicon_tags_11 },
    { 0x7a710fb4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb10fb9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3710fbd, SUC_lexicon_tags_28 },
    { 0xf4f10fbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb10fc1, SUC_lexicon_tags_15 },
    { 0x5f910fc1, SUC_lexicon_tags_38 },
    { 0x1a910fc3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b10fc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x73110fc9, SUC_lexicon_tags_9 },
    { 0x7db10fca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0310fcd, SUC_lexicon_tags_46 },
    { 0x8c110fce, SUC_lexicon_tags_62 },
    { 0x46510fcd, SUC_lexicon_tags_26 },
    { 0xcf910fcd, SUC_lexicon_tags_2 },
    { 0x1af10fcf, SUC_lexicon_tags_2 },
    { 0x6f10fd2, SUC_lexicon_tags_17 },
    { 0xf5110fcf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x58510fd6, SUC_lexicon_tags_9 },
    { 0x23910fd7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f10fdd, SUC_lexicon_tags_55 },
    { 0xbaf10fde, SUC_lexicon_tags_12 },
    { 0xa5f10fdf, SUC_lexicon_tags_11 },
    { 0x78710fde, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac910fe5, SUC_lexicon_tags_23 },
    { 0x2ef10fe6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79510fec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3910ff3, SUC_lexicon_tags_2 },
    { 0x94710ff4, SUC_lexicon_tags_49 },
    { 0xb8110ff3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd910ffb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x57710ffd, SUC_lexicon_tags_0 },
    { 0x9d910ffe, SUC_lexicon_tags_25 },
    { 0x88f10fff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb711005, SUC_lexicon_tags_9 },
    { 0x13111006, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a51100c, SUC_lexicon_tags_65 },
    { 0xef71100d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab11012, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35511015, SUC_lexicon_tags_68 },
    { 0x33111016, SUC_lexicon_tags_23 },
    { 0x74d11017, SUC_lexicon_tags_51 },
    { 0x2cb11016, SUC_lexicon_tags_37 },
    { 0x16d11019, SUC_lexicon_tags_37 },
    { 0xa7b1101a, SUC_lexicon_tags_29 },
    { 0x31311015, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f511020, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9111024, SUC_lexicon_tags_37 },
    { 0x23111025, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x80b11027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9511029, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c11102c, SUC_lexicon_tags_45 },
    { 0x8931102d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b11032, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b11036, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x21511038, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b1103b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf691103d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96911041, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7ad11043, SUC_lexicon_tags_43 },
    { 0x4db11043, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b911047, SUC_lexicon_tags_37 },
    { 0x31b11048, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4171104a, SUC_lexicon_tags_11 },
    { 0xceb1104b, SUC_lexicon_tags_27 },
    { 0x9d51104c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad11054, SUC_lexicon_tags_19 },
    { 0x8c911054, SUC_lexicon_tags_11 },
    { 0xa711056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4111058, SUC_lexicon_tags_63 },
    { 0x27511059, SUC_lexicon_tags_19 },
    { 0x30711059, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a31105d, SUC_lexicon_tags_17 },
    { 0x7191105d, SUC_lexicon_tags_10 },
    { 0xe211105f, SUC_lexicon_tags_14 },
    { 0xf6311060, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a511062, SUC_lexicon_tags_27 },
    { 0x40b11063, SUC_lexicon_tags_13 },
    { 0xb1311064, SUC_lexicon_tags_2 },
    { 0x58b11063, SUC_lexicon_tags_23 },
    { 0x8af11066, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7511069, SUC_lexicon_tags_25 },
    { 0x72111069, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a711070, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6111074, SUC_lexicon_tags_17 },
    { 0xc4511075, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x56711078, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29511080, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f11083, SUC_lexicon_tags_37 },
    { 0x4bf11083, SUC_lexicon_tags_37 },
    { 0xe1d11085, SUC_lexicon_tags_15 },
    { 0x57711084, SUC_lexicon_tags_37 },
    { 0x2f511087, SUC_lexicon_tags_30 },
    { 0xb711088, SUC_lexicon_tags_43 },
    { 0x32f11089, SUC_lexicon_tags_60 },
    { 0xcd911085, SUC_lexicon_tags_14 },
    { 0x5cb1108b, SUC_lexicon_tags_8 },
    { 0xe4b11089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f311091, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9111099, SUC_lexicon_tags_11 },
    { 0x2391109a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d1109f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x401110a1, SUC_lexicon_tags_38 },
    { 0x66f110a2, SUC_lexicon_tags_47 },
    { 0x9ed110a2, SUC_lexicon_tags_12 },
    { 0xa31110a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9110a7, SUC_lexicon_tags_0 },
    { 0x49f110a8, SUC_lexicon_tags_47 },
    { 0xcc7110a7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d110b1, SUC_lexicon_tags_25 },
    { 0x66b110b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5110bb, SUC_lexicon_tags_17 },
    { 0x225110bc, SUC_lexicon_tags_37 },
    { 0x34d110bd, SUC_lexicon_tags_46 },
    { 0x697110bd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39110c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b110c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd0f110c6, SUC_lexicon_tags_37 },
    { 0xd3b110c7, SUC_lexicon_tags_12 },
    { 0x291110c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab110cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e7110cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd03110cf, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0x4bd110d1, SUC_lexicon_tags_2 },
    { 0x987110d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x23110d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b110db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x129110de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4d110e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1110e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d110e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3110ea, SUC_lexicon_tags_30 },
    { 0xf6b110eb, SUC_lexicon_tags_47 },
    { 0xf09110ec, SUC_lexicon_tags_55 },
    { 0x16d110ed, SUC_lexicon_tags_17 },
    { 0xb7b110ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef110f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe9110f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f110ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63911102, SUC_lexicon_tags_21 },
    { 0x1af11103, SUC_lexicon_tags_49 },
    { 0x9511103, SUC_lexicon_tags_60 },
    { 0xf1111104, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33511109, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9d91110b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb11113, SUC_lexicon_tags_47 },
    { 0x4a511113, SUC_lexicon_tags_29 },
    { 0xe3f11113, SUC_lexicon_tags_7 },
    { 0xe3b11114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f511118, SUC_lexicon_tags_12 },
    { 0x86d11118, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a71111d, SUC_lexicon_tags_11 },
    { 0x1dd1111d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x18511121, SUC_lexicon_tags_11 },
    { 0x44711122, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d11125, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f1112c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60511135, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b111137, SUC_lexicon_tags_15 },
    { 0xe9311137, SUC_lexicon_tags_7 },
    { 0x40711137, SUC_lexicon_tags_23 },
    { 0x8c11113a, SUC_lexicon_tags_25 },
    { 0xacf1113b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbaf1113d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf251113f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4711142, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb11146, SUC_lexicon_tags_10 },
    { 0xef11147, SUC_lexicon_tags_17 },
    { 0xe4711148, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb591114a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a91114c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d11154, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb711156, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xec511158, SUC_lexicon_tags_37 },
    { 0x5c311158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2091115b, SUC_lexicon_tags_33 },
    { 0xa2f1115b, SUC_lexicon_tags_47 },
    { 0xb251115d, SUC_lexicon_tags_38 },
    { 0xba51115c, SUC_lexicon_tags_21 },
    { 0xcf11115e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf511161, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31711168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc091116b, SUC_lexicon_tags_15 },
    { 0xebd1116b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x68b1116e, SUC_lexicon_tags_65 },
    { 0xd431116f, SUC_lexicon_tags_6 },
    { 0x9e911170, SUC_lexicon_tags_25 },
    { 0xf171116f, SUC_lexicon_tags_11 },
    { 0x3c711172, SUC_lexicon_tags_2 },
    { 0x4fd11173, SUC_lexicon_tags_0 },
    { 0xbad11172, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f11177, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e71117c, SUC_lexicon_tags_21 },
    { 0xdf91117c, SUC_lexicon_tags_33 },
    { 0xb4b1117e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11311185, SUC_lexicon_tags_37 },
    { 0x88b11186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80b11188, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42d1118a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f1118d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xded11190, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27311192, SUC_lexicon_tags_7 },
    { 0xe2f11193, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf511197, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58711199, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb951119f, SUC_lexicon_tags_37 },
    { 0x2f9111a0, SUC_lexicon_tags_37 },
    { 0x881111a1, SUC_lexicon_tags_12 },
    { 0x757111a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411111a8, SUC_lexicon_tags_53 },
    { 0x925111a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab111ad, SUC_lexicon_tags_17 },
    { 0x759111ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc63111b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693111b4, SUC_lexicon_tags_23 },
    { 0xba7111b5, SUC_lexicon_tags_33 },
    { 0xe07111b6, SUC_lexicon_tags_33 },
    { 0xab5111b6, SUC_lexicon_tags_23 },
    { 0x45b111b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21111bc, SUC_lexicon_tags_60 },
    { 0x52d111bc, SUC_lexicon_tags_26 },
    { 0x487111be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c9111c0, SUC_lexicon_tags_45 },
    { 0xab5111c1, SUC_lexicon_tags_13 },
    { 0x90b111c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3111c5, SUC_lexicon_tags_11 },
    { 0x72f111c6, SUC_lexicon_tags_47 },
    { 0x58d111c7, SUC_lexicon_tags_21 },
    { 0x6b9111c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01111cc, SUC_lexicon_tags_1 },
    { 0xa89111cd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61111d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf9111d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe83111d5, SUC_lexicon_tags_33 },
    { 0xe71111d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a1111d8, SUC_lexicon_tags_17 },
    { 0x9af111d9, SUC_lexicon_tags_7 },
    { 0x435111da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c3111dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab1111de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff111e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33b111e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df111e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe57111e8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57111ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13d111ee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b111f3, SUC_lexicon_tags_38 },
    { 0x6b1111f4, SUC_lexicon_tags_2 },
    { 0xd3b111f5, SUC_lexicon_tags_47 },
    { 0xf4b111f6, SUC_lexicon_tags_14 },
    { 0xe6b111f6, SUC_lexicon_tags_24 },
    { 0xf85111f6, SUC_lexicon_tags_60 },
    { 0xf2d111f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9b5111fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x781111fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad11200, SUC_lexicon_tags_38 },
    { 0x2511201, SUC_lexicon_tags_17 },
    { 0x16711201, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb11207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f1120b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b11120d, SUC_lexicon_tags_2 },
    { 0x8af1120d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61b11210, SUC_lexicon_tags_17 },
    { 0x22b11210, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b11214, SUC_lexicon_tags_31 },
    { 0x72d11215, SUC_lexicon_tags_65 },
    { 0x66b11215, SUC_lexicon_tags_23 },
    { 0xa9511215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91121d, SUC_lexicon_tags_8 },
    { 0x251121e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b11221, SUC_lexicon_tags_28 },
    { 0x42b11221, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6bf11224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3151122e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c911230, SUC_lexicon_tags_17 },
    { 0x42f11231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f511236, SUC_lexicon_tags_7 },
    { 0xff11237, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x84d11239, SUC_lexicon_tags_1176 },
    { 0, NULL },
    { 0x19b1123b, SUC_lexicon_tags_37 },
    { 0xd151123c, SUC_lexicon_tags_68 },
    { 0x6791123b, SUC_lexicon_tags_2 },
    { 0x6b1123e, SUC_lexicon_tags_2 },
    { 0x6b31123c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f11241, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66711245, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf511247, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1124c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a91124e, SUC_lexicon_tags_14 },
    { 0x84f1124f, SUC_lexicon_tags_14 },
    { 0xb0d1124e, SUC_lexicon_tags_17 },
    { 0x96f1124e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77111254, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39111256, SUC_lexicon_tags_15 },
    { 0xaf511256, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x94f11259, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6751125b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8311125e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f11261, SUC_lexicon_tags_1 },
    { 0xfdb11262, SUC_lexicon_tags_65 },
    { 0x9d111263, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xab311265, SUC_lexicon_tags_28 },
    { 0xa0711266, SUC_lexicon_tags_14 },
    { 0x89d11266, SUC_lexicon_tags_2 },
    { 0x81311267, SUC_lexicon_tags_2 },
    { 0xbbb11269, SUC_lexicon_tags_37 },
    { 0x41d1126a, SUC_lexicon_tags_1 },
    { 0xbc71126a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2951126e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe711272, SUC_lexicon_tags_15 },
    { 0x19111273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21111275, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9df11277, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1127d, SUC_lexicon_tags_84 },
    { 0x28b1127e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f11282, SUC_lexicon_tags_43 },
    { 0x76911283, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d511286, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25f11288, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8471128a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xba11128d, SUC_lexicon_tags_25 },
    { 0xd251128e, SUC_lexicon_tags_17 },
    { 0x2fb1128f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b11294, SUC_lexicon_tags_16 },
    { 0x97711295, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df11298, SUC_lexicon_tags_45 },
    { 0x29b11299, SUC_lexicon_tags_49 },
    { 0xcf91129a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb31129c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9112a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9112a6, SUC_lexicon_tags_11 },
    { 0xba9112a7, SUC_lexicon_tags_12 },
    { 0xfb9112a6, SUC_lexicon_tags_2 },
    { 0x387112a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4b112ab, SUC_lexicon_tags_12 },
    { 0xe37112ac, SUC_lexicon_tags_74 },
    { 0x829112ac, SUC_lexicon_tags_29 },
    { 0x6f7112ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463112b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb112bc, SUC_lexicon_tags_30 },
    { 0x1d7112bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7112c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d112c3, SUC_lexicon_tags_65 },
    { 0x849112c3, SUC_lexicon_tags_45 },
    { 0xda7112c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25112c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85112cc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x959112ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427112d3, SUC_lexicon_tags_17 },
    { 0x157112d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x187112d6, SUC_lexicon_tags_37 },
    { 0x5e1112d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27112da, SUC_lexicon_tags_25 },
    { 0x1d9112db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb112e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x21f112e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55d112e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83112ea, SUC_lexicon_tags_25 },
    { 0x8c3112eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cd112ed, SUC_lexicon_tags_47 },
    { 0x551112ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7112f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbb112f3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc05112f5, SUC_lexicon_tags_14 },
    { 0xe9b112f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x489112f8, SUC_lexicon_tags_8 },
    { 0x15112f9, SUC_lexicon_tags_16 },
    { 0x75112fa, SUC_lexicon_tags_11 },
    { 0x235112fb, SUC_lexicon_tags_47 },
    { 0x7f3112fc, SUC_lexicon_tags_37 },
    { 0x5e3112fc, SUC_lexicon_tags_1 },
    { 0x359112fd, SUC_lexicon_tags_16 },
    { 0x7dd112ff, SUC_lexicon_tags_57 },
    { 0x375112fd, SUC_lexicon_tags_7 },
    { 0x7d5112fb, SUC_lexicon_tags_27 },
    { 0x11911302, SUC_lexicon_tags_357 },
    { 0xbad11302, SUC_lexicon_tags_17 },
    { 0xe5511303, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3bd11306, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf911308, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa651130e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36511316, SUC_lexicon_tags_43 },
    { 0x4cf11317, SUC_lexicon_tags_37 },
    { 0xf2111318, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1111131c, SUC_lexicon_tags_15 },
    { 0x58f1131d, SUC_lexicon_tags_25 },
    { 0xbb31131e, SUC_lexicon_tags_25 },
    { 0x4371131e, SUC_lexicon_tags_2 },
    { 0x58d1131c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12f11322, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd11326, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa711132b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xef71132d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b91132f, SUC_lexicon_tags_61 },
    { 0x4e711330, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x34f11332, SUC_lexicon_tags_63 },
    { 0xe7b11333, SUC_lexicon_tags_32 },
    { 0xb8911334, SUC_lexicon_tags_15 },
    { 0xc9f11334, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd1133c, SUC_lexicon_tags_17 },
    { 0x5a11133d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8151133f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98111342, SUC_lexicon_tags_61 },
    { 0x10711343, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x40d11345, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5b11347, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1911349, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x44d1134b, SUC_lexicon_tags_18 },
    { 0xcd51134b, SUC_lexicon_tags_2 },
    { 0x6ff1134c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56911351, SUC_lexicon_tags_55 },
    { 0xfdf11352, SUC_lexicon_tags_17 },
    { 0x18911351, SUC_lexicon_tags_28 },
    { 0x8ed11351, SUC_lexicon_tags_21 },
    { 0x56911355, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d1135b, SUC_lexicon_tags_14 },
    { 0x9fd1135b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53911361, SUC_lexicon_tags_11 },
    { 0xa4511361, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1af11364, SUC_lexicon_tags_31 },
    { 0x7ab11365, SUC_lexicon_tags_17 },
    { 0x44111364, SUC_lexicon_tags_49 },
    { 0x1ff11364, SUC_lexicon_tags_38 },
    { 0x39111368, SUC_lexicon_tags_19 },
    { 0x54511368, SUC_lexicon_tags_25 },
    { 0xb4711369, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6631136e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44d11370, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x63911373, SUC_lexicon_tags_45 },
    { 0x57b11373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5b11376, SUC_lexicon_tags_28 },
    { 0x3f311377, SUC_lexicon_tags_14 },
    { 0x4c111378, SUC_lexicon_tags_25 },
    { 0x14111379, SUC_lexicon_tags_11 },
    { 0x1f11137a, SUC_lexicon_tags_87 },
    { 0x3a31137b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b1137e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa511383, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db11389, SUC_lexicon_tags_15 },
    { 0xe211138a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb1b1138c, SUC_lexicon_tags_15 },
    { 0x4d31138c, SUC_lexicon_tags_20 },
    { 0xf511138e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d11392, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96b11394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab311398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead1139e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1113a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5113a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403113ac, SUC_lexicon_tags_1 },
    { 0xe95113ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6b113af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28f113b1, SUC_lexicon_tags_30 },
    { 0x267113b2, SUC_lexicon_tags_31 },
    { 0x6bf113b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9113b6, SUC_lexicon_tags_37 },
    { 0x1fb113b6, SUC_lexicon_tags_19 },
    { 0xce3113b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3113bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x99d113c1, SUC_lexicon_tags_19 },
    { 0x523113c2, SUC_lexicon_tags_266 },
    { 0xa93113c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23d113c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ab113c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x571113c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d113cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe5113ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x921113d0, SUC_lexicon_tags_37 },
    { 0x551113d1, SUC_lexicon_tags_22 },
    { 0x6c5113d1, SUC_lexicon_tags_14 },
    { 0x777113d3, SUC_lexicon_tags_295 },
    { 0x7bf113d1, SUC_lexicon_tags_24 },
    { 0x573113d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c9113d7, SUC_lexicon_tags_46 },
    { 0xc1b113d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb113dd, SUC_lexicon_tags_38 },
    { 0x60f113de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad113e3, SUC_lexicon_tags_7 },
    { 0xa07113e3, SUC_lexicon_tags_31 },
    { 0xd2d113e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7113e9, SUC_lexicon_tags_2 },
    { 0x641113e9, SUC_lexicon_tags_31 },
    { 0x761113ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd113f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1113f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5113ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20311404, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11408, SUC_lexicon_tags_11 },
    { 0x83511409, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8711140b, SUC_lexicon_tags_1 },
    { 0x76b1140b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xffd1140e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2711411, SUC_lexicon_tags_11 },
    { 0x5df11411, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf11418, SUC_lexicon_tags_4 },
    { 0x2711419, SUC_lexicon_tags_11 },
    { 0xb6b1141a, SUC_lexicon_tags_30 },
    { 0x8671141b, SUC_lexicon_tags_47 },
    { 0x8db1141b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfeb1141e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f11423, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ff11425, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97111429, SUC_lexicon_tags_22 },
    { 0x9d311429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc311142e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9111431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7111438, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41b1143a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af1143e, SUC_lexicon_tags_11 },
    { 0x17d1143e, SUC_lexicon_tags_2 },
    { 0xec711440, SUC_lexicon_tags_17 },
    { 0x7511441, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x10511444, SUC_lexicon_tags_37 },
    { 0x2fd11445, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f11448, SUC_lexicon_tags_6 },
    { 0xed11449, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f1144d, SUC_lexicon_tags_37 },
    { 0xf0f1144e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b11453, SUC_lexicon_tags_11 },
    { 0x3cf11454, SUC_lexicon_tags_7 },
    { 0x67d11455, SUC_lexicon_tags_82 },
    { 0x2e111456, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b71145a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb1145e, SUC_lexicon_tags_5 },
    { 0x76b1145e, SUC_lexicon_tags_10 },
    { 0x7c511460, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd8511462, SUC_lexicon_tags_23 },
    { 0x98911463, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53911467, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xea31146a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d1146d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4511471, SUC_lexicon_tags_21 },
    { 0xc3111472, SUC_lexicon_tags_0 },
    { 0x24711473, SUC_lexicon_tags_4 },
    { 0xc5711473, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad11478, SUC_lexicon_tags_29 },
    { 0xfd11479, SUC_lexicon_tags_1 },
    { 0x44b11479, SUC_lexicon_tags_0 },
    { 0xe031147b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdd1147d, SUC_lexicon_tags_1 },
    { 0x9831147e, SUC_lexicon_tags_19 },
    { 0x8711147f, SUC_lexicon_tags_12 },
    { 0x96111480, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x13511482, SUC_lexicon_tags_7 },
    { 0xed11483, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9511486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b511488, SUC_lexicon_tags_30 },
    { 0xe8711488, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3391148b, SUC_lexicon_tags_6 },
    { 0x5ab1148c, SUC_lexicon_tags_63 },
    { 0xbbf1148d, SUC_lexicon_tags_11 },
    { 0x5131148b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf911490, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b911492, SUC_lexicon_tags_29 },
    { 0xb4f11493, SUC_lexicon_tags_49 },
    { 0x3e711493, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5671149c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3114a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15b114a2, SUC_lexicon_tags_87 },
    { 0xef7114a3, SUC_lexicon_tags_1 },
    { 0xae5114a4, SUC_lexicon_tags_15 },
    { 0x5f114a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb75114a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9114aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d9114ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb9114ae, SUC_lexicon_tags_11 },
    { 0x86d114ae, SUC_lexicon_tags_15 },
    { 0x8df114b0, SUC_lexicon_tags_11 },
    { 0x71b114b1, SUC_lexicon_tags_11 },
    { 0xa83114b2, SUC_lexicon_tags_37 },
    { 0xb09114b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb114b6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d114ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7d9114bc, SUC_lexicon_tags_25 },
    { 0xc9114bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3114c3, SUC_lexicon_tags_37 },
    { 0xc1b114c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x341114c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3114ca, SUC_lexicon_tags_23 },
    { 0xf73114ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7114ce, SUC_lexicon_tags_65 },
    { 0x509114ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1114d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x365114d5, SUC_lexicon_tags_37 },
    { 0x195114d6, SUC_lexicon_tags_25 },
    { 0x9c3114d7, SUC_lexicon_tags_25 },
    { 0xd03114d8, SUC_lexicon_tags_33 },
    { 0x7cf114d6, SUC_lexicon_tags_38 },
    { 0x721114d9, SUC_lexicon_tags_1 },
    { 0xd9f114d9, SUC_lexicon_tags_15 },
    { 0x9f7114db, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db114e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177114e7, SUC_lexicon_tags_26 },
    { 0x4d9114e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91114f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b114f4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe6b114f6, SUC_lexicon_tags_38 },
    { 0x9d5114f7, SUC_lexicon_tags_17 },
    { 0xbd5114f6, SUC_lexicon_tags_19 },
    { 0x13114f9, SUC_lexicon_tags_12 },
    { 0x26f114f9, SUC_lexicon_tags_39 },
    { 0xfd9114f6, SUC_lexicon_tags_21 },
    { 0x951114fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb114ff, SUC_lexicon_tags_2 },
    { 0xf3d11500, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e911505, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d111509, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0x6131150c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a31150f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c711513, SUC_lexicon_tags_14 },
    { 0x73911514, SUC_lexicon_tags_38 },
    { 0x62b11515, SUC_lexicon_tags_39 },
    { 0x21f11513, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9691151a, SUC_lexicon_tags_54 },
    { 0x6851151a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdb1151d, SUC_lexicon_tags_1 },
    { 0xfa91151d, SUC_lexicon_tags_37 },
    { 0x1671151e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8fb11521, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98311524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc511529, SUC_lexicon_tags_15 },
    { 0x8f51152a, SUC_lexicon_tags_43 },
    { 0x9751152a, SUC_lexicon_tags_47 },
    { 0xfe511529, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf711531, SUC_lexicon_tags_3 },
    { 0x64b11532, SUC_lexicon_tags_11 },
    { 0x7b111533, SUC_lexicon_tags_1 },
    { 0x5fd11534, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6911536, SUC_lexicon_tags_15 },
    { 0xd0d11537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb891153a, SUC_lexicon_tags_15 },
    { 0x7d1153b, SUC_lexicon_tags_1 },
    { 0xd451153a, SUC_lexicon_tags_2 },
    { 0xc631153b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d11543, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f11547, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfbb11549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3211154b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6311551, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8911556, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0711560, SUC_lexicon_tags_49 },
    { 0x51911560, SUC_lexicon_tags_10 },
    { 0xd0f11562, SUC_lexicon_tags_0 },
    { 0xbe111563, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9fb11565, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67111574, SUC_lexicon_tags_29 },
    { 0x38b11575, SUC_lexicon_tags_53 },
    { 0x90111575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf11579, SUC_lexicon_tags_17 },
    { 0x80d11579, SUC_lexicon_tags_12 },
    { 0x8df1157a, SUC_lexicon_tags_63 },
    { 0xa7911579, SUC_lexicon_tags_55 },
    { 0x2451157d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93f1157f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe0711581, SUC_lexicon_tags_8 },
    { 0xef111582, SUC_lexicon_tags_19 },
    { 0x8f711583, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e111585, SUC_lexicon_tags_10 },
    { 0xdf11586, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7cf11588, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4891158a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9351158c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca31158e, SUC_lexicon_tags_63 },
    { 0x24f1158f, SUC_lexicon_tags_2 },
    { 0x70f11590, SUC_lexicon_tags_14 },
    { 0x79d11591, SUC_lexicon_tags_30 },
    { 0x68711592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa431159c, SUC_lexicon_tags_46 },
    { 0xc171159d, SUC_lexicon_tags_15 },
    { 0xc091159e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901115a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x411115a4, SUC_lexicon_tags_1 },
    { 0xe43115a4, SUC_lexicon_tags_4 },
    { 0x35115a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb1115a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef115ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d115b0, SUC_lexicon_tags_17 },
    { 0xeb7115b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac5115b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875115b8, SUC_lexicon_tags_15 },
    { 0x779115b9, SUC_lexicon_tags_8 },
    { 0x82f115b8, SUC_lexicon_tags_11 },
    { 0x3e3115bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df115c2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x39115c5, SUC_lexicon_tags_27 },
    { 0x229115c5, SUC_lexicon_tags_19 },
    { 0x6bd115c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55115d2, SUC_lexicon_tags_14 },
    { 0x453115d3, SUC_lexicon_tags_38 },
    { 0x879115d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd115dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65d115df, SUC_lexicon_tags_15 },
    { 0x869115e0, SUC_lexicon_tags_2 },
    { 0x80d115e0, SUC_lexicon_tags_33 },
    { 0x4c5115e1, SUC_lexicon_tags_15 },
    { 0xea5115e3, SUC_lexicon_tags_14 },
    { 0x33115e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb3115e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb115ea, SUC_lexicon_tags_15 },
    { 0x6f7115eb, SUC_lexicon_tags_25 },
    { 0x8f115eb, SUC_lexicon_tags_25 },
    { 0xf21115ed, SUC_lexicon_tags_2 },
    { 0x3f9115ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc47115f0, SUC_lexicon_tags_15 },
    { 0xb3115f1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe63115f3, SUC_lexicon_tags_11 },
    { 0x8d3115f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb115f8, SUC_lexicon_tags_8 },
    { 0x81115f8, SUC_lexicon_tags_7 },
    { 0xec9115fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x867115fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8611160e, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x40711610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92111617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f1161f, SUC_lexicon_tags_16 },
    { 0x7af1161f, SUC_lexicon_tags_43 },
    { 0xce511621, SUC_lexicon_tags_45 },
    { 0x64711621, SUC_lexicon_tags_14 },
    { 0xf8d11620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7511629, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1162e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72711637, SUC_lexicon_tags_37 },
    { 0xb9d11638, SUC_lexicon_tags_14 },
    { 0x78d11639, SUC_lexicon_tags_26 },
    { 0x47d1163a, SUC_lexicon_tags_25 },
    { 0x9571163b, SUC_lexicon_tags_11 },
    { 0xc11163c, SUC_lexicon_tags_14 },
    { 0x8531163c, SUC_lexicon_tags_37 },
    { 0xe911163e, SUC_lexicon_tags_25 },
    { 0xbd111639, SUC_lexicon_tags_38 },
    { 0x32311640, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf911642, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6111646, SUC_lexicon_tags_11 },
    { 0x4d911647, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b1164b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af1164e, SUC_lexicon_tags_47 },
    { 0x56f1164e, SUC_lexicon_tags_160 },
    { 0xf391164e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd11654, SUC_lexicon_tags_15 },
    { 0xccd11655, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b11659, SUC_lexicon_tags_15 },
    { 0x34f1165a, SUC_lexicon_tags_15 },
    { 0x4a711659, SUC_lexicon_tags_11 },
    { 0xab11165c, SUC_lexicon_tags_2 },
    { 0x1791165d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82311660, SUC_lexicon_tags_63 },
    { 0xd0911660, SUC_lexicon_tags_47 },
    { 0x6cf11662, SUC_lexicon_tags_26 },
    { 0x4bf11660, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe2311665, SUC_lexicon_tags_23 },
    { 0xc4311666, SUC_lexicon_tags_32 },
    { 0xbd111666, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb671166b, SUC_lexicon_tags_8 },
    { 0xf5d1166c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeeb1166e, SUC_lexicon_tags_14 },
    { 0x9991166f, SUC_lexicon_tags_7 },
    { 0x86311670, SUC_lexicon_tags_0 },
    { 0x12311671, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x2b311673, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfef11675, SUC_lexicon_tags_15 },
    { 0x59311676, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3af11679, SUC_lexicon_tags_28 },
    { 0x8b51167a, SUC_lexicon_tags_25 },
    { 0xa051167a, SUC_lexicon_tags_14 },
    { 0x23911679, SUC_lexicon_tags_21 },
    { 0xf051167b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b711681, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d11686, SUC_lexicon_tags_7 },
    { 0x25111687, SUC_lexicon_tags_2 },
    { 0xdd111688, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x951168a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5871168f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb111697, SUC_lexicon_tags_37 },
    { 0x3ed11697, SUC_lexicon_tags_1 },
    { 0x5db11699, SUC_lexicon_tags_18 },
    { 0x79911699, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7951169e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x393116a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131116b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13116b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd116b6, SUC_lexicon_tags_5 },
    { 0xc57116b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617116be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf6b116c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9116c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa1116c6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef116cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65116d1, SUC_lexicon_tags_25 },
    { 0x3cb116d1, SUC_lexicon_tags_6 },
    { 0x963116d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f116d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63116db, SUC_lexicon_tags_55 },
    { 0x551116dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9116df, SUC_lexicon_tags_37 },
    { 0x8c5116df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xabd116e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29116eb, SUC_lexicon_tags_12 },
    { 0x623116ec, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x987116ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b116f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5116f4, SUC_lexicon_tags_6 },
    { 0xc5f116f5, SUC_lexicon_tags_62 },
    { 0x7af116f5, SUC_lexicon_tags_19 },
    { 0x353116f6, SUC_lexicon_tags_60 },
    { 0xecf116f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddd116fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7116fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5911708, SUC_lexicon_tags_11 },
    { 0x77111709, SUC_lexicon_tags_25 },
    { 0x91f1170a, SUC_lexicon_tags_56 },
    { 0x7db11709, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e31170d, SUC_lexicon_tags_2 },
    { 0x3171170d, SUC_lexicon_tags_11 },
    { 0x3491170f, SUC_lexicon_tags_26 },
    { 0x95d11710, SUC_lexicon_tags_12 },
    { 0x63111711, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x84911713, SUC_lexicon_tags_15 },
    { 0x5c111714, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c911716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2db11718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5891171a, SUC_lexicon_tags_2 },
    { 0xb6d1171b, SUC_lexicon_tags_43 },
    { 0xe651171a, SUC_lexicon_tags_31 },
    { 0xfb31171c, SUC_lexicon_tags_1 },
    { 0x87d1171e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2111721, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94711724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8911726, SUC_lexicon_tags_15 },
    { 0x77d11727, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x82711729, SUC_lexicon_tags_29 },
    { 0x71511729, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6311172c, SUC_lexicon_tags_11 },
    { 0x3fd1172c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1711730, SUC_lexicon_tags_62 },
    { 0xaa711730, SUC_lexicon_tags_1 },
    { 0x38d11730, SUC_lexicon_tags_60 },
    { 0x72111731, SUC_lexicon_tags_38 },
    { 0x6111731, SUC_lexicon_tags_37 },
    { 0xce711735, SUC_lexicon_tags_7 },
    { 0x71911736, SUC_lexicon_tags_30 },
    { 0x17311735, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1711740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b11748, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d1174c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe511174f, SUC_lexicon_tags_12 },
    { 0x6e711750, SUC_lexicon_tags_66 },
    { 0x20711751, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x94711753, SUC_lexicon_tags_15 },
    { 0x9e111753, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1911762, SUC_lexicon_tags_19 },
    { 0x73b11762, SUC_lexicon_tags_27 },
    { 0xc1d11762, SUC_lexicon_tags_47 },
    { 0x6e111764, SUC_lexicon_tags_61 },
    { 0x2c511765, SUC_lexicon_tags_16 },
    { 0x59b11767, SUC_lexicon_tags_25 },
    { 0xb5b11767, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccd1176a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb31176c, SUC_lexicon_tags_37 },
    { 0x72b1176d, SUC_lexicon_tags_11 },
    { 0x4f51176e, SUC_lexicon_tags_13 },
    { 0xd7d1176f, SUC_lexicon_tags_17 },
    { 0xabd11770, SUC_lexicon_tags_11 },
    { 0xaf71176f, SUC_lexicon_tags_28 },
    { 0xf3311772, SUC_lexicon_tags_1 },
    { 0x4c711773, SUC_lexicon_tags_1 },
    { 0x9991176c, SUC_lexicon_tags_21 },
    { 0x6eb11775, SUC_lexicon_tags_4 },
    { 0x76511776, SUC_lexicon_tags_15 },
    { 0x8b711774, SUC_lexicon_tags_10 },
    { 0x19111778, SUC_lexicon_tags_14 },
    { 0x45311778, SUC_lexicon_tags_13 },
    { 0xcbb11777, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1851177e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef911781, SUC_lexicon_tags_39 },
    { 0xb4111782, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d11786, SUC_lexicon_tags_25 },
    { 0x55d11786, SUC_lexicon_tags_25 },
    { 0xe1711786, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xf531178b, SUC_lexicon_tags_66 },
    { 0x71d1178c, SUC_lexicon_tags_25 },
    { 0xa971178b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd111791, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10d11793, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2b11795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d11799, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167117a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9117a6, SUC_lexicon_tags_7 },
    { 0xa39117a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd79117a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x235117ab, SUC_lexicon_tags_38 },
    { 0x901117ac, SUC_lexicon_tags_39 },
    { 0xd9117ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89117b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb117b3, SUC_lexicon_tags_7 },
    { 0x717117b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd59117b6, SUC_lexicon_tags_33 },
    { 0x389117b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57117c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0d117c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x545117c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed117c9, SUC_lexicon_tags_60 },
    { 0x159117ca, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527117d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83117d7, SUC_lexicon_tags_57 },
    { 0x26b117d7, SUC_lexicon_tags_8 },
    { 0xddb117d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d117df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31117ea, SUC_lexicon_tags_1 },
    { 0x157117ea, SUC_lexicon_tags_37 },
    { 0x709117ec, SUC_lexicon_tags_25 },
    { 0xc17117ed, SUC_lexicon_tags_33 },
    { 0xd8d117ee, SUC_lexicon_tags_60 },
    { 0x8e3117ee, SUC_lexicon_tags_26 },
    { 0x2b3117ed, SUC_lexicon_tags_15 },
    { 0x6bf117f1, SUC_lexicon_tags_17 },
    { 0x46d117f1, SUC_lexicon_tags_38 },
    { 0xb13117f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1117f6, SUC_lexicon_tags_154 },
    { 0xec7117f6, SUC_lexicon_tags_14 },
    { 0xfdf117f8, SUC_lexicon_tags_17 },
    { 0xb7b117f9, SUC_lexicon_tags_148 },
    { 0x891117f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733117fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb11801, SUC_lexicon_tags_43 },
    { 0x18711802, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b11808, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b1180d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc911810, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b11813, SUC_lexicon_tags_87 },
    { 0x18511814, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6eb11816, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d31181c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8911825, SUC_lexicon_tags_47 },
    { 0xd3b11826, SUC_lexicon_tags_63 },
    { 0x68f11827, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d1182b, SUC_lexicon_tags_37 },
    { 0xcb31182b, SUC_lexicon_tags_25 },
    { 0x2f1182d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b11831, SUC_lexicon_tags_25 },
    { 0x26111832, SUC_lexicon_tags_14 },
    { 0xee511832, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f11836, SUC_lexicon_tags_15 },
    { 0xdf911837, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd991183a, SUC_lexicon_tags_25 },
    { 0x5f91183b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3371183e, SUC_lexicon_tags_11 },
    { 0x2d11183f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x81311842, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d11846, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x98711848, SUC_lexicon_tags_4 },
    { 0xa0d11849, SUC_lexicon_tags_14 },
    { 0x8f31184a, SUC_lexicon_tags_27 },
    { 0x1f11184b, SUC_lexicon_tags_84 },
    { 0x6d91184c, SUC_lexicon_tags_6 },
    { 0x9f71184a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6a11184f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33111852, SUC_lexicon_tags_37 },
    { 0x7dd11852, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b11858, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2851185f, SUC_lexicon_tags_33 },
    { 0xe6511860, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f311866, SUC_lexicon_tags_30 },
    { 0x32311867, SUC_lexicon_tags_47 },
    { 0x23b11868, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1051186d, SUC_lexicon_tags_29 },
    { 0xc091186d, SUC_lexicon_tags_31 },
    { 0xe0b1186d, SUC_lexicon_tags_14 },
    { 0x6431186f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f11873, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x59f11875, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe511878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe811187a, SUC_lexicon_tags_14 },
    { 0xaef1187b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99911882, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b11888, SUC_lexicon_tags_49 },
    { 0xd711889, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d1188d, SUC_lexicon_tags_10 },
    { 0x3df1188d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3111890, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af11893, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a511895, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad1189a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d118a3, SUC_lexicon_tags_47 },
    { 0xe51118a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbcd118a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x453118a9, SUC_lexicon_tags_25 },
    { 0x2f1118a9, SUC_lexicon_tags_7 },
    { 0xc1d118a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x525118ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12d118af, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1118b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d9118b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3118b7, SUC_lexicon_tags_17 },
    { 0xe5b118b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d118bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcef118bd, SUC_lexicon_tags_19 },
    { 0x8cd118be, SUC_lexicon_tags_28 },
    { 0x8c7118be, SUC_lexicon_tags_871 },
    { 0x95118bd, SUC_lexicon_tags_37 },
    { 0x3df118c1, SUC_lexicon_tags_32 },
    { 0xb7f118c2, SUC_lexicon_tags_31 },
    { 0x91f118bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3118ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x375118d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d118d5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4ff118d7, SUC_lexicon_tags_15 },
    { 0xb65118d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9118db, SUC_lexicon_tags_16 },
    { 0x525118dc, SUC_lexicon_tags_49 },
    { 0x1f7118db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f118e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad118e8, SUC_lexicon_tags_23 },
    { 0xd9f118e8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x3ab118eb, SUC_lexicon_tags_15 },
    { 0x525118ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1118f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xecf118f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe27118f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b11901, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d11903, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd4911905, SUC_lexicon_tags_38 },
    { 0x3ed11905, SUC_lexicon_tags_43 },
    { 0x11511907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa711909, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4891190b, SUC_lexicon_tags_7 },
    { 0xe691190c, SUC_lexicon_tags_167 },
    { 0xc251190d, SUC_lexicon_tags_21 },
    { 0x25f1190e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f111910, SUC_lexicon_tags_23 },
    { 0xc1d11911, SUC_lexicon_tags_32 },
    { 0xc7311911, SUC_lexicon_tags_2 },
    { 0xb0311912, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8f511915, SUC_lexicon_tags_47 },
    { 0x12111916, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b1191a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b511923, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xff511926, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x69311928, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e11192b, SUC_lexicon_tags_39 },
    { 0xa3b1192c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b91192f, SUC_lexicon_tags_23 },
    { 0x66911930, SUC_lexicon_tags_0 },
    { 0xd0f11931, SUC_lexicon_tags_25 },
    { 0x4fd11932, SUC_lexicon_tags_19 },
    { 0x34111932, SUC_lexicon_tags_43 },
    { 0xba511934, SUC_lexicon_tags_17 },
    { 0xadf11935, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b71193b, SUC_lexicon_tags_14 },
    { 0x7271193b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc091193e, SUC_lexicon_tags_34 },
    { 0x5bf1193f, SUC_lexicon_tags_45 },
    { 0x7f911940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d511946, SUC_lexicon_tags_16 },
    { 0x2fb11946, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa31194c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaff1194e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a911952, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd11955, SUC_lexicon_tags_27 },
    { 0xc0f11956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9911958, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2431195d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce31195f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2111963, SUC_lexicon_tags_23 },
    { 0xf4911963, SUC_lexicon_tags_45 },
    { 0x35711963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f11968, SUC_lexicon_tags_17 },
    { 0xdbb11969, SUC_lexicon_tags_21 },
    { 0x80111969, SUC_lexicon_tags_34 },
    { 0x71d11969, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa291196d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6e91196f, SUC_lexicon_tags_25 },
    { 0xcf311970, SUC_lexicon_tags_31 },
    { 0xc5711971, SUC_lexicon_tags_25 },
    { 0x6b911972, SUC_lexicon_tags_17 },
    { 0x75511971, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6f11975, SUC_lexicon_tags_0 },
    { 0x9d311976, SUC_lexicon_tags_45 },
    { 0xedb11977, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff1197c, SUC_lexicon_tags_11 },
    { 0x11b1197c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c711980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16111982, SUC_lexicon_tags_17 },
    { 0xdcd11983, SUC_lexicon_tags_39 },
    { 0x17f11984, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd11989, SUC_lexicon_tags_25 },
    { 0xee111989, SUC_lexicon_tags_29 },
    { 0xb411198a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed11198f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3711995, SUC_lexicon_tags_62 },
    { 0x3af11995, SUC_lexicon_tags_7 },
    { 0x5f11997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d711999, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f1199c, SUC_lexicon_tags_14 },
    { 0x4011199c, SUC_lexicon_tags_42 },
    { 0x1751199d, SUC_lexicon_tags_60 },
    { 0x9711199f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x965119a1, SUC_lexicon_tags_37 },
    { 0xdef119a1, SUC_lexicon_tags_25 },
    { 0xc71119a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9119a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x773119ac, SUC_lexicon_tags_32 },
    { 0x4db119ad, SUC_lexicon_tags_1 },
    { 0x81119ae, SUC_lexicon_tags_12 },
    { 0x783119af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5119b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43119bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61119c3, SUC_lexicon_tags_26 },
    { 0xf57119c4, SUC_lexicon_tags_1 },
    { 0x8cb119c4, SUC_lexicon_tags_41 },
    { 0x5c3119c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x919119c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa1119ca, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xd97119cc, SUC_lexicon_tags_15 },
    { 0x4c9119cc, SUC_lexicon_tags_0 },
    { 0x50b119cd, SUC_lexicon_tags_65 },
    { 0xd51119cf, SUC_lexicon_tags_11 },
    { 0x62b119cf, SUC_lexicon_tags_17 },
    { 0x33b119ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd119d5, SUC_lexicon_tags_6 },
    { 0x5bf119d6, SUC_lexicon_tags_39 },
    { 0x8ef119d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17119de, SUC_lexicon_tags_16 },
    { 0x31b119df, SUC_lexicon_tags_37 },
    { 0xf29119e0, SUC_lexicon_tags_16 },
    { 0x80b119de, SUC_lexicon_tags_1 },
    { 0x8bd119e2, SUC_lexicon_tags_11 },
    { 0x323119e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57119eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x63b119ed, SUC_lexicon_tags_11 },
    { 0x835119ed, SUC_lexicon_tags_12 },
    { 0xa8b119ef, SUC_lexicon_tags_34 },
    { 0x583119ed, SUC_lexicon_tags_37 },
    { 0xbfd119f0, SUC_lexicon_tags_55 },
    { 0xb73119ef, SUC_lexicon_tags_35 },
    { 0x1db119f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x561119f5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9119f9, SUC_lexicon_tags_15 },
    { 0x7f7119fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b111a00, SUC_lexicon_tags_12 },
    { 0xe3511a01, SUC_lexicon_tags_65 },
    { 0xfa911a02, SUC_lexicon_tags_62 },
    { 0xf6711a01, SUC_lexicon_tags_16 },
    { 0xb8b11a04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89111a08, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c711a0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb511a11, SUC_lexicon_tags_7 },
    { 0x11311a12, SUC_lexicon_tags_15 },
    { 0xa7b11a13, SUC_lexicon_tags_28 },
    { 0xd111a14, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb11a18, SUC_lexicon_tags_28 },
    { 0x8d11a19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2911a1c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f711a1f, SUC_lexicon_tags_17 },
    { 0x59911a20, SUC_lexicon_tags_17 },
    { 0x2e511a1f, SUC_lexicon_tags_39 },
    { 0x3eb11a22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef11a25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b711a29, SUC_lexicon_tags_2 },
    { 0xd111a2a, SUC_lexicon_tags_12 },
    { 0x8c911a2a, SUC_lexicon_tags_160 },
    { 0x77911a2a, SUC_lexicon_tags_14 },
    { 0xe8311a2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38511a2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f111a31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb11a35, SUC_lexicon_tags_14 },
    { 0x7911a36, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd111a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d511a3c, SUC_lexicon_tags_11 },
    { 0x48b11a3d, SUC_lexicon_tags_17 },
    { 0x19911a3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96111a42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50d11a44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9711a47, SUC_lexicon_tags_11 },
    { 0x3c711a47, SUC_lexicon_tags_37 },
    { 0xa0f11a49, SUC_lexicon_tags_2 },
    { 0xdf11a4a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9dd11a4c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9911a50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5cb11a52, SUC_lexicon_tags_17 },
    { 0x18d11a52, SUC_lexicon_tags_15 },
    { 0xfcb11a54, SUC_lexicon_tags_7 },
    { 0x54f11a55, SUC_lexicon_tags_39 },
    { 0xf6911a56, SUC_lexicon_tags_1 },
    { 0x63911a52, SUC_lexicon_tags_12 },
    { 0x6e911a57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4511a5a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x65311a5c, SUC_lexicon_tags_26 },
    { 0x58111a5d, SUC_lexicon_tags_26 },
    { 0x7d111a5e, SUC_lexicon_tags_33 },
    { 0x66f11a5e, SUC_lexicon_tags_7 },
    { 0x6dd11a5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46711a67, SUC_lexicon_tags_1 },
    { 0x75d11a67, SUC_lexicon_tags_37 },
    { 0xf0311a69, SUC_lexicon_tags_35 },
    { 0x35b11a6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x41911a6c, SUC_lexicon_tags_12 },
    { 0xbfd11a6c, SUC_lexicon_tags_33 },
    { 0xd0311a6c, SUC_lexicon_tags_1 },
    { 0x39f11a6f, SUC_lexicon_tags_33 },
    { 0xfb311a70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb11a74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75711a76, SUC_lexicon_tags_2 },
    { 0xc4511a76, SUC_lexicon_tags_11 },
    { 0xa8711a78, SUC_lexicon_tags_38 },
    { 0x83d11a79, SUC_lexicon_tags_26 },
    { 0xff911a78, SUC_lexicon_tags_11 },
    { 0xaa511a7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58911a7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb11a81, SUC_lexicon_tags_55 },
    { 0xa6111a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7311a89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb311a8b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd4911a8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9f11a8f, SUC_lexicon_tags_15 },
    { 0xea911a8f, SUC_lexicon_tags_11 },
    { 0xf2311a91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9711a94, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d11a99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d311a9b, SUC_lexicon_tags_23 },
    { 0x80d11a9c, SUC_lexicon_tags_25 },
    { 0xe8111a9d, SUC_lexicon_tags_28 },
    { 0x5bf11a9e, SUC_lexicon_tags_12 },
    { 0x85d11a9f, SUC_lexicon_tags_36 },
    { 0x4cd11a9f, SUC_lexicon_tags_29 },
    { 0x76b11a9f, SUC_lexicon_tags_1 },
    { 0x9cf11aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f911aa8, SUC_lexicon_tags_23 },
    { 0x48b11aa9, SUC_lexicon_tags_7 },
    { 0x79f11aaa, SUC_lexicon_tags_12 },
    { 0x8c111aab, SUC_lexicon_tags_14 },
    { 0xf7511aab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6111ab3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fd11ab5, SUC_lexicon_tags_83 },
    { 0xce511ab5, SUC_lexicon_tags_10 },
    { 0xa0311ab7, SUC_lexicon_tags_230 },
    { 0x86311ab8, SUC_lexicon_tags_30 },
    { 0x62311ab8, SUC_lexicon_tags_14 },
    { 0x5f911ab5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad11abd, SUC_lexicon_tags_19 },
    { 0x65911abd, SUC_lexicon_tags_11 },
    { 0x42711abf, SUC_lexicon_tags_7 },
    { 0x25f11abf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78f11ac2, SUC_lexicon_tags_75 },
    { 0xb711ac3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56d11ac5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x65511ac8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d11acd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x15711ad0, SUC_lexicon_tags_37 },
    { 0x5e111ad0, SUC_lexicon_tags_7 },
    { 0xa111ad2, SUC_lexicon_tags_17 },
    { 0x11b11ad3, SUC_lexicon_tags_1 },
    { 0x2e711ad4, SUC_lexicon_tags_87 },
    { 0x31911ad5, SUC_lexicon_tags_17 },
    { 0x7b511ad6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8511ada, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1111adc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef11adf, SUC_lexicon_tags_7 },
    { 0xbe511ae0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db11ae3, SUC_lexicon_tags_17 },
    { 0x9db11ae3, SUC_lexicon_tags_0 },
    { 0x2bb11ae5, SUC_lexicon_tags_12 },
    { 0xa4311ae6, SUC_lexicon_tags_25 },
    { 0xae111ae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b11aeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0111aed, SUC_lexicon_tags_37 },
    { 0xd4d11aee, SUC_lexicon_tags_62 },
    { 0x3ff11aee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac111af1, SUC_lexicon_tags_2 },
    { 0xce111af2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd111aff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90911b01, SUC_lexicon_tags_37 },
    { 0x38511b02, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9911b09, SUC_lexicon_tags_38 },
    { 0xf4b11b0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f911b0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99511b12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed11b15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af11b18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4f11b1a, SUC_lexicon_tags_15 },
    { 0x8a711b1b, SUC_lexicon_tags_194 },
    { 0x5711b1c, SUC_lexicon_tags_4 },
    { 0x37f11b1d, SUC_lexicon_tags_104 },
    { 0xc6111b1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38311b24, SUC_lexicon_tags_17 },
    { 0x56311b24, SUC_lexicon_tags_63 },
    { 0x47911b24, SUC_lexicon_tags_77 },
    { 0x84911b26, SUC_lexicon_tags_11 },
    { 0xc6311b26, SUC_lexicon_tags_11 },
    { 0xf0b11b24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cb11b2b, SUC_lexicon_tags_46 },
    { 0x68f11b2b, SUC_lexicon_tags_7 },
    { 0x4d911b2d, SUC_lexicon_tags_27 },
    { 0xba911b2e, SUC_lexicon_tags_7 },
    { 0x83511b2e, SUC_lexicon_tags_47 },
    { 0x8a311b2f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd11b34, SUC_lexicon_tags_18 },
    { 0x3311b35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x12f11b37, SUC_lexicon_tags_11 },
    { 0x5311b38, SUC_lexicon_tags_37 },
    { 0x97111b39, SUC_lexicon_tags_3 },
    { 0xe6b11b39, SUC_lexicon_tags_25 },
    { 0x2d511b3b, SUC_lexicon_tags_30 },
    { 0x4d311b3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b511b3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af11b44, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f11b4a, SUC_lexicon_tags_57 },
    { 0x58f11b4a, SUC_lexicon_tags_37 },
    { 0x86d11b4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda911b4f, SUC_lexicon_tags_498 },
    { 0x48911b50, SUC_lexicon_tags_37 },
    { 0xd4f11b51, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61511b59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe711b5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x24f11b5f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9cb11b61, SUC_lexicon_tags_3 },
    { 0xbb311b61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xff311b65, SUC_lexicon_tags_0 },
    { 0x29111b66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62311b6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x22911b6d, SUC_lexicon_tags_1 },
    { 0x82311b6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15111b72, SUC_lexicon_tags_30 },
    { 0x61b11b72, SUC_lexicon_tags_19 },
    { 0xec311b72, SUC_lexicon_tags_37 },
    { 0xcf111b73, SUC_lexicon_tags_47 },
    { 0x88111b72, SUC_lexicon_tags_15 },
    { 0x75111b77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7711b7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40d11b81, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b11b87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc2b11b89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b911b8d, SUC_lexicon_tags_23 },
    { 0xfdd11b8d, SUC_lexicon_tags_66 },
    { 0xdc311b8d, SUC_lexicon_tags_43 },
    { 0x3a911b90, SUC_lexicon_tags_4 },
    { 0x59111b91, SUC_lexicon_tags_6 },
    { 0xb0711b90, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31711b99, SUC_lexicon_tags_12 },
    { 0xfeb11b9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14511b9d, SUC_lexicon_tags_37 },
    { 0x86b11b9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14311ba0, SUC_lexicon_tags_17 },
    { 0x50f11ba1, SUC_lexicon_tags_7 },
    { 0x16711ba1, SUC_lexicon_tags_25 },
    { 0x45d11ba3, SUC_lexicon_tags_25 },
    { 0x43311ba4, SUC_lexicon_tags_26 },
    { 0x66911ba5, SUC_lexicon_tags_28 },
    { 0x78d11ba6, SUC_lexicon_tags_13 },
    { 0x8d511ba7, SUC_lexicon_tags_63 },
    { 0x3f511ba0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43311bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82511bb5, SUC_lexicon_tags_2 },
    { 0x6ad11bb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f11bb9, SUC_lexicon_tags_2 },
    { 0xd1f11bba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x39711bbc, SUC_lexicon_tags_37 },
    { 0x46111bbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27711bc1, SUC_lexicon_tags_7 },
    { 0x63111bc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0f11bc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf911bc6, SUC_lexicon_tags_148 },
    { 0x7cb11bc7, SUC_lexicon_tags_38 },
    { 0xfb711bc8, SUC_lexicon_tags_2 },
    { 0xcdf11bc8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9911bcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61911bd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6311bd6, SUC_lexicon_tags_29 },
    { 0x7a311bd7, SUC_lexicon_tags_9 },
    { 0x2a111bd7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xaed11bda, SUC_lexicon_tags_8 },
    { 0x19d11bdb, SUC_lexicon_tags_66 },
    { 0x9cb11bdc, SUC_lexicon_tags_36 },
    { 0x8ab11bdc, SUC_lexicon_tags_1 },
    { 0x3a111bde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76511be3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5511be6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72111be8, SUC_lexicon_tags_505 },
    { 0xe9511be8, SUC_lexicon_tags_26 },
    { 0xe5d11bea, SUC_lexicon_tags_82 },
    { 0x8b711bea, SUC_lexicon_tags_47 },
    { 0x7d511beb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced11bf1, SUC_lexicon_tags_37 },
    { 0x3eb11bf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37911bf4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b11bf9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1dd11bfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0911bfd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1d111bff, SUC_lexicon_tags_25 },
    { 0xa8511c00, SUC_lexicon_tags_2 },
    { 0x78f11bff, SUC_lexicon_tags_49 },
    { 0x40911bff, SUC_lexicon_tags_38 },
    { 0x4df11c03, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d511c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf711c09, SUC_lexicon_tags_14 },
    { 0xf511c0a, SUC_lexicon_tags_1 },
    { 0xd3311c0a, SUC_lexicon_tags_60 },
    { 0x11111c0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd311c0e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec311c14, SUC_lexicon_tags_10 },
    { 0x15311c15, SUC_lexicon_tags_25 },
    { 0x10511c14, SUC_lexicon_tags_7 },
    { 0x5711c16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b11c1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47311c1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75111c20, SUC_lexicon_tags_15 },
    { 0xb9511c21, SUC_lexicon_tags_60 },
    { 0xdf111c21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d11c25, SUC_lexicon_tags_6 },
    { 0x2a511c26, SUC_lexicon_tags_63 },
    { 0xf8511c26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3511c29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab111c2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc111c2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47111c31, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d111c35, SUC_lexicon_tags_25 },
    { 0x39911c36, SUC_lexicon_tags_31 },
    { 0xf0711c36, SUC_lexicon_tags_11 },
    { 0xdcf11c35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b111c3c, SUC_lexicon_tags_0 },
    { 0xf7111c3d, SUC_lexicon_tags_17 },
    { 0x91711c3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed11c45, SUC_lexicon_tags_43 },
    { 0x18911c45, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87911c4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfa511c4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x18b11c4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf9911c50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1711c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca711c54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf3d11c56, SUC_lexicon_tags_87 },
    { 0x23b11c57, SUC_lexicon_tags_17 },
    { 0x2dd11c58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca711c5a, SUC_lexicon_tags_31 },
    { 0x8fb11c5b, SUC_lexicon_tags_8 },
    { 0x95111c5c, SUC_lexicon_tags_17 },
    { 0x43d11c5c, SUC_lexicon_tags_49 },
    { 0x35511c5c, SUC_lexicon_tags_7 },
    { 0xee911c5f, SUC_lexicon_tags_17 },
    { 0xbf111c60, SUC_lexicon_tags_16 },
    { 0x30f11c61, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66311c67, SUC_lexicon_tags_7 },
    { 0xc7f11c68, SUC_lexicon_tags_57 },
    { 0xc1111c69, SUC_lexicon_tags_13 },
    { 0x6b11c68, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46311c70, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c511c73, SUC_lexicon_tags_37 },
    { 0x24111c74, SUC_lexicon_tags_7 },
    { 0x80311c75, SUC_lexicon_tags_15 },
    { 0x6d911c75, SUC_lexicon_tags_28 },
    { 0xb9111c76, SUC_lexicon_tags_47 },
    { 0x9ed11c76, SUC_lexicon_tags_39 },
    { 0xce711c75, SUC_lexicon_tags_12 },
    { 0x89f11c76, SUC_lexicon_tags_38 },
    { 0x2af11c7b, SUC_lexicon_tags_47 },
    { 0x33d11c7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x95311c7f, SUC_lexicon_tags_12 },
    { 0x5c311c80, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x34711c82, SUC_lexicon_tags_11 },
    { 0x97111c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13911c88, SUC_lexicon_tags_11 },
    { 0xdf11c88, SUC_lexicon_tags_28 },
    { 0xec311c8a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x7cd11c8c, SUC_lexicon_tags_99 },
    { 0x9c511c8c, SUC_lexicon_tags_57 },
    { 0x1911c8e, SUC_lexicon_tags_7 },
    { 0x84111c8e, SUC_lexicon_tags_1 },
    { 0xa4f11c90, SUC_lexicon_tags_1 },
    { 0x9a111c90, SUC_lexicon_tags_63 },
    { 0x1a111c8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd911c96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb111c9c, SUC_lexicon_tags_23 },
    { 0x57b11c9c, SUC_lexicon_tags_17 },
    { 0x59b11c9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36d11ca0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe711ca4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b11ca7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb711cab, SUC_lexicon_tags_7 },
    { 0x7ef11cab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b11cb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67711cb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb911cbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2711cc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3c311cc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a911cc7, SUC_lexicon_tags_57 },
    { 0x47111cc7, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b11cce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6d11cd0, SUC_lexicon_tags_11 },
    { 0x59511cd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad11cd4, SUC_lexicon_tags_47 },
    { 0xa2711cd5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb7f11cd7, SUC_lexicon_tags_15 },
    { 0x7b511cd7, SUC_lexicon_tags_29 },
    { 0x14f11cd9, SUC_lexicon_tags_15 },
    { 0xa1511cda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce911cdf, SUC_lexicon_tags_27 },
    { 0x18d11ce0, SUC_lexicon_tags_18 },
    { 0x1f911ce0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f11ce8, SUC_lexicon_tags_47 },
    { 0x80f11ce9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0311cee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd11cf1, SUC_lexicon_tags_17 },
    { 0xd1111cf1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b711cf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b11cfa, SUC_lexicon_tags_12 },
    { 0x67d11cfb, SUC_lexicon_tags_37 },
    { 0x98111cfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24b11cfe, SUC_lexicon_tags_16 },
    { 0xc8b11cff, SUC_lexicon_tags_25 },
    { 0xfa111d00, SUC_lexicon_tags_17 },
    { 0x58911d01, SUC_lexicon_tags_17 },
    { 0x75911cff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d11d09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb711d0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57111d15, SUC_lexicon_tags_12 },
    { 0x65f11d16, SUC_lexicon_tags_25 },
    { 0xcab11d17, SUC_lexicon_tags_28 },
    { 0xb6311d17, SUC_lexicon_tags_17 },
    { 0x79111d18, SUC_lexicon_tags_17 },
    { 0x4d11d16, SUC_lexicon_tags_21 },
    { 0xf8511d1b, SUC_lexicon_tags_9 },
    { 0xb711d16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b111d1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d511d21, SUC_lexicon_tags_1 },
    { 0xb111d22, SUC_lexicon_tags_11 },
    { 0x69d11d23, SUC_lexicon_tags_4 },
    { 0x9e711d22, SUC_lexicon_tags_15 },
    { 0xde711d22, SUC_lexicon_tags_47 },
    { 0xbb511d25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97311d2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89111d30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab11d33, SUC_lexicon_tags_82 },
    { 0xbf11d34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x85511d37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0111d3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c511d3e, SUC_lexicon_tags_2 },
    { 0x25311d3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7511d42, SUC_lexicon_tags_15 },
    { 0xd3b11d43, SUC_lexicon_tags_79 },
    { 0xf3311d43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c911d4e, SUC_lexicon_tags_15 },
    { 0xc1711d4f, SUC_lexicon_tags_71 },
    { 0x6fd11d4e, SUC_lexicon_tags_55 },
    { 0x3fd11d4f, SUC_lexicon_tags_36 },
    { 0x3af11d51, SUC_lexicon_tags_45 },
    { 0x7dd11d53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa8f11d55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f111d57, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x63311d5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35511d60, SUC_lexicon_tags_27 },
    { 0x73f11d61, SUC_lexicon_tags_11 },
    { 0x90911d62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61711d65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2c111d67, SUC_lexicon_tags_43 },
    { 0x22b11d68, SUC_lexicon_tags_32 },
    { 0xbd911d69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13111d6d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef11d72, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e311d75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d511d78, SUC_lexicon_tags_0 },
    { 0xc9d11d79, SUC_lexicon_tags_19 },
    { 0x5db11d7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4311d7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f711d7f, SUC_lexicon_tags_11 },
    { 0x74111d80, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a511d85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x84b11d87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0311d89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b11d8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8111d90, SUC_lexicon_tags_68 },
    { 0x87311d91, SUC_lexicon_tags_38 },
    { 0xdc911d92, SUC_lexicon_tags_37 },
    { 0xa1111d93, SUC_lexicon_tags_23 },
    { 0xa5511d94, SUC_lexicon_tags_47 },
    { 0xbd111d95, SUC_lexicon_tags_0 },
    { 0xfef11d90, SUC_lexicon_tags_7 },
    { 0x2d911d97, SUC_lexicon_tags_49 },
    { 0xf6311d93, SUC_lexicon_tags_2 },
    { 0x3ab11d99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f11d9b, SUC_lexicon_tags_47 },
    { 0xf4711d9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x53711d9e, SUC_lexicon_tags_37 },
    { 0xd6511d9e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x61911da1, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a511da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59911da6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc511da8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x11111daa, SUC_lexicon_tags_25 },
    { 0x46911dab, SUC_lexicon_tags_29 },
    { 0x4df11dac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96311db5, SUC_lexicon_tags_12 },
    { 0x97911db6, SUC_lexicon_tags_45 },
    { 0x46d11db7, SUC_lexicon_tags_25 },
    { 0xa9d11db8, SUC_lexicon_tags_17 },
    { 0xf6511db9, SUC_lexicon_tags_49 },
    { 0x73d11db6, SUC_lexicon_tags_47 },
    { 0x52911dbb, SUC_lexicon_tags_17 },
    { 0x3d11dbc, SUC_lexicon_tags_19 },
    { 0xd7f11dbd, SUC_lexicon_tags_37 },
    { 0x43b11dbd, SUC_lexicon_tags_33 },
    { 0xb2711dbf, SUC_lexicon_tags_37 },
    { 0xf9f11dc0, SUC_lexicon_tags_17 },
    { 0x50911dc1, SUC_lexicon_tags_17 },
    { 0xc5111dc2, SUC_lexicon_tags_33 },
    { 0xca311dbe, SUC_lexicon_tags_25 },
    { 0x8a111dc4, SUC_lexicon_tags_13 },
    { 0x8a111dc5, SUC_lexicon_tags_89 },
    { 0x80711dc4, SUC_lexicon_tags_6 },
    { 0x2bd11dc6, SUC_lexicon_tags_49 },
    { 0x97311dc8, SUC_lexicon_tags_28 },
    { 0x9cd11dc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad911dcb, SUC_lexicon_tags_46 },
    { 0xe1911dcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e511dd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xceb11dd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0911dda, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x88111ddc, SUC_lexicon_tags_11 },
    { 0x8b111ddc, SUC_lexicon_tags_37 },
    { 0xd111dde, SUC_lexicon_tags_38 },
    { 0xc6711dde, SUC_lexicon_tags_30 },
    { 0xe7311de0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e711de3, SUC_lexicon_tags_47 },
    { 0x24d11de4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6511de7, SUC_lexicon_tags_14 },
    { 0xa0f11de8, SUC_lexicon_tags_63 },
    { 0x31311de8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cb11deb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d11def, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f11df2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab11df5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36b11df7, SUC_lexicon_tags_37 },
    { 0xf3511df8, SUC_lexicon_tags_1 },
    { 0x18311df8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b11dfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f11dff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1b11e01, SUC_lexicon_tags_7 },
    { 0xdd11e02, SUC_lexicon_tags_2 },
    { 0x1eb11e02, SUC_lexicon_tags_9 },
    { 0x9f511e04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x31d11e06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44111e09, SUC_lexicon_tags_12 },
    { 0x6b511e0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8eb11e0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba511e11, SUC_lexicon_tags_37 },
    { 0xa2d11e11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f11e17, SUC_lexicon_tags_11 },
    { 0xa5511e18, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe8911e1a, SUC_lexicon_tags_1 },
    { 0x70511e1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab11e1e, SUC_lexicon_tags_29 },
    { 0x11911e1e, SUC_lexicon_tags_15 },
    { 0xa3511e1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x28311e23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d11e26, SUC_lexicon_tags_17 },
    { 0x78d11e26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c311e2a, SUC_lexicon_tags_111 },
    { 0x21511e2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7eb11e2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed11e2f, SUC_lexicon_tags_37 },
    { 0x37f11e2f, SUC_lexicon_tags_2 },
    { 0x72111e31, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb11e35, SUC_lexicon_tags_129 },
    { 0xb3f11e35, SUC_lexicon_tags_2 },
    { 0x48311e36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7cb11e39, SUC_lexicon_tags_2 },
    { 0x71d11e3a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5ed11e3c, SUC_lexicon_tags_15 },
    { 0x8a911e3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbbd11e3f, SUC_lexicon_tags_7 },
    { 0x4ff11e40, SUC_lexicon_tags_0 },
    { 0x61911e41, SUC_lexicon_tags_2 },
    { 0x911e41, SUC_lexicon_tags_32 },
    { 0x44911e43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62111e4d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd911e51, SUC_lexicon_tags_25 },
    { 0xe1f11e51, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b11e55, SUC_lexicon_tags_84 },
    { 0x12911e55, SUC_lexicon_tags_37 },
    { 0x3b11e56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f11e5a, SUC_lexicon_tags_2 },
    { 0x63311e5a, SUC_lexicon_tags_37 },
    { 0x51311e5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88911e61, SUC_lexicon_tags_1 },
    { 0xe6711e62, SUC_lexicon_tags_32 },
    { 0xcbf11e63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x84911e66, SUC_lexicon_tags_11 },
    { 0x5f511e66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa2511e69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a911e6d, SUC_lexicon_tags_31 },
    { 0x70511e6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c111e75, SUC_lexicon_tags_11 },
    { 0x18f11e76, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xdbb11e78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82911e7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b11e7d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d11e85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b11e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x111e8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9711e8d, SUC_lexicon_tags_47 },
    { 0x8ed11e8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc311e90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19911e94, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d11e97, SUC_lexicon_tags_2 },
    { 0x80511e98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b111e9a, SUC_lexicon_tags_31 },
    { 0x20d11e9b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1311ea7, SUC_lexicon_tags_31 },
    { 0x73d11ea7, SUC_lexicon_tags_4 },
    { 0x46f11ea9, SUC_lexicon_tags_11 },
    { 0x8f711ea7, SUC_lexicon_tags_12 },
    { 0x42711eab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb511eaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8911eb3, SUC_lexicon_tags_54 },
    { 0x24111eb4, SUC_lexicon_tags_47 },
    { 0xead11eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d11ebb, SUC_lexicon_tags_25 },
    { 0x79311ebc, SUC_lexicon_tags_7 },
    { 0x27b11ebd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b11ec4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f11ec7, SUC_lexicon_tags_7 },
    { 0x45711ec8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9911ecd, SUC_lexicon_tags_23 },
    { 0x68d11ecd, SUC_lexicon_tags_48 },
    { 0x54911ecf, SUC_lexicon_tags_7 },
    { 0xdc111ecf, SUC_lexicon_tags_11 },
    { 0x35911ecd, SUC_lexicon_tags_33 },
    { 0x6d711ecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b11ed6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4111ed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63911edb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9df11edd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf11ee2, SUC_lexicon_tags_37 },
    { 0x91111ee3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa5711ee5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x84b11ee7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2511eec, SUC_lexicon_tags_19 },
    { 0x5eb11eed, SUC_lexicon_tags_25 },
    { 0x5f711eee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d11efb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95f11efd, SUC_lexicon_tags_146 },
    { 0x1cd11efd, SUC_lexicon_tags_19 },
    { 0xe0b11eff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x54b11f01, SUC_lexicon_tags_11 },
    { 0x6e111f01, SUC_lexicon_tags_37 },
    { 0x8bb11f01, SUC_lexicon_tags_38 },
    { 0x8af11f04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ed11f06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26911f08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed11f0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad311f10, SUC_lexicon_tags_14 },
    { 0x4b711f11, SUC_lexicon_tags_37 },
    { 0x5a911f11, SUC_lexicon_tags_29 },
    { 0x97311f13, SUC_lexicon_tags_12 },
    { 0x39d11f14, SUC_lexicon_tags_45 },
    { 0x3df11f13, SUC_lexicon_tags_7 },
    { 0xd3f11f16, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x2d11f18, SUC_lexicon_tags_16 },
    { 0x22111f18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac511f1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x97311f1e, SUC_lexicon_tags_27 },
    { 0x14111f1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b11f26, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c311f2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x61911f2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e911f30, SUC_lexicon_tags_49 },
    { 0x5d711f31, SUC_lexicon_tags_38 },
    { 0xc5511f31, SUC_lexicon_tags_17 },
    { 0x38f11f31, SUC_lexicon_tags_15 },
    { 0xb4911f34, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0111f38, SUC_lexicon_tags_23 },
    { 0x79d11f39, SUC_lexicon_tags_4 },
    { 0xa711f39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcdf11f3c, SUC_lexicon_tags_87 },
    { 0x46b11f3d, SUC_lexicon_tags_30 },
    { 0xe111f3e, SUC_lexicon_tags_39 },
    { 0x15511f3f, SUC_lexicon_tags_0 },
    { 0x7a911f40, SUC_lexicon_tags_60 },
    { 0xd11f41, SUC_lexicon_tags_37 },
    { 0xa0f11f3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafb11f44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b11f48, SUC_lexicon_tags_2 },
    { 0xe2311f48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99711f4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df11f51, SUC_lexicon_tags_25 },
    { 0x54511f52, SUC_lexicon_tags_17 },
    { 0xeaf11f53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1911f5a, SUC_lexicon_tags_9 },
    { 0x75111f5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4f11f5d, SUC_lexicon_tags_37 },
    { 0xebb11f5d, SUC_lexicon_tags_14 },
    { 0x68f11f5d, SUC_lexicon_tags_14 },
    { 0xb9311f60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83511f62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b11f65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b711f68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7111f6e, SUC_lexicon_tags_124 },
    { 0x8c911f6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xddd11f71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd11f74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28511f76, SUC_lexicon_tags_17 },
    { 0x36711f77, SUC_lexicon_tags_47 },
    { 0x5d111f77, SUC_lexicon_tags_25 },
    { 0xeb11f79, SUC_lexicon_tags_17 },
    { 0xb3711f79, SUC_lexicon_tags_28 },
    { 0x86511f78, SUC_lexicon_tags_15 },
    { 0xea911f76, SUC_lexicon_tags_37 },
    { 0x53111f76, SUC_lexicon_tags_17 },
    { 0x7d311f7e, SUC_lexicon_tags_37 },
    { 0xc8b11f7f, SUC_lexicon_tags_14 },
    { 0x31f11f80, SUC_lexicon_tags_1 },
    { 0x94711f80, SUC_lexicon_tags_7 },
    { 0x6a511f7e, SUC_lexicon_tags_33 },
    { 0x54711f82, SUC_lexicon_tags_19 },
    { 0x38511f84, SUC_lexicon_tags_17 },
    { 0x49311f85, SUC_lexicon_tags_15 },
    { 0xaa111f86, SUC_lexicon_tags_43 },
    { 0x81311f87, SUC_lexicon_tags_70 },
    { 0x4c311f82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x76f11f8a, SUC_lexicon_tags_63 },
    { 0x53f11f8a, SUC_lexicon_tags_60 },
    { 0x58711f8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd4911f8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda111f90, SUC_lexicon_tags_27 },
    { 0x3c911f91, SUC_lexicon_tags_37 },
    { 0x18111f90, SUC_lexicon_tags_7 },
    { 0xa2911f93, SUC_lexicon_tags_63 },
    { 0xc3d11f94, SUC_lexicon_tags_3 },
    { 0x9d511f91, SUC_lexicon_tags_11 },
    { 0x76111f96, SUC_lexicon_tags_29 },
    { 0xe4111f92, SUC_lexicon_tags_87 },
    { 0xa0311f98, SUC_lexicon_tags_60 },
    { 0x5911f99, SUC_lexicon_tags_61 },
    { 0x58511f9a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe4511f9c, SUC_lexicon_tags_1 },
    { 0x6f911f9c, SUC_lexicon_tags_21 },
    { 0x92111f9e, SUC_lexicon_tags_19 },
    { 0x7f311f9f, SUC_lexicon_tags_11 },
    { 0xa4511f9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb11fa3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xed711fa5, SUC_lexicon_tags_37 },
    { 0x29511fa6, SUC_lexicon_tags_7 },
    { 0xd6911fa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8f11fa9, SUC_lexicon_tags_63 },
    { 0x48b11faa, SUC_lexicon_tags_28 },
    { 0xa8111fab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x94511fae, SUC_lexicon_tags_2 },
    { 0xec111fae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7711fb3, SUC_lexicon_tags_2 },
    { 0x18511fb4, SUC_lexicon_tags_26 },
    { 0x53111fb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb111fbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb11fbf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c311fc2, SUC_lexicon_tags_9 },
    { 0x32d11fc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d911fc5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b11fcc, SUC_lexicon_tags_7 },
    { 0xeab11fcc, SUC_lexicon_tags_6 },
    { 0x7f311fce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d511fd1, SUC_lexicon_tags_1 },
    { 0xe9711fd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd11fd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66111fda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56311fdc, SUC_lexicon_tags_17 },
    { 0x5511fdc, SUC_lexicon_tags_2 },
    { 0x24b11fdc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc311fe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f11fe3, SUC_lexicon_tags_37 },
    { 0xbdb11fe3, SUC_lexicon_tags_27 },
    { 0x5f311fe5, SUC_lexicon_tags_14 },
    { 0xb3511fe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52711fec, SUC_lexicon_tags_63 },
    { 0xf9f11fed, SUC_lexicon_tags_29 },
    { 0xd4111fee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xabf11ff0, SUC_lexicon_tags_25 },
    { 0x39f11ff1, SUC_lexicon_tags_7 },
    { 0x1dd11ff0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf11ff6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f11ffc, SUC_lexicon_tags_18 },
    { 0x3e911ffd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb12002, SUC_lexicon_tags_60 },
    { 0xc7f12002, SUC_lexicon_tags_22 },
    { 0xb4712004, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd12007, SUC_lexicon_tags_43 },
    { 0x72d12007, SUC_lexicon_tags_25 },
    { 0x93712007, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1911200d, SUC_lexicon_tags_25 },
    { 0xc771200e, SUC_lexicon_tags_1 },
    { 0x6e51200f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9912013, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d12016, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x41312018, SUC_lexicon_tags_16 },
    { 0xbd312019, SUC_lexicon_tags_29 },
    { 0x89112019, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd1201e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67712024, SUC_lexicon_tags_12 },
    { 0x9312025, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb712028, SUC_lexicon_tags_25 },
    { 0x17d12028, SUC_lexicon_tags_96 },
    { 0x6a91202a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6471202d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf12030, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4db12032, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9912037, SUC_lexicon_tags_62 },
    { 0x7d312038, SUC_lexicon_tags_0 },
    { 0x61b12039, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d91203e, SUC_lexicon_tags_12 },
    { 0xf871203f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f12043, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa1d12045, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5d112047, SUC_lexicon_tags_25 },
    { 0x40912047, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6b1204a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b1204f, SUC_lexicon_tags_60 },
    { 0x61b1204f, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57712054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab12057, SUC_lexicon_tags_11 },
    { 0x25d12058, SUC_lexicon_tags_56 },
    { 0xeb12059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e31205b, SUC_lexicon_tags_11 },
    { 0x13d1205c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef12062, SUC_lexicon_tags_2 },
    { 0x86312062, SUC_lexicon_tags_2 },
    { 0xeb512064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35312069, SUC_lexicon_tags_75 },
    { 0x83712069, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5151206c, SUC_lexicon_tags_37 },
    { 0x771206d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f12077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c512082, SUC_lexicon_tags_25 },
    { 0x25512082, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e312087, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb71208a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f1208d, SUC_lexicon_tags_8 },
    { 0x7b51208e, SUC_lexicon_tags_8 },
    { 0x6191208f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52512094, SUC_lexicon_tags_25 },
    { 0xe0d12095, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f12097, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaed12099, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7451209e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d120a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb120a5, SUC_lexicon_tags_19 },
    { 0x871120a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5120a9, SUC_lexicon_tags_56 },
    { 0x361120aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x885120ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741120b4, SUC_lexicon_tags_21 },
    { 0xf59120b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f120bd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xcd5120bf, SUC_lexicon_tags_1 },
    { 0xb4d120c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625120c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa63120c7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7120cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47120ce, SUC_lexicon_tags_23 },
    { 0xe17120cf, SUC_lexicon_tags_14 },
    { 0x6f9120ce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91120d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f120d9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b120dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf120e4, SUC_lexicon_tags_7 },
    { 0xb5f120e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7b5120e7, SUC_lexicon_tags_2 },
    { 0x845120e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacf120ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565120f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b120f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d120fb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xbf9120fd, SUC_lexicon_tags_87 },
    { 0x24f120fe, SUC_lexicon_tags_32 },
    { 0x5bd120ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb712102, SUC_lexicon_tags_11 },
    { 0xe0712103, SUC_lexicon_tags_14 },
    { 0xed912104, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a912107, SUC_lexicon_tags_9 },
    { 0xa512108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe71210a, SUC_lexicon_tags_12 },
    { 0x9e91210b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b31210e, SUC_lexicon_tags_61 },
    { 0x67b1210f, SUC_lexicon_tags_66 },
    { 0xd451210f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39312112, SUC_lexicon_tags_37 },
    { 0x8dd12112, SUC_lexicon_tags_14 },
    { 0xd5d12114, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f312116, SUC_lexicon_tags_47 },
    { 0xa6d12116, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb2112119, SUC_lexicon_tags_65 },
    { 0x3bf1211a, SUC_lexicon_tags_26 },
    { 0x2af1211a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x85d1211d, SUC_lexicon_tags_37 },
    { 0x5811211d, SUC_lexicon_tags_27 },
    { 0x3bf1211f, SUC_lexicon_tags_33 },
    { 0x81312120, SUC_lexicon_tags_37 },
    { 0xc6b12121, SUC_lexicon_tags_25 },
    { 0xac912122, SUC_lexicon_tags_25 },
    { 0xebd12123, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb12127, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2631212e, SUC_lexicon_tags_25 },
    { 0xad31212f, SUC_lexicon_tags_47 },
    { 0xa0712130, SUC_lexicon_tags_9 },
    { 0xb9512131, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x77b12133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff112135, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe8512137, SUC_lexicon_tags_45 },
    { 0x7e312137, SUC_lexicon_tags_10 },
    { 0x17312139, SUC_lexicon_tags_3 },
    { 0x1ab12139, SUC_lexicon_tags_27 },
    { 0x87f1213a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60d1213d, SUC_lexicon_tags_26 },
    { 0x47d1213e, SUC_lexicon_tags_4 },
    { 0x8d31213f, SUC_lexicon_tags_25 },
    { 0x2c512140, SUC_lexicon_tags_11 },
    { 0x17512141, SUC_lexicon_tags_6 },
    { 0xcff1213e, SUC_lexicon_tags_7 },
    { 0xe0d12143, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5551214d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d12154, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf9b12156, SUC_lexicon_tags_45 },
    { 0x78712156, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1811215b, SUC_lexicon_tags_43 },
    { 0x4511215c, SUC_lexicon_tags_15 },
    { 0xabf1215d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96712164, SUC_lexicon_tags_9 },
    { 0x41312165, SUC_lexicon_tags_27 },
    { 0x3f312166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b1216c, SUC_lexicon_tags_77 },
    { 0xb471216c, SUC_lexicon_tags_16 },
    { 0x25f1216c, SUC_lexicon_tags_2 },
    { 0x4ad1216f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87112171, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b1217a, SUC_lexicon_tags_133 },
    { 0xa231217b, SUC_lexicon_tags_57 },
    { 0x72f1217b, SUC_lexicon_tags_37 },
    { 0x69f1217c, SUC_lexicon_tags_29 },
    { 0x17d1217e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97712180, SUC_lexicon_tags_37 },
    { 0xb3512181, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x44512184, SUC_lexicon_tags_4 },
    { 0xd4f12184, SUC_lexicon_tags_12 },
    { 0x6dd12186, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x4e712188, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd1218b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e91218e, SUC_lexicon_tags_28 },
    { 0xc0d1218f, SUC_lexicon_tags_25 },
    { 0x5991218f, SUC_lexicon_tags_12 },
    { 0x60b12191, SUC_lexicon_tags_66 },
    { 0x8e71218e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x87312194, SUC_lexicon_tags_32 },
    { 0x1cd12194, SUC_lexicon_tags_11 },
    { 0xc8f12194, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c912198, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb1219b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b91219e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93121a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef121a4, SUC_lexicon_tags_15 },
    { 0x17f121a5, SUC_lexicon_tags_25 },
    { 0x1b3121a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db121a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d121ab, SUC_lexicon_tags_82 },
    { 0x6d121ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3121af, SUC_lexicon_tags_25 },
    { 0xc37121b0, SUC_lexicon_tags_12 },
    { 0x42b121b1, SUC_lexicon_tags_60 },
    { 0xc2f121b2, SUC_lexicon_tags_11 },
    { 0x893121b2, SUC_lexicon_tags_31 },
    { 0xb85121b4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b121b9, SUC_lexicon_tags_17 },
    { 0x7d7121ba, SUC_lexicon_tags_68 },
    { 0xa31121ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd3121bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x779121c0, SUC_lexicon_tags_11 },
    { 0xbe3121c0, SUC_lexicon_tags_55 },
    { 0xc05121c2, SUC_lexicon_tags_353 },
    { 0xf5f121c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513121c9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8f7121cb, SUC_lexicon_tags_40 },
    { 0x15b121cb, SUC_lexicon_tags_45 },
    { 0xa7d121cd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d121d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7121d8, SUC_lexicon_tags_21 },
    { 0x14d121d9, SUC_lexicon_tags_1 },
    { 0xe01121da, SUC_lexicon_tags_17 },
    { 0x91121db, SUC_lexicon_tags_12 },
    { 0xeef121dc, SUC_lexicon_tags_32 },
    { 0xe4d121dd, SUC_lexicon_tags_7 },
    { 0x363121da, SUC_lexicon_tags_2 },
    { 0x5cd121df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29121e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x47d121e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3121ec, SUC_lexicon_tags_17 },
    { 0x4df121ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497121f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a9121f4, SUC_lexicon_tags_57 },
    { 0xc5f121f5, SUC_lexicon_tags_11 },
    { 0xbc7121f6, SUC_lexicon_tags_1 },
    { 0x7f7121f7, SUC_lexicon_tags_46 },
    { 0x47b121f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97712201, SUC_lexicon_tags_37 },
    { 0x82912202, SUC_lexicon_tags_25 },
    { 0xf2512203, SUC_lexicon_tags_84 },
    { 0x81d12204, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b12208, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff1220b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5451220d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf12211, SUC_lexicon_tags_47 },
    { 0xf0912212, SUC_lexicon_tags_37 },
    { 0x21312213, SUC_lexicon_tags_7 },
    { 0x84712214, SUC_lexicon_tags_49 },
    { 0xf9f12215, SUC_lexicon_tags_37 },
    { 0x45b12215, SUC_lexicon_tags_49 },
    { 0xe2112217, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6191221d, SUC_lexicon_tags_19 },
    { 0x8731221d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xde112221, SUC_lexicon_tags_33 },
    { 0xb512222, SUC_lexicon_tags_26 },
    { 0xbbd12223, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a712226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b1222a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b1222f, SUC_lexicon_tags_11 },
    { 0xbb112230, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d712232, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe4f12234, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10112237, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b1223b, SUC_lexicon_tags_82 },
    { 0xa1f1223b, SUC_lexicon_tags_6 },
    { 0x6771223d, SUC_lexicon_tags_37 },
    { 0xff51223b, SUC_lexicon_tags_2 },
    { 0xe0f1223f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe2d12241, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f12244, SUC_lexicon_tags_39 },
    { 0x76912245, SUC_lexicon_tags_17 },
    { 0x5e112246, SUC_lexicon_tags_1 },
    { 0xb2f12247, SUC_lexicon_tags_49 },
    { 0xf7912248, SUC_lexicon_tags_33 },
    { 0xcaf12249, SUC_lexicon_tags_37 },
    { 0xbdf1224a, SUC_lexicon_tags_47 },
    { 0x32f1224a, SUC_lexicon_tags_351 },
    { 0xd7d1224a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f12253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44912258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86d1225a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98f1225c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a31225e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5af12260, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x84b12262, SUC_lexicon_tags_4 },
    { 0xbcf12262, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb8f12265, SUC_lexicon_tags_35 },
    { 0x6ef12266, SUC_lexicon_tags_25 },
    { 0xa7f12267, SUC_lexicon_tags_5 },
    { 0x3512266, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd12270, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa4912272, SUC_lexicon_tags_34 },
    { 0x3bd12273, SUC_lexicon_tags_63 },
    { 0xa5512273, SUC_lexicon_tags_2 },
    { 0x9f912275, SUC_lexicon_tags_13 },
    { 0xb4312274, SUC_lexicon_tags_2 },
    { 0xc9d12277, SUC_lexicon_tags_28 },
    { 0xdc512278, SUC_lexicon_tags_43 },
    { 0x2eb12277, SUC_lexicon_tags_42 },
    { 0x2ab12277, SUC_lexicon_tags_15 },
    { 0x6112276, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4151227f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc512283, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a712286, SUC_lexicon_tags_1 },
    { 0x67112287, SUC_lexicon_tags_17 },
    { 0x31d12288, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e31228a, SUC_lexicon_tags_351 },
    { 0x1ef1228b, SUC_lexicon_tags_82 },
    { 0xd811228c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2bd1228e, SUC_lexicon_tags_1 },
    { 0x72d1228f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d512297, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42d12299, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4211229b, SUC_lexicon_tags_63 },
    { 0xf0b1229b, SUC_lexicon_tags_16 },
    { 0x29f1229d, SUC_lexicon_tags_29 },
    { 0x40d1229e, SUC_lexicon_tags_1 },
    { 0x3531229f, SUC_lexicon_tags_25 },
    { 0x2571229e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65122a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b122a7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b122ab, SUC_lexicon_tags_1 },
    { 0xcab122ab, SUC_lexicon_tags_61 },
    { 0xd43122ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95122b1, SUC_lexicon_tags_7 },
    { 0xda9122b1, SUC_lexicon_tags_12 },
    { 0x50d122b3, SUC_lexicon_tags_66 },
    { 0x21f122b3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5122ba, SUC_lexicon_tags_87 },
    { 0xff7122bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd122bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1122c7, SUC_lexicon_tags_56 },
    { 0x1c9122c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf122cd, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x9d122cf, SUC_lexicon_tags_1 },
    { 0xd7122d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x459122d3, SUC_lexicon_tags_25 },
    { 0x835122d4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895122da, SUC_lexicon_tags_11 },
    { 0xce5122da, SUC_lexicon_tags_26 },
    { 0x59d122dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdd9122de, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b122e1, SUC_lexicon_tags_37 },
    { 0xfcd122e1, SUC_lexicon_tags_79 },
    { 0x625122e2, SUC_lexicon_tags_26 },
    { 0x88b122e4, SUC_lexicon_tags_37 },
    { 0x419122e5, SUC_lexicon_tags_1 },
    { 0xc69122e5, SUC_lexicon_tags_13 },
    { 0xf5122e7, SUC_lexicon_tags_2 },
    { 0xa01122e4, SUC_lexicon_tags_11 },
    { 0xb35122e9, SUC_lexicon_tags_37 },
    { 0xa37122e5, SUC_lexicon_tags_7 },
    { 0x269122ea, SUC_lexicon_tags_25 },
    { 0xf67122e4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23122ef, SUC_lexicon_tags_17 },
    { 0x1d122ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb05122f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f122f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed3122fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x591122fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1122ff, SUC_lexicon_tags_16 },
    { 0x49912300, SUC_lexicon_tags_19 },
    { 0xcff12300, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfe912303, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff312307, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf7912309, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5a71230b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b51230e, SUC_lexicon_tags_12 },
    { 0xd551230f, SUC_lexicon_tags_1 },
    { 0xdfd12310, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b712314, SUC_lexicon_tags_25 },
    { 0x53b12314, SUC_lexicon_tags_37 },
    { 0x3e912315, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d12320, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d712322, SUC_lexicon_tags_31 },
    { 0xcb12323, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8711232a, SUC_lexicon_tags_61 },
    { 0x7911232a, SUC_lexicon_tags_37 },
    { 0xed1232c, SUC_lexicon_tags_2 },
    { 0xd811232d, SUC_lexicon_tags_63 },
    { 0x4031232e, SUC_lexicon_tags_28 },
    { 0x47d1232a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51112331, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6912334, SUC_lexicon_tags_39 },
    { 0x3cd12335, SUC_lexicon_tags_6 },
    { 0x88312336, SUC_lexicon_tags_68 },
    { 0x1f12337, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db1233b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98912340, SUC_lexicon_tags_57 },
    { 0x29d12340, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x64512343, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x5cd12345, SUC_lexicon_tags_28 },
    { 0x10f12346, SUC_lexicon_tags_115 },
    { 0x57b12347, SUC_lexicon_tags_47 },
    { 0x62312345, SUC_lexicon_tags_23 },
    { 0x77d12346, SUC_lexicon_tags_17 },
    { 0x8af12345, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed1234e, SUC_lexicon_tags_19 },
    { 0x1531234f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbb12351, SUC_lexicon_tags_7 },
    { 0xb1f12351, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb1f12354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad12359, SUC_lexicon_tags_37 },
    { 0x1fd12359, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5ef1235c, SUC_lexicon_tags_1 },
    { 0xa7d1235c, SUC_lexicon_tags_27 },
    { 0xe3f1235e, SUC_lexicon_tags_84 },
    { 0xaf91235f, SUC_lexicon_tags_25 },
    { 0x17f12360, SUC_lexicon_tags_37 },
    { 0x8dd12361, SUC_lexicon_tags_17 },
    { 0xd4512360, SUC_lexicon_tags_17 },
    { 0x90b12363, SUC_lexicon_tags_28 },
    { 0xa1b12360, SUC_lexicon_tags_31 },
    { 0x66312363, SUC_lexicon_tags_39 },
    { 0x32912366, SUC_lexicon_tags_64 },
    { 0x5d912363, SUC_lexicon_tags_25 },
    { 0x7831235f, SUC_lexicon_tags_63 },
    { 0x1b912363, SUC_lexicon_tags_1 },
    { 0x64b1236a, SUC_lexicon_tags_6 },
    { 0x8b31236a, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d12371, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb12374, SUC_lexicon_tags_8 },
    { 0x77f12375, SUC_lexicon_tags_17 },
    { 0xce912375, SUC_lexicon_tags_11 },
    { 0x39712377, SUC_lexicon_tags_37 },
    { 0x27912378, SUC_lexicon_tags_7 },
    { 0x9c712375, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c71237b, SUC_lexicon_tags_15 },
    { 0xbf31237c, SUC_lexicon_tags_42 },
    { 0xadd1237d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf5f1237f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1b12381, SUC_lexicon_tags_2 },
    { 0x80512382, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba31238b, SUC_lexicon_tags_25 },
    { 0xdc51238c, SUC_lexicon_tags_37 },
    { 0x9011238c, SUC_lexicon_tags_14 },
    { 0xa851238e, SUC_lexicon_tags_15 },
    { 0x5cb1238f, SUC_lexicon_tags_11 },
    { 0x1b312390, SUC_lexicon_tags_48 },
    { 0x9ff12391, SUC_lexicon_tags_15 },
    { 0x1b512392, SUC_lexicon_tags_47 },
    { 0x59112392, SUC_lexicon_tags_33 },
    { 0x4e112394, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b12397, SUC_lexicon_tags_17 },
    { 0x9912397, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d71239d, SUC_lexicon_tags_11 },
    { 0x98f1239e, SUC_lexicon_tags_37 },
    { 0xf1239e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff123a6, SUC_lexicon_tags_1 },
    { 0xf07123a7, SUC_lexicon_tags_37 },
    { 0x705123a6, SUC_lexicon_tags_60 },
    { 0x1ed123a9, SUC_lexicon_tags_57 },
    { 0x1df123a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cb123ac, SUC_lexicon_tags_9 },
    { 0x5a3123ad, SUC_lexicon_tags_141 },
    { 0xd75123ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb123b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad123b8, SUC_lexicon_tags_23 },
    { 0x40f123b9, SUC_lexicon_tags_1 },
    { 0xf2b123ba, SUC_lexicon_tags_38 },
    { 0x387123bb, SUC_lexicon_tags_79 },
    { 0xb59123b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xccb123be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81123c2, SUC_lexicon_tags_7 },
    { 0x445123c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d123cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3123cf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x771123d1, SUC_lexicon_tags_25 },
    { 0xd19123d2, SUC_lexicon_tags_151 },
    { 0x333123d3, SUC_lexicon_tags_33 },
    { 0xc2f123d2, SUC_lexicon_tags_1 },
    { 0xbdd123d1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5123e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x89123e4, SUC_lexicon_tags_7 },
    { 0xd95123e5, SUC_lexicon_tags_17 },
    { 0x267123e4, SUC_lexicon_tags_17 },
    { 0x81123e5, SUC_lexicon_tags_17 },
    { 0x5a9123e4, SUC_lexicon_tags_8 },
    { 0x1c9123e8, SUC_lexicon_tags_7 },
    { 0xa21123e5, SUC_lexicon_tags_31 },
    { 0x161123eb, SUC_lexicon_tags_31 },
    { 0x13f123ea, SUC_lexicon_tags_178 },
    { 0x5a9123ed, SUC_lexicon_tags_11 },
    { 0x4b1123ed, SUC_lexicon_tags_60 },
    { 0xa99123ef, SUC_lexicon_tags_43 },
    { 0x1e7123ef, SUC_lexicon_tags_25 },
    { 0x8b7123ef, SUC_lexicon_tags_17 },
    { 0xc91123f2, SUC_lexicon_tags_12 },
    { 0x561123e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3123f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd123fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa71123fe, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f12404, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x96512406, SUC_lexicon_tags_2 },
    { 0x2bd12406, SUC_lexicon_tags_79 },
    { 0x7eb12408, SUC_lexicon_tags_11 },
    { 0x43d12408, SUC_lexicon_tags_45 },
    { 0x78b1240a, SUC_lexicon_tags_14 },
    { 0x4cf1240b, SUC_lexicon_tags_25 },
    { 0x2671240c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd1240f, SUC_lexicon_tags_11 },
    { 0xaa112410, SUC_lexicon_tags_11 },
    { 0xe3312411, SUC_lexicon_tags_10 },
    { 0xbbf12411, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd12415, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d1241b, SUC_lexicon_tags_2 },
    { 0xa8d1241b, SUC_lexicon_tags_2 },
    { 0x9a71241d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d512421, SUC_lexicon_tags_27 },
    { 0xc2312421, SUC_lexicon_tags_12 },
    { 0x1dd12422, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e912426, SUC_lexicon_tags_12 },
    { 0x61312427, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb1242a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac71242e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85712430, SUC_lexicon_tags_12 },
    { 0xaa512430, SUC_lexicon_tags_38 },
    { 0x70512432, SUC_lexicon_tags_7 },
    { 0x13d12433, SUC_lexicon_tags_0 },
    { 0x5ff12433, SUC_lexicon_tags_11 },
    { 0x79912435, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9912437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39512439, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8751243e, SUC_lexicon_tags_14 },
    { 0x28d1243f, SUC_lexicon_tags_49 },
    { 0x83b1243f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb12442, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7712445, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb11244a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8091244e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eb12450, SUC_lexicon_tags_0 },
    { 0x7d712451, SUC_lexicon_tags_0 },
    { 0xa5512452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e112457, SUC_lexicon_tags_2 },
    { 0xc8712458, SUC_lexicon_tags_28 },
    { 0x55112459, SUC_lexicon_tags_25 },
    { 0xde91245a, SUC_lexicon_tags_5 },
    { 0x8d91245a, SUC_lexicon_tags_15 },
    { 0x3cb1245c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe871245f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedf12461, SUC_lexicon_tags_37 },
    { 0xac112462, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x97712464, SUC_lexicon_tags_12 },
    { 0x25312465, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f12468, SUC_lexicon_tags_17 },
    { 0x9ff12469, SUC_lexicon_tags_11 },
    { 0x35b1246a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5551246c, SUC_lexicon_tags_2 },
    { 0xb0f1246d, SUC_lexicon_tags_26 },
    { 0x6f11246d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8eb12470, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15312474, SUC_lexicon_tags_2 },
    { 0xf3112475, SUC_lexicon_tags_15 },
    { 0xac512476, SUC_lexicon_tags_11 },
    { 0x47512477, SUC_lexicon_tags_15 },
    { 0x43112476, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x53b1247a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d12480, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b12485, SUC_lexicon_tags_9 },
    { 0xfa912486, SUC_lexicon_tags_27 },
    { 0xfc512487, SUC_lexicon_tags_118 },
    { 0x7af12488, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d1248e, SUC_lexicon_tags_30 },
    { 0x11b1248f, SUC_lexicon_tags_7 },
    { 0xa9512490, SUC_lexicon_tags_21 },
    { 0xe811248f, SUC_lexicon_tags_37 },
    { 0x59d12490, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23312496, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde71249b, SUC_lexicon_tags_34 },
    { 0xc6b1249c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273124a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af124a4, SUC_lexicon_tags_4 },
    { 0x9f7124a5, SUC_lexicon_tags_44 },
    { 0x22b124a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b124ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349124b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xecf124b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57124b5, SUC_lexicon_tags_12 },
    { 0x40b124b6, SUC_lexicon_tags_15 },
    { 0x1e5124b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d124bb, SUC_lexicon_tags_68 },
    { 0xebb124bb, SUC_lexicon_tags_7 },
    { 0xc05124bd, SUC_lexicon_tags_37 },
    { 0x867124be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7b124c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5d124c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x44b124c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9d124c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d124c9, SUC_lexicon_tags_47 },
    { 0x4eb124ca, SUC_lexicon_tags_25 },
    { 0x917124cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaff124cd, SUC_lexicon_tags_17 },
    { 0x26d124ce, SUC_lexicon_tags_15 },
    { 0x381124cf, SUC_lexicon_tags_6 },
    { 0x545124cf, SUC_lexicon_tags_2 },
    { 0xb71124d1, SUC_lexicon_tags_26 },
    { 0x2d9124d0, SUC_lexicon_tags_17 },
    { 0x687124d3, SUC_lexicon_tags_15 },
    { 0x19124d3, SUC_lexicon_tags_8 },
    { 0x2d7124d5, SUC_lexicon_tags_9 },
    { 0x43124d5, SUC_lexicon_tags_26 },
    { 0x69d124d1, SUC_lexicon_tags_7 },
    { 0xff5124d7, SUC_lexicon_tags_11 },
    { 0xb9f124ce, SUC_lexicon_tags_8 },
    { 0xd4d124da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93124e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe47124e4, SUC_lexicon_tags_11 },
    { 0x3c7124e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b124eb, SUC_lexicon_tags_7 },
    { 0x52f124eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05124f2, SUC_lexicon_tags_17 },
    { 0xa2d124f2, SUC_lexicon_tags_25 },
    { 0x8df124f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7124fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x943124fd, SUC_lexicon_tags_8 },
    { 0x5e3124fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd512503, SUC_lexicon_tags_2 },
    { 0x72112503, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e112508, SUC_lexicon_tags_87 },
    { 0xf2d12509, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b71250c, SUC_lexicon_tags_56 },
    { 0xfcd1250d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3312513, SUC_lexicon_tags_47 },
    { 0xdbd12514, SUC_lexicon_tags_0 },
    { 0xc9112515, SUC_lexicon_tags_32 },
    { 0x3a512516, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1251a, SUC_lexicon_tags_7 },
    { 0x9111251a, SUC_lexicon_tags_12 },
    { 0xfe31251a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99512523, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f712525, SUC_lexicon_tags_38 },
    { 0x3dd12526, SUC_lexicon_tags_68 },
    { 0x9cd12527, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc571252b, SUC_lexicon_tags_53 },
    { 0xd371252c, SUC_lexicon_tags_37 },
    { 0xf31252d, SUC_lexicon_tags_24 },
    { 0xa3b1252b, SUC_lexicon_tags_37 },
    { 0xfc91252f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b12534, SUC_lexicon_tags_47 },
    { 0xe9312535, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d12539, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x671253c, SUC_lexicon_tags_12 },
    { 0x9cf1253d, SUC_lexicon_tags_0 },
    { 0x2551253e, SUC_lexicon_tags_17 },
    { 0x8c11253f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b12545, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a512549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6931254e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb12552, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d312554, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde312558, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb31255b, SUC_lexicon_tags_7 },
    { 0x18f1255b, SUC_lexicon_tags_174 },
    { 0xc3f1255d, SUC_lexicon_tags_11 },
    { 0x7dd1255e, SUC_lexicon_tags_32 },
    { 0x2f51255f, SUC_lexicon_tags_39 },
    { 0x5e512560, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x26512563, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb312568, SUC_lexicon_tags_7 },
    { 0x9b512569, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb271256f, SUC_lexicon_tags_2 },
    { 0xfe912570, SUC_lexicon_tags_45 },
    { 0x9e712571, SUC_lexicon_tags_11 },
    { 0x55f12571, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f712575, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb1257a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab91257e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75512580, SUC_lexicon_tags_38 },
    { 0x48712581, SUC_lexicon_tags_7 },
    { 0xabd12580, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9312589, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc91258e, SUC_lexicon_tags_19 },
    { 0x7251258e, SUC_lexicon_tags_17 },
    { 0xf3d1258f, SUC_lexicon_tags_37 },
    { 0xe311258f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f12594, SUC_lexicon_tags_12 },
    { 0x67d12595, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e91259d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af125a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561125a4, SUC_lexicon_tags_0 },
    { 0xab3125a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db125ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56d125ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d125b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3125b4, SUC_lexicon_tags_19 },
    { 0xd81125b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x181125b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49125bd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971125c2, SUC_lexicon_tags_22 },
    { 0x8ab125c3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9125c7, SUC_lexicon_tags_10 },
    { 0x5df125c7, SUC_lexicon_tags_33 },
    { 0xf21125c9, SUC_lexicon_tags_27 },
    { 0xb4d125c9, SUC_lexicon_tags_1 },
    { 0x835125ca, SUC_lexicon_tags_29 },
    { 0x83d125cc, SUC_lexicon_tags_25 },
    { 0xb5d125c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21125d3, SUC_lexicon_tags_1 },
    { 0x659125d4, SUC_lexicon_tags_25 },
    { 0x637125d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x93d125d7, SUC_lexicon_tags_49 },
    { 0xb3125d8, SUC_lexicon_tags_25 },
    { 0x2c7125d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149125de, SUC_lexicon_tags_43 },
    { 0x8a5125df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b125e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597125e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991125ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3125f3, SUC_lexicon_tags_0 },
    { 0xc83125f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x43b125f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d125fb, SUC_lexicon_tags_46 },
    { 0xf67125fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac712601, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d312604, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95712608, SUC_lexicon_tags_9 },
    { 0xbcf12609, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2931260d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1312612, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbe912614, SUC_lexicon_tags_10 },
    { 0x21312615, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1312618, SUC_lexicon_tags_10 },
    { 0xca312619, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d12627, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d912631, SUC_lexicon_tags_87 },
    { 0xb5712631, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf12635, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66512639, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8171263b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa112642, SUC_lexicon_tags_46 },
    { 0x7ab12642, SUC_lexicon_tags_17 },
    { 0x2b312644, SUC_lexicon_tags_11 },
    { 0xcdd12644, SUC_lexicon_tags_27 },
    { 0x8bb12644, SUC_lexicon_tags_19 },
    { 0x34512646, SUC_lexicon_tags_27 },
    { 0xfc112648, SUC_lexicon_tags_1 },
    { 0x9b12644, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad1264d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9891264f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34912653, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f512656, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8771265e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5312660, SUC_lexicon_tags_11 },
    { 0x66b12661, SUC_lexicon_tags_8 },
    { 0x2cf12660, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d712667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d1266e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb9712670, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db12678, SUC_lexicon_tags_3 },
    { 0xd8f12678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b31267b, SUC_lexicon_tags_2 },
    { 0xcfb1267b, SUC_lexicon_tags_45 },
    { 0x9351267b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65712681, SUC_lexicon_tags_7 },
    { 0x61512681, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38912686, SUC_lexicon_tags_37 },
    { 0xce312687, SUC_lexicon_tags_42 },
    { 0x61712686, SUC_lexicon_tags_39 },
    { 0xdd112686, SUC_lexicon_tags_4 },
    { 0xba91268a, SUC_lexicon_tags_39 },
    { 0xb571268a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaeb1268d, SUC_lexicon_tags_38 },
    { 0x3e71268d, SUC_lexicon_tags_37 },
    { 0x83b1268f, SUC_lexicon_tags_19 },
    { 0x5df12690, SUC_lexicon_tags_11 },
    { 0x78512691, SUC_lexicon_tags_60 },
    { 0x45912691, SUC_lexicon_tags_15 },
    { 0x5f512691, SUC_lexicon_tags_237 },
    { 0x4c912692, SUC_lexicon_tags_46 },
    { 0x4db12692, SUC_lexicon_tags_35 },
    { 0x4c512696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d112698, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc771269a, SUC_lexicon_tags_43 },
    { 0xb1d1269b, SUC_lexicon_tags_26 },
    { 0x4891269a, SUC_lexicon_tags_32 },
    { 0x4f51269d, SUC_lexicon_tags_4 },
    { 0x2e31269e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaab126a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b126a3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1126a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f126ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61126af, SUC_lexicon_tags_37 },
    { 0xb3126b0, SUC_lexicon_tags_6 },
    { 0x72f126b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcb3126b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd13126b5, SUC_lexicon_tags_26 },
    { 0x751126b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9126bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x825126c1, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b126c5, SUC_lexicon_tags_33 },
    { 0xb11126c5, SUC_lexicon_tags_15 },
    { 0x613126c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd65126c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9126cd, SUC_lexicon_tags_37 },
    { 0x8e7126cd, SUC_lexicon_tags_28 },
    { 0x6e1126cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7126d4, SUC_lexicon_tags_11 },
    { 0x4c1126d4, SUC_lexicon_tags_2 },
    { 0x859126d6, SUC_lexicon_tags_43 },
    { 0xff9126d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3126de, SUC_lexicon_tags_30 },
    { 0x119126df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d126e2, SUC_lexicon_tags_27 },
    { 0xf9f126e2, SUC_lexicon_tags_2 },
    { 0xe3b126e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf126ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d126f1, SUC_lexicon_tags_16 },
    { 0x45f126f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xda3126f4, SUC_lexicon_tags_49 },
    { 0x1b126f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35126f8, SUC_lexicon_tags_15 },
    { 0x85d126f9, SUC_lexicon_tags_7 },
    { 0xc99126fa, SUC_lexicon_tags_49 },
    { 0x6e1126fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf126fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79712703, SUC_lexicon_tags_47 },
    { 0xd5912704, SUC_lexicon_tags_31 },
    { 0x93512703, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf2712707, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x30512709, SUC_lexicon_tags_63 },
    { 0x9671270a, SUC_lexicon_tags_1 },
    { 0xfcb1270b, SUC_lexicon_tags_12 },
    { 0x4f1270c, SUC_lexicon_tags_17 },
    { 0xc311270b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35712713, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a712718, SUC_lexicon_tags_15 },
    { 0xe2d12719, SUC_lexicon_tags_31 },
    { 0x4971271a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f12720, SUC_lexicon_tags_23 },
    { 0xe4912721, SUC_lexicon_tags_2 },
    { 0xe0912721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34512724, SUC_lexicon_tags_2 },
    { 0xf7312725, SUC_lexicon_tags_49 },
    { 0x10d12726, SUC_lexicon_tags_31 },
    { 0x15b12727, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe571272c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd1272f, SUC_lexicon_tags_49 },
    { 0x71312730, SUC_lexicon_tags_7 },
    { 0x74d12731, SUC_lexicon_tags_15 },
    { 0xd512732, SUC_lexicon_tags_7 },
    { 0x13912733, SUC_lexicon_tags_66 },
    { 0xea512734, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbb12736, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25512738, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf271273a, SUC_lexicon_tags_0 },
    { 0xf631273b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec91273d, SUC_lexicon_tags_11 },
    { 0x4331273e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74312740, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf12743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b12746, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdad12748, SUC_lexicon_tags_68 },
    { 0xdb312748, SUC_lexicon_tags_37 },
    { 0x2731274a, SUC_lexicon_tags_87 },
    { 0x5891274b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc911274d, SUC_lexicon_tags_37 },
    { 0x6071274d, SUC_lexicon_tags_11 },
    { 0xbe91274f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9112753, SUC_lexicon_tags_19 },
    { 0x8bd12754, SUC_lexicon_tags_49 },
    { 0x76912754, SUC_lexicon_tags_124 },
    { 0xa712755, SUC_lexicon_tags_11 },
    { 0x25512757, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9a912759, SUC_lexicon_tags_8 },
    { 0x3fb1275a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d1275f, SUC_lexicon_tags_103 },
    { 0x1bb1275f, SUC_lexicon_tags_15 },
    { 0xc5312761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11112763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b312768, SUC_lexicon_tags_11 },
    { 0x2bd12769, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe691276e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df12774, SUC_lexicon_tags_25 },
    { 0x6ff12775, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x25312777, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b1277a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf71277d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19712780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d12783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bd12785, SUC_lexicon_tags_57 },
    { 0x20112786, SUC_lexicon_tags_1 },
    { 0x21712787, SUC_lexicon_tags_8 },
    { 0x52d12787, SUC_lexicon_tags_7 },
    { 0xe0112789, SUC_lexicon_tags_11 },
    { 0x611278a, SUC_lexicon_tags_34 },
    { 0xaff12789, SUC_lexicon_tags_68 },
    { 0xb711278c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2db1278e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15312792, SUC_lexicon_tags_6 },
    { 0xa2d12792, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x63f12795, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf411279d, SUC_lexicon_tags_15 },
    { 0x2931279e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb1127a0, SUC_lexicon_tags_47 },
    { 0x767127a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb9127a3, SUC_lexicon_tags_14 },
    { 0xda7127a4, SUC_lexicon_tags_919 },
    { 0, NULL },
    { 0x941127a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x51d127a8, SUC_lexicon_tags_12 },
    { 0xc75127a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x56d127ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7127ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb93127af, SUC_lexicon_tags_30 },
    { 0x98b127b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5127b3, SUC_lexicon_tags_38 },
    { 0x113127b4, SUC_lexicon_tags_48 },
    { 0xdab127b5, SUC_lexicon_tags_17 },
    { 0xe3b127b6, SUC_lexicon_tags_2 },
    { 0xf6d127b7, SUC_lexicon_tags_87 },
    { 0x9f9127b8, SUC_lexicon_tags_60 },
    { 0xe83127b9, SUC_lexicon_tags_7 },
    { 0xcf5127b9, SUC_lexicon_tags_25 },
    { 0x715127bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9127c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x403127cb, SUC_lexicon_tags_27 },
    { 0x4fb127cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61127d1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xef5127d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d127d6, SUC_lexicon_tags_47 },
    { 0xa05127d7, SUC_lexicon_tags_5 },
    { 0x6bf127d7, SUC_lexicon_tags_28 },
    { 0x4b9127d9, SUC_lexicon_tags_37 },
    { 0x2e5127da, SUC_lexicon_tags_32 },
    { 0x265127db, SUC_lexicon_tags_1 },
    { 0x9a3127dc, SUC_lexicon_tags_23 },
    { 0x149127da, SUC_lexicon_tags_60 },
    { 0x5bb127de, SUC_lexicon_tags_28 },
    { 0x383127df, SUC_lexicon_tags_16 },
    { 0x679127e0, SUC_lexicon_tags_33 },
    { 0x2a3127e1, SUC_lexicon_tags_37 },
    { 0x103127e2, SUC_lexicon_tags_14 },
    { 0x9e5127e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b127e7, SUC_lexicon_tags_31 },
    { 0x2b5127e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11127ea, SUC_lexicon_tags_91 },
    { 0x109127eb, SUC_lexicon_tags_79 },
    { 0x491127eb, SUC_lexicon_tags_22 },
    { 0x597127ed, SUC_lexicon_tags_27 },
    { 0x4c9127ee, SUC_lexicon_tags_25 },
    { 0xc9b127ed, SUC_lexicon_tags_0 },
    { 0xb05127ef, SUC_lexicon_tags_66 },
    { 0x31d127ee, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5127f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d127f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d127fd, SUC_lexicon_tags_7 },
    { 0x7cf127fe, SUC_lexicon_tags_25 },
    { 0xbb1127fd, SUC_lexicon_tags_25 },
    { 0xfff12800, SUC_lexicon_tags_56 },
    { 0x40712800, SUC_lexicon_tags_12 },
    { 0x60312802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1512806, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4451280e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96912810, SUC_lexicon_tags_2 },
    { 0xec912810, SUC_lexicon_tags_1 },
    { 0x4e912812, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44712817, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3371281a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1211281e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15912820, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f12823, SUC_lexicon_tags_37 },
    { 0x9df12824, SUC_lexicon_tags_9 },
    { 0x93b12823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b712827, SUC_lexicon_tags_37 },
    { 0x60112828, SUC_lexicon_tags_17 },
    { 0x4b112827, SUC_lexicon_tags_29 },
    { 0xdb12829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cf1282c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff12831, SUC_lexicon_tags_16 },
    { 0x1f12832, SUC_lexicon_tags_2 },
    { 0x1e112832, SUC_lexicon_tags_4 },
    { 0x5bb12833, SUC_lexicon_tags_12 },
    { 0x92b12832, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b12838, SUC_lexicon_tags_7 },
    { 0x48b12839, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3e71283b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f12840, SUC_lexicon_tags_47 },
    { 0xf7312841, SUC_lexicon_tags_28 },
    { 0xb4912841, SUC_lexicon_tags_63 },
    { 0x3f12840, SUC_lexicon_tags_43 },
    { 0x88f12841, SUC_lexicon_tags_45 },
    { 0x9c712845, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3b12847, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d1284c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb431285b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f12860, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f12863, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d12867, SUC_lexicon_tags_25 },
    { 0x22512868, SUC_lexicon_tags_21 },
    { 0x1f512869, SUC_lexicon_tags_8 },
    { 0x13912869, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe811286f, SUC_lexicon_tags_31 },
    { 0x2491286f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x17d12872, SUC_lexicon_tags_7 },
    { 0x7cf12872, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52512875, SUC_lexicon_tags_37 },
    { 0x83112876, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6912879, SUC_lexicon_tags_37 },
    { 0x25d1287a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d1287f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c312884, SUC_lexicon_tags_27 },
    { 0x92312885, SUC_lexicon_tags_25 },
    { 0x4cb12886, SUC_lexicon_tags_11 },
    { 0x2bd12884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f1288c, SUC_lexicon_tags_7 },
    { 0x39f1288d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x40112890, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda912892, SUC_lexicon_tags_11 },
    { 0xd7d12892, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1289b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b1289d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xda11289f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d128a1, SUC_lexicon_tags_0 },
    { 0xa6b128a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb9128a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x301128a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x657128a9, SUC_lexicon_tags_33 },
    { 0xacd128a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x199128ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27128b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf128b4, SUC_lexicon_tags_26 },
    { 0x7c9128b4, SUC_lexicon_tags_60 },
    { 0x97b128b6, SUC_lexicon_tags_37 },
    { 0x5f1128b6, SUC_lexicon_tags_25 },
    { 0xa99128b8, SUC_lexicon_tags_25 },
    { 0xa4b128b6, SUC_lexicon_tags_25 },
    { 0xfaf128ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee3128bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b128bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3128c2, SUC_lexicon_tags_21 },
    { 0x51f128c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c9128c5, SUC_lexicon_tags_2 },
    { 0x4a9128c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125128d1, SUC_lexicon_tags_30 },
    { 0xb9f128d2, SUC_lexicon_tags_15 },
    { 0xa81128d3, SUC_lexicon_tags_33 },
    { 0x829128d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79d128d6, SUC_lexicon_tags_15 },
    { 0xbbd128d6, SUC_lexicon_tags_47 },
    { 0xfed128d8, SUC_lexicon_tags_11 },
    { 0x8b3128d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db128df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09128e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51d128e4, SUC_lexicon_tags_27 },
    { 0xe5b128e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f128e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab128ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255128f3, SUC_lexicon_tags_13 },
    { 0xa87128f4, SUC_lexicon_tags_12 },
    { 0x50d128f4, SUC_lexicon_tags_23 },
    { 0x1d9128f6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9128fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75128fd, SUC_lexicon_tags_67 },
    { 0x94b128fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d12904, SUC_lexicon_tags_19 },
    { 0x60712905, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e11290b, SUC_lexicon_tags_15 },
    { 0xcff1290c, SUC_lexicon_tags_25 },
    { 0xc0b1290d, SUC_lexicon_tags_31 },
    { 0x5651290e, SUC_lexicon_tags_26 },
    { 0xacf1290f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59112912, SUC_lexicon_tags_37 },
    { 0xadb12912, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb911291a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc651291d, SUC_lexicon_tags_45 },
    { 0x3851291d, SUC_lexicon_tags_5 },
    { 0x8af1291e, SUC_lexicon_tags_74 },
    { 0x3dd1291f, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x92112923, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a112925, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf912929, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d1292e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c112932, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b312935, SUC_lexicon_tags_2 },
    { 0xa5f12935, SUC_lexicon_tags_8 },
    { 0xcb912936, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d1293b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5251293e, SUC_lexicon_tags_48 },
    { 0xb351293f, SUC_lexicon_tags_70 },
    { 0x97f12940, SUC_lexicon_tags_27 },
    { 0xf931293f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccd12943, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331294e, SUC_lexicon_tags_87 },
    { 0xa091294f, SUC_lexicon_tags_14 },
    { 0x9712950, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff12955, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef1295a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd1295f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d512962, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdf712964, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b712968, SUC_lexicon_tags_85 },
    { 0xad312969, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d91296c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab12970, SUC_lexicon_tags_4 },
    { 0x26112970, SUC_lexicon_tags_2 },
    { 0xbb112970, SUC_lexicon_tags_19 },
    { 0xd4512972, SUC_lexicon_tags_1 },
    { 0xe3912972, SUC_lexicon_tags_47 },
    { 0xa6f12971, SUC_lexicon_tags_11 },
    { 0xb6312976, SUC_lexicon_tags_31 },
    { 0x59312972, SUC_lexicon_tags_7 },
    { 0x8c712978, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5231297d, SUC_lexicon_tags_14 },
    { 0x5e11297e, SUC_lexicon_tags_25 },
    { 0x3551297d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5a312981, SUC_lexicon_tags_56 },
    { 0x37b12982, SUC_lexicon_tags_37 },
    { 0xd312983, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13912988, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd271298c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc9b1298e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae712992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d12999, SUC_lexicon_tags_60 },
    { 0xdb71299a, SUC_lexicon_tags_37 },
    { 0xa6b1299b, SUC_lexicon_tags_1 },
    { 0xb3b1299b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa911299f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785129a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5129a8, SUC_lexicon_tags_15 },
    { 0x781129a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3129ad, SUC_lexicon_tags_65 },
    { 0x2cf129ad, SUC_lexicon_tags_16 },
    { 0xd1d129ae, SUC_lexicon_tags_138 },
    { 0x6cf129b0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875129b4, SUC_lexicon_tags_15 },
    { 0xe03129b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01129b8, SUC_lexicon_tags_5 },
    { 0xc97129b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x851129bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x557129bd, SUC_lexicon_tags_37 },
    { 0x445129be, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x581129c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfb9129c2, SUC_lexicon_tags_17 },
    { 0xdcb129c3, SUC_lexicon_tags_4 },
    { 0xe7b129c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189129cb, SUC_lexicon_tags_56 },
    { 0x871129cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61d129ce, SUC_lexicon_tags_39 },
    { 0xc7f129ce, SUC_lexicon_tags_15 },
    { 0x5a3129d0, SUC_lexicon_tags_7 },
    { 0xbc5129d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf9129d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27129d7, SUC_lexicon_tags_25 },
    { 0x9f5129d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd0f129da, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x61d129dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9129e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b129ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x153129f2, SUC_lexicon_tags_7 },
    { 0x3d3129f3, SUC_lexicon_tags_17 },
    { 0xa1d129f4, SUC_lexicon_tags_38 },
    { 0x7f7129f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4f129f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe9d129f9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb129ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c112a02, SUC_lexicon_tags_47 },
    { 0x19712a03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9e112a05, SUC_lexicon_tags_11 },
    { 0x51d12a06, SUC_lexicon_tags_2 },
    { 0x4df12a07, SUC_lexicon_tags_34 },
    { 0x6b912a07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbbd12a0a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78312a0e, SUC_lexicon_tags_47 },
    { 0x36712a0f, SUC_lexicon_tags_85 },
    { 0xcc712a0f, SUC_lexicon_tags_37 },
    { 0x1112a10, SUC_lexicon_tags_38 },
    { 0xa7b12a11, SUC_lexicon_tags_6 },
    { 0xd7512a13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x10312a16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xea912a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd12a1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8312a20, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65912a24, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e312a29, SUC_lexicon_tags_24 },
    { 0x7fb12a2a, SUC_lexicon_tags_17 },
    { 0x14f12a2b, SUC_lexicon_tags_63 },
    { 0x15112a2b, SUC_lexicon_tags_12 },
    { 0x56712a2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x91512a30, SUC_lexicon_tags_6 },
    { 0xc2f12a31, SUC_lexicon_tags_39 },
    { 0xaa112a31, SUC_lexicon_tags_1 },
    { 0x85d12a33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b12a37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ab12a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea512a3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0312a3e, SUC_lexicon_tags_60 },
    { 0xe9b12a3f, SUC_lexicon_tags_23 },
    { 0xe7112a40, SUC_lexicon_tags_16 },
    { 0x4d312a3e, SUC_lexicon_tags_46 },
    { 0x18b12a42, SUC_lexicon_tags_25 },
    { 0x51912a43, SUC_lexicon_tags_46 },
    { 0x5512a44, SUC_lexicon_tags_52 },
    { 0x8f912a45, SUC_lexicon_tags_60 },
    { 0x20d12a43, SUC_lexicon_tags_2 },
    { 0x4db12a3e, SUC_lexicon_tags_2 },
    { 0xf9312a48, SUC_lexicon_tags_6 },
    { 0xe9b12a49, SUC_lexicon_tags_17 },
    { 0x17112a49, SUC_lexicon_tags_11 },
    { 0x82d12a4b, SUC_lexicon_tags_10 },
    { 0xb1112a4c, SUC_lexicon_tags_25 },
    { 0x95112a4d, SUC_lexicon_tags_66 },
    { 0x60512a4e, SUC_lexicon_tags_19 },
    { 0x42f12a4c, SUC_lexicon_tags_12 },
    { 0x3e512a50, SUC_lexicon_tags_25 },
    { 0x74912a3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa2112a53, SUC_lexicon_tags_18 },
    { 0xfed12a54, SUC_lexicon_tags_25 },
    { 0x71912a55, SUC_lexicon_tags_26 },
    { 0x83912a56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20b12a58, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3ab12a5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d12a5e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x712a61, SUC_lexicon_tags_11 },
    { 0xa4712a62, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x88f12a64, SUC_lexicon_tags_14 },
    { 0x55912a64, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x10712a67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d512a6a, SUC_lexicon_tags_6 },
    { 0xfe912a6b, SUC_lexicon_tags_43 },
    { 0x96312a6c, SUC_lexicon_tags_9 },
    { 0x56312a6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab12a72, SUC_lexicon_tags_26 },
    { 0xc7f12a73, SUC_lexicon_tags_47 },
    { 0xadf12a74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf912a77, SUC_lexicon_tags_38 },
    { 0xa4b12a77, SUC_lexicon_tags_11 },
    { 0x25d12a79, SUC_lexicon_tags_1 },
    { 0x8b912a78, SUC_lexicon_tags_17 },
    { 0x15f12a7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b112a84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xea312a87, SUC_lexicon_tags_6 },
    { 0xe0112a87, SUC_lexicon_tags_7 },
    { 0x27d12a89, SUC_lexicon_tags_17 },
    { 0xa9f12a8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f112a8d, SUC_lexicon_tags_49 },
    { 0x32112a8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f912a91, SUC_lexicon_tags_21 },
    { 0x7b312a92, SUC_lexicon_tags_25 },
    { 0xb3d12a93, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51512a95, SUC_lexicon_tags_1 },
    { 0xb4312a96, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x11f12a98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f912a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8112aa2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd12aa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb12aa8, SUC_lexicon_tags_1 },
    { 0x21312aa8, SUC_lexicon_tags_2 },
    { 0xe512aaa, SUC_lexicon_tags_27 },
    { 0x1ff12aaa, SUC_lexicon_tags_11 },
    { 0x6b512aab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc912aae, SUC_lexicon_tags_11 },
    { 0xcf312aaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf512ab2, SUC_lexicon_tags_8 },
    { 0x2c912ab3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5b112ab5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b12ab9, SUC_lexicon_tags_29 },
    { 0x3cd12ab9, SUC_lexicon_tags_37 },
    { 0x42712abb, SUC_lexicon_tags_29 },
    { 0x49512abb, SUC_lexicon_tags_25 },
    { 0xc3b12abd, SUC_lexicon_tags_19 },
    { 0x3db12abe, SUC_lexicon_tags_25 },
    { 0x4d912abc, SUC_lexicon_tags_418 },
    { 0x1a512ac0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf312ac2, SUC_lexicon_tags_30 },
    { 0x96712ac3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd12ac7, SUC_lexicon_tags_12 },
    { 0xf4712ac8, SUC_lexicon_tags_2 },
    { 0x5912ac8, SUC_lexicon_tags_79 },
    { 0x2a912ac8, SUC_lexicon_tags_17 },
    { 0xc9d12ac8, SUC_lexicon_tags_29 },
    { 0x4f912acc, SUC_lexicon_tags_77 },
    { 0x23d12acc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f112acf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed12ad3, SUC_lexicon_tags_60 },
    { 0x19112ad3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49112ad7, SUC_lexicon_tags_63 },
    { 0xe3912ad8, SUC_lexicon_tags_3 },
    { 0xec112ad7, SUC_lexicon_tags_11 },
    { 0xda112ada, SUC_lexicon_tags_12 },
    { 0xda512ad8, SUC_lexicon_tags_1 },
    { 0x9af12adc, SUC_lexicon_tags_45 },
    { 0xf0712add, SUC_lexicon_tags_28 },
    { 0xd7b12add, SUC_lexicon_tags_12 },
    { 0x61312adf, SUC_lexicon_tags_1 },
    { 0x4ad12adf, SUC_lexicon_tags_47 },
    { 0x29512ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b12ae5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf12aea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d312aec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8eb12aee, SUC_lexicon_tags_55 },
    { 0xb7b12aef, SUC_lexicon_tags_66 },
    { 0x8312aef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d12af4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d912af7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd8112af9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47112b01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80912b03, SUC_lexicon_tags_49 },
    { 0xecd12b04, SUC_lexicon_tags_11 },
    { 0xb1512b03, SUC_lexicon_tags_49 },
    { 0x8cf12b06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x67f12b08, SUC_lexicon_tags_45 },
    { 0x4d312b08, SUC_lexicon_tags_29 },
    { 0xfcb12b0a, SUC_lexicon_tags_6 },
    { 0xc9912b0b, SUC_lexicon_tags_32 },
    { 0xd1712b09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e912b0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b12b12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e112b19, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x20112b1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f12b23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f12b2a, SUC_lexicon_tags_38 },
    { 0x24b12b2b, SUC_lexicon_tags_47 },
    { 0xfe512b2b, SUC_lexicon_tags_4 },
    { 0xf7512b2d, SUC_lexicon_tags_2 },
    { 0xc4512b2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x36d12b30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff12b36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf12b39, SUC_lexicon_tags_37 },
    { 0xbe112b3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf12b3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4a112b40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d312b45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9512b47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacd12b49, SUC_lexicon_tags_146 },
    { 0x87312b4a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf6112b4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18312b4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb6f12b51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42712b58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda312b5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2312b5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3912b60, SUC_lexicon_tags_7 },
    { 0x4312b61, SUC_lexicon_tags_14 },
    { 0xa1f12b62, SUC_lexicon_tags_16 },
    { 0xfc512b63, SUC_lexicon_tags_11 },
    { 0x33312b64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad712b66, SUC_lexicon_tags_7 },
    { 0xced12b66, SUC_lexicon_tags_82 },
    { 0x5e712b67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb12b6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8d12b6e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b12b78, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7912b7d, SUC_lexicon_tags_15 },
    { 0xe712b7d, SUC_lexicon_tags_38 },
    { 0xdd112b7f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5512b81, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d12b85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d12b8d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c912b95, SUC_lexicon_tags_0 },
    { 0x16112b96, SUC_lexicon_tags_103 },
    { 0x27312b95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9312b9b, SUC_lexicon_tags_0 },
    { 0x69512b9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81912b9f, SUC_lexicon_tags_0 },
    { 0x5312ba0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88512ba2, SUC_lexicon_tags_25 },
    { 0xee112ba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed112ba8, SUC_lexicon_tags_25 },
    { 0x3b312ba9, SUC_lexicon_tags_11 },
    { 0x95d12baa, SUC_lexicon_tags_61 },
    { 0x27512ba8, SUC_lexicon_tags_39 },
    { 0x73112bac, SUC_lexicon_tags_27 },
    { 0x87512bad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7f112baf, SUC_lexicon_tags_60 },
    { 0x99512bb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61712bb4, SUC_lexicon_tags_2 },
    { 0x77712bb4, SUC_lexicon_tags_11 },
    { 0xcaf12bb5, SUC_lexicon_tags_45 },
    { 0xb9312bb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f12bb9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6ed12bbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd12bc3, SUC_lexicon_tags_30 },
    { 0x2c312bc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40512bc6, SUC_lexicon_tags_11 },
    { 0x41112bc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac712bcd, SUC_lexicon_tags_36 },
    { 0x8c312bce, SUC_lexicon_tags_11 },
    { 0xf312bcd, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x1a712bd1, SUC_lexicon_tags_11 },
    { 0x70312bd2, SUC_lexicon_tags_7 },
    { 0x78912bd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa112bd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68f12bd7, SUC_lexicon_tags_14 },
    { 0x22712bd7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f112bdc, SUC_lexicon_tags_7 },
    { 0xe0312bdd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b12be2, SUC_lexicon_tags_1 },
    { 0xa4112be3, SUC_lexicon_tags_7 },
    { 0x38512be4, SUC_lexicon_tags_17 },
    { 0x1a112be4, SUC_lexicon_tags_11 },
    { 0x6bf12be6, SUC_lexicon_tags_37 },
    { 0xf512be7, SUC_lexicon_tags_11 },
    { 0xbe712be8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14912beb, SUC_lexicon_tags_234 },
    { 0x5f312bec, SUC_lexicon_tags_2 },
    { 0x52512bed, SUC_lexicon_tags_25 },
    { 0x33512bed, SUC_lexicon_tags_23 },
    { 0xae512bed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2af12bf1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xce712bf3, SUC_lexicon_tags_25 },
    { 0xf2112bf3, SUC_lexicon_tags_11 },
    { 0x77512bf5, SUC_lexicon_tags_33 },
    { 0xd0d12bf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd712bfb, SUC_lexicon_tags_32 },
    { 0xd2512bfc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5bd12bfe, SUC_lexicon_tags_30 },
    { 0xef12bfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3912c01, SUC_lexicon_tags_56 },
    { 0xf5112c01, SUC_lexicon_tags_11 },
    { 0x64912c02, SUC_lexicon_tags_15 },
    { 0x27112c02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b12c07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xadd12c09, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xcb912c0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95112c0d, SUC_lexicon_tags_5 },
    { 0xcb112c0e, SUC_lexicon_tags_37 },
    { 0xd9312c0f, SUC_lexicon_tags_49 },
    { 0xc0d12c0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6112c12, SUC_lexicon_tags_31 },
    { 0x94b12c13, SUC_lexicon_tags_8 },
    { 0xa2d12c12, SUC_lexicon_tags_8 },
    { 0x77112c15, SUC_lexicon_tags_34 },
    { 0x40112c13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9312c1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94712c1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b712c24, SUC_lexicon_tags_8 },
    { 0xb0712c24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3cd12c27, SUC_lexicon_tags_3 },
    { 0xb9712c28, SUC_lexicon_tags_23 },
    { 0x34712c27, SUC_lexicon_tags_2 },
    { 0x3e512c28, SUC_lexicon_tags_12 },
    { 0x3e912c2b, SUC_lexicon_tags_29 },
    { 0x4e512c2c, SUC_lexicon_tags_12 },
    { 0x9b112c28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f12c32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f512c35, SUC_lexicon_tags_90 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75712c3c, SUC_lexicon_tags_17 },
    { 0x34112c3d, SUC_lexicon_tags_9 },
    { 0xb3d12c3e, SUC_lexicon_tags_937 },
    { 0xff512c3d, SUC_lexicon_tags_1 },
    { 0xbdd12c40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d12c45, SUC_lexicon_tags_37 },
    { 0x16712c46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30f12c48, SUC_lexicon_tags_1 },
    { 0x29112c49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab12c4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b12c50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe112c56, SUC_lexicon_tags_23 },
    { 0xf5712c56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0112c5d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d12c60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9312c62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x56f12c64, SUC_lexicon_tags_19 },
    { 0x66312c65, SUC_lexicon_tags_8 },
    { 0xe5b12c65, SUC_lexicon_tags_6 },
    { 0x16312c64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5712c6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd12c6f, SUC_lexicon_tags_17 },
    { 0xe4d12c70, SUC_lexicon_tags_33 },
    { 0xbd12c71, SUC_lexicon_tags_25 },
    { 0xd1b12c72, SUC_lexicon_tags_80 },
    { 0x5e312c73, SUC_lexicon_tags_1 },
    { 0x6712c72, SUC_lexicon_tags_43 },
    { 0x4c512c75, SUC_lexicon_tags_2 },
    { 0x39d12c76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d12c7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8512c7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x91312c7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa2112c81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa9512c83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a112c85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9df12c87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b12c8b, SUC_lexicon_tags_55 },
    { 0xf7912c8b, SUC_lexicon_tags_14 },
    { 0xce912c8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7112c90, SUC_lexicon_tags_47 },
    { 0xa4d12c90, SUC_lexicon_tags_15 },
    { 0xbd112c92, SUC_lexicon_tags_15 },
    { 0x61f12c93, SUC_lexicon_tags_14 },
    { 0x31712c94, SUC_lexicon_tags_25 },
    { 0x21f12c91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f12c97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb712c99, SUC_lexicon_tags_7 },
    { 0x56112c9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87b12c9c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45312ca0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0112ca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa312ca5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52d12ca7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd112caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbeb12cac, SUC_lexicon_tags_33 },
    { 0xa6d12cac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb12cb0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf712cb5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6312cb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5512cc0, SUC_lexicon_tags_28 },
    { 0x1a112cc0, SUC_lexicon_tags_44 },
    { 0xd712cc2, SUC_lexicon_tags_27 },
    { 0x8bb12cc1, SUC_lexicon_tags_27 },
    { 0x1a912cc0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34912ccc, SUC_lexicon_tags_0 },
    { 0xc112ccd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9912ccf, SUC_lexicon_tags_27 },
    { 0xa8b12cd0, SUC_lexicon_tags_0 },
    { 0x3c512cd0, SUC_lexicon_tags_13 },
    { 0x3cd12cd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7c112cd4, SUC_lexicon_tags_8 },
    { 0x4b912cd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc4312cd7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x55b12cd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97312cdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d12cdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95f12ce1, SUC_lexicon_tags_39 },
    { 0xa6112ce1, SUC_lexicon_tags_17 },
    { 0xd0312ce1, SUC_lexicon_tags_11 },
    { 0xf4712ce3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b12ce7, SUC_lexicon_tags_1 },
    { 0x6f912ce8, SUC_lexicon_tags_28 },
    { 0x5dd12ce8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45712ceb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c312cee, SUC_lexicon_tags_2 },
    { 0xfe912cef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaeb12cf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75712cf3, SUC_lexicon_tags_14 },
    { 0x3fd12cf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbb12cf6, SUC_lexicon_tags_15 },
    { 0x10b12cf7, SUC_lexicon_tags_27 },
    { 0x48912cf6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8912cfb, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62112d00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52b12d02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f712d04, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x65112d06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b12d0b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c712d0e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f312d10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd12d15, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f12d1c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf2312d1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b12d22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9eb12d24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e312d28, SUC_lexicon_tags_1 },
    { 0xa0112d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18d12d2b, SUC_lexicon_tags_25 },
    { 0xb6f12d2c, SUC_lexicon_tags_12 },
    { 0x8d112d2c, SUC_lexicon_tags_45 },
    { 0xa9312d2c, SUC_lexicon_tags_37 },
    { 0x35712d2f, SUC_lexicon_tags_23 },
    { 0xb2912d2d, SUC_lexicon_tags_15 },
    { 0x3b12d31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32112d36, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c912d3a, SUC_lexicon_tags_75 },
    { 0xf4512d3b, SUC_lexicon_tags_27 },
    { 0x76712d3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b12d40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27712d46, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f12d49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf12d51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26512d53, SUC_lexicon_tags_23 },
    { 0x3bd12d54, SUC_lexicon_tags_40 },
    { 0xfe912d54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21312d59, SUC_lexicon_tags_47 },
    { 0x32b12d59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd12d60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef712d62, SUC_lexicon_tags_46 },
    { 0xc0112d63, SUC_lexicon_tags_2 },
    { 0x6912d64, SUC_lexicon_tags_45 },
    { 0xe7912d65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb912d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef12d6e, SUC_lexicon_tags_2 },
    { 0x4f912d6e, SUC_lexicon_tags_37 },
    { 0x5f12d70, SUC_lexicon_tags_12 },
    { 0xa9f12d71, SUC_lexicon_tags_37 },
    { 0x27b12d72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49912d74, SUC_lexicon_tags_40 },
    { 0xddf12d75, SUC_lexicon_tags_37 },
    { 0xadb12d76, SUC_lexicon_tags_11 },
    { 0xceb12d77, SUC_lexicon_tags_37 },
    { 0xf0912d74, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab12d7c, SUC_lexicon_tags_1 },
    { 0x16512d7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b12d81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd512d83, SUC_lexicon_tags_30 },
    { 0xca112d83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99b12d86, SUC_lexicon_tags_8 },
    { 0x2a112d87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5512d8b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f12d8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9112d91, SUC_lexicon_tags_27 },
    { 0x8b12d91, SUC_lexicon_tags_0 },
    { 0x3e112d93, SUC_lexicon_tags_39 },
    { 0xd6112d94, SUC_lexicon_tags_10 },
    { 0x2af12d95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25512d97, SUC_lexicon_tags_25 },
    { 0xb6512d98, SUC_lexicon_tags_43 },
    { 0x22912d97, SUC_lexicon_tags_37 },
    { 0xe8d12d9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d12da1, SUC_lexicon_tags_15 },
    { 0x52312da2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7912da4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d912da7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7912dac, SUC_lexicon_tags_4 },
    { 0x14912dad, SUC_lexicon_tags_11 },
    { 0xb0f12dae, SUC_lexicon_tags_23 },
    { 0xc4512daf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4a312db1, SUC_lexicon_tags_7 },
    { 0x65112db1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcc112db4, SUC_lexicon_tags_30 },
    { 0x3ad12db5, SUC_lexicon_tags_63 },
    { 0x84912db6, SUC_lexicon_tags_25 },
    { 0x9f912db5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b12dba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c512dbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdff12dbe, SUC_lexicon_tags_0 },
    { 0x73b12dbe, SUC_lexicon_tags_160 },
    { 0xf6912dc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d12dc5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc512dcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d12dcf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0512dd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3b12dd5, SUC_lexicon_tags_53 },
    { 0xd2512dd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46912dda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cd12ddc, SUC_lexicon_tags_15 },
    { 0xee712ddd, SUC_lexicon_tags_4 },
    { 0x69d12dde, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d12de1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb912de5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8712ded, SUC_lexicon_tags_7 },
    { 0xed912ded, SUC_lexicon_tags_30 },
    { 0xcb12def, SUC_lexicon_tags_23 },
    { 0xa912def, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeaf12df2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42712df6, SUC_lexicon_tags_7 },
    { 0x4c312df7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7f12df9, SUC_lexicon_tags_17 },
    { 0x5a312dfa, SUC_lexicon_tags_25 },
    { 0x8b12dfb, SUC_lexicon_tags_63 },
    { 0x4ab12dfc, SUC_lexicon_tags_38 },
    { 0x71912dfa, SUC_lexicon_tags_65 },
    { 0xf6712dfe, SUC_lexicon_tags_19 },
    { 0xeb712dff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8512e01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b12e06, SUC_lexicon_tags_25 },
    { 0xa512e07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6912e09, SUC_lexicon_tags_45 },
    { 0x1f112e0a, SUC_lexicon_tags_30 },
    { 0x98312e0b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61912e0f, SUC_lexicon_tags_7 },
    { 0x57b12e10, SUC_lexicon_tags_2 },
    { 0xda312e11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44112e1c, SUC_lexicon_tags_37 },
    { 0xea712e1d, SUC_lexicon_tags_17 },
    { 0xe7312e1e, SUC_lexicon_tags_65 },
    { 0x91d12e1f, SUC_lexicon_tags_1 },
    { 0xc7112e20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b12e27, SUC_lexicon_tags_48 },
    { 0x3c312e28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5912e2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63912e2d, SUC_lexicon_tags_38 },
    { 0xec712e2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f12e32, SUC_lexicon_tags_19 },
    { 0x9a112e32, SUC_lexicon_tags_9 },
    { 0x66912e34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd312e37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x26112e39, SUC_lexicon_tags_1 },
    { 0xbb12e3a, SUC_lexicon_tags_2 },
    { 0x1e912e3b, SUC_lexicon_tags_63 },
    { 0xe7712e3b, SUC_lexicon_tags_19 },
    { 0x4ef12e3d, SUC_lexicon_tags_79 },
    { 0x2912e3e, SUC_lexicon_tags_17 },
    { 0xae712e3f, SUC_lexicon_tags_9 },
    { 0xf8d12e40, SUC_lexicon_tags_17 },
    { 0xd8512e41, SUC_lexicon_tags_25 },
    { 0x63312e3f, SUC_lexicon_tags_0 },
    { 0x92f12e43, SUC_lexicon_tags_26 },
    { 0x76d12e44, SUC_lexicon_tags_33 },
    { 0x3a312e45, SUC_lexicon_tags_12 },
    { 0x68912e42, SUC_lexicon_tags_37 },
    { 0x63912e47, SUC_lexicon_tags_19 },
    { 0x2d712e48, SUC_lexicon_tags_15 },
    { 0xacf12e49, SUC_lexicon_tags_7 },
    { 0x28112e4a, SUC_lexicon_tags_25 },
    { 0xfcb12e49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b12e58, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b12e5b, SUC_lexicon_tags_27 },
    { 0xc0712e5c, SUC_lexicon_tags_53 },
    { 0xb8f12e5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2512e60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x84912e63, SUC_lexicon_tags_25 },
    { 0x3cd12e64, SUC_lexicon_tags_26 },
    { 0x47512e64, SUC_lexicon_tags_14 },
    { 0xb7912e66, SUC_lexicon_tags_55 },
    { 0x41b12e67, SUC_lexicon_tags_12 },
    { 0x53512e65, SUC_lexicon_tags_4 },
    { 0xf9112e66, SUC_lexicon_tags_29 },
    { 0xe3b12e6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd4512e6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40312e70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb912e73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e512e76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74312e79, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba712e7d, SUC_lexicon_tags_43 },
    { 0xd3d12e7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a112e80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd12e84, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb4f12e86, SUC_lexicon_tags_19 },
    { 0x2ff12e87, SUC_lexicon_tags_80 },
    { 0x6c512e88, SUC_lexicon_tags_58 },
    { 0x20112e89, SUC_lexicon_tags_49 },
    { 0xa4112e87, SUC_lexicon_tags_63 },
    { 0x19112e87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4312e8f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7112e91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee112e94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12912e99, SUC_lexicon_tags_23 },
    { 0x72512e9a, SUC_lexicon_tags_17 },
    { 0x45112e9a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c912eab, SUC_lexicon_tags_14 },
    { 0x11d12eac, SUC_lexicon_tags_2 },
    { 0x61712ead, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x84712eb0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x74d12eb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb712eb4, SUC_lexicon_tags_25 },
    { 0x5b512eb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83512eb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x22312eba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf12ebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44312ec1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6b12ec3, SUC_lexicon_tags_7 },
    { 0x46f12ec4, SUC_lexicon_tags_14 },
    { 0x70312ec5, SUC_lexicon_tags_11 },
    { 0xedb12ec5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f12ec9, SUC_lexicon_tags_11 },
    { 0x14b12eca, SUC_lexicon_tags_42 },
    { 0x30312ecb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd12ed1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69512ed4, SUC_lexicon_tags_30 },
    { 0x54512ed5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d112ed7, SUC_lexicon_tags_38 },
    { 0xb9912ed8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c912edc, SUC_lexicon_tags_25 },
    { 0x13712edd, SUC_lexicon_tags_10 },
    { 0x10f12ede, SUC_lexicon_tags_82 },
    { 0x6f912ede, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d12ee1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xded12ee3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f112ee8, SUC_lexicon_tags_7 },
    { 0x6f312ee9, SUC_lexicon_tags_14 },
    { 0xe9712eea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78312eef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39f12ef1, SUC_lexicon_tags_32 },
    { 0xc5512ef1, SUC_lexicon_tags_2 },
    { 0xd5112ef2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37112ef7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x45512ef9, SUC_lexicon_tags_63 },
    { 0x21b12efa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d112efd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e712f05, SUC_lexicon_tags_15 },
    { 0x78712f06, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x49312f08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cb12f0a, SUC_lexicon_tags_17 },
    { 0x61512f0b, SUC_lexicon_tags_7 },
    { 0xa6f12f0b, SUC_lexicon_tags_25 },
    { 0xe8112f0d, SUC_lexicon_tags_37 },
    { 0x46912f0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73b12f10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9912f12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51112f16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f12f1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9912f1e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb912f24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1eb12f26, SUC_lexicon_tags_14 },
    { 0x16712f27, SUC_lexicon_tags_4 },
    { 0x3312f26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80512f2d, SUC_lexicon_tags_0 },
    { 0xfd712f2e, SUC_lexicon_tags_37 },
    { 0x5f512f2f, SUC_lexicon_tags_33 },
    { 0xcd112f2f, SUC_lexicon_tags_37 },
    { 0xb8512f30, SUC_lexicon_tags_17 },
    { 0x63712f31, SUC_lexicon_tags_1 },
    { 0x42f12f2f, SUC_lexicon_tags_12 },
    { 0xdf312f33, SUC_lexicon_tags_8 },
    { 0xac112f2e, SUC_lexicon_tags_30 },
    { 0xa7f12f36, SUC_lexicon_tags_27 },
    { 0x45f12f37, SUC_lexicon_tags_12 },
    { 0x2eb12f38, SUC_lexicon_tags_17 },
    { 0x88512f37, SUC_lexicon_tags_33 },
    { 0x42912f3a, SUC_lexicon_tags_25 },
    { 0x90112f3a, SUC_lexicon_tags_38 },
    { 0xcc112f3b, SUC_lexicon_tags_65 },
    { 0xd7312f39, SUC_lexicon_tags_9 },
    { 0xa9f12f38, SUC_lexicon_tags_2 },
    { 0x27b12f38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d912f43, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7712f4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f12f50, SUC_lexicon_tags_38 },
    { 0x75312f51, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xe5b12f53, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x11512f55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b112f58, SUC_lexicon_tags_9 },
    { 0x9fd12f58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7112f5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x86512f5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf912f62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd112f64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3512f68, SUC_lexicon_tags_1 },
    { 0xfbb12f69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5712f72, SUC_lexicon_tags_14 },
    { 0xc5712f73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3112f78, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf312f7b, SUC_lexicon_tags_11 },
    { 0x8f312f7b, SUC_lexicon_tags_2 },
    { 0x2eb12f7d, SUC_lexicon_tags_2 },
    { 0xbd312f7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df12f81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d12f88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6712f8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c712f8d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd12f90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92912f92, SUC_lexicon_tags_47 },
    { 0xa0512f93, SUC_lexicon_tags_38 },
    { 0x1f912f94, SUC_lexicon_tags_45 },
    { 0x83b12f95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf712f9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2512f9f, SUC_lexicon_tags_38 },
    { 0x5db12fa0, SUC_lexicon_tags_1 },
    { 0xd7d12fa1, SUC_lexicon_tags_25 },
    { 0x81312fa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89112fab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b12fb9, SUC_lexicon_tags_27 },
    { 0x2df12fb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f12fbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e712fc3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4112fc6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d12fca, SUC_lexicon_tags_56 },
    { 0x96312fca, SUC_lexicon_tags_2 },
    { 0x97312fca, SUC_lexicon_tags_30 },
    { 0x87712fcd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2112fd0, SUC_lexicon_tags_39 },
    { 0x36f12fd0, SUC_lexicon_tags_1 },
    { 0x2b912fd2, SUC_lexicon_tags_17 },
    { 0xf8112fd2, SUC_lexicon_tags_37 },
    { 0x1c312fd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf912fd6, SUC_lexicon_tags_151 },
    { 0x6fd12fd7, SUC_lexicon_tags_38 },
    { 0xfd912fd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d512fda, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x13b12fdc, SUC_lexicon_tags_63 },
    { 0xfcd12fdd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xac312fdf, SUC_lexicon_tags_37 },
    { 0x92712fe0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c712fe9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9512fee, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1ed12ff0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2312ff3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5912ff9, SUC_lexicon_tags_0 },
    { 0xc1512ffa, SUC_lexicon_tags_37 },
    { 0xf1d12ff9, SUC_lexicon_tags_37 },
    { 0x29f12ffb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x62d12ffe, SUC_lexicon_tags_25 },
    { 0xd7112fff, SUC_lexicon_tags_37 },
    { 0x8513000, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b13004, SUC_lexicon_tags_65 },
    { 0xd0f13004, SUC_lexicon_tags_17 },
    { 0x9e313004, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e313008, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xabd1300a, SUC_lexicon_tags_15 },
    { 0xf411300b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31300f, SUC_lexicon_tags_47 },
    { 0xe2f13010, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77313014, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1b13016, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb811301a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2591301e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbe313020, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6513023, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f13026, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa1f13028, SUC_lexicon_tags_17 },
    { 0xa3f13029, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3591302d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7571302f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1513033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e513039, SUC_lexicon_tags_37 },
    { 0x15913039, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9291303e, SUC_lexicon_tags_63 },
    { 0x16b1303e, SUC_lexicon_tags_23 },
    { 0x18d1303e, SUC_lexicon_tags_25 },
    { 0xa7713041, SUC_lexicon_tags_0 },
    { 0x39f13042, SUC_lexicon_tags_37 },
    { 0x1ef13043, SUC_lexicon_tags_9 },
    { 0xf0113044, SUC_lexicon_tags_79 },
    { 0x30913045, SUC_lexicon_tags_37 },
    { 0xf6b1303f, SUC_lexicon_tags_79 },
    { 0x15313047, SUC_lexicon_tags_9 },
    { 0xf13045, SUC_lexicon_tags_21 },
    { 0xf4113049, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9051304d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0713055, SUC_lexicon_tags_40 },
    { 0xeb913056, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971305b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7ab1305d, SUC_lexicon_tags_82 },
    { 0xe151305e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82713066, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3913069, SUC_lexicon_tags_39 },
    { 0xf1d1306a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f1306e, SUC_lexicon_tags_7 },
    { 0x1fd1306f, SUC_lexicon_tags_37 },
    { 0x3db13070, SUC_lexicon_tags_11 },
    { 0x9ef1306e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d13075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f513077, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc713079, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbd11307b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a913080, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8011308a, SUC_lexicon_tags_56 },
    { 0xe51308b, SUC_lexicon_tags_25 },
    { 0x70f1308c, SUC_lexicon_tags_28 },
    { 0x5951308d, SUC_lexicon_tags_49 },
    { 0x4371308a, SUC_lexicon_tags_66 },
    { 0xccd1308f, SUC_lexicon_tags_15 },
    { 0x88513090, SUC_lexicon_tags_30 },
    { 0xb871308a, SUC_lexicon_tags_45 },
    { 0xf7313092, SUC_lexicon_tags_1 },
    { 0x3031308b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3dd13095, SUC_lexicon_tags_15 },
    { 0x2eb13095, SUC_lexicon_tags_46 },
    { 0xb5913096, SUC_lexicon_tags_12 },
    { 0xa0313098, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff1309d, SUC_lexicon_tags_43 },
    { 0x1fb1309e, SUC_lexicon_tags_0 },
    { 0xf971309f, SUC_lexicon_tags_1 },
    { 0x6dd130a0, SUC_lexicon_tags_75 },
    { 0xab5130a0, SUC_lexicon_tags_33 },
    { 0xad7130a2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951130a7, SUC_lexicon_tags_77 },
    { 0x6e5130a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef130b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb130b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7130ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa07130bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5130c1, SUC_lexicon_tags_65 },
    { 0x5d130c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b130ca, SUC_lexicon_tags_49 },
    { 0x9f3130ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x437130ce, SUC_lexicon_tags_13 },
    { 0x29f130cf, SUC_lexicon_tags_38 },
    { 0xe7d130cf, SUC_lexicon_tags_15 },
    { 0xfe3130d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfd9130d3, SUC_lexicon_tags_69 },
    { 0x463130d4, SUC_lexicon_tags_68 },
    { 0x70b130d5, SUC_lexicon_tags_40 },
    { 0xd8d130d6, SUC_lexicon_tags_7 },
    { 0x5ff130d7, SUC_lexicon_tags_63 },
    { 0x811130d7, SUC_lexicon_tags_47 },
    { 0x4f3130d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x305130db, SUC_lexicon_tags_1 },
    { 0xb99130db, SUC_lexicon_tags_15 },
    { 0x12d130dd, SUC_lexicon_tags_68 },
    { 0xf25130de, SUC_lexicon_tags_37 },
    { 0x629130dd, SUC_lexicon_tags_25 },
    { 0xf39130db, SUC_lexicon_tags_17 },
    { 0xe8d130e1, SUC_lexicon_tags_3 },
    { 0xe0f130e1, SUC_lexicon_tags_2 },
    { 0x5b7130df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d130f0, SUC_lexicon_tags_0 },
    { 0xe19130f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x881130f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72d130f6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29130fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f130ff, SUC_lexicon_tags_15 },
    { 0xcc313100, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb751310b, SUC_lexicon_tags_10 },
    { 0xe3f1310c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd811310e, SUC_lexicon_tags_9 },
    { 0xded1310e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f13112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80313114, SUC_lexicon_tags_15 },
    { 0xc4513115, SUC_lexicon_tags_34 },
    { 0x51113116, SUC_lexicon_tags_17 },
    { 0xc2113116, SUC_lexicon_tags_37 },
    { 0x40913114, SUC_lexicon_tags_32 },
    { 0x72913117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b1311c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df1311f, SUC_lexicon_tags_27 },
    { 0xf351311f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad13124, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4cb13126, SUC_lexicon_tags_11 },
    { 0xea513127, SUC_lexicon_tags_6 },
    { 0x17313128, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43913132, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3371313a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0b1313c, SUC_lexicon_tags_1 },
    { 0xf031313d, SUC_lexicon_tags_55 },
    { 0xf031313c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb13144, SUC_lexicon_tags_45 },
    { 0xfdd13145, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x70913148, SUC_lexicon_tags_19 },
    { 0xed113149, SUC_lexicon_tags_14 },
    { 0xc4b1314a, SUC_lexicon_tags_34 },
    { 0xb7b1314b, SUC_lexicon_tags_37 },
    { 0x9f71314b, SUC_lexicon_tags_68 },
    { 0x2d51314b, SUC_lexicon_tags_68 },
    { 0x9ef1314e, SUC_lexicon_tags_35 },
    { 0x3c91314f, SUC_lexicon_tags_11 },
    { 0x86f13150, SUC_lexicon_tags_15 },
    { 0x97713150, SUC_lexicon_tags_17 },
    { 0xb6b13150, SUC_lexicon_tags_37 },
    { 0x1fb13153, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x24313155, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4c513157, SUC_lexicon_tags_26 },
    { 0x87113158, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d91315c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeaf1315e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a113162, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f13168, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc471316b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8113170, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f13176, SUC_lexicon_tags_11 },
    { 0x8ed13176, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd1317d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff13185, SUC_lexicon_tags_37 },
    { 0x8c713186, SUC_lexicon_tags_12 },
    { 0x61313185, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b1318a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65113190, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12f13192, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa713195, SUC_lexicon_tags_32 },
    { 0xbeb13196, SUC_lexicon_tags_15 },
    { 0x7ed13197, SUC_lexicon_tags_17 },
    { 0x66113197, SUC_lexicon_tags_7 },
    { 0xca313196, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x731319b, SUC_lexicon_tags_8 },
    { 0x371319b, SUC_lexicon_tags_7 },
    { 0x2b71319d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09131a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35131a7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b131aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1df131ac, SUC_lexicon_tags_15 },
    { 0xeb131ad, SUC_lexicon_tags_11 },
    { 0x3f131ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f131b5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x779131b7, SUC_lexicon_tags_25 },
    { 0xf17131b8, SUC_lexicon_tags_1 },
    { 0xae7131b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d131be, SUC_lexicon_tags_45 },
    { 0xa79131bf, SUC_lexicon_tags_7 },
    { 0x4b5131bf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17131c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd39131c5, SUC_lexicon_tags_46 },
    { 0xd75131c6, SUC_lexicon_tags_48 },
    { 0x567131c7, SUC_lexicon_tags_33 },
    { 0xc37131c8, SUC_lexicon_tags_62 },
    { 0x65b131c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x503131cb, SUC_lexicon_tags_14 },
    { 0xf01131cc, SUC_lexicon_tags_0 },
    { 0x9f3131cd, SUC_lexicon_tags_63 },
    { 0x617131cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801131d5, SUC_lexicon_tags_37 },
    { 0x6f1131d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x593131d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf131dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4d131de, SUC_lexicon_tags_25 },
    { 0xa91131df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76f131e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3131e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x903131e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b131ed, SUC_lexicon_tags_11 },
    { 0xce1131ed, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95131f1, SUC_lexicon_tags_1 },
    { 0x745131f2, SUC_lexicon_tags_16 },
    { 0x5f5131f3, SUC_lexicon_tags_17 },
    { 0x17f131f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee5131f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df13202, SUC_lexicon_tags_24 },
    { 0xd113203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5571320c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d13210, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f13213, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x87913215, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad1321f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8913225, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xed513228, SUC_lexicon_tags_27 },
    { 0x8b713228, SUC_lexicon_tags_9 },
    { 0xb7d13229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f1322e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf313230, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd13234, SUC_lexicon_tags_45 },
    { 0xb5513234, SUC_lexicon_tags_2 },
    { 0xe2913236, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf1323c, SUC_lexicon_tags_39 },
    { 0xf5b1323d, SUC_lexicon_tags_9 },
    { 0xbc31323e, SUC_lexicon_tags_11 },
    { 0x21d1323f, SUC_lexicon_tags_8 },
    { 0x331323f, SUC_lexicon_tags_61 },
    { 0xcff13241, SUC_lexicon_tags_15 },
    { 0xad91323f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf313248, SUC_lexicon_tags_26 },
    { 0x24513249, SUC_lexicon_tags_37 },
    { 0x83f13248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3d1324c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4113252, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ab13254, SUC_lexicon_tags_17 },
    { 0xf1d13254, SUC_lexicon_tags_0 },
    { 0xa8713254, SUC_lexicon_tags_15 },
    { 0x4cd13255, SUC_lexicon_tags_17 },
    { 0xf3b13256, SUC_lexicon_tags_15 },
    { 0x95313254, SUC_lexicon_tags_21 },
    { 0xe1b1325a, SUC_lexicon_tags_7 },
    { 0x2771325a, SUC_lexicon_tags_27 },
    { 0x46f1325c, SUC_lexicon_tags_33 },
    { 0xa111325d, SUC_lexicon_tags_12 },
    { 0xa6913254, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab913264, SUC_lexicon_tags_1 },
    { 0xb2113265, SUC_lexicon_tags_32 },
    { 0x95113266, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b1326d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c513275, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb913278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb1327b, SUC_lexicon_tags_37 },
    { 0xe151327c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf5d1327e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d13282, SUC_lexicon_tags_1 },
    { 0x86913282, SUC_lexicon_tags_12 },
    { 0x6db13284, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x72713286, SUC_lexicon_tags_1 },
    { 0x38713287, SUC_lexicon_tags_1 },
    { 0xf6513288, SUC_lexicon_tags_15 },
    { 0x10313288, SUC_lexicon_tags_56 },
    { 0xfff1328a, SUC_lexicon_tags_1 },
    { 0x3471328b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf313292, SUC_lexicon_tags_6 },
    { 0xf1513293, SUC_lexicon_tags_11 },
    { 0xfc713293, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x21913297, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf351329b, SUC_lexicon_tags_37 },
    { 0x9511329b, SUC_lexicon_tags_172 },
    { 0x51f1329d, SUC_lexicon_tags_7 },
    { 0x2051329c, SUC_lexicon_tags_17 },
    { 0xd71329f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x451132a2, SUC_lexicon_tags_7 },
    { 0x2d5132a3, SUC_lexicon_tags_11 },
    { 0x7ad132a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1132a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b132ad, SUC_lexicon_tags_2 },
    { 0xa43132ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67132b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53132b3, SUC_lexicon_tags_7 },
    { 0xb09132b3, SUC_lexicon_tags_15 },
    { 0xab3132b3, SUC_lexicon_tags_7 },
    { 0x23d132b6, SUC_lexicon_tags_91 },
    { 0x103132b6, SUC_lexicon_tags_33 },
    { 0x5c3132b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad132be, SUC_lexicon_tags_87 },
    { 0x727132bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f132c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705132ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf132cf, SUC_lexicon_tags_45 },
    { 0xf7d132cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d132d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1132d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d132d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f132de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e5132e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5132e3, SUC_lexicon_tags_1 },
    { 0x4b132e3, SUC_lexicon_tags_37 },
    { 0xb31132e3, SUC_lexicon_tags_0 },
    { 0x94d132e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b132eb, SUC_lexicon_tags_14 },
    { 0xa5132ec, SUC_lexicon_tags_12 },
    { 0x461132eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab132f2, SUC_lexicon_tags_12 },
    { 0xa51132f3, SUC_lexicon_tags_1 },
    { 0x7f132f2, SUC_lexicon_tags_38 },
    { 0x449132f5, SUC_lexicon_tags_25 },
    { 0xc25132f5, SUC_lexicon_tags_55 },
    { 0x7d132f7, SUC_lexicon_tags_7 },
    { 0x925132f2, SUC_lexicon_tags_17 },
    { 0x73f132f9, SUC_lexicon_tags_17 },
    { 0xcc7132fa, SUC_lexicon_tags_37 },
    { 0x921132fa, SUC_lexicon_tags_6 },
    { 0xfb5132fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1fb132fe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6c113300, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5513303, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2313307, SUC_lexicon_tags_53 },
    { 0x3bb13308, SUC_lexicon_tags_38 },
    { 0x84d13309, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f13310, SUC_lexicon_tags_12 },
    { 0xa3d13311, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16913314, SUC_lexicon_tags_30 },
    { 0x8e113314, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b13319, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe931331f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x25b13321, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9913326, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae91332e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0d13330, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2b513332, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f713336, SUC_lexicon_tags_49 },
    { 0x95513336, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b1333a, SUC_lexicon_tags_32 },
    { 0x2b91333b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b13340, SUC_lexicon_tags_47 },
    { 0x9b13341, SUC_lexicon_tags_15 },
    { 0x77f13341, SUC_lexicon_tags_53 },
    { 0x88713343, SUC_lexicon_tags_11 },
    { 0xbd113342, SUC_lexicon_tags_63 },
    { 0x96913345, SUC_lexicon_tags_17 },
    { 0xb0313345, SUC_lexicon_tags_38 },
    { 0xc4513347, SUC_lexicon_tags_28 },
    { 0x36f13347, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd711334a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d13350, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x59513352, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ed13354, SUC_lexicon_tags_0 },
    { 0x77113355, SUC_lexicon_tags_16 },
    { 0x86913354, SUC_lexicon_tags_12 },
    { 0x23513357, SUC_lexicon_tags_25 },
    { 0x3b513358, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xc131335a, SUC_lexicon_tags_19 },
    { 0xc751335a, SUC_lexicon_tags_99 },
    { 0x9e71335c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x631335f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b13362, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9513364, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb513367, SUC_lexicon_tags_7 },
    { 0xb6313367, SUC_lexicon_tags_37 },
    { 0xdf13367, SUC_lexicon_tags_160 },
    { 0x4291336a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdef1336c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b13371, SUC_lexicon_tags_2 },
    { 0x7ff13372, SUC_lexicon_tags_8 },
    { 0xc4713372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7313375, SUC_lexicon_tags_1 },
    { 0x89113375, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x9d713378, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd1337c, SUC_lexicon_tags_12 },
    { 0xc9b1337d, SUC_lexicon_tags_12 },
    { 0xbc31337d, SUC_lexicon_tags_37 },
    { 0x6d11337c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f313384, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42b13386, SUC_lexicon_tags_17 },
    { 0x9713387, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f1338c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4713391, SUC_lexicon_tags_10 },
    { 0xcf513392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d913395, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xba913397, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f71339e, SUC_lexicon_tags_16 },
    { 0x9871339f, SUC_lexicon_tags_38 },
    { 0x55b133a0, SUC_lexicon_tags_7 },
    { 0xc37133a0, SUC_lexicon_tags_37 },
    { 0x905133a2, SUC_lexicon_tags_12 },
    { 0x9251339e, SUC_lexicon_tags_17 },
    { 0x73b133a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51133ae, SUC_lexicon_tags_16 },
    { 0xf49133af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1133b5, SUC_lexicon_tags_27 },
    { 0xdbf133b6, SUC_lexicon_tags_45 },
    { 0xbc7133b7, SUC_lexicon_tags_11 },
    { 0xb67133b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7ed133ba, SUC_lexicon_tags_25 },
    { 0xe5b133bb, SUC_lexicon_tags_1 },
    { 0x6af133bc, SUC_lexicon_tags_81 },
    { 0x59b133bd, SUC_lexicon_tags_64 },
    { 0x649133be, SUC_lexicon_tags_39 },
    { 0xc5b133bf, SUC_lexicon_tags_1 },
    { 0x489133bf, SUC_lexicon_tags_21 },
    { 0x8a7133c1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1133c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b133ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdff133cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c5133ce, SUC_lexicon_tags_69 },
    { 0xa01133ce, SUC_lexicon_tags_25 },
    { 0x3af133d0, SUC_lexicon_tags_28 },
    { 0x82f133d1, SUC_lexicon_tags_21 },
    { 0xea1133d2, SUC_lexicon_tags_82 },
    { 0x66d133ce, SUC_lexicon_tags_37 },
    { 0x40d133d4, SUC_lexicon_tags_61 },
    { 0x2d3133d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89133d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90b133da, SUC_lexicon_tags_17 },
    { 0xddd133db, SUC_lexicon_tags_19 },
    { 0x75133db, SUC_lexicon_tags_14 },
    { 0x39b133db, SUC_lexicon_tags_66 },
    { 0x1b9133dd, SUC_lexicon_tags_15 },
    { 0x39133df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7133e5, SUC_lexicon_tags_66 },
    { 0x391133e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b133e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7133ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1133f3, SUC_lexicon_tags_38 },
    { 0x557133f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ef133f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9133fa, SUC_lexicon_tags_46 },
    { 0x6c9133fa, SUC_lexicon_tags_12 },
    { 0xed7133fb, SUC_lexicon_tags_7 },
    { 0x12d133fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f13403, SUC_lexicon_tags_44 },
    { 0x27713404, SUC_lexicon_tags_26 },
    { 0x7f13405, SUC_lexicon_tags_9 },
    { 0x66313406, SUC_lexicon_tags_45 },
    { 0xb3f13407, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6513409, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11340d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d71340f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a513414, SUC_lexicon_tags_1 },
    { 0x8a713415, SUC_lexicon_tags_39 },
    { 0x5eb13416, SUC_lexicon_tags_37 },
    { 0xc4513417, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d1341f, SUC_lexicon_tags_14 },
    { 0xe5b1341f, SUC_lexicon_tags_16 },
    { 0x26d13421, SUC_lexicon_tags_10 },
    { 0xf0d13421, SUC_lexicon_tags_65 },
    { 0x80313423, SUC_lexicon_tags_0 },
    { 0x9b913424, SUC_lexicon_tags_7 },
    { 0x63f13424, SUC_lexicon_tags_15 },
    { 0x4c113425, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56f13428, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c51342e, SUC_lexicon_tags_29 },
    { 0x76b1342f, SUC_lexicon_tags_25 },
    { 0xdff1342f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7713432, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31513434, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af13441, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f713447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2091344a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe31344c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7713450, SUC_lexicon_tags_37 },
    { 0x41d13451, SUC_lexicon_tags_11 },
    { 0xdb713451, SUC_lexicon_tags_77 },
    { 0x2313451, SUC_lexicon_tags_140 },
    { 0x56513454, SUC_lexicon_tags_7 },
    { 0x86b13454, SUC_lexicon_tags_29 },
    { 0xc5b13454, SUC_lexicon_tags_2 },
    { 0x34313457, SUC_lexicon_tags_56 },
    { 0xac113458, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c71345b, SUC_lexicon_tags_28 },
    { 0x31345c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc51345e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d13463, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x78313466, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f1346b, SUC_lexicon_tags_37 },
    { 0xf2f1346c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfb1346e, SUC_lexicon_tags_60 },
    { 0xbc11346f, SUC_lexicon_tags_2 },
    { 0x69f13470, SUC_lexicon_tags_18 },
    { 0x33f1346f, SUC_lexicon_tags_0 },
    { 0x49f13472, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe513475, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1391347d, SUC_lexicon_tags_8 },
    { 0xd611347e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x83713481, SUC_lexicon_tags_12 },
    { 0x6af13482, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5913485, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b1348a, SUC_lexicon_tags_27 },
    { 0xf9f1348b, SUC_lexicon_tags_7 },
    { 0x9931348c, SUC_lexicon_tags_31 },
    { 0xf051348c, SUC_lexicon_tags_46 },
    { 0x4d51348b, SUC_lexicon_tags_31 },
    { 0xd5f1348f, SUC_lexicon_tags_7 },
    { 0xed11348d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac513492, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65d13494, SUC_lexicon_tags_25 },
    { 0x9b913495, SUC_lexicon_tags_17 },
    { 0xdc113495, SUC_lexicon_tags_15 },
    { 0x7e513495, SUC_lexicon_tags_39 },
    { 0x80513495, SUC_lexicon_tags_7 },
    { 0xdad13499, SUC_lexicon_tags_37 },
    { 0xdcb1349a, SUC_lexicon_tags_23 },
    { 0xdd713496, SUC_lexicon_tags_37 },
    { 0x3913494, SUC_lexicon_tags_68 },
    { 0xdb91349d, SUC_lexicon_tags_37 },
    { 0x55f1349d, SUC_lexicon_tags_37 },
    { 0x98f1349f, SUC_lexicon_tags_16 },
    { 0xee3134a0, SUC_lexicon_tags_33 },
    { 0xea71349f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893134aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf53134ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf85134ae, SUC_lexicon_tags_2 },
    { 0x415134af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5134b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x771134b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5134b8, SUC_lexicon_tags_26 },
    { 0x46b134b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf33134bb, SUC_lexicon_tags_48 },
    { 0xb31134bb, SUC_lexicon_tags_19 },
    { 0xe6b134bc, SUC_lexicon_tags_11 },
    { 0x8b1134be, SUC_lexicon_tags_26 },
    { 0x6f1134be, SUC_lexicon_tags_12 },
    { 0x657134bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d134c3, SUC_lexicon_tags_37 },
    { 0xf77134c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d134c9, SUC_lexicon_tags_14 },
    { 0x2d9134ca, SUC_lexicon_tags_37 },
    { 0x569134cb, SUC_lexicon_tags_1 },
    { 0x923134ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf134d7, SUC_lexicon_tags_12 },
    { 0x79f134d7, SUC_lexicon_tags_27 },
    { 0x937134d9, SUC_lexicon_tags_82 },
    { 0x981134da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x337134dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f134df, SUC_lexicon_tags_11 },
    { 0x9c9134df, SUC_lexicon_tags_15 },
    { 0xbf134df, SUC_lexicon_tags_37 },
    { 0x7f5134e1, SUC_lexicon_tags_37 },
    { 0xe07134e2, SUC_lexicon_tags_25 },
    { 0xb45134df, SUC_lexicon_tags_1 },
    { 0x3fd134e5, SUC_lexicon_tags_7 },
    { 0xaf3134e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5134e9, SUC_lexicon_tags_7 },
    { 0xedf134e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13134ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x74b134f1, SUC_lexicon_tags_4 },
    { 0xafb134f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19134f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3134f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9134fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab13503, SUC_lexicon_tags_37 },
    { 0xcd713504, SUC_lexicon_tags_37 },
    { 0x94f13504, SUC_lexicon_tags_11 },
    { 0xe2513504, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d13509, SUC_lexicon_tags_3 },
    { 0xde51350a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e31350e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d113515, SUC_lexicon_tags_11 },
    { 0xee513516, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d51351b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x77f1351d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b13522, SUC_lexicon_tags_17 },
    { 0x90f13522, SUC_lexicon_tags_26 },
    { 0xb7f13523, SUC_lexicon_tags_4 },
    { 0x9c713523, SUC_lexicon_tags_28 },
    { 0x26d13524, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc191352c, SUC_lexicon_tags_22 },
    { 0x7331352c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d13530, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d13534, SUC_lexicon_tags_37 },
    { 0x79313534, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ed13537, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae51353c, SUC_lexicon_tags_47 },
    { 0x6051353c, SUC_lexicon_tags_55 },
    { 0xc691353d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5313540, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1313543, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d13546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45313550, SUC_lexicon_tags_17 },
    { 0x79d13550, SUC_lexicon_tags_10 },
    { 0x10f13552, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xde113554, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31113559, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9411355b, SUC_lexicon_tags_15 },
    { 0xf4b1355c, SUC_lexicon_tags_11 },
    { 0x1671355c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf513561, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd13567, SUC_lexicon_tags_25 },
    { 0x56913567, SUC_lexicon_tags_25 },
    { 0x52913567, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa251356c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d13576, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbb13578, SUC_lexicon_tags_23 },
    { 0x93113579, SUC_lexicon_tags_1 },
    { 0xe313578, SUC_lexicon_tags_11 },
    { 0x2871357b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc751357e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac313580, SUC_lexicon_tags_8 },
    { 0x6c313581, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64713583, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f13586, SUC_lexicon_tags_12 },
    { 0xf4513587, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d1358b, SUC_lexicon_tags_17 },
    { 0x8b71358c, SUC_lexicon_tags_17 },
    { 0xd891358d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee11358f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d13592, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb13599, SUC_lexicon_tags_25 },
    { 0x88113599, SUC_lexicon_tags_12 },
    { 0x21d1359a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd135a1, SUC_lexicon_tags_77 },
    { 0xc37135a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe71135a4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23135a7, SUC_lexicon_tags_47 },
    { 0xa53135a8, SUC_lexicon_tags_1 },
    { 0xe3f135a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab135ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf9135b0, SUC_lexicon_tags_7 },
    { 0x9b135b1, SUC_lexicon_tags_37 },
    { 0x91b135b1, SUC_lexicon_tags_16 },
    { 0x4bb135b2, SUC_lexicon_tags_31 },
    { 0x175135b4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6e7135b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd135b9, SUC_lexicon_tags_1 },
    { 0xc27135ba, SUC_lexicon_tags_7 },
    { 0x58b135b9, SUC_lexicon_tags_1 },
    { 0x903135bc, SUC_lexicon_tags_1310 },
    { 0x925135bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x575135bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe1f135c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd135c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d135ca, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc3d135cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x853135cf, SUC_lexicon_tags_1 },
    { 0xe3d135d0, SUC_lexicon_tags_2 },
    { 0x7db135d1, SUC_lexicon_tags_1 },
    { 0xc61135d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f135d9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9f5135db, SUC_lexicon_tags_2 },
    { 0xfa7135db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d9135de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x755135e0, SUC_lexicon_tags_26 },
    { 0x85d135e1, SUC_lexicon_tags_11 },
    { 0x253135e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837135e9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9135ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87135f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xddb135f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2f7135f8, SUC_lexicon_tags_25 },
    { 0x8f7135f8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3135fe, SUC_lexicon_tags_22 },
    { 0xf93135ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4313602, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6713605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22913607, SUC_lexicon_tags_1 },
    { 0xa8913607, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x291360b, SUC_lexicon_tags_37 },
    { 0xf1f1360c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f13610, SUC_lexicon_tags_47 },
    { 0xa4713611, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe913618, SUC_lexicon_tags_7 },
    { 0x65713618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ad1361b, SUC_lexicon_tags_25 },
    { 0x53b1361c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8931361e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96513622, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f13626, SUC_lexicon_tags_197 },
    { 0x36d13626, SUC_lexicon_tags_7 },
    { 0x49f13628, SUC_lexicon_tags_27 },
    { 0xf3913628, SUC_lexicon_tags_25 },
    { 0x9a313627, SUC_lexicon_tags_10 },
    { 0x9991362b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ad1362d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8113632, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x18313634, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd7f13636, SUC_lexicon_tags_25 },
    { 0x45713636, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x25913639, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3391363b, SUC_lexicon_tags_29 },
    { 0xccd1363c, SUC_lexicon_tags_8 },
    { 0x1a31363d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81513645, SUC_lexicon_tags_10 },
    { 0x90713646, SUC_lexicon_tags_24 },
    { 0x2b513647, SUC_lexicon_tags_0 },
    { 0x75d13648, SUC_lexicon_tags_2 },
    { 0xae113647, SUC_lexicon_tags_60 },
    { 0xa011364a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f1364f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6b13651, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87b13653, SUC_lexicon_tags_43 },
    { 0xbb313653, SUC_lexicon_tags_3 },
    { 0xd1113655, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33113657, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd1365a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af1365e, SUC_lexicon_tags_11 },
    { 0xf5b1365f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55513662, SUC_lexicon_tags_654 },
    { 0, NULL },
    { 0x23d13664, SUC_lexicon_tags_37 },
    { 0x7cf13664, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9313667, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd1366a, SUC_lexicon_tags_66 },
    { 0x9b1366b, SUC_lexicon_tags_49 },
    { 0x2991366c, SUC_lexicon_tags_14 },
    { 0x771366b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x2191366f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d13672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4713674, SUC_lexicon_tags_37 },
    { 0x4bf13675, SUC_lexicon_tags_12 },
    { 0xec913676, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf5313678, SUC_lexicon_tags_17 },
    { 0x76b13678, SUC_lexicon_tags_26 },
    { 0x17d1367a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x32f1367c, SUC_lexicon_tags_172 },
    { 0x70b1367d, SUC_lexicon_tags_1 },
    { 0xd531367e, SUC_lexicon_tags_28 },
    { 0xef1367e, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4713682, SUC_lexicon_tags_15 },
    { 0xa7713683, SUC_lexicon_tags_124 },
    { 0xf2913684, SUC_lexicon_tags_11 },
    { 0xfcb13684, SUC_lexicon_tags_16 },
    { 0x2cf13683, SUC_lexicon_tags_37 },
    { 0xdd113686, SUC_lexicon_tags_30 },
    { 0x3ab13688, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb1368f, SUC_lexicon_tags_62 },
    { 0xd2313690, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb13692, SUC_lexicon_tags_79 },
    { 0xa8b13693, SUC_lexicon_tags_29 },
    { 0xfed13694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed313697, SUC_lexicon_tags_7 },
    { 0x5113698, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb11369a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d136a2, SUC_lexicon_tags_66 },
    { 0x631136a3, SUC_lexicon_tags_4 },
    { 0x58b136a4, SUC_lexicon_tags_15 },
    { 0x3a1136a5, SUC_lexicon_tags_77 },
    { 0x92b136a6, SUC_lexicon_tags_15 },
    { 0x3e9136a7, SUC_lexicon_tags_25 },
    { 0x445136a2, SUC_lexicon_tags_14 },
    { 0x981136a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989136ad, SUC_lexicon_tags_0 },
    { 0xb71136ae, SUC_lexicon_tags_11 },
    { 0xa3f136ae, SUC_lexicon_tags_2 },
    { 0xafd136ad, SUC_lexicon_tags_0 },
    { 0x8b9136b1, SUC_lexicon_tags_27 },
    { 0x915136b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827136b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fd136bb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb136c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2f3136c3, SUC_lexicon_tags_2 },
    { 0xafd136c4, SUC_lexicon_tags_17 },
    { 0x4cd136c4, SUC_lexicon_tags_9 },
    { 0x19d136c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d136cc, SUC_lexicon_tags_170 },
    { 0x28f136cd, SUC_lexicon_tags_21 },
    { 0xf01136ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7136d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f136d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b136db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9a5136dd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df136e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f136e8, SUC_lexicon_tags_6 },
    { 0x205136e9, SUC_lexicon_tags_45 },
    { 0x689136e9, SUC_lexicon_tags_12 },
    { 0x27b136e9, SUC_lexicon_tags_25 },
    { 0x729136e8, SUC_lexicon_tags_32 },
    { 0x30b136ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21136f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x547136f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93136f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471136fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e113700, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x16313702, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63913707, SUC_lexicon_tags_21 },
    { 0x97513708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f13711, SUC_lexicon_tags_38 },
    { 0x31713711, SUC_lexicon_tags_31 },
    { 0xa2b13711, SUC_lexicon_tags_7 },
    { 0xc9913711, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b1371d, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f13720, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x63313722, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59113728, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df1372e, SUC_lexicon_tags_7 },
    { 0xd4d1372f, SUC_lexicon_tags_39 },
    { 0xa2b13730, SUC_lexicon_tags_65 },
    { 0xb4713731, SUC_lexicon_tags_17 },
    { 0xe0d13731, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91913734, SUC_lexicon_tags_14 },
    { 0x7f713735, SUC_lexicon_tags_60 },
    { 0x86f13735, SUC_lexicon_tags_17 },
    { 0xda113735, SUC_lexicon_tags_11 },
    { 0x5913734, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b1373b, SUC_lexicon_tags_47 },
    { 0xba71373c, SUC_lexicon_tags_1 },
    { 0x5471373c, SUC_lexicon_tags_47 },
    { 0x2111373d, SUC_lexicon_tags_37 },
    { 0x1571373f, SUC_lexicon_tags_11 },
    { 0x5391373d, SUC_lexicon_tags_79 },
    { 0xb9713741, SUC_lexicon_tags_17 },
    { 0x92b1373e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b13745, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91513747, SUC_lexicon_tags_30 },
    { 0xdb13748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4711374a, SUC_lexicon_tags_34 },
    { 0xbed1374b, SUC_lexicon_tags_13 },
    { 0x2e11374c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f13751, SUC_lexicon_tags_6 },
    { 0x47513752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2913755, SUC_lexicon_tags_12 },
    { 0x95713755, SUC_lexicon_tags_70 },
    { 0x23d13757, SUC_lexicon_tags_28 },
    { 0x3b13758, SUC_lexicon_tags_15 },
    { 0x6ff13759, SUC_lexicon_tags_30 },
    { 0x6ad1375a, SUC_lexicon_tags_7 },
    { 0xa151375b, SUC_lexicon_tags_43 },
    { 0xd0513755, SUC_lexicon_tags_113 },
    { 0xa4713756, SUC_lexicon_tags_33 },
    { 0x94713759, SUC_lexicon_tags_68 },
    { 0x17d1375f, SUC_lexicon_tags_9 },
    { 0xb1f13758, SUC_lexicon_tags_37 },
    { 0xff313761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd13764, SUC_lexicon_tags_17 },
    { 0xb4313764, SUC_lexicon_tags_47 },
    { 0x1713764, SUC_lexicon_tags_14 },
    { 0x77f13767, SUC_lexicon_tags_56 },
    { 0xf0b13768, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7913775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c113777, SUC_lexicon_tags_12 },
    { 0xf0513778, SUC_lexicon_tags_7 },
    { 0xac513779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d1377c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb513783, SUC_lexicon_tags_7 },
    { 0x93113784, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad713789, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd11378e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75113790, SUC_lexicon_tags_7 },
    { 0x97d13790, SUC_lexicon_tags_17 },
    { 0x71b13792, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa2f13794, SUC_lexicon_tags_43 },
    { 0x45b13795, SUC_lexicon_tags_87 },
    { 0x14113796, SUC_lexicon_tags_0 },
    { 0x7113797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17313799, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8891379b, SUC_lexicon_tags_60 },
    { 0xfd1379c, SUC_lexicon_tags_68 },
    { 0xc391379d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575137aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7137af, SUC_lexicon_tags_25 },
    { 0x151137b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5137b3, SUC_lexicon_tags_25 },
    { 0x6fb137b4, SUC_lexicon_tags_30 },
    { 0x299137b5, SUC_lexicon_tags_33 },
    { 0xf77137b6, SUC_lexicon_tags_37 },
    { 0xb25137b7, SUC_lexicon_tags_12 },
    { 0x527137b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x405137bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf2b137bd, SUC_lexicon_tags_82 },
    { 0xa2d137bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7137c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f137c9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5137cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677137d2, SUC_lexicon_tags_60 },
    { 0x5f1137d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b137d6, SUC_lexicon_tags_37 },
    { 0x19d137d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4e5137d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3137e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x943137e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7f137e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb137e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1137ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5137f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb77137f4, SUC_lexicon_tags_68 },
    { 0xef5137f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab1137f7, SUC_lexicon_tags_118 },
    { 0xa01137f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7137fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafb137fd, SUC_lexicon_tags_21 },
    { 0xa97137fe, SUC_lexicon_tags_82 },
    { 0x4c7137ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42313805, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed113809, SUC_lexicon_tags_15 },
    { 0x5191380a, SUC_lexicon_tags_33 },
    { 0x47f1380b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad13812, SUC_lexicon_tags_54 },
    { 0xa9d13813, SUC_lexicon_tags_15 },
    { 0x2a913813, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc313817, SUC_lexicon_tags_45 },
    { 0x47313818, SUC_lexicon_tags_3 },
    { 0xf4913818, SUC_lexicon_tags_0 },
    { 0xdf91381a, SUC_lexicon_tags_17 },
    { 0x65f13819, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe11381e, SUC_lexicon_tags_38 },
    { 0x4651381f, SUC_lexicon_tags_14 },
    { 0xea513820, SUC_lexicon_tags_26 },
    { 0x4031381e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d13829, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d91382d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc531382f, SUC_lexicon_tags_43 },
    { 0x59d13830, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x80913832, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb13835, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe713837, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12513839, SUC_lexicon_tags_9 },
    { 0x1591383a, SUC_lexicon_tags_15 },
    { 0xe391383b, SUC_lexicon_tags_60 },
    { 0x6dd13839, SUC_lexicon_tags_0 },
    { 0x1c51383d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd13848, SUC_lexicon_tags_15 },
    { 0x76313848, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d1384c, SUC_lexicon_tags_15 },
    { 0xa631384d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63913852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d91385a, SUC_lexicon_tags_14 },
    { 0x9031385b, SUC_lexicon_tags_82 },
    { 0x9591385b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe513861, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x92113863, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c313866, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd1386c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d1386f, SUC_lexicon_tags_11 },
    { 0xb5d13870, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3113873, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54913875, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb13878, SUC_lexicon_tags_48 },
    { 0x5d713879, SUC_lexicon_tags_12 },
    { 0x8f71387a, SUC_lexicon_tags_37 },
    { 0xdab13878, SUC_lexicon_tags_1 },
    { 0xd3d13878, SUC_lexicon_tags_61 },
    { 0x2d71387d, SUC_lexicon_tags_15 },
    { 0xa6f1387b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ef13880, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c913885, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48b13887, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811388f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2313892, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a713896, SUC_lexicon_tags_0 },
    { 0xf6f13897, SUC_lexicon_tags_17 },
    { 0x5913897, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f1389d, SUC_lexicon_tags_17 },
    { 0x7491389e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87138a1, SUC_lexicon_tags_21 },
    { 0x7ed138a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb138a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71138b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3138bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c7138bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f9138bf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1138c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed138c7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55138ca, SUC_lexicon_tags_17 },
    { 0xeb5138cb, SUC_lexicon_tags_27 },
    { 0x1d1138ca, SUC_lexicon_tags_77 },
    { 0x77138cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe73138cf, SUC_lexicon_tags_9 },
    { 0xbad138d0, SUC_lexicon_tags_17 },
    { 0xfcf138d0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b138d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05138db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d138e1, SUC_lexicon_tags_47 },
    { 0xd4b138e2, SUC_lexicon_tags_2 },
    { 0xa19138e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf77138e5, SUC_lexicon_tags_15 },
    { 0x3ef138e6, SUC_lexicon_tags_2 },
    { 0xfff138e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f1138e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x58b138eb, SUC_lexicon_tags_77 },
    { 0xf21138ec, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5138ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b138f3, SUC_lexicon_tags_14 },
    { 0x82d138f4, SUC_lexicon_tags_1 },
    { 0x78d138f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63138f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55138fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x161138fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4913901, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5513903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe151390c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dd1390e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab13913, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc2913915, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f71391a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf1391f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb13923, SUC_lexicon_tags_84 },
    { 0x75b13924, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xac913926, SUC_lexicon_tags_66 },
    { 0xfc913926, SUC_lexicon_tags_1 },
    { 0xdbb13927, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc811392a, SUC_lexicon_tags_46 },
    { 0x2e51392b, SUC_lexicon_tags_8 },
    { 0x9f71392c, SUC_lexicon_tags_17 },
    { 0xef11392b, SUC_lexicon_tags_23 },
    { 0xfeb1392c, SUC_lexicon_tags_15 },
    { 0xfdb1392b, SUC_lexicon_tags_38 },
    { 0x94d1392f, SUC_lexicon_tags_12 },
    { 0xe1713931, SUC_lexicon_tags_34 },
    { 0x96f13931, SUC_lexicon_tags_12 },
    { 0x98913933, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa9113935, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd51393a, SUC_lexicon_tags_27 },
    { 0x1c71393b, SUC_lexicon_tags_11 },
    { 0x67d1393b, SUC_lexicon_tags_63 },
    { 0x2851393d, SUC_lexicon_tags_37 },
    { 0x4611393e, SUC_lexicon_tags_14 },
    { 0x41d1393f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6d913941, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd13944, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31513946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b1394c, SUC_lexicon_tags_11 },
    { 0xf131394d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58713951, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8313954, SUC_lexicon_tags_15 },
    { 0xaa713954, SUC_lexicon_tags_23 },
    { 0xb5713956, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb913959, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf1395c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39913960, SUC_lexicon_tags_43 },
    { 0x7df13961, SUC_lexicon_tags_83 },
    { 0x9d113962, SUC_lexicon_tags_27 },
    { 0xdd713961, SUC_lexicon_tags_25 },
    { 0x3d513961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1891396a, SUC_lexicon_tags_15 },
    { 0x2251396b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xae91396e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa6913970, SUC_lexicon_tags_60 },
    { 0x7bf13970, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b13979, SUC_lexicon_tags_25 },
    { 0x191397a, SUC_lexicon_tags_71 },
    { 0x3b1397b, SUC_lexicon_tags_11 },
    { 0x5611397c, SUC_lexicon_tags_46 },
    { 0xc91397c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4913980, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f913989, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b31398b, SUC_lexicon_tags_27 },
    { 0x5bb1398c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4113992, SUC_lexicon_tags_12 },
    { 0xcf513993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4af13995, SUC_lexicon_tags_12 },
    { 0x27913996, SUC_lexicon_tags_49 },
    { 0x20313997, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4851399c, SUC_lexicon_tags_25 },
    { 0x3b91399c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723139a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x339139a5, SUC_lexicon_tags_19 },
    { 0xf41139a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f139ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf139af, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05139b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd139bd, SUC_lexicon_tags_17 },
    { 0x2df139be, SUC_lexicon_tags_7 },
    { 0x905139bf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb139c4, SUC_lexicon_tags_17 },
    { 0x893139c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349139cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771139d0, SUC_lexicon_tags_26 },
    { 0x62f139d1, SUC_lexicon_tags_11 },
    { 0x12b139d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed139d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b139db, SUC_lexicon_tags_38 },
    { 0xf35139dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbcb139de, SUC_lexicon_tags_25 },
    { 0x19139de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7c9139e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94b139e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2f139e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1139e8, SUC_lexicon_tags_15 },
    { 0xc35139e9, SUC_lexicon_tags_11 },
    { 0x993139ea, SUC_lexicon_tags_9 },
    { 0xaad139eb, SUC_lexicon_tags_25 },
    { 0x87139ea, SUC_lexicon_tags_158 },
    { 0x7b5139ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03139f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f7139f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7139f7, SUC_lexicon_tags_25 },
    { 0xd07139f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb139fd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd9b139ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78113a03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97913a07, SUC_lexicon_tags_30 },
    { 0xf0313a08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f13a0b, SUC_lexicon_tags_82 },
    { 0x8ed13a0c, SUC_lexicon_tags_45 },
    { 0xf5513a0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x28113a10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaaf13a12, SUC_lexicon_tags_45 },
    { 0x99f13a12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40913a16, SUC_lexicon_tags_30 },
    { 0xbff13a16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb713a1e, SUC_lexicon_tags_23 },
    { 0xa5713a1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc313a21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22313a23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2f913a25, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb6f13a27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f13a29, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae513a2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf13a30, SUC_lexicon_tags_11 },
    { 0x26113a30, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63113a36, SUC_lexicon_tags_12 },
    { 0xfd513a37, SUC_lexicon_tags_87 },
    { 0x6c913a37, SUC_lexicon_tags_27 },
    { 0x48113a39, SUC_lexicon_tags_14 },
    { 0x1e913a3a, SUC_lexicon_tags_0 },
    { 0xab13a3b, SUC_lexicon_tags_25 },
    { 0xf8d13a3b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f713a44, SUC_lexicon_tags_0 },
    { 0x4fb13a45, SUC_lexicon_tags_1 },
    { 0x67713a46, SUC_lexicon_tags_15 },
    { 0x97d13a46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c913a4f, SUC_lexicon_tags_48 },
    { 0x9e313a50, SUC_lexicon_tags_14 },
    { 0x63713a51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a713a55, SUC_lexicon_tags_29 },
    { 0x53113a56, SUC_lexicon_tags_11 },
    { 0x73313a56, SUC_lexicon_tags_49 },
    { 0x2e713a58, SUC_lexicon_tags_17 },
    { 0x6ad13a57, SUC_lexicon_tags_15 },
    { 0xc9d13a5a, SUC_lexicon_tags_45 },
    { 0xed13a59, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba513a62, SUC_lexicon_tags_37 },
    { 0x12d13a63, SUC_lexicon_tags_14 },
    { 0x5b913a64, SUC_lexicon_tags_7 },
    { 0x74f13a64, SUC_lexicon_tags_60 },
    { 0x5cf13a64, SUC_lexicon_tags_15 },
    { 0xe9d13a67, SUC_lexicon_tags_38 },
    { 0xc5313a68, SUC_lexicon_tags_49 },
    { 0x54513a69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98313a6b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b313a71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9313a74, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2913a78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f713a7b, SUC_lexicon_tags_17 },
    { 0x5fb13a7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x71513a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10913a81, SUC_lexicon_tags_29 },
    { 0xe3713a81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x91b13a84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d513a86, SUC_lexicon_tags_37 },
    { 0x63313a87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89f13a89, SUC_lexicon_tags_1 },
    { 0x3bf13a8a, SUC_lexicon_tags_39 },
    { 0xd1f13a8b, SUC_lexicon_tags_17 },
    { 0x5b113a8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb13a90, SUC_lexicon_tags_37 },
    { 0x76d13a91, SUC_lexicon_tags_12 },
    { 0x7c513a92, SUC_lexicon_tags_63 },
    { 0xe4313a93, SUC_lexicon_tags_15 },
    { 0xfb713a94, SUC_lexicon_tags_2 },
    { 0xa0b13a91, SUC_lexicon_tags_11 },
    { 0x7e713a96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40b13a98, SUC_lexicon_tags_25 },
    { 0x5d13a98, SUC_lexicon_tags_15 },
    { 0xf6f13a9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb13a9d, SUC_lexicon_tags_28 },
    { 0x22713a9e, SUC_lexicon_tags_33 },
    { 0x74f13a9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1d313aa1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd13aa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa313aa9, SUC_lexicon_tags_80 },
    { 0xd3713aaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f313ab3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b13abd, SUC_lexicon_tags_84 },
    { 0x80313abe, SUC_lexicon_tags_37 },
    { 0x6b13abf, SUC_lexicon_tags_0 },
    { 0x1f113ac0, SUC_lexicon_tags_16 },
    { 0x10f13abf, SUC_lexicon_tags_71 },
    { 0x8c313ac0, SUC_lexicon_tags_15 },
    { 0x43513ac0, SUC_lexicon_tags_21 },
    { 0xb3713ac4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b113ac7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb13acb, SUC_lexicon_tags_32 },
    { 0x4cb13acc, SUC_lexicon_tags_6 },
    { 0x2c713acd, SUC_lexicon_tags_2 },
    { 0xb7113acc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45113ad1, SUC_lexicon_tags_7 },
    { 0xc6f13ad2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79113ad7, SUC_lexicon_tags_1 },
    { 0x6d713ad8, SUC_lexicon_tags_15 },
    { 0xbb913ad9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x90513adb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x46d13add, SUC_lexicon_tags_80 },
    { 0x52b13add, SUC_lexicon_tags_4 },
    { 0xb0113ade, SUC_lexicon_tags_47 },
    { 0x31313ae0, SUC_lexicon_tags_47 },
    { 0x5b713ae0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x37313ae3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a313ae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed513aed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa313af0, SUC_lexicon_tags_0 },
    { 0x63d13af0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3ab13af3, SUC_lexicon_tags_7 },
    { 0x7fb13af3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19913af6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58113af8, SUC_lexicon_tags_1 },
    { 0xb6713af9, SUC_lexicon_tags_46 },
    { 0xed913af9, SUC_lexicon_tags_2 },
    { 0xa5313afb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d13b00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa513b0c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b13b10, SUC_lexicon_tags_57 },
    { 0x4d113b11, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe513b15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20d13b17, SUC_lexicon_tags_0 },
    { 0x61113b18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22113b1d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49513b23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46513b26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x52313b28, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbd13b2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f13b2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f913b31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d13b35, SUC_lexicon_tags_11 },
    { 0xd7d13b36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57f13b38, SUC_lexicon_tags_38 },
    { 0x94513b39, SUC_lexicon_tags_8 },
    { 0x51d13b39, SUC_lexicon_tags_7 },
    { 0x18513b3b, SUC_lexicon_tags_45 },
    { 0x64513b3c, SUC_lexicon_tags_11 },
    { 0x7f313b3b, SUC_lexicon_tags_82 },
    { 0x89513b3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52113b41, SUC_lexicon_tags_15 },
    { 0x34313b41, SUC_lexicon_tags_216 },
    { 0x39d13b43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74513b45, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66513b4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e713b51, SUC_lexicon_tags_8 },
    { 0xa9713b52, SUC_lexicon_tags_106 },
    { 0xe6f13b53, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba713b5b, SUC_lexicon_tags_2 },
    { 0xe9713b5b, SUC_lexicon_tags_25 },
    { 0xc5113b5d, SUC_lexicon_tags_11 },
    { 0x25113b5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12513b60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d13b66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb913b6f, SUC_lexicon_tags_15 },
    { 0xba713b70, SUC_lexicon_tags_40 },
    { 0xd0f13b71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce313b74, SUC_lexicon_tags_60 },
    { 0x98f13b75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb13b7a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e113b7e, SUC_lexicon_tags_15 },
    { 0x2cf13b7e, SUC_lexicon_tags_9 },
    { 0x55713b7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf13b86, SUC_lexicon_tags_37 },
    { 0xb9913b87, SUC_lexicon_tags_12 },
    { 0x4b13b88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b513b8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0713b8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x18913b8e, SUC_lexicon_tags_30 },
    { 0xb1713b8f, SUC_lexicon_tags_37 },
    { 0x51f13b8e, SUC_lexicon_tags_79 },
    { 0xfb113b91, SUC_lexicon_tags_29 },
    { 0x80913b92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62d13b94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff513b98, SUC_lexicon_tags_11 },
    { 0x17f13b99, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65713ba0, SUC_lexicon_tags_14 },
    { 0xbf913ba0, SUC_lexicon_tags_1 },
    { 0x7b913ba1, SUC_lexicon_tags_3 },
    { 0xa0b13ba3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73713bab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b13bae, SUC_lexicon_tags_9 },
    { 0x6d113baf, SUC_lexicon_tags_7 },
    { 0xe8d13baf, SUC_lexicon_tags_25 },
    { 0x83b13bae, SUC_lexicon_tags_15 },
    { 0x15713bb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2713bb7, SUC_lexicon_tags_12 },
    { 0xedd13bb7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa313bbb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb313bc2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b13bc5, SUC_lexicon_tags_0 },
    { 0xba513bc5, SUC_lexicon_tags_8 },
    { 0x9d13bc7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xea113bca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x22313bcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdfd13bcf, SUC_lexicon_tags_0 },
    { 0xce913bd0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48113bd4, SUC_lexicon_tags_12 },
    { 0x5af13bd5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x7f913bd7, SUC_lexicon_tags_11 },
    { 0x6e513bd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43113bdc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9d13bde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf3513be0, SUC_lexicon_tags_37 },
    { 0xac713be1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x37113be4, SUC_lexicon_tags_19 },
    { 0x2bf13be5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d513be7, SUC_lexicon_tags_46 },
    { 0x4f513be8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d13beb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x67513bee, SUC_lexicon_tags_42 },
    { 0x2d513bef, SUC_lexicon_tags_63 },
    { 0x9ff13bef, SUC_lexicon_tags_31 },
    { 0x79513bf0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x57313bf4, SUC_lexicon_tags_37 },
    { 0xd3f13bf4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f13bfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54f13bff, SUC_lexicon_tags_37 },
    { 0x20113bff, SUC_lexicon_tags_11 },
    { 0xfa513c01, SUC_lexicon_tags_12 },
    { 0xb7b13c00, SUC_lexicon_tags_15 },
    { 0x50f13c03, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3a913c05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bf13c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e713c0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45713c0f, SUC_lexicon_tags_17 },
    { 0x41113c10, SUC_lexicon_tags_17 },
    { 0x3af13c0f, SUC_lexicon_tags_31 },
    { 0x55513c12, SUC_lexicon_tags_19 },
    { 0xb8d13c12, SUC_lexicon_tags_62 },
    { 0x4d113c12, SUC_lexicon_tags_60 },
    { 0x7a313c12, SUC_lexicon_tags_55 },
    { 0x39313c13, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb8913c18, SUC_lexicon_tags_38 },
    { 0xbdb13c19, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb8513c1b, SUC_lexicon_tags_17 },
    { 0x63713c1c, SUC_lexicon_tags_1 },
    { 0x94113c1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa313c20, SUC_lexicon_tags_7 },
    { 0x1f113c21, SUC_lexicon_tags_17 },
    { 0x87913c21, SUC_lexicon_tags_12 },
    { 0x4e113c23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x94913c25, SUC_lexicon_tags_1 },
    { 0xc5b13c26, SUC_lexicon_tags_6 },
    { 0x15713c27, SUC_lexicon_tags_87 },
    { 0x3a713c28, SUC_lexicon_tags_28 },
    { 0x55913c28, SUC_lexicon_tags_33 },
    { 0x2e713c29, SUC_lexicon_tags_26 },
    { 0x46113c28, SUC_lexicon_tags_8 },
    { 0x8c513c2c, SUC_lexicon_tags_146 },
    { 0x7fb13c2c, SUC_lexicon_tags_9 },
    { 0xa6113c2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f13c35, SUC_lexicon_tags_55 },
    { 0x40113c36, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10113c3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71313c40, SUC_lexicon_tags_15 },
    { 0xab113c40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38113c43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef513c46, SUC_lexicon_tags_11 },
    { 0xf1313c47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f113c49, SUC_lexicon_tags_27 },
    { 0xe8513c4a, SUC_lexicon_tags_31 },
    { 0xc6f13c4a, SUC_lexicon_tags_160 },
    { 0xbf313c4c, SUC_lexicon_tags_6 },
    { 0x60713c4c, SUC_lexicon_tags_37 },
    { 0xe0313c4e, SUC_lexicon_tags_15 },
    { 0xcbd13c4f, SUC_lexicon_tags_49 },
    { 0x61113c50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb713c54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b513c59, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f13c62, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xab513c65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x86113c67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e313c69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd13c6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b13c70, SUC_lexicon_tags_37 },
    { 0x97513c71, SUC_lexicon_tags_17 },
    { 0x39113c70, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef13c7b, SUC_lexicon_tags_2 },
    { 0xed313c7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2f13c7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72313c80, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb13c85, SUC_lexicon_tags_12 },
    { 0x80f13c86, SUC_lexicon_tags_37 },
    { 0xccb13c87, SUC_lexicon_tags_92 },
    { 0x2e513c88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70913c8c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f13c91, SUC_lexicon_tags_31 },
    { 0x22713c92, SUC_lexicon_tags_56 },
    { 0x90d13c91, SUC_lexicon_tags_30 },
    { 0x74d13c93, SUC_lexicon_tags_37 },
    { 0x2f913c91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd13c98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c913c9b, SUC_lexicon_tags_26 },
    { 0x5d513c9c, SUC_lexicon_tags_23 },
    { 0x92713c9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ad13c9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb13ca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac313ca7, SUC_lexicon_tags_37 },
    { 0x84f13ca8, SUC_lexicon_tags_216 },
    { 0xf1113ca8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f13cac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8913cb0, SUC_lexicon_tags_66 },
    { 0xbe113cb1, SUC_lexicon_tags_17 },
    { 0x32513cb0, SUC_lexicon_tags_11 },
    { 0x11513cb0, SUC_lexicon_tags_11 },
    { 0x21513cb1, SUC_lexicon_tags_17 },
    { 0x59d13cb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c713cb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d13cbf, SUC_lexicon_tags_91 },
    { 0xe3b13cc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f13cc5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8913cca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a313cce, SUC_lexicon_tags_45 },
    { 0xd7113ccf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb13cdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75313ce6, SUC_lexicon_tags_121 },
    { 0x6bb13ce6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b13ceb, SUC_lexicon_tags_66 },
    { 0x6713cec, SUC_lexicon_tags_12 },
    { 0x2313cec, SUC_lexicon_tags_46 },
    { 0x25b13ceb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb713cf1, SUC_lexicon_tags_15 },
    { 0xedb13cf2, SUC_lexicon_tags_43 },
    { 0x8bb13cf1, SUC_lexicon_tags_47 },
    { 0x1d313cf4, SUC_lexicon_tags_25 },
    { 0xe2913cf2, SUC_lexicon_tags_17 },
    { 0x73313cf6, SUC_lexicon_tags_29 },
    { 0x7b713cf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df13cfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf13cfd, SUC_lexicon_tags_34 },
    { 0xe0913cfe, SUC_lexicon_tags_60 },
    { 0x36513cff, SUC_lexicon_tags_12 },
    { 0x4e913d00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20113d05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd1713d07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f13d0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67713d0e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe913d12, SUC_lexicon_tags_96 },
    { 0x77713d12, SUC_lexicon_tags_63 },
    { 0xf7513d14, SUC_lexicon_tags_12 },
    { 0x7c313d15, SUC_lexicon_tags_14 },
    { 0xf4313d15, SUC_lexicon_tags_37 },
    { 0x3e313d15, SUC_lexicon_tags_47 },
    { 0xaa513d18, SUC_lexicon_tags_7 },
    { 0xf9713d19, SUC_lexicon_tags_2 },
    { 0x48113d16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f513d1d, SUC_lexicon_tags_6 },
    { 0x4df13d1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af13d22, SUC_lexicon_tags_1 },
    { 0x4bf13d23, SUC_lexicon_tags_37 },
    { 0x45f13d24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8f113d26, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b13d29, SUC_lexicon_tags_14 },
    { 0x5cb13d2a, SUC_lexicon_tags_15 },
    { 0x11913d2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xda513d2e, SUC_lexicon_tags_37 },
    { 0x99913d2e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0113d33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0313d3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b13d3f, SUC_lexicon_tags_45 },
    { 0x5df13d40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f313d44, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaa513d46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f13d4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd2313d4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b13d51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1f313d53, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d13d56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60d13d58, SUC_lexicon_tags_55 },
    { 0x44713d58, SUC_lexicon_tags_7 },
    { 0x8c513d5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d13d5d, SUC_lexicon_tags_38 },
    { 0x10113d5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e513d63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9b13d65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa1313d67, SUC_lexicon_tags_37 },
    { 0x5b513d67, SUC_lexicon_tags_2 },
    { 0x1b513d69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b13d6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3113d6e, SUC_lexicon_tags_8 },
    { 0xf1313d6e, SUC_lexicon_tags_36 },
    { 0x9713d70, SUC_lexicon_tags_25 },
    { 0xd7513d6f, SUC_lexicon_tags_8 },
    { 0xa2f13d72, SUC_lexicon_tags_63 },
    { 0xa7513d72, SUC_lexicon_tags_30 },
    { 0xb2d13d72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a513d76, SUC_lexicon_tags_15 },
    { 0xfeb13d76, SUC_lexicon_tags_271 },
    { 0, NULL },
    { 0x56113d79, SUC_lexicon_tags_13 },
    { 0x8bb13d7a, SUC_lexicon_tags_87 },
    { 0x47313d7a, SUC_lexicon_tags_11 },
    { 0x4f713d79, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe9113d7e, SUC_lexicon_tags_47 },
    { 0x94b13d7f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f13d82, SUC_lexicon_tags_27 },
    { 0x68513d83, SUC_lexicon_tags_1 },
    { 0xfaf13d83, SUC_lexicon_tags_15 },
    { 0xe8513d84, SUC_lexicon_tags_60 },
    { 0x50b13d86, SUC_lexicon_tags_37 },
    { 0x4d913d87, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d13d8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df13d8f, SUC_lexicon_tags_34 },
    { 0xb2513d90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d13d94, SUC_lexicon_tags_26 },
    { 0x4b513d95, SUC_lexicon_tags_37 },
    { 0x51f13d96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2713d98, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd1d13d9a, SUC_lexicon_tags_25 },
    { 0xccd13d9a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf13da0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b13da6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x14f13da8, SUC_lexicon_tags_39 },
    { 0x9af13da9, SUC_lexicon_tags_15 },
    { 0xe5113da9, SUC_lexicon_tags_25 },
    { 0x2dd13da9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c113db7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x50f13db9, SUC_lexicon_tags_191 },
    { 0x55513db9, SUC_lexicon_tags_30 },
    { 0xda513dba, SUC_lexicon_tags_54 },
    { 0xe9513dbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9b13dbe, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39913dc3, SUC_lexicon_tags_38 },
    { 0x96713dc4, SUC_lexicon_tags_17 },
    { 0x37f13dc4, SUC_lexicon_tags_6 },
    { 0x64513dc5, SUC_lexicon_tags_8 },
    { 0xdf113dc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b13dce, SUC_lexicon_tags_39 },
    { 0xb6d13dce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x34f13dd1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c513dd6, SUC_lexicon_tags_12 },
    { 0x57f13dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8113ddc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32513dde, SUC_lexicon_tags_9 },
    { 0xc2d13ddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7913de1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0513de3, SUC_lexicon_tags_46 },
    { 0xddf13de4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79913dee, SUC_lexicon_tags_27 },
    { 0xebf13def, SUC_lexicon_tags_49 },
    { 0xf4313def, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1313df2, SUC_lexicon_tags_15 },
    { 0x16313df3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd0b13df5, SUC_lexicon_tags_7 },
    { 0x5c313df6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32313dfb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f13dff, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4b113e01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f113e0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd13e0d, SUC_lexicon_tags_11 },
    { 0x62f13e0d, SUC_lexicon_tags_19 },
    { 0x2bb13e0f, SUC_lexicon_tags_17 },
    { 0xec113e10, SUC_lexicon_tags_23 },
    { 0x43f13e0e, SUC_lexicon_tags_11 },
    { 0x46713e12, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xac713e14, SUC_lexicon_tags_2 },
    { 0x22113e14, SUC_lexicon_tags_11 },
    { 0xb113e16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f13e19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12913e1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7913e20, SUC_lexicon_tags_4 },
    { 0xfd513e21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99113e23, SUC_lexicon_tags_38 },
    { 0xaed13e24, SUC_lexicon_tags_65 },
    { 0xaaf13e24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4913e2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef913e32, SUC_lexicon_tags_17 },
    { 0x3bd13e33, SUC_lexicon_tags_17 },
    { 0x24313e33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb5113e36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55d13e38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3513e3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36d13e3f, SUC_lexicon_tags_65 },
    { 0x76d13e3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99513e44, SUC_lexicon_tags_37 },
    { 0x5ed13e44, SUC_lexicon_tags_27 },
    { 0x2d913e45, SUC_lexicon_tags_12 },
    { 0x8b713e47, SUC_lexicon_tags_12 },
    { 0x79913e48, SUC_lexicon_tags_37 },
    { 0x7e313e49, SUC_lexicon_tags_38 },
    { 0xcef13e48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed13e4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff13e51, SUC_lexicon_tags_27 },
    { 0xaa713e52, SUC_lexicon_tags_9 },
    { 0x93d13e51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2a713e55, SUC_lexicon_tags_9 },
    { 0x17713e56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85d13e58, SUC_lexicon_tags_45 },
    { 0x42f13e58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b13e5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d113e5e, SUC_lexicon_tags_33 },
    { 0xcbb13e5e, SUC_lexicon_tags_16 },
    { 0x65913e60, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd113e63, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3113e66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x61113e68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x97913e6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86b13e6c, SUC_lexicon_tags_14 },
    { 0xe8913e6d, SUC_lexicon_tags_25 },
    { 0x89d13e6e, SUC_lexicon_tags_32 },
    { 0x2513e6e, SUC_lexicon_tags_27 },
    { 0x713e6d, SUC_lexicon_tags_10 },
    { 0xdd513e71, SUC_lexicon_tags_75 },
    { 0xd7713e72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd313e75, SUC_lexicon_tags_37 },
    { 0xe2313e76, SUC_lexicon_tags_85 },
    { 0xbb13e76, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0313e7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57313e85, SUC_lexicon_tags_37 },
    { 0x1a513e86, SUC_lexicon_tags_15 },
    { 0xf8313e87, SUC_lexicon_tags_68 },
    { 0x7af13e87, SUC_lexicon_tags_33 },
    { 0x14113e85, SUC_lexicon_tags_16 },
    { 0xe1b13e87, SUC_lexicon_tags_15 },
    { 0x3c513e8a, SUC_lexicon_tags_37 },
    { 0xc5713e8c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x67913e8f, SUC_lexicon_tags_37 },
    { 0xce713e90, SUC_lexicon_tags_39 },
    { 0x78913e91, SUC_lexicon_tags_11 },
    { 0x5f13e92, SUC_lexicon_tags_29 },
    { 0x6913e90, SUC_lexicon_tags_10 },
    { 0xe3113e94, SUC_lexicon_tags_19 },
    { 0xb1713e95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8113e97, SUC_lexicon_tags_7 },
    { 0x76913e97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd113e9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2713e9e, SUC_lexicon_tags_66 },
    { 0x36d13e9f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0913eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66713eaf, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf13eb2, SUC_lexicon_tags_33 },
    { 0xae113eb2, SUC_lexicon_tags_25 },
    { 0x26713eb4, SUC_lexicon_tags_25 },
    { 0x50713eb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3713ebc, SUC_lexicon_tags_6 },
    { 0xd13ebd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99113ec2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcff13ec4, SUC_lexicon_tags_7 },
    { 0xef913ec5, SUC_lexicon_tags_65 },
    { 0x18113ec5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8313ecc, SUC_lexicon_tags_12 },
    { 0x85113ecc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab13ed0, SUC_lexicon_tags_7 },
    { 0x4c313ed1, SUC_lexicon_tags_37 },
    { 0xf7513ed2, SUC_lexicon_tags_8 },
    { 0xd1513ed3, SUC_lexicon_tags_15 },
    { 0x14713ed4, SUC_lexicon_tags_23 },
    { 0x97313ed5, SUC_lexicon_tags_2 },
    { 0x97d13ed6, SUC_lexicon_tags_45 },
    { 0x77113ed6, SUC_lexicon_tags_37 },
    { 0x93913ed8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36513eda, SUC_lexicon_tags_17 },
    { 0x16913edb, SUC_lexicon_tags_11 },
    { 0x3ed13edc, SUC_lexicon_tags_17 },
    { 0xa1113edd, SUC_lexicon_tags_25 },
    { 0x37913ede, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x46113ee1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28713ee3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b13ee8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x16113eea, SUC_lexicon_tags_26 },
    { 0xf5713eeb, SUC_lexicon_tags_11 },
    { 0x8cf13eea, SUC_lexicon_tags_47 },
    { 0xe7f13eed, SUC_lexicon_tags_37 },
    { 0x8f513eee, SUC_lexicon_tags_25 },
    { 0x34d13eed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b13ef2, SUC_lexicon_tags_17 },
    { 0x1d313ef2, SUC_lexicon_tags_19 },
    { 0x10913ef3, SUC_lexicon_tags_1 },
    { 0x2fb13ef5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf13efc, SUC_lexicon_tags_49 },
    { 0x5af13efc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d13f00, SUC_lexicon_tags_11 },
    { 0x60f13f00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70113f05, SUC_lexicon_tags_49 },
    { 0x18513f06, SUC_lexicon_tags_10 },
    { 0x37113f07, SUC_lexicon_tags_64 },
    { 0xf4113f08, SUC_lexicon_tags_30 },
    { 0xd4113f06, SUC_lexicon_tags_45 },
    { 0x33113f0a, SUC_lexicon_tags_12 },
    { 0xc913f0a, SUC_lexicon_tags_62 },
    { 0xdc713f0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45713f1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0313f1e, SUC_lexicon_tags_52 },
    { 0x7e913f1f, SUC_lexicon_tags_17 },
    { 0xc913f1e, SUC_lexicon_tags_15 },
    { 0xf6d13f1f, SUC_lexicon_tags_2 },
    { 0x15713f22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2713f24, SUC_lexicon_tags_17 },
    { 0xe9513f24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d13f28, SUC_lexicon_tags_47 },
    { 0xb6713f29, SUC_lexicon_tags_15 },
    { 0x67b13f2a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe6113f2c, SUC_lexicon_tags_37 },
    { 0x6b513f2d, SUC_lexicon_tags_26 },
    { 0x6d313f2e, SUC_lexicon_tags_11 },
    { 0x55113f2c, SUC_lexicon_tags_31 },
    { 0xe7513f2f, SUC_lexicon_tags_1 },
    { 0x1d513f31, SUC_lexicon_tags_25 },
    { 0x3f513f32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63713f34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab13f37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9713f39, SUC_lexicon_tags_47 },
    { 0x4cb13f3a, SUC_lexicon_tags_37 },
    { 0xf0113f39, SUC_lexicon_tags_38 },
    { 0x5b313f3b, SUC_lexicon_tags_4 },
    { 0x36b13f3d, SUC_lexicon_tags_44 },
    { 0x41513f3d, SUC_lexicon_tags_37 },
    { 0x8b713f3f, SUC_lexicon_tags_29 },
    { 0xc3d13f3f, SUC_lexicon_tags_29 },
    { 0xd1913f3f, SUC_lexicon_tags_27 },
    { 0x55913f3e, SUC_lexicon_tags_26 },
    { 0x6d513f41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd13f46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c913f49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f13f4e, SUC_lexicon_tags_1 },
    { 0xc0713f4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f513f58, SUC_lexicon_tags_23 },
    { 0x95d13f58, SUC_lexicon_tags_37 },
    { 0xba313f59, SUC_lexicon_tags_1 },
    { 0xb1913f5a, SUC_lexicon_tags_25 },
    { 0x44d13f5c, SUC_lexicon_tags_17 },
    { 0x1bb13f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f13f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6913f65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79913f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x41713f69, SUC_lexicon_tags_8 },
    { 0x1a913f6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3913f70, SUC_lexicon_tags_7 },
    { 0xee913f71, SUC_lexicon_tags_25 },
    { 0xdf13f71, SUC_lexicon_tags_55 },
    { 0xd6f13f73, SUC_lexicon_tags_63 },
    { 0x72913f74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ad13f76, SUC_lexicon_tags_43 },
    { 0xbab13f77, SUC_lexicon_tags_11 },
    { 0x8c113f77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b13f7b, SUC_lexicon_tags_2 },
    { 0xb3f13f7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d913f83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18513f86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22f13f88, SUC_lexicon_tags_41 },
    { 0x8f713f89, SUC_lexicon_tags_11 },
    { 0xeb713f89, SUC_lexicon_tags_15 },
    { 0x7ff13f8b, SUC_lexicon_tags_2 },
    { 0x9d513f88, SUC_lexicon_tags_45 },
    { 0x9e913f8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5713f8f, SUC_lexicon_tags_30 },
    { 0x48b13f8f, SUC_lexicon_tags_6 },
    { 0x89d13f91, SUC_lexicon_tags_49 },
    { 0x64b13f92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb13f99, SUC_lexicon_tags_26 },
    { 0x12113f9a, SUC_lexicon_tags_7 },
    { 0x40f13f99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f913fa0, SUC_lexicon_tags_7 },
    { 0xed913fa1, SUC_lexicon_tags_27 },
    { 0xa3113fa2, SUC_lexicon_tags_49 },
    { 0xc4513fa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57713fa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb513fa9, SUC_lexicon_tags_31 },
    { 0x71d13faa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e913fac, SUC_lexicon_tags_45 },
    { 0xbaf13fad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b713fb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23b13fb6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24313fbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18b13fbd, SUC_lexicon_tags_17 },
    { 0xd6f13fbe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f113fc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b713fc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71113fc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa1b13fc7, SUC_lexicon_tags_1 },
    { 0x8e513fc7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xef13fca, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98913fd1, SUC_lexicon_tags_37 },
    { 0x86d13fd1, SUC_lexicon_tags_1 },
    { 0x51113fd1, SUC_lexicon_tags_47 },
    { 0xc5513fd4, SUC_lexicon_tags_47 },
    { 0xb1713fd5, SUC_lexicon_tags_4 },
    { 0xc7713fd5, SUC_lexicon_tags_17 },
    { 0xdd313fd7, SUC_lexicon_tags_2 },
    { 0x56113fd8, SUC_lexicon_tags_17 },
    { 0x76b13fd8, SUC_lexicon_tags_12 },
    { 0xde313fd8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd913fdf, SUC_lexicon_tags_10 },
    { 0x33313fe0, SUC_lexicon_tags_47 },
    { 0xe5513fe0, SUC_lexicon_tags_65 },
    { 0xbab13fe0, SUC_lexicon_tags_20 },
    { 0x30313fe3, SUC_lexicon_tags_23 },
    { 0xf8513fe3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c113fe8, SUC_lexicon_tags_37 },
    { 0x86313fe9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63313ff7, SUC_lexicon_tags_12 },
    { 0xcb13ff8, SUC_lexicon_tags_38 },
    { 0xe3313ff9, SUC_lexicon_tags_798 },
    { 0xe0b13ffa, SUC_lexicon_tags_15 },
    { 0xe313ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b114000, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd14003, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9514007, SUC_lexicon_tags_27 },
    { 0x81714008, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9f1400a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8451400c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa314017, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b1401a, SUC_lexicon_tags_47 },
    { 0x7e31401b, SUC_lexicon_tags_77 },
    { 0x12d1401c, SUC_lexicon_tags_23 },
    { 0xea31401c, SUC_lexicon_tags_84 },
    { 0xbc91401b, SUC_lexicon_tags_80 },
    { 0xfe91401f, SUC_lexicon_tags_8 },
    { 0x19b14020, SUC_lexicon_tags_12 },
    { 0x26314020, SUC_lexicon_tags_30 },
    { 0xfc31401b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2d14024, SUC_lexicon_tags_37 },
    { 0x1d914025, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0x3ab14027, SUC_lexicon_tags_19 },
    { 0xc4b14027, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x111402b, SUC_lexicon_tags_37 },
    { 0xa531402c, SUC_lexicon_tags_2 },
    { 0xa4f1402d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf14035, SUC_lexicon_tags_15 },
    { 0x42714035, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb1403a, SUC_lexicon_tags_46 },
    { 0x9a31403b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9491403e, SUC_lexicon_tags_60 },
    { 0xcf31403f, SUC_lexicon_tags_46 },
    { 0xf314040, SUC_lexicon_tags_8 },
    { 0x5fb14041, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f14046, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd1404f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeb314051, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf14055, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9651405c, SUC_lexicon_tags_1 },
    { 0x4fd1405d, SUC_lexicon_tags_2 },
    { 0x8d1405e, SUC_lexicon_tags_17 },
    { 0xd871405f, SUC_lexicon_tags_12 },
    { 0xc471405e, SUC_lexicon_tags_7 },
    { 0x37f14061, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d14066, SUC_lexicon_tags_60 },
    { 0x63114066, SUC_lexicon_tags_33 },
    { 0xe2d14066, SUC_lexicon_tags_11 },
    { 0x4e114069, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc811406c, SUC_lexicon_tags_28 },
    { 0x731406c, SUC_lexicon_tags_7 },
    { 0xc471406e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49514071, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3114073, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8714075, SUC_lexicon_tags_14 },
    { 0x4cf14076, SUC_lexicon_tags_25 },
    { 0xc5d14077, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd1407b, SUC_lexicon_tags_25 },
    { 0x3011407b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed14084, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xca314086, SUC_lexicon_tags_27 },
    { 0xad514086, SUC_lexicon_tags_17 },
    { 0x98114088, SUC_lexicon_tags_151 },
    { 0x1d714089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb31408c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5cf1408e, SUC_lexicon_tags_26 },
    { 0x66f1408f, SUC_lexicon_tags_23 },
    { 0x8ab1408e, SUC_lexicon_tags_38 },
    { 0x70d14091, SUC_lexicon_tags_17 },
    { 0x5a514091, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb914096, SUC_lexicon_tags_60 },
    { 0xa0714097, SUC_lexicon_tags_30 },
    { 0x3c714098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ff1409a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f140a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f140a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09140ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255140b0, SUC_lexicon_tags_7 },
    { 0x9af140b1, SUC_lexicon_tags_17 },
    { 0xf140b2, SUC_lexicon_tags_13 },
    { 0xfc1140b1, SUC_lexicon_tags_1 },
    { 0xc45140b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857140b9, SUC_lexicon_tags_17 },
    { 0xeff140b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1140bd, SUC_lexicon_tags_61 },
    { 0x69f140be, SUC_lexicon_tags_87 },
    { 0x80f140be, SUC_lexicon_tags_39 },
    { 0x8dd140bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2cb140c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559140c6, SUC_lexicon_tags_2 },
    { 0xecb140c7, SUC_lexicon_tags_17 },
    { 0x497140c8, SUC_lexicon_tags_28 },
    { 0x515140c6, SUC_lexicon_tags_63 },
    { 0x753140ca, SUC_lexicon_tags_14 },
    { 0xc91140c6, SUC_lexicon_tags_2 },
    { 0xab7140c9, SUC_lexicon_tags_37 },
    { 0x8ad140c8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x23d140cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfd5140d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b140d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d140d9, SUC_lexicon_tags_46 },
    { 0x331140da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1140dd, SUC_lexicon_tags_14 },
    { 0x261140de, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc5f140e0, SUC_lexicon_tags_7 },
    { 0x67140e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x41d140e3, SUC_lexicon_tags_11 },
    { 0xe5b140e3, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x551140e6, SUC_lexicon_tags_56 },
    { 0x9cb140e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f140eb, SUC_lexicon_tags_17 },
    { 0x22f140eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f140f6, SUC_lexicon_tags_15 },
    { 0xa1b140f6, SUC_lexicon_tags_0 },
    { 0x1b3140f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3e5140fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed140ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a714106, SUC_lexicon_tags_48 },
    { 0x8d514106, SUC_lexicon_tags_37 },
    { 0xb714108, SUC_lexicon_tags_0 },
    { 0x52f14109, SUC_lexicon_tags_47 },
    { 0x8fd1410a, SUC_lexicon_tags_11 },
    { 0x6bb1410b, SUC_lexicon_tags_37 },
    { 0x71d1410a, SUC_lexicon_tags_38 },
    { 0xcc71410d, SUC_lexicon_tags_45 },
    { 0x1a91410e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x96114110, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c914113, SUC_lexicon_tags_46 },
    { 0xb4b14114, SUC_lexicon_tags_2 },
    { 0x54114115, SUC_lexicon_tags_14 },
    { 0xc7114116, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef1411c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87714120, SUC_lexicon_tags_60 },
    { 0x63714121, SUC_lexicon_tags_16 },
    { 0x84514122, SUC_lexicon_tags_17 },
    { 0xe9114121, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10314125, SUC_lexicon_tags_0 },
    { 0x24514126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0114129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3911412f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb314131, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1314139, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7691413b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba314140, SUC_lexicon_tags_7 },
    { 0x14314141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd714146, SUC_lexicon_tags_33 },
    { 0x40714147, SUC_lexicon_tags_15 },
    { 0xf5f14146, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x60d1414a, SUC_lexicon_tags_12 },
    { 0xc411414b, SUC_lexicon_tags_49 },
    { 0xd51414c, SUC_lexicon_tags_32 },
    { 0xe4b1414c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe9f1414f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfed14151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5114159, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a91415d, SUC_lexicon_tags_14 },
    { 0x5931415e, SUC_lexicon_tags_11 },
    { 0xf291415f, SUC_lexicon_tags_25 },
    { 0x38f1415d, SUC_lexicon_tags_27 },
    { 0x1ef1415f, SUC_lexicon_tags_16 },
    { 0xf7914162, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a11416a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd14171, SUC_lexicon_tags_17 },
    { 0xb5114172, SUC_lexicon_tags_43 },
    { 0xb7b14173, SUC_lexicon_tags_37 },
    { 0xa514172, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f14178, SUC_lexicon_tags_43 },
    { 0x5d514179, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d91417b, SUC_lexicon_tags_27 },
    { 0x8b31417c, SUC_lexicon_tags_39 },
    { 0x83f1417d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb14180, SUC_lexicon_tags_77 },
    { 0xca514181, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d1418b, SUC_lexicon_tags_38 },
    { 0x36d1418b, SUC_lexicon_tags_56 },
    { 0x9211418d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f14190, SUC_lexicon_tags_9 },
    { 0xd7b14191, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b514194, SUC_lexicon_tags_17 },
    { 0x8a914195, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2714198, SUC_lexicon_tags_60 },
    { 0x97514199, SUC_lexicon_tags_7 },
    { 0xc6314199, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b1419e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd141a1, SUC_lexicon_tags_37 },
    { 0xa8f141a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81141a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81d141a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f5141ab, SUC_lexicon_tags_1 },
    { 0x351141ab, SUC_lexicon_tags_9 },
    { 0x8dd141ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f141b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x361141b5, SUC_lexicon_tags_8 },
    { 0xbd5141b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda5141b8, SUC_lexicon_tags_37 },
    { 0x66f141b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x645141bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5141c1, SUC_lexicon_tags_12 },
    { 0xe1b141c1, SUC_lexicon_tags_23 },
    { 0xe8f141c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b141c8, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0x659141ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed141ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb0b141d0, SUC_lexicon_tags_26 },
    { 0x6d5141d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3141d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b141d8, SUC_lexicon_tags_1 },
    { 0x2ff141d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf141dd, SUC_lexicon_tags_15 },
    { 0x317141de, SUC_lexicon_tags_25 },
    { 0x26f141df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd3141e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa43141e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7bd141e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4f141e7, SUC_lexicon_tags_19 },
    { 0xfdb141e8, SUC_lexicon_tags_22 },
    { 0x4a9141e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xad1141eb, SUC_lexicon_tags_23 },
    { 0x8e9141eb, SUC_lexicon_tags_2 },
    { 0xdc7141ec, SUC_lexicon_tags_61 },
    { 0xb71141eb, SUC_lexicon_tags_7 },
    { 0xac3141ef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa2d141f1, SUC_lexicon_tags_25 },
    { 0xf11141f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd141f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10514200, SUC_lexicon_tags_7 },
    { 0x25914201, SUC_lexicon_tags_12 },
    { 0xcbf14202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d714205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a914208, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd1420b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf1420f, SUC_lexicon_tags_19 },
    { 0xfff14210, SUC_lexicon_tags_30 },
    { 0xb8b14211, SUC_lexicon_tags_17 },
    { 0x8e514212, SUC_lexicon_tags_47 },
    { 0xf9314213, SUC_lexicon_tags_37 },
    { 0xc8914214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f14219, SUC_lexicon_tags_19 },
    { 0x8671421a, SUC_lexicon_tags_9 },
    { 0xb1421a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe751421e, SUC_lexicon_tags_2 },
    { 0x3051421e, SUC_lexicon_tags_38 },
    { 0x8e314220, SUC_lexicon_tags_23 },
    { 0x38f14220, SUC_lexicon_tags_15 },
    { 0xdc314220, SUC_lexicon_tags_337 },
    { 0xf0914222, SUC_lexicon_tags_47 },
    { 0x16d14224, SUC_lexicon_tags_31 },
    { 0xd9f14220, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63114227, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1491422a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1422e, SUC_lexicon_tags_12 },
    { 0xedd1422f, SUC_lexicon_tags_17 },
    { 0xa0b1422e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb14234, SUC_lexicon_tags_14 },
    { 0x51d14235, SUC_lexicon_tags_45 },
    { 0x4ad14235, SUC_lexicon_tags_9 },
    { 0x8d314235, SUC_lexicon_tags_7 },
    { 0xc4914238, SUC_lexicon_tags_11 },
    { 0xf4d14235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5d1423b, SUC_lexicon_tags_7 },
    { 0x20f1423c, SUC_lexicon_tags_12 },
    { 0x7b71423c, SUC_lexicon_tags_15 },
    { 0x80d1423e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b514241, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6114243, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x39b14245, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf531424c, SUC_lexicon_tags_43 },
    { 0x54f1424d, SUC_lexicon_tags_82 },
    { 0xe4f1424d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8914257, SUC_lexicon_tags_66 },
    { 0x88714258, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21f1425a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f11425d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25714261, SUC_lexicon_tags_19 },
    { 0x22714262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14114265, SUC_lexicon_tags_7 },
    { 0xc3b14266, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3314268, SUC_lexicon_tags_38 },
    { 0xab914269, SUC_lexicon_tags_7 },
    { 0xb2f1426a, SUC_lexicon_tags_63 },
    { 0x53f1426a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb1426f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1714273, SUC_lexicon_tags_4 },
    { 0xec514273, SUC_lexicon_tags_11 },
    { 0xca114274, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6b114277, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1cd14279, SUC_lexicon_tags_2 },
    { 0x83914279, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af1427e, SUC_lexicon_tags_25 },
    { 0xcd11427e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1d914281, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38914288, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e91428f, SUC_lexicon_tags_15 },
    { 0x6914290, SUC_lexicon_tags_11 },
    { 0x30314291, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5b14293, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1914295, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151142a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125142a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37142a7, SUC_lexicon_tags_17 },
    { 0xdbd142a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f5142aa, SUC_lexicon_tags_7 },
    { 0x10d142ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751142af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03142b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe1142b4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8e3142b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27142b9, SUC_lexicon_tags_23 },
    { 0x15142ba, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8d5142bc, SUC_lexicon_tags_12 },
    { 0xe9f142bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c7142bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x579142c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb142c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227142cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef142d0, SUC_lexicon_tags_27 },
    { 0xd83142d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b142d4, SUC_lexicon_tags_79 },
    { 0xb1142d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9142d7, SUC_lexicon_tags_27 },
    { 0x4b3142d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbb142da, SUC_lexicon_tags_27 },
    { 0x789142db, SUC_lexicon_tags_8 },
    { 0x8fd142db, SUC_lexicon_tags_17 },
    { 0xac7142dc, SUC_lexicon_tags_11 },
    { 0x395142dc, SUC_lexicon_tags_4 },
    { 0xed9142df, SUC_lexicon_tags_37 },
    { 0xac9142dd, SUC_lexicon_tags_37 },
    { 0xb87142e1, SUC_lexicon_tags_2 },
    { 0x8b142e2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717142e7, SUC_lexicon_tags_290 },
    { 0, NULL },
    { 0x2d7142e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5142f1, SUC_lexicon_tags_14 },
    { 0x59b142f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x751142f5, SUC_lexicon_tags_6 },
    { 0x763142f6, SUC_lexicon_tags_1 },
    { 0xde7142f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10b142f9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4d3142fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd714300, SUC_lexicon_tags_28 },
    { 0x6ad14301, SUC_lexicon_tags_15 },
    { 0x2c914300, SUC_lexicon_tags_19 },
    { 0x27514303, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20114305, SUC_lexicon_tags_60 },
    { 0x3314306, SUC_lexicon_tags_32 },
    { 0x8c714307, SUC_lexicon_tags_34 },
    { 0x69914307, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f91430b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2611430e, SUC_lexicon_tags_26 },
    { 0xd51430f, SUC_lexicon_tags_0 },
    { 0x5714310, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16514313, SUC_lexicon_tags_7 },
    { 0x72f14313, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x26114317, SUC_lexicon_tags_40 },
    { 0x1a514318, SUC_lexicon_tags_82 },
    { 0xfa914318, SUC_lexicon_tags_12 },
    { 0xc4114319, SUC_lexicon_tags_29 },
    { 0x3fd1431b, SUC_lexicon_tags_55 },
    { 0x92f14318, SUC_lexicon_tags_15 },
    { 0xb931431d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd14321, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1514324, SUC_lexicon_tags_17 },
    { 0xd6f14325, SUC_lexicon_tags_2 },
    { 0x9bb14326, SUC_lexicon_tags_11 },
    { 0x98514327, SUC_lexicon_tags_31 },
    { 0x82d14328, SUC_lexicon_tags_30 },
    { 0x2ef14328, SUC_lexicon_tags_25 },
    { 0x7b914325, SUC_lexicon_tags_37 },
    { 0x3e71432b, SUC_lexicon_tags_1 },
    { 0xcf91432c, SUC_lexicon_tags_39 },
    { 0x13b14327, SUC_lexicon_tags_68 },
    { 0x7191432d, SUC_lexicon_tags_9 },
    { 0xaff1432f, SUC_lexicon_tags_15 },
    { 0xaa114328, SUC_lexicon_tags_12 },
    { 0xe714329, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51433d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314342, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69b14344, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3314346, SUC_lexicon_tags_8 },
    { 0xce914347, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf1434c, SUC_lexicon_tags_33 },
    { 0x1bf1434c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc491434f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebb14351, SUC_lexicon_tags_37 },
    { 0xfa314352, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf591435c, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x6151435e, SUC_lexicon_tags_14 },
    { 0xfbb1435f, SUC_lexicon_tags_17 },
    { 0xeb514360, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94314367, SUC_lexicon_tags_49 },
    { 0x54114368, SUC_lexicon_tags_17 },
    { 0xa3714367, SUC_lexicon_tags_9 },
    { 0x3091436a, SUC_lexicon_tags_66 },
    { 0x6411436b, SUC_lexicon_tags_7 },
    { 0x7c31436c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3114376, SUC_lexicon_tags_14 },
    { 0x9bb14377, SUC_lexicon_tags_15 },
    { 0xfa914378, SUC_lexicon_tags_6 },
    { 0xd5314379, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7231437d, SUC_lexicon_tags_14 },
    { 0x4091437d, SUC_lexicon_tags_0 },
    { 0xaf91437f, SUC_lexicon_tags_9 },
    { 0xae71437d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b14384, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45f14386, SUC_lexicon_tags_49 },
    { 0x58d14386, SUC_lexicon_tags_25 },
    { 0x65514388, SUC_lexicon_tags_37 },
    { 0xc0f14389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x451438c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed314394, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52114397, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9f14399, SUC_lexicon_tags_25 },
    { 0x94d14399, SUC_lexicon_tags_31 },
    { 0x57f14399, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1f1439d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7cb1439f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdb143a1, SUC_lexicon_tags_12 },
    { 0xea7143a2, SUC_lexicon_tags_47 },
    { 0x261143a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873143a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x457143ac, SUC_lexicon_tags_46 },
    { 0x8f5143ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x711143b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x151143b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d143b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3143bb, SUC_lexicon_tags_49 },
    { 0x6bb143bc, SUC_lexicon_tags_7 },
    { 0xf15143bd, SUC_lexicon_tags_26 },
    { 0xb6d143bb, SUC_lexicon_tags_14 },
    { 0xd47143bd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453143c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x557143c5, SUC_lexicon_tags_37 },
    { 0xc8b143c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b143cb, SUC_lexicon_tags_14 },
    { 0x799143cc, SUC_lexicon_tags_8 },
    { 0x975143cd, SUC_lexicon_tags_25 },
    { 0x60b143cd, SUC_lexicon_tags_9 },
    { 0x867143cf, SUC_lexicon_tags_1 },
    { 0x10d143d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23143db, SUC_lexicon_tags_17 },
    { 0x383143dc, SUC_lexicon_tags_43 },
    { 0x975143dc, SUC_lexicon_tags_32 },
    { 0x2d7143db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133143e4, SUC_lexicon_tags_27 },
    { 0x881143e5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x931143e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09143ef, SUC_lexicon_tags_11 },
    { 0x25b143f0, SUC_lexicon_tags_11 },
    { 0x3f3143f1, SUC_lexicon_tags_15 },
    { 0x1c7143f2, SUC_lexicon_tags_37 },
    { 0x875143f1, SUC_lexicon_tags_53 },
    { 0xaff143f4, SUC_lexicon_tags_17 },
    { 0x969143f5, SUC_lexicon_tags_31 },
    { 0xed9143f6, SUC_lexicon_tags_17 },
    { 0xac7143f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f143fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f514401, SUC_lexicon_tags_15 },
    { 0x1d914401, SUC_lexicon_tags_103 },
    { 0xbdf14403, SUC_lexicon_tags_37 },
    { 0x1af14403, SUC_lexicon_tags_47 },
    { 0xe914401, SUC_lexicon_tags_32 },
    { 0x1cd14404, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13514412, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d14416, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x72714419, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2471441b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f1441e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb514421, SUC_lexicon_tags_1 },
    { 0x3cd14421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24514426, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d114428, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd811442b, SUC_lexicon_tags_12 },
    { 0x17d1442c, SUC_lexicon_tags_138 },
    { 0x50b1442c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x63714430, SUC_lexicon_tags_2 },
    { 0x69d14431, SUC_lexicon_tags_49 },
    { 0xf8d14431, SUC_lexicon_tags_29 },
    { 0xf2314432, SUC_lexicon_tags_70 },
    { 0xbf14434, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd71443d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d714444, SUC_lexicon_tags_11 },
    { 0x2f514444, SUC_lexicon_tags_37 },
    { 0x3d314446, SUC_lexicon_tags_37 },
    { 0xb0f14447, SUC_lexicon_tags_10 },
    { 0x47714448, SUC_lexicon_tags_1 },
    { 0x8eb14448, SUC_lexicon_tags_60 },
    { 0x93514449, SUC_lexicon_tags_37 },
    { 0x5b14447, SUC_lexicon_tags_21 },
    { 0x4711444c, SUC_lexicon_tags_14 },
    { 0x5871444d, SUC_lexicon_tags_25 },
    { 0x3471444d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3514452, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9a114454, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0714457, SUC_lexicon_tags_17 },
    { 0xcc714458, SUC_lexicon_tags_47 },
    { 0xcb14459, SUC_lexicon_tags_27 },
    { 0xfaf1445a, SUC_lexicon_tags_17 },
    { 0x5cd1445a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa714464, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec914466, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5dd14468, SUC_lexicon_tags_37 },
    { 0x1c314468, SUC_lexicon_tags_47 },
    { 0xfa71446a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af1446f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b914471, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad14477, SUC_lexicon_tags_15 },
    { 0xb3314478, SUC_lexicon_tags_43 },
    { 0xc9914479, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd091447f, SUC_lexicon_tags_17 },
    { 0x33514480, SUC_lexicon_tags_17 },
    { 0x77d14481, SUC_lexicon_tags_37 },
    { 0x61d14482, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x82514484, SUC_lexicon_tags_49 },
    { 0x2d514485, SUC_lexicon_tags_15 },
    { 0x16514484, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad514488, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef1448c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1071448f, SUC_lexicon_tags_14 },
    { 0x11b14490, SUC_lexicon_tags_38 },
    { 0xb514491, SUC_lexicon_tags_37 },
    { 0xdeb14490, SUC_lexicon_tags_14 },
    { 0xf0114493, SUC_lexicon_tags_2 },
    { 0x79b14493, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x58514497, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1451449a, SUC_lexicon_tags_25 },
    { 0xd891449a, SUC_lexicon_tags_15 },
    { 0xebb1449a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xda11449e, SUC_lexicon_tags_60 },
    { 0x57d1449e, SUC_lexicon_tags_11 },
    { 0xa51144a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb144a5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5cf144a7, SUC_lexicon_tags_17 },
    { 0x803144a8, SUC_lexicon_tags_38 },
    { 0xab144a8, SUC_lexicon_tags_12 },
    { 0x453144aa, SUC_lexicon_tags_60 },
    { 0x41d144ab, SUC_lexicon_tags_60 },
    { 0x921144ac, SUC_lexicon_tags_87 },
    { 0x8d5144aa, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5144b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x639144b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f144ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787144be, SUC_lexicon_tags_17 },
    { 0xca7144bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe39144c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d144c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad144c9, SUC_lexicon_tags_915 },
    { 0x45d144c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x13d144cc, SUC_lexicon_tags_2 },
    { 0xefb144cc, SUC_lexicon_tags_37 },
    { 0x457144cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815144d6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x165144d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5144dd, SUC_lexicon_tags_12 },
    { 0xb8d144de, SUC_lexicon_tags_7 },
    { 0x9eb144df, SUC_lexicon_tags_47 },
    { 0x1d3144e0, SUC_lexicon_tags_8 },
    { 0x9e9144df, SUC_lexicon_tags_2 },
    { 0x149144e2, SUC_lexicon_tags_38 },
    { 0xeb3144e3, SUC_lexicon_tags_55 },
    { 0xeb1144e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda3144e6, SUC_lexicon_tags_23 },
    { 0x603144e7, SUC_lexicon_tags_11 },
    { 0x815144e8, SUC_lexicon_tags_3 },
    { 0xc9b144e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91144ec, SUC_lexicon_tags_17 },
    { 0x8a1144ed, SUC_lexicon_tags_37 },
    { 0x475144ee, SUC_lexicon_tags_49 },
    { 0xc3f144ed, SUC_lexicon_tags_31 },
    { 0xe25144f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39f144f2, SUC_lexicon_tags_25 },
    { 0x84f144f3, SUC_lexicon_tags_114 },
    { 0x2b7144f2, SUC_lexicon_tags_68 },
    { 0x7bf144f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481144f9, SUC_lexicon_tags_17 },
    { 0xea9144fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5144fd, SUC_lexicon_tags_2 },
    { 0x8d144fe, SUC_lexicon_tags_7 },
    { 0xc47144ff, SUC_lexicon_tags_47 },
    { 0x811144ff, SUC_lexicon_tags_42 },
    { 0x7b514501, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef914514, SUC_lexicon_tags_11 },
    { 0x46114515, SUC_lexicon_tags_25 },
    { 0xf2714515, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd0f14518, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd14521, SUC_lexicon_tags_14 },
    { 0x13914522, SUC_lexicon_tags_25 },
    { 0x18d14523, SUC_lexicon_tags_30 },
    { 0xc6f14524, SUC_lexicon_tags_37 },
    { 0x3f914525, SUC_lexicon_tags_11 },
    { 0xab314525, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb1452a, SUC_lexicon_tags_619 },
    { 0x33b1452b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6011452d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a714531, SUC_lexicon_tags_7 },
    { 0xad514531, SUC_lexicon_tags_12 },
    { 0xda714533, SUC_lexicon_tags_2 },
    { 0x6f114534, SUC_lexicon_tags_65 },
    { 0x23914535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f1453c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31514546, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3714548, SUC_lexicon_tags_34 },
    { 0xe0b14549, SUC_lexicon_tags_1 },
    { 0xf511454a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b14551, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb14554, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc314556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57714558, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9531455a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111455e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14114561, SUC_lexicon_tags_14 },
    { 0x36314562, SUC_lexicon_tags_11 },
    { 0xf2314563, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x55514565, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xec114568, SUC_lexicon_tags_47 },
    { 0x1cd14568, SUC_lexicon_tags_17 },
    { 0xab1456a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f51456c, SUC_lexicon_tags_15 },
    { 0xa51456d, SUC_lexicon_tags_25 },
    { 0xafd1456e, SUC_lexicon_tags_5 },
    { 0xb1b1456e, SUC_lexicon_tags_15 },
    { 0xf311456e, SUC_lexicon_tags_25 },
    { 0xfc714571, SUC_lexicon_tags_7 },
    { 0x1691456e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d1457b, SUC_lexicon_tags_47 },
    { 0xe731457c, SUC_lexicon_tags_25 },
    { 0x8ad1457d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f14582, SUC_lexicon_tags_0 },
    { 0xe7d14582, SUC_lexicon_tags_15 },
    { 0xc6114584, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x52114586, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f14590, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2bf14592, SUC_lexicon_tags_63 },
    { 0x67b14593, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d14599, SUC_lexicon_tags_15 },
    { 0xf831459a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3b1459c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b145a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x813145a5, SUC_lexicon_tags_32 },
    { 0x437145a6, SUC_lexicon_tags_25 },
    { 0xe73145a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763145ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef145b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x173145b3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe13145b5, SUC_lexicon_tags_11 },
    { 0x19145b5, SUC_lexicon_tags_56 },
    { 0xc03145b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9d145b9, SUC_lexicon_tags_2 },
    { 0x8c3145ba, SUC_lexicon_tags_12 },
    { 0x563145bb, SUC_lexicon_tags_11 },
    { 0x88b145bc, SUC_lexicon_tags_7 },
    { 0xe33145bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd145c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63145c7, SUC_lexicon_tags_449 },
    { 0x9dd145c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xead145cb, SUC_lexicon_tags_63 },
    { 0x597145cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb145d0, SUC_lexicon_tags_27 },
    { 0xbc9145d1, SUC_lexicon_tags_55 },
    { 0x161145d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c7145d4, SUC_lexicon_tags_11 },
    { 0xf77145d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7145d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803145dc, SUC_lexicon_tags_29 },
    { 0x14f145dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15145e0, SUC_lexicon_tags_48 },
    { 0xa4d145e1, SUC_lexicon_tags_37 },
    { 0x9e9145e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc9145e4, SUC_lexicon_tags_37 },
    { 0x3f145e5, SUC_lexicon_tags_7 },
    { 0xc17145e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7145e9, SUC_lexicon_tags_10 },
    { 0x1a5145ea, SUC_lexicon_tags_2 },
    { 0xcaf145eb, SUC_lexicon_tags_30 },
    { 0x7b1145ec, SUC_lexicon_tags_1 },
    { 0xa69145ea, SUC_lexicon_tags_11 },
    { 0x4d5145ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ad145f0, SUC_lexicon_tags_12 },
    { 0x667145f1, SUC_lexicon_tags_12 },
    { 0xb51145f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c7145f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03145f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c114600, SUC_lexicon_tags_11 },
    { 0xbe514601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d914608, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb51460a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d1460d, SUC_lexicon_tags_45 },
    { 0x9911460e, SUC_lexicon_tags_14 },
    { 0x5ff1460f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb914613, SUC_lexicon_tags_77 },
    { 0xc314614, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5311461a, SUC_lexicon_tags_26 },
    { 0x29f1461a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa731461d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95514620, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40714624, SUC_lexicon_tags_6 },
    { 0xf1314625, SUC_lexicon_tags_11 },
    { 0xbe914624, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9031462d, SUC_lexicon_tags_1 },
    { 0x25f1462e, SUC_lexicon_tags_7 },
    { 0x46d1462f, SUC_lexicon_tags_32 },
    { 0x94114630, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7b514632, SUC_lexicon_tags_65 },
    { 0x6bd14632, SUC_lexicon_tags_11 },
    { 0xe7514632, SUC_lexicon_tags_9 },
    { 0xdc914635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51314639, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f14644, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf514646, SUC_lexicon_tags_2 },
    { 0x9d114647, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4dd14649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f1464d, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0xea514650, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c914657, SUC_lexicon_tags_60 },
    { 0x12114657, SUC_lexicon_tags_129 },
    { 0x78114658, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c11465c, SUC_lexicon_tags_293 },
    { 0x151465c, SUC_lexicon_tags_87 },
    { 0x7431465c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e314660, SUC_lexicon_tags_2 },
    { 0x44f14660, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df1466b, SUC_lexicon_tags_38 },
    { 0x2ed1466c, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x7131466e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd714671, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d514673, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c114676, SUC_lexicon_tags_31 },
    { 0xb8714677, SUC_lexicon_tags_30 },
    { 0xd2514678, SUC_lexicon_tags_37 },
    { 0x96114677, SUC_lexicon_tags_37 },
    { 0x1314676, SUC_lexicon_tags_15 },
    { 0x8271467b, SUC_lexicon_tags_19 },
    { 0x9d71467b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x491467e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f14684, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1914686, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43514688, SUC_lexicon_tags_39 },
    { 0x46514689, SUC_lexicon_tags_17 },
    { 0x1051468a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9f1468c, SUC_lexicon_tags_64 },
    { 0xc8b1468c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb91468f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83514694, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d314696, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c914698, SUC_lexicon_tags_18 },
    { 0x8b914699, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1071469e, SUC_lexicon_tags_27 },
    { 0x1ad1469e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x673146a2, SUC_lexicon_tags_15 },
    { 0x2bb146a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x855146a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d146ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919146b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25146b9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b146c0, SUC_lexicon_tags_65 },
    { 0x2d9146c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5146c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5146ca, SUC_lexicon_tags_12 },
    { 0xa2d146cb, SUC_lexicon_tags_30 },
    { 0xb19146cc, SUC_lexicon_tags_14 },
    { 0xf2b146cd, SUC_lexicon_tags_7 },
    { 0x10b146cb, SUC_lexicon_tags_11 },
    { 0x987146cf, SUC_lexicon_tags_65 },
    { 0x163146d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x387146d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2db146d5, SUC_lexicon_tags_37 },
    { 0xadd146d6, SUC_lexicon_tags_2 },
    { 0xcb5146d7, SUC_lexicon_tags_25 },
    { 0xe7f146d7, SUC_lexicon_tags_8 },
    { 0xc99146d9, SUC_lexicon_tags_14 },
    { 0xe2b146da, SUC_lexicon_tags_11 },
    { 0x61b146da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101146e0, SUC_lexicon_tags_1 },
    { 0xd85146e1, SUC_lexicon_tags_69 },
    { 0x855146e1, SUC_lexicon_tags_7 },
    { 0x7f9146e3, SUC_lexicon_tags_26 },
    { 0x6cd146e2, SUC_lexicon_tags_6 },
    { 0xed3146e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd146ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61146ed, SUC_lexicon_tags_19 },
    { 0x5d5146ee, SUC_lexicon_tags_11 },
    { 0x6c1146ed, SUC_lexicon_tags_60 },
    { 0x2b5146ed, SUC_lexicon_tags_27 },
    { 0x8d5146f1, SUC_lexicon_tags_4 },
    { 0xd53146ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x203146f4, SUC_lexicon_tags_12 },
    { 0xc73146f4, SUC_lexicon_tags_17 },
    { 0x345146f6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa53146f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f5146fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b146fe, SUC_lexicon_tags_25 },
    { 0xb55146fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34514703, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89514709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b51470e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb14711, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1914713, SUC_lexicon_tags_23 },
    { 0x62d14714, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5114718, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5271471c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75314724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed114728, SUC_lexicon_tags_2 },
    { 0x81114728, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f14736, SUC_lexicon_tags_4 },
    { 0x81b14737, SUC_lexicon_tags_31 },
    { 0x6d714736, SUC_lexicon_tags_9 },
    { 0x1c914737, SUC_lexicon_tags_17 },
    { 0xfe91473a, SUC_lexicon_tags_12 },
    { 0xe771473a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46114740, SUC_lexicon_tags_25 },
    { 0xc1714741, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf14746, SUC_lexicon_tags_12 },
    { 0x94914747, SUC_lexicon_tags_6 },
    { 0xe1114747, SUC_lexicon_tags_60 },
    { 0x8d14746, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc51474e, SUC_lexicon_tags_151 },
    { 0xc951474e, SUC_lexicon_tags_0 },
    { 0x5b11474f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c314754, SUC_lexicon_tags_47 },
    { 0x86714755, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x43714758, SUC_lexicon_tags_46 },
    { 0x9c914759, SUC_lexicon_tags_5 },
    { 0xa9914759, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4531475c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e91475e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf14763, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d14766, SUC_lexicon_tags_15 },
    { 0xc4114767, SUC_lexicon_tags_43 },
    { 0xd4514768, SUC_lexicon_tags_53 },
    { 0xeeb14769, SUC_lexicon_tags_15 },
    { 0x37b1476a, SUC_lexicon_tags_15 },
    { 0x4351476b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x88b1476d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38914770, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x79314772, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63914774, SUC_lexicon_tags_31 },
    { 0x50b14775, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf2714777, SUC_lexicon_tags_29 },
    { 0x64f14778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d1477b, SUC_lexicon_tags_48 },
    { 0xe71477c, SUC_lexicon_tags_12 },
    { 0xbe91477d, SUC_lexicon_tags_1 },
    { 0xdab1477d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f314780, SUC_lexicon_tags_66 },
    { 0xae914780, SUC_lexicon_tags_8 },
    { 0xa1b14782, SUC_lexicon_tags_65 },
    { 0x6f314783, SUC_lexicon_tags_37 },
    { 0x56f14781, SUC_lexicon_tags_25 },
    { 0x1914780, SUC_lexicon_tags_19 },
    { 0xc6314786, SUC_lexicon_tags_47 },
    { 0x90f14787, SUC_lexicon_tags_11 },
    { 0xc3514787, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcdb1478a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f14790, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25914794, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb531479a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc71479d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9147a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa09147a4, SUC_lexicon_tags_34 },
    { 0x9eb147a5, SUC_lexicon_tags_32 },
    { 0xb03147a5, SUC_lexicon_tags_5 },
    { 0x531147a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65147b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775147b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5147be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965147c2, SUC_lexicon_tags_11 },
    { 0x569147c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe7147c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd19147c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5147cd, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f147d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d147d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c7147d5, SUC_lexicon_tags_26 },
    { 0x44b147d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b147d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df147dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55147e0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x387147e3, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b147e9, SUC_lexicon_tags_7 },
    { 0xf37147ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b147f2, SUC_lexicon_tags_14 },
    { 0xbed147f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa59147f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x909147f7, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d147fa, SUC_lexicon_tags_32 },
    { 0xd9d147fb, SUC_lexicon_tags_12 },
    { 0x943147fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e3147fe, SUC_lexicon_tags_11 },
    { 0x6a3147fe, SUC_lexicon_tags_43 },
    { 0x58f147ff, SUC_lexicon_tags_35 },
    { 0xa4914801, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77914805, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3314808, SUC_lexicon_tags_84 },
    { 0x5914809, SUC_lexicon_tags_65 },
    { 0x9e914809, SUC_lexicon_tags_15 },
    { 0xb4b14809, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f11480d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9611480f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b14813, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf914819, SUC_lexicon_tags_23 },
    { 0x59b1481a, SUC_lexicon_tags_25 },
    { 0x291481b, SUC_lexicon_tags_4 },
    { 0x1cf1481c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b91481e, SUC_lexicon_tags_1 },
    { 0x32b1481f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bb14821, SUC_lexicon_tags_25 },
    { 0xadb14822, SUC_lexicon_tags_2 },
    { 0x8af14821, SUC_lexicon_tags_38 },
    { 0xb314824, SUC_lexicon_tags_25 },
    { 0x6714824, SUC_lexicon_tags_15 },
    { 0x5bb14826, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4011482a, SUC_lexicon_tags_37 },
    { 0x1fd1482b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe314832, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a514834, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f1483a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b1483d, SUC_lexicon_tags_37 },
    { 0xfc51483e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb14841, SUC_lexicon_tags_15 },
    { 0x35314842, SUC_lexicon_tags_45 },
    { 0xa3f14842, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x10f14845, SUC_lexicon_tags_17 },
    { 0x74f14846, SUC_lexicon_tags_22 },
    { 0x48714846, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f1484a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb11484c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd291484f, SUC_lexicon_tags_10 },
    { 0xfd314850, SUC_lexicon_tags_6 },
    { 0xd3b14850, SUC_lexicon_tags_31 },
    { 0xa5d14852, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9bf14854, SUC_lexicon_tags_141 },
    { 0xb0f14855, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d514858, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcb1485a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xac31485c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78914862, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b14869, SUC_lexicon_tags_14 },
    { 0xfe714869, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc514870, SUC_lexicon_tags_87 },
    { 0xb4914871, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cf14873, SUC_lexicon_tags_23 },
    { 0x2bb14874, SUC_lexicon_tags_14 },
    { 0x54114873, SUC_lexicon_tags_25 },
    { 0x46b14876, SUC_lexicon_tags_11 },
    { 0x17d14877, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1331487a, SUC_lexicon_tags_7 },
    { 0xb1f1487b, SUC_lexicon_tags_17 },
    { 0x771487a, SUC_lexicon_tags_60 },
    { 0x17f1487d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38714882, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1714885, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa3314887, SUC_lexicon_tags_15 },
    { 0x2c114887, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8491488b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8c71488d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b91488f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f14892, SUC_lexicon_tags_11 },
    { 0xc3114893, SUC_lexicon_tags_17 },
    { 0x17514894, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16514896, SUC_lexicon_tags_25 },
    { 0xa6714896, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x34b14899, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc31489c, SUC_lexicon_tags_26 },
    { 0x29f1489d, SUC_lexicon_tags_68 },
    { 0xf151489d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x65b148a0, SUC_lexicon_tags_38 },
    { 0x273148a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99b148a3, SUC_lexicon_tags_56 },
    { 0xc4f148a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b148a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1148ae, SUC_lexicon_tags_53 },
    { 0xbf5148af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45148bb, SUC_lexicon_tags_37 },
    { 0x71148bc, SUC_lexicon_tags_1 },
    { 0xd25148bc, SUC_lexicon_tags_37 },
    { 0x1bd148bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839148c3, SUC_lexicon_tags_37 },
    { 0x41148c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef148c7, SUC_lexicon_tags_30 },
    { 0x40d148c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1148cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3148d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43148da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03148e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fb148e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1d148e6, SUC_lexicon_tags_17 },
    { 0x2d148e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff148ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483148ef, SUC_lexicon_tags_8 },
    { 0x7d9148ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f148f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb148f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b148fb, SUC_lexicon_tags_2 },
    { 0xf09148fc, SUC_lexicon_tags_27 },
    { 0x199148fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe43148ff, SUC_lexicon_tags_48 },
    { 0xd4d14900, SUC_lexicon_tags_24 },
    { 0x2c7148ff, SUC_lexicon_tags_63 },
    { 0xaad14902, SUC_lexicon_tags_31 },
    { 0x3ad14903, SUC_lexicon_tags_12 },
    { 0x8b914901, SUC_lexicon_tags_17 },
    { 0x27914903, SUC_lexicon_tags_11 },
    { 0xb114903, SUC_lexicon_tags_14 },
    { 0x16b14907, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x63314909, SUC_lexicon_tags_19 },
    { 0x6371490a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e51490d, SUC_lexicon_tags_584 },
    { 0xea71490e, SUC_lexicon_tags_25 },
    { 0xe271490f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73314912, SUC_lexicon_tags_131 },
    { 0xe6914912, SUC_lexicon_tags_37 },
    { 0x87d14914, SUC_lexicon_tags_11 },
    { 0x49714915, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x38114918, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d1491e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1db14920, SUC_lexicon_tags_17 },
    { 0x10714921, SUC_lexicon_tags_61 },
    { 0xa0514921, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4d14924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f14928, SUC_lexicon_tags_11 },
    { 0x3c714929, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfed1492b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914934, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4714936, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1431493a, SUC_lexicon_tags_104 },
    { 0x2951493a, SUC_lexicon_tags_27 },
    { 0x8f91493c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5f91493e, SUC_lexicon_tags_25 },
    { 0x4351493e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8714944, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad14947, SUC_lexicon_tags_8 },
    { 0xc7914948, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9091494b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1611494e, SUC_lexicon_tags_14 },
    { 0x87b1494e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50314953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba114959, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc211495f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc4b14961, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72714966, SUC_lexicon_tags_15 },
    { 0x30514966, SUC_lexicon_tags_65 },
    { 0xc9b14966, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d31496b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99114970, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5914977, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f1497a, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b71497d, SUC_lexicon_tags_7 },
    { 0x2a91497d, SUC_lexicon_tags_12 },
    { 0xd8f1497d, SUC_lexicon_tags_32 },
    { 0x52714980, SUC_lexicon_tags_11 },
    { 0x42d14981, SUC_lexicon_tags_15 },
    { 0xc6314982, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d14985, SUC_lexicon_tags_25 },
    { 0x65b14985, SUC_lexicon_tags_17 },
    { 0xd4914987, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d71498a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd1498d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c114990, SUC_lexicon_tags_31 },
    { 0x75f14991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf851499a, SUC_lexicon_tags_25 },
    { 0x5a51499b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad1499e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcaf149a0, SUC_lexicon_tags_55 },
    { 0x349149a1, SUC_lexicon_tags_17 },
    { 0xf6b149a1, SUC_lexicon_tags_7 },
    { 0xef149a2, SUC_lexicon_tags_14 },
    { 0xc6f149a2, SUC_lexicon_tags_7 },
    { 0x6b9149a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x43b149a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd149aa, SUC_lexicon_tags_19 },
    { 0xb27149ab, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5149ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2af149b0, SUC_lexicon_tags_33 },
    { 0x117149b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf149b5, SUC_lexicon_tags_25 },
    { 0xc3d149b6, SUC_lexicon_tags_25 },
    { 0xb1d149b7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x815149ba, SUC_lexicon_tags_17 },
    { 0xf09149bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d149bf, SUC_lexicon_tags_37 },
    { 0x473149bf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd149c3, SUC_lexicon_tags_17 },
    { 0xb6d149c3, SUC_lexicon_tags_21 },
    { 0x689149c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617149c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb149cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96f149ce, SUC_lexicon_tags_25 },
    { 0x671149cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827149d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b149d9, SUC_lexicon_tags_25 },
    { 0x4ed149da, SUC_lexicon_tags_37 },
    { 0xaa1149db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd9149dd, SUC_lexicon_tags_43 },
    { 0x3f3149de, SUC_lexicon_tags_25 },
    { 0x627149de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c3149e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb73149e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1149e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb149eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc53149ed, SUC_lexicon_tags_37 },
    { 0x64f149ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d1149f0, SUC_lexicon_tags_63 },
    { 0xe57149f1, SUC_lexicon_tags_0 },
    { 0x29149f1, SUC_lexicon_tags_12 },
    { 0xe07149f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf2d149f5, SUC_lexicon_tags_68 },
    { 0xe01149f6, SUC_lexicon_tags_17 },
    { 0xee7149f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8af149f9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3f149fb, SUC_lexicon_tags_38 },
    { 0xc5d149fc, SUC_lexicon_tags_1 },
    { 0x2db149fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f14a04, SUC_lexicon_tags_12 },
    { 0xf914a05, SUC_lexicon_tags_2 },
    { 0xaa714a06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1714a0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0b14a0c, SUC_lexicon_tags_15 },
    { 0x61114a0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf14a10, SUC_lexicon_tags_0 },
    { 0x3b514a11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea914a16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f14a1a, SUC_lexicon_tags_29 },
    { 0xccd14a1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed14a22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d14a28, SUC_lexicon_tags_25 },
    { 0x35714a29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5114a2f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b14a37, SUC_lexicon_tags_7 },
    { 0x5bf14a38, SUC_lexicon_tags_45 },
    { 0xddf14a37, SUC_lexicon_tags_11 },
    { 0x4ad14a39, SUC_lexicon_tags_7 },
    { 0xa0914a3b, SUC_lexicon_tags_32 },
    { 0x2514a3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f14a41, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b514a45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd14a48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62914a4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8514a4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2314a53, SUC_lexicon_tags_87 },
    { 0x14914a53, SUC_lexicon_tags_60 },
    { 0x20714a55, SUC_lexicon_tags_11 },
    { 0x49514a56, SUC_lexicon_tags_46 },
    { 0x7ab14a57, SUC_lexicon_tags_12 },
    { 0xdf114a54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13314a5d, SUC_lexicon_tags_21 },
    { 0x6bf14a5e, SUC_lexicon_tags_11 },
    { 0xa0f14a5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe114a61, SUC_lexicon_tags_55 },
    { 0xb4114a62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f14a65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff14a69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd14a6e, SUC_lexicon_tags_28 },
    { 0x99714a6f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4714a74, SUC_lexicon_tags_2 },
    { 0xfa314a75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b14a7a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50514a82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b14a85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89914a8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27114a91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d914a93, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x59114a95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b14a98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd114a9a, SUC_lexicon_tags_77 },
    { 0xf9b14a9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d714aa0, SUC_lexicon_tags_11 },
    { 0x74514aa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9f14aa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf14aa7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18914aad, SUC_lexicon_tags_37 },
    { 0x9ed14aae, SUC_lexicon_tags_37 },
    { 0xab914aaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1514ab1, SUC_lexicon_tags_7 },
    { 0xd6314ab2, SUC_lexicon_tags_12 },
    { 0x43b14ab3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0914abb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b14abe, SUC_lexicon_tags_66 },
    { 0x99114abf, SUC_lexicon_tags_68 },
    { 0xdc914abe, SUC_lexicon_tags_11 },
    { 0x66914abf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe314acc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41d14ace, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67114ad4, SUC_lexicon_tags_17 },
    { 0x5b14ad4, SUC_lexicon_tags_17 },
    { 0x5d314ad6, SUC_lexicon_tags_23 },
    { 0x5b14ad7, SUC_lexicon_tags_11 },
    { 0x81114ad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70114ada, SUC_lexicon_tags_23 },
    { 0x68714adb, SUC_lexicon_tags_11 },
    { 0x56314adc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7914ae4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x69f14ae6, SUC_lexicon_tags_12 },
    { 0xce114ae7, SUC_lexicon_tags_8 },
    { 0xb6114ae8, SUC_lexicon_tags_1 },
    { 0x6ad14ae9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30b14aeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd1714aed, SUC_lexicon_tags_60 },
    { 0x95114aee, SUC_lexicon_tags_26 },
    { 0xd4f14aef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11114af4, SUC_lexicon_tags_2 },
    { 0x11314af4, SUC_lexicon_tags_44 },
    { 0x78714af6, SUC_lexicon_tags_11 },
    { 0x6c714af7, SUC_lexicon_tags_45 },
    { 0x2514af8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec114afa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9114aff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8b14b01, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x4a114b03, SUC_lexicon_tags_17 },
    { 0xc4514b04, SUC_lexicon_tags_15 },
    { 0x38914b05, SUC_lexicon_tags_37 },
    { 0x44d14b05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1114b08, SUC_lexicon_tags_12 },
    { 0xb1f14b09, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x23114b0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb14b13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x12114b16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b14b1c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11114b22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58b14b24, SUC_lexicon_tags_47 },
    { 0x7f114b25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x43314b27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69514b2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3b14b2d, SUC_lexicon_tags_29 },
    { 0x5b14b2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab914b30, SUC_lexicon_tags_12 },
    { 0x6714b30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2714b37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf14b3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd314b3e, SUC_lexicon_tags_6 },
    { 0x3f14b3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a114b41, SUC_lexicon_tags_1 },
    { 0x15d14b41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf314b44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a114b4c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x96514b4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73114b55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f314b58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d14b5b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82714b60, SUC_lexicon_tags_29 },
    { 0x3f514b61, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x9eb14b63, SUC_lexicon_tags_37 },
    { 0x2b514b63, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe514b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81d14b69, SUC_lexicon_tags_25 },
    { 0xd0914b69, SUC_lexicon_tags_26 },
    { 0x1314b69, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd114b6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf714b70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5314b73, SUC_lexicon_tags_25 },
    { 0xb9914b74, SUC_lexicon_tags_11 },
    { 0x32b14b75, SUC_lexicon_tags_60 },
    { 0x82914b74, SUC_lexicon_tags_0 },
    { 0xc7b14b77, SUC_lexicon_tags_49 },
    { 0xffb14b78, SUC_lexicon_tags_8 },
    { 0x25b14b79, SUC_lexicon_tags_7 },
    { 0x24b14b78, SUC_lexicon_tags_19 },
    { 0x9b714b7b, SUC_lexicon_tags_37 },
    { 0x18314b7c, SUC_lexicon_tags_15 },
    { 0x3714b79, SUC_lexicon_tags_1 },
    { 0x61114b7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b14b80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3dd14b82, SUC_lexicon_tags_35 },
    { 0xd5514b82, SUC_lexicon_tags_87 },
    { 0x8b14b83, SUC_lexicon_tags_21 },
    { 0x62314b84, SUC_lexicon_tags_17 },
    { 0x30714b82, SUC_lexicon_tags_15 },
    { 0xfa114b85, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x87914b89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2b714b8b, SUC_lexicon_tags_60 },
    { 0x3c714b8c, SUC_lexicon_tags_23 },
    { 0x7bf14b8d, SUC_lexicon_tags_7 },
    { 0xc3d14b8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd14b90, SUC_lexicon_tags_28 },
    { 0x8e314b91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef14b94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82314b97, SUC_lexicon_tags_29 },
    { 0x50514b97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52314b9d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14114ba5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x27514ba7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23114baa, SUC_lexicon_tags_6 },
    { 0x69914bab, SUC_lexicon_tags_6 },
    { 0x94b14baa, SUC_lexicon_tags_12 },
    { 0xa0914bac, SUC_lexicon_tags_7 },
    { 0x2ed14bad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6714bb0, SUC_lexicon_tags_15 },
    { 0xd0714bb1, SUC_lexicon_tags_56 },
    { 0xd3314bb2, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc914bb5, SUC_lexicon_tags_11 },
    { 0xc9914bb5, SUC_lexicon_tags_1 },
    { 0xb6b14bb7, SUC_lexicon_tags_29 },
    { 0xf5b14bb7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9df14bba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3514bbe, SUC_lexicon_tags_25 },
    { 0x86f14bbe, SUC_lexicon_tags_12 },
    { 0x4d714bc0, SUC_lexicon_tags_62 },
    { 0x26d14bc1, SUC_lexicon_tags_14 },
    { 0xa514bc2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd5f14bc4, SUC_lexicon_tags_15 },
    { 0xaff14bc4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b14bcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b14bd1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52714bd7, SUC_lexicon_tags_15 },
    { 0x66514bd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70114bdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae514be2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1914be6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7d14be8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5114bec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f14bf0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d114bf6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16514bf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fb14bfa, SUC_lexicon_tags_26 },
    { 0x33d14bfa, SUC_lexicon_tags_5 },
    { 0x60714bfa, SUC_lexicon_tags_1 },
    { 0x73114bfd, SUC_lexicon_tags_61 },
    { 0xd3314bfe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2514c01, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe9114c03, SUC_lexicon_tags_25 },
    { 0x2a314c04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d514c06, SUC_lexicon_tags_12 },
    { 0xbb14c07, SUC_lexicon_tags_25 },
    { 0x3eb14c07, SUC_lexicon_tags_49 },
    { 0x48714c09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d14c10, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f14c16, SUC_lexicon_tags_25 },
    { 0x95d14c16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d914c1b, SUC_lexicon_tags_11 },
    { 0xea714c1c, SUC_lexicon_tags_9 },
    { 0x86514c1d, SUC_lexicon_tags_25 },
    { 0x93d14c1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32914c20, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d14c28, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x36914c2b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x43514c2d, SUC_lexicon_tags_26 },
    { 0x56b14c2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf14c35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2514c3d, SUC_lexicon_tags_28 },
    { 0x5e714c3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80b14c40, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x25f14c42, SUC_lexicon_tags_47 },
    { 0xd2b14c43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77514c46, SUC_lexicon_tags_17 },
    { 0xbcd14c47, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x25514c49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad14c51, SUC_lexicon_tags_7 },
    { 0x7b714c52, SUC_lexicon_tags_37 },
    { 0xbd14c52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4714c5a, SUC_lexicon_tags_11 },
    { 0xac314c5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10314c64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71114c68, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6914c6a, SUC_lexicon_tags_31 },
    { 0x54514c6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9514c6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24d14c70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b14c76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e314c78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xef114c7a, SUC_lexicon_tags_12 },
    { 0x1f914c7b, SUC_lexicon_tags_30 },
    { 0xe4114c7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbb314c7e, SUC_lexicon_tags_11 },
    { 0xa5b14c7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf14c84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d14c87, SUC_lexicon_tags_10 },
    { 0xa7d14c87, SUC_lexicon_tags_1 },
    { 0x5d714c88, SUC_lexicon_tags_11 },
    { 0xde514c87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb9114c8c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54514c91, SUC_lexicon_tags_9 },
    { 0xeb14c92, SUC_lexicon_tags_27 },
    { 0x1dd14c92, SUC_lexicon_tags_25 },
    { 0xc5714c93, SUC_lexicon_tags_17 },
    { 0x4b514c95, SUC_lexicon_tags_15 },
    { 0x33b14c95, SUC_lexicon_tags_32 },
    { 0x31f14c96, SUC_lexicon_tags_31 },
    { 0x3cd14c93, SUC_lexicon_tags_47 },
    { 0x24114c95, SUC_lexicon_tags_48 },
    { 0x34f14c9a, SUC_lexicon_tags_37 },
    { 0x4ef14c98, SUC_lexicon_tags_113 },
    { 0x5fb14c9c, SUC_lexicon_tags_26 },
    { 0xd1714c9c, SUC_lexicon_tags_68 },
    { 0x81b14c9e, SUC_lexicon_tags_37 },
    { 0xd714c9e, SUC_lexicon_tags_37 },
    { 0xeb914c93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf914ca2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x74914ca5, SUC_lexicon_tags_37 },
    { 0x25d14ca6, SUC_lexicon_tags_17 },
    { 0x58d14ca7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfc914ca9, SUC_lexicon_tags_0 },
    { 0x9bb14ca9, SUC_lexicon_tags_15 },
    { 0x1f514cab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57f14cad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3514cb1, SUC_lexicon_tags_2 },
    { 0x8314cb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2314cb5, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x25914cb7, SUC_lexicon_tags_17 },
    { 0x10714cb7, SUC_lexicon_tags_15 },
    { 0x47914cb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f14cbc, SUC_lexicon_tags_37 },
    { 0xb8914cbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b14cc2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd14cc7, SUC_lexicon_tags_65 },
    { 0x2f514cc8, SUC_lexicon_tags_12 },
    { 0x16b14cc9, SUC_lexicon_tags_15 },
    { 0x59314cca, SUC_lexicon_tags_47 },
    { 0xdd314cc9, SUC_lexicon_tags_91 },
    { 0x3ab14ccc, SUC_lexicon_tags_6 },
    { 0x38714cc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc114ccf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9514cd1, SUC_lexicon_tags_1 },
    { 0xe7d14cd1, SUC_lexicon_tags_23 },
    { 0x19b14cd1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5714cd5, SUC_lexicon_tags_224 },
    { 0x9514cd6, SUC_lexicon_tags_11 },
    { 0x99f14cd5, SUC_lexicon_tags_68 },
    { 0xea714cd7, SUC_lexicon_tags_7 },
    { 0x114cd6, SUC_lexicon_tags_81 },
    { 0xaff14cda, SUC_lexicon_tags_11 },
    { 0x34f14cda, SUC_lexicon_tags_6 },
    { 0xe6d14cd8, SUC_lexicon_tags_29 },
    { 0x26f14cdd, SUC_lexicon_tags_11 },
    { 0xd2f14cd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d14ce2, SUC_lexicon_tags_45 },
    { 0xf714ce3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd14ce6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ff14ce8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2114ced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81114cef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa114cf2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x64914cf5, SUC_lexicon_tags_12 },
    { 0x15914cf6, SUC_lexicon_tags_1 },
    { 0xbb14cf7, SUC_lexicon_tags_31 },
    { 0xd8114cf8, SUC_lexicon_tags_8 },
    { 0xed114cf6, SUC_lexicon_tags_29 },
    { 0x8e914cfa, SUC_lexicon_tags_49 },
    { 0xc6914cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3714cff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf14d07, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x41714d09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13914d0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3d14d10, SUC_lexicon_tags_11 },
    { 0xa3714d10, SUC_lexicon_tags_15 },
    { 0x77f14d12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d14d16, SUC_lexicon_tags_11 },
    { 0xe2b14d17, SUC_lexicon_tags_27 },
    { 0x48314d18, SUC_lexicon_tags_7 },
    { 0x87314d19, SUC_lexicon_tags_17 },
    { 0xcb514d1a, SUC_lexicon_tags_28 },
    { 0xad14d19, SUC_lexicon_tags_19 },
    { 0x4d514d1c, SUC_lexicon_tags_45 },
    { 0x11314d1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46714d20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3714d22, SUC_lexicon_tags_18 },
    { 0x6f714d22, SUC_lexicon_tags_19 },
    { 0x8a514d24, SUC_lexicon_tags_15 },
    { 0x68114d25, SUC_lexicon_tags_7 },
    { 0x80114d26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b714d28, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x15914d2a, SUC_lexicon_tags_37 },
    { 0x9c314d2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf714d2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50d14d30, SUC_lexicon_tags_25 },
    { 0xde114d31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd14d38, SUC_lexicon_tags_17 },
    { 0x45314d39, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f14d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c714d3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65b14d40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c314d45, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c314d49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40114d4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1114d4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10914d52, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x72314d54, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b514d5a, SUC_lexicon_tags_47 },
    { 0x19514d5a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x70114d5d, SUC_lexicon_tags_37 },
    { 0x97714d5d, SUC_lexicon_tags_12 },
    { 0x81d14d5f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7314d61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16714d6b, SUC_lexicon_tags_47 },
    { 0xaff14d6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd14d70, SUC_lexicon_tags_148 },
    { 0x40b14d70, SUC_lexicon_tags_39 },
    { 0xfed14d72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44114d7a, SUC_lexicon_tags_39 },
    { 0x98d14d7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x36914d7d, SUC_lexicon_tags_27 },
    { 0x15114d7d, SUC_lexicon_tags_33 },
    { 0x46914d7e, SUC_lexicon_tags_6 },
    { 0x91114d7e, SUC_lexicon_tags_0 },
    { 0xc114d81, SUC_lexicon_tags_27 },
    { 0x1d714d82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4714d86, SUC_lexicon_tags_417 },
    { 0x62114d87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71714d8e, SUC_lexicon_tags_30 },
    { 0x18f14d8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83314d91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67514d95, SUC_lexicon_tags_1 },
    { 0x56714d96, SUC_lexicon_tags_37 },
    { 0xb1114d97, SUC_lexicon_tags_32 },
    { 0xf4314d98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf14d9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47914da1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd514da4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60514da6, SUC_lexicon_tags_30 },
    { 0xc5514da6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x30514daa, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3914db1, SUC_lexicon_tags_27 },
    { 0x9eb14db1, SUC_lexicon_tags_1 },
    { 0x13914db1, SUC_lexicon_tags_15 },
    { 0x1f114db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f14dba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4714dbe, SUC_lexicon_tags_70 },
    { 0xc1d14dbf, SUC_lexicon_tags_1 },
    { 0x97f14dbf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcf14dc2, SUC_lexicon_tags_12 },
    { 0x37914dc3, SUC_lexicon_tags_17 },
    { 0x7b314dc4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdb914dc6, SUC_lexicon_tags_1 },
    { 0xc7d14dc6, SUC_lexicon_tags_8 },
    { 0x74914dc7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc914dcb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa914dd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58314dd7, SUC_lexicon_tags_7 },
    { 0x69b14dd8, SUC_lexicon_tags_14 },
    { 0x79714dd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d714ddc, SUC_lexicon_tags_11 },
    { 0x86f14ddc, SUC_lexicon_tags_11 },
    { 0x8f114ddd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3314de0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0314de2, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x6cd14de4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60314de6, SUC_lexicon_tags_49 },
    { 0x8d314de7, SUC_lexicon_tags_15 },
    { 0xbd714de8, SUC_lexicon_tags_15 },
    { 0x5b314de8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51914deb, SUC_lexicon_tags_15 },
    { 0x4f914dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67714df0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeeb14df2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4314df5, SUC_lexicon_tags_23 },
    { 0x49f14df6, SUC_lexicon_tags_15 },
    { 0x82114df6, SUC_lexicon_tags_37 },
    { 0xab714df8, SUC_lexicon_tags_17 },
    { 0xb1f14df9, SUC_lexicon_tags_3 },
    { 0x6914df8, SUC_lexicon_tags_26 },
    { 0x5a514dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fb14dfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf14dff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe314e09, SUC_lexicon_tags_60 },
    { 0x9b14e0a, SUC_lexicon_tags_10 },
    { 0xde914e0b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaf714e0d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b514e10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad114e12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2c314e14, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x13514e16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d14e1b, SUC_lexicon_tags_1 },
    { 0x23714e1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63d14e1e, SUC_lexicon_tags_71 },
    { 0x94914e1e, SUC_lexicon_tags_7 },
    { 0xb2514e1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fd14e22, SUC_lexicon_tags_37 },
    { 0x5d114e23, SUC_lexicon_tags_11 },
    { 0xd7b14e22, SUC_lexicon_tags_14 },
    { 0x5f914e22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f14e2c, SUC_lexicon_tags_43 },
    { 0x18b14e2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71914e30, SUC_lexicon_tags_38 },
    { 0x22f14e31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11514e37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf8514e39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf914e3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98314e49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x62f14e4b, SUC_lexicon_tags_1 },
    { 0xadd14e4b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7914e52, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x75114e54, SUC_lexicon_tags_1 },
    { 0xdb14e54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f14e59, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89114e5d, SUC_lexicon_tags_12 },
    { 0x10f14e5e, SUC_lexicon_tags_77 },
    { 0x68714e5f, SUC_lexicon_tags_37 },
    { 0x9fb14e5e, SUC_lexicon_tags_23 },
    { 0x7f314e5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85f14e63, SUC_lexicon_tags_65 },
    { 0x5f14e64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde914e68, SUC_lexicon_tags_11 },
    { 0x9a114e69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf14e6c, SUC_lexicon_tags_38 },
    { 0xe4f14e6c, SUC_lexicon_tags_1 },
    { 0x51114e6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f14e74, SUC_lexicon_tags_15 },
    { 0xbd514e74, SUC_lexicon_tags_49 },
    { 0x45314e76, SUC_lexicon_tags_23 },
    { 0x30714e77, SUC_lexicon_tags_47 },
    { 0xa2514e78, SUC_lexicon_tags_27 },
    { 0x2c914e76, SUC_lexicon_tags_7 },
    { 0x2cf14e7a, SUC_lexicon_tags_33 },
    { 0xe4514e7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd514e7e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x3d14e80, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8f714e82, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb3314e84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8714e88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a314e8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d14e95, SUC_lexicon_tags_88 },
    { 0x9c714e96, SUC_lexicon_tags_10 },
    { 0xff14e97, SUC_lexicon_tags_65 },
    { 0xb6f14e98, SUC_lexicon_tags_87 },
    { 0x5a514e99, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0xfd914e9b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x89914e9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e714e9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9714ea2, SUC_lexicon_tags_7 },
    { 0xef514ea2, SUC_lexicon_tags_9 },
    { 0x39914ea3, SUC_lexicon_tags_45 },
    { 0x4e714ea5, SUC_lexicon_tags_25 },
    { 0xe8714ea2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa314ea8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd2b14eaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x83314ead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d914eb4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f14eb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af14ebf, SUC_lexicon_tags_17 },
    { 0x64314ebf, SUC_lexicon_tags_25 },
    { 0x6e914ec1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d114ec3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d514ec5, SUC_lexicon_tags_2 },
    { 0x50d14ec6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba514eca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea514ed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2314edc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x97d14ede, SUC_lexicon_tags_26 },
    { 0x4ed14ede, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c114ee4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1914ee6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7914ee9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf2314eeb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e914ef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c914ef5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7914efb, SUC_lexicon_tags_7 },
    { 0xde714efc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a114eff, SUC_lexicon_tags_11 },
    { 0xf3714f00, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d14f03, SUC_lexicon_tags_17 },
    { 0x8f314f04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd14f07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47f14f09, SUC_lexicon_tags_11 },
    { 0x68d14f0a, SUC_lexicon_tags_2 },
    { 0x4d314f0b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x28f14f0d, SUC_lexicon_tags_47 },
    { 0x26d14f0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72f14f10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63714f13, SUC_lexicon_tags_15 },
    { 0x11314f13, SUC_lexicon_tags_12 },
    { 0xc5114f15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1b14f17, SUC_lexicon_tags_11 },
    { 0xfc314f18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23b14f1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f714f1c, SUC_lexicon_tags_25 },
    { 0x4b514f1d, SUC_lexicon_tags_25 },
    { 0xe4d14f1e, SUC_lexicon_tags_229 },
    { 0xdfb14f1f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x89d14f21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6b14f23, SUC_lexicon_tags_8 },
    { 0x714f24, SUC_lexicon_tags_37 },
    { 0xac514f23, SUC_lexicon_tags_37 },
    { 0xd0514f26, SUC_lexicon_tags_7 },
    { 0xd2914f27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6eb14f29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51514f2e, SUC_lexicon_tags_7 },
    { 0x5af14f2f, SUC_lexicon_tags_37 },
    { 0xdcb14f30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b14f33, SUC_lexicon_tags_14 },
    { 0x92914f34, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3514f38, SUC_lexicon_tags_31 },
    { 0x48914f39, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7514f41, SUC_lexicon_tags_29 },
    { 0x9b314f42, SUC_lexicon_tags_17 },
    { 0x2bb14f43, SUC_lexicon_tags_8 },
    { 0x2ab14f44, SUC_lexicon_tags_56 },
    { 0xfdd14f44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9914f48, SUC_lexicon_tags_17 },
    { 0x3ad14f49, SUC_lexicon_tags_23 },
    { 0x5db14f49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9f14f4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b14f52, SUC_lexicon_tags_0 },
    { 0xaa114f53, SUC_lexicon_tags_33 },
    { 0xfa514f54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5114f56, SUC_lexicon_tags_17 },
    { 0x35f14f57, SUC_lexicon_tags_11 },
    { 0x3c514f57, SUC_lexicon_tags_25 },
    { 0x69514f59, SUC_lexicon_tags_33 },
    { 0x84714f5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x18514f5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd114f60, SUC_lexicon_tags_35 },
    { 0xc6f14f60, SUC_lexicon_tags_11 },
    { 0xdb514f62, SUC_lexicon_tags_33 },
    { 0x15314f61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e714f69, SUC_lexicon_tags_30 },
    { 0xb114f6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcbb14f6c, SUC_lexicon_tags_1 },
    { 0xea914f6c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8514f70, SUC_lexicon_tags_38 },
    { 0x53914f71, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd14f74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7914f77, SUC_lexicon_tags_17 },
    { 0xcb114f77, SUC_lexicon_tags_47 },
    { 0x1a514f79, SUC_lexicon_tags_66 },
    { 0xbe314f7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79514f81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75314f83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x83314f86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda314f8c, SUC_lexicon_tags_48 },
    { 0x8d714f8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90314f91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49714f93, SUC_lexicon_tags_25 },
    { 0xfd514f94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d714f99, SUC_lexicon_tags_15 },
    { 0xcad14f9a, SUC_lexicon_tags_53 },
    { 0xe6b14f9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8e914f9d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b14fa2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x28114fa5, SUC_lexicon_tags_50 },
    { 0xb0114fa5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x33514fa9, SUC_lexicon_tags_49 },
    { 0x58914faa, SUC_lexicon_tags_7 },
    { 0x33f14fab, SUC_lexicon_tags_0 },
    { 0x7fb14faa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1514faf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d14fb8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21514fbc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3b514fbe, SUC_lexicon_tags_8 },
    { 0xaf14fbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47514fc1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a514fc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a914fc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb14fcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66314fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e714fd3, SUC_lexicon_tags_4 },
    { 0x5b514fd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b714fd8, SUC_lexicon_tags_23 },
    { 0xf8b14fd8, SUC_lexicon_tags_19 },
    { 0x90714fda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda114fe5, SUC_lexicon_tags_27 },
    { 0x50714fe6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x79914fe8, SUC_lexicon_tags_37 },
    { 0x97514fe9, SUC_lexicon_tags_12 },
    { 0x16314fe9, SUC_lexicon_tags_80 },
    { 0x86314feb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb314fef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d114ff1, SUC_lexicon_tags_57 },
    { 0x8fb14ff1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b14ff9, SUC_lexicon_tags_45 },
    { 0xc1d14ff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4914ffc, SUC_lexicon_tags_27 },
    { 0xb7714ffd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49514fff, SUC_lexicon_tags_6 },
    { 0xcef15000, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48515004, SUC_lexicon_tags_12 },
    { 0xea915005, SUC_lexicon_tags_37 },
    { 0x8cf15006, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd15009, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c71500e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af15017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1611501d, SUC_lexicon_tags_2 },
    { 0x1d51501e, SUC_lexicon_tags_25 },
    { 0xead1501d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b915027, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891502d, SUC_lexicon_tags_2 },
    { 0x94f1502d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf15031, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7715034, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef115037, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3931503e, SUC_lexicon_tags_12 },
    { 0x29b1503f, SUC_lexicon_tags_60 },
    { 0xdf51503f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69715045, SUC_lexicon_tags_0 },
    { 0xd5b15046, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb15049, SUC_lexicon_tags_37 },
    { 0x33d1504a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa711504c, SUC_lexicon_tags_30 },
    { 0x58b1504d, SUC_lexicon_tags_65 },
    { 0x1431504e, SUC_lexicon_tags_92 },
    { 0x38d1504e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda715054, SUC_lexicon_tags_11 },
    { 0x59915054, SUC_lexicon_tags_55 },
    { 0xe4515054, SUC_lexicon_tags_27 },
    { 0xaf115057, SUC_lexicon_tags_23 },
    { 0x3c515057, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4591505a, SUC_lexicon_tags_2 },
    { 0x2671505a, SUC_lexicon_tags_1 },
    { 0x47d1505c, SUC_lexicon_tags_25 },
    { 0x70d1505d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df15060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb15070, SUC_lexicon_tags_45 },
    { 0x93f15070, SUC_lexicon_tags_25 },
    { 0x48115072, SUC_lexicon_tags_77 },
    { 0x94915070, SUC_lexicon_tags_7 },
    { 0xc2f15074, SUC_lexicon_tags_0 },
    { 0xd0115075, SUC_lexicon_tags_2 },
    { 0x46715073, SUC_lexicon_tags_12 },
    { 0x1f315077, SUC_lexicon_tags_60 },
    { 0xabd15078, SUC_lexicon_tags_25 },
    { 0x35715077, SUC_lexicon_tags_0 },
    { 0xf091507a, SUC_lexicon_tags_65 },
    { 0x41d1507b, SUC_lexicon_tags_66 },
    { 0x9011507a, SUC_lexicon_tags_17 },
    { 0x711507d, SUC_lexicon_tags_13 },
    { 0x6571507e, SUC_lexicon_tags_17 },
    { 0x57b1507e, SUC_lexicon_tags_45 },
    { 0x7a71507d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8915086, SUC_lexicon_tags_12 },
    { 0xc3b15087, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b1508e, SUC_lexicon_tags_67 },
    { 0x3e51508e, SUC_lexicon_tags_11 },
    { 0x52515090, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32915094, SUC_lexicon_tags_2 },
    { 0xe2915095, SUC_lexicon_tags_30 },
    { 0xf5115095, SUC_lexicon_tags_37 },
    { 0x1bb15096, SUC_lexicon_tags_37 },
    { 0xd0315095, SUC_lexicon_tags_37 },
    { 0x88f15099, SUC_lexicon_tags_77 },
    { 0x7e915099, SUC_lexicon_tags_7 },
    { 0x25d1509b, SUC_lexicon_tags_37 },
    { 0xcd115096, SUC_lexicon_tags_60 },
    { 0xc311509d, SUC_lexicon_tags_7 },
    { 0xbe31509d, SUC_lexicon_tags_28 },
    { 0x39b1509f, SUC_lexicon_tags_6 },
    { 0x47b1509b, SUC_lexicon_tags_47 },
    { 0xf07150a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd150a4, SUC_lexicon_tags_49 },
    { 0x97d150a4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9150a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f1150ab, SUC_lexicon_tags_49 },
    { 0x449150ac, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1150af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d150b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9150b7, SUC_lexicon_tags_37 },
    { 0xcd5150b8, SUC_lexicon_tags_7 },
    { 0x861150b8, SUC_lexicon_tags_1 },
    { 0xc65150ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x217150bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26d150bf, SUC_lexicon_tags_7 },
    { 0xdf9150c0, SUC_lexicon_tags_28 },
    { 0x39f150c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0b150c3, SUC_lexicon_tags_45 },
    { 0x381150c4, SUC_lexicon_tags_15 },
    { 0xc3d150c3, SUC_lexicon_tags_10 },
    { 0xf7f150c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfe1150c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a5150ca, SUC_lexicon_tags_4 },
    { 0x4bd150cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb19150cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9150d1, SUC_lexicon_tags_17 },
    { 0x98f150d1, SUC_lexicon_tags_42 },
    { 0x435150d1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x751150d6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79150da, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x693150dc, SUC_lexicon_tags_47 },
    { 0x481150dc, SUC_lexicon_tags_6 },
    { 0x3f9150de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x471150e0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd81150e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddf150e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619150ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x49b150ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27150f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41150f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x573150f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5150fb, SUC_lexicon_tags_148 },
    { 0x1d3150fc, SUC_lexicon_tags_12 },
    { 0xee7150fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24315102, SUC_lexicon_tags_17 },
    { 0x6df15103, SUC_lexicon_tags_15 },
    { 0x7d715104, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d15108, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1171510a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d51510e, SUC_lexicon_tags_12 },
    { 0x2711510f, SUC_lexicon_tags_27 },
    { 0x7fb15110, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba515112, SUC_lexicon_tags_7 },
    { 0xced15112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe515115, SUC_lexicon_tags_11 },
    { 0x5f15116, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7515118, SUC_lexicon_tags_22 },
    { 0xa4515119, SUC_lexicon_tags_12 },
    { 0xd311511a, SUC_lexicon_tags_15 },
    { 0x7a715118, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f1511e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c115121, SUC_lexicon_tags_43 },
    { 0x14b15122, SUC_lexicon_tags_37 },
    { 0xac515123, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15915125, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7331512a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb1512d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb15130, SUC_lexicon_tags_37 },
    { 0x1e115130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7315133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb515137, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d915139, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a91513b, SUC_lexicon_tags_29 },
    { 0x19f1513c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b1513f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x73915142, SUC_lexicon_tags_7 },
    { 0xc5315143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c915146, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab15149, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42d1514b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf15150, SUC_lexicon_tags_25 },
    { 0x60115151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f15155, SUC_lexicon_tags_1 },
    { 0x9f515156, SUC_lexicon_tags_15 },
    { 0x1dd15157, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59315159, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d71515c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdb31515e, SUC_lexicon_tags_25 },
    { 0x83b1515f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xad115162, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb515168, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a71516e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f15171, SUC_lexicon_tags_4 },
    { 0xd8d15172, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45f15174, SUC_lexicon_tags_14 },
    { 0xcfb15175, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x84115178, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d71517f, SUC_lexicon_tags_0 },
    { 0x96515180, SUC_lexicon_tags_14 },
    { 0x25b15181, SUC_lexicon_tags_62 },
    { 0x93315180, SUC_lexicon_tags_34 },
    { 0x42915183, SUC_lexicon_tags_11 },
    { 0xddd15184, SUC_lexicon_tags_120 },
    { 0x12915185, SUC_lexicon_tags_1 },
    { 0xa6f15185, SUC_lexicon_tags_37 },
    { 0x28115186, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xae91518a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb1518e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f15191, SUC_lexicon_tags_15 },
    { 0xb3715191, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16115196, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd115198, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d1519b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ed1519d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ef1519f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d151a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x673151a5, SUC_lexicon_tags_25 },
    { 0xe69151a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9b151a8, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xe83151aa, SUC_lexicon_tags_68 },
    { 0x4d3151ab, SUC_lexicon_tags_1 },
    { 0x975151ac, SUC_lexicon_tags_17 },
    { 0x613151ad, SUC_lexicon_tags_37 },
    { 0x599151ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1151b2, SUC_lexicon_tags_25 },
    { 0x867151b3, SUC_lexicon_tags_19 },
    { 0xad5151b3, SUC_lexicon_tags_11 },
    { 0xce7151b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x197151b7, SUC_lexicon_tags_1 },
    { 0xccf151b7, SUC_lexicon_tags_23 },
    { 0xe37151b9, SUC_lexicon_tags_12 },
    { 0xfdd151b7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd5151bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb4f151be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x393151c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d151c6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53151cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f151cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad151d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb73151d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf5151d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d151db, SUC_lexicon_tags_47 },
    { 0x5e1151dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01151df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92f151e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995151e5, SUC_lexicon_tags_23 },
    { 0x2b151e5, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x9b1151e8, SUC_lexicon_tags_806 },
    { 0x51b151e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24d151eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19151ee, SUC_lexicon_tags_10 },
    { 0xb6d151ef, SUC_lexicon_tags_141 },
    { 0xc01151f0, SUC_lexicon_tags_449 },
    { 0, NULL },
    { 0xfa9151f2, SUC_lexicon_tags_23 },
    { 0xbc1151f3, SUC_lexicon_tags_35 },
    { 0x249151f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db151f9, SUC_lexicon_tags_28 },
    { 0xe77151f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe47151fc, SUC_lexicon_tags_17 },
    { 0x53151fc, SUC_lexicon_tags_37 },
    { 0x613151fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d115200, SUC_lexicon_tags_37 },
    { 0x11315201, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f915207, SUC_lexicon_tags_17 },
    { 0xaa715207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7251520d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5951520f, SUC_lexicon_tags_30 },
    { 0x32915210, SUC_lexicon_tags_27 },
    { 0xc251520f, SUC_lexicon_tags_1 },
    { 0xb3515211, SUC_lexicon_tags_8 },
    { 0x1eb15213, SUC_lexicon_tags_7 },
    { 0x6fb15210, SUC_lexicon_tags_37 },
    { 0x8b515215, SUC_lexicon_tags_17 },
    { 0x2df15216, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf1d15218, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbcb1521a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8411521c, SUC_lexicon_tags_33 },
    { 0x7d71521d, SUC_lexicon_tags_12 },
    { 0xed1521c, SUC_lexicon_tags_37 },
    { 0xd11521f, SUC_lexicon_tags_47 },
    { 0x771521c, SUC_lexicon_tags_47 },
    { 0xb9515221, SUC_lexicon_tags_31 },
    { 0xfd915222, SUC_lexicon_tags_14 },
    { 0x2b715223, SUC_lexicon_tags_14 },
    { 0x74d15224, SUC_lexicon_tags_10 },
    { 0x5715223, SUC_lexicon_tags_2 },
    { 0xe1915226, SUC_lexicon_tags_48 },
    { 0xb8315227, SUC_lexicon_tags_32 },
    { 0x7f915222, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfef1522a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bf1522c, SUC_lexicon_tags_7 },
    { 0x4db1522c, SUC_lexicon_tags_17 },
    { 0x3031522d, SUC_lexicon_tags_37 },
    { 0xeff1522f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2515234, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd15238, SUC_lexicon_tags_25 },
    { 0x3e115239, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36315241, SUC_lexicon_tags_11 },
    { 0xdad15242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2715245, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54d15247, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xc3315249, SUC_lexicon_tags_30 },
    { 0x3991524a, SUC_lexicon_tags_12 },
    { 0x88f1524b, SUC_lexicon_tags_29 },
    { 0x7731524c, SUC_lexicon_tags_7 },
    { 0x66f1524c, SUC_lexicon_tags_6 },
    { 0x7c515249, SUC_lexicon_tags_37 },
    { 0xb551524f, SUC_lexicon_tags_9 },
    { 0x89115250, SUC_lexicon_tags_25 },
    { 0x65115251, SUC_lexicon_tags_25 },
    { 0x79f15250, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x70315255, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6715257, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0515259, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df1525c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24315260, SUC_lexicon_tags_97 },
    { 0x9ab15260, SUC_lexicon_tags_14 },
    { 0x50115262, SUC_lexicon_tags_43 },
    { 0x56315263, SUC_lexicon_tags_45 },
    { 0x65d15263, SUC_lexicon_tags_31 },
    { 0x62d15262, SUC_lexicon_tags_37 },
    { 0x8e715264, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea915268, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd871526a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf771526c, SUC_lexicon_tags_11 },
    { 0xa811526d, SUC_lexicon_tags_7 },
    { 0x40b1526d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12d15270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af15275, SUC_lexicon_tags_17 },
    { 0x68115276, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12915278, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd631527a, SUC_lexicon_tags_46 },
    { 0xf11527a, SUC_lexicon_tags_48 },
    { 0x29b1527c, SUC_lexicon_tags_17 },
    { 0x9a31527c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92715280, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac715284, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43b15286, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1315290, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x88315292, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff15296, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7315299, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1611529b, SUC_lexicon_tags_19 },
    { 0x6891529b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7d91529e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x563152a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f152a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe15152a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121152aa, SUC_lexicon_tags_12 },
    { 0xdc5152aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9152ae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb152b5, SUC_lexicon_tags_11 },
    { 0x57f152b6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb93152b8, SUC_lexicon_tags_279 },
    { 0xc51152b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1a5152bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x219152be, SUC_lexicon_tags_17 },
    { 0xa41152be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193152c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x515152ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b5152cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded152d0, SUC_lexicon_tags_194 },
    { 0x26d152d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d152d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x999152d7, SUC_lexicon_tags_2 },
    { 0x60f152d8, SUC_lexicon_tags_15 },
    { 0x4bf152d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf85152db, SUC_lexicon_tags_27 },
    { 0xd73152dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x769152de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37152e3, SUC_lexicon_tags_38 },
    { 0xf9d152e4, SUC_lexicon_tags_46 },
    { 0xe1152e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf11152e7, SUC_lexicon_tags_4 },
    { 0xde1152e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x219152eb, SUC_lexicon_tags_148 },
    { 0xf5b152ec, SUC_lexicon_tags_12 },
    { 0x6ed152ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d5152ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337152f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b152f8, SUC_lexicon_tags_87 },
    { 0x9f3152f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff152fc, SUC_lexicon_tags_25 },
    { 0xb5f152fd, SUC_lexicon_tags_11 },
    { 0xb27152fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xee515300, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d15303, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5715305, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f15309, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2471530b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e51530e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b15311, SUC_lexicon_tags_37 },
    { 0x54d15312, SUC_lexicon_tags_27 },
    { 0x77115311, SUC_lexicon_tags_11 },
    { 0xa8515313, SUC_lexicon_tags_11 },
    { 0x5b315315, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cd15317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b1531a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x811531d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7915322, SUC_lexicon_tags_17 },
    { 0x55715323, SUC_lexicon_tags_38 },
    { 0x1c715323, SUC_lexicon_tags_12 },
    { 0xeb715325, SUC_lexicon_tags_11 },
    { 0xc6315326, SUC_lexicon_tags_11 },
    { 0xa5515327, SUC_lexicon_tags_60 },
    { 0x4a315328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b1532d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac315330, SUC_lexicon_tags_37 },
    { 0x2b515330, SUC_lexicon_tags_0 },
    { 0x1e315330, SUC_lexicon_tags_2 },
    { 0xf6715333, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36315335, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcdb15337, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d1533a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6d51533c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c315340, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44115342, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8915345, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf15349, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6851534d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd15356, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65515358, SUC_lexicon_tags_61 },
    { 0xc3115358, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f15360, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa5b15362, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8915364, SUC_lexicon_tags_10 },
    { 0xf1115365, SUC_lexicon_tags_25 },
    { 0xa6f15366, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xbbb15368, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2091536a, SUC_lexicon_tags_32 },
    { 0xb5b1536b, SUC_lexicon_tags_7 },
    { 0xad11536b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66f1536e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f15372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ab15374, SUC_lexicon_tags_8 },
    { 0xbad15374, SUC_lexicon_tags_27 },
    { 0xeff15374, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e115378, SUC_lexicon_tags_199 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb15380, SUC_lexicon_tags_33 },
    { 0x5fb15380, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f15388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4391538b, SUC_lexicon_tags_27 },
    { 0x5451538c, SUC_lexicon_tags_11 },
    { 0x5531538b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9891538f, SUC_lexicon_tags_12 },
    { 0xb9b1538f, SUC_lexicon_tags_11 },
    { 0x7ad1538f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad15395, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a515397, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43153a0, SUC_lexicon_tags_2 },
    { 0xdf153a1, SUC_lexicon_tags_7 },
    { 0x809153a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x133153a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9f153a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64f153a9, SUC_lexicon_tags_63 },
    { 0x697153aa, SUC_lexicon_tags_7 },
    { 0x831153ab, SUC_lexicon_tags_55 },
    { 0xfdf153ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b153b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x157153b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4d153b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa8f153b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6bb153ba, SUC_lexicon_tags_37 },
    { 0xfc7153bb, SUC_lexicon_tags_37 },
    { 0x1a9153bc, SUC_lexicon_tags_37 },
    { 0x725153bd, SUC_lexicon_tags_25 },
    { 0x9f9153be, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf153c1, SUC_lexicon_tags_47 },
    { 0xda1153c1, SUC_lexicon_tags_160 },
    { 0x19f153c3, SUC_lexicon_tags_37 },
    { 0x6e5153c4, SUC_lexicon_tags_9 },
    { 0xf7153c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293153ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x763153cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd75153ce, SUC_lexicon_tags_0 },
    { 0xc15153ce, SUC_lexicon_tags_33 },
    { 0x31153d0, SUC_lexicon_tags_37 },
    { 0xc3b153d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df153d4, SUC_lexicon_tags_9 },
    { 0xaf7153d5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x299153d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af153dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d153e0, SUC_lexicon_tags_17 },
    { 0x82b153e1, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x7e7153e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d153e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197153eb, SUC_lexicon_tags_66 },
    { 0x865153ec, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef153f0, SUC_lexicon_tags_37 },
    { 0x5bd153f1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x729153f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x823153f6, SUC_lexicon_tags_7 },
    { 0xb83153f7, SUC_lexicon_tags_29 },
    { 0xa47153f8, SUC_lexicon_tags_8 },
    { 0x597153f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1153fc, SUC_lexicon_tags_37 },
    { 0xb1153fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6f153ff, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x4bd15401, SUC_lexicon_tags_14 },
    { 0x1115402, SUC_lexicon_tags_15 },
    { 0xebb15403, SUC_lexicon_tags_8 },
    { 0x76b15404, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb15408, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c31540f, SUC_lexicon_tags_26 },
    { 0xb5d1540f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32315412, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d15415, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf915418, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d1541b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab1541e, SUC_lexicon_tags_37 },
    { 0x5231541f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49f15421, SUC_lexicon_tags_32 },
    { 0xa2715422, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d15425, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7515428, SUC_lexicon_tags_11 },
    { 0x10315429, SUC_lexicon_tags_45 },
    { 0x8ef1542a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc31542c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3315434, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98515443, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68315447, SUC_lexicon_tags_55 },
    { 0xef515447, SUC_lexicon_tags_30 },
    { 0xbe915448, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cb1544b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb251544f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xadb15451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7115455, SUC_lexicon_tags_33 },
    { 0xa1715456, SUC_lexicon_tags_17 },
    { 0xb7d15456, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf1545a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad15460, SUC_lexicon_tags_25 },
    { 0x7f115460, SUC_lexicon_tags_7 },
    { 0x93315462, SUC_lexicon_tags_37 },
    { 0x24315463, SUC_lexicon_tags_60 },
    { 0x58915462, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b15468, SUC_lexicon_tags_11 },
    { 0xefd15469, SUC_lexicon_tags_39 },
    { 0x4651546a, SUC_lexicon_tags_6 },
    { 0xe091546b, SUC_lexicon_tags_55 },
    { 0xc471546c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62f1546e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x88b15470, SUC_lexicon_tags_29 },
    { 0x2b315471, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf15475, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x50715478, SUC_lexicon_tags_21 },
    { 0x18315479, SUC_lexicon_tags_37 },
    { 0xb9515478, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1115481, SUC_lexicon_tags_0 },
    { 0x9d115481, SUC_lexicon_tags_11 },
    { 0xe8315482, SUC_lexicon_tags_15 },
    { 0x1e915484, SUC_lexicon_tags_61 },
    { 0x49915485, SUC_lexicon_tags_11 },
    { 0xdc115486, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1915488, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71f1548a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f71548c, SUC_lexicon_tags_1 },
    { 0xff71548c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a715492, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7f715494, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed15497, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf4115499, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b1549d, SUC_lexicon_tags_12 },
    { 0x5631549e, SUC_lexicon_tags_15 },
    { 0x9f71549f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b154a7, SUC_lexicon_tags_15 },
    { 0x83b154a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285154ac, SUC_lexicon_tags_25 },
    { 0xeeb154ad, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7154b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x635154b5, SUC_lexicon_tags_37 },
    { 0xe0f154b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x999154b8, SUC_lexicon_tags_21 },
    { 0x317154b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x597154bb, SUC_lexicon_tags_2 },
    { 0x749154bb, SUC_lexicon_tags_27 },
    { 0x8df154bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb154c0, SUC_lexicon_tags_37 },
    { 0x785154c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6c1154c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b154c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf79154c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d9154ca, SUC_lexicon_tags_2 },
    { 0xffd154cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed154d3, SUC_lexicon_tags_11 },
    { 0xd3154d4, SUC_lexicon_tags_15 },
    { 0xff5154d5, SUC_lexicon_tags_13 },
    { 0x283154d6, SUC_lexicon_tags_25 },
    { 0x111154d7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319154dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81154e0, SUC_lexicon_tags_23 },
    { 0xecf154e1, SUC_lexicon_tags_1 },
    { 0x423154e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79b154e4, SUC_lexicon_tags_33 },
    { 0x5ef154e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07154ef, SUC_lexicon_tags_17 },
    { 0xb7b154ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617154f9, SUC_lexicon_tags_15 },
    { 0xfed154f9, SUC_lexicon_tags_1 },
    { 0x86d154fb, SUC_lexicon_tags_26 },
    { 0x409154fc, SUC_lexicon_tags_37 },
    { 0x3f1154fc, SUC_lexicon_tags_25 },
    { 0x641154fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f15504, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4971550b, SUC_lexicon_tags_15 },
    { 0xd751550c, SUC_lexicon_tags_2 },
    { 0x5791550b, SUC_lexicon_tags_12 },
    { 0xcef1550e, SUC_lexicon_tags_39 },
    { 0x6091550f, SUC_lexicon_tags_11 },
    { 0x1b315510, SUC_lexicon_tags_14 },
    { 0x2071550e, SUC_lexicon_tags_2 },
    { 0xf3115512, SUC_lexicon_tags_11 },
    { 0x2091550c, SUC_lexicon_tags_15 },
    { 0xd7715514, SUC_lexicon_tags_37 },
    { 0xfcb15515, SUC_lexicon_tags_0 },
    { 0xd0715516, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d1551e, SUC_lexicon_tags_37 },
    { 0xdb51551e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b15522, SUC_lexicon_tags_4 },
    { 0xd7915522, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x70b15525, SUC_lexicon_tags_15 },
    { 0xa9515526, SUC_lexicon_tags_11 },
    { 0xcbb15526, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d1552c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbc91552e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c715531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7115533, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54b15535, SUC_lexicon_tags_17 },
    { 0x64b15536, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86f15538, SUC_lexicon_tags_38 },
    { 0xc0b15539, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51553c, SUC_lexicon_tags_17 },
    { 0x8ab1553d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a915542, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x66315544, SUC_lexicon_tags_16 },
    { 0xd5515545, SUC_lexicon_tags_1 },
    { 0x1ef15544, SUC_lexicon_tags_37 },
    { 0xa5d15546, SUC_lexicon_tags_17 },
    { 0xcbd15548, SUC_lexicon_tags_1 },
    { 0x87915549, SUC_lexicon_tags_1 },
    { 0xf551554a, SUC_lexicon_tags_8 },
    { 0x8291554b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59715550, SUC_lexicon_tags_2 },
    { 0xd4915551, SUC_lexicon_tags_38 },
    { 0x88715551, SUC_lexicon_tags_43 },
    { 0x43115553, SUC_lexicon_tags_26 },
    { 0x75f15554, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b915557, SUC_lexicon_tags_551 },
    { 0x30315557, SUC_lexicon_tags_5 },
    { 0x4e315558, SUC_lexicon_tags_37 },
    { 0x9fb15558, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d1555f, SUC_lexicon_tags_7 },
    { 0x57b1555f, SUC_lexicon_tags_17 },
    { 0x77115561, SUC_lexicon_tags_15 },
    { 0x8d315562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13915567, SUC_lexicon_tags_31 },
    { 0x51715568, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431556e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae515578, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50d1557a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc31557e, SUC_lexicon_tags_37 },
    { 0x5551557e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b915585, SUC_lexicon_tags_33 },
    { 0xd0915586, SUC_lexicon_tags_15 },
    { 0xf915587, SUC_lexicon_tags_30 },
    { 0x4e715588, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b1558c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f15591, SUC_lexicon_tags_37 },
    { 0xf2915592, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f15598, SUC_lexicon_tags_47 },
    { 0x4f915599, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xa611559b, SUC_lexicon_tags_4 },
    { 0x50d1559c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5171559e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b155a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa6f155a4, SUC_lexicon_tags_15 },
    { 0x22f155a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf33155a7, SUC_lexicon_tags_60 },
    { 0xc59155a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x719155aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b155af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3f155b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809155b8, SUC_lexicon_tags_49 },
    { 0x149155b9, SUC_lexicon_tags_1 },
    { 0xf8f155ba, SUC_lexicon_tags_15 },
    { 0xfb7155b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343155c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f155c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41d155c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed155c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x77b155ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d155cf, SUC_lexicon_tags_47 },
    { 0x91f155d0, SUC_lexicon_tags_9 },
    { 0x2e9155d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x749155d4, SUC_lexicon_tags_47 },
    { 0x11155d4, SUC_lexicon_tags_10 },
    { 0x3bf155d4, SUC_lexicon_tags_7 },
    { 0x575155d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d155dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d155e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7155e4, SUC_lexicon_tags_2 },
    { 0xe05155e5, SUC_lexicon_tags_39 },
    { 0xe69155e5, SUC_lexicon_tags_148 },
    { 0x8c1155e5, SUC_lexicon_tags_60 },
    { 0x489155e8, SUC_lexicon_tags_14 },
    { 0xfc9155e8, SUC_lexicon_tags_87 },
    { 0xb93155e9, SUC_lexicon_tags_37 },
    { 0xc9f155eb, SUC_lexicon_tags_2 },
    { 0x495155e6, SUC_lexicon_tags_7 },
    { 0x113155ed, SUC_lexicon_tags_9 },
    { 0x651155ed, SUC_lexicon_tags_17 },
    { 0x68d155ef, SUC_lexicon_tags_11 },
    { 0x77155ef, SUC_lexicon_tags_37 },
    { 0x867155f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x201155f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf7f155f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeeb155f7, SUC_lexicon_tags_29 },
    { 0x82b155f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d155fd, SUC_lexicon_tags_17 },
    { 0x57155fe, SUC_lexicon_tags_60 },
    { 0x471155ff, SUC_lexicon_tags_65 },
    { 0x2af155fd, SUC_lexicon_tags_1147 },
    { 0x399155ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x98315604, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea115609, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f1560c, SUC_lexicon_tags_27 },
    { 0xab51560d, SUC_lexicon_tags_35 },
    { 0x1b11560e, SUC_lexicon_tags_32 },
    { 0x5031560c, SUC_lexicon_tags_15 },
    { 0x64915610, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x58f15612, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf15615, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f315618, SUC_lexicon_tags_4 },
    { 0x51315618, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a51561e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x34915621, SUC_lexicon_tags_11 },
    { 0xc5f15622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71562d, SUC_lexicon_tags_37 },
    { 0xd311562d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce515632, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd915634, SUC_lexicon_tags_267 },
    { 0x99315634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811563c, SUC_lexicon_tags_31 },
    { 0x8db1563d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf715640, SUC_lexicon_tags_39 },
    { 0xd0515641, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56715645, SUC_lexicon_tags_17 },
    { 0xc6d15646, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbf15648, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f1564c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8915653, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f1565a, SUC_lexicon_tags_25 },
    { 0x2451565b, SUC_lexicon_tags_27 },
    { 0xc11565c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e515661, SUC_lexicon_tags_15 },
    { 0x67b15661, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b915666, SUC_lexicon_tags_23 },
    { 0x69d15666, SUC_lexicon_tags_47 },
    { 0x3ef15667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeef1566a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa31566d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c31566f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61115671, SUC_lexicon_tags_7 },
    { 0x1d15671, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa7d15674, SUC_lexicon_tags_17 },
    { 0xf8515675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6315677, SUC_lexicon_tags_38 },
    { 0xad515677, SUC_lexicon_tags_14 },
    { 0xd8715678, SUC_lexicon_tags_23 },
    { 0x4e11567a, SUC_lexicon_tags_2 },
    { 0xabb1567a, SUC_lexicon_tags_47 },
    { 0x73d1567c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f315683, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d1568a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f15693, SUC_lexicon_tags_14 },
    { 0xe5f15694, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc11569e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x923156a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc17156a3, SUC_lexicon_tags_47 },
    { 0x475156a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1156a8, SUC_lexicon_tags_25 },
    { 0x35b156a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2d7156ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209156b5, SUC_lexicon_tags_65 },
    { 0x649156b6, SUC_lexicon_tags_15 },
    { 0xb85156b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13156bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f3156bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2b156bf, SUC_lexicon_tags_7 },
    { 0xfb9156c0, SUC_lexicon_tags_38 },
    { 0xd4b156bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537156ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b156cd, SUC_lexicon_tags_12 },
    { 0xcf5156ce, SUC_lexicon_tags_4 },
    { 0xc27156cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b156d5, SUC_lexicon_tags_9 },
    { 0xc3b156d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe3d156d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1156db, SUC_lexicon_tags_1 },
    { 0xc7b156dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x883156de, SUC_lexicon_tags_1061 },
    { 0, NULL },
    { 0xa99156e0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x97d156e2, SUC_lexicon_tags_11 },
    { 0xb6b156e3, SUC_lexicon_tags_32 },
    { 0xc19156e2, SUC_lexicon_tags_14 },
    { 0x885156e5, SUC_lexicon_tags_45 },
    { 0xb03156e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec1156e8, SUC_lexicon_tags_12 },
    { 0x91f156e9, SUC_lexicon_tags_43 },
    { 0x387156e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5ff156ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1156f0, SUC_lexicon_tags_12 },
    { 0x1a9156f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x705156f3, SUC_lexicon_tags_38 },
    { 0x66f156f3, SUC_lexicon_tags_82 },
    { 0x385156f4, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b156f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3156fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7b115700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x30715703, SUC_lexicon_tags_48 },
    { 0xe9715704, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0x80115706, SUC_lexicon_tags_17 },
    { 0x46515706, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8171570a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1570f, SUC_lexicon_tags_17 },
    { 0x22d1570f, SUC_lexicon_tags_37 },
    { 0x4011570f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c315714, SUC_lexicon_tags_68 },
    { 0x8fd15715, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f11571a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3971571c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8515720, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47115722, SUC_lexicon_tags_18 },
    { 0xce515723, SUC_lexicon_tags_25 },
    { 0x1c115724, SUC_lexicon_tags_25 },
    { 0x7e915725, SUC_lexicon_tags_8 },
    { 0x4db15726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78b15728, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7031572a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b15732, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf915734, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4f15736, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d15739, SUC_lexicon_tags_33 },
    { 0x2e71573a, SUC_lexicon_tags_3 },
    { 0x7391573b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc11573e, SUC_lexicon_tags_17 },
    { 0x69d1573f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8915742, SUC_lexicon_tags_10 },
    { 0xeb715743, SUC_lexicon_tags_6 },
    { 0x10915742, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23315746, SUC_lexicon_tags_25 },
    { 0x51915747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cd15749, SUC_lexicon_tags_49 },
    { 0xcf31574a, SUC_lexicon_tags_17 },
    { 0x5d915749, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc651574f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c915752, SUC_lexicon_tags_87 },
    { 0x6b15753, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb715757, SUC_lexicon_tags_33 },
    { 0xe715758, SUC_lexicon_tags_21 },
    { 0xc7d15759, SUC_lexicon_tags_1 },
    { 0x16b15757, SUC_lexicon_tags_3 },
    { 0x3211575b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1575e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87115760, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d15763, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0xd3715765, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0915768, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa191576e, SUC_lexicon_tags_32 },
    { 0xa91576f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81115774, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d15778, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5111577a, SUC_lexicon_tags_12 },
    { 0xb4f1577a, SUC_lexicon_tags_27 },
    { 0xa711577c, SUC_lexicon_tags_1 },
    { 0xdd1577d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3731577f, SUC_lexicon_tags_11 },
    { 0x9db15780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d715782, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed115788, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf151578b, SUC_lexicon_tags_47 },
    { 0x4171578c, SUC_lexicon_tags_11 },
    { 0xb331578d, SUC_lexicon_tags_45 },
    { 0x8291578e, SUC_lexicon_tags_17 },
    { 0x92d1578d, SUC_lexicon_tags_16 },
    { 0x1371578e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b15798, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa91579a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9631579c, SUC_lexicon_tags_43 },
    { 0x6171579d, SUC_lexicon_tags_15 },
    { 0x5791579e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc01157a0, SUC_lexicon_tags_27 },
    { 0xfa5157a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91b157a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb49157a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3157a8, SUC_lexicon_tags_15 },
    { 0x3e5157a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe53157ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc89157ad, SUC_lexicon_tags_7 },
    { 0x931157ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5157b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb45157b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf157b6, SUC_lexicon_tags_7 },
    { 0x56f157b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f157ba, SUC_lexicon_tags_1 },
    { 0x3cb157bb, SUC_lexicon_tags_45 },
    { 0x83f157ba, SUC_lexicon_tags_25 },
    { 0xf7d157bb, SUC_lexicon_tags_27 },
    { 0x1a3157bb, SUC_lexicon_tags_17 },
    { 0x8bb157bf, SUC_lexicon_tags_66 },
    { 0xca3157c0, SUC_lexicon_tags_31 },
    { 0xeb1157c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89157c7, SUC_lexicon_tags_61 },
    { 0xa57157c8, SUC_lexicon_tags_60 },
    { 0xb4b157c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e5157cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x309157cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03157d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbb157d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9157d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x617157d8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf73157da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7157dc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf157df, SUC_lexicon_tags_14 },
    { 0xb23157df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x603157e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5157eb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d157f2, SUC_lexicon_tags_45 },
    { 0x25d157f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1157fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed315801, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x21315804, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b15808, SUC_lexicon_tags_9 },
    { 0xfc515809, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd1580c, SUC_lexicon_tags_27 },
    { 0xa7f1580d, SUC_lexicon_tags_15 },
    { 0x58d1580e, SUC_lexicon_tags_12 },
    { 0xb071580d, SUC_lexicon_tags_7 },
    { 0x5731580e, SUC_lexicon_tags_13 },
    { 0xb8b1580f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb515817, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20115819, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb871581c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1851581f, SUC_lexicon_tags_38 },
    { 0xae91581f, SUC_lexicon_tags_37 },
    { 0xaab1581f, SUC_lexicon_tags_40 },
    { 0x7691581f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb515824, SUC_lexicon_tags_28 },
    { 0xd0115825, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f1582b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc11582e, SUC_lexicon_tags_38 },
    { 0x9391582f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x49515832, SUC_lexicon_tags_47 },
    { 0x3d715832, SUC_lexicon_tags_12 },
    { 0x9b715834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1583b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f1583e, SUC_lexicon_tags_29 },
    { 0x9091583f, SUC_lexicon_tags_37 },
    { 0xd0515840, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d15843, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30715845, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b15849, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa871584b, SUC_lexicon_tags_7 },
    { 0x87d1584b, SUC_lexicon_tags_28 },
    { 0xf7b1584b, SUC_lexicon_tags_37 },
    { 0xa731584d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff15853, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94715855, SUC_lexicon_tags_25 },
    { 0x41915856, SUC_lexicon_tags_3 },
    { 0x2b715855, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e11585a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5051585c, SUC_lexicon_tags_33 },
    { 0xf4f1585c, SUC_lexicon_tags_12 },
    { 0x18f1585c, SUC_lexicon_tags_37 },
    { 0x4231585e, SUC_lexicon_tags_37 },
    { 0xfd51585c, SUC_lexicon_tags_437 },
    { 0, NULL },
    { 0xe7f15862, SUC_lexicon_tags_37 },
    { 0x7715863, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41d15865, SUC_lexicon_tags_6 },
    { 0xf1b15866, SUC_lexicon_tags_63 },
    { 0xe5f15867, SUC_lexicon_tags_77 },
    { 0x4bb15865, SUC_lexicon_tags_25 },
    { 0xca915866, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x68b1586b, SUC_lexicon_tags_68 },
    { 0x4211586c, SUC_lexicon_tags_15 },
    { 0xf631586b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d15871, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e715877, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb131587a, SUC_lexicon_tags_26 },
    { 0x1231587a, SUC_lexicon_tags_60 },
    { 0xd331587b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11b1587e, SUC_lexicon_tags_29 },
    { 0x1a11587e, SUC_lexicon_tags_4 },
    { 0x70715880, SUC_lexicon_tags_29 },
    { 0x6f115880, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x54315884, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8515888, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d1588c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa531588f, SUC_lexicon_tags_9 },
    { 0x7271588f, SUC_lexicon_tags_0 },
    { 0x5bf15891, SUC_lexicon_tags_43 },
    { 0x65515890, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30715894, SUC_lexicon_tags_25 },
    { 0xf0f15894, SUC_lexicon_tags_25 },
    { 0x63515896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf15899, SUC_lexicon_tags_13 },
    { 0x25315899, SUC_lexicon_tags_8 },
    { 0x3651589b, SUC_lexicon_tags_15 },
    { 0x11b1589c, SUC_lexicon_tags_116 },
    { 0x6ef1589b, SUC_lexicon_tags_12 },
    { 0xa251589e, SUC_lexicon_tags_27 },
    { 0x4671589f, SUC_lexicon_tags_7 },
    { 0xb571589e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3eb158a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda7158a4, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xc3b158a6, SUC_lexicon_tags_26 },
    { 0xe19158a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947158ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d158ae, SUC_lexicon_tags_25 },
    { 0x3bd158af, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71158b4, SUC_lexicon_tags_37 },
    { 0x8cb158b5, SUC_lexicon_tags_11 },
    { 0x957158b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf3158b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b158bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd21158be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5158c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x651158c5, SUC_lexicon_tags_25 },
    { 0x7f7158c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ef158c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9bb158ca, SUC_lexicon_tags_5 },
    { 0x9f3158cb, SUC_lexicon_tags_29 },
    { 0xec3158cb, SUC_lexicon_tags_12 },
    { 0x129158cd, SUC_lexicon_tags_9 },
    { 0xc95158ce, SUC_lexicon_tags_19 },
    { 0x74d158cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f158d2, SUC_lexicon_tags_35 },
    { 0x2ad158d3, SUC_lexicon_tags_69 },
    { 0x93f158d3, SUC_lexicon_tags_6 },
    { 0x295158d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x963158d7, SUC_lexicon_tags_33 },
    { 0x1fb158d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85158dc, SUC_lexicon_tags_12 },
    { 0xc45158dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9158df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43158e2, SUC_lexicon_tags_30 },
    { 0x949158e2, SUC_lexicon_tags_11 },
    { 0x3c1158e4, SUC_lexicon_tags_3 },
    { 0x38f158e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb158e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91158ec, SUC_lexicon_tags_17 },
    { 0x66f158ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x545158ef, SUC_lexicon_tags_23 },
    { 0x637158f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5158f3, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x53158f6, SUC_lexicon_tags_37 },
    { 0x37158f7, SUC_lexicon_tags_19 },
    { 0xcdb158f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x41158fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3df158fc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8af158fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe0915900, SUC_lexicon_tags_77 },
    { 0xff915901, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92315903, SUC_lexicon_tags_0 },
    { 0x6bd15904, SUC_lexicon_tags_16 },
    { 0xf4115905, SUC_lexicon_tags_26 },
    { 0xf9315904, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab15908, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd091590b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1590e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70915913, SUC_lexicon_tags_38 },
    { 0x76f15914, SUC_lexicon_tags_47 },
    { 0x7b15915, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d15918, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb591591a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5251591d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x4d51591f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2115924, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed715926, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8891592a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf551592e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf15933, SUC_lexicon_tags_37 },
    { 0xdd915933, SUC_lexicon_tags_2 },
    { 0x8a715935, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59115939, SUC_lexicon_tags_59 },
    { 0xed15939, SUC_lexicon_tags_107 },
    { 0xb3d1593b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0915941, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8715947, SUC_lexicon_tags_11 },
    { 0x9f315948, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d1594b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x851594d, SUC_lexicon_tags_9 },
    { 0x48f1594d, SUC_lexicon_tags_25 },
    { 0x95b1594f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f315957, SUC_lexicon_tags_27 },
    { 0x45115958, SUC_lexicon_tags_49 },
    { 0x4515957, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1071595b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af1595e, SUC_lexicon_tags_37 },
    { 0x62d1595f, SUC_lexicon_tags_2 },
    { 0xc3515960, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xd8515962, SUC_lexicon_tags_11 },
    { 0xc8115963, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x50115966, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1715968, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe391596d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8315972, SUC_lexicon_tags_37 },
    { 0xa4715973, SUC_lexicon_tags_65 },
    { 0xb1315973, SUC_lexicon_tags_7 },
    { 0xd115974, SUC_lexicon_tags_14 },
    { 0x9f15976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2271597a, SUC_lexicon_tags_2 },
    { 0x3b31597b, SUC_lexicon_tags_12 },
    { 0xd1597b, SUC_lexicon_tags_37 },
    { 0xe51597c, SUC_lexicon_tags_1 },
    { 0xdab1597c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f15981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b15984, SUC_lexicon_tags_1 },
    { 0x51115985, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9271598b, SUC_lexicon_tags_1 },
    { 0xcb1598b, SUC_lexicon_tags_32 },
    { 0xf671598d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3115994, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdd315996, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8211599a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec11599e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb159a2, SUC_lexicon_tags_25 },
    { 0xf35159a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x471159a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e1159a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813159ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2f1159b0, SUC_lexicon_tags_33 },
    { 0x7ab159b0, SUC_lexicon_tags_2 },
    { 0x2c5159b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8f159b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x209159b6, SUC_lexicon_tags_63 },
    { 0x363159b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe55159b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65159bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x94b159bf, SUC_lexicon_tags_6 },
    { 0xc95159c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18b159c2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd159c6, SUC_lexicon_tags_17 },
    { 0x3ed159c7, SUC_lexicon_tags_12 },
    { 0x555159c8, SUC_lexicon_tags_47 },
    { 0x4d1159c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x379159cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7159d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95159d3, SUC_lexicon_tags_11 },
    { 0x959159d4, SUC_lexicon_tags_61 },
    { 0xffb159d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x571159d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd159da, SUC_lexicon_tags_45 },
    { 0x27159db, SUC_lexicon_tags_27 },
    { 0x741159da, SUC_lexicon_tags_37 },
    { 0x327159dd, SUC_lexicon_tags_25 },
    { 0x55159da, SUC_lexicon_tags_85 },
    { 0x4e3159df, SUC_lexicon_tags_46 },
    { 0x7e7159e0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc2b159e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d159e6, SUC_lexicon_tags_61 },
    { 0x521159e7, SUC_lexicon_tags_17 },
    { 0xfbf159e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2d159ea, SUC_lexicon_tags_15 },
    { 0xf37159ea, SUC_lexicon_tags_23 },
    { 0x7ad159ec, SUC_lexicon_tags_25 },
    { 0x9df159ec, SUC_lexicon_tags_30 },
    { 0x611159ee, SUC_lexicon_tags_1 },
    { 0xaf3159ee, SUC_lexicon_tags_12 },
    { 0x259159ed, SUC_lexicon_tags_38 },
    { 0x3b5159ec, SUC_lexicon_tags_47 },
    { 0x443159ef, SUC_lexicon_tags_45 },
    { 0xd2d159f3, SUC_lexicon_tags_21 },
    { 0x727159f4, SUC_lexicon_tags_14 },
    { 0xcc1159eb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9159f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5159fc, SUC_lexicon_tags_2 },
    { 0x149159fc, SUC_lexicon_tags_31 },
    { 0x31f159fe, SUC_lexicon_tags_65 },
    { 0xadb159ff, SUC_lexicon_tags_27 },
    { 0x6ab159fe, SUC_lexicon_tags_25 },
    { 0x2c115a00, SUC_lexicon_tags_2 },
    { 0xc715a02, SUC_lexicon_tags_43 },
    { 0xc2d159fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28515a0d, SUC_lexicon_tags_28 },
    { 0x69115a0e, SUC_lexicon_tags_63 },
    { 0x34315a0f, SUC_lexicon_tags_25 },
    { 0xd7b15a0f, SUC_lexicon_tags_7 },
    { 0xd7f15a11, SUC_lexicon_tags_25 },
    { 0xd2f15a12, SUC_lexicon_tags_12 },
    { 0x48715a13, SUC_lexicon_tags_31 },
    { 0x4af15a12, SUC_lexicon_tags_11 },
    { 0xdcb15a15, SUC_lexicon_tags_70 },
    { 0xfd515a15, SUC_lexicon_tags_9 },
    { 0x8b715a17, SUC_lexicon_tags_30 },
    { 0x59b15a17, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9315a1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe115a1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61115a22, SUC_lexicon_tags_21 },
    { 0xb0915a22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2fd15a25, SUC_lexicon_tags_11 },
    { 0x71715a26, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9315a2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea915a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d15a3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9115a3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b15a41, SUC_lexicon_tags_7 },
    { 0x3d915a42, SUC_lexicon_tags_57 },
    { 0x60b15a42, SUC_lexicon_tags_47 },
    { 0xbf915a44, SUC_lexicon_tags_27 },
    { 0x23915a45, SUC_lexicon_tags_38 },
    { 0x77315a46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0915a4d, SUC_lexicon_tags_60 },
    { 0x94515a4e, SUC_lexicon_tags_65 },
    { 0x9915a4d, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xda115a51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b15a54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f315a57, SUC_lexicon_tags_53 },
    { 0x6ad15a58, SUC_lexicon_tags_22 },
    { 0xc5515a59, SUC_lexicon_tags_7 },
    { 0x29d15a5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a915a5c, SUC_lexicon_tags_25 },
    { 0xfe315a5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6115a5f, SUC_lexicon_tags_0 },
    { 0x3a515a5f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xdf915a62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef15a65, SUC_lexicon_tags_66 },
    { 0x6d115a66, SUC_lexicon_tags_15 },
    { 0xb1b15a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af15a6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b15a72, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0515a75, SUC_lexicon_tags_47 },
    { 0x8bb15a76, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d15a7a, SUC_lexicon_tags_33 },
    { 0xa1715a7b, SUC_lexicon_tags_10 },
    { 0xf7315a7a, SUC_lexicon_tags_49 },
    { 0xac115a7d, SUC_lexicon_tags_7 },
    { 0x3ff15a7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9115a80, SUC_lexicon_tags_68 },
    { 0x30515a81, SUC_lexicon_tags_7 },
    { 0x84115a81, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b15a88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3315a8a, SUC_lexicon_tags_84 },
    { 0x2a315a8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d15a8f, SUC_lexicon_tags_66 },
    { 0xbe515a90, SUC_lexicon_tags_65 },
    { 0xf1715a91, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f15a97, SUC_lexicon_tags_25 },
    { 0xe6315a98, SUC_lexicon_tags_1 },
    { 0xb2f15a99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2515a9c, SUC_lexicon_tags_3 },
    { 0x3ad15a9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x23115aa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6ef15aa2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb15aa8, SUC_lexicon_tags_2 },
    { 0xff715aa9, SUC_lexicon_tags_2 },
    { 0x46f15aa9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b15aaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46115ab5, SUC_lexicon_tags_42 },
    { 0x71515ab5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db15ab9, SUC_lexicon_tags_17 },
    { 0xccf15aba, SUC_lexicon_tags_37 },
    { 0x7fd15abb, SUC_lexicon_tags_11 },
    { 0x4b715ab9, SUC_lexicon_tags_19 },
    { 0x93515ab9, SUC_lexicon_tags_12 },
    { 0xe7515abe, SUC_lexicon_tags_31 },
    { 0xb7115abf, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0xa3315ac1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a315ac8, SUC_lexicon_tags_17 },
    { 0xa8d15ac9, SUC_lexicon_tags_23 },
    { 0xb2715ac8, SUC_lexicon_tags_82 },
    { 0x76515ac8, SUC_lexicon_tags_1 },
    { 0xca115ac9, SUC_lexicon_tags_47 },
    { 0x1ad15acd, SUC_lexicon_tags_1 },
    { 0x56f15acc, SUC_lexicon_tags_35 },
    { 0x7bd15acf, SUC_lexicon_tags_7 },
    { 0xc6b15ad0, SUC_lexicon_tags_11 },
    { 0x1cb15ad1, SUC_lexicon_tags_33 },
    { 0x4bb15ad2, SUC_lexicon_tags_37 },
    { 0xc6515ad3, SUC_lexicon_tags_11 },
    { 0xac515ad1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69115ad6, SUC_lexicon_tags_60 },
    { 0x73115ad6, SUC_lexicon_tags_56 },
    { 0xcc515ad8, SUC_lexicon_tags_82 },
    { 0x3d715ad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e115ade, SUC_lexicon_tags_14 },
    { 0xaf915ade, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b715ae2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c915ae7, SUC_lexicon_tags_37 },
    { 0xefd15ae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33f15aea, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50915aee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x90d15af0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98915af2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61715af4, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d15af8, SUC_lexicon_tags_11 },
    { 0xc8915af8, SUC_lexicon_tags_23 },
    { 0xd6915af9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8115aff, SUC_lexicon_tags_26 },
    { 0xbf315b00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22f15b02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff315b05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9715b0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb15b0d, SUC_lexicon_tags_11 },
    { 0xb4715b0e, SUC_lexicon_tags_547 },
    { 0x18315b0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67915b14, SUC_lexicon_tags_0 },
    { 0x88d15b14, SUC_lexicon_tags_27 },
    { 0xddb15b15, SUC_lexicon_tags_37 },
    { 0x83915b17, SUC_lexicon_tags_17 },
    { 0xc4315b18, SUC_lexicon_tags_25 },
    { 0x40515b19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed15b1c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x41515b1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3b15b20, SUC_lexicon_tags_32 },
    { 0xd7d15b21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d15b25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b15b29, SUC_lexicon_tags_9 },
    { 0xce915b2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c715b2c, SUC_lexicon_tags_60 },
    { 0xd15b2d, SUC_lexicon_tags_38 },
    { 0x18915b2d, SUC_lexicon_tags_3 },
    { 0x49515b2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f15b33, SUC_lexicon_tags_80 },
    { 0x66b15b33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52f15b36, SUC_lexicon_tags_17 },
    { 0xab515b37, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda315b3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb15b3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78715b42, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5a915b44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xca915b47, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82515b4c, SUC_lexicon_tags_25 },
    { 0xa5515b4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64715b52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99915b57, SUC_lexicon_tags_38 },
    { 0xe0f15b57, SUC_lexicon_tags_31 },
    { 0xab15b58, SUC_lexicon_tags_25 },
    { 0x13315b5a, SUC_lexicon_tags_0 },
    { 0x2b15b5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5715b69, SUC_lexicon_tags_27 },
    { 0xa0f15b6a, SUC_lexicon_tags_218 },
    { 0x46515b6a, SUC_lexicon_tags_19 },
    { 0xd8b15b6a, SUC_lexicon_tags_19 },
    { 0x38115b6b, SUC_lexicon_tags_17 },
    { 0xa515b6d, SUC_lexicon_tags_15 },
    { 0x2eb15b6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9515b73, SUC_lexicon_tags_34 },
    { 0x4d515b74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d15b77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e715b7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb15b82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b15b85, SUC_lexicon_tags_84 },
    { 0x3c715b85, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d15b89, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4315b8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3715b90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77515b94, SUC_lexicon_tags_23 },
    { 0x4cd15b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcab15b97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce315b9a, SUC_lexicon_tags_27 },
    { 0xbed15b9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc9f15b9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5915ba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f15ba5, SUC_lexicon_tags_2 },
    { 0xfb715ba6, SUC_lexicon_tags_17 },
    { 0x25f15ba7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3515bad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f15bb0, SUC_lexicon_tags_17 },
    { 0xc115bb0, SUC_lexicon_tags_7 },
    { 0xdb115bb2, SUC_lexicon_tags_7 },
    { 0x75715bb3, SUC_lexicon_tags_572 },
    { 0x2df15bb3, SUC_lexicon_tags_15 },
    { 0x7c315bb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0715bba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99115bbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x11f15bbe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d15bc3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8515bcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b115bce, SUC_lexicon_tags_68 },
    { 0x3c715bcf, SUC_lexicon_tags_11 },
    { 0x13d15bd0, SUC_lexicon_tags_11 },
    { 0x6515bd0, SUC_lexicon_tags_7 },
    { 0xffd15bd2, SUC_lexicon_tags_23 },
    { 0xf9115bd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cb15bd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92115bd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3515bd9, SUC_lexicon_tags_179 },
    { 0xf3915bda, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x35915bdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb115be0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xacf15be2, SUC_lexicon_tags_10 },
    { 0x38315be3, SUC_lexicon_tags_49 },
    { 0x9e515be3, SUC_lexicon_tags_31 },
    { 0x3b915be2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b715be9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f315bf0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3715bf4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8715bf7, SUC_lexicon_tags_29 },
    { 0xcd15bf7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60d15bfa, SUC_lexicon_tags_32 },
    { 0x42515bfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12315c01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0d15c03, SUC_lexicon_tags_11 },
    { 0x9b15c03, SUC_lexicon_tags_28 },
    { 0xb6515c05, SUC_lexicon_tags_45 },
    { 0x63b15c06, SUC_lexicon_tags_11 },
    { 0xf0115c07, SUC_lexicon_tags_60 },
    { 0xccf15c08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2315c0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d515c0c, SUC_lexicon_tags_23 },
    { 0x1a115c0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e515c0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f15c13, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55915c17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d15c1b, SUC_lexicon_tags_14 },
    { 0x33915c1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3115c1f, SUC_lexicon_tags_15 },
    { 0x76b15c20, SUC_lexicon_tags_4 },
    { 0x7c115c21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f15c24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x82f15c26, SUC_lexicon_tags_66 },
    { 0xd0f15c27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ef15c29, SUC_lexicon_tags_25 },
    { 0x2c315c29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f15c2f, SUC_lexicon_tags_17 },
    { 0x6a515c30, SUC_lexicon_tags_49 },
    { 0x86915c31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1115c36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f15c39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb315c48, SUC_lexicon_tags_56 },
    { 0x3b715c48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98915c53, SUC_lexicon_tags_9 },
    { 0x1a515c54, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84315c58, SUC_lexicon_tags_1 },
    { 0x2af15c59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac115c5b, SUC_lexicon_tags_15 },
    { 0xb515c5b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x62715c5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d915c61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82315c63, SUC_lexicon_tags_29 },
    { 0x30d15c63, SUC_lexicon_tags_2 },
    { 0x50715c65, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5715c6a, SUC_lexicon_tags_19 },
    { 0xd3715c6b, SUC_lexicon_tags_37 },
    { 0x75d15c6a, SUC_lexicon_tags_11 },
    { 0x99715c6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d515c6f, SUC_lexicon_tags_17 },
    { 0x61915c6f, SUC_lexicon_tags_27 },
    { 0xb7515c71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e315c75, SUC_lexicon_tags_47 },
    { 0xe4f15c75, SUC_lexicon_tags_66 },
    { 0x8c915c77, SUC_lexicon_tags_49 },
    { 0x33b15c78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f15c7d, SUC_lexicon_tags_55 },
    { 0x2f115c7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc715c84, SUC_lexicon_tags_12 },
    { 0x915c85, SUC_lexicon_tags_28 },
    { 0x7c115c86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb15c89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef15c8d, SUC_lexicon_tags_17 },
    { 0xa9715c8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed15c92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x25915c95, SUC_lexicon_tags_11 },
    { 0x23d15c96, SUC_lexicon_tags_31 },
    { 0x7c715c97, SUC_lexicon_tags_43 },
    { 0xcd915c98, SUC_lexicon_tags_14 },
    { 0x45b15c98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4315ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b15ca3, SUC_lexicon_tags_61 },
    { 0xd5115ca4, SUC_lexicon_tags_17 },
    { 0x52515ca5, SUC_lexicon_tags_11 },
    { 0x3e715ca6, SUC_lexicon_tags_11 },
    { 0xe8315ca7, SUC_lexicon_tags_47 },
    { 0xa7f15ca8, SUC_lexicon_tags_37 },
    { 0xb3315ca9, SUC_lexicon_tags_45 },
    { 0xf15ca6, SUC_lexicon_tags_65 },
    { 0x1ad15ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61715cad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c915cb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d315cb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd15cbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78915cc4, SUC_lexicon_tags_37 },
    { 0xa8315cc4, SUC_lexicon_tags_49 },
    { 0xd4115cc6, SUC_lexicon_tags_79 },
    { 0x3a115cc6, SUC_lexicon_tags_37 },
    { 0x4715cc7, SUC_lexicon_tags_26 },
    { 0xe5d15cc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d315ccb, SUC_lexicon_tags_6 },
    { 0x42515ccc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c115cd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f15cd3, SUC_lexicon_tags_15 },
    { 0x23715cd4, SUC_lexicon_tags_2 },
    { 0xc0515cd3, SUC_lexicon_tags_11 },
    { 0x19915cd6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8e915cd8, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61515cdc, SUC_lexicon_tags_64 },
    { 0x61b15cdd, SUC_lexicon_tags_47 },
    { 0x3db15cde, SUC_lexicon_tags_68 },
    { 0x1f15cdf, SUC_lexicon_tags_4 },
    { 0xa6915ce0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f15cea, SUC_lexicon_tags_49 },
    { 0x39515ceb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x59515ced, SUC_lexicon_tags_30 },
    { 0xdaf15cee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf15cf1, SUC_lexicon_tags_15 },
    { 0xf5515cf1, SUC_lexicon_tags_37 },
    { 0xd4f15cf3, SUC_lexicon_tags_11 },
    { 0xe3715cf3, SUC_lexicon_tags_687 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4515cfb, SUC_lexicon_tags_7 },
    { 0x3dd15cfb, SUC_lexicon_tags_6 },
    { 0x27715cfc, SUC_lexicon_tags_15 },
    { 0xf7915cfe, SUC_lexicon_tags_27 },
    { 0x24f15cfc, SUC_lexicon_tags_18 },
    { 0x7cd15cfd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72315d07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0715d0d, SUC_lexicon_tags_15 },
    { 0xff15d0e, SUC_lexicon_tags_61 },
    { 0x88915d0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef15d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8915d16, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5115d19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef15d1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc0b15d20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6915d25, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd1f15d27, SUC_lexicon_tags_46 },
    { 0xb915d27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce715d2a, SUC_lexicon_tags_11 },
    { 0x45b15d2a, SUC_lexicon_tags_25 },
    { 0x65915d2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d15d39, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe3b15d3b, SUC_lexicon_tags_37 },
    { 0x33b15d3c, SUC_lexicon_tags_1 },
    { 0xbf15d3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17915d41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25f15d43, SUC_lexicon_tags_4 },
    { 0x78715d44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd315d47, SUC_lexicon_tags_11 },
    { 0x68715d48, SUC_lexicon_tags_2 },
    { 0x41115d49, SUC_lexicon_tags_7 },
    { 0xb1315d48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65315d50, SUC_lexicon_tags_25 },
    { 0xd7715d50, SUC_lexicon_tags_63 },
    { 0x72715d52, SUC_lexicon_tags_19 },
    { 0xbab15d53, SUC_lexicon_tags_27 },
    { 0xa5715d50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4115d57, SUC_lexicon_tags_31 },
    { 0x73f15d57, SUC_lexicon_tags_151 },
    { 0xfe915d59, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe1b15d5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa4915d5d, SUC_lexicon_tags_38 },
    { 0xf5315d5e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x89315d60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb7f15d62, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe7f15d64, SUC_lexicon_tags_38 },
    { 0x24d15d64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d15d69, SUC_lexicon_tags_26 },
    { 0xdff15d69, SUC_lexicon_tags_47 },
    { 0x61915d6b, SUC_lexicon_tags_11 },
    { 0x62f15d69, SUC_lexicon_tags_38 },
    { 0xe9915d6d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f15d70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6115d74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1ab15d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5515d7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58d15d7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0915d83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b715d87, SUC_lexicon_tags_108 },
    { 0x5a515d88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2315d8c, SUC_lexicon_tags_25 },
    { 0xdf915d8d, SUC_lexicon_tags_2 },
    { 0x1bd15d8e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb15d94, SUC_lexicon_tags_14 },
    { 0x41515d95, SUC_lexicon_tags_10 },
    { 0x9eb15d96, SUC_lexicon_tags_37 },
    { 0xd0d15d95, SUC_lexicon_tags_25 },
    { 0x9af15d98, SUC_lexicon_tags_15 },
    { 0xfed15d99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf715d9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f15da1, SUC_lexicon_tags_1 },
    { 0xef15da1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d15da5, SUC_lexicon_tags_25 },
    { 0x85d15da5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf15da9, SUC_lexicon_tags_355 },
    { 0, NULL },
    { 0x6f915dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb15dae, SUC_lexicon_tags_12 },
    { 0x6315daf, SUC_lexicon_tags_28 },
    { 0x86315db0, SUC_lexicon_tags_19 },
    { 0x5f715db0, SUC_lexicon_tags_7 },
    { 0x8d515db2, SUC_lexicon_tags_7 },
    { 0xa7315db2, SUC_lexicon_tags_17 },
    { 0xd5d15db0, SUC_lexicon_tags_31 },
    { 0x13115dae, SUC_lexicon_tags_39 },
    { 0x6e115db6, SUC_lexicon_tags_14 },
    { 0x6c515db7, SUC_lexicon_tags_60 },
    { 0x14f15db8, SUC_lexicon_tags_10 },
    { 0x78515db7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3915dbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3115dbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa915dc5, SUC_lexicon_tags_4 },
    { 0x78f15dc5, SUC_lexicon_tags_27 },
    { 0xdfb15dc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b15dcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a115dcd, SUC_lexicon_tags_37 },
    { 0xb2515dce, SUC_lexicon_tags_17 },
    { 0x1db15dcf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7b515dd1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b915dd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37115dd6, SUC_lexicon_tags_7 },
    { 0x26515dd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d515dd9, SUC_lexicon_tags_55 },
    { 0xf1315dda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb7515ddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35b15dde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc315de1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x47115de4, SUC_lexicon_tags_49 },
    { 0x26715de4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52315dea, SUC_lexicon_tags_15 },
    { 0x9ad15dea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f15def, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad315df4, SUC_lexicon_tags_91 },
    { 0x38d15df4, SUC_lexicon_tags_17 },
    { 0x3c715df6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a515dfa, SUC_lexicon_tags_25 },
    { 0x67715dfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4b15dfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b15e03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df15e0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad15e0d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b915e11, SUC_lexicon_tags_10 },
    { 0x7ef15e12, SUC_lexicon_tags_38 },
    { 0x63f15e13, SUC_lexicon_tags_29 },
    { 0x58d15e12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x49315e17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f915e19, SUC_lexicon_tags_68 },
    { 0x99515e1a, SUC_lexicon_tags_27 },
    { 0x27515e1b, SUC_lexicon_tags_12 },
    { 0x14715e1b, SUC_lexicon_tags_38 },
    { 0x1b15e1a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x315e20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x87315e22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db15e25, SUC_lexicon_tags_25 },
    { 0x4a715e26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6515e29, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xe4d15e2b, SUC_lexicon_tags_37 },
    { 0x44715e2c, SUC_lexicon_tags_14 },
    { 0xd7b15e2c, SUC_lexicon_tags_26 },
    { 0x83515e2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3cb15e30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3315e32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6115e34, SUC_lexicon_tags_49 },
    { 0x5d15e35, SUC_lexicon_tags_53 },
    { 0xad915e36, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b515e3a, SUC_lexicon_tags_42 },
    { 0xdc515e3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5515e3d, SUC_lexicon_tags_12 },
    { 0xaff15e3e, SUC_lexicon_tags_37 },
    { 0x88915e3f, SUC_lexicon_tags_11 },
    { 0x70915e40, SUC_lexicon_tags_1 },
    { 0xe3715e3f, SUC_lexicon_tags_2 },
    { 0x2bb15e3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c515e44, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x50115e47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37715e4b, SUC_lexicon_tags_37 },
    { 0x1a115e4c, SUC_lexicon_tags_2 },
    { 0xb8d15e4d, SUC_lexicon_tags_65 },
    { 0x48d15e4d, SUC_lexicon_tags_45 },
    { 0x74115e4f, SUC_lexicon_tags_0 },
    { 0x6a515e50, SUC_lexicon_tags_21 },
    { 0xb6d15e4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66f15e53, SUC_lexicon_tags_12 },
    { 0xc7915e54, SUC_lexicon_tags_25 },
    { 0x59515e54, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xcf715e57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44315e5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x29315e5e, SUC_lexicon_tags_8 },
    { 0x4515e5f, SUC_lexicon_tags_10 },
    { 0x29f15e5f, SUC_lexicon_tags_7 },
    { 0xb0b15e61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a115e69, SUC_lexicon_tags_37 },
    { 0xefb15e6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6a715e6c, SUC_lexicon_tags_28 },
    { 0x65115e6c, SUC_lexicon_tags_63 },
    { 0x27515e6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18d15e70, SUC_lexicon_tags_23 },
    { 0xc4315e71, SUC_lexicon_tags_44 },
    { 0x66115e72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d15e76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19515e7f, SUC_lexicon_tags_2 },
    { 0x95b15e80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13b15e82, SUC_lexicon_tags_8 },
    { 0xe6715e83, SUC_lexicon_tags_35 },
    { 0x66315e84, SUC_lexicon_tags_1 },
    { 0x29d15e85, SUC_lexicon_tags_55 },
    { 0x96715e86, SUC_lexicon_tags_47 },
    { 0x87315e87, SUC_lexicon_tags_2 },
    { 0x95915e88, SUC_lexicon_tags_27 },
    { 0x5715e89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a315e8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4b15e8e, SUC_lexicon_tags_8 },
    { 0x51f15e8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b15e92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4ff15e94, SUC_lexicon_tags_11 },
    { 0x2b15e95, SUC_lexicon_tags_34 },
    { 0xd715e94, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92715e9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5915ea0, SUC_lexicon_tags_15 },
    { 0xf4115ea1, SUC_lexicon_tags_7 },
    { 0x1bd15ea1, SUC_lexicon_tags_1 },
    { 0x79315ea1, SUC_lexicon_tags_11 },
    { 0x8b15ea4, SUC_lexicon_tags_12 },
    { 0x7f15ea4, SUC_lexicon_tags_25 },
    { 0xd0715ea6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa7915ea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d15eab, SUC_lexicon_tags_28 },
    { 0x55f15eab, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf4d15eae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6b15eb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f15ebb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad15ebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x98515ec1, SUC_lexicon_tags_17 },
    { 0x40f15ec2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b15ec8, SUC_lexicon_tags_17 },
    { 0xf4f15ec9, SUC_lexicon_tags_1 },
    { 0xa6f15ec9, SUC_lexicon_tags_50 },
    { 0x1bb15ec9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40915ecd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xded15ecf, SUC_lexicon_tags_4 },
    { 0xe8115ed0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdad15ed2, SUC_lexicon_tags_45 },
    { 0x13915ed3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b715ed5, SUC_lexicon_tags_60 },
    { 0x7f715ed6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd15edb, SUC_lexicon_tags_45 },
    { 0xb2315edb, SUC_lexicon_tags_1 },
    { 0xf0915edd, SUC_lexicon_tags_87 },
    { 0x8bf15ede, SUC_lexicon_tags_11 },
    { 0x5b715edc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2a315ee1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfff15ee3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32f15ee5, SUC_lexicon_tags_43 },
    { 0xad515ee6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58515eea, SUC_lexicon_tags_9 },
    { 0xf2115eea, SUC_lexicon_tags_55 },
    { 0x53d15eea, SUC_lexicon_tags_0 },
    { 0xd115eed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96315eef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b15ef9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd15efe, SUC_lexicon_tags_6 },
    { 0x7e315eff, SUC_lexicon_tags_23 },
    { 0xeb915f00, SUC_lexicon_tags_2 },
    { 0x6f15f01, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb515f06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c115f09, SUC_lexicon_tags_31 },
    { 0x5b315f09, SUC_lexicon_tags_49 },
    { 0x35515f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b15f0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6915f13, SUC_lexicon_tags_12 },
    { 0xe8f15f14, SUC_lexicon_tags_25 },
    { 0x92b15f14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x36315f18, SUC_lexicon_tags_37 },
    { 0x53f15f19, SUC_lexicon_tags_19 },
    { 0x96f15f19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68b15f1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb15f22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34b15f24, SUC_lexicon_tags_98 },
    { 0xa8715f24, SUC_lexicon_tags_33 },
    { 0x32b15f25, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x1a115f28, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x91315f2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc315f2d, SUC_lexicon_tags_17 },
    { 0xd3515f2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb15f32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90315f34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x90b15f36, SUC_lexicon_tags_87 },
    { 0xd0315f37, SUC_lexicon_tags_11 },
    { 0x8e515f37, SUC_lexicon_tags_7 },
    { 0x78515f39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb15f3d, SUC_lexicon_tags_0 },
    { 0xc9d15f3d, SUC_lexicon_tags_17 },
    { 0x58915f3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55115f45, SUC_lexicon_tags_55 },
    { 0xe9f15f46, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1715f4b, SUC_lexicon_tags_5 },
    { 0x8b915f4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d315f4e, SUC_lexicon_tags_1 },
    { 0x72915f4f, SUC_lexicon_tags_7 },
    { 0x89515f50, SUC_lexicon_tags_29 },
    { 0xe4715f50, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9b315f53, SUC_lexicon_tags_37 },
    { 0x57915f54, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82315f59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf115f5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9715f5f, SUC_lexicon_tags_91 },
    { 0x46715f60, SUC_lexicon_tags_38 },
    { 0xa3d15f61, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9915f66, SUC_lexicon_tags_9 },
    { 0xff515f66, SUC_lexicon_tags_17 },
    { 0xf6315f66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfa115f6a, SUC_lexicon_tags_15 },
    { 0xf7715f6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x35115f6d, SUC_lexicon_tags_9 },
    { 0xc6115f6e, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xe8f15f70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b15f75, SUC_lexicon_tags_7 },
    { 0x26715f76, SUC_lexicon_tags_11 },
    { 0x91115f77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6515f79, SUC_lexicon_tags_12 },
    { 0xe7915f79, SUC_lexicon_tags_17 },
    { 0x73b15f7b, SUC_lexicon_tags_15 },
    { 0x84915f7a, SUC_lexicon_tags_11 },
    { 0x8f15f7b, SUC_lexicon_tags_66 },
    { 0xc2b15f7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a915f80, SUC_lexicon_tags_12 },
    { 0xa2515f81, SUC_lexicon_tags_43 },
    { 0x56d15f82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1715f89, SUC_lexicon_tags_10 },
    { 0x68715f89, SUC_lexicon_tags_17 },
    { 0x23915f8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8915f91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76315f94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8115f97, SUC_lexicon_tags_43 },
    { 0xfb715f98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2115f9a, SUC_lexicon_tags_28 },
    { 0xca515f9b, SUC_lexicon_tags_26 },
    { 0xf8d15f9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbe715f9e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78715fa3, SUC_lexicon_tags_25 },
    { 0x95f15fa4, SUC_lexicon_tags_33 },
    { 0x5f115fa3, SUC_lexicon_tags_28 },
    { 0xf7b15fa3, SUC_lexicon_tags_28 },
    { 0xd7315fa7, SUC_lexicon_tags_17 },
    { 0x78915fa5, SUC_lexicon_tags_28 },
    { 0x2ed15fa9, SUC_lexicon_tags_14 },
    { 0x20515fa5, SUC_lexicon_tags_47 },
    { 0xc915fa4, SUC_lexicon_tags_31 },
    { 0xa1915fac, SUC_lexicon_tags_3 },
    { 0xf3515fac, SUC_lexicon_tags_7 },
    { 0xcb15fae, SUC_lexicon_tags_27 },
    { 0x43b15faa, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98515fb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa7f15fb7, SUC_lexicon_tags_17 },
    { 0xd1715fb8, SUC_lexicon_tags_53 },
    { 0x21f15fb8, SUC_lexicon_tags_0 },
    { 0x57115fba, SUC_lexicon_tags_29 },
    { 0xbe715fbb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9f515fbd, SUC_lexicon_tags_25 },
    { 0x63f15fbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae915fc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf15fcc, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x37315fce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b15fd1, SUC_lexicon_tags_1 },
    { 0xe6115fd1, SUC_lexicon_tags_1670 },
    { 0xa3715fd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23715fd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf7315fda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb15fe2, SUC_lexicon_tags_85 },
    { 0xe9915fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26b15fe5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f15ff1, SUC_lexicon_tags_11 },
    { 0x8a915ff2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7315ff5, SUC_lexicon_tags_2 },
    { 0x53d15ff6, SUC_lexicon_tags_107 },
    { 0x2d515ff7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7915ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb515ffd, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5b315fff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf316002, SUC_lexicon_tags_2 },
    { 0xec116003, SUC_lexicon_tags_63 },
    { 0xf4b16003, SUC_lexicon_tags_43 },
    { 0xabf16003, SUC_lexicon_tags_48 },
    { 0x1c716004, SUC_lexicon_tags_7 },
    { 0x58916007, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3711600a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7611600c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d516013, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18116016, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd16019, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb211601c, SUC_lexicon_tags_47 },
    { 0xfed1601d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79f1601f, SUC_lexicon_tags_33 },
    { 0x4d1601f, SUC_lexicon_tags_6 },
    { 0xe371601f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x71b16023, SUC_lexicon_tags_25 },
    { 0xeef16024, SUC_lexicon_tags_11 },
    { 0x78516024, SUC_lexicon_tags_39 },
    { 0xb7316026, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe571602b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2291602f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe116036, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f516038, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a11603b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d16042, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3316047, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1491604b, SUC_lexicon_tags_63 },
    { 0x1a31604b, SUC_lexicon_tags_519 },
    { 0x3811604b, SUC_lexicon_tags_25 },
    { 0x86b1604e, SUC_lexicon_tags_7 },
    { 0x8c71604f, SUC_lexicon_tags_1 },
    { 0x7b1604e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fd16052, SUC_lexicon_tags_17 },
    { 0x5d716053, SUC_lexicon_tags_7 },
    { 0xe9516054, SUC_lexicon_tags_27 },
    { 0xf8916055, SUC_lexicon_tags_7 },
    { 0xed116056, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b1605d, SUC_lexicon_tags_37 },
    { 0xddd1605e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d16061, SUC_lexicon_tags_11 },
    { 0x60f16061, SUC_lexicon_tags_41 },
    { 0xf6b16061, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12116065, SUC_lexicon_tags_7 },
    { 0xff716066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89516068, SUC_lexicon_tags_53 },
    { 0x7db16068, SUC_lexicon_tags_23 },
    { 0xa6f16068, SUC_lexicon_tags_8 },
    { 0xec51606a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b716075, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbad16077, SUC_lexicon_tags_68 },
    { 0xbd716078, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6791607b, SUC_lexicon_tags_17 },
    { 0x2dd1607b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2931607f, SUC_lexicon_tags_29 },
    { 0x9751607f, SUC_lexicon_tags_37 },
    { 0xf2716081, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d16086, SUC_lexicon_tags_4 },
    { 0x5f716087, SUC_lexicon_tags_37 },
    { 0x88b16087, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e51608c, SUC_lexicon_tags_0 },
    { 0xf0d1608c, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb16091, SUC_lexicon_tags_25 },
    { 0x30b16091, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d16094, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c916097, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e91609a, SUC_lexicon_tags_12 },
    { 0xe211609b, SUC_lexicon_tags_19 },
    { 0xad31609c, SUC_lexicon_tags_9 },
    { 0x5231609d, SUC_lexicon_tags_11 },
    { 0x6571609c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1b160a0, SUC_lexicon_tags_12 },
    { 0xb05160a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51160a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9160b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d160b4, SUC_lexicon_tags_9 },
    { 0x12b160b4, SUC_lexicon_tags_42 },
    { 0x6e5160b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b160b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe1160bb, SUC_lexicon_tags_12 },
    { 0x49b160bb, SUC_lexicon_tags_37 },
    { 0x177160bd, SUC_lexicon_tags_11 },
    { 0x439160bd, SUC_lexicon_tags_37 },
    { 0x5e7160bf, SUC_lexicon_tags_37 },
    { 0xaf3160bf, SUC_lexicon_tags_47 },
    { 0x49f160be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdad160c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x823160c6, SUC_lexicon_tags_25 },
    { 0xbd9160c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb160cc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81160d0, SUC_lexicon_tags_68 },
    { 0x9dd160d1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x527160d3, SUC_lexicon_tags_17 },
    { 0x7a7160d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x505160d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f9160d9, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0x9a1160db, SUC_lexicon_tags_12 },
    { 0x2f7160dc, SUC_lexicon_tags_49 },
    { 0x141160dc, SUC_lexicon_tags_28 },
    { 0x765160de, SUC_lexicon_tags_25 },
    { 0xc37160df, SUC_lexicon_tags_7 },
    { 0x945160e0, SUC_lexicon_tags_47 },
    { 0xbdb160e1, SUC_lexicon_tags_37 },
    { 0x953160dd, SUC_lexicon_tags_15 },
    { 0x7a3160e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89160e6, SUC_lexicon_tags_45 },
    { 0x70d160e7, SUC_lexicon_tags_45 },
    { 0xc69160e6, SUC_lexicon_tags_33 },
    { 0xf3b160e9, SUC_lexicon_tags_2 },
    { 0xccf160ea, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5160ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb160f2, SUC_lexicon_tags_84 },
    { 0x14b160f2, SUC_lexicon_tags_1 },
    { 0x33f160f3, SUC_lexicon_tags_10 },
    { 0xb23160f3, SUC_lexicon_tags_9 },
    { 0xcef160f3, SUC_lexicon_tags_9 },
    { 0xe39160f7, SUC_lexicon_tags_47 },
    { 0xe15160f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09160fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42316102, SUC_lexicon_tags_63 },
    { 0x41716103, SUC_lexicon_tags_37 },
    { 0x57916104, SUC_lexicon_tags_1 },
    { 0x50716105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f16108, SUC_lexicon_tags_12 },
    { 0x7ad16108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0d1610b, SUC_lexicon_tags_13 },
    { 0xf1610c, SUC_lexicon_tags_47 },
    { 0x2db1610c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b16111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f16114, SUC_lexicon_tags_37 },
    { 0x1c316114, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10b16117, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3fb16119, SUC_lexicon_tags_27 },
    { 0x67d1611a, SUC_lexicon_tags_14 },
    { 0x4311611b, SUC_lexicon_tags_37 },
    { 0xb151611c, SUC_lexicon_tags_25 },
    { 0xeb71611c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34716125, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad16128, SUC_lexicon_tags_23 },
    { 0x3f316129, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5511612b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4bd1612d, SUC_lexicon_tags_21 },
    { 0xae91612e, SUC_lexicon_tags_16 },
    { 0x9291612f, SUC_lexicon_tags_25 },
    { 0x45d1612d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xab316132, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f16136, SUC_lexicon_tags_4 },
    { 0x8ff16137, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b16139, SUC_lexicon_tags_26 },
    { 0x5ed1613a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f91613e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5db16140, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e116143, SUC_lexicon_tags_25 },
    { 0x5d716144, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda516146, SUC_lexicon_tags_11 },
    { 0x116146, SUC_lexicon_tags_60 },
    { 0xa9516147, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb91614c, SUC_lexicon_tags_37 },
    { 0x7b71614c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2c11614f, SUC_lexicon_tags_11 },
    { 0xf4316150, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd16156, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf1615a, SUC_lexicon_tags_2 },
    { 0x531615b, SUC_lexicon_tags_37 },
    { 0x1af1615b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a51615f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f116162, SUC_lexicon_tags_12 },
    { 0xb8f16162, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd291616b, SUC_lexicon_tags_25 },
    { 0xbd51616c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1591616e, SUC_lexicon_tags_17 },
    { 0xb431616f, SUC_lexicon_tags_45 },
    { 0x32f1616f, SUC_lexicon_tags_19 },
    { 0xd991616e, SUC_lexicon_tags_82 },
    { 0x15d16172, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16516174, SUC_lexicon_tags_6 },
    { 0xb1d16175, SUC_lexicon_tags_61 },
    { 0xf7116176, SUC_lexicon_tags_37 },
    { 0x6c316177, SUC_lexicon_tags_37 },
    { 0xf8116178, SUC_lexicon_tags_17 },
    { 0x5bb16179, SUC_lexicon_tags_38 },
    { 0x45b1617a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e91617d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1516181, SUC_lexicon_tags_25 },
    { 0x24b16182, SUC_lexicon_tags_37 },
    { 0x26b16183, SUC_lexicon_tags_11 },
    { 0xe7316181, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28916189, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f1618d, SUC_lexicon_tags_12 },
    { 0x851618e, SUC_lexicon_tags_28 },
    { 0xbe91618f, SUC_lexicon_tags_545 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d16195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2116198, SUC_lexicon_tags_14 },
    { 0x50f16199, SUC_lexicon_tags_17 },
    { 0x7e316199, SUC_lexicon_tags_63 },
    { 0x1811619b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d1619e, SUC_lexicon_tags_17 },
    { 0xa771619f, SUC_lexicon_tags_37 },
    { 0x2751619e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dd161a2, SUC_lexicon_tags_354 },
    { 0xfb161a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fd161a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a9161a7, SUC_lexicon_tags_28 },
    { 0xc1161a8, SUC_lexicon_tags_34 },
    { 0x62d161a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb161b2, SUC_lexicon_tags_60 },
    { 0x55d161b2, SUC_lexicon_tags_2 },
    { 0x9a1161b3, SUC_lexicon_tags_25 },
    { 0x475161b5, SUC_lexicon_tags_207 },
    { 0x42d161b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc49161b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13161c1, SUC_lexicon_tags_19 },
    { 0xb7b161c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x595161c4, SUC_lexicon_tags_14 },
    { 0xe4b161c4, SUC_lexicon_tags_17 },
    { 0x55b161c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b161cb, SUC_lexicon_tags_38 },
    { 0x7b1161cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xceb161ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b161d2, SUC_lexicon_tags_11 },
    { 0x645161d3, SUC_lexicon_tags_27 },
    { 0xf77161d4, SUC_lexicon_tags_23 },
    { 0xfe9161d4, SUC_lexicon_tags_82 },
    { 0xa9d161d4, SUC_lexicon_tags_11 },
    { 0xb3d161d7, SUC_lexicon_tags_14 },
    { 0x273161d6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1161dc, SUC_lexicon_tags_79 },
    { 0x84d161dc, SUC_lexicon_tags_10 },
    { 0x9b7161de, SUC_lexicon_tags_25 },
    { 0x16f161df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f1161e1, SUC_lexicon_tags_12 },
    { 0x3eb161e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1bd161e4, SUC_lexicon_tags_37 },
    { 0x451161e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3161e8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d161ed, SUC_lexicon_tags_27 },
    { 0xd73161ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe03161f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381161f4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b161f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3161fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x307161ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76516201, SUC_lexicon_tags_23 },
    { 0x72316202, SUC_lexicon_tags_9 },
    { 0xdc316203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb16206, SUC_lexicon_tags_31 },
    { 0x9d116207, SUC_lexicon_tags_816 },
    { 0xf8b16208, SUC_lexicon_tags_37 },
    { 0xcf716206, SUC_lexicon_tags_27 },
    { 0x4c71620a, SUC_lexicon_tags_2 },
    { 0x44d16209, SUC_lexicon_tags_79 },
    { 0x931620a, SUC_lexicon_tags_7 },
    { 0xc91620d, SUC_lexicon_tags_28 },
    { 0xeb11620e, SUC_lexicon_tags_28 },
    { 0xe471620f, SUC_lexicon_tags_9 },
    { 0x6dd16210, SUC_lexicon_tags_46 },
    { 0xb0916210, SUC_lexicon_tags_55 },
    { 0x73d16212, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d16215, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b16218, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed16221, SUC_lexicon_tags_11 },
    { 0x49716222, SUC_lexicon_tags_89 },
    { 0xf3d16223, SUC_lexicon_tags_12 },
    { 0xe9716224, SUC_lexicon_tags_34 },
    { 0x37d16223, SUC_lexicon_tags_17 },
    { 0xcc716222, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6cd16228, SUC_lexicon_tags_7 },
    { 0xa8316229, SUC_lexicon_tags_37 },
    { 0x6c51622a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb371622c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf431622e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59916234, SUC_lexicon_tags_1 },
    { 0x77d16234, SUC_lexicon_tags_29 },
    { 0x4a116236, SUC_lexicon_tags_45 },
    { 0xd1d16237, SUC_lexicon_tags_11 },
    { 0xa8b16234, SUC_lexicon_tags_37 },
    { 0x11916239, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd31623c, SUC_lexicon_tags_37 },
    { 0xd531623c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc516240, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2116243, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d116245, SUC_lexicon_tags_47 },
    { 0x9d516245, SUC_lexicon_tags_27 },
    { 0x27116247, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4611624b, SUC_lexicon_tags_85 },
    { 0x24d1624c, SUC_lexicon_tags_15 },
    { 0xa2b1624c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xecb1624f, SUC_lexicon_tags_27 },
    { 0x2011624f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38516252, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c316255, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0xd1d16257, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d91625b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d71625d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67116261, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5fb16263, SUC_lexicon_tags_37 },
    { 0x86716263, SUC_lexicon_tags_43 },
    { 0x79316265, SUC_lexicon_tags_9 },
    { 0x16516265, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6131626c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed16273, SUC_lexicon_tags_47 },
    { 0xa9516274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2b16276, SUC_lexicon_tags_17 },
    { 0x6f516277, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa451627b, SUC_lexicon_tags_17 },
    { 0x4db1627c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75716280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbad16282, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2716285, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b16288, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5e51628a, SUC_lexicon_tags_39 },
    { 0x78d1628a, SUC_lexicon_tags_7 },
    { 0x4f91628c, SUC_lexicon_tags_11 },
    { 0x80d1628a, SUC_lexicon_tags_47 },
    { 0x9711628e, SUC_lexicon_tags_2 },
    { 0x76d1628f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4d16291, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x15716293, SUC_lexicon_tags_47 },
    { 0xc2116294, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b16297, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33516299, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df1629f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983162a3, SUC_lexicon_tags_15 },
    { 0x7c9162a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7162aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5162b2, SUC_lexicon_tags_68 },
    { 0x3d9162b2, SUC_lexicon_tags_140 },
    { 0xe4b162b3, SUC_lexicon_tags_27 },
    { 0x587162b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5162b9, SUC_lexicon_tags_43 },
    { 0x709162ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11162bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1162c2, SUC_lexicon_tags_534 },
    { 0x49f162c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d162c6, SUC_lexicon_tags_79 },
    { 0x7b162c7, SUC_lexicon_tags_1 },
    { 0xc11162c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a7162ca, SUC_lexicon_tags_1 },
    { 0x3a5162ca, SUC_lexicon_tags_2 },
    { 0x181162ca, SUC_lexicon_tags_11 },
    { 0x18b162cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797162d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x27162d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb162d7, SUC_lexicon_tags_49 },
    { 0xef5162d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57162e0, SUC_lexicon_tags_2 },
    { 0x69162e0, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51162ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d162f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b162f5, SUC_lexicon_tags_35 },
    { 0xc2d162f6, SUC_lexicon_tags_629 },
    { 0x431162f6, SUC_lexicon_tags_25 },
    { 0x53d162f6, SUC_lexicon_tags_11 },
    { 0x68d162f9, SUC_lexicon_tags_33 },
    { 0x81162f9, SUC_lexicon_tags_0 },
    { 0xf1d162fb, SUC_lexicon_tags_2 },
    { 0x5d5162fc, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xe07162fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b16301, SUC_lexicon_tags_41 },
    { 0x88d16301, SUC_lexicon_tags_39 },
    { 0x26116302, SUC_lexicon_tags_37 },
    { 0x90316301, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe9f16306, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xde316308, SUC_lexicon_tags_10 },
    { 0x54f16309, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f71630d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b16310, SUC_lexicon_tags_14 },
    { 0x52b16310, SUC_lexicon_tags_15 },
    { 0xe716312, SUC_lexicon_tags_27 },
    { 0xf0916313, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef16317, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb51631b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb91631d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9591631f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b116323, SUC_lexicon_tags_45 },
    { 0x9db16324, SUC_lexicon_tags_1 },
    { 0xb7316324, SUC_lexicon_tags_60 },
    { 0x69f16325, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x49916329, SUC_lexicon_tags_15 },
    { 0xbb51632a, SUC_lexicon_tags_37 },
    { 0x2f51632a, SUC_lexicon_tags_7 },
    { 0xc3516329, SUC_lexicon_tags_9 },
    { 0xb071632a, SUC_lexicon_tags_15 },
    { 0x8571632c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd16337, SUC_lexicon_tags_63 },
    { 0x33d16337, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4491633a, SUC_lexicon_tags_1 },
    { 0xd6f1633b, SUC_lexicon_tags_27 },
    { 0x45d1633a, SUC_lexicon_tags_83 },
    { 0x5431633d, SUC_lexicon_tags_0 },
    { 0xcc91633d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb16346, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdcb16348, SUC_lexicon_tags_7 },
    { 0xb1116349, SUC_lexicon_tags_7 },
    { 0x4d31634a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f16353, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4cf16355, SUC_lexicon_tags_1 },
    { 0x45b16356, SUC_lexicon_tags_10 },
    { 0x9b316357, SUC_lexicon_tags_37 },
    { 0x68b16358, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2491635e, SUC_lexicon_tags_39 },
    { 0x23d1635e, SUC_lexicon_tags_60 },
    { 0xf0516360, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58916364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19316368, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad51636c, SUC_lexicon_tags_37 },
    { 0x83d1636d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bf1636f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb16372, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf16374, SUC_lexicon_tags_31 },
    { 0x68d16375, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d16379, SUC_lexicon_tags_23 },
    { 0x26b1637a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6d1637c, SUC_lexicon_tags_30 },
    { 0x5791637c, SUC_lexicon_tags_11 },
    { 0xca31637c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89516380, SUC_lexicon_tags_1 },
    { 0xd2d16381, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70316383, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6316388, SUC_lexicon_tags_15 },
    { 0xb9916389, SUC_lexicon_tags_17 },
    { 0xaaf1638a, SUC_lexicon_tags_17 },
    { 0xf831638b, SUC_lexicon_tags_92 },
    { 0xfcb1638c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f16390, SUC_lexicon_tags_63 },
    { 0x38516391, SUC_lexicon_tags_28 },
    { 0x20316391, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa316398, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b1639b, SUC_lexicon_tags_2 },
    { 0x1411639c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9651639e, SUC_lexicon_tags_47 },
    { 0x8d91639f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1163a2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3163a7, SUC_lexicon_tags_0 },
    { 0x91f163a8, SUC_lexicon_tags_25 },
    { 0x97d163a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b163ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57b163ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b9163b0, SUC_lexicon_tags_7 },
    { 0x251163b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33163b6, SUC_lexicon_tags_12 },
    { 0xa49163b6, SUC_lexicon_tags_68 },
    { 0xfcb163b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xba1163ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed163bd, SUC_lexicon_tags_17 },
    { 0x55f163be, SUC_lexicon_tags_29 },
    { 0xdcb163be, SUC_lexicon_tags_77 },
    { 0x3db163c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fd163c2, SUC_lexicon_tags_2 },
    { 0x2f5163c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f163c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d163c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89d163cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27163d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ef163d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419163d7, SUC_lexicon_tags_54 },
    { 0x79b163d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5163dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x539163de, SUC_lexicon_tags_25 },
    { 0x85163df, SUC_lexicon_tags_2 },
    { 0x6b163e0, SUC_lexicon_tags_19 },
    { 0x44b163e1, SUC_lexicon_tags_11 },
    { 0xb85163e1, SUC_lexicon_tags_37 },
    { 0xd5d163e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b163e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc9163e8, SUC_lexicon_tags_2 },
    { 0x607163e9, SUC_lexicon_tags_11 },
    { 0x319163ea, SUC_lexicon_tags_56 },
    { 0x735163eb, SUC_lexicon_tags_37 },
    { 0xc6b163ea, SUC_lexicon_tags_23 },
    { 0xb03163ed, SUC_lexicon_tags_11 },
    { 0xca7163ee, SUC_lexicon_tags_108 },
    { 0xe13163ea, SUC_lexicon_tags_7 },
    { 0x495163f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f163f3, SUC_lexicon_tags_6 },
    { 0x13b163f4, SUC_lexicon_tags_15 },
    { 0x89f163f4, SUC_lexicon_tags_25 },
    { 0x67d163f4, SUC_lexicon_tags_0 },
    { 0xd63163f7, SUC_lexicon_tags_15 },
    { 0xad1163f7, SUC_lexicon_tags_12 },
    { 0x5f1163f6, SUC_lexicon_tags_25 },
    { 0x9b3163fa, SUC_lexicon_tags_14 },
    { 0xb6b163fb, SUC_lexicon_tags_11 },
    { 0x143163fa, SUC_lexicon_tags_56 },
    { 0xf4f163fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x343163ff, SUC_lexicon_tags_11 },
    { 0xfeb16400, SUC_lexicon_tags_37 },
    { 0x25b16400, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f516409, SUC_lexicon_tags_17 },
    { 0x2271640a, SUC_lexicon_tags_7 },
    { 0x151640b, SUC_lexicon_tags_43 },
    { 0xd3716409, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b31640e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5716414, SUC_lexicon_tags_25 },
    { 0xe8316415, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x41716418, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d51641c, SUC_lexicon_tags_17 },
    { 0x5571641d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95916422, SUC_lexicon_tags_1637 },
    { 0, NULL },
    { 0xf5116424, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x29f16426, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd531642a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d1642f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd2916431, SUC_lexicon_tags_31 },
    { 0x13116431, SUC_lexicon_tags_62 },
    { 0xae316433, SUC_lexicon_tags_23 },
    { 0xc4516434, SUC_lexicon_tags_2 },
    { 0xa5716435, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d16438, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1643c, SUC_lexicon_tags_6 },
    { 0xae71643c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b916442, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c716444, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1316448, SUC_lexicon_tags_15 },
    { 0x86116449, SUC_lexicon_tags_23 },
    { 0xbb31644a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x88d1644c, SUC_lexicon_tags_31 },
    { 0x50b1644d, SUC_lexicon_tags_10 },
    { 0x3ab1644c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xceb16450, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69916457, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7771645b, SUC_lexicon_tags_7 },
    { 0x6811645c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d316461, SUC_lexicon_tags_2 },
    { 0xc1516461, SUC_lexicon_tags_1 },
    { 0xcd916463, SUC_lexicon_tags_17 },
    { 0x57f16462, SUC_lexicon_tags_37 },
    { 0x83716464, SUC_lexicon_tags_12 },
    { 0x10516466, SUC_lexicon_tags_47 },
    { 0x53116467, SUC_lexicon_tags_32 },
    { 0x1cf16466, SUC_lexicon_tags_28 },
    { 0x3c716469, SUC_lexicon_tags_10 },
    { 0x7ef16469, SUC_lexicon_tags_92 },
    { 0x5f16469, SUC_lexicon_tags_47 },
    { 0x3291646c, SUC_lexicon_tags_25 },
    { 0xa271646b, SUC_lexicon_tags_7 },
    { 0xae716461, SUC_lexicon_tags_15 },
    { 0x26716469, SUC_lexicon_tags_37 },
    { 0xef16469, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4516473, SUC_lexicon_tags_12 },
    { 0x75716474, SUC_lexicon_tags_37 },
    { 0x7f516475, SUC_lexicon_tags_59 },
    { 0xbf116475, SUC_lexicon_tags_60 },
    { 0xa8d16477, SUC_lexicon_tags_27 },
    { 0x1f116475, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d31647b, SUC_lexicon_tags_1 },
    { 0xf311647c, SUC_lexicon_tags_12 },
    { 0x1c71647d, SUC_lexicon_tags_37 },
    { 0xaff1647d, SUC_lexicon_tags_11 },
    { 0xc971647b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c316483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b916485, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b16489, SUC_lexicon_tags_7 },
    { 0xfbb1648a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b31648c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5231648e, SUC_lexicon_tags_49 },
    { 0x1c71648e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4516492, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b1649b, SUC_lexicon_tags_7 },
    { 0x98d1649c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf871649f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x539164a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9164a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d164a9, SUC_lexicon_tags_11 },
    { 0xf65164aa, SUC_lexicon_tags_230 },
    { 0xa29164aa, SUC_lexicon_tags_64 },
    { 0xebd164a9, SUC_lexicon_tags_11 },
    { 0xd77164aa, SUC_lexicon_tags_37 },
    { 0xf05164ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f164b2, SUC_lexicon_tags_15 },
    { 0x979164b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f164b6, SUC_lexicon_tags_106 },
    { 0xdf9164b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd164ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9164be, SUC_lexicon_tags_17 },
    { 0x1b164bf, SUC_lexicon_tags_27 },
    { 0x2b1164c0, SUC_lexicon_tags_16 },
    { 0x531164c0, SUC_lexicon_tags_12 },
    { 0x31164c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715164c6, SUC_lexicon_tags_0 },
    { 0x69d164c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71d164c9, SUC_lexicon_tags_40 },
    { 0x83f164ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7164cd, SUC_lexicon_tags_2 },
    { 0x9d7164ce, SUC_lexicon_tags_29 },
    { 0x5eb164cd, SUC_lexicon_tags_66 },
    { 0x731164cf, SUC_lexicon_tags_25 },
    { 0xd7b164d0, SUC_lexicon_tags_15 },
    { 0xa79164d1, SUC_lexicon_tags_25 },
    { 0xe8d164d2, SUC_lexicon_tags_14 },
    { 0x10f164d2, SUC_lexicon_tags_7 },
    { 0x389164d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf164d8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375164df, SUC_lexicon_tags_143 },
    { 0xd05164e0, SUC_lexicon_tags_25 },
    { 0xf83164e1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9164e4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x133164e6, SUC_lexicon_tags_12 },
    { 0x613164e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed5164e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1e5164eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f164ef, SUC_lexicon_tags_11 },
    { 0xfb5164f0, SUC_lexicon_tags_8 },
    { 0xf51164f1, SUC_lexicon_tags_80 },
    { 0xb23164f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d164fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc316500, SUC_lexicon_tags_39 },
    { 0x72f16500, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d716505, SUC_lexicon_tags_140 },
    { 0x1716505, SUC_lexicon_tags_0 },
    { 0x2a116507, SUC_lexicon_tags_11 },
    { 0x40516507, SUC_lexicon_tags_63 },
    { 0x33716507, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x20d1650b, SUC_lexicon_tags_13 },
    { 0x2b71650c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45516511, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d16515, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a916517, SUC_lexicon_tags_28 },
    { 0x6a116518, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc6d1651a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x80d1651c, SUC_lexicon_tags_15 },
    { 0xaa31651c, SUC_lexicon_tags_21 },
    { 0xa631651e, SUC_lexicon_tags_11 },
    { 0xe391651f, SUC_lexicon_tags_17 },
    { 0x1b1651f, SUC_lexicon_tags_17 },
    { 0x18916521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5916525, SUC_lexicon_tags_49 },
    { 0x14d16526, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7151652a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9291652c, SUC_lexicon_tags_0 },
    { 0x9d51652d, SUC_lexicon_tags_68 },
    { 0x82b1652e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2316531, SUC_lexicon_tags_7 },
    { 0xcc916532, SUC_lexicon_tags_7 },
    { 0x6a316533, SUC_lexicon_tags_11 },
    { 0x9716534, SUC_lexicon_tags_60 },
    { 0x2fd16534, SUC_lexicon_tags_17 },
    { 0x16b16534, SUC_lexicon_tags_39 },
    { 0xc5516534, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b71653a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3151653d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3d91653f, SUC_lexicon_tags_66 },
    { 0xc0d16540, SUC_lexicon_tags_48 },
    { 0x16716541, SUC_lexicon_tags_15 },
    { 0xd2d16542, SUC_lexicon_tags_53 },
    { 0xaf916540, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16916547, SUC_lexicon_tags_0 },
    { 0xd1516547, SUC_lexicon_tags_7 },
    { 0x9c316549, SUC_lexicon_tags_15 },
    { 0xd491654a, SUC_lexicon_tags_37 },
    { 0xc0d1654a, SUC_lexicon_tags_43 },
    { 0x3c11654b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd1654f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1716552, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b16556, SUC_lexicon_tags_66 },
    { 0x28916556, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x84d16559, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf111655f, SUC_lexicon_tags_63 },
    { 0x6df1655f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38116565, SUC_lexicon_tags_0 },
    { 0xb1916565, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x32116569, SUC_lexicon_tags_23 },
    { 0x25d16569, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa051656c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d31656f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa0516571, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43516576, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a91657d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27d1657f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63516581, SUC_lexicon_tags_14 },
    { 0xbbb16581, SUC_lexicon_tags_37 },
    { 0x10116583, SUC_lexicon_tags_10 },
    { 0x83f16584, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59916588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf1658c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9151658f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56b16591, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82316596, SUC_lexicon_tags_12 },
    { 0x6716597, SUC_lexicon_tags_47 },
    { 0xa3f16598, SUC_lexicon_tags_27 },
    { 0xec116599, SUC_lexicon_tags_25 },
    { 0xdb11659a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f31659c, SUC_lexicon_tags_1 },
    { 0x1a91659d, SUC_lexicon_tags_8 },
    { 0xfcb1659e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x173165a1, SUC_lexicon_tags_12 },
    { 0x2ef165a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d165a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49165aa, SUC_lexicon_tags_47 },
    { 0x563165aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393165b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d165b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d165ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x847165bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f165c3, SUC_lexicon_tags_12 },
    { 0xfe3165c4, SUC_lexicon_tags_22 },
    { 0x969165c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab165cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b165d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29165d5, SUC_lexicon_tags_63 },
    { 0xb77165d6, SUC_lexicon_tags_32 },
    { 0x8b5165d7, SUC_lexicon_tags_27 },
    { 0x271165d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8c5165da, SUC_lexicon_tags_37 },
    { 0x60b165db, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99165df, SUC_lexicon_tags_43 },
    { 0xc9165e0, SUC_lexicon_tags_6 },
    { 0xe87165e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3165e4, SUC_lexicon_tags_37 },
    { 0x5df165e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1165e9, SUC_lexicon_tags_25 },
    { 0x705165e9, SUC_lexicon_tags_7 },
    { 0x57f165eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x139165ee, SUC_lexicon_tags_23 },
    { 0xfb9165ef, SUC_lexicon_tags_0 },
    { 0xf8f165f0, SUC_lexicon_tags_13 },
    { 0x195165ee, SUC_lexicon_tags_27 },
    { 0x865165ee, SUC_lexicon_tags_1 },
    { 0xd55165f0, SUC_lexicon_tags_49 },
    { 0x19d165f4, SUC_lexicon_tags_31 },
    { 0xe87165f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1165f9, SUC_lexicon_tags_33 },
    { 0x725165fa, SUC_lexicon_tags_37 },
    { 0x907165fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41b165fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4eb165ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82f16601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb16607, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7371660f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3716614, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b716618, SUC_lexicon_tags_45 },
    { 0xe9116618, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f1661d, SUC_lexicon_tags_38 },
    { 0xf431661d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d16621, SUC_lexicon_tags_37 },
    { 0x40b16621, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd716626, SUC_lexicon_tags_38 },
    { 0xd316627, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc2b16629, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc951662b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7551662d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20916633, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb1663b, SUC_lexicon_tags_37 },
    { 0x6ef1663b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf291663f, SUC_lexicon_tags_1 },
    { 0x36116640, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x316643, SUC_lexicon_tags_1 },
    { 0xc6716643, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5516647, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9111664b, SUC_lexicon_tags_69 },
    { 0x9c71664b, SUC_lexicon_tags_11 },
    { 0x371664d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe0f1664f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0516653, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f16656, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b16659, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6391665e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6b16660, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6716662, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43716667, SUC_lexicon_tags_63 },
    { 0xc3b16668, SUC_lexicon_tags_42 },
    { 0x3c116669, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb191666c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf91666e, SUC_lexicon_tags_60 },
    { 0xf191666e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5516671, SUC_lexicon_tags_38 },
    { 0xdb716671, SUC_lexicon_tags_37 },
    { 0x1cb16673, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa911667b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d1667e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d916683, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b16695, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f16699, SUC_lexicon_tags_15 },
    { 0xa471669a, SUC_lexicon_tags_44 },
    { 0x8051669b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x81f1669d, SUC_lexicon_tags_33 },
    { 0x9db1669e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445166a2, SUC_lexicon_tags_25 },
    { 0xf25166a3, SUC_lexicon_tags_7 },
    { 0xb67166a4, SUC_lexicon_tags_25 },
    { 0x2d9166a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff166a9, SUC_lexicon_tags_15 },
    { 0x897166aa, SUC_lexicon_tags_57 },
    { 0x807166aa, SUC_lexicon_tags_47 },
    { 0xa75166aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfaf166ae, SUC_lexicon_tags_14 },
    { 0x895166af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d166b2, SUC_lexicon_tags_11 },
    { 0xf3b166b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803166bd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x735166c0, SUC_lexicon_tags_11 },
    { 0xa7b166c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41166c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b166ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69166d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d166d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3166dc, SUC_lexicon_tags_55 },
    { 0xaad166dd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b166e0, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915166e4, SUC_lexicon_tags_49 },
    { 0xc9f166e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543166eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x81f166ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b166f1, SUC_lexicon_tags_2 },
    { 0xa39166f1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05166f7, SUC_lexicon_tags_9 },
    { 0x22d166f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf166fb, SUC_lexicon_tags_19 },
    { 0x6b7166fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3166ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad16703, SUC_lexicon_tags_33 },
    { 0x53316704, SUC_lexicon_tags_7 },
    { 0x76716705, SUC_lexicon_tags_14 },
    { 0x916705, SUC_lexicon_tags_47 },
    { 0x88b16706, SUC_lexicon_tags_63 },
    { 0xe3f16708, SUC_lexicon_tags_39 },
    { 0x39716706, SUC_lexicon_tags_15 },
    { 0xb8b1670a, SUC_lexicon_tags_23 },
    { 0x28716709, SUC_lexicon_tags_12 },
    { 0x53f1670c, SUC_lexicon_tags_47 },
    { 0x50116705, SUC_lexicon_tags_1 },
    { 0xbdf16708, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85316718, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe391671b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff31671e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbf716720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd16723, SUC_lexicon_tags_112 },
    { 0xbc916724, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b16727, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf9f16729, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b1672e, SUC_lexicon_tags_30 },
    { 0x1e31672f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1916733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55716735, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb1673b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b1673e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19316741, SUC_lexicon_tags_11 },
    { 0x82b16742, SUC_lexicon_tags_17 },
    { 0x79716742, SUC_lexicon_tags_2 },
    { 0x4bd16744, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda516748, SUC_lexicon_tags_43 },
    { 0xd5716749, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f31674d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x71916750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf16755, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f1675e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e316761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdf16763, SUC_lexicon_tags_11 },
    { 0xb8316764, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d316766, SUC_lexicon_tags_7 },
    { 0x4916766, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x771676a, SUC_lexicon_tags_37 },
    { 0x7af1676b, SUC_lexicon_tags_2 },
    { 0xeb71676c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc791676e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6316777, SUC_lexicon_tags_15 },
    { 0xc1316778, SUC_lexicon_tags_14 },
    { 0x71f16778, SUC_lexicon_tags_60 },
    { 0xa011677a, SUC_lexicon_tags_14 },
    { 0xa31677b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac51677e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x46116780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d16783, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9bf16785, SUC_lexicon_tags_33 },
    { 0xd0b16786, SUC_lexicon_tags_63 },
    { 0x9516787, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f1678a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b316790, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4d16792, SUC_lexicon_tags_271 },
    { 0xa9516792, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e316796, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe111679d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad167a0, SUC_lexicon_tags_56 },
    { 0x8fb167a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x607167a3, SUC_lexicon_tags_46 },
    { 0x947167a3, SUC_lexicon_tags_1 },
    { 0xb2d167a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757167b6, SUC_lexicon_tags_45 },
    { 0x7a7167b7, SUC_lexicon_tags_63 },
    { 0x215167b6, SUC_lexicon_tags_43 },
    { 0x4cd167b9, SUC_lexicon_tags_8 },
    { 0xc15167ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7167bf, SUC_lexicon_tags_83 },
    { 0x841167c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39167c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1167c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce5167ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff167cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9167d0, SUC_lexicon_tags_13 },
    { 0x851167d0, SUC_lexicon_tags_80 },
    { 0x1c3167d1, SUC_lexicon_tags_30 },
    { 0x13167d3, SUC_lexicon_tags_9 },
    { 0xf9d167d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2e3167d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc97167d8, SUC_lexicon_tags_9 },
    { 0xbab167d9, SUC_lexicon_tags_25 },
    { 0xa85167da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f167de, SUC_lexicon_tags_7 },
    { 0x199167df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21167e2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6ab167e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9167ee, SUC_lexicon_tags_60 },
    { 0x9a3167ef, SUC_lexicon_tags_8 },
    { 0xd9b167f0, SUC_lexicon_tags_19 },
    { 0x323167f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb167f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd167f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91167fe, SUC_lexicon_tags_39 },
    { 0xa51167ff, SUC_lexicon_tags_17 },
    { 0x2916800, SUC_lexicon_tags_7 },
    { 0x611167ff, SUC_lexicon_tags_49 },
    { 0x55116800, SUC_lexicon_tags_54 },
    { 0xe4316801, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41d16805, SUC_lexicon_tags_4 },
    { 0xaa516806, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb16812, SUC_lexicon_tags_14 },
    { 0xa1116813, SUC_lexicon_tags_47 },
    { 0xdf516813, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ff16816, SUC_lexicon_tags_12 },
    { 0xd5d16817, SUC_lexicon_tags_47 },
    { 0x2c116818, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef1681c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd716821, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6316829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfb1682b, SUC_lexicon_tags_45 },
    { 0xe3d1682c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f1682f, SUC_lexicon_tags_15 },
    { 0x48716830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32716832, SUC_lexicon_tags_49 },
    { 0x17b16833, SUC_lexicon_tags_11 },
    { 0xb9516832, SUC_lexicon_tags_60 },
    { 0x4cd16835, SUC_lexicon_tags_60 },
    { 0x80116836, SUC_lexicon_tags_37 },
    { 0x5ed16837, SUC_lexicon_tags_15 },
    { 0xd5d16837, SUC_lexicon_tags_12 },
    { 0xe5f16836, SUC_lexicon_tags_4 },
    { 0x3811683a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x66b1683c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79916840, SUC_lexicon_tags_7 },
    { 0x9ad16841, SUC_lexicon_tags_37 },
    { 0x20316842, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc7b16844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10516849, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda116854, SUC_lexicon_tags_170 },
    { 0x20916855, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c316858, SUC_lexicon_tags_17 },
    { 0xd2d16859, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb1685c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd831685e, SUC_lexicon_tags_45 },
    { 0xa431685e, SUC_lexicon_tags_34 },
    { 0x9c91685e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae516866, SUC_lexicon_tags_39 },
    { 0xd3f16867, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f1686b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4391686d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e316875, SUC_lexicon_tags_25 },
    { 0x6bb16876, SUC_lexicon_tags_11 },
    { 0xeb916877, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc71687a, SUC_lexicon_tags_15 },
    { 0xabf1687b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe916881, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2f716883, SUC_lexicon_tags_30 },
    { 0x4cb16884, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae51688f, SUC_lexicon_tags_34 },
    { 0x41d16890, SUC_lexicon_tags_63 },
    { 0x98b1688f, SUC_lexicon_tags_5 },
    { 0xa6d16892, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89f16894, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d168aa, SUC_lexicon_tags_27 },
    { 0x951168ab, SUC_lexicon_tags_25 },
    { 0xf01168ab, SUC_lexicon_tags_37 },
    { 0xbed168ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x905168af, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21168b3, SUC_lexicon_tags_61 },
    { 0x4eb168b4, SUC_lexicon_tags_14 },
    { 0x5ab168b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3168c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841168c6, SUC_lexicon_tags_15 },
    { 0xd95168c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f168ca, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95168cd, SUC_lexicon_tags_114 },
    { 0x929168cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x465168d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5168d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1168da, SUC_lexicon_tags_32 },
    { 0x77168db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x757168dd, SUC_lexicon_tags_15 },
    { 0x84b168de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87168e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x765168e3, SUC_lexicon_tags_37 },
    { 0x9db168e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9168e6, SUC_lexicon_tags_48 },
    { 0x87f168e6, SUC_lexicon_tags_37 },
    { 0xcb3168e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x769168ea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9168ee, SUC_lexicon_tags_11 },
    { 0xf45168ee, SUC_lexicon_tags_35 },
    { 0xd03168f0, SUC_lexicon_tags_8 },
    { 0xff7168f1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd168f4, SUC_lexicon_tags_2 },
    { 0x15f168f4, SUC_lexicon_tags_0 },
    { 0xebf168f6, SUC_lexicon_tags_34 },
    { 0x67f168f7, SUC_lexicon_tags_0 },
    { 0xfbb168f8, SUC_lexicon_tags_43 },
    { 0x36f168f9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9168fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf16903, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x51916905, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd51690b, SUC_lexicon_tags_7 },
    { 0xdad1690c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b16910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20316912, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc116915, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5e116917, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a11691a, SUC_lexicon_tags_14 },
    { 0x351691b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb16924, SUC_lexicon_tags_61 },
    { 0x45b16925, SUC_lexicon_tags_14 },
    { 0x43516926, SUC_lexicon_tags_7 },
    { 0x90f16924, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe411692b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a116931, SUC_lexicon_tags_37 },
    { 0xdff16932, SUC_lexicon_tags_37 },
    { 0x10116933, SUC_lexicon_tags_17 },
    { 0xfed16931, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2f16936, SUC_lexicon_tags_1 },
    { 0x4d916937, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x23116939, SUC_lexicon_tags_61 },
    { 0xd711693a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b1693d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b16940, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6916944, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x45d16946, SUC_lexicon_tags_37 },
    { 0x67116947, SUC_lexicon_tags_42 },
    { 0x7f116948, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a91694b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b1694d, SUC_lexicon_tags_12 },
    { 0xbdf1694e, SUC_lexicon_tags_12 },
    { 0xc491694f, SUC_lexicon_tags_1 },
    { 0xe11694f, SUC_lexicon_tags_37 },
    { 0x76b16951, SUC_lexicon_tags_7 },
    { 0x86f16952, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef16954, SUC_lexicon_tags_17 },
    { 0x18516955, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x22916958, SUC_lexicon_tags_129 },
    { 0x4bf16959, SUC_lexicon_tags_1 },
    { 0x33b1695a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba31695d, SUC_lexicon_tags_15 },
    { 0x1971695e, SUC_lexicon_tags_68 },
    { 0xcc71695d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef16966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87f16968, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c51696d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4516970, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f16975, SUC_lexicon_tags_23 },
    { 0x21916975, SUC_lexicon_tags_12 },
    { 0xca716977, SUC_lexicon_tags_2 },
    { 0x8d516978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f1697c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa731697f, SUC_lexicon_tags_4 },
    { 0x68b1697f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x39f16982, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe971698d, SUC_lexicon_tags_15 },
    { 0x9a31698d, SUC_lexicon_tags_37 },
    { 0x7dd1698d, SUC_lexicon_tags_14 },
    { 0xcdd1698e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xab316992, SUC_lexicon_tags_12 },
    { 0x9c916993, SUC_lexicon_tags_11 },
    { 0x47d16994, SUC_lexicon_tags_39 },
    { 0x4e116995, SUC_lexicon_tags_12 },
    { 0x17d16996, SUC_lexicon_tags_47 },
    { 0xd0116993, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad16999, SUC_lexicon_tags_85 },
    { 0x44b16999, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbdd1699c, SUC_lexicon_tags_30 },
    { 0xa0b1699c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1911699f, SUC_lexicon_tags_15 },
    { 0xe7169a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb169a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x973169a6, SUC_lexicon_tags_17 },
    { 0xc65169a7, SUC_lexicon_tags_25 },
    { 0x80f169a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8d169aa, SUC_lexicon_tags_17 },
    { 0xa3169ab, SUC_lexicon_tags_25 },
    { 0xb27169ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81169b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9169b4, SUC_lexicon_tags_2 },
    { 0x935169b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437169bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91169be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d169c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x305169ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x399169cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f169d2, SUC_lexicon_tags_11 },
    { 0x359169d3, SUC_lexicon_tags_17 },
    { 0x737169d3, SUC_lexicon_tags_15 },
    { 0xcb169d5, SUC_lexicon_tags_11 },
    { 0x849169d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f7169d8, SUC_lexicon_tags_1 },
    { 0x49f169d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643169dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1169e1, SUC_lexicon_tags_23 },
    { 0xc01169e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x241169e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b169ee, SUC_lexicon_tags_25 },
    { 0xc9b169ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa3f169f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757169f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681169fb, SUC_lexicon_tags_15 },
    { 0xc15169fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2316a01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9116a04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51916a0f, SUC_lexicon_tags_60 },
    { 0xb9516a10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35716a15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16d16a17, SUC_lexicon_tags_108 },
    { 0x2716a18, SUC_lexicon_tags_7 },
    { 0xa3316a18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd16a1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b16a21, SUC_lexicon_tags_29 },
    { 0xe6d16a21, SUC_lexicon_tags_33 },
    { 0xc6f16a23, SUC_lexicon_tags_8 },
    { 0xa9116a23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x48716a27, SUC_lexicon_tags_16 },
    { 0x63116a28, SUC_lexicon_tags_55 },
    { 0x56716a29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1116a2f, SUC_lexicon_tags_28 },
    { 0xfe516a30, SUC_lexicon_tags_9 },
    { 0xce716a30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6516a38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbab16a3a, SUC_lexicon_tags_2 },
    { 0xcad16a3b, SUC_lexicon_tags_25 },
    { 0x7d916a3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x34716a3e, SUC_lexicon_tags_65 },
    { 0x35d16a3e, SUC_lexicon_tags_14 },
    { 0x2f16a40, SUC_lexicon_tags_1 },
    { 0xe3516a40, SUC_lexicon_tags_25 },
    { 0x62316a42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f916a48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe9f16a4a, SUC_lexicon_tags_25 },
    { 0xf7716a4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6916a51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d516a53, SUC_lexicon_tags_12 },
    { 0x65516a54, SUC_lexicon_tags_25 },
    { 0xa3116a53, SUC_lexicon_tags_11 },
    { 0xab916a55, SUC_lexicon_tags_29 },
    { 0x47b16a53, SUC_lexicon_tags_28 },
    { 0x1b116a54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac516a5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7316a5f, SUC_lexicon_tags_12 },
    { 0x9c316a5f, SUC_lexicon_tags_6 },
    { 0xca916a5f, SUC_lexicon_tags_8 },
    { 0xd4116a62, SUC_lexicon_tags_25 },
    { 0x7f116a63, SUC_lexicon_tags_15 },
    { 0xab16a63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49316a67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad16a6a, SUC_lexicon_tags_39 },
    { 0xd9116a6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dd16a6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79516a71, SUC_lexicon_tags_30 },
    { 0x39716a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d16a75, SUC_lexicon_tags_37 },
    { 0x2fd16a75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d16a7c, SUC_lexicon_tags_47 },
    { 0xca716a7d, SUC_lexicon_tags_0 },
    { 0x60d16a7e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85916a84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x81316a87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b516a8c, SUC_lexicon_tags_269 },
    { 0xa9916a8d, SUC_lexicon_tags_17 },
    { 0x8c316a8c, SUC_lexicon_tags_2 },
    { 0x30116a8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3e116a91, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c316a95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f16a9b, SUC_lexicon_tags_11 },
    { 0x97b16a9c, SUC_lexicon_tags_84 },
    { 0x89316a9d, SUC_lexicon_tags_34 },
    { 0x43916a9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f316aa2, SUC_lexicon_tags_33 },
    { 0x3bf16aa3, SUC_lexicon_tags_25 },
    { 0x65b16aa4, SUC_lexicon_tags_25 },
    { 0x5a116aa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98716aa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d16aab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b16aad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0b16aaf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac516ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18116aba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd16abe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xae716ac0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64316ac2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4dd16ac4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3116ac9, SUC_lexicon_tags_43 },
    { 0xc3116aca, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff16ad3, SUC_lexicon_tags_17 },
    { 0x52d16ad4, SUC_lexicon_tags_238 },
    { 0x5d16ad5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd716ade, SUC_lexicon_tags_60 },
    { 0xcd516adf, SUC_lexicon_tags_10 },
    { 0xa3916adf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b16ae6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e316ae8, SUC_lexicon_tags_7 },
    { 0x6cb16ae9, SUC_lexicon_tags_5 },
    { 0x32716aea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b716aed, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6fb16aef, SUC_lexicon_tags_30 },
    { 0x51516af0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f716af6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2116af8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab16aff, SUC_lexicon_tags_16 },
    { 0x50316b00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f16b08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b16b0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b16b13, SUC_lexicon_tags_63 },
    { 0x85b16b14, SUC_lexicon_tags_17 },
    { 0xcdb16b15, SUC_lexicon_tags_37 },
    { 0x74d16b15, SUC_lexicon_tags_11 },
    { 0x2e516b13, SUC_lexicon_tags_8 },
    { 0x97916b14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b16b1d, SUC_lexicon_tags_7 },
    { 0x84f16b1e, SUC_lexicon_tags_17 },
    { 0x5c516b1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84916b21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69516b26, SUC_lexicon_tags_11 },
    { 0xf2316b26, SUC_lexicon_tags_37 },
    { 0xe2b16b28, SUC_lexicon_tags_113 },
    { 0x2f716b29, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x48716b2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf916b32, SUC_lexicon_tags_17 },
    { 0xe8916b32, SUC_lexicon_tags_27 },
    { 0x40916b34, SUC_lexicon_tags_27 },
    { 0xc2716b32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78516b37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25116b39, SUC_lexicon_tags_37 },
    { 0xa5516b3a, SUC_lexicon_tags_17 },
    { 0xcdf16b39, SUC_lexicon_tags_10 },
    { 0x49d16b39, SUC_lexicon_tags_33 },
    { 0xaaf16b3d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f16b44, SUC_lexicon_tags_351 },
    { 0x2116b45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x38f16b47, SUC_lexicon_tags_40 },
    { 0x99b16b48, SUC_lexicon_tags_57 },
    { 0xb7f16b49, SUC_lexicon_tags_7 },
    { 0xb1f16b48, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f16b4f, SUC_lexicon_tags_13 },
    { 0x9b316b50, SUC_lexicon_tags_69 },
    { 0xd6f16b4f, SUC_lexicon_tags_35 },
    { 0x75516b52, SUC_lexicon_tags_12 },
    { 0x9b516b4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b16b57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x24316b5a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8916b5c, SUC_lexicon_tags_11 },
    { 0xcef16b5c, SUC_lexicon_tags_38 },
    { 0xe2d16b5d, SUC_lexicon_tags_27 },
    { 0x15516b5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee116b61, SUC_lexicon_tags_1 },
    { 0xa8116b61, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f16b6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f16b74, SUC_lexicon_tags_17 },
    { 0x3a116b74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3116b78, SUC_lexicon_tags_12 },
    { 0x3e116b78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38716b7f, SUC_lexicon_tags_7 },
    { 0xc7716b7f, SUC_lexicon_tags_1 },
    { 0x74d16b81, SUC_lexicon_tags_12 },
    { 0xe3316b80, SUC_lexicon_tags_12 },
    { 0x92d16b83, SUC_lexicon_tags_14 },
    { 0x78f16b81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b516b86, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd16b8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51316b8e, SUC_lexicon_tags_12 },
    { 0xbbd16b8f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5716b92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x37116b94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b516b96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc4716b98, SUC_lexicon_tags_51 },
    { 0xf316b98, SUC_lexicon_tags_11 },
    { 0x97716b9a, SUC_lexicon_tags_19 },
    { 0xe6516b98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b16b9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56716ba3, SUC_lexicon_tags_10 },
    { 0x1a916ba4, SUC_lexicon_tags_11 },
    { 0x25d16ba3, SUC_lexicon_tags_29 },
    { 0x90b16ba6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d16baa, SUC_lexicon_tags_61 },
    { 0xe0d16bab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a716bb3, SUC_lexicon_tags_15 },
    { 0xba716bb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce716bbd, SUC_lexicon_tags_9 },
    { 0x68116bbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8516bc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb16bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f16bcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbc116bcd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb16bd1, SUC_lexicon_tags_17 },
    { 0x27d16bd1, SUC_lexicon_tags_67 },
    { 0x38316bd3, SUC_lexicon_tags_26 },
    { 0xa6716bd3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x77916bd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d316bdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74b16bdd, SUC_lexicon_tags_15 },
    { 0x8e516bde, SUC_lexicon_tags_23 },
    { 0x8716bde, SUC_lexicon_tags_23 },
    { 0xac516bdd, SUC_lexicon_tags_65 },
    { 0x2d716be1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcab16be3, SUC_lexicon_tags_25 },
    { 0x82f16be4, SUC_lexicon_tags_17 },
    { 0xaaf16be5, SUC_lexicon_tags_17 },
    { 0x6c516be6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b16bea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee116bed, SUC_lexicon_tags_2 },
    { 0xe5116bed, SUC_lexicon_tags_45 },
    { 0x26316bef, SUC_lexicon_tags_1 },
    { 0x87b16bf0, SUC_lexicon_tags_1 },
    { 0xcbf16bf0, SUC_lexicon_tags_17 },
    { 0xb2316bf2, SUC_lexicon_tags_38 },
    { 0x95916bf3, SUC_lexicon_tags_37 },
    { 0xc716bf4, SUC_lexicon_tags_13 },
    { 0x95b16bf1, SUC_lexicon_tags_33 },
    { 0xe3b16bf0, SUC_lexicon_tags_45 },
    { 0x79716bf6, SUC_lexicon_tags_2 },
    { 0x34716bf8, SUC_lexicon_tags_17 },
    { 0x5e916bed, SUC_lexicon_tags_37 },
    { 0x6e316bf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5116bfc, SUC_lexicon_tags_12 },
    { 0xbdf16bfd, SUC_lexicon_tags_15 },
    { 0x2316bfe, SUC_lexicon_tags_37 },
    { 0x6a516bfd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd16c02, SUC_lexicon_tags_15 },
    { 0xf916c03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ed16c05, SUC_lexicon_tags_2 },
    { 0xddf16c05, SUC_lexicon_tags_57 },
    { 0x1c516c07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x89316c0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d16c0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf16c13, SUC_lexicon_tags_1 },
    { 0xfa316c14, SUC_lexicon_tags_61 },
    { 0x8c316c15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebd16c17, SUC_lexicon_tags_15 },
    { 0xa1f16c17, SUC_lexicon_tags_66 },
    { 0x7c516c19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee716c1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b16c20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54316c22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6916c24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e716c27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69f16c29, SUC_lexicon_tags_31 },
    { 0xa4916c29, SUC_lexicon_tags_34 },
    { 0x13916c2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1716c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76516c31, SUC_lexicon_tags_37 },
    { 0xbab16c31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4716c34, SUC_lexicon_tags_1 },
    { 0x4d116c35, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df16c38, SUC_lexicon_tags_11 },
    { 0x4c916c39, SUC_lexicon_tags_25 },
    { 0x72116c38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b16c3e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d16c42, SUC_lexicon_tags_7 },
    { 0xd5516c43, SUC_lexicon_tags_38 },
    { 0xedb16c44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0716c48, SUC_lexicon_tags_7 },
    { 0x6716c49, SUC_lexicon_tags_4 },
    { 0xa3516c4a, SUC_lexicon_tags_11 },
    { 0x9d16c4a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7b116c4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba516c4f, SUC_lexicon_tags_1 },
    { 0x2b716c4f, SUC_lexicon_tags_26 },
    { 0xac916c51, SUC_lexicon_tags_9 },
    { 0x55716c52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d16c55, SUC_lexicon_tags_37 },
    { 0xbf716c56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa6b16c58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20916c5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d16c5e, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xab16c61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d16c65, SUC_lexicon_tags_28 },
    { 0x59b16c65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f16c6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf16c71, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15516c75, SUC_lexicon_tags_2 },
    { 0xfe116c76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36f16c78, SUC_lexicon_tags_15 },
    { 0x34f16c78, SUC_lexicon_tags_63 },
    { 0x8b916c78, SUC_lexicon_tags_47 },
    { 0xacf16c7b, SUC_lexicon_tags_12 },
    { 0x40916c7c, SUC_lexicon_tags_17 },
    { 0x2db16c7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94916c80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52916c85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12916c87, SUC_lexicon_tags_2 },
    { 0x5716c88, SUC_lexicon_tags_38 },
    { 0x98916c89, SUC_lexicon_tags_37 },
    { 0x67916c8a, SUC_lexicon_tags_68 },
    { 0x4f316c8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b116c8f, SUC_lexicon_tags_4 },
    { 0x91316c90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x516c92, SUC_lexicon_tags_37 },
    { 0x8fd16c93, SUC_lexicon_tags_10 },
    { 0xcef16c94, SUC_lexicon_tags_25 },
    { 0xaa316c95, SUC_lexicon_tags_14 },
    { 0x3c316c96, SUC_lexicon_tags_23 },
    { 0xcbf16c97, SUC_lexicon_tags_12 },
    { 0x17716c98, SUC_lexicon_tags_11 },
    { 0x7a516c94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4316c9b, SUC_lexicon_tags_12 },
    { 0xc6916c9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4716c9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6d16ca0, SUC_lexicon_tags_12 },
    { 0x1a116ca0, SUC_lexicon_tags_49 },
    { 0x53316ca2, SUC_lexicon_tags_7 },
    { 0xa9116ca3, SUC_lexicon_tags_2 },
    { 0x82916ca4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26b16ca6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x82916ca8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77916cab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d316cb2, SUC_lexicon_tags_11 },
    { 0x46b16cb3, SUC_lexicon_tags_47 },
    { 0xc0b16cb4, SUC_lexicon_tags_26 },
    { 0x71916cb3, SUC_lexicon_tags_45 },
    { 0x7e316cb5, SUC_lexicon_tags_30 },
    { 0x66716cb7, SUC_lexicon_tags_37 },
    { 0x40d16cb8, SUC_lexicon_tags_11 },
    { 0xc5d16cb2, SUC_lexicon_tags_27 },
    { 0xc9316cba, SUC_lexicon_tags_1 },
    { 0x1c516cbb, SUC_lexicon_tags_7 },
    { 0xd3f16cbc, SUC_lexicon_tags_37 },
    { 0x9316cbd, SUC_lexicon_tags_12 },
    { 0xb8516cbe, SUC_lexicon_tags_28 },
    { 0x5e116cbf, SUC_lexicon_tags_77 },
    { 0x52b16cbb, SUC_lexicon_tags_3 },
    { 0xb7116cc1, SUC_lexicon_tags_541 },
    { 0x5d516cbd, SUC_lexicon_tags_23 },
    { 0x27b16cbc, SUC_lexicon_tags_26 },
    { 0xc8916cc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97116cce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2516cd1, SUC_lexicon_tags_65 },
    { 0x53f16cd1, SUC_lexicon_tags_23 },
    { 0xd5516cd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20116cd6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57516cd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f16cdc, SUC_lexicon_tags_17 },
    { 0xe5316cdc, SUC_lexicon_tags_0 },
    { 0x2c116cde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e716cea, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41516cf0, SUC_lexicon_tags_28 },
    { 0x79d16cf1, SUC_lexicon_tags_0 },
    { 0x30f16cf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3916cf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded16cfc, SUC_lexicon_tags_29 },
    { 0x49116cfc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xebd16cff, SUC_lexicon_tags_1 },
    { 0x33316cff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4116d07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee916d09, SUC_lexicon_tags_11 },
    { 0xfaf16d09, SUC_lexicon_tags_170 },
    { 0x9b516d0b, SUC_lexicon_tags_28 },
    { 0xddb16d0a, SUC_lexicon_tags_17 },
    { 0xf2d16d0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f16d10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbb16d12, SUC_lexicon_tags_21 },
    { 0x8d516d13, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x5716d15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc116d18, SUC_lexicon_tags_13 },
    { 0xd9d16d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90316d1b, SUC_lexicon_tags_11 },
    { 0xf6316d1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2cf16d1e, SUC_lexicon_tags_1 },
    { 0x98516d1f, SUC_lexicon_tags_25 },
    { 0x7ab16d20, SUC_lexicon_tags_17 },
    { 0xa2b16d21, SUC_lexicon_tags_65 },
    { 0xa7716d22, SUC_lexicon_tags_11 },
    { 0x99516d23, SUC_lexicon_tags_47 },
    { 0x71d16d22, SUC_lexicon_tags_11 },
    { 0xcd716d1e, SUC_lexicon_tags_11 },
    { 0x7a716d26, SUC_lexicon_tags_29 },
    { 0x41b16d26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb16d2f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x56516d32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a716d36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x52316d39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2516d3b, SUC_lexicon_tags_12 },
    { 0x76916d3c, SUC_lexicon_tags_11 },
    { 0x18f16d3c, SUC_lexicon_tags_8 },
    { 0x88f16d3d, SUC_lexicon_tags_47 },
    { 0xf9116d3c, SUC_lexicon_tags_11 },
    { 0x84916d3b, SUC_lexicon_tags_47 },
    { 0xbb16d3f, SUC_lexicon_tags_21 },
    { 0x21916d42, SUC_lexicon_tags_37 },
    { 0x3cf16d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f16d46, SUC_lexicon_tags_45 },
    { 0x4af16d46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94716d4a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a116d4f, SUC_lexicon_tags_25 },
    { 0xe7916d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e716d54, SUC_lexicon_tags_19 },
    { 0x4d916d55, SUC_lexicon_tags_2 },
    { 0x36f16d54, SUC_lexicon_tags_38 },
    { 0xea916d54, SUC_lexicon_tags_138 },
    { 0x2b316d58, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd5716d5a, SUC_lexicon_tags_17 },
    { 0x15716d5b, SUC_lexicon_tags_2 },
    { 0xb7b16d5c, SUC_lexicon_tags_65 },
    { 0x29716d5d, SUC_lexicon_tags_12 },
    { 0xff516d5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71716d62, SUC_lexicon_tags_31 },
    { 0xa9116d62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18916d66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99116d68, SUC_lexicon_tags_20 },
    { 0x49916d68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x30516d6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb16d6f, SUC_lexicon_tags_1 },
    { 0x26b16d70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f16d74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4316d77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddd16d79, SUC_lexicon_tags_37 },
    { 0x5e516d79, SUC_lexicon_tags_19 },
    { 0x79b16d7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x78916d7e, SUC_lexicon_tags_2 },
    { 0x26116d7e, SUC_lexicon_tags_15 },
    { 0xbb316d7f, SUC_lexicon_tags_49 },
    { 0x95716d80, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46716d83, SUC_lexicon_tags_47 },
    { 0x2f316d84, SUC_lexicon_tags_9 },
    { 0xff716d83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b16d88, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf3916d8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2916d90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf1f16d92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5cb16d94, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x57516d96, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f116d99, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x83916d9c, SUC_lexicon_tags_46 },
    { 0xd4b16d9c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4f116d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4116da6, SUC_lexicon_tags_15 },
    { 0x7fd16da7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2f16da9, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x9b516dab, SUC_lexicon_tags_49 },
    { 0x61316dab, SUC_lexicon_tags_17 },
    { 0xa1716dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7316daf, SUC_lexicon_tags_25 },
    { 0xc2316db0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0716db4, SUC_lexicon_tags_11 },
    { 0x26516db4, SUC_lexicon_tags_14 },
    { 0x71f16db5, SUC_lexicon_tags_33 },
    { 0xb1f16db7, SUC_lexicon_tags_43 },
    { 0x6b716db8, SUC_lexicon_tags_9 },
    { 0xd1d16db6, SUC_lexicon_tags_7 },
    { 0xe3d16dba, SUC_lexicon_tags_21 },
    { 0x82f16dbb, SUC_lexicon_tags_19 },
    { 0xf1716dbb, SUC_lexicon_tags_17 },
    { 0xd2b16dbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88116dbf, SUC_lexicon_tags_65 },
    { 0xd7716dc0, SUC_lexicon_tags_2 },
    { 0x79d16dc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5f16dc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c916dc7, SUC_lexicon_tags_0 },
    { 0x25d16dc7, SUC_lexicon_tags_37 },
    { 0x4cb16dc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x64916dcb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab16dd1, SUC_lexicon_tags_15 },
    { 0xf5916dd2, SUC_lexicon_tags_1 },
    { 0x7ff16dd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb116dd9, SUC_lexicon_tags_37 },
    { 0x42516dda, SUC_lexicon_tags_11 },
    { 0x3bd16ddb, SUC_lexicon_tags_17 },
    { 0x62716dd9, SUC_lexicon_tags_27 },
    { 0x89916ddd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x73316ddf, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97716de7, SUC_lexicon_tags_10 },
    { 0x2a316de7, SUC_lexicon_tags_25 },
    { 0x40516de9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed716df0, SUC_lexicon_tags_20 },
    { 0xc2916df0, SUC_lexicon_tags_66 },
    { 0x11516df2, SUC_lexicon_tags_23 },
    { 0x5a116df0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c916dfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ff16dfd, SUC_lexicon_tags_29 },
    { 0x24d16dfe, SUC_lexicon_tags_65 },
    { 0x50916dff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0116e04, SUC_lexicon_tags_7 },
    { 0x23d16e05, SUC_lexicon_tags_6 },
    { 0xb3516e05, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdfb16e08, SUC_lexicon_tags_2 },
    { 0x8a716e08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c716e0d, SUC_lexicon_tags_37 },
    { 0x42d16e0d, SUC_lexicon_tags_26 },
    { 0x37516e0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d716e11, SUC_lexicon_tags_47 },
    { 0x31316e12, SUC_lexicon_tags_11 },
    { 0xead16e13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5eb16e15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f16e18, SUC_lexicon_tags_23 },
    { 0x24716e19, SUC_lexicon_tags_11 },
    { 0x86316e1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4f16e1c, SUC_lexicon_tags_1 },
    { 0x90316e1d, SUC_lexicon_tags_11 },
    { 0x35b16e1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1316e26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9116e28, SUC_lexicon_tags_1 },
    { 0x71316e29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d16e2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8516e2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3116e31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0116e35, SUC_lexicon_tags_19 },
    { 0x59b16e36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95116e38, SUC_lexicon_tags_11 },
    { 0x12716e39, SUC_lexicon_tags_21 },
    { 0x43716e39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b16e3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb316e3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b16e42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99d16e44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab316e47, SUC_lexicon_tags_16 },
    { 0x87116e48, SUC_lexicon_tags_45 },
    { 0xa1b16e49, SUC_lexicon_tags_47 },
    { 0x9cd16e49, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3316e4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab116e51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71316e53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e916e5c, SUC_lexicon_tags_30 },
    { 0xe9316e5d, SUC_lexicon_tags_11 },
    { 0xb9d16e5e, SUC_lexicon_tags_39 },
    { 0x72316e5f, SUC_lexicon_tags_9 },
    { 0xcff16e60, SUC_lexicon_tags_27 },
    { 0x5c516e60, SUC_lexicon_tags_37 },
    { 0x8c916e60, SUC_lexicon_tags_34 },
    { 0xe8d16e61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f516e65, SUC_lexicon_tags_40 },
    { 0x3f116e66, SUC_lexicon_tags_63 },
    { 0x91916e67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbad16e69, SUC_lexicon_tags_65 },
    { 0x90316e6a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28716e70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b716e73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc4f16e75, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a516e79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f16e7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27316e7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1916e82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x56916e84, SUC_lexicon_tags_1 },
    { 0xdf716e85, SUC_lexicon_tags_25 },
    { 0xd5516e86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8116e88, SUC_lexicon_tags_11 },
    { 0x53f16e89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b16e8c, SUC_lexicon_tags_26 },
    { 0xd1516e8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe116e90, SUC_lexicon_tags_4 },
    { 0xbfd16e90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d16e9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4916e9e, SUC_lexicon_tags_17 },
    { 0xadf16e9e, SUC_lexicon_tags_44 },
    { 0x58716ea0, SUC_lexicon_tags_57 },
    { 0xf9b16ea1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf116ea4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf16ea9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84116eab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b16eb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfa716eb4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2516eb8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd516ebb, SUC_lexicon_tags_16 },
    { 0xf0f16ebb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd16ebf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f16ec4, SUC_lexicon_tags_32 },
    { 0x39116ec4, SUC_lexicon_tags_15 },
    { 0x64716ec4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92716ece, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b16ed1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53316ed6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef16edc, SUC_lexicon_tags_107 },
    { 0x4f16edc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa4f16edf, SUC_lexicon_tags_16 },
    { 0x80b16edf, SUC_lexicon_tags_39 },
    { 0x72b16ee0, SUC_lexicon_tags_14 },
    { 0x48f16ee2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd16ee8, SUC_lexicon_tags_158 },
    { 0x9316ee9, SUC_lexicon_tags_15 },
    { 0xed516ee9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2316eec, SUC_lexicon_tags_11 },
    { 0xd5316eed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5116ef1, SUC_lexicon_tags_37 },
    { 0xb0316ef2, SUC_lexicon_tags_60 },
    { 0x96b16ef2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc9716ef5, SUC_lexicon_tags_375 },
    { 0xd6116ef6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f16ef8, SUC_lexicon_tags_60 },
    { 0x58516ef8, SUC_lexicon_tags_1 },
    { 0x9c516efa, SUC_lexicon_tags_1 },
    { 0x8b16efb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f316efe, SUC_lexicon_tags_38 },
    { 0xc7116efe, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd16f02, SUC_lexicon_tags_63 },
    { 0x9516f03, SUC_lexicon_tags_1 },
    { 0x9b116f03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3b16f06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1f16f08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa7916f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4516f0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4116f11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2716f16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e716f1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27716f1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3f16f21, SUC_lexicon_tags_14 },
    { 0xe9d16f22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58716f27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc3b16f29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17116f2d, SUC_lexicon_tags_21 },
    { 0x7d916f2d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c316f32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf516f39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x77316f3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd516f3e, SUC_lexicon_tags_37 },
    { 0xe3b16f3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff16f42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df16f45, SUC_lexicon_tags_2 },
    { 0x2c916f46, SUC_lexicon_tags_29 },
    { 0x14116f47, SUC_lexicon_tags_23 },
    { 0x74f16f46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d16f4c, SUC_lexicon_tags_1 },
    { 0x4d116f4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b16f50, SUC_lexicon_tags_49 },
    { 0xba116f51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6316f55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d116f58, SUC_lexicon_tags_11 },
    { 0x30516f59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xde116f5c, SUC_lexicon_tags_1 },
    { 0x46d16f5d, SUC_lexicon_tags_63 },
    { 0x88b16f5e, SUC_lexicon_tags_10 },
    { 0xe0d16f5f, SUC_lexicon_tags_1 },
    { 0xcc716f5d, SUC_lexicon_tags_60 },
    { 0xe1516f61, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x57716f64, SUC_lexicon_tags_79 },
    { 0xd3f16f65, SUC_lexicon_tags_46 },
    { 0x1cd16f65, SUC_lexicon_tags_39 },
    { 0x26116f65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4916f6c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8716f74, SUC_lexicon_tags_1 },
    { 0x93f16f75, SUC_lexicon_tags_11 },
    { 0xcff16f76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f16f79, SUC_lexicon_tags_3 },
    { 0x116f7a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xac116f7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6bb16f7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14316f81, SUC_lexicon_tags_11 },
    { 0xc7d16f81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf516f84, SUC_lexicon_tags_46 },
    { 0x65b16f84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff16f89, SUC_lexicon_tags_17 },
    { 0x39d16f8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22116f8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d16f92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb5916f94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f16f97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48316f9d, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xea116fa0, SUC_lexicon_tags_14 },
    { 0xc7316fa1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf16fa4, SUC_lexicon_tags_17 },
    { 0x9f116fa5, SUC_lexicon_tags_1 },
    { 0x82916fa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84316faa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f116fac, SUC_lexicon_tags_37 },
    { 0x51b16fac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d16fba, SUC_lexicon_tags_25 },
    { 0xe3516fbb, SUC_lexicon_tags_27 },
    { 0x21116fbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d16fbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d316fc8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83716fd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59516fd4, SUC_lexicon_tags_15 },
    { 0x64716fd5, SUC_lexicon_tags_6 },
    { 0x27d16fd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39b16fd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99116fdd, SUC_lexicon_tags_17 },
    { 0x56316fde, SUC_lexicon_tags_5 },
    { 0x4316fdd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd116fe7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec116fee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x12916ff1, SUC_lexicon_tags_11 },
    { 0xff316ff2, SUC_lexicon_tags_38 },
    { 0x13716ff3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34516ff7, SUC_lexicon_tags_47 },
    { 0x6d716ff8, SUC_lexicon_tags_17 },
    { 0x46716ff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25d16ffb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb16ffd, SUC_lexicon_tags_11 },
    { 0x4f716ffd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d517001, SUC_lexicon_tags_25 },
    { 0x4f917001, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0117006, SUC_lexicon_tags_17 },
    { 0xf2f17007, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d71700c, SUC_lexicon_tags_37 },
    { 0x6831700c, SUC_lexicon_tags_25 },
    { 0xc911700e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff917010, SUC_lexicon_tags_11 },
    { 0x64117011, SUC_lexicon_tags_9 },
    { 0x91917012, SUC_lexicon_tags_0 },
    { 0x51117012, SUC_lexicon_tags_38 },
    { 0x1ff17014, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59517018, SUC_lexicon_tags_17 },
    { 0x6fb17019, SUC_lexicon_tags_25 },
    { 0x6771701a, SUC_lexicon_tags_15 },
    { 0x69b1701b, SUC_lexicon_tags_91 },
    { 0xc51701a, SUC_lexicon_tags_30 },
    { 0x2651701b, SUC_lexicon_tags_47 },
    { 0xffd1701a, SUC_lexicon_tags_63 },
    { 0xf5f17019, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79117025, SUC_lexicon_tags_25 },
    { 0x5ad17025, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d17029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f17032, SUC_lexicon_tags_11 },
    { 0xf5b17032, SUC_lexicon_tags_12 },
    { 0xf8517034, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8717038, SUC_lexicon_tags_25 },
    { 0x11317038, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1703d, SUC_lexicon_tags_14 },
    { 0x551703e, SUC_lexicon_tags_37 },
    { 0x32d1703d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4717042, SUC_lexicon_tags_84 },
    { 0xc8317043, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7717046, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3717048, SUC_lexicon_tags_6 },
    { 0x5c317049, SUC_lexicon_tags_45 },
    { 0x3c717049, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x66f1704c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf17054, SUC_lexicon_tags_37 },
    { 0xa117054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d17059, SUC_lexicon_tags_22 },
    { 0xd917059, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85917062, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd17065, SUC_lexicon_tags_7 },
    { 0xd5717066, SUC_lexicon_tags_12 },
    { 0x51917067, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24117069, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f17071, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabb17073, SUC_lexicon_tags_17 },
    { 0x1db17074, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfaf17076, SUC_lexicon_tags_17 },
    { 0x8ad17076, SUC_lexicon_tags_17 },
    { 0x9b517078, SUC_lexicon_tags_1 },
    { 0x35b17079, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa451707b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5717080, SUC_lexicon_tags_84 },
    { 0x57917081, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1317087, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471708c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2d1708e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8517090, SUC_lexicon_tags_11 },
    { 0x6fb17091, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xddd17093, SUC_lexicon_tags_9 },
    { 0x8c917094, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd117097, SUC_lexicon_tags_27 },
    { 0xdd117097, SUC_lexicon_tags_42 },
    { 0x6d517097, SUC_lexicon_tags_47 },
    { 0xdbf1709a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa791709f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79170a4, SUC_lexicon_tags_124 },
    { 0x151170a4, SUC_lexicon_tags_2 },
    { 0x3db170a6, SUC_lexicon_tags_69 },
    { 0xfc7170a6, SUC_lexicon_tags_39 },
    { 0xb77170a8, SUC_lexicon_tags_17 },
    { 0x583170a7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6b170ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc1170ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f170b3, SUC_lexicon_tags_1351 },
    { 0x7f9170b4, SUC_lexicon_tags_27 },
    { 0x96f170b5, SUC_lexicon_tags_15 },
    { 0xb19170b6, SUC_lexicon_tags_85 },
    { 0xe67170b7, SUC_lexicon_tags_25 },
    { 0x9d3170b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49170be, SUC_lexicon_tags_45 },
    { 0x67170bf, SUC_lexicon_tags_31 },
    { 0xed1170bf, SUC_lexicon_tags_17 },
    { 0xa3f170c0, SUC_lexicon_tags_84 },
    { 0x381170c2, SUC_lexicon_tags_48 },
    { 0x793170c3, SUC_lexicon_tags_37 },
    { 0x273170c4, SUC_lexicon_tags_17 },
    { 0x57f170c3, SUC_lexicon_tags_29 },
    { 0x2e9170be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425170d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5b170da, SUC_lexicon_tags_17 },
    { 0x907170db, SUC_lexicon_tags_68 },
    { 0x831170da, SUC_lexicon_tags_87 },
    { 0xa3170dd, SUC_lexicon_tags_1 },
    { 0xcfb170de, SUC_lexicon_tags_15 },
    { 0x1c5170df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd9170e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273170e5, SUC_lexicon_tags_7 },
    { 0xf93170e5, SUC_lexicon_tags_28 },
    { 0xfd1170e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x753170ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa93170ec, SUC_lexicon_tags_19 },
    { 0x68d170ec, SUC_lexicon_tags_47 },
    { 0x631170ee, SUC_lexicon_tags_14 },
    { 0xf51170ec, SUC_lexicon_tags_806 },
    { 0x253170f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c7170f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ff170f4, SUC_lexicon_tags_45 },
    { 0x141170f5, SUC_lexicon_tags_38 },
    { 0x11b170f6, SUC_lexicon_tags_2 },
    { 0xd37170f7, SUC_lexicon_tags_17 },
    { 0x431170f7, SUC_lexicon_tags_19 },
    { 0x225170f9, SUC_lexicon_tags_27 },
    { 0x431170fa, SUC_lexicon_tags_2 },
    { 0xbbd170fb, SUC_lexicon_tags_27 },
    { 0xadd170fb, SUC_lexicon_tags_17 },
    { 0x337170fd, SUC_lexicon_tags_0 },
    { 0x6e3170fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb2317100, SUC_lexicon_tags_17 },
    { 0x6b117101, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2117103, SUC_lexicon_tags_2 },
    { 0xb5917103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e917108, SUC_lexicon_tags_1 },
    { 0x7b917109, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f17110, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4b317112, SUC_lexicon_tags_37 },
    { 0xe3317113, SUC_lexicon_tags_54 },
    { 0x95517113, SUC_lexicon_tags_17 },
    { 0x90717114, SUC_lexicon_tags_31 },
    { 0x1b17116, SUC_lexicon_tags_25 },
    { 0x9117116, SUC_lexicon_tags_25 },
    { 0x85d17118, SUC_lexicon_tags_63 },
    { 0xf1b17114, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe831711d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb17123, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd6d17125, SUC_lexicon_tags_65 },
    { 0xc9f17125, SUC_lexicon_tags_25 },
    { 0xcd717127, SUC_lexicon_tags_23 },
    { 0xdc517128, SUC_lexicon_tags_38 },
    { 0xc8f17129, SUC_lexicon_tags_38 },
    { 0xb0f1712a, SUC_lexicon_tags_33 },
    { 0x2df17127, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86517130, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ab17132, SUC_lexicon_tags_1 },
    { 0x5eb17133, SUC_lexicon_tags_2 },
    { 0x97717132, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x6e917136, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa651713b, SUC_lexicon_tags_2 },
    { 0xb951713c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c517141, SUC_lexicon_tags_43 },
    { 0x6bd17142, SUC_lexicon_tags_0 },
    { 0x6f317141, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x77f17145, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb2b17147, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbd17149, SUC_lexicon_tags_0 },
    { 0x3b517149, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5317154, SUC_lexicon_tags_2 },
    { 0x62317155, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa3517157, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c31715a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13917160, SUC_lexicon_tags_27 },
    { 0x92d17161, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f17164, SUC_lexicon_tags_38 },
    { 0xb8d17165, SUC_lexicon_tags_15 },
    { 0x49f17166, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c51716c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7517170, SUC_lexicon_tags_33 },
    { 0x1f117171, SUC_lexicon_tags_48 },
    { 0xa3f17172, SUC_lexicon_tags_2 },
    { 0xad17172, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cf17175, SUC_lexicon_tags_68 },
    { 0x65317176, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8717178, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f1717c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac117185, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4df17187, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1811718d, SUC_lexicon_tags_17 },
    { 0xa051718e, SUC_lexicon_tags_37 },
    { 0x5271718f, SUC_lexicon_tags_15 },
    { 0x4351718f, SUC_lexicon_tags_11 },
    { 0x42f1718f, SUC_lexicon_tags_38 },
    { 0x8f917192, SUC_lexicon_tags_17 },
    { 0x49f17193, SUC_lexicon_tags_1 },
    { 0xd5117194, SUC_lexicon_tags_25 },
    { 0xfa917194, SUC_lexicon_tags_40 },
    { 0x84117195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52917199, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb671719e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f171a2, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0xf99171a5, SUC_lexicon_tags_229 },
    { 0x91b171a6, SUC_lexicon_tags_33 },
    { 0xf1171a7, SUC_lexicon_tags_11 },
    { 0x931171a7, SUC_lexicon_tags_66 },
    { 0x89171a5, SUC_lexicon_tags_14 },
    { 0x11b171aa, SUC_lexicon_tags_17 },
    { 0xb4b171aa, SUC_lexicon_tags_23 },
    { 0xc35171a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32d171ae, SUC_lexicon_tags_12 },
    { 0x36f171af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149171b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35171bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefb171bf, SUC_lexicon_tags_26 },
    { 0x569171c0, SUC_lexicon_tags_17 },
    { 0xb9d171c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757171c5, SUC_lexicon_tags_7 },
    { 0x9b1171c6, SUC_lexicon_tags_11 },
    { 0x795171c7, SUC_lexicon_tags_12 },
    { 0x107171c7, SUC_lexicon_tags_65 },
    { 0xa81171c9, SUC_lexicon_tags_7 },
    { 0x56d171c5, SUC_lexicon_tags_70 },
    { 0xbcd171cb, SUC_lexicon_tags_63 },
    { 0x23171cb, SUC_lexicon_tags_47 },
    { 0x9e7171c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3171d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5171d5, SUC_lexicon_tags_14 },
    { 0x383171d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf39171d8, SUC_lexicon_tags_49 },
    { 0xbcf171d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5171dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d171e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d171e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5cb171e9, SUC_lexicon_tags_49 },
    { 0x48f171e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5171ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b171f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x105171f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c517200, SUC_lexicon_tags_17 },
    { 0xcf17200, SUC_lexicon_tags_39 },
    { 0xc7117202, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f17205, SUC_lexicon_tags_26 },
    { 0xe7917205, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe0117208, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc11720c, SUC_lexicon_tags_23 },
    { 0x8a51720d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f17210, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18117216, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5317218, SUC_lexicon_tags_1 },
    { 0x5917219, SUC_lexicon_tags_15 },
    { 0x9117219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71721d, SUC_lexicon_tags_37 },
    { 0xacb1721e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b17227, SUC_lexicon_tags_8 },
    { 0x7df17227, SUC_lexicon_tags_19 },
    { 0xfd917229, SUC_lexicon_tags_6 },
    { 0x9ab1722a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3851722d, SUC_lexicon_tags_151 },
    { 0x5571722e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4f17230, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d717234, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x2f117236, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d717238, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb1723f, SUC_lexicon_tags_66 },
    { 0xa3517240, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b117244, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4b117246, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b1724a, SUC_lexicon_tags_45 },
    { 0x6691724a, SUC_lexicon_tags_66 },
    { 0xa531724c, SUC_lexicon_tags_12 },
    { 0xa231724d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f17251, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ef17253, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74517255, SUC_lexicon_tags_37 },
    { 0xeaf17256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1717258, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd391725d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19917262, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb317264, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f17267, SUC_lexicon_tags_1 },
    { 0x5d517267, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9071726d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf17273, SUC_lexicon_tags_33 },
    { 0x43917273, SUC_lexicon_tags_8 },
    { 0xdd517275, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d17280, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a117287, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3717289, SUC_lexicon_tags_11 },
    { 0x71b17289, SUC_lexicon_tags_28 },
    { 0x99d17289, SUC_lexicon_tags_25 },
    { 0xaff1728c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d91728f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f917291, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21117293, SUC_lexicon_tags_40 },
    { 0xf9d17294, SUC_lexicon_tags_27 },
    { 0xf6f17295, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b17299, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb751729d, SUC_lexicon_tags_61 },
    { 0xe2f1729d, SUC_lexicon_tags_25 },
    { 0x71f1729e, SUC_lexicon_tags_5 },
    { 0x7bf172a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff172a3, SUC_lexicon_tags_37 },
    { 0xb65172a3, SUC_lexicon_tags_25 },
    { 0x879172a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d172a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663172ae, SUC_lexicon_tags_37 },
    { 0xd71172af, SUC_lexicon_tags_63 },
    { 0xa83172ae, SUC_lexicon_tags_25 },
    { 0x22f172ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d172b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf172b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3172be, SUC_lexicon_tags_16 },
    { 0x3a5172be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d172c2, SUC_lexicon_tags_9 },
    { 0x463172c3, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db172cd, SUC_lexicon_tags_24 },
    { 0x191172ce, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1172d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xabb172d4, SUC_lexicon_tags_3 },
    { 0xc9b172d5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09172d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f172dc, SUC_lexicon_tags_37 },
    { 0xb1b172dc, SUC_lexicon_tags_2 },
    { 0xb3172de, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b172e1, SUC_lexicon_tags_187 },
    { 0x179172e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9d172e4, SUC_lexicon_tags_25 },
    { 0x8cb172e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd172ea, SUC_lexicon_tags_19 },
    { 0xd59172eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9172ed, SUC_lexicon_tags_15 },
    { 0x3c5172ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb172f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x42f172f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d172f6, SUC_lexicon_tags_6 },
    { 0x40b172f6, SUC_lexicon_tags_25 },
    { 0x305172f8, SUC_lexicon_tags_34 },
    { 0x5b3172f9, SUC_lexicon_tags_80 },
    { 0x12d172f9, SUC_lexicon_tags_49 },
    { 0x7bd172f9, SUC_lexicon_tags_29 },
    { 0xde5172fc, SUC_lexicon_tags_6 },
    { 0x51b172fd, SUC_lexicon_tags_26 },
    { 0x579172fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76517301, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25117304, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc6117306, SUC_lexicon_tags_12 },
    { 0x78517307, SUC_lexicon_tags_15 },
    { 0x33317308, SUC_lexicon_tags_1 },
    { 0x60117309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe31730d, SUC_lexicon_tags_87 },
    { 0x60b1730e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24917317, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9171731f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb6317321, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9317325, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4117328, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x15d1732a, SUC_lexicon_tags_21 },
    { 0x53f1732b, SUC_lexicon_tags_16 },
    { 0x4d11732b, SUC_lexicon_tags_25 },
    { 0xc811732c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3717332, SUC_lexicon_tags_9 },
    { 0x14917332, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x74117335, SUC_lexicon_tags_1 },
    { 0xdc517335, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d1733b, SUC_lexicon_tags_43 },
    { 0x1731733b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6711733f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8717345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd117348, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe291734a, SUC_lexicon_tags_4 },
    { 0x4fb1734a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd1734e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbad17350, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d17353, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9917359, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x8951735c, SUC_lexicon_tags_8 },
    { 0xa651735d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99117362, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21717364, SUC_lexicon_tags_140 },
    { 0xc4717365, SUC_lexicon_tags_37 },
    { 0xbff17364, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e517369, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c71736b, SUC_lexicon_tags_77 },
    { 0x6611736c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6317372, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48317377, SUC_lexicon_tags_34 },
    { 0x48517378, SUC_lexicon_tags_19 },
    { 0xe5f17379, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e91737b, SUC_lexicon_tags_37 },
    { 0xfb51737c, SUC_lexicon_tags_37 },
    { 0xec31737d, SUC_lexicon_tags_31 },
    { 0x90f1737e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c517382, SUC_lexicon_tags_1 },
    { 0x12917383, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2917385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87d17387, SUC_lexicon_tags_29 },
    { 0x63f17388, SUC_lexicon_tags_28 },
    { 0x32b17387, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd471738c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4471738e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18317390, SUC_lexicon_tags_16 },
    { 0x62f17390, SUC_lexicon_tags_26 },
    { 0x3f517392, SUC_lexicon_tags_49 },
    { 0xebb17393, SUC_lexicon_tags_21 },
    { 0x43317394, SUC_lexicon_tags_37 },
    { 0xc2517390, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf717398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1631739c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547173a3, SUC_lexicon_tags_41 },
    { 0x603173a4, SUC_lexicon_tags_27 },
    { 0xee5173a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff173a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41173ab, SUC_lexicon_tags_55 },
    { 0x1c9173ab, SUC_lexicon_tags_11 },
    { 0x8a1173ac, SUC_lexicon_tags_133 },
    { 0xe61173ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5b173b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4d9173b2, SUC_lexicon_tags_15 },
    { 0xdf3173b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7173ba, SUC_lexicon_tags_17 },
    { 0xf5b173bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x219173bd, SUC_lexicon_tags_364 },
    { 0xe49173bd, SUC_lexicon_tags_1 },
    { 0xdbf173bd, SUC_lexicon_tags_11 },
    { 0xd75173c0, SUC_lexicon_tags_47 },
    { 0x841173c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x271173c4, SUC_lexicon_tags_9 },
    { 0xad1173c5, SUC_lexicon_tags_37 },
    { 0x6d9173c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6fd173c8, SUC_lexicon_tags_37 },
    { 0x6d5173c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77173cf, SUC_lexicon_tags_37 },
    { 0xc79173d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb173d3, SUC_lexicon_tags_26 },
    { 0xa65173d4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x933173d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb173dd, SUC_lexicon_tags_1311 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1173e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90b173e5, SUC_lexicon_tags_55 },
    { 0x5dd173e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3173e9, SUC_lexicon_tags_39 },
    { 0x8c5173ea, SUC_lexicon_tags_17 },
    { 0xa5173eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f173f9, SUC_lexicon_tags_23 },
    { 0x1fd173f9, SUC_lexicon_tags_15 },
    { 0x31b173fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22317401, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab917407, SUC_lexicon_tags_11 },
    { 0x7d717407, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ef1740a, SUC_lexicon_tags_12 },
    { 0x3271740b, SUC_lexicon_tags_12 },
    { 0xe3d1740c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6811740f, SUC_lexicon_tags_4 },
    { 0xc5317410, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73317412, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9517415, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73517417, SUC_lexicon_tags_810 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f1741a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2191741c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40917420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9917422, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8517425, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb17427, SUC_lexicon_tags_1 },
    { 0xd1b17428, SUC_lexicon_tags_15 },
    { 0xbef17428, SUC_lexicon_tags_72 },
    { 0x10517428, SUC_lexicon_tags_37 },
    { 0x1d317428, SUC_lexicon_tags_31 },
    { 0xa791742c, SUC_lexicon_tags_88 },
    { 0x82b17429, SUC_lexicon_tags_1 },
    { 0xcaf1742e, SUC_lexicon_tags_47 },
    { 0x9d51742d, SUC_lexicon_tags_38 },
    { 0xbcd17430, SUC_lexicon_tags_23 },
    { 0xe4917429, SUC_lexicon_tags_47 },
    { 0x50317432, SUC_lexicon_tags_8 },
    { 0x81917432, SUC_lexicon_tags_30 },
    { 0xbb317434, SUC_lexicon_tags_64 },
    { 0xafd17434, SUC_lexicon_tags_30 },
    { 0x73317434, SUC_lexicon_tags_25 },
    { 0x80d1742c, SUC_lexicon_tags_541 },
    { 0x45d1742a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7531743a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd117440, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80717447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6151744b, SUC_lexicon_tags_2 },
    { 0x2651744b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef1744f, SUC_lexicon_tags_38 },
    { 0x1917450, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa917454, SUC_lexicon_tags_33 },
    { 0xfed17454, SUC_lexicon_tags_30 },
    { 0xbcb17454, SUC_lexicon_tags_17 },
    { 0x36317457, SUC_lexicon_tags_14 },
    { 0xa3117458, SUC_lexicon_tags_184 },
    { 0x2ff17458, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511745e, SUC_lexicon_tags_33 },
    { 0x1e91745f, SUC_lexicon_tags_12 },
    { 0x2831745f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b117464, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4e917466, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd3d17468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d1746b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c117471, SUC_lexicon_tags_25 },
    { 0x69d17472, SUC_lexicon_tags_25 },
    { 0xf917472, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b1747d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15117480, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d17483, SUC_lexicon_tags_26 },
    { 0xdef17483, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb3b17486, SUC_lexicon_tags_2 },
    { 0xf3717487, SUC_lexicon_tags_11 },
    { 0x49d17488, SUC_lexicon_tags_7 },
    { 0x93b17489, SUC_lexicon_tags_9 },
    { 0x45b17488, SUC_lexicon_tags_28 },
    { 0x1117487, SUC_lexicon_tags_7 },
    { 0x8991748a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d1748f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5117494, SUC_lexicon_tags_45 },
    { 0x7917495, SUC_lexicon_tags_42 },
    { 0xddd17495, SUC_lexicon_tags_0 },
    { 0xf7b17494, SUC_lexicon_tags_17 },
    { 0x38d17494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47f1749a, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517174a5, SUC_lexicon_tags_12 },
    { 0x5b3174a5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x12b174a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b174ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71174b2, SUC_lexicon_tags_63 },
    { 0x1bf174b3, SUC_lexicon_tags_37 },
    { 0xced174b4, SUC_lexicon_tags_4 },
    { 0xa7174b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f174b8, SUC_lexicon_tags_14 },
    { 0xfb5174b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77174bc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d174c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779174cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b174d0, SUC_lexicon_tags_39 },
    { 0xa11174d0, SUC_lexicon_tags_27 },
    { 0x57f174d2, SUC_lexicon_tags_37 },
    { 0x61b174d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85174d6, SUC_lexicon_tags_12 },
    { 0xe83174d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bd174d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe03174db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b174e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f9174e5, SUC_lexicon_tags_47 },
    { 0xb2b174e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79174ed, SUC_lexicon_tags_2 },
    { 0xdc9174ee, SUC_lexicon_tags_14 },
    { 0x3b7174ef, SUC_lexicon_tags_10 },
    { 0x823174ee, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd0b174f2, SUC_lexicon_tags_16 },
    { 0x459174f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x153174f5, SUC_lexicon_tags_63 },
    { 0x8af174f6, SUC_lexicon_tags_17 },
    { 0xa2b174f6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7174fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af17501, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3917508, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4771750a, SUC_lexicon_tags_1 },
    { 0xa91750a, SUC_lexicon_tags_11 },
    { 0xf291750b, SUC_lexicon_tags_8 },
    { 0x8ed1750c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d17513, SUC_lexicon_tags_23 },
    { 0xd8517514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5951751a, SUC_lexicon_tags_64 },
    { 0x7e91751b, SUC_lexicon_tags_37 },
    { 0xbd91751c, SUC_lexicon_tags_224 },
    { 0xcd51751c, SUC_lexicon_tags_14 },
    { 0xa251751a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50b17520, SUC_lexicon_tags_23 },
    { 0x89d17521, SUC_lexicon_tags_7 },
    { 0x9ff17522, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebb17524, SUC_lexicon_tags_37 },
    { 0xfbb17525, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb8717527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2651752e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a317532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bb17534, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaef17536, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5151753b, SUC_lexicon_tags_29 },
    { 0x7391753b, SUC_lexicon_tags_37 },
    { 0xc891753c, SUC_lexicon_tags_215 },
    { 0x73f1753b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f317545, SUC_lexicon_tags_23 },
    { 0x4cd17545, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2d917548, SUC_lexicon_tags_12 },
    { 0x33517549, SUC_lexicon_tags_29 },
    { 0xde71754a, SUC_lexicon_tags_37 },
    { 0x87b1754a, SUC_lexicon_tags_49 },
    { 0x40117549, SUC_lexicon_tags_2 },
    { 0xc111754d, SUC_lexicon_tags_47 },
    { 0x2c91754e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x50b17550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0917552, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa6b17554, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b917559, SUC_lexicon_tags_62 },
    { 0x76717559, SUC_lexicon_tags_7 },
    { 0x70f1755b, SUC_lexicon_tags_1 },
    { 0xaa31755b, SUC_lexicon_tags_11 },
    { 0xd731755d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3317560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2717562, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b17567, SUC_lexicon_tags_57 },
    { 0x5717568, SUC_lexicon_tags_47 },
    { 0xce317569, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b1756c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5517570, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf517573, SUC_lexicon_tags_7 },
    { 0x5317573, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55517579, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51b1757b, SUC_lexicon_tags_17 },
    { 0x8511757c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb517580, SUC_lexicon_tags_2 },
    { 0xdf117581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a91758b, SUC_lexicon_tags_17 },
    { 0xa3d1758c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63917590, SUC_lexicon_tags_4 },
    { 0xc0117591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16717596, SUC_lexicon_tags_0 },
    { 0x66d17597, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f175a1, SUC_lexicon_tags_17 },
    { 0xdeb175a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x349175a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f175aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7175ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x435175b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b175b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x579175b7, SUC_lexicon_tags_1 },
    { 0x681175b7, SUC_lexicon_tags_1659 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f175bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59175bf, SUC_lexicon_tags_37 },
    { 0xe79175c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69b175c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1175c7, SUC_lexicon_tags_23 },
    { 0x9c1175c7, SUC_lexicon_tags_42 },
    { 0x14b175c9, SUC_lexicon_tags_37 },
    { 0xe5175c9, SUC_lexicon_tags_25 },
    { 0x993175cb, SUC_lexicon_tags_26 },
    { 0x963175cc, SUC_lexicon_tags_1 },
    { 0xf5b175cd, SUC_lexicon_tags_7 },
    { 0x8b3175cd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5f9175d0, SUC_lexicon_tags_4 },
    { 0xb2d175d1, SUC_lexicon_tags_30 },
    { 0xdb175d1, SUC_lexicon_tags_37 },
    { 0xf11175d2, SUC_lexicon_tags_37 },
    { 0xd15175d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe61175d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16f175d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9175db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x859175de, SUC_lexicon_tags_37 },
    { 0x901175df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76b175e1, SUC_lexicon_tags_23 },
    { 0xa9b175e1, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x2c9175e4, SUC_lexicon_tags_15 },
    { 0xccf175e5, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x793175e8, SUC_lexicon_tags_17 },
    { 0xb71175e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d175ec, SUC_lexicon_tags_1 },
    { 0xf4b175ec, SUC_lexicon_tags_17 },
    { 0xb35175ee, SUC_lexicon_tags_28 },
    { 0x51b175ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55175f2, SUC_lexicon_tags_30 },
    { 0xff1175f3, SUC_lexicon_tags_17 },
    { 0x4b3175f3, SUC_lexicon_tags_19 },
    { 0xc33175f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21f175f7, SUC_lexicon_tags_1 },
    { 0x3e9175f7, SUC_lexicon_tags_11 },
    { 0x93175f9, SUC_lexicon_tags_45 },
    { 0x7cb175f9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa55175fc, SUC_lexicon_tags_11 },
    { 0x6a3175fc, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b17601, SUC_lexicon_tags_37 },
    { 0xab117601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90917607, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5bd17609, SUC_lexicon_tags_6 },
    { 0x88f1760a, SUC_lexicon_tags_37 },
    { 0x42117609, SUC_lexicon_tags_12 },
    { 0xb591760c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4531760e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2131761b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7517623, SUC_lexicon_tags_25 },
    { 0xf6717624, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29b17626, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda51762d, SUC_lexicon_tags_7 },
    { 0x58d1762e, SUC_lexicon_tags_11 },
    { 0x851762f, SUC_lexicon_tags_26 },
    { 0x8a31762d, SUC_lexicon_tags_46 },
    { 0x6471762f, SUC_lexicon_tags_43 },
    { 0x6a717631, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86717636, SUC_lexicon_tags_46 },
    { 0x71f17636, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36517639, SUC_lexicon_tags_1 },
    { 0x62d17639, SUC_lexicon_tags_12 },
    { 0xadd1763a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f1763f, SUC_lexicon_tags_19 },
    { 0x9111763f, SUC_lexicon_tags_29 },
    { 0x24b1763f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db17647, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9dd17649, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11764f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff17654, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcbd17656, SUC_lexicon_tags_2 },
    { 0xceb17656, SUC_lexicon_tags_2 },
    { 0x28d17658, SUC_lexicon_tags_13 },
    { 0x9d917658, SUC_lexicon_tags_46 },
    { 0xbcb17657, SUC_lexicon_tags_11 },
    { 0xa891765b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f1765e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b17665, SUC_lexicon_tags_2 },
    { 0x31717666, SUC_lexicon_tags_2 },
    { 0x9fd17667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ff17669, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b917670, SUC_lexicon_tags_84 },
    { 0x8d917670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28117674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b17678, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf91767a, SUC_lexicon_tags_46 },
    { 0x6011767a, SUC_lexicon_tags_37 },
    { 0x9511767c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x651767e, SUC_lexicon_tags_2 },
    { 0x5d91767f, SUC_lexicon_tags_1 },
    { 0x4e317680, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4b917682, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9317684, SUC_lexicon_tags_2 },
    { 0x63117685, SUC_lexicon_tags_13 },
    { 0x70f17686, SUC_lexicon_tags_18 },
    { 0x7b117687, SUC_lexicon_tags_79 },
    { 0x24517688, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c91768b, SUC_lexicon_tags_0 },
    { 0x8711768c, SUC_lexicon_tags_25 },
    { 0xd8f1768d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x511768f, SUC_lexicon_tags_16 },
    { 0x68b17690, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe917692, SUC_lexicon_tags_6 },
    { 0xacf17693, SUC_lexicon_tags_396 },
    { 0xe3b17694, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe151769d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce51769f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f9176a1, SUC_lexicon_tags_26 },
    { 0xd77176a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x805176a5, SUC_lexicon_tags_12 },
    { 0xdfb176a6, SUC_lexicon_tags_29 },
    { 0x519176a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd176af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f176b2, SUC_lexicon_tags_7 },
    { 0xa7176b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d176b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa0f176ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7c3176bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd176c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb176c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d1176c8, SUC_lexicon_tags_2 },
    { 0x94b176c8, SUC_lexicon_tags_37 },
    { 0x18f176c9, SUC_lexicon_tags_160 },
    { 0xaa3176cb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x741176cd, SUC_lexicon_tags_14 },
    { 0xd47176ce, SUC_lexicon_tags_60 },
    { 0x6b176ce, SUC_lexicon_tags_37 },
    { 0x1f1176d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5176d4, SUC_lexicon_tags_25 },
    { 0x3b9176d5, SUC_lexicon_tags_45 },
    { 0x9f9176d6, SUC_lexicon_tags_12 },
    { 0x1ff176d7, SUC_lexicon_tags_30 },
    { 0xac5176d5, SUC_lexicon_tags_25 },
    { 0x5b1176d5, SUC_lexicon_tags_11 },
    { 0xd43176da, SUC_lexicon_tags_68 },
    { 0x8bf176db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a1176dd, SUC_lexicon_tags_37 },
    { 0xdf1176dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9176e5, SUC_lexicon_tags_63 },
    { 0x1d9176e5, SUC_lexicon_tags_25 },
    { 0xf1f176e6, SUC_lexicon_tags_0 },
    { 0x571176e8, SUC_lexicon_tags_11 },
    { 0x7a5176e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b176ec, SUC_lexicon_tags_1 },
    { 0x6e5176ec, SUC_lexicon_tags_19 },
    { 0x283176ee, SUC_lexicon_tags_3 },
    { 0x7fd176ee, SUC_lexicon_tags_26 },
    { 0xca3176f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d176f3, SUC_lexicon_tags_63 },
    { 0x9e7176f3, SUC_lexicon_tags_37 },
    { 0x66f176f3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7176f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab17703, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f17709, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67517714, SUC_lexicon_tags_15 },
    { 0x94d17714, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a317717, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc11771c, SUC_lexicon_tags_47 },
    { 0xf5f1771d, SUC_lexicon_tags_37 },
    { 0x2091771e, SUC_lexicon_tags_10 },
    { 0x30f1771d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13117723, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5671772b, SUC_lexicon_tags_15 },
    { 0xa571772c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4f1772e, SUC_lexicon_tags_16 },
    { 0xd091772f, SUC_lexicon_tags_27 },
    { 0x2cd17730, SUC_lexicon_tags_31 },
    { 0x5cb17731, SUC_lexicon_tags_10 },
    { 0xed17732, SUC_lexicon_tags_10 },
    { 0xa691772f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb517738, SUC_lexicon_tags_2 },
    { 0x3b317739, SUC_lexicon_tags_8 },
    { 0xd0b1773a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab17741, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71317743, SUC_lexicon_tags_15 },
    { 0xa4f17744, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6117746, SUC_lexicon_tags_1 },
    { 0xc7b17747, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf9b17749, SUC_lexicon_tags_29 },
    { 0x9b1774a, SUC_lexicon_tags_37 },
    { 0x9051774b, SUC_lexicon_tags_12 },
    { 0xc471774a, SUC_lexicon_tags_32 },
    { 0xe071774a, SUC_lexicon_tags_23 },
    { 0x65f1774b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99917750, SUC_lexicon_tags_12 },
    { 0x95317751, SUC_lexicon_tags_38 },
    { 0x24917752, SUC_lexicon_tags_17 },
    { 0xa7917751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf17758, SUC_lexicon_tags_7 },
    { 0x93d17758, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b91775e, SUC_lexicon_tags_2 },
    { 0xe6d1775f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9f317761, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb17764, SUC_lexicon_tags_12 },
    { 0x5c317765, SUC_lexicon_tags_11 },
    { 0x35917765, SUC_lexicon_tags_10 },
    { 0xcf917767, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911776c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa117773, SUC_lexicon_tags_46 },
    { 0xfa317774, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f17778, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b11777e, SUC_lexicon_tags_8 },
    { 0x6131777f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d17786, SUC_lexicon_tags_17 },
    { 0x83d17787, SUC_lexicon_tags_6 },
    { 0x1fb17787, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x411778a, SUC_lexicon_tags_84 },
    { 0xef91778b, SUC_lexicon_tags_2 },
    { 0x7691778c, SUC_lexicon_tags_7 },
    { 0x6d71778d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d17791, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb9b17793, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa117799, SUC_lexicon_tags_11 },
    { 0xc471779a, SUC_lexicon_tags_37 },
    { 0x8011779a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7177a0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9177a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf177a6, SUC_lexicon_tags_32 },
    { 0x5b177a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7177ad, SUC_lexicon_tags_37 },
    { 0x1af177ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b177b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725177b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb177bd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19177c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d177c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb177cc, SUC_lexicon_tags_60 },
    { 0x599177cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53177d1, SUC_lexicon_tags_12 },
    { 0x433177d2, SUC_lexicon_tags_187 },
    { 0xce7177d3, SUC_lexicon_tags_8 },
    { 0x9cb177d2, SUC_lexicon_tags_1 },
    { 0xd75177d5, SUC_lexicon_tags_17 },
    { 0xb3177d6, SUC_lexicon_tags_25 },
    { 0x69b177d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1177e0, SUC_lexicon_tags_31 },
    { 0xbf3177e1, SUC_lexicon_tags_21 },
    { 0xf7b177e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10f177e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd3177e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa6b177e8, SUC_lexicon_tags_39 },
    { 0x3ef177e9, SUC_lexicon_tags_17 },
    { 0x54f177e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb21177ec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7ad177ee, SUC_lexicon_tags_31 },
    { 0xb7f177ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa53177f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73177f4, SUC_lexicon_tags_18 },
    { 0x8b1177f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3177fb, SUC_lexicon_tags_30 },
    { 0x39177fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87177ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34117801, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72317804, SUC_lexicon_tags_1516 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd17807, SUC_lexicon_tags_43 },
    { 0xb8117808, SUC_lexicon_tags_17 },
    { 0x59d17809, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1d71780b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb1780e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10117810, SUC_lexicon_tags_49 },
    { 0x4a917811, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8717817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b1781b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18717823, SUC_lexicon_tags_87 },
    { 0x98f17823, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c11782c, SUC_lexicon_tags_8 },
    { 0xae51782c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2117834, SUC_lexicon_tags_17 },
    { 0x89917834, SUC_lexicon_tags_55 },
    { 0x11117836, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1117839, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1783c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8771783e, SUC_lexicon_tags_82 },
    { 0x3871783e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad17843, SUC_lexicon_tags_10 },
    { 0xf9d17844, SUC_lexicon_tags_32 },
    { 0x11f17845, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1d17847, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac717849, SUC_lexicon_tags_27 },
    { 0x92b1784a, SUC_lexicon_tags_43 },
    { 0x9af1784b, SUC_lexicon_tags_37 },
    { 0xfdb1784c, SUC_lexicon_tags_15 },
    { 0xc431784d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37717851, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x32117853, SUC_lexicon_tags_15 },
    { 0xcb317853, SUC_lexicon_tags_27 },
    { 0x8b517855, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1971785c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51517860, SUC_lexicon_tags_45 },
    { 0x26317861, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c317865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f117867, SUC_lexicon_tags_29 },
    { 0x99717868, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb717878, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd1787c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed1787f, SUC_lexicon_tags_25 },
    { 0x8971787f, SUC_lexicon_tags_60 },
    { 0xbdb1787f, SUC_lexicon_tags_12 },
    { 0xe7517880, SUC_lexicon_tags_37 },
    { 0x68117883, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed117885, SUC_lexicon_tags_25 },
    { 0x21917885, SUC_lexicon_tags_12 },
    { 0x35d17885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f917889, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7871788e, SUC_lexicon_tags_12 },
    { 0x4671788f, SUC_lexicon_tags_23 },
    { 0x7d317890, SUC_lexicon_tags_12 },
    { 0x56717891, SUC_lexicon_tags_17 },
    { 0x4a717891, SUC_lexicon_tags_11 },
    { 0xe6717891, SUC_lexicon_tags_12 },
    { 0x3317894, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa7b17896, SUC_lexicon_tags_38 },
    { 0x74917897, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe711789d, SUC_lexicon_tags_43 },
    { 0xe611789e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7178a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b178a4, SUC_lexicon_tags_152 },
    { 0x3f1178a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741178ac, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xbe3178ae, SUC_lexicon_tags_60 },
    { 0x8ff178af, SUC_lexicon_tags_4 },
    { 0xa2b178af, SUC_lexicon_tags_19 },
    { 0x115178b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb03178b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381178b8, SUC_lexicon_tags_32 },
    { 0xc61178b9, SUC_lexicon_tags_37 },
    { 0x131178ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x955178bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05178c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f178c7, SUC_lexicon_tags_37 },
    { 0xe43178c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x235178ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8b1178cc, SUC_lexicon_tags_15 },
    { 0x975178cd, SUC_lexicon_tags_49 },
    { 0xb91178cc, SUC_lexicon_tags_53 },
    { 0xcbf178cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb178d3, SUC_lexicon_tags_28 },
    { 0xf53178d4, SUC_lexicon_tags_8 },
    { 0x76b178d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9178d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61f178da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b178de, SUC_lexicon_tags_15 },
    { 0xed178de, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x99d178e1, SUC_lexicon_tags_33 },
    { 0xf9f178e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe93178e4, SUC_lexicon_tags_27 },
    { 0x22b178e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5178e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b1178e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7178ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181178f3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef178fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6717902, SUC_lexicon_tags_37 },
    { 0x1ed17902, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c917905, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4717908, SUC_lexicon_tags_7 },
    { 0x45317909, SUC_lexicon_tags_8 },
    { 0x9371790a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48f1790c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2911790e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50f17910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d317912, SUC_lexicon_tags_26 },
    { 0x94517912, SUC_lexicon_tags_1 },
    { 0xccf17913, SUC_lexicon_tags_25 },
    { 0x28717913, SUC_lexicon_tags_26 },
    { 0x14f17916, SUC_lexicon_tags_30 },
    { 0x6b17917, SUC_lexicon_tags_43 },
    { 0x94d17913, SUC_lexicon_tags_7 },
    { 0xa3717919, SUC_lexicon_tags_945 },
    { 0x8f91791a, SUC_lexicon_tags_33 },
    { 0x1911791a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8131791f, SUC_lexicon_tags_0 },
    { 0x67117920, SUC_lexicon_tags_17 },
    { 0x41f1791f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6517927, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdcf17929, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7651792b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc471792e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd917936, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x15117939, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b1793d, SUC_lexicon_tags_11 },
    { 0x3d1793d, SUC_lexicon_tags_2 },
    { 0xf7f1793f, SUC_lexicon_tags_37 },
    { 0x2b317940, SUC_lexicon_tags_30 },
    { 0x14717941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe517946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55d17948, SUC_lexicon_tags_11 },
    { 0xbf917949, SUC_lexicon_tags_14 },
    { 0x89317948, SUC_lexicon_tags_82 },
    { 0x8ef1794b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed17950, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38f17952, SUC_lexicon_tags_25 },
    { 0x4a717953, SUC_lexicon_tags_10 },
    { 0x3117954, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b1795a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d17961, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0x9517963, SUC_lexicon_tags_10 },
    { 0xf7f17964, SUC_lexicon_tags_38 },
    { 0xbd717965, SUC_lexicon_tags_7 },
    { 0x3d717963, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45517969, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0f1796b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d1796e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad117975, SUC_lexicon_tags_63 },
    { 0xb9117976, SUC_lexicon_tags_17 },
    { 0xa9717976, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe351797c, SUC_lexicon_tags_64 },
    { 0x311797d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1717981, SUC_lexicon_tags_15 },
    { 0xf2317981, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2391798b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8971798d, SUC_lexicon_tags_49 },
    { 0xd8b1798d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb711799b, SUC_lexicon_tags_47 },
    { 0x1151799c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b179a3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8b3179a5, SUC_lexicon_tags_9 },
    { 0xb7b179a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x301179a8, SUC_lexicon_tags_39 },
    { 0x41179a9, SUC_lexicon_tags_7 },
    { 0xe17179a9, SUC_lexicon_tags_32 },
    { 0x79179a8, SUC_lexicon_tags_9 },
    { 0x27b179a8, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb179b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6b3179b5, SUC_lexicon_tags_1 },
    { 0xc63179b6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x723179b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf179bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5179c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd179c6, SUC_lexicon_tags_12 },
    { 0x6ab179c6, SUC_lexicon_tags_17 },
    { 0x433179c8, SUC_lexicon_tags_15 },
    { 0x3a7179c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xddd179cb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x33179ce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d179d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x911179d4, SUC_lexicon_tags_45 },
    { 0xabd179d5, SUC_lexicon_tags_45 },
    { 0x2a5179d5, SUC_lexicon_tags_25 },
    { 0xa4d179d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841179dc, SUC_lexicon_tags_27 },
    { 0x119179dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf179e0, SUC_lexicon_tags_15 },
    { 0xce9179e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x329179e3, SUC_lexicon_tags_25 },
    { 0x30f179e4, SUC_lexicon_tags_61 },
    { 0xe3d179e4, SUC_lexicon_tags_35 },
    { 0xc7d179e6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b179ed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc89179ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9179f4, SUC_lexicon_tags_55 },
    { 0x2ed179f5, SUC_lexicon_tags_37 },
    { 0xb95179f4, SUC_lexicon_tags_29 },
    { 0x91f179f7, SUC_lexicon_tags_9 },
    { 0x317179f8, SUC_lexicon_tags_7 },
    { 0x339179f7, SUC_lexicon_tags_25 },
    { 0x873179fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x167179fc, SUC_lexicon_tags_63 },
    { 0x197179fd, SUC_lexicon_tags_2 },
    { 0x3d3179fe, SUC_lexicon_tags_37 },
    { 0xf1d179fc, SUC_lexicon_tags_61 },
    { 0x4a717a00, SUC_lexicon_tags_2 },
    { 0x515179fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df17a08, SUC_lexicon_tags_54 },
    { 0xd117a09, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b17a0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d17a12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a517a1a, SUC_lexicon_tags_438 },
    { 0x22317a1a, SUC_lexicon_tags_25 },
    { 0xc6b17a1c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x88717a1e, SUC_lexicon_tags_31 },
    { 0x29117a1f, SUC_lexicon_tags_0 },
    { 0x88d17a20, SUC_lexicon_tags_11 },
    { 0xd4d17a1e, SUC_lexicon_tags_33 },
    { 0x48117a22, SUC_lexicon_tags_45 },
    { 0x32f17a23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d17a26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14117a29, SUC_lexicon_tags_42 },
    { 0x39317a29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd17a2e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x67117a30, SUC_lexicon_tags_17 },
    { 0xc0917a30, SUC_lexicon_tags_7 },
    { 0xc5117a30, SUC_lexicon_tags_12 },
    { 0x7a917a32, SUC_lexicon_tags_25 },
    { 0x53f17a34, SUC_lexicon_tags_63 },
    { 0x9517a35, SUC_lexicon_tags_29 },
    { 0x43917a36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c717a3c, SUC_lexicon_tags_8 },
    { 0x71917a3d, SUC_lexicon_tags_17 },
    { 0xfe717a3c, SUC_lexicon_tags_25 },
    { 0xf5317a3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6717a44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3b17a46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f517a49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb17a4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xacb17a4f, SUC_lexicon_tags_61 },
    { 0xdd517a50, SUC_lexicon_tags_14 },
    { 0x4ab17a50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff17a55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8917a57, SUC_lexicon_tags_25 },
    { 0xf5f17a58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4917a5b, SUC_lexicon_tags_15 },
    { 0x81917a5b, SUC_lexicon_tags_1 },
    { 0xcc517a5b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0517a61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97117a63, SUC_lexicon_tags_37 },
    { 0x5b517a63, SUC_lexicon_tags_49 },
    { 0xdbf17a63, SUC_lexicon_tags_9 },
    { 0x1a117a66, SUC_lexicon_tags_29 },
    { 0x35717a67, SUC_lexicon_tags_131 },
    { 0x99917a63, SUC_lexicon_tags_28 },
    { 0x62517a69, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd17a6f, SUC_lexicon_tags_3 },
    { 0xb9317a6f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x48717a72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefb17a74, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9517a77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccd17a79, SUC_lexicon_tags_1 },
    { 0x7e917a7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x56317a7c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64917a80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0517a84, SUC_lexicon_tags_63 },
    { 0x39117a85, SUC_lexicon_tags_1 },
    { 0xe2917a84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45117a8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc117a8f, SUC_lexicon_tags_21 },
    { 0xc7d17a90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x55317a93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f17aa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde917aa6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad17aa8, SUC_lexicon_tags_12 },
    { 0x25d17aa9, SUC_lexicon_tags_37 },
    { 0xdf17aa9, SUC_lexicon_tags_37 },
    { 0x7ff17aa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f317aad, SUC_lexicon_tags_32 },
    { 0x88d17aad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9917ab1, SUC_lexicon_tags_26 },
    { 0xa917ab2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56717ab4, SUC_lexicon_tags_46 },
    { 0x42917ab4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5b17ab7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d17abe, SUC_lexicon_tags_15 },
    { 0x3cf17abf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f117ac5, SUC_lexicon_tags_14 },
    { 0x34517ac6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b17acb, SUC_lexicon_tags_25 },
    { 0x89117acc, SUC_lexicon_tags_37 },
    { 0x26f17acd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f17ad9, SUC_lexicon_tags_12 },
    { 0x3dd17ada, SUC_lexicon_tags_37 },
    { 0x5b917adb, SUC_lexicon_tags_25 },
    { 0x1a317adc, SUC_lexicon_tags_8 },
    { 0x63b17add, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1517adf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x76117ae1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe917aef, SUC_lexicon_tags_7 },
    { 0xb0117af0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf17af3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4317af5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec517af8, SUC_lexicon_tags_17 },
    { 0xa0f17af9, SUC_lexicon_tags_1 },
    { 0x1d717af8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b17afe, SUC_lexicon_tags_68 },
    { 0xd4f17aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb17b04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b17b09, SUC_lexicon_tags_12 },
    { 0x5ef17b0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f317b0c, SUC_lexicon_tags_23 },
    { 0xcf917b0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3717b10, SUC_lexicon_tags_49 },
    { 0xb0517b11, SUC_lexicon_tags_25 },
    { 0x6bd17b11, SUC_lexicon_tags_11 },
    { 0xced17b13, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x90317b16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8317b1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaef17b1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a917b1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef17b22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a717b24, SUC_lexicon_tags_11 },
    { 0x81717b25, SUC_lexicon_tags_15 },
    { 0x22317b26, SUC_lexicon_tags_16 },
    { 0x31117b24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed17b2a, SUC_lexicon_tags_17 },
    { 0x1b717b2b, SUC_lexicon_tags_12 },
    { 0x82f17b2c, SUC_lexicon_tags_45 },
    { 0x94b17b2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e717b2f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1517b34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e317b37, SUC_lexicon_tags_43 },
    { 0x7af17b38, SUC_lexicon_tags_15 },
    { 0xba117b39, SUC_lexicon_tags_11 },
    { 0x33317b38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4917b3e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8e517b40, SUC_lexicon_tags_33 },
    { 0x3a917b40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b117b46, SUC_lexicon_tags_15 },
    { 0x59d17b47, SUC_lexicon_tags_2 },
    { 0xa4717b48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d17b4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ed17b4e, SUC_lexicon_tags_28 },
    { 0xea317b4e, SUC_lexicon_tags_47 },
    { 0x5917b50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x25317b53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67317b56, SUC_lexicon_tags_45 },
    { 0x44717b57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf17b5a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcd17b5c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d17b5f, SUC_lexicon_tags_37 },
    { 0x74f17b60, SUC_lexicon_tags_40 },
    { 0x40317b61, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b17b64, SUC_lexicon_tags_37 },
    { 0x54f17b65, SUC_lexicon_tags_18 },
    { 0x8fd17b66, SUC_lexicon_tags_12 },
    { 0xe2d17b67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2cd17b69, SUC_lexicon_tags_55 },
    { 0x60d17b6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37317b6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf17b72, SUC_lexicon_tags_25 },
    { 0x92117b73, SUC_lexicon_tags_27 },
    { 0x6f117b73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4117b78, SUC_lexicon_tags_2 },
    { 0x32517b79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed17b7d, SUC_lexicon_tags_14 },
    { 0x9e517b7d, SUC_lexicon_tags_6 },
    { 0x6a717b7f, SUC_lexicon_tags_15 },
    { 0xd117b80, SUC_lexicon_tags_31 },
    { 0xadb17b81, SUC_lexicon_tags_29 },
    { 0x57f17b81, SUC_lexicon_tags_2 },
    { 0x9dd17b82, SUC_lexicon_tags_15 },
    { 0xe4717b82, SUC_lexicon_tags_48 },
    { 0xe1317b82, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d17b8a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5317b93, SUC_lexicon_tags_47 },
    { 0x3b717b94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44917b97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5517b9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed917ba1, SUC_lexicon_tags_27 },
    { 0xecf17ba1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d17ba5, SUC_lexicon_tags_26 },
    { 0x2f717ba6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76317bac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf517baf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d17bb2, SUC_lexicon_tags_37 },
    { 0x7b917bb2, SUC_lexicon_tags_45 },
    { 0xb2717bb4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c717bb7, SUC_lexicon_tags_11 },
    { 0x3b517bb7, SUC_lexicon_tags_18 },
    { 0xa6717bb7, SUC_lexicon_tags_1 },
    { 0x37f17bb7, SUC_lexicon_tags_45 },
    { 0x9a717bb7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c317bbe, SUC_lexicon_tags_68 },
    { 0x51117bbf, SUC_lexicon_tags_7 },
    { 0x60f17bc0, SUC_lexicon_tags_25 },
    { 0x39117bc1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8f17bc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f117bc6, SUC_lexicon_tags_12 },
    { 0xb9717bc7, SUC_lexicon_tags_9 },
    { 0xbf717bc8, SUC_lexicon_tags_1 },
    { 0xf5f17bc7, SUC_lexicon_tags_37 },
    { 0xbc317bc9, SUC_lexicon_tags_15 },
    { 0xf3f17bcb, SUC_lexicon_tags_7 },
    { 0xeb517bc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98517bd8, SUC_lexicon_tags_47 },
    { 0xbc717bd9, SUC_lexicon_tags_37 },
    { 0x117bda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9517bdc, SUC_lexicon_tags_12 },
    { 0x21317bdd, SUC_lexicon_tags_15 },
    { 0x7717bde, SUC_lexicon_tags_45 },
    { 0xfe917bdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b17be5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x87317be8, SUC_lexicon_tags_38 },
    { 0x2a117be8, SUC_lexicon_tags_39 },
    { 0xf2f17be9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x40517bed, SUC_lexicon_tags_11 },
    { 0xaa317bee, SUC_lexicon_tags_1 },
    { 0x3f17bed, SUC_lexicon_tags_6 },
    { 0x9f717bf0, SUC_lexicon_tags_12 },
    { 0x7bd17bf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a117bf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad17bfa, SUC_lexicon_tags_17 },
    { 0x8117bfa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x38917bfe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df17c03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b17c06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d17c09, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b917c0d, SUC_lexicon_tags_68 },
    { 0x20f17c0e, SUC_lexicon_tags_26 },
    { 0xb7b17c0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6717c11, SUC_lexicon_tags_29 },
    { 0x24b17c11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb17c15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9517c1b, SUC_lexicon_tags_11 },
    { 0xf9d17c1c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x9317c1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c517c22, SUC_lexicon_tags_36 },
    { 0x53f17c22, SUC_lexicon_tags_17 },
    { 0x76117c23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94117c26, SUC_lexicon_tags_11 },
    { 0x3a117c27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35917c2a, SUC_lexicon_tags_37 },
    { 0xe8d17c2b, SUC_lexicon_tags_12 },
    { 0x43117c2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd17c33, SUC_lexicon_tags_47 },
    { 0xeb917c34, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0x3f717c36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c317c38, SUC_lexicon_tags_11 },
    { 0x55b17c39, SUC_lexicon_tags_7 },
    { 0x717c38, SUC_lexicon_tags_13 },
    { 0x7bd17c3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcf117c3d, SUC_lexicon_tags_37 },
    { 0x85f17c3d, SUC_lexicon_tags_28 },
    { 0x3bd17c3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23117c41, SUC_lexicon_tags_47 },
    { 0xa9917c42, SUC_lexicon_tags_37 },
    { 0x6d17c43, SUC_lexicon_tags_7 },
    { 0x5fd17c42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ad17c46, SUC_lexicon_tags_25 },
    { 0xed17c47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6eb17c49, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f17c4d, SUC_lexicon_tags_84 },
    { 0xb2317c4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d917c56, SUC_lexicon_tags_33 },
    { 0xa5517c56, SUC_lexicon_tags_4 },
    { 0x72917c58, SUC_lexicon_tags_17 },
    { 0x23117c59, SUC_lexicon_tags_68 },
    { 0x2117c5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe317c5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46917c60, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d17c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b17c68, SUC_lexicon_tags_2 },
    { 0x7bb17c68, SUC_lexicon_tags_37 },
    { 0x7bd17c68, SUC_lexicon_tags_37 },
    { 0x77b17c6b, SUC_lexicon_tags_37 },
    { 0x63b17c6c, SUC_lexicon_tags_11 },
    { 0xdd17c6d, SUC_lexicon_tags_34 },
    { 0x27117c6d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d17c72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f17c77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ab17c79, SUC_lexicon_tags_37 },
    { 0x59917c7a, SUC_lexicon_tags_26 },
    { 0xbfb17c7a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c317c7f, SUC_lexicon_tags_23 },
    { 0x1af17c7f, SUC_lexicon_tags_48 },
    { 0x86117c81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd17c84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa517c8a, SUC_lexicon_tags_1 },
    { 0xf5517c8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af17c8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a517c91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19b17c93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4317c95, SUC_lexicon_tags_15 },
    { 0xfc517c96, SUC_lexicon_tags_45 },
    { 0x4f517c96, SUC_lexicon_tags_2 },
    { 0xd6917c97, SUC_lexicon_tags_45 },
    { 0x30717c97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x91517c9b, SUC_lexicon_tags_2 },
    { 0xe6f17c9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbd17c9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd117ca1, SUC_lexicon_tags_25 },
    { 0xcc517ca1, SUC_lexicon_tags_47 },
    { 0xaa317ca3, SUC_lexicon_tags_16 },
    { 0xa9917ca4, SUC_lexicon_tags_7 },
    { 0x36917ca5, SUC_lexicon_tags_17 },
    { 0xe5f17ca4, SUC_lexicon_tags_37 },
    { 0x87f17ca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d17caa, SUC_lexicon_tags_3 },
    { 0x1e917cab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a117cb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed717cb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5517cbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d17cbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6317cc0, SUC_lexicon_tags_48 },
    { 0xaeb17cc1, SUC_lexicon_tags_6 },
    { 0x12d17cc2, SUC_lexicon_tags_69 },
    { 0x30517cc1, SUC_lexicon_tags_17 },
    { 0x82317cc0, SUC_lexicon_tags_30 },
    { 0x99317cc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef17ccc, SUC_lexicon_tags_25 },
    { 0x94317ccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed517ccf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93917cd2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec917cd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76d17cd9, SUC_lexicon_tags_15 },
    { 0x77517cda, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5717cdf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6a117ce1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd9917ce3, SUC_lexicon_tags_60 },
    { 0x3bb17ce4, SUC_lexicon_tags_45 },
    { 0x63717ce5, SUC_lexicon_tags_37 },
    { 0x4ab17ce5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8ed17ce8, SUC_lexicon_tags_15 },
    { 0xcc917ce8, SUC_lexicon_tags_12 },
    { 0x85917cea, SUC_lexicon_tags_23 },
    { 0x4d717ce8, SUC_lexicon_tags_27 },
    { 0x53b17ce9, SUC_lexicon_tags_39 },
    { 0xeb17ced, SUC_lexicon_tags_23 },
    { 0xbd17cea, SUC_lexicon_tags_60 },
    { 0x51d17ced, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d17cf2, SUC_lexicon_tags_1 },
    { 0xd917cf3, SUC_lexicon_tags_11 },
    { 0x48117cf4, SUC_lexicon_tags_10 },
    { 0x3f717cf5, SUC_lexicon_tags_31 },
    { 0x87d17cf6, SUC_lexicon_tags_45 },
    { 0x9f717cf7, SUC_lexicon_tags_12 },
    { 0xfa317cf5, SUC_lexicon_tags_17 },
    { 0xd3717cf9, SUC_lexicon_tags_2 },
    { 0x2f917cf9, SUC_lexicon_tags_37 },
    { 0x2a317cf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e517cff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb17d02, SUC_lexicon_tags_10 },
    { 0x97117d03, SUC_lexicon_tags_12 },
    { 0x36d17d04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x72117d07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d17d0c, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0xabd17d0e, SUC_lexicon_tags_19 },
    { 0x20f17d0e, SUC_lexicon_tags_4 },
    { 0x74717d10, SUC_lexicon_tags_1 },
    { 0x29517d11, SUC_lexicon_tags_17 },
    { 0x2f317d0f, SUC_lexicon_tags_28 },
    { 0xad517d13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d17d17, SUC_lexicon_tags_14 },
    { 0x88d17d18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x75517d1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4b17d1c, SUC_lexicon_tags_15 },
    { 0xff117d1c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x64f17d1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2f117d21, SUC_lexicon_tags_17 },
    { 0x1cb17d22, SUC_lexicon_tags_7 },
    { 0xe8317d22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57717d29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87717d2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x51717d2f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcd917d31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbcb17d33, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8517d35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb17d37, SUC_lexicon_tags_45 },
    { 0x3ed17d37, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x8c517d3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7317d3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac117d3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d17d43, SUC_lexicon_tags_11 },
    { 0x1df17d43, SUC_lexicon_tags_19 },
    { 0xd5d17d44, SUC_lexicon_tags_13 },
    { 0x4d17d46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc517d49, SUC_lexicon_tags_23 },
    { 0x53b17d4a, SUC_lexicon_tags_37 },
    { 0xedb17d4b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd0f17d4d, SUC_lexicon_tags_2 },
    { 0x6eb17d4e, SUC_lexicon_tags_1 },
    { 0x1cd17d4f, SUC_lexicon_tags_45 },
    { 0x4f917d4e, SUC_lexicon_tags_37 },
    { 0x3fd17d51, SUC_lexicon_tags_12 },
    { 0xfb117d51, SUC_lexicon_tags_33 },
    { 0x4fb17d4e, SUC_lexicon_tags_26 },
    { 0xa9f17d54, SUC_lexicon_tags_34 },
    { 0xdbb17d4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa3f17d57, SUC_lexicon_tags_26 },
    { 0x8fb17d57, SUC_lexicon_tags_1 },
    { 0x5b17d58, SUC_lexicon_tags_23 },
    { 0x1c517d59, SUC_lexicon_tags_33 },
    { 0x5fb17d57, SUC_lexicon_tags_12 },
    { 0x9d17d5c, SUC_lexicon_tags_1 },
    { 0x37117d5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d117d61, SUC_lexicon_tags_23 },
    { 0xb1117d61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b117d69, SUC_lexicon_tags_245 },
    { 0x34517d6a, SUC_lexicon_tags_23 },
    { 0x49d17d69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff17d71, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55717d75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc517d78, SUC_lexicon_tags_29 },
    { 0xce317d79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c917d7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb17d7f, SUC_lexicon_tags_65 },
    { 0x53717d7f, SUC_lexicon_tags_0 },
    { 0xa0b17d81, SUC_lexicon_tags_27 },
    { 0x96317d82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d17d88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x47917d8a, SUC_lexicon_tags_28 },
    { 0x12d17d8b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80517d93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ef17d95, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x77717d97, SUC_lexicon_tags_25 },
    { 0xd3717d98, SUC_lexicon_tags_1 },
    { 0x1d317d99, SUC_lexicon_tags_29 },
    { 0x7517d99, SUC_lexicon_tags_11 },
    { 0xd6717d9b, SUC_lexicon_tags_60 },
    { 0x3cf17d9c, SUC_lexicon_tags_47 },
    { 0x59517d9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb717da0, SUC_lexicon_tags_17 },
    { 0x70b17da1, SUC_lexicon_tags_37 },
    { 0x20917da2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x94b17da4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d717da7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3117daa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfff17dac, SUC_lexicon_tags_45 },
    { 0x5d117dad, SUC_lexicon_tags_6 },
    { 0xfc917dac, SUC_lexicon_tags_37 },
    { 0x7d117daf, SUC_lexicon_tags_2 },
    { 0x9e717dac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7317db4, SUC_lexicon_tags_30 },
    { 0xe6d17db5, SUC_lexicon_tags_6 },
    { 0xc2117db6, SUC_lexicon_tags_25 },
    { 0xa7f17db6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94517dc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c517dc6, SUC_lexicon_tags_19 },
    { 0xbbd17dc6, SUC_lexicon_tags_26 },
    { 0xabb17dc8, SUC_lexicon_tags_15 },
    { 0xc8d17dc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18717dcc, SUC_lexicon_tags_3 },
    { 0xb8517dcc, SUC_lexicon_tags_31 },
    { 0x56d17dce, SUC_lexicon_tags_12 },
    { 0xeff17dce, SUC_lexicon_tags_16 },
    { 0xd3317dd0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0917dd3, SUC_lexicon_tags_23 },
    { 0x72f17dd4, SUC_lexicon_tags_15 },
    { 0x16f17dd5, SUC_lexicon_tags_138 },
    { 0x1f917dd3, SUC_lexicon_tags_1 },
    { 0xa4317dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d317dda, SUC_lexicon_tags_15 },
    { 0x8f917ddb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c117ddf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917de4, SUC_lexicon_tags_15 },
    { 0xe0717de5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90517de7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7517ded, SUC_lexicon_tags_11 },
    { 0x27f17dee, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xcf317df0, SUC_lexicon_tags_12 },
    { 0x1c917df1, SUC_lexicon_tags_11 },
    { 0x1b517df1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1917df5, SUC_lexicon_tags_37 },
    { 0x6b117df6, SUC_lexicon_tags_36 },
    { 0xfc517df7, SUC_lexicon_tags_37 },
    { 0xc1117df6, SUC_lexicon_tags_65 },
    { 0x59917df9, SUC_lexicon_tags_11 },
    { 0x6a517dfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f17dfd, SUC_lexicon_tags_26 },
    { 0x7bf17dfe, SUC_lexicon_tags_12 },
    { 0x8517dff, SUC_lexicon_tags_25 },
    { 0x4ad17e00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0517e02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e117e04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe917e06, SUC_lexicon_tags_60 },
    { 0xcbb17e07, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x48717e09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f17e0c, SUC_lexicon_tags_14 },
    { 0x60d17e0d, SUC_lexicon_tags_27 },
    { 0xf6517e0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24317e12, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b17e16, SUC_lexicon_tags_37 },
    { 0x3f17e16, SUC_lexicon_tags_38 },
    { 0xbf117e18, SUC_lexicon_tags_37 },
    { 0x54717e19, SUC_lexicon_tags_49 },
    { 0xb0917e1a, SUC_lexicon_tags_8 },
    { 0x9b517e17, SUC_lexicon_tags_63 },
    { 0x11317e16, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b17e1f, SUC_lexicon_tags_23 },
    { 0x6c517e20, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x69317e23, SUC_lexicon_tags_79 },
    { 0x54517e24, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3117e2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d317e32, SUC_lexicon_tags_1 },
    { 0x3ef17e32, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x82517e36, SUC_lexicon_tags_38 },
    { 0xd1d17e37, SUC_lexicon_tags_29 },
    { 0xc7d17e38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb317e3a, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x36517e3d, SUC_lexicon_tags_0 },
    { 0xdb517e3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb917e40, SUC_lexicon_tags_85 },
    { 0xf0f17e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef17e45, SUC_lexicon_tags_47 },
    { 0x3ed17e46, SUC_lexicon_tags_63 },
    { 0xc2f17e46, SUC_lexicon_tags_71 },
    { 0x9e117e48, SUC_lexicon_tags_15 },
    { 0x6c917e46, SUC_lexicon_tags_32 },
    { 0x8917e47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71317e4f, SUC_lexicon_tags_9 },
    { 0x22d17e50, SUC_lexicon_tags_43 },
    { 0xfbb17e51, SUC_lexicon_tags_33 },
    { 0x6fd17e52, SUC_lexicon_tags_26 },
    { 0x39d17e53, SUC_lexicon_tags_60 },
    { 0x48d17e54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x82f17e56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39117e58, SUC_lexicon_tags_60 },
    { 0xd0317e59, SUC_lexicon_tags_0 },
    { 0xa8317e59, SUC_lexicon_tags_15 },
    { 0x81717e5a, SUC_lexicon_tags_30 },
    { 0xaf117e5b, SUC_lexicon_tags_17 },
    { 0x19f17e5d, SUC_lexicon_tags_0 },
    { 0x84b17e59, SUC_lexicon_tags_14 },
    { 0x1b517e5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53917e66, SUC_lexicon_tags_49 },
    { 0x37517e66, SUC_lexicon_tags_23 },
    { 0x96f17e68, SUC_lexicon_tags_37 },
    { 0x3c917e69, SUC_lexicon_tags_23 },
    { 0x4c317e6a, SUC_lexicon_tags_23 },
    { 0x71917e67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19717e71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x27d17e73, SUC_lexicon_tags_0 },
    { 0xbbb17e73, SUC_lexicon_tags_15 },
    { 0x5d917e74, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d17e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1317e80, SUC_lexicon_tags_60 },
    { 0x81917e81, SUC_lexicon_tags_45 },
    { 0xa9b17e80, SUC_lexicon_tags_0 },
    { 0xee317e83, SUC_lexicon_tags_47 },
    { 0xa7f17e84, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b17e90, SUC_lexicon_tags_18 },
    { 0x24517e91, SUC_lexicon_tags_31 },
    { 0x63517e90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf317e95, SUC_lexicon_tags_44 },
    { 0xf5317e96, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd1f17e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb717e9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf717ea1, SUC_lexicon_tags_8 },
    { 0x19717ea1, SUC_lexicon_tags_1 },
    { 0xf9d17ea1, SUC_lexicon_tags_30 },
    { 0xce717ea4, SUC_lexicon_tags_23 },
    { 0x77517ea2, SUC_lexicon_tags_43 },
    { 0xb9b17ea3, SUC_lexicon_tags_15 },
    { 0x9ef17ea7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x44317eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d17ead, SUC_lexicon_tags_28 },
    { 0x1b717eae, SUC_lexicon_tags_19 },
    { 0x84917eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12117eb5, SUC_lexicon_tags_27 },
    { 0x3c117eb5, SUC_lexicon_tags_49 },
    { 0x40d17eb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff17ebc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c517ec1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e317ec5, SUC_lexicon_tags_63 },
    { 0xad717ec6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8917ec8, SUC_lexicon_tags_17 },
    { 0x22f17ec9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff517ecf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfad17ed1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b17ed6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d17edc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73517ee4, SUC_lexicon_tags_87 },
    { 0x40917ee5, SUC_lexicon_tags_63 },
    { 0x13917ee5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf1d17ee8, SUC_lexicon_tags_37 },
    { 0xaad17ee9, SUC_lexicon_tags_129 },
    { 0xed717ee9, SUC_lexicon_tags_1511 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff17ef6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5917ef9, SUC_lexicon_tags_77 },
    { 0x69117ef9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15517eff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf317f03, SUC_lexicon_tags_3 },
    { 0x64f17f03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b317f06, SUC_lexicon_tags_32 },
    { 0x18f17f06, SUC_lexicon_tags_25 },
    { 0x73f17f06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf917f0b, SUC_lexicon_tags_11 },
    { 0x1fd17f0c, SUC_lexicon_tags_37 },
    { 0x80b17f0c, SUC_lexicon_tags_47 },
    { 0x25117f0e, SUC_lexicon_tags_27 },
    { 0x78d17f0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20517f11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3bf17f13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8517f1a, SUC_lexicon_tags_49 },
    { 0xca117f1b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4af17f1d, SUC_lexicon_tags_25 },
    { 0x5717f1d, SUC_lexicon_tags_25 },
    { 0x8e117f1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d17f28, SUC_lexicon_tags_68 },
    { 0x32117f28, SUC_lexicon_tags_37 },
    { 0x2e517f2a, SUC_lexicon_tags_37 },
    { 0x7e717f2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d517f2d, SUC_lexicon_tags_15 },
    { 0x5c517f2d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b17f32, SUC_lexicon_tags_11 },
    { 0x2c717f33, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1317f36, SUC_lexicon_tags_2 },
    { 0x37717f37, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd17f3d, SUC_lexicon_tags_15 },
    { 0x9cb17f3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9917f42, SUC_lexicon_tags_2 },
    { 0xafd17f43, SUC_lexicon_tags_9 },
    { 0x84517f44, SUC_lexicon_tags_8 },
    { 0x2c317f45, SUC_lexicon_tags_14 },
    { 0xc9f17f46, SUC_lexicon_tags_87 },
    { 0xd0717f44, SUC_lexicon_tags_23 },
    { 0xad517f48, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5af17f4a, SUC_lexicon_tags_77 },
    { 0xe4b17f4b, SUC_lexicon_tags_27 },
    { 0xde117f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9917f4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf17f51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2d17f53, SUC_lexicon_tags_23 },
    { 0x52317f53, SUC_lexicon_tags_11 },
    { 0xb2517f55, SUC_lexicon_tags_2 },
    { 0xee317f54, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x38917f58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12717f5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa9517f5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17917f5f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4517f63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d917f65, SUC_lexicon_tags_49 },
    { 0x59917f66, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e317f69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaab17f6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d17f6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd17f73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e917f78, SUC_lexicon_tags_37 },
    { 0xf9517f78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39b17f7b, SUC_lexicon_tags_37 },
    { 0x29517f7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92717f7f, SUC_lexicon_tags_11 },
    { 0xdd117f80, SUC_lexicon_tags_13 },
    { 0x84317f80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x30117f83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb117f88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeff17f8a, SUC_lexicon_tags_1 },
    { 0xc7d17f8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67117f97, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc7117f99, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6317f9c, SUC_lexicon_tags_17 },
    { 0xffb17f9d, SUC_lexicon_tags_47 },
    { 0x1d917f9e, SUC_lexicon_tags_15 },
    { 0xa8717f9f, SUC_lexicon_tags_25 },
    { 0x10917f9e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80117fa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2917fa7, SUC_lexicon_tags_49 },
    { 0xdc317fa8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7117fad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e517fb4, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16917fba, SUC_lexicon_tags_188 },
    { 0x3e517fba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63517fbe, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53117fc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e717fc6, SUC_lexicon_tags_0 },
    { 0x7b117fc7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b17fcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb17fcf, SUC_lexicon_tags_37 },
    { 0x83717fd0, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xc9b17fd2, SUC_lexicon_tags_2 },
    { 0xa0717fd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa0517fd5, SUC_lexicon_tags_10 },
    { 0x94b17fd6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e317fd8, SUC_lexicon_tags_38 },
    { 0x98917fd8, SUC_lexicon_tags_11 },
    { 0xd8917fd9, SUC_lexicon_tags_87 },
    { 0x46717fd8, SUC_lexicon_tags_37 },
    { 0xe3717fd8, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xa4b17fde, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d17fe1, SUC_lexicon_tags_2 },
    { 0x32f17fe2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80317fe4, SUC_lexicon_tags_36 },
    { 0x7a117fe4, SUC_lexicon_tags_29 },
    { 0x2d17fe5, SUC_lexicon_tags_25 },
    { 0x91917fe5, SUC_lexicon_tags_47 },
    { 0x7c717fe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17717fee, SUC_lexicon_tags_8 },
    { 0xcb117fef, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xcf117ff1, SUC_lexicon_tags_60 },
    { 0xc2517ff2, SUC_lexicon_tags_15 },
    { 0xf317ff3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a317ff6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6917ff9, SUC_lexicon_tags_49 },
    { 0x26517ffa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d517ffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a517ffe, SUC_lexicon_tags_43 },
    { 0x45f17fff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8318005, SUC_lexicon_tags_42 },
    { 0xf1318006, SUC_lexicon_tags_60 },
    { 0x16318007, SUC_lexicon_tags_2 },
    { 0xae518005, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb18011, SUC_lexicon_tags_12 },
    { 0xf9318012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3518017, SUC_lexicon_tags_37 },
    { 0x27118017, SUC_lexicon_tags_37 },
    { 0x66518019, SUC_lexicon_tags_27 },
    { 0x95d1801a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd1801d, SUC_lexicon_tags_1 },
    { 0xca31801d, SUC_lexicon_tags_10 },
    { 0xc2d1801f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9318022, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d18026, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1011802f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42118031, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3718034, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24918038, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa251803c, SUC_lexicon_tags_1 },
    { 0x9b11803d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf18042, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf518044, SUC_lexicon_tags_11 },
    { 0x30d18044, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9691804a, SUC_lexicon_tags_17 },
    { 0x9e31804a, SUC_lexicon_tags_7 },
    { 0x1d51804b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b1804f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23718053, SUC_lexicon_tags_45 },
    { 0x5a318054, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf18056, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8118058, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d1805b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4471805f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xab118062, SUC_lexicon_tags_84 },
    { 0xb7118063, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x74f18065, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd1806f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdff18071, SUC_lexicon_tags_17 },
    { 0x23d18072, SUC_lexicon_tags_8 },
    { 0xdb318072, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36718079, SUC_lexicon_tags_12 },
    { 0xc4b18079, SUC_lexicon_tags_25 },
    { 0xefd1807b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6318081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c518085, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54518087, SUC_lexicon_tags_7 },
    { 0x4b18088, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe918090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17118094, SUC_lexicon_tags_936 },
    { 0x55d18095, SUC_lexicon_tags_37 },
    { 0xf7d18095, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a91809b, SUC_lexicon_tags_46 },
    { 0x3e91809c, SUC_lexicon_tags_37 },
    { 0x77d1809d, SUC_lexicon_tags_2 },
    { 0x53d1809e, SUC_lexicon_tags_11 },
    { 0x5bb1809f, SUC_lexicon_tags_2 },
    { 0x2cb1809d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47180a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11180a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b180aa, SUC_lexicon_tags_19 },
    { 0x1c1180ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x303180ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d180b0, SUC_lexicon_tags_40 },
    { 0x22d180b0, SUC_lexicon_tags_25 },
    { 0xc11180b2, SUC_lexicon_tags_74 },
    { 0x315180b2, SUC_lexicon_tags_45 },
    { 0xa67180b4, SUC_lexicon_tags_6 },
    { 0x363180b4, SUC_lexicon_tags_39 },
    { 0x9f7180b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5180bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f180be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5180c1, SUC_lexicon_tags_14 },
    { 0x6a1180c2, SUC_lexicon_tags_0 },
    { 0x917180c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf180c7, SUC_lexicon_tags_37 },
    { 0x48f180c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x603180ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833180d1, SUC_lexicon_tags_43 },
    { 0x31180d1, SUC_lexicon_tags_11 },
    { 0x911180d3, SUC_lexicon_tags_45 },
    { 0x8ff180d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d180d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b180e1, SUC_lexicon_tags_26 },
    { 0x20f180e1, SUC_lexicon_tags_8 },
    { 0x7a7180e3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd31180e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7180ea, SUC_lexicon_tags_46 },
    { 0x75f180eb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8b1180ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d180f2, SUC_lexicon_tags_108 },
    { 0x50f180f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9180f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x195180f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91318101, SUC_lexicon_tags_87 },
    { 0x1a518102, SUC_lexicon_tags_8 },
    { 0x32718103, SUC_lexicon_tags_7 },
    { 0xfc118103, SUC_lexicon_tags_15 },
    { 0x58118104, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d118109, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7311810c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85d1810e, SUC_lexicon_tags_11 },
    { 0xaa11810f, SUC_lexicon_tags_17 },
    { 0xd2518110, SUC_lexicon_tags_45 },
    { 0xc31810f, SUC_lexicon_tags_7 },
    { 0xf8d18110, SUC_lexicon_tags_49 },
    { 0xdcd18110, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b18116, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf18120, SUC_lexicon_tags_17 },
    { 0x58518121, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f718124, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46318126, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb9318128, SUC_lexicon_tags_49 },
    { 0x44d18129, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71812d, SUC_lexicon_tags_11 },
    { 0x8851812e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a118132, SUC_lexicon_tags_524 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb1813a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f1813e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b18141, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f18144, SUC_lexicon_tags_17 },
    { 0x10718145, SUC_lexicon_tags_14 },
    { 0xb6318146, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1814a, SUC_lexicon_tags_18 },
    { 0x59f1814b, SUC_lexicon_tags_25 },
    { 0x82d1814c, SUC_lexicon_tags_8 },
    { 0x5871814d, SUC_lexicon_tags_6 },
    { 0x49d1814e, SUC_lexicon_tags_46 },
    { 0x4751814a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd718151, SUC_lexicon_tags_26 },
    { 0xce118152, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb718155, SUC_lexicon_tags_47 },
    { 0x48518155, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5a118158, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f18160, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd18166, SUC_lexicon_tags_32 },
    { 0x5b918167, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d1816f, SUC_lexicon_tags_30 },
    { 0x8fd1816f, SUC_lexicon_tags_33 },
    { 0xa8f18171, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65118177, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad1817a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc118180, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ed18182, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21118187, SUC_lexicon_tags_12 },
    { 0x20118188, SUC_lexicon_tags_10 },
    { 0x1bd18188, SUC_lexicon_tags_30 },
    { 0x16318189, SUC_lexicon_tags_63 },
    { 0x72f18187, SUC_lexicon_tags_11 },
    { 0x8db1818c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd271818e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35118192, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x88918194, SUC_lexicon_tags_11 },
    { 0x9a518194, SUC_lexicon_tags_49 },
    { 0x78118195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb18199, SUC_lexicon_tags_11 },
    { 0x73b1819a, SUC_lexicon_tags_43 },
    { 0xe0518199, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129181a0, SUC_lexicon_tags_33 },
    { 0x769181a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd181a9, SUC_lexicon_tags_91 },
    { 0x55181aa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3181b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5181b4, SUC_lexicon_tags_37 },
    { 0xbcf181b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73181b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfe3181ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x847181bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x66f181be, SUC_lexicon_tags_2 },
    { 0x37d181be, SUC_lexicon_tags_38 },
    { 0xc3181bf, SUC_lexicon_tags_63 },
    { 0x75f181c1, SUC_lexicon_tags_58 },
    { 0x315181c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359181cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b181cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x167181d2, SUC_lexicon_tags_15 },
    { 0x26d181d3, SUC_lexicon_tags_0 },
    { 0xf2f181d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x647181d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48b181d8, SUC_lexicon_tags_25 },
    { 0x905181d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c7181db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5181e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d181e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d181ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x811181ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf181f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x515181f5, SUC_lexicon_tags_11 },
    { 0xdb5181f5, SUC_lexicon_tags_17 },
    { 0x45181f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33d181f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x113181fb, SUC_lexicon_tags_165 },
    { 0xf41181fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbed181fe, SUC_lexicon_tags_36 },
    { 0x847181ff, SUC_lexicon_tags_70 },
    { 0x41518200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b18207, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x61118209, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1871820e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d18210, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2e918212, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xde518214, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd5118216, SUC_lexicon_tags_8 },
    { 0x59918217, SUC_lexicon_tags_39 },
    { 0xcf118218, SUC_lexicon_tags_12 },
    { 0xb2718219, SUC_lexicon_tags_60 },
    { 0xeb1821a, SUC_lexicon_tags_1 },
    { 0xd2f1821a, SUC_lexicon_tags_14 },
    { 0x6331821c, SUC_lexicon_tags_2 },
    { 0xb931821c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f18221, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb118225, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22118229, SUC_lexicon_tags_46 },
    { 0xcc31822a, SUC_lexicon_tags_47 },
    { 0x8d31822b, SUC_lexicon_tags_12 },
    { 0x7bf1822c, SUC_lexicon_tags_68 },
    { 0x1c71822d, SUC_lexicon_tags_1 },
    { 0x21d1822d, SUC_lexicon_tags_37 },
    { 0xa3f1822b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf718231, SUC_lexicon_tags_1 },
    { 0x9fd18232, SUC_lexicon_tags_15 },
    { 0xf5f18231, SUC_lexicon_tags_11 },
    { 0xbbd18233, SUC_lexicon_tags_37 },
    { 0xefd18234, SUC_lexicon_tags_15 },
    { 0x1ed18233, SUC_lexicon_tags_2 },
    { 0xc1f18237, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c318239, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60b1823b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e71823e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef18241, SUC_lexicon_tags_45 },
    { 0xe9118242, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a918246, SUC_lexicon_tags_8 },
    { 0xb118247, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf718249, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x98d1824b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1751824d, SUC_lexicon_tags_15 },
    { 0xe171824d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e318250, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6918254, SUC_lexicon_tags_39 },
    { 0xd18255, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30f18257, SUC_lexicon_tags_2 },
    { 0xd7f18257, SUC_lexicon_tags_38 },
    { 0x5918259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f11825d, SUC_lexicon_tags_30 },
    { 0xef91825e, SUC_lexicon_tags_37 },
    { 0x7991825d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x51d18261, SUC_lexicon_tags_11 },
    { 0x25718261, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3318267, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2151826b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1951826e, SUC_lexicon_tags_29 },
    { 0xa8f1826e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b18273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f18276, SUC_lexicon_tags_7 },
    { 0x6c318277, SUC_lexicon_tags_14 },
    { 0x52b18277, SUC_lexicon_tags_1 },
    { 0xce518279, SUC_lexicon_tags_37 },
    { 0x58b18279, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d18281, SUC_lexicon_tags_27 },
    { 0x31518282, SUC_lexicon_tags_63 },
    { 0x8d18283, SUC_lexicon_tags_0 },
    { 0xb2118284, SUC_lexicon_tags_43 },
    { 0xde318285, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d18288, SUC_lexicon_tags_9 },
    { 0xb0b18289, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f1828c, SUC_lexicon_tags_12 },
    { 0x2b91828d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db18298, SUC_lexicon_tags_17 },
    { 0xe4718298, SUC_lexicon_tags_3 },
    { 0x1831829a, SUC_lexicon_tags_9 },
    { 0x50118299, SUC_lexicon_tags_6 },
    { 0xd611829c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6d1829e, SUC_lexicon_tags_15 },
    { 0xfff1829f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217182a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27182a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665182ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd182b1, SUC_lexicon_tags_11 },
    { 0x437182b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7182b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837182bc, SUC_lexicon_tags_7 },
    { 0xb0d182bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4d182bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa29182c1, SUC_lexicon_tags_31 },
    { 0xe4f182c2, SUC_lexicon_tags_11 },
    { 0x635182c2, SUC_lexicon_tags_54 },
    { 0x23d182c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3182c7, SUC_lexicon_tags_38 },
    { 0xa45182c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef182cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df182cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5182d3, SUC_lexicon_tags_11 },
    { 0xa6d182d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x385182d6, SUC_lexicon_tags_37 },
    { 0xab5182d7, SUC_lexicon_tags_1224 },
    { 0, NULL },
    { 0, NULL },
    { 0x317182da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1182dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x475182df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307182e5, SUC_lexicon_tags_63 },
    { 0xae5182e6, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d182ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x399182ee, SUC_lexicon_tags_1 },
    { 0xd5f182ef, SUC_lexicon_tags_54 },
    { 0x24d182f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x603182f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9182f5, SUC_lexicon_tags_37 },
    { 0x85d182f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x409182f8, SUC_lexicon_tags_14 },
    { 0x717182f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25118302, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x38318305, SUC_lexicon_tags_11 },
    { 0xe4718306, SUC_lexicon_tags_2 },
    { 0xc6518306, SUC_lexicon_tags_28 },
    { 0x74318308, SUC_lexicon_tags_37 },
    { 0x1b318306, SUC_lexicon_tags_17 },
    { 0x751830a, SUC_lexicon_tags_5 },
    { 0xd771830a, SUC_lexicon_tags_11 },
    { 0xcd51830a, SUC_lexicon_tags_138 },
    { 0x731830d, SUC_lexicon_tags_39 },
    { 0xbd91830e, SUC_lexicon_tags_37 },
    { 0xfe51830c, SUC_lexicon_tags_49 },
    { 0xaf91830e, SUC_lexicon_tags_17 },
    { 0xca71830e, SUC_lexicon_tags_17 },
    { 0x45518310, SUC_lexicon_tags_47 },
    { 0x1d71830f, SUC_lexicon_tags_12 },
    { 0xb6718314, SUC_lexicon_tags_25 },
    { 0xc3f18313, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba31831b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9118325, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b18328, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41d1832a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x34d1832c, SUC_lexicon_tags_38 },
    { 0x77f1832d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d18330, SUC_lexicon_tags_2 },
    { 0xeb318331, SUC_lexicon_tags_11 },
    { 0x60118330, SUC_lexicon_tags_2 },
    { 0xc1d18330, SUC_lexicon_tags_15 },
    { 0x40518334, SUC_lexicon_tags_63 },
    { 0x4718334, SUC_lexicon_tags_60 },
    { 0xd4518336, SUC_lexicon_tags_33 },
    { 0xe5f18335, SUC_lexicon_tags_2 },
    { 0xbb918338, SUC_lexicon_tags_11 },
    { 0xabb18338, SUC_lexicon_tags_1 },
    { 0x6471833a, SUC_lexicon_tags_12 },
    { 0x95518333, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7871833f, SUC_lexicon_tags_17 },
    { 0x5111833f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b18345, SUC_lexicon_tags_0 },
    { 0x9bf18346, SUC_lexicon_tags_8 },
    { 0x6f18347, SUC_lexicon_tags_14 },
    { 0xd4d18348, SUC_lexicon_tags_1 },
    { 0x45518345, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f71834f, SUC_lexicon_tags_21 },
    { 0xcbb1834f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90318356, SUC_lexicon_tags_7 },
    { 0x26b18356, SUC_lexicon_tags_25 },
    { 0x8c918356, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a51835c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85b1835e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffb18360, SUC_lexicon_tags_4 },
    { 0x7bb18360, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3df18363, SUC_lexicon_tags_25 },
    { 0x75518364, SUC_lexicon_tags_7 },
    { 0x1f18363, SUC_lexicon_tags_27 },
    { 0xd7918366, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d1836c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b1836f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb18374, SUC_lexicon_tags_33 },
    { 0x43118375, SUC_lexicon_tags_61 },
    { 0xd9918376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a118380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4318386, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfed18388, SUC_lexicon_tags_45 },
    { 0x3c918389, SUC_lexicon_tags_16 },
    { 0xb4d18389, SUC_lexicon_tags_15 },
    { 0x7d1838a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf671838d, SUC_lexicon_tags_32 },
    { 0xb151838e, SUC_lexicon_tags_17 },
    { 0xdab1838d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5318396, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x25118398, SUC_lexicon_tags_30 },
    { 0xa118398, SUC_lexicon_tags_15 },
    { 0x6151839a, SUC_lexicon_tags_11 },
    { 0x3b11839b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb1839e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f183a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1183a4, SUC_lexicon_tags_7 },
    { 0xbed183a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99d183a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa55183a9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7183ad, SUC_lexicon_tags_25 },
    { 0xd19183ae, SUC_lexicon_tags_17 },
    { 0x399183ad, SUC_lexicon_tags_25 },
    { 0x577183ae, SUC_lexicon_tags_10 },
    { 0x16b183ae, SUC_lexicon_tags_25 },
    { 0x217183b2, SUC_lexicon_tags_8 },
    { 0xe27183b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x681183b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b183b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3b183ba, SUC_lexicon_tags_15 },
    { 0xeed183bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee1183bd, SUC_lexicon_tags_23 },
    { 0x6b7183bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x255183c1, SUC_lexicon_tags_65 },
    { 0xc8b183c2, SUC_lexicon_tags_27 },
    { 0x327183c3, SUC_lexicon_tags_2 },
    { 0x2eb183c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59183c8, SUC_lexicon_tags_15 },
    { 0xec3183c8, SUC_lexicon_tags_15 },
    { 0x8e3183c9, SUC_lexicon_tags_2 },
    { 0x3b3183c9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d183cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53d183d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f183d6, SUC_lexicon_tags_86 },
    { 0x769183d6, SUC_lexicon_tags_74 },
    { 0x125183d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34f183da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x229183dc, SUC_lexicon_tags_1 },
    { 0xd99183dd, SUC_lexicon_tags_26 },
    { 0x7fb183de, SUC_lexicon_tags_29 },
    { 0xbe9183dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9183e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b3183e5, SUC_lexicon_tags_1 },
    { 0xb7b183e5, SUC_lexicon_tags_29 },
    { 0x487183e6, SUC_lexicon_tags_1 },
    { 0x24f183e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03183ec, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9fd183ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91183f1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833183f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3183f9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x231183fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b18402, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30118405, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2318409, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe51840c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfad1840e, SUC_lexicon_tags_26 },
    { 0x8071840e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9118412, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb0118414, SUC_lexicon_tags_37 },
    { 0xa8718415, SUC_lexicon_tags_25 },
    { 0xa8518416, SUC_lexicon_tags_7 },
    { 0x8718416, SUC_lexicon_tags_46 },
    { 0x98918418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc671841d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4918422, SUC_lexicon_tags_11 },
    { 0x51318422, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x20318425, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f718427, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x85b18429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a51842b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91842e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde918434, SUC_lexicon_tags_19 },
    { 0x94718435, SUC_lexicon_tags_66 },
    { 0x6b918435, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab91843b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d18440, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33118449, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55318450, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b18458, SUC_lexicon_tags_2 },
    { 0x22f18458, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5531845d, SUC_lexicon_tags_46 },
    { 0x9dd1845d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed518460, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4118468, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba91846e, SUC_lexicon_tags_23 },
    { 0x9091846e, SUC_lexicon_tags_2 },
    { 0x22f18470, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35918479, SUC_lexicon_tags_37 },
    { 0x8c718479, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb1847f, SUC_lexicon_tags_45 },
    { 0xb118480, SUC_lexicon_tags_31 },
    { 0x7cf18481, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b1848a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f1848e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6718490, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b18496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4751849e, SUC_lexicon_tags_1 },
    { 0x7611849f, SUC_lexicon_tags_37 },
    { 0x959184a0, SUC_lexicon_tags_16 },
    { 0x94f184a1, SUC_lexicon_tags_17 },
    { 0x8c3184a1, SUC_lexicon_tags_45 },
    { 0x363184a0, SUC_lexicon_tags_15 },
    { 0x93f184a4, SUC_lexicon_tags_45 },
    { 0xa95184a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d184a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b184ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1184b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6b184b9, SUC_lexicon_tags_17 },
    { 0xea1184b9, SUC_lexicon_tags_55 },
    { 0xcd5184bb, SUC_lexicon_tags_3 },
    { 0x3f184bc, SUC_lexicon_tags_11 },
    { 0xef9184bd, SUC_lexicon_tags_12 },
    { 0x68f184bc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9c3184c0, SUC_lexicon_tags_39 },
    { 0x1d3184c1, SUC_lexicon_tags_15 },
    { 0x5f184c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f184c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7184cb, SUC_lexicon_tags_7 },
    { 0x997184cc, SUC_lexicon_tags_0 },
    { 0xedf184cd, SUC_lexicon_tags_60 },
    { 0x6a3184cd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x85184d0, SUC_lexicon_tags_12 },
    { 0x2ab184d1, SUC_lexicon_tags_17 },
    { 0x329184d2, SUC_lexicon_tags_1 },
    { 0x6ef184d3, SUC_lexicon_tags_17 },
    { 0x743184d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f184d9, SUC_lexicon_tags_16 },
    { 0xa99184da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa57184dc, SUC_lexicon_tags_7 },
    { 0xfa1184dd, SUC_lexicon_tags_37 },
    { 0xcff184de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1b1184e0, SUC_lexicon_tags_15 },
    { 0x7d3184e0, SUC_lexicon_tags_16 },
    { 0x197184e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x687184e5, SUC_lexicon_tags_38 },
    { 0x9b9184e5, SUC_lexicon_tags_37 },
    { 0xe47184e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261184f0, SUC_lexicon_tags_31 },
    { 0x305184f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3184f4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbfb184f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17918502, SUC_lexicon_tags_19 },
    { 0x4bf18503, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf18506, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf1850e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x84118510, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9318513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7f18515, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd1851b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf091851d, SUC_lexicon_tags_37 },
    { 0xdf71851e, SUC_lexicon_tags_33 },
    { 0x3391851e, SUC_lexicon_tags_2 },
    { 0x2151851f, SUC_lexicon_tags_17 },
    { 0xfd718521, SUC_lexicon_tags_29 },
    { 0xebd18522, SUC_lexicon_tags_19 },
    { 0xf118522, SUC_lexicon_tags_71 },
    { 0xd491851d, SUC_lexicon_tags_49 },
    { 0xf4118525, SUC_lexicon_tags_0 },
    { 0xc8f18520, SUC_lexicon_tags_11 },
    { 0x6d118527, SUC_lexicon_tags_1 },
    { 0x78f18528, SUC_lexicon_tags_26 },
    { 0xeab18529, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b1852d, SUC_lexicon_tags_47 },
    { 0xde31852e, SUC_lexicon_tags_2 },
    { 0xaed1852f, SUC_lexicon_tags_2 },
    { 0xfaf18530, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb18535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc318537, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x38b18539, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31f1853b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b18544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b18547, SUC_lexicon_tags_26 },
    { 0x35318548, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd18550, SUC_lexicon_tags_60 },
    { 0x76518550, SUC_lexicon_tags_82 },
    { 0x3d718550, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab18556, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad18558, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3391855c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d18560, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3118562, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb1318564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a318568, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d1856f, SUC_lexicon_tags_26 },
    { 0xb9118570, SUC_lexicon_tags_37 },
    { 0xb4b1856f, SUC_lexicon_tags_7 },
    { 0xe9918570, SUC_lexicon_tags_11 },
    { 0xaf51856f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9718575, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f18578, SUC_lexicon_tags_30 },
    { 0xef918579, SUC_lexicon_tags_4 },
    { 0xd411857a, SUC_lexicon_tags_23 },
    { 0xf2918579, SUC_lexicon_tags_37 },
    { 0x611857c, SUC_lexicon_tags_29 },
    { 0xeaf1857b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb231857f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad18582, SUC_lexicon_tags_11 },
    { 0x65118583, SUC_lexicon_tags_15 },
    { 0x37b18584, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe9518586, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c71858c, SUC_lexicon_tags_37 },
    { 0x4511858d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb318590, SUC_lexicon_tags_11 },
    { 0xc4d18591, SUC_lexicon_tags_7 },
    { 0x48318592, SUC_lexicon_tags_23 },
    { 0xdd18593, SUC_lexicon_tags_37 },
    { 0x80918594, SUC_lexicon_tags_16 },
    { 0x55918592, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f18598, SUC_lexicon_tags_1 },
    { 0xa8518599, SUC_lexicon_tags_38 },
    { 0x74718599, SUC_lexicon_tags_37 },
    { 0x3bb1859b, SUC_lexicon_tags_45 },
    { 0xcc11859c, SUC_lexicon_tags_37 },
    { 0x3531859d, SUC_lexicon_tags_22 },
    { 0xba51859d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b185a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9185ad, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199185b2, SUC_lexicon_tags_47 },
    { 0x5af185b3, SUC_lexicon_tags_60 },
    { 0x21d185b2, SUC_lexicon_tags_38 },
    { 0x7cd185b5, SUC_lexicon_tags_23 },
    { 0x4bd185b6, SUC_lexicon_tags_15 },
    { 0xa4d185b2, SUC_lexicon_tags_2 },
    { 0x3bd185b8, SUC_lexicon_tags_37 },
    { 0xd07185b9, SUC_lexicon_tags_29 },
    { 0x6b7185ba, SUC_lexicon_tags_9 },
    { 0xf99185bb, SUC_lexicon_tags_49 },
    { 0x355185bc, SUC_lexicon_tags_7 },
    { 0xdd3185b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x741185c0, SUC_lexicon_tags_7 },
    { 0x3c7185c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd5185c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f185c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1185ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57185cd, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x13b185cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17185d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x825185d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3185dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x381185df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4cb185e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39185e6, SUC_lexicon_tags_11 },
    { 0x153185e7, SUC_lexicon_tags_27 },
    { 0x979185e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd69185ea, SUC_lexicon_tags_31 },
    { 0x223185eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x969185ed, SUC_lexicon_tags_17 },
    { 0x747185ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55185f2, SUC_lexicon_tags_65 },
    { 0x891185f3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5185f6, SUC_lexicon_tags_38 },
    { 0xd87185f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d5185f9, SUC_lexicon_tags_11 },
    { 0xe97185f9, SUC_lexicon_tags_25 },
    { 0xde3185fa, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7185ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9bd18601, SUC_lexicon_tags_7 },
    { 0x19518602, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d18606, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9151860a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50918610, SUC_lexicon_tags_9 },
    { 0x23718611, SUC_lexicon_tags_12 },
    { 0xd6d18612, SUC_lexicon_tags_691 },
    { 0xfa718612, SUC_lexicon_tags_9 },
    { 0x51918614, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd18619, SUC_lexicon_tags_38 },
    { 0x4411861a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae11861f, SUC_lexicon_tags_25 },
    { 0x6f18620, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9318630, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x58518632, SUC_lexicon_tags_46 },
    { 0x70518633, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f51863a, SUC_lexicon_tags_1548 },
    { 0x43b1863b, SUC_lexicon_tags_9 },
    { 0x28f1863c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x991863e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14b18640, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d518645, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x69f18647, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d1864f, SUC_lexicon_tags_25 },
    { 0xaa71864f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f18653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb18656, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f1865a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa371865d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1271865f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91918663, SUC_lexicon_tags_63 },
    { 0x6fd18664, SUC_lexicon_tags_60 },
    { 0xc6118665, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b51866c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f18670, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d718674, SUC_lexicon_tags_0 },
    { 0xdc918675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b18678, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8eb1867a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2371867c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47918686, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x86518689, SUC_lexicon_tags_4 },
    { 0x88f1868a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f1868d, SUC_lexicon_tags_55 },
    { 0xa811868e, SUC_lexicon_tags_70 },
    { 0xf711868d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9718693, SUC_lexicon_tags_43 },
    { 0x14f18694, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5918697, SUC_lexicon_tags_10 },
    { 0xf1f18698, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d1869e, SUC_lexicon_tags_45 },
    { 0xb5f1869f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x245186a2, SUC_lexicon_tags_46 },
    { 0xf7186a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb186a8, SUC_lexicon_tags_7 },
    { 0x839186a9, SUC_lexicon_tags_8 },
    { 0x5ab186aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a5186ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563186b1, SUC_lexicon_tags_60 },
    { 0x945186b2, SUC_lexicon_tags_65 },
    { 0x53b186b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd77186b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x817186b8, SUC_lexicon_tags_37 },
    { 0x459186b9, SUC_lexicon_tags_15 },
    { 0x4e5186ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d186bd, SUC_lexicon_tags_2 },
    { 0x823186be, SUC_lexicon_tags_38 },
    { 0x591186bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3186c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d186c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb186d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff186d7, SUC_lexicon_tags_63 },
    { 0x8f186d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x485186db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd186e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75186e5, SUC_lexicon_tags_17 },
    { 0xb93186e6, SUC_lexicon_tags_1133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf186eb, SUC_lexicon_tags_6 },
    { 0x4d186eb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcbb186ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfeb186f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869186f5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407186f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359186fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe718700, SUC_lexicon_tags_15 },
    { 0x92918700, SUC_lexicon_tags_12 },
    { 0xd5b18702, SUC_lexicon_tags_25 },
    { 0xbed18701, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32d18705, SUC_lexicon_tags_26 },
    { 0x44118706, SUC_lexicon_tags_17 },
    { 0x19718705, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xdf318709, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85f1870b, SUC_lexicon_tags_43 },
    { 0x2591870c, SUC_lexicon_tags_25 },
    { 0x7531870c, SUC_lexicon_tags_15 },
    { 0x7a71870e, SUC_lexicon_tags_84 },
    { 0x7f51870f, SUC_lexicon_tags_31 },
    { 0x21b18710, SUC_lexicon_tags_106 },
    { 0x5ab18711, SUC_lexicon_tags_2 },
    { 0xb111870f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af18715, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c118718, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d18720, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2d118722, SUC_lexicon_tags_45 },
    { 0xfcf18723, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d918726, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x45118729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b1872d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d18731, SUC_lexicon_tags_23 },
    { 0x2f518732, SUC_lexicon_tags_14 },
    { 0x51d18733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd18737, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f1873a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4271873c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa151873f, SUC_lexicon_tags_11 },
    { 0xe2318740, SUC_lexicon_tags_11 },
    { 0x7dd1873f, SUC_lexicon_tags_37 },
    { 0x1cd18742, SUC_lexicon_tags_7 },
    { 0xa8f18743, SUC_lexicon_tags_17 },
    { 0xa318743, SUC_lexicon_tags_7 },
    { 0x4ef18743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d18748, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f1874c, SUC_lexicon_tags_140 },
    { 0x2f1874d, SUC_lexicon_tags_29 },
    { 0xadf1874e, SUC_lexicon_tags_48 },
    { 0xb511874f, SUC_lexicon_tags_11 },
    { 0x87b1874e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd18755, SUC_lexicon_tags_49 },
    { 0x67f18755, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb1875a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318761, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe9b18763, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x63118766, SUC_lexicon_tags_55 },
    { 0xa9518766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b91876b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77b1876d, SUC_lexicon_tags_49 },
    { 0xe631876e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff718770, SUC_lexicon_tags_46 },
    { 0xa5918771, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15718773, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x27918775, SUC_lexicon_tags_46 },
    { 0x14d18776, SUC_lexicon_tags_1 },
    { 0xbc918776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a11877a, SUC_lexicon_tags_2 },
    { 0x33d1877b, SUC_lexicon_tags_11 },
    { 0xfc31877c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f18786, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62b18788, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb11878a, SUC_lexicon_tags_2 },
    { 0x5891878b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f518790, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7fb18792, SUC_lexicon_tags_25 },
    { 0x53318792, SUC_lexicon_tags_37 },
    { 0x45518793, SUC_lexicon_tags_25 },
    { 0xf8318794, SUC_lexicon_tags_11 },
    { 0x6bd18792, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e318799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb187a1, SUC_lexicon_tags_30 },
    { 0x7b187a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b187a7, SUC_lexicon_tags_25 },
    { 0xf0b187a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3187ab, SUC_lexicon_tags_7 },
    { 0x559187ac, SUC_lexicon_tags_23 },
    { 0x2f187ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72b187af, SUC_lexicon_tags_23 },
    { 0xd7b187b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbed187b2, SUC_lexicon_tags_25 },
    { 0x587187b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7187b7, SUC_lexicon_tags_19 },
    { 0xf5187b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd187bc, SUC_lexicon_tags_43 },
    { 0x66f187bd, SUC_lexicon_tags_25 },
    { 0x589187be, SUC_lexicon_tags_87 },
    { 0x607187bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee1187c1, SUC_lexicon_tags_11 },
    { 0x2df187c2, SUC_lexicon_tags_13 },
    { 0xbc9187c1, SUC_lexicon_tags_46 },
    { 0x873187c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b187c8, SUC_lexicon_tags_27 },
    { 0x9c7187c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f187cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d187d8, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xc43187da, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec9187dc, SUC_lexicon_tags_29 },
    { 0xfa5187dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9187e0, SUC_lexicon_tags_31 },
    { 0x73b187e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7187e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f1187e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529187ec, SUC_lexicon_tags_35 },
    { 0x3e1187ec, SUC_lexicon_tags_15 },
    { 0x9c1187ec, SUC_lexicon_tags_0 },
    { 0x123187ef, SUC_lexicon_tags_11 },
    { 0x25b187f0, SUC_lexicon_tags_28 },
    { 0xd09187f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7187f6, SUC_lexicon_tags_1 },
    { 0xba9187f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe1187f9, SUC_lexicon_tags_33 },
    { 0x667187fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x16b187fc, SUC_lexicon_tags_39 },
    { 0xd29187fd, SUC_lexicon_tags_3 },
    { 0xa2f187fe, SUC_lexicon_tags_41 },
    { 0xa55187fd, SUC_lexicon_tags_9 },
    { 0x19d18800, SUC_lexicon_tags_49 },
    { 0xd8d18801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e518806, SUC_lexicon_tags_2 },
    { 0x8518806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e918809, SUC_lexicon_tags_31 },
    { 0xfd718809, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2db1880c, SUC_lexicon_tags_60 },
    { 0x26b1880d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78d1880f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62718815, SUC_lexicon_tags_15 },
    { 0x51518816, SUC_lexicon_tags_12 },
    { 0x8a118817, SUC_lexicon_tags_11 },
    { 0x1b918818, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf1881b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3451881e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x13718820, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5318822, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfed18824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6971882d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x78118830, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd18834, SUC_lexicon_tags_194 },
    { 0xab118835, SUC_lexicon_tags_1 },
    { 0xec718835, SUC_lexicon_tags_17 },
    { 0xc5d18837, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b11883a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2751883e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e718844, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x20b18846, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2571884c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9af1884e, SUC_lexicon_tags_1 },
    { 0x9cd1884f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0718851, SUC_lexicon_tags_2 },
    { 0xddd18851, SUC_lexicon_tags_26 },
    { 0x1f318853, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6718859, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x76b1885b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3111885e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d18865, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2cb18867, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d1886c, SUC_lexicon_tags_62 },
    { 0x5831886d, SUC_lexicon_tags_37 },
    { 0xe0f1886e, SUC_lexicon_tags_14 },
    { 0xbe31886e, SUC_lexicon_tags_65 },
    { 0x50b18870, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x96f18872, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdb318874, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x91918876, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50118882, SUC_lexicon_tags_1 },
    { 0x2cf18883, SUC_lexicon_tags_56 },
    { 0x29718884, SUC_lexicon_tags_37 },
    { 0xe7b18885, SUC_lexicon_tags_87 },
    { 0x16f18885, SUC_lexicon_tags_44 },
    { 0x15118884, SUC_lexicon_tags_12 },
    { 0x41518888, SUC_lexicon_tags_2 },
    { 0x9318889, SUC_lexicon_tags_1 },
    { 0xa6d18888, SUC_lexicon_tags_43 },
    { 0xd7918883, SUC_lexicon_tags_32 },
    { 0x71918887, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d18891, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc518895, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xeed18897, SUC_lexicon_tags_47 },
    { 0xe6f18897, SUC_lexicon_tags_0 },
    { 0xa7f18899, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd188a1, SUC_lexicon_tags_25 },
    { 0xbb188a2, SUC_lexicon_tags_18 },
    { 0xf77188a1, SUC_lexicon_tags_25 },
    { 0x71d188a4, SUC_lexicon_tags_25 },
    { 0x27188a5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x23b188a7, SUC_lexicon_tags_32 },
    { 0xf55188a7, SUC_lexicon_tags_66 },
    { 0xf7f188a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b188ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3188af, SUC_lexicon_tags_48 },
    { 0x8df188b0, SUC_lexicon_tags_65 },
    { 0xc57188b1, SUC_lexicon_tags_2 },
    { 0x87d188b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f188b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb55188ba, SUC_lexicon_tags_329 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d188bf, SUC_lexicon_tags_3 },
    { 0x75d188c0, SUC_lexicon_tags_0 },
    { 0x39f188c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa73188c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3188cc, SUC_lexicon_tags_47 },
    { 0x9fb188cd, SUC_lexicon_tags_27 },
    { 0xf1f188cd, SUC_lexicon_tags_14 },
    { 0x9188cd, SUC_lexicon_tags_14 },
    { 0xed1188d0, SUC_lexicon_tags_6 },
    { 0xe39188d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919188d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d188db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1188de, SUC_lexicon_tags_1330 },
    { 0x1c1188df, SUC_lexicon_tags_9 },
    { 0x115188de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b7188e2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837188e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a5188ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa3188ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e9188ee, SUC_lexicon_tags_30 },
    { 0xe95188ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73188f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdef188f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xddf188f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42d188f8, SUC_lexicon_tags_21 },
    { 0xee9188f9, SUC_lexicon_tags_66 },
    { 0xd7d188fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965188ff, SUC_lexicon_tags_216 },
    { 0xc1318900, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f118905, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6af18907, SUC_lexicon_tags_45 },
    { 0x35b18907, SUC_lexicon_tags_16 },
    { 0x4b18909, SUC_lexicon_tags_37 },
    { 0x7891890a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1751890c, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63118913, SUC_lexicon_tags_8 },
    { 0x60518913, SUC_lexicon_tags_11 },
    { 0x8e718914, SUC_lexicon_tags_12 },
    { 0x8af18916, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5171891a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x88f1891c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72b1891e, SUC_lexicon_tags_43 },
    { 0x9871891f, SUC_lexicon_tags_17 },
    { 0x69f18920, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x13118923, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d518927, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b1892a, SUC_lexicon_tags_7 },
    { 0xeeb1892b, SUC_lexicon_tags_33 },
    { 0x7731892c, SUC_lexicon_tags_2 },
    { 0x1d31892c, SUC_lexicon_tags_60 },
    { 0xd551892e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd318930, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f118932, SUC_lexicon_tags_46 },
    { 0xad18933, SUC_lexicon_tags_37 },
    { 0x2c118933, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9718938, SUC_lexicon_tags_25 },
    { 0xf2d18939, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7bd1893b, SUC_lexicon_tags_12 },
    { 0x9af1893c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f1893f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a318947, SUC_lexicon_tags_4 },
    { 0x65718948, SUC_lexicon_tags_7 },
    { 0x92918948, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe771894c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb918950, SUC_lexicon_tags_37 },
    { 0x6b318951, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43718953, SUC_lexicon_tags_12 },
    { 0xdd318954, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b71895d, SUC_lexicon_tags_19 },
    { 0x6b11895d, SUC_lexicon_tags_0 },
    { 0x8fd1895e, SUC_lexicon_tags_11 },
    { 0x5911895e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56b18962, SUC_lexicon_tags_33 },
    { 0x82918963, SUC_lexicon_tags_63 },
    { 0x23b18964, SUC_lexicon_tags_30 },
    { 0x3eb18964, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d1896c, SUC_lexicon_tags_7 },
    { 0xfd31896d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf718971, SUC_lexicon_tags_29 },
    { 0xed518971, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0918975, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb331897c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec718980, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b18984, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f18988, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6871898f, SUC_lexicon_tags_25 },
    { 0xbad18990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf118993, SUC_lexicon_tags_21 },
    { 0xd7118994, SUC_lexicon_tags_15 },
    { 0xd4318995, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb18998, SUC_lexicon_tags_22 },
    { 0xdcd18999, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2471899d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed189a5, SUC_lexicon_tags_11 },
    { 0xf35189a6, SUC_lexicon_tags_15 },
    { 0x6dd189a7, SUC_lexicon_tags_33 },
    { 0x8c1189a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b189ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1189b2, SUC_lexicon_tags_87 },
    { 0xd75189b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7189b6, SUC_lexicon_tags_32 },
    { 0xea3189b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f189bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7189bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcb7189c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69189c5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d189cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf9189cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51189cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x173189d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd189d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6b189d9, SUC_lexicon_tags_17 },
    { 0x4a1189d9, SUC_lexicon_tags_15 },
    { 0x839189d9, SUC_lexicon_tags_11 },
    { 0xca3189d9, SUC_lexicon_tags_37 },
    { 0xb63189dd, SUC_lexicon_tags_79 },
    { 0xf8f189de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7189e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x317189e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab189e9, SUC_lexicon_tags_37 },
    { 0xce5189ea, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625189f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x155189f5, SUC_lexicon_tags_74 },
    { 0xa51189f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd189fa, SUC_lexicon_tags_17 },
    { 0xf11189fb, SUC_lexicon_tags_14 },
    { 0x937189fb, SUC_lexicon_tags_9 },
    { 0xfbd189fc, SUC_lexicon_tags_37 },
    { 0x515189fe, SUC_lexicon_tags_15 },
    { 0x701189ff, SUC_lexicon_tags_25 },
    { 0x39b18a00, SUC_lexicon_tags_7 },
    { 0xca7189fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa3d18a03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d518a05, SUC_lexicon_tags_11 },
    { 0x67518a06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x69518a09, SUC_lexicon_tags_21 },
    { 0x9c918a09, SUC_lexicon_tags_17 },
    { 0x24f18a0a, SUC_lexicon_tags_17 },
    { 0xfe118a0c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d18a11, SUC_lexicon_tags_2 },
    { 0x7a118a11, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf18a15, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58318a1e, SUC_lexicon_tags_10 },
    { 0xf6918a1f, SUC_lexicon_tags_25 },
    { 0xa8518a20, SUC_lexicon_tags_15 },
    { 0x32d18a20, SUC_lexicon_tags_23 },
    { 0xec318a22, SUC_lexicon_tags_2 },
    { 0xeb918a21, SUC_lexicon_tags_25 },
    { 0x63b18a24, SUC_lexicon_tags_25 },
    { 0xfa118a24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f918a27, SUC_lexicon_tags_87 },
    { 0x2c118a27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d18a2c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df18a33, SUC_lexicon_tags_30 },
    { 0xa2518a33, SUC_lexicon_tags_17 },
    { 0xff18a35, SUC_lexicon_tags_13 },
    { 0x6b718a36, SUC_lexicon_tags_17 },
    { 0xa5f18a36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf318a39, SUC_lexicon_tags_29 },
    { 0x20f18a3a, SUC_lexicon_tags_37 },
    { 0x5918a3b, SUC_lexicon_tags_60 },
    { 0x4df18a3c, SUC_lexicon_tags_17 },
    { 0xa9718a3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b318a3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d718a42, SUC_lexicon_tags_10 },
    { 0xc3518a43, SUC_lexicon_tags_25 },
    { 0x20b18a44, SUC_lexicon_tags_33 },
    { 0x68918a45, SUC_lexicon_tags_39 },
    { 0xc8518a44, SUC_lexicon_tags_174 },
    { 0x87b18a47, SUC_lexicon_tags_16 },
    { 0xb7b18a48, SUC_lexicon_tags_17 },
    { 0x6bf18a47, SUC_lexicon_tags_1 },
    { 0x30918a43, SUC_lexicon_tags_30 },
    { 0x33118a4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1118a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6f18a51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed18a57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2118a59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa918a5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44318a5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd18a61, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3af18a64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b718a66, SUC_lexicon_tags_4 },
    { 0x9e718a67, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x34d18a69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf18a6b, SUC_lexicon_tags_15 },
    { 0x6ff18a6c, SUC_lexicon_tags_19 },
    { 0x54118a6b, SUC_lexicon_tags_23 },
    { 0x2b18a6d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f18a71, SUC_lexicon_tags_45 },
    { 0x35d18a72, SUC_lexicon_tags_43 },
    { 0xd3118a73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3918a76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb18a7b, SUC_lexicon_tags_62 },
    { 0x30d18a7b, SUC_lexicon_tags_47 },
    { 0xbb918a7d, SUC_lexicon_tags_25 },
    { 0xf0d18a7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x68318a81, SUC_lexicon_tags_1 },
    { 0xb3f18a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd18a88, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x83118a8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18a8e, SUC_lexicon_tags_11 },
    { 0x6e318a8f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7918a92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc318a96, SUC_lexicon_tags_43 },
    { 0xefd18a97, SUC_lexicon_tags_27 },
    { 0x4d318a96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3718a9c, SUC_lexicon_tags_15 },
    { 0xa8318a9d, SUC_lexicon_tags_15 },
    { 0xc2918a9e, SUC_lexicon_tags_23 },
    { 0xccb18a9e, SUC_lexicon_tags_33 },
    { 0x76918a9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4d18aa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f518aab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61718aae, SUC_lexicon_tags_37 },
    { 0x4518aae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81318ab1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc918ab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c518aba, SUC_lexicon_tags_160 },
    { 0x3e918aba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x83d18abd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5118ac1, SUC_lexicon_tags_31 },
    { 0x90918ac1, SUC_lexicon_tags_25 },
    { 0x6d518ac1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5918ac6, SUC_lexicon_tags_10 },
    { 0x11f18ac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff18aca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd18ad1, SUC_lexicon_tags_17 },
    { 0x1a918ad2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9118ad7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60918ad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8718adb, SUC_lexicon_tags_45 },
    { 0x4df18adc, SUC_lexicon_tags_151 },
    { 0x4c718adb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5518ae0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d18ae5, SUC_lexicon_tags_40 },
    { 0x98f18ae5, SUC_lexicon_tags_82 },
    { 0xcb318ae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd18aea, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x50918aed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f18af2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf18af6, SUC_lexicon_tags_11 },
    { 0x4b18af6, SUC_lexicon_tags_25 },
    { 0xdbd18af6, SUC_lexicon_tags_37 },
    { 0xba918af8, SUC_lexicon_tags_29 },
    { 0x2f518af7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd518afe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68318b02, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0xed118b04, SUC_lexicon_tags_25 },
    { 0xdd318b05, SUC_lexicon_tags_66 },
    { 0x76918b04, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8b18b08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc518b0a, SUC_lexicon_tags_15 },
    { 0xea118b0b, SUC_lexicon_tags_56 },
    { 0x27f18b0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f18b13, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72518b1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60718b20, SUC_lexicon_tags_19 },
    { 0xc1318b21, SUC_lexicon_tags_38 },
    { 0x60f18b21, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x51518b25, SUC_lexicon_tags_7 },
    { 0xc8b18b26, SUC_lexicon_tags_12 },
    { 0xb1f18b27, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0318b2a, SUC_lexicon_tags_2 },
    { 0xc0718b2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5318b32, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb18b3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f18b3f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70118b43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d18b46, SUC_lexicon_tags_1 },
    { 0xfaf18b46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8718b4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b18b52, SUC_lexicon_tags_60 },
    { 0x49518b53, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d18b56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f18b5d, SUC_lexicon_tags_19 },
    { 0xd4318b5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d18b62, SUC_lexicon_tags_28 },
    { 0xba118b63, SUC_lexicon_tags_83 },
    { 0xfd518b63, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b18b6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77118b75, SUC_lexicon_tags_7 },
    { 0xfd118b76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f18b7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x94518b7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x39118b81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19718b88, SUC_lexicon_tags_11 },
    { 0x1d318b89, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x44118b8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee118b8d, SUC_lexicon_tags_2 },
    { 0xecf18b8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76918b91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4718b94, SUC_lexicon_tags_100 },
    { 0x5b318b94, SUC_lexicon_tags_27 },
    { 0xa9718b96, SUC_lexicon_tags_27 },
    { 0xe2518b96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10718b99, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb518b9c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x54118b9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c118ba4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b18ba8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8918bab, SUC_lexicon_tags_26 },
    { 0xe3718bac, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xd2b18bae, SUC_lexicon_tags_25 },
    { 0x1e718baf, SUC_lexicon_tags_114 },
    { 0x16f18bb0, SUC_lexicon_tags_159 },
    { 0x5c918bb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee718bb6, SUC_lexicon_tags_66 },
    { 0x5fd18bb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f18bba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4318bc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b918bc5, SUC_lexicon_tags_14 },
    { 0x19318bc6, SUC_lexicon_tags_27 },
    { 0x78318bc7, SUC_lexicon_tags_1 },
    { 0x95718bc8, SUC_lexicon_tags_25 },
    { 0x74f18bc9, SUC_lexicon_tags_32 },
    { 0x61d18bca, SUC_lexicon_tags_37 },
    { 0x7c118bcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc118bcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc518bd1, SUC_lexicon_tags_28 },
    { 0xa3d18bd1, SUC_lexicon_tags_19 },
    { 0xa5d18bd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f18bdd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d18be0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50718be5, SUC_lexicon_tags_12 },
    { 0xf3f18be6, SUC_lexicon_tags_1 },
    { 0x97918be7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe9b18be9, SUC_lexicon_tags_19 },
    { 0x51718be9, SUC_lexicon_tags_47 },
    { 0x60718beb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66918bf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2918bfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f118bff, SUC_lexicon_tags_208 },
    { 0x23718bff, SUC_lexicon_tags_8 },
    { 0xb9718c01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8118c03, SUC_lexicon_tags_11 },
    { 0xa5118c04, SUC_lexicon_tags_15 },
    { 0x1af18c04, SUC_lexicon_tags_45 },
    { 0xad18c05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca318c0c, SUC_lexicon_tags_103 },
    { 0xf1f18c0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6bb18c0f, SUC_lexicon_tags_19 },
    { 0xdf518c10, SUC_lexicon_tags_38 },
    { 0x1a118c11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29518c14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb18c17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e318c19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32918c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f118c22, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b518c26, SUC_lexicon_tags_11 },
    { 0xcb118c27, SUC_lexicon_tags_11 },
    { 0x1a918c28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b18c2d, SUC_lexicon_tags_27 },
    { 0x49518c2e, SUC_lexicon_tags_7 },
    { 0xd7b18c2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec318c35, SUC_lexicon_tags_1 },
    { 0x78f18c35, SUC_lexicon_tags_7 },
    { 0x49518c37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f318c3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed18c3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28118c42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d118c48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f18c4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1318c58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7318c5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c518c64, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51918c6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67918c79, SUC_lexicon_tags_2 },
    { 0x11118c7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb0918c7c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9318c7f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x44518c81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7b18c83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1518c86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1a318c88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7918c8a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d18c8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d18c90, SUC_lexicon_tags_15 },
    { 0x85b18c91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b18c95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b18c98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f18c9b, SUC_lexicon_tags_37 },
    { 0x13518c9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x61b18c9e, SUC_lexicon_tags_55 },
    { 0x6db18c9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d18ca2, SUC_lexicon_tags_13 },
    { 0x1f318ca3, SUC_lexicon_tags_37 },
    { 0x3a318ca4, SUC_lexicon_tags_9 },
    { 0x7d318ca2, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f18cac, SUC_lexicon_tags_10 },
    { 0xbab18cad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f518cb0, SUC_lexicon_tags_15 },
    { 0x97118cb1, SUC_lexicon_tags_25 },
    { 0x16d18cb1, SUC_lexicon_tags_85 },
    { 0xbf518cb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b18cbd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9918cc3, SUC_lexicon_tags_37 },
    { 0x95f18cc4, SUC_lexicon_tags_55 },
    { 0xb1318cc5, SUC_lexicon_tags_2 },
    { 0x9af18cc5, SUC_lexicon_tags_17 },
    { 0xbd518cc6, SUC_lexicon_tags_37 },
    { 0x4b318cc8, SUC_lexicon_tags_17 },
    { 0xe318cc9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x18f18ccb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab18ccf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b18cd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b318cd4, SUC_lexicon_tags_11 },
    { 0x68318cd5, SUC_lexicon_tags_15 },
    { 0x73518cd6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc718cd8, SUC_lexicon_tags_43 },
    { 0xddb18cd8, SUC_lexicon_tags_19 },
    { 0x8cf18cda, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93318cde, SUC_lexicon_tags_55 },
    { 0xcf18cdf, SUC_lexicon_tags_33 },
    { 0x44118cdf, SUC_lexicon_tags_2 },
    { 0x51b18ce0, SUC_lexicon_tags_30 },
    { 0xacb18cdf, SUC_lexicon_tags_48 },
    { 0xd9b18ce2, SUC_lexicon_tags_11 },
    { 0xe2118cde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b18cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49918cf7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e118cfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98b18cfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d18d00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b18d03, SUC_lexicon_tags_60 },
    { 0x68718d04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d18d0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a718d12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29318d18, SUC_lexicon_tags_1 },
    { 0xefb18d19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfb318d1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d918d1d, SUC_lexicon_tags_12 },
    { 0xba718d1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf18d21, SUC_lexicon_tags_37 },
    { 0xef718d22, SUC_lexicon_tags_15 },
    { 0x5b118d22, SUC_lexicon_tags_25 },
    { 0x1318d22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f18d28, SUC_lexicon_tags_25 },
    { 0x4118d29, SUC_lexicon_tags_17 },
    { 0x69518d29, SUC_lexicon_tags_42 },
    { 0xd3718d29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f18d32, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff18d38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e118d3d, SUC_lexicon_tags_19 },
    { 0x6ab18d3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b918d45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d18d48, SUC_lexicon_tags_35 },
    { 0xb5118d49, SUC_lexicon_tags_6 },
    { 0x4d518d4a, SUC_lexicon_tags_63 },
    { 0x55718d4a, SUC_lexicon_tags_65 },
    { 0xf3318d4a, SUC_lexicon_tags_1 },
    { 0xd2518d4d, SUC_lexicon_tags_25 },
    { 0x95f18d4d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af18d58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d18d5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ab18d5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80118d65, SUC_lexicon_tags_65 },
    { 0x3f318d66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a318d68, SUC_lexicon_tags_25 },
    { 0xc3318d69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b118d6b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x10118d6e, SUC_lexicon_tags_33 },
    { 0x4cb18d6e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2c718d71, SUC_lexicon_tags_40 },
    { 0xb8718d71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8518d77, SUC_lexicon_tags_0 },
    { 0x51718d78, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1118d7d, SUC_lexicon_tags_37 },
    { 0xdb718d7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65718d80, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d18d83, SUC_lexicon_tags_45 },
    { 0x1cd18d84, SUC_lexicon_tags_5 },
    { 0x58f18d85, SUC_lexicon_tags_12 },
    { 0x1118d86, SUC_lexicon_tags_6 },
    { 0x44918d85, SUC_lexicon_tags_65 },
    { 0xf7118d88, SUC_lexicon_tags_7 },
    { 0x69518d89, SUC_lexicon_tags_33 },
    { 0x86318d89, SUC_lexicon_tags_30 },
    { 0xf6318d85, SUC_lexicon_tags_14 },
    { 0xd4118d88, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21118d92, SUC_lexicon_tags_14 },
    { 0x4ab18d92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac918d95, SUC_lexicon_tags_7 },
    { 0x4c518d95, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0718d99, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3318d9c, SUC_lexicon_tags_2 },
    { 0x35918d9d, SUC_lexicon_tags_25 },
    { 0xe4918d9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf18da3, SUC_lexicon_tags_32 },
    { 0x41f18da4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82318da7, SUC_lexicon_tags_17 },
    { 0xa0518da7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b18daf, SUC_lexicon_tags_27 },
    { 0x40b18db0, SUC_lexicon_tags_17 },
    { 0x5f518daf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d18db6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc318db9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb718dbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42718dbe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1918dc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d18dc7, SUC_lexicon_tags_25 },
    { 0xddb18dc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40718dca, SUC_lexicon_tags_18 },
    { 0xc1718dcb, SUC_lexicon_tags_6 },
    { 0x21918dcc, SUC_lexicon_tags_39 },
    { 0xe3118dca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93318dd0, SUC_lexicon_tags_236 },
    { 0xccf18dd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb18dd5, SUC_lexicon_tags_10 },
    { 0x97718dd5, SUC_lexicon_tags_87 },
    { 0x2f918dd7, SUC_lexicon_tags_44 },
    { 0x44518dd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab18ddb, SUC_lexicon_tags_15 },
    { 0x80f18ddc, SUC_lexicon_tags_48 },
    { 0x6dd18ddd, SUC_lexicon_tags_11 },
    { 0x90d18ddc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d18de3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x14718de5, SUC_lexicon_tags_7 },
    { 0x5b118de6, SUC_lexicon_tags_37 },
    { 0xf2d18de7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x10518de9, SUC_lexicon_tags_9 },
    { 0x58918de9, SUC_lexicon_tags_15 },
    { 0x41b18deb, SUC_lexicon_tags_134 },
    { 0x4d918dec, SUC_lexicon_tags_12 },
    { 0x65318ded, SUC_lexicon_tags_32 },
    { 0x37518dec, SUC_lexicon_tags_60 },
    { 0x3ed18dee, SUC_lexicon_tags_9 },
    { 0x2e118ded, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43318df2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d18df6, SUC_lexicon_tags_7 },
    { 0x88d18df6, SUC_lexicon_tags_27 },
    { 0x5318df7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb18dfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed518e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb18e0c, SUC_lexicon_tags_2 },
    { 0x3bf18e0d, SUC_lexicon_tags_38 },
    { 0x43118e0e, SUC_lexicon_tags_19 },
    { 0x1f18e0f, SUC_lexicon_tags_7 },
    { 0x76718e0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe918e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad318e16, SUC_lexicon_tags_49 },
    { 0x25b18e17, SUC_lexicon_tags_6 },
    { 0x92318e16, SUC_lexicon_tags_49 },
    { 0x40f18e19, SUC_lexicon_tags_43 },
    { 0x41318e17, SUC_lexicon_tags_45 },
    { 0xc9918e1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x83718e1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98718e20, SUC_lexicon_tags_25 },
    { 0x47518e21, SUC_lexicon_tags_8 },
    { 0xa3118e22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f18e26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe718e29, SUC_lexicon_tags_47 },
    { 0x36718e2a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed18e2d, SUC_lexicon_tags_17 },
    { 0x59318e2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d518e30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb7f18e32, SUC_lexicon_tags_32 },
    { 0xb5718e32, SUC_lexicon_tags_12 },
    { 0xacb18e33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa518e36, SUC_lexicon_tags_6 },
    { 0x52318e37, SUC_lexicon_tags_11 },
    { 0xd518e38, SUC_lexicon_tags_15 },
    { 0x33f18e39, SUC_lexicon_tags_15 },
    { 0x67d18e39, SUC_lexicon_tags_45 },
    { 0x67718e3b, SUC_lexicon_tags_22 },
    { 0x55518e3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e718e42, SUC_lexicon_tags_33 },
    { 0x32718e42, SUC_lexicon_tags_66 },
    { 0xe318e42, SUC_lexicon_tags_37 },
    { 0xbcf18e45, SUC_lexicon_tags_11 },
    { 0x16118e46, SUC_lexicon_tags_14 },
    { 0x5a318e47, SUC_lexicon_tags_87 },
    { 0x9dd18e47, SUC_lexicon_tags_57 },
    { 0xe0918e45, SUC_lexicon_tags_38 },
    { 0x80d18e4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99718e4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3118e4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d18e52, SUC_lexicon_tags_60 },
    { 0xa9d18e53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25f18e55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb118e57, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f18e5c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4db18e5e, SUC_lexicon_tags_7 },
    { 0x6fb18e5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d918e61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1118e64, SUC_lexicon_tags_63 },
    { 0x41d18e64, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbf18e67, SUC_lexicon_tags_9 },
    { 0x38318e67, SUC_lexicon_tags_42 },
    { 0x54718e69, SUC_lexicon_tags_11 },
    { 0xc1d18e6a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51718e72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf18e76, SUC_lexicon_tags_29 },
    { 0x25718e77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf2518e79, SUC_lexicon_tags_37 },
    { 0xeb318e7a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x81b18e7c, SUC_lexicon_tags_31 },
    { 0xf4718e7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5718e80, SUC_lexicon_tags_30 },
    { 0xc0518e81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe8b18e83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14918e87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76718e89, SUC_lexicon_tags_38 },
    { 0xc0718e8a, SUC_lexicon_tags_25 },
    { 0x3ff18e8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d18e91, SUC_lexicon_tags_6 },
    { 0x86518e92, SUC_lexicon_tags_49 },
    { 0xdb518e92, SUC_lexicon_tags_28 },
    { 0x8cb18e91, SUC_lexicon_tags_1 },
    { 0x80518e95, SUC_lexicon_tags_37 },
    { 0xbd118e96, SUC_lexicon_tags_146 },
    { 0x60518e97, SUC_lexicon_tags_68 },
    { 0xb4318e98, SUC_lexicon_tags_47 },
    { 0xa9918e95, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc518e9f, SUC_lexicon_tags_17 },
    { 0x27918e9f, SUC_lexicon_tags_63 },
    { 0xf7718ea1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16518ea8, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f18ead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7718eaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb18eb3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xff518eb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe318ebe, SUC_lexicon_tags_25 },
    { 0xd4518ebf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf718ec2, SUC_lexicon_tags_9 },
    { 0xf7118ec3, SUC_lexicon_tags_17 },
    { 0x13318ec4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf7f18ec6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ef18ec8, SUC_lexicon_tags_11 },
    { 0x13118ec9, SUC_lexicon_tags_663 },
    { 0, NULL },
    { 0x6db18ecb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd8718ecd, SUC_lexicon_tags_26 },
    { 0x12d18ece, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f918ed2, SUC_lexicon_tags_37 },
    { 0xf1f18ed2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd18ed6, SUC_lexicon_tags_12 },
    { 0xa4b18ed7, SUC_lexicon_tags_11 },
    { 0x63d18ed8, SUC_lexicon_tags_12 },
    { 0xff318ed7, SUC_lexicon_tags_15 },
    { 0xa4b18eda, SUC_lexicon_tags_33 },
    { 0xfaf18eda, SUC_lexicon_tags_25 },
    { 0xde518eda, SUC_lexicon_tags_17 },
    { 0x46d18eda, SUC_lexicon_tags_7 },
    { 0xcd718ede, SUC_lexicon_tags_120 },
    { 0xb1518edf, SUC_lexicon_tags_15 },
    { 0xbcf18ee0, SUC_lexicon_tags_16 },
    { 0x96f18ee1, SUC_lexicon_tags_25 },
    { 0x64318ee1, SUC_lexicon_tags_49 },
    { 0xf3918ee3, SUC_lexicon_tags_33 },
    { 0xe9518ee3, SUC_lexicon_tags_4 },
    { 0x46518ee5, SUC_lexicon_tags_37 },
    { 0x2fd18ee6, SUC_lexicon_tags_2 },
    { 0x8cf18ee7, SUC_lexicon_tags_7 },
    { 0x3e318edb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4518eef, SUC_lexicon_tags_39 },
    { 0x63f18eef, SUC_lexicon_tags_37 },
    { 0x23918eef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff118ef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f18efc, SUC_lexicon_tags_15 },
    { 0x88118efd, SUC_lexicon_tags_17 },
    { 0xd9718efe, SUC_lexicon_tags_37 },
    { 0x50318eff, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x19b18f01, SUC_lexicon_tags_27 },
    { 0x6b518f02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce718f05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2d18f07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cf18f09, SUC_lexicon_tags_25 },
    { 0x40318f09, SUC_lexicon_tags_25 },
    { 0x7e318f0a, SUC_lexicon_tags_19 },
    { 0xe518f0b, SUC_lexicon_tags_16 },
    { 0x13118f0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab518f13, SUC_lexicon_tags_92 },
    { 0x98d18f14, SUC_lexicon_tags_49 },
    { 0x3f718f15, SUC_lexicon_tags_35 },
    { 0x9d918f16, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9318f19, SUC_lexicon_tags_1 },
    { 0x63918f1a, SUC_lexicon_tags_32 },
    { 0x2d318f1b, SUC_lexicon_tags_43 },
    { 0xaa518f19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c918f20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2f18f22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab18f25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb118f27, SUC_lexicon_tags_37 },
    { 0x72318f27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57918f2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddd18f2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde518f32, SUC_lexicon_tags_17 },
    { 0x51918f32, SUC_lexicon_tags_13 },
    { 0x5a118f34, SUC_lexicon_tags_14 },
    { 0x4db18f35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb18f38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f718f3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b18f41, SUC_lexicon_tags_2 },
    { 0xd9518f42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e118f4b, SUC_lexicon_tags_12 },
    { 0xb3518f4b, SUC_lexicon_tags_5 },
    { 0xcb918f4d, SUC_lexicon_tags_1 },
    { 0xc0f18f4e, SUC_lexicon_tags_43 },
    { 0x6e318f4d, SUC_lexicon_tags_8 },
    { 0x48118f50, SUC_lexicon_tags_1 },
    { 0x9c118f50, SUC_lexicon_tags_7 },
    { 0xb4f18f52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a918f5a, SUC_lexicon_tags_2 },
    { 0x93318f5a, SUC_lexicon_tags_37 },
    { 0x9318f5c, SUC_lexicon_tags_141 },
    { 0xbbf18f5d, SUC_lexicon_tags_1 },
    { 0xe8918f5b, SUC_lexicon_tags_37 },
    { 0x95518f5f, SUC_lexicon_tags_17 },
    { 0x1f118f60, SUC_lexicon_tags_25 },
    { 0x3ff18f61, SUC_lexicon_tags_23 },
    { 0xbe918f61, SUC_lexicon_tags_1 },
    { 0x66918f63, SUC_lexicon_tags_62 },
    { 0x5718f64, SUC_lexicon_tags_7 },
    { 0x1cf18f64, SUC_lexicon_tags_49 },
    { 0xb9318f65, SUC_lexicon_tags_8 },
    { 0x56d18f66, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e918f6a, SUC_lexicon_tags_12 },
    { 0x5a118f6b, SUC_lexicon_tags_66 },
    { 0x6fd18f6b, SUC_lexicon_tags_15 },
    { 0xef18f6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af18f70, SUC_lexicon_tags_15 },
    { 0x35f18f70, SUC_lexicon_tags_37 },
    { 0xdcd18f71, SUC_lexicon_tags_16 },
    { 0xc5d18f73, SUC_lexicon_tags_37 },
    { 0xbb918f74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14718f76, SUC_lexicon_tags_7 },
    { 0x93118f77, SUC_lexicon_tags_16 },
    { 0x91318f78, SUC_lexicon_tags_17 },
    { 0x5b118f78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b118f7b, SUC_lexicon_tags_15 },
    { 0xaab18f7b, SUC_lexicon_tags_37 },
    { 0x15118f7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c918f80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x48318f83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d18f86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc718f8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78918f8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba918f90, SUC_lexicon_tags_2 },
    { 0x4718f91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb318f93, SUC_lexicon_tags_11 },
    { 0xa4f18f93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0318f96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd18f9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84f18f9d, SUC_lexicon_tags_1 },
    { 0x34918f9e, SUC_lexicon_tags_12 },
    { 0x82118f9f, SUC_lexicon_tags_39 },
    { 0x9b118fa0, SUC_lexicon_tags_43 },
    { 0x81d18fa1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x47118fa3, SUC_lexicon_tags_17 },
    { 0x13518fa4, SUC_lexicon_tags_0 },
    { 0xdb918fa5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3118fa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2718fad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab18fb1, SUC_lexicon_tags_8 },
    { 0x70d18fb2, SUC_lexicon_tags_17 },
    { 0x93b18fb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f18fb9, SUC_lexicon_tags_17 },
    { 0xa5118fb9, SUC_lexicon_tags_84 },
    { 0x3fd18fbb, SUC_lexicon_tags_12 },
    { 0xb318fbc, SUC_lexicon_tags_23 },
    { 0xd9d18fbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc718fbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85b18fc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bd18fc3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6718fcb, SUC_lexicon_tags_38 },
    { 0xf5918fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40318fce, SUC_lexicon_tags_49 },
    { 0x40b18fcf, SUC_lexicon_tags_16 },
    { 0x65318fcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5518fd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x53718fd6, SUC_lexicon_tags_44 },
    { 0x6c118fd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b318fdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf18fe5, SUC_lexicon_tags_49 },
    { 0x13b18fe5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22718fe8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9918fec, SUC_lexicon_tags_32 },
    { 0xb0b18fed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd18ff0, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23518ff4, SUC_lexicon_tags_17 },
    { 0xed18ff5, SUC_lexicon_tags_39 },
    { 0x2cb18ff6, SUC_lexicon_tags_1 },
    { 0xff918ff6, SUC_lexicon_tags_25 },
    { 0x52d18ff8, SUC_lexicon_tags_43 },
    { 0xb2d18ff7, SUC_lexicon_tags_55 },
    { 0x8e918ff4, SUC_lexicon_tags_149 },
    { 0xd6d18ffa, SUC_lexicon_tags_15 },
    { 0xf3b18ffc, SUC_lexicon_tags_25 },
    { 0x2d518ff7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x12119000, SUC_lexicon_tags_10 },
    { 0xef19000, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23319006, SUC_lexicon_tags_1 },
    { 0x23d19007, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d1900b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac919010, SUC_lexicon_tags_25 },
    { 0x31319011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45919018, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb1901e, SUC_lexicon_tags_14 },
    { 0x3f71901f, SUC_lexicon_tags_25 },
    { 0x61f19020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x60719022, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b19025, SUC_lexicon_tags_21 },
    { 0x73319025, SUC_lexicon_tags_65 },
    { 0xcdd19027, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f1902a, SUC_lexicon_tags_32 },
    { 0xaa51902a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d19034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f19038, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe231903b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2811903d, SUC_lexicon_tags_1 },
    { 0xcb31903d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ed19040, SUC_lexicon_tags_45 },
    { 0x41f19041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c719043, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb119049, SUC_lexicon_tags_37 },
    { 0x3a91904a, SUC_lexicon_tags_47 },
    { 0xad11904b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeab1904d, SUC_lexicon_tags_15 },
    { 0x4e11904d, SUC_lexicon_tags_1 },
    { 0x7631904f, SUC_lexicon_tags_15 },
    { 0x4af19050, SUC_lexicon_tags_47 },
    { 0x44f1904d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9119055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91519057, SUC_lexicon_tags_37 },
    { 0x99719058, SUC_lexicon_tags_7 },
    { 0x4cb19057, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd811905c, SUC_lexicon_tags_77 },
    { 0x6991905d, SUC_lexicon_tags_14 },
    { 0xbaf1905e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf19065, SUC_lexicon_tags_66 },
    { 0x9f319065, SUC_lexicon_tags_60 },
    { 0x66519065, SUC_lexicon_tags_17 },
    { 0x40d19066, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6271906a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa511906d, SUC_lexicon_tags_11 },
    { 0x5ad1906e, SUC_lexicon_tags_1 },
    { 0x73f1906f, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec519073, SUC_lexicon_tags_15 },
    { 0x94919074, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xac119077, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xced19079, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6d11907b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xda91907e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x62719081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d1908a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6af1908c, SUC_lexicon_tags_43 },
    { 0xa431908d, SUC_lexicon_tags_27 },
    { 0x73d1908e, SUC_lexicon_tags_60 },
    { 0xc4b1908d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a919093, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9119095, SUC_lexicon_tags_43 },
    { 0x19b19096, SUC_lexicon_tags_14 },
    { 0x319096, SUC_lexicon_tags_17 },
    { 0xc2319098, SUC_lexicon_tags_17 },
    { 0x64919099, SUC_lexicon_tags_30 },
    { 0xe2119099, SUC_lexicon_tags_31 },
    { 0xdcb19099, SUC_lexicon_tags_34 },
    { 0xddf1909a, SUC_lexicon_tags_37 },
    { 0x9d91909d, SUC_lexicon_tags_7 },
    { 0x92d19098, SUC_lexicon_tags_1 },
    { 0xff1909f, SUC_lexicon_tags_25 },
    { 0x4fd190a0, SUC_lexicon_tags_37 },
    { 0xa31909c, SUC_lexicon_tags_3 },
    { 0xc37190a2, SUC_lexicon_tags_37 },
    { 0x21b190a3, SUC_lexicon_tags_84 },
    { 0xe0b190a3, SUC_lexicon_tags_8 },
    { 0xd95190a5, SUC_lexicon_tags_12 },
    { 0x679190a5, SUC_lexicon_tags_28 },
    { 0xb49190a6, SUC_lexicon_tags_12 },
    { 0xde5190a8, SUC_lexicon_tags_17 },
    { 0xa61190a4, SUC_lexicon_tags_66 },
    { 0x57319097, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03190ae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7190b2, SUC_lexicon_tags_37 },
    { 0x26d190b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695190bb, SUC_lexicon_tags_25 },
    { 0xfbd190bc, SUC_lexicon_tags_1 },
    { 0x7b3190bd, SUC_lexicon_tags_15 },
    { 0xc83190bc, SUC_lexicon_tags_11 },
    { 0xad9190bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d190c3, SUC_lexicon_tags_15 },
    { 0x8e3190c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f7190c6, SUC_lexicon_tags_2 },
    { 0xc8f190c7, SUC_lexicon_tags_30 },
    { 0x9b190c8, SUC_lexicon_tags_38 },
    { 0x5f190c9, SUC_lexicon_tags_17 },
    { 0xf1f190c9, SUC_lexicon_tags_26 },
    { 0x6ff190cb, SUC_lexicon_tags_17 },
    { 0x6f3190c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb97190ce, SUC_lexicon_tags_10 },
    { 0x21b190cf, SUC_lexicon_tags_0 },
    { 0x2d190d0, SUC_lexicon_tags_2 },
    { 0xa87190ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b190d8, SUC_lexicon_tags_7 },
    { 0x1c7190d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x457190db, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9b5190dd, SUC_lexicon_tags_63 },
    { 0x749190de, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1190e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x479190e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605190ed, SUC_lexicon_tags_46 },
    { 0x4cf190ee, SUC_lexicon_tags_11 },
    { 0x111190ef, SUC_lexicon_tags_55 },
    { 0xcf7190ef, SUC_lexicon_tags_30 },
    { 0xeb5190ed, SUC_lexicon_tags_17 },
    { 0xc75190f2, SUC_lexicon_tags_15 },
    { 0x97b190f3, SUC_lexicon_tags_53 },
    { 0xf81190f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xded190f7, SUC_lexicon_tags_2 },
    { 0xb57190f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9119100, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b719104, SUC_lexicon_tags_37 },
    { 0x22d19105, SUC_lexicon_tags_4 },
    { 0x95119105, SUC_lexicon_tags_66 },
    { 0x9ab19105, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1ad19109, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd1910d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x17719110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb19115, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6f19117, SUC_lexicon_tags_12 },
    { 0xaa719118, SUC_lexicon_tags_16 },
    { 0x92119119, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1911c, SUC_lexicon_tags_17 },
    { 0x24d1911d, SUC_lexicon_tags_1 },
    { 0x96f1911c, SUC_lexicon_tags_27 },
    { 0xed11911f, SUC_lexicon_tags_47 },
    { 0x2811911f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c119124, SUC_lexicon_tags_23 },
    { 0xfb119125, SUC_lexicon_tags_25 },
    { 0x81b19126, SUC_lexicon_tags_7 },
    { 0x2cb19127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15b19129, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c91912d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbcf1912f, SUC_lexicon_tags_48 },
    { 0xe0919130, SUC_lexicon_tags_14 },
    { 0xbd319131, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f119133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f19138, SUC_lexicon_tags_37 },
    { 0x9ff19139, SUC_lexicon_tags_25 },
    { 0x26d19139, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f1913e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7a519140, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x70719143, SUC_lexicon_tags_25 },
    { 0x74119144, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d1914a, SUC_lexicon_tags_30 },
    { 0xb351914b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4751914f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35319154, SUC_lexicon_tags_11 },
    { 0xc0d19154, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd719157, SUC_lexicon_tags_12 },
    { 0xfcb19157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8d1915a, SUC_lexicon_tags_28 },
    { 0xa651915a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d1915f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2919166, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7b19168, SUC_lexicon_tags_0 },
    { 0xbcb19169, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f1916c, SUC_lexicon_tags_63 },
    { 0x371916d, SUC_lexicon_tags_7 },
    { 0x8451916c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae919171, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5119174, SUC_lexicon_tags_47 },
    { 0x6c319174, SUC_lexicon_tags_57 },
    { 0x13d19174, SUC_lexicon_tags_37 },
    { 0xc2f19174, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9251917d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cd1917f, SUC_lexicon_tags_0 },
    { 0xe771917f, SUC_lexicon_tags_49 },
    { 0x93919181, SUC_lexicon_tags_7 },
    { 0x92919181, SUC_lexicon_tags_49 },
    { 0x5891917f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc0719185, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5db19187, SUC_lexicon_tags_28 },
    { 0xd0d19188, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7931918c, SUC_lexicon_tags_6 },
    { 0xee91918d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e319191, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d19194, SUC_lexicon_tags_14 },
    { 0xd3b19195, SUC_lexicon_tags_11 },
    { 0xe9719195, SUC_lexicon_tags_37 },
    { 0x96d19196, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511191a1, SUC_lexicon_tags_11 },
    { 0x8d5191a1, SUC_lexicon_tags_6 },
    { 0x8c3191a2, SUC_lexicon_tags_48 },
    { 0xdef191a3, SUC_lexicon_tags_23 },
    { 0x979191a5, SUC_lexicon_tags_11 },
    { 0xc35191a6, SUC_lexicon_tags_12 },
    { 0x63191a7, SUC_lexicon_tags_45 },
    { 0x89b191a6, SUC_lexicon_tags_15 },
    { 0xdbb191a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad191ac, SUC_lexicon_tags_17 },
    { 0xebf191ac, SUC_lexicon_tags_25 },
    { 0xe69191ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x493191b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x567191b3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5191b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59191bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc17191be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785191c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759191c7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89191ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x785191d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9191d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fb191d7, SUC_lexicon_tags_11 },
    { 0xb9d191d8, SUC_lexicon_tags_11 },
    { 0xa85191d8, SUC_lexicon_tags_11 },
    { 0x633191d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb17191dc, SUC_lexicon_tags_37 },
    { 0x225191dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6db191df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d191e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfd3191e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1191e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d191ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f191f0, SUC_lexicon_tags_0 },
    { 0xc9d191f1, SUC_lexicon_tags_26 },
    { 0x8b3191f1, SUC_lexicon_tags_15 },
    { 0x465191f1, SUC_lexicon_tags_19 },
    { 0x1e1191f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1191f8, SUC_lexicon_tags_43 },
    { 0x871191f8, SUC_lexicon_tags_11 },
    { 0xa9d191fa, SUC_lexicon_tags_26 },
    { 0x9fd191fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x13191fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d19202, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3719205, SUC_lexicon_tags_12 },
    { 0x40b19206, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9951920b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x3071920e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a719215, SUC_lexicon_tags_24 },
    { 0x5db19216, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3519218, SUC_lexicon_tags_49 },
    { 0xdf519219, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171921e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b719221, SUC_lexicon_tags_31 },
    { 0x14519222, SUC_lexicon_tags_11 },
    { 0x2f319223, SUC_lexicon_tags_2 },
    { 0xa7119223, SUC_lexicon_tags_6 },
    { 0x64b19225, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd519229, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x33d1922b, SUC_lexicon_tags_19 },
    { 0x4f11922c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd551922f, SUC_lexicon_tags_15 },
    { 0x57119230, SUC_lexicon_tags_43 },
    { 0x5a119231, SUC_lexicon_tags_17 },
    { 0xe3d1922f, SUC_lexicon_tags_49 },
    { 0x7cd1922f, SUC_lexicon_tags_38 },
    { 0xebb19234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6519237, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa411923b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65519243, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33519247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d19252, SUC_lexicon_tags_11 },
    { 0x4719252, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1519259, SUC_lexicon_tags_17 },
    { 0xf531925a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd1925d, SUC_lexicon_tags_15 },
    { 0x331925e, SUC_lexicon_tags_11 },
    { 0x23b1925f, SUC_lexicon_tags_15 },
    { 0x2471925e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ef19262, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f19265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc611926a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b1926f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x95d19271, SUC_lexicon_tags_2 },
    { 0x39f19271, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xd0519274, SUC_lexicon_tags_19 },
    { 0xcfd19275, SUC_lexicon_tags_11 },
    { 0xc3f19275, SUC_lexicon_tags_25 },
    { 0xfe319277, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b19283, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee919286, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d1928a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdc51928c, SUC_lexicon_tags_37 },
    { 0x6811928d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92119294, SUC_lexicon_tags_14 },
    { 0x70f19295, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9e919297, SUC_lexicon_tags_15 },
    { 0xe0f19298, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f1929c, SUC_lexicon_tags_7 },
    { 0xbc91929d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f192a0, SUC_lexicon_tags_33 },
    { 0x27192a0, SUC_lexicon_tags_4 },
    { 0x5d1192a0, SUC_lexicon_tags_11 },
    { 0x2f3192a3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89192a7, SUC_lexicon_tags_63 },
    { 0x53b192a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d192ac, SUC_lexicon_tags_0 },
    { 0x281192ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb192b2, SUC_lexicon_tags_1 },
    { 0xc8b192b2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39192ba, SUC_lexicon_tags_15 },
    { 0x6df192ba, SUC_lexicon_tags_49 },
    { 0x41192bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x349192be, SUC_lexicon_tags_31 },
    { 0x5db192be, SUC_lexicon_tags_3 },
    { 0x9b9192bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129192d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15192d4, SUC_lexicon_tags_17 },
    { 0x4cf192d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92d192d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7192da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x383192dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a9192df, SUC_lexicon_tags_7 },
    { 0x71192e0, SUC_lexicon_tags_61 },
    { 0xc15192e1, SUC_lexicon_tags_0 },
    { 0x237192e1, SUC_lexicon_tags_25 },
    { 0x89f192e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7192ec, SUC_lexicon_tags_1 },
    { 0x36d192ed, SUC_lexicon_tags_15 },
    { 0xa83192ee, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15192f5, SUC_lexicon_tags_15 },
    { 0x61d192f6, SUC_lexicon_tags_82 },
    { 0xe45192f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89192fa, SUC_lexicon_tags_47 },
    { 0xa87192fb, SUC_lexicon_tags_17 },
    { 0xc5192fc, SUC_lexicon_tags_16 },
    { 0xc83192fd, SUC_lexicon_tags_14 },
    { 0x205192fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f719308, SUC_lexicon_tags_1 },
    { 0x90d19309, SUC_lexicon_tags_17 },
    { 0x68119308, SUC_lexicon_tags_11 },
    { 0x4651930b, SUC_lexicon_tags_27 },
    { 0x95519309, SUC_lexicon_tags_25 },
    { 0x9519308, SUC_lexicon_tags_37 },
    { 0x9231930e, SUC_lexicon_tags_38 },
    { 0x9011930b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82719314, SUC_lexicon_tags_7 },
    { 0xeeb19315, SUC_lexicon_tags_747 },
    { 0x8b719315, SUC_lexicon_tags_11 },
    { 0xb6119317, SUC_lexicon_tags_318 },
    { 0x98d19317, SUC_lexicon_tags_26 },
    { 0x5a119319, SUC_lexicon_tags_9 },
    { 0xcab19319, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98319322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb1932a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf231932d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe919332, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54319334, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19339, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a51933c, SUC_lexicon_tags_113 },
    { 0xc5f1933d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d19343, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a319347, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e719349, SUC_lexicon_tags_6 },
    { 0x9811934a, SUC_lexicon_tags_12 },
    { 0xcbf1934b, SUC_lexicon_tags_47 },
    { 0xb0d1934b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62719351, SUC_lexicon_tags_113 },
    { 0x3c719351, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x82119354, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd919356, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4971935b, SUC_lexicon_tags_27 },
    { 0xfb31935c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c51935e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x31119360, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d19365, SUC_lexicon_tags_1 },
    { 0xead19366, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf19369, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc251936d, SUC_lexicon_tags_59 },
    { 0x38d1936e, SUC_lexicon_tags_1 },
    { 0x70b1936f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd919371, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19719373, SUC_lexicon_tags_21 },
    { 0xeed19374, SUC_lexicon_tags_2 },
    { 0x62319374, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e719377, SUC_lexicon_tags_56 },
    { 0xf2919378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81f1937a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d19385, SUC_lexicon_tags_47 },
    { 0x15919386, SUC_lexicon_tags_27 },
    { 0xe2b19386, SUC_lexicon_tags_2 },
    { 0x72719386, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf1938b, SUC_lexicon_tags_17 },
    { 0x8e91938c, SUC_lexicon_tags_65 },
    { 0xf031938d, SUC_lexicon_tags_2 },
    { 0x1c71938e, SUC_lexicon_tags_11 },
    { 0xf971938e, SUC_lexicon_tags_60 },
    { 0xf9d19390, SUC_lexicon_tags_0 },
    { 0xa0519391, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0919397, SUC_lexicon_tags_15 },
    { 0x10119398, SUC_lexicon_tags_2 },
    { 0xce719399, SUC_lexicon_tags_37 },
    { 0x1c119397, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc091939c, SUC_lexicon_tags_9 },
    { 0xfb1939d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5193a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe61193a5, SUC_lexicon_tags_47 },
    { 0xe8d193a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb13193a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63193ac, SUC_lexicon_tags_14 },
    { 0x163193ac, SUC_lexicon_tags_25 },
    { 0xc57193ad, SUC_lexicon_tags_60 },
    { 0xe45193ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf85193b1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587193b6, SUC_lexicon_tags_30 },
    { 0xc63193b7, SUC_lexicon_tags_11 },
    { 0x6dd193b8, SUC_lexicon_tags_17 },
    { 0xc87193b9, SUC_lexicon_tags_10 },
    { 0xca7193ba, SUC_lexicon_tags_27 },
    { 0x525193b7, SUC_lexicon_tags_1 },
    { 0xd5193bc, SUC_lexicon_tags_23 },
    { 0x6f3193ba, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51193c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbe7193c2, SUC_lexicon_tags_12 },
    { 0x455193c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44d193c5, SUC_lexicon_tags_17 },
    { 0xce3193c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe75193c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13b193ca, SUC_lexicon_tags_47 },
    { 0x84b193ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b193cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d193d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d193d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x405193d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1eb193db, SUC_lexicon_tags_27 },
    { 0xa49193dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5193df, SUC_lexicon_tags_46 },
    { 0x4a7193e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa79193e2, SUC_lexicon_tags_12 },
    { 0x487193e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10d193e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x17b193e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735193ef, SUC_lexicon_tags_11 },
    { 0x56b193f0, SUC_lexicon_tags_33 },
    { 0x4d193f0, SUC_lexicon_tags_7 },
    { 0x7b193f1, SUC_lexicon_tags_14 },
    { 0x7e5193f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63193fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d193fe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad19404, SUC_lexicon_tags_34 },
    { 0x45119405, SUC_lexicon_tags_3 },
    { 0xd9f19406, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd719411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b19415, SUC_lexicon_tags_17 },
    { 0x28719416, SUC_lexicon_tags_87 },
    { 0xd2119416, SUC_lexicon_tags_25 },
    { 0xfdd19418, SUC_lexicon_tags_0 },
    { 0xd0519419, SUC_lexicon_tags_45 },
    { 0xb3b1941a, SUC_lexicon_tags_7 },
    { 0x76519419, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b1941e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f119425, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b19429, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf611942b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c91942f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf119434, SUC_lexicon_tags_11 },
    { 0x1b719435, SUC_lexicon_tags_17 },
    { 0x8ad19436, SUC_lexicon_tags_7 },
    { 0x6a119436, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x98919439, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf611943b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6151943e, SUC_lexicon_tags_17 },
    { 0x61b1943f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1919443, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27319447, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5391944a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bd1944c, SUC_lexicon_tags_11 },
    { 0x1fb1944d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d519453, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x11f19455, SUC_lexicon_tags_33 },
    { 0x22f19455, SUC_lexicon_tags_17 },
    { 0x81719456, SUC_lexicon_tags_74 },
    { 0x15519456, SUC_lexicon_tags_11 },
    { 0xd3319459, SUC_lexicon_tags_43 },
    { 0xa791945a, SUC_lexicon_tags_63 },
    { 0x7e919455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97519464, SUC_lexicon_tags_7 },
    { 0x39519465, SUC_lexicon_tags_27 },
    { 0xe8719465, SUC_lexicon_tags_65 },
    { 0x39d19465, SUC_lexicon_tags_77 },
    { 0x58f19468, SUC_lexicon_tags_39 },
    { 0x40d19467, SUC_lexicon_tags_87 },
    { 0x2a71946a, SUC_lexicon_tags_14 },
    { 0x92f1946b, SUC_lexicon_tags_2 },
    { 0x1c31946a, SUC_lexicon_tags_1 },
    { 0xb3b1946d, SUC_lexicon_tags_58 },
    { 0x8c31946a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b71947a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa211947f, SUC_lexicon_tags_37 },
    { 0x55f19480, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf19485, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd19489, SUC_lexicon_tags_9 },
    { 0xc1519489, SUC_lexicon_tags_33 },
    { 0x9bb1948b, SUC_lexicon_tags_48 },
    { 0x46d1948c, SUC_lexicon_tags_9 },
    { 0x651948a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6119490, SUC_lexicon_tags_15 },
    { 0x71319490, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5519496, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8331949a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc91949c, SUC_lexicon_tags_16 },
    { 0x8931949c, SUC_lexicon_tags_32 },
    { 0xe31949d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbb7194a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x635194a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f3194a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f194a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3194ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2b194ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17194b1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x875194b3, SUC_lexicon_tags_16 },
    { 0xae7194b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49194b8, SUC_lexicon_tags_7 },
    { 0x33d194b9, SUC_lexicon_tags_44 },
    { 0x1dd194b9, SUC_lexicon_tags_12 },
    { 0xbc9194b9, SUC_lexicon_tags_32 },
    { 0xdd7194b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff194c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x995194c2, SUC_lexicon_tags_11 },
    { 0x51d194c3, SUC_lexicon_tags_56 },
    { 0x42d194c2, SUC_lexicon_tags_15 },
    { 0x5d1194c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x373194c7, SUC_lexicon_tags_37 },
    { 0xa8b194c8, SUC_lexicon_tags_32 },
    { 0x2fd194c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551194d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe3194d2, SUC_lexicon_tags_17 },
    { 0xc51194d2, SUC_lexicon_tags_12 },
    { 0xe4b194d4, SUC_lexicon_tags_30 },
    { 0x80d194d2, SUC_lexicon_tags_12 },
    { 0x37f194d6, SUC_lexicon_tags_113 },
    { 0x313194d6, SUC_lexicon_tags_17 },
    { 0x71194d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5194dc, SUC_lexicon_tags_19 },
    { 0x385194dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x491194e0, SUC_lexicon_tags_15 },
    { 0xe47194e0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x443194e3, SUC_lexicon_tags_2 },
    { 0xf5f194e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945194e8, SUC_lexicon_tags_1 },
    { 0x3ad194e9, SUC_lexicon_tags_146 },
    { 0x77b194ea, SUC_lexicon_tags_29 },
    { 0x3cb194ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1c1194ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf194f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x103194f4, SUC_lexicon_tags_60 },
    { 0x2f5194f5, SUC_lexicon_tags_37 },
    { 0x1194f6, SUC_lexicon_tags_17 },
    { 0x8d9194f7, SUC_lexicon_tags_47 },
    { 0xe11194f6, SUC_lexicon_tags_11 },
    { 0xdc9194f9, SUC_lexicon_tags_37 },
    { 0xfd5194f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649194fe, SUC_lexicon_tags_42 },
    { 0xe97194ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0319507, SUC_lexicon_tags_43 },
    { 0xab19507, SUC_lexicon_tags_37 },
    { 0x37319507, SUC_lexicon_tags_27 },
    { 0xf7719509, SUC_lexicon_tags_45 },
    { 0xd0d19509, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2331950d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xded19510, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d19515, SUC_lexicon_tags_17 },
    { 0x97119515, SUC_lexicon_tags_12 },
    { 0xc5319517, SUC_lexicon_tags_25 },
    { 0xc0719518, SUC_lexicon_tags_2 },
    { 0x7a119519, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8371951d, SUC_lexicon_tags_15 },
    { 0xec91951e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33119526, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3319528, SUC_lexicon_tags_19 },
    { 0x80d19528, SUC_lexicon_tags_16 },
    { 0x91f1952a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a119533, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x66f19535, SUC_lexicon_tags_2 },
    { 0xecb19536, SUC_lexicon_tags_37 },
    { 0x8b319537, SUC_lexicon_tags_6 },
    { 0xbc319538, SUC_lexicon_tags_7 },
    { 0x43119539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b19540, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79919542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37119546, SUC_lexicon_tags_17 },
    { 0x5ab19547, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3771954e, SUC_lexicon_tags_23 },
    { 0x6e31954f, SUC_lexicon_tags_0 },
    { 0x1cf1954e, SUC_lexicon_tags_69 },
    { 0x84319551, SUC_lexicon_tags_77 },
    { 0x1ab19552, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d919557, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9591955c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa519560, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f19565, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x63119567, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86119569, SUC_lexicon_tags_15 },
    { 0xea11956a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbad1956c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa771956e, SUC_lexicon_tags_37 },
    { 0xa9b1956f, SUC_lexicon_tags_25 },
    { 0xc691956f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15919579, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4b51957b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc731957d, SUC_lexicon_tags_37 },
    { 0x951957e, SUC_lexicon_tags_12 },
    { 0x3331957e, SUC_lexicon_tags_29 },
    { 0x7a119580, SUC_lexicon_tags_2 },
    { 0x71f19581, SUC_lexicon_tags_2 },
    { 0xacf1957f, SUC_lexicon_tags_19 },
    { 0x18d19583, SUC_lexicon_tags_12 },
    { 0xa3b19583, SUC_lexicon_tags_15 },
    { 0xa8919583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb1958a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc491958d, SUC_lexicon_tags_9 },
    { 0xc6d1958d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b19596, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1ab19598, SUC_lexicon_tags_25 },
    { 0x37119599, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33f1959b, SUC_lexicon_tags_39 },
    { 0x9cd1959c, SUC_lexicon_tags_46 },
    { 0x1971959d, SUC_lexicon_tags_1348 },
    { 0x1931959e, SUC_lexicon_tags_2 },
    { 0xf0b1959d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf195a6, SUC_lexicon_tags_37 },
    { 0x2bd195a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed195aa, SUC_lexicon_tags_14 },
    { 0xebb195aa, SUC_lexicon_tags_32 },
    { 0x4bb195ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23195b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353195b7, SUC_lexicon_tags_14 },
    { 0x203195b8, SUC_lexicon_tags_15 },
    { 0x7f195b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed195bc, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x36f195be, SUC_lexicon_tags_9 },
    { 0x3f7195bf, SUC_lexicon_tags_15 },
    { 0xd69195c0, SUC_lexicon_tags_7 },
    { 0x2b1195bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d195cb, SUC_lexicon_tags_2 },
    { 0xbcf195cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x393195ce, SUC_lexicon_tags_8 },
    { 0xfa7195cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df195d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9195d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab195dd, SUC_lexicon_tags_60 },
    { 0xda7195dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d195e1, SUC_lexicon_tags_47 },
    { 0xb55195e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1195e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd195f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaf9195f3, SUC_lexicon_tags_17 },
    { 0x2f9195f4, SUC_lexicon_tags_29 },
    { 0x171195f3, SUC_lexicon_tags_14 },
    { 0x121195f4, SUC_lexicon_tags_14 },
    { 0x4ab195f7, SUC_lexicon_tags_30 },
    { 0xe2d195f8, SUC_lexicon_tags_1 },
    { 0x3af195f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x407195fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19319605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e519607, SUC_lexicon_tags_84 },
    { 0xb4519607, SUC_lexicon_tags_27 },
    { 0x72319608, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6531960b, SUC_lexicon_tags_65 },
    { 0xc651960c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc851960e, SUC_lexicon_tags_17 },
    { 0x4231960e, SUC_lexicon_tags_4 },
    { 0x62d19610, SUC_lexicon_tags_9 },
    { 0xf6119611, SUC_lexicon_tags_25 },
    { 0x55d19612, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92519615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce719618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f71961d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db19622, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1119626, SUC_lexicon_tags_25 },
    { 0x6fb19627, SUC_lexicon_tags_17 },
    { 0xa5f19628, SUC_lexicon_tags_37 },
    { 0xe1319628, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc571962b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ab1962d, SUC_lexicon_tags_49 },
    { 0x5551962d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cd19630, SUC_lexicon_tags_1 },
    { 0xfcf19630, SUC_lexicon_tags_10 },
    { 0xf1319630, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5719636, SUC_lexicon_tags_46 },
    { 0x1ef19636, SUC_lexicon_tags_10 },
    { 0x49119636, SUC_lexicon_tags_97 },
    { 0x45519639, SUC_lexicon_tags_30 },
    { 0x6f319637, SUC_lexicon_tags_4 },
    { 0x4b519639, SUC_lexicon_tags_37 },
    { 0xa291963c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d1963f, SUC_lexicon_tags_19 },
    { 0x3a519640, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf919644, SUC_lexicon_tags_511 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b19647, SUC_lexicon_tags_37 },
    { 0x68f19647, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85119652, SUC_lexicon_tags_174 },
    { 0x87d19652, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa919659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b51965b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f19661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21719663, SUC_lexicon_tags_11 },
    { 0xdf919664, SUC_lexicon_tags_60 },
    { 0xf4119665, SUC_lexicon_tags_60 },
    { 0x2af19665, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b91966b, SUC_lexicon_tags_17 },
    { 0x8511966c, SUC_lexicon_tags_25 },
    { 0x73b1966d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d919672, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd19676, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed1967a, SUC_lexicon_tags_37 },
    { 0xb1b1967b, SUC_lexicon_tags_7 },
    { 0xf111967a, SUC_lexicon_tags_37 },
    { 0xc5f1967d, SUC_lexicon_tags_7 },
    { 0xe8d1967d, SUC_lexicon_tags_12 },
    { 0xd211967f, SUC_lexicon_tags_17 },
    { 0x3bf19680, SUC_lexicon_tags_25 },
    { 0xe6119681, SUC_lexicon_tags_39 },
    { 0x6051967e, SUC_lexicon_tags_19 },
    { 0xd971967f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd7519685, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1a119687, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4671968a, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x8971968c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa119692, SUC_lexicon_tags_65 },
    { 0x82919693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9719695, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5271969b, SUC_lexicon_tags_32 },
    { 0xd811969c, SUC_lexicon_tags_15 },
    { 0xd9d1969c, SUC_lexicon_tags_1 },
    { 0x3011969e, SUC_lexicon_tags_2 },
    { 0xdef1969f, SUC_lexicon_tags_47 },
    { 0x26d196a0, SUC_lexicon_tags_37 },
    { 0x9cb196a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33196a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x883196a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07196ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x645196ad, SUC_lexicon_tags_37 },
    { 0x757196ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1196b2, SUC_lexicon_tags_17 },
    { 0x433196b3, SUC_lexicon_tags_25 },
    { 0x1b3196b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd33196b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9196bb, SUC_lexicon_tags_37 },
    { 0xf6b196bc, SUC_lexicon_tags_30 },
    { 0x9ef196bc, SUC_lexicon_tags_15 },
    { 0x5f196be, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbcb196c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b196c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x837196c5, SUC_lexicon_tags_29 },
    { 0xcb9196c6, SUC_lexicon_tags_23 },
    { 0xf0d196c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69d196c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9196cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x115196ce, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab196d4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53196d9, SUC_lexicon_tags_37 },
    { 0xcab196da, SUC_lexicon_tags_11 },
    { 0xea3196d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3196df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93196e3, SUC_lexicon_tags_29 },
    { 0x8e1196e3, SUC_lexicon_tags_82 },
    { 0xf69196e5, SUC_lexicon_tags_45 },
    { 0x4a3196e6, SUC_lexicon_tags_172 },
    { 0x17d196e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d196ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31196f6, SUC_lexicon_tags_26 },
    { 0x6b1196f7, SUC_lexicon_tags_16 },
    { 0xfa3196f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed196fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb19700, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xef319703, SUC_lexicon_tags_37 },
    { 0x5f319704, SUC_lexicon_tags_37 },
    { 0xd0119703, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x99319707, SUC_lexicon_tags_6 },
    { 0x76719708, SUC_lexicon_tags_7 },
    { 0xae519708, SUC_lexicon_tags_37 },
    { 0x42d1970a, SUC_lexicon_tags_25 },
    { 0xc11970b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3211970d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b19712, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe4d19714, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xec919716, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93f19718, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf871971b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2151971d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6491971f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0119724, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad19728, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad1972f, SUC_lexicon_tags_60 },
    { 0xe411972f, SUC_lexicon_tags_2 },
    { 0x91b19731, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4319734, SUC_lexicon_tags_14 },
    { 0x61d19735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48b19737, SUC_lexicon_tags_3 },
    { 0x78719737, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9791973a, SUC_lexicon_tags_38 },
    { 0xfd51973b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a11973d, SUC_lexicon_tags_1 },
    { 0x85b1973e, SUC_lexicon_tags_31 },
    { 0x65d1973f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78919741, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a919748, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf271974b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd891974f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c719757, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1371975a, SUC_lexicon_tags_21 },
    { 0x4b91975b, SUC_lexicon_tags_17 },
    { 0x1bd1975a, SUC_lexicon_tags_11 },
    { 0xeb91975b, SUC_lexicon_tags_60 },
    { 0x44f1975e, SUC_lexicon_tags_65 },
    { 0x9f71975f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf731976f, SUC_lexicon_tags_11 },
    { 0x86319770, SUC_lexicon_tags_17 },
    { 0xa5d19771, SUC_lexicon_tags_1 },
    { 0xd4519772, SUC_lexicon_tags_4 },
    { 0x74519772, SUC_lexicon_tags_15 },
    { 0x60119774, SUC_lexicon_tags_25 },
    { 0x1dd19775, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x17919778, SUC_lexicon_tags_30 },
    { 0x92119779, SUC_lexicon_tags_17 },
    { 0xd1b1977a, SUC_lexicon_tags_19 },
    { 0x6ef19778, SUC_lexicon_tags_28 },
    { 0xaf91977c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa951977f, SUC_lexicon_tags_7 },
    { 0x8b19780, SUC_lexicon_tags_6 },
    { 0xc9919781, SUC_lexicon_tags_70 },
    { 0x95919780, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf19785, SUC_lexicon_tags_28 },
    { 0x77b19786, SUC_lexicon_tags_27 },
    { 0x47d19786, SUC_lexicon_tags_16 },
    { 0xe4f19788, SUC_lexicon_tags_1 },
    { 0x9719789, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8919795, SUC_lexicon_tags_66 },
    { 0x86919795, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b19799, SUC_lexicon_tags_11 },
    { 0x431979a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f197a0, SUC_lexicon_tags_12 },
    { 0x79f197a1, SUC_lexicon_tags_8 },
    { 0x507197a2, SUC_lexicon_tags_17 },
    { 0xa93197a1, SUC_lexicon_tags_47 },
    { 0x473197a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x721197a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe3d197a9, SUC_lexicon_tags_61 },
    { 0x7d1197aa, SUC_lexicon_tags_27 },
    { 0xfd9197a9, SUC_lexicon_tags_6 },
    { 0x417197ac, SUC_lexicon_tags_1 },
    { 0x6f1197ad, SUC_lexicon_tags_1 },
    { 0xd5d197ae, SUC_lexicon_tags_158 },
    { 0x2df197a9, SUC_lexicon_tags_16 },
    { 0x355197aa, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xe63197b2, SUC_lexicon_tags_19 },
    { 0xa7197b3, SUC_lexicon_tags_19 },
    { 0x77b197b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2eb197b6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1197bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7197c0, SUC_lexicon_tags_12 },
    { 0x355197c0, SUC_lexicon_tags_37 },
    { 0x2f7197c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829197c7, SUC_lexicon_tags_6 },
    { 0x2e3197c8, SUC_lexicon_tags_30 },
    { 0x94d197c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91197cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9197d1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f197d4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x529197d7, SUC_lexicon_tags_40 },
    { 0xcab197d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5197dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff197e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed197eb, SUC_lexicon_tags_29 },
    { 0xb89197ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5197ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f197f5, SUC_lexicon_tags_12 },
    { 0xeeb197f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29f197f8, SUC_lexicon_tags_12 },
    { 0x13f197f9, SUC_lexicon_tags_45 },
    { 0xb03197fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd197fd, SUC_lexicon_tags_14 },
    { 0xd33197fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xca519801, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9ed19803, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d1980c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d19812, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88719814, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f19817, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe091981b, SUC_lexicon_tags_1 },
    { 0x3491981c, SUC_lexicon_tags_63 },
    { 0x32d1981d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9971981f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d19822, SUC_lexicon_tags_7 },
    { 0x78319822, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff19826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12b19828, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7251982a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d51982e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91919832, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc919835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b1983e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d919842, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6f719844, SUC_lexicon_tags_44 },
    { 0x5e319844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8971984e, SUC_lexicon_tags_19 },
    { 0xf671984e, SUC_lexicon_tags_33 },
    { 0x4d719850, SUC_lexicon_tags_23 },
    { 0x6b119850, SUC_lexicon_tags_9 },
    { 0x23119852, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1985d, SUC_lexicon_tags_37 },
    { 0xb5d1985e, SUC_lexicon_tags_27 },
    { 0x9111985f, SUC_lexicon_tags_10 },
    { 0x66d1985f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad19865, SUC_lexicon_tags_21 },
    { 0x14719865, SUC_lexicon_tags_60 },
    { 0xf6519867, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3519869, SUC_lexicon_tags_48 },
    { 0x58d1986a, SUC_lexicon_tags_82 },
    { 0x8411986b, SUC_lexicon_tags_37 },
    { 0xaf11986b, SUC_lexicon_tags_49 },
    { 0x3f1986d, SUC_lexicon_tags_138 },
    { 0x9e91986b, SUC_lexicon_tags_15 },
    { 0x4b11986f, SUC_lexicon_tags_87 },
    { 0xfcf19870, SUC_lexicon_tags_17 },
    { 0x74f19871, SUC_lexicon_tags_49 },
    { 0xb0719872, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a719874, SUC_lexicon_tags_25 },
    { 0x96f19875, SUC_lexicon_tags_1 },
    { 0x39919875, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x27f19878, SUC_lexicon_tags_2 },
    { 0xc6319879, SUC_lexicon_tags_0 },
    { 0xb91987a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9591987d, SUC_lexicon_tags_0 },
    { 0x2211987d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b119884, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac91988a, SUC_lexicon_tags_11 },
    { 0x42f1988b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xea31988d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9319893, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b19897, SUC_lexicon_tags_30 },
    { 0xa2519897, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c71989d, SUC_lexicon_tags_15 },
    { 0x6d91989e, SUC_lexicon_tags_8 },
    { 0x9491989e, SUC_lexicon_tags_49 },
    { 0xc47198a0, SUC_lexicon_tags_26 },
    { 0xd491989d, SUC_lexicon_tags_15 },
    { 0x1f11989e, SUC_lexicon_tags_23 },
    { 0x74b198a2, SUC_lexicon_tags_6 },
    { 0xbbb198a1, SUC_lexicon_tags_39 },
    { 0x309198a4, SUC_lexicon_tags_27 },
    { 0x16f198a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec1198a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45d198aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89198b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07198b4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa93198b6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7ab198b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x799198ba, SUC_lexicon_tags_17 },
    { 0x4ef198bb, SUC_lexicon_tags_0 },
    { 0xc4d198bc, SUC_lexicon_tags_12 },
    { 0x469198bc, SUC_lexicon_tags_39 },
    { 0x9e5198be, SUC_lexicon_tags_28 },
    { 0x451198bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b198c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x639198c6, SUC_lexicon_tags_25 },
    { 0x3ed198c7, SUC_lexicon_tags_0 },
    { 0xe0d198c6, SUC_lexicon_tags_15 },
    { 0x207198c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x81b198cb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb198ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd1198d0, SUC_lexicon_tags_15 },
    { 0xda7198d1, SUC_lexicon_tags_7 },
    { 0x2d1198d2, SUC_lexicon_tags_17 },
    { 0xeb5198d3, SUC_lexicon_tags_7 },
    { 0x88b198d4, SUC_lexicon_tags_15 },
    { 0xcf1198d4, SUC_lexicon_tags_19 },
    { 0x22f198d6, SUC_lexicon_tags_43 },
    { 0x4f198d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb198db, SUC_lexicon_tags_7 },
    { 0x2cd198dc, SUC_lexicon_tags_2 },
    { 0xf43198dd, SUC_lexicon_tags_28 },
    { 0xa2f198de, SUC_lexicon_tags_60 },
    { 0x75198df, SUC_lexicon_tags_14 },
    { 0x9f1198dd, SUC_lexicon_tags_31 },
    { 0xd77198e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x303198e3, SUC_lexicon_tags_10 },
    { 0x32f198e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641198eb, SUC_lexicon_tags_244 },
    { 0x3f3198eb, SUC_lexicon_tags_11 },
    { 0x2a5198ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b198f0, SUC_lexicon_tags_47 },
    { 0x55f198f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1198f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x539198fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75198fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xff319900, SUC_lexicon_tags_7 },
    { 0x38519901, SUC_lexicon_tags_46 },
    { 0x51119901, SUC_lexicon_tags_3 },
    { 0x8e519902, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f519908, SUC_lexicon_tags_45 },
    { 0xbcd19909, SUC_lexicon_tags_183 },
    { 0xaf51990a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3951990c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f919910, SUC_lexicon_tags_14 },
    { 0xc1719910, SUC_lexicon_tags_60 },
    { 0x42f19910, SUC_lexicon_tags_17 },
    { 0x5c919910, SUC_lexicon_tags_37 },
    { 0xa9919914, SUC_lexicon_tags_7 },
    { 0x85519915, SUC_lexicon_tags_10 },
    { 0x81519916, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3711991a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd471991d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b19922, SUC_lexicon_tags_25 },
    { 0xca119922, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7919925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d19928, SUC_lexicon_tags_2 },
    { 0x7f119928, SUC_lexicon_tags_48 },
    { 0x7db19929, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab1992e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff319935, SUC_lexicon_tags_43 },
    { 0x4f519936, SUC_lexicon_tags_23 },
    { 0x31319937, SUC_lexicon_tags_12 },
    { 0x89119938, SUC_lexicon_tags_11 },
    { 0x76119936, SUC_lexicon_tags_0 },
    { 0xf3d19937, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd151993e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80919947, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf1994b, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b91994e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba519956, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7291995a, SUC_lexicon_tags_2 },
    { 0xebf1995b, SUC_lexicon_tags_3 },
    { 0xef1995c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b19961, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f19964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9719967, SUC_lexicon_tags_1 },
    { 0x5cf19968, SUC_lexicon_tags_77 },
    { 0x56b19969, SUC_lexicon_tags_47 },
    { 0x9c11996a, SUC_lexicon_tags_6 },
    { 0xfc11996a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f1996e, SUC_lexicon_tags_55 },
    { 0xb031996e, SUC_lexicon_tags_2 },
    { 0xd8719970, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16119974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1997a, SUC_lexicon_tags_33 },
    { 0x9f1997b, SUC_lexicon_tags_1 },
    { 0x1c91997c, SUC_lexicon_tags_3 },
    { 0x2031997c, SUC_lexicon_tags_11 },
    { 0xe4f1997e, SUC_lexicon_tags_33 },
    { 0xc9b1997e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d19983, SUC_lexicon_tags_27 },
    { 0x9e719984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa519989, SUC_lexicon_tags_12 },
    { 0x3cb1998a, SUC_lexicon_tags_37 },
    { 0x4e31998a, SUC_lexicon_tags_23 },
    { 0xc751998c, SUC_lexicon_tags_25 },
    { 0xaeb1998d, SUC_lexicon_tags_1 },
    { 0x6731998a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3d19990, SUC_lexicon_tags_82 },
    { 0x7ef19991, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74319993, SUC_lexicon_tags_12 },
    { 0xa9d19994, SUC_lexicon_tags_27 },
    { 0xb2b19995, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d1999a, SUC_lexicon_tags_25 },
    { 0xe11999b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03199a1, SUC_lexicon_tags_12 },
    { 0x94d199a2, SUC_lexicon_tags_37 },
    { 0xdb199a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb9199a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889199a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99199ac, SUC_lexicon_tags_45 },
    { 0x61f199ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48d199af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191199b4, SUC_lexicon_tags_35 },
    { 0x893199b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e5199b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597199be, SUC_lexicon_tags_34 },
    { 0xd17199be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65199c8, SUC_lexicon_tags_43 },
    { 0x2ef199c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8fb199cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f199d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967199db, SUC_lexicon_tags_12 },
    { 0x8c5199dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395199e0, SUC_lexicon_tags_23 },
    { 0x6d9199e1, SUC_lexicon_tags_63 },
    { 0x987199e0, SUC_lexicon_tags_11 },
    { 0x555199e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9199e7, SUC_lexicon_tags_9 },
    { 0x9d199e8, SUC_lexicon_tags_37 },
    { 0x971199e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25199ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76b199f1, SUC_lexicon_tags_12 },
    { 0x961199f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x345199f4, SUC_lexicon_tags_28 },
    { 0xc5d199f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x835199f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb45199f9, SUC_lexicon_tags_25 },
    { 0x973199f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693199fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98f19a00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3719a03, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f19a09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf19a0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb19a11, SUC_lexicon_tags_77 },
    { 0x63919a12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x40319a14, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef19a24, SUC_lexicon_tags_14 },
    { 0xc3d19a25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa2719a27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb19a2a, SUC_lexicon_tags_43 },
    { 0xe8719a2b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5a119a2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86919a34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5519a36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdab19a38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3319a3a, SUC_lexicon_tags_216 },
    { 0xf0319a3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb119a40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34519a47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23b19a49, SUC_lexicon_tags_33 },
    { 0x42119a49, SUC_lexicon_tags_31 },
    { 0x23519a4b, SUC_lexicon_tags_63 },
    { 0xdf919a4a, SUC_lexicon_tags_33 },
    { 0xb8119a4b, SUC_lexicon_tags_4 },
    { 0xe2919a49, SUC_lexicon_tags_25 },
    { 0x51d19a4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0119a55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd19a5a, SUC_lexicon_tags_17 },
    { 0x7c719a5b, SUC_lexicon_tags_16 },
    { 0xc4119a5c, SUC_lexicon_tags_28 },
    { 0x62519a5d, SUC_lexicon_tags_43 },
    { 0x42119a5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed19a61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44319a65, SUC_lexicon_tags_31 },
    { 0xedb19a66, SUC_lexicon_tags_21 },
    { 0x66719a67, SUC_lexicon_tags_11 },
    { 0x73919a68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb19a6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b19a6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x68519a72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88919a79, SUC_lexicon_tags_11 },
    { 0xbb19a7a, SUC_lexicon_tags_65 },
    { 0xb3119a7b, SUC_lexicon_tags_47 },
    { 0xd3519a7b, SUC_lexicon_tags_10 },
    { 0xbeb19a7b, SUC_lexicon_tags_1 },
    { 0xb7719a7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae119a80, SUC_lexicon_tags_12 },
    { 0x28f19a81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfdd19a83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd19a85, SUC_lexicon_tags_27 },
    { 0xbeb19a86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff19a8b, SUC_lexicon_tags_37 },
    { 0x38d19a8b, SUC_lexicon_tags_17 },
    { 0x2c519a8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8dd19a8f, SUC_lexicon_tags_29 },
    { 0x4f119a90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9319a92, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f19a95, SUC_lexicon_tags_40 },
    { 0x73319a95, SUC_lexicon_tags_2 },
    { 0x40919a97, SUC_lexicon_tags_2 },
    { 0xf5d19a95, SUC_lexicon_tags_11 },
    { 0x33719a99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f719a9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d119aa2, SUC_lexicon_tags_17 },
    { 0x75119aa3, SUC_lexicon_tags_87 },
    { 0x72519aa3, SUC_lexicon_tags_27 },
    { 0xe3519aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb19aa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a519aab, SUC_lexicon_tags_14 },
    { 0xe3f19aac, SUC_lexicon_tags_65 },
    { 0xde919aad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa119ab0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bf19ab2, SUC_lexicon_tags_46 },
    { 0x4cf19ab3, SUC_lexicon_tags_148 },
    { 0x14919ab3, SUC_lexicon_tags_37 },
    { 0x7f319ab3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x12519ab8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f19abd, SUC_lexicon_tags_21 },
    { 0x27319abd, SUC_lexicon_tags_761 },
    { 0xe3d19abd, SUC_lexicon_tags_56 },
    { 0xa5b19ac0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb19ac3, SUC_lexicon_tags_25 },
    { 0x85519ac3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad119ac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47119acc, SUC_lexicon_tags_37 },
    { 0xd4919acc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2919ad0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8519ad2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d919ad7, SUC_lexicon_tags_10 },
    { 0xacf19ad8, SUC_lexicon_tags_12 },
    { 0xbb919ad9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf19adc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4b519ade, SUC_lexicon_tags_31 },
    { 0x2dd19ade, SUC_lexicon_tags_11 },
    { 0x98719ae0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x97b19ae2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcc719ae4, SUC_lexicon_tags_23 },
    { 0xfff19ae4, SUC_lexicon_tags_79 },
    { 0xfff19ae5, SUC_lexicon_tags_37 },
    { 0xac719ae7, SUC_lexicon_tags_15 },
    { 0xac719ae8, SUC_lexicon_tags_2 },
    { 0xab319ae9, SUC_lexicon_tags_7 },
    { 0x9fd19aea, SUC_lexicon_tags_25 },
    { 0x9ff19aea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c919aed, SUC_lexicon_tags_37 },
    { 0xe0319aee, SUC_lexicon_tags_12 },
    { 0xd2d19aee, SUC_lexicon_tags_17 },
    { 0xaf919aee, SUC_lexicon_tags_17 },
    { 0x1bd19aee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef19af5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x37719af7, SUC_lexicon_tags_47 },
    { 0xdb919af7, SUC_lexicon_tags_15 },
    { 0xbf919af7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x43519afc, SUC_lexicon_tags_65 },
    { 0x65f19afd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf8119aff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d19b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65919b0a, SUC_lexicon_tags_12 },
    { 0xb7b19b0a, SUC_lexicon_tags_5 },
    { 0xa9b19b0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d319b0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b19b11, SUC_lexicon_tags_12 },
    { 0x4ab19b12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92719b16, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x75f19b18, SUC_lexicon_tags_47 },
    { 0xd3b19b19, SUC_lexicon_tags_5 },
    { 0x48f19b1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26919b21, SUC_lexicon_tags_25 },
    { 0x17d19b22, SUC_lexicon_tags_0 },
    { 0xb5119b21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe719b25, SUC_lexicon_tags_25 },
    { 0x4ff19b26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa119b2a, SUC_lexicon_tags_1 },
    { 0x98b19b2b, SUC_lexicon_tags_14 },
    { 0x87119b2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed19b31, SUC_lexicon_tags_17 },
    { 0x719b31, SUC_lexicon_tags_12 },
    { 0x8f519b31, SUC_lexicon_tags_11 },
    { 0x16b19b34, SUC_lexicon_tags_14 },
    { 0xecb19b35, SUC_lexicon_tags_3 },
    { 0xc4519b35, SUC_lexicon_tags_25 },
    { 0x71b19b31, SUC_lexicon_tags_1 },
    { 0x63919b38, SUC_lexicon_tags_25 },
    { 0xc0f19b32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f19b3e, SUC_lexicon_tags_0 },
    { 0xa9719b3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4119b44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f19b49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d19b4d, SUC_lexicon_tags_2 },
    { 0x68519b4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56919b50, SUC_lexicon_tags_29 },
    { 0x9119b51, SUC_lexicon_tags_0 },
    { 0xd2f19b51, SUC_lexicon_tags_1 },
    { 0x9e519b53, SUC_lexicon_tags_17 },
    { 0x78519b54, SUC_lexicon_tags_0 },
    { 0x93b19b53, SUC_lexicon_tags_7 },
    { 0xa8519b53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9319b59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc919b5b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd519b60, SUC_lexicon_tags_56 },
    { 0x45919b60, SUC_lexicon_tags_9 },
    { 0x1d119b62, SUC_lexicon_tags_11 },
    { 0x2e119b63, SUC_lexicon_tags_9 },
    { 0xce519b62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b319b69, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfcd19b6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3af19b6d, SUC_lexicon_tags_28 },
    { 0xa5919b6d, SUC_lexicon_tags_17 },
    { 0x51519b6f, SUC_lexicon_tags_23 },
    { 0x48319b6f, SUC_lexicon_tags_38 },
    { 0x19519b71, SUC_lexicon_tags_7 },
    { 0x1b19b70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x63919b75, SUC_lexicon_tags_26 },
    { 0xad119b76, SUC_lexicon_tags_7 },
    { 0xa8119b77, SUC_lexicon_tags_17 },
    { 0x49919b78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4319b7c, SUC_lexicon_tags_57 },
    { 0xfdf19b7d, SUC_lexicon_tags_2 },
    { 0x86319b7e, SUC_lexicon_tags_7 },
    { 0xce519b7f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b519b84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce319b87, SUC_lexicon_tags_63 },
    { 0x59919b87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd19b8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca319b91, SUC_lexicon_tags_23 },
    { 0x1b19b92, SUC_lexicon_tags_63 },
    { 0xe9d19b93, SUC_lexicon_tags_160 },
    { 0x5cd19b92, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b19b99, SUC_lexicon_tags_11 },
    { 0x2df19b9a, SUC_lexicon_tags_2 },
    { 0xd1919b9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x38f19b9d, SUC_lexicon_tags_8 },
    { 0x29519b9d, SUC_lexicon_tags_11 },
    { 0xdf19b9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb519ba2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b19ba5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7519baa, SUC_lexicon_tags_7 },
    { 0x14d19baa, SUC_lexicon_tags_30 },
    { 0x27519bac, SUC_lexicon_tags_11 },
    { 0x6f519bac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c319baf, SUC_lexicon_tags_39 },
    { 0x8a719bb0, SUC_lexicon_tags_25 },
    { 0x3d719bb1, SUC_lexicon_tags_52 },
    { 0xc2719baf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39f19bb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf919bb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8319bb9, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a519bbe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d19bc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd19bc7, SUC_lexicon_tags_60 },
    { 0xf9319bc7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa7719bca, SUC_lexicon_tags_8 },
    { 0x61319bca, SUC_lexicon_tags_0 },
    { 0xa6319bca, SUC_lexicon_tags_45 },
    { 0x4fb19bcd, SUC_lexicon_tags_11 },
    { 0xc6b19bcb, SUC_lexicon_tags_19 },
    { 0x6cb19bca, SUC_lexicon_tags_15 },
    { 0x92919bcf, SUC_lexicon_tags_56 },
    { 0xf5919bca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d19bd8, SUC_lexicon_tags_60 },
    { 0x58d19bd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f19bdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff19be0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f19be3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf719be6, SUC_lexicon_tags_30 },
    { 0x4519be7, SUC_lexicon_tags_29 },
    { 0xab19be8, SUC_lexicon_tags_17 },
    { 0xdff19be9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b919beb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff19bee, SUC_lexicon_tags_45 },
    { 0x68319bee, SUC_lexicon_tags_68 },
    { 0x80f19bef, SUC_lexicon_tags_27 },
    { 0x96319bf1, SUC_lexicon_tags_12 },
    { 0x70719bee, SUC_lexicon_tags_7 },
    { 0x1cb19bf3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db19bf7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdd919bf9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6519bfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d19c08, SUC_lexicon_tags_27 },
    { 0x4af19c09, SUC_lexicon_tags_56 },
    { 0x5f319c09, SUC_lexicon_tags_45 },
    { 0x73f19c0b, SUC_lexicon_tags_23 },
    { 0xea319c08, SUC_lexicon_tags_12 },
    { 0x56b19c0d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7519c10, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b19c18, SUC_lexicon_tags_9 },
    { 0x35919c18, SUC_lexicon_tags_11 },
    { 0xf519c1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd1b19c1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9519c21, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x60119c24, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2519c26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78919c29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd719c2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f19c31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x77719c33, SUC_lexicon_tags_2 },
    { 0xbef19c34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5b319c36, SUC_lexicon_tags_336 },
    { 0x2af19c37, SUC_lexicon_tags_43 },
    { 0x12919c38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61319c3b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x11719c3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd19c40, SUC_lexicon_tags_2 },
    { 0x7cd19c41, SUC_lexicon_tags_32 },
    { 0x74919c42, SUC_lexicon_tags_47 },
    { 0x75919c42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32119c47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c319c49, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d19c4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32319c52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a519c55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39f19c57, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b319c5f, SUC_lexicon_tags_17 },
    { 0x75d19c5f, SUC_lexicon_tags_1 },
    { 0xb3519c5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58519c63, SUC_lexicon_tags_38 },
    { 0xdb19c64, SUC_lexicon_tags_37 },
    { 0xdaf19c65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c319c68, SUC_lexicon_tags_31 },
    { 0x75d19c68, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9cf19c6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6319c6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bb19c70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x45719c72, SUC_lexicon_tags_54 },
    { 0xe1d19c73, SUC_lexicon_tags_38 },
    { 0xcc519c74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39119c7d, SUC_lexicon_tags_23 },
    { 0xf1719c7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3119c86, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2519c8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3319c8c, SUC_lexicon_tags_55 },
    { 0xf2119c8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64f19c8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7319c91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1319c94, SUC_lexicon_tags_37 },
    { 0xfe919c95, SUC_lexicon_tags_25 },
    { 0x50519c94, SUC_lexicon_tags_6 },
    { 0x52719c94, SUC_lexicon_tags_8 },
    { 0xa4b19c95, SUC_lexicon_tags_10 },
    { 0x2bf19c99, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5519c9f, SUC_lexicon_tags_25 },
    { 0x43d19c9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79919cad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbb19caf, SUC_lexicon_tags_3 },
    { 0x45119caf, SUC_lexicon_tags_31 },
    { 0xe3719cb1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee119cb9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b19cbd, SUC_lexicon_tags_31 },
    { 0x65719cbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd719cc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b19cc6, SUC_lexicon_tags_1 },
    { 0x83b19cc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f119ccc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af19cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6919cde, SUC_lexicon_tags_32 },
    { 0x20b19cdf, SUC_lexicon_tags_14 },
    { 0xee319cdf, SUC_lexicon_tags_31 },
    { 0x77f19ce1, SUC_lexicon_tags_11 },
    { 0x47319ce2, SUC_lexicon_tags_17 },
    { 0xb1119ce1, SUC_lexicon_tags_56 },
    { 0x87f19ce4, SUC_lexicon_tags_11 },
    { 0x85319ce5, SUC_lexicon_tags_17 },
    { 0x3f19ce3, SUC_lexicon_tags_46 },
    { 0x55919ce5, SUC_lexicon_tags_17 },
    { 0x46719cdf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f19cee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df19cf1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe7119cf3, SUC_lexicon_tags_9 },
    { 0x3e919cf4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f19cfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf19cfd, SUC_lexicon_tags_6 },
    { 0x66f19cfd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xce519d00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99919d04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d519d07, SUC_lexicon_tags_2 },
    { 0x40b19d08, SUC_lexicon_tags_25 },
    { 0x59f19d09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a519d0b, SUC_lexicon_tags_55 },
    { 0x47319d0c, SUC_lexicon_tags_46 },
    { 0xa8f19d0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44119d0f, SUC_lexicon_tags_19 },
    { 0x1519d10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x60519d13, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x57919d15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52119d19, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x519d1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94319d1e, SUC_lexicon_tags_31 },
    { 0x3d919d1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11519d27, SUC_lexicon_tags_1 },
    { 0x69f19d28, SUC_lexicon_tags_82 },
    { 0x81119d29, SUC_lexicon_tags_17 },
    { 0x14719d28, SUC_lexicon_tags_43 },
    { 0x64519d2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f19d36, SUC_lexicon_tags_15 },
    { 0x7119d37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e319d3e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b19d41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d719d45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce719d47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60319d4c, SUC_lexicon_tags_26 },
    { 0x45d19d4d, SUC_lexicon_tags_1 },
    { 0xe1d19d4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82119d51, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e319d56, SUC_lexicon_tags_47 },
    { 0x22719d57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76119d59, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed19d5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb19d5f, SUC_lexicon_tags_49 },
    { 0x3f919d60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f19d64, SUC_lexicon_tags_11 },
    { 0x3bb19d65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f19d6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42b19d6c, SUC_lexicon_tags_7 },
    { 0xf3119d6c, SUC_lexicon_tags_9 },
    { 0x3c319d6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b119d70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x85919d72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3119d74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46519d78, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f19d7c, SUC_lexicon_tags_17 },
    { 0x4e519d7d, SUC_lexicon_tags_39 },
    { 0x1719d7c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd19d81, SUC_lexicon_tags_63 },
    { 0xa3f19d82, SUC_lexicon_tags_14 },
    { 0xc2319d82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d19d86, SUC_lexicon_tags_49 },
    { 0xf4719d86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67319d8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae319d92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d119da0, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x11119da2, SUC_lexicon_tags_47 },
    { 0xe5d19da3, SUC_lexicon_tags_27 },
    { 0x78719da4, SUC_lexicon_tags_27 },
    { 0x8c319da3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb0f19da7, SUC_lexicon_tags_29 },
    { 0x44119da8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf519dac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44919dae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x67b19db0, SUC_lexicon_tags_28 },
    { 0x25919db0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72719db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb19db7, SUC_lexicon_tags_14 },
    { 0x9cd19db7, SUC_lexicon_tags_25 },
    { 0x1f919db9, SUC_lexicon_tags_61 },
    { 0x67119dba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3519dbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe6919dbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9119dc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff19dc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf19dc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0919dcf, SUC_lexicon_tags_11 },
    { 0x69519dd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb519dd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cf19dd7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a919ddb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b19dde, SUC_lexicon_tags_29 },
    { 0x6db19dde, SUC_lexicon_tags_37 },
    { 0xf5d19de0, SUC_lexicon_tags_26 },
    { 0x67919de1, SUC_lexicon_tags_47 },
    { 0x62919de2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d719de5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x20919de8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f19dec, SUC_lexicon_tags_25 },
    { 0x90b19dec, SUC_lexicon_tags_25 },
    { 0x58519dee, SUC_lexicon_tags_63 },
    { 0x4d519def, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb19df4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3519df8, SUC_lexicon_tags_235 },
    { 0xda519df9, SUC_lexicon_tags_8 },
    { 0x4c519dfa, SUC_lexicon_tags_29 },
    { 0x3cb19dfb, SUC_lexicon_tags_1 },
    { 0x5dd19dfa, SUC_lexicon_tags_27 },
    { 0x23519dfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9719e00, SUC_lexicon_tags_37 },
    { 0x55f19e01, SUC_lexicon_tags_27 },
    { 0x6a519e02, SUC_lexicon_tags_2 },
    { 0xb9d19e03, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7e319e05, SUC_lexicon_tags_19 },
    { 0x82519e06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db19e0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d19e11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb19e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d119e1c, SUC_lexicon_tags_37 },
    { 0xe2919e1d, SUC_lexicon_tags_91 },
    { 0xfcb19e1e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1a319e20, SUC_lexicon_tags_7 },
    { 0x71b19e21, SUC_lexicon_tags_25 },
    { 0x3919e22, SUC_lexicon_tags_15 },
    { 0x65719e23, SUC_lexicon_tags_37 },
    { 0xe7319e24, SUC_lexicon_tags_15 },
    { 0x2af19e22, SUC_lexicon_tags_25 },
    { 0x2c119e26, SUC_lexicon_tags_15 },
    { 0xc7519e27, SUC_lexicon_tags_79 },
    { 0xbfd19e28, SUC_lexicon_tags_68 },
    { 0x3f319e27, SUC_lexicon_tags_14 },
    { 0x50b19e27, SUC_lexicon_tags_37 },
    { 0xa7919e28, SUC_lexicon_tags_25 },
    { 0x25d19e2c, SUC_lexicon_tags_37 },
    { 0x49319e24, SUC_lexicon_tags_39 },
    { 0x1f19e20, SUC_lexicon_tags_2 },
    { 0xaaf19e21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc519e31, SUC_lexicon_tags_8 },
    { 0x28319e31, SUC_lexicon_tags_25 },
    { 0x38b19e31, SUC_lexicon_tags_25 },
    { 0x77519e32, SUC_lexicon_tags_47 },
    { 0xd8719e34, SUC_lexicon_tags_37 },
    { 0x7db19e33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x78f19e38, SUC_lexicon_tags_2 },
    { 0x95319e38, SUC_lexicon_tags_15 },
    { 0x89119e3a, SUC_lexicon_tags_14 },
    { 0x2ad19e38, SUC_lexicon_tags_27 },
    { 0x6df19e38, SUC_lexicon_tags_37 },
    { 0x20319e3d, SUC_lexicon_tags_7 },
    { 0x98519e3e, SUC_lexicon_tags_2 },
    { 0x1d19e3f, SUC_lexicon_tags_31 },
    { 0x78d19e40, SUC_lexicon_tags_42 },
    { 0xc1f19e40, SUC_lexicon_tags_15 },
    { 0xd5519e42, SUC_lexicon_tags_33 },
    { 0xa1919e43, SUC_lexicon_tags_17 },
    { 0xa2119e42, SUC_lexicon_tags_12 },
    { 0x93f19e45, SUC_lexicon_tags_7 },
    { 0x1fd19e46, SUC_lexicon_tags_37 },
    { 0x96319e45, SUC_lexicon_tags_27 },
    { 0xf0d19e48, SUC_lexicon_tags_37 },
    { 0x33b19e49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x30719e4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd119e50, SUC_lexicon_tags_43 },
    { 0xfc319e51, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d919e55, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc9d19e57, SUC_lexicon_tags_15 },
    { 0x7dd19e58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2b19e5a, SUC_lexicon_tags_0 },
    { 0xe5f19e5b, SUC_lexicon_tags_2 },
    { 0x32519e5b, SUC_lexicon_tags_13 },
    { 0x9b519e5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71519e62, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d119e65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b119e69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9719e6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb119e6d, SUC_lexicon_tags_12 },
    { 0x4d19e6e, SUC_lexicon_tags_2 },
    { 0xf0719e6f, SUC_lexicon_tags_24 },
    { 0xba519e70, SUC_lexicon_tags_16 },
    { 0xf2519e71, SUC_lexicon_tags_1 },
    { 0x98319e72, SUC_lexicon_tags_17 },
    { 0xc9919e73, SUC_lexicon_tags_29 },
    { 0xb5d19e74, SUC_lexicon_tags_7 },
    { 0xa9919e71, SUC_lexicon_tags_37 },
    { 0x36119e76, SUC_lexicon_tags_17 },
    { 0x2c519e77, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf19e7a, SUC_lexicon_tags_12 },
    { 0xeb719e7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd119e7e, SUC_lexicon_tags_17 },
    { 0xc0119e7e, SUC_lexicon_tags_27 },
    { 0xdff19e7e, SUC_lexicon_tags_19 },
    { 0x6d519e81, SUC_lexicon_tags_14 },
    { 0x37d19e82, SUC_lexicon_tags_39 },
    { 0x83d19e83, SUC_lexicon_tags_18 },
    { 0x5b719e81, SUC_lexicon_tags_81 },
    { 0x28119e80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x70919e88, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x74519e8a, SUC_lexicon_tags_25 },
    { 0xf8b19e8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb19e8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21119e90, SUC_lexicon_tags_25 },
    { 0xfaf19e91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd19e98, SUC_lexicon_tags_11 },
    { 0xed919e98, SUC_lexicon_tags_26 },
    { 0xb4b19e9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54919e9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6319ea3, SUC_lexicon_tags_38 },
    { 0x5ed19ea3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4519ea6, SUC_lexicon_tags_15 },
    { 0xdf519ea6, SUC_lexicon_tags_1 },
    { 0x68919ea8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31119eb2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcdd19eb4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe919ec4, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad19ec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0719ecb, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2719ed2, SUC_lexicon_tags_31 },
    { 0x16319ed2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x16919ed6, SUC_lexicon_tags_14 },
    { 0xbfb19ed7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d19eda, SUC_lexicon_tags_15 },
    { 0x57919edb, SUC_lexicon_tags_47 },
    { 0x77119edc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f19ede, SUC_lexicon_tags_39 },
    { 0x7d319edf, SUC_lexicon_tags_13 },
    { 0xa5919edf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7119ee7, SUC_lexicon_tags_27 },
    { 0xa8d19ee7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd9b19eea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d19eed, SUC_lexicon_tags_15 },
    { 0xbd719eee, SUC_lexicon_tags_47 },
    { 0x1119eed, SUC_lexicon_tags_15 },
    { 0x81319ef0, SUC_lexicon_tags_25 },
    { 0x2119eee, SUC_lexicon_tags_17 },
    { 0xc7719eed, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x28119ef5, SUC_lexicon_tags_92 },
    { 0xd5319ef5, SUC_lexicon_tags_38 },
    { 0x65919ef7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d19efa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58919efc, SUC_lexicon_tags_15 },
    { 0x60f19efc, SUC_lexicon_tags_45 },
    { 0x66d19efd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22319f03, SUC_lexicon_tags_3 },
    { 0x33519f04, SUC_lexicon_tags_31 },
    { 0x23719f05, SUC_lexicon_tags_19 },
    { 0x43d19f04, SUC_lexicon_tags_1 },
    { 0x2d19f06, SUC_lexicon_tags_19 },
    { 0x39d19f05, SUC_lexicon_tags_27 },
    { 0x59d19f04, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c719f0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91319f0f, SUC_lexicon_tags_23 },
    { 0x7119f10, SUC_lexicon_tags_25 },
    { 0x17719f0f, SUC_lexicon_tags_32 },
    { 0x4c519f10, SUC_lexicon_tags_11 },
    { 0xeff19f13, SUC_lexicon_tags_15 },
    { 0x28919f14, SUC_lexicon_tags_541 },
    { 0x1c319f14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68319f17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x22719f1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63d19f1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4eb19f1e, SUC_lexicon_tags_33 },
    { 0x2f719f1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0319f21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4119f25, SUC_lexicon_tags_47 },
    { 0xaf319f26, SUC_lexicon_tags_55 },
    { 0xfcd19f27, SUC_lexicon_tags_47 },
    { 0xa3719f25, SUC_lexicon_tags_33 },
    { 0x11319f29, SUC_lexicon_tags_43 },
    { 0xe3319f2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3119f2f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13519f36, SUC_lexicon_tags_11 },
    { 0x28b19f37, SUC_lexicon_tags_12 },
    { 0x77319f37, SUC_lexicon_tags_1 },
    { 0xbd719f38, SUC_lexicon_tags_27 },
    { 0xea919f3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dd19f3c, SUC_lexicon_tags_15 },
    { 0x73319f3c, SUC_lexicon_tags_11 },
    { 0xdcf19f3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89719f41, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd19f44, SUC_lexicon_tags_7 },
    { 0xdb919f45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9919f48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbe119f4a, SUC_lexicon_tags_8 },
    { 0x6b119f4b, SUC_lexicon_tags_9 },
    { 0x48d19f4c, SUC_lexicon_tags_19 },
    { 0x50319f4d, SUC_lexicon_tags_33 },
    { 0xd8f19f4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f19f52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5319f57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd1b19f59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb19f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42719f5e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b19f62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c919f67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c119f69, SUC_lexicon_tags_9 },
    { 0x3c319f6a, SUC_lexicon_tags_8 },
    { 0x9c519f6a, SUC_lexicon_tags_15 },
    { 0xf2919f6b, SUC_lexicon_tags_15 },
    { 0xaf19f6a, SUC_lexicon_tags_26 },
    { 0x3f19f6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad719f73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9319f76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8d19f78, SUC_lexicon_tags_25 },
    { 0x59b19f78, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6f319f7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf319f7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6719f81, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd8f19f83, SUC_lexicon_tags_37 },
    { 0xa9f19f84, SUC_lexicon_tags_18 },
    { 0x57919f84, SUC_lexicon_tags_13 },
    { 0x19f19f86, SUC_lexicon_tags_11 },
    { 0xf7519f83, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x86d19f89, SUC_lexicon_tags_2 },
    { 0xf6519f89, SUC_lexicon_tags_30 },
    { 0x68f19f89, SUC_lexicon_tags_25 },
    { 0x94f19f8a, SUC_lexicon_tags_33 },
    { 0xd1319f8d, SUC_lexicon_tags_17 },
    { 0x66919f8d, SUC_lexicon_tags_25 },
    { 0x56b19f8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1319f92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97319f94, SUC_lexicon_tags_25 },
    { 0x4c319f94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd19f98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff19f9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b719faa, SUC_lexicon_tags_33 },
    { 0x1e119fab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82b19fad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f519fb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84119fb6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x57319fb8, SUC_lexicon_tags_17 },
    { 0xbdf19fb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1919fbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9319fbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1119fc1, SUC_lexicon_tags_141 },
    { 0x9a319fc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40119fc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77119fc9, SUC_lexicon_tags_11 },
    { 0x1cf19fc9, SUC_lexicon_tags_25 },
    { 0x98119fca, SUC_lexicon_tags_2 },
    { 0x11b19fcc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc1719fce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd19fd6, SUC_lexicon_tags_7 },
    { 0x2eb19fd6, SUC_lexicon_tags_25 },
    { 0xa519fd8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf19fdd, SUC_lexicon_tags_7 },
    { 0x4af19fde, SUC_lexicon_tags_37 },
    { 0x68519fdf, SUC_lexicon_tags_25 },
    { 0x4c519fdf, SUC_lexicon_tags_68 },
    { 0x44d19fe1, SUC_lexicon_tags_25 },
    { 0x1d19fde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb0319fe4, SUC_lexicon_tags_17 },
    { 0x4c519fe5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5119fe8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x72519fea, SUC_lexicon_tags_62 },
    { 0x48f19feb, SUC_lexicon_tags_2 },
    { 0x1e119fec, SUC_lexicon_tags_10 },
    { 0xf1519fed, SUC_lexicon_tags_37 },
    { 0xf8919fee, SUC_lexicon_tags_1 },
    { 0x1bf19fed, SUC_lexicon_tags_37 },
    { 0x7c319ff0, SUC_lexicon_tags_25 },
    { 0x88319ff1, SUC_lexicon_tags_17 },
    { 0x6bf19ff0, SUC_lexicon_tags_4 },
    { 0x4cb19ff2, SUC_lexicon_tags_37 },
    { 0xcd119ff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76d19ff6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x99119ff8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0519ffa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x39b19ffc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacb19ffe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f1a001, SUC_lexicon_tags_27 },
    { 0x5b91a001, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc71a005, SUC_lexicon_tags_2 },
    { 0x9251a006, SUC_lexicon_tags_53 },
    { 0x48f1a007, SUC_lexicon_tags_27 },
    { 0x42d1a005, SUC_lexicon_tags_15 },
    { 0x9191a009, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe431a00b, SUC_lexicon_tags_11 },
    { 0x8f1a00c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf931a00f, SUC_lexicon_tags_3 },
    { 0xc3d1a00f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8971a012, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5471a014, SUC_lexicon_tags_49 },
    { 0x1571a015, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6891a017, SUC_lexicon_tags_7 },
    { 0x8bd1a018, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7091a01b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa471a01f, SUC_lexicon_tags_66 },
    { 0x3731a020, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191a02a, SUC_lexicon_tags_12 },
    { 0xe911a02b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e51a02f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b1a032, SUC_lexicon_tags_34 },
    { 0xf3f1a032, SUC_lexicon_tags_14 },
    { 0x1e51a034, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a037, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8651a03a, SUC_lexicon_tags_37 },
    { 0x93f1a03b, SUC_lexicon_tags_56 },
    { 0xac71a03c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x80d1a03e, SUC_lexicon_tags_15 },
    { 0xfbb1a03f, SUC_lexicon_tags_23 },
    { 0x2d11a03f, SUC_lexicon_tags_77 },
    { 0x6211a041, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f1a054, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf51a060, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc51a064, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x7931a067, SUC_lexicon_tags_11 },
    { 0x8371a068, SUC_lexicon_tags_1 },
    { 0x8a91a069, SUC_lexicon_tags_1 },
    { 0xf871a06a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1791a06d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f1a073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd71a075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef1a079, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1351a07b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa331a07e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd1a085, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc351a087, SUC_lexicon_tags_47 },
    { 0x34f1a088, SUC_lexicon_tags_60 },
    { 0x3ff1a089, SUC_lexicon_tags_15 },
    { 0xd191a08a, SUC_lexicon_tags_19 },
    { 0x151a088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe131a08d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f91a08f, SUC_lexicon_tags_32 },
    { 0x1171a08f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9751a092, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x891a094, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f1a099, SUC_lexicon_tags_11 },
    { 0x4651a09a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf591a09c, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba31a0a3, SUC_lexicon_tags_21 },
    { 0x3791a0a4, SUC_lexicon_tags_64 },
    { 0x1f11a0a4, SUC_lexicon_tags_26 },
    { 0x3191a0a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f1a0b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d1a0b6, SUC_lexicon_tags_7 },
    { 0xec91a0b7, SUC_lexicon_tags_4 },
    { 0x1d51a0b8, SUC_lexicon_tags_17 },
    { 0x1ad1a0b9, SUC_lexicon_tags_25 },
    { 0x7bb1a0b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd751a0c1, SUC_lexicon_tags_35 },
    { 0x5091a0c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2971a0c9, SUC_lexicon_tags_9 },
    { 0xc1b1a0c9, SUC_lexicon_tags_25 },
    { 0x7c71a0cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb071a0d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9031a0e0, SUC_lexicon_tags_15 },
    { 0xfc31a0e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x371a0e4, SUC_lexicon_tags_13 },
    { 0xb7b1a0e5, SUC_lexicon_tags_31 },
    { 0x2071a0e6, SUC_lexicon_tags_2 },
    { 0x40b1a0e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb1a0ee, SUC_lexicon_tags_15 },
    { 0x31d1a0ef, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb291a0f1, SUC_lexicon_tags_10 },
    { 0x8f1a0f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd71a0f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2351a0fa, SUC_lexicon_tags_19 },
    { 0xb2d1a0fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8111a103, SUC_lexicon_tags_17 },
    { 0x1eb1a104, SUC_lexicon_tags_124 },
    { 0xa591a103, SUC_lexicon_tags_17 },
    { 0xe371a106, SUC_lexicon_tags_8 },
    { 0xaa51a107, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e91a10a, SUC_lexicon_tags_11 },
    { 0x7c91a10b, SUC_lexicon_tags_15 },
    { 0xfc51a10b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2651a10f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b1a112, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb571a114, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f1a117, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8431a11d, SUC_lexicon_tags_19 },
    { 0x8571a11e, SUC_lexicon_tags_14 },
    { 0x911a11f, SUC_lexicon_tags_12 },
    { 0x411a120, SUC_lexicon_tags_15 },
    { 0x32d1a11e, SUC_lexicon_tags_68 },
    { 0x7511a120, SUC_lexicon_tags_39 },
    { 0x8251a123, SUC_lexicon_tags_82 },
    { 0x9031a11f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1191a127, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2251a12d, SUC_lexicon_tags_47 },
    { 0x87d1a12e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a11a131, SUC_lexicon_tags_25 },
    { 0x73d1a131, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd1a135, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b51a139, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7311a13b, SUC_lexicon_tags_158 },
    { 0x5b91a13c, SUC_lexicon_tags_15 },
    { 0x6b1a13b, SUC_lexicon_tags_63 },
    { 0x5431a13e, SUC_lexicon_tags_37 },
    { 0x9ef1a13f, SUC_lexicon_tags_10 },
    { 0xb5f1a13c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd571a142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1a145, SUC_lexicon_tags_37 },
    { 0xdbf1a146, SUC_lexicon_tags_63 },
    { 0x76b1a145, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc91a14a, SUC_lexicon_tags_14 },
    { 0xd2b1a14b, SUC_lexicon_tags_60 },
    { 0x3391a14a, SUC_lexicon_tags_3 },
    { 0x1911a14c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x211a14f, SUC_lexicon_tags_29 },
    { 0xc651a150, SUC_lexicon_tags_26 },
    { 0x3af1a150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb1a154, SUC_lexicon_tags_26 },
    { 0x3cf1a155, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x2f1a157, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x171a159, SUC_lexicon_tags_37 },
    { 0x6fb1a159, SUC_lexicon_tags_25 },
    { 0xde11a15b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f1a165, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb791a169, SUC_lexicon_tags_8 },
    { 0x1671a16a, SUC_lexicon_tags_1 },
    { 0xff31a16b, SUC_lexicon_tags_54 },
    { 0x1f71a16b, SUC_lexicon_tags_34 },
    { 0x3091a16d, SUC_lexicon_tags_12 },
    { 0xbd31a16c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f1a173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58b1a175, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8091a178, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x311a17b, SUC_lexicon_tags_2 },
    { 0x1691a17c, SUC_lexicon_tags_65 },
    { 0xf1b1a17b, SUC_lexicon_tags_17 },
    { 0x3971a17c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1631a181, SUC_lexicon_tags_1 },
    { 0xd9d1a182, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe2f1a184, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4811a18e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e71a192, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9031a199, SUC_lexicon_tags_48 },
    { 0xbc11a19a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b1a19f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf051a1a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5311a1a6, SUC_lexicon_tags_45 },
    { 0x40b1a1a6, SUC_lexicon_tags_28 },
    { 0x9a51a1a6, SUC_lexicon_tags_25 },
    { 0x1f71a1a7, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5571a1ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6511a1b2, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xc511a1b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa91a1b6, SUC_lexicon_tags_2 },
    { 0x98b1a1b7, SUC_lexicon_tags_84 },
    { 0x8b1a1b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xff31a1ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31a1bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4291a1c7, SUC_lexicon_tags_15 },
    { 0x3371a1c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb6f1a1ca, SUC_lexicon_tags_4 },
    { 0x7dd1a1cb, SUC_lexicon_tags_17 },
    { 0x5dd1a1cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb51a1cf, SUC_lexicon_tags_32 },
    { 0x58d1a1cf, SUC_lexicon_tags_37 },
    { 0x2a71a1d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b1a1d5, SUC_lexicon_tags_11 },
    { 0xcf71a1d6, SUC_lexicon_tags_30 },
    { 0x9971a1d5, SUC_lexicon_tags_33 },
    { 0xfb1a1d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5351a1da, SUC_lexicon_tags_15 },
    { 0x1091a1da, SUC_lexicon_tags_66 },
    { 0xc611a1db, SUC_lexicon_tags_11 },
    { 0x99f1a1dd, SUC_lexicon_tags_69 },
    { 0x74b1a1dc, SUC_lexicon_tags_3 },
    { 0xae51a1da, SUC_lexicon_tags_2 },
    { 0x25f1a1e0, SUC_lexicon_tags_15 },
    { 0x1ed1a1e1, SUC_lexicon_tags_38 },
    { 0x18d1a1dd, SUC_lexicon_tags_27 },
    { 0x26f1a1de, SUC_lexicon_tags_15 },
    { 0x5b31a1e4, SUC_lexicon_tags_13 },
    { 0x16f1a1e5, SUC_lexicon_tags_1 },
    { 0xfab1a1e4, SUC_lexicon_tags_29 },
    { 0xc31a1e7, SUC_lexicon_tags_17 },
    { 0x3351a1e0, SUC_lexicon_tags_37 },
    { 0xb9f1a1e9, SUC_lexicon_tags_12 },
    { 0x9cf1a1ea, SUC_lexicon_tags_25 },
    { 0x1cf1a1e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7371a1ed, SUC_lexicon_tags_26 },
    { 0x1231a1ee, SUC_lexicon_tags_11 },
    { 0xd0b1a1ef, SUC_lexicon_tags_33 },
    { 0x5191a1ef, SUC_lexicon_tags_1 },
    { 0xc0d1a1ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f1a1f8, SUC_lexicon_tags_107 },
    { 0xfb51a1f8, SUC_lexicon_tags_25 },
    { 0x3b1a1f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df1a1fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e91a209, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf791a20b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a31a20e, SUC_lexicon_tags_47 },
    { 0x5311a20f, SUC_lexicon_tags_23 },
    { 0x4911a210, SUC_lexicon_tags_77 },
    { 0x7491a211, SUC_lexicon_tags_44 },
    { 0x6311a210, SUC_lexicon_tags_25 },
    { 0x4c91a213, SUC_lexicon_tags_84 },
    { 0xf071a214, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc811a216, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3e91a218, SUC_lexicon_tags_37 },
    { 0x9651a219, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb191a21c, SUC_lexicon_tags_25 },
    { 0x651a21d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4011a220, SUC_lexicon_tags_37 },
    { 0x93d1a220, SUC_lexicon_tags_2 },
    { 0x7af1a221, SUC_lexicon_tags_27 },
    { 0xc971a223, SUC_lexicon_tags_0 },
    { 0xc4b1a224, SUC_lexicon_tags_45 },
    { 0x13d1a225, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7db1a227, SUC_lexicon_tags_69 },
    { 0x8431a227, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc51a22b, SUC_lexicon_tags_84 },
    { 0xe31a22c, SUC_lexicon_tags_39 },
    { 0xedf1a22d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b1a233, SUC_lexicon_tags_11 },
    { 0x17d1a234, SUC_lexicon_tags_19 },
    { 0x4471a234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd271a238, SUC_lexicon_tags_28 },
    { 0xe351a239, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb791a23c, SUC_lexicon_tags_47 },
    { 0x4311a23d, SUC_lexicon_tags_377 },
    { 0x64d1a23d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6271a243, SUC_lexicon_tags_29 },
    { 0x3d11a244, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe771a246, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2531a248, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef1a24b, SUC_lexicon_tags_17 },
    { 0x3971a24b, SUC_lexicon_tags_49 },
    { 0xd331a24d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7211a251, SUC_lexicon_tags_16 },
    { 0x1671a252, SUC_lexicon_tags_13 },
    { 0x71b1a253, SUC_lexicon_tags_60 },
    { 0xb2b1a254, SUC_lexicon_tags_114 },
    { 0x9811a254, SUC_lexicon_tags_15 },
    { 0x8b31a254, SUC_lexicon_tags_46 },
    { 0x5091a257, SUC_lexicon_tags_34 },
    { 0xf91a255, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb1a260, SUC_lexicon_tags_14 },
    { 0xcaf1a261, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3f1a263, SUC_lexicon_tags_25 },
    { 0xbc31a264, SUC_lexicon_tags_48 },
    { 0xf711a263, SUC_lexicon_tags_23 },
    { 0xa111a263, SUC_lexicon_tags_13 },
    { 0x8db1a267, SUC_lexicon_tags_74 },
    { 0xfcd1a267, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1851a26e, SUC_lexicon_tags_17 },
    { 0xb8d1a26e, SUC_lexicon_tags_2 },
    { 0xdd91a270, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b1a272, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f31a27b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb091a283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4671a286, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6db1a288, SUC_lexicon_tags_65 },
    { 0x53f1a289, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0xff91a28b, SUC_lexicon_tags_159 },
    { 0x7851a28c, SUC_lexicon_tags_19 },
    { 0x2dd1a28c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xd271a28f, SUC_lexicon_tags_1 },
    { 0x5311a290, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc91a295, SUC_lexicon_tags_43 },
    { 0x5bd1a295, SUC_lexicon_tags_31 },
    { 0xd911a296, SUC_lexicon_tags_11 },
    { 0x5f71a298, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e31a29e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd771a2a3, SUC_lexicon_tags_25 },
    { 0x3cd1a2a3, SUC_lexicon_tags_12 },
    { 0xa5b1a2a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5271a2a7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f1a2ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6471a2ad, SUC_lexicon_tags_2 },
    { 0x6811a2ad, SUC_lexicon_tags_33 },
    { 0xcff1a2af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c71a2b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc51a2be, SUC_lexicon_tags_19 },
    { 0x52b1a2bf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3531a2c5, SUC_lexicon_tags_12 },
    { 0x5591a2c5, SUC_lexicon_tags_27 },
    { 0x4af1a2c7, SUC_lexicon_tags_16 },
    { 0x48b1a2c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc91a2ca, SUC_lexicon_tags_63 },
    { 0xc291a2cb, SUC_lexicon_tags_25 },
    { 0xfc11a2cc, SUC_lexicon_tags_7 },
    { 0xc431a2cc, SUC_lexicon_tags_12 },
    { 0xabf1a2cd, SUC_lexicon_tags_2 },
    { 0x84d1a2cd, SUC_lexicon_tags_27 },
    { 0x9831a2cd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e71a2d4, SUC_lexicon_tags_12 },
    { 0xb871a2d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafb1a2d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ff1a2d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc831a2dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8dd1a2de, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb71a2e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe91a2e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d1a2e8, SUC_lexicon_tags_2 },
    { 0xddb1a2e9, SUC_lexicon_tags_14 },
    { 0x6811a2ea, SUC_lexicon_tags_1 },
    { 0xa831a2eb, SUC_lexicon_tags_2 },
    { 0x3c11a2ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7a51a2ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2f91a2f0, SUC_lexicon_tags_21 },
    { 0x9ad1a2f0, SUC_lexicon_tags_37 },
    { 0x8ab1a2f2, SUC_lexicon_tags_7 },
    { 0x99f1a2f0, SUC_lexicon_tags_1 },
    { 0x1091a2f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe691a2f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb3f1a2f9, SUC_lexicon_tags_17 },
    { 0xcfb1a2fa, SUC_lexicon_tags_25 },
    { 0xce31a2f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff1a300, SUC_lexicon_tags_50 },
    { 0xdd91a300, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2d91a303, SUC_lexicon_tags_37 },
    { 0xceb1a304, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34d1a306, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc91a309, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cf1a30b, SUC_lexicon_tags_1 },
    { 0x1271a30c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd251a310, SUC_lexicon_tags_38 },
    { 0x7f71a310, SUC_lexicon_tags_38 },
    { 0x1431a311, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x3271a314, SUC_lexicon_tags_11 },
    { 0xbe51a315, SUC_lexicon_tags_2 },
    { 0x5ff1a315, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1591a31b, SUC_lexicon_tags_11 },
    { 0xe011a31c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f11a321, SUC_lexicon_tags_19 },
    { 0x3211a322, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b1a326, SUC_lexicon_tags_79 },
    { 0xca71a326, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9811a32b, SUC_lexicon_tags_43 },
    { 0xe1f1a32c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6091a32f, SUC_lexicon_tags_61 },
    { 0x1891a330, SUC_lexicon_tags_11 },
    { 0xb331a331, SUC_lexicon_tags_21 },
    { 0x1d91a32f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a71a338, SUC_lexicon_tags_8 },
    { 0xc9b1a338, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5231a33d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a31a33f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaff1a341, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc111a345, SUC_lexicon_tags_16 },
    { 0x4831a346, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe511a348, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7411a34d, SUC_lexicon_tags_17 },
    { 0xadd1a34d, SUC_lexicon_tags_1 },
    { 0x17d1a34f, SUC_lexicon_tags_38 },
    { 0x73f1a350, SUC_lexicon_tags_7 },
    { 0x75b1a34f, SUC_lexicon_tags_12 },
    { 0xc5f1a352, SUC_lexicon_tags_25 },
    { 0x96d1a353, SUC_lexicon_tags_15 },
    { 0x1611a34d, SUC_lexicon_tags_25 },
    { 0xa391a355, SUC_lexicon_tags_17 },
    { 0x3891a356, SUC_lexicon_tags_15 },
    { 0xf9d1a354, SUC_lexicon_tags_23 },
    { 0xe851a358, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b31a35c, SUC_lexicon_tags_33 },
    { 0x6351a35d, SUC_lexicon_tags_2 },
    { 0x6cd1a35e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa371a364, SUC_lexicon_tags_12 },
    { 0xb091a365, SUC_lexicon_tags_33 },
    { 0xa2d1a366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d1a36b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb71a36f, SUC_lexicon_tags_37 },
    { 0x4531a370, SUC_lexicon_tags_60 },
    { 0x65b1a371, SUC_lexicon_tags_63 },
    { 0x5e11a370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a11a377, SUC_lexicon_tags_25 },
    { 0x5051a377, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791a37c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1551a37f, SUC_lexicon_tags_11 },
    { 0x2571a37f, SUC_lexicon_tags_11 },
    { 0xfdf1a380, SUC_lexicon_tags_87 },
    { 0x1091a37f, SUC_lexicon_tags_4 },
    { 0xf371a383, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xff11a385, SUC_lexicon_tags_17 },
    { 0xe2f1a386, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x911a388, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe911a38b, SUC_lexicon_tags_39 },
    { 0xd71a38b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d71a38f, SUC_lexicon_tags_28 },
    { 0x27d1a390, SUC_lexicon_tags_11 },
    { 0x1f11a391, SUC_lexicon_tags_15 },
    { 0x6f11a392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd11a394, SUC_lexicon_tags_15 },
    { 0xc871a394, SUC_lexicon_tags_7 },
    { 0xa651a394, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1f51a398, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1491a39a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a91a39c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51a39f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f1a3a3, SUC_lexicon_tags_7 },
    { 0x8151a3a3, SUC_lexicon_tags_18 },
    { 0x5491a3a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f1a3ad, SUC_lexicon_tags_29 },
    { 0x94d1a3ae, SUC_lexicon_tags_7 },
    { 0x4e11a3af, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe451a3b3, SUC_lexicon_tags_17 },
    { 0x96d1a3b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2b1a3b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8231a3b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6051a3c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2291a3c5, SUC_lexicon_tags_12 },
    { 0xa2b1a3c6, SUC_lexicon_tags_19 },
    { 0x6db1a3c5, SUC_lexicon_tags_204 },
    { 0xd5d1a3c8, SUC_lexicon_tags_118 },
    { 0xa3b1a3c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe11a3cd, SUC_lexicon_tags_17 },
    { 0x9d91a3cd, SUC_lexicon_tags_17 },
    { 0x6ad1a3ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b1a3d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb611a3d8, SUC_lexicon_tags_1 },
    { 0x2e11a3d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x851a3db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1591a3df, SUC_lexicon_tags_33 },
    { 0x6f1a3e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b1a3e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8991a3e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x13f1a3e7, SUC_lexicon_tags_60 },
    { 0xf8b1a3e8, SUC_lexicon_tags_11 },
    { 0x7f31a3e8, SUC_lexicon_tags_12 },
    { 0xb4d1a3ea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f1a3f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f51a3f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef1a3f5, SUC_lexicon_tags_29 },
    { 0x9a51a3f6, SUC_lexicon_tags_11 },
    { 0x7911a3f7, SUC_lexicon_tags_31 },
    { 0x2d51a3f7, SUC_lexicon_tags_7 },
    { 0x1551a3f9, SUC_lexicon_tags_27 },
    { 0xf1d1a3fa, SUC_lexicon_tags_15 },
    { 0xeab1a3f7, SUC_lexicon_tags_7 },
    { 0xa811a3f5, SUC_lexicon_tags_11 },
    { 0xe8f1a3fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f1a401, SUC_lexicon_tags_27 },
    { 0x9ed1a402, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf471a407, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa451a409, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad1a40c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3451a40e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf331a413, SUC_lexicon_tags_2 },
    { 0x911a414, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6cd1a416, SUC_lexicon_tags_12 },
    { 0x6df1a417, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f1a41a, SUC_lexicon_tags_11 },
    { 0xea51a41a, SUC_lexicon_tags_7 },
    { 0x2371a41c, SUC_lexicon_tags_91 },
    { 0x5071a41a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11b1a41f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b1a423, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd1a427, SUC_lexicon_tags_17 },
    { 0x2e11a428, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f11a42b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c11a42e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd1a432, SUC_lexicon_tags_2 },
    { 0x1411a433, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b51a435, SUC_lexicon_tags_12 },
    { 0xd191a436, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31a43a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8bd1a43c, SUC_lexicon_tags_1 },
    { 0xa471a43d, SUC_lexicon_tags_12 },
    { 0x9251a43e, SUC_lexicon_tags_15 },
    { 0xa51a43f, SUC_lexicon_tags_23 },
    { 0xf871a43e, SUC_lexicon_tags_47 },
    { 0x4b1a43f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd1a447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf91a449, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4bb1a44b, SUC_lexicon_tags_1 },
    { 0x7631a44c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8271a44e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x371a450, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8a11a452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5671a454, SUC_lexicon_tags_84 },
    { 0xfbd1a455, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d1a461, SUC_lexicon_tags_37 },
    { 0x2591a462, SUC_lexicon_tags_35 },
    { 0xa7f1a463, SUC_lexicon_tags_17 },
    { 0xa0f1a464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4411a469, SUC_lexicon_tags_6 },
    { 0x86d1a46a, SUC_lexicon_tags_37 },
    { 0x42f1a46b, SUC_lexicon_tags_27 },
    { 0x111a46c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b1a476, SUC_lexicon_tags_17 },
    { 0x9d51a476, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d1a47e, SUC_lexicon_tags_37 },
    { 0xcbd1a47f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b1a482, SUC_lexicon_tags_0 },
    { 0x7d51a483, SUC_lexicon_tags_45 },
    { 0xf3b1a484, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc71a486, SUC_lexicon_tags_27 },
    { 0xd1a486, SUC_lexicon_tags_15 },
    { 0x61f1a488, SUC_lexicon_tags_2 },
    { 0x5731a489, SUC_lexicon_tags_35 },
    { 0x5131a486, SUC_lexicon_tags_44 },
    { 0xb1a48b, SUC_lexicon_tags_25 },
    { 0x1551a48c, SUC_lexicon_tags_25 },
    { 0x8351a48b, SUC_lexicon_tags_21 },
    { 0xf771a48d, SUC_lexicon_tags_77 },
    { 0x9f11a48e, SUC_lexicon_tags_29 },
    { 0xb0d1a48f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89b1a492, SUC_lexicon_tags_27 },
    { 0xf3f1a493, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dd1a495, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb451a498, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd171a49d, SUC_lexicon_tags_15 },
    { 0xc2f1a49e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39d1a4a0, SUC_lexicon_tags_37 },
    { 0xa9b1a4a0, SUC_lexicon_tags_1 },
    { 0xdb91a4a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4f1a4a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4971a4ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25f1a4af, SUC_lexicon_tags_15 },
    { 0x671a4af, SUC_lexicon_tags_28 },
    { 0xc491a4b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebd1a4b3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4471a4b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d1a4b9, SUC_lexicon_tags_29 },
    { 0x20f1a4ba, SUC_lexicon_tags_37 },
    { 0x56f1a4bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1171a4c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5b1a4c3, SUC_lexicon_tags_15 },
    { 0xfa11a4c3, SUC_lexicon_tags_49 },
    { 0x4191a4c5, SUC_lexicon_tags_10 },
    { 0x7d1a4c6, SUC_lexicon_tags_47 },
    { 0xb831a4c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa11a4c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1a4cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d1a4d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3511a4d3, SUC_lexicon_tags_11 },
    { 0xcbd1a4d4, SUC_lexicon_tags_64 },
    { 0xdd51a4d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d1a4d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd691a4e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf51a4e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1491a4ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa911a4ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x46b1a4ee, SUC_lexicon_tags_17 },
    { 0xe431a4ef, SUC_lexicon_tags_25 },
    { 0xb091a4f0, SUC_lexicon_tags_55 },
    { 0x5491a4f0, SUC_lexicon_tags_5 },
    { 0x8031a4f1, SUC_lexicon_tags_87 },
    { 0xb611a4f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad1a4f6, SUC_lexicon_tags_1 },
    { 0x9a91a4f6, SUC_lexicon_tags_7 },
    { 0x8dd1a4f8, SUC_lexicon_tags_18 },
    { 0x831a4f9, SUC_lexicon_tags_15 },
    { 0x3271a4f8, SUC_lexicon_tags_29 },
    { 0xf6b1a4fb, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x5a71a4fd, SUC_lexicon_tags_13 },
    { 0x7a71a4fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b71a505, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5971a507, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7271a50b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d31a50d, SUC_lexicon_tags_23 },
    { 0x1831a50e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd71a510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9931a513, SUC_lexicon_tags_32 },
    { 0xbb1a513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x531a516, SUC_lexicon_tags_10 },
    { 0x6931a516, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6411a519, SUC_lexicon_tags_17 },
    { 0x6251a51a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b1a51e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed1a523, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6391a528, SUC_lexicon_tags_74 },
    { 0xcf91a528, SUC_lexicon_tags_37 },
    { 0x51b1a52a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6971a52e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd911a536, SUC_lexicon_tags_27 },
    { 0x3b31a536, SUC_lexicon_tags_25 },
    { 0x1191a538, SUC_lexicon_tags_43 },
    { 0xc231a538, SUC_lexicon_tags_10 },
    { 0xaf51a537, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8871a53d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e11a543, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b31a547, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6491a549, SUC_lexicon_tags_25 },
    { 0x1c91a54a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf091a54c, SUC_lexicon_tags_2 },
    { 0x4d71a54d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa211a553, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5951a556, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6ed1a558, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6591a55a, SUC_lexicon_tags_15 },
    { 0x3cf1a55b, SUC_lexicon_tags_47 },
    { 0x4671a55c, SUC_lexicon_tags_10 },
    { 0x1c91a55a, SUC_lexicon_tags_12 },
    { 0x8851a55e, SUC_lexicon_tags_2 },
    { 0xf2f1a55e, SUC_lexicon_tags_9 },
    { 0xa6d1a55f, SUC_lexicon_tags_7 },
    { 0x2531a561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb11a565, SUC_lexicon_tags_33 },
    { 0x5751a565, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7151a569, SUC_lexicon_tags_45 },
    { 0xc851a56a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1511a56e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb731a575, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4571a577, SUC_lexicon_tags_13 },
    { 0x9a31a578, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9911a580, SUC_lexicon_tags_16 },
    { 0x55d1a580, SUC_lexicon_tags_49 },
    { 0x3df1a582, SUC_lexicon_tags_47 },
    { 0xf31a582, SUC_lexicon_tags_29 },
    { 0x2871a580, SUC_lexicon_tags_37 },
    { 0x4211a582, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611a58a, SUC_lexicon_tags_47 },
    { 0x9c91a58b, SUC_lexicon_tags_49 },
    { 0x4e51a58a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d1a58f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40f1a591, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2711a594, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd931a598, SUC_lexicon_tags_15 },
    { 0x7a71a599, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd511a59c, SUC_lexicon_tags_12 },
    { 0x271a59d, SUC_lexicon_tags_1 },
    { 0x1171a59e, SUC_lexicon_tags_25 },
    { 0x4d1a59e, SUC_lexicon_tags_8 },
    { 0x23b1a5a0, SUC_lexicon_tags_1 },
    { 0x3e51a5a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd451a5a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b1a5a8, SUC_lexicon_tags_15 },
    { 0x8651a5a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f71a5ad, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8731a5b1, SUC_lexicon_tags_25 },
    { 0xab1a5b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb51a5b5, SUC_lexicon_tags_49 },
    { 0x2551a5b5, SUC_lexicon_tags_45 },
    { 0xda51a5b7, SUC_lexicon_tags_170 },
    { 0xe791a5b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xadf1a5ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee51a5c0, SUC_lexicon_tags_29 },
    { 0x9091a5c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cf1a5c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11a5c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e71a5cb, SUC_lexicon_tags_23 },
    { 0xa191a5cb, SUC_lexicon_tags_3 },
    { 0xbad1a5cd, SUC_lexicon_tags_7 },
    { 0x5591a5cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e31a5d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df1a5da, SUC_lexicon_tags_37 },
    { 0x1851a5db, SUC_lexicon_tags_8 },
    { 0x8cd1a5dc, SUC_lexicon_tags_25 },
    { 0xbc11a5dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4a51a5df, SUC_lexicon_tags_17 },
    { 0x75b1a5e0, SUC_lexicon_tags_105 },
    { 0x8d91a5e0, SUC_lexicon_tags_45 },
    { 0x2ad1a5e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfab1a5e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5751a5e7, SUC_lexicon_tags_37 },
    { 0x4d31a5e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d1a5f0, SUC_lexicon_tags_15 },
    { 0x78b1a5f0, SUC_lexicon_tags_2 },
    { 0x4511a5f2, SUC_lexicon_tags_107 },
    { 0x5411a5f2, SUC_lexicon_tags_12 },
    { 0x5c71a5f4, SUC_lexicon_tags_2 },
    { 0x37d1a5f5, SUC_lexicon_tags_2 },
    { 0x3af1a5f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80f1a5f8, SUC_lexicon_tags_11 },
    { 0x66b1a5f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0f1a5fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7271a5ff, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x91f1a601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6331a604, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d1a607, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f71a60a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc91a60f, SUC_lexicon_tags_28 },
    { 0x3591a60f, SUC_lexicon_tags_154 },
    { 0xb271a611, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x31f1a613, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb831a615, SUC_lexicon_tags_12 },
    { 0x2491a616, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b11a618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d31a61e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9411a620, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3251a623, SUC_lexicon_tags_47 },
    { 0xb971a623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2351a628, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec31a62a, SUC_lexicon_tags_2 },
    { 0xc911a62a, SUC_lexicon_tags_33 },
    { 0xf5f1a62b, SUC_lexicon_tags_10 },
    { 0x89b1a62d, SUC_lexicon_tags_37 },
    { 0xa7b1a62e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8351a63c, SUC_lexicon_tags_17 },
    { 0x6db1a63d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa831a640, SUC_lexicon_tags_7 },
    { 0x7251a641, SUC_lexicon_tags_11 },
    { 0xc0b1a642, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5291a645, SUC_lexicon_tags_66 },
    { 0x9411a645, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1671a648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd1a64e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf1a651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5291a654, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xac11a657, SUC_lexicon_tags_15 },
    { 0xbe91a658, SUC_lexicon_tags_25 },
    { 0xd1d1a657, SUC_lexicon_tags_19 },
    { 0x35f1a658, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x95f1a65c, SUC_lexicon_tags_9 },
    { 0xf9d1a65c, SUC_lexicon_tags_7 },
    { 0x6931a65e, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xd51a660, SUC_lexicon_tags_14 },
    { 0xa3d1a660, SUC_lexicon_tags_61 },
    { 0x3a71a660, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d1a668, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xac11a66b, SUC_lexicon_tags_17 },
    { 0x9df1a66c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaed1a66e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc291a672, SUC_lexicon_tags_0 },
    { 0xe031a673, SUC_lexicon_tags_6 },
    { 0x8291a674, SUC_lexicon_tags_25 },
    { 0x2151a675, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3571a67b, SUC_lexicon_tags_37 },
    { 0x7711a67c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4871a683, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3811a688, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d1a68d, SUC_lexicon_tags_43 },
    { 0xe371a68e, SUC_lexicon_tags_14 },
    { 0x6131a68e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7271a691, SUC_lexicon_tags_15 },
    { 0xfd51a692, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af1a698, SUC_lexicon_tags_7 },
    { 0x8f1a699, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd71a69b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf551a69f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b1a6aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c91a6b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d1a6b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9631a6b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b31a6b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe731a6bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7391a6c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcab1a6c7, SUC_lexicon_tags_82 },
    { 0x6191a6c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d71a6ce, SUC_lexicon_tags_11 },
    { 0xdb51a6ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9511a6d5, SUC_lexicon_tags_7 },
    { 0x3fb1a6d6, SUC_lexicon_tags_0 },
    { 0xed51a6d5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d51a6db, SUC_lexicon_tags_7 },
    { 0xeb51a6dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca51a6e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7931a6e3, SUC_lexicon_tags_15 },
    { 0xe531a6e4, SUC_lexicon_tags_160 },
    { 0x1ff1a6e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7651a6e7, SUC_lexicon_tags_57 },
    { 0x1971a6e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d1a6ed, SUC_lexicon_tags_23 },
    { 0x2e11a6ee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1471a6f2, SUC_lexicon_tags_16 },
    { 0xe1b1a6f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9191a6f5, SUC_lexicon_tags_15 },
    { 0x1611a6f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf771a6f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf1a6fe, SUC_lexicon_tags_31 },
    { 0xbb71a6ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9011a701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b1a704, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f1a707, SUC_lexicon_tags_2 },
    { 0x5f51a708, SUC_lexicon_tags_31 },
    { 0xac31a707, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd1a70c, SUC_lexicon_tags_25 },
    { 0x3d11a70d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d1a712, SUC_lexicon_tags_414 },
    { 0x23f1a713, SUC_lexicon_tags_7 },
    { 0x85f1a714, SUC_lexicon_tags_17 },
    { 0x9971a715, SUC_lexicon_tags_37 },
    { 0xea51a715, SUC_lexicon_tags_2 },
    { 0xb3b1a716, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9191a720, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a11a725, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f1a729, SUC_lexicon_tags_15 },
    { 0x1311a729, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa7b1a72c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f1a734, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb1a737, SUC_lexicon_tags_14 },
    { 0x2c31a738, SUC_lexicon_tags_17 },
    { 0x791a737, SUC_lexicon_tags_11 },
    { 0xca11a737, SUC_lexicon_tags_1 },
    { 0x6011a73b, SUC_lexicon_tags_43 },
    { 0xadf1a73b, SUC_lexicon_tags_15 },
    { 0x4a91a738, SUC_lexicon_tags_33 },
    { 0x2371a73e, SUC_lexicon_tags_2 },
    { 0x32b1a73f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af1a742, SUC_lexicon_tags_37 },
    { 0x26f1a743, SUC_lexicon_tags_8 },
    { 0x54d1a742, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x95b1a746, SUC_lexicon_tags_83 },
    { 0x7d51a746, SUC_lexicon_tags_12 },
    { 0x7ef1a748, SUC_lexicon_tags_107 },
    { 0x47f1a747, SUC_lexicon_tags_25 },
    { 0x3331a746, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc091a74c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed1a74f, SUC_lexicon_tags_17 },
    { 0x3f31a74f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e11a755, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1211a758, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff1a75e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351a764, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf1a767, SUC_lexicon_tags_37 },
    { 0x9c71a768, SUC_lexicon_tags_18 },
    { 0x8091a769, SUC_lexicon_tags_23 },
    { 0xf2d1a769, SUC_lexicon_tags_65 },
    { 0x8291a769, SUC_lexicon_tags_27 },
    { 0xfd51a76c, SUC_lexicon_tags_26 },
    { 0x23d1a768, SUC_lexicon_tags_49 },
    { 0x6e31a76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f1a778, SUC_lexicon_tags_7 },
    { 0xffb1a779, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d11a77b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1a77f, SUC_lexicon_tags_23 },
    { 0xd091a77f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeff1a782, SUC_lexicon_tags_61 },
    { 0xf611a783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd611a788, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c71a78f, SUC_lexicon_tags_215 },
    { 0x3ad1a78f, SUC_lexicon_tags_33 },
    { 0x96d1a791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e31a793, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x291a795, SUC_lexicon_tags_61 },
    { 0xb311a796, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f11a798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1631a79d, SUC_lexicon_tags_2 },
    { 0x20b1a79d, SUC_lexicon_tags_40 },
    { 0x6ef1a79f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd1a7a2, SUC_lexicon_tags_25 },
    { 0x1731a7a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd71a7a7, SUC_lexicon_tags_63 },
    { 0x5d91a7a8, SUC_lexicon_tags_38 },
    { 0x6111a7a9, SUC_lexicon_tags_17 },
    { 0x5b91a7aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2f1a7ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2411a7af, SUC_lexicon_tags_7 },
    { 0x5891a7b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xecb1a7b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a31a7b4, SUC_lexicon_tags_25 },
    { 0xa7d1a7b4, SUC_lexicon_tags_8 },
    { 0xf851a7b6, SUC_lexicon_tags_25 },
    { 0xa571a7b5, SUC_lexicon_tags_1 },
    { 0xd5d1a7b6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x66b1a7ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f1a7bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74b1a7bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d91a7c4, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b51a7c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9011a7ce, SUC_lexicon_tags_12 },
    { 0xc5f1a7ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd631a7d1, SUC_lexicon_tags_37 },
    { 0x4651a7d2, SUC_lexicon_tags_30 },
    { 0xea31a7d3, SUC_lexicon_tags_7 },
    { 0x82d1a7d4, SUC_lexicon_tags_85 },
    { 0x3e71a7d5, SUC_lexicon_tags_11 },
    { 0x5051a7d5, SUC_lexicon_tags_17 },
    { 0x5ab1a7d7, SUC_lexicon_tags_29 },
    { 0x97b1a7d8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb1a7dc, SUC_lexicon_tags_25 },
    { 0x3f51a7dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf1a7e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9471a7e9, SUC_lexicon_tags_129 },
    { 0x5d91a7ea, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd31a7ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b1a7f1, SUC_lexicon_tags_2 },
    { 0x1231a7f2, SUC_lexicon_tags_4 },
    { 0xadf1a7f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2131a7f9, SUC_lexicon_tags_60 },
    { 0xc871a7fa, SUC_lexicon_tags_15 },
    { 0xb0f1a7fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1711a801, SUC_lexicon_tags_60 },
    { 0xcd31a801, SUC_lexicon_tags_30 },
    { 0x1bd1a803, SUC_lexicon_tags_39 },
    { 0xf031a804, SUC_lexicon_tags_25 },
    { 0xed71a804, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b11a807, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5031a80a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe331a80c, SUC_lexicon_tags_15 },
    { 0x3eb1a80d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfed1a80f, SUC_lexicon_tags_87 },
    { 0xad91a80f, SUC_lexicon_tags_47 },
    { 0xff51a810, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1a81b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe311a81f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8531a821, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b91a824, SUC_lexicon_tags_11 },
    { 0x67f1a825, SUC_lexicon_tags_152 },
    { 0x2091a826, SUC_lexicon_tags_2 },
    { 0x9cb1a826, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa031a829, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb591a830, SUC_lexicon_tags_49 },
    { 0x47b1a830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d91a835, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf891a837, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc911a83a, SUC_lexicon_tags_17 },
    { 0xc5f1a83a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf1a83f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff91a848, SUC_lexicon_tags_63 },
    { 0x1ed1a849, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf1a84e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f1a853, SUC_lexicon_tags_37 },
    { 0x1991a854, SUC_lexicon_tags_6 },
    { 0x8331a855, SUC_lexicon_tags_14 },
    { 0x9271a854, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7891a858, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb331a85c, SUC_lexicon_tags_15 },
    { 0x28b1a85d, SUC_lexicon_tags_30 },
    { 0xf751a85e, SUC_lexicon_tags_27 },
    { 0xdb91a85e, SUC_lexicon_tags_1 },
    { 0xe5b1a85f, SUC_lexicon_tags_7 },
    { 0x9651a861, SUC_lexicon_tags_15 },
    { 0x17b1a85f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd11a864, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1a869, SUC_lexicon_tags_47 },
    { 0xa0f1a869, SUC_lexicon_tags_37 },
    { 0x16b1a86b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53d1a86d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa91a871, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe871a874, SUC_lexicon_tags_68 },
    { 0xe3b1a874, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f1a879, SUC_lexicon_tags_25 },
    { 0xb4d1a87a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2971a87f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b51a881, SUC_lexicon_tags_14 },
    { 0x5971a882, SUC_lexicon_tags_8 },
    { 0x3a11a883, SUC_lexicon_tags_28 },
    { 0xbf31a884, SUC_lexicon_tags_91 },
    { 0xdc51a883, SUC_lexicon_tags_79 },
    { 0x8d31a886, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf1a88a, SUC_lexicon_tags_49 },
    { 0xe151a88b, SUC_lexicon_tags_49 },
    { 0x93f1a88b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b1a891, SUC_lexicon_tags_10 },
    { 0x2151a891, SUC_lexicon_tags_17 },
    { 0x7dd1a891, SUC_lexicon_tags_57 },
    { 0x8371a894, SUC_lexicon_tags_34 },
    { 0x9eb1a891, SUC_lexicon_tags_0 },
    { 0x5691a892, SUC_lexicon_tags_12 },
    { 0x39b1a897, SUC_lexicon_tags_17 },
    { 0xbfb1a897, SUC_lexicon_tags_6 },
    { 0xa251a898, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8951a89c, SUC_lexicon_tags_49 },
    { 0xf411a89d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b51a8a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe71a8a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a8a8, SUC_lexicon_tags_6 },
    { 0xf9d1a8a9, SUC_lexicon_tags_1 },
    { 0x27d1a8aa, SUC_lexicon_tags_30 },
    { 0x98b1a8a8, SUC_lexicon_tags_91 },
    { 0x7bb1a8ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc631a8b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9751a8b2, SUC_lexicon_tags_37 },
    { 0x92f1a8b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x10f1a8b5, SUC_lexicon_tags_17 },
    { 0x8891a8b6, SUC_lexicon_tags_2 },
    { 0xd7f1a8b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb431a8bc, SUC_lexicon_tags_7 },
    { 0xa931a8bd, SUC_lexicon_tags_11 },
    { 0x2cd1a8be, SUC_lexicon_tags_2 },
    { 0xb411a8bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x87d1a8c1, SUC_lexicon_tags_25 },
    { 0x25b1a8c2, SUC_lexicon_tags_37 },
    { 0x3051a8c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3af1a8c5, SUC_lexicon_tags_2 },
    { 0x8f51a8c6, SUC_lexicon_tags_39 },
    { 0xd1d1a8c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4471a8ca, SUC_lexicon_tags_14 },
    { 0x8c51a8cb, SUC_lexicon_tags_11 },
    { 0x211a8ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92d1a8ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf1a8d2, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5391a8da, SUC_lexicon_tags_7 },
    { 0x9571a8db, SUC_lexicon_tags_9 },
    { 0x21b1a8dc, SUC_lexicon_tags_63 },
    { 0x7351a8db, SUC_lexicon_tags_7 },
    { 0xbc51a8dc, SUC_lexicon_tags_38 },
    { 0x2371a8df, SUC_lexicon_tags_29 },
    { 0x14b1a8e0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe3f1a8e2, SUC_lexicon_tags_15 },
    { 0xe071a8e3, SUC_lexicon_tags_82 },
    { 0xa351a8e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa931a8e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6b1a8e8, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f1a8f3, SUC_lexicon_tags_12 },
    { 0x7911a8f3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa11a8f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df1a8f9, SUC_lexicon_tags_15 },
    { 0xe1d1a8fa, SUC_lexicon_tags_27 },
    { 0x91f1a8fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1071a8ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6291a905, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed71a908, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fb1a90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc51a90c, SUC_lexicon_tags_7 },
    { 0xf031a90d, SUC_lexicon_tags_47 },
    { 0x2c91a90e, SUC_lexicon_tags_2 },
    { 0x15b1a90f, SUC_lexicon_tags_12 },
    { 0x9271a90f, SUC_lexicon_tags_15 },
    { 0x8e91a910, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2111a915, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8511a917, SUC_lexicon_tags_66 },
    { 0x1051a918, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bf1a91a, SUC_lexicon_tags_43 },
    { 0xd9d1a91a, SUC_lexicon_tags_38 },
    { 0xc91a91c, SUC_lexicon_tags_17 },
    { 0x1771a91d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d1a923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a71a925, SUC_lexicon_tags_33 },
    { 0xb511a926, SUC_lexicon_tags_7 },
    { 0xf471a925, SUC_lexicon_tags_17 },
    { 0x9a91a926, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b1a92d, SUC_lexicon_tags_15 },
    { 0xe8d1a92e, SUC_lexicon_tags_1675 },
    { 0xb3f1a92f, SUC_lexicon_tags_14 },
    { 0x631a92f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeab1a932, SUC_lexicon_tags_11 },
    { 0xc6d1a933, SUC_lexicon_tags_30 },
    { 0x4551a932, SUC_lexicon_tags_1 },
    { 0xd8f1a933, SUC_lexicon_tags_25 },
    { 0xc131a935, SUC_lexicon_tags_7 },
    { 0x5951a935, SUC_lexicon_tags_54 },
    { 0xcd91a935, SUC_lexicon_tags_39 },
    { 0x56b1a939, SUC_lexicon_tags_49 },
    { 0x8791a935, SUC_lexicon_tags_1 },
    { 0x8a91a93b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa191a93d, SUC_lexicon_tags_11 },
    { 0xaa31a93e, SUC_lexicon_tags_39 },
    { 0xe5d1a93f, SUC_lexicon_tags_11 },
    { 0xcb71a940, SUC_lexicon_tags_45 },
    { 0xb731a940, SUC_lexicon_tags_23 },
    { 0x35f1a942, SUC_lexicon_tags_25 },
    { 0xb691a942, SUC_lexicon_tags_65 },
    { 0x2671a944, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b1a949, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f11a94c, SUC_lexicon_tags_47 },
    { 0x1931a94d, SUC_lexicon_tags_0 },
    { 0x8391a94e, SUC_lexicon_tags_37 },
    { 0xb891a94c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xda71a952, SUC_lexicon_tags_1 },
    { 0xc2b1a952, SUC_lexicon_tags_1 },
    { 0x2cd1a952, SUC_lexicon_tags_46 },
    { 0x4791a954, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd1a95a, SUC_lexicon_tags_19 },
    { 0xe811a95b, SUC_lexicon_tags_10 },
    { 0x76d1a95c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2791a95e, SUC_lexicon_tags_23 },
    { 0xec11a95f, SUC_lexicon_tags_12 },
    { 0xf331a95f, SUC_lexicon_tags_15 },
    { 0xc7b1a95f, SUC_lexicon_tags_37 },
    { 0x6cf1a960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb711a964, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8091a969, SUC_lexicon_tags_32 },
    { 0x7bb1a96a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e51a974, SUC_lexicon_tags_49 },
    { 0x5b11a974, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4071a97a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6b11a97c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87f1a97e, SUC_lexicon_tags_60 },
    { 0x4af1a97f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1711a988, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf2d1a98a, SUC_lexicon_tags_27 },
    { 0xf571a98a, SUC_lexicon_tags_1 },
    { 0x4ab1a98c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7711a98e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7ef1a990, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3d31a992, SUC_lexicon_tags_25 },
    { 0x6711a993, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d1a996, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3671a99c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8991a9a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d1a9a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d1a9a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa71a9ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a51a9b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe091a9b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe91a9b8, SUC_lexicon_tags_4 },
    { 0x2cf1a9b9, SUC_lexicon_tags_38 },
    { 0xdad1a9ba, SUC_lexicon_tags_83 },
    { 0xad1a9bb, SUC_lexicon_tags_38 },
    { 0x19b1a9b9, SUC_lexicon_tags_63 },
    { 0x2f91a9b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b1a9c5, SUC_lexicon_tags_19 },
    { 0x9f1a9c6, SUC_lexicon_tags_7 },
    { 0xd531a9c6, SUC_lexicon_tags_62 },
    { 0xd1f1a9c8, SUC_lexicon_tags_82 },
    { 0x6611a9c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e71a9cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7b91a9ce, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f51a9d2, SUC_lexicon_tags_37 },
    { 0xb951a9d2, SUC_lexicon_tags_25 },
    { 0xcd51a9d4, SUC_lexicon_tags_1 },
    { 0xfcd1a9d4, SUC_lexicon_tags_5 },
    { 0xa4f1a9d6, SUC_lexicon_tags_0 },
    { 0xa711a9d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42f1a9d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3511a9dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d1a9e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb431a9e4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe3d1a9e6, SUC_lexicon_tags_37 },
    { 0x7f1a9e7, SUC_lexicon_tags_11 },
    { 0x2f91a9e8, SUC_lexicon_tags_8 },
    { 0xaff1a9e6, SUC_lexicon_tags_66 },
    { 0x6db1a9e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd131a9ed, SUC_lexicon_tags_23 },
    { 0xbdd1a9ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bb1a9f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e51a9f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed1a9f8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1a9fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2171aa00, SUC_lexicon_tags_39 },
    { 0x2351aa00, SUC_lexicon_tags_11 },
    { 0x7451aa02, SUC_lexicon_tags_2 },
    { 0xc1d1aa03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9011aa05, SUC_lexicon_tags_7 },
    { 0xb931aa06, SUC_lexicon_tags_63 },
    { 0xb851aa05, SUC_lexicon_tags_33 },
    { 0xe671aa08, SUC_lexicon_tags_8 },
    { 0xbbb1aa09, SUC_lexicon_tags_28 },
    { 0x9f31aa0a, SUC_lexicon_tags_11 },
    { 0x2d1aa0a, SUC_lexicon_tags_35 },
    { 0xf251aa0b, SUC_lexicon_tags_37 },
    { 0xa611aa07, SUC_lexicon_tags_12 },
    { 0x5631aa08, SUC_lexicon_tags_17 },
    { 0x9d91aa0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6591aa15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b1aa19, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d11aa1c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7531aa26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb1aa29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e11aa2f, SUC_lexicon_tags_2 },
    { 0x31b1aa2f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df1aa33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8851aa36, SUC_lexicon_tags_33 },
    { 0xb391aa37, SUC_lexicon_tags_0 },
    { 0x1bf1aa38, SUC_lexicon_tags_30 },
    { 0x42b1aa39, SUC_lexicon_tags_43 },
    { 0x3451aa38, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6211aa3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc391aa3f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6131aa42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2531aa44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b1aa4a, SUC_lexicon_tags_11 },
    { 0x1411aa4a, SUC_lexicon_tags_1 },
    { 0x3bb1aa4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4f1aa4e, SUC_lexicon_tags_6 },
    { 0x3651aa4e, SUC_lexicon_tags_16 },
    { 0x26d1aa50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf11aa57, SUC_lexicon_tags_1 },
    { 0xc91aa58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1351aa5c, SUC_lexicon_tags_38 },
    { 0x6e71aa5d, SUC_lexicon_tags_39 },
    { 0x80d1aa5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8731aa60, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d1aa63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b1aa66, SUC_lexicon_tags_54 },
    { 0x5851aa67, SUC_lexicon_tags_37 },
    { 0xe131aa68, SUC_lexicon_tags_12 },
    { 0xa1f1aa69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b31aa6b, SUC_lexicon_tags_33 },
    { 0x48b1aa6c, SUC_lexicon_tags_56 },
    { 0xb5b1aa6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6351aa6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8531aa73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f1aa7a, SUC_lexicon_tags_87 },
    { 0x251aa7b, SUC_lexicon_tags_10 },
    { 0x2af1aa7c, SUC_lexicon_tags_37 },
    { 0xc751aa7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x971aa7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb91aa81, SUC_lexicon_tags_19 },
    { 0x6ed1aa82, SUC_lexicon_tags_15 },
    { 0xec91aa83, SUC_lexicon_tags_1 },
    { 0xf611aa84, SUC_lexicon_tags_12 },
    { 0xec71aa85, SUC_lexicon_tags_37 },
    { 0x9211aa84, SUC_lexicon_tags_17 },
    { 0x6ad1aa87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe6b1aa89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xded1aa8c, SUC_lexicon_tags_47 },
    { 0x5291aa8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab11aa8f, SUC_lexicon_tags_10 },
    { 0x3c51aa8f, SUC_lexicon_tags_37 },
    { 0x59b1aa91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8191aa94, SUC_lexicon_tags_25 },
    { 0x2651aa95, SUC_lexicon_tags_15 },
    { 0x4d31aa95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f1aa99, SUC_lexicon_tags_45 },
    { 0x2711aa9a, SUC_lexicon_tags_27 },
    { 0x6fd1aa9b, SUC_lexicon_tags_1 },
    { 0x851aa9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef91aaa1, SUC_lexicon_tags_37 },
    { 0x551aaa2, SUC_lexicon_tags_60 },
    { 0xdd31aaa1, SUC_lexicon_tags_47 },
    { 0x8311aaa1, SUC_lexicon_tags_79 },
    { 0x2e31aaa5, SUC_lexicon_tags_15 },
    { 0xfe91aaa5, SUC_lexicon_tags_25 },
    { 0x31d1aaa2, SUC_lexicon_tags_4 },
    { 0x9751aaa1, SUC_lexicon_tags_17 },
    { 0x8ab1aaa3, SUC_lexicon_tags_1 },
    { 0x9911aaa2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf231aab2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f1aab6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f1aaba, SUC_lexicon_tags_60 },
    { 0x5911aabb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d1aabf, SUC_lexicon_tags_23 },
    { 0xc051aac0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7691aac2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6151aac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7131aac8, SUC_lexicon_tags_12 },
    { 0x2391aac9, SUC_lexicon_tags_6 },
    { 0x7391aaca, SUC_lexicon_tags_2 },
    { 0x6fd1aacb, SUC_lexicon_tags_17 },
    { 0xacf1aacc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1aae0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd431aae3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc691aae5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8c11aae7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa271aaea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1aaf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9271aaf3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5731aaf9, SUC_lexicon_tags_8 },
    { 0x1f31aafa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7151aafc, SUC_lexicon_tags_2 },
    { 0x3ab1aafc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe331aaff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d91ab02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1551ab08, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb71ab0b, SUC_lexicon_tags_12 },
    { 0x1a51ab0c, SUC_lexicon_tags_27 },
    { 0xc531ab0b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd691ab12, SUC_lexicon_tags_15 },
    { 0x3dd1ab13, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1911ab17, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b1ab1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f1ab1d, SUC_lexicon_tags_2 },
    { 0x2b91ab1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff31ab22, SUC_lexicon_tags_29 },
    { 0x3c91ab23, SUC_lexicon_tags_45 },
    { 0x7ab1ab24, SUC_lexicon_tags_69 },
    { 0xd6d1ab25, SUC_lexicon_tags_15 },
    { 0x6b1ab26, SUC_lexicon_tags_11 },
    { 0xb851ab27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cf1ab29, SUC_lexicon_tags_11 },
    { 0x1b11ab29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a51ab2d, SUC_lexicon_tags_27 },
    { 0x6391ab2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3ab1ab30, SUC_lexicon_tags_39 },
    { 0x7dd1ab30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f1ab35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1ab38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff1ab3c, SUC_lexicon_tags_31 },
    { 0x5c71ab3d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf1ab41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc991ab45, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8371ab47, SUC_lexicon_tags_45 },
    { 0xafd1ab47, SUC_lexicon_tags_7 },
    { 0x1ad1ab49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcab1ab4b, SUC_lexicon_tags_47 },
    { 0x74d1ab4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3b1ab4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ab53, SUC_lexicon_tags_7 },
    { 0x6bb1ab53, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f71ab5e, SUC_lexicon_tags_15 },
    { 0x431ab5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e31ab68, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d11ab6d, SUC_lexicon_tags_27 },
    { 0xfab1ab6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b11ab72, SUC_lexicon_tags_37 },
    { 0x8a11ab73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e91ab7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51ab7d, SUC_lexicon_tags_65 },
    { 0x9071ab7d, SUC_lexicon_tags_5 },
    { 0xd051ab7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa371ab81, SUC_lexicon_tags_1582 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b1ab85, SUC_lexicon_tags_11 },
    { 0xa711ab86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bf1ab88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5b1ab8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f1ab8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f11ab93, SUC_lexicon_tags_7 },
    { 0x4d11ab94, SUC_lexicon_tags_27 },
    { 0xc871ab94, SUC_lexicon_tags_16 },
    { 0x27d1ab95, SUC_lexicon_tags_27 },
    { 0xf971ab97, SUC_lexicon_tags_63 },
    { 0x75d1ab98, SUC_lexicon_tags_19 },
    { 0x70b1ab99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaf11ab9b, SUC_lexicon_tags_2 },
    { 0x1791ab9c, SUC_lexicon_tags_37 },
    { 0xbd1ab9c, SUC_lexicon_tags_3 },
    { 0xe91ab9e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea51aba6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdaf1aba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc271abab, SUC_lexicon_tags_29 },
    { 0xf7b1abac, SUC_lexicon_tags_2 },
    { 0xc3f1abac, SUC_lexicon_tags_133 },
    { 0xbfd1abac, SUC_lexicon_tags_12 },
    { 0x1e31abad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7551abb4, SUC_lexicon_tags_49 },
    { 0x11d1abb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6111abb9, SUC_lexicon_tags_27 },
    { 0xb491abba, SUC_lexicon_tags_14 },
    { 0x2091abba, SUC_lexicon_tags_47 },
    { 0xf031abba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d1abbf, SUC_lexicon_tags_25 },
    { 0xa3b1abc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b1abc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab1abc8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d1abcd, SUC_lexicon_tags_13 },
    { 0x1f91abcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb1abd2, SUC_lexicon_tags_25 },
    { 0x7831abd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c71abdb, SUC_lexicon_tags_26 },
    { 0x1e91abdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2eb1abde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89f1abe0, SUC_lexicon_tags_37 },
    { 0x37b1abe1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd1abe5, SUC_lexicon_tags_37 },
    { 0xdcd1abe6, SUC_lexicon_tags_7 },
    { 0x81b1abe6, SUC_lexicon_tags_11 },
    { 0xbf71abe6, SUC_lexicon_tags_17 },
    { 0x6a51abe9, SUC_lexicon_tags_46 },
    { 0xa6b1abea, SUC_lexicon_tags_47 },
    { 0x36b1abea, SUC_lexicon_tags_11 },
    { 0xcab1abeb, SUC_lexicon_tags_17 },
    { 0xeef1abe7, SUC_lexicon_tags_7 },
    { 0x9471abea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d1abf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe371abf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb11abfc, SUC_lexicon_tags_45 },
    { 0xc211abfc, SUC_lexicon_tags_29 },
    { 0xa51abfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a91ac01, SUC_lexicon_tags_14 },
    { 0x16d1ac02, SUC_lexicon_tags_33 },
    { 0xa511ac03, SUC_lexicon_tags_1 },
    { 0xbd31ac04, SUC_lexicon_tags_15 },
    { 0xd4d1ac01, SUC_lexicon_tags_37 },
    { 0xf371ac06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d71ac08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b1ac0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc11ac11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7571ac13, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xbcb1ac15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3371ac17, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b11ac1c, SUC_lexicon_tags_0 },
    { 0x8831ac1d, SUC_lexicon_tags_30 },
    { 0xa451ac1e, SUC_lexicon_tags_39 },
    { 0x77f1ac1c, SUC_lexicon_tags_38 },
    { 0x8bd1ac1d, SUC_lexicon_tags_47 },
    { 0x171ac1d, SUC_lexicon_tags_2 },
    { 0x3ad1ac22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4651ac28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a51ac2a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa251ac2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc791ac36, SUC_lexicon_tags_37 },
    { 0xfcd1ac36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d1ac3d, SUC_lexicon_tags_38 },
    { 0x6091ac3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3b1ac40, SUC_lexicon_tags_11 },
    { 0xd871ac41, SUC_lexicon_tags_47 },
    { 0x2811ac41, SUC_lexicon_tags_32 },
    { 0x19b1ac40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbff1ac45, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c91ac48, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd1ac4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf671ac51, SUC_lexicon_tags_9 },
    { 0x1af1ac51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcaf1ac54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6751ac56, SUC_lexicon_tags_27 },
    { 0x76f1ac56, SUC_lexicon_tags_37 },
    { 0xa091ac58, SUC_lexicon_tags_49 },
    { 0x65b1ac59, SUC_lexicon_tags_48 },
    { 0x5111ac5a, SUC_lexicon_tags_37 },
    { 0xe5b1ac5a, SUC_lexicon_tags_11 },
    { 0x55f1ac59, SUC_lexicon_tags_37 },
    { 0x56d1ac5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c11ac66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe871ac6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd151ac70, SUC_lexicon_tags_7 },
    { 0x47d1ac71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe411ac74, SUC_lexicon_tags_7 },
    { 0x9891ac75, SUC_lexicon_tags_15 },
    { 0xeaf1ac76, SUC_lexicon_tags_28 },
    { 0x7b51ac77, SUC_lexicon_tags_27 },
    { 0x6b1ac74, SUC_lexicon_tags_62 },
    { 0x94b1ac75, SUC_lexicon_tags_34 },
    { 0x47f1ac75, SUC_lexicon_tags_32 },
    { 0x9331ac7b, SUC_lexicon_tags_15 },
    { 0x9ed1ac7b, SUC_lexicon_tags_37 },
    { 0x1d11ac78, SUC_lexicon_tags_381 },
    { 0, NULL },
    { 0x7f11ac7f, SUC_lexicon_tags_22 },
    { 0xb471ac7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5431ac85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x94b1ac87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b71ac8e, SUC_lexicon_tags_6 },
    { 0x5871ac8f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf031ac96, SUC_lexicon_tags_11 },
    { 0xd31ac97, SUC_lexicon_tags_15 },
    { 0x5771ac97, SUC_lexicon_tags_8 },
    { 0x30f1ac98, SUC_lexicon_tags_2 },
    { 0xb591ac9a, SUC_lexicon_tags_12 },
    { 0xb211ac9b, SUC_lexicon_tags_33 },
    { 0x7c31ac99, SUC_lexicon_tags_11 },
    { 0x1731ac98, SUC_lexicon_tags_12 },
    { 0x2db1ac9e, SUC_lexicon_tags_33 },
    { 0x9771ac9f, SUC_lexicon_tags_11 },
    { 0x92b1aca0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdef1aca2, SUC_lexicon_tags_38 },
    { 0x67d1aca2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf31aca9, SUC_lexicon_tags_4 },
    { 0xd271aca9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xced1acac, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4671acb0, SUC_lexicon_tags_19 },
    { 0x45f1acb1, SUC_lexicon_tags_1 },
    { 0x7791acb2, SUC_lexicon_tags_17 },
    { 0x6371acb0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f1acb7, SUC_lexicon_tags_129 },
    { 0x3db1acb8, SUC_lexicon_tags_12 },
    { 0x7751acb9, SUC_lexicon_tags_15 },
    { 0xeaf1acba, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d1acbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd1b1acbf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3331acc1, SUC_lexicon_tags_49 },
    { 0x3651acc2, SUC_lexicon_tags_11 },
    { 0x971acc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d1acc6, SUC_lexicon_tags_11 },
    { 0x58f1acc7, SUC_lexicon_tags_37 },
    { 0x5311acc6, SUC_lexicon_tags_28 },
    { 0x5331acc9, SUC_lexicon_tags_0 },
    { 0x7591acca, SUC_lexicon_tags_37 },
    { 0x4111accb, SUC_lexicon_tags_12 },
    { 0x6a11accb, SUC_lexicon_tags_19 },
    { 0x6591accd, SUC_lexicon_tags_2 },
    { 0xab11acce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa1d1acd0, SUC_lexicon_tags_33 },
    { 0x42d1acd1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8fd1acd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c11acd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1ed1acd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef1acdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe791ace0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b1ace4, SUC_lexicon_tags_4 },
    { 0x3b91ace5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73d1ace7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1aced, SUC_lexicon_tags_55 },
    { 0xe811aced, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf771acf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb731acf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd471acfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4111acfe, SUC_lexicon_tags_49 },
    { 0x73f1acfe, SUC_lexicon_tags_19 },
    { 0x4a71acfe, SUC_lexicon_tags_2 },
    { 0x2e71acfe, SUC_lexicon_tags_0 },
    { 0xe8b1acfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4031ad0f, SUC_lexicon_tags_12 },
    { 0x4a71ad10, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b1ad14, SUC_lexicon_tags_7 },
    { 0x3d1ad15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd711ad17, SUC_lexicon_tags_25 },
    { 0x8631ad17, SUC_lexicon_tags_63 },
    { 0xe9d1ad19, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a91ad1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb031ad21, SUC_lexicon_tags_48 },
    { 0x8711ad21, SUC_lexicon_tags_37 },
    { 0x1cb1ad21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2cb1ad25, SUC_lexicon_tags_23 },
    { 0xd7f1ad26, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5611ad28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b51ad2c, SUC_lexicon_tags_12 },
    { 0xc5d1ad2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b51ad32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf1ad37, SUC_lexicon_tags_37 },
    { 0xf1ad38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6251ad3e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df1ad41, SUC_lexicon_tags_29 },
    { 0xc8b1ad42, SUC_lexicon_tags_48 },
    { 0x63d1ad42, SUC_lexicon_tags_37 },
    { 0xd9f1ad44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa451ad49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6331ad50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f1ad53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1ad57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b1ad5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1731ad5d, SUC_lexicon_tags_7 },
    { 0xc8f1ad5e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x28d1ad60, SUC_lexicon_tags_38 },
    { 0x6d91ad61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a11ad64, SUC_lexicon_tags_17 },
    { 0x5d51ad65, SUC_lexicon_tags_17 },
    { 0x75d1ad66, SUC_lexicon_tags_7 },
    { 0xc411ad67, SUC_lexicon_tags_9 },
    { 0xb831ad64, SUC_lexicon_tags_40 },
    { 0x3fb1ad64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2411ad6c, SUC_lexicon_tags_37 },
    { 0x6f31ad6d, SUC_lexicon_tags_37 },
    { 0x7891ad6e, SUC_lexicon_tags_38 },
    { 0x93f1ad6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf31ad7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6431ad7c, SUC_lexicon_tags_14 },
    { 0x60f1ad7d, SUC_lexicon_tags_43 },
    { 0x6231ad7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38d1ad80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1ad83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa571ad86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9771ad88, SUC_lexicon_tags_25 },
    { 0x77b1ad89, SUC_lexicon_tags_7 },
    { 0xc431ad88, SUC_lexicon_tags_65 },
    { 0xc891ad8b, SUC_lexicon_tags_7 },
    { 0x29b1ad8c, SUC_lexicon_tags_40 },
    { 0xcc71ad8c, SUC_lexicon_tags_15 },
    { 0xe031ad88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e31ad91, SUC_lexicon_tags_25 },
    { 0x8b71ad91, SUC_lexicon_tags_25 },
    { 0x9f71ad93, SUC_lexicon_tags_37 },
    { 0xa2b1ad94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c51ad99, SUC_lexicon_tags_0 },
    { 0xa0d1ad99, SUC_lexicon_tags_11 },
    { 0xe731ad9b, SUC_lexicon_tags_30 },
    { 0xb931ad9c, SUC_lexicon_tags_32 },
    { 0xc351ad9b, SUC_lexicon_tags_14 },
    { 0xf71ad9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdbf1ada0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52d1ada2, SUC_lexicon_tags_14 },
    { 0x8c31ada3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f1ada7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf1adaa, SUC_lexicon_tags_3 },
    { 0xcb91adab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f71adaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e51adb3, SUC_lexicon_tags_37 },
    { 0x8091adb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd1adbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a31adc6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9b1adc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4271adcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5051adcf, SUC_lexicon_tags_42 },
    { 0xef51add0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe591add4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b1add8, SUC_lexicon_tags_12 },
    { 0x4331add9, SUC_lexicon_tags_23 },
    { 0x5a71adda, SUC_lexicon_tags_15 },
    { 0xb091addb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bb1addd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd191addf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1971ade1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2071ade3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3551ade6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x55d1ade8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7571adea, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc131adec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb031adef, SUC_lexicon_tags_11 },
    { 0xfb1adf0, SUC_lexicon_tags_25 },
    { 0x6b91adf1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2551adf4, SUC_lexicon_tags_15 },
    { 0xa3b1adf5, SUC_lexicon_tags_65 },
    { 0xf331adf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d71adf8, SUC_lexicon_tags_28 },
    { 0x2431adf8, SUC_lexicon_tags_57 },
    { 0xed1adfa, SUC_lexicon_tags_29 },
    { 0x7231adfa, SUC_lexicon_tags_11 },
    { 0x8b11adfc, SUC_lexicon_tags_105 },
    { 0xaa51adfc, SUC_lexicon_tags_17 },
    { 0x1cd1adf9, SUC_lexicon_tags_14 },
    { 0x6e31adfc, SUC_lexicon_tags_17 },
    { 0xee91adf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8651ae02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1051ae04, SUC_lexicon_tags_11 },
    { 0x4b71ae05, SUC_lexicon_tags_37 },
    { 0x8771ae04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af1ae09, SUC_lexicon_tags_11 },
    { 0x1191ae09, SUC_lexicon_tags_45 },
    { 0xd791ae0b, SUC_lexicon_tags_47 },
    { 0x5031ae0c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa431ae10, SUC_lexicon_tags_81 },
    { 0x1f91ae11, SUC_lexicon_tags_62 },
    { 0xdcd1ae10, SUC_lexicon_tags_2 },
    { 0xffd1ae12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ef1ae15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e51ae18, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2a31ae1a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c31ae1f, SUC_lexicon_tags_5 },
    { 0xa7f1ae20, SUC_lexicon_tags_12 },
    { 0xb5b1ae1f, SUC_lexicon_tags_12 },
    { 0x5f11ae21, SUC_lexicon_tags_2 },
    { 0xcc91ae23, SUC_lexicon_tags_12 },
    { 0x51ae24, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f1ae2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd131ae32, SUC_lexicon_tags_25 },
    { 0xff51ae33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8711ae3e, SUC_lexicon_tags_2 },
    { 0x4e31ae3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf1ae42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c71ae45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c51ae4a, SUC_lexicon_tags_4 },
    { 0x2c51ae4b, SUC_lexicon_tags_25 },
    { 0xfeb1ae4b, SUC_lexicon_tags_15 },
    { 0x2c91ae4a, SUC_lexicon_tags_63 },
    { 0xea51ae4e, SUC_lexicon_tags_25 },
    { 0xd9b1ae4e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d1ae55, SUC_lexicon_tags_15 },
    { 0xf031ae56, SUC_lexicon_tags_37 },
    { 0xdfd1ae57, SUC_lexicon_tags_29 },
    { 0xee51ae55, SUC_lexicon_tags_25 },
    { 0x35f1ae57, SUC_lexicon_tags_37 },
    { 0x5f31ae5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa011ae5d, SUC_lexicon_tags_37 },
    { 0x3051ae5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b1ae60, SUC_lexicon_tags_38 },
    { 0x7631ae60, SUC_lexicon_tags_17 },
    { 0x1f1ae62, SUC_lexicon_tags_13 },
    { 0x56d1ae62, SUC_lexicon_tags_37 },
    { 0xd511ae64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4211ae67, SUC_lexicon_tags_282 },
    { 0xa591ae68, SUC_lexicon_tags_16 },
    { 0xeaf1ae67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1ae6c, SUC_lexicon_tags_12 },
    { 0xe4f1ae6c, SUC_lexicon_tags_1 },
    { 0x71f1ae6c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b1ae72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ad1ae74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5611ae79, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e31ae7c, SUC_lexicon_tags_7 },
    { 0x3f51ae7d, SUC_lexicon_tags_6 },
    { 0x2011ae7d, SUC_lexicon_tags_20 },
    { 0x6c31ae7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b1ae84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa071ae88, SUC_lexicon_tags_21 },
    { 0x11b1ae89, SUC_lexicon_tags_47 },
    { 0x4b51ae89, SUC_lexicon_tags_0 },
    { 0x3cf1ae88, SUC_lexicon_tags_39 },
    { 0x8b1ae8c, SUC_lexicon_tags_14 },
    { 0xb571ae8b, SUC_lexicon_tags_45 },
    { 0xbd11ae8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec91ae93, SUC_lexicon_tags_12 },
    { 0x4c71ae94, SUC_lexicon_tags_37 },
    { 0xe751ae94, SUC_lexicon_tags_19 },
    { 0x731ae94, SUC_lexicon_tags_2 },
    { 0xc4f1ae94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b31ae9a, SUC_lexicon_tags_2 },
    { 0xbe31ae9b, SUC_lexicon_tags_4 },
    { 0xbe71ae9c, SUC_lexicon_tags_15 },
    { 0xcb31ae9d, SUC_lexicon_tags_65 },
    { 0xbbd1ae9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4371aea2, SUC_lexicon_tags_37 },
    { 0x7a31aea3, SUC_lexicon_tags_37 },
    { 0x6ed1aea4, SUC_lexicon_tags_25 },
    { 0x3c71aea5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72f1aea7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f1aeb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95d1aeb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f1aeb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf091aeb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1311aebc, SUC_lexicon_tags_19 },
    { 0xead1aebc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb851aebf, SUC_lexicon_tags_17 },
    { 0x5331aec0, SUC_lexicon_tags_2 },
    { 0x6571aec1, SUC_lexicon_tags_15 },
    { 0xe451aec2, SUC_lexicon_tags_16 },
    { 0x9551aec3, SUC_lexicon_tags_11 },
    { 0x3c31aec4, SUC_lexicon_tags_10 },
    { 0x89f1aec4, SUC_lexicon_tags_61 },
    { 0x3e71aec6, SUC_lexicon_tags_34 },
    { 0x33d1aec7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb1aecd, SUC_lexicon_tags_48 },
    { 0x8cb1aece, SUC_lexicon_tags_1 },
    { 0xbcb1aece, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef1aed9, SUC_lexicon_tags_24 },
    { 0x5ab1aeda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2651aedc, SUC_lexicon_tags_25 },
    { 0x11f1aedd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2331aee1, SUC_lexicon_tags_91 },
    { 0xdef1aee2, SUC_lexicon_tags_19 },
    { 0x2231aee1, SUC_lexicon_tags_37 },
    { 0x7171aee4, SUC_lexicon_tags_13 },
    { 0xe471aee5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9131aee7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb1aeea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9851aeed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7af1aeef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa791aef1, SUC_lexicon_tags_45 },
    { 0xcb71aef2, SUC_lexicon_tags_16 },
    { 0xa611aef2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a51aef5, SUC_lexicon_tags_13 },
    { 0x4891aef6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7891aef8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd3b1aefa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db1af01, SUC_lexicon_tags_1 },
    { 0xc9d1af02, SUC_lexicon_tags_39 },
    { 0x9f51af03, SUC_lexicon_tags_2 },
    { 0x4e31af03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2031af09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f1af0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c71af18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7911af23, SUC_lexicon_tags_25 },
    { 0x9751af24, SUC_lexicon_tags_14 },
    { 0x18b1af25, SUC_lexicon_tags_17 },
    { 0xe7f1af25, SUC_lexicon_tags_80 },
    { 0x7431af27, SUC_lexicon_tags_82 },
    { 0x71af25, SUC_lexicon_tags_92 },
    { 0x85d1af29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63d1af2b, SUC_lexicon_tags_226 },
    { 0xb311af2c, SUC_lexicon_tags_87 },
    { 0x4551af2d, SUC_lexicon_tags_17 },
    { 0x3291af2c, SUC_lexicon_tags_70 },
    { 0x3411af2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d1af3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c31af41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x61d1af43, SUC_lexicon_tags_11 },
    { 0x1271af44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9671af4c, SUC_lexicon_tags_38 },
    { 0xcdd1af4d, SUC_lexicon_tags_15 },
    { 0xe631af4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f1af55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d11af58, SUC_lexicon_tags_21 },
    { 0xc2b1af59, SUC_lexicon_tags_38 },
    { 0x9df1af5a, SUC_lexicon_tags_55 },
    { 0x1a31af5b, SUC_lexicon_tags_439 },
    { 0xfcd1af5a, SUC_lexicon_tags_47 },
    { 0x2731af5b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1e71af5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1551af62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7111af68, SUC_lexicon_tags_60 },
    { 0x1051af69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xca31af6b, SUC_lexicon_tags_37 },
    { 0xaa31af6c, SUC_lexicon_tags_77 },
    { 0xe6f1af6d, SUC_lexicon_tags_55 },
    { 0x6551af6d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5791af76, SUC_lexicon_tags_6 },
    { 0x3ad1af77, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7951af79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f1af7c, SUC_lexicon_tags_28 },
    { 0x9ff1af7c, SUC_lexicon_tags_60 },
    { 0xc331af7e, SUC_lexicon_tags_31 },
    { 0x2b11af7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa791af83, SUC_lexicon_tags_34 },
    { 0x7ef1af84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8f1af86, SUC_lexicon_tags_33 },
    { 0xf551af87, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4431af89, SUC_lexicon_tags_107 },
    { 0x46f1af8a, SUC_lexicon_tags_11 },
    { 0x4651af89, SUC_lexicon_tags_12 },
    { 0x1511af8c, SUC_lexicon_tags_45 },
    { 0x8cb1af8d, SUC_lexicon_tags_0 },
    { 0x16d1af8e, SUC_lexicon_tags_7 },
    { 0xe771af8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9671af91, SUC_lexicon_tags_7 },
    { 0x7351af92, SUC_lexicon_tags_17 },
    { 0xdc91af93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1071af96, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4551af9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc391af9f, SUC_lexicon_tags_52 },
    { 0xf4b1afa0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd71afa2, SUC_lexicon_tags_8 },
    { 0x8271afa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf871afa6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d71afa9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc791afad, SUC_lexicon_tags_27 },
    { 0x3431afad, SUC_lexicon_tags_1 },
    { 0x8c71afae, SUC_lexicon_tags_38 },
    { 0xef31afaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc271afb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f1afb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6311afb9, SUC_lexicon_tags_15 },
    { 0xb1d1afba, SUC_lexicon_tags_65 },
    { 0xd291afba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1afbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b1afc3, SUC_lexicon_tags_34 },
    { 0x16b1afc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf571afca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7bf1afcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed71afcf, SUC_lexicon_tags_37 },
    { 0x28f1afd0, SUC_lexicon_tags_31 },
    { 0x3651afd1, SUC_lexicon_tags_19 },
    { 0x2f91afd0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x90f1afd4, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xba51afd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7d11afd8, SUC_lexicon_tags_38 },
    { 0x951afd9, SUC_lexicon_tags_11 },
    { 0xfbd1afd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae11afde, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x75d1afe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1afe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x691afe6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ad1afe8, SUC_lexicon_tags_29 },
    { 0xf0b1afe9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d1aff1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d1aff4, SUC_lexicon_tags_11 },
    { 0x1b51aff4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe111affa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd231b00a, SUC_lexicon_tags_1 },
    { 0xad71b00b, SUC_lexicon_tags_64 },
    { 0x70d1b00c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd1b012, SUC_lexicon_tags_33 },
    { 0xa451b013, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2e91b015, SUC_lexicon_tags_2 },
    { 0x9551b015, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8411b01c, SUC_lexicon_tags_65 },
    { 0xc051b01d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2671b021, SUC_lexicon_tags_2 },
    { 0x5f51b022, SUC_lexicon_tags_9 },
    { 0xa0b1b023, SUC_lexicon_tags_275 },
    { 0x8c11b022, SUC_lexicon_tags_28 },
    { 0x7f51b025, SUC_lexicon_tags_49 },
    { 0xf7d1b024, SUC_lexicon_tags_27 },
    { 0x4931b027, SUC_lexicon_tags_28 },
    { 0x8531b028, SUC_lexicon_tags_28 },
    { 0xbd11b028, SUC_lexicon_tags_0 },
    { 0x12f1b028, SUC_lexicon_tags_31 },
    { 0x9111b02b, SUC_lexicon_tags_11 },
    { 0x4271b02b, SUC_lexicon_tags_27 },
    { 0x9331b023, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb1b033, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad1b036, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e71b03c, SUC_lexicon_tags_47 },
    { 0xf811b03d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2311b042, SUC_lexicon_tags_39 },
    { 0xfcd1b042, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d91b045, SUC_lexicon_tags_30 },
    { 0x13f1b046, SUC_lexicon_tags_29 },
    { 0xfd31b047, SUC_lexicon_tags_7 },
    { 0x5131b046, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe711b04c, SUC_lexicon_tags_56 },
    { 0xf91b04c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f1b050, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe031b054, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36d1b056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d1b060, SUC_lexicon_tags_129 },
    { 0x5611b061, SUC_lexicon_tags_43 },
    { 0x8ff1b062, SUC_lexicon_tags_57 },
    { 0x77b1b060, SUC_lexicon_tags_107 },
    { 0x36b1b064, SUC_lexicon_tags_14 },
    { 0xc711b063, SUC_lexicon_tags_1 },
    { 0xd0b1b066, SUC_lexicon_tags_1 },
    { 0x6931b066, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb1b069, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa731b079, SUC_lexicon_tags_31 },
    { 0x3cf1b07a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa351b07c, SUC_lexicon_tags_17 },
    { 0x51f1b07d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d1b080, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ab1b082, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe691b085, SUC_lexicon_tags_60 },
    { 0x9d51b086, SUC_lexicon_tags_37 },
    { 0x3991b086, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd1b08a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf1b08c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcc11b08e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b51b093, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd91b095, SUC_lexicon_tags_127 },
    { 0x8c71b095, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce51b098, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb891b0a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9711b0a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf491b0a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc71b0a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa511b0b0, SUC_lexicon_tags_14 },
    { 0x9231b0b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd1b0b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af1b0bb, SUC_lexicon_tags_12 },
    { 0xd3b1b0bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8d1b0be, SUC_lexicon_tags_23 },
    { 0x331b0be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcdf1b0c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb1b0c6, SUC_lexicon_tags_43 },
    { 0xe631b0c7, SUC_lexicon_tags_37 },
    { 0xa1d1b0c8, SUC_lexicon_tags_29 },
    { 0x3771b0c9, SUC_lexicon_tags_13 },
    { 0xb071b0c7, SUC_lexicon_tags_2 },
    { 0x3a71b0cb, SUC_lexicon_tags_16 },
    { 0xe231b0cb, SUC_lexicon_tags_28 },
    { 0x15f1b0cd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b1b0dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9751b0e3, SUC_lexicon_tags_19 },
    { 0xae91b0e4, SUC_lexicon_tags_17 },
    { 0xca31b0e5, SUC_lexicon_tags_12 },
    { 0xd71b0e6, SUC_lexicon_tags_7 },
    { 0x2dd1b0e7, SUC_lexicon_tags_12 },
    { 0xb071b0e7, SUC_lexicon_tags_23 },
    { 0xad31b0e4, SUC_lexicon_tags_32 },
    { 0x2511b0ea, SUC_lexicon_tags_63 },
    { 0x591b0ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb11b0ee, SUC_lexicon_tags_27 },
    { 0xefd1b0ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad1b0f8, SUC_lexicon_tags_25 },
    { 0x7cb1b0f9, SUC_lexicon_tags_12 },
    { 0x5dd1b0f9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7731b0fc, SUC_lexicon_tags_1 },
    { 0x5451b0fd, SUC_lexicon_tags_1 },
    { 0xe971b0fc, SUC_lexicon_tags_17 },
    { 0x5e31b0ff, SUC_lexicon_tags_27 },
    { 0x16f1b0fe, SUC_lexicon_tags_45 },
    { 0xe311b101, SUC_lexicon_tags_2 },
    { 0xead1b101, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x22d1b104, SUC_lexicon_tags_12 },
    { 0x1171b105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab51b10a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f1b10d, SUC_lexicon_tags_6 },
    { 0x5711b10e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe211b113, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc091b115, SUC_lexicon_tags_1 },
    { 0x74f1b115, SUC_lexicon_tags_34 },
    { 0x2891b115, SUC_lexicon_tags_79 },
    { 0x9271b118, SUC_lexicon_tags_82 },
    { 0x5ab1b119, SUC_lexicon_tags_17 },
    { 0x7d51b11a, SUC_lexicon_tags_33 },
    { 0xb0b1b119, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd591b11d, SUC_lexicon_tags_27 },
    { 0xbb51b11d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f91b121, SUC_lexicon_tags_1042 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf071b125, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5051b129, SUC_lexicon_tags_63 },
    { 0xdc71b12a, SUC_lexicon_tags_25 },
    { 0x18f1b12b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c31b130, SUC_lexicon_tags_14 },
    { 0xba91b130, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ed1b133, SUC_lexicon_tags_1 },
    { 0x32b1b134, SUC_lexicon_tags_49 },
    { 0xb4f1b135, SUC_lexicon_tags_25 },
    { 0xe51b133, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e71b138, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8591b13b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d1b141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2d1b143, SUC_lexicon_tags_37 },
    { 0xab31b143, SUC_lexicon_tags_23 },
    { 0x311b145, SUC_lexicon_tags_29 },
    { 0xf311b144, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b1b14a, SUC_lexicon_tags_19 },
    { 0x3d91b14b, SUC_lexicon_tags_30 },
    { 0x39f1b14b, SUC_lexicon_tags_48 },
    { 0x5611b14b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2071b150, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5f11b152, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xca51b154, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8e91b156, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1b15a, SUC_lexicon_tags_44 },
    { 0xa191b15b, SUC_lexicon_tags_25 },
    { 0xdc51b15b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f51b160, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf791b162, SUC_lexicon_tags_37 },
    { 0xc791b162, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451b169, SUC_lexicon_tags_43 },
    { 0x5031b16a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d51b172, SUC_lexicon_tags_27 },
    { 0xd891b173, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a11b185, SUC_lexicon_tags_26 },
    { 0x3dd1b186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b1b18b, SUC_lexicon_tags_15 },
    { 0x21b1b18c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x99f1b18e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb6b1b190, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc71b195, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1031b1a0, SUC_lexicon_tags_1 },
    { 0xc051b1a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a71b1a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e11b1a8, SUC_lexicon_tags_37 },
    { 0x7811b1a8, SUC_lexicon_tags_25 },
    { 0xe171b1aa, SUC_lexicon_tags_331 },
    { 0, NULL },
    { 0xb5f1b1ac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b1b1af, SUC_lexicon_tags_27 },
    { 0xb371b1b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3671b1b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9d1b1b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9231b1b6, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x9371b1b8, SUC_lexicon_tags_17 },
    { 0x271b1b9, SUC_lexicon_tags_1 },
    { 0xaef1b1ba, SUC_lexicon_tags_12 },
    { 0x2bb1b1ba, SUC_lexicon_tags_79 },
    { 0xf791b1bc, SUC_lexicon_tags_17 },
    { 0x4751b1bd, SUC_lexicon_tags_11 },
    { 0xf6d1b1bc, SUC_lexicon_tags_29 },
    { 0xb8d1b1bf, SUC_lexicon_tags_49 },
    { 0xf751b1bd, SUC_lexicon_tags_0 },
    { 0xa451b1c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4431b1c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7711b1c7, SUC_lexicon_tags_43 },
    { 0x6c91b1c8, SUC_lexicon_tags_1 },
    { 0x6271b1c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6251b1cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9791b1ce, SUC_lexicon_tags_47 },
    { 0x9891b1cf, SUC_lexicon_tags_11 },
    { 0x8551b1cf, SUC_lexicon_tags_15 },
    { 0x2711b1d1, SUC_lexicon_tags_37 },
    { 0x6551b1ce, SUC_lexicon_tags_37 },
    { 0x44b1b1d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9431b1d5, SUC_lexicon_tags_72 },
    { 0x1511b1d5, SUC_lexicon_tags_26 },
    { 0x6ab1b1d7, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0x5f91b1d9, SUC_lexicon_tags_15 },
    { 0xd0b1b1da, SUC_lexicon_tags_11 },
    { 0x22d1b1da, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2cf1b1dd, SUC_lexicon_tags_27 },
    { 0x68b1b1dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef1b1e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f1b1e8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d11b1eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f1b1ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d1b1f3, SUC_lexicon_tags_49 },
    { 0xd5b1b1f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c31b201, SUC_lexicon_tags_1 },
    { 0xe871b201, SUC_lexicon_tags_27 },
    { 0x9891b203, SUC_lexicon_tags_14 },
    { 0x2551b204, SUC_lexicon_tags_25 },
    { 0x6171b205, SUC_lexicon_tags_0 },
    { 0xe431b206, SUC_lexicon_tags_15 },
    { 0x6671b205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93d1b209, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe11b20b, SUC_lexicon_tags_47 },
    { 0x7971b20c, SUC_lexicon_tags_37 },
    { 0x9c51b20b, SUC_lexicon_tags_7 },
    { 0xa9b1b20e, SUC_lexicon_tags_25 },
    { 0xef11b20f, SUC_lexicon_tags_2 },
    { 0xf131b210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf1b213, SUC_lexicon_tags_25 },
    { 0xd5d1b214, SUC_lexicon_tags_84 },
    { 0x6891b215, SUC_lexicon_tags_6 },
    { 0xef1b214, SUC_lexicon_tags_4 },
    { 0x2151b217, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe071b219, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b1b21c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7811b221, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd1b224, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xec91b226, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b1b229, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331b22d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba91b232, SUC_lexicon_tags_10 },
    { 0xbed1b232, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc471b236, SUC_lexicon_tags_11 },
    { 0x6251b237, SUC_lexicon_tags_49 },
    { 0x3eb1b236, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce11b23e, SUC_lexicon_tags_12 },
    { 0x5a31b23e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8ef1b241, SUC_lexicon_tags_11 },
    { 0x3e11b242, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd1b249, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b1b24d, SUC_lexicon_tags_45 },
    { 0x6fb1b24d, SUC_lexicon_tags_65 },
    { 0x2e11b24f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x88b1b251, SUC_lexicon_tags_38 },
    { 0xda31b251, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2ed1b254, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8211b257, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb1b25b, SUC_lexicon_tags_28 },
    { 0xdeb1b25c, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce91b262, SUC_lexicon_tags_15 },
    { 0xd2f1b263, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9711b267, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd611b269, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f1b26d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c11b273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d1b276, SUC_lexicon_tags_15 },
    { 0xd691b276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe011b27d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7cf1b27f, SUC_lexicon_tags_11 },
    { 0xeeb1b280, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3191b287, SUC_lexicon_tags_17 },
    { 0x66d1b288, SUC_lexicon_tags_23 },
    { 0x60f1b287, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5131b28b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd1b28e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d1b293, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdeb1b295, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd1b29a, SUC_lexicon_tags_14 },
    { 0xb11b29a, SUC_lexicon_tags_11 },
    { 0xbf71b29c, SUC_lexicon_tags_1 },
    { 0xd831b29b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x571b29f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9c71b2a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1251b2a5, SUC_lexicon_tags_65 },
    { 0x1111b2a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa851b2a8, SUC_lexicon_tags_60 },
    { 0xeb11b2a9, SUC_lexicon_tags_12 },
    { 0x17d1b2aa, SUC_lexicon_tags_29 },
    { 0x9c11b2a9, SUC_lexicon_tags_3 },
    { 0xc451b2ac, SUC_lexicon_tags_17 },
    { 0xe9b1b2ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3511b2af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca91b2b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f1b2b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9571b2b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1b2c1, SUC_lexicon_tags_6 },
    { 0xe731b2c1, SUC_lexicon_tags_19 },
    { 0xc611b2c3, SUC_lexicon_tags_23 },
    { 0x22f1b2c2, SUC_lexicon_tags_30 },
    { 0x8c31b2c5, SUC_lexicon_tags_17 },
    { 0x79f1b2c5, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xa271b2c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe611b2cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5271b2cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8f1b2d1, SUC_lexicon_tags_27 },
    { 0xe6f1b2d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d11b2d4, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c51b2db, SUC_lexicon_tags_14 },
    { 0xa291b2dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c51b2e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1991b2e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d1b2eb, SUC_lexicon_tags_37 },
    { 0x45d1b2ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b11b2ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3911b2f1, SUC_lexicon_tags_25 },
    { 0x7a71b2f2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdd51b2f4, SUC_lexicon_tags_45 },
    { 0x5271b2f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2a91b2f7, SUC_lexicon_tags_9 },
    { 0x5791b2f7, SUC_lexicon_tags_5 },
    { 0x1b91b2f8, SUC_lexicon_tags_30 },
    { 0x1f91b2fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd1b2fd, SUC_lexicon_tags_7 },
    { 0x8391b2fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11b302, SUC_lexicon_tags_66 },
    { 0x1851b303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7531b308, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3611b30a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab11b30c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891b313, SUC_lexicon_tags_63 },
    { 0x7d91b314, SUC_lexicon_tags_25 },
    { 0x16f1b314, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb191b317, SUC_lexicon_tags_43 },
    { 0x3b51b318, SUC_lexicon_tags_12 },
    { 0x4731b319, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6f1b31b, SUC_lexicon_tags_11 },
    { 0xe1b1b31c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd1b321, SUC_lexicon_tags_37 },
    { 0xb111b322, SUC_lexicon_tags_60 },
    { 0xa8d1b322, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c91b325, SUC_lexicon_tags_27 },
    { 0x1151b326, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9611b328, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4811b32a, SUC_lexicon_tags_46 },
    { 0xb8f1b32a, SUC_lexicon_tags_48 },
    { 0xa3d1b32c, SUC_lexicon_tags_25 },
    { 0xc651b32d, SUC_lexicon_tags_0 },
    { 0x9851b32e, SUC_lexicon_tags_15 },
    { 0x62b1b32f, SUC_lexicon_tags_12 },
    { 0x7cf1b330, SUC_lexicon_tags_7 },
    { 0x3571b330, SUC_lexicon_tags_2 },
    { 0x92b1b331, SUC_lexicon_tags_15 },
    { 0xd491b32d, SUC_lexicon_tags_37 },
    { 0xe11b334, SUC_lexicon_tags_66 },
    { 0x1611b335, SUC_lexicon_tags_10 },
    { 0x14d1b336, SUC_lexicon_tags_0 },
    { 0x63f1b336, SUC_lexicon_tags_40 },
    { 0xba71b337, SUC_lexicon_tags_25 },
    { 0xc0f1b339, SUC_lexicon_tags_7 },
    { 0x18b1b333, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331b33e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1b34a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5851b34c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9171b34f, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c11b354, SUC_lexicon_tags_44 },
    { 0x7071b355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6291b358, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa111b35d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd3b1b35f, SUC_lexicon_tags_7 },
    { 0xafd1b35f, SUC_lexicon_tags_0 },
    { 0xb1b35f, SUC_lexicon_tags_107 },
    { 0x72d1b362, SUC_lexicon_tags_12 },
    { 0x8611b363, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd171b36b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3971b36e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6d11b370, SUC_lexicon_tags_11 },
    { 0x4dd1b370, SUC_lexicon_tags_1 },
    { 0xa91b372, SUC_lexicon_tags_43 },
    { 0x2271b372, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd511b376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a71b37a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6e31b37c, SUC_lexicon_tags_1 },
    { 0xab71b37d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f1b380, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e71b384, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb1b388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1b38b, SUC_lexicon_tags_49 },
    { 0xc2b1b38b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc391b38f, SUC_lexicon_tags_66 },
    { 0xc7d1b38f, SUC_lexicon_tags_19 },
    { 0xcc91b38f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1691b395, SUC_lexicon_tags_11 },
    { 0xda11b396, SUC_lexicon_tags_15 },
    { 0x9711b397, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98b1b399, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e71b39c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa271b39e, SUC_lexicon_tags_11 },
    { 0x3191b39f, SUC_lexicon_tags_25 },
    { 0x8691b3a0, SUC_lexicon_tags_37 },
    { 0x84f1b3a1, SUC_lexicon_tags_66 },
    { 0x7631b3a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd1b3a5, SUC_lexicon_tags_37 },
    { 0x1a51b3a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98f1b3a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d1b3ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5b11b3ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51b3b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd91b3b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a31b3bd, SUC_lexicon_tags_35 },
    { 0x3b11b3be, SUC_lexicon_tags_7 },
    { 0xc9b1b3bf, SUC_lexicon_tags_11 },
    { 0x3cb1b3c0, SUC_lexicon_tags_12 },
    { 0x8091b3c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x31b3c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd1b3ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b31b3d8, SUC_lexicon_tags_30 },
    { 0x4131b3d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b3dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xef11b3de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d1b3e3, SUC_lexicon_tags_65 },
    { 0x93d1b3e4, SUC_lexicon_tags_7 },
    { 0xa531b3e5, SUC_lexicon_tags_1 },
    { 0xdd31b3e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe271b3ed, SUC_lexicon_tags_0 },
    { 0x85b1b3ee, SUC_lexicon_tags_30 },
    { 0x16b1b3ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd11b3f3, SUC_lexicon_tags_14 },
    { 0xee91b3f4, SUC_lexicon_tags_25 },
    { 0x3af1b3f3, SUC_lexicon_tags_10 },
    { 0xaf51b3f3, SUC_lexicon_tags_11 },
    { 0x8291b3f7, SUC_lexicon_tags_21 },
    { 0xd231b3f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5791b3ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc111b403, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d1b40a, SUC_lexicon_tags_1 },
    { 0x8d1b40a, SUC_lexicon_tags_7 },
    { 0xe9b1b40b, SUC_lexicon_tags_1 },
    { 0x4371b40a, SUC_lexicon_tags_17 },
    { 0x5b31b40c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b51b414, SUC_lexicon_tags_16 },
    { 0xafb1b414, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd51b417, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8111b41b, SUC_lexicon_tags_1 },
    { 0x67d1b41c, SUC_lexicon_tags_15 },
    { 0x56d1b41c, SUC_lexicon_tags_4 },
    { 0x9d11b41d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb1b422, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf31b426, SUC_lexicon_tags_25 },
    { 0x7871b427, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4d11b429, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3291b42e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f71b431, SUC_lexicon_tags_7 },
    { 0xe31b432, SUC_lexicon_tags_104 },
    { 0xa3d1b433, SUC_lexicon_tags_25 },
    { 0xbc31b433, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9b1b436, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11b439, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b31b447, SUC_lexicon_tags_21 },
    { 0x8591b448, SUC_lexicon_tags_12 },
    { 0xc7b1b449, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d1b44e, SUC_lexicon_tags_37 },
    { 0x9651b44f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3fd1b451, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb031b454, SUC_lexicon_tags_0 },
    { 0xef91b455, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf91b458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae91b45b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34f1b45d, SUC_lexicon_tags_10 },
    { 0x9251b45d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4971b463, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe031b46a, SUC_lexicon_tags_12 },
    { 0x3f71b46a, SUC_lexicon_tags_46 },
    { 0x65b1b46c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3071b46e, SUC_lexicon_tags_19 },
    { 0xd91b46f, SUC_lexicon_tags_7 },
    { 0x2171b46e, SUC_lexicon_tags_25 },
    { 0x1411b471, SUC_lexicon_tags_4 },
    { 0x5771b472, SUC_lexicon_tags_18 },
    { 0x6cb1b473, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd1b476, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9451b478, SUC_lexicon_tags_54 },
    { 0x6251b479, SUC_lexicon_tags_37 },
    { 0x1a51b478, SUC_lexicon_tags_20 },
    { 0x69f1b47b, SUC_lexicon_tags_26 },
    { 0x1d71b47b, SUC_lexicon_tags_32 },
    { 0x791b47d, SUC_lexicon_tags_66 },
    { 0xf991b47d, SUC_lexicon_tags_11 },
    { 0xaf71b47f, SUC_lexicon_tags_11 },
    { 0x91f1b47d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b91b482, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf691b485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74b1b487, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf51b48c, SUC_lexicon_tags_14 },
    { 0xff31b48c, SUC_lexicon_tags_17 },
    { 0x2bd1b48d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f1b491, SUC_lexicon_tags_7 },
    { 0x3ed1b491, SUC_lexicon_tags_8 },
    { 0x971b493, SUC_lexicon_tags_29 },
    { 0x1471b494, SUC_lexicon_tags_60 },
    { 0x2691b494, SUC_lexicon_tags_7 },
    { 0xb131b496, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7251b498, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x8531b49a, SUC_lexicon_tags_96 },
    { 0x7471b49a, SUC_lexicon_tags_2 },
    { 0x2051b49c, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe71b4a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6451b4a2, SUC_lexicon_tags_1 },
    { 0xd151b4a2, SUC_lexicon_tags_32 },
    { 0xa131b4a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7591b4a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8f1b4a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d1b4ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5951b4ae, SUC_lexicon_tags_45 },
    { 0xc111b4af, SUC_lexicon_tags_15 },
    { 0x5ff1b4b0, SUC_lexicon_tags_17 },
    { 0xbeb1b4b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1b4bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3631b4be, SUC_lexicon_tags_47 },
    { 0x551b4bf, SUC_lexicon_tags_37 },
    { 0xa031b4c0, SUC_lexicon_tags_7 },
    { 0x11d1b4c1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x92d1b4c3, SUC_lexicon_tags_79 },
    { 0xf4d1b4c4, SUC_lexicon_tags_63 },
    { 0x4b51b4c4, SUC_lexicon_tags_25 },
    { 0xf2d1b4c6, SUC_lexicon_tags_25 },
    { 0x5471b4c5, SUC_lexicon_tags_47 },
    { 0xc5f1b4c8, SUC_lexicon_tags_8 },
    { 0x8ad1b4c9, SUC_lexicon_tags_3 },
    { 0x2451b4c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa551b4cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda71b4d1, SUC_lexicon_tags_28 },
    { 0xc4d1b4d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91b4d7, SUC_lexicon_tags_21 },
    { 0xfc31b4d8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb991b4dd, SUC_lexicon_tags_25 },
    { 0x7011b4de, SUC_lexicon_tags_37 },
    { 0x90d1b4df, SUC_lexicon_tags_47 },
    { 0xc4f1b4e0, SUC_lexicon_tags_25 },
    { 0x10f1b4e1, SUC_lexicon_tags_17 },
    { 0x411b4e2, SUC_lexicon_tags_17 },
    { 0xe2b1b4e3, SUC_lexicon_tags_11 },
    { 0x71f1b4e3, SUC_lexicon_tags_2 },
    { 0xa1f1b4e5, SUC_lexicon_tags_6 },
    { 0x7171b4e6, SUC_lexicon_tags_1 },
    { 0xe9b1b4e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db1b4ea, SUC_lexicon_tags_15 },
    { 0xef11b4eb, SUC_lexicon_tags_15 },
    { 0xdcd1b4ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1391b4f3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xda71b4f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7eb1b4f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d1b4fc, SUC_lexicon_tags_47 },
    { 0xbe71b4fd, SUC_lexicon_tags_12 },
    { 0x2631b4fe, SUC_lexicon_tags_37 },
    { 0x6351b4fd, SUC_lexicon_tags_7 },
    { 0xd731b4ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11d1b502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52b1b504, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4cd1b506, SUC_lexicon_tags_7 },
    { 0xb2d1b506, SUC_lexicon_tags_47 },
    { 0x7c31b506, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x59f1b50a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e51b50d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe251b511, SUC_lexicon_tags_29 },
    { 0x82f1b512, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc5f1b514, SUC_lexicon_tags_77 },
    { 0x83f1b515, SUC_lexicon_tags_0 },
    { 0x1951b516, SUC_lexicon_tags_32 },
    { 0x35d1b516, SUC_lexicon_tags_6 },
    { 0xb5b1b518, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7bf1b51a, SUC_lexicon_tags_34 },
    { 0xd311b51a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f1b51e, SUC_lexicon_tags_60 },
    { 0x3191b51f, SUC_lexicon_tags_3 },
    { 0xd5f1b51f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde91b522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf1b528, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8671b52a, SUC_lexicon_tags_17 },
    { 0xf0b1b52b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31b530, SUC_lexicon_tags_25 },
    { 0x9531b531, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30d1b533, SUC_lexicon_tags_6 },
    { 0x9e31b534, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3091b536, SUC_lexicon_tags_15 },
    { 0xe1f1b537, SUC_lexicon_tags_1 },
    { 0xbfb1b538, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa731b53d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43d1b53f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3871b541, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb1b546, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd71b54a, SUC_lexicon_tags_42 },
    { 0x6d71b54b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f1b550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb331b555, SUC_lexicon_tags_7 },
    { 0xb291b555, SUC_lexicon_tags_17 },
    { 0xc091b556, SUC_lexicon_tags_23 },
    { 0xde51b558, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc91b55a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc51b562, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf531b566, SUC_lexicon_tags_1 },
    { 0xe531b567, SUC_lexicon_tags_38 },
    { 0x151b568, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f31b56a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc731b570, SUC_lexicon_tags_40 },
    { 0x7e11b571, SUC_lexicon_tags_68 },
    { 0x8f71b572, SUC_lexicon_tags_65 },
    { 0xc2f1b571, SUC_lexicon_tags_23 },
    { 0xc491b574, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5291b577, SUC_lexicon_tags_27 },
    { 0x69f1b578, SUC_lexicon_tags_55 },
    { 0x6791b578, SUC_lexicon_tags_74 },
    { 0x13b1b578, SUC_lexicon_tags_0 },
    { 0x19b1b57b, SUC_lexicon_tags_11 },
    { 0x4ff1b579, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xe911b57e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf131b585, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf611b587, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4d1b589, SUC_lexicon_tags_29 },
    { 0xe271b58a, SUC_lexicon_tags_25 },
    { 0x6eb1b58b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4f91b58d, SUC_lexicon_tags_7 },
    { 0x9751b58e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d1b593, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1b59b, SUC_lexicon_tags_17 },
    { 0x9c11b59b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f1b59f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc3d1b5a1, SUC_lexicon_tags_12 },
    { 0x2f11b5a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bb1b5a4, SUC_lexicon_tags_15 },
    { 0xbe91b5a4, SUC_lexicon_tags_1 },
    { 0x5fb1b5a4, SUC_lexicon_tags_11 },
    { 0x7731b5a7, SUC_lexicon_tags_0 },
    { 0xc9f1b5a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x26d1b5aa, SUC_lexicon_tags_11 },
    { 0x91b5ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x951b5ae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xee51b5b0, SUC_lexicon_tags_15 },
    { 0x15b1b5b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d1b5b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb1b5be, SUC_lexicon_tags_21 },
    { 0xa5d1b5bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa791b5ca, SUC_lexicon_tags_19 },
    { 0x37d1b5cb, SUC_lexicon_tags_60 },
    { 0x8f11b5cc, SUC_lexicon_tags_62 },
    { 0x6811b5cb, SUC_lexicon_tags_37 },
    { 0xa1b1b5ca, SUC_lexicon_tags_6 },
    { 0xee71b5cf, SUC_lexicon_tags_45 },
    { 0xb8d1b5d0, SUC_lexicon_tags_34 },
    { 0xa91b5cf, SUC_lexicon_tags_29 },
    { 0xf231b5ce, SUC_lexicon_tags_37 },
    { 0x35b1b5cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe411b5da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f1b5e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8991b5e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4071b5e6, SUC_lexicon_tags_23 },
    { 0x2f11b5e7, SUC_lexicon_tags_16 },
    { 0x8ad1b5e8, SUC_lexicon_tags_37 },
    { 0xd071b5e8, SUC_lexicon_tags_23 },
    { 0x3931b5ea, SUC_lexicon_tags_43 },
    { 0x25b1b5eb, SUC_lexicon_tags_1 },
    { 0x2a11b5ec, SUC_lexicon_tags_12 },
    { 0xddf1b5e6, SUC_lexicon_tags_37 },
    { 0x93b1b5ee, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd551b5f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7491b5f7, SUC_lexicon_tags_49 },
    { 0xa51b5f8, SUC_lexicon_tags_0 },
    { 0xbfd1b5f9, SUC_lexicon_tags_6 },
    { 0xd251b5fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf951b5fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb791b604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7111b607, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdb1b609, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8f1b60b, SUC_lexicon_tags_8 },
    { 0x5411b60b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df1b610, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa211b613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8451b616, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b1b61b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6c91b61d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7471b61f, SUC_lexicon_tags_26 },
    { 0x2ef1b620, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x36b1b622, SUC_lexicon_tags_15 },
    { 0x3151b623, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe911b62d, SUC_lexicon_tags_13 },
    { 0x1ad1b62e, SUC_lexicon_tags_27 },
    { 0xcd31b62f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40f1b631, SUC_lexicon_tags_67 },
    { 0xe791b632, SUC_lexicon_tags_39 },
    { 0xc4b1b633, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb491b639, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe5b1b63b, SUC_lexicon_tags_81 },
    { 0x1031b63b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4751b63e, SUC_lexicon_tags_33 },
    { 0x1851b63e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5651b641, SUC_lexicon_tags_1 },
    { 0x23f1b642, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b1b646, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3691b648, SUC_lexicon_tags_29 },
    { 0x2811b648, SUC_lexicon_tags_17 },
    { 0x71f1b64a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8d11b64c, SUC_lexicon_tags_25 },
    { 0x2671b64c, SUC_lexicon_tags_17 },
    { 0xcbb1b64e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d1b656, SUC_lexicon_tags_66 },
    { 0x3eb1b656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1b65c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec91b65f, SUC_lexicon_tags_25 },
    { 0x92f1b660, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d91b664, SUC_lexicon_tags_10 },
    { 0xd431b664, SUC_lexicon_tags_21 },
    { 0xdc51b666, SUC_lexicon_tags_31 },
    { 0x911b667, SUC_lexicon_tags_21 },
    { 0x9ab1b666, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6451b66d, SUC_lexicon_tags_17 },
    { 0x2e91b66d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x911b670, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0f1b672, SUC_lexicon_tags_6 },
    { 0xb391b673, SUC_lexicon_tags_12 },
    { 0x9e51b672, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b91b678, SUC_lexicon_tags_15 },
    { 0x8411b679, SUC_lexicon_tags_37 },
    { 0x9f91b67a, SUC_lexicon_tags_16 },
    { 0xb8d1b67b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1211b67e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6351b681, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb1b686, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f1b689, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1711b68d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8351b690, SUC_lexicon_tags_46 },
    { 0x9451b690, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9151b695, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2671b698, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf731b69b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x99d1b69d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7891b6a2, SUC_lexicon_tags_719 },
    { 0x7cd1b6a2, SUC_lexicon_tags_71 },
    { 0xf5b1b6a4, SUC_lexicon_tags_57 },
    { 0x82f1b6a4, SUC_lexicon_tags_68 },
    { 0x2351b6a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad11b6ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd491b6b4, SUC_lexicon_tags_17 },
    { 0xe5d1b6b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd431b6be, SUC_lexicon_tags_17 },
    { 0x9391b6be, SUC_lexicon_tags_26 },
    { 0x911b6c0, SUC_lexicon_tags_25 },
    { 0x99f1b6be, SUC_lexicon_tags_17 },
    { 0xcf71b6c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3f1b6c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b1b6c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a91b6cb, SUC_lexicon_tags_43 },
    { 0xb7f1b6cc, SUC_lexicon_tags_27 },
    { 0x24f1b6cb, SUC_lexicon_tags_25 },
    { 0x13d1b6ce, SUC_lexicon_tags_0 },
    { 0x9d71b6cf, SUC_lexicon_tags_23 },
    { 0xacb1b6d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8d1b6d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb1b6d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb11b6dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf091b6e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe731b6e7, SUC_lexicon_tags_7 },
    { 0x8b51b6e8, SUC_lexicon_tags_14 },
    { 0x7751b6e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd231b6eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x26f1b6ed, SUC_lexicon_tags_33 },
    { 0x5e11b6ee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2891b6f8, SUC_lexicon_tags_6 },
    { 0x40f1b6f9, SUC_lexicon_tags_31 },
    { 0x8dd1b6fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x771b6fd, SUC_lexicon_tags_17 },
    { 0xbdd1b6fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6d71b700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb1b703, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9491b705, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531b70c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9711b70f, SUC_lexicon_tags_15 },
    { 0xc191b710, SUC_lexicon_tags_17 },
    { 0x4371b711, SUC_lexicon_tags_2 },
    { 0x4111b712, SUC_lexicon_tags_1 },
    { 0xce31b711, SUC_lexicon_tags_11 },
    { 0xc031b714, SUC_lexicon_tags_32 },
    { 0x70d1b715, SUC_lexicon_tags_37 },
    { 0xcf71b712, SUC_lexicon_tags_7 },
    { 0x8771b712, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8d11b719, SUC_lexicon_tags_87 },
    { 0x3711b71a, SUC_lexicon_tags_17 },
    { 0xdc91b71b, SUC_lexicon_tags_12 },
    { 0xb411b71c, SUC_lexicon_tags_12 },
    { 0x7011b71b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8551b720, SUC_lexicon_tags_15 },
    { 0x1491b720, SUC_lexicon_tags_25 },
    { 0xd191b722, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7771b72a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a71b72e, SUC_lexicon_tags_25 },
    { 0x6f51b72e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6791b731, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8591b734, SUC_lexicon_tags_15 },
    { 0xaf51b735, SUC_lexicon_tags_32 },
    { 0xfbd1b736, SUC_lexicon_tags_37 },
    { 0x3731b735, SUC_lexicon_tags_7 },
    { 0x2291b738, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa291b742, SUC_lexicon_tags_65 },
    { 0x1571b743, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9171b747, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8351b74a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d1b74d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8731b750, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5911b752, SUC_lexicon_tags_15 },
    { 0x1e91b753, SUC_lexicon_tags_12 },
    { 0x711b753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70d1b756, SUC_lexicon_tags_12 },
    { 0xe5b1b757, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5411b75b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d1b761, SUC_lexicon_tags_79 },
    { 0x47d1b762, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf1b765, SUC_lexicon_tags_6 },
    { 0xa4b1b766, SUC_lexicon_tags_7 },
    { 0xb7f1b766, SUC_lexicon_tags_11 },
    { 0x5951b766, SUC_lexicon_tags_1 },
    { 0xcdf1b769, SUC_lexicon_tags_47 },
    { 0x4531b76a, SUC_lexicon_tags_56 },
    { 0x9dd1b766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1111b76d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b1b771, SUC_lexicon_tags_60 },
    { 0xe731b772, SUC_lexicon_tags_25 },
    { 0x6191b772, SUC_lexicon_tags_30 },
    { 0x4431b771, SUC_lexicon_tags_19 },
    { 0xd351b773, SUC_lexicon_tags_14 },
    { 0xf0f1b776, SUC_lexicon_tags_31 },
    { 0xf431b774, SUC_lexicon_tags_8 },
    { 0x9491b778, SUC_lexicon_tags_25 },
    { 0x1ef1b779, SUC_lexicon_tags_299 },
    { 0x9cd1b775, SUC_lexicon_tags_11 },
    { 0x8d51b776, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7631b77d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd691b77f, SUC_lexicon_tags_43 },
    { 0xc8b1b780, SUC_lexicon_tags_14 },
    { 0xdcb1b780, SUC_lexicon_tags_27 },
    { 0x70d1b782, SUC_lexicon_tags_32 },
    { 0xecf1b783, SUC_lexicon_tags_17 },
    { 0xebb1b784, SUC_lexicon_tags_13 },
    { 0x72f1b785, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf1b78b, SUC_lexicon_tags_14 },
    { 0x1251b78b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f71b790, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d71b793, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc191b797, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3311b799, SUC_lexicon_tags_60 },
    { 0x7e91b79a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9631b79f, SUC_lexicon_tags_6 },
    { 0xa911b79f, SUC_lexicon_tags_25 },
    { 0xdd1b7a1, SUC_lexicon_tags_2 },
    { 0xb51b7a2, SUC_lexicon_tags_48 },
    { 0xf671b7a3, SUC_lexicon_tags_8 },
    { 0xd31b79f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d51b7a9, SUC_lexicon_tags_49 },
    { 0x9f91b7aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbd31b7ac, SUC_lexicon_tags_45 },
    { 0x9e31b7ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f1b7b2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc11b7b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf071b7ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f1b7be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d91b7c4, SUC_lexicon_tags_17 },
    { 0x82d1b7c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9d1b7c7, SUC_lexicon_tags_6 },
    { 0xaef1b7c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd31b7cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5971b7cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b1b7d4, SUC_lexicon_tags_7 },
    { 0x7af1b7d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d1b7d8, SUC_lexicon_tags_16 },
    { 0x99b1b7d9, SUC_lexicon_tags_12 },
    { 0x1cb1b7da, SUC_lexicon_tags_1 },
    { 0x1051b7da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f51b7de, SUC_lexicon_tags_2 },
    { 0x7811b7df, SUC_lexicon_tags_7 },
    { 0x7551b7e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc571b7e4, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d1b7e8, SUC_lexicon_tags_47 },
    { 0xbf31b7e9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d1b7ec, SUC_lexicon_tags_25 },
    { 0x4c91b7ed, SUC_lexicon_tags_28 },
    { 0x85d1b7ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa931b7f1, SUC_lexicon_tags_25 },
    { 0xb991b7f2, SUC_lexicon_tags_29 },
    { 0x4691b7f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x85f1b7f5, SUC_lexicon_tags_37 },
    { 0x7611b7f5, SUC_lexicon_tags_37 },
    { 0xf671b7f7, SUC_lexicon_tags_7 },
    { 0xfe11b7f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b1b7fc, SUC_lexicon_tags_33 },
    { 0xc351b7fd, SUC_lexicon_tags_15 },
    { 0x391b7fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5271b805, SUC_lexicon_tags_6 },
    { 0x2691b805, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf1b808, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcfd1b80a, SUC_lexicon_tags_15 },
    { 0x56b1b80b, SUC_lexicon_tags_37 },
    { 0xcb1b80c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1b80f, SUC_lexicon_tags_43 },
    { 0xa491b810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c51b812, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda51b81a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf491b81c, SUC_lexicon_tags_16 },
    { 0x7e31b81d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e71b820, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7c91b822, SUC_lexicon_tags_11 },
    { 0xd971b823, SUC_lexicon_tags_23 },
    { 0x48d1b824, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b828, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd1b82c, SUC_lexicon_tags_1 },
    { 0xfd11b82c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9651b82f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4991b831, SUC_lexicon_tags_49 },
    { 0x2751b832, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f91b83b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xb711b83d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d1b843, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa8b1b845, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xa931b847, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf671b849, SUC_lexicon_tags_12 },
    { 0x4651b849, SUC_lexicon_tags_28 },
    { 0x6d31b84b, SUC_lexicon_tags_63 },
    { 0x7b11b84c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251b850, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2b1b852, SUC_lexicon_tags_19 },
    { 0x5831b852, SUC_lexicon_tags_17 },
    { 0xa591b854, SUC_lexicon_tags_11 },
    { 0x2911b855, SUC_lexicon_tags_1 },
    { 0x6011b855, SUC_lexicon_tags_63 },
    { 0x1f71b855, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8f1b859, SUC_lexicon_tags_2 },
    { 0x9391b85a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d1b85e, SUC_lexicon_tags_11 },
    { 0xb451b85f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1b862, SUC_lexicon_tags_45 },
    { 0xcc91b862, SUC_lexicon_tags_7 },
    { 0x69d1b864, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d1b86a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe5f1b86c, SUC_lexicon_tags_19 },
    { 0xd3d1b86d, SUC_lexicon_tags_37 },
    { 0x4331b86e, SUC_lexicon_tags_17 },
    { 0xcbd1b86d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa291b871, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b1b874, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9951b877, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0x1b11b879, SUC_lexicon_tags_11 },
    { 0x4f11b87a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d91b881, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7811b887, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f91b88a, SUC_lexicon_tags_34 },
    { 0x2a51b88b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5171b88d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d51b890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49d1b892, SUC_lexicon_tags_49 },
    { 0xf1d1b893, SUC_lexicon_tags_60 },
    { 0xe0d1b894, SUC_lexicon_tags_60 },
    { 0xb011b894, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d91b897, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb1b89c, SUC_lexicon_tags_26 },
    { 0xec71b89c, SUC_lexicon_tags_66 },
    { 0xc9b1b89e, SUC_lexicon_tags_14 },
    { 0x2ed1b89f, SUC_lexicon_tags_14 },
    { 0x6e51b8a0, SUC_lexicon_tags_11 },
    { 0x43b1b89f, SUC_lexicon_tags_1 },
    { 0xf091b8a2, SUC_lexicon_tags_23 },
    { 0xe11b8a1, SUC_lexicon_tags_11 },
    { 0x3591b8a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9e71b8a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff51b8ad, SUC_lexicon_tags_96 },
    { 0xa971b8ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3911b8b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe791b8b8, SUC_lexicon_tags_8 },
    { 0x2391b8b9, SUC_lexicon_tags_113 },
    { 0x73f1b8ba, SUC_lexicon_tags_38 },
    { 0x1311b8b9, SUC_lexicon_tags_37 },
    { 0xf6d1b8bc, SUC_lexicon_tags_47 },
    { 0xdb31b8bc, SUC_lexicon_tags_37 },
    { 0x9011b8be, SUC_lexicon_tags_31 },
    { 0xe4f1b8bf, SUC_lexicon_tags_11 },
    { 0xb9b1b8bc, SUC_lexicon_tags_15 },
    { 0x2751b8c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd4f1b8c3, SUC_lexicon_tags_28 },
    { 0x1391b8c4, SUC_lexicon_tags_8 },
    { 0xe931b8c5, SUC_lexicon_tags_55 },
    { 0x24f1b8c3, SUC_lexicon_tags_38 },
    { 0x151b8c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c11b8ca, SUC_lexicon_tags_14 },
    { 0x5491b8cb, SUC_lexicon_tags_17 },
    { 0x1d51b8cb, SUC_lexicon_tags_12 },
    { 0x98f1b8cc, SUC_lexicon_tags_47 },
    { 0x6331b8ce, SUC_lexicon_tags_6 },
    { 0xee11b8cf, SUC_lexicon_tags_35 },
    { 0xbd91b8d0, SUC_lexicon_tags_11 },
    { 0x2751b8ce, SUC_lexicon_tags_55 },
    { 0xac11b8d2, SUC_lexicon_tags_37 },
    { 0x5191b8d3, SUC_lexicon_tags_31 },
    { 0xe431b8d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1111b8d9, SUC_lexicon_tags_15 },
    { 0xe071b8d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe91b8de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb71b8e2, SUC_lexicon_tags_29 },
    { 0x82f1b8e3, SUC_lexicon_tags_15 },
    { 0x151b8e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c51b8e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3471b8e9, SUC_lexicon_tags_87 },
    { 0xf111b8ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x251b8ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf1b8f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b11b8f3, SUC_lexicon_tags_45 },
    { 0x3191b8f4, SUC_lexicon_tags_37 },
    { 0xd2f1b8f4, SUC_lexicon_tags_201 },
    { 0x7ad1b8f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1271b8f8, SUC_lexicon_tags_57 },
    { 0xc271b8f9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf031b8fb, SUC_lexicon_tags_17 },
    { 0x3e51b8fc, SUC_lexicon_tags_12 },
    { 0x5711b8fd, SUC_lexicon_tags_23 },
    { 0x50f1b8fb, SUC_lexicon_tags_2 },
    { 0x84f1b8ff, SUC_lexicon_tags_12 },
    { 0x4bf1b8ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9651b905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f11b908, SUC_lexicon_tags_2 },
    { 0x5911b908, SUC_lexicon_tags_37 },
    { 0x2cb1b90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3911b90c, SUC_lexicon_tags_48 },
    { 0x3d51b90d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8d1b90f, SUC_lexicon_tags_45 },
    { 0x90f1b910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c31b913, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1451b917, SUC_lexicon_tags_3 },
    { 0x2591b918, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b1b91b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a71b91e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb9b1b920, SUC_lexicon_tags_60 },
    { 0x1551b921, SUC_lexicon_tags_21 },
    { 0xe211b922, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6711b924, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ed1b926, SUC_lexicon_tags_37 },
    { 0xe3f1b926, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d1b92b, SUC_lexicon_tags_29 },
    { 0x8951b92c, SUC_lexicon_tags_66 },
    { 0xfbb1b92d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6731b930, SUC_lexicon_tags_27 },
    { 0x1911b930, SUC_lexicon_tags_54 },
    { 0x5191b932, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d91b940, SUC_lexicon_tags_65 },
    { 0x6991b940, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f51b947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2411b949, SUC_lexicon_tags_37 },
    { 0x7391b94a, SUC_lexicon_tags_0 },
    { 0x6d91b949, SUC_lexicon_tags_6 },
    { 0xbaf1b94c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd1b94f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x27d1b951, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4eb1b953, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xab71b955, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f1b958, SUC_lexicon_tags_31 },
    { 0xea11b959, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f1b961, SUC_lexicon_tags_25 },
    { 0xaf91b962, SUC_lexicon_tags_9 },
    { 0x9c71b963, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5791b966, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb1b969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4331b96b, SUC_lexicon_tags_29 },
    { 0x2251b96c, SUC_lexicon_tags_28 },
    { 0x7871b96d, SUC_lexicon_tags_37 },
    { 0xdf1b96e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df1b971, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b1b975, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc551b97a, SUC_lexicon_tags_23 },
    { 0xeb91b97b, SUC_lexicon_tags_0 },
    { 0xe611b97b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde71b980, SUC_lexicon_tags_17 },
    { 0x3911b980, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x51b983, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe791b985, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b1b988, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd1b98c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1051b990, SUC_lexicon_tags_7 },
    { 0xba31b991, SUC_lexicon_tags_38 },
    { 0x1e31b992, SUC_lexicon_tags_17 },
    { 0xc0d1b992, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa431b997, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4151b99a, SUC_lexicon_tags_11 },
    { 0x2311b99a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1851b9a1, SUC_lexicon_tags_6 },
    { 0x291b9a2, SUC_lexicon_tags_75 },
    { 0x5bb1b9a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1571b9a6, SUC_lexicon_tags_7 },
    { 0x8491b9a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x35b1b9a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd1b9ae, SUC_lexicon_tags_31 },
    { 0x9971b9af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc691b9b3, SUC_lexicon_tags_55 },
    { 0xf0d1b9b3, SUC_lexicon_tags_27 },
    { 0x7631b9b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1591b9b8, SUC_lexicon_tags_47 },
    { 0x1ed1b9b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb91b9c1, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa951b9c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3991b9ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd291b9cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb091b9cf, SUC_lexicon_tags_0 },
    { 0xceb1b9d0, SUC_lexicon_tags_12 },
    { 0xfa71b9d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b91b9d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b91b9d7, SUC_lexicon_tags_1 },
    { 0xe491b9d8, SUC_lexicon_tags_2 },
    { 0xed1b9d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x351b9db, SUC_lexicon_tags_27 },
    { 0x4e71b9db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x331b9de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd591b9e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7111b9e6, SUC_lexicon_tags_19 },
    { 0x14d1b9e6, SUC_lexicon_tags_107 },
    { 0x5971b9e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b1b9f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b1b9f6, SUC_lexicon_tags_31 },
    { 0x1151b9f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4311b9f9, SUC_lexicon_tags_14 },
    { 0xb671b9f9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1b9fe, SUC_lexicon_tags_17 },
    { 0xc211b9ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3151ba01, SUC_lexicon_tags_11 },
    { 0x1631ba02, SUC_lexicon_tags_9 },
    { 0xf9f1ba03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab51ba0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c51ba0d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6b51ba0f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd131ba11, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x69b1ba13, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed1ba22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1991ba24, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0x8891ba27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b1ba2b, SUC_lexicon_tags_70 },
    { 0x3a11ba2b, SUC_lexicon_tags_14 },
    { 0x6d71ba2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xecd1ba2f, SUC_lexicon_tags_29 },
    { 0xab1ba30, SUC_lexicon_tags_69 },
    { 0x511ba31, SUC_lexicon_tags_25 },
    { 0x9e71ba30, SUC_lexicon_tags_2 },
    { 0x36d1ba33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc091ba36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b1ba3d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd251ba44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b1ba47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6191ba4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d1ba4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d1ba53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd071ba58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f91ba5c, SUC_lexicon_tags_12 },
    { 0xf1ba5c, SUC_lexicon_tags_34 },
    { 0x6471ba5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe3d1ba60, SUC_lexicon_tags_45 },
    { 0x2b31ba60, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb491ba63, SUC_lexicon_tags_13 },
    { 0xca11ba64, SUC_lexicon_tags_12 },
    { 0xd951ba65, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xab11ba67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a91ba69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc691ba6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdab1ba6d, SUC_lexicon_tags_12 },
    { 0x9d1ba6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7071ba71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c51ba74, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f1ba77, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e11ba81, SUC_lexicon_tags_46 },
    { 0x4fd1ba81, SUC_lexicon_tags_28 },
    { 0x3af1ba83, SUC_lexicon_tags_12 },
    { 0xeff1ba84, SUC_lexicon_tags_45 },
    { 0xdb71ba85, SUC_lexicon_tags_13 },
    { 0xfa71ba85, SUC_lexicon_tags_38 },
    { 0xb4f1ba85, SUC_lexicon_tags_0 },
    { 0xb6f1ba83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63b1ba8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd371ba8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b1ba93, SUC_lexicon_tags_21 },
    { 0x54b1ba93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5511ba96, SUC_lexicon_tags_1 },
    { 0x1fb1ba97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d1baa1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4551baaa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f1baad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f1bab0, SUC_lexicon_tags_9 },
    { 0xba71bab1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x23f1bab3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce51bab6, SUC_lexicon_tags_45 },
    { 0xb971bab6, SUC_lexicon_tags_0 },
    { 0xfb51bab8, SUC_lexicon_tags_32 },
    { 0xdf91bab6, SUC_lexicon_tags_49 },
    { 0x22f1bab6, SUC_lexicon_tags_63 },
    { 0x9df1babb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29d1babd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa071babf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2ed1bac1, SUC_lexicon_tags_25 },
    { 0x9c71bac2, SUC_lexicon_tags_45 },
    { 0x4491bac2, SUC_lexicon_tags_19 },
    { 0xc771bac4, SUC_lexicon_tags_4 },
    { 0x2391bac5, SUC_lexicon_tags_37 },
    { 0x2a91bac1, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xf051bac8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd1bacd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1211bad0, SUC_lexicon_tags_49 },
    { 0x131bad1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9191bad7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e71bad9, SUC_lexicon_tags_47 },
    { 0x86d1bada, SUC_lexicon_tags_12 },
    { 0x87d1bad9, SUC_lexicon_tags_11 },
    { 0x6511badb, SUC_lexicon_tags_38 },
    { 0x2f51badd, SUC_lexicon_tags_26 },
    { 0xe291badd, SUC_lexicon_tags_60 },
    { 0x7651badf, SUC_lexicon_tags_14 },
    { 0x85d1badf, SUC_lexicon_tags_17 },
    { 0x5191badd, SUC_lexicon_tags_11 },
    { 0x4451bae1, SUC_lexicon_tags_15 },
    { 0xa851bae3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa411bae5, SUC_lexicon_tags_37 },
    { 0xc131bae5, SUC_lexicon_tags_7 },
    { 0xcb31bae7, SUC_lexicon_tags_7 },
    { 0x2211bae7, SUC_lexicon_tags_1 },
    { 0x4771bae7, SUC_lexicon_tags_25 },
    { 0xb3f1bae9, SUC_lexicon_tags_92 },
    { 0x93d1baeb, SUC_lexicon_tags_82 },
    { 0x3631baec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24b1baee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8831baf1, SUC_lexicon_tags_43 },
    { 0x3e51baf1, SUC_lexicon_tags_65 },
    { 0xd831baf2, SUC_lexicon_tags_33 },
    { 0xd591baf4, SUC_lexicon_tags_29 },
    { 0xe211baf3, SUC_lexicon_tags_1 },
    { 0x3f71baf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b71bafb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2071bafe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b1bb01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaa51bb03, SUC_lexicon_tags_2 },
    { 0x7091bb03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5251bb06, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2871bb08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad1bb0b, SUC_lexicon_tags_11 },
    { 0x6211bb0c, SUC_lexicon_tags_27 },
    { 0xa951bb0d, SUC_lexicon_tags_37 },
    { 0x6411bb0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1bb14, SUC_lexicon_tags_45 },
    { 0x3b91bb14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f1bb17, SUC_lexicon_tags_1 },
    { 0x6151bb18, SUC_lexicon_tags_37 },
    { 0x6f1bb19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3f1bb1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf1bb22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3d1bb24, SUC_lexicon_tags_7 },
    { 0x9751bb24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe91bb27, SUC_lexicon_tags_1431 },
    { 0, NULL },
    { 0xee11bb29, SUC_lexicon_tags_23 },
    { 0xf411bb29, SUC_lexicon_tags_60 },
    { 0xfe91bb2b, SUC_lexicon_tags_50 },
    { 0x3d51bb2c, SUC_lexicon_tags_17 },
    { 0x3b1bb2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3711bb2f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4151bb31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f91bb37, SUC_lexicon_tags_25 },
    { 0x25d1bb37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa71bb42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d51bb44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad11bb47, SUC_lexicon_tags_28 },
    { 0x4fd1bb48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3191bb4d, SUC_lexicon_tags_17 },
    { 0xa7f1bb4e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b1bb53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6811bb59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5211bb5c, SUC_lexicon_tags_17 },
    { 0xd331bb5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c71bb66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd1bb69, SUC_lexicon_tags_45 },
    { 0xb8b1bb6a, SUC_lexicon_tags_6 },
    { 0x2311bb6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d11bb6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8151bb6f, SUC_lexicon_tags_2 },
    { 0x22b1bb70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b11bb72, SUC_lexicon_tags_1 },
    { 0xb211bb73, SUC_lexicon_tags_7 },
    { 0x8cb1bb74, SUC_lexicon_tags_6 },
    { 0x7d91bb74, SUC_lexicon_tags_46 },
    { 0x8f71bb76, SUC_lexicon_tags_63 },
    { 0x1351bb77, SUC_lexicon_tags_118 },
    { 0x4f91bb78, SUC_lexicon_tags_19 },
    { 0xc31bb78, SUC_lexicon_tags_15 },
    { 0x16b1bb73, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd1bb7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca31bb82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f51bb84, SUC_lexicon_tags_7 },
    { 0xa011bb85, SUC_lexicon_tags_37 },
    { 0x1411bb85, SUC_lexicon_tags_25 },
    { 0x4e91bb87, SUC_lexicon_tags_63 },
    { 0x76d1bb88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5851bb8e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9411bb92, SUC_lexicon_tags_0 },
    { 0x17b1bb93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcf1bb95, SUC_lexicon_tags_1693 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d1bb98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x95d1bb9a, SUC_lexicon_tags_37 },
    { 0x1f31bb9b, SUC_lexicon_tags_29 },
    { 0x49f1bb9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b91bba3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab1bba7, SUC_lexicon_tags_25 },
    { 0x9711bba8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9971bbaa, SUC_lexicon_tags_14 },
    { 0x6e11bbab, SUC_lexicon_tags_23 },
    { 0xd311bbac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3871bbb6, SUC_lexicon_tags_37 },
    { 0x311bbb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8771bbb9, SUC_lexicon_tags_6 },
    { 0xfad1bbba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3931bbbc, SUC_lexicon_tags_37 },
    { 0xf1bbbd, SUC_lexicon_tags_15 },
    { 0x5871bbbe, SUC_lexicon_tags_49 },
    { 0xb191bbbe, SUC_lexicon_tags_47 },
    { 0xc011bbc0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f1bbc3, SUC_lexicon_tags_37 },
    { 0xeb31bbc4, SUC_lexicon_tags_63 },
    { 0x3ed1bbc5, SUC_lexicon_tags_11 },
    { 0xf1f1bbc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8991bbd3, SUC_lexicon_tags_26 },
    { 0x791bbd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c51bbd6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbc11bbd8, SUC_lexicon_tags_26 },
    { 0xfbf1bbd9, SUC_lexicon_tags_26 },
    { 0xbc11bbd9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2391bbe0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5111bbe3, SUC_lexicon_tags_48 },
    { 0x9051bbe4, SUC_lexicon_tags_87 },
    { 0xd2f1bbe5, SUC_lexicon_tags_19 },
    { 0x611bbe6, SUC_lexicon_tags_23 },
    { 0xa9b1bbe7, SUC_lexicon_tags_35 },
    { 0x8951bbe5, SUC_lexicon_tags_52 },
    { 0xb11bbe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d51bbec, SUC_lexicon_tags_38 },
    { 0x54f1bbec, SUC_lexicon_tags_1 },
    { 0x4111bbee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7871bbf2, SUC_lexicon_tags_15 },
    { 0xc371bbf3, SUC_lexicon_tags_91 },
    { 0xfc91bbf4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f1bbf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcb1bbfa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x13f1bbfc, SUC_lexicon_tags_66 },
    { 0x8151bbfd, SUC_lexicon_tags_19 },
    { 0x6f71bbfe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x731bc00, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca71bc0c, SUC_lexicon_tags_15 },
    { 0xc271bc0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaf1bc0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5cb1bc11, SUC_lexicon_tags_14 },
    { 0x4871bc12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b1bc15, SUC_lexicon_tags_11 },
    { 0x8b11bc16, SUC_lexicon_tags_87 },
    { 0x5d51bc15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bc1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51bc1f, SUC_lexicon_tags_39 },
    { 0xd751bc20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7eb1bc22, SUC_lexicon_tags_7 },
    { 0xb991bc23, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf1bc27, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f1bc2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9511bc2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7871bc30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f71bc34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251bc38, SUC_lexicon_tags_47 },
    { 0x4711bc38, SUC_lexicon_tags_46 },
    { 0x4a91bc3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3111bc3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4b51bc41, SUC_lexicon_tags_0 },
    { 0x5b31bc42, SUC_lexicon_tags_7 },
    { 0x8cd1bc42, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb111bc45, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc491bc48, SUC_lexicon_tags_4 },
    { 0x3af1bc48, SUC_lexicon_tags_41 },
    { 0x6951bc4a, SUC_lexicon_tags_1 },
    { 0x83b1bc4a, SUC_lexicon_tags_60 },
    { 0xfa31bc4a, SUC_lexicon_tags_12 },
    { 0xf331bc4d, SUC_lexicon_tags_42 },
    { 0xd411bc4d, SUC_lexicon_tags_12 },
    { 0x8151bc4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bc54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe051bc57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa591bc5a, SUC_lexicon_tags_1 },
    { 0x5f71bc5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e31bc5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd3d1bc5f, SUC_lexicon_tags_659 },
    { 0x9af1bc5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1831bc64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9891bc68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f1bc75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccf1bc77, SUC_lexicon_tags_47 },
    { 0xcc11bc77, SUC_lexicon_tags_2 },
    { 0x49b1bc77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73b1bc7b, SUC_lexicon_tags_28 },
    { 0xbad1bc7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5971bc7f, SUC_lexicon_tags_38 },
    { 0x9131bc80, SUC_lexicon_tags_45 },
    { 0x6251bc80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b11bc84, SUC_lexicon_tags_85 },
    { 0x4df1bc84, SUC_lexicon_tags_49 },
    { 0x2dd1bc86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc4b1bc88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8931bc8a, SUC_lexicon_tags_2 },
    { 0x88b1bc8a, SUC_lexicon_tags_37 },
    { 0x2531bc8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b1bc95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af1bc98, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed11bc9c, SUC_lexicon_tags_0 },
    { 0x9051bc9c, SUC_lexicon_tags_37 },
    { 0x3011bc9e, SUC_lexicon_tags_82 },
    { 0xcdb1bc9f, SUC_lexicon_tags_2 },
    { 0x75f1bc9c, SUC_lexicon_tags_39 },
    { 0x7d71bca1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9871bca6, SUC_lexicon_tags_47 },
    { 0x5fb1bca7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6651bcab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9311bcaf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5b91bcb1, SUC_lexicon_tags_25 },
    { 0xa011bcb1, SUC_lexicon_tags_37 },
    { 0xb0b1bcb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b1bcb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb1bcba, SUC_lexicon_tags_43 },
    { 0x6e11bcba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b1bcbf, SUC_lexicon_tags_43 },
    { 0xcbd1bcbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1bcc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a71bcc5, SUC_lexicon_tags_47 },
    { 0x6eb1bcc6, SUC_lexicon_tags_7 },
    { 0x6c31bcc6, SUC_lexicon_tags_14 },
    { 0xacb1bcc8, SUC_lexicon_tags_12 },
    { 0x8171bcc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f1bccd, SUC_lexicon_tags_11 },
    { 0xfa71bcce, SUC_lexicon_tags_99 },
    { 0xadf1bccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc931bcd1, SUC_lexicon_tags_15 },
    { 0x66b1bcd2, SUC_lexicon_tags_12 },
    { 0x6531bcd3, SUC_lexicon_tags_25 },
    { 0xb51bcd4, SUC_lexicon_tags_29 },
    { 0x931bcd3, SUC_lexicon_tags_46 },
    { 0x5ed1bcd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f1bcd9, SUC_lexicon_tags_7 },
    { 0xd231bcda, SUC_lexicon_tags_23 },
    { 0x82f1bcdb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb1bce1, SUC_lexicon_tags_25 },
    { 0x1071bce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b1bce5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c51bce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1491bceb, SUC_lexicon_tags_17 },
    { 0xf0f1bcec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4771bcef, SUC_lexicon_tags_9 },
    { 0xb511bcf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb1bcf4, SUC_lexicon_tags_14 },
    { 0x6d51bcf5, SUC_lexicon_tags_1 },
    { 0x10f1bcf6, SUC_lexicon_tags_1 },
    { 0xae51bcf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d91bcf9, SUC_lexicon_tags_19 },
    { 0x9631bcfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c71bcfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4831bcff, SUC_lexicon_tags_17 },
    { 0xf8b1bcff, SUC_lexicon_tags_11 },
    { 0x5131bd01, SUC_lexicon_tags_17 },
    { 0xb051bd00, SUC_lexicon_tags_83 },
    { 0xccd1bd03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d1bd07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8891bd09, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91bd16, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb1bd19, SUC_lexicon_tags_43 },
    { 0x9d11bd1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c11bd1f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3ab1bd21, SUC_lexicon_tags_12 },
    { 0x68d1bd22, SUC_lexicon_tags_11 },
    { 0xb4f1bd23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ef1bd25, SUC_lexicon_tags_26 },
    { 0x5eb1bd25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a51bd28, SUC_lexicon_tags_17 },
    { 0xd491bd28, SUC_lexicon_tags_7 },
    { 0x9551bd29, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d1bd2f, SUC_lexicon_tags_111 },
    { 0x5b1bd30, SUC_lexicon_tags_15 },
    { 0xc031bd31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a31bd33, SUC_lexicon_tags_17 },
    { 0x4b11bd34, SUC_lexicon_tags_7 },
    { 0xfcd1bd35, SUC_lexicon_tags_2 },
    { 0x5451bd36, SUC_lexicon_tags_3 },
    { 0x85b1bd37, SUC_lexicon_tags_16 },
    { 0x1ff1bd38, SUC_lexicon_tags_12 },
    { 0x71f1bd37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd1bd3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb1bd3e, SUC_lexicon_tags_17 },
    { 0x1b1bd3f, SUC_lexicon_tags_37 },
    { 0x7991bd40, SUC_lexicon_tags_17 },
    { 0x651bd41, SUC_lexicon_tags_12 },
    { 0x7871bd41, SUC_lexicon_tags_9 },
    { 0x3df1bd43, SUC_lexicon_tags_43 },
    { 0xb131bd44, SUC_lexicon_tags_25 },
    { 0x57b1bd44, SUC_lexicon_tags_17 },
    { 0x5af1bd44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4df1bd48, SUC_lexicon_tags_0 },
    { 0x57d1bd49, SUC_lexicon_tags_60 },
    { 0x2131bd4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff1bd4f, SUC_lexicon_tags_29 },
    { 0xef51bd50, SUC_lexicon_tags_45 },
    { 0x9f1bd50, SUC_lexicon_tags_201 },
    { 0x8c71bd50, SUC_lexicon_tags_61 },
    { 0x53b1bd53, SUC_lexicon_tags_39 },
    { 0x9c31bd53, SUC_lexicon_tags_10 },
    { 0x2cd1bd55, SUC_lexicon_tags_17 },
    { 0x2fb1bd55, SUC_lexicon_tags_4 },
    { 0xe671bd57, SUC_lexicon_tags_151 },
    { 0x2a71bd58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd191bd5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6651bd60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe731bd6d, SUC_lexicon_tags_37 },
    { 0x2eb1bd6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde91bd71, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb1bd76, SUC_lexicon_tags_2 },
    { 0xc531bd77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b1bd7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1691bd7e, SUC_lexicon_tags_9 },
    { 0x9db1bd7f, SUC_lexicon_tags_38 },
    { 0xdb1bd7e, SUC_lexicon_tags_38 },
    { 0x111bd81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d31bd84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b11bd86, SUC_lexicon_tags_47 },
    { 0x711bd87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d1bd8b, SUC_lexicon_tags_14 },
    { 0x42d1bd8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed1bd91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d1bd95, SUC_lexicon_tags_23 },
    { 0xd31bd95, SUC_lexicon_tags_3 },
    { 0x4f31bd97, SUC_lexicon_tags_11 },
    { 0x4a31bd98, SUC_lexicon_tags_31 },
    { 0x7dd1bd99, SUC_lexicon_tags_28 },
    { 0x3411bd9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b1bd9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9531bda0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x46b1bda2, SUC_lexicon_tags_15 },
    { 0xd231bda3, SUC_lexicon_tags_34 },
    { 0x8891bda3, SUC_lexicon_tags_49 },
    { 0x4e91bda3, SUC_lexicon_tags_11 },
    { 0xcfb1bda3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e51bda8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb4f1bdaa, SUC_lexicon_tags_15 },
    { 0x1631bdab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb1bdae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7d1bdb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc491bdb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe631bdb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9dd1bdb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3811bdb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee11bdc2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d1bdc5, SUC_lexicon_tags_56 },
    { 0xec51bdc6, SUC_lexicon_tags_9 },
    { 0xc511bdc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9931bdd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3651bdd4, SUC_lexicon_tags_30 },
    { 0x2ff1bdd5, SUC_lexicon_tags_17 },
    { 0x9a71bdd6, SUC_lexicon_tags_37 },
    { 0xbf71bdd7, SUC_lexicon_tags_37 },
    { 0xad51bdd7, SUC_lexicon_tags_15 },
    { 0x55b1bdd7, SUC_lexicon_tags_16 },
    { 0xebf1bdda, SUC_lexicon_tags_30 },
    { 0x79d1bdd8, SUC_lexicon_tags_49 },
    { 0xc5b1bdd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8b1bdde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f1bde1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb31bde3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d91bde6, SUC_lexicon_tags_19 },
    { 0xe9f1bde7, SUC_lexicon_tags_25 },
    { 0x8df1bde8, SUC_lexicon_tags_25 },
    { 0x3671bde9, SUC_lexicon_tags_9 },
    { 0xd391bdea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf951bdec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1811bdf0, SUC_lexicon_tags_38 },
    { 0xb951bdf1, SUC_lexicon_tags_9 },
    { 0x7791bdf2, SUC_lexicon_tags_11 },
    { 0xaab1bdf1, SUC_lexicon_tags_14 },
    { 0x9a91bdf0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1bdfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe451be00, SUC_lexicon_tags_1 },
    { 0x80f1be00, SUC_lexicon_tags_47 },
    { 0x3ed1be02, SUC_lexicon_tags_2 },
    { 0x7651be03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2f1be05, SUC_lexicon_tags_11 },
    { 0x2ed1be05, SUC_lexicon_tags_39 },
    { 0x3451be07, SUC_lexicon_tags_25 },
    { 0xb051be07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe311be0d, SUC_lexicon_tags_43 },
    { 0xff71be0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xce51be11, SUC_lexicon_tags_37 },
    { 0xe3b1be11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff31be16, SUC_lexicon_tags_9 },
    { 0x411be16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x78b1be19, SUC_lexicon_tags_11 },
    { 0x1ff1be19, SUC_lexicon_tags_33 },
    { 0x7411be1b, SUC_lexicon_tags_37 },
    { 0xb071be1c, SUC_lexicon_tags_25 },
    { 0xf0f1be1d, SUC_lexicon_tags_19 },
    { 0x1031be1e, SUC_lexicon_tags_13 },
    { 0x4d71be1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d1be23, SUC_lexicon_tags_14 },
    { 0xd6f1be23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd31be27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3371be29, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2871be2c, SUC_lexicon_tags_17 },
    { 0xa231be2d, SUC_lexicon_tags_24 },
    { 0xfd71be2c, SUC_lexicon_tags_3 },
    { 0x8af1be2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b1be32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d91be35, SUC_lexicon_tags_21 },
    { 0xef1be36, SUC_lexicon_tags_29 },
    { 0xc7d1be35, SUC_lexicon_tags_37 },
    { 0x9c51be38, SUC_lexicon_tags_37 },
    { 0x7e51be35, SUC_lexicon_tags_31 },
    { 0x8f11be37, SUC_lexicon_tags_21 },
    { 0x30b1be38, SUC_lexicon_tags_31 },
    { 0xacf1be3c, SUC_lexicon_tags_57 },
    { 0x22d1be36, SUC_lexicon_tags_17 },
    { 0x16f1be3b, SUC_lexicon_tags_19 },
    { 0x95b1be3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3311be44, SUC_lexicon_tags_44 },
    { 0x1f31be45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddf1be47, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7551be4c, SUC_lexicon_tags_45 },
    { 0x9df1be4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd111be52, SUC_lexicon_tags_14 },
    { 0x1151be53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed51be57, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1391be5e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9331be64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb871be66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ed1be68, SUC_lexicon_tags_17 },
    { 0x1571be68, SUC_lexicon_tags_47 },
    { 0x73b1be6a, SUC_lexicon_tags_2 },
    { 0x14f1be6a, SUC_lexicon_tags_0 },
    { 0xd311be68, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51be6f, SUC_lexicon_tags_16 },
    { 0x1ed1be6f, SUC_lexicon_tags_0 },
    { 0x75d1be70, SUC_lexicon_tags_16 },
    { 0x6591be72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3871be74, SUC_lexicon_tags_6 },
    { 0x9151be75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe151be77, SUC_lexicon_tags_2 },
    { 0x8331be78, SUC_lexicon_tags_11 },
    { 0x81d1be79, SUC_lexicon_tags_11 },
    { 0x891be78, SUC_lexicon_tags_351 },
    { 0x9a11be7b, SUC_lexicon_tags_25 },
    { 0x9091be7b, SUC_lexicon_tags_25 },
    { 0x291be7d, SUC_lexicon_tags_1 },
    { 0x4cf1be7d, SUC_lexicon_tags_61 },
    { 0x6eb1be7e, SUC_lexicon_tags_6 },
    { 0x5831be80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebd1be82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2271be93, SUC_lexicon_tags_11 },
    { 0x4751be93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc991be96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1be99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6091be9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4d51be9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3291bea1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb991bea3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b31beab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5951beae, SUC_lexicon_tags_14 },
    { 0xcd1beae, SUC_lexicon_tags_35 },
    { 0x1391beae, SUC_lexicon_tags_25 },
    { 0x7271beb0, SUC_lexicon_tags_15 },
    { 0x9a91beaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5191beb7, SUC_lexicon_tags_14 },
    { 0x26d1beb8, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d51bebc, SUC_lexicon_tags_7 },
    { 0xef1bebd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc771bec1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6511bec3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6a71bec5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad1bec9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2111becc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9071bed1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d1bed5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d1bed9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5af1bedb, SUC_lexicon_tags_235 },
    { 0x1c91bedc, SUC_lexicon_tags_1 },
    { 0xcc91bedc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb1bee2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc151bee4, SUC_lexicon_tags_1 },
    { 0xeb11bee5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe791beea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f1beed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf831bef1, SUC_lexicon_tags_10 },
    { 0xd151bef2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa571bef5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d1bef8, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x2131befb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1991beff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d31bf04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa911bf06, SUC_lexicon_tags_37 },
    { 0x89d1bf07, SUC_lexicon_tags_7 },
    { 0x38d1bf06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b91bf0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa391bf0c, SUC_lexicon_tags_19 },
    { 0x3851bf0c, SUC_lexicon_tags_26 },
    { 0xbdf1bf0e, SUC_lexicon_tags_25 },
    { 0xdb91bf0e, SUC_lexicon_tags_1 },
    { 0x2b31bf0c, SUC_lexicon_tags_25 },
    { 0xd91bf0f, SUC_lexicon_tags_37 },
    { 0x8fd1bf12, SUC_lexicon_tags_33 },
    { 0x94d1bf13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8291bf18, SUC_lexicon_tags_82 },
    { 0x1091bf19, SUC_lexicon_tags_17 },
    { 0x36d1bf1a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f1bf21, SUC_lexicon_tags_7 },
    { 0xfa91bf22, SUC_lexicon_tags_33 },
    { 0xb5f1bf22, SUC_lexicon_tags_12 },
    { 0xfdb1bf24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdb11bf26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x49d1bf28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b1bf2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2671bf32, SUC_lexicon_tags_12 },
    { 0xcab1bf33, SUC_lexicon_tags_16 },
    { 0x8731bf33, SUC_lexicon_tags_55 },
    { 0xd6f1bf33, SUC_lexicon_tags_7 },
    { 0xe631bf36, SUC_lexicon_tags_2 },
    { 0x5ed1bf37, SUC_lexicon_tags_33 },
    { 0x5ab1bf38, SUC_lexicon_tags_2 },
    { 0x2171bf34, SUC_lexicon_tags_17 },
    { 0x8791bf37, SUC_lexicon_tags_0 },
    { 0xad1bf3a, SUC_lexicon_tags_8 },
    { 0x5bf1bf37, SUC_lexicon_tags_65 },
    { 0x3991bf35, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc71bf42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd71bf45, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd291bf47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9651bf49, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8011bf4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd791bf51, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb251bf54, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9691bf5d, SUC_lexicon_tags_15 },
    { 0x2bb1bf5e, SUC_lexicon_tags_37 },
    { 0x711bf5f, SUC_lexicon_tags_12 },
    { 0x2771bf60, SUC_lexicon_tags_6 },
    { 0x1071bf61, SUC_lexicon_tags_15 },
    { 0x6391bf62, SUC_lexicon_tags_7 },
    { 0xa351bf60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef1bf66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcb1bf68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f1bf6b, SUC_lexicon_tags_25 },
    { 0x951bf6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6071bf6f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d1bf73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb211bf79, SUC_lexicon_tags_7 },
    { 0x42b1bf7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe211bf85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a11bf89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d71bf92, SUC_lexicon_tags_47 },
    { 0x9451bf93, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d1bf96, SUC_lexicon_tags_38 },
    { 0x9011bf97, SUC_lexicon_tags_68 },
    { 0x20b1bf98, SUC_lexicon_tags_62 },
    { 0x56f1bf96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8071bfa1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25b1bfa3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe091bfa5, SUC_lexicon_tags_35 },
    { 0x65d1bfa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab71bfa8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9bd1bfaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d1bfad, SUC_lexicon_tags_11 },
    { 0xc2f1bfad, SUC_lexicon_tags_37 },
    { 0x3671bfaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3231bfb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17f1bfb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1bfb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fd1bfbb, SUC_lexicon_tags_37 },
    { 0xb8d1bfbc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c31bfbf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xef51bfc2, SUC_lexicon_tags_82 },
    { 0xbd1bfc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf1bfc9, SUC_lexicon_tags_28 },
    { 0x37d1bfca, SUC_lexicon_tags_28 },
    { 0x7711bfca, SUC_lexicon_tags_8 },
    { 0xcc51bfcc, SUC_lexicon_tags_34 },
    { 0xbd71bfcc, SUC_lexicon_tags_17 },
    { 0xcd31bfca, SUC_lexicon_tags_1 },
    { 0x9471bfcf, SUC_lexicon_tags_9 },
    { 0xc331bfd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab71bfd3, SUC_lexicon_tags_2 },
    { 0xe691bfd3, SUC_lexicon_tags_61 },
    { 0x1331bfd5, SUC_lexicon_tags_25 },
    { 0xa4f1bfd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d1bfde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df1bfe2, SUC_lexicon_tags_17 },
    { 0x9631bfe2, SUC_lexicon_tags_11 },
    { 0x4e11bfe4, SUC_lexicon_tags_23 },
    { 0xca71bfe5, SUC_lexicon_tags_34 },
    { 0xa491bfe5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb511bfe9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f1bff1, SUC_lexicon_tags_31 },
    { 0x2b71bff2, SUC_lexicon_tags_12 },
    { 0x2db1bff1, SUC_lexicon_tags_57 },
    { 0x6dd1bff1, SUC_lexicon_tags_17 },
    { 0xfd91bff5, SUC_lexicon_tags_8 },
    { 0x5b71bff2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d1bffb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc071bffd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe711c001, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb1c006, SUC_lexicon_tags_1 },
    { 0xea91c007, SUC_lexicon_tags_28 },
    { 0xd0d1c007, SUC_lexicon_tags_113 },
    { 0xbdd1c006, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x2631c00b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df1c00f, SUC_lexicon_tags_25 },
    { 0xdbf1c010, SUC_lexicon_tags_37 },
    { 0xd831c011, SUC_lexicon_tags_7 },
    { 0x8c11c012, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0x7d51c014, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf851c016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2011c018, SUC_lexicon_tags_69 },
    { 0xd731c018, SUC_lexicon_tags_21 },
    { 0x5551c019, SUC_lexicon_tags_47 },
    { 0xa3f1c01b, SUC_lexicon_tags_38 },
    { 0x7271c01c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6771c01e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671c022, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee91c029, SUC_lexicon_tags_17 },
    { 0xe4b1c02a, SUC_lexicon_tags_15 },
    { 0xf1c02b, SUC_lexicon_tags_63 },
    { 0x97b1c02a, SUC_lexicon_tags_65 },
    { 0x3051c02d, SUC_lexicon_tags_17 },
    { 0x89b1c02e, SUC_lexicon_tags_17 },
    { 0x7b1c02e, SUC_lexicon_tags_15 },
    { 0xae31c030, SUC_lexicon_tags_63 },
    { 0xa31c02f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe51c035, SUC_lexicon_tags_19 },
    { 0x55f1c035, SUC_lexicon_tags_63 },
    { 0x631c037, SUC_lexicon_tags_45 },
    { 0x3051c038, SUC_lexicon_tags_29 },
    { 0x78f1c039, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f1c03d, SUC_lexicon_tags_9 },
    { 0x1f1c03d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd2d1c040, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb531c047, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3571c04f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd91c051, SUC_lexicon_tags_37 },
    { 0x7191c052, SUC_lexicon_tags_26 },
    { 0xad11c051, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xf151c055, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d31c059, SUC_lexicon_tags_37 },
    { 0xbd1c05a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb91c05e, SUC_lexicon_tags_25 },
    { 0x61b1c05e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf991c062, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x711c064, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb891c066, SUC_lexicon_tags_23 },
    { 0xf171c066, SUC_lexicon_tags_0 },
    { 0x55b1c068, SUC_lexicon_tags_56 },
    { 0x3cf1c067, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91c06d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb1c071, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc11c075, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad1c078, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6131c07a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f1c07f, SUC_lexicon_tags_17 },
    { 0x13b1c080, SUC_lexicon_tags_106 },
    { 0x2db1c07f, SUC_lexicon_tags_49 },
    { 0xa711c082, SUC_lexicon_tags_5 },
    { 0xaf51c080, SUC_lexicon_tags_42 },
    { 0xf411c07f, SUC_lexicon_tags_15 },
    { 0xa351c085, SUC_lexicon_tags_47 },
    { 0xf6d1c081, SUC_lexicon_tags_17 },
    { 0x1c11c081, SUC_lexicon_tags_63 },
    { 0x6ed1c081, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f1c08d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b51c092, SUC_lexicon_tags_24 },
    { 0x2091c092, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9091c0a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1971c0a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2591c0ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x171c0b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9271c0b2, SUC_lexicon_tags_12 },
    { 0xc291c0b3, SUC_lexicon_tags_38 },
    { 0xf151c0b4, SUC_lexicon_tags_11 },
    { 0x3371c0b5, SUC_lexicon_tags_35 },
    { 0x28d1c0b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f1c0b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb271c0bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf851c0c2, SUC_lexicon_tags_12 },
    { 0x6971c0c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22b1c0c5, SUC_lexicon_tags_6 },
    { 0x2131c0c6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e51c0cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8b1c0cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb1c0d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1691c0d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a51c0d9, SUC_lexicon_tags_12 },
    { 0x6611c0da, SUC_lexicon_tags_68 },
    { 0xc4f1c0d9, SUC_lexicon_tags_38 },
    { 0x2551c0dc, SUC_lexicon_tags_23 },
    { 0xd251c0dd, SUC_lexicon_tags_3 },
    { 0x26f1c0da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1c0e1, SUC_lexicon_tags_17 },
    { 0x45d1c0e1, SUC_lexicon_tags_26 },
    { 0x6351c0e3, SUC_lexicon_tags_63 },
    { 0x3f91c0e3, SUC_lexicon_tags_7 },
    { 0xe6f1c0e5, SUC_lexicon_tags_14 },
    { 0xe731c0e5, SUC_lexicon_tags_70 },
    { 0xccd1c0e7, SUC_lexicon_tags_7 },
    { 0xd9b1c0e8, SUC_lexicon_tags_70 },
    { 0x6f91c0e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc931c0ed, SUC_lexicon_tags_103 },
    { 0x9ff1c0ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef1c0f1, SUC_lexicon_tags_60 },
    { 0xd411c0f2, SUC_lexicon_tags_47 },
    { 0x2211c0f3, SUC_lexicon_tags_0 },
    { 0x6b71c0f1, SUC_lexicon_tags_30 },
    { 0x6bb1c0f1, SUC_lexicon_tags_48 },
    { 0xb751c0f4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6871c0f8, SUC_lexicon_tags_11 },
    { 0xc211c0f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf1c0fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd1c101, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7391c106, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2db1c108, SUC_lexicon_tags_3 },
    { 0x3d71c109, SUC_lexicon_tags_29 },
    { 0x5c91c10a, SUC_lexicon_tags_1 },
    { 0xdf11c10a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa391c10d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f1c111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b1c114, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e31c11a, SUC_lexicon_tags_17 },
    { 0x9d51c11a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2231c11e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb1c123, SUC_lexicon_tags_25 },
    { 0xe5b1c123, SUC_lexicon_tags_27 },
    { 0x5bf1c124, SUC_lexicon_tags_55 },
    { 0xefb1c126, SUC_lexicon_tags_47 },
    { 0x9631c127, SUC_lexicon_tags_16 },
    { 0x7d1c128, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1c12c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2951c130, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c51c133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1c13b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9411c142, SUC_lexicon_tags_8 },
    { 0x5fb1c143, SUC_lexicon_tags_57 },
    { 0x9831c144, SUC_lexicon_tags_68 },
    { 0x71f1c145, SUC_lexicon_tags_27 },
    { 0xde91c145, SUC_lexicon_tags_17 },
    { 0x19d1c145, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1331c149, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ff1c14b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251c14f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3091c152, SUC_lexicon_tags_11 },
    { 0x90f1c152, SUC_lexicon_tags_16 },
    { 0xf431c154, SUC_lexicon_tags_31 },
    { 0xcd91c153, SUC_lexicon_tags_6 },
    { 0xc9f1c155, SUC_lexicon_tags_2 },
    { 0xa231c157, SUC_lexicon_tags_26 },
    { 0x2831c157, SUC_lexicon_tags_0 },
    { 0xa131c159, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb11c160, SUC_lexicon_tags_3 },
    { 0xc8d1c161, SUC_lexicon_tags_37 },
    { 0xcab1c160, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f1c167, SUC_lexicon_tags_46 },
    { 0x71d1c168, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe191c16a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fb1c16c, SUC_lexicon_tags_15 },
    { 0x7191c16d, SUC_lexicon_tags_15 },
    { 0x9a91c16e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3371c170, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8631c172, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe311c175, SUC_lexicon_tags_17 },
    { 0x1f1c176, SUC_lexicon_tags_60 },
    { 0xed51c177, SUC_lexicon_tags_25 },
    { 0x3771c175, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d1c182, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xea11c184, SUC_lexicon_tags_1700 },
    { 0xca51c185, SUC_lexicon_tags_37 },
    { 0x3a11c186, SUC_lexicon_tags_14 },
    { 0xab11c186, SUC_lexicon_tags_12 },
    { 0x6f51c188, SUC_lexicon_tags_15 },
    { 0x5731c185, SUC_lexicon_tags_21 },
    { 0x82f1c18a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f1c18e, SUC_lexicon_tags_37 },
    { 0x90d1c18f, SUC_lexicon_tags_15 },
    { 0xced1c190, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3b1c192, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb51c195, SUC_lexicon_tags_12 },
    { 0x9051c196, SUC_lexicon_tags_12 },
    { 0xd651c197, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc811c199, SUC_lexicon_tags_31 },
    { 0xaa31c199, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe031c19f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7b31c1a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f1c1a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af1c1a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d1c1ac, SUC_lexicon_tags_1 },
    { 0x81d1c1ad, SUC_lexicon_tags_12 },
    { 0xa191c1ae, SUC_lexicon_tags_16 },
    { 0xbad1c1af, SUC_lexicon_tags_4 },
    { 0xed11c1b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf1c1b3, SUC_lexicon_tags_15 },
    { 0x2311c1b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3951c1b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e71c1c0, SUC_lexicon_tags_29 },
    { 0x15f1c1c1, SUC_lexicon_tags_1 },
    { 0xc7f1c1c2, SUC_lexicon_tags_2 },
    { 0x3111c1c3, SUC_lexicon_tags_25 },
    { 0x9ef1c1c4, SUC_lexicon_tags_1 },
    { 0x3bb1c1c5, SUC_lexicon_tags_15 },
    { 0xbe71c1c4, SUC_lexicon_tags_87 },
    { 0x4771c1c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf91c1ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6491c1d0, SUC_lexicon_tags_91 },
    { 0x6d51c1d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ef1c1d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9231c1da, SUC_lexicon_tags_47 },
    { 0x4531c1db, SUC_lexicon_tags_61 },
    { 0xdf11c1dc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1291c1e1, SUC_lexicon_tags_522 },
    { 0x93f1c1e2, SUC_lexicon_tags_9 },
    { 0xe251c1e3, SUC_lexicon_tags_7 },
    { 0x8391c1e1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2071c1e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb1c1eb, SUC_lexicon_tags_11 },
    { 0x75d1c1ec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd711c1f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5b1c1f4, SUC_lexicon_tags_47 },
    { 0xcd31c1f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3731c1f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef1c1fc, SUC_lexicon_tags_37 },
    { 0xa731c1fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed1c200, SUC_lexicon_tags_1 },
    { 0x2191c200, SUC_lexicon_tags_25 },
    { 0xbdf1c202, SUC_lexicon_tags_1 },
    { 0x8631c202, SUC_lexicon_tags_17 },
    { 0x39f1c203, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8bf1c206, SUC_lexicon_tags_57 },
    { 0x5c91c206, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d1c20b, SUC_lexicon_tags_56 },
    { 0x2651c20c, SUC_lexicon_tags_11 },
    { 0x3571c20b, SUC_lexicon_tags_13 },
    { 0x4451c20e, SUC_lexicon_tags_60 },
    { 0x9ed1c20f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f1c211, SUC_lexicon_tags_14 },
    { 0xf971c212, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2991c214, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8651c218, SUC_lexicon_tags_63 },
    { 0xab31c219, SUC_lexicon_tags_2 },
    { 0x4811c218, SUC_lexicon_tags_0 },
    { 0xeb71c21b, SUC_lexicon_tags_28 },
    { 0x8091c21c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd1c21f, SUC_lexicon_tags_98 },
    { 0x90f1c21f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d1c224, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b51c226, SUC_lexicon_tags_17 },
    { 0x5eb1c226, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d91c230, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd1c234, SUC_lexicon_tags_37 },
    { 0x1611c235, SUC_lexicon_tags_51 },
    { 0xd291c234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb071c239, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f1c23c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff1c23f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7331c247, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d1c251, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f1c25a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b1c262, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd651c265, SUC_lexicon_tags_275 },
    { 0xdd51c266, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd1c26a, SUC_lexicon_tags_31 },
    { 0xf2b1c26b, SUC_lexicon_tags_11 },
    { 0xc411c26b, SUC_lexicon_tags_12 },
    { 0x1171c26c, SUC_lexicon_tags_17 },
    { 0x8db1c26e, SUC_lexicon_tags_12 },
    { 0xdcb1c26b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe631c271, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61f1c273, SUC_lexicon_tags_49 },
    { 0xb2d1c273, SUC_lexicon_tags_33 },
    { 0xc7d1c275, SUC_lexicon_tags_39 },
    { 0xeb71c276, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7891c278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd31c27d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x27d1c27f, SUC_lexicon_tags_1 },
    { 0xfb51c280, SUC_lexicon_tags_7 },
    { 0xd691c281, SUC_lexicon_tags_38 },
    { 0x5a11c282, SUC_lexicon_tags_17 },
    { 0xf931c283, SUC_lexicon_tags_17 },
    { 0x2951c284, SUC_lexicon_tags_8 },
    { 0xc891c285, SUC_lexicon_tags_1 },
    { 0xbdb1c286, SUC_lexicon_tags_15 },
    { 0x5f71c286, SUC_lexicon_tags_1 },
    { 0x84b1c286, SUC_lexicon_tags_25 },
    { 0x1a91c289, SUC_lexicon_tags_6 },
    { 0x8cd1c289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4dd1c28c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5051c28f, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xa091c291, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4a91c293, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac71c295, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5631c29a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f11c29e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1c2a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4b11c2a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4931c2a6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1c2ab, SUC_lexicon_tags_68 },
    { 0x9451c2ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f1c2b5, SUC_lexicon_tags_53 },
    { 0x66f1c2b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3431c2b8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1cf1c2ba, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6b11c2bc, SUC_lexicon_tags_45 },
    { 0x1d31c2bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb11c2c0, SUC_lexicon_tags_37 },
    { 0xf791c2c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7131c2c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6191c2ca, SUC_lexicon_tags_37 },
    { 0x18b1c2ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb291c2d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f1c2d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d1c2dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7991c2df, SUC_lexicon_tags_45 },
    { 0xf7b1c2e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcc91c2e2, SUC_lexicon_tags_16 },
    { 0x84b1c2e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f1c2e7, SUC_lexicon_tags_17 },
    { 0xa311c2e8, SUC_lexicon_tags_33 },
    { 0x9271c2e8, SUC_lexicon_tags_12 },
    { 0x4db1c2e8, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb1c2ef, SUC_lexicon_tags_25 },
    { 0xbdb1c2f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2311c2f2, SUC_lexicon_tags_65 },
    { 0xe771c2f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c71c2fb, SUC_lexicon_tags_19 },
    { 0x57d1c2fc, SUC_lexicon_tags_11 },
    { 0xf311c2fd, SUC_lexicon_tags_63 },
    { 0x74d1c2fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd371c301, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f1c305, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d1c309, SUC_lexicon_tags_11 },
    { 0x42f1c30a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8551c30c, SUC_lexicon_tags_49 },
    { 0x9591c30d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d91c310, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe191c312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f1c317, SUC_lexicon_tags_15 },
    { 0x9dd1c318, SUC_lexicon_tags_11 },
    { 0xc751c318, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc331c31b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af1c31e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xde11c321, SUC_lexicon_tags_26 },
    { 0x42d1c322, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccb1c324, SUC_lexicon_tags_17 },
    { 0xd9f1c324, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe171c32b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88f1c32d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd151c330, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8531c337, SUC_lexicon_tags_43 },
    { 0xe131c337, SUC_lexicon_tags_17 },
    { 0xd631c339, SUC_lexicon_tags_17 },
    { 0x8c51c339, SUC_lexicon_tags_6 },
    { 0x7cb1c338, SUC_lexicon_tags_17 },
    { 0xd051c33c, SUC_lexicon_tags_37 },
    { 0xbd51c33a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe251c340, SUC_lexicon_tags_25 },
    { 0x7831c340, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6731c347, SUC_lexicon_tags_25 },
    { 0x8a71c348, SUC_lexicon_tags_15 },
    { 0x1db1c349, SUC_lexicon_tags_43 },
    { 0xbb1c34a, SUC_lexicon_tags_2 },
    { 0x2451c349, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6471c34d, SUC_lexicon_tags_23 },
    { 0x8a11c34e, SUC_lexicon_tags_37 },
    { 0x3731c34d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd1c353, SUC_lexicon_tags_37 },
    { 0x2af1c354, SUC_lexicon_tags_15 },
    { 0xd1c354, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b1c358, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5211c35b, SUC_lexicon_tags_2 },
    { 0x2c71c35b, SUC_lexicon_tags_63 },
    { 0x16f1c35c, SUC_lexicon_tags_82 },
    { 0xeab1c35d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f31c360, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb671c364, SUC_lexicon_tags_25 },
    { 0x9651c365, SUC_lexicon_tags_37 },
    { 0x6171c366, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f1c369, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83f1c36b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab1c36f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb1c372, SUC_lexicon_tags_31 },
    { 0x93b1c373, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4cd1c375, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3571c377, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb551c379, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d71c37f, SUC_lexicon_tags_12 },
    { 0x7691c380, SUC_lexicon_tags_37 },
    { 0x55d1c381, SUC_lexicon_tags_1 },
    { 0xbf51c380, SUC_lexicon_tags_19 },
    { 0x6b11c383, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40d1c385, SUC_lexicon_tags_57 },
    { 0x1911c385, SUC_lexicon_tags_27 },
    { 0xd391c387, SUC_lexicon_tags_12 },
    { 0x8b51c385, SUC_lexicon_tags_37 },
    { 0x4011c389, SUC_lexicon_tags_0 },
    { 0x67f1c386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a11c38d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x61b1c38f, SUC_lexicon_tags_65 },
    { 0x31c390, SUC_lexicon_tags_0 },
    { 0x7831c391, SUC_lexicon_tags_27 },
    { 0xdbf1c390, SUC_lexicon_tags_1 },
    { 0xd1b1c391, SUC_lexicon_tags_1 },
    { 0xb5d1c394, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb91c397, SUC_lexicon_tags_13 },
    { 0xbf1c398, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb1c39c, SUC_lexicon_tags_12 },
    { 0x8bb1c39d, SUC_lexicon_tags_65 },
    { 0x9911c39e, SUC_lexicon_tags_37 },
    { 0x4111c39e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3cd1c3a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd351c3a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7611c3a7, SUC_lexicon_tags_0 },
    { 0xf491c3a7, SUC_lexicon_tags_17 },
    { 0xf351c3a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf11c3ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5791c3af, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf651c3b1, SUC_lexicon_tags_37 },
    { 0xeb91c3b2, SUC_lexicon_tags_17 },
    { 0x60d1c3b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xec71c3b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b1c3ba, SUC_lexicon_tags_12 },
    { 0x6331c3ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8191c3bd, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b1c3c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef1c3c3, SUC_lexicon_tags_16 },
    { 0x7fb1c3c4, SUC_lexicon_tags_46 },
    { 0xcf91c3c5, SUC_lexicon_tags_31 },
    { 0x1dd1c3c5, SUC_lexicon_tags_30 },
    { 0xbd11c3c6, SUC_lexicon_tags_25 },
    { 0xf771c3c6, SUC_lexicon_tags_37 },
    { 0x2e71c3c9, SUC_lexicon_tags_39 },
    { 0xbf1c3c8, SUC_lexicon_tags_15 },
    { 0xbcd1c3cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6371c3cd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1c3d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11c3d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf151c3d7, SUC_lexicon_tags_26 },
    { 0xe1f1c3d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f1c3de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd891c3e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc71c3e3, SUC_lexicon_tags_27 },
    { 0x94b1c3e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af1c3ea, SUC_lexicon_tags_11 },
    { 0xf3b1c3eb, SUC_lexicon_tags_45 },
    { 0xf31c3ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3771c3f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x811c3f2, SUC_lexicon_tags_17 },
    { 0xd891c3f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91c3f7, SUC_lexicon_tags_25 },
    { 0x8b31c3f7, SUC_lexicon_tags_31 },
    { 0x8d51c3f7, SUC_lexicon_tags_17 },
    { 0x5d11c3fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d1c3fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe011c404, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e11c407, SUC_lexicon_tags_38 },
    { 0xe511c408, SUC_lexicon_tags_65 },
    { 0x1231c407, SUC_lexicon_tags_6 },
    { 0xe6b1c40a, SUC_lexicon_tags_37 },
    { 0xb151c40b, SUC_lexicon_tags_47 },
    { 0x24b1c408, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e51c40f, SUC_lexicon_tags_28 },
    { 0xc331c410, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d1c413, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d1c416, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf471c41b, SUC_lexicon_tags_26 },
    { 0xf251c41b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4371c421, SUC_lexicon_tags_12 },
    { 0x9091c421, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b1c42a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a31c42c, SUC_lexicon_tags_37 },
    { 0x151c42c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a31c430, SUC_lexicon_tags_2 },
    { 0xedb1c431, SUC_lexicon_tags_45 },
    { 0xf111c431, SUC_lexicon_tags_37 },
    { 0x26f1c433, SUC_lexicon_tags_25 },
    { 0xbbb1c433, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa131c437, SUC_lexicon_tags_45 },
    { 0x6c91c438, SUC_lexicon_tags_15 },
    { 0x4e11c438, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b91c43b, SUC_lexicon_tags_40 },
    { 0x37d1c43b, SUC_lexicon_tags_17 },
    { 0x1b11c43b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5271c442, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc011c444, SUC_lexicon_tags_15 },
    { 0x5791c444, SUC_lexicon_tags_25 },
    { 0xc6d1c446, SUC_lexicon_tags_12 },
    { 0x1e71c447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4651c44a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x411c44d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6c71c44f, SUC_lexicon_tags_57 },
    { 0x6ab1c44f, SUC_lexicon_tags_2 },
    { 0x1731c451, SUC_lexicon_tags_140 },
    { 0xa5f1c451, SUC_lexicon_tags_8 },
    { 0xb751c44f, SUC_lexicon_tags_47 },
    { 0xa171c453, SUC_lexicon_tags_1 },
    { 0xfc71c455, SUC_lexicon_tags_9 },
    { 0x4711c455, SUC_lexicon_tags_43 },
    { 0xf851c456, SUC_lexicon_tags_14 },
    { 0xa551c458, SUC_lexicon_tags_11 },
    { 0x9951c459, SUC_lexicon_tags_17 },
    { 0x9671c456, SUC_lexicon_tags_87 },
    { 0xa111c45b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2171c45d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51c461, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa791c465, SUC_lexicon_tags_1 },
    { 0x6fb1c466, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5331c46a, SUC_lexicon_tags_47 },
    { 0xa4b1c46a, SUC_lexicon_tags_27 },
    { 0x7d31c46b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4251c46f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19b1c471, SUC_lexicon_tags_37 },
    { 0x831c471, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d1c479, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f1c47f, SUC_lexicon_tags_26 },
    { 0xd9b1c480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x391c482, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd691c487, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e11c48a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf91c48c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f1c497, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b1c49e, SUC_lexicon_tags_28 },
    { 0x6031c49e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd71c4a1, SUC_lexicon_tags_171 },
    { 0xe2b1c4a1, SUC_lexicon_tags_7 },
    { 0xe151c4a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9d1c4a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55b1c4a7, SUC_lexicon_tags_15 },
    { 0xc1d1c4a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb1c4ad, SUC_lexicon_tags_7 },
    { 0x3d31c4ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91c4b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5d71c4b3, SUC_lexicon_tags_4 },
    { 0xd3f1c4b3, SUC_lexicon_tags_1 },
    { 0x1f91c4b4, SUC_lexicon_tags_31 },
    { 0xcb11c4b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e11c4b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x31b1c4ba, SUC_lexicon_tags_1 },
    { 0xe291c4bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc71c4bd, SUC_lexicon_tags_65 },
    { 0x2111c4be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed1c4c5, SUC_lexicon_tags_11 },
    { 0x3a51c4c6, SUC_lexicon_tags_29 },
    { 0x8c11c4c7, SUC_lexicon_tags_43 },
    { 0xdd31c4c7, SUC_lexicon_tags_23 },
    { 0x5551c4c9, SUC_lexicon_tags_141 },
    { 0x6ab1c4ca, SUC_lexicon_tags_37 },
    { 0xa511c4ca, SUC_lexicon_tags_25 },
    { 0xec91c4cc, SUC_lexicon_tags_13 },
    { 0xdc31c4cd, SUC_lexicon_tags_15 },
    { 0x46b1c4ce, SUC_lexicon_tags_19 },
    { 0x3fb1c4c9, SUC_lexicon_tags_17 },
    { 0x55f1c4ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2dd1c4d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1371c4d8, SUC_lexicon_tags_27 },
    { 0x831c4d9, SUC_lexicon_tags_25 },
    { 0x2711c4d8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x1251c4dd, SUC_lexicon_tags_38 },
    { 0xd4b1c4de, SUC_lexicon_tags_47 },
    { 0xb551c4dd, SUC_lexicon_tags_16 },
    { 0x8f91c4e0, SUC_lexicon_tags_15 },
    { 0x7ef1c4de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4651c4e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc971c4ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa731c4ed, SUC_lexicon_tags_15 },
    { 0xd651c4ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c4f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed1c4fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d1c4fe, SUC_lexicon_tags_33 },
    { 0x1eb1c4ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5511c503, SUC_lexicon_tags_23 },
    { 0x7a51c504, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf1c50f, SUC_lexicon_tags_44 },
    { 0xdc11c50f, SUC_lexicon_tags_37 },
    { 0x7df1c511, SUC_lexicon_tags_17 },
    { 0x6a11c512, SUC_lexicon_tags_29 },
    { 0x2c71c513, SUC_lexicon_tags_11 },
    { 0x39b1c514, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f1c51a, SUC_lexicon_tags_15 },
    { 0x34f1c51a, SUC_lexicon_tags_37 },
    { 0x3bd1c51c, SUC_lexicon_tags_37 },
    { 0xbd91c51d, SUC_lexicon_tags_37 },
    { 0x731c51e, SUC_lexicon_tags_17 },
    { 0x4511c51f, SUC_lexicon_tags_17 },
    { 0xd231c51e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3591c522, SUC_lexicon_tags_4 },
    { 0x14f1c522, SUC_lexicon_tags_43 },
    { 0x75f1c524, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb1c528, SUC_lexicon_tags_46 },
    { 0x2191c529, SUC_lexicon_tags_25 },
    { 0xed11c529, SUC_lexicon_tags_49 },
    { 0xea31c52b, SUC_lexicon_tags_77 },
    { 0x5971c529, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b1c52f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd651c533, SUC_lexicon_tags_986 },
    { 0x5291c534, SUC_lexicon_tags_37 },
    { 0xe311c535, SUC_lexicon_tags_522 },
    { 0x8e51c536, SUC_lexicon_tags_15 },
    { 0xe011c537, SUC_lexicon_tags_12 },
    { 0xcd11c537, SUC_lexicon_tags_26 },
    { 0x4f51c538, SUC_lexicon_tags_25 },
    { 0x8d51c53a, SUC_lexicon_tags_7 },
    { 0x8df1c53b, SUC_lexicon_tags_66 },
    { 0x2b11c533, SUC_lexicon_tags_11 },
    { 0x1dd1c53a, SUC_lexicon_tags_11 },
    { 0x30f1c537, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5511c540, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6f51c542, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b31c547, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xef31c54a, SUC_lexicon_tags_31 },
    { 0x171c54a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8d91c54d, SUC_lexicon_tags_1 },
    { 0x9fb1c54e, SUC_lexicon_tags_26 },
    { 0x2931c54e, SUC_lexicon_tags_11 },
    { 0x6db1c54d, SUC_lexicon_tags_39 },
    { 0x30f1c54f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1c55a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f51c565, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xea11c568, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf271c56a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xfb31c56c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadd1c56e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe751c573, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf1c57a, SUC_lexicon_tags_29 },
    { 0xf511c57b, SUC_lexicon_tags_116 },
    { 0x2b71c57b, SUC_lexicon_tags_7 },
    { 0x1071c57d, SUC_lexicon_tags_25 },
    { 0x9c31c57e, SUC_lexicon_tags_16 },
    { 0x86b1c57d, SUC_lexicon_tags_15 },
    { 0x2c31c580, SUC_lexicon_tags_23 },
    { 0x4631c581, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3711c584, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd1c587, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x77f1c589, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b1c58c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea31c591, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1591c597, SUC_lexicon_tags_17 },
    { 0x9a11c598, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b1c59d, SUC_lexicon_tags_37 },
    { 0xc7b1c59e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c5a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa51c5a7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3f11c5a9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e71c5ac, SUC_lexicon_tags_63 },
    { 0xa1f1c5ad, SUC_lexicon_tags_47 },
    { 0x66b1c5ae, SUC_lexicon_tags_4 },
    { 0xdfb1c5ad, SUC_lexicon_tags_31 },
    { 0xae31c5b0, SUC_lexicon_tags_61 },
    { 0x25d1c5b1, SUC_lexicon_tags_17 },
    { 0x5a51c5b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6291c5b5, SUC_lexicon_tags_14 },
    { 0xe6b1c5b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc11c5b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1911c5ba, SUC_lexicon_tags_17 },
    { 0x6d91c5bb, SUC_lexicon_tags_32 },
    { 0x7a91c5ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef1c5be, SUC_lexicon_tags_14 },
    { 0x3371c5bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb111c5c4, SUC_lexicon_tags_25 },
    { 0x711c5c5, SUC_lexicon_tags_45 },
    { 0xbab1c5c6, SUC_lexicon_tags_27 },
    { 0x3091c5c7, SUC_lexicon_tags_60 },
    { 0xde11c5c8, SUC_lexicon_tags_37 },
    { 0x1e71c5c7, SUC_lexicon_tags_50 },
    { 0x7d31c5c4, SUC_lexicon_tags_47 },
    { 0x3751c5cb, SUC_lexicon_tags_30 },
    { 0xf451c5cb, SUC_lexicon_tags_11 },
    { 0xbcf1c5cd, SUC_lexicon_tags_25 },
    { 0x1791c5ce, SUC_lexicon_tags_39 },
    { 0x6371c5ce, SUC_lexicon_tags_15 },
    { 0x8b11c5cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc571c5d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7311c5d7, SUC_lexicon_tags_45 },
    { 0x7191c5d8, SUC_lexicon_tags_29 },
    { 0x5331c5d8, SUC_lexicon_tags_37 },
    { 0x2cd1c5d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b1c5dd, SUC_lexicon_tags_47 },
    { 0xe711c5de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa451c5e0, SUC_lexicon_tags_12 },
    { 0x31c5e1, SUC_lexicon_tags_37 },
    { 0x9951c5e1, SUC_lexicon_tags_37 },
    { 0xf131c5e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe831c5eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a91c5f5, SUC_lexicon_tags_33 },
    { 0xf31c5f6, SUC_lexicon_tags_17 },
    { 0x1c11c5f7, SUC_lexicon_tags_85 },
    { 0x7491c5f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf11c5fa, SUC_lexicon_tags_65 },
    { 0x23b1c5fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b1c5ff, SUC_lexicon_tags_15 },
    { 0x9c71c600, SUC_lexicon_tags_77 },
    { 0x3211c5ff, SUC_lexicon_tags_23 },
    { 0xd731c601, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4b1c604, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xae51c606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e71c609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee91c610, SUC_lexicon_tags_2 },
    { 0x6b51c610, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9911c615, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3311c618, SUC_lexicon_tags_25 },
    { 0x8431c618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2131c61c, SUC_lexicon_tags_1 },
    { 0x2b91c61d, SUC_lexicon_tags_171 },
    { 0xfe31c61e, SUC_lexicon_tags_17 },
    { 0x6411c61e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x52f1c621, SUC_lexicon_tags_9 },
    { 0x8ff1c622, SUC_lexicon_tags_17 },
    { 0x9d51c622, SUC_lexicon_tags_17 },
    { 0xf531c623, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39d1c626, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8051c628, SUC_lexicon_tags_14 },
    { 0x6571c629, SUC_lexicon_tags_34 },
    { 0xd231c62a, SUC_lexicon_tags_17 },
    { 0xd551c62a, SUC_lexicon_tags_15 },
    { 0x1bd1c62c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb31c631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af1c635, SUC_lexicon_tags_16 },
    { 0xf3b1c635, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d1c639, SUC_lexicon_tags_43 },
    { 0x3591c63a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f1c63f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb51c642, SUC_lexicon_tags_45 },
    { 0x4e71c643, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5771c645, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b1c64a, SUC_lexicon_tags_7 },
    { 0x5971c64b, SUC_lexicon_tags_37 },
    { 0x5211c64c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f71c64e, SUC_lexicon_tags_37 },
    { 0xdd11c64f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe11c651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a51c654, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf171c657, SUC_lexicon_tags_60 },
    { 0x8971c658, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b1c65e, SUC_lexicon_tags_11 },
    { 0x9591c65f, SUC_lexicon_tags_23 },
    { 0x3591c65f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4411c662, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4711c665, SUC_lexicon_tags_8 },
    { 0xe71c666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba91c669, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c51c66d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31c671, SUC_lexicon_tags_15 },
    { 0xa7b1c672, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x84f1c674, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b1c678, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a31c67b, SUC_lexicon_tags_30 },
    { 0x4251c67b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f1c67f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f1c682, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e71c685, SUC_lexicon_tags_7 },
    { 0xda91c686, SUC_lexicon_tags_14 },
    { 0xa1b1c687, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6ed1c689, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f51c68e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1c691, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f11c694, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1411c698, SUC_lexicon_tags_15 },
    { 0x5431c698, SUC_lexicon_tags_8 },
    { 0xe371c69a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f1c69d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62f1c69f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5011c6a2, SUC_lexicon_tags_25 },
    { 0x4791c6a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9d1c6a5, SUC_lexicon_tags_33 },
    { 0x6071c6a6, SUC_lexicon_tags_1 },
    { 0x1531c6a7, SUC_lexicon_tags_47 },
    { 0x4c31c6a7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x1371c6ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc451c6af, SUC_lexicon_tags_15 },
    { 0x9fb1c6b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1c6b3, SUC_lexicon_tags_30 },
    { 0xa4d1c6b4, SUC_lexicon_tags_33 },
    { 0xafb1c6b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc291c6ba, SUC_lexicon_tags_47 },
    { 0xa3d1c6bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d1c6c1, SUC_lexicon_tags_6 },
    { 0xbcb1c6c2, SUC_lexicon_tags_32 },
    { 0x27b1c6c2, SUC_lexicon_tags_3 },
    { 0xf1d1c6c3, SUC_lexicon_tags_27 },
    { 0x3a91c6c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5971c6c9, SUC_lexicon_tags_23 },
    { 0x9f11c6ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed11c6cc, SUC_lexicon_tags_9 },
    { 0xf71c6cd, SUC_lexicon_tags_60 },
    { 0xbbf1c6ce, SUC_lexicon_tags_87 },
    { 0x4c11c6cf, SUC_lexicon_tags_49 },
    { 0xe311c6d0, SUC_lexicon_tags_60 },
    { 0x4051c6cc, SUC_lexicon_tags_84 },
    { 0x98b1c6d0, SUC_lexicon_tags_12 },
    { 0x4691c6cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3931c6d9, SUC_lexicon_tags_35 },
    { 0x8791c6da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e71c6dc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8351c6e3, SUC_lexicon_tags_12 },
    { 0xe0d1c6e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5b91c6e6, SUC_lexicon_tags_0 },
    { 0xaa91c6e7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1c6ea, SUC_lexicon_tags_17 },
    { 0xec71c6eb, SUC_lexicon_tags_28 },
    { 0xa951c6ec, SUC_lexicon_tags_27 },
    { 0x2fd1c6ec, SUC_lexicon_tags_5 },
    { 0x3bd1c6ee, SUC_lexicon_tags_12 },
    { 0x2571c6ed, SUC_lexicon_tags_2 },
    { 0xd7b1c6ec, SUC_lexicon_tags_15 },
    { 0xa311c6ee, SUC_lexicon_tags_1 },
    { 0xa711c6f2, SUC_lexicon_tags_12 },
    { 0x5551c6f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4431c6f6, SUC_lexicon_tags_25 },
    { 0xb731c6f7, SUC_lexicon_tags_334 },
    { 0x31f1c6f8, SUC_lexicon_tags_37 },
    { 0xd2d1c6f7, SUC_lexicon_tags_63 },
    { 0xc831c6fa, SUC_lexicon_tags_29 },
    { 0xe811c6f7, SUC_lexicon_tags_27 },
    { 0x6cd1c6fc, SUC_lexicon_tags_17 },
    { 0x3471c6fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d1c700, SUC_lexicon_tags_92 },
    { 0xbbd1c700, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b71c707, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcff1c709, SUC_lexicon_tags_11 },
    { 0xfd11c709, SUC_lexicon_tags_37 },
    { 0x6f1c70b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7391c70d, SUC_lexicon_tags_16 },
    { 0x4f71c70e, SUC_lexicon_tags_47 },
    { 0xc471c70d, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0xb211c711, SUC_lexicon_tags_33 },
    { 0x2ef1c712, SUC_lexicon_tags_84 },
    { 0x4ed1c713, SUC_lexicon_tags_30 },
    { 0x3d1c714, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e71c717, SUC_lexicon_tags_7 },
    { 0xe451c718, SUC_lexicon_tags_7 },
    { 0xfad1c717, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb111c71c, SUC_lexicon_tags_63 },
    { 0x1a11c71c, SUC_lexicon_tags_61 },
    { 0xc91c71c, SUC_lexicon_tags_25 },
    { 0xa971c71f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d1c724, SUC_lexicon_tags_17 },
    { 0x65f1c724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc11c728, SUC_lexicon_tags_47 },
    { 0xbd1c729, SUC_lexicon_tags_11 },
    { 0x9891c729, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5551c735, SUC_lexicon_tags_23 },
    { 0xcfb1c735, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d1c73a, SUC_lexicon_tags_37 },
    { 0xae31c73b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x35b1c73d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c31c741, SUC_lexicon_tags_38 },
    { 0x4211c741, SUC_lexicon_tags_17 },
    { 0x3431c742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c91c750, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc091c752, SUC_lexicon_tags_31 },
    { 0xeb31c753, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f31c756, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f51c760, SUC_lexicon_tags_7 },
    { 0x7711c760, SUC_lexicon_tags_6 },
    { 0x7af1c762, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e91c766, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xca91c768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b1c76e, SUC_lexicon_tags_37 },
    { 0xc551c76f, SUC_lexicon_tags_26 },
    { 0x3e91c76f, SUC_lexicon_tags_27 },
    { 0x4bb1c770, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f51c776, SUC_lexicon_tags_49 },
    { 0x2131c777, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8771c77a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5431c77c, SUC_lexicon_tags_2 },
    { 0xb6d1c77c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a11c786, SUC_lexicon_tags_79 },
    { 0x8591c786, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc251c789, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc971c78c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4451c790, SUC_lexicon_tags_27 },
    { 0xd191c791, SUC_lexicon_tags_33 },
    { 0xdfb1c792, SUC_lexicon_tags_37 },
    { 0xd191c791, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5871c79a, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0xc91c79c, SUC_lexicon_tags_27 },
    { 0x1fd1c79d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30b1c79f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6471c7a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3131c7a3, SUC_lexicon_tags_28 },
    { 0x6371c7a4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d1c7a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2451c7a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd1c7ac, SUC_lexicon_tags_17 },
    { 0x85f1c7ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf691c7af, SUC_lexicon_tags_19 },
    { 0xa6b1c7af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa831c7b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa171c7b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9f1c7b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef1c7bf, SUC_lexicon_tags_7 },
    { 0xd7b1c7bf, SUC_lexicon_tags_68 },
    { 0xa6d1c7c1, SUC_lexicon_tags_17 },
    { 0x18f1c7c2, SUC_lexicon_tags_7 },
    { 0x1471c7c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f1c7ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1891c7d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f1c7d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb1c7d9, SUC_lexicon_tags_63 },
    { 0x8bd1c7da, SUC_lexicon_tags_17 },
    { 0xf191c7db, SUC_lexicon_tags_11 },
    { 0xed11c7da, SUC_lexicon_tags_101 },
    { 0xd711c7dc, SUC_lexicon_tags_48 },
    { 0x3f31c7db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb691c7e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e91c7e2, SUC_lexicon_tags_17 },
    { 0x9071c7e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1c7e8, SUC_lexicon_tags_10 },
    { 0x6d11c7e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9751c7ec, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0xe391c7ef, SUC_lexicon_tags_21 },
    { 0xd6f1c7f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3311c7fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e71c7fe, SUC_lexicon_tags_6 },
    { 0xbc51c7ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d11c802, SUC_lexicon_tags_23 },
    { 0x9491c803, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f1c807, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9631c80b, SUC_lexicon_tags_84 },
    { 0x5711c80c, SUC_lexicon_tags_11 },
    { 0x9571c80d, SUC_lexicon_tags_25 },
    { 0xe8b1c80e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3511c810, SUC_lexicon_tags_65 },
    { 0xb2d1c811, SUC_lexicon_tags_1 },
    { 0xb6b1c812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe851c816, SUC_lexicon_tags_7 },
    { 0x1f71c817, SUC_lexicon_tags_15 },
    { 0xe431c816, SUC_lexicon_tags_11 },
    { 0xc8d1c817, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce11c822, SUC_lexicon_tags_25 },
    { 0x1bf1c822, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8db1c825, SUC_lexicon_tags_17 },
    { 0xbb51c826, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5751c829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d71c831, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4211c834, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1431c839, SUC_lexicon_tags_11 },
    { 0x4831c83a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bb1c83c, SUC_lexicon_tags_48 },
    { 0xebb1c83c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e11c840, SUC_lexicon_tags_29 },
    { 0x7e51c841, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xb091c844, SUC_lexicon_tags_27 },
    { 0x42d1c844, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9651c848, SUC_lexicon_tags_6 },
    { 0xde91c848, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c71c84c, SUC_lexicon_tags_38 },
    { 0xcb91c84d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74d1c84f, SUC_lexicon_tags_25 },
    { 0x75f1c850, SUC_lexicon_tags_7 },
    { 0xf0b1c851, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e91c853, SUC_lexicon_tags_11 },
    { 0xaeb1c853, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x24d1c856, SUC_lexicon_tags_25 },
    { 0xb71c856, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f1c85b, SUC_lexicon_tags_21 },
    { 0xd691c85c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd1c85f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d1c864, SUC_lexicon_tags_38 },
    { 0x7471c864, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd831c86b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb611c86d, SUC_lexicon_tags_15 },
    { 0x72f1c86e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31c871, SUC_lexicon_tags_60 },
    { 0xd71c871, SUC_lexicon_tags_37 },
    { 0xacf1c873, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xea11c875, SUC_lexicon_tags_17 },
    { 0xcbf1c876, SUC_lexicon_tags_19 },
    { 0x4771c877, SUC_lexicon_tags_4 },
    { 0x35d1c877, SUC_lexicon_tags_47 },
    { 0xe4d1c879, SUC_lexicon_tags_37 },
    { 0xaa71c878, SUC_lexicon_tags_60 },
    { 0xa211c87b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1131c883, SUC_lexicon_tags_8 },
    { 0x4771c884, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f1c887, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b1c890, SUC_lexicon_tags_1 },
    { 0xb91c891, SUC_lexicon_tags_47 },
    { 0x1271c892, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9f1c894, SUC_lexicon_tags_7 },
    { 0x3e51c895, SUC_lexicon_tags_17 },
    { 0x97b1c896, SUC_lexicon_tags_0 },
    { 0x52b1c896, SUC_lexicon_tags_103 },
    { 0xe3b1c898, SUC_lexicon_tags_19 },
    { 0x37f1c898, SUC_lexicon_tags_24 },
    { 0x75b1c898, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d51c8a3, SUC_lexicon_tags_15 },
    { 0xc731c8a4, SUC_lexicon_tags_15 },
    { 0xd5d1c8a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d11c8ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41b1c8ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b1c8b1, SUC_lexicon_tags_33 },
    { 0x9ad1c8b2, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d1c8b8, SUC_lexicon_tags_10 },
    { 0xad51c8b9, SUC_lexicon_tags_14 },
    { 0x1b91c8b8, SUC_lexicon_tags_0 },
    { 0x8151c8bb, SUC_lexicon_tags_15 },
    { 0xc911c8bb, SUC_lexicon_tags_0 },
    { 0x12d1c8bd, SUC_lexicon_tags_29 },
    { 0x27b1c8b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91c8c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a31c8c2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11c8c5, SUC_lexicon_tags_2 },
    { 0x7531c8c5, SUC_lexicon_tags_17 },
    { 0x1cf1c8c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a51c8c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d71c8cc, SUC_lexicon_tags_45 },
    { 0x9e71c8cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb991c8d1, SUC_lexicon_tags_17 },
    { 0xc6f1c8d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31d1c8d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xad51c8d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f1c8da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4111c8dc, SUC_lexicon_tags_37 },
    { 0x69b1c8dd, SUC_lexicon_tags_9 },
    { 0x3531c8dc, SUC_lexicon_tags_47 },
    { 0x9f91c8df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe071c8e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa671c8e8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1c8eb, SUC_lexicon_tags_2 },
    { 0x7bd1c8ec, SUC_lexicon_tags_11 },
    { 0xe5f1c8ec, SUC_lexicon_tags_15 },
    { 0x3c71c8ee, SUC_lexicon_tags_25 },
    { 0xc911c8ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa71c8f7, SUC_lexicon_tags_1 },
    { 0x911c8f8, SUC_lexicon_tags_2 },
    { 0x1e91c8f9, SUC_lexicon_tags_11 },
    { 0x9671c8f7, SUC_lexicon_tags_45 },
    { 0xbab1c8fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d1c8fe, SUC_lexicon_tags_17 },
    { 0xbd91c8ff, SUC_lexicon_tags_29 },
    { 0x9271c8ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7951c906, SUC_lexicon_tags_3 },
    { 0x6c71c907, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f1c90c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b1c910, SUC_lexicon_tags_55 },
    { 0x54f1c911, SUC_lexicon_tags_46 },
    { 0x4cf1c911, SUC_lexicon_tags_17 },
    { 0x9c51c913, SUC_lexicon_tags_17 },
    { 0xa131c911, SUC_lexicon_tags_7 },
    { 0x2af1c915, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c91c919, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf151c91d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f1c920, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2871c925, SUC_lexicon_tags_28 },
    { 0x8df1c926, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdbd1c928, SUC_lexicon_tags_30 },
    { 0x3cb1c929, SUC_lexicon_tags_16 },
    { 0x951c929, SUC_lexicon_tags_1 },
    { 0x2ab1c92b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf1c930, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1011c932, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d1c935, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe771c937, SUC_lexicon_tags_32 },
    { 0x6d51c937, SUC_lexicon_tags_3 },
    { 0x2ef1c938, SUC_lexicon_tags_13 },
    { 0x8d51c93a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9611c93f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc31c945, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2131c949, SUC_lexicon_tags_55 },
    { 0x971c94a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1c950, SUC_lexicon_tags_12 },
    { 0x9771c951, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7951c954, SUC_lexicon_tags_45 },
    { 0x7ad1c954, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b1c95a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xad1c95c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf851c95e, SUC_lexicon_tags_62 },
    { 0x851c95f, SUC_lexicon_tags_37 },
    { 0xfb51c960, SUC_lexicon_tags_49 },
    { 0xe131c961, SUC_lexicon_tags_12 },
    { 0xd151c962, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9371c966, SUC_lexicon_tags_28 },
    { 0x39d1c967, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb71c96c, SUC_lexicon_tags_1 },
    { 0x9d51c96d, SUC_lexicon_tags_624 },
    { 0xe0f1c96d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d1c971, SUC_lexicon_tags_17 },
    { 0x5271c972, SUC_lexicon_tags_79 },
    { 0x211c971, SUC_lexicon_tags_95 },
    { 0x4db1c974, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1c979, SUC_lexicon_tags_17 },
    { 0x2f11c979, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xab1c97c, SUC_lexicon_tags_17 },
    { 0xc351c97c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x8831c980, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc611c983, SUC_lexicon_tags_11 },
    { 0x89b1c984, SUC_lexicon_tags_37 },
    { 0xfd91c983, SUC_lexicon_tags_1 },
    { 0x2531c986, SUC_lexicon_tags_47 },
    { 0xe731c984, SUC_lexicon_tags_26 },
    { 0xbd1c988, SUC_lexicon_tags_17 },
    { 0x6911c989, SUC_lexicon_tags_2 },
    { 0x6ff1c98a, SUC_lexicon_tags_10 },
    { 0xdb1c98b, SUC_lexicon_tags_15 },
    { 0x8f31c98c, SUC_lexicon_tags_47 },
    { 0xce91c98b, SUC_lexicon_tags_33 },
    { 0x1bf1c98e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2191c990, SUC_lexicon_tags_43 },
    { 0xced1c991, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2eb1c993, SUC_lexicon_tags_28 },
    { 0xe1d1c993, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcd1c996, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad71c998, SUC_lexicon_tags_1 },
    { 0x5371c999, SUC_lexicon_tags_25 },
    { 0x8fb1c999, SUC_lexicon_tags_68 },
    { 0xca91c99b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad1c99e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x66b1c9a0, SUC_lexicon_tags_8 },
    { 0x4471c9a1, SUC_lexicon_tags_65 },
    { 0xeb11c9a2, SUC_lexicon_tags_25 },
    { 0xb6d1c9a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb51c9a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd931c9ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6611c9b0, SUC_lexicon_tags_77 },
    { 0xaaf1c9b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2891c9b3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5611c9b6, SUC_lexicon_tags_17 },
    { 0x4451c9b7, SUC_lexicon_tags_49 },
    { 0x7411c9b8, SUC_lexicon_tags_14 },
    { 0x30d1c9b9, SUC_lexicon_tags_12 },
    { 0x69d1c9ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f1c9be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4371c9c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8711c9c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4091c9c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5691c9ca, SUC_lexicon_tags_11 },
    { 0xf8b1c9cb, SUC_lexicon_tags_56 },
    { 0xf111c9cc, SUC_lexicon_tags_25 },
    { 0x8b31c9cb, SUC_lexicon_tags_17 },
    { 0x9031c9cd, SUC_lexicon_tags_45 },
    { 0xf031c9cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc11c9d2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf691c9d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9151c9d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f1c9de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80d1c9e0, SUC_lexicon_tags_37 },
    { 0x7191c9e1, SUC_lexicon_tags_87 },
    { 0x7531c9e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaf1c9e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa171c9e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7771c9eb, SUC_lexicon_tags_43 },
    { 0xe391c9eb, SUC_lexicon_tags_1 },
    { 0x211c9ed, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6491c9f5, SUC_lexicon_tags_37 },
    { 0x3e51c9f6, SUC_lexicon_tags_16 },
    { 0x2e51c9f7, SUC_lexicon_tags_26 },
    { 0x2191c9f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5331c9fb, SUC_lexicon_tags_11 },
    { 0x5511c9fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe171c9fe, SUC_lexicon_tags_47 },
    { 0xf491c9fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d1ca06, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8751ca0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb151ca0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1451ca15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3631ca18, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x34b1ca1a, SUC_lexicon_tags_47 },
    { 0x4311ca1b, SUC_lexicon_tags_17 },
    { 0xa151ca1c, SUC_lexicon_tags_7 },
    { 0xea51ca1c, SUC_lexicon_tags_15 },
    { 0x6211ca1e, SUC_lexicon_tags_25 },
    { 0xda51ca1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f1ca22, SUC_lexicon_tags_60 },
    { 0xa471ca23, SUC_lexicon_tags_17 },
    { 0x37b1ca23, SUC_lexicon_tags_42 },
    { 0x2b51ca24, SUC_lexicon_tags_12 },
    { 0xc911ca22, SUC_lexicon_tags_48 },
    { 0x5b11ca27, SUC_lexicon_tags_11 },
    { 0x2191ca26, SUC_lexicon_tags_112 },
    { 0x7cf1ca29, SUC_lexicon_tags_25 },
    { 0xf571ca2a, SUC_lexicon_tags_64 },
    { 0x2f31ca29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa691ca30, SUC_lexicon_tags_34 },
    { 0x6c91ca31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xec91ca34, SUC_lexicon_tags_15 },
    { 0xd8b1ca34, SUC_lexicon_tags_11 },
    { 0x7731ca36, SUC_lexicon_tags_7 },
    { 0x9231ca37, SUC_lexicon_tags_60 },
    { 0xeff1ca34, SUC_lexicon_tags_11 },
    { 0x271ca34, SUC_lexicon_tags_63 },
    { 0x5571ca3a, SUC_lexicon_tags_38 },
    { 0x851ca3a, SUC_lexicon_tags_17 },
    { 0x2f11ca3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf91ca3e, SUC_lexicon_tags_17 },
    { 0x6db1ca3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35b1ca41, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d1ca44, SUC_lexicon_tags_17 },
    { 0xebd1ca45, SUC_lexicon_tags_15 },
    { 0x4231ca46, SUC_lexicon_tags_37 },
    { 0xf031ca46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591ca4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfff1ca4d, SUC_lexicon_tags_17 },
    { 0x5831ca4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfc71ca50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafb1ca52, SUC_lexicon_tags_17 },
    { 0x7b51ca52, SUC_lexicon_tags_14 },
    { 0x9191ca52, SUC_lexicon_tags_37 },
    { 0x3ef1ca55, SUC_lexicon_tags_12 },
    { 0x80f1ca56, SUC_lexicon_tags_16 },
    { 0xf151ca57, SUC_lexicon_tags_39 },
    { 0x53f1ca58, SUC_lexicon_tags_38 },
    { 0xc151ca59, SUC_lexicon_tags_65 },
    { 0x4951ca59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f1ca5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf831ca66, SUC_lexicon_tags_60 },
    { 0xdb71ca67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x311ca69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa5f1ca6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d1ca70, SUC_lexicon_tags_13 },
    { 0x42d1ca71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1351ca76, SUC_lexicon_tags_19 },
    { 0x41d1ca77, SUC_lexicon_tags_5 },
    { 0x82f1ca76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3251ca7c, SUC_lexicon_tags_15 },
    { 0x6a91ca7d, SUC_lexicon_tags_55 },
    { 0xdd51ca7c, SUC_lexicon_tags_37 },
    { 0x32d1ca7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7411ca81, SUC_lexicon_tags_15 },
    { 0x6091ca82, SUC_lexicon_tags_15 },
    { 0x771ca82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f1ca90, SUC_lexicon_tags_7 },
    { 0xe5f1ca91, SUC_lexicon_tags_9 },
    { 0x1dd1ca91, SUC_lexicon_tags_11 },
    { 0xf1d1ca93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b1ca96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf1ca99, SUC_lexicon_tags_7 },
    { 0x2231ca9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4171ca9f, SUC_lexicon_tags_39 },
    { 0x5151ca9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74b1caa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2531caa4, SUC_lexicon_tags_17 },
    { 0x2231caa5, SUC_lexicon_tags_2 },
    { 0x8d51caa6, SUC_lexicon_tags_16 },
    { 0x8751caa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf1caaa, SUC_lexicon_tags_37 },
    { 0x66d1caab, SUC_lexicon_tags_30 },
    { 0xe8d1caac, SUC_lexicon_tags_17 },
    { 0xf1f1caad, SUC_lexicon_tags_11 },
    { 0x3b51caae, SUC_lexicon_tags_15 },
    { 0xd411caaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3151cab5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f71cab8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd1cac1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc5d1cac3, SUC_lexicon_tags_3 },
    { 0x70f1cac4, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd031cac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf91cad1, SUC_lexicon_tags_12 },
    { 0xb1f1cad2, SUC_lexicon_tags_26 },
    { 0x9791cad3, SUC_lexicon_tags_45 },
    { 0x6c31cad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9d1cad6, SUC_lexicon_tags_12 },
    { 0x73f1cad7, SUC_lexicon_tags_14 },
    { 0x2071cad8, SUC_lexicon_tags_12 },
    { 0x2031cad7, SUC_lexicon_tags_6 },
    { 0x52b1cad7, SUC_lexicon_tags_91 },
    { 0x2571cadb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7791cadd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b1cae2, SUC_lexicon_tags_11 },
    { 0xe991cae2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a31cae5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd7d1cae7, SUC_lexicon_tags_11 },
    { 0x3171cae7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb91caef, SUC_lexicon_tags_0 },
    { 0x75b1caf0, SUC_lexicon_tags_37 },
    { 0xe951caf1, SUC_lexicon_tags_29 },
    { 0xf371caf1, SUC_lexicon_tags_7 },
    { 0x98f1caf3, SUC_lexicon_tags_12 },
    { 0xfff1caf4, SUC_lexicon_tags_33 },
    { 0x7d31caf4, SUC_lexicon_tags_34 },
    { 0x3511caf6, SUC_lexicon_tags_47 },
    { 0x6151caf7, SUC_lexicon_tags_13 },
    { 0x38f1caf7, SUC_lexicon_tags_15 },
    { 0xbe31caf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f11cb02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b31cb06, SUC_lexicon_tags_27 },
    { 0x8a91cb07, SUC_lexicon_tags_66 },
    { 0x7b31cb07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1cb0b, SUC_lexicon_tags_2 },
    { 0x4af1cb0c, SUC_lexicon_tags_37 },
    { 0x9c91cb0d, SUC_lexicon_tags_25 },
    { 0x9671cb0d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa531cb14, SUC_lexicon_tags_37 },
    { 0x6b11cb15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d1cb19, SUC_lexicon_tags_65 },
    { 0xfb71cb19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2631cb1e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9411cb23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa231cb27, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b51cb2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd131cb31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a91cb36, SUC_lexicon_tags_7 },
    { 0x71d1cb37, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4531cb41, SUC_lexicon_tags_10 },
    { 0xcab1cb42, SUC_lexicon_tags_14 },
    { 0x99b1cb43, SUC_lexicon_tags_37 },
    { 0x2791cb43, SUC_lexicon_tags_0 },
    { 0x2151cb45, SUC_lexicon_tags_144 },
    { 0x51cb45, SUC_lexicon_tags_39 },
    { 0xbbf1cb47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7251cb4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3331cb4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fb1cb4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa871cb56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4051cb61, SUC_lexicon_tags_56 },
    { 0x3f1cb62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e91cb64, SUC_lexicon_tags_11 },
    { 0x69d1cb64, SUC_lexicon_tags_47 },
    { 0xdb31cb66, SUC_lexicon_tags_54 },
    { 0x3191cb65, SUC_lexicon_tags_1 },
    { 0x7451cb68, SUC_lexicon_tags_17 },
    { 0x7931cb68, SUC_lexicon_tags_62 },
    { 0xa2b1cb68, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xcc51cb6c, SUC_lexicon_tags_10 },
    { 0x8011cb6d, SUC_lexicon_tags_12 },
    { 0x6d51cb6d, SUC_lexicon_tags_2 },
    { 0x6271cb6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f1cb73, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d91cb76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11cb80, SUC_lexicon_tags_17 },
    { 0x2311cb80, SUC_lexicon_tags_43 },
    { 0xc311cb81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d1cb85, SUC_lexicon_tags_11 },
    { 0xa751cb85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f11cb8c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb1cb90, SUC_lexicon_tags_8 },
    { 0xa7f1cb90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7971cb94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfe51cb96, SUC_lexicon_tags_38 },
    { 0x951cb96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5351cb9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a51cb9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f71cba7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1cbab, SUC_lexicon_tags_17 },
    { 0x4b51cbab, SUC_lexicon_tags_34 },
    { 0x4191cbab, SUC_lexicon_tags_25 },
    { 0x6111cbae, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4291cbb0, SUC_lexicon_tags_6 },
    { 0x4791cbb1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf211cbb3, SUC_lexicon_tags_33 },
    { 0x7b11cbb4, SUC_lexicon_tags_38 },
    { 0x36d1cbb5, SUC_lexicon_tags_63 },
    { 0xe111cbb3, SUC_lexicon_tags_47 },
    { 0x1ed1cbb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb11cbba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe491cbbc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51cbc0, SUC_lexicon_tags_37 },
    { 0x66f1cbc1, SUC_lexicon_tags_113 },
    { 0xf491cbc2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5591cbc4, SUC_lexicon_tags_37 },
    { 0x1ed1cbc5, SUC_lexicon_tags_15 },
    { 0x2531cbc4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd1cbc9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3911cbcf, SUC_lexicon_tags_599 },
    { 0x7291cbd0, SUC_lexicon_tags_17 },
    { 0x9e91cbd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9531cbd3, SUC_lexicon_tags_61 },
    { 0xbeb1cbd3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e71cbd7, SUC_lexicon_tags_33 },
    { 0x2051cbd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d71cbdb, SUC_lexicon_tags_25 },
    { 0x3fd1cbdc, SUC_lexicon_tags_27 },
    { 0x1bd1cbdd, SUC_lexicon_tags_11 },
    { 0x6bf1cbdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b91cbe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b1cbe8, SUC_lexicon_tags_15 },
    { 0x7691cbe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bd1cbeb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7331cbee, SUC_lexicon_tags_9 },
    { 0x2131cbef, SUC_lexicon_tags_42 },
    { 0x9871cbf0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791cbf4, SUC_lexicon_tags_295 },
    { 0x69f1cbf5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5431cbf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf1cbfc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef1cbff, SUC_lexicon_tags_37 },
    { 0x7e71cbff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcd71cc02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd791cc05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3191cc08, SUC_lexicon_tags_79 },
    { 0x5291cc09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fb1cc0b, SUC_lexicon_tags_7 },
    { 0x5ab1cc0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c31cc0f, SUC_lexicon_tags_12 },
    { 0x9f91cc10, SUC_lexicon_tags_2 },
    { 0x9c51cc10, SUC_lexicon_tags_37 },
    { 0xcdd1cc12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6011cc17, SUC_lexicon_tags_12 },
    { 0xbe51cc17, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7931cc1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe31cc1f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb2d1cc21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6091cc27, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xf211cc29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb191cc30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf691cc32, SUC_lexicon_tags_28 },
    { 0x16d1cc33, SUC_lexicon_tags_60 },
    { 0x6cb1cc32, SUC_lexicon_tags_38 },
    { 0x82b1cc35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed1cc39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9971cc40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf31cc43, SUC_lexicon_tags_0 },
    { 0x2511cc44, SUC_lexicon_tags_11 },
    { 0xd8f1cc45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7351cc47, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe911cc4c, SUC_lexicon_tags_33 },
    { 0xa171cc4d, SUC_lexicon_tags_37 },
    { 0x3171cc4e, SUC_lexicon_tags_2 },
    { 0xf791cc4f, SUC_lexicon_tags_15 },
    { 0xa5b1cc50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1e51cc52, SUC_lexicon_tags_60 },
    { 0x96f1cc52, SUC_lexicon_tags_55 },
    { 0x4611cc54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef91cc5a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f1cc5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad1cc66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74d1cc68, SUC_lexicon_tags_25 },
    { 0xbdf1cc68, SUC_lexicon_tags_2 },
    { 0x7df1cc6a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d1cc72, SUC_lexicon_tags_49 },
    { 0x88b1cc73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d51cc75, SUC_lexicon_tags_14 },
    { 0x3bd1cc75, SUC_lexicon_tags_37 },
    { 0x1271cc77, SUC_lexicon_tags_11 },
    { 0xd1cc77, SUC_lexicon_tags_6 },
    { 0x4b11cc79, SUC_lexicon_tags_65 },
    { 0xa971cc76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1591cc7c, SUC_lexicon_tags_17 },
    { 0xe691cc7d, SUC_lexicon_tags_27 },
    { 0x5331cc7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc351cc88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf91cc8a, SUC_lexicon_tags_37 },
    { 0xde31cc8a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb1cc98, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6371cc9c, SUC_lexicon_tags_0 },
    { 0x3a31cc9d, SUC_lexicon_tags_63 },
    { 0x2291cc9e, SUC_lexicon_tags_19 },
    { 0xe7f1cc9c, SUC_lexicon_tags_39 },
    { 0x7491cca0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xf531cca3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa751ccac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc851ccaf, SUC_lexicon_tags_17 },
    { 0x2591ccaf, SUC_lexicon_tags_7 },
    { 0x7d11ccb1, SUC_lexicon_tags_11 },
    { 0xd071ccb0, SUC_lexicon_tags_7 },
    { 0x771ccb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7911ccb9, SUC_lexicon_tags_45 },
    { 0x82f1ccba, SUC_lexicon_tags_10 },
    { 0x2ef1ccba, SUC_lexicon_tags_14 },
    { 0xa171ccbc, SUC_lexicon_tags_7 },
    { 0xef11ccbc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8711ccbf, SUC_lexicon_tags_17 },
    { 0xfe71ccc0, SUC_lexicon_tags_9 },
    { 0xc6b1ccc1, SUC_lexicon_tags_27 },
    { 0xfe11ccc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4351ccc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa71cccc, SUC_lexicon_tags_11 },
    { 0x6e11cccd, SUC_lexicon_tags_25 },
    { 0x171ccce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b11ccd0, SUC_lexicon_tags_8 },
    { 0xb0b1ccd0, SUC_lexicon_tags_12 },
    { 0x7611ccd0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5a71ccd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc251ccd7, SUC_lexicon_tags_37 },
    { 0xb0b1ccd8, SUC_lexicon_tags_21 },
    { 0xf6f1ccd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2791ccdb, SUC_lexicon_tags_2 },
    { 0x3511ccdb, SUC_lexicon_tags_0 },
    { 0xf051ccdd, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x6371ccdf, SUC_lexicon_tags_7 },
    { 0xd711cce0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5351cce2, SUC_lexicon_tags_88 },
    { 0x9851cce3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3231cce8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1251ccf0, SUC_lexicon_tags_10 },
    { 0xc0d1ccf0, SUC_lexicon_tags_1 },
    { 0xb4b1ccf2, SUC_lexicon_tags_21 },
    { 0x7471ccf2, SUC_lexicon_tags_19 },
    { 0x1f1ccf3, SUC_lexicon_tags_21 },
    { 0x5951ccf5, SUC_lexicon_tags_6 },
    { 0x5031ccf6, SUC_lexicon_tags_1 },
    { 0x2831ccf7, SUC_lexicon_tags_47 },
    { 0xf331ccf8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f1ccfd, SUC_lexicon_tags_12 },
    { 0xf331ccfd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f1cd03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa851cd05, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7011cd09, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf711cd0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda91cd0f, SUC_lexicon_tags_31 },
    { 0x8cf1cd10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7571cd14, SUC_lexicon_tags_1 },
    { 0xbab1cd14, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d1cd1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b1cd27, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x69f1cd29, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd371cd2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda71cd2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2211cd32, SUC_lexicon_tags_15 },
    { 0x6b1cd32, SUC_lexicon_tags_43 },
    { 0xaf31cd34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f1cd38, SUC_lexicon_tags_0 },
    { 0xbd1cd38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11cd3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8891cd44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8f11cd46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ff1cd48, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f1cd4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f1cd50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa91cd56, SUC_lexicon_tags_47 },
    { 0x8e51cd57, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa31cd5b, SUC_lexicon_tags_17 },
    { 0x7191cd5b, SUC_lexicon_tags_23 },
    { 0x3811cd5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb351cd60, SUC_lexicon_tags_63 },
    { 0x82f1cd61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1331cd67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc311cd69, SUC_lexicon_tags_15 },
    { 0x6751cd6a, SUC_lexicon_tags_37 },
    { 0xaf1cd6b, SUC_lexicon_tags_29 },
    { 0xe51cd6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24f1cd6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb3b1cd70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4331cd74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc231cd76, SUC_lexicon_tags_17 },
    { 0x8391cd77, SUC_lexicon_tags_37 },
    { 0xdad1cd76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b1cd7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b1cd7e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6cf1cd80, SUC_lexicon_tags_80 },
    { 0x7c51cd80, SUC_lexicon_tags_27 },
    { 0xac31cd80, SUC_lexicon_tags_37 },
    { 0xfb31cd83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe31cd88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd1cd8b, SUC_lexicon_tags_19 },
    { 0xf811cd8c, SUC_lexicon_tags_37 },
    { 0x70f1cd8d, SUC_lexicon_tags_47 },
    { 0xd4f1cd8d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71cd95, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d1cda1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1571cda3, SUC_lexicon_tags_25 },
    { 0x1ad1cda4, SUC_lexicon_tags_16 },
    { 0x1e91cda5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa351cda8, SUC_lexicon_tags_17 },
    { 0x7271cda8, SUC_lexicon_tags_23 },
    { 0xfd51cda8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d1cdae, SUC_lexicon_tags_11 },
    { 0xd091cdaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e71cdb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9251cdb5, SUC_lexicon_tags_47 },
    { 0x9d91cdb5, SUC_lexicon_tags_7 },
    { 0xeaf1cdb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc71cdba, SUC_lexicon_tags_17 },
    { 0xd371cdbb, SUC_lexicon_tags_17 },
    { 0x5431cdba, SUC_lexicon_tags_2 },
    { 0xb91cdbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb811cdbf, SUC_lexicon_tags_149 },
    { 0xb311cdc0, SUC_lexicon_tags_23 },
    { 0xcb51cdc0, SUC_lexicon_tags_1 },
    { 0x6211cdc2, SUC_lexicon_tags_15 },
    { 0xf031cdc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8b1cdc5, SUC_lexicon_tags_27 },
    { 0x98f1cdc6, SUC_lexicon_tags_7 },
    { 0x7771cdc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6251cdcb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9671cdcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa031cdd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3711cdd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6d1cdd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb171cdd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c71cddb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6531cde0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531cde4, SUC_lexicon_tags_23 },
    { 0xe611cde5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2671cde7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d1cdeb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc371cdf2, SUC_lexicon_tags_55 },
    { 0xae71cdf3, SUC_lexicon_tags_0 },
    { 0x27d1cdf3, SUC_lexicon_tags_13 },
    { 0x28b1cdf5, SUC_lexicon_tags_9 },
    { 0x3f71cdf4, SUC_lexicon_tags_6 },
    { 0x8bb1cdf4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1771cdfa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f1ce01, SUC_lexicon_tags_66 },
    { 0x9811ce01, SUC_lexicon_tags_37 },
    { 0x571ce03, SUC_lexicon_tags_25 },
    { 0xc8f1ce03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9d1ce06, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8771ce0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1ce14, SUC_lexicon_tags_25 },
    { 0x1cd1ce15, SUC_lexicon_tags_14 },
    { 0xce91ce16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e91ce1b, SUC_lexicon_tags_12 },
    { 0x59f1ce1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f71ce1e, SUC_lexicon_tags_37 },
    { 0xeeb1ce1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af1ce22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa691ce26, SUC_lexicon_tags_2 },
    { 0x4c91ce27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa571ce2a, SUC_lexicon_tags_37 },
    { 0x2b1ce2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3991ce32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe271ce35, SUC_lexicon_tags_35 },
    { 0xc1b1ce36, SUC_lexicon_tags_23 },
    { 0xa691ce37, SUC_lexicon_tags_25 },
    { 0x4b51ce38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2491ce3a, SUC_lexicon_tags_34 },
    { 0x2bd1ce3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1ce3f, SUC_lexicon_tags_13 },
    { 0x7b71ce40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f1ce43, SUC_lexicon_tags_14 },
    { 0xf751ce43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ad1ce46, SUC_lexicon_tags_38 },
    { 0xd791ce47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad1ce4a, SUC_lexicon_tags_60 },
    { 0x86f1ce4b, SUC_lexicon_tags_15 },
    { 0x9ab1ce4a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31ce4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9511ce54, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed1ce59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f31ce60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e31ce67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9451ce6b, SUC_lexicon_tags_4 },
    { 0xc691ce6c, SUC_lexicon_tags_39 },
    { 0x38f1ce6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd71ce70, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff1ce75, SUC_lexicon_tags_25 },
    { 0x6b31ce75, SUC_lexicon_tags_7 },
    { 0xa1b1ce77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ce7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca11ce81, SUC_lexicon_tags_2 },
    { 0xe671ce82, SUC_lexicon_tags_29 },
    { 0x9a71ce83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf31ce87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c31ce8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfad1ce8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4911ce90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f11ce94, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc531ce99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc51ce9e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe51cea6, SUC_lexicon_tags_15 },
    { 0x4631cea6, SUC_lexicon_tags_1510 },
    { 0, NULL },
    { 0x32d1cea9, SUC_lexicon_tags_38 },
    { 0xcd11ceaa, SUC_lexicon_tags_27 },
    { 0x7811ceaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb1ceae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db1ceb6, SUC_lexicon_tags_11 },
    { 0x3d71ceb6, SUC_lexicon_tags_29 },
    { 0x12b1ceb8, SUC_lexicon_tags_17 },
    { 0x8cf1ceb6, SUC_lexicon_tags_13 },
    { 0x1791ceba, SUC_lexicon_tags_17 },
    { 0x8e91cebb, SUC_lexicon_tags_45 },
    { 0x9ff1cebc, SUC_lexicon_tags_6 },
    { 0xedf1cebc, SUC_lexicon_tags_47 },
    { 0x9171cebe, SUC_lexicon_tags_39 },
    { 0x7131cebe, SUC_lexicon_tags_1 },
    { 0xb5b1cebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2f1cec2, SUC_lexicon_tags_12 },
    { 0x5df1cec2, SUC_lexicon_tags_2 },
    { 0x7471cec4, SUC_lexicon_tags_179 },
    { 0xfd51cec5, SUC_lexicon_tags_9 },
    { 0xa5b1cec6, SUC_lexicon_tags_37 },
    { 0x8f91cec6, SUC_lexicon_tags_47 },
    { 0x7011cec8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1a71ceca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc531cecc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa1d1cece, SUC_lexicon_tags_49 },
    { 0xafb1cecf, SUC_lexicon_tags_37 },
    { 0x7c51ced0, SUC_lexicon_tags_1 },
    { 0xb1f1cece, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa351ced3, SUC_lexicon_tags_55 },
    { 0xf51ced4, SUC_lexicon_tags_14 },
    { 0x1051ced4, SUC_lexicon_tags_8 },
    { 0x9271ced6, SUC_lexicon_tags_48 },
    { 0x14b1ced7, SUC_lexicon_tags_11 },
    { 0xdf11ced4, SUC_lexicon_tags_11 },
    { 0xfa51ced9, SUC_lexicon_tags_14 },
    { 0xe111ced7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f1cee0, SUC_lexicon_tags_0 },
    { 0xfdf1cee1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x331cee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2571cee6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa271ceec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x70f1ceee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8531cef0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x81d1cef2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xee71cef4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc211cef6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df1cef9, SUC_lexicon_tags_19 },
    { 0x38d1cefa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fb1cefc, SUC_lexicon_tags_25 },
    { 0x3d31cefd, SUC_lexicon_tags_21 },
    { 0x9511cefe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc911cf01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd1cf06, SUC_lexicon_tags_60 },
    { 0x3091cf07, SUC_lexicon_tags_15 },
    { 0xb751cf08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8851cf0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f91cf0d, SUC_lexicon_tags_1 },
    { 0xb511cf0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad1cf15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f71cf18, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f1cf20, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc11cf24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4731cf29, SUC_lexicon_tags_29 },
    { 0xe211cf2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d1cf2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x73b1cf30, SUC_lexicon_tags_12 },
    { 0xe711cf31, SUC_lexicon_tags_6 },
    { 0x4d31cf32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff11cf37, SUC_lexicon_tags_218 },
    { 0xc91cf37, SUC_lexicon_tags_59 },
    { 0xccd1cf37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f1cf3f, SUC_lexicon_tags_17 },
    { 0x43d1cf3f, SUC_lexicon_tags_2 },
    { 0x7bf1cf41, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x28d1cf43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb1cf48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x67f1cf4a, SUC_lexicon_tags_40 },
    { 0x7a91cf4b, SUC_lexicon_tags_25 },
    { 0x4511cf4b, SUC_lexicon_tags_17 },
    { 0x3bb1cf4d, SUC_lexicon_tags_37 },
    { 0x9c51cf4e, SUC_lexicon_tags_63 },
    { 0xf611cf4c, SUC_lexicon_tags_46 },
    { 0xcdf1cf50, SUC_lexicon_tags_37 },
    { 0xb391cf4b, SUC_lexicon_tags_57 },
    { 0x651cf4f, SUC_lexicon_tags_15 },
    { 0xdf11cf4f, SUC_lexicon_tags_2 },
    { 0x3f1cf54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa91cf58, SUC_lexicon_tags_8 },
    { 0xf451cf58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb51cf5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70b1cf5d, SUC_lexicon_tags_63 },
    { 0xa8d1cf5d, SUC_lexicon_tags_60 },
    { 0xa5b1cf5e, SUC_lexicon_tags_48 },
    { 0xd511cf60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b91cf63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f1cf66, SUC_lexicon_tags_262 },
    { 0x2e91cf67, SUC_lexicon_tags_16 },
    { 0x2eb1cf67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8431cf72, SUC_lexicon_tags_17 },
    { 0x3df1cf73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2671cf75, SUC_lexicon_tags_0 },
    { 0xecd1cf76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1cf7e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f1cf81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba11cf89, SUC_lexicon_tags_29 },
    { 0x8871cf89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f31cf8c, SUC_lexicon_tags_7 },
    { 0xef31cf8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe431cf91, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xf411cf93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9291cf95, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e51cf9a, SUC_lexicon_tags_15 },
    { 0xd231cf9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d1cf9e, SUC_lexicon_tags_19 },
    { 0x6671cf9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9571cfa1, SUC_lexicon_tags_11 },
    { 0x4691cfa2, SUC_lexicon_tags_25 },
    { 0x9571cfa1, SUC_lexicon_tags_14 },
    { 0x3551cfa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe371cfa8, SUC_lexicon_tags_17 },
    { 0x6571cfa9, SUC_lexicon_tags_14 },
    { 0xaaf1cfaa, SUC_lexicon_tags_15 },
    { 0x37b1cfaa, SUC_lexicon_tags_37 },
    { 0x84f1cfac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2071cfae, SUC_lexicon_tags_38 },
    { 0x4691cfaf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6191cfb2, SUC_lexicon_tags_25 },
    { 0x6fd1cfb3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d1cfb8, SUC_lexicon_tags_25 },
    { 0x81f1cfb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf11cfbb, SUC_lexicon_tags_17 },
    { 0x65d1cfbb, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x14d1cfbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5771cfc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b1cfc5, SUC_lexicon_tags_25 },
    { 0x19b1cfc6, SUC_lexicon_tags_1 },
    { 0xbc31cfc7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x46b1cfc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2471cfcd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xedf1cfcf, SUC_lexicon_tags_0 },
    { 0x3c11cfd0, SUC_lexicon_tags_25 },
    { 0x63b1cfd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbff1cfd3, SUC_lexicon_tags_26 },
    { 0x3e91cfd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ef1cfd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631cfda, SUC_lexicon_tags_1 },
    { 0x9471cfdb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f1cfe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d1cfe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe51cfea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa891cfee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc551cff3, SUC_lexicon_tags_2 },
    { 0xc091cff4, SUC_lexicon_tags_12 },
    { 0x7cd1cff5, SUC_lexicon_tags_15 },
    { 0x51d1cff4, SUC_lexicon_tags_29 },
    { 0x3571cff6, SUC_lexicon_tags_40 },
    { 0xf691cff8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x4e71cffa, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6bb1cffc, SUC_lexicon_tags_1 },
    { 0x9cd1cffc, SUC_lexicon_tags_129 },
    { 0xec71cffe, SUC_lexicon_tags_17 },
    { 0x48f1cfff, SUC_lexicon_tags_16 },
    { 0x7611d000, SUC_lexicon_tags_49 },
    { 0xbed1d001, SUC_lexicon_tags_11 },
    { 0x6a91cffd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd631d00f, SUC_lexicon_tags_21 },
    { 0x8031d010, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4031d017, SUC_lexicon_tags_26 },
    { 0x53d1d017, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc331d01b, SUC_lexicon_tags_45 },
    { 0x4811d01b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22f1d01e, SUC_lexicon_tags_1 },
    { 0xf531d01e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd1d023, SUC_lexicon_tags_17 },
    { 0x6011d023, SUC_lexicon_tags_15 },
    { 0x5691d025, SUC_lexicon_tags_8 },
    { 0x49f1d026, SUC_lexicon_tags_17 },
    { 0xe91d026, SUC_lexicon_tags_7 },
    { 0xbd71d023, SUC_lexicon_tags_37 },
    { 0x6731d029, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa9b1d02b, SUC_lexicon_tags_92 },
    { 0xbff1d02c, SUC_lexicon_tags_19 },
    { 0xcd11d02c, SUC_lexicon_tags_12 },
    { 0x6fb1d02e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f1d034, SUC_lexicon_tags_28 },
    { 0x85b1d035, SUC_lexicon_tags_65 },
    { 0xb2b1d035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb71d038, SUC_lexicon_tags_37 },
    { 0x4ff1d039, SUC_lexicon_tags_26 },
    { 0xf131d03a, SUC_lexicon_tags_27 },
    { 0x65f1d03a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa751d040, SUC_lexicon_tags_48 },
    { 0x1d1d041, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa991d043, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x4471d046, SUC_lexicon_tags_11 },
    { 0xff11d047, SUC_lexicon_tags_37 },
    { 0x4ed1d048, SUC_lexicon_tags_23 },
    { 0xe3f1d048, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3611d04b, SUC_lexicon_tags_12 },
    { 0x5591d04b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea31d04f, SUC_lexicon_tags_17 },
    { 0xa8b1d050, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6251d052, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6831d054, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1271d056, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa071d05c, SUC_lexicon_tags_43 },
    { 0xf571d05d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xa771d05f, SUC_lexicon_tags_45 },
    { 0xcb31d060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f1d064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb791d069, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2151d06b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b1d06e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8031d070, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2491d074, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8171d077, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b31d07c, SUC_lexicon_tags_32 },
    { 0xf1f1d07d, SUC_lexicon_tags_30 },
    { 0xfdb1d07c, SUC_lexicon_tags_63 },
    { 0xbd1d07f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa831d081, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8111d086, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5931d08a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d71d08d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ff1d08f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d1d092, SUC_lexicon_tags_0 },
    { 0x2071d093, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x51b1d095, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c11d097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15f1d099, SUC_lexicon_tags_296 },
    { 0xe171d09a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b1d09f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7611d0a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4231d0a8, SUC_lexicon_tags_27 },
    { 0xd1f1d0a9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f1d0ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f1d0af, SUC_lexicon_tags_38 },
    { 0x2431d0b0, SUC_lexicon_tags_27 },
    { 0xdd31d0b1, SUC_lexicon_tags_13 },
    { 0xa131d0b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfab1d0b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f11d0b7, SUC_lexicon_tags_32 },
    { 0xea31d0b8, SUC_lexicon_tags_234 },
    { 0xa9b1d0b8, SUC_lexicon_tags_8 },
    { 0xa7f1d0b7, SUC_lexicon_tags_15 },
    { 0xb911d0bb, SUC_lexicon_tags_8 },
    { 0x6271d0bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d1d0c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4951d0c5, SUC_lexicon_tags_25 },
    { 0x2831d0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7351d0c8, SUC_lexicon_tags_31 },
    { 0xdd51d0c9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb291d0cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad1d0ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b1d0d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b1d0d7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6cf1d0d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d1d0dc, SUC_lexicon_tags_27 },
    { 0x91d0dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c31d0e4, SUC_lexicon_tags_37 },
    { 0x8411d0e5, SUC_lexicon_tags_64 },
    { 0x1c51d0e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf1d0e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x691d0ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd831d0ec, SUC_lexicon_tags_55 },
    { 0x2a51d0ed, SUC_lexicon_tags_63 },
    { 0x1691d0ed, SUC_lexicon_tags_2 },
    { 0x3b11d0ef, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb31d0f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b91d0f8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa551d0fc, SUC_lexicon_tags_7 },
    { 0xe971d0fd, SUC_lexicon_tags_63 },
    { 0xd31d0fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d1d102, SUC_lexicon_tags_25 },
    { 0x4b71d102, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf1d108, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd771d10b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x91d1d10d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa031d10f, SUC_lexicon_tags_84 },
    { 0x6911d110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f1d113, SUC_lexicon_tags_11 },
    { 0x6131d113, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc831d116, SUC_lexicon_tags_12 },
    { 0xc0b1d117, SUC_lexicon_tags_0 },
    { 0x9c71d117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d1d121, SUC_lexicon_tags_87 },
    { 0xe011d122, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab1d125, SUC_lexicon_tags_38 },
    { 0x591d126, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6c71d128, SUC_lexicon_tags_12 },
    { 0xd731d129, SUC_lexicon_tags_2 },
    { 0x5ab1d12a, SUC_lexicon_tags_14 },
    { 0x5a31d129, SUC_lexicon_tags_25 },
    { 0x44f1d12c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9b31d12e, SUC_lexicon_tags_63 },
    { 0xe7b1d12f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd31d133, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xc431d136, SUC_lexicon_tags_27 },
    { 0x3931d136, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51d13d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2311d13f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8091d144, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f1d147, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f1d14a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9651d14c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf11d150, SUC_lexicon_tags_97 },
    { 0xad1d151, SUC_lexicon_tags_84 },
    { 0x3c31d151, SUC_lexicon_tags_2 },
    { 0x5e31d153, SUC_lexicon_tags_60 },
    { 0x88b1d153, SUC_lexicon_tags_0 },
    { 0x1131d152, SUC_lexicon_tags_25 },
    { 0xbe91d156, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda91d158, SUC_lexicon_tags_37 },
    { 0xe131d158, SUC_lexicon_tags_17 },
    { 0xc111d159, SUC_lexicon_tags_47 },
    { 0xbbd1d15b, SUC_lexicon_tags_43 },
    { 0x9dd1d15c, SUC_lexicon_tags_60 },
    { 0x57b1d15d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1d160, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91d164, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe51d169, SUC_lexicon_tags_35 },
    { 0x35d1d169, SUC_lexicon_tags_37 },
    { 0xb6f1d16b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d11d16d, SUC_lexicon_tags_1 },
    { 0x7c71d16e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd0d1d170, SUC_lexicon_tags_15 },
    { 0xc131d170, SUC_lexicon_tags_21 },
    { 0x6911d172, SUC_lexicon_tags_12 },
    { 0xb471d173, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe8b1d175, SUC_lexicon_tags_9 },
    { 0xbbb1d175, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x95b1d178, SUC_lexicon_tags_45 },
    { 0x90f1d179, SUC_lexicon_tags_31 },
    { 0x1e91d17a, SUC_lexicon_tags_19 },
    { 0xbd71d17a, SUC_lexicon_tags_7 },
    { 0xf5d1d17c, SUC_lexicon_tags_7 },
    { 0x9d1d17d, SUC_lexicon_tags_77 },
    { 0xa611d17e, SUC_lexicon_tags_1 },
    { 0x3a51d17f, SUC_lexicon_tags_39 },
    { 0xe5f1d180, SUC_lexicon_tags_37 },
    { 0xb131d181, SUC_lexicon_tags_27 },
    { 0x3f51d182, SUC_lexicon_tags_11 },
    { 0xe191d183, SUC_lexicon_tags_15 },
    { 0xbb91d183, SUC_lexicon_tags_7 },
    { 0x39b1d185, SUC_lexicon_tags_27 },
    { 0x4051d186, SUC_lexicon_tags_17 },
    { 0x1031d17f, SUC_lexicon_tags_11 },
    { 0xa1b1d188, SUC_lexicon_tags_12 },
    { 0x6a51d188, SUC_lexicon_tags_7 },
    { 0x9971d18a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2d91d18c, SUC_lexicon_tags_33 },
    { 0xe5f1d18c, SUC_lexicon_tags_10 },
    { 0x9051d18d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc011d194, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f1d199, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa0b1d19b, SUC_lexicon_tags_15 },
    { 0x9d11d19c, SUC_lexicon_tags_37 },
    { 0xe7d1d19c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb251d1a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc271d1a3, SUC_lexicon_tags_60 },
    { 0xb111d1a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa71d1a8, SUC_lexicon_tags_1 },
    { 0x4b1d1a9, SUC_lexicon_tags_31 },
    { 0x1231d1aa, SUC_lexicon_tags_27 },
    { 0x8d1d1aa, SUC_lexicon_tags_26 },
    { 0x7971d1ab, SUC_lexicon_tags_17 },
    { 0xea71d1ac, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1571d1b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8171d1b7, SUC_lexicon_tags_13 },
    { 0x4291d1b8, SUC_lexicon_tags_71 },
    { 0xd4b1d1b9, SUC_lexicon_tags_0 },
    { 0xb5b1d1ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2511d1be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc591d1c4, SUC_lexicon_tags_37 },
    { 0x6731d1c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c91d1ca, SUC_lexicon_tags_29 },
    { 0x42f1d1ca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3fb1d1cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb631d1cf, SUC_lexicon_tags_82 },
    { 0xb011d1d0, SUC_lexicon_tags_45 },
    { 0xeb71d1d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f71d1d4, SUC_lexicon_tags_18 },
    { 0x6c51d1d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad1d1d8, SUC_lexicon_tags_25 },
    { 0x27b1d1d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef1d1df, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x31d1d1e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a31d1e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9611d1e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc91d1eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf391d1f0, SUC_lexicon_tags_611 },
    { 0xb7f1d1f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7611d1f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e71d1f8, SUC_lexicon_tags_27 },
    { 0x1651d1f8, SUC_lexicon_tags_17 },
    { 0x171d1fa, SUC_lexicon_tags_19 },
    { 0xb491d1f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3ab1d1fd, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0xc911d1ff, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe211d20a, SUC_lexicon_tags_14 },
    { 0x50f1d20b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb511d20d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe131d211, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9b31d213, SUC_lexicon_tags_15 },
    { 0xc291d214, SUC_lexicon_tags_1 },
    { 0x67f1d213, SUC_lexicon_tags_38 },
    { 0x6cf1d216, SUC_lexicon_tags_1 },
    { 0xd7f1d216, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb11d224, SUC_lexicon_tags_11 },
    { 0x4871d224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b1d228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6911d22f, SUC_lexicon_tags_25 },
    { 0xe231d230, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb1d234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab1d237, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a71d239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe551d23d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a31d244, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5471d246, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1591d248, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd431d24b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d1d253, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f11d256, SUC_lexicon_tags_33 },
    { 0xa691d256, SUC_lexicon_tags_11 },
    { 0xb331d258, SUC_lexicon_tags_11 },
    { 0x6331d256, SUC_lexicon_tags_63 },
    { 0xd3f1d256, SUC_lexicon_tags_38 },
    { 0xae31d25b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3771d25e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2811d261, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe31d263, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d1d269, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c11d26b, SUC_lexicon_tags_25 },
    { 0x8f91d26b, SUC_lexicon_tags_13 },
    { 0x33d1d26d, SUC_lexicon_tags_7 },
    { 0xea71d26c, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b1d274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e31d276, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf011d279, SUC_lexicon_tags_38 },
    { 0x6ab1d27a, SUC_lexicon_tags_27 },
    { 0x6731d279, SUC_lexicon_tags_26 },
    { 0x5411d27c, SUC_lexicon_tags_23 },
    { 0xbc31d279, SUC_lexicon_tags_17 },
    { 0xa1f1d27e, SUC_lexicon_tags_40 },
    { 0x3ed1d279, SUC_lexicon_tags_14 },
    { 0x8cd1d280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d71d282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3f1d284, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc531d28a, SUC_lexicon_tags_39 },
    { 0x56d1d28b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791d28f, SUC_lexicon_tags_21 },
    { 0x99d1d290, SUC_lexicon_tags_19 },
    { 0xa6d1d291, SUC_lexicon_tags_33 },
    { 0xdc31d290, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4211d294, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f1d298, SUC_lexicon_tags_8 },
    { 0xcb71d299, SUC_lexicon_tags_11 },
    { 0xced1d29a, SUC_lexicon_tags_14 },
    { 0x65d1d29b, SUC_lexicon_tags_37 },
    { 0x3591d298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e31d29e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd371d2a1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f91d2a4, SUC_lexicon_tags_23 },
    { 0x8991d2a4, SUC_lexicon_tags_11 },
    { 0x2bb1d2a6, SUC_lexicon_tags_11 },
    { 0x15d1d2a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d1d2aa, SUC_lexicon_tags_0 },
    { 0x4af1d2ab, SUC_lexicon_tags_6 },
    { 0xb431d2ac, SUC_lexicon_tags_25 },
    { 0xa491d2ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b1d2b0, SUC_lexicon_tags_12 },
    { 0xd391d2b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x171d2b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb71d2b7, SUC_lexicon_tags_33 },
    { 0xdb11d2b8, SUC_lexicon_tags_60 },
    { 0xced1d2b8, SUC_lexicon_tags_37 },
    { 0xc91d2b8, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa371d2be, SUC_lexicon_tags_15 },
    { 0xfc71d2be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe151d2c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7911d2c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed91d2c8, SUC_lexicon_tags_13 },
    { 0xdd51d2c9, SUC_lexicon_tags_14 },
    { 0x4d1d2ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf71d2d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a91d2d4, SUC_lexicon_tags_2 },
    { 0x26b1d2d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d71d2d9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb471d2db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5551d2e2, SUC_lexicon_tags_17 },
    { 0x31f1d2e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9851d2e7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9511d2e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d1d2ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51d2f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9891d2f4, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5591d2f8, SUC_lexicon_tags_37 },
    { 0x8c31d2f9, SUC_lexicon_tags_5 },
    { 0x7571d2fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f1d2fe, SUC_lexicon_tags_15 },
    { 0x4ff1d2ff, SUC_lexicon_tags_4 },
    { 0x2831d2ff, SUC_lexicon_tags_38 },
    { 0x17b1d300, SUC_lexicon_tags_33 },
    { 0xdf91d302, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d1d306, SUC_lexicon_tags_17 },
    { 0xe8f1d307, SUC_lexicon_tags_14 },
    { 0x9451d306, SUC_lexicon_tags_43 },
    { 0x13d1d309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d1d30c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b71d30f, SUC_lexicon_tags_0 },
    { 0x9771d310, SUC_lexicon_tags_25 },
    { 0x9ad1d311, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1531d313, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf1d316, SUC_lexicon_tags_49 },
    { 0x3b51d317, SUC_lexicon_tags_47 },
    { 0xfa91d317, SUC_lexicon_tags_27 },
    { 0xd3d1d318, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x5f91d31b, SUC_lexicon_tags_32 },
    { 0xc71d31c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2591d320, SUC_lexicon_tags_1 },
    { 0xced1d320, SUC_lexicon_tags_25 },
    { 0xffb1d321, SUC_lexicon_tags_18 },
    { 0x5651d323, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f1d327, SUC_lexicon_tags_47 },
    { 0xed31d328, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ed1d32a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6331d330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cf1d332, SUC_lexicon_tags_0 },
    { 0xa251d332, SUC_lexicon_tags_98 },
    { 0x92d1d332, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa411d337, SUC_lexicon_tags_7 },
    { 0xbe51d337, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f1d33b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad1d33f, SUC_lexicon_tags_53 },
    { 0x4291d33f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf751d344, SUC_lexicon_tags_79 },
    { 0xbe91d345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c71d347, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0d1d349, SUC_lexicon_tags_7 },
    { 0x3b91d34a, SUC_lexicon_tags_37 },
    { 0x4ad1d34b, SUC_lexicon_tags_25 },
    { 0x3631d34a, SUC_lexicon_tags_11 },
    { 0x8871d34b, SUC_lexicon_tags_17 },
    { 0x5291d34a, SUC_lexicon_tags_37 },
    { 0x9dd1d34f, SUC_lexicon_tags_26 },
    { 0x9d51d34f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xea71d353, SUC_lexicon_tags_1 },
    { 0xfa71d354, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d1d357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd31d35a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df1d35e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b1d361, SUC_lexicon_tags_11 },
    { 0x5611d362, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3771d364, SUC_lexicon_tags_14 },
    { 0xcfb1d365, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb851d369, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa7b1d36b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf7b1d36d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb1d370, SUC_lexicon_tags_11 },
    { 0x8fb1d371, SUC_lexicon_tags_83 },
    { 0x4ed1d371, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1d31d374, SUC_lexicon_tags_19 },
    { 0xe451d375, SUC_lexicon_tags_1 },
    { 0xc651d375, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e71d379, SUC_lexicon_tags_19 },
    { 0xf1f1d379, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a51d37c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ab1d37e, SUC_lexicon_tags_11 },
    { 0x33f1d37f, SUC_lexicon_tags_26 },
    { 0x1611d380, SUC_lexicon_tags_2 },
    { 0x9491d381, SUC_lexicon_tags_23 },
    { 0x77b1d37e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5271d384, SUC_lexicon_tags_28 },
    { 0x78b1d385, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6691d387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda11d38c, SUC_lexicon_tags_10 },
    { 0x18b1d38d, SUC_lexicon_tags_11 },
    { 0xf91d38d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd871d390, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa371d394, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc791d398, SUC_lexicon_tags_2 },
    { 0xbf71d399, SUC_lexicon_tags_19 },
    { 0x1bd1d39a, SUC_lexicon_tags_15 },
    { 0xfd71d39a, SUC_lexicon_tags_48 },
    { 0x9351d398, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d1d3a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2c51d3a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a11d3ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe391d3ae, SUC_lexicon_tags_23 },
    { 0xb6b1d3af, SUC_lexicon_tags_3 },
    { 0x2ab1d3b0, SUC_lexicon_tags_1 },
    { 0x3cf1d3b1, SUC_lexicon_tags_23 },
    { 0xe491d3b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3351d3b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbfb1d3b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe691d3b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x531d3bc, SUC_lexicon_tags_275 },
    { 0x3e31d3bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b51d3bf, SUC_lexicon_tags_49 },
    { 0x44b1d3bf, SUC_lexicon_tags_7 },
    { 0x9331d3c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd11d3c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa9b1d3c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12b1d3c7, SUC_lexicon_tags_61 },
    { 0x4d1d3c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c51d3cb, SUC_lexicon_tags_30 },
    { 0xa971d3cc, SUC_lexicon_tags_34 },
    { 0xdb31d3cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x95b1d3cf, SUC_lexicon_tags_77 },
    { 0x2d51d3d0, SUC_lexicon_tags_47 },
    { 0x28b1d3d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf71d3d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d1d3d9, SUC_lexicon_tags_8 },
    { 0x58d1d3da, SUC_lexicon_tags_12 },
    { 0x8031d3da, SUC_lexicon_tags_63 },
    { 0x1071d3dc, SUC_lexicon_tags_17 },
    { 0xf571d3da, SUC_lexicon_tags_37 },
    { 0xb211d3d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff1d3e4, SUC_lexicon_tags_37 },
    { 0xce91d3e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd1d3e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db1d3eb, SUC_lexicon_tags_504 },
    { 0, NULL },
    { 0xc8f1d3ed, SUC_lexicon_tags_38 },
    { 0x3591d3ee, SUC_lexicon_tags_95 },
    { 0x1891d3ef, SUC_lexicon_tags_29 },
    { 0xc371d3f0, SUC_lexicon_tags_45 },
    { 0xc2f1d3ee, SUC_lexicon_tags_38 },
    { 0xb851d3f2, SUC_lexicon_tags_7 },
    { 0xc5f1d3f3, SUC_lexicon_tags_15 },
    { 0x7051d3f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc991d3f6, SUC_lexicon_tags_19 },
    { 0xb0d1d3f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b1d3fc, SUC_lexicon_tags_34 },
    { 0x9631d3fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f51d400, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd91d403, SUC_lexicon_tags_17 },
    { 0xd731d403, SUC_lexicon_tags_23 },
    { 0x2bb1d405, SUC_lexicon_tags_84 },
    { 0x57d1d406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a51d40f, SUC_lexicon_tags_17 },
    { 0x55d1d410, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4231d412, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb1d415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b1d418, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7671d41a, SUC_lexicon_tags_17 },
    { 0xaa71d41b, SUC_lexicon_tags_28 },
    { 0x6531d41c, SUC_lexicon_tags_11 },
    { 0xa4d1d41c, SUC_lexicon_tags_1 },
    { 0x6db1d41b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1d425, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf951d427, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e31d429, SUC_lexicon_tags_11 },
    { 0xf91d42a, SUC_lexicon_tags_32 },
    { 0x9751d42a, SUC_lexicon_tags_26 },
    { 0x3451d42c, SUC_lexicon_tags_47 },
    { 0xeff1d42d, SUC_lexicon_tags_26 },
    { 0x23f1d42c, SUC_lexicon_tags_65 },
    { 0xf811d42f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7091d434, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1b1d436, SUC_lexicon_tags_15 },
    { 0xded1d437, SUC_lexicon_tags_57 },
    { 0x9111d438, SUC_lexicon_tags_23 },
    { 0x97d1d439, SUC_lexicon_tags_17 },
    { 0x2cd1d43a, SUC_lexicon_tags_8 },
    { 0xd711d43b, SUC_lexicon_tags_8 },
    { 0x3b71d43a, SUC_lexicon_tags_27 },
    { 0x1471d439, SUC_lexicon_tags_48 },
    { 0xaad1d43e, SUC_lexicon_tags_56 },
    { 0xf1b1d437, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef1d442, SUC_lexicon_tags_30 },
    { 0x81d1d443, SUC_lexicon_tags_7 },
    { 0xac11d444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b1d448, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb1d44b, SUC_lexicon_tags_17 },
    { 0xb591d44c, SUC_lexicon_tags_21 },
    { 0xfb31d44d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc71d452, SUC_lexicon_tags_11 },
    { 0x49b1d452, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8471d457, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3cd1d459, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1471d45c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5391d45e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb1d460, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb31d462, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d31d466, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1eb1d468, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1731d46c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f1d46f, SUC_lexicon_tags_45 },
    { 0x4691d46f, SUC_lexicon_tags_37 },
    { 0x38b1d471, SUC_lexicon_tags_11 },
    { 0xabb1d470, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d475, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d71d478, SUC_lexicon_tags_15 },
    { 0x7611d479, SUC_lexicon_tags_37 },
    { 0x7e91d47a, SUC_lexicon_tags_23 },
    { 0xe4b1d479, SUC_lexicon_tags_2 },
    { 0xfd51d47c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4771d486, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8691d488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ef1d48a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1711d48d, SUC_lexicon_tags_34 },
    { 0x4031d48e, SUC_lexicon_tags_26 },
    { 0x5f51d48f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db1d495, SUC_lexicon_tags_43 },
    { 0x1cf1d496, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1db1d498, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x5571d49b, SUC_lexicon_tags_26 },
    { 0xd5b1d49c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf1d4a0, SUC_lexicon_tags_56 },
    { 0x711d4a0, SUC_lexicon_tags_63 },
    { 0x531d4a2, SUC_lexicon_tags_64 },
    { 0x29f1d4a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f1d4a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c51d4ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf1d4ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3671d4b7, SUC_lexicon_tags_14 },
    { 0x66b1d4b7, SUC_lexicon_tags_60 },
    { 0xcf11d4b9, SUC_lexicon_tags_25 },
    { 0xc271d4ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1cd1d4bc, SUC_lexicon_tags_57 },
    { 0x5bb1d4bc, SUC_lexicon_tags_3 },
    { 0xeb71d4be, SUC_lexicon_tags_35 },
    { 0x8891d4bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f1d4c1, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xf891d4c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb91d4c9, SUC_lexicon_tags_35 },
    { 0xc371d4ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11d4d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b1d4da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32b1d4dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4691d4de, SUC_lexicon_tags_2 },
    { 0x11d4de, SUC_lexicon_tags_30 },
    { 0x9991d4de, SUC_lexicon_tags_4 },
    { 0xfdb1d4e1, SUC_lexicon_tags_66 },
    { 0x2ab1d4e1, SUC_lexicon_tags_16 },
    { 0x4d31d4e3, SUC_lexicon_tags_14 },
    { 0x18f1d4e2, SUC_lexicon_tags_39 },
    { 0xbed1d4e5, SUC_lexicon_tags_28 },
    { 0x2651d4e5, SUC_lexicon_tags_45 },
    { 0xd011d4e5, SUC_lexicon_tags_141 },
    { 0x9291d4e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa291d4ea, SUC_lexicon_tags_37 },
    { 0xe7b1d4ea, SUC_lexicon_tags_17 },
    { 0xccd1d4eb, SUC_lexicon_tags_39 },
    { 0xfd11d4ed, SUC_lexicon_tags_10 },
    { 0x7b71d4ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd1d4f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5451d4f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f1d4fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb31d4fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd1d502, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc551d504, SUC_lexicon_tags_7 },
    { 0x6111d505, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f1d50e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab1d511, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb851d515, SUC_lexicon_tags_11 },
    { 0xe231d516, SUC_lexicon_tags_179 },
    { 0xe8b1d517, SUC_lexicon_tags_32 },
    { 0xf731d518, SUC_lexicon_tags_17 },
    { 0x9471d518, SUC_lexicon_tags_0 },
    { 0xd351d51a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6951d51c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd11d51f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf11d525, SUC_lexicon_tags_17 },
    { 0xde31d526, SUC_lexicon_tags_31 },
    { 0xcc31d527, SUC_lexicon_tags_27 },
    { 0x1291d528, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd271d52b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f51d531, SUC_lexicon_tags_37 },
    { 0x3b11d531, SUC_lexicon_tags_17 },
    { 0x2571d532, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d71d537, SUC_lexicon_tags_6 },
    { 0x71b1d537, SUC_lexicon_tags_33 },
    { 0x4991d537, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa671d53b, SUC_lexicon_tags_151 },
    { 0xb0b1d53c, SUC_lexicon_tags_6 },
    { 0x651d53d, SUC_lexicon_tags_15 },
    { 0x5f11d53e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8251d540, SUC_lexicon_tags_15 },
    { 0x2871d541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff1d544, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc731d54b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd871d551, SUC_lexicon_tags_43 },
    { 0x4c91d552, SUC_lexicon_tags_27 },
    { 0xa731d552, SUC_lexicon_tags_70 },
    { 0xb11d553, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1d55a, SUC_lexicon_tags_25 },
    { 0x41d1d55b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a31d55d, SUC_lexicon_tags_27 },
    { 0x63d1d55e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcd51d560, SUC_lexicon_tags_37 },
    { 0x4811d561, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83b1d563, SUC_lexicon_tags_7 },
    { 0xc951d564, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf1d56d, SUC_lexicon_tags_37 },
    { 0xc551d56e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f1d572, SUC_lexicon_tags_57 },
    { 0x2bd1d573, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f1d57c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6511d57f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f1d584, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f1d586, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf891d588, SUC_lexicon_tags_47 },
    { 0xe071d588, SUC_lexicon_tags_17 },
    { 0xb6d1d58a, SUC_lexicon_tags_61 },
    { 0xd751d58a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde71d592, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3291d597, SUC_lexicon_tags_37 },
    { 0xf8b1d598, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5491d59a, SUC_lexicon_tags_17 },
    { 0x2f51d59a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3871d59d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb1d5a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd1d5ab, SUC_lexicon_tags_19 },
    { 0x1d31d5ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf211d5b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3711d5b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49d1d5b6, SUC_lexicon_tags_113 },
    { 0xc311d5b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd871d5b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd1d5bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f1d5c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1831d5c3, SUC_lexicon_tags_25 },
    { 0x4631d5c3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd1d5c7, SUC_lexicon_tags_12 },
    { 0xca71d5c7, SUC_lexicon_tags_38 },
    { 0xe851d5c9, SUC_lexicon_tags_38 },
    { 0xd411d5c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d1d5cd, SUC_lexicon_tags_17 },
    { 0x9d71d5cd, SUC_lexicon_tags_37 },
    { 0xea51d5ce, SUC_lexicon_tags_17 },
    { 0x1cf1d5d0, SUC_lexicon_tags_12 },
    { 0xec91d5d1, SUC_lexicon_tags_14 },
    { 0xb511d5d2, SUC_lexicon_tags_6 },
    { 0x6d1d5d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a91d5d9, SUC_lexicon_tags_17 },
    { 0x70b1d5da, SUC_lexicon_tags_25 },
    { 0x2991d5db, SUC_lexicon_tags_25 },
    { 0x2471d5dc, SUC_lexicon_tags_37 },
    { 0x5e11d5d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71d5e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a11d5e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe731d5e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2bb1d5e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d1d5ed, SUC_lexicon_tags_39 },
    { 0x6db1d5ee, SUC_lexicon_tags_21 },
    { 0x6971d5ef, SUC_lexicon_tags_11 },
    { 0x82d1d5f0, SUC_lexicon_tags_62 },
    { 0x1bf1d5f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3491d5f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b1d5f8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfb1d5fa, SUC_lexicon_tags_19 },
    { 0xa871d5fa, SUC_lexicon_tags_65 },
    { 0x80f1d5fc, SUC_lexicon_tags_17 },
    { 0x8371d5fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9411d600, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2331d602, SUC_lexicon_tags_7 },
    { 0xb131d602, SUC_lexicon_tags_7 },
    { 0x951d604, SUC_lexicon_tags_1 },
    { 0x43d1d605, SUC_lexicon_tags_38 },
    { 0x56b1d606, SUC_lexicon_tags_38 },
    { 0x5b91d607, SUC_lexicon_tags_37 },
    { 0x1091d608, SUC_lexicon_tags_10 },
    { 0x2ff1d609, SUC_lexicon_tags_63 },
    { 0xa311d609, SUC_lexicon_tags_68 },
    { 0x171d60a, SUC_lexicon_tags_6 },
    { 0xfb51d604, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b1d611, SUC_lexicon_tags_12 },
    { 0x2871d611, SUC_lexicon_tags_25 },
    { 0xa1f1d613, SUC_lexicon_tags_49 },
    { 0x6651d614, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf1d618, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f1d620, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a11d623, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec31d628, SUC_lexicon_tags_3 },
    { 0xdf91d629, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x41f1d62b, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x2771d62e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb1d631, SUC_lexicon_tags_87 },
    { 0xd471d632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa31d634, SUC_lexicon_tags_5 },
    { 0x32d1d635, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc371d63a, SUC_lexicon_tags_30 },
    { 0x62b1d63b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2631d63e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb631d642, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f1d645, SUC_lexicon_tags_23 },
    { 0x9451d645, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb1d64f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf811d652, SUC_lexicon_tags_16 },
    { 0x87f1d652, SUC_lexicon_tags_7 },
    { 0xe1b1d653, SUC_lexicon_tags_37 },
    { 0xb371d654, SUC_lexicon_tags_30 },
    { 0xf171d656, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x2ef1d658, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b51d65c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8131d665, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6671d668, SUC_lexicon_tags_1 },
    { 0x7971d669, SUC_lexicon_tags_23 },
    { 0xde11d66a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4af1d66c, SUC_lexicon_tags_77 },
    { 0x3b91d66d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d1d671, SUC_lexicon_tags_30 },
    { 0x5071d672, SUC_lexicon_tags_25 },
    { 0x7a91d673, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3251d675, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc631d679, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5091d67c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51d67f, SUC_lexicon_tags_23 },
    { 0x5c51d680, SUC_lexicon_tags_1 },
    { 0x5f71d681, SUC_lexicon_tags_16 },
    { 0x9f51d682, SUC_lexicon_tags_1 },
    { 0x37b1d683, SUC_lexicon_tags_9 },
    { 0x2c11d684, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c91d687, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xca51d68a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53d1d68c, SUC_lexicon_tags_1 },
    { 0x5f91d68d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe11d692, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6391d695, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1d69a, SUC_lexicon_tags_2 },
    { 0x9431d69a, SUC_lexicon_tags_1 },
    { 0xe751d69a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf111d69f, SUC_lexicon_tags_12 },
    { 0xe891d69f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf911d6a3, SUC_lexicon_tags_25 },
    { 0x7071d6a4, SUC_lexicon_tags_25 },
    { 0xe4b1d6a4, SUC_lexicon_tags_37 },
    { 0x8f51d6a6, SUC_lexicon_tags_38 },
    { 0xdd11d6a5, SUC_lexicon_tags_55 },
    { 0xc1b1d6a8, SUC_lexicon_tags_112 },
    { 0x8251d6a8, SUC_lexicon_tags_37 },
    { 0x5d11d6a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f1d6ae, SUC_lexicon_tags_27 },
    { 0xa191d6ae, SUC_lexicon_tags_237 },
    { 0xe651d6af, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f11d6b3, SUC_lexicon_tags_4 },
    { 0xb0f1d6b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe471d6b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd1d6bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9351d6c0, SUC_lexicon_tags_25 },
    { 0x3e91d6c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c31d6c5, SUC_lexicon_tags_12 },
    { 0x7651d6c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb4d1d6c8, SUC_lexicon_tags_23 },
    { 0x9c51d6c8, SUC_lexicon_tags_7 },
    { 0x1d31d6ca, SUC_lexicon_tags_49 },
    { 0x3e11d6cb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x65f1d6cd, SUC_lexicon_tags_17 },
    { 0x7111d6ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf811d6d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51d6d9, SUC_lexicon_tags_87 },
    { 0x4151d6d9, SUC_lexicon_tags_17 },
    { 0x5411d6da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe331d6de, SUC_lexicon_tags_25 },
    { 0xcc31d6df, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4611d6e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1bb1d6e6, SUC_lexicon_tags_38 },
    { 0xb5d1d6e7, SUC_lexicon_tags_26 },
    { 0x291d6e7, SUC_lexicon_tags_23 },
    { 0x4a91d6e9, SUC_lexicon_tags_60 },
    { 0x8911d6ea, SUC_lexicon_tags_16 },
    { 0xad91d6ea, SUC_lexicon_tags_37 },
    { 0x1191d6ec, SUC_lexicon_tags_23 },
    { 0x7111d6eb, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x7671d6ef, SUC_lexicon_tags_7 },
    { 0xbfb1d6f0, SUC_lexicon_tags_19 },
    { 0x78d1d6f1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa091d6f4, SUC_lexicon_tags_7 },
    { 0x74b1d6f4, SUC_lexicon_tags_9 },
    { 0xc531d6f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f1d6f9, SUC_lexicon_tags_12 },
    { 0xf851d6fa, SUC_lexicon_tags_19 },
    { 0x9a31d6fb, SUC_lexicon_tags_33 },
    { 0x6d91d6fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa891d6fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3131d700, SUC_lexicon_tags_35 },
    { 0x4711d701, SUC_lexicon_tags_17 },
    { 0x4b91d702, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3311d706, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf2d1d708, SUC_lexicon_tags_12 },
    { 0x2cf1d709, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7171d70d, SUC_lexicon_tags_15 },
    { 0x5f11d70d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8511d710, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x791d713, SUC_lexicon_tags_23 },
    { 0x4271d714, SUC_lexicon_tags_0 },
    { 0xa7f1d714, SUC_lexicon_tags_25 },
    { 0xcc71d713, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd171d718, SUC_lexicon_tags_61 },
    { 0xa9d1d719, SUC_lexicon_tags_25 },
    { 0x8951d71a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5fd1d71c, SUC_lexicon_tags_7 },
    { 0xbeb1d71d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b1d725, SUC_lexicon_tags_13 },
    { 0xd911d726, SUC_lexicon_tags_7 },
    { 0x6151d727, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff1d72d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x8771d72f, SUC_lexicon_tags_47 },
    { 0xe211d730, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b31d733, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad1d736, SUC_lexicon_tags_16 },
    { 0x86f1d737, SUC_lexicon_tags_15 },
    { 0x4311d737, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9651d73a, SUC_lexicon_tags_18 },
    { 0xc5b1d73b, SUC_lexicon_tags_27 },
    { 0x8811d73c, SUC_lexicon_tags_17 },
    { 0x3df1d73a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f1d746, SUC_lexicon_tags_1 },
    { 0xd591d746, SUC_lexicon_tags_27 },
    { 0xa1f1d748, SUC_lexicon_tags_37 },
    { 0x3751d749, SUC_lexicon_tags_5 },
    { 0x7d1d746, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf2d1d74c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa731d74e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d51d750, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5b1d752, SUC_lexicon_tags_8 },
    { 0x49d1d753, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xee71d755, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d31d759, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd1d75f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f1d762, SUC_lexicon_tags_16 },
    { 0xbcd1d763, SUC_lexicon_tags_17 },
    { 0xd71d764, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdbf1d766, SUC_lexicon_tags_9 },
    { 0x32b1d767, SUC_lexicon_tags_31 },
    { 0x8cb1d768, SUC_lexicon_tags_46 },
    { 0x5b91d769, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6c11d76b, SUC_lexicon_tags_16 },
    { 0xc9b1d76c, SUC_lexicon_tags_25 },
    { 0x8211d76d, SUC_lexicon_tags_12 },
    { 0xfbb1d76e, SUC_lexicon_tags_32 },
    { 0xae91d76d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb1d77b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd1d780, SUC_lexicon_tags_37 },
    { 0x1591d781, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3071d784, SUC_lexicon_tags_12 },
    { 0x85d1d785, SUC_lexicon_tags_12 },
    { 0xf951d786, SUC_lexicon_tags_4 },
    { 0xc091d787, SUC_lexicon_tags_33 },
    { 0x5531d788, SUC_lexicon_tags_38 },
    { 0x4c71d788, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf191d78e, SUC_lexicon_tags_19 },
    { 0x1771d78e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d1d796, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71d79e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf1d7a4, SUC_lexicon_tags_37 },
    { 0xbc91d7a5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa411d7a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9531d7aa, SUC_lexicon_tags_15 },
    { 0x18f1d7aa, SUC_lexicon_tags_19 },
    { 0xd291d7ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb31d7b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd1d7b4, SUC_lexicon_tags_17 },
    { 0x5ef1d7b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9211d7b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f1d7bb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x14b1d7bd, SUC_lexicon_tags_19 },
    { 0x5fd1d7be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb591d7c2, SUC_lexicon_tags_45 },
    { 0xcd71d7c3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d1d7c6, SUC_lexicon_tags_40 },
    { 0x42d1d7c7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x14b1d7c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1d7cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4eb1d7ce, SUC_lexicon_tags_37 },
    { 0xdf51d7cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b1d7d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e71d7d7, SUC_lexicon_tags_28 },
    { 0xf131d7d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd1d7de, SUC_lexicon_tags_43 },
    { 0x9f71d7de, SUC_lexicon_tags_46 },
    { 0x9611d7e0, SUC_lexicon_tags_30 },
    { 0x2271d7df, SUC_lexicon_tags_2 },
    { 0x64f1d7e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa591d809, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10f1d80b, SUC_lexicon_tags_43 },
    { 0x7931d80c, SUC_lexicon_tags_17 },
    { 0xbd1d80d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe071d812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe491d816, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d1d81d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9251d825, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4e11d827, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b1d82c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc311d82f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf231d831, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed51d839, SUC_lexicon_tags_37 },
    { 0xf351d83a, SUC_lexicon_tags_12 },
    { 0x13d1d83b, SUC_lexicon_tags_15 },
    { 0xc3f1d83c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2b1d83e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd1d840, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfad1d842, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd1d846, SUC_lexicon_tags_17 },
    { 0xa5f1d847, SUC_lexicon_tags_7 },
    { 0xbbd1d848, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b1d84c, SUC_lexicon_tags_30 },
    { 0xcf11d84c, SUC_lexicon_tags_48 },
    { 0xb9f1d84c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d11d850, SUC_lexicon_tags_473 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a71d85a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd1d862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd51d865, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76d1d867, SUC_lexicon_tags_40 },
    { 0xe911d867, SUC_lexicon_tags_11 },
    { 0x3771d868, SUC_lexicon_tags_4 },
    { 0x56f1d86a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2651d870, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5791d874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc91d878, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa291d87e, SUC_lexicon_tags_1 },
    { 0x1f91d87f, SUC_lexicon_tags_32 },
    { 0x90f1d87f, SUC_lexicon_tags_14 },
    { 0x77b1d881, SUC_lexicon_tags_17 },
    { 0x2f31d880, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e51d889, SUC_lexicon_tags_0 },
    { 0x89d1d889, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1611d893, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8951d895, SUC_lexicon_tags_45 },
    { 0x8191d895, SUC_lexicon_tags_107 },
    { 0x6f71d896, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf291d89a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xde31d89d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9411d89f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe851d8a2, SUC_lexicon_tags_39 },
    { 0xe271d8a3, SUC_lexicon_tags_27 },
    { 0xf491d8a4, SUC_lexicon_tags_17 },
    { 0x8191d8a5, SUC_lexicon_tags_39 },
    { 0x4d71d8a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d1d8ae, SUC_lexicon_tags_25 },
    { 0x6591d8af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d51d8b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d31d8ba, SUC_lexicon_tags_8 },
    { 0x8031d8ba, SUC_lexicon_tags_0 },
    { 0xe6b1d8ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab1d8c1, SUC_lexicon_tags_7 },
    { 0xa9b1d8c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f1d8c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c51d8cb, SUC_lexicon_tags_68 },
    { 0xc5b1d8cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf1d8cf, SUC_lexicon_tags_7 },
    { 0xf391d8cf, SUC_lexicon_tags_15 },
    { 0xc891d8cf, SUC_lexicon_tags_38 },
    { 0xa791d8d2, SUC_lexicon_tags_27 },
    { 0x7a11d8d3, SUC_lexicon_tags_11 },
    { 0x8991d8d3, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c11d8da, SUC_lexicon_tags_74 },
    { 0x55b1d8db, SUC_lexicon_tags_29 },
    { 0x4891d8dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f1d8df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f1d8e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6951d8e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f1d8ec, SUC_lexicon_tags_37 },
    { 0x9811d8ec, SUC_lexicon_tags_44 },
    { 0x2e31d8ed, SUC_lexicon_tags_15 },
    { 0x6591d8ef, SUC_lexicon_tags_2 },
    { 0x3291d8f0, SUC_lexicon_tags_129 },
    { 0xda71d8f0, SUC_lexicon_tags_17 },
    { 0x78b1d8f2, SUC_lexicon_tags_43 },
    { 0x4451d8f3, SUC_lexicon_tags_13 },
    { 0xcd71d8ef, SUC_lexicon_tags_285 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b1d8f7, SUC_lexicon_tags_9 },
    { 0x1d91d8f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c91d8fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5031d8ff, SUC_lexicon_tags_12 },
    { 0xce11d8ff, SUC_lexicon_tags_8 },
    { 0xb1d900, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb91d906, SUC_lexicon_tags_1 },
    { 0xd291d907, SUC_lexicon_tags_25 },
    { 0x6c91d908, SUC_lexicon_tags_12 },
    { 0x2491d907, SUC_lexicon_tags_1 },
    { 0xb011d909, SUC_lexicon_tags_2 },
    { 0xb2d1d90b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd51d90d, SUC_lexicon_tags_38 },
    { 0x9031d90e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8271d910, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf71d913, SUC_lexicon_tags_47 },
    { 0x5f91d913, SUC_lexicon_tags_33 },
    { 0x2e11d914, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd1d91c, SUC_lexicon_tags_13 },
    { 0xfe31d91d, SUC_lexicon_tags_2 },
    { 0x7c91d91c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f1d922, SUC_lexicon_tags_38 },
    { 0xf491d923, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf1d929, SUC_lexicon_tags_2 },
    { 0xe931d92a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d1d930, SUC_lexicon_tags_17 },
    { 0xb711d930, SUC_lexicon_tags_14 },
    { 0x6a11d932, SUC_lexicon_tags_15 },
    { 0x1dd1d933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa351d936, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf431d93c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5071d93e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb71d941, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5091d948, SUC_lexicon_tags_68 },
    { 0x47d1d949, SUC_lexicon_tags_10 },
    { 0xe11d94a, SUC_lexicon_tags_68 },
    { 0xef71d94a, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9471d94f, SUC_lexicon_tags_11 },
    { 0xc7f1d950, SUC_lexicon_tags_37 },
    { 0x1991d94f, SUC_lexicon_tags_68 },
    { 0xc8d1d94f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6111d958, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a31d95a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5091d961, SUC_lexicon_tags_12 },
    { 0x8ed1d962, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1d966, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd411d96d, SUC_lexicon_tags_12 },
    { 0x6731d96e, SUC_lexicon_tags_10 },
    { 0x3f71d96f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d1d973, SUC_lexicon_tags_26 },
    { 0xaa51d974, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf91d978, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6331d97c, SUC_lexicon_tags_159 },
    { 0xe11d97c, SUC_lexicon_tags_37 },
    { 0x2b1d97e, SUC_lexicon_tags_55 },
    { 0x21b1d97f, SUC_lexicon_tags_63 },
    { 0x6bf1d97e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d71d982, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56d1d984, SUC_lexicon_tags_63 },
    { 0x6391d985, SUC_lexicon_tags_7 },
    { 0xb271d984, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef1d989, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2631d98d, SUC_lexicon_tags_37 },
    { 0x6cd1d98e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9071d990, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f1d993, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd491d997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f1d9a0, SUC_lexicon_tags_31 },
    { 0x5d91d9a1, SUC_lexicon_tags_49 },
    { 0xf271d9a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb1d9a5, SUC_lexicon_tags_1 },
    { 0x4091d9a6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x15d1d9a8, SUC_lexicon_tags_228 },
    { 0, NULL },
    { 0xb951d9aa, SUC_lexicon_tags_2 },
    { 0xb571d9aa, SUC_lexicon_tags_7 },
    { 0x25b1d9ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2731d9ae, SUC_lexicon_tags_25 },
    { 0x7871d9af, SUC_lexicon_tags_870 },
    { 0, NULL },
    { 0x9351d9b1, SUC_lexicon_tags_16 },
    { 0x5191d9b2, SUC_lexicon_tags_19 },
    { 0x44f1d9b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3811d9b7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf691d9bf, SUC_lexicon_tags_15 },
    { 0x3651d9bf, SUC_lexicon_tags_11 },
    { 0xebb1d9bf, SUC_lexicon_tags_2 },
    { 0xf7f1d9c1, SUC_lexicon_tags_7 },
    { 0x7431d9c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f1d9c9, SUC_lexicon_tags_2 },
    { 0xe651d9ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd871d9cc, SUC_lexicon_tags_25 },
    { 0xab71d9cd, SUC_lexicon_tags_10 },
    { 0x3971d9cd, SUC_lexicon_tags_4 },
    { 0x19b1d9cf, SUC_lexicon_tags_15 },
    { 0x1011d9ce, SUC_lexicon_tags_1 },
    { 0x28d1d9d1, SUC_lexicon_tags_17 },
    { 0x1fd1d9cc, SUC_lexicon_tags_35 },
    { 0x9bb1d9d0, SUC_lexicon_tags_27 },
    { 0x4491d9d4, SUC_lexicon_tags_37 },
    { 0x11b1d9d5, SUC_lexicon_tags_47 },
    { 0x2a31d9d4, SUC_lexicon_tags_45 },
    { 0xb711d9d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f1d9da, SUC_lexicon_tags_15 },
    { 0x2771d9db, SUC_lexicon_tags_12 },
    { 0x8b51d9dc, SUC_lexicon_tags_17 },
    { 0x8cd1d9db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3b1d9df, SUC_lexicon_tags_1 },
    { 0x36f1d9e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd391d9e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5051d9e4, SUC_lexicon_tags_37 },
    { 0x2a91d9e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d1d9eb, SUC_lexicon_tags_37 },
    { 0x3dd1d9eb, SUC_lexicon_tags_103 },
    { 0x78d1d9ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591d9f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc211d9f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1d9fa, SUC_lexicon_tags_14 },
    { 0x6111d9fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf31d9fe, SUC_lexicon_tags_29 },
    { 0xb1b1d9ff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3bf1da01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2571da04, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9231da07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c91da0a, SUC_lexicon_tags_11 },
    { 0xd431da0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f1da12, SUC_lexicon_tags_17 },
    { 0x7a91da12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b1da19, SUC_lexicon_tags_26 },
    { 0x4ab1da19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5171da1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf91da21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad1da25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99f1da27, SUC_lexicon_tags_17 },
    { 0x6b11da28, SUC_lexicon_tags_35 },
    { 0x55b1da29, SUC_lexicon_tags_60 },
    { 0xa231da2a, SUC_lexicon_tags_14 },
    { 0xf291da2b, SUC_lexicon_tags_11 },
    { 0x62b1da27, SUC_lexicon_tags_29 },
    { 0x5f11da2a, SUC_lexicon_tags_2 },
    { 0xffd1da2e, SUC_lexicon_tags_27 },
    { 0xadd1da2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2f1da31, SUC_lexicon_tags_25 },
    { 0xc211da32, SUC_lexicon_tags_47 },
    { 0xfdd1da33, SUC_lexicon_tags_2 },
    { 0xcff1da34, SUC_lexicon_tags_11 },
    { 0x2eb1da31, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9631da37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1d1da39, SUC_lexicon_tags_236 },
    { 0x5311da3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb871da3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf291da3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6391da40, SUC_lexicon_tags_11 },
    { 0xffd1da41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc531da43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4871da47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf711da49, SUC_lexicon_tags_12 },
    { 0x1cb1da49, SUC_lexicon_tags_68 },
    { 0x25f1da4b, SUC_lexicon_tags_65 },
    { 0x6511da4c, SUC_lexicon_tags_33 },
    { 0x1751da4d, SUC_lexicon_tags_37 },
    { 0x7f11da4e, SUC_lexicon_tags_38 },
    { 0xf751da4c, SUC_lexicon_tags_19 },
    { 0x6831da50, SUC_lexicon_tags_7 },
    { 0x20d1da51, SUC_lexicon_tags_45 },
    { 0x6371da4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b1da55, SUC_lexicon_tags_68 },
    { 0x78d1da55, SUC_lexicon_tags_7 },
    { 0x4dd1da57, SUC_lexicon_tags_1 },
    { 0xbe51da57, SUC_lexicon_tags_37 },
    { 0x31da56, SUC_lexicon_tags_25 },
    { 0xa091da57, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7551da5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a71da5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c91da61, SUC_lexicon_tags_1 },
    { 0x3411da61, SUC_lexicon_tags_25 },
    { 0xa2b1da63, SUC_lexicon_tags_1 },
    { 0xb3b1da62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x471da67, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8b31da69, SUC_lexicon_tags_80 },
    { 0xa771da6a, SUC_lexicon_tags_65 },
    { 0x2db1da6b, SUC_lexicon_tags_2 },
    { 0xcef1da69, SUC_lexicon_tags_11 },
    { 0xe531da69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d1da74, SUC_lexicon_tags_14 },
    { 0xae31da75, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6431da7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f1da7e, SUC_lexicon_tags_37 },
    { 0xaf91da7f, SUC_lexicon_tags_11 },
    { 0x3931da7e, SUC_lexicon_tags_47 },
    { 0xc351da81, SUC_lexicon_tags_7 },
    { 0x97f1da82, SUC_lexicon_tags_17 },
    { 0xe471da83, SUC_lexicon_tags_12 },
    { 0xa8b1da81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb1da88, SUC_lexicon_tags_25 },
    { 0x5271da88, SUC_lexicon_tags_15 },
    { 0xc811da8a, SUC_lexicon_tags_33 },
    { 0xe51da8b, SUC_lexicon_tags_38 },
    { 0x6b11da8c, SUC_lexicon_tags_63 },
    { 0x1d11da8d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3951da90, SUC_lexicon_tags_15 },
    { 0x9411da91, SUC_lexicon_tags_7 },
    { 0x2711da92, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca71da97, SUC_lexicon_tags_70 },
    { 0x6731da98, SUC_lexicon_tags_2 },
    { 0xc871da99, SUC_lexicon_tags_29 },
    { 0x70d1da99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d71da9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81f1daa0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1daa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab1daa6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe471daa8, SUC_lexicon_tags_28 },
    { 0xfa91daa8, SUC_lexicon_tags_6 },
    { 0xfa71daaa, SUC_lexicon_tags_66 },
    { 0x3d71daab, SUC_lexicon_tags_11 },
    { 0xea91daac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7851dab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9251dab4, SUC_lexicon_tags_11 },
    { 0x2bd1dab5, SUC_lexicon_tags_39 },
    { 0x36d1dab5, SUC_lexicon_tags_60 },
    { 0xdc71dab5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa971daba, SUC_lexicon_tags_37 },
    { 0xc4f1dabb, SUC_lexicon_tags_15 },
    { 0x26f1dabc, SUC_lexicon_tags_27 },
    { 0xbb91dabd, SUC_lexicon_tags_25 },
    { 0x7251dabe, SUC_lexicon_tags_1 },
    { 0x5431dabe, SUC_lexicon_tags_15 },
    { 0xe191dac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f1dac4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4231dac8, SUC_lexicon_tags_25 },
    { 0xcff1dac9, SUC_lexicon_tags_1 },
    { 0xb3b1dac9, SUC_lexicon_tags_17 },
    { 0xe251dac9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c71dad5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef1dad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1071dadd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf1dae4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc291daea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8f1daec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b51daee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc571daf3, SUC_lexicon_tags_49 },
    { 0xc0d1daf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf451daf6, SUC_lexicon_tags_49 },
    { 0xfe51daf7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca51dafb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac71daff, SUC_lexicon_tags_25 },
    { 0x5ad1db00, SUC_lexicon_tags_25 },
    { 0x34f1db01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef1db08, SUC_lexicon_tags_66 },
    { 0xfb91db09, SUC_lexicon_tags_25 },
    { 0xc011db0a, SUC_lexicon_tags_83 },
    { 0xbcf1db0b, SUC_lexicon_tags_1 },
    { 0xdaf1db0a, SUC_lexicon_tags_43 },
    { 0xd691db0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9bd1db0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd51db15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9f1db17, SUC_lexicon_tags_31 },
    { 0x8831db18, SUC_lexicon_tags_12 },
    { 0xfb31db17, SUC_lexicon_tags_25 },
    { 0x9871db1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd591db20, SUC_lexicon_tags_37 },
    { 0xbe71db21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b1db24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc931db26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b1db2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c51db2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe111db2f, SUC_lexicon_tags_11 },
    { 0xaf1db30, SUC_lexicon_tags_37 },
    { 0xc631db31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8651db34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4651db37, SUC_lexicon_tags_37 },
    { 0xfa11db38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd1db3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b1db3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31db42, SUC_lexicon_tags_63 },
    { 0xbd51db43, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b1db4a, SUC_lexicon_tags_60 },
    { 0x3b1db4b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4571db54, SUC_lexicon_tags_60 },
    { 0x6591db55, SUC_lexicon_tags_31 },
    { 0x6851db56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d11db59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2df1db5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb691db5f, SUC_lexicon_tags_17 },
    { 0x2c51db60, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9111db62, SUC_lexicon_tags_11 },
    { 0x9bf1db63, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8431db67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f51db6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb1db71, SUC_lexicon_tags_1 },
    { 0xee31db71, SUC_lexicon_tags_68 },
    { 0xd691db72, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2351db7a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf1db7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe631db87, SUC_lexicon_tags_37 },
    { 0xbcd1db88, SUC_lexicon_tags_85 },
    { 0x1051db89, SUC_lexicon_tags_8 },
    { 0x7ab1db8a, SUC_lexicon_tags_11 },
    { 0x6631db8a, SUC_lexicon_tags_0 },
    { 0x6ab1db88, SUC_lexicon_tags_32 },
    { 0x1b11db89, SUC_lexicon_tags_23 },
    { 0x291db88, SUC_lexicon_tags_25 },
    { 0xf871db8d, SUC_lexicon_tags_12 },
    { 0x1af1db90, SUC_lexicon_tags_87 },
    { 0xd951db89, SUC_lexicon_tags_69 },
    { 0xce51db92, SUC_lexicon_tags_1 },
    { 0xde51db92, SUC_lexicon_tags_7 },
    { 0x7d51db94, SUC_lexicon_tags_37 },
    { 0x50b1db95, SUC_lexicon_tags_47 },
    { 0x9df1db92, SUC_lexicon_tags_8 },
    { 0xf471db8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d1db9b, SUC_lexicon_tags_22 },
    { 0x9f31db9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd1db9f, SUC_lexicon_tags_23 },
    { 0xee71db9f, SUC_lexicon_tags_57 },
    { 0xafd1dba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa111dba3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb8b1dba5, SUC_lexicon_tags_7 },
    { 0x12f1dba6, SUC_lexicon_tags_10 },
    { 0xccb1dba7, SUC_lexicon_tags_37 },
    { 0x1b71dba7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c51dbae, SUC_lexicon_tags_25 },
    { 0x7a31dbaf, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x2991dbb1, SUC_lexicon_tags_1 },
    { 0x78d1dbb2, SUC_lexicon_tags_60 },
    { 0xe0f1dbb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb31dbb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd791dbb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fd1dbba, SUC_lexicon_tags_25 },
    { 0x8131dbba, SUC_lexicon_tags_11 },
    { 0x68b1dbba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b51dbbe, SUC_lexicon_tags_23 },
    { 0x1e11dbbe, SUC_lexicon_tags_26 },
    { 0x5271dbc0, SUC_lexicon_tags_25 },
    { 0x7691dbc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dbc6, SUC_lexicon_tags_7 },
    { 0x58b1dbc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9811dbca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d51dbcd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb751dbcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6591dbd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d11dbd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe671dbe3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdff1dbe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb1dbec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9971dbf1, SUC_lexicon_tags_79 },
    { 0xbb31dbf1, SUC_lexicon_tags_87 },
    { 0x1eb1dbf3, SUC_lexicon_tags_21 },
    { 0x94f1dbf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5891dbf7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc831dbfb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951dc03, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb31dc08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50b1dc0a, SUC_lexicon_tags_47 },
    { 0xfb11dc0a, SUC_lexicon_tags_47 },
    { 0xa4f1dc0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9f1dc0e, SUC_lexicon_tags_12 },
    { 0xec91dc0f, SUC_lexicon_tags_23 },
    { 0x45f1dc10, SUC_lexicon_tags_13 },
    { 0x3ad1dc11, SUC_lexicon_tags_61 },
    { 0xf251dc11, SUC_lexicon_tags_7 },
    { 0xd411dc12, SUC_lexicon_tags_17 },
    { 0x7db1dc0e, SUC_lexicon_tags_12 },
    { 0x9411dc14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb871dc1e, SUC_lexicon_tags_14 },
    { 0x6b1dc1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb71dc21, SUC_lexicon_tags_30 },
    { 0xa671dc21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd11dc27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f1dc2b, SUC_lexicon_tags_12 },
    { 0xb751dc2c, SUC_lexicon_tags_15 },
    { 0x1c11dc2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c51dc30, SUC_lexicon_tags_17 },
    { 0x4c11dc31, SUC_lexicon_tags_2 },
    { 0x4351dc32, SUC_lexicon_tags_17 },
    { 0x77d1dc32, SUC_lexicon_tags_37 },
    { 0x9491dc34, SUC_lexicon_tags_26 },
    { 0xcfd1dc32, SUC_lexicon_tags_21 },
    { 0x7831dc36, SUC_lexicon_tags_45 },
    { 0x3191dc37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc771dc3b, SUC_lexicon_tags_47 },
    { 0xaf71dc3b, SUC_lexicon_tags_6 },
    { 0xb5d1dc3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4af1dc3f, SUC_lexicon_tags_37 },
    { 0xcc71dc40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d1dc43, SUC_lexicon_tags_9 },
    { 0xae11dc44, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31dc4d, SUC_lexicon_tags_14 },
    { 0xce71dc4e, SUC_lexicon_tags_31 },
    { 0x5cb1dc4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf251dc54, SUC_lexicon_tags_48 },
    { 0xe711dc55, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8591dc58, SUC_lexicon_tags_8 },
    { 0xcb31dc59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d11dc5d, SUC_lexicon_tags_19 },
    { 0x66f1dc5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfb1dc60, SUC_lexicon_tags_38 },
    { 0x57f1dc60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f1dc6a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d1dc6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c31dc7e, SUC_lexicon_tags_45 },
    { 0x3251dc7f, SUC_lexicon_tags_0 },
    { 0xaeb1dc80, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c71dc8b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f91dc94, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7611dc9c, SUC_lexicon_tags_7 },
    { 0xe0b1dc9d, SUC_lexicon_tags_21 },
    { 0xab71dc9e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd131dca0, SUC_lexicon_tags_12 },
    { 0xc291dca1, SUC_lexicon_tags_25 },
    { 0x98d1dca2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d1dca6, SUC_lexicon_tags_23 },
    { 0xf271dca6, SUC_lexicon_tags_32 },
    { 0x46d1dca8, SUC_lexicon_tags_106 },
    { 0xa751dca8, SUC_lexicon_tags_1 },
    { 0xf4d1dcaa, SUC_lexicon_tags_63 },
    { 0x1711dca7, SUC_lexicon_tags_43 },
    { 0xfdd1dcac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2451dcae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f1dcb5, SUC_lexicon_tags_21 },
    { 0xff31dcb5, SUC_lexicon_tags_2 },
    { 0xcdf1dcb6, SUC_lexicon_tags_11 },
    { 0xb211dcb8, SUC_lexicon_tags_37 },
    { 0x5cf1dcb9, SUC_lexicon_tags_84 },
    { 0x3631dcba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xce71dcbd, SUC_lexicon_tags_38 },
    { 0xded1dcbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8631dcc0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1471dcc4, SUC_lexicon_tags_857 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b91dcc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4551dccb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3871dccf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a91dcd4, SUC_lexicon_tags_19 },
    { 0x81f1dcd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab11dcda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8d1dcdc, SUC_lexicon_tags_26 },
    { 0xe7b1dcdd, SUC_lexicon_tags_13 },
    { 0xd311dcde, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe8d1dce0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa51dceb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa511dced, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc91dcf1, SUC_lexicon_tags_23 },
    { 0xc271dcf2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4311dcf6, SUC_lexicon_tags_37 },
    { 0x7df1dcf7, SUC_lexicon_tags_38 },
    { 0xd9b1dcf8, SUC_lexicon_tags_60 },
    { 0x3f1dcf9, SUC_lexicon_tags_19 },
    { 0xbf11dcfa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc991dcfc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1511dcfe, SUC_lexicon_tags_17 },
    { 0x9811dcff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf11dd04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb791dd06, SUC_lexicon_tags_14 },
    { 0x5211dd07, SUC_lexicon_tags_7 },
    { 0x71d1dd06, SUC_lexicon_tags_28 },
    { 0x8fb1dd06, SUC_lexicon_tags_26 },
    { 0x2451dd0a, SUC_lexicon_tags_11 },
    { 0x3df1dd06, SUC_lexicon_tags_2 },
    { 0x5fb1dd0a, SUC_lexicon_tags_47 },
    { 0x6d11dd0d, SUC_lexicon_tags_17 },
    { 0x75f1dd0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc851dd10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5231dd12, SUC_lexicon_tags_37 },
    { 0xa131dd12, SUC_lexicon_tags_15 },
    { 0x8df1dd13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a91dd17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c71dd1a, SUC_lexicon_tags_21 },
    { 0xcb1dd1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac91dd1e, SUC_lexicon_tags_49 },
    { 0xe3d1dd1f, SUC_lexicon_tags_0 },
    { 0xa951dd1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc231dd22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8f31dd24, SUC_lexicon_tags_2 },
    { 0xca71dd24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd091dd28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe811dd2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d51dd2e, SUC_lexicon_tags_7 },
    { 0xb911dd2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4551dd35, SUC_lexicon_tags_14 },
    { 0x17d1dd35, SUC_lexicon_tags_1 },
    { 0x74f1dd37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291dd3b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51dd42, SUC_lexicon_tags_9 },
    { 0xd431dd43, SUC_lexicon_tags_82 },
    { 0xb731dd44, SUC_lexicon_tags_0 },
    { 0x51d1dd45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf351dd49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd71dd4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f31dd51, SUC_lexicon_tags_25 },
    { 0xf3b1dd52, SUC_lexicon_tags_69 },
    { 0xdbd1dd53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe891dd55, SUC_lexicon_tags_25 },
    { 0x5e91dd55, SUC_lexicon_tags_14 },
    { 0x4491dd57, SUC_lexicon_tags_15 },
    { 0x6651dd58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91dd5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f31dd61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d31dd64, SUC_lexicon_tags_30 },
    { 0x2291dd64, SUC_lexicon_tags_47 },
    { 0x6ff1dd66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8411dd68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8971dd6d, SUC_lexicon_tags_11 },
    { 0xba51dd6e, SUC_lexicon_tags_9 },
    { 0x95d1dd6f, SUC_lexicon_tags_66 },
    { 0x73f1dd70, SUC_lexicon_tags_1 },
    { 0xd071dd71, SUC_lexicon_tags_11 },
    { 0x8e11dd71, SUC_lexicon_tags_2 },
    { 0x91b1dd71, SUC_lexicon_tags_66 },
    { 0x8491dd6d, SUC_lexicon_tags_19 },
    { 0x20b1dd75, SUC_lexicon_tags_37 },
    { 0x6ad1dd76, SUC_lexicon_tags_27 },
    { 0x52f1dd75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb1dd7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf1dd7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x591dd80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5271dd83, SUC_lexicon_tags_17 },
    { 0xf951dd84, SUC_lexicon_tags_27 },
    { 0xa8b1dd84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb651dd8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6611dd8e, SUC_lexicon_tags_184 },
    { 0xa211dd8e, SUC_lexicon_tags_8 },
    { 0xad1dd8e, SUC_lexicon_tags_104 },
    { 0x78f1dd91, SUC_lexicon_tags_35 },
    { 0xf31dd92, SUC_lexicon_tags_12 },
    { 0xf9f1dd8f, SUC_lexicon_tags_23 },
    { 0xe4f1dd94, SUC_lexicon_tags_11 },
    { 0xa791dd95, SUC_lexicon_tags_55 },
    { 0xf611dd8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d31dd99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b1dda0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x151dda2, SUC_lexicon_tags_60 },
    { 0x6f71dda3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa951dda8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fb1ddaa, SUC_lexicon_tags_26 },
    { 0x111ddaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e31ddaf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xff1ddb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f1ddb7, SUC_lexicon_tags_8 },
    { 0xd091ddb8, SUC_lexicon_tags_26 },
    { 0xe4d1ddb7, SUC_lexicon_tags_25 },
    { 0xe8f1ddba, SUC_lexicon_tags_11 },
    { 0xd791ddb7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf11ddc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8711ddc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b31ddca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd1ddcd, SUC_lexicon_tags_1 },
    { 0x5ad1ddce, SUC_lexicon_tags_12 },
    { 0x74b1ddcf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb1ddd3, SUC_lexicon_tags_1 },
    { 0xca31ddd4, SUC_lexicon_tags_47 },
    { 0x1ed1ddd3, SUC_lexicon_tags_2 },
    { 0xdd11ddd6, SUC_lexicon_tags_37 },
    { 0x7151ddd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b71dde3, SUC_lexicon_tags_87 },
    { 0x3d51dde3, SUC_lexicon_tags_1 },
    { 0x1f51dde4, SUC_lexicon_tags_10 },
    { 0x38b1dde3, SUC_lexicon_tags_17 },
    { 0x9e51dde5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcd1dde9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f1ddee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d1ddf0, SUC_lexicon_tags_31 },
    { 0xbb51ddf0, SUC_lexicon_tags_1 },
    { 0x2c31ddf2, SUC_lexicon_tags_60 },
    { 0x60b1ddf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66d1ddf5, SUC_lexicon_tags_0 },
    { 0x8411ddf6, SUC_lexicon_tags_17 },
    { 0xa231ddf6, SUC_lexicon_tags_17 },
    { 0x40f1ddf8, SUC_lexicon_tags_11 },
    { 0x2a51ddf5, SUC_lexicon_tags_25 },
    { 0x9111ddf9, SUC_lexicon_tags_15 },
    { 0xdd91ddf5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa271de03, SUC_lexicon_tags_1 },
    { 0xbc31de03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1de09, SUC_lexicon_tags_8 },
    { 0x5db1de0a, SUC_lexicon_tags_17 },
    { 0x39f1de0b, SUC_lexicon_tags_61 },
    { 0xb611de0c, SUC_lexicon_tags_5 },
    { 0x8331de0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa651de1c, SUC_lexicon_tags_12 },
    { 0xa5d1de1d, SUC_lexicon_tags_25 },
    { 0x4731de1e, SUC_lexicon_tags_15 },
    { 0x8b11de1f, SUC_lexicon_tags_16 },
    { 0xaa51de1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x79f1de22, SUC_lexicon_tags_56 },
    { 0x1b91de23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6691de26, SUC_lexicon_tags_2 },
    { 0xd311de27, SUC_lexicon_tags_25 },
    { 0x6231de26, SUC_lexicon_tags_11 },
    { 0x3e91de29, SUC_lexicon_tags_28 },
    { 0x79d1de29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7291de2c, SUC_lexicon_tags_17 },
    { 0xd0b1de2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791de34, SUC_lexicon_tags_28 },
    { 0x54f1de35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdcb1de37, SUC_lexicon_tags_1 },
    { 0xc251de37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b1de3c, SUC_lexicon_tags_16 },
    { 0xe0f1de3d, SUC_lexicon_tags_15 },
    { 0x2471de3e, SUC_lexicon_tags_2 },
    { 0x8bb1de3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6431de41, SUC_lexicon_tags_17 },
    { 0xa3f1de41, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7b51de44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8571de4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a31de4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf31de54, SUC_lexicon_tags_4 },
    { 0xd351de54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe351de57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b1de5a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd1de5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b91de5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a31de61, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8271de66, SUC_lexicon_tags_45 },
    { 0xf4b1de67, SUC_lexicon_tags_27 },
    { 0xbe11de67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1e31de6a, SUC_lexicon_tags_95 },
    { 0x3031de6a, SUC_lexicon_tags_10 },
    { 0xb511de6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9571de71, SUC_lexicon_tags_48 },
    { 0xd271de72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb051de74, SUC_lexicon_tags_710 },
    { 0x61d1de74, SUC_lexicon_tags_11 },
    { 0xbbb1de76, SUC_lexicon_tags_30 },
    { 0x6c11de77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcef1de79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa431de7b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x311de7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef71de83, SUC_lexicon_tags_43 },
    { 0xd411de83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90d1de86, SUC_lexicon_tags_17 },
    { 0x891de86, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf91de8a, SUC_lexicon_tags_25 },
    { 0xe11de8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xac71de8d, SUC_lexicon_tags_15 },
    { 0xe4d1de8e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b51de90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3851de93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe791de96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb471de9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d91de9f, SUC_lexicon_tags_25 },
    { 0x3bb1dea0, SUC_lexicon_tags_104 },
    { 0x5ff1dea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8851dea4, SUC_lexicon_tags_2 },
    { 0x34d1dea4, SUC_lexicon_tags_21 },
    { 0x4f31dea6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd1deac, SUC_lexicon_tags_45 },
    { 0x22d1deac, SUC_lexicon_tags_37 },
    { 0x77d1deae, SUC_lexicon_tags_43 },
    { 0xa071deae, SUC_lexicon_tags_4 },
    { 0x1071deb0, SUC_lexicon_tags_17 },
    { 0x9251deb1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5771deb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b1deb7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f1debb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf1dec4, SUC_lexicon_tags_33 },
    { 0x1c91dec5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3871dec9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4011decb, SUC_lexicon_tags_4 },
    { 0xdd51decb, SUC_lexicon_tags_25 },
    { 0x3091decd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x79b1decf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa471ded2, SUC_lexicon_tags_37 },
    { 0xae51ded3, SUC_lexicon_tags_101 },
    { 0xf771ded4, SUC_lexicon_tags_11 },
    { 0x2d1ded5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf1ded8, SUC_lexicon_tags_79 },
    { 0xc7b1ded9, SUC_lexicon_tags_2 },
    { 0x9ad1deda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4351dee9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd1def2, SUC_lexicon_tags_17 },
    { 0x7a31def3, SUC_lexicon_tags_11 },
    { 0x28d1def4, SUC_lexicon_tags_25 },
    { 0x51b1def4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5411defd, SUC_lexicon_tags_6 },
    { 0x8031defe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1b1df00, SUC_lexicon_tags_84 },
    { 0x9a11df00, SUC_lexicon_tags_2 },
    { 0x6ef1df02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa751df05, SUC_lexicon_tags_1 },
    { 0x84f1df06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db1df0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6b51df0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa131df0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbb1df10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5011df12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf1df15, SUC_lexicon_tags_29 },
    { 0x9db1df16, SUC_lexicon_tags_0 },
    { 0x9231df16, SUC_lexicon_tags_15 },
    { 0x691df17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e91df1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1ed1df1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc831df26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc91df29, SUC_lexicon_tags_27 },
    { 0x8c91df2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9051df2c, SUC_lexicon_tags_17 },
    { 0x27d1df2c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8631df30, SUC_lexicon_tags_12 },
    { 0x1b91df31, SUC_lexicon_tags_0 },
    { 0x61f1df32, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff1df35, SUC_lexicon_tags_47 },
    { 0x6c11df36, SUC_lexicon_tags_1436 },
    { 0, NULL },
    { 0, NULL },
    { 0xd351df39, SUC_lexicon_tags_37 },
    { 0xaeb1df3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b1df3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e71df40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b51df43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa111df45, SUC_lexicon_tags_73 },
    { 0x67b1df46, SUC_lexicon_tags_2 },
    { 0xcaf1df45, SUC_lexicon_tags_62 },
    { 0x9fb1df47, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7d11df4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9051df4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e31df52, SUC_lexicon_tags_28 },
    { 0x3811df53, SUC_lexicon_tags_12 },
    { 0xffd1df52, SUC_lexicon_tags_15 },
    { 0x1631df55, SUC_lexicon_tags_25 },
    { 0xe891df56, SUC_lexicon_tags_17 },
    { 0xb9d1df56, SUC_lexicon_tags_10 },
    { 0x92b1df56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7091df5a, SUC_lexicon_tags_60 },
    { 0x7191df5b, SUC_lexicon_tags_1 },
    { 0x38f1df5c, SUC_lexicon_tags_19 },
    { 0xcd11df5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b1df64, SUC_lexicon_tags_33 },
    { 0x61b1df64, SUC_lexicon_tags_160 },
    { 0x3cb1df66, SUC_lexicon_tags_45 },
    { 0x631df66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f31df6b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9451df6e, SUC_lexicon_tags_25 },
    { 0x8f51df6f, SUC_lexicon_tags_63 },
    { 0x8111df6e, SUC_lexicon_tags_17 },
    { 0x7ad1df6f, SUC_lexicon_tags_6 },
    { 0xdf31df71, SUC_lexicon_tags_17 },
    { 0x7cf1df71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e31df75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32f1df77, SUC_lexicon_tags_38 },
    { 0x2b71df78, SUC_lexicon_tags_0 },
    { 0x7491df77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a91df82, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f1df86, SUC_lexicon_tags_12 },
    { 0x2011df87, SUC_lexicon_tags_11 },
    { 0xdd31df88, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1df8e, SUC_lexicon_tags_2 },
    { 0xbbf1df8f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30f1df91, SUC_lexicon_tags_48 },
    { 0xbd91df92, SUC_lexicon_tags_39 },
    { 0xdcb1df91, SUC_lexicon_tags_23 },
    { 0x91f1df91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf451df99, SUC_lexicon_tags_19 },
    { 0x6171df9a, SUC_lexicon_tags_1 },
    { 0x5a91df9a, SUC_lexicon_tags_47 },
    { 0x10f1df9c, SUC_lexicon_tags_43 },
    { 0x6311df9d, SUC_lexicon_tags_15 },
    { 0xbb1df9e, SUC_lexicon_tags_16 },
    { 0x4c91df9c, SUC_lexicon_tags_37 },
    { 0xb691dfa0, SUC_lexicon_tags_91 },
    { 0xbd1dfa1, SUC_lexicon_tags_19 },
    { 0xa5b1dfa1, SUC_lexicon_tags_17 },
    { 0xb6d1dfa3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x79f1dfa5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x27d1dfa7, SUC_lexicon_tags_12 },
    { 0x3ab1dfa7, SUC_lexicon_tags_25 },
    { 0x47b1dfa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f11dfac, SUC_lexicon_tags_45 },
    { 0x4451dfad, SUC_lexicon_tags_52 },
    { 0x93f1dfae, SUC_lexicon_tags_12 },
    { 0xce11dfaf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcb51dfb1, SUC_lexicon_tags_54 },
    { 0x4551dfb2, SUC_lexicon_tags_31 },
    { 0xec11dfb3, SUC_lexicon_tags_2 },
    { 0x651dfb4, SUC_lexicon_tags_1 },
    { 0xf371dfb4, SUC_lexicon_tags_1 },
    { 0x5351dfb6, SUC_lexicon_tags_47 },
    { 0x86f1dfb7, SUC_lexicon_tags_33 },
    { 0xc071dfb8, SUC_lexicon_tags_5 },
    { 0xe051dfb8, SUC_lexicon_tags_60 },
    { 0x22f1dfb1, SUC_lexicon_tags_21 },
    { 0xaa11dfb5, SUC_lexicon_tags_57 },
    { 0xa691dfbc, SUC_lexicon_tags_38 },
    { 0xb931dfb4, SUC_lexicon_tags_57 },
    { 0x1eb1dfbd, SUC_lexicon_tags_12 },
    { 0x8af1dfba, SUC_lexicon_tags_32 },
    { 0xc2d1dfb3, SUC_lexicon_tags_7 },
    { 0x951dfbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b1dfc6, SUC_lexicon_tags_32 },
    { 0x3751dfc7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b1dfcc, SUC_lexicon_tags_23 },
    { 0x7df1dfcc, SUC_lexicon_tags_19 },
    { 0x5211dfcd, SUC_lexicon_tags_29 },
    { 0x3171dfcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b1dfd1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3071dfdc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2691dfe0, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb51dfe4, SUC_lexicon_tags_7 },
    { 0x43b1dfe5, SUC_lexicon_tags_37 },
    { 0x4411dfe5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b31dfe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe31dfec, SUC_lexicon_tags_27 },
    { 0xd5d1dfed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2071dff0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6611dff5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d1dff8, SUC_lexicon_tags_37 },
    { 0x3671dff8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2131dffc, SUC_lexicon_tags_14 },
    { 0x3671dffc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xfeb1dfff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9bf1e001, SUC_lexicon_tags_84 },
    { 0xe1f1e001, SUC_lexicon_tags_15 },
    { 0x2091e003, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1791e009, SUC_lexicon_tags_69 },
    { 0x9011e00a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11e00d, SUC_lexicon_tags_27 },
    { 0x2f91e00e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b1e011, SUC_lexicon_tags_11 },
    { 0x41b1e012, SUC_lexicon_tags_15 },
    { 0x5f91e013, SUC_lexicon_tags_25 },
    { 0x71b1e011, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8d1e016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9031e018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73f1e01a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe811e01c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd1e01f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e71e022, SUC_lexicon_tags_15 },
    { 0x9cd1e023, SUC_lexicon_tags_15 },
    { 0x4251e024, SUC_lexicon_tags_108 },
    { 0xc371e023, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf11e027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b11e02e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7851e030, SUC_lexicon_tags_25 },
    { 0xbf91e030, SUC_lexicon_tags_4 },
    { 0x24b1e030, SUC_lexicon_tags_0 },
    { 0x3df1e033, SUC_lexicon_tags_15 },
    { 0xf51e034, SUC_lexicon_tags_11 },
    { 0x4951e034, SUC_lexicon_tags_7 },
    { 0xe031e031, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad1e03c, SUC_lexicon_tags_7 },
    { 0x2fd1e03d, SUC_lexicon_tags_35 },
    { 0x3551e03d, SUC_lexicon_tags_9 },
    { 0xff71e03f, SUC_lexicon_tags_11 },
    { 0xa651e03d, SUC_lexicon_tags_112 },
    { 0x89d1e041, SUC_lexicon_tags_37 },
    { 0xae71e042, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8471e044, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc71e047, SUC_lexicon_tags_9 },
    { 0xf8d1e048, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb291e04f, SUC_lexicon_tags_25 },
    { 0x5491e04f, SUC_lexicon_tags_37 },
    { 0x83b1e051, SUC_lexicon_tags_7 },
    { 0xadd1e052, SUC_lexicon_tags_63 },
    { 0xde31e04f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e05a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5771e05d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9431e063, SUC_lexicon_tags_23 },
    { 0x5a11e064, SUC_lexicon_tags_25 },
    { 0x8831e064, SUC_lexicon_tags_9 },
    { 0xa9b1e066, SUC_lexicon_tags_21 },
    { 0x74d1e066, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x391e06a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f1e071, SUC_lexicon_tags_7 },
    { 0xeaf1e072, SUC_lexicon_tags_0 },
    { 0xbf11e073, SUC_lexicon_tags_31 },
    { 0x4bf1e074, SUC_lexicon_tags_31 },
    { 0xd191e073, SUC_lexicon_tags_7 },
    { 0x9091e072, SUC_lexicon_tags_33 },
    { 0x7611e077, SUC_lexicon_tags_30 },
    { 0x5bf1e078, SUC_lexicon_tags_11 },
    { 0x3871e079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa51e07b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6451e081, SUC_lexicon_tags_63 },
    { 0xbe91e082, SUC_lexicon_tags_12 },
    { 0x3dd1e082, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951e08a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e11e08c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6331e08e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc591e098, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e71e0a3, SUC_lexicon_tags_15 },
    { 0x3771e0a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9151e0ac, SUC_lexicon_tags_65 },
    { 0x43d1e0ad, SUC_lexicon_tags_33 },
    { 0x6031e0ad, SUC_lexicon_tags_7 },
    { 0x3871e0af, SUC_lexicon_tags_17 },
    { 0x46b1e0b0, SUC_lexicon_tags_60 },
    { 0xc4f1e0ac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd311e0ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb991e0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcaf1e0c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe571e0c5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe091e0c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x991e0c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5831e0cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xea31e0cf, SUC_lexicon_tags_9 },
    { 0x82b1e0cf, SUC_lexicon_tags_7 },
    { 0x1a91e0d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa451e0d6, SUC_lexicon_tags_35 },
    { 0x1371e0d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7931e0d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x671e0db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b71e0de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c11e0e0, SUC_lexicon_tags_63 },
    { 0xdc91e0e1, SUC_lexicon_tags_45 },
    { 0x20f1e0e2, SUC_lexicon_tags_25 },
    { 0x6e91e0e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b1e0ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4fd1e0ee, SUC_lexicon_tags_45 },
    { 0x40f1e0ef, SUC_lexicon_tags_12 },
    { 0xd751e0ee, SUC_lexicon_tags_2 },
    { 0xfbf1e0ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3591e0f4, SUC_lexicon_tags_11 },
    { 0xd5d1e0f5, SUC_lexicon_tags_49 },
    { 0x7311e0f6, SUC_lexicon_tags_1 },
    { 0xab31e0f7, SUC_lexicon_tags_35 },
    { 0xcc31e0f7, SUC_lexicon_tags_45 },
    { 0x2b31e0f9, SUC_lexicon_tags_4 },
    { 0xc951e0f9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xba91e0fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe2b1e0ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a31e101, SUC_lexicon_tags_15 },
    { 0x3791e101, SUC_lexicon_tags_2 },
    { 0xa31e101, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5871e106, SUC_lexicon_tags_7 },
    { 0xb5b1e107, SUC_lexicon_tags_19 },
    { 0x4571e108, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6191e10d, SUC_lexicon_tags_15 },
    { 0x8d11e10e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaef1e110, SUC_lexicon_tags_0 },
    { 0x9f11e111, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c91e114, SUC_lexicon_tags_15 },
    { 0x88d1e114, SUC_lexicon_tags_48 },
    { 0xcdf1e115, SUC_lexicon_tags_49 },
    { 0x6691e115, SUC_lexicon_tags_37 },
    { 0xb911e116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a11e11c, SUC_lexicon_tags_88 },
    { 0x4ef1e11d, SUC_lexicon_tags_7 },
    { 0xf2d1e11e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6991e120, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa6d1e122, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b1e12b, SUC_lexicon_tags_21 },
    { 0xdd51e12c, SUC_lexicon_tags_47 },
    { 0xae31e12d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb91e130, SUC_lexicon_tags_17 },
    { 0x6651e131, SUC_lexicon_tags_37 },
    { 0xa451e132, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf531e138, SUC_lexicon_tags_34 },
    { 0x8b51e139, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6591e142, SUC_lexicon_tags_7 },
    { 0xf231e142, SUC_lexicon_tags_26 },
    { 0x5491e142, SUC_lexicon_tags_11 },
    { 0xd1e145, SUC_lexicon_tags_37 },
    { 0x1131e145, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x64d1e148, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6531e14c, SUC_lexicon_tags_1 },
    { 0xd911e14d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4471e152, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x78d1e154, SUC_lexicon_tags_7 },
    { 0xd5d1e154, SUC_lexicon_tags_9 },
    { 0xee91e156, SUC_lexicon_tags_31 },
    { 0xcc31e157, SUC_lexicon_tags_1 },
    { 0xcad1e155, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7cb1e15a, SUC_lexicon_tags_7 },
    { 0x8151e15b, SUC_lexicon_tags_37 },
    { 0x8e91e15b, SUC_lexicon_tags_27 },
    { 0xb0d1e15d, SUC_lexicon_tags_28 },
    { 0xf2d1e15d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb1e164, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ff1e166, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x391e168, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6231e16b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c31e16d, SUC_lexicon_tags_19 },
    { 0xa231e16d, SUC_lexicon_tags_13 },
    { 0xf91e16f, SUC_lexicon_tags_25 },
    { 0x13b1e170, SUC_lexicon_tags_1 },
    { 0xfa91e170, SUC_lexicon_tags_45 },
    { 0x84f1e171, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7511e178, SUC_lexicon_tags_47 },
    { 0x1011e179, SUC_lexicon_tags_17 },
    { 0x7591e17a, SUC_lexicon_tags_234 },
    { 0xd371e17a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc771e18d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6771e191, SUC_lexicon_tags_63 },
    { 0x2e51e192, SUC_lexicon_tags_265 },
    { 0xc5b1e192, SUC_lexicon_tags_56 },
    { 0x20d1e192, SUC_lexicon_tags_14 },
    { 0xcab1e193, SUC_lexicon_tags_17 },
    { 0x1d71e192, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0x5cf1e198, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf1e19b, SUC_lexicon_tags_25 },
    { 0xffb1e19c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa91e19e, SUC_lexicon_tags_11 },
    { 0x3071e19f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d1e1a4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd31e1a7, SUC_lexicon_tags_0 },
    { 0x2bd1e1a8, SUC_lexicon_tags_0 },
    { 0x38b1e1a7, SUC_lexicon_tags_11 },
    { 0xd251e1aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f1e1af, SUC_lexicon_tags_9 },
    { 0xb11e1b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d91e1b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b71e1b5, SUC_lexicon_tags_37 },
    { 0xc331e1b6, SUC_lexicon_tags_15 },
    { 0x7bd1e1b6, SUC_lexicon_tags_2 },
    { 0x8211e1b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc211e1ba, SUC_lexicon_tags_29 },
    { 0x1d1e1ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f1e1c0, SUC_lexicon_tags_15 },
    { 0xacb1e1c1, SUC_lexicon_tags_37 },
    { 0x3f1e1c2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xce31e1c4, SUC_lexicon_tags_45 },
    { 0x1e71e1c5, SUC_lexicon_tags_7 },
    { 0x4551e1c6, SUC_lexicon_tags_37 },
    { 0x50d1e1c7, SUC_lexicon_tags_17 },
    { 0xd0f1e1c7, SUC_lexicon_tags_17 },
    { 0x7831e1c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf791e1cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf71e1cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d31e1d0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3fb1e1d2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb1e1d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10d1e1d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d71e1dd, SUC_lexicon_tags_60 },
    { 0x7611e1de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4131e1e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13d1e1e7, SUC_lexicon_tags_70 },
    { 0xdb51e1e8, SUC_lexicon_tags_27 },
    { 0xd51e1e9, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0x5eb1e1eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6071e1f0, SUC_lexicon_tags_2 },
    { 0x26b1e1f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd1e1f6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa291e1fc, SUC_lexicon_tags_12 },
    { 0x53f1e1fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd1e201, SUC_lexicon_tags_31 },
    { 0x6af1e201, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe811e206, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d71e20a, SUC_lexicon_tags_7 },
    { 0xc8d1e20b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb11e211, SUC_lexicon_tags_11 },
    { 0xe131e212, SUC_lexicon_tags_33 },
    { 0x2731e211, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d71e218, SUC_lexicon_tags_21 },
    { 0x5271e218, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1991e21b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd651e21d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd51e222, SUC_lexicon_tags_25 },
    { 0x59b1e222, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b1e228, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11d1e22a, SUC_lexicon_tags_65 },
    { 0x7f31e22b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9551e22e, SUC_lexicon_tags_68 },
    { 0xa831e22f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f1e232, SUC_lexicon_tags_2 },
    { 0x9d91e232, SUC_lexicon_tags_25 },
    { 0x62d1e234, SUC_lexicon_tags_17 },
    { 0xc571e235, SUC_lexicon_tags_1 },
    { 0x20d1e236, SUC_lexicon_tags_60 },
    { 0x2511e236, SUC_lexicon_tags_2 },
    { 0x61f1e238, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d1e23c, SUC_lexicon_tags_27 },
    { 0x5851e23c, SUC_lexicon_tags_63 },
    { 0xe691e23e, SUC_lexicon_tags_15 },
    { 0xc391e23d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x8691e241, SUC_lexicon_tags_37 },
    { 0xecb1e242, SUC_lexicon_tags_79 },
    { 0x8191e242, SUC_lexicon_tags_29 },
    { 0xbcb1e244, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d11e24a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b1e24d, SUC_lexicon_tags_14 },
    { 0x1451e24d, SUC_lexicon_tags_11 },
    { 0x7db1e24f, SUC_lexicon_tags_81 },
    { 0x6111e24e, SUC_lexicon_tags_47 },
    { 0xb9b1e251, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab31e253, SUC_lexicon_tags_30 },
    { 0x6b31e254, SUC_lexicon_tags_17 },
    { 0x8f31e255, SUC_lexicon_tags_7 },
    { 0x6271e256, SUC_lexicon_tags_1 },
    { 0xa7d1e255, SUC_lexicon_tags_37 },
    { 0xe1f1e254, SUC_lexicon_tags_65 },
    { 0x1f91e256, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3911e25d, SUC_lexicon_tags_68 },
    { 0x5771e25e, SUC_lexicon_tags_96 },
    { 0x66d1e25e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d1e263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0d1e265, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b1e26b, SUC_lexicon_tags_71 },
    { 0xecf1e26c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c91e26e, SUC_lexicon_tags_1 },
    { 0x9b11e26f, SUC_lexicon_tags_32 },
    { 0x3991e270, SUC_lexicon_tags_1 },
    { 0xf7b1e26f, SUC_lexicon_tags_2 },
    { 0x7711e270, SUC_lexicon_tags_1 },
    { 0x9f31e273, SUC_lexicon_tags_7 },
    { 0xf9d1e272, SUC_lexicon_tags_43 },
    { 0x75b1e275, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc491e27a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xea31e27c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5851e282, SUC_lexicon_tags_31 },
    { 0x431e283, SUC_lexicon_tags_0 },
    { 0xc211e282, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f1e28a, SUC_lexicon_tags_1 },
    { 0xe9b1e28b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x671e28e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb1e291, SUC_lexicon_tags_12 },
    { 0xf751e291, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d1e295, SUC_lexicon_tags_12 },
    { 0x5f51e295, SUC_lexicon_tags_27 },
    { 0xe1d1e297, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1091e29a, SUC_lexicon_tags_30 },
    { 0xe6b1e29a, SUC_lexicon_tags_11 },
    { 0xa8f1e29a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80d1e29e, SUC_lexicon_tags_42 },
    { 0x1471e29e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab51e2a1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7071e2a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab31e2a7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xacb1e2a9, SUC_lexicon_tags_12 },
    { 0xeb11e2aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3931e2ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd1e2b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d1e2b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc131e2ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa671e2be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49f1e2c0, SUC_lexicon_tags_11 },
    { 0xf6d1e2c1, SUC_lexicon_tags_5 },
    { 0x4f1e2c2, SUC_lexicon_tags_56 },
    { 0xc1d1e2c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfb1e2c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf1e2c8, SUC_lexicon_tags_17 },
    { 0x5cd1e2c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd451e2ce, SUC_lexicon_tags_47 },
    { 0x5a11e2ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3791e2d1, SUC_lexicon_tags_49 },
    { 0x2dd1e2d1, SUC_lexicon_tags_65 },
    { 0x71b1e2d3, SUC_lexicon_tags_11 },
    { 0x9b11e2d4, SUC_lexicon_tags_25 },
    { 0xcd71e2d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa831e2d7, SUC_lexicon_tags_1 },
    { 0x4d31e2d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a71e2dc, SUC_lexicon_tags_37 },
    { 0x9e51e2dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90f1e2df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4251e2e3, SUC_lexicon_tags_25 },
    { 0xd6b1e2e4, SUC_lexicon_tags_17 },
    { 0xa711e2e3, SUC_lexicon_tags_11 },
    { 0xf5b1e2e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3db1e2e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6311e2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf511e2f0, SUC_lexicon_tags_1 },
    { 0xe271e2f1, SUC_lexicon_tags_16 },
    { 0x4ff1e2f2, SUC_lexicon_tags_0 },
    { 0x9c31e2f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b11e2f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6f1e2f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f71e2fa, SUC_lexicon_tags_2 },
    { 0x6ad1e2fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbbb1e2fd, SUC_lexicon_tags_55 },
    { 0x3ab1e2fe, SUC_lexicon_tags_49 },
    { 0x5c91e2ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcb71e301, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x21b1e303, SUC_lexicon_tags_1 },
    { 0xdc11e304, SUC_lexicon_tags_47 },
    { 0xe871e305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed71e30a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df1e310, SUC_lexicon_tags_37 },
    { 0x3d1e311, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xced1e314, SUC_lexicon_tags_25 },
    { 0x5a11e315, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5b1e317, SUC_lexicon_tags_11 },
    { 0xd931e318, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf351e31c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2131e323, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6011e325, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfab1e327, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2191e329, SUC_lexicon_tags_7 },
    { 0x99d1e329, SUC_lexicon_tags_37 },
    { 0x9ed1e32b, SUC_lexicon_tags_0 },
    { 0xd1b1e329, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5211e32e, SUC_lexicon_tags_43 },
    { 0xe5f1e32f, SUC_lexicon_tags_37 },
    { 0xe371e330, SUC_lexicon_tags_11 },
    { 0xbd1e330, SUC_lexicon_tags_37 },
    { 0x2cd1e332, SUC_lexicon_tags_17 },
    { 0x6711e32f, SUC_lexicon_tags_4 },
    { 0xcc11e334, SUC_lexicon_tags_11 },
    { 0x22d1e335, SUC_lexicon_tags_11 },
    { 0x4d31e336, SUC_lexicon_tags_15 },
    { 0x8151e337, SUC_lexicon_tags_42 },
    { 0x3811e338, SUC_lexicon_tags_11 },
    { 0xfd71e337, SUC_lexicon_tags_47 },
    { 0xa711e33a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291e340, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11b1e342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff31e347, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711e34f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd391e352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea11e354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2f1e356, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf451e35a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5311e35c, SUC_lexicon_tags_37 },
    { 0xd751e35d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9231e35f, SUC_lexicon_tags_8 },
    { 0x251e360, SUC_lexicon_tags_16 },
    { 0x7671e360, SUC_lexicon_tags_15 },
    { 0x15b1e362, SUC_lexicon_tags_17 },
    { 0x69b1e360, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f1e368, SUC_lexicon_tags_2 },
    { 0xd8d1e369, SUC_lexicon_tags_40 },
    { 0xe091e369, SUC_lexicon_tags_8 },
    { 0xe551e36b, SUC_lexicon_tags_12 },
    { 0x39f1e36a, SUC_lexicon_tags_43 },
    { 0xb51e36d, SUC_lexicon_tags_4 },
    { 0xcfd1e36e, SUC_lexicon_tags_25 },
    { 0x9bf1e36f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf1e376, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa531e37a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2511e37f, SUC_lexicon_tags_19 },
    { 0x18b1e380, SUC_lexicon_tags_60 },
    { 0x6271e380, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbed1e383, SUC_lexicon_tags_11 },
    { 0x9d31e383, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb071e38d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d1e392, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7b1e394, SUC_lexicon_tags_12 },
    { 0x6e51e394, SUC_lexicon_tags_32 },
    { 0xfc91e395, SUC_lexicon_tags_17 },
    { 0xe791e397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f1e39a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a11e39f, SUC_lexicon_tags_9 },
    { 0x9ad1e3a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6991e3a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f11e3a6, SUC_lexicon_tags_82 },
    { 0x1f71e3a7, SUC_lexicon_tags_12 },
    { 0x33d1e3a8, SUC_lexicon_tags_32 },
    { 0x7af1e3a9, SUC_lexicon_tags_37 },
    { 0xc411e3aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd511e3b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8651e3b4, SUC_lexicon_tags_37 },
    { 0xddd1e3b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb871e3b8, SUC_lexicon_tags_31 },
    { 0x6c91e3b9, SUC_lexicon_tags_11 },
    { 0xcab1e3b8, SUC_lexicon_tags_14 },
    { 0x9d91e3ba, SUC_lexicon_tags_12 },
    { 0x73f1e3bc, SUC_lexicon_tags_37 },
    { 0x7491e3bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc151e3c1, SUC_lexicon_tags_17 },
    { 0x43d1e3c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e71e3c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff71e3cc, SUC_lexicon_tags_11 },
    { 0xc4d1e3cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b31e3d0, SUC_lexicon_tags_25 },
    { 0xae91e3d1, SUC_lexicon_tags_43 },
    { 0x6611e3d2, SUC_lexicon_tags_2 },
    { 0x1ab1e3d1, SUC_lexicon_tags_15 },
    { 0x7fd1e3d4, SUC_lexicon_tags_66 },
    { 0xea31e3d5, SUC_lexicon_tags_15 },
    { 0x8051e3d6, SUC_lexicon_tags_9 },
    { 0x8651e3d6, SUC_lexicon_tags_65 },
    { 0x8dd1e3d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef31e3da, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9391e3dd, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af1e3e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4d1e3e2, SUC_lexicon_tags_47 },
    { 0x7791e3e3, SUC_lexicon_tags_92 },
    { 0x9891e3e2, SUC_lexicon_tags_4 },
    { 0x3e51e3e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef1e3e8, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0xb6f1e3ea, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1871e3ec, SUC_lexicon_tags_1 },
    { 0xfdb1e3ed, SUC_lexicon_tags_7 },
    { 0x5751e3ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed1e3f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe651e3f6, SUC_lexicon_tags_0 },
    { 0x21b1e3f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2711e3fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe431e3fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd031e3fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6d1e400, SUC_lexicon_tags_2 },
    { 0x3011e401, SUC_lexicon_tags_2 },
    { 0x1831e402, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd791e404, SUC_lexicon_tags_15 },
    { 0x2151e404, SUC_lexicon_tags_2 },
    { 0xdfb1e405, SUC_lexicon_tags_55 },
    { 0xa231e407, SUC_lexicon_tags_17 },
    { 0x44d1e408, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x5371e40b, SUC_lexicon_tags_15 },
    { 0x7771e40c, SUC_lexicon_tags_48 },
    { 0xf391e40c, SUC_lexicon_tags_11 },
    { 0x3e51e40c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6b1e410, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1e413, SUC_lexicon_tags_91 },
    { 0xd331e414, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf1e419, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d1e41d, SUC_lexicon_tags_45 },
    { 0x14d1e41e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2591e420, SUC_lexicon_tags_154 },
    { 0x8751e421, SUC_lexicon_tags_29 },
    { 0x6ad1e421, SUC_lexicon_tags_14 },
    { 0xe251e421, SUC_lexicon_tags_11 },
    { 0x9731e421, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb1e429, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9611e42d, SUC_lexicon_tags_48 },
    { 0x48f1e42d, SUC_lexicon_tags_1 },
    { 0xd391e42d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd431e432, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c71e435, SUC_lexicon_tags_3 },
    { 0x66b1e435, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb671e43e, SUC_lexicon_tags_7 },
    { 0xb831e43f, SUC_lexicon_tags_84 },
    { 0x8b71e43f, SUC_lexicon_tags_11 },
    { 0x1e51e441, SUC_lexicon_tags_7 },
    { 0x691e442, SUC_lexicon_tags_47 },
    { 0xeb31e442, SUC_lexicon_tags_17 },
    { 0xa5d1e43f, SUC_lexicon_tags_63 },
    { 0x8531e441, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf31e44c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71e450, SUC_lexicon_tags_9 },
    { 0xf871e451, SUC_lexicon_tags_12 },
    { 0xc4f1e451, SUC_lexicon_tags_37 },
    { 0x9511e453, SUC_lexicon_tags_37 },
    { 0x4191e454, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1031e456, SUC_lexicon_tags_46 },
    { 0x6951e457, SUC_lexicon_tags_14 },
    { 0x1451e458, SUC_lexicon_tags_19 },
    { 0x9d71e457, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb71e45b, SUC_lexicon_tags_2 },
    { 0xf771e45c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d1e45f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd1e467, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83b1e469, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db1e46c, SUC_lexicon_tags_15 },
    { 0xfb91e46d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53f1e46f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea91e477, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b1e47c, SUC_lexicon_tags_42 },
    { 0x44b1e47d, SUC_lexicon_tags_12 },
    { 0x3cb1e47d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd1e481, SUC_lexicon_tags_11 },
    { 0x87f1e482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c71e484, SUC_lexicon_tags_8 },
    { 0xec31e484, SUC_lexicon_tags_17 },
    { 0x2891e486, SUC_lexicon_tags_2 },
    { 0x74d1e486, SUC_lexicon_tags_2 },
    { 0x75b1e488, SUC_lexicon_tags_47 },
    { 0x771e489, SUC_lexicon_tags_47 },
    { 0xb071e48a, SUC_lexicon_tags_2 },
    { 0x271e48a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5891e48e, SUC_lexicon_tags_25 },
    { 0xc351e48f, SUC_lexicon_tags_2 },
    { 0xce51e490, SUC_lexicon_tags_11 },
    { 0x4271e491, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b1e495, SUC_lexicon_tags_37 },
    { 0xd811e496, SUC_lexicon_tags_8 },
    { 0xf391e497, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd751e49e, SUC_lexicon_tags_25 },
    { 0xba11e49f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51e4a2, SUC_lexicon_tags_39 },
    { 0x4011e4a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa71e4a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b11e4aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2571e4ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b51e4b0, SUC_lexicon_tags_39 },
    { 0x1551e4b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511e4b5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4c31e4b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1e4ba, SUC_lexicon_tags_47 },
    { 0x8a91e4bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7751e4bd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x57f1e4bf, SUC_lexicon_tags_10 },
    { 0x5531e4bf, SUC_lexicon_tags_33 },
    { 0x5631e4c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fb1e4c3, SUC_lexicon_tags_38 },
    { 0x11f1e4c3, SUC_lexicon_tags_1 },
    { 0xc191e4c3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f1e4cf, SUC_lexicon_tags_43 },
    { 0xa5b1e4d0, SUC_lexicon_tags_14 },
    { 0x8731e4d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7331e4d5, SUC_lexicon_tags_11 },
    { 0x4d71e4d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb1e4d9, SUC_lexicon_tags_12 },
    { 0xa151e4d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d1e4e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d1e4e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2971e4ec, SUC_lexicon_tags_45 },
    { 0x391e4ed, SUC_lexicon_tags_25 },
    { 0x4271e4ee, SUC_lexicon_tags_33 },
    { 0x7991e4ef, SUC_lexicon_tags_60 },
    { 0x9631e4ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf31e4f2, SUC_lexicon_tags_44 },
    { 0xb2d1e4f3, SUC_lexicon_tags_7 },
    { 0x4771e4f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x60f1e4f6, SUC_lexicon_tags_8 },
    { 0xa551e4f7, SUC_lexicon_tags_37 },
    { 0x3791e4f7, SUC_lexicon_tags_105 },
    { 0xbbb1e4f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa091e504, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff1e509, SUC_lexicon_tags_19 },
    { 0xd191e509, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf1e50d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe311e511, SUC_lexicon_tags_25 },
    { 0x72f1e512, SUC_lexicon_tags_2 },
    { 0xd7d1e513, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb091e515, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b1e51b, SUC_lexicon_tags_47 },
    { 0x7dd1e51b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae51e51e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc71e522, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2691e524, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b1e52b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b1e52e, SUC_lexicon_tags_37 },
    { 0xaa31e52e, SUC_lexicon_tags_82 },
    { 0xe871e52f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc211e532, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d71e534, SUC_lexicon_tags_33 },
    { 0x1c91e535, SUC_lexicon_tags_45 },
    { 0x8e91e536, SUC_lexicon_tags_11 },
    { 0x5331e537, SUC_lexicon_tags_33 },
    { 0xd051e538, SUC_lexicon_tags_11 },
    { 0x8fb1e539, SUC_lexicon_tags_7 },
    { 0x37b1e53a, SUC_lexicon_tags_12 },
    { 0xffd1e53b, SUC_lexicon_tags_230 },
    { 0x2571e53b, SUC_lexicon_tags_45 },
    { 0xb111e53d, SUC_lexicon_tags_17 },
    { 0xff51e53c, SUC_lexicon_tags_47 },
    { 0x3ef1e53c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7d1e541, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb131e545, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6051e548, SUC_lexicon_tags_15 },
    { 0x89d1e549, SUC_lexicon_tags_37 },
    { 0xc471e54a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31e54e, SUC_lexicon_tags_795 },
    { 0x53d1e54f, SUC_lexicon_tags_0 },
    { 0x23d1e54f, SUC_lexicon_tags_2 },
    { 0xb11e550, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf91e555, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf1e55a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e51e55c, SUC_lexicon_tags_77 },
    { 0x2b91e55d, SUC_lexicon_tags_2 },
    { 0x3c51e55c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x251e560, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf371e566, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2831e56d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e31e572, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa071e575, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9751e577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d1e57c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee91e580, SUC_lexicon_tags_34 },
    { 0x71b1e581, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf591e583, SUC_lexicon_tags_14 },
    { 0x5331e584, SUC_lexicon_tags_60 },
    { 0xd7f1e584, SUC_lexicon_tags_15 },
    { 0x4e11e584, SUC_lexicon_tags_10 },
    { 0xfd31e587, SUC_lexicon_tags_19 },
    { 0x21d1e588, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9fd1e58a, SUC_lexicon_tags_11 },
    { 0x99d1e58b, SUC_lexicon_tags_17 },
    { 0x1971e58b, SUC_lexicon_tags_17 },
    { 0x4691e58b, SUC_lexicon_tags_5 },
    { 0x7291e58e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd1e591, SUC_lexicon_tags_12 },
    { 0x3791e592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee51e595, SUC_lexicon_tags_0 },
    { 0xbdd1e596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8971e59d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b11e5a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4711e5a4, SUC_lexicon_tags_25 },
    { 0xcbb1e5a5, SUC_lexicon_tags_32 },
    { 0xddb1e5a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b1e5ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71d1e5ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed1e5b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9331e5b2, SUC_lexicon_tags_43 },
    { 0xce11e5b2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4871e5ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae91e5bc, SUC_lexicon_tags_34 },
    { 0x8cf1e5bc, SUC_lexicon_tags_28 },
    { 0x9ad1e5be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7491e5c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x331e5c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5411e5c4, SUC_lexicon_tags_2 },
    { 0x2391e5c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e5cd, SUC_lexicon_tags_17 },
    { 0xbd31e5cd, SUC_lexicon_tags_25 },
    { 0xeb11e5cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d1e5d4, SUC_lexicon_tags_47 },
    { 0xd571e5d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7531e5d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f1e5dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3771e5e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c11e5e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d1e5ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3931e5f4, SUC_lexicon_tags_47 },
    { 0xe091e5f5, SUC_lexicon_tags_12 },
    { 0x8951e5f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16f1e5f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf971e5fb, SUC_lexicon_tags_23 },
    { 0x8d1e5fc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x58b1e5fe, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xacf1e600, SUC_lexicon_tags_7 },
    { 0xabd1e601, SUC_lexicon_tags_44 },
    { 0x4311e600, SUC_lexicon_tags_25 },
    { 0x6911e603, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b1e606, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391e60b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f1e610, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb291e614, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf71e618, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe731e61b, SUC_lexicon_tags_17 },
    { 0x1d1e61c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d1e622, SUC_lexicon_tags_1 },
    { 0xe751e622, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d1e627, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7891e629, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf571e636, SUC_lexicon_tags_12 },
    { 0xd011e637, SUC_lexicon_tags_56 },
    { 0xddb1e638, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3431e645, SUC_lexicon_tags_17 },
    { 0xedb1e646, SUC_lexicon_tags_38 },
    { 0xc1b1e647, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5051e649, SUC_lexicon_tags_11 },
    { 0x9f51e649, SUC_lexicon_tags_25 },
    { 0x9cd1e649, SUC_lexicon_tags_27 },
    { 0x5451e64c, SUC_lexicon_tags_28 },
    { 0xae91e64a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x191e650, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1e653, SUC_lexicon_tags_15 },
    { 0xa831e653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f1e657, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f1e65a, SUC_lexicon_tags_84 },
    { 0x8f11e65b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3371e664, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7651e666, SUC_lexicon_tags_14 },
    { 0x83f1e666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9951e66f, SUC_lexicon_tags_55 },
    { 0xe91e670, SUC_lexicon_tags_30 },
    { 0x77b1e66f, SUC_lexicon_tags_11 },
    { 0x2451e672, SUC_lexicon_tags_38 },
    { 0x1231e673, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e31e678, SUC_lexicon_tags_21 },
    { 0x9611e678, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b1e67d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2251e680, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f1e682, SUC_lexicon_tags_16 },
    { 0x5d11e683, SUC_lexicon_tags_46 },
    { 0x3411e684, SUC_lexicon_tags_25 },
    { 0x3831e684, SUC_lexicon_tags_2 },
    { 0xa511e686, SUC_lexicon_tags_37 },
    { 0x27f1e687, SUC_lexicon_tags_17 },
    { 0x9471e686, SUC_lexicon_tags_46 },
    { 0xb631e688, SUC_lexicon_tags_37 },
    { 0x2b91e68a, SUC_lexicon_tags_38 },
    { 0x4d71e68b, SUC_lexicon_tags_37 },
    { 0xf691e684, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2931e691, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4271e693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b31e695, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x7651e697, SUC_lexicon_tags_82 },
    { 0xfb31e698, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b1e69c, SUC_lexicon_tags_35 },
    { 0x4091e69c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0d1e69f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9371e6aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd171e6b2, SUC_lexicon_tags_45 },
    { 0x6131e6b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf1e6bb, SUC_lexicon_tags_1 },
    { 0x1d31e6bc, SUC_lexicon_tags_5 },
    { 0x5791e6bd, SUC_lexicon_tags_43 },
    { 0x3b11e6bc, SUC_lexicon_tags_38 },
    { 0xeb31e6be, SUC_lexicon_tags_47 },
    { 0x3d1e6bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf1e6c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x64f1e6c5, SUC_lexicon_tags_2 },
    { 0x6c31e6c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b1e6c9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd351e6ce, SUC_lexicon_tags_329 },
    { 0, NULL },
    { 0xe351e6d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1e6d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7411e6d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5791e6d8, SUC_lexicon_tags_1 },
    { 0x7fd1e6d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f11e6de, SUC_lexicon_tags_15 },
    { 0xc6f1e6df, SUC_lexicon_tags_16 },
    { 0x1211e6df, SUC_lexicon_tags_26 },
    { 0x791e6e1, SUC_lexicon_tags_19 },
    { 0x2cb1e6e2, SUC_lexicon_tags_25 },
    { 0x4171e6e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef91e6e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe111e6ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7611e6f0, SUC_lexicon_tags_11 },
    { 0x5171e6f1, SUC_lexicon_tags_11 },
    { 0x8c91e6f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb011e6f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c91e6f6, SUC_lexicon_tags_11 },
    { 0x17f1e6f7, SUC_lexicon_tags_28 },
    { 0x4d71e6f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf1e700, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb631e702, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a71e705, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf331e707, SUC_lexicon_tags_1257 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb1e70e, SUC_lexicon_tags_11 },
    { 0xb9f1e70f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd8d1e711, SUC_lexicon_tags_12 },
    { 0xcf51e712, SUC_lexicon_tags_47 },
    { 0x9eb1e713, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe631e718, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b1e71b, SUC_lexicon_tags_1 },
    { 0x25f1e71c, SUC_lexicon_tags_11 },
    { 0x2971e71b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d1e720, SUC_lexicon_tags_17 },
    { 0x39f1e721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211e725, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd071e729, SUC_lexicon_tags_15 },
    { 0x8c11e72a, SUC_lexicon_tags_23 },
    { 0xa0b1e72b, SUC_lexicon_tags_37 },
    { 0xf2f1e72c, SUC_lexicon_tags_23 },
    { 0x11e72d, SUC_lexicon_tags_37 },
    { 0xe211e72e, SUC_lexicon_tags_16 },
    { 0x91b1e72d, SUC_lexicon_tags_45 },
    { 0xbd11e730, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d11e735, SUC_lexicon_tags_30 },
    { 0x5131e736, SUC_lexicon_tags_32 },
    { 0x58f1e737, SUC_lexicon_tags_37 },
    { 0xf031e738, SUC_lexicon_tags_37 },
    { 0x20f1e735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d91e73e, SUC_lexicon_tags_68 },
    { 0x9ab1e73e, SUC_lexicon_tags_39 },
    { 0x5db1e73e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851e744, SUC_lexicon_tags_1204 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b1e747, SUC_lexicon_tags_29 },
    { 0x2031e747, SUC_lexicon_tags_2 },
    { 0xc771e749, SUC_lexicon_tags_37 },
    { 0x9d91e748, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7351e74f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5df1e751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f1e754, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3831e759, SUC_lexicon_tags_25 },
    { 0x6051e75a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9af1e75c, SUC_lexicon_tags_17 },
    { 0x2b1e75d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f1e764, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb8b1e766, SUC_lexicon_tags_25 },
    { 0xb371e767, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79b1e769, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b51e76d, SUC_lexicon_tags_7 },
    { 0xeb71e76d, SUC_lexicon_tags_7 },
    { 0x3211e76f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa71e774, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251e778, SUC_lexicon_tags_37 },
    { 0x9f51e779, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafd1e77b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x971e77d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f1e782, SUC_lexicon_tags_17 },
    { 0x34d1e783, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd51e785, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe931e788, SUC_lexicon_tags_1 },
    { 0x93f1e789, SUC_lexicon_tags_12 },
    { 0xae31e78a, SUC_lexicon_tags_0 },
    { 0xc271e78a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd1e78e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d1e791, SUC_lexicon_tags_12 },
    { 0xc691e792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb1e796, SUC_lexicon_tags_37 },
    { 0x471e797, SUC_lexicon_tags_677 },
    { 0x271e796, SUC_lexicon_tags_2 },
    { 0xc351e799, SUC_lexicon_tags_52 },
    { 0x3771e79a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab91e7a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d1e7a4, SUC_lexicon_tags_37 },
    { 0x3991e7a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75b1e7a7, SUC_lexicon_tags_43 },
    { 0x9551e7a8, SUC_lexicon_tags_28 },
    { 0xc8f1e7a9, SUC_lexicon_tags_68 },
    { 0x5611e7aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf71e7ac, SUC_lexicon_tags_31 },
    { 0x6811e7ad, SUC_lexicon_tags_25 },
    { 0x9071e7ae, SUC_lexicon_tags_57 },
    { 0xa411e7af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3591e7b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c91e7b5, SUC_lexicon_tags_14 },
    { 0x1ad1e7b5, SUC_lexicon_tags_7 },
    { 0x4ff1e7b7, SUC_lexicon_tags_47 },
    { 0xf6b1e7b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1271e7be, SUC_lexicon_tags_11 },
    { 0x7b1e7bf, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x8371e7c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f1e7c6, SUC_lexicon_tags_38 },
    { 0xbd51e7c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f51e7ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30b1e7cc, SUC_lexicon_tags_33 },
    { 0x2351e7cd, SUC_lexicon_tags_2 },
    { 0x1131e7cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf71e7dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf791e7e0, SUC_lexicon_tags_5 },
    { 0xca31e7e1, SUC_lexicon_tags_14 },
    { 0xca11e7e2, SUC_lexicon_tags_48 },
    { 0x3391e7e3, SUC_lexicon_tags_19 },
    { 0xbb31e7e4, SUC_lexicon_tags_33 },
    { 0x60f1e7e3, SUC_lexicon_tags_32 },
    { 0xce71e7e1, SUC_lexicon_tags_6 },
    { 0x7411e7e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd1e7ea, SUC_lexicon_tags_0 },
    { 0x4111e7ea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf1e7ef, SUC_lexicon_tags_45 },
    { 0xcc51e7f0, SUC_lexicon_tags_46 },
    { 0x6db1e7f0, SUC_lexicon_tags_27 },
    { 0x8e11e7f0, SUC_lexicon_tags_1 },
    { 0x2cb1e7f0, SUC_lexicon_tags_46 },
    { 0xa411e7f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5811e7f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa551e7fb, SUC_lexicon_tags_17 },
    { 0xb2f1e7fc, SUC_lexicon_tags_7 },
    { 0x9391e7fc, SUC_lexicon_tags_43 },
    { 0x5451e7fe, SUC_lexicon_tags_21 },
    { 0xac11e7fc, SUC_lexicon_tags_63 },
    { 0x7071e800, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d1e803, SUC_lexicon_tags_91 },
    { 0x7451e804, SUC_lexicon_tags_10 },
    { 0xe351e805, SUC_lexicon_tags_15 },
    { 0x1bd1e805, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x78f1e808, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb1e80b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf1e80e, SUC_lexicon_tags_17 },
    { 0x90f1e80e, SUC_lexicon_tags_37 },
    { 0x8351e80f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d1e818, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32d1e81a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad1e81d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d51e820, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d1e824, SUC_lexicon_tags_15 },
    { 0xa2d1e824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1e829, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dd1e82b, SUC_lexicon_tags_26 },
    { 0xf911e82c, SUC_lexicon_tags_38 },
    { 0xbc91e82b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf31e82f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd471e833, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b1e836, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff1e83e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f1e841, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7471e843, SUC_lexicon_tags_1 },
    { 0xe311e843, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe731e84a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451e850, SUC_lexicon_tags_6 },
    { 0xd71e851, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b1e855, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e51e858, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb91e85e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x7b51e860, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b1e865, SUC_lexicon_tags_28 },
    { 0xe5b1e865, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d1e86d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cf1e86f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5dd1e871, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3011e873, SUC_lexicon_tags_10 },
    { 0xfaf1e874, SUC_lexicon_tags_13 },
    { 0x8bb1e874, SUC_lexicon_tags_11 },
    { 0x3651e876, SUC_lexicon_tags_37 },
    { 0x3eb1e877, SUC_lexicon_tags_7 },
    { 0xefd1e877, SUC_lexicon_tags_60 },
    { 0xf7b1e873, SUC_lexicon_tags_0 },
    { 0x5b71e87a, SUC_lexicon_tags_46 },
    { 0xd5d1e878, SUC_lexicon_tags_15 },
    { 0xd0d1e87b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f11e87e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf2b1e880, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x4c91e882, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x371e884, SUC_lexicon_tags_25 },
    { 0x3df1e885, SUC_lexicon_tags_0 },
    { 0xf811e886, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x71e888, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8811e88a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5751e88c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c71e894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d11e897, SUC_lexicon_tags_33 },
    { 0xacd1e898, SUC_lexicon_tags_57 },
    { 0xf151e899, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1311e89d, SUC_lexicon_tags_25 },
    { 0xb6f1e89e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5031e8a7, SUC_lexicon_tags_8 },
    { 0x4031e8a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb691e8ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x391e8ae, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc7f1e8b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x131e8b2, SUC_lexicon_tags_33 },
    { 0x36f1e8b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b11e8ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7271e8bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda91e8c0, SUC_lexicon_tags_9 },
    { 0x7c11e8c0, SUC_lexicon_tags_37 },
    { 0x3451e8c2, SUC_lexicon_tags_25 },
    { 0x9c11e8c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1671e8c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcc71e8c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94f1e8ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f1e8cd, SUC_lexicon_tags_11 },
    { 0x5111e8cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9b1e8d0, SUC_lexicon_tags_0 },
    { 0xb731e8d1, SUC_lexicon_tags_37 },
    { 0x2cb1e8d2, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db1e8d8, SUC_lexicon_tags_60 },
    { 0x6791e8d9, SUC_lexicon_tags_79 },
    { 0x8f91e8d8, SUC_lexicon_tags_63 },
    { 0x4551e8db, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5231e8dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7431e8df, SUC_lexicon_tags_28 },
    { 0x6a91e8e0, SUC_lexicon_tags_37 },
    { 0x7f31e8e1, SUC_lexicon_tags_15 },
    { 0xddd1e8e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90f1e8e4, SUC_lexicon_tags_7 },
    { 0xa751e8e5, SUC_lexicon_tags_25 },
    { 0xe851e8e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9111e8ea, SUC_lexicon_tags_25 },
    { 0x5c31e8eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f1e8ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1351e8f2, SUC_lexicon_tags_16 },
    { 0xb1e8f2, SUC_lexicon_tags_15 },
    { 0xf8d1e8f4, SUC_lexicon_tags_6 },
    { 0xc5b1e8f5, SUC_lexicon_tags_17 },
    { 0x57f1e8f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5331e8f9, SUC_lexicon_tags_7 },
    { 0xbeb1e8fa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2891e8fd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d91e901, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5791e903, SUC_lexicon_tags_55 },
    { 0x6591e904, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7671e90a, SUC_lexicon_tags_38 },
    { 0x9f1e90b, SUC_lexicon_tags_18 },
    { 0x351e90c, SUC_lexicon_tags_1 },
    { 0xa071e90c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8311e90f, SUC_lexicon_tags_68 },
    { 0x811e90f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5ef1e912, SUC_lexicon_tags_37 },
    { 0x7d51e912, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x21f1e915, SUC_lexicon_tags_37 },
    { 0x2f31e916, SUC_lexicon_tags_34 },
    { 0xacb1e915, SUC_lexicon_tags_39 },
    { 0xda71e918, SUC_lexicon_tags_9 },
    { 0xa331e919, SUC_lexicon_tags_37 },
    { 0x54f1e91a, SUC_lexicon_tags_15 },
    { 0x3951e915, SUC_lexicon_tags_40 },
    { 0x69f1e919, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4d1e91e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd51e920, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7551e922, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f1e926, SUC_lexicon_tags_1 },
    { 0xbcd1e927, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9011e929, SUC_lexicon_tags_12 },
    { 0x1ab1e92a, SUC_lexicon_tags_1 },
    { 0x1a91e92b, SUC_lexicon_tags_1 },
    { 0xcd71e92c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe231e931, SUC_lexicon_tags_37 },
    { 0x4b11e932, SUC_lexicon_tags_38 },
    { 0x16d1e933, SUC_lexicon_tags_7 },
    { 0xe271e933, SUC_lexicon_tags_37 },
    { 0xc91e935, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9271e937, SUC_lexicon_tags_7 },
    { 0x5ad1e938, SUC_lexicon_tags_25 },
    { 0xd1b1e939, SUC_lexicon_tags_12 },
    { 0xe351e939, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2631e93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe171e93f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4471e946, SUC_lexicon_tags_4 },
    { 0xa431e947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f1e94a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa31e94d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e11e954, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x671e957, SUC_lexicon_tags_20 },
    { 0xb911e958, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f1e95f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f1e964, SUC_lexicon_tags_14 },
    { 0x57b1e965, SUC_lexicon_tags_298 },
    { 0xaf51e965, SUC_lexicon_tags_38 },
    { 0x91d1e967, SUC_lexicon_tags_11 },
    { 0xe951e965, SUC_lexicon_tags_11 },
    { 0x7e71e969, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb4f1e96b, SUC_lexicon_tags_11 },
    { 0xb831e96c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc11e96e, SUC_lexicon_tags_33 },
    { 0xeb11e96f, SUC_lexicon_tags_11 },
    { 0x39d1e970, SUC_lexicon_tags_37 },
    { 0x6b31e971, SUC_lexicon_tags_47 },
    { 0xe2d1e970, SUC_lexicon_tags_30 },
    { 0x7eb1e973, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c71e976, SUC_lexicon_tags_80 },
    { 0x56d1e977, SUC_lexicon_tags_11 },
    { 0xe451e978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c91e97a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef1e97d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3171e97f, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e51e982, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa631e987, SUC_lexicon_tags_45 },
    { 0xef91e988, SUC_lexicon_tags_295 },
    { 0xdb91e989, SUC_lexicon_tags_11 },
    { 0xafb1e989, SUC_lexicon_tags_70 },
    { 0x1111e987, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1c71e98d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf1e990, SUC_lexicon_tags_47 },
    { 0x9671e991, SUC_lexicon_tags_38 },
    { 0x5b51e992, SUC_lexicon_tags_16 },
    { 0x5591e993, SUC_lexicon_tags_11 },
    { 0x3551e992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5491e997, SUC_lexicon_tags_9 },
    { 0x771e998, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc551e99c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9851e99e, SUC_lexicon_tags_2 },
    { 0x891e99e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc051e9a2, SUC_lexicon_tags_17 },
    { 0x6d91e9a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6811e9a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd1e9ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f1e9b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7871e9b2, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4051e9b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b1e9bb, SUC_lexicon_tags_17 },
    { 0xe971e9bc, SUC_lexicon_tags_43 },
    { 0x9111e9bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d91e9c1, SUC_lexicon_tags_11 },
    { 0x2171e9c2, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1231e9c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe891e9cb, SUC_lexicon_tags_17 },
    { 0x9db1e9cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc351e9d1, SUC_lexicon_tags_17 },
    { 0x3151e9d1, SUC_lexicon_tags_25 },
    { 0xf051e9d2, SUC_lexicon_tags_7 },
    { 0xf491e9d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1551e9d7, SUC_lexicon_tags_11 },
    { 0x7b31e9d7, SUC_lexicon_tags_27 },
    { 0x9cf1e9d9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe1d1e9db, SUC_lexicon_tags_15 },
    { 0x9911e9db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6f1e9de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6671e9e1, SUC_lexicon_tags_17 },
    { 0x32b1e9e2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd511e9e6, SUC_lexicon_tags_27 },
    { 0xb151e9e7, SUC_lexicon_tags_29 },
    { 0x75d1e9e7, SUC_lexicon_tags_8 },
    { 0x3071e9e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf271e9eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef11e9ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f1e9ef, SUC_lexicon_tags_11 },
    { 0x8cf1e9f0, SUC_lexicon_tags_66 },
    { 0xa491e9f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa671e9f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11e9fb, SUC_lexicon_tags_2 },
    { 0x61f1e9fc, SUC_lexicon_tags_25 },
    { 0x9b31e9fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5551ea02, SUC_lexicon_tags_8 },
    { 0xb811ea03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf331ea0b, SUC_lexicon_tags_30 },
    { 0xfd91ea0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d1ea10, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xde11ea12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5851ea14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56d1ea16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd71ea18, SUC_lexicon_tags_37 },
    { 0x6e91ea18, SUC_lexicon_tags_7 },
    { 0x5f91ea1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad11ea20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b1ea23, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b1ea2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c11ea2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf671ea31, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b1ea34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5971ea37, SUC_lexicon_tags_33 },
    { 0x2f11ea37, SUC_lexicon_tags_37 },
    { 0x26b1ea39, SUC_lexicon_tags_29 },
    { 0x43d1ea39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2b1ea3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51ea41, SUC_lexicon_tags_310 },
    { 0xc711ea42, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7871ea46, SUC_lexicon_tags_11 },
    { 0x6ed1ea47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b1ea4a, SUC_lexicon_tags_15 },
    { 0x94b1ea4b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7251ea4e, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd1ea52, SUC_lexicon_tags_35 },
    { 0xcef1ea52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3351ea55, SUC_lexicon_tags_9 },
    { 0xaa91ea56, SUC_lexicon_tags_23 },
    { 0x4fd1ea56, SUC_lexicon_tags_11 },
    { 0xd1f1ea57, SUC_lexicon_tags_1 },
    { 0xf8d1ea59, SUC_lexicon_tags_63 },
    { 0xfc71ea5a, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0x6b51ea5c, SUC_lexicon_tags_85 },
    { 0x88b1ea5d, SUC_lexicon_tags_27 },
    { 0xccd1ea5e, SUC_lexicon_tags_84 },
    { 0x4df1ea5f, SUC_lexicon_tags_2 },
    { 0x9e31ea60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x271ea62, SUC_lexicon_tags_17 },
    { 0x7bf1ea63, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b1ea68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f1ea6b, SUC_lexicon_tags_19 },
    { 0x6f51ea6c, SUC_lexicon_tags_61 },
    { 0x5731ea6b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6071ea6f, SUC_lexicon_tags_14 },
    { 0x5371ea70, SUC_lexicon_tags_17 },
    { 0x56b1ea71, SUC_lexicon_tags_0 },
    { 0x70d1ea71, SUC_lexicon_tags_23 },
    { 0xeaf1ea71, SUC_lexicon_tags_38 },
    { 0xafb1ea71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d1ea78, SUC_lexicon_tags_2 },
    { 0x2871ea78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8791ea7c, SUC_lexicon_tags_65 },
    { 0xb5f1ea7d, SUC_lexicon_tags_23 },
    { 0xfcf1ea7e, SUC_lexicon_tags_33 },
    { 0x8111ea7c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ab1ea81, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd531ea84, SUC_lexicon_tags_17 },
    { 0x751ea85, SUC_lexicon_tags_49 },
    { 0x8591ea86, SUC_lexicon_tags_87 },
    { 0xe3f1ea87, SUC_lexicon_tags_10 },
    { 0x1c51ea86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4951ea8b, SUC_lexicon_tags_17 },
    { 0x8cb1ea8c, SUC_lexicon_tags_47 },
    { 0xfc51ea8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e31ea8f, SUC_lexicon_tags_31 },
    { 0xe131ea8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391ea94, SUC_lexicon_tags_11 },
    { 0x5f31ea94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb1ea98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d91ea9e, SUC_lexicon_tags_37 },
    { 0xbfb1ea9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb1eaa3, SUC_lexicon_tags_23 },
    { 0x4c31eaa4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c11eaad, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2871eab0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb111eab3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d51eab5, SUC_lexicon_tags_12 },
    { 0x7791eab6, SUC_lexicon_tags_1 },
    { 0x9d91eab6, SUC_lexicon_tags_37 },
    { 0xc1b1eab7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5391eabe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf1eac2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x89b1eac4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8fb1eac6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xa331eac8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6631eace, SUC_lexicon_tags_98 },
    { 0x4951eacf, SUC_lexicon_tags_7 },
    { 0x6911eacf, SUC_lexicon_tags_14 },
    { 0x8b31ead1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db1ead8, SUC_lexicon_tags_87 },
    { 0x1731ead8, SUC_lexicon_tags_49 },
    { 0x8f51eada, SUC_lexicon_tags_15 },
    { 0x11f1eadb, SUC_lexicon_tags_77 },
    { 0x9cf1ead9, SUC_lexicon_tags_25 },
    { 0x8991eadd, SUC_lexicon_tags_11 },
    { 0xe0b1eadd, SUC_lexicon_tags_15 },
    { 0x4d51eadf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95f1eae1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb1eae4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaad1eae6, SUC_lexicon_tags_2 },
    { 0x9ad1eae7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f1eaef, SUC_lexicon_tags_37 },
    { 0xb71eaf0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff1eaf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f1eafa, SUC_lexicon_tags_11 },
    { 0x4351eafa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd451eafd, SUC_lexicon_tags_25 },
    { 0xb51eafd, SUC_lexicon_tags_1 },
    { 0x6bd1eafe, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x61f1eb01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f1eb08, SUC_lexicon_tags_60 },
    { 0xc6d1eb09, SUC_lexicon_tags_35 },
    { 0x6dd1eb08, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf291eb0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1f1eb0e, SUC_lexicon_tags_37 },
    { 0x63b1eb0e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x52b1eb11, SUC_lexicon_tags_2 },
    { 0xef1eb12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4ab1eb14, SUC_lexicon_tags_11 },
    { 0x5391eb15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4b1eb17, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4771eb1a, SUC_lexicon_tags_46 },
    { 0x3291eb1a, SUC_lexicon_tags_11 },
    { 0x91f1eb1c, SUC_lexicon_tags_39 },
    { 0xec71eb1d, SUC_lexicon_tags_34 },
    { 0x2a11eb1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7311eb23, SUC_lexicon_tags_43 },
    { 0x4a51eb24, SUC_lexicon_tags_17 },
    { 0x8831eb23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe551eb29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf451eb2b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6f1eb2d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xf471eb30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd411eb32, SUC_lexicon_tags_17 },
    { 0xccf1eb32, SUC_lexicon_tags_23 },
    { 0x2251eb34, SUC_lexicon_tags_7 },
    { 0x33b1eb33, SUC_lexicon_tags_47 },
    { 0x70f1eb36, SUC_lexicon_tags_0 },
    { 0x8291eb37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd1eb3b, SUC_lexicon_tags_11 },
    { 0xbd91eb3c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaa31eb3e, SUC_lexicon_tags_2 },
    { 0xdf71eb3e, SUC_lexicon_tags_30 },
    { 0x46d1eb40, SUC_lexicon_tags_28 },
    { 0xf511eb3e, SUC_lexicon_tags_25 },
    { 0x6d91eb40, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d1eb46, SUC_lexicon_tags_11 },
    { 0xd1d1eb47, SUC_lexicon_tags_7 },
    { 0xafd1eb48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2011eb4c, SUC_lexicon_tags_14 },
    { 0x9f11eb4c, SUC_lexicon_tags_2 },
    { 0xf851eb4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf1eb54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3471eb58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b31eb5b, SUC_lexicon_tags_37 },
    { 0xd811eb5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4631eb62, SUC_lexicon_tags_47 },
    { 0x73f1eb63, SUC_lexicon_tags_87 },
    { 0x27f1eb64, SUC_lexicon_tags_856 },
    { 0x3091eb62, SUC_lexicon_tags_57 },
    { 0x6891eb66, SUC_lexicon_tags_0 },
    { 0x7d31eb67, SUC_lexicon_tags_17 },
    { 0x1391eb67, SUC_lexicon_tags_17 },
    { 0x8531eb69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3371eb6b, SUC_lexicon_tags_12 },
    { 0x4b51eb6b, SUC_lexicon_tags_9 },
    { 0x8411eb6c, SUC_lexicon_tags_37 },
    { 0x79d1eb6b, SUC_lexicon_tags_2 },
    { 0x8f71eb6c, SUC_lexicon_tags_12 },
    { 0x2911eb70, SUC_lexicon_tags_2 },
    { 0x84b1eb6b, SUC_lexicon_tags_84 },
    { 0x2b51eb72, SUC_lexicon_tags_85 },
    { 0x23d1eb6b, SUC_lexicon_tags_63 },
    { 0x55b1eb74, SUC_lexicon_tags_13 },
    { 0xc711eb6b, SUC_lexicon_tags_0 },
    { 0x33f1eb76, SUC_lexicon_tags_37 },
    { 0x7711eb77, SUC_lexicon_tags_15 },
    { 0x1cf1eb77, SUC_lexicon_tags_17 },
    { 0xa7b1eb79, SUC_lexicon_tags_25 },
    { 0x3211eb72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7031eb7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5131eb82, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc51eb86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d1eb89, SUC_lexicon_tags_11 },
    { 0xf791eb8a, SUC_lexicon_tags_0 },
    { 0x6551eb8b, SUC_lexicon_tags_19 },
    { 0x8c91eb8c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d71eb8f, SUC_lexicon_tags_46 },
    { 0xf871eb90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f1eb94, SUC_lexicon_tags_37 },
    { 0x5bb1eb95, SUC_lexicon_tags_27 },
    { 0x4ff1eb95, SUC_lexicon_tags_14 },
    { 0x5d1eb95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7391eb99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdbb1eb9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f1eb9e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71eba3, SUC_lexicon_tags_11 },
    { 0xd2b1eba4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb831eba8, SUC_lexicon_tags_12 },
    { 0x1511eba8, SUC_lexicon_tags_26 },
    { 0x7ef1ebaa, SUC_lexicon_tags_10 },
    { 0x3b91ebaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df1ebaf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x13d1ebb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc671ebb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf851ebb7, SUC_lexicon_tags_11 },
    { 0x1251ebb7, SUC_lexicon_tags_37 },
    { 0x9791ebb9, SUC_lexicon_tags_63 },
    { 0x3711ebba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8771ebbc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd831ebc2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9551ebc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab1ebcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9431ebcf, SUC_lexicon_tags_11 },
    { 0x1751ebcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa771ebd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a71ebd6, SUC_lexicon_tags_12 },
    { 0x6f11ebd6, SUC_lexicon_tags_29 },
    { 0x631ebd7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf71ebdb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2131ebdd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4191ebe1, SUC_lexicon_tags_12 },
    { 0x7ad1ebe2, SUC_lexicon_tags_2 },
    { 0xfc91ebe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef11ebe9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2871ebeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f51ebee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e31ebf5, SUC_lexicon_tags_4 },
    { 0x3791ebf5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd071ebf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde51ebfc, SUC_lexicon_tags_42 },
    { 0x9d31ebfd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd271ec08, SUC_lexicon_tags_9 },
    { 0xe571ec08, SUC_lexicon_tags_11 },
    { 0x93d1ec09, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab1ec10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b1ec13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c11ec16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5071ec1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d51ec1d, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e91ec22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed1ec24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe611ec2c, SUC_lexicon_tags_204 },
    { 0x8d91ec2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d31ec31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6391ec37, SUC_lexicon_tags_30 },
    { 0xa111ec38, SUC_lexicon_tags_1 },
    { 0xfeb1ec39, SUC_lexicon_tags_118 },
    { 0xde91ec3a, SUC_lexicon_tags_0 },
    { 0x72f1ec3b, SUC_lexicon_tags_0 },
    { 0x3451ec38, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31ec44, SUC_lexicon_tags_15 },
    { 0x7b11ec44, SUC_lexicon_tags_19 },
    { 0x551ec46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9671ec4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab1ec54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df1ec57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc651ec5b, SUC_lexicon_tags_10 },
    { 0x2051ec5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xef51ec5f, SUC_lexicon_tags_25 },
    { 0x48b1ec60, SUC_lexicon_tags_9 },
    { 0x5511ec5f, SUC_lexicon_tags_47 },
    { 0x56b1ec62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d1ec67, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x8411ec6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6851ec6c, SUC_lexicon_tags_17 },
    { 0xe1f1ec6c, SUC_lexicon_tags_45 },
    { 0x1b91ec6e, SUC_lexicon_tags_9 },
    { 0xd311ec6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8231ec71, SUC_lexicon_tags_11 },
    { 0x6ef1ec72, SUC_lexicon_tags_17 },
    { 0x2591ec73, SUC_lexicon_tags_33 },
    { 0xa811ec73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f1ec7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8471ec7c, SUC_lexicon_tags_27 },
    { 0xcc31ec7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b11ec82, SUC_lexicon_tags_61 },
    { 0xdbd1ec83, SUC_lexicon_tags_26 },
    { 0x8431ec84, SUC_lexicon_tags_37 },
    { 0x4131ec83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf191ec87, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f1ec8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba91ec8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf1ec8f, SUC_lexicon_tags_2 },
    { 0x1bb1ec90, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5d11ec92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde71ec95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f1ec99, SUC_lexicon_tags_12 },
    { 0xebf1ec9a, SUC_lexicon_tags_43 },
    { 0x91f1ec9a, SUC_lexicon_tags_17 },
    { 0xb0b1ec9c, SUC_lexicon_tags_7 },
    { 0xe511ec9b, SUC_lexicon_tags_38 },
    { 0xc251ec9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa011eca0, SUC_lexicon_tags_5 },
    { 0xf791eca1, SUC_lexicon_tags_12 },
    { 0xe991eca2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f1eca9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3051ecad, SUC_lexicon_tags_13 },
    { 0x9d91ecae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3931ecb3, SUC_lexicon_tags_15 },
    { 0xff91ecb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4611ecb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31ecbc, SUC_lexicon_tags_2 },
    { 0xf6d1ecbd, SUC_lexicon_tags_17 },
    { 0xf5d1ecbe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e71ecc3, SUC_lexicon_tags_39 },
    { 0xb391ecc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd71ecc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedd1ecc8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f1eccc, SUC_lexicon_tags_32 },
    { 0xc871eccc, SUC_lexicon_tags_2 },
    { 0xf971eccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8511ecd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d51ecd4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b71ecdb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf591ece1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f71ece5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8291ece7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1ecea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd1ecee, SUC_lexicon_tags_38 },
    { 0x7751ecef, SUC_lexicon_tags_15 },
    { 0x84f1ecef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb1ecf8, SUC_lexicon_tags_49 },
    { 0xedf1ecf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af1ecfc, SUC_lexicon_tags_33 },
    { 0x3bb1ecfc, SUC_lexicon_tags_37 },
    { 0x24b1ecfd, SUC_lexicon_tags_1 },
    { 0xeab1ecff, SUC_lexicon_tags_12 },
    { 0x84f1ecfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd1ed0a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xca11ed0c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4cd1ed0e, SUC_lexicon_tags_1 },
    { 0xa591ed0f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe5f1ed11, SUC_lexicon_tags_7 },
    { 0x2d11ed11, SUC_lexicon_tags_1061 },
    { 0x17d1ed13, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x3391ed16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4291ed18, SUC_lexicon_tags_25 },
    { 0xec31ed19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6471ed23, SUC_lexicon_tags_7 },
    { 0xd3f1ed24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe971ed28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d71ed2a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa271ed2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2191ed2e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc951ed35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8751ed3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f1ed3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9791ed42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ed45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5391ed48, SUC_lexicon_tags_33 },
    { 0x71d1ed49, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x31d1ed4b, SUC_lexicon_tags_7 },
    { 0xaeb1ed4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36d1ed4e, SUC_lexicon_tags_17 },
    { 0x8cf1ed4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bd1ed51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb691ed5a, SUC_lexicon_tags_27 },
    { 0x8811ed5a, SUC_lexicon_tags_47 },
    { 0x25b1ed5b, SUC_lexicon_tags_39 },
    { 0x5c51ed5d, SUC_lexicon_tags_81 },
    { 0x3871ed5d, SUC_lexicon_tags_37 },
    { 0x15b1ed5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe171ed61, SUC_lexicon_tags_37 },
    { 0x5a91ed62, SUC_lexicon_tags_17 },
    { 0xd71ed63, SUC_lexicon_tags_25 },
    { 0x3d71ed63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb271ed68, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf1ed6c, SUC_lexicon_tags_174 },
    { 0x3c91ed6d, SUC_lexicon_tags_15 },
    { 0x5f51ed6d, SUC_lexicon_tags_38 },
    { 0x4191ed6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af1ed81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b91ed83, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b31ed88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8771ed8b, SUC_lexicon_tags_17 },
    { 0x3951ed8c, SUC_lexicon_tags_7 },
    { 0x7211ed8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411ed92, SUC_lexicon_tags_48 },
    { 0xcb71ed92, SUC_lexicon_tags_45 },
    { 0xa11ed94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b1ed97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391ed9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b1eda0, SUC_lexicon_tags_1 },
    { 0x3371eda1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfb71eda3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2731eda5, SUC_lexicon_tags_17 },
    { 0x3591eda6, SUC_lexicon_tags_6 },
    { 0x4611eda6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b1edaa, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0x7751edad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4191edb2, SUC_lexicon_tags_70 },
    { 0x4471edb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3631edb5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6b31edb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb91edbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85d1edbe, SUC_lexicon_tags_2 },
    { 0x6171edbf, SUC_lexicon_tags_12 },
    { 0x5991edc0, SUC_lexicon_tags_25 },
    { 0x7b91edc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff1edc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1edcd, SUC_lexicon_tags_23 },
    { 0x3f51edce, SUC_lexicon_tags_15 },
    { 0xabf1edce, SUC_lexicon_tags_22 },
    { 0x61b1edd0, SUC_lexicon_tags_11 },
    { 0x1471edd1, SUC_lexicon_tags_1 },
    { 0x9eb1edd2, SUC_lexicon_tags_12 },
    { 0x76d1edd3, SUC_lexicon_tags_31 },
    { 0xac31edd1, SUC_lexicon_tags_30 },
    { 0xa5b1edce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3551edd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7551edde, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb991ede3, SUC_lexicon_tags_25 },
    { 0x6651ede4, SUC_lexicon_tags_25 },
    { 0x3c71ede5, SUC_lexicon_tags_12 },
    { 0x211ede5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8f1ede8, SUC_lexicon_tags_39 },
    { 0xb631ede9, SUC_lexicon_tags_19 },
    { 0xea31edea, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d1edee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d1edf4, SUC_lexicon_tags_7 },
    { 0x42f1edf5, SUC_lexicon_tags_2 },
    { 0x2e11edf5, SUC_lexicon_tags_63 },
    { 0xc8b1edf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5b1edf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91d1edfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f1ee00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee11ee04, SUC_lexicon_tags_37 },
    { 0x8851ee05, SUC_lexicon_tags_65 },
    { 0x13f1ee05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1db1ee08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7371ee0b, SUC_lexicon_tags_31 },
    { 0x9591ee0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e31ee0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f1ee16, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11ee1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1ee1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb91ee21, SUC_lexicon_tags_19 },
    { 0x1d11ee22, SUC_lexicon_tags_28 },
    { 0x3791ee23, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4fb1ee25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc531ee2a, SUC_lexicon_tags_27 },
    { 0xdb91ee2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b71ee2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d11ee36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc451ee38, SUC_lexicon_tags_7 },
    { 0x57d1ee39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7271ee3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b1ee42, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f1ee46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfcd1ee48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8471ee4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8191ee4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd571ee52, SUC_lexicon_tags_2 },
    { 0xad1ee53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5211ee56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1f1ee58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d1ee5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6751ee5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd31ee65, SUC_lexicon_tags_661 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc71ee68, SUC_lexicon_tags_17 },
    { 0xab31ee69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3ff1ee6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fd1ee6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x271ee70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe151ee75, SUC_lexicon_tags_6 },
    { 0x3711ee76, SUC_lexicon_tags_2 },
    { 0xb811ee77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffd1ee79, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7171ee7d, SUC_lexicon_tags_15 },
    { 0xf5b1ee7e, SUC_lexicon_tags_37 },
    { 0xa8d1ee7f, SUC_lexicon_tags_14 },
    { 0x1531ee80, SUC_lexicon_tags_8 },
    { 0x4131ee7e, SUC_lexicon_tags_38 },
    { 0x9211ee82, SUC_lexicon_tags_17 },
    { 0x1cb1ee83, SUC_lexicon_tags_39 },
    { 0xe851ee84, SUC_lexicon_tags_2 },
    { 0xfa71ee83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1151ee87, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5571ee8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f1ee8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1ee93, SUC_lexicon_tags_25 },
    { 0xf651ee94, SUC_lexicon_tags_7 },
    { 0x31f1ee95, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xde31ee97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1791ee99, SUC_lexicon_tags_30 },
    { 0xf891ee99, SUC_lexicon_tags_37 },
    { 0x2f91ee9b, SUC_lexicon_tags_12 },
    { 0xd9b1ee9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x67f1ee9e, SUC_lexicon_tags_64 },
    { 0x2a91ee9f, SUC_lexicon_tags_15 },
    { 0x9d91eea0, SUC_lexicon_tags_2 },
    { 0x311ee9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3431eea3, SUC_lexicon_tags_1 },
    { 0x4e51eea4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25b1eea6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7711eeac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d1eeb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6411eeb7, SUC_lexicon_tags_17 },
    { 0x20f1eeb8, SUC_lexicon_tags_19 },
    { 0x6911eeb9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b1eebd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3531eec2, SUC_lexicon_tags_17 },
    { 0xde11eec2, SUC_lexicon_tags_6 },
    { 0xcf1eec2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b1eec7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf751eec9, SUC_lexicon_tags_19 },
    { 0x6e51eeca, SUC_lexicon_tags_45 },
    { 0xe531eecb, SUC_lexicon_tags_10 },
    { 0x2011eecb, SUC_lexicon_tags_61 },
    { 0xaa11eecb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71eed1, SUC_lexicon_tags_1 },
    { 0xd671eed1, SUC_lexicon_tags_17 },
    { 0xe71eed3, SUC_lexicon_tags_6 },
    { 0x8871eed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc191eed7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2211eeda, SUC_lexicon_tags_27 },
    { 0x6b1eedb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc11eee3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1eee7, SUC_lexicon_tags_37 },
    { 0x3fb1eee8, SUC_lexicon_tags_15 },
    { 0xec31eee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e51eef0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f1eef3, SUC_lexicon_tags_17 },
    { 0xd991eef4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5871eef6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8231eefc, SUC_lexicon_tags_17 },
    { 0x3cb1eefc, SUC_lexicon_tags_91 },
    { 0xa251eefc, SUC_lexicon_tags_77 },
    { 0x19d1eeff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d1ef02, SUC_lexicon_tags_25 },
    { 0xf131ef03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db1ef06, SUC_lexicon_tags_15 },
    { 0x1811ef07, SUC_lexicon_tags_2 },
    { 0x5751ef07, SUC_lexicon_tags_29 },
    { 0x9cb1ef08, SUC_lexicon_tags_2 },
    { 0x5991ef0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b1ef0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8271ef11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2151ef13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd591ef15, SUC_lexicon_tags_57 },
    { 0xc9d1ef15, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf1ef20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd1ef26, SUC_lexicon_tags_25 },
    { 0x8eb1ef27, SUC_lexicon_tags_2 },
    { 0xfed1ef28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xff71ef2b, SUC_lexicon_tags_2 },
    { 0xf531ef2c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b51ef2f, SUC_lexicon_tags_25 },
    { 0xb691ef30, SUC_lexicon_tags_8 },
    { 0x6d11ef31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e71ef34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd231ef36, SUC_lexicon_tags_29 },
    { 0xd1f1ef36, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb1ef3a, SUC_lexicon_tags_2 },
    { 0x9d71ef3b, SUC_lexicon_tags_7 },
    { 0xc191ef3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe851ef42, SUC_lexicon_tags_37 },
    { 0x46f1ef43, SUC_lexicon_tags_10 },
    { 0x80b1ef43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8111ef4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3531ef4f, SUC_lexicon_tags_33 },
    { 0x2231ef50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b1ef53, SUC_lexicon_tags_17 },
    { 0xa9f1ef54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f1ef58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1691ef5b, SUC_lexicon_tags_240 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b1ef60, SUC_lexicon_tags_26 },
    { 0x76f1ef61, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8fd1ef63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa931ef66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad1ef6c, SUC_lexicon_tags_207 },
    { 0xf851ef6d, SUC_lexicon_tags_17 },
    { 0xadf1ef6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d91ef70, SUC_lexicon_tags_14 },
    { 0x6991ef71, SUC_lexicon_tags_37 },
    { 0x8fd1ef72, SUC_lexicon_tags_25 },
    { 0x4d71ef73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f1ef7c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa11ef80, SUC_lexicon_tags_1 },
    { 0x2611ef81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaed1ef83, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x81d1ef85, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa691ef88, SUC_lexicon_tags_45 },
    { 0x4151ef89, SUC_lexicon_tags_33 },
    { 0x7a71ef8a, SUC_lexicon_tags_15 },
    { 0xe4b1ef8b, SUC_lexicon_tags_11 },
    { 0xa951ef8a, SUC_lexicon_tags_23 },
    { 0xb8d1ef8c, SUC_lexicon_tags_29 },
    { 0x2591ef88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8971ef94, SUC_lexicon_tags_55 },
    { 0xc451ef95, SUC_lexicon_tags_23 },
    { 0x2a31ef96, SUC_lexicon_tags_1 },
    { 0x9f91ef97, SUC_lexicon_tags_47 },
    { 0xdfd1ef98, SUC_lexicon_tags_7 },
    { 0x4651ef99, SUC_lexicon_tags_6 },
    { 0x8ed1ef99, SUC_lexicon_tags_138 },
    { 0xbe11ef9b, SUC_lexicon_tags_82 },
    { 0xac51ef9b, SUC_lexicon_tags_26 },
    { 0x98b1ef9d, SUC_lexicon_tags_25 },
    { 0x251ef9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2911efa2, SUC_lexicon_tags_7 },
    { 0x49d1efa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b1efa8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4d51efaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1831efb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd731efb2, SUC_lexicon_tags_7 },
    { 0x5351efb2, SUC_lexicon_tags_2 },
    { 0x25d1efb4, SUC_lexicon_tags_46 },
    { 0xcc71efb5, SUC_lexicon_tags_15 },
    { 0x28f1efb6, SUC_lexicon_tags_15 },
    { 0x8231efb7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f1efc0, SUC_lexicon_tags_19 },
    { 0xceb1efc0, SUC_lexicon_tags_14 },
    { 0x5cb1efc2, SUC_lexicon_tags_12 },
    { 0x8291efc3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ad1efc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc671efd2, SUC_lexicon_tags_1 },
    { 0xb091efd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5d1efd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b1efd9, SUC_lexicon_tags_29 },
    { 0xa191efda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5271efdc, SUC_lexicon_tags_11 },
    { 0xb851efdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b1efe2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4791efe6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f1efec, SUC_lexicon_tags_3 },
    { 0x8f51efed, SUC_lexicon_tags_37 },
    { 0xd3b1efee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2891eff3, SUC_lexicon_tags_11 },
    { 0xacb1eff4, SUC_lexicon_tags_2 },
    { 0x6171eff5, SUC_lexicon_tags_10 },
    { 0x4dd1eff6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa671effb, SUC_lexicon_tags_0 },
    { 0x3ad1effb, SUC_lexicon_tags_63 },
    { 0xca71effc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb31efff, SUC_lexicon_tags_503 },
    { 0xe571f000, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a71f005, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb1f008, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f1f00d, SUC_lexicon_tags_77 },
    { 0x1eb1f00e, SUC_lexicon_tags_37 },
    { 0x4831f00f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x2eb1f011, SUC_lexicon_tags_43 },
    { 0xdab1f012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeab1f014, SUC_lexicon_tags_1 },
    { 0x3131f015, SUC_lexicon_tags_68 },
    { 0xe931f016, SUC_lexicon_tags_29 },
    { 0xec91f015, SUC_lexicon_tags_17 },
    { 0x4d51f016, SUC_lexicon_tags_1 },
    { 0x4371f015, SUC_lexicon_tags_47 },
    { 0x85f1f01a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe551f01e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f1f024, SUC_lexicon_tags_2 },
    { 0x23b1f025, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf31f029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf1f02b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59d1f02d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef1f030, SUC_lexicon_tags_33 },
    { 0x33d1f030, SUC_lexicon_tags_6 },
    { 0xc991f032, SUC_lexicon_tags_11 },
    { 0x8951f031, SUC_lexicon_tags_7 },
    { 0x3711f034, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0f1f036, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd1f03c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xee31f03e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe971f041, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed11f043, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3271f048, SUC_lexicon_tags_12 },
    { 0x9bb1f049, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4e51f04b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b1f04e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3171f050, SUC_lexicon_tags_11 },
    { 0xbdf1f051, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f1f055, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d31f058, SUC_lexicon_tags_11 },
    { 0x5bf1f059, SUC_lexicon_tags_1 },
    { 0xcb31f059, SUC_lexicon_tags_37 },
    { 0x3a51f05b, SUC_lexicon_tags_66 },
    { 0xd6f1f05c, SUC_lexicon_tags_17 },
    { 0x8731f05c, SUC_lexicon_tags_47 },
    { 0x5671f058, SUC_lexicon_tags_23 },
    { 0xd011f05f, SUC_lexicon_tags_1573 },
    { 0xf671f060, SUC_lexicon_tags_49 },
    { 0x1cf1f061, SUC_lexicon_tags_25 },
    { 0xceb1f05d, SUC_lexicon_tags_23 },
    { 0x9e11f060, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d1f069, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf1f073, SUC_lexicon_tags_21 },
    { 0x12d1f073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b1f078, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x79b1f07a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f1f07d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1f080, SUC_lexicon_tags_45 },
    { 0x6a51f081, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc211f087, SUC_lexicon_tags_2 },
    { 0xc11f088, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb071f08a, SUC_lexicon_tags_14 },
    { 0x33f1f08a, SUC_lexicon_tags_29 },
    { 0xc451f08a, SUC_lexicon_tags_27 },
    { 0xaa71f08d, SUC_lexicon_tags_8 },
    { 0x7d11f08d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e71f092, SUC_lexicon_tags_31 },
    { 0xb6d1f093, SUC_lexicon_tags_68 },
    { 0xc7f1f093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f51f09e, SUC_lexicon_tags_1 },
    { 0xefb1f09f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9951f0a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a51f0a3, SUC_lexicon_tags_11 },
    { 0x62f1f0a4, SUC_lexicon_tags_17 },
    { 0xa631f0a4, SUC_lexicon_tags_33 },
    { 0xd0d1f0a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d1f0ae, SUC_lexicon_tags_25 },
    { 0xfe11f0af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd511f0b2, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd11f0b9, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x2d11f0bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9871f0c1, SUC_lexicon_tags_12 },
    { 0x391f0c2, SUC_lexicon_tags_33 },
    { 0xfd51f0c3, SUC_lexicon_tags_19 },
    { 0xab1f0c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6691f0c9, SUC_lexicon_tags_37 },
    { 0x2f51f0ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3151f0cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf651f0d2, SUC_lexicon_tags_17 },
    { 0xc5b1f0d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11f0d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc531f0da, SUC_lexicon_tags_47 },
    { 0xb6f1f0db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e11f0dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f1f0e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa51f0e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd871f0ea, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcc31f0ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4411f0ee, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed1f0f3, SUC_lexicon_tags_47 },
    { 0x8ff1f0f4, SUC_lexicon_tags_60 },
    { 0x99b1f0f5, SUC_lexicon_tags_33 },
    { 0xaaf1f0f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc651f0f9, SUC_lexicon_tags_17 },
    { 0x8251f0fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f71f0fc, SUC_lexicon_tags_60 },
    { 0x8351f0fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8431f104, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc91f108, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb331f10a, SUC_lexicon_tags_10 },
    { 0x99b1f10b, SUC_lexicon_tags_7 },
    { 0xedf1f10c, SUC_lexicon_tags_7 },
    { 0x5651f10c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae31f115, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4571f117, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb11f11b, SUC_lexicon_tags_82 },
    { 0x7fb1f11c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff1f121, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5651f124, SUC_lexicon_tags_2 },
    { 0xa371f125, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd1f12a, SUC_lexicon_tags_63 },
    { 0xfcd1f12b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d1f130, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d1f133, SUC_lexicon_tags_43 },
    { 0xa3b1f133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1931f136, SUC_lexicon_tags_25 },
    { 0x8871f136, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b51f13d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9d1f13f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd911f145, SUC_lexicon_tags_47 },
    { 0x1a11f146, SUC_lexicon_tags_17 },
    { 0x4b11f147, SUC_lexicon_tags_17 },
    { 0x8551f148, SUC_lexicon_tags_25 },
    { 0xdf51f148, SUC_lexicon_tags_16 },
    { 0x6411f14a, SUC_lexicon_tags_15 },
    { 0x7431f145, SUC_lexicon_tags_77 },
    { 0xc91f146, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf251f14e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4f31f150, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb1f159, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f71f15c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd151f160, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae91f164, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe1f1f166, SUC_lexicon_tags_16 },
    { 0x2871f167, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1c11f169, SUC_lexicon_tags_12 },
    { 0xc3f1f16a, SUC_lexicon_tags_14 },
    { 0xc871f16a, SUC_lexicon_tags_16 },
    { 0x8eb1f16c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4291f16f, SUC_lexicon_tags_43 },
    { 0x4191f16f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef1f178, SUC_lexicon_tags_11 },
    { 0x2491f178, SUC_lexicon_tags_47 },
    { 0xacd1f17a, SUC_lexicon_tags_25 },
    { 0xc9f1f17a, SUC_lexicon_tags_37 },
    { 0x9811f17c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d1f181, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x7371f184, SUC_lexicon_tags_27 },
    { 0x4e11f184, SUC_lexicon_tags_7 },
    { 0xe871f186, SUC_lexicon_tags_2 },
    { 0x5631f185, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6991f18a, SUC_lexicon_tags_43 },
    { 0xf311f18b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f1f18e, SUC_lexicon_tags_1 },
    { 0x57f1f18e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd71f193, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfbb1f195, SUC_lexicon_tags_24 },
    { 0xc5d1f195, SUC_lexicon_tags_37 },
    { 0x7f11f197, SUC_lexicon_tags_37 },
    { 0xb091f198, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4f1f19a, SUC_lexicon_tags_17 },
    { 0xbc31f19b, SUC_lexicon_tags_1 },
    { 0x4b91f19c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f51f1a0, SUC_lexicon_tags_65 },
    { 0xe171f1a1, SUC_lexicon_tags_47 },
    { 0x11f1f1a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa231f1a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b1f1a7, SUC_lexicon_tags_11 },
    { 0x7d1f1a7, SUC_lexicon_tags_47 },
    { 0xd211f1a9, SUC_lexicon_tags_39 },
    { 0x9291f1aa, SUC_lexicon_tags_31 },
    { 0xa731f1a9, SUC_lexicon_tags_37 },
    { 0x96b1f1a9, SUC_lexicon_tags_17 },
    { 0xcd91f1ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe131f1af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb1f1b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7ab1f1b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2ef1f1b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9511f1b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d1f1bd, SUC_lexicon_tags_19 },
    { 0x2ed1f1bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe431f1c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac31f1c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f1f1c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2051f1cd, SUC_lexicon_tags_11 },
    { 0xe8d1f1cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a51f1d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4331f1d6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2471f1db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391f1df, SUC_lexicon_tags_83 },
    { 0x1c11f1e0, SUC_lexicon_tags_11 },
    { 0x6591f1df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2971f1e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c11f1ed, SUC_lexicon_tags_9 },
    { 0xb251f1ee, SUC_lexicon_tags_2 },
    { 0x5351f1ee, SUC_lexicon_tags_3 },
    { 0xd551f1f0, SUC_lexicon_tags_37 },
    { 0x27d1f1f1, SUC_lexicon_tags_7 },
    { 0xfe71f1f0, SUC_lexicon_tags_57 },
    { 0xe731f1ed, SUC_lexicon_tags_1 },
    { 0x6171f1f4, SUC_lexicon_tags_11 },
    { 0xdd1f1f2, SUC_lexicon_tags_11 },
    { 0x1e51f1ee, SUC_lexicon_tags_35 },
    { 0x87d1f1ee, SUC_lexicon_tags_21 },
    { 0x6571f1f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3531f1fa, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xb471f1fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdd31f1ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2051f204, SUC_lexicon_tags_27 },
    { 0xd0f1f205, SUC_lexicon_tags_7 },
    { 0x9351f205, SUC_lexicon_tags_21 },
    { 0x311f207, SUC_lexicon_tags_12 },
    { 0x80f1f205, SUC_lexicon_tags_38 },
    { 0x6471f209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5071f20b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1871f20d, SUC_lexicon_tags_9 },
    { 0xd171f20d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac91f213, SUC_lexicon_tags_30 },
    { 0x4f31f213, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf1f218, SUC_lexicon_tags_17 },
    { 0x551f218, SUC_lexicon_tags_45 },
    { 0x3d71f21a, SUC_lexicon_tags_27 },
    { 0x2431f219, SUC_lexicon_tags_47 },
    { 0x711f21c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba71f223, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7751f227, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b1f22a, SUC_lexicon_tags_15 },
    { 0xf691f22b, SUC_lexicon_tags_49 },
    { 0x3891f22c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2cf1f22e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e71f230, SUC_lexicon_tags_2 },
    { 0x6e91f230, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d1f235, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cb1f237, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d1f23d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1851f241, SUC_lexicon_tags_25 },
    { 0xbd71f242, SUC_lexicon_tags_26 },
    { 0xfb31f241, SUC_lexicon_tags_60 },
    { 0xd811f244, SUC_lexicon_tags_19 },
    { 0x82f1f241, SUC_lexicon_tags_9 },
    { 0x1991f245, SUC_lexicon_tags_75 },
    { 0xc851f247, SUC_lexicon_tags_25 },
    { 0x891f248, SUC_lexicon_tags_12 },
    { 0x63d1f248, SUC_lexicon_tags_17 },
    { 0x7251f24a, SUC_lexicon_tags_15 },
    { 0xc791f249, SUC_lexicon_tags_1 },
    { 0x2171f24c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0b1f24e, SUC_lexicon_tags_17 },
    { 0x6fb1f24f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf11f251, SUC_lexicon_tags_37 },
    { 0x1a91f251, SUC_lexicon_tags_1 },
    { 0x24d1f253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87d1f255, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x74f1f257, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf11f25b, SUC_lexicon_tags_17 },
    { 0x2a11f25c, SUC_lexicon_tags_12 },
    { 0x4091f25d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf31f260, SUC_lexicon_tags_66 },
    { 0xa791f261, SUC_lexicon_tags_25 },
    { 0x8f31f262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1231f264, SUC_lexicon_tags_25 },
    { 0x1151f265, SUC_lexicon_tags_17 },
    { 0xacf1f264, SUC_lexicon_tags_7 },
    { 0xfe51f265, SUC_lexicon_tags_12 },
    { 0xdd71f266, SUC_lexicon_tags_37 },
    { 0xba31f265, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5051f26b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b71f26f, SUC_lexicon_tags_33 },
    { 0xe591f26f, SUC_lexicon_tags_33 },
    { 0xa4b1f271, SUC_lexicon_tags_49 },
    { 0x9391f272, SUC_lexicon_tags_2 },
    { 0x62d1f273, SUC_lexicon_tags_37 },
    { 0xe8f1f272, SUC_lexicon_tags_19 },
    { 0x1371f275, SUC_lexicon_tags_11 },
    { 0xef71f271, SUC_lexicon_tags_25 },
    { 0x7831f277, SUC_lexicon_tags_8 },
    { 0x2731f278, SUC_lexicon_tags_19 },
    { 0xa951f279, SUC_lexicon_tags_12 },
    { 0xb891f27a, SUC_lexicon_tags_0 },
    { 0x8ff1f275, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5671f282, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d1f288, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf6f1f28a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf851f28d, SUC_lexicon_tags_57 },
    { 0x63f1f28e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xcbd1f290, SUC_lexicon_tags_37 },
    { 0xa2f1f291, SUC_lexicon_tags_11 },
    { 0x7ad1f290, SUC_lexicon_tags_7 },
    { 0xdef1f293, SUC_lexicon_tags_1 },
    { 0x8211f294, SUC_lexicon_tags_8 },
    { 0xd091f294, SUC_lexicon_tags_13 },
    { 0x7051f296, SUC_lexicon_tags_17 },
    { 0x4a31f297, SUC_lexicon_tags_11 },
    { 0xf1f297, SUC_lexicon_tags_16 },
    { 0x7cf1f297, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5851f29b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e91f2a0, SUC_lexicon_tags_11 },
    { 0xf751f2a0, SUC_lexicon_tags_49 },
    { 0xe111f2a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd551f2a7, SUC_lexicon_tags_45 },
    { 0xaf91f2a8, SUC_lexicon_tags_17 },
    { 0xc1b1f2a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa651f2ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3851f2ad, SUC_lexicon_tags_87 },
    { 0xa6b1f2ad, SUC_lexicon_tags_81 },
    { 0x55d1f2af, SUC_lexicon_tags_11 },
    { 0x7111f2b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8671f2b5, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f1f2b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d1f2bc, SUC_lexicon_tags_15 },
    { 0x3b31f2bd, SUC_lexicon_tags_38 },
    { 0x59f1f2bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f91f2c1, SUC_lexicon_tags_27 },
    { 0x93f1f2c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2331f2c6, SUC_lexicon_tags_23 },
    { 0x5d31f2c7, SUC_lexicon_tags_8 },
    { 0x3551f2c7, SUC_lexicon_tags_27 },
    { 0x5791f2c9, SUC_lexicon_tags_32 },
    { 0xa831f2ca, SUC_lexicon_tags_9 },
    { 0xe8f1f2cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d1f2d4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa251f2d6, SUC_lexicon_tags_29 },
    { 0xa91f2d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3491f2d9, SUC_lexicon_tags_47 },
    { 0xd491f2da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4071f2dc, SUC_lexicon_tags_37 },
    { 0xa31f2dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b1f2e0, SUC_lexicon_tags_12 },
    { 0x2ef1f2e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe151f2e3, SUC_lexicon_tags_17 },
    { 0xcdf1f2e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d91f2e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff1f2eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf8b1f2ed, SUC_lexicon_tags_37 },
    { 0x6591f2ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6751f2f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d51f2f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5391f2fa, SUC_lexicon_tags_11 },
    { 0xfa31f2fb, SUC_lexicon_tags_29 },
    { 0x1811f2fc, SUC_lexicon_tags_31 },
    { 0xb531f2fd, SUC_lexicon_tags_17 },
    { 0xb8b1f2fd, SUC_lexicon_tags_9 },
    { 0xa251f2ff, SUC_lexicon_tags_30 },
    { 0x3351f2fa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5471f303, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa11f30a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x651f30d, SUC_lexicon_tags_11 },
    { 0x2ad1f30d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x45d1f310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1051f313, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf1f317, SUC_lexicon_tags_17 },
    { 0x7a71f317, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d1f322, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8571f324, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4171f326, SUC_lexicon_tags_15 },
    { 0x7411f326, SUC_lexicon_tags_14 },
    { 0xc171f326, SUC_lexicon_tags_25 },
    { 0xf0b1f326, SUC_lexicon_tags_7 },
    { 0x31b1f32a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6991f32c, SUC_lexicon_tags_5 },
    { 0x75b1f32c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f91f32f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d1f332, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b1f33a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc91f33c, SUC_lexicon_tags_12 },
    { 0x1c51f33d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1591f340, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f1f346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f1f354, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd1f359, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f1f35c, SUC_lexicon_tags_22 },
    { 0xe9f1f35c, SUC_lexicon_tags_14 },
    { 0x4931f35e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f1f363, SUC_lexicon_tags_30 },
    { 0x771f363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc931f367, SUC_lexicon_tags_77 },
    { 0xfcf1f367, SUC_lexicon_tags_19 },
    { 0x7cb1f369, SUC_lexicon_tags_45 },
    { 0x9e71f36a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f11f36d, SUC_lexicon_tags_1 },
    { 0x4a91f36e, SUC_lexicon_tags_37 },
    { 0x61b1f36f, SUC_lexicon_tags_12 },
    { 0xf211f36e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb1f374, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb331f376, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db1f37b, SUC_lexicon_tags_27 },
    { 0x1091f37b, SUC_lexicon_tags_32 },
    { 0x2a91f37d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xa51f37f, SUC_lexicon_tags_26 },
    { 0x7f71f380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5551f385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1891f38b, SUC_lexicon_tags_2 },
    { 0x8b91f38c, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c71f38f, SUC_lexicon_tags_1 },
    { 0x18d1f38f, SUC_lexicon_tags_12 },
    { 0x5f51f38f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f1f394, SUC_lexicon_tags_12 },
    { 0x7451f395, SUC_lexicon_tags_40 },
    { 0x6db1f395, SUC_lexicon_tags_7 },
    { 0xf711f397, SUC_lexicon_tags_39 },
    { 0x98b1f398, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d91f39f, SUC_lexicon_tags_8 },
    { 0xd4d1f3a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc11f3a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b1f3aa, SUC_lexicon_tags_11 },
    { 0x21b1f3ab, SUC_lexicon_tags_17 },
    { 0x791f3ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b1f3b4, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xaed1f3b6, SUC_lexicon_tags_25 },
    { 0xfef1f3b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xffd1f3b9, SUC_lexicon_tags_29 },
    { 0x1611f3b9, SUC_lexicon_tags_15 },
    { 0xa191f3ba, SUC_lexicon_tags_7 },
    { 0x8811f3bc, SUC_lexicon_tags_7 },
    { 0x8fd1f3bd, SUC_lexicon_tags_2 },
    { 0xa4f1f3be, SUC_lexicon_tags_7 },
    { 0x7c51f3bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b91f3c6, SUC_lexicon_tags_25 },
    { 0x98f1f3c7, SUC_lexicon_tags_1 },
    { 0x73f1f3c8, SUC_lexicon_tags_8 },
    { 0xc4b1f3c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x191f3cb, SUC_lexicon_tags_17 },
    { 0x6bf1f3cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d71f3cf, SUC_lexicon_tags_9 },
    { 0x6351f3cf, SUC_lexicon_tags_1 },
    { 0xf2b1f3d1, SUC_lexicon_tags_7 },
    { 0xc5b1f3d1, SUC_lexicon_tags_2 },
    { 0x72b1f3d3, SUC_lexicon_tags_11 },
    { 0x2451f3d4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8b71f3d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d1f3d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf1f3db, SUC_lexicon_tags_4 },
    { 0x9e91f3dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b11f3de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf31f3e0, SUC_lexicon_tags_85 },
    { 0xd411f3e0, SUC_lexicon_tags_26 },
    { 0xaf91f3e2, SUC_lexicon_tags_0 },
    { 0x6c71f3e3, SUC_lexicon_tags_37 },
    { 0xd351f3e3, SUC_lexicon_tags_46 },
    { 0xd6d1f3e5, SUC_lexicon_tags_27 },
    { 0x1211f3e5, SUC_lexicon_tags_37 },
    { 0x6cd1f3e4, SUC_lexicon_tags_63 },
    { 0xec91f3e6, SUC_lexicon_tags_32 },
    { 0x23b1f3e9, SUC_lexicon_tags_17 },
    { 0x1031f3e9, SUC_lexicon_tags_0 },
    { 0xca91f3e4, SUC_lexicon_tags_17 },
    { 0x94b1f3e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4671f3f1, SUC_lexicon_tags_250 },
    { 0xe951f3f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c51f3f6, SUC_lexicon_tags_25 },
    { 0xde51f3f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36b1f3f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xabd1f3fb, SUC_lexicon_tags_1 },
    { 0xdfb1f3fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91f405, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x52b1f407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb1f40b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7771f40e, SUC_lexicon_tags_33 },
    { 0x1891f40f, SUC_lexicon_tags_1 },
    { 0x8111f40f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8631f414, SUC_lexicon_tags_19 },
    { 0xd431f415, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f419, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71f420, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f423, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb391f425, SUC_lexicon_tags_3 },
    { 0x2531f426, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xebd1f428, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4491f42a, SUC_lexicon_tags_13 },
    { 0x28f1f42a, SUC_lexicon_tags_23 },
    { 0x8831f42c, SUC_lexicon_tags_2 },
    { 0xe031f42d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30f1f42f, SUC_lexicon_tags_2 },
    { 0x4751f430, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5971f434, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e51f437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f1f43b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3551f43d, SUC_lexicon_tags_26 },
    { 0x3d1f43e, SUC_lexicon_tags_7 },
    { 0x32f1f43f, SUC_lexicon_tags_26 },
    { 0xb71f43f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc691f442, SUC_lexicon_tags_1 },
    { 0x3a11f443, SUC_lexicon_tags_7 },
    { 0x9f51f444, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9871f446, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cf1f448, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d1f44b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb511f453, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xee91f456, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37f1f458, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9751f45c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f1f45f, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7011f473, SUC_lexicon_tags_12 },
    { 0x7491f474, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcb71f476, SUC_lexicon_tags_12 },
    { 0x15b1f476, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab1f47c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3251f47f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d71f487, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af1f48b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7351f494, SUC_lexicon_tags_37 },
    { 0xd5f1f494, SUC_lexicon_tags_12 },
    { 0x8571f496, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb071f498, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x37b1f49a, SUC_lexicon_tags_37 },
    { 0xfdf1f49a, SUC_lexicon_tags_17 },
    { 0x1c11f49c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6b1f49e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb1f4a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd031f4a7, SUC_lexicon_tags_29 },
    { 0xe2b1f4a8, SUC_lexicon_tags_9 },
    { 0x1091f4a7, SUC_lexicon_tags_56 },
    { 0xe011f4a8, SUC_lexicon_tags_70 },
    { 0x8311f4ab, SUC_lexicon_tags_12 },
    { 0xf5f1f4ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d1f4af, SUC_lexicon_tags_28 },
    { 0x5231f4af, SUC_lexicon_tags_7 },
    { 0xc6d1f4b0, SUC_lexicon_tags_0 },
    { 0x7a71f4b2, SUC_lexicon_tags_7 },
    { 0xe9f1f4b3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2f31f4b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1751f4ba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x71f1f4bc, SUC_lexicon_tags_7 },
    { 0x1d71f4bc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d1f4c1, SUC_lexicon_tags_68 },
    { 0x5ff1f4c2, SUC_lexicon_tags_7 },
    { 0x89b1f4c1, SUC_lexicon_tags_44 },
    { 0x3231f4c2, SUC_lexicon_tags_11 },
    { 0x1df1f4c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1131f4c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa11f4c9, SUC_lexicon_tags_21 },
    { 0xf7d1f4ca, SUC_lexicon_tags_37 },
    { 0x8551f4ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2051f4ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c91f4d7, SUC_lexicon_tags_17 },
    { 0xcff1f4d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x251f4da, SUC_lexicon_tags_19 },
    { 0xc1b1f4db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d1f4e1, SUC_lexicon_tags_25 },
    { 0xbaf1f4e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4331f4e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7471f4e8, SUC_lexicon_tags_11 },
    { 0x5bd1f4e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6491f4ec, SUC_lexicon_tags_27 },
    { 0x7431f4ed, SUC_lexicon_tags_6 },
    { 0xcab1f4ee, SUC_lexicon_tags_77 },
    { 0x9c11f4ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e91f4f2, SUC_lexicon_tags_1 },
    { 0x11f4f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3111f4f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1f4fc, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b1f500, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd311f505, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4811f50c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1f512, SUC_lexicon_tags_2 },
    { 0x5bb1f513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc51f517, SUC_lexicon_tags_8 },
    { 0x3ef1f517, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1611f51b, SUC_lexicon_tags_37 },
    { 0x1871f51c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd1f521, SUC_lexicon_tags_1 },
    { 0xa111f521, SUC_lexicon_tags_37 },
    { 0xe691f523, SUC_lexicon_tags_112 },
    { 0xa931f524, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1931f527, SUC_lexicon_tags_37 },
    { 0xbfb1f528, SUC_lexicon_tags_10 },
    { 0x7691f528, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba71f52e, SUC_lexicon_tags_7 },
    { 0x2f11f52f, SUC_lexicon_tags_25 },
    { 0xe3f1f52f, SUC_lexicon_tags_25 },
    { 0x4771f531, SUC_lexicon_tags_37 },
    { 0x431f531, SUC_lexicon_tags_12 },
    { 0x4b71f533, SUC_lexicon_tags_12 },
    { 0x7951f52f, SUC_lexicon_tags_27 },
    { 0x2e11f535, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a11f537, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8151f539, SUC_lexicon_tags_6 },
    { 0x6731f53a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f1f53e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf1f542, SUC_lexicon_tags_15 },
    { 0xa771f542, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe131f547, SUC_lexicon_tags_37 },
    { 0x4651f548, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x20f1f54a, SUC_lexicon_tags_7 },
    { 0xe791f54b, SUC_lexicon_tags_57 },
    { 0x3a31f54b, SUC_lexicon_tags_63 },
    { 0x4051f54b, SUC_lexicon_tags_7 },
    { 0x5291f54e, SUC_lexicon_tags_63 },
    { 0x6131f54f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f1f553, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa711f560, SUC_lexicon_tags_3 },
    { 0x6b31f560, SUC_lexicon_tags_17 },
    { 0x45d1f562, SUC_lexicon_tags_61 },
    { 0xb2b1f563, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc131f565, SUC_lexicon_tags_37 },
    { 0x7671f566, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed51f56a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3151f56c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xca51f56e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca31f573, SUC_lexicon_tags_16 },
    { 0x64d1f573, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b51f57a, SUC_lexicon_tags_31 },
    { 0x38f1f57a, SUC_lexicon_tags_2 },
    { 0x8f11f57c, SUC_lexicon_tags_2 },
    { 0xae11f57d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b1f581, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5391f58a, SUC_lexicon_tags_25 },
    { 0xdf91f58a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf371f591, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9751f593, SUC_lexicon_tags_8 },
    { 0x7ef1f594, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f11f598, SUC_lexicon_tags_0 },
    { 0x54f1f598, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc111f59b, SUC_lexicon_tags_15 },
    { 0x1c91f59c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe11f59e, SUC_lexicon_tags_37 },
    { 0xe451f59e, SUC_lexicon_tags_11 },
    { 0xfd51f59e, SUC_lexicon_tags_29 },
    { 0x4651f59f, SUC_lexicon_tags_29 },
    { 0xec51f5a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf5d1f5a4, SUC_lexicon_tags_25 },
    { 0x7771f5a5, SUC_lexicon_tags_45 },
    { 0x1811f5a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a71f5ad, SUC_lexicon_tags_11 },
    { 0x5d91f5ae, SUC_lexicon_tags_14 },
    { 0xd8d1f5af, SUC_lexicon_tags_25 },
    { 0xa011f5ae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd451f5b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x55b1f5b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb911f5b6, SUC_lexicon_tags_11 },
    { 0xf51f5b7, SUC_lexicon_tags_77 },
    { 0x6eb1f5b8, SUC_lexicon_tags_57 },
    { 0xf5d1f5b9, SUC_lexicon_tags_37 },
    { 0x89b1f5b8, SUC_lexicon_tags_27 },
    { 0xe8d1f5b9, SUC_lexicon_tags_69 },
    { 0x8d71f5ba, SUC_lexicon_tags_63 },
    { 0xdd71f5bd, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa71f5c1, SUC_lexicon_tags_15 },
    { 0x4a71f5c2, SUC_lexicon_tags_5 },
    { 0xc071f5c3, SUC_lexicon_tags_2 },
    { 0x5191f5c2, SUC_lexicon_tags_1 },
    { 0xdf51f5c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb691f5c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a91f5ce, SUC_lexicon_tags_136 },
    { 0x22b1f5cf, SUC_lexicon_tags_15 },
    { 0xc971f5d0, SUC_lexicon_tags_23 },
    { 0xc711f5d1, SUC_lexicon_tags_7 },
    { 0x33b1f5d2, SUC_lexicon_tags_12 },
    { 0x8351f5ce, SUC_lexicon_tags_9 },
    { 0x6b31f5d4, SUC_lexicon_tags_1 },
    { 0x2211f5d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b11f5d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1f5da, SUC_lexicon_tags_38 },
    { 0xa471f5db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed91f5e1, SUC_lexicon_tags_11 },
    { 0xd7d1f5e1, SUC_lexicon_tags_31 },
    { 0x71d1f5e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab1f5e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc51f5ea, SUC_lexicon_tags_33 },
    { 0x7091f5ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f1f5ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d1f5f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7011f5f7, SUC_lexicon_tags_1 },
    { 0x1c11f5f7, SUC_lexicon_tags_12 },
    { 0x8571f5f8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x41d1f5fb, SUC_lexicon_tags_60 },
    { 0xef1f5fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffb1f5fe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2ef1f600, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d71f604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f91f607, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3af1f609, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b1f60f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d71f617, SUC_lexicon_tags_25 },
    { 0xfb1f618, SUC_lexicon_tags_49 },
    { 0xcd71f618, SUC_lexicon_tags_37 },
    { 0xaaf1f61a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xad1f61c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d1f620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbb1f622, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b1f625, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9351f62c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b11f62f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x94b1f631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce51f634, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1371f636, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2291f638, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2051f63e, SUC_lexicon_tags_47 },
    { 0xa071f63f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb1f642, SUC_lexicon_tags_92 },
    { 0x4451f643, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c91f645, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f11f647, SUC_lexicon_tags_25 },
    { 0xe071f647, SUC_lexicon_tags_27 },
    { 0x6931f648, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b1f64c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c71f65e, SUC_lexicon_tags_14 },
    { 0xd1b1f65e, SUC_lexicon_tags_15 },
    { 0x7f71f65f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf371f662, SUC_lexicon_tags_2 },
    { 0x4851f662, SUC_lexicon_tags_37 },
    { 0x8491f664, SUC_lexicon_tags_8 },
    { 0x5431f665, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe811f66b, SUC_lexicon_tags_25 },
    { 0x36f1f66c, SUC_lexicon_tags_12 },
    { 0x94d1f66c, SUC_lexicon_tags_14 },
    { 0x76d1f66d, SUC_lexicon_tags_7 },
    { 0x6e31f66f, SUC_lexicon_tags_7 },
    { 0x64d1f66d, SUC_lexicon_tags_9 },
    { 0x1991f671, SUC_lexicon_tags_13 },
    { 0xf6b1f672, SUC_lexicon_tags_47 },
    { 0x1af1f671, SUC_lexicon_tags_1 },
    { 0x1c91f670, SUC_lexicon_tags_19 },
    { 0x15d1f673, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f1f679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4771f67c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b1f683, SUC_lexicon_tags_17 },
    { 0x8f31f683, SUC_lexicon_tags_45 },
    { 0xc0b1f683, SUC_lexicon_tags_15 },
    { 0x86f1f683, SUC_lexicon_tags_25 },
    { 0xba91f687, SUC_lexicon_tags_25 },
    { 0x8371f688, SUC_lexicon_tags_28 },
    { 0xcb71f688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa671f68c, SUC_lexicon_tags_61 },
    { 0xc9b1f68d, SUC_lexicon_tags_17 },
    { 0x5bf1f68e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc531f690, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1251f693, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b1f696, SUC_lexicon_tags_2 },
    { 0x62b1f696, SUC_lexicon_tags_47 },
    { 0xc731f697, SUC_lexicon_tags_498 },
    { 0x4f71f699, SUC_lexicon_tags_31 },
    { 0x6131f699, SUC_lexicon_tags_2 },
    { 0x8371f69b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9b71f69d, SUC_lexicon_tags_7 },
    { 0xc351f69d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf751f6a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf51f6a2, SUC_lexicon_tags_15 },
    { 0xe991f6a2, SUC_lexicon_tags_14 },
    { 0x2771f6a4, SUC_lexicon_tags_106 },
    { 0x74b1f6a5, SUC_lexicon_tags_37 },
    { 0xa851f6a6, SUC_lexicon_tags_65 },
    { 0xd9b1f6a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3d11f6a9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd9f1f6ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7f1f6ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9b1f6af, SUC_lexicon_tags_7 },
    { 0x3f11f6b0, SUC_lexicon_tags_14 },
    { 0x9931f6b0, SUC_lexicon_tags_39 },
    { 0x4571f6b2, SUC_lexicon_tags_37 },
    { 0xab51f6b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2ed1f6b5, SUC_lexicon_tags_17 },
    { 0xfe51f6b6, SUC_lexicon_tags_43 },
    { 0xb131f6b6, SUC_lexicon_tags_60 },
    { 0x5891f6b7, SUC_lexicon_tags_55 },
    { 0xffb1f6b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1f6bf, SUC_lexicon_tags_7 },
    { 0x5171f6bf, SUC_lexicon_tags_12 },
    { 0x2791f6c1, SUC_lexicon_tags_70 },
    { 0xcf31f6c2, SUC_lexicon_tags_47 },
    { 0x7631f6c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd311f6cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9691f6d1, SUC_lexicon_tags_38 },
    { 0x4db1f6d1, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x3f1f6d4, SUC_lexicon_tags_4 },
    { 0x21b1f6d5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f11f6dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1691f6de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9471f6e3, SUC_lexicon_tags_21 },
    { 0xdc31f6e4, SUC_lexicon_tags_0 },
    { 0xa51f6e4, SUC_lexicon_tags_19 },
    { 0x29f1f6e4, SUC_lexicon_tags_19 },
    { 0x1b11f6e7, SUC_lexicon_tags_25 },
    { 0x3cd1f6e8, SUC_lexicon_tags_29 },
    { 0xf911f6e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b51f6ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e51f6f2, SUC_lexicon_tags_17 },
    { 0xbaf1f6f3, SUC_lexicon_tags_26 },
    { 0x36b1f6f4, SUC_lexicon_tags_22 },
    { 0x45d1f6f5, SUC_lexicon_tags_68 },
    { 0xa3d1f6f3, SUC_lexicon_tags_7 },
    { 0x4e31f6f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7ad1f6f9, SUC_lexicon_tags_37 },
    { 0x9f51f6fa, SUC_lexicon_tags_2 },
    { 0xdb31f6f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac71f6fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1771f6ff, SUC_lexicon_tags_43 },
    { 0x34d1f6ff, SUC_lexicon_tags_38 },
    { 0x3e71f700, SUC_lexicon_tags_37 },
    { 0x3ad1f700, SUC_lexicon_tags_7 },
    { 0x2571f703, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d71f70b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e51f70d, SUC_lexicon_tags_38 },
    { 0x151f70d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf351f710, SUC_lexicon_tags_33 },
    { 0xcb51f711, SUC_lexicon_tags_19 },
    { 0x6331f712, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5b1f714, SUC_lexicon_tags_7 },
    { 0x8ed1f715, SUC_lexicon_tags_9 },
    { 0x6e91f716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d1f71e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40f1f720, SUC_lexicon_tags_23 },
    { 0x5ad1f720, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d1f725, SUC_lexicon_tags_27 },
    { 0x7fd1f726, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x7691f728, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1771f72d, SUC_lexicon_tags_17 },
    { 0xd971f72e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a11f730, SUC_lexicon_tags_25 },
    { 0x1cd1f731, SUC_lexicon_tags_63 },
    { 0xaab1f732, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8291f736, SUC_lexicon_tags_17 },
    { 0x8c71f737, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d1f73f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1d1f741, SUC_lexicon_tags_69 },
    { 0xf8b1f742, SUC_lexicon_tags_25 },
    { 0xa3f1f743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b1f74d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab1f752, SUC_lexicon_tags_6 },
    { 0x41b1f753, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c11f75f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1f763, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee71f767, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf371f769, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe151f76c, SUC_lexicon_tags_2 },
    { 0xf911f76d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6971f773, SUC_lexicon_tags_2 },
    { 0x1551f773, SUC_lexicon_tags_14 },
    { 0x31f774, SUC_lexicon_tags_2 },
    { 0x21d1f776, SUC_lexicon_tags_26 },
    { 0x47f1f777, SUC_lexicon_tags_17 },
    { 0x7091f778, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f51f77d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd1f780, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d1f788, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f91f78a, SUC_lexicon_tags_14 },
    { 0xef51f78a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1f78e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b31f791, SUC_lexicon_tags_4 },
    { 0x1d31f792, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc311f796, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9731f799, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa8b1f79b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6011f7a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f1f7a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1f7ac, SUC_lexicon_tags_2 },
    { 0xd571f7ac, SUC_lexicon_tags_1 },
    { 0xd151f7ad, SUC_lexicon_tags_12 },
    { 0x24b1f7af, SUC_lexicon_tags_14 },
    { 0x9071f7b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a51f7b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7b1f7b7, SUC_lexicon_tags_2 },
    { 0x4671f7b8, SUC_lexicon_tags_37 },
    { 0x8e51f7b7, SUC_lexicon_tags_16 },
    { 0x6591f7ba, SUC_lexicon_tags_0 },
    { 0xce31f7bb, SUC_lexicon_tags_0 },
    { 0xe991f7bc, SUC_lexicon_tags_43 },
    { 0x79b1f7bc, SUC_lexicon_tags_15 },
    { 0x73d1f7be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa391f7c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf1f7cc, SUC_lexicon_tags_7 },
    { 0xcdd1f7cc, SUC_lexicon_tags_23 },
    { 0xda91f7ce, SUC_lexicon_tags_25 },
    { 0xb9f1f7ce, SUC_lexicon_tags_23 },
    { 0x57d1f7ce, SUC_lexicon_tags_12 },
    { 0x8871f7d0, SUC_lexicon_tags_17 },
    { 0x12d1f7d1, SUC_lexicon_tags_2 },
    { 0x8c51f7d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a51f7d7, SUC_lexicon_tags_37 },
    { 0xef71f7d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b51f7de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1231f7e3, SUC_lexicon_tags_37 },
    { 0xd051f7e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f71f7e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb1f7ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f51f7ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d1f7f2, SUC_lexicon_tags_68 },
    { 0x7f11f7f3, SUC_lexicon_tags_28 },
    { 0x1771f7f4, SUC_lexicon_tags_7 },
    { 0x5191f7f5, SUC_lexicon_tags_28 },
    { 0x7db1f7f6, SUC_lexicon_tags_55 },
    { 0x7031f7f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b1f7fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8091f800, SUC_lexicon_tags_19 },
    { 0x8351f801, SUC_lexicon_tags_60 },
    { 0xd8f1f801, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc671f805, SUC_lexicon_tags_0 },
    { 0xc731f806, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4251f808, SUC_lexicon_tags_8 },
    { 0xa891f809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb1f812, SUC_lexicon_tags_11 },
    { 0x45b1f813, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5891f816, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6431f81b, SUC_lexicon_tags_60 },
    { 0xe4d1f81c, SUC_lexicon_tags_37 },
    { 0x4951f81c, SUC_lexicon_tags_8 },
    { 0x9271f81e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db1f821, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbad1f823, SUC_lexicon_tags_47 },
    { 0x41d1f824, SUC_lexicon_tags_12 },
    { 0x78d1f825, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb551f827, SUC_lexicon_tags_17 },
    { 0x5651f827, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e71f831, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf631f835, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f1f838, SUC_lexicon_tags_7 },
    { 0x62b1f839, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d11f83d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e11f83f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f71f849, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe91f84f, SUC_lexicon_tags_37 },
    { 0xc951f84f, SUC_lexicon_tags_43 },
    { 0x88f1f850, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f1f856, SUC_lexicon_tags_8 },
    { 0x6b91f857, SUC_lexicon_tags_12 },
    { 0x7d31f857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31f85c, SUC_lexicon_tags_23 },
    { 0x89f1f85d, SUC_lexicon_tags_7 },
    { 0xe4d1f85e, SUC_lexicon_tags_21 },
    { 0xf631f85f, SUC_lexicon_tags_11 },
    { 0xf011f860, SUC_lexicon_tags_31 },
    { 0x24b1f85e, SUC_lexicon_tags_38 },
    { 0x36b1f862, SUC_lexicon_tags_14 },
    { 0xdb51f863, SUC_lexicon_tags_11 },
    { 0x6d1f864, SUC_lexicon_tags_45 },
    { 0xe771f865, SUC_lexicon_tags_21 },
    { 0x8f1f865, SUC_lexicon_tags_17 },
    { 0xad31f867, SUC_lexicon_tags_19 },
    { 0x111f868, SUC_lexicon_tags_42 },
    { 0xb891f866, SUC_lexicon_tags_77 },
    { 0xfd1f866, SUC_lexicon_tags_2 },
    { 0xdb11f864, SUC_lexicon_tags_45 },
    { 0xa671f865, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef1f874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1f878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f51f87b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc991f87d, SUC_lexicon_tags_33 },
    { 0xc631f87e, SUC_lexicon_tags_11 },
    { 0x46d1f87e, SUC_lexicon_tags_34 },
    { 0xbaf1f87e, SUC_lexicon_tags_113 },
    { 0xd1d1f881, SUC_lexicon_tags_11 },
    { 0x1bd1f882, SUC_lexicon_tags_32 },
    { 0x13b1f883, SUC_lexicon_tags_45 },
    { 0x1f71f884, SUC_lexicon_tags_49 },
    { 0x4791f885, SUC_lexicon_tags_37 },
    { 0x73b1f885, SUC_lexicon_tags_43 },
    { 0x4b31f887, SUC_lexicon_tags_27 },
    { 0xa011f884, SUC_lexicon_tags_1 },
    { 0xdf51f883, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf331f88c, SUC_lexicon_tags_17 },
    { 0xea91f88c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5451f894, SUC_lexicon_tags_77 },
    { 0x7f51f894, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b31f89c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc571f89f, SUC_lexicon_tags_65 },
    { 0x92f1f8a0, SUC_lexicon_tags_1 },
    { 0x5771f8a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9dd1f8a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb1f8a6, SUC_lexicon_tags_21 },
    { 0xf691f8a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f1f8ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11f8af, SUC_lexicon_tags_37 },
    { 0x671f8af, SUC_lexicon_tags_7 },
    { 0xbd31f8af, SUC_lexicon_tags_11 },
    { 0x1891f8b1, SUC_lexicon_tags_11 },
    { 0x6391f8b3, SUC_lexicon_tags_12 },
    { 0xa5f1f8af, SUC_lexicon_tags_64 },
    { 0xe871f8b5, SUC_lexicon_tags_28 },
    { 0x80b1f8b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac71f8bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfc71f8be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd931f8c0, SUC_lexicon_tags_2 },
    { 0xdb1f8c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5671f8c3, SUC_lexicon_tags_17 },
    { 0xb0f1f8c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2331f8c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3451f8c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf1f8ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb1f8ce, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5011f8d2, SUC_lexicon_tags_37 },
    { 0x4411f8d2, SUC_lexicon_tags_37 },
    { 0x8db1f8d2, SUC_lexicon_tags_0 },
    { 0xccb1f8d5, SUC_lexicon_tags_17 },
    { 0xcf51f8d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad1f8da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x67d1f8dc, SUC_lexicon_tags_14 },
    { 0x7f51f8dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f1f8e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2811f8e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd711f8e7, SUC_lexicon_tags_17 },
    { 0xc7d1f8e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2791f8ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb1f8f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1031f8fd, SUC_lexicon_tags_15 },
    { 0x2d71f8fe, SUC_lexicon_tags_63 },
    { 0xe071f8fd, SUC_lexicon_tags_37 },
    { 0x9d71f8ff, SUC_lexicon_tags_1 },
    { 0x63d1f901, SUC_lexicon_tags_27 },
    { 0xb1b1f902, SUC_lexicon_tags_66 },
    { 0x4451f8ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc511f90e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6451f911, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b1f914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d1f91b, SUC_lexicon_tags_17 },
    { 0x9591f91c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db1f921, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb91f923, SUC_lexicon_tags_27 },
    { 0xf1f923, SUC_lexicon_tags_31 },
    { 0x9351f925, SUC_lexicon_tags_25 },
    { 0x2d71f923, SUC_lexicon_tags_55 },
    { 0x6931f927, SUC_lexicon_tags_8 },
    { 0xe011f928, SUC_lexicon_tags_10 },
    { 0x7451f929, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5971f92c, SUC_lexicon_tags_63 },
    { 0x28b1f92d, SUC_lexicon_tags_7 },
    { 0x1b1f92e, SUC_lexicon_tags_2 },
    { 0xdb11f92f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31f935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac31f93b, SUC_lexicon_tags_68 },
    { 0xeb51f93c, SUC_lexicon_tags_23 },
    { 0x2911f93c, SUC_lexicon_tags_38 },
    { 0x73b1f93c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d1f941, SUC_lexicon_tags_25 },
    { 0x8db1f942, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d31f947, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1db1f949, SUC_lexicon_tags_25 },
    { 0x6f91f94a, SUC_lexicon_tags_45 },
    { 0x4891f94a, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1f94f, SUC_lexicon_tags_8 },
    { 0x2151f94f, SUC_lexicon_tags_11 },
    { 0x20d1f950, SUC_lexicon_tags_14 },
    { 0xd451f951, SUC_lexicon_tags_6 },
    { 0xa331f953, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a31f960, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2571f962, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d1f965, SUC_lexicon_tags_15 },
    { 0xf551f966, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b1f96b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b1f96e, SUC_lexicon_tags_15 },
    { 0x8a11f96e, SUC_lexicon_tags_12 },
    { 0x3811f96f, SUC_lexicon_tags_14 },
    { 0xe571f971, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa371f975, SUC_lexicon_tags_37 },
    { 0xc5d1f976, SUC_lexicon_tags_69 },
    { 0x4d91f976, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c11f979, SUC_lexicon_tags_37 },
    { 0x3691f979, SUC_lexicon_tags_37 },
    { 0xe491f97b, SUC_lexicon_tags_9 },
    { 0x22d1f97c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa511f97e, SUC_lexicon_tags_49 },
    { 0x74f1f97f, SUC_lexicon_tags_37 },
    { 0x30f1f97e, SUC_lexicon_tags_2 },
    { 0x96b1f981, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1711f984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2111f986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x151f988, SUC_lexicon_tags_47 },
    { 0xf991f989, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4931f98f, SUC_lexicon_tags_87 },
    { 0xb291f98f, SUC_lexicon_tags_8 },
    { 0xc3d1f990, SUC_lexicon_tags_37 },
    { 0xa071f992, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5631f995, SUC_lexicon_tags_2 },
    { 0x99d1f996, SUC_lexicon_tags_17 },
    { 0x35d1f996, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb91f99a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc191f99d, SUC_lexicon_tags_49 },
    { 0xe11f99d, SUC_lexicon_tags_60 },
    { 0x33d1f99d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f1f9a2, SUC_lexicon_tags_43 },
    { 0x74d1f9a3, SUC_lexicon_tags_15 },
    { 0x6c11f9a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d1f9a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d71f9a9, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2351f9ad, SUC_lexicon_tags_7 },
    { 0x8a11f9ae, SUC_lexicon_tags_37 },
    { 0xaf31f9ae, SUC_lexicon_tags_12 },
    { 0x1ef1f9af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9bd1f9b2, SUC_lexicon_tags_2 },
    { 0x4db1f9b2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b1f9b8, SUC_lexicon_tags_63 },
    { 0x9311f9b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1691f9be, SUC_lexicon_tags_0 },
    { 0x3f31f9bf, SUC_lexicon_tags_6 },
    { 0xdd31f9bf, SUC_lexicon_tags_55 },
    { 0x6e31f9bf, SUC_lexicon_tags_79 },
    { 0x6c11f9c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f1f9c5, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8111f9c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4811f9cc, SUC_lexicon_tags_23 },
    { 0x5ab1f9cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb11f9d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b1f9d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec11f9da, SUC_lexicon_tags_37 },
    { 0x8c51f9db, SUC_lexicon_tags_19 },
    { 0xfc31f9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8091f9e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82f1f9e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a31f9e5, SUC_lexicon_tags_8 },
    { 0x2591f9e6, SUC_lexicon_tags_2 },
    { 0x2f51f9e5, SUC_lexicon_tags_33 },
    { 0xa451f9e8, SUC_lexicon_tags_25 },
    { 0xa611f9e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9551f9eb, SUC_lexicon_tags_11 },
    { 0x2131f9ec, SUC_lexicon_tags_12 },
    { 0x6e51f9ec, SUC_lexicon_tags_299 },
    { 0x44b1f9ed, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7cf1f9f0, SUC_lexicon_tags_12 },
    { 0x4571f9f1, SUC_lexicon_tags_27 },
    { 0xc0f1f9f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1031f9f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6451f9f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831fa01, SUC_lexicon_tags_2 },
    { 0x5191fa02, SUC_lexicon_tags_11 },
    { 0xb031fa03, SUC_lexicon_tags_13 },
    { 0xa5f1fa03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd1fa08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1291fa0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bd1fa0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb1fa17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f1fa1e, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xb5d1fa20, SUC_lexicon_tags_9 },
    { 0xc4d1fa21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4931fa24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90f1fa26, SUC_lexicon_tags_25 },
    { 0x16d1fa27, SUC_lexicon_tags_27 },
    { 0x2f51fa27, SUC_lexicon_tags_11 },
    { 0xd391fa29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d51fa30, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdb51fa32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa811fa37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9231fa3c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f1fa46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x19f1fa48, SUC_lexicon_tags_0 },
    { 0x2e51fa49, SUC_lexicon_tags_25 },
    { 0xc471fa4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd1fa4e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f91fa51, SUC_lexicon_tags_1 },
    { 0x6551fa51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc671fa57, SUC_lexicon_tags_37 },
    { 0xf9d1fa57, SUC_lexicon_tags_66 },
    { 0x7df1fa58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6911fa63, SUC_lexicon_tags_2 },
    { 0x93d1fa63, SUC_lexicon_tags_11 },
    { 0x3e11fa63, SUC_lexicon_tags_25 },
    { 0x5f71fa64, SUC_lexicon_tags_15 },
    { 0x2491fa67, SUC_lexicon_tags_2 },
    { 0x7451fa67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d1fa73, SUC_lexicon_tags_25 },
    { 0x61d1fa74, SUC_lexicon_tags_33 },
    { 0xed1fa74, SUC_lexicon_tags_23 },
    { 0xd691fa76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d1fa7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa131fa7e, SUC_lexicon_tags_19 },
    { 0xb5b1fa7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd751fa82, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1fa8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f51fa90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7db1fa92, SUC_lexicon_tags_63 },
    { 0x14d1fa92, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d1fa9a, SUC_lexicon_tags_46 },
    { 0x9051fa9b, SUC_lexicon_tags_2 },
    { 0x9c71fa9c, SUC_lexicon_tags_15 },
    { 0xb1b1fa9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d1faa2, SUC_lexicon_tags_11 },
    { 0xa251faa3, SUC_lexicon_tags_27 },
    { 0x8db1faa2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbbd1faa6, SUC_lexicon_tags_46 },
    { 0xe851faa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f1faaa, SUC_lexicon_tags_33 },
    { 0xfa91faaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4971faad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc691fab0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5cf1fab2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d1faba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9871fabe, SUC_lexicon_tags_1 },
    { 0x6871fabe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8491fac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb1fac5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xda31fac7, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xe811fac9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f1fad3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f51fad5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa611fad7, SUC_lexicon_tags_37 },
    { 0x38f1fad7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe231fadb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x61d1fadd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd1fae1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa391fae5, SUC_lexicon_tags_6 },
    { 0xfb31fae5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc711fae8, SUC_lexicon_tags_11 },
    { 0x36d1fae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2f1faeb, SUC_lexicon_tags_17 },
    { 0xbe71faec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd1faef, SUC_lexicon_tags_26 },
    { 0xa691faf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc671faf2, SUC_lexicon_tags_11 },
    { 0x3b1faf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed1faf7, SUC_lexicon_tags_30 },
    { 0x9771faf8, SUC_lexicon_tags_25 },
    { 0xd571faf7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b1fafd, SUC_lexicon_tags_13 },
    { 0x1b71fafe, SUC_lexicon_tags_63 },
    { 0xb1d1fafe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f91fb03, SUC_lexicon_tags_10 },
    { 0x1471fb04, SUC_lexicon_tags_25 },
    { 0x8571fb03, SUC_lexicon_tags_31 },
    { 0xc6d1fb06, SUC_lexicon_tags_124 },
    { 0xb971fb07, SUC_lexicon_tags_28 },
    { 0x4ff1fb08, SUC_lexicon_tags_82 },
    { 0x83f1fb08, SUC_lexicon_tags_7 },
    { 0xfb71fb0a, SUC_lexicon_tags_37 },
    { 0x5291fb0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9671fb0f, SUC_lexicon_tags_37 },
    { 0xfa11fb10, SUC_lexicon_tags_7 },
    { 0x5391fb10, SUC_lexicon_tags_15 },
    { 0xa091fb11, SUC_lexicon_tags_48 },
    { 0xdb11fb10, SUC_lexicon_tags_17 },
    { 0xc71fb11, SUC_lexicon_tags_92 },
    { 0xcb91fb15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x351fb18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7071fb1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22b1fb1c, SUC_lexicon_tags_14 },
    { 0x4411fb1c, SUC_lexicon_tags_17 },
    { 0xee91fb1e, SUC_lexicon_tags_19 },
    { 0xc751fb1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f1fb23, SUC_lexicon_tags_12 },
    { 0x54f1fb24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe11fb26, SUC_lexicon_tags_27 },
    { 0x94d1fb26, SUC_lexicon_tags_69 },
    { 0x4af1fb27, SUC_lexicon_tags_1 },
    { 0xc7b1fb27, SUC_lexicon_tags_0 },
    { 0xc9d1fb2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca71fb2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b71fb32, SUC_lexicon_tags_11 },
    { 0x11f1fb32, SUC_lexicon_tags_11 },
    { 0xacb1fb34, SUC_lexicon_tags_6 },
    { 0xb391fb35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5751fb37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5551fb3a, SUC_lexicon_tags_23 },
    { 0xfcd1fb3a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5071fb3d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x73f1fb3f, SUC_lexicon_tags_12 },
    { 0x3cb1fb3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f31fb42, SUC_lexicon_tags_34 },
    { 0x37f1fb42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf031fb45, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0xb3f1fb47, SUC_lexicon_tags_27 },
    { 0x25b1fb47, SUC_lexicon_tags_61 },
    { 0x1431fb47, SUC_lexicon_tags_19 },
    { 0x6dd1fb4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4851fb4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae51fb52, SUC_lexicon_tags_77 },
    { 0x2111fb52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6111fb56, SUC_lexicon_tags_17 },
    { 0xad1fb57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9af1fb59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d1fb61, SUC_lexicon_tags_0 },
    { 0x4f51fb62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab71fb68, SUC_lexicon_tags_66 },
    { 0x8111fb68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3511fb6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b1fb73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x41f1fb75, SUC_lexicon_tags_25 },
    { 0xf0d1fb75, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b1fb79, SUC_lexicon_tags_56 },
    { 0xdf71fb7a, SUC_lexicon_tags_17 },
    { 0x3e91fb7b, SUC_lexicon_tags_23 },
    { 0xbd1fb7c, SUC_lexicon_tags_25 },
    { 0xe291fb7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf51fb81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc371fb83, SUC_lexicon_tags_43 },
    { 0x48b1fb83, SUC_lexicon_tags_16 },
    { 0x9bd1fb85, SUC_lexicon_tags_3 },
    { 0xd291fb86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf1fb8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd11fb8d, SUC_lexicon_tags_27 },
    { 0x70f1fb8d, SUC_lexicon_tags_64 },
    { 0x3fb1fb8f, SUC_lexicon_tags_72 },
    { 0x2b31fb8f, SUC_lexicon_tags_65 },
    { 0xfad1fb8e, SUC_lexicon_tags_60 },
    { 0x19d1fb8d, SUC_lexicon_tags_30 },
    { 0x4811fb93, SUC_lexicon_tags_36 },
    { 0xb4b1fb93, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2571fb98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe71fb9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5111fb9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3991fb9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9951fba1, SUC_lexicon_tags_1 },
    { 0x9571fba2, SUC_lexicon_tags_17 },
    { 0x6c31fba1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa031fba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28f1fba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6911fbaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c31fbae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca91fbb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe191fbb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21d1fbb7, SUC_lexicon_tags_17 },
    { 0x7f71fbb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48d1fbba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb51fbbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9591fbc1, SUC_lexicon_tags_32 },
    { 0x5651fbc2, SUC_lexicon_tags_25 },
    { 0x1971fbc3, SUC_lexicon_tags_42 },
    { 0x7051fbc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8651fbc8, SUC_lexicon_tags_1 },
    { 0xaa71fbc9, SUC_lexicon_tags_56 },
    { 0x6cf1fbca, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab1fbcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d1fbd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe311fbd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed1fbe3, SUC_lexicon_tags_1 },
    { 0x7a51fbe3, SUC_lexicon_tags_17 },
    { 0xbab1fbe5, SUC_lexicon_tags_216 },
    { 0xad1fbe6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f71fbe9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b1fbec, SUC_lexicon_tags_25 },
    { 0xb251fbec, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xebf1fbef, SUC_lexicon_tags_61 },
    { 0x2731fbef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1fbf4, SUC_lexicon_tags_2 },
    { 0x3cd1fbf4, SUC_lexicon_tags_4 },
    { 0x26f1fbf6, SUC_lexicon_tags_31 },
    { 0xda51fbf5, SUC_lexicon_tags_9 },
    { 0xad71fbf8, SUC_lexicon_tags_63 },
    { 0xa9b1fbf8, SUC_lexicon_tags_19 },
    { 0x63d1fbfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d71fbfe, SUC_lexicon_tags_18 },
    { 0x83f1fbff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1fc03, SUC_lexicon_tags_26 },
    { 0x7131fc03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb1fc07, SUC_lexicon_tags_11 },
    { 0xd6f1fc07, SUC_lexicon_tags_15 },
    { 0xe271fc08, SUC_lexicon_tags_27 },
    { 0x1731fc07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2231fc17, SUC_lexicon_tags_1664 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe711fc1b, SUC_lexicon_tags_37 },
    { 0x7df1fc1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c31fc25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1571fc27, SUC_lexicon_tags_14 },
    { 0xa191fc28, SUC_lexicon_tags_19 },
    { 0x9f91fc28, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb71fc30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd1fc34, SUC_lexicon_tags_40 },
    { 0x40b1fc34, SUC_lexicon_tags_45 },
    { 0xceb1fc35, SUC_lexicon_tags_15 },
    { 0x1f1fc37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bd1fc39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f11fc3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3551fc3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2b1fc40, SUC_lexicon_tags_12 },
    { 0x5831fc41, SUC_lexicon_tags_11 },
    { 0x1011fc42, SUC_lexicon_tags_84 },
    { 0xd671fc43, SUC_lexicon_tags_15 },
    { 0x7611fc41, SUC_lexicon_tags_43 },
    { 0xa1f1fc43, SUC_lexicon_tags_25 },
    { 0xe1b1fc46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b1fc49, SUC_lexicon_tags_43 },
    { 0x4c91fc49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b1fc4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef1fc51, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3b71fc53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa971fc55, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeb1fc57, SUC_lexicon_tags_25 },
    { 0x7691fc58, SUC_lexicon_tags_30 },
    { 0x411fc59, SUC_lexicon_tags_9 },
    { 0xfc11fc59, SUC_lexicon_tags_7 },
    { 0x47f1fc5b, SUC_lexicon_tags_7 },
    { 0x6a11fc5c, SUC_lexicon_tags_12 },
    { 0x5071fc5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab1fc62, SUC_lexicon_tags_14 },
    { 0xb1fc63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab1fc68, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6171fc6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2591fc6e, SUC_lexicon_tags_8 },
    { 0x4a11fc6f, SUC_lexicon_tags_13 },
    { 0x7e51fc70, SUC_lexicon_tags_31 },
    { 0xa9d1fc71, SUC_lexicon_tags_27 },
    { 0x5c11fc71, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f1fc75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45b1fc77, SUC_lexicon_tags_49 },
    { 0xefb1fc77, SUC_lexicon_tags_11 },
    { 0xc031fc77, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3911fc7b, SUC_lexicon_tags_25 },
    { 0xec71fc7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa11fc80, SUC_lexicon_tags_15 },
    { 0x3231fc81, SUC_lexicon_tags_11 },
    { 0x2f1fc80, SUC_lexicon_tags_28 },
    { 0x5eb1fc80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f31fc87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d51fc8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1d1fc8c, SUC_lexicon_tags_31 },
    { 0x63b1fc8c, SUC_lexicon_tags_1 },
    { 0xf7b1fc8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4711fc92, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb1fc95, SUC_lexicon_tags_12 },
    { 0x1331fc96, SUC_lexicon_tags_39 },
    { 0xd551fc95, SUC_lexicon_tags_14 },
    { 0x2bb1fc95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa331fca6, SUC_lexicon_tags_46 },
    { 0x4071fca7, SUC_lexicon_tags_38 },
    { 0xebb1fca8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xac71fcaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2b1fcac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce11fcb1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x58b1fcb3, SUC_lexicon_tags_26 },
    { 0x9911fcb4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc711fcb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b1fcba, SUC_lexicon_tags_11 },
    { 0xf251fcbb, SUC_lexicon_tags_17 },
    { 0x59f1fcbc, SUC_lexicon_tags_26 },
    { 0x30d1fcbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbf1fcbf, SUC_lexicon_tags_37 },
    { 0xe6f1fcbf, SUC_lexicon_tags_31 },
    { 0x53f1fcc1, SUC_lexicon_tags_27 },
    { 0xfff1fcbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3211fcc5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1931fcc8, SUC_lexicon_tags_37 },
    { 0x2931fcc9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b1fcce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4d1fcd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f1fcd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb391fcd5, SUC_lexicon_tags_10 },
    { 0xd231fcd5, SUC_lexicon_tags_62 },
    { 0xe9f1fcd7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd331fcdc, SUC_lexicon_tags_45 },
    { 0x9971fcdc, SUC_lexicon_tags_68 },
    { 0x9731fcde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5931fce1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xabb1fce3, SUC_lexicon_tags_37 },
    { 0xdc71fce4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1051fce6, SUC_lexicon_tags_17 },
    { 0xfdf1fce6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f1fcec, SUC_lexicon_tags_13 },
    { 0x1e51fced, SUC_lexicon_tags_11 },
    { 0x3cf1fcee, SUC_lexicon_tags_12 },
    { 0x8611fcee, SUC_lexicon_tags_46 },
    { 0xf6f1fcf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cb1fcf2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x751fcf4, SUC_lexicon_tags_15 },
    { 0x1591fcf4, SUC_lexicon_tags_11 },
    { 0x3bf1fcf6, SUC_lexicon_tags_1 },
    { 0xb011fcf6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe031fcfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b1fcfd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4f1fcff, SUC_lexicon_tags_49 },
    { 0x3611fcff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d51fd02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd31fd04, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf851fd07, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb451fd09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d1fd0c, SUC_lexicon_tags_13 },
    { 0x3691fd0d, SUC_lexicon_tags_79 },
    { 0xeb31fd0e, SUC_lexicon_tags_9 },
    { 0x2331fd0d, SUC_lexicon_tags_38 },
    { 0x32f1fd10, SUC_lexicon_tags_12 },
    { 0x6011fd0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1831fd13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2391fd1b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4171fd20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb71fd23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd1fd27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f1fd2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe771fd34, SUC_lexicon_tags_6 },
    { 0x7431fd34, SUC_lexicon_tags_15 },
    { 0xfd31fd36, SUC_lexicon_tags_0 },
    { 0x7351fd37, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7691fd39, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d1fd3d, SUC_lexicon_tags_19 },
    { 0xbbd1fd3e, SUC_lexicon_tags_0 },
    { 0x1791fd3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9751fd44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb351fd47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6931fd50, SUC_lexicon_tags_21 },
    { 0xe071fd50, SUC_lexicon_tags_11 },
    { 0x9011fd52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b1fd58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8331fd5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5611fd5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d1fd62, SUC_lexicon_tags_1 },
    { 0xdf51fd62, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdff1fd65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f1fd68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a31fd6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde11fd6f, SUC_lexicon_tags_19 },
    { 0xe351fd70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f31fd73, SUC_lexicon_tags_17 },
    { 0xc0d1fd73, SUC_lexicon_tags_37 },
    { 0xe851fd75, SUC_lexicon_tags_38 },
    { 0x2611fd76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xead1fd79, SUC_lexicon_tags_43 },
    { 0x8c51fd79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd031fd7d, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0x2231fd80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d1fd83, SUC_lexicon_tags_45 },
    { 0x791fd84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251fd8b, SUC_lexicon_tags_0 },
    { 0xa191fd8c, SUC_lexicon_tags_62 },
    { 0xfdd1fd8d, SUC_lexicon_tags_126 },
    { 0x39b1fd8c, SUC_lexicon_tags_40 },
    { 0xfbb1fd8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2f1fd91, SUC_lexicon_tags_37 },
    { 0x3691fd92, SUC_lexicon_tags_37 },
    { 0x27d1fd91, SUC_lexicon_tags_15 },
    { 0xe611fd94, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff1fd99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4271fd9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7511fda4, SUC_lexicon_tags_11 },
    { 0xe8b1fda5, SUC_lexicon_tags_25 },
    { 0x55d1fda6, SUC_lexicon_tags_17 },
    { 0x4031fda7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb4b1fda9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7cd1fdab, SUC_lexicon_tags_29 },
    { 0xad91fdac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1771fdae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed11fdb2, SUC_lexicon_tags_38 },
    { 0xaeb1fdb3, SUC_lexicon_tags_28 },
    { 0x92d1fdb3, SUC_lexicon_tags_45 },
    { 0x911fdb3, SUC_lexicon_tags_2 },
    { 0x1711fdb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f1fdbc, SUC_lexicon_tags_0 },
    { 0x4a11fdbd, SUC_lexicon_tags_29 },
    { 0x4fb1fdbe, SUC_lexicon_tags_15 },
    { 0x60d1fdbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2911fdc1, SUC_lexicon_tags_1 },
    { 0x8ab1fdc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f1fdc7, SUC_lexicon_tags_26 },
    { 0xb1d1fdc8, SUC_lexicon_tags_63 },
    { 0x1b1fdc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b1fdcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3571fdd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa351fdd5, SUC_lexicon_tags_15 },
    { 0xabd1fdd6, SUC_lexicon_tags_90 },
    { 0x67d1fdd7, SUC_lexicon_tags_33 },
    { 0x95b1fdd7, SUC_lexicon_tags_12 },
    { 0x56f1fdd6, SUC_lexicon_tags_80 },
    { 0xe651fdda, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x26f1fddc, SUC_lexicon_tags_12 },
    { 0x5771fddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf11fddf, SUC_lexicon_tags_38 },
    { 0x2b11fde0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8b1fde2, SUC_lexicon_tags_33 },
    { 0xf351fde3, SUC_lexicon_tags_25 },
    { 0x3c71fde3, SUC_lexicon_tags_907 },
    { 0x5d91fde5, SUC_lexicon_tags_38 },
    { 0xa8b1fde4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4b1fde8, SUC_lexicon_tags_17 },
    { 0x30b1fde9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa311fdec, SUC_lexicon_tags_1 },
    { 0x7a91fded, SUC_lexicon_tags_37 },
    { 0xe471fdee, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xc671fdf0, SUC_lexicon_tags_37 },
    { 0x6971fdf0, SUC_lexicon_tags_8 },
    { 0xa251fdf2, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e31fdf6, SUC_lexicon_tags_25 },
    { 0x23f1fdf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4351fdfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e71fe00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf1fe03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa51fe06, SUC_lexicon_tags_7 },
    { 0xbbb1fe06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x251fe0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5311fe0c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdb31fe0e, SUC_lexicon_tags_49 },
    { 0x17f1fe0f, SUC_lexicon_tags_77 },
    { 0x5cd1fe0e, SUC_lexicon_tags_17 },
    { 0xf271fe11, SUC_lexicon_tags_27 },
    { 0xb191fe12, SUC_lexicon_tags_57 },
    { 0xa1d1fe13, SUC_lexicon_tags_8 },
    { 0xe491fe14, SUC_lexicon_tags_15 },
    { 0xf91fe15, SUC_lexicon_tags_21 },
    { 0x7ad1fe16, SUC_lexicon_tags_25 },
    { 0x7091fe12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x97d1fe19, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef1fe1c, SUC_lexicon_tags_63 },
    { 0xd0d1fe1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67d1fe1f, SUC_lexicon_tags_9 },
    { 0xf31fe20, SUC_lexicon_tags_25 },
    { 0xbb1fe1f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf1fe25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b71fe2b, SUC_lexicon_tags_12 },
    { 0x8391fe2c, SUC_lexicon_tags_11 },
    { 0x4071fe2d, SUC_lexicon_tags_23 },
    { 0xf31fe2d, SUC_lexicon_tags_11 },
    { 0x5df1fe2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8991fe32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf731fe35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b1fe3c, SUC_lexicon_tags_13 },
    { 0xacb1fe3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6671fe3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed1fe43, SUC_lexicon_tags_14 },
    { 0xa3b1fe44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2551fe4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f1fe50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d1fe54, SUC_lexicon_tags_12 },
    { 0xa191fe55, SUC_lexicon_tags_18 },
    { 0xae91fe56, SUC_lexicon_tags_27 },
    { 0x1e91fe57, SUC_lexicon_tags_37 },
    { 0x83b1fe58, SUC_lexicon_tags_27 },
    { 0xae91fe58, SUC_lexicon_tags_28 },
    { 0x11fe5a, SUC_lexicon_tags_25 },
    { 0x36d1fe5a, SUC_lexicon_tags_17 },
    { 0xc791fe5b, SUC_lexicon_tags_17 },
    { 0x6091fe5d, SUC_lexicon_tags_37 },
    { 0x55f1fe5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8431fe61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f1fe64, SUC_lexicon_tags_19 },
    { 0x1651fe65, SUC_lexicon_tags_2 },
    { 0xd5f1fe65, SUC_lexicon_tags_8 },
    { 0x6fb1fe64, SUC_lexicon_tags_55 },
    { 0x8d91fe68, SUC_lexicon_tags_34 },
    { 0x4411fe69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5291fe6b, SUC_lexicon_tags_77 },
    { 0x9cd1fe6c, SUC_lexicon_tags_65 },
    { 0x9471fe6b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7ab1fe6f, SUC_lexicon_tags_23 },
    { 0x6db1fe6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe431fe72, SUC_lexicon_tags_8 },
    { 0x3411fe73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2031fe78, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a71fe7b, SUC_lexicon_tags_16 },
    { 0x7d71fe7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f11fe81, SUC_lexicon_tags_25 },
    { 0x79d1fe82, SUC_lexicon_tags_47 },
    { 0xf111fe82, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x9c71fe85, SUC_lexicon_tags_17 },
    { 0xcf51fe86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b1fe8a, SUC_lexicon_tags_23 },
    { 0xe151fe8a, SUC_lexicon_tags_17 },
    { 0x5251fe8a, SUC_lexicon_tags_7 },
    { 0x1131fe8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fd1fe8f, SUC_lexicon_tags_33 },
    { 0x1ef1fe90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6d1fe92, SUC_lexicon_tags_9 },
    { 0x6511fe93, SUC_lexicon_tags_12 },
    { 0x2211fe94, SUC_lexicon_tags_33 },
    { 0x9031fe95, SUC_lexicon_tags_37 },
    { 0xb2d1fe96, SUC_lexicon_tags_14 },
    { 0x7f31fe94, SUC_lexicon_tags_63 },
    { 0x3891fe98, SUC_lexicon_tags_79 },
    { 0x20f1fe95, SUC_lexicon_tags_19 },
    { 0x27d1fe9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd951fe9c, SUC_lexicon_tags_7 },
    { 0xfc31fe9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f1fea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4411feab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1111feaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed1feb3, SUC_lexicon_tags_33 },
    { 0xc311feb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd611feb7, SUC_lexicon_tags_12 },
    { 0x9b31feb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc471fec5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8431fec9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf771fecc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f1fed0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf751fed2, SUC_lexicon_tags_1 },
    { 0x2d11fed2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b1fed6, SUC_lexicon_tags_22 },
    { 0x5471fed7, SUC_lexicon_tags_37 },
    { 0xf491fed6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d31fedd, SUC_lexicon_tags_44 },
    { 0x8f51fede, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7111fee4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db1fef0, SUC_lexicon_tags_17 },
    { 0x7d11fef1, SUC_lexicon_tags_12 },
    { 0x1d31fef2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa131fef5, SUC_lexicon_tags_9 },
    { 0xb191fef6, SUC_lexicon_tags_47 },
    { 0xb991fef6, SUC_lexicon_tags_47 },
    { 0xa3b1fef7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b1fefc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f71fefe, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x3031ff00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b1ff05, SUC_lexicon_tags_37 },
    { 0x7071ff06, SUC_lexicon_tags_38 },
    { 0x47d1ff06, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6851ff0c, SUC_lexicon_tags_136 },
    { 0xf4f1ff0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b1ff10, SUC_lexicon_tags_37 },
    { 0xc1d1ff11, SUC_lexicon_tags_15 },
    { 0x9411ff12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda11ff18, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c11ff1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5211ff1f, SUC_lexicon_tags_15 },
    { 0x9f11ff20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf1ff24, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7891ff28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdfb1ff2a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x57f1ff2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c91ff30, SUC_lexicon_tags_68 },
    { 0x4f1ff31, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a31ff34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf111ff3a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2c11ff3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2811ff3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7591ff42, SUC_lexicon_tags_15 },
    { 0x4b91ff43, SUC_lexicon_tags_15 },
    { 0x18f1ff44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcd1ff46, SUC_lexicon_tags_65 },
    { 0x1031ff47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e71ff4e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7611ff55, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x32d1ff57, SUC_lexicon_tags_15 },
    { 0x36b1ff58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebd1ff5a, SUC_lexicon_tags_7 },
    { 0x4971ff5a, SUC_lexicon_tags_37 },
    { 0x46f1ff5b, SUC_lexicon_tags_27 },
    { 0x67d1ff5c, SUC_lexicon_tags_12 },
    { 0x9391ff5e, SUC_lexicon_tags_25 },
    { 0xc591ff5f, SUC_lexicon_tags_8 },
    { 0xb4d1ff5d, SUC_lexicon_tags_7 },
    { 0xa611ff5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76b1ff63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6d1ff65, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf431ff68, SUC_lexicon_tags_1633 },
    { 0x9dd1ff69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd371ff6b, SUC_lexicon_tags_17 },
    { 0xc031ff6c, SUC_lexicon_tags_32 },
    { 0xbb71ff6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f51ff70, SUC_lexicon_tags_17 },
    { 0x5b11ff71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd71ff78, SUC_lexicon_tags_12 },
    { 0x3a51ff79, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd611ff7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b1ff81, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5571ff84, SUC_lexicon_tags_37 },
    { 0x5b31ff85, SUC_lexicon_tags_16 },
    { 0xe9d1ff85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d91ff91, SUC_lexicon_tags_32 },
    { 0xfef1ff92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5331ff98, SUC_lexicon_tags_80 },
    { 0x4411ff99, SUC_lexicon_tags_0 },
    { 0x7d71ff99, SUC_lexicon_tags_7 },
    { 0x8991ff9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d1ffa1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5591ffa6, SUC_lexicon_tags_37 },
    { 0xfe71ffa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdbf1ffa9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b1ffac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d1ffaf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc11ffb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f1ffb5, SUC_lexicon_tags_17 },
    { 0x2291ffb6, SUC_lexicon_tags_15 },
    { 0x1931ffb6, SUC_lexicon_tags_15 },
    { 0x5e11ffb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b1ffbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf1ffbe, SUC_lexicon_tags_1 },
    { 0xc6f1ffbf, SUC_lexicon_tags_57 },
    { 0xeb91ffc0, SUC_lexicon_tags_1 },
    { 0x2631ffbe, SUC_lexicon_tags_11 },
    { 0xd831ffbf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc731ffc6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b1ffca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2491ffce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8451ffd1, SUC_lexicon_tags_9 },
    { 0x8551ffd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6431ffd4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9191ffd6, SUC_lexicon_tags_18 },
    { 0x4791ffd6, SUC_lexicon_tags_15 },
    { 0x6171ffd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4871ffdc, SUC_lexicon_tags_8 },
    { 0xe8b1ffdd, SUC_lexicon_tags_46 },
    { 0x6231ffdd, SUC_lexicon_tags_30 },
    { 0x62f1ffdf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5bb1ffe1, SUC_lexicon_tags_37 },
    { 0x1c71ffe2, SUC_lexicon_tags_43 },
    { 0x2eb1ffe3, SUC_lexicon_tags_68 },
    { 0xb131ffe1, SUC_lexicon_tags_12 },
    { 0xfc31ffe5, SUC_lexicon_tags_38 },
    { 0x78d1ffe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb1ffe9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x351ffeb, SUC_lexicon_tags_17 },
    { 0x29b1ffec, SUC_lexicon_tags_39 },
    { 0xe331ffeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7771ffef, SUC_lexicon_tags_17 },
    { 0x3dd1ffef, SUC_lexicon_tags_25 },
    { 0xbbd1fff1, SUC_lexicon_tags_7 },
    { 0x5711fff0, SUC_lexicon_tags_15 },
    { 0x9d31fff0, SUC_lexicon_tags_23 },
    { 0x6431fff4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b1fff7, SUC_lexicon_tags_11 },
    { 0x39d1fff7, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b1fffb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c11fffd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7720000, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x41520003, SUC_lexicon_tags_0 },
    { 0x1cd20004, SUC_lexicon_tags_68 },
    { 0x7fb20005, SUC_lexicon_tags_47 },
    { 0xdc120006, SUC_lexicon_tags_38 },
    { 0x82720003, SUC_lexicon_tags_1 },
    { 0xb2b20008, SUC_lexicon_tags_6 },
    { 0xc3120004, SUC_lexicon_tags_11 },
    { 0x2412000a, SUC_lexicon_tags_47 },
    { 0x352000b, SUC_lexicon_tags_12 },
    { 0x2472000c, SUC_lexicon_tags_68 },
    { 0xdc52000b, SUC_lexicon_tags_10 },
    { 0xab12000e, SUC_lexicon_tags_17 },
    { 0x16f2000f, SUC_lexicon_tags_17 },
    { 0x4192000e, SUC_lexicon_tags_19 },
    { 0xd892000f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb20014, SUC_lexicon_tags_25 },
    { 0xef20015, SUC_lexicon_tags_1 },
    { 0x46d20014, SUC_lexicon_tags_11 },
    { 0x1a120017, SUC_lexicon_tags_11 },
    { 0x57b20018, SUC_lexicon_tags_12 },
    { 0xab920018, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb2001c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9672001f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb20027, SUC_lexicon_tags_25 },
    { 0xffd20028, SUC_lexicon_tags_63 },
    { 0xfdd20029, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f2002d, SUC_lexicon_tags_19 },
    { 0x9c52002d, SUC_lexicon_tags_37 },
    { 0xd212002d, SUC_lexicon_tags_63 },
    { 0x87120030, SUC_lexicon_tags_7 },
    { 0xd6320031, SUC_lexicon_tags_37 },
    { 0xc5320032, SUC_lexicon_tags_26 },
    { 0xc8f20033, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b20036, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d20040, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f20045, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db2004c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2320051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97920053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d20058, SUC_lexicon_tags_30 },
    { 0xbfd20059, SUC_lexicon_tags_11 },
    { 0x2b2005a, SUC_lexicon_tags_49 },
    { 0xb372005b, SUC_lexicon_tags_14 },
    { 0xd872005c, SUC_lexicon_tags_33 },
    { 0x6692005d, SUC_lexicon_tags_9 },
    { 0x37d2005b, SUC_lexicon_tags_17 },
    { 0x1e32005f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48320061, SUC_lexicon_tags_31 },
    { 0x1dd20062, SUC_lexicon_tags_43 },
    { 0x35520063, SUC_lexicon_tags_38 },
    { 0x21720064, SUC_lexicon_tags_26 },
    { 0x69120064, SUC_lexicon_tags_15 },
    { 0x30920064, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b20069, SUC_lexicon_tags_1 },
    { 0x48f2006a, SUC_lexicon_tags_49 },
    { 0x80b2006a, SUC_lexicon_tags_1 },
    { 0xe9b2006a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf632006f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce320072, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c120079, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda72007b, SUC_lexicon_tags_16 },
    { 0x8c12007c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6a32007e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65720083, SUC_lexicon_tags_37 },
    { 0x95b20084, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1c320086, SUC_lexicon_tags_28 },
    { 0x7d20086, SUC_lexicon_tags_60 },
    { 0xe7120087, SUC_lexicon_tags_7 },
    { 0xff720089, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5492008b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8112008e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f20095, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d320097, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f2009b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1d2009d, SUC_lexicon_tags_57 },
    { 0x84f2009d, SUC_lexicon_tags_37 },
    { 0x8f2009e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9200a6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53200b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x293200b5, SUC_lexicon_tags_23 },
    { 0xaf5200b6, SUC_lexicon_tags_21 },
    { 0x6c7200b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x825200b9, SUC_lexicon_tags_19 },
    { 0xab5200b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21200be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f200c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf200c3, SUC_lexicon_tags_230 },
    { 0x4a7200c4, SUC_lexicon_tags_7 },
    { 0x11f200c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275200cc, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b200d1, SUC_lexicon_tags_25 },
    { 0xd79200d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99200d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99200d9, SUC_lexicon_tags_7 },
    { 0xa3200d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf1f200dc, SUC_lexicon_tags_1 },
    { 0x853200dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95200e2, SUC_lexicon_tags_37 },
    { 0xc9d200e2, SUC_lexicon_tags_11 },
    { 0x991200e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1f200e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9200eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7200ee, SUC_lexicon_tags_60 },
    { 0xf37200ef, SUC_lexicon_tags_45 },
    { 0x3f1200f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239200f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60b200f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfd200f8, SUC_lexicon_tags_17 },
    { 0x217200f9, SUC_lexicon_tags_1 },
    { 0xd7f200f8, SUC_lexicon_tags_476 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519200ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a920103, SUC_lexicon_tags_68 },
    { 0x86520104, SUC_lexicon_tags_33 },
    { 0x90b20105, SUC_lexicon_tags_106 },
    { 0x45120106, SUC_lexicon_tags_1 },
    { 0xc9120107, SUC_lexicon_tags_11 },
    { 0x32b20108, SUC_lexicon_tags_37 },
    { 0xf1120109, SUC_lexicon_tags_14 },
    { 0xd652010a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bd2010c, SUC_lexicon_tags_12 },
    { 0xc1d2010d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1120112, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe6520114, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4120116, SUC_lexicon_tags_11 },
    { 0x95920117, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe7920119, SUC_lexicon_tags_17 },
    { 0x93b2011a, SUC_lexicon_tags_1 },
    { 0xf12011a, SUC_lexicon_tags_2 },
    { 0xd0b2011c, SUC_lexicon_tags_15 },
    { 0xb8d2011d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3072011f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6d320121, SUC_lexicon_tags_12 },
    { 0xedd20122, SUC_lexicon_tags_11 },
    { 0xa4720122, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5320125, SUC_lexicon_tags_17 },
    { 0x4f120126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5572012a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad120130, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a720134, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb2013d, SUC_lexicon_tags_27 },
    { 0x2c92013e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5720144, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3572014a, SUC_lexicon_tags_12 },
    { 0x6a72014b, SUC_lexicon_tags_37 },
    { 0x7792014c, SUC_lexicon_tags_57 },
    { 0x3372014d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa672014f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30320151, SUC_lexicon_tags_7 },
    { 0x6f520151, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2720155, SUC_lexicon_tags_7 },
    { 0x3b20156, SUC_lexicon_tags_29 },
    { 0x9c520156, SUC_lexicon_tags_22 },
    { 0x3bf20156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1252015a, SUC_lexicon_tags_47 },
    { 0xafb2015b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2912015e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa920161, SUC_lexicon_tags_38 },
    { 0x7c520162, SUC_lexicon_tags_8 },
    { 0x93520163, SUC_lexicon_tags_25 },
    { 0x23520162, SUC_lexicon_tags_56 },
    { 0x70d20165, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9c920167, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9212016d, SUC_lexicon_tags_7 },
    { 0x9172016e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2a120170, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae120176, SUC_lexicon_tags_12 },
    { 0x57b20177, SUC_lexicon_tags_1 },
    { 0x73320176, SUC_lexicon_tags_126 },
    { 0x3a120179, SUC_lexicon_tags_26 },
    { 0x9992017a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd520182, SUC_lexicon_tags_27 },
    { 0x2ef20182, SUC_lexicon_tags_25 },
    { 0x3fb20184, SUC_lexicon_tags_17 },
    { 0xcd520185, SUC_lexicon_tags_55 },
    { 0xf9920185, SUC_lexicon_tags_68 },
    { 0xcb20185, SUC_lexicon_tags_15 },
    { 0x28920186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb672018b, SUC_lexicon_tags_33 },
    { 0x4872018c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa932018e, SUC_lexicon_tags_31 },
    { 0xbc72018f, SUC_lexicon_tags_6 },
    { 0x2d92018f, SUC_lexicon_tags_48 },
    { 0xc3d2018f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a520195, SUC_lexicon_tags_8 },
    { 0xbbf20195, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe6720198, SUC_lexicon_tags_31 },
    { 0x39720198, SUC_lexicon_tags_39 },
    { 0x8bf20198, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb201a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc97201a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad201a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843201ad, SUC_lexicon_tags_19 },
    { 0x67201ae, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff201b2, SUC_lexicon_tags_11 },
    { 0xbe9201b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d201b8, SUC_lexicon_tags_37 },
    { 0x9fd201b9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b201be, SUC_lexicon_tags_54 },
    { 0x543201bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x889201c1, SUC_lexicon_tags_12 },
    { 0x1a7201c1, SUC_lexicon_tags_23 },
    { 0x3dd201c3, SUC_lexicon_tags_39 },
    { 0xb5d201c1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5dd201c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d5201c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7201cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1201d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x99d201d3, SUC_lexicon_tags_25 },
    { 0x4b9201d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817201de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f201e1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x26b201e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a5201e5, SUC_lexicon_tags_45 },
    { 0xb23201e5, SUC_lexicon_tags_10 },
    { 0xbc9201e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f201f0, SUC_lexicon_tags_2 },
    { 0x247201f0, SUC_lexicon_tags_15 },
    { 0x417201f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d201f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x641201f7, SUC_lexicon_tags_2 },
    { 0x17f201f8, SUC_lexicon_tags_2 },
    { 0xa09201f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91201fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdaf201fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7120200, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5320206, SUC_lexicon_tags_7 },
    { 0xb7520207, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xb392020a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c120214, SUC_lexicon_tags_12 },
    { 0x5fd20215, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1bd20217, SUC_lexicon_tags_7 },
    { 0xabf20218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5532021b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2892021f, SUC_lexicon_tags_29 },
    { 0x46f2021f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe3f20222, SUC_lexicon_tags_19 },
    { 0xc9320223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f920227, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa320229, SUC_lexicon_tags_376 },
    { 0xd6d20229, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaaf2022c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8ef2022e, SUC_lexicon_tags_26 },
    { 0x84d2022f, SUC_lexicon_tags_32 },
    { 0xe5b20230, SUC_lexicon_tags_12 },
    { 0xd072022f, SUC_lexicon_tags_17 },
    { 0xc2320231, SUC_lexicon_tags_15 },
    { 0xab20232, SUC_lexicon_tags_7 },
    { 0xe0120234, SUC_lexicon_tags_8 },
    { 0x42f20235, SUC_lexicon_tags_37 },
    { 0xf4d20233, SUC_lexicon_tags_45 },
    { 0x44d20231, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe312023b, SUC_lexicon_tags_37 },
    { 0x2012023c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d2023f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95120246, SUC_lexicon_tags_1 },
    { 0x45320247, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c52024f, SUC_lexicon_tags_15 },
    { 0xa0d20250, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76920252, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x65520255, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d2025b, SUC_lexicon_tags_13 },
    { 0x9bb2025c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41520260, SUC_lexicon_tags_11 },
    { 0x6a920261, SUC_lexicon_tags_33 },
    { 0xa6f20262, SUC_lexicon_tags_2 },
    { 0xb2b20263, SUC_lexicon_tags_15 },
    { 0x16120263, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d520267, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd312026a, SUC_lexicon_tags_49 },
    { 0xd472026a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1920271, SUC_lexicon_tags_15 },
    { 0xdf520271, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b20276, SUC_lexicon_tags_43 },
    { 0xe0320277, SUC_lexicon_tags_35 },
    { 0xd5720278, SUC_lexicon_tags_40 },
    { 0x39120279, SUC_lexicon_tags_112 },
    { 0x90120277, SUC_lexicon_tags_2 },
    { 0xa1920278, SUC_lexicon_tags_17 },
    { 0x49d2027a, SUC_lexicon_tags_12 },
    { 0x4e12027a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f20280, SUC_lexicon_tags_7 },
    { 0x9520281, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8e120283, SUC_lexicon_tags_2 },
    { 0x96d20284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79320288, SUC_lexicon_tags_37 },
    { 0x24f20289, SUC_lexicon_tags_60 },
    { 0x5f52028a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8912028f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7552029a, SUC_lexicon_tags_33 },
    { 0x63b2029b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6332029d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f202a3, SUC_lexicon_tags_37 },
    { 0xd81202a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x885202a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf202a9, SUC_lexicon_tags_15 },
    { 0x603202aa, SUC_lexicon_tags_6 },
    { 0xcaf202ab, SUC_lexicon_tags_6 },
    { 0x771202ac, SUC_lexicon_tags_7 },
    { 0xd95202ad, SUC_lexicon_tags_1 },
    { 0x1b9202ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17d202b0, SUC_lexicon_tags_45 },
    { 0xf87202b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda1202b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31202ba, SUC_lexicon_tags_15 },
    { 0x4dd202ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f202c1, SUC_lexicon_tags_2 },
    { 0xae9202c2, SUC_lexicon_tags_15 },
    { 0x2e5202c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c5202c5, SUC_lexicon_tags_37 },
    { 0xc59202c6, SUC_lexicon_tags_2 },
    { 0x8a3202c7, SUC_lexicon_tags_15 },
    { 0x8d9202c8, SUC_lexicon_tags_11 },
    { 0xf75202c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b202cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f202d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d202de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7f202e0, SUC_lexicon_tags_40 },
    { 0xe59202e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc3202e3, SUC_lexicon_tags_11 },
    { 0xd9b202e4, SUC_lexicon_tags_6 },
    { 0x825202e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475202eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf202ee, SUC_lexicon_tags_12 },
    { 0x157202ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83202f6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded202fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75202ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x29520301, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x55120303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf120307, SUC_lexicon_tags_14 },
    { 0x9d120307, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa772030b, SUC_lexicon_tags_38 },
    { 0x19d2030c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xad12030f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1320311, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78120314, SUC_lexicon_tags_14 },
    { 0xf7f20315, SUC_lexicon_tags_28 },
    { 0xb4120314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfff20318, SUC_lexicon_tags_15 },
    { 0xddb20319, SUC_lexicon_tags_17 },
    { 0xcc52031a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x532031c, SUC_lexicon_tags_7 },
    { 0x9f32031d, SUC_lexicon_tags_14 },
    { 0x1f72031c, SUC_lexicon_tags_14 },
    { 0x22b2031e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6920322, SUC_lexicon_tags_15 },
    { 0xb5120323, SUC_lexicon_tags_38 },
    { 0xcbf20324, SUC_lexicon_tags_12 },
    { 0xf7520323, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5f20327, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x5db20329, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6292032b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba52032e, SUC_lexicon_tags_47 },
    { 0xc732032e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x97320331, SUC_lexicon_tags_0 },
    { 0xfe720332, SUC_lexicon_tags_37 },
    { 0x13d20331, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa472033a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec120341, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd320346, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b2034a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb652034c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f20351, SUC_lexicon_tags_23 },
    { 0xfb20352, SUC_lexicon_tags_12 },
    { 0x2b520353, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed120359, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa592035e, SUC_lexicon_tags_60 },
    { 0x5812035f, SUC_lexicon_tags_43 },
    { 0x57920360, SUC_lexicon_tags_2 },
    { 0xb8720360, SUC_lexicon_tags_7 },
    { 0x5c920361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf20366, SUC_lexicon_tags_112 },
    { 0x7520367, SUC_lexicon_tags_60 },
    { 0x39b20368, SUC_lexicon_tags_17 },
    { 0x68f20369, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f20374, SUC_lexicon_tags_33 },
    { 0xcc320375, SUC_lexicon_tags_17 },
    { 0x13b20376, SUC_lexicon_tags_87 },
    { 0x69120376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b92037a, SUC_lexicon_tags_17 },
    { 0xd472037b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3120380, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7520383, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2320385, SUC_lexicon_tags_19 },
    { 0xcd920385, SUC_lexicon_tags_29 },
    { 0x22d20387, SUC_lexicon_tags_37 },
    { 0x43920387, SUC_lexicon_tags_38 },
    { 0xe9b20386, SUC_lexicon_tags_17 },
    { 0x85f2038a, SUC_lexicon_tags_47 },
    { 0xaf520388, SUC_lexicon_tags_37 },
    { 0xc5f20389, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1af2038e, SUC_lexicon_tags_14 },
    { 0x6572038f, SUC_lexicon_tags_15 },
    { 0xa592038f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7120392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81b20394, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x63b20396, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6320399, SUC_lexicon_tags_8 },
    { 0x9d320399, SUC_lexicon_tags_10 },
    { 0x8ff2039b, SUC_lexicon_tags_49 },
    { 0xf412039a, SUC_lexicon_tags_15 },
    { 0xc692039d, SUC_lexicon_tags_25 },
    { 0x7732039e, SUC_lexicon_tags_37 },
    { 0x97b2039e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x887203a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329203a9, SUC_lexicon_tags_37 },
    { 0xe8b203aa, SUC_lexicon_tags_11 },
    { 0x5e1203ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf65203ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a5203af, SUC_lexicon_tags_12 },
    { 0x1b3203af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfc9203b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f203b6, SUC_lexicon_tags_11 },
    { 0x7f9203b7, SUC_lexicon_tags_17 },
    { 0x433203b8, SUC_lexicon_tags_27 },
    { 0xc8b203b9, SUC_lexicon_tags_11 },
    { 0xb11203ba, SUC_lexicon_tags_2 },
    { 0xcd203bb, SUC_lexicon_tags_1 },
    { 0x1b9203bc, SUC_lexicon_tags_47 },
    { 0x203203b9, SUC_lexicon_tags_57 },
    { 0xc1b203ba, SUC_lexicon_tags_33 },
    { 0x809203bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x909203c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297203c5, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9203ca, SUC_lexicon_tags_23 },
    { 0xe63203cb, SUC_lexicon_tags_12 },
    { 0x24f203ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x939203ce, SUC_lexicon_tags_61 },
    { 0x99d203ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3203d2, SUC_lexicon_tags_49 },
    { 0x5e7203d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x221203d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6df203d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b203db, SUC_lexicon_tags_17 },
    { 0x735203dc, SUC_lexicon_tags_7 },
    { 0x71b203db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x903203df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e7203e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137203e5, SUC_lexicon_tags_6 },
    { 0x485203e6, SUC_lexicon_tags_2 },
    { 0x603203e7, SUC_lexicon_tags_37 },
    { 0x109203e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55203ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27203fb, SUC_lexicon_tags_17 },
    { 0x417203fc, SUC_lexicon_tags_9 },
    { 0xfef203fc, SUC_lexicon_tags_4 },
    { 0x41b203fc, SUC_lexicon_tags_1 },
    { 0xde1203fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd20403, SUC_lexicon_tags_17 },
    { 0x63d20404, SUC_lexicon_tags_14 },
    { 0xfe520405, SUC_lexicon_tags_0 },
    { 0x19b20406, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x94520408, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e72040a, SUC_lexicon_tags_26 },
    { 0xc92040b, SUC_lexicon_tags_41 },
    { 0xbef2040c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xde72040f, SUC_lexicon_tags_29 },
    { 0x1c120410, SUC_lexicon_tags_8 },
    { 0x4b320410, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef920418, SUC_lexicon_tags_19 },
    { 0xdc320418, SUC_lexicon_tags_25 },
    { 0x4d32041a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf2041d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb20422, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6520426, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b72042d, SUC_lexicon_tags_27 },
    { 0xaab2042d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f120432, SUC_lexicon_tags_17 },
    { 0xe6520433, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea920435, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x8920438, SUC_lexicon_tags_7 },
    { 0xc9320439, SUC_lexicon_tags_26 },
    { 0xd9d2043a, SUC_lexicon_tags_25 },
    { 0x60f2043a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3412043e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9320442, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead20446, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a520449, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59d2044b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d2044e, SUC_lexicon_tags_60 },
    { 0x5372044f, SUC_lexicon_tags_25 },
    { 0x5792044e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x27b20452, SUC_lexicon_tags_37 },
    { 0x40d20452, SUC_lexicon_tags_17 },
    { 0xd5320454, SUC_lexicon_tags_0 },
    { 0xb2d20454, SUC_lexicon_tags_33 },
    { 0x1a920456, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6dd20458, SUC_lexicon_tags_647 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b12045c, SUC_lexicon_tags_7 },
    { 0x59d2045d, SUC_lexicon_tags_17 },
    { 0xb852045e, SUC_lexicon_tags_47 },
    { 0xb3f2045f, SUC_lexicon_tags_12 },
    { 0x2c120460, SUC_lexicon_tags_8 },
    { 0x35d20461, SUC_lexicon_tags_7 },
    { 0xd2f20462, SUC_lexicon_tags_17 },
    { 0x4b320463, SUC_lexicon_tags_7 },
    { 0x3b320462, SUC_lexicon_tags_17 },
    { 0xf9920463, SUC_lexicon_tags_15 },
    { 0xcbb20460, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcd20468, SUC_lexicon_tags_12 },
    { 0x54f20469, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b2046f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc920471, SUC_lexicon_tags_14 },
    { 0xf20472, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1320476, SUC_lexicon_tags_14 },
    { 0x2bb20476, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa992047a, SUC_lexicon_tags_81 },
    { 0x6ed2047b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f20481, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb920484, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f20487, SUC_lexicon_tags_26 },
    { 0x37120487, SUC_lexicon_tags_26 },
    { 0xd6720487, SUC_lexicon_tags_14 },
    { 0x4112048a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb292048d, SUC_lexicon_tags_11 },
    { 0x26d2048e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xced20491, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x28520493, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d2049a, SUC_lexicon_tags_11 },
    { 0xfe12049b, SUC_lexicon_tags_7 },
    { 0xaed2049a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc8b2049e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45204a1, SUC_lexicon_tags_27 },
    { 0xce1204a1, SUC_lexicon_tags_63 },
    { 0xff3204a3, SUC_lexicon_tags_82 },
    { 0xba9204a1, SUC_lexicon_tags_11 },
    { 0x5df204a3, SUC_lexicon_tags_17 },
    { 0xde9204a6, SUC_lexicon_tags_15 },
    { 0x355204a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9dd204a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc05204ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x387204ad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d204b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11204bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4b204bd, SUC_lexicon_tags_2 },
    { 0x8db204be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b3204c0, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43204c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7204c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71204ca, SUC_lexicon_tags_17 },
    { 0x747204cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b204d1, SUC_lexicon_tags_9 },
    { 0x87204d2, SUC_lexicon_tags_11 },
    { 0x943204d2, SUC_lexicon_tags_17 },
    { 0x6fd204d4, SUC_lexicon_tags_11 },
    { 0x3f9204d3, SUC_lexicon_tags_23 },
    { 0x31d204d6, SUC_lexicon_tags_17 },
    { 0x457204d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd204dc, SUC_lexicon_tags_19 },
    { 0x6b3204dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x501204df, SUC_lexicon_tags_17 },
    { 0x88d204e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23204e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53204e6, SUC_lexicon_tags_15 },
    { 0xa5f204e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85204eb, SUC_lexicon_tags_18 },
    { 0x485204eb, SUC_lexicon_tags_49 },
    { 0x585204ed, SUC_lexicon_tags_14 },
    { 0x65204ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47204f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47b204f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9d204f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01204fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5204ff, SUC_lexicon_tags_12 },
    { 0x3d720500, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d20503, SUC_lexicon_tags_11 },
    { 0xf6b20504, SUC_lexicon_tags_7 },
    { 0xdcb20503, SUC_lexicon_tags_30 },
    { 0x76520504, SUC_lexicon_tags_2 },
    { 0x1a720505, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f2050a, SUC_lexicon_tags_25 },
    { 0xcfb2050b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3e52050d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xca120510, SUC_lexicon_tags_2 },
    { 0xc0720511, SUC_lexicon_tags_51 },
    { 0x65b20512, SUC_lexicon_tags_25 },
    { 0x75d20513, SUC_lexicon_tags_46 },
    { 0xe4720513, SUC_lexicon_tags_30 },
    { 0x43f20515, SUC_lexicon_tags_11 },
    { 0xf3b20514, SUC_lexicon_tags_12 },
    { 0xe0120510, SUC_lexicon_tags_11 },
    { 0x98f20518, SUC_lexicon_tags_25 },
    { 0xd7f20519, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7292051b, SUC_lexicon_tags_46 },
    { 0xadb2051c, SUC_lexicon_tags_26 },
    { 0xba72051c, SUC_lexicon_tags_87 },
    { 0x3572051e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f20521, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39d20523, SUC_lexicon_tags_49 },
    { 0x6db20524, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd8320526, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea52052a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2112052e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae720530, SUC_lexicon_tags_12 },
    { 0xfdd20531, SUC_lexicon_tags_25 },
    { 0x1920532, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x73320534, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32120539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8072053c, SUC_lexicon_tags_39 },
    { 0x4f2053c, SUC_lexicon_tags_37 },
    { 0xcf72053c, SUC_lexicon_tags_37 },
    { 0x67d2053f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd20547, SUC_lexicon_tags_11 },
    { 0x8a520548, SUC_lexicon_tags_2 },
    { 0xf6520549, SUC_lexicon_tags_11 },
    { 0xf8b2054a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f72054c, SUC_lexicon_tags_17 },
    { 0x712054d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f92054f, SUC_lexicon_tags_37 },
    { 0xbd520550, SUC_lexicon_tags_45 },
    { 0xf1720551, SUC_lexicon_tags_37 },
    { 0x84f20551, SUC_lexicon_tags_26 },
    { 0xc7720552, SUC_lexicon_tags_37 },
    { 0xc3f20552, SUC_lexicon_tags_65 },
    { 0x43920555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2920558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd20562, SUC_lexicon_tags_17 },
    { 0xd9320563, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3012056a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad720570, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b20574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e720576, SUC_lexicon_tags_27 },
    { 0xce120577, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7452057a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff2057d, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97520581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc120583, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xef720586, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d120589, SUC_lexicon_tags_85 },
    { 0xf112058a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b2058d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b20595, SUC_lexicon_tags_329 },
    { 0x17520596, SUC_lexicon_tags_4 },
    { 0x66d20597, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96920599, SUC_lexicon_tags_30 },
    { 0x2152059a, SUC_lexicon_tags_66 },
    { 0x5152059b, SUC_lexicon_tags_43 },
    { 0x32b2059c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf092059e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11205a1, SUC_lexicon_tags_11 },
    { 0xe99205a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5205a6, SUC_lexicon_tags_47 },
    { 0x687205a7, SUC_lexicon_tags_17 },
    { 0x5dd205a8, SUC_lexicon_tags_9 },
    { 0x3cf205a7, SUC_lexicon_tags_25 },
    { 0xf11205a6, SUC_lexicon_tags_2 },
    { 0x19d205a6, SUC_lexicon_tags_66 },
    { 0xc45205ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41205b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9205b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x621205bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545205c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689205c4, SUC_lexicon_tags_33 },
    { 0x6ed205c4, SUC_lexicon_tags_15 },
    { 0x7b205c6, SUC_lexicon_tags_30 },
    { 0xc4d205c7, SUC_lexicon_tags_15 },
    { 0xc77205c7, SUC_lexicon_tags_31 },
    { 0x8db205c9, SUC_lexicon_tags_25 },
    { 0x7fb205c8, SUC_lexicon_tags_12 },
    { 0x81d205cb, SUC_lexicon_tags_38 },
    { 0x99d205cc, SUC_lexicon_tags_10 },
    { 0x5fd205cc, SUC_lexicon_tags_49 },
    { 0x353205ce, SUC_lexicon_tags_27 },
    { 0x765205ce, SUC_lexicon_tags_35 },
    { 0x145205cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x905205d3, SUC_lexicon_tags_1 },
    { 0x277205d4, SUC_lexicon_tags_11 },
    { 0xe9f205d4, SUC_lexicon_tags_17 },
    { 0x72d205d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b205d9, SUC_lexicon_tags_37 },
    { 0xc9b205da, SUC_lexicon_tags_56 },
    { 0xd7f205da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa43205dd, SUC_lexicon_tags_37 },
    { 0x39f205de, SUC_lexicon_tags_14 },
    { 0x3a9205df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d205e2, SUC_lexicon_tags_15 },
    { 0x4ef205e3, SUC_lexicon_tags_25 },
    { 0xf2d205e3, SUC_lexicon_tags_15 },
    { 0x729205e5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d205e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x687205ea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb205ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01205f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75f205f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb5205f9, SUC_lexicon_tags_38 },
    { 0xba9205fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63720601, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd20604, SUC_lexicon_tags_19 },
    { 0x4ed20605, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd12060e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa1920610, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11520618, SUC_lexicon_tags_41 },
    { 0xfa920619, SUC_lexicon_tags_33 },
    { 0x2a920619, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b52061e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff20623, SUC_lexicon_tags_2 },
    { 0x4d720624, SUC_lexicon_tags_9 },
    { 0x44120625, SUC_lexicon_tags_37 },
    { 0x13b20626, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d520628, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67720631, SUC_lexicon_tags_16 },
    { 0x3b20631, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12520638, SUC_lexicon_tags_15 },
    { 0xa7920639, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d2063f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b20643, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6692064a, SUC_lexicon_tags_61 },
    { 0x97f2064a, SUC_lexicon_tags_17 },
    { 0xf492064c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x2972064f, SUC_lexicon_tags_8 },
    { 0xccf2064f, SUC_lexicon_tags_13 },
    { 0xdb20651, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7a720653, SUC_lexicon_tags_1 },
    { 0x36d20654, SUC_lexicon_tags_19 },
    { 0x91320653, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af20659, SUC_lexicon_tags_5 },
    { 0xee120659, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b2065d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb720661, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb20667, SUC_lexicon_tags_20 },
    { 0x63b20668, SUC_lexicon_tags_4 },
    { 0xc0120669, SUC_lexicon_tags_11 },
    { 0x8592066a, SUC_lexicon_tags_55 },
    { 0x72f2066b, SUC_lexicon_tags_68 },
    { 0x1dd2066b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b2066f, SUC_lexicon_tags_77 },
    { 0x7bd20670, SUC_lexicon_tags_87 },
    { 0x6f120670, SUC_lexicon_tags_37 },
    { 0xdc520670, SUC_lexicon_tags_235 },
    { 0x8c720673, SUC_lexicon_tags_38 },
    { 0x64520674, SUC_lexicon_tags_17 },
    { 0x7120675, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94720677, SUC_lexicon_tags_2 },
    { 0x84f20678, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf152067a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf212067c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f20680, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10d20682, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96920688, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe632068b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c52068f, SUC_lexicon_tags_37 },
    { 0xd5920690, SUC_lexicon_tags_17 },
    { 0x58920691, SUC_lexicon_tags_11 },
    { 0x40b20691, SUC_lexicon_tags_47 },
    { 0x6320693, SUC_lexicon_tags_19 },
    { 0x27920693, SUC_lexicon_tags_32 },
    { 0x68720694, SUC_lexicon_tags_66 },
    { 0x3d20696, SUC_lexicon_tags_25 },
    { 0x4bf20697, SUC_lexicon_tags_77 },
    { 0xc0d20698, SUC_lexicon_tags_79 },
    { 0x76920696, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d2069e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27206a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfdb206a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef206a6, SUC_lexicon_tags_17 },
    { 0xe9b206a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b206ad, SUC_lexicon_tags_1 },
    { 0x199206ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f206b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2d206b7, SUC_lexicon_tags_60 },
    { 0x921206b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdd206ba, SUC_lexicon_tags_5 },
    { 0xb7b206ba, SUC_lexicon_tags_2 },
    { 0x96d206bb, SUC_lexicon_tags_47 },
    { 0xe0f206bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33206c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd35206c3, SUC_lexicon_tags_47 },
    { 0xd3f206c4, SUC_lexicon_tags_47 },
    { 0xc8d206c5, SUC_lexicon_tags_2 },
    { 0xa1206c6, SUC_lexicon_tags_37 },
    { 0x649206c6, SUC_lexicon_tags_15 },
    { 0xbab206c8, SUC_lexicon_tags_43 },
    { 0x867206c9, SUC_lexicon_tags_37 },
    { 0xa61206c5, SUC_lexicon_tags_16 },
    { 0x1e3206cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13206d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0b206d5, SUC_lexicon_tags_2 },
    { 0xb1b206d6, SUC_lexicon_tags_37 },
    { 0x3ad206d7, SUC_lexicon_tags_11 },
    { 0x55206d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x289206da, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1206dd, SUC_lexicon_tags_7 },
    { 0xae7206de, SUC_lexicon_tags_16 },
    { 0x103206de, SUC_lexicon_tags_61 },
    { 0x435206de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61206e4, SUC_lexicon_tags_1 },
    { 0x523206e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x579206e7, SUC_lexicon_tags_45 },
    { 0x105206e7, SUC_lexicon_tags_34 },
    { 0x195206e9, SUC_lexicon_tags_11 },
    { 0x305206ea, SUC_lexicon_tags_12 },
    { 0x79f206e7, SUC_lexicon_tags_15 },
    { 0x1a7206ec, SUC_lexicon_tags_0 },
    { 0xf51206ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1206f1, SUC_lexicon_tags_60 },
    { 0x89d206f2, SUC_lexicon_tags_37 },
    { 0xc7f206f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7206f6, SUC_lexicon_tags_37 },
    { 0x3f3206f7, SUC_lexicon_tags_56 },
    { 0xd29206f8, SUC_lexicon_tags_17 },
    { 0x265206f9, SUC_lexicon_tags_11 },
    { 0x68b206f6, SUC_lexicon_tags_11 },
    { 0x295206fb, SUC_lexicon_tags_23 },
    { 0x11f206fb, SUC_lexicon_tags_23 },
    { 0xa7f206fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91d206ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d320702, SUC_lexicon_tags_1 },
    { 0x12f20702, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb120705, SUC_lexicon_tags_201 },
    { 0xfdb20706, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd52070b, SUC_lexicon_tags_47 },
    { 0x2152070c, SUC_lexicon_tags_7 },
    { 0x6df2070d, SUC_lexicon_tags_0 },
    { 0x7c12070d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4720710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20714, SUC_lexicon_tags_11 },
    { 0x3c920714, SUC_lexicon_tags_2 },
    { 0xc1b20716, SUC_lexicon_tags_15 },
    { 0xdd20717, SUC_lexicon_tags_12 },
    { 0x32520718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a72071d, SUC_lexicon_tags_11 },
    { 0x6d32071e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbd20720, SUC_lexicon_tags_0 },
    { 0x6f20720, SUC_lexicon_tags_1 },
    { 0x73520722, SUC_lexicon_tags_63 },
    { 0x72920723, SUC_lexicon_tags_25 },
    { 0xd2f20724, SUC_lexicon_tags_17 },
    { 0x2eb20723, SUC_lexicon_tags_41 },
    { 0x8b920726, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9a720728, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb352072d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x66520730, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3520733, SUC_lexicon_tags_23 },
    { 0x69320734, SUC_lexicon_tags_25 },
    { 0x8cb20735, SUC_lexicon_tags_17 },
    { 0x36320735, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf52073d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf20740, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb720745, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d20748, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c32074d, SUC_lexicon_tags_14 },
    { 0x9a92074e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b20753, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d20758, SUC_lexicon_tags_15 },
    { 0x1ad20759, SUC_lexicon_tags_11 },
    { 0x33d20759, SUC_lexicon_tags_15 },
    { 0x7272075b, SUC_lexicon_tags_10 },
    { 0xfcf2075b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd20761, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2076a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5132076c, SUC_lexicon_tags_26 },
    { 0x1f2076c, SUC_lexicon_tags_25 },
    { 0x7d2076e, SUC_lexicon_tags_32 },
    { 0xfb72076c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x57920772, SUC_lexicon_tags_11 },
    { 0xcc320773, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbd120775, SUC_lexicon_tags_15 },
    { 0xa6b20775, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f2077a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6072077e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27120780, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32320786, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30320789, SUC_lexicon_tags_38 },
    { 0x3f2078a, SUC_lexicon_tags_25 },
    { 0x31f2078a, SUC_lexicon_tags_37 },
    { 0x2652078c, SUC_lexicon_tags_25 },
    { 0x5b72078c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28720791, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb20795, SUC_lexicon_tags_11 },
    { 0x90b20796, SUC_lexicon_tags_15 },
    { 0x8a320797, SUC_lexicon_tags_19 },
    { 0x80d20796, SUC_lexicon_tags_17 },
    { 0x3f720797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef32079b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4772079e, SUC_lexicon_tags_43 },
    { 0xca72079e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b207a8, SUC_lexicon_tags_708 },
    { 0, NULL },
    { 0, NULL },
    { 0x729207ab, SUC_lexicon_tags_47 },
    { 0x381207ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf207ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b207b2, SUC_lexicon_tags_23 },
    { 0xeb3207b3, SUC_lexicon_tags_34 },
    { 0xb71207b2, SUC_lexicon_tags_48 },
    { 0xacb207b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f207b9, SUC_lexicon_tags_38 },
    { 0xab5207b9, SUC_lexicon_tags_295 },
    { 0xa4d207bb, SUC_lexicon_tags_4 },
    { 0x2b3207b9, SUC_lexicon_tags_34 },
    { 0xeab207bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569207c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf207c6, SUC_lexicon_tags_12 },
    { 0x439207c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37207ca, SUC_lexicon_tags_45 },
    { 0xeb7207cb, SUC_lexicon_tags_146 },
    { 0x63d207cb, SUC_lexicon_tags_47 },
    { 0x8b1207cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7207d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23207d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b207dc, SUC_lexicon_tags_17 },
    { 0xa3f207dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x959207df, SUC_lexicon_tags_37 },
    { 0x543207e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29207e5, SUC_lexicon_tags_0 },
    { 0x6b3207e6, SUC_lexicon_tags_90 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135207ea, SUC_lexicon_tags_21 },
    { 0x903207eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d207ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x415207f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e1207f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715207f6, SUC_lexicon_tags_77 },
    { 0x149207f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7207fb, SUC_lexicon_tags_37 },
    { 0x255207fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d20805, SUC_lexicon_tags_37 },
    { 0x78920805, SUC_lexicon_tags_48 },
    { 0xcd20807, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa0120809, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1db2080b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b20812, SUC_lexicon_tags_1 },
    { 0x4920812, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df20816, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b2081c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd20821, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a320826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d2082b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7c52082d, SUC_lexicon_tags_13 },
    { 0x8992082e, SUC_lexicon_tags_6 },
    { 0xe152082e, SUC_lexicon_tags_15 },
    { 0x8972082f, SUC_lexicon_tags_25 },
    { 0xe0b2082d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab520833, SUC_lexicon_tags_60 },
    { 0x80520833, SUC_lexicon_tags_37 },
    { 0x46120835, SUC_lexicon_tags_9 },
    { 0x84f20833, SUC_lexicon_tags_17 },
    { 0x8b720837, SUC_lexicon_tags_11 },
    { 0x67920837, SUC_lexicon_tags_53 },
    { 0x17520837, SUC_lexicon_tags_4 },
    { 0x7b52083a, SUC_lexicon_tags_35 },
    { 0xf9520839, SUC_lexicon_tags_11 },
    { 0x22d2083c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b20840, SUC_lexicon_tags_15 },
    { 0x94720841, SUC_lexicon_tags_17 },
    { 0xaf520841, SUC_lexicon_tags_33 },
    { 0x31f20843, SUC_lexicon_tags_23 },
    { 0x1e320843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd520847, SUC_lexicon_tags_28 },
    { 0xa0920847, SUC_lexicon_tags_26 },
    { 0x8a120849, SUC_lexicon_tags_7 },
    { 0xe5d20847, SUC_lexicon_tags_25 },
    { 0x9f720848, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc772084d, SUC_lexicon_tags_45 },
    { 0xc052084e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7720853, SUC_lexicon_tags_27 },
    { 0xb9d20854, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaf20856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f2085d, SUC_lexicon_tags_1 },
    { 0x7bf2085e, SUC_lexicon_tags_49 },
    { 0x4572085f, SUC_lexicon_tags_7 },
    { 0xe072085f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3512086b, SUC_lexicon_tags_2 },
    { 0x9d72086b, SUC_lexicon_tags_29 },
    { 0x1452086d, SUC_lexicon_tags_12 },
    { 0x5892086d, SUC_lexicon_tags_12 },
    { 0xbbb2086c, SUC_lexicon_tags_45 },
    { 0xb8320870, SUC_lexicon_tags_30 },
    { 0xa912086d, SUC_lexicon_tags_45 },
    { 0x25d20872, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f20876, SUC_lexicon_tags_7 },
    { 0xd2b20876, SUC_lexicon_tags_47 },
    { 0x55120876, SUC_lexicon_tags_43 },
    { 0x75320876, SUC_lexicon_tags_7 },
    { 0xc852087a, SUC_lexicon_tags_17 },
    { 0x9e720879, SUC_lexicon_tags_60 },
    { 0xb3320878, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6512088b, SUC_lexicon_tags_0 },
    { 0xe032088b, SUC_lexicon_tags_12 },
    { 0x60f2088d, SUC_lexicon_tags_2 },
    { 0x6732088e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d20891, SUC_lexicon_tags_6 },
    { 0x50320891, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab20896, SUC_lexicon_tags_60 },
    { 0x68920897, SUC_lexicon_tags_1 },
    { 0xfbf20897, SUC_lexicon_tags_92 },
    { 0x81b20899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9752089b, SUC_lexicon_tags_17 },
    { 0xd9b2089b, SUC_lexicon_tags_91 },
    { 0x63f2089c, SUC_lexicon_tags_710 },
    { 0x992089e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b208a1, SUC_lexicon_tags_1 },
    { 0x6df208a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf95208a4, SUC_lexicon_tags_27 },
    { 0x6c9208a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77208a8, SUC_lexicon_tags_12 },
    { 0xe7f208a8, SUC_lexicon_tags_15 },
    { 0x5cb208aa, SUC_lexicon_tags_77 },
    { 0xc73208a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd1208ad, SUC_lexicon_tags_38 },
    { 0x2b5208ae, SUC_lexicon_tags_17 },
    { 0x97d208af, SUC_lexicon_tags_2 },
    { 0xdb9208af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f7208b2, SUC_lexicon_tags_1 },
    { 0x537208b3, SUC_lexicon_tags_27 },
    { 0x93f208b3, SUC_lexicon_tags_2 },
    { 0xa3d208b5, SUC_lexicon_tags_49 },
    { 0xc2d208b5, SUC_lexicon_tags_25 },
    { 0x429208b6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499208bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63208c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2d208c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff208c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef208cf, SUC_lexicon_tags_1 },
    { 0xdc9208cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab208d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b208d8, SUC_lexicon_tags_38 },
    { 0xd15208d8, SUC_lexicon_tags_191 },
    { 0x111208da, SUC_lexicon_tags_37 },
    { 0x99208db, SUC_lexicon_tags_9 },
    { 0xf97208d9, SUC_lexicon_tags_25 },
    { 0x259208dc, SUC_lexicon_tags_282 },
    { 0x40f208de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf208e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3208e5, SUC_lexicon_tags_30 },
    { 0xc71208e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f208ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf208ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf07208ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5208f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa15208f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85208fb, SUC_lexicon_tags_25 },
    { 0x8a3208fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7208ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6920906, SUC_lexicon_tags_37 },
    { 0x98d20906, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x65b20909, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd2090e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde320913, SUC_lexicon_tags_12 },
    { 0xe1320914, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22920917, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4052091a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db2091d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc6b2091f, SUC_lexicon_tags_8 },
    { 0xcd320920, SUC_lexicon_tags_27 },
    { 0x40520921, SUC_lexicon_tags_25 },
    { 0x2132091f, SUC_lexicon_tags_6 },
    { 0xb2320920, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d20926, SUC_lexicon_tags_0 },
    { 0xda320926, SUC_lexicon_tags_48 },
    { 0x1b720928, SUC_lexicon_tags_91 },
    { 0x6f20929, SUC_lexicon_tags_81 },
    { 0x3f92092a, SUC_lexicon_tags_11 },
    { 0xedf2092b, SUC_lexicon_tags_15 },
    { 0xa52092b, SUC_lexicon_tags_12 },
    { 0xb2520928, SUC_lexicon_tags_17 },
    { 0x8c32092e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52320931, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb520933, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x23120935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b2093d, SUC_lexicon_tags_28 },
    { 0xc752093d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb6720940, SUC_lexicon_tags_47 },
    { 0x8f120941, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda320945, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x77720947, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a32094c, SUC_lexicon_tags_398 },
    { 0xa0b2094d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78320952, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd0720954, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79b20956, SUC_lexicon_tags_15 },
    { 0xdf520956, SUC_lexicon_tags_68 },
    { 0x5e920958, SUC_lexicon_tags_1 },
    { 0x37b20959, SUC_lexicon_tags_11 },
    { 0x47b20958, SUC_lexicon_tags_25 },
    { 0x87320959, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f2095f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd8320961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d20966, SUC_lexicon_tags_15 },
    { 0x5d120967, SUC_lexicon_tags_14 },
    { 0x49b20968, SUC_lexicon_tags_26 },
    { 0xb20966, SUC_lexicon_tags_26 },
    { 0x8e120966, SUC_lexicon_tags_9 },
    { 0xb052096b, SUC_lexicon_tags_25 },
    { 0xb1b20967, SUC_lexicon_tags_23 },
    { 0x8852096d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x18b2096f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0520973, SUC_lexicon_tags_60 },
    { 0x1af20974, SUC_lexicon_tags_62 },
    { 0xb1d20974, SUC_lexicon_tags_26 },
    { 0xad520976, SUC_lexicon_tags_19 },
    { 0x7520976, SUC_lexicon_tags_25 },
    { 0x26720978, SUC_lexicon_tags_80 },
    { 0x7db20975, SUC_lexicon_tags_15 },
    { 0x55b20979, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb2097f, SUC_lexicon_tags_46 },
    { 0xc7520980, SUC_lexicon_tags_48 },
    { 0x69720981, SUC_lexicon_tags_4 },
    { 0xe9120980, SUC_lexicon_tags_65 },
    { 0x28120981, SUC_lexicon_tags_35 },
    { 0x31f20982, SUC_lexicon_tags_45 },
    { 0x13320981, SUC_lexicon_tags_7 },
    { 0xcb520986, SUC_lexicon_tags_28 },
    { 0x80520980, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52098f, SUC_lexicon_tags_35 },
    { 0xdcd2098f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29320992, SUC_lexicon_tags_17 },
    { 0xea120992, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d720999, SUC_lexicon_tags_15 },
    { 0xf6920999, SUC_lexicon_tags_1 },
    { 0x56720999, SUC_lexicon_tags_14 },
    { 0xc32099a, SUC_lexicon_tags_10 },
    { 0x7032099b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef209a0, SUC_lexicon_tags_15 },
    { 0x383209a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563209ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab209ae, SUC_lexicon_tags_806 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625209b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09209ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ab209bc, SUC_lexicon_tags_63 },
    { 0xc35209bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3209c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9209c3, SUC_lexicon_tags_80 },
    { 0x9fd209c4, SUC_lexicon_tags_9 },
    { 0x4ab209c4, SUC_lexicon_tags_37 },
    { 0x50f209c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21209c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff209cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39f209cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259209da, SUC_lexicon_tags_17 },
    { 0xdf209db, SUC_lexicon_tags_25 },
    { 0xfb9209dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167209e2, SUC_lexicon_tags_60 },
    { 0xdc1209e2, SUC_lexicon_tags_43 },
    { 0x16b209e2, SUC_lexicon_tags_25 },
    { 0xf55209e5, SUC_lexicon_tags_7 },
    { 0x2ad209e3, SUC_lexicon_tags_26 },
    { 0x6a1209e7, SUC_lexicon_tags_2 },
    { 0xd35209e7, SUC_lexicon_tags_7 },
    { 0xf51209e2, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5209ec, SUC_lexicon_tags_23 },
    { 0x28f209ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f209f6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9209f9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f209fd, SUC_lexicon_tags_27 },
    { 0xa6f209fd, SUC_lexicon_tags_14 },
    { 0x497209ff, SUC_lexicon_tags_25 },
    { 0xbff20a00, SUC_lexicon_tags_38 },
    { 0x20720a01, SUC_lexicon_tags_7 },
    { 0x9f20a01, SUC_lexicon_tags_12 },
    { 0x9df20a01, SUC_lexicon_tags_47 },
    { 0xc6720a04, SUC_lexicon_tags_6 },
    { 0xfa120a04, SUC_lexicon_tags_12 },
    { 0xe3f20a06, SUC_lexicon_tags_25 },
    { 0xff209ff, SUC_lexicon_tags_10 },
    { 0x48320a08, SUC_lexicon_tags_12 },
    { 0xce320a02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89720a0b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c920a11, SUC_lexicon_tags_2 },
    { 0x6320a12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79720a19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f20a1c, SUC_lexicon_tags_25 },
    { 0xecf20a1d, SUC_lexicon_tags_2 },
    { 0x3f920a1c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x21120a20, SUC_lexicon_tags_49 },
    { 0x57320a21, SUC_lexicon_tags_63 },
    { 0x90120a22, SUC_lexicon_tags_14 },
    { 0x2f20a23, SUC_lexicon_tags_10 },
    { 0xfdd20a21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0f20a26, SUC_lexicon_tags_27 },
    { 0x39d20a27, SUC_lexicon_tags_26 },
    { 0xbc120a26, SUC_lexicon_tags_104 },
    { 0x26320a29, SUC_lexicon_tags_60 },
    { 0x38320a29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6520a2d, SUC_lexicon_tags_7 },
    { 0xff920a2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f920a31, SUC_lexicon_tags_8 },
    { 0xbe320a32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbb320a34, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x39320a36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af20a39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31d20a3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2520a3e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c320a46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38920a48, SUC_lexicon_tags_68 },
    { 0x8a320a48, SUC_lexicon_tags_2 },
    { 0x16120a4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f20a4d, SUC_lexicon_tags_43 },
    { 0xfab20a4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f20a53, SUC_lexicon_tags_14 },
    { 0x95320a53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3920a5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b20a66, SUC_lexicon_tags_1 },
    { 0x2f720a67, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf20a6b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf20a75, SUC_lexicon_tags_40 },
    { 0xa4520a75, SUC_lexicon_tags_15 },
    { 0xfbf20a77, SUC_lexicon_tags_37 },
    { 0xaad20a78, SUC_lexicon_tags_0 },
    { 0xcdf20a79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f20a7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42320a81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92120a85, SUC_lexicon_tags_25 },
    { 0x1f920a86, SUC_lexicon_tags_19 },
    { 0xecf20a86, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xab320a89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8720a90, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54720a92, SUC_lexicon_tags_29 },
    { 0x76720a92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4d20a95, SUC_lexicon_tags_1 },
    { 0xcdd20a95, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x61b20a98, SUC_lexicon_tags_29 },
    { 0x38720a98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c920aa2, SUC_lexicon_tags_15 },
    { 0xfcd20aa3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2520aa9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf20aac, SUC_lexicon_tags_91 },
    { 0x5eb20aad, SUC_lexicon_tags_14 },
    { 0x50320aac, SUC_lexicon_tags_2 },
    { 0xcfb20aac, SUC_lexicon_tags_11 },
    { 0x40320aac, SUC_lexicon_tags_53 },
    { 0xbc120ab1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d20ab6, SUC_lexicon_tags_2 },
    { 0xa7120ab7, SUC_lexicon_tags_15 },
    { 0xe2b20ab8, SUC_lexicon_tags_2 },
    { 0xc0320ab8, SUC_lexicon_tags_37 },
    { 0x14d20ab7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9720abc, SUC_lexicon_tags_63 },
    { 0xf1f20abd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb20ac2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1cb20ac4, SUC_lexicon_tags_25 },
    { 0x57f20ac5, SUC_lexicon_tags_9 },
    { 0x8dd20ac5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35920aca, SUC_lexicon_tags_87 },
    { 0x7d520acb, SUC_lexicon_tags_2 },
    { 0xb2120acc, SUC_lexicon_tags_11 },
    { 0x6c520acc, SUC_lexicon_tags_39 },
    { 0xe3f20acc, SUC_lexicon_tags_37 },
    { 0x55b20acf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x67120ad1, SUC_lexicon_tags_30 },
    { 0xcb320ad2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7920ad7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df20ada, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91520add, SUC_lexicon_tags_17 },
    { 0xebf20add, SUC_lexicon_tags_17 },
    { 0x37920adf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x77d20ae1, SUC_lexicon_tags_1 },
    { 0x3b320ae2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b20ae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8720ae8, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x11b20aea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82520af1, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a720af4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f20af7, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab720afb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xbbf20afd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5520aff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe920b01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab20b04, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b20b07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b20b11, SUC_lexicon_tags_47 },
    { 0x5d720b12, SUC_lexicon_tags_60 },
    { 0xae920b13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd20b17, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf20b1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a320b1f, SUC_lexicon_tags_15 },
    { 0xd8320b1f, SUC_lexicon_tags_32 },
    { 0xee320b21, SUC_lexicon_tags_19 },
    { 0x7b120b22, SUC_lexicon_tags_0 },
    { 0xe0b20b23, SUC_lexicon_tags_21 },
    { 0x9dd20b24, SUC_lexicon_tags_0 },
    { 0xfd920b22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41320b33, SUC_lexicon_tags_12 },
    { 0x5a520b34, SUC_lexicon_tags_26 },
    { 0xd6f20b33, SUC_lexicon_tags_43 },
    { 0x75320b36, SUC_lexicon_tags_13 },
    { 0xeb120b37, SUC_lexicon_tags_11 },
    { 0xd8920b38, SUC_lexicon_tags_12 },
    { 0x7d520b39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb20b3f, SUC_lexicon_tags_19 },
    { 0xaf120b40, SUC_lexicon_tags_7 },
    { 0x12720b3f, SUC_lexicon_tags_11 },
    { 0xda320b3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7f920b44, SUC_lexicon_tags_4 },
    { 0x95f20b45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78920b4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8920b5a, SUC_lexicon_tags_1232 },
    { 0, NULL },
    { 0, NULL },
    { 0x81720b5d, SUC_lexicon_tags_31 },
    { 0x36320b5e, SUC_lexicon_tags_11 },
    { 0xea320b5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82520b61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f20b66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff20b6b, SUC_lexicon_tags_4 },
    { 0xe5f20b6c, SUC_lexicon_tags_12 },
    { 0xd3b20b6d, SUC_lexicon_tags_336 },
    { 0xb5520b6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8320b71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95720b73, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d20b76, SUC_lexicon_tags_26 },
    { 0xa6f20b77, SUC_lexicon_tags_2 },
    { 0x19d20b78, SUC_lexicon_tags_1 },
    { 0x3cd20b78, SUC_lexicon_tags_25 },
    { 0xaf320b79, SUC_lexicon_tags_1695 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e120b7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79720b83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2520b86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6120b89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa920b8d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1b720b8f, SUC_lexicon_tags_6 },
    { 0x1c920b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d20b98, SUC_lexicon_tags_11 },
    { 0xd6520b99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f920ba1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa720ba5, SUC_lexicon_tags_12 },
    { 0x3ff20ba6, SUC_lexicon_tags_55 },
    { 0x9c520ba7, SUC_lexicon_tags_19 },
    { 0x11720ba5, SUC_lexicon_tags_7 },
    { 0xca720ba7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14520bae, SUC_lexicon_tags_12 },
    { 0xa0f20baf, SUC_lexicon_tags_220 },
    { 0x9dd20bb0, SUC_lexicon_tags_12 },
    { 0xafd20bb1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf0d20bb3, SUC_lexicon_tags_33 },
    { 0x7fd20bb4, SUC_lexicon_tags_17 },
    { 0x56120bb5, SUC_lexicon_tags_23 },
    { 0xf4b20bb5, SUC_lexicon_tags_12 },
    { 0xc6b20bb5, SUC_lexicon_tags_10 },
    { 0x22920bb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x28b20bba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f120bbd, SUC_lexicon_tags_37 },
    { 0x15720bbe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86f20bc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf20bc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb920bcd, SUC_lexicon_tags_34 },
    { 0x37920bce, SUC_lexicon_tags_32 },
    { 0xf4320bce, SUC_lexicon_tags_64 },
    { 0xf1d20bce, SUC_lexicon_tags_37 },
    { 0x9a720bcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4520bd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65920bd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4320bdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0320bdf, SUC_lexicon_tags_32 },
    { 0xb5f20bdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc6920be2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8920be9, SUC_lexicon_tags_17 },
    { 0xd2720be9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef20bed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80d20bef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87520bf4, SUC_lexicon_tags_12 },
    { 0x10920bf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e120bf7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd3d20bf9, SUC_lexicon_tags_37 },
    { 0xe4920bf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b20c02, SUC_lexicon_tags_47 },
    { 0xe1b20c03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0920c07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d20c0b, SUC_lexicon_tags_7 },
    { 0x75120c0c, SUC_lexicon_tags_1 },
    { 0x4d720c0b, SUC_lexicon_tags_37 },
    { 0xda120c0e, SUC_lexicon_tags_1 },
    { 0xe1520c0d, SUC_lexicon_tags_45 },
    { 0x19720c0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb720c16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5320c19, SUC_lexicon_tags_27 },
    { 0x4a520c19, SUC_lexicon_tags_8 },
    { 0x47b20c19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0520c20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3f20c22, SUC_lexicon_tags_57 },
    { 0x19920c23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaf320c25, SUC_lexicon_tags_17 },
    { 0x91720c26, SUC_lexicon_tags_14 },
    { 0x16d20c27, SUC_lexicon_tags_11 },
    { 0x1d320c28, SUC_lexicon_tags_2 },
    { 0x6e720c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b20c2c, SUC_lexicon_tags_9 },
    { 0x3fd20c2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2520c32, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf920c36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b20c3e, SUC_lexicon_tags_6 },
    { 0xe7f20c3e, SUC_lexicon_tags_68 },
    { 0x47b20c40, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae520c44, SUC_lexicon_tags_49 },
    { 0xc1320c45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6920c4a, SUC_lexicon_tags_15 },
    { 0xd8b20c4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42520c50, SUC_lexicon_tags_9 },
    { 0x53320c51, SUC_lexicon_tags_11 },
    { 0x68b20c52, SUC_lexicon_tags_7 },
    { 0x9af20c53, SUC_lexicon_tags_0 },
    { 0xb4920c54, SUC_lexicon_tags_7 },
    { 0x48320c55, SUC_lexicon_tags_37 },
    { 0xe6d20c54, SUC_lexicon_tags_7 },
    { 0xcf320c57, SUC_lexicon_tags_17 },
    { 0x9cb20c55, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd20c5d, SUC_lexicon_tags_11 },
    { 0xb3d20c5e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d520c69, SUC_lexicon_tags_63 },
    { 0x66d20c6a, SUC_lexicon_tags_26 },
    { 0xcb20c69, SUC_lexicon_tags_2 },
    { 0xda520c69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe120c70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f20c73, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd0520c75, SUC_lexicon_tags_0 },
    { 0xbf720c76, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb20c7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63b20c81, SUC_lexicon_tags_11 },
    { 0x44920c82, SUC_lexicon_tags_37 },
    { 0x5eb20c82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b20c89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39520c8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71920c91, SUC_lexicon_tags_49 },
    { 0xa7720c92, SUC_lexicon_tags_25 },
    { 0xd9920c92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9e520c95, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32720c99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b720c9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21b20c9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b20ca4, SUC_lexicon_tags_12 },
    { 0x34b20ca5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f20cab, SUC_lexicon_tags_7 },
    { 0xd520cac, SUC_lexicon_tags_11 },
    { 0x6ef20cab, SUC_lexicon_tags_15 },
    { 0x90120cae, SUC_lexicon_tags_37 },
    { 0xf9720caf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a720cb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b120cb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57b20cb5, SUC_lexicon_tags_17 },
    { 0x5f20cb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49920cb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79d20cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6fd20cbc, SUC_lexicon_tags_31 },
    { 0x38f20cbd, SUC_lexicon_tags_37 },
    { 0x48d20cbe, SUC_lexicon_tags_2 },
    { 0x56f20cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5120cc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x720cc7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6320ccb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd20ccf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x93120cd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f720cd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f20cd9, SUC_lexicon_tags_29 },
    { 0x4c320cda, SUC_lexicon_tags_33 },
    { 0xad20cdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f720ce6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b20cee, SUC_lexicon_tags_53 },
    { 0x6ab20cef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb2720cf1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x97120cf4, SUC_lexicon_tags_61 },
    { 0x9b320cf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e120cf7, SUC_lexicon_tags_17 },
    { 0x37920cf8, SUC_lexicon_tags_15 },
    { 0xa7320cf8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b120cfc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd520d02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b20d06, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b20d0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d20d11, SUC_lexicon_tags_15 },
    { 0x75720d12, SUC_lexicon_tags_1 },
    { 0xcc120d13, SUC_lexicon_tags_21 },
    { 0x18720d14, SUC_lexicon_tags_1 },
    { 0x9b320d14, SUC_lexicon_tags_45 },
    { 0xb0320d16, SUC_lexicon_tags_12 },
    { 0xff720d14, SUC_lexicon_tags_37 },
    { 0xc7d20d18, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4920d20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf20d23, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d20d26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb0720d28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0920d2a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x81d20d2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4520d31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38320d33, SUC_lexicon_tags_39 },
    { 0xdf120d34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed20d39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25920d3e, SUC_lexicon_tags_14 },
    { 0xe2320d3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcff20d41, SUC_lexicon_tags_26 },
    { 0x6c120d42, SUC_lexicon_tags_0 },
    { 0x3e920d43, SUC_lexicon_tags_29 },
    { 0xe5b20d42, SUC_lexicon_tags_5 },
    { 0xd5b20d44, SUC_lexicon_tags_1 },
    { 0x69720d46, SUC_lexicon_tags_12 },
    { 0x85520d47, SUC_lexicon_tags_8 },
    { 0x4c520d48, SUC_lexicon_tags_31 },
    { 0x6d20d49, SUC_lexicon_tags_37 },
    { 0x68d20d4a, SUC_lexicon_tags_37 },
    { 0xd0120d4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad720d50, SUC_lexicon_tags_61 },
    { 0xb9520d51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdf20d53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3320d55, SUC_lexicon_tags_30 },
    { 0xcf720d56, SUC_lexicon_tags_37 },
    { 0xe9720d57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20120d5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56520d5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcd20d5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9920d61, SUC_lexicon_tags_11 },
    { 0x85f20d62, SUC_lexicon_tags_47 },
    { 0x95320d62, SUC_lexicon_tags_86 },
    { 0xaa720d62, SUC_lexicon_tags_2 },
    { 0x80f20d65, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b20d69, SUC_lexicon_tags_1 },
    { 0x1cd20d6a, SUC_lexicon_tags_25 },
    { 0x9f20d6a, SUC_lexicon_tags_11 },
    { 0xf0320d69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d520d70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80120d74, SUC_lexicon_tags_37 },
    { 0x82b20d74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf520d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68b20d81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb20d84, SUC_lexicon_tags_507 },
    { 0x2f120d84, SUC_lexicon_tags_11 },
    { 0x12d20d85, SUC_lexicon_tags_14 },
    { 0x37520d86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c720d89, SUC_lexicon_tags_7 },
    { 0x7f120d89, SUC_lexicon_tags_0 },
    { 0xff920d8b, SUC_lexicon_tags_30 },
    { 0xfb920d8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8320d8e, SUC_lexicon_tags_1 },
    { 0x9c320d8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b520d92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55720da0, SUC_lexicon_tags_28 },
    { 0xa0920da1, SUC_lexicon_tags_1 },
    { 0xc3120da2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3320da4, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x19120da6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5d20da8, SUC_lexicon_tags_12 },
    { 0xec720da9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8120dad, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x2f920daf, SUC_lexicon_tags_19 },
    { 0x28d20daf, SUC_lexicon_tags_7 },
    { 0x56720db1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4320db3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x38120db5, SUC_lexicon_tags_56 },
    { 0x5df20db5, SUC_lexicon_tags_68 },
    { 0x84320db7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92120dbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3320dc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f20dc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e120dc6, SUC_lexicon_tags_29 },
    { 0x66520dc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23720dcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d20dd2, SUC_lexicon_tags_17 },
    { 0xe7920dd2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad20dd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7320dda, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c120ddf, SUC_lexicon_tags_6 },
    { 0x61720de0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d20de9, SUC_lexicon_tags_25 },
    { 0xea120de9, SUC_lexicon_tags_7 },
    { 0x99720de9, SUC_lexicon_tags_1 },
    { 0xee720deb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5920def, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3920df2, SUC_lexicon_tags_170 },
    { 0x82320df3, SUC_lexicon_tags_11 },
    { 0xe5920df4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b20dfa, SUC_lexicon_tags_33 },
    { 0xa8320dfa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb20dfe, SUC_lexicon_tags_46 },
    { 0x22720dfe, SUC_lexicon_tags_0 },
    { 0xeff20e00, SUC_lexicon_tags_25 },
    { 0xe7320e01, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5320e10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b20e13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5720e17, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x720e19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56120e1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x60d20e1d, SUC_lexicon_tags_1 },
    { 0x6cd20e1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4520e22, SUC_lexicon_tags_7 },
    { 0x35920e23, SUC_lexicon_tags_39 },
    { 0x18720e22, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9320e29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33520e2c, SUC_lexicon_tags_108 },
    { 0xa9920e2d, SUC_lexicon_tags_37 },
    { 0x99120e2e, SUC_lexicon_tags_37 },
    { 0xeb520e2f, SUC_lexicon_tags_77 },
    { 0xa1720e30, SUC_lexicon_tags_1 },
    { 0xd7920e31, SUC_lexicon_tags_1 },
    { 0xf1920e31, SUC_lexicon_tags_0 },
    { 0x73720e2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78d20e35, SUC_lexicon_tags_15 },
    { 0xf3b20e36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11120e38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b20e3f, SUC_lexicon_tags_12 },
    { 0xc5320e3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf120e44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b20e4d, SUC_lexicon_tags_0 },
    { 0xe1f20e4d, SUC_lexicon_tags_32 },
    { 0xe8b20e4f, SUC_lexicon_tags_39 },
    { 0xa0b20e50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab320e52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x91120e54, SUC_lexicon_tags_23 },
    { 0x9cd20e54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaa720e57, SUC_lexicon_tags_29 },
    { 0x1c120e57, SUC_lexicon_tags_25 },
    { 0x5c520e57, SUC_lexicon_tags_9 },
    { 0x8af20e5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6520e66, SUC_lexicon_tags_6 },
    { 0xf1120e67, SUC_lexicon_tags_2 },
    { 0xf3f20e68, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x63920e6a, SUC_lexicon_tags_17 },
    { 0xefb20e6a, SUC_lexicon_tags_42 },
    { 0xa6920e6b, SUC_lexicon_tags_24 },
    { 0x90b20e6d, SUC_lexicon_tags_43 },
    { 0x11720e6e, SUC_lexicon_tags_25 },
    { 0x15720e6d, SUC_lexicon_tags_37 },
    { 0x58d20e70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b520e72, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff20e75, SUC_lexicon_tags_65 },
    { 0xda320e76, SUC_lexicon_tags_62 },
    { 0xaa120e77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3120e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1320e7c, SUC_lexicon_tags_33 },
    { 0x99920e7d, SUC_lexicon_tags_0 },
    { 0xfdd20e7e, SUC_lexicon_tags_57 },
    { 0x7eb20e7d, SUC_lexicon_tags_1544 },
    { 0x3d320e7f, SUC_lexicon_tags_55 },
    { 0xaa720e81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed20e84, SUC_lexicon_tags_22 },
    { 0x1af20e85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe320e89, SUC_lexicon_tags_25 },
    { 0xef920e8a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0320e8d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c120e90, SUC_lexicon_tags_11 },
    { 0xba920e91, SUC_lexicon_tags_38 },
    { 0xdc720e92, SUC_lexicon_tags_3 },
    { 0x5120e93, SUC_lexicon_tags_7 },
    { 0xde720e92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a520e97, SUC_lexicon_tags_2 },
    { 0x64920e98, SUC_lexicon_tags_2 },
    { 0xccb20e98, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d20e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69b20e9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f20ea9, SUC_lexicon_tags_0 },
    { 0xc6720ea9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe320eae, SUC_lexicon_tags_48 },
    { 0x2b20eaf, SUC_lexicon_tags_17 },
    { 0xecf20eae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b20eb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b20ec1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1320eca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7920ecc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6920ece, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8720eda, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e520ede, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef20ee1, SUC_lexicon_tags_25 },
    { 0x6f20ee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65d20ee4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5c120ee6, SUC_lexicon_tags_8 },
    { 0xf5f20ee7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70920eee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e920ef0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67720ef5, SUC_lexicon_tags_25 },
    { 0x44320ef5, SUC_lexicon_tags_15 },
    { 0xb9f20ef7, SUC_lexicon_tags_11 },
    { 0xe5320ef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b20eff, SUC_lexicon_tags_13 },
    { 0x72b20f00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef20f03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44120f07, SUC_lexicon_tags_9 },
    { 0x23920f08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6520f0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18b20f0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd20f11, SUC_lexicon_tags_11 },
    { 0x7ef20f12, SUC_lexicon_tags_12 },
    { 0x4b120f11, SUC_lexicon_tags_25 },
    { 0x21520f12, SUC_lexicon_tags_1162 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d20f17, SUC_lexicon_tags_25 },
    { 0xfcd20f17, SUC_lexicon_tags_28 },
    { 0x7b920f19, SUC_lexicon_tags_65 },
    { 0xe9d20f1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66b20f1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19320f1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e520f20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60120f23, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd3320f25, SUC_lexicon_tags_63 },
    { 0x3db20f26, SUC_lexicon_tags_11 },
    { 0x59f20f27, SUC_lexicon_tags_25 },
    { 0xa3920f28, SUC_lexicon_tags_37 },
    { 0xe7720f28, SUC_lexicon_tags_25 },
    { 0x23120f27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66b20f2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xac920f2f, SUC_lexicon_tags_38 },
    { 0x24b20f30, SUC_lexicon_tags_12 },
    { 0x26720f30, SUC_lexicon_tags_26 },
    { 0xe9920f2f, SUC_lexicon_tags_15 },
    { 0xead20f31, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8120f39, SUC_lexicon_tags_37 },
    { 0xc5320f3a, SUC_lexicon_tags_15 },
    { 0x69720f39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6720f3d, SUC_lexicon_tags_1 },
    { 0x64b20f3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a120f40, SUC_lexicon_tags_23 },
    { 0x1fb20f41, SUC_lexicon_tags_26 },
    { 0xd3920f42, SUC_lexicon_tags_39 },
    { 0xc7120f40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb20f45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d20f48, SUC_lexicon_tags_2 },
    { 0x43d20f49, SUC_lexicon_tags_29 },
    { 0x7a320f49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f20f55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69320f59, SUC_lexicon_tags_47 },
    { 0x16720f5a, SUC_lexicon_tags_42 },
    { 0xdcd20f59, SUC_lexicon_tags_32 },
    { 0xaa520f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44d20f5e, SUC_lexicon_tags_12 },
    { 0x52520f5f, SUC_lexicon_tags_61 },
    { 0x53b20f5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e920f64, SUC_lexicon_tags_33 },
    { 0x64d20f65, SUC_lexicon_tags_1 },
    { 0xb920f64, SUC_lexicon_tags_19 },
    { 0xeb920f66, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x36920f6a, SUC_lexicon_tags_38 },
    { 0xf3120f6b, SUC_lexicon_tags_2 },
    { 0x7bb20f6c, SUC_lexicon_tags_39 },
    { 0x70f20f6c, SUC_lexicon_tags_19 },
    { 0x6f720f6e, SUC_lexicon_tags_15 },
    { 0x5d20f6e, SUC_lexicon_tags_12 },
    { 0x9520f70, SUC_lexicon_tags_38 },
    { 0xa3d20f71, SUC_lexicon_tags_11 },
    { 0xfa320f6f, SUC_lexicon_tags_7 },
    { 0xa0520f6f, SUC_lexicon_tags_68 },
    { 0xac320f74, SUC_lexicon_tags_25 },
    { 0xfb320f6d, SUC_lexicon_tags_6 },
    { 0x11f20f76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1320f78, SUC_lexicon_tags_45 },
    { 0x2f920f78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1320f7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8320f80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb20f83, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2520f89, SUC_lexicon_tags_15 },
    { 0xa1520f8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72320f90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48320f97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7120f99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0120f9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18520fa0, SUC_lexicon_tags_47 },
    { 0x73720fa1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a720fa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51120fac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f20faf, SUC_lexicon_tags_230 },
    { 0x72920fb0, SUC_lexicon_tags_1 },
    { 0xdc920faf, SUC_lexicon_tags_37 },
    { 0xe1d20fb1, SUC_lexicon_tags_38 },
    { 0x86d20fb0, SUC_lexicon_tags_26 },
    { 0x25320fb2, SUC_lexicon_tags_2 },
    { 0x6a520fb5, SUC_lexicon_tags_27 },
    { 0xb7720fb6, SUC_lexicon_tags_25 },
    { 0x21920fb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97320fbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b320fc0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b920fc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62b20fc6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd320fca, SUC_lexicon_tags_2 },
    { 0xcf920fcb, SUC_lexicon_tags_14 },
    { 0x47320fcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8320fcf, SUC_lexicon_tags_61 },
    { 0x9b120fd0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43720fd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb20fd9, SUC_lexicon_tags_82 },
    { 0xa9b20fda, SUC_lexicon_tags_2 },
    { 0x7e720fdb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa0b20fdd, SUC_lexicon_tags_32 },
    { 0x69720fdd, SUC_lexicon_tags_11 },
    { 0x86f20fdf, SUC_lexicon_tags_92 },
    { 0x3ed20fde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70120fe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61320fea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca920fef, SUC_lexicon_tags_33 },
    { 0x13520ff0, SUC_lexicon_tags_63 },
    { 0x83b20fef, SUC_lexicon_tags_15 },
    { 0x5f920ff2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a520ff5, SUC_lexicon_tags_11 },
    { 0xfa320ff5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa120ff9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1c120ffb, SUC_lexicon_tags_2 },
    { 0x4ef20ffc, SUC_lexicon_tags_38 },
    { 0xd0120ffd, SUC_lexicon_tags_25 },
    { 0xeb20ffd, SUC_lexicon_tags_0 },
    { 0x96f20fff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbab21001, SUC_lexicon_tags_17 },
    { 0xb8321002, SUC_lexicon_tags_37 },
    { 0x8ab21002, SUC_lexicon_tags_17 },
    { 0x35921003, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaed21006, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5921009, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d21013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2092101c, SUC_lexicon_tags_11 },
    { 0x7c52101c, SUC_lexicon_tags_38 },
    { 0xfcf2101e, SUC_lexicon_tags_9 },
    { 0x7992101d, SUC_lexicon_tags_33 },
    { 0xae12101f, SUC_lexicon_tags_3 },
    { 0xc8d21021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72921028, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b72102c, SUC_lexicon_tags_25 },
    { 0xce52102c, SUC_lexicon_tags_9 },
    { 0xb472102e, SUC_lexicon_tags_7 },
    { 0xccf2102d, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8321032, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb21036, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74521038, SUC_lexicon_tags_47 },
    { 0xea321039, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b2103e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb21044, SUC_lexicon_tags_12 },
    { 0xa3f21045, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28121049, SUC_lexicon_tags_31 },
    { 0x3532104a, SUC_lexicon_tags_46 },
    { 0x2f92104b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5d2104d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x81521050, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b21057, SUC_lexicon_tags_38 },
    { 0x3c721058, SUC_lexicon_tags_11 },
    { 0x44921058, SUC_lexicon_tags_12 },
    { 0xbad2105a, SUC_lexicon_tags_49 },
    { 0x9bf2105b, SUC_lexicon_tags_10 },
    { 0x5452105c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd592105e, SUC_lexicon_tags_12 },
    { 0xc12105f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d121062, SUC_lexicon_tags_17 },
    { 0xa7721063, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d21068, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xab72106b, SUC_lexicon_tags_15 },
    { 0x4a52106c, SUC_lexicon_tags_6 },
    { 0x33d2106d, SUC_lexicon_tags_106 },
    { 0x9f32106c, SUC_lexicon_tags_11 },
    { 0x6f92106b, SUC_lexicon_tags_87 },
    { 0x12d2106c, SUC_lexicon_tags_15 },
    { 0xd692106e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c121077, SUC_lexicon_tags_2 },
    { 0x8df21078, SUC_lexicon_tags_27 },
    { 0xe9f21079, SUC_lexicon_tags_38 },
    { 0x9ff2107a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af2107d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5921080, SUC_lexicon_tags_28 },
    { 0xa3921081, SUC_lexicon_tags_2 },
    { 0x8d121082, SUC_lexicon_tags_9 },
    { 0x8df21082, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d21087, SUC_lexicon_tags_17 },
    { 0x4bd21088, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a32108a, SUC_lexicon_tags_0 },
    { 0x1692108a, SUC_lexicon_tags_61 },
    { 0x6db2108b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb21090, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb21095, SUC_lexicon_tags_47 },
    { 0x73321096, SUC_lexicon_tags_11 },
    { 0xa6b21097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1b21099, SUC_lexicon_tags_14 },
    { 0xa852109a, SUC_lexicon_tags_11 },
    { 0x512109b, SUC_lexicon_tags_12 },
    { 0x92b2109c, SUC_lexicon_tags_7 },
    { 0xa4f21099, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd992109f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f210a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b210a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x185210a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf210b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d210b4, SUC_lexicon_tags_21 },
    { 0x10b210b5, SUC_lexicon_tags_27 },
    { 0x789210b5, SUC_lexicon_tags_25 },
    { 0x1cd210b7, SUC_lexicon_tags_49 },
    { 0x6ed210b8, SUC_lexicon_tags_45 },
    { 0xb57210b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x813210bb, SUC_lexicon_tags_13 },
    { 0xdb210bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x165210be, SUC_lexicon_tags_9 },
    { 0x355210be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1210c2, SUC_lexicon_tags_17 },
    { 0x193210c3, SUC_lexicon_tags_7 },
    { 0x5bb210c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68b210c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f210d1, SUC_lexicon_tags_25 },
    { 0x7ed210d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f210d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd210d8, SUC_lexicon_tags_0 },
    { 0x657210d9, SUC_lexicon_tags_25 },
    { 0xc21210da, SUC_lexicon_tags_37 },
    { 0x1f3210da, SUC_lexicon_tags_17 },
    { 0x48b210dc, SUC_lexicon_tags_63 },
    { 0xe83210dc, SUC_lexicon_tags_31 },
    { 0x9d7210d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b210e3, SUC_lexicon_tags_37 },
    { 0x39f210e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x773210e7, SUC_lexicon_tags_7 },
    { 0x699210e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f210f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9fd210f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x737210f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d7210f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x219210f9, SUC_lexicon_tags_16 },
    { 0x6eb210f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67321101, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x78721103, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b21107, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf2110b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba921112, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71f21114, SUC_lexicon_tags_13 },
    { 0xca921115, SUC_lexicon_tags_3 },
    { 0xfe121114, SUC_lexicon_tags_30 },
    { 0x4721117, SUC_lexicon_tags_308 },
    { 0xaaf21115, SUC_lexicon_tags_17 },
    { 0xe2121119, SUC_lexicon_tags_10 },
    { 0x1b32111a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2111d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f21120, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x70521123, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf21128, SUC_lexicon_tags_18 },
    { 0xf7121129, SUC_lexicon_tags_2 },
    { 0x1c321129, SUC_lexicon_tags_9 },
    { 0x27d2112a, SUC_lexicon_tags_27 },
    { 0x21b2112c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed921130, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f21134, SUC_lexicon_tags_37 },
    { 0xc3321134, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd0b21137, SUC_lexicon_tags_84 },
    { 0xbd121138, SUC_lexicon_tags_17 },
    { 0x46921139, SUC_lexicon_tags_15 },
    { 0xaa921139, SUC_lexicon_tags_63 },
    { 0x1a321139, SUC_lexicon_tags_17 },
    { 0x5052113a, SUC_lexicon_tags_23 },
    { 0xaf72113d, SUC_lexicon_tags_35 },
    { 0xd2d2113d, SUC_lexicon_tags_77 },
    { 0xc772113f, SUC_lexicon_tags_19 },
    { 0x99b2113d, SUC_lexicon_tags_17 },
    { 0x1472113a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbbd21143, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeed21145, SUC_lexicon_tags_14 },
    { 0xd6121146, SUC_lexicon_tags_25 },
    { 0xac921145, SUC_lexicon_tags_577 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf2114c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74721152, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef21155, SUC_lexicon_tags_17 },
    { 0x56121156, SUC_lexicon_tags_2 },
    { 0xfe921157, SUC_lexicon_tags_30 },
    { 0x11121158, SUC_lexicon_tags_11 },
    { 0xd0321159, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94b2115b, SUC_lexicon_tags_1 },
    { 0xeed2115c, SUC_lexicon_tags_12 },
    { 0x1972115d, SUC_lexicon_tags_37 },
    { 0xf392115e, SUC_lexicon_tags_12 },
    { 0x6892115f, SUC_lexicon_tags_13 },
    { 0xbad2115f, SUC_lexicon_tags_2 },
    { 0x8ff2115e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1f321163, SUC_lexicon_tags_1 },
    { 0x7bf21164, SUC_lexicon_tags_7 },
    { 0x47721163, SUC_lexicon_tags_37 },
    { 0xc1321164, SUC_lexicon_tags_29 },
    { 0x46721167, SUC_lexicon_tags_28 },
    { 0xdd121168, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12116e, SUC_lexicon_tags_12 },
    { 0x5ed2116f, SUC_lexicon_tags_38 },
    { 0xd4121170, SUC_lexicon_tags_37 },
    { 0x1d321171, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15921178, SUC_lexicon_tags_12 },
    { 0x2f321179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30921185, SUC_lexicon_tags_17 },
    { 0xa9b21185, SUC_lexicon_tags_17 },
    { 0x14d21187, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d2118b, SUC_lexicon_tags_1 },
    { 0x7212118b, SUC_lexicon_tags_38 },
    { 0x61f2118c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d21194, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x43b21196, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6d521198, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b211a2, SUC_lexicon_tags_26 },
    { 0xda3211a3, SUC_lexicon_tags_2 },
    { 0xd2b211a3, SUC_lexicon_tags_7 },
    { 0xdb1211a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703211ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679211b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99211b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x473211b7, SUC_lexicon_tags_11 },
    { 0x4dd211b8, SUC_lexicon_tags_25 },
    { 0x371211b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x423211bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1211bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53211c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36b211c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7211c7, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0x3e7211c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7211cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b211d2, SUC_lexicon_tags_41 },
    { 0x9cf211d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbaf211d5, SUC_lexicon_tags_7 },
    { 0x9df211d6, SUC_lexicon_tags_1 },
    { 0x797211d7, SUC_lexicon_tags_4 },
    { 0x3c9211d8, SUC_lexicon_tags_4 },
    { 0x333211d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf211dd, SUC_lexicon_tags_2 },
    { 0x377211de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff211e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf99211e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36b211e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35211ec, SUC_lexicon_tags_11 },
    { 0x6b1211ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9211f0, SUC_lexicon_tags_9 },
    { 0x155211f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1211f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1211fd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1af211ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85f21201, SUC_lexicon_tags_2 },
    { 0x40b21202, SUC_lexicon_tags_45 },
    { 0x521203, SUC_lexicon_tags_23 },
    { 0x7321204, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d21208, SUC_lexicon_tags_0 },
    { 0xe3f21209, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88f2120b, SUC_lexicon_tags_17 },
    { 0xa12120c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdd2120e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88721210, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0921216, SUC_lexicon_tags_17 },
    { 0x8fd21217, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcd721219, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff2121c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f21222, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2521224, SUC_lexicon_tags_0 },
    { 0x72721224, SUC_lexicon_tags_70 },
    { 0x9db21224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d72122d, SUC_lexicon_tags_1 },
    { 0xccb2122e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3eb21230, SUC_lexicon_tags_14 },
    { 0xd4121231, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x95721234, SUC_lexicon_tags_11 },
    { 0xb7d21234, SUC_lexicon_tags_7 },
    { 0x97321236, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec52123e, SUC_lexicon_tags_31 },
    { 0x9352123e, SUC_lexicon_tags_11 },
    { 0xc7f21240, SUC_lexicon_tags_16 },
    { 0x46b21241, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef721243, SUC_lexicon_tags_14 },
    { 0xf6121243, SUC_lexicon_tags_25 },
    { 0x45121245, SUC_lexicon_tags_17 },
    { 0x4e121244, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf21249, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3132124b, SUC_lexicon_tags_17 },
    { 0x9352124c, SUC_lexicon_tags_39 },
    { 0x5012124d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x55b2124f, SUC_lexicon_tags_19 },
    { 0x6fd21250, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d21253, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd21257, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8bb21259, SUC_lexicon_tags_17 },
    { 0x23321259, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd721264, SUC_lexicon_tags_9 },
    { 0x22b21265, SUC_lexicon_tags_87 },
    { 0xb8721265, SUC_lexicon_tags_2 },
    { 0xf4121264, SUC_lexicon_tags_37 },
    { 0xd1321268, SUC_lexicon_tags_7 },
    { 0x41f21269, SUC_lexicon_tags_15 },
    { 0x81b2126a, SUC_lexicon_tags_107 },
    { 0x2192126b, SUC_lexicon_tags_37 },
    { 0x54721266, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2912126e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xef721271, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f21278, SUC_lexicon_tags_2 },
    { 0x9db21279, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1912127c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdb52127e, SUC_lexicon_tags_28 },
    { 0x572127f, SUC_lexicon_tags_11 },
    { 0xf592127f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7721282, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5721285, SUC_lexicon_tags_11 },
    { 0x5d721285, SUC_lexicon_tags_14 },
    { 0x75b21287, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd7d21289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f2128d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5321291, SUC_lexicon_tags_28 },
    { 0x91721292, SUC_lexicon_tags_16 },
    { 0x9af21293, SUC_lexicon_tags_17 },
    { 0xbe921294, SUC_lexicon_tags_45 },
    { 0x8721293, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c721297, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7fb21299, SUC_lexicon_tags_1 },
    { 0xab12129a, SUC_lexicon_tags_15 },
    { 0x9d72129b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fb2129d, SUC_lexicon_tags_25 },
    { 0xaf52129d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x417212a0, SUC_lexicon_tags_57 },
    { 0x52b212a1, SUC_lexicon_tags_87 },
    { 0x3c9212a2, SUC_lexicon_tags_15 },
    { 0x893212a1, SUC_lexicon_tags_25 },
    { 0x59d212a4, SUC_lexicon_tags_2 },
    { 0x5d212a2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b212a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad212ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f212b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb45212b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357212b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf212be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde7212c0, SUC_lexicon_tags_37 },
    { 0x223212c1, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29212cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819212d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd212dc, SUC_lexicon_tags_7 },
    { 0xc89212dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1212e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35212e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387212ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887212ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd91212f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51b212f2, SUC_lexicon_tags_60 },
    { 0x44b212f3, SUC_lexicon_tags_14 },
    { 0x82f212f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x209212f6, SUC_lexicon_tags_37 },
    { 0xe25212f6, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1212fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9212ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b21304, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d2130e, SUC_lexicon_tags_1 },
    { 0x4a92130e, SUC_lexicon_tags_17 },
    { 0xa672130e, SUC_lexicon_tags_28 },
    { 0xbcd21311, SUC_lexicon_tags_37 },
    { 0xa0f21312, SUC_lexicon_tags_16 },
    { 0xa2121313, SUC_lexicon_tags_11 },
    { 0xd6321314, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf132131a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed521323, SUC_lexicon_tags_2 },
    { 0x49121324, SUC_lexicon_tags_30 },
    { 0x13521324, SUC_lexicon_tags_23 },
    { 0xc4521326, SUC_lexicon_tags_12 },
    { 0xbeb21327, SUC_lexicon_tags_0 },
    { 0xa9521328, SUC_lexicon_tags_32 },
    { 0x1f121329, SUC_lexicon_tags_27 },
    { 0x9e921326, SUC_lexicon_tags_47 },
    { 0xbd121328, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f21333, SUC_lexicon_tags_19 },
    { 0xc6f21334, SUC_lexicon_tags_30 },
    { 0x1a121335, SUC_lexicon_tags_0 },
    { 0x52721335, SUC_lexicon_tags_30 },
    { 0xeb321337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7032133e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x26121340, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd521344, SUC_lexicon_tags_23 },
    { 0x23921345, SUC_lexicon_tags_1 },
    { 0x6c321346, SUC_lexicon_tags_11 },
    { 0xdd321345, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23921349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6712134b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2321354, SUC_lexicon_tags_46 },
    { 0xdeb21354, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4d721357, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6812135b, SUC_lexicon_tags_37 },
    { 0x6132135b, SUC_lexicon_tags_1098 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f21362, SUC_lexicon_tags_4 },
    { 0x23721363, SUC_lexicon_tags_14 },
    { 0xf5b21364, SUC_lexicon_tags_37 },
    { 0xea121365, SUC_lexicon_tags_21 },
    { 0xbf921366, SUC_lexicon_tags_38 },
    { 0xe1f21363, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x88921369, SUC_lexicon_tags_37 },
    { 0xb1321369, SUC_lexicon_tags_100 },
    { 0xe4f2136a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a321370, SUC_lexicon_tags_54 },
    { 0x26b21371, SUC_lexicon_tags_11 },
    { 0xd0121372, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c121377, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8f21379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe521381, SUC_lexicon_tags_91 },
    { 0x27521382, SUC_lexicon_tags_31 },
    { 0x2521381, SUC_lexicon_tags_13 },
    { 0x74d21384, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c321387, SUC_lexicon_tags_11 },
    { 0x7c921388, SUC_lexicon_tags_37 },
    { 0xf5721389, SUC_lexicon_tags_14 },
    { 0x74b21389, SUC_lexicon_tags_37 },
    { 0xac12138b, SUC_lexicon_tags_6 },
    { 0x6052138c, SUC_lexicon_tags_27 },
    { 0x3f52138a, SUC_lexicon_tags_25 },
    { 0xcb721388, SUC_lexicon_tags_7 },
    { 0x5892138c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21393, SUC_lexicon_tags_37 },
    { 0x10921394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d21397, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b2139a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2139f, SUC_lexicon_tags_29 },
    { 0x45b2139f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1213a7, SUC_lexicon_tags_7 },
    { 0x7d3213a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f213b3, SUC_lexicon_tags_25 },
    { 0x88d213b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f213b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d213ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff213bd, SUC_lexicon_tags_60 },
    { 0x7ff213be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83213c1, SUC_lexicon_tags_11 },
    { 0xe9b213c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f213c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x693213cb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x921213cd, SUC_lexicon_tags_43 },
    { 0x641213ce, SUC_lexicon_tags_1 },
    { 0x387213cf, SUC_lexicon_tags_0 },
    { 0xc17213cd, SUC_lexicon_tags_16 },
    { 0x389213cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xff9213d3, SUC_lexicon_tags_11 },
    { 0x8e3213d4, SUC_lexicon_tags_148 },
    { 0xa33213d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33213dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x533213df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x797213e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1213e5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b213e8, SUC_lexicon_tags_14 },
    { 0xc5f213e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf213ed, SUC_lexicon_tags_0 },
    { 0xb1b213ed, SUC_lexicon_tags_4 },
    { 0x8d5213ee, SUC_lexicon_tags_43 },
    { 0x815213ef, SUC_lexicon_tags_13 },
    { 0xa73213f1, SUC_lexicon_tags_25 },
    { 0x399213ee, SUC_lexicon_tags_65 },
    { 0x7c1213f3, SUC_lexicon_tags_25 },
    { 0xfa9213f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957213fb, SUC_lexicon_tags_27 },
    { 0xc3f213fc, SUC_lexicon_tags_45 },
    { 0xf25213fb, SUC_lexicon_tags_17 },
    { 0x381213fe, SUC_lexicon_tags_17 },
    { 0xda9213fe, SUC_lexicon_tags_2 },
    { 0xdb213ff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d21404, SUC_lexicon_tags_8 },
    { 0x75d21405, SUC_lexicon_tags_3 },
    { 0x55121406, SUC_lexicon_tags_85 },
    { 0x68121406, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x40b21409, SUC_lexicon_tags_37 },
    { 0xeb921409, SUC_lexicon_tags_35 },
    { 0xdb52140b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b32140d, SUC_lexicon_tags_14 },
    { 0x4e72140e, SUC_lexicon_tags_15 },
    { 0x3b2140f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc921411, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86321413, SUC_lexicon_tags_27 },
    { 0x22d21414, SUC_lexicon_tags_13 },
    { 0xaef21414, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd2141b, SUC_lexicon_tags_621 },
    { 0xd952141c, SUC_lexicon_tags_41 },
    { 0x8df2141c, SUC_lexicon_tags_7 },
    { 0x5a92141e, SUC_lexicon_tags_23 },
    { 0x7ad2141f, SUC_lexicon_tags_3 },
    { 0x5072141f, SUC_lexicon_tags_33 },
    { 0xc0521421, SUC_lexicon_tags_37 },
    { 0x51721421, SUC_lexicon_tags_12 },
    { 0xa4521422, SUC_lexicon_tags_26 },
    { 0x8b521423, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8721430, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb521433, SUC_lexicon_tags_25 },
    { 0xd8921433, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c721438, SUC_lexicon_tags_12 },
    { 0x9f521439, SUC_lexicon_tags_15 },
    { 0xb0d21439, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432143e, SUC_lexicon_tags_16 },
    { 0x4a72143f, SUC_lexicon_tags_11 },
    { 0x52f2143f, SUC_lexicon_tags_2 },
    { 0x3db2143e, SUC_lexicon_tags_37 },
    { 0x91521442, SUC_lexicon_tags_37 },
    { 0x8dd21441, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1721445, SUC_lexicon_tags_6 },
    { 0xbe321446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31321448, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad521450, SUC_lexicon_tags_3 },
    { 0xf1b21451, SUC_lexicon_tags_10 },
    { 0x9eb21452, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b21456, SUC_lexicon_tags_17 },
    { 0xac921457, SUC_lexicon_tags_47 },
    { 0x5a121457, SUC_lexicon_tags_45 },
    { 0x9d21456, SUC_lexicon_tags_12 },
    { 0x7b121457, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b21463, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8721471, SUC_lexicon_tags_7 },
    { 0x70521471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68321474, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x95921476, SUC_lexicon_tags_68 },
    { 0xf9121477, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc4121479, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x79d2147b, SUC_lexicon_tags_2 },
    { 0xf3b2147c, SUC_lexicon_tags_17 },
    { 0x6b72147d, SUC_lexicon_tags_48 },
    { 0xce72147e, SUC_lexicon_tags_39 },
    { 0x6032147f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26921481, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21489, SUC_lexicon_tags_14 },
    { 0xa7f2148a, SUC_lexicon_tags_11 },
    { 0xccb21489, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f21492, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b21495, SUC_lexicon_tags_1 },
    { 0x74521496, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f32149b, SUC_lexicon_tags_39 },
    { 0x832149c, SUC_lexicon_tags_66 },
    { 0xd852149c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5214a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7214a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f214ab, SUC_lexicon_tags_6 },
    { 0xcc1214ac, SUC_lexicon_tags_26 },
    { 0xbd1214ad, SUC_lexicon_tags_17 },
    { 0xab9214ad, SUC_lexicon_tags_33 },
    { 0xa03214ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197214b3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f214b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x671214ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1214bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd214c5, SUC_lexicon_tags_21 },
    { 0x1e3214c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ed214c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1214cc, SUC_lexicon_tags_32 },
    { 0xa83214cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9214d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5d5214d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b214d5, SUC_lexicon_tags_25 },
    { 0x4e5214d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761214dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47f214de, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed214eb, SUC_lexicon_tags_7 },
    { 0x431214ec, SUC_lexicon_tags_7 },
    { 0x3b5214ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x995214ef, SUC_lexicon_tags_27 },
    { 0x2cd214f0, SUC_lexicon_tags_37 },
    { 0x411214f1, SUC_lexicon_tags_37 },
    { 0x393214f2, SUC_lexicon_tags_30 },
    { 0x477214f1, SUC_lexicon_tags_27 },
    { 0x417214ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50d214f6, SUC_lexicon_tags_14 },
    { 0x10b214f6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83214fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f3214fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63521503, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0721505, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd2150b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa521510, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74721517, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x16121519, SUC_lexicon_tags_37 },
    { 0xe672151a, SUC_lexicon_tags_17 },
    { 0x5d52151b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe312151f, SUC_lexicon_tags_15 },
    { 0xf2f2151f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe921524, SUC_lexicon_tags_46 },
    { 0x8a321525, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9721529, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1892152d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3521531, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7121534, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2521536, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb21539, SUC_lexicon_tags_1 },
    { 0x1232153a, SUC_lexicon_tags_27 },
    { 0x8bd2153b, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b921544, SUC_lexicon_tags_60 },
    { 0xac521545, SUC_lexicon_tags_6 },
    { 0x79d21545, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9572154a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24721551, SUC_lexicon_tags_84 },
    { 0xc2321552, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd121558, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x92f2155a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3772155e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3521561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34721563, SUC_lexicon_tags_11 },
    { 0xd4521564, SUC_lexicon_tags_262 },
    { 0x12721565, SUC_lexicon_tags_77 },
    { 0x1af21566, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb1921568, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f2156e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a521570, SUC_lexicon_tags_23 },
    { 0x61d21571, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21121575, SUC_lexicon_tags_25 },
    { 0xb5321575, SUC_lexicon_tags_27 },
    { 0xd6321575, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd12157f, SUC_lexicon_tags_56 },
    { 0xb0d21580, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f21584, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd8f21586, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac921588, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb932158a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6f2158c, SUC_lexicon_tags_1 },
    { 0xdd72158c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8f92158f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7521592, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f21595, SUC_lexicon_tags_33 },
    { 0xfc321596, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9321599, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8192159c, SUC_lexicon_tags_15 },
    { 0x1c72159d, SUC_lexicon_tags_65 },
    { 0x7952159d, SUC_lexicon_tags_11 },
    { 0xbdb2159d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62f215a1, SUC_lexicon_tags_15 },
    { 0x707215a2, SUC_lexicon_tags_25 },
    { 0xf41215a3, SUC_lexicon_tags_1 },
    { 0x8f7215a4, SUC_lexicon_tags_11 },
    { 0x4df215a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa1215a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f215af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x495215b1, SUC_lexicon_tags_5 },
    { 0x241215b2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1215b5, SUC_lexicon_tags_25 },
    { 0x62b215b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5f5215b8, SUC_lexicon_tags_17 },
    { 0xaf1215b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f215bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8f5215c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf9215c3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x28d215c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95215cc, SUC_lexicon_tags_38 },
    { 0x417215cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x231215cf, SUC_lexicon_tags_17 },
    { 0xf11215d0, SUC_lexicon_tags_45 },
    { 0x333215d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x67d215d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd215dc, SUC_lexicon_tags_32 },
    { 0x777215dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46f215df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf75215e1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341215e6, SUC_lexicon_tags_42 },
    { 0x623215e6, SUC_lexicon_tags_17 },
    { 0x74b215e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9215eb, SUC_lexicon_tags_1 },
    { 0x62f215ec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3215f4, SUC_lexicon_tags_12 },
    { 0xd13215f5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17215fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb19215fd, SUC_lexicon_tags_16 },
    { 0xbb7215fe, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x55721600, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa921603, SUC_lexicon_tags_11 },
    { 0x3f21604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b21607, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf492160a, SUC_lexicon_tags_1310 },
    { 0xb5d2160b, SUC_lexicon_tags_33 },
    { 0xc5b2160c, SUC_lexicon_tags_31 },
    { 0x87f2160d, SUC_lexicon_tags_23 },
    { 0xb2b2160a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8f21610, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6af21612, SUC_lexicon_tags_33 },
    { 0x9a521613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2321615, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf612161a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d521621, SUC_lexicon_tags_56 },
    { 0xf9521622, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98321626, SUC_lexicon_tags_14 },
    { 0x11921627, SUC_lexicon_tags_25 },
    { 0x25f21628, SUC_lexicon_tags_6 },
    { 0x58321629, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2472162c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8fb2162e, SUC_lexicon_tags_37 },
    { 0x5792162f, SUC_lexicon_tags_12 },
    { 0x52f21630, SUC_lexicon_tags_38 },
    { 0x26321631, SUC_lexicon_tags_26 },
    { 0x57b21632, SUC_lexicon_tags_15 },
    { 0xabb21633, SUC_lexicon_tags_15 },
    { 0xf0f21634, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f2163a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd292163d, SUC_lexicon_tags_65 },
    { 0xbd12163e, SUC_lexicon_tags_33 },
    { 0xf752163f, SUC_lexicon_tags_27 },
    { 0x3372163e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2bf21642, SUC_lexicon_tags_30 },
    { 0x1b521643, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d21646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e32164a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f2164d, SUC_lexicon_tags_8 },
    { 0xb32164d, SUC_lexicon_tags_17 },
    { 0xe652164f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3921651, SUC_lexicon_tags_33 },
    { 0x70721651, SUC_lexicon_tags_2 },
    { 0xecb21652, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab2165c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e721661, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cf21663, SUC_lexicon_tags_0 },
    { 0xc7f21663, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8d21666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f321671, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea921674, SUC_lexicon_tags_37 },
    { 0x53b21675, SUC_lexicon_tags_15 },
    { 0x23321675, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52167d, SUC_lexicon_tags_17 },
    { 0xef72167e, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xe2b21680, SUC_lexicon_tags_15 },
    { 0xead21681, SUC_lexicon_tags_11 },
    { 0xc0521682, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb21688, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d21693, SUC_lexicon_tags_55 },
    { 0x28b21694, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2f521696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82521698, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc72169c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38f2169e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe99216a0, SUC_lexicon_tags_12 },
    { 0xff1216a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f216a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f216a9, SUC_lexicon_tags_47 },
    { 0xe03216a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b216ad, SUC_lexicon_tags_8 },
    { 0x7f216ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc7216b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x491216b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1216b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dd216b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7216c2, SUC_lexicon_tags_15 },
    { 0x111216c2, SUC_lexicon_tags_11 },
    { 0x273216c2, SUC_lexicon_tags_12 },
    { 0x9af216c3, SUC_lexicon_tags_17 },
    { 0x615216c6, SUC_lexicon_tags_38 },
    { 0x5d5216c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f216ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb216cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf216cf, SUC_lexicon_tags_27 },
    { 0x823216d0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77216de, SUC_lexicon_tags_17 },
    { 0x3b216de, SUC_lexicon_tags_17 },
    { 0x283216e0, SUC_lexicon_tags_11 },
    { 0xb73216e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d216e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x767216e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d216ec, SUC_lexicon_tags_17 },
    { 0x901216ed, SUC_lexicon_tags_73 },
    { 0x125216ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7216f1, SUC_lexicon_tags_26 },
    { 0x31f216f2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa3f216f4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x457216f6, SUC_lexicon_tags_33 },
    { 0xae5216f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf7216f9, SUC_lexicon_tags_37 },
    { 0x517216fa, SUC_lexicon_tags_43 },
    { 0xf51216fb, SUC_lexicon_tags_26 },
    { 0x6bd216fb, SUC_lexicon_tags_47 },
    { 0x709216fd, SUC_lexicon_tags_63 },
    { 0x2e5216fd, SUC_lexicon_tags_49 },
    { 0x889216fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1721701, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83521706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d21709, SUC_lexicon_tags_37 },
    { 0x2392170a, SUC_lexicon_tags_106 },
    { 0x1f32170a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9192170d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd721710, SUC_lexicon_tags_275 },
    { 0xfab21711, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f21717, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f2171c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb2171f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c921721, SUC_lexicon_tags_37 },
    { 0x27f21722, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8b21724, SUC_lexicon_tags_35 },
    { 0xb5b21725, SUC_lexicon_tags_37 },
    { 0xe7121726, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf9121728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c52172a, SUC_lexicon_tags_84 },
    { 0x3a12172a, SUC_lexicon_tags_34 },
    { 0x6bf2172a, SUC_lexicon_tags_4 },
    { 0x2132172b, SUC_lexicon_tags_65 },
    { 0xc4f2172e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17721732, SUC_lexicon_tags_17 },
    { 0x7af21733, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b21736, SUC_lexicon_tags_17 },
    { 0x9fd21737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b2173c, SUC_lexicon_tags_31 },
    { 0x252173d, SUC_lexicon_tags_19 },
    { 0xda32173e, SUC_lexicon_tags_37 },
    { 0x63b2173e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x58321742, SUC_lexicon_tags_37 },
    { 0xe1521743, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb21748, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d92174a, SUC_lexicon_tags_49 },
    { 0xdb32174b, SUC_lexicon_tags_27 },
    { 0x4292174c, SUC_lexicon_tags_65 },
    { 0x13d2174a, SUC_lexicon_tags_14 },
    { 0xb212174b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f21753, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85f21755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd72175b, SUC_lexicon_tags_1 },
    { 0x5512175b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb812175f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6721762, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49321765, SUC_lexicon_tags_11 },
    { 0x17921766, SUC_lexicon_tags_31 },
    { 0x54521766, SUC_lexicon_tags_498 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d52176d, SUC_lexicon_tags_25 },
    { 0x2152176e, SUC_lexicon_tags_38 },
    { 0xc4d2176e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4321772, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4a121774, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe521777, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28121779, SUC_lexicon_tags_37 },
    { 0xbcf21779, SUC_lexicon_tags_23 },
    { 0xf592177b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60d2177d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d21782, SUC_lexicon_tags_25 },
    { 0x28321782, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd121786, SUC_lexicon_tags_12 },
    { 0x9ab21787, SUC_lexicon_tags_37 },
    { 0x8ed21788, SUC_lexicon_tags_16 },
    { 0x10721789, SUC_lexicon_tags_25 },
    { 0x59121787, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d21794, SUC_lexicon_tags_7 },
    { 0x2f21794, SUC_lexicon_tags_37 },
    { 0x91121796, SUC_lexicon_tags_32 },
    { 0xc6521797, SUC_lexicon_tags_47 },
    { 0x4a921796, SUC_lexicon_tags_46 },
    { 0x9e521798, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d52179c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd2179f, SUC_lexicon_tags_27 },
    { 0xaad217a0, SUC_lexicon_tags_47 },
    { 0x5f217a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1d217a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d217a9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xca7217ab, SUC_lexicon_tags_37 },
    { 0x4a3217ac, SUC_lexicon_tags_12 },
    { 0x625217ac, SUC_lexicon_tags_37 },
    { 0x2a5217ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29217b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb217b6, SUC_lexicon_tags_25 },
    { 0x5df217b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d217ba, SUC_lexicon_tags_13 },
    { 0x8bd217bb, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xe75217bd, SUC_lexicon_tags_44 },
    { 0x9e5217be, SUC_lexicon_tags_25 },
    { 0x27217bf, SUC_lexicon_tags_23 },
    { 0xfdd217c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233217c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x133217c9, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb217cd, SUC_lexicon_tags_1 },
    { 0x99b217ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf39217d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c1217d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa6b217d4, SUC_lexicon_tags_0 },
    { 0xc59217d5, SUC_lexicon_tags_25 },
    { 0x34b217d5, SUC_lexicon_tags_37 },
    { 0x743217d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73b217d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57217e5, SUC_lexicon_tags_33 },
    { 0xdf9217e6, SUC_lexicon_tags_39 },
    { 0x253217e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2b1217e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f217f1, SUC_lexicon_tags_38 },
    { 0x955217f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6d217f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3217f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d217fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c321802, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x41321804, SUC_lexicon_tags_23 },
    { 0x22d21805, SUC_lexicon_tags_12 },
    { 0x81321806, SUC_lexicon_tags_37 },
    { 0x47921806, SUC_lexicon_tags_16 },
    { 0x46121804, SUC_lexicon_tags_68 },
    { 0xefb21809, SUC_lexicon_tags_1 },
    { 0xe392180a, SUC_lexicon_tags_55 },
    { 0xac32180b, SUC_lexicon_tags_46 },
    { 0x40f2180b, SUC_lexicon_tags_12 },
    { 0x8b72180d, SUC_lexicon_tags_9 },
    { 0x1db2180e, SUC_lexicon_tags_37 },
    { 0x6b32180f, SUC_lexicon_tags_28 },
    { 0xb9321809, SUC_lexicon_tags_3 },
    { 0xb2b21811, SUC_lexicon_tags_82 },
    { 0x6b721812, SUC_lexicon_tags_7 },
    { 0xcf521806, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad21819, SUC_lexicon_tags_17 },
    { 0xfb721819, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xac72181c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d21820, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x97921822, SUC_lexicon_tags_11 },
    { 0xfb721822, SUC_lexicon_tags_37 },
    { 0x3db21824, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df21827, SUC_lexicon_tags_10 },
    { 0xe2921827, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a32182d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf21831, SUC_lexicon_tags_2 },
    { 0xfab21831, SUC_lexicon_tags_11 },
    { 0x7fb21832, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0921836, SUC_lexicon_tags_37 },
    { 0xc1321837, SUC_lexicon_tags_12 },
    { 0x26321838, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b32183e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3b21840, SUC_lexicon_tags_8 },
    { 0x2af21840, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e521847, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4db21849, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4f2184b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0f2184d, SUC_lexicon_tags_2 },
    { 0x1a52184d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca921855, SUC_lexicon_tags_39 },
    { 0xd7521855, SUC_lexicon_tags_23 },
    { 0x42321857, SUC_lexicon_tags_25 },
    { 0xbd21858, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732185c, SUC_lexicon_tags_6 },
    { 0x22f2185d, SUC_lexicon_tags_12 },
    { 0xcfd2185c, SUC_lexicon_tags_7 },
    { 0x9e92185f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f21864, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1e921866, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd21869, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f12186f, SUC_lexicon_tags_17 },
    { 0x5c721870, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8321876, SUC_lexicon_tags_61 },
    { 0xe5721877, SUC_lexicon_tags_11 },
    { 0x7d721878, SUC_lexicon_tags_25 },
    { 0x9eb21879, SUC_lexicon_tags_17 },
    { 0x4fb2187a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc152187d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d21881, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4921883, SUC_lexicon_tags_21 },
    { 0xce921884, SUC_lexicon_tags_38 },
    { 0x58b21885, SUC_lexicon_tags_15 },
    { 0x3af21884, SUC_lexicon_tags_11 },
    { 0x70321886, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3392188a, SUC_lexicon_tags_47 },
    { 0x54b2188b, SUC_lexicon_tags_2 },
    { 0x792188b, SUC_lexicon_tags_17 },
    { 0x50f2188d, SUC_lexicon_tags_1 },
    { 0x1f2188c, SUC_lexicon_tags_17 },
    { 0x6df2188f, SUC_lexicon_tags_26 },
    { 0x40b21890, SUC_lexicon_tags_38 },
    { 0x5692188f, SUC_lexicon_tags_47 },
    { 0x7b12188d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e721896, SUC_lexicon_tags_34 },
    { 0x15f21896, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a121899, SUC_lexicon_tags_17 },
    { 0xa152189a, SUC_lexicon_tags_11 },
    { 0x20521899, SUC_lexicon_tags_27 },
    { 0x62b2189c, SUC_lexicon_tags_1 },
    { 0x33f2189d, SUC_lexicon_tags_17 },
    { 0x9d52189d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d218a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x583218a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7218a9, SUC_lexicon_tags_60 },
    { 0x2d3218aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91218af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x917218b1, SUC_lexicon_tags_25 },
    { 0x5eb218b1, SUC_lexicon_tags_2 },
    { 0xe3b218b3, SUC_lexicon_tags_146 },
    { 0x91218b4, SUC_lexicon_tags_10 },
    { 0xbb3218b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x701218b7, SUC_lexicon_tags_7 },
    { 0x7df218b8, SUC_lexicon_tags_19 },
    { 0xed9218b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d218bd, SUC_lexicon_tags_79 },
    { 0x257218be, SUC_lexicon_tags_45 },
    { 0x2f218bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f218c3, SUC_lexicon_tags_31 },
    { 0xeb218c4, SUC_lexicon_tags_38 },
    { 0x173218c3, SUC_lexicon_tags_23 },
    { 0xa71218c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697218ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57d218cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0b218ce, SUC_lexicon_tags_23 },
    { 0xd0b218ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d218d4, SUC_lexicon_tags_63 },
    { 0x967218d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d218d9, SUC_lexicon_tags_25 },
    { 0x553218da, SUC_lexicon_tags_11 },
    { 0x4e3218d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd218de, SUC_lexicon_tags_11 },
    { 0x1d9218df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cf218e1, SUC_lexicon_tags_10 },
    { 0x5e1218e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xae1218e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57218e9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x165218eb, SUC_lexicon_tags_100 },
    { 0xfb5218ec, SUC_lexicon_tags_27 },
    { 0x495218ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93218f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x39218f3, SUC_lexicon_tags_27 },
    { 0x94b218f4, SUC_lexicon_tags_32 },
    { 0x5bf218f4, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xe0f218f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3218fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91721901, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c721905, SUC_lexicon_tags_60 },
    { 0xa1f21906, SUC_lexicon_tags_28 },
    { 0x15921907, SUC_lexicon_tags_7 },
    { 0xd9f21905, SUC_lexicon_tags_37 },
    { 0xd4721905, SUC_lexicon_tags_12 },
    { 0xab32190a, SUC_lexicon_tags_25 },
    { 0x65b2190a, SUC_lexicon_tags_37 },
    { 0x98921907, SUC_lexicon_tags_1 },
    { 0xfc921906, SUC_lexicon_tags_62 },
    { 0xc692190d, SUC_lexicon_tags_25 },
    { 0x6592190f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d21912, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x18321914, SUC_lexicon_tags_84 },
    { 0x1cd21915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf321917, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2192191c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe652191e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76121926, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d2192a, SUC_lexicon_tags_37 },
    { 0x8cb2192a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7bb2192d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda921931, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc721939, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f21940, SUC_lexicon_tags_25 },
    { 0x32f21941, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae321944, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c321947, SUC_lexicon_tags_33 },
    { 0xf3921947, SUC_lexicon_tags_21 },
    { 0x97921949, SUC_lexicon_tags_15 },
    { 0xee12194a, SUC_lexicon_tags_34 },
    { 0x21b2194b, SUC_lexicon_tags_77 },
    { 0x7652194c, SUC_lexicon_tags_108 },
    { 0x59f2194c, SUC_lexicon_tags_37 },
    { 0x3f52194e, SUC_lexicon_tags_4 },
    { 0x3bf2194f, SUC_lexicon_tags_70 },
    { 0xd712194e, SUC_lexicon_tags_23 },
    { 0x4cf21951, SUC_lexicon_tags_518 },
    { 0x8232194e, SUC_lexicon_tags_31 },
    { 0x74921949, SUC_lexicon_tags_31 },
    { 0xe7f21954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33d21956, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2152195a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d72195d, SUC_lexicon_tags_70 },
    { 0x5172195e, SUC_lexicon_tags_37 },
    { 0x8d2195f, SUC_lexicon_tags_25 },
    { 0xab92195d, SUC_lexicon_tags_10 },
    { 0x27d21960, SUC_lexicon_tags_25 },
    { 0x8f121962, SUC_lexicon_tags_13 },
    { 0x22921963, SUC_lexicon_tags_60 },
    { 0xbfd21964, SUC_lexicon_tags_23 },
    { 0xf3f21965, SUC_lexicon_tags_29 },
    { 0x9b721966, SUC_lexicon_tags_23 },
    { 0xdf121967, SUC_lexicon_tags_47 },
    { 0x1f21968, SUC_lexicon_tags_17 },
    { 0x83b21969, SUC_lexicon_tags_25 },
    { 0x8e321964, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d21970, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b21973, SUC_lexicon_tags_17 },
    { 0xf9f21974, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8321978, SUC_lexicon_tags_11 },
    { 0xcad21979, SUC_lexicon_tags_29 },
    { 0x1f921978, SUC_lexicon_tags_14 },
    { 0x43121979, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93321986, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b2198c, SUC_lexicon_tags_12 },
    { 0x3672198c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb921994, SUC_lexicon_tags_6 },
    { 0x1d521995, SUC_lexicon_tags_25 },
    { 0x12721995, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d2199a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a12199d, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0x1552199f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x251219a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9219ab, SUC_lexicon_tags_15 },
    { 0x94b219ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17219af, SUC_lexicon_tags_63 },
    { 0xc1d219af, SUC_lexicon_tags_25 },
    { 0xd43219b1, SUC_lexicon_tags_0 },
    { 0x925219b2, SUC_lexicon_tags_7 },
    { 0xe95219b3, SUC_lexicon_tags_15 },
    { 0xf51219b3, SUC_lexicon_tags_11 },
    { 0xee1219b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9eb219b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x261219ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82d219bc, SUC_lexicon_tags_36 },
    { 0x2e5219bd, SUC_lexicon_tags_46 },
    { 0x769219bd, SUC_lexicon_tags_28 },
    { 0x63b219bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf219c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x115219c7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75219ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5219cd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd219d0, SUC_lexicon_tags_47 },
    { 0x225219d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3219d4, SUC_lexicon_tags_23 },
    { 0xccb219d4, SUC_lexicon_tags_17 },
    { 0xf13219d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f219db, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1219e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x817219e3, SUC_lexicon_tags_15 },
    { 0xb57219e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9219e7, SUC_lexicon_tags_29 },
    { 0x119219e8, SUC_lexicon_tags_17 },
    { 0x8f1219e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93219ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d219f2, SUC_lexicon_tags_46 },
    { 0xa33219f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd219f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b219fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9521a02, SUC_lexicon_tags_15 },
    { 0x9e721a03, SUC_lexicon_tags_11 },
    { 0x3e121a03, SUC_lexicon_tags_11 },
    { 0x2dd21a02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c121a0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f21a0f, SUC_lexicon_tags_49 },
    { 0x62321a0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed21a14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf921a16, SUC_lexicon_tags_387 },
    { 0xa3721a17, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xee721a19, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6b721a1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34121a1e, SUC_lexicon_tags_25 },
    { 0xe521a1e, SUC_lexicon_tags_25 },
    { 0x5c521a20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e721a23, SUC_lexicon_tags_2 },
    { 0x9a321a24, SUC_lexicon_tags_47 },
    { 0x46121a25, SUC_lexicon_tags_23 },
    { 0x1fb21a24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb4d21a28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34721a2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9321a2f, SUC_lexicon_tags_8 },
    { 0x8ed21a30, SUC_lexicon_tags_43 },
    { 0xc6f21a2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29521a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b21a39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4d21a3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed721a41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c721a45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a921a49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51921a4b, SUC_lexicon_tags_37 },
    { 0x4af21a4c, SUC_lexicon_tags_49 },
    { 0x47121a4d, SUC_lexicon_tags_33 },
    { 0x9fb21a4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40921a50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f21a53, SUC_lexicon_tags_14 },
    { 0x44f21a54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcc521a56, SUC_lexicon_tags_6 },
    { 0xf7b21a57, SUC_lexicon_tags_9 },
    { 0x41121a58, SUC_lexicon_tags_12 },
    { 0x87121a59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89121a5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd721a61, SUC_lexicon_tags_11 },
    { 0xe9f21a61, SUC_lexicon_tags_7 },
    { 0xab921a63, SUC_lexicon_tags_23 },
    { 0x4e721a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9121a67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f21a6b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x59521a6e, SUC_lexicon_tags_12 },
    { 0x1bf21a6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98921a73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f21a7e, SUC_lexicon_tags_84 },
    { 0xc6721a7e, SUC_lexicon_tags_17 },
    { 0xd5d21a80, SUC_lexicon_tags_12 },
    { 0xc1721a7e, SUC_lexicon_tags_14 },
    { 0xae121a82, SUC_lexicon_tags_65 },
    { 0xf2d21a82, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x50521a85, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2721a8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb721a8c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xea321a8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b21a91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe321a95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8b21a97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9ab21a99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc521aa4, SUC_lexicon_tags_23 },
    { 0xe8b21aa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x80121aa8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc4721aaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2121ab0, SUC_lexicon_tags_8 },
    { 0x50b21ab1, SUC_lexicon_tags_8 },
    { 0x99f21ab1, SUC_lexicon_tags_32 },
    { 0x9a521ab2, SUC_lexicon_tags_45 },
    { 0xa0921ab3, SUC_lexicon_tags_17 },
    { 0x4721ab2, SUC_lexicon_tags_30 },
    { 0x89921ab1, SUC_lexicon_tags_820 },
    { 0x3f21ab7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b21abb, SUC_lexicon_tags_7 },
    { 0x9f321abc, SUC_lexicon_tags_25 },
    { 0x3e721abb, SUC_lexicon_tags_7 },
    { 0xa1921abe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e521ac4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f21ac9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83d21acb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x52b21acd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa0b21acf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce121ad6, SUC_lexicon_tags_0 },
    { 0x54921ad7, SUC_lexicon_tags_1 },
    { 0x73f21ad8, SUC_lexicon_tags_27 },
    { 0xe1321ad9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc721add, SUC_lexicon_tags_15 },
    { 0x8d521ade, SUC_lexicon_tags_63 },
    { 0x3a921adf, SUC_lexicon_tags_11 },
    { 0xe4921ae0, SUC_lexicon_tags_30 },
    { 0x29921add, SUC_lexicon_tags_17 },
    { 0xf3321adf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52721ae4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7b21ae6, SUC_lexicon_tags_1 },
    { 0xc2d21ae7, SUC_lexicon_tags_37 },
    { 0x6cb21ae7, SUC_lexicon_tags_1 },
    { 0x61321ae9, SUC_lexicon_tags_42 },
    { 0xbcb21ae8, SUC_lexicon_tags_37 },
    { 0xcc321ae8, SUC_lexicon_tags_55 },
    { 0xa7121aec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d521af1, SUC_lexicon_tags_9 },
    { 0x99121af2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b21af5, SUC_lexicon_tags_32 },
    { 0x1e321af6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d21afb, SUC_lexicon_tags_63 },
    { 0x8db21afb, SUC_lexicon_tags_26 },
    { 0xebd21afb, SUC_lexicon_tags_37 },
    { 0x35b21afc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x96121b00, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7c121b02, SUC_lexicon_tags_60 },
    { 0x96321b02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f21b08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b721b0f, SUC_lexicon_tags_21 },
    { 0xb9321b10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe721b17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x12d21b19, SUC_lexicon_tags_15 },
    { 0x2ff21b1a, SUC_lexicon_tags_23 },
    { 0xaa721b1b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3121b1e, SUC_lexicon_tags_40 },
    { 0x90f21b1f, SUC_lexicon_tags_17 },
    { 0x31521b1f, SUC_lexicon_tags_11 },
    { 0x33d21b20, SUC_lexicon_tags_44 },
    { 0x32b21b22, SUC_lexicon_tags_38 },
    { 0xb21b23, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcbb21b25, SUC_lexicon_tags_2 },
    { 0xbd721b26, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x12721b29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2121b2c, SUC_lexicon_tags_37 },
    { 0xfd721b2d, SUC_lexicon_tags_17 },
    { 0x8a921b2e, SUC_lexicon_tags_82 },
    { 0x49d21b2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7121b32, SUC_lexicon_tags_14 },
    { 0xfd321b33, SUC_lexicon_tags_27 },
    { 0x31321b32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b21b38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d21b3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf721b3d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d721b41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9921b43, SUC_lexicon_tags_26 },
    { 0x79921b44, SUC_lexicon_tags_1 },
    { 0xb2321b44, SUC_lexicon_tags_49 },
    { 0x7121b46, SUC_lexicon_tags_27 },
    { 0xf5721b43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86521b4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b21b50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10121b52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b21b57, SUC_lexicon_tags_17 },
    { 0xd4121b58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a921b62, SUC_lexicon_tags_68 },
    { 0x6fd21b63, SUC_lexicon_tags_35 },
    { 0xc9121b64, SUC_lexicon_tags_18 },
    { 0x25921b64, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0x37721b67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f521b6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4521b6c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f21b70, SUC_lexicon_tags_25 },
    { 0xa4f21b71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48521b76, SUC_lexicon_tags_1 },
    { 0x75b21b77, SUC_lexicon_tags_262 },
    { 0xa4321b78, SUC_lexicon_tags_37 },
    { 0x3f121b78, SUC_lexicon_tags_2 },
    { 0xb2b21b7a, SUC_lexicon_tags_42 },
    { 0x75921b7b, SUC_lexicon_tags_29 },
    { 0x36721b7b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f21b81, SUC_lexicon_tags_23 },
    { 0xe6921b82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8121b86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2bd21b88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69521b8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54321b8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95721b97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f21b9b, SUC_lexicon_tags_7 },
    { 0x41b21b9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf21b9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f21ba2, SUC_lexicon_tags_28 },
    { 0x2321ba2, SUC_lexicon_tags_39 },
    { 0x18d21ba4, SUC_lexicon_tags_37 },
    { 0xae921ba3, SUC_lexicon_tags_44 },
    { 0xf4f21ba6, SUC_lexicon_tags_7 },
    { 0x3c721ba2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda921bab, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f21bb2, SUC_lexicon_tags_7 },
    { 0x34721bb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b21bb6, SUC_lexicon_tags_21 },
    { 0x86921bb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe0521bb9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8121bbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95721bc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a721bcc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5521bd6, SUC_lexicon_tags_68 },
    { 0x8af21bd7, SUC_lexicon_tags_1 },
    { 0xfb121bd6, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0x84f21bda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb921bdc, SUC_lexicon_tags_14 },
    { 0xb6f21bdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe8921bdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d21be3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5b121be5, SUC_lexicon_tags_25 },
    { 0xd2f21be6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b21be9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf521beb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54521bee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29321bf4, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb521bfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3b21bfe, SUC_lexicon_tags_37 },
    { 0xf4121bff, SUC_lexicon_tags_11 },
    { 0x28f21bff, SUC_lexicon_tags_53 },
    { 0x12921c01, SUC_lexicon_tags_140 },
    { 0x9d921c01, SUC_lexicon_tags_15 },
    { 0xfa321bff, SUC_lexicon_tags_19 },
    { 0xe1921c04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84321c08, SUC_lexicon_tags_43 },
    { 0x29d21c08, SUC_lexicon_tags_12 },
    { 0x1d121c0a, SUC_lexicon_tags_11 },
    { 0xd9521c0b, SUC_lexicon_tags_1 },
    { 0xadf21c0c, SUC_lexicon_tags_25 },
    { 0x9ab21c0d, SUC_lexicon_tags_15 },
    { 0xd6b21c0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a921c11, SUC_lexicon_tags_37 },
    { 0xedf21c12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5df21c14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9cf21c16, SUC_lexicon_tags_25 },
    { 0x2921c17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52b21c19, SUC_lexicon_tags_26 },
    { 0xa3921c1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64921c1f, SUC_lexicon_tags_7 },
    { 0x1bb21c20, SUC_lexicon_tags_15 },
    { 0xa8721c21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9d21c23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1cd21c25, SUC_lexicon_tags_17 },
    { 0x63d21c25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcff21c28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d21c2e, SUC_lexicon_tags_30 },
    { 0x8ab21c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54921c32, SUC_lexicon_tags_17 },
    { 0xfcb21c33, SUC_lexicon_tags_31 },
    { 0x5e721c34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d21c3a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d21c3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2f21c3f, SUC_lexicon_tags_28 },
    { 0x25321c3f, SUC_lexicon_tags_12 },
    { 0xf6321c41, SUC_lexicon_tags_1 },
    { 0x69f21c42, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd321c45, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca321c5a, SUC_lexicon_tags_38 },
    { 0x4ad21c5b, SUC_lexicon_tags_37 },
    { 0xe9521c5c, SUC_lexicon_tags_1 },
    { 0x53521c5d, SUC_lexicon_tags_6 },
    { 0x93f21c5d, SUC_lexicon_tags_43 },
    { 0xb521c5d, SUC_lexicon_tags_43 },
    { 0x86f21c60, SUC_lexicon_tags_37 },
    { 0x78b21c61, SUC_lexicon_tags_7 },
    { 0xd8721c62, SUC_lexicon_tags_25 },
    { 0x86b21c62, SUC_lexicon_tags_28 },
    { 0x65921c62, SUC_lexicon_tags_37 },
    { 0xa3521c65, SUC_lexicon_tags_37 },
    { 0x6d21c66, SUC_lexicon_tags_30 },
    { 0xe8721c67, SUC_lexicon_tags_27 },
    { 0xbe121c68, SUC_lexicon_tags_15 },
    { 0x5b321c66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9721c78, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce921c81, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7c721c83, SUC_lexicon_tags_17 },
    { 0xac521c84, SUC_lexicon_tags_37 },
    { 0xa1721c84, SUC_lexicon_tags_11 },
    { 0x45f21c84, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x87921c89, SUC_lexicon_tags_29 },
    { 0xaeb21c8a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18521c8e, SUC_lexicon_tags_29 },
    { 0x97721c8f, SUC_lexicon_tags_25 },
    { 0xd5721c8f, SUC_lexicon_tags_7 },
    { 0xa0d21c91, SUC_lexicon_tags_2 },
    { 0x5b121c91, SUC_lexicon_tags_1 },
    { 0xad121c93, SUC_lexicon_tags_23 },
    { 0x47321c94, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x32721c96, SUC_lexicon_tags_33 },
    { 0xe5121c97, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b321c9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81321c9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64121ca7, SUC_lexicon_tags_25 },
    { 0x1721ca8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x50321cab, SUC_lexicon_tags_8 },
    { 0xf5721cac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13521caf, SUC_lexicon_tags_23 },
    { 0x3d321cb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18321cb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68321cb7, SUC_lexicon_tags_9 },
    { 0x2e521cb8, SUC_lexicon_tags_60 },
    { 0xf7321cb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6121cbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9921cc2, SUC_lexicon_tags_27 },
    { 0x55d21cc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86321cc5, SUC_lexicon_tags_15 },
    { 0xd0d21cc6, SUC_lexicon_tags_1 },
    { 0x16321cc7, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x87321cca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb121ccd, SUC_lexicon_tags_2 },
    { 0x73d21ccd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f21cd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a121cd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad21cde, SUC_lexicon_tags_11 },
    { 0xe0521cdf, SUC_lexicon_tags_7 },
    { 0x66d21ce0, SUC_lexicon_tags_12 },
    { 0xb1921ce1, SUC_lexicon_tags_23 },
    { 0xe8f21ce2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25721ce4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8121ce7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b21cea, SUC_lexicon_tags_1 },
    { 0x8b921cea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56721cf0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b321cf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f721d01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5921d07, SUC_lexicon_tags_47 },
    { 0xef521d08, SUC_lexicon_tags_19 },
    { 0x7521d09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e921d0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d21d11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfc721d13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85721d18, SUC_lexicon_tags_8 },
    { 0x54121d18, SUC_lexicon_tags_19 },
    { 0xc3921d18, SUC_lexicon_tags_10 },
    { 0x98521d1b, SUC_lexicon_tags_1 },
    { 0xf6d21d1c, SUC_lexicon_tags_60 },
    { 0x64921d1b, SUC_lexicon_tags_45 },
    { 0xfcb21d1e, SUC_lexicon_tags_1 },
    { 0x60921d1f, SUC_lexicon_tags_23 },
    { 0x9721d20, SUC_lexicon_tags_19 },
    { 0x43b21d1f, SUC_lexicon_tags_1 },
    { 0xab321d22, SUC_lexicon_tags_1 },
    { 0x8f21d20, SUC_lexicon_tags_6 },
    { 0x19d21d22, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x60321d27, SUC_lexicon_tags_27 },
    { 0xa2321d28, SUC_lexicon_tags_61 },
    { 0xcdd21d29, SUC_lexicon_tags_40 },
    { 0xa5d21d29, SUC_lexicon_tags_30 },
    { 0x77121d2b, SUC_lexicon_tags_55 },
    { 0x7fb21d27, SUC_lexicon_tags_47 },
    { 0xe9b21d28, SUC_lexicon_tags_30 },
    { 0x30121d2e, SUC_lexicon_tags_37 },
    { 0x7ff21d2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d21d34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb321d39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a121d3c, SUC_lexicon_tags_14 },
    { 0xf8f21d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19b21d3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d21d42, SUC_lexicon_tags_35 },
    { 0x87521d42, SUC_lexicon_tags_37 },
    { 0x82d21d44, SUC_lexicon_tags_37 },
    { 0xb8f21d45, SUC_lexicon_tags_2 },
    { 0xac321d45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9121d4a, SUC_lexicon_tags_37 },
    { 0xb2721d4b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f21d51, SUC_lexicon_tags_11 },
    { 0x5ef21d51, SUC_lexicon_tags_49 },
    { 0x8d521d53, SUC_lexicon_tags_37 },
    { 0x87f21d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bd21d56, SUC_lexicon_tags_7 },
    { 0xe1121d56, SUC_lexicon_tags_11 },
    { 0xae921d58, SUC_lexicon_tags_25 },
    { 0x54321d58, SUC_lexicon_tags_37 },
    { 0xb7d21d56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f21d5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2321d64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d121d6c, SUC_lexicon_tags_19 },
    { 0x60d21d6d, SUC_lexicon_tags_1 },
    { 0x7d321d6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6321d70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81921d76, SUC_lexicon_tags_26 },
    { 0xfcf21d77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75521d7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab21d80, SUC_lexicon_tags_15 },
    { 0x25921d81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60b21d83, SUC_lexicon_tags_49 },
    { 0x21921d84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb21d87, SUC_lexicon_tags_23 },
    { 0xadf21d88, SUC_lexicon_tags_17 },
    { 0x2a321d88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89321d8f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0321d93, SUC_lexicon_tags_65 },
    { 0x4a721d93, SUC_lexicon_tags_28 },
    { 0x41f21d95, SUC_lexicon_tags_16 },
    { 0xe9d21d95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c121d98, SUC_lexicon_tags_23 },
    { 0xbd721d99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed921d9b, SUC_lexicon_tags_60 },
    { 0xf6b21d9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b21da2, SUC_lexicon_tags_2 },
    { 0x2dd21da3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48121da5, SUC_lexicon_tags_65 },
    { 0xaeb21da5, SUC_lexicon_tags_11 },
    { 0x9c321da6, SUC_lexicon_tags_37 },
    { 0xe4b21da8, SUC_lexicon_tags_31 },
    { 0xdc121da9, SUC_lexicon_tags_7 },
    { 0x6f521daa, SUC_lexicon_tags_11 },
    { 0xaff21da5, SUC_lexicon_tags_17 },
    { 0x9cf21dac, SUC_lexicon_tags_7 },
    { 0x21d21dad, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x5521db0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x37721db2, SUC_lexicon_tags_17 },
    { 0xcd521db3, SUC_lexicon_tags_149 },
    { 0x3e321db4, SUC_lexicon_tags_57 },
    { 0x43521db2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77521db9, SUC_lexicon_tags_32 },
    { 0x64721db9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfd21dbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10721dbe, SUC_lexicon_tags_7 },
    { 0x67b21dbf, SUC_lexicon_tags_1 },
    { 0xfad21dc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2f21dc2, SUC_lexicon_tags_37 },
    { 0x6f321dc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f921dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81121dcc, SUC_lexicon_tags_69 },
    { 0x7c521dcd, SUC_lexicon_tags_0 },
    { 0x3721dce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74b21dd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f21dd4, SUC_lexicon_tags_1 },
    { 0x2321dd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74321dd8, SUC_lexicon_tags_7 },
    { 0x91521dd9, SUC_lexicon_tags_14 },
    { 0x65121dda, SUC_lexicon_tags_29 },
    { 0xba921ddb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93921ddf, SUC_lexicon_tags_2 },
    { 0xad321de0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db21de5, SUC_lexicon_tags_25 },
    { 0x51d21de6, SUC_lexicon_tags_82 },
    { 0x8a321de7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7121de9, SUC_lexicon_tags_43 },
    { 0xfbd21dea, SUC_lexicon_tags_37 },
    { 0x95721de9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47321def, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad21df4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd21df7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b21dfa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91321dff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e321e04, SUC_lexicon_tags_43 },
    { 0xce321e05, SUC_lexicon_tags_2 },
    { 0xb121e05, SUC_lexicon_tags_15 },
    { 0xca321e07, SUC_lexicon_tags_38 },
    { 0x20921e06, SUC_lexicon_tags_11 },
    { 0xdef21e09, SUC_lexicon_tags_9 },
    { 0x66921e06, SUC_lexicon_tags_13 },
    { 0x94d21e0a, SUC_lexicon_tags_12 },
    { 0x4b521e09, SUC_lexicon_tags_14 },
    { 0x9f321e0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b21e10, SUC_lexicon_tags_15 },
    { 0x39521e11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc521e14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2921e16, SUC_lexicon_tags_47 },
    { 0x60521e17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f721e19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0121e1c, SUC_lexicon_tags_2 },
    { 0x87f21e1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6121e31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd2f21e33, SUC_lexicon_tags_1 },
    { 0xe521e34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3921e36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21721e38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70f21e3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5721e3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7f21e40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e921e43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x88921e45, SUC_lexicon_tags_45 },
    { 0xb721e45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xed721e48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7321e4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bd21e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61b21e4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d21e51, SUC_lexicon_tags_15 },
    { 0x40d21e52, SUC_lexicon_tags_11 },
    { 0x21d21e51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30521e57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71721e5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12521e63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6321e67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9b21e69, SUC_lexicon_tags_1 },
    { 0x2d521e6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9f21e6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d21e70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55b21e72, SUC_lexicon_tags_17 },
    { 0xcff21e73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0921e77, SUC_lexicon_tags_53 },
    { 0xbf121e78, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33121e7f, SUC_lexicon_tags_26 },
    { 0x72521e80, SUC_lexicon_tags_7 },
    { 0x2b521e81, SUC_lexicon_tags_124 },
    { 0xe1121e82, SUC_lexicon_tags_43 },
    { 0x4bb21e81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab21e87, SUC_lexicon_tags_15 },
    { 0x5521e88, SUC_lexicon_tags_15 },
    { 0xddd21e89, SUC_lexicon_tags_17 },
    { 0x5f321e8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d21e8e, SUC_lexicon_tags_45 },
    { 0xf3d21e8f, SUC_lexicon_tags_48 },
    { 0x77321e90, SUC_lexicon_tags_82 },
    { 0x6f321e90, SUC_lexicon_tags_47 },
    { 0x5c321e92, SUC_lexicon_tags_37 },
    { 0xf3121e93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d21e97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28d21e99, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x71921e9b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff21e9f, SUC_lexicon_tags_4 },
    { 0xcaf21ea0, SUC_lexicon_tags_33 },
    { 0x1321ea1, SUC_lexicon_tags_33 },
    { 0x7e521e9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe121ea8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb21eae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52321eb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79521eb6, SUC_lexicon_tags_16 },
    { 0xac321eb7, SUC_lexicon_tags_1 },
    { 0xa4321eb8, SUC_lexicon_tags_33 },
    { 0x6fb21eb8, SUC_lexicon_tags_14 },
    { 0x16321eb8, SUC_lexicon_tags_0 },
    { 0xcbf21ebb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad721ebd, SUC_lexicon_tags_31 },
    { 0x8d521ebe, SUC_lexicon_tags_40 },
    { 0x3ef21ebd, SUC_lexicon_tags_60 },
    { 0x6121ec0, SUC_lexicon_tags_15 },
    { 0x94721ec1, SUC_lexicon_tags_14 },
    { 0x88321ec0, SUC_lexicon_tags_7 },
    { 0x86721ec2, SUC_lexicon_tags_28 },
    { 0x5f121ec4, SUC_lexicon_tags_17 },
    { 0x10d21ec2, SUC_lexicon_tags_11 },
    { 0xf8521ec0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab21ec9, SUC_lexicon_tags_23 },
    { 0x83321eca, SUC_lexicon_tags_34 },
    { 0xddb21eca, SUC_lexicon_tags_17 },
    { 0x2ab21ec9, SUC_lexicon_tags_31 },
    { 0x29f21ecd, SUC_lexicon_tags_77 },
    { 0x67921ece, SUC_lexicon_tags_12 },
    { 0xeb521ecf, SUC_lexicon_tags_11 },
    { 0xdf521ecf, SUC_lexicon_tags_81 },
    { 0xb5121eca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc321ed3, SUC_lexicon_tags_28 },
    { 0x5cf21ed3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5f21ed6, SUC_lexicon_tags_17 },
    { 0xa7121ed6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed921edf, SUC_lexicon_tags_31 },
    { 0xa9b21ee0, SUC_lexicon_tags_7 },
    { 0x8bd21ee1, SUC_lexicon_tags_31 },
    { 0xf8d21ee2, SUC_lexicon_tags_79 },
    { 0xfa921ee1, SUC_lexicon_tags_7 },
    { 0x60321ee2, SUC_lexicon_tags_47 },
    { 0x32121ee5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b21ee8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb521eea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5921eec, SUC_lexicon_tags_0 },
    { 0xfdb21eec, SUC_lexicon_tags_33 },
    { 0x5f121eee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6921ef1, SUC_lexicon_tags_61 },
    { 0x8c321ef2, SUC_lexicon_tags_37 },
    { 0xc3f21ef3, SUC_lexicon_tags_37 },
    { 0x9d521ef3, SUC_lexicon_tags_37 },
    { 0x36121ef5, SUC_lexicon_tags_41 },
    { 0xb4d21ef5, SUC_lexicon_tags_39 },
    { 0xfb721ef3, SUC_lexicon_tags_12 },
    { 0xacd21ef3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1321efa, SUC_lexicon_tags_17 },
    { 0x1af21efb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b21efe, SUC_lexicon_tags_1 },
    { 0xa1921efe, SUC_lexicon_tags_12 },
    { 0x40121f00, SUC_lexicon_tags_29 },
    { 0x47921efe, SUC_lexicon_tags_15 },
    { 0x1eb21eff, SUC_lexicon_tags_60 },
    { 0xa3321f02, SUC_lexicon_tags_37 },
    { 0x64921f04, SUC_lexicon_tags_142 },
    { 0x2e321f05, SUC_lexicon_tags_30 },
    { 0xfb921eff, SUC_lexicon_tags_37 },
    { 0x4ff21f02, SUC_lexicon_tags_25 },
    { 0xbd721f01, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfc921f0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef21f10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd21f17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fd21f19, SUC_lexicon_tags_63 },
    { 0x71121f1a, SUC_lexicon_tags_39 },
    { 0x8b921f1b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe0121f1d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa721f20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d921f24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb721f26, SUC_lexicon_tags_29 },
    { 0xf9d21f26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27d21f29, SUC_lexicon_tags_243 },
    { 0x32321f29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf21f34, SUC_lexicon_tags_31 },
    { 0xf1f21f35, SUC_lexicon_tags_12 },
    { 0x38321f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b21f41, SUC_lexicon_tags_87 },
    { 0x2dd21f42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f21f45, SUC_lexicon_tags_16 },
    { 0x40721f46, SUC_lexicon_tags_7 },
    { 0xf2b21f47, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5521f4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65721f52, SUC_lexicon_tags_49 },
    { 0x9ff21f53, SUC_lexicon_tags_10 },
    { 0x16521f52, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb521f57, SUC_lexicon_tags_21 },
    { 0x18321f57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f21f5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdd921f5d, SUC_lexicon_tags_1 },
    { 0xff721f5e, SUC_lexicon_tags_12 },
    { 0x32321f5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58521f62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b21f68, SUC_lexicon_tags_11 },
    { 0xcc521f69, SUC_lexicon_tags_12 },
    { 0xb1f21f69, SUC_lexicon_tags_17 },
    { 0xd5321f6b, SUC_lexicon_tags_37 },
    { 0x9e521f6c, SUC_lexicon_tags_19 },
    { 0x87b21f6c, SUC_lexicon_tags_27 },
    { 0x73b21f69, SUC_lexicon_tags_2 },
    { 0x86521f6e, SUC_lexicon_tags_25 },
    { 0xd6121f6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb21f73, SUC_lexicon_tags_1 },
    { 0x7f21f74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x56d21f76, SUC_lexicon_tags_1 },
    { 0xcbd21f77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0721f7a, SUC_lexicon_tags_0 },
    { 0xfc321f7a, SUC_lexicon_tags_1 },
    { 0xd4f21f7c, SUC_lexicon_tags_106 },
    { 0xa3921f7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4721f7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7121f81, SUC_lexicon_tags_28 },
    { 0x51d21f82, SUC_lexicon_tags_19 },
    { 0xf1d21f82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17721f87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48721f8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x921f8d, SUC_lexicon_tags_17 },
    { 0x2ab21f8e, SUC_lexicon_tags_63 },
    { 0x71121f8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d721f92, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8d521f94, SUC_lexicon_tags_47 },
    { 0xa8b21f95, SUC_lexicon_tags_7 },
    { 0x34521f96, SUC_lexicon_tags_38 },
    { 0xf6921f96, SUC_lexicon_tags_2 },
    { 0xc0121f98, SUC_lexicon_tags_11 },
    { 0x69f21f98, SUC_lexicon_tags_7 },
    { 0x8f521f99, SUC_lexicon_tags_16 },
    { 0x77721f96, SUC_lexicon_tags_7 },
    { 0x9eb21f9c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af21fa3, SUC_lexicon_tags_37 },
    { 0x1ef21fa4, SUC_lexicon_tags_23 },
    { 0xc921fa5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2121fa8, SUC_lexicon_tags_27 },
    { 0x7c121fa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34721fac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b21fb1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x12921fb4, SUC_lexicon_tags_11 },
    { 0x45d21fb5, SUC_lexicon_tags_11 },
    { 0xbb321fb6, SUC_lexicon_tags_25 },
    { 0x6a321fb7, SUC_lexicon_tags_30 },
    { 0x47b21fb5, SUC_lexicon_tags_49 },
    { 0xdd21fb5, SUC_lexicon_tags_17 },
    { 0x51121fba, SUC_lexicon_tags_37 },
    { 0x12321fba, SUC_lexicon_tags_11 },
    { 0x2f21fbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed21fbf, SUC_lexicon_tags_7 },
    { 0xb1521fbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d521fc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f21fc8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x56121fca, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36721fd1, SUC_lexicon_tags_7 },
    { 0x92121fd2, SUC_lexicon_tags_17 },
    { 0xa721fd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd921fda, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92721fde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0721fe0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30921fe2, SUC_lexicon_tags_27 },
    { 0xcc321fe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45521feb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc921fed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ef21fef, SUC_lexicon_tags_4 },
    { 0xf3d21ff0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8f321ff2, SUC_lexicon_tags_4 },
    { 0xd3b21ff3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1f921ff5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3521ff7, SUC_lexicon_tags_55 },
    { 0xf0521ff8, SUC_lexicon_tags_3 },
    { 0x4ad21ff9, SUC_lexicon_tags_48 },
    { 0x28121ff9, SUC_lexicon_tags_7 },
    { 0x15f21ffb, SUC_lexicon_tags_63 },
    { 0xe9121ffc, SUC_lexicon_tags_1 },
    { 0x3bd21ffc, SUC_lexicon_tags_37 },
    { 0xa8921ffd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab722004, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f2200a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c12200e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1522010, SUC_lexicon_tags_15 },
    { 0xb3722011, SUC_lexicon_tags_8 },
    { 0x3d122012, SUC_lexicon_tags_11 },
    { 0x96b22013, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c322017, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f2201e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3922020, SUC_lexicon_tags_45 },
    { 0xb0b22020, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b522026, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2352202f, SUC_lexicon_tags_29 },
    { 0x85722030, SUC_lexicon_tags_47 },
    { 0xb1322031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fd22033, SUC_lexicon_tags_66 },
    { 0x5a522034, SUC_lexicon_tags_17 },
    { 0x75d22034, SUC_lexicon_tags_32 },
    { 0x72522036, SUC_lexicon_tags_11 },
    { 0x39522037, SUC_lexicon_tags_47 },
    { 0x75b22037, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf192203a, SUC_lexicon_tags_14 },
    { 0xd672203a, SUC_lexicon_tags_37 },
    { 0xd3f2203a, SUC_lexicon_tags_12 },
    { 0x61b2203b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef22041, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83d22043, SUC_lexicon_tags_49 },
    { 0x81922043, SUC_lexicon_tags_25 },
    { 0xd1b22045, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8722048, SUC_lexicon_tags_25 },
    { 0xfff22048, SUC_lexicon_tags_55 },
    { 0x6d72204a, SUC_lexicon_tags_17 },
    { 0x7d2204a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6832204e, SUC_lexicon_tags_425 },
    { 0x86d2204f, SUC_lexicon_tags_16 },
    { 0x88d22050, SUC_lexicon_tags_38 },
    { 0xc7922050, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12522053, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2922059, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1b2205b, SUC_lexicon_tags_5 },
    { 0xf612205c, SUC_lexicon_tags_15 },
    { 0x7732205d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf652205f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9122063, SUC_lexicon_tags_47 },
    { 0xddb22064, SUC_lexicon_tags_1 },
    { 0x3ab22065, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf122068, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb2206c, SUC_lexicon_tags_11 },
    { 0x4bf2206d, SUC_lexicon_tags_61 },
    { 0x89d2206e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2722070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6322074, SUC_lexicon_tags_79 },
    { 0x5cb22074, SUC_lexicon_tags_49 },
    { 0xe0b22076, SUC_lexicon_tags_38 },
    { 0xe3f22075, SUC_lexicon_tags_49 },
    { 0x8db22078, SUC_lexicon_tags_11 },
    { 0xc9b22078, SUC_lexicon_tags_10 },
    { 0xbc52207a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe372207c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5632207e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16322083, SUC_lexicon_tags_2 },
    { 0x51b22084, SUC_lexicon_tags_2 },
    { 0x4db22084, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b22088, SUC_lexicon_tags_63 },
    { 0x5bb22088, SUC_lexicon_tags_12 },
    { 0xab32208a, SUC_lexicon_tags_43 },
    { 0x9cd2208b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6712208f, SUC_lexicon_tags_17 },
    { 0x32b22090, SUC_lexicon_tags_7 },
    { 0x35b2208f, SUC_lexicon_tags_39 },
    { 0xdd92208f, SUC_lexicon_tags_27 },
    { 0x44322093, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9fb22095, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b22098, SUC_lexicon_tags_11 },
    { 0xb5922099, SUC_lexicon_tags_37 },
    { 0x7fb2209a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d2209d, SUC_lexicon_tags_37 },
    { 0x5812209e, SUC_lexicon_tags_60 },
    { 0x2832209d, SUC_lexicon_tags_28 },
    { 0xe112209f, SUC_lexicon_tags_2 },
    { 0xed12209e, SUC_lexicon_tags_53 },
    { 0xa12209f, SUC_lexicon_tags_43 },
    { 0x3832209e, SUC_lexicon_tags_2 },
    { 0x2b52209e, SUC_lexicon_tags_15 },
    { 0x5f1220a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x503220a8, SUC_lexicon_tags_47 },
    { 0x269220a8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x92d220ab, SUC_lexicon_tags_23 },
    { 0xecd220ac, SUC_lexicon_tags_27 },
    { 0x7ed220ad, SUC_lexicon_tags_27 },
    { 0x83f220ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5220b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7220b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d220bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae3220bf, SUC_lexicon_tags_15 },
    { 0x409220c0, SUC_lexicon_tags_68 },
    { 0x4dd220bf, SUC_lexicon_tags_69 },
    { 0xeaf220c2, SUC_lexicon_tags_43 },
    { 0xe95220bf, SUC_lexicon_tags_37 },
    { 0x10b220c0, SUC_lexicon_tags_11 },
    { 0xc55220c5, SUC_lexicon_tags_9 },
    { 0xbd5220c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9220cb, SUC_lexicon_tags_8 },
    { 0x38d220cb, SUC_lexicon_tags_0 },
    { 0xe1220cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f220d1, SUC_lexicon_tags_43 },
    { 0x285220d1, SUC_lexicon_tags_26 },
    { 0x863220d3, SUC_lexicon_tags_77 },
    { 0x4ed220d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf220dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b220e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d220e5, SUC_lexicon_tags_63 },
    { 0xecf220e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1220ea, SUC_lexicon_tags_37 },
    { 0x275220eb, SUC_lexicon_tags_7 },
    { 0x911220ec, SUC_lexicon_tags_13 },
    { 0x3df220ed, SUC_lexicon_tags_34 },
    { 0x481220eb, SUC_lexicon_tags_11 },
    { 0x623220ef, SUC_lexicon_tags_107 },
    { 0x969220f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd220fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa322100, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf322102, SUC_lexicon_tags_0 },
    { 0x49522102, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57322108, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab72210a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d12210d, SUC_lexicon_tags_25 },
    { 0x8992210d, SUC_lexicon_tags_3 },
    { 0x19d2210f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d522116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f322118, SUC_lexicon_tags_12 },
    { 0x29922119, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb052211e, SUC_lexicon_tags_11 },
    { 0x6032211f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9522121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8922126, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd2212a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2432212c, SUC_lexicon_tags_0 },
    { 0xe9b2212d, SUC_lexicon_tags_7 },
    { 0xf9b2212d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d122133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd22137, SUC_lexicon_tags_68 },
    { 0x73922138, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x772213b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc652213f, SUC_lexicon_tags_37 },
    { 0x55722140, SUC_lexicon_tags_17 },
    { 0x31f22140, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x43d22143, SUC_lexicon_tags_55 },
    { 0x60b22144, SUC_lexicon_tags_1 },
    { 0x6f722145, SUC_lexicon_tags_28 },
    { 0x46d22144, SUC_lexicon_tags_57 },
    { 0xe0322146, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf2214b, SUC_lexicon_tags_27 },
    { 0xf052214c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b52214e, SUC_lexicon_tags_7 },
    { 0xf0f2214e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97122153, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35922158, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd32215b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b2215d, SUC_lexicon_tags_11 },
    { 0x41d2215d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7672216e, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d22172, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f22175, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x322177, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff22182, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9322185, SUC_lexicon_tags_74 },
    { 0x5d122186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa532218a, SUC_lexicon_tags_9 },
    { 0x7732218b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf2218f, SUC_lexicon_tags_25 },
    { 0x2d922190, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e522196, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef2219b, SUC_lexicon_tags_36 },
    { 0xe332219c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9221a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879221a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd3221ab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x965221ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc7d221b0, SUC_lexicon_tags_14 },
    { 0xb0f221b1, SUC_lexicon_tags_17 },
    { 0x165221b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6eb221b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe19221b6, SUC_lexicon_tags_25 },
    { 0x87f221b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b221bc, SUC_lexicon_tags_47 },
    { 0xb35221bd, SUC_lexicon_tags_37 },
    { 0x687221be, SUC_lexicon_tags_37 },
    { 0x5d5221bc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xee7221c1, SUC_lexicon_tags_55 },
    { 0xc29221c2, SUC_lexicon_tags_37 },
    { 0xbd7221c1, SUC_lexicon_tags_34 },
    { 0xa93221c1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f221c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827221cb, SUC_lexicon_tags_25 },
    { 0x939221cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537221d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda9221d4, SUC_lexicon_tags_25 },
    { 0x349221d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x355221d7, SUC_lexicon_tags_6 },
    { 0xf5221d8, SUC_lexicon_tags_19 },
    { 0x9df221d8, SUC_lexicon_tags_49 },
    { 0x197221da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8dd221dc, SUC_lexicon_tags_31 },
    { 0x409221dc, SUC_lexicon_tags_15 },
    { 0x18b221de, SUC_lexicon_tags_364 },
    { 0x1db221dc, SUC_lexicon_tags_24 },
    { 0xa99221e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe19221e2, SUC_lexicon_tags_15 },
    { 0x5c1221e3, SUC_lexicon_tags_25 },
    { 0xd4b221e3, SUC_lexicon_tags_1 },
    { 0xb85221e4, SUC_lexicon_tags_79 },
    { 0xf01221e5, SUC_lexicon_tags_15 },
    { 0x373221e7, SUC_lexicon_tags_12 },
    { 0x693221e4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3c3221ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x479221ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe85221ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9221f2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7221f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x20f221f9, SUC_lexicon_tags_17 },
    { 0x729221fa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd221ff, SUC_lexicon_tags_7 },
    { 0x60f22200, SUC_lexicon_tags_7 },
    { 0x2ed22201, SUC_lexicon_tags_15 },
    { 0x54f22200, SUC_lexicon_tags_37 },
    { 0x6f522203, SUC_lexicon_tags_37 },
    { 0xf5922204, SUC_lexicon_tags_14 },
    { 0x7922205, SUC_lexicon_tags_1 },
    { 0x16322204, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb72220c, SUC_lexicon_tags_65 },
    { 0xa5b2220c, SUC_lexicon_tags_11 },
    { 0x592220e, SUC_lexicon_tags_0 },
    { 0x7892220f, SUC_lexicon_tags_8 },
    { 0x2f52220e, SUC_lexicon_tags_38 },
    { 0x7f32220e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65322217, SUC_lexicon_tags_7 },
    { 0x5b122218, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a12221a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc72221f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f22222, SUC_lexicon_tags_2 },
    { 0x3ef22223, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a522227, SUC_lexicon_tags_8 },
    { 0x42b22228, SUC_lexicon_tags_8 },
    { 0x80b22229, SUC_lexicon_tags_26 },
    { 0x7ab2222a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92d2222d, SUC_lexicon_tags_27 },
    { 0xfd92222e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8322232, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f522236, SUC_lexicon_tags_28 },
    { 0x50d22237, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc722243, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb22246, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4792224b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x18f2224d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb22250, SUC_lexicon_tags_7 },
    { 0xe7922251, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d22254, SUC_lexicon_tags_1 },
    { 0xd4f22255, SUC_lexicon_tags_17 },
    { 0xc8522255, SUC_lexicon_tags_9 },
    { 0xb4f22257, SUC_lexicon_tags_26 },
    { 0x3f922254, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd392225a, SUC_lexicon_tags_49 },
    { 0xad72225b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b2225e, SUC_lexicon_tags_11 },
    { 0xa872225f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc922268, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5d72226a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b2226c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce32226e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd22276, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc52227a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae72227d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2572227f, SUC_lexicon_tags_1 },
    { 0xb752227f, SUC_lexicon_tags_17 },
    { 0x4e722281, SUC_lexicon_tags_33 },
    { 0x1c522282, SUC_lexicon_tags_66 },
    { 0xcd522283, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d22287, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca52228e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xde922290, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a722294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f22298, SUC_lexicon_tags_2 },
    { 0xbff22299, SUC_lexicon_tags_112 },
    { 0x3c12229a, SUC_lexicon_tags_23 },
    { 0x5772229b, SUC_lexicon_tags_19 },
    { 0xe1f2229a, SUC_lexicon_tags_37 },
    { 0xaa32229b, SUC_lexicon_tags_15 },
    { 0xfc92229e, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77222a2, SUC_lexicon_tags_17 },
    { 0x817222a3, SUC_lexicon_tags_2 },
    { 0x683222a4, SUC_lexicon_tags_39 },
    { 0x425222a4, SUC_lexicon_tags_16 },
    { 0xb6b222a6, SUC_lexicon_tags_0 },
    { 0x97d222a6, SUC_lexicon_tags_60 },
    { 0xc11222a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f222ab, SUC_lexicon_tags_25 },
    { 0x8b9222ab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd07222ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77222b1, SUC_lexicon_tags_47 },
    { 0x105222b2, SUC_lexicon_tags_47 },
    { 0x78b222b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85f222b5, SUC_lexicon_tags_47 },
    { 0x1af222b6, SUC_lexicon_tags_46 },
    { 0x949222b7, SUC_lexicon_tags_2 },
    { 0xf0d222b6, SUC_lexicon_tags_6 },
    { 0x29b222b9, SUC_lexicon_tags_158 },
    { 0xb79222ba, SUC_lexicon_tags_10 },
    { 0x431222bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd222be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f222c2, SUC_lexicon_tags_17 },
    { 0x9af222c3, SUC_lexicon_tags_66 },
    { 0xeb9222c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf222c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b9222c8, SUC_lexicon_tags_17 },
    { 0x9ff222c8, SUC_lexicon_tags_1 },
    { 0x5ad222ca, SUC_lexicon_tags_39 },
    { 0x971222cb, SUC_lexicon_tags_17 },
    { 0x821222c9, SUC_lexicon_tags_2 },
    { 0xddd222cd, SUC_lexicon_tags_12 },
    { 0x88d222cd, SUC_lexicon_tags_19 },
    { 0xb49222cf, SUC_lexicon_tags_0 },
    { 0xf73222d0, SUC_lexicon_tags_17 },
    { 0xb77222d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631222d6, SUC_lexicon_tags_63 },
    { 0x77b222d7, SUC_lexicon_tags_30 },
    { 0xfbd222d8, SUC_lexicon_tags_17 },
    { 0x585222d9, SUC_lexicon_tags_19 },
    { 0xca5222d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef222e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45222ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3222ed, SUC_lexicon_tags_37 },
    { 0xa33222ed, SUC_lexicon_tags_12 },
    { 0x5b1222ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69222f1, SUC_lexicon_tags_37 },
    { 0x65d222f1, SUC_lexicon_tags_13 },
    { 0xc07222f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd97222f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1222f9, SUC_lexicon_tags_12 },
    { 0xa7b222fa, SUC_lexicon_tags_12 },
    { 0xc67222f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4b9222fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1322301, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2322304, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f22308, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc872230a, SUC_lexicon_tags_45 },
    { 0xcf72230b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d22312, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb22317, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9722319, SUC_lexicon_tags_27 },
    { 0x1a12231a, SUC_lexicon_tags_37 },
    { 0xf092231b, SUC_lexicon_tags_7 },
    { 0x36d2231c, SUC_lexicon_tags_7 },
    { 0xab722319, SUC_lexicon_tags_28 },
    { 0x8072231e, SUC_lexicon_tags_39 },
    { 0x8692231d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d922321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf322326, SUC_lexicon_tags_33 },
    { 0xa7322327, SUC_lexicon_tags_11 },
    { 0xdf922328, SUC_lexicon_tags_12 },
    { 0x4b322328, SUC_lexicon_tags_37 },
    { 0xce52232a, SUC_lexicon_tags_0 },
    { 0xab72232b, SUC_lexicon_tags_70 },
    { 0x3b72232b, SUC_lexicon_tags_17 },
    { 0x69122326, SUC_lexicon_tags_36 },
    { 0x5e12232d, SUC_lexicon_tags_25 },
    { 0x872232f, SUC_lexicon_tags_17 },
    { 0x85922330, SUC_lexicon_tags_17 },
    { 0x7b322326, SUC_lexicon_tags_7 },
    { 0x85322330, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2a722334, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4922337, SUC_lexicon_tags_92 },
    { 0x7c722337, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf752233e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7722342, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f722349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f2234d, SUC_lexicon_tags_27 },
    { 0x85f2234e, SUC_lexicon_tags_12 },
    { 0x6832234f, SUC_lexicon_tags_25 },
    { 0x4f722350, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2322355, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72322359, SUC_lexicon_tags_13 },
    { 0x792235a, SUC_lexicon_tags_42 },
    { 0xb672235b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a92235e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f22361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa472236c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc12236e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6722371, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd122373, SUC_lexicon_tags_1 },
    { 0x9ad22373, SUC_lexicon_tags_2 },
    { 0x75122375, SUC_lexicon_tags_47 },
    { 0xbbd22374, SUC_lexicon_tags_7 },
    { 0x29d22376, SUC_lexicon_tags_45 },
    { 0x15522374, SUC_lexicon_tags_29 },
    { 0xc6522377, SUC_lexicon_tags_7 },
    { 0x9d2237a, SUC_lexicon_tags_87 },
    { 0x6c522379, SUC_lexicon_tags_17 },
    { 0x69f2237b, SUC_lexicon_tags_37 },
    { 0x4112237d, SUC_lexicon_tags_14 },
    { 0xc8b2237d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f122384, SUC_lexicon_tags_160 },
    { 0xf6b22384, SUC_lexicon_tags_15 },
    { 0xcc922386, SUC_lexicon_tags_11 },
    { 0x8c522387, SUC_lexicon_tags_24 },
    { 0x74922388, SUC_lexicon_tags_16 },
    { 0x2ed22389, SUC_lexicon_tags_74 },
    { 0x2c322388, SUC_lexicon_tags_17 },
    { 0xfff22389, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d2238f, SUC_lexicon_tags_15 },
    { 0x6c722390, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b22398, SUC_lexicon_tags_60 },
    { 0xb9522399, SUC_lexicon_tags_37 },
    { 0xb2d2239a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83223a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b223a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951223aa, SUC_lexicon_tags_9 },
    { 0x933223aa, SUC_lexicon_tags_386 },
    { 0, NULL },
    { 0x46d223ad, SUC_lexicon_tags_19 },
    { 0x3a7223ae, SUC_lexicon_tags_12 },
    { 0xeb5223af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x205223b2, SUC_lexicon_tags_17 },
    { 0x8a7223b2, SUC_lexicon_tags_29 },
    { 0xa19223b4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3d9223b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x717223b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9223bc, SUC_lexicon_tags_10 },
    { 0xfef223bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9223c0, SUC_lexicon_tags_7 },
    { 0x893223c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5d223c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x285223c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d223c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d223cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941223d2, SUC_lexicon_tags_43 },
    { 0x87223d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437223dd, SUC_lexicon_tags_62 },
    { 0x67b223dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57223e4, SUC_lexicon_tags_17 },
    { 0xddf223e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa61223e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x905223ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d223f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x533223f3, SUC_lexicon_tags_27 },
    { 0xdd1223f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9223f8, SUC_lexicon_tags_16 },
    { 0x6cd223f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x201223fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65223fe, SUC_lexicon_tags_25 },
    { 0x8fd223ff, SUC_lexicon_tags_10 },
    { 0x59522400, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3cd22402, SUC_lexicon_tags_1 },
    { 0xfa522403, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1122405, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9922407, SUC_lexicon_tags_0 },
    { 0x79f22408, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd992240a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f2240e, SUC_lexicon_tags_7 },
    { 0xb012240f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49922411, SUC_lexicon_tags_30 },
    { 0x8322411, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xafb22414, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfeb22416, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0322419, SUC_lexicon_tags_37 },
    { 0x8632241a, SUC_lexicon_tags_23 },
    { 0xc7122419, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf2241f, SUC_lexicon_tags_15 },
    { 0x28f22420, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f22424, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9f922426, SUC_lexicon_tags_2 },
    { 0xedf22427, SUC_lexicon_tags_47 },
    { 0xea322428, SUC_lexicon_tags_19 },
    { 0x21522428, SUC_lexicon_tags_77 },
    { 0x5cb22429, SUC_lexicon_tags_17 },
    { 0xdc522428, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb32242d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f32242f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52122431, SUC_lexicon_tags_60 },
    { 0xc7b22432, SUC_lexicon_tags_14 },
    { 0xbeb22433, SUC_lexicon_tags_2 },
    { 0xf1b22433, SUC_lexicon_tags_11 },
    { 0x36722434, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x99122437, SUC_lexicon_tags_1 },
    { 0xbfb22438, SUC_lexicon_tags_10 },
    { 0x9b322438, SUC_lexicon_tags_25 },
    { 0x5dd2243a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b2243e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b22441, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15922443, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c52244a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd352244c, SUC_lexicon_tags_47 },
    { 0xc652244d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3722453, SUC_lexicon_tags_21 },
    { 0x4e522454, SUC_lexicon_tags_25 },
    { 0xa6322454, SUC_lexicon_tags_38 },
    { 0xc7d22456, SUC_lexicon_tags_8 },
    { 0xa7d22457, SUC_lexicon_tags_65 },
    { 0xac522457, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f22462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b22465, SUC_lexicon_tags_25 },
    { 0x9cd22466, SUC_lexicon_tags_15 },
    { 0xeeb22467, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd2246d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f22473, SUC_lexicon_tags_23 },
    { 0x19122474, SUC_lexicon_tags_17 },
    { 0xb6722474, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b22478, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2732247b, SUC_lexicon_tags_30 },
    { 0x30d2247c, SUC_lexicon_tags_24 },
    { 0xef72247d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7522487, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3812248b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1112248d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9d2248f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45122493, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82122499, SUC_lexicon_tags_15 },
    { 0x96122499, SUC_lexicon_tags_27 },
    { 0x2e32249a, SUC_lexicon_tags_92 },
    { 0x52b2249c, SUC_lexicon_tags_11 },
    { 0xaf72249d, SUC_lexicon_tags_37 },
    { 0xa792249e, SUC_lexicon_tags_7 },
    { 0x8d2249a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5224a2, SUC_lexicon_tags_8 },
    { 0x51f224a2, SUC_lexicon_tags_87 },
    { 0x3c5224a4, SUC_lexicon_tags_17 },
    { 0x645224a5, SUC_lexicon_tags_25 },
    { 0xc8d224a6, SUC_lexicon_tags_37 },
    { 0x73f224a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x213224a9, SUC_lexicon_tags_15 },
    { 0x44d224aa, SUC_lexicon_tags_37 },
    { 0xcdf224ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd2b224ad, SUC_lexicon_tags_0 },
    { 0xe13224ad, SUC_lexicon_tags_33 },
    { 0x9fb224af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3224b4, SUC_lexicon_tags_26 },
    { 0x50f224b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5224b8, SUC_lexicon_tags_28 },
    { 0xa0d224b8, SUC_lexicon_tags_0 },
    { 0xca5224b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05224c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53224cb, SUC_lexicon_tags_12 },
    { 0xc49224cc, SUC_lexicon_tags_66 },
    { 0x537224cd, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x277224d0, SUC_lexicon_tags_7 },
    { 0x6f224d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bb224d3, SUC_lexicon_tags_11 },
    { 0x967224d4, SUC_lexicon_tags_37 },
    { 0xf27224d5, SUC_lexicon_tags_82 },
    { 0xe3b224d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73b224d8, SUC_lexicon_tags_11 },
    { 0xc19224d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb224de, SUC_lexicon_tags_15 },
    { 0xb69224df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21224e2, SUC_lexicon_tags_10 },
    { 0xa6d224e2, SUC_lexicon_tags_6 },
    { 0xc47224e3, SUC_lexicon_tags_11 },
    { 0xbbf224e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7224ee, SUC_lexicon_tags_26 },
    { 0xdad224ee, SUC_lexicon_tags_12 },
    { 0x5a1224f0, SUC_lexicon_tags_17 },
    { 0x3b9224f0, SUC_lexicon_tags_38 },
    { 0xba3224f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x893224f4, SUC_lexicon_tags_10 },
    { 0xb4d224f4, SUC_lexicon_tags_17 },
    { 0x6eb224f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9224fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db224fd, SUC_lexicon_tags_46 },
    { 0xad3224fe, SUC_lexicon_tags_39 },
    { 0xfef224fd, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc8722501, SUC_lexicon_tags_15 },
    { 0x67322502, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5522506, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfc722508, SUC_lexicon_tags_25 },
    { 0xce922509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce52250c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xb952250f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45122511, SUC_lexicon_tags_17 },
    { 0x3cf22512, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed722515, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a922518, SUC_lexicon_tags_32 },
    { 0x7b322519, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf2251c, SUC_lexicon_tags_17 },
    { 0xeeb2251d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x21922520, SUC_lexicon_tags_30 },
    { 0xf0122521, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d122525, SUC_lexicon_tags_21 },
    { 0x4d722526, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb2252e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc722538, SUC_lexicon_tags_56 },
    { 0x69d22538, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc952253b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad52253f, SUC_lexicon_tags_43 },
    { 0xa1d22540, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80322546, SUC_lexicon_tags_25 },
    { 0x19f22546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe212254f, SUC_lexicon_tags_46 },
    { 0x73d22550, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1122554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7992255a, SUC_lexicon_tags_1 },
    { 0x6bb2255b, SUC_lexicon_tags_151 },
    { 0x5db2255c, SUC_lexicon_tags_27 },
    { 0x412255d, SUC_lexicon_tags_12 },
    { 0xe8d2255e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca722564, SUC_lexicon_tags_12 },
    { 0x13922564, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa552256e, SUC_lexicon_tags_45 },
    { 0xce12256e, SUC_lexicon_tags_392 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43322573, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9922578, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x872257c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4b92257e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b22583, SUC_lexicon_tags_39 },
    { 0x52d22584, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba522586, SUC_lexicon_tags_37 },
    { 0x23b22587, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f32258a, SUC_lexicon_tags_12 },
    { 0x5692258b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc252258f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d22592, SUC_lexicon_tags_15 },
    { 0x16922592, SUC_lexicon_tags_15 },
    { 0x9ef22594, SUC_lexicon_tags_40 },
    { 0xa9d22595, SUC_lexicon_tags_19 },
    { 0x13722594, SUC_lexicon_tags_15 },
    { 0x9922597, SUC_lexicon_tags_13 },
    { 0xdc122592, SUC_lexicon_tags_31 },
    { 0x86122598, SUC_lexicon_tags_29 },
    { 0xcd22595, SUC_lexicon_tags_2 },
    { 0xdc52259b, SUC_lexicon_tags_0 },
    { 0x9c52259c, SUC_lexicon_tags_15 },
    { 0xf1b2259c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d225a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b225a4, SUC_lexicon_tags_37 },
    { 0xc3f225a5, SUC_lexicon_tags_91 },
    { 0x445225a4, SUC_lexicon_tags_9 },
    { 0xafb225a7, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x3af225a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d225b1, SUC_lexicon_tags_30 },
    { 0x6b1225b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafd225b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x719225b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321225bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5f225bd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d225c0, SUC_lexicon_tags_17 },
    { 0x645225c0, SUC_lexicon_tags_27 },
    { 0xced225c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77225c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237225cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf13225cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb225d1, SUC_lexicon_tags_15 },
    { 0x735225d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b225d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9225dc, SUC_lexicon_tags_31 },
    { 0x3d3225dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x275225e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6b225e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779225e9, SUC_lexicon_tags_21 },
    { 0x65d225ea, SUC_lexicon_tags_60 },
    { 0xe21225ea, SUC_lexicon_tags_11 },
    { 0x3a9225eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5225ef, SUC_lexicon_tags_49 },
    { 0x317225f0, SUC_lexicon_tags_25 },
    { 0x553225f1, SUC_lexicon_tags_11 },
    { 0xd65225ef, SUC_lexicon_tags_12 },
    { 0xac1225f0, SUC_lexicon_tags_15 },
    { 0x855225f1, SUC_lexicon_tags_15 },
    { 0x79f225f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325225f9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409225fd, SUC_lexicon_tags_43 },
    { 0xe8f225fe, SUC_lexicon_tags_26 },
    { 0x6a9225fd, SUC_lexicon_tags_57 },
    { 0x73d22600, SUC_lexicon_tags_17 },
    { 0x36b22601, SUC_lexicon_tags_7 },
    { 0x71d22602, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d22608, SUC_lexicon_tags_28 },
    { 0x62b22608, SUC_lexicon_tags_1 },
    { 0xae322608, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd992260d, SUC_lexicon_tags_37 },
    { 0x1352260e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60922613, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef22616, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc8b22618, SUC_lexicon_tags_2 },
    { 0x8122619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef22632, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7722637, SUC_lexicon_tags_6 },
    { 0x61b22638, SUC_lexicon_tags_5 },
    { 0xf8722639, SUC_lexicon_tags_37 },
    { 0x97b2263a, SUC_lexicon_tags_31 },
    { 0x5612263b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf2263e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68b22640, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b22644, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe722646, SUC_lexicon_tags_2 },
    { 0xb1f22647, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7012264c, SUC_lexicon_tags_49 },
    { 0x91f2264d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf092264f, SUC_lexicon_tags_48 },
    { 0x9c122650, SUC_lexicon_tags_23 },
    { 0xa2922651, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42922653, SUC_lexicon_tags_1 },
    { 0x8c922653, SUC_lexicon_tags_37 },
    { 0x35122653, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d92265b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb22661, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36722663, SUC_lexicon_tags_7 },
    { 0x7b522664, SUC_lexicon_tags_43 },
    { 0x2fb22665, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee922669, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc692266f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29922673, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x7522676, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd72267d, SUC_lexicon_tags_33 },
    { 0x6712267e, SUC_lexicon_tags_35 },
    { 0x15b2267f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df22682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c122685, SUC_lexicon_tags_7 },
    { 0xf5722685, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d122689, SUC_lexicon_tags_37 },
    { 0xcb92268a, SUC_lexicon_tags_70 },
    { 0x8372268b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f2268e, SUC_lexicon_tags_43 },
    { 0x7432268f, SUC_lexicon_tags_23 },
    { 0xc5d22690, SUC_lexicon_tags_17 },
    { 0xc0522691, SUC_lexicon_tags_13 },
    { 0x51f22692, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf226a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb226a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95226aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5226b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf55226b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471226ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41226bf, SUC_lexicon_tags_33 },
    { 0x6a9226c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161226c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x957226c6, SUC_lexicon_tags_55 },
    { 0x871226c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b226ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d226cd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa73226cf, SUC_lexicon_tags_15 },
    { 0x783226d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x819226d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa43226d4, SUC_lexicon_tags_30 },
    { 0x747226d5, SUC_lexicon_tags_14 },
    { 0x84f226d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1226d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38d226db, SUC_lexicon_tags_49 },
    { 0x6bf226dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b226df, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505226e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d226f2, SUC_lexicon_tags_13 },
    { 0x1e9226f2, SUC_lexicon_tags_23 },
    { 0x61f226f4, SUC_lexicon_tags_12 },
    { 0xea7226f3, SUC_lexicon_tags_49 },
    { 0x67f226f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479226fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23226fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed22706, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3f922708, SUC_lexicon_tags_8 },
    { 0xfa322709, SUC_lexicon_tags_14 },
    { 0x8ef2270a, SUC_lexicon_tags_37 },
    { 0xf292270a, SUC_lexicon_tags_8 },
    { 0x6772270a, SUC_lexicon_tags_84 },
    { 0x5bd2270b, SUC_lexicon_tags_45 },
    { 0x2f72270e, SUC_lexicon_tags_23 },
    { 0x8f922708, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7322711, SUC_lexicon_tags_25 },
    { 0x50d22712, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a522717, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd2271e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d2272a, SUC_lexicon_tags_26 },
    { 0x5cb2272b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc032272e, SUC_lexicon_tags_5 },
    { 0x44f2272e, SUC_lexicon_tags_45 },
    { 0xb972272e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34322735, SUC_lexicon_tags_43 },
    { 0xab722735, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b12273a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf2273d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a92273f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9322745, SUC_lexicon_tags_85 },
    { 0x38922746, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbf122748, SUC_lexicon_tags_33 },
    { 0xa5d22749, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d2274c, SUC_lexicon_tags_23 },
    { 0x70b2274c, SUC_lexicon_tags_0 },
    { 0x632274e, SUC_lexicon_tags_11 },
    { 0x9612274e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a922752, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2f22754, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8922757, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x41322759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba72275b, SUC_lexicon_tags_60 },
    { 0x4772275c, SUC_lexicon_tags_14 },
    { 0x6912275c, SUC_lexicon_tags_27 },
    { 0x46f2275d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d22762, SUC_lexicon_tags_28 },
    { 0xda322763, SUC_lexicon_tags_11 },
    { 0x57922763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9122769, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c52276c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6252276e, SUC_lexicon_tags_87 },
    { 0x57d2276e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1522772, SUC_lexicon_tags_33 },
    { 0xa722773, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4ab22775, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1722779, SUC_lexicon_tags_43 },
    { 0x42f2277a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b22782, SUC_lexicon_tags_141 },
    { 0x72f22783, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4cd22785, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfd722787, SUC_lexicon_tags_11 },
    { 0xd6b22787, SUC_lexicon_tags_31 },
    { 0xb7d22788, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2d2278b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9d2278d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eb2278f, SUC_lexicon_tags_2 },
    { 0xe0d22790, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b322793, SUC_lexicon_tags_0 },
    { 0x1fd22794, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42722798, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b2279f, SUC_lexicon_tags_37 },
    { 0x3232279f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877227a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7227aa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3227af, SUC_lexicon_tags_2 },
    { 0xf227af, SUC_lexicon_tags_109 },
    { 0x815227b0, SUC_lexicon_tags_11 },
    { 0x6b9227af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15227b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7227ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab227be, SUC_lexicon_tags_17 },
    { 0x9db227be, SUC_lexicon_tags_87 },
    { 0xf83227c0, SUC_lexicon_tags_17 },
    { 0x379227c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x467227c3, SUC_lexicon_tags_17 },
    { 0xcf5227c4, SUC_lexicon_tags_19 },
    { 0x1d9227c5, SUC_lexicon_tags_17 },
    { 0x583227c3, SUC_lexicon_tags_1 },
    { 0x7c1227c6, SUC_lexicon_tags_23 },
    { 0x6b5227c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x64f227ca, SUC_lexicon_tags_17 },
    { 0xcfd227cb, SUC_lexicon_tags_37 },
    { 0x52d227ca, SUC_lexicon_tags_33 },
    { 0xafb227cd, SUC_lexicon_tags_6 },
    { 0x427227cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x33b227d0, SUC_lexicon_tags_19 },
    { 0x13d227d1, SUC_lexicon_tags_47 },
    { 0xfc3227d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f227d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44b227d7, SUC_lexicon_tags_13 },
    { 0xd2f227d8, SUC_lexicon_tags_47 },
    { 0x3f5227d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5227dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab227e0, SUC_lexicon_tags_19 },
    { 0x2f5227e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1227e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f9227e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99227ef, SUC_lexicon_tags_1 },
    { 0xc0b227ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83227f3, SUC_lexicon_tags_37 },
    { 0x4cd227f4, SUC_lexicon_tags_27 },
    { 0xd23227f5, SUC_lexicon_tags_8 },
    { 0x8ed227f6, SUC_lexicon_tags_47 },
    { 0x8d9227f6, SUC_lexicon_tags_198 },
    { 0xff9227f8, SUC_lexicon_tags_7 },
    { 0xe3b227f7, SUC_lexicon_tags_144 },
    { 0x925227f7, SUC_lexicon_tags_82 },
    { 0x505227f9, SUC_lexicon_tags_37 },
    { 0xfe7227fc, SUC_lexicon_tags_1 },
    { 0xcd9227fc, SUC_lexicon_tags_63 },
    { 0x519227fe, SUC_lexicon_tags_54 },
    { 0x94f227ff, SUC_lexicon_tags_97 },
    { 0x8c3227ff, SUC_lexicon_tags_2 },
    { 0xad922801, SUC_lexicon_tags_65 },
    { 0x675227f9, SUC_lexicon_tags_12 },
    { 0x79d22800, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80122809, SUC_lexicon_tags_27 },
    { 0xc032280a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16522810, SUC_lexicon_tags_25 },
    { 0xd1322811, SUC_lexicon_tags_2 },
    { 0x23f22810, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d922815, SUC_lexicon_tags_79 },
    { 0xff922816, SUC_lexicon_tags_25 },
    { 0x22522816, SUC_lexicon_tags_6 },
    { 0xda322816, SUC_lexicon_tags_37 },
    { 0x3ed22818, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b22822, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x16322824, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6322826, SUC_lexicon_tags_57 },
    { 0x68122827, SUC_lexicon_tags_87 },
    { 0x25522828, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bd2282a, SUC_lexicon_tags_21 },
    { 0xa212282a, SUC_lexicon_tags_15 },
    { 0x3672282b, SUC_lexicon_tags_53 },
    { 0x2032282d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf22833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f122835, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x42722838, SUC_lexicon_tags_12 },
    { 0x6e322839, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3952283c, SUC_lexicon_tags_23 },
    { 0xd992283c, SUC_lexicon_tags_25 },
    { 0x5532283e, SUC_lexicon_tags_55 },
    { 0xf3f2283e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20122847, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9932284b, SUC_lexicon_tags_11 },
    { 0xa92284c, SUC_lexicon_tags_0 },
    { 0xd392284d, SUC_lexicon_tags_18 },
    { 0xa272284e, SUC_lexicon_tags_85 },
    { 0xd6b2284d, SUC_lexicon_tags_12 },
    { 0xc7722850, SUC_lexicon_tags_17 },
    { 0xa412284d, SUC_lexicon_tags_37 },
    { 0xff322852, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82922858, SUC_lexicon_tags_28 },
    { 0xe2b22859, SUC_lexicon_tags_28 },
    { 0xc3f22859, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8692285d, SUC_lexicon_tags_39 },
    { 0x78f2285e, SUC_lexicon_tags_17 },
    { 0x4f12285e, SUC_lexicon_tags_31 },
    { 0xfb922860, SUC_lexicon_tags_25 },
    { 0xd932285f, SUC_lexicon_tags_17 },
    { 0x28b2285d, SUC_lexicon_tags_44 },
    { 0x43922863, SUC_lexicon_tags_87 },
    { 0x6f322864, SUC_lexicon_tags_60 },
    { 0x41522865, SUC_lexicon_tags_60 },
    { 0x8a722862, SUC_lexicon_tags_37 },
    { 0xb9922860, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf332286d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3922872, SUC_lexicon_tags_1 },
    { 0xd6522873, SUC_lexicon_tags_8 },
    { 0x36522873, SUC_lexicon_tags_8 },
    { 0x3d722875, SUC_lexicon_tags_37 },
    { 0xeef22875, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb6722878, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc92287b, SUC_lexicon_tags_7 },
    { 0xc5b2287c, SUC_lexicon_tags_49 },
    { 0x4f32287d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f722883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d322885, SUC_lexicon_tags_2 },
    { 0x1b522885, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b22889, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb22893, SUC_lexicon_tags_21 },
    { 0xbf922893, SUC_lexicon_tags_11 },
    { 0x41f22895, SUC_lexicon_tags_39 },
    { 0x13f22896, SUC_lexicon_tags_108 },
    { 0xbe922897, SUC_lexicon_tags_12 },
    { 0xe9722893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8452289a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3812289d, SUC_lexicon_tags_25 },
    { 0x8fb2289e, SUC_lexicon_tags_25 },
    { 0x56d2289d, SUC_lexicon_tags_1 },
    { 0x1a3228a0, SUC_lexicon_tags_63 },
    { 0x4c5228a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30d228a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x179228a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af228ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2d7228ae, SUC_lexicon_tags_1 },
    { 0x425228ae, SUC_lexicon_tags_15 },
    { 0xa3f228ae, SUC_lexicon_tags_7 },
    { 0x633228b1, SUC_lexicon_tags_9 },
    { 0x211228b1, SUC_lexicon_tags_11 },
    { 0x477228b3, SUC_lexicon_tags_23 },
    { 0xe85228ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9b7228b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd95228b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f228be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09228c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfeb228c6, SUC_lexicon_tags_62 },
    { 0x321228c7, SUC_lexicon_tags_15 },
    { 0xb4d228c7, SUC_lexicon_tags_7 },
    { 0x3b228c9, SUC_lexicon_tags_20 },
    { 0x417228ca, SUC_lexicon_tags_14 },
    { 0xe9d228c9, SUC_lexicon_tags_2 },
    { 0xc53228cc, SUC_lexicon_tags_11 },
    { 0xcff228c9, SUC_lexicon_tags_2 },
    { 0x425228ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1a7228d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3228d5, SUC_lexicon_tags_48 },
    { 0xdbb228d6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b228d9, SUC_lexicon_tags_33 },
    { 0xd77228d9, SUC_lexicon_tags_25 },
    { 0xfb1228db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fb228dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x247228e0, SUC_lexicon_tags_47 },
    { 0xad228e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed228eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b228ed, SUC_lexicon_tags_17 },
    { 0x523228ee, SUC_lexicon_tags_11 },
    { 0x925228ed, SUC_lexicon_tags_151 },
    { 0x9ff228f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b1228f2, SUC_lexicon_tags_47 },
    { 0x51f228f3, SUC_lexicon_tags_104 },
    { 0xf0d228f4, SUC_lexicon_tags_17 },
    { 0x647228f5, SUC_lexicon_tags_23 },
    { 0x85d228f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f228fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9122900, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8922902, SUC_lexicon_tags_11 },
    { 0x19b22903, SUC_lexicon_tags_87 },
    { 0x1fd22903, SUC_lexicon_tags_17 },
    { 0x18722904, SUC_lexicon_tags_15 },
    { 0x50122906, SUC_lexicon_tags_47 },
    { 0xd8922906, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x4312290a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa852290d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccd2290f, SUC_lexicon_tags_17 },
    { 0x18d22910, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5122914, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d22916, SUC_lexicon_tags_37 },
    { 0x9bb22917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9612291a, SUC_lexicon_tags_8 },
    { 0x1a52291b, SUC_lexicon_tags_55 },
    { 0xc72291c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe922923, SUC_lexicon_tags_37 },
    { 0x4d122923, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad22927, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2db22929, SUC_lexicon_tags_0 },
    { 0x4cb2292a, SUC_lexicon_tags_60 },
    { 0xf5d2292b, SUC_lexicon_tags_23 },
    { 0x6f2292c, SUC_lexicon_tags_17 },
    { 0x7bb2292a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b22930, SUC_lexicon_tags_59 },
    { 0xad522931, SUC_lexicon_tags_31 },
    { 0x55f22932, SUC_lexicon_tags_37 },
    { 0x58f22933, SUC_lexicon_tags_33 },
    { 0x10122934, SUC_lexicon_tags_29 },
    { 0x37722935, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d22938, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb72293b, SUC_lexicon_tags_14 },
    { 0x9a52293b, SUC_lexicon_tags_87 },
    { 0x3f72293c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8172293f, SUC_lexicon_tags_37 },
    { 0x83f2293f, SUC_lexicon_tags_7 },
    { 0x84522941, SUC_lexicon_tags_11 },
    { 0x79522942, SUC_lexicon_tags_10 },
    { 0xb6322943, SUC_lexicon_tags_14 },
    { 0x78722944, SUC_lexicon_tags_7 },
    { 0x36322943, SUC_lexicon_tags_33 },
    { 0x31f22940, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6122948, SUC_lexicon_tags_6 },
    { 0xb5322949, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b2294c, SUC_lexicon_tags_2 },
    { 0x2352294c, SUC_lexicon_tags_7 },
    { 0xda32294c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12922951, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab522956, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d2295b, SUC_lexicon_tags_30 },
    { 0xb192295b, SUC_lexicon_tags_87 },
    { 0x14b2295d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c32295f, SUC_lexicon_tags_111 },
    { 0xd212295f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c322962, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd22967, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc72296a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6352296e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28b22970, SUC_lexicon_tags_434 },
    { 0x71922971, SUC_lexicon_tags_45 },
    { 0xda722972, SUC_lexicon_tags_7 },
    { 0xe5b22971, SUC_lexicon_tags_16 },
    { 0x96d22973, SUC_lexicon_tags_63 },
    { 0x33322975, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x43322978, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb512297b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0522984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf322987, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1112298a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbad2298c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba322990, SUC_lexicon_tags_43 },
    { 0x7db22991, SUC_lexicon_tags_61 },
    { 0xe7f22991, SUC_lexicon_tags_3 },
    { 0x8eb22993, SUC_lexicon_tags_56 },
    { 0x10122993, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a722999, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf2299c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf229a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd229a6, SUC_lexicon_tags_65 },
    { 0xa37229a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71b229a9, SUC_lexicon_tags_7 },
    { 0x495229a9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd0d229ac, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd229b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ab229ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7229c2, SUC_lexicon_tags_29 },
    { 0xccf229c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d229c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c3229c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f229cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x781229cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc59229d1, SUC_lexicon_tags_29 },
    { 0xcdd229d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb229d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd229db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65229de, SUC_lexicon_tags_11 },
    { 0x2b3229df, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf87229e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x711229e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0f229e5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x26b229e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xff9229e9, SUC_lexicon_tags_29 },
    { 0xc11229ea, SUC_lexicon_tags_58 },
    { 0x78d229eb, SUC_lexicon_tags_37 },
    { 0xb4f229ec, SUC_lexicon_tags_30 },
    { 0x75f229ed, SUC_lexicon_tags_23 },
    { 0x481229ee, SUC_lexicon_tags_8 },
    { 0xd17229ef, SUC_lexicon_tags_45 },
    { 0x1e3229ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1b229f2, SUC_lexicon_tags_2 },
    { 0xc7b229f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527229f7, SUC_lexicon_tags_12 },
    { 0xdc3229f7, SUC_lexicon_tags_2 },
    { 0xba3229f9, SUC_lexicon_tags_27 },
    { 0x70d229fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9f1229fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a722a00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7f22a02, SUC_lexicon_tags_160 },
    { 0x6122a03, SUC_lexicon_tags_1 },
    { 0x42d22a03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6922a07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8f522a09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9122a0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b22a10, SUC_lexicon_tags_33 },
    { 0xc9722a11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc522a13, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb22a16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82522a1c, SUC_lexicon_tags_27 },
    { 0xad522a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb322a23, SUC_lexicon_tags_4 },
    { 0xa7f22a24, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f22a27, SUC_lexicon_tags_25 },
    { 0xa6d22a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64922a2c, SUC_lexicon_tags_47 },
    { 0x26322a2c, SUC_lexicon_tags_3 },
    { 0x6f322a2e, SUC_lexicon_tags_37 },
    { 0xe3522a2f, SUC_lexicon_tags_267 },
    { 0x14722a30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe122a34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af22a38, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6122a3d, SUC_lexicon_tags_11 },
    { 0xee522a3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4322a41, SUC_lexicon_tags_37 },
    { 0x78322a42, SUC_lexicon_tags_62 },
    { 0x1f22a41, SUC_lexicon_tags_15 },
    { 0xef722a44, SUC_lexicon_tags_15 },
    { 0x86d22a42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x23722a47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35522a4d, SUC_lexicon_tags_12 },
    { 0xf8722a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f22a58, SUC_lexicon_tags_6 },
    { 0xb7722a58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5dd22a5b, SUC_lexicon_tags_6 },
    { 0xe1722a5b, SUC_lexicon_tags_7 },
    { 0xae322a5c, SUC_lexicon_tags_2 },
    { 0x9ad22a5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32522a60, SUC_lexicon_tags_37 },
    { 0xbfb22a61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86722a6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a322a79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23322a7b, SUC_lexicon_tags_12 },
    { 0x9d722a7c, SUC_lexicon_tags_27 },
    { 0xf2f22a7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f22a80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b122a83, SUC_lexicon_tags_30 },
    { 0x43f22a84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab122a87, SUC_lexicon_tags_15 },
    { 0x12722a88, SUC_lexicon_tags_5 },
    { 0xae922a89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b22a8d, SUC_lexicon_tags_2 },
    { 0xc522a8d, SUC_lexicon_tags_9 },
    { 0x82f22a8e, SUC_lexicon_tags_11 },
    { 0x8e122a8f, SUC_lexicon_tags_60 },
    { 0x60922a91, SUC_lexicon_tags_25 },
    { 0xcc922a8d, SUC_lexicon_tags_18 },
    { 0x61922a90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef22a96, SUC_lexicon_tags_2 },
    { 0xd2522a97, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b22a9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd9f22a9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d522a9f, SUC_lexicon_tags_17 },
    { 0xa5b22a9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6122aa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8522aa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad22aac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc122aae, SUC_lexicon_tags_38 },
    { 0x35922aaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49722ab3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d22ab7, SUC_lexicon_tags_35 },
    { 0xb0b22ab7, SUC_lexicon_tags_7 },
    { 0xe1322ab9, SUC_lexicon_tags_61 },
    { 0x35522ab8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x64f22abc, SUC_lexicon_tags_27 },
    { 0x7d922abd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81722ac3, SUC_lexicon_tags_25 },
    { 0xa3522ac4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1922ac7, SUC_lexicon_tags_25 },
    { 0xcdd22ac8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b22acf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62722ad1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x67122ad3, SUC_lexicon_tags_47 },
    { 0xd3522ad4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2322ad7, SUC_lexicon_tags_7 },
    { 0xbcd22ad8, SUC_lexicon_tags_64 },
    { 0x91722ad9, SUC_lexicon_tags_25 },
    { 0x99522ad7, SUC_lexicon_tags_15 },
    { 0x3b922adb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb7b22add, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70322ae1, SUC_lexicon_tags_43 },
    { 0xdd722ae2, SUC_lexicon_tags_21 },
    { 0x77322ae1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x47722ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64322aea, SUC_lexicon_tags_38 },
    { 0x3bf22aeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b22af1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3722af7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7cb22af9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf722afc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b322b00, SUC_lexicon_tags_7 },
    { 0xd3922b00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xef722b04, SUC_lexicon_tags_37 },
    { 0x37122b04, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f22b08, SUC_lexicon_tags_14 },
    { 0x45322b08, SUC_lexicon_tags_15 },
    { 0x57722b0a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9922b0f, SUC_lexicon_tags_15 },
    { 0xb4b22b10, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f322b13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f322b1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xedd22b1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9122b20, SUC_lexicon_tags_40 },
    { 0x62b22b21, SUC_lexicon_tags_37 },
    { 0xc5922b22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53522b26, SUC_lexicon_tags_568 },
    { 0x54d22b27, SUC_lexicon_tags_3 },
    { 0x7322b26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d22b2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f22b31, SUC_lexicon_tags_11 },
    { 0x57d22b32, SUC_lexicon_tags_2 },
    { 0x58f22b33, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb6722b35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6f322b37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e322b3b, SUC_lexicon_tags_65 },
    { 0x46d22b3c, SUC_lexicon_tags_7 },
    { 0x5a322b3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd722b40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4122b45, SUC_lexicon_tags_45 },
    { 0x16922b46, SUC_lexicon_tags_0 },
    { 0x9f122b45, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22322b4b, SUC_lexicon_tags_29 },
    { 0xa4722b4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6d22b4e, SUC_lexicon_tags_7 },
    { 0x1b322b4f, SUC_lexicon_tags_26 },
    { 0xd1322b4f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b22b53, SUC_lexicon_tags_85 },
    { 0xd3f22b53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b22b59, SUC_lexicon_tags_40 },
    { 0x4dd22b5a, SUC_lexicon_tags_11 },
    { 0x2322b59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e922b60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8522b62, SUC_lexicon_tags_40 },
    { 0x70322b63, SUC_lexicon_tags_11 },
    { 0xab722b64, SUC_lexicon_tags_37 },
    { 0x55b22b65, SUC_lexicon_tags_25 },
    { 0xe3d22b63, SUC_lexicon_tags_92 },
    { 0x42322b67, SUC_lexicon_tags_37 },
    { 0xaf22b67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf522b6a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af22b73, SUC_lexicon_tags_49 },
    { 0x53922b74, SUC_lexicon_tags_359 },
    { 0x61722b75, SUC_lexicon_tags_19 },
    { 0x43b22b76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b22b79, SUC_lexicon_tags_16 },
    { 0x52522b7a, SUC_lexicon_tags_14 },
    { 0x6dd22b79, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6122b81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6bd22b83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc1b22b85, SUC_lexicon_tags_30 },
    { 0x43722b85, SUC_lexicon_tags_37 },
    { 0x95f22b87, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d22b8f, SUC_lexicon_tags_68 },
    { 0xf9f22b8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6122b92, SUC_lexicon_tags_15 },
    { 0x62522b92, SUC_lexicon_tags_21 },
    { 0xadd22b94, SUC_lexicon_tags_15 },
    { 0x1f122b92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f22b9e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73522ba3, SUC_lexicon_tags_60 },
    { 0xddf22ba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f22ba7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6f22ba9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x27322bab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d22bb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c522bb5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4d22bb7, SUC_lexicon_tags_23 },
    { 0xdd122bb7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6922bbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90522bc0, SUC_lexicon_tags_45 },
    { 0xf7922bc1, SUC_lexicon_tags_47 },
    { 0x5b322bc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27f22bc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59922bc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67922bca, SUC_lexicon_tags_28 },
    { 0x722bcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51122bce, SUC_lexicon_tags_33 },
    { 0xd9b22bcf, SUC_lexicon_tags_12 },
    { 0x61122bd0, SUC_lexicon_tags_25 },
    { 0xd1722bd1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xecf22bd3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce922bd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab22bd9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f22be1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef22be4, SUC_lexicon_tags_26 },
    { 0xd8322be5, SUC_lexicon_tags_25 },
    { 0xcf122be6, SUC_lexicon_tags_15 },
    { 0xd6522be7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f22beb, SUC_lexicon_tags_8 },
    { 0x5ad22beb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x54722bee, SUC_lexicon_tags_37 },
    { 0x5322bef, SUC_lexicon_tags_63 },
    { 0x47322bf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd22bf4, SUC_lexicon_tags_27 },
    { 0x8b22bf5, SUC_lexicon_tags_15 },
    { 0x41322bf5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x35722bf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30322bfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f122c02, SUC_lexicon_tags_2 },
    { 0xe7d22c03, SUC_lexicon_tags_25 },
    { 0x12b22c04, SUC_lexicon_tags_12 },
    { 0x27122c05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b122c07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28922c09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db22c0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f322c12, SUC_lexicon_tags_12 },
    { 0x53f22c13, SUC_lexicon_tags_11 },
    { 0x1df22c14, SUC_lexicon_tags_36 },
    { 0xb5522c12, SUC_lexicon_tags_92 },
    { 0x25322c16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a722c18, SUC_lexicon_tags_4 },
    { 0x27f22c18, SUC_lexicon_tags_37 },
    { 0xc2b22c18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50922c26, SUC_lexicon_tags_17 },
    { 0xae922c27, SUC_lexicon_tags_26 },
    { 0xaed22c28, SUC_lexicon_tags_15 },
    { 0x89b22c28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60b22c2b, SUC_lexicon_tags_60 },
    { 0xa5f22c2c, SUC_lexicon_tags_14 },
    { 0x3f922c2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4e722c2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45722c32, SUC_lexicon_tags_8 },
    { 0x38322c33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea722c3a, SUC_lexicon_tags_8 },
    { 0xb1922c3b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x40922c3d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x83122c3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xba722c42, SUC_lexicon_tags_19 },
    { 0x7d322c42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8722c46, SUC_lexicon_tags_19 },
    { 0x9c722c47, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15922c4b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd722c53, SUC_lexicon_tags_19 },
    { 0x57922c53, SUC_lexicon_tags_27 },
    { 0x54f22c55, SUC_lexicon_tags_2 },
    { 0xbd122c56, SUC_lexicon_tags_136 },
    { 0x6f522c54, SUC_lexicon_tags_15 },
    { 0xb6522c55, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x92922c5b, SUC_lexicon_tags_37 },
    { 0x3cb22c5c, SUC_lexicon_tags_7 },
    { 0x44922c5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1122c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb522c65, SUC_lexicon_tags_29 },
    { 0xf1b22c66, SUC_lexicon_tags_7 },
    { 0xa3b22c67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8722c6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55522c6f, SUC_lexicon_tags_25 },
    { 0x92522c6f, SUC_lexicon_tags_7 },
    { 0xc9b22c6f, SUC_lexicon_tags_39 },
    { 0x34322c71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e122c76, SUC_lexicon_tags_1 },
    { 0x78122c76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab22c7a, SUC_lexicon_tags_29 },
    { 0x62722c7b, SUC_lexicon_tags_10 },
    { 0xf2322c7b, SUC_lexicon_tags_64 },
    { 0xc722c7d, SUC_lexicon_tags_17 },
    { 0x58722c7c, SUC_lexicon_tags_0 },
    { 0xf4722c7e, SUC_lexicon_tags_77 },
    { 0xdbb22c7f, SUC_lexicon_tags_19 },
    { 0x59522c7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f22c86, SUC_lexicon_tags_12 },
    { 0x1a322c87, SUC_lexicon_tags_11 },
    { 0x7d122c88, SUC_lexicon_tags_63 },
    { 0xb4f22c88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f22c8d, SUC_lexicon_tags_12 },
    { 0xe7722c8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32122c90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b22c9a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc922c9d, SUC_lexicon_tags_10 },
    { 0x79722c9d, SUC_lexicon_tags_29 },
    { 0x70122c9f, SUC_lexicon_tags_0 },
    { 0x2722ca0, SUC_lexicon_tags_43 },
    { 0x9d922c9e, SUC_lexicon_tags_1 },
    { 0xab122ca2, SUC_lexicon_tags_45 },
    { 0x1ff22ca3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15922ca6, SUC_lexicon_tags_17 },
    { 0x122ca7, SUC_lexicon_tags_27 },
    { 0xe8522ca7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb122cae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f22cb1, SUC_lexicon_tags_30 },
    { 0x34d22cb2, SUC_lexicon_tags_61 },
    { 0x322cb2, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b22cb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fd22cb8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf2322cba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b22cbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0d22cbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b22cc2, SUC_lexicon_tags_6 },
    { 0x8eb22cc2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc2d22cc5, SUC_lexicon_tags_69 },
    { 0x2f122cc5, SUC_lexicon_tags_1 },
    { 0xfc722cc7, SUC_lexicon_tags_1 },
    { 0x68122cc6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd22cce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5122cd2, SUC_lexicon_tags_1 },
    { 0xb6922cd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73322cd7, SUC_lexicon_tags_37 },
    { 0x3cd22cd7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0922cdc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e922ce0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaf22ce2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69922ce8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1522cef, SUC_lexicon_tags_11 },
    { 0x65f22cf0, SUC_lexicon_tags_14 },
    { 0x50122cf1, SUC_lexicon_tags_31 },
    { 0x8af22cf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d22cf6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb922cfb, SUC_lexicon_tags_66 },
    { 0xcd322cfc, SUC_lexicon_tags_6 },
    { 0x3f722cfc, SUC_lexicon_tags_32 },
    { 0x9fd22cfe, SUC_lexicon_tags_47 },
    { 0x59b22cfe, SUC_lexicon_tags_118 },
    { 0x88d22d00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96522d02, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd22d06, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6322d0e, SUC_lexicon_tags_21 },
    { 0xcc922d0f, SUC_lexicon_tags_12 },
    { 0x89f22d0f, SUC_lexicon_tags_37 },
    { 0x13122d11, SUC_lexicon_tags_26 },
    { 0x58b22d11, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf122d18, SUC_lexicon_tags_10 },
    { 0x74322d18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb522d1e, SUC_lexicon_tags_17 },
    { 0xd4922d1f, SUC_lexicon_tags_60 },
    { 0x91322d20, SUC_lexicon_tags_1 },
    { 0xf5f22d20, SUC_lexicon_tags_19 },
    { 0x95722d22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd9d22d24, SUC_lexicon_tags_12 },
    { 0x9d122d24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc5722d27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f22d2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db22d30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x89722d32, SUC_lexicon_tags_17 },
    { 0xd8d22d33, SUC_lexicon_tags_14 },
    { 0x3f722d34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4f22d36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef22d3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d922d3f, SUC_lexicon_tags_79 },
    { 0xbe722d3f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8ff22d42, SUC_lexicon_tags_14 },
    { 0x54722d42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f22d4b, SUC_lexicon_tags_2 },
    { 0x79922d4c, SUC_lexicon_tags_25 },
    { 0x3922d4d, SUC_lexicon_tags_23 },
    { 0x4b522d4e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15922d52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6722d54, SUC_lexicon_tags_7 },
    { 0x8522d54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8322d57, SUC_lexicon_tags_12 },
    { 0x10b22d57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af22d5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a322d5f, SUC_lexicon_tags_14 },
    { 0x20f22d60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34122d63, SUC_lexicon_tags_71 },
    { 0xe7122d64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b22d67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd8522d69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4522d6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc522d6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb22d73, SUC_lexicon_tags_63 },
    { 0x37b22d73, SUC_lexicon_tags_7 },
    { 0x39b22d75, SUC_lexicon_tags_1 },
    { 0x20b22d73, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xdd922d78, SUC_lexicon_tags_14 },
    { 0x56122d78, SUC_lexicon_tags_15 },
    { 0xf122d7a, SUC_lexicon_tags_47 },
    { 0xf5722d7b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9f22d7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6722d7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1522d81, SUC_lexicon_tags_30 },
    { 0xf9522d81, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x52122d85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1522d8d, SUC_lexicon_tags_12 },
    { 0x48122d8e, SUC_lexicon_tags_7 },
    { 0x2322d8f, SUC_lexicon_tags_11 },
    { 0xab322d8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x78d22d92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad922d98, SUC_lexicon_tags_7 },
    { 0x41922d99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf22d9c, SUC_lexicon_tags_27 },
    { 0x3af22d9d, SUC_lexicon_tags_1 },
    { 0xc4922d9d, SUC_lexicon_tags_87 },
    { 0xfb322d9f, SUC_lexicon_tags_15 },
    { 0xf6522d9f, SUC_lexicon_tags_7 },
    { 0x2e522d9d, SUC_lexicon_tags_17 },
    { 0x2e522da1, SUC_lexicon_tags_46 },
    { 0x8a122d9c, SUC_lexicon_tags_66 },
    { 0x6d122da2, SUC_lexicon_tags_1 },
    { 0xde122da5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab22da9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf922dad, SUC_lexicon_tags_37 },
    { 0xf5322dad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeef22db0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d22db3, SUC_lexicon_tags_7 },
    { 0xa8122db3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d22dba, SUC_lexicon_tags_9 },
    { 0x7e922dba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xed722dbd, SUC_lexicon_tags_11 },
    { 0x75522dbd, SUC_lexicon_tags_17 },
    { 0xf2922dbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2122dc3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb22dc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b922dcc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62922dd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f22dd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae322dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63122dd9, SUC_lexicon_tags_15 },
    { 0x6e722dda, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf0922ddc, SUC_lexicon_tags_29 },
    { 0x9e722ddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd22de0, SUC_lexicon_tags_38 },
    { 0x84722de0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde322de3, SUC_lexicon_tags_85 },
    { 0x34922de3, SUC_lexicon_tags_38 },
    { 0x89522de5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd22de8, SUC_lexicon_tags_1 },
    { 0x5cf22de9, SUC_lexicon_tags_0 },
    { 0x7122dea, SUC_lexicon_tags_17 },
    { 0x95922deb, SUC_lexicon_tags_1 },
    { 0x72b22dec, SUC_lexicon_tags_25 },
    { 0xbdb22ded, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed22df2, SUC_lexicon_tags_56 },
    { 0xee122df2, SUC_lexicon_tags_37 },
    { 0xc1922df4, SUC_lexicon_tags_12 },
    { 0x5dd22df5, SUC_lexicon_tags_17 },
    { 0x48122df2, SUC_lexicon_tags_17 },
    { 0xa6522df4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce722dfd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa1122dff, SUC_lexicon_tags_39 },
    { 0xa2b22e00, SUC_lexicon_tags_15 },
    { 0x94d22e01, SUC_lexicon_tags_12 },
    { 0xa8922e00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf22e04, SUC_lexicon_tags_17 },
    { 0xeb922e05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xebb22e07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78722e0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b722e13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d522e16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcfb22e18, SUC_lexicon_tags_2 },
    { 0xcd722e19, SUC_lexicon_tags_15 },
    { 0xe6122e1a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6722e1e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff22e21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80122e28, SUC_lexicon_tags_12 },
    { 0x8322e29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e322e2c, SUC_lexicon_tags_47 },
    { 0x59b22e2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d22e32, SUC_lexicon_tags_1 },
    { 0x29122e32, SUC_lexicon_tags_87 },
    { 0xdc522e32, SUC_lexicon_tags_43 },
    { 0xa8f22e32, SUC_lexicon_tags_11 },
    { 0x2a522e36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf122e38, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b922e3d, SUC_lexicon_tags_45 },
    { 0x3df22e3e, SUC_lexicon_tags_33 },
    { 0x26d22e3f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x37122e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41722e48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d22e4a, SUC_lexicon_tags_32 },
    { 0x19522e4b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec122e4d, SUC_lexicon_tags_35 },
    { 0xe2322e4e, SUC_lexicon_tags_2 },
    { 0x84922e4f, SUC_lexicon_tags_390 },
    { 0x67522e4f, SUC_lexicon_tags_11 },
    { 0x29d22e51, SUC_lexicon_tags_1 },
    { 0xd9d22e52, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30122e56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f22e5b, SUC_lexicon_tags_12 },
    { 0x19f22e5b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb22e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf22e67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3eb22e69, SUC_lexicon_tags_25 },
    { 0x88322e6a, SUC_lexicon_tags_95 },
    { 0x73922e6b, SUC_lexicon_tags_14 },
    { 0x85322e6c, SUC_lexicon_tags_63 },
    { 0x15522e6d, SUC_lexicon_tags_32 },
    { 0xdcb22e6e, SUC_lexicon_tags_8 },
    { 0xfab22e6e, SUC_lexicon_tags_8 },
    { 0xcf722e6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfa322e72, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d22e75, SUC_lexicon_tags_336 },
    { 0x97722e76, SUC_lexicon_tags_44 },
    { 0xa5d22e77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc2922e79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b522e82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb22e85, SUC_lexicon_tags_47 },
    { 0x7df22e85, SUC_lexicon_tags_37 },
    { 0xc8122e87, SUC_lexicon_tags_10 },
    { 0x61b22e88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d22e8f, SUC_lexicon_tags_12 },
    { 0x7cf22e90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x56d22e92, SUC_lexicon_tags_33 },
    { 0x36522e93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7522e97, SUC_lexicon_tags_23 },
    { 0x11d22e98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d22e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb22e9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed522ea4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c922ea9, SUC_lexicon_tags_49 },
    { 0x28d22eaa, SUC_lexicon_tags_49 },
    { 0xc8322eab, SUC_lexicon_tags_17 },
    { 0x3d922eaa, SUC_lexicon_tags_25 },
    { 0xce322eaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe522eb0, SUC_lexicon_tags_30 },
    { 0x85722eb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af22eb4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x95922eb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4522eb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70922ebd, SUC_lexicon_tags_11 },
    { 0x45522ebe, SUC_lexicon_tags_38 },
    { 0xf7322ebe, SUC_lexicon_tags_14 },
    { 0x122ec0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff22ec3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8a322ec5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51d22ec7, SUC_lexicon_tags_1 },
    { 0x3fb22ec8, SUC_lexicon_tags_15 },
    { 0x4922ec8, SUC_lexicon_tags_15 },
    { 0x3f522eca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb322ecd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51522ecf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e322ed8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9522edb, SUC_lexicon_tags_37 },
    { 0x2f722edc, SUC_lexicon_tags_7 },
    { 0xd4b22edd, SUC_lexicon_tags_12 },
    { 0xf0f22ede, SUC_lexicon_tags_11 },
    { 0x3a322edf, SUC_lexicon_tags_57 },
    { 0x79722ee0, SUC_lexicon_tags_37 },
    { 0xf5722ee0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63922ee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63522eea, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b322ef4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf22ef9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b522f00, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe0d22f02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd5d22f04, SUC_lexicon_tags_15 },
    { 0xebd22f05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35b22f07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60b22f09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6522f0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77922f12, SUC_lexicon_tags_1 },
    { 0x2c322f12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62322f16, SUC_lexicon_tags_63 },
    { 0x60b22f16, SUC_lexicon_tags_45 },
    { 0x8f122f17, SUC_lexicon_tags_15 },
    { 0xad722f19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd522f1b, SUC_lexicon_tags_15 },
    { 0x36f22f1c, SUC_lexicon_tags_9 },
    { 0xbbd22f1d, SUC_lexicon_tags_37 },
    { 0x64d22f1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf2f22f20, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xab22f22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f22f26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b922f28, SUC_lexicon_tags_19 },
    { 0xc4b22f29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa722f2c, SUC_lexicon_tags_31 },
    { 0x43d22f2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12322f2f, SUC_lexicon_tags_30 },
    { 0xde322f30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3322f32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x53322f34, SUC_lexicon_tags_37 },
    { 0x33f22f35, SUC_lexicon_tags_8 },
    { 0x29122f34, SUC_lexicon_tags_11 },
    { 0xce122f37, SUC_lexicon_tags_37 },
    { 0x46d22f38, SUC_lexicon_tags_10 },
    { 0xae922f39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd922f3b, SUC_lexicon_tags_38 },
    { 0xd6922f3b, SUC_lexicon_tags_91 },
    { 0xbab22f3b, SUC_lexicon_tags_7 },
    { 0x28d22f3e, SUC_lexicon_tags_79 },
    { 0xe7b22f3f, SUC_lexicon_tags_12 },
    { 0x3c722f40, SUC_lexicon_tags_25 },
    { 0xf9b22f40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d22f49, SUC_lexicon_tags_43 },
    { 0x8db22f4a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b22f4d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7122f50, SUC_lexicon_tags_1392 },
    { 0x9ad22f50, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d22f54, SUC_lexicon_tags_12 },
    { 0x4ed22f55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a922f57, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33522f5d, SUC_lexicon_tags_11 },
    { 0xfd322f5d, SUC_lexicon_tags_82 },
    { 0x32322f5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd22f69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb522f70, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd22f79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f22f7c, SUC_lexicon_tags_15 },
    { 0xe5122f7d, SUC_lexicon_tags_33 },
    { 0xa8322f7e, SUC_lexicon_tags_15 },
    { 0xaa922f7f, SUC_lexicon_tags_1 },
    { 0xd2f22f80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27922f83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e322f89, SUC_lexicon_tags_11 },
    { 0xd4d22f89, SUC_lexicon_tags_15 },
    { 0x42d22f8b, SUC_lexicon_tags_25 },
    { 0xc3522f8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1522f8e, SUC_lexicon_tags_66 },
    { 0xdc122f8f, SUC_lexicon_tags_11 },
    { 0x6cb22f8e, SUC_lexicon_tags_43 },
    { 0x4e122f8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0d22f93, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf6522f95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25122f9b, SUC_lexicon_tags_27 },
    { 0xa3922f9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbd22f9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27722fa0, SUC_lexicon_tags_21 },
    { 0xbd522fa1, SUC_lexicon_tags_11 },
    { 0xfbf22fa1, SUC_lexicon_tags_6 },
    { 0xad522fa1, SUC_lexicon_tags_13 },
    { 0x8bd22fa1, SUC_lexicon_tags_74 },
    { 0x6b522fa0, SUC_lexicon_tags_15 },
    { 0x74722fa6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xaa922fa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21922fad, SUC_lexicon_tags_27 },
    { 0xdbf22fae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedb22fb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb722fbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f22fc1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f522fc5, SUC_lexicon_tags_37 },
    { 0x14f22fc6, SUC_lexicon_tags_11 },
    { 0x67722fc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53d22fc9, SUC_lexicon_tags_2 },
    { 0x9f922fca, SUC_lexicon_tags_19 },
    { 0x4322fca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb9722fcd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x72722fcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9922fd2, SUC_lexicon_tags_23 },
    { 0x5c722fd3, SUC_lexicon_tags_2 },
    { 0x8d722fd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9122fd7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb7722fd9, SUC_lexicon_tags_45 },
    { 0xe9f22fda, SUC_lexicon_tags_82 },
    { 0xef922fda, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x34d22fdd, SUC_lexicon_tags_14 },
    { 0x8cf22fde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf22fe0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbeb22fe2, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0x75b22fe4, SUC_lexicon_tags_31 },
    { 0xdbb22fe5, SUC_lexicon_tags_1 },
    { 0xb5d22fe5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x29b22fe8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd8322fea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x56522fed, SUC_lexicon_tags_4 },
    { 0x24522fee, SUC_lexicon_tags_12 },
    { 0xdbd22fef, SUC_lexicon_tags_32 },
    { 0x28722ff0, SUC_lexicon_tags_77 },
    { 0xc7922ff0, SUC_lexicon_tags_68 },
    { 0x7f22ff2, SUC_lexicon_tags_0 },
    { 0xce122fef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b322ff7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe722ffb, SUC_lexicon_tags_42 },
    { 0x7db22ffb, SUC_lexicon_tags_37 },
    { 0xa4722ffd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98723001, SUC_lexicon_tags_17 },
    { 0xd2d23001, SUC_lexicon_tags_45 },
    { 0x44923003, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef23008, SUC_lexicon_tags_19 },
    { 0xfa723009, SUC_lexicon_tags_19 },
    { 0x95d2300a, SUC_lexicon_tags_26 },
    { 0xa692300b, SUC_lexicon_tags_26 },
    { 0xfc72300a, SUC_lexicon_tags_29 },
    { 0x1f72300d, SUC_lexicon_tags_92 },
    { 0xb552300e, SUC_lexicon_tags_15 },
    { 0x23f2300f, SUC_lexicon_tags_37 },
    { 0x4912300e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa6923012, SUC_lexicon_tags_7 },
    { 0x67b23012, SUC_lexicon_tags_66 },
    { 0xa8b23012, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4123017, SUC_lexicon_tags_9 },
    { 0x12d23018, SUC_lexicon_tags_11 },
    { 0x4ff23018, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d23021, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x68123023, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x70123025, SUC_lexicon_tags_8 },
    { 0xd5323026, SUC_lexicon_tags_47 },
    { 0xabd23027, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2232302a, SUC_lexicon_tags_7 },
    { 0x4992302b, SUC_lexicon_tags_65 },
    { 0x90f2302c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ab2302e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x35523030, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x84723033, SUC_lexicon_tags_15 },
    { 0x85f23034, SUC_lexicon_tags_27 },
    { 0x26723035, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4752303a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab2303d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf72303f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee723043, SUC_lexicon_tags_55 },
    { 0x81b23044, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af2304a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15d2304c, SUC_lexicon_tags_12 },
    { 0xc3b2304d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67723053, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7752305b, SUC_lexicon_tags_47 },
    { 0x22d2305c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e52305f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48523063, SUC_lexicon_tags_11 },
    { 0xead23064, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb23067, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x88f23069, SUC_lexicon_tags_37 },
    { 0xcd2306a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe832306d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb23071, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee123078, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f2307c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7523081, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x24923083, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d23086, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a72308c, SUC_lexicon_tags_17 },
    { 0xaf32308d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0123091, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72923095, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5523097, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc632309b, SUC_lexicon_tags_7 },
    { 0x7672309c, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485230a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13230a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf71230aa, SUC_lexicon_tags_29 },
    { 0xaab230aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeeb230ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135230b2, SUC_lexicon_tags_45 },
    { 0xa19230b3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb230b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x537230b9, SUC_lexicon_tags_15 },
    { 0x7e1230ba, SUC_lexicon_tags_15 },
    { 0xf51230ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23230c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcd7230c2, SUC_lexicon_tags_25 },
    { 0xaef230c3, SUC_lexicon_tags_27 },
    { 0x9a9230c4, SUC_lexicon_tags_37 },
    { 0xe4f230c5, SUC_lexicon_tags_37 },
    { 0xa41230c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543230ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x833230cc, SUC_lexicon_tags_55 },
    { 0xab3230cd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989230d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7230e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xce5230e2, SUC_lexicon_tags_15 },
    { 0xa33230e3, SUC_lexicon_tags_27 },
    { 0x2bf230e4, SUC_lexicon_tags_7 },
    { 0x499230e3, SUC_lexicon_tags_30 },
    { 0x9b5230e3, SUC_lexicon_tags_25 },
    { 0xf9230e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x137230ea, SUC_lexicon_tags_47 },
    { 0x1c3230ea, SUC_lexicon_tags_62 },
    { 0xb17230ec, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xb0b230ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279230f5, SUC_lexicon_tags_25 },
    { 0x89230f6, SUC_lexicon_tags_65 },
    { 0xc77230f6, SUC_lexicon_tags_26 },
    { 0xe49230f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf230fb, SUC_lexicon_tags_47 },
    { 0x66f230fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3230ff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x44123101, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x15323103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b23109, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba72310f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62d23111, SUC_lexicon_tags_2 },
    { 0x9cb23111, SUC_lexicon_tags_49 },
    { 0xda123113, SUC_lexicon_tags_11 },
    { 0xc2b23114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6723116, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c923119, SUC_lexicon_tags_8 },
    { 0xd9f2311a, SUC_lexicon_tags_2 },
    { 0xa992311b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1632311d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f23123, SUC_lexicon_tags_15 },
    { 0xbdd23123, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x57923127, SUC_lexicon_tags_9 },
    { 0x6b123128, SUC_lexicon_tags_60 },
    { 0xd8f23129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8923131, SUC_lexicon_tags_17 },
    { 0xad723131, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfa323134, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6523137, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b2313a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9892313c, SUC_lexicon_tags_47 },
    { 0x2892313d, SUC_lexicon_tags_9 },
    { 0x1d2313d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1723140, SUC_lexicon_tags_87 },
    { 0x52f23140, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3a323143, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3f23145, SUC_lexicon_tags_34 },
    { 0x8dd23146, SUC_lexicon_tags_7 },
    { 0x4d523146, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x27123149, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a72314b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14f2314d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b923154, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27723159, SUC_lexicon_tags_7 },
    { 0x9b92315a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb92315e, SUC_lexicon_tags_11 },
    { 0xf192315f, SUC_lexicon_tags_37 },
    { 0xd4523160, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcd123162, SUC_lexicon_tags_6 },
    { 0xe5123163, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf7b23165, SUC_lexicon_tags_2 },
    { 0x89b23166, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7272316b, SUC_lexicon_tags_37 },
    { 0x9012316c, SUC_lexicon_tags_49 },
    { 0x2b92316b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x34b2316f, SUC_lexicon_tags_15 },
    { 0x9ff2316f, SUC_lexicon_tags_7 },
    { 0x93b23170, SUC_lexicon_tags_7 },
    { 0x3ad23172, SUC_lexicon_tags_8 },
    { 0xe1923173, SUC_lexicon_tags_40 },
    { 0xc9923174, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51923178, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0x1852317b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f523181, SUC_lexicon_tags_11 },
    { 0xf8b23182, SUC_lexicon_tags_25 },
    { 0xf7123181, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0923186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43523189, SUC_lexicon_tags_25 },
    { 0x65f2318a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x10b2318c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb23194, SUC_lexicon_tags_104 },
    { 0xe4523194, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfbb23197, SUC_lexicon_tags_7 },
    { 0xbaf23198, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3052319b, SUC_lexicon_tags_2 },
    { 0x6672319b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa92319f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93b231a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd231a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacd231aa, SUC_lexicon_tags_11 },
    { 0x7ab231ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf231ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b231b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad231bf, SUC_lexicon_tags_12 },
    { 0x63b231bf, SUC_lexicon_tags_1 },
    { 0x8ad231c1, SUC_lexicon_tags_38 },
    { 0x771231bf, SUC_lexicon_tags_2 },
    { 0xcd5231c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x51b231c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59231c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9231cc, SUC_lexicon_tags_1 },
    { 0x183231cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5231d0, SUC_lexicon_tags_25 },
    { 0xe73231d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff231d9, SUC_lexicon_tags_17 },
    { 0x531231da, SUC_lexicon_tags_17 },
    { 0x3d3231db, SUC_lexicon_tags_16 },
    { 0x209231da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995231e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7231e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x83f231e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b231ed, SUC_lexicon_tags_15 },
    { 0xe53231ed, SUC_lexicon_tags_40 },
    { 0xad9231ef, SUC_lexicon_tags_15 },
    { 0x351231f0, SUC_lexicon_tags_17 },
    { 0x521231ef, SUC_lexicon_tags_49 },
    { 0xdc7231f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207231f9, SUC_lexicon_tags_11 },
    { 0xd9f231fa, SUC_lexicon_tags_60 },
    { 0x949231fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fd231fd, SUC_lexicon_tags_34 },
    { 0x57f231fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f23203, SUC_lexicon_tags_60 },
    { 0x7123203, SUC_lexicon_tags_33 },
    { 0x5f23205, SUC_lexicon_tags_2 },
    { 0x13923205, SUC_lexicon_tags_15 },
    { 0x75123207, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4732320a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9732320c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a323211, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d23214, SUC_lexicon_tags_19 },
    { 0xa323214, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc572321d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d23223, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43123227, SUC_lexicon_tags_17 },
    { 0x3c723227, SUC_lexicon_tags_38 },
    { 0x7b23228, SUC_lexicon_tags_11 },
    { 0xd0923228, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc632322e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5fb23230, SUC_lexicon_tags_17 },
    { 0x63723231, SUC_lexicon_tags_37 },
    { 0xf2d23232, SUC_lexicon_tags_2 },
    { 0xe6f23233, SUC_lexicon_tags_27 },
    { 0x36b23233, SUC_lexicon_tags_37 },
    { 0x76323233, SUC_lexicon_tags_9 },
    { 0x90b23236, SUC_lexicon_tags_21 },
    { 0x73723237, SUC_lexicon_tags_39 },
    { 0x29b23238, SUC_lexicon_tags_0 },
    { 0xf5523239, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc52323d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1923242, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d523249, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc392324b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0723250, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2723252, SUC_lexicon_tags_60 },
    { 0xb8b23253, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2523256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf2325b, SUC_lexicon_tags_46 },
    { 0x6972325c, SUC_lexicon_tags_11 },
    { 0xe1b2325c, SUC_lexicon_tags_11 },
    { 0xb6b2325e, SUC_lexicon_tags_38 },
    { 0xfd12325f, SUC_lexicon_tags_33 },
    { 0xaf2325f, SUC_lexicon_tags_55 },
    { 0xc2b23261, SUC_lexicon_tags_25 },
    { 0x29b23262, SUC_lexicon_tags_62 },
    { 0xbb723263, SUC_lexicon_tags_13 },
    { 0x8bb23260, SUC_lexicon_tags_23 },
    { 0x7b523265, SUC_lexicon_tags_33 },
    { 0x28923260, SUC_lexicon_tags_2 },
    { 0x9a523267, SUC_lexicon_tags_7 },
    { 0x9672325f, SUC_lexicon_tags_29 },
    { 0x98323269, SUC_lexicon_tags_65 },
    { 0xd5923269, SUC_lexicon_tags_25 },
    { 0xe932326b, SUC_lexicon_tags_12 },
    { 0x6c12326c, SUC_lexicon_tags_12 },
    { 0xd712326c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d92326f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9323271, SUC_lexicon_tags_21 },
    { 0x96d23272, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72327d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db23280, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2f523282, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58723286, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f23289, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2092328b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa12328f, SUC_lexicon_tags_46 },
    { 0xb7b2328f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19723294, SUC_lexicon_tags_37 },
    { 0xd8723295, SUC_lexicon_tags_68 },
    { 0x57d23296, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80d23298, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f2329c, SUC_lexicon_tags_7 },
    { 0xfe32329d, SUC_lexicon_tags_11 },
    { 0xaa12329e, SUC_lexicon_tags_11 },
    { 0x79f2329f, SUC_lexicon_tags_11 },
    { 0x4232329f, SUC_lexicon_tags_37 },
    { 0xca7232a1, SUC_lexicon_tags_37 },
    { 0xa212329f, SUC_lexicon_tags_56 },
    { 0x6f9232a1, SUC_lexicon_tags_47 },
    { 0x969232a0, SUC_lexicon_tags_63 },
    { 0xe57232a0, SUC_lexicon_tags_23 },
    { 0x251232a6, SUC_lexicon_tags_15 },
    { 0x74d232a7, SUC_lexicon_tags_118 },
    { 0x3cd232a8, SUC_lexicon_tags_21 },
    { 0xd3f232a8, SUC_lexicon_tags_31 },
    { 0x517232aa, SUC_lexicon_tags_10 },
    { 0xaa7232aa, SUC_lexicon_tags_27 },
    { 0x2c9232ab, SUC_lexicon_tags_12 },
    { 0x17232a6, SUC_lexicon_tags_2 },
    { 0x577232a8, SUC_lexicon_tags_11 },
    { 0xe63232af, SUC_lexicon_tags_39 },
    { 0xefd232b0, SUC_lexicon_tags_1 },
    { 0x6b232af, SUC_lexicon_tags_19 },
    { 0x5ab232b0, SUC_lexicon_tags_12 },
    { 0xef232b3, SUC_lexicon_tags_33 },
    { 0x165232ac, SUC_lexicon_tags_11 },
    { 0x75b232b5, SUC_lexicon_tags_2 },
    { 0xa89232b5, SUC_lexicon_tags_25 },
    { 0x431232b7, SUC_lexicon_tags_17 },
    { 0x707232b8, SUC_lexicon_tags_21 },
    { 0xc43232b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b232bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1232bf, SUC_lexicon_tags_69 },
    { 0xc1b232c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x969232c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a3232c5, SUC_lexicon_tags_8 },
    { 0xf4f232c6, SUC_lexicon_tags_17 },
    { 0x389232c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x395232c9, SUC_lexicon_tags_30 },
    { 0xab7232ca, SUC_lexicon_tags_10 },
    { 0xa3d232ca, SUC_lexicon_tags_38 },
    { 0x97d232ca, SUC_lexicon_tags_7 },
    { 0xbf5232cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2b232cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11d232d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859232d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f232dc, SUC_lexicon_tags_11 },
    { 0xe11232dc, SUC_lexicon_tags_43 },
    { 0xb35232de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35232e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489232ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5232f3, SUC_lexicon_tags_40 },
    { 0xc53232f3, SUC_lexicon_tags_39 },
    { 0x2bf232f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5232f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfd3232fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b23300, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4723302, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb1523304, SUC_lexicon_tags_85 },
    { 0x2e523305, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f2330c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76923310, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc5123312, SUC_lexicon_tags_70 },
    { 0xc3b23312, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34723316, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2f23318, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a92331d, SUC_lexicon_tags_37 },
    { 0xe2f2331e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5723320, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78123327, SUC_lexicon_tags_17 },
    { 0xdcf23328, SUC_lexicon_tags_4 },
    { 0x64923329, SUC_lexicon_tags_12 },
    { 0x7323329, SUC_lexicon_tags_15 },
    { 0x41923329, SUC_lexicon_tags_60 },
    { 0xe3d23329, SUC_lexicon_tags_9 },
    { 0x8f32332d, SUC_lexicon_tags_66 },
    { 0x9712332e, SUC_lexicon_tags_11 },
    { 0x6512332e, SUC_lexicon_tags_7 },
    { 0x5f23330, SUC_lexicon_tags_7 },
    { 0x15523330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7612333d, SUC_lexicon_tags_402 },
    { 0x44b2333d, SUC_lexicon_tags_87 },
    { 0x5a92333e, SUC_lexicon_tags_1 },
    { 0xfab2333f, SUC_lexicon_tags_3 },
    { 0xb4923341, SUC_lexicon_tags_29 },
    { 0x50723342, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc7b23344, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa2923346, SUC_lexicon_tags_91 },
    { 0xaef23347, SUC_lexicon_tags_64 },
    { 0xfcf23347, SUC_lexicon_tags_16 },
    { 0xddb23348, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f32334b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb2334e, SUC_lexicon_tags_8 },
    { 0x7d12334f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55123353, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82323358, SUC_lexicon_tags_1 },
    { 0x37723359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f52335b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5523362, SUC_lexicon_tags_14 },
    { 0x58123363, SUC_lexicon_tags_42 },
    { 0x7d923364, SUC_lexicon_tags_37 },
    { 0xcad23364, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b2336c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45523371, SUC_lexicon_tags_0 },
    { 0x39723372, SUC_lexicon_tags_12 },
    { 0xa8923373, SUC_lexicon_tags_17 },
    { 0x36f23374, SUC_lexicon_tags_2 },
    { 0xa0923371, SUC_lexicon_tags_7 },
    { 0x4b923376, SUC_lexicon_tags_1 },
    { 0x93723375, SUC_lexicon_tags_14 },
    { 0xaa923378, SUC_lexicon_tags_45 },
    { 0xe9323372, SUC_lexicon_tags_5 },
    { 0xa8f23372, SUC_lexicon_tags_39 },
    { 0xb5323379, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xda72337d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35323384, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c323387, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc12338a, SUC_lexicon_tags_4 },
    { 0x2f92338b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1723395, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x61d23397, SUC_lexicon_tags_17 },
    { 0x4d723397, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6972339c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b2339f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251233a3, SUC_lexicon_tags_25 },
    { 0xc2f233a4, SUC_lexicon_tags_12 },
    { 0x153233a4, SUC_lexicon_tags_37 },
    { 0x9d7233a5, SUC_lexicon_tags_0 },
    { 0x179233a7, SUC_lexicon_tags_12 },
    { 0x5fd233a8, SUC_lexicon_tags_27 },
    { 0x527233a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf65233ab, SUC_lexicon_tags_31 },
    { 0x385233ac, SUC_lexicon_tags_15 },
    { 0x63f233ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x32f233af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1233b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7233b8, SUC_lexicon_tags_12 },
    { 0xefb233b8, SUC_lexicon_tags_47 },
    { 0x295233b9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439233be, SUC_lexicon_tags_42 },
    { 0xc71233be, SUC_lexicon_tags_27 },
    { 0x42f233bf, SUC_lexicon_tags_34 },
    { 0xc13233c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d233c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05233cc, SUC_lexicon_tags_30 },
    { 0x19b233cc, SUC_lexicon_tags_19 },
    { 0x1ad233cd, SUC_lexicon_tags_33 },
    { 0xd13233cf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9233d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f233d8, SUC_lexicon_tags_11 },
    { 0x4db233d8, SUC_lexicon_tags_39 },
    { 0xa57233da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0d233dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd233e3, SUC_lexicon_tags_1 },
    { 0xf81233e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdc1233e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x505233e8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29233eb, SUC_lexicon_tags_60 },
    { 0xf21233ec, SUC_lexicon_tags_15 },
    { 0x337233ed, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87233f3, SUC_lexicon_tags_49 },
    { 0x74b233f4, SUC_lexicon_tags_11 },
    { 0xbf7233f5, SUC_lexicon_tags_7 },
    { 0x50b233f4, SUC_lexicon_tags_27 },
    { 0x1bd233f3, SUC_lexicon_tags_11 },
    { 0x65f233f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x999233fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1123400, SUC_lexicon_tags_60 },
    { 0x55d23400, SUC_lexicon_tags_33 },
    { 0x77923402, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x65923405, SUC_lexicon_tags_23 },
    { 0x9b923406, SUC_lexicon_tags_11 },
    { 0xdf523406, SUC_lexicon_tags_25 },
    { 0x2d323405, SUC_lexicon_tags_11 },
    { 0x9923408, SUC_lexicon_tags_47 },
    { 0xa0b23406, SUC_lexicon_tags_25 },
    { 0x14123406, SUC_lexicon_tags_3 },
    { 0x24b23406, SUC_lexicon_tags_72 },
    { 0x20f2340a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87523411, SUC_lexicon_tags_12 },
    { 0x38b23412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd23418, SUC_lexicon_tags_37 },
    { 0x2c323419, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6392341b, SUC_lexicon_tags_25 },
    { 0xe932341c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe752341e, SUC_lexicon_tags_29 },
    { 0xdd32341f, SUC_lexicon_tags_201 },
    { 0xb2b2341f, SUC_lexicon_tags_60 },
    { 0x6a723420, SUC_lexicon_tags_27 },
    { 0x98f23422, SUC_lexicon_tags_23 },
    { 0x40b23423, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8b923425, SUC_lexicon_tags_15 },
    { 0x33323426, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xab523429, SUC_lexicon_tags_47 },
    { 0xf72342a, SUC_lexicon_tags_15 },
    { 0x2012342b, SUC_lexicon_tags_21 },
    { 0x6b123429, SUC_lexicon_tags_9 },
    { 0x7df2342d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c723431, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x11f23433, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca523439, SUC_lexicon_tags_25 },
    { 0xe2f23439, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2712343c, SUC_lexicon_tags_1 },
    { 0x7d32343c, SUC_lexicon_tags_47 },
    { 0x7f92343e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22d23440, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11123443, SUC_lexicon_tags_23 },
    { 0xb523443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0f23446, SUC_lexicon_tags_17 },
    { 0x5c923447, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28523449, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b2344c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b23453, SUC_lexicon_tags_60 },
    { 0x94523454, SUC_lexicon_tags_8 },
    { 0xfb923454, SUC_lexicon_tags_45 },
    { 0xae123453, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed52345c, SUC_lexicon_tags_45 },
    { 0x76b2345d, SUC_lexicon_tags_7 },
    { 0x74d2345c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2523460, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc7723462, SUC_lexicon_tags_17 },
    { 0x7a723462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d23468, SUC_lexicon_tags_1 },
    { 0xf0723469, SUC_lexicon_tags_37 },
    { 0x67f23468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50b2346c, SUC_lexicon_tags_129 },
    { 0x7592346d, SUC_lexicon_tags_25 },
    { 0x18b2346e, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0xeef23470, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0723473, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35923475, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9052347a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d2347e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4123481, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb23488, SUC_lexicon_tags_15 },
    { 0xe2923488, SUC_lexicon_tags_31 },
    { 0xa12348a, SUC_lexicon_tags_32 },
    { 0x3312348a, SUC_lexicon_tags_7 },
    { 0x3c12348c, SUC_lexicon_tags_77 },
    { 0xd892348a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5092348f, SUC_lexicon_tags_7 },
    { 0x2bd2348f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef23493, SUC_lexicon_tags_79 },
    { 0x42523494, SUC_lexicon_tags_7 },
    { 0xdfb23493, SUC_lexicon_tags_75 },
    { 0x23f23496, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97923498, SUC_lexicon_tags_27 },
    { 0xb2923499, SUC_lexicon_tags_7 },
    { 0xfe72349a, SUC_lexicon_tags_1 },
    { 0xef92349a, SUC_lexicon_tags_37 },
    { 0x2132349c, SUC_lexicon_tags_11 },
    { 0xf72349d, SUC_lexicon_tags_17 },
    { 0xde12349e, SUC_lexicon_tags_25 },
    { 0x1cd2349f, SUC_lexicon_tags_63 },
    { 0x3012349f, SUC_lexicon_tags_2 },
    { 0x219234a1, SUC_lexicon_tags_21 },
    { 0xdc52349c, SUC_lexicon_tags_66 },
    { 0x625234a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d234a6, SUC_lexicon_tags_27 },
    { 0x83f234a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde5234a9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x581234ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b234b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x45d234b2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1234b6, SUC_lexicon_tags_68 },
    { 0x19234b6, SUC_lexicon_tags_48 },
    { 0xa8b234b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b234be, SUC_lexicon_tags_13 },
    { 0xa1f234bf, SUC_lexicon_tags_17 },
    { 0xfd1234be, SUC_lexicon_tags_15 },
    { 0xc1234c1, SUC_lexicon_tags_33 },
    { 0x8f1234c2, SUC_lexicon_tags_19 },
    { 0x647234c3, SUC_lexicon_tags_4 },
    { 0x939234c4, SUC_lexicon_tags_12 },
    { 0xdd234c5, SUC_lexicon_tags_12 },
    { 0x6af234c5, SUC_lexicon_tags_2 },
    { 0xa71234c7, SUC_lexicon_tags_23 },
    { 0x7f3234c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x501234cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55234ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7234d3, SUC_lexicon_tags_45 },
    { 0xe99234d4, SUC_lexicon_tags_45 },
    { 0x17f234d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x377234d7, SUC_lexicon_tags_4 },
    { 0xc61234d7, SUC_lexicon_tags_23 },
    { 0x917234d7, SUC_lexicon_tags_83 },
    { 0x113234da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d234e1, SUC_lexicon_tags_11 },
    { 0xcd1234e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb234e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d234e8, SUC_lexicon_tags_108 },
    { 0x5d1234e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19234ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65234f2, SUC_lexicon_tags_9 },
    { 0x947234f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5ff234f5, SUC_lexicon_tags_37 },
    { 0x9b1234f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x245234f8, SUC_lexicon_tags_253 },
    { 0xe1d234f9, SUC_lexicon_tags_29 },
    { 0x97b234f9, SUC_lexicon_tags_25 },
    { 0x44f234fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x307234fe, SUC_lexicon_tags_30 },
    { 0xf9d234fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf1523501, SUC_lexicon_tags_45 },
    { 0xb6923501, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9723506, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e12350f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1523511, SUC_lexicon_tags_151 },
    { 0xb3723511, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d323517, SUC_lexicon_tags_43 },
    { 0xd1f23518, SUC_lexicon_tags_37 },
    { 0xc8523519, SUC_lexicon_tags_15 },
    { 0x2b12351a, SUC_lexicon_tags_23 },
    { 0xc252351a, SUC_lexicon_tags_47 },
    { 0x8df2351b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f2351f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x46d23521, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ad23523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d23526, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b2352b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b72352f, SUC_lexicon_tags_11 },
    { 0xee92352f, SUC_lexicon_tags_9 },
    { 0x82723531, SUC_lexicon_tags_27 },
    { 0x6d723531, SUC_lexicon_tags_26 },
    { 0x7dd23533, SUC_lexicon_tags_0 },
    { 0xee723534, SUC_lexicon_tags_61 },
    { 0xc0f23530, SUC_lexicon_tags_6 },
    { 0x3af23536, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x15923538, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59d2353a, SUC_lexicon_tags_12 },
    { 0x5212353b, SUC_lexicon_tags_26 },
    { 0x4e32353c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c12353e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3323545, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6cd23547, SUC_lexicon_tags_37 },
    { 0xee523548, SUC_lexicon_tags_30 },
    { 0xbcf23547, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8672354b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4f2354d, SUC_lexicon_tags_37 },
    { 0x4372354e, SUC_lexicon_tags_11 },
    { 0xe792354f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d23553, SUC_lexicon_tags_43 },
    { 0xd5523554, SUC_lexicon_tags_23 },
    { 0x5a123554, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x59323558, SUC_lexicon_tags_14 },
    { 0xb4923559, SUC_lexicon_tags_11 },
    { 0x9523559, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d2355e, SUC_lexicon_tags_7 },
    { 0x8832355f, SUC_lexicon_tags_2 },
    { 0x22d2355e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd2356d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d23572, SUC_lexicon_tags_65 },
    { 0x86f23572, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a123575, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab23579, SUC_lexicon_tags_1 },
    { 0x6bf2357a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2332357d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb923581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35923586, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4123588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d2358b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f523591, SUC_lexicon_tags_55 },
    { 0xcef23592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb235a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1235aa, SUC_lexicon_tags_129 },
    { 0xfb3235aa, SUC_lexicon_tags_31 },
    { 0xd1235ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bf235ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d235b2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x25235b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb235b9, SUC_lexicon_tags_9 },
    { 0x559235ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x995235bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x849235c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11235c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebd235c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9235ca, SUC_lexicon_tags_46 },
    { 0x7ef235cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde9235cd, SUC_lexicon_tags_131 },
    { 0x861235cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5235d1, SUC_lexicon_tags_37 },
    { 0xc15235d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7235d8, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x28b235da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1d1235dc, SUC_lexicon_tags_43 },
    { 0xb4d235dc, SUC_lexicon_tags_28 },
    { 0xfff235de, SUC_lexicon_tags_47 },
    { 0xcd3235dd, SUC_lexicon_tags_49 },
    { 0x65b235de, SUC_lexicon_tags_17 },
    { 0x149235df, SUC_lexicon_tags_14 },
    { 0x5d3235e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x999235e4, SUC_lexicon_tags_47 },
    { 0x37d235e5, SUC_lexicon_tags_37 },
    { 0x7b5235e5, SUC_lexicon_tags_54 },
    { 0x83d235e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b235eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f235f0, SUC_lexicon_tags_17 },
    { 0xf4d235f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b235f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59235fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf0b23600, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd3923602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c123605, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38f23607, SUC_lexicon_tags_17 },
    { 0xcdd23608, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7152360b, SUC_lexicon_tags_7 },
    { 0xce92360c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f52360f, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x123612, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d923614, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e123618, SUC_lexicon_tags_26 },
    { 0xf5723619, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d2361c, SUC_lexicon_tags_19 },
    { 0x672361d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xced2361f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0123621, SUC_lexicon_tags_2 },
    { 0xd3323622, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf832362c, SUC_lexicon_tags_846 },
    { 0xfd52362c, SUC_lexicon_tags_37 },
    { 0xd632362e, SUC_lexicon_tags_43 },
    { 0x62f2362e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x98523632, SUC_lexicon_tags_1 },
    { 0xdc923633, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8923638, SUC_lexicon_tags_45 },
    { 0xdcb23639, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d12363f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f723641, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9323643, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22123647, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc5323649, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f2364d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f23650, SUC_lexicon_tags_32 },
    { 0xe6d23651, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf23655, SUC_lexicon_tags_11 },
    { 0x7db23656, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28523659, SUC_lexicon_tags_381 },
    { 0xd012365a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8372365e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d23661, SUC_lexicon_tags_37 },
    { 0x2af23662, SUC_lexicon_tags_12 },
    { 0x64323663, SUC_lexicon_tags_1 },
    { 0xb9523664, SUC_lexicon_tags_31 },
    { 0xf5f23665, SUC_lexicon_tags_25 },
    { 0xa7f23665, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6123669, SUC_lexicon_tags_7 },
    { 0xf3f23669, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6db2366c, SUC_lexicon_tags_672 },
    { 0, NULL },
    { 0xad32366e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d23671, SUC_lexicon_tags_47 },
    { 0xcad23672, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x48123675, SUC_lexicon_tags_11 },
    { 0x74f23676, SUC_lexicon_tags_30 },
    { 0x52123677, SUC_lexicon_tags_47 },
    { 0x7cb23677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf2367b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2b2367d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4723680, SUC_lexicon_tags_12 },
    { 0xd2523681, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b923686, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6bb23688, SUC_lexicon_tags_23 },
    { 0x8f123689, SUC_lexicon_tags_7 },
    { 0x272368a, SUC_lexicon_tags_10 },
    { 0xee12368b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f23690, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26523692, SUC_lexicon_tags_27 },
    { 0x16523693, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30923695, SUC_lexicon_tags_2 },
    { 0x27f23696, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4523699, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb2369d, SUC_lexicon_tags_63 },
    { 0xabd2369e, SUC_lexicon_tags_27 },
    { 0xcc72369f, SUC_lexicon_tags_27 },
    { 0x83f2369d, SUC_lexicon_tags_46 },
    { 0xdf9236a1, SUC_lexicon_tags_37 },
    { 0x61d2369d, SUC_lexicon_tags_29 },
    { 0x843236a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa33236a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2d236a7, SUC_lexicon_tags_1 },
    { 0xe71236a7, SUC_lexicon_tags_2 },
    { 0xbbf236a9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf8b236ab, SUC_lexicon_tags_27 },
    { 0x7c3236ab, SUC_lexicon_tags_37 },
    { 0xfdf236ad, SUC_lexicon_tags_60 },
    { 0x9dd236ae, SUC_lexicon_tags_7 },
    { 0x1ed236af, SUC_lexicon_tags_37 },
    { 0x2e5236b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x957236b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x301236b6, SUC_lexicon_tags_5 },
    { 0x9a3236b7, SUC_lexicon_tags_11 },
    { 0x721236b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x929236ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x931236bc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7236bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79236c2, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb236ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff236cd, SUC_lexicon_tags_49 },
    { 0xe55236ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7236d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7236e0, SUC_lexicon_tags_31 },
    { 0xded236e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62b236e3, SUC_lexicon_tags_11 },
    { 0xb59236e4, SUC_lexicon_tags_17 },
    { 0xb49236e4, SUC_lexicon_tags_26 },
    { 0x4e5236e6, SUC_lexicon_tags_16 },
    { 0x49f236e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309236ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f236f3, SUC_lexicon_tags_56 },
    { 0x525236f4, SUC_lexicon_tags_37 },
    { 0xfe5236f5, SUC_lexicon_tags_0 },
    { 0x3b3236f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5236fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d236fe, SUC_lexicon_tags_19 },
    { 0xc3b236ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc9323701, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f323706, SUC_lexicon_tags_70 },
    { 0x3ef23706, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32370a, SUC_lexicon_tags_68 },
    { 0x83f2370b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f23710, SUC_lexicon_tags_6 },
    { 0x5c323711, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7b23713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab923717, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2b723719, SUC_lexicon_tags_7 },
    { 0x4152371a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64523721, SUC_lexicon_tags_11 },
    { 0xc1723721, SUC_lexicon_tags_23 },
    { 0x53f23723, SUC_lexicon_tags_83 },
    { 0xdc523724, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb23726, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12372a, SUC_lexicon_tags_25 },
    { 0x1952372b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf972372e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b523732, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xca323734, SUC_lexicon_tags_7 },
    { 0x4bd23735, SUC_lexicon_tags_55 },
    { 0x82923736, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a523738, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d2373d, SUC_lexicon_tags_17 },
    { 0x3692373e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b23742, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec923744, SUC_lexicon_tags_30 },
    { 0xbd723744, SUC_lexicon_tags_37 },
    { 0xab123746, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd23749, SUC_lexicon_tags_60 },
    { 0x9c12374a, SUC_lexicon_tags_51 },
    { 0xa8b2374b, SUC_lexicon_tags_11 },
    { 0x3732374c, SUC_lexicon_tags_3 },
    { 0x9a92374c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b23751, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d23754, SUC_lexicon_tags_17 },
    { 0x10523755, SUC_lexicon_tags_26 },
    { 0x98323756, SUC_lexicon_tags_0 },
    { 0x8d723757, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2375a, SUC_lexicon_tags_6 },
    { 0x4592375b, SUC_lexicon_tags_2 },
    { 0xd052375c, SUC_lexicon_tags_17 },
    { 0x432375c, SUC_lexicon_tags_11 },
    { 0x56f2375b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f23762, SUC_lexicon_tags_11 },
    { 0x66523763, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b2376f, SUC_lexicon_tags_15 },
    { 0x2bb23770, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e523772, SUC_lexicon_tags_17 },
    { 0xd1f23773, SUC_lexicon_tags_11 },
    { 0x8523774, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3f523776, SUC_lexicon_tags_27 },
    { 0x8723776, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80323779, SUC_lexicon_tags_0 },
    { 0xc72377a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb752377d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdd2377f, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb323783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c323789, SUC_lexicon_tags_37 },
    { 0xc032378a, SUC_lexicon_tags_16 },
    { 0x1592378a, SUC_lexicon_tags_11 },
    { 0x2132378c, SUC_lexicon_tags_14 },
    { 0xe4123789, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d923791, SUC_lexicon_tags_2 },
    { 0x2ad23792, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xadb23794, SUC_lexicon_tags_104 },
    { 0xa9523795, SUC_lexicon_tags_14 },
    { 0x41b23796, SUC_lexicon_tags_17 },
    { 0x10f23794, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe612379d, SUC_lexicon_tags_2 },
    { 0x52379e, SUC_lexicon_tags_28 },
    { 0x3672379f, SUC_lexicon_tags_28 },
    { 0x56b2379e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9237a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfbf237a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdaf237a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59237ae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb27237b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5237b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcf1237b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf69237b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d237bc, SUC_lexicon_tags_77 },
    { 0x8d9237bc, SUC_lexicon_tags_12 },
    { 0x721237bd, SUC_lexicon_tags_25 },
    { 0xc37237bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9237c2, SUC_lexicon_tags_23 },
    { 0xe65237c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf67237c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5237c7, SUC_lexicon_tags_10 },
    { 0x78b237c7, SUC_lexicon_tags_65 },
    { 0x781237c9, SUC_lexicon_tags_17 },
    { 0xc7f237ca, SUC_lexicon_tags_17 },
    { 0x13d237cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd237d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc53237d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff237d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x713237d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9237dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5237e4, SUC_lexicon_tags_11 },
    { 0x4e5237e5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd77237e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75237eb, SUC_lexicon_tags_15 },
    { 0x6bf237eb, SUC_lexicon_tags_27 },
    { 0x2b7237ed, SUC_lexicon_tags_0 },
    { 0xb65237ed, SUC_lexicon_tags_66 },
    { 0x307237ef, SUC_lexicon_tags_77 },
    { 0x8d3237f0, SUC_lexicon_tags_7 },
    { 0xc77237f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7237f4, SUC_lexicon_tags_25 },
    { 0xce3237f4, SUC_lexicon_tags_61 },
    { 0xf95237f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3f7237f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2db237fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xebd237fc, SUC_lexicon_tags_15 },
    { 0x86b237fd, SUC_lexicon_tags_12 },
    { 0x2b7237fd, SUC_lexicon_tags_39 },
    { 0xf15237ff, SUC_lexicon_tags_2 },
    { 0xaad237ff, SUC_lexicon_tags_12 },
    { 0xd05237fc, SUC_lexicon_tags_16 },
    { 0x18323802, SUC_lexicon_tags_17 },
    { 0x785237fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0323807, SUC_lexicon_tags_40 },
    { 0x23923807, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe92380a, SUC_lexicon_tags_1 },
    { 0x89b2380b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3812380f, SUC_lexicon_tags_57 },
    { 0x24323810, SUC_lexicon_tags_11 },
    { 0xe2d23811, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75923815, SUC_lexicon_tags_14 },
    { 0x53323816, SUC_lexicon_tags_45 },
    { 0xc9323816, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8272381a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab2381d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda52381f, SUC_lexicon_tags_0 },
    { 0x1a523820, SUC_lexicon_tags_174 },
    { 0xe7923820, SUC_lexicon_tags_14 },
    { 0x85d2381f, SUC_lexicon_tags_14 },
    { 0xd0b2381f, SUC_lexicon_tags_10 },
    { 0x43d23823, SUC_lexicon_tags_15 },
    { 0x5ff23825, SUC_lexicon_tags_29 },
    { 0x9a923822, SUC_lexicon_tags_17 },
    { 0x52b23822, SUC_lexicon_tags_6 },
    { 0xb2123828, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d2382b, SUC_lexicon_tags_16 },
    { 0x1bb2382c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d23831, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x59923833, SUC_lexicon_tags_17 },
    { 0xe1f23834, SUC_lexicon_tags_12 },
    { 0x3fb23835, SUC_lexicon_tags_12 },
    { 0x3d923836, SUC_lexicon_tags_16 },
    { 0x51b23837, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa32383a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0d2383c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaaf2383e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb23842, SUC_lexicon_tags_2 },
    { 0x12123843, SUC_lexicon_tags_11 },
    { 0x89123843, SUC_lexicon_tags_0 },
    { 0x1d323844, SUC_lexicon_tags_17 },
    { 0x68323846, SUC_lexicon_tags_17 },
    { 0xcc123847, SUC_lexicon_tags_49 },
    { 0xe2b23848, SUC_lexicon_tags_37 },
    { 0xac923849, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9f2384b, SUC_lexicon_tags_19 },
    { 0x19b2384c, SUC_lexicon_tags_10 },
    { 0xd8b2384d, SUC_lexicon_tags_7 },
    { 0xb8b2384d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b723855, SUC_lexicon_tags_1 },
    { 0x40323856, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5323858, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe292385a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb092385c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc332385f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9323865, SUC_lexicon_tags_23 },
    { 0x42323865, SUC_lexicon_tags_9 },
    { 0x67323867, SUC_lexicon_tags_79 },
    { 0xe4923868, SUC_lexicon_tags_33 },
    { 0x55123869, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab2386d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xad52386f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b23879, SUC_lexicon_tags_15 },
    { 0x4412387a, SUC_lexicon_tags_1 },
    { 0x48f2387b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea52387f, SUC_lexicon_tags_309 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4123886, SUC_lexicon_tags_60 },
    { 0xd8723887, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd92388d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc592388f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x68523892, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde523897, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa392389b, SUC_lexicon_tags_140 },
    { 0x1bd2389c, SUC_lexicon_tags_11 },
    { 0xfc32389b, SUC_lexicon_tags_12 },
    { 0xb852389b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x761238a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85d238a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5238ab, SUC_lexicon_tags_17 },
    { 0x8fd238ac, SUC_lexicon_tags_25 },
    { 0x643238ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7238b5, SUC_lexicon_tags_87 },
    { 0xe83238b6, SUC_lexicon_tags_48 },
    { 0x335238b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9238be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf43238c0, SUC_lexicon_tags_2 },
    { 0x201238c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af238c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5b238c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939238cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1bf238ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd47238d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf238d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd238d8, SUC_lexicon_tags_17 },
    { 0x869238d9, SUC_lexicon_tags_49 },
    { 0xbb9238da, SUC_lexicon_tags_37 },
    { 0xf55238db, SUC_lexicon_tags_6 },
    { 0x349238dc, SUC_lexicon_tags_39 },
    { 0x345238dd, SUC_lexicon_tags_26 },
    { 0xa4d238dc, SUC_lexicon_tags_59 },
    { 0x115238df, SUC_lexicon_tags_31 },
    { 0x9f9238e0, SUC_lexicon_tags_7 },
    { 0xf1238df, SUC_lexicon_tags_8 },
    { 0x237238df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x271238e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x957238e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf7238e8, SUC_lexicon_tags_11 },
    { 0xfb7238e9, SUC_lexicon_tags_2 },
    { 0x4bf238ea, SUC_lexicon_tags_37 },
    { 0x11238eb, SUC_lexicon_tags_11 },
    { 0x63238ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x647238ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6b238f0, SUC_lexicon_tags_1 },
    { 0x16d238f1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d238f5, SUC_lexicon_tags_12 },
    { 0xaab238f6, SUC_lexicon_tags_2 },
    { 0xdf1238f6, SUC_lexicon_tags_37 },
    { 0x671238f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x21b238fa, SUC_lexicon_tags_10 },
    { 0x8f9238fb, SUC_lexicon_tags_17 },
    { 0xf07238fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c123900, SUC_lexicon_tags_25 },
    { 0x4a923900, SUC_lexicon_tags_7 },
    { 0xdcd23902, SUC_lexicon_tags_25 },
    { 0xec723903, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0d23905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f323908, SUC_lexicon_tags_37 },
    { 0x1e723909, SUC_lexicon_tags_37 },
    { 0x5df2390a, SUC_lexicon_tags_14 },
    { 0x5af2390a, SUC_lexicon_tags_25 },
    { 0xdd123909, SUC_lexicon_tags_19 },
    { 0xb112390d, SUC_lexicon_tags_15 },
    { 0x61d2390d, SUC_lexicon_tags_37 },
    { 0xfcd2390f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb23912, SUC_lexicon_tags_11 },
    { 0x2323912, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9123917, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d2391a, SUC_lexicon_tags_17 },
    { 0xc6b2391a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7932391d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb832391f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x91723921, SUC_lexicon_tags_2 },
    { 0x4d923922, SUC_lexicon_tags_34 },
    { 0xa2523923, SUC_lexicon_tags_60 },
    { 0xf9d23924, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd23927, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x54d23929, SUC_lexicon_tags_66 },
    { 0xbdb23929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbd2392c, SUC_lexicon_tags_37 },
    { 0xb672392c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebf2392f, SUC_lexicon_tags_61 },
    { 0xeab2392f, SUC_lexicon_tags_38 },
    { 0xeaf2392f, SUC_lexicon_tags_7 },
    { 0xb2123932, SUC_lexicon_tags_2 },
    { 0xeb723933, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4d123935, SUC_lexicon_tags_1 },
    { 0x43523936, SUC_lexicon_tags_23 },
    { 0xf1f23936, SUC_lexicon_tags_17 },
    { 0x94123938, SUC_lexicon_tags_29 },
    { 0x4c123939, SUC_lexicon_tags_11 },
    { 0x68323938, SUC_lexicon_tags_13 },
    { 0x7f523938, SUC_lexicon_tags_17 },
    { 0xcb72393c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff32393e, SUC_lexicon_tags_47 },
    { 0x5b52393f, SUC_lexicon_tags_2 },
    { 0x65f23940, SUC_lexicon_tags_21 },
    { 0xe552393f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9123945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4892394f, SUC_lexicon_tags_55 },
    { 0xbc123950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68523955, SUC_lexicon_tags_11 },
    { 0xcd723956, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf92395d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab23964, SUC_lexicon_tags_23 },
    { 0x46f23965, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e92396a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x89f2396c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e72396f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff23972, SUC_lexicon_tags_23 },
    { 0x69323973, SUC_lexicon_tags_1 },
    { 0xf1723972, SUC_lexicon_tags_47 },
    { 0x9323973, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b23979, SUC_lexicon_tags_12 },
    { 0xb4d2397a, SUC_lexicon_tags_27 },
    { 0x4832397b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdd12397d, SUC_lexicon_tags_10 },
    { 0xf812397e, SUC_lexicon_tags_1 },
    { 0xb932397e, SUC_lexicon_tags_38 },
    { 0x6972397f, SUC_lexicon_tags_14 },
    { 0x38d2397f, SUC_lexicon_tags_15 },
    { 0xa8323982, SUC_lexicon_tags_28 },
    { 0xf0d23983, SUC_lexicon_tags_12 },
    { 0x86923984, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e323989, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89b2398b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b12398e, SUC_lexicon_tags_47 },
    { 0x4192398e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x3b723991, SUC_lexicon_tags_25 },
    { 0xe4123991, SUC_lexicon_tags_49 },
    { 0x39323993, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8192399d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3239a1, SUC_lexicon_tags_27 },
    { 0x9fd239a2, SUC_lexicon_tags_11 },
    { 0x7c5239a1, SUC_lexicon_tags_65 },
    { 0x999239a4, SUC_lexicon_tags_27 },
    { 0x779239a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x833239a8, SUC_lexicon_tags_8 },
    { 0xb3239a9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29239b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf49239b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaef239b5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05239b8, SUC_lexicon_tags_37 },
    { 0x1ff239b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d239bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef239bf, SUC_lexicon_tags_11 },
    { 0x15239c0, SUC_lexicon_tags_7 },
    { 0xd03239c1, SUC_lexicon_tags_37 },
    { 0xe7239c0, SUC_lexicon_tags_1 },
    { 0x977239bf, SUC_lexicon_tags_17 },
    { 0x437239c0, SUC_lexicon_tags_38 },
    { 0xbc5239c2, SUC_lexicon_tags_46 },
    { 0xf29239c1, SUC_lexicon_tags_12 },
    { 0x403239c6, SUC_lexicon_tags_16 },
    { 0x3e9239c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1fd239ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x469239cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d239cf, SUC_lexicon_tags_11 },
    { 0x759239d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69239d4, SUC_lexicon_tags_33 },
    { 0x51d239d5, SUC_lexicon_tags_26 },
    { 0x9e3239d6, SUC_lexicon_tags_45 },
    { 0xdd1239d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741239dc, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f239e0, SUC_lexicon_tags_12 },
    { 0x17d239e1, SUC_lexicon_tags_77 },
    { 0x729239e2, SUC_lexicon_tags_59 },
    { 0x445239e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x327239e6, SUC_lexicon_tags_64 },
    { 0x575239e6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbc7239e9, SUC_lexicon_tags_25 },
    { 0x1a7239ea, SUC_lexicon_tags_2 },
    { 0xa7d239eb, SUC_lexicon_tags_27 },
    { 0x83f239ec, SUC_lexicon_tags_47 },
    { 0x19b239e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34d239ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b239f2, SUC_lexicon_tags_14 },
    { 0x5f7239f2, SUC_lexicon_tags_1 },
    { 0x7b3239f4, SUC_lexicon_tags_37 },
    { 0xdd239f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x273239f7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8d1239f9, SUC_lexicon_tags_15 },
    { 0xb239fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8123a01, SUC_lexicon_tags_2 },
    { 0x57f23a01, SUC_lexicon_tags_65 },
    { 0xb3f23a01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48523a09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7723a0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea323a0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1523a10, SUC_lexicon_tags_17 },
    { 0x4bf23a11, SUC_lexicon_tags_20 },
    { 0x84f23a12, SUC_lexicon_tags_0 },
    { 0x55b23a12, SUC_lexicon_tags_33 },
    { 0xcf23a14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16f23a16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef23a19, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xec123a1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e923a1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x18123a1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6723a26, SUC_lexicon_tags_11 },
    { 0x9323a27, SUC_lexicon_tags_33 },
    { 0x82123a28, SUC_lexicon_tags_11 },
    { 0x8d523a28, SUC_lexicon_tags_8 },
    { 0xfbb23a26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad23a2d, SUC_lexicon_tags_27 },
    { 0x79523a2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88123a30, SUC_lexicon_tags_1 },
    { 0xcc923a31, SUC_lexicon_tags_4 },
    { 0xe1d23a32, SUC_lexicon_tags_17 },
    { 0x66723a33, SUC_lexicon_tags_18 },
    { 0xf3323a32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0923a36, SUC_lexicon_tags_65 },
    { 0x9d23a36, SUC_lexicon_tags_7 },
    { 0x9b723a36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa4d23a3a, SUC_lexicon_tags_12 },
    { 0x5d923a3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9b23a3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46523a3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1323a42, SUC_lexicon_tags_25 },
    { 0xf8f23a43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b723a46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51923a4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x20f23a4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc923a51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c323a54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d23a58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xab123a5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50f23a5d, SUC_lexicon_tags_16 },
    { 0x40f23a5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x80923a60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5123a63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbe323a65, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54323a67, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb23a6a, SUC_lexicon_tags_26 },
    { 0x84123a6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x59523a6d, SUC_lexicon_tags_354 },
    { 0xb3323a6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc7523a70, SUC_lexicon_tags_15 },
    { 0x2b923a70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b23a75, SUC_lexicon_tags_1 },
    { 0xcf523a76, SUC_lexicon_tags_91 },
    { 0x3cf23a77, SUC_lexicon_tags_12 },
    { 0xc9523a78, SUC_lexicon_tags_30 },
    { 0x15923a78, SUC_lexicon_tags_30 },
    { 0x46923a77, SUC_lexicon_tags_37 },
    { 0xccd23a7b, SUC_lexicon_tags_2 },
    { 0x71d23a7c, SUC_lexicon_tags_17 },
    { 0xfd723a7b, SUC_lexicon_tags_9 },
    { 0x98923a7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b323a80, SUC_lexicon_tags_1 },
    { 0x12923a81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c723a85, SUC_lexicon_tags_11 },
    { 0xff23a86, SUC_lexicon_tags_60 },
    { 0xa7723a87, SUC_lexicon_tags_15 },
    { 0x2bd23a88, SUC_lexicon_tags_7 },
    { 0x52523a89, SUC_lexicon_tags_17 },
    { 0x2b523a8a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c723a90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbf23a92, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x3d123a94, SUC_lexicon_tags_38 },
    { 0x3eb23a95, SUC_lexicon_tags_12 },
    { 0x77123a95, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6a523a98, SUC_lexicon_tags_17 },
    { 0x75923a98, SUC_lexicon_tags_25 },
    { 0x96b23a99, SUC_lexicon_tags_23 },
    { 0x76f23a9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf23a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79523aa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d23aa3, SUC_lexicon_tags_11 },
    { 0x89523aa3, SUC_lexicon_tags_14 },
    { 0xd2b23aa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe323aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bf23aaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d23ab4, SUC_lexicon_tags_42 },
    { 0x3d323ab4, SUC_lexicon_tags_13 },
    { 0x7ab23ab6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf723aba, SUC_lexicon_tags_15 },
    { 0x8dd23abb, SUC_lexicon_tags_37 },
    { 0x2f123abc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab23ac0, SUC_lexicon_tags_39 },
    { 0xfb323ac0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5323ac7, SUC_lexicon_tags_7 },
    { 0x38523ac8, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf723acf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec523ad1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x11f23ad3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94123ad9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x23323adc, SUC_lexicon_tags_11 },
    { 0x82b23add, SUC_lexicon_tags_68 },
    { 0x14923ade, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x47923ae1, SUC_lexicon_tags_30 },
    { 0x7e723ae2, SUC_lexicon_tags_25 },
    { 0xc3723ae2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f23ae8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x29923aeb, SUC_lexicon_tags_12 },
    { 0xf6523aec, SUC_lexicon_tags_3 },
    { 0x68123aed, SUC_lexicon_tags_11 },
    { 0x8ab23aed, SUC_lexicon_tags_8 },
    { 0x47b23aef, SUC_lexicon_tags_60 },
    { 0x88523aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b23af3, SUC_lexicon_tags_17 },
    { 0xdaf23af4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a123aff, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xce523b01, SUC_lexicon_tags_6 },
    { 0x81523b02, SUC_lexicon_tags_12 },
    { 0xd5123b03, SUC_lexicon_tags_11 },
    { 0xa8123b03, SUC_lexicon_tags_1 },
    { 0x75f23b04, SUC_lexicon_tags_1 },
    { 0x9ab23b06, SUC_lexicon_tags_2 },
    { 0x8d323b04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61f23b09, SUC_lexicon_tags_47 },
    { 0x48d23b0a, SUC_lexicon_tags_11 },
    { 0xe3123b0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x99723b0e, SUC_lexicon_tags_27 },
    { 0x4323b0f, SUC_lexicon_tags_66 },
    { 0xa3923b10, SUC_lexicon_tags_296 },
    { 0xba323b11, SUC_lexicon_tags_30 },
    { 0x8f323b11, SUC_lexicon_tags_38 },
    { 0x47f23b10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde923b15, SUC_lexicon_tags_27 },
    { 0xff923b16, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb23b22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2123b24, SUC_lexicon_tags_12 },
    { 0x29f23b25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d23b2c, SUC_lexicon_tags_16 },
    { 0x28323b2d, SUC_lexicon_tags_19 },
    { 0xd0323b2d, SUC_lexicon_tags_27 },
    { 0xacb23b2f, SUC_lexicon_tags_27 },
    { 0xa323b30, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb23b37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4323b39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56723b3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88523b3f, SUC_lexicon_tags_23 },
    { 0xf8323b40, SUC_lexicon_tags_48 },
    { 0xb723b3f, SUC_lexicon_tags_37 },
    { 0x4c323b41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1323b46, SUC_lexicon_tags_31 },
    { 0xa3f23b47, SUC_lexicon_tags_4 },
    { 0x30f23b47, SUC_lexicon_tags_12 },
    { 0xb6d23b48, SUC_lexicon_tags_38 },
    { 0x72123b46, SUC_lexicon_tags_1 },
    { 0x8eb23b4b, SUC_lexicon_tags_25 },
    { 0xfbb23b49, SUC_lexicon_tags_46 },
    { 0x81b23b4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3af23b4f, SUC_lexicon_tags_9 },
    { 0xe7123b50, SUC_lexicon_tags_15 },
    { 0xa0f23b51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x64123b53, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb23b5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc3123b5e, SUC_lexicon_tags_12 },
    { 0x9f523b5f, SUC_lexicon_tags_39 },
    { 0x60123b60, SUC_lexicon_tags_92 },
    { 0x9c923b60, SUC_lexicon_tags_30 },
    { 0x3ab23b60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33923b64, SUC_lexicon_tags_47 },
    { 0x51b23b64, SUC_lexicon_tags_85 },
    { 0xca123b66, SUC_lexicon_tags_15 },
    { 0x4723b66, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4923b6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2123b6c, SUC_lexicon_tags_14 },
    { 0xbc123b6d, SUC_lexicon_tags_27 },
    { 0x70723b6c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b123b71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa723b75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b723b78, SUC_lexicon_tags_15 },
    { 0xb23b79, SUC_lexicon_tags_118 },
    { 0x33523b7a, SUC_lexicon_tags_428 },
    { 0x3f523b7b, SUC_lexicon_tags_17 },
    { 0xb7723b7b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b923b81, SUC_lexicon_tags_49 },
    { 0x3b523b82, SUC_lexicon_tags_12 },
    { 0x96923b83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0923b87, SUC_lexicon_tags_37 },
    { 0x9f23b88, SUC_lexicon_tags_23 },
    { 0x3a723b89, SUC_lexicon_tags_43 },
    { 0x8ad23b87, SUC_lexicon_tags_63 },
    { 0x62723b8b, SUC_lexicon_tags_11 },
    { 0xe2923b8c, SUC_lexicon_tags_148 },
    { 0xaad23b87, SUC_lexicon_tags_21 },
    { 0xa0b23b8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17923b92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2123b9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f123ba0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9523ba3, SUC_lexicon_tags_12 },
    { 0x86923ba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b523ba6, SUC_lexicon_tags_16 },
    { 0x83123ba7, SUC_lexicon_tags_42 },
    { 0x80523ba8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24123baa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd23bb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f923bb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c323bbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a923bc1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b23bc7, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0x89523bc9, SUC_lexicon_tags_9 },
    { 0x38d23bca, SUC_lexicon_tags_11 },
    { 0x1dd23bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd723bd0, SUC_lexicon_tags_2 },
    { 0xc923bd1, SUC_lexicon_tags_19 },
    { 0x80523bd0, SUC_lexicon_tags_1 },
    { 0x9d523bd3, SUC_lexicon_tags_47 },
    { 0x39f23bd3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe4323bd6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd23bdb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc7f23bdd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xac123be0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f23be5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad923be9, SUC_lexicon_tags_15 },
    { 0x9c723bea, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c523bf0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d23bf3, SUC_lexicon_tags_48 },
    { 0x32523bf4, SUC_lexicon_tags_27 },
    { 0x69523bf5, SUC_lexicon_tags_23 },
    { 0xdff23bf5, SUC_lexicon_tags_1 },
    { 0x4fd23bf4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc9323bf9, SUC_lexicon_tags_12 },
    { 0x2a523bfa, SUC_lexicon_tags_7 },
    { 0x19f23bfb, SUC_lexicon_tags_160 },
    { 0x6ed23bfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab523c02, SUC_lexicon_tags_11 },
    { 0x47323c02, SUC_lexicon_tags_68 },
    { 0x6c523c04, SUC_lexicon_tags_25 },
    { 0xdbd23c05, SUC_lexicon_tags_32 },
    { 0x2cb23c06, SUC_lexicon_tags_126 },
    { 0x34323c06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41f23c09, SUC_lexicon_tags_27 },
    { 0xaa323c09, SUC_lexicon_tags_17 },
    { 0xdd723c0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b23c0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfd23c11, SUC_lexicon_tags_9 },
    { 0x56523c11, SUC_lexicon_tags_15 },
    { 0x60b23c13, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e523c18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b23c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd23c20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93523c24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e523c26, SUC_lexicon_tags_12 },
    { 0xdc123c27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79723c29, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x52723c2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95b23c2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc723c31, SUC_lexicon_tags_17 },
    { 0x2a323c31, SUC_lexicon_tags_27 },
    { 0x83f23c31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f23c37, SUC_lexicon_tags_60 },
    { 0x39d23c38, SUC_lexicon_tags_15 },
    { 0xc1723c37, SUC_lexicon_tags_27 },
    { 0xe3d23c3a, SUC_lexicon_tags_92 },
    { 0x18523c39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x15923c3e, SUC_lexicon_tags_43 },
    { 0xd4f23c3e, SUC_lexicon_tags_49 },
    { 0x94523c40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90923c45, SUC_lexicon_tags_25 },
    { 0x4d323c46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96f23c48, SUC_lexicon_tags_37 },
    { 0x4523c49, SUC_lexicon_tags_19 },
    { 0x56f23c4a, SUC_lexicon_tags_1 },
    { 0xe323c48, SUC_lexicon_tags_12 },
    { 0xb5f23c48, SUC_lexicon_tags_4 },
    { 0x9f723c4a, SUC_lexicon_tags_15 },
    { 0x85323c4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6923c51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a523c55, SUC_lexicon_tags_1 },
    { 0x78d23c56, SUC_lexicon_tags_65 },
    { 0x46b23c56, SUC_lexicon_tags_25 },
    { 0xfa923c58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72923c5a, SUC_lexicon_tags_16 },
    { 0x1d123c5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1723c60, SUC_lexicon_tags_15 },
    { 0xf1723c60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc8923c63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6523c65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f23c69, SUC_lexicon_tags_26 },
    { 0x44923c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c123c6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f23c70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a523c73, SUC_lexicon_tags_37 },
    { 0xa3f23c73, SUC_lexicon_tags_46 },
    { 0x8af23c73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80123c7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b23c7d, SUC_lexicon_tags_15 },
    { 0xe6123c7d, SUC_lexicon_tags_56 },
    { 0x95323c7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f23c83, SUC_lexicon_tags_38 },
    { 0x7b523c84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf123c8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd23c8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf23c90, SUC_lexicon_tags_25 },
    { 0x4df23c91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b23c94, SUC_lexicon_tags_25 },
    { 0xc2723c95, SUC_lexicon_tags_37 },
    { 0x3b123c96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff23c9e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6723ca3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f23cac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x16523caf, SUC_lexicon_tags_2 },
    { 0x5123cb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7523cb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf123cc1, SUC_lexicon_tags_17 },
    { 0xfe523cc2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7923cc5, SUC_lexicon_tags_11 },
    { 0x2b723cc5, SUC_lexicon_tags_13 },
    { 0xce923cc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86923ccb, SUC_lexicon_tags_12 },
    { 0x3f723ccc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6523cd2, SUC_lexicon_tags_1 },
    { 0x6c123cd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f23cd8, SUC_lexicon_tags_7 },
    { 0x79323cd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41d23cdb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xca323cdd, SUC_lexicon_tags_8 },
    { 0x66323cde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb23ce6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb23ce9, SUC_lexicon_tags_16 },
    { 0xe7f23cea, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e123cf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f23cf4, SUC_lexicon_tags_79 },
    { 0xc2323cf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf923cf9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x41d23cfb, SUC_lexicon_tags_17 },
    { 0xba123cfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2a323cfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f23d01, SUC_lexicon_tags_17 },
    { 0x62723d02, SUC_lexicon_tags_37 },
    { 0x33f23d03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x79b23d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d23d08, SUC_lexicon_tags_20 },
    { 0x24123d08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7923d0b, SUC_lexicon_tags_12 },
    { 0xc6b23d0c, SUC_lexicon_tags_19 },
    { 0x1cd23d0d, SUC_lexicon_tags_38 },
    { 0xc1523d0e, SUC_lexicon_tags_1 },
    { 0x43323d0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf923d16, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f23d19, SUC_lexicon_tags_21 },
    { 0x64323d1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1523d1f, SUC_lexicon_tags_17 },
    { 0x37523d20, SUC_lexicon_tags_11 },
    { 0xe3d23d20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc523d23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8923d25, SUC_lexicon_tags_45 },
    { 0x2ed23d26, SUC_lexicon_tags_82 },
    { 0x50f23d27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9723d2b, SUC_lexicon_tags_14 },
    { 0x72f23d2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7123d36, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f23d3d, SUC_lexicon_tags_15 },
    { 0x43923d3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1323d41, SUC_lexicon_tags_85 },
    { 0x3e923d41, SUC_lexicon_tags_25 },
    { 0xa7123d41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1923d48, SUC_lexicon_tags_32 },
    { 0x14f23d49, SUC_lexicon_tags_2 },
    { 0xbf723d48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6323d4d, SUC_lexicon_tags_7 },
    { 0x7c323d4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a723d50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd23d54, SUC_lexicon_tags_27 },
    { 0x1c123d55, SUC_lexicon_tags_68 },
    { 0xeb923d54, SUC_lexicon_tags_17 },
    { 0xe5b23d54, SUC_lexicon_tags_79 },
    { 0x39923d58, SUC_lexicon_tags_45 },
    { 0xc8323d55, SUC_lexicon_tags_38 },
    { 0xe7723d5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27d23d5c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d23d5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb23d63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d123d65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa923d6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfb23d6f, SUC_lexicon_tags_25 },
    { 0x7a323d70, SUC_lexicon_tags_0 },
    { 0xd9323d70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14523d7c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd6523d7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae523d80, SUC_lexicon_tags_14 },
    { 0xfc723d81, SUC_lexicon_tags_33 },
    { 0x5d523d82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb723d87, SUC_lexicon_tags_63 },
    { 0xb7323d88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20323d8c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8723d90, SUC_lexicon_tags_25 },
    { 0x47523d91, SUC_lexicon_tags_6 },
    { 0xfd323d92, SUC_lexicon_tags_2 },
    { 0x20323d91, SUC_lexicon_tags_37 },
    { 0x4bd23d94, SUC_lexicon_tags_2 },
    { 0x7f523d94, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf923d9a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb23d9f, SUC_lexicon_tags_17 },
    { 0x39b23da0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f23da2, SUC_lexicon_tags_2 },
    { 0x61123da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d23dae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa723db0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x11f23db2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e523db5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x22323db7, SUC_lexicon_tags_6 },
    { 0x33f23db7, SUC_lexicon_tags_11 },
    { 0x1d23db9, SUC_lexicon_tags_17 },
    { 0xe3d23db7, SUC_lexicon_tags_10 },
    { 0xc523dbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1523dbf, SUC_lexicon_tags_17 },
    { 0x8cd23dbf, SUC_lexicon_tags_33 },
    { 0x7f323dbf, SUC_lexicon_tags_0 },
    { 0xc3123dc2, SUC_lexicon_tags_43 },
    { 0x44b23dc3, SUC_lexicon_tags_1 },
    { 0xe5323dc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7df23dc6, SUC_lexicon_tags_7 },
    { 0x9af23dc6, SUC_lexicon_tags_15 },
    { 0x74f23dc8, SUC_lexicon_tags_2 },
    { 0x77923dc9, SUC_lexicon_tags_17 },
    { 0x25d23dca, SUC_lexicon_tags_16 },
    { 0x1ef23dc8, SUC_lexicon_tags_6 },
    { 0x5bd23dc9, SUC_lexicon_tags_12 },
    { 0x32723dcd, SUC_lexicon_tags_17 },
    { 0xd6123dcd, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b523dd4, SUC_lexicon_tags_7 },
    { 0xb2d23dd4, SUC_lexicon_tags_2 },
    { 0x9923dd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67523dd9, SUC_lexicon_tags_12 },
    { 0x10b23dda, SUC_lexicon_tags_12 },
    { 0x80d23dda, SUC_lexicon_tags_12 },
    { 0xd4923ddc, SUC_lexicon_tags_31 },
    { 0x42723ddb, SUC_lexicon_tags_45 },
    { 0xa6123dda, SUC_lexicon_tags_12 },
    { 0xb3923dd9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7523de7, SUC_lexicon_tags_17 },
    { 0x23f23de8, SUC_lexicon_tags_49 },
    { 0x50d23de9, SUC_lexicon_tags_45 },
    { 0x13b23dea, SUC_lexicon_tags_27 },
    { 0xf9323de9, SUC_lexicon_tags_25 },
    { 0xa1f23dec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad23def, SUC_lexicon_tags_1 },
    { 0x77f23def, SUC_lexicon_tags_11 },
    { 0x59d23df1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b23df6, SUC_lexicon_tags_24 },
    { 0xb3323df6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90323dfa, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71923e00, SUC_lexicon_tags_7 },
    { 0xeab23e01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28323e08, SUC_lexicon_tags_2 },
    { 0x19d23e09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe523e0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8723e13, SUC_lexicon_tags_7 },
    { 0xf2123e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7f23e16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26723e1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11923e22, SUC_lexicon_tags_17 },
    { 0xb8323e23, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2723e29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b23e2e, SUC_lexicon_tags_23 },
    { 0xe9923e2e, SUC_lexicon_tags_37 },
    { 0xa3f23e30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3323e37, SUC_lexicon_tags_70 },
    { 0x63b23e38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31f23e3a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b23e3e, SUC_lexicon_tags_31 },
    { 0x1323e3f, SUC_lexicon_tags_63 },
    { 0x1b23e3f, SUC_lexicon_tags_38 },
    { 0xfa123e41, SUC_lexicon_tags_37 },
    { 0x50323e41, SUC_lexicon_tags_11 },
    { 0x34723e43, SUC_lexicon_tags_25 },
    { 0xfb23e44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe3f23e46, SUC_lexicon_tags_11 },
    { 0x12723e47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f23e4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56523e4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c523e4e, SUC_lexicon_tags_32 },
    { 0xca523e4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa323e53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x90323e55, SUC_lexicon_tags_8 },
    { 0x90123e56, SUC_lexicon_tags_37 },
    { 0x70923e55, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97523e5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb23e5e, SUC_lexicon_tags_44 },
    { 0x69923e5f, SUC_lexicon_tags_11 },
    { 0xe7523e60, SUC_lexicon_tags_48 },
    { 0xc7b23e5e, SUC_lexicon_tags_37 },
    { 0xe9b23e62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55d23e64, SUC_lexicon_tags_68 },
    { 0x59523e65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b923e68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13723e6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e923e6e, SUC_lexicon_tags_45 },
    { 0x25923e6e, SUC_lexicon_tags_35 },
    { 0xe7f23e6e, SUC_lexicon_tags_23 },
    { 0x34323e6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d23e7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x11123e7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53523e7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5323e81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc723e87, SUC_lexicon_tags_25 },
    { 0x75923e88, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5323e92, SUC_lexicon_tags_26 },
    { 0xcd123e93, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43523e97, SUC_lexicon_tags_25 },
    { 0x3f23e98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x23323e9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7f23e9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b23ea1, SUC_lexicon_tags_17 },
    { 0x81d23ea2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f23ea7, SUC_lexicon_tags_43 },
    { 0xd5923ea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7123eae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5a923eb0, SUC_lexicon_tags_28 },
    { 0x6b123eb0, SUC_lexicon_tags_17 },
    { 0x39b23eb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56d23eb4, SUC_lexicon_tags_15 },
    { 0xedf23eb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaa723eb7, SUC_lexicon_tags_25 },
    { 0x7b723eb7, SUC_lexicon_tags_14 },
    { 0x58123eb9, SUC_lexicon_tags_18 },
    { 0xc1123eb9, SUC_lexicon_tags_37 },
    { 0x82523eba, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc6b23ebd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b23ec0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x19923ec2, SUC_lexicon_tags_32 },
    { 0x99523ec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd723eca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x21123ecc, SUC_lexicon_tags_46 },
    { 0xa4723ecd, SUC_lexicon_tags_11 },
    { 0x51d23ece, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x4f323ed0, SUC_lexicon_tags_17 },
    { 0x79523ed1, SUC_lexicon_tags_15 },
    { 0xb5723ed1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79323ed4, SUC_lexicon_tags_25 },
    { 0xf9123ed4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd123ed8, SUC_lexicon_tags_70 },
    { 0x37723ed8, SUC_lexicon_tags_37 },
    { 0x4523eda, SUC_lexicon_tags_25 },
    { 0x3b323edb, SUC_lexicon_tags_11 },
    { 0x6d723edb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f23edf, SUC_lexicon_tags_25 },
    { 0xeb523edf, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x26723ee3, SUC_lexicon_tags_31 },
    { 0xe1d23ee4, SUC_lexicon_tags_0 },
    { 0xdad23ee5, SUC_lexicon_tags_11 },
    { 0x77b23ee4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf923ef0, SUC_lexicon_tags_49 },
    { 0x30123ef0, SUC_lexicon_tags_218 },
    { 0x9bf23ef1, SUC_lexicon_tags_2 },
    { 0x58f23ef0, SUC_lexicon_tags_49 },
    { 0x8eb23ef1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x86723ef6, SUC_lexicon_tags_14 },
    { 0x29123ef7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f23efb, SUC_lexicon_tags_27 },
    { 0x13f23efc, SUC_lexicon_tags_42 },
    { 0x96523efd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f723f00, SUC_lexicon_tags_63 },
    { 0xb9923f00, SUC_lexicon_tags_27 },
    { 0xf523f02, SUC_lexicon_tags_23 },
    { 0x52323f02, SUC_lexicon_tags_17 },
    { 0x1b23f04, SUC_lexicon_tags_7 },
    { 0x14323f03, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91523f09, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3923f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a123f11, SUC_lexicon_tags_45 },
    { 0x2dd23f12, SUC_lexicon_tags_25 },
    { 0x17d23f11, SUC_lexicon_tags_11 },
    { 0xcb323f12, SUC_lexicon_tags_69 },
    { 0x64d23f15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0523f18, SUC_lexicon_tags_17 },
    { 0x18b23f18, SUC_lexicon_tags_17 },
    { 0x29723f1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f323f1c, SUC_lexicon_tags_2 },
    { 0x26523f1c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23123f28, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcb723f2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d23f2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9123f30, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8723f33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e323f36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d23f3a, SUC_lexicon_tags_11 },
    { 0xabf23f3a, SUC_lexicon_tags_32 },
    { 0xecb23f3c, SUC_lexicon_tags_47 },
    { 0xb1923f3b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd3f23f3f, SUC_lexicon_tags_108 },
    { 0x7f23f3f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3e723f42, SUC_lexicon_tags_17 },
    { 0xf6d23f43, SUC_lexicon_tags_39 },
    { 0x92923f44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x45b23f46, SUC_lexicon_tags_3 },
    { 0xc2d23f46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9f23f49, SUC_lexicon_tags_25 },
    { 0xb4f23f49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf23f4c, SUC_lexicon_tags_513 },
    { 0xdc323f4c, SUC_lexicon_tags_8 },
    { 0xccb23f4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x46723f51, SUC_lexicon_tags_140 },
    { 0x2323f52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d123f54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e723f56, SUC_lexicon_tags_12 },
    { 0xfa323f57, SUC_lexicon_tags_15 },
    { 0xcfb23f57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f23f5c, SUC_lexicon_tags_25 },
    { 0x14123f5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x51f23f5f, SUC_lexicon_tags_47 },
    { 0xa5f23f60, SUC_lexicon_tags_2 },
    { 0x5eb23f5f, SUC_lexicon_tags_2 },
    { 0x7ef23f62, SUC_lexicon_tags_63 },
    { 0xa1f23f5f, SUC_lexicon_tags_9 },
    { 0xb9123f63, SUC_lexicon_tags_29 },
    { 0x2d323f65, SUC_lexicon_tags_11 },
    { 0xe4523f66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7523f6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb23f6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73723f74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x87723f77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b23f7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b923f7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cd23f7f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb23f83, SUC_lexicon_tags_25 },
    { 0x65523f84, SUC_lexicon_tags_15 },
    { 0x50b23f85, SUC_lexicon_tags_43 },
    { 0x9b323f86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xae923f88, SUC_lexicon_tags_19 },
    { 0x38523f88, SUC_lexicon_tags_33 },
    { 0x6eb23f8a, SUC_lexicon_tags_25 },
    { 0xf6123f8b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xef523f8d, SUC_lexicon_tags_63 },
    { 0x3f723f8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb23f91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d23f95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b923f97, SUC_lexicon_tags_572 },
    { 0xeeb23f98, SUC_lexicon_tags_7 },
    { 0x7b523f99, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d123f9b, SUC_lexicon_tags_10 },
    { 0xd8123f9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5923f9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3123fa0, SUC_lexicon_tags_14 },
    { 0x9e123fa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef23fa5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x14923fa7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8123fae, SUC_lexicon_tags_25 },
    { 0x8e123faf, SUC_lexicon_tags_37 },
    { 0xe8523fae, SUC_lexicon_tags_9 },
    { 0xf6d23fb1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd23fb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee323fbf, SUC_lexicon_tags_25 },
    { 0x4f723fc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a323fc2, SUC_lexicon_tags_19 },
    { 0xe1d23fc3, SUC_lexicon_tags_23 },
    { 0xc7b23fc3, SUC_lexicon_tags_37 },
    { 0x38f23fc5, SUC_lexicon_tags_49 },
    { 0x66f23fc6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x55323fc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95f23fca, SUC_lexicon_tags_25 },
    { 0x9a723fcb, SUC_lexicon_tags_84 },
    { 0x8a523fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20323fce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb23fd1, SUC_lexicon_tags_12 },
    { 0x5ef23fd1, SUC_lexicon_tags_16 },
    { 0x75123fd3, SUC_lexicon_tags_26 },
    { 0x22723fd3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9b23fd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6323fd9, SUC_lexicon_tags_17 },
    { 0xed723fda, SUC_lexicon_tags_37 },
    { 0x15b23fda, SUC_lexicon_tags_30 },
    { 0x4d123fd9, SUC_lexicon_tags_27 },
    { 0xc1d23fdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2523fe2, SUC_lexicon_tags_37 },
    { 0xff323fe3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62923fe7, SUC_lexicon_tags_26 },
    { 0x5ad23fe8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xde923feb, SUC_lexicon_tags_2 },
    { 0x1b923fec, SUC_lexicon_tags_17 },
    { 0x95323fed, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b323ff8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb23ffc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x94523ffe, SUC_lexicon_tags_11 },
    { 0x6e323fff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d924002, SUC_lexicon_tags_23 },
    { 0xf9f24002, SUC_lexicon_tags_17 },
    { 0xe2324004, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe124006, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c92400a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d12400d, SUC_lexicon_tags_66 },
    { 0xcf12400e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc0b24010, SUC_lexicon_tags_2 },
    { 0xe0924011, SUC_lexicon_tags_37 },
    { 0x9db24012, SUC_lexicon_tags_37 },
    { 0x25724012, SUC_lexicon_tags_19 },
    { 0xfd324011, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d24018, SUC_lexicon_tags_63 },
    { 0xa9124019, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc552401b, SUC_lexicon_tags_1 },
    { 0x992401c, SUC_lexicon_tags_34 },
    { 0x8c12401b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0124021, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe3f24023, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x44524025, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6124027, SUC_lexicon_tags_62 },
    { 0x95524027, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda32402d, SUC_lexicon_tags_14 },
    { 0x51b2402e, SUC_lexicon_tags_37 },
    { 0x1e32402d, SUC_lexicon_tags_49 },
    { 0xb92402d, SUC_lexicon_tags_14 },
    { 0x6fd24031, SUC_lexicon_tags_60 },
    { 0xc724032, SUC_lexicon_tags_28 },
    { 0x48f24033, SUC_lexicon_tags_25 },
    { 0xe0124033, SUC_lexicon_tags_7 },
    { 0x11924035, SUC_lexicon_tags_31 },
    { 0xe8724036, SUC_lexicon_tags_12 },
    { 0x3e324035, SUC_lexicon_tags_37 },
    { 0xe6f24035, SUC_lexicon_tags_11 },
    { 0xc5124039, SUC_lexicon_tags_11 },
    { 0x8f52403a, SUC_lexicon_tags_25 },
    { 0x57724034, SUC_lexicon_tags_15 },
    { 0x8724039, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd24044, SUC_lexicon_tags_129 },
    { 0x7b924044, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f924048, SUC_lexicon_tags_48 },
    { 0x1ad24048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea92404c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xad12404e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef24052, SUC_lexicon_tags_47 },
    { 0x12524052, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9924058, SUC_lexicon_tags_27 },
    { 0xb0524058, SUC_lexicon_tags_0 },
    { 0x4fb24059, SUC_lexicon_tags_1 },
    { 0x3812405b, SUC_lexicon_tags_15 },
    { 0xd6b2405c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea32405e, SUC_lexicon_tags_14 },
    { 0x4d32405f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32924065, SUC_lexicon_tags_2 },
    { 0xbc924066, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1b24068, SUC_lexicon_tags_37 },
    { 0x27924068, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5312406b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd672406f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb24074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38b24076, SUC_lexicon_tags_45 },
    { 0xc5924077, SUC_lexicon_tags_17 },
    { 0xb7924078, SUC_lexicon_tags_15 },
    { 0x9ef24079, SUC_lexicon_tags_45 },
    { 0xe8324076, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8cf2407c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87d2407e, SUC_lexicon_tags_39 },
    { 0xb4b2407f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9524082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f24086, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x11d24088, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x2892408b, SUC_lexicon_tags_87 },
    { 0x68b2408b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7632408e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7324092, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x31724094, SUC_lexicon_tags_1 },
    { 0x1eb24094, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4524097, SUC_lexicon_tags_7 },
    { 0x31124098, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x68b2409a, SUC_lexicon_tags_25 },
    { 0x58b2409b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe4f2409d, SUC_lexicon_tags_38 },
    { 0xc852409e, SUC_lexicon_tags_9 },
    { 0x1712409f, SUC_lexicon_tags_23 },
    { 0x2a72409e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b240a3, SUC_lexicon_tags_28 },
    { 0x257240a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215240a8, SUC_lexicon_tags_114 },
    { 0x91240a8, SUC_lexicon_tags_7 },
    { 0xc49240a8, SUC_lexicon_tags_25 },
    { 0xac3240ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdad240ad, SUC_lexicon_tags_7 },
    { 0x44d240ae, SUC_lexicon_tags_47 },
    { 0x6a3240af, SUC_lexicon_tags_47 },
    { 0xaa1240af, SUC_lexicon_tags_66 },
    { 0x36d240b1, SUC_lexicon_tags_37 },
    { 0xb97240b2, SUC_lexicon_tags_4 },
    { 0x243240b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed240b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d240bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f240bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x783240c2, SUC_lexicon_tags_7 },
    { 0x8bf240c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f240c6, SUC_lexicon_tags_6 },
    { 0x4f240c7, SUC_lexicon_tags_1 },
    { 0xe59240c8, SUC_lexicon_tags_7 },
    { 0xe4f240c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac1240cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5240d8, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7240db, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc87240dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47d240df, SUC_lexicon_tags_37 },
    { 0x629240e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3240e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b7240e6, SUC_lexicon_tags_45 },
    { 0xcad240e6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcf1240e9, SUC_lexicon_tags_7 },
    { 0xb23240ea, SUC_lexicon_tags_43 },
    { 0xa2b240eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3d240ed, SUC_lexicon_tags_11 },
    { 0x301240ed, SUC_lexicon_tags_37 },
    { 0xad3240ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5240f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1240fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfff240fc, SUC_lexicon_tags_2 },
    { 0x6bf240fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddd240ff, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x62f24101, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c924105, SUC_lexicon_tags_33 },
    { 0xd9f24105, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb752410e, SUC_lexicon_tags_60 },
    { 0x11b2410f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe924115, SUC_lexicon_tags_25 },
    { 0xc3724116, SUC_lexicon_tags_15 },
    { 0x24d24117, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3724119, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8352411c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f24127, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b2412c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b24137, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5ff24139, SUC_lexicon_tags_63 },
    { 0x67d2413a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed2413f, SUC_lexicon_tags_25 },
    { 0xd6724140, SUC_lexicon_tags_27 },
    { 0x8bd2413f, SUC_lexicon_tags_17 },
    { 0x33b2413f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1252414a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcaf2414c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x852414e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x524153, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38724157, SUC_lexicon_tags_37 },
    { 0xca124158, SUC_lexicon_tags_27 },
    { 0x51124159, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc532415b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7832415f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b24161, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x60324163, SUC_lexicon_tags_10 },
    { 0x69b24163, SUC_lexicon_tags_11 },
    { 0x17924165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b724167, SUC_lexicon_tags_6 },
    { 0xfcb24168, SUC_lexicon_tags_1 },
    { 0x22124169, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9f2416b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3492416d, SUC_lexicon_tags_6 },
    { 0x3472416e, SUC_lexicon_tags_8 },
    { 0xbe12416d, SUC_lexicon_tags_17 },
    { 0x3fb24170, SUC_lexicon_tags_27 },
    { 0xbef24171, SUC_lexicon_tags_4 },
    { 0xbeb24172, SUC_lexicon_tags_6 },
    { 0x13d24173, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f52417a, SUC_lexicon_tags_17 },
    { 0xec52417b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcf2417d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f924180, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d124183, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed324187, SUC_lexicon_tags_552 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4792418b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9492418d, SUC_lexicon_tags_14 },
    { 0x9db2418d, SUC_lexicon_tags_0 },
    { 0x81f2418e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d124193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f24196, SUC_lexicon_tags_17 },
    { 0xf8324197, SUC_lexicon_tags_22 },
    { 0xe9724197, SUC_lexicon_tags_19 },
    { 0xebf24199, SUC_lexicon_tags_23 },
    { 0xa8b24199, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c92419d, SUC_lexicon_tags_7 },
    { 0x7232419e, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x281241a0, SUC_lexicon_tags_84 },
    { 0xc85241a1, SUC_lexicon_tags_17 },
    { 0x7c5241a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda1241a4, SUC_lexicon_tags_37 },
    { 0xf93241a4, SUC_lexicon_tags_61 },
    { 0x661241a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x115241a9, SUC_lexicon_tags_63 },
    { 0x135241aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d241ac, SUC_lexicon_tags_0 },
    { 0x4a1241ad, SUC_lexicon_tags_27 },
    { 0x71d241ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ff241b0, SUC_lexicon_tags_28 },
    { 0x45241b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x461241b4, SUC_lexicon_tags_37 },
    { 0x25f241b5, SUC_lexicon_tags_2 },
    { 0x637241b6, SUC_lexicon_tags_37 },
    { 0xef241b6, SUC_lexicon_tags_4 },
    { 0x845241b8, SUC_lexicon_tags_12 },
    { 0x8b1241b9, SUC_lexicon_tags_12 },
    { 0x6b9241ba, SUC_lexicon_tags_7 },
    { 0x5e1241b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd241bf, SUC_lexicon_tags_0 },
    { 0x7241bf, SUC_lexicon_tags_17 },
    { 0xa4f241c1, SUC_lexicon_tags_26 },
    { 0xb5f241c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79b241c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xefd241c6, SUC_lexicon_tags_15 },
    { 0x26d241c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa15241c9, SUC_lexicon_tags_11 },
    { 0xe99241c9, SUC_lexicon_tags_37 },
    { 0x4e7241c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33241cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x201241d1, SUC_lexicon_tags_31 },
    { 0x331241d2, SUC_lexicon_tags_2 },
    { 0x49241d3, SUC_lexicon_tags_0 },
    { 0xe51241d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461241da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3241de, SUC_lexicon_tags_46 },
    { 0x22f241de, SUC_lexicon_tags_38 },
    { 0x443241e0, SUC_lexicon_tags_17 },
    { 0x16b241e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb241e3, SUC_lexicon_tags_204 },
    { 0x485241e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b241e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5241ec, SUC_lexicon_tags_103 },
    { 0x291241ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67241f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd241f9, SUC_lexicon_tags_14 },
    { 0xf83241fa, SUC_lexicon_tags_15 },
    { 0xe91241fb, SUC_lexicon_tags_65 },
    { 0x925241fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25f241fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x93f24200, SUC_lexicon_tags_0 },
    { 0x3f324201, SUC_lexicon_tags_7 },
    { 0x7f524202, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd372420a, SUC_lexicon_tags_27 },
    { 0xdfd2420b, SUC_lexicon_tags_11 },
    { 0x1ed2420c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f324212, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8724215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5324219, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa012421c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f32422a, SUC_lexicon_tags_62 },
    { 0x2592422b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda12422d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9072422f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b24232, SUC_lexicon_tags_62 },
    { 0xecb24232, SUC_lexicon_tags_17 },
    { 0xb2324232, SUC_lexicon_tags_2 },
    { 0xdbf24235, SUC_lexicon_tags_63 },
    { 0xf4724236, SUC_lexicon_tags_17 },
    { 0xf3924232, SUC_lexicon_tags_8 },
    { 0xc0b24238, SUC_lexicon_tags_28 },
    { 0x4df24239, SUC_lexicon_tags_29 },
    { 0x9192423a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f24247, SUC_lexicon_tags_85 },
    { 0xdd524248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9692424a, SUC_lexicon_tags_53 },
    { 0xadf2424b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb732424e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba724258, SUC_lexicon_tags_27 },
    { 0xbb524259, SUC_lexicon_tags_23 },
    { 0x1f92425a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d324260, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf24265, SUC_lexicon_tags_17 },
    { 0xe8d24266, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b2426a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe5f2426c, SUC_lexicon_tags_7 },
    { 0xd832426c, SUC_lexicon_tags_10 },
    { 0xd7f2426c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce324270, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee724274, SUC_lexicon_tags_17 },
    { 0x9df24275, SUC_lexicon_tags_17 },
    { 0x8cb24276, SUC_lexicon_tags_17 },
    { 0x28724276, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6b24279, SUC_lexicon_tags_37 },
    { 0x2fd2427a, SUC_lexicon_tags_56 },
    { 0xb0b24279, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9c72427d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c52427f, SUC_lexicon_tags_37 },
    { 0xeb524280, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f24288, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x27d2428a, SUC_lexicon_tags_4 },
    { 0x5fd2428b, SUC_lexicon_tags_82 },
    { 0xbcf2428c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98524296, SUC_lexicon_tags_17 },
    { 0x1e124296, SUC_lexicon_tags_25 },
    { 0xee324298, SUC_lexicon_tags_1 },
    { 0x60324296, SUC_lexicon_tags_23 },
    { 0x9712429a, SUC_lexicon_tags_7 },
    { 0xacb24299, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b2429f, SUC_lexicon_tags_79 },
    { 0x4a9242a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df242a3, SUC_lexicon_tags_25 },
    { 0xd79242a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b242ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x909242ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd242b4, SUC_lexicon_tags_19 },
    { 0x115242b4, SUC_lexicon_tags_29 },
    { 0x521242b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf85242b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c7242ba, SUC_lexicon_tags_8 },
    { 0xf0d242bb, SUC_lexicon_tags_15 },
    { 0xd2f242bc, SUC_lexicon_tags_25 },
    { 0xfab242bb, SUC_lexicon_tags_0 },
    { 0xf6d242bd, SUC_lexicon_tags_55 },
    { 0xf5b242bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639242c5, SUC_lexicon_tags_61 },
    { 0xf01242c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b7242c8, SUC_lexicon_tags_48 },
    { 0x1b242c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7242cf, SUC_lexicon_tags_27 },
    { 0x1d7242d0, SUC_lexicon_tags_5 },
    { 0x33b242d0, SUC_lexicon_tags_28 },
    { 0x825242d2, SUC_lexicon_tags_106 },
    { 0x819242d3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1242d6, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9242df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d242e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5af242e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x943242e6, SUC_lexicon_tags_18 },
    { 0xd21242e6, SUC_lexicon_tags_37 },
    { 0x917242e8, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d242eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7242f0, SUC_lexicon_tags_46 },
    { 0x609242f1, SUC_lexicon_tags_9 },
    { 0x6a5242f1, SUC_lexicon_tags_38 },
    { 0x94b242f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5242f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1242fa, SUC_lexicon_tags_38 },
    { 0x733242fb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x18b242fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa324301, SUC_lexicon_tags_14 },
    { 0x1d524302, SUC_lexicon_tags_17 },
    { 0xee324302, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3592430b, SUC_lexicon_tags_7 },
    { 0xdf92430b, SUC_lexicon_tags_37 },
    { 0x6432430d, SUC_lexicon_tags_11 },
    { 0x1872430b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa524310, SUC_lexicon_tags_6 },
    { 0xa9524311, SUC_lexicon_tags_12 },
    { 0x3b124312, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db24316, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c52431a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b2431d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48924322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4124324, SUC_lexicon_tags_37 },
    { 0xe9924324, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69524331, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5324335, SUC_lexicon_tags_47 },
    { 0x74724336, SUC_lexicon_tags_26 },
    { 0xff524336, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3eb24339, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb2433d, SUC_lexicon_tags_12 },
    { 0x2e12433d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b24342, SUC_lexicon_tags_1 },
    { 0x29d24343, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x24124345, SUC_lexicon_tags_10 },
    { 0xc8b24345, SUC_lexicon_tags_38 },
    { 0xe124345, SUC_lexicon_tags_7 },
    { 0xaef24346, SUC_lexicon_tags_17 },
    { 0xd4324349, SUC_lexicon_tags_178 },
    { 0x64b2434a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9412434d, SUC_lexicon_tags_8 },
    { 0xd92434e, SUC_lexicon_tags_10 },
    { 0xbe12434f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b24352, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f24356, SUC_lexicon_tags_316 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd24359, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a72435b, SUC_lexicon_tags_8 },
    { 0x7ab2435b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1724360, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31324362, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb24366, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d24369, SUC_lexicon_tags_19 },
    { 0x5924369, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf612436f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x88d24371, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d2437d, SUC_lexicon_tags_2 },
    { 0x7172437d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1df24380, SUC_lexicon_tags_2 },
    { 0x32124380, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd24386, SUC_lexicon_tags_92 },
    { 0xee324387, SUC_lexicon_tags_4 },
    { 0x81124388, SUC_lexicon_tags_25 },
    { 0x58924387, SUC_lexicon_tags_1 },
    { 0xdf12438a, SUC_lexicon_tags_17 },
    { 0x9924388, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6872438d, SUC_lexicon_tags_37 },
    { 0xfd72438e, SUC_lexicon_tags_38 },
    { 0x3c32438f, SUC_lexicon_tags_8 },
    { 0xe4324390, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcff24392, SUC_lexicon_tags_27 },
    { 0xed524393, SUC_lexicon_tags_1 },
    { 0x86b24393, SUC_lexicon_tags_63 },
    { 0x7f324395, SUC_lexicon_tags_1 },
    { 0x72124393, SUC_lexicon_tags_2 },
    { 0x80f24397, SUC_lexicon_tags_25 },
    { 0xd6324398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2d2439a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2092439d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9243a8, SUC_lexicon_tags_1 },
    { 0x3e7243a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1243ad, SUC_lexicon_tags_45 },
    { 0xced243ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d243b1, SUC_lexicon_tags_25 },
    { 0x429243b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85243b5, SUC_lexicon_tags_11 },
    { 0xcc9243b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297243bc, SUC_lexicon_tags_12 },
    { 0x4bb243bd, SUC_lexicon_tags_32 },
    { 0xc57243bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbdd243c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbad243c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc7243c4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab243c7, SUC_lexicon_tags_23 },
    { 0xf5d243c8, SUC_lexicon_tags_11 },
    { 0xd9b243c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7243cd, SUC_lexicon_tags_15 },
    { 0xf3d243ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d243d3, SUC_lexicon_tags_12 },
    { 0x207243d4, SUC_lexicon_tags_30 },
    { 0x797243d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f243d8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x841243da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x269243dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x553243de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x723243e0, SUC_lexicon_tags_25 },
    { 0x22d243e1, SUC_lexicon_tags_25 },
    { 0x573243e2, SUC_lexicon_tags_0 },
    { 0x7ff243e3, SUC_lexicon_tags_43 },
    { 0x7f3243e4, SUC_lexicon_tags_11 },
    { 0x2d1243e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f5243e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf243f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d243fc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d24400, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58724404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2124406, SUC_lexicon_tags_66 },
    { 0x1fb24407, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a32440c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaed2440e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b24412, SUC_lexicon_tags_43 },
    { 0x97724412, SUC_lexicon_tags_23 },
    { 0xde324412, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b12441a, SUC_lexicon_tags_17 },
    { 0x7192441b, SUC_lexicon_tags_27 },
    { 0x22f2441b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f24423, SUC_lexicon_tags_1 },
    { 0xefd24424, SUC_lexicon_tags_31 },
    { 0x9d924425, SUC_lexicon_tags_7 },
    { 0xc1d24425, SUC_lexicon_tags_15 },
    { 0xc324424, SUC_lexicon_tags_57 },
    { 0x7924428, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb924433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36724435, SUC_lexicon_tags_66 },
    { 0xc2124436, SUC_lexicon_tags_49 },
    { 0x72b24435, SUC_lexicon_tags_7 },
    { 0x71b24438, SUC_lexicon_tags_8 },
    { 0x2eb24438, SUC_lexicon_tags_11 },
    { 0x8ad2443a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb2443d, SUC_lexicon_tags_46 },
    { 0x8172443e, SUC_lexicon_tags_1 },
    { 0x3c12443d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb24442, SUC_lexicon_tags_7 },
    { 0xeb24442, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f24445, SUC_lexicon_tags_7 },
    { 0xd7124446, SUC_lexicon_tags_17 },
    { 0xd9b24447, SUC_lexicon_tags_7 },
    { 0x7eb24446, SUC_lexicon_tags_37 },
    { 0x71324448, SUC_lexicon_tags_32 },
    { 0x55f24449, SUC_lexicon_tags_7 },
    { 0xee524447, SUC_lexicon_tags_46 },
    { 0x2232444c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3fb2444e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40b24450, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb924456, SUC_lexicon_tags_19 },
    { 0x54324457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e12445d, SUC_lexicon_tags_52 },
    { 0x9132445d, SUC_lexicon_tags_11 },
    { 0x7e32445e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd5924461, SUC_lexicon_tags_2 },
    { 0xf1324462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b24468, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd2446d, SUC_lexicon_tags_77 },
    { 0xb412446e, SUC_lexicon_tags_37 },
    { 0xaa52446f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b24474, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3324484, SUC_lexicon_tags_37 },
    { 0x5d924485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6d24487, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb24494, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b24498, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e92449c, SUC_lexicon_tags_68 },
    { 0x4652449c, SUC_lexicon_tags_30 },
    { 0xf5b2449e, SUC_lexicon_tags_68 },
    { 0x4132449f, SUC_lexicon_tags_57 },
    { 0xbfb2449f, SUC_lexicon_tags_13 },
    { 0x77d2449c, SUC_lexicon_tags_118 },
    { 0xf6d244a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3f244a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d244a8, SUC_lexicon_tags_138 },
    { 0x287244a9, SUC_lexicon_tags_31 },
    { 0xf5d244aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17244ac, SUC_lexicon_tags_12 },
    { 0x56b244ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb244b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x255244b9, SUC_lexicon_tags_0 },
    { 0xbbb244ba, SUC_lexicon_tags_49 },
    { 0x729244bb, SUC_lexicon_tags_11 },
    { 0xc7b244bc, SUC_lexicon_tags_37 },
    { 0x95b244bd, SUC_lexicon_tags_7 },
    { 0x253244be, SUC_lexicon_tags_22 },
    { 0x8d7244bb, SUC_lexicon_tags_60 },
    { 0xeb244c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x903244c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79244c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b7244c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf8b244ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe47244cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x777244ce, SUC_lexicon_tags_48 },
    { 0xe6b244cf, SUC_lexicon_tags_9 },
    { 0x989244ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89244d5, SUC_lexicon_tags_60 },
    { 0x41f244d6, SUC_lexicon_tags_11 },
    { 0xf19244d6, SUC_lexicon_tags_49 },
    { 0xc83244d8, SUC_lexicon_tags_25 },
    { 0x6d1244d9, SUC_lexicon_tags_25 },
    { 0x93244d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc07244dc, SUC_lexicon_tags_1 },
    { 0x175244dd, SUC_lexicon_tags_63 },
    { 0xd7244de, SUC_lexicon_tags_0 },
    { 0x909244de, SUC_lexicon_tags_6 },
    { 0x137244df, SUC_lexicon_tags_17 },
    { 0xa45244e1, SUC_lexicon_tags_2 },
    { 0x615244e2, SUC_lexicon_tags_63 },
    { 0xf79244e3, SUC_lexicon_tags_63 },
    { 0xb13244e4, SUC_lexicon_tags_87 },
    { 0x777244e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3244e7, SUC_lexicon_tags_393 },
    { 0xe7f244e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x961244ea, SUC_lexicon_tags_16 },
    { 0x201244eb, SUC_lexicon_tags_0 },
    { 0xffd244ec, SUC_lexicon_tags_3 },
    { 0x483244ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed244f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7244f4, SUC_lexicon_tags_27 },
    { 0xa2f244f4, SUC_lexicon_tags_11 },
    { 0x34f244f4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f244fa, SUC_lexicon_tags_0 },
    { 0x415244fb, SUC_lexicon_tags_18 },
    { 0x9a9244fa, SUC_lexicon_tags_15 },
    { 0x51b244fc, SUC_lexicon_tags_17 },
    { 0xbfd244fd, SUC_lexicon_tags_40 },
    { 0x2dd244ff, SUC_lexicon_tags_25 },
    { 0x93d244ff, SUC_lexicon_tags_28 },
    { 0x1b9244fe, SUC_lexicon_tags_15 },
    { 0xb5d24502, SUC_lexicon_tags_47 },
    { 0x5cb24503, SUC_lexicon_tags_25 },
    { 0xbe524504, SUC_lexicon_tags_12 },
    { 0x3f324502, SUC_lexicon_tags_146 },
    { 0x90124502, SUC_lexicon_tags_1 },
    { 0x2af24502, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd2450a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b24518, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5032451e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa4324520, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43924525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae924529, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c32452d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac324535, SUC_lexicon_tags_3 },
    { 0xc0d24535, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb2453e, SUC_lexicon_tags_17 },
    { 0x6292453f, SUC_lexicon_tags_49 },
    { 0x6bd24540, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdd124542, SUC_lexicon_tags_1 },
    { 0xc5324543, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfbd24545, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6124548, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x35b2454a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14324556, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db2455c, SUC_lexicon_tags_16 },
    { 0x6f2455c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d24569, SUC_lexicon_tags_35 },
    { 0x60324569, SUC_lexicon_tags_9 },
    { 0x3512456b, SUC_lexicon_tags_1 },
    { 0x752456c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b24572, SUC_lexicon_tags_7 },
    { 0x4f724573, SUC_lexicon_tags_87 },
    { 0xa1524572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab24579, SUC_lexicon_tags_2 },
    { 0x2ed2457a, SUC_lexicon_tags_7 },
    { 0x5852457a, SUC_lexicon_tags_269 },
    { 0x1c724579, SUC_lexicon_tags_70 },
    { 0xce12457d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae924582, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93b24584, SUC_lexicon_tags_66 },
    { 0xfef24584, SUC_lexicon_tags_38 },
    { 0x3f924585, SUC_lexicon_tags_33 },
    { 0x71924587, SUC_lexicon_tags_11 },
    { 0xf1124587, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f2458d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a72458f, SUC_lexicon_tags_11 },
    { 0xb6124590, SUC_lexicon_tags_14 },
    { 0x2ef24590, SUC_lexicon_tags_11 },
    { 0x61d24590, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d24598, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea12459a, SUC_lexicon_tags_31 },
    { 0x532459b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5245a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed245a7, SUC_lexicon_tags_60 },
    { 0x7a3245a8, SUC_lexicon_tags_56 },
    { 0x4f245a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xac3245ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93245b2, SUC_lexicon_tags_65 },
    { 0x315245b3, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f245c0, SUC_lexicon_tags_40 },
    { 0x32d245c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e9245c3, SUC_lexicon_tags_11 },
    { 0x10f245c4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xa11245c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f1245c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6f245ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e1245cc, SUC_lexicon_tags_37 },
    { 0xd31245cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95245d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43245d7, SUC_lexicon_tags_37 },
    { 0xef3245d8, SUC_lexicon_tags_84 },
    { 0xc27245d9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283245dd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415245e2, SUC_lexicon_tags_11 },
    { 0x975245e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73245ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1f1245ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x141245f1, SUC_lexicon_tags_23 },
    { 0xf2b245f2, SUC_lexicon_tags_478 },
    { 0x2e5245f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x621245f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb245fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x621245fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x74f24600, SUC_lexicon_tags_7 },
    { 0x23b24601, SUC_lexicon_tags_12 },
    { 0x8d124602, SUC_lexicon_tags_25 },
    { 0x87d24603, SUC_lexicon_tags_2 },
    { 0x95524604, SUC_lexicon_tags_38 },
    { 0x53f24600, SUC_lexicon_tags_1 },
    { 0x5ef24602, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6732460c, SUC_lexicon_tags_7 },
    { 0x3792460d, SUC_lexicon_tags_47 },
    { 0xeab2460e, SUC_lexicon_tags_2 },
    { 0xd272460f, SUC_lexicon_tags_116 },
    { 0xd32460d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71924614, SUC_lexicon_tags_37 },
    { 0xd8724615, SUC_lexicon_tags_10 },
    { 0xe3d24615, SUC_lexicon_tags_11 },
    { 0xd9b24617, SUC_lexicon_tags_12 },
    { 0x63124618, SUC_lexicon_tags_43 },
    { 0x72b24617, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5432461c, SUC_lexicon_tags_62 },
    { 0x85d2461d, SUC_lexicon_tags_16 },
    { 0x9792461e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef24622, SUC_lexicon_tags_87 },
    { 0x32324623, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6f524625, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7a924627, SUC_lexicon_tags_12 },
    { 0x84b24627, SUC_lexicon_tags_37 },
    { 0x50924629, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b2462c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1952462e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xecb24630, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb524633, SUC_lexicon_tags_37 },
    { 0x11f24634, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad24637, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c124639, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b2463e, SUC_lexicon_tags_6 },
    { 0x2192463f, SUC_lexicon_tags_11 },
    { 0xfd924640, SUC_lexicon_tags_17 },
    { 0xd924641, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1524643, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91324648, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd2464b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf92464d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c124650, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff24652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27324654, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x70324657, SUC_lexicon_tags_15 },
    { 0x21524657, SUC_lexicon_tags_15 },
    { 0x7b24659, SUC_lexicon_tags_21 },
    { 0x2bf2465a, SUC_lexicon_tags_47 },
    { 0xd7d2465b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa92465d, SUC_lexicon_tags_38 },
    { 0x9992465e, SUC_lexicon_tags_15 },
    { 0xccb2465d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d324661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f24665, SUC_lexicon_tags_7 },
    { 0x87524666, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8d724668, SUC_lexicon_tags_108 },
    { 0x1e924669, SUC_lexicon_tags_12 },
    { 0x1dd2466a, SUC_lexicon_tags_7 },
    { 0x63724669, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28f2466d, SUC_lexicon_tags_1 },
    { 0x8e52466e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa1924670, SUC_lexicon_tags_28 },
    { 0x57124671, SUC_lexicon_tags_19 },
    { 0x31924672, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7b24674, SUC_lexicon_tags_15 },
    { 0xa5324675, SUC_lexicon_tags_46 },
    { 0xfbd24674, SUC_lexicon_tags_60 },
    { 0x3524675, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf532467c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe832467e, SUC_lexicon_tags_14 },
    { 0x3ff2467f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce324681, SUC_lexicon_tags_15 },
    { 0x32724681, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0524684, SUC_lexicon_tags_37 },
    { 0x34524685, SUC_lexicon_tags_7 },
    { 0x9bb24686, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b12468c, SUC_lexicon_tags_7 },
    { 0xcb2468d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb24690, SUC_lexicon_tags_17 },
    { 0xf0b24690, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61724693, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf4124695, SUC_lexicon_tags_26 },
    { 0xf8924696, SUC_lexicon_tags_63 },
    { 0xb7924696, SUC_lexicon_tags_72 },
    { 0x57d24698, SUC_lexicon_tags_11 },
    { 0xff24698, SUC_lexicon_tags_12 },
    { 0xbe52469a, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2312469f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951246a3, SUC_lexicon_tags_11 },
    { 0xb0b246a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db246aa, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x305246ad, SUC_lexicon_tags_31 },
    { 0xfd3246ae, SUC_lexicon_tags_12 },
    { 0xb59246ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x359246b1, SUC_lexicon_tags_15 },
    { 0x28b246b1, SUC_lexicon_tags_37 },
    { 0x327246b3, SUC_lexicon_tags_13 },
    { 0xc15246b4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b246ba, SUC_lexicon_tags_1 },
    { 0x8bd246bb, SUC_lexicon_tags_12 },
    { 0x3ab246bc, SUC_lexicon_tags_11 },
    { 0xb7b246bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb246c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x183246c3, SUC_lexicon_tags_17 },
    { 0xbdf246c3, SUC_lexicon_tags_61 },
    { 0x13d246c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761246c9, SUC_lexicon_tags_37 },
    { 0x1b246c9, SUC_lexicon_tags_6 },
    { 0xbf246c9, SUC_lexicon_tags_30 },
    { 0x75246ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681246d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df246d4, SUC_lexicon_tags_23 },
    { 0x28f246d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab246db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef246de, SUC_lexicon_tags_14 },
    { 0xceb246df, SUC_lexicon_tags_6 },
    { 0xaf5246de, SUC_lexicon_tags_14 },
    { 0x11f246e1, SUC_lexicon_tags_37 },
    { 0x583246e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x663246e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b246e8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463246ec, SUC_lexicon_tags_37 },
    { 0xa51246ed, SUC_lexicon_tags_21 },
    { 0xb9f246ee, SUC_lexicon_tags_37 },
    { 0x4a3246ef, SUC_lexicon_tags_60 },
    { 0x39d246ee, SUC_lexicon_tags_37 },
    { 0xee1246f1, SUC_lexicon_tags_15 },
    { 0x85d246f2, SUC_lexicon_tags_17 },
    { 0x6d7246ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197246f8, SUC_lexicon_tags_47 },
    { 0x649246f9, SUC_lexicon_tags_65 },
    { 0xbe1246f9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f246fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa324700, SUC_lexicon_tags_37 },
    { 0xc9324700, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48724703, SUC_lexicon_tags_4 },
    { 0x29324704, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f24707, SUC_lexicon_tags_31 },
    { 0xe9524707, SUC_lexicon_tags_29 },
    { 0x4d124708, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f72470d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd24711, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b24714, SUC_lexicon_tags_8 },
    { 0x5ff24715, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22924719, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbeb2471b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3532471d, SUC_lexicon_tags_17 },
    { 0x32d2471d, SUC_lexicon_tags_11 },
    { 0xae52471f, SUC_lexicon_tags_15 },
    { 0x4b32471f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36324723, SUC_lexicon_tags_25 },
    { 0xa924724, SUC_lexicon_tags_27 },
    { 0xe7524724, SUC_lexicon_tags_79 },
    { 0xfb724725, SUC_lexicon_tags_29 },
    { 0xe6b24724, SUC_lexicon_tags_49 },
    { 0xbf524728, SUC_lexicon_tags_12 },
    { 0x99f24729, SUC_lexicon_tags_23 },
    { 0xd9f24725, SUC_lexicon_tags_16 },
    { 0x3952472b, SUC_lexicon_tags_17 },
    { 0xfcb24726, SUC_lexicon_tags_31 },
    { 0x66d24725, SUC_lexicon_tags_15 },
    { 0xb912472e, SUC_lexicon_tags_49 },
    { 0x14f2472e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0524733, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d524735, SUC_lexicon_tags_25 },
    { 0xd9724736, SUC_lexicon_tags_37 },
    { 0xbc524737, SUC_lexicon_tags_47 },
    { 0x16124735, SUC_lexicon_tags_11 },
    { 0x1eb24739, SUC_lexicon_tags_0 },
    { 0x1152473a, SUC_lexicon_tags_14 },
    { 0xfcd2473b, SUC_lexicon_tags_4 },
    { 0xc4f2473c, SUC_lexicon_tags_37 },
    { 0x99f2473d, SUC_lexicon_tags_29 },
    { 0xe412473c, SUC_lexicon_tags_13 },
    { 0x4ad2473f, SUC_lexicon_tags_39 },
    { 0xddb24736, SUC_lexicon_tags_55 },
    { 0xad724741, SUC_lexicon_tags_15 },
    { 0x3212473c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a524747, SUC_lexicon_tags_12 },
    { 0xc9124747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6632474f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd24753, SUC_lexicon_tags_108 },
    { 0x3d124754, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x23724756, SUC_lexicon_tags_23 },
    { 0xe4b24757, SUC_lexicon_tags_26 },
    { 0xd3924758, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9a92475a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d2475e, SUC_lexicon_tags_39 },
    { 0xd8d2475f, SUC_lexicon_tags_77 },
    { 0x4572475e, SUC_lexicon_tags_25 },
    { 0x6772475e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7b24763, SUC_lexicon_tags_37 },
    { 0x3d324763, SUC_lexicon_tags_38 },
    { 0x2d724763, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe8524767, SUC_lexicon_tags_62 },
    { 0xbf724768, SUC_lexicon_tags_33 },
    { 0x55b24769, SUC_lexicon_tags_9 },
    { 0xe0924769, SUC_lexicon_tags_7 },
    { 0xd8524769, SUC_lexicon_tags_8 },
    { 0x9ab2476c, SUC_lexicon_tags_1 },
    { 0x89d2476d, SUC_lexicon_tags_37 },
    { 0x1432476e, SUC_lexicon_tags_80 },
    { 0x80d2476e, SUC_lexicon_tags_43 },
    { 0xccf24770, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0524772, SUC_lexicon_tags_15 },
    { 0xd0d24772, SUC_lexicon_tags_830 },
    { 0xd9d24773, SUC_lexicon_tags_8 },
    { 0x6924774, SUC_lexicon_tags_46 },
    { 0x6c124774, SUC_lexicon_tags_47 },
    { 0xed24777, SUC_lexicon_tags_25 },
    { 0xc8924778, SUC_lexicon_tags_33 },
    { 0x39324779, SUC_lexicon_tags_17 },
    { 0x27d2477a, SUC_lexicon_tags_7 },
    { 0x74b24777, SUC_lexicon_tags_11 },
    { 0x6b52477b, SUC_lexicon_tags_23 },
    { 0x16b2477d, SUC_lexicon_tags_8 },
    { 0x6492477b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf9124780, SUC_lexicon_tags_8 },
    { 0x35724781, SUC_lexicon_tags_37 },
    { 0x95724782, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0324787, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xded24789, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e32478b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8392478d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1124790, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd24795, SUC_lexicon_tags_15 },
    { 0xc9124796, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c324798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c32479b, SUC_lexicon_tags_22 },
    { 0xb12479b, SUC_lexicon_tags_55 },
    { 0xa952479d, SUC_lexicon_tags_11 },
    { 0xd512479d, SUC_lexicon_tags_8 },
    { 0xe092479f, SUC_lexicon_tags_17 },
    { 0x7d2479f, SUC_lexicon_tags_25 },
    { 0xc1b2479f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x377247a4, SUC_lexicon_tags_56 },
    { 0xa23247a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11247a8, SUC_lexicon_tags_38 },
    { 0x9f9247a9, SUC_lexicon_tags_26 },
    { 0x261247aa, SUC_lexicon_tags_38 },
    { 0x627247a9, SUC_lexicon_tags_7 },
    { 0xcc5247a9, SUC_lexicon_tags_17 },
    { 0x467247ad, SUC_lexicon_tags_43 },
    { 0x713247ae, SUC_lexicon_tags_18 },
    { 0x2bb247af, SUC_lexicon_tags_12 },
    { 0xd97247af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d247b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c3247b6, SUC_lexicon_tags_2 },
    { 0x877247b6, SUC_lexicon_tags_43 },
    { 0x5e7247b7, SUC_lexicon_tags_7 },
    { 0x4b3247b6, SUC_lexicon_tags_7 },
    { 0xccb247b9, SUC_lexicon_tags_43 },
    { 0xb3247bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad247bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789247c4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87247c7, SUC_lexicon_tags_44 },
    { 0x389247c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x93d247ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ad247cc, SUC_lexicon_tags_49 },
    { 0x629247cd, SUC_lexicon_tags_37 },
    { 0xc3247ce, SUC_lexicon_tags_6 },
    { 0xd39247cf, SUC_lexicon_tags_60 },
    { 0x7cb247cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b247d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43247db, SUC_lexicon_tags_55 },
    { 0x91f247dc, SUC_lexicon_tags_5 },
    { 0x2d7247dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x383247e0, SUC_lexicon_tags_27 },
    { 0x7db247e1, SUC_lexicon_tags_17 },
    { 0x1f247e2, SUC_lexicon_tags_37 },
    { 0x3f1247e3, SUC_lexicon_tags_25 },
    { 0x5a1247e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07247eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd247ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5247f1, SUC_lexicon_tags_47 },
    { 0x789247f2, SUC_lexicon_tags_1 },
    { 0xae5247f3, SUC_lexicon_tags_7 },
    { 0xd05247f4, SUC_lexicon_tags_17 },
    { 0x61b247f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d247f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f247fc, SUC_lexicon_tags_7 },
    { 0x2e7247fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd21247ff, SUC_lexicon_tags_37 },
    { 0xd6324800, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef24803, SUC_lexicon_tags_14 },
    { 0x3c924804, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x53724806, SUC_lexicon_tags_27 },
    { 0x1f524806, SUC_lexicon_tags_15 },
    { 0x4b124808, SUC_lexicon_tags_27 },
    { 0xe6724809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42324811, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d24814, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b2481b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefd2481d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd24820, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a924825, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8bf24827, SUC_lexicon_tags_439 },
    { 0x49d24828, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa132482a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0124830, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f24835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75d24837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa692483d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b24840, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe4924842, SUC_lexicon_tags_31 },
    { 0xcd924842, SUC_lexicon_tags_15 },
    { 0x7f924844, SUC_lexicon_tags_15 },
    { 0x44524845, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb2484a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd192484e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37524850, SUC_lexicon_tags_12 },
    { 0xa5d24850, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36d24853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b324857, SUC_lexicon_tags_57 },
    { 0x18324857, SUC_lexicon_tags_19 },
    { 0x41324857, SUC_lexicon_tags_28 },
    { 0xbcf2485a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb724862, SUC_lexicon_tags_13 },
    { 0x24324862, SUC_lexicon_tags_15 },
    { 0xd9724862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96524869, SUC_lexicon_tags_2 },
    { 0xb12486a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd352486d, SUC_lexicon_tags_502 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d24871, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34724875, SUC_lexicon_tags_2 },
    { 0x9d24876, SUC_lexicon_tags_7 },
    { 0xa1724875, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0724879, SUC_lexicon_tags_45 },
    { 0x3812487a, SUC_lexicon_tags_14 },
    { 0x2692487a, SUC_lexicon_tags_15 },
    { 0x2bf2487b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf324881, SUC_lexicon_tags_17 },
    { 0x5bf24882, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3324886, SUC_lexicon_tags_9 },
    { 0x15b24886, SUC_lexicon_tags_11 },
    { 0xf4124888, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ed2488a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee32488c, SUC_lexicon_tags_63 },
    { 0xfb12488c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7d92488f, SUC_lexicon_tags_6 },
    { 0xb2524890, SUC_lexicon_tags_17 },
    { 0xb6d24891, SUC_lexicon_tags_38 },
    { 0xc8b24890, SUC_lexicon_tags_7 },
    { 0x58524892, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f2489c, SUC_lexicon_tags_1 },
    { 0x41d2489c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41248a0, SUC_lexicon_tags_23 },
    { 0xc83248a1, SUC_lexicon_tags_47 },
    { 0x10b248a1, SUC_lexicon_tags_2 },
    { 0xc81248a1, SUC_lexicon_tags_19 },
    { 0xc73248a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3248a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91d248aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b248af, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5248b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721248b7, SUC_lexicon_tags_63 },
    { 0xf7f248b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc5248ba, SUC_lexicon_tags_11 },
    { 0x615248bb, SUC_lexicon_tags_14 },
    { 0xa9248bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa45248be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7248ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a1248cc, SUC_lexicon_tags_15 },
    { 0x3df248cd, SUC_lexicon_tags_25 },
    { 0x31248ce, SUC_lexicon_tags_47 },
    { 0x9c5248cf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x471248d1, SUC_lexicon_tags_8 },
    { 0xb75248d2, SUC_lexicon_tags_31 },
    { 0x6af248d1, SUC_lexicon_tags_45 },
    { 0xb61248d4, SUC_lexicon_tags_9 },
    { 0xd1b248d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb248d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59248de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x779248e1, SUC_lexicon_tags_17 },
    { 0xbf9248e1, SUC_lexicon_tags_37 },
    { 0x7f1248e3, SUC_lexicon_tags_7 },
    { 0x2e7248e3, SUC_lexicon_tags_54 },
    { 0xaf1248e5, SUC_lexicon_tags_45 },
    { 0x5b5248e6, SUC_lexicon_tags_39 },
    { 0x929248e6, SUC_lexicon_tags_8 },
    { 0xf21248e8, SUC_lexicon_tags_15 },
    { 0x72d248e9, SUC_lexicon_tags_32 },
    { 0x247248e8, SUC_lexicon_tags_1509 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55248ee, SUC_lexicon_tags_0 },
    { 0xb85248ee, SUC_lexicon_tags_27 },
    { 0x553248f0, SUC_lexicon_tags_85 },
    { 0xe57248f1, SUC_lexicon_tags_27 },
    { 0x8d3248f1, SUC_lexicon_tags_15 },
    { 0x7bf248f3, SUC_lexicon_tags_46 },
    { 0xc9d248f0, SUC_lexicon_tags_17 },
    { 0x657248ee, SUC_lexicon_tags_7 },
    { 0xab5248f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3248fc, SUC_lexicon_tags_57 },
    { 0xc85248fd, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0x737248ff, SUC_lexicon_tags_26 },
    { 0x44b248ff, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5324903, SUC_lexicon_tags_0 },
    { 0x98f24904, SUC_lexicon_tags_1 },
    { 0xaa324905, SUC_lexicon_tags_29 },
    { 0x9ab24906, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f24912, SUC_lexicon_tags_27 },
    { 0x73b24913, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x46524915, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6d24917, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd912491c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99d2491e, SUC_lexicon_tags_18 },
    { 0x3912491e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe7724921, SUC_lexicon_tags_2 },
    { 0x53124922, SUC_lexicon_tags_57 },
    { 0xcd324922, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38124925, SUC_lexicon_tags_2 },
    { 0xf1b24926, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe124928, SUC_lexicon_tags_7 },
    { 0x2db24929, SUC_lexicon_tags_1 },
    { 0x3272492a, SUC_lexicon_tags_14 },
    { 0x6f12492a, SUC_lexicon_tags_37 },
    { 0xba12492a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfad2492e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd7924930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3924932, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52524934, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee12493a, SUC_lexicon_tags_8 },
    { 0xa352493b, SUC_lexicon_tags_11 },
    { 0xe992493b, SUC_lexicon_tags_48 },
    { 0x97d2493d, SUC_lexicon_tags_11 },
    { 0x6cd2493e, SUC_lexicon_tags_12 },
    { 0x7672493c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0324942, SUC_lexicon_tags_22 },
    { 0x6d124943, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad324947, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb832494f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb24954, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4924958, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc72495c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d24960, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb24964, SUC_lexicon_tags_57 },
    { 0x5eb24965, SUC_lexicon_tags_1 },
    { 0x46d24965, SUC_lexicon_tags_46 },
    { 0x29124965, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc072496a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf24976, SUC_lexicon_tags_66 },
    { 0x83b24976, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4f24979, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63724989, SUC_lexicon_tags_25 },
    { 0x5152498a, SUC_lexicon_tags_17 },
    { 0xc072498a, SUC_lexicon_tags_47 },
    { 0x1af2498c, SUC_lexicon_tags_84 },
    { 0x6152498b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9124990, SUC_lexicon_tags_15 },
    { 0xdbf24991, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35724994, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d24999, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ff2499b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd2499e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81249a8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871249af, SUC_lexicon_tags_14 },
    { 0x48d249b0, SUC_lexicon_tags_17 },
    { 0x375249b1, SUC_lexicon_tags_65 },
    { 0x9eb249b2, SUC_lexicon_tags_45 },
    { 0xff3249b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x607249b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x95d249b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x253249ba, SUC_lexicon_tags_47 },
    { 0x8dd249bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7b249bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x429249bf, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff249c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5249c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7249c9, SUC_lexicon_tags_33 },
    { 0xeb1249ca, SUC_lexicon_tags_29 },
    { 0xab249cb, SUC_lexicon_tags_37 },
    { 0x897249cb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303249d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1249d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x449249d6, SUC_lexicon_tags_9 },
    { 0x8d9249d6, SUC_lexicon_tags_31 },
    { 0x4b1249d7, SUC_lexicon_tags_29 },
    { 0x38d249d9, SUC_lexicon_tags_2 },
    { 0x40f249d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83249e2, SUC_lexicon_tags_92 },
    { 0x1af249e3, SUC_lexicon_tags_15 },
    { 0xe01249e4, SUC_lexicon_tags_45 },
    { 0x599249e5, SUC_lexicon_tags_32 },
    { 0xcd1249e5, SUC_lexicon_tags_14 },
    { 0x6cb249e7, SUC_lexicon_tags_11 },
    { 0xfd249e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x243249eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561249f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43249f8, SUC_lexicon_tags_15 },
    { 0xf93249f9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb3b249fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea924a07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a724a0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3924a0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed24a12, SUC_lexicon_tags_33 },
    { 0x60f24a12, SUC_lexicon_tags_37 },
    { 0x56724a13, SUC_lexicon_tags_37 },
    { 0xe6f24a15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde924a18, SUC_lexicon_tags_7 },
    { 0x63524a18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1924a1f, SUC_lexicon_tags_12 },
    { 0x77124a20, SUC_lexicon_tags_37 },
    { 0x71524a21, SUC_lexicon_tags_35 },
    { 0x44524a22, SUC_lexicon_tags_47 },
    { 0x38324a22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x124a25, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb24a2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12724a32, SUC_lexicon_tags_15 },
    { 0xdb724a33, SUC_lexicon_tags_48 },
    { 0x1924a33, SUC_lexicon_tags_37 },
    { 0x4124a35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8324a39, SUC_lexicon_tags_37 },
    { 0x6bf24a3a, SUC_lexicon_tags_23 },
    { 0xb2f24a3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37524a40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d24a43, SUC_lexicon_tags_17 },
    { 0xe4524a43, SUC_lexicon_tags_43 },
    { 0xde724a45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1524a52, SUC_lexicon_tags_25 },
    { 0x2d324a53, SUC_lexicon_tags_1 },
    { 0x86724a54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0124a5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98324a65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b24a6c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8124a74, SUC_lexicon_tags_25 },
    { 0x5ed24a75, SUC_lexicon_tags_29 },
    { 0x53324a76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39324a7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6324a80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68124a83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39d24a85, SUC_lexicon_tags_1 },
    { 0xad124a86, SUC_lexicon_tags_7 },
    { 0xb3324a86, SUC_lexicon_tags_1 },
    { 0x52524a88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83924a8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3d24a8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xddd24a90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x26924a93, SUC_lexicon_tags_2 },
    { 0xe7f24a94, SUC_lexicon_tags_9 },
    { 0x30f24a95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49f24a97, SUC_lexicon_tags_12 },
    { 0xbb24a98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd24a9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1924a9c, SUC_lexicon_tags_0 },
    { 0xce524a9c, SUC_lexicon_tags_226 },
    { 0x56b24a9d, SUC_lexicon_tags_118 },
    { 0x91b24a9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0724aa5, SUC_lexicon_tags_34 },
    { 0x63b24aa6, SUC_lexicon_tags_65 },
    { 0x52124aa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b324aa9, SUC_lexicon_tags_7 },
    { 0x98d24aaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5bb24aac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9a324aae, SUC_lexicon_tags_12 },
    { 0x47324aae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17f24ab1, SUC_lexicon_tags_43 },
    { 0x17f24ab2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a124ab5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad24ab8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65924ac2, SUC_lexicon_tags_11 },
    { 0x54f24ac2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f24ac6, SUC_lexicon_tags_45 },
    { 0xcf324ac6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f124aca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5324acd, SUC_lexicon_tags_48 },
    { 0xcd724acd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad524ad4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf24ad7, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0xef524ada, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b524adc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7724ade, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67124ae2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x50324ae4, SUC_lexicon_tags_37 },
    { 0xb724ae5, SUC_lexicon_tags_42 },
    { 0x35f24ae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80524aeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72124aed, SUC_lexicon_tags_15 },
    { 0x10b24aee, SUC_lexicon_tags_38 },
    { 0x75b24aee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63324af2, SUC_lexicon_tags_4 },
    { 0x56124af3, SUC_lexicon_tags_63 },
    { 0xc4924af4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ab24af6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1ad24af8, SUC_lexicon_tags_45 },
    { 0xf9924af9, SUC_lexicon_tags_29 },
    { 0x4bb24af9, SUC_lexicon_tags_60 },
    { 0x82324afb, SUC_lexicon_tags_31 },
    { 0x77924afa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54924b04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf24b10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd524b12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd24b15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x41324b18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7524b1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65924b1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f24b22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4324b24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x71324b27, SUC_lexicon_tags_37 },
    { 0xe0f24b28, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63724b2e, SUC_lexicon_tags_79 },
    { 0xe7b24b2f, SUC_lexicon_tags_38 },
    { 0x23d24b2f, SUC_lexicon_tags_14 },
    { 0x42f24b31, SUC_lexicon_tags_25 },
    { 0xd4324b32, SUC_lexicon_tags_15 },
    { 0x4c324b33, SUC_lexicon_tags_12 },
    { 0x2a324b34, SUC_lexicon_tags_30 },
    { 0x9e924b34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f24b3a, SUC_lexicon_tags_60 },
    { 0x29924b3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xced24b3d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d124b40, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2524b44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f724b48, SUC_lexicon_tags_19 },
    { 0x3cd24b49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0524b4c, SUC_lexicon_tags_7 },
    { 0xb5724b4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d924b50, SUC_lexicon_tags_60 },
    { 0xb6524b51, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f24b54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4124b56, SUC_lexicon_tags_1 },
    { 0x2fb24b57, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3124b5b, SUC_lexicon_tags_37 },
    { 0xdd324b5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d24b61, SUC_lexicon_tags_37 },
    { 0xba324b62, SUC_lexicon_tags_0 },
    { 0xf5324b62, SUC_lexicon_tags_2 },
    { 0x92d24b61, SUC_lexicon_tags_25 },
    { 0xb7f24b63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b24b69, SUC_lexicon_tags_25 },
    { 0x4d724b6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6c924b6c, SUC_lexicon_tags_43 },
    { 0x54124b6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1524b74, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd24b7b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a924b7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f24b83, SUC_lexicon_tags_7 },
    { 0xba724b84, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f524b89, SUC_lexicon_tags_23 },
    { 0x2af24b8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82124b8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x27b24b8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59724b99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38924b9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b924ba2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5ab24ba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b24ba7, SUC_lexicon_tags_12 },
    { 0xf4724ba8, SUC_lexicon_tags_2 },
    { 0x68524ba9, SUC_lexicon_tags_111 },
    { 0xa9524ba9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d24bad, SUC_lexicon_tags_37 },
    { 0xc6524bae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52924bb1, SUC_lexicon_tags_45 },
    { 0x81924bb2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee124bb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2324bb9, SUC_lexicon_tags_2 },
    { 0xe0924bba, SUC_lexicon_tags_0 },
    { 0x92d24bbb, SUC_lexicon_tags_38 },
    { 0xe7524bba, SUC_lexicon_tags_48 },
    { 0x97124bbb, SUC_lexicon_tags_19 },
    { 0x4e524bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88f24bc0, SUC_lexicon_tags_11 },
    { 0x7b324bc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd924bc4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee324bd4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d324bd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62324bda, SUC_lexicon_tags_0 },
    { 0x4e324bda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3724bde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65524be3, SUC_lexicon_tags_12 },
    { 0x2f524be4, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1724be8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f324bea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3124bec, SUC_lexicon_tags_11 },
    { 0xab924bed, SUC_lexicon_tags_47 },
    { 0x11924bee, SUC_lexicon_tags_23 },
    { 0xd8524bed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb924bf4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3724bfa, SUC_lexicon_tags_2 },
    { 0x27324bfb, SUC_lexicon_tags_2 },
    { 0x3df24bfc, SUC_lexicon_tags_15 },
    { 0xcef24bfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98724c08, SUC_lexicon_tags_201 },
    { 0xd0924c09, SUC_lexicon_tags_45 },
    { 0xd0124c09, SUC_lexicon_tags_17 },
    { 0xbc524c0b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x56724c0d, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8524c16, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f24c1a, SUC_lexicon_tags_33 },
    { 0x29324c1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b24c1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d324c25, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f24c29, SUC_lexicon_tags_14 },
    { 0x80724c2a, SUC_lexicon_tags_25 },
    { 0xcef24c2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb24c2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b24c30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d24c35, SUC_lexicon_tags_1045 },
    { 0xef24c36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcb124c38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f24c3b, SUC_lexicon_tags_27 },
    { 0x7f524c3b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b24c41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x124c47, SUC_lexicon_tags_66 },
    { 0x63f24c48, SUC_lexicon_tags_13 },
    { 0x2d24c47, SUC_lexicon_tags_63 },
    { 0xcd24c4a, SUC_lexicon_tags_25 },
    { 0xdff24c4b, SUC_lexicon_tags_29 },
    { 0xb4324c4b, SUC_lexicon_tags_17 },
    { 0x22124c4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x43524c50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e724c54, SUC_lexicon_tags_60 },
    { 0xe3324c55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc7524c57, SUC_lexicon_tags_43 },
    { 0x58324c58, SUC_lexicon_tags_1 },
    { 0x29524c59, SUC_lexicon_tags_9 },
    { 0x31724c59, SUC_lexicon_tags_6 },
    { 0xe7d24c59, SUC_lexicon_tags_38 },
    { 0xdf24c57, SUC_lexicon_tags_224 },
    { 0xe8b24c5b, SUC_lexicon_tags_15 },
    { 0x45924c5b, SUC_lexicon_tags_23 },
    { 0x60f24c5f, SUC_lexicon_tags_38 },
    { 0xfd524c60, SUC_lexicon_tags_66 },
    { 0x32124c5a, SUC_lexicon_tags_1 },
    { 0x42724c62, SUC_lexicon_tags_7 },
    { 0x3bd24c62, SUC_lexicon_tags_1 },
    { 0x92524c64, SUC_lexicon_tags_15 },
    { 0xab924c64, SUC_lexicon_tags_31 },
    { 0x5d924c64, SUC_lexicon_tags_37 },
    { 0xc6924c62, SUC_lexicon_tags_0 },
    { 0xe3b24c67, SUC_lexicon_tags_17 },
    { 0xc324c5d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x48524c6b, SUC_lexicon_tags_15 },
    { 0x67f24c6c, SUC_lexicon_tags_47 },
    { 0x1a324c6d, SUC_lexicon_tags_25 },
    { 0x9f124c6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef124c70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab924c7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f24c81, SUC_lexicon_tags_7 },
    { 0xa3724c82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d124c8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7724c8e, SUC_lexicon_tags_17 },
    { 0xcf324c8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b24c92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75924c95, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d24c99, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc524c9b, SUC_lexicon_tags_15 },
    { 0xc7d24c9c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b24c9f, SUC_lexicon_tags_12 },
    { 0xe7b24c9f, SUC_lexicon_tags_1 },
    { 0x39524ca1, SUC_lexicon_tags_37 },
    { 0x57124ca2, SUC_lexicon_tags_26 },
    { 0x18324ca3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0724ca6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6124ca9, SUC_lexicon_tags_37 },
    { 0x47524caa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf24cae, SUC_lexicon_tags_2 },
    { 0xf3524caf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x50b24cb1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9ed24cb3, SUC_lexicon_tags_37 },
    { 0xc4324cb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b24cb8, SUC_lexicon_tags_12 },
    { 0xf524cb9, SUC_lexicon_tags_11 },
    { 0x27d24cb9, SUC_lexicon_tags_11 },
    { 0xb6d24cbb, SUC_lexicon_tags_37 },
    { 0x1af24cba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf524cbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a924cc1, SUC_lexicon_tags_1 },
    { 0x8eb24cc1, SUC_lexicon_tags_13 },
    { 0x33524cc3, SUC_lexicon_tags_19 },
    { 0xcef24cc4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b524ccb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16724ccd, SUC_lexicon_tags_120 },
    { 0x63524cce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed124cd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3124cd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab24cda, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4524cde, SUC_lexicon_tags_1 },
    { 0x4d924cdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0d24ce1, SUC_lexicon_tags_8 },
    { 0x87724ce2, SUC_lexicon_tags_6 },
    { 0x57d24ce3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34524ce5, SUC_lexicon_tags_11 },
    { 0xe0924ce5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1fb24ce8, SUC_lexicon_tags_2 },
    { 0x3924ce9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf324ced, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff324cf1, SUC_lexicon_tags_39 },
    { 0x6324cf1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x32d24cf4, SUC_lexicon_tags_27 },
    { 0x8d524cf4, SUC_lexicon_tags_7 },
    { 0xe0b24cf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d24cf9, SUC_lexicon_tags_11 },
    { 0xdb124cf9, SUC_lexicon_tags_1 },
    { 0x67124cf9, SUC_lexicon_tags_7 },
    { 0x9a324cfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff24d00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x14f24d02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1124d05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad24d0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb24d10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a324d17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94324d19, SUC_lexicon_tags_15 },
    { 0x7e524d1a, SUC_lexicon_tags_38 },
    { 0x31324d19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf24d1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda524d22, SUC_lexicon_tags_80 },
    { 0xdaf24d23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f24d26, SUC_lexicon_tags_62 },
    { 0xbb124d27, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x94724d2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d24d30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac524d33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b524d36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18f24d38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31d24d3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10924d3c, SUC_lexicon_tags_37 },
    { 0x2b24d3c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd24d49, SUC_lexicon_tags_9 },
    { 0x9d924d4a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab24d4d, SUC_lexicon_tags_15 },
    { 0x72b24d4d, SUC_lexicon_tags_2 },
    { 0x69124d4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64924d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d724d5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14724d5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2724d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1124d66, SUC_lexicon_tags_17 },
    { 0x32724d66, SUC_lexicon_tags_6 },
    { 0x5b324d68, SUC_lexicon_tags_17 },
    { 0x77124d68, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4724d6c, SUC_lexicon_tags_29 },
    { 0x8324d6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f24d71, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5724d74, SUC_lexicon_tags_1 },
    { 0x7d324d74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce724d7a, SUC_lexicon_tags_37 },
    { 0x5a524d7a, SUC_lexicon_tags_33 },
    { 0x50324d7c, SUC_lexicon_tags_17 },
    { 0x5ef24d7d, SUC_lexicon_tags_30 },
    { 0xe4924d7d, SUC_lexicon_tags_25 },
    { 0xfc324d7b, SUC_lexicon_tags_43 },
    { 0xe2124d7f, SUC_lexicon_tags_11 },
    { 0xc8d24d81, SUC_lexicon_tags_17 },
    { 0x37724d82, SUC_lexicon_tags_26 },
    { 0xd0924d83, SUC_lexicon_tags_66 },
    { 0xe6f24d84, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x50724d86, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20724d8a, SUC_lexicon_tags_45 },
    { 0x5cb24d8b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xc5d24d8d, SUC_lexicon_tags_11 },
    { 0x7fd24d8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb24d95, SUC_lexicon_tags_11 },
    { 0x48f24d96, SUC_lexicon_tags_16 },
    { 0x2c924d97, SUC_lexicon_tags_60 },
    { 0xd8b24d97, SUC_lexicon_tags_25 },
    { 0x1cd24d98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2d24d9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7924da0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1324da4, SUC_lexicon_tags_33 },
    { 0xee124da5, SUC_lexicon_tags_60 },
    { 0x4f324da6, SUC_lexicon_tags_12 },
    { 0x96324da7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2d724da9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda524dac, SUC_lexicon_tags_37 },
    { 0xa9924dad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0124db1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f324db6, SUC_lexicon_tags_25 },
    { 0x1f324db7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7c524db9, SUC_lexicon_tags_17 },
    { 0x32f24dba, SUC_lexicon_tags_12 },
    { 0x26124dbb, SUC_lexicon_tags_7 },
    { 0xbb524dbc, SUC_lexicon_tags_25 },
    { 0xc924dba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3324dc1, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b524dc4, SUC_lexicon_tags_21 },
    { 0xe3524dc5, SUC_lexicon_tags_25 },
    { 0x1f924dc6, SUC_lexicon_tags_6 },
    { 0x9a724dc6, SUC_lexicon_tags_12 },
    { 0xcd324dc6, SUC_lexicon_tags_14 },
    { 0xc1724dc5, SUC_lexicon_tags_9 },
    { 0x9ad24dca, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2a324dcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25d24dce, SUC_lexicon_tags_35 },
    { 0x8bf24dce, SUC_lexicon_tags_27 },
    { 0x17724dd0, SUC_lexicon_tags_2 },
    { 0xcb24dd1, SUC_lexicon_tags_7 },
    { 0x3af24dce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b24dd7, SUC_lexicon_tags_37 },
    { 0x8b724dd8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14124dde, SUC_lexicon_tags_19 },
    { 0x81f24ddf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb324de1, SUC_lexicon_tags_0 },
    { 0x6924de2, SUC_lexicon_tags_12 },
    { 0x8e924de2, SUC_lexicon_tags_63 },
    { 0x4ef24de4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9124deb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e124dee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20524df3, SUC_lexicon_tags_47 },
    { 0xd8924df3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9724df6, SUC_lexicon_tags_43 },
    { 0x65d24df7, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db24dfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a924dfd, SUC_lexicon_tags_1 },
    { 0x9b124dfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4924e00, SUC_lexicon_tags_1 },
    { 0x22124e01, SUC_lexicon_tags_17 },
    { 0x8ed24e02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d24e05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b24e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab24e0c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b524e10, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81724e14, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d24e1c, SUC_lexicon_tags_47 },
    { 0x23f24e1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f124e1f, SUC_lexicon_tags_11 },
    { 0x1c324e20, SUC_lexicon_tags_15 },
    { 0xa9524e21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ed24e23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32724e28, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d24e2b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3524e2e, SUC_lexicon_tags_15 },
    { 0xdb524e2e, SUC_lexicon_tags_33 },
    { 0xe6724e30, SUC_lexicon_tags_55 },
    { 0x69d24e30, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db24e37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb24e3b, SUC_lexicon_tags_17 },
    { 0x8124e3c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99924e41, SUC_lexicon_tags_37 },
    { 0x86524e42, SUC_lexicon_tags_35 },
    { 0xbd124e43, SUC_lexicon_tags_30 },
    { 0xc2724e41, SUC_lexicon_tags_7 },
    { 0x8f724e45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36d24e47, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xca324e4a, SUC_lexicon_tags_142 },
    { 0x4ef24e4a, SUC_lexicon_tags_14 },
    { 0xca924e4c, SUC_lexicon_tags_37 },
    { 0x1f124e4d, SUC_lexicon_tags_17 },
    { 0xc8724e4e, SUC_lexicon_tags_26 },
    { 0xcb524e4f, SUC_lexicon_tags_7 },
    { 0x35324e50, SUC_lexicon_tags_17 },
    { 0xdaf24e50, SUC_lexicon_tags_25 },
    { 0x3b24e52, SUC_lexicon_tags_60 },
    { 0x27924e53, SUC_lexicon_tags_37 },
    { 0x8b24e51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb24e58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff724e5a, SUC_lexicon_tags_39 },
    { 0xcdd24e5b, SUC_lexicon_tags_11 },
    { 0x70d24e5c, SUC_lexicon_tags_12 },
    { 0x87924e5d, SUC_lexicon_tags_14 },
    { 0x5df24e5b, SUC_lexicon_tags_39 },
    { 0xacb24e5e, SUC_lexicon_tags_17 },
    { 0xc6f24e5d, SUC_lexicon_tags_33 },
    { 0x21124e61, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45924e68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb324e6c, SUC_lexicon_tags_61 },
    { 0xef924e6d, SUC_lexicon_tags_1 },
    { 0xa3524e6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d924e73, SUC_lexicon_tags_25 },
    { 0x93324e74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94124e7c, SUC_lexicon_tags_15 },
    { 0xf3d24e7c, SUC_lexicon_tags_77 },
    { 0x43f24e7d, SUC_lexicon_tags_14 },
    { 0x1e524e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3724e82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4324e86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0324e8e, SUC_lexicon_tags_1 },
    { 0xa2b24e8e, SUC_lexicon_tags_14 },
    { 0x31b24e90, SUC_lexicon_tags_12 },
    { 0x94524e91, SUC_lexicon_tags_68 },
    { 0xcc724e91, SUC_lexicon_tags_38 },
    { 0x79b24e91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xea524e96, SUC_lexicon_tags_50 },
    { 0xac724e97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7924e9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2d24e9d, SUC_lexicon_tags_16 },
    { 0x6e524e9e, SUC_lexicon_tags_11 },
    { 0x84f24e9f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30724ea3, SUC_lexicon_tags_27 },
    { 0x36124ea4, SUC_lexicon_tags_11 },
    { 0xb3124ea5, SUC_lexicon_tags_11 },
    { 0x3e324ea3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f24eaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86724eae, SUC_lexicon_tags_15 },
    { 0x13524eaf, SUC_lexicon_tags_17 },
    { 0x65f24eb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80124eb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0124eb6, SUC_lexicon_tags_15 },
    { 0x59124eb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd724ebb, SUC_lexicon_tags_37 },
    { 0x8f724ebb, SUC_lexicon_tags_329 },
    { 0, NULL },
    { 0xb6124ebe, SUC_lexicon_tags_8 },
    { 0x47b24ebf, SUC_lexicon_tags_12 },
    { 0x27324ec0, SUC_lexicon_tags_25 },
    { 0xab524ec1, SUC_lexicon_tags_27 },
    { 0xdd924ec1, SUC_lexicon_tags_15 },
    { 0xf3d24ec3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xab724ec5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3b24ec7, SUC_lexicon_tags_25 },
    { 0x1f924ec8, SUC_lexicon_tags_1 },
    { 0xd9f24ec9, SUC_lexicon_tags_2 },
    { 0x7a924ec8, SUC_lexicon_tags_26 },
    { 0x4b124ecb, SUC_lexicon_tags_2 },
    { 0xa6724eca, SUC_lexicon_tags_15 },
    { 0xc1b24ecd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaeb24ecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82124ed1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4cd24ed3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xff524ed5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96324edb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83924ede, SUC_lexicon_tags_30 },
    { 0xf9d24edf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x28d24ee1, SUC_lexicon_tags_27 },
    { 0xa6324ee1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e324eee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16b24ef0, SUC_lexicon_tags_37 },
    { 0xa6724ef1, SUC_lexicon_tags_19 },
    { 0xff124ef2, SUC_lexicon_tags_6 },
    { 0x3eb24ef3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b24ef8, SUC_lexicon_tags_14 },
    { 0xe5724ef9, SUC_lexicon_tags_30 },
    { 0xd4f24efa, SUC_lexicon_tags_11 },
    { 0x3eb24ef8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f24eff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47524f04, SUC_lexicon_tags_63 },
    { 0x5a124f04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7924f0b, SUC_lexicon_tags_534 },
    { 0xeed24f0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d124f0f, SUC_lexicon_tags_30 },
    { 0xb9324f10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d924f12, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x55924f15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc724f19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc724f22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39524f27, SUC_lexicon_tags_37 },
    { 0xd8924f27, SUC_lexicon_tags_25 },
    { 0x8f324f29, SUC_lexicon_tags_2 },
    { 0xec724f2a, SUC_lexicon_tags_75 },
    { 0x68f24f2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65f24f2d, SUC_lexicon_tags_11 },
    { 0xa7924f2e, SUC_lexicon_tags_16 },
    { 0x35724f2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86724f31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b24f34, SUC_lexicon_tags_151 },
    { 0xc0724f35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3d24f37, SUC_lexicon_tags_55 },
    { 0xc4f24f37, SUC_lexicon_tags_49 },
    { 0x3d24f39, SUC_lexicon_tags_29 },
    { 0xc5b24f39, SUC_lexicon_tags_68 },
    { 0x25724f39, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4924f3e, SUC_lexicon_tags_25 },
    { 0xe8b24f3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe524f41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2724f49, SUC_lexicon_tags_33 },
    { 0xdeb24f4a, SUC_lexicon_tags_1 },
    { 0x18524f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xded24f4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2df24f50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b24f53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10124f56, SUC_lexicon_tags_12 },
    { 0x9b324f57, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95724f62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70524f6f, SUC_lexicon_tags_198 },
    { 0xb6324f70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8924f72, SUC_lexicon_tags_61 },
    { 0x24b24f73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa124f79, SUC_lexicon_tags_25 },
    { 0x19d24f7a, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f24f7e, SUC_lexicon_tags_26 },
    { 0xacf24f7e, SUC_lexicon_tags_21 },
    { 0x3cd24f80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a324f83, SUC_lexicon_tags_12 },
    { 0x28524f83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf24f8d, SUC_lexicon_tags_17 },
    { 0x33924f8e, SUC_lexicon_tags_0 },
    { 0x6ef24f8d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b24f92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc724f98, SUC_lexicon_tags_16 },
    { 0x85324f98, SUC_lexicon_tags_25 },
    { 0xd7d24f9a, SUC_lexicon_tags_46 },
    { 0x28124f98, SUC_lexicon_tags_11 },
    { 0x14f24f9a, SUC_lexicon_tags_63 },
    { 0x1fb24f98, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f24fa2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90124faa, SUC_lexicon_tags_2 },
    { 0x2f24fab, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x21124fad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d24fb2, SUC_lexicon_tags_17 },
    { 0x5c924fb3, SUC_lexicon_tags_27 },
    { 0x43124fb4, SUC_lexicon_tags_8 },
    { 0xb9324fb5, SUC_lexicon_tags_38 },
    { 0x46924fb6, SUC_lexicon_tags_63 },
    { 0xa524fb6, SUC_lexicon_tags_47 },
    { 0x26124fb8, SUC_lexicon_tags_45 },
    { 0x95924fb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68724fbb, SUC_lexicon_tags_9 },
    { 0x15324fbb, SUC_lexicon_tags_13 },
    { 0xabf24fbc, SUC_lexicon_tags_7 },
    { 0xfd24fbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61d24fc0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5524fc4, SUC_lexicon_tags_37 },
    { 0xab724fc5, SUC_lexicon_tags_33 },
    { 0x57524fc6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b724fc8, SUC_lexicon_tags_9 },
    { 0xedd24fc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48724fcb, SUC_lexicon_tags_37 },
    { 0xa924fcc, SUC_lexicon_tags_56 },
    { 0xd2124fcd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67b24fcf, SUC_lexicon_tags_11 },
    { 0xa2324fd0, SUC_lexicon_tags_9 },
    { 0x10524fd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2724fd3, SUC_lexicon_tags_66 },
    { 0x79724fd3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf524fd9, SUC_lexicon_tags_29 },
    { 0x86f24fda, SUC_lexicon_tags_43 },
    { 0x91b24fdb, SUC_lexicon_tags_1 },
    { 0x40924fdc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x31324fde, SUC_lexicon_tags_7 },
    { 0xaaf24fde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d24fe6, SUC_lexicon_tags_27 },
    { 0xc8924fe7, SUC_lexicon_tags_12 },
    { 0xf8d24fe8, SUC_lexicon_tags_64 },
    { 0x81f24fe9, SUC_lexicon_tags_230 },
    { 0x4f924fea, SUC_lexicon_tags_9 },
    { 0x3d124fea, SUC_lexicon_tags_14 },
    { 0x86924fea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5d24fee, SUC_lexicon_tags_66 },
    { 0x2d724fee, SUC_lexicon_tags_29 },
    { 0x4924ff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f24ff2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1124ff6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e124ffa, SUC_lexicon_tags_45 },
    { 0x7e724ffb, SUC_lexicon_tags_11 },
    { 0x51f24ffa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f725000, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91f25002, SUC_lexicon_tags_23 },
    { 0x9325003, SUC_lexicon_tags_0 },
    { 0xbe525004, SUC_lexicon_tags_11 },
    { 0x73325005, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32325009, SUC_lexicon_tags_17 },
    { 0x6ef25009, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xef32500c, SUC_lexicon_tags_14 },
    { 0x6892500c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b52500f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b25014, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3432501b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6972501d, SUC_lexicon_tags_49 },
    { 0xa512501d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d25023, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd25026, SUC_lexicon_tags_32 },
    { 0x14f25027, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb25030, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8325033, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d725036, SUC_lexicon_tags_37 },
    { 0x8fd25036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa32503b, SUC_lexicon_tags_31 },
    { 0xbfd2503c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5c32503e, SUC_lexicon_tags_17 },
    { 0x7832503f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf925041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33125045, SUC_lexicon_tags_63 },
    { 0x29325046, SUC_lexicon_tags_11 },
    { 0x17325045, SUC_lexicon_tags_1 },
    { 0xeeb25048, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d2504f, SUC_lexicon_tags_12 },
    { 0x8272504f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x90b25052, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12925057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b2505e, SUC_lexicon_tags_17 },
    { 0x2572505f, SUC_lexicon_tags_46 },
    { 0xb3125060, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2125062, SUC_lexicon_tags_47 },
    { 0x2e125063, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8732506f, SUC_lexicon_tags_62 },
    { 0x22125070, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d25074, SUC_lexicon_tags_11 },
    { 0xccf25075, SUC_lexicon_tags_69 },
    { 0x1b725076, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d525079, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a925080, SUC_lexicon_tags_12 },
    { 0x4525081, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8925086, SUC_lexicon_tags_12 },
    { 0x50725087, SUC_lexicon_tags_32 },
    { 0x45f25088, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d2508c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb2508f, SUC_lexicon_tags_45 },
    { 0xd8925090, SUC_lexicon_tags_15 },
    { 0xd3925091, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd25095, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x28125098, SUC_lexicon_tags_37 },
    { 0x71f25098, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72509d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf250a0, SUC_lexicon_tags_33 },
    { 0xed250a1, SUC_lexicon_tags_15 },
    { 0x639250a0, SUC_lexicon_tags_24 },
    { 0x5c3250a0, SUC_lexicon_tags_17 },
    { 0x161250a4, SUC_lexicon_tags_60 },
    { 0x49d250a4, SUC_lexicon_tags_43 },
    { 0x163250a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25250aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af250ae, SUC_lexicon_tags_17 },
    { 0xda9250af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f250b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27250b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d250bb, SUC_lexicon_tags_30 },
    { 0x549250bc, SUC_lexicon_tags_38 },
    { 0xe3f250bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1250c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b250c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94b250c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x827250c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9db250cb, SUC_lexicon_tags_6 },
    { 0x255250cb, SUC_lexicon_tags_26 },
    { 0x30b250cd, SUC_lexicon_tags_7 },
    { 0x437250cc, SUC_lexicon_tags_31 },
    { 0x3bf250cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e5250d1, SUC_lexicon_tags_33 },
    { 0x1f1250d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdab250d4, SUC_lexicon_tags_11 },
    { 0xafd250d4, SUC_lexicon_tags_191 },
    { 0x239250d6, SUC_lexicon_tags_2 },
    { 0x4af250d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99250de, SUC_lexicon_tags_25 },
    { 0xdf250df, SUC_lexicon_tags_37 },
    { 0x39f250df, SUC_lexicon_tags_25 },
    { 0x48d250df, SUC_lexicon_tags_15 },
    { 0xe57250e2, SUC_lexicon_tags_2 },
    { 0xf0b250e3, SUC_lexicon_tags_158 },
    { 0x4bd250e3, SUC_lexicon_tags_15 },
    { 0x57250e3, SUC_lexicon_tags_57 },
    { 0x65f250e6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f250e9, SUC_lexicon_tags_22 },
    { 0xb07250e9, SUC_lexicon_tags_63 },
    { 0x723250eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33250f0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75250f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x701250f8, SUC_lexicon_tags_77 },
    { 0xf6d250f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd75250fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a325100, SUC_lexicon_tags_21 },
    { 0xe5125101, SUC_lexicon_tags_37 },
    { 0x98925102, SUC_lexicon_tags_37 },
    { 0x4d125102, SUC_lexicon_tags_30 },
    { 0x6eb25100, SUC_lexicon_tags_1 },
    { 0xb7325105, SUC_lexicon_tags_37 },
    { 0xd7b25105, SUC_lexicon_tags_19 },
    { 0x98925107, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaf925109, SUC_lexicon_tags_65 },
    { 0xb7b2510a, SUC_lexicon_tags_12 },
    { 0xfd92510b, SUC_lexicon_tags_7 },
    { 0xb5b25109, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd25110, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb25113, SUC_lexicon_tags_27 },
    { 0x4fd25114, SUC_lexicon_tags_60 },
    { 0x41325115, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2925118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f2511a, SUC_lexicon_tags_14 },
    { 0xfb32511b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32511f, SUC_lexicon_tags_33 },
    { 0xf1925120, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40725122, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef25125, SUC_lexicon_tags_18 },
    { 0xfb925126, SUC_lexicon_tags_43 },
    { 0xc125127, SUC_lexicon_tags_19 },
    { 0x82125128, SUC_lexicon_tags_14 },
    { 0x7ab25128, SUC_lexicon_tags_9 },
    { 0xb1925127, SUC_lexicon_tags_4 },
    { 0x48725125, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1672512f, SUC_lexicon_tags_11 },
    { 0x7372512f, SUC_lexicon_tags_47 },
    { 0x9e325130, SUC_lexicon_tags_63 },
    { 0xfe725132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b25135, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1325138, SUC_lexicon_tags_80 },
    { 0xdaf25139, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d2513d, SUC_lexicon_tags_43 },
    { 0xa112513e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe525140, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf925143, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xafb25145, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd25148, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6db2514a, SUC_lexicon_tags_33 },
    { 0x4c92514a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4032514d, SUC_lexicon_tags_8 },
    { 0x1d2514e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a125155, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29b25157, SUC_lexicon_tags_25 },
    { 0x65d25157, SUC_lexicon_tags_0 },
    { 0x30125159, SUC_lexicon_tags_57 },
    { 0xeaf25158, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc892515d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d25160, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39325164, SUC_lexicon_tags_43 },
    { 0xdbb25165, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab25169, SUC_lexicon_tags_17 },
    { 0xcf72516a, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d2516e, SUC_lexicon_tags_37 },
    { 0xa692516e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfe125171, SUC_lexicon_tags_25 },
    { 0x4cd25172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec725176, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f25179, SUC_lexicon_tags_11 },
    { 0x2bb2517a, SUC_lexicon_tags_15 },
    { 0x7712517a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db2517e, SUC_lexicon_tags_4 },
    { 0x4c92517f, SUC_lexicon_tags_37 },
    { 0x6d92517f, SUC_lexicon_tags_1 },
    { 0x13325181, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d2518a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5912518d, SUC_lexicon_tags_207 },
    { 0x61d2518e, SUC_lexicon_tags_12 },
    { 0x43b2518f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d25195, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7251a0, SUC_lexicon_tags_39 },
    { 0xd61251a0, SUC_lexicon_tags_17 },
    { 0xa39251a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf251a7, SUC_lexicon_tags_55 },
    { 0x469251a7, SUC_lexicon_tags_8 },
    { 0xe9251a9, SUC_lexicon_tags_31 },
    { 0xf7b251aa, SUC_lexicon_tags_66 },
    { 0xf53251aa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5251af, SUC_lexicon_tags_25 },
    { 0x75251b0, SUC_lexicon_tags_7 },
    { 0x3a5251af, SUC_lexicon_tags_1 },
    { 0x61251b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377251b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b251b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3251be, SUC_lexicon_tags_7 },
    { 0xc33251bf, SUC_lexicon_tags_37 },
    { 0xf55251bf, SUC_lexicon_tags_37 },
    { 0x425251c1, SUC_lexicon_tags_1 },
    { 0x685251be, SUC_lexicon_tags_25 },
    { 0xe8f251be, SUC_lexicon_tags_1 },
    { 0xf87251c2, SUC_lexicon_tags_48 },
    { 0xad3251c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83251ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d5251cc, SUC_lexicon_tags_17 },
    { 0x41251cd, SUC_lexicon_tags_79 },
    { 0x1d3251ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b251d6, SUC_lexicon_tags_1 },
    { 0x9c5251d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03251e0, SUC_lexicon_tags_30 },
    { 0x8bb251e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91251e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc39251e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f251eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d251ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719251f3, SUC_lexicon_tags_2 },
    { 0x39d251f4, SUC_lexicon_tags_1 },
    { 0x9b9251f4, SUC_lexicon_tags_43 },
    { 0x595251f5, SUC_lexicon_tags_16 },
    { 0x677251f7, SUC_lexicon_tags_46 },
    { 0xc97251f8, SUC_lexicon_tags_33 },
    { 0xdc7251f8, SUC_lexicon_tags_11 },
    { 0xd7f251fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3a1251fc, SUC_lexicon_tags_6 },
    { 0xaf5251fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd7b251ff, SUC_lexicon_tags_15 },
    { 0x2a725200, SUC_lexicon_tags_63 },
    { 0xfd7251ff, SUC_lexicon_tags_15 },
    { 0x9af251ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf25206, SUC_lexicon_tags_17 },
    { 0x3725207, SUC_lexicon_tags_19 },
    { 0xd6525208, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7112520a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb25211, SUC_lexicon_tags_2 },
    { 0xbc325212, SUC_lexicon_tags_37 },
    { 0x46b25212, SUC_lexicon_tags_32 },
    { 0xf5f25211, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69725217, SUC_lexicon_tags_26 },
    { 0x11d25218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71725220, SUC_lexicon_tags_12 },
    { 0x3a725220, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e525224, SUC_lexicon_tags_10 },
    { 0xde325225, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x97925227, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b25230, SUC_lexicon_tags_14 },
    { 0xa1b25231, SUC_lexicon_tags_33 },
    { 0x19725232, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4525235, SUC_lexicon_tags_17 },
    { 0xf3f25236, SUC_lexicon_tags_37 },
    { 0xc4125235, SUC_lexicon_tags_23 },
    { 0xf125238, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4152523e, SUC_lexicon_tags_17 },
    { 0x8e52523f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x54325241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c525246, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xffd25248, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a72524b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb2524e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac725255, SUC_lexicon_tags_37 },
    { 0x3e525256, SUC_lexicon_tags_37 },
    { 0xaa525255, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d2525b, SUC_lexicon_tags_11 },
    { 0x8112525c, SUC_lexicon_tags_95 },
    { 0x7f72525b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe125262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb25268, SUC_lexicon_tags_47 },
    { 0x2a125269, SUC_lexicon_tags_8 },
    { 0x83725269, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c32526c, SUC_lexicon_tags_908 },
    { 0x9cd2526c, SUC_lexicon_tags_7 },
    { 0xf872526e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43325274, SUC_lexicon_tags_664 },
    { 0xae525275, SUC_lexicon_tags_25 },
    { 0x6ab25276, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24d25278, SUC_lexicon_tags_14 },
    { 0x56f25279, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf2527d, SUC_lexicon_tags_27 },
    { 0x2972527d, SUC_lexicon_tags_37 },
    { 0xfb92527f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ab25281, SUC_lexicon_tags_39 },
    { 0x37525282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84325284, SUC_lexicon_tags_13 },
    { 0x3f525285, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4932528b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1412528d, SUC_lexicon_tags_19 },
    { 0xe832528d, SUC_lexicon_tags_15 },
    { 0x7e92528e, SUC_lexicon_tags_14 },
    { 0x98125290, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85125294, SUC_lexicon_tags_37 },
    { 0xd3b25295, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc925299, SUC_lexicon_tags_37 },
    { 0x5a725299, SUC_lexicon_tags_0 },
    { 0x79525299, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d2529f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1252a6, SUC_lexicon_tags_37 },
    { 0xadf252a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x75d252a9, SUC_lexicon_tags_38 },
    { 0x791252aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853252ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe7252b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7252b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5252b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf77252b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x585252bc, SUC_lexicon_tags_37 },
    { 0x7a5252bc, SUC_lexicon_tags_29 },
    { 0x955252be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd252c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x205252c5, SUC_lexicon_tags_17 },
    { 0xf25252c5, SUC_lexicon_tags_17 },
    { 0xeff252c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbb5252c9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab252ce, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb252d1, SUC_lexicon_tags_45 },
    { 0xb59252d2, SUC_lexicon_tags_11 },
    { 0x697252d3, SUC_lexicon_tags_61 },
    { 0xc5252d2, SUC_lexicon_tags_29 },
    { 0x253252d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b252dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaf252de, SUC_lexicon_tags_37 },
    { 0xd07252df, SUC_lexicon_tags_11 },
    { 0xbd7252e0, SUC_lexicon_tags_30 },
    { 0x169252e0, SUC_lexicon_tags_25 },
    { 0x859252e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x635252e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435252eb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1b5252ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x745252f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7252f3, SUC_lexicon_tags_2 },
    { 0x3d1252f3, SUC_lexicon_tags_11 },
    { 0xcbb252f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3b252f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfff252f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaed252fb, SUC_lexicon_tags_37 },
    { 0x5f9252fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0925303, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b25309, SUC_lexicon_tags_49 },
    { 0x31b25309, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3872530d, SUC_lexicon_tags_48 },
    { 0xe7d2530d, SUC_lexicon_tags_37 },
    { 0x6212530e, SUC_lexicon_tags_47 },
    { 0xe4d2530d, SUC_lexicon_tags_31 },
    { 0x78525311, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70125319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd792531b, SUC_lexicon_tags_8 },
    { 0x3632531c, SUC_lexicon_tags_33 },
    { 0xbfd2531d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x3f72531f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c125325, SUC_lexicon_tags_45 },
    { 0x75325326, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f2532c, SUC_lexicon_tags_6 },
    { 0x8452532d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf912532f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9f25331, SUC_lexicon_tags_7 },
    { 0x2a725332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9bf25334, SUC_lexicon_tags_49 },
    { 0x9d525334, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd25338, SUC_lexicon_tags_42 },
    { 0xabb25338, SUC_lexicon_tags_1 },
    { 0x3af2533a, SUC_lexicon_tags_37 },
    { 0x80d2533b, SUC_lexicon_tags_1 },
    { 0xdab2533b, SUC_lexicon_tags_11 },
    { 0xe852533c, SUC_lexicon_tags_37 },
    { 0x8852533a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa925340, SUC_lexicon_tags_14 },
    { 0x9dd25340, SUC_lexicon_tags_21 },
    { 0x32b25342, SUC_lexicon_tags_23 },
    { 0xb5b25342, SUC_lexicon_tags_8 },
    { 0xa3f25344, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa2525346, SUC_lexicon_tags_38 },
    { 0x63f25347, SUC_lexicon_tags_37 },
    { 0x65925348, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f2534b, SUC_lexicon_tags_49 },
    { 0x2ed2534c, SUC_lexicon_tags_1285 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b725351, SUC_lexicon_tags_11 },
    { 0x26325351, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af25356, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f2535a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b2535e, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4525362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f25365, SUC_lexicon_tags_7 },
    { 0xc7d25366, SUC_lexicon_tags_25 },
    { 0xb6b25366, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6692536a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4712536c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a325372, SUC_lexicon_tags_32 },
    { 0x62d25373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd432537a, SUC_lexicon_tags_25 },
    { 0x2bb2537a, SUC_lexicon_tags_38 },
    { 0x1832537c, SUC_lexicon_tags_15 },
    { 0xcad2537b, SUC_lexicon_tags_2 },
    { 0x4b52537c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf3725380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7925387, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6892538a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e72538e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75725390, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99325399, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b2539f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151253a3, SUC_lexicon_tags_0 },
    { 0xc6f253a4, SUC_lexicon_tags_13 },
    { 0x737253a4, SUC_lexicon_tags_7 },
    { 0xe4f253a6, SUC_lexicon_tags_55 },
    { 0x321253a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4b253a9, SUC_lexicon_tags_23 },
    { 0x2bb253aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b253ae, SUC_lexicon_tags_25 },
    { 0x8e7253ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d253b4, SUC_lexicon_tags_1 },
    { 0x899253b5, SUC_lexicon_tags_9 },
    { 0xf49253b6, SUC_lexicon_tags_37 },
    { 0xad3253b4, SUC_lexicon_tags_2 },
    { 0xd0b253b8, SUC_lexicon_tags_25 },
    { 0x8e7253b9, SUC_lexicon_tags_11 },
    { 0x517253ba, SUC_lexicon_tags_38 },
    { 0xee1253b8, SUC_lexicon_tags_32 },
    { 0x88d253b5, SUC_lexicon_tags_160 },
    { 0x29b253bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1253c0, SUC_lexicon_tags_16 },
    { 0x5b9253c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6a1253c3, SUC_lexicon_tags_8 },
    { 0x155253c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa8b253c6, SUC_lexicon_tags_12 },
    { 0xba5253c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc95253c9, SUC_lexicon_tags_15 },
    { 0xc51253c9, SUC_lexicon_tags_2 },
    { 0x573253c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163253d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1cd253d3, SUC_lexicon_tags_12 },
    { 0x1c3253d4, SUC_lexicon_tags_8 },
    { 0x639253d5, SUC_lexicon_tags_37 },
    { 0x3cf253d3, SUC_lexicon_tags_1 },
    { 0x6ad253d3, SUC_lexicon_tags_23 },
    { 0x5a1253d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55253dd, SUC_lexicon_tags_15 },
    { 0x2a1253de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b253e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75253e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445253ed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d253f0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89253f5, SUC_lexicon_tags_15 },
    { 0x623253f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30525404, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4c525406, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xd2125408, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5552540a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb272540c, SUC_lexicon_tags_37 },
    { 0x7652540d, SUC_lexicon_tags_47 },
    { 0xd552540e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10125410, SUC_lexicon_tags_2 },
    { 0x40725411, SUC_lexicon_tags_19 },
    { 0xcf125412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d25416, SUC_lexicon_tags_19 },
    { 0xe9d25417, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27d25419, SUC_lexicon_tags_11 },
    { 0x4852541a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x712541c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f25421, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1725424, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b25428, SUC_lexicon_tags_8 },
    { 0xafd25428, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf2542d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b525430, SUC_lexicon_tags_15 },
    { 0x8c325431, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b725436, SUC_lexicon_tags_7 },
    { 0xd0925437, SUC_lexicon_tags_7 },
    { 0xba725438, SUC_lexicon_tags_1 },
    { 0xd9d25436, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b12543c, SUC_lexicon_tags_0 },
    { 0xb472543d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb25443, SUC_lexicon_tags_2 },
    { 0x73525444, SUC_lexicon_tags_12 },
    { 0x16b25445, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92544c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e325452, SUC_lexicon_tags_11 },
    { 0x99125453, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe325455, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72925458, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf952545c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41725464, SUC_lexicon_tags_29 },
    { 0x6125465, SUC_lexicon_tags_7 },
    { 0xd1725464, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d2546a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf792546d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e325470, SUC_lexicon_tags_38 },
    { 0xfab25471, SUC_lexicon_tags_1 },
    { 0x1f925472, SUC_lexicon_tags_38 },
    { 0x12525473, SUC_lexicon_tags_11 },
    { 0x16f25473, SUC_lexicon_tags_27 },
    { 0x69925475, SUC_lexicon_tags_25 },
    { 0x76125475, SUC_lexicon_tags_65 },
    { 0xa6325474, SUC_lexicon_tags_69 },
    { 0xf725474, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2725480, SUC_lexicon_tags_17 },
    { 0x5af25480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee925484, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x33325486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9125489, SUC_lexicon_tags_48 },
    { 0xc5f2548a, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xedb2548c, SUC_lexicon_tags_1 },
    { 0x3ab2548c, SUC_lexicon_tags_19 },
    { 0x3752548d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2725493, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd2549a, SUC_lexicon_tags_1 },
    { 0xa392549a, SUC_lexicon_tags_96 },
    { 0xa392549c, SUC_lexicon_tags_15 },
    { 0x1052549c, SUC_lexicon_tags_1 },
    { 0xf752549e, SUC_lexicon_tags_25 },
    { 0x2fd2549f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697254a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05254ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x153254b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37254b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa11254b5, SUC_lexicon_tags_12 },
    { 0x27254b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30f254b8, SUC_lexicon_tags_60 },
    { 0x47d254b9, SUC_lexicon_tags_12 },
    { 0xd69254b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x957254bc, SUC_lexicon_tags_37 },
    { 0x37254bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82d254bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x45b254c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1254c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57254ca, SUC_lexicon_tags_7 },
    { 0x471254ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5254d1, SUC_lexicon_tags_66 },
    { 0x811254d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99254d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9254d8, SUC_lexicon_tags_9 },
    { 0xeb254d9, SUC_lexicon_tags_15 },
    { 0xffd254da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b254eb, SUC_lexicon_tags_12 },
    { 0xc33254eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df254f1, SUC_lexicon_tags_1007 },
    { 0x6a7254f2, SUC_lexicon_tags_35 },
    { 0x3b3254f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d254f8, SUC_lexicon_tags_25 },
    { 0x649254f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaef254fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7f254fd, SUC_lexicon_tags_47 },
    { 0xc33254fd, SUC_lexicon_tags_10 },
    { 0x1ed254fe, SUC_lexicon_tags_15 },
    { 0xaf9254fe, SUC_lexicon_tags_37 },
    { 0xae525501, SUC_lexicon_tags_38 },
    { 0x27d25502, SUC_lexicon_tags_97 },
    { 0xf1b25503, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d25507, SUC_lexicon_tags_25 },
    { 0x1e525508, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d25510, SUC_lexicon_tags_1 },
    { 0x2df25511, SUC_lexicon_tags_7 },
    { 0x87f25511, SUC_lexicon_tags_29 },
    { 0xd5f25513, SUC_lexicon_tags_66 },
    { 0x3fb25512, SUC_lexicon_tags_59 },
    { 0x11525514, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x42325517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70125519, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1af2551b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d25522, SUC_lexicon_tags_6 },
    { 0xeb725523, SUC_lexicon_tags_17 },
    { 0x5a925524, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47725528, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5632552d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe925530, SUC_lexicon_tags_17 },
    { 0x57525531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61925533, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed25536, SUC_lexicon_tags_23 },
    { 0x29125537, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0xe6725539, SUC_lexicon_tags_15 },
    { 0x54925539, SUC_lexicon_tags_31 },
    { 0x98725539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86725540, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d25545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9725547, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xff25549, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d92554b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4972554e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe725555, SUC_lexicon_tags_25 },
    { 0x3ed25555, SUC_lexicon_tags_17 },
    { 0xcb25557, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf92555a, SUC_lexicon_tags_17 },
    { 0xd6b2555b, SUC_lexicon_tags_7 },
    { 0xcd12555b, SUC_lexicon_tags_9 },
    { 0x8bd2555d, SUC_lexicon_tags_43 },
    { 0xb352555e, SUC_lexicon_tags_23 },
    { 0x7952555f, SUC_lexicon_tags_0 },
    { 0x1b2555f, SUC_lexicon_tags_151 },
    { 0xd8b25561, SUC_lexicon_tags_2 },
    { 0x4b925562, SUC_lexicon_tags_25 },
    { 0xb332555e, SUC_lexicon_tags_47 },
    { 0x32b2555e, SUC_lexicon_tags_11 },
    { 0xb4f2555d, SUC_lexicon_tags_477 },
    { 0x49d25566, SUC_lexicon_tags_6 },
    { 0xa125567, SUC_lexicon_tags_30 },
    { 0x49525564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a12556b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f25572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d25577, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x67f25579, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9912557b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda72557e, SUC_lexicon_tags_18 },
    { 0xe592557e, SUC_lexicon_tags_17 },
    { 0x6b125580, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37325582, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb1925584, SUC_lexicon_tags_7 },
    { 0xdb125585, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9bd25587, SUC_lexicon_tags_25 },
    { 0x84125588, SUC_lexicon_tags_2 },
    { 0x26725589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4312558c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d2558f, SUC_lexicon_tags_7 },
    { 0x49f2558f, SUC_lexicon_tags_11 },
    { 0xe7125591, SUC_lexicon_tags_198 },
    { 0xee12558f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80325594, SUC_lexicon_tags_7 },
    { 0x82d25595, SUC_lexicon_tags_2 },
    { 0x30725596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f2559d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39255a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd59255a3, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x973255a5, SUC_lexicon_tags_37 },
    { 0xec3255a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1255ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4ef255ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f255b1, SUC_lexicon_tags_1 },
    { 0x955255b2, SUC_lexicon_tags_7 },
    { 0xb71255b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b255b7, SUC_lexicon_tags_7 },
    { 0x37f255b7, SUC_lexicon_tags_2 },
    { 0xed255b8, SUC_lexicon_tags_2 },
    { 0xe75255b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23255c5, SUC_lexicon_tags_16 },
    { 0xb9d255c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b255ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x543255cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81d255ce, SUC_lexicon_tags_12 },
    { 0xfc3255cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x699255d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7255d6, SUC_lexicon_tags_12 },
    { 0x6bd255d7, SUC_lexicon_tags_37 },
    { 0x215255d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985255de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd21255e0, SUC_lexicon_tags_30 },
    { 0xe83255e0, SUC_lexicon_tags_2 },
    { 0x3b255e2, SUC_lexicon_tags_12 },
    { 0x349255e3, SUC_lexicon_tags_77 },
    { 0xf81255e4, SUC_lexicon_tags_12 },
    { 0x4d1255e3, SUC_lexicon_tags_15 },
    { 0xc87255e4, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db255e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7255ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x189255ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcc1255f1, SUC_lexicon_tags_46 },
    { 0x4ed255f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321255f6, SUC_lexicon_tags_45 },
    { 0x785255f7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29255fb, SUC_lexicon_tags_55 },
    { 0xdb9255fc, SUC_lexicon_tags_15 },
    { 0x6a3255fd, SUC_lexicon_tags_49 },
    { 0xf6f255fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb25604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb125607, SUC_lexicon_tags_17 },
    { 0xa6925607, SUC_lexicon_tags_12 },
    { 0x46325607, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ab2560b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d25610, SUC_lexicon_tags_2 },
    { 0x3bd25611, SUC_lexicon_tags_15 },
    { 0x14f25611, SUC_lexicon_tags_47 },
    { 0x49d25613, SUC_lexicon_tags_27 },
    { 0x9f325614, SUC_lexicon_tags_4 },
    { 0xf2b25613, SUC_lexicon_tags_17 },
    { 0x12525616, SUC_lexicon_tags_38 },
    { 0xadf25617, SUC_lexicon_tags_15 },
    { 0x82525618, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74f2561a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67125622, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65525629, SUC_lexicon_tags_17 },
    { 0xa7525629, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6bb2562c, SUC_lexicon_tags_11 },
    { 0xa4f2562d, SUC_lexicon_tags_15 },
    { 0x4672562e, SUC_lexicon_tags_31 },
    { 0xd1b2562e, SUC_lexicon_tags_14 },
    { 0x86325630, SUC_lexicon_tags_17 },
    { 0x6bf25630, SUC_lexicon_tags_17 },
    { 0x19325631, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55725636, SUC_lexicon_tags_3 },
    { 0xe4725637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4832563a, SUC_lexicon_tags_31 },
    { 0xf0f2563b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc6f2563d, SUC_lexicon_tags_15 },
    { 0xf2b2563d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3325641, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21f25643, SUC_lexicon_tags_35 },
    { 0xba325644, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20b25646, SUC_lexicon_tags_25 },
    { 0x3725647, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca72564c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62f2564e, SUC_lexicon_tags_17 },
    { 0xf92564f, SUC_lexicon_tags_11 },
    { 0x3812564f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60f25652, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b25656, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f25662, SUC_lexicon_tags_12 },
    { 0x4db25662, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86725668, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe512566e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f25674, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a52567d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xea72567f, SUC_lexicon_tags_17 },
    { 0xca72567f, SUC_lexicon_tags_17 },
    { 0x3f725680, SUC_lexicon_tags_7 },
    { 0x75925682, SUC_lexicon_tags_27 },
    { 0x7d525683, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x31325686, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2568b, SUC_lexicon_tags_37 },
    { 0xd372568c, SUC_lexicon_tags_47 },
    { 0xe612568d, SUC_lexicon_tags_6 },
    { 0xbb12568d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf3525690, SUC_lexicon_tags_17 },
    { 0xf5925691, SUC_lexicon_tags_37 },
    { 0x4f325692, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72925696, SUC_lexicon_tags_17 },
    { 0x5ef25697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde32569a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5252569c, SUC_lexicon_tags_14 },
    { 0x7112569d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4012569f, SUC_lexicon_tags_87 },
    { 0xcf256a0, SUC_lexicon_tags_25 },
    { 0xa212569f, SUC_lexicon_tags_7 },
    { 0xb7d256a2, SUC_lexicon_tags_27 },
    { 0x8e72569f, SUC_lexicon_tags_25 },
    { 0xcf9256a4, SUC_lexicon_tags_47 },
    { 0x677256a5, SUC_lexicon_tags_77 },
    { 0xc01256a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37256ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab256af, SUC_lexicon_tags_1 },
    { 0x979256b0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1256b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7256be, SUC_lexicon_tags_17 },
    { 0xa7f256be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9f256c1, SUC_lexicon_tags_68 },
    { 0xb61256c1, SUC_lexicon_tags_15 },
    { 0xe05256c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705256c9, SUC_lexicon_tags_1 },
    { 0x8ff256c9, SUC_lexicon_tags_337 },
    { 0xd2d256ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3256d1, SUC_lexicon_tags_22 },
    { 0x381256d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607256d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x67b256da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931256df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe79256e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3256ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b256ee, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xaa1256f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3256f5, SUC_lexicon_tags_14 },
    { 0x457256f6, SUC_lexicon_tags_7 },
    { 0x2bf256f7, SUC_lexicon_tags_6 },
    { 0x105256f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789256fd, SUC_lexicon_tags_25 },
    { 0x82f256fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3925700, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc612570a, SUC_lexicon_tags_11 },
    { 0xaeb2570b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3f72570d, SUC_lexicon_tags_25 },
    { 0x9fd2570e, SUC_lexicon_tags_1 },
    { 0x34b2570d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0125711, SUC_lexicon_tags_27 },
    { 0xd6f25711, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x65125715, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf32571f, SUC_lexicon_tags_1 },
    { 0x1f32571f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xee125722, SUC_lexicon_tags_32 },
    { 0x1f725723, SUC_lexicon_tags_1 },
    { 0x85725724, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a725726, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff325729, SUC_lexicon_tags_28 },
    { 0x28125729, SUC_lexicon_tags_49 },
    { 0x14d2572b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63b2572d, SUC_lexicon_tags_66 },
    { 0x97f2572e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23725736, SUC_lexicon_tags_12 },
    { 0xdd325737, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7c325739, SUC_lexicon_tags_15 },
    { 0x36725739, SUC_lexicon_tags_22 },
    { 0xf72573a, SUC_lexicon_tags_31 },
    { 0x62d25739, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad25741, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b25747, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a12574b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b325755, SUC_lexicon_tags_30 },
    { 0x31325755, SUC_lexicon_tags_15 },
    { 0xb5325755, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b2575d, SUC_lexicon_tags_31 },
    { 0xc2d2575d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8325761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73525767, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b125769, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2752576b, SUC_lexicon_tags_17 },
    { 0xf672576b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8f2576e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf25771, SUC_lexicon_tags_33 },
    { 0x8e925772, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3525774, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51725780, SUC_lexicon_tags_14 },
    { 0xebb25780, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf925784, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df25789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb852578f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb25793, SUC_lexicon_tags_16 },
    { 0xd1b25793, SUC_lexicon_tags_15 },
    { 0xf1125795, SUC_lexicon_tags_69 },
    { 0xfe325796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5325798, SUC_lexicon_tags_8 },
    { 0xbc925799, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a72579c, SUC_lexicon_tags_11 },
    { 0xacd2579d, SUC_lexicon_tags_194 },
    { 0x26d2579d, SUC_lexicon_tags_9 },
    { 0xd6b2579d, SUC_lexicon_tags_31 },
    { 0xd72579f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b257a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f257a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af257af, SUC_lexicon_tags_1 },
    { 0xc71257b0, SUC_lexicon_tags_27 },
    { 0xb35257b1, SUC_lexicon_tags_26 },
    { 0x255257b2, SUC_lexicon_tags_0 },
    { 0xe17257b3, SUC_lexicon_tags_31 },
    { 0x2f9257b2, SUC_lexicon_tags_11 },
    { 0x921257b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1257bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b257be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d257c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc7257c4, SUC_lexicon_tags_0 },
    { 0x94f257c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1257d0, SUC_lexicon_tags_8 },
    { 0x671257d1, SUC_lexicon_tags_15 },
    { 0x88f257d1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xca3257d4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09257d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf49257da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41257df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b257e2, SUC_lexicon_tags_46 },
    { 0xaff257e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01257ec, SUC_lexicon_tags_12 },
    { 0x63d257ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b257f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793257f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f257fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdbb257ff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28525805, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0325807, SUC_lexicon_tags_63 },
    { 0x2a725807, SUC_lexicon_tags_15 },
    { 0x60325807, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c32580c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7672580f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0925812, SUC_lexicon_tags_31 },
    { 0xb6b25813, SUC_lexicon_tags_21 },
    { 0x4cf25813, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49925818, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb872581a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd2581e, SUC_lexicon_tags_77 },
    { 0xd552581f, SUC_lexicon_tags_16 },
    { 0xa352581f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3f925822, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d925826, SUC_lexicon_tags_37 },
    { 0x7b925827, SUC_lexicon_tags_23 },
    { 0x3c125828, SUC_lexicon_tags_28 },
    { 0x54925829, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6e52582b, SUC_lexicon_tags_0 },
    { 0x75f2582c, SUC_lexicon_tags_2 },
    { 0xa592582d, SUC_lexicon_tags_17 },
    { 0x1612582d, SUC_lexicon_tags_11 },
    { 0x42f2582f, SUC_lexicon_tags_42 },
    { 0x58f2582b, SUC_lexicon_tags_38 },
    { 0xf0b2582f, SUC_lexicon_tags_30 },
    { 0xd3b2582e, SUC_lexicon_tags_19 },
    { 0xecd25831, SUC_lexicon_tags_27 },
    { 0x5152582b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2725836, SUC_lexicon_tags_65 },
    { 0x95f25837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f2583a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e12583e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb125840, SUC_lexicon_tags_25 },
    { 0x50525841, SUC_lexicon_tags_19 },
    { 0xb9d25840, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b725848, SUC_lexicon_tags_23 },
    { 0x75b25849, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x26b2584b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d2584e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86125852, SUC_lexicon_tags_27 },
    { 0x90325853, SUC_lexicon_tags_2 },
    { 0xb6925854, SUC_lexicon_tags_19 },
    { 0x40925852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2325857, SUC_lexicon_tags_45 },
    { 0x88d25858, SUC_lexicon_tags_7 },
    { 0x96925858, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe7b2585b, SUC_lexicon_tags_37 },
    { 0x292585c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeb2585e, SUC_lexicon_tags_37 },
    { 0xf052585f, SUC_lexicon_tags_2 },
    { 0xc8b2585e, SUC_lexicon_tags_55 },
    { 0x6112585f, SUC_lexicon_tags_28 },
    { 0x2852585f, SUC_lexicon_tags_27 },
    { 0x90525863, SUC_lexicon_tags_37 },
    { 0x8f125863, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc5725866, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38725868, SUC_lexicon_tags_45 },
    { 0x4b925869, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3dd2586b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f2586f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18925873, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x76525876, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee12587c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb692587f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x89125882, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9725888, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee32588c, SUC_lexicon_tags_7 },
    { 0xd912588d, SUC_lexicon_tags_0 },
    { 0x9732588c, SUC_lexicon_tags_46 },
    { 0x8cf2588d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8db25891, SUC_lexicon_tags_63 },
    { 0xce525891, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x8a125894, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f52589e, SUC_lexicon_tags_3 },
    { 0xb972589f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d258a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197258ad, SUC_lexicon_tags_49 },
    { 0xd03258ae, SUC_lexicon_tags_15 },
    { 0xef258ad, SUC_lexicon_tags_7 },
    { 0xf1258b0, SUC_lexicon_tags_1 },
    { 0x229258b0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x271258b3, SUC_lexicon_tags_46 },
    { 0x62f258b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95258b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585258bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5258c0, SUC_lexicon_tags_28 },
    { 0x983258c1, SUC_lexicon_tags_17 },
    { 0x419258c0, SUC_lexicon_tags_17 },
    { 0x709258c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x135258c5, SUC_lexicon_tags_11 },
    { 0xbbf258c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x969258c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x47b258cb, SUC_lexicon_tags_17 },
    { 0xe1d258cc, SUC_lexicon_tags_17 },
    { 0x55f258cb, SUC_lexicon_tags_84 },
    { 0x201258cc, SUC_lexicon_tags_7 },
    { 0x397258cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777258d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3258dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb7258de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5258e1, SUC_lexicon_tags_7 },
    { 0x2f7258e2, SUC_lexicon_tags_1 },
    { 0x8c3258e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d258e6, SUC_lexicon_tags_45 },
    { 0xdf3258e7, SUC_lexicon_tags_2 },
    { 0xdf1258e8, SUC_lexicon_tags_33 },
    { 0x753258e8, SUC_lexicon_tags_29 },
    { 0x1dd258e9, SUC_lexicon_tags_60 },
    { 0x2ad258ea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d258ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d258f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76b258f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9258f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343258fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10925901, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9db25903, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b2590d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c925912, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a525916, SUC_lexicon_tags_15 },
    { 0x87d25916, SUC_lexicon_tags_12 },
    { 0x9cd25918, SUC_lexicon_tags_7 },
    { 0xcbf25919, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5572591d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbef2591f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x45325921, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x68525924, SUC_lexicon_tags_17 },
    { 0x5cd25925, SUC_lexicon_tags_49 },
    { 0x5af25926, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4a525928, SUC_lexicon_tags_8 },
    { 0xbeb25929, SUC_lexicon_tags_60 },
    { 0xa4725929, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd2592e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7ad25930, SUC_lexicon_tags_107 },
    { 0x74725931, SUC_lexicon_tags_43 },
    { 0x27925932, SUC_lexicon_tags_15 },
    { 0x19f25931, SUC_lexicon_tags_37 },
    { 0x66f25933, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b725938, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d25948, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b325950, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x13b25952, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f2595a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb312595c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed325960, SUC_lexicon_tags_25 },
    { 0xe3325961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b725968, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f2596d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x84125970, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5925978, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5dd2597a, SUC_lexicon_tags_6 },
    { 0xa332597b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa892597d, SUC_lexicon_tags_47 },
    { 0xb12597e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d25981, SUC_lexicon_tags_10 },
    { 0xe7725982, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37725984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e925988, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf952598a, SUC_lexicon_tags_63 },
    { 0x2af2598b, SUC_lexicon_tags_27 },
    { 0xaef2598b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2598f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed25993, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8512599a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6392599c, SUC_lexicon_tags_12 },
    { 0x8e12599d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717259a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9259a7, SUC_lexicon_tags_10 },
    { 0xacb259a7, SUC_lexicon_tags_55 },
    { 0x57259a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef3259ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x295259ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373259b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f259bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1259c0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d259c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x511259c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x841259c7, SUC_lexicon_tags_7 },
    { 0xca3259c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb259cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317259d2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xe7b259d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x75b259d6, SUC_lexicon_tags_14 },
    { 0x9d259d7, SUC_lexicon_tags_65 },
    { 0x721259d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb259db, SUC_lexicon_tags_17 },
    { 0x109259dc, SUC_lexicon_tags_1 },
    { 0x153259dd, SUC_lexicon_tags_1 },
    { 0x901259de, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b259e8, SUC_lexicon_tags_7 },
    { 0x77f259e9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x74f259eb, SUC_lexicon_tags_12 },
    { 0x9cd259ec, SUC_lexicon_tags_11 },
    { 0x7ef259ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab259f1, SUC_lexicon_tags_30 },
    { 0xe6f259f2, SUC_lexicon_tags_32 },
    { 0x949259f2, SUC_lexicon_tags_12 },
    { 0xe43259f4, SUC_lexicon_tags_26 },
    { 0x96d259f5, SUC_lexicon_tags_1 },
    { 0x407259f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761259fa, SUC_lexicon_tags_33 },
    { 0x81b259fa, SUC_lexicon_tags_14 },
    { 0xb41259fc, SUC_lexicon_tags_14 },
    { 0xba7259fd, SUC_lexicon_tags_46 },
    { 0x279259fd, SUC_lexicon_tags_56 },
    { 0xf37259fe, SUC_lexicon_tags_31 },
    { 0xfaf25a00, SUC_lexicon_tags_38 },
    { 0x96d259ff, SUC_lexicon_tags_45 },
    { 0xd73259fc, SUC_lexicon_tags_15 },
    { 0xd8b25a03, SUC_lexicon_tags_11 },
    { 0x9ff25a04, SUC_lexicon_tags_15 },
    { 0x35325a04, SUC_lexicon_tags_37 },
    { 0x70f25a06, SUC_lexicon_tags_19 },
    { 0x41525a07, SUC_lexicon_tags_21 },
    { 0x94925a02, SUC_lexicon_tags_13 },
    { 0x6f125a09, SUC_lexicon_tags_46 },
    { 0x54525a09, SUC_lexicon_tags_17 },
    { 0x39f259fd, SUC_lexicon_tags_2 },
    { 0x76725a0c, SUC_lexicon_tags_12 },
    { 0x86325a0b, SUC_lexicon_tags_60 },
    { 0x7cd25a01, SUC_lexicon_tags_13 },
    { 0x7e125a0f, SUC_lexicon_tags_15 },
    { 0x1ed25a10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x64525a12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef25a17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84725a1b, SUC_lexicon_tags_60 },
    { 0x96725a1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a725a1f, SUC_lexicon_tags_15 },
    { 0xdd525a20, SUC_lexicon_tags_1 },
    { 0x12325a21, SUC_lexicon_tags_2 },
    { 0x4525a22, SUC_lexicon_tags_10 },
    { 0x6b725a22, SUC_lexicon_tags_25 },
    { 0xc2325a23, SUC_lexicon_tags_65 },
    { 0xef525a25, SUC_lexicon_tags_17 },
    { 0x9c725a26, SUC_lexicon_tags_15 },
    { 0x6925a25, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd4925a29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58525a32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f925a36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd25a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15525a3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5325a3e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf525a41, SUC_lexicon_tags_25 },
    { 0x5f125a42, SUC_lexicon_tags_77 },
    { 0x8bd25a43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb25a46, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x43d25a48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d25a4d, SUC_lexicon_tags_11 },
    { 0xa8925a4e, SUC_lexicon_tags_23 },
    { 0x31125a4e, SUC_lexicon_tags_26 },
    { 0x4dd25a4e, SUC_lexicon_tags_25 },
    { 0xc8725a50, SUC_lexicon_tags_0 },
    { 0x9b325a4d, SUC_lexicon_tags_12 },
    { 0xd2f25a53, SUC_lexicon_tags_69 },
    { 0x29b25a54, SUC_lexicon_tags_15 },
    { 0x84f25a54, SUC_lexicon_tags_22 },
    { 0x4d125a56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf325a5e, SUC_lexicon_tags_47 },
    { 0x15525a5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9f725a61, SUC_lexicon_tags_37 },
    { 0xdff25a62, SUC_lexicon_tags_17 },
    { 0xe8525a62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d25a66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f325a68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa6f25a6a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5725a70, SUC_lexicon_tags_434 },
    { 0, NULL },
    { 0x44925a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72325a74, SUC_lexicon_tags_0 },
    { 0xd3b25a75, SUC_lexicon_tags_25 },
    { 0x9db25a76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8725a78, SUC_lexicon_tags_25 },
    { 0x86125a79, SUC_lexicon_tags_0 },
    { 0x36d25a79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe125a80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x60b25a82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x68d25a84, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d725a90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xafd25a92, SUC_lexicon_tags_1 },
    { 0x4e525a92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1b25a95, SUC_lexicon_tags_60 },
    { 0xf9525a96, SUC_lexicon_tags_0 },
    { 0xaa725a97, SUC_lexicon_tags_45 },
    { 0xdbb25a97, SUC_lexicon_tags_11 },
    { 0x62725a97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84525a9b, SUC_lexicon_tags_38 },
    { 0x7c525a9c, SUC_lexicon_tags_65 },
    { 0x15125a9c, SUC_lexicon_tags_95 },
    { 0x7b725a9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a525aa0, SUC_lexicon_tags_12 },
    { 0x70725aa0, SUC_lexicon_tags_30 },
    { 0xd0b25aa2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f325aa4, SUC_lexicon_tags_49 },
    { 0xdb125aa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9925aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9125aae, SUC_lexicon_tags_25 },
    { 0xad525aaf, SUC_lexicon_tags_32 },
    { 0xf2725ab0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2525ab2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78125ab6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded25aba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d25abe, SUC_lexicon_tags_17 },
    { 0xfd325abe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb25ac4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x98125ac6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f25aca, SUC_lexicon_tags_2 },
    { 0x34b25aca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x37125ace, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb925ad3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b25ad7, SUC_lexicon_tags_23 },
    { 0xa6925ad8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2125add, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f525ae0, SUC_lexicon_tags_11 },
    { 0x32325ae1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc6525ae3, SUC_lexicon_tags_12 },
    { 0xcc525ae4, SUC_lexicon_tags_7 },
    { 0xfdf25ae4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8bb25ae7, SUC_lexicon_tags_60 },
    { 0xa0525ae8, SUC_lexicon_tags_55 },
    { 0x3f525ae8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f925aeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb525af1, SUC_lexicon_tags_0 },
    { 0x25325af2, SUC_lexicon_tags_17 },
    { 0xd1725af1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f25af5, SUC_lexicon_tags_11 },
    { 0x41725af6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xee925af8, SUC_lexicon_tags_27 },
    { 0xa5f25af9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb525afc, SUC_lexicon_tags_26 },
    { 0xa2525afd, SUC_lexicon_tags_27 },
    { 0xb1725afe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x76525b01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf25b05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1b25b07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d25b0b, SUC_lexicon_tags_12 },
    { 0x5a725b0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd25b10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5525b12, SUC_lexicon_tags_37 },
    { 0xa8525b13, SUC_lexicon_tags_68 },
    { 0x48d25b13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f725b16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3b25b18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3325b1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b25b20, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d25b23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68525b26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60325b2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f25b30, SUC_lexicon_tags_37 },
    { 0xfcb25b31, SUC_lexicon_tags_19 },
    { 0x25d25b32, SUC_lexicon_tags_15 },
    { 0x47325b33, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x30125b36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb25b3b, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd25b43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93325b45, SUC_lexicon_tags_47 },
    { 0xb7b25b46, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf8725b48, SUC_lexicon_tags_23 },
    { 0x91925b49, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbbd25b4b, SUC_lexicon_tags_2 },
    { 0xa5725b4c, SUC_lexicon_tags_14 },
    { 0x88f25b4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96125b51, SUC_lexicon_tags_11 },
    { 0xf8725b51, SUC_lexicon_tags_31 },
    { 0x2925b52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f25b5f, SUC_lexicon_tags_15 },
    { 0xa5b25b5f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd25b65, SUC_lexicon_tags_31 },
    { 0xb4f25b65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a925b6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87725b6c, SUC_lexicon_tags_45 },
    { 0x39525b6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8525b6f, SUC_lexicon_tags_8 },
    { 0xd9525b70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9d25b72, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce525b84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f25b87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7925b90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d25b95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed725b9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x23725b9d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17325ba4, SUC_lexicon_tags_12 },
    { 0x17325ba4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87325baa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x39d25bac, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x35525bae, SUC_lexicon_tags_11 },
    { 0x6df25bae, SUC_lexicon_tags_37 },
    { 0xb3b25bb0, SUC_lexicon_tags_23 },
    { 0xf1525bb0, SUC_lexicon_tags_15 },
    { 0x89525bb0, SUC_lexicon_tags_25 },
    { 0x72925bb3, SUC_lexicon_tags_17 },
    { 0xcc925bb0, SUC_lexicon_tags_12 },
    { 0x61f25bb5, SUC_lexicon_tags_27 },
    { 0x4c325bb0, SUC_lexicon_tags_48 },
    { 0x76725bb7, SUC_lexicon_tags_12 },
    { 0x5cb25bb4, SUC_lexicon_tags_47 },
    { 0x59925bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b25bbe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x43d25bc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2525bc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3325bc6, SUC_lexicon_tags_17 },
    { 0x99325bc7, SUC_lexicon_tags_13 },
    { 0xfbf25bc8, SUC_lexicon_tags_14 },
    { 0xf6b25bc7, SUC_lexicon_tags_17 },
    { 0xd3925bc9, SUC_lexicon_tags_1 },
    { 0xb25bc9, SUC_lexicon_tags_26 },
    { 0x50125bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b25bd0, SUC_lexicon_tags_49 },
    { 0x6bf25bd1, SUC_lexicon_tags_65 },
    { 0xe7725bd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa725bd5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db25bdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64725bdf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f25be4, SUC_lexicon_tags_21 },
    { 0xd0525be4, SUC_lexicon_tags_108 },
    { 0x39f25be5, SUC_lexicon_tags_264 },
    { 0x22925be4, SUC_lexicon_tags_0 },
    { 0x725be6, SUC_lexicon_tags_7 },
    { 0xaf325be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c125bed, SUC_lexicon_tags_25 },
    { 0x46f25bee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x97925bf0, SUC_lexicon_tags_11 },
    { 0x77f25bf1, SUC_lexicon_tags_77 },
    { 0x94925bf1, SUC_lexicon_tags_257 },
    { 0x37d25bf1, SUC_lexicon_tags_68 },
    { 0x3b25bf3, SUC_lexicon_tags_37 },
    { 0x18d25bf5, SUC_lexicon_tags_7 },
    { 0x44125bf2, SUC_lexicon_tags_29 },
    { 0x14125bf3, SUC_lexicon_tags_19 },
    { 0xaff25bf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefb25bfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28b25bfc, SUC_lexicon_tags_25 },
    { 0xf3525bfc, SUC_lexicon_tags_31 },
    { 0xaff25bfe, SUC_lexicon_tags_28 },
    { 0x2c925bfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f25c03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f25c0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b25c11, SUC_lexicon_tags_54 },
    { 0x3d725c11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b25c16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab25c19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x57b25c1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd925c1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9b525c21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4d25c23, SUC_lexicon_tags_17 },
    { 0xb9525c24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9725c2f, SUC_lexicon_tags_45 },
    { 0xffb25c2f, SUC_lexicon_tags_32 },
    { 0x92b25c31, SUC_lexicon_tags_12 },
    { 0x65525c32, SUC_lexicon_tags_27 },
    { 0xadf25c32, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x78f25c35, SUC_lexicon_tags_58 },
    { 0xc6b25c36, SUC_lexicon_tags_11 },
    { 0x16325c37, SUC_lexicon_tags_49 },
    { 0x25525c36, SUC_lexicon_tags_17 },
    { 0xf4725c39, SUC_lexicon_tags_29 },
    { 0xaa125c36, SUC_lexicon_tags_11 },
    { 0x7b725c35, SUC_lexicon_tags_7 },
    { 0xc2b25c3c, SUC_lexicon_tags_25 },
    { 0x22d25c3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f25c40, SUC_lexicon_tags_17 },
    { 0x59325c40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d25c45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53725c49, SUC_lexicon_tags_28 },
    { 0xb8525c49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62725c52, SUC_lexicon_tags_29 },
    { 0xb7725c53, SUC_lexicon_tags_68 },
    { 0x6dd25c54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d125c58, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb7d25c5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31125c5c, SUC_lexicon_tags_37 },
    { 0xecd25c5c, SUC_lexicon_tags_17 },
    { 0x89f25c5e, SUC_lexicon_tags_27 },
    { 0x8f525c5e, SUC_lexicon_tags_15 },
    { 0x28325c5c, SUC_lexicon_tags_17 },
    { 0xa9125c5f, SUC_lexicon_tags_35 },
    { 0x3ff25c62, SUC_lexicon_tags_12 },
    { 0x46f25c63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x32525c65, SUC_lexicon_tags_12 },
    { 0xba525c66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4325c68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef125c70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6925c77, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c125c7b, SUC_lexicon_tags_151 },
    { 0x40525c7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b925c7f, SUC_lexicon_tags_7 },
    { 0x88d25c80, SUC_lexicon_tags_37 },
    { 0x6ed25c80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2925c84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66125c89, SUC_lexicon_tags_21 },
    { 0x48325c89, SUC_lexicon_tags_7 },
    { 0x53b25c8a, SUC_lexicon_tags_49 },
    { 0x25125c8c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4ef25c8e, SUC_lexicon_tags_33 },
    { 0xb925c8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf25c98, SUC_lexicon_tags_17 },
    { 0xaeb25c98, SUC_lexicon_tags_17 },
    { 0xe4f25c99, SUC_lexicon_tags_29 },
    { 0x44725c98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69725c9e, SUC_lexicon_tags_11 },
    { 0xb3325c9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9f25ca1, SUC_lexicon_tags_0 },
    { 0x4ef25ca2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f25ca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f25cae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x62125cb0, SUC_lexicon_tags_56 },
    { 0x55925cb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6525cb4, SUC_lexicon_tags_1 },
    { 0xddd25cb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60b25cb7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b25cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf25cbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd525cc2, SUC_lexicon_tags_45 },
    { 0xd9f25cc3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b725cca, SUC_lexicon_tags_37 },
    { 0x41f25cca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba925cd0, SUC_lexicon_tags_43 },
    { 0x24525cd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f25cd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4125cdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62725ce2, SUC_lexicon_tags_49 },
    { 0xfc925ce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e525ce5, SUC_lexicon_tags_15 },
    { 0xf8325ce6, SUC_lexicon_tags_1 },
    { 0xe2d25ce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce725ceb, SUC_lexicon_tags_15 },
    { 0xb1725cec, SUC_lexicon_tags_17 },
    { 0x18b25ced, SUC_lexicon_tags_37 },
    { 0x5125ced, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3a725cf0, SUC_lexicon_tags_1 },
    { 0x2ed25cf1, SUC_lexicon_tags_7 },
    { 0xebf25cf0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe1f25cf4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x50d25cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd125cfa, SUC_lexicon_tags_37 },
    { 0x98525cfb, SUC_lexicon_tags_17 },
    { 0x5c325cfc, SUC_lexicon_tags_37 },
    { 0xe5525cfc, SUC_lexicon_tags_7 },
    { 0xde125cfc, SUC_lexicon_tags_63 },
    { 0x5dd25cfc, SUC_lexicon_tags_4 },
    { 0xcc925cfc, SUC_lexicon_tags_17 },
    { 0x78325cfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb25d04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff25d08, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x71125d0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12125d13, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x64125d15, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x67325d18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff25d1b, SUC_lexicon_tags_6 },
    { 0x2df25d1c, SUC_lexicon_tags_65 },
    { 0x8c325d1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7325d20, SUC_lexicon_tags_15 },
    { 0xa0925d21, SUC_lexicon_tags_37 },
    { 0x2b325d21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xac925d24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56325d2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d25d3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f125d42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37925d4a, SUC_lexicon_tags_13 },
    { 0x2cd25d4b, SUC_lexicon_tags_31 },
    { 0x23325d4b, SUC_lexicon_tags_7 },
    { 0x4b125d4b, SUC_lexicon_tags_7 },
    { 0xc2d25d4e, SUC_lexicon_tags_1 },
    { 0x79925d4f, SUC_lexicon_tags_37 },
    { 0xe325d4f, SUC_lexicon_tags_31 },
    { 0x40f25d51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30325d55, SUC_lexicon_tags_24 },
    { 0xf6725d56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2925d5a, SUC_lexicon_tags_15 },
    { 0xe1f25d5a, SUC_lexicon_tags_2 },
    { 0x93525d5c, SUC_lexicon_tags_19 },
    { 0x73725d5c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97525d63, SUC_lexicon_tags_11 },
    { 0x7bb25d64, SUC_lexicon_tags_21 },
    { 0xe8d25d65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab25d6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b25d72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7925d74, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b25d77, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b25d7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58525d7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55b25d7e, SUC_lexicon_tags_47 },
    { 0x6cd25d7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e725d81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8325d83, SUC_lexicon_tags_23 },
    { 0x7c725d84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdad25d86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8bf25d88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf725d8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4525d8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56725d8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc725d91, SUC_lexicon_tags_82 },
    { 0x21525d92, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31125d96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81125d9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c925d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f525da7, SUC_lexicon_tags_37 },
    { 0x6bd25da7, SUC_lexicon_tags_6 },
    { 0xef525da8, SUC_lexicon_tags_6 },
    { 0xb5d25daa, SUC_lexicon_tags_6 },
    { 0x84725da7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d25db2, SUC_lexicon_tags_17 },
    { 0xb6325db3, SUC_lexicon_tags_63 },
    { 0xaf25db4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b25dc1, SUC_lexicon_tags_11 },
    { 0x75f25dc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4925dc4, SUC_lexicon_tags_25 },
    { 0xd1f25dc4, SUC_lexicon_tags_2 },
    { 0x67d25dc6, SUC_lexicon_tags_14 },
    { 0xdb725dc7, SUC_lexicon_tags_2 },
    { 0xe4f25dc8, SUC_lexicon_tags_23 },
    { 0x8ef25dc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb525dce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d25dd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53f25dd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7dd25dd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0125ddc, SUC_lexicon_tags_7 },
    { 0xc6125ddc, SUC_lexicon_tags_23 },
    { 0x3d325dde, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb25de1, SUC_lexicon_tags_7 },
    { 0x15325de2, SUC_lexicon_tags_12 },
    { 0x66525de3, SUC_lexicon_tags_25 },
    { 0x1b125de3, SUC_lexicon_tags_12 },
    { 0xc5b25de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb925dea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd25def, SUC_lexicon_tags_17 },
    { 0xff125df0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd325df2, SUC_lexicon_tags_31 },
    { 0xd9f25df3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x82525df5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7325dfc, SUC_lexicon_tags_39 },
    { 0x1cf25dfd, SUC_lexicon_tags_11 },
    { 0x43125dfe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x14b25e00, SUC_lexicon_tags_37 },
    { 0xeaf25e01, SUC_lexicon_tags_25 },
    { 0x9df25e00, SUC_lexicon_tags_16 },
    { 0xf0b25e03, SUC_lexicon_tags_1 },
    { 0x7a125e02, SUC_lexicon_tags_15 },
    { 0x89925e05, SUC_lexicon_tags_45 },
    { 0xc0725e06, SUC_lexicon_tags_37 },
    { 0xcfb25e06, SUC_lexicon_tags_0 },
    { 0x97525e08, SUC_lexicon_tags_12 },
    { 0x58125e09, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe0725e0b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac925e12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6d25e14, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x60f25e16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3dd25e18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad25e1d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb525e20, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e325e27, SUC_lexicon_tags_4 },
    { 0xa5125e28, SUC_lexicon_tags_11 },
    { 0x72f25e28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x67f25e2b, SUC_lexicon_tags_17 },
    { 0x61125e2c, SUC_lexicon_tags_2 },
    { 0x15925e2b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51125e33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9925e36, SUC_lexicon_tags_1 },
    { 0xfff25e36, SUC_lexicon_tags_2 },
    { 0x72d25e38, SUC_lexicon_tags_55 },
    { 0x27f25e39, SUC_lexicon_tags_34 },
    { 0xf4d25e37, SUC_lexicon_tags_25 },
    { 0x2b25e3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37125e43, SUC_lexicon_tags_60 },
    { 0x5a925e44, SUC_lexicon_tags_12 },
    { 0x38125e44, SUC_lexicon_tags_29 },
    { 0x2d325e44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2cd25e48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48925e4a, SUC_lexicon_tags_10 },
    { 0x5b325e4a, SUC_lexicon_tags_48 },
    { 0x31f25e4c, SUC_lexicon_tags_75 },
    { 0x90f25e4d, SUC_lexicon_tags_2 },
    { 0x75f25e4b, SUC_lexicon_tags_27 },
    { 0x48525e4f, SUC_lexicon_tags_12 },
    { 0xbdb25e4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78525e54, SUC_lexicon_tags_505 },
    { 0xa1525e54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9125e59, SUC_lexicon_tags_11 },
    { 0x2725e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6525e5e, SUC_lexicon_tags_63 },
    { 0xfa525e5e, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded25e75, SUC_lexicon_tags_17 },
    { 0xc7b25e76, SUC_lexicon_tags_19 },
    { 0xacb25e76, SUC_lexicon_tags_127 },
    { 0xe1925e78, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2125e7b, SUC_lexicon_tags_1 },
    { 0x57f25e7b, SUC_lexicon_tags_43 },
    { 0x3df25e7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47325e86, SUC_lexicon_tags_26 },
    { 0x8db25e87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73d25e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58125e8b, SUC_lexicon_tags_107 },
    { 0x3c925e8b, SUC_lexicon_tags_17 },
    { 0xb4125e8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefd25e8f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14125e95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45925e98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7525e9b, SUC_lexicon_tags_7 },
    { 0x45d25e9c, SUC_lexicon_tags_25 },
    { 0x6c725e9d, SUC_lexicon_tags_1094 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a725ea0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb725ea4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3925ea7, SUC_lexicon_tags_5 },
    { 0x89f25ea8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d25eab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68f25ead, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x38525eb0, SUC_lexicon_tags_23 },
    { 0xe4b25eb1, SUC_lexicon_tags_12 },
    { 0xb8f25eb2, SUC_lexicon_tags_6 },
    { 0x46325eb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b25eb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdb25eb9, SUC_lexicon_tags_40 },
    { 0xc725eba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab25ebe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff125ec0, SUC_lexicon_tags_11 },
    { 0x4525ec1, SUC_lexicon_tags_0 },
    { 0x80325ec1, SUC_lexicon_tags_45 },
    { 0xe7f25ec3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x91f25ec5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb125eca, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b25ed1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbf25ed3, SUC_lexicon_tags_46 },
    { 0x19d25ed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ef25ed6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf25edc, SUC_lexicon_tags_32 },
    { 0xddd25edd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78125ee1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37525eed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6125eef, SUC_lexicon_tags_28 },
    { 0x6b925ef0, SUC_lexicon_tags_45 },
    { 0xa6b25eef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83925ef3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56925ef5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b25ef9, SUC_lexicon_tags_25 },
    { 0x59f25efa, SUC_lexicon_tags_12 },
    { 0xa3325efb, SUC_lexicon_tags_48 },
    { 0x6125efb, SUC_lexicon_tags_37 },
    { 0xb525efb, SUC_lexicon_tags_3 },
    { 0x9fb25efe, SUC_lexicon_tags_11 },
    { 0xec525efc, SUC_lexicon_tags_7 },
    { 0xefd25efe, SUC_lexicon_tags_15 },
    { 0x2ad25f00, SUC_lexicon_tags_19 },
    { 0xa9f25ef9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63525f04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2525f08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38b25f0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55125f0e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f25f12, SUC_lexicon_tags_7 },
    { 0xaa125f13, SUC_lexicon_tags_28 },
    { 0x63b25f12, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2325f1a, SUC_lexicon_tags_9 },
    { 0xa2525f1b, SUC_lexicon_tags_31 },
    { 0x6fb25f1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33325f1f, SUC_lexicon_tags_3 },
    { 0x42f25f20, SUC_lexicon_tags_170 },
    { 0xfc125f21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33125f24, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x17f25f26, SUC_lexicon_tags_43 },
    { 0xfff25f27, SUC_lexicon_tags_1 },
    { 0xc7725f28, SUC_lexicon_tags_69 },
    { 0x22325f29, SUC_lexicon_tags_7 },
    { 0xa1b25f2a, SUC_lexicon_tags_935 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff25f30, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x25525f33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25725f37, SUC_lexicon_tags_1 },
    { 0x65b25f38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88d25f3a, SUC_lexicon_tags_25 },
    { 0x33b25f3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7525f3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed125f40, SUC_lexicon_tags_29 },
    { 0xd6f25f41, SUC_lexicon_tags_25 },
    { 0xe8525f41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99525f45, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf25f55, SUC_lexicon_tags_29 },
    { 0xe4325f56, SUC_lexicon_tags_21 },
    { 0x4b725f57, SUC_lexicon_tags_45 },
    { 0xc725f57, SUC_lexicon_tags_33 },
    { 0xa8b25f59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61725f5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55125f65, SUC_lexicon_tags_5 },
    { 0xea325f66, SUC_lexicon_tags_14 },
    { 0x12b25f67, SUC_lexicon_tags_423 },
    { 0x11525f67, SUC_lexicon_tags_3 },
    { 0xef725f69, SUC_lexicon_tags_68 },
    { 0x96725f6a, SUC_lexicon_tags_7 },
    { 0xf3725f65, SUC_lexicon_tags_12 },
    { 0x39d25f6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ff25f6e, SUC_lexicon_tags_12 },
    { 0x3e525f6f, SUC_lexicon_tags_2 },
    { 0x48f25f6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81525f74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27525f79, SUC_lexicon_tags_15 },
    { 0x5fd25f7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9e325f7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9ff25f7e, SUC_lexicon_tags_25 },
    { 0x86b25f7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8725f86, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2725f8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf25f8e, SUC_lexicon_tags_47 },
    { 0xfa325f8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b125f95, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b25f9a, SUC_lexicon_tags_2 },
    { 0xf4325f9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9325fa2, SUC_lexicon_tags_25 },
    { 0xdab25fa2, SUC_lexicon_tags_127 },
    { 0xa8525fa4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d25fab, SUC_lexicon_tags_47 },
    { 0x6f725fab, SUC_lexicon_tags_11 },
    { 0x5d525fad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25525fb3, SUC_lexicon_tags_8 },
    { 0xb5525fb4, SUC_lexicon_tags_12 },
    { 0xf3925fb5, SUC_lexicon_tags_6 },
    { 0xeab25fb6, SUC_lexicon_tags_11 },
    { 0x40325fb7, SUC_lexicon_tags_7 },
    { 0x8f25fb8, SUC_lexicon_tags_7 },
    { 0x84b25fb9, SUC_lexicon_tags_26 },
    { 0x99925fba, SUC_lexicon_tags_19 },
    { 0xa0d25fbb, SUC_lexicon_tags_8 },
    { 0xd2325fba, SUC_lexicon_tags_148 },
    { 0xf1925fbd, SUC_lexicon_tags_39 },
    { 0x37725fbb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb7b25fc0, SUC_lexicon_tags_3 },
    { 0xe9d25fc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b25fc4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f25fc8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc0525fca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e525fcc, SUC_lexicon_tags_37 },
    { 0x58d25fcc, SUC_lexicon_tags_17 },
    { 0x33f25fcc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f25fd1, SUC_lexicon_tags_1 },
    { 0x1c725fd2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x30f25fd4, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xfb125fd6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc6325fd8, SUC_lexicon_tags_12 },
    { 0x56125fd9, SUC_lexicon_tags_6 },
    { 0xb8725fda, SUC_lexicon_tags_63 },
    { 0x8b525fd9, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x71325fdd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1125fe0, SUC_lexicon_tags_0 },
    { 0xbf525fe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba525fe5, SUC_lexicon_tags_28 },
    { 0x3c325fe6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4325fec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xba525fef, SUC_lexicon_tags_65 },
    { 0x4f125ff0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3bb25ff2, SUC_lexicon_tags_23 },
    { 0x14f25ff3, SUC_lexicon_tags_12 },
    { 0x74d25ff4, SUC_lexicon_tags_0 },
    { 0x5d125ff2, SUC_lexicon_tags_27 },
    { 0x84325ff4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c525ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5325ffd, SUC_lexicon_tags_23 },
    { 0x49925ffe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd926002, SUC_lexicon_tags_30 },
    { 0x28d26003, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c726005, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd2600c, SUC_lexicon_tags_43 },
    { 0xc792600d, SUC_lexicon_tags_17 },
    { 0x2692600d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d26011, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d26014, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42926016, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f26019, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f2601e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42526022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef26025, SUC_lexicon_tags_2 },
    { 0xc6126026, SUC_lexicon_tags_12 },
    { 0xf4726026, SUC_lexicon_tags_8 },
    { 0x81126028, SUC_lexicon_tags_17 },
    { 0x48726025, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3f32602b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee72602d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5192602f, SUC_lexicon_tags_61 },
    { 0x80b26030, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af26034, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d526037, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa132603a, SUC_lexicon_tags_15 },
    { 0xb972603a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d26040, SUC_lexicon_tags_7 },
    { 0xe0d26041, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce126049, SUC_lexicon_tags_30 },
    { 0xdc92604a, SUC_lexicon_tags_2 },
    { 0xe192604b, SUC_lexicon_tags_37 },
    { 0x8ff2604c, SUC_lexicon_tags_17 },
    { 0xe052604d, SUC_lexicon_tags_9 },
    { 0xc912604c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbf326050, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3526058, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa472605c, SUC_lexicon_tags_37 },
    { 0xe1f2605d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92126062, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc9926064, SUC_lexicon_tags_0 },
    { 0x7d726065, SUC_lexicon_tags_25 },
    { 0xab926066, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe152606a, SUC_lexicon_tags_37 },
    { 0x5792606a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe526073, SUC_lexicon_tags_3 },
    { 0xa2326074, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b26078, SUC_lexicon_tags_37 },
    { 0x14b26079, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba32607b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3726082, SUC_lexicon_tags_2 },
    { 0x6c926082, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fd26085, SUC_lexicon_tags_37 },
    { 0x77526086, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e326088, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2ed2608a, SUC_lexicon_tags_37 },
    { 0xa972608b, SUC_lexicon_tags_29 },
    { 0x4632608b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab26090, SUC_lexicon_tags_37 },
    { 0x1b926090, SUC_lexicon_tags_31 },
    { 0xb3d26090, SUC_lexicon_tags_37 },
    { 0x4fd26093, SUC_lexicon_tags_29 },
    { 0x70f26094, SUC_lexicon_tags_17 },
    { 0x64d26093, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f26098, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9992609c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53260a4, SUC_lexicon_tags_12 },
    { 0x2f9260a4, SUC_lexicon_tags_15 },
    { 0x327260a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3260a9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35260ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff260af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x823260b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399260b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x597260b8, SUC_lexicon_tags_17 },
    { 0xc31260b9, SUC_lexicon_tags_31 },
    { 0x88d260b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91260bc, SUC_lexicon_tags_0 },
    { 0xe23260bd, SUC_lexicon_tags_7 },
    { 0x367260be, SUC_lexicon_tags_23 },
    { 0x5c7260bf, SUC_lexicon_tags_25 },
    { 0x7ed260c0, SUC_lexicon_tags_14 },
    { 0x995260c1, SUC_lexicon_tags_198 },
    { 0x87260c0, SUC_lexicon_tags_32 },
    { 0x589260c3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65260c6, SUC_lexicon_tags_29 },
    { 0xc4d260c6, SUC_lexicon_tags_60 },
    { 0xfb7260c8, SUC_lexicon_tags_37 },
    { 0x521260c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183260cd, SUC_lexicon_tags_2 },
    { 0x3c7260cd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x79d260d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x487260d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93260d6, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x8a5260d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b260de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4d260e0, SUC_lexicon_tags_29 },
    { 0xf0f260e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d260e5, SUC_lexicon_tags_2 },
    { 0x633260e5, SUC_lexicon_tags_3 },
    { 0xfb7260e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787260eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf260ee, SUC_lexicon_tags_15 },
    { 0xd3d260ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x155260f1, SUC_lexicon_tags_17 },
    { 0x4c9260f1, SUC_lexicon_tags_37 },
    { 0x697260f2, SUC_lexicon_tags_12 },
    { 0xc1f260f4, SUC_lexicon_tags_14 },
    { 0x8ed260f5, SUC_lexicon_tags_30 },
    { 0x4eb260f6, SUC_lexicon_tags_7 },
    { 0xa7260f7, SUC_lexicon_tags_33 },
    { 0x751260f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5260fd, SUC_lexicon_tags_9 },
    { 0x86b260fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e926100, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f726104, SUC_lexicon_tags_7 },
    { 0x8ef26105, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f26107, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc352610a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb26111, SUC_lexicon_tags_38 },
    { 0xb526112, SUC_lexicon_tags_7 },
    { 0x58b26112, SUC_lexicon_tags_21 },
    { 0x17926113, SUC_lexicon_tags_7 },
    { 0x90126112, SUC_lexicon_tags_7 },
    { 0xe8b26116, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8326119, SUC_lexicon_tags_1 },
    { 0x2892611a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e526121, SUC_lexicon_tags_47 },
    { 0x1a126122, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b26125, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a12612a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d52612e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf26136, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3926143, SUC_lexicon_tags_15 },
    { 0x8c126144, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8f26146, SUC_lexicon_tags_1 },
    { 0xfd726147, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e92614b, SUC_lexicon_tags_43 },
    { 0x3432614c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2052614e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x73b26150, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40526153, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1326158, SUC_lexicon_tags_65 },
    { 0xda726159, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f2615d, SUC_lexicon_tags_66 },
    { 0x1952615e, SUC_lexicon_tags_27 },
    { 0x1032615e, SUC_lexicon_tags_15 },
    { 0xc072615e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5726162, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10126166, SUC_lexicon_tags_2 },
    { 0x40526167, SUC_lexicon_tags_55 },
    { 0xcd326166, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa092616d, SUC_lexicon_tags_46 },
    { 0x25d2616e, SUC_lexicon_tags_46 },
    { 0xaa52616e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91726175, SUC_lexicon_tags_2 },
    { 0x55b26176, SUC_lexicon_tags_37 },
    { 0xe7726175, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf726179, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4772617d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcad2617f, SUC_lexicon_tags_0 },
    { 0xdb52617f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf26183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab926185, SUC_lexicon_tags_11 },
    { 0x61726186, SUC_lexicon_tags_10 },
    { 0xa8b26187, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf512618a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecf2618c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xef12618f, SUC_lexicon_tags_12 },
    { 0x19126190, SUC_lexicon_tags_17 },
    { 0xbff26190, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1726195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88726198, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9052619e, SUC_lexicon_tags_49 },
    { 0x7db2619f, SUC_lexicon_tags_33 },
    { 0x34d261a0, SUC_lexicon_tags_27 },
    { 0xefb261a1, SUC_lexicon_tags_23 },
    { 0x4b5261a0, SUC_lexicon_tags_11 },
    { 0x7e3261a3, SUC_lexicon_tags_15 },
    { 0x1ad261a4, SUC_lexicon_tags_0 },
    { 0xd1b261a4, SUC_lexicon_tags_3 },
    { 0xe55261a3, SUC_lexicon_tags_29 },
    { 0x73f261a7, SUC_lexicon_tags_11 },
    { 0xb93261a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x10f261aa, SUC_lexicon_tags_37 },
    { 0x135261aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x549261ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x295261b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f261b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a1261b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173261b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf6d261bb, SUC_lexicon_tags_411 },
    { 0, NULL },
    { 0x2fd261bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f261c1, SUC_lexicon_tags_15 },
    { 0xed9261c1, SUC_lexicon_tags_2 },
    { 0x20b261c1, SUC_lexicon_tags_2 },
    { 0xe71261c4, SUC_lexicon_tags_15 },
    { 0x267261c2, SUC_lexicon_tags_33 },
    { 0xcc9261c1, SUC_lexicon_tags_25 },
    { 0xba7261c7, SUC_lexicon_tags_63 },
    { 0x6db261c2, SUC_lexicon_tags_17 },
    { 0x819261c9, SUC_lexicon_tags_5 },
    { 0x23f261c2, SUC_lexicon_tags_17 },
    { 0xd41261c4, SUC_lexicon_tags_63 },
    { 0x317261cc, SUC_lexicon_tags_27 },
    { 0x381261cc, SUC_lexicon_tags_8 },
    { 0x757261c4, SUC_lexicon_tags_17 },
    { 0xf53261cf, SUC_lexicon_tags_11 },
    { 0x805261d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5261d5, SUC_lexicon_tags_37 },
    { 0x4ff261d6, SUC_lexicon_tags_2 },
    { 0x789261d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x437261d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d261e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5261eb, SUC_lexicon_tags_0 },
    { 0x749261ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca9261ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaf261f0, SUC_lexicon_tags_14 },
    { 0x601261f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa79261f3, SUC_lexicon_tags_2 },
    { 0x4ab261f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25261f8, SUC_lexicon_tags_15 },
    { 0x263261f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbdd261fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7261ff, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b26207, SUC_lexicon_tags_43 },
    { 0x3e526208, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20f2620a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab26216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b2621b, SUC_lexicon_tags_17 },
    { 0xb412621c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98526222, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29126224, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff2622a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc872622c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92326234, SUC_lexicon_tags_1 },
    { 0x77326235, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd726238, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x872623a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b2623f, SUC_lexicon_tags_7 },
    { 0xdd726240, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb126245, SUC_lexicon_tags_15 },
    { 0x69926246, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd2624b, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xd452624d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42726251, SUC_lexicon_tags_1 },
    { 0xdfb26251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d26257, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e726265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6172626a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab26270, SUC_lexicon_tags_43 },
    { 0x45926271, SUC_lexicon_tags_7 },
    { 0xb1126272, SUC_lexicon_tags_12 },
    { 0x2bb26273, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18526276, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8112627c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6772627f, SUC_lexicon_tags_32 },
    { 0xe312627f, SUC_lexicon_tags_12 },
    { 0x80926280, SUC_lexicon_tags_27 },
    { 0xa4b26280, SUC_lexicon_tags_27 },
    { 0xc3b26281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57926287, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa3926289, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf672628d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe4b2628f, SUC_lexicon_tags_47 },
    { 0x2f72628f, SUC_lexicon_tags_37 },
    { 0x16f26291, SUC_lexicon_tags_29 },
    { 0x4f326291, SUC_lexicon_tags_37 },
    { 0xa126290, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f2629d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7262a6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1262ab, SUC_lexicon_tags_16 },
    { 0xb1f262ac, SUC_lexicon_tags_1 },
    { 0x477262ab, SUC_lexicon_tags_37 },
    { 0xe2d262ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c1262b0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf262ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b262bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03262c1, SUC_lexicon_tags_15 },
    { 0xcf3262c2, SUC_lexicon_tags_2 },
    { 0x5f5262c3, SUC_lexicon_tags_45 },
    { 0x595262c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83f262c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x163262c8, SUC_lexicon_tags_14 },
    { 0x9f5262c9, SUC_lexicon_tags_17 },
    { 0xdd1262c9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x93262cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b262cf, SUC_lexicon_tags_23 },
    { 0xf79262d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83262d8, SUC_lexicon_tags_17 },
    { 0x169262d9, SUC_lexicon_tags_7 },
    { 0x523262da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe55262dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9c7262de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87262e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf262e3, SUC_lexicon_tags_16 },
    { 0xe65262e4, SUC_lexicon_tags_87 },
    { 0x1c7262e5, SUC_lexicon_tags_9 },
    { 0x8f3262e6, SUC_lexicon_tags_8 },
    { 0x801262e6, SUC_lexicon_tags_45 },
    { 0x139262e8, SUC_lexicon_tags_26 },
    { 0x4c9262e6, SUC_lexicon_tags_16 },
    { 0x791262e9, SUC_lexicon_tags_37 },
    { 0x4ef262eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x105262ed, SUC_lexicon_tags_19 },
    { 0xc59262ee, SUC_lexicon_tags_47 },
    { 0x2bb262ef, SUC_lexicon_tags_37 },
    { 0x823262f0, SUC_lexicon_tags_27 },
    { 0x761262f1, SUC_lexicon_tags_14 },
    { 0x613262ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d262f7, SUC_lexicon_tags_4 },
    { 0x5a9262f8, SUC_lexicon_tags_25 },
    { 0xb81262f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5d1262fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb26302, SUC_lexicon_tags_16 },
    { 0x2cb26303, SUC_lexicon_tags_7 },
    { 0x90b26303, SUC_lexicon_tags_25 },
    { 0xe5d26305, SUC_lexicon_tags_11 },
    { 0x85126306, SUC_lexicon_tags_25 },
    { 0x6a326305, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x13526309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4892630b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f2630e, SUC_lexicon_tags_59 },
    { 0x2bf2630f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18726311, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83726318, SUC_lexicon_tags_25 },
    { 0x28126319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x552631c, SUC_lexicon_tags_25 },
    { 0xd232631d, SUC_lexicon_tags_25 },
    { 0x6432631c, SUC_lexicon_tags_11 },
    { 0x3292631f, SUC_lexicon_tags_15 },
    { 0xf8726320, SUC_lexicon_tags_2 },
    { 0xf1126321, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc0d26323, SUC_lexicon_tags_401 },
    { 0, NULL },
    { 0x1ed26325, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b2632a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b26330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a126335, SUC_lexicon_tags_23 },
    { 0x5cf26336, SUC_lexicon_tags_37 },
    { 0x9126336, SUC_lexicon_tags_6 },
    { 0xe3b26338, SUC_lexicon_tags_55 },
    { 0x9db26336, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x792633b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e126340, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb3f26342, SUC_lexicon_tags_31 },
    { 0xd6526343, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad26346, SUC_lexicon_tags_7 },
    { 0x23726346, SUC_lexicon_tags_11 },
    { 0x8cb26348, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7832634b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1032634d, SUC_lexicon_tags_15 },
    { 0x1b12634e, SUC_lexicon_tags_1 },
    { 0xb092634f, SUC_lexicon_tags_28 },
    { 0x6612634f, SUC_lexicon_tags_11 },
    { 0xf852634e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79526360, SUC_lexicon_tags_2 },
    { 0xdc126361, SUC_lexicon_tags_25 },
    { 0xfd926362, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a326367, SUC_lexicon_tags_9 },
    { 0x45926367, SUC_lexicon_tags_63 },
    { 0x1d126368, SUC_lexicon_tags_63 },
    { 0x5a926369, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b72636c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd26370, SUC_lexicon_tags_12 },
    { 0xa1326370, SUC_lexicon_tags_23 },
    { 0xf9526372, SUC_lexicon_tags_38 },
    { 0xb26370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c726376, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f526379, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c526380, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbd326382, SUC_lexicon_tags_49 },
    { 0x25926382, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad2638a, SUC_lexicon_tags_25 },
    { 0x3eb2638a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a32638e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa1f26390, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5526392, SUC_lexicon_tags_1 },
    { 0x26d26393, SUC_lexicon_tags_45 },
    { 0x67f26394, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5526397, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1d26399, SUC_lexicon_tags_35 },
    { 0xf7f26399, SUC_lexicon_tags_46 },
    { 0x232639b, SUC_lexicon_tags_37 },
    { 0xb5d2639b, SUC_lexicon_tags_84 },
    { 0xd772639d, SUC_lexicon_tags_29 },
    { 0x1432639d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d263a1, SUC_lexicon_tags_49 },
    { 0x283263a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b263a9, SUC_lexicon_tags_2 },
    { 0xe3b263aa, SUC_lexicon_tags_7 },
    { 0x4c3263ab, SUC_lexicon_tags_17 },
    { 0x273263ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ad263ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x757263b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3263b5, SUC_lexicon_tags_0 },
    { 0xced263b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35263b9, SUC_lexicon_tags_17 },
    { 0xbd7263ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x647263bc, SUC_lexicon_tags_11 },
    { 0x471263bd, SUC_lexicon_tags_15 },
    { 0x18f263bd, SUC_lexicon_tags_11 },
    { 0x449263bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95263c2, SUC_lexicon_tags_25 },
    { 0x85d263c3, SUC_lexicon_tags_17 },
    { 0x1a1263c4, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d263c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853263cb, SUC_lexicon_tags_23 },
    { 0xa6f263cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7263cf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x559263d2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd263d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x69d263d9, SUC_lexicon_tags_25 },
    { 0x997263da, SUC_lexicon_tags_26 },
    { 0x901263da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11263de, SUC_lexicon_tags_11 },
    { 0x8f9263df, SUC_lexicon_tags_10 },
    { 0xcb9263df, SUC_lexicon_tags_11 },
    { 0x873263e1, SUC_lexicon_tags_15 },
    { 0x9cd263e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x589263e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73263e7, SUC_lexicon_tags_25 },
    { 0x21f263e8, SUC_lexicon_tags_68 },
    { 0xc3d263e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ab263eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf263ee, SUC_lexicon_tags_1 },
    { 0xe0f263ef, SUC_lexicon_tags_12 },
    { 0x20d263ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc47263f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6bd263f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9263fc, SUC_lexicon_tags_25 },
    { 0xe17263fc, SUC_lexicon_tags_12 },
    { 0x851263fd, SUC_lexicon_tags_55 },
    { 0x1e9263ff, SUC_lexicon_tags_88 },
    { 0x26b26400, SUC_lexicon_tags_30 },
    { 0x1b326401, SUC_lexicon_tags_25 },
    { 0xbf526402, SUC_lexicon_tags_14 },
    { 0x849263fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb26408, SUC_lexicon_tags_66 },
    { 0x9f526409, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4512640c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe532640e, SUC_lexicon_tags_118 },
    { 0xe572640f, SUC_lexicon_tags_2 },
    { 0xa512640e, SUC_lexicon_tags_10 },
    { 0xee12640e, SUC_lexicon_tags_7 },
    { 0x20126412, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7726414, SUC_lexicon_tags_14 },
    { 0x1926414, SUC_lexicon_tags_91 },
    { 0x4726415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9532641b, SUC_lexicon_tags_63 },
    { 0xeb72641c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd012641e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87326424, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7092642a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x392642c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f12642e, SUC_lexicon_tags_92 },
    { 0xf972642f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88926432, SUC_lexicon_tags_38 },
    { 0x19926432, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb52643a, SUC_lexicon_tags_11 },
    { 0x17d2643a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa972643d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x792643f, SUC_lexicon_tags_8 },
    { 0xc4d2643f, SUC_lexicon_tags_2 },
    { 0xcaf26441, SUC_lexicon_tags_37 },
    { 0x3372643f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c526446, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f2644d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91126450, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0x15926452, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10526456, SUC_lexicon_tags_63 },
    { 0x19926456, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd272645b, SUC_lexicon_tags_1 },
    { 0xa7f2645b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc326460, SUC_lexicon_tags_7 },
    { 0x9f926461, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc526463, SUC_lexicon_tags_57 },
    { 0xa5f26464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb526466, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc726468, SUC_lexicon_tags_39 },
    { 0xac726468, SUC_lexicon_tags_26 },
    { 0xa812646a, SUC_lexicon_tags_17 },
    { 0x4a726469, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b2646e, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x76126471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f726479, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xead2647b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b2647f, SUC_lexicon_tags_37 },
    { 0x4cf26480, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce726486, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd2648b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd672648d, SUC_lexicon_tags_27 },
    { 0x7f32648d, SUC_lexicon_tags_47 },
    { 0x7232648f, SUC_lexicon_tags_4 },
    { 0x7e92648f, SUC_lexicon_tags_31 },
    { 0x71d26491, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd126494, SUC_lexicon_tags_15 },
    { 0x79726495, SUC_lexicon_tags_6 },
    { 0xdcf26496, SUC_lexicon_tags_40 },
    { 0x63b26497, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x58126499, SUC_lexicon_tags_11 },
    { 0x41b26499, SUC_lexicon_tags_0 },
    { 0x8712649b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6652649f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1264a6, SUC_lexicon_tags_28 },
    { 0xcb1264a6, SUC_lexicon_tags_45 },
    { 0x89f264a8, SUC_lexicon_tags_15 },
    { 0x733264a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a9264ab, SUC_lexicon_tags_55 },
    { 0x935264ac, SUC_lexicon_tags_27 },
    { 0x7cd264ad, SUC_lexicon_tags_59 },
    { 0x87d264ae, SUC_lexicon_tags_39 },
    { 0x96f264ad, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x231264b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfc7264b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f264b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabb264b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7264bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x133264be, SUC_lexicon_tags_39 },
    { 0x6cb264be, SUC_lexicon_tags_15 },
    { 0xb11264c0, SUC_lexicon_tags_351 },
    { 0xa61264c1, SUC_lexicon_tags_31 },
    { 0xbe3264c0, SUC_lexicon_tags_1195 },
    { 0xbc1264c2, SUC_lexicon_tags_19 },
    { 0xf59264c3, SUC_lexicon_tags_6 },
    { 0xa55264c5, SUC_lexicon_tags_15 },
    { 0xd63264c6, SUC_lexicon_tags_8 },
    { 0xc73264c7, SUC_lexicon_tags_30 },
    { 0x811264c8, SUC_lexicon_tags_8 },
    { 0x5af264c9, SUC_lexicon_tags_77 },
    { 0x9df264c0, SUC_lexicon_tags_12 },
    { 0x583264c3, SUC_lexicon_tags_12 },
    { 0xc65264c1, SUC_lexicon_tags_65 },
    { 0x175264c0, SUC_lexicon_tags_15 },
    { 0x77d264ce, SUC_lexicon_tags_25 },
    { 0xb71264c4, SUC_lexicon_tags_12 },
    { 0xd11264d0, SUC_lexicon_tags_37 },
    { 0x2ed264d0, SUC_lexicon_tags_12 },
    { 0xbf3264d2, SUC_lexicon_tags_25 },
    { 0x7b9264d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x251264d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb2f264d7, SUC_lexicon_tags_29 },
    { 0x497264d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23264dc, SUC_lexicon_tags_14 },
    { 0x585264dd, SUC_lexicon_tags_91 },
    { 0xbc5264de, SUC_lexicon_tags_8 },
    { 0xa09264dc, SUC_lexicon_tags_30 },
    { 0x31f264de, SUC_lexicon_tags_19 },
    { 0xc11264e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3264e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf97264e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac1264eb, SUC_lexicon_tags_17 },
    { 0xd15264eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9264ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b264f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdb264f5, SUC_lexicon_tags_29 },
    { 0x9ed264f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x381264f9, SUC_lexicon_tags_0 },
    { 0xe35264f9, SUC_lexicon_tags_11 },
    { 0x33d264fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd97264fd, SUC_lexicon_tags_9 },
    { 0x98d264fe, SUC_lexicon_tags_12 },
    { 0x515264ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e326503, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d26508, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f2650b, SUC_lexicon_tags_37 },
    { 0x3e72650b, SUC_lexicon_tags_17 },
    { 0xa012650b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b26515, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb526519, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdef2651b, SUC_lexicon_tags_2 },
    { 0x8f12651c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xced2651e, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x70d26520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb26524, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c92652b, SUC_lexicon_tags_30 },
    { 0xf0f2652c, SUC_lexicon_tags_45 },
    { 0x58d2652c, SUC_lexicon_tags_19 },
    { 0x28d2652c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3326531, SUC_lexicon_tags_11 },
    { 0x8e326532, SUC_lexicon_tags_23 },
    { 0x51126532, SUC_lexicon_tags_11 },
    { 0x71726534, SUC_lexicon_tags_21 },
    { 0x6d726532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36726543, SUC_lexicon_tags_40 },
    { 0x73526544, SUC_lexicon_tags_7 },
    { 0x6b126545, SUC_lexicon_tags_7 },
    { 0x55726546, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d26549, SUC_lexicon_tags_49 },
    { 0x2012654a, SUC_lexicon_tags_3 },
    { 0x27d2654a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5726553, SUC_lexicon_tags_9 },
    { 0x53526554, SUC_lexicon_tags_69 },
    { 0x44b26555, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f2655a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7732655c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e926560, SUC_lexicon_tags_12 },
    { 0xe6126560, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb26568, SUC_lexicon_tags_25 },
    { 0xa0b26568, SUC_lexicon_tags_15 },
    { 0x24526569, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7526570, SUC_lexicon_tags_1 },
    { 0x52b26571, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db26575, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc726577, SUC_lexicon_tags_29 },
    { 0x2c126577, SUC_lexicon_tags_15 },
    { 0xec726579, SUC_lexicon_tags_14 },
    { 0xce92657a, SUC_lexicon_tags_10 },
    { 0xa0326579, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd852657e, SUC_lexicon_tags_25 },
    { 0x3fd2657f, SUC_lexicon_tags_33 },
    { 0xff526580, SUC_lexicon_tags_12 },
    { 0x99d26581, SUC_lexicon_tags_15 },
    { 0xc3d2657e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xed926584, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x65126586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6326589, SUC_lexicon_tags_7 },
    { 0x4012658a, SUC_lexicon_tags_60 },
    { 0x5db26589, SUC_lexicon_tags_27 },
    { 0x8af2658c, SUC_lexicon_tags_26 },
    { 0xe7f2658d, SUC_lexicon_tags_1 },
    { 0x4dd2658b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc726590, SUC_lexicon_tags_1 },
    { 0x48326590, SUC_lexicon_tags_1 },
    { 0x85f26592, SUC_lexicon_tags_30 },
    { 0xdd26593, SUC_lexicon_tags_45 },
    { 0xf4726594, SUC_lexicon_tags_49 },
    { 0xf0b26595, SUC_lexicon_tags_49 },
    { 0x26d26596, SUC_lexicon_tags_2 },
    { 0xda526595, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc772659f, SUC_lexicon_tags_11 },
    { 0x871265a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x359265a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x67f265a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9265ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f265b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79265b3, SUC_lexicon_tags_15 },
    { 0x63d265b4, SUC_lexicon_tags_29 },
    { 0x179265b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33f265b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0b265b9, SUC_lexicon_tags_2 },
    { 0x421265ba, SUC_lexicon_tags_1 },
    { 0xd7265ba, SUC_lexicon_tags_25 },
    { 0x101265bc, SUC_lexicon_tags_17 },
    { 0xfb9265b9, SUC_lexicon_tags_2 },
    { 0xbdf265be, SUC_lexicon_tags_29 },
    { 0xb33265be, SUC_lexicon_tags_37 },
    { 0x397265bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f1265c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b265c5, SUC_lexicon_tags_39 },
    { 0xc1b265c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845265ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf63265cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d265d0, SUC_lexicon_tags_9 },
    { 0xd55265d1, SUC_lexicon_tags_19 },
    { 0xed7265d2, SUC_lexicon_tags_37 },
    { 0x545265d2, SUC_lexicon_tags_37 },
    { 0xd3d265d4, SUC_lexicon_tags_63 },
    { 0x75f265d3, SUC_lexicon_tags_95 },
    { 0x643265d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb265d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3265dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5265e5, SUC_lexicon_tags_12 },
    { 0xb5b265e6, SUC_lexicon_tags_9 },
    { 0x27f265e7, SUC_lexicon_tags_17 },
    { 0xfdd265e8, SUC_lexicon_tags_15 },
    { 0xacd265e8, SUC_lexicon_tags_17 },
    { 0x5bd265ea, SUC_lexicon_tags_19 },
    { 0x257265eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d265ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6db265ef, SUC_lexicon_tags_87 },
    { 0x33265f0, SUC_lexicon_tags_24 },
    { 0xa3265f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973265f5, SUC_lexicon_tags_29 },
    { 0x941265f5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9265fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xadd265fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c7265ff, SUC_lexicon_tags_23 },
    { 0xe126600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1152660c, SUC_lexicon_tags_38 },
    { 0x5352660d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d26615, SUC_lexicon_tags_12 },
    { 0xbf26616, SUC_lexicon_tags_33 },
    { 0x30926615, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d26619, SUC_lexicon_tags_2 },
    { 0x1392661a, SUC_lexicon_tags_37 },
    { 0x6bd2661b, SUC_lexicon_tags_2 },
    { 0xc752661b, SUC_lexicon_tags_31 },
    { 0xd232661b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83126621, SUC_lexicon_tags_29 },
    { 0xf6b26622, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe1f26624, SUC_lexicon_tags_17 },
    { 0xaf26625, SUC_lexicon_tags_14 },
    { 0x66726626, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd372662d, SUC_lexicon_tags_47 },
    { 0xfb12662e, SUC_lexicon_tags_30 },
    { 0xd672662f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb26634, SUC_lexicon_tags_48 },
    { 0xfcd26635, SUC_lexicon_tags_47 },
    { 0x39326634, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc32663a, SUC_lexicon_tags_34 },
    { 0x2df2663b, SUC_lexicon_tags_15 },
    { 0x5372663c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0f2663e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5926643, SUC_lexicon_tags_57 },
    { 0x15726644, SUC_lexicon_tags_7 },
    { 0x7dd26645, SUC_lexicon_tags_48 },
    { 0xe4926645, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8852664a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e52664e, SUC_lexicon_tags_11 },
    { 0xbe92664e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b926651, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb26655, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd972665a, SUC_lexicon_tags_0 },
    { 0xf392665b, SUC_lexicon_tags_1 },
    { 0x6192665b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5526665, SUC_lexicon_tags_17 },
    { 0x7bf26665, SUC_lexicon_tags_39 },
    { 0xec526667, SUC_lexicon_tags_47 },
    { 0x9e926667, SUC_lexicon_tags_25 },
    { 0x3d726669, SUC_lexicon_tags_15 },
    { 0x9f926667, SUC_lexicon_tags_7 },
    { 0xc2d26668, SUC_lexicon_tags_12 },
    { 0x4532666c, SUC_lexicon_tags_17 },
    { 0x6632666c, SUC_lexicon_tags_47 },
    { 0xf3d2666d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xea726671, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a126674, SUC_lexicon_tags_37 },
    { 0x6c126674, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb926677, SUC_lexicon_tags_68 },
    { 0xda526677, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42326683, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4326685, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d326687, SUC_lexicon_tags_31 },
    { 0x8f326687, SUC_lexicon_tags_38 },
    { 0xd3926687, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d26696, SUC_lexicon_tags_46 },
    { 0x55b26696, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf26699, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4272669b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11266a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427266a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b266af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x589266b1, SUC_lexicon_tags_31 },
    { 0xc1266b2, SUC_lexicon_tags_45 },
    { 0x5a7266b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54f266b5, SUC_lexicon_tags_13 },
    { 0xecb266b5, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0x8c3266b8, SUC_lexicon_tags_63 },
    { 0x5b9266b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x253266bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x281266be, SUC_lexicon_tags_17 },
    { 0x779266bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f266c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x899266ca, SUC_lexicon_tags_25 },
    { 0x6bd266ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3266d1, SUC_lexicon_tags_16 },
    { 0x98d266d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x733266d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x615266d6, SUC_lexicon_tags_30 },
    { 0x217266d7, SUC_lexicon_tags_16 },
    { 0x99b266d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd266db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f7266dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c5266df, SUC_lexicon_tags_15 },
    { 0xf27266df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9266e6, SUC_lexicon_tags_37 },
    { 0xf43266e6, SUC_lexicon_tags_11 },
    { 0xef7266e8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9bb266ea, SUC_lexicon_tags_33 },
    { 0x421266ea, SUC_lexicon_tags_37 },
    { 0xf83266ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43266ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd49266f1, SUC_lexicon_tags_25 },
    { 0xa27266f2, SUC_lexicon_tags_14 },
    { 0x1e9266f3, SUC_lexicon_tags_12 },
    { 0x72b266f4, SUC_lexicon_tags_19 },
    { 0xbaf266f2, SUC_lexicon_tags_38 },
    { 0x745266f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x903266f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a3266fa, SUC_lexicon_tags_7 },
    { 0xe6d266fb, SUC_lexicon_tags_79 },
    { 0xed3266fb, SUC_lexicon_tags_37 },
    { 0x1f1266fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x16126700, SUC_lexicon_tags_87 },
    { 0xfb726701, SUC_lexicon_tags_41 },
    { 0xc0326701, SUC_lexicon_tags_77 },
    { 0xf7926703, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60926709, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x32d2670b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b726711, SUC_lexicon_tags_25 },
    { 0x47326712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81526715, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed526718, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd912671d, SUC_lexicon_tags_7 },
    { 0x5d72671e, SUC_lexicon_tags_10 },
    { 0xbfd2671e, SUC_lexicon_tags_37 },
    { 0x54f2671e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf26725, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x88526727, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7312672b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e926736, SUC_lexicon_tags_25 },
    { 0x86b26737, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3772673b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa452673d, SUC_lexicon_tags_6 },
    { 0x5732673d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57326744, SUC_lexicon_tags_27 },
    { 0xf7b26745, SUC_lexicon_tags_4 },
    { 0xd6326745, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1852674b, SUC_lexicon_tags_0 },
    { 0x6ff2674b, SUC_lexicon_tags_12 },
    { 0x9e92674b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb126750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23926752, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5c726754, SUC_lexicon_tags_15 },
    { 0xea926754, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd72675b, SUC_lexicon_tags_60 },
    { 0x3372675c, SUC_lexicon_tags_44 },
    { 0xf2f2675d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff726763, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x16b26765, SUC_lexicon_tags_129 },
    { 0xc8326766, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7326768, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2126770, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52326776, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb092677e, SUC_lexicon_tags_23 },
    { 0x2f52677e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f26786, SUC_lexicon_tags_16 },
    { 0x1c526787, SUC_lexicon_tags_9 },
    { 0x98726787, SUC_lexicon_tags_12 },
    { 0xc4526789, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x392678e, SUC_lexicon_tags_28 },
    { 0x3d12678f, SUC_lexicon_tags_12 },
    { 0xf2926790, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xde526792, SUC_lexicon_tags_11 },
    { 0x5a326793, SUC_lexicon_tags_38 },
    { 0x80926794, SUC_lexicon_tags_43 },
    { 0x8eb26795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87326799, SUC_lexicon_tags_47 },
    { 0x3e926799, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe5b2679c, SUC_lexicon_tags_150 },
    { 0xb6f2679d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f267a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2cd267a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc03267aa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b267af, SUC_lexicon_tags_97 },
    { 0xb77267b0, SUC_lexicon_tags_25 },
    { 0x583267af, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d267b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe67267bb, SUC_lexicon_tags_17 },
    { 0xdc3267bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05267c3, SUC_lexicon_tags_37 },
    { 0x505267c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5f9267c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845267d2, SUC_lexicon_tags_27 },
    { 0xcd7267d3, SUC_lexicon_tags_6 },
    { 0x4e7267d4, SUC_lexicon_tags_46 },
    { 0xc31267d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5267d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5267db, SUC_lexicon_tags_25 },
    { 0x877267db, SUC_lexicon_tags_44 },
    { 0xc7b267db, SUC_lexicon_tags_12 },
    { 0x97267de, SUC_lexicon_tags_28 },
    { 0x8e7267df, SUC_lexicon_tags_12 },
    { 0x4d1267df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x197267e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99267e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f267eb, SUC_lexicon_tags_2 },
    { 0xc8d267eb, SUC_lexicon_tags_60 },
    { 0xa33267ed, SUC_lexicon_tags_25 },
    { 0x96d267ee, SUC_lexicon_tags_11 },
    { 0xf59267ee, SUC_lexicon_tags_27 },
    { 0xf0d267ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90f267f2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15267f6, SUC_lexicon_tags_49 },
    { 0xf13267f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d267fc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9cb267fe, SUC_lexicon_tags_12 },
    { 0x57267ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91126801, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3726806, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1526808, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2332680a, SUC_lexicon_tags_37 },
    { 0xf252680a, SUC_lexicon_tags_11 },
    { 0xfa52680c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed26813, SUC_lexicon_tags_8 },
    { 0x16926814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc92681e, SUC_lexicon_tags_11 },
    { 0x7672681f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e526824, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a326826, SUC_lexicon_tags_21 },
    { 0x94f26827, SUC_lexicon_tags_7 },
    { 0xea726827, SUC_lexicon_tags_15 },
    { 0x63126829, SUC_lexicon_tags_37 },
    { 0xaff2682a, SUC_lexicon_tags_26 },
    { 0xeef2682b, SUC_lexicon_tags_14 },
    { 0xaed2682a, SUC_lexicon_tags_60 },
    { 0xa8726826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b26836, SUC_lexicon_tags_28 },
    { 0xf9926837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f2683d, SUC_lexicon_tags_47 },
    { 0xaad2683e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa926846, SUC_lexicon_tags_0 },
    { 0xb4d26847, SUC_lexicon_tags_25 },
    { 0x1ab26847, SUC_lexicon_tags_23 },
    { 0x96d26847, SUC_lexicon_tags_16 },
    { 0xa0926847, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x48f2684c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x632684e, SUC_lexicon_tags_2 },
    { 0x3a52684f, SUC_lexicon_tags_39 },
    { 0xf492684f, SUC_lexicon_tags_39 },
    { 0xf792684f, SUC_lexicon_tags_37 },
    { 0xba526852, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb26857, SUC_lexicon_tags_15 },
    { 0xa326858, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x2ab2685a, SUC_lexicon_tags_38 },
    { 0x4f2685a, SUC_lexicon_tags_1 },
    { 0x6032685b, SUC_lexicon_tags_0 },
    { 0xc272685b, SUC_lexicon_tags_39 },
    { 0x11f2685e, SUC_lexicon_tags_1 },
    { 0xb6d2685b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6726862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e326865, SUC_lexicon_tags_15 },
    { 0xfbd26866, SUC_lexicon_tags_17 },
    { 0xbd326867, SUC_lexicon_tags_11 },
    { 0xf7526867, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc032686c, SUC_lexicon_tags_12 },
    { 0x1b52686c, SUC_lexicon_tags_25 },
    { 0xbc52686e, SUC_lexicon_tags_11 },
    { 0x4e52686f, SUC_lexicon_tags_17 },
    { 0x10726870, SUC_lexicon_tags_84 },
    { 0x9bb2686f, SUC_lexicon_tags_27 },
    { 0x46926870, SUC_lexicon_tags_23 },
    { 0x64f2686f, SUC_lexicon_tags_6 },
    { 0x9af26874, SUC_lexicon_tags_43 },
    { 0xdc72686c, SUC_lexicon_tags_33 },
    { 0xaad26876, SUC_lexicon_tags_17 },
    { 0xa926877, SUC_lexicon_tags_37 },
    { 0x17f26877, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9d12687a, SUC_lexicon_tags_11 },
    { 0x6212687b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12687f, SUC_lexicon_tags_22 },
    { 0x6ff26880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xec926882, SUC_lexicon_tags_2 },
    { 0x80526882, SUC_lexicon_tags_0 },
    { 0x81126884, SUC_lexicon_tags_23 },
    { 0xb7326884, SUC_lexicon_tags_56 },
    { 0xa1126886, SUC_lexicon_tags_25 },
    { 0x4b326886, SUC_lexicon_tags_11 },
    { 0x33f26888, SUC_lexicon_tags_47 },
    { 0x74126889, SUC_lexicon_tags_7 },
    { 0xee726888, SUC_lexicon_tags_27 },
    { 0xd2688b, SUC_lexicon_tags_11 },
    { 0x7af26888, SUC_lexicon_tags_53 },
    { 0xd472688d, SUC_lexicon_tags_10 },
    { 0xf792688d, SUC_lexicon_tags_47 },
    { 0xc452688a, SUC_lexicon_tags_14 },
    { 0x4c726890, SUC_lexicon_tags_25 },
    { 0xbf926890, SUC_lexicon_tags_15 },
    { 0x53d26887, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e526899, SUC_lexicon_tags_7 },
    { 0x4472689a, SUC_lexicon_tags_33 },
    { 0x2a52689a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5412689e, SUC_lexicon_tags_1 },
    { 0xc5d2689f, SUC_lexicon_tags_11 },
    { 0xd63268a0, SUC_lexicon_tags_45 },
    { 0xaf5268a1, SUC_lexicon_tags_12 },
    { 0x9d9268a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc15268a4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7268a7, SUC_lexicon_tags_2 },
    { 0xff7268a8, SUC_lexicon_tags_17 },
    { 0xfcb268a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab268ae, SUC_lexicon_tags_27 },
    { 0x975268af, SUC_lexicon_tags_25 },
    { 0xc17268b0, SUC_lexicon_tags_6 },
    { 0xdf268af, SUC_lexicon_tags_7 },
    { 0xbd3268af, SUC_lexicon_tags_14 },
    { 0x441268af, SUC_lexicon_tags_151 },
    { 0x1f9268b4, SUC_lexicon_tags_77 },
    { 0xefb268b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5268b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63268bc, SUC_lexicon_tags_1 },
    { 0x447268bc, SUC_lexicon_tags_117 },
    { 0x11f268bd, SUC_lexicon_tags_11 },
    { 0x3bf268be, SUC_lexicon_tags_26 },
    { 0x5a7268c0, SUC_lexicon_tags_17 },
    { 0x3e7268c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3268c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431268c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f268cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15268d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23268d8, SUC_lexicon_tags_17 },
    { 0x5af268d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcdf268db, SUC_lexicon_tags_11 },
    { 0xe4b268dc, SUC_lexicon_tags_37 },
    { 0x8f9268db, SUC_lexicon_tags_37 },
    { 0x5c3268de, SUC_lexicon_tags_39 },
    { 0xc79268df, SUC_lexicon_tags_6 },
    { 0xfe7268dd, SUC_lexicon_tags_16 },
    { 0x7a7268db, SUC_lexicon_tags_27 },
    { 0x849268de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ad268e4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7268e8, SUC_lexicon_tags_37 },
    { 0x6b268e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af268ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9268f2, SUC_lexicon_tags_4 },
    { 0x4b9268f2, SUC_lexicon_tags_30 },
    { 0x79f268f4, SUC_lexicon_tags_33 },
    { 0xe9f268f4, SUC_lexicon_tags_37 },
    { 0xb1268f5, SUC_lexicon_tags_64 },
    { 0xeb9268f7, SUC_lexicon_tags_47 },
    { 0xb95268f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x353268fa, SUC_lexicon_tags_12 },
    { 0x12d268fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f5268fd, SUC_lexicon_tags_63 },
    { 0xf4b268fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3526900, SUC_lexicon_tags_38 },
    { 0x8d926901, SUC_lexicon_tags_37 },
    { 0xa126902, SUC_lexicon_tags_23 },
    { 0xca126903, SUC_lexicon_tags_11 },
    { 0xf0f26904, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd926906, SUC_lexicon_tags_124 },
    { 0x82d26907, SUC_lexicon_tags_25 },
    { 0xe8126906, SUC_lexicon_tags_12 },
    { 0x78b26909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8312690b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f26913, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe1126915, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x88526918, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf532691f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad26923, SUC_lexicon_tags_28 },
    { 0x3a326924, SUC_lexicon_tags_7 },
    { 0x2326925, SUC_lexicon_tags_11 },
    { 0xe5d26926, SUC_lexicon_tags_25 },
    { 0xe8d26927, SUC_lexicon_tags_14 },
    { 0x85b26925, SUC_lexicon_tags_920 },
    { 0x5b926923, SUC_lexicon_tags_7 },
    { 0xfd2692a, SUC_lexicon_tags_16 },
    { 0x51126923, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b92692e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa726932, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f26935, SUC_lexicon_tags_26 },
    { 0x2ff26935, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbeb26938, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cd2693a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb72693d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc726940, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2726944, SUC_lexicon_tags_87 },
    { 0x5c926945, SUC_lexicon_tags_53 },
    { 0x1fd26946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b52694d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26126950, SUC_lexicon_tags_55 },
    { 0x4c126950, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd126953, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb26960, SUC_lexicon_tags_19 },
    { 0xaf26961, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3926964, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf326968, SUC_lexicon_tags_15 },
    { 0xef926968, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7e32696b, SUC_lexicon_tags_85 },
    { 0x73d2696b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85326972, SUC_lexicon_tags_386 },
    { 0, NULL },
    { 0x86326974, SUC_lexicon_tags_60 },
    { 0xedb26974, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x64b26977, SUC_lexicon_tags_45 },
    { 0x70d26978, SUC_lexicon_tags_33 },
    { 0x68b26979, SUC_lexicon_tags_38 },
    { 0x6f32697a, SUC_lexicon_tags_45 },
    { 0xf332697a, SUC_lexicon_tags_12 },
    { 0xc752697c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaff2697e, SUC_lexicon_tags_28 },
    { 0xe552697f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5726981, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76326984, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48926988, SUC_lexicon_tags_25 },
    { 0x7326988, SUC_lexicon_tags_23 },
    { 0x7a12698a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68526990, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7126992, SUC_lexicon_tags_47 },
    { 0x20726993, SUC_lexicon_tags_12 },
    { 0x63b26994, SUC_lexicon_tags_33 },
    { 0x8126994, SUC_lexicon_tags_27 },
    { 0x7b726996, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x18b26998, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1812699b, SUC_lexicon_tags_31 },
    { 0x6c92699c, SUC_lexicon_tags_15 },
    { 0xca12699d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d269a2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35269a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x941269a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd9b269ab, SUC_lexicon_tags_7 },
    { 0xccf269ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485269b4, SUC_lexicon_tags_15 },
    { 0xb43269b5, SUC_lexicon_tags_42 },
    { 0x13d269b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf269bb, SUC_lexicon_tags_12 },
    { 0x501269bc, SUC_lexicon_tags_45 },
    { 0x2d1269bd, SUC_lexicon_tags_11 },
    { 0xb49269be, SUC_lexicon_tags_27 },
    { 0x617269bd, SUC_lexicon_tags_23 },
    { 0xcf269c0, SUC_lexicon_tags_12 },
    { 0x9e9269c1, SUC_lexicon_tags_1 },
    { 0x149269be, SUC_lexicon_tags_77 },
    { 0xfff269c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb49269c5, SUC_lexicon_tags_38 },
    { 0xe31269c6, SUC_lexicon_tags_63 },
    { 0x73b269c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9269ca, SUC_lexicon_tags_57 },
    { 0xb11269ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd1269cd, SUC_lexicon_tags_33 },
    { 0x83d269ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f269d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d269d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99269dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3269e0, SUC_lexicon_tags_84 },
    { 0x91d269e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbab269e3, SUC_lexicon_tags_11 },
    { 0x45d269e4, SUC_lexicon_tags_48 },
    { 0x3bf269e5, SUC_lexicon_tags_32 },
    { 0x109269e6, SUC_lexicon_tags_1 },
    { 0xb3b269e5, SUC_lexicon_tags_17 },
    { 0x9b1269e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9269eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xab269ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcaf269f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7269f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1269ff, SUC_lexicon_tags_31 },
    { 0x5db269ff, SUC_lexicon_tags_8 },
    { 0xe0926a01, SUC_lexicon_tags_6 },
    { 0xef126a02, SUC_lexicon_tags_25 },
    { 0x61269ff, SUC_lexicon_tags_37 },
    { 0xaa526a02, SUC_lexicon_tags_25 },
    { 0x14526a05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b26a09, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92526a10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9526a18, SUC_lexicon_tags_7 },
    { 0x1c326a18, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd0126a1b, SUC_lexicon_tags_23 },
    { 0x4d926a1c, SUC_lexicon_tags_63 },
    { 0x8c126a1d, SUC_lexicon_tags_8 },
    { 0x3cb26a1e, SUC_lexicon_tags_17 },
    { 0xae726a1f, SUC_lexicon_tags_17 },
    { 0x36d26a20, SUC_lexicon_tags_2 },
    { 0xfb126a21, SUC_lexicon_tags_33 },
    { 0xd2126a22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1526a26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3326a2a, SUC_lexicon_tags_23 },
    { 0x95f26a2a, SUC_lexicon_tags_48 },
    { 0xf3726a2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76926a2e, SUC_lexicon_tags_43 },
    { 0xbe926a2f, SUC_lexicon_tags_87 },
    { 0x3cd26a2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d26a33, SUC_lexicon_tags_1 },
    { 0xddd26a33, SUC_lexicon_tags_53 },
    { 0x59126a35, SUC_lexicon_tags_14 },
    { 0x61526a36, SUC_lexicon_tags_1 },
    { 0xdb926a37, SUC_lexicon_tags_60 },
    { 0x4f526a34, SUC_lexicon_tags_48 },
    { 0x3b926a34, SUC_lexicon_tags_12 },
    { 0x2cd26a38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f726a3d, SUC_lexicon_tags_3 },
    { 0xbd726a3e, SUC_lexicon_tags_11 },
    { 0x77526a3f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb5326a41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a926a44, SUC_lexicon_tags_6 },
    { 0x22b26a45, SUC_lexicon_tags_25 },
    { 0xbef26a46, SUC_lexicon_tags_57 },
    { 0x30126a47, SUC_lexicon_tags_10 },
    { 0x5526a48, SUC_lexicon_tags_49 },
    { 0x73926a47, SUC_lexicon_tags_37 },
    { 0xdb926a4a, SUC_lexicon_tags_1 },
    { 0x57f26a4b, SUC_lexicon_tags_37 },
    { 0x2b26a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7526a4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0526a52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85326a58, SUC_lexicon_tags_12 },
    { 0xfdb26a59, SUC_lexicon_tags_17 },
    { 0x83b26a5a, SUC_lexicon_tags_37 },
    { 0x8f326a5b, SUC_lexicon_tags_19 },
    { 0xd7126a5b, SUC_lexicon_tags_27 },
    { 0xed726a5c, SUC_lexicon_tags_11 },
    { 0xcc726a5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf926a61, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b26a65, SUC_lexicon_tags_33 },
    { 0x9326a65, SUC_lexicon_tags_1 },
    { 0xfcb26a67, SUC_lexicon_tags_4 },
    { 0xd5726a67, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77526a71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d326a74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2926a76, SUC_lexicon_tags_46 },
    { 0xc0d26a77, SUC_lexicon_tags_14 },
    { 0x2b526a78, SUC_lexicon_tags_15 },
    { 0x2e126a78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53926a7d, SUC_lexicon_tags_11 },
    { 0xcff26a7e, SUC_lexicon_tags_30 },
    { 0xa6f26a7e, SUC_lexicon_tags_6 },
    { 0xe6126a80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9726a85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a726a88, SUC_lexicon_tags_36 },
    { 0xa7126a88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df26a8c, SUC_lexicon_tags_17 },
    { 0xa2126a8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x24726a8f, SUC_lexicon_tags_26 },
    { 0xfcb26a8f, SUC_lexicon_tags_11 },
    { 0xe7f26a91, SUC_lexicon_tags_9 },
    { 0xc926a91, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15126a9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d26a9d, SUC_lexicon_tags_293 },
    { 0xfeb26a9e, SUC_lexicon_tags_18 },
    { 0x90326a9d, SUC_lexicon_tags_2 },
    { 0x7ad26aa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1726aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f26aa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af26aac, SUC_lexicon_tags_15 },
    { 0x4f26aad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb26ab2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b26ab8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9126ac0, SUC_lexicon_tags_17 },
    { 0xc5b26ac1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfd26ac3, SUC_lexicon_tags_7 },
    { 0xcc726ac4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f26ac7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83d26ac9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x49526acb, SUC_lexicon_tags_15 },
    { 0x28726acc, SUC_lexicon_tags_45 },
    { 0x63526acc, SUC_lexicon_tags_0 },
    { 0x7d26acd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfd526ad0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8bd26ad2, SUC_lexicon_tags_11 },
    { 0x29b26ad2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9fb26ad5, SUC_lexicon_tags_15 },
    { 0xf6f26ad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75126ad8, SUC_lexicon_tags_0 },
    { 0xf9326ad8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11726adb, SUC_lexicon_tags_28 },
    { 0x51926adc, SUC_lexicon_tags_48 },
    { 0x6b726add, SUC_lexicon_tags_2 },
    { 0xfe726ade, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2526ae7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f26aea, SUC_lexicon_tags_70 },
    { 0x10126aea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d526aee, SUC_lexicon_tags_37 },
    { 0xd9d26aee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd126af5, SUC_lexicon_tags_46 },
    { 0xddd26af6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62f26af8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8126afd, SUC_lexicon_tags_12 },
    { 0x6126afe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b26b01, SUC_lexicon_tags_68 },
    { 0x17526b01, SUC_lexicon_tags_0 },
    { 0x15f26b01, SUC_lexicon_tags_37 },
    { 0x6f926b02, SUC_lexicon_tags_14 },
    { 0xb3b26b05, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b26b0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69726b0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc326b10, SUC_lexicon_tags_14 },
    { 0xd3726b10, SUC_lexicon_tags_38 },
    { 0xfcd26b11, SUC_lexicon_tags_126 },
    { 0x20f26b10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d26b1d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f26b20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e326b23, SUC_lexicon_tags_7 },
    { 0xd3126b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b526b26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8b26b28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5926b2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b126b31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9326b37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b26b3d, SUC_lexicon_tags_9 },
    { 0xf1b26b3e, SUC_lexicon_tags_25 },
    { 0x23b26b3f, SUC_lexicon_tags_2 },
    { 0xf3126b3e, SUC_lexicon_tags_437 },
    { 0, NULL },
    { 0x52f26b42, SUC_lexicon_tags_29 },
    { 0x7eb26b43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb526b45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb26b48, SUC_lexicon_tags_33 },
    { 0x46f26b49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f26b4c, SUC_lexicon_tags_30 },
    { 0x8dd26b4c, SUC_lexicon_tags_25 },
    { 0xd7326b4d, SUC_lexicon_tags_12 },
    { 0x9b326b4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x91926b51, SUC_lexicon_tags_11 },
    { 0x34b26b52, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47526b59, SUC_lexicon_tags_8 },
    { 0x78526b59, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x4fd26b5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d26b5f, SUC_lexicon_tags_17 },
    { 0xbd726b5f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf26b66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3926b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1eb26b6a, SUC_lexicon_tags_31 },
    { 0x48b26b6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c726b77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb26b7a, SUC_lexicon_tags_37 },
    { 0xf7926b7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb926b7e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca126b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98326b84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11726b88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc0326b8a, SUC_lexicon_tags_38 },
    { 0x34f26b8a, SUC_lexicon_tags_0 },
    { 0xdcb26b8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d26b90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5326b94, SUC_lexicon_tags_11 },
    { 0x81726b94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf26b98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3526b9b, SUC_lexicon_tags_16 },
    { 0xffb26b9c, SUC_lexicon_tags_124 },
    { 0x35d26b9c, SUC_lexicon_tags_17 },
    { 0x36926b9d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x13126ba0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac726ba4, SUC_lexicon_tags_15 },
    { 0x21126ba5, SUC_lexicon_tags_8 },
    { 0x7e526ba5, SUC_lexicon_tags_63 },
    { 0x89926ba7, SUC_lexicon_tags_68 },
    { 0xd1f26ba7, SUC_lexicon_tags_21 },
    { 0x87726ba9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79126bb5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xc5f26bb7, SUC_lexicon_tags_49 },
    { 0xc7326bb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x63926bba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82b26bbc, SUC_lexicon_tags_15 },
    { 0x92726bbd, SUC_lexicon_tags_12 },
    { 0xe9526bbc, SUC_lexicon_tags_9 },
    { 0xbe326bbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0726bc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f26bc8, SUC_lexicon_tags_45 },
    { 0xdcb26bc8, SUC_lexicon_tags_8 },
    { 0x24d26bc8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x45d26bcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af26bcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82326bd2, SUC_lexicon_tags_72 },
    { 0xa1126bd3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81526bdc, SUC_lexicon_tags_37 },
    { 0x6cd26bdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b526be2, SUC_lexicon_tags_17 },
    { 0x72326be3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb26be7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13726bea, SUC_lexicon_tags_61 },
    { 0x21326bea, SUC_lexicon_tags_15 },
    { 0x43726bea, SUC_lexicon_tags_55 },
    { 0x17526bec, SUC_lexicon_tags_15 },
    { 0x80b26bee, SUC_lexicon_tags_21 },
    { 0x44526bea, SUC_lexicon_tags_14 },
    { 0xe9726bf0, SUC_lexicon_tags_1 },
    { 0x39326bec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf26bf5, SUC_lexicon_tags_45 },
    { 0xf6926bf6, SUC_lexicon_tags_2 },
    { 0x36b26bf7, SUC_lexicon_tags_57 },
    { 0x1df26bf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf26bfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd26bff, SUC_lexicon_tags_18 },
    { 0x7ff26c00, SUC_lexicon_tags_37 },
    { 0xc1d26c01, SUC_lexicon_tags_26 },
    { 0x8f926c02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e726c05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a126c08, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6c326c0a, SUC_lexicon_tags_45 },
    { 0xdd326c0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b26c0f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec126c16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e526c19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef26c1c, SUC_lexicon_tags_29 },
    { 0x80726c1d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b326c24, SUC_lexicon_tags_49 },
    { 0xb26c25, SUC_lexicon_tags_43 },
    { 0x5d26c26, SUC_lexicon_tags_22 },
    { 0xc0926c27, SUC_lexicon_tags_39 },
    { 0x9f526c28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d26c2c, SUC_lexicon_tags_87 },
    { 0xb926c2c, SUC_lexicon_tags_2 },
    { 0x91126c2c, SUC_lexicon_tags_2 },
    { 0x6ad26c2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4c926c31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc926c34, SUC_lexicon_tags_7 },
    { 0x35726c35, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84526c3b, SUC_lexicon_tags_6 },
    { 0x49526c3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db26c40, SUC_lexicon_tags_8 },
    { 0xc1d26c41, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2926c47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b26c4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35926c4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b26c54, SUC_lexicon_tags_23 },
    { 0x6df26c55, SUC_lexicon_tags_17 },
    { 0xf4326c56, SUC_lexicon_tags_1 },
    { 0xbb526c55, SUC_lexicon_tags_37 },
    { 0x23b26c58, SUC_lexicon_tags_11 },
    { 0x9b326c56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd5726c5b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa9326c5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed26c61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb26c64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef26c67, SUC_lexicon_tags_43 },
    { 0x1b526c67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd526c6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d26c73, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45526c79, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb26c83, SUC_lexicon_tags_27 },
    { 0x80326c84, SUC_lexicon_tags_33 },
    { 0xb3926c83, SUC_lexicon_tags_6 },
    { 0x25126c86, SUC_lexicon_tags_35 },
    { 0xb9726c87, SUC_lexicon_tags_83 },
    { 0x3f126c87, SUC_lexicon_tags_11 },
    { 0x72926c89, SUC_lexicon_tags_31 },
    { 0xf4f26c89, SUC_lexicon_tags_38 },
    { 0x84d26c8a, SUC_lexicon_tags_87 },
    { 0x54126c8b, SUC_lexicon_tags_29 },
    { 0xe2d26c8b, SUC_lexicon_tags_1 },
    { 0x74326c8d, SUC_lexicon_tags_47 },
    { 0xf7326c8f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d26c98, SUC_lexicon_tags_85 },
    { 0xbf926c99, SUC_lexicon_tags_37 },
    { 0x4a126c99, SUC_lexicon_tags_2 },
    { 0xf1b26c98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee326ca0, SUC_lexicon_tags_25 },
    { 0xf1726ca1, SUC_lexicon_tags_46 },
    { 0xae726ca2, SUC_lexicon_tags_2 },
    { 0x37726ca1, SUC_lexicon_tags_84 },
    { 0xc0d26ca4, SUC_lexicon_tags_2 },
    { 0x45526ca1, SUC_lexicon_tags_23 },
    { 0x90726ca6, SUC_lexicon_tags_68 },
    { 0x84326ca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f26cab, SUC_lexicon_tags_47 },
    { 0x86f26cac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x55d26cae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32126cb4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58326cb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf726cc0, SUC_lexicon_tags_6 },
    { 0x80326cc1, SUC_lexicon_tags_187 },
    { 0x7bf26cc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff526cc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f26ccb, SUC_lexicon_tags_37 },
    { 0x4c926ccb, SUC_lexicon_tags_15 },
    { 0x3c126ccd, SUC_lexicon_tags_1 },
    { 0x59926cce, SUC_lexicon_tags_15 },
    { 0x73d26ccf, SUC_lexicon_tags_39 },
    { 0x58f26cce, SUC_lexicon_tags_12 },
    { 0xa5726ccf, SUC_lexicon_tags_33 },
    { 0x61b26cd2, SUC_lexicon_tags_65 },
    { 0xbcd26ccf, SUC_lexicon_tags_17 },
    { 0x97f26cce, SUC_lexicon_tags_37 },
    { 0x37f26cd2, SUC_lexicon_tags_9 },
    { 0x5b926cd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b126cd8, SUC_lexicon_tags_25 },
    { 0x3cd26cd9, SUC_lexicon_tags_47 },
    { 0x6ed26cd9, SUC_lexicon_tags_25 },
    { 0x82f26cdb, SUC_lexicon_tags_19 },
    { 0x24f26cdc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40726ce1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x88526ce4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49926ce7, SUC_lexicon_tags_82 },
    { 0x1e126ce8, SUC_lexicon_tags_4 },
    { 0x3e326ce7, SUC_lexicon_tags_47 },
    { 0xbb726ce8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaad26cec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb26cf1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d26cf5, SUC_lexicon_tags_11 },
    { 0x5b926cf6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5df26cf8, SUC_lexicon_tags_11 },
    { 0xc8326cf8, SUC_lexicon_tags_38 },
    { 0x47926cfa, SUC_lexicon_tags_1 },
    { 0xb4f26cf9, SUC_lexicon_tags_15 },
    { 0x87b26cfc, SUC_lexicon_tags_0 },
    { 0x96926cf8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f526d00, SUC_lexicon_tags_3 },
    { 0xa3f26d01, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xce126d04, SUC_lexicon_tags_38 },
    { 0x45726d04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d26d0d, SUC_lexicon_tags_7 },
    { 0xf5326d0e, SUC_lexicon_tags_45 },
    { 0x7d526d0f, SUC_lexicon_tags_17 },
    { 0xe6726d10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd26d13, SUC_lexicon_tags_11 },
    { 0xaf326d14, SUC_lexicon_tags_17 },
    { 0x83526d15, SUC_lexicon_tags_11 },
    { 0xc0726d16, SUC_lexicon_tags_2 },
    { 0x7b26d17, SUC_lexicon_tags_2 },
    { 0xd6126d15, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b26d1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6326d1d, SUC_lexicon_tags_12 },
    { 0x9e726d1d, SUC_lexicon_tags_15 },
    { 0x3cb26d1f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf26d23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f26d26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0926d28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed26d2d, SUC_lexicon_tags_1 },
    { 0x80926d2e, SUC_lexicon_tags_7 },
    { 0xe7526d2e, SUC_lexicon_tags_70 },
    { 0x21726d2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86726d33, SUC_lexicon_tags_14 },
    { 0xaaf26d34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26926d37, SUC_lexicon_tags_37 },
    { 0x14526d38, SUC_lexicon_tags_7 },
    { 0xf3b26d37, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9df26d3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x44326d3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3326d40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd26d43, SUC_lexicon_tags_17 },
    { 0x4b526d44, SUC_lexicon_tags_47 },
    { 0xff926d45, SUC_lexicon_tags_37 },
    { 0x3ef26d43, SUC_lexicon_tags_11 },
    { 0xc9126d47, SUC_lexicon_tags_1 },
    { 0xcf126d48, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7126d4b, SUC_lexicon_tags_4 },
    { 0x92526d4c, SUC_lexicon_tags_7 },
    { 0x28d26d4d, SUC_lexicon_tags_17 },
    { 0x87126d4c, SUC_lexicon_tags_107 },
    { 0x34b26d4d, SUC_lexicon_tags_26 },
    { 0x4a726d4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c326d54, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f26d57, SUC_lexicon_tags_1 },
    { 0xcc126d58, SUC_lexicon_tags_17 },
    { 0x8af26d57, SUC_lexicon_tags_2 },
    { 0xc3726d5a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9f926d5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xda926d5e, SUC_lexicon_tags_46 },
    { 0x54f26d5e, SUC_lexicon_tags_1 },
    { 0xd6526d5e, SUC_lexicon_tags_0 },
    { 0xc4326d61, SUC_lexicon_tags_14 },
    { 0x36d26d62, SUC_lexicon_tags_21 },
    { 0x6b726d62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d26d6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6f26d71, SUC_lexicon_tags_26 },
    { 0x52b26d72, SUC_lexicon_tags_43 },
    { 0x49526d73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd26d76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x86b26d78, SUC_lexicon_tags_11 },
    { 0x67726d79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3326d7f, SUC_lexicon_tags_25 },
    { 0x40f26d7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79726d86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af26d8a, SUC_lexicon_tags_17 },
    { 0x1d26d8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f26d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf526d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d26d96, SUC_lexicon_tags_23 },
    { 0xcb26d97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x61526d99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8126d9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3926da0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49526da2, SUC_lexicon_tags_17 },
    { 0xf0326da3, SUC_lexicon_tags_92 },
    { 0xdd526da2, SUC_lexicon_tags_70 },
    { 0x29126da5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f26da7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb7726da9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b26dac, SUC_lexicon_tags_64 },
    { 0x18526dad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a726db0, SUC_lexicon_tags_25 },
    { 0xc9326db1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e726db5, SUC_lexicon_tags_27 },
    { 0xe4926db6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x97526db9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d26dbd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad26dc0, SUC_lexicon_tags_19 },
    { 0x47126dc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b26dc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb26dc7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4126dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed26dcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4726dd2, SUC_lexicon_tags_25 },
    { 0xc7f26dd3, SUC_lexicon_tags_17 },
    { 0xfa126dd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b26dd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc1d26dda, SUC_lexicon_tags_12 },
    { 0xa4d26ddb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51726de2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x16d26de4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c926df1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37526df6, SUC_lexicon_tags_15 },
    { 0x58d26df7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1526df9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6f26dfb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79526e04, SUC_lexicon_tags_47 },
    { 0xbeb26e05, SUC_lexicon_tags_55 },
    { 0x60f26e05, SUC_lexicon_tags_47 },
    { 0xc6f26e05, SUC_lexicon_tags_64 },
    { 0x83926e08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f26e0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd26e10, SUC_lexicon_tags_25 },
    { 0xed926e11, SUC_lexicon_tags_25 },
    { 0xbbd26e12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d26e18, SUC_lexicon_tags_26 },
    { 0x32f26e19, SUC_lexicon_tags_47 },
    { 0x9fd26e19, SUC_lexicon_tags_43 },
    { 0x88b26e18, SUC_lexicon_tags_7 },
    { 0xcd726e1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x77d26e1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2926e21, SUC_lexicon_tags_37 },
    { 0xe726e22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5c726e24, SUC_lexicon_tags_1 },
    { 0x49726e24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28326e27, SUC_lexicon_tags_1 },
    { 0xcf326e28, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x48926e2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e126e2e, SUC_lexicon_tags_25 },
    { 0x3f726e2f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x80526e32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd326e34, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb26e37, SUC_lexicon_tags_31 },
    { 0xb7b26e37, SUC_lexicon_tags_0 },
    { 0x4af26e39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceb26e3b, SUC_lexicon_tags_43 },
    { 0x3c926e3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7f26e3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f26e42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a926e45, SUC_lexicon_tags_7 },
    { 0x6d926e45, SUC_lexicon_tags_47 },
    { 0x9bf26e47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d26e4d, SUC_lexicon_tags_6 },
    { 0x45726e4d, SUC_lexicon_tags_33 },
    { 0x89f26e4d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf26e5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x30326e5d, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x8ad26e5f, SUC_lexicon_tags_36 },
    { 0xaa126e5f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e126e66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b26e6c, SUC_lexicon_tags_15 },
    { 0x41326e6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd26e73, SUC_lexicon_tags_62 },
    { 0x7c326e74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f26e7c, SUC_lexicon_tags_15 },
    { 0xdeb26e7c, SUC_lexicon_tags_33 },
    { 0xf8326e7e, SUC_lexicon_tags_2 },
    { 0x7f326e7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef726e81, SUC_lexicon_tags_11 },
    { 0xc6d26e81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e926e85, SUC_lexicon_tags_14 },
    { 0x74526e85, SUC_lexicon_tags_10 },
    { 0x25526e87, SUC_lexicon_tags_12 },
    { 0xeaf26e87, SUC_lexicon_tags_17 },
    { 0xef726e89, SUC_lexicon_tags_235 },
    { 0x7bb26e86, SUC_lexicon_tags_31 },
    { 0xedb26e8a, SUC_lexicon_tags_65 },
    { 0x69d26e85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd26e91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a526e94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d26e9f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a926ea2, SUC_lexicon_tags_1 },
    { 0x7f326ea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59926ea6, SUC_lexicon_tags_37 },
    { 0x8c126ea7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x56326eaa, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84326eb3, SUC_lexicon_tags_23 },
    { 0x5e926eb3, SUC_lexicon_tags_37 },
    { 0xd7d26eb5, SUC_lexicon_tags_57 },
    { 0x41926eb5, SUC_lexicon_tags_53 },
    { 0x3b326eb7, SUC_lexicon_tags_20 },
    { 0xe4126eb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f526ebb, SUC_lexicon_tags_15 },
    { 0x77926ebc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f26ebf, SUC_lexicon_tags_11 },
    { 0x99526ebf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd326ec4, SUC_lexicon_tags_28 },
    { 0xc1d26ec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19326eca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69326ecd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8726ed4, SUC_lexicon_tags_25 },
    { 0x50926ed5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x40726ed7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa726ed9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd26edd, SUC_lexicon_tags_4 },
    { 0xf7f26ede, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb3326ee0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd26ee6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x25126ee8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x11726eea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77526eed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d26ef0, SUC_lexicon_tags_11 },
    { 0x11726ef1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a726ef3, SUC_lexicon_tags_45 },
    { 0x9e726ef4, SUC_lexicon_tags_61 },
    { 0xcc526ef5, SUC_lexicon_tags_46 },
    { 0xf2d26ef6, SUC_lexicon_tags_63 },
    { 0x9e526ef4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd26efb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5726f02, SUC_lexicon_tags_45 },
    { 0x9e926f03, SUC_lexicon_tags_2 },
    { 0x7a926f03, SUC_lexicon_tags_39 },
    { 0x48b26f05, SUC_lexicon_tags_33 },
    { 0xccd26f05, SUC_lexicon_tags_45 },
    { 0x13926f06, SUC_lexicon_tags_79 },
    { 0xff526f08, SUC_lexicon_tags_27 },
    { 0xcb526f07, SUC_lexicon_tags_7 },
    { 0x43926f0a, SUC_lexicon_tags_11 },
    { 0x81b26f0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x49526f0d, SUC_lexicon_tags_25 },
    { 0xa2b26f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf26f12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x92326f14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5726f19, SUC_lexicon_tags_19 },
    { 0x34f26f1a, SUC_lexicon_tags_37 },
    { 0xcbf26f1b, SUC_lexicon_tags_21 },
    { 0xd3126f1c, SUC_lexicon_tags_1 },
    { 0x8b326f1d, SUC_lexicon_tags_37 },
    { 0x58926f1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4126f20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b26f27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d926f29, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b26f2c, SUC_lexicon_tags_1 },
    { 0x56b26f2d, SUC_lexicon_tags_17 },
    { 0xc0b26f2d, SUC_lexicon_tags_117 },
    { 0x61526f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d26f32, SUC_lexicon_tags_7 },
    { 0xf8926f33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73726f36, SUC_lexicon_tags_11 },
    { 0xaf526f37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35926f39, SUC_lexicon_tags_11 },
    { 0x85526f3a, SUC_lexicon_tags_38 },
    { 0xaf26f3a, SUC_lexicon_tags_7 },
    { 0x76726f39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe9326f3e, SUC_lexicon_tags_12 },
    { 0xc0726f3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda726f43, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e526f45, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbe526f47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f26f4c, SUC_lexicon_tags_19 },
    { 0x88d26f4d, SUC_lexicon_tags_25 },
    { 0x5c126f4e, SUC_lexicon_tags_30 },
    { 0x76926f4f, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a926f52, SUC_lexicon_tags_9 },
    { 0xd1726f53, SUC_lexicon_tags_45 },
    { 0x51d26f54, SUC_lexicon_tags_56 },
    { 0x9d526f54, SUC_lexicon_tags_38 },
    { 0x18d26f56, SUC_lexicon_tags_17 },
    { 0x7126f57, SUC_lexicon_tags_6 },
    { 0x8326f57, SUC_lexicon_tags_15 },
    { 0xe1726f57, SUC_lexicon_tags_42 },
    { 0xedd26f52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1b526f5c, SUC_lexicon_tags_0 },
    { 0x34526f5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf126f60, SUC_lexicon_tags_11 },
    { 0xd6b26f60, SUC_lexicon_tags_17 },
    { 0x46526f62, SUC_lexicon_tags_16 },
    { 0x85d26f63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9326f6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f26f70, SUC_lexicon_tags_37 },
    { 0x83926f71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f326f74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd26f7a, SUC_lexicon_tags_15 },
    { 0xadb26f7b, SUC_lexicon_tags_15 },
    { 0xb6126f7c, SUC_lexicon_tags_61 },
    { 0x38126f7b, SUC_lexicon_tags_17 },
    { 0x8cf26f7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8d526f80, SUC_lexicon_tags_17 },
    { 0x17d26f80, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d26f84, SUC_lexicon_tags_218 },
    { 0xaeb26f85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b26f8a, SUC_lexicon_tags_33 },
    { 0xc2526f8b, SUC_lexicon_tags_12 },
    { 0x1ed26f8c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd0326f8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed26f93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6726f96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b26f9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd726fa6, SUC_lexicon_tags_11 },
    { 0x65b26fa7, SUC_lexicon_tags_1 },
    { 0x8c926fa7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e726faa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e126fae, SUC_lexicon_tags_29 },
    { 0xe1926faf, SUC_lexicon_tags_26 },
    { 0xde726fae, SUC_lexicon_tags_16 },
    { 0x86b26faf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4726fb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ab26fb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef126fbb, SUC_lexicon_tags_11 },
    { 0x23f26fbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6926fc0, SUC_lexicon_tags_2 },
    { 0xa2d26fc1, SUC_lexicon_tags_30 },
    { 0xa5d26fc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f26fc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x82526fc9, SUC_lexicon_tags_61 },
    { 0x71b26fca, SUC_lexicon_tags_28 },
    { 0xeeb26fca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f26fcf, SUC_lexicon_tags_7 },
    { 0x326fd0, SUC_lexicon_tags_17 },
    { 0x9fd26fd1, SUC_lexicon_tags_38 },
    { 0xccb26fcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x59126fd5, SUC_lexicon_tags_16 },
    { 0x7e926fd5, SUC_lexicon_tags_19 },
    { 0x1d926fd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61526fd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff26fdd, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec926fe1, SUC_lexicon_tags_19 },
    { 0x13726fe2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f26fe9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f26ff1, SUC_lexicon_tags_7 },
    { 0x4726ff2, SUC_lexicon_tags_1 },
    { 0xa8926ff2, SUC_lexicon_tags_7 },
    { 0xd2526ff1, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xd0326ff6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x36926ff9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad526ffb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d127003, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3927009, SUC_lexicon_tags_14 },
    { 0x3932700a, SUC_lexicon_tags_63 },
    { 0x3032700b, SUC_lexicon_tags_17 },
    { 0x8732700c, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xd772700f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f27014, SUC_lexicon_tags_27 },
    { 0x5e527014, SUC_lexicon_tags_11 },
    { 0xc9f27016, SUC_lexicon_tags_23 },
    { 0xcd527017, SUC_lexicon_tags_17 },
    { 0x3f27018, SUC_lexicon_tags_38 },
    { 0xc9327018, SUC_lexicon_tags_27 },
    { 0x6c727019, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3972701e, SUC_lexicon_tags_11 },
    { 0x9272701f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d927022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d727027, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x13327029, SUC_lexicon_tags_7 },
    { 0xa927029, SUC_lexicon_tags_66 },
    { 0xa1f27029, SUC_lexicon_tags_11 },
    { 0x3292702a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x9012702f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf27032, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22127038, SUC_lexicon_tags_23 },
    { 0xaad27039, SUC_lexicon_tags_62 },
    { 0x7892703a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46327046, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f2704a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9312704f, SUC_lexicon_tags_38 },
    { 0x31d27050, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x69727053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc52705b, SUC_lexicon_tags_16 },
    { 0xecd2705b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7832705e, SUC_lexicon_tags_6 },
    { 0xa272705f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x86927061, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x31327064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d2706a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe52706d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5132706f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f27075, SUC_lexicon_tags_15 },
    { 0x35127075, SUC_lexicon_tags_46 },
    { 0x22d27077, SUC_lexicon_tags_17 },
    { 0xa6927078, SUC_lexicon_tags_70 },
    { 0x5b27078, SUC_lexicon_tags_25 },
    { 0xbc727078, SUC_lexicon_tags_44 },
    { 0x4b52707b, SUC_lexicon_tags_0 },
    { 0xdc927079, SUC_lexicon_tags_2 },
    { 0xaa92707b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec72707f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5527086, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d72708a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9bd2708c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf772708f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2727098, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f12709a, SUC_lexicon_tags_26 },
    { 0x6ab2709b, SUC_lexicon_tags_60 },
    { 0x1cf2709a, SUC_lexicon_tags_37 },
    { 0x3ed2709d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9270a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4b270a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7270aa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdbf270ac, SUC_lexicon_tags_6 },
    { 0xa0b270ac, SUC_lexicon_tags_15 },
    { 0x8eb270ae, SUC_lexicon_tags_12 },
    { 0x419270af, SUC_lexicon_tags_11 },
    { 0x9c3270af, SUC_lexicon_tags_25 },
    { 0xb47270b0, SUC_lexicon_tags_37 },
    { 0x249270ae, SUC_lexicon_tags_25 },
    { 0xff270b2, SUC_lexicon_tags_2 },
    { 0xdc3270ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75270b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3270bb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3a9270bd, SUC_lexicon_tags_17 },
    { 0xbed270bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe83270c0, SUC_lexicon_tags_7 },
    { 0x4df270c0, SUC_lexicon_tags_25 },
    { 0x3af270c1, SUC_lexicon_tags_65 },
    { 0x431270c2, SUC_lexicon_tags_14 },
    { 0xe37270c1, SUC_lexicon_tags_79 },
    { 0xced270c5, SUC_lexicon_tags_27 },
    { 0x2cd270c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9a3270c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af270cc, SUC_lexicon_tags_11 },
    { 0xefb270cd, SUC_lexicon_tags_31 },
    { 0xedb270ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57b270d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51d270d2, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685270d6, SUC_lexicon_tags_7 },
    { 0xf59270d7, SUC_lexicon_tags_17 },
    { 0xdc9270d8, SUC_lexicon_tags_17 },
    { 0xcbd270d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d270dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa3270de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x825270e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf270e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd270e8, SUC_lexicon_tags_30 },
    { 0x6cd270e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9270f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3270f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d270fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa57270fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b27104, SUC_lexicon_tags_25 },
    { 0x21527105, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d2710f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed27112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78b27114, SUC_lexicon_tags_31 },
    { 0x54d27115, SUC_lexicon_tags_1 },
    { 0x61527116, SUC_lexicon_tags_37 },
    { 0x15f27114, SUC_lexicon_tags_37 },
    { 0x89727115, SUC_lexicon_tags_7 },
    { 0x34127119, SUC_lexicon_tags_17 },
    { 0xcad27117, SUC_lexicon_tags_69 },
    { 0xc532711b, SUC_lexicon_tags_38 },
    { 0xec32711b, SUC_lexicon_tags_14 },
    { 0x5a92711c, SUC_lexicon_tags_47 },
    { 0xe0b2711e, SUC_lexicon_tags_27 },
    { 0x3092711c, SUC_lexicon_tags_47 },
    { 0xea127120, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c127123, SUC_lexicon_tags_12 },
    { 0xd4b27124, SUC_lexicon_tags_22 },
    { 0x58d27125, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5327127, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe112712a, SUC_lexicon_tags_47 },
    { 0x22b2712a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa352712f, SUC_lexicon_tags_11 },
    { 0x10327130, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2527134, SUC_lexicon_tags_55 },
    { 0xe7527135, SUC_lexicon_tags_15 },
    { 0x8fb27136, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35d27138, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1bb2713a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc72713f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe2127141, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed27148, SUC_lexicon_tags_33 },
    { 0xcb127149, SUC_lexicon_tags_37 },
    { 0x8b327148, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x87b2714c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f27150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55727154, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1e327156, SUC_lexicon_tags_48 },
    { 0x4c527157, SUC_lexicon_tags_63 },
    { 0xfb727158, SUC_lexicon_tags_30 },
    { 0xfcb27159, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a92715b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4992715f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e527166, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcb527168, SUC_lexicon_tags_17 },
    { 0xb7127169, SUC_lexicon_tags_38 },
    { 0xd2127169, SUC_lexicon_tags_11 },
    { 0x94f2716b, SUC_lexicon_tags_47 },
    { 0xc132716a, SUC_lexicon_tags_12 },
    { 0xaeb2716d, SUC_lexicon_tags_54 },
    { 0x5092716c, SUC_lexicon_tags_6 },
    { 0x2d92716f, SUC_lexicon_tags_23 },
    { 0x12927170, SUC_lexicon_tags_17 },
    { 0xf1527171, SUC_lexicon_tags_37 },
    { 0x25927172, SUC_lexicon_tags_23 },
    { 0x54327173, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad27176, SUC_lexicon_tags_2 },
    { 0x44327177, SUC_lexicon_tags_40 },
    { 0xf9527177, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2472717b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff27183, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef127185, SUC_lexicon_tags_11 },
    { 0x7f727186, SUC_lexicon_tags_29 },
    { 0x92527187, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x18127189, SUC_lexicon_tags_47 },
    { 0x5c72718a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xe452718d, SUC_lexicon_tags_47 },
    { 0x1152718e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc0d27190, SUC_lexicon_tags_7 },
    { 0x10927190, SUC_lexicon_tags_53 },
    { 0xf3927192, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e327199, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e12719e, SUC_lexicon_tags_37 },
    { 0x43f2719f, SUC_lexicon_tags_26 },
    { 0x569271a0, SUC_lexicon_tags_0 },
    { 0x2cf2719e, SUC_lexicon_tags_37 },
    { 0xf77271a2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd31271a4, SUC_lexicon_tags_17 },
    { 0xc9271a5, SUC_lexicon_tags_87 },
    { 0x3ef271a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebf271a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1271ad, SUC_lexicon_tags_11 },
    { 0x4b7271ad, SUC_lexicon_tags_6 },
    { 0x781271ae, SUC_lexicon_tags_33 },
    { 0xed1271b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d271b3, SUC_lexicon_tags_47 },
    { 0x4c9271b4, SUC_lexicon_tags_40 },
    { 0xc55271b5, SUC_lexicon_tags_17 },
    { 0x90f271b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7271bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8b271bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9271c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9271c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261271c9, SUC_lexicon_tags_37 },
    { 0x7b271c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45271d0, SUC_lexicon_tags_558 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed271d3, SUC_lexicon_tags_37 },
    { 0xa4d271d4, SUC_lexicon_tags_7 },
    { 0x16b271d5, SUC_lexicon_tags_46 },
    { 0xf9f271d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ab271d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd55271da, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d271dd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d271e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b271e6, SUC_lexicon_tags_33 },
    { 0x5bd271e7, SUC_lexicon_tags_37 },
    { 0xa81271e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x779271ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663271ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc87271f0, SUC_lexicon_tags_37 },
    { 0xe33271f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78d271f3, SUC_lexicon_tags_38 },
    { 0xd7f271f4, SUC_lexicon_tags_63 },
    { 0xc2f271f5, SUC_lexicon_tags_47 },
    { 0xb75271f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96b271f8, SUC_lexicon_tags_25 },
    { 0xd19271f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1271fe, SUC_lexicon_tags_43 },
    { 0x74d271ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f27203, SUC_lexicon_tags_29 },
    { 0x23927204, SUC_lexicon_tags_11 },
    { 0x6f327204, SUC_lexicon_tags_6 },
    { 0xd9327206, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9b27208, SUC_lexicon_tags_79 },
    { 0x96b27208, SUC_lexicon_tags_13 },
    { 0x8cf2720a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7372720e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x69727210, SUC_lexicon_tags_38 },
    { 0xff327211, SUC_lexicon_tags_12 },
    { 0x46d27211, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab27215, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1727219, SUC_lexicon_tags_17 },
    { 0xa9d2721a, SUC_lexicon_tags_46 },
    { 0xf492721b, SUC_lexicon_tags_25 },
    { 0x87f2721c, SUC_lexicon_tags_30 },
    { 0x85b2721a, SUC_lexicon_tags_14 },
    { 0x50d2721b, SUC_lexicon_tags_12 },
    { 0x7292721d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9727226, SUC_lexicon_tags_60 },
    { 0xd5d27227, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd2722a, SUC_lexicon_tags_8 },
    { 0x1132722b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa527233, SUC_lexicon_tags_25 },
    { 0x76d27233, SUC_lexicon_tags_17 },
    { 0x39b27233, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x19127237, SUC_lexicon_tags_27 },
    { 0xf4b27238, SUC_lexicon_tags_15 },
    { 0x9ad27238, SUC_lexicon_tags_32 },
    { 0xcdb2723a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb872723e, SUC_lexicon_tags_17 },
    { 0x66f2723f, SUC_lexicon_tags_21 },
    { 0x60127240, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4527242, SUC_lexicon_tags_37 },
    { 0x48727243, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7127246, SUC_lexicon_tags_43 },
    { 0x28b27247, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5332724d, SUC_lexicon_tags_63 },
    { 0xa112724e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b27252, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a527254, SUC_lexicon_tags_15 },
    { 0x2cb27255, SUC_lexicon_tags_29 },
    { 0xe2327255, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa527258, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc852725a, SUC_lexicon_tags_12 },
    { 0xf852725a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb127261, SUC_lexicon_tags_1 },
    { 0x4ad27262, SUC_lexicon_tags_87 },
    { 0x45f27262, SUC_lexicon_tags_23 },
    { 0x6927263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7f27266, SUC_lexicon_tags_31 },
    { 0x6127267, SUC_lexicon_tags_63 },
    { 0xc1727266, SUC_lexicon_tags_0 },
    { 0xdaf27269, SUC_lexicon_tags_11 },
    { 0x5b27266, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe832726d, SUC_lexicon_tags_2 },
    { 0x4752726e, SUC_lexicon_tags_11 },
    { 0xc352726e, SUC_lexicon_tags_73 },
    { 0x4ef27270, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a727272, SUC_lexicon_tags_15 },
    { 0xdcd27272, SUC_lexicon_tags_17 },
    { 0xe4727272, SUC_lexicon_tags_15 },
    { 0x7d727275, SUC_lexicon_tags_25 },
    { 0xced27272, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d27279, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa672727b, SUC_lexicon_tags_25 },
    { 0x8492727c, SUC_lexicon_tags_25 },
    { 0x20f2727d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3d2727f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d27282, SUC_lexicon_tags_7 },
    { 0x8c927283, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c727286, SUC_lexicon_tags_1 },
    { 0x4a527287, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8792728a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b2728f, SUC_lexicon_tags_49 },
    { 0x3cf27290, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57127294, SUC_lexicon_tags_37 },
    { 0xa3127295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2327297, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81527299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a12729b, SUC_lexicon_tags_1 },
    { 0xc832729c, SUC_lexicon_tags_15 },
    { 0x5612729d, SUC_lexicon_tags_11 },
    { 0x8492729e, SUC_lexicon_tags_25 },
    { 0x9912729e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153272a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05272a6, SUC_lexicon_tags_43 },
    { 0x945272a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7272aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2d272ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3272af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9272b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52f272b4, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b272b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x439272ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1272c0, SUC_lexicon_tags_2 },
    { 0xfb5272c0, SUC_lexicon_tags_3 },
    { 0xcfb272c0, SUC_lexicon_tags_63 },
    { 0xcc1272c3, SUC_lexicon_tags_43 },
    { 0xc57272c4, SUC_lexicon_tags_4 },
    { 0xe89272c4, SUC_lexicon_tags_37 },
    { 0xc01272c5, SUC_lexicon_tags_11 },
    { 0x777272c7, SUC_lexicon_tags_25 },
    { 0x637272c8, SUC_lexicon_tags_11 },
    { 0xa5b272c9, SUC_lexicon_tags_41 },
    { 0x769272c5, SUC_lexicon_tags_31 },
    { 0x19b272c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x449272cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad9272cf, SUC_lexicon_tags_7 },
    { 0xda7272d0, SUC_lexicon_tags_7 },
    { 0x525272d1, SUC_lexicon_tags_17 },
    { 0xf95272d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f272d7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x389272d9, SUC_lexicon_tags_7 },
    { 0x9c7272da, SUC_lexicon_tags_4 },
    { 0xa05272d9, SUC_lexicon_tags_6 },
    { 0x729272d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa3272de, SUC_lexicon_tags_11 },
    { 0x603272df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x469272e1, SUC_lexicon_tags_29 },
    { 0xf01272e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd272e5, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345272ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x529272ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9272f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853272f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9d272f9, SUC_lexicon_tags_25 },
    { 0x54b272f9, SUC_lexicon_tags_0 },
    { 0x57f272fb, SUC_lexicon_tags_2 },
    { 0x6a3272fc, SUC_lexicon_tags_41 },
    { 0x5ad272fd, SUC_lexicon_tags_37 },
    { 0xe6d272fd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b727303, SUC_lexicon_tags_37 },
    { 0x50f27304, SUC_lexicon_tags_25 },
    { 0x79327303, SUC_lexicon_tags_7 },
    { 0x9dd27306, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd27309, SUC_lexicon_tags_9 },
    { 0x5cb2730a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57f2730c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f27311, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7727313, SUC_lexicon_tags_70 },
    { 0x17d27314, SUC_lexicon_tags_0 },
    { 0x87f27315, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb672731a, SUC_lexicon_tags_1240 },
    { 0x27f2731a, SUC_lexicon_tags_1 },
    { 0x2932731c, SUC_lexicon_tags_25 },
    { 0x5e12731d, SUC_lexicon_tags_9 },
    { 0x69d2731d, SUC_lexicon_tags_15 },
    { 0x3c52731d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d27323, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9592732a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd327332, SUC_lexicon_tags_37 },
    { 0x79127333, SUC_lexicon_tags_27 },
    { 0xe727334, SUC_lexicon_tags_1 },
    { 0xc6127335, SUC_lexicon_tags_31 },
    { 0x96927336, SUC_lexicon_tags_6 },
    { 0xcd727336, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xab27339, SUC_lexicon_tags_7 },
    { 0x2152733a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9527344, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c327348, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe152734b, SUC_lexicon_tags_48 },
    { 0xa072734b, SUC_lexicon_tags_1061 },
    { 0xbb72734c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef27350, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff27354, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d27358, SUC_lexicon_tags_46 },
    { 0xb9d27359, SUC_lexicon_tags_17 },
    { 0xdaf2735a, SUC_lexicon_tags_60 },
    { 0xd812735b, SUC_lexicon_tags_82 },
    { 0xcb52735c, SUC_lexicon_tags_12 },
    { 0x50327359, SUC_lexicon_tags_0 },
    { 0xfd32735d, SUC_lexicon_tags_63 },
    { 0x61f27358, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e727362, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd927364, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x72127366, SUC_lexicon_tags_7 },
    { 0x88727367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad27375, SUC_lexicon_tags_17 },
    { 0x5dd27375, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f2737f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3927388, SUC_lexicon_tags_27 },
    { 0x8d127388, SUC_lexicon_tags_2 },
    { 0xbb92738a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4952738e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d27392, SUC_lexicon_tags_70 },
    { 0xc4f27393, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2527396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8852739e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617273a2, SUC_lexicon_tags_19 },
    { 0x7d3273a2, SUC_lexicon_tags_17 },
    { 0xd273a2, SUC_lexicon_tags_2 },
    { 0xef5273a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81273a9, SUC_lexicon_tags_12 },
    { 0x3b1273aa, SUC_lexicon_tags_37 },
    { 0xfa9273ab, SUC_lexicon_tags_28 },
    { 0x8f9273ac, SUC_lexicon_tags_0 },
    { 0xd8d273ad, SUC_lexicon_tags_7 },
    { 0x7a3273ae, SUC_lexicon_tags_12 },
    { 0x7f9273ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929273b5, SUC_lexicon_tags_27 },
    { 0x1eb273b6, SUC_lexicon_tags_43 },
    { 0x937273b7, SUC_lexicon_tags_15 },
    { 0xf2f273b7, SUC_lexicon_tags_27 },
    { 0x5eb273b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd273bf, SUC_lexicon_tags_25 },
    { 0x69f273c0, SUC_lexicon_tags_14 },
    { 0x437273bf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d273c4, SUC_lexicon_tags_11 },
    { 0xb7b273c5, SUC_lexicon_tags_1 },
    { 0x31b273c5, SUC_lexicon_tags_25 },
    { 0x5cb273c7, SUC_lexicon_tags_15 },
    { 0x443273c5, SUC_lexicon_tags_1 },
    { 0xec7273c9, SUC_lexicon_tags_60 },
    { 0xb7273ca, SUC_lexicon_tags_87 },
    { 0xe2f273cb, SUC_lexicon_tags_49 },
    { 0x4a7273ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77273d2, SUC_lexicon_tags_17 },
    { 0x72f273d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5c5273d5, SUC_lexicon_tags_29 },
    { 0xb3d273d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x167273d8, SUC_lexicon_tags_69 },
    { 0xdcf273d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b273e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f273e4, SUC_lexicon_tags_15 },
    { 0x507273e5, SUC_lexicon_tags_12 },
    { 0x64d273e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1273e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd273f0, SUC_lexicon_tags_23 },
    { 0xcc5273f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x973273f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9273f6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd05273f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba9273fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x51b273fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x909273ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0527403, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42727405, SUC_lexicon_tags_37 },
    { 0xa5127406, SUC_lexicon_tags_11 },
    { 0x6c727407, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d2740a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf92740c, SUC_lexicon_tags_33 },
    { 0xfd12740d, SUC_lexicon_tags_25 },
    { 0xb312740e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd27413, SUC_lexicon_tags_47 },
    { 0x6cb27414, SUC_lexicon_tags_15 },
    { 0xe927415, SUC_lexicon_tags_34 },
    { 0xb6727414, SUC_lexicon_tags_7 },
    { 0xbc527417, SUC_lexicon_tags_11 },
    { 0x9a327418, SUC_lexicon_tags_77 },
    { 0xf4727419, SUC_lexicon_tags_9 },
    { 0xbf27414, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed2741f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4527423, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b27427, SUC_lexicon_tags_1415 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f2742a, SUC_lexicon_tags_14 },
    { 0x5692742b, SUC_lexicon_tags_27 },
    { 0x7912742b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb952742e, SUC_lexicon_tags_45 },
    { 0x6092742f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb7127431, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f27434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b327436, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d27439, SUC_lexicon_tags_10 },
    { 0x1752743a, SUC_lexicon_tags_15 },
    { 0xb312743b, SUC_lexicon_tags_47 },
    { 0x5f72743a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x852743f, SUC_lexicon_tags_12 },
    { 0xdbb27440, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xda327442, SUC_lexicon_tags_45 },
    { 0x3e327443, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99f27445, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb2744b, SUC_lexicon_tags_27 },
    { 0x1f72744b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf2744f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4e927451, SUC_lexicon_tags_2 },
    { 0x1727452, SUC_lexicon_tags_11 },
    { 0x77d27451, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6927457, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x21b27459, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ef2745b, SUC_lexicon_tags_38 },
    { 0x54f2745c, SUC_lexicon_tags_2 },
    { 0x2792745c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5812745f, SUC_lexicon_tags_25 },
    { 0xe327460, SUC_lexicon_tags_37 },
    { 0x24d27461, SUC_lexicon_tags_15 },
    { 0xfdb27461, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1112746a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x70f2746c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1f12746e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa4527470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d2747d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d27481, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43327484, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x26727487, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a527489, SUC_lexicon_tags_29 },
    { 0xb072748a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9732748d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd52748f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc4527491, SUC_lexicon_tags_113 },
    { 0x1a527492, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d127494, SUC_lexicon_tags_63 },
    { 0x3af27494, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x42927498, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa472749b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb332749d, SUC_lexicon_tags_15 },
    { 0x7b12749e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b274a2, SUC_lexicon_tags_2 },
    { 0x53d274a3, SUC_lexicon_tags_30 },
    { 0x4bb274a4, SUC_lexicon_tags_2 },
    { 0x51274a4, SUC_lexicon_tags_30 },
    { 0x4b3274a6, SUC_lexicon_tags_47 },
    { 0xd9d274a7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x663274aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53274b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdef274b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9274b7, SUC_lexicon_tags_6 },
    { 0x23d274b8, SUC_lexicon_tags_12 },
    { 0x963274b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x733274bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41274bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b274c4, SUC_lexicon_tags_1254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143274c8, SUC_lexicon_tags_17 },
    { 0xe07274c9, SUC_lexicon_tags_15 },
    { 0xbef274c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253274ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf274d1, SUC_lexicon_tags_16 },
    { 0x42f274d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53d274d4, SUC_lexicon_tags_49 },
    { 0x33d274d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5274d8, SUC_lexicon_tags_47 },
    { 0x4eb274d8, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x42f274db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x341274de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585274e3, SUC_lexicon_tags_15 },
    { 0x36f274e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x12b274e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb29274e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b274ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b274f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89274f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x985274f6, SUC_lexicon_tags_15 },
    { 0xd89274f7, SUC_lexicon_tags_63 },
    { 0x729274f7, SUC_lexicon_tags_1 },
    { 0xe95274f7, SUC_lexicon_tags_7 },
    { 0x307274f8, SUC_lexicon_tags_60 },
    { 0xd07274fb, SUC_lexicon_tags_1 },
    { 0x339274f7, SUC_lexicon_tags_26 },
    { 0x485274fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14527504, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6327507, SUC_lexicon_tags_31 },
    { 0xa1527508, SUC_lexicon_tags_56 },
    { 0xd5927509, SUC_lexicon_tags_39 },
    { 0xa0b2750a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb92750d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe127511, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef27515, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b27518, SUC_lexicon_tags_17 },
    { 0xe927519, SUC_lexicon_tags_14 },
    { 0x2d27519, SUC_lexicon_tags_10 },
    { 0xf632751a, SUC_lexicon_tags_47 },
    { 0x11127519, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5372751e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae127522, SUC_lexicon_tags_13 },
    { 0xcf527523, SUC_lexicon_tags_37 },
    { 0xb2127522, SUC_lexicon_tags_7 },
    { 0xd727525, SUC_lexicon_tags_0 },
    { 0x14927526, SUC_lexicon_tags_103 },
    { 0x36327527, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa952752a, SUC_lexicon_tags_79 },
    { 0xb592752b, SUC_lexicon_tags_12 },
    { 0xc372752c, SUC_lexicon_tags_46 },
    { 0x2432752b, SUC_lexicon_tags_7 },
    { 0xe252752e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93527536, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb2753f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x74927541, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29727545, SUC_lexicon_tags_37 },
    { 0x74327546, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb2754a, SUC_lexicon_tags_8 },
    { 0x1292754b, SUC_lexicon_tags_37 },
    { 0xcc12754c, SUC_lexicon_tags_9 },
    { 0xb492754a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97727554, SUC_lexicon_tags_11 },
    { 0x4cb27555, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b2755c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89727560, SUC_lexicon_tags_17 },
    { 0x19527561, SUC_lexicon_tags_19 },
    { 0xd27560, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda527564, SUC_lexicon_tags_17 },
    { 0x14d27565, SUC_lexicon_tags_37 },
    { 0x2e727565, SUC_lexicon_tags_12 },
    { 0xb527566, SUC_lexicon_tags_6 },
    { 0x71127566, SUC_lexicon_tags_605 },
    { 0xddb27569, SUC_lexicon_tags_55 },
    { 0x4f52756a, SUC_lexicon_tags_66 },
    { 0x76d27569, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8127571, SUC_lexicon_tags_10 },
    { 0xa3727571, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5d27574, SUC_lexicon_tags_14 },
    { 0x1e127575, SUC_lexicon_tags_25 },
    { 0x5a527575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63127578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc012757a, SUC_lexicon_tags_9 },
    { 0x22f2757b, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12757e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db27584, SUC_lexicon_tags_25 },
    { 0xe6b27585, SUC_lexicon_tags_3 },
    { 0x97527586, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c52758a, SUC_lexicon_tags_45 },
    { 0x3dd2758b, SUC_lexicon_tags_11 },
    { 0x3592758c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc927591, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd27595, SUC_lexicon_tags_11 },
    { 0x36927596, SUC_lexicon_tags_60 },
    { 0xc4727597, SUC_lexicon_tags_6 },
    { 0xc2b27596, SUC_lexicon_tags_7 },
    { 0xe5b27599, SUC_lexicon_tags_65 },
    { 0xab727599, SUC_lexicon_tags_17 },
    { 0x2112759b, SUC_lexicon_tags_33 },
    { 0x1112759c, SUC_lexicon_tags_45 },
    { 0xc6927596, SUC_lexicon_tags_47 },
    { 0x9932759c, SUC_lexicon_tags_7 },
    { 0x1f92759f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x291275a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209275a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637275ac, SUC_lexicon_tags_28 },
    { 0xd3f275ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663275b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b275b4, SUC_lexicon_tags_23 },
    { 0x269275b4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4d7275b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd275ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25275c1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4d1275c3, SUC_lexicon_tags_79 },
    { 0x203275c3, SUC_lexicon_tags_2 },
    { 0xb07275c5, SUC_lexicon_tags_33 },
    { 0xb11275c5, SUC_lexicon_tags_47 },
    { 0xaf5275c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b1275c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9275cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x705275d0, SUC_lexicon_tags_38 },
    { 0xd91275d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd7275d3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b275d8, SUC_lexicon_tags_5 },
    { 0x40f275d8, SUC_lexicon_tags_47 },
    { 0x315275da, SUC_lexicon_tags_30 },
    { 0xd63275db, SUC_lexicon_tags_9 },
    { 0xb6d275d8, SUC_lexicon_tags_12 },
    { 0xd7275d9, SUC_lexicon_tags_4 },
    { 0x47b275de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d275e1, SUC_lexicon_tags_7 },
    { 0xf89275e1, SUC_lexicon_tags_68 },
    { 0x593275e3, SUC_lexicon_tags_66 },
    { 0xa1d275e4, SUC_lexicon_tags_9 },
    { 0xe5f275e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3275ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973275f1, SUC_lexicon_tags_65 },
    { 0x8eb275f2, SUC_lexicon_tags_39 },
    { 0x2f7275f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc63275f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885275fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x707275fc, SUC_lexicon_tags_17 },
    { 0x51b275fd, SUC_lexicon_tags_11 },
    { 0xc4f275fc, SUC_lexicon_tags_37 },
    { 0xa65275fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b27602, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b27609, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed27611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7727613, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2927615, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6127618, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xda12761b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5632761d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95927620, SUC_lexicon_tags_28 },
    { 0xfcb27620, SUC_lexicon_tags_23 },
    { 0xac127622, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9c927624, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e327628, SUC_lexicon_tags_9 },
    { 0xc0d27629, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6792762d, SUC_lexicon_tags_7 },
    { 0xfbb2762e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15527632, SUC_lexicon_tags_2 },
    { 0xb9327632, SUC_lexicon_tags_12 },
    { 0x15127632, SUC_lexicon_tags_37 },
    { 0xe8527635, SUC_lexicon_tags_17 },
    { 0x81127636, SUC_lexicon_tags_37 },
    { 0x8927632, SUC_lexicon_tags_60 },
    { 0x78f27638, SUC_lexicon_tags_27 },
    { 0x2c127639, SUC_lexicon_tags_63 },
    { 0x3d72763a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2763d, SUC_lexicon_tags_11 },
    { 0x9cb2763e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f27641, SUC_lexicon_tags_11 },
    { 0x85127641, SUC_lexicon_tags_7 },
    { 0x7927642, SUC_lexicon_tags_30 },
    { 0xa4527644, SUC_lexicon_tags_37 },
    { 0xead27644, SUC_lexicon_tags_32 },
    { 0x41527646, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdbf27648, SUC_lexicon_tags_17 },
    { 0x4f127649, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf2764c, SUC_lexicon_tags_1 },
    { 0xd192764d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdf2764f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a327652, SUC_lexicon_tags_11 },
    { 0x32327653, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b2765f, SUC_lexicon_tags_25 },
    { 0x91d27660, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab27665, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f2766a, SUC_lexicon_tags_44 },
    { 0xe4b2766a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31527671, SUC_lexicon_tags_31 },
    { 0x1e927672, SUC_lexicon_tags_84 },
    { 0x3b327673, SUC_lexicon_tags_17 },
    { 0x94b27672, SUC_lexicon_tags_23 },
    { 0xcf327675, SUC_lexicon_tags_2 },
    { 0x72f27675, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e127678, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5ab2767a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8752767d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe132767f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf527682, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d27685, SUC_lexicon_tags_10 },
    { 0x9f927686, SUC_lexicon_tags_25 },
    { 0x21f27687, SUC_lexicon_tags_47 },
    { 0xf1d27688, SUC_lexicon_tags_37 },
    { 0xcd327689, SUC_lexicon_tags_15 },
    { 0xe9f2768a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab2768d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8372768f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd27698, SUC_lexicon_tags_6 },
    { 0x87f27699, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x99f2769b, SUC_lexicon_tags_49 },
    { 0x4092769c, SUC_lexicon_tags_38 },
    { 0xefb2769b, SUC_lexicon_tags_1 },
    { 0xb0d2769e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5c9276a0, SUC_lexicon_tags_68 },
    { 0x63d276a0, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09276a6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa73276a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb9276aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57f276ac, SUC_lexicon_tags_11 },
    { 0x69d276ad, SUC_lexicon_tags_27 },
    { 0xf97276ae, SUC_lexicon_tags_47 },
    { 0x4a5276ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf73276b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd85276b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad276b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd53276b8, SUC_lexicon_tags_19 },
    { 0xe9d276b8, SUC_lexicon_tags_1465 },
    { 0x869276ba, SUC_lexicon_tags_113 },
    { 0xe8d276bb, SUC_lexicon_tags_33 },
    { 0x16d276bc, SUC_lexicon_tags_14 },
    { 0x853276bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f276c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b276c3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117276cb, SUC_lexicon_tags_11 },
    { 0x8b1276cb, SUC_lexicon_tags_25 },
    { 0xbef276cd, SUC_lexicon_tags_63 },
    { 0xbe1276ce, SUC_lexicon_tags_32 },
    { 0x847276cb, SUC_lexicon_tags_46 },
    { 0xb75276d0, SUC_lexicon_tags_12 },
    { 0x8a3276d1, SUC_lexicon_tags_68 },
    { 0xb81276cd, SUC_lexicon_tags_37 },
    { 0xabf276d3, SUC_lexicon_tags_27 },
    { 0x691276d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d276d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x977276db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x257276dd, SUC_lexicon_tags_15 },
    { 0xebb276dd, SUC_lexicon_tags_7 },
    { 0x3bf276de, SUC_lexicon_tags_33 },
    { 0xc61276e0, SUC_lexicon_tags_47 },
    { 0x5ab276e1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b276e4, SUC_lexicon_tags_37 },
    { 0x38f276e4, SUC_lexicon_tags_0 },
    { 0xb9d276e6, SUC_lexicon_tags_12 },
    { 0xa03276e7, SUC_lexicon_tags_3 },
    { 0xf5b276e8, SUC_lexicon_tags_34 },
    { 0xd0b276e9, SUC_lexicon_tags_14 },
    { 0xd25276e9, SUC_lexicon_tags_21 },
    { 0x98d276e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97276f1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x96d276f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15276f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0b276f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf276fb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b27706, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc327708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccf2770a, SUC_lexicon_tags_37 },
    { 0xcb52770b, SUC_lexicon_tags_0 },
    { 0xf852770c, SUC_lexicon_tags_39 },
    { 0x2192770d, SUC_lexicon_tags_37 },
    { 0x392770e, SUC_lexicon_tags_7 },
    { 0x2992770f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6d27711, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d727715, SUC_lexicon_tags_0 },
    { 0x7d327716, SUC_lexicon_tags_63 },
    { 0x6127716, SUC_lexicon_tags_12 },
    { 0xfb127717, SUC_lexicon_tags_124 },
    { 0xbed27719, SUC_lexicon_tags_1 },
    { 0xad2771a, SUC_lexicon_tags_2 },
    { 0x66d2771b, SUC_lexicon_tags_27 },
    { 0x7992771c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1327720, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3327723, SUC_lexicon_tags_15 },
    { 0x5bb27724, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc7d27726, SUC_lexicon_tags_37 },
    { 0x92d27726, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7292772b, SUC_lexicon_tags_2 },
    { 0xe0b2772c, SUC_lexicon_tags_15 },
    { 0xb4d2772c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf927730, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b27736, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbd127738, SUC_lexicon_tags_7 },
    { 0x33527739, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7492773d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f27740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb127743, SUC_lexicon_tags_9 },
    { 0xeff27744, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7527748, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2092774a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc552774d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f27752, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8ed27754, SUC_lexicon_tags_5 },
    { 0xb6d27755, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd27759, SUC_lexicon_tags_39 },
    { 0xc532775a, SUC_lexicon_tags_11 },
    { 0x5572775b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6512775e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb27762, SUC_lexicon_tags_48 },
    { 0x8eb27762, SUC_lexicon_tags_0 },
    { 0x3c327764, SUC_lexicon_tags_12 },
    { 0x45b27765, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb127769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b2776c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b27770, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a927779, SUC_lexicon_tags_33 },
    { 0x77927779, SUC_lexicon_tags_107 },
    { 0x1f72777a, SUC_lexicon_tags_23 },
    { 0xb292777b, SUC_lexicon_tags_82 },
    { 0x59b2777c, SUC_lexicon_tags_1317 },
    { 0, NULL },
    { 0x4d72777f, SUC_lexicon_tags_1 },
    { 0x22b27780, SUC_lexicon_tags_17 },
    { 0x35d27780, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b27787, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9472778b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0327798, SUC_lexicon_tags_2 },
    { 0xc9127799, SUC_lexicon_tags_7 },
    { 0x99b2779a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb832779d, SUC_lexicon_tags_1 },
    { 0x7f2779e, SUC_lexicon_tags_13 },
    { 0xcb52779e, SUC_lexicon_tags_27 },
    { 0xaab2779f, SUC_lexicon_tags_2 },
    { 0xbb52779f, SUC_lexicon_tags_14 },
    { 0xc1f277a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x381277a4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987277a8, SUC_lexicon_tags_11 },
    { 0xbd7277a9, SUC_lexicon_tags_9 },
    { 0x4d3277aa, SUC_lexicon_tags_11 },
    { 0x82b277ab, SUC_lexicon_tags_1 },
    { 0xab9277ac, SUC_lexicon_tags_17 },
    { 0xda7277ad, SUC_lexicon_tags_27 },
    { 0xaa3277ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd277b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599277b6, SUC_lexicon_tags_37 },
    { 0xc39277b6, SUC_lexicon_tags_14 },
    { 0xda1277b8, SUC_lexicon_tags_3 },
    { 0xe23277b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb277bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45277bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x945277c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa3277c3, SUC_lexicon_tags_26 },
    { 0x99f277c4, SUC_lexicon_tags_23 },
    { 0x853277c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d277ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d277cd, SUC_lexicon_tags_37 },
    { 0x789277ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f277d1, SUC_lexicon_tags_27 },
    { 0x45b277d2, SUC_lexicon_tags_10 },
    { 0x767277d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913277dd, SUC_lexicon_tags_8 },
    { 0x747277de, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21277e1, SUC_lexicon_tags_11 },
    { 0x269277e2, SUC_lexicon_tags_25 },
    { 0x107277e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85277e7, SUC_lexicon_tags_26 },
    { 0x59b277e8, SUC_lexicon_tags_0 },
    { 0x79d277e9, SUC_lexicon_tags_28 },
    { 0x8bd277e7, SUC_lexicon_tags_8 },
    { 0x65b277eb, SUC_lexicon_tags_37 },
    { 0xca3277ec, SUC_lexicon_tags_11 },
    { 0x709277ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5277f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46b277f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe13277f5, SUC_lexicon_tags_1 },
    { 0xb61277f6, SUC_lexicon_tags_12 },
    { 0x49f277f5, SUC_lexicon_tags_25 },
    { 0xdd9277f8, SUC_lexicon_tags_29 },
    { 0xeb1277f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4f277fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81277ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x24727801, SUC_lexicon_tags_11 },
    { 0xe6b27802, SUC_lexicon_tags_15 },
    { 0xb8527803, SUC_lexicon_tags_65 },
    { 0x4f27801, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68727808, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a92780a, SUC_lexicon_tags_11 },
    { 0x63d2780b, SUC_lexicon_tags_37 },
    { 0x9fb2780a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1672780f, SUC_lexicon_tags_154 },
    { 0x7c927810, SUC_lexicon_tags_43 },
    { 0x4352780f, SUC_lexicon_tags_70 },
    { 0xb8f27812, SUC_lexicon_tags_13 },
    { 0xbab27813, SUC_lexicon_tags_37 },
    { 0xdb27810, SUC_lexicon_tags_7 },
    { 0xe8d27815, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7252781b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc192781e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1527820, SUC_lexicon_tags_106 },
    { 0xdf27821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeed27823, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12127827, SUC_lexicon_tags_17 },
    { 0x3727828, SUC_lexicon_tags_27 },
    { 0x1bf27829, SUC_lexicon_tags_12 },
    { 0xdad2782a, SUC_lexicon_tags_17 },
    { 0xaeb27828, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df2782f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc927831, SUC_lexicon_tags_84 },
    { 0x33927831, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4527836, SUC_lexicon_tags_34 },
    { 0x57727837, SUC_lexicon_tags_17 },
    { 0x93f27838, SUC_lexicon_tags_15 },
    { 0x6f927838, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb2783f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4327844, SUC_lexicon_tags_7 },
    { 0xdcb27844, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f2784c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcdf2784e, SUC_lexicon_tags_26 },
    { 0x7412784e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb727852, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa8127854, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb27859, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda32785e, SUC_lexicon_tags_17 },
    { 0x1592785e, SUC_lexicon_tags_17 },
    { 0x9d72785f, SUC_lexicon_tags_47 },
    { 0x99727860, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25327866, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d27869, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8b12786b, SUC_lexicon_tags_17 },
    { 0x7dd2786b, SUC_lexicon_tags_38 },
    { 0x6292786d, SUC_lexicon_tags_65 },
    { 0xc5f2786e, SUC_lexicon_tags_32 },
    { 0x3d32786f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47327874, SUC_lexicon_tags_11 },
    { 0xd6727875, SUC_lexicon_tags_17 },
    { 0xd3b27876, SUC_lexicon_tags_9 },
    { 0xb5527874, SUC_lexicon_tags_27 },
    { 0xa8f27878, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d72787b, SUC_lexicon_tags_15 },
    { 0xef92787c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff27884, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed27888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2788d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73327890, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd27894, SUC_lexicon_tags_7 },
    { 0xa8527895, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94327899, SUC_lexicon_tags_11 },
    { 0x4992789a, SUC_lexicon_tags_125 },
    { 0x1272789a, SUC_lexicon_tags_12 },
    { 0xbb32789c, SUC_lexicon_tags_48 },
    { 0xe852789d, SUC_lexicon_tags_66 },
    { 0xde72789d, SUC_lexicon_tags_9 },
    { 0x8df2789d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf278a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb278a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa03278a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed278ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6b278ae, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7278b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10b278b9, SUC_lexicon_tags_25 },
    { 0x557278ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f278c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x745278c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279278c7, SUC_lexicon_tags_6 },
    { 0x24f278c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d278d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d278d4, SUC_lexicon_tags_0 },
    { 0x791278d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b278d7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf55278d9, SUC_lexicon_tags_55 },
    { 0xe0d278d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d7278dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8e7278de, SUC_lexicon_tags_37 },
    { 0x87d278de, SUC_lexicon_tags_61 },
    { 0xb35278e0, SUC_lexicon_tags_27 },
    { 0xbe1278e1, SUC_lexicon_tags_25 },
    { 0x549278e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f278ee, SUC_lexicon_tags_2 },
    { 0x747278ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977278f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b27900, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97327905, SUC_lexicon_tags_15 },
    { 0x9f927905, SUC_lexicon_tags_38 },
    { 0x7f927905, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5fd27909, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x34d2790b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x212790d, SUC_lexicon_tags_12 },
    { 0xa52790e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd0327910, SUC_lexicon_tags_0 },
    { 0x5dd27911, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7672791d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75327922, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e327924, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8cf27926, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8152792c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b27931, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27727933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0327939, SUC_lexicon_tags_33 },
    { 0xe312793a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a92793c, SUC_lexicon_tags_31 },
    { 0xded2793d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e327942, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x94d27944, SUC_lexicon_tags_85 },
    { 0xead27945, SUC_lexicon_tags_12 },
    { 0x9a127946, SUC_lexicon_tags_12 },
    { 0x61927945, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93327949, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee32794d, SUC_lexicon_tags_49 },
    { 0x6632794d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf27953, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x30727955, SUC_lexicon_tags_31 },
    { 0xfe927956, SUC_lexicon_tags_49 },
    { 0x21727956, SUC_lexicon_tags_1004 },
    { 0x11927956, SUC_lexicon_tags_25 },
    { 0x16b27959, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff2795c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d27964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52d27966, SUC_lexicon_tags_4 },
    { 0x5a927967, SUC_lexicon_tags_69 },
    { 0xad27968, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0x7cb2796a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792796e, SUC_lexicon_tags_0 },
    { 0xbd72796f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2127973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf27976, SUC_lexicon_tags_34 },
    { 0x8af27977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc752797b, SUC_lexicon_tags_23 },
    { 0x7e12797c, SUC_lexicon_tags_81 },
    { 0xb3b2797d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea527980, SUC_lexicon_tags_14 },
    { 0x76f27980, SUC_lexicon_tags_2 },
    { 0xf1727981, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af27985, SUC_lexicon_tags_12 },
    { 0xd1527986, SUC_lexicon_tags_15 },
    { 0x5a127987, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd812798c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc327992, SUC_lexicon_tags_9 },
    { 0x8927992, SUC_lexicon_tags_8 },
    { 0x61d27993, SUC_lexicon_tags_14 },
    { 0xf2127995, SUC_lexicon_tags_46 },
    { 0xaa327996, SUC_lexicon_tags_46 },
    { 0x2f527997, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf692799a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9279a1, SUC_lexicon_tags_37 },
    { 0x1a3279a1, SUC_lexicon_tags_19 },
    { 0x68f279a3, SUC_lexicon_tags_38 },
    { 0x5c7279a4, SUC_lexicon_tags_2 },
    { 0xe49279a1, SUC_lexicon_tags_25 },
    { 0xbed279a5, SUC_lexicon_tags_60 },
    { 0x979279a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b279ac, SUC_lexicon_tags_38 },
    { 0xffd279ad, SUC_lexicon_tags_14 },
    { 0x3d3279ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3279b2, SUC_lexicon_tags_12 },
    { 0xc71279b2, SUC_lexicon_tags_1 },
    { 0x83b279b2, SUC_lexicon_tags_60 },
    { 0x611279b5, SUC_lexicon_tags_208 },
    { 0xf5b279b6, SUC_lexicon_tags_23 },
    { 0xc99279b7, SUC_lexicon_tags_21 },
    { 0x1a7279b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25279bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x939279bf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xecf279c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec5279c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7279c8, SUC_lexicon_tags_49 },
    { 0x6b3279c8, SUC_lexicon_tags_44 },
    { 0xf75279ca, SUC_lexicon_tags_38 },
    { 0xb8d279ca, SUC_lexicon_tags_1 },
    { 0xabf279c8, SUC_lexicon_tags_0 },
    { 0xcd7279cd, SUC_lexicon_tags_43 },
    { 0x367279ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f279d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f279d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743279dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101279e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5279e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3279ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9279f2, SUC_lexicon_tags_15 },
    { 0x271279f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcef279f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15279f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd93279f9, SUC_lexicon_tags_7 },
    { 0xa2b279fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b9279fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b127a01, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab27a05, SUC_lexicon_tags_0 },
    { 0x47b27a06, SUC_lexicon_tags_7 },
    { 0xa5527a07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4bb27a09, SUC_lexicon_tags_7 },
    { 0x7b527a09, SUC_lexicon_tags_14 },
    { 0x93127a0b, SUC_lexicon_tags_2 },
    { 0xa4127a0b, SUC_lexicon_tags_11 },
    { 0xfe327a0b, SUC_lexicon_tags_43 },
    { 0xed727a0b, SUC_lexicon_tags_32 },
    { 0xb5f27a0e, SUC_lexicon_tags_19 },
    { 0x30d27a10, SUC_lexicon_tags_56 },
    { 0x32d27a0b, SUC_lexicon_tags_63 },
    { 0xf3f27a10, SUC_lexicon_tags_37 },
    { 0xba927a13, SUC_lexicon_tags_25 },
    { 0xab127a13, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52327a22, SUC_lexicon_tags_17 },
    { 0x28b27a23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebf27a25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x43727a27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b27a2a, SUC_lexicon_tags_46 },
    { 0xbdb27a2b, SUC_lexicon_tags_16 },
    { 0x69927a2b, SUC_lexicon_tags_12 },
    { 0x42127a2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb27a32, SUC_lexicon_tags_15 },
    { 0xfe327a33, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d327a3a, SUC_lexicon_tags_47 },
    { 0xaf327a3a, SUC_lexicon_tags_15 },
    { 0x9ef27a3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3527a3f, SUC_lexicon_tags_15 },
    { 0x88f27a40, SUC_lexicon_tags_1 },
    { 0x13127a41, SUC_lexicon_tags_2 },
    { 0x98f27a41, SUC_lexicon_tags_37 },
    { 0xae527a43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db27a46, SUC_lexicon_tags_11 },
    { 0x25127a46, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc7327a49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb27a4c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x45727a4f, SUC_lexicon_tags_6 },
    { 0x80d27a50, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb927a55, SUC_lexicon_tags_17 },
    { 0x6ad27a55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb927a59, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c527a5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8127a62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd2127a64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b27a67, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0xc2927a69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61927a70, SUC_lexicon_tags_37 },
    { 0xbe127a71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b27a74, SUC_lexicon_tags_4 },
    { 0x48d27a75, SUC_lexicon_tags_14 },
    { 0xb8127a76, SUC_lexicon_tags_28 },
    { 0xd7327a76, SUC_lexicon_tags_12 },
    { 0x6b927a78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x28727a7b, SUC_lexicon_tags_19 },
    { 0x9db27a7c, SUC_lexicon_tags_1 },
    { 0x81d27a7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b27a80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d727a87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b27a8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab927a8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f27a95, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5727a99, SUC_lexicon_tags_70 },
    { 0x2d727a9a, SUC_lexicon_tags_84 },
    { 0xad327a99, SUC_lexicon_tags_40 },
    { 0x58f27a9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38927a9f, SUC_lexicon_tags_17 },
    { 0x2dd27aa0, SUC_lexicon_tags_27 },
    { 0x21b27aa1, SUC_lexicon_tags_25 },
    { 0x56727aa2, SUC_lexicon_tags_30 },
    { 0x7c527a9f, SUC_lexicon_tags_6 },
    { 0xe2b27aa4, SUC_lexicon_tags_2 },
    { 0x65927aa4, SUC_lexicon_tags_17 },
    { 0xf4727aa6, SUC_lexicon_tags_28 },
    { 0xedb27aa7, SUC_lexicon_tags_0 },
    { 0x5b727aa2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb927aac, SUC_lexicon_tags_198 },
    { 0x14b27aac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d27ab2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x59327ab4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c927ab8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5327abc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x55327abe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7127ac0, SUC_lexicon_tags_7 },
    { 0xbfd27ac1, SUC_lexicon_tags_1 },
    { 0xc6927ac2, SUC_lexicon_tags_9 },
    { 0xb1927ac2, SUC_lexicon_tags_7 },
    { 0xefd27ac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12127ac8, SUC_lexicon_tags_9 },
    { 0x25527ac9, SUC_lexicon_tags_27 },
    { 0x28f27aca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30727ace, SUC_lexicon_tags_21 },
    { 0x90b27acf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1e527ad1, SUC_lexicon_tags_15 },
    { 0x54327ad2, SUC_lexicon_tags_2 },
    { 0xa1527ad3, SUC_lexicon_tags_15 },
    { 0x97927ad4, SUC_lexicon_tags_29 },
    { 0x5b927ad5, SUC_lexicon_tags_11 },
    { 0x3ad27ad3, SUC_lexicon_tags_25 },
    { 0x9d27ad4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x48727ada, SUC_lexicon_tags_7 },
    { 0x50d27ada, SUC_lexicon_tags_66 },
    { 0x6b127adb, SUC_lexicon_tags_37 },
    { 0x8bf27adb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d27ae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10327ae6, SUC_lexicon_tags_1 },
    { 0x81527ae6, SUC_lexicon_tags_25 },
    { 0x49b27ae8, SUC_lexicon_tags_16 },
    { 0x13f27ae8, SUC_lexicon_tags_7 },
    { 0xc2b27ae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff727aec, SUC_lexicon_tags_10 },
    { 0xf1527aec, SUC_lexicon_tags_2 },
    { 0x1d927aed, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9727af1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19327af6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa127afb, SUC_lexicon_tags_25 },
    { 0x60b27afb, SUC_lexicon_tags_25 },
    { 0x37527afd, SUC_lexicon_tags_1 },
    { 0xaef27afc, SUC_lexicon_tags_38 },
    { 0xd2127afd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2527b04, SUC_lexicon_tags_77 },
    { 0xad527b05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7727b08, SUC_lexicon_tags_12 },
    { 0x50d27b09, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd27b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba727b10, SUC_lexicon_tags_10 },
    { 0xe9527b11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xee727b14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x62b27b16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8327b1c, SUC_lexicon_tags_13 },
    { 0x85327b1c, SUC_lexicon_tags_6 },
    { 0x74d27b1e, SUC_lexicon_tags_33 },
    { 0x83527b1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4527b24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76927b26, SUC_lexicon_tags_2 },
    { 0x2df27b27, SUC_lexicon_tags_27 },
    { 0xf7927b28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3f927b2a, SUC_lexicon_tags_33 },
    { 0xfd727b2a, SUC_lexicon_tags_14 },
    { 0x42d27b2b, SUC_lexicon_tags_0 },
    { 0x5e327b2b, SUC_lexicon_tags_29 },
    { 0x9b127b2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f27b32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad27b37, SUC_lexicon_tags_17 },
    { 0x12527b38, SUC_lexicon_tags_1 },
    { 0x33d27b39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d327b47, SUC_lexicon_tags_48 },
    { 0xb9327b48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1727b4c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd27b57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a727b59, SUC_lexicon_tags_30 },
    { 0x6c927b5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb6527b5c, SUC_lexicon_tags_43 },
    { 0x1df27b5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb127b5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64927b64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d27b67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12527b69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6527b6d, SUC_lexicon_tags_1 },
    { 0x7af27b6e, SUC_lexicon_tags_14 },
    { 0xb127b6f, SUC_lexicon_tags_39 },
    { 0x28d27b6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d27b76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd8927b78, SUC_lexicon_tags_43 },
    { 0x25127b79, SUC_lexicon_tags_12 },
    { 0xf9327b7a, SUC_lexicon_tags_16 },
    { 0xdb927b7b, SUC_lexicon_tags_37 },
    { 0x28d27b7c, SUC_lexicon_tags_65 },
    { 0x3df27b7d, SUC_lexicon_tags_23 },
    { 0x10927b7e, SUC_lexicon_tags_12 },
    { 0x7727b7f, SUC_lexicon_tags_11 },
    { 0x5dd27b78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84127b83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b27b87, SUC_lexicon_tags_28 },
    { 0xec327b87, SUC_lexicon_tags_746 },
    { 0xf5b27b87, SUC_lexicon_tags_4 },
    { 0xab127b8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f27b8e, SUC_lexicon_tags_25 },
    { 0xd4527b8f, SUC_lexicon_tags_11 },
    { 0x12527b8f, SUC_lexicon_tags_23 },
    { 0x40927b91, SUC_lexicon_tags_19 },
    { 0x6f327b91, SUC_lexicon_tags_48 },
    { 0x35d27b91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7327b96, SUC_lexicon_tags_7 },
    { 0x6f27b97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f727b9a, SUC_lexicon_tags_25 },
    { 0x6d927b9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae727ba0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4927ba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0527ba8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d27bac, SUC_lexicon_tags_13 },
    { 0x33527bad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9727bb3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b27bb6, SUC_lexicon_tags_14 },
    { 0xa927bb7, SUC_lexicon_tags_63 },
    { 0x6b527bb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47d27bba, SUC_lexicon_tags_47 },
    { 0x1cf27bba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67727bbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e927bc3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x81d27bc5, SUC_lexicon_tags_35 },
    { 0x13527bc6, SUC_lexicon_tags_14 },
    { 0x7d327bc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a127bcd, SUC_lexicon_tags_25 },
    { 0x81f27bce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb27bd1, SUC_lexicon_tags_4 },
    { 0x42127bd2, SUC_lexicon_tags_0 },
    { 0xe0b27bd3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x64f27bd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98727bd7, SUC_lexicon_tags_45 },
    { 0x3fb27bd7, SUC_lexicon_tags_80 },
    { 0xc8527bd9, SUC_lexicon_tags_15 },
    { 0xdef27bd7, SUC_lexicon_tags_47 },
    { 0x1fb27bdb, SUC_lexicon_tags_12 },
    { 0x32327bdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x58b27bde, SUC_lexicon_tags_2 },
    { 0x46727bdf, SUC_lexicon_tags_832 },
    { 0x46b27be0, SUC_lexicon_tags_15 },
    { 0xb6927be1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b27be4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x37927be7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x48527bea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6327bf0, SUC_lexicon_tags_11 },
    { 0x9d527bf0, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xfb27bf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26127bf5, SUC_lexicon_tags_14 },
    { 0xb4b27bf6, SUC_lexicon_tags_0 },
    { 0x29f27bf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed927bfd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2927c03, SUC_lexicon_tags_12 },
    { 0x14b27c04, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd27c0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52f27c0f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x7fb27c11, SUC_lexicon_tags_9 },
    { 0x56127c12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10b27c14, SUC_lexicon_tags_37 },
    { 0xf527c14, SUC_lexicon_tags_33 },
    { 0x18f27c15, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde327c1f, SUC_lexicon_tags_19 },
    { 0x7a327c1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18127c22, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff27c25, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x64127c28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf27c2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95327c39, SUC_lexicon_tags_47 },
    { 0x31327c3a, SUC_lexicon_tags_14 },
    { 0xe8927c3b, SUC_lexicon_tags_30 },
    { 0xaf27c3c, SUC_lexicon_tags_65 },
    { 0x29b27c3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd27c42, SUC_lexicon_tags_25 },
    { 0x4f527c43, SUC_lexicon_tags_23 },
    { 0xccd27c43, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b27c48, SUC_lexicon_tags_47 },
    { 0x78127c48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22527c4c, SUC_lexicon_tags_54 },
    { 0x88527c4c, SUC_lexicon_tags_2 },
    { 0xfb327c4e, SUC_lexicon_tags_49 },
    { 0xc2327c4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd527c54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28727c56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29927c5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58127c5e, SUC_lexicon_tags_2 },
    { 0x74727c5f, SUC_lexicon_tags_45 },
    { 0x13527c60, SUC_lexicon_tags_38 },
    { 0xa3d27c5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81f27c63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea127c68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e927c6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb127c75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f27c7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab27c82, SUC_lexicon_tags_17 },
    { 0x3e927c83, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x57927c86, SUC_lexicon_tags_37 },
    { 0x55127c87, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c927c8a, SUC_lexicon_tags_1 },
    { 0xa7127c8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbeb27c8d, SUC_lexicon_tags_12 },
    { 0xe6b27c8e, SUC_lexicon_tags_6 },
    { 0xa6b27c8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5127c98, SUC_lexicon_tags_16 },
    { 0x8e527c98, SUC_lexicon_tags_10 },
    { 0xd8727c9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d27ca0, SUC_lexicon_tags_37 },
    { 0x8d927ca0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed27ca5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc6727ca7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d527cae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc327cb2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef27cb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97f27cb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcb27cb9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f27cbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9727cbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2927cc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa6f27cc4, SUC_lexicon_tags_7 },
    { 0x5c727cc5, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d27cc8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3e927cca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b327ccc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a327cd0, SUC_lexicon_tags_47 },
    { 0x21327cd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0527cd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10327cd6, SUC_lexicon_tags_8 },
    { 0x16327cd7, SUC_lexicon_tags_17 },
    { 0xdb27cd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0f27cda, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a327cdd, SUC_lexicon_tags_39 },
    { 0x83927cde, SUC_lexicon_tags_25 },
    { 0xaa927cdf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd927ce4, SUC_lexicon_tags_2 },
    { 0xdf127ce4, SUC_lexicon_tags_33 },
    { 0xf3f27ce5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5527cea, SUC_lexicon_tags_27 },
    { 0x6b927ceb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfd27ced, SUC_lexicon_tags_12 },
    { 0xec127cee, SUC_lexicon_tags_2 },
    { 0x62927ced, SUC_lexicon_tags_11 },
    { 0x8bf27cf0, SUC_lexicon_tags_25 },
    { 0x7f327cf1, SUC_lexicon_tags_38 },
    { 0xf6b27cee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8927cf6, SUC_lexicon_tags_38 },
    { 0x8ff27cf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2727cfa, SUC_lexicon_tags_2 },
    { 0x1e327cfb, SUC_lexicon_tags_47 },
    { 0xa7127cfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b27cff, SUC_lexicon_tags_14 },
    { 0xba127cff, SUC_lexicon_tags_2 },
    { 0xf3d27d00, SUC_lexicon_tags_33 },
    { 0xb8127d02, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x98327d04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f27d09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb27d0f, SUC_lexicon_tags_8 },
    { 0xbd527d10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93b27d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95127d14, SUC_lexicon_tags_38 },
    { 0x42527d14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7327d19, SUC_lexicon_tags_1011 },
    { 0, NULL },
    { 0x90527d1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d27d1e, SUC_lexicon_tags_37 },
    { 0x70b27d1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b27d22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30327d24, SUC_lexicon_tags_27 },
    { 0xe3927d25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63927d29, SUC_lexicon_tags_2 },
    { 0x54927d29, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb2d27d2c, SUC_lexicon_tags_18 },
    { 0x6d327d2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fb27d2f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xacb27d31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa8327d33, SUC_lexicon_tags_9 },
    { 0xd7127d33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc5527d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31327d38, SUC_lexicon_tags_1 },
    { 0xeb727d38, SUC_lexicon_tags_11 },
    { 0xa9127d3a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d27d3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb2727d3f, SUC_lexicon_tags_12 },
    { 0xc5327d40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdb27d42, SUC_lexicon_tags_15 },
    { 0x86d27d43, SUC_lexicon_tags_7 },
    { 0x46527d43, SUC_lexicon_tags_17 },
    { 0xfb327d43, SUC_lexicon_tags_11 },
    { 0xea127d45, SUC_lexicon_tags_27 },
    { 0x51327d44, SUC_lexicon_tags_15 },
    { 0x7bf27d48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b27d4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39527d4e, SUC_lexicon_tags_12 },
    { 0x5d927d4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f127d53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b27d56, SUC_lexicon_tags_89 },
    { 0xff127d57, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5eb27d59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f27d5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f27d64, SUC_lexicon_tags_30 },
    { 0x28b27d64, SUC_lexicon_tags_25 },
    { 0x38127d65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd5927d68, SUC_lexicon_tags_6 },
    { 0x11f27d69, SUC_lexicon_tags_47 },
    { 0x33927d6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3327d70, SUC_lexicon_tags_11 },
    { 0xf127d71, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4b727d73, SUC_lexicon_tags_49 },
    { 0x3a327d74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5527d76, SUC_lexicon_tags_31 },
    { 0xa2727d77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f27d86, SUC_lexicon_tags_11 },
    { 0xa2f27d87, SUC_lexicon_tags_15 },
    { 0x3f27d88, SUC_lexicon_tags_46 },
    { 0x8ed27d88, SUC_lexicon_tags_12 },
    { 0x89727d88, SUC_lexicon_tags_6 },
    { 0x79527d8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e727d8d, SUC_lexicon_tags_14 },
    { 0x52d27d8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8f27d90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd127d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14527da1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f27da5, SUC_lexicon_tags_11 },
    { 0x96527da6, SUC_lexicon_tags_47 },
    { 0x54727da5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d27dac, SUC_lexicon_tags_17 },
    { 0xe4327dad, SUC_lexicon_tags_12 },
    { 0x85927dae, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x4327db0, SUC_lexicon_tags_15 },
    { 0xb927db1, SUC_lexicon_tags_33 },
    { 0xeb727db1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7127db5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77127dba, SUC_lexicon_tags_12 },
    { 0xd5527dba, SUC_lexicon_tags_0 },
    { 0x6df27dbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2f27dbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d27dc2, SUC_lexicon_tags_63 },
    { 0x12127dc2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2727dc8, SUC_lexicon_tags_37 },
    { 0x3a327dc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf727dd5, SUC_lexicon_tags_60 },
    { 0x14727dd5, SUC_lexicon_tags_19 },
    { 0x74527dd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f27ddb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d527de0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5a127de2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8527de5, SUC_lexicon_tags_40 },
    { 0x5bd27de6, SUC_lexicon_tags_37 },
    { 0xa2d27de7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x27727de9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d27dec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2927def, SUC_lexicon_tags_37 },
    { 0xd1327def, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30327df2, SUC_lexicon_tags_16 },
    { 0x80727df2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d27df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6927dfb, SUC_lexicon_tags_37 },
    { 0x9ad27dfb, SUC_lexicon_tags_9 },
    { 0x6927dfd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x48127dff, SUC_lexicon_tags_12 },
    { 0xce727dff, SUC_lexicon_tags_49 },
    { 0x68127e01, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd8527e03, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9fd27e05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f727e08, SUC_lexicon_tags_27 },
    { 0x21327e08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65b27e0b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x49727e0d, SUC_lexicon_tags_6 },
    { 0x38727e0e, SUC_lexicon_tags_37 },
    { 0xc0727e0f, SUC_lexicon_tags_0 },
    { 0x11f27e0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7527e13, SUC_lexicon_tags_49 },
    { 0x90127e13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4327e16, SUC_lexicon_tags_2 },
    { 0x25f27e17, SUC_lexicon_tags_55 },
    { 0xc327e18, SUC_lexicon_tags_1 },
    { 0xa5b27e19, SUC_lexicon_tags_12 },
    { 0xc1527e1a, SUC_lexicon_tags_11 },
    { 0xc4927e1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f27e1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17127e28, SUC_lexicon_tags_10 },
    { 0x94527e29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee927e2d, SUC_lexicon_tags_14 },
    { 0x7fd27e2e, SUC_lexicon_tags_25 },
    { 0xd27e2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8927e37, SUC_lexicon_tags_6 },
    { 0xdf327e37, SUC_lexicon_tags_45 },
    { 0x62727e37, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17527e3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d27e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5927e44, SUC_lexicon_tags_17 },
    { 0xf9127e45, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2d927e47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab27e4a, SUC_lexicon_tags_33 },
    { 0xcb927e4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f27e4f, SUC_lexicon_tags_1 },
    { 0x5dd27e50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a927e55, SUC_lexicon_tags_22 },
    { 0x37327e56, SUC_lexicon_tags_1 },
    { 0x7c927e57, SUC_lexicon_tags_9 },
    { 0x54727e58, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x69527e5a, SUC_lexicon_tags_25 },
    { 0x62d27e5a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x15327e5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f27e63, SUC_lexicon_tags_1 },
    { 0x1e327e63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d327e6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5327e6e, SUC_lexicon_tags_27 },
    { 0xee927e6f, SUC_lexicon_tags_25 },
    { 0x64527e70, SUC_lexicon_tags_140 },
    { 0xdb27e6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9327e73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdeb27e75, SUC_lexicon_tags_83 },
    { 0xcd527e76, SUC_lexicon_tags_11 },
    { 0xa6327e76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90727e79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xad27e7b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff27e7e, SUC_lexicon_tags_12 },
    { 0xbed27e7f, SUC_lexicon_tags_63 },
    { 0x81327e80, SUC_lexicon_tags_38 },
    { 0xdd327e81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed27e87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3d27e89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5327e8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab27e93, SUC_lexicon_tags_17 },
    { 0x26d27e94, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f27e97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaab27e99, SUC_lexicon_tags_37 },
    { 0xda327e9a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb727e9f, SUC_lexicon_tags_26 },
    { 0x87927ea0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2927ea6, SUC_lexicon_tags_11 },
    { 0x78327ea7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b27eaa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5527eb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1327eb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed327ebc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b27ebf, SUC_lexicon_tags_37 },
    { 0x79727ebf, SUC_lexicon_tags_2 },
    { 0xc5727ec1, SUC_lexicon_tags_7 },
    { 0xb3127ec1, SUC_lexicon_tags_30 },
    { 0xed327ebf, SUC_lexicon_tags_12 },
    { 0x6e127ec2, SUC_lexicon_tags_14 },
    { 0xd7127ec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f27ecd, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x32927ecf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b27ed4, SUC_lexicon_tags_47 },
    { 0xfdf27ed5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x15927ed8, SUC_lexicon_tags_69 },
    { 0x70727ed9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a127edb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x50127ede, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3327ee3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b527ee7, SUC_lexicon_tags_25 },
    { 0xc3127ee7, SUC_lexicon_tags_11 },
    { 0x6c927ee9, SUC_lexicon_tags_57 },
    { 0x81b27ee9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5fb27eec, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d27ef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3127ef7, SUC_lexicon_tags_7 },
    { 0x75927ef7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd27efb, SUC_lexicon_tags_28 },
    { 0x67127efc, SUC_lexicon_tags_17 },
    { 0x1a727efc, SUC_lexicon_tags_9 },
    { 0x12927efd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31927f02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b527f04, SUC_lexicon_tags_1 },
    { 0xf3f27f05, SUC_lexicon_tags_10 },
    { 0xfb727f06, SUC_lexicon_tags_25 },
    { 0xbdb27f07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78d27f09, SUC_lexicon_tags_17 },
    { 0xa1527f0a, SUC_lexicon_tags_25 },
    { 0xa0b27f0b, SUC_lexicon_tags_9 },
    { 0xd5727f0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f27f17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e327f1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28927f1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37b27f21, SUC_lexicon_tags_8 },
    { 0xc8d27f22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8b27f24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xff127f26, SUC_lexicon_tags_25 },
    { 0x27727f26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1527f2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82d27f31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad27f34, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e527f39, SUC_lexicon_tags_11 },
    { 0x48d27f3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe127f40, SUC_lexicon_tags_6 },
    { 0xba127f41, SUC_lexicon_tags_1 },
    { 0xf9527f42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f27f45, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2db27f47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf27f4d, SUC_lexicon_tags_4 },
    { 0xcf327f4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88727f56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xed27f59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89127f5d, SUC_lexicon_tags_17 },
    { 0xb1327f5d, SUC_lexicon_tags_38 },
    { 0x80127f5d, SUC_lexicon_tags_25 },
    { 0xf727f5e, SUC_lexicon_tags_25 },
    { 0xdd127f61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18327f65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327f69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d727f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef27f72, SUC_lexicon_tags_12 },
    { 0x7b927f72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50927f78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc127f7b, SUC_lexicon_tags_49 },
    { 0x2f27f7c, SUC_lexicon_tags_1 },
    { 0x6ab27f7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4727f7f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b27f89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41127f8d, SUC_lexicon_tags_1 },
    { 0x91127f8e, SUC_lexicon_tags_16 },
    { 0xa7327f8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99327f91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae727f93, SUC_lexicon_tags_25 },
    { 0xccb27f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34327f96, SUC_lexicon_tags_15 },
    { 0x5f327f97, SUC_lexicon_tags_70 },
    { 0x3df27f98, SUC_lexicon_tags_19 },
    { 0x9f727f99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b27f9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39327fa2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd27fa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e127faa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d27fac, SUC_lexicon_tags_15 },
    { 0x5a127fad, SUC_lexicon_tags_25 },
    { 0x3c927fae, SUC_lexicon_tags_39 },
    { 0xe0927faf, SUC_lexicon_tags_9 },
    { 0x91b27fac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a327fb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd27fbb, SUC_lexicon_tags_17 },
    { 0xd3927fbc, SUC_lexicon_tags_11 },
    { 0xc6327fbd, SUC_lexicon_tags_11 },
    { 0xfa327fbd, SUC_lexicon_tags_33 },
    { 0xb0b27fbf, SUC_lexicon_tags_37 },
    { 0xde327fbf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4127fc2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xba127fc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaeb27fc7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7d727fc9, SUC_lexicon_tags_31 },
    { 0xcc927fc9, SUC_lexicon_tags_25 },
    { 0x54b27fcb, SUC_lexicon_tags_8 },
    { 0xe0727fcb, SUC_lexicon_tags_43 },
    { 0xa9b27fcc, SUC_lexicon_tags_15 },
    { 0x73327fce, SUC_lexicon_tags_15 },
    { 0x54d27fcf, SUC_lexicon_tags_34 },
    { 0x58127fd0, SUC_lexicon_tags_0 },
    { 0x4727fca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f27fd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17927fda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66927fdf, SUC_lexicon_tags_82 },
    { 0xe1327fdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5527fe3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b27fe8, SUC_lexicon_tags_37 },
    { 0x11b27fe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5327fed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd27ff0, SUC_lexicon_tags_33 },
    { 0xeef27ff1, SUC_lexicon_tags_118 },
    { 0xe6927ff2, SUC_lexicon_tags_14 },
    { 0xdcf27ff3, SUC_lexicon_tags_27 },
    { 0x20727ff4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d27ff7, SUC_lexicon_tags_30 },
    { 0xc0927ff7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb27ffb, SUC_lexicon_tags_25 },
    { 0x2c327ffc, SUC_lexicon_tags_23 },
    { 0xb9927ffd, SUC_lexicon_tags_33 },
    { 0x5bd27ffe, SUC_lexicon_tags_37 },
    { 0x20127ffc, SUC_lexicon_tags_51 },
    { 0x44b28000, SUC_lexicon_tags_25 },
    { 0xd1528000, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x77928003, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd28006, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db28011, SUC_lexicon_tags_15 },
    { 0xc5b28012, SUC_lexicon_tags_85 },
    { 0xcf128012, SUC_lexicon_tags_31 },
    { 0x14128013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5212801a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7232801e, SUC_lexicon_tags_37 },
    { 0xb92801f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x69b28021, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a328024, SUC_lexicon_tags_39 },
    { 0xf0128025, SUC_lexicon_tags_7 },
    { 0x6a128026, SUC_lexicon_tags_42 },
    { 0x9db28027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe392802c, SUC_lexicon_tags_6 },
    { 0x12802d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x72f2802f, SUC_lexicon_tags_61 },
    { 0x6292802f, SUC_lexicon_tags_1 },
    { 0xc0728031, SUC_lexicon_tags_33 },
    { 0xa9128032, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xee728034, SUC_lexicon_tags_25 },
    { 0x39b28035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8728043, SUC_lexicon_tags_8 },
    { 0x76328044, SUC_lexicon_tags_37 },
    { 0x66728045, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f128048, SUC_lexicon_tags_2 },
    { 0x81328049, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcf12804b, SUC_lexicon_tags_11 },
    { 0x5112804c, SUC_lexicon_tags_37 },
    { 0xc0f2804d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7612804f, SUC_lexicon_tags_15 },
    { 0xdc328050, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db28053, SUC_lexicon_tags_21 },
    { 0x16328054, SUC_lexicon_tags_37 },
    { 0x78328055, SUC_lexicon_tags_23 },
    { 0x22b28056, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d2805b, SUC_lexicon_tags_25 },
    { 0xfa32805c, SUC_lexicon_tags_37 },
    { 0xf852805d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f2806c, SUC_lexicon_tags_25 },
    { 0x8db2806c, SUC_lexicon_tags_66 },
    { 0x4f72806e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4128070, SUC_lexicon_tags_15 },
    { 0x1a928070, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd5b28073, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1528075, SUC_lexicon_tags_0 },
    { 0x78d28076, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfe328078, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x81f2807a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b2807d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79128083, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51328088, SUC_lexicon_tags_15 },
    { 0xc8b28089, SUC_lexicon_tags_7 },
    { 0x8d28089, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc3f2808c, SUC_lexicon_tags_85 },
    { 0xfdd2808d, SUC_lexicon_tags_79 },
    { 0x672808e, SUC_lexicon_tags_28 },
    { 0x2472808e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88728096, SUC_lexicon_tags_7 },
    { 0xadb28097, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a12809a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd2809c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x672809f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f280a5, SUC_lexicon_tags_14 },
    { 0xdfb280a6, SUC_lexicon_tags_37 },
    { 0x463280a7, SUC_lexicon_tags_29 },
    { 0x8cb280a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d280b4, SUC_lexicon_tags_7 },
    { 0x48f280b5, SUC_lexicon_tags_7 },
    { 0x3bd280b5, SUC_lexicon_tags_38 },
    { 0x5a1280b7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45280bd, SUC_lexicon_tags_37 },
    { 0x711280be, SUC_lexicon_tags_27 },
    { 0xc47280bd, SUC_lexicon_tags_2 },
    { 0x65b280c0, SUC_lexicon_tags_2 },
    { 0xc7280be, SUC_lexicon_tags_37 },
    { 0xa7f280c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf280c7, SUC_lexicon_tags_25 },
    { 0x505280c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb280cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fd280cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85280d0, SUC_lexicon_tags_63 },
    { 0x319280d1, SUC_lexicon_tags_85 },
    { 0x4c7280d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29280d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31280db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3b9280dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3280e0, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95280e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x305280e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05280e8, SUC_lexicon_tags_29 },
    { 0x349280e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd280ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd9280f1, SUC_lexicon_tags_79 },
    { 0x3ad280f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f280f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa03280f8, SUC_lexicon_tags_37 },
    { 0x1e7280f9, SUC_lexicon_tags_1 },
    { 0x7f280f8, SUC_lexicon_tags_17 },
    { 0x29f280fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6728102, SUC_lexicon_tags_42 },
    { 0x9eb28103, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6528107, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9072810a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b2810d, SUC_lexicon_tags_37 },
    { 0x8412810d, SUC_lexicon_tags_9 },
    { 0x8cb2810f, SUC_lexicon_tags_17 },
    { 0xc712810f, SUC_lexicon_tags_37 },
    { 0xe6528111, SUC_lexicon_tags_23 },
    { 0xb3b28111, SUC_lexicon_tags_15 },
    { 0x43528113, SUC_lexicon_tags_17 },
    { 0x5eb28114, SUC_lexicon_tags_12 },
    { 0x73f28112, SUC_lexicon_tags_12 },
    { 0x9eb28115, SUC_lexicon_tags_6 },
    { 0x24b28117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5328119, SUC_lexicon_tags_27 },
    { 0x70f2811a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe432811e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85328123, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f28129, SUC_lexicon_tags_87 },
    { 0x96d2812a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd932812d, SUC_lexicon_tags_25 },
    { 0x9b92812d, SUC_lexicon_tags_27 },
    { 0x68f2812f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a728135, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f2813c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x532813f, SUC_lexicon_tags_11 },
    { 0x3632813f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38128145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34f28147, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9d928149, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b28150, SUC_lexicon_tags_0 },
    { 0x63528151, SUC_lexicon_tags_27 },
    { 0x2d328152, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b528154, SUC_lexicon_tags_37 },
    { 0xb2128154, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d2815a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d2815e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa0b28160, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd28163, SUC_lexicon_tags_37 },
    { 0x5c528163, SUC_lexicon_tags_25 },
    { 0x69128165, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77928167, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf392816c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd32816f, SUC_lexicon_tags_47 },
    { 0xf872816f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38b28172, SUC_lexicon_tags_49 },
    { 0xbf328173, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0328179, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd12817f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65f28181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a728183, SUC_lexicon_tags_107 },
    { 0xc928184, SUC_lexicon_tags_37 },
    { 0x5ef28185, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa852818a, SUC_lexicon_tags_296 },
    { 0x212818b, SUC_lexicon_tags_25 },
    { 0xa4b2818b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb528194, SUC_lexicon_tags_14 },
    { 0x6928195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d28199, SUC_lexicon_tags_10 },
    { 0x9b2819a, SUC_lexicon_tags_46 },
    { 0x94f2819a, SUC_lexicon_tags_2 },
    { 0xfd2819b, SUC_lexicon_tags_1 },
    { 0x8ab28199, SUC_lexicon_tags_7 },
    { 0x4992819e, SUC_lexicon_tags_11 },
    { 0x3492819f, SUC_lexicon_tags_38 },
    { 0xfc7281a0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x535281a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735281ab, SUC_lexicon_tags_0 },
    { 0xdd3281ab, SUC_lexicon_tags_23 },
    { 0x245281ab, SUC_lexicon_tags_37 },
    { 0xaa5281ac, SUC_lexicon_tags_25 },
    { 0xf65281ac, SUC_lexicon_tags_25 },
    { 0x6c7281b0, SUC_lexicon_tags_60 },
    { 0xb33281ae, SUC_lexicon_tags_31 },
    { 0xffb281ab, SUC_lexicon_tags_14 },
    { 0xb55281b1, SUC_lexicon_tags_15 },
    { 0x641281b4, SUC_lexicon_tags_15 },
    { 0x927281b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21281b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57281bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b281c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5281c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7281cc, SUC_lexicon_tags_2 },
    { 0xa95281cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365281d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x987281d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd281dd, SUC_lexicon_tags_259 },
    { 0x971281dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x719281e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355281e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb5b281e7, SUC_lexicon_tags_28 },
    { 0xea1281e7, SUC_lexicon_tags_0 },
    { 0x837281e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215281ef, SUC_lexicon_tags_19 },
    { 0x425281f0, SUC_lexicon_tags_11 },
    { 0x757281f0, SUC_lexicon_tags_8 },
    { 0xfc7281f2, SUC_lexicon_tags_8 },
    { 0x679281f2, SUC_lexicon_tags_49 },
    { 0x835281f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b281f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7281fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e728200, SUC_lexicon_tags_27 },
    { 0xaf128201, SUC_lexicon_tags_77 },
    { 0xc7728202, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa5b28204, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0528207, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0xadb28209, SUC_lexicon_tags_25 },
    { 0x2b728209, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4592820d, SUC_lexicon_tags_19 },
    { 0x3f32820e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9928218, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2892821a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8632821e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb28222, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14b28224, SUC_lexicon_tags_2 },
    { 0x3ab28225, SUC_lexicon_tags_25 },
    { 0x34b28226, SUC_lexicon_tags_43 },
    { 0x7a928226, SUC_lexicon_tags_17 },
    { 0x73328228, SUC_lexicon_tags_82 },
    { 0x6a728227, SUC_lexicon_tags_7 },
    { 0x20328227, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe772822e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf328231, SUC_lexicon_tags_7 },
    { 0x3fd28232, SUC_lexicon_tags_63 },
    { 0x20328233, SUC_lexicon_tags_87 },
    { 0x89928231, SUC_lexicon_tags_65 },
    { 0xa5328235, SUC_lexicon_tags_19 },
    { 0xb0128235, SUC_lexicon_tags_17 },
    { 0x98128237, SUC_lexicon_tags_28 },
    { 0xe2b28235, SUC_lexicon_tags_12 },
    { 0x4f128235, SUC_lexicon_tags_64 },
    { 0x89728239, SUC_lexicon_tags_25 },
    { 0x2df28236, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf28245, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c528247, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe672824b, SUC_lexicon_tags_19 },
    { 0x5cb2824c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef328251, SUC_lexicon_tags_67 },
    { 0x1a528251, SUC_lexicon_tags_11 },
    { 0xd0528253, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b28256, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8672825a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2825d, SUC_lexicon_tags_1 },
    { 0x892825e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f28262, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70128266, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42b28268, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xae72826a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5452826e, SUC_lexicon_tags_14 },
    { 0xd5d2826e, SUC_lexicon_tags_33 },
    { 0x5928270, SUC_lexicon_tags_56 },
    { 0x33928270, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8328275, SUC_lexicon_tags_33 },
    { 0x7f528276, SUC_lexicon_tags_11 },
    { 0x85b28275, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b2827b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb12827d, SUC_lexicon_tags_7 },
    { 0x4952827d, SUC_lexicon_tags_15 },
    { 0xd5f2827f, SUC_lexicon_tags_25 },
    { 0x43328280, SUC_lexicon_tags_1 },
    { 0xb632827f, SUC_lexicon_tags_29 },
    { 0x14128282, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd728286, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df28289, SUC_lexicon_tags_11 },
    { 0x4d32828a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7132828c, SUC_lexicon_tags_10 },
    { 0x192828d, SUC_lexicon_tags_17 },
    { 0xe112828d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d28297, SUC_lexicon_tags_2 },
    { 0x7f128298, SUC_lexicon_tags_25 },
    { 0xb9928298, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b2829f, SUC_lexicon_tags_29 },
    { 0xb282a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa5b282a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x193282a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9282a9, SUC_lexicon_tags_45 },
    { 0xa03282aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7282ad, SUC_lexicon_tags_22 },
    { 0xa29282ae, SUC_lexicon_tags_26 },
    { 0x88f282af, SUC_lexicon_tags_14 },
    { 0x33f282b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46b282b2, SUC_lexicon_tags_19 },
    { 0x519282b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53282b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x569282b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9282bc, SUC_lexicon_tags_25 },
    { 0xc39282bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30b282bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1b3282c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef282c5, SUC_lexicon_tags_15 },
    { 0xfb282c6, SUC_lexicon_tags_17 },
    { 0xc0b282c5, SUC_lexicon_tags_10 },
    { 0x5fd282c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b282ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d282d1, SUC_lexicon_tags_25 },
    { 0xce3282d1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1282d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f282da, SUC_lexicon_tags_30 },
    { 0x1fd282db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x247282de, SUC_lexicon_tags_15 },
    { 0xcff282df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbe9282e1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5282e6, SUC_lexicon_tags_25 },
    { 0xaf1282e6, SUC_lexicon_tags_14 },
    { 0xe6d282e8, SUC_lexicon_tags_25 },
    { 0x7a1282e9, SUC_lexicon_tags_21 },
    { 0x457282e8, SUC_lexicon_tags_15 },
    { 0xff5282eb, SUC_lexicon_tags_68 },
    { 0xbd282eb, SUC_lexicon_tags_23 },
    { 0x3dd282eb, SUC_lexicon_tags_34 },
    { 0x3cb282e9, SUC_lexicon_tags_12 },
    { 0xcd282ef, SUC_lexicon_tags_1 },
    { 0xf59282ec, SUC_lexicon_tags_11 },
    { 0x38b282f1, SUC_lexicon_tags_63 },
    { 0x4f3282f2, SUC_lexicon_tags_25 },
    { 0x3282ef, SUC_lexicon_tags_25 },
    { 0x317282f4, SUC_lexicon_tags_60 },
    { 0x9e1282f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2d282f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619282fb, SUC_lexicon_tags_23 },
    { 0xe0d282fb, SUC_lexicon_tags_2 },
    { 0x3d282fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f28302, SUC_lexicon_tags_17 },
    { 0x11d28303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0d28305, SUC_lexicon_tags_28 },
    { 0xac328306, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86f28308, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6912830a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffb2830c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb4f2830e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x43128310, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af28317, SUC_lexicon_tags_47 },
    { 0xb2128318, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f2831c, SUC_lexicon_tags_16 },
    { 0xf292831c, SUC_lexicon_tags_43 },
    { 0xbd2831d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbdf28320, SUC_lexicon_tags_10 },
    { 0xaf728321, SUC_lexicon_tags_29 },
    { 0xf4528320, SUC_lexicon_tags_45 },
    { 0xf6528320, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd528327, SUC_lexicon_tags_25 },
    { 0xd0f28328, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73b2832a, SUC_lexicon_tags_141 },
    { 0xaa72832a, SUC_lexicon_tags_17 },
    { 0xc612832a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b28333, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89528335, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64128339, SUC_lexicon_tags_26 },
    { 0x67f28339, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd872833c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7752833e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd28346, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc7f28348, SUC_lexicon_tags_1 },
    { 0x9eb28348, SUC_lexicon_tags_61 },
    { 0xa1b2834a, SUC_lexicon_tags_45 },
    { 0x3972834a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f32834d, SUC_lexicon_tags_28 },
    { 0x6992834d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd28356, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd7d28358, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b2835c, SUC_lexicon_tags_2 },
    { 0xe7f2835c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x17928360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b28364, SUC_lexicon_tags_60 },
    { 0xebd28365, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb6128367, SUC_lexicon_tags_14 },
    { 0xb3928367, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f2836b, SUC_lexicon_tags_16 },
    { 0xb992836c, SUC_lexicon_tags_14 },
    { 0x2252836b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf28374, SUC_lexicon_tags_47 },
    { 0x1e128375, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f2837b, SUC_lexicon_tags_14 },
    { 0xa432837c, SUC_lexicon_tags_29 },
    { 0xb352837d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8cb2837f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df28383, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b28389, SUC_lexicon_tags_12 },
    { 0x2f92838a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27d2838c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe32838f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x96928392, SUC_lexicon_tags_27 },
    { 0x7bd28393, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94128395, SUC_lexicon_tags_17 },
    { 0x4a128396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab2839a, SUC_lexicon_tags_37 },
    { 0xf352839b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83283a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7283a3, SUC_lexicon_tags_1 },
    { 0x567283a3, SUC_lexicon_tags_49 },
    { 0x905283a3, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5283af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde3283b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9283b7, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x4f7283b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757283bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff283c4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x461283c6, SUC_lexicon_tags_23 },
    { 0x633283c7, SUC_lexicon_tags_55 },
    { 0x99283c8, SUC_lexicon_tags_23 },
    { 0x253283c8, SUC_lexicon_tags_21 },
    { 0xf9f283ca, SUC_lexicon_tags_15 },
    { 0x4f3283cb, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669283cf, SUC_lexicon_tags_38 },
    { 0x62b283d0, SUC_lexicon_tags_17 },
    { 0x65283cf, SUC_lexicon_tags_33 },
    { 0xd41283d2, SUC_lexicon_tags_18 },
    { 0xf9b283d2, SUC_lexicon_tags_39 },
    { 0x2eb283d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd283da, SUC_lexicon_tags_61 },
    { 0x18f283db, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x147283de, SUC_lexicon_tags_94 },
    { 0x68b283df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175283e6, SUC_lexicon_tags_17 },
    { 0x583283e7, SUC_lexicon_tags_2 },
    { 0x1f5283e6, SUC_lexicon_tags_29 },
    { 0xa25283e9, SUC_lexicon_tags_14 },
    { 0x383283ea, SUC_lexicon_tags_9 },
    { 0xcf283e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x201283ed, SUC_lexicon_tags_29 },
    { 0xb07283ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d5283f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b283f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3283f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x207283fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f928402, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50128404, SUC_lexicon_tags_25 },
    { 0x48d28405, SUC_lexicon_tags_27 },
    { 0x27928406, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9fb28408, SUC_lexicon_tags_6 },
    { 0xffd28409, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d2840d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d528413, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa3528415, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12328418, SUC_lexicon_tags_48 },
    { 0xd7128419, SUC_lexicon_tags_47 },
    { 0xcdb28418, SUC_lexicon_tags_12 },
    { 0x2d2841b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4772841d, SUC_lexicon_tags_49 },
    { 0xd992841e, SUC_lexicon_tags_17 },
    { 0x6672841d, SUC_lexicon_tags_0 },
    { 0x74528420, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b328422, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d28425, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb492842b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xad32842d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5ed2842f, SUC_lexicon_tags_14 },
    { 0x4c328430, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdbf28432, SUC_lexicon_tags_1 },
    { 0x8a928433, SUC_lexicon_tags_11 },
    { 0x67128432, SUC_lexicon_tags_21 },
    { 0xd9928435, SUC_lexicon_tags_25 },
    { 0xdc728435, SUC_lexicon_tags_37 },
    { 0xe9528433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x572843c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d72843e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x64128441, SUC_lexicon_tags_13 },
    { 0x5c128442, SUC_lexicon_tags_31 },
    { 0xfd728441, SUC_lexicon_tags_35 },
    { 0xfad28444, SUC_lexicon_tags_79 },
    { 0xb3528445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f28449, SUC_lexicon_tags_7 },
    { 0x2f52844a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x8412844c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2972844e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf28452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f28458, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b2845c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6128460, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21b28462, SUC_lexicon_tags_84 },
    { 0x96928463, SUC_lexicon_tags_17 },
    { 0x56328462, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf328466, SUC_lexicon_tags_15 },
    { 0xfcb28466, SUC_lexicon_tags_17 },
    { 0x75f28467, SUC_lexicon_tags_16 },
    { 0x5928469, SUC_lexicon_tags_47 },
    { 0xeb92846a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4f2846c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d52846e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba328471, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x91b28473, SUC_lexicon_tags_11 },
    { 0x57928474, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6128478, SUC_lexicon_tags_15 },
    { 0xf9f28479, SUC_lexicon_tags_17 },
    { 0x9a128479, SUC_lexicon_tags_21 },
    { 0xd092847b, SUC_lexicon_tags_9 },
    { 0x2792847b, SUC_lexicon_tags_0 },
    { 0xe72847c, SUC_lexicon_tags_63 },
    { 0x1492847e, SUC_lexicon_tags_8 },
    { 0xf5b2847e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a928481, SUC_lexicon_tags_54 },
    { 0xe2b28482, SUC_lexicon_tags_68 },
    { 0xccb28483, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf28487, SUC_lexicon_tags_7 },
    { 0x47328487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b32848e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f28491, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0xda328494, SUC_lexicon_tags_43 },
    { 0x94728495, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5152849a, SUC_lexicon_tags_1 },
    { 0xf7d2849b, SUC_lexicon_tags_28 },
    { 0xcb72849b, SUC_lexicon_tags_27 },
    { 0xadb2849d, SUC_lexicon_tags_14 },
    { 0x6af2849a, SUC_lexicon_tags_0 },
    { 0x9212849f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9284a4, SUC_lexicon_tags_26 },
    { 0xd39284a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d284ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7284b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfef284b2, SUC_lexicon_tags_17 },
    { 0x8a5284b3, SUC_lexicon_tags_17 },
    { 0x53f284b3, SUC_lexicon_tags_45 },
    { 0xf9f284b5, SUC_lexicon_tags_45 },
    { 0xf65284b6, SUC_lexicon_tags_37 },
    { 0x973284b7, SUC_lexicon_tags_32 },
    { 0xd15284b8, SUC_lexicon_tags_19 },
    { 0xc07284b8, SUC_lexicon_tags_12 },
    { 0x7bb284ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4b284bc, SUC_lexicon_tags_77 },
    { 0x491284bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781284c1, SUC_lexicon_tags_10 },
    { 0x709284c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b284c6, SUC_lexicon_tags_141 },
    { 0x8f9284c6, SUC_lexicon_tags_45 },
    { 0x555284c6, SUC_lexicon_tags_7 },
    { 0x29f284c9, SUC_lexicon_tags_11 },
    { 0xa97284ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb284cd, SUC_lexicon_tags_44 },
    { 0x53b284ce, SUC_lexicon_tags_46 },
    { 0x1ff284cf, SUC_lexicon_tags_8 },
    { 0xcb9284cf, SUC_lexicon_tags_14 },
    { 0x5cf284cf, SUC_lexicon_tags_27 },
    { 0x679284d2, SUC_lexicon_tags_11 },
    { 0x2c1284d3, SUC_lexicon_tags_38 },
    { 0x74d284ce, SUC_lexicon_tags_14 },
    { 0x4bf284d4, SUC_lexicon_tags_23 },
    { 0x14d284d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbdb284d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3b284da, SUC_lexicon_tags_41 },
    { 0xcf7284da, SUC_lexicon_tags_12 },
    { 0xe75284dc, SUC_lexicon_tags_1 },
    { 0xc39284dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19284e0, SUC_lexicon_tags_12 },
    { 0x12f284e1, SUC_lexicon_tags_43 },
    { 0x66d284e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815284e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7284f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1284f7, SUC_lexicon_tags_28 },
    { 0x50b284f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb284ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x14128502, SUC_lexicon_tags_9 },
    { 0x75b28503, SUC_lexicon_tags_11 },
    { 0x39528502, SUC_lexicon_tags_19 },
    { 0x2c928505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccd28507, SUC_lexicon_tags_45 },
    { 0xc1728508, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d32850c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6128511, SUC_lexicon_tags_37 },
    { 0x3a728511, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6328516, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d328518, SUC_lexicon_tags_16 },
    { 0xd3728519, SUC_lexicon_tags_15 },
    { 0xfcd2851a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d2851e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26f28520, SUC_lexicon_tags_35 },
    { 0xa5b28521, SUC_lexicon_tags_15 },
    { 0xc6128522, SUC_lexicon_tags_15 },
    { 0x19528522, SUC_lexicon_tags_8 },
    { 0xfb928524, SUC_lexicon_tags_5 },
    { 0x4cd28525, SUC_lexicon_tags_15 },
    { 0x59f28524, SUC_lexicon_tags_0 },
    { 0x38128523, SUC_lexicon_tags_37 },
    { 0x82f28521, SUC_lexicon_tags_30 },
    { 0x25f28529, SUC_lexicon_tags_18 },
    { 0xd1d28524, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x132852d, SUC_lexicon_tags_38 },
    { 0x3ab2852d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f28535, SUC_lexicon_tags_12 },
    { 0xb6728535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9572853b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd28541, SUC_lexicon_tags_11 },
    { 0x17f28541, SUC_lexicon_tags_38 },
    { 0x13328543, SUC_lexicon_tags_14 },
    { 0x3c728541, SUC_lexicon_tags_37 },
    { 0xac528545, SUC_lexicon_tags_15 },
    { 0x63728544, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b128549, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d2854e, SUC_lexicon_tags_60 },
    { 0xe052854e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79528559, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6592855b, SUC_lexicon_tags_49 },
    { 0x5ab2855b, SUC_lexicon_tags_37 },
    { 0xd6f2855d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92728562, SUC_lexicon_tags_33 },
    { 0x9ad28562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd528565, SUC_lexicon_tags_9 },
    { 0xef528566, SUC_lexicon_tags_14 },
    { 0x1a328566, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa832856e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe528570, SUC_lexicon_tags_9 },
    { 0xb0d28571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7528574, SUC_lexicon_tags_28 },
    { 0xc0728574, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7528577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f2857a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8528581, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9072858d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d28590, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b28597, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c12859a, SUC_lexicon_tags_13 },
    { 0x17d2859b, SUC_lexicon_tags_39 },
    { 0xf5b2859b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1285a0, SUC_lexicon_tags_17 },
    { 0x419285a1, SUC_lexicon_tags_7 },
    { 0x823285a0, SUC_lexicon_tags_45 },
    { 0x47285a3, SUC_lexicon_tags_31 },
    { 0x513285a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad285a9, SUC_lexicon_tags_45 },
    { 0xff7285a9, SUC_lexicon_tags_9 },
    { 0x7f285ab, SUC_lexicon_tags_37 },
    { 0xef3285ac, SUC_lexicon_tags_7 },
    { 0x61f285ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdff285af, SUC_lexicon_tags_14 },
    { 0xcd1285af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b285b4, SUC_lexicon_tags_7 },
    { 0x8e5285b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xebd285b7, SUC_lexicon_tags_48 },
    { 0xf85285b7, SUC_lexicon_tags_21 },
    { 0x775285b7, SUC_lexicon_tags_50 },
    { 0x409285ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac3285bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f285c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x211285c4, SUC_lexicon_tags_0 },
    { 0x52d285c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x19b285c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9285cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9285cf, SUC_lexicon_tags_25 },
    { 0x16f285d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537285d8, SUC_lexicon_tags_14 },
    { 0xf1285d9, SUC_lexicon_tags_11 },
    { 0x893285d9, SUC_lexicon_tags_14 },
    { 0xe4b285da, SUC_lexicon_tags_8 },
    { 0x589285d9, SUC_lexicon_tags_27 },
    { 0xe8b285dd, SUC_lexicon_tags_37 },
    { 0x7c1285de, SUC_lexicon_tags_17 },
    { 0x613285dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc7285e1, SUC_lexicon_tags_33 },
    { 0xf23285e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31285e5, SUC_lexicon_tags_48 },
    { 0x353285e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x839285e8, SUC_lexicon_tags_10 },
    { 0x7e1285e9, SUC_lexicon_tags_12 },
    { 0x489285e9, SUC_lexicon_tags_28 },
    { 0xfb285eb, SUC_lexicon_tags_9 },
    { 0xd47285eb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b285f1, SUC_lexicon_tags_1 },
    { 0x117285f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2f1285f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f285f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5285fa, SUC_lexicon_tags_25 },
    { 0xc8d285fb, SUC_lexicon_tags_61 },
    { 0x3fd285fb, SUC_lexicon_tags_7 },
    { 0x5e3285fb, SUC_lexicon_tags_1 },
    { 0xa9f285fe, SUC_lexicon_tags_11 },
    { 0x90f285ff, SUC_lexicon_tags_17 },
    { 0x489285fe, SUC_lexicon_tags_49 },
    { 0x9ed285ff, SUC_lexicon_tags_27 },
    { 0x41928602, SUC_lexicon_tags_12 },
    { 0x2d728603, SUC_lexicon_tags_29 },
    { 0x96928603, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c528607, SUC_lexicon_tags_12 },
    { 0xe3128608, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f2860d, SUC_lexicon_tags_7 },
    { 0xf772860e, SUC_lexicon_tags_9 },
    { 0x4f32860e, SUC_lexicon_tags_56 },
    { 0xb6b28610, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18b28612, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92d28614, SUC_lexicon_tags_2 },
    { 0x4d928614, SUC_lexicon_tags_64 },
    { 0x52128616, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5728619, SUC_lexicon_tags_0 },
    { 0x40928619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b2861e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x77f28620, SUC_lexicon_tags_60 },
    { 0x8d928620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd328623, SUC_lexicon_tags_32 },
    { 0xab728624, SUC_lexicon_tags_141 },
    { 0x9fd28625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49728629, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd52862c, SUC_lexicon_tags_49 },
    { 0xc212862d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x63b2862f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7928632, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9528637, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb2863a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f28641, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14928643, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f28648, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9192864a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e928654, SUC_lexicon_tags_17 },
    { 0x33d28655, SUC_lexicon_tags_27 },
    { 0xbbb28654, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d2865b, SUC_lexicon_tags_6 },
    { 0xcb52865c, SUC_lexicon_tags_9 },
    { 0xd1d2865b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda128661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22928667, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x29928669, SUC_lexicon_tags_7 },
    { 0x65328669, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2932866e, SUC_lexicon_tags_33 },
    { 0xc1d2866e, SUC_lexicon_tags_7 },
    { 0x96128670, SUC_lexicon_tags_15 },
    { 0xcdf28670, SUC_lexicon_tags_30 },
    { 0xd3f28670, SUC_lexicon_tags_16 },
    { 0x4728670, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e728676, SUC_lexicon_tags_43 },
    { 0xc0128677, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc52867d, SUC_lexicon_tags_15 },
    { 0x9ef2867e, SUC_lexicon_tags_15 },
    { 0xf9d2867f, SUC_lexicon_tags_104 },
    { 0x80b2867f, SUC_lexicon_tags_25 },
    { 0x95328681, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db28685, SUC_lexicon_tags_16 },
    { 0xd5328686, SUC_lexicon_tags_11 },
    { 0xb2928687, SUC_lexicon_tags_31 },
    { 0x55928688, SUC_lexicon_tags_12 },
    { 0x7128686, SUC_lexicon_tags_151 },
    { 0x31328688, SUC_lexicon_tags_7 },
    { 0x35d28687, SUC_lexicon_tags_33 },
    { 0x64f2868a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c72868e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b728694, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bd28696, SUC_lexicon_tags_17 },
    { 0xcc128696, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd2869b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12869d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5286a4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x87f286a6, SUC_lexicon_tags_58 },
    { 0x29286a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1286ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913286b1, SUC_lexicon_tags_37 },
    { 0xc7b286b2, SUC_lexicon_tags_65 },
    { 0x953286b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61286b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x779286bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9f286bd, SUC_lexicon_tags_127 },
    { 0x753286bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca3286c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb63286c2, SUC_lexicon_tags_38 },
    { 0x801286c3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x631286c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x261286c8, SUC_lexicon_tags_3 },
    { 0x589286c9, SUC_lexicon_tags_48 },
    { 0xad9286ca, SUC_lexicon_tags_25 },
    { 0x23d286c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3286cf, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f286d8, SUC_lexicon_tags_12 },
    { 0x713286d8, SUC_lexicon_tags_43 },
    { 0xb75286da, SUC_lexicon_tags_17 },
    { 0xf13286db, SUC_lexicon_tags_26 },
    { 0x535286dc, SUC_lexicon_tags_19 },
    { 0xe6d286dd, SUC_lexicon_tags_14 },
    { 0x6fd286de, SUC_lexicon_tags_15 },
    { 0xa27286de, SUC_lexicon_tags_45 },
    { 0x76b286db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9286e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5286ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x871286ee, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x585286f1, SUC_lexicon_tags_12 },
    { 0x6dd286f2, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f286f5, SUC_lexicon_tags_0 },
    { 0xd11286f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x945286f8, SUC_lexicon_tags_37 },
    { 0x2d5286f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf23286fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5286fe, SUC_lexicon_tags_12 },
    { 0x9d1286ff, SUC_lexicon_tags_55 },
    { 0x6a928700, SUC_lexicon_tags_38 },
    { 0xab28701, SUC_lexicon_tags_0 },
    { 0xb6128700, SUC_lexicon_tags_1 },
    { 0xcdb28703, SUC_lexicon_tags_57 },
    { 0x6e128704, SUC_lexicon_tags_69 },
    { 0x4928703, SUC_lexicon_tags_7 },
    { 0xc0d28706, SUC_lexicon_tags_12 },
    { 0x14528707, SUC_lexicon_tags_25 },
    { 0x63528707, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce92870e, SUC_lexicon_tags_48 },
    { 0xf752870f, SUC_lexicon_tags_12 },
    { 0x4612870f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf528715, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x12528717, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcb528719, SUC_lexicon_tags_65 },
    { 0x2092871a, SUC_lexicon_tags_37 },
    { 0x6692871a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7d2871d, SUC_lexicon_tags_27 },
    { 0xd32871e, SUC_lexicon_tags_79 },
    { 0xf992871e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe5528721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94b28723, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1728728, SUC_lexicon_tags_7 },
    { 0x95728729, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb92872b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2872f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f28733, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9728735, SUC_lexicon_tags_17 },
    { 0x96728736, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b728738, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb2873b, SUC_lexicon_tags_25 },
    { 0xeb72873b, SUC_lexicon_tags_131 },
    { 0xd612873d, SUC_lexicon_tags_15 },
    { 0x1bd2873e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40328740, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6928746, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x32128748, SUC_lexicon_tags_38 },
    { 0x88928748, SUC_lexicon_tags_21 },
    { 0xfc12874a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf292874c, SUC_lexicon_tags_29 },
    { 0x3ef2874c, SUC_lexicon_tags_8 },
    { 0x7612874c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f28756, SUC_lexicon_tags_8 },
    { 0x77d28756, SUC_lexicon_tags_12 },
    { 0xf1b28758, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc12875b, SUC_lexicon_tags_40 },
    { 0xc9d2875c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2528762, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2c128764, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e928767, SUC_lexicon_tags_19 },
    { 0x31d28768, SUC_lexicon_tags_201 },
    { 0xa928769, SUC_lexicon_tags_25 },
    { 0x3c92876a, SUC_lexicon_tags_40 },
    { 0x61b2876b, SUC_lexicon_tags_29 },
    { 0x4772876c, SUC_lexicon_tags_15 },
    { 0x5c728767, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f28771, SUC_lexicon_tags_14 },
    { 0xf0528772, SUC_lexicon_tags_4 },
    { 0x6328771, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb28778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d12877b, SUC_lexicon_tags_84 },
    { 0x4f32877c, SUC_lexicon_tags_0 },
    { 0x9612877c, SUC_lexicon_tags_37 },
    { 0xa1f2877d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b328782, SUC_lexicon_tags_21 },
    { 0xbb928783, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8f28785, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6392878b, SUC_lexicon_tags_12 },
    { 0xb6d2878c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea32878e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31328791, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10328793, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x59128796, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72d28798, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7072879b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xef92879d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d287a0, SUC_lexicon_tags_19 },
    { 0xc7b287a1, SUC_lexicon_tags_14 },
    { 0xc91287a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225287a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57287a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01287ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf31287b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d287b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x41d287b6, SUC_lexicon_tags_77 },
    { 0xc3b287b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac7287b9, SUC_lexicon_tags_1 },
    { 0xc35287b9, SUC_lexicon_tags_85 },
    { 0xe27287bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47287be, SUC_lexicon_tags_11 },
    { 0xae7287bf, SUC_lexicon_tags_4 },
    { 0xfe9287be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f287c5, SUC_lexicon_tags_49 },
    { 0xb99287c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07287c9, SUC_lexicon_tags_32 },
    { 0xa9d287ca, SUC_lexicon_tags_6 },
    { 0xead287cb, SUC_lexicon_tags_9 },
    { 0x7d7287c9, SUC_lexicon_tags_10 },
    { 0x41b287cd, SUC_lexicon_tags_14 },
    { 0x2b3287ce, SUC_lexicon_tags_15 },
    { 0xc77287cd, SUC_lexicon_tags_17 },
    { 0x72b287d0, SUC_lexicon_tags_37 },
    { 0x1e5287cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b287d6, SUC_lexicon_tags_10 },
    { 0x8d3287d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55287da, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x537287dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf287e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787287e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd287ea, SUC_lexicon_tags_11 },
    { 0x24d287eb, SUC_lexicon_tags_45 },
    { 0xdcd287eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b287f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x759287f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71287fe, SUC_lexicon_tags_28 },
    { 0x5d5287ff, SUC_lexicon_tags_25 },
    { 0xeab287ff, SUC_lexicon_tags_33 },
    { 0x81128801, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf128804, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc1b28806, SUC_lexicon_tags_68 },
    { 0x12528807, SUC_lexicon_tags_33 },
    { 0xc8128806, SUC_lexicon_tags_15 },
    { 0x63f28808, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xafd2880b, SUC_lexicon_tags_47 },
    { 0x9f32880c, SUC_lexicon_tags_2 },
    { 0xf032880c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x21f2880f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46728815, SUC_lexicon_tags_45 },
    { 0xed928816, SUC_lexicon_tags_28 },
    { 0xa1528815, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d2881b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6472881d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad28824, SUC_lexicon_tags_68 },
    { 0x61f28825, SUC_lexicon_tags_15 },
    { 0xeef28826, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b2882c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40528830, SUC_lexicon_tags_34 },
    { 0xafd28831, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6128837, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d72883d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f28843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82528849, SUC_lexicon_tags_11 },
    { 0xa72884a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1652884c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9192884e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af28852, SUC_lexicon_tags_23 },
    { 0xcc928853, SUC_lexicon_tags_37 },
    { 0x5d28854, SUC_lexicon_tags_47 },
    { 0x56328854, SUC_lexicon_tags_37 },
    { 0x69728855, SUC_lexicon_tags_26 },
    { 0x87128857, SUC_lexicon_tags_7 },
    { 0x67d28858, SUC_lexicon_tags_17 },
    { 0x72928859, SUC_lexicon_tags_0 },
    { 0xd5328859, SUC_lexicon_tags_35 },
    { 0xa4d2885b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4572885f, SUC_lexicon_tags_37 },
    { 0xe8f28860, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdab28862, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40b28864, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4292886a, SUC_lexicon_tags_17 },
    { 0x14b2886a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7632886e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb28871, SUC_lexicon_tags_2 },
    { 0xf2728872, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b328878, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb28885, SUC_lexicon_tags_15 },
    { 0xb1d28885, SUC_lexicon_tags_37 },
    { 0x5b728885, SUC_lexicon_tags_8 },
    { 0xaa328887, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe3b2888a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e728890, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f28896, SUC_lexicon_tags_7 },
    { 0x8cd28897, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d2889c, SUC_lexicon_tags_23 },
    { 0xe752889d, SUC_lexicon_tags_30 },
    { 0x8db2889e, SUC_lexicon_tags_11 },
    { 0x8eb2889e, SUC_lexicon_tags_37 },
    { 0xd15288a0, SUC_lexicon_tags_15 },
    { 0xa4b288a0, SUC_lexicon_tags_11 },
    { 0x7712889e, SUC_lexicon_tags_34 },
    { 0x249288a3, SUC_lexicon_tags_9 },
    { 0xda7288a3, SUC_lexicon_tags_60 },
    { 0x7af288a4, SUC_lexicon_tags_33 },
    { 0x8d1288a5, SUC_lexicon_tags_37 },
    { 0x163288a7, SUC_lexicon_tags_16 },
    { 0x133288a1, SUC_lexicon_tags_11 },
    { 0xafb288a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195288af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45288b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xab3288b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d288b9, SUC_lexicon_tags_37 },
    { 0x2ab288ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb288be, SUC_lexicon_tags_39 },
    { 0x151288bf, SUC_lexicon_tags_7 },
    { 0xa71288be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x34b288c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb25288c4, SUC_lexicon_tags_30 },
    { 0xec7288c5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b288ca, SUC_lexicon_tags_14 },
    { 0xb69288cb, SUC_lexicon_tags_77 },
    { 0x959288cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807288d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7288d8, SUC_lexicon_tags_37 },
    { 0x28b288d9, SUC_lexicon_tags_37 },
    { 0xb15288d9, SUC_lexicon_tags_21 },
    { 0x7a1288db, SUC_lexicon_tags_7 },
    { 0x3cf288dc, SUC_lexicon_tags_11 },
    { 0xb13288dd, SUC_lexicon_tags_5 },
    { 0x535288da, SUC_lexicon_tags_2 },
    { 0xddb288dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x675288e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x317288e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa4d288e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d9288e8, SUC_lexicon_tags_0 },
    { 0x611288e8, SUC_lexicon_tags_19 },
    { 0xbe9288e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x393288ec, SUC_lexicon_tags_7 },
    { 0xb288ec, SUC_lexicon_tags_19 },
    { 0x811288ee, SUC_lexicon_tags_7 },
    { 0x1bd288ef, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe3d288f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x509288f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x875288f7, SUC_lexicon_tags_37 },
    { 0x161288f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa5288fa, SUC_lexicon_tags_40 },
    { 0x949288fa, SUC_lexicon_tags_14 },
    { 0x791288fb, SUC_lexicon_tags_37 },
    { 0x65288fd, SUC_lexicon_tags_31 },
    { 0xfdf288fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb28901, SUC_lexicon_tags_46 },
    { 0x16f28902, SUC_lexicon_tags_47 },
    { 0xd2528903, SUC_lexicon_tags_27 },
    { 0xa4128902, SUC_lexicon_tags_12 },
    { 0x60928905, SUC_lexicon_tags_39 },
    { 0x84b28904, SUC_lexicon_tags_49 },
    { 0xdcb28907, SUC_lexicon_tags_29 },
    { 0x52528908, SUC_lexicon_tags_23 },
    { 0x72d28908, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d2890c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d72890e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb28913, SUC_lexicon_tags_15 },
    { 0x52328913, SUC_lexicon_tags_0 },
    { 0x15d28915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d12891c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2fb2891e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f28921, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf28927, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19928929, SUC_lexicon_tags_1 },
    { 0x42528929, SUC_lexicon_tags_62 },
    { 0x53f2892b, SUC_lexicon_tags_91 },
    { 0x5d72892a, SUC_lexicon_tags_25 },
    { 0x9b52892d, SUC_lexicon_tags_1 },
    { 0xe692892a, SUC_lexicon_tags_11 },
    { 0xcd32892b, SUC_lexicon_tags_1 },
    { 0x8472892a, SUC_lexicon_tags_30 },
    { 0x38528931, SUC_lexicon_tags_37 },
    { 0xd5328932, SUC_lexicon_tags_7 },
    { 0x27928932, SUC_lexicon_tags_8 },
    { 0xb528932, SUC_lexicon_tags_12 },
    { 0xab328935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8072893a, SUC_lexicon_tags_29 },
    { 0xd832893b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf728940, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbb728942, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d28945, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd2894a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc312894c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd392894e, SUC_lexicon_tags_17 },
    { 0x2652894e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff28952, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc128955, SUC_lexicon_tags_37 },
    { 0x8b928956, SUC_lexicon_tags_17 },
    { 0x6128957, SUC_lexicon_tags_1 },
    { 0xb9d28957, SUC_lexicon_tags_30 },
    { 0x5b328958, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f2895c, SUC_lexicon_tags_37 },
    { 0xe9d2895d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d928967, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48128969, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8712896c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d2896f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a928973, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb528976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf728978, SUC_lexicon_tags_23 },
    { 0x33328978, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef2897c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3bd2897e, SUC_lexicon_tags_14 },
    { 0x5792897f, SUC_lexicon_tags_38 },
    { 0xa7d28980, SUC_lexicon_tags_6 },
    { 0x50128980, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7712898a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba92898c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf872898f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b128992, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d28995, SUC_lexicon_tags_10 },
    { 0x77928996, SUC_lexicon_tags_11 },
    { 0xc5728997, SUC_lexicon_tags_69 },
    { 0x66d28998, SUC_lexicon_tags_100 },
    { 0x84928999, SUC_lexicon_tags_33 },
    { 0x5c52899a, SUC_lexicon_tags_87 },
    { 0x2c52899a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e32899d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1452899f, SUC_lexicon_tags_23 },
    { 0xb8f2899f, SUC_lexicon_tags_44 },
    { 0x1d1289a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d289a7, SUC_lexicon_tags_40 },
    { 0x649289a8, SUC_lexicon_tags_17 },
    { 0xb7b289a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489289ad, SUC_lexicon_tags_31 },
    { 0x55289ad, SUC_lexicon_tags_33 },
    { 0xaf9289ae, SUC_lexicon_tags_19 },
    { 0xde3289ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d289b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7289bb, SUC_lexicon_tags_15 },
    { 0xb73289bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x231289be, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb289c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61289c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8e7289ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfa3289cc, SUC_lexicon_tags_43 },
    { 0xcc9289cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f289d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x13f289d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa8f289d4, SUC_lexicon_tags_60 },
    { 0x475289d5, SUC_lexicon_tags_27 },
    { 0x331289d6, SUC_lexicon_tags_1 },
    { 0x3e1289d7, SUC_lexicon_tags_49 },
    { 0x77d289d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd3289da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4f289dc, SUC_lexicon_tags_27 },
    { 0x5a5289dd, SUC_lexicon_tags_37 },
    { 0x61f289de, SUC_lexicon_tags_48 },
    { 0xd4d289df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f289e2, SUC_lexicon_tags_47 },
    { 0x25f289e3, SUC_lexicon_tags_1 },
    { 0xaf5289e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45289e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x899289e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b289ec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa03289ee, SUC_lexicon_tags_43 },
    { 0x2b7289ef, SUC_lexicon_tags_25 },
    { 0x4bb289ee, SUC_lexicon_tags_0 },
    { 0x135289f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999289fb, SUC_lexicon_tags_37 },
    { 0x741289fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f9289fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53d28a00, SUC_lexicon_tags_30 },
    { 0xf6328a00, SUC_lexicon_tags_25 },
    { 0xbbf28a02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70928a08, SUC_lexicon_tags_38 },
    { 0x82528a09, SUC_lexicon_tags_17 },
    { 0x12328a0a, SUC_lexicon_tags_60 },
    { 0x8d528a0b, SUC_lexicon_tags_30 },
    { 0xc6928a0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b928a10, SUC_lexicon_tags_23 },
    { 0xef728a11, SUC_lexicon_tags_55 },
    { 0xfdd28a11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed28a15, SUC_lexicon_tags_21 },
    { 0x22728a15, SUC_lexicon_tags_37 },
    { 0xbab28a16, SUC_lexicon_tags_10 },
    { 0xfb528a18, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b28a1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe928a23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb328a28, SUC_lexicon_tags_1 },
    { 0x33928a29, SUC_lexicon_tags_40 },
    { 0x37528a2a, SUC_lexicon_tags_7 },
    { 0x51128a2b, SUC_lexicon_tags_37 },
    { 0xabd28a2a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc128a2f, SUC_lexicon_tags_70 },
    { 0x3f28a30, SUC_lexicon_tags_14 },
    { 0xa9928a31, SUC_lexicon_tags_17 },
    { 0xd9b28a32, SUC_lexicon_tags_12 },
    { 0x13b28a32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0728a35, SUC_lexicon_tags_0 },
    { 0x74f28a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b28a3c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8bf28a3e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b28a42, SUC_lexicon_tags_38 },
    { 0xd0b28a43, SUC_lexicon_tags_12 },
    { 0xbd928a44, SUC_lexicon_tags_118 },
    { 0xb4928a45, SUC_lexicon_tags_1 },
    { 0x21f28a46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb28a4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb28a4d, SUC_lexicon_tags_62 },
    { 0xeff28a4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b528a53, SUC_lexicon_tags_82 },
    { 0xe6928a53, SUC_lexicon_tags_21 },
    { 0xd5d28a55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6528a58, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcef28a5a, SUC_lexicon_tags_25 },
    { 0x19728a5b, SUC_lexicon_tags_11 },
    { 0xcdb28a5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1128a60, SUC_lexicon_tags_49 },
    { 0x74128a61, SUC_lexicon_tags_11 },
    { 0x1a328a62, SUC_lexicon_tags_15 },
    { 0x26328a63, SUC_lexicon_tags_55 },
    { 0xf0328a60, SUC_lexicon_tags_46 },
    { 0x2f328a65, SUC_lexicon_tags_19 },
    { 0x78128a66, SUC_lexicon_tags_23 },
    { 0x4af28a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0328a69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6328a6d, SUC_lexicon_tags_14 },
    { 0x52528a6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb928a70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31528a75, SUC_lexicon_tags_65 },
    { 0x67128a76, SUC_lexicon_tags_1083 },
    { 0x11f28a77, SUC_lexicon_tags_31 },
    { 0x8c328a77, SUC_lexicon_tags_33 },
    { 0xd7b28a79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4928a7f, SUC_lexicon_tags_77 },
    { 0xef528a80, SUC_lexicon_tags_17 },
    { 0x5e928a80, SUC_lexicon_tags_60 },
    { 0xd5b28a82, SUC_lexicon_tags_60 },
    { 0x9328a83, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7cd28a85, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f28a89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51328a8e, SUC_lexicon_tags_892 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d28a93, SUC_lexicon_tags_1 },
    { 0xdcb28a94, SUC_lexicon_tags_19 },
    { 0x74728a93, SUC_lexicon_tags_19 },
    { 0x39728a96, SUC_lexicon_tags_56 },
    { 0x17d28a96, SUC_lexicon_tags_19 },
    { 0x98f28a98, SUC_lexicon_tags_17 },
    { 0x1b128a95, SUC_lexicon_tags_1 },
    { 0xff728a93, SUC_lexicon_tags_21 },
    { 0x7b328a93, SUC_lexicon_tags_127 },
    { 0xa7728a9c, SUC_lexicon_tags_7 },
    { 0x35128a9a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb1928a9f, SUC_lexicon_tags_26 },
    { 0x1bd28aa0, SUC_lexicon_tags_39 },
    { 0xb2128aa0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f28aa4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b28aa7, SUC_lexicon_tags_55 },
    { 0x73128aa8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b28aae, SUC_lexicon_tags_2 },
    { 0x4ed28aaf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7eb28ab1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff28ab5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe128ab8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f28ac0, SUC_lexicon_tags_12 },
    { 0x6b328ac0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b28ac4, SUC_lexicon_tags_37 },
    { 0xb5f28ac4, SUC_lexicon_tags_47 },
    { 0x36528ac6, SUC_lexicon_tags_27 },
    { 0x51d28ac4, SUC_lexicon_tags_23 },
    { 0xd2728ac8, SUC_lexicon_tags_25 },
    { 0x63928ac8, SUC_lexicon_tags_29 },
    { 0x38528aca, SUC_lexicon_tags_37 },
    { 0xefd28aca, SUC_lexicon_tags_17 },
    { 0xedd28acc, SUC_lexicon_tags_12 },
    { 0x44728acd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8128acf, SUC_lexicon_tags_7 },
    { 0x1b328ad0, SUC_lexicon_tags_43 },
    { 0xc1b28ad0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb28ad5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b28ad8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31928ada, SUC_lexicon_tags_63 },
    { 0x52528adb, SUC_lexicon_tags_14 },
    { 0xf7f28adc, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5528adf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe8128ae1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x80728ae3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x61b28ae5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab28aef, SUC_lexicon_tags_0 },
    { 0xf1328aef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x16928af3, SUC_lexicon_tags_7 },
    { 0x25928af4, SUC_lexicon_tags_1 },
    { 0xb0328af5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f28af8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56528afe, SUC_lexicon_tags_37 },
    { 0xc5b28aff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7328b08, SUC_lexicon_tags_2 },
    { 0xdcd28b08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f728b0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68328b10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb528b14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4928b1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8d928b1c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69128b21, SUC_lexicon_tags_11 },
    { 0xdd328b22, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2ff28b24, SUC_lexicon_tags_37 },
    { 0xeb128b24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66528b2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19528b30, SUC_lexicon_tags_37 },
    { 0x8ab28b31, SUC_lexicon_tags_82 },
    { 0x2f928b31, SUC_lexicon_tags_37 },
    { 0x42728b33, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad28b39, SUC_lexicon_tags_11 },
    { 0x3e728b3a, SUC_lexicon_tags_68 },
    { 0xb7128b3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef28b41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f28b45, SUC_lexicon_tags_1082 },
    { 0xe6d28b46, SUC_lexicon_tags_47 },
    { 0xf8128b45, SUC_lexicon_tags_8 },
    { 0xd0728b45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f328b4e, SUC_lexicon_tags_1 },
    { 0x1c128b4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad28b55, SUC_lexicon_tags_12 },
    { 0x2bb28b56, SUC_lexicon_tags_23 },
    { 0x14b28b57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a128b5c, SUC_lexicon_tags_33 },
    { 0x48b28b5d, SUC_lexicon_tags_37 },
    { 0x46b28b5c, SUC_lexicon_tags_25 },
    { 0x70d28b5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd28b62, SUC_lexicon_tags_37 },
    { 0xdb728b62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e328b69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc928b6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df28b72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2728b79, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdaf28b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38928b7e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5c328b80, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9728b85, SUC_lexicon_tags_10 },
    { 0x36b28b85, SUC_lexicon_tags_10 },
    { 0x54128b87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86928b89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6528b8c, SUC_lexicon_tags_79 },
    { 0x83128b8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94728b91, SUC_lexicon_tags_27 },
    { 0x35128b92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe328b94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c328b9b, SUC_lexicon_tags_47 },
    { 0x9bd28b9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x46928b9e, SUC_lexicon_tags_8 },
    { 0xf6328b9f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20328ba8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb28bad, SUC_lexicon_tags_37 },
    { 0x50928bad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1128bb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a728bb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xceb28bba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95128bbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed28bc1, SUC_lexicon_tags_104 },
    { 0x3cb28bc1, SUC_lexicon_tags_39 },
    { 0x7b928bc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99f28bc5, SUC_lexicon_tags_1 },
    { 0xe4728bc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3af28bc8, SUC_lexicon_tags_42 },
    { 0xb3528bc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d28bcc, SUC_lexicon_tags_11 },
    { 0x6c928bcd, SUC_lexicon_tags_14 },
    { 0x1a328bce, SUC_lexicon_tags_79 },
    { 0x34728bcf, SUC_lexicon_tags_7 },
    { 0x93728bcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a328bd3, SUC_lexicon_tags_6 },
    { 0xe8f28bd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6328bd6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe3328bd8, SUC_lexicon_tags_17 },
    { 0xec928bd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d28bde, SUC_lexicon_tags_11 },
    { 0x65328bdf, SUC_lexicon_tags_12 },
    { 0x29328be0, SUC_lexicon_tags_57 },
    { 0xead28be1, SUC_lexicon_tags_12 },
    { 0x1b128be2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfb728be4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5728be6, SUC_lexicon_tags_1 },
    { 0x51b28be7, SUC_lexicon_tags_17 },
    { 0xed128be8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x58728beb, SUC_lexicon_tags_37 },
    { 0xb6528bec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a128bf2, SUC_lexicon_tags_37 },
    { 0x9cb28bf3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b28bf6, SUC_lexicon_tags_27 },
    { 0x10b28bf7, SUC_lexicon_tags_30 },
    { 0xe5d28bf8, SUC_lexicon_tags_30 },
    { 0xa2528bf9, SUC_lexicon_tags_60 },
    { 0x2a128bfa, SUC_lexicon_tags_33 },
    { 0x80f28bfb, SUC_lexicon_tags_11 },
    { 0xf4128bf8, SUC_lexicon_tags_0 },
    { 0x8b928bf7, SUC_lexicon_tags_7 },
    { 0x84928bfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1dd28c00, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x96728c02, SUC_lexicon_tags_1 },
    { 0x6d128c03, SUC_lexicon_tags_37 },
    { 0xc9328c03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74528c06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19f28c08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1328c0d, SUC_lexicon_tags_47 },
    { 0xc4d28c0d, SUC_lexicon_tags_33 },
    { 0x4b528c0f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x81d28c11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b28c15, SUC_lexicon_tags_0 },
    { 0x19b28c16, SUC_lexicon_tags_25 },
    { 0xc6728c15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69f28c19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf728c1c, SUC_lexicon_tags_1 },
    { 0xd8f28c1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6528c22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0528c26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7928c2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6328c30, SUC_lexicon_tags_62 },
    { 0x55128c31, SUC_lexicon_tags_17 },
    { 0x2eb28c30, SUC_lexicon_tags_37 },
    { 0xb6f28c33, SUC_lexicon_tags_11 },
    { 0x59d28c34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43328c37, SUC_lexicon_tags_14 },
    { 0xf0f28c38, SUC_lexicon_tags_12 },
    { 0x16728c38, SUC_lexicon_tags_84 },
    { 0xda128c38, SUC_lexicon_tags_11 },
    { 0xdd528c38, SUC_lexicon_tags_37 },
    { 0x6ed28c3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8128c40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1728c43, SUC_lexicon_tags_1 },
    { 0xc8128c43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f528c4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45328c52, SUC_lexicon_tags_11 },
    { 0xa7528c53, SUC_lexicon_tags_1 },
    { 0x8d928c54, SUC_lexicon_tags_30 },
    { 0x86b28c54, SUC_lexicon_tags_84 },
    { 0xa128c54, SUC_lexicon_tags_37 },
    { 0xc0728c57, SUC_lexicon_tags_25 },
    { 0xa4f28c58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b28c5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24328c5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4f928c61, SUC_lexicon_tags_47 },
    { 0xc4328c61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbef28c64, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f328c68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d28c6b, SUC_lexicon_tags_25 },
    { 0x62528c6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb328c70, SUC_lexicon_tags_6 },
    { 0xd4328c71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b28c77, SUC_lexicon_tags_68 },
    { 0xb0728c78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb728c7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc728c81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc5d28c83, SUC_lexicon_tags_4 },
    { 0xaf528c83, SUC_lexicon_tags_46 },
    { 0xcff28c85, SUC_lexicon_tags_47 },
    { 0x3c928c84, SUC_lexicon_tags_17 },
    { 0x7a128c84, SUC_lexicon_tags_43 },
    { 0x2cf28c88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9d28c8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf28c8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80728c8f, SUC_lexicon_tags_2 },
    { 0x97f28c90, SUC_lexicon_tags_2 },
    { 0xa5728c91, SUC_lexicon_tags_7 },
    { 0xf2328c92, SUC_lexicon_tags_37 },
    { 0x40328c93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc2928c95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6128c99, SUC_lexicon_tags_1 },
    { 0xc9f28c99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58328c9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe528c9f, SUC_lexicon_tags_54 },
    { 0x2b728ca0, SUC_lexicon_tags_14 },
    { 0x7d328ca0, SUC_lexicon_tags_19 },
    { 0x94328ca0, SUC_lexicon_tags_11 },
    { 0x2eb28ca3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1ad28ca5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x48d28ca7, SUC_lexicon_tags_45 },
    { 0x60b28ca7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6f528caa, SUC_lexicon_tags_38 },
    { 0xa0d28caa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0128cb0, SUC_lexicon_tags_7 },
    { 0x64128cb1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x77528cb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b28cb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95328cba, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x18728cbc, SUC_lexicon_tags_30 },
    { 0x1fb28cbd, SUC_lexicon_tags_42 },
    { 0xf6328cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92528cc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42128cc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8528cc8, SUC_lexicon_tags_10 },
    { 0x3e128cc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xde328ccb, SUC_lexicon_tags_63 },
    { 0xed928ccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd28cce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37328cd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d28cd4, SUC_lexicon_tags_0 },
    { 0xdd328cd4, SUC_lexicon_tags_27 },
    { 0xbb28cd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d28cdc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb5d28cde, SUC_lexicon_tags_25 },
    { 0x28128cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b28ce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb28ce5, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x69f28ce7, SUC_lexicon_tags_82 },
    { 0x20928ce7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0d28cea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd28ced, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8928cf3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x31728cf5, SUC_lexicon_tags_26 },
    { 0xcef28cf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31b28cf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d28cfc, SUC_lexicon_tags_11 },
    { 0x3ff28cfd, SUC_lexicon_tags_12 },
    { 0x37728cfe, SUC_lexicon_tags_0 },
    { 0x85b28cff, SUC_lexicon_tags_23 },
    { 0x2db28cfc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa328d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28928d09, SUC_lexicon_tags_28 },
    { 0x62328d0a, SUC_lexicon_tags_12 },
    { 0xa5f28d09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4528d0e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f28d12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d28d15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90728d1a, SUC_lexicon_tags_33 },
    { 0x89f28d1b, SUC_lexicon_tags_15 },
    { 0xb3728d1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab728d1e, SUC_lexicon_tags_45 },
    { 0xb7d28d1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f528d21, SUC_lexicon_tags_47 },
    { 0x7df28d21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a128d24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7328d28, SUC_lexicon_tags_4 },
    { 0xe7d28d29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb28d2c, SUC_lexicon_tags_21 },
    { 0x25f28d2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e128d30, SUC_lexicon_tags_2 },
    { 0x41128d30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9fb28d33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x15928d36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57b28d38, SUC_lexicon_tags_38 },
    { 0x83328d38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d28d3d, SUC_lexicon_tags_3 },
    { 0x79f28d3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd728d41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40928d44, SUC_lexicon_tags_25 },
    { 0x29728d45, SUC_lexicon_tags_2 },
    { 0x9928d45, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55528d53, SUC_lexicon_tags_37 },
    { 0x8b128d54, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9328d58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xff328d5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a128d5d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6728d60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb7728d62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff28d66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc128d69, SUC_lexicon_tags_87 },
    { 0xb4b28d69, SUC_lexicon_tags_11 },
    { 0x75f28d6b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b28d6f, SUC_lexicon_tags_37 },
    { 0xf4728d70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x87b28d72, SUC_lexicon_tags_60 },
    { 0x35f28d72, SUC_lexicon_tags_27 },
    { 0x73d28d74, SUC_lexicon_tags_68 },
    { 0x84528d75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1b28d77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec128d7e, SUC_lexicon_tags_8 },
    { 0x6c128d7f, SUC_lexicon_tags_1 },
    { 0x17928d7f, SUC_lexicon_tags_12 },
    { 0xc4b28d80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x58528d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85928d88, SUC_lexicon_tags_39 },
    { 0x4ff28d89, SUC_lexicon_tags_29 },
    { 0x71128d8a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2128d91, SUC_lexicon_tags_46 },
    { 0x2fb28d92, SUC_lexicon_tags_25 },
    { 0xb8528d93, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7528d96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55928d9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86728d9e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x36928da0, SUC_lexicon_tags_6 },
    { 0xf9d28da1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf28da3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e728da5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85528daa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17728dac, SUC_lexicon_tags_49 },
    { 0x9f328dac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x29128daf, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe528db5, SUC_lexicon_tags_12 },
    { 0xc4728db6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0128db9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d128dbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdff28dc0, SUC_lexicon_tags_25 },
    { 0xa9328dc1, SUC_lexicon_tags_15 },
    { 0x13f28dc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f28dc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a728dca, SUC_lexicon_tags_82 },
    { 0xe8f28dca, SUC_lexicon_tags_2 },
    { 0x30328dcc, SUC_lexicon_tags_18 },
    { 0x29728dcd, SUC_lexicon_tags_7 },
    { 0x9eb28dcb, SUC_lexicon_tags_0 },
    { 0x93728dcf, SUC_lexicon_tags_19 },
    { 0x16928dcf, SUC_lexicon_tags_49 },
    { 0x9d728dd1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6328dd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d928dda, SUC_lexicon_tags_27 },
    { 0x6a128ddb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d28dde, SUC_lexicon_tags_37 },
    { 0x8d928dde, SUC_lexicon_tags_12 },
    { 0xc0328de0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9328de4, SUC_lexicon_tags_0 },
    { 0x65b28de5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f728de8, SUC_lexicon_tags_25 },
    { 0xfa328de8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23528deb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd528dee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7528df1, SUC_lexicon_tags_25 },
    { 0x18b28df1, SUC_lexicon_tags_12 },
    { 0x31328df3, SUC_lexicon_tags_10 },
    { 0x65b28df3, SUC_lexicon_tags_47 },
    { 0xf5528df4, SUC_lexicon_tags_10 },
    { 0x86328df5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a528dfb, SUC_lexicon_tags_2 },
    { 0x9d28dfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e928e02, SUC_lexicon_tags_55 },
    { 0xf6128e02, SUC_lexicon_tags_2 },
    { 0x7e928e04, SUC_lexicon_tags_30 },
    { 0x3ff28e03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d28e08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f28e0c, SUC_lexicon_tags_17 },
    { 0xf4b28e0d, SUC_lexicon_tags_37 },
    { 0x1bd28e0e, SUC_lexicon_tags_7 },
    { 0x24128e0c, SUC_lexicon_tags_23 },
    { 0x19928e10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x88d28e12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacb28e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33928e17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76128e19, SUC_lexicon_tags_14 },
    { 0x59328e1a, SUC_lexicon_tags_17 },
    { 0x58f28e1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef28e1e, SUC_lexicon_tags_21 },
    { 0x2df28e1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d28e22, SUC_lexicon_tags_26 },
    { 0x32f28e23, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd7328e25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x59128e28, SUC_lexicon_tags_27 },
    { 0x36d28e28, SUC_lexicon_tags_12 },
    { 0x72728e28, SUC_lexicon_tags_6 },
    { 0xfaf28e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b28e34, SUC_lexicon_tags_10 },
    { 0x88528e34, SUC_lexicon_tags_25 },
    { 0x53b28e36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d28e39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc528e3c, SUC_lexicon_tags_27 },
    { 0x9f328e3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d928e3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab28e49, SUC_lexicon_tags_4 },
    { 0x99528e49, SUC_lexicon_tags_27 },
    { 0xa9d28e4b, SUC_lexicon_tags_11 },
    { 0xfb928e4c, SUC_lexicon_tags_0 },
    { 0x33128e4d, SUC_lexicon_tags_1 },
    { 0xaed28e4a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb28e51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5d528e53, SUC_lexicon_tags_21 },
    { 0x2328e54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x73f28e57, SUC_lexicon_tags_43 },
    { 0x1f728e58, SUC_lexicon_tags_7 },
    { 0xf3b28e57, SUC_lexicon_tags_47 },
    { 0x4ad28e57, SUC_lexicon_tags_7 },
    { 0x45928e5b, SUC_lexicon_tags_60 },
    { 0x6f928e5c, SUC_lexicon_tags_60 },
    { 0x94b28e5c, SUC_lexicon_tags_23 },
    { 0x24928e5e, SUC_lexicon_tags_10 },
    { 0x61728e57, SUC_lexicon_tags_1 },
    { 0x54128e60, SUC_lexicon_tags_35 },
    { 0x58528e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc728e65, SUC_lexicon_tags_37 },
    { 0x67528e66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa8728e68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x52328e6a, SUC_lexicon_tags_7 },
    { 0xc2328e6b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x69728e6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b328e72, SUC_lexicon_tags_6 },
    { 0x66528e72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1db28e75, SUC_lexicon_tags_27 },
    { 0x31128e75, SUC_lexicon_tags_9 },
    { 0xff28e75, SUC_lexicon_tags_19 },
    { 0x42728e78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1528e7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57d28e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b128e81, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5928e87, SUC_lexicon_tags_29 },
    { 0xbd528e88, SUC_lexicon_tags_12 },
    { 0xc8928e89, SUC_lexicon_tags_9 },
    { 0x89f28e8a, SUC_lexicon_tags_4 },
    { 0x42328e8b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83928e91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b28e94, SUC_lexicon_tags_31 },
    { 0x41328e95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b928e97, SUC_lexicon_tags_0 },
    { 0x3af28e97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4128e9c, SUC_lexicon_tags_2 },
    { 0x6eb28e9d, SUC_lexicon_tags_33 },
    { 0x88928e9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d28ea1, SUC_lexicon_tags_6 },
    { 0x62528ea2, SUC_lexicon_tags_17 },
    { 0x1c928ea2, SUC_lexicon_tags_17 },
    { 0xdf528ea4, SUC_lexicon_tags_9 },
    { 0x9bb28ea5, SUC_lexicon_tags_2 },
    { 0xb6728ea6, SUC_lexicon_tags_27 },
    { 0x20f28ea2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf28eab, SUC_lexicon_tags_49 },
    { 0x9b928eac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47328eb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39328eba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x42728ebc, SUC_lexicon_tags_62 },
    { 0xbc528ebc, SUC_lexicon_tags_37 },
    { 0xb4128ebe, SUC_lexicon_tags_6 },
    { 0x13d28ebe, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb28ec4, SUC_lexicon_tags_17 },
    { 0xa728ec5, SUC_lexicon_tags_37 },
    { 0x46f28ec6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb928ec9, SUC_lexicon_tags_38 },
    { 0x5d928eca, SUC_lexicon_tags_45 },
    { 0xa5b28eca, SUC_lexicon_tags_37 },
    { 0x20328ecc, SUC_lexicon_tags_1 },
    { 0xa9d28ecc, SUC_lexicon_tags_68 },
    { 0x26b28ecd, SUC_lexicon_tags_75 },
    { 0x20f28ecd, SUC_lexicon_tags_43 },
    { 0x1dd28ed0, SUC_lexicon_tags_29 },
    { 0xfa528ed1, SUC_lexicon_tags_15 },
    { 0x9e728ed0, SUC_lexicon_tags_15 },
    { 0xb7728ed3, SUC_lexicon_tags_7 },
    { 0x9e928ed4, SUC_lexicon_tags_25 },
    { 0x60528ed5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc5328ed7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1928edb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63328ee1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8528ee5, SUC_lexicon_tags_43 },
    { 0x15128ee5, SUC_lexicon_tags_7 },
    { 0xa4128ee7, SUC_lexicon_tags_2 },
    { 0xff328ee5, SUC_lexicon_tags_43 },
    { 0xb0128ee9, SUC_lexicon_tags_7 },
    { 0xb4d28ee9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d28ef0, SUC_lexicon_tags_29 },
    { 0xbb528ef1, SUC_lexicon_tags_2 },
    { 0xbfb28ef0, SUC_lexicon_tags_38 },
    { 0x86128ef3, SUC_lexicon_tags_23 },
    { 0x7128ef4, SUC_lexicon_tags_1 },
    { 0xf9928ef4, SUC_lexicon_tags_11 },
    { 0x67728ef6, SUC_lexicon_tags_42 },
    { 0x9c328ef7, SUC_lexicon_tags_32 },
    { 0x88b28ef4, SUC_lexicon_tags_21 },
    { 0x3d528ef4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67928eff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4528f03, SUC_lexicon_tags_33 },
    { 0x928f04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91728f0c, SUC_lexicon_tags_2 },
    { 0x3528f0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9728f15, SUC_lexicon_tags_49 },
    { 0x3ed28f16, SUC_lexicon_tags_19 },
    { 0xcbb28f17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8528f1b, SUC_lexicon_tags_7 },
    { 0x74f28f1c, SUC_lexicon_tags_61 },
    { 0x6a328f1d, SUC_lexicon_tags_38 },
    { 0xbf328f1b, SUC_lexicon_tags_33 },
    { 0x5ff28f1f, SUC_lexicon_tags_11 },
    { 0x8a528f20, SUC_lexicon_tags_1 },
    { 0x81728f20, SUC_lexicon_tags_63 },
    { 0xd3328f1c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb328f24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b528f27, SUC_lexicon_tags_13 },
    { 0x6dd28f27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ff28f2a, SUC_lexicon_tags_17 },
    { 0x14b28f2b, SUC_lexicon_tags_2 },
    { 0x10928f2c, SUC_lexicon_tags_14 },
    { 0xad128f2d, SUC_lexicon_tags_0 },
    { 0x67728f2a, SUC_lexicon_tags_26 },
    { 0x4df28f2f, SUC_lexicon_tags_1 },
    { 0xf6f28f2d, SUC_lexicon_tags_43 },
    { 0x27928f31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79328f3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7928f3e, SUC_lexicon_tags_25 },
    { 0xe5928f3e, SUC_lexicon_tags_12 },
    { 0x40d28f40, SUC_lexicon_tags_34 },
    { 0x53328f41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b28f45, SUC_lexicon_tags_11 },
    { 0xb5728f46, SUC_lexicon_tags_2 },
    { 0xde328f47, SUC_lexicon_tags_45 },
    { 0x2d928f48, SUC_lexicon_tags_7 },
    { 0x86128f48, SUC_lexicon_tags_77 },
    { 0x35f28f47, SUC_lexicon_tags_23 },
    { 0x82728f45, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b28f58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39f28f5a, SUC_lexicon_tags_12 },
    { 0x36128f5a, SUC_lexicon_tags_9 },
    { 0x4e128f5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f28f62, SUC_lexicon_tags_55 },
    { 0xd9528f62, SUC_lexicon_tags_11 },
    { 0x5b328f64, SUC_lexicon_tags_5 },
    { 0xa9128f65, SUC_lexicon_tags_8 },
    { 0x40928f65, SUC_lexicon_tags_8 },
    { 0xc928f66, SUC_lexicon_tags_7 },
    { 0x33b28f64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b28f74, SUC_lexicon_tags_28 },
    { 0x2d528f75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83528f7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84928f82, SUC_lexicon_tags_1337 },
    { 0, NULL },
    { 0xf3b28f84, SUC_lexicon_tags_14 },
    { 0x63928f85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad28f89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad28f8d, SUC_lexicon_tags_37 },
    { 0xcf728f8d, SUC_lexicon_tags_15 },
    { 0x61928f8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1328f99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb28fa0, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb28faa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9ab28fac, SUC_lexicon_tags_4 },
    { 0x33928fad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe128fb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23928fb3, SUC_lexicon_tags_37 },
    { 0xd9b28fb3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa6d28fb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95328fc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97528fc3, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b28fc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4b28fc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0f28fcb, SUC_lexicon_tags_23 },
    { 0x3b328fcc, SUC_lexicon_tags_25 },
    { 0x20328fcd, SUC_lexicon_tags_14 },
    { 0xd4b28fce, SUC_lexicon_tags_33 },
    { 0xce528fcf, SUC_lexicon_tags_19 },
    { 0x92328fcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa328fd2, SUC_lexicon_tags_37 },
    { 0xd9d28fd3, SUC_lexicon_tags_60 },
    { 0x63328fd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d28fd6, SUC_lexicon_tags_38 },
    { 0xe2328fd7, SUC_lexicon_tags_11 },
    { 0xaa728fd8, SUC_lexicon_tags_31 },
    { 0x1eb28fd9, SUC_lexicon_tags_10 },
    { 0x11b28fd9, SUC_lexicon_tags_140 },
    { 0x33528fdb, SUC_lexicon_tags_29 },
    { 0xb5528fdc, SUC_lexicon_tags_44 },
    { 0x2ad28fdd, SUC_lexicon_tags_45 },
    { 0xc7528fde, SUC_lexicon_tags_23 },
    { 0xc528fdd, SUC_lexicon_tags_11 },
    { 0x63328fe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd528fe3, SUC_lexicon_tags_16 },
    { 0x86d28fe3, SUC_lexicon_tags_19 },
    { 0x34328fe3, SUC_lexicon_tags_61 },
    { 0xbfd28fe3, SUC_lexicon_tags_15 },
    { 0x9e528fe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd128fea, SUC_lexicon_tags_31 },
    { 0xde328feb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4728ff0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d928ff4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e328ff7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5c728ff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29d28ffb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaaf28ffd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b29000, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa129002, SUC_lexicon_tags_2 },
    { 0xa0929003, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f29006, SUC_lexicon_tags_8 },
    { 0x66129007, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b2900a, SUC_lexicon_tags_35 },
    { 0x7cf2900b, SUC_lexicon_tags_12 },
    { 0xa672900c, SUC_lexicon_tags_7 },
    { 0xc12900c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49129010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f29013, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58529015, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4329019, SUC_lexicon_tags_15 },
    { 0x4f529019, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf972901c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af2901f, SUC_lexicon_tags_0 },
    { 0x22f29020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ab29022, SUC_lexicon_tags_27 },
    { 0x2ab29023, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf129026, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6329029, SUC_lexicon_tags_26 },
    { 0x57929029, SUC_lexicon_tags_17 },
    { 0x4452902b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8929030, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x38129033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2929035, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd632903a, SUC_lexicon_tags_3 },
    { 0x3612903b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf2903f, SUC_lexicon_tags_11 },
    { 0x27d29040, SUC_lexicon_tags_91 },
    { 0x86929041, SUC_lexicon_tags_11 },
    { 0xf5129042, SUC_lexicon_tags_48 },
    { 0x1b329042, SUC_lexicon_tags_45 },
    { 0xda729044, SUC_lexicon_tags_17 },
    { 0xec929045, SUC_lexicon_tags_46 },
    { 0x11b29044, SUC_lexicon_tags_23 },
    { 0x3d329047, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad2904c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b2904f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff929051, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d29055, SUC_lexicon_tags_17 },
    { 0x78129056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df2905c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0529060, SUC_lexicon_tags_74 },
    { 0x7bb29060, SUC_lexicon_tags_45 },
    { 0x6fb29061, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40d29064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95d29066, SUC_lexicon_tags_49 },
    { 0xac129066, SUC_lexicon_tags_19 },
    { 0x3cf29068, SUC_lexicon_tags_43 },
    { 0x13929069, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9812906b, SUC_lexicon_tags_15 },
    { 0x9f32906c, SUC_lexicon_tags_2 },
    { 0x74f2906d, SUC_lexicon_tags_9 },
    { 0x5692906e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f29075, SUC_lexicon_tags_15 },
    { 0x78929076, SUC_lexicon_tags_2 },
    { 0xa4f29076, SUC_lexicon_tags_19 },
    { 0xdaf29078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2952907b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa529083, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93329086, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12529093, SUC_lexicon_tags_14 },
    { 0x73329093, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78129098, SUC_lexicon_tags_0 },
    { 0x13529099, SUC_lexicon_tags_2 },
    { 0xacf29098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded2909f, SUC_lexicon_tags_4 },
    { 0xe73290a0, SUC_lexicon_tags_39 },
    { 0x647290a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd290a3, SUC_lexicon_tags_19 },
    { 0x927290a4, SUC_lexicon_tags_25 },
    { 0x1e1290a5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x13f290a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x39f290a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x559290ab, SUC_lexicon_tags_27 },
    { 0x41290ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7290b0, SUC_lexicon_tags_75 },
    { 0x899290b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27290b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355290b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7290bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d290c1, SUC_lexicon_tags_60 },
    { 0x1ad290c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f290cb, SUC_lexicon_tags_0 },
    { 0x697290cb, SUC_lexicon_tags_47 },
    { 0xdbf290cb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xae1290cf, SUC_lexicon_tags_25 },
    { 0x563290d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b290d4, SUC_lexicon_tags_6 },
    { 0x4c3290d5, SUC_lexicon_tags_11 },
    { 0xf6b290d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x601290d8, SUC_lexicon_tags_2 },
    { 0x97d290d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1290dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5290e6, SUC_lexicon_tags_10 },
    { 0x4ed290e6, SUC_lexicon_tags_2 },
    { 0x8b3290e7, SUC_lexicon_tags_7 },
    { 0xa87290e7, SUC_lexicon_tags_24 },
    { 0xe3d290ea, SUC_lexicon_tags_7 },
    { 0xfeb290e9, SUC_lexicon_tags_620 },
    { 0, NULL },
    { 0x18d290ed, SUC_lexicon_tags_15 },
    { 0xfc9290ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x231290f0, SUC_lexicon_tags_37 },
    { 0x21f290f1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc41290f3, SUC_lexicon_tags_85 },
    { 0xf290f4, SUC_lexicon_tags_33 },
    { 0x32b290f5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f290f9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xe37290fb, SUC_lexicon_tags_25 },
    { 0xf03290fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf2910a, SUC_lexicon_tags_11 },
    { 0x6a92910b, SUC_lexicon_tags_37 },
    { 0x4cf2910c, SUC_lexicon_tags_18 },
    { 0x6012910c, SUC_lexicon_tags_45 },
    { 0x2772910b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85129112, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d29115, SUC_lexicon_tags_37 },
    { 0xdd929116, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b29119, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa672911c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9929122, SUC_lexicon_tags_11 },
    { 0xdf29123, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x129125, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23129130, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d29133, SUC_lexicon_tags_49 },
    { 0x4cf29133, SUC_lexicon_tags_6 },
    { 0xd9529135, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda529139, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf29143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f2914d, SUC_lexicon_tags_15 },
    { 0xefb2914d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f29152, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc6729154, SUC_lexicon_tags_6 },
    { 0x48329155, SUC_lexicon_tags_29 },
    { 0xbb529156, SUC_lexicon_tags_9 },
    { 0xf5329156, SUC_lexicon_tags_2 },
    { 0x6729157, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc32915f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x31329161, SUC_lexicon_tags_12 },
    { 0x5ad29162, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27329164, SUC_lexicon_tags_28 },
    { 0xab129164, SUC_lexicon_tags_1 },
    { 0x4a329165, SUC_lexicon_tags_11 },
    { 0xd6329167, SUC_lexicon_tags_17 },
    { 0x4b929168, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9312916d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba72916f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd29173, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x93729175, SUC_lexicon_tags_9 },
    { 0x1df29176, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2917a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e529180, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9929184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd29188, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9212918a, SUC_lexicon_tags_0 },
    { 0xa92918b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf372918d, SUC_lexicon_tags_14 },
    { 0x7352918e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd529194, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c129199, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c92919b, SUC_lexicon_tags_29 },
    { 0xc032919c, SUC_lexicon_tags_8 },
    { 0xc012919d, SUC_lexicon_tags_7 },
    { 0xc512919d, SUC_lexicon_tags_37 },
    { 0x3612919e, SUC_lexicon_tags_38 },
    { 0xbab291a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535291a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x473291ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab291b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ff291b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7291b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5291b9, SUC_lexicon_tags_18 },
    { 0x409291ba, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb291c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937291ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec3291cc, SUC_lexicon_tags_17 },
    { 0xbb5291cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f291d2, SUC_lexicon_tags_45 },
    { 0x69f291d2, SUC_lexicon_tags_37 },
    { 0x4c1291d4, SUC_lexicon_tags_45 },
    { 0x333291d4, SUC_lexicon_tags_17 },
    { 0x2a3291d6, SUC_lexicon_tags_11 },
    { 0x459291d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7291da, SUC_lexicon_tags_32 },
    { 0x509291da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37291df, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85291e4, SUC_lexicon_tags_4 },
    { 0xde5291e4, SUC_lexicon_tags_4 },
    { 0xe89291e6, SUC_lexicon_tags_17 },
    { 0xc65291e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215291eb, SUC_lexicon_tags_37 },
    { 0xe67291eb, SUC_lexicon_tags_63 },
    { 0x815291ed, SUC_lexicon_tags_15 },
    { 0xfc3291ee, SUC_lexicon_tags_29 },
    { 0x38b291ed, SUC_lexicon_tags_13 },
    { 0x2d1291ee, SUC_lexicon_tags_1 },
    { 0x223291ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd291f5, SUC_lexicon_tags_19 },
    { 0x487291f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccd291f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb09291fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1291fc, SUC_lexicon_tags_10 },
    { 0xa17291fd, SUC_lexicon_tags_9 },
    { 0xb6f291fe, SUC_lexicon_tags_25 },
    { 0x17b291fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e12920b, SUC_lexicon_tags_12 },
    { 0xb1f2920b, SUC_lexicon_tags_1 },
    { 0x8bb2920b, SUC_lexicon_tags_23 },
    { 0x8c72920d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x99529211, SUC_lexicon_tags_27 },
    { 0x11329212, SUC_lexicon_tags_30 },
    { 0x2e129213, SUC_lexicon_tags_15 },
    { 0xb6129213, SUC_lexicon_tags_7 },
    { 0x1e129215, SUC_lexicon_tags_12 },
    { 0xa0b29216, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab2921f, SUC_lexicon_tags_23 },
    { 0xc129220, SUC_lexicon_tags_60 },
    { 0xf1b29220, SUC_lexicon_tags_17 },
    { 0xc2529220, SUC_lexicon_tags_25 },
    { 0xa8329223, SUC_lexicon_tags_118 },
    { 0xa9929222, SUC_lexicon_tags_34 },
    { 0x1a729222, SUC_lexicon_tags_9 },
    { 0xca129226, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d29229, SUC_lexicon_tags_11 },
    { 0xa8129229, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8729231, SUC_lexicon_tags_11 },
    { 0xfcf29232, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d29236, SUC_lexicon_tags_37 },
    { 0xf1329236, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57129240, SUC_lexicon_tags_1 },
    { 0x46b29241, SUC_lexicon_tags_0 },
    { 0xf9729241, SUC_lexicon_tags_12 },
    { 0x30929241, SUC_lexicon_tags_12 },
    { 0x52d29244, SUC_lexicon_tags_47 },
    { 0xf8f29245, SUC_lexicon_tags_26 },
    { 0x36929246, SUC_lexicon_tags_14 },
    { 0xbbf29247, SUC_lexicon_tags_61 },
    { 0x14529241, SUC_lexicon_tags_4 },
    { 0xe5329249, SUC_lexicon_tags_38 },
    { 0x3a92924a, SUC_lexicon_tags_17 },
    { 0x42d2924a, SUC_lexicon_tags_30 },
    { 0x81129246, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6db2924e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x98329251, SUC_lexicon_tags_12 },
    { 0xb1929251, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2129258, SUC_lexicon_tags_12 },
    { 0xeb29259, SUC_lexicon_tags_37 },
    { 0x7d52925a, SUC_lexicon_tags_11 },
    { 0xc1f2925b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff72925d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f529262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0329265, SUC_lexicon_tags_15 },
    { 0x63b29265, SUC_lexicon_tags_12 },
    { 0x9cf29265, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x3772926a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8132926d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6732926f, SUC_lexicon_tags_38 },
    { 0x66f29270, SUC_lexicon_tags_4 },
    { 0x45b2926f, SUC_lexicon_tags_15 },
    { 0x50329272, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df2927a, SUC_lexicon_tags_8 },
    { 0xe5d2927b, SUC_lexicon_tags_37 },
    { 0xf2927c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1972927e, SUC_lexicon_tags_7 },
    { 0xeab2927f, SUC_lexicon_tags_37 },
    { 0xa1729280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad29283, SUC_lexicon_tags_10 },
    { 0x5b729284, SUC_lexicon_tags_7 },
    { 0x5b29285, SUC_lexicon_tags_17 },
    { 0x93729285, SUC_lexicon_tags_4 },
    { 0x68d29287, SUC_lexicon_tags_38 },
    { 0xbfb29288, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1d2928a, SUC_lexicon_tags_0 },
    { 0xadd2928b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a929293, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129299, SUC_lexicon_tags_11 },
    { 0x79d2929a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2812929d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb292a2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff292a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f292ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd8f292ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5292b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1292b5, SUC_lexicon_tags_17 },
    { 0xa2d292b6, SUC_lexicon_tags_22 },
    { 0x5f1292b6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6b3292b9, SUC_lexicon_tags_53 },
    { 0x33d292ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x843292bc, SUC_lexicon_tags_15 },
    { 0x1c5292bd, SUC_lexicon_tags_17 },
    { 0x447292bd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b292c5, SUC_lexicon_tags_37 },
    { 0xf8f292c6, SUC_lexicon_tags_14 },
    { 0x759292c7, SUC_lexicon_tags_15 },
    { 0x6eb292c8, SUC_lexicon_tags_6 },
    { 0x51d292c9, SUC_lexicon_tags_47 },
    { 0x6dd292ca, SUC_lexicon_tags_45 },
    { 0xe7d292c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b292d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4b292d3, SUC_lexicon_tags_0 },
    { 0xab292d4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb292dc, SUC_lexicon_tags_0 },
    { 0x573292dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af292e0, SUC_lexicon_tags_14 },
    { 0xb27292e1, SUC_lexicon_tags_47 },
    { 0x7e9292e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573292e8, SUC_lexicon_tags_11 },
    { 0xe5d292e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5292f3, SUC_lexicon_tags_1 },
    { 0xd15292f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9292f7, SUC_lexicon_tags_1 },
    { 0x577292f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f292fc, SUC_lexicon_tags_8 },
    { 0xc8d292fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1bf292ff, SUC_lexicon_tags_25 },
    { 0xcf529300, SUC_lexicon_tags_11 },
    { 0x23529301, SUC_lexicon_tags_1 },
    { 0x69d29301, SUC_lexicon_tags_7 },
    { 0x74129301, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb2930b, SUC_lexicon_tags_68 },
    { 0x3c32930b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9129311, SUC_lexicon_tags_45 },
    { 0x6c729312, SUC_lexicon_tags_2 },
    { 0x7a129313, SUC_lexicon_tags_17 },
    { 0x3e129314, SUC_lexicon_tags_49 },
    { 0x68929315, SUC_lexicon_tags_15 },
    { 0x1b29316, SUC_lexicon_tags_12 },
    { 0x97f29317, SUC_lexicon_tags_27 },
    { 0xb4f29316, SUC_lexicon_tags_15 },
    { 0xdc729312, SUC_lexicon_tags_47 },
    { 0xd6f2931a, SUC_lexicon_tags_50 },
    { 0x752931a, SUC_lexicon_tags_2 },
    { 0x70b2931b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd2931f, SUC_lexicon_tags_17 },
    { 0xd9929320, SUC_lexicon_tags_37 },
    { 0xbbf2931f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff129328, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2bf2932a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb2932f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f729332, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df29335, SUC_lexicon_tags_12 },
    { 0x51529336, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf2933f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa329343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46529345, SUC_lexicon_tags_80 },
    { 0x9d929346, SUC_lexicon_tags_30 },
    { 0xdd529347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f129349, SUC_lexicon_tags_77 },
    { 0x1272934a, SUC_lexicon_tags_33 },
    { 0xc492934b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0f2934d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b129350, SUC_lexicon_tags_14 },
    { 0xbc129351, SUC_lexicon_tags_14 },
    { 0xbd729352, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6129355, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db29358, SUC_lexicon_tags_8 },
    { 0xb9129359, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa112935b, SUC_lexicon_tags_29 },
    { 0x83f2935c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e72935e, SUC_lexicon_tags_1 },
    { 0x19b2935f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf0329361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ef29363, SUC_lexicon_tags_16 },
    { 0xa9929363, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc112936a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x57b2936c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c92936f, SUC_lexicon_tags_11 },
    { 0x96529370, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90729372, SUC_lexicon_tags_11 },
    { 0x7b729372, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7cb29375, SUC_lexicon_tags_7 },
    { 0x85b29376, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb592937c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x81b2937e, SUC_lexicon_tags_47 },
    { 0x7fd2937f, SUC_lexicon_tags_11 },
    { 0xaa92937f, SUC_lexicon_tags_19 },
    { 0x71f29381, SUC_lexicon_tags_14 },
    { 0x86d29382, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94329384, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc3729386, SUC_lexicon_tags_25 },
    { 0x1e729387, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3d129389, SUC_lexicon_tags_45 },
    { 0xff2938a, SUC_lexicon_tags_37 },
    { 0x46b29389, SUC_lexicon_tags_17 },
    { 0xcbd2938a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5b12938e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf329392, SUC_lexicon_tags_28 },
    { 0xb3729392, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1729395, SUC_lexicon_tags_21 },
    { 0xc5f29395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27329399, SUC_lexicon_tags_0 },
    { 0x25d2939a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb72939c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f52939f, SUC_lexicon_tags_37 },
    { 0xbd7293a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5293a4, SUC_lexicon_tags_29 },
    { 0x609293a5, SUC_lexicon_tags_16 },
    { 0xfa1293a5, SUC_lexicon_tags_15 },
    { 0x899293a7, SUC_lexicon_tags_7 },
    { 0xb83293a8, SUC_lexicon_tags_1 },
    { 0x129293a8, SUC_lexicon_tags_6 },
    { 0x5a9293aa, SUC_lexicon_tags_27 },
    { 0x6bb293a6, SUC_lexicon_tags_7 },
    { 0xa4d293a7, SUC_lexicon_tags_68 },
    { 0x4e3293ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd293b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3293bd, SUC_lexicon_tags_65 },
    { 0xb09293bd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4e7293c0, SUC_lexicon_tags_1 },
    { 0xd33293c1, SUC_lexicon_tags_66 },
    { 0xf03293c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x223293c4, SUC_lexicon_tags_17 },
    { 0x47d293c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x43f293c7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x93293ca, SUC_lexicon_tags_46 },
    { 0xa53293cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x121293cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91293d3, SUC_lexicon_tags_17 },
    { 0xeff293d3, SUC_lexicon_tags_23 },
    { 0x793293d5, SUC_lexicon_tags_11 },
    { 0xf1d293d3, SUC_lexicon_tags_2 },
    { 0xaaf293d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25293db, SUC_lexicon_tags_2 },
    { 0x595293dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe37293de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857293e2, SUC_lexicon_tags_27 },
    { 0xb8b293e3, SUC_lexicon_tags_53 },
    { 0xeb9293e4, SUC_lexicon_tags_68 },
    { 0xdf5293e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579293ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x741293ef, SUC_lexicon_tags_15 },
    { 0x175293f0, SUC_lexicon_tags_31 },
    { 0x2f293f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421293f5, SUC_lexicon_tags_37 },
    { 0xc77293f5, SUC_lexicon_tags_21 },
    { 0xa73293f7, SUC_lexicon_tags_25 },
    { 0x73d293f8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45293fe, SUC_lexicon_tags_17 },
    { 0x50b293ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf29405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25529408, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb2940b, SUC_lexicon_tags_17 },
    { 0x9cf2940c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38b2940e, SUC_lexicon_tags_23 },
    { 0xe572940e, SUC_lexicon_tags_27 },
    { 0xdc12940f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x15529412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc929414, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2929418, SUC_lexicon_tags_37 },
    { 0x6a529419, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f92941b, SUC_lexicon_tags_12 },
    { 0x2172941c, SUC_lexicon_tags_37 },
    { 0x6e32941c, SUC_lexicon_tags_23 },
    { 0xb752941b, SUC_lexicon_tags_9 },
    { 0xd7f2941f, SUC_lexicon_tags_56 },
    { 0x2f529420, SUC_lexicon_tags_69 },
    { 0x92f29420, SUC_lexicon_tags_42 },
    { 0xb7f29420, SUC_lexicon_tags_14 },
    { 0x31b29420, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa5729425, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc329429, SUC_lexicon_tags_26 },
    { 0x44b29429, SUC_lexicon_tags_39 },
    { 0xd212942b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e52942d, SUC_lexicon_tags_49 },
    { 0xbad2942d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb129436, SUC_lexicon_tags_4 },
    { 0x2e129437, SUC_lexicon_tags_33 },
    { 0x42b29437, SUC_lexicon_tags_11 },
    { 0xc0129439, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf32943e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbed29440, SUC_lexicon_tags_14 },
    { 0x75f29440, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d329443, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd29448, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc352944c, SUC_lexicon_tags_33 },
    { 0x46d2944c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c129451, SUC_lexicon_tags_5 },
    { 0x75929452, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f129456, SUC_lexicon_tags_7 },
    { 0x37729457, SUC_lexicon_tags_91 },
    { 0xddd29458, SUC_lexicon_tags_0 },
    { 0x97729459, SUC_lexicon_tags_33 },
    { 0x52d29459, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6b2945c, SUC_lexicon_tags_11 },
    { 0xc1f2945d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbd72945f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c129466, SUC_lexicon_tags_15 },
    { 0x4b529467, SUC_lexicon_tags_49 },
    { 0xed329467, SUC_lexicon_tags_17 },
    { 0xf0b29469, SUC_lexicon_tags_17 },
    { 0x72946a, SUC_lexicon_tags_31 },
    { 0xb9b2946b, SUC_lexicon_tags_1 },
    { 0xc472946c, SUC_lexicon_tags_11 },
    { 0x1072946c, SUC_lexicon_tags_29 },
    { 0xe2b2946e, SUC_lexicon_tags_1 },
    { 0xb2b2946d, SUC_lexicon_tags_12 },
    { 0x632946d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96329474, SUC_lexicon_tags_49 },
    { 0x95529475, SUC_lexicon_tags_25 },
    { 0xa7329476, SUC_lexicon_tags_15 },
    { 0xda129477, SUC_lexicon_tags_82 },
    { 0x9cb29478, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4332947a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0929482, SUC_lexicon_tags_23 },
    { 0x1e129483, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b529486, SUC_lexicon_tags_37 },
    { 0x8d329487, SUC_lexicon_tags_26 },
    { 0x15529488, SUC_lexicon_tags_7 },
    { 0xfcd29488, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe452948d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f29494, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3892949b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d294a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65d294a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x343294a4, SUC_lexicon_tags_12 },
    { 0xa4b294a5, SUC_lexicon_tags_37 },
    { 0xecf294a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267294b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7294b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1294b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7294bc, SUC_lexicon_tags_17 },
    { 0x3a5294bd, SUC_lexicon_tags_25 },
    { 0xc4b294bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b294c3, SUC_lexicon_tags_15 },
    { 0x7f5294c4, SUC_lexicon_tags_48 },
    { 0xf9b294c5, SUC_lexicon_tags_60 },
    { 0xefd294c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b294c9, SUC_lexicon_tags_55 },
    { 0x5c1294ca, SUC_lexicon_tags_16 },
    { 0x9b7294c9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x531294cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7294d4, SUC_lexicon_tags_14 },
    { 0xa81294d4, SUC_lexicon_tags_12 },
    { 0xb5294d6, SUC_lexicon_tags_17 },
    { 0x35b294d4, SUC_lexicon_tags_37 },
    { 0xa01294d8, SUC_lexicon_tags_16 },
    { 0x247294d5, SUC_lexicon_tags_32 },
    { 0xcd7294da, SUC_lexicon_tags_15 },
    { 0x2e1294db, SUC_lexicon_tags_48 },
    { 0xef5294dc, SUC_lexicon_tags_17 },
    { 0x445294dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x411294e0, SUC_lexicon_tags_2 },
    { 0x899294e1, SUC_lexicon_tags_25 },
    { 0x529294e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc9294e4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07294e7, SUC_lexicon_tags_32 },
    { 0xb8f294e7, SUC_lexicon_tags_6 },
    { 0x11b294e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d294ec, SUC_lexicon_tags_25 },
    { 0x5a1294ed, SUC_lexicon_tags_11 },
    { 0x9bf294ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f294f2, SUC_lexicon_tags_49 },
    { 0x1bd294f3, SUC_lexicon_tags_14 },
    { 0xa3b294f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c7294f6, SUC_lexicon_tags_48 },
    { 0xdf1294f7, SUC_lexicon_tags_14 },
    { 0x4e9294f7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7294fd, SUC_lexicon_tags_47 },
    { 0x483294fe, SUC_lexicon_tags_11 },
    { 0xb99294ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6529502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd29508, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5012950c, SUC_lexicon_tags_77 },
    { 0x4812950d, SUC_lexicon_tags_43 },
    { 0x61f2950e, SUC_lexicon_tags_25 },
    { 0x2792950f, SUC_lexicon_tags_11 },
    { 0xcd729510, SUC_lexicon_tags_8 },
    { 0xd329511, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3329513, SUC_lexicon_tags_1 },
    { 0xa2729513, SUC_lexicon_tags_33 },
    { 0x8d729515, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6612951c, SUC_lexicon_tags_46 },
    { 0x6132951c, SUC_lexicon_tags_7 },
    { 0x72d2951e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d29521, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11729528, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b2952c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e52952e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3329532, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x59529535, SUC_lexicon_tags_4 },
    { 0x40b29535, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b129539, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b2953d, SUC_lexicon_tags_39 },
    { 0xf7d2953e, SUC_lexicon_tags_77 },
    { 0x7912953f, SUC_lexicon_tags_19 },
    { 0xd0b29540, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d29544, SUC_lexicon_tags_31 },
    { 0x25129544, SUC_lexicon_tags_26 },
    { 0xfff29545, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6732954c, SUC_lexicon_tags_172 },
    { 0xb532954d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8032954f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a329552, SUC_lexicon_tags_26 },
    { 0x7329552, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x35729555, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb29558, SUC_lexicon_tags_32 },
    { 0xb6929559, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6652955e, SUC_lexicon_tags_46 },
    { 0x17b2955e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f29562, SUC_lexicon_tags_17 },
    { 0x6f729563, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f29568, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd052956e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0129571, SUC_lexicon_tags_1 },
    { 0x29329571, SUC_lexicon_tags_27 },
    { 0xfb729573, SUC_lexicon_tags_32 },
    { 0xff129574, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d29577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d2957d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41529586, SUC_lexicon_tags_25 },
    { 0xc8329587, SUC_lexicon_tags_25 },
    { 0x47729588, SUC_lexicon_tags_15 },
    { 0xd7b29587, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4832958d, SUC_lexicon_tags_224 },
    { 0xe112958d, SUC_lexicon_tags_82 },
    { 0xcb12958f, SUC_lexicon_tags_11 },
    { 0xc5b2958f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68529592, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4cb29594, SUC_lexicon_tags_9 },
    { 0x86529595, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9929599, SUC_lexicon_tags_15 },
    { 0x6ef2959a, SUC_lexicon_tags_8 },
    { 0x1652959b, SUC_lexicon_tags_33 },
    { 0x5852959b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3052959e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef295a2, SUC_lexicon_tags_12 },
    { 0x4f1295a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33295aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe99295ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x309295af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42d295b1, SUC_lexicon_tags_25 },
    { 0x82d295b2, SUC_lexicon_tags_17 },
    { 0xbb5295b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393295b9, SUC_lexicon_tags_25 },
    { 0xf23295b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x13b295bc, SUC_lexicon_tags_29 },
    { 0x4e9295bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x263295bf, SUC_lexicon_tags_11 },
    { 0xc1295c0, SUC_lexicon_tags_57 },
    { 0x9e9295c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d295c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73295c8, SUC_lexicon_tags_6 },
    { 0xbe7295c8, SUC_lexicon_tags_1 },
    { 0xf79295c8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb295cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d295d1, SUC_lexicon_tags_29 },
    { 0xadd295d1, SUC_lexicon_tags_66 },
    { 0x75d295d3, SUC_lexicon_tags_29 },
    { 0x4f295d1, SUC_lexicon_tags_9 },
    { 0x227295d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9295dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b295e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d295e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403295ea, SUC_lexicon_tags_37 },
    { 0xd33295eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b295ee, SUC_lexicon_tags_17 },
    { 0xc61295ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdad295f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8dd295f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x641295f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211295fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe33295fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ed295fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f29606, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b29609, SUC_lexicon_tags_39 },
    { 0x9372960a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed2960d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93329611, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x42129614, SUC_lexicon_tags_3 },
    { 0x91929614, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x56329617, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6972961b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x972961d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1772961f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe172962d, SUC_lexicon_tags_56 },
    { 0xdcb2962d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d29632, SUC_lexicon_tags_6 },
    { 0x45129633, SUC_lexicon_tags_7 },
    { 0x2a529632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b29637, SUC_lexicon_tags_2 },
    { 0xda729638, SUC_lexicon_tags_17 },
    { 0xdd29638, SUC_lexicon_tags_10 },
    { 0xbed2963a, SUC_lexicon_tags_11 },
    { 0x8cb2963b, SUC_lexicon_tags_25 },
    { 0x9c729637, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b529640, SUC_lexicon_tags_31 },
    { 0xd9f29641, SUC_lexicon_tags_17 },
    { 0xc9129640, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d29648, SUC_lexicon_tags_12 },
    { 0x4fb29648, SUC_lexicon_tags_12 },
    { 0xa9f2964a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcd12964c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdfb2964e, SUC_lexicon_tags_37 },
    { 0x7d12964e, SUC_lexicon_tags_14 },
    { 0x1ed29650, SUC_lexicon_tags_1 },
    { 0x91529651, SUC_lexicon_tags_60 },
    { 0x43729652, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53329656, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c72965c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36329661, SUC_lexicon_tags_37 },
    { 0xacb29662, SUC_lexicon_tags_47 },
    { 0x4f729662, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd29666, SUC_lexicon_tags_14 },
    { 0x56129667, SUC_lexicon_tags_29 },
    { 0x93729666, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f32966c, SUC_lexicon_tags_65 },
    { 0x8d92966d, SUC_lexicon_tags_2 },
    { 0xa3d2966e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33129673, SUC_lexicon_tags_38 },
    { 0xe729673, SUC_lexicon_tags_38 },
    { 0x3fd29673, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd29678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1912967c, SUC_lexicon_tags_11 },
    { 0xb832967d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb29687, SUC_lexicon_tags_9 },
    { 0xbcd29688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6d2968a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6132968d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x612968f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66329692, SUC_lexicon_tags_25 },
    { 0x90f29692, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x16129696, SUC_lexicon_tags_11 },
    { 0xe7f29696, SUC_lexicon_tags_55 },
    { 0x34729698, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd2969d, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd296a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17296a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291296a8, SUC_lexicon_tags_129 },
    { 0xee3296a9, SUC_lexicon_tags_0 },
    { 0xc0d296a9, SUC_lexicon_tags_19 },
    { 0x835296a9, SUC_lexicon_tags_7 },
    { 0x1b7296ac, SUC_lexicon_tags_47 },
    { 0x329296ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcf296af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411296b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f296bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965296c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x673296c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x903296c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb296ca, SUC_lexicon_tags_37 },
    { 0x949296cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c7296cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x943296cf, SUC_lexicon_tags_65 },
    { 0xf49296d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39296d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f296d8, SUC_lexicon_tags_15 },
    { 0x841296d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93296dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec7296df, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3296e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x837296e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347296e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42d296ea, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b296ed, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95296f9, SUC_lexicon_tags_23 },
    { 0xb1b296fa, SUC_lexicon_tags_37 },
    { 0x1c1296fb, SUC_lexicon_tags_47 },
    { 0x77b296fb, SUC_lexicon_tags_4 },
    { 0x8b9296fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d929700, SUC_lexicon_tags_25 },
    { 0x13129701, SUC_lexicon_tags_15 },
    { 0x8a329702, SUC_lexicon_tags_29 },
    { 0x43b29700, SUC_lexicon_tags_27 },
    { 0x8e529704, SUC_lexicon_tags_17 },
    { 0xff929704, SUC_lexicon_tags_49 },
    { 0xc2b29706, SUC_lexicon_tags_43 },
    { 0x8a329704, SUC_lexicon_tags_25 },
    { 0x5d529705, SUC_lexicon_tags_63 },
    { 0x89d29704, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f2970f, SUC_lexicon_tags_38 },
    { 0xfcb2970f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e129718, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb2971c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fb2971e, SUC_lexicon_tags_1 },
    { 0x6e52971e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1729721, SUC_lexicon_tags_17 },
    { 0xc8929722, SUC_lexicon_tags_15 },
    { 0xe0b29723, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a929727, SUC_lexicon_tags_33 },
    { 0x50b29728, SUC_lexicon_tags_15 },
    { 0xecd29728, SUC_lexicon_tags_47 },
    { 0x4252972a, SUC_lexicon_tags_43 },
    { 0x6cd2972a, SUC_lexicon_tags_1 },
    { 0xa412972c, SUC_lexicon_tags_3 },
    { 0x4a52972d, SUC_lexicon_tags_12 },
    { 0x65d29728, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a529733, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd29739, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd12973c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecb2973e, SUC_lexicon_tags_25 },
    { 0xd792973e, SUC_lexicon_tags_1 },
    { 0x692973f, SUC_lexicon_tags_10 },
    { 0xa3f29741, SUC_lexicon_tags_17 },
    { 0x57f29741, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5129745, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb872974a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x752974c, SUC_lexicon_tags_12 },
    { 0xf132974c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b29750, SUC_lexicon_tags_11 },
    { 0xe3529751, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x31f29753, SUC_lexicon_tags_6 },
    { 0x65729753, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc52975b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9db2975d, SUC_lexicon_tags_60 },
    { 0xd532975d, SUC_lexicon_tags_11 },
    { 0x6bb2975f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed529766, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f29769, SUC_lexicon_tags_67 },
    { 0xbc529769, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6392976f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d29773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f29777, SUC_lexicon_tags_17 },
    { 0x1ab29778, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf612977b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef929785, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf29789, SUC_lexicon_tags_37 },
    { 0x1c12978a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f12978c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb2978f, SUC_lexicon_tags_60 },
    { 0x86129790, SUC_lexicon_tags_15 },
    { 0x45d29791, SUC_lexicon_tags_2 },
    { 0x63b29790, SUC_lexicon_tags_38 },
    { 0x9e129793, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52729796, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x8df29798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f32979d, SUC_lexicon_tags_27 },
    { 0x7972979d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x61b297a0, SUC_lexicon_tags_25 },
    { 0xd55297a1, SUC_lexicon_tags_49 },
    { 0x2cf297a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x749297a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21297a8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9297ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967297b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f297b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523297bc, SUC_lexicon_tags_25 },
    { 0xd45297bc, SUC_lexicon_tags_1 },
    { 0x249297be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79297c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7af297c3, SUC_lexicon_tags_12 },
    { 0xc63297c4, SUC_lexicon_tags_25 },
    { 0x891297c5, SUC_lexicon_tags_37 },
    { 0x619297c3, SUC_lexicon_tags_15 },
    { 0x5c9297c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805297cb, SUC_lexicon_tags_12 },
    { 0x797297cc, SUC_lexicon_tags_16 },
    { 0xd33297cd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x315297d0, SUC_lexicon_tags_38 },
    { 0xccb297d1, SUC_lexicon_tags_12 },
    { 0x95297d0, SUC_lexicon_tags_1 },
    { 0x4a7297d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71297d7, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x5eb297d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4b297db, SUC_lexicon_tags_7 },
    { 0xff1297dc, SUC_lexicon_tags_19 },
    { 0x5e1297dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03297e1, SUC_lexicon_tags_11 },
    { 0x9fb297e2, SUC_lexicon_tags_7 },
    { 0x8f9297e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49297e9, SUC_lexicon_tags_21 },
    { 0x563297ea, SUC_lexicon_tags_237 },
    { 0xd69297eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b9297ed, SUC_lexicon_tags_31 },
    { 0xb91297ed, SUC_lexicon_tags_2 },
    { 0xcc3297ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x609297f1, SUC_lexicon_tags_1 },
    { 0xabb297f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11529803, SUC_lexicon_tags_32 },
    { 0xca729804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2329809, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd2980c, SUC_lexicon_tags_7 },
    { 0x9052980c, SUC_lexicon_tags_17 },
    { 0xa9d2980d, SUC_lexicon_tags_37 },
    { 0x8352980f, SUC_lexicon_tags_1 },
    { 0x25729810, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7929814, SUC_lexicon_tags_0 },
    { 0xe4329815, SUC_lexicon_tags_27 },
    { 0x82729815, SUC_lexicon_tags_25 },
    { 0x29b29816, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8532981a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc672981c, SUC_lexicon_tags_12 },
    { 0x9d2981c, SUC_lexicon_tags_7 },
    { 0x24d2981e, SUC_lexicon_tags_27 },
    { 0xb8d2981f, SUC_lexicon_tags_37 },
    { 0x12929820, SUC_lexicon_tags_38 },
    { 0xef32981c, SUC_lexicon_tags_17 },
    { 0xca32981c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef29825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cb29827, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87729829, SUC_lexicon_tags_17 },
    { 0xb732982a, SUC_lexicon_tags_1 },
    { 0xa412982b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd232982d, SUC_lexicon_tags_11 },
    { 0xc452982e, SUC_lexicon_tags_43 },
    { 0x8a92982f, SUC_lexicon_tags_1 },
    { 0x3e729830, SUC_lexicon_tags_45 },
    { 0xbdd2982d, SUC_lexicon_tags_7 },
    { 0xdf529832, SUC_lexicon_tags_5 },
    { 0x13929833, SUC_lexicon_tags_26 },
    { 0x7172982d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8929839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc352983b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82529841, SUC_lexicon_tags_160 },
    { 0x8129842, SUC_lexicon_tags_17 },
    { 0x7bd29843, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77529849, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x412984c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x792984e, SUC_lexicon_tags_11 },
    { 0x1912984f, SUC_lexicon_tags_68 },
    { 0x80529850, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf929854, SUC_lexicon_tags_11 },
    { 0x39529855, SUC_lexicon_tags_15 },
    { 0x30d29856, SUC_lexicon_tags_2 },
    { 0x4bd29857, SUC_lexicon_tags_15 },
    { 0xef529857, SUC_lexicon_tags_25 },
    { 0x2ad29858, SUC_lexicon_tags_43 },
    { 0xae32985a, SUC_lexicon_tags_25 },
    { 0xebd2985b, SUC_lexicon_tags_37 },
    { 0xa252985b, SUC_lexicon_tags_15 },
    { 0x40b2985d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73f2985f, SUC_lexicon_tags_17 },
    { 0xd132985f, SUC_lexicon_tags_37 },
    { 0xe9929860, SUC_lexicon_tags_17 },
    { 0x66929862, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x70529864, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50129866, SUC_lexicon_tags_12 },
    { 0xeff29866, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab2986b, SUC_lexicon_tags_37 },
    { 0xeeb2986c, SUC_lexicon_tags_37 },
    { 0x3f32986c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d29870, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe7b29872, SUC_lexicon_tags_2 },
    { 0x1d929873, SUC_lexicon_tags_17 },
    { 0x6fb29873, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4692987a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38929884, SUC_lexicon_tags_27 },
    { 0x2f29885, SUC_lexicon_tags_75 },
    { 0x15929886, SUC_lexicon_tags_0 },
    { 0x19929886, SUC_lexicon_tags_37 },
    { 0x58b29888, SUC_lexicon_tags_98 },
    { 0x74129889, SUC_lexicon_tags_25 },
    { 0x6092988a, SUC_lexicon_tags_7 },
    { 0xb3d2988b, SUC_lexicon_tags_9 },
    { 0xcfd29885, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf1b2988e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62929890, SUC_lexicon_tags_15 },
    { 0x40129891, SUC_lexicon_tags_63 },
    { 0xba729892, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8529896, SUC_lexicon_tags_6 },
    { 0xc4f29897, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4d329899, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673298a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd298aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x803298ac, SUC_lexicon_tags_1 },
    { 0xedd298ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43298b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe71298b2, SUC_lexicon_tags_45 },
    { 0xbd5298b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xea5298b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af298b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd89298bb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x523298be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb9298c0, SUC_lexicon_tags_17 },
    { 0x1f9298c1, SUC_lexicon_tags_38 },
    { 0x1c3298c2, SUC_lexicon_tags_84 },
    { 0xc9d298c3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b298c7, SUC_lexicon_tags_11 },
    { 0x225298c8, SUC_lexicon_tags_27 },
    { 0xb2d298c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23298d2, SUC_lexicon_tags_1 },
    { 0xf97298d2, SUC_lexicon_tags_27 },
    { 0x3bb298d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3298d8, SUC_lexicon_tags_25 },
    { 0x7d5298d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5298db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9b298dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x979298df, SUC_lexicon_tags_1 },
    { 0x189298e0, SUC_lexicon_tags_66 },
    { 0x1b1298df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7298e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f298ec, SUC_lexicon_tags_38 },
    { 0x31f298ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89298f0, SUC_lexicon_tags_0 },
    { 0x28f298f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f298f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x57298f5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab298f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655298fd, SUC_lexicon_tags_7 },
    { 0xb5f298fd, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xc7729900, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b29903, SUC_lexicon_tags_17 },
    { 0x51529904, SUC_lexicon_tags_168 },
    { 0xd4929905, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3e729907, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x79529909, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbd32990b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd9b2990d, SUC_lexicon_tags_11 },
    { 0xc292990d, SUC_lexicon_tags_38 },
    { 0xb772990f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f29916, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x43129918, SUC_lexicon_tags_13 },
    { 0xc7b29918, SUC_lexicon_tags_37 },
    { 0x1092991a, SUC_lexicon_tags_390 },
    { 0x4492991b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90729922, SUC_lexicon_tags_68 },
    { 0xd6529923, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e129928, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d2992d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75529930, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d29933, SUC_lexicon_tags_29 },
    { 0xd7529933, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e929936, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49929939, SUC_lexicon_tags_2 },
    { 0x2292993a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b12993c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a12993e, SUC_lexicon_tags_49 },
    { 0xb052993f, SUC_lexicon_tags_1 },
    { 0x67f29940, SUC_lexicon_tags_35 },
    { 0x6bd2993f, SUC_lexicon_tags_32 },
    { 0xf0329942, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c129945, SUC_lexicon_tags_19 },
    { 0xfc129946, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2652994a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d329950, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53529957, SUC_lexicon_tags_17 },
    { 0x84729958, SUC_lexicon_tags_40 },
    { 0xb7529957, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d72995b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb532995d, SUC_lexicon_tags_47 },
    { 0x612995e, SUC_lexicon_tags_37 },
    { 0x81d2995e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f29963, SUC_lexicon_tags_57 },
    { 0xfcf29963, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d29967, SUC_lexicon_tags_12 },
    { 0x34929968, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1812996b, SUC_lexicon_tags_25 },
    { 0x4f32996c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf672996f, SUC_lexicon_tags_49 },
    { 0x4ff2996f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d29973, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad29976, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x32129978, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf692997a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4929984, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd872998e, SUC_lexicon_tags_23 },
    { 0x8052998f, SUC_lexicon_tags_2 },
    { 0xf6b2998e, SUC_lexicon_tags_66 },
    { 0x3929991, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x41729993, SUC_lexicon_tags_7 },
    { 0x1bd29994, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf29997, SUC_lexicon_tags_23 },
    { 0xd9129997, SUC_lexicon_tags_19 },
    { 0x7a329999, SUC_lexicon_tags_15 },
    { 0x57f29999, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4872999c, SUC_lexicon_tags_30 },
    { 0x92999d, SUC_lexicon_tags_25 },
    { 0xf212999d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9299a6, SUC_lexicon_tags_47 },
    { 0x86f299a7, SUC_lexicon_tags_8 },
    { 0xd25299a8, SUC_lexicon_tags_11 },
    { 0x83299a7, SUC_lexicon_tags_37 },
    { 0xc7d299aa, SUC_lexicon_tags_14 },
    { 0x645299ab, SUC_lexicon_tags_30 },
    { 0x505299a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9299b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8eb299b2, SUC_lexicon_tags_2 },
    { 0xc5299b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13299b6, SUC_lexicon_tags_15 },
    { 0x41d299b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b299bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d299c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1299cd, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179299d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5299d9, SUC_lexicon_tags_12 },
    { 0x701299d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b299df, SUC_lexicon_tags_49 },
    { 0xf35299df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf299e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85299e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7299eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x129299ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x80b299ef, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773299f4, SUC_lexicon_tags_12 },
    { 0x6d1299f5, SUC_lexicon_tags_47 },
    { 0x9f3299f6, SUC_lexicon_tags_12 },
    { 0x5af299f7, SUC_lexicon_tags_37 },
    { 0x8d1299f7, SUC_lexicon_tags_79 },
    { 0x19f299f6, SUC_lexicon_tags_27 },
    { 0xdff299fa, SUC_lexicon_tags_7 },
    { 0x9e7299fa, SUC_lexicon_tags_82 },
    { 0xa1299fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x315299ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b29a02, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a329a07, SUC_lexicon_tags_37 },
    { 0x28b29a07, SUC_lexicon_tags_31 },
    { 0x2b329a09, SUC_lexicon_tags_2 },
    { 0xf6529a0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11129a0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc729a13, SUC_lexicon_tags_275 },
    { 0x58b29a14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed29a18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f29a1d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbcb29a1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f29a23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb29a27, SUC_lexicon_tags_13 },
    { 0xa4f29a28, SUC_lexicon_tags_25 },
    { 0x99729a29, SUC_lexicon_tags_66 },
    { 0xb8b29a28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e129a2c, SUC_lexicon_tags_151 },
    { 0xf6f29a2d, SUC_lexicon_tags_7 },
    { 0xca729a2c, SUC_lexicon_tags_11 },
    { 0xcb729a2c, SUC_lexicon_tags_74 },
    { 0xbd29a30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1329a34, SUC_lexicon_tags_9 },
    { 0x59329a34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76329a39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75529a3f, SUC_lexicon_tags_7 },
    { 0xcb929a40, SUC_lexicon_tags_37 },
    { 0xde529a40, SUC_lexicon_tags_2 },
    { 0x2d329a42, SUC_lexicon_tags_7 },
    { 0xb0329a43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b29a47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7529a4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf29a4d, SUC_lexicon_tags_15 },
    { 0x4ed29a4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b29a56, SUC_lexicon_tags_46 },
    { 0xe1d29a57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16929a59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f29a5c, SUC_lexicon_tags_37 },
    { 0xba329a5c, SUC_lexicon_tags_7 },
    { 0x3e529a5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7729a62, SUC_lexicon_tags_15 },
    { 0xb29a62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9529a65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x32129a67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57129a6d, SUC_lexicon_tags_0 },
    { 0xe3729a6e, SUC_lexicon_tags_113 },
    { 0xcfb29a6e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6129a75, SUC_lexicon_tags_27 },
    { 0xc4d29a76, SUC_lexicon_tags_12 },
    { 0x96129a77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98129a7b, SUC_lexicon_tags_7 },
    { 0xa2329a7c, SUC_lexicon_tags_47 },
    { 0xaaf29a7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8929a83, SUC_lexicon_tags_9 },
    { 0xec129a84, SUC_lexicon_tags_25 },
    { 0x4fb29a85, SUC_lexicon_tags_10 },
    { 0xea129a85, SUC_lexicon_tags_15 },
    { 0x12529a84, SUC_lexicon_tags_9 },
    { 0x9b29a88, SUC_lexicon_tags_40 },
    { 0xf3129a89, SUC_lexicon_tags_10 },
    { 0x16b29a8a, SUC_lexicon_tags_27 },
    { 0x29329a8b, SUC_lexicon_tags_31 },
    { 0x6af29a8c, SUC_lexicon_tags_207 },
    { 0x6dd29a8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67529a90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84b29a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b29a96, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3f529a98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96729a9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89d29a9c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x51729a9e, SUC_lexicon_tags_14 },
    { 0x24d29a9f, SUC_lexicon_tags_1 },
    { 0x13329aa0, SUC_lexicon_tags_23 },
    { 0xa0b29aa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb29aa8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0329aac, SUC_lexicon_tags_60 },
    { 0x21f29aad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d29ab1, SUC_lexicon_tags_68 },
    { 0x2a729ab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95729ab4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8329ab7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2329ab9, SUC_lexicon_tags_15 },
    { 0xc7329aba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1129abe, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x70f29ac0, SUC_lexicon_tags_2 },
    { 0xee329ac1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61729ac5, SUC_lexicon_tags_68 },
    { 0xcc529ac5, SUC_lexicon_tags_2 },
    { 0x69329ac7, SUC_lexicon_tags_14 },
    { 0xebb29ac5, SUC_lexicon_tags_11 },
    { 0xf8d29ac9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d29ace, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72929ad6, SUC_lexicon_tags_37 },
    { 0x77129ad7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2729ade, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf129ae3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7729ae8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdbf29aea, SUC_lexicon_tags_37 },
    { 0xa8129aeb, SUC_lexicon_tags_1 },
    { 0xd3729aeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c729af1, SUC_lexicon_tags_17 },
    { 0x58b29af2, SUC_lexicon_tags_10 },
    { 0x57f29af2, SUC_lexicon_tags_87 },
    { 0xded29af2, SUC_lexicon_tags_7 },
    { 0xdd729af5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb29af7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf129af9, SUC_lexicon_tags_2 },
    { 0x44929afa, SUC_lexicon_tags_7 },
    { 0xdf529afb, SUC_lexicon_tags_25 },
    { 0x8e529afb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d29b05, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa529b08, SUC_lexicon_tags_15 },
    { 0x94b29b09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85d29b0b, SUC_lexicon_tags_31 },
    { 0x72d29b0c, SUC_lexicon_tags_11 },
    { 0xea729b0b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d29b13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb329b17, SUC_lexicon_tags_15 },
    { 0x39729b18, SUC_lexicon_tags_7 },
    { 0xd2329b19, SUC_lexicon_tags_33 },
    { 0xbd929b1a, SUC_lexicon_tags_23 },
    { 0x90129b1b, SUC_lexicon_tags_11 },
    { 0x1c129b1c, SUC_lexicon_tags_2 },
    { 0x36d29b1d, SUC_lexicon_tags_21 },
    { 0x46129b1d, SUC_lexicon_tags_37 },
    { 0x30b29b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c129b21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21129b23, SUC_lexicon_tags_392 },
    { 0, NULL },
    { 0xff729b25, SUC_lexicon_tags_15 },
    { 0x5d929b26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3329b28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55729b2c, SUC_lexicon_tags_37 },
    { 0x6b29b2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba729b31, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6929b34, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd1529b36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55529b3f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5bf29b41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27329b44, SUC_lexicon_tags_31 },
    { 0x76d29b45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b29b48, SUC_lexicon_tags_27 },
    { 0x3f129b49, SUC_lexicon_tags_15 },
    { 0xf3b29b4a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5729b4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab29b52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e329b5a, SUC_lexicon_tags_9 },
    { 0xf129b5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x82929b5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8929b60, SUC_lexicon_tags_68 },
    { 0x8ef29b61, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x15d29b63, SUC_lexicon_tags_43 },
    { 0x35729b64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d29b68, SUC_lexicon_tags_82 },
    { 0xcdb29b69, SUC_lexicon_tags_63 },
    { 0x7f529b6a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6b129b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2329b6e, SUC_lexicon_tags_37 },
    { 0xc4529b6f, SUC_lexicon_tags_2 },
    { 0xc2529b70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf29b72, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x65329b74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37329b76, SUC_lexicon_tags_47 },
    { 0x5bf29b77, SUC_lexicon_tags_77 },
    { 0x9a929b78, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x78929b7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c129b7d, SUC_lexicon_tags_45 },
    { 0x56129b7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac329b83, SUC_lexicon_tags_28 },
    { 0x5d29b84, SUC_lexicon_tags_25 },
    { 0xe3529b85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d29b89, SUC_lexicon_tags_33 },
    { 0xe4529b8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7929b90, SUC_lexicon_tags_12 },
    { 0x3e529b91, SUC_lexicon_tags_25 },
    { 0x5dd29b92, SUC_lexicon_tags_37 },
    { 0x86d29b93, SUC_lexicon_tags_25 },
    { 0x2a129b93, SUC_lexicon_tags_11 },
    { 0x54729b95, SUC_lexicon_tags_62 },
    { 0x10329b95, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6729b99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d529b9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b29b9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b329ba5, SUC_lexicon_tags_25 },
    { 0xeb529ba5, SUC_lexicon_tags_17 },
    { 0x7ef29ba5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f29ba9, SUC_lexicon_tags_45 },
    { 0x9ed29baa, SUC_lexicon_tags_48 },
    { 0xe0b29ba9, SUC_lexicon_tags_48 },
    { 0xdb29baa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31729bb3, SUC_lexicon_tags_44 },
    { 0x51729bb4, SUC_lexicon_tags_2 },
    { 0xf3729bb5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9529bb9, SUC_lexicon_tags_15 },
    { 0x4bd29bba, SUC_lexicon_tags_31 },
    { 0xc4929bb9, SUC_lexicon_tags_4 },
    { 0xd2729bbb, SUC_lexicon_tags_151 },
    { 0xbff29bbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd929bc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d29bca, SUC_lexicon_tags_17 },
    { 0x8a329bcb, SUC_lexicon_tags_45 },
    { 0xa1529bcc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x13329bce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6929bd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d29bd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51b29bd8, SUC_lexicon_tags_3 },
    { 0x44729bd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96729bdb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89129be3, SUC_lexicon_tags_6 },
    { 0xa5329be4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37329be6, SUC_lexicon_tags_49 },
    { 0x54729be7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b29bec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3cf29bee, SUC_lexicon_tags_47 },
    { 0x6c729bef, SUC_lexicon_tags_31 },
    { 0x10b29bf0, SUC_lexicon_tags_63 },
    { 0xd3329bf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f329bf4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f29bf7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xba129bfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f29bff, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b729c02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a329c06, SUC_lexicon_tags_15 },
    { 0x97129c07, SUC_lexicon_tags_7 },
    { 0xea329c08, SUC_lexicon_tags_37 },
    { 0xa4f29c09, SUC_lexicon_tags_12 },
    { 0x19529c08, SUC_lexicon_tags_6 },
    { 0xa3129c0b, SUC_lexicon_tags_46 },
    { 0xe7529c0c, SUC_lexicon_tags_18 },
    { 0xcf929c0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d29c13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3729c15, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f529c19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f29c28, SUC_lexicon_tags_2 },
    { 0x23b29c29, SUC_lexicon_tags_65 },
    { 0x56129c29, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8dd29c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d29c30, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd29c34, SUC_lexicon_tags_11 },
    { 0xaa129c34, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59329c41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0f29c43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f29c47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d29c4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b29c54, SUC_lexicon_tags_6 },
    { 0x69129c54, SUC_lexicon_tags_16 },
    { 0x78329c54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a929c5f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70729c63, SUC_lexicon_tags_0 },
    { 0x1bd29c64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9329c67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b329c6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d29c71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2529c77, SUC_lexicon_tags_82 },
    { 0xb3b29c78, SUC_lexicon_tags_43 },
    { 0x5d329c79, SUC_lexicon_tags_19 },
    { 0xc8729c7a, SUC_lexicon_tags_17 },
    { 0x88929c79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d29c80, SUC_lexicon_tags_46 },
    { 0x40929c80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d29c84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b29c88, SUC_lexicon_tags_1 },
    { 0x8d29c89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ed29c8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaaf29c8d, SUC_lexicon_tags_12 },
    { 0x6a729c8d, SUC_lexicon_tags_17 },
    { 0x99f29c8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e929c91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c329c93, SUC_lexicon_tags_60 },
    { 0xc0d29c94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9329c96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62729c9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe729c9e, SUC_lexicon_tags_47 },
    { 0xee929c9f, SUC_lexicon_tags_7 },
    { 0xb0529ca0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff29ca4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x25d29ca6, SUC_lexicon_tags_0 },
    { 0xe0b29ca6, SUC_lexicon_tags_35 },
    { 0x83b29ca7, SUC_lexicon_tags_31 },
    { 0xa6529ca9, SUC_lexicon_tags_37 },
    { 0x38d29ca8, SUC_lexicon_tags_12 },
    { 0xcab29cab, SUC_lexicon_tags_17 },
    { 0x27f29cac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff29cb1, SUC_lexicon_tags_42 },
    { 0xa2929cb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28729cb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c929cb8, SUC_lexicon_tags_22 },
    { 0x71129cb9, SUC_lexicon_tags_17 },
    { 0x70d29cba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd29cbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb29cc1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf929cc3, SUC_lexicon_tags_28 },
    { 0xbfb29cc4, SUC_lexicon_tags_15 },
    { 0xe8729cc5, SUC_lexicon_tags_66 },
    { 0x4c929cc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef29cca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab929ccc, SUC_lexicon_tags_25 },
    { 0xe2b29ccd, SUC_lexicon_tags_11 },
    { 0x21d29cce, SUC_lexicon_tags_19 },
    { 0x53129ccc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xded29cd2, SUC_lexicon_tags_23 },
    { 0xe6329cd3, SUC_lexicon_tags_37 },
    { 0x53129cd4, SUC_lexicon_tags_55 },
    { 0x79f29cd4, SUC_lexicon_tags_17 },
    { 0x23d29cd6, SUC_lexicon_tags_33 },
    { 0x62929cd7, SUC_lexicon_tags_74 },
    { 0x1cf29cd8, SUC_lexicon_tags_15 },
    { 0x37b29cd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2129ce1, SUC_lexicon_tags_15 },
    { 0xb0329ce2, SUC_lexicon_tags_17 },
    { 0x79929ce3, SUC_lexicon_tags_12 },
    { 0xd9329ce3, SUC_lexicon_tags_27 },
    { 0xe1f29ce3, SUC_lexicon_tags_19 },
    { 0x5bd29ce5, SUC_lexicon_tags_151 },
    { 0x92329ce4, SUC_lexicon_tags_29 },
    { 0x1fd29ce7, SUC_lexicon_tags_7 },
    { 0xadd29ce9, SUC_lexicon_tags_24 },
    { 0xa8529cea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18529cec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b29cf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f29cf7, SUC_lexicon_tags_55 },
    { 0x10129cf8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70929cfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84529cfe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf9729d00, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df29d04, SUC_lexicon_tags_12 },
    { 0xd5129d05, SUC_lexicon_tags_21 },
    { 0xa2b29d06, SUC_lexicon_tags_19 },
    { 0x4fb29d04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62929d0f, SUC_lexicon_tags_31 },
    { 0x1af29d10, SUC_lexicon_tags_1 },
    { 0xe3329d11, SUC_lexicon_tags_23 },
    { 0x5129d11, SUC_lexicon_tags_13 },
    { 0xf4529d13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4cd29d15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd929d1c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b29d1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b29d23, SUC_lexicon_tags_33 },
    { 0x30f29d24, SUC_lexicon_tags_17 },
    { 0x3d29d25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x23529d27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92929d29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18329d2d, SUC_lexicon_tags_17 },
    { 0xef129d2e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f29d31, SUC_lexicon_tags_12 },
    { 0x28729d31, SUC_lexicon_tags_30 },
    { 0xfb529d32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18129d35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d129d39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5329d3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb529d42, SUC_lexicon_tags_10 },
    { 0xe2129d43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf929d54, SUC_lexicon_tags_61 },
    { 0x2d929d55, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf29d5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5929d5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x16129d60, SUC_lexicon_tags_9 },
    { 0xd1129d61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x16729d63, SUC_lexicon_tags_8 },
    { 0x5d929d64, SUC_lexicon_tags_16 },
    { 0x25129d65, SUC_lexicon_tags_17 },
    { 0xfb929d63, SUC_lexicon_tags_45 },
    { 0x5ff29d63, SUC_lexicon_tags_19 },
    { 0xeff29d67, SUC_lexicon_tags_61 },
    { 0x6a729d69, SUC_lexicon_tags_23 },
    { 0x2d529d69, SUC_lexicon_tags_35 },
    { 0x7fb29d68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d729d6d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41329d71, SUC_lexicon_tags_2 },
    { 0x1bb29d72, SUC_lexicon_tags_47 },
    { 0x41529d73, SUC_lexicon_tags_4 },
    { 0x5db29d72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d29d78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a729d7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb8f29d80, SUC_lexicon_tags_1 },
    { 0x1cd29d81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a329d83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8329d86, SUC_lexicon_tags_45 },
    { 0xc9d29d87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb29d8b, SUC_lexicon_tags_0 },
    { 0xccb29d8c, SUC_lexicon_tags_17 },
    { 0x10b29d8d, SUC_lexicon_tags_47 },
    { 0xc1929d8e, SUC_lexicon_tags_6 },
    { 0x62d29d8e, SUC_lexicon_tags_29 },
    { 0x49329d8d, SUC_lexicon_tags_1 },
    { 0x3bb29d91, SUC_lexicon_tags_2 },
    { 0xdef29d92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92529d96, SUC_lexicon_tags_49 },
    { 0xe6f29d96, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7e329d99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f29d9c, SUC_lexicon_tags_48 },
    { 0xa8129d9d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf29da1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1f29da3, SUC_lexicon_tags_1 },
    { 0x55329da3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8729da7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x20d29da9, SUC_lexicon_tags_23 },
    { 0xad129daa, SUC_lexicon_tags_37 },
    { 0xf7929dab, SUC_lexicon_tags_49 },
    { 0x16d29dac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb929db1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1329db8, SUC_lexicon_tags_68 },
    { 0xe5729db9, SUC_lexicon_tags_28 },
    { 0xddf29db8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90929dc0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe7729dc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf29dc5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbbd29dc7, SUC_lexicon_tags_9 },
    { 0xfa729dc7, SUC_lexicon_tags_11 },
    { 0xec529dc9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef329dce, SUC_lexicon_tags_0 },
    { 0x77d29dce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec329dd2, SUC_lexicon_tags_12 },
    { 0xb5d29dd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4929dd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x58329dd8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x26329ddb, SUC_lexicon_tags_26 },
    { 0xd2329ddc, SUC_lexicon_tags_26 },
    { 0xc6529ddc, SUC_lexicon_tags_15 },
    { 0xe5129dde, SUC_lexicon_tags_45 },
    { 0x7bd29dde, SUC_lexicon_tags_17 },
    { 0xdc329ddf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4129de3, SUC_lexicon_tags_26 },
    { 0xd6b29de3, SUC_lexicon_tags_2 },
    { 0x72929de5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8129dec, SUC_lexicon_tags_61 },
    { 0xbbd29dec, SUC_lexicon_tags_17 },
    { 0x71b29dee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e529df0, SUC_lexicon_tags_63 },
    { 0x68d29df1, SUC_lexicon_tags_21 },
    { 0xc7729df2, SUC_lexicon_tags_27 },
    { 0x61729df2, SUC_lexicon_tags_17 },
    { 0x7e129df2, SUC_lexicon_tags_113 },
    { 0xa0329df1, SUC_lexicon_tags_17 },
    { 0xea529df6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d329df8, SUC_lexicon_tags_11 },
    { 0x6ab29df9, SUC_lexicon_tags_38 },
    { 0x31d29dfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63729dfc, SUC_lexicon_tags_47 },
    { 0x5a729dfc, SUC_lexicon_tags_65 },
    { 0xf0329dfd, SUC_lexicon_tags_14 },
    { 0xbb529dff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xec329e01, SUC_lexicon_tags_248 },
    { 0, NULL },
    { 0x41f29e03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x81729e05, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef29e09, SUC_lexicon_tags_6 },
    { 0x15d29e09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa129e0d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x4c529e0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c729e13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f29e16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ed29e18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4929e1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e529e1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3929e24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90129e28, SUC_lexicon_tags_14 },
    { 0x65529e29, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d29e2c, SUC_lexicon_tags_7 },
    { 0x7f729e2c, SUC_lexicon_tags_35 },
    { 0xced29e2e, SUC_lexicon_tags_10 },
    { 0xf2329e2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f29e32, SUC_lexicon_tags_11 },
    { 0xa729e33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b29e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15529e3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97f29e3e, SUC_lexicon_tags_25 },
    { 0xe4d29e3f, SUC_lexicon_tags_17 },
    { 0x1e529e3e, SUC_lexicon_tags_25 },
    { 0xcd929e41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb729e48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2c729e4a, SUC_lexicon_tags_32 },
    { 0xe2329e4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d29e53, SUC_lexicon_tags_4 },
    { 0xd3129e53, SUC_lexicon_tags_31 },
    { 0xc329e53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1129e5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c929e5f, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e129e62, SUC_lexicon_tags_17 },
    { 0xbb729e63, SUC_lexicon_tags_15 },
    { 0xe2d29e64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9929e69, SUC_lexicon_tags_17 },
    { 0xa729e6a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d729e70, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0xb4929e72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac729e74, SUC_lexicon_tags_2 },
    { 0x57329e75, SUC_lexicon_tags_37 },
    { 0xf1729e76, SUC_lexicon_tags_30 },
    { 0x27929e77, SUC_lexicon_tags_15 },
    { 0xea929e78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6bb29e7a, SUC_lexicon_tags_65 },
    { 0xbf29e7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcad29e7d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43b29e7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d29e87, SUC_lexicon_tags_32 },
    { 0xa3b29e88, SUC_lexicon_tags_63 },
    { 0x79929e89, SUC_lexicon_tags_2 },
    { 0xb9929e89, SUC_lexicon_tags_4 },
    { 0xf29e8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44329e8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29529e92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9b729e94, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8ff29e96, SUC_lexicon_tags_25 },
    { 0x82929e96, SUC_lexicon_tags_37 },
    { 0xdab29e98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd29e9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d329ea0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd2529ea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd29ea8, SUC_lexicon_tags_30 },
    { 0xd3729ea9, SUC_lexicon_tags_38 },
    { 0x1f129eaa, SUC_lexicon_tags_19 },
    { 0x4a529eaa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x67929eae, SUC_lexicon_tags_27 },
    { 0x74929eae, SUC_lexicon_tags_2 },
    { 0xe5529eb0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7929eb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b29eb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84129ebd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56729ec1, SUC_lexicon_tags_1 },
    { 0x3bd29ec2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f29ec5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd329ec8, SUC_lexicon_tags_15 },
    { 0xfd729ec9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17729ecc, SUC_lexicon_tags_1 },
    { 0xbff29ecd, SUC_lexicon_tags_21 },
    { 0xb5d29ece, SUC_lexicon_tags_37 },
    { 0x1c329ecf, SUC_lexicon_tags_12 },
    { 0x95529ece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b529ed3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf29ed7, SUC_lexicon_tags_4 },
    { 0x45529ed8, SUC_lexicon_tags_2 },
    { 0xb0b29ed9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e729edd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc929ee3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3329ee7, SUC_lexicon_tags_27 },
    { 0x33529ee7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d29eef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53929ef5, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c729f00, SUC_lexicon_tags_11 },
    { 0x85129f01, SUC_lexicon_tags_37 },
    { 0x45b29f02, SUC_lexicon_tags_11 },
    { 0x5a529f01, SUC_lexicon_tags_25 },
    { 0xbb529f00, SUC_lexicon_tags_23 },
    { 0x8b929f05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd729f08, SUC_lexicon_tags_2 },
    { 0xbc529f08, SUC_lexicon_tags_8 },
    { 0x2a129f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed129f0c, SUC_lexicon_tags_27 },
    { 0x7cf29f0d, SUC_lexicon_tags_28 },
    { 0x80529f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4529f12, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x92529f15, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe0b29f17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54529f1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb29f23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9529f26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce129f2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5329f2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36329f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f29f34, SUC_lexicon_tags_17 },
    { 0x6d129f35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67329f3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f29f40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa329f45, SUC_lexicon_tags_17 },
    { 0x76f29f45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xded29f49, SUC_lexicon_tags_17 },
    { 0xced29f49, SUC_lexicon_tags_19 },
    { 0x5df29f4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd8929f4d, SUC_lexicon_tags_77 },
    { 0x41129f4e, SUC_lexicon_tags_14 },
    { 0xf1b29f4f, SUC_lexicon_tags_30 },
    { 0xa2929f50, SUC_lexicon_tags_1 },
    { 0x4fd29f4e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x81929f53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xec129f56, SUC_lexicon_tags_37 },
    { 0x89129f57, SUC_lexicon_tags_29 },
    { 0x77929f57, SUC_lexicon_tags_108 },
    { 0x7e529f56, SUC_lexicon_tags_15 },
    { 0xbbb29f5a, SUC_lexicon_tags_7 },
    { 0xee329f5b, SUC_lexicon_tags_83 },
    { 0xdc329f5c, SUC_lexicon_tags_33 },
    { 0x5c329f5d, SUC_lexicon_tags_37 },
    { 0x26f29f5e, SUC_lexicon_tags_5 },
    { 0x8cb29f5e, SUC_lexicon_tags_129 },
    { 0x72329f5f, SUC_lexicon_tags_8 },
    { 0x10129f61, SUC_lexicon_tags_11 },
    { 0x88729f61, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x61929f64, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29929f6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc329f70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2729f72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2729f7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93529f7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d29f81, SUC_lexicon_tags_17 },
    { 0xcb929f82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd29f88, SUC_lexicon_tags_49 },
    { 0x2bd29f88, SUC_lexicon_tags_19 },
    { 0xed729f8a, SUC_lexicon_tags_0 },
    { 0xce729f8b, SUC_lexicon_tags_11 },
    { 0xc129f8a, SUC_lexicon_tags_12 },
    { 0x12f29f8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb29f92, SUC_lexicon_tags_28 },
    { 0xb3929f92, SUC_lexicon_tags_17 },
    { 0xda329f93, SUC_lexicon_tags_60 },
    { 0x3f529f95, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d29f99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3329fa6, SUC_lexicon_tags_2 },
    { 0x70f29fa7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd6329fa9, SUC_lexicon_tags_63 },
    { 0x68b29faa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc529fad, SUC_lexicon_tags_7 },
    { 0x29f29fae, SUC_lexicon_tags_19 },
    { 0x53929faf, SUC_lexicon_tags_43 },
    { 0x88929faf, SUC_lexicon_tags_21 },
    { 0xbf129faf, SUC_lexicon_tags_1 },
    { 0x47d29fb2, SUC_lexicon_tags_11 },
    { 0xc9329fb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f29fb6, SUC_lexicon_tags_7 },
    { 0xd7f29fb7, SUC_lexicon_tags_47 },
    { 0x61329fb6, SUC_lexicon_tags_12 },
    { 0xef729fb7, SUC_lexicon_tags_37 },
    { 0x65b29fba, SUC_lexicon_tags_17 },
    { 0x76529fbb, SUC_lexicon_tags_66 },
    { 0xc8d29fbc, SUC_lexicon_tags_15 },
    { 0x42729fb7, SUC_lexicon_tags_23 },
    { 0xb6329fbe, SUC_lexicon_tags_11 },
    { 0x75929fbf, SUC_lexicon_tags_37 },
    { 0xdeb29fc0, SUC_lexicon_tags_15 },
    { 0x3bf29fb7, SUC_lexicon_tags_49 },
    { 0xf5129fbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd29fc8, SUC_lexicon_tags_17 },
    { 0xcf129fc8, SUC_lexicon_tags_87 },
    { 0xbfb29fca, SUC_lexicon_tags_2 },
    { 0xe4529fcb, SUC_lexicon_tags_15 },
    { 0xd9d29fcc, SUC_lexicon_tags_15 },
    { 0x1929fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc929fd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1d729fd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4329fd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c929fd8, SUC_lexicon_tags_28 },
    { 0x1329fd8, SUC_lexicon_tags_1 },
    { 0x8e329fd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b329fdc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce929fe4, SUC_lexicon_tags_26 },
    { 0xc8b29fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e129fea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93b29fec, SUC_lexicon_tags_1 },
    { 0x45729fec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47529ff2, SUC_lexicon_tags_7 },
    { 0x6929ff3, SUC_lexicon_tags_26 },
    { 0xaa329ff2, SUC_lexicon_tags_6 },
    { 0x36d29ff2, SUC_lexicon_tags_38 },
    { 0xa129ff6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9929ff9, SUC_lexicon_tags_21 },
    { 0x40529ffa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6729ffd, SUC_lexicon_tags_45 },
    { 0x59b29ffe, SUC_lexicon_tags_0 },
    { 0x99329fff, SUC_lexicon_tags_25 },
    { 0x57d2a000, SUC_lexicon_tags_15 },
    { 0xc132a001, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc72a004, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b2a00b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae12a00e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2432a010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f2a015, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ff2a017, SUC_lexicon_tags_295 },
    { 0xb932a018, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca32a01d, SUC_lexicon_tags_63 },
    { 0x6ef2a01e, SUC_lexicon_tags_14 },
    { 0x2052a01f, SUC_lexicon_tags_87 },
    { 0xf692a01e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf132a023, SUC_lexicon_tags_37 },
    { 0x9092a024, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x55f2a026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2252a02c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x79d2a02f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712a03b, SUC_lexicon_tags_69 },
    { 0xfa12a03c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c72a03e, SUC_lexicon_tags_60 },
    { 0x7892a03f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48f2a041, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d2a044, SUC_lexicon_tags_17 },
    { 0x2e72a045, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6652a047, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa712a04b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf52a051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde92a058, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7672a05a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28f2a05c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8472a065, SUC_lexicon_tags_33 },
    { 0x7f72a066, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6b2a068, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef2a06c, SUC_lexicon_tags_12 },
    { 0x1a12a06d, SUC_lexicon_tags_37 },
    { 0x4bd2a06e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c52a070, SUC_lexicon_tags_11 },
    { 0x1bf2a071, SUC_lexicon_tags_55 },
    { 0x4472a070, SUC_lexicon_tags_23 },
    { 0xbd2a070, SUC_lexicon_tags_26 },
    { 0x70d2a070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf392a078, SUC_lexicon_tags_9 },
    { 0xbcd2a078, SUC_lexicon_tags_31 },
    { 0x50b2a07a, SUC_lexicon_tags_68 },
    { 0xe52a07b, SUC_lexicon_tags_11 },
    { 0xe3f2a07a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd312a07f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb772a085, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6632a087, SUC_lexicon_tags_27 },
    { 0x8ab2a088, SUC_lexicon_tags_116 },
    { 0x2a92a089, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27b2a08b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2332a08f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2752a097, SUC_lexicon_tags_14 },
    { 0x272a098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a32a09a, SUC_lexicon_tags_25 },
    { 0x7cf2a09a, SUC_lexicon_tags_92 },
    { 0x5352a09c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xea52a09f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf312a0a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2eb2a0a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa912a0a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb92a0ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe612a0ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4072a0b0, SUC_lexicon_tags_63 },
    { 0xd7d2a0b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb32a0b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d2a0b8, SUC_lexicon_tags_10 },
    { 0xe372a0b9, SUC_lexicon_tags_63 },
    { 0x2ff2a0ba, SUC_lexicon_tags_14 },
    { 0xbe92a0ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4992a0be, SUC_lexicon_tags_17 },
    { 0x31b2a0bf, SUC_lexicon_tags_17 },
    { 0x9952a0be, SUC_lexicon_tags_15 },
    { 0x3d72a0bf, SUC_lexicon_tags_32 },
    { 0x4652a0c2, SUC_lexicon_tags_2 },
    { 0xd892a0c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6772a0c6, SUC_lexicon_tags_2 },
    { 0xd0b2a0c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc92a0c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf512a0d2, SUC_lexicon_tags_48 },
    { 0x2b72a0d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61d2a0d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b2a0d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd8d2a0db, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff2a0de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b92a0e2, SUC_lexicon_tags_38 },
    { 0x22b2a0e3, SUC_lexicon_tags_11 },
    { 0xc452a0e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3452a0eb, SUC_lexicon_tags_50 },
    { 0x4192a0eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f2a0ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1352a0f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcd2a0f5, SUC_lexicon_tags_37 },
    { 0xc5f2a0f5, SUC_lexicon_tags_7 },
    { 0x10f2a0f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3192a0ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f2a104, SUC_lexicon_tags_11 },
    { 0x53f2a105, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9ff2a107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d2a10a, SUC_lexicon_tags_14 },
    { 0x2dd2a10b, SUC_lexicon_tags_2 },
    { 0x30b2a10c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3192a115, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a72a117, SUC_lexicon_tags_26 },
    { 0x8052a118, SUC_lexicon_tags_26 },
    { 0x48b2a118, SUC_lexicon_tags_9 },
    { 0x4e72a118, SUC_lexicon_tags_1 },
    { 0x5c92a11b, SUC_lexicon_tags_12 },
    { 0x4052a11c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1912a11e, SUC_lexicon_tags_7 },
    { 0xcf92a11f, SUC_lexicon_tags_1 },
    { 0xc012a11e, SUC_lexicon_tags_47 },
    { 0x9832a11e, SUC_lexicon_tags_6 },
    { 0x8872a122, SUC_lexicon_tags_13 },
    { 0xc832a122, SUC_lexicon_tags_0 },
    { 0xc852a120, SUC_lexicon_tags_2 },
    { 0x7532a123, SUC_lexicon_tags_38 },
    { 0xe132a124, SUC_lexicon_tags_7 },
    { 0x2cf2a127, SUC_lexicon_tags_17 },
    { 0xe372a127, SUC_lexicon_tags_17 },
    { 0x3012a129, SUC_lexicon_tags_7 },
    { 0x2172a12a, SUC_lexicon_tags_12 },
    { 0x6c72a129, SUC_lexicon_tags_10 },
    { 0xa812a12c, SUC_lexicon_tags_43 },
    { 0x652a12d, SUC_lexicon_tags_37 },
    { 0xe612a12e, SUC_lexicon_tags_27 },
    { 0x6ad2a12f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17d2a131, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb92a135, SUC_lexicon_tags_8 },
    { 0x2512a136, SUC_lexicon_tags_25 },
    { 0x59b2a135, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf792a139, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x132a13c, SUC_lexicon_tags_25 },
    { 0xe5d2a13d, SUC_lexicon_tags_17 },
    { 0x6ef2a13e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd2a144, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f2a147, SUC_lexicon_tags_17 },
    { 0x18f2a147, SUC_lexicon_tags_25 },
    { 0x2e32a149, SUC_lexicon_tags_37 },
    { 0xd172a14a, SUC_lexicon_tags_11 },
    { 0xa0d2a14a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc52a14e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d2a153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb732a159, SUC_lexicon_tags_43 },
    { 0x932a15a, SUC_lexicon_tags_66 },
    { 0xc52a15a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecb2a15d, SUC_lexicon_tags_25 },
    { 0x9332a15e, SUC_lexicon_tags_43 },
    { 0x6d52a15f, SUC_lexicon_tags_11 },
    { 0x8c32a160, SUC_lexicon_tags_91 },
    { 0x8492a15d, SUC_lexicon_tags_10 },
    { 0x7ad2a162, SUC_lexicon_tags_11 },
    { 0xec92a162, SUC_lexicon_tags_32 },
    { 0x4852a164, SUC_lexicon_tags_49 },
    { 0x672a162, SUC_lexicon_tags_37 },
    { 0xccf2a161, SUC_lexicon_tags_17 },
    { 0xa12a162, SUC_lexicon_tags_37 },
    { 0x3972a168, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8812a16b, SUC_lexicon_tags_1 },
    { 0x9d72a16b, SUC_lexicon_tags_17 },
    { 0xd632a16d, SUC_lexicon_tags_17 },
    { 0xc672a16e, SUC_lexicon_tags_17 },
    { 0x3d72a16f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e72a172, SUC_lexicon_tags_7 },
    { 0x20d2a173, SUC_lexicon_tags_37 },
    { 0x8d32a172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5472a176, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2dd2a178, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d2a17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d92a17f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d2a182, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f2a187, SUC_lexicon_tags_11 },
    { 0x3252a187, SUC_lexicon_tags_15 },
    { 0xd752a189, SUC_lexicon_tags_47 },
    { 0x8b92a18a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf2a18f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4572a191, SUC_lexicon_tags_43 },
    { 0x4532a192, SUC_lexicon_tags_65 },
    { 0x6172a191, SUC_lexicon_tags_37 },
    { 0x1752a194, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20d2a196, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff2a19e, SUC_lexicon_tags_31 },
    { 0xfc32a19f, SUC_lexicon_tags_224 },
    { 0x88d2a1a0, SUC_lexicon_tags_27 },
    { 0x6212a1a1, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x3392a1a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34b2a1a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe612a1a8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xf832a1aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b92a1ac, SUC_lexicon_tags_7 },
    { 0x9072a1ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d32a1b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x28d2a1b3, SUC_lexicon_tags_25 },
    { 0xa2f2a1b4, SUC_lexicon_tags_25 },
    { 0xb652a1b5, SUC_lexicon_tags_66 },
    { 0xac32a1b4, SUC_lexicon_tags_2 },
    { 0x4332a1b7, SUC_lexicon_tags_6 },
    { 0x3b92a1b8, SUC_lexicon_tags_87 },
    { 0x1e52a1b9, SUC_lexicon_tags_39 },
    { 0x4352a1b6, SUC_lexicon_tags_2 },
    { 0x31d2a1bb, SUC_lexicon_tags_15 },
    { 0x91f2a1b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bf2a1be, SUC_lexicon_tags_0 },
    { 0xf872a1bf, SUC_lexicon_tags_31 },
    { 0x1772a1c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d32a1c2, SUC_lexicon_tags_60 },
    { 0xe3d2a1c2, SUC_lexicon_tags_11 },
    { 0xd72a1c3, SUC_lexicon_tags_12 },
    { 0xc0b2a1c5, SUC_lexicon_tags_11 },
    { 0x672a1c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x232a1cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf9b2a1cd, SUC_lexicon_tags_43 },
    { 0x552a1ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc852a1d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f92a1db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb2a1e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb852a1e7, SUC_lexicon_tags_17 },
    { 0x7cb2a1e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa212a1ed, SUC_lexicon_tags_14 },
    { 0x9832a1ed, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b2a1f2, SUC_lexicon_tags_63 },
    { 0x94f2a1f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f52a1f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d2a1fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d2a205, SUC_lexicon_tags_37 },
    { 0x2d52a206, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb2a209, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa52a20b, SUC_lexicon_tags_68 },
    { 0xfd72a20b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd2a20f, SUC_lexicon_tags_70 },
    { 0x5ef2a20f, SUC_lexicon_tags_53 },
    { 0x92d2a211, SUC_lexicon_tags_14 },
    { 0xc1d2a210, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b2a215, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2712a217, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab2a21c, SUC_lexicon_tags_1 },
    { 0xff52a21d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde52a220, SUC_lexicon_tags_633 },
    { 0xaf12a221, SUC_lexicon_tags_25 },
    { 0x6ef2a222, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb792a225, SUC_lexicon_tags_12 },
    { 0xd172a226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb732a228, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c72a22a, SUC_lexicon_tags_30 },
    { 0x45f2a22a, SUC_lexicon_tags_152 },
    { 0xe1d2a22a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc92a22e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd532a230, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe732a232, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc752a235, SUC_lexicon_tags_2 },
    { 0x4b2a236, SUC_lexicon_tags_14 },
    { 0x5052a236, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e52a23f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x47f2a241, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5792a243, SUC_lexicon_tags_8 },
    { 0x2732a244, SUC_lexicon_tags_27 },
    { 0xe5f2a245, SUC_lexicon_tags_15 },
    { 0x92d2a245, SUC_lexicon_tags_1 },
    { 0xb2f2a247, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x5772a249, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x412a24c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca72a251, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa432a253, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x452a256, SUC_lexicon_tags_12 },
    { 0xdad2a256, SUC_lexicon_tags_15 },
    { 0x6412a257, SUC_lexicon_tags_55 },
    { 0x38f2a259, SUC_lexicon_tags_15 },
    { 0x39d2a259, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b2a25d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f92a261, SUC_lexicon_tags_11 },
    { 0xcf32a262, SUC_lexicon_tags_40 },
    { 0xe812a261, SUC_lexicon_tags_6 },
    { 0x98b2a264, SUC_lexicon_tags_7 },
    { 0x47d2a264, SUC_lexicon_tags_27 },
    { 0x1f52a266, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e12a26c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe52a279, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe032a27b, SUC_lexicon_tags_37 },
    { 0xe872a27c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x64d2a27e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cd2a280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c92a284, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c52a28b, SUC_lexicon_tags_79 },
    { 0x9152a28b, SUC_lexicon_tags_37 },
    { 0x9552a28d, SUC_lexicon_tags_12 },
    { 0xee92a28e, SUC_lexicon_tags_27 },
    { 0xc112a28c, SUC_lexicon_tags_77 },
    { 0x92b2a290, SUC_lexicon_tags_25 },
    { 0x4c92a291, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc432a294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd272a297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b52a29a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9132a2a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba32a2a3, SUC_lexicon_tags_0 },
    { 0xed52a2a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b52a2ac, SUC_lexicon_tags_17 },
    { 0x2352a2ad, SUC_lexicon_tags_15 },
    { 0xef32a2ae, SUC_lexicon_tags_5 },
    { 0xb872a2af, SUC_lexicon_tags_37 },
    { 0x52d2a2ae, SUC_lexicon_tags_27 },
    { 0xbb72a2ae, SUC_lexicon_tags_12 },
    { 0x7852a2b2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3392a2b4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d2a2b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c52a2ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6df2a2bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd132a2c2, SUC_lexicon_tags_29 },
    { 0x1372a2c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d2a2cd, SUC_lexicon_tags_23 },
    { 0x9e12a2ce, SUC_lexicon_tags_15 },
    { 0x21b2a2cf, SUC_lexicon_tags_6 },
    { 0x6272a2cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b2a2d5, SUC_lexicon_tags_37 },
    { 0xecd2a2d6, SUC_lexicon_tags_11 },
    { 0xfc92a2d7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5972a2da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb132a2df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab52a2e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a12a2e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff2a2eb, SUC_lexicon_tags_17 },
    { 0xb532a2ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb2a2f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff12a2f8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5512a2fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b2a2ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3652a301, SUC_lexicon_tags_79 },
    { 0xf192a302, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b2a30b, SUC_lexicon_tags_23 },
    { 0x5592a30c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f52a311, SUC_lexicon_tags_47 },
    { 0x10f2a312, SUC_lexicon_tags_14 },
    { 0x1092a313, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b2a31b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa712a31e, SUC_lexicon_tags_15 },
    { 0xfbd2a31f, SUC_lexicon_tags_48 },
    { 0x94b2a320, SUC_lexicon_tags_4 },
    { 0x1ef2a320, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6972a323, SUC_lexicon_tags_7 },
    { 0x792a323, SUC_lexicon_tags_25 },
    { 0x87b2a323, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d2a32a, SUC_lexicon_tags_45 },
    { 0xf832a32b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2832a32e, SUC_lexicon_tags_17 },
    { 0x2e32a32f, SUC_lexicon_tags_25 },
    { 0xbeb2a32f, SUC_lexicon_tags_25 },
    { 0x7612a331, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb712a336, SUC_lexicon_tags_11 },
    { 0xbe32a337, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e32a339, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4072a33d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf92a341, SUC_lexicon_tags_37 },
    { 0x6112a342, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d2a349, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf2a353, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb2a357, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb3f2a359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3212a35d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa472a35f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd512a361, SUC_lexicon_tags_14 },
    { 0x8fb2a362, SUC_lexicon_tags_1 },
    { 0xedf2a363, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb952a366, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3632a368, SUC_lexicon_tags_25 },
    { 0xf532a369, SUC_lexicon_tags_16 },
    { 0x8992a369, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb2a36e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xceb2a370, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead2a375, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f2a378, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d32a37b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a92a382, SUC_lexicon_tags_7 },
    { 0xf32a383, SUC_lexicon_tags_7 },
    { 0xc8b2a383, SUC_lexicon_tags_17 },
    { 0xa992a384, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xef52a387, SUC_lexicon_tags_7 },
    { 0x3ab2a388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172a38c, SUC_lexicon_tags_12 },
    { 0xa72a38d, SUC_lexicon_tags_42 },
    { 0x7032a38e, SUC_lexicon_tags_23 },
    { 0xc272a38f, SUC_lexicon_tags_1 },
    { 0xd732a38e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb92a393, SUC_lexicon_tags_25 },
    { 0xec12a394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b2a39a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f92a39c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a92a39e, SUC_lexicon_tags_2 },
    { 0xf752a39f, SUC_lexicon_tags_17 },
    { 0x3d12a3a0, SUC_lexicon_tags_17 },
    { 0x1bd2a39f, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad2a3a4, SUC_lexicon_tags_43 },
    { 0x9d52a3a5, SUC_lexicon_tags_7 },
    { 0xfbd2a3a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed2a3aa, SUC_lexicon_tags_2 },
    { 0xbe72a3aa, SUC_lexicon_tags_15 },
    { 0xd392a3aa, SUC_lexicon_tags_27 },
    { 0xc12a3ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb52a3b2, SUC_lexicon_tags_15 },
    { 0xbc92a3b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa312a3be, SUC_lexicon_tags_46 },
    { 0x4a92a3bf, SUC_lexicon_tags_17 },
    { 0xc32a3c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1412a3c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7052a3ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe472a3cc, SUC_lexicon_tags_63 },
    { 0xca32a3cd, SUC_lexicon_tags_0 },
    { 0x152a3cc, SUC_lexicon_tags_45 },
    { 0x34f2a3cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a92a3d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0b2a3d4, SUC_lexicon_tags_35 },
    { 0xd592a3d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9732a3da, SUC_lexicon_tags_33 },
    { 0x9db2a3da, SUC_lexicon_tags_15 },
    { 0x3e72a3db, SUC_lexicon_tags_47 },
    { 0x5fb2a3dc, SUC_lexicon_tags_56 },
    { 0x1f2a3de, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbbb2a3e0, SUC_lexicon_tags_11 },
    { 0x112a3e1, SUC_lexicon_tags_38 },
    { 0x5132a3e1, SUC_lexicon_tags_7 },
    { 0x6cb2a3e3, SUC_lexicon_tags_29 },
    { 0xeb2a3e4, SUC_lexicon_tags_16 },
    { 0x1012a3e1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x212a3e8, SUC_lexicon_tags_9 },
    { 0x7692a3e9, SUC_lexicon_tags_34 },
    { 0x5b72a3ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf172a3ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x34b2a3ee, SUC_lexicon_tags_37 },
    { 0xa0f2a3ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4572a3f5, SUC_lexicon_tags_56 },
    { 0xa3f2a3f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf2a3fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e92a3fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d92a400, SUC_lexicon_tags_31 },
    { 0x8f72a401, SUC_lexicon_tags_17 },
    { 0x1db2a401, SUC_lexicon_tags_4 },
    { 0x5a72a402, SUC_lexicon_tags_17 },
    { 0x16f2a402, SUC_lexicon_tags_87 },
    { 0xccf2a400, SUC_lexicon_tags_11 },
    { 0xf512a406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf872a409, SUC_lexicon_tags_37 },
    { 0xe552a40a, SUC_lexicon_tags_37 },
    { 0xb2b2a409, SUC_lexicon_tags_12 },
    { 0x3912a40c, SUC_lexicon_tags_2 },
    { 0x4bb2a40d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6392a40f, SUC_lexicon_tags_28 },
    { 0xd12a40f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf52a412, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d92a415, SUC_lexicon_tags_47 },
    { 0x57d2a416, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b12a418, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7532a41b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3832a41e, SUC_lexicon_tags_38 },
    { 0xebb2a41f, SUC_lexicon_tags_17 },
    { 0xb932a41e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9052a425, SUC_lexicon_tags_2 },
    { 0x6ef2a426, SUC_lexicon_tags_1 },
    { 0x75d2a426, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5952a429, SUC_lexicon_tags_49 },
    { 0x5212a42a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7092a432, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8872a435, SUC_lexicon_tags_84 },
    { 0xabd2a436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad2a43a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2672a43d, SUC_lexicon_tags_48 },
    { 0xc2d2a43d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f2a440, SUC_lexicon_tags_11 },
    { 0x4e32a440, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa672a443, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b12a446, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d32a449, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3732a44c, SUC_lexicon_tags_743 },
    { 0xb3f2a44c, SUC_lexicon_tags_2 },
    { 0xd452a44e, SUC_lexicon_tags_12 },
    { 0x7f72a44f, SUC_lexicon_tags_71 },
    { 0x8632a44e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2872a454, SUC_lexicon_tags_27 },
    { 0xa452a455, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd52a45d, SUC_lexicon_tags_19 },
    { 0xc4f2a45d, SUC_lexicon_tags_12 },
    { 0x6112a45f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1152a461, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5272a464, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa172a468, SUC_lexicon_tags_15 },
    { 0xe5d2a468, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8752a46c, SUC_lexicon_tags_121 },
    { 0x4492a46d, SUC_lexicon_tags_1059 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb12a470, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b32a473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b2a478, SUC_lexicon_tags_27 },
    { 0xb012a479, SUC_lexicon_tags_9 },
    { 0xb812a47a, SUC_lexicon_tags_28 },
    { 0x8772a478, SUC_lexicon_tags_45 },
    { 0xa012a47a, SUC_lexicon_tags_55 },
    { 0xbfd2a47d, SUC_lexicon_tags_30 },
    { 0xac72a47e, SUC_lexicon_tags_11 },
    { 0x8712a47f, SUC_lexicon_tags_12 },
    { 0xa672a47d, SUC_lexicon_tags_47 },
    { 0x7bf2a481, SUC_lexicon_tags_12 },
    { 0xa7d2a482, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x70b2a484, SUC_lexicon_tags_23 },
    { 0xd472a484, SUC_lexicon_tags_15 },
    { 0xfb12a485, SUC_lexicon_tags_15 },
    { 0x60f2a487, SUC_lexicon_tags_25 },
    { 0x32d2a488, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f32a48b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e12a48e, SUC_lexicon_tags_12 },
    { 0x26b2a48e, SUC_lexicon_tags_2 },
    { 0x4612a490, SUC_lexicon_tags_27 },
    { 0xbc92a48f, SUC_lexicon_tags_17 },
    { 0x1252a492, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc0d2a494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b2a498, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x85b2a49a, SUC_lexicon_tags_53 },
    { 0x3f2a49b, SUC_lexicon_tags_0 },
    { 0x8952a49c, SUC_lexicon_tags_15 },
    { 0xe832a49b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a92a49f, SUC_lexicon_tags_2 },
    { 0x8892a49f, SUC_lexicon_tags_11 },
    { 0xd792a4a0, SUC_lexicon_tags_30 },
    { 0x4852a4a2, SUC_lexicon_tags_15 },
    { 0x7012a4a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf012a4ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a32a4b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc152a4b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b2a4bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd2a4bf, SUC_lexicon_tags_43 },
    { 0x4812a4c0, SUC_lexicon_tags_8 },
    { 0x2852a4c0, SUC_lexicon_tags_33 },
    { 0xf332a4c0, SUC_lexicon_tags_14 },
    { 0xff32a4c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f2a4c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x892a4ce, SUC_lexicon_tags_38 },
    { 0x17b2a4ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96f2a4d1, SUC_lexicon_tags_37 },
    { 0x8812a4d1, SUC_lexicon_tags_77 },
    { 0xe5d2a4d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf272a4d8, SUC_lexicon_tags_63 },
    { 0x83b2a4d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df2a4dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f52a4de, SUC_lexicon_tags_28 },
    { 0xb9d2a4de, SUC_lexicon_tags_48 },
    { 0x96f2a4de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1612a4e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9d12a4e4, SUC_lexicon_tags_26 },
    { 0xd332a4e5, SUC_lexicon_tags_95 },
    { 0xb912a4e4, SUC_lexicon_tags_19 },
    { 0xa4b2a4e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xef52a4e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2172a4ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eb2a4ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f2a4f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2ed2a4f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb192a4f7, SUC_lexicon_tags_23 },
    { 0x2092a4f8, SUC_lexicon_tags_15 },
    { 0xdef2a4f7, SUC_lexicon_tags_17 },
    { 0x8072a4f9, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xcff2a4fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7d2a4fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f92a501, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x512a504, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff2a508, SUC_lexicon_tags_37 },
    { 0x3572a509, SUC_lexicon_tags_32 },
    { 0xac52a508, SUC_lexicon_tags_12 },
    { 0x6af2a508, SUC_lexicon_tags_43 },
    { 0xe6d2a50b, SUC_lexicon_tags_11 },
    { 0x6bb2a50d, SUC_lexicon_tags_17 },
    { 0xfc12a50e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e52a511, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c92a514, SUC_lexicon_tags_37 },
    { 0x9672a515, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5552a519, SUC_lexicon_tags_14 },
    { 0xdc12a519, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc352a51c, SUC_lexicon_tags_71 },
    { 0x35d2a51d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc392a525, SUC_lexicon_tags_39 },
    { 0x2452a526, SUC_lexicon_tags_7 },
    { 0xe0d2a527, SUC_lexicon_tags_38 },
    { 0xff32a528, SUC_lexicon_tags_347 },
    { 0xe692a528, SUC_lexicon_tags_10 },
    { 0x6ef2a52a, SUC_lexicon_tags_2 },
    { 0xe952a527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac12a52d, SUC_lexicon_tags_29 },
    { 0x4d2a52e, SUC_lexicon_tags_51 },
    { 0x7012a52f, SUC_lexicon_tags_17 },
    { 0xdf52a52f, SUC_lexicon_tags_189 },
    { 0x67b2a530, SUC_lexicon_tags_9 },
    { 0xdbb2a532, SUC_lexicon_tags_2 },
    { 0x7b92a533, SUC_lexicon_tags_26 },
    { 0x9e52a534, SUC_lexicon_tags_27 },
    { 0xb2a535, SUC_lexicon_tags_17 },
    { 0x9292a536, SUC_lexicon_tags_65 },
    { 0x44d2a537, SUC_lexicon_tags_74 },
    { 0x90b2a537, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e12a53d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb72a544, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x552a546, SUC_lexicon_tags_12 },
    { 0x5c52a547, SUC_lexicon_tags_15 },
    { 0xea92a547, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f2a54d, SUC_lexicon_tags_17 },
    { 0x6bd2a54e, SUC_lexicon_tags_39 },
    { 0x3532a54f, SUC_lexicon_tags_6 },
    { 0x6432a550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1552a556, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe232a558, SUC_lexicon_tags_1 },
    { 0xfb32a559, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b92a55f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db2a564, SUC_lexicon_tags_12 },
    { 0x1f12a564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7772a569, SUC_lexicon_tags_2 },
    { 0x43b2a569, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5512a56f, SUC_lexicon_tags_65 },
    { 0x55f2a570, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f2a573, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3b72a575, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f2a57b, SUC_lexicon_tags_32 },
    { 0x86f2a57c, SUC_lexicon_tags_15 },
    { 0x4712a57b, SUC_lexicon_tags_57 },
    { 0x72d2a57e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a12a580, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed52a583, SUC_lexicon_tags_45 },
    { 0x7292a583, SUC_lexicon_tags_30 },
    { 0x9052a585, SUC_lexicon_tags_63 },
    { 0x4c32a585, SUC_lexicon_tags_14 },
    { 0x89f2a585, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452a58e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8452a590, SUC_lexicon_tags_37 },
    { 0xe0b2a590, SUC_lexicon_tags_11 },
    { 0x3d52a590, SUC_lexicon_tags_65 },
    { 0x9f2a593, SUC_lexicon_tags_28 },
    { 0x9772a594, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c52a598, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb032a59f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf152a5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2232a5a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9672a5a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ef2a5a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f52a5ae, SUC_lexicon_tags_2 },
    { 0xf112a5af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd632a5b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b2a5b9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5592a5bb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf712a5bd, SUC_lexicon_tags_15 },
    { 0x58b2a5be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad2a5c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f2a5c6, SUC_lexicon_tags_45 },
    { 0xec52a5c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd112a5ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6412a5cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ed2a5cf, SUC_lexicon_tags_29 },
    { 0x2e12a5cf, SUC_lexicon_tags_7 },
    { 0x3472a5d1, SUC_lexicon_tags_27 },
    { 0xec52a5d1, SUC_lexicon_tags_15 },
    { 0x1172a5d1, SUC_lexicon_tags_45 },
    { 0x30d2a5d4, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef52a5d8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6f92a5da, SUC_lexicon_tags_1 },
    { 0x8b12a5da, SUC_lexicon_tags_33 },
    { 0xe552a5dc, SUC_lexicon_tags_19 },
    { 0x58d2a5dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc252a5df, SUC_lexicon_tags_60 },
    { 0x7bf2a5e0, SUC_lexicon_tags_15 },
    { 0x4e92a5e1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d2a5e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1892a5ea, SUC_lexicon_tags_37 },
    { 0x4bd2a5eb, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0xf1d2a5ed, SUC_lexicon_tags_17 },
    { 0x9eb2a5ed, SUC_lexicon_tags_14 },
    { 0xa832a5ed, SUC_lexicon_tags_38 },
    { 0x49f2a5f0, SUC_lexicon_tags_25 },
    { 0xa872a5f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b72a5fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd2a5fe, SUC_lexicon_tags_1 },
    { 0x7b2a5ff, SUC_lexicon_tags_10 },
    { 0xab92a5fe, SUC_lexicon_tags_60 },
    { 0x4872a5fe, SUC_lexicon_tags_6 },
    { 0xd932a602, SUC_lexicon_tags_77 },
    { 0x9752a603, SUC_lexicon_tags_2 },
    { 0x5a32a603, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x13f2a606, SUC_lexicon_tags_17 },
    { 0xe432a607, SUC_lexicon_tags_7 },
    { 0x6a12a607, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7652a60e, SUC_lexicon_tags_29 },
    { 0xaf92a60e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9872a612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f2a616, SUC_lexicon_tags_84 },
    { 0xeff2a617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd172a619, SUC_lexicon_tags_7 },
    { 0xbe72a619, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd52a61d, SUC_lexicon_tags_47 },
    { 0x4772a61e, SUC_lexicon_tags_11 },
    { 0x74f2a61e, SUC_lexicon_tags_47 },
    { 0xe552a61e, SUC_lexicon_tags_1 },
    { 0xe192a620, SUC_lexicon_tags_11 },
    { 0x8712a622, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc052a628, SUC_lexicon_tags_7 },
    { 0x5bf2a628, SUC_lexicon_tags_30 },
    { 0xb12a629, SUC_lexicon_tags_17 },
    { 0x8f12a62b, SUC_lexicon_tags_1 },
    { 0xbdd2a62c, SUC_lexicon_tags_23 },
    { 0x5092a62b, SUC_lexicon_tags_61 },
    { 0xe292a62e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead2a632, SUC_lexicon_tags_47 },
    { 0xf4f2a633, SUC_lexicon_tags_29 },
    { 0xbf52a634, SUC_lexicon_tags_9 },
    { 0x2312a632, SUC_lexicon_tags_0 },
    { 0x8b32a636, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb12a63b, SUC_lexicon_tags_27 },
    { 0xb152a63c, SUC_lexicon_tags_6 },
    { 0x5ff2a63c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x16f2a63f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f2a642, SUC_lexicon_tags_38 },
    { 0xa892a642, SUC_lexicon_tags_30 },
    { 0xb992a644, SUC_lexicon_tags_60 },
    { 0x59f2a645, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf52a647, SUC_lexicon_tags_47 },
    { 0x3792a648, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6712a64a, SUC_lexicon_tags_56 },
    { 0x7352a64b, SUC_lexicon_tags_138 },
    { 0x3d92a64c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb32a64e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d12a650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6912a655, SUC_lexicon_tags_37 },
    { 0xec52a655, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1992a658, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38f2a65a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7992a65e, SUC_lexicon_tags_31 },
    { 0x8772a65f, SUC_lexicon_tags_68 },
    { 0xb612a65e, SUC_lexicon_tags_15 },
    { 0x1cb2a660, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e12a663, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbcd2a665, SUC_lexicon_tags_8 },
    { 0x8672a666, SUC_lexicon_tags_11 },
    { 0xe232a667, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af2a66a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8ef2a66c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3872a673, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b2a67b, SUC_lexicon_tags_7 },
    { 0x1c32a67c, SUC_lexicon_tags_6 },
    { 0x98b2a67c, SUC_lexicon_tags_25 },
    { 0xc652a67e, SUC_lexicon_tags_1 },
    { 0x8c52a67d, SUC_lexicon_tags_27 },
    { 0xd772a67c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32a68a, SUC_lexicon_tags_1 },
    { 0x7232a68b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa632a68f, SUC_lexicon_tags_12 },
    { 0xe6d2a690, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad2a699, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd32a69c, SUC_lexicon_tags_60 },
    { 0xf8d2a69d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8bb2a69f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6492a6a1, SUC_lexicon_tags_63 },
    { 0xb692a6a2, SUC_lexicon_tags_64 },
    { 0xb332a6a3, SUC_lexicon_tags_47 },
    { 0xca52a6a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf12a6a6, SUC_lexicon_tags_27 },
    { 0x8ed2a6a7, SUC_lexicon_tags_37 },
    { 0xdc12a6a8, SUC_lexicon_tags_15 },
    { 0x8f32a6a8, SUC_lexicon_tags_167 },
    { 0x5f2a6a8, SUC_lexicon_tags_12 },
    { 0x57f2a6ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x66b2a6ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb412a6b2, SUC_lexicon_tags_29 },
    { 0xfaf2a6b2, SUC_lexicon_tags_17 },
    { 0xc492a6b4, SUC_lexicon_tags_48 },
    { 0x16d2a6b4, SUC_lexicon_tags_30 },
    { 0x4512a6b6, SUC_lexicon_tags_27 },
    { 0xb1d2a6b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad2a6ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbef2a6bc, SUC_lexicon_tags_13 },
    { 0x3632a6bd, SUC_lexicon_tags_33 },
    { 0xf932a6be, SUC_lexicon_tags_11 },
    { 0xd2a6bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d2a6c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c32a6c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd292a6cc, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xad12a6ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b2a6d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d52a6d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fd2a6d7, SUC_lexicon_tags_38 },
    { 0x4ed2a6d7, SUC_lexicon_tags_16 },
    { 0xbfd2a6d7, SUC_lexicon_tags_15 },
    { 0x70b2a6da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df2a6e1, SUC_lexicon_tags_138 },
    { 0xf252a6e1, SUC_lexicon_tags_2 },
    { 0xaed2a6e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8672a6ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7852a6f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f2a6f6, SUC_lexicon_tags_0 },
    { 0x14f2a6f7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf32a6f9, SUC_lexicon_tags_30 },
    { 0x4952a6fa, SUC_lexicon_tags_63 },
    { 0x2e32a6f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18f2a6fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3852a702, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeb12a704, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d2a707, SUC_lexicon_tags_0 },
    { 0x5392a707, SUC_lexicon_tags_11 },
    { 0xf472a709, SUC_lexicon_tags_63 },
    { 0x27f2a70a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f2a711, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b2a714, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3612a716, SUC_lexicon_tags_2 },
    { 0x792a716, SUC_lexicon_tags_30 },
    { 0x8d72a718, SUC_lexicon_tags_31 },
    { 0x8fb2a719, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd672a720, SUC_lexicon_tags_37 },
    { 0xc472a720, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdd12a723, SUC_lexicon_tags_39 },
    { 0x7252a724, SUC_lexicon_tags_56 },
    { 0x4fd2a725, SUC_lexicon_tags_49 },
    { 0xe1b2a723, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x992a728, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xf1b2a72a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d2a731, SUC_lexicon_tags_13 },
    { 0x2132a731, SUC_lexicon_tags_6 },
    { 0xab2a733, SUC_lexicon_tags_15 },
    { 0x2df2a733, SUC_lexicon_tags_17 },
    { 0x3cf2a735, SUC_lexicon_tags_16 },
    { 0x7892a734, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf112a73b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f2a73f, SUC_lexicon_tags_38 },
    { 0x7bf2a740, SUC_lexicon_tags_138 },
    { 0x10b2a741, SUC_lexicon_tags_10 },
    { 0x312a742, SUC_lexicon_tags_68 },
    { 0xf2f2a742, SUC_lexicon_tags_23 },
    { 0xb12a744, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab2a74a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf32a74c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd2a750, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6892a754, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f52a757, SUC_lexicon_tags_7 },
    { 0x2c32a758, SUC_lexicon_tags_25 },
    { 0xbe52a759, SUC_lexicon_tags_9 },
    { 0x6fd2a75a, SUC_lexicon_tags_45 },
    { 0x7c32a757, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d52a765, SUC_lexicon_tags_36 },
    { 0x63b2a766, SUC_lexicon_tags_1 },
    { 0x4c32a767, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4d2a769, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7272a76f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb2a774, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc972a777, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9672a77b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4352a77d, SUC_lexicon_tags_6 },
    { 0xea52a77d, SUC_lexicon_tags_39 },
    { 0x82d2a77f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa772a782, SUC_lexicon_tags_17 },
    { 0xfa52a783, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe372a785, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2112a78c, SUC_lexicon_tags_37 },
    { 0x6852a78d, SUC_lexicon_tags_39 },
    { 0xb232a78e, SUC_lexicon_tags_63 },
    { 0x5172a78f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1392a791, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5092a79a, SUC_lexicon_tags_39 },
    { 0x9592a79a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf2a7a8, SUC_lexicon_tags_15 },
    { 0x22d2a7a9, SUC_lexicon_tags_2 },
    { 0x9d92a7aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9d12a7ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e52a7af, SUC_lexicon_tags_87 },
    { 0x7152a7af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d2a7b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b2a7b8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9632a7c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab2a7c5, SUC_lexicon_tags_1 },
    { 0xa2b2a7c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b2a7c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d72a7cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf752a7ce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb3b2a7d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7432a7da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4232a7de, SUC_lexicon_tags_28 },
    { 0x4932a7de, SUC_lexicon_tags_39 },
    { 0xe2f2a7e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x17b2a7e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd2a7eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe92a7ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2ad2a7f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d2a7f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b72a7f8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd2a7fc, SUC_lexicon_tags_6 },
    { 0x5b92a7fd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b2a801, SUC_lexicon_tags_7 },
    { 0xde72a802, SUC_lexicon_tags_1 },
    { 0xa6b2a803, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f2a807, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf072a80b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd2a80f, SUC_lexicon_tags_5 },
    { 0x7392a810, SUC_lexicon_tags_55 },
    { 0x7f52a80f, SUC_lexicon_tags_14 },
    { 0xea52a811, SUC_lexicon_tags_7 },
    { 0x98d2a80f, SUC_lexicon_tags_23 },
    { 0xecf2a813, SUC_lexicon_tags_25 },
    { 0x8db2a815, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8532a817, SUC_lexicon_tags_63 },
    { 0x16f2a818, SUC_lexicon_tags_37 },
    { 0x4672a819, SUC_lexicon_tags_11 },
    { 0x7652a819, SUC_lexicon_tags_11 },
    { 0xc132a81b, SUC_lexicon_tags_11 },
    { 0xc792a81c, SUC_lexicon_tags_7 },
    { 0xed2a81a, SUC_lexicon_tags_63 },
    { 0x6bb2a817, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4572a821, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1a52a823, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec32a825, SUC_lexicon_tags_1 },
    { 0xec12a826, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe5d2a828, SUC_lexicon_tags_37 },
    { 0xefb2a829, SUC_lexicon_tags_17 },
    { 0x70b2a82a, SUC_lexicon_tags_1 },
    { 0xf512a828, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792a831, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc232a833, SUC_lexicon_tags_17 },
    { 0xfcd2a834, SUC_lexicon_tags_23 },
    { 0x66f2a835, SUC_lexicon_tags_15 },
    { 0x25b2a836, SUC_lexicon_tags_47 },
    { 0x1c32a835, SUC_lexicon_tags_33 },
    { 0xc812a838, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3832a83b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d12a83f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e12a846, SUC_lexicon_tags_8 },
    { 0x46d2a847, SUC_lexicon_tags_2 },
    { 0xa0b2a847, SUC_lexicon_tags_23 },
    { 0x6212a846, SUC_lexicon_tags_11 },
    { 0x7032a84a, SUC_lexicon_tags_14 },
    { 0x95f2a84b, SUC_lexicon_tags_12 },
    { 0xdc92a84c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3492a84e, SUC_lexicon_tags_0 },
    { 0x20d2a84e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c12a853, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a52a855, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd272a857, SUC_lexicon_tags_17 },
    { 0x55b2a858, SUC_lexicon_tags_17 },
    { 0xcef2a858, SUC_lexicon_tags_1 },
    { 0x40d2a858, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3652a85e, SUC_lexicon_tags_25 },
    { 0x4fb2a85f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9052a861, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9472a865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a72a86d, SUC_lexicon_tags_2 },
    { 0x81f2a86e, SUC_lexicon_tags_15 },
    { 0xfa32a86f, SUC_lexicon_tags_17 },
    { 0x7612a870, SUC_lexicon_tags_38 },
    { 0x6432a871, SUC_lexicon_tags_37 },
    { 0xa112a872, SUC_lexicon_tags_33 },
    { 0xa8b2a872, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6632a875, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x672a879, SUC_lexicon_tags_14 },
    { 0xfc32a87a, SUC_lexicon_tags_74 },
    { 0x7372a879, SUC_lexicon_tags_11 },
    { 0x2d2a879, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b2a87f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb972a881, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddb2a883, SUC_lexicon_tags_17 },
    { 0xf772a884, SUC_lexicon_tags_37 },
    { 0x5312a883, SUC_lexicon_tags_12 },
    { 0xccb2a886, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60b2a888, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7af2a88a, SUC_lexicon_tags_42 },
    { 0x2ef2a88b, SUC_lexicon_tags_63 },
    { 0xef72a88c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc232a88e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b2a892, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a92a895, SUC_lexicon_tags_19 },
    { 0xe3b2a896, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef2a899, SUC_lexicon_tags_198 },
    { 0xe9b2a899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe72a89d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa12a8a0, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0x81b2a8a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b2a8a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad2a8a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4812a8ad, SUC_lexicon_tags_35 },
    { 0xea72a8ae, SUC_lexicon_tags_1 },
    { 0xf152a8ae, SUC_lexicon_tags_43 },
    { 0x8e52a8b0, SUC_lexicon_tags_37 },
    { 0x3132a8b1, SUC_lexicon_tags_66 },
    { 0x692a8b1, SUC_lexicon_tags_42 },
    { 0x6f52a8ad, SUC_lexicon_tags_38 },
    { 0xc032a8ad, SUC_lexicon_tags_27 },
    { 0x2852a8b5, SUC_lexicon_tags_63 },
    { 0x30b2a8b6, SUC_lexicon_tags_0 },
    { 0x3dd2a8b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d2a8bb, SUC_lexicon_tags_12 },
    { 0xadb2a8bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd392a8bf, SUC_lexicon_tags_2 },
    { 0x4e72a8c0, SUC_lexicon_tags_9 },
    { 0xabf2a8c0, SUC_lexicon_tags_9 },
    { 0x6712a8c0, SUC_lexicon_tags_4 },
    { 0x62b2a8c3, SUC_lexicon_tags_5 },
    { 0x60b2a8c3, SUC_lexicon_tags_7 },
    { 0x9f52a8c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c12a8cd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab2a8d7, SUC_lexicon_tags_14 },
    { 0x4652a8d7, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2a8dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf72a8de, SUC_lexicon_tags_47 },
    { 0xf512a8df, SUC_lexicon_tags_35 },
    { 0xb3b2a8e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2d52a8e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d2a8e5, SUC_lexicon_tags_16 },
    { 0x8ab2a8e6, SUC_lexicon_tags_11 },
    { 0xfbf2a8e7, SUC_lexicon_tags_23 },
    { 0x82f2a8e6, SUC_lexicon_tags_6 },
    { 0x95f2a8e9, SUC_lexicon_tags_25 },
    { 0x8dd2a8ea, SUC_lexicon_tags_37 },
    { 0x8ff2a8e7, SUC_lexicon_tags_11 },
    { 0x452a8ec, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b2a8ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc612a8f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d2a8f5, SUC_lexicon_tags_17 },
    { 0x3172a8f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7992a8fa, SUC_lexicon_tags_2 },
    { 0x2cf2a8fa, SUC_lexicon_tags_9 },
    { 0x6d12a8fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67f2a8fe, SUC_lexicon_tags_11 },
    { 0xaa72a8fe, SUC_lexicon_tags_27 },
    { 0x7072a900, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49b2a902, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa012a90e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b2a914, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc92a916, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b2a91a, SUC_lexicon_tags_11 },
    { 0xdf92a91b, SUC_lexicon_tags_6 },
    { 0x58f2a91b, SUC_lexicon_tags_68 },
    { 0xa4d2a91d, SUC_lexicon_tags_33 },
    { 0x7532a91e, SUC_lexicon_tags_68 },
    { 0xc12a91e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc312a926, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832a92a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7712a92e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4cb2a930, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf2a933, SUC_lexicon_tags_2 },
    { 0xd712a934, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a12a937, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4732a939, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc752a93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6992a93e, SUC_lexicon_tags_2 },
    { 0xc592a93e, SUC_lexicon_tags_14 },
    { 0x51b2a940, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9d52a942, SUC_lexicon_tags_46 },
    { 0x8f12a942, SUC_lexicon_tags_25 },
    { 0x4d2a942, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3032a947, SUC_lexicon_tags_7 },
    { 0xa1b2a948, SUC_lexicon_tags_1 },
    { 0xf092a949, SUC_lexicon_tags_23 },
    { 0xe3f2a94a, SUC_lexicon_tags_15 },
    { 0xe052a94a, SUC_lexicon_tags_0 },
    { 0xf7f2a94c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfe72a94e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed2a953, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8912a956, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8452a959, SUC_lexicon_tags_50 },
    { 0x4b52a95a, SUC_lexicon_tags_33 },
    { 0xaa72a95b, SUC_lexicon_tags_37 },
    { 0x1352a95b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b2a962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd2a969, SUC_lexicon_tags_27 },
    { 0xb72a96a, SUC_lexicon_tags_1 },
    { 0xda32a969, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xda32a96d, SUC_lexicon_tags_17 },
    { 0x5d12a96e, SUC_lexicon_tags_2 },
    { 0xe4f2a96f, SUC_lexicon_tags_1 },
    { 0xaf2a970, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xed12a973, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b2a976, SUC_lexicon_tags_33 },
    { 0xb1f2a977, SUC_lexicon_tags_37 },
    { 0xe112a978, SUC_lexicon_tags_14 },
    { 0x49d2a978, SUC_lexicon_tags_0 },
    { 0x4772a97a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ed2a97c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8172a980, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc292a984, SUC_lexicon_tags_7 },
    { 0x45f2a985, SUC_lexicon_tags_2 },
    { 0x6292a986, SUC_lexicon_tags_12 },
    { 0x5852a986, SUC_lexicon_tags_2 },
    { 0x8612a988, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d2a98b, SUC_lexicon_tags_23 },
    { 0x4c32a98c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d72a98e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc52a990, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1752a995, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81d2a997, SUC_lexicon_tags_12 },
    { 0x12d2a998, SUC_lexicon_tags_19 },
    { 0x1452a998, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f72a99b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10f2a99d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43d2a99f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa472a9a5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd2a9a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x72f2a9aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df2a9ad, SUC_lexicon_tags_15 },
    { 0xdff2a9ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4732a9b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe52a9b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff52a9b8, SUC_lexicon_tags_0 },
    { 0x4e92a9b9, SUC_lexicon_tags_25 },
    { 0x2ff2a9ba, SUC_lexicon_tags_30 },
    { 0xd2d2a9b9, SUC_lexicon_tags_77 },
    { 0x9af2a9b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f32a9be, SUC_lexicon_tags_7 },
    { 0x5312a9bf, SUC_lexicon_tags_37 },
    { 0x28d2a9c0, SUC_lexicon_tags_9 },
    { 0x7772a9c1, SUC_lexicon_tags_25 },
    { 0x94d2a9c0, SUC_lexicon_tags_1 },
    { 0x5572a9c3, SUC_lexicon_tags_17 },
    { 0xf72a9be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c32a9cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3612a9d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf972a9d4, SUC_lexicon_tags_30 },
    { 0x6232a9d5, SUC_lexicon_tags_12 },
    { 0x992a9d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2512a9da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6872a9de, SUC_lexicon_tags_12 },
    { 0x5332a9df, SUC_lexicon_tags_15 },
    { 0xd72a9e0, SUC_lexicon_tags_43 },
    { 0x5832a9e1, SUC_lexicon_tags_17 },
    { 0x1ad2a9de, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb5f2a9e4, SUC_lexicon_tags_12 },
    { 0xecf2a9e4, SUC_lexicon_tags_27 },
    { 0xa8d2a9e5, SUC_lexicon_tags_2 },
    { 0xbed2a9e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e32a9ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2fb2a9ec, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b32a9f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb2a9f3, SUC_lexicon_tags_49 },
    { 0x8d72a9f3, SUC_lexicon_tags_63 },
    { 0x77b2a9f5, SUC_lexicon_tags_13 },
    { 0x1452a9f5, SUC_lexicon_tags_1 },
    { 0xeb72a9f7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf592a9fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb292aa03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf992aa05, SUC_lexicon_tags_29 },
    { 0x6bd2aa06, SUC_lexicon_tags_47 },
    { 0x1112aa07, SUC_lexicon_tags_7 },
    { 0xd332aa06, SUC_lexicon_tags_16 },
    { 0x36f2aa09, SUC_lexicon_tags_29 },
    { 0x6152aa05, SUC_lexicon_tags_25 },
    { 0xe072aa0b, SUC_lexicon_tags_63 },
    { 0x34d2aa0c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f2aa12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b2aa20, SUC_lexicon_tags_58 },
    { 0xd372aa21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff32aa23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa812aa2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa452aa30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xed12aa32, SUC_lexicon_tags_29 },
    { 0xe172aa33, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef32aa37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4572aa39, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8532aa3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a52aa3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f92aa3f, SUC_lexicon_tags_15 },
    { 0x3892aa40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7f2aa42, SUC_lexicon_tags_7 },
    { 0x9ef2aa43, SUC_lexicon_tags_4 },
    { 0x90d2aa43, SUC_lexicon_tags_26 },
    { 0x64b2aa45, SUC_lexicon_tags_31 },
    { 0x89f2aa42, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d2aa4b, SUC_lexicon_tags_15 },
    { 0x7e72aa4b, SUC_lexicon_tags_87 },
    { 0x7df2aa4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5452aa50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd2aa57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6af2aa59, SUC_lexicon_tags_63 },
    { 0x1d72aa59, SUC_lexicon_tags_12 },
    { 0x4d32aa5b, SUC_lexicon_tags_63 },
    { 0x9e52aa5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8d2aa5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6192aa60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32aa62, SUC_lexicon_tags_1 },
    { 0x8e52aa63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe572aa65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb32aa67, SUC_lexicon_tags_14 },
    { 0xe312aa68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb652aa6a, SUC_lexicon_tags_15 },
    { 0x1812aa6b, SUC_lexicon_tags_10 },
    { 0x6fd2aa6b, SUC_lexicon_tags_92 },
    { 0x3652aa6a, SUC_lexicon_tags_28 },
    { 0xc492aa6e, SUC_lexicon_tags_11 },
    { 0x72d2aa6f, SUC_lexicon_tags_2 },
    { 0x2432aa70, SUC_lexicon_tags_2 },
    { 0xb892aa71, SUC_lexicon_tags_7 },
    { 0x992aa72, SUC_lexicon_tags_25 },
    { 0x39f2aa70, SUC_lexicon_tags_8 },
    { 0x95f2aa6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf672aa76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef52aa78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd532aa7a, SUC_lexicon_tags_25 },
    { 0x872aa7a, SUC_lexicon_tags_66 },
    { 0x9e12aa7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff52aa7e, SUC_lexicon_tags_7 },
    { 0x9e12aa7e, SUC_lexicon_tags_65 },
    { 0xf52aa80, SUC_lexicon_tags_19 },
    { 0x3b72aa81, SUC_lexicon_tags_12 },
    { 0x3912aa81, SUC_lexicon_tags_30 },
    { 0x5812aa83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db2aa86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad72aa8a, SUC_lexicon_tags_25 },
    { 0xa92aa8b, SUC_lexicon_tags_1 },
    { 0xbb72aa8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf112aa8f, SUC_lexicon_tags_12 },
    { 0xd4d2aa90, SUC_lexicon_tags_10 },
    { 0x3072aa91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6212aa97, SUC_lexicon_tags_11 },
    { 0x83d2aa98, SUC_lexicon_tags_25 },
    { 0xaef2aa99, SUC_lexicon_tags_7 },
    { 0x3ab2aa99, SUC_lexicon_tags_17 },
    { 0xa652aa99, SUC_lexicon_tags_30 },
    { 0xd92aa99, SUC_lexicon_tags_7 },
    { 0x9352aa9d, SUC_lexicon_tags_17 },
    { 0x7152aa9e, SUC_lexicon_tags_25 },
    { 0x4cd2aa9f, SUC_lexicon_tags_37 },
    { 0xf12aaa0, SUC_lexicon_tags_48 },
    { 0x29f2aaa1, SUC_lexicon_tags_4 },
    { 0x6f12aaa1, SUC_lexicon_tags_45 },
    { 0x3c32aaa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5612aaa5, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e32aaa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6352aaaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb212aaaf, SUC_lexicon_tags_1 },
    { 0xf3f2aab0, SUC_lexicon_tags_55 },
    { 0x9b92aab1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc392aab4, SUC_lexicon_tags_7 },
    { 0x9ad2aab4, SUC_lexicon_tags_11 },
    { 0x4a12aab6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x57b2aab8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ed2aaba, SUC_lexicon_tags_65 },
    { 0x18d2aabb, SUC_lexicon_tags_11 },
    { 0xfe32aabb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8cd2aabe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb2aac1, SUC_lexicon_tags_113 },
    { 0xa552aac2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b2aac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff2aaca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d2aace, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2432aad1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf12aad4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef32aad7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe92aade, SUC_lexicon_tags_47 },
    { 0x69f2aadf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd2aae4, SUC_lexicon_tags_39 },
    { 0x2232aae5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb2aae8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9132aaf8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaa72aafa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8412aafd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd2ab08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf2ab0b, SUC_lexicon_tags_37 },
    { 0xa952ab0b, SUC_lexicon_tags_65 },
    { 0xd5d2ab0d, SUC_lexicon_tags_6 },
    { 0x6512ab0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4572ab11, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d2ab15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb752ab17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd752ab1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf772ab1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe832ab25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f52ab28, SUC_lexicon_tags_7 },
    { 0xd672ab29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8272ab2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6772ab32, SUC_lexicon_tags_0 },
    { 0x61b2ab33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2712ab36, SUC_lexicon_tags_15 },
    { 0x8ad2ab36, SUC_lexicon_tags_1 },
    { 0x23b2ab38, SUC_lexicon_tags_11 },
    { 0xff2ab39, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5752ab3b, SUC_lexicon_tags_17 },
    { 0x6752ab3b, SUC_lexicon_tags_21 },
    { 0xbeb2ab3d, SUC_lexicon_tags_2 },
    { 0x3ad2ab3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c52ab41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9812ab4b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f2ab52, SUC_lexicon_tags_37 },
    { 0x51b2ab52, SUC_lexicon_tags_1262 },
    { 0, NULL },
    { 0, NULL },
    { 0x2012ab56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x45f2ab58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed2ab5b, SUC_lexicon_tags_29 },
    { 0xabb2ab5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b2ab5e, SUC_lexicon_tags_3 },
    { 0x6232ab5e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe272ab64, SUC_lexicon_tags_14 },
    { 0x7cb2ab64, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb752ab67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff12ab6e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4192ab71, SUC_lexicon_tags_19 },
    { 0xd912ab72, SUC_lexicon_tags_63 },
    { 0x5b92ab73, SUC_lexicon_tags_28 },
    { 0xc732ab72, SUC_lexicon_tags_49 },
    { 0x1b52ab71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f2ab7a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4272ab83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd92ab85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44d2ab87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b2ab91, SUC_lexicon_tags_9 },
    { 0xa1f2ab92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f72ab9c, SUC_lexicon_tags_12 },
    { 0x99f2ab9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d2aba0, SUC_lexicon_tags_1 },
    { 0x1ad2aba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x812aba4, SUC_lexicon_tags_1 },
    { 0x6ed2aba5, SUC_lexicon_tags_19 },
    { 0xaa12aba5, SUC_lexicon_tags_12 },
    { 0x35f2aba7, SUC_lexicon_tags_0 },
    { 0xe312aba7, SUC_lexicon_tags_33 },
    { 0xd6d2aba9, SUC_lexicon_tags_28 },
    { 0xb832aba9, SUC_lexicon_tags_17 },
    { 0x3b92abab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f72abaf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d92abb4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf72abb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb692abbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde72abbf, SUC_lexicon_tags_47 },
    { 0xbf32abc0, SUC_lexicon_tags_14 },
    { 0x5852abc1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b2abc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc52abc9, SUC_lexicon_tags_19 },
    { 0x4032abc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f2abcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb992abcf, SUC_lexicon_tags_60 },
    { 0xa232abd0, SUC_lexicon_tags_48 },
    { 0xed12abd1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb12abd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb92abd6, SUC_lexicon_tags_12 },
    { 0xa272abd7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7952abd9, SUC_lexicon_tags_37 },
    { 0x9912abda, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b2abdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd52abe6, SUC_lexicon_tags_32 },
    { 0x6932abe6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x80d2abe9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e92abf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c12abf6, SUC_lexicon_tags_47 },
    { 0x7bb2abf7, SUC_lexicon_tags_37 },
    { 0x3cf2abf8, SUC_lexicon_tags_51 },
    { 0x2192abf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe272abfc, SUC_lexicon_tags_11 },
    { 0x9f92abfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb712ac03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7572ac05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b2ac09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd32ac0d, SUC_lexicon_tags_33 },
    { 0x56f2ac0e, SUC_lexicon_tags_47 },
    { 0xddb2ac0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4d2ac11, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b2ac14, SUC_lexicon_tags_14 },
    { 0xbb52ac14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xef92ac18, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d72ac24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d2ac2c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa652ac2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b2ac32, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec12ac39, SUC_lexicon_tags_42 },
    { 0xfa72ac3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe592ac3c, SUC_lexicon_tags_2 },
    { 0x5df2ac3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8812ac40, SUC_lexicon_tags_30 },
    { 0x62f2ac41, SUC_lexicon_tags_45 },
    { 0xed2ac42, SUC_lexicon_tags_30 },
    { 0xc6d2ac40, SUC_lexicon_tags_2 },
    { 0x5812ac40, SUC_lexicon_tags_37 },
    { 0x212ac41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe132ac4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0b2ac50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9752ac53, SUC_lexicon_tags_0 },
    { 0x86b2ac54, SUC_lexicon_tags_30 },
    { 0xaf52ac55, SUC_lexicon_tags_47 },
    { 0xc772ac55, SUC_lexicon_tags_8 },
    { 0x3392ac55, SUC_lexicon_tags_17 },
    { 0xc3b2ac58, SUC_lexicon_tags_2 },
    { 0x6dd2ac59, SUC_lexicon_tags_12 },
    { 0x8df2ac58, SUC_lexicon_tags_12 },
    { 0xa8b2ac5b, SUC_lexicon_tags_37 },
    { 0x11d2ac5c, SUC_lexicon_tags_9 },
    { 0x4832ac5b, SUC_lexicon_tags_17 },
    { 0xccb2ac5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2672ac61, SUC_lexicon_tags_7 },
    { 0x82b2ac61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x17d2ac64, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9d12ac66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2232ac6a, SUC_lexicon_tags_57 },
    { 0x2a52ac6b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5772ac6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf972ac76, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe432ac78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5252ac7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38d2ac7e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d12ac82, SUC_lexicon_tags_2 },
    { 0xd232ac82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2cb2ac85, SUC_lexicon_tags_47 },
    { 0x7852ac86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4332ac89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d2ac8d, SUC_lexicon_tags_17 },
    { 0xdf72ac8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f2ac91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d2ac96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6512ac98, SUC_lexicon_tags_2 },
    { 0x5e72ac98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcad2ac9b, SUC_lexicon_tags_2 },
    { 0xa592ac9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2aca1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde32acac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb2acaf, SUC_lexicon_tags_37 },
    { 0xb652acb0, SUC_lexicon_tags_63 },
    { 0xe0f2acb1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf2acba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d32acbf, SUC_lexicon_tags_15 },
    { 0x9a12acc0, SUC_lexicon_tags_29 },
    { 0xea32acbf, SUC_lexicon_tags_25 },
    { 0x9452acc2, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0xf592acc4, SUC_lexicon_tags_17 },
    { 0x1532acc4, SUC_lexicon_tags_25 },
    { 0x852acc4, SUC_lexicon_tags_220 },
    { 0xd172acc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf492acc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef2accc, SUC_lexicon_tags_1 },
    { 0x3812accc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f2acd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1d2acd3, SUC_lexicon_tags_3 },
    { 0xbdd2acd4, SUC_lexicon_tags_14 },
    { 0xf272acd4, SUC_lexicon_tags_27 },
    { 0x8892acd5, SUC_lexicon_tags_8 },
    { 0xe032acd7, SUC_lexicon_tags_47 },
    { 0xec32acd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8992acda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2512acdd, SUC_lexicon_tags_31 },
    { 0x3e52acde, SUC_lexicon_tags_35 },
    { 0x19b2acde, SUC_lexicon_tags_77 },
    { 0x20d2ace0, SUC_lexicon_tags_25 },
    { 0x74b2ace1, SUC_lexicon_tags_37 },
    { 0xf6b2ace2, SUC_lexicon_tags_10 },
    { 0xf812ace3, SUC_lexicon_tags_27 },
    { 0xb5f2acdf, SUC_lexicon_tags_0 },
    { 0xd592ace5, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f2ace9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d2acef, SUC_lexicon_tags_27 },
    { 0x3a32acf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c72acf2, SUC_lexicon_tags_10 },
    { 0x8fb2acf3, SUC_lexicon_tags_25 },
    { 0xf292acf4, SUC_lexicon_tags_49 },
    { 0xecf2acf5, SUC_lexicon_tags_485 },
    { 0x1f72acf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6852acfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab2ad08, SUC_lexicon_tags_17 },
    { 0x1812ad09, SUC_lexicon_tags_11 },
    { 0x7e32ad09, SUC_lexicon_tags_25 },
    { 0x152ad0b, SUC_lexicon_tags_37 },
    { 0x68b2ad09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa072ad0e, SUC_lexicon_tags_47 },
    { 0xe832ad0f, SUC_lexicon_tags_23 },
    { 0x1732ad0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf32ad13, SUC_lexicon_tags_1 },
    { 0x23f2ad14, SUC_lexicon_tags_17 },
    { 0xf32ad14, SUC_lexicon_tags_14 },
    { 0x5052ad16, SUC_lexicon_tags_37 },
    { 0x6932ad17, SUC_lexicon_tags_31 },
    { 0x9b2ad15, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf2ad1d, SUC_lexicon_tags_33 },
    { 0x78b2ad1d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf792ad20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5432ad27, SUC_lexicon_tags_14 },
    { 0x9612ad28, SUC_lexicon_tags_7 },
    { 0xd932ad29, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d2ad2f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa752ad34, SUC_lexicon_tags_57 },
    { 0xfaf2ad35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab2ad3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d2ad43, SUC_lexicon_tags_47 },
    { 0xf7d2ad44, SUC_lexicon_tags_12 },
    { 0x11b2ad43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d2ad4f, SUC_lexicon_tags_13 },
    { 0x47b2ad50, SUC_lexicon_tags_37 },
    { 0xe192ad51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b12ad53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdf72ad55, SUC_lexicon_tags_8 },
    { 0xa3d2ad56, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1712ad58, SUC_lexicon_tags_14 },
    { 0xfed2ad59, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x9652ad5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2ad63, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xca72ad66, SUC_lexicon_tags_60 },
    { 0x2772ad66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9192ad74, SUC_lexicon_tags_17 },
    { 0x1d92ad75, SUC_lexicon_tags_23 },
    { 0x7552ad74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a92ad78, SUC_lexicon_tags_25 },
    { 0x1eb2ad79, SUC_lexicon_tags_38 },
    { 0x15b2ad78, SUC_lexicon_tags_11 },
    { 0x6912ad78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f2ad83, SUC_lexicon_tags_14 },
    { 0xfd12ad84, SUC_lexicon_tags_66 },
    { 0x42d2ad85, SUC_lexicon_tags_23 },
    { 0x76b2ad83, SUC_lexicon_tags_47 },
    { 0x3b32ad83, SUC_lexicon_tags_47 },
    { 0xfcf2ad84, SUC_lexicon_tags_2 },
    { 0x7552ad86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x912ad8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d2ad92, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf2ad99, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8712ad9c, SUC_lexicon_tags_2 },
    { 0x9e92ad9d, SUC_lexicon_tags_27 },
    { 0x2272ad9d, SUC_lexicon_tags_25 },
    { 0xe9f2ad9f, SUC_lexicon_tags_27 },
    { 0xefb2ada0, SUC_lexicon_tags_2 },
    { 0xe712ada0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f2ada9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7512adae, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb932adb1, SUC_lexicon_tags_25 },
    { 0x8872adb2, SUC_lexicon_tags_12 },
    { 0x9db2adb3, SUC_lexicon_tags_37 },
    { 0xc52adb3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5352adb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7252adc2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7272adc5, SUC_lexicon_tags_14 },
    { 0x4bb2adc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af2adc9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3432adcc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xee12adce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f32add1, SUC_lexicon_tags_1 },
    { 0x2452add2, SUC_lexicon_tags_25 },
    { 0x94f2add3, SUC_lexicon_tags_29 },
    { 0x5352add3, SUC_lexicon_tags_12 },
    { 0x8b2add4, SUC_lexicon_tags_3 },
    { 0x58f2add6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdc52add8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe312adda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4bb2addc, SUC_lexicon_tags_43 },
    { 0x652addd, SUC_lexicon_tags_26 },
    { 0xee52addd, SUC_lexicon_tags_16 },
    { 0x4792addf, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xe852ade1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4292ade3, SUC_lexicon_tags_55 },
    { 0xbfd2ade4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f2ade7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3cb2ade9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a12adf2, SUC_lexicon_tags_37 },
    { 0xb8d2adf3, SUC_lexicon_tags_60 },
    { 0x9df2adf4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb032adf8, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xf5b2adfa, SUC_lexicon_tags_0 },
    { 0x53f2adfb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5132adfe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf432ae01, SUC_lexicon_tags_13 },
    { 0xe252ae02, SUC_lexicon_tags_1 },
    { 0x9992ae03, SUC_lexicon_tags_12 },
    { 0x9e92ae03, SUC_lexicon_tags_87 },
    { 0xe492ae03, SUC_lexicon_tags_1 },
    { 0x3552ae01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8272ae0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd712ae0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbef2ae0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8412ae10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4572ae13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd2ae18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d2ae1b, SUC_lexicon_tags_87 },
    { 0x65d2ae1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3672ae1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f2ae23, SUC_lexicon_tags_26 },
    { 0xe012ae24, SUC_lexicon_tags_15 },
    { 0x91b2ae24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5f2ae27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa472ae29, SUC_lexicon_tags_9 },
    { 0x45f2ae29, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd512ae2d, SUC_lexicon_tags_29 },
    { 0x52ae2e, SUC_lexicon_tags_8 },
    { 0x7d72ae2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba52ae33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22f2ae35, SUC_lexicon_tags_12 },
    { 0x4f92ae35, SUC_lexicon_tags_29 },
    { 0x4a32ae37, SUC_lexicon_tags_44 },
    { 0x3052ae38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6432ae3a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f92ae3f, SUC_lexicon_tags_34 },
    { 0x2552ae3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4252ae45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b2ae48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f2ae4d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x852ae50, SUC_lexicon_tags_19 },
    { 0xffb2ae51, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xea12ae53, SUC_lexicon_tags_11 },
    { 0x6bd2ae54, SUC_lexicon_tags_57 },
    { 0x56b2ae55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52ae59, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x832ae5b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xab72ae5e, SUC_lexicon_tags_47 },
    { 0xb052ae5f, SUC_lexicon_tags_25 },
    { 0x1a12ae5e, SUC_lexicon_tags_118 },
    { 0x1472ae5e, SUC_lexicon_tags_26 },
    { 0xd852ae62, SUC_lexicon_tags_12 },
    { 0xbd92ae62, SUC_lexicon_tags_14 },
    { 0xa9d2ae63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6712ae68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc32ae6b, SUC_lexicon_tags_36 },
    { 0x8bf2ae6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f2ae73, SUC_lexicon_tags_70 },
    { 0x2c32ae73, SUC_lexicon_tags_25 },
    { 0x8252ae74, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad2ae7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b2ae88, SUC_lexicon_tags_12 },
    { 0x8ab2ae88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8952ae8c, SUC_lexicon_tags_49 },
    { 0x84f2ae8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed92ae8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab2ae9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab52aea1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f2aeb0, SUC_lexicon_tags_15 },
    { 0xd092aeb0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3672aeb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad12aeb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd2aebb, SUC_lexicon_tags_7 },
    { 0x7572aebb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe32aec0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x88d2aec2, SUC_lexicon_tags_17 },
    { 0x56d2aec3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b2aec8, SUC_lexicon_tags_47 },
    { 0x6192aec9, SUC_lexicon_tags_12 },
    { 0x4472aec9, SUC_lexicon_tags_39 },
    { 0x5812aecb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb2aecf, SUC_lexicon_tags_15 },
    { 0x9fb2aed0, SUC_lexicon_tags_11 },
    { 0x3b2aed1, SUC_lexicon_tags_2 },
    { 0x7b92aed1, SUC_lexicon_tags_25 },
    { 0x96b2aed0, SUC_lexicon_tags_17 },
    { 0x79d2aed3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d12aed6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b2aed9, SUC_lexicon_tags_2 },
    { 0xf412aed9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd12aee0, SUC_lexicon_tags_33 },
    { 0x61f2aee1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf52aee8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9852aeea, SUC_lexicon_tags_1310 },
    { 0x5df2aeeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd592aeed, SUC_lexicon_tags_16 },
    { 0x3232aeed, SUC_lexicon_tags_2 },
    { 0xba72aeef, SUC_lexicon_tags_11 },
    { 0x212aeef, SUC_lexicon_tags_37 },
    { 0xb1b2aef1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7272aef6, SUC_lexicon_tags_7 },
    { 0xa7b2aef7, SUC_lexicon_tags_2 },
    { 0xfc92aef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ab2aefa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0f2aefc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe12af05, SUC_lexicon_tags_19 },
    { 0x1eb2af05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2812af0c, SUC_lexicon_tags_12 },
    { 0xfdd2af0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xff72af0f, SUC_lexicon_tags_65 },
    { 0x9d52af10, SUC_lexicon_tags_21 },
    { 0x8652af0f, SUC_lexicon_tags_198 },
    { 0x7bd2af11, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x132af15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8fd2af17, SUC_lexicon_tags_14 },
    { 0x3072af18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb2af1d, SUC_lexicon_tags_17 },
    { 0x9e12af1e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xcfb2af20, SUC_lexicon_tags_15 },
    { 0xf392af21, SUC_lexicon_tags_17 },
    { 0x7e12af21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2252af26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x492af28, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x9392af2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e92af2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf2af31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c92af39, SUC_lexicon_tags_10 },
    { 0x7f52af3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2af40, SUC_lexicon_tags_47 },
    { 0x68f2af41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d72af44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef32af46, SUC_lexicon_tags_6 },
    { 0xf172af47, SUC_lexicon_tags_25 },
    { 0x2eb2af48, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d12af50, SUC_lexicon_tags_60 },
    { 0x112af51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b32af54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3c12af56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa72af58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed2af5b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb2af5f, SUC_lexicon_tags_17 },
    { 0x77f2af60, SUC_lexicon_tags_17 },
    { 0x9ad2af5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d2af66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5752af69, SUC_lexicon_tags_29 },
    { 0x5412af69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1992af6d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9352af72, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x18b2af74, SUC_lexicon_tags_23 },
    { 0x2952af75, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7152af77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb2af7b, SUC_lexicon_tags_37 },
    { 0xbd2af7b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3532af7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad12af81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5132af84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94d2af86, SUC_lexicon_tags_55 },
    { 0x7a92af86, SUC_lexicon_tags_33 },
    { 0xcaf2af87, SUC_lexicon_tags_26 },
    { 0x3172af89, SUC_lexicon_tags_603 },
    { 0xb0b2af89, SUC_lexicon_tags_25 },
    { 0x3712af89, SUC_lexicon_tags_15 },
    { 0x8152af8a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x492af90, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3732af92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c92af98, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f2af9c, SUC_lexicon_tags_25 },
    { 0xf692af9c, SUC_lexicon_tags_38 },
    { 0x3152af9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb632afa0, SUC_lexicon_tags_37 },
    { 0x2cf2afa1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed92afa6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9392afa8, SUC_lexicon_tags_0 },
    { 0xe172afa8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8532afac, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1eb2afae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa832afb0, SUC_lexicon_tags_17 },
    { 0xadd2afb1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4092afb4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc472afb9, SUC_lexicon_tags_38 },
    { 0x8ab2afb9, SUC_lexicon_tags_11 },
    { 0x9972afbb, SUC_lexicon_tags_9 },
    { 0x8ad2afbc, SUC_lexicon_tags_25 },
    { 0x2b52afbc, SUC_lexicon_tags_37 },
    { 0xf412afbd, SUC_lexicon_tags_17 },
    { 0x4152afbf, SUC_lexicon_tags_0 },
    { 0x3952afbe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6852afc8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312afce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe972afd0, SUC_lexicon_tags_25 },
    { 0x19d2afd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6732afd5, SUC_lexicon_tags_60 },
    { 0xc852afd5, SUC_lexicon_tags_25 },
    { 0x2c72afd5, SUC_lexicon_tags_118 },
    { 0x7652afd8, SUC_lexicon_tags_12 },
    { 0x6072afd9, SUC_lexicon_tags_2 },
    { 0x6592afd9, SUC_lexicon_tags_10 },
    { 0xc092afda, SUC_lexicon_tags_2 },
    { 0x272afdc, SUC_lexicon_tags_70 },
    { 0x7032afd8, SUC_lexicon_tags_0 },
    { 0x1df2afd8, SUC_lexicon_tags_16 },
    { 0xc32afdd, SUC_lexicon_tags_56 },
    { 0x9052afd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b32afe2, SUC_lexicon_tags_33 },
    { 0xa612afe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4232afec, SUC_lexicon_tags_37 },
    { 0xa7b2afec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30d2afef, SUC_lexicon_tags_7 },
    { 0xe9f2afef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7b32aff2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22f2aff4, SUC_lexicon_tags_37 },
    { 0xcb52aff5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d2aff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b32afff, SUC_lexicon_tags_38 },
    { 0x4732b000, SUC_lexicon_tags_548 },
    { 0x4752afff, SUC_lexicon_tags_34 },
    { 0xd3d2b002, SUC_lexicon_tags_34 },
    { 0x4a52b001, SUC_lexicon_tags_8 },
    { 0xb412b004, SUC_lexicon_tags_64 },
    { 0x64b2b004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f2b008, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e32b013, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2412b015, SUC_lexicon_tags_33 },
    { 0x35b2b016, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32d2b018, SUC_lexicon_tags_67 },
    { 0x75d2b019, SUC_lexicon_tags_14 },
    { 0x6df2b019, SUC_lexicon_tags_35 },
    { 0x60d2b01a, SUC_lexicon_tags_11 },
    { 0x6b32b01a, SUC_lexicon_tags_12 },
    { 0x69f2b01d, SUC_lexicon_tags_10 },
    { 0xd632b01e, SUC_lexicon_tags_2 },
    { 0xb72b01b, SUC_lexicon_tags_27 },
    { 0xf9f2b020, SUC_lexicon_tags_95 },
    { 0xff32b021, SUC_lexicon_tags_45 },
    { 0xe852b021, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd2b025, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9272b02c, SUC_lexicon_tags_8 },
    { 0x6292b02d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d2b030, SUC_lexicon_tags_11 },
    { 0x6732b031, SUC_lexicon_tags_106 },
    { 0xfa92b032, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8792b036, SUC_lexicon_tags_56 },
    { 0x2d52b037, SUC_lexicon_tags_12 },
    { 0x2ff2b037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20d2b03a, SUC_lexicon_tags_19 },
    { 0xfdf2b03a, SUC_lexicon_tags_42 },
    { 0xe6d2b03a, SUC_lexicon_tags_15 },
    { 0x6f32b03d, SUC_lexicon_tags_7 },
    { 0x5792b03e, SUC_lexicon_tags_48 },
    { 0x6812b03f, SUC_lexicon_tags_47 },
    { 0x4ad2b03e, SUC_lexicon_tags_25 },
    { 0x3ed2b041, SUC_lexicon_tags_49 },
    { 0xee52b040, SUC_lexicon_tags_43 },
    { 0x2592b043, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc712b047, SUC_lexicon_tags_1 },
    { 0x7f72b048, SUC_lexicon_tags_29 },
    { 0x9772b049, SUC_lexicon_tags_12 },
    { 0x60d2b04a, SUC_lexicon_tags_66 },
    { 0x5992b047, SUC_lexicon_tags_8 },
    { 0x2a52b04c, SUC_lexicon_tags_253 },
    { 0x64b2b047, SUC_lexicon_tags_25 },
    { 0x7952b04b, SUC_lexicon_tags_28 },
    { 0x8fd2b04f, SUC_lexicon_tags_33 },
    { 0x1712b048, SUC_lexicon_tags_7 },
    { 0x4032b04d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde92b058, SUC_lexicon_tags_30 },
    { 0xce92b058, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfed2b05b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xde12b05d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd32b05f, SUC_lexicon_tags_2 },
    { 0x372b060, SUC_lexicon_tags_10 },
    { 0x54d2b060, SUC_lexicon_tags_17 },
    { 0x7052b062, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9312b067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd672b06b, SUC_lexicon_tags_63 },
    { 0x8b12b06b, SUC_lexicon_tags_2 },
    { 0x9af2b06d, SUC_lexicon_tags_10 },
    { 0x45f2b06e, SUC_lexicon_tags_12 },
    { 0xc6b2b06b, SUC_lexicon_tags_29 },
    { 0x3e72b070, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d2b074, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe6b2b076, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4812b07b, SUC_lexicon_tags_37 },
    { 0x8392b07c, SUC_lexicon_tags_31 },
    { 0x5ab2b07d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa332b07f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8f2b081, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2b084, SUC_lexicon_tags_25 },
    { 0x912b085, SUC_lexicon_tags_31 },
    { 0x2752b084, SUC_lexicon_tags_47 },
    { 0x2052b087, SUC_lexicon_tags_63 },
    { 0xaa72b088, SUC_lexicon_tags_47 },
    { 0xfd92b089, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d2b08c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff2b090, SUC_lexicon_tags_66 },
    { 0xad92b091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e52b096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f12b099, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc352b09d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e32b0a3, SUC_lexicon_tags_17 },
    { 0xa552b0a4, SUC_lexicon_tags_17 },
    { 0xbb72b0a5, SUC_lexicon_tags_37 },
    { 0x8b2b0a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1cd2b0a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd152b0aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff2b0ad, SUC_lexicon_tags_17 },
    { 0x9d92b0ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b2b0bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd052b0c2, SUC_lexicon_tags_25 },
    { 0x71b2b0c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced2b0c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6232b0cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172b0d3, SUC_lexicon_tags_25 },
    { 0x5ad2b0d4, SUC_lexicon_tags_7 },
    { 0xe3f2b0d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb812b0d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d2b0db, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb92b0de, SUC_lexicon_tags_65 },
    { 0x1a72b0df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c32b0e1, SUC_lexicon_tags_0 },
    { 0xddb2b0e2, SUC_lexicon_tags_55 },
    { 0x8372b0e1, SUC_lexicon_tags_33 },
    { 0xd032b0e4, SUC_lexicon_tags_7 },
    { 0x1992b0e4, SUC_lexicon_tags_2 },
    { 0x9df2b0e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb2b0ef, SUC_lexicon_tags_65 },
    { 0xf7d2b0f0, SUC_lexicon_tags_49 },
    { 0x63b2b0f1, SUC_lexicon_tags_11 },
    { 0x53f2b0f1, SUC_lexicon_tags_7 },
    { 0x2d92b0f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2b52b0f5, SUC_lexicon_tags_12 },
    { 0x1c72b0f5, SUC_lexicon_tags_18 },
    { 0xcf72b0f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd2b0fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd2b0fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17d2b100, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e12b107, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x4fd2b109, SUC_lexicon_tags_1 },
    { 0x58f2b109, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee92b10c, SUC_lexicon_tags_15 },
    { 0x45f2b10d, SUC_lexicon_tags_17 },
    { 0xd612b10e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb2b111, SUC_lexicon_tags_12 },
    { 0x1cd2b112, SUC_lexicon_tags_66 },
    { 0x29f2b112, SUC_lexicon_tags_25 },
    { 0x8f92b111, SUC_lexicon_tags_12 },
    { 0xc1b2b115, SUC_lexicon_tags_6 },
    { 0x4132b116, SUC_lexicon_tags_7 },
    { 0xf9b2b117, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb632b11b, SUC_lexicon_tags_4 },
    { 0x1d52b11c, SUC_lexicon_tags_47 },
    { 0x6ed2b11c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f2b120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e12b124, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd332b12a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe572b12c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4b32b12e, SUC_lexicon_tags_56 },
    { 0xb112b12f, SUC_lexicon_tags_2 },
    { 0xd8f2b12f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xce12b133, SUC_lexicon_tags_82 },
    { 0xe7d2b134, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1812b136, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xb9d2b138, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f12b13b, SUC_lexicon_tags_55 },
    { 0xee52b13c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8452b13f, SUC_lexicon_tags_10 },
    { 0xcc72b140, SUC_lexicon_tags_17 },
    { 0x5b52b140, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d32b143, SUC_lexicon_tags_1 },
    { 0x1012b143, SUC_lexicon_tags_79 },
    { 0x1ff2b145, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc052b14e, SUC_lexicon_tags_30 },
    { 0x7452b14f, SUC_lexicon_tags_7 },
    { 0xb0d2b14f, SUC_lexicon_tags_507 },
    { 0xf1d2b14e, SUC_lexicon_tags_43 },
    { 0xb8b2b152, SUC_lexicon_tags_21 },
    { 0x1db2b153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd792b157, SUC_lexicon_tags_38 },
    { 0x9c52b158, SUC_lexicon_tags_11 },
    { 0xc332b158, SUC_lexicon_tags_3 },
    { 0xc272b15a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2b15e, SUC_lexicon_tags_2 },
    { 0xe612b15f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb12b16d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd2b171, SUC_lexicon_tags_39 },
    { 0x4772b172, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc612b174, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf912b17f, SUC_lexicon_tags_35 },
    { 0x70f2b180, SUC_lexicon_tags_12 },
    { 0x112b181, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f2b185, SUC_lexicon_tags_15 },
    { 0x1b52b186, SUC_lexicon_tags_25 },
    { 0xd672b186, SUC_lexicon_tags_25 },
    { 0x5512b188, SUC_lexicon_tags_7 },
    { 0x7732b188, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x24f2b18b, SUC_lexicon_tags_15 },
    { 0x1332b18b, SUC_lexicon_tags_12 },
    { 0x78d2b18b, SUC_lexicon_tags_15 },
    { 0xf8d2b18e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x9bd2b190, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57b2b192, SUC_lexicon_tags_28 },
    { 0xbbf2b193, SUC_lexicon_tags_91 },
    { 0xd932b192, SUC_lexicon_tags_37 },
    { 0x5252b195, SUC_lexicon_tags_7 },
    { 0xab32b196, SUC_lexicon_tags_1 },
    { 0x6432b196, SUC_lexicon_tags_37 },
    { 0x5992b198, SUC_lexicon_tags_37 },
    { 0x6132b195, SUC_lexicon_tags_27 },
    { 0x572b19a, SUC_lexicon_tags_77 },
    { 0xf9d2b19b, SUC_lexicon_tags_0 },
    { 0xe292b19b, SUC_lexicon_tags_37 },
    { 0x332b19c, SUC_lexicon_tags_0 },
    { 0x6bb2b19e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf032b1a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d52b1a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452b1a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfe72b1ab, SUC_lexicon_tags_65 },
    { 0x8ad2b1ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7292b1b0, SUC_lexicon_tags_12 },
    { 0xac52b1b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df2b1b7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e32b1bc, SUC_lexicon_tags_33 },
    { 0xdf2b1bd, SUC_lexicon_tags_35 },
    { 0xd3b2b1be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8152b1c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1292b1c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f32b1ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfb92b1cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d2b1cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc492b1d2, SUC_lexicon_tags_12 },
    { 0x5c92b1d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f2b1d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e32b1de, SUC_lexicon_tags_84 },
    { 0xdd32b1de, SUC_lexicon_tags_45 },
    { 0xc5f2b1e0, SUC_lexicon_tags_60 },
    { 0x1b92b1e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd772b1e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd612b1e7, SUC_lexicon_tags_7 },
    { 0x7152b1e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb12b1ee, SUC_lexicon_tags_30 },
    { 0xceb2b1ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b2b1f3, SUC_lexicon_tags_14 },
    { 0x1d52b1f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4692b1f7, SUC_lexicon_tags_88 },
    { 0x792b1f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c52b1fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f2b205, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f2b208, SUC_lexicon_tags_27 },
    { 0xeed2b209, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb232b20c, SUC_lexicon_tags_27 },
    { 0x4f32b20c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb12b213, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed12b219, SUC_lexicon_tags_45 },
    { 0xc892b219, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2352b222, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe32b224, SUC_lexicon_tags_77 },
    { 0x21f2b224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d12b227, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4912b22b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb52b22e, SUC_lexicon_tags_15 },
    { 0xfa12b22f, SUC_lexicon_tags_29 },
    { 0x62b2b230, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeb72b232, SUC_lexicon_tags_37 },
    { 0x6452b233, SUC_lexicon_tags_8 },
    { 0x76d2b232, SUC_lexicon_tags_416 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d2b238, SUC_lexicon_tags_19 },
    { 0x2cf2b239, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc692b23e, SUC_lexicon_tags_14 },
    { 0x29b2b23e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3d2b241, SUC_lexicon_tags_16 },
    { 0xb932b242, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd432b244, SUC_lexicon_tags_17 },
    { 0x2eb2b245, SUC_lexicon_tags_27 },
    { 0x5ff2b245, SUC_lexicon_tags_30 },
    { 0x8a32b244, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f2b24a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa512b24e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f2b253, SUC_lexicon_tags_1 },
    { 0x1c12b253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc612b257, SUC_lexicon_tags_65 },
    { 0x7832b258, SUC_lexicon_tags_15 },
    { 0x812b259, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fb2b25b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a52b261, SUC_lexicon_tags_15 },
    { 0xdd92b261, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbbd2b264, SUC_lexicon_tags_37 },
    { 0x4e92b265, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7532b26a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb2b26d, SUC_lexicon_tags_27 },
    { 0x8f32b26d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2432b270, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d2b274, SUC_lexicon_tags_63 },
    { 0x1d2b275, SUC_lexicon_tags_23 },
    { 0xd652b275, SUC_lexicon_tags_15 },
    { 0xe972b274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c52b27b, SUC_lexicon_tags_7 },
    { 0x7bd2b27c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7452b283, SUC_lexicon_tags_84 },
    { 0xc172b284, SUC_lexicon_tags_27 },
    { 0xac12b283, SUC_lexicon_tags_6 },
    { 0xee72b286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b2b289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f92b28b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2b28f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5532b297, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a32b29f, SUC_lexicon_tags_8 },
    { 0xe332b2a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb2b2a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd72b2a9, SUC_lexicon_tags_6 },
    { 0x7b72b2aa, SUC_lexicon_tags_9 },
    { 0xe412b2ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc512b2ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd2b2b8, SUC_lexicon_tags_6 },
    { 0x1472b2b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2672b2bc, SUC_lexicon_tags_0 },
    { 0x532b2bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc752b2bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b72b2c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f72b2c8, SUC_lexicon_tags_15 },
    { 0xaff2b2c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad2b2cd, SUC_lexicon_tags_124 },
    { 0xe192b2ce, SUC_lexicon_tags_25 },
    { 0xdd72b2cf, SUC_lexicon_tags_19 },
    { 0xb472b2cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b12b2d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc5d2b2d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a32b2d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe652b2e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6232b2e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b2b2e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac12b2eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3952b2ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2972b2f9, SUC_lexicon_tags_12 },
    { 0x6a12b2f9, SUC_lexicon_tags_37 },
    { 0x9972b2fb, SUC_lexicon_tags_23 },
    { 0x1b12b2fc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d2b300, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf432b303, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf132b305, SUC_lexicon_tags_32 },
    { 0xce12b305, SUC_lexicon_tags_68 },
    { 0xa892b307, SUC_lexicon_tags_38 },
    { 0xe1b2b308, SUC_lexicon_tags_17 },
    { 0x9a12b309, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x92f2b30b, SUC_lexicon_tags_35 },
    { 0xc852b30c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe972b30e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfeb2b310, SUC_lexicon_tags_6 },
    { 0x292b310, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe212b315, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x452b317, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xed92b31a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc272b31f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce12b321, SUC_lexicon_tags_25 },
    { 0x4fb2b321, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6592b327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22d2b329, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab92b32d, SUC_lexicon_tags_55 },
    { 0x8ab2b32e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd2b335, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b92b33a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f2b33c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef12b33e, SUC_lexicon_tags_15 },
    { 0xa312b33f, SUC_lexicon_tags_25 },
    { 0x352b340, SUC_lexicon_tags_34 },
    { 0x52b2b341, SUC_lexicon_tags_37 },
    { 0xea32b33e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f2b345, SUC_lexicon_tags_107 },
    { 0x6f92b346, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x972b348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f2b34d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7432b351, SUC_lexicon_tags_63 },
    { 0xf872b352, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa772b359, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7172b35e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd2b361, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab2b364, SUC_lexicon_tags_32 },
    { 0x1eb2b364, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d2b36b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xffb2b36d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5052b371, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4392b373, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea12b376, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe392b37c, SUC_lexicon_tags_48 },
    { 0x7132b37d, SUC_lexicon_tags_47 },
    { 0xac92b37e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb2b380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaf2b382, SUC_lexicon_tags_30 },
    { 0xd12b383, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b2b388, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe192b38a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d2b390, SUC_lexicon_tags_12 },
    { 0xd0b2b391, SUC_lexicon_tags_2 },
    { 0x27b2b391, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a52b39a, SUC_lexicon_tags_12 },
    { 0x68b2b39b, SUC_lexicon_tags_11 },
    { 0xbd32b39b, SUC_lexicon_tags_13 },
    { 0xe972b39d, SUC_lexicon_tags_11 },
    { 0xc2b2b39b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f2b3a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5352b3ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb2b3b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6892b3b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec52b3b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db2b3c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c32b3cf, SUC_lexicon_tags_25 },
    { 0x80f2b3d0, SUC_lexicon_tags_25 },
    { 0xe0d2b3d1, SUC_lexicon_tags_39 },
    { 0xbfd2b3d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab2b3d5, SUC_lexicon_tags_45 },
    { 0xa592b3d5, SUC_lexicon_tags_17 },
    { 0xb192b3d7, SUC_lexicon_tags_1 },
    { 0xc812b3d7, SUC_lexicon_tags_11 },
    { 0x8c32b3d8, SUC_lexicon_tags_2 },
    { 0x63d2b3d5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f2b3df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb2b3e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad92b3ea, SUC_lexicon_tags_17 },
    { 0xc72b3ea, SUC_lexicon_tags_7 },
    { 0x6192b3ec, SUC_lexicon_tags_12 },
    { 0x2672b3ea, SUC_lexicon_tags_23 },
    { 0x2592b3ee, SUC_lexicon_tags_66 },
    { 0x2b72b3ef, SUC_lexicon_tags_26 },
    { 0x6d2b3eb, SUC_lexicon_tags_9 },
    { 0xda92b3f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d2b3f4, SUC_lexicon_tags_0 },
    { 0xeff2b3f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb2b3f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4232b3fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2952b401, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f2b404, SUC_lexicon_tags_11 },
    { 0xbf52b404, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x3452b407, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe912b40b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaef2b40d, SUC_lexicon_tags_7 },
    { 0x9d2b40e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf412b411, SUC_lexicon_tags_2 },
    { 0xc952b411, SUC_lexicon_tags_43 },
    { 0xb6d2b411, SUC_lexicon_tags_2 },
    { 0x73b2b414, SUC_lexicon_tags_46 },
    { 0xde92b415, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd2b418, SUC_lexicon_tags_17 },
    { 0x4632b419, SUC_lexicon_tags_15 },
    { 0x7312b41a, SUC_lexicon_tags_28 },
    { 0x94f2b418, SUC_lexicon_tags_15 },
    { 0x6b12b41c, SUC_lexicon_tags_1 },
    { 0x8b2b41c, SUC_lexicon_tags_25 },
    { 0xeb92b41c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30b2b420, SUC_lexicon_tags_11 },
    { 0x9bf2b421, SUC_lexicon_tags_60 },
    { 0x7bf2b422, SUC_lexicon_tags_14 },
    { 0x4792b423, SUC_lexicon_tags_27 },
    { 0xcc92b421, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d2b428, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b2b42b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db2b432, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa212b436, SUC_lexicon_tags_11 },
    { 0xca32b436, SUC_lexicon_tags_15 },
    { 0x9472b438, SUC_lexicon_tags_1 },
    { 0xbc92b438, SUC_lexicon_tags_27 },
    { 0xfb12b439, SUC_lexicon_tags_11 },
    { 0xddd2b43a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1712b43d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb352b440, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f2b444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e92b44d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x23b2b44f, SUC_lexicon_tags_37 },
    { 0x2d12b450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb2b453, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f92b458, SUC_lexicon_tags_23 },
    { 0xc1f2b459, SUC_lexicon_tags_12 },
    { 0x2c52b45a, SUC_lexicon_tags_25 },
    { 0x2a72b45b, SUC_lexicon_tags_11 },
    { 0xc8d2b45c, SUC_lexicon_tags_61 },
    { 0x3372b45d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33d2b45f, SUC_lexicon_tags_21 },
    { 0xd692b460, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1772b469, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd2b46d, SUC_lexicon_tags_15 },
    { 0xa772b46e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc5f2b470, SUC_lexicon_tags_17 },
    { 0x2f12b470, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7852b474, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b52b476, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf2b479, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe752b47b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf2b480, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9232b482, SUC_lexicon_tags_14 },
    { 0x7072b482, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a52b491, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf12b498, SUC_lexicon_tags_17 },
    { 0x3e32b499, SUC_lexicon_tags_17 },
    { 0x1032b499, SUC_lexicon_tags_30 },
    { 0xb432b499, SUC_lexicon_tags_25 },
    { 0xe92b49c, SUC_lexicon_tags_37 },
    { 0xecb2b49c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6f2b49f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b2b4a5, SUC_lexicon_tags_13 },
    { 0x172b4a5, SUC_lexicon_tags_7 },
    { 0xe8f2b4a7, SUC_lexicon_tags_37 },
    { 0xbbd2b4a8, SUC_lexicon_tags_2 },
    { 0x4d2b4a9, SUC_lexicon_tags_140 },
    { 0x392b4a9, SUC_lexicon_tags_25 },
    { 0x9ab2b4a9, SUC_lexicon_tags_25 },
    { 0xab32b4ab, SUC_lexicon_tags_7 },
    { 0xcd2b4aa, SUC_lexicon_tags_27 },
    { 0x65b2b4ae, SUC_lexicon_tags_37 },
    { 0xfab2b4ae, SUC_lexicon_tags_30 },
    { 0xc532b4af, SUC_lexicon_tags_43 },
    { 0x9152b4ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfcb2b4b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd872b4b8, SUC_lexicon_tags_6 },
    { 0x2892b4b8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d2b4bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e92b4bf, SUC_lexicon_tags_1096 },
    { 0, NULL },
    { 0, NULL },
    { 0xd492b4c2, SUC_lexicon_tags_45 },
    { 0xf712b4c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6752b4c8, SUC_lexicon_tags_45 },
    { 0xf752b4c8, SUC_lexicon_tags_0 },
    { 0xf512b4ca, SUC_lexicon_tags_27 },
    { 0x44f2b4cb, SUC_lexicon_tags_7 },
    { 0x5d32b4cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd2b4d0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e72b4d3, SUC_lexicon_tags_33 },
    { 0xb812b4d3, SUC_lexicon_tags_12 },
    { 0x8032b4d5, SUC_lexicon_tags_37 },
    { 0xdbb2b4d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x212b4d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1612b4dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa872b4e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d2b4ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf212b4f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe72b4f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe632b4fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4072b500, SUC_lexicon_tags_16 },
    { 0x74b2b501, SUC_lexicon_tags_30 },
    { 0x99f2b502, SUC_lexicon_tags_37 },
    { 0x3032b503, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f2b507, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3952b50a, SUC_lexicon_tags_201 },
    { 0x1912b50a, SUC_lexicon_tags_1 },
    { 0xf0d2b50c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9a12b50e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b2b511, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd212b514, SUC_lexicon_tags_12 },
    { 0xb4d2b515, SUC_lexicon_tags_37 },
    { 0xbad2b515, SUC_lexicon_tags_49 },
    { 0x4372b514, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xe092b51a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b2b51f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd272b523, SUC_lexicon_tags_37 },
    { 0xb6f2b523, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9612b526, SUC_lexicon_tags_31 },
    { 0xa2f2b526, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f2b52c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1612b52e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2912b534, SUC_lexicon_tags_12 },
    { 0xc4b2b535, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f2b539, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f2b53e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d92b540, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78d2b542, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb2b546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf2b54b, SUC_lexicon_tags_2 },
    { 0x4252b54b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5952b559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d2b55d, SUC_lexicon_tags_17 },
    { 0xcc72b55e, SUC_lexicon_tags_1 },
    { 0x2c12b55f, SUC_lexicon_tags_37 },
    { 0x7b12b55d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb2b566, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dd2b568, SUC_lexicon_tags_42 },
    { 0xe9d2b569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b92b56b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1112b56d, SUC_lexicon_tags_2 },
    { 0x6a52b56e, SUC_lexicon_tags_23 },
    { 0xf032b56f, SUC_lexicon_tags_25 },
    { 0xf892b570, SUC_lexicon_tags_25 },
    { 0x1ad2b570, SUC_lexicon_tags_7 },
    { 0xf932b56f, SUC_lexicon_tags_33 },
    { 0x6472b573, SUC_lexicon_tags_11 },
    { 0x8df2b574, SUC_lexicon_tags_56 },
    { 0xe592b573, SUC_lexicon_tags_37 },
    { 0xb932b575, SUC_lexicon_tags_43 },
    { 0x7f52b577, SUC_lexicon_tags_15 },
    { 0xec52b578, SUC_lexicon_tags_12 },
    { 0xe8f2b578, SUC_lexicon_tags_33 },
    { 0xff52b579, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b2b580, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb2b584, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6352b586, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc412b588, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bf2b58a, SUC_lexicon_tags_17 },
    { 0x69d2b58b, SUC_lexicon_tags_25 },
    { 0x8632b58c, SUC_lexicon_tags_15 },
    { 0xfb92b58c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca92b58f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x80d2b591, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61b2b593, SUC_lexicon_tags_12 },
    { 0x2d92b593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa12b596, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9792b59a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf2b59e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ab2b5a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb72b5a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc112b5a4, SUC_lexicon_tags_1 },
    { 0x7512b5a5, SUC_lexicon_tags_55 },
    { 0x7352b5a6, SUC_lexicon_tags_49 },
    { 0xc592b5a6, SUC_lexicon_tags_65 },
    { 0x7572b5a5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed2b5ab, SUC_lexicon_tags_37 },
    { 0x752b5ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f2b5b0, SUC_lexicon_tags_68 },
    { 0xaab2b5b1, SUC_lexicon_tags_15 },
    { 0x57b2b5b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8352b5b5, SUC_lexicon_tags_1035 },
    { 0, NULL },
    { 0, NULL },
    { 0xd612b5b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd2b5bc, SUC_lexicon_tags_43 },
    { 0x91d2b5bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef2b5c0, SUC_lexicon_tags_49 },
    { 0x8c32b5c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d52b5c4, SUC_lexicon_tags_1 },
    { 0x7192b5c4, SUC_lexicon_tags_9 },
    { 0x4952b5c6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf2b5cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe232b5d1, SUC_lexicon_tags_161 },
    { 0x55d2b5d2, SUC_lexicon_tags_11 },
    { 0x74b2b5d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f72b5d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272b5da, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd2b5e5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe412b5ed, SUC_lexicon_tags_7 },
    { 0xf2f2b5ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d2b5f4, SUC_lexicon_tags_45 },
    { 0xe1b2b5f5, SUC_lexicon_tags_26 },
    { 0x1752b5f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9192b5f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd492b5fb, SUC_lexicon_tags_23 },
    { 0xbc92b5fc, SUC_lexicon_tags_10 },
    { 0xba52b5fc, SUC_lexicon_tags_39 },
    { 0x30d2b5fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd32b601, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f12b605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af2b608, SUC_lexicon_tags_17 },
    { 0xbd2b609, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9252b60d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd2b611, SUC_lexicon_tags_37 },
    { 0x7b32b612, SUC_lexicon_tags_43 },
    { 0x5dd2b612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8092b615, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b2b61a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f2b61d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d2b621, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6372b624, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5b2b626, SUC_lexicon_tags_37 },
    { 0xfc92b626, SUC_lexicon_tags_63 },
    { 0x7212b628, SUC_lexicon_tags_25 },
    { 0xea32b629, SUC_lexicon_tags_17 },
    { 0xfcd2b629, SUC_lexicon_tags_11 },
    { 0x81f2b629, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec12b62f, SUC_lexicon_tags_9 },
    { 0x1f2b62f, SUC_lexicon_tags_35 },
    { 0xf1f2b631, SUC_lexicon_tags_37 },
    { 0x9292b630, SUC_lexicon_tags_87 },
    { 0x6d92b633, SUC_lexicon_tags_33 },
    { 0x7c32b633, SUC_lexicon_tags_11 },
    { 0x9892b635, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf2b638, SUC_lexicon_tags_21 },
    { 0x2252b639, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9672b63f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3212b644, SUC_lexicon_tags_3 },
    { 0x8b92b645, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b2b64b, SUC_lexicon_tags_8 },
    { 0x1092b64c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f72b64f, SUC_lexicon_tags_17 },
    { 0xf12b64f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f72b652, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4852b654, SUC_lexicon_tags_63 },
    { 0x1ad2b655, SUC_lexicon_tags_2 },
    { 0xa2b2b655, SUC_lexicon_tags_25 },
    { 0xc872b656, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x2872b659, SUC_lexicon_tags_15 },
    { 0x33b2b659, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6b52b65c, SUC_lexicon_tags_65 },
    { 0x49f2b65d, SUC_lexicon_tags_31 },
    { 0x34d2b65e, SUC_lexicon_tags_26 },
    { 0xf972b65c, SUC_lexicon_tags_43 },
    { 0x4992b660, SUC_lexicon_tags_14 },
    { 0xa112b65c, SUC_lexicon_tags_14 },
    { 0x9292b65d, SUC_lexicon_tags_17 },
    { 0x712b660, SUC_lexicon_tags_15 },
    { 0xc712b664, SUC_lexicon_tags_37 },
    { 0x64b2b665, SUC_lexicon_tags_15 },
    { 0x8c72b65c, SUC_lexicon_tags_25 },
    { 0x26b2b667, SUC_lexicon_tags_17 },
    { 0xdc52b667, SUC_lexicon_tags_0 },
    { 0xa852b669, SUC_lexicon_tags_34 },
    { 0xa352b665, SUC_lexicon_tags_0 },
    { 0x7452b66b, SUC_lexicon_tags_28 },
    { 0xb192b669, SUC_lexicon_tags_37 },
    { 0x6b52b66d, SUC_lexicon_tags_45 },
    { 0xddf2b66d, SUC_lexicon_tags_18 },
    { 0x99f2b66f, SUC_lexicon_tags_19 },
    { 0x9f72b66e, SUC_lexicon_tags_11 },
    { 0x9fb2b671, SUC_lexicon_tags_87 },
    { 0x7bd2b66d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaf52b674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b52b676, SUC_lexicon_tags_11 },
    { 0x5052b677, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f2b679, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6152b67b, SUC_lexicon_tags_28 },
    { 0xa152b67c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7112b67f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1752b683, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad2b686, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb852b68a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb612b68c, SUC_lexicon_tags_2 },
    { 0x95f2b68d, SUC_lexicon_tags_17 },
    { 0x1572b68e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b2b692, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab2b699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c92b69b, SUC_lexicon_tags_45 },
    { 0x57b2b69b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af2b6a1, SUC_lexicon_tags_43 },
    { 0xa7b2b6a2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4312b6a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3412b6a8, SUC_lexicon_tags_63 },
    { 0xcd72b6a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc472b6ac, SUC_lexicon_tags_60 },
    { 0x4b32b6ad, SUC_lexicon_tags_19 },
    { 0x1192b6ae, SUC_lexicon_tags_40 },
    { 0x2832b6ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f52b6b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf132b6ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8752b6bc, SUC_lexicon_tags_29 },
    { 0x9912b6bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af2b6c1, SUC_lexicon_tags_37 },
    { 0xc512b6c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5172b6c5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd852b6c7, SUC_lexicon_tags_33 },
    { 0x2432b6c7, SUC_lexicon_tags_37 },
    { 0xbc12b6c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21d2b6cb, SUC_lexicon_tags_74 },
    { 0x1cf2b6cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe952b6ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd2b6d3, SUC_lexicon_tags_82 },
    { 0xd2b6d4, SUC_lexicon_tags_17 },
    { 0x5f32b6d5, SUC_lexicon_tags_15 },
    { 0x26d2b6d6, SUC_lexicon_tags_2 },
    { 0x7232b6d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d92b6dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2b6df, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f2b6e6, SUC_lexicon_tags_11 },
    { 0xedf2b6e6, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf2b6ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb2b6f0, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d2b6f4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12b6fa, SUC_lexicon_tags_12 },
    { 0xa9b2b6fa, SUC_lexicon_tags_7 },
    { 0x6292b6fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90b2b6fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x832b700, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa692b703, SUC_lexicon_tags_301 },
    { 0x4b12b704, SUC_lexicon_tags_25 },
    { 0x8df2b704, SUC_lexicon_tags_0 },
    { 0x57f2b706, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b2b709, SUC_lexicon_tags_14 },
    { 0x8fb2b70a, SUC_lexicon_tags_25 },
    { 0x75b2b70a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4372b70d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd2b710, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6792b716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4012b71a, SUC_lexicon_tags_28 },
    { 0x3912b71b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a72b720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2b725, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a92b729, SUC_lexicon_tags_3 },
    { 0xbb92b729, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb2b72d, SUC_lexicon_tags_4 },
    { 0xf1f2b72d, SUC_lexicon_tags_12 },
    { 0x6872b72d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d2b732, SUC_lexicon_tags_6 },
    { 0x6612b732, SUC_lexicon_tags_56 },
    { 0xfcb2b734, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xa2b2b736, SUC_lexicon_tags_37 },
    { 0xa8d2b736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe292b73a, SUC_lexicon_tags_37 },
    { 0x1212b73b, SUC_lexicon_tags_9 },
    { 0xcbd2b73a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8512b740, SUC_lexicon_tags_1 },
    { 0x35b2b741, SUC_lexicon_tags_25 },
    { 0x6bb2b741, SUC_lexicon_tags_25 },
    { 0x6b52b741, SUC_lexicon_tags_11 },
    { 0x83d2b740, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d2b74c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcd72b74e, SUC_lexicon_tags_7 },
    { 0xefd2b74e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa32b751, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bd2b753, SUC_lexicon_tags_12 },
    { 0xdd92b753, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xea52b757, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4092b75b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f2b75e, SUC_lexicon_tags_31 },
    { 0x1b2b75e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92b764, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b2b768, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1012b76b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5752b76d, SUC_lexicon_tags_25 },
    { 0xeb52b76d, SUC_lexicon_tags_11 },
    { 0x36d2b76f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7192b771, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5d2b773, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1f2b775, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5bd2b777, SUC_lexicon_tags_56 },
    { 0xe9b2b777, SUC_lexicon_tags_3 },
    { 0x5af2b778, SUC_lexicon_tags_11 },
    { 0x38d2b77a, SUC_lexicon_tags_14 },
    { 0x26b2b77b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xca52b77d, SUC_lexicon_tags_25 },
    { 0xc9b2b77e, SUC_lexicon_tags_0 },
    { 0x95f2b77e, SUC_lexicon_tags_0 },
    { 0xe52b77e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b32b782, SUC_lexicon_tags_35 },
    { 0x79d2b783, SUC_lexicon_tags_7 },
    { 0x64b2b784, SUC_lexicon_tags_45 },
    { 0x6872b785, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x1c52b787, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f2b78a, SUC_lexicon_tags_60 },
    { 0x58b2b78b, SUC_lexicon_tags_0 },
    { 0x72d2b78c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7332b790, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x59f2b792, SUC_lexicon_tags_12 },
    { 0x4d12b792, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3bd2b795, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8552b799, SUC_lexicon_tags_26 },
    { 0xa9b2b799, SUC_lexicon_tags_17 },
    { 0xef32b79b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b52b7a0, SUC_lexicon_tags_2 },
    { 0x45f2b7a1, SUC_lexicon_tags_33 },
    { 0xadd2b7a2, SUC_lexicon_tags_39 },
    { 0x7612b7a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x212b7a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb292b7ab, SUC_lexicon_tags_64 },
    { 0x76f2b7ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d92b7af, SUC_lexicon_tags_27 },
    { 0x84b2b7b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f2b7b4, SUC_lexicon_tags_31 },
    { 0xaff2b7b5, SUC_lexicon_tags_55 },
    { 0xe72b7b5, SUC_lexicon_tags_80 },
    { 0x13d2b7b7, SUC_lexicon_tags_17 },
    { 0x1c72b7b8, SUC_lexicon_tags_27 },
    { 0x4ad2b7b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1992b7bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd192b7bd, SUC_lexicon_tags_7 },
    { 0x9a12b7be, SUC_lexicon_tags_47 },
    { 0xbbd2b7be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5e92b7c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb32b7c4, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xbe12b7c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b2b7ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2b7ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9252b7d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b52b7d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2912b7da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf732b7dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xed32b7de, SUC_lexicon_tags_35 },
    { 0x9c32b7df, SUC_lexicon_tags_37 },
    { 0x5432b7e0, SUC_lexicon_tags_17 },
    { 0xc112b7e1, SUC_lexicon_tags_17 },
    { 0xc392b7e0, SUC_lexicon_tags_49 },
    { 0x4a52b7e3, SUC_lexicon_tags_12 },
    { 0xf1d2b7e4, SUC_lexicon_tags_32 },
    { 0x6032b7e5, SUC_lexicon_tags_17 },
    { 0x4132b7e2, SUC_lexicon_tags_6 },
    { 0xca12b7e7, SUC_lexicon_tags_47 },
    { 0x2792b7e7, SUC_lexicon_tags_17 },
    { 0x4bb2b7e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b2b7f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952b7f5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f52b7fc, SUC_lexicon_tags_19 },
    { 0xaef2b7fc, SUC_lexicon_tags_2 },
    { 0x6572b7fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3692b800, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb2b803, SUC_lexicon_tags_26 },
    { 0x4772b804, SUC_lexicon_tags_47 },
    { 0x4452b805, SUC_lexicon_tags_14 },
    { 0x7d12b806, SUC_lexicon_tags_14 },
    { 0xa0b2b807, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1fd2b809, SUC_lexicon_tags_57 },
    { 0xfe32b809, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf2b80e, SUC_lexicon_tags_12 },
    { 0x8092b80f, SUC_lexicon_tags_77 },
    { 0x63d2b80f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb32b812, SUC_lexicon_tags_732 },
    { 0, NULL },
    { 0xe412b814, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x212b816, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe992b818, SUC_lexicon_tags_13 },
    { 0xda12b818, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b2b81d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f2b822, SUC_lexicon_tags_17 },
    { 0x8892b823, SUC_lexicon_tags_11 },
    { 0x56d2b823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc692b827, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf892b82c, SUC_lexicon_tags_6 },
    { 0xac12b82d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf2b835, SUC_lexicon_tags_29 },
    { 0x7fb2b835, SUC_lexicon_tags_14 },
    { 0x8c92b837, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fb2b839, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf2b83c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ff2b83e, SUC_lexicon_tags_17 },
    { 0x60d2b83f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3532b844, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9952b84b, SUC_lexicon_tags_11 },
    { 0xb5f2b84c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb2b857, SUC_lexicon_tags_23 },
    { 0x56f2b857, SUC_lexicon_tags_795 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f12b85e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8892b861, SUC_lexicon_tags_12 },
    { 0x3d2b861, SUC_lexicon_tags_140 },
    { 0x3092b863, SUC_lexicon_tags_11 },
    { 0xeaf2b863, SUC_lexicon_tags_49 },
    { 0x2352b864, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaed2b867, SUC_lexicon_tags_66 },
    { 0x1912b868, SUC_lexicon_tags_10 },
    { 0x60b2b867, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5172b86c, SUC_lexicon_tags_9 },
    { 0x90b2b86d, SUC_lexicon_tags_7 },
    { 0x96b2b86d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b2b877, SUC_lexicon_tags_45 },
    { 0x6d52b877, SUC_lexicon_tags_27 },
    { 0x5b72b879, SUC_lexicon_tags_26 },
    { 0x172b879, SUC_lexicon_tags_23 },
    { 0x23d2b87b, SUC_lexicon_tags_12 },
    { 0x1232b878, SUC_lexicon_tags_8 },
    { 0xfb2b87c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7612b87f, SUC_lexicon_tags_65 },
    { 0x3072b880, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe112b885, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd1f2b887, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2652b889, SUC_lexicon_tags_15 },
    { 0x31d2b889, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3672b88c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5972b88e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f2b897, SUC_lexicon_tags_60 },
    { 0xa832b898, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b2b89f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3b72b8a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f2b8a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f92b8a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6692b8ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7872b8ae, SUC_lexicon_tags_11 },
    { 0x87d2b8af, SUC_lexicon_tags_25 },
    { 0xdab2b8b0, SUC_lexicon_tags_23 },
    { 0x452b8b1, SUC_lexicon_tags_37 },
    { 0xd5f2b8b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3c32b8b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab2b8b8, SUC_lexicon_tags_25 },
    { 0x61b2b8b9, SUC_lexicon_tags_45 },
    { 0x6932b8b9, SUC_lexicon_tags_37 },
    { 0xb392b8bb, SUC_lexicon_tags_31 },
    { 0xa3f2b8bc, SUC_lexicon_tags_23 },
    { 0xedd2b8bd, SUC_lexicon_tags_17 },
    { 0xc652b8be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x812b8c3, SUC_lexicon_tags_17 },
    { 0x9d52b8c4, SUC_lexicon_tags_106 },
    { 0xcf92b8c4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5dd2b8c7, SUC_lexicon_tags_63 },
    { 0xb5b2b8c8, SUC_lexicon_tags_57 },
    { 0xc5f2b8c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df2b8cd, SUC_lexicon_tags_27 },
    { 0x85d2b8ce, SUC_lexicon_tags_28 },
    { 0x3d32b8cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d32b8d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda12b8d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b2b8d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d92b8de, SUC_lexicon_tags_66 },
    { 0x9372b8df, SUC_lexicon_tags_38 },
    { 0xd992b8e0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8d32b8e2, SUC_lexicon_tags_2 },
    { 0x5f72b8e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6792b8e5, SUC_lexicon_tags_13 },
    { 0x10f2b8e5, SUC_lexicon_tags_17 },
    { 0xd812b8e7, SUC_lexicon_tags_17 },
    { 0x6552b8e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb32b8eb, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xc32b8ed, SUC_lexicon_tags_10 },
    { 0xc12b8ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb652b8f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2232b8f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2272b8f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23b2b8fa, SUC_lexicon_tags_37 },
    { 0xc792b8fa, SUC_lexicon_tags_158 },
    { 0x82f2b8fb, SUC_lexicon_tags_27 },
    { 0xd832b8fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb832b8ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d92b903, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd2b908, SUC_lexicon_tags_17 },
    { 0x5c72b908, SUC_lexicon_tags_8 },
    { 0x8bb2b908, SUC_lexicon_tags_1 },
    { 0x9832b90a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1412b90e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f2b913, SUC_lexicon_tags_19 },
    { 0xc732b913, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd472b916, SUC_lexicon_tags_25 },
    { 0xd452b917, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf092b921, SUC_lexicon_tags_87 },
    { 0x1af2b922, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6672b925, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d92b92e, SUC_lexicon_tags_17 },
    { 0xee92b92f, SUC_lexicon_tags_37 },
    { 0x8c92b92e, SUC_lexicon_tags_10 },
    { 0xc952b930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe092b934, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5272b936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f2b93a, SUC_lexicon_tags_6 },
    { 0xab52b93a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd12b93e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf72b941, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd932b944, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda52b946, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd932b94a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6572b956, SUC_lexicon_tags_12 },
    { 0xc1d2b957, SUC_lexicon_tags_2 },
    { 0xe092b956, SUC_lexicon_tags_13 },
    { 0x9f32b959, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7392b95b, SUC_lexicon_tags_45 },
    { 0x19d2b95c, SUC_lexicon_tags_17 },
    { 0xa92b95c, SUC_lexicon_tags_12 },
    { 0x8e72b95e, SUC_lexicon_tags_57 },
    { 0x48f2b95c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd332b962, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xdab2b964, SUC_lexicon_tags_77 },
    { 0xdf92b965, SUC_lexicon_tags_6 },
    { 0xb432b966, SUC_lexicon_tags_23 },
    { 0xdd52b967, SUC_lexicon_tags_17 },
    { 0x5d12b964, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4912b96b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4d2b96d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52b973, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d2b977, SUC_lexicon_tags_2 },
    { 0x5c52b978, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8612b97b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa852b97e, SUC_lexicon_tags_37 },
    { 0xd892b97f, SUC_lexicon_tags_25 },
    { 0x8b32b97f, SUC_lexicon_tags_37 },
    { 0xd3b2b97f, SUC_lexicon_tags_27 },
    { 0x24d2b981, SUC_lexicon_tags_55 },
    { 0x3d12b980, SUC_lexicon_tags_15 },
    { 0xb712b97e, SUC_lexicon_tags_11 },
    { 0xf5b2b985, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d32b98a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8612b98f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c12b994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a52b99b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x65f2b99d, SUC_lexicon_tags_15 },
    { 0xf012b99e, SUC_lexicon_tags_17 },
    { 0xc4b2b99e, SUC_lexicon_tags_27 },
    { 0x5ab2b99e, SUC_lexicon_tags_1 },
    { 0xeb52b9a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd72b9a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3812b9ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8412b9b0, SUC_lexicon_tags_8 },
    { 0x7032b9b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bb2b9b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c92b9b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4052b9b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b2b9bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4892b9c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe372b9c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb52b9c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d2b9c9, SUC_lexicon_tags_14 },
    { 0xbdb2b9ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef2b9cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d2b9cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd92b9d2, SUC_lexicon_tags_82 },
    { 0x53f2b9d2, SUC_lexicon_tags_25 },
    { 0xeb32b9d4, SUC_lexicon_tags_17 },
    { 0x3812b9d3, SUC_lexicon_tags_61 },
    { 0xbf92b9d6, SUC_lexicon_tags_17 },
    { 0xb1b2b9d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b2b9db, SUC_lexicon_tags_2 },
    { 0x4b32b9dc, SUC_lexicon_tags_14 },
    { 0xbc32b9dc, SUC_lexicon_tags_65 },
    { 0xd3f2b9de, SUC_lexicon_tags_45 },
    { 0x88d2b9df, SUC_lexicon_tags_7 },
    { 0xfa52b9de, SUC_lexicon_tags_15 },
    { 0x2232b9dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d2b9ea, SUC_lexicon_tags_37 },
    { 0x56b2b9eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfc32b9ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b2b9f3, SUC_lexicon_tags_33 },
    { 0x5172b9f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5332b9f7, SUC_lexicon_tags_17 },
    { 0xb4d2b9f8, SUC_lexicon_tags_705 },
    { 0x2e52b9f9, SUC_lexicon_tags_66 },
    { 0xd172b9f9, SUC_lexicon_tags_38 },
    { 0x21d2b9fa, SUC_lexicon_tags_17 },
    { 0x3b2b9fa, SUC_lexicon_tags_19 },
    { 0x5b2b9fb, SUC_lexicon_tags_19 },
    { 0xd272b9fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d2ba03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad2ba06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f2ba09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f2ba0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d2ba10, SUC_lexicon_tags_60 },
    { 0xc92ba11, SUC_lexicon_tags_49 },
    { 0x3192ba12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e12ba15, SUC_lexicon_tags_17 },
    { 0xe132ba16, SUC_lexicon_tags_15 },
    { 0xecf2ba17, SUC_lexicon_tags_37 },
    { 0x3172ba16, SUC_lexicon_tags_29 },
    { 0xc52ba19, SUC_lexicon_tags_47 },
    { 0x2352ba19, SUC_lexicon_tags_17 },
    { 0x8812ba18, SUC_lexicon_tags_13 },
    { 0xb492ba17, SUC_lexicon_tags_35 },
    { 0xec32ba1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4452ba1f, SUC_lexicon_tags_12 },
    { 0x5cb2ba20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e52ba22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7c32ba24, SUC_lexicon_tags_0 },
    { 0x7252ba24, SUC_lexicon_tags_17 },
    { 0xc8d2ba26, SUC_lexicon_tags_11 },
    { 0xe012ba27, SUC_lexicon_tags_12 },
    { 0x2692ba26, SUC_lexicon_tags_14 },
    { 0x33f2ba29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d2ba2d, SUC_lexicon_tags_37 },
    { 0x3cb2ba2e, SUC_lexicon_tags_8 },
    { 0x3512ba2f, SUC_lexicon_tags_4 },
    { 0xb252ba30, SUC_lexicon_tags_19 },
    { 0x6332ba31, SUC_lexicon_tags_23 },
    { 0x1172ba31, SUC_lexicon_tags_17 },
    { 0x30d2ba33, SUC_lexicon_tags_17 },
    { 0x48f2ba31, SUC_lexicon_tags_11 },
    { 0x31d2ba35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2412ba38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ff2ba3a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb32ba3c, SUC_lexicon_tags_40 },
    { 0x2172ba3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf52ba40, SUC_lexicon_tags_31 },
    { 0xf052ba40, SUC_lexicon_tags_14 },
    { 0x6f2ba42, SUC_lexicon_tags_140 },
    { 0x6612ba40, SUC_lexicon_tags_42 },
    { 0x72d2ba42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b2ba48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e12ba50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0d2ba52, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf2ba56, SUC_lexicon_tags_6 },
    { 0x5252ba56, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa112ba5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6552ba63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad12ba6a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x98d2ba6c, SUC_lexicon_tags_47 },
    { 0xab32ba6d, SUC_lexicon_tags_26 },
    { 0xadd2ba6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb2ba73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1812ba75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab2ba7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47b2ba7e, SUC_lexicon_tags_160 },
    { 0xfc72ba7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe752ba86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f2ba8b, SUC_lexicon_tags_25 },
    { 0xc1b2ba8c, SUC_lexicon_tags_15 },
    { 0xed72ba8c, SUC_lexicon_tags_17 },
    { 0xb972ba8d, SUC_lexicon_tags_31 },
    { 0xfbb2ba8f, SUC_lexicon_tags_11 },
    { 0xae72ba90, SUC_lexicon_tags_47 },
    { 0x73f2ba8d, SUC_lexicon_tags_29 },
    { 0x74b2ba92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa812ba95, SUC_lexicon_tags_12 },
    { 0x15d2ba96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f2ba99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2baa1, SUC_lexicon_tags_25 },
    { 0x3f92baa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bd2baa4, SUC_lexicon_tags_37 },
    { 0xfaf2baa5, SUC_lexicon_tags_63 },
    { 0xb532baa6, SUC_lexicon_tags_13 },
    { 0x2ab2baa7, SUC_lexicon_tags_11 },
    { 0x8bf2baa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ff2baaa, SUC_lexicon_tags_9 },
    { 0xc372baab, SUC_lexicon_tags_1 },
    { 0x9932baac, SUC_lexicon_tags_7 },
    { 0xcb52baad, SUC_lexicon_tags_11 },
    { 0xe412baae, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f2bab3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e52bab7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7612baba, SUC_lexicon_tags_14 },
    { 0x3132babb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc172babe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f2bac3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb52bac7, SUC_lexicon_tags_12 },
    { 0x6d2bac8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95b2baca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c52bacc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2ab2bace, SUC_lexicon_tags_47 },
    { 0x46b2bace, SUC_lexicon_tags_17 },
    { 0x9cb2bace, SUC_lexicon_tags_19 },
    { 0x69f2bad1, SUC_lexicon_tags_11 },
    { 0xa852bacf, SUC_lexicon_tags_25 },
    { 0x76d2bad0, SUC_lexicon_tags_1 },
    { 0xfef2bad4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae12badb, SUC_lexicon_tags_65 },
    { 0xc372badc, SUC_lexicon_tags_14 },
    { 0xb632badd, SUC_lexicon_tags_27 },
    { 0x30f2badd, SUC_lexicon_tags_9 },
    { 0x65d2badf, SUC_lexicon_tags_33 },
    { 0x17b2badb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f52bae2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c92bae4, SUC_lexicon_tags_25 },
    { 0x82d2bae4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd032bae9, SUC_lexicon_tags_45 },
    { 0x8492baea, SUC_lexicon_tags_1 },
    { 0x3ff2baeb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3552baf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe732bafa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b32bafc, SUC_lexicon_tags_2 },
    { 0x2bf2bafd, SUC_lexicon_tags_3 },
    { 0x3112bafe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x852bb0b, SUC_lexicon_tags_1 },
    { 0xc6d2bb0b, SUC_lexicon_tags_43 },
    { 0xfeb2bb0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf832bb10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d2bb15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a92bb18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b2bb29, SUC_lexicon_tags_9 },
    { 0x48b2bb2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b2bb31, SUC_lexicon_tags_26 },
    { 0xcd92bb31, SUC_lexicon_tags_10 },
    { 0x3632bb33, SUC_lexicon_tags_65 },
    { 0x1732bb34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff2bb3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72b2bb3d, SUC_lexicon_tags_2 },
    { 0xc4d2bb3e, SUC_lexicon_tags_7 },
    { 0x93b2bb3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12bb45, SUC_lexicon_tags_15 },
    { 0x9072bb45, SUC_lexicon_tags_17 },
    { 0x6892bb46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1d2bb49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x632bb4e, SUC_lexicon_tags_25 },
    { 0x4212bb4e, SUC_lexicon_tags_38 },
    { 0xf212bb4e, SUC_lexicon_tags_25 },
    { 0x9752bb51, SUC_lexicon_tags_9 },
    { 0x46b2bb52, SUC_lexicon_tags_15 },
    { 0x4512bb51, SUC_lexicon_tags_37 },
    { 0x75d2bb54, SUC_lexicon_tags_39 },
    { 0xca12bb55, SUC_lexicon_tags_7 },
    { 0x13f2bb55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb72bb5d, SUC_lexicon_tags_0 },
    { 0x2452bb5e, SUC_lexicon_tags_64 },
    { 0xa172bb5f, SUC_lexicon_tags_46 },
    { 0x63b2bb60, SUC_lexicon_tags_29 },
    { 0x1c32bb5e, SUC_lexicon_tags_2 },
    { 0x8192bb5d, SUC_lexicon_tags_25 },
    { 0x2d92bb61, SUC_lexicon_tags_11 },
    { 0x4652bb61, SUC_lexicon_tags_25 },
    { 0x6552bb65, SUC_lexicon_tags_23 },
    { 0x632bb5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf812bb68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb252bb6a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf832bb6c, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0x27f2bb6e, SUC_lexicon_tags_30 },
    { 0x1512bb6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d2bb76, SUC_lexicon_tags_12 },
    { 0xb32bb77, SUC_lexicon_tags_49 },
    { 0x7ef2bb77, SUC_lexicon_tags_17 },
    { 0x1372bb78, SUC_lexicon_tags_19 },
    { 0xdc72bb76, SUC_lexicon_tags_37 },
    { 0xe8b2bb78, SUC_lexicon_tags_87 },
    { 0x612bb7c, SUC_lexicon_tags_49 },
    { 0xe3b2bb7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd2bb80, SUC_lexicon_tags_56 },
    { 0xad52bb80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2752bb83, SUC_lexicon_tags_25 },
    { 0x4832bb84, SUC_lexicon_tags_13 },
    { 0x8d12bb83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bb9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8452bba3, SUC_lexicon_tags_15 },
    { 0xd612bba3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2792bba7, SUC_lexicon_tags_44 },
    { 0xcb72bba8, SUC_lexicon_tags_37 },
    { 0x8d52bba9, SUC_lexicon_tags_60 },
    { 0xfe32bba8, SUC_lexicon_tags_17 },
    { 0x4af2bbaa, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x1b32bbad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6692bbaf, SUC_lexicon_tags_13 },
    { 0x2b12bbaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2bbb4, SUC_lexicon_tags_37 },
    { 0x8052bbb5, SUC_lexicon_tags_17 },
    { 0x8712bbb6, SUC_lexicon_tags_43 },
    { 0xd8d2bbb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5f52bbb9, SUC_lexicon_tags_0 },
    { 0xa5b2bbba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9552bbbf, SUC_lexicon_tags_138 },
    { 0x9672bbc0, SUC_lexicon_tags_11 },
    { 0x5232bbc1, SUC_lexicon_tags_7 },
    { 0x7112bbc2, SUC_lexicon_tags_63 },
    { 0x24f2bbc1, SUC_lexicon_tags_15 },
    { 0xf972bbc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d2bbcb, SUC_lexicon_tags_28 },
    { 0x1572bbcc, SUC_lexicon_tags_25 },
    { 0x7872bbcd, SUC_lexicon_tags_8 },
    { 0xf432bbce, SUC_lexicon_tags_5 },
    { 0xbd2bbcf, SUC_lexicon_tags_13 },
    { 0xf72bbd0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x7632bbd3, SUC_lexicon_tags_37 },
    { 0xc472bbd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73d2bbd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe132bbdc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc52bbdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7972bbe2, SUC_lexicon_tags_23 },
    { 0x5f32bbe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56f2bbe5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa92bbe7, SUC_lexicon_tags_60 },
    { 0x8ef2bbe8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b2bbeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad2bbee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5732bbf2, SUC_lexicon_tags_33 },
    { 0xf172bbf2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3132bbf8, SUC_lexicon_tags_37 },
    { 0x9892bbf8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb2bbfc, SUC_lexicon_tags_2 },
    { 0x2ab2bbfc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8872bc01, SUC_lexicon_tags_47 },
    { 0xf12bc02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a72bc05, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa6f2bc07, SUC_lexicon_tags_4 },
    { 0x6292bc07, SUC_lexicon_tags_12 },
    { 0xacb2bc09, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3e32bc0b, SUC_lexicon_tags_7 },
    { 0x1a52bc0c, SUC_lexicon_tags_44 },
    { 0x3512bc0d, SUC_lexicon_tags_8 },
    { 0xa672bc0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb692bc12, SUC_lexicon_tags_23 },
    { 0xb652bc12, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xde12bc15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd2bc18, SUC_lexicon_tags_60 },
    { 0x59d2bc19, SUC_lexicon_tags_31 },
    { 0xc532bc19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1792bc1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb12bc21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f2bc27, SUC_lexicon_tags_17 },
    { 0x2d32bc28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a12bc2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6f2bc2e, SUC_lexicon_tags_11 },
    { 0xc3d2bc2f, SUC_lexicon_tags_7 },
    { 0x7232bc30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8b2bc32, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b2bc38, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x71f2bc3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf32bc3e, SUC_lexicon_tags_69 },
    { 0x6892bc3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc272bc47, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d32bc4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba32bc4c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb572bc4f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd2bc52, SUC_lexicon_tags_11 },
    { 0xff32bc53, SUC_lexicon_tags_12 },
    { 0xd672bc54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d2bc57, SUC_lexicon_tags_33 },
    { 0x952bc57, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x572bc5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3bb2bc5d, SUC_lexicon_tags_9 },
    { 0x7f92bc5d, SUC_lexicon_tags_30 },
    { 0xe012bc5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb2bc64, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd092bc6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf72bc74, SUC_lexicon_tags_29 },
    { 0x11b2bc75, SUC_lexicon_tags_11 },
    { 0x6332bc76, SUC_lexicon_tags_8 },
    { 0x70f2bc77, SUC_lexicon_tags_41 },
    { 0xf952bc78, SUC_lexicon_tags_65 },
    { 0x2232bc78, SUC_lexicon_tags_59 },
    { 0x452bc7a, SUC_lexicon_tags_7 },
    { 0xaf92bc7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2bc7e, SUC_lexicon_tags_57 },
    { 0x4232bc7f, SUC_lexicon_tags_47 },
    { 0xd812bc7e, SUC_lexicon_tags_17 },
    { 0xe9f2bc81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cf2bc83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d2bc87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d12bc89, SUC_lexicon_tags_7 },
    { 0xf972bc89, SUC_lexicon_tags_25 },
    { 0x45f2bc8b, SUC_lexicon_tags_7 },
    { 0xe272bc89, SUC_lexicon_tags_12 },
    { 0x26b2bc8b, SUC_lexicon_tags_37 },
    { 0x2192bc8e, SUC_lexicon_tags_47 },
    { 0xd612bc8f, SUC_lexicon_tags_2 },
    { 0xd452bc8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f2bc94, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e92bc97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9352bc9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d12bca2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2132bcac, SUC_lexicon_tags_47 },
    { 0x28b2bcad, SUC_lexicon_tags_18 },
    { 0x552bcae, SUC_lexicon_tags_27 },
    { 0x3c12bcaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b2bcb1, SUC_lexicon_tags_11 },
    { 0xc472bcb2, SUC_lexicon_tags_33 },
    { 0x4952bcb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8b2bcb5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb2bcc0, SUC_lexicon_tags_25 },
    { 0xa6d2bcc1, SUC_lexicon_tags_26 },
    { 0xd32bcc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f2bcc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd972bcca, SUC_lexicon_tags_28 },
    { 0x6ab2bcca, SUC_lexicon_tags_14 },
    { 0x5e72bccb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfe12bcce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc432bcd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e32bcd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72bcdc, SUC_lexicon_tags_37 },
    { 0xa12bcdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7d2bcdf, SUC_lexicon_tags_37 },
    { 0x2db2bce0, SUC_lexicon_tags_39 },
    { 0xb5d2bce1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd672bce6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb052bcea, SUC_lexicon_tags_60 },
    { 0x5412bceb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33b2bced, SUC_lexicon_tags_46 },
    { 0x33b2bcee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1292bcf2, SUC_lexicon_tags_17 },
    { 0x3252bcf3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8592bcf7, SUC_lexicon_tags_15 },
    { 0x5832bcf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53d2bcfa, SUC_lexicon_tags_27 },
    { 0x172bcfa, SUC_lexicon_tags_15 },
    { 0x4772bcfc, SUC_lexicon_tags_8 },
    { 0x3432bcfc, SUC_lexicon_tags_10 },
    { 0x37f2bcfc, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x3012bd00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc632bd03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4452bd0c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7e72bd0e, SUC_lexicon_tags_37 },
    { 0xed32bd0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb272bd16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6052bd18, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa952bd1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612bd23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b2bd27, SUC_lexicon_tags_4 },
    { 0xd292bd28, SUC_lexicon_tags_15 },
    { 0x6e32bd28, SUC_lexicon_tags_12 },
    { 0x3c52bd28, SUC_lexicon_tags_37 },
    { 0x87f2bd2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6512bd2e, SUC_lexicon_tags_15 },
    { 0xf052bd2f, SUC_lexicon_tags_12 },
    { 0x67f2bd30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9b2bd32, SUC_lexicon_tags_15 },
    { 0x6012bd33, SUC_lexicon_tags_17 },
    { 0x33d2bd33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e52bd38, SUC_lexicon_tags_22 },
    { 0x20b2bd38, SUC_lexicon_tags_23 },
    { 0xe072bd3a, SUC_lexicon_tags_11 },
    { 0x9112bd39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d52bd3e, SUC_lexicon_tags_15 },
    { 0x872bd3f, SUC_lexicon_tags_12 },
    { 0xb232bd3f, SUC_lexicon_tags_48 },
    { 0x352bd3f, SUC_lexicon_tags_0 },
    { 0x8532bd42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcab2bd44, SUC_lexicon_tags_28 },
    { 0x79b2bd45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1532bd4e, SUC_lexicon_tags_25 },
    { 0xd252bd4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f2bd54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32d2bd56, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9092bd58, SUC_lexicon_tags_23 },
    { 0x2fd2bd59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb2bd5d, SUC_lexicon_tags_12 },
    { 0x3df2bd5d, SUC_lexicon_tags_1 },
    { 0x1912bd5d, SUC_lexicon_tags_257 },
    { 0xc952bd5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2212bd69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa972bd6c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f2bd71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc192bd73, SUC_lexicon_tags_17 },
    { 0x2c32bd74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1852bd77, SUC_lexicon_tags_19 },
    { 0x6f92bd78, SUC_lexicon_tags_7 },
    { 0x7c72bd79, SUC_lexicon_tags_17 },
    { 0xc632bd7a, SUC_lexicon_tags_17 },
    { 0x8872bd7b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6372bd80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6512bd82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf2bd85, SUC_lexicon_tags_25 },
    { 0xdf92bd85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3452bd8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1472bd8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa892bd8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6432bd92, SUC_lexicon_tags_25 },
    { 0x64f2bd93, SUC_lexicon_tags_14 },
    { 0x96b2bd93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd72bd96, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xe512bd99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25d2bd9b, SUC_lexicon_tags_1 },
    { 0x55b2bd9b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a72bd9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xea12bda0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2bda3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b2bda9, SUC_lexicon_tags_30 },
    { 0x13d2bdaa, SUC_lexicon_tags_4 },
    { 0x69f2bdab, SUC_lexicon_tags_15 },
    { 0x5132bdaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa72bdae, SUC_lexicon_tags_23 },
    { 0x55b2bdae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa992bdb4, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xf72bdb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90d2bdb8, SUC_lexicon_tags_49 },
    { 0x5452bdb8, SUC_lexicon_tags_57 },
    { 0x8992bdba, SUC_lexicon_tags_17 },
    { 0xa392bdba, SUC_lexicon_tags_62 },
    { 0x3692bdbc, SUC_lexicon_tags_17 },
    { 0x772bdbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b2bdc2, SUC_lexicon_tags_32 },
    { 0x3772bdc3, SUC_lexicon_tags_47 },
    { 0x4b12bdc4, SUC_lexicon_tags_11 },
    { 0xf0f2bdc4, SUC_lexicon_tags_11 },
    { 0xe6d2bdc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce72bdc8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x70f2bdca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc072bdce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6992bdd0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x472bdd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x80d2bdd4, SUC_lexicon_tags_6 },
    { 0xd712bdd4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a92bdd8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f92bddd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad2bde2, SUC_lexicon_tags_11 },
    { 0x8052bde3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2eb2bde5, SUC_lexicon_tags_26 },
    { 0xeff2bde6, SUC_lexicon_tags_59 },
    { 0x30b2bde7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf52bdeb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad12bdf2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf72bdf6, SUC_lexicon_tags_450 },
    { 0x4312bdf7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf2bdfd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe732bdff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9372be02, SUC_lexicon_tags_25 },
    { 0xb232be03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa712be05, SUC_lexicon_tags_37 },
    { 0xbef2be06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d2be0b, SUC_lexicon_tags_82 },
    { 0x2e52be0c, SUC_lexicon_tags_8 },
    { 0x6852be0d, SUC_lexicon_tags_47 },
    { 0x3cb2be0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d92be10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9732be14, SUC_lexicon_tags_39 },
    { 0xa5d2be15, SUC_lexicon_tags_11 },
    { 0x16b2be16, SUC_lexicon_tags_33 },
    { 0xd472be16, SUC_lexicon_tags_12 },
    { 0x2532be17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8b2be1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c32be22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2372be24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b2be29, SUC_lexicon_tags_12 },
    { 0xb4d2be2a, SUC_lexicon_tags_1 },
    { 0xcdd2be29, SUC_lexicon_tags_28 },
    { 0x2692be2c, SUC_lexicon_tags_12 },
    { 0xa0f2be29, SUC_lexicon_tags_87 },
    { 0x72d2be2e, SUC_lexicon_tags_29 },
    { 0xd1b2be2d, SUC_lexicon_tags_60 },
    { 0xd5b2be30, SUC_lexicon_tags_11 },
    { 0x51f2be2e, SUC_lexicon_tags_12 },
    { 0x2ed2be32, SUC_lexicon_tags_17 },
    { 0x95f2be32, SUC_lexicon_tags_38 },
    { 0x1832be30, SUC_lexicon_tags_25 },
    { 0x6192be33, SUC_lexicon_tags_63 },
    { 0xfe32be34, SUC_lexicon_tags_11 },
    { 0xc632be34, SUC_lexicon_tags_25 },
    { 0xff72be35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbc72be3a, SUC_lexicon_tags_8 },
    { 0x90d2be3b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f2be3f, SUC_lexicon_tags_37 },
    { 0xb4f2be40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc952be45, SUC_lexicon_tags_43 },
    { 0x46f2be46, SUC_lexicon_tags_238 },
    { 0x61d2be45, SUC_lexicon_tags_17 },
    { 0xcd72be48, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8bd2be4a, SUC_lexicon_tags_37 },
    { 0x92b2be4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7672be4d, SUC_lexicon_tags_2 },
    { 0x1ad2be4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b2be52, SUC_lexicon_tags_11 },
    { 0xd032be52, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8772be55, SUC_lexicon_tags_66 },
    { 0xffb2be56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7992be58, SUC_lexicon_tags_15 },
    { 0x4af2be58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe072be5c, SUC_lexicon_tags_19 },
    { 0x9392be5d, SUC_lexicon_tags_15 },
    { 0xda32be5d, SUC_lexicon_tags_7 },
    { 0x1492be5e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f2be64, SUC_lexicon_tags_31 },
    { 0xc9d2be64, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd32be67, SUC_lexicon_tags_0 },
    { 0x7a72be68, SUC_lexicon_tags_4 },
    { 0xa7d2be69, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xec92be6c, SUC_lexicon_tags_26 },
    { 0x7eb2be6d, SUC_lexicon_tags_37 },
    { 0xfe52be6e, SUC_lexicon_tags_17 },
    { 0xf752be6e, SUC_lexicon_tags_38 },
    { 0xf212be6e, SUC_lexicon_tags_33 },
    { 0x4492be6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd52be73, SUC_lexicon_tags_25 },
    { 0x2df2be73, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x29b2be76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3772be7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b2be7f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1592be81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d2be83, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5432be87, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132be8b, SUC_lexicon_tags_1 },
    { 0x28d2be8c, SUC_lexicon_tags_7 },
    { 0x92d2be8b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9392be8f, SUC_lexicon_tags_45 },
    { 0x8712be90, SUC_lexicon_tags_2 },
    { 0xcc52be91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa932be96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c72be9d, SUC_lexicon_tags_176 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa552bea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd2bea4, SUC_lexicon_tags_19 },
    { 0xb72bea5, SUC_lexicon_tags_10 },
    { 0xba52bea6, SUC_lexicon_tags_25 },
    { 0x1152bea4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff2beac, SUC_lexicon_tags_38 },
    { 0x82f2bead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8872beb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b2beb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb52bec9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e72becd, SUC_lexicon_tags_43 },
    { 0xf652becd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf2bed2, SUC_lexicon_tags_14 },
    { 0xf032bed2, SUC_lexicon_tags_9 },
    { 0x972bed4, SUC_lexicon_tags_37 },
    { 0xa492bed5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2f12bed7, SUC_lexicon_tags_14 },
    { 0x8332bed8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a52beda, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa5d2bedc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b2bee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab92bee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71f2beea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d92beec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb2bef4, SUC_lexicon_tags_21 },
    { 0xc9f2bef5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9912bef7, SUC_lexicon_tags_37 },
    { 0x64d2bef7, SUC_lexicon_tags_17 },
    { 0x2ad2bef7, SUC_lexicon_tags_112 },
    { 0x65b2bef7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb292befc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc212bf00, SUC_lexicon_tags_37 },
    { 0x2af2bf01, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa732bf05, SUC_lexicon_tags_12 },
    { 0x2392bf06, SUC_lexicon_tags_11 },
    { 0xc312bf07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b2bf0c, SUC_lexicon_tags_14 },
    { 0xd612bf0d, SUC_lexicon_tags_37 },
    { 0xf9d2bf0c, SUC_lexicon_tags_60 },
    { 0xda92bf0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3412bf12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1252bf14, SUC_lexicon_tags_12 },
    { 0xb1b2bf15, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd452bf1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5632bf1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d2bf29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf2bf2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x73d2bf30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc532bf32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bf35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9632bf37, SUC_lexicon_tags_7 },
    { 0x3092bf38, SUC_lexicon_tags_32 },
    { 0xb352bf39, SUC_lexicon_tags_10 },
    { 0xc4f2bf3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c92bf3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e72bf40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf412bf43, SUC_lexicon_tags_17 },
    { 0xee12bf44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d2bf4b, SUC_lexicon_tags_6 },
    { 0x2c52bf4c, SUC_lexicon_tags_12 },
    { 0xbd92bf4d, SUC_lexicon_tags_47 },
    { 0x6f72bf4c, SUC_lexicon_tags_8 },
    { 0x9d2bf4f, SUC_lexicon_tags_29 },
    { 0x5a12bf4f, SUC_lexicon_tags_26 },
    { 0xca72bf50, SUC_lexicon_tags_34 },
    { 0x3372bf52, SUC_lexicon_tags_47 },
    { 0x6072bf51, SUC_lexicon_tags_45 },
    { 0x1eb2bf4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8412bf59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf452bf5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3012bf65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd552bf6a, SUC_lexicon_tags_43 },
    { 0x6ad2bf6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b72bf70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb92bf73, SUC_lexicon_tags_0 },
    { 0x90d2bf73, SUC_lexicon_tags_12 },
    { 0x8e12bf75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d2bf7a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x82b2bf7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab2bf80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7eb2bf82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc852bf8b, SUC_lexicon_tags_17 },
    { 0xb9b2bf8c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6132bf8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa32bf93, SUC_lexicon_tags_7 },
    { 0xedd2bf93, SUC_lexicon_tags_26 },
    { 0x28b2bf93, SUC_lexicon_tags_60 },
    { 0x3db2bf93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6d2bf98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8932bf9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6072bfa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa732bfa8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb652bfaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9132bfad, SUC_lexicon_tags_16 },
    { 0x6392bfae, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xc172bfb0, SUC_lexicon_tags_2 },
    { 0xc752bfb1, SUC_lexicon_tags_103 },
    { 0x1312bfb0, SUC_lexicon_tags_2 },
    { 0xb7f2bfb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc632bfb5, SUC_lexicon_tags_38 },
    { 0x6492bfb5, SUC_lexicon_tags_30 },
    { 0xbe92bfb7, SUC_lexicon_tags_33 },
    { 0x33f2bfb8, SUC_lexicon_tags_15 },
    { 0xb492bfb8, SUC_lexicon_tags_9 },
    { 0x45b2bfb5, SUC_lexicon_tags_69 },
    { 0xe352bfbb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d92bfc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b12bfc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d32bfc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2792bfcc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc2f2bfce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa072bfd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef2bfd7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd2bfda, SUC_lexicon_tags_11 },
    { 0x772bfdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8872bfde, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb2b2bfe0, SUC_lexicon_tags_15 },
    { 0x1192bfe1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f72bfe4, SUC_lexicon_tags_0 },
    { 0x4932bfe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d12bfeb, SUC_lexicon_tags_27 },
    { 0xf4f2bfeb, SUC_lexicon_tags_60 },
    { 0xe172bfed, SUC_lexicon_tags_6 },
    { 0x5312bfeb, SUC_lexicon_tags_43 },
    { 0xa4f2bfef, SUC_lexicon_tags_43 },
    { 0xa4d2bfef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab2bff9, SUC_lexicon_tags_11 },
    { 0x432bff9, SUC_lexicon_tags_49 },
    { 0xfff2bff9, SUC_lexicon_tags_1 },
    { 0x5e32bffa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc412bffe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a72c001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f2c004, SUC_lexicon_tags_60 },
    { 0x1632c005, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b32c009, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf72c00e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc672c010, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb2c014, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3112c016, SUC_lexicon_tags_26 },
    { 0x2a92c017, SUC_lexicon_tags_87 },
    { 0xa6d2c018, SUC_lexicon_tags_18 },
    { 0x5592c017, SUC_lexicon_tags_30 },
    { 0x4a72c019, SUC_lexicon_tags_57 },
    { 0x9692c017, SUC_lexicon_tags_17 },
    { 0x6d2c018, SUC_lexicon_tags_69 },
    { 0xb352c018, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb972c022, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6552c024, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d92c026, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c12c02c, SUC_lexicon_tags_25 },
    { 0x1292c02c, SUC_lexicon_tags_7 },
    { 0xa2f2c02e, SUC_lexicon_tags_15 },
    { 0x1052c02d, SUC_lexicon_tags_17 },
    { 0x7592c02f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe472c03b, SUC_lexicon_tags_19 },
    { 0xc592c03c, SUC_lexicon_tags_37 },
    { 0x1cb2c03d, SUC_lexicon_tags_60 },
    { 0xb8d2c03e, SUC_lexicon_tags_10 },
    { 0xd4f2c03b, SUC_lexicon_tags_11 },
    { 0x2bd2c040, SUC_lexicon_tags_11 },
    { 0xc732c040, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3632c046, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2972c048, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6212c04b, SUC_lexicon_tags_33 },
    { 0x5d92c04c, SUC_lexicon_tags_25 },
    { 0x6a72c04d, SUC_lexicon_tags_25 },
    { 0xcf2c04c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9012c050, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb212c058, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d2c05d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf952c05f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e92c063, SUC_lexicon_tags_4 },
    { 0x2b92c063, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc692c068, SUC_lexicon_tags_65 },
    { 0x3e12c068, SUC_lexicon_tags_49 },
    { 0x22f2c069, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f2c06e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9112c070, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x90f2c072, SUC_lexicon_tags_6 },
    { 0x5972c073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf212c076, SUC_lexicon_tags_25 },
    { 0xb632c076, SUC_lexicon_tags_49 },
    { 0xb0b2c076, SUC_lexicon_tags_2 },
    { 0x7af2c076, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f2c080, SUC_lexicon_tags_25 },
    { 0x1032c080, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb72c083, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f2c088, SUC_lexicon_tags_7 },
    { 0xff72c089, SUC_lexicon_tags_15 },
    { 0x4132c089, SUC_lexicon_tags_37 },
    { 0x9272c08b, SUC_lexicon_tags_37 },
    { 0x4bd2c08c, SUC_lexicon_tags_18 },
    { 0x1a12c08b, SUC_lexicon_tags_11 },
    { 0xbb72c08e, SUC_lexicon_tags_80 },
    { 0xd6f2c08f, SUC_lexicon_tags_11 },
    { 0x9bf2c090, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f2c093, SUC_lexicon_tags_7 },
    { 0x4452c094, SUC_lexicon_tags_28 },
    { 0xbe12c095, SUC_lexicon_tags_25 },
    { 0xd792c096, SUC_lexicon_tags_25 },
    { 0xf112c097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb2c09b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd292c09f, SUC_lexicon_tags_218 },
    { 0x9532c09f, SUC_lexicon_tags_7 },
    { 0x5892c0a1, SUC_lexicon_tags_54 },
    { 0xbb2c0a1, SUC_lexicon_tags_17 },
    { 0x20d2c0a1, SUC_lexicon_tags_7 },
    { 0x85d2c0a0, SUC_lexicon_tags_28 },
    { 0xf7d2c0a5, SUC_lexicon_tags_13 },
    { 0xf9b2c0a5, SUC_lexicon_tags_69 },
    { 0x7632c0a0, SUC_lexicon_tags_15 },
    { 0x9312c0a8, SUC_lexicon_tags_37 },
    { 0xbbb2c0a9, SUC_lexicon_tags_91 },
    { 0xb852c0a8, SUC_lexicon_tags_66 },
    { 0x1112c0a2, SUC_lexicon_tags_37 },
    { 0xd632c0ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a92c0ae, SUC_lexicon_tags_17 },
    { 0x3072c0af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe152c0b4, SUC_lexicon_tags_17 },
    { 0x9532c0b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b2c0b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe392c0ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d2c0bf, SUC_lexicon_tags_43 },
    { 0xb1d2c0c0, SUC_lexicon_tags_25 },
    { 0xc52c0c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed2c0c7, SUC_lexicon_tags_1 },
    { 0xf112c0c8, SUC_lexicon_tags_65 },
    { 0x6b32c0c9, SUC_lexicon_tags_17 },
    { 0x9e12c0c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4692c0cd, SUC_lexicon_tags_15 },
    { 0xd332c0cd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f2c0d2, SUC_lexicon_tags_23 },
    { 0x9af2c0d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c32c0d6, SUC_lexicon_tags_48 },
    { 0xe92c0d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd2c0d9, SUC_lexicon_tags_2 },
    { 0x6bf2c0da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5192c0dd, SUC_lexicon_tags_1 },
    { 0xf7d2c0de, SUC_lexicon_tags_25 },
    { 0x29f2c0df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf0b2c0e1, SUC_lexicon_tags_12 },
    { 0x7172c0e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8272c0e5, SUC_lexicon_tags_49 },
    { 0xf7f2c0e5, SUC_lexicon_tags_37 },
    { 0xba32c0e6, SUC_lexicon_tags_65 },
    { 0x6a92c0e8, SUC_lexicon_tags_17 },
    { 0x3412c0e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38f2c0eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c72c0ed, SUC_lexicon_tags_170 },
    { 0x1712c0ee, SUC_lexicon_tags_7 },
    { 0x3af2c0ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdcb2c0f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1912c0f3, SUC_lexicon_tags_25 },
    { 0xbb12c0f4, SUC_lexicon_tags_11 },
    { 0x852c0f4, SUC_lexicon_tags_26 },
    { 0xcc32c0f6, SUC_lexicon_tags_79 },
    { 0x2512c0f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a32c0fa, SUC_lexicon_tags_17 },
    { 0xc92c0fa, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xa632c0fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b2c101, SUC_lexicon_tags_29 },
    { 0x22f2c102, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f2c108, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5fd2c10a, SUC_lexicon_tags_19 },
    { 0xf932c10a, SUC_lexicon_tags_37 },
    { 0x6292c10c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7052c10f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd12c116, SUC_lexicon_tags_25 },
    { 0x5d52c117, SUC_lexicon_tags_60 },
    { 0x1692c118, SUC_lexicon_tags_28 },
    { 0x4bf2c117, SUC_lexicon_tags_23 },
    { 0xa512c119, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xed12c11c, SUC_lexicon_tags_11 },
    { 0xff92c11c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99d2c11f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b2c124, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf692c126, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d52c128, SUC_lexicon_tags_27 },
    { 0xd712c129, SUC_lexicon_tags_27 },
    { 0xc5d2c12a, SUC_lexicon_tags_12 },
    { 0xf392c12b, SUC_lexicon_tags_19 },
    { 0xcad2c129, SUC_lexicon_tags_11 },
    { 0x9b2c12d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e32c132, SUC_lexicon_tags_2 },
    { 0x7f52c133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc12c136, SUC_lexicon_tags_14 },
    { 0x4512c136, SUC_lexicon_tags_37 },
    { 0x1572c138, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4c72c13a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9172c13c, SUC_lexicon_tags_19 },
    { 0x9e32c13c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe72c140, SUC_lexicon_tags_34 },
    { 0x1eb2c141, SUC_lexicon_tags_11 },
    { 0x5532c141, SUC_lexicon_tags_87 },
    { 0x6032c143, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xd252c145, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd2c149, SUC_lexicon_tags_47 },
    { 0x9972c149, SUC_lexicon_tags_7 },
    { 0xeb2c14a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b52c14e, SUC_lexicon_tags_71 },
    { 0x1eb2c14f, SUC_lexicon_tags_17 },
    { 0x4852c14e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5392c153, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f2c160, SUC_lexicon_tags_4 },
    { 0x3332c161, SUC_lexicon_tags_7 },
    { 0x93d2c161, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c92c166, SUC_lexicon_tags_38 },
    { 0xd612c167, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x72b2c169, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf2c170, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc992c173, SUC_lexicon_tags_7 },
    { 0xb792c173, SUC_lexicon_tags_1 },
    { 0x1172c173, SUC_lexicon_tags_2 },
    { 0x4b72c174, SUC_lexicon_tags_60 },
    { 0x7572c177, SUC_lexicon_tags_23 },
    { 0xeed2c174, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd2c17c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8b2c17e, SUC_lexicon_tags_342 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d12c181, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4fb2c183, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac92c185, SUC_lexicon_tags_27 },
    { 0xf912c186, SUC_lexicon_tags_1155 },
    { 0x5b72c187, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf2c18a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb872c193, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf12c197, SUC_lexicon_tags_0 },
    { 0x2052c198, SUC_lexicon_tags_15 },
    { 0x8992c198, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa752c19c, SUC_lexicon_tags_1 },
    { 0x7b52c19d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf2c1a1, SUC_lexicon_tags_11 },
    { 0xa112c1a1, SUC_lexicon_tags_26 },
    { 0x6512c1a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe532c1a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a32c1ad, SUC_lexicon_tags_17 },
    { 0x3632c1ad, SUC_lexicon_tags_12 },
    { 0x9c32c1ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b2c1b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5632c1b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e92c1ba, SUC_lexicon_tags_26 },
    { 0x3dd2c1bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e12c1be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2352c1c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa792c1c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad2c1d0, SUC_lexicon_tags_48 },
    { 0x2b2c1d1, SUC_lexicon_tags_8 },
    { 0xd32c1d1, SUC_lexicon_tags_33 },
    { 0x772c1d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea12c1d6, SUC_lexicon_tags_37 },
    { 0x3312c1d6, SUC_lexicon_tags_1 },
    { 0xd672c1d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcff2c1da, SUC_lexicon_tags_10 },
    { 0x35f2c1db, SUC_lexicon_tags_26 },
    { 0x1a12c1da, SUC_lexicon_tags_37 },
    { 0x112c1dd, SUC_lexicon_tags_15 },
    { 0xaf92c1dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda12c1e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca92c1e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e92c1ec, SUC_lexicon_tags_12 },
    { 0xdb32c1ed, SUC_lexicon_tags_14 },
    { 0xcf52c1ee, SUC_lexicon_tags_37 },
    { 0x67b2c1ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f72c1f8, SUC_lexicon_tags_37 },
    { 0x6592c1f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf372c1fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d92c1fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef72c201, SUC_lexicon_tags_15 },
    { 0xe2b2c202, SUC_lexicon_tags_15 },
    { 0x1c52c201, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9572c209, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x652c20c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b2c217, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d2c21b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9012c21d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4852c220, SUC_lexicon_tags_49 },
    { 0x52b2c221, SUC_lexicon_tags_37 },
    { 0x4e12c220, SUC_lexicon_tags_30 },
    { 0xe952c221, SUC_lexicon_tags_16 },
    { 0x5ad2c223, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37b2c226, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d2c229, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b72c22d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e52c22f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb92c232, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc212c236, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e92c238, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2312c23b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda72c242, SUC_lexicon_tags_17 },
    { 0x8912c243, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2c247, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f2c24e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2592c250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f2c255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b2c258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab92c25b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc92c25d, SUC_lexicon_tags_11 },
    { 0x83b2c25d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b2c263, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf792c267, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f2c26a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f2c26e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3972c271, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5532c274, SUC_lexicon_tags_87 },
    { 0x65b2c274, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb592c27b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb52c282, SUC_lexicon_tags_45 },
    { 0x3c92c282, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e32c288, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5092c290, SUC_lexicon_tags_6 },
    { 0xbfb2c290, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f2c298, SUC_lexicon_tags_37 },
    { 0x7432c299, SUC_lexicon_tags_37 },
    { 0x8b52c29a, SUC_lexicon_tags_55 },
    { 0xe9b2c298, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x8252c29e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8152c2a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x452c2a2, SUC_lexicon_tags_8 },
    { 0x43f2c2a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9672c2a5, SUC_lexicon_tags_12 },
    { 0x2a72c2a5, SUC_lexicon_tags_60 },
    { 0x3892c2a7, SUC_lexicon_tags_12 },
    { 0xb2d2c2a7, SUC_lexicon_tags_26 },
    { 0x4d2c2a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb12c2ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda72c2ae, SUC_lexicon_tags_16 },
    { 0x3872c2ae, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0x79d2c2b1, SUC_lexicon_tags_151 },
    { 0x9512c2b1, SUC_lexicon_tags_45 },
    { 0x5812c2b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc692c2b6, SUC_lexicon_tags_7 },
    { 0x18d2c2b7, SUC_lexicon_tags_43 },
    { 0xc5f2c2b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df2c2bf, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc812c2c3, SUC_lexicon_tags_47 },
    { 0x69d2c2c4, SUC_lexicon_tags_47 },
    { 0xa932c2c5, SUC_lexicon_tags_26 },
    { 0x6d32c2c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f2c2c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e32c2cb, SUC_lexicon_tags_12 },
    { 0xa2f2c2cc, SUC_lexicon_tags_19 },
    { 0x8612c2cd, SUC_lexicon_tags_117 },
    { 0xd152c2ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e72c2d0, SUC_lexicon_tags_1 },
    { 0x22b2c2d1, SUC_lexicon_tags_87 },
    { 0x75f2c2d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f72c2d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f2c2d7, SUC_lexicon_tags_45 },
    { 0x4d32c2d8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f2c2db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1d2c2dd, SUC_lexicon_tags_14 },
    { 0x9752c2dd, SUC_lexicon_tags_6 },
    { 0xd692c2dd, SUC_lexicon_tags_17 },
    { 0xe3d2c2e0, SUC_lexicon_tags_45 },
    { 0xac52c2e0, SUC_lexicon_tags_25 },
    { 0xe712c2de, SUC_lexicon_tags_47 },
    { 0x7d92c2e3, SUC_lexicon_tags_32 },
    { 0xf292c2de, SUC_lexicon_tags_66 },
    { 0x5eb2c2e5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e12c2e8, SUC_lexicon_tags_69 },
    { 0x32f2c2e8, SUC_lexicon_tags_11 },
    { 0xa012c2e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3052c2f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5192c2f5, SUC_lexicon_tags_37 },
    { 0x72c2f5, SUC_lexicon_tags_8 },
    { 0x81b2c2f7, SUC_lexicon_tags_17 },
    { 0x5bf2c2f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c92c2fc, SUC_lexicon_tags_17 },
    { 0x24f2c2fd, SUC_lexicon_tags_43 },
    { 0xa572c2fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc892c303, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f2c30c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee72c30e, SUC_lexicon_tags_0 },
    { 0x772c30f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d2c313, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa392c316, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b2c31b, SUC_lexicon_tags_34 },
    { 0xaef2c31c, SUC_lexicon_tags_15 },
    { 0xbb12c31d, SUC_lexicon_tags_65 },
    { 0xb92c31e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd472c320, SUC_lexicon_tags_11 },
    { 0x8992c321, SUC_lexicon_tags_17 },
    { 0x4612c321, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x5512c324, SUC_lexicon_tags_27 },
    { 0xb712c325, SUC_lexicon_tags_11 },
    { 0x6112c325, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xd9b2c328, SUC_lexicon_tags_37 },
    { 0x1cd2c329, SUC_lexicon_tags_15 },
    { 0x1d32c32a, SUC_lexicon_tags_12 },
    { 0x6bd2c32b, SUC_lexicon_tags_23 },
    { 0x8192c329, SUC_lexicon_tags_84 },
    { 0x1072c32d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b2c331, SUC_lexicon_tags_65 },
    { 0x7912c332, SUC_lexicon_tags_25 },
    { 0x252c333, SUC_lexicon_tags_2 },
    { 0x3112c333, SUC_lexicon_tags_15 },
    { 0x6bb2c335, SUC_lexicon_tags_28 },
    { 0x2cb2c334, SUC_lexicon_tags_37 },
    { 0x2832c337, SUC_lexicon_tags_26 },
    { 0x4772c338, SUC_lexicon_tags_11 },
    { 0x3132c337, SUC_lexicon_tags_37 },
    { 0x4432c335, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7432c33d, SUC_lexicon_tags_252 },
    { 0xdcb2c33d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f2c341, SUC_lexicon_tags_91 },
    { 0x37f2c341, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9052c344, SUC_lexicon_tags_30 },
    { 0xd792c345, SUC_lexicon_tags_45 },
    { 0x22b2c345, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e72c34c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b52c350, SUC_lexicon_tags_77 },
    { 0xecb2c350, SUC_lexicon_tags_1 },
    { 0xe7f2c352, SUC_lexicon_tags_38 },
    { 0x27b2c350, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb2c356, SUC_lexicon_tags_37 },
    { 0x8792c356, SUC_lexicon_tags_7 },
    { 0x2292c358, SUC_lexicon_tags_14 },
    { 0x56b2c359, SUC_lexicon_tags_17 },
    { 0xdf32c35a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1df2c35c, SUC_lexicon_tags_0 },
    { 0x9852c35c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf392c35f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdcd2c361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d32c364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd792c367, SUC_lexicon_tags_77 },
    { 0x1bb2c368, SUC_lexicon_tags_17 },
    { 0x7892c369, SUC_lexicon_tags_8 },
    { 0x3c92c368, SUC_lexicon_tags_351 },
    { 0x1c92c369, SUC_lexicon_tags_25 },
    { 0xeb92c36c, SUC_lexicon_tags_55 },
    { 0xb172c36d, SUC_lexicon_tags_17 },
    { 0xc152c36d, SUC_lexicon_tags_39 },
    { 0xf7b2c367, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e92c372, SUC_lexicon_tags_23 },
    { 0x3872c373, SUC_lexicon_tags_11 },
    { 0x88b2c374, SUC_lexicon_tags_4 },
    { 0x9572c374, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d32c378, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df2c37f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf2c38a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac72c38c, SUC_lexicon_tags_11 },
    { 0x6472c38d, SUC_lexicon_tags_33 },
    { 0x68f2c38e, SUC_lexicon_tags_1 },
    { 0xba32c38f, SUC_lexicon_tags_57 },
    { 0xd592c38c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb2c393, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab2c39b, SUC_lexicon_tags_68 },
    { 0xb0d2c39c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12c3a1, SUC_lexicon_tags_37 },
    { 0xa432c3a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad2c3a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9352c3ac, SUC_lexicon_tags_47 },
    { 0x1832c3ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5612c3af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaf2c3b1, SUC_lexicon_tags_60 },
    { 0x4632c3b2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc9d2c3b4, SUC_lexicon_tags_11 },
    { 0xf652c3b4, SUC_lexicon_tags_107 },
    { 0xa532c3b4, SUC_lexicon_tags_10 },
    { 0x612c3b6, SUC_lexicon_tags_0 },
    { 0x26b2c3b8, SUC_lexicon_tags_1 },
    { 0x6692c3b9, SUC_lexicon_tags_2 },
    { 0x8db2c3b9, SUC_lexicon_tags_7 },
    { 0xf292c3b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c12c3c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf072c3c6, SUC_lexicon_tags_51 },
    { 0xec52c3c7, SUC_lexicon_tags_10 },
    { 0x4c12c3c8, SUC_lexicon_tags_36 },
    { 0xbcd2c3c7, SUC_lexicon_tags_83 },
    { 0x9072c3c9, SUC_lexicon_tags_1 },
    { 0x41b2c3cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e92c3cf, SUC_lexicon_tags_17 },
    { 0xf952c3d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb92c3d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d2c3d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb2c3da, SUC_lexicon_tags_30 },
    { 0xdbf2c3db, SUC_lexicon_tags_80 },
    { 0x2772c3da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x152c3de, SUC_lexicon_tags_55 },
    { 0x8c32c3de, SUC_lexicon_tags_55 },
    { 0xc452c3e0, SUC_lexicon_tags_63 },
    { 0x29f2c3e1, SUC_lexicon_tags_37 },
    { 0x3852c3e1, SUC_lexicon_tags_15 },
    { 0x2192c3e3, SUC_lexicon_tags_34 },
    { 0xa4d2c3e3, SUC_lexicon_tags_11 },
    { 0x2132c3e4, SUC_lexicon_tags_12 },
    { 0x6192c3e6, SUC_lexicon_tags_7 },
    { 0x2e72c3e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e32c3eb, SUC_lexicon_tags_37 },
    { 0x2f92c3ec, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x5332c3ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf932c3f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212c3f7, SUC_lexicon_tags_15 },
    { 0xf7d2c3f8, SUC_lexicon_tags_56 },
    { 0x2312c3f8, SUC_lexicon_tags_2 },
    { 0x4e72c3fa, SUC_lexicon_tags_14 },
    { 0xb0b2c3fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab2c3ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4472c402, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd2c405, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b2c409, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5d2c40b, SUC_lexicon_tags_11 },
    { 0x8f52c40c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d2c40f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a52c412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1852c415, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9ad2c417, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf2c41a, SUC_lexicon_tags_28 },
    { 0x5ab2c41a, SUC_lexicon_tags_2 },
    { 0x3712c41c, SUC_lexicon_tags_60 },
    { 0x1cf2c41c, SUC_lexicon_tags_1 },
    { 0xef52c41b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8172c420, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2852c425, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f2c42c, SUC_lexicon_tags_3 },
    { 0x2ab2c42d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf12c432, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6012c434, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29f2c436, SUC_lexicon_tags_11 },
    { 0xaa32c437, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf972c43b, SUC_lexicon_tags_61 },
    { 0x6952c43b, SUC_lexicon_tags_29 },
    { 0xd152c43d, SUC_lexicon_tags_15 },
    { 0x3b32c43e, SUC_lexicon_tags_121 },
    { 0x30d2c43f, SUC_lexicon_tags_11 },
    { 0x6f12c43f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb2c443, SUC_lexicon_tags_11 },
    { 0xa232c444, SUC_lexicon_tags_33 },
    { 0x6db2c443, SUC_lexicon_tags_37 },
    { 0x112c446, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1632c44a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf2c44e, SUC_lexicon_tags_66 },
    { 0x8fd2c44f, SUC_lexicon_tags_37 },
    { 0xfad2c450, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e12c452, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3992c459, SUC_lexicon_tags_0 },
    { 0xe492c459, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed2c45f, SUC_lexicon_tags_1 },
    { 0xd3d2c45f, SUC_lexicon_tags_15 },
    { 0x5592c45f, SUC_lexicon_tags_7 },
    { 0xb2f2c460, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x932c464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b2c469, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d2c46e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd432c473, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b52c475, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1272c479, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae72c47d, SUC_lexicon_tags_12 },
    { 0x2312c47e, SUC_lexicon_tags_6 },
    { 0xeaf2c47f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e32c486, SUC_lexicon_tags_25 },
    { 0x3d52c487, SUC_lexicon_tags_37 },
    { 0x2012c488, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6db2c48a, SUC_lexicon_tags_26 },
    { 0x4f72c48b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9692c48f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af2c492, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5512c498, SUC_lexicon_tags_49 },
    { 0x6132c499, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd52c49d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8592c49f, SUC_lexicon_tags_37 },
    { 0x9112c4a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa892c4a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb552c4ab, SUC_lexicon_tags_84 },
    { 0x3372c4ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8592c4b5, SUC_lexicon_tags_17 },
    { 0xc772c4b6, SUC_lexicon_tags_30 },
    { 0x33f2c4b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a72c4c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b2c4cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff32c4d5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5392c4d9, SUC_lexicon_tags_9 },
    { 0xd852c4da, SUC_lexicon_tags_45 },
    { 0x8172c4da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2512c4e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c52c4e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc32c4e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f2c4ea, SUC_lexicon_tags_25 },
    { 0x5052c4eb, SUC_lexicon_tags_31 },
    { 0xa32c4ec, SUC_lexicon_tags_56 },
    { 0xbd12c4ea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa672c4ef, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d2c4f2, SUC_lexicon_tags_2 },
    { 0x2b32c4f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd2c4f6, SUC_lexicon_tags_17 },
    { 0x9872c4f7, SUC_lexicon_tags_15 },
    { 0xd612c4f8, SUC_lexicon_tags_65 },
    { 0x1012c4f6, SUC_lexicon_tags_2 },
    { 0x6d72c4fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce32c501, SUC_lexicon_tags_28 },
    { 0xedd2c501, SUC_lexicon_tags_21 },
    { 0xb352c503, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7232c506, SUC_lexicon_tags_37 },
    { 0xfff2c507, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7b52c509, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc32c50b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb2c510, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7832c512, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2332c514, SUC_lexicon_tags_3 },
    { 0x9372c515, SUC_lexicon_tags_17 },
    { 0x8f2c514, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad52c51b, SUC_lexicon_tags_14 },
    { 0x4a32c51c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc92c52e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d2c532, SUC_lexicon_tags_17 },
    { 0x26d2c532, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6b2c535, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9932c53b, SUC_lexicon_tags_1 },
    { 0xd12c53c, SUC_lexicon_tags_224 },
    { 0x952c53b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb72c53f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8012c543, SUC_lexicon_tags_49 },
    { 0x8752c544, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc92c547, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xce32c54a, SUC_lexicon_tags_11 },
    { 0xf6d2c54b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5672c54d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5212c54f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8972c552, SUC_lexicon_tags_82 },
    { 0x2972c553, SUC_lexicon_tags_46 },
    { 0x1192c552, SUC_lexicon_tags_37 },
    { 0x5df2c553, SUC_lexicon_tags_26 },
    { 0x6f52c552, SUC_lexicon_tags_33 },
    { 0xf0d2c557, SUC_lexicon_tags_1 },
    { 0xa852c558, SUC_lexicon_tags_25 },
    { 0xe1d2c558, SUC_lexicon_tags_27 },
    { 0x1892c556, SUC_lexicon_tags_23 },
    { 0x7812c552, SUC_lexicon_tags_86 },
    { 0x5b32c55a, SUC_lexicon_tags_7 },
    { 0x9572c55d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e72c561, SUC_lexicon_tags_52 },
    { 0x2972c561, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9452c568, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f2c56c, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe832c571, SUC_lexicon_tags_38 },
    { 0x4012c572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf312c574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b2c578, SUC_lexicon_tags_37 },
    { 0xbc32c579, SUC_lexicon_tags_14 },
    { 0xfe52c579, SUC_lexicon_tags_2 },
    { 0xd992c57b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f2c581, SUC_lexicon_tags_47 },
    { 0xc1f2c582, SUC_lexicon_tags_1 },
    { 0x4572c582, SUC_lexicon_tags_7 },
    { 0x3e52c583, SUC_lexicon_tags_17 },
    { 0x4c12c585, SUC_lexicon_tags_15 },
    { 0x8b32c586, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe552c58b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b2c590, SUC_lexicon_tags_63 },
    { 0x3752c590, SUC_lexicon_tags_21 },
    { 0xf2c592, SUC_lexicon_tags_38 },
    { 0x9cd2c593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e12c598, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1bb2c59a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba72c5a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4f2c5a7, SUC_lexicon_tags_17 },
    { 0xdbb2c5a8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1532c5af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6212c5b1, SUC_lexicon_tags_12 },
    { 0xfd52c5b2, SUC_lexicon_tags_47 },
    { 0x4f12c5b3, SUC_lexicon_tags_25 },
    { 0xc12c5b3, SUC_lexicon_tags_1 },
    { 0x27f2c5b5, SUC_lexicon_tags_1 },
    { 0x9292c5b5, SUC_lexicon_tags_28 },
    { 0xbd52c5b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c52c5ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4b2c5bc, SUC_lexicon_tags_47 },
    { 0x2112c5bc, SUC_lexicon_tags_60 },
    { 0x3db2c5be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa672c5c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44d2c5c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x272c5cb, SUC_lexicon_tags_37 },
    { 0x6592c5cc, SUC_lexicon_tags_11 },
    { 0x3772c5cd, SUC_lexicon_tags_1 },
    { 0x8e92c5cb, SUC_lexicon_tags_18 },
    { 0x35d2c5cf, SUC_lexicon_tags_37 },
    { 0x6b2c5d0, SUC_lexicon_tags_17 },
    { 0xaa52c5d0, SUC_lexicon_tags_7 },
    { 0x7672c5cc, SUC_lexicon_tags_9 },
    { 0xc972c5d3, SUC_lexicon_tags_25 },
    { 0xc2b2c5d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb2c5d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc032c5dc, SUC_lexicon_tags_11 },
    { 0x33d2c5dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb12c5df, SUC_lexicon_tags_25 },
    { 0x1ff2c5e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f2c5e6, SUC_lexicon_tags_2 },
    { 0x1132c5e7, SUC_lexicon_tags_15 },
    { 0xe532c5e6, SUC_lexicon_tags_12 },
    { 0x1912c5e7, SUC_lexicon_tags_27 },
    { 0x31f2c5ea, SUC_lexicon_tags_1 },
    { 0x3c12c5e7, SUC_lexicon_tags_12 },
    { 0xed2c5e7, SUC_lexicon_tags_0 },
    { 0xc8f2c5ed, SUC_lexicon_tags_8 },
    { 0x1152c5ee, SUC_lexicon_tags_31 },
    { 0x35d2c5ef, SUC_lexicon_tags_19 },
    { 0x1d12c5ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb152c5f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b92c5f8, SUC_lexicon_tags_26 },
    { 0xe2d2c5f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9752c5ff, SUC_lexicon_tags_25 },
    { 0xb6f2c600, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6592c602, SUC_lexicon_tags_30 },
    { 0xf1b2c602, SUC_lexicon_tags_113 },
    { 0xc652c603, SUC_lexicon_tags_23 },
    { 0xed52c602, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd272c608, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x272c60b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfd2c60d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1e92c60f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d52c615, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1952c618, SUC_lexicon_tags_14 },
    { 0x8a52c618, SUC_lexicon_tags_33 },
    { 0x9632c618, SUC_lexicon_tags_37 },
    { 0x2a12c61a, SUC_lexicon_tags_11 },
    { 0x56d2c61c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c52c61f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb972c622, SUC_lexicon_tags_17 },
    { 0x9b72c623, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa52c627, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d2c629, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d2c62c, SUC_lexicon_tags_5 },
    { 0xcf12c62c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd2c632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec12c637, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1432c63b, SUC_lexicon_tags_7 },
    { 0xdd92c63b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb2c640, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ff2c642, SUC_lexicon_tags_17 },
    { 0x6232c643, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1952c646, SUC_lexicon_tags_1 },
    { 0xa232c647, SUC_lexicon_tags_12 },
    { 0xbb2c648, SUC_lexicon_tags_68 },
    { 0x3972c647, SUC_lexicon_tags_45 },
    { 0x5332c649, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2c64e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1392c650, SUC_lexicon_tags_1 },
    { 0xc192c651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9632c653, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b2c657, SUC_lexicon_tags_8 },
    { 0x2f2c658, SUC_lexicon_tags_37 },
    { 0x78f2c659, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b2c65e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af2c662, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f12c665, SUC_lexicon_tags_33 },
    { 0x1252c666, SUC_lexicon_tags_23 },
    { 0x792c665, SUC_lexicon_tags_35 },
    { 0x3192c668, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5392c66a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x72c66d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x28d2c66f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x312c672, SUC_lexicon_tags_16 },
    { 0x2852c673, SUC_lexicon_tags_63 },
    { 0xa7d2c674, SUC_lexicon_tags_14 },
    { 0x5b32c672, SUC_lexicon_tags_11 },
    { 0xc192c676, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f2c679, SUC_lexicon_tags_17 },
    { 0xe892c67a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2c67f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7192c681, SUC_lexicon_tags_27 },
    { 0x6bb2c682, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4212c684, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef2c68c, SUC_lexicon_tags_28 },
    { 0x4072c68c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8912c695, SUC_lexicon_tags_3 },
    { 0xc4b2c696, SUC_lexicon_tags_15 },
    { 0x47b2c697, SUC_lexicon_tags_26 },
    { 0x4d12c696, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f2c6a2, SUC_lexicon_tags_7 },
    { 0xf872c6a3, SUC_lexicon_tags_37 },
    { 0xbb92c6a2, SUC_lexicon_tags_77 },
    { 0x56b2c6a5, SUC_lexicon_tags_25 },
    { 0x56f2c6a5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d72c6af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4bd2c6b1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x74d2c6b3, SUC_lexicon_tags_120 },
    { 0xcc12c6b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9772c6b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x45b2c6ba, SUC_lexicon_tags_8 },
    { 0xabd2c6bb, SUC_lexicon_tags_25 },
    { 0x1ef2c6ba, SUC_lexicon_tags_45 },
    { 0xd0f2c6bd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xd412c6c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f72c6c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6132c6c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d2c6cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd2c6cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5792c6d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c92c6d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf712c6d7, SUC_lexicon_tags_37 },
    { 0x60b2c6d7, SUC_lexicon_tags_12 },
    { 0x5792c6d9, SUC_lexicon_tags_8 },
    { 0x932c6da, SUC_lexicon_tags_29 },
    { 0x6f52c6db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9652c6de, SUC_lexicon_tags_43 },
    { 0x2432c6df, SUC_lexicon_tags_37 },
    { 0x1f92c6df, SUC_lexicon_tags_37 },
    { 0x3b2c6e1, SUC_lexicon_tags_25 },
    { 0x3db2c6e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d12c6e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe72c6e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x96f2c6eb, SUC_lexicon_tags_1 },
    { 0x83d2c6ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbc92c6ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d2c6f2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b2c6f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb912c6fc, SUC_lexicon_tags_2 },
    { 0x6cd2c6fd, SUC_lexicon_tags_11 },
    { 0x7412c6fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7532c700, SUC_lexicon_tags_171 },
    { 0xe6f2c701, SUC_lexicon_tags_46 },
    { 0xac32c702, SUC_lexicon_tags_15 },
    { 0xe432c703, SUC_lexicon_tags_25 },
    { 0xfdb2c704, SUC_lexicon_tags_8 },
    { 0xb52c700, SUC_lexicon_tags_27 },
    { 0xe9b2c706, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe752c708, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe1f2c70a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f2c70d, SUC_lexicon_tags_7 },
    { 0x9772c70e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc292c713, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d92c719, SUC_lexicon_tags_15 },
    { 0x4e32c71a, SUC_lexicon_tags_17 },
    { 0x7f2c71b, SUC_lexicon_tags_49 },
    { 0x3e72c71b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b2c720, SUC_lexicon_tags_2 },
    { 0x3ef2c721, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa72c723, SUC_lexicon_tags_26 },
    { 0xb3b2c723, SUC_lexicon_tags_1 },
    { 0xdd92c725, SUC_lexicon_tags_26 },
    { 0x2452c726, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7cf2c728, SUC_lexicon_tags_7 },
    { 0xd532c729, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4852c72b, SUC_lexicon_tags_15 },
    { 0xd092c72c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4912c731, SUC_lexicon_tags_25 },
    { 0xd92c731, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf9b2c734, SUC_lexicon_tags_21 },
    { 0xa572c735, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d52c73c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5712c740, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d2c749, SUC_lexicon_tags_19 },
    { 0x4372c74a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b2c74c, SUC_lexicon_tags_37 },
    { 0x5792c74c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xef2c750, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4852c757, SUC_lexicon_tags_1 },
    { 0xd4f2c758, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd52c75a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f32c75f, SUC_lexicon_tags_4 },
    { 0x5992c760, SUC_lexicon_tags_1 },
    { 0x852c761, SUC_lexicon_tags_64 },
    { 0x812c762, SUC_lexicon_tags_49 },
    { 0x6cf2c762, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5832c766, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b12c76b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf192c770, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93d2c772, SUC_lexicon_tags_14 },
    { 0x17d2c772, SUC_lexicon_tags_2 },
    { 0xb2d2c774, SUC_lexicon_tags_5 },
    { 0x2072c775, SUC_lexicon_tags_14 },
    { 0x5b92c774, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa92c778, SUC_lexicon_tags_68 },
    { 0x5ef2c779, SUC_lexicon_tags_24 },
    { 0x6fb2c77a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f32c77d, SUC_lexicon_tags_0 },
    { 0x6272c77e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa72c783, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcdd2c785, SUC_lexicon_tags_17 },
    { 0x4732c786, SUC_lexicon_tags_49 },
    { 0xe532c787, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8452c789, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f2c78f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f2c797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef2c79d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1872c79f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87f2c7a1, SUC_lexicon_tags_25 },
    { 0x16f2c7a2, SUC_lexicon_tags_0 },
    { 0x2c72c7a1, SUC_lexicon_tags_11 },
    { 0x4812c7a1, SUC_lexicon_tags_37 },
    { 0xb112c7a5, SUC_lexicon_tags_25 },
    { 0x5b32c7a6, SUC_lexicon_tags_7 },
    { 0xea12c7a3, SUC_lexicon_tags_28 },
    { 0x3ab2c7a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f2c7ac, SUC_lexicon_tags_25 },
    { 0x8312c7ad, SUC_lexicon_tags_25 },
    { 0x1392c7ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6092c7b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2392c7b8, SUC_lexicon_tags_12 },
    { 0x6472c7b9, SUC_lexicon_tags_29 },
    { 0x5932c7ba, SUC_lexicon_tags_37 },
    { 0x43d2c7ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bf2c7bd, SUC_lexicon_tags_7 },
    { 0x54f2c7bd, SUC_lexicon_tags_12 },
    { 0xded2c7be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5712c7c1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd2c7c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1092c7cb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb2c7db, SUC_lexicon_tags_63 },
    { 0x13b2c7db, SUC_lexicon_tags_39 },
    { 0x3012c7dc, SUC_lexicon_tags_69 },
    { 0xc072c7de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f2c7e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf352c7ea, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd2c7ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3452c7f1, SUC_lexicon_tags_8 },
    { 0xbe12c7f2, SUC_lexicon_tags_2 },
    { 0xcd2c7f1, SUC_lexicon_tags_7 },
    { 0x5cd2c7f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda92c7f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb692c802, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc412c804, SUC_lexicon_tags_32 },
    { 0xe1f2c805, SUC_lexicon_tags_25 },
    { 0x78d2c806, SUC_lexicon_tags_10 },
    { 0x96d2c805, SUC_lexicon_tags_25 },
    { 0xee72c808, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2852c80d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8952c810, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9172c813, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb412c817, SUC_lexicon_tags_12 },
    { 0x4b92c818, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4832c81b, SUC_lexicon_tags_37 },
    { 0x5ab2c81c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd2c81f, SUC_lexicon_tags_0 },
    { 0x1752c81f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9d2c822, SUC_lexicon_tags_27 },
    { 0x2a12c823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc752c829, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd832c82c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f12c82f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9992c831, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7fb2c833, SUC_lexicon_tags_49 },
    { 0xd392c834, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b2c838, SUC_lexicon_tags_15 },
    { 0x96b2c839, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd672c83c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe332c840, SUC_lexicon_tags_14 },
    { 0x2092c840, SUC_lexicon_tags_2 },
    { 0xf712c842, SUC_lexicon_tags_26 },
    { 0xb4f2c843, SUC_lexicon_tags_37 },
    { 0xa132c843, SUC_lexicon_tags_14 },
    { 0x6af2c844, SUC_lexicon_tags_38 },
    { 0x4f52c845, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc972c848, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8552c84a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e52c850, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70b2c852, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd2c855, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd2c859, SUC_lexicon_tags_43 },
    { 0x6652c85a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6612c865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b2c86d, SUC_lexicon_tags_68 },
    { 0xe0b2c86e, SUC_lexicon_tags_7 },
    { 0xe972c86f, SUC_lexicon_tags_42 },
    { 0xf552c86e, SUC_lexicon_tags_37 },
    { 0xd2f2c86d, SUC_lexicon_tags_27 },
    { 0xd252c872, SUC_lexicon_tags_43 },
    { 0x9db2c86d, SUC_lexicon_tags_27 },
    { 0x2832c874, SUC_lexicon_tags_6 },
    { 0x4df2c86e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3272c877, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0d2c879, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f2c87d, SUC_lexicon_tags_11 },
    { 0xa692c87d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b92c880, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd752c884, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db2c88b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f2c88e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa912c890, SUC_lexicon_tags_16 },
    { 0x36d2c891, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8652c895, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b2c89a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe272c89d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4452c8a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4932c8a9, SUC_lexicon_tags_11 },
    { 0x1592c8a9, SUC_lexicon_tags_55 },
    { 0xb472c8ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd72c8b3, SUC_lexicon_tags_16 },
    { 0x4d12c8b3, SUC_lexicon_tags_19 },
    { 0xb332c8b4, SUC_lexicon_tags_14 },
    { 0x58d2c8b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xced2c8b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6b2c8ba, SUC_lexicon_tags_10 },
    { 0x3332c8ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9fb2c8bd, SUC_lexicon_tags_14 },
    { 0xe652c8be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe972c8c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f2c8c7, SUC_lexicon_tags_26 },
    { 0x1ab2c8c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd2c8cc, SUC_lexicon_tags_36 },
    { 0x34d2c8cd, SUC_lexicon_tags_144 },
    { 0xa8d2c8cd, SUC_lexicon_tags_43 },
    { 0x9432c8cf, SUC_lexicon_tags_23 },
    { 0xd1b2c8cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbff2c8d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa712c8d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe732c8da, SUC_lexicon_tags_3 },
    { 0x6932c8db, SUC_lexicon_tags_14 },
    { 0x3212c8da, SUC_lexicon_tags_7 },
    { 0xa832c8da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d2c8e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a32c8e5, SUC_lexicon_tags_23 },
    { 0x5332c8e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28f2c8e8, SUC_lexicon_tags_27 },
    { 0xfad2c8e9, SUC_lexicon_tags_9 },
    { 0xbd92c8ea, SUC_lexicon_tags_27 },
    { 0x2cb2c8eb, SUC_lexicon_tags_60 },
    { 0x40f2c8e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f2c8ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xde92c8f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2c8f6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3a92c8f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f2c8fe, SUC_lexicon_tags_33 },
    { 0xbdd2c8fe, SUC_lexicon_tags_33 },
    { 0xe952c900, SUC_lexicon_tags_17 },
    { 0x8872c901, SUC_lexicon_tags_49 },
    { 0x4a32c8fe, SUC_lexicon_tags_47 },
    { 0x9ad2c903, SUC_lexicon_tags_60 },
    { 0x4fd2c903, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9fb2c906, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1072c90a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1832c912, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f2c916, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1012c91c, SUC_lexicon_tags_17 },
    { 0x88f2c91d, SUC_lexicon_tags_7 },
    { 0x44f2c91c, SUC_lexicon_tags_66 },
    { 0x4fd2c91f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa52c923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb052c925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe892c927, SUC_lexicon_tags_15 },
    { 0x7232c928, SUC_lexicon_tags_35 },
    { 0x4672c929, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d12c92c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c92c930, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c72c932, SUC_lexicon_tags_43 },
    { 0xbcb2c933, SUC_lexicon_tags_13 },
    { 0xa252c933, SUC_lexicon_tags_7 },
    { 0xf632c934, SUC_lexicon_tags_11 },
    { 0xe812c932, SUC_lexicon_tags_15 },
    { 0x6592c935, SUC_lexicon_tags_19 },
    { 0xca32c933, SUC_lexicon_tags_59 },
    { 0x8ff2c935, SUC_lexicon_tags_47 },
    { 0xc632c93a, SUC_lexicon_tags_15 },
    { 0xe872c934, SUC_lexicon_tags_25 },
    { 0x4d32c93c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd2c940, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9092c944, SUC_lexicon_tags_7 },
    { 0xcaf2c945, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xef72c948, SUC_lexicon_tags_29 },
    { 0x39d2c949, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f12c950, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6b2c952, SUC_lexicon_tags_28 },
    { 0x4432c953, SUC_lexicon_tags_17 },
    { 0x3492c953, SUC_lexicon_tags_9 },
    { 0x4392c955, SUC_lexicon_tags_26 },
    { 0xa832c956, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472c95b, SUC_lexicon_tags_60 },
    { 0xb612c95b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f32c960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f2c965, SUC_lexicon_tags_11 },
    { 0xf552c966, SUC_lexicon_tags_4 },
    { 0x1b2c967, SUC_lexicon_tags_19 },
    { 0xfa52c968, SUC_lexicon_tags_11 },
    { 0xf432c969, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf692c96b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5512c96d, SUC_lexicon_tags_0 },
    { 0xf832c96d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f2c973, SUC_lexicon_tags_60 },
    { 0xc72c974, SUC_lexicon_tags_31 },
    { 0x71d2c973, SUC_lexicon_tags_32 },
    { 0x3b32c973, SUC_lexicon_tags_11 },
    { 0x55d2c977, SUC_lexicon_tags_1 },
    { 0xe6b2c978, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4072c97a, SUC_lexicon_tags_49 },
    { 0x2792c97b, SUC_lexicon_tags_33 },
    { 0x2812c97a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe252c97f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x772c982, SUC_lexicon_tags_7 },
    { 0xb352c983, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2732c987, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xea52c98a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x752c98e, SUC_lexicon_tags_17 },
    { 0xf9d2c98f, SUC_lexicon_tags_17 },
    { 0x7ff2c990, SUC_lexicon_tags_14 },
    { 0xa92c991, SUC_lexicon_tags_6 },
    { 0xc8b2c992, SUC_lexicon_tags_11 },
    { 0x74f2c991, SUC_lexicon_tags_0 },
    { 0x6952c98e, SUC_lexicon_tags_2 },
    { 0x24b2c995, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b2c99f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a92c9a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa832c9a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf2c9ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa5f2c9b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf2c9b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b72c9b6, SUC_lexicon_tags_289 },
    { 0x2192c9b6, SUC_lexicon_tags_17 },
    { 0x8ad2c9b8, SUC_lexicon_tags_1 },
    { 0xd392c9b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb12c9bb, SUC_lexicon_tags_17 },
    { 0xbf2c9bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d2c9c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5772c9c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf72c9cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9692c9d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8f2c9d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd2c9d9, SUC_lexicon_tags_1 },
    { 0xb7b2c9da, SUC_lexicon_tags_79 },
    { 0xced2c9db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3d2c9dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9652c9e3, SUC_lexicon_tags_3 },
    { 0x7df2c9e4, SUC_lexicon_tags_37 },
    { 0x56b2c9e5, SUC_lexicon_tags_38 },
    { 0x6cb2c9e5, SUC_lexicon_tags_2 },
    { 0xeed2c9e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf4d2c9e9, SUC_lexicon_tags_24 },
    { 0xbc52c9e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912c9f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed92c9f7, SUC_lexicon_tags_12 },
    { 0x5612c9f8, SUC_lexicon_tags_46 },
    { 0x1e92c9f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b32c9fb, SUC_lexicon_tags_11 },
    { 0x21d2c9fb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8b92c9fe, SUC_lexicon_tags_172 },
    { 0xd632c9ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f2ca03, SUC_lexicon_tags_30 },
    { 0x63d2ca03, SUC_lexicon_tags_37 },
    { 0xb6d2ca03, SUC_lexicon_tags_17 },
    { 0xa6d2ca06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8ab2ca08, SUC_lexicon_tags_68 },
    { 0x3592ca09, SUC_lexicon_tags_37 },
    { 0x1092ca0a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe072ca0c, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a72ca11, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef2ca19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xab32ca1b, SUC_lexicon_tags_34 },
    { 0xce92ca1b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb2ca20, SUC_lexicon_tags_1 },
    { 0x52d2ca21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6292ca25, SUC_lexicon_tags_39 },
    { 0x9b12ca25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b32ca2d, SUC_lexicon_tags_11 },
    { 0xb232ca2d, SUC_lexicon_tags_38 },
    { 0xad2ca2e, SUC_lexicon_tags_26 },
    { 0x6af2ca2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a12ca34, SUC_lexicon_tags_117 },
    { 0x75d2ca35, SUC_lexicon_tags_14 },
    { 0x2952ca34, SUC_lexicon_tags_34 },
    { 0xedd2ca37, SUC_lexicon_tags_25 },
    { 0x9732ca37, SUC_lexicon_tags_1 },
    { 0x9592ca39, SUC_lexicon_tags_37 },
    { 0x3b32ca3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf972ca3c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d2ca46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792ca4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f2ca53, SUC_lexicon_tags_7 },
    { 0xf9b2ca54, SUC_lexicon_tags_26 },
    { 0x5c52ca55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb2ca59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f2ca5c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6352ca5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d2ca64, SUC_lexicon_tags_37 },
    { 0xbd72ca65, SUC_lexicon_tags_1 },
    { 0xbad2ca66, SUC_lexicon_tags_12 },
    { 0xd3f2ca67, SUC_lexicon_tags_7 },
    { 0x40f2ca66, SUC_lexicon_tags_27 },
    { 0x8e52ca64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c52ca6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4672ca6f, SUC_lexicon_tags_7 },
    { 0x1d52ca70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab2ca73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4d2ca75, SUC_lexicon_tags_17 },
    { 0x79d2ca76, SUC_lexicon_tags_30 },
    { 0xd172ca77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7032ca79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e32ca7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2d2ca7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef32ca81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f92ca84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65f2ca86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10b2ca88, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f2ca8c, SUC_lexicon_tags_2 },
    { 0xf272ca8d, SUC_lexicon_tags_17 },
    { 0xb9f2ca8c, SUC_lexicon_tags_23 },
    { 0x8f72ca8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f92ca97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb692ca9d, SUC_lexicon_tags_19 },
    { 0xe292ca9e, SUC_lexicon_tags_179 },
    { 0x1812ca9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf2caa2, SUC_lexicon_tags_37 },
    { 0x2a92caa2, SUC_lexicon_tags_15 },
    { 0x7152caa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d52caa7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6392caaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4572caae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8672cab1, SUC_lexicon_tags_30 },
    { 0x5d72cab2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd2cab5, SUC_lexicon_tags_91 },
    { 0xebf2cab6, SUC_lexicon_tags_7 },
    { 0xb7b2cab7, SUC_lexicon_tags_12 },
    { 0x9d12cab5, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3692cac2, SUC_lexicon_tags_12 },
    { 0xaff2cac3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd2cac6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3832cac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d2cacb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd2cacd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a52cad0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732cad4, SUC_lexicon_tags_25 },
    { 0x2232cad4, SUC_lexicon_tags_4 },
    { 0xeb92cad6, SUC_lexicon_tags_7 },
    { 0xb092cad6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb52cade, SUC_lexicon_tags_11 },
    { 0x59d2cade, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd532cae2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf952cae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb2caeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed92caed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39d2caef, SUC_lexicon_tags_2 },
    { 0xcbf2caf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5932caf7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912cafd, SUC_lexicon_tags_40 },
    { 0xecd2cafd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3892cb05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe772cb07, SUC_lexicon_tags_12 },
    { 0xf332cb08, SUC_lexicon_tags_107 },
    { 0x7792cb09, SUC_lexicon_tags_49 },
    { 0xf0f2cb07, SUC_lexicon_tags_87 },
    { 0x3072cb0b, SUC_lexicon_tags_38 },
    { 0x6472cb0c, SUC_lexicon_tags_11 },
    { 0x4dd2cb0d, SUC_lexicon_tags_37 },
    { 0xd652cb0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f72cb15, SUC_lexicon_tags_15 },
    { 0x63d2cb16, SUC_lexicon_tags_11 },
    { 0xc212cb17, SUC_lexicon_tags_79 },
    { 0xa112cb17, SUC_lexicon_tags_11 },
    { 0x532cb16, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b2cb1d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7852cb21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4f2cb23, SUC_lexicon_tags_6 },
    { 0x6ed2cb23, SUC_lexicon_tags_21 },
    { 0x2f12cb24, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x60f2cb27, SUC_lexicon_tags_47 },
    { 0xbf52cb27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc92cb2c, SUC_lexicon_tags_7 },
    { 0x6f92cb2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae92cb31, SUC_lexicon_tags_42 },
    { 0x2832cb31, SUC_lexicon_tags_12 },
    { 0x6ff2cb33, SUC_lexicon_tags_79 },
    { 0xed92cb32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f2cb3c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7592cb3e, SUC_lexicon_tags_37 },
    { 0x12d2cb3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x43d2cb41, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8552cb48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab52cb4d, SUC_lexicon_tags_801 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb612cb51, SUC_lexicon_tags_37 },
    { 0xe152cb51, SUC_lexicon_tags_11 },
    { 0x8f52cb51, SUC_lexicon_tags_17 },
    { 0x7db2cb54, SUC_lexicon_tags_43 },
    { 0x9812cb53, SUC_lexicon_tags_15 },
    { 0xec32cb56, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2072cb5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7312cb60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff2cb65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e92cb68, SUC_lexicon_tags_65 },
    { 0xd172cb69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb652cb6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b2cb6e, SUC_lexicon_tags_30 },
    { 0x8f72cb6f, SUC_lexicon_tags_14 },
    { 0x8f92cb6e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1272cb75, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xecb2cb77, SUC_lexicon_tags_15 },
    { 0x1732cb78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e32cb7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb2cb84, SUC_lexicon_tags_12 },
    { 0x9a72cb85, SUC_lexicon_tags_27 },
    { 0xa392cb85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2d2cb88, SUC_lexicon_tags_17 },
    { 0x4212cb89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f2cb8c, SUC_lexicon_tags_37 },
    { 0x9152cb8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c72cb94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74b2cb96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6bd2cb98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2cb9d, SUC_lexicon_tags_37 },
    { 0x8612cb9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1532cba1, SUC_lexicon_tags_27 },
    { 0x1132cba2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf92cba4, SUC_lexicon_tags_15 },
    { 0x5292cba5, SUC_lexicon_tags_44 },
    { 0x3052cba6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe352cba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa312cbaa, SUC_lexicon_tags_1 },
    { 0x4152cbaa, SUC_lexicon_tags_30 },
    { 0x8392cbac, SUC_lexicon_tags_6 },
    { 0xca52cbaa, SUC_lexicon_tags_47 },
    { 0x59f2cbae, SUC_lexicon_tags_63 },
    { 0x95f2cbaf, SUC_lexicon_tags_26 },
    { 0x3332cbac, SUC_lexicon_tags_12 },
    { 0xc8d2cbb1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2cbb4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d92cbb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8092cbbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd2cbbf, SUC_lexicon_tags_45 },
    { 0xc4f2cbc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f92cbc2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb2cbc7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8612cbc9, SUC_lexicon_tags_11 },
    { 0x3eb2cbca, SUC_lexicon_tags_1 },
    { 0xf192cbcb, SUC_lexicon_tags_8 },
    { 0xd2f2cbcb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8252cbce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x44b2cbd0, SUC_lexicon_tags_33 },
    { 0xf952cbd1, SUC_lexicon_tags_15 },
    { 0xdcb2cbd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f2cbd6, SUC_lexicon_tags_14 },
    { 0x4c72cbd7, SUC_lexicon_tags_37 },
    { 0x82d2cbd7, SUC_lexicon_tags_1 },
    { 0x32b2cbd7, SUC_lexicon_tags_63 },
    { 0xb512cbd6, SUC_lexicon_tags_0 },
    { 0x8eb2cbdb, SUC_lexicon_tags_25 },
    { 0xbed2cbdb, SUC_lexicon_tags_39 },
    { 0xb872cbdb, SUC_lexicon_tags_60 },
    { 0x6032cbde, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f2cbe3, SUC_lexicon_tags_14 },
    { 0xab52cbe4, SUC_lexicon_tags_12 },
    { 0x4b2cbe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf752cbe8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc12cbeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e32cbee, SUC_lexicon_tags_1451 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f2cbf2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4272cbf7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc12cbfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d2cbfe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1252cc00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd52cc04, SUC_lexicon_tags_2 },
    { 0x7692cc04, SUC_lexicon_tags_2 },
    { 0xf952cc06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2032cc08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7692cc0c, SUC_lexicon_tags_136 },
    { 0x3372cc0c, SUC_lexicon_tags_19 },
    { 0xec92cc0e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc952cc10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c32cc16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd92cc1c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b32cc1e, SUC_lexicon_tags_29 },
    { 0x1352cc1f, SUC_lexicon_tags_69 },
    { 0xbf2cc1f, SUC_lexicon_tags_7 },
    { 0x6892cc20, SUC_lexicon_tags_12 },
    { 0xbd32cc22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa512cc25, SUC_lexicon_tags_31 },
    { 0xfe72cc25, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x612cc28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1332cc2a, SUC_lexicon_tags_17 },
    { 0xb352cc2b, SUC_lexicon_tags_66 },
    { 0x772cc2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x952cc31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56d2cc33, SUC_lexicon_tags_49 },
    { 0x7472cc34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8952cc36, SUC_lexicon_tags_12 },
    { 0x7712cc37, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3972cc46, SUC_lexicon_tags_10 },
    { 0x9d92cc47, SUC_lexicon_tags_148 },
    { 0xf7b2cc48, SUC_lexicon_tags_17 },
    { 0xbff2cc49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d2cc51, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa952cc5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2cc5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d72cc60, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f72cc65, SUC_lexicon_tags_62 },
    { 0x2212cc66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9272cc6a, SUC_lexicon_tags_15 },
    { 0x5932cc6a, SUC_lexicon_tags_7 },
    { 0x19d2cc6c, SUC_lexicon_tags_17 },
    { 0x24d2cc6b, SUC_lexicon_tags_7 },
    { 0xb7d2cc6e, SUC_lexicon_tags_29 },
    { 0xdb52cc6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff72cc7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad2cc80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad2cc86, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1f92cc88, SUC_lexicon_tags_39 },
    { 0x1792cc89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1d2cc8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb2cc92, SUC_lexicon_tags_26 },
    { 0xd7d2cc92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc692cc95, SUC_lexicon_tags_7 },
    { 0xba32cc96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9112cc98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff2cc9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b2cc9e, SUC_lexicon_tags_14 },
    { 0xce52cc9e, SUC_lexicon_tags_65 },
    { 0xa532cca0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c72cca5, SUC_lexicon_tags_25 },
    { 0x2eb2cca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf632ccaa, SUC_lexicon_tags_1 },
    { 0xb1b2ccaa, SUC_lexicon_tags_6 },
    { 0x9ad2ccac, SUC_lexicon_tags_7 },
    { 0xd972ccac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4252ccb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5732ccb4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3372ccb6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xad92ccb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c52ccbc, SUC_lexicon_tags_37 },
    { 0xf512ccbd, SUC_lexicon_tags_1 },
    { 0xde32ccbe, SUC_lexicon_tags_25 },
    { 0xe9f2ccbe, SUC_lexicon_tags_9 },
    { 0x54f2ccc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7232ccc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb892ccc8, SUC_lexicon_tags_17 },
    { 0x3bf2ccc8, SUC_lexicon_tags_37 },
    { 0xc112ccc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7472cccc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b12ccd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf2ccd6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xec72ccd9, SUC_lexicon_tags_1 },
    { 0xb3b2ccda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4232cce4, SUC_lexicon_tags_26 },
    { 0xc3b2cce5, SUC_lexicon_tags_11 },
    { 0x1ed2cce5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1372cceb, SUC_lexicon_tags_82 },
    { 0x89d2ccec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3232ccee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4032ccf0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5d12ccf2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe312ccf5, SUC_lexicon_tags_7 },
    { 0xaa12ccf5, SUC_lexicon_tags_2 },
    { 0x3a32ccf5, SUC_lexicon_tags_37 },
    { 0x1a12ccf8, SUC_lexicon_tags_17 },
    { 0xef52ccf8, SUC_lexicon_tags_15 },
    { 0x76d2ccf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb2cd07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d32cd0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f32cd12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb452cd15, SUC_lexicon_tags_45 },
    { 0x8832cd16, SUC_lexicon_tags_12 },
    { 0x8db2cd17, SUC_lexicon_tags_2 },
    { 0xfc32cd17, SUC_lexicon_tags_32 },
    { 0xdf32cd19, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f2cd1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e72cd22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd1d2cd24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1812cd26, SUC_lexicon_tags_8 },
    { 0x8d2cd27, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb2cd2e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf272cd31, SUC_lexicon_tags_47 },
    { 0xab52cd32, SUC_lexicon_tags_25 },
    { 0xac72cd31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb572cd35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7252cd3b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2452cd3e, SUC_lexicon_tags_2 },
    { 0x47d2cd3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadb2cd41, SUC_lexicon_tags_37 },
    { 0xa672cd42, SUC_lexicon_tags_25 },
    { 0x9e32cd43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d2cd46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5572cd49, SUC_lexicon_tags_1 },
    { 0x98b2cd4a, SUC_lexicon_tags_17 },
    { 0x1892cd4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc012cd4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf012cd53, SUC_lexicon_tags_1 },
    { 0x2f32cd54, SUC_lexicon_tags_63 },
    { 0xad92cd55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44d2cd57, SUC_lexicon_tags_4 },
    { 0xf4f2cd58, SUC_lexicon_tags_26 },
    { 0x93f2cd57, SUC_lexicon_tags_23 },
    { 0xab12cd57, SUC_lexicon_tags_79 },
    { 0x1432cd58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f2cd62, SUC_lexicon_tags_20 },
    { 0x4dd2cd63, SUC_lexicon_tags_12 },
    { 0x4d92cd64, SUC_lexicon_tags_46 },
    { 0x8112cd65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5892cd70, SUC_lexicon_tags_19 },
    { 0x35b2cd70, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xe792cd73, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2952cd76, SUC_lexicon_tags_25 },
    { 0x20d2cd77, SUC_lexicon_tags_43 },
    { 0x9ef2cd78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9eb2cd7a, SUC_lexicon_tags_15 },
    { 0x292cd7a, SUC_lexicon_tags_11 },
    { 0x2fb2cd7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x212cd7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2cd82, SUC_lexicon_tags_2 },
    { 0xab32cd83, SUC_lexicon_tags_47 },
    { 0xdaf2cd84, SUC_lexicon_tags_14 },
    { 0xf932cd85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d2cd8b, SUC_lexicon_tags_27 },
    { 0x2bb2cd8c, SUC_lexicon_tags_27 },
    { 0x2a32cd8d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfe72cd8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb2cd94, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1292cd98, SUC_lexicon_tags_59 },
    { 0xd932cd99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb92cd9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad92cda3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcab2cda5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf3d2cda7, SUC_lexicon_tags_45 },
    { 0xaf12cda7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb852cdad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf2cdaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8972cdb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f12cdb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b32cdba, SUC_lexicon_tags_14 },
    { 0xfb32cdbb, SUC_lexicon_tags_17 },
    { 0x632cdbc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7cd2cdbe, SUC_lexicon_tags_31 },
    { 0x89b2cdbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa892cdc2, SUC_lexicon_tags_30 },
    { 0x2512cdc2, SUC_lexicon_tags_30 },
    { 0x5392cdc4, SUC_lexicon_tags_0 },
    { 0x37b2cdc5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b2cdcf, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xa912cdd1, SUC_lexicon_tags_15 },
    { 0x1cd2cdd2, SUC_lexicon_tags_17 },
    { 0xebb2cdd3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x3212cdd5, SUC_lexicon_tags_3 },
    { 0xbfb2cdd6, SUC_lexicon_tags_55 },
    { 0x7cd2cdd6, SUC_lexicon_tags_14 },
    { 0xcbb2cdd6, SUC_lexicon_tags_47 },
    { 0x39b2cdd9, SUC_lexicon_tags_9 },
    { 0xe3d2cdd9, SUC_lexicon_tags_45 },
    { 0x2312cdd9, SUC_lexicon_tags_28 },
    { 0x2b12cdd8, SUC_lexicon_tags_17 },
    { 0x99f2cddd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3992cddf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb2cde6, SUC_lexicon_tags_15 },
    { 0xe1b2cde7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f2cdf3, SUC_lexicon_tags_30 },
    { 0x8e32cdf4, SUC_lexicon_tags_33 },
    { 0x4512cdf5, SUC_lexicon_tags_11 },
    { 0x54f2cdf6, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xcd12cdf8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x36d2cdfa, SUC_lexicon_tags_37 },
    { 0x1052cdfb, SUC_lexicon_tags_61 },
    { 0xa012cdfc, SUC_lexicon_tags_46 },
    { 0xd2d2cdfa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a72ce01, SUC_lexicon_tags_2 },
    { 0xe672ce01, SUC_lexicon_tags_61 },
    { 0x7552ce03, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6272ce05, SUC_lexicon_tags_14 },
    { 0x96b2ce06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf52ce09, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e12ce13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7f2ce15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd92ce26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d2ce2b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5832ce2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x532ce31, SUC_lexicon_tags_37 },
    { 0x2e72ce32, SUC_lexicon_tags_27 },
    { 0xf4b2ce33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb532ce36, SUC_lexicon_tags_61 },
    { 0x7752ce37, SUC_lexicon_tags_45 },
    { 0xefb2ce38, SUC_lexicon_tags_25 },
    { 0x4332ce38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4412ce3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d2ce41, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1772ce43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8832ce46, SUC_lexicon_tags_35 },
    { 0x2cf2ce47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf2ce4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f2ce50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3212ce52, SUC_lexicon_tags_12 },
    { 0x852ce53, SUC_lexicon_tags_9 },
    { 0x53f2ce52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf692ce5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf52ce60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe092ce62, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f2ce69, SUC_lexicon_tags_12 },
    { 0x6df2ce6a, SUC_lexicon_tags_14 },
    { 0x3bb2ce6b, SUC_lexicon_tags_37 },
    { 0x8ed2ce6c, SUC_lexicon_tags_53 },
    { 0x24f2ce6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x212ce6f, SUC_lexicon_tags_11 },
    { 0x2692ce70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d2ce73, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef2ce77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1d2ce79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa212ce7c, SUC_lexicon_tags_50 },
    { 0x5552ce7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad2ce80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b2ce84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e92ce88, SUC_lexicon_tags_38 },
    { 0x8052ce88, SUC_lexicon_tags_11 },
    { 0x3612ce8a, SUC_lexicon_tags_2 },
    { 0x4b72ce8b, SUC_lexicon_tags_16 },
    { 0xfb12ce88, SUC_lexicon_tags_63 },
    { 0xdf72ce8d, SUC_lexicon_tags_37 },
    { 0xaff2ce8d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d2ce98, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0x8792ce9a, SUC_lexicon_tags_15 },
    { 0x2f52ce9a, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xd492ce9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d2cea2, SUC_lexicon_tags_17 },
    { 0x1332cea2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6812cea5, SUC_lexicon_tags_63 },
    { 0x77f2cea6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c72cea8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad92ceaa, SUC_lexicon_tags_25 },
    { 0xde12ceab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ceae, SUC_lexicon_tags_87 },
    { 0xe4d2ceaf, SUC_lexicon_tags_60 },
    { 0x9f72ceaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7152ceb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x892ceb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8372ceb8, SUC_lexicon_tags_33 },
    { 0xc932ceb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b72cec1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd72cec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x252cec6, SUC_lexicon_tags_7 },
    { 0x75b2cec6, SUC_lexicon_tags_28 },
    { 0x2a92cec6, SUC_lexicon_tags_40 },
    { 0x8a32cec9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a32cecc, SUC_lexicon_tags_194 },
    { 0x55d2cecd, SUC_lexicon_tags_17 },
    { 0x4fd2cecd, SUC_lexicon_tags_15 },
    { 0xde32cecd, SUC_lexicon_tags_48 },
    { 0xf2b2cecd, SUC_lexicon_tags_38 },
    { 0x4072ced1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f2ced4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c32ced7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6792ced9, SUC_lexicon_tags_60 },
    { 0x4cf2ceda, SUC_lexicon_tags_22 },
    { 0xb8f2ceda, SUC_lexicon_tags_45 },
    { 0x5f52cedc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x472cedf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8812cee6, SUC_lexicon_tags_15 },
    { 0x6af2cee7, SUC_lexicon_tags_7 },
    { 0xf792cee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9092ceec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5892ceef, SUC_lexicon_tags_3 },
    { 0xb1b2ceef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2872cef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf2cef7, SUC_lexicon_tags_33 },
    { 0x3272cef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ab2cefa, SUC_lexicon_tags_63 },
    { 0x6b72cefb, SUC_lexicon_tags_16 },
    { 0xf292cefc, SUC_lexicon_tags_63 },
    { 0x9732cefc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b2cf01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb172cf03, SUC_lexicon_tags_11 },
    { 0x9b32cf03, SUC_lexicon_tags_29 },
    { 0xf992cf05, SUC_lexicon_tags_12 },
    { 0x6172cf05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf3b2cf08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f32cf11, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe732cf15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1932cf19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xba52cf1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7d32cf1d, SUC_lexicon_tags_10 },
    { 0xbd52cf1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1532cf22, SUC_lexicon_tags_8 },
    { 0x4952cf22, SUC_lexicon_tags_25 },
    { 0xa912cf24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6492cf27, SUC_lexicon_tags_33 },
    { 0x8452cf27, SUC_lexicon_tags_46 },
    { 0x4a52cf29, SUC_lexicon_tags_37 },
    { 0x5c12cf29, SUC_lexicon_tags_25 },
    { 0x8d2cf2b, SUC_lexicon_tags_25 },
    { 0xadd2cf2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd672cf2e, SUC_lexicon_tags_12 },
    { 0x29d2cf2f, SUC_lexicon_tags_17 },
    { 0xfd2cf2e, SUC_lexicon_tags_17 },
    { 0x1052cf31, SUC_lexicon_tags_47 },
    { 0x5ed2cf32, SUC_lexicon_tags_103 },
    { 0xdff2cf33, SUC_lexicon_tags_29 },
    { 0xe832cf30, SUC_lexicon_tags_26 },
    { 0x50f2cf2f, SUC_lexicon_tags_38 },
    { 0x92cf36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2bb2cf38, SUC_lexicon_tags_1 },
    { 0x5772cf38, SUC_lexicon_tags_45 },
    { 0x7e72cf3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x72f2cf3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef32cf3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5472cf40, SUC_lexicon_tags_27 },
    { 0xbbb2cf41, SUC_lexicon_tags_43 },
    { 0x11f2cf42, SUC_lexicon_tags_0 },
    { 0x5952cf40, SUC_lexicon_tags_11 },
    { 0xaeb2cf44, SUC_lexicon_tags_1 },
    { 0x66b2cf45, SUC_lexicon_tags_4 },
    { 0x1b92cf46, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff2cf4d, SUC_lexicon_tags_60 },
    { 0x7d32cf4e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5132cf50, SUC_lexicon_tags_11 },
    { 0x3fb2cf51, SUC_lexicon_tags_22 },
    { 0xf292cf51, SUC_lexicon_tags_0 },
    { 0xc192cf52, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d2cf57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x212cf5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b2cf62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d2cf66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5e12cf68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b32cf6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2112cf6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7272cf71, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf92cf74, SUC_lexicon_tags_2 },
    { 0x6d32cf75, SUC_lexicon_tags_47 },
    { 0x8252cf75, SUC_lexicon_tags_37 },
    { 0xcb92cf77, SUC_lexicon_tags_66 },
    { 0x30d2cf78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b2cf7f, SUC_lexicon_tags_25 },
    { 0x5412cf80, SUC_lexicon_tags_37 },
    { 0x74b2cf7f, SUC_lexicon_tags_9 },
    { 0x292cf82, SUC_lexicon_tags_2 },
    { 0xb3b2cf82, SUC_lexicon_tags_48 },
    { 0x5872cf82, SUC_lexicon_tags_15 },
    { 0xe6f2cf80, SUC_lexicon_tags_87 },
    { 0xbab2cf86, SUC_lexicon_tags_46 },
    { 0xe932cf86, SUC_lexicon_tags_17 },
    { 0x3b12cf7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8212cf8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9f2cf90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb472cf96, SUC_lexicon_tags_26 },
    { 0xec92cf97, SUC_lexicon_tags_107 },
    { 0x46b2cf98, SUC_lexicon_tags_57 },
    { 0x8eb2cf96, SUC_lexicon_tags_11 },
    { 0xe4f2cf9a, SUC_lexicon_tags_2 },
    { 0x9792cf97, SUC_lexicon_tags_60 },
    { 0xed52cf9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d2cfa3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfc32cfa5, SUC_lexicon_tags_37 },
    { 0xd2d2cfa5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8312cfa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb2cfad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9192cfaf, SUC_lexicon_tags_17 },
    { 0xb52cfaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2392cfb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf092cfb4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb0b2cfb6, SUC_lexicon_tags_56 },
    { 0xf212cfb6, SUC_lexicon_tags_28 },
    { 0x19b2cfb8, SUC_lexicon_tags_37 },
    { 0xcfd2cfb8, SUC_lexicon_tags_17 },
    { 0x11f2cfb9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b72cfbd, SUC_lexicon_tags_15 },
    { 0x20f2cfbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd72cfc1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb272cfc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f2cfc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf832cfc9, SUC_lexicon_tags_27 },
    { 0x5812cfca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b2cfd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9072cfd5, SUC_lexicon_tags_15 },
    { 0x2cd2cfd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x99d2cfd8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa752cfda, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa392cfdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x412cfdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a72cfe1, SUC_lexicon_tags_15 },
    { 0x9232cfe1, SUC_lexicon_tags_19 },
    { 0xfb2cfe1, SUC_lexicon_tags_33 },
    { 0xf6b2cfe2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcb2cfe6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf992cfe9, SUC_lexicon_tags_0 },
    { 0x9672cfe9, SUC_lexicon_tags_17 },
    { 0xcbd2cfe9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c12cfed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8832cfef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9252cff1, SUC_lexicon_tags_0 },
    { 0xc492cff1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb2cff7, SUC_lexicon_tags_45 },
    { 0x2a72cff8, SUC_lexicon_tags_37 },
    { 0xde72cff9, SUC_lexicon_tags_47 },
    { 0x45f2cff9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd32cffd, SUC_lexicon_tags_43 },
    { 0x67d2cffe, SUC_lexicon_tags_151 },
    { 0x81f2cfff, SUC_lexicon_tags_47 },
    { 0x6b12cffe, SUC_lexicon_tags_63 },
    { 0x2e52d001, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2d006, SUC_lexicon_tags_11 },
    { 0x6a12d006, SUC_lexicon_tags_29 },
    { 0x3252d007, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa912d00b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb332d00d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3292d00f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf92d012, SUC_lexicon_tags_31 },
    { 0xb772d013, SUC_lexicon_tags_15 },
    { 0x1632d013, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d32d01a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9772d01f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5392d021, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cb2d023, SUC_lexicon_tags_27 },
    { 0xee52d023, SUC_lexicon_tags_2 },
    { 0xbd2d025, SUC_lexicon_tags_23 },
    { 0x1172d026, SUC_lexicon_tags_9 },
    { 0xf432d027, SUC_lexicon_tags_10 },
    { 0x5672d027, SUC_lexicon_tags_14 },
    { 0x8e32d024, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb2d02e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f2d031, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe32d034, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa412d038, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2d03e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d2d042, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f2d046, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9652d048, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d12d04b, SUC_lexicon_tags_15 },
    { 0xeb52d04c, SUC_lexicon_tags_1 },
    { 0xe3b2d04d, SUC_lexicon_tags_37 },
    { 0x2cf2d04b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe52d053, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe732d059, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5152d05b, SUC_lexicon_tags_30 },
    { 0x8432d05b, SUC_lexicon_tags_21 },
    { 0x7fb2d05b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bf2d05f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda32d061, SUC_lexicon_tags_12 },
    { 0xf592d061, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc132d066, SUC_lexicon_tags_31 },
    { 0xf452d066, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb932d06c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd112d06f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6332d076, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a72d079, SUC_lexicon_tags_27 },
    { 0xb232d07a, SUC_lexicon_tags_19 },
    { 0xfd92d07a, SUC_lexicon_tags_1 },
    { 0x3632d07a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9892d080, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8712d083, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe52d088, SUC_lexicon_tags_2 },
    { 0x98d2d088, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b2d08d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed2d091, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c12d094, SUC_lexicon_tags_26 },
    { 0x14d2d094, SUC_lexicon_tags_25 },
    { 0xcbd2d096, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf32d09a, SUC_lexicon_tags_0 },
    { 0xd612d09b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x22f2d09d, SUC_lexicon_tags_47 },
    { 0x35f2d09e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9412d0a1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd2d0a4, SUC_lexicon_tags_17 },
    { 0xbc72d0a5, SUC_lexicon_tags_29 },
    { 0x932d0a6, SUC_lexicon_tags_49 },
    { 0x5c52d0a7, SUC_lexicon_tags_207 },
    { 0x2932d0a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d2d0ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6052d0b1, SUC_lexicon_tags_1 },
    { 0x38d2d0b2, SUC_lexicon_tags_46 },
    { 0x25f2d0b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb2d0b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe92d0b9, SUC_lexicon_tags_60 },
    { 0xd7f2d0b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c72d0c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db2d0c7, SUC_lexicon_tags_7 },
    { 0x792d0c7, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d2d0cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd2d0d1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c32d0d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d12d0d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b2d0da, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b72d0dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa472d0e2, SUC_lexicon_tags_82 },
    { 0xe672d0e3, SUC_lexicon_tags_46 },
    { 0x7252d0e4, SUC_lexicon_tags_49 },
    { 0xa172d0e3, SUC_lexicon_tags_12 },
    { 0xaf72d0e4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdef2d0e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592d0ec, SUC_lexicon_tags_6 },
    { 0x38d2d0ed, SUC_lexicon_tags_7 },
    { 0x7192d0ee, SUC_lexicon_tags_37 },
    { 0x92d0ef, SUC_lexicon_tags_28 },
    { 0xdf32d0f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe52d0f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a12d0fa, SUC_lexicon_tags_9 },
    { 0xc372d0fb, SUC_lexicon_tags_38 },
    { 0x86b2d0fb, SUC_lexicon_tags_3 },
    { 0x42f2d0fa, SUC_lexicon_tags_7 },
    { 0x7bb2d0fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe72d106, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9df2d108, SUC_lexicon_tags_47 },
    { 0x6b12d109, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f12d10d, SUC_lexicon_tags_8 },
    { 0x63f2d10e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc772d110, SUC_lexicon_tags_1 },
    { 0xcc12d111, SUC_lexicon_tags_7 },
    { 0x90b2d111, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8e52d114, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa52d117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c92d119, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5292d11b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf552d11d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b2d126, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8552d129, SUC_lexicon_tags_34 },
    { 0xa592d12a, SUC_lexicon_tags_15 },
    { 0x492d12a, SUC_lexicon_tags_27 },
    { 0x132d12c, SUC_lexicon_tags_45 },
    { 0x2632d12d, SUC_lexicon_tags_45 },
    { 0xfdd2d12e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf52d131, SUC_lexicon_tags_393 },
    { 0, NULL },
    { 0xd552d133, SUC_lexicon_tags_42 },
    { 0x58f2d133, SUC_lexicon_tags_27 },
    { 0xb72d134, SUC_lexicon_tags_49 },
    { 0x70f2d136, SUC_lexicon_tags_284 },
    { 0x852d133, SUC_lexicon_tags_14 },
    { 0x8872d138, SUC_lexicon_tags_55 },
    { 0xc3b2d139, SUC_lexicon_tags_25 },
    { 0x9612d139, SUC_lexicon_tags_14 },
    { 0xfa92d13b, SUC_lexicon_tags_60 },
    { 0xccf2d13b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa972d13e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x472d142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded2d148, SUC_lexicon_tags_17 },
    { 0xf592d148, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1472d14c, SUC_lexicon_tags_2 },
    { 0xa8f2d14d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5952d152, SUC_lexicon_tags_25 },
    { 0x7dd2d153, SUC_lexicon_tags_204 },
    { 0xb132d154, SUC_lexicon_tags_49 },
    { 0x4af2d152, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1892d159, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5d2d15b, SUC_lexicon_tags_100 },
    { 0x1032d15c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e92d15e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa812d165, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf2d16d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8eb2d16f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe332d174, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a32d177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xded2d179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7032d17b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32d2d17d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2692d185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x872d187, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24d2d189, SUC_lexicon_tags_1 },
    { 0xfb12d189, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x5b2d18c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad2d191, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9692d197, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f2d19b, SUC_lexicon_tags_11 },
    { 0x6872d19c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5492d1a2, SUC_lexicon_tags_25 },
    { 0x3952d1a3, SUC_lexicon_tags_49 },
    { 0xb192d1a4, SUC_lexicon_tags_68 },
    { 0x4c52d1a5, SUC_lexicon_tags_25 },
    { 0xe7d2d1a3, SUC_lexicon_tags_17 },
    { 0xb352d1a6, SUC_lexicon_tags_2 },
    { 0x53d2d1a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2412d1ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f2d1af, SUC_lexicon_tags_11 },
    { 0x1e12d1af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1572d1b2, SUC_lexicon_tags_7 },
    { 0x96d2d1b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd2d1b8, SUC_lexicon_tags_2 },
    { 0x5b72d1b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff2d1bc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5992d1be, SUC_lexicon_tags_1 },
    { 0x1eb2d1bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d72d1c1, SUC_lexicon_tags_17 },
    { 0x4852d1c1, SUC_lexicon_tags_15 },
    { 0xe352d1c3, SUC_lexicon_tags_11 },
    { 0xeab2d1c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd112d1c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6b2d1c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb2d1cb, SUC_lexicon_tags_38 },
    { 0x1a32d1cc, SUC_lexicon_tags_11 },
    { 0x7ad2d1cc, SUC_lexicon_tags_34 },
    { 0xaeb2d1cb, SUC_lexicon_tags_37 },
    { 0x3812d1cd, SUC_lexicon_tags_11 },
    { 0x44b2d1d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7732d1d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b2d1d6, SUC_lexicon_tags_27 },
    { 0xe72d1d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5092d1d9, SUC_lexicon_tags_62 },
    { 0x4c12d1da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8852d1dd, SUC_lexicon_tags_25 },
    { 0xd132d1dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa292d1e0, SUC_lexicon_tags_12 },
    { 0x93d2d1e0, SUC_lexicon_tags_26 },
    { 0xde32d1e1, SUC_lexicon_tags_43 },
    { 0x4152d1e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2af2d1e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa052d1e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1172d1ed, SUC_lexicon_tags_9 },
    { 0x9392d1ee, SUC_lexicon_tags_70 },
    { 0xd872d1ef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd152d1f2, SUC_lexicon_tags_37 },
    { 0xa652d1f3, SUC_lexicon_tags_605 },
    { 0xb9f2d1f3, SUC_lexicon_tags_6 },
    { 0xa9f2d1f4, SUC_lexicon_tags_47 },
    { 0xee32d1f6, SUC_lexicon_tags_25 },
    { 0xe6b2d1f4, SUC_lexicon_tags_17 },
    { 0x6632d1f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc052d1fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4f2d1fd, SUC_lexicon_tags_22 },
    { 0x4992d1fd, SUC_lexicon_tags_15 },
    { 0x42b2d1fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc772d201, SUC_lexicon_tags_17 },
    { 0xb812d201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b2d204, SUC_lexicon_tags_45 },
    { 0x79d2d205, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0b2d207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cd2d209, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b2d20e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f2d210, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf52d218, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe592d220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b2d223, SUC_lexicon_tags_7 },
    { 0x3af2d224, SUC_lexicon_tags_31 },
    { 0x2ad2d223, SUC_lexicon_tags_25 },
    { 0x25b2d226, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa432d22e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2572d233, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bb2d235, SUC_lexicon_tags_9 },
    { 0xda52d235, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0b2d238, SUC_lexicon_tags_28 },
    { 0x3212d239, SUC_lexicon_tags_1 },
    { 0xb292d23a, SUC_lexicon_tags_17 },
    { 0xef92d23b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd932d23f, SUC_lexicon_tags_38 },
    { 0xeb52d240, SUC_lexicon_tags_47 },
    { 0xda92d241, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f12d243, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2d247, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df2d24b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x152d24d, SUC_lexicon_tags_37 },
    { 0x6512d24e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23d2d250, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc112d252, SUC_lexicon_tags_207 },
    { 0x19d2d253, SUC_lexicon_tags_25 },
    { 0x1c52d254, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8132d257, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa812d259, SUC_lexicon_tags_11 },
    { 0xd572d25a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5112d25e, SUC_lexicon_tags_47 },
    { 0x6d2d25f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2492d262, SUC_lexicon_tags_2 },
    { 0x1d12d263, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde92d266, SUC_lexicon_tags_1 },
    { 0xf152d267, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ad2d269, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d2d26e, SUC_lexicon_tags_1 },
    { 0x92f2d26e, SUC_lexicon_tags_17 },
    { 0xac72d270, SUC_lexicon_tags_28 },
    { 0xc612d270, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd72d275, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf92d278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1552d27b, SUC_lexicon_tags_11 },
    { 0x30f2d27c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd0f2d27e, SUC_lexicon_tags_41 },
    { 0xd112d27f, SUC_lexicon_tags_12 },
    { 0x1eb2d280, SUC_lexicon_tags_37 },
    { 0xa332d27e, SUC_lexicon_tags_14 },
    { 0x4392d282, SUC_lexicon_tags_16 },
    { 0xc652d282, SUC_lexicon_tags_14 },
    { 0xa952d284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1092d286, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df2d28a, SUC_lexicon_tags_33 },
    { 0x1ab2d28a, SUC_lexicon_tags_46 },
    { 0x4c12d28b, SUC_lexicon_tags_11 },
    { 0x7372d28d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4232d28f, SUC_lexicon_tags_17 },
    { 0x24f2d290, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b2d299, SUC_lexicon_tags_68 },
    { 0x9ad2d299, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8972d2a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a12d2aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd2d2b4, SUC_lexicon_tags_49 },
    { 0x8852d2b5, SUC_lexicon_tags_27 },
    { 0xe2f2d2b5, SUC_lexicon_tags_43 },
    { 0xf072d2b5, SUC_lexicon_tags_25 },
    { 0x9a92d2b5, SUC_lexicon_tags_27 },
    { 0x43d2d2b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b92d2bd, SUC_lexicon_tags_15 },
    { 0xb0f2d2bd, SUC_lexicon_tags_1 },
    { 0x3772d2bd, SUC_lexicon_tags_15 },
    { 0x4152d2bd, SUC_lexicon_tags_33 },
    { 0xef32d2c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b12d2c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73f2d2c9, SUC_lexicon_tags_44 },
    { 0xe492d2ca, SUC_lexicon_tags_14 },
    { 0x8172d2c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc572d2ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2d2d2d0, SUC_lexicon_tags_2 },
    { 0x76b2d2d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xff12d2d3, SUC_lexicon_tags_31 },
    { 0x1a12d2d4, SUC_lexicon_tags_25 },
    { 0x40b2d2d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee52d2d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf2d2de, SUC_lexicon_tags_27 },
    { 0x1832d2de, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe4f2d2e1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7272d2e3, SUC_lexicon_tags_39 },
    { 0x6812d2e4, SUC_lexicon_tags_63 },
    { 0xb72d2e5, SUC_lexicon_tags_1 },
    { 0x2e12d2e6, SUC_lexicon_tags_60 },
    { 0x6dd2d2e7, SUC_lexicon_tags_11 },
    { 0xe4f2d2e8, SUC_lexicon_tags_11 },
    { 0xe632d2e9, SUC_lexicon_tags_1 },
    { 0x7db2d2ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d2d2f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf352d2f4, SUC_lexicon_tags_25 },
    { 0xb92d2f4, SUC_lexicon_tags_16 },
    { 0x2c72d2f6, SUC_lexicon_tags_1 },
    { 0xcf12d2f7, SUC_lexicon_tags_26 },
    { 0x8f2d2f8, SUC_lexicon_tags_17 },
    { 0x7b32d2f6, SUC_lexicon_tags_12 },
    { 0xeb92d2f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd272d2fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5452d300, SUC_lexicon_tags_11 },
    { 0xf712d301, SUC_lexicon_tags_66 },
    { 0xbfd2d300, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e72d304, SUC_lexicon_tags_25 },
    { 0xd1b2d305, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd332d308, SUC_lexicon_tags_12 },
    { 0x2632d309, SUC_lexicon_tags_5 },
    { 0xeb2d30a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb52d30c, SUC_lexicon_tags_9 },
    { 0x1832d30d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2f2d30f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f2d313, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53f2d315, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f2d318, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b2d320, SUC_lexicon_tags_63 },
    { 0xc3b2d321, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef2d324, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf832d326, SUC_lexicon_tags_9 },
    { 0x4d52d326, SUC_lexicon_tags_15 },
    { 0x1b32d328, SUC_lexicon_tags_154 },
    { 0x7152d329, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2952d331, SUC_lexicon_tags_2 },
    { 0x2fd2d332, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf412d336, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f2d33a, SUC_lexicon_tags_56 },
    { 0x4612d33b, SUC_lexicon_tags_91 },
    { 0xe012d33c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd612d33e, SUC_lexicon_tags_15 },
    { 0xaab2d33f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8232d346, SUC_lexicon_tags_25 },
    { 0x6512d347, SUC_lexicon_tags_25 },
    { 0x2712d347, SUC_lexicon_tags_37 },
    { 0xb2d2d346, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb352d34f, SUC_lexicon_tags_45 },
    { 0xb3b2d350, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6ad2d352, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2d358, SUC_lexicon_tags_3 },
    { 0xfd32d359, SUC_lexicon_tags_2 },
    { 0xf1b2d35a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56b2d35c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b72d361, SUC_lexicon_tags_26 },
    { 0x49f2d361, SUC_lexicon_tags_5 },
    { 0xb7b2d363, SUC_lexicon_tags_10 },
    { 0x6752d363, SUC_lexicon_tags_8 },
    { 0x9c32d364, SUC_lexicon_tags_27 },
    { 0x8cb2d366, SUC_lexicon_tags_60 },
    { 0x5812d367, SUC_lexicon_tags_37 },
    { 0x6cd2d364, SUC_lexicon_tags_27 },
    { 0xec52d365, SUC_lexicon_tags_85 },
    { 0xa5b2d36a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf072d372, SUC_lexicon_tags_29 },
    { 0xcf12d373, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4392d376, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf0b2d378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c72d37c, SUC_lexicon_tags_23 },
    { 0x3cf2d37d, SUC_lexicon_tags_37 },
    { 0x68b2d37d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab2d381, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2432d383, SUC_lexicon_tags_47 },
    { 0x9872d384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4632d386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3992d38c, SUC_lexicon_tags_2 },
    { 0x7ad2d38c, SUC_lexicon_tags_12 },
    { 0xf512d38c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d2d391, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb192d395, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51f2d397, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9832d399, SUC_lexicon_tags_2 },
    { 0x9f92d39a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73d2d39c, SUC_lexicon_tags_11 },
    { 0xd032d39c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe772d3a1, SUC_lexicon_tags_17 },
    { 0x9e52d3a2, SUC_lexicon_tags_37 },
    { 0x7b92d3a1, SUC_lexicon_tags_47 },
    { 0x2eb2d3a1, SUC_lexicon_tags_12 },
    { 0x5652d3a5, SUC_lexicon_tags_37 },
    { 0xd1d2d3a5, SUC_lexicon_tags_11 },
    { 0x9b72d3a7, SUC_lexicon_tags_25 },
    { 0xb692d3a1, SUC_lexicon_tags_27 },
    { 0xa172d3a9, SUC_lexicon_tags_2 },
    { 0x8bf2d3a9, SUC_lexicon_tags_1 },
    { 0x1892d3ab, SUC_lexicon_tags_32 },
    { 0xdd32d3ac, SUC_lexicon_tags_25 },
    { 0x83d2d3ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97f2d3af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9712d3b2, SUC_lexicon_tags_0 },
    { 0x6e72d3b3, SUC_lexicon_tags_25 },
    { 0x76b2d3b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d12d3b7, SUC_lexicon_tags_15 },
    { 0xa5f2d3b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd52d3ba, SUC_lexicon_tags_37 },
    { 0xdfd2d3bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x67f2d3bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d52d3c0, SUC_lexicon_tags_17 },
    { 0x3a12d3c1, SUC_lexicon_tags_43 },
    { 0xb032d3c2, SUC_lexicon_tags_37 },
    { 0x54d2d3c3, SUC_lexicon_tags_1 },
    { 0xd592d3c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe692d3c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b72d3c8, SUC_lexicon_tags_54 },
    { 0x8b72d3c9, SUC_lexicon_tags_14 },
    { 0xf852d3ca, SUC_lexicon_tags_34 },
    { 0xeb32d3c9, SUC_lexicon_tags_49 },
    { 0xf552d3ca, SUC_lexicon_tags_7 },
    { 0x9332d3cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc792d3d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b52d3d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bd2d3da, SUC_lexicon_tags_25 },
    { 0x2fb2d3db, SUC_lexicon_tags_49 },
    { 0xb92d3dc, SUC_lexicon_tags_27 },
    { 0x8cd2d3dd, SUC_lexicon_tags_48 },
    { 0x6052d3dd, SUC_lexicon_tags_34 },
    { 0xaa12d3db, SUC_lexicon_tags_62 },
    { 0xc7b2d3df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b2d3e6, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x4552d3e8, SUC_lexicon_tags_14 },
    { 0x3412d3e9, SUC_lexicon_tags_47 },
    { 0x4d72d3e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5e72d3ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb172d3ee, SUC_lexicon_tags_11 },
    { 0xbb2d3ee, SUC_lexicon_tags_107 },
    { 0x1eb2d3f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab2d3f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e12d3f5, SUC_lexicon_tags_34 },
    { 0xd572d3f6, SUC_lexicon_tags_17 },
    { 0x4352d3f6, SUC_lexicon_tags_2 },
    { 0xb952d3f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e52d3fc, SUC_lexicon_tags_1 },
    { 0x1b2d3fd, SUC_lexicon_tags_5 },
    { 0xae32d3fc, SUC_lexicon_tags_25 },
    { 0xd7f2d3ff, SUC_lexicon_tags_2 },
    { 0x8512d3fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8132d403, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd8d2d405, SUC_lexicon_tags_45 },
    { 0x4712d406, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x23b2d408, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb2d40d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad2d411, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df2d414, SUC_lexicon_tags_28 },
    { 0x88d2d415, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b32d419, SUC_lexicon_tags_8 },
    { 0xf3f2d41a, SUC_lexicon_tags_47 },
    { 0x4f2d419, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd852d41e, SUC_lexicon_tags_25 },
    { 0x4fb2d41e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd8d2d421, SUC_lexicon_tags_54 },
    { 0x1152d421, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa192d428, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f2d42c, SUC_lexicon_tags_2 },
    { 0x6d2d42d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x84f2d42f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e92d433, SUC_lexicon_tags_11 },
    { 0xacf2d433, SUC_lexicon_tags_3 },
    { 0xc232d435, SUC_lexicon_tags_7 },
    { 0xd732d435, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc72d439, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf2d441, SUC_lexicon_tags_28 },
    { 0x89b2d442, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8772d445, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc112d44b, SUC_lexicon_tags_8 },
    { 0xca12d44c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf452d44e, SUC_lexicon_tags_12 },
    { 0xedd2d44f, SUC_lexicon_tags_1 },
    { 0x37d2d44e, SUC_lexicon_tags_37 },
    { 0x9252d451, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3752d457, SUC_lexicon_tags_25 },
    { 0xb5d2d457, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3792d45e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x57d2d460, SUC_lexicon_tags_17 },
    { 0x93b2d461, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1192d465, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd012d46b, SUC_lexicon_tags_17 },
    { 0xf9b2d46b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf412d475, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2352d478, SUC_lexicon_tags_7 },
    { 0xb3b2d478, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef2d47c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd052d47f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb92d483, SUC_lexicon_tags_14 },
    { 0xfaf2d484, SUC_lexicon_tags_140 },
    { 0x7ed2d485, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e32d487, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7cd2d489, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcad2d48b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd392d48d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f2d495, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf412d498, SUC_lexicon_tags_11 },
    { 0x9bb2d498, SUC_lexicon_tags_7 },
    { 0x73f2d49a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc832d49f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54f2d4a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb092d4a5, SUC_lexicon_tags_38 },
    { 0xe612d4a6, SUC_lexicon_tags_2 },
    { 0xb8f2d4a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b2d4ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b2d4b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f2d4b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc32d4ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb2d4be, SUC_lexicon_tags_23 },
    { 0xa7d2d4be, SUC_lexicon_tags_38 },
    { 0xc012d4bf, SUC_lexicon_tags_25 },
    { 0xfef2d4c1, SUC_lexicon_tags_1 },
    { 0x7552d4c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe32d4c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4892d4c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x88b2d4c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb52d4ca, SUC_lexicon_tags_12 },
    { 0xeb72d4ca, SUC_lexicon_tags_106 },
    { 0x5932d4cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd2d4d3, SUC_lexicon_tags_11 },
    { 0x8592d4d4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9392d4db, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf652d4e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f2d4e7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x81d2d4e9, SUC_lexicon_tags_37 },
    { 0x2ed2d4e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b2d4ec, SUC_lexicon_tags_12 },
    { 0x3812d4ec, SUC_lexicon_tags_2 },
    { 0x6272d4ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b2d4f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2112d4f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde92d4fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84d2d500, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x26b2d502, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3172d506, SUC_lexicon_tags_49 },
    { 0x16b2d506, SUC_lexicon_tags_7 },
    { 0x2d52d506, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9512d50d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2372d516, SUC_lexicon_tags_2 },
    { 0xcfd2d516, SUC_lexicon_tags_17 },
    { 0x4cf2d518, SUC_lexicon_tags_25 },
    { 0x54f2d517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa812d51b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d2d524, SUC_lexicon_tags_37 },
    { 0x8772d524, SUC_lexicon_tags_15 },
    { 0xe612d526, SUC_lexicon_tags_11 },
    { 0x30f2d527, SUC_lexicon_tags_16 },
    { 0x4b32d526, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4f32d52a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfdf2d52c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f2d52f, SUC_lexicon_tags_45 },
    { 0xb692d530, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7cb2d532, SUC_lexicon_tags_11 },
    { 0xd492d533, SUC_lexicon_tags_12 },
    { 0x9872d533, SUC_lexicon_tags_82 },
    { 0x75d2d535, SUC_lexicon_tags_30 },
    { 0x4172d536, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd12d539, SUC_lexicon_tags_28 },
    { 0xc4b2d53a, SUC_lexicon_tags_43 },
    { 0x2c12d53b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c52d540, SUC_lexicon_tags_1 },
    { 0x4d52d541, SUC_lexicon_tags_47 },
    { 0x4292d542, SUC_lexicon_tags_19 },
    { 0x9572d543, SUC_lexicon_tags_37 },
    { 0xbf92d543, SUC_lexicon_tags_19 },
    { 0xdb2d541, SUC_lexicon_tags_39 },
    { 0xaf32d546, SUC_lexicon_tags_11 },
    { 0xb7d2d546, SUC_lexicon_tags_69 },
    { 0xa952d548, SUC_lexicon_tags_14 },
    { 0xf412d549, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabd2d54b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc732d54f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b2d553, SUC_lexicon_tags_28 },
    { 0x9f32d554, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf2d557, SUC_lexicon_tags_43 },
    { 0x9e92d557, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb712d562, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2072d565, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf892d568, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db2d570, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc272d573, SUC_lexicon_tags_16 },
    { 0x7df2d573, SUC_lexicon_tags_19 },
    { 0xff92d575, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b2d578, SUC_lexicon_tags_61 },
    { 0x4df2d579, SUC_lexicon_tags_7 },
    { 0xfb2d57a, SUC_lexicon_tags_7 },
    { 0xd5b2d578, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe612d580, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed2d586, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe4b2d588, SUC_lexicon_tags_37 },
    { 0x5032d588, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd092d58e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8d2d590, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1432d599, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c32d5ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5232d5af, SUC_lexicon_tags_11 },
    { 0xc752d5b0, SUC_lexicon_tags_37 },
    { 0x9dd2d5b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7812d5b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb2d5b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9352d5ba, SUC_lexicon_tags_43 },
    { 0x8832d5bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2d5be, SUC_lexicon_tags_25 },
    { 0x72d2d5bf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf912d5c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab72d5c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8772d5cc, SUC_lexicon_tags_47 },
    { 0x5332d5cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb2d5d0, SUC_lexicon_tags_15 },
    { 0xfa92d5d1, SUC_lexicon_tags_18 },
    { 0xc7f2d5d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2612d5d4, SUC_lexicon_tags_15 },
    { 0xf172d5d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b2d5d9, SUC_lexicon_tags_84 },
    { 0xe9d2d5d9, SUC_lexicon_tags_11 },
    { 0x27d2d5db, SUC_lexicon_tags_1 },
    { 0x64d2d5da, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd992d5de, SUC_lexicon_tags_12 },
    { 0x9f12d5df, SUC_lexicon_tags_11 },
    { 0xe6d2d5e0, SUC_lexicon_tags_39 },
    { 0xd832d5e1, SUC_lexicon_tags_17 },
    { 0xb452d5df, SUC_lexicon_tags_1 },
    { 0x3812d5e3, SUC_lexicon_tags_10 },
    { 0x9072d5e0, SUC_lexicon_tags_17 },
    { 0xa6d2d5e5, SUC_lexicon_tags_151 },
    { 0x4e12d5e5, SUC_lexicon_tags_572 },
    { 0xfc32d5e1, SUC_lexicon_tags_14 },
    { 0xccf2d5e0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xfc72d5ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9012d5ee, SUC_lexicon_tags_47 },
    { 0xbdd2d5ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae52d5f3, SUC_lexicon_tags_84 },
    { 0xfa72d5f3, SUC_lexicon_tags_194 },
    { 0x6552d5f5, SUC_lexicon_tags_63 },
    { 0xc6f2d5f5, SUC_lexicon_tags_47 },
    { 0x14b2d5f7, SUC_lexicon_tags_47 },
    { 0x8672d5f5, SUC_lexicon_tags_37 },
    { 0x6272d5f9, SUC_lexicon_tags_29 },
    { 0x8a72d5fa, SUC_lexicon_tags_37 },
    { 0xb1b2d5f9, SUC_lexicon_tags_8 },
    { 0x6fd2d5fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5ef2d5fe, SUC_lexicon_tags_66 },
    { 0x8db2d5ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd2d603, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1172d606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2d60c, SUC_lexicon_tags_30 },
    { 0xaf72d60d, SUC_lexicon_tags_47 },
    { 0x42b2d60e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3292d610, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30b2d612, SUC_lexicon_tags_30 },
    { 0x8212d613, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f2d616, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb892d61c, SUC_lexicon_tags_49 },
    { 0xab12d61c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf572d61f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xec52d621, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf32d624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b12d626, SUC_lexicon_tags_1 },
    { 0x6252d626, SUC_lexicon_tags_14 },
    { 0xc9d2d628, SUC_lexicon_tags_2 },
    { 0x1112d629, SUC_lexicon_tags_25 },
    { 0xd892d62a, SUC_lexicon_tags_23 },
    { 0x9872d629, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a52d631, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f2d635, SUC_lexicon_tags_146 },
    { 0xbcf2d636, SUC_lexicon_tags_17 },
    { 0xea32d637, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd872d63c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3672d63f, SUC_lexicon_tags_7 },
    { 0xb4d2d640, SUC_lexicon_tags_29 },
    { 0x8552d641, SUC_lexicon_tags_14 },
    { 0xe1f2d642, SUC_lexicon_tags_34 },
    { 0xf1d2d640, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ed2d645, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2bf2d647, SUC_lexicon_tags_63 },
    { 0xfed2d648, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c92d64b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x46f2d64d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf772d651, SUC_lexicon_tags_12 },
    { 0x4412d651, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f2d657, SUC_lexicon_tags_17 },
    { 0xb9d2d658, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1052d65a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc072d65c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7532d65f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe612d664, SUC_lexicon_tags_65 },
    { 0x6ad2d665, SUC_lexicon_tags_19 },
    { 0x992d666, SUC_lexicon_tags_25 },
    { 0x2b12d666, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x5432d66a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc432d66c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeef2d66e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1812d676, SUC_lexicon_tags_7 },
    { 0x6472d677, SUC_lexicon_tags_6 },
    { 0xf352d677, SUC_lexicon_tags_25 },
    { 0xabf2d679, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc12d67b, SUC_lexicon_tags_37 },
    { 0x29f2d67c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf752d67e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd2d684, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba72d688, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5472d68a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5032d68e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b2d691, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf12d693, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1e12d695, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd592d69d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4612d69f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef2d6a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d2d6a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd412d6ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbdf2d6ad, SUC_lexicon_tags_7 },
    { 0xd812d6ae, SUC_lexicon_tags_2 },
    { 0xf352d6ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d2d6b4, SUC_lexicon_tags_19 },
    { 0x79b2d6b5, SUC_lexicon_tags_70 },
    { 0xb172d6b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7112d6b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b2d6c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe892d6c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb2d6c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e72d6c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb2d6cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd852d6cf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf212d6d1, SUC_lexicon_tags_2 },
    { 0x3ff2d6d1, SUC_lexicon_tags_11 },
    { 0x7052d6d3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa432d6d5, SUC_lexicon_tags_25 },
    { 0xdeb2d6d6, SUC_lexicon_tags_11 },
    { 0xe2f2d6d7, SUC_lexicon_tags_25 },
    { 0xbe52d6d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b2d6db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfc32d6dd, SUC_lexicon_tags_37 },
    { 0x252d6de, SUC_lexicon_tags_37 },
    { 0x2652d6dd, SUC_lexicon_tags_25 },
    { 0xf7f2d6e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f52d6e5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb52d6e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5032d6f3, SUC_lexicon_tags_65 },
    { 0xc32d6f3, SUC_lexicon_tags_11 },
    { 0x1532d6f5, SUC_lexicon_tags_9 },
    { 0xcf92d6f5, SUC_lexicon_tags_17 },
    { 0x6ef2d6f7, SUC_lexicon_tags_12 },
    { 0x9d32d6f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f92d6fc, SUC_lexicon_tags_2 },
    { 0x832d6fd, SUC_lexicon_tags_12 },
    { 0x2c52d6fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b2d701, SUC_lexicon_tags_37 },
    { 0x3372d702, SUC_lexicon_tags_82 },
    { 0xad2d703, SUC_lexicon_tags_0 },
    { 0x6f32d703, SUC_lexicon_tags_23 },
    { 0xc352d705, SUC_lexicon_tags_65 },
    { 0x19d2d701, SUC_lexicon_tags_27 },
    { 0xb012d707, SUC_lexicon_tags_27 },
    { 0xfd32d708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5392d70a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9f2d70c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6992d70f, SUC_lexicon_tags_37 },
    { 0x3f12d70f, SUC_lexicon_tags_1560 },
    { 0x7e72d711, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda32d717, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc12d719, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9792d71d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b2d720, SUC_lexicon_tags_19 },
    { 0x2f2d720, SUC_lexicon_tags_16 },
    { 0x1bd2d721, SUC_lexicon_tags_21 },
    { 0x7e12d723, SUC_lexicon_tags_32 },
    { 0x6df2d723, SUC_lexicon_tags_10 },
    { 0x4bb2d724, SUC_lexicon_tags_11 },
    { 0x7772d722, SUC_lexicon_tags_13 },
    { 0x93f2d727, SUC_lexicon_tags_4 },
    { 0x38d2d724, SUC_lexicon_tags_63 },
    { 0xa552d720, SUC_lexicon_tags_49 },
    { 0xe52d72a, SUC_lexicon_tags_13 },
    { 0xc792d721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d2d731, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x92d2d733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b2d73a, SUC_lexicon_tags_25 },
    { 0x752d73a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb92d73d, SUC_lexicon_tags_33 },
    { 0x9492d73e, SUC_lexicon_tags_12 },
    { 0xe7d2d73f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b92d741, SUC_lexicon_tags_49 },
    { 0xe4d2d742, SUC_lexicon_tags_26 },
    { 0x5072d743, SUC_lexicon_tags_37 },
    { 0xec92d744, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2532d74c, SUC_lexicon_tags_1010 },
    { 0x5e32d74d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc52d751, SUC_lexicon_tags_15 },
    { 0xe772d751, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa12d75c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1492d75e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2832d760, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6c12d762, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f2d765, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1592d767, SUC_lexicon_tags_31 },
    { 0xcc12d768, SUC_lexicon_tags_19 },
    { 0x9a52d769, SUC_lexicon_tags_46 },
    { 0xa012d769, SUC_lexicon_tags_17 },
    { 0x2652d769, SUC_lexicon_tags_37 },
    { 0xa9b2d76c, SUC_lexicon_tags_17 },
    { 0x312d76d, SUC_lexicon_tags_57 },
    { 0x7e52d76d, SUC_lexicon_tags_13 },
    { 0xd5d2d76f, SUC_lexicon_tags_10 },
    { 0x6bb2d770, SUC_lexicon_tags_61 },
    { 0xccd2d76f, SUC_lexicon_tags_21 },
    { 0x1572d772, SUC_lexicon_tags_15 },
    { 0xb2b2d76e, SUC_lexicon_tags_3 },
    { 0xebf2d76c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c32d77a, SUC_lexicon_tags_33 },
    { 0x3712d77b, SUC_lexicon_tags_1 },
    { 0xb612d77c, SUC_lexicon_tags_1 },
    { 0x7252d77b, SUC_lexicon_tags_8 },
    { 0x71f2d77e, SUC_lexicon_tags_43 },
    { 0xaab2d77b, SUC_lexicon_tags_37 },
    { 0xbd12d77a, SUC_lexicon_tags_39 },
    { 0x88d2d781, SUC_lexicon_tags_19 },
    { 0x2a12d782, SUC_lexicon_tags_2 },
    { 0xe4f2d783, SUC_lexicon_tags_82 },
    { 0x2672d780, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df2d788, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bf2d78a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5472d78d, SUC_lexicon_tags_11 },
    { 0xd812d78d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea92d793, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d32d796, SUC_lexicon_tags_17 },
    { 0xcb12d796, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d2d79c, SUC_lexicon_tags_55 },
    { 0x3792d79d, SUC_lexicon_tags_25 },
    { 0xf5b2d79e, SUC_lexicon_tags_14 },
    { 0x3d92d79c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a92d7a1, SUC_lexicon_tags_62 },
    { 0xca12d7a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4432d7a7, SUC_lexicon_tags_17 },
    { 0x2532d7a8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb12d7ae, SUC_lexicon_tags_9 },
    { 0x96d2d7af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac92d7b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2d7b6, SUC_lexicon_tags_7 },
    { 0x9f2d7b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc012d7ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71d2d7bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6692d7c4, SUC_lexicon_tags_37 },
    { 0x63f2d7c5, SUC_lexicon_tags_13 },
    { 0xee32d7c6, SUC_lexicon_tags_255 },
    { 0xd32d7c7, SUC_lexicon_tags_25 },
    { 0xc312d7c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x18b2d7ca, SUC_lexicon_tags_37 },
    { 0xf952d7cb, SUC_lexicon_tags_15 },
    { 0x9d72d7cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xebb2d7ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b2d7d8, SUC_lexicon_tags_23 },
    { 0xdef2d7d9, SUC_lexicon_tags_14 },
    { 0xb412d7da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7af2d7dc, SUC_lexicon_tags_27 },
    { 0x9972d7dd, SUC_lexicon_tags_38 },
    { 0x5592d7dc, SUC_lexicon_tags_7 },
    { 0x70d2d7de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df2d7e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3bf2d7e5, SUC_lexicon_tags_11 },
    { 0xe6f2d7e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6012d7e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac72d7ed, SUC_lexicon_tags_13 },
    { 0x93d2d7ee, SUC_lexicon_tags_12 },
    { 0xdcb2d7ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6112d7f1, SUC_lexicon_tags_30 },
    { 0xb752d7f2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff2d7f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed2d7fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6dd2d7fc, SUC_lexicon_tags_16 },
    { 0x3f2d7fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac12d806, SUC_lexicon_tags_2 },
    { 0xe252d806, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x72b2d809, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1072d80b, SUC_lexicon_tags_37 },
    { 0x27f2d80c, SUC_lexicon_tags_7 },
    { 0x8d92d80d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b92d815, SUC_lexicon_tags_60 },
    { 0xd752d816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5512d81a, SUC_lexicon_tags_30 },
    { 0xc12d81b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf2d821, SUC_lexicon_tags_14 },
    { 0x2212d822, SUC_lexicon_tags_48 },
    { 0xebf2d823, SUC_lexicon_tags_17 },
    { 0x8a72d821, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec52d828, SUC_lexicon_tags_39 },
    { 0x5ed2d828, SUC_lexicon_tags_19 },
    { 0xda12d828, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9652d82d, SUC_lexicon_tags_1 },
    { 0x1372d82d, SUC_lexicon_tags_63 },
    { 0xe8d2d82f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f2d832, SUC_lexicon_tags_235 },
    { 0x4d92d833, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb2d836, SUC_lexicon_tags_38 },
    { 0xd5f2d837, SUC_lexicon_tags_25 },
    { 0xdb32d837, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f2d83b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad12d842, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd52d844, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x95b2d846, SUC_lexicon_tags_7 },
    { 0xe5f2d847, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd092d849, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc072d852, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9192d859, SUC_lexicon_tags_27 },
    { 0x27b2d859, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb5f2d85c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e72d863, SUC_lexicon_tags_33 },
    { 0x672d864, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf2d86c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2432d870, SUC_lexicon_tags_3 },
    { 0xb532d871, SUC_lexicon_tags_23 },
    { 0x8452d871, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8312d875, SUC_lexicon_tags_17 },
    { 0xe432d875, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2312d879, SUC_lexicon_tags_17 },
    { 0x35b2d87a, SUC_lexicon_tags_7 },
    { 0x74f2d87a, SUC_lexicon_tags_1 },
    { 0x7b52d879, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2d880, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbd2d882, SUC_lexicon_tags_2 },
    { 0xa552d882, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8432d886, SUC_lexicon_tags_7 },
    { 0xfd72d887, SUC_lexicon_tags_25 },
    { 0x2cb2d888, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d2d894, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9912d899, SUC_lexicon_tags_25 },
    { 0x93d2d89a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7472d89d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b2d8a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef52d8a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e72d8a6, SUC_lexicon_tags_30 },
    { 0x5bb2d8a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd352d8a9, SUC_lexicon_tags_44 },
    { 0x5572d8aa, SUC_lexicon_tags_7 },
    { 0xd732d8aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf72d8ae, SUC_lexicon_tags_11 },
    { 0xf992d8af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5d2d8b1, SUC_lexicon_tags_7 },
    { 0x6cb2d8b2, SUC_lexicon_tags_12 },
    { 0x4ed2d8b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c32d8b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf592d8ba, SUC_lexicon_tags_23 },
    { 0xc932d8ba, SUC_lexicon_tags_29 },
    { 0xf312d8bc, SUC_lexicon_tags_7 },
    { 0x41d2d8bd, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xbbf2d8bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae52d8c4, SUC_lexicon_tags_37 },
    { 0x2092d8c5, SUC_lexicon_tags_12 },
    { 0xcf2d8c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b2d8cd, SUC_lexicon_tags_11 },
    { 0xcb92d8cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc52d8d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5212d8db, SUC_lexicon_tags_6 },
    { 0xda72d8dc, SUC_lexicon_tags_30 },
    { 0x872d8dc, SUC_lexicon_tags_17 },
    { 0x5152d8de, SUC_lexicon_tags_37 },
    { 0xa72d8df, SUC_lexicon_tags_25 },
    { 0x99b2d8df, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d32d8e3, SUC_lexicon_tags_15 },
    { 0x2ad2d8e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8d2d8e6, SUC_lexicon_tags_16 },
    { 0x1232d8e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7832d8ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf332d8ed, SUC_lexicon_tags_65 },
    { 0x2df2d8ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x452d8f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x52b2d8f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d72d8f8, SUC_lexicon_tags_0 },
    { 0x7912d8f8, SUC_lexicon_tags_17 },
    { 0x2c92d8fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ff2d8fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b2d8ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc492d906, SUC_lexicon_tags_29 },
    { 0x7572d906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc192d909, SUC_lexicon_tags_63 },
    { 0xbb32d90a, SUC_lexicon_tags_25 },
    { 0x5952d90a, SUC_lexicon_tags_37 },
    { 0x55b2d90c, SUC_lexicon_tags_16 },
    { 0x4b32d90c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b2d911, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b2d916, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x76b2d918, SUC_lexicon_tags_55 },
    { 0x6052d919, SUC_lexicon_tags_1 },
    { 0x4212d91a, SUC_lexicon_tags_160 },
    { 0x5652d91b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8872d91d, SUC_lexicon_tags_2 },
    { 0xc932d91e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52f2d920, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6272d924, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa672d926, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa152d928, SUC_lexicon_tags_11 },
    { 0x3c32d928, SUC_lexicon_tags_25 },
    { 0xef92d928, SUC_lexicon_tags_19 },
    { 0x3452d92b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4552d92f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd072d935, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89d2d937, SUC_lexicon_tags_17 },
    { 0xfd52d938, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9112d93f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7892d942, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d2d946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab2d94a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d2d950, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ab2d952, SUC_lexicon_tags_0 },
    { 0x2b92d953, SUC_lexicon_tags_23 },
    { 0xedb2d952, SUC_lexicon_tags_63 },
    { 0x6eb2d955, SUC_lexicon_tags_14 },
    { 0x19b2d956, SUC_lexicon_tags_30 },
    { 0xbf2d957, SUC_lexicon_tags_25 },
    { 0x39b2d958, SUC_lexicon_tags_38 },
    { 0xdc92d959, SUC_lexicon_tags_31 },
    { 0x8a72d95a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab52d95e, SUC_lexicon_tags_8 },
    { 0xbe52d95f, SUC_lexicon_tags_26 },
    { 0x3fb2d960, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xfbf2d962, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4012d964, SUC_lexicon_tags_25 },
    { 0xae72d965, SUC_lexicon_tags_68 },
    { 0x6e32d966, SUC_lexicon_tags_174 },
    { 0x65f2d966, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f2d96a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f2d96e, SUC_lexicon_tags_2 },
    { 0xa312d96f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b12d975, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5032d979, SUC_lexicon_tags_39 },
    { 0x1d72d979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb2d97e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe152d987, SUC_lexicon_tags_17 },
    { 0xb9d2d988, SUC_lexicon_tags_45 },
    { 0x7b32d988, SUC_lexicon_tags_11 },
    { 0x8732d987, SUC_lexicon_tags_15 },
    { 0xd4f2d98b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc32d994, SUC_lexicon_tags_4 },
    { 0xab92d995, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b2d99b, SUC_lexicon_tags_39 },
    { 0x1f2d99c, SUC_lexicon_tags_17 },
    { 0xac92d99b, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0xb2f2d99f, SUC_lexicon_tags_951 },
    { 0xba32d9a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6592d9a2, SUC_lexicon_tags_37 },
    { 0x6572d9a3, SUC_lexicon_tags_1 },
    { 0xb032d9a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6012d9a6, SUC_lexicon_tags_65 },
    { 0x88d2d9a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc9f2d9a9, SUC_lexicon_tags_26 },
    { 0xd8f2d9aa, SUC_lexicon_tags_2 },
    { 0x9cf2d9ab, SUC_lexicon_tags_17 },
    { 0x76f2d9ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d2d9af, SUC_lexicon_tags_7 },
    { 0xf832d9b0, SUC_lexicon_tags_11 },
    { 0x8612d9b0, SUC_lexicon_tags_83 },
    { 0x2ad2d9b2, SUC_lexicon_tags_25 },
    { 0x1e72d9b2, SUC_lexicon_tags_9 },
    { 0xb2d9b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc452d9b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b52d9b8, SUC_lexicon_tags_56 },
    { 0xcf2d9b9, SUC_lexicon_tags_23 },
    { 0x69d2d9ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2372d9be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4372d9c2, SUC_lexicon_tags_38 },
    { 0x592d9c2, SUC_lexicon_tags_12 },
    { 0xa4b2d9c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9792d9c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc32d9c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d2d9cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf352d9d6, SUC_lexicon_tags_80 },
    { 0xe192d9d7, SUC_lexicon_tags_11 },
    { 0xcf72d9d8, SUC_lexicon_tags_23 },
    { 0xd952d9d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3272d9df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9972d9e4, SUC_lexicon_tags_39 },
    { 0xc252d9e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3132d9e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f2d9eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e12d9f0, SUC_lexicon_tags_28 },
    { 0xa8b2d9f1, SUC_lexicon_tags_23 },
    { 0x68b2d9f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b92d9f4, SUC_lexicon_tags_14 },
    { 0xdf2d9f5, SUC_lexicon_tags_12 },
    { 0xe6d2d9f6, SUC_lexicon_tags_6 },
    { 0x8012d9f6, SUC_lexicon_tags_60 },
    { 0x912d9f5, SUC_lexicon_tags_37 },
    { 0x2012d9f9, SUC_lexicon_tags_15 },
    { 0x172d9fa, SUC_lexicon_tags_45 },
    { 0xf232d9fa, SUC_lexicon_tags_63 },
    { 0xa012d9f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d2d9ff, SUC_lexicon_tags_140 },
    { 0x6b32da00, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x32f2da02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6312da05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb2da0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6832da0d, SUC_lexicon_tags_25 },
    { 0x53f2da0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xced2da11, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d2da17, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf252da1b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc32da1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf2da24, SUC_lexicon_tags_8 },
    { 0xb8f2da25, SUC_lexicon_tags_39 },
    { 0x2232da25, SUC_lexicon_tags_7 },
    { 0x6672da27, SUC_lexicon_tags_70 },
    { 0x3e12da26, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72da2b, SUC_lexicon_tags_57 },
    { 0x6f92da2b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb2da2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb2da33, SUC_lexicon_tags_8 },
    { 0xb472da33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb2da39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce12da3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b2da41, SUC_lexicon_tags_65 },
    { 0xc52da42, SUC_lexicon_tags_33 },
    { 0x4cd2da41, SUC_lexicon_tags_45 },
    { 0xb952da42, SUC_lexicon_tags_15 },
    { 0x212da44, SUC_lexicon_tags_13 },
    { 0xe992da46, SUC_lexicon_tags_66 },
    { 0xbf2da44, SUC_lexicon_tags_43 },
    { 0xf92da45, SUC_lexicon_tags_31 },
    { 0xe512da49, SUC_lexicon_tags_11 },
    { 0x4232da47, SUC_lexicon_tags_48 },
    { 0x33d2da4b, SUC_lexicon_tags_8 },
    { 0x1db2da4b, SUC_lexicon_tags_17 },
    { 0x1d92da4d, SUC_lexicon_tags_74 },
    { 0xb92da4d, SUC_lexicon_tags_33 },
    { 0x5cd2da4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa192da52, SUC_lexicon_tags_15 },
    { 0x86d2da53, SUC_lexicon_tags_136 },
    { 0x3d92da54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa972da56, SUC_lexicon_tags_11 },
    { 0xf132da56, SUC_lexicon_tags_8 },
    { 0x1192da58, SUC_lexicon_tags_4 },
    { 0xfdb2da57, SUC_lexicon_tags_2 },
    { 0xa692da5a, SUC_lexicon_tags_31 },
    { 0x25b2da5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x492da5e, SUC_lexicon_tags_23 },
    { 0x6552da5e, SUC_lexicon_tags_23 },
    { 0x1a12da5e, SUC_lexicon_tags_17 },
    { 0x8992da61, SUC_lexicon_tags_2 },
    { 0xd732da62, SUC_lexicon_tags_95 },
    { 0x3952da5e, SUC_lexicon_tags_27 },
    { 0xd1f2da64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2652da67, SUC_lexicon_tags_14 },
    { 0xf4b2da68, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb592da74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2432da77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd372da7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb392da7e, SUC_lexicon_tags_4 },
    { 0xf732da7f, SUC_lexicon_tags_8 },
    { 0x7cf2da80, SUC_lexicon_tags_25 },
    { 0x6892da81, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xad52da84, SUC_lexicon_tags_23 },
    { 0xc032da85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff2da89, SUC_lexicon_tags_23 },
    { 0x91d2da8a, SUC_lexicon_tags_1 },
    { 0xe272da8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8072da8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5752da91, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf52da98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f2da9b, SUC_lexicon_tags_62 },
    { 0x6792da9c, SUC_lexicon_tags_28 },
    { 0x4f92da9c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f2daa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6d2daa5, SUC_lexicon_tags_6 },
    { 0xe012daa5, SUC_lexicon_tags_0 },
    { 0x54d2daa7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f2daaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c12daac, SUC_lexicon_tags_45 },
    { 0x7cd2daad, SUC_lexicon_tags_14 },
    { 0xab52daac, SUC_lexicon_tags_7 },
    { 0x8f52daaf, SUC_lexicon_tags_0 },
    { 0x8dd2daaf, SUC_lexicon_tags_63 },
    { 0xc4f2dab1, SUC_lexicon_tags_14 },
    { 0x41b2daae, SUC_lexicon_tags_16 },
    { 0x2d2daad, SUC_lexicon_tags_2 },
    { 0x2992dab4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc512dabf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a92dac1, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0x5df2dac3, SUC_lexicon_tags_17 },
    { 0x2452dac3, SUC_lexicon_tags_534 },
    { 0, NULL },
    { 0x2752dac6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2daca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5272dacc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d92dace, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe352dad2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac32dad7, SUC_lexicon_tags_37 },
    { 0x45f2dad8, SUC_lexicon_tags_23 },
    { 0xba52dad7, SUC_lexicon_tags_25 },
    { 0x2992dada, SUC_lexicon_tags_7 },
    { 0xad52dada, SUC_lexicon_tags_37 },
    { 0x70b2dadc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb2dadf, SUC_lexicon_tags_7 },
    { 0x2912dae0, SUC_lexicon_tags_17 },
    { 0xa9f2dae1, SUC_lexicon_tags_3 },
    { 0x5b2dae2, SUC_lexicon_tags_1 },
    { 0xd052dae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bb2dae5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3012dae7, SUC_lexicon_tags_21 },
    { 0xd6d2dae7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40b2daea, SUC_lexicon_tags_29 },
    { 0x7f12daeb, SUC_lexicon_tags_37 },
    { 0x3872daeb, SUC_lexicon_tags_46 },
    { 0xf952daea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3f2daef, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc752dafc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb2db00, SUC_lexicon_tags_33 },
    { 0x2f12db01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7052db05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc592db08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f2db12, SUC_lexicon_tags_37 },
    { 0x95b2db13, SUC_lexicon_tags_7 },
    { 0x63b2db14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4332db16, SUC_lexicon_tags_35 },
    { 0x7892db16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d2db1b, SUC_lexicon_tags_2 },
    { 0xd812db1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc12db1f, SUC_lexicon_tags_12 },
    { 0xc372db20, SUC_lexicon_tags_37 },
    { 0xed32db1f, SUC_lexicon_tags_33 },
    { 0x9932db22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeef2db24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe52db26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c52db29, SUC_lexicon_tags_83 },
    { 0x2632db2a, SUC_lexicon_tags_27 },
    { 0x52b2db2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed2db2f, SUC_lexicon_tags_26 },
    { 0xaab2db30, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc512db37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3432db39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x772db3b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6332db3d, SUC_lexicon_tags_12 },
    { 0xc312db3e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e12db43, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d72db49, SUC_lexicon_tags_11 },
    { 0xcfb2db49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5192db4d, SUC_lexicon_tags_95 },
    { 0x11f2db4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1352db50, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd72db53, SUC_lexicon_tags_12 },
    { 0x7db2db53, SUC_lexicon_tags_37 },
    { 0xe312db55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1772db58, SUC_lexicon_tags_37 },
    { 0x3112db59, SUC_lexicon_tags_4 },
    { 0xab72db59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc12db5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2852db63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a32db6a, SUC_lexicon_tags_2 },
    { 0x46f2db6b, SUC_lexicon_tags_65 },
    { 0x6532db6b, SUC_lexicon_tags_47 },
    { 0x28b2db6b, SUC_lexicon_tags_9 },
    { 0x22b2db6c, SUC_lexicon_tags_31 },
    { 0xfad2db6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdcd2db71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d2db74, SUC_lexicon_tags_35 },
    { 0x72d2db75, SUC_lexicon_tags_33 },
    { 0xa132db76, SUC_lexicon_tags_7 },
    { 0x3cf2db77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c12db7a, SUC_lexicon_tags_10 },
    { 0x6d52db7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1892db7e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x99d2db80, SUC_lexicon_tags_43 },
    { 0x8872db81, SUC_lexicon_tags_0 },
    { 0xad52db81, SUC_lexicon_tags_60 },
    { 0xaef2db82, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8dd2db85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf72db88, SUC_lexicon_tags_19 },
    { 0x3412db88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9352db8c, SUC_lexicon_tags_17 },
    { 0x94b2db8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2432db91, SUC_lexicon_tags_2 },
    { 0x9a52db92, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb2db96, SUC_lexicon_tags_26 },
    { 0x1c12db97, SUC_lexicon_tags_55 },
    { 0x3f12db98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3292db9a, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf832dba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf52dba2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd832dba7, SUC_lexicon_tags_15 },
    { 0x6d32dba7, SUC_lexicon_tags_6 },
    { 0xe932dba7, SUC_lexicon_tags_37 },
    { 0x8c92dba7, SUC_lexicon_tags_49 },
    { 0x1452dbab, SUC_lexicon_tags_39 },
    { 0x1232dbac, SUC_lexicon_tags_15 },
    { 0xe572dbac, SUC_lexicon_tags_47 },
    { 0x2b72dbae, SUC_lexicon_tags_2 },
    { 0xdd12dbae, SUC_lexicon_tags_1 },
    { 0xd1d2dbae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9372dbb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad2dbb6, SUC_lexicon_tags_0 },
    { 0xd752dbb7, SUC_lexicon_tags_13 },
    { 0x4bf2dbb8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f2dbbf, SUC_lexicon_tags_0 },
    { 0x46d2dbc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5792dbc3, SUC_lexicon_tags_39 },
    { 0x9a52dbc4, SUC_lexicon_tags_1 },
    { 0x4812dbc5, SUC_lexicon_tags_29 },
    { 0x1bb2dbc6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb2dbd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6352dbd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f92dbde, SUC_lexicon_tags_28 },
    { 0xda32dbdf, SUC_lexicon_tags_835 },
    { 0x53f2dbde, SUC_lexicon_tags_48 },
    { 0x88b2dbe1, SUC_lexicon_tags_37 },
    { 0xadf2dbe1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4b2dbe4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb032dbe6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d2dbea, SUC_lexicon_tags_16 },
    { 0xfaf2dbea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9f2dbed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5d52dbef, SUC_lexicon_tags_1236 },
    { 0, NULL },
    { 0xbc72dbf1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a32dbfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d2dc02, SUC_lexicon_tags_15 },
    { 0xc0d2dc03, SUC_lexicon_tags_11 },
    { 0x6fb2dc04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1852dc07, SUC_lexicon_tags_8 },
    { 0x2232dc07, SUC_lexicon_tags_25 },
    { 0x3f2dc09, SUC_lexicon_tags_37 },
    { 0x3972dc0a, SUC_lexicon_tags_37 },
    { 0xb032dc0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7512dc12, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8712dc14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e72dc16, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e12dc1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d2dc23, SUC_lexicon_tags_23 },
    { 0xf72dc23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd2dc27, SUC_lexicon_tags_15 },
    { 0xffd2dc28, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d2dc2f, SUC_lexicon_tags_37 },
    { 0xd052dc30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e72dc32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b92dc36, SUC_lexicon_tags_21 },
    { 0xd532dc37, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb632dc3a, SUC_lexicon_tags_61 },
    { 0xc512dc3a, SUC_lexicon_tags_95 },
    { 0x812dc3c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad12dc45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd012dc47, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1672dc52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b2dc56, SUC_lexicon_tags_230 },
    { 0x77d2dc57, SUC_lexicon_tags_0 },
    { 0xe392dc57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb512dc5a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x4192dc5d, SUC_lexicon_tags_4 },
    { 0xe6f2dc5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd432dc61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3f2dc63, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa92dc65, SUC_lexicon_tags_80 },
    { 0x4392dc66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9212dc6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2852dc6f, SUC_lexicon_tags_38 },
    { 0x64f2dc6f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xff92dc72, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8272dc77, SUC_lexicon_tags_43 },
    { 0x7e92dc77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda92dc7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42f2dc7d, SUC_lexicon_tags_25 },
    { 0xdf2dc7d, SUC_lexicon_tags_15 },
    { 0x6f32dc7f, SUC_lexicon_tags_87 },
    { 0x1632dc80, SUC_lexicon_tags_84 },
    { 0xc012dc81, SUC_lexicon_tags_37 },
    { 0x5e92dc81, SUC_lexicon_tags_6 },
    { 0x8f2dc80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb2dc88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f2dc8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c32dc8f, SUC_lexicon_tags_6 },
    { 0x22d2dc90, SUC_lexicon_tags_42 },
    { 0x4fb2dc90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa32dc97, SUC_lexicon_tags_58 },
    { 0x2d32dc97, SUC_lexicon_tags_45 },
    { 0xa112dc99, SUC_lexicon_tags_126 },
    { 0x31f2dc98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d2dc9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb72dca6, SUC_lexicon_tags_98 },
    { 0x6ab2dca6, SUC_lexicon_tags_13 },
    { 0x9b12dca8, SUC_lexicon_tags_243 },
    { 0xa632dca9, SUC_lexicon_tags_45 },
    { 0x7b52dca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa092dcad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df2dcb0, SUC_lexicon_tags_11 },
    { 0xeb12dcb1, SUC_lexicon_tags_17 },
    { 0x1212dcb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa12dcb6, SUC_lexicon_tags_25 },
    { 0xddb2dcb6, SUC_lexicon_tags_6 },
    { 0x8cd2dcb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b32dcbd, SUC_lexicon_tags_2 },
    { 0x6512dcbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b2dcc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa832dcc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf92dcc7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d2dcca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd972dccc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2dcd1, SUC_lexicon_tags_2 },
    { 0x8af2dcd2, SUC_lexicon_tags_17 },
    { 0xe52dcd1, SUC_lexicon_tags_37 },
    { 0xce32dcd4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d2dcd7, SUC_lexicon_tags_11 },
    { 0x43f2dcd8, SUC_lexicon_tags_14 },
    { 0x6712dcd7, SUC_lexicon_tags_47 },
    { 0x552dcda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd2dcdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82d2dce1, SUC_lexicon_tags_6 },
    { 0xd6b2dce2, SUC_lexicon_tags_60 },
    { 0x9e32dce3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef2dce9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8472dceb, SUC_lexicon_tags_25 },
    { 0x5752dcec, SUC_lexicon_tags_26 },
    { 0xfc92dcec, SUC_lexicon_tags_48 },
    { 0x9dd2dcee, SUC_lexicon_tags_10 },
    { 0x4c92dcef, SUC_lexicon_tags_48 },
    { 0x6632dced, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3152dcf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f12dcf5, SUC_lexicon_tags_14 },
    { 0x39b2dcf6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8952dcfc, SUC_lexicon_tags_63 },
    { 0xdb2dcfd, SUC_lexicon_tags_27 },
    { 0xf2b2dcfd, SUC_lexicon_tags_25 },
    { 0xeb12dcfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51f2dd01, SUC_lexicon_tags_17 },
    { 0xb52dd02, SUC_lexicon_tags_2 },
    { 0x52b2dd03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad2dd09, SUC_lexicon_tags_28 },
    { 0x5212dd0a, SUC_lexicon_tags_7 },
    { 0x2d52dd0a, SUC_lexicon_tags_33 },
    { 0x7cf2dd0c, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d32dd10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e92dd14, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb12dd16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f92dd18, SUC_lexicon_tags_38 },
    { 0x5c72dd19, SUC_lexicon_tags_17 },
    { 0xc372dd1a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8912dd1c, SUC_lexicon_tags_17 },
    { 0xdb52dd1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d32dd1f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c72dd24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6092dd26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf12dd2d, SUC_lexicon_tags_2 },
    { 0x3712dd2e, SUC_lexicon_tags_0 },
    { 0x2bd2dd2f, SUC_lexicon_tags_15 },
    { 0x52d2dd2d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1312dd32, SUC_lexicon_tags_7 },
    { 0x8ff2dd33, SUC_lexicon_tags_16 },
    { 0x2672dd32, SUC_lexicon_tags_21 },
    { 0xe7f2dd35, SUC_lexicon_tags_16 },
    { 0x8af2dd35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe92dd3e, SUC_lexicon_tags_38 },
    { 0xc192dd3f, SUC_lexicon_tags_1 },
    { 0xced2dd40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d2dd44, SUC_lexicon_tags_37 },
    { 0x9dd2dd44, SUC_lexicon_tags_82 },
    { 0xfd32dd44, SUC_lexicon_tags_114 },
    { 0x62f2dd47, SUC_lexicon_tags_2 },
    { 0x96f2dd48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b2dd4e, SUC_lexicon_tags_37 },
    { 0x5bb2dd4f, SUC_lexicon_tags_12 },
    { 0xba92dd50, SUC_lexicon_tags_0 },
    { 0xde52dd51, SUC_lexicon_tags_21 },
    { 0x79b2dd52, SUC_lexicon_tags_2 },
    { 0xd652dd53, SUC_lexicon_tags_17 },
    { 0x3e92dd54, SUC_lexicon_tags_17 },
    { 0x152dd55, SUC_lexicon_tags_17 },
    { 0x83d2dd56, SUC_lexicon_tags_25 },
    { 0x6ab2dd57, SUC_lexicon_tags_1 },
    { 0x8af2dd58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5912dd5b, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf2dd64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b12dd6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8512dd72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d52dd75, SUC_lexicon_tags_17 },
    { 0x4ab2dd75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c32dd7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b2dd80, SUC_lexicon_tags_25 },
    { 0x5512dd80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42f2dd83, SUC_lexicon_tags_11 },
    { 0xf6f2dd84, SUC_lexicon_tags_37 },
    { 0x4472dd84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf2dd88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb72dd8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb092dd91, SUC_lexicon_tags_45 },
    { 0xcdb2dd91, SUC_lexicon_tags_15 },
    { 0x6312dd93, SUC_lexicon_tags_26 },
    { 0x24f2dd93, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb2dd99, SUC_lexicon_tags_47 },
    { 0xbb32dd9a, SUC_lexicon_tags_11 },
    { 0xc32dd9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8f2dd9d, SUC_lexicon_tags_60 },
    { 0x1252dd9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b2dda1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4792ddaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b32ddaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3f2ddb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1cf2ddb3, SUC_lexicon_tags_49 },
    { 0x55b2ddb4, SUC_lexicon_tags_11 },
    { 0x7612ddb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa612ddba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead2ddc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd472ddc5, SUC_lexicon_tags_1 },
    { 0x5452ddc6, SUC_lexicon_tags_23 },
    { 0xc092ddc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d72ddca, SUC_lexicon_tags_12 },
    { 0xf632ddcb, SUC_lexicon_tags_63 },
    { 0x7d52ddcc, SUC_lexicon_tags_14 },
    { 0x2672ddca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8172ddd3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2732ddda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f2ddde, SUC_lexicon_tags_27 },
    { 0x9a32dddf, SUC_lexicon_tags_25 },
    { 0x9792dde0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c72dde2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e72dde4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff2dde8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x312ddee, SUC_lexicon_tags_30 },
    { 0x2072ddef, SUC_lexicon_tags_25 },
    { 0x56f2ddef, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2ddf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f2ddff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b2de02, SUC_lexicon_tags_11 },
    { 0xa72de02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6172de05, SUC_lexicon_tags_2 },
    { 0x7e52de05, SUC_lexicon_tags_21 },
    { 0xd692de05, SUC_lexicon_tags_11 },
    { 0xc912de08, SUC_lexicon_tags_30 },
    { 0x9e12de09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c92de0b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x81d2de0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf852de0f, SUC_lexicon_tags_12 },
    { 0xd5d2de10, SUC_lexicon_tags_15 },
    { 0xbef2de11, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xda52de13, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x56f2de16, SUC_lexicon_tags_1 },
    { 0x5ed2de17, SUC_lexicon_tags_2 },
    { 0xaa72de16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1112de20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe992de25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc172de2a, SUC_lexicon_tags_11 },
    { 0xe372de2a, SUC_lexicon_tags_17 },
    { 0x2392de2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab2de32, SUC_lexicon_tags_7 },
    { 0x4f52de33, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5c72de35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7592de3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5412de3f, SUC_lexicon_tags_25 },
    { 0xb712de3f, SUC_lexicon_tags_3 },
    { 0xf212de40, SUC_lexicon_tags_9 },
    { 0x8912de42, SUC_lexicon_tags_11 },
    { 0xa872de43, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2012de45, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9152de4c, SUC_lexicon_tags_27 },
    { 0x7ef2de4d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d2de50, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcf92de52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc52de5a, SUC_lexicon_tags_1 },
    { 0xb112de5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c92de5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f32de61, SUC_lexicon_tags_6 },
    { 0x392de62, SUC_lexicon_tags_7 },
    { 0xae32de63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f2de67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7a52de69, SUC_lexicon_tags_4 },
    { 0xa852de6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdc12de6c, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0x572de6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa052de71, SUC_lexicon_tags_37 },
    { 0xe652de72, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb672de77, SUC_lexicon_tags_17 },
    { 0x94d2de77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc72de7a, SUC_lexicon_tags_11 },
    { 0x21f2de7b, SUC_lexicon_tags_17 },
    { 0x1c12de7b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5f32de7e, SUC_lexicon_tags_1 },
    { 0x2e72de7f, SUC_lexicon_tags_15 },
    { 0x6992de80, SUC_lexicon_tags_30 },
    { 0x8492de81, SUC_lexicon_tags_28 },
    { 0xebb2de82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xec52de85, SUC_lexicon_tags_11 },
    { 0x9352de86, SUC_lexicon_tags_17 },
    { 0x1192de86, SUC_lexicon_tags_23 },
    { 0x1ff2de87, SUC_lexicon_tags_47 },
    { 0x7f52de89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f2de8e, SUC_lexicon_tags_37 },
    { 0xf692de8f, SUC_lexicon_tags_12 },
    { 0x9cb2de90, SUC_lexicon_tags_2 },
    { 0x7f72de91, SUC_lexicon_tags_45 },
    { 0x6db2de8f, SUC_lexicon_tags_25 },
    { 0x2432de91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7292de98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4052de9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb992dea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7212deac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a72deb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27f2deb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2372deb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f2debe, SUC_lexicon_tags_36 },
    { 0xdcd2debe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd132dec3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f12dec8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9732decb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa12dece, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac12ded2, SUC_lexicon_tags_3 },
    { 0x6a32ded3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda52ded8, SUC_lexicon_tags_15 },
    { 0x4a72ded9, SUC_lexicon_tags_1 },
    { 0x51d2ded9, SUC_lexicon_tags_1 },
    { 0xd452ded8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df2dede, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf052dee2, SUC_lexicon_tags_26 },
    { 0x1d52dee2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd372dee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3612dee7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb512dee9, SUC_lexicon_tags_2 },
    { 0x90f2deea, SUC_lexicon_tags_16 },
    { 0xddf2deeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd92deef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3712def2, SUC_lexicon_tags_31 },
    { 0x50b2def3, SUC_lexicon_tags_7 },
    { 0x3ed2def4, SUC_lexicon_tags_19 },
    { 0xf492def5, SUC_lexicon_tags_6 },
    { 0x97f2def6, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x13b2def8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf592defa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x92defd, SUC_lexicon_tags_43 },
    { 0x6072defe, SUC_lexicon_tags_61 },
    { 0x63d2deff, SUC_lexicon_tags_14 },
    { 0x7872deff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x912df02, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad2df11, SUC_lexicon_tags_46 },
    { 0x1072df12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8092df17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b2df1b, SUC_lexicon_tags_17 },
    { 0x2392df1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d2df20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea72df22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9752df25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2832df2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad2df2d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5752df2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc72df32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd2df39, SUC_lexicon_tags_37 },
    { 0x7c12df39, SUC_lexicon_tags_12 },
    { 0x4472df3b, SUC_lexicon_tags_38 },
    { 0x75f2df39, SUC_lexicon_tags_27 },
    { 0xc572df3d, SUC_lexicon_tags_28 },
    { 0x9692df3e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed2df43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e32df4a, SUC_lexicon_tags_25 },
    { 0x3b52df4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c52df51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f2df54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e92df56, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb2df59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf732df5c, SUC_lexicon_tags_26 },
    { 0xa2f2df5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1032df61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6792df64, SUC_lexicon_tags_25 },
    { 0xa272df64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x812df67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9792df69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d32df6b, SUC_lexicon_tags_47 },
    { 0xc292df6b, SUC_lexicon_tags_17 },
    { 0x50b2df6b, SUC_lexicon_tags_46 },
    { 0x5852df6d, SUC_lexicon_tags_38 },
    { 0xc8f2df6f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7972df71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1112df75, SUC_lexicon_tags_65 },
    { 0x4092df76, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3612df7f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3e72df81, SUC_lexicon_tags_17 },
    { 0xebf2df82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe032df86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58b2df88, SUC_lexicon_tags_11 },
    { 0x4c32df89, SUC_lexicon_tags_15 },
    { 0xb192df89, SUC_lexicon_tags_43 },
    { 0x7872df8b, SUC_lexicon_tags_45 },
    { 0x7632df8b, SUC_lexicon_tags_26 },
    { 0xf572df8d, SUC_lexicon_tags_12 },
    { 0x8df2df8e, SUC_lexicon_tags_2 },
    { 0xde52df8f, SUC_lexicon_tags_7 },
    { 0xffd2df8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7592df92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a32df97, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db2df9a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5fd2df9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf72df9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a92dfa9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xef52dfac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb72dfae, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc992dfb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76b2dfb3, SUC_lexicon_tags_45 },
    { 0x5252dfb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf9d2dfb6, SUC_lexicon_tags_4 },
    { 0x37b2dfb7, SUC_lexicon_tags_37 },
    { 0xe072dfb8, SUC_lexicon_tags_15 },
    { 0x9512dfb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x62f2dfbb, SUC_lexicon_tags_12 },
    { 0xeed2dfbc, SUC_lexicon_tags_55 },
    { 0xf1d2dfbc, SUC_lexicon_tags_2 },
    { 0xc32dfbd, SUC_lexicon_tags_39 },
    { 0x3252dfbf, SUC_lexicon_tags_25 },
    { 0x5492dfc0, SUC_lexicon_tags_8 },
    { 0xcbd2dfbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf912dfc6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6452dfcc, SUC_lexicon_tags_7 },
    { 0x4852dfcd, SUC_lexicon_tags_1 },
    { 0xaf92dfce, SUC_lexicon_tags_11 },
    { 0xf272dfce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3532dfd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f72dfd8, SUC_lexicon_tags_11 },
    { 0x77f2dfd8, SUC_lexicon_tags_33 },
    { 0xcbf2dfda, SUC_lexicon_tags_2 },
    { 0xe472dfdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d2dfe1, SUC_lexicon_tags_31 },
    { 0x8c32dfe2, SUC_lexicon_tags_11 },
    { 0x2e52dfe2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa1f2dfe5, SUC_lexicon_tags_114 },
    { 0xdd52dfe6, SUC_lexicon_tags_40 },
    { 0x10b2dfe7, SUC_lexicon_tags_7 },
    { 0xb992dfe7, SUC_lexicon_tags_15 },
    { 0xf692dfe9, SUC_lexicon_tags_29 },
    { 0xa6f2dfe9, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x4792dfec, SUC_lexicon_tags_30 },
    { 0xa052dfec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf752dfef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3412dff1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6152dff5, SUC_lexicon_tags_38 },
    { 0x4dd2dff6, SUC_lexicon_tags_15 },
    { 0x6452dff6, SUC_lexicon_tags_17 },
    { 0x6bd2dff8, SUC_lexicon_tags_37 },
    { 0xf952dff9, SUC_lexicon_tags_11 },
    { 0x8932dffa, SUC_lexicon_tags_25 },
    { 0x7372dffa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa612dffd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4f2dfff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b2e005, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6952e00c, SUC_lexicon_tags_26 },
    { 0xc832e00c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd2e010, SUC_lexicon_tags_12 },
    { 0xa332e011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b2e019, SUC_lexicon_tags_17 },
    { 0x9e52e01a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d2e01d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd72e022, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1432e025, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f2e028, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef2e02c, SUC_lexicon_tags_1 },
    { 0xef52e02d, SUC_lexicon_tags_37 },
    { 0xf712e02d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b32e031, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f2e036, SUC_lexicon_tags_84 },
    { 0xab32e037, SUC_lexicon_tags_29 },
    { 0x18d2e037, SUC_lexicon_tags_29 },
    { 0x21f2e036, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b2e03d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xda52e040, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6012e047, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f52e054, SUC_lexicon_tags_11 },
    { 0x8e32e055, SUC_lexicon_tags_9 },
    { 0x81f2e054, SUC_lexicon_tags_37 },
    { 0xfcb2e057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf52e059, SUC_lexicon_tags_15 },
    { 0x13d2e059, SUC_lexicon_tags_37 },
    { 0x9e52e05b, SUC_lexicon_tags_12 },
    { 0xfe92e05b, SUC_lexicon_tags_60 },
    { 0x9fd2e05c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f12e061, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed52e067, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5fd2e069, SUC_lexicon_tags_11 },
    { 0x1732e069, SUC_lexicon_tags_95 },
    { 0x77d2e06b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55f2e06d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb2e071, SUC_lexicon_tags_77 },
    { 0xbd92e072, SUC_lexicon_tags_11 },
    { 0xd752e072, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x66b2e075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f2e077, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe292e079, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2cd2e07b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd2e07f, SUC_lexicon_tags_17 },
    { 0x4d92e07f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b2e084, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ef2e086, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5a92e088, SUC_lexicon_tags_14 },
    { 0x8412e089, SUC_lexicon_tags_17 },
    { 0xaa12e08a, SUC_lexicon_tags_17 },
    { 0xf72e088, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb872e08d, SUC_lexicon_tags_0 },
    { 0x9c12e08e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c52e091, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc192e093, SUC_lexicon_tags_28 },
    { 0xe912e094, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4852e096, SUC_lexicon_tags_25 },
    { 0x2692e096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab2e09a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d2e09f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef2e0a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8512e0a5, SUC_lexicon_tags_29 },
    { 0x3372e0a6, SUC_lexicon_tags_13 },
    { 0x19f2e0a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab92e0aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8592e0ad, SUC_lexicon_tags_15 },
    { 0x112e0ae, SUC_lexicon_tags_17 },
    { 0xa752e0af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf2e0b4, SUC_lexicon_tags_9 },
    { 0x57f2e0b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe532e0b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b2e0bd, SUC_lexicon_tags_61 },
    { 0x2172e0be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc512e0c0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f2e0c5, SUC_lexicon_tags_32 },
    { 0xa472e0c6, SUC_lexicon_tags_32 },
    { 0xd7b2e0c7, SUC_lexicon_tags_7 },
    { 0xda12e0c5, SUC_lexicon_tags_45 },
    { 0xa532e0c7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb92e0ce, SUC_lexicon_tags_11 },
    { 0x2812e0cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3912e0d3, SUC_lexicon_tags_27 },
    { 0xb32e0d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5932e0d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf72e0dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa92e0e2, SUC_lexicon_tags_11 },
    { 0x4b92e0e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf2e0ed, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc012e0f0, SUC_lexicon_tags_16 },
    { 0x5f52e0f0, SUC_lexicon_tags_27 },
    { 0x2a32e0f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc572e0f6, SUC_lexicon_tags_27 },
    { 0xaef2e0f7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf2e0fc, SUC_lexicon_tags_15 },
    { 0x9f52e0fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde52e100, SUC_lexicon_tags_23 },
    { 0xb492e101, SUC_lexicon_tags_2 },
    { 0xb6b2e101, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3272e105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6392e10c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf2e112, SUC_lexicon_tags_12 },
    { 0x3692e113, SUC_lexicon_tags_14 },
    { 0xa192e113, SUC_lexicon_tags_11 },
    { 0xe72e115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6972e118, SUC_lexicon_tags_25 },
    { 0x4a32e118, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8052e11b, SUC_lexicon_tags_2 },
    { 0x3ab2e11c, SUC_lexicon_tags_17 },
    { 0xe3d2e11d, SUC_lexicon_tags_14 },
    { 0x2b12e11e, SUC_lexicon_tags_7 },
    { 0x49f2e11f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5132e121, SUC_lexicon_tags_37 },
    { 0xb712e122, SUC_lexicon_tags_17 },
    { 0xb692e123, SUC_lexicon_tags_60 },
    { 0xeb52e124, SUC_lexicon_tags_1 },
    { 0xc72e125, SUC_lexicon_tags_2 },
    { 0xe792e123, SUC_lexicon_tags_8 },
    { 0x8112e124, SUC_lexicon_tags_15 },
    { 0x9f92e126, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb412e12a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7d2e12c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe652e12e, SUC_lexicon_tags_12 },
    { 0xe732e12f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27f2e131, SUC_lexicon_tags_15 },
    { 0x7572e131, SUC_lexicon_tags_39 },
    { 0xdeb2e133, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e136, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0b2e138, SUC_lexicon_tags_16 },
    { 0xe532e139, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa852e13b, SUC_lexicon_tags_63 },
    { 0xfd72e13c, SUC_lexicon_tags_65 },
    { 0x48d2e13d, SUC_lexicon_tags_25 },
    { 0x5892e13e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce12e140, SUC_lexicon_tags_17 },
    { 0x46f2e140, SUC_lexicon_tags_4 },
    { 0xd2e141, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc232e146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde32e149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35b2e14b, SUC_lexicon_tags_302 },
    { 0xa312e14b, SUC_lexicon_tags_15 },
    { 0xbdf2e14b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d2e150, SUC_lexicon_tags_37 },
    { 0x6992e151, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee32e15a, SUC_lexicon_tags_17 },
    { 0x1d72e15b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf2e15d, SUC_lexicon_tags_12 },
    { 0x14b2e15e, SUC_lexicon_tags_31 },
    { 0xcc12e15f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87f2e161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccb2e163, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a12e166, SUC_lexicon_tags_7 },
    { 0x8372e167, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e12e16a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d2e16d, SUC_lexicon_tags_25 },
    { 0x3f12e16e, SUC_lexicon_tags_4 },
    { 0x30f2e16d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4732e173, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d52e177, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8a32e179, SUC_lexicon_tags_17 },
    { 0x6e32e17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6b2e17c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f2e180, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb2e186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x912e18a, SUC_lexicon_tags_47 },
    { 0x79d2e18a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5512e18e, SUC_lexicon_tags_17 },
    { 0x2052e18f, SUC_lexicon_tags_560 },
    { 0, NULL },
    { 0x35b2e191, SUC_lexicon_tags_37 },
    { 0x4732e192, SUC_lexicon_tags_17 },
    { 0x7132e193, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad2e197, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c92e199, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb8f2e19b, SUC_lexicon_tags_17 },
    { 0x8292e19c, SUC_lexicon_tags_25 },
    { 0xf7b2e19b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b2e1a0, SUC_lexicon_tags_14 },
    { 0x4112e1a1, SUC_lexicon_tags_38 },
    { 0x66b2e1a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2d2e1a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d2e1ac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb2e1b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b32e1b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc72e1b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d72e1b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54f2e1ba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc92e1be, SUC_lexicon_tags_2 },
    { 0x31f2e1bf, SUC_lexicon_tags_63 },
    { 0xbcd2e1bf, SUC_lexicon_tags_26 },
    { 0x97f2e1c1, SUC_lexicon_tags_15 },
    { 0x8272e1c1, SUC_lexicon_tags_11 },
    { 0xe852e1c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbb2e1c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb2e1c9, SUC_lexicon_tags_37 },
    { 0xb5f2e1c9, SUC_lexicon_tags_27 },
    { 0xb252e1cb, SUC_lexicon_tags_11 },
    { 0x1272e1cc, SUC_lexicon_tags_37 },
    { 0x9d72e1cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb2e1d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea52e1d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff92e1d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d2e1db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f2e1e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1412e1e8, SUC_lexicon_tags_37 },
    { 0xd5f2e1e9, SUC_lexicon_tags_9 },
    { 0x6bf2e1ea, SUC_lexicon_tags_37 },
    { 0x9992e1ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e92e1ee, SUC_lexicon_tags_26 },
    { 0x25b2e1ef, SUC_lexicon_tags_45 },
    { 0x2a92e1ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa32e1f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48f2e1f5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe12e1f8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xec32e1fa, SUC_lexicon_tags_37 },
    { 0x3a72e1fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf672e201, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa692e207, SUC_lexicon_tags_7 },
    { 0x3352e208, SUC_lexicon_tags_7 },
    { 0x42f2e209, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc872e20c, SUC_lexicon_tags_37 },
    { 0x8712e20d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfa92e20f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x8712e211, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8552e215, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe972e219, SUC_lexicon_tags_17 },
    { 0xef52e219, SUC_lexicon_tags_23 },
    { 0x5792e21a, SUC_lexicon_tags_14 },
    { 0x9ad2e219, SUC_lexicon_tags_7 },
    { 0x6692e219, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f2e221, SUC_lexicon_tags_33 },
    { 0x8392e222, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f32e224, SUC_lexicon_tags_38 },
    { 0x22f2e225, SUC_lexicon_tags_29 },
    { 0xfb12e225, SUC_lexicon_tags_146 },
    { 0xd392e225, SUC_lexicon_tags_49 },
    { 0x7632e225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ab2e22a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe912e22c, SUC_lexicon_tags_4 },
    { 0x97b2e22d, SUC_lexicon_tags_37 },
    { 0xced2e22d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f2e231, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa7b2e233, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d2e237, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d2e23a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d2e23e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd52e242, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7912e245, SUC_lexicon_tags_6 },
    { 0x5ef2e246, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f32e248, SUC_lexicon_tags_32 },
    { 0x9092e249, SUC_lexicon_tags_11 },
    { 0xf732e24a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa92e24d, SUC_lexicon_tags_38 },
    { 0x9cb2e24e, SUC_lexicon_tags_1 },
    { 0xa412e24e, SUC_lexicon_tags_17 },
    { 0x6252e250, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3692e252, SUC_lexicon_tags_5 },
    { 0x5552e253, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d2e257, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6a52e259, SUC_lexicon_tags_43 },
    { 0x4e12e25a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d2e25e, SUC_lexicon_tags_1 },
    { 0x372e25e, SUC_lexicon_tags_49 },
    { 0x6932e260, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe6f2e262, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe92e265, SUC_lexicon_tags_8 },
    { 0xdf92e266, SUC_lexicon_tags_28 },
    { 0xa172e266, SUC_lexicon_tags_25 },
    { 0xce72e267, SUC_lexicon_tags_47 },
    { 0x7212e269, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe32e26c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d2e272, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f72e27d, SUC_lexicon_tags_7 },
    { 0x66f2e27e, SUC_lexicon_tags_12 },
    { 0xb72e27f, SUC_lexicon_tags_17 },
    { 0xabd2e280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1152e283, SUC_lexicon_tags_12 },
    { 0xbf72e284, SUC_lexicon_tags_33 },
    { 0xd132e285, SUC_lexicon_tags_1 },
    { 0x29b2e286, SUC_lexicon_tags_47 },
    { 0xcf2e287, SUC_lexicon_tags_43 },
    { 0xb912e285, SUC_lexicon_tags_11 },
    { 0x9892e289, SUC_lexicon_tags_7 },
    { 0x9e92e289, SUC_lexicon_tags_7 },
    { 0xf2f2e28b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9812e28d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6312e292, SUC_lexicon_tags_27 },
    { 0x2e52e292, SUC_lexicon_tags_15 },
    { 0x2292e293, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d2e29a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe32e29f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27b2e2a1, SUC_lexicon_tags_43 },
    { 0x63b2e2a2, SUC_lexicon_tags_31 },
    { 0x81f2e2a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23b2e2a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d72e2a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d2e2ab, SUC_lexicon_tags_11 },
    { 0x53f2e2ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd2e2af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe92e2b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6e52e2ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1352e2c4, SUC_lexicon_tags_69 },
    { 0x3052e2c5, SUC_lexicon_tags_6 },
    { 0x8132e2c6, SUC_lexicon_tags_12 },
    { 0x8432e2c6, SUC_lexicon_tags_49 },
    { 0xde12e2c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcb2e2ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb52e2cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3272e2d1, SUC_lexicon_tags_47 },
    { 0x99f2e2d1, SUC_lexicon_tags_47 },
    { 0x592e2d2, SUC_lexicon_tags_15 },
    { 0xfaf2e2d4, SUC_lexicon_tags_0 },
    { 0x10d2e2d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd592e2d7, SUC_lexicon_tags_160 },
    { 0x64d2e2d7, SUC_lexicon_tags_4 },
    { 0x6672e2d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2e2dd, SUC_lexicon_tags_2 },
    { 0xd1f2e2de, SUC_lexicon_tags_37 },
    { 0x6792e2dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef2e2e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb2e2e6, SUC_lexicon_tags_17 },
    { 0xc792e2e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ff2e2e9, SUC_lexicon_tags_33 },
    { 0xc992e2ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab12e2ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7632e2ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d2e2f8, SUC_lexicon_tags_55 },
    { 0x7cd2e2f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5412e2fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d2e303, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5372e306, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f2e30c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe232e30e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x972e310, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8152e315, SUC_lexicon_tags_7 },
    { 0x58b2e315, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd2e319, SUC_lexicon_tags_7 },
    { 0x25b2e31a, SUC_lexicon_tags_0 },
    { 0x1bd2e31b, SUC_lexicon_tags_2 },
    { 0x4a92e319, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a12e31e, SUC_lexicon_tags_15 },
    { 0xcb2e31f, SUC_lexicon_tags_31 },
    { 0x3312e31f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6f2e322, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc012e327, SUC_lexicon_tags_25 },
    { 0xedb2e328, SUC_lexicon_tags_37 },
    { 0x9d2e327, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95b2e32b, SUC_lexicon_tags_34 },
    { 0x1972e32c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c12e332, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f2e335, SUC_lexicon_tags_61 },
    { 0xaff2e336, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf272e33a, SUC_lexicon_tags_28 },
    { 0x54d2e33b, SUC_lexicon_tags_28 },
    { 0x12d2e33a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd2e345, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xbc52e347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f2e34a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84d2e34c, SUC_lexicon_tags_11 },
    { 0xc292e34c, SUC_lexicon_tags_7 },
    { 0x9192e34c, SUC_lexicon_tags_47 },
    { 0xe9f2e34e, SUC_lexicon_tags_16 },
    { 0xb92e350, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb92e353, SUC_lexicon_tags_7 },
    { 0xb452e353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3712e356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d12e359, SUC_lexicon_tags_10 },
    { 0xa6f2e35a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xc0f2e35c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7072e362, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2692e367, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe012e36d, SUC_lexicon_tags_56 },
    { 0x88f2e36e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf852e371, SUC_lexicon_tags_12 },
    { 0xb172e372, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6192e374, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc12e378, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0b2e37a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe932e37e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c32e382, SUC_lexicon_tags_7 },
    { 0xaad2e382, SUC_lexicon_tags_128 },
    { 0xa352e384, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf212e386, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcfd2e388, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe192e38e, SUC_lexicon_tags_60 },
    { 0xb812e38e, SUC_lexicon_tags_7 },
    { 0xf112e390, SUC_lexicon_tags_60 },
    { 0xb052e38e, SUC_lexicon_tags_0 },
    { 0xa772e392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d2e39a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7532e39d, SUC_lexicon_tags_55 },
    { 0x2452e39e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb2e3a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc012e3a6, SUC_lexicon_tags_70 },
    { 0xc512e3a7, SUC_lexicon_tags_43 },
    { 0x9b72e3a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe2d2e3aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e72e3ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d32e3b0, SUC_lexicon_tags_30 },
    { 0x39b2e3b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f2e3ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb232e3bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d72e3c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12e3cd, SUC_lexicon_tags_6 },
    { 0xbbb2e3ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde72e3d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f2e3da, SUC_lexicon_tags_37 },
    { 0xbd92e3da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe12e3e1, SUC_lexicon_tags_68 },
    { 0x8e52e3e1, SUC_lexicon_tags_12 },
    { 0x1ff2e3e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1912e3e8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca72e3ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda12e3ef, SUC_lexicon_tags_17 },
    { 0x1672e3f0, SUC_lexicon_tags_12 },
    { 0xea72e3f1, SUC_lexicon_tags_26 },
    { 0x60f2e3ef, SUC_lexicon_tags_55 },
    { 0xa652e3f3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa532e3f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75f2e3fa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4692e3fc, SUC_lexicon_tags_25 },
    { 0x37f2e3fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37f2e3ff, SUC_lexicon_tags_84 },
    { 0x9852e400, SUC_lexicon_tags_48 },
    { 0x7392e400, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b2e404, SUC_lexicon_tags_38 },
    { 0xb672e405, SUC_lexicon_tags_354 },
    { 0x5712e406, SUC_lexicon_tags_60 },
    { 0x5db2e407, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7192e409, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc832e40b, SUC_lexicon_tags_12 },
    { 0x5f72e40c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee12e410, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf32e414, SUC_lexicon_tags_48 },
    { 0x2d12e414, SUC_lexicon_tags_11 },
    { 0xac12e416, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef2e41a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9af2e41c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1572e41f, SUC_lexicon_tags_12 },
    { 0xd1d2e420, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1752e423, SUC_lexicon_tags_1 },
    { 0xc192e423, SUC_lexicon_tags_28 },
    { 0xfe92e424, SUC_lexicon_tags_37 },
    { 0xdd72e426, SUC_lexicon_tags_11 },
    { 0x5892e426, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea12e430, SUC_lexicon_tags_16 },
    { 0x8fb2e431, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf2e434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83f2e436, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe012e438, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a32e43a, SUC_lexicon_tags_9 },
    { 0x1832e43b, SUC_lexicon_tags_37 },
    { 0x1af2e43b, SUC_lexicon_tags_7 },
    { 0x4572e43b, SUC_lexicon_tags_5 },
    { 0x6ed2e43d, SUC_lexicon_tags_11 },
    { 0x8d92e43a, SUC_lexicon_tags_48 },
    { 0x4c12e43a, SUC_lexicon_tags_74 },
    { 0x2d52e441, SUC_lexicon_tags_0 },
    { 0xd92e440, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d2e445, SUC_lexicon_tags_60 },
    { 0x3352e445, SUC_lexicon_tags_9 },
    { 0xfd12e447, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf012e449, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd52e44c, SUC_lexicon_tags_8 },
    { 0x3212e44d, SUC_lexicon_tags_60 },
    { 0x3bf2e44e, SUC_lexicon_tags_27 },
    { 0xfcb2e44f, SUC_lexicon_tags_9 },
    { 0x9552e450, SUC_lexicon_tags_61 },
    { 0x1d92e451, SUC_lexicon_tags_37 },
    { 0x6852e452, SUC_lexicon_tags_11 },
    { 0x3bd2e44c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f2e458, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa7f2e45a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b2e45c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb372e45e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb052e462, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5dd2e464, SUC_lexicon_tags_27 },
    { 0x95f2e464, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49b2e467, SUC_lexicon_tags_47 },
    { 0x38b2e468, SUC_lexicon_tags_37 },
    { 0xab92e468, SUC_lexicon_tags_17 },
    { 0x9432e46a, SUC_lexicon_tags_1 },
    { 0x8492e468, SUC_lexicon_tags_57 },
    { 0xc892e46c, SUC_lexicon_tags_11 },
    { 0xfcb2e46c, SUC_lexicon_tags_38 },
    { 0xd072e46b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4092e471, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb472e474, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5532e47b, SUC_lexicon_tags_14 },
    { 0x76b2e47c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d2e47f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab2e484, SUC_lexicon_tags_116 },
    { 0x3f52e485, SUC_lexicon_tags_15 },
    { 0x292e486, SUC_lexicon_tags_14 },
    { 0xadb2e486, SUC_lexicon_tags_49 },
    { 0x692e488, SUC_lexicon_tags_60 },
    { 0xbcd2e485, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0xe6f2e48b, SUC_lexicon_tags_6 },
    { 0xa1d2e48b, SUC_lexicon_tags_45 },
    { 0x6672e48b, SUC_lexicon_tags_37 },
    { 0x7472e48b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad2e493, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4eb2e495, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0xda12e498, SUC_lexicon_tags_29 },
    { 0xde92e499, SUC_lexicon_tags_2 },
    { 0x2e92e499, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b2e4a0, SUC_lexicon_tags_17 },
    { 0xed2e4a0, SUC_lexicon_tags_15 },
    { 0x4a52e4a2, SUC_lexicon_tags_17 },
    { 0x9b32e4a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6972e4a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd32e4ac, SUC_lexicon_tags_39 },
    { 0xacf2e4ad, SUC_lexicon_tags_113 },
    { 0x1592e4ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb652e4b0, SUC_lexicon_tags_27 },
    { 0x5872e4b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f2e4b6, SUC_lexicon_tags_32 },
    { 0x3ef2e4b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab12e4bb, SUC_lexicon_tags_21 },
    { 0x6eb2e4bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b2e4c0, SUC_lexicon_tags_10 },
    { 0xa1b2e4c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b2e4c9, SUC_lexicon_tags_15 },
    { 0xf972e4ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a52e4cc, SUC_lexicon_tags_85 },
    { 0x1072e4cc, SUC_lexicon_tags_4 },
    { 0xfb92e4ce, SUC_lexicon_tags_48 },
    { 0x5212e4cf, SUC_lexicon_tags_6 },
    { 0x2a52e4d0, SUC_lexicon_tags_1 },
    { 0x5032e4d1, SUC_lexicon_tags_11 },
    { 0x9672e4d1, SUC_lexicon_tags_11 },
    { 0xabf2e4d1, SUC_lexicon_tags_43 },
    { 0x8972e4cf, SUC_lexicon_tags_1 },
    { 0xb0f2e4d5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b2e4da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa4b2e4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3852e4de, SUC_lexicon_tags_65 },
    { 0x4af2e4de, SUC_lexicon_tags_11 },
    { 0x6752e4e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f52e4e6, SUC_lexicon_tags_63 },
    { 0x1d12e4e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b92e4f2, SUC_lexicon_tags_11 },
    { 0x6112e4f3, SUC_lexicon_tags_7 },
    { 0x8df2e4f3, SUC_lexicon_tags_14 },
    { 0xb4b2e4f5, SUC_lexicon_tags_43 },
    { 0xc492e4f5, SUC_lexicon_tags_43 },
    { 0x70b2e4f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d2e4fd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf852e502, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff2e507, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfb72e509, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4692e50b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb32e50e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb2e511, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6332e513, SUC_lexicon_tags_25 },
    { 0x7872e514, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5532e518, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b92e51b, SUC_lexicon_tags_57 },
    { 0x9672e51c, SUC_lexicon_tags_25 },
    { 0x2272e51c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb2e525, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xab32e527, SUC_lexicon_tags_17 },
    { 0xbbb2e528, SUC_lexicon_tags_61 },
    { 0xdb32e529, SUC_lexicon_tags_40 },
    { 0x60b2e52a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4a32e52c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d52e532, SUC_lexicon_tags_25 },
    { 0x15d2e533, SUC_lexicon_tags_29 },
    { 0x1432e534, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c92e53a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe252e53c, SUC_lexicon_tags_1 },
    { 0xbb92e53c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab2e540, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d2e546, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc32e549, SUC_lexicon_tags_33 },
    { 0x8052e549, SUC_lexicon_tags_12 },
    { 0x95b2e54b, SUC_lexicon_tags_1 },
    { 0x7eb2e54c, SUC_lexicon_tags_47 },
    { 0x6032e54b, SUC_lexicon_tags_47 },
    { 0xf6d2e54c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe292e551, SUC_lexicon_tags_14 },
    { 0x2e72e551, SUC_lexicon_tags_17 },
    { 0xa8b2e553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb452e559, SUC_lexicon_tags_11 },
    { 0xbe52e55a, SUC_lexicon_tags_4 },
    { 0xeaf2e559, SUC_lexicon_tags_30 },
    { 0xb452e55c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x832e55e, SUC_lexicon_tags_12 },
    { 0x4f52e55f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa792e564, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd772e569, SUC_lexicon_tags_13 },
    { 0xa212e56a, SUC_lexicon_tags_379 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e12e56f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa452e571, SUC_lexicon_tags_27 },
    { 0xa312e572, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc8d2e574, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7312e576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee32e578, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7512e57c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8732e580, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c12e58a, SUC_lexicon_tags_0 },
    { 0x50d2e58b, SUC_lexicon_tags_30 },
    { 0x5ef2e58a, SUC_lexicon_tags_17 },
    { 0xf152e58d, SUC_lexicon_tags_17 },
    { 0x85f2e58e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2e595, SUC_lexicon_tags_10 },
    { 0x6852e595, SUC_lexicon_tags_27 },
    { 0x5b72e595, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x7332e59a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5f2e59c, SUC_lexicon_tags_45 },
    { 0x4cd2e59c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d2e5a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b52e5a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a52e5b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b2e5b6, SUC_lexicon_tags_140 },
    { 0x6412e5b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb252e5ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc812e5c4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x21d2e5c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaf32e5c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1852e5cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd52e5ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc412e5d1, SUC_lexicon_tags_82 },
    { 0xdf32e5d2, SUC_lexicon_tags_10 },
    { 0xf6b2e5d3, SUC_lexicon_tags_17 },
    { 0x5bb2e5d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af2e5d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2e5dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf32e5df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5f2e5e1, SUC_lexicon_tags_7 },
    { 0xc992e5e2, SUC_lexicon_tags_17 },
    { 0xeff2e5e2, SUC_lexicon_tags_11 },
    { 0x8f2e5e4, SUC_lexicon_tags_45 },
    { 0x4532e5e5, SUC_lexicon_tags_1 },
    { 0xc4d2e5e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd2e5e9, SUC_lexicon_tags_25 },
    { 0x1452e5e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd2e5ee, SUC_lexicon_tags_25 },
    { 0xa32e5ef, SUC_lexicon_tags_2 },
    { 0x7f52e5ee, SUC_lexicon_tags_27 },
    { 0x332e5ef, SUC_lexicon_tags_12 },
    { 0xeb92e5f2, SUC_lexicon_tags_0 },
    { 0x7ff2e5f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4892e5f6, SUC_lexicon_tags_12 },
    { 0xa6b2e5f7, SUC_lexicon_tags_11 },
    { 0x5872e5f8, SUC_lexicon_tags_14 },
    { 0x7292e5f8, SUC_lexicon_tags_37 },
    { 0x9292e5fa, SUC_lexicon_tags_17 },
    { 0xfc52e5fa, SUC_lexicon_tags_15 },
    { 0xa912e5f6, SUC_lexicon_tags_85 },
    { 0x3892e5fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f2e601, SUC_lexicon_tags_27 },
    { 0x7072e602, SUC_lexicon_tags_46 },
    { 0x5f72e602, SUC_lexicon_tags_62 },
    { 0x4092e604, SUC_lexicon_tags_5 },
    { 0x3772e602, SUC_lexicon_tags_6 },
    { 0xf9b2e601, SUC_lexicon_tags_26 },
    { 0x5012e607, SUC_lexicon_tags_108 },
    { 0xb7b2e608, SUC_lexicon_tags_9 },
    { 0x86d2e609, SUC_lexicon_tags_25 },
    { 0xe172e609, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xccd2e60c, SUC_lexicon_tags_2 },
    { 0x6f92e60d, SUC_lexicon_tags_65 },
    { 0xafd2e60e, SUC_lexicon_tags_27 },
    { 0x2d2e60f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4052e612, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7672e614, SUC_lexicon_tags_17 },
    { 0x7c52e615, SUC_lexicon_tags_49 },
    { 0x6b12e616, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf2e61e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6732e623, SUC_lexicon_tags_4 },
    { 0xd492e623, SUC_lexicon_tags_69 },
    { 0x97b2e625, SUC_lexicon_tags_2 },
    { 0x4392e626, SUC_lexicon_tags_14 },
    { 0xb612e624, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a32e62b, SUC_lexicon_tags_17 },
    { 0xabd2e62b, SUC_lexicon_tags_6 },
    { 0x4972e62d, SUC_lexicon_tags_17 },
    { 0x64b2e62e, SUC_lexicon_tags_47 },
    { 0x8072e62f, SUC_lexicon_tags_79 },
    { 0x6412e630, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb532e632, SUC_lexicon_tags_30 },
    { 0x2512e633, SUC_lexicon_tags_4 },
    { 0xceb2e634, SUC_lexicon_tags_77 },
    { 0xa292e633, SUC_lexicon_tags_11 },
    { 0xda32e634, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed2e640, SUC_lexicon_tags_30 },
    { 0x2512e640, SUC_lexicon_tags_25 },
    { 0x4532e642, SUC_lexicon_tags_17 },
    { 0x8832e640, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3472e647, SUC_lexicon_tags_27 },
    { 0xd752e647, SUC_lexicon_tags_46 },
    { 0xea52e649, SUC_lexicon_tags_1 },
    { 0x12f2e648, SUC_lexicon_tags_17 },
    { 0xfd32e649, SUC_lexicon_tags_25 },
    { 0xeb72e64b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe392e650, SUC_lexicon_tags_27 },
    { 0x26f2e651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf32e654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac32e656, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b2e659, SUC_lexicon_tags_62 },
    { 0x9892e65a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1cb2e65c, SUC_lexicon_tags_38 },
    { 0xff2e65d, SUC_lexicon_tags_15 },
    { 0x7bb2e65d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb832e665, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f2e668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8512e66e, SUC_lexicon_tags_17 },
    { 0x6412e66f, SUC_lexicon_tags_37 },
    { 0xccf2e670, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9592e673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d2e678, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb052e67a, SUC_lexicon_tags_27 },
    { 0x9172e67a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd192e682, SUC_lexicon_tags_2 },
    { 0xcaf2e683, SUC_lexicon_tags_12 },
    { 0x2cd2e684, SUC_lexicon_tags_31 },
    { 0x56f2e685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cb2e687, SUC_lexicon_tags_49 },
    { 0xeb12e688, SUC_lexicon_tags_3 },
    { 0x80d2e687, SUC_lexicon_tags_25 },
    { 0x4532e68a, SUC_lexicon_tags_25 },
    { 0xa1b2e68b, SUC_lexicon_tags_1 },
    { 0x3372e68c, SUC_lexicon_tags_60 },
    { 0x1132e688, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b12e693, SUC_lexicon_tags_7 },
    { 0xd92e694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef2e69b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6792e69e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd952e6a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe292e6a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3352e6a7, SUC_lexicon_tags_31 },
    { 0x3ff2e6a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5012e6ab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb1b2e6ad, SUC_lexicon_tags_45 },
    { 0x45f2e6ae, SUC_lexicon_tags_17 },
    { 0xf92e6ae, SUC_lexicon_tags_25 },
    { 0x96d2e6ae, SUC_lexicon_tags_11 },
    { 0xb412e6b0, SUC_lexicon_tags_39 },
    { 0x5612e6b0, SUC_lexicon_tags_11 },
    { 0x5f52e6b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2052e6b7, SUC_lexicon_tags_84 },
    { 0xa392e6b8, SUC_lexicon_tags_84 },
    { 0xb832e6b8, SUC_lexicon_tags_1 },
    { 0xab2e6b7, SUC_lexicon_tags_23 },
    { 0x4b92e6bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb2e6c0, SUC_lexicon_tags_25 },
    { 0x952e6c1, SUC_lexicon_tags_8 },
    { 0x952e6c2, SUC_lexicon_tags_25 },
    { 0x4d2e6c3, SUC_lexicon_tags_27 },
    { 0x15f2e6c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9412e6c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc272e6cf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf232e6d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd2e6d8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7452e6dc, SUC_lexicon_tags_42 },
    { 0x22f2e6dd, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1332e6e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5832e6e4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4072e6e6, SUC_lexicon_tags_19 },
    { 0x6c72e6e7, SUC_lexicon_tags_47 },
    { 0x59d2e6e8, SUC_lexicon_tags_9 },
    { 0xb612e6e9, SUC_lexicon_tags_23 },
    { 0xa792e6e6, SUC_lexicon_tags_15 },
    { 0x6fd2e6e6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9492e6ed, SUC_lexicon_tags_17 },
    { 0xa312e6ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81d2e6f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x652e6f2, SUC_lexicon_tags_47 },
    { 0xa572e6f3, SUC_lexicon_tags_1 },
    { 0xb0d2e6f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7372e6f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab2e6fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6472e700, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db2e716, SUC_lexicon_tags_38 },
    { 0xc212e717, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6652e719, SUC_lexicon_tags_12 },
    { 0x8c72e71a, SUC_lexicon_tags_12 },
    { 0xb752e71b, SUC_lexicon_tags_32 },
    { 0x372e71c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7092e71e, SUC_lexicon_tags_2 },
    { 0xf692e71e, SUC_lexicon_tags_7 },
    { 0xba72e720, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb2e723, SUC_lexicon_tags_39 },
    { 0xe992e724, SUC_lexicon_tags_8 },
    { 0xbf52e725, SUC_lexicon_tags_15 },
    { 0xab32e724, SUC_lexicon_tags_27 },
    { 0x9712e727, SUC_lexicon_tags_6 },
    { 0xc412e727, SUC_lexicon_tags_12 },
    { 0xe552e723, SUC_lexicon_tags_23 },
    { 0x3db2e728, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5632e72e, SUC_lexicon_tags_47 },
    { 0xadf2e72f, SUC_lexicon_tags_37 },
    { 0x99f2e730, SUC_lexicon_tags_31 },
    { 0x61b2e730, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c72e735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a92e738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf532e73a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2232e73c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa632e73e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xccb2e740, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39b2e742, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf2e747, SUC_lexicon_tags_12 },
    { 0x66b2e747, SUC_lexicon_tags_47 },
    { 0xb612e748, SUC_lexicon_tags_15 },
    { 0xf812e749, SUC_lexicon_tags_37 },
    { 0x5cd2e74b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x10d2e74d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd2e755, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xeff2e757, SUC_lexicon_tags_11 },
    { 0x1412e758, SUC_lexicon_tags_25 },
    { 0x5052e758, SUC_lexicon_tags_1 },
    { 0xb672e75a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4272e75d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5092e75f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f2e763, SUC_lexicon_tags_62 },
    { 0xcd52e763, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1712e769, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9052e771, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad92e775, SUC_lexicon_tags_49 },
    { 0xbd32e776, SUC_lexicon_tags_21 },
    { 0xdc52e775, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad2e77c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc832e77f, SUC_lexicon_tags_485 },
    { 0x9132e780, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b2e787, SUC_lexicon_tags_10 },
    { 0x36d2e788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c12e78a, SUC_lexicon_tags_68 },
    { 0xab72e78b, SUC_lexicon_tags_79 },
    { 0xa372e78b, SUC_lexicon_tags_11 },
    { 0xd732e78d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c52e78f, SUC_lexicon_tags_64 },
    { 0xd6d2e78f, SUC_lexicon_tags_15 },
    { 0x1572e791, SUC_lexicon_tags_60 },
    { 0xca52e792, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc72e796, SUC_lexicon_tags_25 },
    { 0x6812e796, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb52e799, SUC_lexicon_tags_37 },
    { 0x2612e79a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8672e79e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf852e7a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae52e7a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bd2e7a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb2e7b0, SUC_lexicon_tags_35 },
    { 0x8132e7b1, SUC_lexicon_tags_15 },
    { 0x4292e7b1, SUC_lexicon_tags_61 },
    { 0x4bd2e7b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47d2e7b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb2e7ba, SUC_lexicon_tags_17 },
    { 0x7bf2e7bb, SUC_lexicon_tags_15 },
    { 0x3a12e7bb, SUC_lexicon_tags_27 },
    { 0x8992e7bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x20d2e7bf, SUC_lexicon_tags_9 },
    { 0xa7f2e7c0, SUC_lexicon_tags_37 },
    { 0xd132e7c1, SUC_lexicon_tags_46 },
    { 0x4612e7bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1392e7c5, SUC_lexicon_tags_55 },
    { 0x1212e7c6, SUC_lexicon_tags_68 },
    { 0x3192e7c7, SUC_lexicon_tags_0 },
    { 0x69d2e7c5, SUC_lexicon_tags_0 },
    { 0x6f12e7c7, SUC_lexicon_tags_80 },
    { 0x3972e7ca, SUC_lexicon_tags_25 },
    { 0x97b2e7c9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8e72e7cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb2e7d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b32e7d2, SUC_lexicon_tags_7 },
    { 0xa492e7d2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f2e7d7, SUC_lexicon_tags_27 },
    { 0x6392e7d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7872e7de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ff2e7e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b92e7e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f2e7ea, SUC_lexicon_tags_23 },
    { 0x9532e7eb, SUC_lexicon_tags_17 },
    { 0xdf2e7ec, SUC_lexicon_tags_17 },
    { 0x7912e7ec, SUC_lexicon_tags_37 },
    { 0xbd12e7ea, SUC_lexicon_tags_16 },
    { 0x5392e7ef, SUC_lexicon_tags_17 },
    { 0x452e7f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f2e7f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6b2e7f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4ab2e7f9, SUC_lexicon_tags_31 },
    { 0x2692e7f9, SUC_lexicon_tags_27 },
    { 0xb212e7fb, SUC_lexicon_tags_7 },
    { 0x66f2e7fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d2e7ff, SUC_lexicon_tags_17 },
    { 0x3d2e7ff, SUC_lexicon_tags_19 },
    { 0x6532e800, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d12e803, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b52e809, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b92e80c, SUC_lexicon_tags_37 },
    { 0xcd72e80d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2252e810, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc12e815, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd272e81b, SUC_lexicon_tags_23 },
    { 0x4a92e81c, SUC_lexicon_tags_30 },
    { 0x6d32e81d, SUC_lexicon_tags_30 },
    { 0xdad2e81e, SUC_lexicon_tags_84 },
    { 0x5eb2e81c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc32e821, SUC_lexicon_tags_53 },
    { 0xcb52e822, SUC_lexicon_tags_47 },
    { 0x65d2e821, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9132e82a, SUC_lexicon_tags_39 },
    { 0x3e32e82a, SUC_lexicon_tags_151 },
    { 0xb692e82b, SUC_lexicon_tags_25 },
    { 0x2b2e82d, SUC_lexicon_tags_138 },
    { 0x9292e82e, SUC_lexicon_tags_23 },
    { 0xeef2e82d, SUC_lexicon_tags_11 },
    { 0x4a52e82e, SUC_lexicon_tags_27 },
    { 0xd832e82f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8752e83c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd192e83f, SUC_lexicon_tags_31 },
    { 0xc7b2e840, SUC_lexicon_tags_17 },
    { 0x8db2e841, SUC_lexicon_tags_14 },
    { 0x712e842, SUC_lexicon_tags_16 },
    { 0x9c52e842, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6052e84b, SUC_lexicon_tags_25 },
    { 0xc892e84c, SUC_lexicon_tags_23 },
    { 0xfe72e84d, SUC_lexicon_tags_17 },
    { 0xbbb2e84d, SUC_lexicon_tags_11 },
    { 0x99b2e84d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcc32e851, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef2e861, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb092e863, SUC_lexicon_tags_7 },
    { 0xb532e864, SUC_lexicon_tags_15 },
    { 0xfc12e864, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdc72e867, SUC_lexicon_tags_37 },
    { 0xb712e868, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2272e86e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b2e872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf2e879, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf2e882, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6192e888, SUC_lexicon_tags_55 },
    { 0xcaf2e888, SUC_lexicon_tags_7 },
    { 0xaa52e889, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb7f2e88c, SUC_lexicon_tags_27 },
    { 0x40f2e88d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd0d2e88f, SUC_lexicon_tags_15 },
    { 0x2fd2e88f, SUC_lexicon_tags_11 },
    { 0x79d2e88f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8212e894, SUC_lexicon_tags_16 },
    { 0x3132e895, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f32e89f, SUC_lexicon_tags_47 },
    { 0xda72e8a0, SUC_lexicon_tags_43 },
    { 0xb5b2e8a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a12e8a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b12e8aa, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0x1152e8ac, SUC_lexicon_tags_14 },
    { 0x2252e8ad, SUC_lexicon_tags_25 },
    { 0xce52e8ae, SUC_lexicon_tags_17 },
    { 0x3d12e8af, SUC_lexicon_tags_27 },
    { 0x3552e8ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc972e8ba, SUC_lexicon_tags_19 },
    { 0x3692e8bb, SUC_lexicon_tags_12 },
    { 0xb652e8bc, SUC_lexicon_tags_15 },
    { 0x6df2e8bb, SUC_lexicon_tags_25 },
    { 0xe532e8be, SUC_lexicon_tags_26 },
    { 0xd1d2e8bf, SUC_lexicon_tags_23 },
    { 0xb52e8bf, SUC_lexicon_tags_37 },
    { 0xe472e8bf, SUC_lexicon_tags_26 },
    { 0x5cd2e8c2, SUC_lexicon_tags_37 },
    { 0xbc32e8c3, SUC_lexicon_tags_25 },
    { 0xbcb2e8ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf132e8c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa172e8c9, SUC_lexicon_tags_11 },
    { 0x5532e8c9, SUC_lexicon_tags_9 },
    { 0x7c72e8ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e72e8cf, SUC_lexicon_tags_17 },
    { 0x1832e8cf, SUC_lexicon_tags_2 },
    { 0xbbd2e8d0, SUC_lexicon_tags_43 },
    { 0x2152e8d0, SUC_lexicon_tags_82 },
    { 0x5ab2e8d3, SUC_lexicon_tags_38 },
    { 0x7732e8d4, SUC_lexicon_tags_17 },
    { 0x55f2e8d5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed2e8d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d2e8e3, SUC_lexicon_tags_15 },
    { 0x26b2e8e3, SUC_lexicon_tags_12 },
    { 0x4f92e8e5, SUC_lexicon_tags_45 },
    { 0x9cb2e8e6, SUC_lexicon_tags_13 },
    { 0xbdb2e8e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f2e8ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d2e8f6, SUC_lexicon_tags_2 },
    { 0x4092e8f7, SUC_lexicon_tags_37 },
    { 0x3312e8f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d2e8fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b2e8fe, SUC_lexicon_tags_11 },
    { 0xb3b2e8ff, SUC_lexicon_tags_12 },
    { 0x4732e900, SUC_lexicon_tags_37 },
    { 0xfb2e8ff, SUC_lexicon_tags_25 },
    { 0xf192e901, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf2e906, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9632e90b, SUC_lexicon_tags_7 },
    { 0xa452e90c, SUC_lexicon_tags_66 },
    { 0x27f2e90c, SUC_lexicon_tags_55 },
    { 0x97d2e90c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b2e911, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd52e913, SUC_lexicon_tags_30 },
    { 0xa2b2e914, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d2e917, SUC_lexicon_tags_37 },
    { 0xbcd2e918, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e12e91a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18f2e91c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x14d2e91e, SUC_lexicon_tags_45 },
    { 0xf9b2e91f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4532e921, SUC_lexicon_tags_17 },
    { 0x70b2e922, SUC_lexicon_tags_23 },
    { 0xbd2e922, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4572e925, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5492e929, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd2e932, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b32e935, SUC_lexicon_tags_10 },
    { 0xbbf2e935, SUC_lexicon_tags_66 },
    { 0xb0b2e937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb2e93b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd132e93e, SUC_lexicon_tags_25 },
    { 0xe8b2e93f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8b2e941, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a72e943, SUC_lexicon_tags_11 },
    { 0x4c72e943, SUC_lexicon_tags_47 },
    { 0x4572e945, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d2e948, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f2e94b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4892e94e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1472e951, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb2e955, SUC_lexicon_tags_27 },
    { 0xe8b2e956, SUC_lexicon_tags_47 },
    { 0x1692e957, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb2e95b, SUC_lexicon_tags_17 },
    { 0xbbd2e95c, SUC_lexicon_tags_12 },
    { 0x1af2e95d, SUC_lexicon_tags_47 },
    { 0x9012e95e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8592e962, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e72e965, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b2e96a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b2e96c, SUC_lexicon_tags_6 },
    { 0xf2f2e96d, SUC_lexicon_tags_62 },
    { 0x1d12e96d, SUC_lexicon_tags_18 },
    { 0xed52e96f, SUC_lexicon_tags_32 },
    { 0xcf92e96d, SUC_lexicon_tags_15 },
    { 0xabb2e96f, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x9e12e973, SUC_lexicon_tags_12 },
    { 0xce92e974, SUC_lexicon_tags_21 },
    { 0x352e974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f2e979, SUC_lexicon_tags_1 },
    { 0x2f2e979, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b2e985, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4612e988, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb912e98a, SUC_lexicon_tags_31 },
    { 0x6ad2e98b, SUC_lexicon_tags_1 },
    { 0x8a92e98a, SUC_lexicon_tags_12 },
    { 0x2752e98d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce52e98f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8852e995, SUC_lexicon_tags_11 },
    { 0x25f2e996, SUC_lexicon_tags_11 },
    { 0x6b52e997, SUC_lexicon_tags_11 },
    { 0xecf2e998, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd2e99d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf2e9a4, SUC_lexicon_tags_27 },
    { 0x8852e9a4, SUC_lexicon_tags_25 },
    { 0x8252e9a6, SUC_lexicon_tags_17 },
    { 0xbfd2e9a4, SUC_lexicon_tags_25 },
    { 0xd352e9a8, SUC_lexicon_tags_39 },
    { 0x3232e9a9, SUC_lexicon_tags_35 },
    { 0xb972e9a8, SUC_lexicon_tags_37 },
    { 0x3d92e9aa, SUC_lexicon_tags_47 },
    { 0xbdf2e9a5, SUC_lexicon_tags_83 },
    { 0x4e72e9a6, SUC_lexicon_tags_25 },
    { 0x1232e9ae, SUC_lexicon_tags_29 },
    { 0x2112e9ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6292e9b2, SUC_lexicon_tags_7 },
    { 0x952e9b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2992e9bb, SUC_lexicon_tags_1 },
    { 0xb252e9bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e92e9bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8572e9c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e52e9c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f32e9c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1012e9cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc252e9d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e92e9d5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2912e9d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb892e9d9, SUC_lexicon_tags_17 },
    { 0x9ed2e9da, SUC_lexicon_tags_17 },
    { 0x5372e9db, SUC_lexicon_tags_66 },
    { 0xb812e9db, SUC_lexicon_tags_8 },
    { 0x5f72e9dd, SUC_lexicon_tags_11 },
    { 0x8b72e9de, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x1af2e9e0, SUC_lexicon_tags_17 },
    { 0xaa12e9e1, SUC_lexicon_tags_17 },
    { 0x24d2e9e2, SUC_lexicon_tags_11 },
    { 0xe0b2e9e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5772e9e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f72e9e9, SUC_lexicon_tags_27 },
    { 0x4932e9ea, SUC_lexicon_tags_9 },
    { 0xca52e9eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b2e9f1, SUC_lexicon_tags_23 },
    { 0x1b32e9f2, SUC_lexicon_tags_31 },
    { 0x7632e9f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f2e9f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb212e9fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10d2e9fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb92ea03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89b2ea05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab32ea09, SUC_lexicon_tags_17 },
    { 0x7c92ea09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab2ea0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f12ea11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3072ea15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69b2ea17, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4472ea1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce12ea1d, SUC_lexicon_tags_37 },
    { 0xa232ea1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3872ea29, SUC_lexicon_tags_35 },
    { 0x38d2ea2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6452ea2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b2ea2f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd172ea31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd2d2ea33, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf2ea36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7472ea39, SUC_lexicon_tags_15 },
    { 0x8312ea3a, SUC_lexicon_tags_63 },
    { 0xa552ea3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9dd2ea3d, SUC_lexicon_tags_12 },
    { 0x8412ea3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d2ea41, SUC_lexicon_tags_32 },
    { 0xff92ea42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d2ea45, SUC_lexicon_tags_1 },
    { 0x3ef2ea46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc52ea4b, SUC_lexicon_tags_49 },
    { 0xec32ea4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc072ea4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1d2ea50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb2ea53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd52ea55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5192ea58, SUC_lexicon_tags_23 },
    { 0xadd2ea58, SUC_lexicon_tags_17 },
    { 0x6572ea58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ea5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f2ea62, SUC_lexicon_tags_17 },
    { 0x4652ea63, SUC_lexicon_tags_2 },
    { 0x2d92ea64, SUC_lexicon_tags_15 },
    { 0x66d2ea65, SUC_lexicon_tags_30 },
    { 0xf6f2ea63, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c52ea6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4df2ea6c, SUC_lexicon_tags_23 },
    { 0x6a32ea6c, SUC_lexicon_tags_28 },
    { 0x8f32ea6e, SUC_lexicon_tags_26 },
    { 0x42f2ea6e, SUC_lexicon_tags_19 },
    { 0xe092ea6f, SUC_lexicon_tags_12 },
    { 0x3832ea71, SUC_lexicon_tags_16 },
    { 0x1592ea6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca12ea76, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d2ea79, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc572ea7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb2ea83, SUC_lexicon_tags_15 },
    { 0x9ed2ea83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f2ea90, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2952ea94, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed2ea97, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfeb2ea99, SUC_lexicon_tags_11 },
    { 0xb72ea9a, SUC_lexicon_tags_64 },
    { 0x7572ea9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd2ea9e, SUC_lexicon_tags_45 },
    { 0x90f2ea9f, SUC_lexicon_tags_151 },
    { 0xbf92eaa0, SUC_lexicon_tags_37 },
    { 0x7e72eaa0, SUC_lexicon_tags_37 },
    { 0x6e72eaa2, SUC_lexicon_tags_104 },
    { 0xb332eaa3, SUC_lexicon_tags_17 },
    { 0x7832eaa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8232eaa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad2eaac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b2eaae, SUC_lexicon_tags_11 },
    { 0xfdf2eaaf, SUC_lexicon_tags_37 },
    { 0x1032eab0, SUC_lexicon_tags_11 },
    { 0xcdd2eab1, SUC_lexicon_tags_302 },
    { 0xad92eab1, SUC_lexicon_tags_17 },
    { 0xdcf2eab3, SUC_lexicon_tags_0 },
    { 0xd6b2eab3, SUC_lexicon_tags_60 },
    { 0xb7d2eab3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x54f2eab7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4632eab9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1552eabe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7812eac1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d32eac3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe312eac8, SUC_lexicon_tags_6 },
    { 0x1eb2eac9, SUC_lexicon_tags_21 },
    { 0xa8d2eaca, SUC_lexicon_tags_34 },
    { 0xb552eacb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6272ead4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf112eadd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde12eae0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2bd2eae2, SUC_lexicon_tags_32 },
    { 0xce32eae3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1672eae5, SUC_lexicon_tags_19 },
    { 0xef72eae6, SUC_lexicon_tags_15 },
    { 0x3df2eae7, SUC_lexicon_tags_4 },
    { 0xf732eae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d2eaf1, SUC_lexicon_tags_17 },
    { 0x9bb2eaf2, SUC_lexicon_tags_37 },
    { 0xf0f2eaf3, SUC_lexicon_tags_31 },
    { 0x3f72eaf3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1172eaf7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2952eaf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93d2eafb, SUC_lexicon_tags_12 },
    { 0xc212eafb, SUC_lexicon_tags_25 },
    { 0xf92eafd, SUC_lexicon_tags_12 },
    { 0xc7f2eafc, SUC_lexicon_tags_65 },
    { 0x5752eafc, SUC_lexicon_tags_55 },
    { 0xe352eb00, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8572eb02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd972eb05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b2eb0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c52eb0f, SUC_lexicon_tags_45 },
    { 0x65f2eb0f, SUC_lexicon_tags_43 },
    { 0x85d2eb11, SUC_lexicon_tags_6 },
    { 0xac32eb0f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6672eb15, SUC_lexicon_tags_14 },
    { 0x46d2eb15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd272eb1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1192eb22, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b2eb28, SUC_lexicon_tags_17 },
    { 0xeab2eb29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb2eb2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc332eb31, SUC_lexicon_tags_25 },
    { 0xfc32eb31, SUC_lexicon_tags_38 },
    { 0x9ad2eb31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb2eb36, SUC_lexicon_tags_12 },
    { 0x3d32eb37, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x352eb3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c32eb3c, SUC_lexicon_tags_10 },
    { 0x7372eb3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac72eb42, SUC_lexicon_tags_23 },
    { 0x6412eb43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd12eb47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d2eb4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf572eb51, SUC_lexicon_tags_23 },
    { 0x98d2eb52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd2eb56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2132eb5c, SUC_lexicon_tags_65 },
    { 0xfe12eb5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x32d2eb5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b2eb63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed52eb6f, SUC_lexicon_tags_17 },
    { 0xabb2eb6f, SUC_lexicon_tags_12 },
    { 0x12d2eb71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c12eb76, SUC_lexicon_tags_29 },
    { 0xd3b2eb77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4992eb79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97d2eb7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa352eb7d, SUC_lexicon_tags_11 },
    { 0xf5d2eb7e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7372eb84, SUC_lexicon_tags_45 },
    { 0xcf92eb84, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1072eb88, SUC_lexicon_tags_7 },
    { 0x9d2eb89, SUC_lexicon_tags_63 },
    { 0x8d52eb89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd2eb90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f72eb92, SUC_lexicon_tags_37 },
    { 0xba12eb93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd2eb97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d52eb9b, SUC_lexicon_tags_27 },
    { 0xfed2eb9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a32eba2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2412ebb0, SUC_lexicon_tags_15 },
    { 0xa12ebb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf2ebb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2432ebb9, SUC_lexicon_tags_7 },
    { 0xf472ebb9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4992ebbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd632ebbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab92ebc0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9072ebc6, SUC_lexicon_tags_11 },
    { 0xce52ebc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca32ebca, SUC_lexicon_tags_37 },
    { 0x7f52ebca, SUC_lexicon_tags_17 },
    { 0x68f2ebcc, SUC_lexicon_tags_17 },
    { 0xedf2ebcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b92ebcf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde12ebd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d2ebdb, SUC_lexicon_tags_17 },
    { 0x42b2ebdc, SUC_lexicon_tags_16 },
    { 0x1872ebdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5112ebe0, SUC_lexicon_tags_38 },
    { 0xd32ebe0, SUC_lexicon_tags_25 },
    { 0xbb2ebe2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf852ebe9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b2ebee, SUC_lexicon_tags_66 },
    { 0xf852ebef, SUC_lexicon_tags_27 },
    { 0x7d52ebef, SUC_lexicon_tags_68 },
    { 0xf6b2ebf0, SUC_lexicon_tags_15 },
    { 0xc772ebf2, SUC_lexicon_tags_17 },
    { 0x8eb2ebf3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef2ebf7, SUC_lexicon_tags_7 },
    { 0xe012ebf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f2ebfb, SUC_lexicon_tags_25 },
    { 0x73d2ebfb, SUC_lexicon_tags_57 },
    { 0xe7f2ebfd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd192ec04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a32ec0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd2ec0f, SUC_lexicon_tags_37 },
    { 0x3232ec10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa392ec12, SUC_lexicon_tags_65 },
    { 0x27b2ec13, SUC_lexicon_tags_11 },
    { 0x3852ec14, SUC_lexicon_tags_27 },
    { 0x6af2ec14, SUC_lexicon_tags_12 },
    { 0x76b2ec14, SUC_lexicon_tags_29 },
    { 0x6892ec14, SUC_lexicon_tags_87 },
    { 0xff12ec13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc52ec1b, SUC_lexicon_tags_11 },
    { 0x68f2ec1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x272ec1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb2ec23, SUC_lexicon_tags_38 },
    { 0xa592ec23, SUC_lexicon_tags_7 },
    { 0xd952ec24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbad2ec27, SUC_lexicon_tags_45 },
    { 0x7012ec27, SUC_lexicon_tags_25 },
    { 0xb972ec29, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf0f2ec2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd2ec36, SUC_lexicon_tags_9 },
    { 0x2432ec37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd92ec39, SUC_lexicon_tags_82 },
    { 0x2752ec3a, SUC_lexicon_tags_11 },
    { 0x49f2ec3b, SUC_lexicon_tags_32 },
    { 0xce92ec3a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd052ec40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2112ec45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef72ec48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9012ec4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff2ec4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e12ec52, SUC_lexicon_tags_136 },
    { 0xd652ec53, SUC_lexicon_tags_10 },
    { 0x23b2ec53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc752ec5e, SUC_lexicon_tags_17 },
    { 0xc272ec5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9952ec68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cb2ec6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b2ec6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd912ec71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f2ec76, SUC_lexicon_tags_17 },
    { 0x21b2ec76, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe652ec7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88f2ec7f, SUC_lexicon_tags_46 },
    { 0xe0f2ec80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af2ec89, SUC_lexicon_tags_33 },
    { 0xbb12ec8a, SUC_lexicon_tags_60 },
    { 0x6912ec89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef2ec94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b2ec98, SUC_lexicon_tags_49 },
    { 0x32d2ec98, SUC_lexicon_tags_33 },
    { 0xb532ec99, SUC_lexicon_tags_118 },
    { 0x1d32ec99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c12eca1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd0f2eca3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5092eca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5692eca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef72ecab, SUC_lexicon_tags_63 },
    { 0xc152ecab, SUC_lexicon_tags_19 },
    { 0xf1b2ecab, SUC_lexicon_tags_7 },
    { 0xca32ecac, SUC_lexicon_tags_25 },
    { 0x8c72ecaf, SUC_lexicon_tags_17 },
    { 0x9ef2ecaf, SUC_lexicon_tags_68 },
    { 0x84b2ecae, SUC_lexicon_tags_25 },
    { 0xf4b2ecb2, SUC_lexicon_tags_65 },
    { 0xda32ecad, SUC_lexicon_tags_12 },
    { 0x5b32ecb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad12ecb9, SUC_lexicon_tags_8 },
    { 0x6ab2ecb9, SUC_lexicon_tags_61 },
    { 0x1b2ecbb, SUC_lexicon_tags_35 },
    { 0x5432ecbb, SUC_lexicon_tags_43 },
    { 0x6692ecbb, SUC_lexicon_tags_49 },
    { 0x8692ecbe, SUC_lexicon_tags_26 },
    { 0xe692ecbe, SUC_lexicon_tags_46 },
    { 0x4fb2ecba, SUC_lexicon_tags_47 },
    { 0x97b2ecbd, SUC_lexicon_tags_31 },
    { 0x8072ecc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f2ecc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc712ecc8, SUC_lexicon_tags_25 },
    { 0xde92ecc9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2832eccb, SUC_lexicon_tags_759 },
    { 0, NULL },
    { 0x66d2eccd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fd2eccf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f2ecd6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdd92ecd8, SUC_lexicon_tags_17 },
    { 0x9c72ecd8, SUC_lexicon_tags_33 },
    { 0xe332ecda, SUC_lexicon_tags_9 },
    { 0xba72ecd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb2ecdd, SUC_lexicon_tags_2 },
    { 0xfe72ecde, SUC_lexicon_tags_77 },
    { 0x5f12ecde, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xd552ece1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6372ece6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f32ecea, SUC_lexicon_tags_23 },
    { 0x4992eceb, SUC_lexicon_tags_711 },
    { 0x74f2ecec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8852ecef, SUC_lexicon_tags_87 },
    { 0xeed2ecef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa392ecf2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf2ecf6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1f12ecf8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa072ecfc, SUC_lexicon_tags_138 },
    { 0x5b32ecfd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f2ed00, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb412ed06, SUC_lexicon_tags_29 },
    { 0x4192ed06, SUC_lexicon_tags_456 },
    { 0x8b92ed08, SUC_lexicon_tags_15 },
    { 0x44f2ed09, SUC_lexicon_tags_15 },
    { 0x83d2ed0a, SUC_lexicon_tags_37 },
    { 0x5232ed0b, SUC_lexicon_tags_1 },
    { 0x30d2ed0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9892ed0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb2ed14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5292ed19, SUC_lexicon_tags_47 },
    { 0x5852ed19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5352ed1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef32ed20, SUC_lexicon_tags_7 },
    { 0x2372ed21, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x652ed23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6692ed25, SUC_lexicon_tags_2 },
    { 0xcbd2ed25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17f2ed28, SUC_lexicon_tags_66 },
    { 0xa1d2ed29, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd8f2ed2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d2ed36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcf2ed38, SUC_lexicon_tags_17 },
    { 0x9c52ed39, SUC_lexicon_tags_25 },
    { 0x5372ed3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d72ed41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5192ed44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa92ed4b, SUC_lexicon_tags_17 },
    { 0xb8b2ed4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6692ed50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd2ed53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b32ed55, SUC_lexicon_tags_1249 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd2ed59, SUC_lexicon_tags_4 },
    { 0x13f2ed59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc092ed5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xef32ed61, SUC_lexicon_tags_12 },
    { 0xf12ed61, SUC_lexicon_tags_29 },
    { 0xb12ed63, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d92ed66, SUC_lexicon_tags_12 },
    { 0x6952ed67, SUC_lexicon_tags_46 },
    { 0xbf32ed66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4592ed6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27b2ed6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b2ed71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b2ed74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95d2ed76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xacd2ed78, SUC_lexicon_tags_43 },
    { 0x7252ed78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd152ed7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad2ed81, SUC_lexicon_tags_54 },
    { 0x3bf2ed81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d2ed85, SUC_lexicon_tags_0 },
    { 0x2cf2ed85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf192ed91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5592ed93, SUC_lexicon_tags_9 },
    { 0x2272ed93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba52ed96, SUC_lexicon_tags_11 },
    { 0xc8b2ed97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6732ed9b, SUC_lexicon_tags_38 },
    { 0xa7b2ed9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72eda3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x37f2eda5, SUC_lexicon_tags_7 },
    { 0xe2b2eda5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f12eda9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa0b2edab, SUC_lexicon_tags_25 },
    { 0x6d52edab, SUC_lexicon_tags_11 },
    { 0xa912edad, SUC_lexicon_tags_69 },
    { 0xb7b2edad, SUC_lexicon_tags_23 },
    { 0xce92edac, SUC_lexicon_tags_65 },
    { 0x7012edab, SUC_lexicon_tags_15 },
    { 0x6df2edae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b2edb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f2edb6, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x4a32edb8, SUC_lexicon_tags_15 },
    { 0xb7f2edb9, SUC_lexicon_tags_57 },
    { 0xd732edba, SUC_lexicon_tags_46 },
    { 0xe432edbb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6672edc5, SUC_lexicon_tags_15 },
    { 0x9372edc6, SUC_lexicon_tags_60 },
    { 0x63b2edc7, SUC_lexicon_tags_1 },
    { 0x7512edc8, SUC_lexicon_tags_1074 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc412edcc, SUC_lexicon_tags_15 },
    { 0xfef2edcd, SUC_lexicon_tags_9 },
    { 0x3592edce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd1b2edd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d32edd3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d2edd9, SUC_lexicon_tags_45 },
    { 0x2452edda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e92edde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f72ede3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f72ede8, SUC_lexicon_tags_2 },
    { 0x3112ede9, SUC_lexicon_tags_57 },
    { 0x6a72ede9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7952edec, SUC_lexicon_tags_37 },
    { 0xd352eded, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc012edef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe292edf5, SUC_lexicon_tags_25 },
    { 0x52b2edf5, SUC_lexicon_tags_27 },
    { 0xacd2edf7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd32edfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf2ee00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d2ee03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2af2ee05, SUC_lexicon_tags_17 },
    { 0x6b52ee06, SUC_lexicon_tags_37 },
    { 0xf832ee07, SUC_lexicon_tags_28 },
    { 0x1a12ee05, SUC_lexicon_tags_33 },
    { 0xe772ee08, SUC_lexicon_tags_11 },
    { 0xdfb2ee0a, SUC_lexicon_tags_27 },
    { 0x2f52ee0b, SUC_lexicon_tags_12 },
    { 0xc3b2ee0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed52ee0f, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0xda12ee12, SUC_lexicon_tags_10 },
    { 0xabb2ee13, SUC_lexicon_tags_63 },
    { 0xce12ee14, SUC_lexicon_tags_63 },
    { 0x90b2ee13, SUC_lexicon_tags_11 },
    { 0xab72ee16, SUC_lexicon_tags_47 },
    { 0x9ef2ee16, SUC_lexicon_tags_0 },
    { 0xaf32ee15, SUC_lexicon_tags_3 },
    { 0x8b12ee14, SUC_lexicon_tags_5 },
    { 0x8852ee15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff2ee26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe132ee29, SUC_lexicon_tags_38 },
    { 0xfb2ee2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda72ee2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb52ee33, SUC_lexicon_tags_39 },
    { 0xbc72ee34, SUC_lexicon_tags_9 },
    { 0xcc92ee33, SUC_lexicon_tags_27 },
    { 0xffb2ee36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x48f2ee38, SUC_lexicon_tags_1 },
    { 0x7432ee39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf12ee3b, SUC_lexicon_tags_11 },
    { 0xcdd2ee3c, SUC_lexicon_tags_2 },
    { 0x3d52ee3c, SUC_lexicon_tags_33 },
    { 0xadd2ee3e, SUC_lexicon_tags_68 },
    { 0x75d2ee3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4052ee43, SUC_lexicon_tags_17 },
    { 0x9cf2ee43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef12ee49, SUC_lexicon_tags_45 },
    { 0x9872ee4a, SUC_lexicon_tags_4 },
    { 0x372ee4a, SUC_lexicon_tags_1 },
    { 0x4e72ee4b, SUC_lexicon_tags_60 },
    { 0x8f92ee4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce12ee52, SUC_lexicon_tags_1 },
    { 0x72f2ee53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8592ee56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7772ee58, SUC_lexicon_tags_0 },
    { 0x8b12ee58, SUC_lexicon_tags_17 },
    { 0x8312ee58, SUC_lexicon_tags_37 },
    { 0x8032ee59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0b2ee5d, SUC_lexicon_tags_37 },
    { 0x9a32ee5d, SUC_lexicon_tags_4 },
    { 0xe032ee5f, SUC_lexicon_tags_12 },
    { 0x3792ee60, SUC_lexicon_tags_11 },
    { 0x39f2ee5f, SUC_lexicon_tags_27 },
    { 0xd7b2ee60, SUC_lexicon_tags_19 },
    { 0xf992ee5e, SUC_lexicon_tags_49 },
    { 0xdb12ee64, SUC_lexicon_tags_13 },
    { 0x53d2ee65, SUC_lexicon_tags_60 },
    { 0x2772ee66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd32ee6d, SUC_lexicon_tags_2 },
    { 0x6212ee6e, SUC_lexicon_tags_27 },
    { 0xcc52ee6f, SUC_lexicon_tags_53 },
    { 0xafb2ee6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d72ee73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f2ee75, SUC_lexicon_tags_37 },
    { 0x90d2ee76, SUC_lexicon_tags_16 },
    { 0x1b2ee76, SUC_lexicon_tags_29 },
    { 0x2bf2ee78, SUC_lexicon_tags_0 },
    { 0xc812ee79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9712ee7c, SUC_lexicon_tags_10 },
    { 0x8d92ee7d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2672ee80, SUC_lexicon_tags_26 },
    { 0xaff2ee80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f12ee83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5892ee85, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf672ee87, SUC_lexicon_tags_37 },
    { 0xe72ee87, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f52ee8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8812ee8f, SUC_lexicon_tags_23 },
    { 0xa712ee90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d2ee92, SUC_lexicon_tags_7 },
    { 0x7292ee93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd8d2ee95, SUC_lexicon_tags_23 },
    { 0x8872ee96, SUC_lexicon_tags_17 },
    { 0xdfd2ee96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd72ee9a, SUC_lexicon_tags_17 },
    { 0x4c32ee9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a92ee9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b52eea2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56b2eea4, SUC_lexicon_tags_33 },
    { 0x9992eea4, SUC_lexicon_tags_43 },
    { 0x80b2eea6, SUC_lexicon_tags_27 },
    { 0xf432eea7, SUC_lexicon_tags_25 },
    { 0x5552eea8, SUC_lexicon_tags_68 },
    { 0xb9d2eea4, SUC_lexicon_tags_15 },
    { 0xc6f2eea5, SUC_lexicon_tags_3 },
    { 0x9032eea6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f52eeae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e52eeb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9872eeb5, SUC_lexicon_tags_65 },
    { 0xcb52eeb5, SUC_lexicon_tags_27 },
    { 0x7ef2eeb6, SUC_lexicon_tags_12 },
    { 0x3872eeb5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf2eebd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6352eec0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa012eec3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64f2eec5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b2eec8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbf52eeca, SUC_lexicon_tags_15 },
    { 0x2232eecb, SUC_lexicon_tags_9 },
    { 0xc632eeca, SUC_lexicon_tags_11 },
    { 0xeeb2eecc, SUC_lexicon_tags_27 },
    { 0xfeb2eece, SUC_lexicon_tags_12 },
    { 0x3872eecd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5312eed2, SUC_lexicon_tags_22 },
    { 0x6df2eed3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdc12eed5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb2eed8, SUC_lexicon_tags_37 },
    { 0x38f2eed9, SUC_lexicon_tags_25 },
    { 0xf992eeda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ed2eedc, SUC_lexicon_tags_17 },
    { 0x1872eedd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b2eee0, SUC_lexicon_tags_23 },
    { 0xe852eee0, SUC_lexicon_tags_25 },
    { 0x4852eee2, SUC_lexicon_tags_11 },
    { 0x3a92eee1, SUC_lexicon_tags_38 },
    { 0x2b72eee4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf2eee7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f72eeee, SUC_lexicon_tags_25 },
    { 0x65d2eeef, SUC_lexicon_tags_95 },
    { 0x9472eef0, SUC_lexicon_tags_14 },
    { 0x1572eef1, SUC_lexicon_tags_17 },
    { 0x43f2eef2, SUC_lexicon_tags_63 },
    { 0x64b2eef2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6eb2eef5, SUC_lexicon_tags_108 },
    { 0x2192eef6, SUC_lexicon_tags_41 },
    { 0xa892eef7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b2eefa, SUC_lexicon_tags_14 },
    { 0x9b92eefa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd592ef02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5452ef05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d2ef07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a92ef09, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd392ef13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd2ef16, SUC_lexicon_tags_25 },
    { 0xd1f2ef17, SUC_lexicon_tags_49 },
    { 0x2132ef18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf92ef1c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1292ef1f, SUC_lexicon_tags_32 },
    { 0xe8d2ef20, SUC_lexicon_tags_55 },
    { 0x152ef21, SUC_lexicon_tags_11 },
    { 0xbef2ef21, SUC_lexicon_tags_68 },
    { 0x2e12ef23, SUC_lexicon_tags_1 },
    { 0x46d2ef24, SUC_lexicon_tags_37 },
    { 0x4eb2ef25, SUC_lexicon_tags_17 },
    { 0x8832ef26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3232ef29, SUC_lexicon_tags_23 },
    { 0xd272ef2a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x852ef2d, SUC_lexicon_tags_1 },
    { 0x4732ef2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4672ef30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75d2ef32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f2ef3e, SUC_lexicon_tags_29 },
    { 0x392ef3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3872ef48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb2ef4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1832ef53, SUC_lexicon_tags_17 },
    { 0xe372ef54, SUC_lexicon_tags_11 },
    { 0x5692ef54, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f32ef58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8272ef5a, SUC_lexicon_tags_15 },
    { 0x5172ef5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabb2ef5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec72ef60, SUC_lexicon_tags_63 },
    { 0x8112ef60, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f2ef64, SUC_lexicon_tags_9 },
    { 0x5fb2ef65, SUC_lexicon_tags_26 },
    { 0xc712ef65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c52ef69, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xab72ef6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebd2ef6d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x132ef71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2312ef73, SUC_lexicon_tags_25 },
    { 0xf7d2ef74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8df2ef76, SUC_lexicon_tags_15 },
    { 0x4272ef77, SUC_lexicon_tags_28 },
    { 0x1b32ef78, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c92ef7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc112ef80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf952ef83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d52ef85, SUC_lexicon_tags_56 },
    { 0x3512ef86, SUC_lexicon_tags_1 },
    { 0x89f2ef86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc352ef8d, SUC_lexicon_tags_37 },
    { 0x9812ef8e, SUC_lexicon_tags_37 },
    { 0x52d2ef8f, SUC_lexicon_tags_30 },
    { 0x79f2ef8d, SUC_lexicon_tags_63 },
    { 0x5d12ef8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b2ef95, SUC_lexicon_tags_6 },
    { 0xa812ef96, SUC_lexicon_tags_36 },
    { 0x4f92ef97, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb2ef9c, SUC_lexicon_tags_60 },
    { 0x6e12ef9d, SUC_lexicon_tags_25 },
    { 0x2fd2ef9e, SUC_lexicon_tags_15 },
    { 0x9ad2ef9f, SUC_lexicon_tags_12 },
    { 0x84f2efa0, SUC_lexicon_tags_27 },
    { 0x1472ef9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7eb2efa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x732efa7, SUC_lexicon_tags_12 },
    { 0x88b2efa8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f2efab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c52efae, SUC_lexicon_tags_17 },
    { 0x12b2efae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d2efb2, SUC_lexicon_tags_38 },
    { 0xaeb2efb3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb292efb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2272efb9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f2efbd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd772efbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9952efc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db2efc4, SUC_lexicon_tags_29 },
    { 0x6d12efc5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8812efc7, SUC_lexicon_tags_37 },
    { 0x59b2efc8, SUC_lexicon_tags_25 },
    { 0x80f2efc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7f2efcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112efd0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1052efd8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f2efde, SUC_lexicon_tags_49 },
    { 0xbc92efdf, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xae72efe2, SUC_lexicon_tags_48 },
    { 0xe2f2efe3, SUC_lexicon_tags_49 },
    { 0xa9b2efe4, SUC_lexicon_tags_37 },
    { 0x7892efe4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97d2efe7, SUC_lexicon_tags_4 },
    { 0xe8b2efe7, SUC_lexicon_tags_11 },
    { 0x7b92efe8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f92efed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d12efef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c72eff5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe312eff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d92eff9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d32effb, SUC_lexicon_tags_25 },
    { 0xad72effb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb952f004, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6612f008, SUC_lexicon_tags_12 },
    { 0x4432f009, SUC_lexicon_tags_17 },
    { 0x6872f00a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd352f00c, SUC_lexicon_tags_38 },
    { 0x1352f00d, SUC_lexicon_tags_25 },
    { 0x8f32f00d, SUC_lexicon_tags_25 },
    { 0x2252f00f, SUC_lexicon_tags_25 },
    { 0x21d2f010, SUC_lexicon_tags_0 },
    { 0x90b2f011, SUC_lexicon_tags_66 },
    { 0x6232f00d, SUC_lexicon_tags_31 },
    { 0x3632f013, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d2f01b, SUC_lexicon_tags_37 },
    { 0x6c12f01c, SUC_lexicon_tags_37 },
    { 0x5d52f01c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4772f020, SUC_lexicon_tags_16 },
    { 0xe5b2f020, SUC_lexicon_tags_1 },
    { 0xf712f022, SUC_lexicon_tags_4 },
    { 0x3c12f023, SUC_lexicon_tags_47 },
    { 0xc312f023, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4012f026, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd032f029, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9292f02d, SUC_lexicon_tags_35 },
    { 0x7a12f02e, SUC_lexicon_tags_1 },
    { 0x97f2f02f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4272f033, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b92f037, SUC_lexicon_tags_10 },
    { 0xcfd2f038, SUC_lexicon_tags_27 },
    { 0x6c72f039, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f2f03c, SUC_lexicon_tags_37 },
    { 0x4532f03d, SUC_lexicon_tags_4 },
    { 0xc32f03c, SUC_lexicon_tags_27 },
    { 0xbd92f03f, SUC_lexicon_tags_29 },
    { 0x2372f040, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaf12f042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27d2f044, SUC_lexicon_tags_37 },
    { 0xdf72f045, SUC_lexicon_tags_43 },
    { 0x3752f046, SUC_lexicon_tags_8 },
    { 0xd952f047, SUC_lexicon_tags_42 },
    { 0x63f2f048, SUC_lexicon_tags_85 },
    { 0xb292f049, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbb2f04b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4032f050, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a12f052, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd7f2f054, SUC_lexicon_tags_11 },
    { 0xf832f055, SUC_lexicon_tags_7 },
    { 0x7cd2f056, SUC_lexicon_tags_43 },
    { 0xa752f057, SUC_lexicon_tags_43 },
    { 0x472f054, SUC_lexicon_tags_17 },
    { 0xff72f059, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5592f05c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b2f061, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa172f066, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda92f06d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2812f06f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f2f073, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b92f075, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3412f07c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab2f085, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe692f087, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d2f090, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6232f092, SUC_lexicon_tags_182 },
    { 0xdf92f093, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1392f09b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf12f0a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd2f0a6, SUC_lexicon_tags_33 },
    { 0xf312f0a6, SUC_lexicon_tags_37 },
    { 0x2e52f0a7, SUC_lexicon_tags_29 },
    { 0xe752f0a8, SUC_lexicon_tags_43 },
    { 0xa1b2f0a9, SUC_lexicon_tags_17 },
    { 0xb6d2f0ab, SUC_lexicon_tags_45 },
    { 0x5bf2f0ab, SUC_lexicon_tags_11 },
    { 0x6e12f0ad, SUC_lexicon_tags_25 },
    { 0xde72f0a8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b52f0b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd52f0b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c92f0b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df2f0bf, SUC_lexicon_tags_17 },
    { 0x1692f0c0, SUC_lexicon_tags_19 },
    { 0x8dd2f0c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6212f0c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb412f0c7, SUC_lexicon_tags_12 },
    { 0xea12f0c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b2f0d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1b2f0d3, SUC_lexicon_tags_11 },
    { 0x64b2f0d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd2f0d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b2f0dc, SUC_lexicon_tags_77 },
    { 0xa5b2f0dc, SUC_lexicon_tags_36 },
    { 0x4412f0de, SUC_lexicon_tags_60 },
    { 0x67d2f0df, SUC_lexicon_tags_7 },
    { 0xd8d2f0e0, SUC_lexicon_tags_11 },
    { 0x5c52f0e1, SUC_lexicon_tags_12 },
    { 0xa692f0df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5592f0e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5892f0e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d2f0ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b32f0ee, SUC_lexicon_tags_37 },
    { 0x88b2f0ef, SUC_lexicon_tags_2 },
    { 0x18f2f0f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f2f0f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8572f0f9, SUC_lexicon_tags_11 },
    { 0x78f2f0fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb052f102, SUC_lexicon_tags_37 },
    { 0xd012f103, SUC_lexicon_tags_3 },
    { 0xf772f103, SUC_lexicon_tags_37 },
    { 0xe5f2f105, SUC_lexicon_tags_30 },
    { 0x1512f106, SUC_lexicon_tags_7 },
    { 0x75d2f107, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb172f10a, SUC_lexicon_tags_27 },
    { 0xedd2f10b, SUC_lexicon_tags_38 },
    { 0x8312f10c, SUC_lexicon_tags_55 },
    { 0x32b2f10d, SUC_lexicon_tags_26 },
    { 0x7e52f10d, SUC_lexicon_tags_30 },
    { 0x75b2f10c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc012f112, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82d2f114, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26f2f116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6332f11a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd2f11e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8372f121, SUC_lexicon_tags_2 },
    { 0x7872f122, SUC_lexicon_tags_1 },
    { 0xd752f123, SUC_lexicon_tags_1 },
    { 0x1d32f123, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f52f12d, SUC_lexicon_tags_12 },
    { 0x5492f12d, SUC_lexicon_tags_17 },
    { 0xed72f12f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f2f138, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x712f13b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4512f13d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc692f143, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d72f145, SUC_lexicon_tags_57 },
    { 0x9572f146, SUC_lexicon_tags_11 },
    { 0x1532f147, SUC_lexicon_tags_25 },
    { 0x4352f148, SUC_lexicon_tags_531 },
    { 0xe872f149, SUC_lexicon_tags_11 },
    { 0x4a52f146, SUC_lexicon_tags_8 },
    { 0xffd2f14b, SUC_lexicon_tags_7 },
    { 0x5612f14c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b2f14f, SUC_lexicon_tags_37 },
    { 0xd492f150, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf52f157, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5752f159, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b52f15f, SUC_lexicon_tags_22 },
    { 0x7c92f160, SUC_lexicon_tags_138 },
    { 0x7432f15f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2f16a, SUC_lexicon_tags_14 },
    { 0xd8f2f16b, SUC_lexicon_tags_18 },
    { 0xb2b2f16b, SUC_lexicon_tags_9 },
    { 0x4852f16a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb72f171, SUC_lexicon_tags_54 },
    { 0xd0f2f172, SUC_lexicon_tags_120 },
    { 0xd2f2f173, SUC_lexicon_tags_45 },
    { 0xc352f171, SUC_lexicon_tags_12 },
    { 0xd3d2f175, SUC_lexicon_tags_91 },
    { 0x1af2f176, SUC_lexicon_tags_112 },
    { 0xb52f173, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c12f179, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf132f180, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x972f184, SUC_lexicon_tags_11 },
    { 0xfad2f184, SUC_lexicon_tags_12 },
    { 0x9ad2f184, SUC_lexicon_tags_17 },
    { 0xd5b2f187, SUC_lexicon_tags_15 },
    { 0x3e52f188, SUC_lexicon_tags_24 },
    { 0xded2f186, SUC_lexicon_tags_26 },
    { 0xe132f18a, SUC_lexicon_tags_1 },
    { 0x30b2f188, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5772f18d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc972f192, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3112f196, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5252f198, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49b2f19a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b2f19f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf2f1a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf972f1a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2572f1af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b72f1b4, SUC_lexicon_tags_11 },
    { 0xa012f1b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f72f1be, SUC_lexicon_tags_11 },
    { 0xed92f1bf, SUC_lexicon_tags_30 },
    { 0x4392f1c0, SUC_lexicon_tags_17 },
    { 0xa4f2f1c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f92f1c6, SUC_lexicon_tags_13 },
    { 0x9fb2f1c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f2f1ca, SUC_lexicon_tags_2 },
    { 0xee52f1cb, SUC_lexicon_tags_11 },
    { 0x6032f1cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef32f1cf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2492f1d4, SUC_lexicon_tags_6 },
    { 0x9712f1d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fb2f1d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b2f1da, SUC_lexicon_tags_212 },
    { 0x2272f1da, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe192f1e1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a12f1e4, SUC_lexicon_tags_45 },
    { 0x6432f1e5, SUC_lexicon_tags_65 },
    { 0xb072f1e6, SUC_lexicon_tags_12 },
    { 0xcbb2f1e4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x93f2f1e9, SUC_lexicon_tags_7 },
    { 0x9e72f1e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdf2f1ec, SUC_lexicon_tags_16 },
    { 0xe7b2f1ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf352f1f2, SUC_lexicon_tags_19 },
    { 0xc392f1f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d2f1f7, SUC_lexicon_tags_63 },
    { 0x8252f1f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f92f1fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8552f201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d2f20b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb2f20e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab2f211, SUC_lexicon_tags_29 },
    { 0x3ef2f212, SUC_lexicon_tags_14 },
    { 0x4f12f213, SUC_lexicon_tags_15 },
    { 0xcf92f214, SUC_lexicon_tags_12 },
    { 0x1b52f214, SUC_lexicon_tags_15 },
    { 0x17b2f216, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c32f219, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d2f21d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c72f222, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa52f224, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf452f227, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f22c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc252f22e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d2f231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5312f235, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88d2f237, SUC_lexicon_tags_33 },
    { 0x4b52f238, SUC_lexicon_tags_23 },
    { 0x9192f238, SUC_lexicon_tags_7 },
    { 0x1832f238, SUC_lexicon_tags_17 },
    { 0x1fd2f238, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e72f243, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f72f24a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf552f24c, SUC_lexicon_tags_37 },
    { 0x3e52f24c, SUC_lexicon_tags_47 },
    { 0xcb72f24e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf72f250, SUC_lexicon_tags_45 },
    { 0xd672f251, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d2f258, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a52f25a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f2f25f, SUC_lexicon_tags_12 },
    { 0x8c52f260, SUC_lexicon_tags_6 },
    { 0x2972f260, SUC_lexicon_tags_30 },
    { 0x3c32f25f, SUC_lexicon_tags_2 },
    { 0x7c12f25f, SUC_lexicon_tags_29 },
    { 0x3112f264, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2f92f266, SUC_lexicon_tags_47 },
    { 0xb52f266, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd2f26b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f2f26e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab92f270, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec72f276, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6932f27c, SUC_lexicon_tags_6 },
    { 0x79f2f27c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d92f283, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d32f28a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf372f295, SUC_lexicon_tags_37 },
    { 0x1892f296, SUC_lexicon_tags_37 },
    { 0xba52f297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cd2f299, SUC_lexicon_tags_129 },
    { 0x37d2f29a, SUC_lexicon_tags_43 },
    { 0x6ff2f29b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d2f2a1, SUC_lexicon_tags_25 },
    { 0xf8f2f2a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbdf2f2a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd52f2a6, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f2f2b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5532f2b4, SUC_lexicon_tags_1081 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd512f2bb, SUC_lexicon_tags_40 },
    { 0x1f72f2bb, SUC_lexicon_tags_46 },
    { 0x7052f2bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb2f2c6, SUC_lexicon_tags_7 },
    { 0xa4b2f2c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d2f2ca, SUC_lexicon_tags_15 },
    { 0xe132f2cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29d2f2cd, SUC_lexicon_tags_15 },
    { 0xe972f2cd, SUC_lexicon_tags_48 },
    { 0x8772f2cf, SUC_lexicon_tags_55 },
    { 0x60d2f2cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e32f2d3, SUC_lexicon_tags_43 },
    { 0xcef2f2d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1872f2d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d2f2d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x612f2dd, SUC_lexicon_tags_107 },
    { 0x75d2f2dd, SUC_lexicon_tags_23 },
    { 0xec52f2df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6592f2e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5572f2e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4472f2e9, SUC_lexicon_tags_30 },
    { 0xf852f2e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4112f2ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df2f2ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf2f2f3, SUC_lexicon_tags_49 },
    { 0xb1b2f2f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6c92f2f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d2f2fa, SUC_lexicon_tags_27 },
    { 0xb452f2fb, SUC_lexicon_tags_6 },
    { 0xb2b2f2fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x72b2f2fe, SUC_lexicon_tags_4 },
    { 0x78b2f2ff, SUC_lexicon_tags_14 },
    { 0x78f2f2fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc012f308, SUC_lexicon_tags_2 },
    { 0x52b2f309, SUC_lexicon_tags_2 },
    { 0x4952f30a, SUC_lexicon_tags_84 },
    { 0x9432f309, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2932f30d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed2f310, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb792f314, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd712f318, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf972f31c, SUC_lexicon_tags_28 },
    { 0x8d52f31d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab32f324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e52f327, SUC_lexicon_tags_47 },
    { 0x7992f328, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb2f32d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd2f333, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98b2f335, SUC_lexicon_tags_25 },
    { 0xeb32f336, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc332f339, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd2f342, SUC_lexicon_tags_6 },
    { 0x2a52f342, SUC_lexicon_tags_15 },
    { 0xfe52f344, SUC_lexicon_tags_11 },
    { 0x272f345, SUC_lexicon_tags_8 },
    { 0x9f72f343, SUC_lexicon_tags_25 },
    { 0x24b2f347, SUC_lexicon_tags_23 },
    { 0x4392f347, SUC_lexicon_tags_32 },
    { 0xb112f349, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef2f34d, SUC_lexicon_tags_11 },
    { 0xe7d2f34e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa072f352, SUC_lexicon_tags_17 },
    { 0xc4d2f352, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb3d2f355, SUC_lexicon_tags_35 },
    { 0xa472f356, SUC_lexicon_tags_9 },
    { 0xd592f357, SUC_lexicon_tags_12 },
    { 0xb532f358, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f12f35b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a32f35d, SUC_lexicon_tags_91 },
    { 0x8a12f35d, SUC_lexicon_tags_25 },
    { 0x12f2f35f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e72f363, SUC_lexicon_tags_27 },
    { 0x6c12f364, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc752f368, SUC_lexicon_tags_77 },
    { 0x29d2f369, SUC_lexicon_tags_45 },
    { 0xaab2f369, SUC_lexicon_tags_11 },
    { 0x5432f368, SUC_lexicon_tags_17 },
    { 0xce52f369, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefd2f36e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9052f371, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd2f376, SUC_lexicon_tags_11 },
    { 0xee32f377, SUC_lexicon_tags_17 },
    { 0x6c32f378, SUC_lexicon_tags_27 },
    { 0x9e12f378, SUC_lexicon_tags_3 },
    { 0xa372f37a, SUC_lexicon_tags_47 },
    { 0x5e92f37b, SUC_lexicon_tags_17 },
    { 0xdcb2f379, SUC_lexicon_tags_65 },
    { 0x9052f37b, SUC_lexicon_tags_1 },
    { 0xd372f37e, SUC_lexicon_tags_15 },
    { 0xa532f37f, SUC_lexicon_tags_15 },
    { 0xb492f37c, SUC_lexicon_tags_4 },
    { 0xb732f381, SUC_lexicon_tags_2 },
    { 0xe952f382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe132f38a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf2f38d, SUC_lexicon_tags_61 },
    { 0x8132f38e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa692f391, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x32f2f393, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9312f397, SUC_lexicon_tags_13 },
    { 0x6fd2f398, SUC_lexicon_tags_11 },
    { 0x9332f399, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5072f39c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb0b2f39e, SUC_lexicon_tags_11 },
    { 0xcd32f39e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc252f3a1, SUC_lexicon_tags_12 },
    { 0xfeb2f3a2, SUC_lexicon_tags_14 },
    { 0xfbb2f3a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d32f3a6, SUC_lexicon_tags_15 },
    { 0x8dd2f3a7, SUC_lexicon_tags_38 },
    { 0x5e52f3a7, SUC_lexicon_tags_12 },
    { 0xc2f2f3a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x39b2f3ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b2f3b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d2f3b3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b32f3b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd2f3be, SUC_lexicon_tags_38 },
    { 0xe3f2f3bf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f2f3c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1bf2f3c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f2f3ca, SUC_lexicon_tags_56 },
    { 0x4252f3ca, SUC_lexicon_tags_10 },
    { 0xc612f3cc, SUC_lexicon_tags_1 },
    { 0x852f3cc, SUC_lexicon_tags_1 },
    { 0x6ab2f3ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc2f2f3d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b2f3d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb92f3d9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf6d2f3db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b2f3e0, SUC_lexicon_tags_0 },
    { 0x7f2f3e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fb2f3e3, SUC_lexicon_tags_1 },
    { 0xfa72f3e4, SUC_lexicon_tags_3 },
    { 0x9612f3e5, SUC_lexicon_tags_7 },
    { 0x36d2f3e6, SUC_lexicon_tags_10 },
    { 0x66d2f3e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f2f3ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc52f3ee, SUC_lexicon_tags_12 },
    { 0xbf2f3ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c52f3f3, SUC_lexicon_tags_8 },
    { 0xbd2f3f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb472f3f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff2f407, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x67d2f409, SUC_lexicon_tags_7 },
    { 0xc712f40a, SUC_lexicon_tags_2 },
    { 0x8912f40b, SUC_lexicon_tags_9 },
    { 0x9112f40b, SUC_lexicon_tags_9 },
    { 0x352f40a, SUC_lexicon_tags_79 },
    { 0x92b2f40a, SUC_lexicon_tags_27 },
    { 0x3792f40e, SUC_lexicon_tags_15 },
    { 0xd7b2f410, SUC_lexicon_tags_27 },
    { 0xd752f410, SUC_lexicon_tags_57 },
    { 0x6332f410, SUC_lexicon_tags_92 },
    { 0xa832f411, SUC_lexicon_tags_17 },
    { 0x32d2f40b, SUC_lexicon_tags_0 },
    { 0xc132f415, SUC_lexicon_tags_63 },
    { 0x3932f416, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x712f41a, SUC_lexicon_tags_1 },
    { 0xa32f41b, SUC_lexicon_tags_47 },
    { 0x41d2f41c, SUC_lexicon_tags_11 },
    { 0x75f2f41d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca32f41f, SUC_lexicon_tags_25 },
    { 0x76d2f420, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b2f424, SUC_lexicon_tags_7 },
    { 0xb8b2f424, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b72f42c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc692f42e, SUC_lexicon_tags_0 },
    { 0x4d72f42f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d2f434, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e12f437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda72f43a, SUC_lexicon_tags_17 },
    { 0x6852f43b, SUC_lexicon_tags_7 },
    { 0xa92f43b, SUC_lexicon_tags_30 },
    { 0xa952f43d, SUC_lexicon_tags_57 },
    { 0x1432f43e, SUC_lexicon_tags_17 },
    { 0x95d2f43a, SUC_lexicon_tags_47 },
    { 0xb9f2f440, SUC_lexicon_tags_32 },
    { 0x1212f43b, SUC_lexicon_tags_38 },
    { 0x91f2f442, SUC_lexicon_tags_8 },
    { 0x1072f443, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b72f447, SUC_lexicon_tags_14 },
    { 0x3972f448, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc92f44b, SUC_lexicon_tags_11 },
    { 0xa4b2f44c, SUC_lexicon_tags_15 },
    { 0x90b2f44c, SUC_lexicon_tags_25 },
    { 0xdfb2f44e, SUC_lexicon_tags_37 },
    { 0x4612f44f, SUC_lexicon_tags_11 },
    { 0x35f2f450, SUC_lexicon_tags_11 },
    { 0x7092f451, SUC_lexicon_tags_8 },
    { 0xba92f452, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1532f45a, SUC_lexicon_tags_4 },
    { 0xe432f45a, SUC_lexicon_tags_37 },
    { 0xaa92f45c, SUC_lexicon_tags_1 },
    { 0x9c92f45b, SUC_lexicon_tags_11 },
    { 0xfeb2f45e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b12f469, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x7e52f46b, SUC_lexicon_tags_14 },
    { 0x28f2f46c, SUC_lexicon_tags_60 },
    { 0xbad2f46d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9292f471, SUC_lexicon_tags_30 },
    { 0xd5d2f471, SUC_lexicon_tags_2 },
    { 0xe412f473, SUC_lexicon_tags_82 },
    { 0x3bf2f471, SUC_lexicon_tags_23 },
    { 0x1d92f471, SUC_lexicon_tags_37 },
    { 0xebb2f474, SUC_lexicon_tags_2 },
    { 0x5a92f476, SUC_lexicon_tags_25 },
    { 0x3b12f478, SUC_lexicon_tags_47 },
    { 0x6a92f478, SUC_lexicon_tags_7 },
    { 0xe2f2f47a, SUC_lexicon_tags_25 },
    { 0x63b2f47a, SUC_lexicon_tags_12 },
    { 0xe112f478, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe472f47f, SUC_lexicon_tags_43 },
    { 0xdc12f47f, SUC_lexicon_tags_25 },
    { 0xe632f481, SUC_lexicon_tags_7 },
    { 0x5112f482, SUC_lexicon_tags_11 },
    { 0x44d2f483, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe72f48d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8472f490, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff2f493, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d92f499, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed2f4a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x492f4a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x172f4a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb2f4a9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x4872f4ac, SUC_lexicon_tags_12 },
    { 0x1652f4ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4492f4b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3972f4b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc3d2f4b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec92f4ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf752f4bd, SUC_lexicon_tags_0 },
    { 0x2772f4bd, SUC_lexicon_tags_2 },
    { 0xc152f4be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3df2f4c1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b2f4c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc512f4c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec72f4c8, SUC_lexicon_tags_7 },
    { 0x46d2f4c9, SUC_lexicon_tags_65 },
    { 0xa5f2f4c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1fb2f4cc, SUC_lexicon_tags_17 },
    { 0x3592f4cd, SUC_lexicon_tags_7 },
    { 0xe7f2f4ce, SUC_lexicon_tags_29 },
    { 0x7712f4cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda72f4d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e92f4df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x912f4e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa9d2f4e3, SUC_lexicon_tags_37 },
    { 0xc252f4e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f92f4eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd032f4ee, SUC_lexicon_tags_43 },
    { 0x5232f4ef, SUC_lexicon_tags_14 },
    { 0x5092f4f0, SUC_lexicon_tags_30 },
    { 0xa352f4f0, SUC_lexicon_tags_19 },
    { 0xb8f2f4f2, SUC_lexicon_tags_6 },
    { 0xf2b2f4f0, SUC_lexicon_tags_38 },
    { 0x3af2f4f3, SUC_lexicon_tags_49 },
    { 0x10d2f4f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c52f4f9, SUC_lexicon_tags_0 },
    { 0x2c72f4f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd72f4fc, SUC_lexicon_tags_8 },
    { 0xd6d2f4fd, SUC_lexicon_tags_0 },
    { 0x5712f4fe, SUC_lexicon_tags_25 },
    { 0x3132f4ff, SUC_lexicon_tags_19 },
    { 0xda32f500, SUC_lexicon_tags_0 },
    { 0xcc92f501, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d72f504, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4732f509, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd2f50d, SUC_lexicon_tags_63 },
    { 0xda92f50d, SUC_lexicon_tags_2 },
    { 0x3332f50f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1712f511, SUC_lexicon_tags_8 },
    { 0x2532f511, SUC_lexicon_tags_9 },
    { 0x9112f513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc692f515, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf592f519, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6392f51d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe472f51f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8812f521, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd432f525, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe8d2f527, SUC_lexicon_tags_55 },
    { 0x1d92f528, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3592f52a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d2f535, SUC_lexicon_tags_21 },
    { 0xafb2f536, SUC_lexicon_tags_16 },
    { 0xbcf2f537, SUC_lexicon_tags_12 },
    { 0x4952f535, SUC_lexicon_tags_214 },
    { 0x4e32f536, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf72f53e, SUC_lexicon_tags_37 },
    { 0x1412f53e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc592f542, SUC_lexicon_tags_31 },
    { 0xd1f2f543, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb532f54e, SUC_lexicon_tags_45 },
    { 0x3e32f54e, SUC_lexicon_tags_27 },
    { 0x9cb2f550, SUC_lexicon_tags_48 },
    { 0xc7b2f550, SUC_lexicon_tags_28 },
    { 0x1792f550, SUC_lexicon_tags_26 },
    { 0x2e72f553, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf612f558, SUC_lexicon_tags_2 },
    { 0x3a52f558, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff2f55c, SUC_lexicon_tags_1 },
    { 0x2f52f55c, SUC_lexicon_tags_43 },
    { 0x75d2f55e, SUC_lexicon_tags_17 },
    { 0x8912f55d, SUC_lexicon_tags_11 },
    { 0x55d2f560, SUC_lexicon_tags_45 },
    { 0xa3f2f560, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf392f563, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc92f565, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d2f571, SUC_lexicon_tags_11 },
    { 0x9b72f571, SUC_lexicon_tags_38 },
    { 0x47f2f573, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7912f57b, SUC_lexicon_tags_25 },
    { 0x7712f57c, SUC_lexicon_tags_15 },
    { 0xfdd2f57c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x96f2f57f, SUC_lexicon_tags_1 },
    { 0x1b72f580, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd752f583, SUC_lexicon_tags_28 },
    { 0x20b2f584, SUC_lexicon_tags_0 },
    { 0xbef2f584, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfb2f587, SUC_lexicon_tags_27 },
    { 0x9ff2f588, SUC_lexicon_tags_2 },
    { 0x6772f588, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf612f58d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2412f590, SUC_lexicon_tags_11 },
    { 0xffb2f591, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df2f594, SUC_lexicon_tags_7 },
    { 0x26f2f594, SUC_lexicon_tags_37 },
    { 0x6412f594, SUC_lexicon_tags_1 },
    { 0xe732f596, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3652f599, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1f32f59b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x77b2f59d, SUC_lexicon_tags_4 },
    { 0x2192f59d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e12f5a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9532f5a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb2f5ab, SUC_lexicon_tags_14 },
    { 0x99b2f5ac, SUC_lexicon_tags_25 },
    { 0x43d2f5ac, SUC_lexicon_tags_37 },
    { 0x4692f5ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f2f5b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2532f5b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8412f5b9, SUC_lexicon_tags_47 },
    { 0x8ab2f5b9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xe932f5bd, SUC_lexicon_tags_17 },
    { 0x7c12f5bd, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a12f5c1, SUC_lexicon_tags_30 },
    { 0xcfd2f5c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b2f5c5, SUC_lexicon_tags_11 },
    { 0x7b92f5c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f2f5ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3092f5cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5792f5d1, SUC_lexicon_tags_2 },
    { 0xeb52f5d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf2f5d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f2f5d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3112f5df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef2f5e3, SUC_lexicon_tags_6 },
    { 0x6e12f5e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe8b2f5e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4f2f5e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d2f5f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d2f5f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf892f5f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb32f5fc, SUC_lexicon_tags_61 },
    { 0xd032f5fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1832f600, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9b2f602, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d12f606, SUC_lexicon_tags_282 },
    { 0x2a72f606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed12f609, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9512f60d, SUC_lexicon_tags_1 },
    { 0x8e72f60e, SUC_lexicon_tags_39 },
    { 0x5cb2f60e, SUC_lexicon_tags_6 },
    { 0xe872f60d, SUC_lexicon_tags_63 },
    { 0xdef2f610, SUC_lexicon_tags_28 },
    { 0x57b2f611, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf2f615, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd52f618, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x952f61b, SUC_lexicon_tags_25 },
    { 0x8f2f61c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf2f627, SUC_lexicon_tags_27 },
    { 0x9872f628, SUC_lexicon_tags_13 },
    { 0xc952f629, SUC_lexicon_tags_37 },
    { 0x6d92f627, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d2f62d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b2f633, SUC_lexicon_tags_25 },
    { 0x4b72f634, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba32f638, SUC_lexicon_tags_7 },
    { 0xe452f639, SUC_lexicon_tags_19 },
    { 0x5872f63a, SUC_lexicon_tags_23 },
    { 0x55f2f63a, SUC_lexicon_tags_60 },
    { 0x292f63c, SUC_lexicon_tags_29 },
    { 0x32f2f63c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7892f645, SUC_lexicon_tags_0 },
    { 0x5532f646, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b2f64c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8df2f64e, SUC_lexicon_tags_8 },
    { 0x7ab2f64e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xe212f651, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb72f656, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6af2f658, SUC_lexicon_tags_34 },
    { 0xaff2f659, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd7d2f65b, SUC_lexicon_tags_11 },
    { 0x7432f65c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc32f661, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf2f669, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe812f66b, SUC_lexicon_tags_31 },
    { 0xb612f66c, SUC_lexicon_tags_15 },
    { 0xa132f66b, SUC_lexicon_tags_35 },
    { 0x8b2f66d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x432f670, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc952f672, SUC_lexicon_tags_14 },
    { 0x4692f673, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x72f675, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed2f67a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d2f67f, SUC_lexicon_tags_11 },
    { 0x1012f680, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb232f686, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d92f689, SUC_lexicon_tags_15 },
    { 0x43d2f689, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe32f68c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df2f690, SUC_lexicon_tags_15 },
    { 0xa212f691, SUC_lexicon_tags_45 },
    { 0x5272f692, SUC_lexicon_tags_33 },
    { 0xcbb2f693, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf912f69b, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab52f6a0, SUC_lexicon_tags_38 },
    { 0x7432f6a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb2f6a4, SUC_lexicon_tags_84 },
    { 0x6a92f6a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa112f6a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d2f6ad, SUC_lexicon_tags_7 },
    { 0x64d2f6ad, SUC_lexicon_tags_38 },
    { 0x2e72f6ad, SUC_lexicon_tags_125 },
    { 0x6ff2f6ae, SUC_lexicon_tags_237 },
    { 0x5c52f6ad, SUC_lexicon_tags_7 },
    { 0xa272f6b1, SUC_lexicon_tags_2 },
    { 0xe092f6b3, SUC_lexicon_tags_7 },
    { 0x4a52f6b4, SUC_lexicon_tags_57 },
    { 0xe6f2f6b4, SUC_lexicon_tags_29 },
    { 0x652f6b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb72f6b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc712f6bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff52f6bf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3d52f6c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd932f6c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdb2f6c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd212f6cc, SUC_lexicon_tags_4 },
    { 0xbfd2f6cd, SUC_lexicon_tags_14 },
    { 0xeab2f6ce, SUC_lexicon_tags_12 },
    { 0xef72f6cf, SUC_lexicon_tags_15 },
    { 0x3cf2f6d0, SUC_lexicon_tags_2 },
    { 0x6152f6d1, SUC_lexicon_tags_11 },
    { 0xd592f6ce, SUC_lexicon_tags_36 },
    { 0xcef2f6d3, SUC_lexicon_tags_38 },
    { 0x1af2f6d4, SUC_lexicon_tags_15 },
    { 0xcd2f6ce, SUC_lexicon_tags_11 },
    { 0x6f32f6d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6672f6da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad72f6e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1592f6e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b2f6e8, SUC_lexicon_tags_53 },
    { 0x11f2f6e9, SUC_lexicon_tags_33 },
    { 0x55d2f6ea, SUC_lexicon_tags_23 },
    { 0x5c52f6eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e52f6ef, SUC_lexicon_tags_45 },
    { 0x6a72f6ef, SUC_lexicon_tags_66 },
    { 0x8b2f6ef, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8572f6f4, SUC_lexicon_tags_53 },
    { 0x5e92f6f4, SUC_lexicon_tags_19 },
    { 0xad32f6f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8632f6fb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe012f6fd, SUC_lexicon_tags_29 },
    { 0x37d2f6fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b12f702, SUC_lexicon_tags_1 },
    { 0x6d32f702, SUC_lexicon_tags_38 },
    { 0xef72f703, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d52f706, SUC_lexicon_tags_28 },
    { 0x7d92f707, SUC_lexicon_tags_53 },
    { 0x38b2f708, SUC_lexicon_tags_2 },
    { 0xf92f709, SUC_lexicon_tags_23 },
    { 0x9cb2f70a, SUC_lexicon_tags_15 },
    { 0x4f72f70a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d2f712, SUC_lexicon_tags_43 },
    { 0x6d32f712, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa732f715, SUC_lexicon_tags_2 },
    { 0xac32f716, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0f2f718, SUC_lexicon_tags_37 },
    { 0x6292f719, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2d2f71b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfb2f71d, SUC_lexicon_tags_47 },
    { 0x3c12f71d, SUC_lexicon_tags_11 },
    { 0x5292f71f, SUC_lexicon_tags_17 },
    { 0x23d2f71d, SUC_lexicon_tags_19 },
    { 0x21f2f720, SUC_lexicon_tags_7 },
    { 0xee92f722, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf72f72c, SUC_lexicon_tags_29 },
    { 0xea32f72c, SUC_lexicon_tags_4 },
    { 0x1d32f72d, SUC_lexicon_tags_29 },
    { 0xff52f72f, SUC_lexicon_tags_12 },
    { 0xa3b2f730, SUC_lexicon_tags_26 },
    { 0xf392f731, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x50f2f733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6072f736, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf32f73e, SUC_lexicon_tags_29 },
    { 0xd972f73e, SUC_lexicon_tags_15 },
    { 0x20d2f740, SUC_lexicon_tags_37 },
    { 0xe9d2f740, SUC_lexicon_tags_37 },
    { 0x3912f741, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6372f744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd632f748, SUC_lexicon_tags_17 },
    { 0x6fd2f749, SUC_lexicon_tags_38 },
    { 0x8992f74a, SUC_lexicon_tags_47 },
    { 0x7bd2f748, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2172f74d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d2f752, SUC_lexicon_tags_10 },
    { 0x6ab2f753, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd72f762, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf052f765, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1d2f767, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df2f76b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x412f76e, SUC_lexicon_tags_17 },
    { 0xc1f2f76e, SUC_lexicon_tags_37 },
    { 0x8052f76e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a12f775, SUC_lexicon_tags_17 },
    { 0x4092f776, SUC_lexicon_tags_11 },
    { 0xe6b2f776, SUC_lexicon_tags_11 },
    { 0xb792f776, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x172f77e, SUC_lexicon_tags_17 },
    { 0xa932f77f, SUC_lexicon_tags_66 },
    { 0xa892f780, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd2f784, SUC_lexicon_tags_66 },
    { 0x7952f785, SUC_lexicon_tags_2 },
    { 0x3a92f786, SUC_lexicon_tags_9 },
    { 0x8472f787, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d72f789, SUC_lexicon_tags_39 },
    { 0x7492f78a, SUC_lexicon_tags_19 },
    { 0x9552f789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b52f78d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb72f791, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f2f79b, SUC_lexicon_tags_7 },
    { 0x45f2f79b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4272f79f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa772f7a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d92f7a9, SUC_lexicon_tags_17 },
    { 0x6b32f7aa, SUC_lexicon_tags_17 },
    { 0xfab2f7ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a32f7ae, SUC_lexicon_tags_17 },
    { 0x3012f7ae, SUC_lexicon_tags_10 },
    { 0xc452f7b0, SUC_lexicon_tags_11 },
    { 0x2b92f7ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf972f7b4, SUC_lexicon_tags_12 },
    { 0x10d2f7b5, SUC_lexicon_tags_84 },
    { 0xcfd2f7b6, SUC_lexicon_tags_33 },
    { 0xfe12f7b7, SUC_lexicon_tags_63 },
    { 0xbaf2f7b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf32f7bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x832f7c9, SUC_lexicon_tags_45 },
    { 0xa832f7ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2d2f7cc, SUC_lexicon_tags_2 },
    { 0x7412f7cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f2f7d1, SUC_lexicon_tags_21 },
    { 0x1e12f7d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3692f7d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5952f7d8, SUC_lexicon_tags_25 },
    { 0xdd2f7d9, SUC_lexicon_tags_2 },
    { 0x3572f7da, SUC_lexicon_tags_82 },
    { 0x2b32f7d8, SUC_lexicon_tags_45 },
    { 0x4372f7dc, SUC_lexicon_tags_1 },
    { 0x3e52f7dc, SUC_lexicon_tags_27 },
    { 0xbcd2f7db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf72f7e1, SUC_lexicon_tags_1 },
    { 0x8f72f7e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd632f7e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df2f7ea, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe92f7ec, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf672f7ef, SUC_lexicon_tags_65 },
    { 0xf1f2f7f0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc52f7f3, SUC_lexicon_tags_95 },
    { 0xfc12f7f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9732f7f6, SUC_lexicon_tags_7 },
    { 0x3e92f7f7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8932f7ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf772f803, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xffd2f805, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d2f808, SUC_lexicon_tags_2 },
    { 0x64b2f808, SUC_lexicon_tags_103 },
    { 0xe492f808, SUC_lexicon_tags_32 },
    { 0xd112f80b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9b32f80d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff32f811, SUC_lexicon_tags_30 },
    { 0x2952f811, SUC_lexicon_tags_34 },
    { 0xf9b2f813, SUC_lexicon_tags_39 },
    { 0xb4d2f813, SUC_lexicon_tags_2 },
    { 0x3552f815, SUC_lexicon_tags_87 },
    { 0x93d2f816, SUC_lexicon_tags_14 },
    { 0x7e92f817, SUC_lexicon_tags_27 },
    { 0xc7f2f816, SUC_lexicon_tags_6 },
    { 0xd1d2f818, SUC_lexicon_tags_15 },
    { 0x3b32f81a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad2f81d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6672f81f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f72f821, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4412f823, SUC_lexicon_tags_127 },
    { 0x55f2f823, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd0b2f826, SUC_lexicon_tags_15 },
    { 0xb032f827, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2d2f829, SUC_lexicon_tags_31 },
    { 0xb6f2f82a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b92f82e, SUC_lexicon_tags_37 },
    { 0x8f2f82f, SUC_lexicon_tags_7 },
    { 0xddd2f82f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92f2f832, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b32f836, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6372f838, SUC_lexicon_tags_15 },
    { 0x7c92f839, SUC_lexicon_tags_61 },
    { 0x81d2f839, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e32f83e, SUC_lexicon_tags_16 },
    { 0xb792f83e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xca92f842, SUC_lexicon_tags_10 },
    { 0xb12f843, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4b2f845, SUC_lexicon_tags_15 },
    { 0xe992f845, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf832f848, SUC_lexicon_tags_38 },
    { 0xfc72f849, SUC_lexicon_tags_7 },
    { 0xfe12f849, SUC_lexicon_tags_49 },
    { 0xb12f84b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1052f84e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd2f852, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x7e12f854, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd932f858, SUC_lexicon_tags_3 },
    { 0x6e32f858, SUC_lexicon_tags_49 },
    { 0xe112f859, SUC_lexicon_tags_6 },
    { 0xd612f85a, SUC_lexicon_tags_30 },
    { 0x6812f858, SUC_lexicon_tags_3 },
    { 0x4092f859, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b12f860, SUC_lexicon_tags_33 },
    { 0xe7b2f861, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4592f863, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8652f86e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4472f870, SUC_lexicon_tags_2 },
    { 0x6f92f871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b2f875, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ff2f877, SUC_lexicon_tags_27 },
    { 0x7432f877, SUC_lexicon_tags_2 },
    { 0x9ef2f879, SUC_lexicon_tags_38 },
    { 0x6fb2f879, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf692f887, SUC_lexicon_tags_27 },
    { 0x4b32f888, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xffd2f88a, SUC_lexicon_tags_8 },
    { 0x8db2f88b, SUC_lexicon_tags_7 },
    { 0x18f2f88c, SUC_lexicon_tags_25 },
    { 0x2e32f88d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9b32f88f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19f2f891, SUC_lexicon_tags_23 },
    { 0x42b2f892, SUC_lexicon_tags_26 },
    { 0xa4f2f893, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f2f89a, SUC_lexicon_tags_46 },
    { 0xf2b2f89b, SUC_lexicon_tags_29 },
    { 0xea52f89c, SUC_lexicon_tags_19 },
    { 0x6912f89c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2332f89f, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x372f8a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5272f8a5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xac52f8a7, SUC_lexicon_tags_2 },
    { 0x13f2f8a8, SUC_lexicon_tags_23 },
    { 0x57d2f8a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8072f8ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b72f8b1, SUC_lexicon_tags_37 },
    { 0x3a32f8b1, SUC_lexicon_tags_2 },
    { 0x67d2f8b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab2f8b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc252f8b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4732f8bb, SUC_lexicon_tags_47 },
    { 0xec12f8bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e52f8c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f2f8c3, SUC_lexicon_tags_12 },
    { 0x2632f8c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f2f8c8, SUC_lexicon_tags_63 },
    { 0x6b12f8c9, SUC_lexicon_tags_17 },
    { 0xedd2f8ca, SUC_lexicon_tags_11 },
    { 0x5732f8ca, SUC_lexicon_tags_26 },
    { 0xf112f8cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ef2f8ce, SUC_lexicon_tags_29 },
    { 0xd6b2f8cf, SUC_lexicon_tags_11 },
    { 0x3792f8d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc572f8d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a92f8d8, SUC_lexicon_tags_33 },
    { 0xd72f8d8, SUC_lexicon_tags_99 },
    { 0xebb2f8da, SUC_lexicon_tags_83 },
    { 0x9212f8db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2432f8dd, SUC_lexicon_tags_3 },
    { 0xf9d2f8de, SUC_lexicon_tags_37 },
    { 0x7dd2f8de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7bb2f8e1, SUC_lexicon_tags_14 },
    { 0x97f2f8e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6792f8e5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c12f8ec, SUC_lexicon_tags_11 },
    { 0x6932f8ed, SUC_lexicon_tags_11 },
    { 0x8f32f8ee, SUC_lexicon_tags_1 },
    { 0xfd12f8ed, SUC_lexicon_tags_28 },
    { 0xcf12f8f0, SUC_lexicon_tags_65 },
    { 0xaa92f8ed, SUC_lexicon_tags_25 },
    { 0xa9b2f8f1, SUC_lexicon_tags_45 },
    { 0x2a12f8f0, SUC_lexicon_tags_33 },
    { 0x9ad2f8f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eb2f8f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed72f8fe, SUC_lexicon_tags_37 },
    { 0xdcf2f8ff, SUC_lexicon_tags_103 },
    { 0x31d2f900, SUC_lexicon_tags_101 },
    { 0xa792f901, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d2f904, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d2f907, SUC_lexicon_tags_23 },
    { 0x9012f908, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d2f915, SUC_lexicon_tags_68 },
    { 0xa4d2f916, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd2f919, SUC_lexicon_tags_26 },
    { 0xc852f91a, SUC_lexicon_tags_14 },
    { 0x81b2f919, SUC_lexicon_tags_15 },
    { 0x37d2f91c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf2f922, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8612f924, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb2f927, SUC_lexicon_tags_1 },
    { 0xd9d2f927, SUC_lexicon_tags_25 },
    { 0x86b2f927, SUC_lexicon_tags_22 },
    { 0x9d52f927, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe152f92d, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xecd2f92f, SUC_lexicon_tags_4 },
    { 0x8e52f92f, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c92f933, SUC_lexicon_tags_17 },
    { 0xdd32f934, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6212f939, SUC_lexicon_tags_47 },
    { 0x832f93a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d2f93d, SUC_lexicon_tags_17 },
    { 0xa532f93e, SUC_lexicon_tags_28 },
    { 0x7392f93f, SUC_lexicon_tags_47 },
    { 0x71d2f940, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e12f943, SUC_lexicon_tags_25 },
    { 0x42d2f944, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdf12f946, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29d2f948, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbe32f94a, SUC_lexicon_tags_7 },
    { 0xc0b2f94b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9712f950, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa972f959, SUC_lexicon_tags_29 },
    { 0x2db2f95a, SUC_lexicon_tags_37 },
    { 0x94d2f95a, SUC_lexicon_tags_14 },
    { 0xf692f95c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2832f95f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb2f962, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f92f964, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e32f96c, SUC_lexicon_tags_25 },
    { 0x3692f96c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d2f974, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe452f97f, SUC_lexicon_tags_1 },
    { 0xffd2f97f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e92f986, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b92f989, SUC_lexicon_tags_82 },
    { 0x57d2f98a, SUC_lexicon_tags_15 },
    { 0x5712f98b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8072f992, SUC_lexicon_tags_60 },
    { 0x97b2f993, SUC_lexicon_tags_37 },
    { 0xd6b2f994, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb392f99b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f92f99d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe212f99f, SUC_lexicon_tags_12 },
    { 0x3c72f9a0, SUC_lexicon_tags_25 },
    { 0xacd2f99f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc72f9a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9352f9a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b12f9a8, SUC_lexicon_tags_37 },
    { 0xc9f2f9a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe212f9b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf2f9ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe12f9be, SUC_lexicon_tags_25 },
    { 0xf0d2f9bf, SUC_lexicon_tags_11 },
    { 0x6252f9bf, SUC_lexicon_tags_9 },
    { 0x152f9c0, SUC_lexicon_tags_27 },
    { 0x66f2f9c2, SUC_lexicon_tags_37 },
    { 0x7df2f9be, SUC_lexicon_tags_65 },
    { 0x62f2f9bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8ff2f9c6, SUC_lexicon_tags_14 },
    { 0xd632f9c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f2f9cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3852f9ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd432f9d0, SUC_lexicon_tags_601 },
    { 0xd672f9d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xca32f9d3, SUC_lexicon_tags_63 },
    { 0x1e52f9d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6472f9d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b2f9da, SUC_lexicon_tags_30 },
    { 0x4a92f9db, SUC_lexicon_tags_16 },
    { 0xa552f9db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec72f9de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfdf2f9e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6172f9e2, SUC_lexicon_tags_11 },
    { 0x1512f9e3, SUC_lexicon_tags_33 },
    { 0xecf2f9e2, SUC_lexicon_tags_37 },
    { 0xafd2f9e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6012f9e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b2f9ed, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff2f9f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b2f9f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48f2f9fa, SUC_lexicon_tags_1 },
    { 0xb032f9fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb52fa07, SUC_lexicon_tags_11 },
    { 0xbb52fa08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfef2fa0a, SUC_lexicon_tags_15 },
    { 0x1a72fa0b, SUC_lexicon_tags_17 },
    { 0x632fa0c, SUC_lexicon_tags_92 },
    { 0xa932fa0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f12fa10, SUC_lexicon_tags_30 },
    { 0xe4f2fa11, SUC_lexicon_tags_14 },
    { 0x112fa10, SUC_lexicon_tags_0 },
    { 0x5cf2fa12, SUC_lexicon_tags_45 },
    { 0xadf2fa14, SUC_lexicon_tags_43 },
    { 0xe752fa14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f72fa19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3992fa1b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2432fa1f, SUC_lexicon_tags_15 },
    { 0x2112fa1f, SUC_lexicon_tags_12 },
    { 0xb612fa20, SUC_lexicon_tags_7 },
    { 0xb1f2fa22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b2fa25, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf2fa2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f72fa30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7252fa32, SUC_lexicon_tags_25 },
    { 0x4312fa33, SUC_lexicon_tags_31 },
    { 0x7f72fa34, SUC_lexicon_tags_25 },
    { 0x9df2fa33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b2fa3f, SUC_lexicon_tags_63 },
    { 0xfc12fa3f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcf2fa42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1b2fa44, SUC_lexicon_tags_15 },
    { 0xf32fa44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8b2fa47, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f2fa4e, SUC_lexicon_tags_38 },
    { 0x7852fa4f, SUC_lexicon_tags_13 },
    { 0x6172fa4f, SUC_lexicon_tags_27 },
    { 0xa212fa51, SUC_lexicon_tags_27 },
    { 0xb72fa52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6212fa54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b2fa5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd152fa5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f2fa64, SUC_lexicon_tags_25 },
    { 0x67b2fa64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe752fa69, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf412fa6e, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d2fa76, SUC_lexicon_tags_12 },
    { 0x4112fa77, SUC_lexicon_tags_2 },
    { 0xe92fa76, SUC_lexicon_tags_48 },
    { 0x3f92fa79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff2fa7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b92fa81, SUC_lexicon_tags_9 },
    { 0xe172fa81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x432fa8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef2fa92, SUC_lexicon_tags_59 },
    { 0xe872fa93, SUC_lexicon_tags_3 },
    { 0xf092fa94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb72fa96, SUC_lexicon_tags_79 },
    { 0xb4b2fa97, SUC_lexicon_tags_13 },
    { 0xffd2fa96, SUC_lexicon_tags_9 },
    { 0x4912fa99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d52fa9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b32fa9f, SUC_lexicon_tags_16 },
    { 0x712fa9f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3cb2faa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe112faa4, SUC_lexicon_tags_29 },
    { 0x8832faa4, SUC_lexicon_tags_25 },
    { 0xac52faa6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b2faa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2d2faab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf852faae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf2fab5, SUC_lexicon_tags_38 },
    { 0x5532fab6, SUC_lexicon_tags_80 },
    { 0xdff2fab5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3952faba, SUC_lexicon_tags_27 },
    { 0x8a92fabb, SUC_lexicon_tags_60 },
    { 0xea12faba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebf2fabe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4852fac0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d2faca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4332fad1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf792fad3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c52fad5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a32fad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc192fadc, SUC_lexicon_tags_32 },
    { 0xf5d2fadc, SUC_lexicon_tags_11 },
    { 0x8c92fadd, SUC_lexicon_tags_87 },
    { 0x4552fadd, SUC_lexicon_tags_61 },
    { 0x9272fae0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2352fae4, SUC_lexicon_tags_14 },
    { 0x4052fae4, SUC_lexicon_tags_32 },
    { 0x1652fae6, SUC_lexicon_tags_15 },
    { 0x8af2fae6, SUC_lexicon_tags_15 },
    { 0x2392fae5, SUC_lexicon_tags_2 },
    { 0xb892fae7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6712faeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf252faed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc52faf1, SUC_lexicon_tags_25 },
    { 0x6612faf1, SUC_lexicon_tags_12 },
    { 0xf6f2faf3, SUC_lexicon_tags_47 },
    { 0x80f2faf3, SUC_lexicon_tags_60 },
    { 0x8c52faf2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x812faf7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f2fafa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xebb2fafc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4432fb00, SUC_lexicon_tags_37 },
    { 0x3b2fb01, SUC_lexicon_tags_7 },
    { 0x6732fb01, SUC_lexicon_tags_15 },
    { 0xa7f2fb03, SUC_lexicon_tags_79 },
    { 0x7212fb04, SUC_lexicon_tags_31 },
    { 0x6012fb05, SUC_lexicon_tags_45 },
    { 0x13b2fb05, SUC_lexicon_tags_1 },
    { 0x4eb2fb05, SUC_lexicon_tags_63 },
    { 0x12d2fb00, SUC_lexicon_tags_30 },
    { 0xe92fb01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb72fb0e, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b2fb11, SUC_lexicon_tags_2 },
    { 0xe792fb11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc072fb15, SUC_lexicon_tags_1 },
    { 0xc872fb15, SUC_lexicon_tags_1 },
    { 0xc852fb17, SUC_lexicon_tags_16 },
    { 0x6932fb17, SUC_lexicon_tags_1 },
    { 0xa492fb19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8552fb1d, SUC_lexicon_tags_57 },
    { 0x30f2fb1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa52fb26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e92fb2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbc32fb2c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb32fb36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3012fb3c, SUC_lexicon_tags_45 },
    { 0x8812fb3d, SUC_lexicon_tags_12 },
    { 0xb4d2fb3d, SUC_lexicon_tags_2 },
    { 0x8a12fb3f, SUC_lexicon_tags_11 },
    { 0x8f72fb40, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5192fb47, SUC_lexicon_tags_23 },
    { 0xaf72fb48, SUC_lexicon_tags_0 },
    { 0x24d2fb49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b2fb4d, SUC_lexicon_tags_37 },
    { 0x8472fb4e, SUC_lexicon_tags_1 },
    { 0x33d2fb4f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f2fb59, SUC_lexicon_tags_26 },
    { 0xa5d2fb59, SUC_lexicon_tags_1 },
    { 0xac32fb5b, SUC_lexicon_tags_23 },
    { 0xa2b2fb5c, SUC_lexicon_tags_30 },
    { 0x9352fb5d, SUC_lexicon_tags_11 },
    { 0x1592fb5c, SUC_lexicon_tags_8 },
    { 0x5cd2fb5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d2fb63, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b2fb66, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d2fb6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf072fb70, SUC_lexicon_tags_33 },
    { 0xc0b2fb70, SUC_lexicon_tags_3 },
    { 0x4352fb71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d72fb76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd2fb7d, SUC_lexicon_tags_1 },
    { 0xf112fb7e, SUC_lexicon_tags_25 },
    { 0x1b32fb7f, SUC_lexicon_tags_75 },
    { 0x9592fb7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe92fb84, SUC_lexicon_tags_17 },
    { 0x8fd2fb85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fb88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc732fb8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb112fb94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5d2fb96, SUC_lexicon_tags_25 },
    { 0x5252fb97, SUC_lexicon_tags_12 },
    { 0x7f32fb98, SUC_lexicon_tags_91 },
    { 0xb452fb96, SUC_lexicon_tags_15 },
    { 0x7912fb99, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2d52fb9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b12fb9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc192fba0, SUC_lexicon_tags_43 },
    { 0x5cd2fba1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a52fba6, SUC_lexicon_tags_33 },
    { 0x3592fba7, SUC_lexicon_tags_25 },
    { 0xa4b2fba6, SUC_lexicon_tags_47 },
    { 0x9f32fba6, SUC_lexicon_tags_18 },
    { 0xa852fbaa, SUC_lexicon_tags_18 },
    { 0xe852fbaa, SUC_lexicon_tags_74 },
    { 0x9812fbac, SUC_lexicon_tags_2 },
    { 0xe872fbad, SUC_lexicon_tags_30 },
    { 0x7432fbad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b2fbb1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f2fbbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5932fbbe, SUC_lexicon_tags_49 },
    { 0x712fbbf, SUC_lexicon_tags_63 },
    { 0x7a52fbbe, SUC_lexicon_tags_3 },
    { 0xf732fbbf, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b2fbc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb332fbca, SUC_lexicon_tags_11 },
    { 0xf492fbcb, SUC_lexicon_tags_19 },
    { 0x51f2fbcc, SUC_lexicon_tags_14 },
    { 0xdad2fbcc, SUC_lexicon_tags_17 },
    { 0xe3f2fbcc, SUC_lexicon_tags_12 },
    { 0xa12fbcf, SUC_lexicon_tags_79 },
    { 0xa2b2fbca, SUC_lexicon_tags_92 },
    { 0x88d2fbcb, SUC_lexicon_tags_15 },
    { 0x892fbd2, SUC_lexicon_tags_1 },
    { 0xde32fbd3, SUC_lexicon_tags_79 },
    { 0x9892fbd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d2fbd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b32fbdb, SUC_lexicon_tags_6 },
    { 0x6692fbdc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea32fbde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd412fbe0, SUC_lexicon_tags_7 },
    { 0x97f2fbe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c32fbe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb692fbea, SUC_lexicon_tags_4 },
    { 0xb0f2fbeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a12fbed, SUC_lexicon_tags_9 },
    { 0xd752fbee, SUC_lexicon_tags_15 },
    { 0x5912fbef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe832fbf4, SUC_lexicon_tags_11 },
    { 0xcef2fbf5, SUC_lexicon_tags_25 },
    { 0xdad2fbf5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6cb2fbf8, SUC_lexicon_tags_12 },
    { 0xa92fbf8, SUC_lexicon_tags_11 },
    { 0x6a12fbfa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f92fbfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c92fc06, SUC_lexicon_tags_6 },
    { 0xa352fc06, SUC_lexicon_tags_85 },
    { 0x1c52fc08, SUC_lexicon_tags_47 },
    { 0x9512fc08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2412fc0b, SUC_lexicon_tags_26 },
    { 0x1bd2fc0c, SUC_lexicon_tags_0 },
    { 0x3872fc0c, SUC_lexicon_tags_14 },
    { 0x6532fc0e, SUC_lexicon_tags_25 },
    { 0xf432fc0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c92fc11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8392fc14, SUC_lexicon_tags_49 },
    { 0x33d2fc15, SUC_lexicon_tags_10 },
    { 0x73f2fc15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f2fc1c, SUC_lexicon_tags_14 },
    { 0x9412fc1d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b2fc25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf12fc28, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d2fc2c, SUC_lexicon_tags_29 },
    { 0xd232fc2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb72fc32, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf472fc34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1bb2fc36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f2fc39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfc52fc3b, SUC_lexicon_tags_91 },
    { 0xdbd2fc3c, SUC_lexicon_tags_25 },
    { 0x2652fc3d, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x5692fc40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc372fc44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe312fc4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf2fc4f, SUC_lexicon_tags_37 },
    { 0xab72fc50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6532fc54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf72fc56, SUC_lexicon_tags_60 },
    { 0xd5b2fc57, SUC_lexicon_tags_12 },
    { 0xd052fc58, SUC_lexicon_tags_7 },
    { 0x1b12fc57, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d2fc5e, SUC_lexicon_tags_45 },
    { 0x3ff2fc5f, SUC_lexicon_tags_1 },
    { 0xde12fc5e, SUC_lexicon_tags_7 },
    { 0x272fc61, SUC_lexicon_tags_49 },
    { 0x5512fc62, SUC_lexicon_tags_11 },
    { 0x4e52fc62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3752fc65, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe592fc68, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd52fc6f, SUC_lexicon_tags_17 },
    { 0xda32fc70, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa8b2fc72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4632fc74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df2fc77, SUC_lexicon_tags_31 },
    { 0xae32fc78, SUC_lexicon_tags_49 },
    { 0x90f2fc79, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b2fc7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c12fc80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8312fc83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf412fc86, SUC_lexicon_tags_15 },
    { 0xa172fc86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3cd2fc89, SUC_lexicon_tags_25 },
    { 0x952fc89, SUC_lexicon_tags_0 },
    { 0x7a12fc8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb012fc93, SUC_lexicon_tags_38 },
    { 0x9b52fc94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3332fc97, SUC_lexicon_tags_15 },
    { 0x5632fc98, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff32fc9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb032fca1, SUC_lexicon_tags_17 },
    { 0x1d32fca2, SUC_lexicon_tags_14 },
    { 0x3f2fca1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8832fca9, SUC_lexicon_tags_11 },
    { 0xb2f2fcaa, SUC_lexicon_tags_2 },
    { 0x21f2fcab, SUC_lexicon_tags_30 },
    { 0xa972fcac, SUC_lexicon_tags_151 },
    { 0x4152fcad, SUC_lexicon_tags_7 },
    { 0xe612fca9, SUC_lexicon_tags_7 },
    { 0x7db2fcac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af2fcb3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc932fcb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b2fcbc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1192fcc3, SUC_lexicon_tags_9 },
    { 0x1b52fcc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f2fcc6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d2fccd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f12fcd3, SUC_lexicon_tags_47 },
    { 0x972fcd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc412fcdb, SUC_lexicon_tags_11 },
    { 0x8ef2fcdc, SUC_lexicon_tags_15 },
    { 0x7a32fcdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d32fce4, SUC_lexicon_tags_31 },
    { 0x3652fce5, SUC_lexicon_tags_17 },
    { 0xa7b2fce6, SUC_lexicon_tags_0 },
    { 0xd732fce5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa9d2fce9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d12fcf3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9412fcf5, SUC_lexicon_tags_0 },
    { 0xc232fcf5, SUC_lexicon_tags_46 },
    { 0x1892fcf7, SUC_lexicon_tags_6 },
    { 0x42f2fcf8, SUC_lexicon_tags_7 },
    { 0x8852fcf8, SUC_lexicon_tags_0 },
    { 0x7dd2fcfa, SUC_lexicon_tags_17 },
    { 0xcfd2fcfb, SUC_lexicon_tags_30 },
    { 0x9f72fcfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3b2fcfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b32fd00, SUC_lexicon_tags_2 },
    { 0xf792fd00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xecd2fd03, SUC_lexicon_tags_12 },
    { 0x9b32fd04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b2fd09, SUC_lexicon_tags_14 },
    { 0x2232fd0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x46d2fd0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad2fd13, SUC_lexicon_tags_39 },
    { 0xa812fd14, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7b12fd16, SUC_lexicon_tags_37 },
    { 0x5d2fd16, SUC_lexicon_tags_170 },
    { 0x88d2fd17, SUC_lexicon_tags_37 },
    { 0xddd2fd19, SUC_lexicon_tags_34 },
    { 0x2292fd1a, SUC_lexicon_tags_12 },
    { 0x1712fd1b, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d2fd1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f12fd21, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8352fd23, SUC_lexicon_tags_79 },
    { 0xa032fd23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa792fd28, SUC_lexicon_tags_30 },
    { 0xa092fd29, SUC_lexicon_tags_25 },
    { 0x4bb2fd2a, SUC_lexicon_tags_63 },
    { 0x72fd2b, SUC_lexicon_tags_26 },
    { 0xe772fd2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db2fd31, SUC_lexicon_tags_14 },
    { 0xd752fd32, SUC_lexicon_tags_30 },
    { 0x9d92fd31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf7d2fd35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed2fd39, SUC_lexicon_tags_2 },
    { 0xecf2fd39, SUC_lexicon_tags_7 },
    { 0x4292fd39, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa52fd3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f2fd42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44f2fd44, SUC_lexicon_tags_0 },
    { 0x46f2fd45, SUC_lexicon_tags_1 },
    { 0x32b2fd46, SUC_lexicon_tags_37 },
    { 0x512fd46, SUC_lexicon_tags_60 },
    { 0xc12fd48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3192fd4a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd852fd4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd12fd52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef12fd54, SUC_lexicon_tags_1 },
    { 0x2ef2fd54, SUC_lexicon_tags_26 },
    { 0x4b2fd54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d72fd59, SUC_lexicon_tags_8 },
    { 0x9a52fd5a, SUC_lexicon_tags_31 },
    { 0x7292fd59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b2fd5e, SUC_lexicon_tags_25 },
    { 0x2232fd5f, SUC_lexicon_tags_146 },
    { 0x8792fd60, SUC_lexicon_tags_45 },
    { 0x7e92fd5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe152fd65, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc12fd69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b92fd6c, SUC_lexicon_tags_25 },
    { 0xbf12fd6d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb72fd70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d2fd76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f12fd79, SUC_lexicon_tags_1 },
    { 0xea32fd79, SUC_lexicon_tags_19 },
    { 0x1752fd7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb12fd7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3492fd80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5432fd83, SUC_lexicon_tags_1 },
    { 0xbd12fd84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff2fd86, SUC_lexicon_tags_160 },
    { 0xd7b2fd87, SUC_lexicon_tags_25 },
    { 0x34b2fd88, SUC_lexicon_tags_37 },
    { 0x4392fd89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7532fd8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f2fd90, SUC_lexicon_tags_14 },
    { 0xa112fd90, SUC_lexicon_tags_27 },
    { 0x3f72fd90, SUC_lexicon_tags_37 },
    { 0xfd12fd92, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb12fd97, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf992fd99, SUC_lexicon_tags_25 },
    { 0xe512fd99, SUC_lexicon_tags_63 },
    { 0x4b32fd99, SUC_lexicon_tags_7 },
    { 0x3572fd9c, SUC_lexicon_tags_23 },
    { 0x4332fd9d, SUC_lexicon_tags_10 },
    { 0x6cd2fd9e, SUC_lexicon_tags_7 },
    { 0x2d72fd9f, SUC_lexicon_tags_28 },
    { 0x5eb2fd9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7852fda7, SUC_lexicon_tags_15 },
    { 0x1fb2fda8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf2fdae, SUC_lexicon_tags_14 },
    { 0x43f2fdaf, SUC_lexicon_tags_8 },
    { 0x9a52fdb0, SUC_lexicon_tags_17 },
    { 0x5cd2fdaf, SUC_lexicon_tags_7 },
    { 0xe4f2fdb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7352fdb8, SUC_lexicon_tags_7 },
    { 0x2132fdb8, SUC_lexicon_tags_25 },
    { 0xef12fdba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd972fdbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5452fdc1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6892fdc3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f2fdc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3452fdc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe272fdcc, SUC_lexicon_tags_15 },
    { 0xf972fdcd, SUC_lexicon_tags_7 },
    { 0x772fdce, SUC_lexicon_tags_25 },
    { 0x3e92fdcf, SUC_lexicon_tags_17 },
    { 0x1892fdcd, SUC_lexicon_tags_70 },
    { 0xab92fdcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b72fdd4, SUC_lexicon_tags_2 },
    { 0x1d2fdd5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7f52fdd7, SUC_lexicon_tags_19 },
    { 0x9f32fdd8, SUC_lexicon_tags_26 },
    { 0x3332fdd8, SUC_lexicon_tags_15 },
    { 0xed92fdda, SUC_lexicon_tags_31 },
    { 0x2552fddb, SUC_lexicon_tags_9 },
    { 0x4932fddb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d2fde0, SUC_lexicon_tags_0 },
    { 0x7352fde1, SUC_lexicon_tags_75 },
    { 0xc7f2fde1, SUC_lexicon_tags_5 },
    { 0x5712fde3, SUC_lexicon_tags_21 },
    { 0x14f2fde4, SUC_lexicon_tags_70 },
    { 0x46f2fde4, SUC_lexicon_tags_25 },
    { 0xd9b2fde3, SUC_lexicon_tags_37 },
    { 0xc792fde7, SUC_lexicon_tags_17 },
    { 0x8db2fde8, SUC_lexicon_tags_11 },
    { 0xfbb2fde0, SUC_lexicon_tags_9 },
    { 0xf672fde5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xef92fdec, SUC_lexicon_tags_25 },
    { 0x2a92fded, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb2fdf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b2fdf5, SUC_lexicon_tags_28 },
    { 0xb2fdf6, SUC_lexicon_tags_37 },
    { 0xd6d2fdf6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2052fdf9, SUC_lexicon_tags_12 },
    { 0xab32fdf9, SUC_lexicon_tags_197 },
    { 0x15f2fdfb, SUC_lexicon_tags_38 },
    { 0x4f52fdfb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf32fe02, SUC_lexicon_tags_11 },
    { 0x7c92fe02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b2fe07, SUC_lexicon_tags_43 },
    { 0xf1b2fe07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b32fe0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7672fe0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af2fe12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d2fe17, SUC_lexicon_tags_37 },
    { 0xf2b2fe18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e32fe1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5512fe1c, SUC_lexicon_tags_25 },
    { 0xf4b2fe1d, SUC_lexicon_tags_37 },
    { 0xa372fe1e, SUC_lexicon_tags_7 },
    { 0x5612fe1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b2fe22, SUC_lexicon_tags_14 },
    { 0xbc52fe23, SUC_lexicon_tags_63 },
    { 0xa252fe23, SUC_lexicon_tags_31 },
    { 0x13d2fe25, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e52fe28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x88b2fe2a, SUC_lexicon_tags_26 },
    { 0x43b2fe2b, SUC_lexicon_tags_1 },
    { 0xf012fe2a, SUC_lexicon_tags_0 },
    { 0xb7b2fe2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeb32fe2f, SUC_lexicon_tags_2 },
    { 0xe9b2fe30, SUC_lexicon_tags_0 },
    { 0x27f2fe31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f72fe33, SUC_lexicon_tags_26 },
    { 0xfe92fe34, SUC_lexicon_tags_1 },
    { 0xbb92fe35, SUC_lexicon_tags_37 },
    { 0xf6b2fe33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe512fe39, SUC_lexicon_tags_92 },
    { 0x10b2fe39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f52fe3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7372fe43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d2fe49, SUC_lexicon_tags_17 },
    { 0x8c72fe4a, SUC_lexicon_tags_30 },
    { 0xbe52fe4b, SUC_lexicon_tags_48 },
    { 0xfd52fe4c, SUC_lexicon_tags_9 },
    { 0x4332fe4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf92fe4f, SUC_lexicon_tags_30 },
    { 0x2e12fe50, SUC_lexicon_tags_27 },
    { 0xf8f2fe51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe932fe53, SUC_lexicon_tags_25 },
    { 0x6a72fe53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cb2fe56, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d2fe59, SUC_lexicon_tags_65 },
    { 0x6d32fe59, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d2fe5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b2fe61, SUC_lexicon_tags_16 },
    { 0x9ad2fe62, SUC_lexicon_tags_10 },
    { 0x5392fe63, SUC_lexicon_tags_12 },
    { 0xd372fe64, SUC_lexicon_tags_7 },
    { 0x38f2fe63, SUC_lexicon_tags_45 },
    { 0x6372fe63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4592fe68, SUC_lexicon_tags_3 },
    { 0xae92fe68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x152fe6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b2fe72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xac72fe74, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6712fe76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c12fe7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9dd2fe7c, SUC_lexicon_tags_7 },
    { 0x812fe7d, SUC_lexicon_tags_27 },
    { 0x6fb2fe7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5632fe82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc72fe85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb392fe8a, SUC_lexicon_tags_7 },
    { 0xd772fe8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd452fe8d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f2fe90, SUC_lexicon_tags_25 },
    { 0x3d2fe91, SUC_lexicon_tags_9 },
    { 0x2652fe92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f2fe95, SUC_lexicon_tags_15 },
    { 0xdef2fe96, SUC_lexicon_tags_8 },
    { 0x8032fe97, SUC_lexicon_tags_17 },
    { 0x9a92fe95, SUC_lexicon_tags_26 },
    { 0xa572fe97, SUC_lexicon_tags_41 },
    { 0xa7d2fe97, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa9d2fe9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b2fea0, SUC_lexicon_tags_29 },
    { 0x3752fea0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ff2fea3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab2fea7, SUC_lexicon_tags_275 },
    { 0x1152fea8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf2feae, SUC_lexicon_tags_35 },
    { 0x4c92feaf, SUC_lexicon_tags_55 },
    { 0xfc52feb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa432feb2, SUC_lexicon_tags_28 },
    { 0x7cb2feb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff2feb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ef2feb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf432febc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe992febf, SUC_lexicon_tags_59 },
    { 0x77b2febf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc252fec3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x12fec5, SUC_lexicon_tags_11 },
    { 0xcc72fec5, SUC_lexicon_tags_82 },
    { 0x1792fec7, SUC_lexicon_tags_56 },
    { 0x4412fec8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef2fed0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3d2fed2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad2fed5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2f2fed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d2fedc, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff2fee2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f2fee6, SUC_lexicon_tags_29 },
    { 0x7d92fee7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d2feea, SUC_lexicon_tags_6 },
    { 0xd3b2feeb, SUC_lexicon_tags_37 },
    { 0x1b32feec, SUC_lexicon_tags_1 },
    { 0xd032feec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2832feef, SUC_lexicon_tags_15 },
    { 0xe332fef0, SUC_lexicon_tags_1 },
    { 0xb092fef1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5892fef3, SUC_lexicon_tags_28 },
    { 0x71d2fef4, SUC_lexicon_tags_45 },
    { 0x9f32fef5, SUC_lexicon_tags_17 },
    { 0x612fef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fd2fef8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb012fefa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7832fefc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec12ff02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee32ff04, SUC_lexicon_tags_17 },
    { 0x6212ff05, SUC_lexicon_tags_15 },
    { 0xf892ff06, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc932ff0d, SUC_lexicon_tags_38 },
    { 0xf6f2ff0e, SUC_lexicon_tags_1 },
    { 0xa0b2ff0f, SUC_lexicon_tags_0 },
    { 0x2832ff10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6932ff13, SUC_lexicon_tags_29 },
    { 0xf112ff14, SUC_lexicon_tags_15 },
    { 0x9a12ff15, SUC_lexicon_tags_0 },
    { 0xa832ff15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28d2ff18, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed32ff1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed72ff22, SUC_lexicon_tags_37 },
    { 0xec92ff23, SUC_lexicon_tags_63 },
    { 0x6092ff22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ff27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc7f2ff29, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f52ff2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd2ff33, SUC_lexicon_tags_63 },
    { 0x6652ff34, SUC_lexicon_tags_1 },
    { 0x8df2ff35, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8712ff37, SUC_lexicon_tags_14 },
    { 0x3e92ff37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc912ff3c, SUC_lexicon_tags_37 },
    { 0x1f2ff3d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d12ff41, SUC_lexicon_tags_10 },
    { 0x39f2ff42, SUC_lexicon_tags_26 },
    { 0xf872ff43, SUC_lexicon_tags_46 },
    { 0x70d2ff44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa12ff49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3d2ff4b, SUC_lexicon_tags_60 },
    { 0xe952ff4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1372ff4e, SUC_lexicon_tags_15 },
    { 0xa592ff4f, SUC_lexicon_tags_25 },
    { 0x7692ff4e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6852ff53, SUC_lexicon_tags_27 },
    { 0xfab2ff54, SUC_lexicon_tags_17 },
    { 0x4ff2ff55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8192ff58, SUC_lexicon_tags_16 },
    { 0xb912ff59, SUC_lexicon_tags_17 },
    { 0x23f2ff58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9972ff5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd092ff65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf2ff6c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a72ff73, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d2ff76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7992ff79, SUC_lexicon_tags_37 },
    { 0x472ff79, SUC_lexicon_tags_12 },
    { 0x3d12ff79, SUC_lexicon_tags_11 },
    { 0x81b2ff7a, SUC_lexicon_tags_17 },
    { 0xcb52ff7a, SUC_lexicon_tags_39 },
    { 0x4152ff7d, SUC_lexicon_tags_1 },
    { 0xc852ff7f, SUC_lexicon_tags_25 },
    { 0xdd2ff7f, SUC_lexicon_tags_7 },
    { 0x4f52ff81, SUC_lexicon_tags_25 },
    { 0x5752ff7d, SUC_lexicon_tags_15 },
    { 0x112ff7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c92ff88, SUC_lexicon_tags_16 },
    { 0xd872ff88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa332ff90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab2ff97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b2ff9c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb52ff9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcdd2ffa1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b2ffa4, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0xae52ffa6, SUC_lexicon_tags_11 },
    { 0x11b2ffa7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8572ffa9, SUC_lexicon_tags_39 },
    { 0x43d2ffaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa12ffaf, SUC_lexicon_tags_37 },
    { 0xd292ffb0, SUC_lexicon_tags_8 },
    { 0x1152ffaf, SUC_lexicon_tags_30 },
    { 0x5d2ffb0, SUC_lexicon_tags_25 },
    { 0xdd12ffb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff2ffb7, SUC_lexicon_tags_31 },
    { 0x7d72ffb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a72ffba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3db2ffbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d2ffc3, SUC_lexicon_tags_12 },
    { 0x2e92ffc4, SUC_lexicon_tags_15 },
    { 0xe452ffc5, SUC_lexicon_tags_14 },
    { 0xceb2ffc6, SUC_lexicon_tags_29 },
    { 0xe1f2ffc7, SUC_lexicon_tags_37 },
    { 0xdfb2ffc8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f12ffcc, SUC_lexicon_tags_17 },
    { 0x7ab2ffcc, SUC_lexicon_tags_38 },
    { 0x7f52ffce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d92ffd0, SUC_lexicon_tags_1 },
    { 0xfdb2ffd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8312ffd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e72ffd5, SUC_lexicon_tags_38 },
    { 0x7eb2ffd6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2692ffd9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x112ffe5, SUC_lexicon_tags_75 },
    { 0x86f2ffe5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3152ffe8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd9b2ffea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4892fff5, SUC_lexicon_tags_37 },
    { 0x10d2fff6, SUC_lexicon_tags_14 },
    { 0x79f2fff7, SUC_lexicon_tags_0 },
    { 0xb9f2fff5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8f2fffa, SUC_lexicon_tags_15 },
    { 0x7e52fffb, SUC_lexicon_tags_0 },
    { 0x1a72fffa, SUC_lexicon_tags_37 },
    { 0x2fb2fffb, SUC_lexicon_tags_44 },
    { 0x6152fffc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x97330000, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f30003, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac530009, SUC_lexicon_tags_3 },
    { 0x1fb30009, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa973000f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x87130011, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e730013, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x2330015, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x27730017, SUC_lexicon_tags_104 },
    { 0x2d330018, SUC_lexicon_tags_1 },
    { 0x41f30019, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b3001f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73f30021, SUC_lexicon_tags_11 },
    { 0x3b330022, SUC_lexicon_tags_25 },
    { 0xc9730023, SUC_lexicon_tags_0 },
    { 0xbff30024, SUC_lexicon_tags_23 },
    { 0x2ed30024, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf9530027, SUC_lexicon_tags_1 },
    { 0x62130027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e13002a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f3002c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1913002f, SUC_lexicon_tags_25 },
    { 0x48530030, SUC_lexicon_tags_31 },
    { 0x55930031, SUC_lexicon_tags_14 },
    { 0x4d930032, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef30037, SUC_lexicon_tags_45 },
    { 0x2a730037, SUC_lexicon_tags_1 },
    { 0x8cf30038, SUC_lexicon_tags_32 },
    { 0x2753003a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xedd3003c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97130041, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbf930043, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69d30045, SUC_lexicon_tags_37 },
    { 0x22330046, SUC_lexicon_tags_37 },
    { 0x78730045, SUC_lexicon_tags_25 },
    { 0xf30046, SUC_lexicon_tags_31 },
    { 0x7d330049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x753004b, SUC_lexicon_tags_1 },
    { 0x5813004c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b33004e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd730052, SUC_lexicon_tags_33 },
    { 0xee730052, SUC_lexicon_tags_17 },
    { 0xac130054, SUC_lexicon_tags_1 },
    { 0xd2f30055, SUC_lexicon_tags_60 },
    { 0x330055, SUC_lexicon_tags_2 },
    { 0x3d30057, SUC_lexicon_tags_1 },
    { 0x50930052, SUC_lexicon_tags_2 },
    { 0xa4530059, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f3005e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d530063, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f30066, SUC_lexicon_tags_37 },
    { 0x6cd30067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e93006d, SUC_lexicon_tags_23 },
    { 0x9c13006e, SUC_lexicon_tags_4 },
    { 0x3dd3006f, SUC_lexicon_tags_69 },
    { 0x59b30070, SUC_lexicon_tags_17 },
    { 0x6c130070, SUC_lexicon_tags_8 },
    { 0xabd30072, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd30078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9173007c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ab3007e, SUC_lexicon_tags_0 },
    { 0xcf13007e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb30086, SUC_lexicon_tags_25 },
    { 0xb7530086, SUC_lexicon_tags_16 },
    { 0x9330086, SUC_lexicon_tags_12 },
    { 0xa2f30089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd993008c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c730091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3cf30093, SUC_lexicon_tags_68 },
    { 0xc930093, SUC_lexicon_tags_21 },
    { 0x57330095, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xab130098, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5413009f, SUC_lexicon_tags_27 },
    { 0x7d33009f, SUC_lexicon_tags_29 },
    { 0x6c9300a1, SUC_lexicon_tags_11 },
    { 0xc29300a2, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0x547300a4, SUC_lexicon_tags_29 },
    { 0x473300a4, SUC_lexicon_tags_25 },
    { 0x497300a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b300a9, SUC_lexicon_tags_63 },
    { 0x471300a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf45300ac, SUC_lexicon_tags_3 },
    { 0x97300ad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd300b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdc9300b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b300b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675300b9, SUC_lexicon_tags_15 },
    { 0x995300ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecf300bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45300c3, SUC_lexicon_tags_17 },
    { 0x969300c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d300cc, SUC_lexicon_tags_35 },
    { 0xfb300cd, SUC_lexicon_tags_11 },
    { 0x35f300cd, SUC_lexicon_tags_19 },
    { 0x341300ce, SUC_lexicon_tags_68 },
    { 0x61d300d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x561300d3, SUC_lexicon_tags_11 },
    { 0xd67300d4, SUC_lexicon_tags_21 },
    { 0x545300d3, SUC_lexicon_tags_64 },
    { 0x54f300d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67300db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df300de, SUC_lexicon_tags_1 },
    { 0x407300df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347300e5, SUC_lexicon_tags_72 },
    { 0xc9d300e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89300ea, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f300f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39300fc, SUC_lexicon_tags_11 },
    { 0xcdb300fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x851300ff, SUC_lexicon_tags_4 },
    { 0x65300ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0730103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d30109, SUC_lexicon_tags_37 },
    { 0x4253010a, SUC_lexicon_tags_7 },
    { 0xc373010b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x573010d, SUC_lexicon_tags_16 },
    { 0x4ab3010e, SUC_lexicon_tags_6 },
    { 0xbf33010d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xffd30111, SUC_lexicon_tags_21 },
    { 0x60d30111, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x77730114, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb30118, SUC_lexicon_tags_1 },
    { 0x97f30119, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5273011b, SUC_lexicon_tags_28 },
    { 0xb013011c, SUC_lexicon_tags_25 },
    { 0xe973011c, SUC_lexicon_tags_2 },
    { 0x66d3011c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d30127, SUC_lexicon_tags_11 },
    { 0xc0f30127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc353012c, SUC_lexicon_tags_37 },
    { 0x73b3012d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9c13012f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x90930132, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x16d30134, SUC_lexicon_tags_68 },
    { 0x79530135, SUC_lexicon_tags_60 },
    { 0xbc130136, SUC_lexicon_tags_23 },
    { 0xbe530137, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef3013a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5493013c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b30140, SUC_lexicon_tags_7 },
    { 0x2a530140, SUC_lexicon_tags_43 },
    { 0x82b30141, SUC_lexicon_tags_12 },
    { 0xc6130143, SUC_lexicon_tags_2 },
    { 0x6330141, SUC_lexicon_tags_1 },
    { 0x31530145, SUC_lexicon_tags_7 },
    { 0x3d730146, SUC_lexicon_tags_61 },
    { 0xd4130145, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc0b30149, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b730150, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb530154, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca930156, SUC_lexicon_tags_25 },
    { 0x81f30157, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce130159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c93015c, SUC_lexicon_tags_1 },
    { 0xea13015d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd30162, SUC_lexicon_tags_2 },
    { 0xe8530163, SUC_lexicon_tags_117 },
    { 0x54f30164, SUC_lexicon_tags_15 },
    { 0xffb30163, SUC_lexicon_tags_2 },
    { 0x7b730166, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4553016a, SUC_lexicon_tags_33 },
    { 0xded3016b, SUC_lexicon_tags_37 },
    { 0xf633016c, SUC_lexicon_tags_37 },
    { 0x19d3016d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf30170, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x98d30172, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb5d30174, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c530178, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b30180, SUC_lexicon_tags_62 },
    { 0x38530181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cd30183, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b930186, SUC_lexicon_tags_0 },
    { 0x51130186, SUC_lexicon_tags_17 },
    { 0x17f30187, SUC_lexicon_tags_49 },
    { 0x77f30188, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5513018f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3d330191, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b30194, SUC_lexicon_tags_43 },
    { 0x4f730195, SUC_lexicon_tags_1 },
    { 0x12930195, SUC_lexicon_tags_38 },
    { 0x3ff30195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4733019a, SUC_lexicon_tags_17 },
    { 0x233019b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d301a4, SUC_lexicon_tags_30 },
    { 0x2df301a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa9301a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d301ac, SUC_lexicon_tags_37 },
    { 0xc69301ad, SUC_lexicon_tags_7 },
    { 0x401301ac, SUC_lexicon_tags_17 },
    { 0x9c9301ad, SUC_lexicon_tags_25 },
    { 0x67f301b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b301b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7301b5, SUC_lexicon_tags_19 },
    { 0x1fb301b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x741301b9, SUC_lexicon_tags_12 },
    { 0x531301ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843301c1, SUC_lexicon_tags_37 },
    { 0x5d5301c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f301c6, SUC_lexicon_tags_25 },
    { 0xb93301c7, SUC_lexicon_tags_34 },
    { 0x5e5301c8, SUC_lexicon_tags_87 },
    { 0xf99301c7, SUC_lexicon_tags_31 },
    { 0x539301c7, SUC_lexicon_tags_17 },
    { 0x82f301c6, SUC_lexicon_tags_185 },
    { 0x19d301c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd35301ce, SUC_lexicon_tags_37 },
    { 0xc6f301cf, SUC_lexicon_tags_31 },
    { 0xf51301cf, SUC_lexicon_tags_15 },
    { 0x8e3301d0, SUC_lexicon_tags_8 },
    { 0x93d301d2, SUC_lexicon_tags_25 },
    { 0xdcb301d3, SUC_lexicon_tags_34 },
    { 0xd4f301d3, SUC_lexicon_tags_12 },
    { 0xbab301d3, SUC_lexicon_tags_11 },
    { 0xf27301d4, SUC_lexicon_tags_69 },
    { 0x3c5301d7, SUC_lexicon_tags_15 },
    { 0x69d301d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b301db, SUC_lexicon_tags_12 },
    { 0xa59301dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71301e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b301e5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa9301e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf301e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x447301ec, SUC_lexicon_tags_48 },
    { 0x931301ec, SUC_lexicon_tags_4 },
    { 0x311301ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5f301f0, SUC_lexicon_tags_7 },
    { 0x671301f0, SUC_lexicon_tags_31 },
    { 0x8e7301f2, SUC_lexicon_tags_1 },
    { 0xaa7301f3, SUC_lexicon_tags_30 },
    { 0x7ed301f4, SUC_lexicon_tags_19 },
    { 0xf1d301f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b9301f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c9301f9, SUC_lexicon_tags_45 },
    { 0xb7b301fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb301ff, SUC_lexicon_tags_1 },
    { 0x40130200, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa1330202, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab30208, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9893020c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1730210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b30215, SUC_lexicon_tags_7 },
    { 0xd2b30215, SUC_lexicon_tags_9 },
    { 0x52730217, SUC_lexicon_tags_39 },
    { 0x74730218, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c33021d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb213021f, SUC_lexicon_tags_9 },
    { 0x20d30220, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2530222, SUC_lexicon_tags_2 },
    { 0xb4d30223, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7130226, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1973022a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e33022e, SUC_lexicon_tags_2 },
    { 0x6ef3022e, SUC_lexicon_tags_16 },
    { 0x28130230, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2130233, SUC_lexicon_tags_61 },
    { 0x1fb30234, SUC_lexicon_tags_43 },
    { 0xc6d30235, SUC_lexicon_tags_82 },
    { 0x8c730236, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33023a, SUC_lexicon_tags_21 },
    { 0x1013023b, SUC_lexicon_tags_91 },
    { 0x91d3023c, SUC_lexicon_tags_7 },
    { 0xcdf3023c, SUC_lexicon_tags_25 },
    { 0x7c33023b, SUC_lexicon_tags_27 },
    { 0xf6d3023f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4530241, SUC_lexicon_tags_26 },
    { 0x2af30241, SUC_lexicon_tags_2 },
    { 0x7e930243, SUC_lexicon_tags_28 },
    { 0x1ab30243, SUC_lexicon_tags_35 },
    { 0xa5930242, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92930247, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1df30249, SUC_lexicon_tags_9 },
    { 0xd9f30249, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95d3024c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18130255, SUC_lexicon_tags_7 },
    { 0xcc130256, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xda130259, SUC_lexicon_tags_60 },
    { 0x8053025a, SUC_lexicon_tags_25 },
    { 0x6813025b, SUC_lexicon_tags_47 },
    { 0x6ed3025b, SUC_lexicon_tags_33 },
    { 0x3cb3025a, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf30262, SUC_lexicon_tags_37 },
    { 0xc7b30263, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4b30265, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b30271, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x70130274, SUC_lexicon_tags_7 },
    { 0xb530274, SUC_lexicon_tags_38 },
    { 0x75d30275, SUC_lexicon_tags_17 },
    { 0x8c130277, SUC_lexicon_tags_46 },
    { 0x2b130276, SUC_lexicon_tags_45 },
    { 0xac730279, SUC_lexicon_tags_3 },
    { 0x18130279, SUC_lexicon_tags_11 },
    { 0x5530277, SUC_lexicon_tags_11 },
    { 0x6ff30274, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e73027e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1b930280, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a530282, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x20730285, SUC_lexicon_tags_42 },
    { 0xdbb30286, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee530288, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd930291, SUC_lexicon_tags_25 },
    { 0x2530292, SUC_lexicon_tags_16 },
    { 0xa730293, SUC_lexicon_tags_136 },
    { 0x97f30294, SUC_lexicon_tags_2 },
    { 0xb2930294, SUC_lexicon_tags_2 },
    { 0xb3330296, SUC_lexicon_tags_37 },
    { 0x7c130293, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd153029d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697302a2, SUC_lexicon_tags_2 },
    { 0x6bd302a3, SUC_lexicon_tags_17 },
    { 0x2a9302a4, SUC_lexicon_tags_38 },
    { 0x599302a5, SUC_lexicon_tags_30 },
    { 0x9ad302a5, SUC_lexicon_tags_57 },
    { 0xd05302a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8eb302a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5302ac, SUC_lexicon_tags_6 },
    { 0xbd302ad, SUC_lexicon_tags_66 },
    { 0xe5302ad, SUC_lexicon_tags_17 },
    { 0x5df302af, SUC_lexicon_tags_37 },
    { 0xff5302ad, SUC_lexicon_tags_38 },
    { 0xa23302b1, SUC_lexicon_tags_66 },
    { 0x8c1302b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x137302b5, SUC_lexicon_tags_15 },
    { 0x833302b6, SUC_lexicon_tags_7 },
    { 0x379302b5, SUC_lexicon_tags_9 },
    { 0xd9f302b8, SUC_lexicon_tags_54 },
    { 0x85302b8, SUC_lexicon_tags_7 },
    { 0xe41302ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6fb302bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x911302bf, SUC_lexicon_tags_1 },
    { 0xc9f302c0, SUC_lexicon_tags_37 },
    { 0x423302c1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571302c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5302ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b302d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9302d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9302e0, SUC_lexicon_tags_31 },
    { 0x98d302e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb302e4, SUC_lexicon_tags_14 },
    { 0xe8b302e4, SUC_lexicon_tags_107 },
    { 0xd43302e6, SUC_lexicon_tags_23 },
    { 0x8c7302e7, SUC_lexicon_tags_37 },
    { 0x4f302e8, SUC_lexicon_tags_37 },
    { 0x3b302e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573302ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x377302f1, SUC_lexicon_tags_47 },
    { 0xbc3302f2, SUC_lexicon_tags_3 },
    { 0xff302f2, SUC_lexicon_tags_21 },
    { 0x16f302f3, SUC_lexicon_tags_27 },
    { 0xa41302f5, SUC_lexicon_tags_33 },
    { 0xd35302f5, SUC_lexicon_tags_28 },
    { 0x58f302f1, SUC_lexicon_tags_63 },
    { 0xe0f302f8, SUC_lexicon_tags_23 },
    { 0x971302f7, SUC_lexicon_tags_2 },
    { 0x93b302f8, SUC_lexicon_tags_7 },
    { 0xacb302f5, SUC_lexicon_tags_2 },
    { 0x489302fc, SUC_lexicon_tags_33 },
    { 0xd63302fd, SUC_lexicon_tags_38 },
    { 0xcb9302fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27730305, SUC_lexicon_tags_38 },
    { 0x3df30306, SUC_lexicon_tags_24 },
    { 0x5c930307, SUC_lexicon_tags_11 },
    { 0x2d330307, SUC_lexicon_tags_12 },
    { 0x26d30308, SUC_lexicon_tags_17 },
    { 0x6bf3030a, SUC_lexicon_tags_12 },
    { 0x31d30307, SUC_lexicon_tags_2 },
    { 0xf253030c, SUC_lexicon_tags_12 },
    { 0x6173030d, SUC_lexicon_tags_1 },
    { 0xf493030d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb730313, SUC_lexicon_tags_2 },
    { 0x92330314, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64130317, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b3031c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3753031e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d30323, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68530329, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8b3032b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95330333, SUC_lexicon_tags_21 },
    { 0x31d30333, SUC_lexicon_tags_12 },
    { 0xb9530335, SUC_lexicon_tags_12 },
    { 0xe9730336, SUC_lexicon_tags_17 },
    { 0xd0530336, SUC_lexicon_tags_37 },
    { 0x42330338, SUC_lexicon_tags_11 },
    { 0x1b930339, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f3033c, SUC_lexicon_tags_13 },
    { 0xa1d3033d, SUC_lexicon_tags_29 },
    { 0x63b3033e, SUC_lexicon_tags_148 },
    { 0x8f53033e, SUC_lexicon_tags_45 },
    { 0x5293033c, SUC_lexicon_tags_65 },
    { 0xfa530341, SUC_lexicon_tags_37 },
    { 0xdc13033e, SUC_lexicon_tags_7 },
    { 0x25d3033f, SUC_lexicon_tags_19 },
    { 0x8e730344, SUC_lexicon_tags_29 },
    { 0xb0f3033e, SUC_lexicon_tags_23 },
    { 0x9e730346, SUC_lexicon_tags_9 },
    { 0x33f3033f, SUC_lexicon_tags_39 },
    { 0x94530348, SUC_lexicon_tags_7 },
    { 0xb7d30349, SUC_lexicon_tags_63 },
    { 0xed330348, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b3034e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b30355, SUC_lexicon_tags_15 },
    { 0x76530355, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3f130358, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2213035b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x27d3035d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41930360, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xced30363, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d3036a, SUC_lexicon_tags_25 },
    { 0xb213036b, SUC_lexicon_tags_17 },
    { 0xca93036c, SUC_lexicon_tags_12 },
    { 0xb6b3036d, SUC_lexicon_tags_17 },
    { 0x91f3036e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd30371, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9730378, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9413037a, SUC_lexicon_tags_47 },
    { 0xb7f3037a, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0530381, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5730383, SUC_lexicon_tags_13 },
    { 0xd3d30384, SUC_lexicon_tags_6 },
    { 0x55730383, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6253038f, SUC_lexicon_tags_7 },
    { 0x413038f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd30393, SUC_lexicon_tags_60 },
    { 0xa6b30394, SUC_lexicon_tags_27 },
    { 0x74f30393, SUC_lexicon_tags_182 },
    { 0xbb930396, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a330398, SUC_lexicon_tags_2 },
    { 0x7dd30399, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4913039c, SUC_lexicon_tags_11 },
    { 0x40b3039d, SUC_lexicon_tags_60 },
    { 0x4793039e, SUC_lexicon_tags_57 },
    { 0x3953039e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x969303a1, SUC_lexicon_tags_107 },
    { 0x3ef303a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b9303a4, SUC_lexicon_tags_11 },
    { 0xcbd303a5, SUC_lexicon_tags_37 },
    { 0xb25303a4, SUC_lexicon_tags_23 },
    { 0x6cd303a4, SUC_lexicon_tags_12 },
    { 0xd03303a6, SUC_lexicon_tags_14 },
    { 0x5d5303a6, SUC_lexicon_tags_11 },
    { 0x559303aa, SUC_lexicon_tags_71 },
    { 0x885303ab, SUC_lexicon_tags_48 },
    { 0x8ad303a5, SUC_lexicon_tags_65 },
    { 0x2b9303ad, SUC_lexicon_tags_2 },
    { 0x253303a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd303b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x409303b6, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181303bb, SUC_lexicon_tags_7 },
    { 0xb41303bc, SUC_lexicon_tags_11 },
    { 0xb93303bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1303c4, SUC_lexicon_tags_37 },
    { 0x9f5303c5, SUC_lexicon_tags_43 },
    { 0x8b5303c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f303c9, SUC_lexicon_tags_13 },
    { 0x257303c9, SUC_lexicon_tags_43 },
    { 0xa5b303c9, SUC_lexicon_tags_37 },
    { 0x5c9303cc, SUC_lexicon_tags_55 },
    { 0x16b303cd, SUC_lexicon_tags_12 },
    { 0xe3303cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1303d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f303db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb303de, SUC_lexicon_tags_12 },
    { 0x82d303df, SUC_lexicon_tags_9 },
    { 0x817303de, SUC_lexicon_tags_55 },
    { 0x8b5303df, SUC_lexicon_tags_8 },
    { 0x2a1303e2, SUC_lexicon_tags_33 },
    { 0x63d303e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1303e7, SUC_lexicon_tags_17 },
    { 0x1d303e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x917303ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf303ee, SUC_lexicon_tags_4 },
    { 0x845303ef, SUC_lexicon_tags_87 },
    { 0x547303f0, SUC_lexicon_tags_37 },
    { 0x693303ef, SUC_lexicon_tags_33 },
    { 0x535303f2, SUC_lexicon_tags_11 },
    { 0x495303f3, SUC_lexicon_tags_358 },
    { 0x1e3303f3, SUC_lexicon_tags_2 },
    { 0x7d7303f5, SUC_lexicon_tags_4 },
    { 0x5d7303f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x101303f9, SUC_lexicon_tags_17 },
    { 0x8f3303fa, SUC_lexicon_tags_28 },
    { 0xd37303fb, SUC_lexicon_tags_27 },
    { 0x459303fc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9e7303fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe730402, SUC_lexicon_tags_15 },
    { 0x65b30402, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead30408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda33040b, SUC_lexicon_tags_48 },
    { 0xda73040c, SUC_lexicon_tags_11 },
    { 0x3ad3040d, SUC_lexicon_tags_27 },
    { 0xc1d3040b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b30411, SUC_lexicon_tags_25 },
    { 0x2fd30412, SUC_lexicon_tags_23 },
    { 0xa5d30412, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a330416, SUC_lexicon_tags_56 },
    { 0xa6b30417, SUC_lexicon_tags_12 },
    { 0xd5530418, SUC_lexicon_tags_2 },
    { 0x90330419, SUC_lexicon_tags_0 },
    { 0x64530416, SUC_lexicon_tags_12 },
    { 0x1e930417, SUC_lexicon_tags_1 },
    { 0xfcb3041c, SUC_lexicon_tags_47 },
    { 0x26d3041c, SUC_lexicon_tags_12 },
    { 0x6933041e, SUC_lexicon_tags_10 },
    { 0x1193041f, SUC_lexicon_tags_11 },
    { 0xb2b3041e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d130426, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x88b30428, SUC_lexicon_tags_39 },
    { 0x88330428, SUC_lexicon_tags_60 },
    { 0x48d30428, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea33042f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55730432, SUC_lexicon_tags_48 },
    { 0xe3f30433, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5473043d, SUC_lexicon_tags_26 },
    { 0x64d3043d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d930443, SUC_lexicon_tags_15 },
    { 0x12130443, SUC_lexicon_tags_17 },
    { 0xd8930444, SUC_lexicon_tags_15 },
    { 0xb2730446, SUC_lexicon_tags_70 },
    { 0xda730447, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb3044c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b330451, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80330453, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe9130455, SUC_lexicon_tags_47 },
    { 0x88730456, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a93045a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3153045c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8133045e, SUC_lexicon_tags_33 },
    { 0x3e53045e, SUC_lexicon_tags_15 },
    { 0x2130460, SUC_lexicon_tags_32 },
    { 0x78b30460, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c730463, SUC_lexicon_tags_7 },
    { 0xcbb30463, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x68530467, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1013046a, SUC_lexicon_tags_56 },
    { 0xf1f3046b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af3046e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x92130471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaaf30473, SUC_lexicon_tags_19 },
    { 0x8930473, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b3047d, SUC_lexicon_tags_25 },
    { 0xd913047e, SUC_lexicon_tags_25 },
    { 0xf253047f, SUC_lexicon_tags_49 },
    { 0x57330480, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb30483, SUC_lexicon_tags_14 },
    { 0x49530483, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xff30486, SUC_lexicon_tags_43 },
    { 0x68530487, SUC_lexicon_tags_10 },
    { 0xfc530488, SUC_lexicon_tags_14 },
    { 0x36530488, SUC_lexicon_tags_63 },
    { 0x66b3048a, SUC_lexicon_tags_47 },
    { 0xe473048b, SUC_lexicon_tags_27 },
    { 0x47b3048c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f30493, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95d30495, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6653049a, SUC_lexicon_tags_2 },
    { 0x2733049b, SUC_lexicon_tags_19 },
    { 0xced3049a, SUC_lexicon_tags_25 },
    { 0xc453049a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x499304a0, SUC_lexicon_tags_42 },
    { 0xc9f304a1, SUC_lexicon_tags_15 },
    { 0x3dd304a1, SUC_lexicon_tags_37 },
    { 0x125304a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3304a9, SUC_lexicon_tags_9 },
    { 0xa53304aa, SUC_lexicon_tags_23 },
    { 0x5f1304aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3304b0, SUC_lexicon_tags_61 },
    { 0x3b5304b1, SUC_lexicon_tags_12 },
    { 0xafb304b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7304ba, SUC_lexicon_tags_40 },
    { 0xe95304bb, SUC_lexicon_tags_16 },
    { 0x27b304bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd33304be, SUC_lexicon_tags_21 },
    { 0xa3b304bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157304c4, SUC_lexicon_tags_37 },
    { 0xdf9304c5, SUC_lexicon_tags_0 },
    { 0x84f304c5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17304c9, SUC_lexicon_tags_82 },
    { 0x1eb304ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59304cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555304d1, SUC_lexicon_tags_30 },
    { 0x6b1304d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449304da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x551304dc, SUC_lexicon_tags_2 },
    { 0xc69304dd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b304e1, SUC_lexicon_tags_6 },
    { 0x6bb304e1, SUC_lexicon_tags_1 },
    { 0x14b304e3, SUC_lexicon_tags_17 },
    { 0xb09304e3, SUC_lexicon_tags_14 },
    { 0x7d1304e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f304e8, SUC_lexicon_tags_8 },
    { 0xfc1304e9, SUC_lexicon_tags_49 },
    { 0x41b304ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353304f0, SUC_lexicon_tags_47 },
    { 0x553304f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb304f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949304fb, SUC_lexicon_tags_12 },
    { 0x16d304fc, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d30500, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3ff30502, SUC_lexicon_tags_11 },
    { 0x12330503, SUC_lexicon_tags_23 },
    { 0x4eb30502, SUC_lexicon_tags_1 },
    { 0x2ed30505, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0130509, SUC_lexicon_tags_64 },
    { 0x9893050a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133050f, SUC_lexicon_tags_19 },
    { 0xccf30510, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8930513, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb30516, SUC_lexicon_tags_21 },
    { 0x6d930517, SUC_lexicon_tags_56 },
    { 0xceb30518, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1553051e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40130527, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3130530, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1330533, SUC_lexicon_tags_8 },
    { 0x74730533, SUC_lexicon_tags_25 },
    { 0x36730535, SUC_lexicon_tags_47 },
    { 0x4c330533, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4693053a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1093053c, SUC_lexicon_tags_11 },
    { 0xf4d3053c, SUC_lexicon_tags_15 },
    { 0x97f3053e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92730541, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68130547, SUC_lexicon_tags_43 },
    { 0x51b30548, SUC_lexicon_tags_19 },
    { 0x5c930548, SUC_lexicon_tags_12 },
    { 0x30f30549, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x81b3054c, SUC_lexicon_tags_25 },
    { 0xf1b3054c, SUC_lexicon_tags_1 },
    { 0xb6b3054e, SUC_lexicon_tags_45 },
    { 0x17b3054f, SUC_lexicon_tags_23 },
    { 0x43f30550, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f930552, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43730554, SUC_lexicon_tags_63 },
    { 0x2d930554, SUC_lexicon_tags_7 },
    { 0x6c730556, SUC_lexicon_tags_14 },
    { 0xd9930557, SUC_lexicon_tags_34 },
    { 0x60530558, SUC_lexicon_tags_17 },
    { 0xdd730557, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d3055d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b30561, SUC_lexicon_tags_11 },
    { 0xb4930562, SUC_lexicon_tags_0 },
    { 0x7f30563, SUC_lexicon_tags_21 },
    { 0x25730563, SUC_lexicon_tags_2 },
    { 0x77b30564, SUC_lexicon_tags_7 },
    { 0xd0d30563, SUC_lexicon_tags_11 },
    { 0x2d730567, SUC_lexicon_tags_55 },
    { 0xe5930568, SUC_lexicon_tags_63 },
    { 0x730569, SUC_lexicon_tags_42 },
    { 0xded3056a, SUC_lexicon_tags_40 },
    { 0x74b3056a, SUC_lexicon_tags_68 },
    { 0x3af3056a, SUC_lexicon_tags_45 },
    { 0xd733056a, SUC_lexicon_tags_7 },
    { 0x5793056e, SUC_lexicon_tags_23 },
    { 0xcfd3056e, SUC_lexicon_tags_15 },
    { 0xf8b3056d, SUC_lexicon_tags_3 },
    { 0xfbd3056f, SUC_lexicon_tags_11 },
    { 0x3dd30571, SUC_lexicon_tags_57 },
    { 0xef330571, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2930577, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16730579, SUC_lexicon_tags_15 },
    { 0xadb3057a, SUC_lexicon_tags_26 },
    { 0xdcd3057b, SUC_lexicon_tags_30 },
    { 0x5593057c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4013057e, SUC_lexicon_tags_21 },
    { 0x89d3057e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8530585, SUC_lexicon_tags_45 },
    { 0xbf330586, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f930589, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5733058d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d30590, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41f30592, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb130594, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b530596, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb730598, SUC_lexicon_tags_17 },
    { 0x6bd30598, SUC_lexicon_tags_17 },
    { 0xcdb3059a, SUC_lexicon_tags_1 },
    { 0x80930598, SUC_lexicon_tags_38 },
    { 0x4fd30599, SUC_lexicon_tags_47 },
    { 0x153059d, SUC_lexicon_tags_25 },
    { 0xf0f3059d, SUC_lexicon_tags_17 },
    { 0x7b73059f, SUC_lexicon_tags_2 },
    { 0x40f3059b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f305a3, SUC_lexicon_tags_84 },
    { 0xf3f305a4, SUC_lexicon_tags_47 },
    { 0xecf305a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b3305a7, SUC_lexicon_tags_15 },
    { 0x239305a7, SUC_lexicon_tags_53 },
    { 0x58d305a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d305ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeed305ae, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xdfb305b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51f305b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x70f305b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5305bf, SUC_lexicon_tags_31 },
    { 0xb45305c0, SUC_lexicon_tags_61 },
    { 0xf99305c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb305c8, SUC_lexicon_tags_63 },
    { 0xa19305c8, SUC_lexicon_tags_19 },
    { 0xc23305c8, SUC_lexicon_tags_11 },
    { 0xa07305ca, SUC_lexicon_tags_146 },
    { 0x7c9305cc, SUC_lexicon_tags_11 },
    { 0x2e7305cd, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9305d2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x639305d4, SUC_lexicon_tags_12 },
    { 0x23d305d5, SUC_lexicon_tags_1 },
    { 0x3b7305d5, SUC_lexicon_tags_53 },
    { 0xf71305d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35305d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7305dd, SUC_lexicon_tags_37 },
    { 0x441305dd, SUC_lexicon_tags_11 },
    { 0x51b305dd, SUC_lexicon_tags_25 },
    { 0xb09305e0, SUC_lexicon_tags_11 },
    { 0xe73305e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa7b305e3, SUC_lexicon_tags_2 },
    { 0x537305e4, SUC_lexicon_tags_47 },
    { 0x2bb305e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1305e9, SUC_lexicon_tags_47 },
    { 0x929305e9, SUC_lexicon_tags_30 },
    { 0xf55305eb, SUC_lexicon_tags_37 },
    { 0x17305eb, SUC_lexicon_tags_39 },
    { 0xe59305ed, SUC_lexicon_tags_25 },
    { 0x28d305ed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x811305f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01305f6, SUC_lexicon_tags_71 },
    { 0x1f305f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9305fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbad305fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b305ff, SUC_lexicon_tags_38 },
    { 0xccd305ff, SUC_lexicon_tags_15 },
    { 0xc7930600, SUC_lexicon_tags_65 },
    { 0xb7530602, SUC_lexicon_tags_17 },
    { 0xe8930602, SUC_lexicon_tags_45 },
    { 0xcb130603, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a330607, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61f30609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7853060c, SUC_lexicon_tags_1 },
    { 0x9df3060d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91330610, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca730617, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xab730619, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xf5b3061b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe53061e, SUC_lexicon_tags_1011 },
    { 0x4653061f, SUC_lexicon_tags_11 },
    { 0x9f13061f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1930623, SUC_lexicon_tags_4 },
    { 0xce730623, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c130626, SUC_lexicon_tags_11 },
    { 0x61b30627, SUC_lexicon_tags_31 },
    { 0xf7b30628, SUC_lexicon_tags_14 },
    { 0xf2f30628, SUC_lexicon_tags_27 },
    { 0x2d730628, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b30633, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88530635, SUC_lexicon_tags_61 },
    { 0xa8330636, SUC_lexicon_tags_37 },
    { 0x54d30637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf3063a, SUC_lexicon_tags_17 },
    { 0xb1d3063b, SUC_lexicon_tags_113 },
    { 0x62b3063a, SUC_lexicon_tags_30 },
    { 0xd7b3063d, SUC_lexicon_tags_12 },
    { 0xe8d3063a, SUC_lexicon_tags_3 },
    { 0x5053063f, SUC_lexicon_tags_32 },
    { 0x5173063d, SUC_lexicon_tags_27 },
    { 0x87330641, SUC_lexicon_tags_2 },
    { 0x33930640, SUC_lexicon_tags_27 },
    { 0xdb30643, SUC_lexicon_tags_0 },
    { 0xeb130644, SUC_lexicon_tags_63 },
    { 0xdb930645, SUC_lexicon_tags_19 },
    { 0xc8530641, SUC_lexicon_tags_17 },
    { 0xa9130647, SUC_lexicon_tags_1 },
    { 0x93b30648, SUC_lexicon_tags_87 },
    { 0xb930649, SUC_lexicon_tags_11 },
    { 0x6413063b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9113064d, SUC_lexicon_tags_45 },
    { 0x25d3064e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67130650, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x62930652, SUC_lexicon_tags_25 },
    { 0xa9f30652, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff30658, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6313065a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe393065e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34330660, SUC_lexicon_tags_25 },
    { 0xaf930661, SUC_lexicon_tags_27 },
    { 0xce530662, SUC_lexicon_tags_1 },
    { 0x75530661, SUC_lexicon_tags_45 },
    { 0x31730661, SUC_lexicon_tags_17 },
    { 0x2eb30665, SUC_lexicon_tags_7 },
    { 0xbaf30666, SUC_lexicon_tags_79 },
    { 0x79330667, SUC_lexicon_tags_7 },
    { 0x8bb30663, SUC_lexicon_tags_38 },
    { 0x8a730669, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31530670, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5d30672, SUC_lexicon_tags_25 },
    { 0x55f30673, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83130678, SUC_lexicon_tags_12 },
    { 0xb1130679, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23d3067b, SUC_lexicon_tags_21 },
    { 0x9573067b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9113067e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa9330680, SUC_lexicon_tags_31 },
    { 0x57930680, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x58330684, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9073068b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb130690, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b730695, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9930697, SUC_lexicon_tags_25 },
    { 0xc4d30698, SUC_lexicon_tags_47 },
    { 0x130698, SUC_lexicon_tags_19 },
    { 0x2ad3069a, SUC_lexicon_tags_25 },
    { 0x3df3069a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd33069f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5306a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83306ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57306ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x331306b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x261306b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df306b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x215306bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bb306be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa87306c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d306c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7306cb, SUC_lexicon_tags_12 },
    { 0x5d5306cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf55306ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91306d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3306d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xff7306da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49306dd, SUC_lexicon_tags_25 },
    { 0x823306de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccb306e0, SUC_lexicon_tags_816 },
    { 0x17b306e1, SUC_lexicon_tags_21 },
    { 0xb306e2, SUC_lexicon_tags_15 },
    { 0x57b306e3, SUC_lexicon_tags_17 },
    { 0xb03306e4, SUC_lexicon_tags_37 },
    { 0xdeb306e2, SUC_lexicon_tags_25 },
    { 0x81d306e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xce3306e8, SUC_lexicon_tags_15 },
    { 0xf63306e9, SUC_lexicon_tags_38 },
    { 0x3b9306ea, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b306ed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6a9306ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7df306f1, SUC_lexicon_tags_12 },
    { 0xb21306f2, SUC_lexicon_tags_11 },
    { 0x9b9306f3, SUC_lexicon_tags_45 },
    { 0x6ed306f1, SUC_lexicon_tags_37 },
    { 0x81306f5, SUC_lexicon_tags_15 },
    { 0x11f306f1, SUC_lexicon_tags_60 },
    { 0x697306f7, SUC_lexicon_tags_2 },
    { 0x9cb306f4, SUC_lexicon_tags_1 },
    { 0x64d306f9, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x31d306fb, SUC_lexicon_tags_25 },
    { 0xd71306fc, SUC_lexicon_tags_7 },
    { 0x233306fc, SUC_lexicon_tags_6 },
    { 0xccb306fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f30708, SUC_lexicon_tags_2 },
    { 0x4bd30708, SUC_lexicon_tags_17 },
    { 0x4d330709, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd3d3070c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd113070e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93130712, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb30718, SUC_lexicon_tags_60 },
    { 0xb0930718, SUC_lexicon_tags_19 },
    { 0xaa13071a, SUC_lexicon_tags_11 },
    { 0xd4d3071b, SUC_lexicon_tags_25 },
    { 0xe2f3071c, SUC_lexicon_tags_55 },
    { 0xed53071b, SUC_lexicon_tags_45 },
    { 0x7f33071c, SUC_lexicon_tags_12 },
    { 0x7c13071f, SUC_lexicon_tags_104 },
    { 0x2db3071b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b30723, SUC_lexicon_tags_9 },
    { 0x1fd30724, SUC_lexicon_tags_29 },
    { 0x45f30725, SUC_lexicon_tags_60 },
    { 0x8b530724, SUC_lexicon_tags_12 },
    { 0xcdf30725, SUC_lexicon_tags_43 },
    { 0x37930728, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb3072f, SUC_lexicon_tags_26 },
    { 0x93730730, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd30732, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53530734, SUC_lexicon_tags_14 },
    { 0x22f30735, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf730738, SUC_lexicon_tags_17 },
    { 0xe2330739, SUC_lexicon_tags_17 },
    { 0x6c530739, SUC_lexicon_tags_7 },
    { 0x7693073b, SUC_lexicon_tags_15 },
    { 0xcd53073b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f30740, SUC_lexicon_tags_33 },
    { 0xf8730741, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87b30743, SUC_lexicon_tags_11 },
    { 0x1e530744, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf873074a, SUC_lexicon_tags_17 },
    { 0x5f33074b, SUC_lexicon_tags_25 },
    { 0x9253074b, SUC_lexicon_tags_28 },
    { 0xb353074d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d930752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95130755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7130758, SUC_lexicon_tags_27 },
    { 0xf5330759, SUC_lexicon_tags_19 },
    { 0x4913075a, SUC_lexicon_tags_63 },
    { 0x24f3075b, SUC_lexicon_tags_12 },
    { 0xdbd3075a, SUC_lexicon_tags_17 },
    { 0x5933075a, SUC_lexicon_tags_1 },
    { 0x1a93075e, SUC_lexicon_tags_25 },
    { 0x2c53075c, SUC_lexicon_tags_12 },
    { 0xc2b30760, SUC_lexicon_tags_11 },
    { 0xb4530761, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b30764, SUC_lexicon_tags_34 },
    { 0xec330765, SUC_lexicon_tags_49 },
    { 0xd7d30766, SUC_lexicon_tags_7 },
    { 0x73130767, SUC_lexicon_tags_3 },
    { 0x47f30768, SUC_lexicon_tags_70 },
    { 0x79d30769, SUC_lexicon_tags_17 },
    { 0x9133076a, SUC_lexicon_tags_28 },
    { 0xed530766, SUC_lexicon_tags_14 },
    { 0xbc33076c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6b3076e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e530772, SUC_lexicon_tags_17 },
    { 0xa7730773, SUC_lexicon_tags_38 },
    { 0xc3730774, SUC_lexicon_tags_63 },
    { 0xf7d30775, SUC_lexicon_tags_1 },
    { 0x9d930772, SUC_lexicon_tags_23 },
    { 0xcfb30777, SUC_lexicon_tags_25 },
    { 0xf0b30777, SUC_lexicon_tags_1 },
    { 0x87730777, SUC_lexicon_tags_7 },
    { 0xf8b3077a, SUC_lexicon_tags_25 },
    { 0x5f93077b, SUC_lexicon_tags_37 },
    { 0x45530777, SUC_lexicon_tags_10 },
    { 0xb9930774, SUC_lexicon_tags_23 },
    { 0x7a73077e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd1d30780, SUC_lexicon_tags_37 },
    { 0xdd730781, SUC_lexicon_tags_25 },
    { 0xc7f30782, SUC_lexicon_tags_15 },
    { 0x1f130781, SUC_lexicon_tags_33 },
    { 0x25b30784, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7330787, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65330791, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3330793, SUC_lexicon_tags_10 },
    { 0x5f730794, SUC_lexicon_tags_2 },
    { 0x72730794, SUC_lexicon_tags_37 },
    { 0x92130796, SUC_lexicon_tags_28 },
    { 0x89730796, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2853079a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb853079f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59307a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa31307a7, SUC_lexicon_tags_15 },
    { 0xe63307a7, SUC_lexicon_tags_32 },
    { 0xceb307a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x887307ac, SUC_lexicon_tags_8 },
    { 0x1a5307ad, SUC_lexicon_tags_15 },
    { 0xf39307ae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x61f307b0, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf307b5, SUC_lexicon_tags_9 },
    { 0x31b307b6, SUC_lexicon_tags_12 },
    { 0x6c7307b7, SUC_lexicon_tags_25 },
    { 0x845307b8, SUC_lexicon_tags_26 },
    { 0x1a7307b9, SUC_lexicon_tags_63 },
    { 0x7ad307ba, SUC_lexicon_tags_17 },
    { 0x4b5307b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef307be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcf3307c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12f307c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d307c5, SUC_lexicon_tags_0 },
    { 0xda3307c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7dd307c8, SUC_lexicon_tags_45 },
    { 0x38f307c8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b307cc, SUC_lexicon_tags_32 },
    { 0x797307cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15307d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d307d8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b307db, SUC_lexicon_tags_17 },
    { 0xfed307dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e9307de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f307e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13307e6, SUC_lexicon_tags_15 },
    { 0x103307e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b307ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x823307ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb307f0, SUC_lexicon_tags_15 },
    { 0x3b3307f1, SUC_lexicon_tags_45 },
    { 0xddb307f2, SUC_lexicon_tags_37 },
    { 0x9b9307f0, SUC_lexicon_tags_1 },
    { 0x6a3307f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f307f6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13307fd, SUC_lexicon_tags_37 },
    { 0xadd307fd, SUC_lexicon_tags_37 },
    { 0x32b307fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1ef30801, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x48530804, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66f30806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac930809, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b3080f, SUC_lexicon_tags_11 },
    { 0xe593080f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9f30812, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f30816, SUC_lexicon_tags_7 },
    { 0x27930817, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4793081c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb73081f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d530821, SUC_lexicon_tags_25 },
    { 0x13b30822, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafb30824, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x42730827, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7153082e, SUC_lexicon_tags_12 },
    { 0x6833082f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf30833, SUC_lexicon_tags_30 },
    { 0x2d530834, SUC_lexicon_tags_68 },
    { 0x3a130835, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf873083b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb93083e, SUC_lexicon_tags_27 },
    { 0x1b93083f, SUC_lexicon_tags_17 },
    { 0x96530840, SUC_lexicon_tags_1 },
    { 0x4e53083f, SUC_lexicon_tags_45 },
    { 0x2093083e, SUC_lexicon_tags_63 },
    { 0xf2b3083f, SUC_lexicon_tags_47 },
    { 0x45f3083f, SUC_lexicon_tags_25 },
    { 0xadf30844, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb30849, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3633084b, SUC_lexicon_tags_15 },
    { 0x3ff3084c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd4f3084e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64330850, SUC_lexicon_tags_11 },
    { 0xf8f30851, SUC_lexicon_tags_1 },
    { 0x66130852, SUC_lexicon_tags_496 },
    { 0xfff30852, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75330857, SUC_lexicon_tags_17 },
    { 0xc7130858, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d3085c, SUC_lexicon_tags_30 },
    { 0x2513085d, SUC_lexicon_tags_1 },
    { 0x8c93085d, SUC_lexicon_tags_14 },
    { 0x3393085f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75530863, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b30867, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d3086b, SUC_lexicon_tags_2 },
    { 0xe353086b, SUC_lexicon_tags_1 },
    { 0x39f3086d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71f3086f, SUC_lexicon_tags_7 },
    { 0x5f30870, SUC_lexicon_tags_7 },
    { 0x19330871, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b30877, SUC_lexicon_tags_22 },
    { 0x5db30878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5f3087a, SUC_lexicon_tags_17 },
    { 0x613087b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b33087f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f530883, SUC_lexicon_tags_9 },
    { 0x75f30884, SUC_lexicon_tags_6 },
    { 0x59730885, SUC_lexicon_tags_65 },
    { 0xfef30885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2d30888, SUC_lexicon_tags_8 },
    { 0xb2530889, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f30891, SUC_lexicon_tags_0 },
    { 0x22130892, SUC_lexicon_tags_25 },
    { 0xfdf30893, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x65130895, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad3089a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811308a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3308aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5308ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83308b0, SUC_lexicon_tags_49 },
    { 0x877308b1, SUC_lexicon_tags_68 },
    { 0x1b9308b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d308b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793308bb, SUC_lexicon_tags_15 },
    { 0xc1308bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515308c2, SUC_lexicon_tags_7 },
    { 0xf31308c3, SUC_lexicon_tags_23 },
    { 0xda1308c4, SUC_lexicon_tags_63 },
    { 0xbf1308c4, SUC_lexicon_tags_1 },
    { 0x2d5308c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477308cc, SUC_lexicon_tags_0 },
    { 0x205308cd, SUC_lexicon_tags_10 },
    { 0xd01308ce, SUC_lexicon_tags_48 },
    { 0xd5b308cf, SUC_lexicon_tags_21 },
    { 0x31b308ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b308d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8db308d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19308d9, SUC_lexicon_tags_3 },
    { 0x50b308da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdaf308dc, SUC_lexicon_tags_0 },
    { 0x1b5308dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409308e1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527308e5, SUC_lexicon_tags_37 },
    { 0xab9308e6, SUC_lexicon_tags_25 },
    { 0x355308e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f308ea, SUC_lexicon_tags_25 },
    { 0xff1308eb, SUC_lexicon_tags_9 },
    { 0x153308ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa15308ee, SUC_lexicon_tags_17 },
    { 0x9e5308ef, SUC_lexicon_tags_0 },
    { 0x823308ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7a7308f2, SUC_lexicon_tags_75 },
    { 0x9dd308f3, SUC_lexicon_tags_37 },
    { 0x169308f4, SUC_lexicon_tags_2 },
    { 0x3ed308f5, SUC_lexicon_tags_359 },
    { 0x783308f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x833308f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ef308fb, SUC_lexicon_tags_60 },
    { 0x987308fc, SUC_lexicon_tags_37 },
    { 0x109308fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d30903, SUC_lexicon_tags_7 },
    { 0x24930904, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f3090b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d3090e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf530918, SUC_lexicon_tags_106 },
    { 0xe930918, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x42f3091b, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x9593091d, SUC_lexicon_tags_12 },
    { 0xdcf3091e, SUC_lexicon_tags_25 },
    { 0xd6f3091f, SUC_lexicon_tags_34 },
    { 0x4ed3091f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42130924, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc673092c, SUC_lexicon_tags_23 },
    { 0x8b73092c, SUC_lexicon_tags_17 },
    { 0xc133092e, SUC_lexicon_tags_14 },
    { 0xaa93092f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80530931, SUC_lexicon_tags_9 },
    { 0x55b30932, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc930936, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac33093c, SUC_lexicon_tags_7 },
    { 0x2fd3093d, SUC_lexicon_tags_15 },
    { 0xc813093d, SUC_lexicon_tags_25 },
    { 0xf33093f, SUC_lexicon_tags_12 },
    { 0xc1d30940, SUC_lexicon_tags_37 },
    { 0xfe530941, SUC_lexicon_tags_37 },
    { 0xd4b30942, SUC_lexicon_tags_47 },
    { 0xc13093f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12330947, SUC_lexicon_tags_48 },
    { 0x81930948, SUC_lexicon_tags_14 },
    { 0x8c530949, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7cf3094b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc930952, SUC_lexicon_tags_11 },
    { 0xe8130952, SUC_lexicon_tags_33 },
    { 0xcf530954, SUC_lexicon_tags_66 },
    { 0x1c130953, SUC_lexicon_tags_14 },
    { 0x6a930952, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67d30958, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3213095b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead30960, SUC_lexicon_tags_25 },
    { 0x35330961, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf913096b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3f3096d, SUC_lexicon_tags_19 },
    { 0x44d3096d, SUC_lexicon_tags_1 },
    { 0x9633096d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x98730972, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfaf30974, SUC_lexicon_tags_15 },
    { 0x98b30975, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4730977, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa313097c, SUC_lexicon_tags_265 },
    { 0x8bf3097d, SUC_lexicon_tags_2 },
    { 0xba93097c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b30985, SUC_lexicon_tags_25 },
    { 0x36930986, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f30989, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b930994, SUC_lexicon_tags_6 },
    { 0x66f30995, SUC_lexicon_tags_29 },
    { 0xafb30996, SUC_lexicon_tags_68 },
    { 0xf2930996, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f93099f, SUC_lexicon_tags_37 },
    { 0x795309a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x635309a3, SUC_lexicon_tags_15 },
    { 0x809309a4, SUC_lexicon_tags_17 },
    { 0xc93309a4, SUC_lexicon_tags_10 },
    { 0x5a9309a6, SUC_lexicon_tags_1 },
    { 0xed9309a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ed309a9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x907309ab, SUC_lexicon_tags_29 },
    { 0x3c9309ab, SUC_lexicon_tags_4 },
    { 0xfe7309ad, SUC_lexicon_tags_193 },
    { 0x2ff309ad, SUC_lexicon_tags_25 },
    { 0x101309af, SUC_lexicon_tags_6 },
    { 0x1bf309b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x997309b2, SUC_lexicon_tags_23 },
    { 0x529309b3, SUC_lexicon_tags_17 },
    { 0x707309b4, SUC_lexicon_tags_17 },
    { 0x727309b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaad309b7, SUC_lexicon_tags_84 },
    { 0xb29309b8, SUC_lexicon_tags_226 },
    { 0x8af309b9, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337309be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d309cb, SUC_lexicon_tags_1 },
    { 0x283309cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b1309ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x247309d0, SUC_lexicon_tags_37 },
    { 0x2b9309d1, SUC_lexicon_tags_17 },
    { 0x415309d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f309d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c5309d9, SUC_lexicon_tags_63 },
    { 0x68b309da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229309df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd309f0, SUC_lexicon_tags_2 },
    { 0x3a5309f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x97b309f3, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295309f7, SUC_lexicon_tags_9 },
    { 0x8f7309f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc55309fa, SUC_lexicon_tags_37 },
    { 0x4db309fb, SUC_lexicon_tags_12 },
    { 0x9ad309fc, SUC_lexicon_tags_12 },
    { 0x675309fc, SUC_lexicon_tags_43 },
    { 0xf1d309fe, SUC_lexicon_tags_0 },
    { 0xc6d309fe, SUC_lexicon_tags_45 },
    { 0xf9f309fe, SUC_lexicon_tags_2 },
    { 0x38330a01, SUC_lexicon_tags_49 },
    { 0x96130a01, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x12d30a04, SUC_lexicon_tags_0 },
    { 0x47130a05, SUC_lexicon_tags_49 },
    { 0x9c730a06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8cb30a08, SUC_lexicon_tags_7 },
    { 0x6d30a09, SUC_lexicon_tags_7 },
    { 0xfa930a0a, SUC_lexicon_tags_25 },
    { 0x69f30a08, SUC_lexicon_tags_77 },
    { 0xaaf30a0c, SUC_lexicon_tags_7 },
    { 0x5b730a0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x38930a10, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92530a12, SUC_lexicon_tags_65 },
    { 0x5ab30a13, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7130a17, SUC_lexicon_tags_10 },
    { 0xf5530a18, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd4730a1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2730a1c, SUC_lexicon_tags_9 },
    { 0x21130a1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb30a20, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6af30a22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40730a24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d330a27, SUC_lexicon_tags_27 },
    { 0x93130a28, SUC_lexicon_tags_21 },
    { 0xdf730a29, SUC_lexicon_tags_42 },
    { 0xae330a2a, SUC_lexicon_tags_37 },
    { 0xebb30a2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb30a33, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61930a3d, SUC_lexicon_tags_18 },
    { 0xb8f30a3e, SUC_lexicon_tags_68 },
    { 0x16530a3e, SUC_lexicon_tags_31 },
    { 0x6cb30a40, SUC_lexicon_tags_27 },
    { 0x4b130a41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b30a4c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a330a52, SUC_lexicon_tags_7 },
    { 0x48330a53, SUC_lexicon_tags_13 },
    { 0x7b330a54, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62130a58, SUC_lexicon_tags_2 },
    { 0xad530a59, SUC_lexicon_tags_55 },
    { 0x1530a5a, SUC_lexicon_tags_11 },
    { 0x49330a5b, SUC_lexicon_tags_23 },
    { 0xd8b30a5b, SUC_lexicon_tags_47 },
    { 0xfb730a5c, SUC_lexicon_tags_1 },
    { 0x21d30a5b, SUC_lexicon_tags_15 },
    { 0xf6d30a5f, SUC_lexicon_tags_17 },
    { 0x41f30a5e, SUC_lexicon_tags_9 },
    { 0xc5730a61, SUC_lexicon_tags_2 },
    { 0xcf730a5d, SUC_lexicon_tags_37 },
    { 0xfc330a63, SUC_lexicon_tags_29 },
    { 0x77330a64, SUC_lexicon_tags_2 },
    { 0x62530a64, SUC_lexicon_tags_2 },
    { 0x2c530a66, SUC_lexicon_tags_37 },
    { 0x61330a64, SUC_lexicon_tags_61 },
    { 0x31b30a62, SUC_lexicon_tags_11 },
    { 0xdd330a59, SUC_lexicon_tags_951 },
    { 0x44130a69, SUC_lexicon_tags_33 },
    { 0xc1330a6b, SUC_lexicon_tags_113 },
    { 0x5f730a6b, SUC_lexicon_tags_27 },
    { 0x49930a60, SUC_lexicon_tags_15 },
    { 0x3e930a6e, SUC_lexicon_tags_34 },
    { 0x4f730a6f, SUC_lexicon_tags_11 },
    { 0xda330a67, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f30a76, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xe6930a78, SUC_lexicon_tags_15 },
    { 0xc4b30a79, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x56930a7b, SUC_lexicon_tags_16 },
    { 0x4b130a7b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc730a81, SUC_lexicon_tags_25 },
    { 0x98130a82, SUC_lexicon_tags_2 },
    { 0xe3530a82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x98b30a85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x930a8b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x83930a8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e330a90, SUC_lexicon_tags_38 },
    { 0x50130a91, SUC_lexicon_tags_34 },
    { 0x41130a92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bb30a94, SUC_lexicon_tags_6 },
    { 0x25130a95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b30a98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa330a9b, SUC_lexicon_tags_29 },
    { 0xe3b30a9b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbd130a9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4530aa0, SUC_lexicon_tags_87 },
    { 0x2b530aa1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd330aaa, SUC_lexicon_tags_68 },
    { 0x37930aab, SUC_lexicon_tags_23 },
    { 0x10f30aaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59930ab4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d30ab8, SUC_lexicon_tags_888 },
    { 0x2b930ab9, SUC_lexicon_tags_0 },
    { 0x71f30aba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d30abe, SUC_lexicon_tags_2 },
    { 0xabb30abf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3530ac9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46130acb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f30ace, SUC_lexicon_tags_40 },
    { 0x59330ace, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1330ad4, SUC_lexicon_tags_25 },
    { 0x95730ad4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf130ad7, SUC_lexicon_tags_0 },
    { 0xd9d30ad8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9130add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2330ae8, SUC_lexicon_tags_33 },
    { 0x76930ae9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0130af6, SUC_lexicon_tags_49 },
    { 0xdc530af7, SUC_lexicon_tags_2 },
    { 0xbc330af8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xad30afa, SUC_lexicon_tags_37 },
    { 0x51d30afb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df30afe, SUC_lexicon_tags_27 },
    { 0x10130aff, SUC_lexicon_tags_17 },
    { 0x4e130aff, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf30b05, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37930b09, SUC_lexicon_tags_7 },
    { 0x92530b0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b30b14, SUC_lexicon_tags_25 },
    { 0x6df30b15, SUC_lexicon_tags_11 },
    { 0xea330b16, SUC_lexicon_tags_37 },
    { 0x81130b16, SUC_lexicon_tags_30 },
    { 0x30b30b18, SUC_lexicon_tags_10 },
    { 0x62b30b19, SUC_lexicon_tags_37 },
    { 0xe7730b19, SUC_lexicon_tags_16 },
    { 0x1bd30b14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xef330b1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b30b21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f730b26, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37730b2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4930b2d, SUC_lexicon_tags_32 },
    { 0xa5d30b2e, SUC_lexicon_tags_224 },
    { 0x3f530b2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78b30b31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2730b33, SUC_lexicon_tags_17 },
    { 0x81530b34, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xec130b36, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d330b3a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xab30b3c, SUC_lexicon_tags_6 },
    { 0x28930b3c, SUC_lexicon_tags_33 },
    { 0x1cd30b3e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b30b41, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3af30b43, SUC_lexicon_tags_21 },
    { 0x20d30b44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2930b47, SUC_lexicon_tags_33 },
    { 0xac130b48, SUC_lexicon_tags_1 },
    { 0x8e730b49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17930b4b, SUC_lexicon_tags_17 },
    { 0x64b30b4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93730b50, SUC_lexicon_tags_37 },
    { 0xfed30b50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28d30b53, SUC_lexicon_tags_33 },
    { 0xb3130b54, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1930b59, SUC_lexicon_tags_17 },
    { 0x56b30b5a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4730b63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xde730b66, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f30b6a, SUC_lexicon_tags_116 },
    { 0x90d30b6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb330b6d, SUC_lexicon_tags_43 },
    { 0x9e130b6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd30b71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52130b74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8e130b76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35130b7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af30b7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7130b89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xebb30b8b, SUC_lexicon_tags_1 },
    { 0x8a530b8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec130b8e, SUC_lexicon_tags_14 },
    { 0xc3730b8f, SUC_lexicon_tags_68 },
    { 0xa9b30b90, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff30b94, SUC_lexicon_tags_21 },
    { 0x16d30b94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f30b98, SUC_lexicon_tags_61 },
    { 0xc3930b98, SUC_lexicon_tags_37 },
    { 0xbc130b9a, SUC_lexicon_tags_10 },
    { 0xf8130b9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24930baa, SUC_lexicon_tags_61 },
    { 0x92930bab, SUC_lexicon_tags_17 },
    { 0xaa330bac, SUC_lexicon_tags_16 },
    { 0xc6b30bac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x86730baf, SUC_lexicon_tags_44 },
    { 0xd7530bb0, SUC_lexicon_tags_63 },
    { 0xa1130bb1, SUC_lexicon_tags_99 },
    { 0xc0f30bb2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13930bb8, SUC_lexicon_tags_46 },
    { 0x8b130bb8, SUC_lexicon_tags_23 },
    { 0xb8730bba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43330bbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9730bc0, SUC_lexicon_tags_47 },
    { 0x58b30bc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c330bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb30bc8, SUC_lexicon_tags_19 },
    { 0x77130bc8, SUC_lexicon_tags_31 },
    { 0x4930bc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e530bcd, SUC_lexicon_tags_11 },
    { 0xa4330bce, SUC_lexicon_tags_28 },
    { 0x50930bcf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b30bd2, SUC_lexicon_tags_1 },
    { 0x9a330bd3, SUC_lexicon_tags_1 },
    { 0x6d530bd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4730bd9, SUC_lexicon_tags_12 },
    { 0x44130bda, SUC_lexicon_tags_17 },
    { 0xc5730bdb, SUC_lexicon_tags_25 },
    { 0xe1330bdc, SUC_lexicon_tags_12 },
    { 0x13130bdd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd130be0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x87930be2, SUC_lexicon_tags_25 },
    { 0x2df30be3, SUC_lexicon_tags_38 },
    { 0x6b30be2, SUC_lexicon_tags_60 },
    { 0x82130be3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d330beb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed30bee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe130bf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab30bf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3930bfa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53b30bfc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b730c00, SUC_lexicon_tags_1 },
    { 0xcdb30c01, SUC_lexicon_tags_17 },
    { 0x4b730c02, SUC_lexicon_tags_65 },
    { 0xed530c03, SUC_lexicon_tags_30 },
    { 0xead30c04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d130c07, SUC_lexicon_tags_15 },
    { 0x1ad30c07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd30c0b, SUC_lexicon_tags_17 },
    { 0xdd730c0c, SUC_lexicon_tags_12 },
    { 0xe3530c0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b30c10, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df30c1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4930c1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf30c24, SUC_lexicon_tags_37 },
    { 0xe1330c24, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b530c29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b530c2e, SUC_lexicon_tags_24 },
    { 0x9e330c2f, SUC_lexicon_tags_12 },
    { 0x44330c30, SUC_lexicon_tags_79 },
    { 0xbc530c31, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x27730c34, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa930c38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb30c3b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe8f30c3d, SUC_lexicon_tags_131 },
    { 0xd2130c3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0530c42, SUC_lexicon_tags_17 },
    { 0x46530c43, SUC_lexicon_tags_118 },
    { 0x27930c44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4530c4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f30c4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d130c52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c330c54, SUC_lexicon_tags_17 },
    { 0xf9930c55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3930c5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b30c5e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2530c62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f330c64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b30c67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadd30c69, SUC_lexicon_tags_23 },
    { 0xd6f30c6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80730c6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b30c72, SUC_lexicon_tags_39 },
    { 0x60530c73, SUC_lexicon_tags_29 },
    { 0xcff30c73, SUC_lexicon_tags_26 },
    { 0x6f30c75, SUC_lexicon_tags_6 },
    { 0x9d530c76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe730c79, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad730c7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdb930c80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8330c88, SUC_lexicon_tags_70 },
    { 0x6930c89, SUC_lexicon_tags_25 },
    { 0xd6730c88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7530c91, SUC_lexicon_tags_16 },
    { 0x5b330c92, SUC_lexicon_tags_25 },
    { 0x3ed30c91, SUC_lexicon_tags_37 },
    { 0xad730c91, SUC_lexicon_tags_72 },
    { 0xf9530c95, SUC_lexicon_tags_23 },
    { 0x5cd30c93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b30c99, SUC_lexicon_tags_6 },
    { 0x15730c9a, SUC_lexicon_tags_25 },
    { 0x7a330c9b, SUC_lexicon_tags_8 },
    { 0xb0930c9c, SUC_lexicon_tags_37 },
    { 0x7b730c9d, SUC_lexicon_tags_15 },
    { 0xd1730c99, SUC_lexicon_tags_17 },
    { 0xa0330c9c, SUC_lexicon_tags_37 },
    { 0x75330ca0, SUC_lexicon_tags_7 },
    { 0xdbf30ca1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef130ca3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93530ca6, SUC_lexicon_tags_37 },
    { 0x8b30ca6, SUC_lexicon_tags_17 },
    { 0xf8330ca8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee130caa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3d30cac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e930cae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6f730cb0, SUC_lexicon_tags_17 },
    { 0x3c130cb1, SUC_lexicon_tags_98 },
    { 0xabf30cb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e330cb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d530cba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b330cc3, SUC_lexicon_tags_37 },
    { 0xcc930cc4, SUC_lexicon_tags_60 },
    { 0x56330cc5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe130cc9, SUC_lexicon_tags_12 },
    { 0x73130cca, SUC_lexicon_tags_37 },
    { 0xb5330ccb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30330cd1, SUC_lexicon_tags_47 },
    { 0xa6730cd2, SUC_lexicon_tags_11 },
    { 0xd530cd3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab30cdb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e130cdf, SUC_lexicon_tags_19 },
    { 0x5cd30ce0, SUC_lexicon_tags_14 },
    { 0x16d30ce1, SUC_lexicon_tags_0 },
    { 0x9bb30cdf, SUC_lexicon_tags_60 },
    { 0x5dd30ce3, SUC_lexicon_tags_38 },
    { 0x4d730ce3, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37930cea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x89330ced, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a130cf4, SUC_lexicon_tags_16 },
    { 0xc4930cf5, SUC_lexicon_tags_35 },
    { 0x32930cf4, SUC_lexicon_tags_33 },
    { 0xa4730cf7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b130cfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45330cff, SUC_lexicon_tags_19 },
    { 0xd0b30d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd30d03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6130d06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x99330d08, SUC_lexicon_tags_1 },
    { 0xe9530d09, SUC_lexicon_tags_61 },
    { 0x9730d09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43730d0c, SUC_lexicon_tags_8 },
    { 0xfbf30d0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb930d10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef30d15, SUC_lexicon_tags_37 },
    { 0x14b30d16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d930d18, SUC_lexicon_tags_39 },
    { 0xeeb30d19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86b30d1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b530d20, SUC_lexicon_tags_37 },
    { 0x21330d21, SUC_lexicon_tags_2 },
    { 0x65f30d21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39730d28, SUC_lexicon_tags_2 },
    { 0x3930d28, SUC_lexicon_tags_11 },
    { 0x84730d2a, SUC_lexicon_tags_25 },
    { 0x3e730d2a, SUC_lexicon_tags_6 },
    { 0xf6330d2a, SUC_lexicon_tags_57 },
    { 0xedb30d2b, SUC_lexicon_tags_12 },
    { 0xd1f30d2a, SUC_lexicon_tags_17 },
    { 0x3c730d2f, SUC_lexicon_tags_49 },
    { 0x61b30d30, SUC_lexicon_tags_49 },
    { 0x85b30d31, SUC_lexicon_tags_37 },
    { 0xfaf30d30, SUC_lexicon_tags_10 },
    { 0xd3f30d33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98730d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde330d3d, SUC_lexicon_tags_60 },
    { 0x9ed30d3d, SUC_lexicon_tags_10 },
    { 0xe9d30d3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b30d44, SUC_lexicon_tags_21 },
    { 0xc9b30d45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41930d4b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3dd30d4d, SUC_lexicon_tags_12 },
    { 0xe2730d4d, SUC_lexicon_tags_15 },
    { 0x1c730d4f, SUC_lexicon_tags_12 },
    { 0x56730d4d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc130d5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54b30d5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b30d5f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c930d64, SUC_lexicon_tags_66 },
    { 0x17d30d64, SUC_lexicon_tags_43 },
    { 0x95730d66, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf930d6b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac530d6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b30d70, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1fd30d72, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa130d79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce130d7b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf30d86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f30d8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8330d8e, SUC_lexicon_tags_8 },
    { 0x2bb30d8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f30d93, SUC_lexicon_tags_2 },
    { 0x58d30d93, SUC_lexicon_tags_25 },
    { 0xd530d95, SUC_lexicon_tags_77 },
    { 0x13330d94, SUC_lexicon_tags_37 },
    { 0xd7730d94, SUC_lexicon_tags_17 },
    { 0x11930d94, SUC_lexicon_tags_39 },
    { 0xcfb30d98, SUC_lexicon_tags_6 },
    { 0xa4b30d98, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb30d9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8b330da1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70130da9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa6330dab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a730dae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d730db3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd130db6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d730dbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b30dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24730dc3, SUC_lexicon_tags_11 },
    { 0x92730dc4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a330dc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1b30dc8, SUC_lexicon_tags_37 },
    { 0xde330dc8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda730dcd, SUC_lexicon_tags_15 },
    { 0xc8b30dce, SUC_lexicon_tags_10 },
    { 0x69130dcf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x20f30dd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e530dd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b30dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4eb30dd9, SUC_lexicon_tags_11 },
    { 0xa6530dda, SUC_lexicon_tags_37 },
    { 0x7ab30ddb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42130ddf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfb330de1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcff30de3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9330de6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1730dec, SUC_lexicon_tags_27 },
    { 0x15130ded, SUC_lexicon_tags_17 },
    { 0x1c530dee, SUC_lexicon_tags_32 },
    { 0x10d30def, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f930df4, SUC_lexicon_tags_45 },
    { 0x1eb30df4, SUC_lexicon_tags_64 },
    { 0x4e530df6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c330dfc, SUC_lexicon_tags_12 },
    { 0x80730dfd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc730e00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83130e05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x98b30e07, SUC_lexicon_tags_29 },
    { 0xaf130e08, SUC_lexicon_tags_87 },
    { 0x41330e09, SUC_lexicon_tags_11 },
    { 0x59f30e09, SUC_lexicon_tags_25 },
    { 0x27b30e0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x96130e0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f130e10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4130e13, SUC_lexicon_tags_19 },
    { 0x56530e14, SUC_lexicon_tags_12 },
    { 0xf0330e15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89b30e17, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf8b30e19, SUC_lexicon_tags_1 },
    { 0xc4130e1a, SUC_lexicon_tags_17 },
    { 0x97130e19, SUC_lexicon_tags_8 },
    { 0x8730e1c, SUC_lexicon_tags_12 },
    { 0x7f930e1c, SUC_lexicon_tags_29 },
    { 0x5f330e1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7130e24, SUC_lexicon_tags_47 },
    { 0xa8930e24, SUC_lexicon_tags_15 },
    { 0xe9d30e26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb30e2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d30e30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b30e33, SUC_lexicon_tags_65 },
    { 0x65730e34, SUC_lexicon_tags_37 },
    { 0x83130e33, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1730e39, SUC_lexicon_tags_60 },
    { 0x8130e3a, SUC_lexicon_tags_12 },
    { 0x70d30e39, SUC_lexicon_tags_45 },
    { 0x8b930e3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9130e40, SUC_lexicon_tags_74 },
    { 0x72b30e41, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71130e47, SUC_lexicon_tags_49 },
    { 0x2ff30e47, SUC_lexicon_tags_17 },
    { 0xbed30e48, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xcd730e4b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x51d30e4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1730e4f, SUC_lexicon_tags_0 },
    { 0xc9530e50, SUC_lexicon_tags_12 },
    { 0xa3930e50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b30e54, SUC_lexicon_tags_33 },
    { 0xf6f30e55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d30e63, SUC_lexicon_tags_0 },
    { 0x53d30e64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1c930e66, SUC_lexicon_tags_23 },
    { 0x4cb30e67, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc530e69, SUC_lexicon_tags_30 },
    { 0x32d30e69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36730e71, SUC_lexicon_tags_3 },
    { 0x1130e72, SUC_lexicon_tags_10 },
    { 0xa3f30e72, SUC_lexicon_tags_66 },
    { 0x8130e74, SUC_lexicon_tags_26 },
    { 0x64330e71, SUC_lexicon_tags_17 },
    { 0x1e130e76, SUC_lexicon_tags_15 },
    { 0x67b30e76, SUC_lexicon_tags_11 },
    { 0x4c930e78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40130e7a, SUC_lexicon_tags_14 },
    { 0x7f930e7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a330e7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82730e83, SUC_lexicon_tags_12 },
    { 0xd330e84, SUC_lexicon_tags_12 },
    { 0x42d30e85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef30e88, SUC_lexicon_tags_31 },
    { 0x3d30e89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5930e8b, SUC_lexicon_tags_11 },
    { 0x76d30e8c, SUC_lexicon_tags_11 },
    { 0x16530e8d, SUC_lexicon_tags_1 },
    { 0x56130e8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd930e90, SUC_lexicon_tags_2 },
    { 0x53930e90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad30e9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1330e9f, SUC_lexicon_tags_5 },
    { 0xdcf30ea0, SUC_lexicon_tags_7 },
    { 0x8cd30e9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a530ea5, SUC_lexicon_tags_0 },
    { 0x64930ea6, SUC_lexicon_tags_46 },
    { 0xe5930ea7, SUC_lexicon_tags_37 },
    { 0x6f130ea8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b30eab, SUC_lexicon_tags_16 },
    { 0xb0730eac, SUC_lexicon_tags_4 },
    { 0xff330ead, SUC_lexicon_tags_60 },
    { 0x8cb30eac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5330eb5, SUC_lexicon_tags_49 },
    { 0xafd30eb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28130eb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24330ebb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab30ec0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e730ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed30ec7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8d130ec9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x49130ecb, SUC_lexicon_tags_1 },
    { 0xa7f30ecc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2130ed1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76130ed4, SUC_lexicon_tags_37 },
    { 0x4af30ed4, SUC_lexicon_tags_33 },
    { 0x6db30ed6, SUC_lexicon_tags_15 },
    { 0xb2730ed4, SUC_lexicon_tags_23 },
    { 0xca530ed6, SUC_lexicon_tags_16 },
    { 0x9c730ed5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b30edc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69f30ede, SUC_lexicon_tags_63 },
    { 0xb9530edf, SUC_lexicon_tags_10 },
    { 0x78730ee0, SUC_lexicon_tags_17 },
    { 0xc9d30ee1, SUC_lexicon_tags_32 },
    { 0x2f330ee2, SUC_lexicon_tags_49 },
    { 0xa3330ee3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f930ee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49130ee9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c930ef0, SUC_lexicon_tags_38 },
    { 0x130ef1, SUC_lexicon_tags_14 },
    { 0x3b730ef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b30ef5, SUC_lexicon_tags_30 },
    { 0xfe530ef5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff530ef9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf30efc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa130f00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2330f02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6130f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71d30f13, SUC_lexicon_tags_6 },
    { 0x17930f13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c730f17, SUC_lexicon_tags_12 },
    { 0x28b30f18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5730f1e, SUC_lexicon_tags_23 },
    { 0xae730f1f, SUC_lexicon_tags_7 },
    { 0xdc930f20, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x48330f23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b30f2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a730f30, SUC_lexicon_tags_62 },
    { 0xf5f30f30, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed30f34, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd330f38, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49530f3e, SUC_lexicon_tags_4 },
    { 0xe3d30f3f, SUC_lexicon_tags_65 },
    { 0xb2d30f40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3930f43, SUC_lexicon_tags_1 },
    { 0x76b30f44, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb30f49, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x79d30f4b, SUC_lexicon_tags_25 },
    { 0xaab30f4c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50330f51, SUC_lexicon_tags_11 },
    { 0x48530f51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef30f55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3130f58, SUC_lexicon_tags_28 },
    { 0xb730f58, SUC_lexicon_tags_34 },
    { 0x8f530f5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f30f6b, SUC_lexicon_tags_31 },
    { 0x23130f6c, SUC_lexicon_tags_63 },
    { 0xaa730f6c, SUC_lexicon_tags_87 },
    { 0x10730f6e, SUC_lexicon_tags_15 },
    { 0xf8330f6e, SUC_lexicon_tags_33 },
    { 0xe8b30f70, SUC_lexicon_tags_0 },
    { 0x3cd30f71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27530f76, SUC_lexicon_tags_12 },
    { 0x35930f77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e530f79, SUC_lexicon_tags_2 },
    { 0x7f530f79, SUC_lexicon_tags_2 },
    { 0xbf330f7b, SUC_lexicon_tags_17 },
    { 0x61b30f7b, SUC_lexicon_tags_43 },
    { 0xb7930f7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51730f84, SUC_lexicon_tags_7 },
    { 0xfe330f84, SUC_lexicon_tags_105 },
    { 0xa2b30f86, SUC_lexicon_tags_37 },
    { 0x68930f87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f330f8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x73d30f8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3730f95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd30f9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d30fa6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d30fab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf30fae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c130fb1, SUC_lexicon_tags_31 },
    { 0x80b30fb2, SUC_lexicon_tags_17 },
    { 0x79530fb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x88b30fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa130fb8, SUC_lexicon_tags_27 },
    { 0x8e330fb9, SUC_lexicon_tags_10 },
    { 0x71f30fb8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x41330fbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f30fbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8130fc2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d30fc7, SUC_lexicon_tags_19 },
    { 0x2d30fc7, SUC_lexicon_tags_12 },
    { 0xcfd30fc9, SUC_lexicon_tags_45 },
    { 0xa0930fc7, SUC_lexicon_tags_7 },
    { 0xc7930fcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd30fce, SUC_lexicon_tags_38 },
    { 0x81930fce, SUC_lexicon_tags_17 },
    { 0x71530fcf, SUC_lexicon_tags_1 },
    { 0xe9530fd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28730fd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7130fd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e330fd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d30fdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4530fe0, SUC_lexicon_tags_12 },
    { 0x91530fe1, SUC_lexicon_tags_25 },
    { 0x5df30fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a130fe4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b30fe7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd30feb, SUC_lexicon_tags_33 },
    { 0x7d30feb, SUC_lexicon_tags_46 },
    { 0xd5330fec, SUC_lexicon_tags_45 },
    { 0x90130fed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f530ff0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8730ff4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac530ff7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x59d30ff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31531001, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86131003, SUC_lexicon_tags_11 },
    { 0x3731004, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcfd31006, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x85b31008, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b73100d, SUC_lexicon_tags_38 },
    { 0xb853100e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e931012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a731015, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6b31017, SUC_lexicon_tags_38 },
    { 0x131018, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4153101b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53f3101d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d31024, SUC_lexicon_tags_11 },
    { 0x5b731025, SUC_lexicon_tags_17 },
    { 0x73531025, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf131029, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xae13102c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f3102f, SUC_lexicon_tags_55 },
    { 0x7e33102f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7d31032, SUC_lexicon_tags_138 },
    { 0xfcb31033, SUC_lexicon_tags_37 },
    { 0x77931034, SUC_lexicon_tags_12 },
    { 0xa1531034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf213103b, SUC_lexicon_tags_2 },
    { 0x3813103c, SUC_lexicon_tags_2 },
    { 0xe7b3103d, SUC_lexicon_tags_37 },
    { 0x413103d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b31041, SUC_lexicon_tags_25 },
    { 0x20331042, SUC_lexicon_tags_47 },
    { 0xecf31043, SUC_lexicon_tags_23 },
    { 0xe1731042, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b731047, SUC_lexicon_tags_2 },
    { 0x93f31048, SUC_lexicon_tags_44 },
    { 0x75d31047, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7693104d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1353104f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b131053, SUC_lexicon_tags_12 },
    { 0xad731054, SUC_lexicon_tags_6 },
    { 0x34131053, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x83731057, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe93105b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a13105f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f31063, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb131068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2993106b, SUC_lexicon_tags_35 },
    { 0xf893106c, SUC_lexicon_tags_63 },
    { 0x6733106c, SUC_lexicon_tags_47 },
    { 0x8133106d, SUC_lexicon_tags_45 },
    { 0xba33106f, SUC_lexicon_tags_19 },
    { 0xf131070, SUC_lexicon_tags_87 },
    { 0x84b31070, SUC_lexicon_tags_37 },
    { 0x21131072, SUC_lexicon_tags_81 },
    { 0xbb331073, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd31077, SUC_lexicon_tags_7 },
    { 0x731078, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf273107a, SUC_lexicon_tags_27 },
    { 0x7af3107b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d3107e, SUC_lexicon_tags_14 },
    { 0xac13107e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1331082, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44d31084, SUC_lexicon_tags_11 },
    { 0x90131084, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3108d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8193108f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe2531091, SUC_lexicon_tags_49 },
    { 0xb2f31092, SUC_lexicon_tags_32 },
    { 0x19931093, SUC_lexicon_tags_63 },
    { 0x4a731092, SUC_lexicon_tags_1 },
    { 0xb2731094, SUC_lexicon_tags_30 },
    { 0xf3131092, SUC_lexicon_tags_107 },
    { 0xd2f31093, SUC_lexicon_tags_0 },
    { 0x4a531093, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59310a2, SUC_lexicon_tags_129 },
    { 0x55310a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5310a7, SUC_lexicon_tags_0 },
    { 0xeff310a8, SUC_lexicon_tags_10 },
    { 0xd27310a9, SUC_lexicon_tags_13 },
    { 0x9d1310a8, SUC_lexicon_tags_7 },
    { 0x6d3310ab, SUC_lexicon_tags_7 },
    { 0x953310a9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x547310ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b310b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd07310b7, SUC_lexicon_tags_106 },
    { 0xd87310b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94b310ba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af310c4, SUC_lexicon_tags_47 },
    { 0xbd7310c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87d310c7, SUC_lexicon_tags_14 },
    { 0xcf3310c8, SUC_lexicon_tags_2 },
    { 0x25f310c9, SUC_lexicon_tags_7 },
    { 0x347310c9, SUC_lexicon_tags_11 },
    { 0x72f310cb, SUC_lexicon_tags_9 },
    { 0xbd1310cc, SUC_lexicon_tags_15 },
    { 0xe67310c7, SUC_lexicon_tags_19 },
    { 0x49b310ce, SUC_lexicon_tags_19 },
    { 0xc53310cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe71310d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d310d4, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f310db, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa7f310dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb310e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x467310e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x459310e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef310e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd310f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf93310f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9310f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x183310f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41731100, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x83131102, SUC_lexicon_tags_1 },
    { 0x54531102, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda931109, SUC_lexicon_tags_25 },
    { 0xfa53110a, SUC_lexicon_tags_1 },
    { 0x3d31109, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7133110f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d31114, SUC_lexicon_tags_14 },
    { 0x29531115, SUC_lexicon_tags_37 },
    { 0x1ef31115, SUC_lexicon_tags_37 },
    { 0x6c131117, SUC_lexicon_tags_12 },
    { 0xe3331118, SUC_lexicon_tags_12 },
    { 0xfe131119, SUC_lexicon_tags_44 },
    { 0x53b31118, SUC_lexicon_tags_47 },
    { 0x22d3111b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65b3111d, SUC_lexicon_tags_69 },
    { 0x33d3111e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31b31120, SUC_lexicon_tags_13 },
    { 0x5f931121, SUC_lexicon_tags_27 },
    { 0x2db31120, SUC_lexicon_tags_28 },
    { 0x5c531121, SUC_lexicon_tags_44 },
    { 0x80d31124, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x39131126, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b33112e, SUC_lexicon_tags_2 },
    { 0xd233112f, SUC_lexicon_tags_37 },
    { 0x2c331130, SUC_lexicon_tags_26 },
    { 0xc4331131, SUC_lexicon_tags_60 },
    { 0xe3f31132, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b31138, SUC_lexicon_tags_37 },
    { 0xbf131139, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b73113b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb3113e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a331144, SUC_lexicon_tags_65 },
    { 0xc0131144, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f31148, SUC_lexicon_tags_7 },
    { 0x9d731148, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x213114c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f3114f, SUC_lexicon_tags_45 },
    { 0x7413114f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d731152, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6c531154, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x47331156, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe053115a, SUC_lexicon_tags_25 },
    { 0xfe13115b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf3115f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5331161, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3db31163, SUC_lexicon_tags_25 },
    { 0xcc931163, SUC_lexicon_tags_25 },
    { 0xc1731165, SUC_lexicon_tags_116 },
    { 0x73d31164, SUC_lexicon_tags_26 },
    { 0x3c331167, SUC_lexicon_tags_31 },
    { 0x75b31168, SUC_lexicon_tags_17 },
    { 0xa8931168, SUC_lexicon_tags_23 },
    { 0xcb531166, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9513116c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba531170, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b31177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd93117a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c93117c, SUC_lexicon_tags_39 },
    { 0x3ad3117d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3531180, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2531184, SUC_lexicon_tags_33 },
    { 0xa8b31185, SUC_lexicon_tags_38 },
    { 0xbe131186, SUC_lexicon_tags_4 },
    { 0x75931187, SUC_lexicon_tags_15 },
    { 0xf31185, SUC_lexicon_tags_92 },
    { 0x43531189, SUC_lexicon_tags_11 },
    { 0x9733118a, SUC_lexicon_tags_49 },
    { 0x53731186, SUC_lexicon_tags_60 },
    { 0xa4b3118c, SUC_lexicon_tags_25 },
    { 0xde93118d, SUC_lexicon_tags_55 },
    { 0x23f3118e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a131190, SUC_lexicon_tags_37 },
    { 0xf0d31191, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7131197, SUC_lexicon_tags_11 },
    { 0x9eb31197, SUC_lexicon_tags_34 },
    { 0xd8731199, SUC_lexicon_tags_47 },
    { 0x73931198, SUC_lexicon_tags_25 },
    { 0xfc53119b, SUC_lexicon_tags_61 },
    { 0x82331198, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5213119f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f311a2, SUC_lexicon_tags_31 },
    { 0x5fd311a3, SUC_lexicon_tags_43 },
    { 0xcc3311a2, SUC_lexicon_tags_19 },
    { 0x2e7311a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x583311a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827311ad, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7311b2, SUC_lexicon_tags_25 },
    { 0x315311b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5e5311b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3311bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd311c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f311cd, SUC_lexicon_tags_29 },
    { 0xa11311cd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589311d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9311dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd311df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6f5311e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ef311e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1311e6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f311e9, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577311ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89311f2, SUC_lexicon_tags_20 },
    { 0xdd7311f3, SUC_lexicon_tags_23 },
    { 0xb15311f4, SUC_lexicon_tags_28 },
    { 0xe79311f4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43311f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559311fe, SUC_lexicon_tags_7 },
    { 0x8cf311ff, SUC_lexicon_tags_4 },
    { 0x7a331200, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1731209, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb13120c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6931210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79331212, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f531215, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x47131218, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd933121a, SUC_lexicon_tags_2 },
    { 0xf13121b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55331220, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb931223, SUC_lexicon_tags_12 },
    { 0xe9331223, SUC_lexicon_tags_49 },
    { 0xb6731225, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d31229, SUC_lexicon_tags_15 },
    { 0x52131229, SUC_lexicon_tags_17 },
    { 0x7d93122b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a731231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bd31233, SUC_lexicon_tags_7 },
    { 0x63b31234, SUC_lexicon_tags_28 },
    { 0xe731233, SUC_lexicon_tags_17 },
    { 0xa3331236, SUC_lexicon_tags_49 },
    { 0x26731236, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b13123a, SUC_lexicon_tags_65 },
    { 0x5bf3123b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb653123f, SUC_lexicon_tags_11 },
    { 0x3f131240, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb3124a, SUC_lexicon_tags_11 },
    { 0x8593124b, SUC_lexicon_tags_37 },
    { 0xa1b3124a, SUC_lexicon_tags_22 },
    { 0xa913124d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f31250, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x71d31252, SUC_lexicon_tags_7 },
    { 0xe9731252, SUC_lexicon_tags_25 },
    { 0xc6931254, SUC_lexicon_tags_2 },
    { 0xa5731254, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x70731258, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7673125a, SUC_lexicon_tags_61 },
    { 0x9f13125b, SUC_lexicon_tags_12 },
    { 0xba73125a, SUC_lexicon_tags_2 },
    { 0xc993125d, SUC_lexicon_tags_57 },
    { 0xfa33125e, SUC_lexicon_tags_9 },
    { 0xe293125f, SUC_lexicon_tags_25 },
    { 0xad31260, SUC_lexicon_tags_28 },
    { 0x95931261, SUC_lexicon_tags_23 },
    { 0xa8f31261, SUC_lexicon_tags_23 },
    { 0xb4131263, SUC_lexicon_tags_42 },
    { 0xe9731261, SUC_lexicon_tags_4 },
    { 0x35131263, SUC_lexicon_tags_71 },
    { 0xd6731264, SUC_lexicon_tags_65 },
    { 0xa7b31264, SUC_lexicon_tags_87 },
    { 0xc8b31262, SUC_lexicon_tags_31 },
    { 0x91531260, SUC_lexicon_tags_15 },
    { 0x7ff31267, SUC_lexicon_tags_4 },
    { 0xb7b3126b, SUC_lexicon_tags_68 },
    { 0x9233126c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb31272, SUC_lexicon_tags_7 },
    { 0x2c731273, SUC_lexicon_tags_37 },
    { 0x42931274, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd31277, SUC_lexicon_tags_43 },
    { 0x2b731278, SUC_lexicon_tags_11 },
    { 0x14331278, SUC_lexicon_tags_55 },
    { 0x10731278, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb3127e, SUC_lexicon_tags_25 },
    { 0x9453127f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf1b31281, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64131285, SUC_lexicon_tags_60 },
    { 0x9fd31286, SUC_lexicon_tags_17 },
    { 0x89331287, SUC_lexicon_tags_25 },
    { 0x72331285, SUC_lexicon_tags_25 },
    { 0x1d31289, SUC_lexicon_tags_37 },
    { 0x3e131289, SUC_lexicon_tags_37 },
    { 0x9ab3128b, SUC_lexicon_tags_60 },
    { 0xc2f3128c, SUC_lexicon_tags_11 },
    { 0xbab3128c, SUC_lexicon_tags_1 },
    { 0xd373128c, SUC_lexicon_tags_30 },
    { 0x7053128f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61b31291, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61131295, SUC_lexicon_tags_27 },
    { 0xbef31296, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62131298, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f13129e, SUC_lexicon_tags_37 },
    { 0x6293129e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7312a4, SUC_lexicon_tags_63 },
    { 0xe9b312a4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xbd5312a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x891312a9, SUC_lexicon_tags_15 },
    { 0xa0b312aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a7312ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1312af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3312bc, SUC_lexicon_tags_1 },
    { 0x743312bd, SUC_lexicon_tags_45 },
    { 0x47d312be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f312c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x363312c4, SUC_lexicon_tags_17 },
    { 0x6b1312c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb312c8, SUC_lexicon_tags_25 },
    { 0x7f9312c9, SUC_lexicon_tags_45 },
    { 0x459312c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe75312cc, SUC_lexicon_tags_0 },
    { 0xd5312cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f312d0, SUC_lexicon_tags_35 },
    { 0x9b312d1, SUC_lexicon_tags_25 },
    { 0x5a3312d2, SUC_lexicon_tags_43 },
    { 0xbd3312d3, SUC_lexicon_tags_26 },
    { 0x589312d4, SUC_lexicon_tags_29 },
    { 0xdd312d5, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x215312d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11312db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x853312dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925312e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x601312e4, SUC_lexicon_tags_23 },
    { 0xbff312e5, SUC_lexicon_tags_2 },
    { 0xccd312e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d312ea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d312ed, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xee3312ef, SUC_lexicon_tags_31 },
    { 0x763312f0, SUC_lexicon_tags_30 },
    { 0x92f312f1, SUC_lexicon_tags_15 },
    { 0x3dd312f1, SUC_lexicon_tags_105 },
    { 0x3d3312f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33312f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613312fb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef312ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe5331301, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6131306, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f3130c, SUC_lexicon_tags_27 },
    { 0x2453130d, SUC_lexicon_tags_26 },
    { 0xee33130e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8731310, SUC_lexicon_tags_26 },
    { 0x39b31311, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb31315, SUC_lexicon_tags_33 },
    { 0xa531316, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfdb31318, SUC_lexicon_tags_4 },
    { 0x96b31319, SUC_lexicon_tags_14 },
    { 0x5073131a, SUC_lexicon_tags_11 },
    { 0x53b3131a, SUC_lexicon_tags_0 },
    { 0x20d31318, SUC_lexicon_tags_31 },
    { 0x5e931319, SUC_lexicon_tags_12 },
    { 0xa513131b, SUC_lexicon_tags_31 },
    { 0x68b31318, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39331323, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb31326, SUC_lexicon_tags_15 },
    { 0x8eb31327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d3132a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f31330, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x80531333, SUC_lexicon_tags_37 },
    { 0xcfd31333, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd31337, SUC_lexicon_tags_65 },
    { 0xf0f31338, SUC_lexicon_tags_28 },
    { 0xf1531338, SUC_lexicon_tags_34 },
    { 0x18d3133a, SUC_lexicon_tags_16 },
    { 0xd8d3133b, SUC_lexicon_tags_1 },
    { 0x3eb3133b, SUC_lexicon_tags_12 },
    { 0x8cb3133d, SUC_lexicon_tags_30 },
    { 0xb673133e, SUC_lexicon_tags_60 },
    { 0xeeb31339, SUC_lexicon_tags_7 },
    { 0x50731340, SUC_lexicon_tags_32 },
    { 0x49731341, SUC_lexicon_tags_16 },
    { 0xa1931340, SUC_lexicon_tags_12 },
    { 0xe1531343, SUC_lexicon_tags_15 },
    { 0xef131343, SUC_lexicon_tags_43 },
    { 0x9df31345, SUC_lexicon_tags_49 },
    { 0x70931346, SUC_lexicon_tags_14 },
    { 0x66531347, SUC_lexicon_tags_37 },
    { 0xa8331341, SUC_lexicon_tags_45 },
    { 0xca331347, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe033134b, SUC_lexicon_tags_44 },
    { 0x6d13134c, SUC_lexicon_tags_11 },
    { 0x1653134d, SUC_lexicon_tags_8 },
    { 0x91b3134e, SUC_lexicon_tags_54 },
    { 0x4453134f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d31353, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3d331355, SUC_lexicon_tags_12 },
    { 0x90131356, SUC_lexicon_tags_26 },
    { 0xa3d31356, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x49d31359, SUC_lexicon_tags_7 },
    { 0x5eb3135a, SUC_lexicon_tags_68 },
    { 0xcb3135b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d3135e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47531362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b31367, SUC_lexicon_tags_7 },
    { 0x87331368, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfc73136a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x27b3136c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xddf3136e, SUC_lexicon_tags_17 },
    { 0x70d3136e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88731373, SUC_lexicon_tags_11 },
    { 0xf1d31374, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8133137c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec131381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d31385, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f3138a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x66b3138c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xab33138f, SUC_lexicon_tags_7 },
    { 0xd1b31390, SUC_lexicon_tags_23 },
    { 0xaaf3138f, SUC_lexicon_tags_37 },
    { 0x50b3138f, SUC_lexicon_tags_8 },
    { 0x18931393, SUC_lexicon_tags_2 },
    { 0x59131394, SUC_lexicon_tags_6 },
    { 0x38931394, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f31399, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d33139b, SUC_lexicon_tags_49 },
    { 0xa5f3139c, SUC_lexicon_tags_1 },
    { 0x4e73139b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x61d3139f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d313a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x21f313ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59313ae, SUC_lexicon_tags_6 },
    { 0x621313ae, SUC_lexicon_tags_47 },
    { 0x1c3313b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f313b2, SUC_lexicon_tags_25 },
    { 0x20d313b2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x293313b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693313ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91313c1, SUC_lexicon_tags_82 },
    { 0x7ff313c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe3f313c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5313c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b313cc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed313d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5313d2, SUC_lexicon_tags_2 },
    { 0x91313d3, SUC_lexicon_tags_43 },
    { 0xa27313d3, SUC_lexicon_tags_47 },
    { 0x43d313d5, SUC_lexicon_tags_23 },
    { 0xf3d313d6, SUC_lexicon_tags_1 },
    { 0x919313d7, SUC_lexicon_tags_12 },
    { 0x907313d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad313dc, SUC_lexicon_tags_13 },
    { 0x73d313dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x189313e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5d313e2, SUC_lexicon_tags_26 },
    { 0x417313e3, SUC_lexicon_tags_37 },
    { 0x537313e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x593313e7, SUC_lexicon_tags_21 },
    { 0xf1b313e8, SUC_lexicon_tags_39 },
    { 0x69313e9, SUC_lexicon_tags_17 },
    { 0xaed313ea, SUC_lexicon_tags_12 },
    { 0x4d1313eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb313f0, SUC_lexicon_tags_7 },
    { 0xa79313f1, SUC_lexicon_tags_48 },
    { 0x273313f2, SUC_lexicon_tags_37 },
    { 0x1af313f3, SUC_lexicon_tags_15 },
    { 0xb2f313f3, SUC_lexicon_tags_33 },
    { 0xad1313f4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1d3313f7, SUC_lexicon_tags_17 },
    { 0xa9313f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6eb313fa, SUC_lexicon_tags_41 },
    { 0xf9b313fa, SUC_lexicon_tags_45 },
    { 0xff3313fc, SUC_lexicon_tags_17 },
    { 0x459313fd, SUC_lexicon_tags_57 },
    { 0x655313fe, SUC_lexicon_tags_15 },
    { 0x4e1313fd, SUC_lexicon_tags_47 },
    { 0xdd3313fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f31407, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe593140d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1453140f, SUC_lexicon_tags_8 },
    { 0x56d3140f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecf31412, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56931414, SUC_lexicon_tags_15 },
    { 0x6c131414, SUC_lexicon_tags_107 },
    { 0x21331415, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f131419, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9133141e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9131424, SUC_lexicon_tags_82 },
    { 0x86731425, SUC_lexicon_tags_13 },
    { 0xaeb31426, SUC_lexicon_tags_12 },
    { 0xf8131424, SUC_lexicon_tags_68 },
    { 0x9b331424, SUC_lexicon_tags_17 },
    { 0xdc731429, SUC_lexicon_tags_56 },
    { 0x8e53142a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233142e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7331430, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f31436, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d3143c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8133143e, SUC_lexicon_tags_25 },
    { 0x7933143f, SUC_lexicon_tags_39 },
    { 0x1f331440, SUC_lexicon_tags_63 },
    { 0x1a931441, SUC_lexicon_tags_11 },
    { 0xba53143e, SUC_lexicon_tags_25 },
    { 0x77731443, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed31447, SUC_lexicon_tags_38 },
    { 0x5d531448, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d3144c, SUC_lexicon_tags_11 },
    { 0xa693144d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfdd3144f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea131451, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76331454, SUC_lexicon_tags_15 },
    { 0x8131455, SUC_lexicon_tags_2 },
    { 0x52331454, SUC_lexicon_tags_37 },
    { 0xf1931457, SUC_lexicon_tags_27 },
    { 0x70731455, SUC_lexicon_tags_15 },
    { 0x41d31459, SUC_lexicon_tags_15 },
    { 0x94331454, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd3145f, SUC_lexicon_tags_31 },
    { 0xf31460, SUC_lexicon_tags_17 },
    { 0x35731461, SUC_lexicon_tags_2 },
    { 0x2f331462, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49331466, SUC_lexicon_tags_47 },
    { 0xb7f31467, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b33146a, SUC_lexicon_tags_19 },
    { 0xe5d3146a, SUC_lexicon_tags_37 },
    { 0xa833146a, SUC_lexicon_tags_51 },
    { 0x22d3146d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c53146f, SUC_lexicon_tags_23 },
    { 0x15131470, SUC_lexicon_tags_15 },
    { 0x4db31470, SUC_lexicon_tags_1 },
    { 0xa0731472, SUC_lexicon_tags_1 },
    { 0x8531470, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b31478, SUC_lexicon_tags_6 },
    { 0xa8531478, SUC_lexicon_tags_12 },
    { 0xa673147a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d3147e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcd31480, SUC_lexicon_tags_7 },
    { 0xeb931480, SUC_lexicon_tags_46 },
    { 0x58331480, SUC_lexicon_tags_84 },
    { 0xf5f31483, SUC_lexicon_tags_15 },
    { 0x61331480, SUC_lexicon_tags_17 },
    { 0x60b31485, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x26331487, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x2413148a, SUC_lexicon_tags_12 },
    { 0x4a33148b, SUC_lexicon_tags_3 },
    { 0x6913148c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb673148e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef31494, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd31497, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0531499, SUC_lexicon_tags_2 },
    { 0x7af3149a, SUC_lexicon_tags_11 },
    { 0x2993149b, SUC_lexicon_tags_3 },
    { 0x4473149c, SUC_lexicon_tags_2 },
    { 0x4ed3149d, SUC_lexicon_tags_15 },
    { 0xd133149e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f314a1, SUC_lexicon_tags_37 },
    { 0xa9f314a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d314a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x673314a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2d314a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf314ae, SUC_lexicon_tags_0 },
    { 0xbdf314af, SUC_lexicon_tags_17 },
    { 0x7c1314b0, SUC_lexicon_tags_25 },
    { 0x31f314b0, SUC_lexicon_tags_17 },
    { 0x5ef314b0, SUC_lexicon_tags_37 },
    { 0x4ab314b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b314b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x139314b9, SUC_lexicon_tags_27 },
    { 0xd5314ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60d314bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1314bf, SUC_lexicon_tags_60 },
    { 0xb73314bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb314c3, SUC_lexicon_tags_2 },
    { 0xc21314c3, SUC_lexicon_tags_17 },
    { 0xb0b314c5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b314ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x883314cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f314d0, SUC_lexicon_tags_32 },
    { 0xefd314d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861314d5, SUC_lexicon_tags_7 },
    { 0xd23314d6, SUC_lexicon_tags_11 },
    { 0xd0d314d6, SUC_lexicon_tags_2 },
    { 0xc1f314d6, SUC_lexicon_tags_23 },
    { 0xb65314d6, SUC_lexicon_tags_47 },
    { 0x937314da, SUC_lexicon_tags_19 },
    { 0x523314d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5314e8, SUC_lexicon_tags_12 },
    { 0xd19314e8, SUC_lexicon_tags_25 },
    { 0xefb314e8, SUC_lexicon_tags_6 },
    { 0xd314e8, SUC_lexicon_tags_60 },
    { 0xd0b314ec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7314f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x531314f8, SUC_lexicon_tags_17 },
    { 0x92f314f9, SUC_lexicon_tags_28 },
    { 0x689314f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d314fd, SUC_lexicon_tags_37 },
    { 0x1a9314fe, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x60b31500, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d31504, SUC_lexicon_tags_23 },
    { 0xe6b31504, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f31508, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b3150b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a731517, SUC_lexicon_tags_45 },
    { 0x15b31518, SUC_lexicon_tags_43 },
    { 0xfc531518, SUC_lexicon_tags_12 },
    { 0x9553151a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d3151e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41131520, SUC_lexicon_tags_63 },
    { 0x54931520, SUC_lexicon_tags_1 },
    { 0x4fd31522, SUC_lexicon_tags_15 },
    { 0xc5531523, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1331526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d31530, SUC_lexicon_tags_129 },
    { 0x27b31531, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b31546, SUC_lexicon_tags_28 },
    { 0xde531546, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f3154a, SUC_lexicon_tags_9 },
    { 0xf1b3154b, SUC_lexicon_tags_6 },
    { 0xb913154c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1753154e, SUC_lexicon_tags_33 },
    { 0xccf3154f, SUC_lexicon_tags_48 },
    { 0x2393154f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b931552, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4d531554, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf31557, SUC_lexicon_tags_25 },
    { 0x7c931558, SUC_lexicon_tags_19 },
    { 0x4df31559, SUC_lexicon_tags_11 },
    { 0xeb73155a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6613155d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb31563, SUC_lexicon_tags_17 },
    { 0xc3d31564, SUC_lexicon_tags_25 },
    { 0xb1131564, SUC_lexicon_tags_55 },
    { 0xb3731565, SUC_lexicon_tags_26 },
    { 0xfc531567, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x55931569, SUC_lexicon_tags_1 },
    { 0x7e33156a, SUC_lexicon_tags_2 },
    { 0x6b53156a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef3156f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e731574, SUC_lexicon_tags_46 },
    { 0x33531574, SUC_lexicon_tags_11 },
    { 0xb2531576, SUC_lexicon_tags_45 },
    { 0xea731576, SUC_lexicon_tags_306 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4033157c, SUC_lexicon_tags_15 },
    { 0xb6d3157d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d331581, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x71531583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f31587, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x75d31589, SUC_lexicon_tags_15 },
    { 0x39f3158a, SUC_lexicon_tags_12 },
    { 0xb1931589, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f3158f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad31595, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db31598, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a93159e, SUC_lexicon_tags_21 },
    { 0x7653159e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x751315a2, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x3f315a4, SUC_lexicon_tags_2 },
    { 0x9ab315a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb315aa, SUC_lexicon_tags_55 },
    { 0x1b1315ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa67315ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x46f315af, SUC_lexicon_tags_8 },
    { 0xdb3315b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa63315b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931315bf, SUC_lexicon_tags_46 },
    { 0x11b315c0, SUC_lexicon_tags_7 },
    { 0x381315c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92b315c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797315c7, SUC_lexicon_tags_39 },
    { 0x5db315c7, SUC_lexicon_tags_12 },
    { 0xff5315c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d315d0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77315d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3315d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f3315da, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69315e2, SUC_lexicon_tags_62 },
    { 0xd1315e3, SUC_lexicon_tags_25 },
    { 0x2ef315e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf73315e6, SUC_lexicon_tags_133 },
    { 0xf51315e7, SUC_lexicon_tags_84 },
    { 0xf35315e7, SUC_lexicon_tags_8 },
    { 0x487315e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe89315eb, SUC_lexicon_tags_87 },
    { 0xe51315ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3315f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b1315f2, SUC_lexicon_tags_38 },
    { 0xeaf315f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d315f7, SUC_lexicon_tags_11 },
    { 0x38f315f8, SUC_lexicon_tags_43 },
    { 0xa51315f8, SUC_lexicon_tags_37 },
    { 0xb87315f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x859315fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82731603, SUC_lexicon_tags_56 },
    { 0x731604, SUC_lexicon_tags_37 },
    { 0x22931604, SUC_lexicon_tags_11 },
    { 0x7c731606, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd931608, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f3160c, SUC_lexicon_tags_2 },
    { 0xfaf3160d, SUC_lexicon_tags_14 },
    { 0x26b3160e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27131612, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c531615, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd6531617, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3331621, SUC_lexicon_tags_18 },
    { 0xc8931622, SUC_lexicon_tags_422 },
    { 0x52531623, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933162f, SUC_lexicon_tags_7 },
    { 0x8eb31630, SUC_lexicon_tags_11 },
    { 0x40131631, SUC_lexicon_tags_33 },
    { 0xb2131632, SUC_lexicon_tags_65 },
    { 0x40731633, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29b31635, SUC_lexicon_tags_14 },
    { 0x1e331636, SUC_lexicon_tags_2 },
    { 0xee731636, SUC_lexicon_tags_47 },
    { 0xbbd31635, SUC_lexicon_tags_31 },
    { 0x45d31637, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf693163c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fb3163e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b31642, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb31645, SUC_lexicon_tags_33 },
    { 0xbe131646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d31649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5833164b, SUC_lexicon_tags_47 },
    { 0x52b3164b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4693164f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc331652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a331659, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9873165c, SUC_lexicon_tags_2 },
    { 0xc693165d, SUC_lexicon_tags_17 },
    { 0x6f73165d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xde731661, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f31665, SUC_lexicon_tags_25 },
    { 0xc8731666, SUC_lexicon_tags_38 },
    { 0xa8331667, SUC_lexicon_tags_7 },
    { 0x3e931668, SUC_lexicon_tags_17 },
    { 0x91131669, SUC_lexicon_tags_11 },
    { 0xc2731667, SUC_lexicon_tags_21 },
    { 0xc5d3166b, SUC_lexicon_tags_2 },
    { 0x9a531668, SUC_lexicon_tags_7 },
    { 0x9553166d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1b3166f, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x45b31671, SUC_lexicon_tags_60 },
    { 0xdc731671, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50f31674, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x37d31676, SUC_lexicon_tags_11 },
    { 0x70b31677, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b53167c, SUC_lexicon_tags_7 },
    { 0x4a13167d, SUC_lexicon_tags_4 },
    { 0xcdf3167d, SUC_lexicon_tags_14 },
    { 0xb7b3167d, SUC_lexicon_tags_1 },
    { 0xf833167d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa8d31682, SUC_lexicon_tags_0 },
    { 0x2f731683, SUC_lexicon_tags_45 },
    { 0xcb131684, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf9731686, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a131689, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b3168c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b31691, SUC_lexicon_tags_30 },
    { 0x1f31692, SUC_lexicon_tags_10 },
    { 0xa0f31692, SUC_lexicon_tags_23 },
    { 0xddd31694, SUC_lexicon_tags_15 },
    { 0x61731695, SUC_lexicon_tags_60 },
    { 0xe9f31696, SUC_lexicon_tags_1 },
    { 0x44b31696, SUC_lexicon_tags_13 },
    { 0xc6331696, SUC_lexicon_tags_8 },
    { 0xbf131695, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265316a3, SUC_lexicon_tags_25 },
    { 0xb73316a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7316ad, SUC_lexicon_tags_7 },
    { 0xdbf316ae, SUC_lexicon_tags_38 },
    { 0xc83316ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15316b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243316b5, SUC_lexicon_tags_63 },
    { 0x8bb316b6, SUC_lexicon_tags_37 },
    { 0x32f316b7, SUC_lexicon_tags_27 },
    { 0x83f316b7, SUC_lexicon_tags_25 },
    { 0x28d316b5, SUC_lexicon_tags_21 },
    { 0xa89316ba, SUC_lexicon_tags_37 },
    { 0xb53316bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd316c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x407316c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd3b316c6, SUC_lexicon_tags_28 },
    { 0x625316c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x733316c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99316cc, SUC_lexicon_tags_8 },
    { 0xa11316cd, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93316d1, SUC_lexicon_tags_8 },
    { 0x893316d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadd316d4, SUC_lexicon_tags_15 },
    { 0xe89316d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97316d8, SUC_lexicon_tags_11 },
    { 0x741316d9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d316dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb9316de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d316e1, SUC_lexicon_tags_23 },
    { 0xcb3316e2, SUC_lexicon_tags_43 },
    { 0x293316e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x515316e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed316eb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49316f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd29316f3, SUC_lexicon_tags_25 },
    { 0xedd316f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb7f316f6, SUC_lexicon_tags_17 },
    { 0x101316f6, SUC_lexicon_tags_0 },
    { 0x225316f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x509316fb, SUC_lexicon_tags_2 },
    { 0x551316fb, SUC_lexicon_tags_21 },
    { 0x627316fb, SUC_lexicon_tags_56 },
    { 0xf6d316fe, SUC_lexicon_tags_26 },
    { 0x4dd316ff, SUC_lexicon_tags_13 },
    { 0x91531700, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0531706, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd3170a, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x1413170c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xee13170e, SUC_lexicon_tags_28 },
    { 0x9853170e, SUC_lexicon_tags_23 },
    { 0xbf53170f, SUC_lexicon_tags_27 },
    { 0x8b31711, SUC_lexicon_tags_29 },
    { 0x27d31712, SUC_lexicon_tags_28 },
    { 0x7df31713, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xca531715, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20931717, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a33171a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9331720, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b31724, SUC_lexicon_tags_0 },
    { 0x36331724, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3931735, SUC_lexicon_tags_2 },
    { 0x45931735, SUC_lexicon_tags_15 },
    { 0xa6331737, SUC_lexicon_tags_367 },
    { 0, NULL },
    { 0x80f31739, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b31745, SUC_lexicon_tags_12 },
    { 0xcc731746, SUC_lexicon_tags_35 },
    { 0xaf731746, SUC_lexicon_tags_30 },
    { 0xa131748, SUC_lexicon_tags_2 },
    { 0xcf531747, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x10d3174b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ff3174d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b31752, SUC_lexicon_tags_27 },
    { 0x29131753, SUC_lexicon_tags_25 },
    { 0xbc931753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf33175b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f3175f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd131769, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xce73176b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac93176e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4df31770, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75331772, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9131775, SUC_lexicon_tags_0 },
    { 0xb6931775, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe193177c, SUC_lexicon_tags_12 },
    { 0x6c53177d, SUC_lexicon_tags_47 },
    { 0x77f3177d, SUC_lexicon_tags_1 },
    { 0x5eb3177f, SUC_lexicon_tags_1 },
    { 0x58b31780, SUC_lexicon_tags_17 },
    { 0xfc73177f, SUC_lexicon_tags_87 },
    { 0x48331782, SUC_lexicon_tags_47 },
    { 0x82731783, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c531786, SUC_lexicon_tags_7 },
    { 0x3e931787, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24b31789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d3178f, SUC_lexicon_tags_60 },
    { 0xc6d3178f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda531795, SUC_lexicon_tags_63 },
    { 0x91731796, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3f31798, SUC_lexicon_tags_25 },
    { 0xea131799, SUC_lexicon_tags_8 },
    { 0xfe93179a, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3179f, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d317a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2d317a4, SUC_lexicon_tags_31 },
    { 0x75f317a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x855317a8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5317ab, SUC_lexicon_tags_2 },
    { 0x9ad317ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe91317ae, SUC_lexicon_tags_9 },
    { 0xba1317af, SUC_lexicon_tags_30 },
    { 0x271317b0, SUC_lexicon_tags_621 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7317b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x689317b9, SUC_lexicon_tags_2 },
    { 0x9c1317b9, SUC_lexicon_tags_47 },
    { 0x9d317b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb317bf, SUC_lexicon_tags_37 },
    { 0x1a1317c0, SUC_lexicon_tags_15 },
    { 0x3d1317c0, SUC_lexicon_tags_8 },
    { 0x741317c2, SUC_lexicon_tags_2 },
    { 0x39317bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61317c8, SUC_lexicon_tags_37 },
    { 0x997317c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7317cd, SUC_lexicon_tags_28 },
    { 0x553317ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x259317d0, SUC_lexicon_tags_11 },
    { 0x137317d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f317d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x401317d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827317dc, SUC_lexicon_tags_5 },
    { 0xad9317dd, SUC_lexicon_tags_16 },
    { 0xa4b317dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x615317e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09317e8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57317ed, SUC_lexicon_tags_82 },
    { 0xeed317ed, SUC_lexicon_tags_2 },
    { 0x361317ed, SUC_lexicon_tags_81 },
    { 0x273317ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x79b317f2, SUC_lexicon_tags_37 },
    { 0x569317f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42b317f5, SUC_lexicon_tags_25 },
    { 0xd31317f5, SUC_lexicon_tags_11 },
    { 0x6c3317f7, SUC_lexicon_tags_45 },
    { 0x8cd317f7, SUC_lexicon_tags_7 },
    { 0x225317f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb73317fb, SUC_lexicon_tags_34 },
    { 0x6bf317fc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x72b317fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f31803, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf1931805, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf3180a, SUC_lexicon_tags_17 },
    { 0x9113180b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb331811, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b31816, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1d31818, SUC_lexicon_tags_33 },
    { 0x7b331819, SUC_lexicon_tags_17 },
    { 0xa713181a, SUC_lexicon_tags_7 },
    { 0x3413181a, SUC_lexicon_tags_22 },
    { 0x8933181c, SUC_lexicon_tags_37 },
    { 0x5733181c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db31821, SUC_lexicon_tags_43 },
    { 0xb7331821, SUC_lexicon_tags_7 },
    { 0xf2131823, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0xb8b31825, SUC_lexicon_tags_30 },
    { 0xaef31826, SUC_lexicon_tags_42 },
    { 0x31531827, SUC_lexicon_tags_21 },
    { 0x3f31825, SUC_lexicon_tags_96 },
    { 0xea131827, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe053182b, SUC_lexicon_tags_92 },
    { 0x54b3182c, SUC_lexicon_tags_37 },
    { 0x64d3182b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ab3182f, SUC_lexicon_tags_12 },
    { 0x6ff31830, SUC_lexicon_tags_7 },
    { 0x9f131831, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0931837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb73183a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d13183e, SUC_lexicon_tags_12 },
    { 0xe173183f, SUC_lexicon_tags_30 },
    { 0x80b3183f, SUC_lexicon_tags_27 },
    { 0xccd3183f, SUC_lexicon_tags_28 },
    { 0x5733183e, SUC_lexicon_tags_25 },
    { 0xeb531843, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8931847, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af3184d, SUC_lexicon_tags_7 },
    { 0xb693184d, SUC_lexicon_tags_7 },
    { 0x7193184e, SUC_lexicon_tags_11 },
    { 0x1fd31850, SUC_lexicon_tags_12 },
    { 0xa0531851, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d131853, SUC_lexicon_tags_2 },
    { 0x58131853, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xebd31856, SUC_lexicon_tags_39 },
    { 0x7931857, SUC_lexicon_tags_25 },
    { 0x88f31857, SUC_lexicon_tags_7 },
    { 0xaab31856, SUC_lexicon_tags_33 },
    { 0xa433185a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1593185c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84d3185e, SUC_lexicon_tags_12 },
    { 0x5dd3185f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db31862, SUC_lexicon_tags_33 },
    { 0xcd131862, SUC_lexicon_tags_37 },
    { 0xc8531863, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f31868, SUC_lexicon_tags_37 },
    { 0xfb731868, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb3186c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af31871, SUC_lexicon_tags_15 },
    { 0x82d31872, SUC_lexicon_tags_19 },
    { 0xa0531872, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf293187b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d3187f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87731883, SUC_lexicon_tags_37 },
    { 0x5bd31884, SUC_lexicon_tags_17 },
    { 0xbe531885, SUC_lexicon_tags_15 },
    { 0x3e531884, SUC_lexicon_tags_11 },
    { 0xf2b31887, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7933188b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed33188e, SUC_lexicon_tags_12 },
    { 0x7b33188f, SUC_lexicon_tags_3 },
    { 0xd8f3188f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52931895, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd873189a, SUC_lexicon_tags_25 },
    { 0x8d53189b, SUC_lexicon_tags_9 },
    { 0xb93189c, SUC_lexicon_tags_69 },
    { 0xc653189b, SUC_lexicon_tags_14 },
    { 0x5db3189c, SUC_lexicon_tags_29 },
    { 0x8453189e, SUC_lexicon_tags_29 },
    { 0x91f3189e, SUC_lexicon_tags_14 },
    { 0x9db3189d, SUC_lexicon_tags_11 },
    { 0x727318a2, SUC_lexicon_tags_37 },
    { 0x3c3318a2, SUC_lexicon_tags_7 },
    { 0xf3d318a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9318a7, SUC_lexicon_tags_12 },
    { 0xd8f318a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x381318aa, SUC_lexicon_tags_38 },
    { 0x8c5318ab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd37318ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89b318af, SUC_lexicon_tags_336 },
    { 0xd73318b0, SUC_lexicon_tags_12 },
    { 0xadf318b1, SUC_lexicon_tags_29 },
    { 0x515318b1, SUC_lexicon_tags_37 },
    { 0xaf3318b1, SUC_lexicon_tags_27 },
    { 0xc95318b4, SUC_lexicon_tags_38 },
    { 0xcf7318b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3318b9, SUC_lexicon_tags_9 },
    { 0x227318ba, SUC_lexicon_tags_15 },
    { 0x65f318bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xead318bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x817318c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc5318c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x215318c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f318c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xedf318ca, SUC_lexicon_tags_26 },
    { 0xd25318cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f318cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc63318d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f318d6, SUC_lexicon_tags_23 },
    { 0x74d318d7, SUC_lexicon_tags_79 },
    { 0x26d318d8, SUC_lexicon_tags_61 },
    { 0xb37318d8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x479318dc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71318e1, SUC_lexicon_tags_1 },
    { 0x139318e1, SUC_lexicon_tags_31 },
    { 0x977318e1, SUC_lexicon_tags_2 },
    { 0xf8f318e2, SUC_lexicon_tags_17 },
    { 0x9f318e1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x693318e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb47318ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95318ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97318f1, SUC_lexicon_tags_82 },
    { 0x605318f2, SUC_lexicon_tags_2 },
    { 0x261318f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75318fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937318ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9531904, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef331906, SUC_lexicon_tags_87 },
    { 0xb2931907, SUC_lexicon_tags_49 },
    { 0x8e331906, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33f3190a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf3190f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f31913, SUC_lexicon_tags_17 },
    { 0x2b131913, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b3191c, SUC_lexicon_tags_14 },
    { 0x34f3191c, SUC_lexicon_tags_17 },
    { 0xd853191d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df31923, SUC_lexicon_tags_252 },
    { 0x42131924, SUC_lexicon_tags_11 },
    { 0x13731925, SUC_lexicon_tags_45 },
    { 0xba131926, SUC_lexicon_tags_2 },
    { 0xa8731925, SUC_lexicon_tags_25 },
    { 0x20731925, SUC_lexicon_tags_29 },
    { 0xf531929, SUC_lexicon_tags_11 },
    { 0x9d331929, SUC_lexicon_tags_15 },
    { 0xe9b3192a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb533192f, SUC_lexicon_tags_48 },
    { 0xe3531930, SUC_lexicon_tags_37 },
    { 0x9253192f, SUC_lexicon_tags_11 },
    { 0xf3d31932, SUC_lexicon_tags_11 },
    { 0x42b31933, SUC_lexicon_tags_1 },
    { 0x65f31932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56731937, SUC_lexicon_tags_463 },
    { 0x85f31937, SUC_lexicon_tags_37 },
    { 0x2c931939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3153193f, SUC_lexicon_tags_32 },
    { 0x13931940, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x43b31942, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5af31944, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd31946, SUC_lexicon_tags_38 },
    { 0x8bb31947, SUC_lexicon_tags_1 },
    { 0x61b31948, SUC_lexicon_tags_67 },
    { 0x17531947, SUC_lexicon_tags_11 },
    { 0x6cb31947, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6d73194c, SUC_lexicon_tags_2 },
    { 0xe293194d, SUC_lexicon_tags_44 },
    { 0x5b13194e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b31958, SUC_lexicon_tags_17 },
    { 0x5c931959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96d3195b, SUC_lexicon_tags_25 },
    { 0x4013195c, SUC_lexicon_tags_437 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6331962, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a331965, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x83d31967, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x84531969, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe813196f, SUC_lexicon_tags_9 },
    { 0xe2f31970, SUC_lexicon_tags_0 },
    { 0x29f31971, SUC_lexicon_tags_44 },
    { 0xd5531971, SUC_lexicon_tags_30 },
    { 0x46331971, SUC_lexicon_tags_47 },
    { 0xb8931970, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d31977, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec73197b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a73197e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4931981, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7331987, SUC_lexicon_tags_11 },
    { 0xf7731987, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d31990, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2e531992, SUC_lexicon_tags_63 },
    { 0x47931992, SUC_lexicon_tags_7 },
    { 0xda331994, SUC_lexicon_tags_23 },
    { 0x1f331994, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9873199d, SUC_lexicon_tags_91 },
    { 0x6093199e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83319a1, SUC_lexicon_tags_7 },
    { 0x625319a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfa9319a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe7d319a6, SUC_lexicon_tags_12 },
    { 0x66f319a6, SUC_lexicon_tags_15 },
    { 0x5c1319a8, SUC_lexicon_tags_1 },
    { 0x787319a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707319ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97319b3, SUC_lexicon_tags_10 },
    { 0x7fb319b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x491319b7, SUC_lexicon_tags_17 },
    { 0x1ef319b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141319bc, SUC_lexicon_tags_30 },
    { 0xab5319bd, SUC_lexicon_tags_12 },
    { 0x283319be, SUC_lexicon_tags_7 },
    { 0x48b319be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d319c5, SUC_lexicon_tags_37 },
    { 0x8b1319c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9319c9, SUC_lexicon_tags_10 },
    { 0x339319ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x89319cd, SUC_lexicon_tags_15 },
    { 0x41d319ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f319d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f319d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d319d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d319da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x535319dc, SUC_lexicon_tags_30 },
    { 0xf93319dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d319e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d319e4, SUC_lexicon_tags_36 },
    { 0x3dd319e4, SUC_lexicon_tags_7 },
    { 0x835319e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97319ea, SUC_lexicon_tags_30 },
    { 0xc11319ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591319f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec7319f3, SUC_lexicon_tags_25 },
    { 0xb4f319f4, SUC_lexicon_tags_28 },
    { 0xcd7319f5, SUC_lexicon_tags_11 },
    { 0x51b319f4, SUC_lexicon_tags_11 },
    { 0x6af319f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53319f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b319fc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2731a09, SUC_lexicon_tags_26 },
    { 0x91d31a0a, SUC_lexicon_tags_25 },
    { 0x78531a0a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84931a0d, SUC_lexicon_tags_2 },
    { 0x79131a0e, SUC_lexicon_tags_38 },
    { 0xfdd31a0e, SUC_lexicon_tags_26 },
    { 0x50f31a0f, SUC_lexicon_tags_1 },
    { 0x5db31a11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a131a14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3931a17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0931a1b, SUC_lexicon_tags_17 },
    { 0x48d31a1c, SUC_lexicon_tags_23 },
    { 0x98331a1b, SUC_lexicon_tags_29 },
    { 0x8f31a1d, SUC_lexicon_tags_84 },
    { 0xb8b31a1e, SUC_lexicon_tags_63 },
    { 0xd2331a20, SUC_lexicon_tags_17 },
    { 0x21931a20, SUC_lexicon_tags_60 },
    { 0xdcf31a22, SUC_lexicon_tags_7 },
    { 0xa3b31a23, SUC_lexicon_tags_15 },
    { 0x1af31a23, SUC_lexicon_tags_17 },
    { 0x70531a23, SUC_lexicon_tags_26 },
    { 0x31531a24, SUC_lexicon_tags_7 },
    { 0xc7d31a26, SUC_lexicon_tags_5 },
    { 0xd9331a26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d31a2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe4331a2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa331a33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1331a36, SUC_lexicon_tags_28 },
    { 0x43131a37, SUC_lexicon_tags_1 },
    { 0x9d331a37, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d931a3b, SUC_lexicon_tags_125 },
    { 0x8f31a3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6331a3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63931a41, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x32d31a43, SUC_lexicon_tags_27 },
    { 0xb3131a44, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee531a49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1731a50, SUC_lexicon_tags_7 },
    { 0xe4531a51, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9931a57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef31a59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca131a61, SUC_lexicon_tags_37 },
    { 0x9e931a61, SUC_lexicon_tags_12 },
    { 0x8b931a63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6931a6c, SUC_lexicon_tags_32 },
    { 0x3d31a6d, SUC_lexicon_tags_0 },
    { 0x83131a6c, SUC_lexicon_tags_7 },
    { 0xc331a6f, SUC_lexicon_tags_7 },
    { 0xd0931a6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b31a73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf31a77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b31a7b, SUC_lexicon_tags_47 },
    { 0x60931a7c, SUC_lexicon_tags_63 },
    { 0xfd131a7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6531a80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a331a83, SUC_lexicon_tags_84 },
    { 0xaf531a84, SUC_lexicon_tags_38 },
    { 0x35331a85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4c931a87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5131a8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2131a98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3531a9a, SUC_lexicon_tags_15 },
    { 0xe6131a9b, SUC_lexicon_tags_45 },
    { 0x47731a9b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x99d31a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91531aa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b31aa3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33931aa5, SUC_lexicon_tags_11 },
    { 0xc6331aa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65731aac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e331ab2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x42531ab5, SUC_lexicon_tags_1 },
    { 0x76131ab5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2c731ab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1531aba, SUC_lexicon_tags_13 },
    { 0xdd331aba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd31ac1, SUC_lexicon_tags_15 },
    { 0x7bb31ac1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x72d31ac4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d31ac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd731acc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab31ad1, SUC_lexicon_tags_25 },
    { 0x55531ad2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x91331ad4, SUC_lexicon_tags_15 },
    { 0x69131ad5, SUC_lexicon_tags_17 },
    { 0xf131ad5, SUC_lexicon_tags_7 },
    { 0x64f31ad5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10331adc, SUC_lexicon_tags_17 },
    { 0xcd931add, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b731ae1, SUC_lexicon_tags_37 },
    { 0xcdb31ae2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7931ae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b531ae9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xfa731aeb, SUC_lexicon_tags_37 },
    { 0xf5f31aeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d731aef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4b531af1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5db31af3, SUC_lexicon_tags_133 },
    { 0xb9b31af4, SUC_lexicon_tags_47 },
    { 0x5f131af5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1c331af7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11531b03, SUC_lexicon_tags_4 },
    { 0xffd31b04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d31b0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2931b0c, SUC_lexicon_tags_25 },
    { 0x22b31b0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc0b31b0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x86131b12, SUC_lexicon_tags_23 },
    { 0x6ff31b13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4731b1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96531b21, SUC_lexicon_tags_37 },
    { 0x6d31b22, SUC_lexicon_tags_23 },
    { 0x44731b23, SUC_lexicon_tags_25 },
    { 0x1c931b23, SUC_lexicon_tags_25 },
    { 0x8c531b25, SUC_lexicon_tags_37 },
    { 0xc7f31b25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59131b29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c731b2e, SUC_lexicon_tags_15 },
    { 0xe8b31b2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed31b36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87f31b38, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb31b3d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b31b40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68331b46, SUC_lexicon_tags_27 },
    { 0xdb731b47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a131b49, SUC_lexicon_tags_7 },
    { 0x3af31b4a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b31b53, SUC_lexicon_tags_43 },
    { 0xe931b53, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3eb31b56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec331b5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc331b5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb31b64, SUC_lexicon_tags_32 },
    { 0x7131b65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x44531b67, SUC_lexicon_tags_56 },
    { 0x66131b68, SUC_lexicon_tags_34 },
    { 0xa8131b68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2331b6e, SUC_lexicon_tags_66 },
    { 0xa4931b6f, SUC_lexicon_tags_0 },
    { 0xf1731b70, SUC_lexicon_tags_0 },
    { 0x4fb31b71, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x81f31b73, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe331b75, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbd731b77, SUC_lexicon_tags_9 },
    { 0x81531b78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31131b83, SUC_lexicon_tags_13 },
    { 0x6e331b83, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61531b88, SUC_lexicon_tags_23 },
    { 0x20531b89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59931b8c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b31b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef31b99, SUC_lexicon_tags_37 },
    { 0x18731b9a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff31b9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2331ba5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x21131ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8731ba9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce531bad, SUC_lexicon_tags_45 },
    { 0x11931bae, SUC_lexicon_tags_127 },
    { 0xc5331bae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d31bb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x76931bb5, SUC_lexicon_tags_12 },
    { 0xb1131bb6, SUC_lexicon_tags_2 },
    { 0xecd31bb6, SUC_lexicon_tags_112 },
    { 0xd0531bb7, SUC_lexicon_tags_15 },
    { 0x79931bb5, SUC_lexicon_tags_11 },
    { 0xd731bb9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13131bc3, SUC_lexicon_tags_16 },
    { 0xa3331bc3, SUC_lexicon_tags_17 },
    { 0x96531bc5, SUC_lexicon_tags_11 },
    { 0x7f931bc6, SUC_lexicon_tags_0 },
    { 0xd3d31bc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91931bcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70531bd1, SUC_lexicon_tags_47 },
    { 0xc8931bd2, SUC_lexicon_tags_6 },
    { 0xe731bd2, SUC_lexicon_tags_812 },
    { 0x74331bd4, SUC_lexicon_tags_14 },
    { 0x3f731bd4, SUC_lexicon_tags_11 },
    { 0xd0d31bd6, SUC_lexicon_tags_60 },
    { 0x6d131bd4, SUC_lexicon_tags_1 },
    { 0x6d531bd8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38131bdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85131bde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6b31be0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16331be2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed31be6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf3f31be8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff31bec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x83b31bee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18931bf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69931bf5, SUC_lexicon_tags_14 },
    { 0x80d31bf6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe8531bf8, SUC_lexicon_tags_20 },
    { 0xacb31bf8, SUC_lexicon_tags_19 },
    { 0xccb31bfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17531bfc, SUC_lexicon_tags_17 },
    { 0xe8731bfd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x99531bff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x45b31c01, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b31c04, SUC_lexicon_tags_7 },
    { 0xa4531c05, SUC_lexicon_tags_33 },
    { 0xde531c06, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd31c09, SUC_lexicon_tags_49 },
    { 0xcfd31c0a, SUC_lexicon_tags_30 },
    { 0xcbf31c0a, SUC_lexicon_tags_37 },
    { 0x9bb31c0b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62131c11, SUC_lexicon_tags_84 },
    { 0x2cf31c12, SUC_lexicon_tags_31 },
    { 0x40d31c11, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5a531c15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16931c1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b31c20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8f31c22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b31c25, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x75731c27, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d31c2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f331c2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bd31c2f, SUC_lexicon_tags_2 },
    { 0xe3f31c30, SUC_lexicon_tags_6 },
    { 0x28331c30, SUC_lexicon_tags_33 },
    { 0x4c931c30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5331c39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b31c3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x42131c42, SUC_lexicon_tags_8 },
    { 0xb4d31c43, SUC_lexicon_tags_7 },
    { 0x14331c44, SUC_lexicon_tags_15 },
    { 0xf931c43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf6d31c47, SUC_lexicon_tags_14 },
    { 0xff731c48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31131c4d, SUC_lexicon_tags_25 },
    { 0x1331c4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa931c51, SUC_lexicon_tags_627 },
    { 0x7e331c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91131c57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91931c5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe131c5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f31c64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb31c68, SUC_lexicon_tags_14 },
    { 0xfa331c68, SUC_lexicon_tags_49 },
    { 0xfb131c69, SUC_lexicon_tags_7 },
    { 0x70531c69, SUC_lexicon_tags_3 },
    { 0xde731c6a, SUC_lexicon_tags_15 },
    { 0xebd31c6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58531c73, SUC_lexicon_tags_1 },
    { 0x7731c74, SUC_lexicon_tags_47 },
    { 0x77931c74, SUC_lexicon_tags_34 },
    { 0xfb531c74, SUC_lexicon_tags_11 },
    { 0xf4b31c77, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9df31c79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0931c7c, SUC_lexicon_tags_37 },
    { 0xdfd31c7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8db31c7f, SUC_lexicon_tags_37 },
    { 0xd5331c7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c131c82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x48331c85, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e731c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b531c91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8731c96, SUC_lexicon_tags_1 },
    { 0x1cd31c96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69731c9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5331c9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x73731c9f, SUC_lexicon_tags_27 },
    { 0xf1d31ca0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x18931ca3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa731ca5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x50531ca8, SUC_lexicon_tags_29 },
    { 0xa4131ca8, SUC_lexicon_tags_25 },
    { 0x62f31caa, SUC_lexicon_tags_2 },
    { 0xa4331caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78931cad, SUC_lexicon_tags_25 },
    { 0xf6731cae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b131cb1, SUC_lexicon_tags_2 },
    { 0x1cf31cb2, SUC_lexicon_tags_16 },
    { 0x64931cb1, SUC_lexicon_tags_8 },
    { 0xebb31cb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc8731cb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x31d31cb8, SUC_lexicon_tags_17 },
    { 0x99531cb9, SUC_lexicon_tags_17 },
    { 0xb2d31cba, SUC_lexicon_tags_77 },
    { 0xdd31cba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b731cc4, SUC_lexicon_tags_7 },
    { 0x11d31cc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f31cc8, SUC_lexicon_tags_47 },
    { 0x7af31cc9, SUC_lexicon_tags_48 },
    { 0xa2d31cca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0331ccc, SUC_lexicon_tags_12 },
    { 0x6ff31ccd, SUC_lexicon_tags_37 },
    { 0x5a731ccc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4331cd0, SUC_lexicon_tags_12 },
    { 0x2ab31cd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1331cd5, SUC_lexicon_tags_17 },
    { 0x84f31cd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb731cd8, SUC_lexicon_tags_45 },
    { 0xf0931cd9, SUC_lexicon_tags_8 },
    { 0x27131cda, SUC_lexicon_tags_1 },
    { 0x38b31cdb, SUC_lexicon_tags_45 },
    { 0x85931cdc, SUC_lexicon_tags_34 },
    { 0x4bd31cd8, SUC_lexicon_tags_1 },
    { 0xf2531cde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6d31ce0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x49331ce2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b31ce5, SUC_lexicon_tags_63 },
    { 0x8f931ce5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a331ce8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2eb31cea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb31cef, SUC_lexicon_tags_31 },
    { 0x9d531cef, SUC_lexicon_tags_60 },
    { 0xc3931cf1, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x8b931cf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d31cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba531cf8, SUC_lexicon_tags_31 },
    { 0x52731cf9, SUC_lexicon_tags_47 },
    { 0xd4531cfa, SUC_lexicon_tags_68 },
    { 0xa4531cf8, SUC_lexicon_tags_17 },
    { 0x5a731cfb, SUC_lexicon_tags_11 },
    { 0x16f31cfd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b31d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a531d02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd31d0f, SUC_lexicon_tags_1 },
    { 0x28331d10, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x99731d12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f31d19, SUC_lexicon_tags_11 },
    { 0x86531d1a, SUC_lexicon_tags_14 },
    { 0x4d531d19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10931d1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87531d20, SUC_lexicon_tags_37 },
    { 0x56331d21, SUC_lexicon_tags_12 },
    { 0x9b731d20, SUC_lexicon_tags_37 },
    { 0x91d31d23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3731d25, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1531d28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe531d2d, SUC_lexicon_tags_0 },
    { 0xc8531d2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ad31d30, SUC_lexicon_tags_17 },
    { 0x7731d31, SUC_lexicon_tags_38 },
    { 0x27731d31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44731d34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b31d37, SUC_lexicon_tags_84 },
    { 0xfe131d37, SUC_lexicon_tags_31 },
    { 0xa5331d39, SUC_lexicon_tags_25 },
    { 0xb5531d39, SUC_lexicon_tags_231 },
    { 0xf2531d3b, SUC_lexicon_tags_2 },
    { 0x87d31d39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d31d40, SUC_lexicon_tags_17 },
    { 0x88531d41, SUC_lexicon_tags_66 },
    { 0xdb331d42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b531d44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f31d4a, SUC_lexicon_tags_11 },
    { 0x10931d4b, SUC_lexicon_tags_201 },
    { 0x89531d4c, SUC_lexicon_tags_1 },
    { 0x7c731d4d, SUC_lexicon_tags_17 },
    { 0x8bb31d4e, SUC_lexicon_tags_17 },
    { 0x4f331d4c, SUC_lexicon_tags_33 },
    { 0x77331d50, SUC_lexicon_tags_60 },
    { 0x1ef31d51, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x31531d54, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x76531d56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55131d59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d31d5b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1f531d5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a731d5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca131d6a, SUC_lexicon_tags_12 },
    { 0x27131d6b, SUC_lexicon_tags_60 },
    { 0x25931d6b, SUC_lexicon_tags_49 },
    { 0xb1531d6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef31d73, SUC_lexicon_tags_12 },
    { 0x81131d74, SUC_lexicon_tags_12 },
    { 0x23531d75, SUC_lexicon_tags_31 },
    { 0xa2f31d76, SUC_lexicon_tags_17 },
    { 0xd8931d77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c131d7c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d31d83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db31d87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc1331d89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x70331d8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af31d90, SUC_lexicon_tags_33 },
    { 0x54b31d91, SUC_lexicon_tags_15 },
    { 0x39331d92, SUC_lexicon_tags_61 },
    { 0x85331d93, SUC_lexicon_tags_55 },
    { 0xdc931d94, SUC_lexicon_tags_17 },
    { 0x10b31d92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b31d99, SUC_lexicon_tags_1 },
    { 0xbe531d9a, SUC_lexicon_tags_9 },
    { 0xb8b31d9a, SUC_lexicon_tags_11 },
    { 0xfad31d99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25731d9e, SUC_lexicon_tags_0 },
    { 0xa1331d9f, SUC_lexicon_tags_37 },
    { 0x19531da0, SUC_lexicon_tags_4 },
    { 0xe0331d9f, SUC_lexicon_tags_67 },
    { 0x4e131d9f, SUC_lexicon_tags_32 },
    { 0x4dd31da3, SUC_lexicon_tags_60 },
    { 0x1e131d9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4531daa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f31dae, SUC_lexicon_tags_7 },
    { 0x3bf31daf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f31db1, SUC_lexicon_tags_2 },
    { 0xb3d31db1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb931db6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c931dba, SUC_lexicon_tags_43 },
    { 0x91d31dba, SUC_lexicon_tags_60 },
    { 0x27b31dbb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4331dc1, SUC_lexicon_tags_2 },
    { 0xba331dc1, SUC_lexicon_tags_65 },
    { 0x71931dc3, SUC_lexicon_tags_1 },
    { 0xcf31dc2, SUC_lexicon_tags_26 },
    { 0x81131dc5, SUC_lexicon_tags_35 },
    { 0xaaf31dc2, SUC_lexicon_tags_17 },
    { 0x9d131dc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83331dcd, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xf8f31dcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f31dd4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40531dd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6131dda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x50531ddd, SUC_lexicon_tags_127 },
    { 0x5a131dde, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xd5331de0, SUC_lexicon_tags_25 },
    { 0xe6931de1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b31de9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1b531deb, SUC_lexicon_tags_12 },
    { 0x95b31dec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaed31dee, SUC_lexicon_tags_28 },
    { 0x86331def, SUC_lexicon_tags_21 },
    { 0x74131df0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75731df6, SUC_lexicon_tags_7 },
    { 0xef331df7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8331dfa, SUC_lexicon_tags_1 },
    { 0x26931dfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb331e02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7331e05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b931e0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a131e0d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe8d31e0f, SUC_lexicon_tags_43 },
    { 0x31531e0f, SUC_lexicon_tags_47 },
    { 0xca531e11, SUC_lexicon_tags_1 },
    { 0x99731e10, SUC_lexicon_tags_14 },
    { 0xf7331e0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ed31e15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d31e1a, SUC_lexicon_tags_30 },
    { 0x10b31e1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57b31e1d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x75731e1f, SUC_lexicon_tags_17 },
    { 0x71731e1f, SUC_lexicon_tags_70 },
    { 0x4a731e20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee331e23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52d31e25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d31e28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14d31e2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9d31e2c, SUC_lexicon_tags_38 },
    { 0x4d531e2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb31e32, SUC_lexicon_tags_1709 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89331e36, SUC_lexicon_tags_8 },
    { 0xe9331e37, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xdbf31e39, SUC_lexicon_tags_33 },
    { 0x27731e3a, SUC_lexicon_tags_37 },
    { 0x5f931e3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1d31e3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f31e42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d131e46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x48331e48, SUC_lexicon_tags_7 },
    { 0x9f531e49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ef31e4b, SUC_lexicon_tags_2 },
    { 0x6dd31e4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5131e50, SUC_lexicon_tags_15 },
    { 0x5bd31e51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a131e53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x67b31e55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25931e59, SUC_lexicon_tags_28 },
    { 0x76931e59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c931e5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d31e67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b31e6b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf1731e6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b731e6f, SUC_lexicon_tags_6 },
    { 0xe2f31e70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab31e75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65b31e77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5d731e79, SUC_lexicon_tags_8 },
    { 0x45731e79, SUC_lexicon_tags_15 },
    { 0x7a531e7a, SUC_lexicon_tags_33 },
    { 0x2d131e7c, SUC_lexicon_tags_27 },
    { 0x91331e7a, SUC_lexicon_tags_47 },
    { 0xe2d31e7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd31e82, SUC_lexicon_tags_28 },
    { 0x2b31e83, SUC_lexicon_tags_27 },
    { 0x8f331e84, SUC_lexicon_tags_10 },
    { 0x79b31e83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28131e89, SUC_lexicon_tags_25 },
    { 0x2e331e8a, SUC_lexicon_tags_7 },
    { 0xd8d31e8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd31e8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71931e90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f131e94, SUC_lexicon_tags_7 },
    { 0x50131e95, SUC_lexicon_tags_43 },
    { 0x79d31e96, SUC_lexicon_tags_85 },
    { 0xfb531e97, SUC_lexicon_tags_47 },
    { 0x6bb31e98, SUC_lexicon_tags_9 },
    { 0x1af31e99, SUC_lexicon_tags_17 },
    { 0xc8f31e99, SUC_lexicon_tags_55 },
    { 0xe531e9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ed31e9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d931ea4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9131ea6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61131ea8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b31eab, SUC_lexicon_tags_25 },
    { 0xa6331eac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b131eb5, SUC_lexicon_tags_7 },
    { 0x2ff31eb6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4731eb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d31ebc, SUC_lexicon_tags_17 },
    { 0x9bb31ebd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c731ec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45931ec5, SUC_lexicon_tags_14 },
    { 0x2c731ec6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14731ecf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x63331ed1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b31ed9, SUC_lexicon_tags_9 },
    { 0xb4131eda, SUC_lexicon_tags_11 },
    { 0x66931edb, SUC_lexicon_tags_47 },
    { 0x2ad31edc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc7931ede, SUC_lexicon_tags_12 },
    { 0x51331ede, SUC_lexicon_tags_37 },
    { 0x5c331ede, SUC_lexicon_tags_49 },
    { 0x23331ee1, SUC_lexicon_tags_6 },
    { 0x8dd31ee2, SUC_lexicon_tags_17 },
    { 0xb5131ee3, SUC_lexicon_tags_37 },
    { 0x36131ee4, SUC_lexicon_tags_11 },
    { 0xd1b31ee5, SUC_lexicon_tags_48 },
    { 0xf1331ee6, SUC_lexicon_tags_44 },
    { 0x72531ee6, SUC_lexicon_tags_14 },
    { 0x6f131ee6, SUC_lexicon_tags_7 },
    { 0x42931ee6, SUC_lexicon_tags_38 },
    { 0x40b31eea, SUC_lexicon_tags_87 },
    { 0xc4131eea, SUC_lexicon_tags_9 },
    { 0x10d31eec, SUC_lexicon_tags_23 },
    { 0x18d31eed, SUC_lexicon_tags_70 },
    { 0x81931eed, SUC_lexicon_tags_15 },
    { 0xd1331edf, SUC_lexicon_tags_27 },
    { 0x2cd31ef0, SUC_lexicon_tags_12 },
    { 0xa8731ef1, SUC_lexicon_tags_47 },
    { 0x5ff31ef2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2331ef9, SUC_lexicon_tags_11 },
    { 0xf5931efa, SUC_lexicon_tags_11 },
    { 0xdcf31efb, SUC_lexicon_tags_29 },
    { 0xfb331ef9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75531efe, SUC_lexicon_tags_26 },
    { 0xeb731eff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf31f02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a331f06, SUC_lexicon_tags_12 },
    { 0xfff31f07, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0x54b31f09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb31f0e, SUC_lexicon_tags_41 },
    { 0x70b31f0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e531f14, SUC_lexicon_tags_37 },
    { 0xcb531f15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x34531f18, SUC_lexicon_tags_43 },
    { 0x86331f19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5931f1f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3bd31f21, SUC_lexicon_tags_32 },
    { 0xc8131f22, SUC_lexicon_tags_2 },
    { 0x52d31f21, SUC_lexicon_tags_31 },
    { 0xbf931f24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52331f27, SUC_lexicon_tags_12 },
    { 0xb7731f28, SUC_lexicon_tags_2 },
    { 0x72531f27, SUC_lexicon_tags_63 },
    { 0x81b31f2a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd31f2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5331f30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25931f32, SUC_lexicon_tags_8 },
    { 0x7e531f33, SUC_lexicon_tags_47 },
    { 0x73931f32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d31f39, SUC_lexicon_tags_25 },
    { 0xb7331f3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f131f40, SUC_lexicon_tags_1 },
    { 0x39931f40, SUC_lexicon_tags_1 },
    { 0x35531f42, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b31f47, SUC_lexicon_tags_7 },
    { 0x7e531f47, SUC_lexicon_tags_60 },
    { 0x4ed31f49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37331f4e, SUC_lexicon_tags_534 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb31f55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11131f57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f931f59, SUC_lexicon_tags_37 },
    { 0x3e531f59, SUC_lexicon_tags_26 },
    { 0xa0531f5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a131f62, SUC_lexicon_tags_60 },
    { 0x8a331f63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52731f65, SUC_lexicon_tags_25 },
    { 0x1b731f66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c331f68, SUC_lexicon_tags_37 },
    { 0x25f31f69, SUC_lexicon_tags_11 },
    { 0x80b31f68, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x13331f6d, SUC_lexicon_tags_23 },
    { 0x2c531f6e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf6131f70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0731f75, SUC_lexicon_tags_48 },
    { 0xa4931f76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf31f79, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34131f7f, SUC_lexicon_tags_57 },
    { 0xe7931f80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3731f83, SUC_lexicon_tags_15 },
    { 0xcff31f83, SUC_lexicon_tags_63 },
    { 0x97531f84, SUC_lexicon_tags_12 },
    { 0xec131f86, SUC_lexicon_tags_25 },
    { 0xff31f84, SUC_lexicon_tags_33 },
    { 0xeb531f83, SUC_lexicon_tags_37 },
    { 0xc8331f89, SUC_lexicon_tags_17 },
    { 0xf9331f8a, SUC_lexicon_tags_56 },
    { 0xb5131f8b, SUC_lexicon_tags_27 },
    { 0xd4131f8c, SUC_lexicon_tags_34 },
    { 0xcc931f8b, SUC_lexicon_tags_1 },
    { 0xfe531f8e, SUC_lexicon_tags_37 },
    { 0x8ef31f8b, SUC_lexicon_tags_27 },
    { 0x54131f90, SUC_lexicon_tags_9 },
    { 0xae931f90, SUC_lexicon_tags_69 },
    { 0xbbf31f88, SUC_lexicon_tags_9 },
    { 0x77731f8d, SUC_lexicon_tags_6 },
    { 0x50931f94, SUC_lexicon_tags_47 },
    { 0x6db31f95, SUC_lexicon_tags_2 },
    { 0x7d531f95, SUC_lexicon_tags_31 },
    { 0xbb731f97, SUC_lexicon_tags_7 },
    { 0xf4131f95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x17131f9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4d131f9d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72931f9f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xed131fa1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3731faa, SUC_lexicon_tags_23 },
    { 0xaf731fab, SUC_lexicon_tags_1 },
    { 0x40b31fab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd31faf, SUC_lexicon_tags_7 },
    { 0xa2331faf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6b31fb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e131fb8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57331fbc, SUC_lexicon_tags_25 },
    { 0x97331fbd, SUC_lexicon_tags_25 },
    { 0x68d31fbe, SUC_lexicon_tags_0 },
    { 0x68131fbc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e531fc2, SUC_lexicon_tags_27 },
    { 0x18331fc3, SUC_lexicon_tags_17 },
    { 0xea331fc4, SUC_lexicon_tags_28 },
    { 0x58331fc5, SUC_lexicon_tags_2 },
    { 0xc6f31fc6, SUC_lexicon_tags_25 },
    { 0xb0731fc7, SUC_lexicon_tags_19 },
    { 0x66b31fc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40531fcf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x42d31fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73d31fd3, SUC_lexicon_tags_56 },
    { 0x2f731fd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66531fd9, SUC_lexicon_tags_48 },
    { 0x5531fda, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb31fdd, SUC_lexicon_tags_14 },
    { 0x99931fde, SUC_lexicon_tags_12 },
    { 0x31531fde, SUC_lexicon_tags_37 },
    { 0x77331fe0, SUC_lexicon_tags_1 },
    { 0x64531fe1, SUC_lexicon_tags_11 },
    { 0xfd931fe2, SUC_lexicon_tags_43 },
    { 0x98531fe0, SUC_lexicon_tags_7 },
    { 0x3fb31fe4, SUC_lexicon_tags_17 },
    { 0x99931fe5, SUC_lexicon_tags_1 },
    { 0xd9331fde, SUC_lexicon_tags_45 },
    { 0x62131fdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e931fe9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b531fec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4531fef, SUC_lexicon_tags_15 },
    { 0x8a531fef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6331ff2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d31ff6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7ab31ff8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2131ffa, SUC_lexicon_tags_60 },
    { 0x55131ffb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40f31ffd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9731fff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf32004, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52132006, SUC_lexicon_tags_15 },
    { 0xf32007, SUC_lexicon_tags_17 },
    { 0x7e732008, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x66b3200a, SUC_lexicon_tags_30 },
    { 0xc173200b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81b3200d, SUC_lexicon_tags_0 },
    { 0x5573200e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f32011, SUC_lexicon_tags_19 },
    { 0xc0732012, SUC_lexicon_tags_6 },
    { 0x6af32013, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4132019, SUC_lexicon_tags_1 },
    { 0xd6532019, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe913201f, SUC_lexicon_tags_6 },
    { 0xebd3201f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe132022, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbdb32024, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1f32026, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d33202d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b332030, SUC_lexicon_tags_27 },
    { 0x7cb32031, SUC_lexicon_tags_25 },
    { 0xe7d32030, SUC_lexicon_tags_16 },
    { 0x2dd32033, SUC_lexicon_tags_46 },
    { 0xe8332034, SUC_lexicon_tags_40 },
    { 0x14332035, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d32038, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b3203d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfbf3203f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd132041, SUC_lexicon_tags_1 },
    { 0x24932042, SUC_lexicon_tags_2 },
    { 0xb1b32043, SUC_lexicon_tags_60 },
    { 0xe1132044, SUC_lexicon_tags_63 },
    { 0xb3732041, SUC_lexicon_tags_17 },
    { 0xe5532044, SUC_lexicon_tags_19 },
    { 0x1cd32047, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92b32049, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d13204d, SUC_lexicon_tags_8 },
    { 0xc853204d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a332052, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc332056, SUC_lexicon_tags_25 },
    { 0x39932057, SUC_lexicon_tags_1 },
    { 0xfa132056, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x64f3205a, SUC_lexicon_tags_68 },
    { 0x1453205b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d32060, SUC_lexicon_tags_27 },
    { 0xd0332061, SUC_lexicon_tags_26 },
    { 0x85932062, SUC_lexicon_tags_1 },
    { 0xe9d32062, SUC_lexicon_tags_12 },
    { 0x71332061, SUC_lexicon_tags_33 },
    { 0xfa732060, SUC_lexicon_tags_2 },
    { 0xe3132066, SUC_lexicon_tags_61 },
    { 0xd2332063, SUC_lexicon_tags_26 },
    { 0x1ed32068, SUC_lexicon_tags_27 },
    { 0xc2532060, SUC_lexicon_tags_11 },
    { 0x8173206a, SUC_lexicon_tags_23 },
    { 0x15f3206b, SUC_lexicon_tags_45 },
    { 0xb0532063, SUC_lexicon_tags_57 },
    { 0x8bb3206d, SUC_lexicon_tags_9 },
    { 0xe5d3206b, SUC_lexicon_tags_37 },
    { 0xf53206d, SUC_lexicon_tags_4 },
    { 0xfbf3206b, SUC_lexicon_tags_12 },
    { 0x2f732071, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x9c132073, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3932075, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfd732077, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c73207c, SUC_lexicon_tags_7 },
    { 0xa853207d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab32080, SUC_lexicon_tags_7 },
    { 0x96b32080, SUC_lexicon_tags_53 },
    { 0x4ad32080, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc132084, SUC_lexicon_tags_0 },
    { 0x58732084, SUC_lexicon_tags_47 },
    { 0x1f532086, SUC_lexicon_tags_49 },
    { 0xeb932087, SUC_lexicon_tags_17 },
    { 0x4332088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d93208b, SUC_lexicon_tags_26 },
    { 0x2db3208c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc732092, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70132097, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b3209a, SUC_lexicon_tags_15 },
    { 0x3653209a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3b3209d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673320a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa51320a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3320a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3d320a8, SUC_lexicon_tags_2 },
    { 0x6d3320a9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8a9320ab, SUC_lexicon_tags_11 },
    { 0x539320ab, SUC_lexicon_tags_47 },
    { 0xeb9320ad, SUC_lexicon_tags_29 },
    { 0x2a3320ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f320b4, SUC_lexicon_tags_7 },
    { 0xa5b320b4, SUC_lexicon_tags_1 },
    { 0x173320b4, SUC_lexicon_tags_84 },
    { 0xce7320b6, SUC_lexicon_tags_11 },
    { 0x187320b8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45320bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7320c0, SUC_lexicon_tags_37 },
    { 0x167320c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9320c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcb320c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x295320c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713320cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1320d0, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x22b320d2, SUC_lexicon_tags_37 },
    { 0xbd1320d3, SUC_lexicon_tags_45 },
    { 0x199320d2, SUC_lexicon_tags_11 },
    { 0xa65320d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803320de, SUC_lexicon_tags_14 },
    { 0xa2b320df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5f320e1, SUC_lexicon_tags_47 },
    { 0xce7320e2, SUC_lexicon_tags_37 },
    { 0xcbf320e3, SUC_lexicon_tags_7 },
    { 0x7df320e4, SUC_lexicon_tags_87 },
    { 0xf9320e1, SUC_lexicon_tags_30 },
    { 0x77d320e3, SUC_lexicon_tags_87 },
    { 0xa03320e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db320eb, SUC_lexicon_tags_37 },
    { 0x95b320ec, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9320f1, SUC_lexicon_tags_2 },
    { 0xfdf320f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bb320f4, SUC_lexicon_tags_7 },
    { 0x725320f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf320f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29320fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b320fd, SUC_lexicon_tags_63 },
    { 0xb9320fd, SUC_lexicon_tags_65 },
    { 0xbfb320ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70d32101, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x17332104, SUC_lexicon_tags_47 },
    { 0x97332105, SUC_lexicon_tags_19 },
    { 0x27332106, SUC_lexicon_tags_1 },
    { 0x3db32107, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd293210b, SUC_lexicon_tags_29 },
    { 0xf833210c, SUC_lexicon_tags_21 },
    { 0x7ad3210d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xc733210f, SUC_lexicon_tags_359 },
    { 0x55532110, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65332122, SUC_lexicon_tags_25 },
    { 0x75932123, SUC_lexicon_tags_19 },
    { 0x5e532124, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff3212b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d3212e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4353213a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6b3213c, SUC_lexicon_tags_647 },
    { 0, NULL },
    { 0x2273213e, SUC_lexicon_tags_37 },
    { 0x95f3213e, SUC_lexicon_tags_4 },
    { 0x7d93213f, SUC_lexicon_tags_12 },
    { 0x4493213e, SUC_lexicon_tags_29 },
    { 0x54b32142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f3214a, SUC_lexicon_tags_49 },
    { 0xef33214b, SUC_lexicon_tags_18 },
    { 0x7eb3214c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f3214f, SUC_lexicon_tags_66 },
    { 0x9e73214f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87732153, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b32157, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x37732159, SUC_lexicon_tags_7 },
    { 0x1433215a, SUC_lexicon_tags_37 },
    { 0x68732159, SUC_lexicon_tags_39 },
    { 0x90f3215c, SUC_lexicon_tags_23 },
    { 0x4653215d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x29d3215f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d32163, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a732165, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a532168, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3913216a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2333216d, SUC_lexicon_tags_37 },
    { 0xb853216d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b32172, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78b32174, SUC_lexicon_tags_1 },
    { 0x83132174, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3153217c, SUC_lexicon_tags_37 },
    { 0x5b3217d, SUC_lexicon_tags_38 },
    { 0x94f3217d, SUC_lexicon_tags_7 },
    { 0xde13217e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79b32181, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd32185, SUC_lexicon_tags_51 },
    { 0x14332186, SUC_lexicon_tags_33 },
    { 0x9dd32187, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad932189, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f3218d, SUC_lexicon_tags_25 },
    { 0xccd3218e, SUC_lexicon_tags_1 },
    { 0xe4b3218e, SUC_lexicon_tags_165 },
    { 0xa1932190, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2932192, SUC_lexicon_tags_56 },
    { 0xf3f32192, SUC_lexicon_tags_58 },
    { 0xcd132193, SUC_lexicon_tags_17 },
    { 0xe2732195, SUC_lexicon_tags_34 },
    { 0x2932195, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51532198, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4553219a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763321a1, SUC_lexicon_tags_25 },
    { 0xb3b321a2, SUC_lexicon_tags_6 },
    { 0x9c5321a3, SUC_lexicon_tags_17 },
    { 0xd41321a4, SUC_lexicon_tags_1 },
    { 0x9e1321a1, SUC_lexicon_tags_45 },
    { 0x80b321a6, SUC_lexicon_tags_19 },
    { 0x515321a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5321aa, SUC_lexicon_tags_7 },
    { 0x77b321ab, SUC_lexicon_tags_40 },
    { 0xcc7321ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd17321ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d321b1, SUC_lexicon_tags_15 },
    { 0x889321b1, SUC_lexicon_tags_37 },
    { 0x9d3321b3, SUC_lexicon_tags_43 },
    { 0x3c5321b4, SUC_lexicon_tags_25 },
    { 0x7b321b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd321bb, SUC_lexicon_tags_25 },
    { 0xf09321bc, SUC_lexicon_tags_33 },
    { 0xe35321bc, SUC_lexicon_tags_3 },
    { 0x1e9321be, SUC_lexicon_tags_9 },
    { 0x3cf321bf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7321c3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe99321c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df321c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d321d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d321d9, SUC_lexicon_tags_1 },
    { 0x5fd321da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b321dd, SUC_lexicon_tags_4 },
    { 0x7a1321de, SUC_lexicon_tags_12 },
    { 0x4c9321de, SUC_lexicon_tags_33 },
    { 0x275321e0, SUC_lexicon_tags_23 },
    { 0x955321e0, SUC_lexicon_tags_31 },
    { 0x2cf321df, SUC_lexicon_tags_8 },
    { 0x823321e3, SUC_lexicon_tags_17 },
    { 0x6c1321e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30f321e6, SUC_lexicon_tags_17 },
    { 0x745321e7, SUC_lexicon_tags_37 },
    { 0x2c5321e8, SUC_lexicon_tags_70 },
    { 0xa91321e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf321f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9321fe, SUC_lexicon_tags_30 },
    { 0x689321ff, SUC_lexicon_tags_2 },
    { 0x9f1321ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2332202, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d532207, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1833220e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87332210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f32217, SUC_lexicon_tags_12 },
    { 0x45d32217, SUC_lexicon_tags_38 },
    { 0x21332219, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b3221c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7f3221e, SUC_lexicon_tags_43 },
    { 0x2c53221f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6932222, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e732226, SUC_lexicon_tags_37 },
    { 0xa532226, SUC_lexicon_tags_11 },
    { 0x66132228, SUC_lexicon_tags_12 },
    { 0xd4132226, SUC_lexicon_tags_15 },
    { 0x78332228, SUC_lexicon_tags_56 },
    { 0x2613222b, SUC_lexicon_tags_27 },
    { 0x3af3222c, SUC_lexicon_tags_62 },
    { 0xb7f3222d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a732232, SUC_lexicon_tags_29 },
    { 0x2c132233, SUC_lexicon_tags_82 },
    { 0xad132234, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3e732236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7332238, SUC_lexicon_tags_18 },
    { 0x27732239, SUC_lexicon_tags_2 },
    { 0x77132238, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb653223f, SUC_lexicon_tags_0 },
    { 0x28f3223f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f32244, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4732249, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb3224e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e732252, SUC_lexicon_tags_77 },
    { 0x41532253, SUC_lexicon_tags_40 },
    { 0xefb32252, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e532259, SUC_lexicon_tags_11 },
    { 0x14532259, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecd3225c, SUC_lexicon_tags_46 },
    { 0x3113225d, SUC_lexicon_tags_19 },
    { 0x3df3225e, SUC_lexicon_tags_43 },
    { 0x1273225f, SUC_lexicon_tags_19 },
    { 0x4573225e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e132265, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5193226f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e132272, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x70532275, SUC_lexicon_tags_12 },
    { 0xdd932275, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x30532279, SUC_lexicon_tags_43 },
    { 0xba93227a, SUC_lexicon_tags_7 },
    { 0x2493227b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4b3227d, SUC_lexicon_tags_37 },
    { 0xce33227e, SUC_lexicon_tags_17 },
    { 0xd0f3227f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f32282, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd32286, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x24f32288, SUC_lexicon_tags_11 },
    { 0x6f532288, SUC_lexicon_tags_66 },
    { 0x96d32288, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e73228d, SUC_lexicon_tags_25 },
    { 0x83f3228d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d32291, SUC_lexicon_tags_19 },
    { 0x78b32291, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x71532294, SUC_lexicon_tags_25 },
    { 0x44732295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b32299, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8e53229b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58d3229d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f322a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17322a4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d322a8, SUC_lexicon_tags_9 },
    { 0xf3322a8, SUC_lexicon_tags_33 },
    { 0x52b322a8, SUC_lexicon_tags_30 },
    { 0x3b3322ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f322ad, SUC_lexicon_tags_30 },
    { 0xb95322ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b322b1, SUC_lexicon_tags_1 },
    { 0x301322b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad322b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b3322b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed322bc, SUC_lexicon_tags_41 },
    { 0xe2d322bd, SUC_lexicon_tags_1 },
    { 0x4b1322be, SUC_lexicon_tags_47 },
    { 0x227322bc, SUC_lexicon_tags_402 },
    { 0x6d3322c0, SUC_lexicon_tags_1 },
    { 0xab322c1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x20d322c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed322c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e3322c9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65322cd, SUC_lexicon_tags_33 },
    { 0x175322cd, SUC_lexicon_tags_31 },
    { 0x889322cf, SUC_lexicon_tags_15 },
    { 0x457322d0, SUC_lexicon_tags_28 },
    { 0x4d3322d1, SUC_lexicon_tags_2 },
    { 0x165322d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb47322d4, SUC_lexicon_tags_27 },
    { 0x4a9322d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181322d9, SUC_lexicon_tags_14 },
    { 0x573322da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959322e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69322e3, SUC_lexicon_tags_37 },
    { 0xddd322e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf01322e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607322ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121322f2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519322f7, SUC_lexicon_tags_1 },
    { 0xe2b322f8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b322fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x337322ff, SUC_lexicon_tags_7 },
    { 0x35532300, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ad32302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8932307, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd33230c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed93230f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a132311, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd32314, SUC_lexicon_tags_15 },
    { 0x84b32315, SUC_lexicon_tags_37 },
    { 0x7b532316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed32318, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1193231a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3393231c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a13231e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d32322, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d32326, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f3232b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f3232e, SUC_lexicon_tags_6 },
    { 0xb973232f, SUC_lexicon_tags_10 },
    { 0x9dd32330, SUC_lexicon_tags_30 },
    { 0x48732331, SUC_lexicon_tags_25 },
    { 0x83532331, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e132335, SUC_lexicon_tags_47 },
    { 0x5cb32336, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33233b, SUC_lexicon_tags_17 },
    { 0xe913233b, SUC_lexicon_tags_8 },
    { 0xca93233d, SUC_lexicon_tags_8 },
    { 0x7c73233e, SUC_lexicon_tags_15 },
    { 0xb7f3233f, SUC_lexicon_tags_15 },
    { 0xaf33233f, SUC_lexicon_tags_12 },
    { 0x8913233f, SUC_lexicon_tags_25 },
    { 0xa0b32342, SUC_lexicon_tags_11 },
    { 0x753233c, SUC_lexicon_tags_316 },
    { 0x2bb32344, SUC_lexicon_tags_25 },
    { 0xbe532342, SUC_lexicon_tags_12 },
    { 0x3ff32346, SUC_lexicon_tags_46 },
    { 0x8fb32346, SUC_lexicon_tags_25 },
    { 0x83d32345, SUC_lexicon_tags_12 },
    { 0xce132342, SUC_lexicon_tags_25 },
    { 0x1a93234a, SUC_lexicon_tags_7 },
    { 0x8ab3233e, SUC_lexicon_tags_7 },
    { 0x2e132347, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5673234f, SUC_lexicon_tags_0 },
    { 0xd5b32350, SUC_lexicon_tags_4 },
    { 0xeb732351, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8732353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6073235a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b3235e, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x81732360, SUC_lexicon_tags_63 },
    { 0x34532361, SUC_lexicon_tags_19 },
    { 0x2bf32362, SUC_lexicon_tags_113 },
    { 0x9c732363, SUC_lexicon_tags_63 },
    { 0x44732364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d132366, SUC_lexicon_tags_43 },
    { 0x66b32367, SUC_lexicon_tags_0 },
    { 0xeeb32366, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3236d, SUC_lexicon_tags_7 },
    { 0x89f3236e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f32371, SUC_lexicon_tags_1216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d32375, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b32378, SUC_lexicon_tags_1 },
    { 0xb1932379, SUC_lexicon_tags_33 },
    { 0x36b3237a, SUC_lexicon_tags_37 },
    { 0x5933237a, SUC_lexicon_tags_29 },
    { 0x19132379, SUC_lexicon_tags_14 },
    { 0x7f13237d, SUC_lexicon_tags_11 },
    { 0x12b3237d, SUC_lexicon_tags_3 },
    { 0x2f32379, SUC_lexicon_tags_13 },
    { 0x3c73237d, SUC_lexicon_tags_46 },
    { 0xa5132381, SUC_lexicon_tags_11 },
    { 0xab532382, SUC_lexicon_tags_12 },
    { 0x9773237e, SUC_lexicon_tags_37 },
    { 0x15b32384, SUC_lexicon_tags_160 },
    { 0xf8d32383, SUC_lexicon_tags_55 },
    { 0xacb32386, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9932388, SUC_lexicon_tags_37 },
    { 0xcc132389, SUC_lexicon_tags_15 },
    { 0x173238a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb5d3238c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f32391, SUC_lexicon_tags_49 },
    { 0x91b32392, SUC_lexicon_tags_13 },
    { 0x3cd32391, SUC_lexicon_tags_15 },
    { 0x1d932393, SUC_lexicon_tags_11 },
    { 0xc732395, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4ef32397, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2b32399, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e33239d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d323a0, SUC_lexicon_tags_28 },
    { 0xd6f323a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x743323a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8d323a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf39323a8, SUC_lexicon_tags_46 },
    { 0x4bf323a8, SUC_lexicon_tags_7 },
    { 0xbdf323a8, SUC_lexicon_tags_7 },
    { 0x947323ab, SUC_lexicon_tags_16 },
    { 0x1d1323ac, SUC_lexicon_tags_1 },
    { 0x923323ab, SUC_lexicon_tags_17 },
    { 0xb9b323ae, SUC_lexicon_tags_37 },
    { 0x587323af, SUC_lexicon_tags_17 },
    { 0xbb7323b0, SUC_lexicon_tags_25 },
    { 0x8cd323b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd323b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6bf323ba, SUC_lexicon_tags_27 },
    { 0x167323ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd79323bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7db323bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf323c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25323cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf323d4, SUC_lexicon_tags_1 },
    { 0x78b323d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3323d9, SUC_lexicon_tags_17 },
    { 0xde5323da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f323e2, SUC_lexicon_tags_37 },
    { 0x74d323e3, SUC_lexicon_tags_37 },
    { 0xbe7323e3, SUC_lexicon_tags_25 },
    { 0x39b323e3, SUC_lexicon_tags_12 },
    { 0x143323e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a5323e8, SUC_lexicon_tags_7 },
    { 0x44d323e9, SUC_lexicon_tags_21 },
    { 0xd1323e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea9323ec, SUC_lexicon_tags_63 },
    { 0xf4d323ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3323f1, SUC_lexicon_tags_885 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b323f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcb323f6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x961323f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23323fd, SUC_lexicon_tags_2 },
    { 0xaf323fe, SUC_lexicon_tags_14 },
    { 0xe33323ff, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x20b32401, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x60f32403, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5c332405, SUC_lexicon_tags_12 },
    { 0x73f32405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b32409, SUC_lexicon_tags_1 },
    { 0x1a13240a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8ab3240c, SUC_lexicon_tags_8 },
    { 0xe6f3240c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9d3240f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e532417, SUC_lexicon_tags_69 },
    { 0xe8932417, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43532420, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f32425, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42132427, SUC_lexicon_tags_25 },
    { 0xa1932428, SUC_lexicon_tags_84 },
    { 0x7332427, SUC_lexicon_tags_17 },
    { 0x6b13242a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1493242d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef32432, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d32436, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd1d32438, SUC_lexicon_tags_14 },
    { 0x1ef32438, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd32442, SUC_lexicon_tags_26 },
    { 0x76732443, SUC_lexicon_tags_11 },
    { 0x45332444, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdd332446, SUC_lexicon_tags_11 },
    { 0xf0f32447, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b3244a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbe33244c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b932452, SUC_lexicon_tags_18 },
    { 0x6f932453, SUC_lexicon_tags_62 },
    { 0x5e932454, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2f132456, SUC_lexicon_tags_38 },
    { 0xfd532457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d3245a, SUC_lexicon_tags_7 },
    { 0xf073245b, SUC_lexicon_tags_12 },
    { 0x3313245c, SUC_lexicon_tags_37 },
    { 0xcbf3245d, SUC_lexicon_tags_12 },
    { 0xb773245a, SUC_lexicon_tags_2 },
    { 0x8653245f, SUC_lexicon_tags_21 },
    { 0xf5d32460, SUC_lexicon_tags_138 },
    { 0xd9f32461, SUC_lexicon_tags_17 },
    { 0x3f932462, SUC_lexicon_tags_15 },
    { 0xb53245d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6d32465, SUC_lexicon_tags_17 },
    { 0x54332466, SUC_lexicon_tags_12 },
    { 0xc5f32467, SUC_lexicon_tags_37 },
    { 0xf8132468, SUC_lexicon_tags_25 },
    { 0xa5932469, SUC_lexicon_tags_2 },
    { 0x33b32468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe993246d, SUC_lexicon_tags_25 },
    { 0xbf53246d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6532473, SUC_lexicon_tags_341 },
    { 0x94732474, SUC_lexicon_tags_0 },
    { 0x30132475, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc393247b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d3247e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa532480, SUC_lexicon_tags_70 },
    { 0xab732481, SUC_lexicon_tags_11 },
    { 0xfdb32482, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb32487, SUC_lexicon_tags_2 },
    { 0xafb32488, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff32492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f932498, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd3249c, SUC_lexicon_tags_17 },
    { 0x9f93249c, SUC_lexicon_tags_33 },
    { 0xb93249d, SUC_lexicon_tags_21 },
    { 0xab53249f, SUC_lexicon_tags_7 },
    { 0x413324a0, SUC_lexicon_tags_37 },
    { 0xc413249e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x171324a3, SUC_lexicon_tags_33 },
    { 0xdd9324a3, SUC_lexicon_tags_6 },
    { 0x3f9324a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f324a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbd3324aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaa3324ac, SUC_lexicon_tags_11 },
    { 0xa8f324ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693324b1, SUC_lexicon_tags_19 },
    { 0x6d5324b2, SUC_lexicon_tags_11 },
    { 0xa01324b3, SUC_lexicon_tags_17 },
    { 0x74f324b4, SUC_lexicon_tags_25 },
    { 0x7f3324b5, SUC_lexicon_tags_17 },
    { 0x249324b5, SUC_lexicon_tags_27 },
    { 0x9f7324b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7324ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9db324bc, SUC_lexicon_tags_47 },
    { 0x32f324bd, SUC_lexicon_tags_31 },
    { 0x5a9324bd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79324c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba3324c6, SUC_lexicon_tags_87 },
    { 0xcdb324c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7db324c9, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab324cd, SUC_lexicon_tags_15 },
    { 0xea5324ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1324d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x529324d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79324d8, SUC_lexicon_tags_2 },
    { 0x425324d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x931324dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d324e2, SUC_lexicon_tags_11 },
    { 0x69d324e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19324e7, SUC_lexicon_tags_45 },
    { 0x1bb324e7, SUC_lexicon_tags_48 },
    { 0xe87324e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0f324eb, SUC_lexicon_tags_27 },
    { 0x433324eb, SUC_lexicon_tags_18 },
    { 0x659324ed, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3cf324ef, SUC_lexicon_tags_15 },
    { 0xb17324f0, SUC_lexicon_tags_0 },
    { 0xa9f324ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd324f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b324f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d324fc, SUC_lexicon_tags_11 },
    { 0x49324fd, SUC_lexicon_tags_25 },
    { 0xc2b324fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b32506, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8af32508, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd53250b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4253250d, SUC_lexicon_tags_151 },
    { 0x6db3250e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2932511, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f32517, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b3251d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xed3251f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1732523, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac132529, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf3252d, SUC_lexicon_tags_44 },
    { 0x9973252e, SUC_lexicon_tags_66 },
    { 0x7613252d, SUC_lexicon_tags_25 },
    { 0x37d32530, SUC_lexicon_tags_47 },
    { 0x32332530, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d32534, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed132539, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6373253b, SUC_lexicon_tags_25 },
    { 0x7533253c, SUC_lexicon_tags_21 },
    { 0x1f73253d, SUC_lexicon_tags_17 },
    { 0xd13253e, SUC_lexicon_tags_17 },
    { 0x61b3253f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff32542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x92332545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f3254a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a93254c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d32550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x38932553, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a532557, SUC_lexicon_tags_11 },
    { 0x78d32557, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x433255b, SUC_lexicon_tags_14 },
    { 0xb053255b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d32561, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42732563, SUC_lexicon_tags_106 },
    { 0x76332563, SUC_lexicon_tags_14 },
    { 0x84332565, SUC_lexicon_tags_5 },
    { 0x42b32566, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a33256a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14332576, SUC_lexicon_tags_15 },
    { 0x2a932576, SUC_lexicon_tags_30 },
    { 0xa8932578, SUC_lexicon_tags_42 },
    { 0xb6532579, SUC_lexicon_tags_7 },
    { 0xc6b3257a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e932580, SUC_lexicon_tags_7 },
    { 0xbeb32581, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a132587, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf893258b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6533258f, SUC_lexicon_tags_12 },
    { 0xb73258f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1532592, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x11132595, SUC_lexicon_tags_25 },
    { 0xf4f32596, SUC_lexicon_tags_61 },
    { 0x9e132596, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7673259a, SUC_lexicon_tags_0 },
    { 0x44f3259a, SUC_lexicon_tags_85 },
    { 0xe1d3259c, SUC_lexicon_tags_9 },
    { 0xd0b3259a, SUC_lexicon_tags_185 },
    { 0x7ed3259d, SUC_lexicon_tags_45 },
    { 0x4553259f, SUC_lexicon_tags_87 },
    { 0x55d325a0, SUC_lexicon_tags_26 },
    { 0xfb3259a, SUC_lexicon_tags_113 },
    { 0x8c9325a2, SUC_lexicon_tags_32 },
    { 0xa29325a3, SUC_lexicon_tags_2 },
    { 0x30d325a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb325a7, SUC_lexicon_tags_19 },
    { 0xbed325a8, SUC_lexicon_tags_17 },
    { 0x69b325a8, SUC_lexicon_tags_25 },
    { 0x825325a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f325af, SUC_lexicon_tags_8 },
    { 0x8d5325b0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x795325b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d5325b5, SUC_lexicon_tags_13 },
    { 0x92b325b6, SUC_lexicon_tags_83 },
    { 0x175325b7, SUC_lexicon_tags_17 },
    { 0x927325b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x453325ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b325bf, SUC_lexicon_tags_11 },
    { 0x38f325c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x27325c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d325c6, SUC_lexicon_tags_55 },
    { 0xb43325c7, SUC_lexicon_tags_29 },
    { 0x385325c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ed325ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7325d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe53325d2, SUC_lexicon_tags_7 },
    { 0x1cd325d3, SUC_lexicon_tags_56 },
    { 0xc5b325d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1325dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7325e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d1325e4, SUC_lexicon_tags_42 },
    { 0x68d325e5, SUC_lexicon_tags_11 },
    { 0xcb5325e5, SUC_lexicon_tags_29 },
    { 0x61d325e5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x85f325e9, SUC_lexicon_tags_2 },
    { 0xa21325ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303325f5, SUC_lexicon_tags_1 },
    { 0x397325f6, SUC_lexicon_tags_15 },
    { 0x845325f5, SUC_lexicon_tags_25 },
    { 0x72b325f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db325fb, SUC_lexicon_tags_28 },
    { 0x135325fb, SUC_lexicon_tags_21 },
    { 0x6f325fd, SUC_lexicon_tags_17 },
    { 0x3b7325fc, SUC_lexicon_tags_37 },
    { 0xbb5325fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d32603, SUC_lexicon_tags_16 },
    { 0x52732604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0932607, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad3260b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3260e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x71f32610, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd532613, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f32616, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab3261d, SUC_lexicon_tags_2 },
    { 0xae53261e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b732622, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9532627, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b3262c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7df3262e, SUC_lexicon_tags_33 },
    { 0x8053262f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfe132631, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5532633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7d32635, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf32640, SUC_lexicon_tags_48 },
    { 0x732641, SUC_lexicon_tags_47 },
    { 0xf2f32641, SUC_lexicon_tags_11 },
    { 0xa7732641, SUC_lexicon_tags_0 },
    { 0x77d32641, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7132646, SUC_lexicon_tags_11 },
    { 0x42932647, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3013264a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b32650, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f32654, SUC_lexicon_tags_43 },
    { 0xd2b32655, SUC_lexicon_tags_7 },
    { 0x7cd32655, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f3265b, SUC_lexicon_tags_49 },
    { 0x36f3265b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7473265f, SUC_lexicon_tags_17 },
    { 0x4a132660, SUC_lexicon_tags_11 },
    { 0x1ff3265f, SUC_lexicon_tags_62 },
    { 0x5ab32662, SUC_lexicon_tags_11 },
    { 0x6633265f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6732666, SUC_lexicon_tags_1 },
    { 0xeb132667, SUC_lexicon_tags_7 },
    { 0x22d32667, SUC_lexicon_tags_37 },
    { 0x26b32669, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b33266e, SUC_lexicon_tags_23 },
    { 0xc133266e, SUC_lexicon_tags_39 },
    { 0xbbf32670, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4932674, SUC_lexicon_tags_9 },
    { 0x6fb32675, SUC_lexicon_tags_45 },
    { 0x41132674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16332678, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473267e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x41732681, SUC_lexicon_tags_60 },
    { 0xf0132682, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x2df32684, SUC_lexicon_tags_0 },
    { 0xbef32685, SUC_lexicon_tags_17 },
    { 0x6532684, SUC_lexicon_tags_6 },
    { 0xb8b32687, SUC_lexicon_tags_29 },
    { 0xd9132688, SUC_lexicon_tags_33 },
    { 0xa2b32684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6b3268b, SUC_lexicon_tags_19 },
    { 0x38b3268b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8873268f, SUC_lexicon_tags_7 },
    { 0x24532690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4932694, SUC_lexicon_tags_26 },
    { 0x34332695, SUC_lexicon_tags_6 },
    { 0xc9d32696, SUC_lexicon_tags_27 },
    { 0xc2532697, SUC_lexicon_tags_17 },
    { 0x21f32695, SUC_lexicon_tags_14 },
    { 0x7fb32699, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899326a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7326a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b326a8, SUC_lexicon_tags_7 },
    { 0xe9b326a8, SUC_lexicon_tags_63 },
    { 0xd33326aa, SUC_lexicon_tags_16 },
    { 0xbd3326a9, SUC_lexicon_tags_43 },
    { 0xbc3326ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a1326ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f326b4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x369326b7, SUC_lexicon_tags_21 },
    { 0x3c1326b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9326bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9326c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x197326c2, SUC_lexicon_tags_25 },
    { 0x3bf326c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x211326c6, SUC_lexicon_tags_35 },
    { 0xc79326c7, SUC_lexicon_tags_23 },
    { 0x6dd326c8, SUC_lexicon_tags_48 },
    { 0x40b326c8, SUC_lexicon_tags_64 },
    { 0x47d326ca, SUC_lexicon_tags_14 },
    { 0xbb9326c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6f326cd, SUC_lexicon_tags_17 },
    { 0x819326ce, SUC_lexicon_tags_17 },
    { 0xb57326cf, SUC_lexicon_tags_15 },
    { 0x6f5326d0, SUC_lexicon_tags_8 },
    { 0xedb326cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b326d6, SUC_lexicon_tags_62 },
    { 0xe05326d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9326de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7326e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa33326e3, SUC_lexicon_tags_23 },
    { 0x63b326e4, SUC_lexicon_tags_17 },
    { 0x413326e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb45326e7, SUC_lexicon_tags_17 },
    { 0x595326e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x391326ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891326ee, SUC_lexicon_tags_37 },
    { 0xdd7326ee, SUC_lexicon_tags_15 },
    { 0x375326f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b326f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbff326f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd3326f7, SUC_lexicon_tags_14 },
    { 0xd53326f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665326fe, SUC_lexicon_tags_1008 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4332703, SUC_lexicon_tags_39 },
    { 0x6cf32704, SUC_lexicon_tags_23 },
    { 0x7a532704, SUC_lexicon_tags_48 },
    { 0xdfd32706, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8f32708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8993270c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce332713, SUC_lexicon_tags_23 },
    { 0xa9132714, SUC_lexicon_tags_2 },
    { 0xdb32713, SUC_lexicon_tags_61 },
    { 0xf7732716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ed32718, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4433271c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe673271f, SUC_lexicon_tags_15 },
    { 0xefb32720, SUC_lexicon_tags_2 },
    { 0x40932721, SUC_lexicon_tags_15 },
    { 0x8c332720, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x16932725, SUC_lexicon_tags_17 },
    { 0xe7132725, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4ef32728, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3213272a, SUC_lexicon_tags_2 },
    { 0xd553272b, SUC_lexicon_tags_25 },
    { 0x99f3272b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ff3272e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaeb32730, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xec932733, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9b132735, SUC_lexicon_tags_25 },
    { 0x58132736, SUC_lexicon_tags_45 },
    { 0xb1132737, SUC_lexicon_tags_2 },
    { 0x99532735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd732740, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b732747, SUC_lexicon_tags_7 },
    { 0x8b532747, SUC_lexicon_tags_35 },
    { 0x6d732749, SUC_lexicon_tags_14 },
    { 0x40132749, SUC_lexicon_tags_27 },
    { 0xd6b3274b, SUC_lexicon_tags_43 },
    { 0x54d3274b, SUC_lexicon_tags_17 },
    { 0x13274b, SUC_lexicon_tags_12 },
    { 0x49f3274c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x11532750, SUC_lexicon_tags_9 },
    { 0x20b32751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefb32753, SUC_lexicon_tags_35 },
    { 0x3fd32754, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x95732757, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf3275a, SUC_lexicon_tags_11 },
    { 0x5533275a, SUC_lexicon_tags_38 },
    { 0xbc93275c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3573275f, SUC_lexicon_tags_8 },
    { 0x3bf3275f, SUC_lexicon_tags_12 },
    { 0x52532760, SUC_lexicon_tags_21 },
    { 0x79d32762, SUC_lexicon_tags_80 },
    { 0x3a53275f, SUC_lexicon_tags_28 },
    { 0x8ef3275f, SUC_lexicon_tags_11 },
    { 0xae732762, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51132769, SUC_lexicon_tags_70 },
    { 0x2013276a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b3276d, SUC_lexicon_tags_17 },
    { 0x7d33276e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bf32770, SUC_lexicon_tags_11 },
    { 0x95d32770, SUC_lexicon_tags_33 },
    { 0x76532772, SUC_lexicon_tags_7 },
    { 0x7f932772, SUC_lexicon_tags_25 },
    { 0x58532774, SUC_lexicon_tags_17 },
    { 0x3fb32772, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d32778, SUC_lexicon_tags_25 },
    { 0x83d32779, SUC_lexicon_tags_60 },
    { 0xbdf32779, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5a53277c, SUC_lexicon_tags_82 },
    { 0x37f3277d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f32783, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x70932785, SUC_lexicon_tags_12 },
    { 0x60d32785, SUC_lexicon_tags_25 },
    { 0x56d32787, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b3278a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7533278c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4732791, SUC_lexicon_tags_23 },
    { 0xb8f32792, SUC_lexicon_tags_33 },
    { 0x4dd32793, SUC_lexicon_tags_68 },
    { 0x4cd32792, SUC_lexicon_tags_2 },
    { 0x57f32794, SUC_lexicon_tags_37 },
    { 0xe9732796, SUC_lexicon_tags_2 },
    { 0x2b32795, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9327a0, SUC_lexicon_tags_15 },
    { 0xa6d327a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e5327a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671327a8, SUC_lexicon_tags_15 },
    { 0x1b1327a9, SUC_lexicon_tags_19 },
    { 0xa73327a8, SUC_lexicon_tags_12 },
    { 0x93327a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb327b0, SUC_lexicon_tags_15 },
    { 0xeef327b1, SUC_lexicon_tags_10 },
    { 0x829327b2, SUC_lexicon_tags_12 },
    { 0xe99327b3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc8f327b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87327bb, SUC_lexicon_tags_12 },
    { 0x48f327bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf7327be, SUC_lexicon_tags_37 },
    { 0xb1d327bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13327c2, SUC_lexicon_tags_37 },
    { 0x9eb327c3, SUC_lexicon_tags_5 },
    { 0x66b327c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193327ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccd327cc, SUC_lexicon_tags_37 },
    { 0xaf7327cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed327d0, SUC_lexicon_tags_15 },
    { 0xceb327d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41327d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf327d6, SUC_lexicon_tags_25 },
    { 0xed9327d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x485327d9, SUC_lexicon_tags_64 },
    { 0x66b327da, SUC_lexicon_tags_49 },
    { 0x84b327da, SUC_lexicon_tags_1 },
    { 0x455327dc, SUC_lexicon_tags_27 },
    { 0xb65327d9, SUC_lexicon_tags_17 },
    { 0xf2f327dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x40b327e0, SUC_lexicon_tags_87 },
    { 0xcb1327e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x787327e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635327e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439327ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x803327ef, SUC_lexicon_tags_82 },
    { 0xe95327f0, SUC_lexicon_tags_1 },
    { 0x135327f0, SUC_lexicon_tags_62 },
    { 0x33f327ef, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925327f6, SUC_lexicon_tags_46 },
    { 0x461327f7, SUC_lexicon_tags_19 },
    { 0xbe9327f6, SUC_lexicon_tags_84 },
    { 0x2ad327f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f32801, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db32805, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57d32807, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd732809, SUC_lexicon_tags_25 },
    { 0xf9d32809, SUC_lexicon_tags_43 },
    { 0xe493280b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4a13280d, SUC_lexicon_tags_37 },
    { 0x70f3280e, SUC_lexicon_tags_11 },
    { 0x393280d, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a532817, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89732819, SUC_lexicon_tags_66 },
    { 0xf8d3281a, SUC_lexicon_tags_17 },
    { 0x61f3281b, SUC_lexicon_tags_60 },
    { 0x6353281b, SUC_lexicon_tags_37 },
    { 0xc2b3281a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa933281f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xed32822, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef32825, SUC_lexicon_tags_15 },
    { 0xbff32826, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd9532828, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb033282a, SUC_lexicon_tags_7 },
    { 0xa533282b, SUC_lexicon_tags_39 },
    { 0x2fd3282c, SUC_lexicon_tags_4 },
    { 0xd913282c, SUC_lexicon_tags_1 },
    { 0xcff3282d, SUC_lexicon_tags_17 },
    { 0x7273282a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18932831, SUC_lexicon_tags_16 },
    { 0x45d32831, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b32836, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x63132839, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9673283b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d33283d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57332842, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f132844, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85532848, SUC_lexicon_tags_33 },
    { 0x32532849, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd733284d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9cb3284f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95932852, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf532855, SUC_lexicon_tags_12 },
    { 0xa6d32856, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f32859, SUC_lexicon_tags_30 },
    { 0x9b53285a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c13285c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59132860, SUC_lexicon_tags_7 },
    { 0x6af32861, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b332863, SUC_lexicon_tags_21 },
    { 0xfd532864, SUC_lexicon_tags_68 },
    { 0xd9532865, SUC_lexicon_tags_37 },
    { 0x31732866, SUC_lexicon_tags_17 },
    { 0x2b532867, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3893286a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc253286d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55732873, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f3287d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db32881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf93288a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d732895, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6593289b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x393289d, SUC_lexicon_tags_17 },
    { 0xdcb3289e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3328a2, SUC_lexicon_tags_23 },
    { 0x569328a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485328a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x259328a9, SUC_lexicon_tags_43 },
    { 0x1db328a9, SUC_lexicon_tags_17 },
    { 0xea7328ab, SUC_lexicon_tags_10 },
    { 0x3cd328aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489328b1, SUC_lexicon_tags_12 },
    { 0xdc7328b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x305328b5, SUC_lexicon_tags_25 },
    { 0x1f9328b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9328ba, SUC_lexicon_tags_48 },
    { 0x97d328bb, SUC_lexicon_tags_7 },
    { 0x373328bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad9328be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb3328c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b328c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf328c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239328cc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3328d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59328df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xac3328e1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5328e6, SUC_lexicon_tags_19 },
    { 0xa73328e7, SUC_lexicon_tags_23 },
    { 0x81d328e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1328ec, SUC_lexicon_tags_25 },
    { 0x7328ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe9b328ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17328f6, SUC_lexicon_tags_12 },
    { 0x6c7328f7, SUC_lexicon_tags_17 },
    { 0xff9328f8, SUC_lexicon_tags_7 },
    { 0x19d328f9, SUC_lexicon_tags_47 },
    { 0xb0f328f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777328ff, SUC_lexicon_tags_2 },
    { 0x6c5328ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1cd32902, SUC_lexicon_tags_12 },
    { 0xb0f32902, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2932905, SUC_lexicon_tags_19 },
    { 0xa0f32906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d33290c, SUC_lexicon_tags_9 },
    { 0x4dd3290c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb53290f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7732923, SUC_lexicon_tags_37 },
    { 0x62f32924, SUC_lexicon_tags_27 },
    { 0x8cb32923, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb0932927, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3292d, SUC_lexicon_tags_15 },
    { 0x65f3292e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb32933, SUC_lexicon_tags_12 },
    { 0x27732934, SUC_lexicon_tags_68 },
    { 0xae932935, SUC_lexicon_tags_47 },
    { 0x21332936, SUC_lexicon_tags_1 },
    { 0x73932937, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6d732939, SUC_lexicon_tags_87 },
    { 0xd453293a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa153293c, SUC_lexicon_tags_37 },
    { 0x1853293d, SUC_lexicon_tags_25 },
    { 0xee93293e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1132945, SUC_lexicon_tags_37 },
    { 0x4ab32946, SUC_lexicon_tags_11 },
    { 0x86932945, SUC_lexicon_tags_23 },
    { 0xa1532946, SUC_lexicon_tags_37 },
    { 0xfd532945, SUC_lexicon_tags_57 },
    { 0xc5732946, SUC_lexicon_tags_11 },
    { 0xc732949, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc553294d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3d3294f, SUC_lexicon_tags_49 },
    { 0xded32950, SUC_lexicon_tags_11 },
    { 0xdaf32951, SUC_lexicon_tags_67 },
    { 0x64932951, SUC_lexicon_tags_1 },
    { 0xefb32951, SUC_lexicon_tags_66 },
    { 0x24332951, SUC_lexicon_tags_2 },
    { 0x28d32955, SUC_lexicon_tags_1 },
    { 0x532953, SUC_lexicon_tags_7 },
    { 0x5bb3294f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2332959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb73295c, SUC_lexicon_tags_15 },
    { 0xa153295d, SUC_lexicon_tags_11 },
    { 0x6a73295d, SUC_lexicon_tags_2 },
    { 0xa173295e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78332961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3532964, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xed932967, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa533296c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34332970, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4932975, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf32979, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a73297b, SUC_lexicon_tags_12 },
    { 0x4593297c, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xbe53297e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b32981, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x61932984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81732987, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b3298c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a132998, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67329a0, SUC_lexicon_tags_0 },
    { 0x813329a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x661329a3, SUC_lexicon_tags_43 },
    { 0x7f3329a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x701329a6, SUC_lexicon_tags_10 },
    { 0x99329a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf9329a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63329ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe01329ae, SUC_lexicon_tags_37 },
    { 0x2e1329af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9329b5, SUC_lexicon_tags_87 },
    { 0x3d5329b6, SUC_lexicon_tags_25 },
    { 0xe2f329b7, SUC_lexicon_tags_47 },
    { 0x4eb329b7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab329bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7329bf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11329c4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669329c9, SUC_lexicon_tags_29 },
    { 0xcc3329ca, SUC_lexicon_tags_14 },
    { 0xb61329ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3329ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b1329d0, SUC_lexicon_tags_32 },
    { 0xd0f329d1, SUC_lexicon_tags_29 },
    { 0xf01329d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x143329d4, SUC_lexicon_tags_37 },
    { 0xc57329d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741329dd, SUC_lexicon_tags_7 },
    { 0x3ff329de, SUC_lexicon_tags_15 },
    { 0x303329de, SUC_lexicon_tags_7 },
    { 0x787329e0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff329e3, SUC_lexicon_tags_13 },
    { 0x887329e4, SUC_lexicon_tags_37 },
    { 0xa11329e5, SUC_lexicon_tags_62 },
    { 0xd81329e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b329f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d329fb, SUC_lexicon_tags_15 },
    { 0xf41329fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x725329fe, SUC_lexicon_tags_55 },
    { 0xe0b329ff, SUC_lexicon_tags_3 },
    { 0x30d32a00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69732a02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22132a0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f32a10, SUC_lexicon_tags_4 },
    { 0xacb32a10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd532a1a, SUC_lexicon_tags_1 },
    { 0x6d932a1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5332a1d, SUC_lexicon_tags_17 },
    { 0xaf132a1e, SUC_lexicon_tags_7 },
    { 0x3af32a1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb32a23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29732a29, SUC_lexicon_tags_63 },
    { 0x53f32a2a, SUC_lexicon_tags_26 },
    { 0x3d532a2a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87732a33, SUC_lexicon_tags_12 },
    { 0xa8132a34, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db32a39, SUC_lexicon_tags_23 },
    { 0x3c732a3a, SUC_lexicon_tags_25 },
    { 0xaf132a39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb932a3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b32a43, SUC_lexicon_tags_2 },
    { 0x68132a44, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe732a47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5732a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba932a53, SUC_lexicon_tags_17 },
    { 0x73932a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1332a5c, SUC_lexicon_tags_11 },
    { 0xd0332a5d, SUC_lexicon_tags_7 },
    { 0xc3132a5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed32a62, SUC_lexicon_tags_39 },
    { 0xfdf32a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf332a66, SUC_lexicon_tags_2 },
    { 0x4ad32a66, SUC_lexicon_tags_68 },
    { 0xc3732a66, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb32a6c, SUC_lexicon_tags_27 },
    { 0x84b32a6c, SUC_lexicon_tags_16 },
    { 0xbd532a6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff32a72, SUC_lexicon_tags_15 },
    { 0x9dd32a73, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2932a77, SUC_lexicon_tags_27 },
    { 0x4a532a77, SUC_lexicon_tags_46 },
    { 0xa1532a79, SUC_lexicon_tags_1 },
    { 0x60332a79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba132a7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe332a7e, SUC_lexicon_tags_27 },
    { 0xdf32a7f, SUC_lexicon_tags_7 },
    { 0xceb32a7f, SUC_lexicon_tags_11 },
    { 0xf2932a7f, SUC_lexicon_tags_30 },
    { 0x41f32a82, SUC_lexicon_tags_12 },
    { 0xef932a81, SUC_lexicon_tags_31 },
    { 0x67932a84, SUC_lexicon_tags_32 },
    { 0x15532a83, SUC_lexicon_tags_13 },
    { 0x91932a84, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x64532a88, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x57132a8a, SUC_lexicon_tags_7 },
    { 0xc1132a8b, SUC_lexicon_tags_37 },
    { 0x9b32a8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34732a90, SUC_lexicon_tags_28 },
    { 0x1d132a91, SUC_lexicon_tags_6 },
    { 0xdd532a90, SUC_lexicon_tags_15 },
    { 0xfad32a90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf132a99, SUC_lexicon_tags_11 },
    { 0xa9332a9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e732a9d, SUC_lexicon_tags_37 },
    { 0x64d32a9e, SUC_lexicon_tags_19 },
    { 0x2d532a9e, SUC_lexicon_tags_47 },
    { 0x8a332aa0, SUC_lexicon_tags_43 },
    { 0x45732aa1, SUC_lexicon_tags_39 },
    { 0x2fb32aa2, SUC_lexicon_tags_33 },
    { 0x18332aa1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7532aaa, SUC_lexicon_tags_21 },
    { 0xcf332aab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1132aae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c732ab5, SUC_lexicon_tags_12 },
    { 0xf6332ab5, SUC_lexicon_tags_12 },
    { 0x57132ab7, SUC_lexicon_tags_12 },
    { 0xee732ab8, SUC_lexicon_tags_39 },
    { 0x5bd32ab5, SUC_lexicon_tags_37 },
    { 0x57d32aba, SUC_lexicon_tags_17 },
    { 0x20332ab9, SUC_lexicon_tags_0 },
    { 0xf6d32abc, SUC_lexicon_tags_19 },
    { 0xba732abd, SUC_lexicon_tags_17 },
    { 0xb9f32abe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1732ac1, SUC_lexicon_tags_7 },
    { 0x63532ac2, SUC_lexicon_tags_46 },
    { 0x9c732ac3, SUC_lexicon_tags_48 },
    { 0x29d32ac4, SUC_lexicon_tags_11 },
    { 0xaaf32ac5, SUC_lexicon_tags_1 },
    { 0xb9732ac5, SUC_lexicon_tags_1 },
    { 0xa1332ac7, SUC_lexicon_tags_8 },
    { 0xbbd32ac8, SUC_lexicon_tags_46 },
    { 0xd3332ac9, SUC_lexicon_tags_15 },
    { 0x94132ac7, SUC_lexicon_tags_250 },
    { 0x85b32ac1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c132acd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd5732acf, SUC_lexicon_tags_92 },
    { 0x32f32ad0, SUC_lexicon_tags_31 },
    { 0x42932ad1, SUC_lexicon_tags_37 },
    { 0xbdd32ad2, SUC_lexicon_tags_49 },
    { 0x56532ad2, SUC_lexicon_tags_15 },
    { 0x64f32ad4, SUC_lexicon_tags_1 },
    { 0x8df32acf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c932add, SUC_lexicon_tags_19 },
    { 0x31932ade, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfc332ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb32ae6, SUC_lexicon_tags_31 },
    { 0x83d32ae7, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2332af1, SUC_lexicon_tags_237 },
    { 0xc3d32af2, SUC_lexicon_tags_2 },
    { 0xd3f32af2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1932af5, SUC_lexicon_tags_1 },
    { 0x6b132af5, SUC_lexicon_tags_33 },
    { 0x6cb32af5, SUC_lexicon_tags_55 },
    { 0x67532af8, SUC_lexicon_tags_60 },
    { 0xf7132af8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d32afc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb332b01, SUC_lexicon_tags_27 },
    { 0xe332b01, SUC_lexicon_tags_19 },
    { 0xd2f32b03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb32b07, SUC_lexicon_tags_37 },
    { 0x51d32b07, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f532b0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d932b10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x44732b13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c532b21, SUC_lexicon_tags_301 },
    { 0xa132b21, SUC_lexicon_tags_2 },
    { 0x48d32b22, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d32b26, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d932b2f, SUC_lexicon_tags_25 },
    { 0x1b732b2f, SUC_lexicon_tags_98 },
    { 0xa6d32b30, SUC_lexicon_tags_9 },
    { 0x16732b2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e532b36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37132b3b, SUC_lexicon_tags_12 },
    { 0xbe532b3c, SUC_lexicon_tags_23 },
    { 0x2cb32b3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca732b43, SUC_lexicon_tags_25 },
    { 0xa332b44, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff732b4b, SUC_lexicon_tags_7 },
    { 0x13732b4c, SUC_lexicon_tags_104 },
    { 0x22532b4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b32b51, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f132b59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x41332b5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8b32b5e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfc332b60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6f132b62, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x88d32b64, SUC_lexicon_tags_33 },
    { 0xc1532b64, SUC_lexicon_tags_11 },
    { 0xfd332b66, SUC_lexicon_tags_0 },
    { 0xe7132b67, SUC_lexicon_tags_27 },
    { 0x55d32b67, SUC_lexicon_tags_4 },
    { 0x7a132b67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfc932b6b, SUC_lexicon_tags_45 },
    { 0x1f132b6b, SUC_lexicon_tags_28 },
    { 0xa2b32b6d, SUC_lexicon_tags_77 },
    { 0x1332b6e, SUC_lexicon_tags_17 },
    { 0x7e932b6d, SUC_lexicon_tags_33 },
    { 0x6ed32b6d, SUC_lexicon_tags_30 },
    { 0x79332b71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c932b74, SUC_lexicon_tags_12 },
    { 0x13532b75, SUC_lexicon_tags_14 },
    { 0x35532b76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x52532b79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d332b7c, SUC_lexicon_tags_11 },
    { 0xb4b32b7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44732b7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40932b83, SUC_lexicon_tags_61 },
    { 0x82b32b84, SUC_lexicon_tags_87 },
    { 0x9b532b85, SUC_lexicon_tags_23 },
    { 0xb7b32b86, SUC_lexicon_tags_17 },
    { 0x5c532b85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80332b90, SUC_lexicon_tags_9 },
    { 0xe9d32b90, SUC_lexicon_tags_26 },
    { 0x46132b91, SUC_lexicon_tags_27 },
    { 0x7eb32b93, SUC_lexicon_tags_49 },
    { 0x23532b93, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf32b97, SUC_lexicon_tags_26 },
    { 0x50132b98, SUC_lexicon_tags_26 },
    { 0xf6132b99, SUC_lexicon_tags_0 },
    { 0x9c532b9a, SUC_lexicon_tags_17 },
    { 0x97332b97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46d32b9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bb32b9f, SUC_lexicon_tags_37 },
    { 0xde732ba0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd32ba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0132bac, SUC_lexicon_tags_37 },
    { 0xdb132bad, SUC_lexicon_tags_7 },
    { 0xe9332bad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63732bb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1732bb2, SUC_lexicon_tags_6 },
    { 0xad32bb3, SUC_lexicon_tags_12 },
    { 0x1c732bb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda732bb7, SUC_lexicon_tags_27 },
    { 0x7d332bb8, SUC_lexicon_tags_30 },
    { 0xf532bb9, SUC_lexicon_tags_15 },
    { 0xe9f32bba, SUC_lexicon_tags_31 },
    { 0x2b732bbb, SUC_lexicon_tags_1 },
    { 0xa5f32bbc, SUC_lexicon_tags_31 },
    { 0xd5532bbd, SUC_lexicon_tags_37 },
    { 0x62332bbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f732bc0, SUC_lexicon_tags_17 },
    { 0xc132bc1, SUC_lexicon_tags_63 },
    { 0x27132bc2, SUC_lexicon_tags_14 },
    { 0x22d32bc3, SUC_lexicon_tags_27 },
    { 0x61532bc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef32bc7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc732bce, SUC_lexicon_tags_17 },
    { 0x25f32bce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe932bd5, SUC_lexicon_tags_37 },
    { 0x58532bd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x46332bd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9732bda, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d32be4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b532be6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f32beb, SUC_lexicon_tags_1 },
    { 0x3c732beb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56f32bee, SUC_lexicon_tags_27 },
    { 0x8cb32bee, SUC_lexicon_tags_27 },
    { 0xbef32bf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x82f32bf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e132bf4, SUC_lexicon_tags_43 },
    { 0xab932bf5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x37132bf7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xba532bf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb32bfe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df32c01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b32c05, SUC_lexicon_tags_25 },
    { 0x39d32c06, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b32c09, SUC_lexicon_tags_1 },
    { 0x2ef32c0a, SUC_lexicon_tags_29 },
    { 0x62b32c0b, SUC_lexicon_tags_63 },
    { 0xdef32c09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1132c0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79332c11, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x70332c13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4932c16, SUC_lexicon_tags_8 },
    { 0xe4d32c17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79732c1a, SUC_lexicon_tags_37 },
    { 0xed532c1b, SUC_lexicon_tags_44 },
    { 0x15132c1c, SUC_lexicon_tags_17 },
    { 0xf2b32c1c, SUC_lexicon_tags_33 },
    { 0x5e732c1e, SUC_lexicon_tags_1 },
    { 0x1bf32c1e, SUC_lexicon_tags_26 },
    { 0xfa132c20, SUC_lexicon_tags_30 },
    { 0xb1d32c21, SUC_lexicon_tags_2 },
    { 0x9bb32c21, SUC_lexicon_tags_39 },
    { 0xd8732c1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4e132c25, SUC_lexicon_tags_6 },
    { 0xa7132c26, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x88132c29, SUC_lexicon_tags_1 },
    { 0x51532c2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc8d32c2c, SUC_lexicon_tags_2 },
    { 0x4f332c2d, SUC_lexicon_tags_60 },
    { 0xf3d32c2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44732c30, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2932c34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b32c39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83532c3b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda532c40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b32c45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab32c49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60932c4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8132c54, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x53732c56, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e932c59, SUC_lexicon_tags_17 },
    { 0x17532c5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b532c5f, SUC_lexicon_tags_37 },
    { 0x4c132c5f, SUC_lexicon_tags_49 },
    { 0xfe932c61, SUC_lexicon_tags_8 },
    { 0xfeb32c62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9f32c64, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xaaf32c66, SUC_lexicon_tags_0 },
    { 0x16932c66, SUC_lexicon_tags_17 },
    { 0xd5332c68, SUC_lexicon_tags_16 },
    { 0x7bb32c69, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab32c6d, SUC_lexicon_tags_46 },
    { 0xde932c6e, SUC_lexicon_tags_14 },
    { 0x83932c6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2df32c71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6932c74, SUC_lexicon_tags_47 },
    { 0xeed32c75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd32c79, SUC_lexicon_tags_15 },
    { 0xccb32c7a, SUC_lexicon_tags_9 },
    { 0x58d32c7b, SUC_lexicon_tags_38 },
    { 0xb0532c7a, SUC_lexicon_tags_26 },
    { 0xf8b32c7d, SUC_lexicon_tags_0 },
    { 0x28f32c7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd32c86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4932c88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf532c8a, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc532c92, SUC_lexicon_tags_37 },
    { 0x62f32c93, SUC_lexicon_tags_47 },
    { 0x90132c94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6732c98, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3bd32c9a, SUC_lexicon_tags_66 },
    { 0xba332c9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7932c9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4932ca1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54932ca4, SUC_lexicon_tags_1 },
    { 0x15d32ca5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f32ca8, SUC_lexicon_tags_68 },
    { 0xaa532ca9, SUC_lexicon_tags_0 },
    { 0xfc732caa, SUC_lexicon_tags_15 },
    { 0xe0932ca9, SUC_lexicon_tags_0 },
    { 0xafb32cac, SUC_lexicon_tags_43 },
    { 0x6b732cad, SUC_lexicon_tags_15 },
    { 0x9b932cae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf2332cb0, SUC_lexicon_tags_25 },
    { 0x1df32cb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb32cb6, SUC_lexicon_tags_7 },
    { 0x23b32cb7, SUC_lexicon_tags_8 },
    { 0xc132cb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3932cc1, SUC_lexicon_tags_8 },
    { 0xbed32cc2, SUC_lexicon_tags_37 },
    { 0x70532cc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e132cc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb32ccc, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f32cd4, SUC_lexicon_tags_32 },
    { 0x12b32cd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f32cda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b32cdd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a132ce0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2f132ce2, SUC_lexicon_tags_1 },
    { 0x4e332ce2, SUC_lexicon_tags_19 },
    { 0xa4332ce2, SUC_lexicon_tags_0 },
    { 0x87932ce5, SUC_lexicon_tags_15 },
    { 0xda932ce6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b32cea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8fd32cec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfaf32cee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf132cf1, SUC_lexicon_tags_60 },
    { 0x4ed32cf2, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0532cf7, SUC_lexicon_tags_15 },
    { 0xdc132cf8, SUC_lexicon_tags_2 },
    { 0x7a332cf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafd32cfb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b332d03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f32d06, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3132d0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x58532d0c, SUC_lexicon_tags_15 },
    { 0x5dd32d0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe132d11, SUC_lexicon_tags_47 },
    { 0x80b32d11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53332d17, SUC_lexicon_tags_25 },
    { 0xb3532d18, SUC_lexicon_tags_10 },
    { 0x19532d19, SUC_lexicon_tags_14 },
    { 0x2f532d18, SUC_lexicon_tags_28 },
    { 0xb6d32d17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d132d1e, SUC_lexicon_tags_1 },
    { 0x77f32d1f, SUC_lexicon_tags_9 },
    { 0x80332d20, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d32d23, SUC_lexicon_tags_21 },
    { 0x1cb32d24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f332d2c, SUC_lexicon_tags_29 },
    { 0xe0532d2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f32d31, SUC_lexicon_tags_47 },
    { 0xd7332d31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad32d35, SUC_lexicon_tags_45 },
    { 0xd0d32d36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42332d38, SUC_lexicon_tags_25 },
    { 0x62b32d39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78332d3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d732d3d, SUC_lexicon_tags_37 },
    { 0xc8932d3d, SUC_lexicon_tags_19 },
    { 0x92d32d3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4932d44, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x3932d46, SUC_lexicon_tags_23 },
    { 0xec332d46, SUC_lexicon_tags_2 },
    { 0xec932d46, SUC_lexicon_tags_6 },
    { 0xa8132d47, SUC_lexicon_tags_1 },
    { 0x61132d4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df32d51, SUC_lexicon_tags_27 },
    { 0x6b532d52, SUC_lexicon_tags_12 },
    { 0x4a132d52, SUC_lexicon_tags_0 },
    { 0x69132d54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x95732d57, SUC_lexicon_tags_0 },
    { 0x4e932d58, SUC_lexicon_tags_12 },
    { 0x29532d59, SUC_lexicon_tags_42 },
    { 0x51b32d5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd32d5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c732d60, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded32d6e, SUC_lexicon_tags_45 },
    { 0x56532d6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3532d7f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x93d32d81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f32d84, SUC_lexicon_tags_61 },
    { 0x9ed32d84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe932d87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91532d89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f32d8c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97932d8e, SUC_lexicon_tags_2 },
    { 0x66d32d8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2132d92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f332d94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1532d98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58732d9c, SUC_lexicon_tags_33 },
    { 0x3ed32d9d, SUC_lexicon_tags_69 },
    { 0x61132d9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0332da1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b32da4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc132daa, SUC_lexicon_tags_4 },
    { 0xbb332daa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96932dad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b32db2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d32db7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda532dbd, SUC_lexicon_tags_19 },
    { 0xd7932dbd, SUC_lexicon_tags_2 },
    { 0x67332dbe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x58132dc1, SUC_lexicon_tags_1 },
    { 0xf1732dc1, SUC_lexicon_tags_43 },
    { 0xc1132dc3, SUC_lexicon_tags_17 },
    { 0x52332dc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10b32dc6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x59932dc9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x48532dcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d32dd4, SUC_lexicon_tags_10 },
    { 0xf8932dd5, SUC_lexicon_tags_37 },
    { 0x78732dd6, SUC_lexicon_tags_10 },
    { 0xcbd32dd7, SUC_lexicon_tags_17 },
    { 0xe0b32dd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5932ddb, SUC_lexicon_tags_25 },
    { 0xf9732ddb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e332ddf, SUC_lexicon_tags_37 },
    { 0xd6732ddf, SUC_lexicon_tags_7 },
    { 0xe4b32de1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0732de4, SUC_lexicon_tags_12 },
    { 0xa1332de5, SUC_lexicon_tags_25 },
    { 0xfb932de6, SUC_lexicon_tags_61 },
    { 0x65332de4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x58932dea, SUC_lexicon_tags_37 },
    { 0x4c732deb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5932dee, SUC_lexicon_tags_27 },
    { 0xeab32def, SUC_lexicon_tags_15 },
    { 0xe9f32df0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d32df3, SUC_lexicon_tags_25 },
    { 0x20332df4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82732df8, SUC_lexicon_tags_29 },
    { 0xed732df9, SUC_lexicon_tags_47 },
    { 0xd5932df8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75132dfd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37332e01, SUC_lexicon_tags_17 },
    { 0x2e932e02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9932e05, SUC_lexicon_tags_63 },
    { 0x86f32e06, SUC_lexicon_tags_57 },
    { 0x3932e07, SUC_lexicon_tags_11 },
    { 0xffb32e06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb32e0e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x23932e10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x41732e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95532e1c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x47d32e1e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdb932e20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b32e23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb332e27, SUC_lexicon_tags_43 },
    { 0xaff32e27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf32e30, SUC_lexicon_tags_11 },
    { 0x44532e31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d32e34, SUC_lexicon_tags_33 },
    { 0x1df32e34, SUC_lexicon_tags_151 },
    { 0xcab32e36, SUC_lexicon_tags_22 },
    { 0xe5532e34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f32e3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d32e42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d32e47, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6dd32e49, SUC_lexicon_tags_0 },
    { 0x7af32e4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72532e53, SUC_lexicon_tags_35 },
    { 0x1b332e54, SUC_lexicon_tags_30 },
    { 0xb3b32e55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1332e58, SUC_lexicon_tags_0 },
    { 0xf7732e58, SUC_lexicon_tags_63 },
    { 0x71b32e59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2d932e5c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd1132e5e, SUC_lexicon_tags_0 },
    { 0x64532e5e, SUC_lexicon_tags_46 },
    { 0xd5332e5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x63d32e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11b32e64, SUC_lexicon_tags_28 },
    { 0xe2932e65, SUC_lexicon_tags_1 },
    { 0x7f32e66, SUC_lexicon_tags_2 },
    { 0x89d32e67, SUC_lexicon_tags_14 },
    { 0x74332e67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4732e6f, SUC_lexicon_tags_34 },
    { 0xb4732e70, SUC_lexicon_tags_27 },
    { 0x20332e6f, SUC_lexicon_tags_1 },
    { 0x14f32e72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b732e75, SUC_lexicon_tags_23 },
    { 0xb1132e76, SUC_lexicon_tags_12 },
    { 0x53f32e77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63932e7b, SUC_lexicon_tags_6 },
    { 0x14f32e7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x41732e7e, SUC_lexicon_tags_37 },
    { 0x2ed32e7f, SUC_lexicon_tags_45 },
    { 0xa7b32e80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff32e85, SUC_lexicon_tags_85 },
    { 0xb0d32e86, SUC_lexicon_tags_33 },
    { 0x78f32e87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d532e89, SUC_lexicon_tags_0 },
    { 0xf3732e89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84132e90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4932e97, SUC_lexicon_tags_7 },
    { 0xea332e98, SUC_lexicon_tags_1 },
    { 0x3f32e98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x85532e9c, SUC_lexicon_tags_390 },
    { 0xf4532e9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d32ea4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e532ea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f32eab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9d32ead, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31b32eaf, SUC_lexicon_tags_12 },
    { 0x9f32eaf, SUC_lexicon_tags_26 },
    { 0xe4932eb1, SUC_lexicon_tags_65 },
    { 0xb5932eb2, SUC_lexicon_tags_2 },
    { 0xf4132eb1, SUC_lexicon_tags_15 },
    { 0xf7732eb4, SUC_lexicon_tags_0 },
    { 0x11532eb4, SUC_lexicon_tags_39 },
    { 0xe5932eb6, SUC_lexicon_tags_37 },
    { 0x1f932eb7, SUC_lexicon_tags_7 },
    { 0x28332eb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59532eba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f332ebe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c532ec1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4932ec6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb132ec8, SUC_lexicon_tags_25 },
    { 0x59d32ec8, SUC_lexicon_tags_2 },
    { 0x27f32eca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d532ecc, SUC_lexicon_tags_37 },
    { 0xcd332ecd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecd32ecf, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90932ed5, SUC_lexicon_tags_25 },
    { 0xa0732ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d932edd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4d32edf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa732ee5, SUC_lexicon_tags_87 },
    { 0xe732ee6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcd932ee8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f32eeb, SUC_lexicon_tags_30 },
    { 0x3a932eec, SUC_lexicon_tags_642 },
    { 0xd0f32eec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96532ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3132ef9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f932efc, SUC_lexicon_tags_68 },
    { 0x51f32efc, SUC_lexicon_tags_33 },
    { 0x36532efe, SUC_lexicon_tags_22 },
    { 0x81732efe, SUC_lexicon_tags_29 },
    { 0x5f332efe, SUC_lexicon_tags_17 },
    { 0xa0132f00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x36532f04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x22332f06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1132f0b, SUC_lexicon_tags_13 },
    { 0xc732f0b, SUC_lexicon_tags_21 },
    { 0xffd32f0d, SUC_lexicon_tags_45 },
    { 0x87332f0d, SUC_lexicon_tags_11 },
    { 0xa2532f0f, SUC_lexicon_tags_17 },
    { 0x2c732f10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e332f18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb132f1a, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x6cd32f1c, SUC_lexicon_tags_11 },
    { 0xf7f32f1d, SUC_lexicon_tags_25 },
    { 0xb2932f1e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f32f21, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x1f532f23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d32f29, SUC_lexicon_tags_69 },
    { 0xcc532f2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x89532f2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1532f31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d732f36, SUC_lexicon_tags_17 },
    { 0xa3532f36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ab32f39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70532f3b, SUC_lexicon_tags_37 },
    { 0x18f32f3c, SUC_lexicon_tags_12 },
    { 0x94132f3d, SUC_lexicon_tags_37 },
    { 0xd7132f3e, SUC_lexicon_tags_1 },
    { 0x87b32f3f, SUC_lexicon_tags_9 },
    { 0x3d132f40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0b32f42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbff32f44, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f32f47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38332f4b, SUC_lexicon_tags_11 },
    { 0xcff32f4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3132f58, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x92d32f5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc732f5d, SUC_lexicon_tags_15 },
    { 0xeb332f5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c932f60, SUC_lexicon_tags_32 },
    { 0x8a132f61, SUC_lexicon_tags_1657 },
    { 0x81b32f62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf532f68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb6d32f6a, SUC_lexicon_tags_28 },
    { 0x22b32f6a, SUC_lexicon_tags_1 },
    { 0x43932f6c, SUC_lexicon_tags_15 },
    { 0x98d32f6c, SUC_lexicon_tags_34 },
    { 0x1932f6b, SUC_lexicon_tags_105 },
    { 0x65332f6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c932f74, SUC_lexicon_tags_37 },
    { 0xd9532f75, SUC_lexicon_tags_19 },
    { 0x52b32f75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a932f78, SUC_lexicon_tags_63 },
    { 0xae732f79, SUC_lexicon_tags_26 },
    { 0xf1f32f7a, SUC_lexicon_tags_27 },
    { 0x91d32f7b, SUC_lexicon_tags_12 },
    { 0x1af32f78, SUC_lexicon_tags_47 },
    { 0xd3d32f7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e132f82, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x25b32f84, SUC_lexicon_tags_12 },
    { 0x3a932f85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b32f8a, SUC_lexicon_tags_23 },
    { 0x9bd32f8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87532f8f, SUC_lexicon_tags_15 },
    { 0x69332f90, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b32f93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5932f97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0332f9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12d32f9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f32fa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9332fa9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39d32fab, SUC_lexicon_tags_2 },
    { 0x2e132fac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc732fb2, SUC_lexicon_tags_20 },
    { 0x90932fb2, SUC_lexicon_tags_7 },
    { 0x66d32fb4, SUC_lexicon_tags_12 },
    { 0xda932fb2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x70532fb8, SUC_lexicon_tags_14 },
    { 0x48b32fb8, SUC_lexicon_tags_37 },
    { 0x50132fba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b32fc0, SUC_lexicon_tags_2 },
    { 0xe2732fc1, SUC_lexicon_tags_2 },
    { 0xa5532fc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8932fc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d32fca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d32fcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b32fcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c132fd1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e132fd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e332fda, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf132fdd, SUC_lexicon_tags_49 },
    { 0xf7732fde, SUC_lexicon_tags_79 },
    { 0x5a332fdd, SUC_lexicon_tags_2 },
    { 0xd5f32fdf, SUC_lexicon_tags_0 },
    { 0xf32fdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b532fe3, SUC_lexicon_tags_65 },
    { 0xa5d32fe3, SUC_lexicon_tags_11 },
    { 0x31f32fe5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9b932fe7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0532fed, SUC_lexicon_tags_11 },
    { 0x14d32fee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f532ff2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88732ff4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x10b32ff6, SUC_lexicon_tags_25 },
    { 0xa3732ff7, SUC_lexicon_tags_15 },
    { 0x56132ff7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee732ffc, SUC_lexicon_tags_7 },
    { 0x90532ffd, SUC_lexicon_tags_7 },
    { 0x98532ffe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76933003, SUC_lexicon_tags_65 },
    { 0x52d33003, SUC_lexicon_tags_45 },
    { 0xd1733005, SUC_lexicon_tags_23 },
    { 0xff933006, SUC_lexicon_tags_12 },
    { 0xc2b33007, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d3300c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1133013, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b33016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff3301a, SUC_lexicon_tags_12 },
    { 0x1933301a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa673301f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x98933022, SUC_lexicon_tags_23 },
    { 0x73d33023, SUC_lexicon_tags_55 },
    { 0x77133024, SUC_lexicon_tags_37 },
    { 0x17533024, SUC_lexicon_tags_12 },
    { 0x91d33026, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x44b33028, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8f93302a, SUC_lexicon_tags_37 },
    { 0xa53302a, SUC_lexicon_tags_61 },
    { 0x4273302a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae53302f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x69d33031, SUC_lexicon_tags_11 },
    { 0x58333032, SUC_lexicon_tags_9 },
    { 0x8c933032, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf4133035, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x84d33037, SUC_lexicon_tags_7 },
    { 0x5c333037, SUC_lexicon_tags_49 },
    { 0x88533039, SUC_lexicon_tags_83 },
    { 0xaaf33037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x273303c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d33040, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91333042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15333047, SUC_lexicon_tags_15 },
    { 0x69d33047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa113304a, SUC_lexicon_tags_1 },
    { 0x7473304b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51933054, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa2d33056, SUC_lexicon_tags_6 },
    { 0xd6f33056, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f3305c, SUC_lexicon_tags_7 },
    { 0x1353305d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf813305f, SUC_lexicon_tags_12 },
    { 0x3593305f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb33063, SUC_lexicon_tags_25 },
    { 0x4f733064, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9cb33066, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd333068, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad3306b, SUC_lexicon_tags_18 },
    { 0x8513306c, SUC_lexicon_tags_23 },
    { 0x9df3306d, SUC_lexicon_tags_0 },
    { 0xd3d3306d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8533070, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34933073, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xba533075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7773307b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59933080, SUC_lexicon_tags_29 },
    { 0xaeb33081, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52333086, SUC_lexicon_tags_17 },
    { 0x7b133087, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc813308b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e33308d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb33092, SUC_lexicon_tags_15 },
    { 0x68933092, SUC_lexicon_tags_17 },
    { 0x46d33094, SUC_lexicon_tags_17 },
    { 0x12f33095, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d33098, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f3309b, SUC_lexicon_tags_16 },
    { 0xa093309c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff330a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8b3330a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97330a8, SUC_lexicon_tags_31 },
    { 0x435330a9, SUC_lexicon_tags_12 },
    { 0xa15330aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9330ae, SUC_lexicon_tags_4 },
    { 0x8df330af, SUC_lexicon_tags_8 },
    { 0x741330af, SUC_lexicon_tags_15 },
    { 0xdd9330ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb33330b3, SUC_lexicon_tags_15 },
    { 0x10b330b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5330bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd330be, SUC_lexicon_tags_12 },
    { 0x32f330bf, SUC_lexicon_tags_1 },
    { 0x655330bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb330ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab9330cc, SUC_lexicon_tags_60 },
    { 0x3d330cc, SUC_lexicon_tags_63 },
    { 0xf9f330cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29330d2, SUC_lexicon_tags_87 },
    { 0x20b330d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x301330d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa0d330d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2f330d9, SUC_lexicon_tags_7 },
    { 0xe65330da, SUC_lexicon_tags_14 },
    { 0xe05330db, SUC_lexicon_tags_11 },
    { 0x4e3330da, SUC_lexicon_tags_38 },
    { 0x485330dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc03330df, SUC_lexicon_tags_16 },
    { 0xd61330e0, SUC_lexicon_tags_68 },
    { 0xc2f330e0, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21330e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605330eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7330ee, SUC_lexicon_tags_37 },
    { 0xa49330ee, SUC_lexicon_tags_12 },
    { 0x91d330f0, SUC_lexicon_tags_23 },
    { 0x441330f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5330f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb5330f6, SUC_lexicon_tags_37 },
    { 0xa3f330f7, SUC_lexicon_tags_45 },
    { 0xb0b330f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x615330fa, SUC_lexicon_tags_87 },
    { 0xb7d330fb, SUC_lexicon_tags_1 },
    { 0x7af330fa, SUC_lexicon_tags_17 },
    { 0x2cf330fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x249330ff, SUC_lexicon_tags_25 },
    { 0x1933100, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xebd33102, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x26d33104, SUC_lexicon_tags_25 },
    { 0x4e333104, SUC_lexicon_tags_37 },
    { 0x2ed33104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df3310c, SUC_lexicon_tags_33 },
    { 0x8af3310c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x28333110, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb2533112, SUC_lexicon_tags_4 },
    { 0xd2133113, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeab33115, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa133119, SUC_lexicon_tags_32 },
    { 0xc0f3311a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41133122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f33125, SUC_lexicon_tags_84 },
    { 0xf7b33126, SUC_lexicon_tags_12 },
    { 0xb3d33127, SUC_lexicon_tags_45 },
    { 0xac533126, SUC_lexicon_tags_49 },
    { 0xbcb33126, SUC_lexicon_tags_2 },
    { 0xa3b3312a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0133130, SUC_lexicon_tags_11 },
    { 0xa3933131, SUC_lexicon_tags_25 },
    { 0xbc533132, SUC_lexicon_tags_66 },
    { 0x98133131, SUC_lexicon_tags_25 },
    { 0xa1133134, SUC_lexicon_tags_1 },
    { 0x5d33134, SUC_lexicon_tags_37 },
    { 0xc5933130, SUC_lexicon_tags_19 },
    { 0x82d33135, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf833313a, SUC_lexicon_tags_1 },
    { 0xf2f3313a, SUC_lexicon_tags_65 },
    { 0x46f3313c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb3313f, SUC_lexicon_tags_1 },
    { 0x41d33140, SUC_lexicon_tags_0 },
    { 0x8f33141, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b33144, SUC_lexicon_tags_19 },
    { 0x8eb33145, SUC_lexicon_tags_17 },
    { 0x92b33146, SUC_lexicon_tags_118 },
    { 0x6ff33147, SUC_lexicon_tags_9 },
    { 0x91733148, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8693314a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8933151, SUC_lexicon_tags_39 },
    { 0xc8f33152, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf33159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b53315f, SUC_lexicon_tags_7 },
    { 0xbab33160, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd33166, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xca133169, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3513316b, SUC_lexicon_tags_37 },
    { 0x6ed3316b, SUC_lexicon_tags_1261 },
    { 0x70b3316d, SUC_lexicon_tags_2 },
    { 0x3c53316d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d33171, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x70933173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d333177, SUC_lexicon_tags_11 },
    { 0x44333177, SUC_lexicon_tags_12 },
    { 0x17b33179, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf3317c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c13317f, SUC_lexicon_tags_14 },
    { 0x1073317f, SUC_lexicon_tags_1 },
    { 0x81333181, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c133185, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f33188, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3d53318a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb693318c, SUC_lexicon_tags_1 },
    { 0x9f73318d, SUC_lexicon_tags_14 },
    { 0xfe73318c, SUC_lexicon_tags_38 },
    { 0xb7d3318c, SUC_lexicon_tags_1 },
    { 0x98733190, SUC_lexicon_tags_1 },
    { 0x2d733191, SUC_lexicon_tags_27 },
    { 0xaed33192, SUC_lexicon_tags_2 },
    { 0xf3933191, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48933196, SUC_lexicon_tags_39 },
    { 0x4f133196, SUC_lexicon_tags_47 },
    { 0xb5f33198, SUC_lexicon_tags_92 },
    { 0x2333199, SUC_lexicon_tags_91 },
    { 0xdbd33199, SUC_lexicon_tags_7 },
    { 0xd813319b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd3319e, SUC_lexicon_tags_44 },
    { 0xc7d3319e, SUC_lexicon_tags_29 },
    { 0xc27331a0, SUC_lexicon_tags_25 },
    { 0x185331a0, SUC_lexicon_tags_26 },
    { 0xb9b331a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1331a6, SUC_lexicon_tags_17 },
    { 0x457331a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d331aa, SUC_lexicon_tags_17 },
    { 0xa71331aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb19331ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b331b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x955331b5, SUC_lexicon_tags_46 },
    { 0xcb9331b5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3331b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x903331bc, SUC_lexicon_tags_21 },
    { 0xfb3331bd, SUC_lexicon_tags_4 },
    { 0x769331bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x455331c1, SUC_lexicon_tags_65 },
    { 0xdd1331c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d331c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1331cc, SUC_lexicon_tags_29 },
    { 0x4bf331cc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x595331cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13331dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d331e1, SUC_lexicon_tags_32 },
    { 0xfed331e2, SUC_lexicon_tags_19 },
    { 0x5a5331e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55331e9, SUC_lexicon_tags_34 },
    { 0x527331e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527331ee, SUC_lexicon_tags_17 },
    { 0xc3b331ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x235331f2, SUC_lexicon_tags_15 },
    { 0x963331f3, SUC_lexicon_tags_47 },
    { 0xa97331f2, SUC_lexicon_tags_39 },
    { 0x8d3331f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1331fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9b331fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f33200, SUC_lexicon_tags_40 },
    { 0x31333201, SUC_lexicon_tags_77 },
    { 0xb5733201, SUC_lexicon_tags_17 },
    { 0xe4133203, SUC_lexicon_tags_11 },
    { 0xae333200, SUC_lexicon_tags_84 },
    { 0xc4f33205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65133207, SUC_lexicon_tags_2 },
    { 0x4f133208, SUC_lexicon_tags_17 },
    { 0x78f33207, SUC_lexicon_tags_8 },
    { 0x31f3320a, SUC_lexicon_tags_26 },
    { 0xea13320b, SUC_lexicon_tags_15 },
    { 0xe9b33207, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f33212, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd0b33214, SUC_lexicon_tags_43 },
    { 0x78533214, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb133218, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d3321b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c33321d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17733220, SUC_lexicon_tags_16 },
    { 0x96933221, SUC_lexicon_tags_17 },
    { 0xbd133222, SUC_lexicon_tags_0 },
    { 0xe8133223, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6333227, SUC_lexicon_tags_17 },
    { 0xc4333228, SUC_lexicon_tags_79 },
    { 0xc1933229, SUC_lexicon_tags_61 },
    { 0x44733229, SUC_lexicon_tags_1 },
    { 0xa253322a, SUC_lexicon_tags_18 },
    { 0x4ef3322a, SUC_lexicon_tags_63 },
    { 0xedd3322d, SUC_lexicon_tags_26 },
    { 0x1ad3322e, SUC_lexicon_tags_11 },
    { 0xa553322a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f33232, SUC_lexicon_tags_37 },
    { 0x3af33233, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d333241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73933243, SUC_lexicon_tags_14 },
    { 0x9f533244, SUC_lexicon_tags_31 },
    { 0x78f33244, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7753324b, SUC_lexicon_tags_11 },
    { 0x7513324c, SUC_lexicon_tags_45 },
    { 0x2a13324d, SUC_lexicon_tags_7 },
    { 0x2f93324e, SUC_lexicon_tags_68 },
    { 0x8313324f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd4533251, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fb33253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13325a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7533260, SUC_lexicon_tags_49 },
    { 0xefb33261, SUC_lexicon_tags_0 },
    { 0xfdb33260, SUC_lexicon_tags_171 },
    { 0x2d333263, SUC_lexicon_tags_38 },
    { 0x69d33263, SUC_lexicon_tags_12 },
    { 0xf4333265, SUC_lexicon_tags_8 },
    { 0x66d33265, SUC_lexicon_tags_47 },
    { 0x14933267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f3326c, SUC_lexicon_tags_12 },
    { 0xb9b3326d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30333271, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc733274, SUC_lexicon_tags_15 },
    { 0xb7f33274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1933277, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe13327a, SUC_lexicon_tags_26 },
    { 0x2413327b, SUC_lexicon_tags_23 },
    { 0xe713327c, SUC_lexicon_tags_2 },
    { 0x5973327d, SUC_lexicon_tags_23 },
    { 0xb33327e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68133280, SUC_lexicon_tags_11 },
    { 0xe2733280, SUC_lexicon_tags_9 },
    { 0x74d33281, SUC_lexicon_tags_253 },
    { 0x9ed33283, SUC_lexicon_tags_37 },
    { 0x8d133283, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbd33286, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b3328f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3333296, SUC_lexicon_tags_45 },
    { 0x17d33297, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff3329a, SUC_lexicon_tags_4 },
    { 0x9c33329b, SUC_lexicon_tags_8 },
    { 0x1d73329c, SUC_lexicon_tags_8 },
    { 0xd8d3329b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x695332a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57332a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149332ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d332ae, SUC_lexicon_tags_15 },
    { 0x35b332ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff332b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9b7332b6, SUC_lexicon_tags_14 },
    { 0x553332b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc3f332b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff332bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d332c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5332c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5332ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf332ce, SUC_lexicon_tags_15 },
    { 0x53f332cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91332d2, SUC_lexicon_tags_13 },
    { 0x807332d3, SUC_lexicon_tags_7 },
    { 0x9fd332d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f332d9, SUC_lexicon_tags_11 },
    { 0x3b9332da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b332dd, SUC_lexicon_tags_43 },
    { 0xefd332de, SUC_lexicon_tags_37 },
    { 0xe53332df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xded332e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf7332e3, SUC_lexicon_tags_43 },
    { 0x7c5332e3, SUC_lexicon_tags_11 },
    { 0x78d332e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3332ea, SUC_lexicon_tags_15 },
    { 0x51b332ea, SUC_lexicon_tags_2 },
    { 0xba5332ec, SUC_lexicon_tags_7 },
    { 0x1d332ed, SUC_lexicon_tags_158 },
    { 0xc2b332ee, SUC_lexicon_tags_7 },
    { 0x107332ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x279332f2, SUC_lexicon_tags_11 },
    { 0x5bf332f2, SUC_lexicon_tags_36 },
    { 0x64d332f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf3332f6, SUC_lexicon_tags_74 },
    { 0x2a9332f7, SUC_lexicon_tags_4 },
    { 0x5af332f8, SUC_lexicon_tags_25 },
    { 0x29f332f6, SUC_lexicon_tags_48 },
    { 0x327332f9, SUC_lexicon_tags_1 },
    { 0xbb1332fb, SUC_lexicon_tags_14 },
    { 0x7c5332fb, SUC_lexicon_tags_32 },
    { 0x779332f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x493332ff, SUC_lexicon_tags_37 },
    { 0x48f33300, SUC_lexicon_tags_28 },
    { 0x8f933300, SUC_lexicon_tags_2 },
    { 0xff733301, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6733306, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff33309, SUC_lexicon_tags_84 },
    { 0xa9133309, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f3330d, SUC_lexicon_tags_38 },
    { 0x1913330e, SUC_lexicon_tags_49 },
    { 0x9bd3330f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2133312, SUC_lexicon_tags_19 },
    { 0x84333312, SUC_lexicon_tags_37 },
    { 0x12133312, SUC_lexicon_tags_9 },
    { 0xe2f33314, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5733317, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e933319, SUC_lexicon_tags_1 },
    { 0x56133319, SUC_lexicon_tags_37 },
    { 0x5173331a, SUC_lexicon_tags_25 },
    { 0xb093331a, SUC_lexicon_tags_17 },
    { 0x5973331d, SUC_lexicon_tags_23 },
    { 0xb9d3331e, SUC_lexicon_tags_31 },
    { 0xfcf3331e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46733325, SUC_lexicon_tags_65 },
    { 0xe7b33326, SUC_lexicon_tags_1 },
    { 0x75133327, SUC_lexicon_tags_13 },
    { 0xa0f33328, SUC_lexicon_tags_12 },
    { 0x95b33326, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb3332c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e133334, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb33337, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5b33339, SUC_lexicon_tags_55 },
    { 0x33133339, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84133340, SUC_lexicon_tags_10 },
    { 0xe2133340, SUC_lexicon_tags_17 },
    { 0x6933340, SUC_lexicon_tags_47 },
    { 0x76333343, SUC_lexicon_tags_15 },
    { 0xd6d33344, SUC_lexicon_tags_11 },
    { 0x68333340, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x30f33347, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5893334b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78d3334d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f33351, SUC_lexicon_tags_15 },
    { 0x36f33351, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe6733354, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1333359, SUC_lexicon_tags_6 },
    { 0x4d733359, SUC_lexicon_tags_37 },
    { 0x5253335b, SUC_lexicon_tags_27 },
    { 0x3013335c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e13335e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f333361, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd6133363, SUC_lexicon_tags_14 },
    { 0xed933363, SUC_lexicon_tags_19 },
    { 0xef533363, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f3336b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb373336f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee133374, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1593337d, SUC_lexicon_tags_1 },
    { 0xf873337e, SUC_lexicon_tags_16 },
    { 0xf853337f, SUC_lexicon_tags_62 },
    { 0x60533380, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1533384, SUC_lexicon_tags_6 },
    { 0x5db33385, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf933387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd3338e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5d33390, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17133392, SUC_lexicon_tags_37 },
    { 0xe2733392, SUC_lexicon_tags_8 },
    { 0xf3733393, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43133399, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43f3339b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317333a0, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3333a3, SUC_lexicon_tags_32 },
    { 0x561333a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69333a8, SUC_lexicon_tags_11 },
    { 0xdc9333a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677333af, SUC_lexicon_tags_11 },
    { 0x1c1333b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b333b3, SUC_lexicon_tags_46 },
    { 0xd79333b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9333ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd333bd, SUC_lexicon_tags_17 },
    { 0x387333be, SUC_lexicon_tags_31 },
    { 0xac5333be, SUC_lexicon_tags_9 },
    { 0x2d3333bf, SUC_lexicon_tags_63 },
    { 0x56b333c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb333c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503333c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d333cf, SUC_lexicon_tags_7 },
    { 0x659333d0, SUC_lexicon_tags_37 },
    { 0xf5d333d1, SUC_lexicon_tags_15 },
    { 0x5b3333d2, SUC_lexicon_tags_7 },
    { 0xb5333d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdb333d5, SUC_lexicon_tags_282 },
    { 0x709333d6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75333d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05333dc, SUC_lexicon_tags_11 },
    { 0xb59333dc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xead333df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d333e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85333e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59333eb, SUC_lexicon_tags_15 },
    { 0x737333ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5333f3, SUC_lexicon_tags_15 },
    { 0x579333f4, SUC_lexicon_tags_17 },
    { 0x23333f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75333fb, SUC_lexicon_tags_25 },
    { 0x77333fb, SUC_lexicon_tags_21 },
    { 0xf87333fd, SUC_lexicon_tags_47 },
    { 0xbd1333fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf133402, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff733406, SUC_lexicon_tags_0 },
    { 0x5df33407, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x28933409, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4753340b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc213340d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb33411, SUC_lexicon_tags_14 },
    { 0xeaf33412, SUC_lexicon_tags_39 },
    { 0x15333411, SUC_lexicon_tags_1 },
    { 0xb7733411, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21533418, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb93341b, SUC_lexicon_tags_38 },
    { 0x5293341c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6733341f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18333421, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf33424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69533426, SUC_lexicon_tags_60 },
    { 0x7e933427, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f3342a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf273342e, SUC_lexicon_tags_33 },
    { 0xaa13342f, SUC_lexicon_tags_8 },
    { 0x48133430, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d33435, SUC_lexicon_tags_27 },
    { 0x3bb33435, SUC_lexicon_tags_79 },
    { 0xcf33437, SUC_lexicon_tags_60 },
    { 0x13533436, SUC_lexicon_tags_8 },
    { 0xb7733439, SUC_lexicon_tags_15 },
    { 0x74133436, SUC_lexicon_tags_1 },
    { 0x99333437, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1613343d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74133444, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8053344a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1d73344c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d933450, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5533454, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x65533457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b3345b, SUC_lexicon_tags_21 },
    { 0x95f3345c, SUC_lexicon_tags_33 },
    { 0x9dd3345b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d133461, SUC_lexicon_tags_26 },
    { 0xb6933462, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f33465, SUC_lexicon_tags_25 },
    { 0x5d933465, SUC_lexicon_tags_14 },
    { 0x5af33467, SUC_lexicon_tags_7 },
    { 0xd0133467, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb3346f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a333472, SUC_lexicon_tags_27 },
    { 0x5533473, SUC_lexicon_tags_84 },
    { 0x93733474, SUC_lexicon_tags_43 },
    { 0x6a533475, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7733477, SUC_lexicon_tags_15 },
    { 0xa6533478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd33485, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3733487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13348a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64933492, SUC_lexicon_tags_6 },
    { 0x40933493, SUC_lexicon_tags_14 },
    { 0xd6733494, SUC_lexicon_tags_12 },
    { 0x16f33493, SUC_lexicon_tags_9 },
    { 0xfe933492, SUC_lexicon_tags_17 },
    { 0xc5333497, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21733499, SUC_lexicon_tags_15 },
    { 0xd8b3349a, SUC_lexicon_tags_84 },
    { 0x69733499, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951334a3, SUC_lexicon_tags_123 },
    { 0xcc3334a3, SUC_lexicon_tags_19 },
    { 0x305334a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa6f334a7, SUC_lexicon_tags_33 },
    { 0xfb1334a7, SUC_lexicon_tags_59 },
    { 0x59b334a9, SUC_lexicon_tags_31 },
    { 0x325334aa, SUC_lexicon_tags_17 },
    { 0x4ed334aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab7334ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3334b0, SUC_lexicon_tags_8 },
    { 0x2f334b0, SUC_lexicon_tags_79 },
    { 0xc15334b2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb55334b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5334b7, SUC_lexicon_tags_27 },
    { 0xbab334b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b334bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36d334bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e1334c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab334c6, SUC_lexicon_tags_47 },
    { 0xfe7334c7, SUC_lexicon_tags_28 },
    { 0x1fb334c7, SUC_lexicon_tags_21 },
    { 0x9fd334c6, SUC_lexicon_tags_20 },
    { 0x51d334ca, SUC_lexicon_tags_25 },
    { 0xe79334c7, SUC_lexicon_tags_63 },
    { 0x3a9334cc, SUC_lexicon_tags_79 },
    { 0xf0b334cb, SUC_lexicon_tags_37 },
    { 0x9b5334ce, SUC_lexicon_tags_23 },
    { 0x643334cf, SUC_lexicon_tags_15 },
    { 0x5eb334cf, SUC_lexicon_tags_2 },
    { 0x5fb334c6, SUC_lexicon_tags_84 },
    { 0x9e1334d2, SUC_lexicon_tags_6 },
    { 0x251334d2, SUC_lexicon_tags_15 },
    { 0x911334d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9bf334d6, SUC_lexicon_tags_31 },
    { 0x39f334d7, SUC_lexicon_tags_7 },
    { 0xf7b334d7, SUC_lexicon_tags_12 },
    { 0x7b5334d9, SUC_lexicon_tags_43 },
    { 0x30d334da, SUC_lexicon_tags_27 },
    { 0x5b7334d6, SUC_lexicon_tags_60 },
    { 0x35f334dc, SUC_lexicon_tags_65 },
    { 0x1d5334dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea5334df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1334e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9334e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f334ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37334f2, SUC_lexicon_tags_25 },
    { 0xd07334f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf53334f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x709334f7, SUC_lexicon_tags_43 },
    { 0xa55334f8, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7334fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91334ff, SUC_lexicon_tags_11 },
    { 0xab133500, SUC_lexicon_tags_26 },
    { 0xab933500, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa933509, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d33514, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f3351c, SUC_lexicon_tags_68 },
    { 0xa6b3351c, SUC_lexicon_tags_1 },
    { 0x3993351e, SUC_lexicon_tags_37 },
    { 0x8073351f, SUC_lexicon_tags_9 },
    { 0x45733520, SUC_lexicon_tags_113 },
    { 0x72f33521, SUC_lexicon_tags_33 },
    { 0x4f533520, SUC_lexicon_tags_26 },
    { 0xa4933523, SUC_lexicon_tags_9 },
    { 0x37b3351f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b33527, SUC_lexicon_tags_11 },
    { 0xc1d33528, SUC_lexicon_tags_3 },
    { 0x1e333529, SUC_lexicon_tags_47 },
    { 0xd233352a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d333535, SUC_lexicon_tags_3 },
    { 0x12133535, SUC_lexicon_tags_7 },
    { 0x80533535, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb8933539, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf333353e, SUC_lexicon_tags_65 },
    { 0x90b3353f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a733545, SUC_lexicon_tags_15 },
    { 0x333545, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf933549, SUC_lexicon_tags_566 },
    { 0x5e333549, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd3354e, SUC_lexicon_tags_65 },
    { 0xdb13354e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x55333551, SUC_lexicon_tags_1 },
    { 0x25f33552, SUC_lexicon_tags_11 },
    { 0x533551, SUC_lexicon_tags_7 },
    { 0xb2933554, SUC_lexicon_tags_25 },
    { 0x65333555, SUC_lexicon_tags_1009 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd573355a, SUC_lexicon_tags_12 },
    { 0x37d3355b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9493355e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f33561, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd33564, SUC_lexicon_tags_37 },
    { 0x8d533565, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xd1933567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a533569, SUC_lexicon_tags_11 },
    { 0xde733569, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b73356d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e933574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0933577, SUC_lexicon_tags_43 },
    { 0xf8933577, SUC_lexicon_tags_11 },
    { 0xd2933579, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc93357c, SUC_lexicon_tags_17 },
    { 0x1253357d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4933580, SUC_lexicon_tags_25 },
    { 0xb1733581, SUC_lexicon_tags_7 },
    { 0x56f33582, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7f33584, SUC_lexicon_tags_17 },
    { 0x6533584, SUC_lexicon_tags_16 },
    { 0xa6133585, SUC_lexicon_tags_15 },
    { 0x13b33587, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7733589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfb3358b, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6733592, SUC_lexicon_tags_65 },
    { 0x1a333593, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x34f33595, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17733598, SUC_lexicon_tags_15 },
    { 0x39d33599, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x7393359c, SUC_lexicon_tags_17 },
    { 0x1273359d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f13359f, SUC_lexicon_tags_15 },
    { 0x6573359f, SUC_lexicon_tags_43 },
    { 0x4a1335a1, SUC_lexicon_tags_29 },
    { 0xe09335a0, SUC_lexicon_tags_11 },
    { 0x84f3359f, SUC_lexicon_tags_4 },
    { 0xb07335a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b335a7, SUC_lexicon_tags_473 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823335ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987335b3, SUC_lexicon_tags_11 },
    { 0xc3f335b4, SUC_lexicon_tags_2 },
    { 0x8c5335b5, SUC_lexicon_tags_12 },
    { 0x88b335b6, SUC_lexicon_tags_60 },
    { 0xf2d335b7, SUC_lexicon_tags_12 },
    { 0x49335b4, SUC_lexicon_tags_11 },
    { 0x3c7335b9, SUC_lexicon_tags_46 },
    { 0xddb335ba, SUC_lexicon_tags_17 },
    { 0xced335bb, SUC_lexicon_tags_37 },
    { 0x625335bc, SUC_lexicon_tags_12 },
    { 0x217335b7, SUC_lexicon_tags_47 },
    { 0x291335b4, SUC_lexicon_tags_12 },
    { 0xa9d335b3, SUC_lexicon_tags_17 },
    { 0x3fd335c0, SUC_lexicon_tags_60 },
    { 0xb07335c1, SUC_lexicon_tags_2 },
    { 0x757335c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae1335c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21335c8, SUC_lexicon_tags_23 },
    { 0xc99335c9, SUC_lexicon_tags_14 },
    { 0xc81335c9, SUC_lexicon_tags_33 },
    { 0x51d335cb, SUC_lexicon_tags_8 },
    { 0x35b335ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653335db, SUC_lexicon_tags_29 },
    { 0x511335dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3335e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923335e6, SUC_lexicon_tags_7 },
    { 0xa1f335e6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f335f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x795335f3, SUC_lexicon_tags_55 },
    { 0x4f7335f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1335f6, SUC_lexicon_tags_3 },
    { 0x263335f6, SUC_lexicon_tags_25 },
    { 0xf75335f8, SUC_lexicon_tags_140 },
    { 0xf01335f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b33600, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3933602, SUC_lexicon_tags_77 },
    { 0xdf933603, SUC_lexicon_tags_12 },
    { 0xab533603, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c133606, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd8333608, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf73360b, SUC_lexicon_tags_1 },
    { 0x3c73360c, SUC_lexicon_tags_12 },
    { 0xeab3360d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d333610, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33533613, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1133615, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f33619, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8553361b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c133622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ad33624, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70733628, SUC_lexicon_tags_15 },
    { 0x8fd33628, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbd3362b, SUC_lexicon_tags_8 },
    { 0x1153362c, SUC_lexicon_tags_1 },
    { 0xba13362c, SUC_lexicon_tags_2 },
    { 0x2a93362b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf33632, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb33638, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73363f, SUC_lexicon_tags_7 },
    { 0x4c333640, SUC_lexicon_tags_26 },
    { 0xd473363f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b33646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b3364f, SUC_lexicon_tags_25 },
    { 0xa0d3364f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88733652, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df3365a, SUC_lexicon_tags_7 },
    { 0xf953365b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3365f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfcf33661, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed733663, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a133666, SUC_lexicon_tags_1 },
    { 0xa9733667, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8973366c, SUC_lexicon_tags_31 },
    { 0xfcb3366d, SUC_lexicon_tags_46 },
    { 0x7733366e, SUC_lexicon_tags_11 },
    { 0xba73366e, SUC_lexicon_tags_38 },
    { 0x9913366e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadd33672, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fb33674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7133676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3367a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb33680, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8933683, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb33687, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76133689, SUC_lexicon_tags_15 },
    { 0x2f3368a, SUC_lexicon_tags_30 },
    { 0xb1f3368a, SUC_lexicon_tags_66 },
    { 0x88d3368b, SUC_lexicon_tags_14 },
    { 0xdb3368c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f33695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac733699, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc773369b, SUC_lexicon_tags_2 },
    { 0xf213369c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f53369e, SUC_lexicon_tags_63 },
    { 0x5013369f, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67336a2, SUC_lexicon_tags_23 },
    { 0xad5336a3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d336a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xea9336a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5336ae, SUC_lexicon_tags_14 },
    { 0x6ed336ae, SUC_lexicon_tags_39 },
    { 0xf35336ae, SUC_lexicon_tags_1 },
    { 0xd3b336b0, SUC_lexicon_tags_30 },
    { 0x77b336b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f336b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d336ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x197336bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x415336bf, SUC_lexicon_tags_17 },
    { 0x5f7336c0, SUC_lexicon_tags_47 },
    { 0x7336bf, SUC_lexicon_tags_31 },
    { 0xe9d336c1, SUC_lexicon_tags_95 },
    { 0x1b1336c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7336c6, SUC_lexicon_tags_7 },
    { 0x177336c6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95336cb, SUC_lexicon_tags_28 },
    { 0x69f336cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd336cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171336d4, SUC_lexicon_tags_63 },
    { 0xe9f336d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89336de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9336e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db336e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba5336e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b336f3, SUC_lexicon_tags_7 },
    { 0x865336f4, SUC_lexicon_tags_23 },
    { 0x3e3336f5, SUC_lexicon_tags_11 },
    { 0x75b336f5, SUC_lexicon_tags_65 },
    { 0xa93336f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191336fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5733702, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x62933704, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd3370b, SUC_lexicon_tags_28 },
    { 0x1753370b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1213370f, SUC_lexicon_tags_9 },
    { 0xadb33710, SUC_lexicon_tags_2 },
    { 0x1f733711, SUC_lexicon_tags_28 },
    { 0x7833370f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4333714, SUC_lexicon_tags_25 },
    { 0xc8d33714, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3933718, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd93371d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed33724, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x30533726, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d3372e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4333732, SUC_lexicon_tags_27 },
    { 0xe5b33732, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd133735, SUC_lexicon_tags_17 },
    { 0x70f33736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d73373b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x71f3373d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8933744, SUC_lexicon_tags_39 },
    { 0xf2b33745, SUC_lexicon_tags_1 },
    { 0xa6733744, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb473374b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed3374f, SUC_lexicon_tags_12 },
    { 0x3b3374f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee333758, SUC_lexicon_tags_1 },
    { 0x65733758, SUC_lexicon_tags_2 },
    { 0xe2933759, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1cd3375c, SUC_lexicon_tags_0 },
    { 0x8bd3375d, SUC_lexicon_tags_25 },
    { 0x82f3375e, SUC_lexicon_tags_44 },
    { 0x6593375f, SUC_lexicon_tags_47 },
    { 0xb0f3375f, SUC_lexicon_tags_12 },
    { 0x87533761, SUC_lexicon_tags_31 },
    { 0x33533762, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1933766, SUC_lexicon_tags_15 },
    { 0x41f33767, SUC_lexicon_tags_7 },
    { 0x8cb33768, SUC_lexicon_tags_45 },
    { 0x2bd33768, SUC_lexicon_tags_17 },
    { 0x4d93376a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20d3376c, SUC_lexicon_tags_15 },
    { 0xcf93376c, SUC_lexicon_tags_15 },
    { 0x733376e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4733771, SUC_lexicon_tags_23 },
    { 0xddd33772, SUC_lexicon_tags_2 },
    { 0xddd33773, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1933779, SUC_lexicon_tags_12 },
    { 0x71133779, SUC_lexicon_tags_23 },
    { 0xb4d3377b, SUC_lexicon_tags_15 },
    { 0x2bb33779, SUC_lexicon_tags_37 },
    { 0x3bf33779, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb33780, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2733784, SUC_lexicon_tags_0 },
    { 0xe2933784, SUC_lexicon_tags_16 },
    { 0x21133784, SUC_lexicon_tags_2 },
    { 0xe4b33787, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9833378c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa33378e, SUC_lexicon_tags_7 },
    { 0x6af3378e, SUC_lexicon_tags_30 },
    { 0x9e33378f, SUC_lexicon_tags_38 },
    { 0x7fd3378e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d33794, SUC_lexicon_tags_49 },
    { 0x82733795, SUC_lexicon_tags_5 },
    { 0x2ed33796, SUC_lexicon_tags_12 },
    { 0xef533795, SUC_lexicon_tags_19 },
    { 0xe5533798, SUC_lexicon_tags_17 },
    { 0x76133799, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc733379d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1337a3, SUC_lexicon_tags_2 },
    { 0x719337a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf57337a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1d337a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb1337aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x725337ac, SUC_lexicon_tags_37 },
    { 0x55f337ad, SUC_lexicon_tags_77 },
    { 0x1a7337ad, SUC_lexicon_tags_17 },
    { 0xdfb337ae, SUC_lexicon_tags_7 },
    { 0x6f5337ad, SUC_lexicon_tags_1 },
    { 0xe63337b1, SUC_lexicon_tags_12 },
    { 0x121337b2, SUC_lexicon_tags_60 },
    { 0x47d337b3, SUC_lexicon_tags_17 },
    { 0x8d5337b3, SUC_lexicon_tags_25 },
    { 0x8d3337b3, SUC_lexicon_tags_1 },
    { 0x3b337ae, SUC_lexicon_tags_11 },
    { 0x4dd337b1, SUC_lexicon_tags_17 },
    { 0xd7b337b8, SUC_lexicon_tags_21 },
    { 0x8cd337b9, SUC_lexicon_tags_10 },
    { 0x18f337b7, SUC_lexicon_tags_28 },
    { 0x69b337af, SUC_lexicon_tags_37 },
    { 0xe65337b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb337bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f337c2, SUC_lexicon_tags_44 },
    { 0x47d337c3, SUC_lexicon_tags_2 },
    { 0xcb3337c4, SUC_lexicon_tags_79 },
    { 0x2c7337c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x189337c7, SUC_lexicon_tags_43 },
    { 0x3c5337c8, SUC_lexicon_tags_31 },
    { 0x4ed337c9, SUC_lexicon_tags_49 },
    { 0x7cd337c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x981337cc, SUC_lexicon_tags_9 },
    { 0x715337cd, SUC_lexicon_tags_23 },
    { 0xda5337ce, SUC_lexicon_tags_33 },
    { 0x3e7337cf, SUC_lexicon_tags_12 },
    { 0x149337cf, SUC_lexicon_tags_48 },
    { 0x705337ce, SUC_lexicon_tags_27 },
    { 0xc1b337d2, SUC_lexicon_tags_1 },
    { 0xe3f337d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87337d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49337e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x71b337e2, SUC_lexicon_tags_11 },
    { 0x767337e3, SUC_lexicon_tags_56 },
    { 0xa9337e3, SUC_lexicon_tags_17 },
    { 0x629337e2, SUC_lexicon_tags_77 },
    { 0xb65337e5, SUC_lexicon_tags_11 },
    { 0xa29337e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x107337ea, SUC_lexicon_tags_31 },
    { 0x573337eb, SUC_lexicon_tags_37 },
    { 0x2ef337ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x475337ef, SUC_lexicon_tags_11 },
    { 0x969337ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d337fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9f7337fe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd3733800, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50133804, SUC_lexicon_tags_26 },
    { 0x1f333805, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc93380a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed733810, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb33814, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x36933817, SUC_lexicon_tags_23 },
    { 0x75d33818, SUC_lexicon_tags_37 },
    { 0x7a133819, SUC_lexicon_tags_11 },
    { 0x4733817, SUC_lexicon_tags_25 },
    { 0xfed33818, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d33822, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf733824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd333828, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf33382c, SUC_lexicon_tags_30 },
    { 0xe1d3382d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d33832, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88933837, SUC_lexicon_tags_30 },
    { 0x4db33837, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8b53383a, SUC_lexicon_tags_15 },
    { 0xed33383a, SUC_lexicon_tags_29 },
    { 0x9793383c, SUC_lexicon_tags_26 },
    { 0xf233383d, SUC_lexicon_tags_17 },
    { 0x5d13383e, SUC_lexicon_tags_12 },
    { 0x7013383f, SUC_lexicon_tags_49 },
    { 0x9bd3383f, SUC_lexicon_tags_37 },
    { 0x6713383f, SUC_lexicon_tags_85 },
    { 0x7133383d, SUC_lexicon_tags_68 },
    { 0x4b73383d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f33848, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1753384a, SUC_lexicon_tags_25 },
    { 0x9293384b, SUC_lexicon_tags_87 },
    { 0x2553384c, SUC_lexicon_tags_7 },
    { 0x51d3384c, SUC_lexicon_tags_208 },
    { 0x9113384d, SUC_lexicon_tags_34 },
    { 0x3d53384c, SUC_lexicon_tags_1 },
    { 0xa353384b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf33859, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7013385f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe333861, SUC_lexicon_tags_31 },
    { 0x50533862, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37933865, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb5133867, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213386b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xef93386e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77133873, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96133875, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x28533877, SUC_lexicon_tags_34 },
    { 0x1533877, SUC_lexicon_tags_12 },
    { 0xc1b33879, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa613387f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d33883, SUC_lexicon_tags_12 },
    { 0x81133884, SUC_lexicon_tags_25 },
    { 0xf8933883, SUC_lexicon_tags_37 },
    { 0x67b33886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd33889, SUC_lexicon_tags_2 },
    { 0xfa333889, SUC_lexicon_tags_25 },
    { 0x34333889, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8373388f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3133892, SUC_lexicon_tags_12 },
    { 0xd7333893, SUC_lexicon_tags_37 },
    { 0x14733893, SUC_lexicon_tags_46 },
    { 0x4f933892, SUC_lexicon_tags_37 },
    { 0x41933895, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b3389c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f338a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xec9338a5, SUC_lexicon_tags_33 },
    { 0xd3d338a5, SUC_lexicon_tags_23 },
    { 0x429338a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x641338a9, SUC_lexicon_tags_15 },
    { 0xd9b338aa, SUC_lexicon_tags_0 },
    { 0x113338ab, SUC_lexicon_tags_45 },
    { 0xf33338a9, SUC_lexicon_tags_37 },
    { 0x7d7338aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd338b0, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xaf3338b2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b338b5, SUC_lexicon_tags_17 },
    { 0xcbd338b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db338ba, SUC_lexicon_tags_49 },
    { 0x2f7338bb, SUC_lexicon_tags_17 },
    { 0x6e1338bc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7338c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf55338ca, SUC_lexicon_tags_26 },
    { 0x4b7338ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693338d4, SUC_lexicon_tags_17 },
    { 0xba5338d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d338d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d338dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46f338df, SUC_lexicon_tags_11 },
    { 0x249338e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d338eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9338ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbbb338ef, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7338ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85333904, SUC_lexicon_tags_15 },
    { 0x42d33905, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x61f33907, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec13390d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd013390f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf533913, SUC_lexicon_tags_45 },
    { 0x23133914, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7333917, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x9533919, SUC_lexicon_tags_25 },
    { 0xe8d33919, SUC_lexicon_tags_14 },
    { 0x7893391b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd3391e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17133923, SUC_lexicon_tags_27 },
    { 0x8a533924, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce933926, SUC_lexicon_tags_17 },
    { 0xb1333926, SUC_lexicon_tags_10 },
    { 0x25533926, SUC_lexicon_tags_37 },
    { 0x3f333929, SUC_lexicon_tags_25 },
    { 0xa5b33929, SUC_lexicon_tags_28 },
    { 0x6253392b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34933932, SUC_lexicon_tags_57 },
    { 0xfc133933, SUC_lexicon_tags_1 },
    { 0x35733934, SUC_lexicon_tags_33 },
    { 0x9b33935, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0x8ff33937, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38f33939, SUC_lexicon_tags_15 },
    { 0x9733393a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe573393d, SUC_lexicon_tags_25 },
    { 0xa653393e, SUC_lexicon_tags_46 },
    { 0x3f73393f, SUC_lexicon_tags_29 },
    { 0x7fb33940, SUC_lexicon_tags_19 },
    { 0x54533941, SUC_lexicon_tags_47 },
    { 0x4ab33940, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85d33944, SUC_lexicon_tags_17 },
    { 0x3fb33945, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x75333948, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d3394c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd813394f, SUC_lexicon_tags_14 },
    { 0x77f33950, SUC_lexicon_tags_49 },
    { 0x42b3394f, SUC_lexicon_tags_17 },
    { 0x84b33950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c133957, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d93395a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d733960, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff333962, SUC_lexicon_tags_63 },
    { 0xb1933963, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d33966, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f33976, SUC_lexicon_tags_12 },
    { 0xef333977, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f333982, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x73533985, SUC_lexicon_tags_26 },
    { 0xe3133986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453398c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9333990, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44333997, SUC_lexicon_tags_30 },
    { 0xd8333997, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd53399b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9053399e, SUC_lexicon_tags_33 },
    { 0x53b3399f, SUC_lexicon_tags_55 },
    { 0x2793399f, SUC_lexicon_tags_15 },
    { 0x4ef339a1, SUC_lexicon_tags_1 },
    { 0xf53339a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51339ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdff339b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42f339b2, SUC_lexicon_tags_60 },
    { 0x12b339b3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x7af339b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x145339b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x603339ba, SUC_lexicon_tags_15 },
    { 0xef5339bb, SUC_lexicon_tags_25 },
    { 0xd6f339bb, SUC_lexicon_tags_87 },
    { 0xf3f339bb, SUC_lexicon_tags_37 },
    { 0x821339be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb339cd, SUC_lexicon_tags_25 },
    { 0x629339ce, SUC_lexicon_tags_23 },
    { 0xbc7339cd, SUC_lexicon_tags_1 },
    { 0x7e5339d0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3339d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07339d6, SUC_lexicon_tags_92 },
    { 0x8f3339d7, SUC_lexicon_tags_7 },
    { 0xa03339d8, SUC_lexicon_tags_15 },
    { 0xbaf339d6, SUC_lexicon_tags_37 },
    { 0x9bb339d7, SUC_lexicon_tags_19 },
    { 0x5f7339d7, SUC_lexicon_tags_10 },
    { 0xd33339d6, SUC_lexicon_tags_79 },
    { 0x285339dd, SUC_lexicon_tags_1 },
    { 0xd51339dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf45339e0, SUC_lexicon_tags_79 },
    { 0xcdd339e1, SUC_lexicon_tags_25 },
    { 0x2af339e2, SUC_lexicon_tags_8 },
    { 0xeb339e3, SUC_lexicon_tags_25 },
    { 0xc65339e0, SUC_lexicon_tags_55 },
    { 0x1a5339e5, SUC_lexicon_tags_15 },
    { 0x4f1339e5, SUC_lexicon_tags_2 },
    { 0x2d5339e7, SUC_lexicon_tags_17 },
    { 0x929339e6, SUC_lexicon_tags_1718 },
    { 0xc53339e9, SUC_lexicon_tags_7 },
    { 0xe25339e9, SUC_lexicon_tags_45 },
    { 0x443339eb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x655339ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf339f3, SUC_lexicon_tags_25 },
    { 0x29d339f4, SUC_lexicon_tags_15 },
    { 0xf79339f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5339fa, SUC_lexicon_tags_53 },
    { 0x1eb339fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65333a03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f33a06, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc7733a08, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x39733a0a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae133a0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb33a13, SUC_lexicon_tags_534 },
    { 0x57733a14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63f33a16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e933a1c, SUC_lexicon_tags_15 },
    { 0xf4333a1d, SUC_lexicon_tags_81 },
    { 0xae533a1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7533a20, SUC_lexicon_tags_70 },
    { 0xa1d33a20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf333a26, SUC_lexicon_tags_14 },
    { 0x5d133a27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4133a2c, SUC_lexicon_tags_7 },
    { 0x84333a2c, SUC_lexicon_tags_33 },
    { 0xa0733a2e, SUC_lexicon_tags_17 },
    { 0x3c533a2f, SUC_lexicon_tags_25 },
    { 0x4b33a2f, SUC_lexicon_tags_34 },
    { 0x3bd33a2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a733a34, SUC_lexicon_tags_1 },
    { 0x1dd33a34, SUC_lexicon_tags_2 },
    { 0x6df33a36, SUC_lexicon_tags_14 },
    { 0x78333a37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2fd33a39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f33a3c, SUC_lexicon_tags_47 },
    { 0x89933a3c, SUC_lexicon_tags_11 },
    { 0x75f33a3e, SUC_lexicon_tags_12 },
    { 0x77333a3e, SUC_lexicon_tags_46 },
    { 0xbed33a3e, SUC_lexicon_tags_144 },
    { 0xb6933a3e, SUC_lexicon_tags_1 },
    { 0xd6733a42, SUC_lexicon_tags_11 },
    { 0x9f33a43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d33a48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8b33a4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf33a4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3333a52, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x91f33a54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d33a58, SUC_lexicon_tags_68 },
    { 0xd6f33a59, SUC_lexicon_tags_37 },
    { 0x87133a5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed33a5e, SUC_lexicon_tags_17 },
    { 0x6ad33a5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c133a62, SUC_lexicon_tags_47 },
    { 0x48733a62, SUC_lexicon_tags_11 },
    { 0x39f33a64, SUC_lexicon_tags_37 },
    { 0xc4533a64, SUC_lexicon_tags_9 },
    { 0x8fd33a66, SUC_lexicon_tags_46 },
    { 0x7fb33a65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f33a6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b133a6e, SUC_lexicon_tags_32 },
    { 0x34333a6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3333a79, SUC_lexicon_tags_68 },
    { 0x8b533a79, SUC_lexicon_tags_33 },
    { 0xa2133a79, SUC_lexicon_tags_25 },
    { 0x21533a7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7933a82, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f33a86, SUC_lexicon_tags_27 },
    { 0x31533a86, SUC_lexicon_tags_4 },
    { 0xf6b33a88, SUC_lexicon_tags_11 },
    { 0x70f33a89, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b33a92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14533a95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb33a98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd533a9c, SUC_lexicon_tags_27 },
    { 0x48d33a9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1f33a9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65533aa2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f33aa6, SUC_lexicon_tags_15 },
    { 0xe1533aa7, SUC_lexicon_tags_47 },
    { 0x78933aa8, SUC_lexicon_tags_17 },
    { 0xb2b33aa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17533aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf33ab1, SUC_lexicon_tags_11 },
    { 0xe2f33ab2, SUC_lexicon_tags_25 },
    { 0x3e933ab3, SUC_lexicon_tags_30 },
    { 0x6af33ab1, SUC_lexicon_tags_49 },
    { 0xd6133ab5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26333ab9, SUC_lexicon_tags_25 },
    { 0x89333aba, SUC_lexicon_tags_37 },
    { 0x73d33ab9, SUC_lexicon_tags_65 },
    { 0xcc533abc, SUC_lexicon_tags_11 },
    { 0x85b33abc, SUC_lexicon_tags_27 },
    { 0x24d33abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbd33ac0, SUC_lexicon_tags_25 },
    { 0xc1f33ac1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1bb33ac3, SUC_lexicon_tags_1 },
    { 0x65733ac4, SUC_lexicon_tags_1 },
    { 0x1a133ac5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x20b33ac7, SUC_lexicon_tags_7 },
    { 0xd7733ac8, SUC_lexicon_tags_87 },
    { 0xc5533ac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb33acd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e533ad1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b33ada, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2533adc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x56d33ade, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb4733ae0, SUC_lexicon_tags_37 },
    { 0x33133ae0, SUC_lexicon_tags_12 },
    { 0x55533ae2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef33ae8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21133aea, SUC_lexicon_tags_27 },
    { 0xf9d33aeb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17933af4, SUC_lexicon_tags_0 },
    { 0xa7333af4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75133af7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x47533af9, SUC_lexicon_tags_0 },
    { 0x8933af9, SUC_lexicon_tags_47 },
    { 0xb1733afb, SUC_lexicon_tags_28 },
    { 0xf5b33afa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f33aff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed533b03, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd33b05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb333b0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa0333b0d, SUC_lexicon_tags_15 },
    { 0x21f33b0d, SUC_lexicon_tags_107 },
    { 0x1db33b0e, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff33b14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb33b17, SUC_lexicon_tags_1 },
    { 0xc6f33b17, SUC_lexicon_tags_27 },
    { 0xfbd33b19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf33b1d, SUC_lexicon_tags_3 },
    { 0xe3733b1e, SUC_lexicon_tags_15 },
    { 0x4f133b1e, SUC_lexicon_tags_11 },
    { 0xc8733b20, SUC_lexicon_tags_32 },
    { 0xc2b33b20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4533b25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b33b28, SUC_lexicon_tags_17 },
    { 0xecf33b28, SUC_lexicon_tags_7 },
    { 0x92133b2a, SUC_lexicon_tags_47 },
    { 0x74933b29, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb33b30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bf33b32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb33b37, SUC_lexicon_tags_17 },
    { 0x9ed33b37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4533b3a, SUC_lexicon_tags_82 },
    { 0x51d33b3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17533b40, SUC_lexicon_tags_15 },
    { 0x4c333b41, SUC_lexicon_tags_37 },
    { 0xba933b41, SUC_lexicon_tags_30 },
    { 0xba733b41, SUC_lexicon_tags_179 },
    { 0x22733b44, SUC_lexicon_tags_30 },
    { 0xbbf33b45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b33b48, SUC_lexicon_tags_38 },
    { 0x43f33b49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced33b4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89133b50, SUC_lexicon_tags_15 },
    { 0x13133b50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29933b55, SUC_lexicon_tags_26 },
    { 0x62d33b56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad33b59, SUC_lexicon_tags_1 },
    { 0xe5533b59, SUC_lexicon_tags_11 },
    { 0x77d33b5b, SUC_lexicon_tags_11 },
    { 0xfe333b5c, SUC_lexicon_tags_27 },
    { 0x24333b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3f33b5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf533b61, SUC_lexicon_tags_37 },
    { 0xfa133b62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22933b64, SUC_lexicon_tags_84 },
    { 0x20d33b65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1333b6c, SUC_lexicon_tags_7 },
    { 0xc6f33b6d, SUC_lexicon_tags_30 },
    { 0x31f33b6e, SUC_lexicon_tags_65 },
    { 0xa333b6e, SUC_lexicon_tags_87 },
    { 0x55333b6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b33b7e, SUC_lexicon_tags_1 },
    { 0x94d33b7e, SUC_lexicon_tags_9 },
    { 0x3fb33b80, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x17733b82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d33b87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xde533b89, SUC_lexicon_tags_63 },
    { 0x78333b89, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0733b96, SUC_lexicon_tags_22 },
    { 0x96133b96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x54133b9a, SUC_lexicon_tags_150 },
    { 0xebb33b9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2933b9d, SUC_lexicon_tags_32 },
    { 0x71733b9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc533ba2, SUC_lexicon_tags_12 },
    { 0x5b533ba3, SUC_lexicon_tags_6 },
    { 0x5fd33ba3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa0f33ba6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d933ba9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35933bae, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d33bb8, SUC_lexicon_tags_1 },
    { 0x3c133bb9, SUC_lexicon_tags_12 },
    { 0x29b33bb8, SUC_lexicon_tags_15 },
    { 0xf3f33bbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82733bbf, SUC_lexicon_tags_91 },
    { 0x42733bc0, SUC_lexicon_tags_47 },
    { 0x13933bc0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x82733bc3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10533bcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d33bcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56933bd1, SUC_lexicon_tags_6 },
    { 0x40533bd1, SUC_lexicon_tags_25 },
    { 0xc0133bd2, SUC_lexicon_tags_43 },
    { 0xf9533bd4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d33bde, SUC_lexicon_tags_19 },
    { 0xd733bdf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef33be2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f533be5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0733be9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x31133beb, SUC_lexicon_tags_15 },
    { 0x7b133bec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a533bf2, SUC_lexicon_tags_25 },
    { 0x90933bf3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76133bf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2333bf9, SUC_lexicon_tags_60 },
    { 0xd5533bf9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x42733bfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff733c02, SUC_lexicon_tags_27 },
    { 0x60933c03, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb133c06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54333c08, SUC_lexicon_tags_25 },
    { 0xa9f33c08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d33c0d, SUC_lexicon_tags_2 },
    { 0xc5933c0d, SUC_lexicon_tags_45 },
    { 0x20133c0f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32133c15, SUC_lexicon_tags_29 },
    { 0xbd333c15, SUC_lexicon_tags_37 },
    { 0xaff33c15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3133c19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d33c1d, SUC_lexicon_tags_68 },
    { 0x15b33c1e, SUC_lexicon_tags_17 },
    { 0xefb33c1f, SUC_lexicon_tags_23 },
    { 0x5a733c1f, SUC_lexicon_tags_37 },
    { 0xa7f33c21, SUC_lexicon_tags_7 },
    { 0x9cd33c1f, SUC_lexicon_tags_35 },
    { 0xc7b33c1e, SUC_lexicon_tags_15 },
    { 0xffd33c24, SUC_lexicon_tags_25 },
    { 0x4e533c25, SUC_lexicon_tags_32 },
    { 0x58933c1f, SUC_lexicon_tags_25 },
    { 0xd7733c27, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb5333c29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16d33c2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf333c2f, SUC_lexicon_tags_37 },
    { 0xcef33c2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f33c33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x47133c35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8e333c37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x62133c39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12933c3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc933c45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x29733c48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa533c4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50533c50, SUC_lexicon_tags_25 },
    { 0xe8133c51, SUC_lexicon_tags_25 },
    { 0xbcf33c51, SUC_lexicon_tags_9 },
    { 0xff933c50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91b33c55, SUC_lexicon_tags_17 },
    { 0x4b733c56, SUC_lexicon_tags_14 },
    { 0x4b133c57, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdaf33c59, SUC_lexicon_tags_25 },
    { 0x84733c5a, SUC_lexicon_tags_37 },
    { 0x2e133c5b, SUC_lexicon_tags_29 },
    { 0xfa533c5c, SUC_lexicon_tags_25 },
    { 0x4c733c5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f33c63, SUC_lexicon_tags_4 },
    { 0x4c733c63, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa6d33c66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1df33c68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce333c6a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3333c6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b33c70, SUC_lexicon_tags_19 },
    { 0x71533c70, SUC_lexicon_tags_11 },
    { 0xadd33c71, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d33c76, SUC_lexicon_tags_32 },
    { 0xf3b33c77, SUC_lexicon_tags_55 },
    { 0xad333c78, SUC_lexicon_tags_11 },
    { 0xf0d33c79, SUC_lexicon_tags_65 },
    { 0x16d33c78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe333c83, SUC_lexicon_tags_27 },
    { 0xdb333c84, SUC_lexicon_tags_43 },
    { 0x89333c85, SUC_lexicon_tags_23 },
    { 0xc4d33c86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f33c89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf33c8d, SUC_lexicon_tags_10 },
    { 0xcfd33c8d, SUC_lexicon_tags_17 },
    { 0xd1733c8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc933c92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e133c98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab733c9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9a133c9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65333ca6, SUC_lexicon_tags_49 },
    { 0xe5f33ca7, SUC_lexicon_tags_17 },
    { 0xab733ca8, SUC_lexicon_tags_17 },
    { 0xe8533ca6, SUC_lexicon_tags_37 },
    { 0x54d33ca7, SUC_lexicon_tags_33 },
    { 0x95333cab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4333caf, SUC_lexicon_tags_17 },
    { 0x9d733cb0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4d933cb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72333cb4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c533cb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d33cbd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe333ccd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7333cd0, SUC_lexicon_tags_11 },
    { 0xcb733cd1, SUC_lexicon_tags_11 },
    { 0x79533cd2, SUC_lexicon_tags_0 },
    { 0x2ab33cd1, SUC_lexicon_tags_24 },
    { 0x57b33cd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20333cd6, SUC_lexicon_tags_56 },
    { 0xc6533cd7, SUC_lexicon_tags_49 },
    { 0xbaf33cd6, SUC_lexicon_tags_234 },
    { 0xad533cd9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d33cde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d33ce4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4cb33ce6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35533cea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde733cee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc933cf0, SUC_lexicon_tags_12 },
    { 0x1c933cf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd33cf4, SUC_lexicon_tags_13 },
    { 0x22333cf5, SUC_lexicon_tags_9 },
    { 0x30733cf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d33cfb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b733d01, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xf1533d03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72f33d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95133d07, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5933d15, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x70933d18, SUC_lexicon_tags_1310 },
    { 0, NULL },
    { 0x35f33d1a, SUC_lexicon_tags_49 },
    { 0xa9533d1b, SUC_lexicon_tags_2 },
    { 0xf5333d1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a533d22, SUC_lexicon_tags_25 },
    { 0xcab33d23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b33d26, SUC_lexicon_tags_23 },
    { 0x16333d27, SUC_lexicon_tags_1 },
    { 0xc6133d27, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb33d2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf33d32, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf933d36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81933d38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77333d3d, SUC_lexicon_tags_6 },
    { 0xcb933d3d, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6333d41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x63133d43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabd33d45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48133d47, SUC_lexicon_tags_43 },
    { 0xe4d33d48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d33d4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6133d4f, SUC_lexicon_tags_6 },
    { 0xb6b33d50, SUC_lexicon_tags_4 },
    { 0xd3533d51, SUC_lexicon_tags_66 },
    { 0xb1933d50, SUC_lexicon_tags_47 },
    { 0xcb733d53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52333d56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0933d59, SUC_lexicon_tags_19 },
    { 0xcb333d5a, SUC_lexicon_tags_58 },
    { 0xd8333d5b, SUC_lexicon_tags_25 },
    { 0xadd33d5c, SUC_lexicon_tags_2 },
    { 0x90133d5b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff33d64, SUC_lexicon_tags_15 },
    { 0x91733d64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f33d67, SUC_lexicon_tags_60 },
    { 0x10f33d68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x22b33d6a, SUC_lexicon_tags_37 },
    { 0x67f33d6b, SUC_lexicon_tags_15 },
    { 0x72933d6c, SUC_lexicon_tags_39 },
    { 0x31733d6d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1b533d6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b933d72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68933d76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd33d7b, SUC_lexicon_tags_0 },
    { 0x3ad33d7c, SUC_lexicon_tags_47 },
    { 0x11b33d7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66933d85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b33d8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c733d94, SUC_lexicon_tags_7 },
    { 0x9f33d94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3733d97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f33d9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8f33d9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d733da0, SUC_lexicon_tags_37 },
    { 0x2333da0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdfd33da3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5533da7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef33daa, SUC_lexicon_tags_28 },
    { 0x4ad33dab, SUC_lexicon_tags_63 },
    { 0x98b33dab, SUC_lexicon_tags_34 },
    { 0x5ab33dad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d33db5, SUC_lexicon_tags_3 },
    { 0x8f533db6, SUC_lexicon_tags_19 },
    { 0x9c133db6, SUC_lexicon_tags_27 },
    { 0x49933db8, SUC_lexicon_tags_25 },
    { 0xa7f33db9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4b533dbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf933dbd, SUC_lexicon_tags_11 },
    { 0x56133dbd, SUC_lexicon_tags_15 },
    { 0x87f33dbf, SUC_lexicon_tags_17 },
    { 0xbb33dc0, SUC_lexicon_tags_53 },
    { 0xe6333dc1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xadd33dc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0133dcc, SUC_lexicon_tags_11 },
    { 0x4a933dcc, SUC_lexicon_tags_1 },
    { 0x93d33dcc, SUC_lexicon_tags_1 },
    { 0xded33dcf, SUC_lexicon_tags_1 },
    { 0x15d33dcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa333dd2, SUC_lexicon_tags_2 },
    { 0x3133dd3, SUC_lexicon_tags_14 },
    { 0x8b533dd4, SUC_lexicon_tags_1 },
    { 0xf4f33dd5, SUC_lexicon_tags_34 },
    { 0x82f33dd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a733dd8, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x5bb33dda, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb333dde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f533de0, SUC_lexicon_tags_11 },
    { 0x5c133de0, SUC_lexicon_tags_49 },
    { 0x6d533de2, SUC_lexicon_tags_23 },
    { 0x2e533de3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b933de6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc533de8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5333dea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe3b33dec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad33df0, SUC_lexicon_tags_11 },
    { 0x79733df1, SUC_lexicon_tags_2 },
    { 0x1c333df1, SUC_lexicon_tags_7 },
    { 0xc7333df0, SUC_lexicon_tags_25 },
    { 0x77f33df4, SUC_lexicon_tags_27 },
    { 0x7b333df5, SUC_lexicon_tags_2 },
    { 0x55333df6, SUC_lexicon_tags_28 },
    { 0xf9b33df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0133dfa, SUC_lexicon_tags_15 },
    { 0x3bd33dfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8333dfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9933e02, SUC_lexicon_tags_1 },
    { 0xa7533e03, SUC_lexicon_tags_63 },
    { 0x5d333e04, SUC_lexicon_tags_17 },
    { 0x3fb33e05, SUC_lexicon_tags_6 },
    { 0x83b33e06, SUC_lexicon_tags_15 },
    { 0xc1733e07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2533e09, SUC_lexicon_tags_9 },
    { 0x4ff33e0a, SUC_lexicon_tags_6 },
    { 0x5f33e09, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c333e0e, SUC_lexicon_tags_46 },
    { 0xbef33e0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b333e13, SUC_lexicon_tags_7 },
    { 0xa7b33e14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3933e18, SUC_lexicon_tags_17 },
    { 0xf5533e19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e333e1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47533e1f, SUC_lexicon_tags_25 },
    { 0x7b333e20, SUC_lexicon_tags_49 },
    { 0x96d33e21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b33e24, SUC_lexicon_tags_23 },
    { 0xe7f33e25, SUC_lexicon_tags_25 },
    { 0xd1b33e25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0d33e28, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x42333e2a, SUC_lexicon_tags_7 },
    { 0xe8533e2a, SUC_lexicon_tags_11 },
    { 0x8b133e2b, SUC_lexicon_tags_7 },
    { 0xaf733e2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd33e31, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb0b33e33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e733e36, SUC_lexicon_tags_6 },
    { 0xfc733e37, SUC_lexicon_tags_9 },
    { 0x4f533e38, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b33e3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59133e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7733e44, SUC_lexicon_tags_33 },
    { 0x61f33e45, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb5933e47, SUC_lexicon_tags_15 },
    { 0x64d33e47, SUC_lexicon_tags_37 },
    { 0xb4933e49, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd33e4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d33e56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x99333e58, SUC_lexicon_tags_2 },
    { 0xd5b33e59, SUC_lexicon_tags_15 },
    { 0x68133e59, SUC_lexicon_tags_29 },
    { 0xd4333e5b, SUC_lexicon_tags_25 },
    { 0xa2933e5b, SUC_lexicon_tags_40 },
    { 0x1eb33e59, SUC_lexicon_tags_27 },
    { 0x17733e5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc9133e60, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf33e64, SUC_lexicon_tags_8 },
    { 0x37733e65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6933e68, SUC_lexicon_tags_50 },
    { 0x27d33e68, SUC_lexicon_tags_2 },
    { 0x21533e69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27333e73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27133e77, SUC_lexicon_tags_38 },
    { 0x6f533e78, SUC_lexicon_tags_13 },
    { 0xcc333e78, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5cf33e7b, SUC_lexicon_tags_37 },
    { 0x41f33e7c, SUC_lexicon_tags_2 },
    { 0xcbd33e7d, SUC_lexicon_tags_19 },
    { 0x39933e7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d33e81, SUC_lexicon_tags_2 },
    { 0x88933e82, SUC_lexicon_tags_11 },
    { 0xedf33e81, SUC_lexicon_tags_1 },
    { 0xf33e84, SUC_lexicon_tags_92 },
    { 0xe9133e84, SUC_lexicon_tags_17 },
    { 0x14d33e83, SUC_lexicon_tags_37 },
    { 0xf1d33e82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3533e8b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45133e90, SUC_lexicon_tags_3 },
    { 0xf7733e90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f33e97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36333e99, SUC_lexicon_tags_14 },
    { 0x46133e99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f33e9e, SUC_lexicon_tags_1095 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d33ea1, SUC_lexicon_tags_70 },
    { 0x91133ea2, SUC_lexicon_tags_17 },
    { 0xb0133ea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c733ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60333eaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3333eac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b333eb2, SUC_lexicon_tags_12 },
    { 0x13733eb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44133ebe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd33ec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3133ec3, SUC_lexicon_tags_19 },
    { 0x4e733ec4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f33ec7, SUC_lexicon_tags_37 },
    { 0x6f333ec7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd33ecd, SUC_lexicon_tags_12 },
    { 0xd2133ece, SUC_lexicon_tags_55 },
    { 0x2e933ecf, SUC_lexicon_tags_47 },
    { 0x98333ece, SUC_lexicon_tags_31 },
    { 0x5d33ece, SUC_lexicon_tags_7 },
    { 0xf7b33ed2, SUC_lexicon_tags_29 },
    { 0x9af33ed3, SUC_lexicon_tags_9 },
    { 0xacf33ed4, SUC_lexicon_tags_45 },
    { 0x6e933ed5, SUC_lexicon_tags_45 },
    { 0x13533ed6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33333ed8, SUC_lexicon_tags_7 },
    { 0x94b33ed9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x47333edb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1333ee1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0133ee3, SUC_lexicon_tags_221 },
    { 0x51133ee4, SUC_lexicon_tags_46 },
    { 0x5bb33ee5, SUC_lexicon_tags_33 },
    { 0x16933ee6, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b33ee9, SUC_lexicon_tags_37 },
    { 0x17d33eea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf33eed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64733eef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdf33ef1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d533ef8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d33efc, SUC_lexicon_tags_1 },
    { 0x5e133efd, SUC_lexicon_tags_11 },
    { 0x52f33efe, SUC_lexicon_tags_47 },
    { 0x3cd33efe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab33f02, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa1533f04, SUC_lexicon_tags_2 },
    { 0x2ad33f04, SUC_lexicon_tags_16 },
    { 0xb7733f06, SUC_lexicon_tags_58 },
    { 0xb133f06, SUC_lexicon_tags_29 },
    { 0x39133f08, SUC_lexicon_tags_15 },
    { 0x78133f09, SUC_lexicon_tags_21 },
    { 0x90d33f0a, SUC_lexicon_tags_19 },
    { 0xd0333f0b, SUC_lexicon_tags_39 },
    { 0xbc733f05, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd33f18, SUC_lexicon_tags_12 },
    { 0x5bf33f18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf33f1d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x80f33f1f, SUC_lexicon_tags_2 },
    { 0xafb33f20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67533f26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e933f2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9bd33f2c, SUC_lexicon_tags_37 },
    { 0x4733f2d, SUC_lexicon_tags_43 },
    { 0xf1b33f2c, SUC_lexicon_tags_38 },
    { 0x85d33f2e, SUC_lexicon_tags_7 },
    { 0xeef33f30, SUC_lexicon_tags_39 },
    { 0xeed33f31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4f33f33, SUC_lexicon_tags_32 },
    { 0x5db33f34, SUC_lexicon_tags_2 },
    { 0x66533f35, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xce133f38, SUC_lexicon_tags_2 },
    { 0x61933f38, SUC_lexicon_tags_95 },
    { 0xe0733f39, SUC_lexicon_tags_0 },
    { 0xca933f3b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7933f41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb733f48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6133f4b, SUC_lexicon_tags_38 },
    { 0x59533f4c, SUC_lexicon_tags_31 },
    { 0x90733f4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5533f54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96b33f56, SUC_lexicon_tags_12 },
    { 0x6f33f56, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf1533f59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x40733f5c, SUC_lexicon_tags_0 },
    { 0xaff33f5c, SUC_lexicon_tags_6 },
    { 0xd9d33f5d, SUC_lexicon_tags_37 },
    { 0x4d933f5f, SUC_lexicon_tags_16 },
    { 0x6bb33f60, SUC_lexicon_tags_14 },
    { 0x39533f5c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf4533f63, SUC_lexicon_tags_26 },
    { 0xff33f64, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee133f68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe133f6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9fd33f71, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d33f75, SUC_lexicon_tags_14 },
    { 0x72733f76, SUC_lexicon_tags_11 },
    { 0xef933f77, SUC_lexicon_tags_11 },
    { 0xf4333f76, SUC_lexicon_tags_13 },
    { 0x1ff33f79, SUC_lexicon_tags_15 },
    { 0x3c333f7a, SUC_lexicon_tags_37 },
    { 0x8733f7a, SUC_lexicon_tags_47 },
    { 0xc0333f7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4333f83, SUC_lexicon_tags_60 },
    { 0x67f33f84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd533f93, SUC_lexicon_tags_75 },
    { 0xe9f33f94, SUC_lexicon_tags_31 },
    { 0x8dd33f94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f733f97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72133f9c, SUC_lexicon_tags_30 },
    { 0x13133f9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76533f9f, SUC_lexicon_tags_0 },
    { 0xb7b33f9f, SUC_lexicon_tags_43 },
    { 0x92533fa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84733fa4, SUC_lexicon_tags_37 },
    { 0x3eb33fa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a133fa7, SUC_lexicon_tags_38 },
    { 0x1ad33fa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c733fad, SUC_lexicon_tags_28 },
    { 0x91533fae, SUC_lexicon_tags_28 },
    { 0x9b133faf, SUC_lexicon_tags_12 },
    { 0xf9933fae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf33fb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe533fba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26933fbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd4d33fbe, SUC_lexicon_tags_37 },
    { 0x2bd33fbe, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xe3733fc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70b33fc3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32933fc9, SUC_lexicon_tags_37 },
    { 0xefb33fca, SUC_lexicon_tags_15 },
    { 0xe133fc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b33fcf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6ed33fd1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x41d33fd3, SUC_lexicon_tags_10 },
    { 0xf4d33fd4, SUC_lexicon_tags_11 },
    { 0x87533fd5, SUC_lexicon_tags_26 },
    { 0x97b33fd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f33fd9, SUC_lexicon_tags_15 },
    { 0x4f533fda, SUC_lexicon_tags_11 },
    { 0x5c333fdb, SUC_lexicon_tags_204 },
    { 0xd3133fdc, SUC_lexicon_tags_2 },
    { 0x9b733fdb, SUC_lexicon_tags_6 },
    { 0x6b133fd9, SUC_lexicon_tags_91 },
    { 0x5a733fdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf733fe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe333fe4, SUC_lexicon_tags_45 },
    { 0xd0933fe5, SUC_lexicon_tags_84 },
    { 0x44133fe5, SUC_lexicon_tags_15 },
    { 0x2c333fe4, SUC_lexicon_tags_85 },
    { 0x2b733fe7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4cd33fea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ab33fec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88733ff1, SUC_lexicon_tags_56 },
    { 0xef933ff2, SUC_lexicon_tags_65 },
    { 0x9df33ff3, SUC_lexicon_tags_17 },
    { 0x65b33ff4, SUC_lexicon_tags_37 },
    { 0xd7b33ff4, SUC_lexicon_tags_6 },
    { 0xe3b33ff6, SUC_lexicon_tags_16 },
    { 0xa2f33ff6, SUC_lexicon_tags_15 },
    { 0xf0933ff8, SUC_lexicon_tags_37 },
    { 0xf0133ff5, SUC_lexicon_tags_11 },
    { 0x35533ffa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x87f33ffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35333fff, SUC_lexicon_tags_1 },
    { 0xfa734000, SUC_lexicon_tags_14 },
    { 0x2d33fff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74134003, SUC_lexicon_tags_8 },
    { 0xcb134004, SUC_lexicon_tags_33 },
    { 0xd6734005, SUC_lexicon_tags_28 },
    { 0x95b34006, SUC_lexicon_tags_17 },
    { 0xb8734007, SUC_lexicon_tags_15 },
    { 0xbe934008, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f34012, SUC_lexicon_tags_37 },
    { 0xa0d34013, SUC_lexicon_tags_13 },
    { 0x10334012, SUC_lexicon_tags_37 },
    { 0x39334015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7134019, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f934020, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd34026, SUC_lexicon_tags_25 },
    { 0x68734027, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f3402b, SUC_lexicon_tags_21 },
    { 0x61f3402c, SUC_lexicon_tags_12 },
    { 0x4f53402d, SUC_lexicon_tags_29 },
    { 0x4a53402d, SUC_lexicon_tags_12 },
    { 0x49d3402e, SUC_lexicon_tags_31 },
    { 0xfa934030, SUC_lexicon_tags_60 },
    { 0xd6534031, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfe134033, SUC_lexicon_tags_1 },
    { 0xaad34034, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2734036, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8734038, SUC_lexicon_tags_19 },
    { 0x65f34039, SUC_lexicon_tags_60 },
    { 0xe7134039, SUC_lexicon_tags_41 },
    { 0xcf334038, SUC_lexicon_tags_2 },
    { 0x29d3403c, SUC_lexicon_tags_44 },
    { 0x1d13403d, SUC_lexicon_tags_29 },
    { 0xfed3403c, SUC_lexicon_tags_7 },
    { 0x57d3403f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9c334041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20334047, SUC_lexicon_tags_25 },
    { 0x29734047, SUC_lexicon_tags_3 },
    { 0x21b34049, SUC_lexicon_tags_19 },
    { 0x973404a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3893404d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d134050, SUC_lexicon_tags_12 },
    { 0x92934051, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef334056, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ab34058, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58f3405a, SUC_lexicon_tags_17 },
    { 0xe5b3405b, SUC_lexicon_tags_63 },
    { 0x2e13405b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc134060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7934063, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd34067, SUC_lexicon_tags_15 },
    { 0xb6334068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1f3406a, SUC_lexicon_tags_36 },
    { 0x3073406a, SUC_lexicon_tags_12 },
    { 0xae13406a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1934071, SUC_lexicon_tags_49 },
    { 0x19534071, SUC_lexicon_tags_97 },
    { 0xd8334073, SUC_lexicon_tags_118 },
    { 0xa0334074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d3407b, SUC_lexicon_tags_28 },
    { 0x8ef3407b, SUC_lexicon_tags_37 },
    { 0x3bb3407c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x78934080, SUC_lexicon_tags_47 },
    { 0xd6534081, SUC_lexicon_tags_129 },
    { 0x6cd34082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd34085, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8f534087, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b3408c, SUC_lexicon_tags_87 },
    { 0x813408d, SUC_lexicon_tags_35 },
    { 0xdf13408c, SUC_lexicon_tags_46 },
    { 0xedb3408c, SUC_lexicon_tags_11 },
    { 0x5cb3408e, SUC_lexicon_tags_19 },
    { 0x8bd34091, SUC_lexicon_tags_7 },
    { 0x8ef34092, SUC_lexicon_tags_17 },
    { 0x5e73408e, SUC_lexicon_tags_21 },
    { 0x34934094, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f734098, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b3409c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5340a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x597340a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb340a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8af340a8, SUC_lexicon_tags_47 },
    { 0x6f5340a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f340ac, SUC_lexicon_tags_37 },
    { 0xf4d340ad, SUC_lexicon_tags_16 },
    { 0x37b340ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x389340b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8f340b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xacd340b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cd340b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17340be, SUC_lexicon_tags_8 },
    { 0x8f1340be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x669340c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03340c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8df340c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71340cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa97340ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69340d2, SUC_lexicon_tags_7 },
    { 0x6c1340d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x575340d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x853340d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443340db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9340df, SUC_lexicon_tags_17 },
    { 0x815340df, SUC_lexicon_tags_37 },
    { 0x995340e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b340ea, SUC_lexicon_tags_7 },
    { 0xc3f340ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5340ef, SUC_lexicon_tags_7 },
    { 0x631340f0, SUC_lexicon_tags_15 },
    { 0xf4b340f1, SUC_lexicon_tags_25 },
    { 0xecf340f2, SUC_lexicon_tags_16 },
    { 0x925340f3, SUC_lexicon_tags_133 },
    { 0x1b7340f2, SUC_lexicon_tags_38 },
    { 0xaa1340f5, SUC_lexicon_tags_46 },
    { 0x6f7340f2, SUC_lexicon_tags_38 },
    { 0x717340f7, SUC_lexicon_tags_37 },
    { 0x26f340f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d34105, SUC_lexicon_tags_12 },
    { 0x31734106, SUC_lexicon_tags_7 },
    { 0x29b34107, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd4b34109, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23d3410b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd3410d, SUC_lexicon_tags_61 },
    { 0xdb3410e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x29f34110, SUC_lexicon_tags_87 },
    { 0xf3f34111, SUC_lexicon_tags_4 },
    { 0xb2534112, SUC_lexicon_tags_38 },
    { 0x72534110, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8534117, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8d34119, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad3411d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6033411f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50934123, SUC_lexicon_tags_2 },
    { 0xf6f34123, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b34127, SUC_lexicon_tags_7 },
    { 0xb3f34127, SUC_lexicon_tags_7 },
    { 0xe3334129, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeaf3412b, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5633412f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66534133, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f134136, SUC_lexicon_tags_136 },
    { 0xe6b34136, SUC_lexicon_tags_17 },
    { 0xb8934136, SUC_lexicon_tags_23 },
    { 0xd1734138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8173413c, SUC_lexicon_tags_28 },
    { 0x67f3413d, SUC_lexicon_tags_31 },
    { 0xd033413e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4134141, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf34146, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb3414a, SUC_lexicon_tags_25 },
    { 0xaaf3414b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3534150, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6d34152, SUC_lexicon_tags_2 },
    { 0xaf34153, SUC_lexicon_tags_16 },
    { 0xddf34153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd34157, SUC_lexicon_tags_7 },
    { 0xd534158, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b3415b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2633415e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b34165, SUC_lexicon_tags_17 },
    { 0xcaf34166, SUC_lexicon_tags_45 },
    { 0x67734167, SUC_lexicon_tags_14 },
    { 0xfa134168, SUC_lexicon_tags_25 },
    { 0x1c334169, SUC_lexicon_tags_8 },
    { 0xfb3416a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7953416c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17334172, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56134174, SUC_lexicon_tags_0 },
    { 0x9534175, SUC_lexicon_tags_6 },
    { 0x2ef34176, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5134178, SUC_lexicon_tags_55 },
    { 0x8ed34179, SUC_lexicon_tags_11 },
    { 0x6934179, SUC_lexicon_tags_29 },
    { 0x5c53417b, SUC_lexicon_tags_15 },
    { 0x6833417c, SUC_lexicon_tags_82 },
    { 0x2773417d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f34183, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x25934186, SUC_lexicon_tags_1 },
    { 0xec134186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd34196, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73419c, SUC_lexicon_tags_17 },
    { 0xe6d3419d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f341a2, SUC_lexicon_tags_53 },
    { 0xbaf341a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab341a8, SUC_lexicon_tags_17 },
    { 0xe6d341a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x397341ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45341b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75341b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe05341b6, SUC_lexicon_tags_17 },
    { 0x30b341b6, SUC_lexicon_tags_29 },
    { 0x385341b8, SUC_lexicon_tags_43 },
    { 0x231341b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x403341bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb95341be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3341c5, SUC_lexicon_tags_17 },
    { 0x611341c5, SUC_lexicon_tags_15 },
    { 0x4cb341c6, SUC_lexicon_tags_15 },
    { 0x7b9341c5, SUC_lexicon_tags_17 },
    { 0x1d9341c6, SUC_lexicon_tags_14 },
    { 0x7fb341c6, SUC_lexicon_tags_26 },
    { 0x23f341cb, SUC_lexicon_tags_14 },
    { 0xe1f341c7, SUC_lexicon_tags_17 },
    { 0xa7b341cd, SUC_lexicon_tags_47 },
    { 0x597341ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed341d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b341da, SUC_lexicon_tags_43 },
    { 0xda1341db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3341df, SUC_lexicon_tags_7 },
    { 0x4d9341e0, SUC_lexicon_tags_14 },
    { 0xa77341e1, SUC_lexicon_tags_12 },
    { 0x861341e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39341e5, SUC_lexicon_tags_29 },
    { 0xbc5341e6, SUC_lexicon_tags_1 },
    { 0xfe1341e7, SUC_lexicon_tags_14 },
    { 0xd4b341e7, SUC_lexicon_tags_9 },
    { 0xfa5341e9, SUC_lexicon_tags_15 },
    { 0x273341ea, SUC_lexicon_tags_38 },
    { 0x519341e8, SUC_lexicon_tags_11 },
    { 0xdc1341e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173341f0, SUC_lexicon_tags_25 },
    { 0xe6b341f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91341f5, SUC_lexicon_tags_37 },
    { 0xd9341f5, SUC_lexicon_tags_43 },
    { 0xd29341f7, SUC_lexicon_tags_25 },
    { 0x65b341f5, SUC_lexicon_tags_47 },
    { 0x93f341f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd341fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16f341ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f134206, SUC_lexicon_tags_17 },
    { 0x26534207, SUC_lexicon_tags_15 },
    { 0x12134207, SUC_lexicon_tags_49 },
    { 0xd4934207, SUC_lexicon_tags_12 },
    { 0x6eb3420a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd973420c, SUC_lexicon_tags_6 },
    { 0x8e13420c, SUC_lexicon_tags_45 },
    { 0xc273420e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd34211, SUC_lexicon_tags_27 },
    { 0x66934212, SUC_lexicon_tags_38 },
    { 0x60134213, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75334217, SUC_lexicon_tags_1 },
    { 0x11734218, SUC_lexicon_tags_14 },
    { 0xcdb34219, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xbbd3421b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b53421e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7a134220, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84134222, SUC_lexicon_tags_65 },
    { 0xff134223, SUC_lexicon_tags_15 },
    { 0xedb34224, SUC_lexicon_tags_49 },
    { 0x4db34225, SUC_lexicon_tags_37 },
    { 0x4e734224, SUC_lexicon_tags_23 },
    { 0x9734227, SUC_lexicon_tags_37 },
    { 0xf5334224, SUC_lexicon_tags_60 },
    { 0xa0f34229, SUC_lexicon_tags_49 },
    { 0xdcf3422a, SUC_lexicon_tags_25 },
    { 0x12f3422a, SUC_lexicon_tags_39 },
    { 0x8093422a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa893422e, SUC_lexicon_tags_37 },
    { 0xc373422e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf6534231, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf593423e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55134240, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef34246, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea734248, SUC_lexicon_tags_5 },
    { 0x95d34249, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b73424c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62534251, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c734254, SUC_lexicon_tags_68 },
    { 0x8cd34255, SUC_lexicon_tags_23 },
    { 0x10134256, SUC_lexicon_tags_27 },
    { 0x1c134257, SUC_lexicon_tags_37 },
    { 0xae934258, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xda13425a, SUC_lexicon_tags_29 },
    { 0x3b33425b, SUC_lexicon_tags_1 },
    { 0xf253425a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c534262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cd34264, SUC_lexicon_tags_19 },
    { 0xc0734265, SUC_lexicon_tags_11 },
    { 0xb8f34265, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49534268, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe153426c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a134273, SUC_lexicon_tags_23 },
    { 0xc8534274, SUC_lexicon_tags_11 },
    { 0xdc534273, SUC_lexicon_tags_55 },
    { 0xb7134276, SUC_lexicon_tags_47 },
    { 0xf4f34275, SUC_lexicon_tags_44 },
    { 0xd9534274, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3734280, SUC_lexicon_tags_49 },
    { 0xcd34280, SUC_lexicon_tags_2 },
    { 0x43b34280, SUC_lexicon_tags_47 },
    { 0x67f34281, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x38d34285, SUC_lexicon_tags_37 },
    { 0x1d934285, SUC_lexicon_tags_14 },
    { 0xe334287, SUC_lexicon_tags_6 },
    { 0x15b34288, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc513428a, SUC_lexicon_tags_15 },
    { 0x27b3428b, SUC_lexicon_tags_87 },
    { 0x4c73428b, SUC_lexicon_tags_37 },
    { 0x8873428a, SUC_lexicon_tags_23 },
    { 0xdf13428b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f34295, SUC_lexicon_tags_45 },
    { 0x2ff34296, SUC_lexicon_tags_77 },
    { 0xef134297, SUC_lexicon_tags_151 },
    { 0x1e134298, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b53429a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7913429d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73342a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b342a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63342ad, SUC_lexicon_tags_10 },
    { 0x6d5342ae, SUC_lexicon_tags_80 },
    { 0x34f342af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d342b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d342b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x637342b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9342bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85f342bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc11342c1, SUC_lexicon_tags_15 },
    { 0x2df342c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa97342c4, SUC_lexicon_tags_16 },
    { 0xa07342c5, SUC_lexicon_tags_43 },
    { 0x419342c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc2b342c8, SUC_lexicon_tags_12 },
    { 0x321342c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x247342cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa03342cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc57342cf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7fd342d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf17342d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x125342d6, SUC_lexicon_tags_43 },
    { 0x845342d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927342dc, SUC_lexicon_tags_14 },
    { 0x64d342dc, SUC_lexicon_tags_2 },
    { 0xa8d342dc, SUC_lexicon_tags_27 },
    { 0x943342de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc99342e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x607342e3, SUC_lexicon_tags_17 },
    { 0xc07342e4, SUC_lexicon_tags_68 },
    { 0xcf7342e4, SUC_lexicon_tags_56 },
    { 0x659342e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8f342e8, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x59f342ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe45342ec, SUC_lexicon_tags_25 },
    { 0x3cb342ec, SUC_lexicon_tags_7 },
    { 0xdf9342ec, SUC_lexicon_tags_16 },
    { 0xd89342ef, SUC_lexicon_tags_39 },
    { 0x283342ef, SUC_lexicon_tags_12 },
    { 0xeab342f1, SUC_lexicon_tags_1 },
    { 0xfb342f1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x43342f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d342f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf31342fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d34306, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf34312, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2b34314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5534316, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ad34318, SUC_lexicon_tags_22 },
    { 0x7b334319, SUC_lexicon_tags_15 },
    { 0x9d34319, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4293431d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b34322, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4334326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa633432a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a334330, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d734333, SUC_lexicon_tags_25 },
    { 0xcbf34334, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc534337, SUC_lexicon_tags_1 },
    { 0xa3734338, SUC_lexicon_tags_38 },
    { 0x6ff34339, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7013433b, SUC_lexicon_tags_38 },
    { 0x4c93433c, SUC_lexicon_tags_1 },
    { 0xab3433b, SUC_lexicon_tags_1 },
    { 0x9013433e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f34344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42134346, SUC_lexicon_tags_15 },
    { 0xe5534347, SUC_lexicon_tags_0 },
    { 0xb3334347, SUC_lexicon_tags_1 },
    { 0x9c934349, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb373434c, SUC_lexicon_tags_47 },
    { 0xd8b3434d, SUC_lexicon_tags_15 },
    { 0x1d33434e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf934350, SUC_lexicon_tags_32 },
    { 0xe0534351, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61134353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb534356, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd734358, SUC_lexicon_tags_12 },
    { 0x56734359, SUC_lexicon_tags_37 },
    { 0x23d3435a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b34360, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b34363, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1f34365, SUC_lexicon_tags_11 },
    { 0xc134366, SUC_lexicon_tags_28 },
    { 0xc7134367, SUC_lexicon_tags_32 },
    { 0x51534368, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b3436c, SUC_lexicon_tags_27 },
    { 0xcb53436d, SUC_lexicon_tags_1 },
    { 0xb7b3436d, SUC_lexicon_tags_6 },
    { 0x6a13436f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b34377, SUC_lexicon_tags_47 },
    { 0x19134378, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873437d, SUC_lexicon_tags_1 },
    { 0x9713437e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b934383, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16534385, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x16334387, SUC_lexicon_tags_65 },
    { 0xff334388, SUC_lexicon_tags_7 },
    { 0x55134387, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f33438c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb134390, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb34395, SUC_lexicon_tags_17 },
    { 0x3cf34395, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76734399, SUC_lexicon_tags_19 },
    { 0xb0f34399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb3439d, SUC_lexicon_tags_49 },
    { 0x3d13439d, SUC_lexicon_tags_60 },
    { 0xe953439f, SUC_lexicon_tags_15 },
    { 0xb29343a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63343a5, SUC_lexicon_tags_27 },
    { 0x1bf343a6, SUC_lexicon_tags_11 },
    { 0x413343a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdd9343a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87343ac, SUC_lexicon_tags_0 },
    { 0x1f3343ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501343b5, SUC_lexicon_tags_15 },
    { 0xe55343b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd343b9, SUC_lexicon_tags_2 },
    { 0xa0f343ba, SUC_lexicon_tags_25 },
    { 0xc07343ba, SUC_lexicon_tags_17 },
    { 0x491343bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f3343be, SUC_lexicon_tags_23 },
    { 0x453343be, SUC_lexicon_tags_25 },
    { 0xda9343c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667343c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab343c9, SUC_lexicon_tags_11 },
    { 0x787343c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xba3343cc, SUC_lexicon_tags_37 },
    { 0x17343cd, SUC_lexicon_tags_21 },
    { 0x16d343ce, SUC_lexicon_tags_43 },
    { 0x4b343ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25343d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1343d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb95343d7, SUC_lexicon_tags_2 },
    { 0x67343d8, SUC_lexicon_tags_25 },
    { 0x8bf343d8, SUC_lexicon_tags_46 },
    { 0x9ef343da, SUC_lexicon_tags_11 },
    { 0x39b343db, SUC_lexicon_tags_7 },
    { 0x663343dc, SUC_lexicon_tags_14 },
    { 0x55b343dd, SUC_lexicon_tags_0 },
    { 0x2bf343de, SUC_lexicon_tags_0 },
    { 0x257343df, SUC_lexicon_tags_17 },
    { 0x8d5343e0, SUC_lexicon_tags_6 },
    { 0xba7343e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f343e4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81343ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9343ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69343f3, SUC_lexicon_tags_38 },
    { 0xb9f343f4, SUC_lexicon_tags_27 },
    { 0xc77343f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57b343f7, SUC_lexicon_tags_60 },
    { 0x439343f8, SUC_lexicon_tags_23 },
    { 0x405343f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2b343fb, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1934407, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb3f34409, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb413440b, SUC_lexicon_tags_38 },
    { 0x2b3440c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d34411, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d34414, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f334417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af3441a, SUC_lexicon_tags_25 },
    { 0x98d3441a, SUC_lexicon_tags_4 },
    { 0xb6f3441b, SUC_lexicon_tags_33 },
    { 0x6913441d, SUC_lexicon_tags_25 },
    { 0x9713441d, SUC_lexicon_tags_1 },
    { 0x6253441f, SUC_lexicon_tags_30 },
    { 0x8af3441f, SUC_lexicon_tags_49 },
    { 0xa4134421, SUC_lexicon_tags_123 },
    { 0xa6d34422, SUC_lexicon_tags_11 },
    { 0x5dd3441e, SUC_lexicon_tags_51 },
    { 0x3213441d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf193442a, SUC_lexicon_tags_4 },
    { 0x31d3442a, SUC_lexicon_tags_17 },
    { 0x3dd3442b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6853442f, SUC_lexicon_tags_12 },
    { 0xd7d3442f, SUC_lexicon_tags_37 },
    { 0xbf73442f, SUC_lexicon_tags_19 },
    { 0x4df34432, SUC_lexicon_tags_47 },
    { 0x68534433, SUC_lexicon_tags_37 },
    { 0x10734432, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85934439, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b34441, SUC_lexicon_tags_17 },
    { 0x79534441, SUC_lexicon_tags_4 },
    { 0xf1d34443, SUC_lexicon_tags_45 },
    { 0xc2b34444, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd4534446, SUC_lexicon_tags_17 },
    { 0x89334447, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b3444b, SUC_lexicon_tags_32 },
    { 0x4f93444b, SUC_lexicon_tags_17 },
    { 0xf9f3444d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59134452, SUC_lexicon_tags_2 },
    { 0x86134453, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a934458, SUC_lexicon_tags_49 },
    { 0x9c734459, SUC_lexicon_tags_38 },
    { 0x3233445a, SUC_lexicon_tags_65 },
    { 0x85b3445b, SUC_lexicon_tags_27 },
    { 0xd873445b, SUC_lexicon_tags_19 },
    { 0x7a33445b, SUC_lexicon_tags_11 },
    { 0x3a73445b, SUC_lexicon_tags_25 },
    { 0x9d53445f, SUC_lexicon_tags_25 },
    { 0x19b3445d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b34464, SUC_lexicon_tags_0 },
    { 0x8e134464, SUC_lexicon_tags_34 },
    { 0x30f34466, SUC_lexicon_tags_45 },
    { 0xfcf34467, SUC_lexicon_tags_12 },
    { 0xa3f34468, SUC_lexicon_tags_2 },
    { 0x1f934469, SUC_lexicon_tags_27 },
    { 0xcd334467, SUC_lexicon_tags_7 },
    { 0x1b13446b, SUC_lexicon_tags_17 },
    { 0x83d3446b, SUC_lexicon_tags_7 },
    { 0x5933446d, SUC_lexicon_tags_17 },
    { 0x42134467, SUC_lexicon_tags_17 },
    { 0xd2d3446f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xf7734471, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6a134473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e134475, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18734478, SUC_lexicon_tags_1 },
    { 0x2d734478, SUC_lexicon_tags_13 },
    { 0x5553447a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb3447d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7f3447f, SUC_lexicon_tags_17 },
    { 0x45534480, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d34484, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17934489, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb13448d, SUC_lexicon_tags_1 },
    { 0xb233448e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf34492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f34496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3334498, SUC_lexicon_tags_30 },
    { 0x71b34499, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13f3449b, SUC_lexicon_tags_17 },
    { 0x2cd3449b, SUC_lexicon_tags_13 },
    { 0xca53449d, SUC_lexicon_tags_15 },
    { 0x19f3449e, SUC_lexicon_tags_37 },
    { 0x6a13449f, SUC_lexicon_tags_30 },
    { 0x22d344a0, SUC_lexicon_tags_37 },
    { 0xccd3449f, SUC_lexicon_tags_30 },
    { 0xd9b344a2, SUC_lexicon_tags_9 },
    { 0x9773449b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x599344a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e5344a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1344ab, SUC_lexicon_tags_28 },
    { 0xb344ac, SUC_lexicon_tags_17 },
    { 0x7ab344ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xabb344af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01344b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791344ba, SUC_lexicon_tags_17 },
    { 0x835344bb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xffb344bd, SUC_lexicon_tags_87 },
    { 0x6af344be, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1344c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a5344c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x929344c5, SUC_lexicon_tags_37 },
    { 0xf79344c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71b344c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7344cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d344d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b344d9, SUC_lexicon_tags_63 },
    { 0x49344da, SUC_lexicon_tags_32 },
    { 0xdc9344db, SUC_lexicon_tags_15 },
    { 0x2e3344db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdfb344de, SUC_lexicon_tags_17 },
    { 0xe2b344df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe8b344e1, SUC_lexicon_tags_45 },
    { 0xeef344e1, SUC_lexicon_tags_26 },
    { 0xb0b344e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf344e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285344ea, SUC_lexicon_tags_25 },
    { 0xb55344eb, SUC_lexicon_tags_47 },
    { 0xbcb344ec, SUC_lexicon_tags_12 },
    { 0xd1d344ed, SUC_lexicon_tags_37 },
    { 0x66d344ee, SUC_lexicon_tags_17 },
    { 0x549344ed, SUC_lexicon_tags_1 },
    { 0x2c7344ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7344f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d344f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a1344fa, SUC_lexicon_tags_17 },
    { 0xc8f344fb, SUC_lexicon_tags_20 },
    { 0x817344fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f34502, SUC_lexicon_tags_3 },
    { 0xb4534503, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f3450c, SUC_lexicon_tags_31 },
    { 0x5e93450d, SUC_lexicon_tags_37 },
    { 0xa093450e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2534515, SUC_lexicon_tags_8 },
    { 0x6af34515, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4934519, SUC_lexicon_tags_32 },
    { 0xb0d3451a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf893451c, SUC_lexicon_tags_0 },
    { 0xc9f3451d, SUC_lexicon_tags_2 },
    { 0x9653451d, SUC_lexicon_tags_10 },
    { 0xd453451f, SUC_lexicon_tags_27 },
    { 0x63b34520, SUC_lexicon_tags_15 },
    { 0xb8134520, SUC_lexicon_tags_27 },
    { 0xe933451e, SUC_lexicon_tags_12 },
    { 0x6e13451c, SUC_lexicon_tags_17 },
    { 0xad33451c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2ff34526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5534528, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb3452e, SUC_lexicon_tags_23 },
    { 0x8c73452e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd34535, SUC_lexicon_tags_9 },
    { 0x7cb34535, SUC_lexicon_tags_37 },
    { 0x30734537, SUC_lexicon_tags_30 },
    { 0x8b534538, SUC_lexicon_tags_2 },
    { 0x3e134539, SUC_lexicon_tags_11 },
    { 0x77b3453a, SUC_lexicon_tags_27 },
    { 0xd6b3453a, SUC_lexicon_tags_17 },
    { 0x8b93453c, SUC_lexicon_tags_34 },
    { 0xb633453a, SUC_lexicon_tags_7 },
    { 0xc153453e, SUC_lexicon_tags_38 },
    { 0x1113453e, SUC_lexicon_tags_19 },
    { 0x8d13453f, SUC_lexicon_tags_11 },
    { 0x74b34540, SUC_lexicon_tags_57 },
    { 0x91134542, SUC_lexicon_tags_31 },
    { 0x56134542, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24334546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d34549, SUC_lexicon_tags_37 },
    { 0xe873454a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd653454e, SUC_lexicon_tags_31 },
    { 0xfd33454f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3934558, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b3455b, SUC_lexicon_tags_25 },
    { 0x75f3455b, SUC_lexicon_tags_12 },
    { 0x3053455b, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x39934560, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x82134562, SUC_lexicon_tags_55 },
    { 0x72134562, SUC_lexicon_tags_37 },
    { 0x5cd34564, SUC_lexicon_tags_25 },
    { 0x97934565, SUC_lexicon_tags_15 },
    { 0x14134564, SUC_lexicon_tags_11 },
    { 0x16f34565, SUC_lexicon_tags_14 },
    { 0x91134568, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb34574, SUC_lexicon_tags_65 },
    { 0x6fb34575, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20734577, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52f34579, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d34582, SUC_lexicon_tags_25 },
    { 0x6d934583, SUC_lexicon_tags_16 },
    { 0xc3134583, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f3458f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x66734592, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61934596, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1334598, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x533459a, SUC_lexicon_tags_65 },
    { 0x98b3459b, SUC_lexicon_tags_29 },
    { 0x5173459c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21f3459e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdd345a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83d345a2, SUC_lexicon_tags_12 },
    { 0x261345a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d345a6, SUC_lexicon_tags_2 },
    { 0x4f3345a7, SUC_lexicon_tags_14 },
    { 0x7ef345a7, SUC_lexicon_tags_37 },
    { 0xa45345a9, SUC_lexicon_tags_37 },
    { 0x563345aa, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137345ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b345b3, SUC_lexicon_tags_62 },
    { 0x95f345b3, SUC_lexicon_tags_12 },
    { 0x7ad345b5, SUC_lexicon_tags_6 },
    { 0x8cd345b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94f345b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d345bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb345c0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xc57345c2, SUC_lexicon_tags_15 },
    { 0x7e5345c2, SUC_lexicon_tags_7 },
    { 0x211345c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf345c9, SUC_lexicon_tags_16 },
    { 0x1ed345ca, SUC_lexicon_tags_17 },
    { 0x1f345ca, SUC_lexicon_tags_32 },
    { 0x9ef345cc, SUC_lexicon_tags_1 },
    { 0xbcb345ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d345d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x53345d2, SUC_lexicon_tags_17 },
    { 0xf0d345d3, SUC_lexicon_tags_23 },
    { 0x49345d2, SUC_lexicon_tags_4 },
    { 0x8d5345d5, SUC_lexicon_tags_60 },
    { 0xca9345d2, SUC_lexicon_tags_47 },
    { 0xd1345d3, SUC_lexicon_tags_26 },
    { 0xe4f345d8, SUC_lexicon_tags_12 },
    { 0x281345d9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xbf7345db, SUC_lexicon_tags_26 },
    { 0xedd345dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f345e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc53345e2, SUC_lexicon_tags_29 },
    { 0xd7b345e3, SUC_lexicon_tags_17 },
    { 0x78d345e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63345e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7345ed, SUC_lexicon_tags_28 },
    { 0xf41345ee, SUC_lexicon_tags_47 },
    { 0x9a5345ef, SUC_lexicon_tags_27 },
    { 0xff345f0, SUC_lexicon_tags_11 },
    { 0xc2b345f1, SUC_lexicon_tags_60 },
    { 0xcf9345ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3345f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733345fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61f345fc, SUC_lexicon_tags_26 },
    { 0x733345fd, SUC_lexicon_tags_17 },
    { 0x96f345fc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x37134601, SUC_lexicon_tags_19 },
    { 0xdc334602, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e134609, SUC_lexicon_tags_15 },
    { 0x6313460a, SUC_lexicon_tags_25 },
    { 0x3893460b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x993460d, SUC_lexicon_tags_28 },
    { 0x9bb3460e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x71334611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57334613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f34616, SUC_lexicon_tags_9 },
    { 0x61134616, SUC_lexicon_tags_25 },
    { 0xe1134617, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2d73461a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd173461c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45934621, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd34625, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b34628, SUC_lexicon_tags_7 },
    { 0x75f34629, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd573462c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77134631, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc134633, SUC_lexicon_tags_11 },
    { 0x28f34634, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86534639, SUC_lexicon_tags_228 },
    { 0, NULL },
    { 0, NULL },
    { 0x653463c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x86f3463e, SUC_lexicon_tags_14 },
    { 0x7513463f, SUC_lexicon_tags_30 },
    { 0x7d734640, SUC_lexicon_tags_37 },
    { 0xd2b34641, SUC_lexicon_tags_15 },
    { 0xce734641, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x35934644, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3464a, SUC_lexicon_tags_17 },
    { 0x9373464b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf8f3464d, SUC_lexicon_tags_262 },
    { 0xe753464d, SUC_lexicon_tags_17 },
    { 0x8653464f, SUC_lexicon_tags_6 },
    { 0xd34650, SUC_lexicon_tags_49 },
    { 0x90734650, SUC_lexicon_tags_38 },
    { 0xa934652, SUC_lexicon_tags_60 },
    { 0xb3534651, SUC_lexicon_tags_1 },
    { 0xcef34654, SUC_lexicon_tags_1 },
    { 0x7293464d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87d34657, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d53465a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d34661, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3b734663, SUC_lexicon_tags_12 },
    { 0xfc934664, SUC_lexicon_tags_23 },
    { 0x12734665, SUC_lexicon_tags_113 },
    { 0x22134665, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd134668, SUC_lexicon_tags_48 },
    { 0x45f34669, SUC_lexicon_tags_37 },
    { 0xd8f34668, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38134675, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b34678, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x20b3467a, SUC_lexicon_tags_17 },
    { 0xeab3467b, SUC_lexicon_tags_27 },
    { 0xaed3467a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88334680, SUC_lexicon_tags_27 },
    { 0x73134681, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b734684, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7f734686, SUC_lexicon_tags_17 },
    { 0x8d534687, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af3468b, SUC_lexicon_tags_11 },
    { 0x8733468c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4673468e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x17f34690, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d134692, SUC_lexicon_tags_30 },
    { 0x7a934693, SUC_lexicon_tags_6 },
    { 0xbd134693, SUC_lexicon_tags_1 },
    { 0x1bb34694, SUC_lexicon_tags_38 },
    { 0xca134696, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf34699, SUC_lexicon_tags_7 },
    { 0x6473469a, SUC_lexicon_tags_23 },
    { 0xd7b3469b, SUC_lexicon_tags_2 },
    { 0xab13469b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd346a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a9346a3, SUC_lexicon_tags_95 },
    { 0x2a3346a4, SUC_lexicon_tags_33 },
    { 0xdbd346a4, SUC_lexicon_tags_28 },
    { 0x121346a6, SUC_lexicon_tags_17 },
    { 0x5fd346a3, SUC_lexicon_tags_141 },
    { 0x5d7346a8, SUC_lexicon_tags_2 },
    { 0x611346a9, SUC_lexicon_tags_17 },
    { 0x453346a9, SUC_lexicon_tags_14 },
    { 0x54d346ab, SUC_lexicon_tags_3 },
    { 0xc8b346ab, SUC_lexicon_tags_15 },
    { 0x30b346ad, SUC_lexicon_tags_37 },
    { 0x525346ae, SUC_lexicon_tags_25 },
    { 0xe33346a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x91346b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd346b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf346c1, SUC_lexicon_tags_25 },
    { 0x80f346c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8d346c4, SUC_lexicon_tags_47 },
    { 0x15f346c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f346c9, SUC_lexicon_tags_34 },
    { 0x7ab346c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249346d2, SUC_lexicon_tags_37 },
    { 0x38f346d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd9f346d5, SUC_lexicon_tags_25 },
    { 0x403346d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359346dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9346e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767346e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47346e9, SUC_lexicon_tags_16 },
    { 0x6db346e9, SUC_lexicon_tags_31 },
    { 0xc89346e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1346ee, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f346f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d346f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387346f9, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64734700, SUC_lexicon_tags_59 },
    { 0xfe934701, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe334709, SUC_lexicon_tags_37 },
    { 0xb073470a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f73470d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2334712, SUC_lexicon_tags_12 },
    { 0xc1934713, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b34719, SUC_lexicon_tags_11 },
    { 0x17d3471a, SUC_lexicon_tags_25 },
    { 0xf633471a, SUC_lexicon_tags_28 },
    { 0x43b3471c, SUC_lexicon_tags_15 },
    { 0xeab3471c, SUC_lexicon_tags_1 },
    { 0x473471a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26d34720, SUC_lexicon_tags_23 },
    { 0xf6534721, SUC_lexicon_tags_15 },
    { 0x89734722, SUC_lexicon_tags_43 },
    { 0x6c134723, SUC_lexicon_tags_2 },
    { 0x88f34724, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f34727, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6134730, SUC_lexicon_tags_14 },
    { 0x7bd34731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94134737, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xbe334739, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xafb3473b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xee93473e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8f34740, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2134747, SUC_lexicon_tags_26 },
    { 0xc6334748, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf913474d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d134750, SUC_lexicon_tags_12 },
    { 0x2ed34751, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf734754, SUC_lexicon_tags_25 },
    { 0xab334755, SUC_lexicon_tags_7 },
    { 0xc7334756, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc73475a, SUC_lexicon_tags_37 },
    { 0x9453475b, SUC_lexicon_tags_237 },
    { 0x4993475c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d93475e, SUC_lexicon_tags_12 },
    { 0xc733475f, SUC_lexicon_tags_65 },
    { 0x28f3475f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2534763, SUC_lexicon_tags_2 },
    { 0xfdf34764, SUC_lexicon_tags_37 },
    { 0x49b34763, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4133476c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb3476f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b34775, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb913477e, SUC_lexicon_tags_2 },
    { 0x813477f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf334782, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3334787, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff93478b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f34790, SUC_lexicon_tags_28 },
    { 0xea134791, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa93479e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7347a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd35347a3, SUC_lexicon_tags_14 },
    { 0x41347a4, SUC_lexicon_tags_13 },
    { 0xc19347a4, SUC_lexicon_tags_39 },
    { 0x829347a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5347ac, SUC_lexicon_tags_17 },
    { 0xb6f347ad, SUC_lexicon_tags_31 },
    { 0x465347ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x635347b0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293347bb, SUC_lexicon_tags_65 },
    { 0x59f347bb, SUC_lexicon_tags_2 },
    { 0x3ad347bd, SUC_lexicon_tags_47 },
    { 0x40b347bc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d347c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe11347c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff347c6, SUC_lexicon_tags_7 },
    { 0xe69347c6, SUC_lexicon_tags_454 },
    { 0x39347c7, SUC_lexicon_tags_17 },
    { 0x2ad347c9, SUC_lexicon_tags_28 },
    { 0x11347ca, SUC_lexicon_tags_37 },
    { 0x859347ca, SUC_lexicon_tags_12 },
    { 0xfdf347c9, SUC_lexicon_tags_25 },
    { 0x17347cd, SUC_lexicon_tags_42 },
    { 0x215347cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x351347d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e7347d2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x43d347d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49347d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d347dc, SUC_lexicon_tags_77 },
    { 0x651347dd, SUC_lexicon_tags_38 },
    { 0x8a5347dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x391347e0, SUC_lexicon_tags_63 },
    { 0x377347e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf347e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x629347e8, SUC_lexicon_tags_0 },
    { 0x6f347e9, SUC_lexicon_tags_75 },
    { 0x159347ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc59347ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f347f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f347ff, SUC_lexicon_tags_23 },
    { 0x6ad347ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6334803, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb34807, SUC_lexicon_tags_45 },
    { 0xf0134808, SUC_lexicon_tags_15 },
    { 0x47534807, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb373480d, SUC_lexicon_tags_45 },
    { 0x453480e, SUC_lexicon_tags_38 },
    { 0x4633480e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f34813, SUC_lexicon_tags_25 },
    { 0x9f534814, SUC_lexicon_tags_781 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f134817, SUC_lexicon_tags_25 },
    { 0x27934817, SUC_lexicon_tags_27 },
    { 0x10134819, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5eb3481b, SUC_lexicon_tags_25 },
    { 0x8593481c, SUC_lexicon_tags_18 },
    { 0x8253481d, SUC_lexicon_tags_17 },
    { 0xc0f3481c, SUC_lexicon_tags_37 },
    { 0xa353481f, SUC_lexicon_tags_37 },
    { 0xdbb3481f, SUC_lexicon_tags_11 },
    { 0x6f534821, SUC_lexicon_tags_23 },
    { 0xb0134820, SUC_lexicon_tags_37 },
    { 0xa0934823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd934829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb3482c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a33482f, SUC_lexicon_tags_7 },
    { 0xbe73482f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb34837, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb53483a, SUC_lexicon_tags_1 },
    { 0xd873483b, SUC_lexicon_tags_17 },
    { 0x5a53483b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15134840, SUC_lexicon_tags_9 },
    { 0x56734840, SUC_lexicon_tags_2 },
    { 0x65734842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23b34844, SUC_lexicon_tags_9 },
    { 0x55934845, SUC_lexicon_tags_17 },
    { 0x3b734846, SUC_lexicon_tags_7 },
    { 0x17934845, SUC_lexicon_tags_43 },
    { 0x29b34848, SUC_lexicon_tags_46 },
    { 0x40134848, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf3484d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10934852, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f34855, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc973485a, SUC_lexicon_tags_11 },
    { 0x1f3485a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc53485e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9134864, SUC_lexicon_tags_28 },
    { 0xfb334865, SUC_lexicon_tags_12 },
    { 0x25f34866, SUC_lexicon_tags_15 },
    { 0x31f34867, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55f34869, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc313486e, SUC_lexicon_tags_45 },
    { 0x3f33486f, SUC_lexicon_tags_16 },
    { 0x1534870, SUC_lexicon_tags_12 },
    { 0x8c534871, SUC_lexicon_tags_38 },
    { 0xf5934872, SUC_lexicon_tags_23 },
    { 0x3ab34873, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x66d34875, SUC_lexicon_tags_12 },
    { 0x92334876, SUC_lexicon_tags_31 },
    { 0x73134877, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x90334879, SUC_lexicon_tags_15 },
    { 0xfb13487a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9833487e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c334881, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5af34883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff734886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48d34888, SUC_lexicon_tags_10 },
    { 0x3134889, SUC_lexicon_tags_8 },
    { 0x6553488a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9853488c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe153488e, SUC_lexicon_tags_14 },
    { 0xfc93488f, SUC_lexicon_tags_10 },
    { 0x89134890, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11734892, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e734895, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x30934897, SUC_lexicon_tags_23 },
    { 0x7cf34897, SUC_lexicon_tags_37 },
    { 0xacd34899, SUC_lexicon_tags_60 },
    { 0xae734899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2773489e, SUC_lexicon_tags_11 },
    { 0xc0f3489f, SUC_lexicon_tags_37 },
    { 0x189348a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13348a6, SUC_lexicon_tags_27 },
    { 0x69348a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f348aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9348b0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21348b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59f348b5, SUC_lexicon_tags_81 },
    { 0x12b348b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9348bd, SUC_lexicon_tags_62 },
    { 0x5b348be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14f348c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b348c5, SUC_lexicon_tags_11 },
    { 0xc1348c6, SUC_lexicon_tags_14 },
    { 0xdb9348c7, SUC_lexicon_tags_19 },
    { 0x65348c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x739348cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3348cf, SUC_lexicon_tags_87 },
    { 0xe49348d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215348d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231348da, SUC_lexicon_tags_15 },
    { 0x335348da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc0b348dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6b9348df, SUC_lexicon_tags_7 },
    { 0xf7b348e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa93348e2, SUC_lexicon_tags_31 },
    { 0x99b348e3, SUC_lexicon_tags_55 },
    { 0x9ab348e4, SUC_lexicon_tags_23 },
    { 0x1f5348e5, SUC_lexicon_tags_49 },
    { 0x73348e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79348e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x333348eb, SUC_lexicon_tags_17 },
    { 0x33b348ec, SUC_lexicon_tags_65 },
    { 0xcf7348ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7348f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xec1348f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd348fa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d348fd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f534901, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9534907, SUC_lexicon_tags_25 },
    { 0x33534908, SUC_lexicon_tags_37 },
    { 0xbb734909, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x9e53490b, SUC_lexicon_tags_19 },
    { 0xdbb3490b, SUC_lexicon_tags_25 },
    { 0x1f93490d, SUC_lexicon_tags_33 },
    { 0xf9f3490e, SUC_lexicon_tags_1 },
    { 0x3b13490f, SUC_lexicon_tags_19 },
    { 0x47534910, SUC_lexicon_tags_806 },
    { 0, NULL },
    { 0xd4534912, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81934918, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef3491f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb34927, SUC_lexicon_tags_12 },
    { 0xd2d34928, SUC_lexicon_tags_43 },
    { 0x18734927, SUC_lexicon_tags_10 },
    { 0x74934928, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7073492d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f34938, SUC_lexicon_tags_25 },
    { 0x6a734938, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f34940, SUC_lexicon_tags_46 },
    { 0x6d534941, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb34944, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9553494a, SUC_lexicon_tags_23 },
    { 0xae53494a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e934951, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b34955, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b13495c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab34961, SUC_lexicon_tags_113 },
    { 0x5eb34962, SUC_lexicon_tags_14 },
    { 0xed534962, SUC_lexicon_tags_23 },
    { 0xdf334964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf134966, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24534968, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd753496b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb1d3496d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3134970, SUC_lexicon_tags_30 },
    { 0xa3534970, SUC_lexicon_tags_47 },
    { 0xea334970, SUC_lexicon_tags_50 },
    { 0x77d34970, SUC_lexicon_tags_12 },
    { 0xcc134974, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94734976, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd513497d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbbd3497f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe0134981, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f934987, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x39934989, SUC_lexicon_tags_7 },
    { 0xae53498a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41f3498c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9334993, SUC_lexicon_tags_11 },
    { 0xafd34994, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9934998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ed3499a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d349a1, SUC_lexicon_tags_21 },
    { 0xf5d349a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b349a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9349a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xccb349aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b3349ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1349b0, SUC_lexicon_tags_19 },
    { 0xa37349b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09349b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b349b9, SUC_lexicon_tags_12 },
    { 0x71b349ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1349c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957349ca, SUC_lexicon_tags_23 },
    { 0x46d349cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d9349cd, SUC_lexicon_tags_17 },
    { 0xbf7349ce, SUC_lexicon_tags_66 },
    { 0x39b349cf, SUC_lexicon_tags_1 },
    { 0x83d349ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7349d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf349d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x779349da, SUC_lexicon_tags_7 },
    { 0xf6d349da, SUC_lexicon_tags_37 },
    { 0x86d349da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b349e2, SUC_lexicon_tags_25 },
    { 0xecd349e3, SUC_lexicon_tags_11 },
    { 0x67f349e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x691349e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7349ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd349ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x78b349f0, SUC_lexicon_tags_1 },
    { 0x663349f1, SUC_lexicon_tags_7 },
    { 0x5e7349f2, SUC_lexicon_tags_12 },
    { 0x461349f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3349f8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9349fd, SUC_lexicon_tags_37 },
    { 0x27349fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f734a08, SUC_lexicon_tags_11 },
    { 0x83934a09, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a934a0c, SUC_lexicon_tags_19 },
    { 0x71734a0c, SUC_lexicon_tags_46 },
    { 0xf5534a0e, SUC_lexicon_tags_9 },
    { 0x9f534a0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d534a14, SUC_lexicon_tags_27 },
    { 0x87334a15, SUC_lexicon_tags_11 },
    { 0x44334a16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26334a18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x90334a1a, SUC_lexicon_tags_11 },
    { 0x61534a1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b34a21, SUC_lexicon_tags_16 },
    { 0x91934a22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64d34a24, SUC_lexicon_tags_11 },
    { 0x90134a24, SUC_lexicon_tags_28 },
    { 0xdf134a26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25b34a28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d34a2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0534a30, SUC_lexicon_tags_81 },
    { 0xde934a31, SUC_lexicon_tags_2 },
    { 0xc3334a32, SUC_lexicon_tags_7 },
    { 0xbf334a33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x44734a36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xde334a39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2734a3d, SUC_lexicon_tags_77 },
    { 0x49d34a3d, SUC_lexicon_tags_65 },
    { 0xb7934a3f, SUC_lexicon_tags_17 },
    { 0x6c734a3f, SUC_lexicon_tags_7 },
    { 0xc8b34a3f, SUC_lexicon_tags_17 },
    { 0xadf34a3f, SUC_lexicon_tags_7 },
    { 0x12b34a42, SUC_lexicon_tags_15 },
    { 0x97d34a3e, SUC_lexicon_tags_1 },
    { 0x33134a45, SUC_lexicon_tags_7 },
    { 0x4f334a42, SUC_lexicon_tags_10 },
    { 0xe2734a44, SUC_lexicon_tags_37 },
    { 0xdd734a47, SUC_lexicon_tags_44 },
    { 0x1db34a49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b34a50, SUC_lexicon_tags_37 },
    { 0x36134a51, SUC_lexicon_tags_303 },
    { 0xc3734a52, SUC_lexicon_tags_14 },
    { 0x9d734a53, SUC_lexicon_tags_63 },
    { 0xe134a51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa3534a56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x42534a59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d34a5d, SUC_lexicon_tags_37 },
    { 0xf8d34a5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f34a63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f34a66, SUC_lexicon_tags_25 },
    { 0x7d534a67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4b34a69, SUC_lexicon_tags_15 },
    { 0x8dd34a6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93934a6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x28534a72, SUC_lexicon_tags_37 },
    { 0xf8734a72, SUC_lexicon_tags_1 },
    { 0x98f34a73, SUC_lexicon_tags_1 },
    { 0xe5b34a72, SUC_lexicon_tags_7 },
    { 0xc4334a74, SUC_lexicon_tags_45 },
    { 0x7eb34a77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d334a79, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14334a80, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x66734a82, SUC_lexicon_tags_32 },
    { 0x27d34a82, SUC_lexicon_tags_17 },
    { 0xbad34a84, SUC_lexicon_tags_11 },
    { 0xdb534a84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a334a88, SUC_lexicon_tags_12 },
    { 0xf8934a89, SUC_lexicon_tags_12 },
    { 0x6ff34a8a, SUC_lexicon_tags_25 },
    { 0x29334a8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b34a8e, SUC_lexicon_tags_82 },
    { 0x19734a8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd9734a91, SUC_lexicon_tags_17 },
    { 0xf9d34a91, SUC_lexicon_tags_60 },
    { 0xde734a91, SUC_lexicon_tags_7 },
    { 0x91534a93, SUC_lexicon_tags_2 },
    { 0x88d34a94, SUC_lexicon_tags_25 },
    { 0x6f34a96, SUC_lexicon_tags_87 },
    { 0x41534a96, SUC_lexicon_tags_12 },
    { 0x54f34a96, SUC_lexicon_tags_15 },
    { 0xc2134a99, SUC_lexicon_tags_30 },
    { 0x9ef34a9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84334a9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28734aa2, SUC_lexicon_tags_12 },
    { 0xa6f34aa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb534aa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f34ab2, SUC_lexicon_tags_19 },
    { 0x19d34ab2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f34ab7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x70934ab9, SUC_lexicon_tags_37 },
    { 0x1c934ab9, SUC_lexicon_tags_25 },
    { 0xc3d34ab9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8534abe, SUC_lexicon_tags_21 },
    { 0xa1f34abf, SUC_lexicon_tags_49 },
    { 0x62934ac0, SUC_lexicon_tags_2 },
    { 0xdd734ac1, SUC_lexicon_tags_37 },
    { 0x7934ac1, SUC_lexicon_tags_37 },
    { 0x60f34ac3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21334ac7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a134ac9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e334acc, SUC_lexicon_tags_66 },
    { 0xea334acd, SUC_lexicon_tags_37 },
    { 0xb7f34acd, SUC_lexicon_tags_16 },
    { 0x99534acf, SUC_lexicon_tags_19 },
    { 0xa8534ad0, SUC_lexicon_tags_37 },
    { 0xa8134ad1, SUC_lexicon_tags_48 },
    { 0xbd734ad2, SUC_lexicon_tags_37 },
    { 0xec134ad3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d534ad9, SUC_lexicon_tags_1 },
    { 0x96934ada, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b34add, SUC_lexicon_tags_29 },
    { 0x73534ade, SUC_lexicon_tags_33 },
    { 0x3f334adf, SUC_lexicon_tags_13 },
    { 0xbdd34ae0, SUC_lexicon_tags_31 },
    { 0xffb34ae1, SUC_lexicon_tags_33 },
    { 0x23334add, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d34ae5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32534aea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87134aee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5934af1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x99d34af3, SUC_lexicon_tags_7 },
    { 0x75134af3, SUC_lexicon_tags_7 },
    { 0x81d34af3, SUC_lexicon_tags_7 },
    { 0xefd34af6, SUC_lexicon_tags_60 },
    { 0x1f134af7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32934afa, SUC_lexicon_tags_49 },
    { 0x8ef34afa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf34b00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad34b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6334b06, SUC_lexicon_tags_47 },
    { 0x6c734b07, SUC_lexicon_tags_11 },
    { 0xab334b07, SUC_lexicon_tags_7 },
    { 0x80134b08, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f34b0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d34b13, SUC_lexicon_tags_19 },
    { 0x73134b13, SUC_lexicon_tags_37 },
    { 0x32f34b14, SUC_lexicon_tags_14 },
    { 0x5f734b14, SUC_lexicon_tags_17 },
    { 0xee534b17, SUC_lexicon_tags_12 },
    { 0xdb334b17, SUC_lexicon_tags_8 },
    { 0x54b34b19, SUC_lexicon_tags_28 },
    { 0xa9134b1a, SUC_lexicon_tags_7 },
    { 0x79b34b1b, SUC_lexicon_tags_14 },
    { 0xc8534b13, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b34b22, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd34b26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a134b2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16134b31, SUC_lexicon_tags_7 },
    { 0x73334b31, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb34b39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b34b3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab34b43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe3f34b45, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d34b4a, SUC_lexicon_tags_35 },
    { 0x6f134b4b, SUC_lexicon_tags_2 },
    { 0xe6b34b4c, SUC_lexicon_tags_19 },
    { 0x20b34b4b, SUC_lexicon_tags_0 },
    { 0xbf34b4e, SUC_lexicon_tags_11 },
    { 0x6ad34b4d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6134b52, SUC_lexicon_tags_1 },
    { 0xe2134b52, SUC_lexicon_tags_19 },
    { 0xa4334b54, SUC_lexicon_tags_9 },
    { 0xfb734b52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f34b5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc534b5d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x22934b5f, SUC_lexicon_tags_17 },
    { 0x1cf34b5f, SUC_lexicon_tags_49 },
    { 0x8c134b5f, SUC_lexicon_tags_17 },
    { 0x83f34b62, SUC_lexicon_tags_9 },
    { 0x7f934b63, SUC_lexicon_tags_11 },
    { 0xa534b64, SUC_lexicon_tags_8 },
    { 0xcf534b65, SUC_lexicon_tags_2 },
    { 0x77d34b65, SUC_lexicon_tags_43 },
    { 0xf6f34b64, SUC_lexicon_tags_17 },
    { 0xb7334b68, SUC_lexicon_tags_19 },
    { 0x9ed34b69, SUC_lexicon_tags_31 },
    { 0x83134b68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x82934b6c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb334b70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfa334b72, SUC_lexicon_tags_68 },
    { 0xd4734b73, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x74134b76, SUC_lexicon_tags_15 },
    { 0xbf734b76, SUC_lexicon_tags_7 },
    { 0x4fb34b78, SUC_lexicon_tags_12 },
    { 0xc4d34b79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe8f34b7b, SUC_lexicon_tags_37 },
    { 0xb9334b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6334b7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f34b84, SUC_lexicon_tags_1 },
    { 0x7cf34b85, SUC_lexicon_tags_60 },
    { 0x17334b85, SUC_lexicon_tags_68 },
    { 0x23f34b87, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db34b8b, SUC_lexicon_tags_12 },
    { 0x32f34b8c, SUC_lexicon_tags_29 },
    { 0x39534b8d, SUC_lexicon_tags_11 },
    { 0x72734b8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39934b92, SUC_lexicon_tags_2 },
    { 0xbbd34b92, SUC_lexicon_tags_11 },
    { 0x34734b94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f534b98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19334b9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db34ba0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd34ba3, SUC_lexicon_tags_12 },
    { 0xd6134ba4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75934ba7, SUC_lexicon_tags_14 },
    { 0x4a934ba7, SUC_lexicon_tags_1 },
    { 0xb8b34ba9, SUC_lexicon_tags_37 },
    { 0xcc334baa, SUC_lexicon_tags_16 },
    { 0x94d34bab, SUC_lexicon_tags_9 },
    { 0xafb34bac, SUC_lexicon_tags_15 },
    { 0xb5334bad, SUC_lexicon_tags_47 },
    { 0xac734bac, SUC_lexicon_tags_46 },
    { 0x80334ba9, SUC_lexicon_tags_12 },
    { 0xfbb34baf, SUC_lexicon_tags_17 },
    { 0xb0b34ba8, SUC_lexicon_tags_21 },
    { 0x4c534bae, SUC_lexicon_tags_15 },
    { 0x8cb34bb3, SUC_lexicon_tags_31 },
    { 0x97b34bb0, SUC_lexicon_tags_37 },
    { 0x57334baa, SUC_lexicon_tags_37 },
    { 0x40d34bb6, SUC_lexicon_tags_38 },
    { 0x68134bb7, SUC_lexicon_tags_12 },
    { 0x9fd34bb6, SUC_lexicon_tags_17 },
    { 0x8ad34bb8, SUC_lexicon_tags_37 },
    { 0x69734bba, SUC_lexicon_tags_37 },
    { 0x53b34bbb, SUC_lexicon_tags_15 },
    { 0x29934bbc, SUC_lexicon_tags_7 },
    { 0x7a734bbc, SUC_lexicon_tags_10 },
    { 0x72934bbb, SUC_lexicon_tags_38 },
    { 0xdf334bbf, SUC_lexicon_tags_19 },
    { 0x49b34bc0, SUC_lexicon_tags_6 },
    { 0x3ad34bbc, SUC_lexicon_tags_11 },
    { 0xc9534baf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x15334bc5, SUC_lexicon_tags_17 },
    { 0x40734bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7334bc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf34bcc, SUC_lexicon_tags_60 },
    { 0xb9f34bcd, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd34bd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90534bd4, SUC_lexicon_tags_15 },
    { 0x35734bd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f34bda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41534bde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df34be5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd34be8, SUC_lexicon_tags_49 },
    { 0xddf34be8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2134bed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f34bf1, SUC_lexicon_tags_7 },
    { 0x6ad34bf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d334bfa, SUC_lexicon_tags_0 },
    { 0xd4934bfb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2334c02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0734c08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbd134c0a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2934c13, SUC_lexicon_tags_29 },
    { 0x53534c14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb934c1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e534c25, SUC_lexicon_tags_41 },
    { 0x68b34c26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x96334c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff34c2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf134c36, SUC_lexicon_tags_38 },
    { 0x68f34c37, SUC_lexicon_tags_27 },
    { 0xa5f34c37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2734c3e, SUC_lexicon_tags_21 },
    { 0x6a134c3f, SUC_lexicon_tags_63 },
    { 0x1ed34c40, SUC_lexicon_tags_16 },
    { 0xae534c3f, SUC_lexicon_tags_25 },
    { 0x1df34c42, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59134c46, SUC_lexicon_tags_26 },
    { 0x4a934c47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bf34c49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8534c4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2934c54, SUC_lexicon_tags_46 },
    { 0xc8b34c54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6934c58, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77134c5d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b34c61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a334c67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90334c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f934c6c, SUC_lexicon_tags_46 },
    { 0x47134c6d, SUC_lexicon_tags_47 },
    { 0x1db34c6e, SUC_lexicon_tags_10 },
    { 0x4eb34c6e, SUC_lexicon_tags_37 },
    { 0xa5934c70, SUC_lexicon_tags_47 },
    { 0x9534c6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf734c73, SUC_lexicon_tags_87 },
    { 0xb4b34c74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa934c76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x52334c78, SUC_lexicon_tags_13 },
    { 0x78934c79, SUC_lexicon_tags_11 },
    { 0x2b934c7a, SUC_lexicon_tags_11 },
    { 0xc6534c78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67934c7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed34c82, SUC_lexicon_tags_12 },
    { 0xf1734c83, SUC_lexicon_tags_38 },
    { 0xd5934c84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c934c87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x95934c8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b34c8f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x69734c91, SUC_lexicon_tags_2 },
    { 0x59134c91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a934c95, SUC_lexicon_tags_7 },
    { 0x90534c96, SUC_lexicon_tags_11 },
    { 0xaaf34c97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4334c99, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x74134c9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d34c9f, SUC_lexicon_tags_27 },
    { 0xbed34ca0, SUC_lexicon_tags_19 },
    { 0xb5734c9f, SUC_lexicon_tags_47 },
    { 0x4b34ca2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff34ca5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75134ca7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd934caa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6934cb0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb7134cb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf734cb4, SUC_lexicon_tags_28 },
    { 0xeeb34cb5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2934cba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b34cbf, SUC_lexicon_tags_17 },
    { 0xdd934cc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46134cc7, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x2b534cc9, SUC_lexicon_tags_1 },
    { 0x14334cc9, SUC_lexicon_tags_11 },
    { 0x98534ccb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95134cd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21d34cd2, SUC_lexicon_tags_37 },
    { 0xafb34cd3, SUC_lexicon_tags_37 },
    { 0xeb34cd3, SUC_lexicon_tags_7 },
    { 0x6db34cd4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2934cd8, SUC_lexicon_tags_49 },
    { 0x1334cd9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f34cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61334cde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42534ce3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f34ce9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b34cf1, SUC_lexicon_tags_17 },
    { 0x1d134cf2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86934cf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb34cfc, SUC_lexicon_tags_44 },
    { 0xa6d34cfd, SUC_lexicon_tags_39 },
    { 0x6734cfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f734d03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3d34d05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd3534d07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45934d09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5dd34d0b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd34d0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb934d13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98334d17, SUC_lexicon_tags_129 },
    { 0x75534d18, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb34d20, SUC_lexicon_tags_37 },
    { 0x55334d20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8134d23, SUC_lexicon_tags_23 },
    { 0xaef34d23, SUC_lexicon_tags_17 },
    { 0xf9134d25, SUC_lexicon_tags_2 },
    { 0x42f34d25, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x39134d29, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3134d2f, SUC_lexicon_tags_30 },
    { 0x64134d2f, SUC_lexicon_tags_7 },
    { 0xc5d34d31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x934d33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80734d36, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd9534d38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a334d3c, SUC_lexicon_tags_1 },
    { 0x9ed34d3d, SUC_lexicon_tags_0 },
    { 0x82134d3d, SUC_lexicon_tags_4 },
    { 0xab334d3c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f34d42, SUC_lexicon_tags_65 },
    { 0x9cb34d43, SUC_lexicon_tags_1 },
    { 0xea734d44, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb934d4b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6a934d4d, SUC_lexicon_tags_49 },
    { 0xfb934d4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x85334d51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10534d54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d734d57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7934d5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72734d63, SUC_lexicon_tags_15 },
    { 0x8a534d63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b34d66, SUC_lexicon_tags_19 },
    { 0x86534d67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af34d6c, SUC_lexicon_tags_37 },
    { 0xd8934d6d, SUC_lexicon_tags_17 },
    { 0xf5534d6d, SUC_lexicon_tags_30 },
    { 0x78734d6d, SUC_lexicon_tags_23 },
    { 0x6b934d6e, SUC_lexicon_tags_66 },
    { 0xdcd34d6f, SUC_lexicon_tags_37 },
    { 0x5af34d6f, SUC_lexicon_tags_28 },
    { 0xb3b34d6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd8334d75, SUC_lexicon_tags_17 },
    { 0x1b134d75, SUC_lexicon_tags_37 },
    { 0x51734d77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xedd34d79, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80334d7e, SUC_lexicon_tags_15 },
    { 0xb8134d7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf334d81, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2534d88, SUC_lexicon_tags_80 },
    { 0x93b34d89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c734d8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c534d91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe334d9b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf4534d9d, SUC_lexicon_tags_1 },
    { 0x60534d9e, SUC_lexicon_tags_22 },
    { 0x30d34d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f34da2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51934da4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc9334da6, SUC_lexicon_tags_19 },
    { 0x88d34da6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e534daa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf734dad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38734db3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9334db8, SUC_lexicon_tags_7 },
    { 0x94b34db9, SUC_lexicon_tags_17 },
    { 0xea934dba, SUC_lexicon_tags_38 },
    { 0x2fb34dbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d34dbe, SUC_lexicon_tags_26 },
    { 0x77534dbf, SUC_lexicon_tags_11 },
    { 0x92f34dbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91934dc2, SUC_lexicon_tags_37 },
    { 0xa9134dc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6134dc5, SUC_lexicon_tags_11 },
    { 0x84934dc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee934dcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb334dd0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5934dd7, SUC_lexicon_tags_1 },
    { 0x4734dd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd34ddc, SUC_lexicon_tags_25 },
    { 0xdc734ddd, SUC_lexicon_tags_19 },
    { 0xb9b34dde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13334de1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4334de5, SUC_lexicon_tags_0 },
    { 0xc5534de6, SUC_lexicon_tags_15 },
    { 0x27134de6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b34df0, SUC_lexicon_tags_23 },
    { 0xa0b34df1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1ad34df3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3334df7, SUC_lexicon_tags_82 },
    { 0xa4934df8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd34dff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f34e03, SUC_lexicon_tags_38 },
    { 0xe4734e04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f34e07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c734e0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7734e0e, SUC_lexicon_tags_0 },
    { 0xad934e0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b934e11, SUC_lexicon_tags_11 },
    { 0x15934e12, SUC_lexicon_tags_1 },
    { 0xbd934e11, SUC_lexicon_tags_19 },
    { 0xac534e14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44134e16, SUC_lexicon_tags_12 },
    { 0x47d34e17, SUC_lexicon_tags_15 },
    { 0xbed34e18, SUC_lexicon_tags_60 },
    { 0x15934e19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab34e1e, SUC_lexicon_tags_25 },
    { 0xded34e1e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x79b34e21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47334e23, SUC_lexicon_tags_19 },
    { 0xe1534e23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af34e29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0534e2f, SUC_lexicon_tags_17 },
    { 0x44134e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d34e36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed34e3c, SUC_lexicon_tags_28 },
    { 0x21334e3c, SUC_lexicon_tags_23 },
    { 0xf9334e3c, SUC_lexicon_tags_17 },
    { 0xa8134e3d, SUC_lexicon_tags_5 },
    { 0x76134e40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x68734e42, SUC_lexicon_tags_12 },
    { 0x43f34e43, SUC_lexicon_tags_8 },
    { 0x6b534e44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52334e47, SUC_lexicon_tags_1 },
    { 0x49534e48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae534e4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1134e4d, SUC_lexicon_tags_59 },
    { 0x79934e4d, SUC_lexicon_tags_17 },
    { 0x9734e4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ad34e51, SUC_lexicon_tags_7 },
    { 0xb1134e51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x21134e55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30934e58, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x49134e5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x37134e5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x75934e60, SUC_lexicon_tags_2 },
    { 0x11334e60, SUC_lexicon_tags_33 },
    { 0xf1334e62, SUC_lexicon_tags_0 },
    { 0x73f34e63, SUC_lexicon_tags_32 },
    { 0xa334e64, SUC_lexicon_tags_6 },
    { 0xc4534e64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d934e69, SUC_lexicon_tags_39 },
    { 0xfd134e69, SUC_lexicon_tags_35 },
    { 0x4734e69, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b34e6e, SUC_lexicon_tags_63 },
    { 0x58934e6f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x85934e72, SUC_lexicon_tags_11 },
    { 0x63734e72, SUC_lexicon_tags_11 },
    { 0xba534e73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf34e77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b34e7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4d34e81, SUC_lexicon_tags_0 },
    { 0x43734e82, SUC_lexicon_tags_8 },
    { 0xe1734e83, SUC_lexicon_tags_12 },
    { 0x55734e82, SUC_lexicon_tags_19 },
    { 0x4bf34e83, SUC_lexicon_tags_8 },
    { 0x60134e85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f34e8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1534e8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd934e8f, SUC_lexicon_tags_37 },
    { 0x29b34e90, SUC_lexicon_tags_2 },
    { 0xa6134e91, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe7934e93, SUC_lexicon_tags_11 },
    { 0xb9d34e94, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff34e9a, SUC_lexicon_tags_108 },
    { 0xe2b34e9b, SUC_lexicon_tags_27 },
    { 0xf7b34e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a734e9f, SUC_lexicon_tags_37 },
    { 0xba734ea0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3534ea2, SUC_lexicon_tags_17 },
    { 0x95934ea3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ab34ea5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x16934ea7, SUC_lexicon_tags_25 },
    { 0x5f934ea8, SUC_lexicon_tags_16 },
    { 0x9d934ea7, SUC_lexicon_tags_0 },
    { 0xba734eaa, SUC_lexicon_tags_43 },
    { 0xcc734eaa, SUC_lexicon_tags_10 },
    { 0x27934eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37534eae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4d34eb0, SUC_lexicon_tags_47 },
    { 0xcd934eb1, SUC_lexicon_tags_1 },
    { 0xbb934eb0, SUC_lexicon_tags_9 },
    { 0xcad34eb3, SUC_lexicon_tags_23 },
    { 0x96334eb0, SUC_lexicon_tags_11 },
    { 0xbdf34eb0, SUC_lexicon_tags_8 },
    { 0x8c334eb6, SUC_lexicon_tags_74 },
    { 0x9a334eb2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e934ebf, SUC_lexicon_tags_37 },
    { 0xd8934ec0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12734ec4, SUC_lexicon_tags_0 },
    { 0x99534ec4, SUC_lexicon_tags_16 },
    { 0x43134ec6, SUC_lexicon_tags_33 },
    { 0x86934ec6, SUC_lexicon_tags_73 },
    { 0x3d534ec8, SUC_lexicon_tags_11 },
    { 0xf5734ec8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57734ecf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89f34ed1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfdf34ed3, SUC_lexicon_tags_2 },
    { 0x6dd34ed3, SUC_lexicon_tags_45 },
    { 0xfab34ed5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d334ed9, SUC_lexicon_tags_8 },
    { 0xa7334eda, SUC_lexicon_tags_11 },
    { 0xa734eda, SUC_lexicon_tags_31 },
    { 0x89734ed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99334ee6, SUC_lexicon_tags_55 },
    { 0x93134ee7, SUC_lexicon_tags_9 },
    { 0xb8b34ee8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44334eee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4b734ef0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x36d34ef2, SUC_lexicon_tags_11 },
    { 0xc3934ef2, SUC_lexicon_tags_17 },
    { 0xc2934ef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f34ef9, SUC_lexicon_tags_27 },
    { 0x4bf34efa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43f34efc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce734f03, SUC_lexicon_tags_1 },
    { 0xf1334f04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94334f08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7534f0e, SUC_lexicon_tags_15 },
    { 0x56334f0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c134f11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc2534f13, SUC_lexicon_tags_12 },
    { 0x5134f14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e734f16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db34f1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8df34f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a534f23, SUC_lexicon_tags_11 },
    { 0xadd34f24, SUC_lexicon_tags_6 },
    { 0x9b134f25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc0534f27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1734f2b, SUC_lexicon_tags_17 },
    { 0x34734f2b, SUC_lexicon_tags_1 },
    { 0x7bf34f2d, SUC_lexicon_tags_10 },
    { 0xf5534f2e, SUC_lexicon_tags_47 },
    { 0x3db34f2d, SUC_lexicon_tags_15 },
    { 0x20d34f2d, SUC_lexicon_tags_55 },
    { 0x23b34f2f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5934f34, SUC_lexicon_tags_66 },
    { 0xe0934f35, SUC_lexicon_tags_55 },
    { 0xce334f36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18534f42, SUC_lexicon_tags_30 },
    { 0x8134f43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcf34f45, SUC_lexicon_tags_12 },
    { 0x96b34f46, SUC_lexicon_tags_77 },
    { 0xe7134f46, SUC_lexicon_tags_37 },
    { 0xdd534f48, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8334f4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44134f55, SUC_lexicon_tags_25 },
    { 0x9fb34f56, SUC_lexicon_tags_25 },
    { 0xcb34f57, SUC_lexicon_tags_49 },
    { 0x74934f58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf34f64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6534f68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f34f6c, SUC_lexicon_tags_33 },
    { 0xb7334f6d, SUC_lexicon_tags_9 },
    { 0xa0134f6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xee934f71, SUC_lexicon_tags_45 },
    { 0xcc534f72, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc3734f74, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xb8b34f76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42734f7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd34f82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ff34f84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67134f8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb734f92, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xffb34f94, SUC_lexicon_tags_37 },
    { 0x18b34f94, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0x30934f97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb5534f99, SUC_lexicon_tags_2 },
    { 0x28134f9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa534f9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd534fa1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4a334fa3, SUC_lexicon_tags_63 },
    { 0xcf934fa4, SUC_lexicon_tags_13 },
    { 0x5d134fa4, SUC_lexicon_tags_29 },
    { 0x9e934fa4, SUC_lexicon_tags_27 },
    { 0x17934fa7, SUC_lexicon_tags_43 },
    { 0xff934fa4, SUC_lexicon_tags_25 },
    { 0x52f34fa9, SUC_lexicon_tags_15 },
    { 0xc134faa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9534fad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84934faf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce934fb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c134fb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46734fba, SUC_lexicon_tags_33 },
    { 0xc9734fbb, SUC_lexicon_tags_11 },
    { 0x60f34fbc, SUC_lexicon_tags_1 },
    { 0x13534fba, SUC_lexicon_tags_17 },
    { 0x11b34fbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe134fc0, SUC_lexicon_tags_12 },
    { 0x54f34fc1, SUC_lexicon_tags_1 },
    { 0x8a934fc2, SUC_lexicon_tags_12 },
    { 0xecd34fc2, SUC_lexicon_tags_17 },
    { 0xe334fc4, SUC_lexicon_tags_4 },
    { 0x56534fc5, SUC_lexicon_tags_3 },
    { 0xcbf34fc6, SUC_lexicon_tags_37 },
    { 0x68134fc7, SUC_lexicon_tags_45 },
    { 0x71734fc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b334fcd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6534fd5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9934fd8, SUC_lexicon_tags_0 },
    { 0x6e534fd8, SUC_lexicon_tags_15 },
    { 0x47734fda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad134fdd, SUC_lexicon_tags_17 },
    { 0x5df34fdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83534fe1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0534fe8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f34fee, SUC_lexicon_tags_4 },
    { 0x9bb34fef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75734ff1, SUC_lexicon_tags_25 },
    { 0xb2934ff2, SUC_lexicon_tags_17 },
    { 0xe7134ff3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41934ff6, SUC_lexicon_tags_37 },
    { 0x35b34ff7, SUC_lexicon_tags_15 },
    { 0x28134ff8, SUC_lexicon_tags_49 },
    { 0x7ff34ff8, SUC_lexicon_tags_192 },
    { 0xddf34ff7, SUC_lexicon_tags_28 },
    { 0x7e334ffb, SUC_lexicon_tags_14 },
    { 0xead34ff8, SUC_lexicon_tags_1 },
    { 0x70f34ffd, SUC_lexicon_tags_56 },
    { 0xf9134ff9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f35001, SUC_lexicon_tags_37 },
    { 0xad135002, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf693500f, SUC_lexicon_tags_14 },
    { 0x2a73500f, SUC_lexicon_tags_17 },
    { 0xd35011, SUC_lexicon_tags_14 },
    { 0xa3335011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe393501c, SUC_lexicon_tags_7 },
    { 0x9ff3501d, SUC_lexicon_tags_19 },
    { 0x17b3501d, SUC_lexicon_tags_1 },
    { 0x4e33501f, SUC_lexicon_tags_11 },
    { 0xe313501e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd935023, SUC_lexicon_tags_43 },
    { 0xa7535023, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7935026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d35029, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5453502d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7333502f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb135031, SUC_lexicon_tags_15 },
    { 0xdcd35032, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77935040, SUC_lexicon_tags_14 },
    { 0x6f935041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6f35043, SUC_lexicon_tags_9 },
    { 0x74935044, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19335048, SUC_lexicon_tags_19 },
    { 0xaa135049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b3504d, SUC_lexicon_tags_37 },
    { 0xd4f3504e, SUC_lexicon_tags_38 },
    { 0x5433504f, SUC_lexicon_tags_45 },
    { 0x1335050, SUC_lexicon_tags_1 },
    { 0x9f35050, SUC_lexicon_tags_12 },
    { 0x5cf35051, SUC_lexicon_tags_15 },
    { 0xb693504f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d35059, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f3505e, SUC_lexicon_tags_11 },
    { 0x8d93505f, SUC_lexicon_tags_43 },
    { 0xe73505e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfb35062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47935068, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db3506b, SUC_lexicon_tags_15 },
    { 0xc533506c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x94d3506e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4735070, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b35077, SUC_lexicon_tags_33 },
    { 0xf5b35078, SUC_lexicon_tags_37 },
    { 0x51b35079, SUC_lexicon_tags_65 },
    { 0xbfb3507a, SUC_lexicon_tags_4 },
    { 0x7bf3507b, SUC_lexicon_tags_1 },
    { 0x10d3507c, SUC_lexicon_tags_25 },
    { 0x5af3507d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f35081, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb193508a, SUC_lexicon_tags_11 },
    { 0xb193508b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42535090, SUC_lexicon_tags_17 },
    { 0x70f35091, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcf135093, SUC_lexicon_tags_61 },
    { 0x48b35093, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b35098, SUC_lexicon_tags_38 },
    { 0x6f535098, SUC_lexicon_tags_29 },
    { 0x1c13509a, SUC_lexicon_tags_29 },
    { 0xcb335098, SUC_lexicon_tags_1 },
    { 0x6bd3509b, SUC_lexicon_tags_15 },
    { 0x2a13509d, SUC_lexicon_tags_37 },
    { 0xa2b35098, SUC_lexicon_tags_3 },
    { 0x5cb3509f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73d350a1, SUC_lexicon_tags_27 },
    { 0x10d350a2, SUC_lexicon_tags_19 },
    { 0xbbf350a3, SUC_lexicon_tags_17 },
    { 0xb13350a1, SUC_lexicon_tags_11 },
    { 0x841350a3, SUC_lexicon_tags_31 },
    { 0x949350a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983350ab, SUC_lexicon_tags_21 },
    { 0x209350ac, SUC_lexicon_tags_7 },
    { 0x54d350ad, SUC_lexicon_tags_17 },
    { 0xf7350ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b350b3, SUC_lexicon_tags_17 },
    { 0xda5350b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f350b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9350ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f350c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7350c6, SUC_lexicon_tags_2 },
    { 0xe8f350c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcf350c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663350d0, SUC_lexicon_tags_15 },
    { 0xd05350d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63350d6, SUC_lexicon_tags_1 },
    { 0xdf1350d7, SUC_lexicon_tags_15 },
    { 0x84f350d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbf350da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x163350dc, SUC_lexicon_tags_0 },
    { 0x99f350dd, SUC_lexicon_tags_11 },
    { 0x5d350dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd350e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd3350e6, SUC_lexicon_tags_11 },
    { 0x1f9350e6, SUC_lexicon_tags_32 },
    { 0x5fd350e8, SUC_lexicon_tags_15 },
    { 0x57350e7, SUC_lexicon_tags_38 },
    { 0x7d7350ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef350f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x659350f3, SUC_lexicon_tags_795 },
    { 0x787350f4, SUC_lexicon_tags_79 },
    { 0xcf350f5, SUC_lexicon_tags_17 },
    { 0x8d7350f6, SUC_lexicon_tags_2 },
    { 0xbfd350f7, SUC_lexicon_tags_33 },
    { 0xf1d350f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d350fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4ef350fe, SUC_lexicon_tags_15 },
    { 0x28d350ff, SUC_lexicon_tags_8 },
    { 0x15b350fe, SUC_lexicon_tags_25 },
    { 0x24d350ff, SUC_lexicon_tags_37 },
    { 0x33350ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f35107, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74d35109, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5735113, SUC_lexicon_tags_17 },
    { 0xc4735114, SUC_lexicon_tags_35 },
    { 0x7bf35115, SUC_lexicon_tags_25 },
    { 0x67d35116, SUC_lexicon_tags_7 },
    { 0xb2f35115, SUC_lexicon_tags_0 },
    { 0xfbd35116, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5313511a, SUC_lexicon_tags_11 },
    { 0x7553511a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1935121, SUC_lexicon_tags_127 },
    { 0x82b35122, SUC_lexicon_tags_17 },
    { 0x9e135121, SUC_lexicon_tags_48 },
    { 0x1a735124, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd35127, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa735129, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x50d3512b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93d3512d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd535130, SUC_lexicon_tags_2 },
    { 0x35b35131, SUC_lexicon_tags_47 },
    { 0x7b35131, SUC_lexicon_tags_2 },
    { 0xc9735133, SUC_lexicon_tags_2 },
    { 0x3335133, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x45135136, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f3513c, SUC_lexicon_tags_61 },
    { 0x5e93513d, SUC_lexicon_tags_17 },
    { 0xa173513c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab35141, SUC_lexicon_tags_7 },
    { 0x90135142, SUC_lexicon_tags_27 },
    { 0x2b935143, SUC_lexicon_tags_55 },
    { 0x3135144, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac935148, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d3514c, SUC_lexicon_tags_1 },
    { 0x5053514c, SUC_lexicon_tags_25 },
    { 0xb453514e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8df35150, SUC_lexicon_tags_11 },
    { 0xdc135151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f535153, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x39135155, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8335157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7b35159, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a735160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ed35162, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd35169, SUC_lexicon_tags_27 },
    { 0xfb93516a, SUC_lexicon_tags_40 },
    { 0x35f3516a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4335170, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d35173, SUC_lexicon_tags_12 },
    { 0x37935173, SUC_lexicon_tags_0 },
    { 0x96535174, SUC_lexicon_tags_29 },
    { 0x2f35176, SUC_lexicon_tags_15 },
    { 0xbcb35173, SUC_lexicon_tags_2 },
    { 0x46f35178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc953517a, SUC_lexicon_tags_11 },
    { 0x29d3517b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xc193517e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4b35180, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x91535182, SUC_lexicon_tags_11 },
    { 0xf4b35183, SUC_lexicon_tags_757 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b35188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d13518d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbb53518f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7535192, SUC_lexicon_tags_25 },
    { 0x17935193, SUC_lexicon_tags_1 },
    { 0x84735194, SUC_lexicon_tags_17 },
    { 0x2d35192, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x51935197, SUC_lexicon_tags_65 },
    { 0x83735197, SUC_lexicon_tags_39 },
    { 0xfcd35197, SUC_lexicon_tags_47 },
    { 0x2ef35199, SUC_lexicon_tags_15 },
    { 0x93f35198, SUC_lexicon_tags_37 },
    { 0x7ed35197, SUC_lexicon_tags_37 },
    { 0xc293519d, SUC_lexicon_tags_61 },
    { 0xcc13519e, SUC_lexicon_tags_11 },
    { 0x9893519e, SUC_lexicon_tags_12 },
    { 0xaed3519e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42f351a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345351a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x187351ac, SUC_lexicon_tags_17 },
    { 0x207351ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377351b1, SUC_lexicon_tags_19 },
    { 0x6f7351b2, SUC_lexicon_tags_11 },
    { 0xcf3351b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b351b6, SUC_lexicon_tags_11 },
    { 0xcf351b6, SUC_lexicon_tags_37 },
    { 0x5351b7, SUC_lexicon_tags_25 },
    { 0x8cf351b9, SUC_lexicon_tags_45 },
    { 0xab7351b9, SUC_lexicon_tags_17 },
    { 0xf5b351bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f351c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79351d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c9351d3, SUC_lexicon_tags_27 },
    { 0x1c7351d4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7351d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff351db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b351dd, SUC_lexicon_tags_38 },
    { 0x169351de, SUC_lexicon_tags_6 },
    { 0x217351dd, SUC_lexicon_tags_66 },
    { 0x8ef351e0, SUC_lexicon_tags_2 },
    { 0x55d351e1, SUC_lexicon_tags_25 },
    { 0xaff351e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe5b351e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d351e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa11351e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3351f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5f351f2, SUC_lexicon_tags_19 },
    { 0x57351f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1351f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a5351f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857351fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf9b35200, SUC_lexicon_tags_25 },
    { 0x2dd35201, SUC_lexicon_tags_82 },
    { 0x1e135202, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b35207, SUC_lexicon_tags_17 },
    { 0xfcb35207, SUC_lexicon_tags_37 },
    { 0x2bd35209, SUC_lexicon_tags_27 },
    { 0xb0135209, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d13520d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49735213, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f35217, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8273521a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd573521e, SUC_lexicon_tags_17 },
    { 0x9473521f, SUC_lexicon_tags_6 },
    { 0x6033521f, SUC_lexicon_tags_35 },
    { 0x7bf3521e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41335224, SUC_lexicon_tags_45 },
    { 0x8c335224, SUC_lexicon_tags_26 },
    { 0xb8d35226, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x31135229, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc453522c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e935232, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af35235, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b35240, SUC_lexicon_tags_214 },
    { 0x8eb35241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1535247, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad13524b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd3524f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8135256, SUC_lexicon_tags_11 },
    { 0x1735256, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff3525d, SUC_lexicon_tags_2 },
    { 0xddb3525e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddf35260, SUC_lexicon_tags_9 },
    { 0x3f535260, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e935264, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34735268, SUC_lexicon_tags_87 },
    { 0x35d35268, SUC_lexicon_tags_15 },
    { 0xf6d3526a, SUC_lexicon_tags_1 },
    { 0xe013526a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4333526e, SUC_lexicon_tags_8 },
    { 0xee13526f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x63535271, SUC_lexicon_tags_107 },
    { 0x6b535272, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xabd35274, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64735278, SUC_lexicon_tags_15 },
    { 0xf0935279, SUC_lexicon_tags_14 },
    { 0xa2d35279, SUC_lexicon_tags_19 },
    { 0xb413527b, SUC_lexicon_tags_33 },
    { 0xef3527a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5cd3527e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9b35280, SUC_lexicon_tags_27 },
    { 0xf6d35281, SUC_lexicon_tags_40 },
    { 0x57535280, SUC_lexicon_tags_23 },
    { 0x94f35282, SUC_lexicon_tags_7 },
    { 0x9d35284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca135288, SUC_lexicon_tags_8 },
    { 0x5f535289, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2853528f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x39535292, SUC_lexicon_tags_9 },
    { 0x72135293, SUC_lexicon_tags_39 },
    { 0xd535292, SUC_lexicon_tags_1 },
    { 0xaef35295, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d35299, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3733529b, SUC_lexicon_tags_2 },
    { 0x1cb3529b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125352a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41352a5, SUC_lexicon_tags_146 },
    { 0x9ef352a6, SUC_lexicon_tags_15 },
    { 0x59352a6, SUC_lexicon_tags_25 },
    { 0x7c9352a8, SUC_lexicon_tags_14 },
    { 0x907352a9, SUC_lexicon_tags_91 },
    { 0xd33352a9, SUC_lexicon_tags_12 },
    { 0x46f352a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d352af, SUC_lexicon_tags_30 },
    { 0x83b352af, SUC_lexicon_tags_11 },
    { 0xe1f352b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1352b5, SUC_lexicon_tags_17 },
    { 0x489352b6, SUC_lexicon_tags_11 },
    { 0x1e5352b7, SUC_lexicon_tags_15 },
    { 0xbed352b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83352bb, SUC_lexicon_tags_19 },
    { 0x60d352bb, SUC_lexicon_tags_46 },
    { 0xa9b352bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d352c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x833352c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371352c8, SUC_lexicon_tags_1 },
    { 0xe53352c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37352ce, SUC_lexicon_tags_15 },
    { 0xec9352cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51352d2, SUC_lexicon_tags_8 },
    { 0xe07352d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51352d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd352d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d352e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69352e5, SUC_lexicon_tags_28 },
    { 0x433352e6, SUC_lexicon_tags_14 },
    { 0x63352e7, SUC_lexicon_tags_16 },
    { 0xe7d352e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7c5352ea, SUC_lexicon_tags_19 },
    { 0xd31352eb, SUC_lexicon_tags_136 },
    { 0x2c9352ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f352f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d3352f2, SUC_lexicon_tags_63 },
    { 0x7e3352f3, SUC_lexicon_tags_17 },
    { 0xa0d352f4, SUC_lexicon_tags_207 },
    { 0x81352f5, SUC_lexicon_tags_55 },
    { 0xc11352f5, SUC_lexicon_tags_23 },
    { 0xe41352f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80535301, SUC_lexicon_tags_40 },
    { 0x89d35301, SUC_lexicon_tags_10 },
    { 0x3c335303, SUC_lexicon_tags_1 },
    { 0xdad35302, SUC_lexicon_tags_12 },
    { 0xd1935305, SUC_lexicon_tags_11 },
    { 0x75535301, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f3530a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd513530d, SUC_lexicon_tags_11 },
    { 0x6a33530e, SUC_lexicon_tags_37 },
    { 0xac73530f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f35313, SUC_lexicon_tags_10 },
    { 0x38b35314, SUC_lexicon_tags_15 },
    { 0xdc735315, SUC_lexicon_tags_17 },
    { 0x3c335316, SUC_lexicon_tags_47 },
    { 0xca735316, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3433531a, SUC_lexicon_tags_12 },
    { 0x79f3531a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c53531e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65935325, SUC_lexicon_tags_27 },
    { 0xc4f35325, SUC_lexicon_tags_26 },
    { 0xa6135327, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd633532a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d35332, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4135338, SUC_lexicon_tags_0 },
    { 0xfb35338, SUC_lexicon_tags_25 },
    { 0x2373533a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53533e, SUC_lexicon_tags_45 },
    { 0x7bb3533f, SUC_lexicon_tags_31 },
    { 0xf5b3533f, SUC_lexicon_tags_60 },
    { 0xa6b35341, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97735349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1833534e, SUC_lexicon_tags_15 },
    { 0x2c33534f, SUC_lexicon_tags_4 },
    { 0xabb3534e, SUC_lexicon_tags_1 },
    { 0x9d535351, SUC_lexicon_tags_15 },
    { 0x91b3534e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbeb35354, SUC_lexicon_tags_12 },
    { 0x97d35355, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28d35357, SUC_lexicon_tags_31 },
    { 0x95935358, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e735363, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d135366, SUC_lexicon_tags_37 },
    { 0x3df35366, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74f35369, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b3536e, SUC_lexicon_tags_45 },
    { 0x9273536f, SUC_lexicon_tags_39 },
    { 0x1fb3536f, SUC_lexicon_tags_25 },
    { 0xaef3536f, SUC_lexicon_tags_12 },
    { 0x7f93536f, SUC_lexicon_tags_6 },
    { 0x10d35372, SUC_lexicon_tags_63 },
    { 0xf2735374, SUC_lexicon_tags_11 },
    { 0xe2335371, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d3537a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x273537c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97735381, SUC_lexicon_tags_63 },
    { 0x4935382, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa735389, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb893538c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7935395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd935399, SUC_lexicon_tags_14 },
    { 0x49f3539a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8df3539c, SUC_lexicon_tags_48 },
    { 0xec93539d, SUC_lexicon_tags_33 },
    { 0x793539e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5353a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3eb353a3, SUC_lexicon_tags_17 },
    { 0x3c3353a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf45353a6, SUC_lexicon_tags_60 },
    { 0xdb9353a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa91353a9, SUC_lexicon_tags_11 },
    { 0x5c3353aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43353ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1353b6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1c9353b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x251353ba, SUC_lexicon_tags_37 },
    { 0xbb353bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf353bd, SUC_lexicon_tags_59 },
    { 0xf75353be, SUC_lexicon_tags_30 },
    { 0x8cd353bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1353c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83353c6, SUC_lexicon_tags_21 },
    { 0x69b353c7, SUC_lexicon_tags_7 },
    { 0xd23353c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x357353ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd353cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549353d2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d353d5, SUC_lexicon_tags_55 },
    { 0x977353d5, SUC_lexicon_tags_1 },
    { 0x10f353d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63353dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571353e0, SUC_lexicon_tags_13 },
    { 0xe3b353e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x363353e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273353e7, SUC_lexicon_tags_47 },
    { 0x2d3353e7, SUC_lexicon_tags_0 },
    { 0x3fb353e7, SUC_lexicon_tags_25 },
    { 0x10f353ea, SUC_lexicon_tags_25 },
    { 0x35353eb, SUC_lexicon_tags_47 },
    { 0x391353eb, SUC_lexicon_tags_77 },
    { 0x127353ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb5353ef, SUC_lexicon_tags_66 },
    { 0x8f353f0, SUC_lexicon_tags_21 },
    { 0x375353f0, SUC_lexicon_tags_69 },
    { 0x8cd353ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb353f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f353fa, SUC_lexicon_tags_23 },
    { 0x1bf353fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5353fe, SUC_lexicon_tags_14 },
    { 0xf6d353fe, SUC_lexicon_tags_45 },
    { 0x153353ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d535402, SUC_lexicon_tags_17 },
    { 0x22935403, SUC_lexicon_tags_25 },
    { 0x96535404, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d3540d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x13535410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd535412, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1335418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc853541a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4193541d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa593541f, SUC_lexicon_tags_15 },
    { 0x7f935420, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe135422, SUC_lexicon_tags_31 },
    { 0x66135423, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x96f35425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3f35427, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5153542f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b935432, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e13543a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5673543e, SUC_lexicon_tags_2 },
    { 0xeab3543f, SUC_lexicon_tags_14 },
    { 0xa0f35440, SUC_lexicon_tags_63 },
    { 0x47535441, SUC_lexicon_tags_12 },
    { 0x75935442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad35446, SUC_lexicon_tags_1 },
    { 0x8335446, SUC_lexicon_tags_12 },
    { 0xc4d35447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbf3544a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf893544e, SUC_lexicon_tags_11 },
    { 0x12d3544e, SUC_lexicon_tags_37 },
    { 0x8c135450, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59f35452, SUC_lexicon_tags_60 },
    { 0x4735453, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae535464, SUC_lexicon_tags_47 },
    { 0xa7335465, SUC_lexicon_tags_25 },
    { 0x1df35466, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1233546a, SUC_lexicon_tags_10 },
    { 0x8db3546a, SUC_lexicon_tags_49 },
    { 0x4af3546c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x91b3546e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd35478, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b93547c, SUC_lexicon_tags_28 },
    { 0x66d3547c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xefd3547f, SUC_lexicon_tags_61 },
    { 0x2173547f, SUC_lexicon_tags_12 },
    { 0x42b35480, SUC_lexicon_tags_61 },
    { 0xce335482, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c335487, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd3548a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0935490, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb335492, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x21935495, SUC_lexicon_tags_47 },
    { 0xa9935496, SUC_lexicon_tags_27 },
    { 0xa5735497, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98b35499, SUC_lexicon_tags_37 },
    { 0x29135499, SUC_lexicon_tags_49 },
    { 0xbe33549a, SUC_lexicon_tags_7 },
    { 0xb5b35499, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8cb3549e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957354a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x817354a5, SUC_lexicon_tags_55 },
    { 0x4f354a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x399354a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x905354aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3354ad, SUC_lexicon_tags_7 },
    { 0x49f354ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea5354b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4dd354b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe1b354b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b354ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5354be, SUC_lexicon_tags_33 },
    { 0x7cf354bf, SUC_lexicon_tags_63 },
    { 0x869354c0, SUC_lexicon_tags_60 },
    { 0x5ed354be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf21354c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc23354c5, SUC_lexicon_tags_27 },
    { 0x403354c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7354c9, SUC_lexicon_tags_60 },
    { 0x81f354c9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x981354cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad354d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1354d5, SUC_lexicon_tags_1 },
    { 0xd59354d6, SUC_lexicon_tags_17 },
    { 0xac3354d5, SUC_lexicon_tags_37 },
    { 0xfe5354d8, SUC_lexicon_tags_17 },
    { 0xb75354d9, SUC_lexicon_tags_6 },
    { 0x4e3354da, SUC_lexicon_tags_68 },
    { 0xdcd354da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd354e0, SUC_lexicon_tags_48 },
    { 0x9f5354e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f354e5, SUC_lexicon_tags_11 },
    { 0x16b354e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87354ea, SUC_lexicon_tags_15 },
    { 0xb0f354eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3354f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x189354f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x907354f4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab354f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d354ff, SUC_lexicon_tags_57 },
    { 0xddd354ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf35509, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1bd3550b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdcf3550d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb35510, SUC_lexicon_tags_38 },
    { 0x2b935510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4735513, SUC_lexicon_tags_68 },
    { 0x85335514, SUC_lexicon_tags_7 },
    { 0xee535514, SUC_lexicon_tags_37 },
    { 0x96735516, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x53335518, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcc73551a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x893551c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7013551e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93535520, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28935522, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x60135525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d35529, SUC_lexicon_tags_4 },
    { 0xc9f35529, SUC_lexicon_tags_39 },
    { 0x833552b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b135533, SUC_lexicon_tags_46 },
    { 0xc2535533, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a335536, SUC_lexicon_tags_10 },
    { 0x51535537, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb3553a, SUC_lexicon_tags_65 },
    { 0xa633553b, SUC_lexicon_tags_43 },
    { 0xd893553c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7793553e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x33b35540, SUC_lexicon_tags_7 },
    { 0x4e935541, SUC_lexicon_tags_15 },
    { 0xb8735542, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xdb735544, SUC_lexicon_tags_9 },
    { 0xad35545, SUC_lexicon_tags_23 },
    { 0x8e935545, SUC_lexicon_tags_1 },
    { 0x81535547, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x32b35549, SUC_lexicon_tags_2 },
    { 0x37f3554a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc913554e, SUC_lexicon_tags_5 },
    { 0x2093554f, SUC_lexicon_tags_12 },
    { 0xb1535550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90335552, SUC_lexicon_tags_45 },
    { 0x37f35552, SUC_lexicon_tags_50 },
    { 0x23f35554, SUC_lexicon_tags_12 },
    { 0x17135555, SUC_lexicon_tags_25 },
    { 0x64535555, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x47d35558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd135561, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b35565, SUC_lexicon_tags_23 },
    { 0xf1935566, SUC_lexicon_tags_66 },
    { 0xbcb35567, SUC_lexicon_tags_11 },
    { 0x21b35566, SUC_lexicon_tags_25 },
    { 0xa8935567, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee93556c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb833557f, SUC_lexicon_tags_30 },
    { 0x2e335580, SUC_lexicon_tags_11 },
    { 0x9735581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b935588, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb1f3558a, SUC_lexicon_tags_12 },
    { 0x42d3558b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1af3558d, SUC_lexicon_tags_17 },
    { 0x70d3558e, SUC_lexicon_tags_113 },
    { 0x5113558f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc0135591, SUC_lexicon_tags_15 },
    { 0xe8135592, SUC_lexicon_tags_11 },
    { 0x2ff35593, SUC_lexicon_tags_1 },
    { 0x67135594, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3f35596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d3559c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655355a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77355a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad355a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd355ab, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9355b0, SUC_lexicon_tags_1 },
    { 0xda3355b0, SUC_lexicon_tags_1 },
    { 0xded355b1, SUC_lexicon_tags_37 },
    { 0x19b355b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x903355b5, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab355b8, SUC_lexicon_tags_32 },
    { 0xe49355b9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b355be, SUC_lexicon_tags_12 },
    { 0x695355bf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d355c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd49355c6, SUC_lexicon_tags_12 },
    { 0x2df355c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31355ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9b9355cc, SUC_lexicon_tags_15 },
    { 0xe27355cd, SUC_lexicon_tags_80 },
    { 0x5c9355ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d355d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07355d8, SUC_lexicon_tags_8 },
    { 0xbb355d9, SUC_lexicon_tags_15 },
    { 0x8ed355d9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x45b355dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3355de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd7d355e0, SUC_lexicon_tags_37 },
    { 0x83d355e1, SUC_lexicon_tags_11 },
    { 0xcbd355e1, SUC_lexicon_tags_0 },
    { 0x7a7355e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd355e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7355ee, SUC_lexicon_tags_15 },
    { 0x4d3355ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4cd355f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9cd355f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x49d355f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11355fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb35602, SUC_lexicon_tags_19 },
    { 0xd2935603, SUC_lexicon_tags_39 },
    { 0xf9335603, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdeb35606, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b35610, SUC_lexicon_tags_12 },
    { 0xf8135610, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xc5d35613, SUC_lexicon_tags_25 },
    { 0xdb535614, SUC_lexicon_tags_21 },
    { 0x9535615, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc53561b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f3561e, SUC_lexicon_tags_30 },
    { 0x2ff3561f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab35622, SUC_lexicon_tags_9 },
    { 0x9a935623, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93562d, SUC_lexicon_tags_7 },
    { 0x4ed3562e, SUC_lexicon_tags_135 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb35631, SUC_lexicon_tags_7 },
    { 0x32735631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad35634, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10135636, SUC_lexicon_tags_7 },
    { 0xbad35637, SUC_lexicon_tags_27 },
    { 0x6e735636, SUC_lexicon_tags_151 },
    { 0x34f35639, SUC_lexicon_tags_14 },
    { 0x16f35639, SUC_lexicon_tags_37 },
    { 0xa693563b, SUC_lexicon_tags_25 },
    { 0x4c335639, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff35640, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82935646, SUC_lexicon_tags_66 },
    { 0xfd135647, SUC_lexicon_tags_43 },
    { 0x32735648, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x19d3564a, SUC_lexicon_tags_16 },
    { 0xe613564b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77335654, SUC_lexicon_tags_6 },
    { 0xb1135654, SUC_lexicon_tags_1 },
    { 0xbeb35654, SUC_lexicon_tags_17 },
    { 0x22535657, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f3565a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6d3565c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd153565e, SUC_lexicon_tags_30 },
    { 0x1ab3565e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b35662, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d535667, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5953566a, SUC_lexicon_tags_39 },
    { 0x5cf3566b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfdb3566d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccb3566f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd3535671, SUC_lexicon_tags_12 },
    { 0x6e935672, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8353567a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2053567d, SUC_lexicon_tags_6 },
    { 0x25b3567d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51135681, SUC_lexicon_tags_17 },
    { 0xcb935682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef35687, SUC_lexicon_tags_15 },
    { 0x64d35687, SUC_lexicon_tags_3 },
    { 0x29135689, SUC_lexicon_tags_0 },
    { 0xd3568a, SUC_lexicon_tags_27 },
    { 0xed3568b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ab3568d, SUC_lexicon_tags_45 },
    { 0xb9d3568e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbfb35690, SUC_lexicon_tags_37 },
    { 0xf5935690, SUC_lexicon_tags_37 },
    { 0x3d35692, SUC_lexicon_tags_23 },
    { 0x62f35691, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8333569e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1356a1, SUC_lexicon_tags_61 },
    { 0x9ab356a2, SUC_lexicon_tags_25 },
    { 0x175356a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993356a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67d356ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7356af, SUC_lexicon_tags_2 },
    { 0x7d9356af, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf356b8, SUC_lexicon_tags_49 },
    { 0x801356b9, SUC_lexicon_tags_22 },
    { 0x797356ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x537356bc, SUC_lexicon_tags_45 },
    { 0xf1d356bc, SUC_lexicon_tags_27 },
    { 0x58d356be, SUC_lexicon_tags_2 },
    { 0x1df356bf, SUC_lexicon_tags_33 },
    { 0xe6b356c0, SUC_lexicon_tags_33 },
    { 0x229356c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1356c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873356c8, SUC_lexicon_tags_43 },
    { 0xc87356c9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8cb356cb, SUC_lexicon_tags_7 },
    { 0xa21356cc, SUC_lexicon_tags_17 },
    { 0x405356cd, SUC_lexicon_tags_12 },
    { 0x519356ce, SUC_lexicon_tags_27 },
    { 0xc35356cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb356d8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f356dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b356e1, SUC_lexicon_tags_46 },
    { 0x6ab356e1, SUC_lexicon_tags_29 },
    { 0xb09356e3, SUC_lexicon_tags_118 },
    { 0xb15356e4, SUC_lexicon_tags_15 },
    { 0x9db356e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf356e8, SUC_lexicon_tags_17 },
    { 0xac5356e9, SUC_lexicon_tags_38 },
    { 0x711356e9, SUC_lexicon_tags_29 },
    { 0x4f5356eb, SUC_lexicon_tags_87 },
    { 0x995356ec, SUC_lexicon_tags_2 },
    { 0xceb356ed, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893356f1, SUC_lexicon_tags_15 },
    { 0x7ef356f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87356f5, SUC_lexicon_tags_43 },
    { 0x207356f6, SUC_lexicon_tags_15 },
    { 0xc53356f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79b356f9, SUC_lexicon_tags_25 },
    { 0x65d356fa, SUC_lexicon_tags_1 },
    { 0x713356f9, SUC_lexicon_tags_37 },
    { 0xdaf356fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76135700, SUC_lexicon_tags_16 },
    { 0x97135700, SUC_lexicon_tags_6 },
    { 0x1f735702, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b35707, SUC_lexicon_tags_15 },
    { 0xbbb35708, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db3570d, SUC_lexicon_tags_12 },
    { 0x8f33570d, SUC_lexicon_tags_37 },
    { 0x9a93570f, SUC_lexicon_tags_19 },
    { 0xc8f3570f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30535715, SUC_lexicon_tags_25 },
    { 0xbb535715, SUC_lexicon_tags_37 },
    { 0xa8b35715, SUC_lexicon_tags_60 },
    { 0xce335718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf773571a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a93571d, SUC_lexicon_tags_19 },
    { 0xdbb3571e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db35722, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd35726, SUC_lexicon_tags_16 },
    { 0xe135726, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e33572b, SUC_lexicon_tags_0 },
    { 0x4b13572c, SUC_lexicon_tags_7 },
    { 0xf2f3572c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d35730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d73573f, SUC_lexicon_tags_25 },
    { 0xf5335740, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72335742, SUC_lexicon_tags_44 },
    { 0x9a335743, SUC_lexicon_tags_15 },
    { 0x5ad35743, SUC_lexicon_tags_15 },
    { 0x8b735744, SUC_lexicon_tags_38 },
    { 0x98135746, SUC_lexicon_tags_12 },
    { 0x61135747, SUC_lexicon_tags_84 },
    { 0x1b535748, SUC_lexicon_tags_48 },
    { 0xa7935748, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb935753, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe535755, SUC_lexicon_tags_55 },
    { 0xe0d35755, SUC_lexicon_tags_74 },
    { 0xd2735756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd3575c, SUC_lexicon_tags_2 },
    { 0x8a93575d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab35762, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x33935764, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a135766, SUC_lexicon_tags_6 },
    { 0xa535767, SUC_lexicon_tags_37 },
    { 0x45f35767, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa1d3576a, SUC_lexicon_tags_17 },
    { 0x5f3576a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x19d3576d, SUC_lexicon_tags_2 },
    { 0xe2f3576e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d35770, SUC_lexicon_tags_23 },
    { 0xfc135770, SUC_lexicon_tags_9 },
    { 0x33335772, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb35775, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1653577b, SUC_lexicon_tags_11 },
    { 0x5973577c, SUC_lexicon_tags_12 },
    { 0x6f13577b, SUC_lexicon_tags_37 },
    { 0x3e33577e, SUC_lexicon_tags_15 },
    { 0x5a93577f, SUC_lexicon_tags_12 },
    { 0xfb135780, SUC_lexicon_tags_25 },
    { 0x43135780, SUC_lexicon_tags_37 },
    { 0x4e53577f, SUC_lexicon_tags_1 },
    { 0x9d35783, SUC_lexicon_tags_43 },
    { 0x28935784, SUC_lexicon_tags_1 },
    { 0x50535783, SUC_lexicon_tags_63 },
    { 0x20f35786, SUC_lexicon_tags_47 },
    { 0x12f35784, SUC_lexicon_tags_25 },
    { 0x98135785, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee93578e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32335794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f35799, SUC_lexicon_tags_14 },
    { 0x3813579a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed3579d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x557357a0, SUC_lexicon_tags_11 },
    { 0x975357a0, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb357a4, SUC_lexicon_tags_19 },
    { 0xe53357a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf357ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383357b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x297357b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf357b6, SUC_lexicon_tags_25 },
    { 0x731357b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887357bc, SUC_lexicon_tags_17 },
    { 0xe2d357bc, SUC_lexicon_tags_2 },
    { 0x4e1357bc, SUC_lexicon_tags_7 },
    { 0x95b357bc, SUC_lexicon_tags_31 },
    { 0x121357bf, SUC_lexicon_tags_17 },
    { 0xd6b357c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfa9357c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625357c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87357d1, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x4d1357d3, SUC_lexicon_tags_11 },
    { 0xfb9357d4, SUC_lexicon_tags_21 },
    { 0xd45357d5, SUC_lexicon_tags_15 },
    { 0xc21357d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1cd357d8, SUC_lexicon_tags_45 },
    { 0x291357d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x333357db, SUC_lexicon_tags_11 },
    { 0x9c9357db, SUC_lexicon_tags_112 },
    { 0xf39357dd, SUC_lexicon_tags_77 },
    { 0x4f357de, SUC_lexicon_tags_68 },
    { 0x397357de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1357e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x193357e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d357e8, SUC_lexicon_tags_69 },
    { 0x439357e9, SUC_lexicon_tags_25 },
    { 0x837357ea, SUC_lexicon_tags_27 },
    { 0x837357ea, SUC_lexicon_tags_13 },
    { 0xd17357ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3357f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7357f6, SUC_lexicon_tags_9 },
    { 0x27357f7, SUC_lexicon_tags_0 },
    { 0x5a9357f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x843357fa, SUC_lexicon_tags_11 },
    { 0xe59357fb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb357ff, SUC_lexicon_tags_23 },
    { 0xb45357ff, SUC_lexicon_tags_17 },
    { 0x56b35801, SUC_lexicon_tags_32 },
    { 0xa8f35801, SUC_lexicon_tags_39 },
    { 0x5ab35803, SUC_lexicon_tags_0 },
    { 0x3f35804, SUC_lexicon_tags_148 },
    { 0xe7335800, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe4d35807, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7513580d, SUC_lexicon_tags_58 },
    { 0x51f3580e, SUC_lexicon_tags_29 },
    { 0x9613580e, SUC_lexicon_tags_45 },
    { 0xdf33580d, SUC_lexicon_tags_7 },
    { 0xacd3580e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf35816, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3581a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db3581f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb35823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32135826, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe535829, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99935830, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1735834, SUC_lexicon_tags_43 },
    { 0xd535835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc135838, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf2d3583a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7d3583c, SUC_lexicon_tags_19 },
    { 0xe0b3583c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd35843, SUC_lexicon_tags_3 },
    { 0x38d35844, SUC_lexicon_tags_17 },
    { 0x37535845, SUC_lexicon_tags_105 },
    { 0x24735846, SUC_lexicon_tags_12 },
    { 0xaeb35847, SUC_lexicon_tags_27 },
    { 0xd935848, SUC_lexicon_tags_3 },
    { 0xf3335849, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xe033584c, SUC_lexicon_tags_25 },
    { 0xb633584d, SUC_lexicon_tags_47 },
    { 0x9d73584c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb135850, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f35854, SUC_lexicon_tags_61 },
    { 0x98735855, SUC_lexicon_tags_7 },
    { 0x45135855, SUC_lexicon_tags_25 },
    { 0xc935857, SUC_lexicon_tags_171 },
    { 0xd6935857, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3ef3585a, SUC_lexicon_tags_63 },
    { 0xdf3585b, SUC_lexicon_tags_47 },
    { 0x12d3585c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f35860, SUC_lexicon_tags_37 },
    { 0x71f35861, SUC_lexicon_tags_49 },
    { 0xf6135862, SUC_lexicon_tags_32 },
    { 0x2cd35863, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95535868, SUC_lexicon_tags_32 },
    { 0x71935869, SUC_lexicon_tags_12 },
    { 0x4ef35868, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd35870, SUC_lexicon_tags_25 },
    { 0x62b35870, SUC_lexicon_tags_26 },
    { 0xd8135871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb735875, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b35879, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x34d3587b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff35880, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b735887, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79d35889, SUC_lexicon_tags_7 },
    { 0x4dd35889, SUC_lexicon_tags_47 },
    { 0x333588b, SUC_lexicon_tags_31 },
    { 0xbe33588c, SUC_lexicon_tags_25 },
    { 0xad93588d, SUC_lexicon_tags_6 },
    { 0x1273588c, SUC_lexicon_tags_2 },
    { 0xd1d3588f, SUC_lexicon_tags_19 },
    { 0x7133588b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad35893, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc213589b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf1f3589d, SUC_lexicon_tags_23 },
    { 0xbd33589d, SUC_lexicon_tags_37 },
    { 0x3513589d, SUC_lexicon_tags_13 },
    { 0x1453589e, SUC_lexicon_tags_37 },
    { 0x2f13589e, SUC_lexicon_tags_151 },
    { 0x473358a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f358a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889358aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b358b1, SUC_lexicon_tags_4 },
    { 0x3b1358b2, SUC_lexicon_tags_100 },
    { 0x4fd358b1, SUC_lexicon_tags_37 },
    { 0x9a1358b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843358bc, SUC_lexicon_tags_33 },
    { 0x2a5358bc, SUC_lexicon_tags_19 },
    { 0xb7b358be, SUC_lexicon_tags_15 },
    { 0x127358bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f5358c1, SUC_lexicon_tags_0 },
    { 0x275358c1, SUC_lexicon_tags_52 },
    { 0xb13358c2, SUC_lexicon_tags_7 },
    { 0xa3d358c4, SUC_lexicon_tags_37 },
    { 0x643358c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9b358c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b1358c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3b1358cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3358cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867358d3, SUC_lexicon_tags_28 },
    { 0x58d358d4, SUC_lexicon_tags_2 },
    { 0x6f358d5, SUC_lexicon_tags_37 },
    { 0x655358d6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f358df, SUC_lexicon_tags_27 },
    { 0x72f358df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561358e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17358e9, SUC_lexicon_tags_19 },
    { 0x307358e9, SUC_lexicon_tags_37 },
    { 0xd73358eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x593358ed, SUC_lexicon_tags_27 },
    { 0x1ff358ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71358f2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67358f6, SUC_lexicon_tags_9 },
    { 0xc39358f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x723358f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1358fc, SUC_lexicon_tags_17 },
    { 0xc6f358fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x643358ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f35908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c53590c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8135910, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d35916, SUC_lexicon_tags_9 },
    { 0xab735917, SUC_lexicon_tags_9 },
    { 0xe2135918, SUC_lexicon_tags_12 },
    { 0x4b935919, SUC_lexicon_tags_37 },
    { 0xbb53591a, SUC_lexicon_tags_6 },
    { 0xec535919, SUC_lexicon_tags_37 },
    { 0xa6f3591c, SUC_lexicon_tags_87 },
    { 0x6b3591d, SUC_lexicon_tags_29 },
    { 0x8f3591e, SUC_lexicon_tags_17 },
    { 0x7a93591e, SUC_lexicon_tags_14 },
    { 0xb0935920, SUC_lexicon_tags_65 },
    { 0x4b535920, SUC_lexicon_tags_1 },
    { 0xc4935922, SUC_lexicon_tags_38 },
    { 0x77535923, SUC_lexicon_tags_17 },
    { 0xbb73591d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb35927, SUC_lexicon_tags_2 },
    { 0x9e735927, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3853592d, SUC_lexicon_tags_37 },
    { 0x6a73592e, SUC_lexicon_tags_37 },
    { 0x5ed3592f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8735938, SUC_lexicon_tags_17 },
    { 0xbbf35938, SUC_lexicon_tags_11 },
    { 0x6db35938, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15735943, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7b135945, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9d135947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb3594c, SUC_lexicon_tags_7 },
    { 0xe633594d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49b3594f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f35955, SUC_lexicon_tags_66 },
    { 0x71f35956, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x96335959, SUC_lexicon_tags_38 },
    { 0x9d93595a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc53595c, SUC_lexicon_tags_25 },
    { 0xf493595d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df35960, SUC_lexicon_tags_37 },
    { 0x3f735960, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42535963, SUC_lexicon_tags_37 },
    { 0x7c935963, SUC_lexicon_tags_2 },
    { 0x5ef35963, SUC_lexicon_tags_7 },
    { 0xf2735966, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd35969, SUC_lexicon_tags_26 },
    { 0x98735969, SUC_lexicon_tags_47 },
    { 0xbf73596b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3596e, SUC_lexicon_tags_2 },
    { 0x41b3596f, SUC_lexicon_tags_7 },
    { 0x8273596f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f35973, SUC_lexicon_tags_25 },
    { 0x70935974, SUC_lexicon_tags_33 },
    { 0xce135973, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf35979, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d73597b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d3597f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf93598e, SUC_lexicon_tags_15 },
    { 0x9713598e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21135994, SUC_lexicon_tags_9 },
    { 0x16b35995, SUC_lexicon_tags_2 },
    { 0xca735995, SUC_lexicon_tags_7 },
    { 0x1e335997, SUC_lexicon_tags_12 },
    { 0x23735998, SUC_lexicon_tags_15 },
    { 0x9735996, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x44f3599b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db359a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x673359a4, SUC_lexicon_tags_37 },
    { 0x4dd359a5, SUC_lexicon_tags_57 },
    { 0x537359a5, SUC_lexicon_tags_15 },
    { 0x8a3359a5, SUC_lexicon_tags_17 },
    { 0xff5359a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1359ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x815359b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47359b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9359ba, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x46d359bc, SUC_lexicon_tags_37 },
    { 0x4a1359bd, SUC_lexicon_tags_74 },
    { 0xc95359be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d359c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d359ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xceb359d0, SUC_lexicon_tags_37 },
    { 0xf1f359d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x423359d3, SUC_lexicon_tags_15 },
    { 0xf19359d3, SUC_lexicon_tags_80 },
    { 0x24f359d5, SUC_lexicon_tags_14 },
    { 0x443359d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcef359d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9359dd, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded359e3, SUC_lexicon_tags_8 },
    { 0xfeb359e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125359e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3359f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebd359f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b359fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd359fc, SUC_lexicon_tags_17 },
    { 0xc83359fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbf359ff, SUC_lexicon_tags_29 },
    { 0x699359ff, SUC_lexicon_tags_17 },
    { 0xaf535a01, SUC_lexicon_tags_14 },
    { 0xe9735a01, SUC_lexicon_tags_37 },
    { 0x37b35a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b35a08, SUC_lexicon_tags_12 },
    { 0x97b35a09, SUC_lexicon_tags_11 },
    { 0xc8335a0a, SUC_lexicon_tags_17 },
    { 0x60535a0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55335a0d, SUC_lexicon_tags_15 },
    { 0xef35a0e, SUC_lexicon_tags_60 },
    { 0x6eb35a0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d35a14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30735a18, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f135a1d, SUC_lexicon_tags_39 },
    { 0x99b35a1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73735a22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6d35a24, SUC_lexicon_tags_61 },
    { 0x65d35a25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b935a28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x84735a2a, SUC_lexicon_tags_2 },
    { 0x58135a2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf35a31, SUC_lexicon_tags_7 },
    { 0x55535a32, SUC_lexicon_tags_30 },
    { 0x7535a33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d335a37, SUC_lexicon_tags_1 },
    { 0x9b35a37, SUC_lexicon_tags_12 },
    { 0xd135a38, SUC_lexicon_tags_27 },
    { 0x28335a3a, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d35a3d, SUC_lexicon_tags_25 },
    { 0x54b35a3d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b35a42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e135a47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8e535a49, SUC_lexicon_tags_37 },
    { 0x38935a49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67735a50, SUC_lexicon_tags_63 },
    { 0xc6135a51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d35a54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb9735a56, SUC_lexicon_tags_82 },
    { 0x8bb35a56, SUC_lexicon_tags_113 },
    { 0x5a335a58, SUC_lexicon_tags_0 },
    { 0xbbb35a58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4e735a5b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcad35a5d, SUC_lexicon_tags_8 },
    { 0x45135a5e, SUC_lexicon_tags_2 },
    { 0x2a335a5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf35a63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51d35a65, SUC_lexicon_tags_5 },
    { 0xe2135a66, SUC_lexicon_tags_12 },
    { 0x1e535a67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60135a6c, SUC_lexicon_tags_61 },
    { 0x1e135a6d, SUC_lexicon_tags_37 },
    { 0x92935a6e, SUC_lexicon_tags_4 },
    { 0x62735a6f, SUC_lexicon_tags_97 },
    { 0x7935a70, SUC_lexicon_tags_136 },
    { 0x33935a71, SUC_lexicon_tags_26 },
    { 0x21135a70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x35d35a74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdf935a76, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf35a79, SUC_lexicon_tags_12 },
    { 0xcc135a7a, SUC_lexicon_tags_14 },
    { 0x75735a7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea935a7e, SUC_lexicon_tags_7 },
    { 0x82f35a7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea535a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf35a85, SUC_lexicon_tags_174 },
    { 0x7cf35a85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x16f35a88, SUC_lexicon_tags_25 },
    { 0x8fb35a89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3535a8e, SUC_lexicon_tags_88 },
    { 0x3ef35a8e, SUC_lexicon_tags_11 },
    { 0xde735a90, SUC_lexicon_tags_1 },
    { 0x58b35a90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a935a98, SUC_lexicon_tags_37 },
    { 0x86f35a99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb35a9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51b35a9f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x13335aa2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2e335aa4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff935aa6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x77b35aa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f35aac, SUC_lexicon_tags_11 },
    { 0xaf335aac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x43135ab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34735ab2, SUC_lexicon_tags_37 },
    { 0x7fd35ab3, SUC_lexicon_tags_27 },
    { 0x3dd35ab3, SUC_lexicon_tags_0 },
    { 0x35935ab5, SUC_lexicon_tags_17 },
    { 0x8935ab4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda135abb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf2935abd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d535ac1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef35ac4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7735ad3, SUC_lexicon_tags_1 },
    { 0x8c135ad4, SUC_lexicon_tags_0 },
    { 0x81335ad4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c335ad8, SUC_lexicon_tags_25 },
    { 0x1df35ad8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ef35adb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b335adf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8735ae1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b35ae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86335ae7, SUC_lexicon_tags_17 },
    { 0x1335ae8, SUC_lexicon_tags_27 },
    { 0xe6b35ae9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b35aef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dd35af1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd535af3, SUC_lexicon_tags_47 },
    { 0x4135af4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x80b35af6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39735af8, SUC_lexicon_tags_12 },
    { 0x72135af9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95935b06, SUC_lexicon_tags_11 },
    { 0xd1735b06, SUC_lexicon_tags_2 },
    { 0xdb35b08, SUC_lexicon_tags_25 },
    { 0x97335b09, SUC_lexicon_tags_25 },
    { 0xf7f35b09, SUC_lexicon_tags_29 },
    { 0x79335b0b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f35b14, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0935b18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd35b1c, SUC_lexicon_tags_45 },
    { 0xd2535b1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34735b1f, SUC_lexicon_tags_37 },
    { 0x1e735b20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f935b22, SUC_lexicon_tags_10 },
    { 0xa0135b23, SUC_lexicon_tags_0 },
    { 0x90b35b24, SUC_lexicon_tags_33 },
    { 0x93f35b25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5335b2a, SUC_lexicon_tags_26 },
    { 0x3f35b2a, SUC_lexicon_tags_15 },
    { 0xa3335b2c, SUC_lexicon_tags_23 },
    { 0x3cd35b2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75335b33, SUC_lexicon_tags_39 },
    { 0x6ad35b33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d35b38, SUC_lexicon_tags_25 },
    { 0xa1f35b38, SUC_lexicon_tags_2 },
    { 0x95335b39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb35b3d, SUC_lexicon_tags_33 },
    { 0x18b35b3e, SUC_lexicon_tags_1 },
    { 0xaed35b3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x10535b41, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16735b47, SUC_lexicon_tags_46 },
    { 0x47135b48, SUC_lexicon_tags_33 },
    { 0x79f35b49, SUC_lexicon_tags_57 },
    { 0xe9735b47, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90135b4c, SUC_lexicon_tags_25 },
    { 0x8b735b4c, SUC_lexicon_tags_10 },
    { 0xa335b4e, SUC_lexicon_tags_54 },
    { 0xbc335b4c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe135b56, SUC_lexicon_tags_43 },
    { 0xabd35b57, SUC_lexicon_tags_8 },
    { 0xd1f35b58, SUC_lexicon_tags_16 },
    { 0x4db35b59, SUC_lexicon_tags_2 },
    { 0x82d35b59, SUC_lexicon_tags_14 },
    { 0xe2935b5a, SUC_lexicon_tags_61 },
    { 0x66135b5c, SUC_lexicon_tags_65 },
    { 0x2d135b5b, SUC_lexicon_tags_15 },
    { 0x16135b5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b35b61, SUC_lexicon_tags_37 },
    { 0xd2d35b61, SUC_lexicon_tags_37 },
    { 0x69f35b63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99535b6a, SUC_lexicon_tags_6 },
    { 0x7ab35b6a, SUC_lexicon_tags_1 },
    { 0x13935b6c, SUC_lexicon_tags_65 },
    { 0x8ff35b6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7135b70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6735b73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f35b76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41b35b78, SUC_lexicon_tags_23 },
    { 0x54735b79, SUC_lexicon_tags_38 },
    { 0xf1335b7a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24935b7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2135b80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b735b88, SUC_lexicon_tags_16 },
    { 0xd9f35b89, SUC_lexicon_tags_26 },
    { 0x6fd35b89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d35b8d, SUC_lexicon_tags_15 },
    { 0x37735b8e, SUC_lexicon_tags_37 },
    { 0xbcd35b8f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2f335b91, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5935b95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdab35b97, SUC_lexicon_tags_252 },
    { 0x51f35b98, SUC_lexicon_tags_12 },
    { 0x5d535b98, SUC_lexicon_tags_16 },
    { 0x75335b98, SUC_lexicon_tags_8 },
    { 0x5e735b9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36135ba1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xabd35ba3, SUC_lexicon_tags_1 },
    { 0x935ba4, SUC_lexicon_tags_15 },
    { 0x9f335ba5, SUC_lexicon_tags_37 },
    { 0x52d35ba6, SUC_lexicon_tags_12 },
    { 0x40535ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed735ba9, SUC_lexicon_tags_15 },
    { 0x1c535baa, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b35bb1, SUC_lexicon_tags_33 },
    { 0x14d35bb2, SUC_lexicon_tags_43 },
    { 0xee135bb2, SUC_lexicon_tags_62 },
    { 0x15735bb4, SUC_lexicon_tags_17 },
    { 0xeb735bb5, SUC_lexicon_tags_3 },
    { 0x81935bb6, SUC_lexicon_tags_4 },
    { 0x5a335bb7, SUC_lexicon_tags_36 },
    { 0xfd935bb6, SUC_lexicon_tags_66 },
    { 0x49935bb4, SUC_lexicon_tags_79 },
    { 0x3f135bba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d35bc2, SUC_lexicon_tags_25 },
    { 0xaa535bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef35bc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35135bcd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x58135bcf, SUC_lexicon_tags_13 },
    { 0x68d35bd0, SUC_lexicon_tags_45 },
    { 0x3b135bd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e935bd3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c735bdd, SUC_lexicon_tags_11 },
    { 0x68b35bde, SUC_lexicon_tags_37 },
    { 0xd0d35bdf, SUC_lexicon_tags_37 },
    { 0xe935be0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f35be3, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3735be9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf935bec, SUC_lexicon_tags_29 },
    { 0xba335bed, SUC_lexicon_tags_1 },
    { 0xab935bee, SUC_lexicon_tags_45 },
    { 0x7d535bee, SUC_lexicon_tags_16 },
    { 0x32d35bed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb35bf4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed935bfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff35bfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82335c00, SUC_lexicon_tags_17 },
    { 0xc4735c01, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x49735c03, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xec535c06, SUC_lexicon_tags_68 },
    { 0x9c335c07, SUC_lexicon_tags_2 },
    { 0xc2535c08, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x20935c0b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d35c0f, SUC_lexicon_tags_14 },
    { 0x7c735c10, SUC_lexicon_tags_14 },
    { 0x1e135c0f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a935c14, SUC_lexicon_tags_27 },
    { 0x88f35c14, SUC_lexicon_tags_11 },
    { 0x8af35c16, SUC_lexicon_tags_28 },
    { 0xa1135c15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55335c19, SUC_lexicon_tags_19 },
    { 0x9335c1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b335c1f, SUC_lexicon_tags_13 },
    { 0x6c935c20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6135c22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a135c26, SUC_lexicon_tags_11 },
    { 0xd535c27, SUC_lexicon_tags_9 },
    { 0x7a935c28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b35c2b, SUC_lexicon_tags_62 },
    { 0x2b335c2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b35c2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66735c33, SUC_lexicon_tags_15 },
    { 0x82d35c34, SUC_lexicon_tags_43 },
    { 0x75935c35, SUC_lexicon_tags_1 },
    { 0xef935c36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61935c3f, SUC_lexicon_tags_39 },
    { 0x86f35c40, SUC_lexicon_tags_12 },
    { 0xa0735c40, SUC_lexicon_tags_17 },
    { 0x9b135c40, SUC_lexicon_tags_25 },
    { 0xb735c3f, SUC_lexicon_tags_23 },
    { 0x40135c44, SUC_lexicon_tags_2 },
    { 0x71b35c45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd35c49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b135c4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d35c4f, SUC_lexicon_tags_4 },
    { 0x6a535c50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15535c55, SUC_lexicon_tags_11 },
    { 0xc8535c56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30335c5e, SUC_lexicon_tags_37 },
    { 0x23535c5e, SUC_lexicon_tags_12 },
    { 0xdf535c5f, SUC_lexicon_tags_79 },
    { 0x55135c5e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab35c64, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f35c69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb4535c6b, SUC_lexicon_tags_8 },
    { 0xf8935c6b, SUC_lexicon_tags_17 },
    { 0x8e535c6c, SUC_lexicon_tags_12 },
    { 0xea535c6e, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef35c74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9535c78, SUC_lexicon_tags_15 },
    { 0x4135c79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x74d35c7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8335c83, SUC_lexicon_tags_56 },
    { 0xc6935c84, SUC_lexicon_tags_2 },
    { 0x58535c84, SUC_lexicon_tags_4 },
    { 0x58135c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69735c8e, SUC_lexicon_tags_11 },
    { 0x3fd35c8e, SUC_lexicon_tags_25 },
    { 0x49535c90, SUC_lexicon_tags_11 },
    { 0xc3b35c8f, SUC_lexicon_tags_7 },
    { 0x2c935c92, SUC_lexicon_tags_55 },
    { 0xded35c92, SUC_lexicon_tags_27 },
    { 0xc9b35c94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d35c97, SUC_lexicon_tags_31 },
    { 0xf5d35c98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d35c9d, SUC_lexicon_tags_17 },
    { 0x62335c9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2f35ca0, SUC_lexicon_tags_29 },
    { 0x67335ca0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8fb35ca3, SUC_lexicon_tags_43 },
    { 0x1a335ca4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11935ca7, SUC_lexicon_tags_1 },
    { 0x34335ca8, SUC_lexicon_tags_10 },
    { 0x7ed35ca9, SUC_lexicon_tags_15 },
    { 0x6c335ca8, SUC_lexicon_tags_0 },
    { 0x64735cab, SUC_lexicon_tags_17 },
    { 0x39535cab, SUC_lexicon_tags_1 },
    { 0x35335cad, SUC_lexicon_tags_25 },
    { 0x30b35cab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df35cb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53935cb9, SUC_lexicon_tags_33 },
    { 0xf6d35cba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24535cbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f535cc0, SUC_lexicon_tags_32 },
    { 0xdd535cc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd35cc6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f35cc9, SUC_lexicon_tags_49 },
    { 0xb0335cca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d35cd6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf9535cd8, SUC_lexicon_tags_39 },
    { 0xc4535cd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12535cdd, SUC_lexicon_tags_87 },
    { 0x3535cdd, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41535cea, SUC_lexicon_tags_7 },
    { 0x86135ceb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56d35ced, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77735cf1, SUC_lexicon_tags_25 },
    { 0x85d35cf2, SUC_lexicon_tags_37 },
    { 0x26d35cf3, SUC_lexicon_tags_2 },
    { 0xeb35cf4, SUC_lexicon_tags_21 },
    { 0x6a735cf5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6735cf8, SUC_lexicon_tags_15 },
    { 0x1b35cf9, SUC_lexicon_tags_12 },
    { 0x2f735cfa, SUC_lexicon_tags_6 },
    { 0xc7f35cf8, SUC_lexicon_tags_60 },
    { 0x21135cfc, SUC_lexicon_tags_8 },
    { 0x60335cfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff35d00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d535d04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d35d09, SUC_lexicon_tags_35 },
    { 0xed735d0a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb35d0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb35d10, SUC_lexicon_tags_124 },
    { 0x19b35d11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63735d14, SUC_lexicon_tags_1 },
    { 0x8ad35d14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f735d19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5f35d1b, SUC_lexicon_tags_2 },
    { 0x2cf35d1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad735d26, SUC_lexicon_tags_12 },
    { 0x29935d26, SUC_lexicon_tags_55 },
    { 0x33935d28, SUC_lexicon_tags_23 },
    { 0x6fd35d28, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x69535d2b, SUC_lexicon_tags_37 },
    { 0xb3735d2c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf935d32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46935d35, SUC_lexicon_tags_15 },
    { 0xe8735d36, SUC_lexicon_tags_17 },
    { 0xd6335d35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90935d3a, SUC_lexicon_tags_13 },
    { 0x11335d3a, SUC_lexicon_tags_11 },
    { 0x8bb35d3b, SUC_lexicon_tags_7 },
    { 0x3c535d3c, SUC_lexicon_tags_12 },
    { 0x3bf35d3e, SUC_lexicon_tags_11 },
    { 0x10935d3e, SUC_lexicon_tags_17 },
    { 0xfa135d40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f35d43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12b35d45, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x32935d47, SUC_lexicon_tags_2 },
    { 0x46535d48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb8b35d4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a935d4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc735d4f, SUC_lexicon_tags_12 },
    { 0xa0d35d4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a135d5e, SUC_lexicon_tags_31 },
    { 0x2b335d5f, SUC_lexicon_tags_48 },
    { 0xfbd35d5e, SUC_lexicon_tags_17 },
    { 0xff935d61, SUC_lexicon_tags_43 },
    { 0x46f35d61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f35d6b, SUC_lexicon_tags_26 },
    { 0x8e935d6b, SUC_lexicon_tags_37 },
    { 0xd7335d6d, SUC_lexicon_tags_207 },
    { 0x7f735d6d, SUC_lexicon_tags_11 },
    { 0xc0f35d6f, SUC_lexicon_tags_11 },
    { 0xfcb35d70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4935d73, SUC_lexicon_tags_34 },
    { 0x2fd35d74, SUC_lexicon_tags_47 },
    { 0xfd535d74, SUC_lexicon_tags_16 },
    { 0xe7135d75, SUC_lexicon_tags_38 },
    { 0xb3135d76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3d135d79, SUC_lexicon_tags_1 },
    { 0x12935d7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0335d83, SUC_lexicon_tags_7 },
    { 0x7e935d83, SUC_lexicon_tags_14 },
    { 0xe2535d85, SUC_lexicon_tags_1 },
    { 0x8bb35d85, SUC_lexicon_tags_23 },
    { 0xced35d87, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3135d8a, SUC_lexicon_tags_27 },
    { 0x8e135d8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90335d8d, SUC_lexicon_tags_49 },
    { 0x28135d8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0d35d90, SUC_lexicon_tags_1 },
    { 0x25335d91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7d35d93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43135d96, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52935d98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d935d9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f35da2, SUC_lexicon_tags_37 },
    { 0xf5735da3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8335da5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd335da7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f935daf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab335db2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8335db5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xad735db7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe535dba, SUC_lexicon_tags_23 },
    { 0xad935dba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54935dbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d735dc0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x94335dc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7735dc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3735dcb, SUC_lexicon_tags_19 },
    { 0x34d35dcb, SUC_lexicon_tags_33 },
    { 0x6d35dcd, SUC_lexicon_tags_17 },
    { 0x77b35dce, SUC_lexicon_tags_15 },
    { 0x735dcf, SUC_lexicon_tags_37 },
    { 0xdd535dd0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x98135dd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f35dd7, SUC_lexicon_tags_25 },
    { 0x23d35dd8, SUC_lexicon_tags_4 },
    { 0x2d535dd9, SUC_lexicon_tags_8 },
    { 0xa1d35dda, SUC_lexicon_tags_17 },
    { 0xccd35ddb, SUC_lexicon_tags_87 },
    { 0xf8735ddb, SUC_lexicon_tags_2 },
    { 0x8f35dd7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa3335ddf, SUC_lexicon_tags_216 },
    { 0x2f35ddf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f735de6, SUC_lexicon_tags_37 },
    { 0x90535de6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29935dec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb8135dee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7335df1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65135df6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54735dfe, SUC_lexicon_tags_25 },
    { 0xbe935dff, SUC_lexicon_tags_63 },
    { 0x42535dfe, SUC_lexicon_tags_17 },
    { 0xdb335e01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf8f35e03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b35e07, SUC_lexicon_tags_37 },
    { 0x7735e08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e935e0b, SUC_lexicon_tags_37 },
    { 0x77935e0b, SUC_lexicon_tags_15 },
    { 0x5b135e0d, SUC_lexicon_tags_10 },
    { 0x60b35e0b, SUC_lexicon_tags_19 },
    { 0x4935e0f, SUC_lexicon_tags_15 },
    { 0x48b35e0b, SUC_lexicon_tags_46 },
    { 0x10935e11, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x99735e13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c735e1a, SUC_lexicon_tags_17 },
    { 0x50b35e1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3335e1d, SUC_lexicon_tags_14 },
    { 0x83335e1e, SUC_lexicon_tags_87 },
    { 0x37135e1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e135e25, SUC_lexicon_tags_3 },
    { 0x24b35e25, SUC_lexicon_tags_11 },
    { 0x2d935e27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6335e29, SUC_lexicon_tags_11 },
    { 0x87335e2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d335e2c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53935e30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x33735e32, SUC_lexicon_tags_37 },
    { 0x56d35e33, SUC_lexicon_tags_12 },
    { 0xe9b35e32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b35e39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69535e3d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb35e40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d35e53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc735e55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe4735e57, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1135e5c, SUC_lexicon_tags_38 },
    { 0xfc135e5d, SUC_lexicon_tags_1 },
    { 0x47135e5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2935e6a, SUC_lexicon_tags_65 },
    { 0x30b35e6b, SUC_lexicon_tags_0 },
    { 0xae935e6a, SUC_lexicon_tags_17 },
    { 0x15735e6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed135e6f, SUC_lexicon_tags_26 },
    { 0x84335e70, SUC_lexicon_tags_47 },
    { 0xdef35e71, SUC_lexicon_tags_8 },
    { 0x56535e71, SUC_lexicon_tags_1435 },
    { 0, NULL },
    { 0, NULL },
    { 0x93335e75, SUC_lexicon_tags_60 },
    { 0xf3535e76, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b35e79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x98935e7b, SUC_lexicon_tags_2 },
    { 0xa9b35e7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x65f35e7e, SUC_lexicon_tags_61 },
    { 0x90d35e7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf9935e81, SUC_lexicon_tags_15 },
    { 0x73b35e81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16535e87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89b35e89, SUC_lexicon_tags_8 },
    { 0x39735e8a, SUC_lexicon_tags_9 },
    { 0x51735e89, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48535e96, SUC_lexicon_tags_47 },
    { 0xc8d35e97, SUC_lexicon_tags_27 },
    { 0x79f35e96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d35e9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55535ea0, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce135ea4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4135ea8, SUC_lexicon_tags_1 },
    { 0xd6d35ea8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee335ead, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54935eaf, SUC_lexicon_tags_45 },
    { 0xf1d35eb0, SUC_lexicon_tags_15 },
    { 0x8cf35eb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce735eb4, SUC_lexicon_tags_37 },
    { 0x49f35eb5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f735ebd, SUC_lexicon_tags_2 },
    { 0x6bf35ebd, SUC_lexicon_tags_17 },
    { 0x2e335ebf, SUC_lexicon_tags_6 },
    { 0xb1735ec0, SUC_lexicon_tags_25 },
    { 0x62b35ec1, SUC_lexicon_tags_40 },
    { 0x5cd35ec2, SUC_lexicon_tags_47 },
    { 0xc935ec0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf35ec5, SUC_lexicon_tags_37 },
    { 0xa935ec6, SUC_lexicon_tags_15 },
    { 0x4d535ec7, SUC_lexicon_tags_152 },
    { 0x7e335ec8, SUC_lexicon_tags_60 },
    { 0x3cd35ec6, SUC_lexicon_tags_25 },
    { 0xed935eca, SUC_lexicon_tags_7 },
    { 0x67d35eca, SUC_lexicon_tags_31 },
    { 0x5eb35ec8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d35ed2, SUC_lexicon_tags_18 },
    { 0x4535ed3, SUC_lexicon_tags_17 },
    { 0xc335ed3, SUC_lexicon_tags_16 },
    { 0x3a335ed5, SUC_lexicon_tags_66 },
    { 0xba535ed3, SUC_lexicon_tags_37 },
    { 0x2a535ed7, SUC_lexicon_tags_33 },
    { 0xf7b35ed8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xab135eda, SUC_lexicon_tags_7 },
    { 0xf2935eda, SUC_lexicon_tags_15 },
    { 0xfa735edc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23935ee0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1735ee3, SUC_lexicon_tags_37 },
    { 0xaf935ee4, SUC_lexicon_tags_256 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d35ee7, SUC_lexicon_tags_1 },
    { 0x3e935ee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea535ef2, SUC_lexicon_tags_12 },
    { 0x10935ef3, SUC_lexicon_tags_17 },
    { 0x67b35ef4, SUC_lexicon_tags_4 },
    { 0x42f35ef4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa335ef9, SUC_lexicon_tags_9 },
    { 0xec535ef9, SUC_lexicon_tags_17 },
    { 0xae935efb, SUC_lexicon_tags_47 },
    { 0x6d735efa, SUC_lexicon_tags_57 },
    { 0xef135efd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e935f01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60535f05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6f35f07, SUC_lexicon_tags_63 },
    { 0xb2d35f08, SUC_lexicon_tags_45 },
    { 0x9fd35f09, SUC_lexicon_tags_21 },
    { 0x7e735f0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d35f0d, SUC_lexicon_tags_77 },
    { 0x74735f0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79135f13, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d35f18, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f335f1d, SUC_lexicon_tags_45 },
    { 0xe7535f1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4b35f20, SUC_lexicon_tags_1 },
    { 0xe135f21, SUC_lexicon_tags_19 },
    { 0x82d35f22, SUC_lexicon_tags_2 },
    { 0xccb35f22, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3135f26, SUC_lexicon_tags_2 },
    { 0x2f535f27, SUC_lexicon_tags_12 },
    { 0x36d35f28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5bb35f2a, SUC_lexicon_tags_191 },
    { 0xe2935f2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6d35f2d, SUC_lexicon_tags_7 },
    { 0xa7935f2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf535f31, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a935f35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3135f3a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcbb35f3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a535f3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f35f42, SUC_lexicon_tags_0 },
    { 0x5a935f43, SUC_lexicon_tags_7 },
    { 0x1a135f44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfd35f46, SUC_lexicon_tags_6 },
    { 0xef35f46, SUC_lexicon_tags_38 },
    { 0xaff35f48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb535f4b, SUC_lexicon_tags_25 },
    { 0x17f35f4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e335f4f, SUC_lexicon_tags_19 },
    { 0x87935f4f, SUC_lexicon_tags_1 },
    { 0x78535f51, SUC_lexicon_tags_47 },
    { 0x83735f50, SUC_lexicon_tags_28 },
    { 0x4e535f53, SUC_lexicon_tags_38 },
    { 0xf6735f54, SUC_lexicon_tags_37 },
    { 0x88b35f54, SUC_lexicon_tags_27 },
    { 0x80935f55, SUC_lexicon_tags_26 },
    { 0xeb535f57, SUC_lexicon_tags_53 },
    { 0xab335f4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c735f5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x135f61, SUC_lexicon_tags_6 },
    { 0x7ad35f62, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3af35f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c935f69, SUC_lexicon_tags_60 },
    { 0x5e135f6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d735f70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d735f77, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc135f7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25535f7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb935f81, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbeb35f83, SUC_lexicon_tags_27 },
    { 0xcc135f83, SUC_lexicon_tags_32 },
    { 0xdd535f84, SUC_lexicon_tags_37 },
    { 0x4dd35f86, SUC_lexicon_tags_11 },
    { 0x75935f87, SUC_lexicon_tags_63 },
    { 0xd4d35f85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92735f8f, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0x67935f91, SUC_lexicon_tags_25 },
    { 0xfe935f91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42735f98, SUC_lexicon_tags_37 },
    { 0x94b35f99, SUC_lexicon_tags_27 },
    { 0x68935f98, SUC_lexicon_tags_9 },
    { 0x9e135f9b, SUC_lexicon_tags_7 },
    { 0x4935f99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6735fa0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3935fa4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x91735fa6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc0535fa8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5b935faa, SUC_lexicon_tags_12 },
    { 0x62735fab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f35fb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x53535fb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb935fba, SUC_lexicon_tags_214 },
    { 0xb9735fba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b35fbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f535fc2, SUC_lexicon_tags_37 },
    { 0xed935fc3, SUC_lexicon_tags_14 },
    { 0x8f935fc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3135fc6, SUC_lexicon_tags_6 },
    { 0xa6335fc7, SUC_lexicon_tags_30 },
    { 0x2b535fc8, SUC_lexicon_tags_11 },
    { 0xa7535fc9, SUC_lexicon_tags_46 },
    { 0x83f35fca, SUC_lexicon_tags_23 },
    { 0x2dd35fc9, SUC_lexicon_tags_11 },
    { 0x3dd35fc9, SUC_lexicon_tags_49 },
    { 0xd7b35fcd, SUC_lexicon_tags_25 },
    { 0x34135fcd, SUC_lexicon_tags_40 },
    { 0xdb935fcb, SUC_lexicon_tags_25 },
    { 0xb7335fce, SUC_lexicon_tags_22 },
    { 0xae535fcd, SUC_lexicon_tags_0 },
    { 0x46135fd2, SUC_lexicon_tags_47 },
    { 0x45535fd3, SUC_lexicon_tags_27 },
    { 0xe1535fd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f35fd9, SUC_lexicon_tags_63 },
    { 0x52735fda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b35fdd, SUC_lexicon_tags_23 },
    { 0x18135fde, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d35fe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70d35fe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7335fe7, SUC_lexicon_tags_49 },
    { 0x5f535fe8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94935fec, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5935fef, SUC_lexicon_tags_23 },
    { 0xbcb35fef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f35ff4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db35ff7, SUC_lexicon_tags_46 },
    { 0xdef35ff8, SUC_lexicon_tags_27 },
    { 0xed735ff7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd335ffc, SUC_lexicon_tags_9 },
    { 0x61535ffc, SUC_lexicon_tags_423 },
    { 0, NULL },
    { 0xe1f35fff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88536003, SUC_lexicon_tags_17 },
    { 0xbf736004, SUC_lexicon_tags_25 },
    { 0xe1336005, SUC_lexicon_tags_17 },
    { 0x28736004, SUC_lexicon_tags_112 },
    { 0x1a736007, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8336011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20336017, SUC_lexicon_tags_37 },
    { 0x70f36017, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b36021, SUC_lexicon_tags_46 },
    { 0xd36022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b336025, SUC_lexicon_tags_118 },
    { 0xa5736025, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b36029, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad33602d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f36032, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb36038, SUC_lexicon_tags_12 },
    { 0x25b36038, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd33603d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2e53603f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x88536042, SUC_lexicon_tags_47 },
    { 0xabb36043, SUC_lexicon_tags_7 },
    { 0xa3936042, SUC_lexicon_tags_11 },
    { 0x536045, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b13604b, SUC_lexicon_tags_32 },
    { 0x7dd3604c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd493604f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9b736051, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23736053, SUC_lexicon_tags_11 },
    { 0x42b36054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46136057, SUC_lexicon_tags_37 },
    { 0xf9136058, SUC_lexicon_tags_27 },
    { 0x2e936057, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d3605d, SUC_lexicon_tags_18 },
    { 0x7fb3605e, SUC_lexicon_tags_11 },
    { 0x8b33605f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e736062, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x43736064, SUC_lexicon_tags_2 },
    { 0x6d736065, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f3606e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad36071, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6736076, SUC_lexicon_tags_26 },
    { 0xdb536076, SUC_lexicon_tags_17 },
    { 0xbc336078, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5873607f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd36083, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3e336085, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5333608c, SUC_lexicon_tags_84 },
    { 0x573608d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4736091, SUC_lexicon_tags_8 },
    { 0x3d736091, SUC_lexicon_tags_19 },
    { 0x70f36093, SUC_lexicon_tags_9 },
    { 0x17936093, SUC_lexicon_tags_17 },
    { 0xa2536094, SUC_lexicon_tags_14 },
    { 0xb6d36096, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28936099, SUC_lexicon_tags_7 },
    { 0x5213609a, SUC_lexicon_tags_33 },
    { 0x8b33609a, SUC_lexicon_tags_27 },
    { 0x7b13609c, SUC_lexicon_tags_27 },
    { 0x13d3609c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35360a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac3360a8, SUC_lexicon_tags_17 },
    { 0x639360a9, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af360ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b360b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53360b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17360b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x569360ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a5360bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x313360bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd360c4, SUC_lexicon_tags_33 },
    { 0x207360c5, SUC_lexicon_tags_45 },
    { 0x10f360c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d360cd, SUC_lexicon_tags_46 },
    { 0x1b9360ce, SUC_lexicon_tags_2 },
    { 0xbc9360cf, SUC_lexicon_tags_2 },
    { 0x157360d0, SUC_lexicon_tags_6 },
    { 0x595360cf, SUC_lexicon_tags_55 },
    { 0xd69360d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa15360d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x765360d6, SUC_lexicon_tags_17 },
    { 0x883360d7, SUC_lexicon_tags_3 },
    { 0xb37360d8, SUC_lexicon_tags_25 },
    { 0x43b360d7, SUC_lexicon_tags_25 },
    { 0x2f1360da, SUC_lexicon_tags_9 },
    { 0xc6f360db, SUC_lexicon_tags_43 },
    { 0xfa7360da, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807360e2, SUC_lexicon_tags_72 },
    { 0xb11360e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b9360e5, SUC_lexicon_tags_1 },
    { 0x35d360e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff360ea, SUC_lexicon_tags_22 },
    { 0xa33360ea, SUC_lexicon_tags_27 },
    { 0xff360ec, SUC_lexicon_tags_21 },
    { 0xb69360ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed360f0, SUC_lexicon_tags_35 },
    { 0x791360f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643360f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65360fb, SUC_lexicon_tags_3 },
    { 0xe3d360fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x479360fe, SUC_lexicon_tags_37 },
    { 0xf8d360ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72736103, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6b36105, SUC_lexicon_tags_37 },
    { 0xb5136106, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb36109, SUC_lexicon_tags_15 },
    { 0xf4d3610a, SUC_lexicon_tags_7 },
    { 0x33d3610a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2813610d, SUC_lexicon_tags_15 },
    { 0x3433610e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a336111, SUC_lexicon_tags_45 },
    { 0x94b36112, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb36116, SUC_lexicon_tags_25 },
    { 0x65f36117, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc5d36119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d93611e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61936123, SUC_lexicon_tags_11 },
    { 0xbf936124, SUC_lexicon_tags_37 },
    { 0xe3b36124, SUC_lexicon_tags_65 },
    { 0xc5336126, SUC_lexicon_tags_47 },
    { 0x7bf36126, SUC_lexicon_tags_43 },
    { 0x12136125, SUC_lexicon_tags_38 },
    { 0x3e136129, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdf73612b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa33612f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6936133, SUC_lexicon_tags_39 },
    { 0xad36134, SUC_lexicon_tags_11 },
    { 0xee736134, SUC_lexicon_tags_11 },
    { 0x91736134, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f36139, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f3613c, SUC_lexicon_tags_37 },
    { 0x6cd3613d, SUC_lexicon_tags_37 },
    { 0x9953613e, SUC_lexicon_tags_49 },
    { 0x5a73613d, SUC_lexicon_tags_15 },
    { 0xdcb3613d, SUC_lexicon_tags_11 },
    { 0x25b3613f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46336143, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x18f36145, SUC_lexicon_tags_65 },
    { 0x4d736146, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e536148, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf13614b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf273614f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85936154, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3536157, SUC_lexicon_tags_15 },
    { 0xac336157, SUC_lexicon_tags_28 },
    { 0xff536159, SUC_lexicon_tags_25 },
    { 0x3b336159, SUC_lexicon_tags_29 },
    { 0x9d13615b, SUC_lexicon_tags_27 },
    { 0xe893615c, SUC_lexicon_tags_2 },
    { 0x5e736159, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd13615f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf36163, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71536168, SUC_lexicon_tags_334 },
    { 0x28136169, SUC_lexicon_tags_19 },
    { 0x6f936169, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b36170, SUC_lexicon_tags_39 },
    { 0xfb936171, SUC_lexicon_tags_48 },
    { 0xb336171, SUC_lexicon_tags_11 },
    { 0xa3d36170, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f36177, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xbab36179, SUC_lexicon_tags_43 },
    { 0x14b3617a, SUC_lexicon_tags_15 },
    { 0x55b3617b, SUC_lexicon_tags_10 },
    { 0x7a53617c, SUC_lexicon_tags_25 },
    { 0x5e53617a, SUC_lexicon_tags_2 },
    { 0xd9d3617b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42936187, SUC_lexicon_tags_7 },
    { 0xc4936187, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1513618a, SUC_lexicon_tags_16 },
    { 0x3b3618a, SUC_lexicon_tags_63 },
    { 0xd073618c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df36190, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d3619c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x1493619f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787361a4, SUC_lexicon_tags_37 },
    { 0x23b361a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525361ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf1361ae, SUC_lexicon_tags_0 },
    { 0x2ad361af, SUC_lexicon_tags_25 },
    { 0x701361af, SUC_lexicon_tags_6 },
    { 0xe5361b1, SUC_lexicon_tags_10 },
    { 0x6db361b2, SUC_lexicon_tags_37 },
    { 0xfb7361b2, SUC_lexicon_tags_87 },
    { 0x6fb361b2, SUC_lexicon_tags_14 },
    { 0x57b361b5, SUC_lexicon_tags_7 },
    { 0xe53361b5, SUC_lexicon_tags_16 },
    { 0x343361ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb361bd, SUC_lexicon_tags_11 },
    { 0x579361bd, SUC_lexicon_tags_476 },
    { 0, NULL },
    { 0, NULL },
    { 0x867361c1, SUC_lexicon_tags_13 },
    { 0xc77361c2, SUC_lexicon_tags_15 },
    { 0x6af361c2, SUC_lexicon_tags_1 },
    { 0x9c7361c3, SUC_lexicon_tags_87 },
    { 0xead361c4, SUC_lexicon_tags_11 },
    { 0x96f361c6, SUC_lexicon_tags_27 },
    { 0x103361c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e7361c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x103361cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7361d1, SUC_lexicon_tags_27 },
    { 0x673361d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb1361d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5361d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43361db, SUC_lexicon_tags_91 },
    { 0x7d1361dc, SUC_lexicon_tags_12 },
    { 0x9ff361dd, SUC_lexicon_tags_15 },
    { 0x511361dc, SUC_lexicon_tags_0 },
    { 0x699361db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc91361e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x981361e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99361e8, SUC_lexicon_tags_0 },
    { 0x773361e8, SUC_lexicon_tags_1 },
    { 0x3b1361e9, SUC_lexicon_tags_15 },
    { 0x979361e9, SUC_lexicon_tags_19 },
    { 0xb8d361ec, SUC_lexicon_tags_13 },
    { 0xba7361e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a9361ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a7361f1, SUC_lexicon_tags_12 },
    { 0xf89361f1, SUC_lexicon_tags_15 },
    { 0x9ef361f3, SUC_lexicon_tags_25 },
    { 0x757361f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x353361f6, SUC_lexicon_tags_37 },
    { 0x13d361f7, SUC_lexicon_tags_2 },
    { 0xb1b361f7, SUC_lexicon_tags_33 },
    { 0xc13361f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d361ff, SUC_lexicon_tags_15 },
    { 0x5cb361ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee936202, SUC_lexicon_tags_25 },
    { 0x2db36203, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2536206, SUC_lexicon_tags_27 },
    { 0xbe536207, SUC_lexicon_tags_7 },
    { 0x6fb36208, SUC_lexicon_tags_10 },
    { 0x49136209, SUC_lexicon_tags_25 },
    { 0x2673620a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba13620d, SUC_lexicon_tags_28 },
    { 0x6893620e, SUC_lexicon_tags_15 },
    { 0xe353620f, SUC_lexicon_tags_25 },
    { 0x3d536210, SUC_lexicon_tags_17 },
    { 0x9936210, SUC_lexicon_tags_19 },
    { 0xbc536210, SUC_lexicon_tags_12 },
    { 0x69f36213, SUC_lexicon_tags_45 },
    { 0x46936211, SUC_lexicon_tags_59 },
    { 0x38336215, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f3621b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77936222, SUC_lexicon_tags_7 },
    { 0x2fd36223, SUC_lexicon_tags_0 },
    { 0x82f36223, SUC_lexicon_tags_25 },
    { 0xbc736223, SUC_lexicon_tags_29 },
    { 0xb3536225, SUC_lexicon_tags_19 },
    { 0x1af36226, SUC_lexicon_tags_49 },
    { 0x71536228, SUC_lexicon_tags_27 },
    { 0x24f36229, SUC_lexicon_tags_13 },
    { 0xac53622a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf213622c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x11f3622e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf136230, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xed36233, SUC_lexicon_tags_12 },
    { 0xd1336234, SUC_lexicon_tags_11 },
    { 0xac336235, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3913623b, SUC_lexicon_tags_25 },
    { 0x67f3623c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c73623e, SUC_lexicon_tags_49 },
    { 0xb1b3623f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x95b36241, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89936247, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6113624a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x25d3624c, SUC_lexicon_tags_37 },
    { 0xf953624d, SUC_lexicon_tags_5 },
    { 0x39b3624c, SUC_lexicon_tags_25 },
    { 0x5e33624f, SUC_lexicon_tags_15 },
    { 0xa136250, SUC_lexicon_tags_188 },
    { 0x7e336251, SUC_lexicon_tags_17 },
    { 0x18336251, SUC_lexicon_tags_7 },
    { 0x47536253, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbbf36255, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9253625d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f936261, SUC_lexicon_tags_17 },
    { 0x71736261, SUC_lexicon_tags_63 },
    { 0x88536263, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f36266, SUC_lexicon_tags_12 },
    { 0x1bb36266, SUC_lexicon_tags_11 },
    { 0xbd336266, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9393626c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f736272, SUC_lexicon_tags_7 },
    { 0xdd136273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f36276, SUC_lexicon_tags_2 },
    { 0x9c536277, SUC_lexicon_tags_17 },
    { 0x52d36277, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d33627a, SUC_lexicon_tags_48 },
    { 0xe293627a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf133627e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2536280, SUC_lexicon_tags_38 },
    { 0x1e336281, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe0f36283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf36286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a73628a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xadd3628c, SUC_lexicon_tags_33 },
    { 0xf473628d, SUC_lexicon_tags_7 },
    { 0x7c33628d, SUC_lexicon_tags_31 },
    { 0xf5d3628d, SUC_lexicon_tags_30 },
    { 0xf5336290, SUC_lexicon_tags_43 },
    { 0xa8d36290, SUC_lexicon_tags_11 },
    { 0x76f3628e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41736299, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d3629c, SUC_lexicon_tags_107 },
    { 0x333629c, SUC_lexicon_tags_215 },
    { 0x5573629c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5362a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5362a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f362a8, SUC_lexicon_tags_37 },
    { 0xfd1362a8, SUC_lexicon_tags_6 },
    { 0xa67362aa, SUC_lexicon_tags_11 },
    { 0xc2f362aa, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1362b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x201362b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x87d362b5, SUC_lexicon_tags_37 },
    { 0x87f362b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d362ba, SUC_lexicon_tags_2 },
    { 0x501362bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1ad362bd, SUC_lexicon_tags_13 },
    { 0x33d362be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d362c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521362c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3362cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599362d7, SUC_lexicon_tags_7 },
    { 0x9b1362d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae3362da, SUC_lexicon_tags_125 },
    { 0x467362db, SUC_lexicon_tags_6 },
    { 0x393362dc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x77b362de, SUC_lexicon_tags_12 },
    { 0xfc3362de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d362e2, SUC_lexicon_tags_19 },
    { 0xad9362e3, SUC_lexicon_tags_12 },
    { 0x1c1362e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9362ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd362f4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431362f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb362fb, SUC_lexicon_tags_77 },
    { 0x6a3362fc, SUC_lexicon_tags_16 },
    { 0xdfd362fd, SUC_lexicon_tags_79 },
    { 0xf19362fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7fb36300, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb36304, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2d536306, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd3630c, SUC_lexicon_tags_11 },
    { 0xb053630d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36536313, SUC_lexicon_tags_10 },
    { 0x28d36314, SUC_lexicon_tags_37 },
    { 0x20f36315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53936318, SUC_lexicon_tags_18 },
    { 0x8c936319, SUC_lexicon_tags_30 },
    { 0x50b36318, SUC_lexicon_tags_37 },
    { 0xb0d3631b, SUC_lexicon_tags_12 },
    { 0xc133631c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad36320, SUC_lexicon_tags_1 },
    { 0xdeb36321, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8c136323, SUC_lexicon_tags_9 },
    { 0x7fb36324, SUC_lexicon_tags_23 },
    { 0x4b36325, SUC_lexicon_tags_12 },
    { 0xe7336325, SUC_lexicon_tags_37 },
    { 0x28336326, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfaf36329, SUC_lexicon_tags_12 },
    { 0x2253632a, SUC_lexicon_tags_43 },
    { 0xf973632a, SUC_lexicon_tags_47 },
    { 0x6a33632c, SUC_lexicon_tags_23 },
    { 0xd293632d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc633632f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3936332, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c536334, SUC_lexicon_tags_37 },
    { 0x63736334, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f3633c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80136340, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4336344, SUC_lexicon_tags_25 },
    { 0x54b36344, SUC_lexicon_tags_60 },
    { 0xe4736346, SUC_lexicon_tags_129 },
    { 0xe7b36347, SUC_lexicon_tags_19 },
    { 0x7d536346, SUC_lexicon_tags_16 },
    { 0xe6136344, SUC_lexicon_tags_12 },
    { 0xa713634a, SUC_lexicon_tags_6 },
    { 0xecd3634a, SUC_lexicon_tags_29 },
    { 0x9373634c, SUC_lexicon_tags_12 },
    { 0x1eb3634c, SUC_lexicon_tags_49 },
    { 0xd793634b, SUC_lexicon_tags_25 },
    { 0xb893634f, SUC_lexicon_tags_79 },
    { 0xfab3634f, SUC_lexicon_tags_17 },
    { 0x993634d, SUC_lexicon_tags_45 },
    { 0xaa936352, SUC_lexicon_tags_19 },
    { 0x77136353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9db36355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf3635c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ed3635e, SUC_lexicon_tags_25 },
    { 0x6fb3635f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f36365, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f136367, SUC_lexicon_tags_29 },
    { 0x1d536367, SUC_lexicon_tags_17 },
    { 0xbef36369, SUC_lexicon_tags_63 },
    { 0xde936369, SUC_lexicon_tags_21 },
    { 0x95336367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d3636f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d36375, SUC_lexicon_tags_37 },
    { 0x47536376, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb3637d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd36380, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5136382, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9536385, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5536389, SUC_lexicon_tags_23 },
    { 0x9213638a, SUC_lexicon_tags_28 },
    { 0xe7b3638b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b93638e, SUC_lexicon_tags_62 },
    { 0x5bd3638f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff936391, SUC_lexicon_tags_9 },
    { 0x71b36392, SUC_lexicon_tags_170 },
    { 0xf1d36393, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8336398, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa7d3639a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd593639c, SUC_lexicon_tags_11 },
    { 0x4373639d, SUC_lexicon_tags_9 },
    { 0xc8b3639d, SUC_lexicon_tags_275 },
    { 0xdbd3639c, SUC_lexicon_tags_15 },
    { 0x8ab363a0, SUC_lexicon_tags_11 },
    { 0x9813639c, SUC_lexicon_tags_37 },
    { 0x661363a2, SUC_lexicon_tags_27 },
    { 0xdeb3639d, SUC_lexicon_tags_12 },
    { 0x62d363a4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b363a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x867363a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa05363ab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907363b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675363ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x57b363bc, SUC_lexicon_tags_33 },
    { 0xdb363bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x913363bf, SUC_lexicon_tags_26 },
    { 0x10f363bf, SUC_lexicon_tags_25 },
    { 0x5b5363bf, SUC_lexicon_tags_15 },
    { 0xa2d363c2, SUC_lexicon_tags_12 },
    { 0xf15363c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd1363c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b363cb, SUC_lexicon_tags_12 },
    { 0x711363cc, SUC_lexicon_tags_38 },
    { 0xc45363cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbff363cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ef363d1, SUC_lexicon_tags_30 },
    { 0xcdb363d1, SUC_lexicon_tags_12 },
    { 0xad5363d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f363d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1363da, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9363de, SUC_lexicon_tags_49 },
    { 0x6eb363df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b363e8, SUC_lexicon_tags_60 },
    { 0x357363e9, SUC_lexicon_tags_59 },
    { 0xc21363ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3363ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd5363ef, SUC_lexicon_tags_17 },
    { 0x57d363f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7363f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d363f9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39363fd, SUC_lexicon_tags_7 },
    { 0x8e7363fe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc73640b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f3640e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x97736410, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d36414, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab36418, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa053641f, SUC_lexicon_tags_82 },
    { 0x6673641f, SUC_lexicon_tags_17 },
    { 0x4533641f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5136423, SUC_lexicon_tags_61 },
    { 0xfe136424, SUC_lexicon_tags_26 },
    { 0xefb36425, SUC_lexicon_tags_7 },
    { 0x5f36423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6493642e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57136430, SUC_lexicon_tags_28 },
    { 0x69936431, SUC_lexicon_tags_54 },
    { 0x4d536432, SUC_lexicon_tags_17 },
    { 0x89136431, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97936436, SUC_lexicon_tags_23 },
    { 0x91b36437, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91336439, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab36440, SUC_lexicon_tags_11 },
    { 0xa7b36441, SUC_lexicon_tags_11 },
    { 0xa8f36442, SUC_lexicon_tags_15 },
    { 0x2bd36442, SUC_lexicon_tags_1 },
    { 0xcb336442, SUC_lexicon_tags_2 },
    { 0xd3936445, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb53644b, SUC_lexicon_tags_15 },
    { 0xed3644b, SUC_lexicon_tags_37 },
    { 0x8dd3644d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3313644f, SUC_lexicon_tags_11 },
    { 0x8ad36450, SUC_lexicon_tags_6 },
    { 0x7eb36451, SUC_lexicon_tags_11 },
    { 0x67336452, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff36459, SUC_lexicon_tags_71 },
    { 0x3136459, SUC_lexicon_tags_2 },
    { 0xae53645b, SUC_lexicon_tags_1 },
    { 0xf473645b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb193645f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2136461, SUC_lexicon_tags_15 },
    { 0xe9136462, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8336465, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d36469, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb093646b, SUC_lexicon_tags_43 },
    { 0xa4b3646c, SUC_lexicon_tags_230 },
    { 0xa53646c, SUC_lexicon_tags_1 },
    { 0x8093646e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f36471, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf36474, SUC_lexicon_tags_12 },
    { 0x6e336474, SUC_lexicon_tags_9 },
    { 0x33336476, SUC_lexicon_tags_77 },
    { 0xe9d36477, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f93647c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa536482, SUC_lexicon_tags_31 },
    { 0xd7d36483, SUC_lexicon_tags_34 },
    { 0x30736484, SUC_lexicon_tags_8 },
    { 0xe0136485, SUC_lexicon_tags_17 },
    { 0x1fd36482, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2736488, SUC_lexicon_tags_1 },
    { 0x67136488, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973648f, SUC_lexicon_tags_6 },
    { 0x2613648f, SUC_lexicon_tags_15 },
    { 0x40936490, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6136493, SUC_lexicon_tags_60 },
    { 0xfad36493, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f3649b, SUC_lexicon_tags_2 },
    { 0xb833649c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5713649e, SUC_lexicon_tags_17 },
    { 0x7733649f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fd364a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35364a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ab364a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79364ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43364b1, SUC_lexicon_tags_325 },
    { 0, NULL },
    { 0x97f364b3, SUC_lexicon_tags_49 },
    { 0x50d364b3, SUC_lexicon_tags_61 },
    { 0x8ed364b5, SUC_lexicon_tags_47 },
    { 0xef3364b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d364bf, SUC_lexicon_tags_30 },
    { 0xb6f364bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1fb364c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe81364c4, SUC_lexicon_tags_17 },
    { 0x6df364c5, SUC_lexicon_tags_17 },
    { 0x4d1364c6, SUC_lexicon_tags_31 },
    { 0x679364c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5364cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b364d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6bd364d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d364d9, SUC_lexicon_tags_43 },
    { 0x36b364da, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x881364dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9364e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3364ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3364ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5364f7, SUC_lexicon_tags_21 },
    { 0x1fd364f8, SUC_lexicon_tags_17 },
    { 0xdc5364f9, SUC_lexicon_tags_1 },
    { 0xaab364fa, SUC_lexicon_tags_4 },
    { 0xb8b364fb, SUC_lexicon_tags_19 },
    { 0x82f364fb, SUC_lexicon_tags_68 },
    { 0x33364f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcbf364ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d136501, SUC_lexicon_tags_6 },
    { 0xe5936502, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0f36504, SUC_lexicon_tags_19 },
    { 0xf7336504, SUC_lexicon_tags_26 },
    { 0x47336506, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d53650f, SUC_lexicon_tags_68 },
    { 0x40536510, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb3336512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd336517, SUC_lexicon_tags_37 },
    { 0x86d36517, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdb3651a, SUC_lexicon_tags_9 },
    { 0x9073651b, SUC_lexicon_tags_38 },
    { 0x893651c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd36520, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0736527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ef36529, SUC_lexicon_tags_12 },
    { 0x15d3652a, SUC_lexicon_tags_25 },
    { 0x16536529, SUC_lexicon_tags_18 },
    { 0x4d93652c, SUC_lexicon_tags_48 },
    { 0x3013652c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c736531, SUC_lexicon_tags_25 },
    { 0x67936531, SUC_lexicon_tags_31 },
    { 0x31536533, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3136535, SUC_lexicon_tags_8 },
    { 0xd9336535, SUC_lexicon_tags_65 },
    { 0x90d36536, SUC_lexicon_tags_55 },
    { 0x8e136536, SUC_lexicon_tags_30 },
    { 0x58f36539, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12136541, SUC_lexicon_tags_6 },
    { 0x1c336541, SUC_lexicon_tags_47 },
    { 0xc9d36543, SUC_lexicon_tags_66 },
    { 0xeeb36544, SUC_lexicon_tags_14 },
    { 0xc2336541, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23736549, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c33654f, SUC_lexicon_tags_17 },
    { 0xd1f3654f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f136553, SUC_lexicon_tags_37 },
    { 0x84936554, SUC_lexicon_tags_55 },
    { 0x38936555, SUC_lexicon_tags_37 },
    { 0x6536556, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4973655a, SUC_lexicon_tags_11 },
    { 0xa113655b, SUC_lexicon_tags_43 },
    { 0xf773655b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfc53655e, SUC_lexicon_tags_25 },
    { 0x8c93655f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13336563, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3df36565, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df36569, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea736570, SUC_lexicon_tags_12 },
    { 0x1136570, SUC_lexicon_tags_8 },
    { 0xe1736572, SUC_lexicon_tags_11 },
    { 0x30536573, SUC_lexicon_tags_11 },
    { 0x23536571, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e136578, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4833657c, SUC_lexicon_tags_37 },
    { 0xc573657d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d36583, SUC_lexicon_tags_37 },
    { 0x5c536583, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7893658f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeb736591, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7a136593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31736595, SUC_lexicon_tags_19 },
    { 0xad736595, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x55536598, SUC_lexicon_tags_37 },
    { 0xaa336598, SUC_lexicon_tags_37 },
    { 0xaf136598, SUC_lexicon_tags_37 },
    { 0xd3b36599, SUC_lexicon_tags_182 },
    { 0xc413659c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d365a0, SUC_lexicon_tags_26 },
    { 0xea5365a1, SUC_lexicon_tags_25 },
    { 0x58d365a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9365a5, SUC_lexicon_tags_47 },
    { 0x26d365a5, SUC_lexicon_tags_31 },
    { 0xbcd365a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89365ac, SUC_lexicon_tags_7 },
    { 0xfdd365ac, SUC_lexicon_tags_40 },
    { 0x183365ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5365b3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f365b9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1365bd, SUC_lexicon_tags_12 },
    { 0x2a3365be, SUC_lexicon_tags_48 },
    { 0x387365bf, SUC_lexicon_tags_47 },
    { 0x443365c0, SUC_lexicon_tags_3 },
    { 0x519365c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11365c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69b365c8, SUC_lexicon_tags_3 },
    { 0xf9f365c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b365cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe21365cf, SUC_lexicon_tags_30 },
    { 0x45d365d0, SUC_lexicon_tags_17 },
    { 0xee5365d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96f365d3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f365d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15365da, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f365e1, SUC_lexicon_tags_9 },
    { 0x6f3365e2, SUC_lexicon_tags_2 },
    { 0x1bb365e1, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67365e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201365ec, SUC_lexicon_tags_19 },
    { 0x593365ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x781365f0, SUC_lexicon_tags_11 },
    { 0x775365f0, SUC_lexicon_tags_15 },
    { 0x919365f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf23365f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801365f8, SUC_lexicon_tags_1 },
    { 0xeb7365f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b365fc, SUC_lexicon_tags_38 },
    { 0x7f9365fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec536603, SUC_lexicon_tags_0 },
    { 0xa6336604, SUC_lexicon_tags_27 },
    { 0xaff36604, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb36608, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff3660b, SUC_lexicon_tags_7 },
    { 0x6fd3660c, SUC_lexicon_tags_37 },
    { 0xb553660c, SUC_lexicon_tags_22 },
    { 0xb3d3660e, SUC_lexicon_tags_37 },
    { 0x3fb3660e, SUC_lexicon_tags_3 },
    { 0xacf36610, SUC_lexicon_tags_43 },
    { 0x45f36611, SUC_lexicon_tags_3 },
    { 0x77136612, SUC_lexicon_tags_31 },
    { 0x3213660c, SUC_lexicon_tags_7 },
    { 0x2b736614, SUC_lexicon_tags_15 },
    { 0xe4d3660e, SUC_lexicon_tags_1 },
    { 0x45136612, SUC_lexicon_tags_15 },
    { 0x29536617, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53661d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x41d3661f, SUC_lexicon_tags_1 },
    { 0x5eb3661f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd3662d, SUC_lexicon_tags_30 },
    { 0x7e53662e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12536631, SUC_lexicon_tags_19 },
    { 0x1c936632, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xedb36634, SUC_lexicon_tags_64 },
    { 0x39536635, SUC_lexicon_tags_29 },
    { 0x95336636, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x97736638, SUC_lexicon_tags_11 },
    { 0xb5b36639, SUC_lexicon_tags_48 },
    { 0x1d3663a, SUC_lexicon_tags_28 },
    { 0xc093663a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf6f3663d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x60736640, SUC_lexicon_tags_37 },
    { 0xc5f36640, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe4f36643, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd36647, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe773664a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd593664c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6413664e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa336651, SUC_lexicon_tags_12 },
    { 0xc2136652, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6936656, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd36661, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b36664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f36667, SUC_lexicon_tags_31 },
    { 0x24536667, SUC_lexicon_tags_15 },
    { 0xd736669, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1653666b, SUC_lexicon_tags_37 },
    { 0x9fd3666b, SUC_lexicon_tags_60 },
    { 0x8c73666d, SUC_lexicon_tags_47 },
    { 0x5853666e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfc136670, SUC_lexicon_tags_43 },
    { 0xeeb36671, SUC_lexicon_tags_9 },
    { 0x55f36672, SUC_lexicon_tags_27 },
    { 0x1f936673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e536675, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80736677, SUC_lexicon_tags_45 },
    { 0xb0336677, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f73667a, SUC_lexicon_tags_12 },
    { 0x2e93667b, SUC_lexicon_tags_108 },
    { 0x5d13667a, SUC_lexicon_tags_17 },
    { 0x1b73667d, SUC_lexicon_tags_39 },
    { 0x373667e, SUC_lexicon_tags_11 },
    { 0x2d3667d, SUC_lexicon_tags_39 },
    { 0x48336680, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x31736683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f36687, SUC_lexicon_tags_37 },
    { 0xb4736688, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa9d3668a, SUC_lexicon_tags_11 },
    { 0xd8d3668b, SUC_lexicon_tags_57 },
    { 0x79d3668a, SUC_lexicon_tags_1 },
    { 0xe93668d, SUC_lexicon_tags_28 },
    { 0x7533668d, SUC_lexicon_tags_17 },
    { 0x4033668d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x40736691, SUC_lexicon_tags_44 },
    { 0xb0936691, SUC_lexicon_tags_8 },
    { 0xf3f36691, SUC_lexicon_tags_11 },
    { 0xdbb36693, SUC_lexicon_tags_12 },
    { 0x4d536695, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf36698, SUC_lexicon_tags_11 },
    { 0x22d36699, SUC_lexicon_tags_7 },
    { 0xc293669a, SUC_lexicon_tags_27 },
    { 0x8a93669b, SUC_lexicon_tags_37 },
    { 0x34136698, SUC_lexicon_tags_15 },
    { 0x80f36699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x123366a0, SUC_lexicon_tags_2 },
    { 0x1b1366a1, SUC_lexicon_tags_0 },
    { 0x461366a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc4b366a4, SUC_lexicon_tags_40 },
    { 0xb67366a5, SUC_lexicon_tags_27 },
    { 0x911366a6, SUC_lexicon_tags_19 },
    { 0x1b5366a6, SUC_lexicon_tags_29 },
    { 0x28b366a7, SUC_lexicon_tags_8 },
    { 0xff9366a8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d366ad, SUC_lexicon_tags_2 },
    { 0x7c7366ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfb5366b0, SUC_lexicon_tags_37 },
    { 0x8d3366b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95366b8, SUC_lexicon_tags_17 },
    { 0x6ff366b9, SUC_lexicon_tags_28 },
    { 0x623366ba, SUC_lexicon_tags_11 },
    { 0x945366bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15366bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x407366c1, SUC_lexicon_tags_1 },
    { 0x88d366c1, SUC_lexicon_tags_11 },
    { 0xed366c3, SUC_lexicon_tags_14 },
    { 0x449366c4, SUC_lexicon_tags_10 },
    { 0x743366c5, SUC_lexicon_tags_29 },
    { 0x621366c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9366c9, SUC_lexicon_tags_15 },
    { 0xb05366ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff366cf, SUC_lexicon_tags_14 },
    { 0x1fd366cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49366d6, SUC_lexicon_tags_6 },
    { 0x5cb366d7, SUC_lexicon_tags_25 },
    { 0x895366d7, SUC_lexicon_tags_65 },
    { 0xfd1366d6, SUC_lexicon_tags_5 },
    { 0x9c7366da, SUC_lexicon_tags_10 },
    { 0x815366db, SUC_lexicon_tags_37 },
    { 0xc91366dc, SUC_lexicon_tags_15 },
    { 0xf366dd, SUC_lexicon_tags_25 },
    { 0xded366de, SUC_lexicon_tags_23 },
    { 0x213366db, SUC_lexicon_tags_2 },
    { 0xed9366e0, SUC_lexicon_tags_29 },
    { 0x805366e1, SUC_lexicon_tags_16 },
    { 0x84b366dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x749366e4, SUC_lexicon_tags_60 },
    { 0x239366e5, SUC_lexicon_tags_6 },
    { 0x21366e5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7366e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99366f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4f366f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3366fa, SUC_lexicon_tags_63 },
    { 0x57b366fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3366fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f36701, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77936709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6853670b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6936710, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26536712, SUC_lexicon_tags_37 },
    { 0x6df36712, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5573671b, SUC_lexicon_tags_55 },
    { 0x8eb3671c, SUC_lexicon_tags_37 },
    { 0xe0d3671d, SUC_lexicon_tags_7 },
    { 0x5f33671d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd36725, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f936727, SUC_lexicon_tags_47 },
    { 0x73336728, SUC_lexicon_tags_11 },
    { 0xfeb36729, SUC_lexicon_tags_1 },
    { 0x80936729, SUC_lexicon_tags_12 },
    { 0x3dd36728, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74136736, SUC_lexicon_tags_45 },
    { 0x33536737, SUC_lexicon_tags_37 },
    { 0xdfd36736, SUC_lexicon_tags_35 },
    { 0x95b36739, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe536744, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xab336747, SUC_lexicon_tags_84 },
    { 0x6d936748, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb453674f, SUC_lexicon_tags_12 },
    { 0x15f36750, SUC_lexicon_tags_7 },
    { 0xd6736751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8536758, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4233675a, SUC_lexicon_tags_82 },
    { 0x9293675b, SUC_lexicon_tags_26 },
    { 0xbf13675c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1536760, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5f336762, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99f36764, SUC_lexicon_tags_66 },
    { 0x47f36765, SUC_lexicon_tags_33 },
    { 0x46b36766, SUC_lexicon_tags_25 },
    { 0xde536767, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d36773, SUC_lexicon_tags_14 },
    { 0xf0736773, SUC_lexicon_tags_60 },
    { 0x80736773, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd36778, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa573677a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7873677c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6f3677e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a336781, SUC_lexicon_tags_308 },
    { 0x7d36782, SUC_lexicon_tags_1 },
    { 0xc1536781, SUC_lexicon_tags_25 },
    { 0x3e336783, SUC_lexicon_tags_11 },
    { 0x2cb36785, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54936787, SUC_lexicon_tags_11 },
    { 0x8bb36788, SUC_lexicon_tags_11 },
    { 0x80d36788, SUC_lexicon_tags_33 },
    { 0x22f3678a, SUC_lexicon_tags_2 },
    { 0x2ab36789, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f3678e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f36792, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57d36794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f36797, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe313679a, SUC_lexicon_tags_17 },
    { 0x8cd3679b, SUC_lexicon_tags_47 },
    { 0x5153679b, SUC_lexicon_tags_26 },
    { 0x6d93679c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bb3679f, SUC_lexicon_tags_49 },
    { 0xab367a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9ef367a2, SUC_lexicon_tags_12 },
    { 0x1af367a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d367a7, SUC_lexicon_tags_2 },
    { 0x2bd367a8, SUC_lexicon_tags_60 },
    { 0x81367a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf07367ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b367b3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xb8b367b5, SUC_lexicon_tags_31 },
    { 0x66f367b6, SUC_lexicon_tags_218 },
    { 0x685367b7, SUC_lexicon_tags_12 },
    { 0x69d367b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233367be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x907367c0, SUC_lexicon_tags_29 },
    { 0xb3367c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe63367c3, SUC_lexicon_tags_14 },
    { 0xb13367c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b367c7, SUC_lexicon_tags_61 },
    { 0x145367c8, SUC_lexicon_tags_2 },
    { 0x671367c9, SUC_lexicon_tags_37 },
    { 0x577367ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b367cd, SUC_lexicon_tags_82 },
    { 0x17367ce, SUC_lexicon_tags_3 },
    { 0x95f367cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4d367d1, SUC_lexicon_tags_1 },
    { 0x777367d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5f367d4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaf9367d6, SUC_lexicon_tags_23 },
    { 0xc79367d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5367da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x59d367dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x445367df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3d5367e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457367e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8d367e7, SUC_lexicon_tags_47 },
    { 0xe79367e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5367ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93367f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9b367f5, SUC_lexicon_tags_29 },
    { 0x3d5367f6, SUC_lexicon_tags_1 },
    { 0xa8f367f7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f36800, SUC_lexicon_tags_43 },
    { 0x34d36800, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf36804, SUC_lexicon_tags_1 },
    { 0xdad36804, SUC_lexicon_tags_12 },
    { 0x7ed36806, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f3680a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfdd3680c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1c33680e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d736812, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3493681a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f13681c, SUC_lexicon_tags_38 },
    { 0xbdb3681d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc73681f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91936824, SUC_lexicon_tags_37 },
    { 0x59536825, SUC_lexicon_tags_7 },
    { 0xf1336826, SUC_lexicon_tags_25 },
    { 0x89336827, SUC_lexicon_tags_25 },
    { 0x4e536827, SUC_lexicon_tags_26 },
    { 0xa0736828, SUC_lexicon_tags_17 },
    { 0xa0136826, SUC_lexicon_tags_60 },
    { 0xf6136824, SUC_lexicon_tags_23 },
    { 0x83d3682c, SUC_lexicon_tags_43 },
    { 0xea53682c, SUC_lexicon_tags_37 },
    { 0x6313682d, SUC_lexicon_tags_48 },
    { 0xef93682a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda736834, SUC_lexicon_tags_2 },
    { 0x9736834, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99936839, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f93683b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42b3683d, SUC_lexicon_tags_49 },
    { 0x98f3683e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x64536841, SUC_lexicon_tags_2 },
    { 0x95b36841, SUC_lexicon_tags_19 },
    { 0x9ff36843, SUC_lexicon_tags_27 },
    { 0xf4f36843, SUC_lexicon_tags_37 },
    { 0x11936843, SUC_lexicon_tags_38 },
    { 0xfb36846, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x20d36848, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0136851, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1136853, SUC_lexicon_tags_2 },
    { 0x81f36854, SUC_lexicon_tags_27 },
    { 0xe8536854, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c13685c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5373685e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20136863, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb336868, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd3686f, SUC_lexicon_tags_761 },
    { 0x95536870, SUC_lexicon_tags_17 },
    { 0x4a536871, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5336873, SUC_lexicon_tags_37 },
    { 0xc9336874, SUC_lexicon_tags_77 },
    { 0xed936873, SUC_lexicon_tags_87 },
    { 0x63536876, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf813687c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe93687e, SUC_lexicon_tags_2 },
    { 0x26f3687f, SUC_lexicon_tags_19 },
    { 0xbdd36880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55136888, SUC_lexicon_tags_6 },
    { 0xaed36889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5013688b, SUC_lexicon_tags_17 },
    { 0xaed3688b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c53688f, SUC_lexicon_tags_68 },
    { 0xf5136890, SUC_lexicon_tags_65 },
    { 0xda736890, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa136893, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b36898, SUC_lexicon_tags_46 },
    { 0x43936899, SUC_lexicon_tags_68 },
    { 0x24936898, SUC_lexicon_tags_11 },
    { 0x85b36899, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9f3689d, SUC_lexicon_tags_25 },
    { 0x7a53689e, SUC_lexicon_tags_12 },
    { 0x8853689f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x531368a1, SUC_lexicon_tags_23 },
    { 0x10d368a2, SUC_lexicon_tags_2 },
    { 0x64f368a2, SUC_lexicon_tags_17 },
    { 0xa61368a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x395368a6, SUC_lexicon_tags_1 },
    { 0x953368a7, SUC_lexicon_tags_14 },
    { 0xa25368a8, SUC_lexicon_tags_724 },
    { 0xc55368a9, SUC_lexicon_tags_47 },
    { 0xccb368a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437368b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x74b368b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x115368b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79368bd, SUC_lexicon_tags_3 },
    { 0x19b368bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b368c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571368c5, SUC_lexicon_tags_38 },
    { 0x29368c5, SUC_lexicon_tags_12 },
    { 0xb49368c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x503368c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1d368cb, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x211368ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf41368d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a7368d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7d7368d4, SUC_lexicon_tags_1 },
    { 0x87368d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7368d9, SUC_lexicon_tags_81 },
    { 0xbd9368d9, SUC_lexicon_tags_12 },
    { 0x4a9368db, SUC_lexicon_tags_77 },
    { 0x139368dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x565368df, SUC_lexicon_tags_23 },
    { 0x303368e0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa01368e2, SUC_lexicon_tags_45 },
    { 0x447368e3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xc8d368e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54b368e7, SUC_lexicon_tags_37 },
    { 0x84f368e7, SUC_lexicon_tags_13 },
    { 0x5b368e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e3368eb, SUC_lexicon_tags_37 },
    { 0xef7368ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a9368ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149368f2, SUC_lexicon_tags_7 },
    { 0x557368f3, SUC_lexicon_tags_37 },
    { 0x3b3368f4, SUC_lexicon_tags_49 },
    { 0x559368f5, SUC_lexicon_tags_16 },
    { 0x63368f5, SUC_lexicon_tags_12 },
    { 0xb37368f6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xd27368f9, SUC_lexicon_tags_5 },
    { 0x60f368fa, SUC_lexicon_tags_49 },
    { 0x2b3368fb, SUC_lexicon_tags_33 },
    { 0x7d1368fc, SUC_lexicon_tags_17 },
    { 0x1b7368fb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d36903, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77f36905, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f36908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d33690f, SUC_lexicon_tags_2 },
    { 0xd4536910, SUC_lexicon_tags_11 },
    { 0x85d36911, SUC_lexicon_tags_12 },
    { 0xe6936912, SUC_lexicon_tags_7 },
    { 0xd7b36913, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f36917, SUC_lexicon_tags_2 },
    { 0xe4b36918, SUC_lexicon_tags_11 },
    { 0xddb36919, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8dd3691b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc136923, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3336931, SUC_lexicon_tags_2 },
    { 0xd2336932, SUC_lexicon_tags_23 },
    { 0xd9f36932, SUC_lexicon_tags_82 },
    { 0xb8f36934, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x88f36936, SUC_lexicon_tags_7 },
    { 0xad136936, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd93693b, SUC_lexicon_tags_1 },
    { 0x3a73693b, SUC_lexicon_tags_56 },
    { 0x61b3693d, SUC_lexicon_tags_38 },
    { 0xebd3693e, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5936941, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12136947, SUC_lexicon_tags_45 },
    { 0x46736948, SUC_lexicon_tags_77 },
    { 0x22936949, SUC_lexicon_tags_37 },
    { 0x5df3694a, SUC_lexicon_tags_12 },
    { 0x34f3694a, SUC_lexicon_tags_11 },
    { 0x4a33694c, SUC_lexicon_tags_63 },
    { 0x15d36948, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d936951, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e936958, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd13695d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c736960, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f36964, SUC_lexicon_tags_38 },
    { 0x11536965, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb736967, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d3696a, SUC_lexicon_tags_21 },
    { 0x90f3696b, SUC_lexicon_tags_10 },
    { 0x3333696c, SUC_lexicon_tags_17 },
    { 0x34b3696c, SUC_lexicon_tags_1 },
    { 0xd713696c, SUC_lexicon_tags_17 },
    { 0x8a73696f, SUC_lexicon_tags_70 },
    { 0x38b36970, SUC_lexicon_tags_15 },
    { 0x6453696d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b93697c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f36981, SUC_lexicon_tags_11 },
    { 0x85736981, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61736984, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa3336986, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f3698b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9773698d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2b3698f, SUC_lexicon_tags_68 },
    { 0xaf73698f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d336993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91536997, SUC_lexicon_tags_47 },
    { 0xab936997, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad3699b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7d93699d, SUC_lexicon_tags_27 },
    { 0xb4d3699d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76d369a0, SUC_lexicon_tags_14 },
    { 0xead369a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb369a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f369ad, SUC_lexicon_tags_14 },
    { 0xc55369ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb369b3, SUC_lexicon_tags_27 },
    { 0xfd3369b4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe15369b6, SUC_lexicon_tags_895 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823369ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd5369bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd369bf, SUC_lexicon_tags_118 },
    { 0x8c7369bf, SUC_lexicon_tags_47 },
    { 0x3a3369c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1369d0, SUC_lexicon_tags_174 },
    { 0x161369d1, SUC_lexicon_tags_38 },
    { 0x565369d0, SUC_lexicon_tags_11 },
    { 0x6f1369d3, SUC_lexicon_tags_39 },
    { 0xe61369d4, SUC_lexicon_tags_33 },
    { 0x491369d5, SUC_lexicon_tags_1 },
    { 0x679369d6, SUC_lexicon_tags_17 },
    { 0xa47369d6, SUC_lexicon_tags_4 },
    { 0x59d369d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b369db, SUC_lexicon_tags_94 },
    { 0xfaf369dc, SUC_lexicon_tags_11 },
    { 0xc9f369dd, SUC_lexicon_tags_29 },
    { 0x34f369db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7369e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61b369e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9369e9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2e9369eb, SUC_lexicon_tags_45 },
    { 0x3d7369ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957369f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669369f5, SUC_lexicon_tags_25 },
    { 0x919369f6, SUC_lexicon_tags_0 },
    { 0x887369f5, SUC_lexicon_tags_207 },
    { 0xbbb369f5, SUC_lexicon_tags_47 },
    { 0xf81369f9, SUC_lexicon_tags_17 },
    { 0xb6f369f8, SUC_lexicon_tags_2 },
    { 0xe33369fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561369ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b36a03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b36a0c, SUC_lexicon_tags_10 },
    { 0x27936a0d, SUC_lexicon_tags_21 },
    { 0x1b336a0d, SUC_lexicon_tags_0 },
    { 0xddd36a0e, SUC_lexicon_tags_12 },
    { 0x736a10, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d36a15, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d36a20, SUC_lexicon_tags_47 },
    { 0xfcf36a20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b36a24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f36a2d, SUC_lexicon_tags_19 },
    { 0x1b936a2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef36a32, SUC_lexicon_tags_49 },
    { 0x98936a33, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe136a3b, SUC_lexicon_tags_9 },
    { 0xf0d36a3b, SUC_lexicon_tags_26 },
    { 0x14d36a3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36936a40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5936a42, SUC_lexicon_tags_7 },
    { 0xed936a42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47536a45, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b36a48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b336a4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x56536a50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34136a55, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29336a5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f136a5c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b36a62, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9a536a64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79736a67, SUC_lexicon_tags_0 },
    { 0x83936a68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb2b36a6a, SUC_lexicon_tags_15 },
    { 0xc0536a6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2136a6e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x51b36a70, SUC_lexicon_tags_4 },
    { 0x40d36a71, SUC_lexicon_tags_1 },
    { 0xc6936a72, SUC_lexicon_tags_15 },
    { 0x1bd36a72, SUC_lexicon_tags_12 },
    { 0x13136a74, SUC_lexicon_tags_47 },
    { 0xff536a75, SUC_lexicon_tags_11 },
    { 0x41136a72, SUC_lexicon_tags_7 },
    { 0x15f36a71, SUC_lexicon_tags_25 },
    { 0x5d36a78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e136a7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ed36a7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef36a80, SUC_lexicon_tags_56 },
    { 0x2936a81, SUC_lexicon_tags_12 },
    { 0xa9136a81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb1b36a84, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad36a88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65736a8c, SUC_lexicon_tags_2 },
    { 0x7fd36a8d, SUC_lexicon_tags_37 },
    { 0x3f36a8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd536a93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb3f36a95, SUC_lexicon_tags_14 },
    { 0x46936a96, SUC_lexicon_tags_30 },
    { 0x67b36a97, SUC_lexicon_tags_35 },
    { 0x9a936a98, SUC_lexicon_tags_31 },
    { 0x90936a99, SUC_lexicon_tags_47 },
    { 0x9c736a9a, SUC_lexicon_tags_40 },
    { 0x70336a98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd36aa0, SUC_lexicon_tags_11 },
    { 0x1af36aa1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c136aa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f36aaa, SUC_lexicon_tags_37 },
    { 0xf8736aab, SUC_lexicon_tags_12 },
    { 0x8ef36aaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x85336aaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af36ab3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d736ab6, SUC_lexicon_tags_60 },
    { 0xfe536ab6, SUC_lexicon_tags_25 },
    { 0xccf36ab7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d36abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf336abe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d36ac1, SUC_lexicon_tags_17 },
    { 0x6736ac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d36ac5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f36ac8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x59536acb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe536acf, SUC_lexicon_tags_12 },
    { 0x4c536ad0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae936ad5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd136ada, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41536ade, SUC_lexicon_tags_17 },
    { 0x61136adf, SUC_lexicon_tags_10 },
    { 0x4fb36ade, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77736ae3, SUC_lexicon_tags_30 },
    { 0x92136ae4, SUC_lexicon_tags_4 },
    { 0xd4936ae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1736ae7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f36aea, SUC_lexicon_tags_28 },
    { 0x5bb36aeb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc4736aed, SUC_lexicon_tags_72 },
    { 0x4b536aee, SUC_lexicon_tags_46 },
    { 0x9d936aee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf36af2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24536af8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e936afd, SUC_lexicon_tags_6 },
    { 0x59f36afe, SUC_lexicon_tags_2 },
    { 0xbfd36aff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x55b36b01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f36b04, SUC_lexicon_tags_12 },
    { 0x11736b05, SUC_lexicon_tags_17 },
    { 0xd6b36b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfef36b08, SUC_lexicon_tags_17 },
    { 0xc0936b09, SUC_lexicon_tags_49 },
    { 0x5fd36b09, SUC_lexicon_tags_68 },
    { 0x62736b0b, SUC_lexicon_tags_88 },
    { 0xd2536b0b, SUC_lexicon_tags_6 },
    { 0x1f36b0c, SUC_lexicon_tags_11 },
    { 0xd0336b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2736b12, SUC_lexicon_tags_15 },
    { 0x3f936b13, SUC_lexicon_tags_29 },
    { 0x4c936b13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af36b17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80136b19, SUC_lexicon_tags_0 },
    { 0xd336b1a, SUC_lexicon_tags_45 },
    { 0x87f36b1a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe0b36b1d, SUC_lexicon_tags_57 },
    { 0x67136b1e, SUC_lexicon_tags_15 },
    { 0xb8536b1f, SUC_lexicon_tags_25 },
    { 0xc0d36b1e, SUC_lexicon_tags_19 },
    { 0x2db36b21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4536b23, SUC_lexicon_tags_28 },
    { 0x25336b24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d36b28, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x13336b2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1336b2e, SUC_lexicon_tags_27 },
    { 0x2536b2f, SUC_lexicon_tags_19 },
    { 0x34f36b30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb336b32, SUC_lexicon_tags_31 },
    { 0xa2d36b32, SUC_lexicon_tags_7 },
    { 0x1c736b34, SUC_lexicon_tags_66 },
    { 0x9af36b33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4db36b37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x18936b3a, SUC_lexicon_tags_65 },
    { 0xa1d36b3a, SUC_lexicon_tags_37 },
    { 0x7b736b3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c936b3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x93f36b40, SUC_lexicon_tags_11 },
    { 0x98f36b41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5136b45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcd536b47, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0736b4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa6736b4d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e536b4f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48936b53, SUC_lexicon_tags_14 },
    { 0xfd736b54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f736b56, SUC_lexicon_tags_3 },
    { 0x25f36b57, SUC_lexicon_tags_38 },
    { 0xa8536b56, SUC_lexicon_tags_27 },
    { 0x83536b59, SUC_lexicon_tags_11 },
    { 0xcdf36b59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d36b5e, SUC_lexicon_tags_45 },
    { 0xabb36b5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a136b61, SUC_lexicon_tags_2 },
    { 0xd7736b61, SUC_lexicon_tags_11 },
    { 0x2d336b61, SUC_lexicon_tags_61 },
    { 0x2f936b63, SUC_lexicon_tags_29 },
    { 0x34736b65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb36b6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6936b72, SUC_lexicon_tags_34 },
    { 0xaf736b73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6936b75, SUC_lexicon_tags_12 },
    { 0xb2536b76, SUC_lexicon_tags_14 },
    { 0x9f736b77, SUC_lexicon_tags_2 },
    { 0x69f36b76, SUC_lexicon_tags_7 },
    { 0x1936b79, SUC_lexicon_tags_12 },
    { 0x91b36b79, SUC_lexicon_tags_1 },
    { 0xbe336b75, SUC_lexicon_tags_17 },
    { 0xc2b36b7c, SUC_lexicon_tags_9 },
    { 0x3ad36b7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa0936b7f, SUC_lexicon_tags_7 },
    { 0x84336b80, SUC_lexicon_tags_11 },
    { 0xf7936b7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf36b88, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d36b8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7336b8e, SUC_lexicon_tags_37 },
    { 0x65536b8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b36b98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f36b9d, SUC_lexicon_tags_7 },
    { 0xa6136b9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6536ba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc336ba5, SUC_lexicon_tags_7 },
    { 0x4bf36ba5, SUC_lexicon_tags_15 },
    { 0x4f936ba6, SUC_lexicon_tags_11 },
    { 0xe9936ba6, SUC_lexicon_tags_168 },
    { 0x95b36ba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d36bac, SUC_lexicon_tags_2 },
    { 0x4f136bac, SUC_lexicon_tags_14 },
    { 0x84336bac, SUC_lexicon_tags_7 },
    { 0xdff36bae, SUC_lexicon_tags_2 },
    { 0x39936bb0, SUC_lexicon_tags_38 },
    { 0x7f936bad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f736bb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f736bb7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x25d36bb9, SUC_lexicon_tags_19 },
    { 0x9ff36bba, SUC_lexicon_tags_38 },
    { 0x20936bba, SUC_lexicon_tags_7 },
    { 0x6f136bbc, SUC_lexicon_tags_0 },
    { 0x3e136bbd, SUC_lexicon_tags_37 },
    { 0x68736bbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd36bca, SUC_lexicon_tags_33 },
    { 0x91b36bca, SUC_lexicon_tags_37 },
    { 0xd3536bcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f36bce, SUC_lexicon_tags_11 },
    { 0xe6f36bcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ff36bd1, SUC_lexicon_tags_12 },
    { 0xe4936bd2, SUC_lexicon_tags_12 },
    { 0x4d336bd2, SUC_lexicon_tags_37 },
    { 0xf9936bd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb36bd7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb136bda, SUC_lexicon_tags_38 },
    { 0x68136bdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9d36bdd, SUC_lexicon_tags_63 },
    { 0x6bf36bde, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f36be1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b36be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8336be9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40536bee, SUC_lexicon_tags_45 },
    { 0x58336bef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f136bf1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1936bfb, SUC_lexicon_tags_37 },
    { 0xed936bfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8536c00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18336c04, SUC_lexicon_tags_59 },
    { 0x8136c05, SUC_lexicon_tags_26 },
    { 0xf9d36c06, SUC_lexicon_tags_162 },
    { 0x59936c07, SUC_lexicon_tags_60 },
    { 0x99336c05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd36c19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53736c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd336c1f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62936c25, SUC_lexicon_tags_2 },
    { 0xbcb36c26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb36c2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca136c2c, SUC_lexicon_tags_27 },
    { 0x5c936c2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x79736c2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4136c37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb336c3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3936c42, SUC_lexicon_tags_28 },
    { 0x22d36c43, SUC_lexicon_tags_23 },
    { 0xf4736c44, SUC_lexicon_tags_7 },
    { 0x42936c42, SUC_lexicon_tags_27 },
    { 0x65536c46, SUC_lexicon_tags_38 },
    { 0x10936c45, SUC_lexicon_tags_61 },
    { 0x96b36c45, SUC_lexicon_tags_19 },
    { 0x2a936c44, SUC_lexicon_tags_32 },
    { 0x4af36c43, SUC_lexicon_tags_19 },
    { 0x7f136c44, SUC_lexicon_tags_17 },
    { 0x3c736c44, SUC_lexicon_tags_17 },
    { 0xac136c4a, SUC_lexicon_tags_79 },
    { 0x70536c4e, SUC_lexicon_tags_7 },
    { 0x51736c4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28536c51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25136c5a, SUC_lexicon_tags_11 },
    { 0xb5d36c5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c936c61, SUC_lexicon_tags_14 },
    { 0xf8136c62, SUC_lexicon_tags_25 },
    { 0x6e136c63, SUC_lexicon_tags_12 },
    { 0xf4936c63, SUC_lexicon_tags_37 },
    { 0x24936c65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7f536c67, SUC_lexicon_tags_25 },
    { 0x12736c67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe736c6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8336c70, SUC_lexicon_tags_37 },
    { 0x2bb36c71, SUC_lexicon_tags_6 },
    { 0xd736c71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2536c77, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xe2736c79, SUC_lexicon_tags_68 },
    { 0x38736c7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d36c80, SUC_lexicon_tags_0 },
    { 0x7cb36c81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4736c84, SUC_lexicon_tags_46 },
    { 0xfcf36c85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55136c89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50336c8c, SUC_lexicon_tags_11 },
    { 0xe0136c8d, SUC_lexicon_tags_37 },
    { 0xa4136c8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa736c90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c136c95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52136c97, SUC_lexicon_tags_32 },
    { 0x49136c97, SUC_lexicon_tags_215 },
    { 0x4a536c99, SUC_lexicon_tags_2 },
    { 0xa3736c9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f536c9d, SUC_lexicon_tags_21 },
    { 0xff736c9e, SUC_lexicon_tags_27 },
    { 0x46136c9f, SUC_lexicon_tags_39 },
    { 0x79736ca0, SUC_lexicon_tags_25 },
    { 0xc5136ca1, SUC_lexicon_tags_7 },
    { 0x29936ca2, SUC_lexicon_tags_108 },
    { 0x53936ca1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db36ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2736ca8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c536cae, SUC_lexicon_tags_39 },
    { 0xf6136cae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4936cb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff36cbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x90936cc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcb336cc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b36cc6, SUC_lexicon_tags_45 },
    { 0x6af36cc7, SUC_lexicon_tags_43 },
    { 0xc3b36cc8, SUC_lexicon_tags_60 },
    { 0x7136cc6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89136ccf, SUC_lexicon_tags_49 },
    { 0x8f736cd0, SUC_lexicon_tags_23 },
    { 0x82936cd1, SUC_lexicon_tags_17 },
    { 0x3a536cd1, SUC_lexicon_tags_17 },
    { 0x21f36cd1, SUC_lexicon_tags_17 },
    { 0x5bb36cd3, SUC_lexicon_tags_7 },
    { 0x62936cd5, SUC_lexicon_tags_25 },
    { 0xd3136cd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6936cd9, SUC_lexicon_tags_9 },
    { 0x10736cda, SUC_lexicon_tags_23 },
    { 0xcef36cda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d536cdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37f36ce1, SUC_lexicon_tags_63 },
    { 0x73b36ce1, SUC_lexicon_tags_7 },
    { 0xcab36ce3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x15f36ce5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbb36ce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b36ceb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f336cee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f736cf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb36cf9, SUC_lexicon_tags_15 },
    { 0xb2736cfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a336cfc, SUC_lexicon_tags_17 },
    { 0xa4136cfd, SUC_lexicon_tags_25 },
    { 0x9cd36cfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2336d0b, SUC_lexicon_tags_7 },
    { 0xa6936d0c, SUC_lexicon_tags_42 },
    { 0x87936d0c, SUC_lexicon_tags_35 },
    { 0x45f36d0e, SUC_lexicon_tags_39 },
    { 0x7ff36d0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68736d11, SUC_lexicon_tags_25 },
    { 0x68536d12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21336d15, SUC_lexicon_tags_8 },
    { 0xacd36d16, SUC_lexicon_tags_72 },
    { 0x67736d17, SUC_lexicon_tags_9 },
    { 0x9d136d18, SUC_lexicon_tags_56 },
    { 0x26b36d16, SUC_lexicon_tags_26 },
    { 0xe2136d1a, SUC_lexicon_tags_11 },
    { 0x36336d1b, SUC_lexicon_tags_34 },
    { 0xc3736d1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136d25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10536d2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8536d2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc336d34, SUC_lexicon_tags_23 },
    { 0x54936d34, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x64f36d37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc936d3e, SUC_lexicon_tags_75 },
    { 0xc3736d3e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef36d45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b36d49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e536d4c, SUC_lexicon_tags_7 },
    { 0xbd536d4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4e736d4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65936d51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56536d53, SUC_lexicon_tags_77 },
    { 0x7d36d53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5336d56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f36d5b, SUC_lexicon_tags_27 },
    { 0x9b136d5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4b36d5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b36d62, SUC_lexicon_tags_15 },
    { 0x53136d63, SUC_lexicon_tags_1 },
    { 0x50f36d64, SUC_lexicon_tags_1 },
    { 0xabf36d64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f36d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf936d6e, SUC_lexicon_tags_21 },
    { 0x9db36d6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b336d75, SUC_lexicon_tags_21 },
    { 0x7e936d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9536d78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60536d7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c736d7d, SUC_lexicon_tags_11 },
    { 0x61136d7e, SUC_lexicon_tags_47 },
    { 0x84b36d7f, SUC_lexicon_tags_2 },
    { 0x45336d80, SUC_lexicon_tags_6 },
    { 0x49736d81, SUC_lexicon_tags_49 },
    { 0xd8b36d82, SUC_lexicon_tags_15 },
    { 0xcf136d83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e336d85, SUC_lexicon_tags_2 },
    { 0xc6336d86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f36d89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68536d8b, SUC_lexicon_tags_15 },
    { 0xdc936d8c, SUC_lexicon_tags_37 },
    { 0x48b36d8d, SUC_lexicon_tags_8 },
    { 0x60136d8e, SUC_lexicon_tags_11 },
    { 0x54b36d8f, SUC_lexicon_tags_25 },
    { 0x72d36d90, SUC_lexicon_tags_25 },
    { 0xfc336d91, SUC_lexicon_tags_36 },
    { 0xee136d91, SUC_lexicon_tags_37 },
    { 0xc9b36d93, SUC_lexicon_tags_12 },
    { 0x61f36d93, SUC_lexicon_tags_49 },
    { 0x84f36d95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8936d9f, SUC_lexicon_tags_1 },
    { 0x6ad36d9f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74736da6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69736db0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5736db3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d36db8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcf36dba, SUC_lexicon_tags_17 },
    { 0x90b36dbb, SUC_lexicon_tags_84 },
    { 0x25b36dbc, SUC_lexicon_tags_37 },
    { 0xff36dbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b36dc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x43136dc6, SUC_lexicon_tags_6 },
    { 0x91136dc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b36dca, SUC_lexicon_tags_28 },
    { 0x5d936dca, SUC_lexicon_tags_37 },
    { 0x4bf36dca, SUC_lexicon_tags_13 },
    { 0xe6736dcd, SUC_lexicon_tags_17 },
    { 0x96136dcb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x46136dd0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0536dd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc336ddb, SUC_lexicon_tags_57 },
    { 0xcaf36ddc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x83536ddf, SUC_lexicon_tags_37 },
    { 0x36936de0, SUC_lexicon_tags_15 },
    { 0xb5336ddf, SUC_lexicon_tags_45 },
    { 0xba136de2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4136de5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6936de9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x83d36deb, SUC_lexicon_tags_12 },
    { 0x1c736dec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5136df1, SUC_lexicon_tags_37 },
    { 0xccb36df2, SUC_lexicon_tags_12 },
    { 0xdcf36df2, SUC_lexicon_tags_63 },
    { 0xf7536df1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a736df8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3ad36dfa, SUC_lexicon_tags_40 },
    { 0x97d36dfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1336dfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d36e01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29f36e03, SUC_lexicon_tags_21 },
    { 0x8d736e04, SUC_lexicon_tags_29 },
    { 0x8f536e05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7936e08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b36e0b, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0x98336e0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2336e11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67736e15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c736e18, SUC_lexicon_tags_29 },
    { 0xfd136e18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f36e1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda536e23, SUC_lexicon_tags_17 },
    { 0xce136e24, SUC_lexicon_tags_46 },
    { 0xbc936e25, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f36e2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52136e2f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x91736e32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d36e37, SUC_lexicon_tags_37 },
    { 0x59336e38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d36e3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf936e42, SUC_lexicon_tags_38 },
    { 0xf4336e43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30136e45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3f36e47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3fd36e49, SUC_lexicon_tags_28 },
    { 0xf6736e4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d36e4e, SUC_lexicon_tags_33 },
    { 0x52736e4f, SUC_lexicon_tags_19 },
    { 0x81536e4f, SUC_lexicon_tags_17 },
    { 0x63136e4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed36e56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb2936e58, SUC_lexicon_tags_18 },
    { 0x5bd36e59, SUC_lexicon_tags_0 },
    { 0xaf336e5a, SUC_lexicon_tags_17 },
    { 0x1bf36e5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5136e5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0536e61, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4936e63, SUC_lexicon_tags_27 },
    { 0x44936e64, SUC_lexicon_tags_7 },
    { 0xdf536e64, SUC_lexicon_tags_37 },
    { 0xc7336e64, SUC_lexicon_tags_14 },
    { 0x1f36e66, SUC_lexicon_tags_11 },
    { 0xc6d36e64, SUC_lexicon_tags_10 },
    { 0xda336e69, SUC_lexicon_tags_2 },
    { 0x1df36e63, SUC_lexicon_tags_7 },
    { 0x54536e6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5336e72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f936e79, SUC_lexicon_tags_8 },
    { 0xdbb36e7a, SUC_lexicon_tags_12 },
    { 0x90336e7a, SUC_lexicon_tags_12 },
    { 0x84936e7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f736e80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a336e85, SUC_lexicon_tags_12 },
    { 0x61d36e85, SUC_lexicon_tags_25 },
    { 0x6ad36e87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94f36e89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef36e8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d36e97, SUC_lexicon_tags_45 },
    { 0xca336e98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9536ea5, SUC_lexicon_tags_4 },
    { 0xd5136ea6, SUC_lexicon_tags_0 },
    { 0x57936ea6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8536ea9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd0f36eab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd536eb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68136eb5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32736eb9, SUC_lexicon_tags_7 },
    { 0x7c536eba, SUC_lexicon_tags_43 },
    { 0xa6f36ebb, SUC_lexicon_tags_25 },
    { 0xbd736ebc, SUC_lexicon_tags_1 },
    { 0xc4936ebd, SUC_lexicon_tags_65 },
    { 0x1f536ebe, SUC_lexicon_tags_56 },
    { 0xf2536ebf, SUC_lexicon_tags_17 },
    { 0x1d336ec0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x31736ec3, SUC_lexicon_tags_12 },
    { 0xf9536ec4, SUC_lexicon_tags_343 },
    { 0x4c736ec5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a536ec9, SUC_lexicon_tags_38 },
    { 0x33d36eca, SUC_lexicon_tags_12 },
    { 0x9f136eca, SUC_lexicon_tags_62 },
    { 0x95f36ecc, SUC_lexicon_tags_15 },
    { 0x7ab36ecc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27936ed3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c536ed5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16336ed9, SUC_lexicon_tags_87 },
    { 0xafb36ed9, SUC_lexicon_tags_6 },
    { 0xe8936edb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f36ede, SUC_lexicon_tags_19 },
    { 0x46136edf, SUC_lexicon_tags_15 },
    { 0xb7336ee0, SUC_lexicon_tags_45 },
    { 0xbdf36ee0, SUC_lexicon_tags_46 },
    { 0xe0936ee2, SUC_lexicon_tags_66 },
    { 0x1a536ee1, SUC_lexicon_tags_28 },
    { 0x60936ee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4336ee6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e736eee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa936ef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16736ef7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad36f00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x44f36f02, SUC_lexicon_tags_15 },
    { 0x8d36f03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98136f08, SUC_lexicon_tags_63 },
    { 0xbf736f09, SUC_lexicon_tags_37 },
    { 0xf9936f09, SUC_lexicon_tags_43 },
    { 0x49d36f0b, SUC_lexicon_tags_42 },
    { 0x2b736f08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff36f16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x66136f18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64536f1b, SUC_lexicon_tags_25 },
    { 0xa0536f1c, SUC_lexicon_tags_61 },
    { 0xd4f36f1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e936f1f, SUC_lexicon_tags_27 },
    { 0xc3d36f1f, SUC_lexicon_tags_9 },
    { 0x4c736f21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x67536f23, SUC_lexicon_tags_0 },
    { 0x9d36f24, SUC_lexicon_tags_11 },
    { 0x59f36f25, SUC_lexicon_tags_27 },
    { 0x9f936f26, SUC_lexicon_tags_84 },
    { 0x5df36f26, SUC_lexicon_tags_30 },
    { 0x22d36f27, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x89736f2b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b36f2e, SUC_lexicon_tags_63 },
    { 0x36f36f2f, SUC_lexicon_tags_47 },
    { 0xd536f2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b36f32, SUC_lexicon_tags_13 },
    { 0xe7f36f33, SUC_lexicon_tags_29 },
    { 0xa1336f34, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd0536f36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2336f42, SUC_lexicon_tags_44 },
    { 0x2db36f43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad36f49, SUC_lexicon_tags_11 },
    { 0xa7b36f4a, SUC_lexicon_tags_15 },
    { 0x9b736f4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d36f4e, SUC_lexicon_tags_15 },
    { 0xf1f36f4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b36f52, SUC_lexicon_tags_37 },
    { 0xdd36f53, SUC_lexicon_tags_25 },
    { 0x8f136f54, SUC_lexicon_tags_27 },
    { 0xdf36f53, SUC_lexicon_tags_17 },
    { 0x23736f56, SUC_lexicon_tags_45 },
    { 0xf1536f56, SUC_lexicon_tags_82 },
    { 0xc9f36f58, SUC_lexicon_tags_1 },
    { 0xac336f59, SUC_lexicon_tags_14 },
    { 0x28f36f59, SUC_lexicon_tags_1 },
    { 0x41336f59, SUC_lexicon_tags_43 },
    { 0x43536f56, SUC_lexicon_tags_11 },
    { 0x99b36f55, SUC_lexicon_tags_15 },
    { 0xa136f5e, SUC_lexicon_tags_37 },
    { 0xdbb36f5e, SUC_lexicon_tags_37 },
    { 0x5cd36f60, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e736f6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3f36f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3d36f6e, SUC_lexicon_tags_30 },
    { 0xee736f6f, SUC_lexicon_tags_17 },
    { 0x29136f70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x26336f72, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xed136f74, SUC_lexicon_tags_33 },
    { 0x83b36f75, SUC_lexicon_tags_63 },
    { 0x71536f75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94f36f78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae936f7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a536f7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c536f8b, SUC_lexicon_tags_37 },
    { 0x51936f8b, SUC_lexicon_tags_10 },
    { 0x43d36f8c, SUC_lexicon_tags_19 },
    { 0xce536f8e, SUC_lexicon_tags_47 },
    { 0x98736f8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb36f95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9136f9b, SUC_lexicon_tags_4 },
    { 0x13136f9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db36f9f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf3f36fa1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b36fa4, SUC_lexicon_tags_1 },
    { 0x52f36fa5, SUC_lexicon_tags_38 },
    { 0x1f336fa4, SUC_lexicon_tags_7 },
    { 0xbc736fa7, SUC_lexicon_tags_0 },
    { 0x65d36fa8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdf336faa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1e336fac, SUC_lexicon_tags_0 },
    { 0xb8f36fad, SUC_lexicon_tags_16 },
    { 0x5fd36fae, SUC_lexicon_tags_14 },
    { 0x1a136fac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4936fb2, SUC_lexicon_tags_55 },
    { 0xed736fb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d36fb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81536fb8, SUC_lexicon_tags_25 },
    { 0xab36fb9, SUC_lexicon_tags_29 },
    { 0x27136fb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b36fc2, SUC_lexicon_tags_25 },
    { 0x3eb36fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b36fc6, SUC_lexicon_tags_0 },
    { 0x60536fc6, SUC_lexicon_tags_12 },
    { 0x55936fc7, SUC_lexicon_tags_47 },
    { 0xbe536fc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf36fcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd36fd1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9936fd4, SUC_lexicon_tags_27 },
    { 0x4f536fd4, SUC_lexicon_tags_23 },
    { 0x22f36fd4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f36fda, SUC_lexicon_tags_48 },
    { 0xc0136fdb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8936fe1, SUC_lexicon_tags_15 },
    { 0xe336fe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f336fe8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9936fed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd936ff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0936ff6, SUC_lexicon_tags_15 },
    { 0x23936ff6, SUC_lexicon_tags_43 },
    { 0x73336ff8, SUC_lexicon_tags_45 },
    { 0xc6936ff9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2536ffe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d37002, SUC_lexicon_tags_2 },
    { 0x79f37002, SUC_lexicon_tags_10 },
    { 0x87f37004, SUC_lexicon_tags_9 },
    { 0x6b137005, SUC_lexicon_tags_19 },
    { 0x47937006, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x74f37008, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe873700c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcc93700e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf137010, SUC_lexicon_tags_28 },
    { 0x95537010, SUC_lexicon_tags_15 },
    { 0xfcd37012, SUC_lexicon_tags_7 },
    { 0x51137013, SUC_lexicon_tags_11 },
    { 0xde737012, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d137019, SUC_lexicon_tags_17 },
    { 0x78d37019, SUC_lexicon_tags_60 },
    { 0x1373701b, SUC_lexicon_tags_24 },
    { 0xcd3701b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8833701e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16937023, SUC_lexicon_tags_37 },
    { 0x7c937023, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf37027, SUC_lexicon_tags_22 },
    { 0xfa537028, SUC_lexicon_tags_47 },
    { 0xff137029, SUC_lexicon_tags_15 },
    { 0xa6d3702a, SUC_lexicon_tags_1 },
    { 0xd93702b, SUC_lexicon_tags_21 },
    { 0xed73702a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4933702f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84137034, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed3703d, SUC_lexicon_tags_0 },
    { 0x9793703e, SUC_lexicon_tags_415 },
    { 0x1273703f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f37048, SUC_lexicon_tags_11 },
    { 0x1db37049, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2d3704b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe337051, SUC_lexicon_tags_25 },
    { 0xa3737052, SUC_lexicon_tags_15 },
    { 0x2eb37053, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f3705d, SUC_lexicon_tags_37 },
    { 0x30f3705e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd9d37060, SUC_lexicon_tags_27 },
    { 0xcbb37061, SUC_lexicon_tags_12 },
    { 0x8af37061, SUC_lexicon_tags_26 },
    { 0xe0937061, SUC_lexicon_tags_87 },
    { 0xb4737064, SUC_lexicon_tags_32 },
    { 0x6737063, SUC_lexicon_tags_56 },
    { 0xa737063, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x55737069, SUC_lexicon_tags_14 },
    { 0x51537069, SUC_lexicon_tags_12 },
    { 0xa733706b, SUC_lexicon_tags_34 },
    { 0xb7f3706b, SUC_lexicon_tags_33 },
    { 0xd693706d, SUC_lexicon_tags_14 },
    { 0x813706e, SUC_lexicon_tags_29 },
    { 0x2f53706b, SUC_lexicon_tags_66 },
    { 0x38b37070, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea137072, SUC_lexicon_tags_12 },
    { 0xd6537073, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1b537075, SUC_lexicon_tags_11 },
    { 0xdc737076, SUC_lexicon_tags_1656 },
    { 0x24d37077, SUC_lexicon_tags_2 },
    { 0xc1137078, SUC_lexicon_tags_82 },
    { 0x12937079, SUC_lexicon_tags_26 },
    { 0x7c93707a, SUC_lexicon_tags_47 },
    { 0x8413707b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1937082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4937084, SUC_lexicon_tags_17 },
    { 0xf7137085, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xa537088, SUC_lexicon_tags_61 },
    { 0xec937089, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdc33708b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x933708d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb753708f, SUC_lexicon_tags_43 },
    { 0xcaf37090, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0xd0537092, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbe937094, SUC_lexicon_tags_11 },
    { 0x7af37094, SUC_lexicon_tags_23 },
    { 0x1af37096, SUC_lexicon_tags_11 },
    { 0xdfb37097, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9eb37099, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x43b3709b, SUC_lexicon_tags_19 },
    { 0xd5d3709c, SUC_lexicon_tags_47 },
    { 0x18b3709d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1413709f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xff1370a1, SUC_lexicon_tags_44 },
    { 0xbb3370a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743370aa, SUC_lexicon_tags_63 },
    { 0xb5d370ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db370b0, SUC_lexicon_tags_25 },
    { 0xa1370b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x901370b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x155370b5, SUC_lexicon_tags_19 },
    { 0x4e7370b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3f370b8, SUC_lexicon_tags_682 },
    { 0, NULL },
    { 0xffd370ba, SUC_lexicon_tags_7 },
    { 0x2e7370ba, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f370be, SUC_lexicon_tags_17 },
    { 0xe3d370bf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b370ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab370ce, SUC_lexicon_tags_37 },
    { 0x2e5370ce, SUC_lexicon_tags_9 },
    { 0xa33370cf, SUC_lexicon_tags_37 },
    { 0x67d370cf, SUC_lexicon_tags_68 },
    { 0x859370d2, SUC_lexicon_tags_29 },
    { 0x1c3370d3, SUC_lexicon_tags_26 },
    { 0xc77370d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x155370d7, SUC_lexicon_tags_30 },
    { 0x205370d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x275370da, SUC_lexicon_tags_2 },
    { 0x3c3370da, SUC_lexicon_tags_2 },
    { 0xab1370dc, SUC_lexicon_tags_25 },
    { 0xeed370dd, SUC_lexicon_tags_47 },
    { 0x51b370de, SUC_lexicon_tags_11 },
    { 0xb8b370de, SUC_lexicon_tags_11 },
    { 0xa35370de, SUC_lexicon_tags_87 },
    { 0x3e1370dd, SUC_lexicon_tags_17 },
    { 0x153370e2, SUC_lexicon_tags_12 },
    { 0xe89370e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57370ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1370f4, SUC_lexicon_tags_0 },
    { 0xadd370f5, SUC_lexicon_tags_27 },
    { 0x6fd370f5, SUC_lexicon_tags_20 },
    { 0xdf370f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x67f370f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd370fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f37103, SUC_lexicon_tags_14 },
    { 0xe337103, SUC_lexicon_tags_37 },
    { 0x15d37105, SUC_lexicon_tags_26 },
    { 0x10937106, SUC_lexicon_tags_32 },
    { 0x75737104, SUC_lexicon_tags_1 },
    { 0x5b337108, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62537110, SUC_lexicon_tags_2 },
    { 0xaa137110, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a337117, SUC_lexicon_tags_26 },
    { 0xa1937117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e93711c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42b3711e, SUC_lexicon_tags_23 },
    { 0x2793711f, SUC_lexicon_tags_23 },
    { 0xef93711f, SUC_lexicon_tags_12 },
    { 0xb8537121, SUC_lexicon_tags_87 },
    { 0xb9f37122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c737126, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f3712d, SUC_lexicon_tags_23 },
    { 0xc273712e, SUC_lexicon_tags_17 },
    { 0xdc93712e, SUC_lexicon_tags_37 },
    { 0x1cf37130, SUC_lexicon_tags_7 },
    { 0x8cb37131, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e137134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b737137, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db3713d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d337140, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6b37142, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf937144, SUC_lexicon_tags_8 },
    { 0xdc337145, SUC_lexicon_tags_47 },
    { 0x10937146, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd113714b, SUC_lexicon_tags_80 },
    { 0xd193714c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac13714f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8537151, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfd537153, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a137156, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd3715a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x1013715c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33715f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a337161, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff37165, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7333716a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97537170, SUC_lexicon_tags_37 },
    { 0xbff37171, SUC_lexicon_tags_11 },
    { 0x4dd37172, SUC_lexicon_tags_11 },
    { 0x33737171, SUC_lexicon_tags_27 },
    { 0x88d37174, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xbf937176, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf0337178, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1433717a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c33717d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6653717f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab337184, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a737187, SUC_lexicon_tags_2 },
    { 0xdb937188, SUC_lexicon_tags_63 },
    { 0x74137187, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63f3718b, SUC_lexicon_tags_12 },
    { 0x14d3718c, SUC_lexicon_tags_37 },
    { 0x5b93718c, SUC_lexicon_tags_11 },
    { 0x6193718e, SUC_lexicon_tags_8 },
    { 0x7ef3718c, SUC_lexicon_tags_6 },
    { 0x7ab3718d, SUC_lexicon_tags_17 },
    { 0xaa33718b, SUC_lexicon_tags_17 },
    { 0x29d37190, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27537196, SUC_lexicon_tags_37 },
    { 0x5b37197, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed3719d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295371a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f371a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9371a6, SUC_lexicon_tags_39 },
    { 0x4f1371a6, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5371aa, SUC_lexicon_tags_7 },
    { 0x413371aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd371af, SUC_lexicon_tags_31 },
    { 0xd2b371b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f371b4, SUC_lexicon_tags_32 },
    { 0xcdb371b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3371b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xff3371bb, SUC_lexicon_tags_2 },
    { 0xe35371bc, SUC_lexicon_tags_1 },
    { 0xcbf371bd, SUC_lexicon_tags_37 },
    { 0x6a7371bc, SUC_lexicon_tags_31 },
    { 0x45f371bf, SUC_lexicon_tags_25 },
    { 0x98b371c0, SUC_lexicon_tags_66 },
    { 0x3e9371c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb3371c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a9371c5, SUC_lexicon_tags_11 },
    { 0x145371c5, SUC_lexicon_tags_26 },
    { 0x98f371c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd39371c9, SUC_lexicon_tags_2 },
    { 0x325371c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11371cf, SUC_lexicon_tags_49 },
    { 0x8f9371cf, SUC_lexicon_tags_8 },
    { 0x2e5371d1, SUC_lexicon_tags_14 },
    { 0xf1d371d2, SUC_lexicon_tags_1 },
    { 0x52d371d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab371da, SUC_lexicon_tags_25 },
    { 0x675371db, SUC_lexicon_tags_11 },
    { 0x71d371da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb91371de, SUC_lexicon_tags_30 },
    { 0x925371df, SUC_lexicon_tags_7 },
    { 0xfcd371de, SUC_lexicon_tags_11 },
    { 0x469371de, SUC_lexicon_tags_624 },
    { 0xe79371e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b371e6, SUC_lexicon_tags_19 },
    { 0xa2b371e7, SUC_lexicon_tags_19 },
    { 0xbd7371e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x78f371ea, SUC_lexicon_tags_23 },
    { 0xc4f371eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x211371ee, SUC_lexicon_tags_15 },
    { 0xfbf371ef, SUC_lexicon_tags_0 },
    { 0x1ed371ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb47371f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb371f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d371fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9371ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f93720b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb3720e, SUC_lexicon_tags_26 },
    { 0xf4d3720e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b37219, SUC_lexicon_tags_12 },
    { 0xfe13721a, SUC_lexicon_tags_33 },
    { 0xead37219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd073721d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff73721f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f937223, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x25f37225, SUC_lexicon_tags_25 },
    { 0x82537225, SUC_lexicon_tags_17 },
    { 0xa7537227, SUC_lexicon_tags_45 },
    { 0x18b37228, SUC_lexicon_tags_31 },
    { 0xd337229, SUC_lexicon_tags_11 },
    { 0x8993722a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b3722e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8337232, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9937234, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75537236, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf3723a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1653723e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1537240, SUC_lexicon_tags_11 },
    { 0x49537241, SUC_lexicon_tags_37 },
    { 0x1eb37241, SUC_lexicon_tags_0 },
    { 0x7fd37243, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b37246, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133724a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcb93724c, SUC_lexicon_tags_60 },
    { 0x5b53724d, SUC_lexicon_tags_47 },
    { 0xad73724d, SUC_lexicon_tags_2 },
    { 0x3513724f, SUC_lexicon_tags_2 },
    { 0x1413724f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee137256, SUC_lexicon_tags_40 },
    { 0x3fb37256, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf3725b, SUC_lexicon_tags_8 },
    { 0xced3725c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc993725e, SUC_lexicon_tags_15 },
    { 0x9d13725f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d37265, SUC_lexicon_tags_2 },
    { 0xd7d37265, SUC_lexicon_tags_17 },
    { 0x21737265, SUC_lexicon_tags_65 },
    { 0xaad37268, SUC_lexicon_tags_65 },
    { 0x86f37269, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43737270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d37273, SUC_lexicon_tags_32 },
    { 0x75d37274, SUC_lexicon_tags_2 },
    { 0xcf937274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56b37277, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d3727b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1373727d, SUC_lexicon_tags_11 },
    { 0x8c73727e, SUC_lexicon_tags_23 },
    { 0xb73727d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c337281, SUC_lexicon_tags_9 },
    { 0x25537282, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b37285, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5737287, SUC_lexicon_tags_66 },
    { 0xaf737288, SUC_lexicon_tags_12 },
    { 0x90537288, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2137293, SUC_lexicon_tags_1 },
    { 0x89137293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1f37296, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb3729c, SUC_lexicon_tags_11 },
    { 0x8613729d, SUC_lexicon_tags_25 },
    { 0x56d3729e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1372a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x877372a3, SUC_lexicon_tags_2 },
    { 0x411372a4, SUC_lexicon_tags_3 },
    { 0x91372a5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5cf372a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857372ab, SUC_lexicon_tags_124 },
    { 0x601372ab, SUC_lexicon_tags_17 },
    { 0x525372ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc01372af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3372b2, SUC_lexicon_tags_32 },
    { 0x261372b2, SUC_lexicon_tags_1 },
    { 0x3ff372b2, SUC_lexicon_tags_15 },
    { 0x295372b5, SUC_lexicon_tags_25 },
    { 0x8d5372b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755372bd, SUC_lexicon_tags_60 },
    { 0x9af372bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f372c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d372c5, SUC_lexicon_tags_37 },
    { 0x47d372c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77372cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x807372d1, SUC_lexicon_tags_421 },
    { 0xde1372d1, SUC_lexicon_tags_12 },
    { 0x405372d3, SUC_lexicon_tags_37 },
    { 0xa33372d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8a9372d6, SUC_lexicon_tags_60 },
    { 0xb53372d7, SUC_lexicon_tags_37 },
    { 0x51f372d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6d372da, SUC_lexicon_tags_17 },
    { 0x29f372db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f372e1, SUC_lexicon_tags_17 },
    { 0x125372e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b372e6, SUC_lexicon_tags_1 },
    { 0xdd3372e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89372f0, SUC_lexicon_tags_25 },
    { 0xb03372f1, SUC_lexicon_tags_60 },
    { 0xf03372f1, SUC_lexicon_tags_12 },
    { 0x5b9372f3, SUC_lexicon_tags_1 },
    { 0xcf5372f3, SUC_lexicon_tags_0 },
    { 0xfeb372f5, SUC_lexicon_tags_17 },
    { 0x6c5372f4, SUC_lexicon_tags_60 },
    { 0x567372f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73372fa, SUC_lexicon_tags_30 },
    { 0x331372fb, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef37301, SUC_lexicon_tags_12 },
    { 0x98537301, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2153730d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf37310, SUC_lexicon_tags_0 },
    { 0xf5f37310, SUC_lexicon_tags_35 },
    { 0x86b37310, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2937315, SUC_lexicon_tags_14 },
    { 0x4337316, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed3731f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x11737321, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86937326, SUC_lexicon_tags_45 },
    { 0xe2137327, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d33732a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5033732c, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x7eb3732e, SUC_lexicon_tags_40 },
    { 0x33f3732f, SUC_lexicon_tags_56 },
    { 0x5f33732f, SUC_lexicon_tags_39 },
    { 0xc137331, SUC_lexicon_tags_103 },
    { 0x20d37332, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82737336, SUC_lexicon_tags_30 },
    { 0x9b737337, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b3733e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1c737340, SUC_lexicon_tags_1 },
    { 0xe9137341, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda337349, SUC_lexicon_tags_25 },
    { 0xde33734a, SUC_lexicon_tags_37 },
    { 0xd8537349, SUC_lexicon_tags_8 },
    { 0xa493734a, SUC_lexicon_tags_1 },
    { 0x1653734d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f37351, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59337355, SUC_lexicon_tags_15 },
    { 0x2e537356, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9973735e, SUC_lexicon_tags_8 },
    { 0x7e93735f, SUC_lexicon_tags_2 },
    { 0xe1b37360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10937367, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b37369, SUC_lexicon_tags_19 },
    { 0x6f53736a, SUC_lexicon_tags_38 },
    { 0xdaf3736b, SUC_lexicon_tags_25 },
    { 0x9213736c, SUC_lexicon_tags_11 },
    { 0xecd3736c, SUC_lexicon_tags_25 },
    { 0xbe93736c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5337376, SUC_lexicon_tags_9 },
    { 0xa2137377, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1537379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53737e, SUC_lexicon_tags_37 },
    { 0x9873737f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x97537382, SUC_lexicon_tags_2 },
    { 0xa8d37383, SUC_lexicon_tags_8 },
    { 0xcc737384, SUC_lexicon_tags_43 },
    { 0xdaf37382, SUC_lexicon_tags_2 },
    { 0x4fb37386, SUC_lexicon_tags_53 },
    { 0x25737387, SUC_lexicon_tags_60 },
    { 0x5e337383, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8753738a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6293738e, SUC_lexicon_tags_2 },
    { 0x28f3738f, SUC_lexicon_tags_49 },
    { 0xe9537390, SUC_lexicon_tags_11 },
    { 0x50d37390, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d37398, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a53739a, SUC_lexicon_tags_12 },
    { 0x2c13739b, SUC_lexicon_tags_3 },
    { 0x2d53739c, SUC_lexicon_tags_46 },
    { 0x70b3739a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57373a1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7373ac, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x225373ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b373b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97373b5, SUC_lexicon_tags_47 },
    { 0xe2d373b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe35373b8, SUC_lexicon_tags_45 },
    { 0x4b1373b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb373bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5373c3, SUC_lexicon_tags_2 },
    { 0xa55373c4, SUC_lexicon_tags_12 },
    { 0xa67373c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3373c8, SUC_lexicon_tags_15 },
    { 0x133373c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf373cc, SUC_lexicon_tags_12 },
    { 0x703373cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f373d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9c3373d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9f373d6, SUC_lexicon_tags_19 },
    { 0x335373d7, SUC_lexicon_tags_17 },
    { 0x121373d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b373dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe37373de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfb3373e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f373e4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15373e7, SUC_lexicon_tags_38 },
    { 0xd41373e8, SUC_lexicon_tags_49 },
    { 0x287373e9, SUC_lexicon_tags_6 },
    { 0x323373ea, SUC_lexicon_tags_37 },
    { 0x127373ea, SUC_lexicon_tags_43 },
    { 0xdd3373eb, SUC_lexicon_tags_25 },
    { 0xc01373e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249373f1, SUC_lexicon_tags_17 },
    { 0x553373f2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31373f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13373fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x833373ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fd37401, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f37405, SUC_lexicon_tags_43 },
    { 0x6c137405, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d3740c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb37412, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x68137414, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71737416, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x3e337418, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d3741d, SUC_lexicon_tags_45 },
    { 0x2393741d, SUC_lexicon_tags_113 },
    { 0xd093741f, SUC_lexicon_tags_17 },
    { 0xe513741f, SUC_lexicon_tags_1 },
    { 0x6337420, SUC_lexicon_tags_8 },
    { 0x1bd37422, SUC_lexicon_tags_60 },
    { 0xcd137423, SUC_lexicon_tags_19 },
    { 0x57d3741e, SUC_lexicon_tags_1 },
    { 0x3a937425, SUC_lexicon_tags_21 },
    { 0xfc137426, SUC_lexicon_tags_1 },
    { 0x36b37427, SUC_lexicon_tags_25 },
    { 0x5c137427, SUC_lexicon_tags_1 },
    { 0xf8937429, SUC_lexicon_tags_17 },
    { 0x5ab3741f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9273742c, SUC_lexicon_tags_14 },
    { 0x2d3742c, SUC_lexicon_tags_1 },
    { 0x52d3742e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f37432, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f37438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813743f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59337443, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0937448, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37b3744a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54337450, SUC_lexicon_tags_16 },
    { 0xfcd37451, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe6537453, SUC_lexicon_tags_27 },
    { 0x2af37454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39337456, SUC_lexicon_tags_0 },
    { 0xa2937457, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2113745f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f37465, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x30b37467, SUC_lexicon_tags_13 },
    { 0x1cb37467, SUC_lexicon_tags_0 },
    { 0x22137468, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c73746e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97737472, SUC_lexicon_tags_15 },
    { 0xb537473, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a137475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2937479, SUC_lexicon_tags_19 },
    { 0xd5b37479, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1b3747c, SUC_lexicon_tags_12 },
    { 0xcf3747c, SUC_lexicon_tags_11 },
    { 0x5613747e, SUC_lexicon_tags_151 },
    { 0x33f3747c, SUC_lexicon_tags_262 },
    { 0xdd33747f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6ad37482, SUC_lexicon_tags_11 },
    { 0xb4937483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f73748d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24d3748f, SUC_lexicon_tags_6 },
    { 0x33337490, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf37493, SUC_lexicon_tags_28 },
    { 0xec537493, SUC_lexicon_tags_25 },
    { 0x5f337495, SUC_lexicon_tags_12 },
    { 0x8d37496, SUC_lexicon_tags_9 },
    { 0xfd937495, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9ad37499, SUC_lexicon_tags_53 },
    { 0xcfb3749a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xff73749d, SUC_lexicon_tags_25 },
    { 0x6a53749e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92b374a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa35374a2, SUC_lexicon_tags_15 },
    { 0x3c9374a3, SUC_lexicon_tags_45 },
    { 0xbd1374a4, SUC_lexicon_tags_68 },
    { 0x645374a2, SUC_lexicon_tags_17 },
    { 0x1a1374a4, SUC_lexicon_tags_17 },
    { 0xef374a7, SUC_lexicon_tags_38 },
    { 0xc45374a8, SUC_lexicon_tags_8 },
    { 0xc75374a9, SUC_lexicon_tags_11 },
    { 0xf0d374a9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x229374ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317374b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3374b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db374ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x551374bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7374c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9374ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xddf374d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x47b374d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x533374d4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d374d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdab374db, SUC_lexicon_tags_1 },
    { 0x1ab374dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa89374de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871374e2, SUC_lexicon_tags_11 },
    { 0x76b374e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421374eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd51374ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8f374ef, SUC_lexicon_tags_11 },
    { 0x401374f0, SUC_lexicon_tags_25 },
    { 0x29374f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ab374f3, SUC_lexicon_tags_92 },
    { 0xf01374f4, SUC_lexicon_tags_45 },
    { 0x2c3374f5, SUC_lexicon_tags_17 },
    { 0x3d5374f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949374fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe937500, SUC_lexicon_tags_23 },
    { 0xfd937501, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x71337503, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6137508, SUC_lexicon_tags_17 },
    { 0xb9d37509, SUC_lexicon_tags_25 },
    { 0x3713750a, SUC_lexicon_tags_1 },
    { 0x7333750a, SUC_lexicon_tags_7 },
    { 0x47337508, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa93750f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78d37511, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6737513, SUC_lexicon_tags_17 },
    { 0x6a137514, SUC_lexicon_tags_12 },
    { 0xd5737514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7737517, SUC_lexicon_tags_65 },
    { 0x14537517, SUC_lexicon_tags_37 },
    { 0x70d37518, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3fd3751b, SUC_lexicon_tags_17 },
    { 0x4a73751c, SUC_lexicon_tags_12 },
    { 0x9593751d, SUC_lexicon_tags_21 },
    { 0x88f3751e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ff37520, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb137523, SUC_lexicon_tags_45 },
    { 0xd7537524, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39737529, SUC_lexicon_tags_0 },
    { 0xe373752a, SUC_lexicon_tags_15 },
    { 0x3f33752b, SUC_lexicon_tags_31 },
    { 0xf7d3752c, SUC_lexicon_tags_11 },
    { 0xeff3752c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d37530, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9137532, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b337535, SUC_lexicon_tags_25 },
    { 0x11137536, SUC_lexicon_tags_15 },
    { 0x5df37535, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c13753b, SUC_lexicon_tags_37 },
    { 0x1bf3753b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8dd3753e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f537540, SUC_lexicon_tags_2 },
    { 0x73737541, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaef37543, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71337545, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6f37547, SUC_lexicon_tags_33 },
    { 0xb4537548, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd613754a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a33754d, SUC_lexicon_tags_17 },
    { 0x17b3754d, SUC_lexicon_tags_15 },
    { 0x65b3754f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x77937551, SUC_lexicon_tags_0 },
    { 0xf0137551, SUC_lexicon_tags_10 },
    { 0x46d37553, SUC_lexicon_tags_9 },
    { 0xe4937551, SUC_lexicon_tags_100 },
    { 0xb37555, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4b337557, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8433755b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4337560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db37563, SUC_lexicon_tags_38 },
    { 0x46537564, SUC_lexicon_tags_17 },
    { 0xbb737564, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x90937567, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d13756d, SUC_lexicon_tags_7 },
    { 0xf673756d, SUC_lexicon_tags_45 },
    { 0x26f3756d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1737572, SUC_lexicon_tags_31 },
    { 0x70f37573, SUC_lexicon_tags_1 },
    { 0x2ef37573, SUC_lexicon_tags_9 },
    { 0x94337575, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ed37577, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1253757b, SUC_lexicon_tags_37 },
    { 0x8e13757c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41937586, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa73758a, SUC_lexicon_tags_27 },
    { 0x293758a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4613758f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11937591, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1af37593, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb37597, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9253759a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad3759d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x107375a0, SUC_lexicon_tags_17 },
    { 0x515375a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x141375a4, SUC_lexicon_tags_31 },
    { 0xa27375a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb375b5, SUC_lexicon_tags_28 },
    { 0xb43375b6, SUC_lexicon_tags_17 },
    { 0x29d375b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa37375b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1375bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297375c7, SUC_lexicon_tags_25 },
    { 0x285375c8, SUC_lexicon_tags_1 },
    { 0x65b375c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xee1375cb, SUC_lexicon_tags_14 },
    { 0x957375cc, SUC_lexicon_tags_26 },
    { 0xdc3375cd, SUC_lexicon_tags_38 },
    { 0x50b375cb, SUC_lexicon_tags_39 },
    { 0xef5375cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2b375d1, SUC_lexicon_tags_6 },
    { 0x3fb375d1, SUC_lexicon_tags_7 },
    { 0xa47375d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17375d6, SUC_lexicon_tags_63 },
    { 0xf4b375d7, SUC_lexicon_tags_25 },
    { 0x365375d8, SUC_lexicon_tags_31 },
    { 0x23f375d7, SUC_lexicon_tags_7 },
    { 0x613375da, SUC_lexicon_tags_45 },
    { 0x375375da, SUC_lexicon_tags_37 },
    { 0x865375db, SUC_lexicon_tags_14 },
    { 0x77d375dd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3375e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23375e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17b375e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05375f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17d375f7, SUC_lexicon_tags_87 },
    { 0x91f375f8, SUC_lexicon_tags_2 },
    { 0xaa3375f7, SUC_lexicon_tags_2 },
    { 0xc81375f8, SUC_lexicon_tags_14 },
    { 0xec7375f8, SUC_lexicon_tags_38 },
    { 0xa91375fc, SUC_lexicon_tags_10 },
    { 0xe9375fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae537602, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c537605, SUC_lexicon_tags_0 },
    { 0x34537606, SUC_lexicon_tags_0 },
    { 0xacf37606, SUC_lexicon_tags_6 },
    { 0xb0937607, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcf33760a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba93760c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d37613, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc1537615, SUC_lexicon_tags_0 },
    { 0x93337616, SUC_lexicon_tags_46 },
    { 0xd5d37616, SUC_lexicon_tags_25 },
    { 0x50937618, SUC_lexicon_tags_103 },
    { 0xd5337618, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb3761d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9873761f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72d37621, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f937627, SUC_lexicon_tags_14 },
    { 0xadf37628, SUC_lexicon_tags_11 },
    { 0x24f37629, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8937631, SUC_lexicon_tags_82 },
    { 0xceb37631, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b37636, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e93763a, SUC_lexicon_tags_38 },
    { 0x68b3763b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c13763d, SUC_lexicon_tags_23 },
    { 0xb7d3763e, SUC_lexicon_tags_37 },
    { 0x2813763f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29537642, SUC_lexicon_tags_63 },
    { 0x81d37643, SUC_lexicon_tags_63 },
    { 0xa1337644, SUC_lexicon_tags_17 },
    { 0x3bf37644, SUC_lexicon_tags_7 },
    { 0xd9137644, SUC_lexicon_tags_60 },
    { 0xdf37647, SUC_lexicon_tags_24 },
    { 0x63737646, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf193764a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f3764d, SUC_lexicon_tags_7 },
    { 0xe8f3764e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a737651, SUC_lexicon_tags_37 },
    { 0xa937652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe473765a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44f3765c, SUC_lexicon_tags_23 },
    { 0xaad3765d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f37660, SUC_lexicon_tags_28 },
    { 0x4ff37661, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8337666, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4413766c, SUC_lexicon_tags_1 },
    { 0x54d3766d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c937671, SUC_lexicon_tags_39 },
    { 0x1b337672, SUC_lexicon_tags_1 },
    { 0xfbd37673, SUC_lexicon_tags_15 },
    { 0xf2f37673, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23d37676, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0xfbd37678, SUC_lexicon_tags_25 },
    { 0x17137679, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd3767e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x85937681, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf113768b, SUC_lexicon_tags_6 },
    { 0x8d13768c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32137693, SUC_lexicon_tags_60 },
    { 0x7c737694, SUC_lexicon_tags_37 },
    { 0x9c937694, SUC_lexicon_tags_30 },
    { 0x63737696, SUC_lexicon_tags_12 },
    { 0xd5137693, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf3769a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7673769e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x429376a0, SUC_lexicon_tags_11 },
    { 0x20b376a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3376a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1376b2, SUC_lexicon_tags_33 },
    { 0x7d3376b3, SUC_lexicon_tags_15 },
    { 0x43b376b4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f376bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb376c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x129376c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7376c5, SUC_lexicon_tags_37 },
    { 0xc91376c5, SUC_lexicon_tags_1 },
    { 0x4c5376c7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f376ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb376d0, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905376d5, SUC_lexicon_tags_4 },
    { 0x90d376d5, SUC_lexicon_tags_26 },
    { 0x15f376d7, SUC_lexicon_tags_2 },
    { 0xb0d376d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b376dc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x10b376de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe35376e0, SUC_lexicon_tags_38 },
    { 0x405376e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f376e6, SUC_lexicon_tags_7 },
    { 0xddf376e7, SUC_lexicon_tags_28 },
    { 0xf63376e6, SUC_lexicon_tags_37 },
    { 0x30f376e9, SUC_lexicon_tags_43 },
    { 0xad1376ea, SUC_lexicon_tags_87 },
    { 0x80b376e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb376f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfe1376f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x519376fa, SUC_lexicon_tags_47 },
    { 0x7f1376fa, SUC_lexicon_tags_1 },
    { 0x93b376fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8dd376fe, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x45737700, SUC_lexicon_tags_27 },
    { 0x44b37701, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27f37703, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x51d37705, SUC_lexicon_tags_2 },
    { 0xa4137705, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x81737709, SUC_lexicon_tags_1 },
    { 0xbe73770a, SUC_lexicon_tags_12 },
    { 0xa3f3770b, SUC_lexicon_tags_12 },
    { 0x3a537709, SUC_lexicon_tags_9 },
    { 0xacf3770d, SUC_lexicon_tags_4 },
    { 0x6b33770c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef37711, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2937715, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63337719, SUC_lexicon_tags_19 },
    { 0x60737719, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b3771e, SUC_lexicon_tags_2 },
    { 0xa373771e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x2b737721, SUC_lexicon_tags_23 },
    { 0xcd537721, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84537726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15f37728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf3772d, SUC_lexicon_tags_2 },
    { 0xf593772e, SUC_lexicon_tags_39 },
    { 0x76d3772f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf37734, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab53773b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d33773e, SUC_lexicon_tags_25 },
    { 0xe933773e, SUC_lexicon_tags_27 },
    { 0x2f337740, SUC_lexicon_tags_79 },
    { 0x34b3773f, SUC_lexicon_tags_15 },
    { 0x9b937742, SUC_lexicon_tags_23 },
    { 0x2d33773f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b37746, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80337749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9833774d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3937752, SUC_lexicon_tags_11 },
    { 0xd6d37753, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf937758, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x29f3775a, SUC_lexicon_tags_15 },
    { 0x8833775a, SUC_lexicon_tags_33 },
    { 0x7273775c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf473775f, SUC_lexicon_tags_11 },
    { 0x3893775f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f537766, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88337771, SUC_lexicon_tags_6 },
    { 0x40537772, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa937775, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9513777a, SUC_lexicon_tags_33 },
    { 0xde93777a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a137780, SUC_lexicon_tags_14 },
    { 0x5fb37781, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37937787, SUC_lexicon_tags_64 },
    { 0x89537788, SUC_lexicon_tags_15 },
    { 0x7d37789, SUC_lexicon_tags_33 },
    { 0x8b137787, SUC_lexicon_tags_15 },
    { 0xb8f37789, SUC_lexicon_tags_31 },
    { 0xb6337789, SUC_lexicon_tags_63 },
    { 0xd413778d, SUC_lexicon_tags_7 },
    { 0x4f3778d, SUC_lexicon_tags_47 },
    { 0x8453778d, SUC_lexicon_tags_17 },
    { 0xab33778a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda337792, SUC_lexicon_tags_37 },
    { 0xac937793, SUC_lexicon_tags_11 },
    { 0x26f37794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20537796, SUC_lexicon_tags_1 },
    { 0x63937797, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b3779a, SUC_lexicon_tags_31 },
    { 0xbe93779b, SUC_lexicon_tags_25 },
    { 0x70b3779b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601377a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d377a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd0d377a7, SUC_lexicon_tags_23 },
    { 0x7b9377a7, SUC_lexicon_tags_25 },
    { 0x259377a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269377ad, SUC_lexicon_tags_17 },
    { 0x18f377ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b9377b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f377b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23377b8, SUC_lexicon_tags_28 },
    { 0xf1377b9, SUC_lexicon_tags_15 },
    { 0x875377ba, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df377bd, SUC_lexicon_tags_98 },
    { 0x8fb377be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b377c4, SUC_lexicon_tags_6 },
    { 0x605377c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247377cb, SUC_lexicon_tags_11 },
    { 0x87f377cc, SUC_lexicon_tags_30 },
    { 0x3f5377cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff377d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe31377d6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f377db, SUC_lexicon_tags_0 },
    { 0x565377dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd1377de, SUC_lexicon_tags_87 },
    { 0x3af377df, SUC_lexicon_tags_37 },
    { 0x145377e0, SUC_lexicon_tags_29 },
    { 0x6f1377e1, SUC_lexicon_tags_25 },
    { 0x3e5377e0, SUC_lexicon_tags_48 },
    { 0xbe7377df, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x275377e5, SUC_lexicon_tags_15 },
    { 0x4eb377e6, SUC_lexicon_tags_46 },
    { 0x9df377e6, SUC_lexicon_tags_37 },
    { 0xec3377e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35377ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07377f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd377f6, SUC_lexicon_tags_37 },
    { 0x503377f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3377fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8f377fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f37800, SUC_lexicon_tags_7 },
    { 0x8e737801, SUC_lexicon_tags_15 },
    { 0x69d37802, SUC_lexicon_tags_11 },
    { 0xf6537803, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f37806, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb3780a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d3780d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8737813, SUC_lexicon_tags_39 },
    { 0xedb37814, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26137817, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2d37819, SUC_lexicon_tags_37 },
    { 0x9f13781a, SUC_lexicon_tags_17 },
    { 0xd3781b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7ab3781d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b37821, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef37827, SUC_lexicon_tags_65 },
    { 0xd4137828, SUC_lexicon_tags_2 },
    { 0xec337827, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3f3782b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8937831, SUC_lexicon_tags_12 },
    { 0x96d37832, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd337834, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b3783a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa0b3783c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd37841, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d37844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93737846, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19737848, SUC_lexicon_tags_65 },
    { 0xa0f37848, SUC_lexicon_tags_17 },
    { 0x53337849, SUC_lexicon_tags_45 },
    { 0x8193784a, SUC_lexicon_tags_38 },
    { 0x6293784b, SUC_lexicon_tags_12 },
    { 0x3f73784d, SUC_lexicon_tags_31 },
    { 0x3fb3784d, SUC_lexicon_tags_11 },
    { 0xb173784f, SUC_lexicon_tags_30 },
    { 0x79f37850, SUC_lexicon_tags_1 },
    { 0x62f37851, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x78737854, SUC_lexicon_tags_11 },
    { 0x31b37855, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x3f937857, SUC_lexicon_tags_8 },
    { 0x71937858, SUC_lexicon_tags_14 },
    { 0xb737859, SUC_lexicon_tags_11 },
    { 0x1fd37858, SUC_lexicon_tags_46 },
    { 0x2333785b, SUC_lexicon_tags_47 },
    { 0xf933785b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d337861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54937863, SUC_lexicon_tags_18 },
    { 0xadf37863, SUC_lexicon_tags_37 },
    { 0xe4537865, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d537869, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd693786c, SUC_lexicon_tags_60 },
    { 0x6713786c, SUC_lexicon_tags_27 },
    { 0xced3786e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9337870, SUC_lexicon_tags_8 },
    { 0xd4f37871, SUC_lexicon_tags_49 },
    { 0x65f37870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b37875, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b37879, SUC_lexicon_tags_37 },
    { 0x3cd3787a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb53787d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe33787f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50537884, SUC_lexicon_tags_9 },
    { 0x43337885, SUC_lexicon_tags_46 },
    { 0xb7d37884, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d3788a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d937897, SUC_lexicon_tags_61 },
    { 0x52337898, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e73789b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75378a2, SUC_lexicon_tags_33 },
    { 0x8c3378a3, SUC_lexicon_tags_25 },
    { 0xa85378a4, SUC_lexicon_tags_1 },
    { 0xc13378a2, SUC_lexicon_tags_19 },
    { 0x48b378a5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe4d378a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d378ac, SUC_lexicon_tags_1 },
    { 0x967378ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x53f378af, SUC_lexicon_tags_14 },
    { 0x4b1378b0, SUC_lexicon_tags_3 },
    { 0xfcb378b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb378b4, SUC_lexicon_tags_11 },
    { 0xfdf378b4, SUC_lexicon_tags_85 },
    { 0x5ff378b6, SUC_lexicon_tags_17 },
    { 0xf7f378b7, SUC_lexicon_tags_70 },
    { 0x6f1378b8, SUC_lexicon_tags_55 },
    { 0xbb378b9, SUC_lexicon_tags_63 },
    { 0x651378ba, SUC_lexicon_tags_44 },
    { 0xde1378bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b378be, SUC_lexicon_tags_37 },
    { 0x25f378bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97378c5, SUC_lexicon_tags_69 },
    { 0xc15378c5, SUC_lexicon_tags_30 },
    { 0xe43378c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x661378ca, SUC_lexicon_tags_1 },
    { 0x5fb378cb, SUC_lexicon_tags_9 },
    { 0x923378cc, SUC_lexicon_tags_33 },
    { 0xf3b378cb, SUC_lexicon_tags_11 },
    { 0x38d378cd, SUC_lexicon_tags_2 },
    { 0x45d378cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33f378d1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf49378d3, SUC_lexicon_tags_28 },
    { 0x63d378d4, SUC_lexicon_tags_14 },
    { 0x5f1378d4, SUC_lexicon_tags_21 },
    { 0xc97378d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb378da, SUC_lexicon_tags_6 },
    { 0x877378da, SUC_lexicon_tags_66 },
    { 0x9a9378dc, SUC_lexicon_tags_36 },
    { 0x50f378dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795378e4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xae9378e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5378f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x331378f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9378f8, SUC_lexicon_tags_38 },
    { 0x8d5378f9, SUC_lexicon_tags_29 },
    { 0x357378f9, SUC_lexicon_tags_47 },
    { 0xfa3378f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf378fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f37904, SUC_lexicon_tags_15 },
    { 0x74937905, SUC_lexicon_tags_65 },
    { 0x23b37904, SUC_lexicon_tags_31 },
    { 0xe6937904, SUC_lexicon_tags_1 },
    { 0x82337908, SUC_lexicon_tags_47 },
    { 0x1ef37909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfb3790b, SUC_lexicon_tags_8 },
    { 0xb193790c, SUC_lexicon_tags_26 },
    { 0x7db3790c, SUC_lexicon_tags_2 },
    { 0x9373790e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fb37910, SUC_lexicon_tags_32 },
    { 0xa2b37911, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78537919, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9233791c, SUC_lexicon_tags_49 },
    { 0x8ed3791d, SUC_lexicon_tags_23 },
    { 0xc333791e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x2bf37920, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8937926, SUC_lexicon_tags_25 },
    { 0x29137927, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc337929, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39137933, SUC_lexicon_tags_7 },
    { 0x1af37933, SUC_lexicon_tags_45 },
    { 0x80937935, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f37938, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x193793a, SUC_lexicon_tags_45 },
    { 0xf9b3793b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xb313793d, SUC_lexicon_tags_45 },
    { 0x13793d, SUC_lexicon_tags_46 },
    { 0xf053793f, SUC_lexicon_tags_23 },
    { 0xc6137940, SUC_lexicon_tags_1 },
    { 0x9e337941, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80937943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70b37945, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf413794d, SUC_lexicon_tags_1 },
    { 0xf693794e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b37952, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f337955, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b537958, SUC_lexicon_tags_6 },
    { 0xdd37959, SUC_lexicon_tags_23 },
    { 0x67d37959, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf953795c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd913795e, SUC_lexicon_tags_0 },
    { 0x8ad3795f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x97337962, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d37968, SUC_lexicon_tags_207 },
    { 0xc937968, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2673796b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6b3796d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf37971, SUC_lexicon_tags_37 },
    { 0x8b737972, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b537974, SUC_lexicon_tags_12 },
    { 0x72f37975, SUC_lexicon_tags_65 },
    { 0x85337976, SUC_lexicon_tags_37 },
    { 0x2f37976, SUC_lexicon_tags_29 },
    { 0x83737978, SUC_lexicon_tags_63 },
    { 0xcd337974, SUC_lexicon_tags_43 },
    { 0x5f537978, SUC_lexicon_tags_17 },
    { 0xd5b37977, SUC_lexicon_tags_38 },
    { 0x93d3797c, SUC_lexicon_tags_7 },
    { 0xf0b3797a, SUC_lexicon_tags_9 },
    { 0xe153797e, SUC_lexicon_tags_25 },
    { 0x58d3797f, SUC_lexicon_tags_2 },
    { 0xd6937980, SUC_lexicon_tags_19 },
    { 0xc3f37981, SUC_lexicon_tags_7 },
    { 0x72737980, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b37988, SUC_lexicon_tags_25 },
    { 0x64f37988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf173798b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ab3798d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xecb3798f, SUC_lexicon_tags_43 },
    { 0x55137990, SUC_lexicon_tags_37 },
    { 0xfab37991, SUC_lexicon_tags_46 },
    { 0xe3d37990, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad737998, SUC_lexicon_tags_9 },
    { 0xd2b37999, SUC_lexicon_tags_15 },
    { 0xc5d3799a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff53799c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9379a0, SUC_lexicon_tags_45 },
    { 0x953379a1, SUC_lexicon_tags_17 },
    { 0xda3379a2, SUC_lexicon_tags_33 },
    { 0xa6d379a3, SUC_lexicon_tags_2 },
    { 0xceb379a4, SUC_lexicon_tags_47 },
    { 0x78f379a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd31379a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03379ac, SUC_lexicon_tags_66 },
    { 0x2e7379ad, SUC_lexicon_tags_45 },
    { 0xfad379ae, SUC_lexicon_tags_11 },
    { 0x5c1379af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b3379b1, SUC_lexicon_tags_11 },
    { 0x3e3379b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x135379b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89379c0, SUC_lexicon_tags_30 },
    { 0x299379c0, SUC_lexicon_tags_17 },
    { 0xb1379c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd03379c4, SUC_lexicon_tags_10 },
    { 0x323379c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1379c8, SUC_lexicon_tags_17 },
    { 0xb9b379c9, SUC_lexicon_tags_11 },
    { 0xf19379c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac7379cc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8a1379ce, SUC_lexicon_tags_25 },
    { 0x597379cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d379d8, SUC_lexicon_tags_12 },
    { 0xc35379d8, SUC_lexicon_tags_14 },
    { 0xe73379da, SUC_lexicon_tags_14 },
    { 0xf1379db, SUC_lexicon_tags_1 },
    { 0x90b379d9, SUC_lexicon_tags_11 },
    { 0x8af379dd, SUC_lexicon_tags_27 },
    { 0xc85379db, SUC_lexicon_tags_2 },
    { 0x9eb379dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441379e3, SUC_lexicon_tags_7 },
    { 0x317379e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9379e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9379eb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x55379ee, SUC_lexicon_tags_61 },
    { 0xaa5379ee, SUC_lexicon_tags_6 },
    { 0x8ff379ee, SUC_lexicon_tags_37 },
    { 0x65f379f1, SUC_lexicon_tags_66 },
    { 0xe59379f1, SUC_lexicon_tags_11 },
    { 0x799379f3, SUC_lexicon_tags_1 },
    { 0x445379f2, SUC_lexicon_tags_6 },
    { 0x335379f5, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb379fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x273379ff, SUC_lexicon_tags_7 },
    { 0x6d137a00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeb937a02, SUC_lexicon_tags_17 },
    { 0x1137a03, SUC_lexicon_tags_49 },
    { 0x5cf37a03, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd37a0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f37a13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d737a19, SUC_lexicon_tags_68 },
    { 0x19d37a19, SUC_lexicon_tags_30 },
    { 0xdbd37a1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcab37a1d, SUC_lexicon_tags_1 },
    { 0x6c937a1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd37a24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9737a2b, SUC_lexicon_tags_2 },
    { 0xf7337a2b, SUC_lexicon_tags_12 },
    { 0x50d37a2c, SUC_lexicon_tags_27 },
    { 0x58b37a2e, SUC_lexicon_tags_3 },
    { 0x64d37a2f, SUC_lexicon_tags_23 },
    { 0xd6f37a2c, SUC_lexicon_tags_60 },
    { 0x2737a31, SUC_lexicon_tags_2 },
    { 0x50f37a30, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe337a34, SUC_lexicon_tags_27 },
    { 0xd3737a35, SUC_lexicon_tags_46 },
    { 0xe7537a36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3737a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e537a3b, SUC_lexicon_tags_19 },
    { 0x53737a3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9737a3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa337a40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c337a49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d937a4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x94737a51, SUC_lexicon_tags_11 },
    { 0x60137a52, SUC_lexicon_tags_2 },
    { 0xff337a53, SUC_lexicon_tags_15 },
    { 0xd9d37a52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef37a59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41737a5b, SUC_lexicon_tags_23 },
    { 0xf5d37a5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d37a5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63337a62, SUC_lexicon_tags_0 },
    { 0xe2537a63, SUC_lexicon_tags_48 },
    { 0x11537a64, SUC_lexicon_tags_1 },
    { 0xbe537a64, SUC_lexicon_tags_17 },
    { 0xa1537a66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b37a69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6b37a6b, SUC_lexicon_tags_19 },
    { 0xa8137a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc337a70, SUC_lexicon_tags_18 },
    { 0xb5f37a71, SUC_lexicon_tags_17 },
    { 0x33337a70, SUC_lexicon_tags_1 },
    { 0xf6337a71, SUC_lexicon_tags_33 },
    { 0x3537a71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50337a76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c737a7d, SUC_lexicon_tags_79 },
    { 0x22337a7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7bb37a80, SUC_lexicon_tags_49 },
    { 0x27d37a81, SUC_lexicon_tags_4 },
    { 0x80f37a81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xed137a85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20137a89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f37a8c, SUC_lexicon_tags_12 },
    { 0x28537a8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6337a8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd37a93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41137a98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x57937a9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18137a9d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c537aa1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d137aa4, SUC_lexicon_tags_7 },
    { 0x13b37aa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25537aa8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x97137aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f137aad, SUC_lexicon_tags_15 },
    { 0x91137aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0937ab2, SUC_lexicon_tags_23 },
    { 0x2f537ab3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46f37ab5, SUC_lexicon_tags_32 },
    { 0x5a937ab6, SUC_lexicon_tags_11 },
    { 0xb9f37ab6, SUC_lexicon_tags_17 },
    { 0xd7137ab8, SUC_lexicon_tags_17 },
    { 0x67537ab9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59937abc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93337ac0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1737aca, SUC_lexicon_tags_29 },
    { 0x18137acb, SUC_lexicon_tags_31 },
    { 0xf6b37acc, SUC_lexicon_tags_49 },
    { 0xa5737acd, SUC_lexicon_tags_31 },
    { 0xf3937acb, SUC_lexicon_tags_7 },
    { 0xd1937acf, SUC_lexicon_tags_7 },
    { 0x8f937ad0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe337ad4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89137ad8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79b37ada, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53737adf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0537ae6, SUC_lexicon_tags_15 },
    { 0x41f37ae7, SUC_lexicon_tags_46 },
    { 0x20737ae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0337aea, SUC_lexicon_tags_37 },
    { 0x4a337aeb, SUC_lexicon_tags_12 },
    { 0x77537aec, SUC_lexicon_tags_37 },
    { 0xef137aec, SUC_lexicon_tags_170 },
    { 0x33f37aeb, SUC_lexicon_tags_11 },
    { 0x96537aef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a937af3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b37af9, SUC_lexicon_tags_21 },
    { 0x67137afa, SUC_lexicon_tags_65 },
    { 0x64d37afb, SUC_lexicon_tags_15 },
    { 0xdb537afc, SUC_lexicon_tags_32 },
    { 0x15137afd, SUC_lexicon_tags_11 },
    { 0xd8737af9, SUC_lexicon_tags_10 },
    { 0xa8937afa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f937b03, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b937b06, SUC_lexicon_tags_25 },
    { 0x7af37b06, SUC_lexicon_tags_1 },
    { 0x7d737b08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d37b0b, SUC_lexicon_tags_37 },
    { 0xe6537b0c, SUC_lexicon_tags_37 },
    { 0x7137b0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed37b10, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42337b16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c737b1b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x90337b1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c137b21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee737b23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab37b28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff37b2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91537b33, SUC_lexicon_tags_27 },
    { 0xb5737b34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b337b37, SUC_lexicon_tags_17 },
    { 0xe137b37, SUC_lexicon_tags_9 },
    { 0x57737b38, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f37b3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6137b3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15537b43, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7737b46, SUC_lexicon_tags_25 },
    { 0x1a737b47, SUC_lexicon_tags_25 },
    { 0x70137b47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa937b4a, SUC_lexicon_tags_37 },
    { 0x7c137b4b, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0937b51, SUC_lexicon_tags_91 },
    { 0x30737b52, SUC_lexicon_tags_25 },
    { 0x83d37b53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5737b56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d37b5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d37b61, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b37b64, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad37b68, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b37b6b, SUC_lexicon_tags_217 },
    { 0x43937b6c, SUC_lexicon_tags_15 },
    { 0xa8937b6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b37b70, SUC_lexicon_tags_32 },
    { 0xf3f37b70, SUC_lexicon_tags_37 },
    { 0x81137b72, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95737b7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b37b84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe937b88, SUC_lexicon_tags_12 },
    { 0x11f37b89, SUC_lexicon_tags_37 },
    { 0x2f137b88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a137b8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12737b91, SUC_lexicon_tags_0 },
    { 0x82537b92, SUC_lexicon_tags_45 },
    { 0x86537b92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63937b97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4737ba0, SUC_lexicon_tags_25 },
    { 0xef37ba0, SUC_lexicon_tags_37 },
    { 0xb9737ba1, SUC_lexicon_tags_16 },
    { 0xc5537ba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b737ba7, SUC_lexicon_tags_45 },
    { 0x34937ba7, SUC_lexicon_tags_37 },
    { 0xdf137ba9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d937bac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1737bb0, SUC_lexicon_tags_8 },
    { 0x51f37bb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d37bb9, SUC_lexicon_tags_12 },
    { 0xe6737bb9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x63137bbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d37bc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d37bc5, SUC_lexicon_tags_37 },
    { 0x45d37bc5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcb937bc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2f37bca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12937bd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24737bdb, SUC_lexicon_tags_26 },
    { 0xf537bdb, SUC_lexicon_tags_66 },
    { 0x67b37bdd, SUC_lexicon_tags_1 },
    { 0xe0137bde, SUC_lexicon_tags_87 },
    { 0xbbb37bdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17137be1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf137be6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa7137be8, SUC_lexicon_tags_8 },
    { 0xbab37be8, SUC_lexicon_tags_2 },
    { 0xe8f37bea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d37bee, SUC_lexicon_tags_37 },
    { 0x52f37bef, SUC_lexicon_tags_15 },
    { 0xf3937bee, SUC_lexicon_tags_17 },
    { 0xee337bf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7937bf6, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x8c337bf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14737bfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf137bfe, SUC_lexicon_tags_1 },
    { 0x70337bff, SUC_lexicon_tags_7 },
    { 0xcff37bfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc537c06, SUC_lexicon_tags_7 },
    { 0x5ff37c07, SUC_lexicon_tags_17 },
    { 0xcfb37c08, SUC_lexicon_tags_55 },
    { 0xce137c06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1337c0e, SUC_lexicon_tags_30 },
    { 0xb3d37c0e, SUC_lexicon_tags_12 },
    { 0x3cb37c10, SUC_lexicon_tags_29 },
    { 0x5ed37c10, SUC_lexicon_tags_17 },
    { 0x39737c12, SUC_lexicon_tags_0 },
    { 0xfa937c13, SUC_lexicon_tags_2 },
    { 0x59137c13, SUC_lexicon_tags_2 },
    { 0x7f337c15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd737c1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9537c21, SUC_lexicon_tags_25 },
    { 0x71d37c22, SUC_lexicon_tags_7 },
    { 0xc1337c22, SUC_lexicon_tags_11 },
    { 0x9f737c22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa737c2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0137c2d, SUC_lexicon_tags_17 },
    { 0x91f37c2e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb937c31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16737c33, SUC_lexicon_tags_2 },
    { 0x5b537c34, SUC_lexicon_tags_37 },
    { 0x3d737c34, SUC_lexicon_tags_63 },
    { 0x1bb37c34, SUC_lexicon_tags_17 },
    { 0x5937c37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74937c39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd37c3c, SUC_lexicon_tags_87 },
    { 0x31937c3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab37c45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55737c4a, SUC_lexicon_tags_37 },
    { 0x24b37c4b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87737c53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f37c57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d737c59, SUC_lexicon_tags_32 },
    { 0xb3737c5a, SUC_lexicon_tags_29 },
    { 0x2ef37c5b, SUC_lexicon_tags_1 },
    { 0xe8137c5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62b37c5e, SUC_lexicon_tags_0 },
    { 0x2737c5f, SUC_lexicon_tags_520 },
    { 0x19137c60, SUC_lexicon_tags_38 },
    { 0x5a137c60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef937c63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73537c67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86737c6f, SUC_lexicon_tags_37 },
    { 0x22737c70, SUC_lexicon_tags_27 },
    { 0xcbb37c71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca937c73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d37c76, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74937c7a, SUC_lexicon_tags_11 },
    { 0xfe937c7b, SUC_lexicon_tags_12 },
    { 0xd1537c7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf137c7f, SUC_lexicon_tags_8 },
    { 0x41337c80, SUC_lexicon_tags_7 },
    { 0x16137c80, SUC_lexicon_tags_26 },
    { 0x8ab37c82, SUC_lexicon_tags_7 },
    { 0x99937c83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b37c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b937c8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b37c97, SUC_lexicon_tags_14 },
    { 0x4d137c98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b937c9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28937ca3, SUC_lexicon_tags_27 },
    { 0xc5537ca4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcb137ca6, SUC_lexicon_tags_17 },
    { 0x14937ca6, SUC_lexicon_tags_158 },
    { 0x78737ca8, SUC_lexicon_tags_12 },
    { 0x10b37ca8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1137cab, SUC_lexicon_tags_37 },
    { 0x2ab37cab, SUC_lexicon_tags_12 },
    { 0xf737cab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b337caf, SUC_lexicon_tags_2 },
    { 0x6e137caf, SUC_lexicon_tags_11 },
    { 0x41737cb0, SUC_lexicon_tags_30 },
    { 0x9f737cb2, SUC_lexicon_tags_17 },
    { 0x9d937caf, SUC_lexicon_tags_38 },
    { 0xd3d37cb4, SUC_lexicon_tags_11 },
    { 0xcfb37cb5, SUC_lexicon_tags_1 },
    { 0x69f37cb6, SUC_lexicon_tags_14 },
    { 0x63937cb7, SUC_lexicon_tags_26 },
    { 0x88937cb8, SUC_lexicon_tags_115 },
    { 0xcbf37cb4, SUC_lexicon_tags_12 },
    { 0x9f337cba, SUC_lexicon_tags_47 },
    { 0xa4737cba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1f37cbd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xef37cbf, SUC_lexicon_tags_2 },
    { 0xa4737cbf, SUC_lexicon_tags_10 },
    { 0xd4f37cc1, SUC_lexicon_tags_11 },
    { 0x40337cc2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe537cc5, SUC_lexicon_tags_15 },
    { 0x64f37cc6, SUC_lexicon_tags_25 },
    { 0x10537cc7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd7f37cc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d37ccd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc937cd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacf37cd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5337cd7, SUC_lexicon_tags_12 },
    { 0x4dd37cd8, SUC_lexicon_tags_27 },
    { 0x9ad37cd9, SUC_lexicon_tags_7 },
    { 0x90b37cda, SUC_lexicon_tags_25 },
    { 0x36b37cdb, SUC_lexicon_tags_15 },
    { 0x84537cdc, SUC_lexicon_tags_34 },
    { 0x5bb37cd9, SUC_lexicon_tags_35 },
    { 0x3d537cde, SUC_lexicon_tags_17 },
    { 0x46f37cd7, SUC_lexicon_tags_12 },
    { 0x63f37ce0, SUC_lexicon_tags_1 },
    { 0x18937cda, SUC_lexicon_tags_68 },
    { 0xfdd37ce2, SUC_lexicon_tags_47 },
    { 0x40537ce3, SUC_lexicon_tags_6 },
    { 0x13737cd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6737ce9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8a937ceb, SUC_lexicon_tags_49 },
    { 0x5f937cec, SUC_lexicon_tags_17 },
    { 0xb6537ced, SUC_lexicon_tags_1 },
    { 0xff537cec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa737cfa, SUC_lexicon_tags_49 },
    { 0xf1337cfb, SUC_lexicon_tags_4 },
    { 0x26f37cfc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x97137cff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f37d04, SUC_lexicon_tags_37 },
    { 0xf8137d04, SUC_lexicon_tags_4 },
    { 0xcc337d06, SUC_lexicon_tags_40 },
    { 0xfa737d07, SUC_lexicon_tags_15 },
    { 0xfdd37d08, SUC_lexicon_tags_7 },
    { 0x46737d09, SUC_lexicon_tags_1 },
    { 0x60137d04, SUC_lexicon_tags_17 },
    { 0x94537d04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd37d0d, SUC_lexicon_tags_48 },
    { 0xa2337d0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb37d13, SUC_lexicon_tags_9 },
    { 0xc8f37d14, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a937d17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f37d1a, SUC_lexicon_tags_1 },
    { 0xd2337d1a, SUC_lexicon_tags_30 },
    { 0x40537d1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4137d1f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab37d22, SUC_lexicon_tags_869 },
    { 0, NULL },
    { 0xa1737d24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52d37d26, SUC_lexicon_tags_14 },
    { 0xb0537d27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f37d2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f37d2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2f37d31, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x7f537d33, SUC_lexicon_tags_11 },
    { 0x4fd37d34, SUC_lexicon_tags_25 },
    { 0x76b37d34, SUC_lexicon_tags_25 },
    { 0x46f37d36, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb737d39, SUC_lexicon_tags_45 },
    { 0xc137d3a, SUC_lexicon_tags_0 },
    { 0xeb937d3a, SUC_lexicon_tags_43 },
    { 0xc9337d3b, SUC_lexicon_tags_7 },
    { 0x1d337d3d, SUC_lexicon_tags_19 },
    { 0x6b737d39, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x79d37d40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb337d4b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x7137d4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd37d52, SUC_lexicon_tags_2 },
    { 0x38b37d52, SUC_lexicon_tags_17 },
    { 0x25f37d54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63937d58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c337d5d, SUC_lexicon_tags_25 },
    { 0x3e937d5e, SUC_lexicon_tags_25 },
    { 0xa9937d5d, SUC_lexicon_tags_8 },
    { 0x2f137d60, SUC_lexicon_tags_37 },
    { 0x4ab37d60, SUC_lexicon_tags_47 },
    { 0xc3f37d62, SUC_lexicon_tags_37 },
    { 0xc3737d63, SUC_lexicon_tags_25 },
    { 0x70537d63, SUC_lexicon_tags_1 },
    { 0xdaf37d63, SUC_lexicon_tags_11 },
    { 0x5937d61, SUC_lexicon_tags_0 },
    { 0x72d37d62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb37d6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3b37d6f, SUC_lexicon_tags_49 },
    { 0x50537d70, SUC_lexicon_tags_47 },
    { 0xa1937d71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8db37d73, SUC_lexicon_tags_11 },
    { 0x2bf37d74, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb1937d76, SUC_lexicon_tags_7 },
    { 0xd1737d77, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x27f37d79, SUC_lexicon_tags_47 },
    { 0x5e337d79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1537d7e, SUC_lexicon_tags_30 },
    { 0x9fb37d7e, SUC_lexicon_tags_12 },
    { 0x9bb37d80, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f37d84, SUC_lexicon_tags_12 },
    { 0xee737d84, SUC_lexicon_tags_3 },
    { 0xdbf37d84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6bd37d88, SUC_lexicon_tags_25 },
    { 0x67f37d88, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb37d8e, SUC_lexicon_tags_15 },
    { 0x31737d8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d37d93, SUC_lexicon_tags_11 },
    { 0x2a937d94, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9337d99, SUC_lexicon_tags_40 },
    { 0x6137d9a, SUC_lexicon_tags_12 },
    { 0x55537d9b, SUC_lexicon_tags_37 },
    { 0xd2937d9c, SUC_lexicon_tags_1 },
    { 0x7f537d9d, SUC_lexicon_tags_18 },
    { 0x3cb37d9e, SUC_lexicon_tags_15 },
    { 0x5b137d9d, SUC_lexicon_tags_14 },
    { 0xed937da0, SUC_lexicon_tags_0 },
    { 0x2a737d9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31737da3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92d37da5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea537da7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd37dac, SUC_lexicon_tags_445 },
    { 0x97537dad, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb37db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50137db7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84137dbb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6537dc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb37dc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83737dc6, SUC_lexicon_tags_29 },
    { 0x7cd37dc7, SUC_lexicon_tags_62 },
    { 0x3bf37dc6, SUC_lexicon_tags_31 },
    { 0x98137dc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f37dce, SUC_lexicon_tags_25 },
    { 0xb1137dcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b37dd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x57d37dd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d37dd9, SUC_lexicon_tags_35 },
    { 0x1ff37dda, SUC_lexicon_tags_15 },
    { 0xf3337dd9, SUC_lexicon_tags_43 },
    { 0xe6137ddc, SUC_lexicon_tags_2 },
    { 0x19737ddd, SUC_lexicon_tags_63 },
    { 0x4fb37ddd, SUC_lexicon_tags_60 },
    { 0xe5737ddf, SUC_lexicon_tags_15 },
    { 0xdbd37de0, SUC_lexicon_tags_49 },
    { 0xb0537de0, SUC_lexicon_tags_42 },
    { 0xa3737ddc, SUC_lexicon_tags_37 },
    { 0xe6f37de3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe5337de5, SUC_lexicon_tags_7 },
    { 0xecf37de5, SUC_lexicon_tags_25 },
    { 0xe1d37de7, SUC_lexicon_tags_37 },
    { 0xc5537de8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12137dea, SUC_lexicon_tags_11 },
    { 0xfdf37deb, SUC_lexicon_tags_47 },
    { 0x8b737dec, SUC_lexicon_tags_15 },
    { 0xa1b37ded, SUC_lexicon_tags_12 },
    { 0x91f37ded, SUC_lexicon_tags_14 },
    { 0x98337dee, SUC_lexicon_tags_0 },
    { 0xc1937dec, SUC_lexicon_tags_37 },
    { 0x46137df1, SUC_lexicon_tags_8 },
    { 0x7c737df1, SUC_lexicon_tags_87 },
    { 0xe2937df3, SUC_lexicon_tags_107 },
    { 0x6f137df4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf37dfa, SUC_lexicon_tags_48 },
    { 0xccf37dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b37dfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db37e02, SUC_lexicon_tags_7 },
    { 0x6b537e02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56737e07, SUC_lexicon_tags_23 },
    { 0xa7737e07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db37e0c, SUC_lexicon_tags_29 },
    { 0xf7b37e0c, SUC_lexicon_tags_12 },
    { 0x6fb37e0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87937e12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4737e17, SUC_lexicon_tags_1 },
    { 0x19937e17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a937e1d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf537e20, SUC_lexicon_tags_6 },
    { 0xe0737e20, SUC_lexicon_tags_82 },
    { 0xcff37e22, SUC_lexicon_tags_1 },
    { 0x83f37e22, SUC_lexicon_tags_37 },
    { 0x2e337e22, SUC_lexicon_tags_15 },
    { 0xaad37e25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8737e28, SUC_lexicon_tags_2 },
    { 0xf0f37e29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1937e2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x21737e2e, SUC_lexicon_tags_26 },
    { 0x9f737e2f, SUC_lexicon_tags_47 },
    { 0x87b37e2f, SUC_lexicon_tags_17 },
    { 0xa3d37e31, SUC_lexicon_tags_45 },
    { 0x86537e31, SUC_lexicon_tags_23 },
    { 0x13937e33, SUC_lexicon_tags_19 },
    { 0xcf137e34, SUC_lexicon_tags_34 },
    { 0xcf37e33, SUC_lexicon_tags_47 },
    { 0x6e137e36, SUC_lexicon_tags_28 },
    { 0x33f37e37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c737e39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf737e3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d37e41, SUC_lexicon_tags_160 },
    { 0x4d537e42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f37e48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb37e4c, SUC_lexicon_tags_37 },
    { 0x8f137e4c, SUC_lexicon_tags_3 },
    { 0x9bf37e4e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e537e55, SUC_lexicon_tags_37 },
    { 0x2b937e56, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdad37e58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4d37e5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf37e5f, SUC_lexicon_tags_14 },
    { 0x60737e5f, SUC_lexicon_tags_31 },
    { 0xe0537e61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7d37e63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cd37e65, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x78137e67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5737e6b, SUC_lexicon_tags_7 },
    { 0x4f337e6c, SUC_lexicon_tags_35 },
    { 0xabd37e6b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb937e73, SUC_lexicon_tags_15 },
    { 0x49137e74, SUC_lexicon_tags_23 },
    { 0x2a937e73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39737e81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9d37e83, SUC_lexicon_tags_17 },
    { 0x49537e84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e537e86, SUC_lexicon_tags_2 },
    { 0x24537e86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48937e8a, SUC_lexicon_tags_1 },
    { 0x84f37e8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8d37e8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58137e90, SUC_lexicon_tags_45 },
    { 0x19737e90, SUC_lexicon_tags_10 },
    { 0x41537e92, SUC_lexicon_tags_38 },
    { 0x59937e93, SUC_lexicon_tags_12 },
    { 0xf6f37e94, SUC_lexicon_tags_2 },
    { 0xba737e93, SUC_lexicon_tags_12 },
    { 0x4f737e93, SUC_lexicon_tags_2 },
    { 0x72d37e94, SUC_lexicon_tags_11 },
    { 0xd1337e94, SUC_lexicon_tags_38 },
    { 0xa4137e99, SUC_lexicon_tags_47 },
    { 0x81737e9a, SUC_lexicon_tags_17 },
    { 0x21f37e99, SUC_lexicon_tags_1 },
    { 0x5937e9c, SUC_lexicon_tags_28 },
    { 0x66b37e99, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c537ea1, SUC_lexicon_tags_11 },
    { 0x73337ea1, SUC_lexicon_tags_35 },
    { 0xc1b37ea3, SUC_lexicon_tags_29 },
    { 0x2e137ea1, SUC_lexicon_tags_17 },
    { 0x85137ea3, SUC_lexicon_tags_5 },
    { 0x28537ea6, SUC_lexicon_tags_9 },
    { 0xec937ea2, SUC_lexicon_tags_43 },
    { 0x97337ea7, SUC_lexicon_tags_11 },
    { 0xfaf37ea9, SUC_lexicon_tags_0 },
    { 0xba537eaa, SUC_lexicon_tags_29 },
    { 0x65737eab, SUC_lexicon_tags_0 },
    { 0x39337ea9, SUC_lexicon_tags_201 },
    { 0xe0337ead, SUC_lexicon_tags_0 },
    { 0xd9137eac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac337eb6, SUC_lexicon_tags_33 },
    { 0xe337eb7, SUC_lexicon_tags_37 },
    { 0x94937eb8, SUC_lexicon_tags_27 },
    { 0xe5937eb9, SUC_lexicon_tags_8 },
    { 0x9f937eb8, SUC_lexicon_tags_87 },
    { 0xba137ebb, SUC_lexicon_tags_11 },
    { 0x69d37eb9, SUC_lexicon_tags_15 },
    { 0x9a737ebd, SUC_lexicon_tags_17 },
    { 0x74137ebe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x87d37ec0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb37ec4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe8d37ec6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x13137ec9, SUC_lexicon_tags_15 },
    { 0xe9d37eca, SUC_lexicon_tags_11 },
    { 0x68537ecb, SUC_lexicon_tags_29 },
    { 0xbcb37ecb, SUC_lexicon_tags_19 },
    { 0x1a337ecd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84737ed1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8537ed4, SUC_lexicon_tags_30 },
    { 0x10937ed4, SUC_lexicon_tags_17 },
    { 0x59137ed6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40337edb, SUC_lexicon_tags_11 },
    { 0x74f37edc, SUC_lexicon_tags_38 },
    { 0xcf37edd, SUC_lexicon_tags_77 },
    { 0xbef37ede, SUC_lexicon_tags_15 },
    { 0xa1737edd, SUC_lexicon_tags_47 },
    { 0x38d37ee0, SUC_lexicon_tags_68 },
    { 0x96137ee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f37ee6, SUC_lexicon_tags_12 },
    { 0x3c337ee7, SUC_lexicon_tags_15 },
    { 0xdc537ee7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2f37eea, SUC_lexicon_tags_21 },
    { 0x66b37eea, SUC_lexicon_tags_17 },
    { 0xfeb37eea, SUC_lexicon_tags_61 },
    { 0x18137eed, SUC_lexicon_tags_10 },
    { 0xd0337eee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfb37ef0, SUC_lexicon_tags_7 },
    { 0xfb737ef0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b37ef5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6137efa, SUC_lexicon_tags_23 },
    { 0x18937efa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x66337efd, SUC_lexicon_tags_19 },
    { 0xc3737efe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c537f00, SUC_lexicon_tags_19 },
    { 0xbfd37f01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef37f04, SUC_lexicon_tags_11 },
    { 0x5d137f04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f337f09, SUC_lexicon_tags_23 },
    { 0x3c937f0a, SUC_lexicon_tags_84 },
    { 0xde337f0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcf137f0d, SUC_lexicon_tags_29 },
    { 0x2cb37f0e, SUC_lexicon_tags_12 },
    { 0xcbf37f0e, SUC_lexicon_tags_7 },
    { 0xc5737f10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f537f12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8337f17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc537f1a, SUC_lexicon_tags_17 },
    { 0x34737f1b, SUC_lexicon_tags_46 },
    { 0x3f137f1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f37f22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d37f27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0337f2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d37f2f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7f737f31, SUC_lexicon_tags_12 },
    { 0x63337f32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb37f35, SUC_lexicon_tags_12 },
    { 0x7ff37f36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f37f39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd37f3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe3937f41, SUC_lexicon_tags_15 },
    { 0x2ab37f41, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x99537f45, SUC_lexicon_tags_6 },
    { 0x71337f45, SUC_lexicon_tags_12 },
    { 0x76537f45, SUC_lexicon_tags_25 },
    { 0xa4937f47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26d37f4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad37f4d, SUC_lexicon_tags_57 },
    { 0x9a537f4e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73937f54, SUC_lexicon_tags_46 },
    { 0x1c137f55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28737f57, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c937f5a, SUC_lexicon_tags_7 },
    { 0x10137f5b, SUC_lexicon_tags_38 },
    { 0xf2337f5c, SUC_lexicon_tags_46 },
    { 0xe6737f5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47937f66, SUC_lexicon_tags_17 },
    { 0xf8537f67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f37f6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf37f71, SUC_lexicon_tags_23 },
    { 0x2dd37f72, SUC_lexicon_tags_26 },
    { 0x5a337f72, SUC_lexicon_tags_12 },
    { 0xd9937f72, SUC_lexicon_tags_61 },
    { 0x8bf37f75, SUC_lexicon_tags_3 },
    { 0xdd137f76, SUC_lexicon_tags_37 },
    { 0x3fb37f77, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf0137f79, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7f737f7b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc4137f7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f137f83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f37f86, SUC_lexicon_tags_47 },
    { 0xe1537f87, SUC_lexicon_tags_38 },
    { 0xa8b37f88, SUC_lexicon_tags_15 },
    { 0xe7737f89, SUC_lexicon_tags_8 },
    { 0x82937f89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91137f8c, SUC_lexicon_tags_1216 },
    { 0x2ff37f8d, SUC_lexicon_tags_0 },
    { 0x37f37f8e, SUC_lexicon_tags_43 },
    { 0x8d137f8f, SUC_lexicon_tags_68 },
    { 0x35937f8f, SUC_lexicon_tags_25 },
    { 0xd6337f90, SUC_lexicon_tags_105 },
    { 0xdcd37f8c, SUC_lexicon_tags_7 },
    { 0x77537f8c, SUC_lexicon_tags_25 },
    { 0x89337f94, SUC_lexicon_tags_15 },
    { 0xf1d37f95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20137f9d, SUC_lexicon_tags_17 },
    { 0xda937f9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65737fa0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c937fa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb37fab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1337fae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f37fb1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd5f37fb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x50737fb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7937fb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60937fbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf37fc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5337fc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85937fc6, SUC_lexicon_tags_28 },
    { 0xf9537fc7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbe337fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2337fcc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb37fcf, SUC_lexicon_tags_2 },
    { 0xcfd37fd0, SUC_lexicon_tags_45 },
    { 0x8937fd0, SUC_lexicon_tags_37 },
    { 0xfb937fd2, SUC_lexicon_tags_7 },
    { 0xdef37fd1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3537fd6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d37fd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c537fdc, SUC_lexicon_tags_39 },
    { 0x42537fdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d737fdf, SUC_lexicon_tags_1 },
    { 0xe5f37fe0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b137fe3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf37fe7, SUC_lexicon_tags_12 },
    { 0x25d37fe8, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x9b937fea, SUC_lexicon_tags_38 },
    { 0xb0137fea, SUC_lexicon_tags_129 },
    { 0x1a137fec, SUC_lexicon_tags_29 },
    { 0x55d37fed, SUC_lexicon_tags_17 },
    { 0x88537fec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8337ff2, SUC_lexicon_tags_15 },
    { 0x2e937ff2, SUC_lexicon_tags_17 },
    { 0x27f37ff4, SUC_lexicon_tags_63 },
    { 0xbd337ff4, SUC_lexicon_tags_27 },
    { 0xf0f37ff5, SUC_lexicon_tags_55 },
    { 0xeff37ff7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd137ffb, SUC_lexicon_tags_33 },
    { 0x45937ffb, SUC_lexicon_tags_28 },
    { 0x70137ffd, SUC_lexicon_tags_38 },
    { 0x91137ffb, SUC_lexicon_tags_38 },
    { 0x15337ffd, SUC_lexicon_tags_9 },
    { 0x7ad37fff, SUC_lexicon_tags_7 },
    { 0xb4938001, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87538003, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d38009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82f3800b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x86f3800d, SUC_lexicon_tags_11 },
    { 0x1893800d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad38012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f93801a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d3801d, SUC_lexicon_tags_8 },
    { 0xa1b3801e, SUC_lexicon_tags_9 },
    { 0xb593801f, SUC_lexicon_tags_45 },
    { 0x5e93801f, SUC_lexicon_tags_45 },
    { 0x1b738021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8338028, SUC_lexicon_tags_1 },
    { 0x10138028, SUC_lexicon_tags_69 },
    { 0x65d3802a, SUC_lexicon_tags_84 },
    { 0x1e138029, SUC_lexicon_tags_28 },
    { 0x2f73802b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0738030, SUC_lexicon_tags_11 },
    { 0x83d38031, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11138038, SUC_lexicon_tags_33 },
    { 0x74738038, SUC_lexicon_tags_7 },
    { 0xdf33803a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2493803d, SUC_lexicon_tags_49 },
    { 0x5353803e, SUC_lexicon_tags_124 },
    { 0xca93803f, SUC_lexicon_tags_34 },
    { 0xa893803f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2a938042, SUC_lexicon_tags_26 },
    { 0xc8738043, SUC_lexicon_tags_46 },
    { 0x2b938043, SUC_lexicon_tags_81 },
    { 0x3dd38045, SUC_lexicon_tags_45 },
    { 0x49d38046, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad938048, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b3804e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ef38050, SUC_lexicon_tags_37 },
    { 0xf0d38051, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf38055, SUC_lexicon_tags_45 },
    { 0x8ab38056, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d38059, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf3805d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f38062, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b538066, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad38069, SUC_lexicon_tags_47 },
    { 0xd113806a, SUC_lexicon_tags_0 },
    { 0x36f3806a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6d3806d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5938072, SUC_lexicon_tags_254 },
    { 0x7d338072, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57338079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13807d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef38081, SUC_lexicon_tags_25 },
    { 0xd7b38082, SUC_lexicon_tags_68 },
    { 0x4d738081, SUC_lexicon_tags_44 },
    { 0x7d338084, SUC_lexicon_tags_33 },
    { 0x60738085, SUC_lexicon_tags_8 },
    { 0x94d38085, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x31738089, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9873808c, SUC_lexicon_tags_63 },
    { 0x7993808d, SUC_lexicon_tags_151 },
    { 0x9033808d, SUC_lexicon_tags_7 },
    { 0x4253808f, SUC_lexicon_tags_57 },
    { 0xc5b38090, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xecb38092, SUC_lexicon_tags_43 },
    { 0x7eb38092, SUC_lexicon_tags_14 },
    { 0xbd138094, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2538097, SUC_lexicon_tags_103 },
    { 0xdb738098, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc693809a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b3809d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef380a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d380a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b380ae, SUC_lexicon_tags_64 },
    { 0xb1f380ae, SUC_lexicon_tags_15 },
    { 0x995380ae, SUC_lexicon_tags_75 },
    { 0x2f1380b1, SUC_lexicon_tags_7 },
    { 0x3c9380b2, SUC_lexicon_tags_6 },
    { 0x6a7380af, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35380b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8df380b9, SUC_lexicon_tags_7 },
    { 0x59f380ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x639380bc, SUC_lexicon_tags_47 },
    { 0xdd5380bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x993380bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337380c6, SUC_lexicon_tags_48 },
    { 0x397380c6, SUC_lexicon_tags_10 },
    { 0xc77380c6, SUC_lexicon_tags_12 },
    { 0xd51380c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5380cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaa5380d1, SUC_lexicon_tags_19 },
    { 0x9c9380d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67380d7, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7380dc, SUC_lexicon_tags_28 },
    { 0xcf5380dc, SUC_lexicon_tags_47 },
    { 0x9ef380de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed380e4, SUC_lexicon_tags_29 },
    { 0xda9380e5, SUC_lexicon_tags_248 },
    { 0, NULL },
    { 0x641380e7, SUC_lexicon_tags_12 },
    { 0xce9380e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb93380ea, SUC_lexicon_tags_47 },
    { 0x87d380ea, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97380f0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x38b380f2, SUC_lexicon_tags_60 },
    { 0x3df380f3, SUC_lexicon_tags_37 },
    { 0xf17380f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69380f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b38103, SUC_lexicon_tags_15 },
    { 0x30338104, SUC_lexicon_tags_11 },
    { 0xf9b38104, SUC_lexicon_tags_7 },
    { 0x41b38104, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3738109, SUC_lexicon_tags_343 },
    { 0xa233810a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0938113, SUC_lexicon_tags_0 },
    { 0x42b38113, SUC_lexicon_tags_29 },
    { 0xf9138115, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc313811a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc913811c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd3811f, SUC_lexicon_tags_613 },
    { 0, NULL },
    { 0x98d38121, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e738125, SUC_lexicon_tags_25 },
    { 0xc3b38125, SUC_lexicon_tags_23 },
    { 0x17738125, SUC_lexicon_tags_29 },
    { 0xc7738128, SUC_lexicon_tags_33 },
    { 0xa2738129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb38130, SUC_lexicon_tags_28 },
    { 0xab938131, SUC_lexicon_tags_174 },
    { 0xa7338131, SUC_lexicon_tags_31 },
    { 0xdd938133, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e738139, SUC_lexicon_tags_11 },
    { 0x5e53813a, SUC_lexicon_tags_68 },
    { 0xdb33813b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe333813f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd38145, SUC_lexicon_tags_48 },
    { 0x2c138146, SUC_lexicon_tags_28 },
    { 0x40538147, SUC_lexicon_tags_25 },
    { 0xf0938148, SUC_lexicon_tags_27 },
    { 0x89538147, SUC_lexicon_tags_15 },
    { 0x58d3814a, SUC_lexicon_tags_17 },
    { 0xe8b38146, SUC_lexicon_tags_45 },
    { 0x5cf38146, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5938153, SUC_lexicon_tags_7 },
    { 0x93938154, SUC_lexicon_tags_49 },
    { 0xcc738155, SUC_lexicon_tags_17 },
    { 0xa5938153, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d3815a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dd3815c, SUC_lexicon_tags_22 },
    { 0x4df3815c, SUC_lexicon_tags_7 },
    { 0x6313815e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc4138160, SUC_lexicon_tags_55 },
    { 0xf7938161, SUC_lexicon_tags_11 },
    { 0x4f38162, SUC_lexicon_tags_12 },
    { 0x27938163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff38167, SUC_lexicon_tags_23 },
    { 0x1cd38168, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d3816b, SUC_lexicon_tags_62 },
    { 0x5f53816b, SUC_lexicon_tags_55 },
    { 0x2273816d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d38170, SUC_lexicon_tags_11 },
    { 0xa2938171, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3cf38173, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8738175, SUC_lexicon_tags_17 },
    { 0x18538176, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc338178, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc233817b, SUC_lexicon_tags_11 },
    { 0xdd3817c, SUC_lexicon_tags_7 },
    { 0xd1b3817d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdd3817f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73d38181, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xed938184, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2138187, SUC_lexicon_tags_47 },
    { 0x24b38188, SUC_lexicon_tags_11 },
    { 0x6cf38189, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9933818b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb813818d, SUC_lexicon_tags_30 },
    { 0x2d53818e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0738191, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7938195, SUC_lexicon_tags_1 },
    { 0xd738196, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1353819c, SUC_lexicon_tags_794 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb381a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb2b381a3, SUC_lexicon_tags_81 },
    { 0xec1381a3, SUC_lexicon_tags_17 },
    { 0x18d381a4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1381ab, SUC_lexicon_tags_21 },
    { 0xe19381ac, SUC_lexicon_tags_7 },
    { 0x28b381ac, SUC_lexicon_tags_39 },
    { 0x65d381ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x683381b0, SUC_lexicon_tags_63 },
    { 0x947381b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b381b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x85f381b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7381bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d381bf, SUC_lexicon_tags_80 },
    { 0x6ef381bf, SUC_lexicon_tags_84 },
    { 0x80b381c1, SUC_lexicon_tags_30 },
    { 0x625381c2, SUC_lexicon_tags_58 },
    { 0x6c3381c1, SUC_lexicon_tags_43 },
    { 0x9a5381c1, SUC_lexicon_tags_31 },
    { 0x5e9381c2, SUC_lexicon_tags_140 },
    { 0xc1b381c3, SUC_lexicon_tags_13 },
    { 0x471381c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x993381ca, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1381ce, SUC_lexicon_tags_7 },
    { 0xb3f381cf, SUC_lexicon_tags_32 },
    { 0x2c1381cf, SUC_lexicon_tags_28 },
    { 0xba7381d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x703381d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x445381d6, SUC_lexicon_tags_16 },
    { 0x471381d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9381de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x675381e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8b381e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd9381e4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69381e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369381f3, SUC_lexicon_tags_33 },
    { 0xcbd381f4, SUC_lexicon_tags_37 },
    { 0xdd1381f5, SUC_lexicon_tags_36 },
    { 0x687381f6, SUC_lexicon_tags_6 },
    { 0x55d381f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3381fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05381fe, SUC_lexicon_tags_31 },
    { 0x435381fe, SUC_lexicon_tags_1 },
    { 0x69d381ff, SUC_lexicon_tags_30 },
    { 0xcb381ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb38204, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x16738207, SUC_lexicon_tags_26 },
    { 0x68938208, SUC_lexicon_tags_11 },
    { 0x71b38209, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3393820b, SUC_lexicon_tags_37 },
    { 0x9bd3820c, SUC_lexicon_tags_63 },
    { 0xf293820c, SUC_lexicon_tags_68 },
    { 0x4cd3820e, SUC_lexicon_tags_37 },
    { 0xefd3820f, SUC_lexicon_tags_37 },
    { 0xce13820e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7f38212, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7738215, SUC_lexicon_tags_49 },
    { 0x76b38215, SUC_lexicon_tags_32 },
    { 0x68d38217, SUC_lexicon_tags_31 },
    { 0xdcf38215, SUC_lexicon_tags_30 },
    { 0x5ef38219, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9053821c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f38225, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97938227, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f3822a, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab3822f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa938232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b93823a, SUC_lexicon_tags_1 },
    { 0x8953823a, SUC_lexicon_tags_151 },
    { 0xc273823c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b3823f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74138243, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc538246, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f73824d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7613824f, SUC_lexicon_tags_12 },
    { 0xb573824f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7138253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0738255, SUC_lexicon_tags_37 },
    { 0x8b938255, SUC_lexicon_tags_19 },
    { 0x46338256, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e93825b, SUC_lexicon_tags_7 },
    { 0x9f33825c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69338261, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d93826e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d38272, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f38279, SUC_lexicon_tags_2 },
    { 0x56138279, SUC_lexicon_tags_17 },
    { 0xeb93827b, SUC_lexicon_tags_57 },
    { 0x8f93827c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc938281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef38285, SUC_lexicon_tags_1 },
    { 0xc8938285, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3828c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f38290, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc538297, SUC_lexicon_tags_51 },
    { 0x7ad38298, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x49b3829a, SUC_lexicon_tags_0 },
    { 0x7cb3829b, SUC_lexicon_tags_7 },
    { 0xc073829a, SUC_lexicon_tags_0 },
    { 0xea53829c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcaf3829f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20f382a1, SUC_lexicon_tags_7 },
    { 0xb6d382a2, SUC_lexicon_tags_9 },
    { 0x10f382a2, SUC_lexicon_tags_37 },
    { 0x549382a4, SUC_lexicon_tags_7 },
    { 0xda5382a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x895382a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9382ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff382b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x973382b5, SUC_lexicon_tags_15 },
    { 0xc1f382b6, SUC_lexicon_tags_9 },
    { 0x239382b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d382bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f1382be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d382c7, SUC_lexicon_tags_25 },
    { 0x5f5382c7, SUC_lexicon_tags_46 },
    { 0x7d382c9, SUC_lexicon_tags_7 },
    { 0xca5382ca, SUC_lexicon_tags_7 },
    { 0x2df382cb, SUC_lexicon_tags_45 },
    { 0xcf5382cb, SUC_lexicon_tags_47 },
    { 0x48f382c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd27382cf, SUC_lexicon_tags_13 },
    { 0xfe9382d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d382d4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37382da, SUC_lexicon_tags_13 },
    { 0x551382db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb6d382dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98d382df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1382e2, SUC_lexicon_tags_38 },
    { 0x467382e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x885382e5, SUC_lexicon_tags_37 },
    { 0x93382e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b382e9, SUC_lexicon_tags_23 },
    { 0xeb9382ea, SUC_lexicon_tags_15 },
    { 0x96b382ea, SUC_lexicon_tags_47 },
    { 0x2e1382ec, SUC_lexicon_tags_30 },
    { 0x8e5382ec, SUC_lexicon_tags_2 },
    { 0x683382ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d382f3, SUC_lexicon_tags_47 },
    { 0x903382f4, SUC_lexicon_tags_7 },
    { 0x77f382f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af382f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f382fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb38303, SUC_lexicon_tags_11 },
    { 0x83938304, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f38308, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b53830e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f538319, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf333831e, SUC_lexicon_tags_25 },
    { 0xdb33831f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82738324, SUC_lexicon_tags_25 },
    { 0xe0938325, SUC_lexicon_tags_19 },
    { 0xd0d38326, SUC_lexicon_tags_37 },
    { 0xea338327, SUC_lexicon_tags_7 },
    { 0x72338326, SUC_lexicon_tags_11 },
    { 0xfb738329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d3832d, SUC_lexicon_tags_6 },
    { 0x9d53832e, SUC_lexicon_tags_328 },
    { 0, NULL },
    { 0xad338330, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef938332, SUC_lexicon_tags_0 },
    { 0x79b38332, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d38339, SUC_lexicon_tags_26 },
    { 0x493833a, SUC_lexicon_tags_1 },
    { 0xd0b3833b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26538342, SUC_lexicon_tags_1 },
    { 0x3ab38342, SUC_lexicon_tags_15 },
    { 0x47738344, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a13834a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9fb3834c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab738350, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf738354, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb38357, SUC_lexicon_tags_11 },
    { 0x41138358, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b3835b, SUC_lexicon_tags_13 },
    { 0x3353835b, SUC_lexicon_tags_48 },
    { 0x7033835d, SUC_lexicon_tags_11 },
    { 0x2f53835e, SUC_lexicon_tags_15 },
    { 0xed3835d, SUC_lexicon_tags_39 },
    { 0xde73835d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x40538363, SUC_lexicon_tags_37 },
    { 0x48338364, SUC_lexicon_tags_31 },
    { 0x54338365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5938369, SUC_lexicon_tags_43 },
    { 0x2553836a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf13836c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec338373, SUC_lexicon_tags_0 },
    { 0xdfd38374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf38377, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d3837c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30538380, SUC_lexicon_tags_29 },
    { 0x99d38381, SUC_lexicon_tags_0 },
    { 0x5b538382, SUC_lexicon_tags_29 },
    { 0x5138382, SUC_lexicon_tags_17 },
    { 0x96f38384, SUC_lexicon_tags_27 },
    { 0xb3138385, SUC_lexicon_tags_1 },
    { 0x13938385, SUC_lexicon_tags_60 },
    { 0xdab38382, SUC_lexicon_tags_1623 },
    { 0xbe538384, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1973838c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d3838f, SUC_lexicon_tags_17 },
    { 0x5773838f, SUC_lexicon_tags_52 },
    { 0xaad38390, SUC_lexicon_tags_17 },
    { 0x1353838f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8338396, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233839a, SUC_lexicon_tags_46 },
    { 0x6db3839b, SUC_lexicon_tags_37 },
    { 0x12f3839a, SUC_lexicon_tags_47 },
    { 0xa53839b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf073839f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x959383a2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x153383a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd37383a6, SUC_lexicon_tags_11 },
    { 0xdaf383a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3383ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b383b3, SUC_lexicon_tags_11 },
    { 0xcb7383b4, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69383bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f383bf, SUC_lexicon_tags_21 },
    { 0xc51383bf, SUC_lexicon_tags_7 },
    { 0x51f383c1, SUC_lexicon_tags_17 },
    { 0xbe1383c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x82b383c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x783383c6, SUC_lexicon_tags_6 },
    { 0x3d1383c6, SUC_lexicon_tags_7 },
    { 0xd59383c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x313383ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x227383cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7383d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb25383d2, SUC_lexicon_tags_65 },
    { 0x4e7383d3, SUC_lexicon_tags_15 },
    { 0xcd9383d2, SUC_lexicon_tags_1507 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89383dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd383df, SUC_lexicon_tags_11 },
    { 0xff3383df, SUC_lexicon_tags_9 },
    { 0xd3f383e0, SUC_lexicon_tags_13 },
    { 0x37d383e2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f383e5, SUC_lexicon_tags_92 },
    { 0x741383e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b383ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507383ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3383f1, SUC_lexicon_tags_1 },
    { 0x1e3383f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ab383f4, SUC_lexicon_tags_33 },
    { 0xc37383f5, SUC_lexicon_tags_69 },
    { 0xa1b383f5, SUC_lexicon_tags_2 },
    { 0xc75383f7, SUC_lexicon_tags_70 },
    { 0x4df383f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7a9383fa, SUC_lexicon_tags_38 },
    { 0xcdd383fb, SUC_lexicon_tags_47 },
    { 0xd59383fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1383ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4738403, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d38405, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6f38407, SUC_lexicon_tags_6 },
    { 0x39f38408, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d33840a, SUC_lexicon_tags_37 },
    { 0x2793840b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8093840e, SUC_lexicon_tags_14 },
    { 0xa713840f, SUC_lexicon_tags_49 },
    { 0xbdf38410, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49138414, SUC_lexicon_tags_15 },
    { 0x88d38415, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbe738417, SUC_lexicon_tags_17 },
    { 0xd0538417, SUC_lexicon_tags_38 },
    { 0x59538418, SUC_lexicon_tags_28 },
    { 0xf4d3841a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4933841f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x54538421, SUC_lexicon_tags_19 },
    { 0x47338421, SUC_lexicon_tags_2 },
    { 0x9c538423, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef38426, SUC_lexicon_tags_108 },
    { 0x35f38427, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb3842d, SUC_lexicon_tags_37 },
    { 0x5f13842e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6538432, SUC_lexicon_tags_0 },
    { 0x6e738433, SUC_lexicon_tags_37 },
    { 0x9eb38434, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1b38436, SUC_lexicon_tags_17 },
    { 0x26138437, SUC_lexicon_tags_30 },
    { 0x54138438, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2153843b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b3843f, SUC_lexicon_tags_12 },
    { 0x7813843f, SUC_lexicon_tags_1 },
    { 0x97938440, SUC_lexicon_tags_26 },
    { 0xaad38440, SUC_lexicon_tags_19 },
    { 0x5ab38443, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb38445, SUC_lexicon_tags_37 },
    { 0x71538446, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74738449, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2873844b, SUC_lexicon_tags_11 },
    { 0x94d3844b, SUC_lexicon_tags_61 },
    { 0x65d3844d, SUC_lexicon_tags_25 },
    { 0xb653844e, SUC_lexicon_tags_14 },
    { 0x89b3844e, SUC_lexicon_tags_7 },
    { 0x7853844e, SUC_lexicon_tags_15 },
    { 0xa613844e, SUC_lexicon_tags_17 },
    { 0xaef38452, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a738457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd3845d, SUC_lexicon_tags_37 },
    { 0x87f3845e, SUC_lexicon_tags_37 },
    { 0x9573845e, SUC_lexicon_tags_8 },
    { 0xc5d3845e, SUC_lexicon_tags_17 },
    { 0x9fd38461, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb338465, SUC_lexicon_tags_23 },
    { 0xbcf38466, SUC_lexicon_tags_25 },
    { 0x31938467, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1ab38469, SUC_lexicon_tags_0 },
    { 0x2cd3846a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf633846d, SUC_lexicon_tags_2 },
    { 0xa393846e, SUC_lexicon_tags_23 },
    { 0x24b3846d, SUC_lexicon_tags_15 },
    { 0x1d538470, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2938475, SUC_lexicon_tags_37 },
    { 0xdff38476, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x76738479, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f38480, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23338485, SUC_lexicon_tags_32 },
    { 0xb6938486, SUC_lexicon_tags_37 },
    { 0x1f738485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f3848e, SUC_lexicon_tags_30 },
    { 0xc8f3848f, SUC_lexicon_tags_12 },
    { 0x6b338490, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d738493, SUC_lexicon_tags_26 },
    { 0xf38494, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c338499, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef3849e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe93384a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x203384a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745384a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeef384a9, SUC_lexicon_tags_11 },
    { 0x9d9384aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f384ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d9384b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5384b5, SUC_lexicon_tags_17 },
    { 0xf33384b6, SUC_lexicon_tags_11 },
    { 0xf0d384b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07384bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa43384bf, SUC_lexicon_tags_37 },
    { 0x50f384c0, SUC_lexicon_tags_43 },
    { 0x39d384c1, SUC_lexicon_tags_33 },
    { 0xa83384c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab384c5, SUC_lexicon_tags_63 },
    { 0xcf1384c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x101384c9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x92f384cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53384d1, SUC_lexicon_tags_11 },
    { 0x3d1384d1, SUC_lexicon_tags_4 },
    { 0x469384d3, SUC_lexicon_tags_12 },
    { 0x631384d4, SUC_lexicon_tags_57 },
    { 0xe5f384d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547384d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf384e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x44d384e6, SUC_lexicon_tags_8 },
    { 0x9bb384e6, SUC_lexicon_tags_45 },
    { 0xe29384e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25d384ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6d384ec, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65384ef, SUC_lexicon_tags_2 },
    { 0xfd7384f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe91384f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd384f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x77b384fa, SUC_lexicon_tags_17 },
    { 0xacd384fa, SUC_lexicon_tags_98 },
    { 0x345384fc, SUC_lexicon_tags_25 },
    { 0xe81384fd, SUC_lexicon_tags_17 },
    { 0x655384fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8338503, SUC_lexicon_tags_129 },
    { 0xdcd38504, SUC_lexicon_tags_1 },
    { 0x23f38503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb753850b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5fb3850d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51938511, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e338516, SUC_lexicon_tags_27 },
    { 0x49538517, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2cb38519, SUC_lexicon_tags_47 },
    { 0x2993851a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9073851e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3df38520, SUC_lexicon_tags_9 },
    { 0x8c338521, SUC_lexicon_tags_25 },
    { 0x78538522, SUC_lexicon_tags_0 },
    { 0x7cd38521, SUC_lexicon_tags_37 },
    { 0xb7938521, SUC_lexicon_tags_32 },
    { 0x6fd38525, SUC_lexicon_tags_11 },
    { 0xbad38525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69f38528, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50d3852a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb38536, SUC_lexicon_tags_40 },
    { 0x3d738537, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b938539, SUC_lexicon_tags_1 },
    { 0xda53853a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78338542, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe338545, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c738548, SUC_lexicon_tags_15 },
    { 0xc738549, SUC_lexicon_tags_7 },
    { 0xa93854a, SUC_lexicon_tags_84 },
    { 0xc4f38548, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe513854f, SUC_lexicon_tags_1 },
    { 0x2f738550, SUC_lexicon_tags_11 },
    { 0xed138551, SUC_lexicon_tags_15 },
    { 0xef938552, SUC_lexicon_tags_31 },
    { 0xbcf38552, SUC_lexicon_tags_23 },
    { 0xa5538554, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d3855b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x9433855e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e138560, SUC_lexicon_tags_37 },
    { 0x29d38560, SUC_lexicon_tags_63 },
    { 0xa8338562, SUC_lexicon_tags_31 },
    { 0xdd138562, SUC_lexicon_tags_45 },
    { 0xb1b38562, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x35b38566, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3938569, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ab3856b, SUC_lexicon_tags_2 },
    { 0xb733856c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd38576, SUC_lexicon_tags_39 },
    { 0x95d38577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba938585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d338589, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b33858d, SUC_lexicon_tags_25 },
    { 0xad93858e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82738590, SUC_lexicon_tags_1 },
    { 0x68b38590, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2938593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2df38595, SUC_lexicon_tags_14 },
    { 0x8c538596, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8eb38598, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b3859c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f385a0, SUC_lexicon_tags_31 },
    { 0xb9d385a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd385a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7385a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3385b2, SUC_lexicon_tags_37 },
    { 0xd7b385b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183385ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe07385bc, SUC_lexicon_tags_23 },
    { 0x963385bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x657385bf, SUC_lexicon_tags_17 },
    { 0x795385bf, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa1b385c2, SUC_lexicon_tags_38 },
    { 0xdbf385c3, SUC_lexicon_tags_7 },
    { 0xbf3385c3, SUC_lexicon_tags_48 },
    { 0x7f9385c4, SUC_lexicon_tags_17 },
    { 0xe95385c5, SUC_lexicon_tags_2 },
    { 0xcb3385c7, SUC_lexicon_tags_63 },
    { 0xf15385c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa6f385ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x421385cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88b385cf, SUC_lexicon_tags_37 },
    { 0x8b1385d0, SUC_lexicon_tags_47 },
    { 0xfe1385d1, SUC_lexicon_tags_2 },
    { 0xa59385d2, SUC_lexicon_tags_37 },
    { 0xe87385d3, SUC_lexicon_tags_31 },
    { 0xb3385d4, SUC_lexicon_tags_17 },
    { 0xc67385d5, SUC_lexicon_tags_25 },
    { 0xa57385d5, SUC_lexicon_tags_11 },
    { 0x261385d5, SUC_lexicon_tags_0 },
    { 0x18b385d8, SUC_lexicon_tags_38 },
    { 0x37b385d7, SUC_lexicon_tags_133 },
    { 0xf91385d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9c7385dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f385e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x203385e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc5385e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701385ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x111385ee, SUC_lexicon_tags_11 },
    { 0x21385ee, SUC_lexicon_tags_19 },
    { 0xe1385f0, SUC_lexicon_tags_49 },
    { 0xe7d385f1, SUC_lexicon_tags_11 },
    { 0x3f3385f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695385f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49385fa, SUC_lexicon_tags_12 },
    { 0x70d385fb, SUC_lexicon_tags_37 },
    { 0x9ff385fc, SUC_lexicon_tags_14 },
    { 0x63f385fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7138608, SUC_lexicon_tags_22 },
    { 0xb9738609, SUC_lexicon_tags_77 },
    { 0x9cb38608, SUC_lexicon_tags_11 },
    { 0x12538608, SUC_lexicon_tags_87 },
    { 0x44d3860c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf38611, SUC_lexicon_tags_12 },
    { 0x99d38611, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a938614, SUC_lexicon_tags_46 },
    { 0x1fd38614, SUC_lexicon_tags_17 },
    { 0x23d38616, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f3861a, SUC_lexicon_tags_30 },
    { 0x4273861b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x34f3861d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb113861f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3538621, SUC_lexicon_tags_37 },
    { 0x3d138622, SUC_lexicon_tags_29 },
    { 0xdc538621, SUC_lexicon_tags_63 },
    { 0x8ef38624, SUC_lexicon_tags_7 },
    { 0xec738624, SUC_lexicon_tags_17 },
    { 0x9b738621, SUC_lexicon_tags_2 },
    { 0x54138623, SUC_lexicon_tags_23 },
    { 0xa938625, SUC_lexicon_tags_49 },
    { 0xa1338629, SUC_lexicon_tags_77 },
    { 0x44938629, SUC_lexicon_tags_32 },
    { 0x7f13862b, SUC_lexicon_tags_4 },
    { 0xc4b3862a, SUC_lexicon_tags_37 },
    { 0x93338629, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61738631, SUC_lexicon_tags_47 },
    { 0x1938631, SUC_lexicon_tags_48 },
    { 0x91938631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7538635, SUC_lexicon_tags_60 },
    { 0xe9f38636, SUC_lexicon_tags_63 },
    { 0x83938637, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95538639, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b3863c, SUC_lexicon_tags_56 },
    { 0x29b3863d, SUC_lexicon_tags_17 },
    { 0xfb93863d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcdd38640, SUC_lexicon_tags_17 },
    { 0x3c738640, SUC_lexicon_tags_53 },
    { 0xb38642, SUC_lexicon_tags_46 },
    { 0x22138643, SUC_lexicon_tags_191 },
    { 0xd5138642, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x60938647, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5538649, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6353864b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x81b3864d, SUC_lexicon_tags_47 },
    { 0xf713864e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a738652, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f3865a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf3865d, SUC_lexicon_tags_8 },
    { 0x97d3865d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d38661, SUC_lexicon_tags_29 },
    { 0xc5538662, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a938664, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3938667, SUC_lexicon_tags_68 },
    { 0xcfd38668, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf733866a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1eb3866c, SUC_lexicon_tags_6 },
    { 0x3333866c, SUC_lexicon_tags_15 },
    { 0x5b53866e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68338671, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e538674, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6193867a, SUC_lexicon_tags_4 },
    { 0xdc13867b, SUC_lexicon_tags_37 },
    { 0x8893867b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff38681, SUC_lexicon_tags_53 },
    { 0xd6538681, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b3868f, SUC_lexicon_tags_12 },
    { 0x7573868f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6138692, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x88138694, SUC_lexicon_tags_7 },
    { 0x83738694, SUC_lexicon_tags_38 },
    { 0x2a138694, SUC_lexicon_tags_38 },
    { 0xef138697, SUC_lexicon_tags_60 },
    { 0x10b38698, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7933869a, SUC_lexicon_tags_26 },
    { 0x96d3869a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa093869d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3386a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xedf386a2, SUC_lexicon_tags_9 },
    { 0x7dd386a3, SUC_lexicon_tags_70 },
    { 0xe3f386a4, SUC_lexicon_tags_77 },
    { 0x42f386a2, SUC_lexicon_tags_11 },
    { 0xf33386a2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37386aa, SUC_lexicon_tags_7 },
    { 0x419386aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615386af, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x501386b1, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0xcff386b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d386b8, SUC_lexicon_tags_43 },
    { 0x69386b9, SUC_lexicon_tags_15 },
    { 0x4bd386ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11386bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f9386bf, SUC_lexicon_tags_7 },
    { 0x591386c0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x309386c2, SUC_lexicon_tags_9 },
    { 0xf9f386c2, SUC_lexicon_tags_25 },
    { 0x499386c4, SUC_lexicon_tags_7 },
    { 0xe95386c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1386c8, SUC_lexicon_tags_53 },
    { 0xe03386c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2f7386cb, SUC_lexicon_tags_7 },
    { 0xe0b386cc, SUC_lexicon_tags_25 },
    { 0x687386cb, SUC_lexicon_tags_38 },
    { 0x231386ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc5386d0, SUC_lexicon_tags_293 },
    { 0x2cb386d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23386d6, SUC_lexicon_tags_30 },
    { 0x8f5386d7, SUC_lexicon_tags_5 },
    { 0xc77386d8, SUC_lexicon_tags_37 },
    { 0x807386d9, SUC_lexicon_tags_12 },
    { 0xae7386d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61386e0, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9386e4, SUC_lexicon_tags_14 },
    { 0xaa9386e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ab386e7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1d386e9, SUC_lexicon_tags_4 },
    { 0x49f386ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdef386ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe95386ee, SUC_lexicon_tags_12 },
    { 0xb1d386ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xecf386f1, SUC_lexicon_tags_37 },
    { 0x7ad386f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9386f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdd386f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x761386fa, SUC_lexicon_tags_29 },
    { 0xad386fa, SUC_lexicon_tags_30 },
    { 0xbc9386fc, SUC_lexicon_tags_60 },
    { 0xa3386fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b38700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8338702, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b38705, SUC_lexicon_tags_1 },
    { 0x41d38705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde938708, SUC_lexicon_tags_25 },
    { 0xc3138709, SUC_lexicon_tags_29 },
    { 0xbe738709, SUC_lexicon_tags_77 },
    { 0x8b73870a, SUC_lexicon_tags_12 },
    { 0x3193870a, SUC_lexicon_tags_63 },
    { 0x1a93870c, SUC_lexicon_tags_3 },
    { 0x6593870e, SUC_lexicon_tags_12 },
    { 0xc173870f, SUC_lexicon_tags_218 },
    { 0x97738710, SUC_lexicon_tags_37 },
    { 0x70738708, SUC_lexicon_tags_30 },
    { 0xd913870e, SUC_lexicon_tags_23 },
    { 0x2d53870d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b38719, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6cf3871b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xe913871e, SUC_lexicon_tags_39 },
    { 0xbcd3871f, SUC_lexicon_tags_2 },
    { 0x2e33871e, SUC_lexicon_tags_113 },
    { 0x88138721, SUC_lexicon_tags_12 },
    { 0x84b3871e, SUC_lexicon_tags_19 },
    { 0xf933871f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35b38725, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f38729, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd38732, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b38734, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b3873e, SUC_lexicon_tags_72 },
    { 0x71b3873f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57538743, SUC_lexicon_tags_15 },
    { 0xce938744, SUC_lexicon_tags_1 },
    { 0x1dd38744, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x1c338747, SUC_lexicon_tags_12 },
    { 0x33338747, SUC_lexicon_tags_43 },
    { 0xca938749, SUC_lexicon_tags_49 },
    { 0x3c13874a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf7b3874c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd38750, SUC_lexicon_tags_6 },
    { 0x88138751, SUC_lexicon_tags_56 },
    { 0x4df38752, SUC_lexicon_tags_16 },
    { 0x50f38753, SUC_lexicon_tags_1 },
    { 0xd0b38753, SUC_lexicon_tags_56 },
    { 0x9d338755, SUC_lexicon_tags_33 },
    { 0xd8938756, SUC_lexicon_tags_0 },
    { 0x20938757, SUC_lexicon_tags_6 },
    { 0x56138758, SUC_lexicon_tags_2 },
    { 0xaa738753, SUC_lexicon_tags_38 },
    { 0x7ab38752, SUC_lexicon_tags_38 },
    { 0x413875b, SUC_lexicon_tags_16 },
    { 0xc5d3875a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f38766, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff738769, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b3876c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2233876e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa138772, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43138777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6338779, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec73877b, SUC_lexicon_tags_37 },
    { 0x4373877c, SUC_lexicon_tags_14 },
    { 0x59b3877d, SUC_lexicon_tags_16 },
    { 0x2393877d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9338781, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb38783, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f3878d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37b3878f, SUC_lexicon_tags_82 },
    { 0xb9938790, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a138793, SUC_lexicon_tags_48 },
    { 0xdbf38793, SUC_lexicon_tags_1 },
    { 0x2ef38795, SUC_lexicon_tags_14 },
    { 0xb4d38796, SUC_lexicon_tags_198 },
    { 0x7e538796, SUC_lexicon_tags_6 },
    { 0xe2138795, SUC_lexicon_tags_25 },
    { 0xce938798, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d387a0, SUC_lexicon_tags_37 },
    { 0x763387a1, SUC_lexicon_tags_7 },
    { 0xad9387a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed387aa, SUC_lexicon_tags_2 },
    { 0x161387ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7387ae, SUC_lexicon_tags_12 },
    { 0x56d387ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d387b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbcb387b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d387b7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69387ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51387bd, SUC_lexicon_tags_13 },
    { 0x77387be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3387c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95387c7, SUC_lexicon_tags_27 },
    { 0x37f387c8, SUC_lexicon_tags_11 },
    { 0xb95387c9, SUC_lexicon_tags_0 },
    { 0x4a1387ca, SUC_lexicon_tags_17 },
    { 0x6a7387ca, SUC_lexicon_tags_2 },
    { 0x9c9387c7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79387d2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab387d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead387d9, SUC_lexicon_tags_74 },
    { 0xb09387da, SUC_lexicon_tags_437 },
    { 0xa95387db, SUC_lexicon_tags_60 },
    { 0x5ab387da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41387df, SUC_lexicon_tags_43 },
    { 0x4a9387df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85387e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x623387e6, SUC_lexicon_tags_16 },
    { 0x417387e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569387ec, SUC_lexicon_tags_2 },
    { 0xcd387ed, SUC_lexicon_tags_17 },
    { 0x85f387ed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f387f5, SUC_lexicon_tags_11 },
    { 0x103387f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf387fc, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0x3b1387fe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced38802, SUC_lexicon_tags_47 },
    { 0xbd338803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1338806, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x26338809, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4113880d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c738810, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf5138812, SUC_lexicon_tags_55 },
    { 0x8b38813, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b38816, SUC_lexicon_tags_7 },
    { 0x9ff38817, SUC_lexicon_tags_19 },
    { 0x2c138817, SUC_lexicon_tags_55 },
    { 0x89938817, SUC_lexicon_tags_14 },
    { 0x5a93881a, SUC_lexicon_tags_8 },
    { 0x2633881a, SUC_lexicon_tags_4 },
    { 0x7413881a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d3881f, SUC_lexicon_tags_17 },
    { 0xbf938820, SUC_lexicon_tags_66 },
    { 0x9b38821, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5113882c, SUC_lexicon_tags_66 },
    { 0x93d3882d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2338833, SUC_lexicon_tags_11 },
    { 0x4a738834, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8d938836, SUC_lexicon_tags_11 },
    { 0x1c138837, SUC_lexicon_tags_25 },
    { 0x58b38838, SUC_lexicon_tags_7 },
    { 0xe8538839, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a13883d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b3884e, SUC_lexicon_tags_27 },
    { 0x673884f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc538855, SUC_lexicon_tags_45 },
    { 0xa2938856, SUC_lexicon_tags_1 },
    { 0x17138857, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b3885a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b38862, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f138865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f3886f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a538871, SUC_lexicon_tags_28 },
    { 0xcbd38871, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3f38874, SUC_lexicon_tags_95 },
    { 0x99d38875, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd538877, SUC_lexicon_tags_7 },
    { 0x3d938878, SUC_lexicon_tags_17 },
    { 0x3d38879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d3887d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1738880, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad38883, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4ff38885, SUC_lexicon_tags_27 },
    { 0x89138885, SUC_lexicon_tags_34 },
    { 0xd4138885, SUC_lexicon_tags_47 },
    { 0xf3d38888, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad73888e, SUC_lexicon_tags_63 },
    { 0x193888f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d38893, SUC_lexicon_tags_1 },
    { 0x4fd38893, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31338898, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6f3889a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb213889c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6b3889e, SUC_lexicon_tags_25 },
    { 0x8bd3889f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x499388a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x943388a3, SUC_lexicon_tags_25 },
    { 0x353388a3, SUC_lexicon_tags_7 },
    { 0xf81388a4, SUC_lexicon_tags_2 },
    { 0x697388a4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdd5388a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35b388aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x735388ad, SUC_lexicon_tags_32 },
    { 0x8f3388ad, SUC_lexicon_tags_15 },
    { 0x117388ad, SUC_lexicon_tags_15 },
    { 0xbc5388b0, SUC_lexicon_tags_35 },
    { 0x7e7388af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa5388b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf37388b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b388bb, SUC_lexicon_tags_1 },
    { 0x41f388bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d388c2, SUC_lexicon_tags_47 },
    { 0x12b388c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x273388c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc45388c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc79388c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x139388cc, SUC_lexicon_tags_43 },
    { 0x2ed388cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691388d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b388d5, SUC_lexicon_tags_11 },
    { 0xc1b388d5, SUC_lexicon_tags_49 },
    { 0xbfd388d7, SUC_lexicon_tags_25 },
    { 0x6b388d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d388df, SUC_lexicon_tags_3 },
    { 0x32d388e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3388e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x259388e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b388ed, SUC_lexicon_tags_11 },
    { 0xf23388ee, SUC_lexicon_tags_77 },
    { 0x1f3388ee, SUC_lexicon_tags_31 },
    { 0x2d1388f0, SUC_lexicon_tags_37 },
    { 0x6bd388ed, SUC_lexicon_tags_47 },
    { 0x377388ed, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3388fb, SUC_lexicon_tags_31 },
    { 0xab9388fc, SUC_lexicon_tags_25 },
    { 0x3e9388fc, SUC_lexicon_tags_30 },
    { 0x61b388fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3938904, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde938908, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa0b3890a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb693890f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b538913, SUC_lexicon_tags_12 },
    { 0xd9138914, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc738918, SUC_lexicon_tags_14 },
    { 0x49338919, SUC_lexicon_tags_47 },
    { 0xdd13891a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2013891d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97938921, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40138925, SUC_lexicon_tags_9 },
    { 0x24f38925, SUC_lexicon_tags_7 },
    { 0xdf938927, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b38932, SUC_lexicon_tags_63 },
    { 0x4a938933, SUC_lexicon_tags_15 },
    { 0x5b38934, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24938936, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f38939, SUC_lexicon_tags_37 },
    { 0x1ed3893a, SUC_lexicon_tags_9 },
    { 0x60f3893a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e338941, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4338943, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcc538945, SUC_lexicon_tags_7 },
    { 0x3ff38946, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf38949, SUC_lexicon_tags_12 },
    { 0x5a13894a, SUC_lexicon_tags_47 },
    { 0xf3b3894a, SUC_lexicon_tags_8 },
    { 0x8053894a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2138950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f738952, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd738954, SUC_lexicon_tags_15 },
    { 0xcc538955, SUC_lexicon_tags_37 },
    { 0x65738954, SUC_lexicon_tags_2 },
    { 0x5fd38954, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd3895a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef73895c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5138966, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbab38968, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa033896b, SUC_lexicon_tags_11 },
    { 0x9293896b, SUC_lexicon_tags_6 },
    { 0x5ad3896d, SUC_lexicon_tags_25 },
    { 0xb73896b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d338971, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd38979, SUC_lexicon_tags_9 },
    { 0x7753897a, SUC_lexicon_tags_42 },
    { 0x9a93897a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3993897d, SUC_lexicon_tags_39 },
    { 0x9b73897e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a138985, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b3898b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe553898d, SUC_lexicon_tags_63 },
    { 0x8b13898d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf38995, SUC_lexicon_tags_6 },
    { 0xf7b38996, SUC_lexicon_tags_29 },
    { 0x81f38997, SUC_lexicon_tags_1 },
    { 0xe8938998, SUC_lexicon_tags_48 },
    { 0x8d138996, SUC_lexicon_tags_46 },
    { 0xe7b38996, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfd33899c, SUC_lexicon_tags_6 },
    { 0x1033899d, SUC_lexicon_tags_14 },
    { 0xa9d3899e, SUC_lexicon_tags_11 },
    { 0x4f3899f, SUC_lexicon_tags_11 },
    { 0xd2f3899f, SUC_lexicon_tags_1 },
    { 0x9df389a1, SUC_lexicon_tags_9 },
    { 0x55d389a2, SUC_lexicon_tags_12 },
    { 0xfd1389a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e1389a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf23389a7, SUC_lexicon_tags_1 },
    { 0xe65389a8, SUC_lexicon_tags_15 },
    { 0xcb1389a8, SUC_lexicon_tags_7 },
    { 0xa31389aa, SUC_lexicon_tags_17 },
    { 0xfe1389aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75389ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74d389b0, SUC_lexicon_tags_0 },
    { 0x8ad389b0, SUC_lexicon_tags_4 },
    { 0xdd9389b2, SUC_lexicon_tags_11 },
    { 0x985389b2, SUC_lexicon_tags_26 },
    { 0x137389b4, SUC_lexicon_tags_28 },
    { 0x2a1389b3, SUC_lexicon_tags_29 },
    { 0xa89389b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37389bd, SUC_lexicon_tags_19 },
    { 0x219389bd, SUC_lexicon_tags_11 },
    { 0xa05389bf, SUC_lexicon_tags_14 },
    { 0x29f389bf, SUC_lexicon_tags_32 },
    { 0x9c3389c1, SUC_lexicon_tags_28 },
    { 0x761389bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d389c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf91389ca, SUC_lexicon_tags_15 },
    { 0xaf389cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xff7389cd, SUC_lexicon_tags_55 },
    { 0x60b389ce, SUC_lexicon_tags_1 },
    { 0xf73389cf, SUC_lexicon_tags_14 },
    { 0x70d389d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7389db, SUC_lexicon_tags_50 },
    { 0x3a9389dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f389e1, SUC_lexicon_tags_7 },
    { 0x67b389e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x203389e4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f389e9, SUC_lexicon_tags_0 },
    { 0x9c9389e9, SUC_lexicon_tags_55 },
    { 0x1b7389eb, SUC_lexicon_tags_47 },
    { 0x9c3389eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x313389ee, SUC_lexicon_tags_1 },
    { 0xe15389ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949389f4, SUC_lexicon_tags_17 },
    { 0x5f7389f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbe9389f7, SUC_lexicon_tags_37 },
    { 0x6c3389f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b389fb, SUC_lexicon_tags_0 },
    { 0x6d1389fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5938a01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8338a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1538a07, SUC_lexicon_tags_2 },
    { 0x74f38a08, SUC_lexicon_tags_26 },
    { 0x9538a08, SUC_lexicon_tags_63 },
    { 0x3938a0a, SUC_lexicon_tags_293 },
    { 0xea738a0b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58738a0f, SUC_lexicon_tags_27 },
    { 0x21d38a0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab38a13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa938a15, SUC_lexicon_tags_37 },
    { 0x4f538a15, SUC_lexicon_tags_37 },
    { 0xd0d38a17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xff538a19, SUC_lexicon_tags_60 },
    { 0xf4538a1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd38a20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2138a22, SUC_lexicon_tags_1 },
    { 0x4f938a23, SUC_lexicon_tags_37 },
    { 0x60f38a24, SUC_lexicon_tags_113 },
    { 0x43f38a23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d38a28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f38a31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4538a35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb38a38, SUC_lexicon_tags_48 },
    { 0xed138a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed138a3b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e138a3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2538a46, SUC_lexicon_tags_17 },
    { 0xe4338a47, SUC_lexicon_tags_8 },
    { 0x4bb38a47, SUC_lexicon_tags_0 },
    { 0xe9f38a47, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc938a4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8e138a4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc538a52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1738a54, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15538a5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe1738a5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x20538a61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd38a64, SUC_lexicon_tags_37 },
    { 0x49538a65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89d38a67, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4938a6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe2f38a6d, SUC_lexicon_tags_23 },
    { 0x7ab38a6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf38a72, SUC_lexicon_tags_1 },
    { 0x31338a72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89538a78, SUC_lexicon_tags_6 },
    { 0xda138a78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22338a7e, SUC_lexicon_tags_1 },
    { 0x1c138a7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d38a83, SUC_lexicon_tags_12 },
    { 0x95938a84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed38a88, SUC_lexicon_tags_37 },
    { 0x77738a89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63338a91, SUC_lexicon_tags_28 },
    { 0x3db38a92, SUC_lexicon_tags_7 },
    { 0x61f38a92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab38a97, SUC_lexicon_tags_28 },
    { 0x50938a98, SUC_lexicon_tags_182 },
    { 0x91538a99, SUC_lexicon_tags_15 },
    { 0xf4f38a97, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xff138a9c, SUC_lexicon_tags_37 },
    { 0x3d738a9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xead38aa0, SUC_lexicon_tags_37 },
    { 0x3f538aa1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x13f38aa3, SUC_lexicon_tags_49 },
    { 0xa8f38aa4, SUC_lexicon_tags_6 },
    { 0xbbd38aa4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6938aa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4938aab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7bd38aad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x26738aaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d38ab4, SUC_lexicon_tags_47 },
    { 0x5338ab5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d38aba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba538abd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d38ac0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e938ac4, SUC_lexicon_tags_11 },
    { 0xdab38ac4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27b38ac7, SUC_lexicon_tags_31 },
    { 0x5a138ac8, SUC_lexicon_tags_68 },
    { 0x14b38ac9, SUC_lexicon_tags_17 },
    { 0x7d338ac8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2f38acc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8138ace, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a538ad2, SUC_lexicon_tags_61 },
    { 0x94f38ad3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x71f38ad5, SUC_lexicon_tags_15 },
    { 0xb938ad6, SUC_lexicon_tags_7 },
    { 0xc9f38ad7, SUC_lexicon_tags_34 },
    { 0x8e538ad8, SUC_lexicon_tags_9 },
    { 0x6a938ad9, SUC_lexicon_tags_33 },
    { 0x88138ad9, SUC_lexicon_tags_50 },
    { 0x6e738ada, SUC_lexicon_tags_27 },
    { 0xc1538adb, SUC_lexicon_tags_17 },
    { 0xf8338add, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x90938adf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b38ae4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47538aee, SUC_lexicon_tags_29 },
    { 0x4f38aef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b138af5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65738af7, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7738afd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f538b05, SUC_lexicon_tags_1 },
    { 0x9bd38b06, SUC_lexicon_tags_25 },
    { 0x6e538b07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18138b0d, SUC_lexicon_tags_2 },
    { 0x9ad38b0e, SUC_lexicon_tags_25 },
    { 0xfa738b0f, SUC_lexicon_tags_37 },
    { 0x6c738b10, SUC_lexicon_tags_30 },
    { 0xefd38b11, SUC_lexicon_tags_12 },
    { 0x4dd38b0d, SUC_lexicon_tags_9 },
    { 0xeb538b13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9538b20, SUC_lexicon_tags_7 },
    { 0x6ad38b21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab38b25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7138b28, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae338b2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d938b2e, SUC_lexicon_tags_0 },
    { 0x83d38b2f, SUC_lexicon_tags_7 },
    { 0xd2938b30, SUC_lexicon_tags_14 },
    { 0xd3d38b30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c738b34, SUC_lexicon_tags_7 },
    { 0x51f38b35, SUC_lexicon_tags_0 },
    { 0x1fd38b35, SUC_lexicon_tags_25 },
    { 0xe2538b37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b338b39, SUC_lexicon_tags_65 },
    { 0xd538b3a, SUC_lexicon_tags_10 },
    { 0x2ed38b3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5cb38b3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45338b3f, SUC_lexicon_tags_47 },
    { 0x2738b40, SUC_lexicon_tags_15 },
    { 0xea338b3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa138b46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad38b50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f38b56, SUC_lexicon_tags_23 },
    { 0x12538b57, SUC_lexicon_tags_33 },
    { 0x89f38b57, SUC_lexicon_tags_14 },
    { 0x3a538b59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef38b5b, SUC_lexicon_tags_25 },
    { 0x10538b5c, SUC_lexicon_tags_12 },
    { 0xffb38b5c, SUC_lexicon_tags_15 },
    { 0x6d338b5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1738b60, SUC_lexicon_tags_19 },
    { 0x21738b61, SUC_lexicon_tags_10 },
    { 0x6a538b61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd738b68, SUC_lexicon_tags_8 },
    { 0xc6138b69, SUC_lexicon_tags_26 },
    { 0x20738b6a, SUC_lexicon_tags_17 },
    { 0xd0538b6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34f38b6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50138b70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9538b72, SUC_lexicon_tags_27 },
    { 0xd8d38b72, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x92b38b75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b938b77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff538b7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24d38b7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83938b81, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b38b85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b38b8d, SUC_lexicon_tags_3 },
    { 0x29938b8e, SUC_lexicon_tags_46 },
    { 0x7f38b8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3db38b91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf938b99, SUC_lexicon_tags_15 },
    { 0x73138b99, SUC_lexicon_tags_32 },
    { 0xb138b9b, SUC_lexicon_tags_23 },
    { 0x62b38b9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88338b9e, SUC_lexicon_tags_11 },
    { 0x138b9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3738ba3, SUC_lexicon_tags_1 },
    { 0x64938ba3, SUC_lexicon_tags_63 },
    { 0x2a138ba5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a938ba7, SUC_lexicon_tags_59 },
    { 0x73538ba7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63338bac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9938bb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb338bb5, SUC_lexicon_tags_17 },
    { 0x41738bb6, SUC_lexicon_tags_56 },
    { 0x3d338bb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33138bb9, SUC_lexicon_tags_37 },
    { 0x3ad38bba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd138bbc, SUC_lexicon_tags_25 },
    { 0x74738bbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0538bbf, SUC_lexicon_tags_37 },
    { 0x22738bc0, SUC_lexicon_tags_49 },
    { 0x69538bbf, SUC_lexicon_tags_33 },
    { 0x23338bc2, SUC_lexicon_tags_17 },
    { 0x8f938bc3, SUC_lexicon_tags_113 },
    { 0x84d38bc4, SUC_lexicon_tags_23 },
    { 0x1f738bc5, SUC_lexicon_tags_14 },
    { 0x8bd38bc6, SUC_lexicon_tags_29 },
    { 0xc738bc7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d38bcc, SUC_lexicon_tags_37 },
    { 0x69538bcc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b38bd6, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0x6a538bd8, SUC_lexicon_tags_25 },
    { 0x51738bd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91738bdb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef38be1, SUC_lexicon_tags_37 },
    { 0x23938be2, SUC_lexicon_tags_25 },
    { 0x17938be3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7738beb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x36338bed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd338bf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44138bf3, SUC_lexicon_tags_8 },
    { 0xd4d38bf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f138bf8, SUC_lexicon_tags_25 },
    { 0x4a538bf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86338bfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe538c01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97538c03, SUC_lexicon_tags_14 },
    { 0x3b338c03, SUC_lexicon_tags_66 },
    { 0xf3738c05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7738c0c, SUC_lexicon_tags_25 },
    { 0x33538c0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd38c12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3938c15, SUC_lexicon_tags_9 },
    { 0x78338c16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7938c18, SUC_lexicon_tags_11 },
    { 0xcdf38c19, SUC_lexicon_tags_25 },
    { 0x91d38c1a, SUC_lexicon_tags_63 },
    { 0x8cb38c1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29938c1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9738c24, SUC_lexicon_tags_2 },
    { 0xcfd38c24, SUC_lexicon_tags_14 },
    { 0xf8738c25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df38c31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f38c37, SUC_lexicon_tags_9 },
    { 0xb5d38c38, SUC_lexicon_tags_29 },
    { 0xc0138c38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0938c3b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96138c41, SUC_lexicon_tags_2 },
    { 0xf3138c42, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7938c45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb38c48, SUC_lexicon_tags_14 },
    { 0x17338c49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0138c4d, SUC_lexicon_tags_140 },
    { 0x21938c4d, SUC_lexicon_tags_65 },
    { 0xd1b38c4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b38c52, SUC_lexicon_tags_11 },
    { 0xb7f38c53, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3538c56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d38c5a, SUC_lexicon_tags_6 },
    { 0x9ed38c5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa538c5d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x53738c5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75138c63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0738c66, SUC_lexicon_tags_148 },
    { 0x44738c66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf38c6c, SUC_lexicon_tags_23 },
    { 0xfe538c6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2538c70, SUC_lexicon_tags_11 },
    { 0xe6938c71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d338c74, SUC_lexicon_tags_3 },
    { 0x2b738c75, SUC_lexicon_tags_7 },
    { 0xa1538c74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad38c7a, SUC_lexicon_tags_7 },
    { 0xad738c7b, SUC_lexicon_tags_7 },
    { 0xb538c7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc138c80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x90338c83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d38c91, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f38c98, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21338c9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4338ca5, SUC_lexicon_tags_7 },
    { 0xe5b38ca6, SUC_lexicon_tags_56 },
    { 0xce138ca6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6538caa, SUC_lexicon_tags_33 },
    { 0x30b38cab, SUC_lexicon_tags_7 },
    { 0x67138cac, SUC_lexicon_tags_2 },
    { 0x36138cad, SUC_lexicon_tags_45 },
    { 0x71938cad, SUC_lexicon_tags_17 },
    { 0x96738cac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f38cb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b38cb9, SUC_lexicon_tags_35 },
    { 0xbb38cba, SUC_lexicon_tags_37 },
    { 0x5e738cbb, SUC_lexicon_tags_56 },
    { 0xa5138cba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea938cc0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f38cc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3738cc7, SUC_lexicon_tags_14 },
    { 0x12138cc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd38ccb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b38cce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab38cd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ff38cd4, SUC_lexicon_tags_17 },
    { 0x2c738cd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73938cda, SUC_lexicon_tags_12 },
    { 0x12d38cdb, SUC_lexicon_tags_37 },
    { 0xe9738cdc, SUC_lexicon_tags_7 },
    { 0xc538cdd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfb938cdf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64738ce3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa1338ce5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4338cea, SUC_lexicon_tags_31 },
    { 0xa8338cea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb38cef, SUC_lexicon_tags_29 },
    { 0x17338cef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xae538cf2, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4738cf5, SUC_lexicon_tags_158 },
    { 0x16b38cf5, SUC_lexicon_tags_82 },
    { 0x81f38cf7, SUC_lexicon_tags_48 },
    { 0x41b38cf8, SUC_lexicon_tags_57 },
    { 0xd538cf9, SUC_lexicon_tags_15 },
    { 0xe8d38cf8, SUC_lexicon_tags_85 },
    { 0xe6738cf6, SUC_lexicon_tags_15 },
    { 0xc3738cfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3338cfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8d38d00, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f338d06, SUC_lexicon_tags_17 },
    { 0x73f38d06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9af38d09, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d38d0c, SUC_lexicon_tags_32 },
    { 0xb6f38d0d, SUC_lexicon_tags_107 },
    { 0xcdf38d0e, SUC_lexicon_tags_63 },
    { 0xee738d0c, SUC_lexicon_tags_63 },
    { 0xbef38d10, SUC_lexicon_tags_64 },
    { 0x20b38d10, SUC_lexicon_tags_25 },
    { 0x29d38d11, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67938d16, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc3338d18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24738d1a, SUC_lexicon_tags_27 },
    { 0xa9338d1b, SUC_lexicon_tags_12 },
    { 0x2df38d1b, SUC_lexicon_tags_25 },
    { 0xf4338d1d, SUC_lexicon_tags_23 },
    { 0xacb38d1d, SUC_lexicon_tags_34 },
    { 0xa4138d1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29738d21, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f38d29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d38d2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed38d31, SUC_lexicon_tags_10 },
    { 0xa9338d32, SUC_lexicon_tags_49 },
    { 0x7938d33, SUC_lexicon_tags_276 },
    { 0x27138d33, SUC_lexicon_tags_37 },
    { 0x9338d35, SUC_lexicon_tags_107 },
    { 0xc0b38d36, SUC_lexicon_tags_2 },
    { 0x98938d33, SUC_lexicon_tags_15 },
    { 0x3938d31, SUC_lexicon_tags_61 },
    { 0xa2f38d32, SUC_lexicon_tags_2 },
    { 0x62538d3a, SUC_lexicon_tags_38 },
    { 0x3b538d3a, SUC_lexicon_tags_23 },
    { 0xe9d38d3c, SUC_lexicon_tags_0 },
    { 0x2c738d3d, SUC_lexicon_tags_60 },
    { 0x23938d3e, SUC_lexicon_tags_57 },
    { 0x3a138d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa938d41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d138d43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4338d46, SUC_lexicon_tags_30 },
    { 0x1f738d47, SUC_lexicon_tags_6 },
    { 0xc8338d48, SUC_lexicon_tags_57 },
    { 0xe4f38d46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde738d4d, SUC_lexicon_tags_14 },
    { 0xc5138d4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb38d51, SUC_lexicon_tags_60 },
    { 0x95738d51, SUC_lexicon_tags_15 },
    { 0x76f38d53, SUC_lexicon_tags_11 },
    { 0x44538d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d38d58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb138d5d, SUC_lexicon_tags_1 },
    { 0x50338d5e, SUC_lexicon_tags_2 },
    { 0xd7338d5f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d38d69, SUC_lexicon_tags_25 },
    { 0xb3938d6a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9bb38d6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87138d70, SUC_lexicon_tags_34 },
    { 0x2fb38d70, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61938d77, SUC_lexicon_tags_10 },
    { 0xc7738d77, SUC_lexicon_tags_39 },
    { 0x5f138d79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22938d7b, SUC_lexicon_tags_1 },
    { 0x86138d7c, SUC_lexicon_tags_25 },
    { 0x9a738d7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x338d80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83b38d82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61138d85, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xff338d88, SUC_lexicon_tags_42 },
    { 0x48d38d89, SUC_lexicon_tags_12 },
    { 0xfeb38d8a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f38d8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5338d90, SUC_lexicon_tags_38 },
    { 0x26138d90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e738d98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0738d9b, SUC_lexicon_tags_17 },
    { 0x51938d9c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x11738d9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96f38da0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xce338da2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb338da5, SUC_lexicon_tags_37 },
    { 0xea338da6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2338da9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8db38dab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24538daf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8d38db1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e538db5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xec338db8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68138dbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa938dbf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8938dc3, SUC_lexicon_tags_32 },
    { 0x6bb38dc3, SUC_lexicon_tags_25 },
    { 0x1b538dc5, SUC_lexicon_tags_301 },
    { 0xffb38dc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d738dc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc738dcc, SUC_lexicon_tags_45 },
    { 0x93938dcd, SUC_lexicon_tags_11 },
    { 0x44d38dcc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x11338dd1, SUC_lexicon_tags_38 },
    { 0x81738dd2, SUC_lexicon_tags_17 },
    { 0x8cb38dd3, SUC_lexicon_tags_3 },
    { 0x99938dd4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f38dd7, SUC_lexicon_tags_38 },
    { 0xef738dd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c738dda, SUC_lexicon_tags_7 },
    { 0x8538dda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8c938ddd, SUC_lexicon_tags_62 },
    { 0xb338dde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f38de5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x77938de8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d338df1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4938dfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x30538dfd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b38e01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c938e03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4938e08, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc6f38e0a, SUC_lexicon_tags_38 },
    { 0x99338e0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x27338e0e, SUC_lexicon_tags_15 },
    { 0x49f38e0e, SUC_lexicon_tags_1 },
    { 0x41338e0e, SUC_lexicon_tags_13 },
    { 0xb4d38e0e, SUC_lexicon_tags_57 },
    { 0x49d38e11, SUC_lexicon_tags_65 },
    { 0x38538e0e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb38e18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b38e1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c538e20, SUC_lexicon_tags_37 },
    { 0xe3d38e21, SUC_lexicon_tags_25 },
    { 0xe5938e20, SUC_lexicon_tags_25 },
    { 0xde338e23, SUC_lexicon_tags_23 },
    { 0x3738e20, SUC_lexicon_tags_39 },
    { 0x5a938e22, SUC_lexicon_tags_38 },
    { 0x7a938e26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5538e2c, SUC_lexicon_tags_25 },
    { 0x13138e2c, SUC_lexicon_tags_37 },
    { 0x73738e2e, SUC_lexicon_tags_15 },
    { 0x6b138e2d, SUC_lexicon_tags_61 },
    { 0xf7738e30, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf338e33, SUC_lexicon_tags_68 },
    { 0xc3f38e33, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c738e37, SUC_lexicon_tags_25 },
    { 0x66538e37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1b538e3a, SUC_lexicon_tags_19 },
    { 0xa3138e3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70538e3f, SUC_lexicon_tags_12 },
    { 0x37f38e40, SUC_lexicon_tags_11 },
    { 0x99f38e41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f38e47, SUC_lexicon_tags_12 },
    { 0xa2538e48, SUC_lexicon_tags_2 },
    { 0x94938e47, SUC_lexicon_tags_38 },
    { 0x76d38e49, SUC_lexicon_tags_16 },
    { 0xf1f38e4a, SUC_lexicon_tags_63 },
    { 0x72538e4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7538e4f, SUC_lexicon_tags_16 },
    { 0xb7738e4f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf338e53, SUC_lexicon_tags_21 },
    { 0x6d38e53, SUC_lexicon_tags_43 },
    { 0xf5738e55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f38e5b, SUC_lexicon_tags_62 },
    { 0x73738e5b, SUC_lexicon_tags_7 },
    { 0x74d38e5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb38e64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85138e6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3538e73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf38e76, SUC_lexicon_tags_66 },
    { 0x89938e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3138e7a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab38e7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb3538e81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaef38e83, SUC_lexicon_tags_47 },
    { 0x2e938e83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25138e89, SUC_lexicon_tags_27 },
    { 0x7d38e8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94138e8f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe9f38e91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8138e93, SUC_lexicon_tags_12 },
    { 0x69f38e94, SUC_lexicon_tags_12 },
    { 0x3c338e94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83338e99, SUC_lexicon_tags_7 },
    { 0x81538e9a, SUC_lexicon_tags_37 },
    { 0xced38e9b, SUC_lexicon_tags_33 },
    { 0x95738e9c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1538e9f, SUC_lexicon_tags_17 },
    { 0xda738ea0, SUC_lexicon_tags_12 },
    { 0x35738ea1, SUC_lexicon_tags_55 },
    { 0x46d38ea2, SUC_lexicon_tags_37 },
    { 0x28f38e9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd38ea6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d38eae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8938eb0, SUC_lexicon_tags_17 },
    { 0xb4938eb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec538eb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67338eba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f38ec1, SUC_lexicon_tags_45 },
    { 0x31138ec1, SUC_lexicon_tags_29 },
    { 0xfc538ec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73538ec6, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b38ecc, SUC_lexicon_tags_17 },
    { 0xa2738ecd, SUC_lexicon_tags_11 },
    { 0x98d38ece, SUC_lexicon_tags_0 },
    { 0x98f38ecd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x66338ed2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4538ed4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61b38ed6, SUC_lexicon_tags_63 },
    { 0xcbb38ed7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x99f38ed9, SUC_lexicon_tags_11 },
    { 0x6a938ed9, SUC_lexicon_tags_35 },
    { 0x87b38eda, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x3cf38edd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1338ee3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d538eea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6138eed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79338ef1, SUC_lexicon_tags_14 },
    { 0xa1b38ef2, SUC_lexicon_tags_17 },
    { 0x64538ef3, SUC_lexicon_tags_38 },
    { 0x3f338ef3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37338ef6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b38efa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df38efd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f538f07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x36338f09, SUC_lexicon_tags_11 },
    { 0x5eb38f0a, SUC_lexicon_tags_16 },
    { 0x1c938f0b, SUC_lexicon_tags_7 },
    { 0x6dd38f0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb38f10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a138f13, SUC_lexicon_tags_7 },
    { 0x20538f13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b38f19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75738f1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x78338f1f, SUC_lexicon_tags_14 },
    { 0xa738f20, SUC_lexicon_tags_23 },
    { 0xdd338f21, SUC_lexicon_tags_7 },
    { 0x2ff38f1f, SUC_lexicon_tags_70 },
    { 0x90538f20, SUC_lexicon_tags_87 },
    { 0x58938f20, SUC_lexicon_tags_49 },
    { 0x27f38f23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0538f2e, SUC_lexicon_tags_28 },
    { 0xd2d38f2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb2138f31, SUC_lexicon_tags_47 },
    { 0xc4738f32, SUC_lexicon_tags_2 },
    { 0x1c738f32, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x24738f36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcf38f38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4138f3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97938f43, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f38f49, SUC_lexicon_tags_87 },
    { 0xb8338f49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d38f53, SUC_lexicon_tags_15 },
    { 0x11338f53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43138f56, SUC_lexicon_tags_37 },
    { 0x27538f57, SUC_lexicon_tags_47 },
    { 0xe7138f58, SUC_lexicon_tags_47 },
    { 0xe7b38f59, SUC_lexicon_tags_4 },
    { 0x55738f56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc0938f5c, SUC_lexicon_tags_70 },
    { 0x53738f5d, SUC_lexicon_tags_9 },
    { 0x42f38f5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b738f62, SUC_lexicon_tags_19 },
    { 0x9f138f63, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b38f68, SUC_lexicon_tags_39 },
    { 0xf5738f69, SUC_lexicon_tags_1 },
    { 0xbaf38f6a, SUC_lexicon_tags_23 },
    { 0xbb738f68, SUC_lexicon_tags_0 },
    { 0x3a938f6c, SUC_lexicon_tags_7 },
    { 0xb2738f6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14f38f6f, SUC_lexicon_tags_47 },
    { 0xc0338f6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e938f74, SUC_lexicon_tags_45 },
    { 0xe6338f75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef38f79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b38f7c, SUC_lexicon_tags_30 },
    { 0x3a138f7d, SUC_lexicon_tags_38 },
    { 0x29938f7d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f38f81, SUC_lexicon_tags_14 },
    { 0xf6138f82, SUC_lexicon_tags_95 },
    { 0x67d38f83, SUC_lexicon_tags_43 },
    { 0x17138f82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b38f87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1138f8a, SUC_lexicon_tags_7 },
    { 0xd8338f8b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x22938f8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76738f90, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x538f93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c138f9c, SUC_lexicon_tags_47 },
    { 0x89d38f9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6138fa3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e338fa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b38fae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20138fb0, SUC_lexicon_tags_2 },
    { 0xeb938fb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10138fb6, SUC_lexicon_tags_16 },
    { 0x1bf38fb6, SUC_lexicon_tags_12 },
    { 0x6ed38fb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a338fbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbbd38fbe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x42138fc0, SUC_lexicon_tags_12 },
    { 0x8ff38fc0, SUC_lexicon_tags_25 },
    { 0x53d38fc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d38fc6, SUC_lexicon_tags_63 },
    { 0x72b38fc7, SUC_lexicon_tags_63 },
    { 0x18b38fc8, SUC_lexicon_tags_49 },
    { 0x69338fc8, SUC_lexicon_tags_2 },
    { 0x8f338fca, SUC_lexicon_tags_17 },
    { 0x7ab38fca, SUC_lexicon_tags_27 },
    { 0xbf38fca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20138fd6, SUC_lexicon_tags_2 },
    { 0xb7538fd7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab38fe2, SUC_lexicon_tags_37 },
    { 0xeb338fe3, SUC_lexicon_tags_23 },
    { 0x4c138fe3, SUC_lexicon_tags_13 },
    { 0x66f38fe5, SUC_lexicon_tags_11 },
    { 0x21338fe6, SUC_lexicon_tags_23 },
    { 0xdcd38fe7, SUC_lexicon_tags_11 },
    { 0xea138fe8, SUC_lexicon_tags_27 },
    { 0x53b38fe9, SUC_lexicon_tags_27 },
    { 0x3fd38fe7, SUC_lexicon_tags_15 },
    { 0x13b38fe9, SUC_lexicon_tags_25 },
    { 0x9ed38fe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1738ff4, SUC_lexicon_tags_8 },
    { 0xced38ff5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c938ff7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47938ffb, SUC_lexicon_tags_37 },
    { 0x36338ffb, SUC_lexicon_tags_11 },
    { 0x74338ffd, SUC_lexicon_tags_25 },
    { 0xa4538ffe, SUC_lexicon_tags_2 },
    { 0xe8338ffe, SUC_lexicon_tags_28 },
    { 0xa8738ffc, SUC_lexicon_tags_15 },
    { 0x7d739001, SUC_lexicon_tags_30 },
    { 0xf2b39001, SUC_lexicon_tags_3 },
    { 0x33f38ffd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d39006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaf39008, SUC_lexicon_tags_136 },
    { 0xdbf39009, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4d3900b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30339016, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b39019, SUC_lexicon_tags_14 },
    { 0xbbf3901a, SUC_lexicon_tags_23 },
    { 0xbd33901a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d93901e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3ab39020, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33902c, SUC_lexicon_tags_39 },
    { 0xdf33902d, SUC_lexicon_tags_47 },
    { 0xbab3902c, SUC_lexicon_tags_27 },
    { 0x1453902f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa4339031, SUC_lexicon_tags_7 },
    { 0xd4939032, SUC_lexicon_tags_19 },
    { 0xb1539031, SUC_lexicon_tags_37 },
    { 0x2dd39032, SUC_lexicon_tags_6 },
    { 0xaff39034, SUC_lexicon_tags_37 },
    { 0x91739036, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa33903a, SUC_lexicon_tags_47 },
    { 0xe4b3903b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf3903e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4539040, SUC_lexicon_tags_1 },
    { 0x9a539040, SUC_lexicon_tags_37 },
    { 0x37d39042, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe539048, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71d3904a, SUC_lexicon_tags_22 },
    { 0xb8f3904b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab39054, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0339058, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77939063, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x79b39065, SUC_lexicon_tags_7 },
    { 0x19939066, SUC_lexicon_tags_32 },
    { 0xa6f39067, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93139069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b3906c, SUC_lexicon_tags_11 },
    { 0x6ef3906d, SUC_lexicon_tags_17 },
    { 0xbc33906e, SUC_lexicon_tags_25 },
    { 0xde93906f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5939071, SUC_lexicon_tags_1 },
    { 0xcd39072, SUC_lexicon_tags_7 },
    { 0xdc139072, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32539076, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9553907d, SUC_lexicon_tags_25 },
    { 0xa733907d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f39082, SUC_lexicon_tags_11 },
    { 0xd7b39083, SUC_lexicon_tags_12 },
    { 0xe5f39083, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63739087, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8313908a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b53908c, SUC_lexicon_tags_33 },
    { 0x3b13908d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d39091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5339095, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71390a2, SUC_lexicon_tags_33 },
    { 0x1d7390a3, SUC_lexicon_tags_17 },
    { 0x965390a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1390a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f390ab, SUC_lexicon_tags_27 },
    { 0xad3390ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x501390af, SUC_lexicon_tags_22 },
    { 0x905390af, SUC_lexicon_tags_2 },
    { 0x9d390b1, SUC_lexicon_tags_17 },
    { 0xa3f390b2, SUC_lexicon_tags_1 },
    { 0xbf3390b3, SUC_lexicon_tags_43 },
    { 0xbd1390b4, SUC_lexicon_tags_1 },
    { 0x72b390b5, SUC_lexicon_tags_2 },
    { 0xf33390b3, SUC_lexicon_tags_106 },
    { 0x6b5390b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa0f390b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7fd390bb, SUC_lexicon_tags_25 },
    { 0x3b390bb, SUC_lexicon_tags_1 },
    { 0x55f390bd, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x70b390bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01390c3, SUC_lexicon_tags_14 },
    { 0x8b7390c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b390c8, SUC_lexicon_tags_32 },
    { 0x8b390c9, SUC_lexicon_tags_7 },
    { 0x8d1390ca, SUC_lexicon_tags_27 },
    { 0xc0f390cb, SUC_lexicon_tags_19 },
    { 0x8c5390c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x415390ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x65b390d0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695390d5, SUC_lexicon_tags_11 },
    { 0x65b390d5, SUC_lexicon_tags_1 },
    { 0x1ff390d5, SUC_lexicon_tags_27 },
    { 0x1cd390d8, SUC_lexicon_tags_9 },
    { 0x5c5390d8, SUC_lexicon_tags_62 },
    { 0x6f9390d8, SUC_lexicon_tags_65 },
    { 0xcb3390db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9390de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9390e6, SUC_lexicon_tags_38 },
    { 0xb9d390e6, SUC_lexicon_tags_7 },
    { 0xc05390e8, SUC_lexicon_tags_1 },
    { 0x555390e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa15390eb, SUC_lexicon_tags_37 },
    { 0xf8f390ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x903390ee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5390f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfb390f4, SUC_lexicon_tags_15 },
    { 0xb75390f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307390fa, SUC_lexicon_tags_37 },
    { 0xa27390fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117390ff, SUC_lexicon_tags_14 },
    { 0x1d39100, SUC_lexicon_tags_178 },
    { 0xdef390ff, SUC_lexicon_tags_39 },
    { 0xd9139101, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9453910a, SUC_lexicon_tags_55 },
    { 0x7113910a, SUC_lexicon_tags_43 },
    { 0x9b53910c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe513910f, SUC_lexicon_tags_11 },
    { 0x7553910f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d339114, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf39116, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60539119, SUC_lexicon_tags_11 },
    { 0xc6f3911a, SUC_lexicon_tags_8 },
    { 0xe513911a, SUC_lexicon_tags_0 },
    { 0xfd13911c, SUC_lexicon_tags_0 },
    { 0x97d3911d, SUC_lexicon_tags_31 },
    { 0xa493911e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d539123, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x58f39125, SUC_lexicon_tags_25 },
    { 0xb4939126, SUC_lexicon_tags_49 },
    { 0x52b39125, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3d39129, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7353912c, SUC_lexicon_tags_87 },
    { 0x193912d, SUC_lexicon_tags_10 },
    { 0xd473912c, SUC_lexicon_tags_4 },
    { 0xf2b3912f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd39133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc339139, SUC_lexicon_tags_2 },
    { 0x7e53913a, SUC_lexicon_tags_47 },
    { 0xbe13913b, SUC_lexicon_tags_37 },
    { 0xd213913c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb73913e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d39142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb3914b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf39152, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66139154, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7139156, SUC_lexicon_tags_45 },
    { 0xcf39157, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9339160, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x20739163, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x92939165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed539168, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f39171, SUC_lexicon_tags_25 },
    { 0xd6b39172, SUC_lexicon_tags_42 },
    { 0x44d39172, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25739177, SUC_lexicon_tags_11 },
    { 0xdb739178, SUC_lexicon_tags_10 },
    { 0xfdd39179, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a73917b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x5af3917d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b39180, SUC_lexicon_tags_70 },
    { 0x7cb39181, SUC_lexicon_tags_2 },
    { 0x32539181, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b339184, SUC_lexicon_tags_10 },
    { 0xb1339184, SUC_lexicon_tags_2 },
    { 0x13139186, SUC_lexicon_tags_1 },
    { 0x39339187, SUC_lexicon_tags_43 },
    { 0xa739185, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd3918d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5539190, SUC_lexicon_tags_19 },
    { 0xb1339191, SUC_lexicon_tags_1 },
    { 0x65d39192, SUC_lexicon_tags_12 },
    { 0x60b39192, SUC_lexicon_tags_11 },
    { 0x74939193, SUC_lexicon_tags_10 },
    { 0x94f39192, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b39199, SUC_lexicon_tags_11 },
    { 0x9ff3919a, SUC_lexicon_tags_17 },
    { 0x27139199, SUC_lexicon_tags_39 },
    { 0x4bd3919c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb391a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf59391a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x615391a4, SUC_lexicon_tags_2 },
    { 0xb25391a5, SUC_lexicon_tags_46 },
    { 0xd4f391a5, SUC_lexicon_tags_19 },
    { 0x111391a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1391ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc37391ae, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37391b3, SUC_lexicon_tags_17 },
    { 0x25f391b4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xef391b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741391bb, SUC_lexicon_tags_8 },
    { 0x3bd391bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x117391be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c9391c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb391c3, SUC_lexicon_tags_30 },
    { 0xa73391c4, SUC_lexicon_tags_80 },
    { 0x2a5391c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x119391c8, SUC_lexicon_tags_29 },
    { 0x8d1391c9, SUC_lexicon_tags_39 },
    { 0x4b391ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d391cf, SUC_lexicon_tags_0 },
    { 0x627391cf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767391d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf3391d6, SUC_lexicon_tags_25 },
    { 0xdff391d6, SUC_lexicon_tags_2 },
    { 0x9b391d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d391e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b391e4, SUC_lexicon_tags_12 },
    { 0xc81391e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349391e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2db391eb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb0f391ed, SUC_lexicon_tags_7 },
    { 0x873391ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3391f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9fb391f3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3391f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f391fb, SUC_lexicon_tags_28 },
    { 0x37d391fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x123391fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf9d39200, SUC_lexicon_tags_1 },
    { 0x1cb39200, SUC_lexicon_tags_21 },
    { 0x5ff39202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36139204, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2939207, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x78d39209, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f3920d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd39215, SUC_lexicon_tags_1 },
    { 0xb6939216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d93921f, SUC_lexicon_tags_37 },
    { 0xfa739220, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac539224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b39227, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7993922e, SUC_lexicon_tags_79 },
    { 0x3ff3922f, SUC_lexicon_tags_262 },
    { 0xc9f39230, SUC_lexicon_tags_2 },
    { 0xaa73922e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b339234, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f339236, SUC_lexicon_tags_27 },
    { 0xe2f39237, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9693923d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5939241, SUC_lexicon_tags_30 },
    { 0xb6939241, SUC_lexicon_tags_17 },
    { 0xbaf39243, SUC_lexicon_tags_7 },
    { 0x71f39244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6773924f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9139251, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd39254, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd339257, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f3925a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d3925d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6539265, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x39b39267, SUC_lexicon_tags_274 },
    { 0x29539268, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x8373926b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xac53926d, SUC_lexicon_tags_17 },
    { 0x833926e, SUC_lexicon_tags_7 },
    { 0xea33926f, SUC_lexicon_tags_25 },
    { 0x8bd39270, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf39274, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d39277, SUC_lexicon_tags_1 },
    { 0xead39278, SUC_lexicon_tags_70 },
    { 0x31539279, SUC_lexicon_tags_25 },
    { 0xf973927a, SUC_lexicon_tags_17 },
    { 0x55139278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45539280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7939285, SUC_lexicon_tags_45 },
    { 0xbf939285, SUC_lexicon_tags_21 },
    { 0xafb39287, SUC_lexicon_tags_22 },
    { 0xbf139286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1893928a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb3928f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4139293, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d39297, SUC_lexicon_tags_23 },
    { 0xf5339298, SUC_lexicon_tags_14 },
    { 0x40b39298, SUC_lexicon_tags_2 },
    { 0x393929a, SUC_lexicon_tags_37 },
    { 0xa5139298, SUC_lexicon_tags_2 },
    { 0xe2b39298, SUC_lexicon_tags_55 },
    { 0xc0139297, SUC_lexicon_tags_49 },
    { 0x4833929e, SUC_lexicon_tags_37 },
    { 0xdcb3929e, SUC_lexicon_tags_12 },
    { 0x76d392a0, SUC_lexicon_tags_7 },
    { 0x951392a1, SUC_lexicon_tags_11 },
    { 0xae53929d, SUC_lexicon_tags_6 },
    { 0x869392a3, SUC_lexicon_tags_25 },
    { 0xcd392a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x77f392a6, SUC_lexicon_tags_17 },
    { 0xf6f392a7, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xfb5392a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56f392ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69392b1, SUC_lexicon_tags_11 },
    { 0x9f5392b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66f392b4, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133392bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d392be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b392c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x587392c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d392c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf392cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9392cf, SUC_lexicon_tags_37 },
    { 0x19b392d0, SUC_lexicon_tags_29 },
    { 0x607392d1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397392dd, SUC_lexicon_tags_12 },
    { 0x3b1392dd, SUC_lexicon_tags_27 },
    { 0x62b392df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xac1392e1, SUC_lexicon_tags_65 },
    { 0xb29392e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169392e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf89392e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bf392eb, SUC_lexicon_tags_43 },
    { 0x299392ec, SUC_lexicon_tags_15 },
    { 0x201392ec, SUC_lexicon_tags_44 },
    { 0x865392ee, SUC_lexicon_tags_37 },
    { 0x393392ef, SUC_lexicon_tags_11 },
    { 0x3e5392ee, SUC_lexicon_tags_10 },
    { 0xf31392ee, SUC_lexicon_tags_14 },
    { 0xb29392ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863392f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf392f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x895392fb, SUC_lexicon_tags_43 },
    { 0x62f392fb, SUC_lexicon_tags_17 },
    { 0x1e5392fb, SUC_lexicon_tags_46 },
    { 0x483392fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21939304, SUC_lexicon_tags_3 },
    { 0x8b139304, SUC_lexicon_tags_15 },
    { 0x9fd39306, SUC_lexicon_tags_16 },
    { 0x69539307, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x90339309, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a13930c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1113930e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1939310, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x20339313, SUC_lexicon_tags_47 },
    { 0xa0539314, SUC_lexicon_tags_25 },
    { 0xc9b39315, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833931a, SUC_lexicon_tags_0 },
    { 0xbf13931b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0339322, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db39326, SUC_lexicon_tags_12 },
    { 0xe6739326, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd7b39329, SUC_lexicon_tags_47 },
    { 0xa7539329, SUC_lexicon_tags_31 },
    { 0x4233932a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f139330, SUC_lexicon_tags_31 },
    { 0x37d39331, SUC_lexicon_tags_2 },
    { 0xe5d39332, SUC_lexicon_tags_6 },
    { 0xd4139333, SUC_lexicon_tags_12 },
    { 0x8b539332, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x20139336, SUC_lexicon_tags_38 },
    { 0x5eb39337, SUC_lexicon_tags_37 },
    { 0x82b39336, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa973933a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xda73933d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa113933f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3139346, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f13934b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe513934d, SUC_lexicon_tags_37 },
    { 0xe5d3934e, SUC_lexicon_tags_28 },
    { 0x31d3934f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6939352, SUC_lexicon_tags_15 },
    { 0xd2739353, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf39356, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71d39358, SUC_lexicon_tags_15 },
    { 0xd0339359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52d3935b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7333935e, SUC_lexicon_tags_8 },
    { 0x5513935f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bf39361, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x73939363, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1d39365, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad39369, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd273936c, SUC_lexicon_tags_47 },
    { 0xe893936d, SUC_lexicon_tags_6 },
    { 0x97f3936c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41539372, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b39375, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b3937a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb53937d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86b3937f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b39382, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff739386, SUC_lexicon_tags_230 },
    { 0x5b339386, SUC_lexicon_tags_10 },
    { 0x78d39387, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf3938e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdc139390, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d539393, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92939395, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c739397, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33939b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9393a0, SUC_lexicon_tags_37 },
    { 0x7af393a1, SUC_lexicon_tags_38 },
    { 0x9d1393a1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5393a6, SUC_lexicon_tags_47 },
    { 0xd95393a7, SUC_lexicon_tags_11 },
    { 0x7e3393a8, SUC_lexicon_tags_9 },
    { 0x96f393a6, SUC_lexicon_tags_8 },
    { 0x395393aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db393ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddb393af, SUC_lexicon_tags_11 },
    { 0x3e1393b0, SUC_lexicon_tags_43 },
    { 0x539393b0, SUC_lexicon_tags_27 },
    { 0x4d9393b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1393b5, SUC_lexicon_tags_47 },
    { 0x9d3393b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61393ba, SUC_lexicon_tags_98 },
    { 0xdc7393bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x115393bd, SUC_lexicon_tags_7 },
    { 0x503393bd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1d5393c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95393c5, SUC_lexicon_tags_11 },
    { 0x87f393c5, SUC_lexicon_tags_7 },
    { 0x775393c7, SUC_lexicon_tags_2 },
    { 0xd5393c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19393cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81393d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53393d4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x431393d6, SUC_lexicon_tags_27 },
    { 0xfbf393d6, SUC_lexicon_tags_33 },
    { 0xac7393d8, SUC_lexicon_tags_1 },
    { 0x2db393d8, SUC_lexicon_tags_11 },
    { 0x28f393da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249393e2, SUC_lexicon_tags_34 },
    { 0x463393e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x837393e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3393f0, SUC_lexicon_tags_17 },
    { 0x935393f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3393f4, SUC_lexicon_tags_165 },
    { 0xffb393f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7393f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f393fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511393ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d39408, SUC_lexicon_tags_37 },
    { 0xcd739409, SUC_lexicon_tags_15 },
    { 0x21b3940a, SUC_lexicon_tags_60 },
    { 0x1eb39409, SUC_lexicon_tags_7 },
    { 0xb939409, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb39413, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8739419, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b3941c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xca33941f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd39425, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f3942a, SUC_lexicon_tags_1 },
    { 0xa673942a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd633942f, SUC_lexicon_tags_15 },
    { 0xcff39430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3df39432, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3b539434, SUC_lexicon_tags_2 },
    { 0x69b39435, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xac339437, SUC_lexicon_tags_3 },
    { 0x78d39437, SUC_lexicon_tags_71 },
    { 0x9eb39439, SUC_lexicon_tags_16 },
    { 0xe9139438, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b53943f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd139442, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e739445, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x21539447, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d3944f, SUC_lexicon_tags_7 },
    { 0xda33944f, SUC_lexicon_tags_17 },
    { 0x36739450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe539453, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb39456, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa133945b, SUC_lexicon_tags_17 },
    { 0x4813945c, SUC_lexicon_tags_11 },
    { 0x5d13945c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8633945f, SUC_lexicon_tags_87 },
    { 0x4c339460, SUC_lexicon_tags_60 },
    { 0x6cb39461, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb139467, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f33946a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xedb3946c, SUC_lexicon_tags_11 },
    { 0x5413946c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd633946f, SUC_lexicon_tags_4 },
    { 0xb2139470, SUC_lexicon_tags_19 },
    { 0x1339470, SUC_lexicon_tags_45 },
    { 0x60539472, SUC_lexicon_tags_13 },
    { 0x7f53946f, SUC_lexicon_tags_25 },
    { 0xcf139474, SUC_lexicon_tags_6 },
    { 0x4d539474, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5253947a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4833947f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e139481, SUC_lexicon_tags_25 },
    { 0xd7b39482, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc4339484, SUC_lexicon_tags_11 },
    { 0x88539484, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd3739487, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bb39489, SUC_lexicon_tags_1 },
    { 0xae539489, SUC_lexicon_tags_82 },
    { 0x4dd3948b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37b3948d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d39499, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc953949d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5394a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf394a5, SUC_lexicon_tags_60 },
    { 0x9394a5, SUC_lexicon_tags_17 },
    { 0xb21394a7, SUC_lexicon_tags_60 },
    { 0xced394a8, SUC_lexicon_tags_48 },
    { 0x41f394a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67394ac, SUC_lexicon_tags_60 },
    { 0xc45394ac, SUC_lexicon_tags_14 },
    { 0xc23394ae, SUC_lexicon_tags_27 },
    { 0x5db394af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9394b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x541394b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9a5394b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x343394b9, SUC_lexicon_tags_65 },
    { 0xb03394b9, SUC_lexicon_tags_27 },
    { 0x78b394bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x761394bd, SUC_lexicon_tags_39 },
    { 0xe5f394be, SUC_lexicon_tags_9 },
    { 0xae5394bf, SUC_lexicon_tags_66 },
    { 0x213394be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x973394c2, SUC_lexicon_tags_68 },
    { 0xead394c3, SUC_lexicon_tags_19 },
    { 0xd27394c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349394c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5394cd, SUC_lexicon_tags_17 },
    { 0x429394cd, SUC_lexicon_tags_37 },
    { 0x5e1394ce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x715394d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835394d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d394d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd394dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x651394e0, SUC_lexicon_tags_57 },
    { 0x6e9394e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75394e9, SUC_lexicon_tags_1 },
    { 0xf61394ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b394ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5394f0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5394f4, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x181394f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63394fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1394ff, SUC_lexicon_tags_17 },
    { 0xcef39500, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193950b, SUC_lexicon_tags_1 },
    { 0x7bf3950b, SUC_lexicon_tags_6 },
    { 0xa7f3950d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b39511, SUC_lexicon_tags_16 },
    { 0x26939512, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f39515, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b3951b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b13951d, SUC_lexicon_tags_25 },
    { 0x3033951e, SUC_lexicon_tags_17 },
    { 0x7093951f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff39522, SUC_lexicon_tags_17 },
    { 0x46b39522, SUC_lexicon_tags_144 },
    { 0x2bb39522, SUC_lexicon_tags_23 },
    { 0x7cd39525, SUC_lexicon_tags_14 },
    { 0x81339525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553952e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50939530, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6939532, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f39536, SUC_lexicon_tags_46 },
    { 0x81139537, SUC_lexicon_tags_7 },
    { 0xf2539537, SUC_lexicon_tags_39 },
    { 0x5dd39539, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e13953b, SUC_lexicon_tags_17 },
    { 0x5eb3953c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa73953f, SUC_lexicon_tags_2 },
    { 0xe539540, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea539548, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8453954b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79339552, SUC_lexicon_tags_9 },
    { 0xfd539552, SUC_lexicon_tags_37 },
    { 0x67539554, SUC_lexicon_tags_60 },
    { 0x57139554, SUC_lexicon_tags_21 },
    { 0x28d39556, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13955a, SUC_lexicon_tags_32 },
    { 0x59b3955b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11b3955d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa373955f, SUC_lexicon_tags_16 },
    { 0xef39560, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6739564, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d53956b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef3956e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4139570, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x56339572, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x16339574, SUC_lexicon_tags_151 },
    { 0x4ab39575, SUC_lexicon_tags_26 },
    { 0xe8939574, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec93957b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d93957f, SUC_lexicon_tags_15 },
    { 0x4b93957f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60939582, SUC_lexicon_tags_38 },
    { 0x65d39582, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b39586, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78f39588, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4733958a, SUC_lexicon_tags_17 },
    { 0xd353958b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bd3958d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb39590, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x32f39592, SUC_lexicon_tags_12 },
    { 0xe9b39593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b39596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ef39598, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d3959a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc833959e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5395a3, SUC_lexicon_tags_37 },
    { 0x239395a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5395ac, SUC_lexicon_tags_19 },
    { 0xfcb395ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x745395af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa03395b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c9395b3, SUC_lexicon_tags_37 },
    { 0x33395b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7395b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b395c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32f395c2, SUC_lexicon_tags_19 },
    { 0xfd3395c3, SUC_lexicon_tags_37 },
    { 0xf8d395c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5c3395c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8d395c8, SUC_lexicon_tags_7 },
    { 0xfdf395c8, SUC_lexicon_tags_17 },
    { 0xc3f395ca, SUC_lexicon_tags_7 },
    { 0x97d395cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97395ce, SUC_lexicon_tags_45 },
    { 0x781395cf, SUC_lexicon_tags_47 },
    { 0xb27395d0, SUC_lexicon_tags_103 },
    { 0xc19395d0, SUC_lexicon_tags_3 },
    { 0x58f395d1, SUC_lexicon_tags_13 },
    { 0x365395d0, SUC_lexicon_tags_1 },
    { 0x7b1395d4, SUC_lexicon_tags_8 },
    { 0x43395d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff395d8, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0x13d395da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef395df, SUC_lexicon_tags_37 },
    { 0x11395df, SUC_lexicon_tags_11 },
    { 0x1b5395e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c5395e3, SUC_lexicon_tags_7 },
    { 0xb0b395e4, SUC_lexicon_tags_34 },
    { 0xd7f395e5, SUC_lexicon_tags_2 },
    { 0x979395e5, SUC_lexicon_tags_30 },
    { 0xbe3395e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x175395ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56f395ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab395f2, SUC_lexicon_tags_25 },
    { 0x4c5395f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95395fa, SUC_lexicon_tags_38 },
    { 0xe89395fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34339602, SUC_lexicon_tags_12 },
    { 0xd7939602, SUC_lexicon_tags_26 },
    { 0x1bf39603, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac139608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a53960c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b39613, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7f39615, SUC_lexicon_tags_7 },
    { 0xe4939615, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5739618, SUC_lexicon_tags_38 },
    { 0xc7b39619, SUC_lexicon_tags_15 },
    { 0xbad39619, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa613961e, SUC_lexicon_tags_21 },
    { 0x33961f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10339628, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8253962f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2139631, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd253963a, SUC_lexicon_tags_12 },
    { 0x54f3963b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x5e53963d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c13963f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df39642, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xc6939644, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4339646, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b539649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13964d, SUC_lexicon_tags_1 },
    { 0x62d3964e, SUC_lexicon_tags_8 },
    { 0xc3b3964e, SUC_lexicon_tags_11 },
    { 0x4873964f, SUC_lexicon_tags_1 },
    { 0xa5d39651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1339658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x573965a, SUC_lexicon_tags_14 },
    { 0x3873965b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9173965d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f39662, SUC_lexicon_tags_201 },
    { 0xfd739662, SUC_lexicon_tags_31 },
    { 0xde739664, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f39667, SUC_lexicon_tags_23 },
    { 0x41739668, SUC_lexicon_tags_25 },
    { 0xa2739669, SUC_lexicon_tags_12 },
    { 0xf653966a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacb3966c, SUC_lexicon_tags_30 },
    { 0x2413966d, SUC_lexicon_tags_31 },
    { 0xc9b3966d, SUC_lexicon_tags_38 },
    { 0x4a13966f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3339671, SUC_lexicon_tags_27 },
    { 0xecf39671, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f39675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66739679, SUC_lexicon_tags_28 },
    { 0xa0f39679, SUC_lexicon_tags_37 },
    { 0x3b13967a, SUC_lexicon_tags_38 },
    { 0x1a73967c, SUC_lexicon_tags_49 },
    { 0xd53967d, SUC_lexicon_tags_29 },
    { 0x5b33967b, SUC_lexicon_tags_30 },
    { 0x85339679, SUC_lexicon_tags_11 },
    { 0x10d39680, SUC_lexicon_tags_17 },
    { 0x52339680, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7739683, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82139687, SUC_lexicon_tags_17 },
    { 0xf8139688, SUC_lexicon_tags_47 },
    { 0xe3939689, SUC_lexicon_tags_14 },
    { 0xd339689, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcfd3968c, SUC_lexicon_tags_2 },
    { 0x1433968d, SUC_lexicon_tags_9 },
    { 0xa453968e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a539691, SUC_lexicon_tags_48 },
    { 0x9dd39692, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7ed39694, SUC_lexicon_tags_2 },
    { 0xd7b39695, SUC_lexicon_tags_17 },
    { 0x60939695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b39699, SUC_lexicon_tags_27 },
    { 0x69d39699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d3969f, SUC_lexicon_tags_12 },
    { 0x421396a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87396a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49396a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe81396a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11396ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393396af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5cd396b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd77396b3, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad396b8, SUC_lexicon_tags_17 },
    { 0xba5396b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xefb396bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3396be, SUC_lexicon_tags_8 },
    { 0xc6b396bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d396c3, SUC_lexicon_tags_34 },
    { 0xca3396c3, SUC_lexicon_tags_15 },
    { 0xffb396c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b396c9, SUC_lexicon_tags_1696 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57396d0, SUC_lexicon_tags_35 },
    { 0xc97396d1, SUC_lexicon_tags_31 },
    { 0x973396d2, SUC_lexicon_tags_47 },
    { 0x9db396d0, SUC_lexicon_tags_17 },
    { 0x8bf396d0, SUC_lexicon_tags_16 },
    { 0x629396d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5396dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7396e0, SUC_lexicon_tags_10 },
    { 0x6d9396e1, SUC_lexicon_tags_2 },
    { 0x32d396e2, SUC_lexicon_tags_6 },
    { 0x663396e3, SUC_lexicon_tags_19 },
    { 0x49d396e1, SUC_lexicon_tags_27 },
    { 0xd83396e5, SUC_lexicon_tags_21 },
    { 0xff396e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5396e9, SUC_lexicon_tags_19 },
    { 0x4d1396e9, SUC_lexicon_tags_29 },
    { 0x559396eb, SUC_lexicon_tags_15 },
    { 0x4f3396ec, SUC_lexicon_tags_28 },
    { 0x48b396ec, SUC_lexicon_tags_2 },
    { 0x361396ee, SUC_lexicon_tags_20 },
    { 0x3a1396ee, SUC_lexicon_tags_62 },
    { 0x663396f0, SUC_lexicon_tags_6 },
    { 0xd81396f1, SUC_lexicon_tags_14 },
    { 0xda7396f1, SUC_lexicon_tags_63 },
    { 0x671396f1, SUC_lexicon_tags_47 },
    { 0x679396ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43396f9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x301396fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe07396fe, SUC_lexicon_tags_15 },
    { 0xe07396fe, SUC_lexicon_tags_47 },
    { 0x33139700, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xab339702, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaaf39704, SUC_lexicon_tags_17 },
    { 0x9bb39705, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe2f39707, SUC_lexicon_tags_17 },
    { 0xebd39708, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b3970c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa913970e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe8d39710, SUC_lexicon_tags_1 },
    { 0x77939711, SUC_lexicon_tags_12 },
    { 0x6d739710, SUC_lexicon_tags_27 },
    { 0x73939713, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf539718, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d3971c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc6b3971e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df39721, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77139725, SUC_lexicon_tags_65 },
    { 0x2e339725, SUC_lexicon_tags_7 },
    { 0xccb39727, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd3972b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f3972e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5339733, SUC_lexicon_tags_60 },
    { 0x81b39734, SUC_lexicon_tags_2 },
    { 0x60739735, SUC_lexicon_tags_23 },
    { 0xfc539736, SUC_lexicon_tags_17 },
    { 0x6ff39734, SUC_lexicon_tags_6 },
    { 0x8dd39736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19f3973a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39741, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a739748, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9739753, SUC_lexicon_tags_25 },
    { 0x7d139754, SUC_lexicon_tags_25 },
    { 0x6b139755, SUC_lexicon_tags_165 },
    { 0x9ad39753, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d39759, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a73975d, SUC_lexicon_tags_23 },
    { 0x6b93975e, SUC_lexicon_tags_25 },
    { 0xb173975d, SUC_lexicon_tags_27 },
    { 0x4a339760, SUC_lexicon_tags_1 },
    { 0x94139761, SUC_lexicon_tags_9 },
    { 0xfe139762, SUC_lexicon_tags_37 },
    { 0xadf39762, SUC_lexicon_tags_29 },
    { 0x9b939761, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1db39766, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86339769, SUC_lexicon_tags_55 },
    { 0xfdb3976a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea739770, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe0339772, SUC_lexicon_tags_8 },
    { 0x99339773, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f39776, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2153977d, SUC_lexicon_tags_47 },
    { 0x92b3977e, SUC_lexicon_tags_65 },
    { 0xba73977f, SUC_lexicon_tags_38 },
    { 0xa693977f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7539784, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda339789, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb653978c, SUC_lexicon_tags_48 },
    { 0x3e93978d, SUC_lexicon_tags_16 },
    { 0x2f33978e, SUC_lexicon_tags_1 },
    { 0xd73978e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd0b39791, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d39796, SUC_lexicon_tags_17 },
    { 0xe4f39797, SUC_lexicon_tags_25 },
    { 0x34139798, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc373979c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd397a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3397a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3397a8, SUC_lexicon_tags_37 },
    { 0x217397a9, SUC_lexicon_tags_26 },
    { 0x87397aa, SUC_lexicon_tags_42 },
    { 0x2ed397aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee1397ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23397b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x913397b5, SUC_lexicon_tags_12 },
    { 0xaa1397b5, SUC_lexicon_tags_26 },
    { 0x3bd397b7, SUC_lexicon_tags_17 },
    { 0x43b397b7, SUC_lexicon_tags_7 },
    { 0xd05397b9, SUC_lexicon_tags_25 },
    { 0xc1f397b7, SUC_lexicon_tags_11 },
    { 0xdcd397bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa69397bd, SUC_lexicon_tags_12 },
    { 0x44b397bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145397c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3397c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcb397c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b397ce, SUC_lexicon_tags_11 },
    { 0x2ef397cf, SUC_lexicon_tags_11 },
    { 0x989397cf, SUC_lexicon_tags_43 },
    { 0xce9397ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4b397d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbab397d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57397d9, SUC_lexicon_tags_7 },
    { 0xb93397da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d397e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8d7397e3, SUC_lexicon_tags_2 },
    { 0x3b1397e4, SUC_lexicon_tags_60 },
    { 0xd75397e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x15397e7, SUC_lexicon_tags_47 },
    { 0xdcf397e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7397ef, SUC_lexicon_tags_23 },
    { 0x8ad397ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x523397f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d397f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18b397f8, SUC_lexicon_tags_33 },
    { 0xbb1397f9, SUC_lexicon_tags_1 },
    { 0x405397fa, SUC_lexicon_tags_14 },
    { 0x3b5397f9, SUC_lexicon_tags_12 },
    { 0xcaf397fa, SUC_lexicon_tags_37 },
    { 0x8af397fd, SUC_lexicon_tags_14 },
    { 0xfd397f9, SUC_lexicon_tags_0 },
    { 0xe89397ff, SUC_lexicon_tags_17 },
    { 0xea939800, SUC_lexicon_tags_1 },
    { 0x6dd39801, SUC_lexicon_tags_17 },
    { 0xfc9397f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceb39804, SUC_lexicon_tags_25 },
    { 0x99f39804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80739808, SUC_lexicon_tags_38 },
    { 0x2ab39809, SUC_lexicon_tags_14 },
    { 0x4ff3980a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f39810, SUC_lexicon_tags_6 },
    { 0x33b39810, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3539817, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x95f39819, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x14d3981b, SUC_lexicon_tags_1 },
    { 0x3b53981b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8633981e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d39825, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd253982b, SUC_lexicon_tags_15 },
    { 0x19f3982b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f73982e, SUC_lexicon_tags_34 },
    { 0x8573982f, SUC_lexicon_tags_17 },
    { 0xc9b39830, SUC_lexicon_tags_25 },
    { 0x2bf39830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2eb39833, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b39839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf393983b, SUC_lexicon_tags_7 },
    { 0x5853983c, SUC_lexicon_tags_38 },
    { 0x6a93983d, SUC_lexicon_tags_25 },
    { 0x3173983e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74139840, SUC_lexicon_tags_28 },
    { 0x52539841, SUC_lexicon_tags_7 },
    { 0x3e939841, SUC_lexicon_tags_2 },
    { 0xc8539840, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf9739845, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x14539847, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7113984b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd39854, SUC_lexicon_tags_7 },
    { 0x6c739854, SUC_lexicon_tags_23 },
    { 0x25b39856, SUC_lexicon_tags_2 },
    { 0xf8b39856, SUC_lexicon_tags_47 },
    { 0x83f39856, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0xf0b3985a, SUC_lexicon_tags_84 },
    { 0x2b3985b, SUC_lexicon_tags_47 },
    { 0x8333985c, SUC_lexicon_tags_33 },
    { 0x8f33985c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb39860, SUC_lexicon_tags_30 },
    { 0x21939861, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc139865, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9139867, SUC_lexicon_tags_11 },
    { 0x7eb39867, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c53986a, SUC_lexicon_tags_68 },
    { 0xeed3986b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4973986f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4139871, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f539873, SUC_lexicon_tags_137 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d39876, SUC_lexicon_tags_29 },
    { 0x99339877, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b13987b, SUC_lexicon_tags_29 },
    { 0x1c33987b, SUC_lexicon_tags_2 },
    { 0xb353987c, SUC_lexicon_tags_2 },
    { 0xd1f3987e, SUC_lexicon_tags_14 },
    { 0x8c93987c, SUC_lexicon_tags_10 },
    { 0x53339880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa939887, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4fb39889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b3988f, SUC_lexicon_tags_7 },
    { 0x63939890, SUC_lexicon_tags_7 },
    { 0x3539891, SUC_lexicon_tags_1 },
    { 0x6f339892, SUC_lexicon_tags_56 },
    { 0xe0339893, SUC_lexicon_tags_12 },
    { 0x6339894, SUC_lexicon_tags_27 },
    { 0x75d39894, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd3989a, SUC_lexicon_tags_12 },
    { 0x6793989b, SUC_lexicon_tags_25 },
    { 0xfc53989c, SUC_lexicon_tags_25 },
    { 0x6673989c, SUC_lexicon_tags_47 },
    { 0x2d93989e, SUC_lexicon_tags_11 },
    { 0x9c33989f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5cd398a1, SUC_lexicon_tags_34 },
    { 0xda5398a2, SUC_lexicon_tags_7 },
    { 0xb39398a3, SUC_lexicon_tags_5 },
    { 0xecd398a4, SUC_lexicon_tags_37 },
    { 0x735398a5, SUC_lexicon_tags_84 },
    { 0xaa9398a6, SUC_lexicon_tags_45 },
    { 0xb0f398a6, SUC_lexicon_tags_1 },
    { 0x927398a7, SUC_lexicon_tags_17 },
    { 0xb8d398a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93398b1, SUC_lexicon_tags_7 },
    { 0xb45398b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf398ba, SUC_lexicon_tags_17 },
    { 0xac1398bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x439398bd, SUC_lexicon_tags_55 },
    { 0xe43398bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41398c1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d398c7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfdb398c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1398cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d398d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5398d5, SUC_lexicon_tags_12 },
    { 0xaa5398d6, SUC_lexicon_tags_25 },
    { 0x1bd398d7, SUC_lexicon_tags_37 },
    { 0x571398d8, SUC_lexicon_tags_37 },
    { 0x559398d9, SUC_lexicon_tags_6 },
    { 0xfd5398da, SUC_lexicon_tags_47 },
    { 0x43b398db, SUC_lexicon_tags_46 },
    { 0xf7398d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x873398df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b398e6, SUC_lexicon_tags_11 },
    { 0x7d3398e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d398ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79f398ec, SUC_lexicon_tags_11 },
    { 0x595398ec, SUC_lexicon_tags_11 },
    { 0x6af398ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81398f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41398f5, SUC_lexicon_tags_7 },
    { 0xfdd398f6, SUC_lexicon_tags_3 },
    { 0xa2b398f7, SUC_lexicon_tags_21 },
    { 0xf1398f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9398fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15939908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c33990a, SUC_lexicon_tags_28 },
    { 0xed13990b, SUC_lexicon_tags_11 },
    { 0x893990c, SUC_lexicon_tags_37 },
    { 0x1453990b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d39913, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f3991a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53991e, SUC_lexicon_tags_19 },
    { 0xee73991f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b139923, SUC_lexicon_tags_1 },
    { 0x5df39924, SUC_lexicon_tags_13 },
    { 0x76739923, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed739929, SUC_lexicon_tags_65 },
    { 0xacf3992a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13992d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b339935, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab3993c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f39942, SUC_lexicon_tags_15 },
    { 0x99739943, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f39948, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe53994d, SUC_lexicon_tags_49 },
    { 0x9d33994d, SUC_lexicon_tags_17 },
    { 0x8373994f, SUC_lexicon_tags_7 },
    { 0x4cb3994d, SUC_lexicon_tags_38 },
    { 0x48739951, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56139957, SUC_lexicon_tags_1 },
    { 0x4d739958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43f3995a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d3995e, SUC_lexicon_tags_11 },
    { 0x193995f, SUC_lexicon_tags_31 },
    { 0x10b3995f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c139967, SUC_lexicon_tags_87 },
    { 0xe0d39967, SUC_lexicon_tags_15 },
    { 0x93139967, SUC_lexicon_tags_25 },
    { 0x86f3996a, SUC_lexicon_tags_11 },
    { 0xb33996a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7f3996d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d39971, SUC_lexicon_tags_1 },
    { 0x339972, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf39975, SUC_lexicon_tags_15 },
    { 0x12b39976, SUC_lexicon_tags_2 },
    { 0x93139977, SUC_lexicon_tags_15 },
    { 0xd4539978, SUC_lexicon_tags_1 },
    { 0xdb539975, SUC_lexicon_tags_14 },
    { 0x2913997a, SUC_lexicon_tags_26 },
    { 0x42939976, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60f3997d, SUC_lexicon_tags_61 },
    { 0xe773997d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaed39980, SUC_lexicon_tags_9 },
    { 0x78b39980, SUC_lexicon_tags_29 },
    { 0xa7b39982, SUC_lexicon_tags_9 },
    { 0x6bf39983, SUC_lexicon_tags_31 },
    { 0x51939980, SUC_lexicon_tags_2 },
    { 0x72d39980, SUC_lexicon_tags_17 },
    { 0xfd339986, SUC_lexicon_tags_15 },
    { 0xbcb39981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8273998b, SUC_lexicon_tags_905 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf873998f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43139997, SUC_lexicon_tags_37 },
    { 0x35d39997, SUC_lexicon_tags_19 },
    { 0x11d39999, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa273999e, SUC_lexicon_tags_127 },
    { 0xf273999e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7399a2, SUC_lexicon_tags_31 },
    { 0x32f399a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825399b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7399bd, SUC_lexicon_tags_84 },
    { 0x87d399be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f399c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63399ca, SUC_lexicon_tags_151 },
    { 0x5b399ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151399d0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5399d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56d399d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1399d7, SUC_lexicon_tags_37 },
    { 0x7df399d8, SUC_lexicon_tags_77 },
    { 0x1c7399d9, SUC_lexicon_tags_81 },
    { 0x87f399d9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad399dd, SUC_lexicon_tags_8 },
    { 0x245399de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf399e3, SUC_lexicon_tags_17 },
    { 0x137399e4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f399e8, SUC_lexicon_tags_28 },
    { 0xfc9399e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x541399ec, SUC_lexicon_tags_25 },
    { 0x783399ed, SUC_lexicon_tags_1 },
    { 0x2a7399ee, SUC_lexicon_tags_20 },
    { 0x9f3399ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfd7399f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff399f5, SUC_lexicon_tags_17 },
    { 0xd7399f6, SUC_lexicon_tags_12 },
    { 0x589399f6, SUC_lexicon_tags_63 },
    { 0x203399f6, SUC_lexicon_tags_0 },
    { 0x967399f9, SUC_lexicon_tags_15 },
    { 0xa51399fa, SUC_lexicon_tags_9 },
    { 0xcd1399fb, SUC_lexicon_tags_31 },
    { 0x55b399fb, SUC_lexicon_tags_14 },
    { 0x9bb399f7, SUC_lexicon_tags_11 },
    { 0x77b399fa, SUC_lexicon_tags_13 },
    { 0x797399f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75539a01, SUC_lexicon_tags_33 },
    { 0x92539a01, SUC_lexicon_tags_29 },
    { 0x98139a02, SUC_lexicon_tags_37 },
    { 0x9d39a04, SUC_lexicon_tags_7 },
    { 0xff339a04, SUC_lexicon_tags_38 },
    { 0x38939a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b39a0b, SUC_lexicon_tags_402 },
    { 0x14139a0c, SUC_lexicon_tags_17 },
    { 0x21939a0d, SUC_lexicon_tags_7 },
    { 0x38b39a0e, SUC_lexicon_tags_71 },
    { 0x65b39a0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfdb39a11, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f39a16, SUC_lexicon_tags_0 },
    { 0x28d39a16, SUC_lexicon_tags_13 },
    { 0xc8939a18, SUC_lexicon_tags_518 },
    { 0xe9339a18, SUC_lexicon_tags_42 },
    { 0xe7939a19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82139a1d, SUC_lexicon_tags_43 },
    { 0x9cd39a1e, SUC_lexicon_tags_14 },
    { 0x8739a1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc1539a21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd39a28, SUC_lexicon_tags_11 },
    { 0x1d939a28, SUC_lexicon_tags_2 },
    { 0x2a939a28, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b39a33, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf339a3a, SUC_lexicon_tags_33 },
    { 0xe3f39a3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b739a41, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f39a45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c739a49, SUC_lexicon_tags_6 },
    { 0xb1b39a49, SUC_lexicon_tags_11 },
    { 0x9f539a4b, SUC_lexicon_tags_2 },
    { 0xd139a4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x51339a4f, SUC_lexicon_tags_11 },
    { 0x5939a50, SUC_lexicon_tags_0 },
    { 0xffd39a50, SUC_lexicon_tags_12 },
    { 0x10f39a52, SUC_lexicon_tags_17 },
    { 0xd2339a53, SUC_lexicon_tags_46 },
    { 0x51539a51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f39a5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8939a61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13939a6d, SUC_lexicon_tags_47 },
    { 0x31b39a6e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8f339a70, SUC_lexicon_tags_1 },
    { 0x2c939a71, SUC_lexicon_tags_77 },
    { 0x36d39a71, SUC_lexicon_tags_25 },
    { 0x79f39a71, SUC_lexicon_tags_17 },
    { 0xff39a74, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3739a7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f39a81, SUC_lexicon_tags_26 },
    { 0xa2b39a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37139a84, SUC_lexicon_tags_11 },
    { 0x4bf39a84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7339a89, SUC_lexicon_tags_25 },
    { 0xc1339a8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b39a90, SUC_lexicon_tags_14 },
    { 0x23f39a90, SUC_lexicon_tags_15 },
    { 0xf9139a90, SUC_lexicon_tags_2 },
    { 0x72539a92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f939a96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83139aa0, SUC_lexicon_tags_45 },
    { 0x2b39aa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb539aa4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3139aa7, SUC_lexicon_tags_81 },
    { 0x43539aa7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e339aac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb339aae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af39ab6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b39ab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7539abc, SUC_lexicon_tags_17 },
    { 0xccd39abd, SUC_lexicon_tags_39 },
    { 0xcfb39abd, SUC_lexicon_tags_15 },
    { 0x10d39abe, SUC_lexicon_tags_32 },
    { 0x2cd39ac0, SUC_lexicon_tags_45 },
    { 0x94939ac1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7bf39ac3, SUC_lexicon_tags_17 },
    { 0x6539ac4, SUC_lexicon_tags_25 },
    { 0x44b39ac5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64739ac8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8339ad1, SUC_lexicon_tags_0 },
    { 0x6939ad1, SUC_lexicon_tags_25 },
    { 0x82739ad3, SUC_lexicon_tags_0 },
    { 0xf7939ad4, SUC_lexicon_tags_17 },
    { 0x3f139ad1, SUC_lexicon_tags_25 },
    { 0xe6539ad2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb739ad8, SUC_lexicon_tags_30 },
    { 0xbb739ad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6939ade, SUC_lexicon_tags_15 },
    { 0xc6b39adf, SUC_lexicon_tags_37 },
    { 0xdbb39ae0, SUC_lexicon_tags_91 },
    { 0x7c539ae0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf39ae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb339ae8, SUC_lexicon_tags_27 },
    { 0xd9f39ae8, SUC_lexicon_tags_19 },
    { 0x32339aea, SUC_lexicon_tags_45 },
    { 0x1ab39aea, SUC_lexicon_tags_4 },
    { 0x2ff39aea, SUC_lexicon_tags_2 },
    { 0x36739aed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d139af0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1939af2, SUC_lexicon_tags_32 },
    { 0x35939af3, SUC_lexicon_tags_17 },
    { 0x88d39af4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd39afb, SUC_lexicon_tags_17 },
    { 0x4e739afc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f39b00, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e539b04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe739b09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87739b0e, SUC_lexicon_tags_12 },
    { 0xe9339b0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e139b13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1939b15, SUC_lexicon_tags_57 },
    { 0x23339b16, SUC_lexicon_tags_7 },
    { 0x94539b17, SUC_lexicon_tags_14 },
    { 0x16539b18, SUC_lexicon_tags_7 },
    { 0x739b19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0939b1f, SUC_lexicon_tags_12 },
    { 0x5f39b20, SUC_lexicon_tags_54 },
    { 0x7b139b21, SUC_lexicon_tags_0 },
    { 0x14939b21, SUC_lexicon_tags_12 },
    { 0x2ed39b23, SUC_lexicon_tags_68 },
    { 0xa4139b24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7539b27, SUC_lexicon_tags_18 },
    { 0x88339b27, SUC_lexicon_tags_17 },
    { 0x33939b28, SUC_lexicon_tags_15 },
    { 0x25f39b27, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b39b2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x91d39b30, SUC_lexicon_tags_57 },
    { 0x40739b31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4739b35, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d39b3a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd139b3d, SUC_lexicon_tags_54 },
    { 0xe4139b3d, SUC_lexicon_tags_19 },
    { 0x83539b3e, SUC_lexicon_tags_15 },
    { 0xd4139b3d, SUC_lexicon_tags_12 },
    { 0x97939b41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd39b45, SUC_lexicon_tags_17 },
    { 0xa7539b45, SUC_lexicon_tags_25 },
    { 0xcf139b45, SUC_lexicon_tags_27 },
    { 0x8cd39b47, SUC_lexicon_tags_47 },
    { 0xb539b48, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b39b4c, SUC_lexicon_tags_37 },
    { 0x12939b4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2939b50, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd0d39b52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8339b54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb39b57, SUC_lexicon_tags_45 },
    { 0x40539b58, SUC_lexicon_tags_376 },
    { 0xd4539b59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb39b5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d39b60, SUC_lexicon_tags_31 },
    { 0x50539b61, SUC_lexicon_tags_9 },
    { 0xd2539b62, SUC_lexicon_tags_2 },
    { 0xc1139b62, SUC_lexicon_tags_17 },
    { 0x66739b64, SUC_lexicon_tags_12 },
    { 0x95f39b64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4d39b67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe339b6c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf139b6f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde539b75, SUC_lexicon_tags_11 },
    { 0x95339b75, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x23739b79, SUC_lexicon_tags_45 },
    { 0x3ff39b7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd39b80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd339b83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf39b8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ad39b8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15739b91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25339b93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd539b96, SUC_lexicon_tags_37 },
    { 0xc2939b97, SUC_lexicon_tags_23 },
    { 0xbd339b97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33939b9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f39bb1, SUC_lexicon_tags_230 },
    { 0x9739bb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a939bba, SUC_lexicon_tags_17 },
    { 0x9f39bbb, SUC_lexicon_tags_17 },
    { 0x2a739bbb, SUC_lexicon_tags_16 },
    { 0x71539bbd, SUC_lexicon_tags_7 },
    { 0x8e139bbc, SUC_lexicon_tags_17 },
    { 0x18739bbf, SUC_lexicon_tags_47 },
    { 0xe139bc0, SUC_lexicon_tags_7 },
    { 0x9b939bc1, SUC_lexicon_tags_12 },
    { 0x13339bbe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xed739bc5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xed539bc8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa739bcc, SUC_lexicon_tags_43 },
    { 0xf6b39bcc, SUC_lexicon_tags_12 },
    { 0xdc539bce, SUC_lexicon_tags_18 },
    { 0xf4339bcd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f39bd4, SUC_lexicon_tags_33 },
    { 0x2bf39bd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b39bdc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7d39bde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb939be1, SUC_lexicon_tags_17 },
    { 0xc9139be2, SUC_lexicon_tags_1 },
    { 0xcf739be3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4139be8, SUC_lexicon_tags_8 },
    { 0xd0b39be9, SUC_lexicon_tags_63 },
    { 0x7c939be9, SUC_lexicon_tags_4 },
    { 0xe8b39beb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad39bf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca939bfb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70339c03, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b39c09, SUC_lexicon_tags_11 },
    { 0x38b39c0a, SUC_lexicon_tags_12 },
    { 0xfc939c09, SUC_lexicon_tags_15 },
    { 0xb6139c0c, SUC_lexicon_tags_11 },
    { 0x8e339c0d, SUC_lexicon_tags_66 },
    { 0xbdb39c0e, SUC_lexicon_tags_37 },
    { 0xb8d39c0e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb939c14, SUC_lexicon_tags_21 },
    { 0x12539c15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf139c1a, SUC_lexicon_tags_25 },
    { 0xc2d39c1b, SUC_lexicon_tags_33 },
    { 0x62d39c1c, SUC_lexicon_tags_4 },
    { 0x50939c1c, SUC_lexicon_tags_11 },
    { 0xd8939c1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x339c20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d39c23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69739c28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51939c2b, SUC_lexicon_tags_2 },
    { 0x8f139c2c, SUC_lexicon_tags_14 },
    { 0x98739c2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaf539c2f, SUC_lexicon_tags_1 },
    { 0x14f39c2f, SUC_lexicon_tags_1 },
    { 0x65739c31, SUC_lexicon_tags_9 },
    { 0xf2f39c32, SUC_lexicon_tags_8 },
    { 0x94139c32, SUC_lexicon_tags_10 },
    { 0xa8739c33, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x24339c37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f39c3a, SUC_lexicon_tags_38 },
    { 0xbeb39c3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97539c3e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcf339c40, SUC_lexicon_tags_45 },
    { 0xac339c41, SUC_lexicon_tags_17 },
    { 0x7cf39c40, SUC_lexicon_tags_1 },
    { 0x4b739c43, SUC_lexicon_tags_10 },
    { 0xe7539c44, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x23539c46, SUC_lexicon_tags_35 },
    { 0xd0d39c47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f539c4b, SUC_lexicon_tags_11 },
    { 0xe1739c4b, SUC_lexicon_tags_60 },
    { 0x9d339c4c, SUC_lexicon_tags_7 },
    { 0x29f39c4e, SUC_lexicon_tags_0 },
    { 0xc9139c4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27939c58, SUC_lexicon_tags_47 },
    { 0x9d139c59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff39c5c, SUC_lexicon_tags_25 },
    { 0xe939c5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8339c64, SUC_lexicon_tags_11 },
    { 0x8ff39c65, SUC_lexicon_tags_43 },
    { 0xa9b39c66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7139c69, SUC_lexicon_tags_15 },
    { 0x54539c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13139c6e, SUC_lexicon_tags_11 },
    { 0xeab39c6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e339c72, SUC_lexicon_tags_25 },
    { 0x4c539c72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20739c77, SUC_lexicon_tags_12 },
    { 0x4c739c78, SUC_lexicon_tags_25 },
    { 0x78939c79, SUC_lexicon_tags_92 },
    { 0x75f39c78, SUC_lexicon_tags_14 },
    { 0xf3539c78, SUC_lexicon_tags_7 },
    { 0x42d39c7c, SUC_lexicon_tags_37 },
    { 0x68f39c7c, SUC_lexicon_tags_4 },
    { 0xbe139c7c, SUC_lexicon_tags_1 },
    { 0xbb939c7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x71139c81, SUC_lexicon_tags_38 },
    { 0x88139c82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b39c85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0739c8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7939c90, SUC_lexicon_tags_27 },
    { 0x18339c91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f39c97, SUC_lexicon_tags_0 },
    { 0xb8b39c97, SUC_lexicon_tags_7 },
    { 0x55d39c98, SUC_lexicon_tags_44 },
    { 0xd8d39c97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd139c9c, SUC_lexicon_tags_37 },
    { 0x2f939c9d, SUC_lexicon_tags_61 },
    { 0xa5539c9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f39ca8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f939cad, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2a739caf, SUC_lexicon_tags_39 },
    { 0xfdd39cb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebd39cb2, SUC_lexicon_tags_11 },
    { 0x49539cb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb739cb6, SUC_lexicon_tags_0 },
    { 0x5bf39cb7, SUC_lexicon_tags_30 },
    { 0x85d39cb8, SUC_lexicon_tags_15 },
    { 0x92b39cb9, SUC_lexicon_tags_17 },
    { 0xddd39cb9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x75739cbc, SUC_lexicon_tags_38 },
    { 0x25539cbd, SUC_lexicon_tags_0 },
    { 0x5c539cbe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6f39cc0, SUC_lexicon_tags_9 },
    { 0x4539cc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e939cc4, SUC_lexicon_tags_17 },
    { 0x38339cc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb39cce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15739cd0, SUC_lexicon_tags_31 },
    { 0x62939cd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75539cd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef39cdd, SUC_lexicon_tags_47 },
    { 0xa4739cdd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76939ce4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebf39ce6, SUC_lexicon_tags_47 },
    { 0xe2139ce6, SUC_lexicon_tags_82 },
    { 0x76739ce8, SUC_lexicon_tags_12 },
    { 0x9e939ce9, SUC_lexicon_tags_61 },
    { 0x5f539cea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b39cee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x74139cf0, SUC_lexicon_tags_10 },
    { 0x15f39cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30339cf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x82f39cf5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5139cf8, SUC_lexicon_tags_26 },
    { 0x61539cf9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b539cff, SUC_lexicon_tags_33 },
    { 0x72939cff, SUC_lexicon_tags_25 },
    { 0x4939d01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6739d04, SUC_lexicon_tags_26 },
    { 0x30139d04, SUC_lexicon_tags_41 },
    { 0x3d39d06, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x53539d08, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xedb39d0a, SUC_lexicon_tags_17 },
    { 0xb1139d0a, SUC_lexicon_tags_27 },
    { 0x51139d0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe339d12, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdd39d14, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b39d1a, SUC_lexicon_tags_2 },
    { 0xf1739d1b, SUC_lexicon_tags_23 },
    { 0xc539d1c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x29739d1e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab39d23, SUC_lexicon_tags_45 },
    { 0xf9f39d24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0539d28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64939d2d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9cd39d2f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xfe139d31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd39d35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29939d3b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1939d3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8139d41, SUC_lexicon_tags_15 },
    { 0x1ed39d41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f39d4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5739d4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69339d51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe539d58, SUC_lexicon_tags_1 },
    { 0x27939d58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85739d5b, SUC_lexicon_tags_9 },
    { 0x83d39d5c, SUC_lexicon_tags_31 },
    { 0xa739d5d, SUC_lexicon_tags_47 },
    { 0xd3739d5e, SUC_lexicon_tags_2 },
    { 0xa7539d5d, SUC_lexicon_tags_37 },
    { 0x87b39d5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f139d69, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b739d6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd39d6f, SUC_lexicon_tags_47 },
    { 0x70d39d70, SUC_lexicon_tags_118 },
    { 0x84739d71, SUC_lexicon_tags_2 },
    { 0xf39d6f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x44939d74, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87939d7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e339d7e, SUC_lexicon_tags_9 },
    { 0xf2739d7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82739d82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b39d86, SUC_lexicon_tags_7 },
    { 0x2c739d86, SUC_lexicon_tags_37 },
    { 0xc8f39d88, SUC_lexicon_tags_11 },
    { 0xc0f39d87, SUC_lexicon_tags_30 },
    { 0xf5339d8a, SUC_lexicon_tags_17 },
    { 0xd9139d8b, SUC_lexicon_tags_1 },
    { 0xe2939d8c, SUC_lexicon_tags_38 },
    { 0x24339d8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8339d91, SUC_lexicon_tags_14 },
    { 0xdd539d92, SUC_lexicon_tags_15 },
    { 0x18339d92, SUC_lexicon_tags_8 },
    { 0x86939d94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f39d9a, SUC_lexicon_tags_4 },
    { 0x12539d9b, SUC_lexicon_tags_204 },
    { 0x73739d9c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b39da3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7939da5, SUC_lexicon_tags_21 },
    { 0xf339da5, SUC_lexicon_tags_8 },
    { 0xf5739da7, SUC_lexicon_tags_0 },
    { 0xdeb39da8, SUC_lexicon_tags_37 },
    { 0x74539da7, SUC_lexicon_tags_11 },
    { 0xf4139da9, SUC_lexicon_tags_28 },
    { 0xca539da5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b39dae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f39db1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2139db4, SUC_lexicon_tags_32 },
    { 0x64d39db5, SUC_lexicon_tags_11 },
    { 0xfdd39db5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f539dba, SUC_lexicon_tags_17 },
    { 0xbd339dbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7939dbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73139dc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1339dc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d139dc5, SUC_lexicon_tags_15 },
    { 0x2b539dc5, SUC_lexicon_tags_26 },
    { 0x57d39dc5, SUC_lexicon_tags_11 },
    { 0xa6739dc5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4b39dca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45339dd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9739dd3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0739dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b339dda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f939ddc, SUC_lexicon_tags_37 },
    { 0xa939ddd, SUC_lexicon_tags_45 },
    { 0x2a139dde, SUC_lexicon_tags_10 },
    { 0x43739ddf, SUC_lexicon_tags_43 },
    { 0xedd39de0, SUC_lexicon_tags_17 },
    { 0xe8339ddf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16539de7, SUC_lexicon_tags_16 },
    { 0x91339de8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2bd39dea, SUC_lexicon_tags_0 },
    { 0xd8339deb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x21b39ded, SUC_lexicon_tags_11 },
    { 0x11739dee, SUC_lexicon_tags_12 },
    { 0xb5739def, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a339df1, SUC_lexicon_tags_2 },
    { 0xa9339df2, SUC_lexicon_tags_1 },
    { 0xb8f39df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35739df5, SUC_lexicon_tags_8 },
    { 0x6d939df5, SUC_lexicon_tags_38 },
    { 0x67539df7, SUC_lexicon_tags_60 },
    { 0xa1f39df8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6339e00, SUC_lexicon_tags_0 },
    { 0x91739e01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20b39e03, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xedb39e05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b139e0a, SUC_lexicon_tags_28 },
    { 0xaaf39e0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d39e0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67139e11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9339e14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb39e1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6d39e1c, SUC_lexicon_tags_1 },
    { 0xd8d39e1d, SUC_lexicon_tags_46 },
    { 0xbb539e1c, SUC_lexicon_tags_82 },
    { 0x46739e1d, SUC_lexicon_tags_23 },
    { 0x74b39e1d, SUC_lexicon_tags_0 },
    { 0xea739e1c, SUC_lexicon_tags_1 },
    { 0x9db39e21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2739e25, SUC_lexicon_tags_60 },
    { 0xb5339e26, SUC_lexicon_tags_103 },
    { 0x5cd39e27, SUC_lexicon_tags_15 },
    { 0xb5f39e28, SUC_lexicon_tags_12 },
    { 0xd2539e29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xabb39e2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56539e2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa339e30, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf39e36, SUC_lexicon_tags_9 },
    { 0x5a339e37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7539e3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde739e3e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x87539e41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57b39e43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f39e46, SUC_lexicon_tags_65 },
    { 0x4f539e47, SUC_lexicon_tags_60 },
    { 0x73139e47, SUC_lexicon_tags_0 },
    { 0x8a339e49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5339e50, SUC_lexicon_tags_7 },
    { 0x63b39e51, SUC_lexicon_tags_37 },
    { 0x9739e51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5939e56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8539e59, SUC_lexicon_tags_47 },
    { 0x96339e59, SUC_lexicon_tags_25 },
    { 0x8d939e5b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7539e5e, SUC_lexicon_tags_68 },
    { 0x36739e5f, SUC_lexicon_tags_2 },
    { 0x45339e60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c339e64, SUC_lexicon_tags_11 },
    { 0xaa339e65, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed939e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd39e71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf39e73, SUC_lexicon_tags_12 },
    { 0xa3f39e74, SUC_lexicon_tags_37 },
    { 0x7b139e73, SUC_lexicon_tags_1 },
    { 0x3ef39e73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22539e78, SUC_lexicon_tags_15 },
    { 0x63f39e79, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b539e7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35539e7d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d39e82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd39e86, SUC_lexicon_tags_6 },
    { 0x9eb39e87, SUC_lexicon_tags_17 },
    { 0xdc339e87, SUC_lexicon_tags_63 },
    { 0x68539e89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd39e90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb39e93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19f39e95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b339e98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf39e9b, SUC_lexicon_tags_244 },
    { 0x85539e9b, SUC_lexicon_tags_1 },
    { 0xa5f39e9c, SUC_lexicon_tags_45 },
    { 0xc0b39e9c, SUC_lexicon_tags_27 },
    { 0x54939e9d, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53539ea7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67539ea9, SUC_lexicon_tags_45 },
    { 0xb2539ea9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ff39eac, SUC_lexicon_tags_11 },
    { 0xbb939eac, SUC_lexicon_tags_7 },
    { 0x89339eae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20539eb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe539eb8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa539ebb, SUC_lexicon_tags_588 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8539ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d39ec5, SUC_lexicon_tags_63 },
    { 0x1e539ec5, SUC_lexicon_tags_17 },
    { 0x2db39ec7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9339ecb, SUC_lexicon_tags_36 },
    { 0xa8d39ecb, SUC_lexicon_tags_7 },
    { 0x5e739ecc, SUC_lexicon_tags_17 },
    { 0x65339ece, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd39ed1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d39edf, SUC_lexicon_tags_12 },
    { 0xd0339ee0, SUC_lexicon_tags_27 },
    { 0x2c939ee0, SUC_lexicon_tags_55 },
    { 0xf9d39ee2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa2139ee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20139ee8, SUC_lexicon_tags_34 },
    { 0xceb39ee9, SUC_lexicon_tags_28 },
    { 0x63339ee9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff139eec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f39ef0, SUC_lexicon_tags_38 },
    { 0x8b539ef1, SUC_lexicon_tags_60 },
    { 0x45939ef2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f539ef9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x22d39efb, SUC_lexicon_tags_63 },
    { 0x82b39efc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d139eff, SUC_lexicon_tags_28 },
    { 0x86739eff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9939f07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed139f0a, SUC_lexicon_tags_11 },
    { 0x27739f0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f39f0f, SUC_lexicon_tags_63 },
    { 0x85b39f10, SUC_lexicon_tags_17 },
    { 0x37339f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb739f19, SUC_lexicon_tags_9 },
    { 0xe9939f1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14339f1c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb2539f1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f39f21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f939f24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8939f26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1d939f28, SUC_lexicon_tags_15 },
    { 0xf9139f28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb939f2c, SUC_lexicon_tags_5 },
    { 0x4f339f2c, SUC_lexicon_tags_11 },
    { 0xa8139f2e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f39f33, SUC_lexicon_tags_11 },
    { 0xe1939f33, SUC_lexicon_tags_87 },
    { 0xdb739f35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6139f38, SUC_lexicon_tags_60 },
    { 0x2d939f39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d39f40, SUC_lexicon_tags_46 },
    { 0x91939f41, SUC_lexicon_tags_15 },
    { 0xc4139f42, SUC_lexicon_tags_45 },
    { 0x58d39f43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f39f49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9339f4f, SUC_lexicon_tags_37 },
    { 0x61b39f4f, SUC_lexicon_tags_46 },
    { 0xfe539f51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2939f53, SUC_lexicon_tags_28 },
    { 0xffb39f54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90739f56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc739f5c, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde139f63, SUC_lexicon_tags_65 },
    { 0x9d39f64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd39f68, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe339f6d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15539f73, SUC_lexicon_tags_37 },
    { 0xfed39f74, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34939f77, SUC_lexicon_tags_39 },
    { 0x19339f77, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c739f7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b939f7e, SUC_lexicon_tags_11 },
    { 0x57739f7f, SUC_lexicon_tags_32 },
    { 0x58139f80, SUC_lexicon_tags_33 },
    { 0x8a139f80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbed39f83, SUC_lexicon_tags_11 },
    { 0xbcb39f84, SUC_lexicon_tags_1 },
    { 0x64739f84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb39f89, SUC_lexicon_tags_37 },
    { 0xa3539f89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46139f8f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x37739f92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b39f95, SUC_lexicon_tags_11 },
    { 0x95139f95, SUC_lexicon_tags_25 },
    { 0x35d39f97, SUC_lexicon_tags_17 },
    { 0x2fb39f97, SUC_lexicon_tags_48 },
    { 0x26939f96, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3c339f9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf139f9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36f39fa0, SUC_lexicon_tags_1 },
    { 0x84d39fa0, SUC_lexicon_tags_11 },
    { 0x3f539fa2, SUC_lexicon_tags_11 },
    { 0x9d739fa3, SUC_lexicon_tags_37 },
    { 0x4f739fa4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf939fa6, SUC_lexicon_tags_39 },
    { 0x2b539fa6, SUC_lexicon_tags_15 },
    { 0x4b139fa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77939faa, SUC_lexicon_tags_29 },
    { 0x4139fab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a339fae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac339fb2, SUC_lexicon_tags_15 },
    { 0xfe939fb3, SUC_lexicon_tags_31 },
    { 0x9939fb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f139fb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b39fbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c339fbe, SUC_lexicon_tags_46 },
    { 0xcc739fbe, SUC_lexicon_tags_27 },
    { 0x19739fc0, SUC_lexicon_tags_1253 },
    { 0xe4f39fbf, SUC_lexicon_tags_11 },
    { 0xd0139fbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4139fc5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd39fca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74939fcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43d39fd1, SUC_lexicon_tags_7 },
    { 0x81539fd2, SUC_lexicon_tags_26 },
    { 0x14d39fd2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x84739fd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59f39fd7, SUC_lexicon_tags_19 },
    { 0xb4739fd8, SUC_lexicon_tags_11 },
    { 0xbd739fd9, SUC_lexicon_tags_37 },
    { 0xb139fd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d39fe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d39fe4, SUC_lexicon_tags_25 },
    { 0xe6539fe5, SUC_lexicon_tags_37 },
    { 0x6f939fe6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2539fe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b39fed, SUC_lexicon_tags_37 },
    { 0x57339fed, SUC_lexicon_tags_69 },
    { 0x4c339fef, SUC_lexicon_tags_12 },
    { 0x8f39ff0, SUC_lexicon_tags_45 },
    { 0x96b39ff1, SUC_lexicon_tags_8 },
    { 0xfc939ff2, SUC_lexicon_tags_37 },
    { 0xd6d39ff2, SUC_lexicon_tags_11 },
    { 0x70d39ff4, SUC_lexicon_tags_6 },
    { 0x3dd39ff5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b39ff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c539ffb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea739ffe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb173a003, SUC_lexicon_tags_11 },
    { 0xe3d3a003, SUC_lexicon_tags_8 },
    { 0xeb53a005, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b33a009, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793a00d, SUC_lexicon_tags_38 },
    { 0xf613a00e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4d3a010, SUC_lexicon_tags_17 },
    { 0xf8f3a011, SUC_lexicon_tags_30 },
    { 0xdd93a011, SUC_lexicon_tags_65 },
    { 0xb773a013, SUC_lexicon_tags_11 },
    { 0x93a014, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefb3a016, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d3a01f, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x64b3a021, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb3a024, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed73a02d, SUC_lexicon_tags_68 },
    { 0xf813a02d, SUC_lexicon_tags_15 },
    { 0x3653a02e, SUC_lexicon_tags_27 },
    { 0x6433a030, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a73a034, SUC_lexicon_tags_4 },
    { 0xb1d3a035, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb973a038, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3ff3a03a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d3a03d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b3a040, SUC_lexicon_tags_1 },
    { 0x3b13a041, SUC_lexicon_tags_12 },
    { 0xd393a042, SUC_lexicon_tags_11 },
    { 0x1b73a042, SUC_lexicon_tags_47 },
    { 0xdb3a044, SUC_lexicon_tags_16 },
    { 0xaef3a044, SUC_lexicon_tags_14 },
    { 0xf3b3a041, SUC_lexicon_tags_8 },
    { 0xdb73a042, SUC_lexicon_tags_12 },
    { 0x9b13a048, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3a04e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ad3a050, SUC_lexicon_tags_2 },
    { 0xbe93a051, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33a054, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5233a057, SUC_lexicon_tags_6 },
    { 0xcd3a058, SUC_lexicon_tags_38 },
    { 0x6473a057, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7133a061, SUC_lexicon_tags_5 },
    { 0xa2f3a062, SUC_lexicon_tags_8 },
    { 0xcab3a063, SUC_lexicon_tags_11 },
    { 0x7113a063, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5993a068, SUC_lexicon_tags_25 },
    { 0x6213a068, SUC_lexicon_tags_16 },
    { 0x5c93a06a, SUC_lexicon_tags_11 },
    { 0x28f3a068, SUC_lexicon_tags_2 },
    { 0x6813a06b, SUC_lexicon_tags_17 },
    { 0xd153a069, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f3a070, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0d3a072, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc93a079, SUC_lexicon_tags_1 },
    { 0x8813a07a, SUC_lexicon_tags_40 },
    { 0x6793a07b, SUC_lexicon_tags_35 },
    { 0x133a079, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d3a07f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2833a083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b3a090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28b3a092, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5eb3a094, SUC_lexicon_tags_2 },
    { 0xd93a094, SUC_lexicon_tags_43 },
    { 0xf0f3a096, SUC_lexicon_tags_19 },
    { 0xfbd3a096, SUC_lexicon_tags_14 },
    { 0x7533a098, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8953a09a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd3a0a4, SUC_lexicon_tags_25 },
    { 0x96b3a0a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab73a0a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73a0ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6f3a0af, SUC_lexicon_tags_60 },
    { 0x7bb3a0b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb973a0b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9793a0b9, SUC_lexicon_tags_2 },
    { 0x6c13a0ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab73a0c0, SUC_lexicon_tags_3 },
    { 0x4793a0c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3a0c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a13a0cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa973a0cf, SUC_lexicon_tags_92 },
    { 0x8ff3a0cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2553a0d7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa313a0da, SUC_lexicon_tags_66 },
    { 0x9493a0db, SUC_lexicon_tags_19 },
    { 0x1d33a0db, SUC_lexicon_tags_17 },
    { 0xaff3a0dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb3a0e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82f3a0e4, SUC_lexicon_tags_25 },
    { 0xc713a0e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b3a0e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda73a0ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x72b3a0ef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c13a0f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2fd3a0f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe573a0fd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9553a101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f3a106, SUC_lexicon_tags_4 },
    { 0x62f3a106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b3a10b, SUC_lexicon_tags_63 },
    { 0x2833a10c, SUC_lexicon_tags_1 },
    { 0x8793a10b, SUC_lexicon_tags_17 },
    { 0x3c53a10d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4873a111, SUC_lexicon_tags_14 },
    { 0xba13a112, SUC_lexicon_tags_11 },
    { 0x2133a111, SUC_lexicon_tags_37 },
    { 0x3713a114, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9473a11a, SUC_lexicon_tags_0 },
    { 0xb873a11b, SUC_lexicon_tags_16 },
    { 0x7e73a11c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2153a11e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef3a125, SUC_lexicon_tags_4 },
    { 0x2873a126, SUC_lexicon_tags_47 },
    { 0x9cb3a125, SUC_lexicon_tags_82 },
    { 0xb293a128, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a33a132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29f3a134, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe9d3a136, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f3a13a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe093a13c, SUC_lexicon_tags_0 },
    { 0x8c53a13c, SUC_lexicon_tags_7 },
    { 0x1e53a13c, SUC_lexicon_tags_47 },
    { 0x46b3a13f, SUC_lexicon_tags_31 },
    { 0xf9d3a140, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20d3a142, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xeb13a144, SUC_lexicon_tags_17 },
    { 0x9713a145, SUC_lexicon_tags_68 },
    { 0xded3a146, SUC_lexicon_tags_60 },
    { 0xb713a147, SUC_lexicon_tags_45 },
    { 0x5433a148, SUC_lexicon_tags_37 },
    { 0xab73a148, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc73a14f, SUC_lexicon_tags_27 },
    { 0xd373a150, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb3a153, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c13a158, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f53a15a, SUC_lexicon_tags_43 },
    { 0xd393a15b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6073a15d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6393a15f, SUC_lexicon_tags_17 },
    { 0xa153a160, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8973a163, SUC_lexicon_tags_17 },
    { 0x3813a164, SUC_lexicon_tags_25 },
    { 0xb513a164, SUC_lexicon_tags_11 },
    { 0x7e33a164, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba53a16d, SUC_lexicon_tags_28 },
    { 0xcd93a16d, SUC_lexicon_tags_15 },
    { 0x55f3a16f, SUC_lexicon_tags_9 },
    { 0xd1f3a170, SUC_lexicon_tags_37 },
    { 0x4213a16f, SUC_lexicon_tags_25 },
    { 0x6893a172, SUC_lexicon_tags_4 },
    { 0xdff3a173, SUC_lexicon_tags_10 },
    { 0x5df3a173, SUC_lexicon_tags_46 },
    { 0x7393a170, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a73a177, SUC_lexicon_tags_133 },
    { 0xf973a178, SUC_lexicon_tags_21 },
    { 0xda53a179, SUC_lexicon_tags_156 },
    { 0x85f3a178, SUC_lexicon_tags_29 },
    { 0x5693a177, SUC_lexicon_tags_2 },
    { 0x6313a17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c93a180, SUC_lexicon_tags_68 },
    { 0x7a13a181, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8113a189, SUC_lexicon_tags_11 },
    { 0xdc73a189, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713a18f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7933a192, SUC_lexicon_tags_15 },
    { 0xd733a192, SUC_lexicon_tags_7 },
    { 0x5233a192, SUC_lexicon_tags_17 },
    { 0x9413a195, SUC_lexicon_tags_2 },
    { 0xccd3a196, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7773a199, SUC_lexicon_tags_7 },
    { 0x25f3a19a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc13a19d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa1f3a19f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd3a1a6, SUC_lexicon_tags_24 },
    { 0xbc33a1a7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xdfd3a1a9, SUC_lexicon_tags_40 },
    { 0xe9f3a1aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f3a1ae, SUC_lexicon_tags_9 },
    { 0xedb3a1ae, SUC_lexicon_tags_25 },
    { 0x7173a1b0, SUC_lexicon_tags_14 },
    { 0x44f3a1b1, SUC_lexicon_tags_232 },
    { 0x4553a1af, SUC_lexicon_tags_27 },
    { 0x5f3a1b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7ab3a1b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeed3a1b7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6673a1b9, SUC_lexicon_tags_38 },
    { 0xded3a1b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x25b3a1bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b3a1c5, SUC_lexicon_tags_14 },
    { 0x92f3a1c6, SUC_lexicon_tags_19 },
    { 0x4473a1c7, SUC_lexicon_tags_66 },
    { 0x45f3a1c8, SUC_lexicon_tags_12 },
    { 0xaff3a1c8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb173a1cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc313a1d1, SUC_lexicon_tags_46 },
    { 0x84b3a1d1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf3a1d6, SUC_lexicon_tags_45 },
    { 0xee33a1d7, SUC_lexicon_tags_11 },
    { 0xc913a1d8, SUC_lexicon_tags_1 },
    { 0xfb93a1d9, SUC_lexicon_tags_19 },
    { 0xd253a1d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab53a1dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4553a1e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadd3a1e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x413a1e5, SUC_lexicon_tags_11 },
    { 0xc333a1e5, SUC_lexicon_tags_15 },
    { 0xacb3a1e6, SUC_lexicon_tags_49 },
    { 0x8d73a1e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff3a1ec, SUC_lexicon_tags_23 },
    { 0x853a1ed, SUC_lexicon_tags_4 },
    { 0x9d13a1ee, SUC_lexicon_tags_2 },
    { 0xe5f3a1ed, SUC_lexicon_tags_15 },
    { 0x5d73a1f0, SUC_lexicon_tags_11 },
    { 0x8bf3a1ef, SUC_lexicon_tags_15 },
    { 0x8773a1ee, SUC_lexicon_tags_25 },
    { 0x1273a1f3, SUC_lexicon_tags_30 },
    { 0x5473a1f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7133a1f6, SUC_lexicon_tags_33 },
    { 0xc913a1f6, SUC_lexicon_tags_19 },
    { 0xbd93a1f8, SUC_lexicon_tags_17 },
    { 0x8c13a1f9, SUC_lexicon_tags_1 },
    { 0x81d3a1f9, SUC_lexicon_tags_28 },
    { 0x2313a1fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1053a1ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7853a201, SUC_lexicon_tags_7 },
    { 0x1b73a202, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f3a205, SUC_lexicon_tags_25 },
    { 0xb8b3a206, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d3a212, SUC_lexicon_tags_7 },
    { 0x5973a213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f3a219, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e93a222, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5553a228, SUC_lexicon_tags_11 },
    { 0xc313a229, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f3a22e, SUC_lexicon_tags_31 },
    { 0x8d53a22f, SUC_lexicon_tags_8 },
    { 0xaf73a230, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa593a236, SUC_lexicon_tags_17 },
    { 0x29d3a237, SUC_lexicon_tags_7 },
    { 0x7bd3a238, SUC_lexicon_tags_55 },
    { 0x1fd3a238, SUC_lexicon_tags_12 },
    { 0x7bd3a237, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa2b3a23c, SUC_lexicon_tags_11 },
    { 0x4093a23d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f3a240, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb53a242, SUC_lexicon_tags_10 },
    { 0xad53a243, SUC_lexicon_tags_32 },
    { 0x153a243, SUC_lexicon_tags_0 },
    { 0xd4d3a244, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc53a247, SUC_lexicon_tags_11 },
    { 0xf613a247, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae53a24a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc053a24e, SUC_lexicon_tags_5 },
    { 0x9213a24f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb33a253, SUC_lexicon_tags_12 },
    { 0x2cd3a253, SUC_lexicon_tags_2 },
    { 0x7e93a254, SUC_lexicon_tags_7 },
    { 0x3993a255, SUC_lexicon_tags_1 },
    { 0x3f93a255, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3333a25c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x28d3a25e, SUC_lexicon_tags_9 },
    { 0x35d3a25e, SUC_lexicon_tags_15 },
    { 0x3e13a25e, SUC_lexicon_tags_9 },
    { 0x3913a260, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe853a263, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a33a267, SUC_lexicon_tags_25 },
    { 0x84b3a268, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c73a26b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3a271, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7853a273, SUC_lexicon_tags_11 },
    { 0x48b3a274, SUC_lexicon_tags_37 },
    { 0xb1d3a274, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf873a277, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba93a27c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d3a27f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82b3a281, SUC_lexicon_tags_60 },
    { 0xed33a282, SUC_lexicon_tags_15 },
    { 0x2013a283, SUC_lexicon_tags_28 },
    { 0xb273a281, SUC_lexicon_tags_23 },
    { 0x15d3a282, SUC_lexicon_tags_7 },
    { 0x55d3a286, SUC_lexicon_tags_7 },
    { 0xee53a284, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b3a28a, SUC_lexicon_tags_25 },
    { 0x1b13a28a, SUC_lexicon_tags_2 },
    { 0xcdd3a28c, SUC_lexicon_tags_23 },
    { 0x12b3a28c, SUC_lexicon_tags_68 },
    { 0xc473a28e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe213a295, SUC_lexicon_tags_2 },
    { 0x2a33a296, SUC_lexicon_tags_39 },
    { 0x2b13a297, SUC_lexicon_tags_23 },
    { 0x79b3a297, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6913a29c, SUC_lexicon_tags_12 },
    { 0x9af3a29d, SUC_lexicon_tags_11 },
    { 0x93f3a29e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6933a2a1, SUC_lexicon_tags_77 },
    { 0x1193a2a2, SUC_lexicon_tags_2 },
    { 0xca13a2a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe13a2a8, SUC_lexicon_tags_60 },
    { 0xcfd3a2a9, SUC_lexicon_tags_8 },
    { 0x8d53a2a9, SUC_lexicon_tags_7 },
    { 0x5fd3a2ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec53a2ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf13a2b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf33a2b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc93a2bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa133a2bd, SUC_lexicon_tags_37 },
    { 0x1893a2bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8813a2c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f93a2c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce93a2d1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c13a2d6, SUC_lexicon_tags_65 },
    { 0x5e93a2d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513a2db, SUC_lexicon_tags_40 },
    { 0xefd3a2dc, SUC_lexicon_tags_17 },
    { 0x59b3a2dd, SUC_lexicon_tags_17 },
    { 0x5333a2dd, SUC_lexicon_tags_11 },
    { 0x6c53a2df, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x49f3a2e1, SUC_lexicon_tags_11 },
    { 0x48d3a2e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373a2e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4873a2ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdcb3a2ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1b3a2ee, SUC_lexicon_tags_32 },
    { 0xc053a2ef, SUC_lexicon_tags_17 },
    { 0xbd53a2f0, SUC_lexicon_tags_37 },
    { 0x5733a2f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a2f8, SUC_lexicon_tags_27 },
    { 0x68d3a2f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d93a2fc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x153a2fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf3a301, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3f3a303, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d3a306, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xa393a308, SUC_lexicon_tags_15 },
    { 0x1573a308, SUC_lexicon_tags_1 },
    { 0x7053a30a, SUC_lexicon_tags_37 },
    { 0xd473a309, SUC_lexicon_tags_23 },
    { 0x3e73a30b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f3a30f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d3a312, SUC_lexicon_tags_2 },
    { 0x4193a312, SUC_lexicon_tags_49 },
    { 0x1433a314, SUC_lexicon_tags_2 },
    { 0xefb3a315, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3213a318, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xc793a31b, SUC_lexicon_tags_56 },
    { 0xd2d3a31c, SUC_lexicon_tags_60 },
    { 0xed3a31d, SUC_lexicon_tags_7 },
    { 0x5e33a31e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d3a321, SUC_lexicon_tags_17 },
    { 0xc253a322, SUC_lexicon_tags_28 },
    { 0x34d3a323, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3393a327, SUC_lexicon_tags_37 },
    { 0xe413a327, SUC_lexicon_tags_17 },
    { 0xe193a329, SUC_lexicon_tags_1 },
    { 0x6273a329, SUC_lexicon_tags_37 },
    { 0xff73a328, SUC_lexicon_tags_30 },
    { 0x4633a32c, SUC_lexicon_tags_33 },
    { 0xfbb3a32d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f3a331, SUC_lexicon_tags_37 },
    { 0xc373a332, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4973a334, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f3a337, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d3a33d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d3a341, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9593a343, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f3a34a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf53a34c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6593a350, SUC_lexicon_tags_17 },
    { 0x9873a351, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9f73a353, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3a35a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b3a35d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe33a360, SUC_lexicon_tags_12 },
    { 0x7693a361, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913a369, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb893a36c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb33a370, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7073a374, SUC_lexicon_tags_45 },
    { 0x8af3a375, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc173a37a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6573a37c, SUC_lexicon_tags_15 },
    { 0xf633a37d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4453a37f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253a386, SUC_lexicon_tags_19 },
    { 0xc2b3a387, SUC_lexicon_tags_13 },
    { 0xca93a388, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x173a38a, SUC_lexicon_tags_37 },
    { 0x3493a38a, SUC_lexicon_tags_6 },
    { 0xe753a38c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33a392, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccb3a394, SUC_lexicon_tags_1 },
    { 0x3fd3a395, SUC_lexicon_tags_11 },
    { 0xc4b3a396, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6553a39a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x98f3a39c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1513a39e, SUC_lexicon_tags_23 },
    { 0x9253a39e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed33a3a1, SUC_lexicon_tags_7 },
    { 0x9013a3a1, SUC_lexicon_tags_1 },
    { 0xb613a3a3, SUC_lexicon_tags_43 },
    { 0x1a33a3a2, SUC_lexicon_tags_9 },
    { 0xa913a3a5, SUC_lexicon_tags_15 },
    { 0x8493a3a6, SUC_lexicon_tags_2 },
    { 0x40b3a3a7, SUC_lexicon_tags_26 },
    { 0x4033a3a7, SUC_lexicon_tags_2 },
    { 0x7633a3a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe293a3ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb3a3b3, SUC_lexicon_tags_37 },
    { 0x57b3a3b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc873a3bc, SUC_lexicon_tags_25 },
    { 0x36b3a3bc, SUC_lexicon_tags_27 },
    { 0xa4f3a3be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b3a3c4, SUC_lexicon_tags_43 },
    { 0x7893a3c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4693a3c7, SUC_lexicon_tags_22 },
    { 0x3fb3a3c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d3a3cb, SUC_lexicon_tags_11 },
    { 0xb853a3cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93b3a3ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f3a3d1, SUC_lexicon_tags_25 },
    { 0xcdb3a3d2, SUC_lexicon_tags_33 },
    { 0xf993a3d3, SUC_lexicon_tags_25 },
    { 0xad73a3d3, SUC_lexicon_tags_27 },
    { 0x2c93a3d3, SUC_lexicon_tags_47 },
    { 0xfd73a3d3, SUC_lexicon_tags_21 },
    { 0xefb3a3d5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef3a3db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x50f3a3dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8133a3e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a33a3e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6773a3f2, SUC_lexicon_tags_0 },
    { 0xdcb3a3f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3733a3f8, SUC_lexicon_tags_11 },
    { 0x37d3a3f9, SUC_lexicon_tags_17 },
    { 0xa13a3fa, SUC_lexicon_tags_15 },
    { 0x2533a3fb, SUC_lexicon_tags_17 },
    { 0xaa93a3fb, SUC_lexicon_tags_12 },
    { 0xc013a3fa, SUC_lexicon_tags_11 },
    { 0x1393a3fd, SUC_lexicon_tags_25 },
    { 0x83b3a3f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7d3a401, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d3a404, SUC_lexicon_tags_9 },
    { 0x4cb3a405, SUC_lexicon_tags_2 },
    { 0x353a406, SUC_lexicon_tags_60 },
    { 0xc0b3a407, SUC_lexicon_tags_30 },
    { 0x2b13a404, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3273a40a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3a40e, SUC_lexicon_tags_54 },
    { 0xf753a40f, SUC_lexicon_tags_47 },
    { 0x9033a40e, SUC_lexicon_tags_38 },
    { 0x11f3a411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd3a417, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b3a41a, SUC_lexicon_tags_39 },
    { 0xf1d3a41b, SUC_lexicon_tags_17 },
    { 0x55d3a41b, SUC_lexicon_tags_27 },
    { 0xfb93a41d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7a13a41f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f3a424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9053a427, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8933a42c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x47b3a42e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce13a432, SUC_lexicon_tags_65 },
    { 0x2b53a432, SUC_lexicon_tags_1 },
    { 0x5ff3a434, SUC_lexicon_tags_26 },
    { 0x61f3a434, SUC_lexicon_tags_37 },
    { 0xdb3a436, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7193a438, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2293a43d, SUC_lexicon_tags_31 },
    { 0xe3b3a43d, SUC_lexicon_tags_17 },
    { 0x6093a43f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe053a444, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d13a448, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc13a44a, SUC_lexicon_tags_1 },
    { 0xecb3a44b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb73a457, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f33a45c, SUC_lexicon_tags_17 },
    { 0xb893a45d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79d3a45f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b93a462, SUC_lexicon_tags_1 },
    { 0xa33a462, SUC_lexicon_tags_14 },
    { 0xc893a464, SUC_lexicon_tags_59 },
    { 0xbdb3a464, SUC_lexicon_tags_57 },
    { 0x93a466, SUC_lexicon_tags_29 },
    { 0xd2d3a466, SUC_lexicon_tags_5 },
    { 0x8693a466, SUC_lexicon_tags_49 },
    { 0x17d3a467, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca53a46d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9273a476, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f3a47c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6173a47e, SUC_lexicon_tags_7 },
    { 0xbab3a47f, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x7133a481, SUC_lexicon_tags_12 },
    { 0xe853a482, SUC_lexicon_tags_60 },
    { 0x2ef3a482, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab3a488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab3a48d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d3a493, SUC_lexicon_tags_37 },
    { 0xdcb3a493, SUC_lexicon_tags_7 },
    { 0x5673a495, SUC_lexicon_tags_17 },
    { 0x4773a496, SUC_lexicon_tags_19 },
    { 0x7db3a496, SUC_lexicon_tags_12 },
    { 0x1ad3a498, SUC_lexicon_tags_1 },
    { 0xc133a499, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7233a49d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4033a49f, SUC_lexicon_tags_37 },
    { 0x1373a4a0, SUC_lexicon_tags_69 },
    { 0x49f3a4a1, SUC_lexicon_tags_7 },
    { 0x5d33a4a2, SUC_lexicon_tags_11 },
    { 0x72b3a4a3, SUC_lexicon_tags_15 },
    { 0xa613a4a3, SUC_lexicon_tags_6 },
    { 0x1393a4a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd3a4aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9f3a4ac, SUC_lexicon_tags_68 },
    { 0x8793a4ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf13a4af, SUC_lexicon_tags_30 },
    { 0xed3a4b0, SUC_lexicon_tags_15 },
    { 0x5293a4b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4bf3a4b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab3a4b7, SUC_lexicon_tags_33 },
    { 0x93d3a4b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd3a4c0, SUC_lexicon_tags_29 },
    { 0x65b3a4c1, SUC_lexicon_tags_12 },
    { 0x5a53a4c2, SUC_lexicon_tags_15 },
    { 0xc6f3a4c2, SUC_lexicon_tags_2 },
    { 0x6ab3a4c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5933a4c7, SUC_lexicon_tags_7 },
    { 0x3893a4c8, SUC_lexicon_tags_12 },
    { 0xa213a4c9, SUC_lexicon_tags_1 },
    { 0x23f3a4c8, SUC_lexicon_tags_15 },
    { 0x36b3a4cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b3a4cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9b3a4d1, SUC_lexicon_tags_14 },
    { 0x4c53a4d2, SUC_lexicon_tags_37 },
    { 0xe7d3a4d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc7b3a4d5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe33a4d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba73a4db, SUC_lexicon_tags_31 },
    { 0xabf3a4dc, SUC_lexicon_tags_12 },
    { 0x9bf3a4dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd353a4e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5f3a4e5, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x1293a4e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a73a4e9, SUC_lexicon_tags_47 },
    { 0xc173a4e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b3a4ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa733a4f2, SUC_lexicon_tags_28 },
    { 0x313a4f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d3a4f9, SUC_lexicon_tags_38 },
    { 0xb553a4f9, SUC_lexicon_tags_11 },
    { 0xb493a4fb, SUC_lexicon_tags_23 },
    { 0xe033a4fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d3a503, SUC_lexicon_tags_7 },
    { 0x9833a504, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f3a507, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c13a50b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3993a50e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f3a511, SUC_lexicon_tags_17 },
    { 0x3793a512, SUC_lexicon_tags_38 },
    { 0x2293a513, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793a51a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4473a524, SUC_lexicon_tags_1 },
    { 0x7e53a525, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2293a527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe673a529, SUC_lexicon_tags_60 },
    { 0x62b3a529, SUC_lexicon_tags_14 },
    { 0x72f3a52b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b3a530, SUC_lexicon_tags_25 },
    { 0x8453a531, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8133a534, SUC_lexicon_tags_15 },
    { 0x21b3a534, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2633a539, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa693a53f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab73a543, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf573a545, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe733a556, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c93a55a, SUC_lexicon_tags_39 },
    { 0xc2b3a55b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc93a561, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x67d3a563, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f3a56b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb73a56d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b3a575, SUC_lexicon_tags_2 },
    { 0x54d3a576, SUC_lexicon_tags_15 },
    { 0x4673a577, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa453a57a, SUC_lexicon_tags_33 },
    { 0x6393a57b, SUC_lexicon_tags_26 },
    { 0xc8f3a57a, SUC_lexicon_tags_11 },
    { 0x8873a57c, SUC_lexicon_tags_37 },
    { 0x4f73a57c, SUC_lexicon_tags_103 },
    { 0xfdf3a57d, SUC_lexicon_tags_15 },
    { 0x1bb3a57f, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0x7073a582, SUC_lexicon_tags_7 },
    { 0x5e13a582, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb733a585, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8793a58e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3193a595, SUC_lexicon_tags_193 },
    { 0x22b3a595, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5253a599, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf213a59d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb093a5a5, SUC_lexicon_tags_17 },
    { 0x7453a5a6, SUC_lexicon_tags_31 },
    { 0x6c33a5a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2453a5aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5253a5ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53a5af, SUC_lexicon_tags_79 },
    { 0xe953a5af, SUC_lexicon_tags_25 },
    { 0x2973a5b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1233a5ba, SUC_lexicon_tags_15 },
    { 0xe613a5bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7613a5bf, SUC_lexicon_tags_32 },
    { 0xe013a5bf, SUC_lexicon_tags_11 },
    { 0xe713a5c1, SUC_lexicon_tags_66 },
    { 0xce33a5c0, SUC_lexicon_tags_27 },
    { 0xd513a5bf, SUC_lexicon_tags_47 },
    { 0x4f93a5c4, SUC_lexicon_tags_11 },
    { 0x3a13a5c5, SUC_lexicon_tags_65 },
    { 0x7f53a5c6, SUC_lexicon_tags_7 },
    { 0xd913a5c1, SUC_lexicon_tags_49 },
    { 0x8593a5c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7f3a5ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf553a5cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b53a5cf, SUC_lexicon_tags_235 },
    { 0x4473a5d0, SUC_lexicon_tags_31 },
    { 0xc9f3a5d1, SUC_lexicon_tags_14 },
    { 0xb333a5d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed3a5d5, SUC_lexicon_tags_6 },
    { 0xc8f3a5d5, SUC_lexicon_tags_15 },
    { 0xd7f3a5d7, SUC_lexicon_tags_47 },
    { 0xca33a5d8, SUC_lexicon_tags_1 },
    { 0x9fd3a5d9, SUC_lexicon_tags_8 },
    { 0x67d3a5da, SUC_lexicon_tags_7 },
    { 0x5bf3a5db, SUC_lexicon_tags_25 },
    { 0x773a5db, SUC_lexicon_tags_37 },
    { 0xb5b3a5d8, SUC_lexicon_tags_37 },
    { 0x28f3a5de, SUC_lexicon_tags_23 },
    { 0xb2f3a5d5, SUC_lexicon_tags_1 },
    { 0xca93a5e0, SUC_lexicon_tags_46 },
    { 0xce93a5e0, SUC_lexicon_tags_11 },
    { 0xe8b3a5d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe813a5e4, SUC_lexicon_tags_74 },
    { 0x5dd3a5e5, SUC_lexicon_tags_19 },
    { 0xaaf3a5e6, SUC_lexicon_tags_27 },
    { 0xf693a5e7, SUC_lexicon_tags_37 },
    { 0xef53a5e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3473a5ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d3a5f3, SUC_lexicon_tags_11 },
    { 0xc6d3a5f4, SUC_lexicon_tags_12 },
    { 0x7d53a5f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b53a601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28f3a603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf273a607, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d3a614, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa013a618, SUC_lexicon_tags_26 },
    { 0x1813a619, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b3a624, SUC_lexicon_tags_1 },
    { 0xc253a625, SUC_lexicon_tags_53 },
    { 0x5f53a626, SUC_lexicon_tags_8 },
    { 0xf3a626, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a13a629, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb3a62d, SUC_lexicon_tags_30 },
    { 0x7553a62d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb93a630, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae53a635, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b3a638, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b3a63d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d3a645, SUC_lexicon_tags_17 },
    { 0x8333a646, SUC_lexicon_tags_49 },
    { 0x22d3a647, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3653a649, SUC_lexicon_tags_23 },
    { 0x83b3a64a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc793a64f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc653a655, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf733a657, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a65c, SUC_lexicon_tags_11 },
    { 0xa393a65c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb3a669, SUC_lexicon_tags_17 },
    { 0x4f13a66a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea93a66d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a73a672, SUC_lexicon_tags_1 },
    { 0x40f3a672, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xb393a676, SUC_lexicon_tags_25 },
    { 0x7f93a676, SUC_lexicon_tags_7 },
    { 0x6773a677, SUC_lexicon_tags_10 },
    { 0x2073a679, SUC_lexicon_tags_11 },
    { 0xb593a679, SUC_lexicon_tags_255 },
    { 0x1913a67b, SUC_lexicon_tags_11 },
    { 0xfdd3a679, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed3a67f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff3a683, SUC_lexicon_tags_17 },
    { 0x2713a683, SUC_lexicon_tags_3 },
    { 0x4bd3a685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab3a68e, SUC_lexicon_tags_17 },
    { 0x373a68f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bd3a691, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b3a698, SUC_lexicon_tags_30 },
    { 0x69b3a698, SUC_lexicon_tags_25 },
    { 0x3af3a699, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3a69f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef3a6a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8573a6a6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7673a6ac, SUC_lexicon_tags_45 },
    { 0x913a6ad, SUC_lexicon_tags_17 },
    { 0x1253a6ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb173a6b4, SUC_lexicon_tags_27 },
    { 0x27b3a6b5, SUC_lexicon_tags_47 },
    { 0xb73a6b6, SUC_lexicon_tags_7 },
    { 0xba13a6b7, SUC_lexicon_tags_43 },
    { 0xd253a6b7, SUC_lexicon_tags_1 },
    { 0xe1d3a6b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab3a6bc, SUC_lexicon_tags_37 },
    { 0xa0b3a6bd, SUC_lexicon_tags_60 },
    { 0x34b3a6be, SUC_lexicon_tags_60 },
    { 0xae33a6be, SUC_lexicon_tags_10 },
    { 0xc6d3a6bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c33a6c2, SUC_lexicon_tags_47 },
    { 0x2033a6c3, SUC_lexicon_tags_15 },
    { 0xd933a6c4, SUC_lexicon_tags_29 },
    { 0x4e93a6c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9633a6c7, SUC_lexicon_tags_15 },
    { 0xec93a6c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf873a6ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccd3a6cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d33a6d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9d33a6d2, SUC_lexicon_tags_60 },
    { 0x9e73a6d2, SUC_lexicon_tags_23 },
    { 0x7313a6d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d3a6d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b53a6da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed3a6dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf453a6e1, SUC_lexicon_tags_22 },
    { 0x8693a6e2, SUC_lexicon_tags_63 },
    { 0xd353a6e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5dd3a6e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3eb3a6e7, SUC_lexicon_tags_14 },
    { 0x6053a6e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3b3a6ea, SUC_lexicon_tags_14 },
    { 0x6153a6eb, SUC_lexicon_tags_45 },
    { 0x5713a6ec, SUC_lexicon_tags_27 },
    { 0x1d33a6ed, SUC_lexicon_tags_1 },
    { 0x2173a6ee, SUC_lexicon_tags_17 },
    { 0xeb53a6ef, SUC_lexicon_tags_15 },
    { 0x1ef3a6ef, SUC_lexicon_tags_17 },
    { 0x22d3a6f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3a6f4, SUC_lexicon_tags_33 },
    { 0x68b3a6f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb3a6fb, SUC_lexicon_tags_23 },
    { 0x8193a6fc, SUC_lexicon_tags_19 },
    { 0xb573a6fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa013a701, SUC_lexicon_tags_27 },
    { 0xeed3a701, SUC_lexicon_tags_38 },
    { 0x8bb3a703, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d93a707, SUC_lexicon_tags_37 },
    { 0x6813a708, SUC_lexicon_tags_16 },
    { 0x3973a709, SUC_lexicon_tags_60 },
    { 0x45b3a70a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93a70d, SUC_lexicon_tags_12 },
    { 0x7e53a70e, SUC_lexicon_tags_43 },
    { 0x19f3a70d, SUC_lexicon_tags_37 },
    { 0x6f33a70f, SUC_lexicon_tags_11 },
    { 0xf0d3a711, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2993a713, SUC_lexicon_tags_11 },
    { 0xd113a714, SUC_lexicon_tags_65 },
    { 0x5373a715, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6413a717, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b3a720, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda73a726, SUC_lexicon_tags_26 },
    { 0xe233a727, SUC_lexicon_tags_61 },
    { 0x2f53a727, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb13a72c, SUC_lexicon_tags_21 },
    { 0x2193a72c, SUC_lexicon_tags_7 },
    { 0x87b3a72e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x293a731, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc453a736, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b3a73a, SUC_lexicon_tags_36 },
    { 0x2653a73b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc493a73d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb53a744, SUC_lexicon_tags_9 },
    { 0x8bd3a745, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3a74e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd3a759, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5093a75d, SUC_lexicon_tags_7 },
    { 0x6d93a75e, SUC_lexicon_tags_25 },
    { 0x7813a75f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf653a761, SUC_lexicon_tags_60 },
    { 0x53d3a762, SUC_lexicon_tags_17 },
    { 0x5b3a761, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb3a767, SUC_lexicon_tags_49 },
    { 0x1473a768, SUC_lexicon_tags_9 },
    { 0xc7f3a769, SUC_lexicon_tags_25 },
    { 0x6633a76a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b13a76f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d13a771, SUC_lexicon_tags_29 },
    { 0x6133a772, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xab13a774, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3073a776, SUC_lexicon_tags_17 },
    { 0x5513a776, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c13a77c, SUC_lexicon_tags_2 },
    { 0x1433a77d, SUC_lexicon_tags_25 },
    { 0xb9d3a77e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe53a784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3713a786, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab3a78a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d3a790, SUC_lexicon_tags_11 },
    { 0x26d3a791, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533a797, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6233a79a, SUC_lexicon_tags_11 },
    { 0x3593a79b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa993a79e, SUC_lexicon_tags_12 },
    { 0x61f3a79f, SUC_lexicon_tags_17 },
    { 0xefd3a7a0, SUC_lexicon_tags_2 },
    { 0xd173a7a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2733a7a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a53a7a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af3a7ac, SUC_lexicon_tags_15 },
    { 0xe6d3a7ad, SUC_lexicon_tags_64 },
    { 0x7553a7ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb93a7b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b3a7b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f13a7bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a7c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f93a7c7, SUC_lexicon_tags_1 },
    { 0xb233a7c8, SUC_lexicon_tags_27 },
    { 0xa713a7c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50f3a7cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b3a7ce, SUC_lexicon_tags_25 },
    { 0x4ad3a7ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a7d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf73a7da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff3a7de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3773a7e2, SUC_lexicon_tags_79 },
    { 0x5233a7e2, SUC_lexicon_tags_45 },
    { 0xe753a7e4, SUC_lexicon_tags_14 },
    { 0x53d3a7e5, SUC_lexicon_tags_82 },
    { 0x2ab3a7e6, SUC_lexicon_tags_7 },
    { 0x4c93a7e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa573a7ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6653a7fb, SUC_lexicon_tags_35 },
    { 0x2f3a7fc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe293a7fe, SUC_lexicon_tags_2 },
    { 0x1ff3a7ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd73a805, SUC_lexicon_tags_158 },
    { 0x1973a806, SUC_lexicon_tags_49 },
    { 0xa673a807, SUC_lexicon_tags_6 },
    { 0x3f33a808, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4053a810, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb13a815, SUC_lexicon_tags_27 },
    { 0x2273a815, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad3a819, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d73a81c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1913a820, SUC_lexicon_tags_12 },
    { 0xd793a821, SUC_lexicon_tags_17 },
    { 0x6773a820, SUC_lexicon_tags_1 },
    { 0xf173a823, SUC_lexicon_tags_37 },
    { 0x7c93a821, SUC_lexicon_tags_279 },
    { 0xbdb3a825, SUC_lexicon_tags_26 },
    { 0xf073a821, SUC_lexicon_tags_25 },
    { 0xa873a827, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32d3a829, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f3a82c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93f3a82e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1853a832, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3a836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8073a83a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc393a83d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa413a840, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f3a844, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf3a84b, SUC_lexicon_tags_11 },
    { 0xa173a84c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3a850, SUC_lexicon_tags_2 },
    { 0x2253a851, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc8f3a853, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3773a855, SUC_lexicon_tags_37 },
    { 0x4bf3a856, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72d3a858, SUC_lexicon_tags_10 },
    { 0x8df3a859, SUC_lexicon_tags_12 },
    { 0xca93a859, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9913a85e, SUC_lexicon_tags_16 },
    { 0xbbb3a85e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db3a862, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2b13a864, SUC_lexicon_tags_23 },
    { 0xa013a865, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9453a868, SUC_lexicon_tags_15 },
    { 0x7f13a869, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd3a86e, SUC_lexicon_tags_61 },
    { 0xe4f3a86e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5913a872, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b3a875, SUC_lexicon_tags_17 },
    { 0x7453a876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa933a878, SUC_lexicon_tags_15 },
    { 0x65b3a879, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6553a87f, SUC_lexicon_tags_48 },
    { 0x4813a880, SUC_lexicon_tags_38 },
    { 0x6cd3a881, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe893a884, SUC_lexicon_tags_7 },
    { 0xac13a885, SUC_lexicon_tags_28 },
    { 0x8db3a886, SUC_lexicon_tags_31 },
    { 0xecf3a887, SUC_lexicon_tags_46 },
    { 0x6e53a888, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3a88b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f3a88e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5913a890, SUC_lexicon_tags_7 },
    { 0xb5f3a891, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef3a897, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75b3a899, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc793a89c, SUC_lexicon_tags_1 },
    { 0x6773a89d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd3a8a1, SUC_lexicon_tags_8 },
    { 0x9193a8a2, SUC_lexicon_tags_37 },
    { 0xe3b3a8a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe893a8a6, SUC_lexicon_tags_14 },
    { 0xd033a8a6, SUC_lexicon_tags_25 },
    { 0xbf3a8a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb633a8aa, SUC_lexicon_tags_45 },
    { 0x39b3a8ab, SUC_lexicon_tags_12 },
    { 0xd6b3a8ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3113a8ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b3a8b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d3a8b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29f3a8b5, SUC_lexicon_tags_1 },
    { 0x5ad3a8b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed13a8ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b3a8be, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b3a8c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8cf3a8c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88d3a8ca, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d73a8cd, SUC_lexicon_tags_30 },
    { 0x1d3a8cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c53a8d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff3a8d6, SUC_lexicon_tags_16 },
    { 0x2733a8d7, SUC_lexicon_tags_2 },
    { 0xda93a8d8, SUC_lexicon_tags_39 },
    { 0xd6b3a8d8, SUC_lexicon_tags_38 },
    { 0x3fb3a8da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d3a8dd, SUC_lexicon_tags_37 },
    { 0xf453a8de, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x1373a8e0, SUC_lexicon_tags_37 },
    { 0x38f3a8e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef53a8e5, SUC_lexicon_tags_29 },
    { 0xa233a8e6, SUC_lexicon_tags_3 },
    { 0x7533a8e7, SUC_lexicon_tags_17 },
    { 0xd4d3a8e8, SUC_lexicon_tags_14 },
    { 0xd4b3a8e9, SUC_lexicon_tags_37 },
    { 0x2bb3a8ea, SUC_lexicon_tags_26 },
    { 0x50f3a8eb, SUC_lexicon_tags_48 },
    { 0x5473a8eb, SUC_lexicon_tags_27 },
    { 0x6f53a8ed, SUC_lexicon_tags_37 },
    { 0x1913a8ee, SUC_lexicon_tags_18 },
    { 0x6213a8ed, SUC_lexicon_tags_60 },
    { 0xd713a8eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1053a8f3, SUC_lexicon_tags_79 },
    { 0x3793a8f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf3a8fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x133a8fc, SUC_lexicon_tags_60 },
    { 0xff13a8fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b3a901, SUC_lexicon_tags_28 },
    { 0x7033a902, SUC_lexicon_tags_25 },
    { 0x85b3a903, SUC_lexicon_tags_28 },
    { 0xeaf3a904, SUC_lexicon_tags_23 },
    { 0xa933a903, SUC_lexicon_tags_14 },
    { 0x2313a906, SUC_lexicon_tags_25 },
    { 0x4d53a907, SUC_lexicon_tags_27 },
    { 0xc833a906, SUC_lexicon_tags_33 },
    { 0x8c33a907, SUC_lexicon_tags_47 },
    { 0xf613a906, SUC_lexicon_tags_11 },
    { 0x5813a90b, SUC_lexicon_tags_14 },
    { 0x4393a908, SUC_lexicon_tags_31 },
    { 0x56f3a908, SUC_lexicon_tags_45 },
    { 0x1c73a909, SUC_lexicon_tags_37 },
    { 0x5f3a90e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed3a912, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f33a915, SUC_lexicon_tags_8 },
    { 0x9d3a916, SUC_lexicon_tags_38 },
    { 0x8d33a917, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa473a91b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab33a91f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6193a921, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d3a924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7353a928, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8633a92d, SUC_lexicon_tags_26 },
    { 0x9e53a92d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a33a931, SUC_lexicon_tags_30 },
    { 0x32f3a932, SUC_lexicon_tags_1 },
    { 0x60d3a932, SUC_lexicon_tags_14 },
    { 0x4d93a934, SUC_lexicon_tags_12 },
    { 0x5533a934, SUC_lexicon_tags_0 },
    { 0x3c13a934, SUC_lexicon_tags_37 },
    { 0x3d33a936, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b53a93e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d93a944, SUC_lexicon_tags_25 },
    { 0xfaf3a945, SUC_lexicon_tags_17 },
    { 0x9313a946, SUC_lexicon_tags_25 },
    { 0x11f3a945, SUC_lexicon_tags_47 },
    { 0xa3f3a948, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3f3a94a, SUC_lexicon_tags_15 },
    { 0xd5d3a94a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xfaf3a94d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad3a950, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b3a955, SUC_lexicon_tags_33 },
    { 0x8753a956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b3a95e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf93a96f, SUC_lexicon_tags_19 },
    { 0x27b3a970, SUC_lexicon_tags_15 },
    { 0xbd33a970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce93a976, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf193a978, SUC_lexicon_tags_15 },
    { 0x84f3a978, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed3a97f, SUC_lexicon_tags_24 },
    { 0x4f53a980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9f3a982, SUC_lexicon_tags_31 },
    { 0xd3a982, SUC_lexicon_tags_10 },
    { 0xf853a984, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3193a986, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa173a98b, SUC_lexicon_tags_17 },
    { 0xa273a98c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f3a98f, SUC_lexicon_tags_2 },
    { 0x8fd3a98f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x173a993, SUC_lexicon_tags_11 },
    { 0xc173a994, SUC_lexicon_tags_50 },
    { 0x30b3a993, SUC_lexicon_tags_11 },
    { 0x1733a996, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa293a999, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd4b3a99b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3a99f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f33a9a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3593a9a9, SUC_lexicon_tags_12 },
    { 0xefb3a9a9, SUC_lexicon_tags_8 },
    { 0x4233a9ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa13a9ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa253a9b2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x113a9b5, SUC_lexicon_tags_45 },
    { 0x9b13a9b6, SUC_lexicon_tags_13 },
    { 0xbcd3a9b7, SUC_lexicon_tags_46 },
    { 0xb1f3a9b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1353a9ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f3a9bd, SUC_lexicon_tags_194 },
    { 0xf453a9bd, SUC_lexicon_tags_1 },
    { 0x7b13a9bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x333a9c2, SUC_lexicon_tags_27 },
    { 0x7ef3a9c3, SUC_lexicon_tags_1 },
    { 0xdcf3a9c4, SUC_lexicon_tags_11 },
    { 0xfa93a9c2, SUC_lexicon_tags_17 },
    { 0x5b93a9c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5753a9cb, SUC_lexicon_tags_45 },
    { 0x3e13a9cb, SUC_lexicon_tags_0 },
    { 0x6d93a9cb, SUC_lexicon_tags_25 },
    { 0x95d3a9ce, SUC_lexicon_tags_37 },
    { 0xccb3a9cf, SUC_lexicon_tags_121 },
    { 0x8293a9cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b13a9d3, SUC_lexicon_tags_31 },
    { 0xc093a9d4, SUC_lexicon_tags_17 },
    { 0xe473a9d3, SUC_lexicon_tags_1 },
    { 0xd353a9d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaa93a9d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc733a9da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad3a9de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc673a9e4, SUC_lexicon_tags_31 },
    { 0x8333a9e5, SUC_lexicon_tags_17 },
    { 0x7cf3a9e6, SUC_lexicon_tags_7 },
    { 0xea93a9e5, SUC_lexicon_tags_26 },
    { 0x4253a9e8, SUC_lexicon_tags_47 },
    { 0x9153a9e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x34f3a9eb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd813a9ed, SUC_lexicon_tags_15 },
    { 0x3b73a9ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b3a9f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x89b3a9f3, SUC_lexicon_tags_10 },
    { 0xfd13a9f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa993a9f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4b3a9f8, SUC_lexicon_tags_7 },
    { 0x8493a9f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5613a9fe, SUC_lexicon_tags_48 },
    { 0xe793a9ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf3aa03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf973aa07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f3aa0c, SUC_lexicon_tags_17 },
    { 0x9433aa0d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x7b33aa0f, SUC_lexicon_tags_63 },
    { 0xbdb3aa10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25f3aa12, SUC_lexicon_tags_47 },
    { 0xc0d3aa13, SUC_lexicon_tags_11 },
    { 0x5ab3aa14, SUC_lexicon_tags_37 },
    { 0x52d3aa15, SUC_lexicon_tags_15 },
    { 0xd153aa16, SUC_lexicon_tags_1017 },
    { 0x5b3aa16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7453aa19, SUC_lexicon_tags_53 },
    { 0x4333aa1a, SUC_lexicon_tags_25 },
    { 0x98d3aa1b, SUC_lexicon_tags_45 },
    { 0xb8b3aa1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1373aa1f, SUC_lexicon_tags_4 },
    { 0xcf33aa20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd73aa23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2953aa26, SUC_lexicon_tags_43 },
    { 0xddf3aa26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f93aa2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf993aa33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa493aa3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cb3aa3e, SUC_lexicon_tags_1 },
    { 0x5f93aa3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1113aa42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6853aa44, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8ff3aa46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc693aa48, SUC_lexicon_tags_9 },
    { 0x1fb3aa49, SUC_lexicon_tags_7 },
    { 0x54f3aa4a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc113aa4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98b3aa4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5473aa51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc153aa59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2573aa63, SUC_lexicon_tags_1 },
    { 0xaad3aa64, SUC_lexicon_tags_14 },
    { 0xbcb3aa65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d3aa68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4933aa6d, SUC_lexicon_tags_11 },
    { 0x4f13aa6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe473aa70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b3aa73, SUC_lexicon_tags_6 },
    { 0x5cd3aa73, SUC_lexicon_tags_51 },
    { 0xf993aa75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4273aa7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6813aa81, SUC_lexicon_tags_16 },
    { 0x24f3aa82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cd3aa84, SUC_lexicon_tags_37 },
    { 0x3fb3aa85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf833aa87, SUC_lexicon_tags_15 },
    { 0x4db3aa87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea53aa8b, SUC_lexicon_tags_0 },
    { 0x80d3aa8b, SUC_lexicon_tags_38 },
    { 0x1173aa8b, SUC_lexicon_tags_8 },
    { 0xd813aa8c, SUC_lexicon_tags_43 },
    { 0x60f3aa8d, SUC_lexicon_tags_6 },
    { 0x3893aa90, SUC_lexicon_tags_45 },
    { 0xf013aa91, SUC_lexicon_tags_13 },
    { 0x1af3aa91, SUC_lexicon_tags_0 },
    { 0xf513aa90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d3aa9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf3aa9d, SUC_lexicon_tags_23 },
    { 0x5733aa9e, SUC_lexicon_tags_49 },
    { 0x5073aa9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf553aaa1, SUC_lexicon_tags_48 },
    { 0xc813aaa1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5753aaa4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f3aaa7, SUC_lexicon_tags_103 },
    { 0xddd3aaa8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1773aaab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7553aaad, SUC_lexicon_tags_12 },
    { 0x36f3aaae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2373aab3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f3aab8, SUC_lexicon_tags_30 },
    { 0xb973aab8, SUC_lexicon_tags_37 },
    { 0xe093aab9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b53aabc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c93aac2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7673aac4, SUC_lexicon_tags_11 },
    { 0x5513aac5, SUC_lexicon_tags_17 },
    { 0xf8b3aac4, SUC_lexicon_tags_37 },
    { 0x8933aac7, SUC_lexicon_tags_1 },
    { 0xd2b3aac7, SUC_lexicon_tags_46 },
    { 0xdeb3aac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6013aacf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe013aad2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4313aad4, SUC_lexicon_tags_37 },
    { 0x3733aad4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc53aada, SUC_lexicon_tags_25 },
    { 0x53aada, SUC_lexicon_tags_11 },
    { 0xcf73aadc, SUC_lexicon_tags_43 },
    { 0xdf3aadb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2953aadf, SUC_lexicon_tags_11 },
    { 0x7b13aadf, SUC_lexicon_tags_11 },
    { 0x4793aadf, SUC_lexicon_tags_13 },
    { 0x1eb3aae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c93aae5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b53aae8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8193aaea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb3aaed, SUC_lexicon_tags_18 },
    { 0x153aaee, SUC_lexicon_tags_37 },
    { 0x1713aaef, SUC_lexicon_tags_0 },
    { 0x7553aaf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9333aaf3, SUC_lexicon_tags_37 },
    { 0x9d73aaf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc93aaf6, SUC_lexicon_tags_0 },
    { 0x6973aaf7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5373aafb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2493ab00, SUC_lexicon_tags_12 },
    { 0xf7d3ab00, SUC_lexicon_tags_7 },
    { 0x7dd3ab02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e13ab05, SUC_lexicon_tags_37 },
    { 0x8c53ab05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff3ab09, SUC_lexicon_tags_55 },
    { 0x5b93ab0a, SUC_lexicon_tags_11 },
    { 0x9cd3ab0a, SUC_lexicon_tags_6 },
    { 0x14d3ab0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d13ab10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd4b3ab12, SUC_lexicon_tags_42 },
    { 0x6473ab12, SUC_lexicon_tags_80 },
    { 0xfe93ab12, SUC_lexicon_tags_0 },
    { 0x7833ab15, SUC_lexicon_tags_12 },
    { 0xc3d3ab16, SUC_lexicon_tags_23 },
    { 0x12d3ab16, SUC_lexicon_tags_1 },
    { 0xcc93ab15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad53ab1e, SUC_lexicon_tags_39 },
    { 0xee13ab1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf3ab24, SUC_lexicon_tags_98 },
    { 0xabf3ab25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb3ab28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d53ab2f, SUC_lexicon_tags_43 },
    { 0x933ab30, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc213ab33, SUC_lexicon_tags_27 },
    { 0x193ab34, SUC_lexicon_tags_11 },
    { 0x6e13ab35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ad3ab37, SUC_lexicon_tags_35 },
    { 0xc8d3ab37, SUC_lexicon_tags_65 },
    { 0x153ab39, SUC_lexicon_tags_1 },
    { 0x33b3ab3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd3ab3e, SUC_lexicon_tags_2 },
    { 0x58b3ab3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa73ab41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2453ab43, SUC_lexicon_tags_45 },
    { 0x15f3ab44, SUC_lexicon_tags_37 },
    { 0x8853ab44, SUC_lexicon_tags_0 },
    { 0xecd3ab46, SUC_lexicon_tags_43 },
    { 0x1653ab45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68f3ab49, SUC_lexicon_tags_25 },
    { 0x3733ab4a, SUC_lexicon_tags_27 },
    { 0xde33ab4b, SUC_lexicon_tags_12 },
    { 0x1bd3ab4c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8693ab4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2613ab52, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13ab59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d3ab5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73ab61, SUC_lexicon_tags_1 },
    { 0xde53ab61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db3ab69, SUC_lexicon_tags_57 },
    { 0xa413ab69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d3ab6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xedb3ab6f, SUC_lexicon_tags_37 },
    { 0x9eb3ab70, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfe33ab72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb3ab77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcd13ab79, SUC_lexicon_tags_55 },
    { 0xdbd3ab79, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb93ab7d, SUC_lexicon_tags_1 },
    { 0x16b3ab7e, SUC_lexicon_tags_2 },
    { 0x55b3ab7f, SUC_lexicon_tags_37 },
    { 0xb973ab7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e33ab8c, SUC_lexicon_tags_49 },
    { 0xa8f3ab8d, SUC_lexicon_tags_120 },
    { 0x4b33ab8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e33ab93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa013ab95, SUC_lexicon_tags_30 },
    { 0x5773ab96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb373ab99, SUC_lexicon_tags_18 },
    { 0x3d33ab9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93ab9f, SUC_lexicon_tags_60 },
    { 0x6fd3aba0, SUC_lexicon_tags_23 },
    { 0xb13aba0, SUC_lexicon_tags_21 },
    { 0xe833aba0, SUC_lexicon_tags_3 },
    { 0xea13aba3, SUC_lexicon_tags_8 },
    { 0x66b3aba4, SUC_lexicon_tags_25 },
    { 0xb3d3aba4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a13abaa, SUC_lexicon_tags_25 },
    { 0xc753abab, SUC_lexicon_tags_49 },
    { 0x78d3abac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe913abae, SUC_lexicon_tags_39 },
    { 0x933abae, SUC_lexicon_tags_87 },
    { 0xa33abb0, SUC_lexicon_tags_12 },
    { 0xbc53abb0, SUC_lexicon_tags_11 },
    { 0x5273abb2, SUC_lexicon_tags_27 },
    { 0xb613abb1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3abb6, SUC_lexicon_tags_15 },
    { 0xff93abb7, SUC_lexicon_tags_11 },
    { 0xd913abb8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3493abbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab33abc2, SUC_lexicon_tags_9 },
    { 0x1e33abc2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb93abca, SUC_lexicon_tags_11 },
    { 0x58b3abcb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d3abce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f73abd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ab3abd2, SUC_lexicon_tags_12 },
    { 0x693abd3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4173abd8, SUC_lexicon_tags_7 },
    { 0x3c13abd8, SUC_lexicon_tags_10 },
    { 0xff53abd8, SUC_lexicon_tags_37 },
    { 0x5df3abd8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4813abdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b3abe0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef3abec, SUC_lexicon_tags_9 },
    { 0x3a73abec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa513abf0, SUC_lexicon_tags_25 },
    { 0x773abf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f3abf5, SUC_lexicon_tags_28 },
    { 0x6833abf6, SUC_lexicon_tags_65 },
    { 0x70f3abf7, SUC_lexicon_tags_1 },
    { 0x7f33abf8, SUC_lexicon_tags_15 },
    { 0x7f93abf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda33abfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6473abfe, SUC_lexicon_tags_28 },
    { 0xd913abff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5993ac08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d3ac0b, SUC_lexicon_tags_29 },
    { 0x12d3ac0b, SUC_lexicon_tags_34 },
    { 0xb193ac0d, SUC_lexicon_tags_11 },
    { 0x5b13ac0e, SUC_lexicon_tags_19 },
    { 0x46d3ac0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6593ac13, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe093ac15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b3ac18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcff3ac1a, SUC_lexicon_tags_27 },
    { 0xb9f3ac1b, SUC_lexicon_tags_15 },
    { 0xa8b3ac1b, SUC_lexicon_tags_25 },
    { 0xf1d3ac1d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc93ac20, SUC_lexicon_tags_6 },
    { 0xc3f3ac21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5893ac24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f3ac27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff3ac2a, SUC_lexicon_tags_17 },
    { 0x4093ac2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8333ac2e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73ac31, SUC_lexicon_tags_84 },
    { 0x3093ac32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad13ac37, SUC_lexicon_tags_7 },
    { 0x50b3ac37, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7833ac3b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d3ac3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1af3ac41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc833ac43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9833ac4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc93ac4e, SUC_lexicon_tags_25 },
    { 0x7e73ac4f, SUC_lexicon_tags_47 },
    { 0x78d3ac4e, SUC_lexicon_tags_15 },
    { 0xb173ac51, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9233ac55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd93ac57, SUC_lexicon_tags_55 },
    { 0x1d93ac58, SUC_lexicon_tags_45 },
    { 0xb3ac59, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8853ac5d, SUC_lexicon_tags_31 },
    { 0xa113ac5e, SUC_lexicon_tags_11 },
    { 0xa893ac5f, SUC_lexicon_tags_47 },
    { 0x95d3ac60, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa673ac62, SUC_lexicon_tags_17 },
    { 0x20b3ac62, SUC_lexicon_tags_0 },
    { 0x4d33ac63, SUC_lexicon_tags_37 },
    { 0x85b3ac63, SUC_lexicon_tags_15 },
    { 0x4813ac66, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2813ac6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe773ac6d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa933ac71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xecb3ac73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2553ac78, SUC_lexicon_tags_38 },
    { 0x6d93ac78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b3ac7e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6033ac80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa313ac83, SUC_lexicon_tags_302 },
    { 0x7933ac84, SUC_lexicon_tags_18 },
    { 0x2633ac83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf33ac87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x35f3ac89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe813ac8b, SUC_lexicon_tags_21 },
    { 0x833ac8c, SUC_lexicon_tags_17 },
    { 0x7ab3ac8d, SUC_lexicon_tags_26 },
    { 0xb1d3ac8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5d53ac90, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5653ac93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933ac97, SUC_lexicon_tags_11 },
    { 0xbc73ac97, SUC_lexicon_tags_46 },
    { 0x3553ac99, SUC_lexicon_tags_29 },
    { 0x2df3ac99, SUC_lexicon_tags_12 },
    { 0x3533ac9b, SUC_lexicon_tags_2 },
    { 0x9333ac9b, SUC_lexicon_tags_17 },
    { 0x77b3ac9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd3aca0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13d3aca2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3413aca5, SUC_lexicon_tags_11 },
    { 0x7553aca6, SUC_lexicon_tags_11 },
    { 0x94d3aca6, SUC_lexicon_tags_12 },
    { 0xc793aca8, SUC_lexicon_tags_11 },
    { 0xccf3aca6, SUC_lexicon_tags_43 },
    { 0x4813acaa, SUC_lexicon_tags_2 },
    { 0xe633acaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3acaf, SUC_lexicon_tags_63 },
    { 0xcaf3acaf, SUC_lexicon_tags_4 },
    { 0xd573acb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94f3acb3, SUC_lexicon_tags_1 },
    { 0x39d3acb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2d3acb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd213acbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3993acbe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b53acc3, SUC_lexicon_tags_2 },
    { 0xf693acc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5533acc6, SUC_lexicon_tags_1382 },
    { 0, NULL },
    { 0, NULL },
    { 0x593acc9, SUC_lexicon_tags_33 },
    { 0x4593acca, SUC_lexicon_tags_11 },
    { 0x90d3accb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7d33accd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7f3accf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef13acd4, SUC_lexicon_tags_70 },
    { 0xc5f3acd5, SUC_lexicon_tags_84 },
    { 0xcc73acd6, SUC_lexicon_tags_7 },
    { 0x3d13acd6, SUC_lexicon_tags_47 },
    { 0x37b3acd8, SUC_lexicon_tags_25 },
    { 0x7b33acd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa513acdc, SUC_lexicon_tags_2 },
    { 0x333acdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e33ace9, SUC_lexicon_tags_19 },
    { 0x9653acea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe513acec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d3acf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b3acf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a33acfb, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x20f3acfd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb3ad04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c73ad09, SUC_lexicon_tags_188 },
    { 0xa293ad0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7293ad0c, SUC_lexicon_tags_17 },
    { 0x9ff3ad0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb3ad10, SUC_lexicon_tags_17 },
    { 0x9193ad10, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ad17, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3a93ad19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5d3ad1b, SUC_lexicon_tags_66 },
    { 0x7033ad1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d3ad22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xefb3ad24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf3ad28, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1213ad2a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173ad30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc853ad32, SUC_lexicon_tags_38 },
    { 0x7db3ad32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c73ad36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f3ad3b, SUC_lexicon_tags_7 },
    { 0x1633ad3c, SUC_lexicon_tags_14 },
    { 0xc7b3ad3d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80b3ad3f, SUC_lexicon_tags_11 },
    { 0xdf3ad40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7633ad48, SUC_lexicon_tags_12 },
    { 0xe2d3ad49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d3ad50, SUC_lexicon_tags_8 },
    { 0x9f3ad50, SUC_lexicon_tags_38 },
    { 0xcc73ad52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed3ad5a, SUC_lexicon_tags_10 },
    { 0x1973ad5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f3ad61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba33ad63, SUC_lexicon_tags_65 },
    { 0x48b3ad63, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f73ad67, SUC_lexicon_tags_46 },
    { 0x2033ad68, SUC_lexicon_tags_21 },
    { 0xee53ad69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d3ad6d, SUC_lexicon_tags_238 },
    { 0x2af3ad6d, SUC_lexicon_tags_15 },
    { 0x5133ad6f, SUC_lexicon_tags_37 },
    { 0x2313ad70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4013ad72, SUC_lexicon_tags_14 },
    { 0x193ad72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcfb3ad75, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff3ad78, SUC_lexicon_tags_28 },
    { 0x8653ad79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fb3ad7b, SUC_lexicon_tags_45 },
    { 0x9453ad7c, SUC_lexicon_tags_19 },
    { 0x5953ad7d, SUC_lexicon_tags_15 },
    { 0x2693ad7e, SUC_lexicon_tags_7 },
    { 0xb9b3ad7d, SUC_lexicon_tags_45 },
    { 0xd1f3ad80, SUC_lexicon_tags_97 },
    { 0x90d3ad81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbef3ad83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca73ad85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xead3ad88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7393ad8f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7353ad91, SUC_lexicon_tags_63 },
    { 0x5173ad91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4033ad97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1753ad9a, SUC_lexicon_tags_15 },
    { 0x29d3ad9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e33ada1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa753ada3, SUC_lexicon_tags_49 },
    { 0x48b3ada4, SUC_lexicon_tags_25 },
    { 0x4273ada3, SUC_lexicon_tags_25 },
    { 0x6973ada6, SUC_lexicon_tags_17 },
    { 0xf0f3ada3, SUC_lexicon_tags_48 },
    { 0xb433ada8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f3adab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab3adae, SUC_lexicon_tags_26 },
    { 0xfa13adaf, SUC_lexicon_tags_2 },
    { 0x2093adae, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4cd3adb2, SUC_lexicon_tags_31 },
    { 0x8f73adb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f3adb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x36d3adb8, SUC_lexicon_tags_31 },
    { 0x31d3adb8, SUC_lexicon_tags_17 },
    { 0xbdf3adba, SUC_lexicon_tags_53 },
    { 0x2e73adba, SUC_lexicon_tags_17 },
    { 0x6d3adbc, SUC_lexicon_tags_10 },
    { 0xf8d3adbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53adc8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1613adca, SUC_lexicon_tags_15 },
    { 0x6393adcb, SUC_lexicon_tags_24 },
    { 0x4a73adca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d3adcf, SUC_lexicon_tags_56 },
    { 0x7ff3add0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd473add7, SUC_lexicon_tags_0 },
    { 0x2593add8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb3addc, SUC_lexicon_tags_230 },
    { 0x5613addd, SUC_lexicon_tags_63 },
    { 0xc4b3adde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7573ade0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf973ade4, SUC_lexicon_tags_15 },
    { 0x2093ade5, SUC_lexicon_tags_9 },
    { 0xbeb3ade6, SUC_lexicon_tags_37 },
    { 0x93b3ade4, SUC_lexicon_tags_27 },
    { 0x9d33ade8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1413adea, SUC_lexicon_tags_2 },
    { 0x1533adea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7973adef, SUC_lexicon_tags_8 },
    { 0xd0d3adef, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x76b3adf2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xde33adf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee93adf6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x3093adf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d73adfc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a93adff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb093ae01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6233ae03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba73ae06, SUC_lexicon_tags_69 },
    { 0x7a13ae07, SUC_lexicon_tags_17 },
    { 0xf813ae08, SUC_lexicon_tags_37 },
    { 0x8693ae09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1853ae0e, SUC_lexicon_tags_7 },
    { 0x62f3ae0e, SUC_lexicon_tags_30 },
    { 0x5953ae10, SUC_lexicon_tags_15 },
    { 0x7673ae10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb733ae13, SUC_lexicon_tags_26 },
    { 0x2cf3ae14, SUC_lexicon_tags_1 },
    { 0x933ae14, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeff3ae17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1b3ae19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f3ae1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5313ae21, SUC_lexicon_tags_27 },
    { 0x2933ae22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31f3ae24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e33ae28, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3773ae34, SUC_lexicon_tags_10 },
    { 0x41f3ae35, SUC_lexicon_tags_25 },
    { 0x3913ae35, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b3ae39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8e33ae3b, SUC_lexicon_tags_33 },
    { 0x2353ae3c, SUC_lexicon_tags_7 },
    { 0xcdf3ae3b, SUC_lexicon_tags_38 },
    { 0x8b33ae3c, SUC_lexicon_tags_193 },
    { 0xdc73ae3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf3ae42, SUC_lexicon_tags_30 },
    { 0xf1b3ae43, SUC_lexicon_tags_1477 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2013ae47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e13ae49, SUC_lexicon_tags_6 },
    { 0xbcb3ae4a, SUC_lexicon_tags_32 },
    { 0x1633ae4b, SUC_lexicon_tags_48 },
    { 0x5fd3ae4c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc33ae52, SUC_lexicon_tags_1 },
    { 0xedb3ae53, SUC_lexicon_tags_17 },
    { 0x48b3ae52, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2293ae5c, SUC_lexicon_tags_23 },
    { 0x57b3ae5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3873ae62, SUC_lexicon_tags_1 },
    { 0x5453ae63, SUC_lexicon_tags_7 },
    { 0x5313ae64, SUC_lexicon_tags_33 },
    { 0xe313ae63, SUC_lexicon_tags_37 },
    { 0x6413ae66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1273ae68, SUC_lexicon_tags_7 },
    { 0xe853ae69, SUC_lexicon_tags_37 },
    { 0xe073ae6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc753ae74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd113ae79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9d3ae7b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e33ae7e, SUC_lexicon_tags_16 },
    { 0x9533ae7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd1b3ae81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f13ae85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff73ae89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72d3ae8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc9d3ae8d, SUC_lexicon_tags_8 },
    { 0x2173ae8e, SUC_lexicon_tags_27 },
    { 0x2b3ae8e, SUC_lexicon_tags_77 },
    { 0xc613ae90, SUC_lexicon_tags_0 },
    { 0x5013ae91, SUC_lexicon_tags_15 },
    { 0x4a33ae8f, SUC_lexicon_tags_82 },
    { 0x8953ae8d, SUC_lexicon_tags_29 },
    { 0x1eb3ae91, SUC_lexicon_tags_63 },
    { 0x4c33ae8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4573ae98, SUC_lexicon_tags_17 },
    { 0x393ae99, SUC_lexicon_tags_28 },
    { 0xb933ae9a, SUC_lexicon_tags_17 },
    { 0x5a33ae99, SUC_lexicon_tags_6 },
    { 0x7253ae9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e13ae9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bd3aea0, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xb5d3aea2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe673aea8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5973aeac, SUC_lexicon_tags_14 },
    { 0x5f13aead, SUC_lexicon_tags_25 },
    { 0x9c13aead, SUC_lexicon_tags_37 },
    { 0x2d33aead, SUC_lexicon_tags_25 },
    { 0x14b3aeb0, SUC_lexicon_tags_49 },
    { 0x8bd3aeb1, SUC_lexicon_tags_17 },
    { 0xb5b3aeae, SUC_lexicon_tags_33 },
    { 0xebd3aeae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcef3aeb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1f3aeb7, SUC_lexicon_tags_17 },
    { 0xe4d3aeb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b3aebe, SUC_lexicon_tags_61 },
    { 0xb8d3aebf, SUC_lexicon_tags_7 },
    { 0x4fd3aec0, SUC_lexicon_tags_12 },
    { 0x4993aec1, SUC_lexicon_tags_48 },
    { 0xa2b3aec0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb9d3aec4, SUC_lexicon_tags_1 },
    { 0x59f3aec5, SUC_lexicon_tags_24 },
    { 0x3893aec6, SUC_lexicon_tags_2 },
    { 0x80f3aec7, SUC_lexicon_tags_8 },
    { 0xfa33aec7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc653aecc, SUC_lexicon_tags_7 },
    { 0x273aecc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3d3aecf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe933aed1, SUC_lexicon_tags_11 },
    { 0xabf3aed1, SUC_lexicon_tags_47 },
    { 0xdab3aed3, SUC_lexicon_tags_12 },
    { 0xd4d3aed4, SUC_lexicon_tags_6 },
    { 0x1553aed5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7433aed8, SUC_lexicon_tags_2 },
    { 0x6f53aed9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x12b3aedb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f13aedf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc73aee6, SUC_lexicon_tags_49 },
    { 0xdcd3aee6, SUC_lexicon_tags_28 },
    { 0x2ad3aee7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9233aeea, SUC_lexicon_tags_12 },
    { 0x94f3aeeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30d3aeed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f3aef1, SUC_lexicon_tags_23 },
    { 0xa593aef2, SUC_lexicon_tags_17 },
    { 0x7fb3aef3, SUC_lexicon_tags_12 },
    { 0x9b53aef4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa53aef6, SUC_lexicon_tags_19 },
    { 0xe7b3aef7, SUC_lexicon_tags_32 },
    { 0xfff3aef8, SUC_lexicon_tags_15 },
    { 0x5ad3aef7, SUC_lexicon_tags_17 },
    { 0x9533aefa, SUC_lexicon_tags_37 },
    { 0xf9f3aef6, SUC_lexicon_tags_4 },
    { 0xe873aefc, SUC_lexicon_tags_4 },
    { 0x8593aefd, SUC_lexicon_tags_8 },
    { 0x69b3aefc, SUC_lexicon_tags_65 },
    { 0x9413aeff, SUC_lexicon_tags_47 },
    { 0x3a93aeff, SUC_lexicon_tags_2 },
    { 0xcbd3af01, SUC_lexicon_tags_25 },
    { 0xca13af01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b93af05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc873af0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3933af10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd833af12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9db3af14, SUC_lexicon_tags_47 },
    { 0x4513af15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd3af1b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef73af20, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b3af23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc73af25, SUC_lexicon_tags_8 },
    { 0x3d73af25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc833af29, SUC_lexicon_tags_49 },
    { 0x8553af2a, SUC_lexicon_tags_25 },
    { 0x8353af2b, SUC_lexicon_tags_11 },
    { 0x32b3af2b, SUC_lexicon_tags_15 },
    { 0xe3d3af2d, SUC_lexicon_tags_7 },
    { 0x18b3af2b, SUC_lexicon_tags_36 },
    { 0xf613af29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe273af34, SUC_lexicon_tags_29 },
    { 0xf2f3af35, SUC_lexicon_tags_3 },
    { 0x9c33af36, SUC_lexicon_tags_17 },
    { 0x8cd3af37, SUC_lexicon_tags_37 },
    { 0x6493af34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3e33af3a, SUC_lexicon_tags_7 },
    { 0xcb53af3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1253af3d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xed33af3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d3af42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa413af4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d3af50, SUC_lexicon_tags_6 },
    { 0x1fd3af51, SUC_lexicon_tags_65 },
    { 0x9eb3af52, SUC_lexicon_tags_0 },
    { 0x4f13af52, SUC_lexicon_tags_45 },
    { 0x73b3af54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c73af57, SUC_lexicon_tags_0 },
    { 0x2993af58, SUC_lexicon_tags_17 },
    { 0x71b3af59, SUC_lexicon_tags_7 },
    { 0x76b3af59, SUC_lexicon_tags_45 },
    { 0xa7d3af5b, SUC_lexicon_tags_17 },
    { 0x5433af5c, SUC_lexicon_tags_29 },
    { 0x5fb3af5b, SUC_lexicon_tags_2 },
    { 0x52f3af5b, SUC_lexicon_tags_15 },
    { 0x4493af5b, SUC_lexicon_tags_4 },
    { 0x6253af60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b3af63, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f3af6b, SUC_lexicon_tags_37 },
    { 0x6373af6c, SUC_lexicon_tags_28 },
    { 0x8a93af6d, SUC_lexicon_tags_17 },
    { 0x9713af6d, SUC_lexicon_tags_7 },
    { 0x6733af6d, SUC_lexicon_tags_63 },
    { 0x3493af70, SUC_lexicon_tags_11 },
    { 0x7e53af71, SUC_lexicon_tags_1 },
    { 0xad3af70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d3af7b, SUC_lexicon_tags_49 },
    { 0x90b3af7c, SUC_lexicon_tags_37 },
    { 0xcdb3af7c, SUC_lexicon_tags_0 },
    { 0x4393af7e, SUC_lexicon_tags_26 },
    { 0x64b3af7e, SUC_lexicon_tags_14 },
    { 0xcd13af7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1233af82, SUC_lexicon_tags_17 },
    { 0xd473af83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8fb3af85, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c93af88, SUC_lexicon_tags_7 },
    { 0xeab3af89, SUC_lexicon_tags_28 },
    { 0x1b93af8a, SUC_lexicon_tags_45 },
    { 0x34f3af8b, SUC_lexicon_tags_60 },
    { 0xc433af8c, SUC_lexicon_tags_9 },
    { 0x6053af8d, SUC_lexicon_tags_61 },
    { 0xf593af8d, SUC_lexicon_tags_16 },
    { 0xe1b3af8f, SUC_lexicon_tags_38 },
    { 0x4ed3af90, SUC_lexicon_tags_88 },
    { 0xd313af91, SUC_lexicon_tags_7 },
    { 0x91b3af92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe333af94, SUC_lexicon_tags_6 },
    { 0x4fd3af94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe893af98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd5b3af9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9973af9d, SUC_lexicon_tags_48 },
    { 0x99f3af9d, SUC_lexicon_tags_7 },
    { 0x88f3af9f, SUC_lexicon_tags_27 },
    { 0x413afa0, SUC_lexicon_tags_19 },
    { 0x6693afa1, SUC_lexicon_tags_47 },
    { 0xbcb3afa2, SUC_lexicon_tags_1 },
    { 0x9513afa1, SUC_lexicon_tags_47 },
    { 0x79b3af9f, SUC_lexicon_tags_30 },
    { 0x9a73afa3, SUC_lexicon_tags_63 },
    { 0xa313afa6, SUC_lexicon_tags_33 },
    { 0xff73afa7, SUC_lexicon_tags_11 },
    { 0x6173afa8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7253afaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe13afac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb3afaf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd453afb4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b33afbb, SUC_lexicon_tags_7 },
    { 0x4df3afbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b3afc0, SUC_lexicon_tags_31 },
    { 0x7f93afc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x333afc3, SUC_lexicon_tags_48 },
    { 0xcd33afc4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe8b3afc6, SUC_lexicon_tags_38 },
    { 0xc053afc7, SUC_lexicon_tags_47 },
    { 0xb233afc7, SUC_lexicon_tags_8 },
    { 0x6b93afc8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f3afd4, SUC_lexicon_tags_7 },
    { 0x8473afd5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5f53afd7, SUC_lexicon_tags_3 },
    { 0xc9f3afd8, SUC_lexicon_tags_47 },
    { 0x11b3afd9, SUC_lexicon_tags_7 },
    { 0x6253afda, SUC_lexicon_tags_60 },
    { 0xbbd3afda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd3afde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2af3afe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a33afe3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xef13afe6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b33afe9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf533afeb, SUC_lexicon_tags_12 },
    { 0xe5d3afeb, SUC_lexicon_tags_23 },
    { 0x693afed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4133aff4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf3affb, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf93affe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3013b000, SUC_lexicon_tags_11 },
    { 0xb3d3b001, SUC_lexicon_tags_7 },
    { 0xebb3b002, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c53b006, SUC_lexicon_tags_11 },
    { 0xac93b006, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f3b00d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4453b00f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc333b016, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e53b019, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453b01f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1133b021, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4873b025, SUC_lexicon_tags_47 },
    { 0x9373b025, SUC_lexicon_tags_27 },
    { 0xf1f3b027, SUC_lexicon_tags_31 },
    { 0x7693b028, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4ff3b02a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xaab3b02c, SUC_lexicon_tags_37 },
    { 0x63b3b02d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b3b033, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb53b03b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d3b040, SUC_lexicon_tags_27 },
    { 0xe433b041, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa733b045, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa693b047, SUC_lexicon_tags_45 },
    { 0xc253b047, SUC_lexicon_tags_57 },
    { 0xb453b047, SUC_lexicon_tags_11 },
    { 0x9fd3b04a, SUC_lexicon_tags_7 },
    { 0x3793b047, SUC_lexicon_tags_8 },
    { 0xf3b049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa373b04e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x953b051, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7d33b053, SUC_lexicon_tags_32 },
    { 0xc133b053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13b058, SUC_lexicon_tags_2 },
    { 0xca73b059, SUC_lexicon_tags_31 },
    { 0x7353b05a, SUC_lexicon_tags_1116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc73b060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x813b062, SUC_lexicon_tags_33 },
    { 0x5193b063, SUC_lexicon_tags_37 },
    { 0x58d3b064, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc313b068, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d53b06b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8fb3b06d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d13b06f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf573b072, SUC_lexicon_tags_37 },
    { 0xfbb3b073, SUC_lexicon_tags_32 },
    { 0xcc93b074, SUC_lexicon_tags_3 },
    { 0x6573b072, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df3b082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f3b084, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff93b089, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3213b08d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x34b3b08f, SUC_lexicon_tags_15 },
    { 0xf873b090, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf953b099, SUC_lexicon_tags_15 },
    { 0x9d3b09a, SUC_lexicon_tags_17 },
    { 0xf9b3b09b, SUC_lexicon_tags_29 },
    { 0xbb33b09a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf3b0a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf73b0a5, SUC_lexicon_tags_38 },
    { 0x83d3b0a6, SUC_lexicon_tags_1 },
    { 0x8cb3b0a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7313b0ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc533b0af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f3b0b6, SUC_lexicon_tags_38 },
    { 0x4e53b0b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1113b0ba, SUC_lexicon_tags_92 },
    { 0xf593b0bb, SUC_lexicon_tags_60 },
    { 0x9ff3b0bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2653b0c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b3b0c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc473b0c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c73b0cc, SUC_lexicon_tags_2 },
    { 0xf053b0cc, SUC_lexicon_tags_21 },
    { 0xa673b0cd, SUC_lexicon_tags_2 },
    { 0x9273b0cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8a33b0d1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3b0d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x73b3b0d6, SUC_lexicon_tags_60 },
    { 0x36f3b0d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7f53b0d9, SUC_lexicon_tags_11 },
    { 0xac33b0da, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a53b0dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f3b0e3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2173b0e5, SUC_lexicon_tags_12 },
    { 0x1f3b0e5, SUC_lexicon_tags_33 },
    { 0xaf3b0e7, SUC_lexicon_tags_11 },
    { 0xd273b0e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f93b0ea, SUC_lexicon_tags_45 },
    { 0x73f3b0eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4193b0ee, SUC_lexicon_tags_37 },
    { 0x233b0ef, SUC_lexicon_tags_121 },
    { 0x82f3b0ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f3b0f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec33b0f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d3b0fb, SUC_lexicon_tags_17 },
    { 0x7db3b0fc, SUC_lexicon_tags_29 },
    { 0x62b3b0fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c93b103, SUC_lexicon_tags_12 },
    { 0x8993b104, SUC_lexicon_tags_82 },
    { 0xd293b105, SUC_lexicon_tags_7 },
    { 0x8c13b104, SUC_lexicon_tags_9 },
    { 0xb113b105, SUC_lexicon_tags_107 },
    { 0x23f3b105, SUC_lexicon_tags_84 },
    { 0x1c33b109, SUC_lexicon_tags_49 },
    { 0x4173b109, SUC_lexicon_tags_66 },
    { 0x6453b10b, SUC_lexicon_tags_30 },
    { 0x6df3b10c, SUC_lexicon_tags_37 },
    { 0x75f3b104, SUC_lexicon_tags_43 },
    { 0x70b3b10e, SUC_lexicon_tags_0 },
    { 0x3213b10f, SUC_lexicon_tags_49 },
    { 0x85f3b10a, SUC_lexicon_tags_7 },
    { 0x24b3b10a, SUC_lexicon_tags_25 },
    { 0x38d3b112, SUC_lexicon_tags_39 },
    { 0x1973b113, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb3b11b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea73b11d, SUC_lexicon_tags_26 },
    { 0x17b3b11d, SUC_lexicon_tags_19 },
    { 0x1353b11f, SUC_lexicon_tags_7 },
    { 0xc953b11f, SUC_lexicon_tags_37 },
    { 0x2df3b11e, SUC_lexicon_tags_4 },
    { 0x973b122, SUC_lexicon_tags_25 },
    { 0xfaf3b11f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b3b126, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x88d3b128, SUC_lexicon_tags_8 },
    { 0x1ed3b129, SUC_lexicon_tags_15 },
    { 0x5753b12a, SUC_lexicon_tags_161 },
    { 0x193b12b, SUC_lexicon_tags_1 },
    { 0xeb13b12c, SUC_lexicon_tags_17 },
    { 0xeeb3b12d, SUC_lexicon_tags_13 },
    { 0xe673b12e, SUC_lexicon_tags_49 },
    { 0x9913b12c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8813b137, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d13b139, SUC_lexicon_tags_37 },
    { 0xa5f3b13a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe913b13c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a53b13f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9d3b141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8013b143, SUC_lexicon_tags_31 },
    { 0xa893b144, SUC_lexicon_tags_38 },
    { 0x2ab3b145, SUC_lexicon_tags_46 },
    { 0x3393b146, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26f3b148, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c53b14d, SUC_lexicon_tags_47 },
    { 0x2833b14d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cf3b150, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9673b154, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe813b157, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb3b15c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d3b15e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f3b166, SUC_lexicon_tags_22 },
    { 0x4053b167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8573b16b, SUC_lexicon_tags_7 },
    { 0x5173b16c, SUC_lexicon_tags_61 },
    { 0x5313b16d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xec73b170, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d53b173, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbb3b175, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9533b178, SUC_lexicon_tags_12 },
    { 0xdc93b178, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3b17f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b3b183, SUC_lexicon_tags_12 },
    { 0x8813b184, SUC_lexicon_tags_17 },
    { 0xfbf3b184, SUC_lexicon_tags_2 },
    { 0x11d3b186, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x473b188, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfed3b18a, SUC_lexicon_tags_15 },
    { 0xf3b18b, SUC_lexicon_tags_15 },
    { 0xbbd3b18a, SUC_lexicon_tags_12 },
    { 0xf673b18d, SUC_lexicon_tags_60 },
    { 0xb1d3b18a, SUC_lexicon_tags_26 },
    { 0xfb93b18f, SUC_lexicon_tags_25 },
    { 0x44d3b18f, SUC_lexicon_tags_25 },
    { 0x53b18b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8173b193, SUC_lexicon_tags_31 },
    { 0x3513b194, SUC_lexicon_tags_19 },
    { 0x34f3b194, SUC_lexicon_tags_61 },
    { 0x84b3b194, SUC_lexicon_tags_1 },
    { 0xcdb3b197, SUC_lexicon_tags_25 },
    { 0x66d3b195, SUC_lexicon_tags_33 },
    { 0xe533b197, SUC_lexicon_tags_61 },
    { 0x7393b19a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x50f3b19c, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1073b1a3, SUC_lexicon_tags_14 },
    { 0x6e53b1a3, SUC_lexicon_tags_45 },
    { 0x7333b1a4, SUC_lexicon_tags_9 },
    { 0xf673b1a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2593b1a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2993b1ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b13b1b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc713b1b7, SUC_lexicon_tags_2 },
    { 0xffb3b1b8, SUC_lexicon_tags_60 },
    { 0x553b1b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff3b1bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5353b1be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5233b1c1, SUC_lexicon_tags_17 },
    { 0x4fb3b1c1, SUC_lexicon_tags_12 },
    { 0xa2f3b1c2, SUC_lexicon_tags_33 },
    { 0x8953b1c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b53b1c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7193b1c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda73b1ce, SUC_lexicon_tags_7 },
    { 0xaff3b1cf, SUC_lexicon_tags_9 },
    { 0xb373b1ce, SUC_lexicon_tags_11 },
    { 0x2b73b1d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f3b1d7, SUC_lexicon_tags_77 },
    { 0x11d3b1d8, SUC_lexicon_tags_8 },
    { 0x2a33b1d7, SUC_lexicon_tags_2 },
    { 0x8333b1da, SUC_lexicon_tags_105 },
    { 0x2073b1db, SUC_lexicon_tags_53 },
    { 0x2bf3b1da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b3b1e2, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xe073b1e4, SUC_lexicon_tags_2 },
    { 0xb0b3b1e4, SUC_lexicon_tags_28 },
    { 0x933b1e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5793b1e8, SUC_lexicon_tags_11 },
    { 0xeeb3b1e8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb2b3b1eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5013b1ed, SUC_lexicon_tags_27 },
    { 0xf613b1ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4473b1f0, SUC_lexicon_tags_28 },
    { 0xcf33b1f1, SUC_lexicon_tags_25 },
    { 0xe1b3b1f1, SUC_lexicon_tags_61 },
    { 0x2033b1f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb093b1f5, SUC_lexicon_tags_38 },
    { 0x7713b1f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3813b1f8, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d3b1fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe413b1ff, SUC_lexicon_tags_212 },
    { 0x9f53b200, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb3b204, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1dd3b206, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x62d3b208, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7933b20b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d3b20e, SUC_lexicon_tags_47 },
    { 0xdc93b20f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd013b211, SUC_lexicon_tags_38 },
    { 0xc473b211, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe873b217, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9053b21b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf333b21e, SUC_lexicon_tags_11 },
    { 0xa473b21f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf3b228, SUC_lexicon_tags_7 },
    { 0xb153b229, SUC_lexicon_tags_37 },
    { 0xd893b229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1053b231, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26d3b233, SUC_lexicon_tags_14 },
    { 0x6853b234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0f3b236, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8473b238, SUC_lexicon_tags_2 },
    { 0xdf13b239, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f93b23c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x62b3b23e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa653b240, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf3b243, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf973b245, SUC_lexicon_tags_33 },
    { 0x5833b245, SUC_lexicon_tags_15 },
    { 0xbb13b247, SUC_lexicon_tags_12 },
    { 0x3553b247, SUC_lexicon_tags_33 },
    { 0x2933b249, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0x2993b24b, SUC_lexicon_tags_1 },
    { 0xc753b24c, SUC_lexicon_tags_939 },
    { 0x2553b24b, SUC_lexicon_tags_28 },
    { 0x3df3b24e, SUC_lexicon_tags_12 },
    { 0xc173b24f, SUC_lexicon_tags_63 },
    { 0xe093b24c, SUC_lexicon_tags_2 },
    { 0xaa33b251, SUC_lexicon_tags_15 },
    { 0x51d3b252, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1af3b254, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d3b257, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f3b263, SUC_lexicon_tags_46 },
    { 0xd9d3b264, SUC_lexicon_tags_60 },
    { 0xf5f3b263, SUC_lexicon_tags_17 },
    { 0xa3d3b266, SUC_lexicon_tags_207 },
    { 0x2193b263, SUC_lexicon_tags_45 },
    { 0x7213b268, SUC_lexicon_tags_2 },
    { 0x473b265, SUC_lexicon_tags_40 },
    { 0x61b3b26a, SUC_lexicon_tags_17 },
    { 0x76d3b263, SUC_lexicon_tags_82 },
    { 0x3613b265, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b3b272, SUC_lexicon_tags_11 },
    { 0x8e13b272, SUC_lexicon_tags_11 },
    { 0xbe13b274, SUC_lexicon_tags_48 },
    { 0x2db3b275, SUC_lexicon_tags_14 },
    { 0x99b3b276, SUC_lexicon_tags_66 },
    { 0x77d3b277, SUC_lexicon_tags_13 },
    { 0x5d13b272, SUC_lexicon_tags_45 },
    { 0x3d33b278, SUC_lexicon_tags_12 },
    { 0x2d3b27a, SUC_lexicon_tags_27 },
    { 0xf73b27b, SUC_lexicon_tags_2 },
    { 0x8f93b27c, SUC_lexicon_tags_1 },
    { 0x98d3b278, SUC_lexicon_tags_252 },
    { 0xa7f3b27e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b3b283, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9093b285, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9973b28a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613b291, SUC_lexicon_tags_47 },
    { 0xa6b3b291, SUC_lexicon_tags_19 },
    { 0xa4d3b292, SUC_lexicon_tags_37 },
    { 0x59f3b292, SUC_lexicon_tags_2 },
    { 0x4013b294, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73b298, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d3b29e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1f53b2a0, SUC_lexicon_tags_2 },
    { 0xd193b2a1, SUC_lexicon_tags_47 },
    { 0xb193b2a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa73b2a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4913b2a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9bd3b2a8, SUC_lexicon_tags_2 },
    { 0x7933b2a9, SUC_lexicon_tags_37 },
    { 0xe0d3b2a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8653b2b3, SUC_lexicon_tags_7 },
    { 0xdd73b2b4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d3b2b9, SUC_lexicon_tags_25 },
    { 0xe693b2b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd53b2bc, SUC_lexicon_tags_15 },
    { 0x7b93b2bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b3b2c1, SUC_lexicon_tags_25 },
    { 0x4173b2c2, SUC_lexicon_tags_0 },
    { 0x3a73b2c1, SUC_lexicon_tags_17 },
    { 0x26b3b2c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b3b2c7, SUC_lexicon_tags_25 },
    { 0x11f3b2c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f73b2cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d13b2cd, SUC_lexicon_tags_37 },
    { 0x9833b2cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6753b2d0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4cb3b2d2, SUC_lexicon_tags_14 },
    { 0xdbb3b2d3, SUC_lexicon_tags_37 },
    { 0x8a73b2d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f3b2d7, SUC_lexicon_tags_11 },
    { 0x2ff3b2d8, SUC_lexicon_tags_11 },
    { 0x6af3b2d8, SUC_lexicon_tags_1 },
    { 0x7b73b2d9, SUC_lexicon_tags_10 },
    { 0xebb3b2db, SUC_lexicon_tags_9 },
    { 0xaa93b2d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b3b2e5, SUC_lexicon_tags_13 },
    { 0xdb53b2e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf73b2e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1293b2ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b3b2f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8513b2f5, SUC_lexicon_tags_11 },
    { 0x8273b2f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4693b2fd, SUC_lexicon_tags_49 },
    { 0x5453b2fe, SUC_lexicon_tags_23 },
    { 0xed3b2ff, SUC_lexicon_tags_25 },
    { 0xaff3b2ff, SUC_lexicon_tags_11 },
    { 0xcd93b301, SUC_lexicon_tags_17 },
    { 0xf8f3b302, SUC_lexicon_tags_14 },
    { 0x3ef3b303, SUC_lexicon_tags_8 },
    { 0x2a73b304, SUC_lexicon_tags_2 },
    { 0x91b3b305, SUC_lexicon_tags_12 },
    { 0xf373b305, SUC_lexicon_tags_11 },
    { 0xd833b307, SUC_lexicon_tags_35 },
    { 0x6f3b307, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe193b30a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec93b30f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8393b314, SUC_lexicon_tags_26 },
    { 0x23b3b315, SUC_lexicon_tags_25 },
    { 0x6cf3b316, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xac93b318, SUC_lexicon_tags_14 },
    { 0x5c33b319, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f13b31b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6fd3b31d, SUC_lexicon_tags_17 },
    { 0x8b93b31d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef3b321, SUC_lexicon_tags_18 },
    { 0x5113b322, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273b326, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c33b32c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7513b32e, SUC_lexicon_tags_47 },
    { 0x3cf3b32f, SUC_lexicon_tags_28 },
    { 0xd513b32f, SUC_lexicon_tags_2 },
    { 0x5613b32e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc913b339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d3b346, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f3b34a, SUC_lexicon_tags_49 },
    { 0xe9f3b34a, SUC_lexicon_tags_2 },
    { 0x3b73b34c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b53b34e, SUC_lexicon_tags_7 },
    { 0x5573b34e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3353b354, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4113b357, SUC_lexicon_tags_63 },
    { 0xd193b358, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b73b35a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf793b35c, SUC_lexicon_tags_39 },
    { 0x3b93b35d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x13b3b35f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8533b362, SUC_lexicon_tags_7 },
    { 0xb733b363, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3233b367, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5993b36c, SUC_lexicon_tags_25 },
    { 0x1ed3b36c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe873b36f, SUC_lexicon_tags_25 },
    { 0x3a93b370, SUC_lexicon_tags_25 },
    { 0xe5f3b371, SUC_lexicon_tags_14 },
    { 0x6473b372, SUC_lexicon_tags_47 },
    { 0x7ff3b372, SUC_lexicon_tags_61 },
    { 0x7c13b373, SUC_lexicon_tags_11 },
    { 0x5373b375, SUC_lexicon_tags_15 },
    { 0x1d3b375, SUC_lexicon_tags_33 },
    { 0x7ff3b373, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x8a53b379, SUC_lexicon_tags_11 },
    { 0xc653b37a, SUC_lexicon_tags_2 },
    { 0xe0f3b37a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1213b37f, SUC_lexicon_tags_19 },
    { 0xba53b380, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb3b384, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13b388, SUC_lexicon_tags_17 },
    { 0x5453b389, SUC_lexicon_tags_17 },
    { 0x1cf3b38a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7833b390, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda73b392, SUC_lexicon_tags_37 },
    { 0xaf33b393, SUC_lexicon_tags_1 },
    { 0x3413b394, SUC_lexicon_tags_12 },
    { 0xd713b395, SUC_lexicon_tags_32 },
    { 0x3cf3b396, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x733b398, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0x1993b39a, SUC_lexicon_tags_9 },
    { 0xe253b39a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2693b39d, SUC_lexicon_tags_14 },
    { 0xf8f3b39d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x49f3b3a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c33b3a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6573b3a7, SUC_lexicon_tags_47 },
    { 0x8f33b3a7, SUC_lexicon_tags_27 },
    { 0x5573b3a9, SUC_lexicon_tags_15 },
    { 0x80f3b3a9, SUC_lexicon_tags_7 },
    { 0x5293b3ab, SUC_lexicon_tags_12 },
    { 0x5ff3b3ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4973b3ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf893b3b0, SUC_lexicon_tags_25 },
    { 0x1193b3b1, SUC_lexicon_tags_1 },
    { 0x9bd3b3b2, SUC_lexicon_tags_11 },
    { 0xbab3b3b3, SUC_lexicon_tags_13 },
    { 0x2f93b3b4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb3b3b8, SUC_lexicon_tags_37 },
    { 0x8b53b3b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65d3b3bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc33b3bd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb113b3c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb3b3c6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf513b3c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9253b3cf, SUC_lexicon_tags_7 },
    { 0x6df3b3d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb73b3d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df3b3d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b3b3dc, SUC_lexicon_tags_15 },
    { 0xd953b3dd, SUC_lexicon_tags_12 },
    { 0x1013b3de, SUC_lexicon_tags_1 },
    { 0x7073b3df, SUC_lexicon_tags_1 },
    { 0x89f3b3e0, SUC_lexicon_tags_91 },
    { 0x8d13b3e1, SUC_lexicon_tags_69 },
    { 0xdcd3b3e0, SUC_lexicon_tags_23 },
    { 0xd1b3b3e3, SUC_lexicon_tags_7 },
    { 0xc233b3e4, SUC_lexicon_tags_3 },
    { 0xaaf3b3e5, SUC_lexicon_tags_46 },
    { 0x1b3b3e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db3b3ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9393b3ec, SUC_lexicon_tags_63 },
    { 0xd673b3ed, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc73b3f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f3b3f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5313b3f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff93b3fb, SUC_lexicon_tags_7 },
    { 0xa913b3fc, SUC_lexicon_tags_23 },
    { 0x8313b3fc, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d3b400, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x62b3b402, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2973b404, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e13b40f, SUC_lexicon_tags_16 },
    { 0x9b3b410, SUC_lexicon_tags_7 },
    { 0xf9f3b40f, SUC_lexicon_tags_11 },
    { 0x33f3b412, SUC_lexicon_tags_26 },
    { 0xbdd3b413, SUC_lexicon_tags_7 },
    { 0x93b3b414, SUC_lexicon_tags_43 },
    { 0xd053b414, SUC_lexicon_tags_47 },
    { 0xe3d3b416, SUC_lexicon_tags_29 },
    { 0xe413b413, SUC_lexicon_tags_6 },
    { 0x70f3b418, SUC_lexicon_tags_29 },
    { 0x86d3b419, SUC_lexicon_tags_15 },
    { 0xf13b419, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9673b41c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec13b420, SUC_lexicon_tags_19 },
    { 0xcdd3b421, SUC_lexicon_tags_7 },
    { 0x2b53b422, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b3b426, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd3b42a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb613b42e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f3b431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb93b433, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd293b436, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a13b440, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31f3b442, SUC_lexicon_tags_17 },
    { 0x8f53b443, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa513b445, SUC_lexicon_tags_37 },
    { 0xc673b445, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa373b448, SUC_lexicon_tags_49 },
    { 0xc813b448, SUC_lexicon_tags_148 },
    { 0xb193b448, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd513b44c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe353b44f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7413b453, SUC_lexicon_tags_12 },
    { 0xb4b3b454, SUC_lexicon_tags_49 },
    { 0xc353b455, SUC_lexicon_tags_1 },
    { 0x16f3b456, SUC_lexicon_tags_37 },
    { 0x313b456, SUC_lexicon_tags_19 },
    { 0xa93b458, SUC_lexicon_tags_37 },
    { 0x42f3b459, SUC_lexicon_tags_661 },
    { 0xf393b456, SUC_lexicon_tags_40 },
    { 0xdf53b45b, SUC_lexicon_tags_47 },
    { 0x6353b45c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4593b461, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6b13b463, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d3b466, SUC_lexicon_tags_30 },
    { 0x72f3b466, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f3b46b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db3b470, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8933b475, SUC_lexicon_tags_0 },
    { 0x7973b475, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc853b47a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7233b47d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf453b48b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x533b48d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9093b490, SUC_lexicon_tags_25 },
    { 0x4f33b491, SUC_lexicon_tags_37 },
    { 0x64d3b492, SUC_lexicon_tags_17 },
    { 0xb113b492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd53b495, SUC_lexicon_tags_38 },
    { 0x8a53b496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4493b498, SUC_lexicon_tags_7 },
    { 0x97d3b498, SUC_lexicon_tags_15 },
    { 0x1133b49a, SUC_lexicon_tags_1 },
    { 0xe2b3b49a, SUC_lexicon_tags_30 },
    { 0x2d13b49c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5333b4a1, SUC_lexicon_tags_47 },
    { 0x2953b4a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce33b4a6, SUC_lexicon_tags_29 },
    { 0xae13b4a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2753b4aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f3b4ae, SUC_lexicon_tags_2 },
    { 0xed93b4ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1573b4b4, SUC_lexicon_tags_19 },
    { 0x37f3b4b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d3b4b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa733b4be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd3b4c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e33b4c2, SUC_lexicon_tags_1 },
    { 0xa2f3b4c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b3b4c6, SUC_lexicon_tags_61 },
    { 0x3033b4c6, SUC_lexicon_tags_2 },
    { 0x18f3b4c8, SUC_lexicon_tags_8 },
    { 0xf9f3b4c9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x3b93b4cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9313b4d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6573b4d3, SUC_lexicon_tags_27 },
    { 0x1013b4d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb893b4d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb3b4dd, SUC_lexicon_tags_26 },
    { 0xeef3b4de, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf3b4e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5273b4e5, SUC_lexicon_tags_12 },
    { 0xed3b4e6, SUC_lexicon_tags_35 },
    { 0x5b53b4e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf53b4ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa433b4ec, SUC_lexicon_tags_31 },
    { 0x1d3b4ed, SUC_lexicon_tags_29 },
    { 0x1bf3b4ed, SUC_lexicon_tags_11 },
    { 0xb533b4ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a33b4f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x53b3b4f8, SUC_lexicon_tags_53 },
    { 0xfe33b4f8, SUC_lexicon_tags_15 },
    { 0x2973b4fa, SUC_lexicon_tags_12 },
    { 0x4bd3b4fb, SUC_lexicon_tags_12 },
    { 0x6bf3b4fb, SUC_lexicon_tags_31 },
    { 0xa0d3b4f8, SUC_lexicon_tags_70 },
    { 0x6c13b4fe, SUC_lexicon_tags_178 },
    { 0x9793b4fd, SUC_lexicon_tags_60 },
    { 0x9f13b500, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0f3b502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb33b505, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf453b507, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3b50e, SUC_lexicon_tags_15 },
    { 0xd5f3b50f, SUC_lexicon_tags_16 },
    { 0x2a33b50f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d3b515, SUC_lexicon_tags_15 },
    { 0x2d93b515, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d3b51b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2cb3b51d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe233b522, SUC_lexicon_tags_31 },
    { 0xf073b523, SUC_lexicon_tags_41 },
    { 0x553b523, SUC_lexicon_tags_27 },
    { 0x8533b522, SUC_lexicon_tags_17 },
    { 0xffd3b526, SUC_lexicon_tags_43 },
    { 0x94b3b525, SUC_lexicon_tags_26 },
    { 0xc9b3b528, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa293b52a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e93b534, SUC_lexicon_tags_3 },
    { 0xf793b534, SUC_lexicon_tags_38 },
    { 0x33d3b536, SUC_lexicon_tags_101 },
    { 0x8af3b537, SUC_lexicon_tags_2 },
    { 0x733b535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5d3b53a, SUC_lexicon_tags_1 },
    { 0x5c53b53a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x933b53e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4f93b540, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d93b542, SUC_lexicon_tags_11 },
    { 0xa793b542, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8e93b545, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6413b549, SUC_lexicon_tags_13 },
    { 0x3673b54a, SUC_lexicon_tags_160 },
    { 0x973b54b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8cd3b54d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a53b550, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb3b554, SUC_lexicon_tags_647 },
    { 0, NULL },
    { 0xe0b3b556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc573b55a, SUC_lexicon_tags_19 },
    { 0x19f3b55b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6113b55f, SUC_lexicon_tags_1 },
    { 0x1b33b55f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b53b563, SUC_lexicon_tags_45 },
    { 0xc113b564, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe153b56b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa153b573, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef3b577, SUC_lexicon_tags_23 },
    { 0x1373b578, SUC_lexicon_tags_9 },
    { 0xe793b577, SUC_lexicon_tags_2 },
    { 0x91f3b57a, SUC_lexicon_tags_33 },
    { 0xf253b57b, SUC_lexicon_tags_19 },
    { 0x72b3b57c, SUC_lexicon_tags_7 },
    { 0xd6b3b57c, SUC_lexicon_tags_38 },
    { 0x7913b57b, SUC_lexicon_tags_25 },
    { 0x9f73b57f, SUC_lexicon_tags_2 },
    { 0x94f3b580, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x45f3b582, SUC_lexicon_tags_7 },
    { 0x1e33b582, SUC_lexicon_tags_2 },
    { 0x1cd3b584, SUC_lexicon_tags_19 },
    { 0xf593b583, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a13b588, SUC_lexicon_tags_30 },
    { 0xf393b589, SUC_lexicon_tags_43 },
    { 0x1173b58a, SUC_lexicon_tags_7 },
    { 0x9b13b58a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9db3b58d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f3b591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8853b593, SUC_lexicon_tags_23 },
    { 0x4093b594, SUC_lexicon_tags_8 },
    { 0xb373b595, SUC_lexicon_tags_8 },
    { 0x97b3b593, SUC_lexicon_tags_17 },
    { 0x6f53b597, SUC_lexicon_tags_49 },
    { 0xe373b597, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6633b59a, SUC_lexicon_tags_37 },
    { 0x5673b59b, SUC_lexicon_tags_15 },
    { 0x7253b59a, SUC_lexicon_tags_7 },
    { 0xa1d3b59d, SUC_lexicon_tags_29 },
    { 0x8493b59c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7113b5a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf093b5a2, SUC_lexicon_tags_37 },
    { 0xd653b5a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d3b5a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3473b5ac, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd693b5ae, SUC_lexicon_tags_27 },
    { 0xa633b5af, SUC_lexicon_tags_25 },
    { 0x5453b5b0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf3b5b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3633b5b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9d3b5b9, SUC_lexicon_tags_43 },
    { 0x3113b5ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb3b5bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc113b5c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6973b5d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f13b5d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe073b5d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab3b5dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f3b5e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x833b5e5, SUC_lexicon_tags_11 },
    { 0xb393b5e6, SUC_lexicon_tags_23 },
    { 0xfd33b5e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8693b5ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5013b5ed, SUC_lexicon_tags_95 },
    { 0x5873b5ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3b5f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe13b5f3, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x38f3b5f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b3b5fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xda93b600, SUC_lexicon_tags_2 },
    { 0xb013b601, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a33b605, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad3b608, SUC_lexicon_tags_0 },
    { 0x7633b609, SUC_lexicon_tags_47 },
    { 0x8cd3b609, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf033b60d, SUC_lexicon_tags_6 },
    { 0x1813b60e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d3b612, SUC_lexicon_tags_25 },
    { 0x5eb3b612, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x693b616, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6353b618, SUC_lexicon_tags_7 },
    { 0x7353b618, SUC_lexicon_tags_1 },
    { 0x2753b61a, SUC_lexicon_tags_2 },
    { 0x4473b61b, SUC_lexicon_tags_7 },
    { 0xc913b618, SUC_lexicon_tags_28 },
    { 0x2233b61c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x633b61f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4933b621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6733b623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613b62c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe4d3b62e, SUC_lexicon_tags_37 },
    { 0x5a13b62f, SUC_lexicon_tags_14 },
    { 0xab73b630, SUC_lexicon_tags_12 },
    { 0x8333b631, SUC_lexicon_tags_38 },
    { 0xc053b631, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb3b634, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c73b63b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d3b63f, SUC_lexicon_tags_30 },
    { 0xd413b63f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b3b646, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6193b648, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x31b3b64a, SUC_lexicon_tags_83 },
    { 0xd273b64b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd3b650, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf673b654, SUC_lexicon_tags_8 },
    { 0x253b654, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e13b657, SUC_lexicon_tags_61 },
    { 0xc933b658, SUC_lexicon_tags_5 },
    { 0xffb3b658, SUC_lexicon_tags_8 },
    { 0x3573b657, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf3b65e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc753b660, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb3b666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb3b66c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ff3b66e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xde93b671, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d3b675, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc013b678, SUC_lexicon_tags_47 },
    { 0x8273b679, SUC_lexicon_tags_39 },
    { 0xee73b67a, SUC_lexicon_tags_2 },
    { 0x1cf3b67a, SUC_lexicon_tags_43 },
    { 0x7cd3b678, SUC_lexicon_tags_18 },
    { 0xab13b67b, SUC_lexicon_tags_11 },
    { 0xcd53b67e, SUC_lexicon_tags_46 },
    { 0x3433b67f, SUC_lexicon_tags_9 },
    { 0x2013b680, SUC_lexicon_tags_17 },
    { 0x3213b680, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd193b686, SUC_lexicon_tags_35 },
    { 0xa593b687, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f3b68b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2af3b68d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a13b690, SUC_lexicon_tags_25 },
    { 0x1d53b691, SUC_lexicon_tags_11 },
    { 0xc853b692, SUC_lexicon_tags_42 },
    { 0x36b3b693, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab93b698, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3973b69c, SUC_lexicon_tags_49 },
    { 0x1093b69d, SUC_lexicon_tags_85 },
    { 0x9013b69e, SUC_lexicon_tags_4 },
    { 0x22f3b69e, SUC_lexicon_tags_25 },
    { 0xdcd3b69c, SUC_lexicon_tags_60 },
    { 0x6013b69f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2413b6a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd113b6a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x593b6a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4cd3b6ab, SUC_lexicon_tags_6 },
    { 0x973b6ac, SUC_lexicon_tags_2 },
    { 0x60d3b6ad, SUC_lexicon_tags_33 },
    { 0xd0d3b6ae, SUC_lexicon_tags_64 },
    { 0x6a13b6ae, SUC_lexicon_tags_37 },
    { 0xccb3b6ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b3b6b6, SUC_lexicon_tags_11 },
    { 0x6913b6b7, SUC_lexicon_tags_37 },
    { 0xa7b3b6b7, SUC_lexicon_tags_17 },
    { 0xfeb3b6b6, SUC_lexicon_tags_36 },
    { 0x6673b6b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b53b6bd, SUC_lexicon_tags_11 },
    { 0xd253b6be, SUC_lexicon_tags_30 },
    { 0x6db3b6be, SUC_lexicon_tags_11 },
    { 0xb5d3b6c0, SUC_lexicon_tags_11 },
    { 0x5433b6c1, SUC_lexicon_tags_37 },
    { 0xd3d3b6c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9933b6c6, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa73b6c9, SUC_lexicon_tags_12 },
    { 0xa6f3b6ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93b6d5, SUC_lexicon_tags_25 },
    { 0x1673b6d6, SUC_lexicon_tags_11 },
    { 0x1d3b6d6, SUC_lexicon_tags_32 },
    { 0x7f33b6d5, SUC_lexicon_tags_30 },
    { 0x2653b6d9, SUC_lexicon_tags_1 },
    { 0xb853b6d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca33b6df, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f3b6e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb533b6e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833b6ea, SUC_lexicon_tags_55 },
    { 0x5eb3b6ea, SUC_lexicon_tags_21 },
    { 0x4833b6eb, SUC_lexicon_tags_60 },
    { 0xadf3b6ed, SUC_lexicon_tags_2 },
    { 0x8393b6ea, SUC_lexicon_tags_42 },
    { 0x97d3b6ef, SUC_lexicon_tags_27 },
    { 0xe273b6f0, SUC_lexicon_tags_11 },
    { 0x7ed3b6f0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e33b6f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac93b6f7, SUC_lexicon_tags_11 },
    { 0xba93b6f8, SUC_lexicon_tags_17 },
    { 0x2cb3b6f9, SUC_lexicon_tags_2 },
    { 0xc093b6fa, SUC_lexicon_tags_2 },
    { 0x5e73b6fb, SUC_lexicon_tags_33 },
    { 0x8213b6fc, SUC_lexicon_tags_2 },
    { 0x2993b6fb, SUC_lexicon_tags_17 },
    { 0x53b3b6f7, SUC_lexicon_tags_2 },
    { 0x43f3b6ff, SUC_lexicon_tags_17 },
    { 0x22f3b6fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5393b70a, SUC_lexicon_tags_29 },
    { 0x5ed3b70b, SUC_lexicon_tags_17 },
    { 0xe0d3b70c, SUC_lexicon_tags_28 },
    { 0xcc53b70d, SUC_lexicon_tags_12 },
    { 0xd1f3b70d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f3b712, SUC_lexicon_tags_422 },
    { 0x7cb3b713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe553b715, SUC_lexicon_tags_17 },
    { 0xf553b716, SUC_lexicon_tags_37 },
    { 0x1d33b717, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4e33b719, SUC_lexicon_tags_26 },
    { 0xea13b71a, SUC_lexicon_tags_27 },
    { 0x2113b71b, SUC_lexicon_tags_0 },
    { 0xf0b3b71c, SUC_lexicon_tags_0 },
    { 0xe153b71d, SUC_lexicon_tags_19 },
    { 0x253b71b, SUC_lexicon_tags_7 },
    { 0x6ed3b71d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f93b726, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc133b72c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3293b72e, SUC_lexicon_tags_37 },
    { 0xc13b72e, SUC_lexicon_tags_16 },
    { 0xcaf3b730, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd653b735, SUC_lexicon_tags_33 },
    { 0xc333b735, SUC_lexicon_tags_37 },
    { 0xe1f3b737, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb433b73d, SUC_lexicon_tags_17 },
    { 0xf793b73e, SUC_lexicon_tags_37 },
    { 0xacf3b73f, SUC_lexicon_tags_77 },
    { 0x5313b740, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xb2b3b742, SUC_lexicon_tags_25 },
    { 0x5513b743, SUC_lexicon_tags_2 },
    { 0x1c93b743, SUC_lexicon_tags_1 },
    { 0x9b3b742, SUC_lexicon_tags_15 },
    { 0x4753b742, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d3b749, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3753b74b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93b750, SUC_lexicon_tags_30 },
    { 0x4a73b750, SUC_lexicon_tags_61 },
    { 0xa573b750, SUC_lexicon_tags_14 },
    { 0xe993b753, SUC_lexicon_tags_37 },
    { 0x9113b754, SUC_lexicon_tags_2 },
    { 0x5053b754, SUC_lexicon_tags_3 },
    { 0x2273b755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44b3b758, SUC_lexicon_tags_47 },
    { 0xf5b3b758, SUC_lexicon_tags_25 },
    { 0xddb3b75a, SUC_lexicon_tags_29 },
    { 0x89b3b75b, SUC_lexicon_tags_2 },
    { 0x3673b75c, SUC_lexicon_tags_7 },
    { 0xa0f3b75c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb313b75f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf3b762, SUC_lexicon_tags_84 },
    { 0xcab3b763, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6753b767, SUC_lexicon_tags_34 },
    { 0x6fb3b767, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b53b76f, SUC_lexicon_tags_48 },
    { 0xfb53b770, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b53b772, SUC_lexicon_tags_15 },
    { 0x2dd3b773, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1813b777, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7df3b779, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b3b780, SUC_lexicon_tags_47 },
    { 0xe573b781, SUC_lexicon_tags_38 },
    { 0x8ef3b782, SUC_lexicon_tags_7 },
    { 0xbf13b780, SUC_lexicon_tags_1 },
    { 0xf393b784, SUC_lexicon_tags_17 },
    { 0x8ef3b785, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53b78a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b3b78d, SUC_lexicon_tags_11 },
    { 0x5b33b78e, SUC_lexicon_tags_7 },
    { 0xf633b78d, SUC_lexicon_tags_15 },
    { 0x7f3b78f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb613b795, SUC_lexicon_tags_30 },
    { 0xc6d3b795, SUC_lexicon_tags_37 },
    { 0xbc33b797, SUC_lexicon_tags_11 },
    { 0xec13b796, SUC_lexicon_tags_7 },
    { 0x9473b797, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d53b79b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60d3b79d, SUC_lexicon_tags_68 },
    { 0xd4b3b79e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb013b7a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d3b7a4, SUC_lexicon_tags_14 },
    { 0x3cf3b7a5, SUC_lexicon_tags_11 },
    { 0xc593b7a5, SUC_lexicon_tags_15 },
    { 0xd53b7a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b3b7aa, SUC_lexicon_tags_0 },
    { 0xf133b7ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdd13b7ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc053b7af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa33b7b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7493b7b8, SUC_lexicon_tags_19 },
    { 0x9ad3b7b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c13b7bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f13b7c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d73b7c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb013b7c5, SUC_lexicon_tags_37 },
    { 0xff3b7c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf493b7c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc473b7ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff3b7cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fb3b7cf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x43d3b7d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x253b7d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf673b7d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf913b7df, SUC_lexicon_tags_0 },
    { 0x7093b7df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb3b7e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5813b7ed, SUC_lexicon_tags_2 },
    { 0x7d3b7ed, SUC_lexicon_tags_777 },
    { 0xbb33b7ee, SUC_lexicon_tags_23 },
    { 0x3c53b7ef, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe233b7f2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb3b7fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6b3b7fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1073b800, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5213b807, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1713b813, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2f3b815, SUC_lexicon_tags_49 },
    { 0x1ef3b815, SUC_lexicon_tags_10 },
    { 0x6a33b817, SUC_lexicon_tags_2 },
    { 0x16f3b817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4153b81a, SUC_lexicon_tags_49 },
    { 0x7513b81b, SUC_lexicon_tags_37 },
    { 0xf673b81a, SUC_lexicon_tags_45 },
    { 0xc933b81a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4293b824, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1153b826, SUC_lexicon_tags_1 },
    { 0xc633b826, SUC_lexicon_tags_37 },
    { 0xacb3b828, SUC_lexicon_tags_25 },
    { 0x6c33b827, SUC_lexicon_tags_42 },
    { 0x67b3b826, SUC_lexicon_tags_26 },
    { 0xeb3b828, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe693b831, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa853b834, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x69f3b836, SUC_lexicon_tags_7 },
    { 0x20f3b837, SUC_lexicon_tags_11 },
    { 0x96f3b838, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1d73b83a, SUC_lexicon_tags_37 },
    { 0x7b73b83b, SUC_lexicon_tags_14 },
    { 0xeff3b83b, SUC_lexicon_tags_37 },
    { 0xff93b83d, SUC_lexicon_tags_7 },
    { 0x7e13b83d, SUC_lexicon_tags_30 },
    { 0xa933b83f, SUC_lexicon_tags_37 },
    { 0xc333b840, SUC_lexicon_tags_0 },
    { 0x1053b83f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ed3b843, SUC_lexicon_tags_6 },
    { 0xb193b843, SUC_lexicon_tags_17 },
    { 0xa73b845, SUC_lexicon_tags_15 },
    { 0xd6d3b846, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x113b849, SUC_lexicon_tags_0 },
    { 0x7933b84a, SUC_lexicon_tags_151 },
    { 0x4993b84a, SUC_lexicon_tags_7 },
    { 0x2773b84a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x23b3b84e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e53b851, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3cb3b853, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb3b85d, SUC_lexicon_tags_25 },
    { 0x3993b85d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d33b864, SUC_lexicon_tags_19 },
    { 0x6493b865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb33b868, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8293b86a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f93b872, SUC_lexicon_tags_3 },
    { 0x3f3b872, SUC_lexicon_tags_0 },
    { 0x4333b873, SUC_lexicon_tags_26 },
    { 0x9853b875, SUC_lexicon_tags_25 },
    { 0xd853b873, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3d73b878, SUC_lexicon_tags_55 },
    { 0xc533b878, SUC_lexicon_tags_45 },
    { 0x5593b87a, SUC_lexicon_tags_23 },
    { 0xa3d3b87a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0d3b87d, SUC_lexicon_tags_13 },
    { 0xb773b87e, SUC_lexicon_tags_27 },
    { 0x8173b87e, SUC_lexicon_tags_15 },
    { 0xf653b87e, SUC_lexicon_tags_11 },
    { 0xc8b3b87e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x413b884, SUC_lexicon_tags_15 },
    { 0x6133b885, SUC_lexicon_tags_26 },
    { 0x59f3b886, SUC_lexicon_tags_15 },
    { 0xf113b887, SUC_lexicon_tags_63 },
    { 0xf053b886, SUC_lexicon_tags_12 },
    { 0x7d33b886, SUC_lexicon_tags_1 },
    { 0x5a93b88a, SUC_lexicon_tags_19 },
    { 0xc773b88a, SUC_lexicon_tags_17 },
    { 0xd753b88c, SUC_lexicon_tags_31 },
    { 0xffb3b888, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d33b890, SUC_lexicon_tags_11 },
    { 0xad53b890, SUC_lexicon_tags_27 },
    { 0x1d33b891, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde33b894, SUC_lexicon_tags_12 },
    { 0x61f3b895, SUC_lexicon_tags_27 },
    { 0x6753b896, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1593b89c, SUC_lexicon_tags_72 },
    { 0x6fb3b89d, SUC_lexicon_tags_49 },
    { 0x87f3b89e, SUC_lexicon_tags_31 },
    { 0xd193b89f, SUC_lexicon_tags_25 },
    { 0x6f53b8a0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3173b8a3, SUC_lexicon_tags_46 },
    { 0x9333b8a3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf73b8aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xab73b8ad, SUC_lexicon_tags_15 },
    { 0xd213b8ad, SUC_lexicon_tags_48 },
    { 0xc353b8af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7033b8b1, SUC_lexicon_tags_12 },
    { 0x12f3b8b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5313b8b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f73b8b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb53b8bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd53b8bf, SUC_lexicon_tags_47 },
    { 0xe273b8c0, SUC_lexicon_tags_19 },
    { 0xdd3b8c1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a33b8c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc893b8ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9633b8cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb3b8cf, SUC_lexicon_tags_6 },
    { 0xa973b8cf, SUC_lexicon_tags_17 },
    { 0xa7d3b8d0, SUC_lexicon_tags_32 },
    { 0x3b3b8d2, SUC_lexicon_tags_17 },
    { 0xb0b3b8d2, SUC_lexicon_tags_2 },
    { 0xc413b8d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af3b8d9, SUC_lexicon_tags_11 },
    { 0xc8f3b8da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb3b8df, SUC_lexicon_tags_33 },
    { 0xdab3b8df, SUC_lexicon_tags_1 },
    { 0x6513b8e1, SUC_lexicon_tags_131 },
    { 0xfe93b8e1, SUC_lexicon_tags_6 },
    { 0x6c13b8e3, SUC_lexicon_tags_17 },
    { 0xca13b8df, SUC_lexicon_tags_47 },
    { 0x9e13b8e1, SUC_lexicon_tags_55 },
    { 0xa4f3b8e1, SUC_lexicon_tags_37 },
    { 0x64b3b8e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d3b8f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f93b8f8, SUC_lexicon_tags_2 },
    { 0xbd93b8f9, SUC_lexicon_tags_11 },
    { 0xb8f3b8f9, SUC_lexicon_tags_38 },
    { 0xd973b8fb, SUC_lexicon_tags_6 },
    { 0x8af3b8fa, SUC_lexicon_tags_47 },
    { 0xd973b8f8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xe5f3b8ff, SUC_lexicon_tags_63 },
    { 0x1993b900, SUC_lexicon_tags_2 },
    { 0x7013b900, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7933b905, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5f33b907, SUC_lexicon_tags_1 },
    { 0xb693b908, SUC_lexicon_tags_19 },
    { 0x1393b909, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5633b910, SUC_lexicon_tags_9 },
    { 0xbb93b911, SUC_lexicon_tags_8 },
    { 0x6233b911, SUC_lexicon_tags_17 },
    { 0xf1b3b912, SUC_lexicon_tags_1 },
    { 0x16d3b911, SUC_lexicon_tags_19 },
    { 0x4933b913, SUC_lexicon_tags_28 },
    { 0x8253b916, SUC_lexicon_tags_14 },
    { 0x5633b916, SUC_lexicon_tags_21 },
    { 0x3c53b918, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe813b91a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b13b91e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa393b929, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2253b92b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b73b92e, SUC_lexicon_tags_1 },
    { 0xe5f3b92f, SUC_lexicon_tags_25 },
    { 0xb1d3b92e, SUC_lexicon_tags_27 },
    { 0x2f3b931, SUC_lexicon_tags_69 },
    { 0xfb3b932, SUC_lexicon_tags_65 },
    { 0x2f13b933, SUC_lexicon_tags_9 },
    { 0x4dd3b934, SUC_lexicon_tags_37 },
    { 0x2153b935, SUC_lexicon_tags_1 },
    { 0xdd33b931, SUC_lexicon_tags_25 },
    { 0x6c33b937, SUC_lexicon_tags_37 },
    { 0x31d3b938, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9733b93c, SUC_lexicon_tags_11 },
    { 0xf533b93c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9813b940, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe13b942, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f3b945, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f93b948, SUC_lexicon_tags_21 },
    { 0x62f3b948, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f33b950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe53b952, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4853b955, SUC_lexicon_tags_255 },
    { 0x73b3b956, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3b959, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5df3b95b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9493b95e, SUC_lexicon_tags_12 },
    { 0x6893b95e, SUC_lexicon_tags_15 },
    { 0x2073b960, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x85b3b962, SUC_lexicon_tags_12 },
    { 0xd73b963, SUC_lexicon_tags_87 },
    { 0x5253b964, SUC_lexicon_tags_37 },
    { 0xe213b965, SUC_lexicon_tags_10 },
    { 0xb693b964, SUC_lexicon_tags_61 },
    { 0xb453b964, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b53b96a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca93b96c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd3b96f, SUC_lexicon_tags_66 },
    { 0x3a53b970, SUC_lexicon_tags_43 },
    { 0x5853b971, SUC_lexicon_tags_17 },
    { 0x8713b971, SUC_lexicon_tags_17 },
    { 0xffb3b96f, SUC_lexicon_tags_15 },
    { 0x27d3b972, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8573b978, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a53b97c, SUC_lexicon_tags_35 },
    { 0x793b97d, SUC_lexicon_tags_16 },
    { 0x3153b97e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b3b985, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3973b988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3853b98b, SUC_lexicon_tags_7 },
    { 0x433b98b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3b98f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6053b991, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3b3b993, SUC_lexicon_tags_6 },
    { 0x3593b994, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6313b997, SUC_lexicon_tags_38 },
    { 0x9733b998, SUC_lexicon_tags_2 },
    { 0x1ef3b998, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3b99f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x29b3b9a1, SUC_lexicon_tags_26 },
    { 0x3d93b9a2, SUC_lexicon_tags_3 },
    { 0xe413b9a1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc293b9a8, SUC_lexicon_tags_106 },
    { 0xa5d3b9a9, SUC_lexicon_tags_87 },
    { 0x7313b9aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd953b9ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf313b9af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4993b9b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb3b9b6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9813b9ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4373b9bc, SUC_lexicon_tags_47 },
    { 0xbd13b9bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f3b9c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f73b9c3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293b9cc, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x31b3b9ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf393b9d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2fd3b9d3, SUC_lexicon_tags_70 },
    { 0x7693b9d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe033b9dc, SUC_lexicon_tags_17 },
    { 0xe373b9dd, SUC_lexicon_tags_25 },
    { 0x3ad3b9de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7673b9e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf713b9e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c53b9ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4893b9f1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3493b9f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8913b9fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf253b9fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3053ba03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbaf3ba05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb3ba08, SUC_lexicon_tags_12 },
    { 0x36f3ba09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2233ba13, SUC_lexicon_tags_17 },
    { 0x84d3ba14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd33ba1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4053ba21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f3ba24, SUC_lexicon_tags_0 },
    { 0x3333ba24, SUC_lexicon_tags_11 },
    { 0x2293ba25, SUC_lexicon_tags_17 },
    { 0xa193ba25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa633ba2c, SUC_lexicon_tags_49 },
    { 0xc173ba2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bd3ba2f, SUC_lexicon_tags_45 },
    { 0x3eb3ba30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7b3ba32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3133ba3c, SUC_lexicon_tags_0 },
    { 0xf793ba3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c73ba40, SUC_lexicon_tags_4 },
    { 0x7453ba41, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef53ba4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a93ba52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2413ba55, SUC_lexicon_tags_11 },
    { 0x8ed3ba56, SUC_lexicon_tags_19 },
    { 0xbef3ba55, SUC_lexicon_tags_64 },
    { 0xb1b3ba57, SUC_lexicon_tags_23 },
    { 0x62b3ba59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f3ba5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1053ba66, SUC_lexicon_tags_17 },
    { 0x9273ba67, SUC_lexicon_tags_48 },
    { 0xe933ba68, SUC_lexicon_tags_33 },
    { 0x4f53ba68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd473ba6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea93ba6e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b3ba72, SUC_lexicon_tags_0 },
    { 0x35f3ba73, SUC_lexicon_tags_60 },
    { 0xa913ba74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f3ba7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc33ba7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f3ba82, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe1b3ba84, SUC_lexicon_tags_10 },
    { 0x10d3ba84, SUC_lexicon_tags_1 },
    { 0x5493ba84, SUC_lexicon_tags_19 },
    { 0x2733ba84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c53ba8c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8173ba8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad3ba93, SUC_lexicon_tags_6 },
    { 0x60f3ba94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b53ba99, SUC_lexicon_tags_0 },
    { 0xf233ba9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4773ba9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x70b3ba9e, SUC_lexicon_tags_17 },
    { 0x36d3ba9e, SUC_lexicon_tags_19 },
    { 0x1e13baa0, SUC_lexicon_tags_8 },
    { 0x1573baa1, SUC_lexicon_tags_57 },
    { 0x6a93baa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa373baa7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x76d3baa9, SUC_lexicon_tags_43 },
    { 0x2973baaa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad3baad, SUC_lexicon_tags_15 },
    { 0x7d3baae, SUC_lexicon_tags_21 },
    { 0x88f3baaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x36f3bab1, SUC_lexicon_tags_1 },
    { 0x1853bab1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe6d3bab4, SUC_lexicon_tags_254 },
    { 0xef53bab5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce53bab9, SUC_lexicon_tags_60 },
    { 0x33baba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab93babc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab3babf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d3bac3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x43b3bac5, SUC_lexicon_tags_25 },
    { 0x4b13bac6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7153bac8, SUC_lexicon_tags_25 },
    { 0x1993bac9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f3bace, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1453bad0, SUC_lexicon_tags_60 },
    { 0xe513bad1, SUC_lexicon_tags_10 },
    { 0xa1d3bad2, SUC_lexicon_tags_60 },
    { 0x5413bad3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8dd3bad5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9473badb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa013badd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c53bae1, SUC_lexicon_tags_26 },
    { 0xe273bae2, SUC_lexicon_tags_19 },
    { 0xe873bae3, SUC_lexicon_tags_49 },
    { 0x8f3bae1, SUC_lexicon_tags_65 },
    { 0xa233bae4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d3baeb, SUC_lexicon_tags_87 },
    { 0xda93baec, SUC_lexicon_tags_12 },
    { 0x35b3baed, SUC_lexicon_tags_2 },
    { 0xaef3baec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d3baf1, SUC_lexicon_tags_43 },
    { 0x2753baf2, SUC_lexicon_tags_6 },
    { 0xe973baf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc093bafa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa813bafd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf3bb03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b3bb09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa473bb0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d13bb14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b3bb19, SUC_lexicon_tags_25 },
    { 0x27b3bb1a, SUC_lexicon_tags_7 },
    { 0xb113bb1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d13bb22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe913bb24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6d3bb26, SUC_lexicon_tags_19 },
    { 0x7153bb27, SUC_lexicon_tags_9 },
    { 0xabb3bb28, SUC_lexicon_tags_11 },
    { 0xd7b3bb29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f13bb2c, SUC_lexicon_tags_46 },
    { 0x7953bb2c, SUC_lexicon_tags_11 },
    { 0xd413bb2e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f3bb31, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xd353bb33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7853bb37, SUC_lexicon_tags_19 },
    { 0x6d3bb37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd633bb3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd133bb3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1953bb42, SUC_lexicon_tags_14 },
    { 0x3873bb43, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f3bb46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceb3bb48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5933bb4c, SUC_lexicon_tags_37 },
    { 0xee73bb4d, SUC_lexicon_tags_11 },
    { 0xbe33bb4d, SUC_lexicon_tags_25 },
    { 0x4eb3bb4c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe853bb52, SUC_lexicon_tags_15 },
    { 0x1233bb53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a33bb58, SUC_lexicon_tags_47 },
    { 0x8ef3bb59, SUC_lexicon_tags_7 },
    { 0xd1f3bb5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7713bb5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed3bb67, SUC_lexicon_tags_39 },
    { 0xf893bb68, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b73bb6b, SUC_lexicon_tags_23 },
    { 0x52d3bb6c, SUC_lexicon_tags_59 },
    { 0x8db3bb6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd3bb72, SUC_lexicon_tags_1 },
    { 0x1f53bb73, SUC_lexicon_tags_8 },
    { 0x8953bb74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa073bb76, SUC_lexicon_tags_27 },
    { 0x9a13bb77, SUC_lexicon_tags_37 },
    { 0x18d3bb77, SUC_lexicon_tags_12 },
    { 0xea93bb77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf853bb7f, SUC_lexicon_tags_47 },
    { 0x71b3bb80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe6d3bb82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f73bb8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd3bb8f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe913bb92, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9133bb94, SUC_lexicon_tags_17 },
    { 0x15f3bb95, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6953bba3, SUC_lexicon_tags_53 },
    { 0xa5f3bba3, SUC_lexicon_tags_14 },
    { 0x9593bba4, SUC_lexicon_tags_9 },
    { 0x52d3bba6, SUC_lexicon_tags_32 },
    { 0xa133bba7, SUC_lexicon_tags_15 },
    { 0x3b33bba8, SUC_lexicon_tags_26 },
    { 0x31f3bba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe973bbac, SUC_lexicon_tags_31 },
    { 0x1b73bbad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7b3bbaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb3bbb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd3bbba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf73bbbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb833bbbe, SUC_lexicon_tags_41 },
    { 0x3fd3bbbf, SUC_lexicon_tags_2 },
    { 0x7893bbbf, SUC_lexicon_tags_49 },
    { 0xb6f3bbbe, SUC_lexicon_tags_9 },
    { 0xe853bbc2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x49d3bbc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f3bbc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe913bbca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53bbce, SUC_lexicon_tags_17 },
    { 0x9813bbce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed3bbd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a53bbd6, SUC_lexicon_tags_49 },
    { 0xaf93bbd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4053bbda, SUC_lexicon_tags_45 },
    { 0x7b3bbda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5413bbe0, SUC_lexicon_tags_37 },
    { 0x2013bbe1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x5213bbe3, SUC_lexicon_tags_57 },
    { 0x43f3bbe4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb993bbe7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe053bbec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7833bbf0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a93bbf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd3bbf5, SUC_lexicon_tags_33 },
    { 0x2b53bbf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9993bbf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe33bbfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb3bbff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6dd3bc01, SUC_lexicon_tags_37 },
    { 0x98d3bc02, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8933bc05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed3bc08, SUC_lexicon_tags_17 },
    { 0x3693bc09, SUC_lexicon_tags_29 },
    { 0x9cf3bc0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedd3bc0c, SUC_lexicon_tags_17 },
    { 0x3f53bc0d, SUC_lexicon_tags_47 },
    { 0x9313bc0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6313bc11, SUC_lexicon_tags_11 },
    { 0x2073bc12, SUC_lexicon_tags_17 },
    { 0x97d3bc13, SUC_lexicon_tags_11 },
    { 0xd153bc11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33f3bc16, SUC_lexicon_tags_37 },
    { 0x79b3bc16, SUC_lexicon_tags_26 },
    { 0xf033bc17, SUC_lexicon_tags_35 },
    { 0x1393bc19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3753bc1c, SUC_lexicon_tags_25 },
    { 0x11d3bc1d, SUC_lexicon_tags_17 },
    { 0x4d53bc1c, SUC_lexicon_tags_17 },
    { 0x77b3bc1d, SUC_lexicon_tags_49 },
    { 0x27b3bc1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e73bc26, SUC_lexicon_tags_28 },
    { 0x8733bc26, SUC_lexicon_tags_68 },
    { 0xad33bc28, SUC_lexicon_tags_0 },
    { 0xc493bc29, SUC_lexicon_tags_39 },
    { 0xc0d3bc27, SUC_lexicon_tags_25 },
    { 0x1553bc2a, SUC_lexicon_tags_16 },
    { 0x9fb3bc2c, SUC_lexicon_tags_49 },
    { 0xffb3bc2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9d3bc2f, SUC_lexicon_tags_3 },
    { 0x7bf3bc30, SUC_lexicon_tags_11 },
    { 0x8c73bc2f, SUC_lexicon_tags_49 },
    { 0xafd3bc31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2513bc3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a73bc41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3453bc46, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d3bc4a, SUC_lexicon_tags_2 },
    { 0xdb13bc4b, SUC_lexicon_tags_79 },
    { 0x7113bc4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f13bc54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1553bc56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1133bc5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b73bc5f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af3bc63, SUC_lexicon_tags_45 },
    { 0xa1b3bc64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf693bc69, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa833bc6e, SUC_lexicon_tags_43 },
    { 0x6153bc6f, SUC_lexicon_tags_66 },
    { 0x3f93bc6f, SUC_lexicon_tags_41 },
    { 0x1093bc71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c73bc77, SUC_lexicon_tags_11 },
    { 0x33bc78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb453bc7c, SUC_lexicon_tags_17 },
    { 0xbff3bc7d, SUC_lexicon_tags_47 },
    { 0xfc93bc7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d3bc88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xea53bc8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7df3bc8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf73bc8f, SUC_lexicon_tags_15 },
    { 0xcd93bc90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3bc94, SUC_lexicon_tags_29 },
    { 0x9ad3bc95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a73bc9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6933bc9e, SUC_lexicon_tags_2 },
    { 0x9793bc9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d3bca3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f3bca6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa553bcaa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7973bcad, SUC_lexicon_tags_27 },
    { 0x493bcae, SUC_lexicon_tags_1 },
    { 0xe073bcae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeeb3bcb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x93bcb3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3bcb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3433bcbc, SUC_lexicon_tags_13 },
    { 0x7613bcbd, SUC_lexicon_tags_17 },
    { 0xcd13bcbe, SUC_lexicon_tags_2 },
    { 0xeb3bcbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff3bcc2, SUC_lexicon_tags_14 },
    { 0x50d3bcc2, SUC_lexicon_tags_23 },
    { 0x693bcc4, SUC_lexicon_tags_38 },
    { 0xa193bcc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c93bcca, SUC_lexicon_tags_1 },
    { 0xf073bcca, SUC_lexicon_tags_17 },
    { 0x4f33bccc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e13bcd1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6653bcd4, SUC_lexicon_tags_38 },
    { 0x5dd3bcd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb73bcd7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd3bcda, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d73bcdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6533bce0, SUC_lexicon_tags_37 },
    { 0x8313bce1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3bce5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd3bced, SUC_lexicon_tags_61 },
    { 0xbd33bcee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49b3bcf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf3bcf7, SUC_lexicon_tags_11 },
    { 0x9cb3bcf7, SUC_lexicon_tags_27 },
    { 0x76b3bcf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa713bcfb, SUC_lexicon_tags_11 },
    { 0x87f3bcfc, SUC_lexicon_tags_11 },
    { 0x2b93bcfd, SUC_lexicon_tags_17 },
    { 0x17b3bcfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f3bd00, SUC_lexicon_tags_15 },
    { 0x8933bd00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf93bd03, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xb9f3bd05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb33bd0a, SUC_lexicon_tags_19 },
    { 0xf893bd0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe93bd0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd413bd12, SUC_lexicon_tags_255 },
    { 0xfa13bd13, SUC_lexicon_tags_7 },
    { 0xdd33bd14, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xdaf3bd16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b53bd1a, SUC_lexicon_tags_37 },
    { 0xcb53bd1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe493bd20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1233bd25, SUC_lexicon_tags_45 },
    { 0x9293bd26, SUC_lexicon_tags_11 },
    { 0x233bd27, SUC_lexicon_tags_12 },
    { 0xf193bd27, SUC_lexicon_tags_26 },
    { 0x9f33bd29, SUC_lexicon_tags_37 },
    { 0xe973bd2a, SUC_lexicon_tags_65 },
    { 0x8d3bd29, SUC_lexicon_tags_15 },
    { 0x6353bd2c, SUC_lexicon_tags_38 },
    { 0x9013bd2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb3bd30, SUC_lexicon_tags_25 },
    { 0x4d53bd30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd873bd34, SUC_lexicon_tags_7 },
    { 0x8bf3bd34, SUC_lexicon_tags_77 },
    { 0xf33bd36, SUC_lexicon_tags_200 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6053bd3d, SUC_lexicon_tags_642 },
    { 0, NULL },
    { 0x7ab3bd3f, SUC_lexicon_tags_0 },
    { 0xfb3bd40, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff3bd48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcad3bd4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd3bd50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bd56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d3bd59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x813bd5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7173bd60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3bd64, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9933bd67, SUC_lexicon_tags_38 },
    { 0xbe53bd67, SUC_lexicon_tags_13 },
    { 0xeef3bd69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5713bd6c, SUC_lexicon_tags_15 },
    { 0x5ef3bd6c, SUC_lexicon_tags_3 },
    { 0xce53bd6e, SUC_lexicon_tags_4 },
    { 0x4133bd6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xedf3bd71, SUC_lexicon_tags_38 },
    { 0xe7d3bd72, SUC_lexicon_tags_37 },
    { 0xd233bd73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2cb3bd75, SUC_lexicon_tags_12 },
    { 0xd753bd75, SUC_lexicon_tags_15 },
    { 0x1013bd77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x953bd79, SUC_lexicon_tags_63 },
    { 0x2cb3bd7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f3bd7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a73bd81, SUC_lexicon_tags_17 },
    { 0xf053bd81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc073bd84, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d33bd8c, SUC_lexicon_tags_63 },
    { 0x6413bd8d, SUC_lexicon_tags_194 },
    { 0x7453bd8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc173bd90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6353bd92, SUC_lexicon_tags_2 },
    { 0xad73bd92, SUC_lexicon_tags_201 },
    { 0x4173bd92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4813bd96, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9973bd98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d3bd9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe793bda1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd7b3bda3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cb3bda5, SUC_lexicon_tags_4 },
    { 0xe973bda6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4973bdaa, SUC_lexicon_tags_29 },
    { 0xa333bdab, SUC_lexicon_tags_37 },
    { 0xfe33bdab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb053bdae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c33bdb3, SUC_lexicon_tags_11 },
    { 0x3153bdb4, SUC_lexicon_tags_11 },
    { 0x3893bdb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8073bdba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3693bdbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe253bdbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2873bdc3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa93bdc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f3bdc8, SUC_lexicon_tags_19 },
    { 0x933bdc8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdcf3bdcb, SUC_lexicon_tags_25 },
    { 0xf8f3bdcc, SUC_lexicon_tags_63 },
    { 0xa473bdcb, SUC_lexicon_tags_29 },
    { 0x9d73bdce, SUC_lexicon_tags_57 },
    { 0xf33bdcf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5873bdd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc213bdd4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x7053bdd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7133bdda, SUC_lexicon_tags_65 },
    { 0xf7b3bddb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d73bdde, SUC_lexicon_tags_26 },
    { 0x44b3bddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb093bde4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6d3bde6, SUC_lexicon_tags_1 },
    { 0x10b3bde7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8133bdea, SUC_lexicon_tags_7 },
    { 0x77f3bdea, SUC_lexicon_tags_33 },
    { 0xa93bdeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff3bdf1, SUC_lexicon_tags_2 },
    { 0x1453bdf1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d33bdf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19f3bdfb, SUC_lexicon_tags_25 },
    { 0xb373bdfb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x25b3bdfe, SUC_lexicon_tags_60 },
    { 0x3cd3bdff, SUC_lexicon_tags_62 },
    { 0xc5f3be00, SUC_lexicon_tags_14 },
    { 0x8473be01, SUC_lexicon_tags_23 },
    { 0x613be02, SUC_lexicon_tags_45 },
    { 0xd0b3be02, SUC_lexicon_tags_15 },
    { 0x1533be03, SUC_lexicon_tags_8 },
    { 0x2273bdff, SUC_lexicon_tags_47 },
    { 0xf053be06, SUC_lexicon_tags_29 },
    { 0x5c93be03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f3be0a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb913be0e, SUC_lexicon_tags_40 },
    { 0xfb73be0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b3be13, SUC_lexicon_tags_15 },
    { 0x1873be13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf3be17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2593be1d, SUC_lexicon_tags_7 },
    { 0x5113be1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe33be20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b3be23, SUC_lexicon_tags_37 },
    { 0xbc53be23, SUC_lexicon_tags_25 },
    { 0xcef3be25, SUC_lexicon_tags_17 },
    { 0x7cd3be26, SUC_lexicon_tags_19 },
    { 0x22f3be24, SUC_lexicon_tags_15 },
    { 0x3e33be24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc493be2c, SUC_lexicon_tags_7 },
    { 0x8df3be2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4b3be2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2f3be31, SUC_lexicon_tags_17 },
    { 0x16f3be32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf53be37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99d3be39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcb73be3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ed3be3d, SUC_lexicon_tags_63 },
    { 0x9433be3d, SUC_lexicon_tags_7 },
    { 0xac13be3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf573be41, SUC_lexicon_tags_12 },
    { 0x41d3be41, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b3be46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8913be49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6813be4f, SUC_lexicon_tags_5 },
    { 0xc433be50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe893be55, SUC_lexicon_tags_250 },
    { 0x26b3be55, SUC_lexicon_tags_32 },
    { 0x7cd3be57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5573be59, SUC_lexicon_tags_25 },
    { 0xc873be5a, SUC_lexicon_tags_15 },
    { 0x1973be5a, SUC_lexicon_tags_8 },
    { 0x5433be5c, SUC_lexicon_tags_23 },
    { 0xb6f3be5d, SUC_lexicon_tags_12 },
    { 0x10d3be5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc673be65, SUC_lexicon_tags_21 },
    { 0x5e73be66, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af3be6d, SUC_lexicon_tags_27 },
    { 0x2d3be6e, SUC_lexicon_tags_15 },
    { 0x9013be6e, SUC_lexicon_tags_3 },
    { 0xa0d3be6f, SUC_lexicon_tags_37 },
    { 0x1a33be71, SUC_lexicon_tags_46 },
    { 0x4333be72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb3be77, SUC_lexicon_tags_17 },
    { 0x8413be77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb0d3be7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb253be7d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb8f3be7f, SUC_lexicon_tags_25 },
    { 0x3d3be80, SUC_lexicon_tags_46 },
    { 0xbb93be80, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xdcf3be83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe813be85, SUC_lexicon_tags_9 },
    { 0x7e93be85, SUC_lexicon_tags_17 },
    { 0xaa93be87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb453be8b, SUC_lexicon_tags_49 },
    { 0x8db3be8c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3233be94, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d3be97, SUC_lexicon_tags_11 },
    { 0x7453be98, SUC_lexicon_tags_37 },
    { 0x47d3be98, SUC_lexicon_tags_15 },
    { 0xb1d3be9a, SUC_lexicon_tags_11 },
    { 0x90f3be9a, SUC_lexicon_tags_37 },
    { 0x9753be98, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1413be9f, SUC_lexicon_tags_25 },
    { 0x72d3bea0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af3bea3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a53bea8, SUC_lexicon_tags_1 },
    { 0xda73bea9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f53bead, SUC_lexicon_tags_53 },
    { 0xf8d3bead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa53beb2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf53beb6, SUC_lexicon_tags_15 },
    { 0x13d3beb7, SUC_lexicon_tags_1 },
    { 0xfbd3beb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6f3beba, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d13bebe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4053bec0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x193bec2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e33bec9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8b3becb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c53becd, SUC_lexicon_tags_14 },
    { 0x12d3becd, SUC_lexicon_tags_38 },
    { 0xc513becf, SUC_lexicon_tags_25 },
    { 0xfb93bed0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9073bed2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x533bed4, SUC_lexicon_tags_12 },
    { 0xa973bed5, SUC_lexicon_tags_28 },
    { 0x8293bed5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f3bee1, SUC_lexicon_tags_38 },
    { 0xaef3bee2, SUC_lexicon_tags_12 },
    { 0xd673bee2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71d3bee5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8493bee7, SUC_lexicon_tags_37 },
    { 0xaa13bee7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90d3beea, SUC_lexicon_tags_0 },
    { 0xe333beea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a33beee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd193bef0, SUC_lexicon_tags_32 },
    { 0x2bf3bef1, SUC_lexicon_tags_46 },
    { 0x4b73bef2, SUC_lexicon_tags_15 },
    { 0x7633bef3, SUC_lexicon_tags_8 },
    { 0x7473bef3, SUC_lexicon_tags_8 },
    { 0xd453bef2, SUC_lexicon_tags_37 },
    { 0x873bef6, SUC_lexicon_tags_26 },
    { 0x47b3bef7, SUC_lexicon_tags_2 },
    { 0xc4b3bef7, SUC_lexicon_tags_37 },
    { 0xebb3bef8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1733befc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf3beff, SUC_lexicon_tags_11 },
    { 0xdaf3bf00, SUC_lexicon_tags_27 },
    { 0xaaf3bf01, SUC_lexicon_tags_37 },
    { 0x4af3beff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4873bf05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb3bf08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3f3bf0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f73bf0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xff3bf0f, SUC_lexicon_tags_27 },
    { 0xf293bf0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b3bf13, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7433bf18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7b3bf1a, SUC_lexicon_tags_9 },
    { 0xb353bf1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b13bf1f, SUC_lexicon_tags_37 },
    { 0x23b3bf20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8593bf24, SUC_lexicon_tags_14 },
    { 0x353bf25, SUC_lexicon_tags_15 },
    { 0xd693bf25, SUC_lexicon_tags_0 },
    { 0xfbb3bf27, SUC_lexicon_tags_11 },
    { 0x65f3bf24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa673bf2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5df3bf2d, SUC_lexicon_tags_62 },
    { 0x1573bf2e, SUC_lexicon_tags_12 },
    { 0xa593bf2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec53bf31, SUC_lexicon_tags_15 },
    { 0x7d73bf31, SUC_lexicon_tags_38 },
    { 0x9cd3bf31, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b3bf39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad3bf3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef3bf3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13bf42, SUC_lexicon_tags_40 },
    { 0xf913bf43, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff73bf47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd3bf4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff93bf4e, SUC_lexicon_tags_8 },
    { 0x64f3bf4f, SUC_lexicon_tags_11 },
    { 0xe453bf50, SUC_lexicon_tags_19 },
    { 0x77d3bf51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b13bf54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad93bf5b, SUC_lexicon_tags_29 },
    { 0x17b3bf5c, SUC_lexicon_tags_0 },
    { 0x1c33bf5b, SUC_lexicon_tags_92 },
    { 0xf813bf5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc773bf66, SUC_lexicon_tags_48 },
    { 0x5fb3bf66, SUC_lexicon_tags_37 },
    { 0x7a93bf68, SUC_lexicon_tags_33 },
    { 0xa0f3bf69, SUC_lexicon_tags_7 },
    { 0x1ef3bf68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7053bf6f, SUC_lexicon_tags_23 },
    { 0x3433bf70, SUC_lexicon_tags_15 },
    { 0xf3d3bf71, SUC_lexicon_tags_25 },
    { 0x6953bf71, SUC_lexicon_tags_37 },
    { 0x3913bf73, SUC_lexicon_tags_87 },
    { 0x9733bf74, SUC_lexicon_tags_27 },
    { 0x8073bf74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c73bf78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xda33bf7a, SUC_lexicon_tags_20 },
    { 0x71d3bf7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb3bf7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75d3bf81, SUC_lexicon_tags_11 },
    { 0x8453bf82, SUC_lexicon_tags_37 },
    { 0x3f73bf82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf3bf8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd73bf8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf3bf97, SUC_lexicon_tags_2 },
    { 0x1493bf97, SUC_lexicon_tags_15 },
    { 0x6ed3bf99, SUC_lexicon_tags_7 },
    { 0x2ad3bf9a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x6333bf9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1793bf9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3173bfa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad3bfa6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e33bfa9, SUC_lexicon_tags_56 },
    { 0xf093bfaa, SUC_lexicon_tags_46 },
    { 0x1db3bfab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf3bfae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdf53bfb0, SUC_lexicon_tags_41 },
    { 0xf633bfb0, SUC_lexicon_tags_45 },
    { 0xc073bfb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7333bfba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f33bfc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a73bfcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1cf3bfce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe193bfd4, SUC_lexicon_tags_1 },
    { 0xdc33bfd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb3bfd8, SUC_lexicon_tags_63 },
    { 0x7eb3bfd9, SUC_lexicon_tags_30 },
    { 0x5d73bfd9, SUC_lexicon_tags_57 },
    { 0xbf93bfdb, SUC_lexicon_tags_17 },
    { 0xdb3bfdb, SUC_lexicon_tags_37 },
    { 0xbc13bfdd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xad33bfe0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa13bfe8, SUC_lexicon_tags_15 },
    { 0xc873bfe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5513bfee, SUC_lexicon_tags_15 },
    { 0x1ab3bfef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x893bff1, SUC_lexicon_tags_37 },
    { 0x6473bff2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8b3bff4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf6b3bff6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4093bff9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf013bffd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff53c001, SUC_lexicon_tags_11 },
    { 0x3373c002, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb3c006, SUC_lexicon_tags_8 },
    { 0x3bd3c006, SUC_lexicon_tags_17 },
    { 0x5e13c006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe873c00a, SUC_lexicon_tags_44 },
    { 0xf13c00a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed3c00e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6a13c010, SUC_lexicon_tags_12 },
    { 0x1c93c011, SUC_lexicon_tags_60 },
    { 0xcbd3c011, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc713c014, SUC_lexicon_tags_4 },
    { 0x56b3c015, SUC_lexicon_tags_7 },
    { 0x513c015, SUC_lexicon_tags_9 },
    { 0x8bd3c017, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1bb3c019, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d3c01e, SUC_lexicon_tags_49 },
    { 0xa6d3c01f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d13c021, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e73c028, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd53c02b, SUC_lexicon_tags_28 },
    { 0xfe53c02c, SUC_lexicon_tags_13 },
    { 0xa053c02d, SUC_lexicon_tags_0 },
    { 0x9a73c02b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3c031, SUC_lexicon_tags_36 },
    { 0xf573c031, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa13c035, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf453c037, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xea33c03a, SUC_lexicon_tags_19 },
    { 0x9413c03a, SUC_lexicon_tags_23 },
    { 0x573c03c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad93c040, SUC_lexicon_tags_45 },
    { 0x8213c041, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f3c045, SUC_lexicon_tags_49 },
    { 0x3733c045, SUC_lexicon_tags_12 },
    { 0x9e33c045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b3c04b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f3c052, SUC_lexicon_tags_48 },
    { 0x9293c052, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x253c055, SUC_lexicon_tags_9 },
    { 0xbe93c055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4133c058, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc993c05a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x633c05d, SUC_lexicon_tags_43 },
    { 0xd0f3c05e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf873c064, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f13c066, SUC_lexicon_tags_45 },
    { 0xcc93c066, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b3c06a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e73c06e, SUC_lexicon_tags_0 },
    { 0xd5d3c06e, SUC_lexicon_tags_15 },
    { 0xc993c06e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7b3c072, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9553c07c, SUC_lexicon_tags_43 },
    { 0x7bb3c07d, SUC_lexicon_tags_37 },
    { 0x6013c07c, SUC_lexicon_tags_7 },
    { 0x12b3c07f, SUC_lexicon_tags_25 },
    { 0xc853c080, SUC_lexicon_tags_17 },
    { 0xabd3c081, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5c53c083, SUC_lexicon_tags_53 },
    { 0x8333c083, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3513c088, SUC_lexicon_tags_87 },
    { 0x1093c088, SUC_lexicon_tags_45 },
    { 0x4ed3c088, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5493c08c, SUC_lexicon_tags_16 },
    { 0x9333c08d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed3c091, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3bd3c093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d3c097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b3c09a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb13c09e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa913c0a1, SUC_lexicon_tags_7 },
    { 0xec53c0a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb3c0a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5b3c0aa, SUC_lexicon_tags_47 },
    { 0x35f3c0ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4873c0b1, SUC_lexicon_tags_37 },
    { 0x4c93c0b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf033c0b4, SUC_lexicon_tags_37 },
    { 0xf853c0b5, SUC_lexicon_tags_25 },
    { 0xcb53c0b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1033c0b9, SUC_lexicon_tags_7 },
    { 0xbc93c0ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8053c0bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c13c0cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5373c0ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xba93c0d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd773c0d5, SUC_lexicon_tags_7 },
    { 0xa973c0d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9bd3c0d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f3c0db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2033c0de, SUC_lexicon_tags_29 },
    { 0x6533c0de, SUC_lexicon_tags_37 },
    { 0x1493c0df, SUC_lexicon_tags_47 },
    { 0x47d3c0e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b3c0e4, SUC_lexicon_tags_2 },
    { 0x3e93c0e4, SUC_lexicon_tags_47 },
    { 0xadb3c0e6, SUC_lexicon_tags_66 },
    { 0x30d3c0e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f53c0ea, SUC_lexicon_tags_107 },
    { 0xae73c0ea, SUC_lexicon_tags_43 },
    { 0xa3b3c0ec, SUC_lexicon_tags_11 },
    { 0x46f3c0ed, SUC_lexicon_tags_47 },
    { 0x4d73c0ee, SUC_lexicon_tags_0 },
    { 0x1fd3c0ed, SUC_lexicon_tags_45 },
    { 0x4fd3c0f0, SUC_lexicon_tags_17 },
    { 0x9553c0f1, SUC_lexicon_tags_37 },
    { 0xfdb3c0f2, SUC_lexicon_tags_12 },
    { 0xf993c0f3, SUC_lexicon_tags_37 },
    { 0xcd93c0f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed3c0f7, SUC_lexicon_tags_42 },
    { 0x3293c0f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7313c0fa, SUC_lexicon_tags_32 },
    { 0x9593c0fb, SUC_lexicon_tags_37 },
    { 0xf193c0fc, SUC_lexicon_tags_21 },
    { 0x6453c0fd, SUC_lexicon_tags_23 },
    { 0x9a33c0fb, SUC_lexicon_tags_17 },
    { 0x5a53c0ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaef3c101, SUC_lexicon_tags_48 },
    { 0x66d3c102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9f3c104, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2db3c106, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1513c109, SUC_lexicon_tags_37 },
    { 0x1e93c10a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c10e, SUC_lexicon_tags_21 },
    { 0x6cf3c10e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe453c114, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed3c11d, SUC_lexicon_tags_43 },
    { 0xa373c11e, SUC_lexicon_tags_16 },
    { 0x6533c11f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fb3c121, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d53c12b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4273c133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe233c135, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd13c13d, SUC_lexicon_tags_25 },
    { 0x5bb3c13e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b13c142, SUC_lexicon_tags_2 },
    { 0x7cf3c142, SUC_lexicon_tags_2 },
    { 0xab33c142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5893c149, SUC_lexicon_tags_32 },
    { 0x4953c14a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1d3c14c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8113c14e, SUC_lexicon_tags_23 },
    { 0x6ef3c14f, SUC_lexicon_tags_15 },
    { 0xf8f3c150, SUC_lexicon_tags_17 },
    { 0x9813c14f, SUC_lexicon_tags_11 },
    { 0xa0d3c14f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf833c158, SUC_lexicon_tags_38 },
    { 0xb2f3c158, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d33c15c, SUC_lexicon_tags_4 },
    { 0xce93c15c, SUC_lexicon_tags_39 },
    { 0x7493c15d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c13c165, SUC_lexicon_tags_2 },
    { 0xeb13c165, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4273c168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb233c16c, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0x5773c16e, SUC_lexicon_tags_23 },
    { 0x2013c16f, SUC_lexicon_tags_3 },
    { 0x2113c16e, SUC_lexicon_tags_48 },
    { 0x3f3c171, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8053c178, SUC_lexicon_tags_91 },
    { 0x31b3c178, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xf053c17c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd93c17e, SUC_lexicon_tags_13 },
    { 0x2153c17f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b53c182, SUC_lexicon_tags_25 },
    { 0xb333c182, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5793c185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab3c188, SUC_lexicon_tags_9 },
    { 0x5d73c189, SUC_lexicon_tags_33 },
    { 0x6fd3c18a, SUC_lexicon_tags_22 },
    { 0xd133c18b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91f3c18d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9033c192, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce13c198, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5673c19d, SUC_lexicon_tags_27 },
    { 0xd573c19e, SUC_lexicon_tags_2 },
    { 0x95f3c19f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2613c1a1, SUC_lexicon_tags_25 },
    { 0xba13c1a2, SUC_lexicon_tags_11 },
    { 0xb213c1a3, SUC_lexicon_tags_45 },
    { 0x9c93c1a4, SUC_lexicon_tags_45 },
    { 0x4cb3c1a5, SUC_lexicon_tags_17 },
    { 0xe973c1a6, SUC_lexicon_tags_15 },
    { 0xbd73c1a7, SUC_lexicon_tags_11 },
    { 0x13c1a7, SUC_lexicon_tags_37 },
    { 0xc8b3c1a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2313c1ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c33c1ad, SUC_lexicon_tags_29 },
    { 0x4d13c1ae, SUC_lexicon_tags_11 },
    { 0x6fd3c1ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8573c1b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f3c1b9, SUC_lexicon_tags_11 },
    { 0xced3c1ba, SUC_lexicon_tags_43 },
    { 0x4613c1ba, SUC_lexicon_tags_32 },
    { 0xb193c1bb, SUC_lexicon_tags_12 },
    { 0x713c1bd, SUC_lexicon_tags_19 },
    { 0x8d53c1bc, SUC_lexicon_tags_15 },
    { 0x2a93c1bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86b3c1c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3c1c4, SUC_lexicon_tags_12 },
    { 0xec13c1c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ad3c1c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd133c1ca, SUC_lexicon_tags_37 },
    { 0x6bd3c1ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f3c1ce, SUC_lexicon_tags_7 },
    { 0x4c33c1cf, SUC_lexicon_tags_28 },
    { 0x7993c1d0, SUC_lexicon_tags_29 },
    { 0x6b73c1ce, SUC_lexicon_tags_26 },
    { 0xe4b3c1d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc53c1d8, SUC_lexicon_tags_17 },
    { 0x12f3c1d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9493c1db, SUC_lexicon_tags_37 },
    { 0x18f3c1dc, SUC_lexicon_tags_19 },
    { 0xef53c1db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f3c1e8, SUC_lexicon_tags_43 },
    { 0x9b33c1e9, SUC_lexicon_tags_37 },
    { 0x79d3c1ea, SUC_lexicon_tags_107 },
    { 0x373c1e8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd93c1ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf3c1f3, SUC_lexicon_tags_63 },
    { 0x4af3c1f3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d3c1f8, SUC_lexicon_tags_27 },
    { 0x9d3c1f9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d3c1fc, SUC_lexicon_tags_26 },
    { 0xb3d3c1fd, SUC_lexicon_tags_551 },
    { 0xe4d3c1fe, SUC_lexicon_tags_1 },
    { 0x49b3c1ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd73c205, SUC_lexicon_tags_7 },
    { 0x9673c206, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8eb3c208, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe673c20b, SUC_lexicon_tags_27 },
    { 0xdd93c20c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33c212, SUC_lexicon_tags_1 },
    { 0x8a13c213, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693c21c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc953c221, SUC_lexicon_tags_12 },
    { 0x73b3c222, SUC_lexicon_tags_57 },
    { 0x9b53c222, SUC_lexicon_tags_11 },
    { 0x74d3c224, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3c226, SUC_lexicon_tags_15 },
    { 0x7573c226, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fd3c229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4213c22e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cb3c230, SUC_lexicon_tags_63 },
    { 0x2633c231, SUC_lexicon_tags_44 },
    { 0x5633c232, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x3913c234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c53c237, SUC_lexicon_tags_12 },
    { 0xa473c238, SUC_lexicon_tags_11 },
    { 0xed33c239, SUC_lexicon_tags_49 },
    { 0x3093c23a, SUC_lexicon_tags_21 },
    { 0xe33c237, SUC_lexicon_tags_25 },
    { 0x8633c23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab3c23f, SUC_lexicon_tags_11 },
    { 0x153c240, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5853c244, SUC_lexicon_tags_38 },
    { 0xd0f3c244, SUC_lexicon_tags_25 },
    { 0x3ff3c246, SUC_lexicon_tags_8 },
    { 0xe233c246, SUC_lexicon_tags_40 },
    { 0xd73c248, SUC_lexicon_tags_18 },
    { 0x92f3c249, SUC_lexicon_tags_207 },
    { 0x4113c248, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2853c24d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xaed3c24f, SUC_lexicon_tags_7 },
    { 0xa233c250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8693c25a, SUC_lexicon_tags_2 },
    { 0x4433c25a, SUC_lexicon_tags_27 },
    { 0x8013c25c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e33c262, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x3693c265, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xee33c267, SUC_lexicon_tags_1 },
    { 0xdf33c267, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6053c26a, SUC_lexicon_tags_61 },
    { 0xc673c26b, SUC_lexicon_tags_2 },
    { 0x4f13c26b, SUC_lexicon_tags_15 },
    { 0x5c13c26d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x313c270, SUC_lexicon_tags_66 },
    { 0x8353c270, SUC_lexicon_tags_27 },
    { 0x53c272, SUC_lexicon_tags_1 },
    { 0x68b3c273, SUC_lexicon_tags_21 },
    { 0x7393c271, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xde93c276, SUC_lexicon_tags_30 },
    { 0xe813c277, SUC_lexicon_tags_37 },
    { 0xa253c278, SUC_lexicon_tags_19 },
    { 0x14d3c279, SUC_lexicon_tags_146 },
    { 0x1633c279, SUC_lexicon_tags_13 },
    { 0x2cb3c279, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b3c27e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b13c280, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xda93c282, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c73c284, SUC_lexicon_tags_17 },
    { 0x25b3c284, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d3c289, SUC_lexicon_tags_12 },
    { 0x8df3c28a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b3c28d, SUC_lexicon_tags_63 },
    { 0x8673c28e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cb3c290, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5633c293, SUC_lexicon_tags_17 },
    { 0xb793c294, SUC_lexicon_tags_17 },
    { 0xc853c294, SUC_lexicon_tags_17 },
    { 0x7793c295, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa893c299, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f3c29c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a53c29f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b3c2a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6f3c2a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b3c2a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f3c2b1, SUC_lexicon_tags_27 },
    { 0x2cf3c2b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x24f3c2b4, SUC_lexicon_tags_0 },
    { 0x8d33c2b5, SUC_lexicon_tags_23 },
    { 0x69f3c2b6, SUC_lexicon_tags_37 },
    { 0x5073c2b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb3c2bf, SUC_lexicon_tags_17 },
    { 0x7133c2bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b3c2cb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xca33c2cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8d3c2cf, SUC_lexicon_tags_2 },
    { 0xaad3c2d0, SUC_lexicon_tags_222 },
    { 0, NULL },
    { 0xb693c2d2, SUC_lexicon_tags_27 },
    { 0xf533c2d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x813c2d6, SUC_lexicon_tags_25 },
    { 0xdf33c2d7, SUC_lexicon_tags_17 },
    { 0x95b3c2d8, SUC_lexicon_tags_15 },
    { 0x2233c2d6, SUC_lexicon_tags_1 },
    { 0x92b3c2da, SUC_lexicon_tags_46 },
    { 0x53f3c2d6, SUC_lexicon_tags_9 },
    { 0x3bd3c2dc, SUC_lexicon_tags_48 },
    { 0x7f13c2dd, SUC_lexicon_tags_11 },
    { 0xbe33c2d7, SUC_lexicon_tags_47 },
    { 0x1993c2df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d3c2e3, SUC_lexicon_tags_12 },
    { 0x7cd3c2e3, SUC_lexicon_tags_25 },
    { 0x2c53c2e5, SUC_lexicon_tags_84 },
    { 0xc693c2e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea73c2f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d3c2f3, SUC_lexicon_tags_43 },
    { 0xe5f3c2f3, SUC_lexicon_tags_37 },
    { 0x6f3c2f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36f3c2f7, SUC_lexicon_tags_11 },
    { 0x31f3c2f7, SUC_lexicon_tags_31 },
    { 0x5533c2f8, SUC_lexicon_tags_17 },
    { 0xe53c2f9, SUC_lexicon_tags_45 },
    { 0x67f3c2fb, SUC_lexicon_tags_38 },
    { 0x29d3c2fc, SUC_lexicon_tags_45 },
    { 0x8453c2fc, SUC_lexicon_tags_25 },
    { 0x9b3c2fb, SUC_lexicon_tags_9 },
    { 0x8f73c2ff, SUC_lexicon_tags_47 },
    { 0x9ab3c2fb, SUC_lexicon_tags_26 },
    { 0xcb33c301, SUC_lexicon_tags_1 },
    { 0xf0d3c301, SUC_lexicon_tags_9 },
    { 0xf153c2f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b3c30b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae53c310, SUC_lexicon_tags_75 },
    { 0xfe73c311, SUC_lexicon_tags_1 },
    { 0xe053c312, SUC_lexicon_tags_28 },
    { 0x85b3c311, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f3c316, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7513c318, SUC_lexicon_tags_15 },
    { 0x3033c318, SUC_lexicon_tags_4 },
    { 0x7773c319, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcb73c31c, SUC_lexicon_tags_29 },
    { 0x87b3c31d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d3c322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b13c327, SUC_lexicon_tags_0 },
    { 0x3953c327, SUC_lexicon_tags_17 },
    { 0xa413c329, SUC_lexicon_tags_7 },
    { 0x7d93c327, SUC_lexicon_tags_17 },
    { 0x5fd3c32a, SUC_lexicon_tags_23 },
    { 0xbf33c32b, SUC_lexicon_tags_45 },
    { 0xa873c32a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa493c331, SUC_lexicon_tags_11 },
    { 0xb913c332, SUC_lexicon_tags_1 },
    { 0x4b73c331, SUC_lexicon_tags_1 },
    { 0x1093c334, SUC_lexicon_tags_37 },
    { 0x9cb3c335, SUC_lexicon_tags_33 },
    { 0x1273c333, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5173c33a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe8d3c33c, SUC_lexicon_tags_77 },
    { 0x11d3c33c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b13c346, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc893c34a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef3c34f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f3c359, SUC_lexicon_tags_30 },
    { 0x9f93c35a, SUC_lexicon_tags_47 },
    { 0x98d3c35b, SUC_lexicon_tags_23 },
    { 0x7ff3c35b, SUC_lexicon_tags_7 },
    { 0x9573c35a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb3c360, SUC_lexicon_tags_25 },
    { 0xa533c360, SUC_lexicon_tags_9 },
    { 0xf593c362, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb633c364, SUC_lexicon_tags_37 },
    { 0x9153c365, SUC_lexicon_tags_8 },
    { 0xdbb3c365, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5353c36a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9873c36c, SUC_lexicon_tags_17 },
    { 0x3ff3c36d, SUC_lexicon_tags_19 },
    { 0x6333c36e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa173c371, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf333c374, SUC_lexicon_tags_17 },
    { 0x673c375, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd13c379, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5033c37b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b33c37f, SUC_lexicon_tags_107 },
    { 0xabb3c380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4353c385, SUC_lexicon_tags_118 },
    { 0x6b73c386, SUC_lexicon_tags_17 },
    { 0x6253c387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb3c38d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2253c38f, SUC_lexicon_tags_63 },
    { 0xb53c38f, SUC_lexicon_tags_11 },
    { 0x5393c390, SUC_lexicon_tags_7 },
    { 0xa073c392, SUC_lexicon_tags_8 },
    { 0x5cb3c392, SUC_lexicon_tags_38 },
    { 0x6ab3c394, SUC_lexicon_tags_1 },
    { 0xc4f3c394, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba53c397, SUC_lexicon_tags_6 },
    { 0x44b3c398, SUC_lexicon_tags_569 },
    { 0x2293c398, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6493c39d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b3c3a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32b3c3a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f3c3a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9253c3ac, SUC_lexicon_tags_37 },
    { 0x8c53c3ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f3c3b0, SUC_lexicon_tags_29 },
    { 0xf0b3c3b1, SUC_lexicon_tags_30 },
    { 0xfd3c3b2, SUC_lexicon_tags_26 },
    { 0x71b3c3b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf773c3b6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x13d3c3b8, SUC_lexicon_tags_1 },
    { 0xa733c3b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2973c3bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1293c3c5, SUC_lexicon_tags_38 },
    { 0x52d3c3c6, SUC_lexicon_tags_17 },
    { 0x6233c3c7, SUC_lexicon_tags_65 },
    { 0x153c3c8, SUC_lexicon_tags_60 },
    { 0x1373c3c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e53c3cc, SUC_lexicon_tags_7 },
    { 0xf733c3cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb3c3d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb3c3d7, SUC_lexicon_tags_1 },
    { 0xc8d3c3d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5d3c3da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5f3c3dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f3c3e4, SUC_lexicon_tags_15 },
    { 0x7113c3e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db3c3e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3b33c3ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f3c3ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f3c3f2, SUC_lexicon_tags_39 },
    { 0x8013c3f2, SUC_lexicon_tags_49 },
    { 0x78f3c3f2, SUC_lexicon_tags_282 },
    { 0xeb13c3f5, SUC_lexicon_tags_87 },
    { 0x8ef3c3f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd3c3fb, SUC_lexicon_tags_23 },
    { 0x373c3fb, SUC_lexicon_tags_23 },
    { 0x43f3c3fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e93c407, SUC_lexicon_tags_12 },
    { 0xcbd3c407, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6c73c40a, SUC_lexicon_tags_2 },
    { 0xd233c40a, SUC_lexicon_tags_4 },
    { 0x36d3c40a, SUC_lexicon_tags_1 },
    { 0xc33c40c, SUC_lexicon_tags_12 },
    { 0xf0f3c40e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa953c414, SUC_lexicon_tags_43 },
    { 0xc053c415, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d3c41c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea33c423, SUC_lexicon_tags_1 },
    { 0xb473c424, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd173c427, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf1d3c429, SUC_lexicon_tags_17 },
    { 0x88d3c42a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc553c436, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca33c443, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa533c447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5d3c449, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2333c44b, SUC_lexicon_tags_49 },
    { 0xac13c44b, SUC_lexicon_tags_113 },
    { 0x95b3c44b, SUC_lexicon_tags_1 },
    { 0x32d3c44e, SUC_lexicon_tags_25 },
    { 0xc313c44f, SUC_lexicon_tags_47 },
    { 0xd3d3c44e, SUC_lexicon_tags_14 },
    { 0xc1f3c44d, SUC_lexicon_tags_37 },
    { 0xf273c452, SUC_lexicon_tags_17 },
    { 0xf9d3c451, SUC_lexicon_tags_29 },
    { 0x373c454, SUC_lexicon_tags_28 },
    { 0x1d53c455, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb3c459, SUC_lexicon_tags_17 },
    { 0x3e73c45a, SUC_lexicon_tags_12 },
    { 0xdcd3c45a, SUC_lexicon_tags_106 },
    { 0x6113c45b, SUC_lexicon_tags_11 },
    { 0xe7f3c45a, SUC_lexicon_tags_0 },
    { 0x2b53c45e, SUC_lexicon_tags_14 },
    { 0x1053c45f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc93c464, SUC_lexicon_tags_17 },
    { 0x9053c465, SUC_lexicon_tags_60 },
    { 0xe673c464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b53c468, SUC_lexicon_tags_25 },
    { 0x2153c469, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6153c46e, SUC_lexicon_tags_15 },
    { 0xe0f3c46f, SUC_lexicon_tags_37 },
    { 0x1433c470, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b3c476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b3c47b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb3c47e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x7a33c480, SUC_lexicon_tags_23 },
    { 0xc173c481, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1333c484, SUC_lexicon_tags_126 },
    { 0xe753c485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d3c48d, SUC_lexicon_tags_33 },
    { 0x54f3c48d, SUC_lexicon_tags_45 },
    { 0x2f33c48f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2f3c491, SUC_lexicon_tags_10 },
    { 0xf53c492, SUC_lexicon_tags_27 },
    { 0xf153c493, SUC_lexicon_tags_82 },
    { 0xa693c491, SUC_lexicon_tags_46 },
    { 0x8b53c493, SUC_lexicon_tags_28 },
    { 0xbc73c492, SUC_lexicon_tags_37 },
    { 0xc033c497, SUC_lexicon_tags_7 },
    { 0xd393c498, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff3c49b, SUC_lexicon_tags_27 },
    { 0xced3c49c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f3c4a0, SUC_lexicon_tags_7 },
    { 0xf0d3c4a0, SUC_lexicon_tags_11 },
    { 0xf093c4a2, SUC_lexicon_tags_0 },
    { 0x5113c4a2, SUC_lexicon_tags_11 },
    { 0x8e93c4a1, SUC_lexicon_tags_23 },
    { 0xe733c4a5, SUC_lexicon_tags_28 },
    { 0xc5b3c4a3, SUC_lexicon_tags_47 },
    { 0x9633c4a7, SUC_lexicon_tags_11 },
    { 0x4fb3c4a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2433c4aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3c4ae, SUC_lexicon_tags_11 },
    { 0xa673c4af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4193c4b1, SUC_lexicon_tags_37 },
    { 0xa593c4b1, SUC_lexicon_tags_63 },
    { 0xed73c4b3, SUC_lexicon_tags_25 },
    { 0x5133c4b4, SUC_lexicon_tags_11 },
    { 0x12f3c4b4, SUC_lexicon_tags_1 },
    { 0x3d3c4b6, SUC_lexicon_tags_31 },
    { 0xac93c4b6, SUC_lexicon_tags_12 },
    { 0x3e73c4b8, SUC_lexicon_tags_25 },
    { 0xf9f3c4b6, SUC_lexicon_tags_33 },
    { 0x9153c4b6, SUC_lexicon_tags_21 },
    { 0x38f3c4b3, SUC_lexicon_tags_37 },
    { 0x4b93c4b3, SUC_lexicon_tags_514 },
    { 0xfa73c4bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f93c4c2, SUC_lexicon_tags_60 },
    { 0xb1f3c4c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf833c4c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c33c4c8, SUC_lexicon_tags_28 },
    { 0x5f3c4c9, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xaf73c4cb, SUC_lexicon_tags_17 },
    { 0xdd73c4cb, SUC_lexicon_tags_45 },
    { 0x48b3c4cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e53c4d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c13c4d4, SUC_lexicon_tags_31 },
    { 0x8993c4d5, SUC_lexicon_tags_5 },
    { 0x96b3c4d6, SUC_lexicon_tags_13 },
    { 0xbf13c4d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d53c4d9, SUC_lexicon_tags_31 },
    { 0x5313c4da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa713c4dc, SUC_lexicon_tags_11 },
    { 0x413c4dd, SUC_lexicon_tags_23 },
    { 0xd233c4de, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x7993c4e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8853c4e6, SUC_lexicon_tags_15 },
    { 0x1f33c4e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c33c4eb, SUC_lexicon_tags_37 },
    { 0xfa53c4ec, SUC_lexicon_tags_63 },
    { 0x9cf3c4eb, SUC_lexicon_tags_12 },
    { 0xdb3c4ee, SUC_lexicon_tags_15 },
    { 0xdd53c4ed, SUC_lexicon_tags_14 },
    { 0xeff3c4f0, SUC_lexicon_tags_29 },
    { 0x43b3c4f0, SUC_lexicon_tags_33 },
    { 0x98b3c4f2, SUC_lexicon_tags_31 },
    { 0x5a93c4f0, SUC_lexicon_tags_29 },
    { 0xa33c4f4, SUC_lexicon_tags_23 },
    { 0x2673c4ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb753c4f8, SUC_lexicon_tags_37 },
    { 0x3dd3c4f8, SUC_lexicon_tags_30 },
    { 0x5573c4fa, SUC_lexicon_tags_12 },
    { 0xf073c4fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b53c4fd, SUC_lexicon_tags_15 },
    { 0x9cd3c4fe, SUC_lexicon_tags_66 },
    { 0x5b53c4fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4453c504, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b3c507, SUC_lexicon_tags_60 },
    { 0x7973c508, SUC_lexicon_tags_12 },
    { 0x5613c508, SUC_lexicon_tags_47 },
    { 0xc8d3c509, SUC_lexicon_tags_37 },
    { 0x9473c50b, SUC_lexicon_tags_12 },
    { 0xb9b3c50c, SUC_lexicon_tags_83 },
    { 0x6d33c50d, SUC_lexicon_tags_14 },
    { 0xa9d3c50e, SUC_lexicon_tags_45 },
    { 0x9d3c50f, SUC_lexicon_tags_97 },
    { 0x5333c50d, SUC_lexicon_tags_31 },
    { 0x9053c511, SUC_lexicon_tags_38 },
    { 0xbc13c508, SUC_lexicon_tags_6 },
    { 0x6893c513, SUC_lexicon_tags_12 },
    { 0xcaf3c514, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x253c517, SUC_lexicon_tags_14 },
    { 0x16b3c518, SUC_lexicon_tags_37 },
    { 0x1c13c518, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf3c51b, SUC_lexicon_tags_4 },
    { 0x8f13c51b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab93c520, SUC_lexicon_tags_28 },
    { 0x6bf3c521, SUC_lexicon_tags_37 },
    { 0x5f93c522, SUC_lexicon_tags_65 },
    { 0x6a13c520, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1153c526, SUC_lexicon_tags_37 },
    { 0x5af3c527, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb3c52c, SUC_lexicon_tags_16 },
    { 0x8c53c52d, SUC_lexicon_tags_46 },
    { 0x7093c52e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x37d3c530, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d3c537, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x15b3c539, SUC_lexicon_tags_17 },
    { 0xd873c539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7f3c53c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2153c540, SUC_lexicon_tags_27 },
    { 0xf873c540, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd3c544, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51d3c546, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x993c548, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeaf3c54a, SUC_lexicon_tags_9 },
    { 0xd9f3c54b, SUC_lexicon_tags_12 },
    { 0x8d33c54b, SUC_lexicon_tags_4 },
    { 0x3a33c54b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d33c550, SUC_lexicon_tags_11 },
    { 0xb033c551, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e13c554, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf193c556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69b3c558, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a73c55c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3c564, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x72d3c566, SUC_lexicon_tags_17 },
    { 0xf413c566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd3c56a, SUC_lexicon_tags_7 },
    { 0x2953c56b, SUC_lexicon_tags_30 },
    { 0x7953c56a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f73c56f, SUC_lexicon_tags_9 },
    { 0x9453c570, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf513c572, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf673c575, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa273c577, SUC_lexicon_tags_522 },
    { 0x3ed3c578, SUC_lexicon_tags_7 },
    { 0xcf73c579, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7d3c57b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x36d3c57d, SUC_lexicon_tags_17 },
    { 0x6313c57e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6673c580, SUC_lexicon_tags_15 },
    { 0xe3d3c581, SUC_lexicon_tags_11 },
    { 0xea73c582, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8eb3c584, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7733c586, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4253c58a, SUC_lexicon_tags_15 },
    { 0xc9b3c58b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b3c593, SUC_lexicon_tags_37 },
    { 0xe313c594, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef3c59b, SUC_lexicon_tags_12 },
    { 0x6053c59b, SUC_lexicon_tags_37 },
    { 0xad33c59d, SUC_lexicon_tags_27 },
    { 0xd873c59e, SUC_lexicon_tags_7 },
    { 0x3bb3c59f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4453c5a1, SUC_lexicon_tags_1 },
    { 0xd373c5a2, SUC_lexicon_tags_37 },
    { 0x9573c5a3, SUC_lexicon_tags_11 },
    { 0x77f3c5a3, SUC_lexicon_tags_60 },
    { 0x7653c5a5, SUC_lexicon_tags_11 },
    { 0xff73c5a6, SUC_lexicon_tags_17 },
    { 0x99b3c5a1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f3c5aa, SUC_lexicon_tags_1 },
    { 0xb453c5ab, SUC_lexicon_tags_2 },
    { 0x5cd3c5ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd3c5af, SUC_lexicon_tags_10 },
    { 0x3033c5af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af3c5b4, SUC_lexicon_tags_7 },
    { 0xb213c5b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd73c5b8, SUC_lexicon_tags_43 },
    { 0xdf3c5b8, SUC_lexicon_tags_28 },
    { 0x3773c5b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2133c5be, SUC_lexicon_tags_38 },
    { 0xa453c5bf, SUC_lexicon_tags_0 },
    { 0x87b3c5c0, SUC_lexicon_tags_7 },
    { 0x7cd3c5c1, SUC_lexicon_tags_11 },
    { 0xf273c5c2, SUC_lexicon_tags_17 },
    { 0x9d53c5c3, SUC_lexicon_tags_32 },
    { 0xbf13c5c3, SUC_lexicon_tags_47 },
    { 0xaed3c5c4, SUC_lexicon_tags_17 },
    { 0x95b3c5c6, SUC_lexicon_tags_7 },
    { 0x4433c5c7, SUC_lexicon_tags_4 },
    { 0x31f3c5c6, SUC_lexicon_tags_23 },
    { 0xb1b3c5c6, SUC_lexicon_tags_1 },
    { 0x9013c5ca, SUC_lexicon_tags_7 },
    { 0x9fb3c5cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6593c5cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce93c5d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5233c5d4, SUC_lexicon_tags_65 },
    { 0x7b73c5d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6d3c5d7, SUC_lexicon_tags_3 },
    { 0x6c73c5d8, SUC_lexicon_tags_0 },
    { 0xb673c5d8, SUC_lexicon_tags_1 },
    { 0x9c73c5d9, SUC_lexicon_tags_56 },
    { 0xe253c5db, SUC_lexicon_tags_49 },
    { 0x31d3c5dc, SUC_lexicon_tags_63 },
    { 0x9553c5db, SUC_lexicon_tags_15 },
    { 0xa073c5db, SUC_lexicon_tags_2 },
    { 0xd573c5df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3c5e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefd3c5e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b3c5e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb3c5ec, SUC_lexicon_tags_11 },
    { 0xf33c5ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3e93c5ef, SUC_lexicon_tags_17 },
    { 0x9433c5f0, SUC_lexicon_tags_25 },
    { 0xe193c5f1, SUC_lexicon_tags_6 },
    { 0xd3c5f2, SUC_lexicon_tags_45 },
    { 0x7573c5f2, SUC_lexicon_tags_12 },
    { 0x4a53c5f2, SUC_lexicon_tags_2 },
    { 0x4533c5f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f93c5f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1613c5fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3ed3c5ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8653c601, SUC_lexicon_tags_61 },
    { 0x32d3c602, SUC_lexicon_tags_11 },
    { 0x5313c603, SUC_lexicon_tags_17 },
    { 0xf633c602, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9773c60a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6413c60d, SUC_lexicon_tags_55 },
    { 0x653c60e, SUC_lexicon_tags_37 },
    { 0xd453c60f, SUC_lexicon_tags_65 },
    { 0x65f3c60e, SUC_lexicon_tags_38 },
    { 0xbe13c611, SUC_lexicon_tags_2 },
    { 0xc8b3c611, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f3c615, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe13c61a, SUC_lexicon_tags_6 },
    { 0x15b3c61b, SUC_lexicon_tags_11 },
    { 0xc6f3c61a, SUC_lexicon_tags_1 },
    { 0x1eb3c61d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb3c622, SUC_lexicon_tags_2 },
    { 0x48d3c622, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1d3c625, SUC_lexicon_tags_25 },
    { 0x7cd3c626, SUC_lexicon_tags_26 },
    { 0xb633c627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d3c62a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7b3c62c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a33c631, SUC_lexicon_tags_136 },
    { 0x7ab3c632, SUC_lexicon_tags_14 },
    { 0x5f73c632, SUC_lexicon_tags_12 },
    { 0x38b3c633, SUC_lexicon_tags_75 },
    { 0x31b3c632, SUC_lexicon_tags_37 },
    { 0x36b3c636, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9473c638, SUC_lexicon_tags_25 },
    { 0xa53c638, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb653c642, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb633c646, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f3c64c, SUC_lexicon_tags_12 },
    { 0xd0b3c64d, SUC_lexicon_tags_33 },
    { 0xa073c64d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f3c651, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7693c654, SUC_lexicon_tags_12 },
    { 0x58d3c655, SUC_lexicon_tags_66 },
    { 0xf53c656, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xba13c658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b3c662, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad73c666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f93c669, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf33c66f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c13c672, SUC_lexicon_tags_15 },
    { 0x9093c673, SUC_lexicon_tags_28 },
    { 0x54d3c672, SUC_lexicon_tags_32 },
    { 0xb8f3c675, SUC_lexicon_tags_63 },
    { 0x6533c676, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd73c678, SUC_lexicon_tags_33 },
    { 0xbd53c679, SUC_lexicon_tags_37 },
    { 0x6173c679, SUC_lexicon_tags_15 },
    { 0x8fd3c679, SUC_lexicon_tags_1 },
    { 0x4613c678, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xf673c67e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2f3c680, SUC_lexicon_tags_27 },
    { 0xc1f3c681, SUC_lexicon_tags_15 },
    { 0xdb13c682, SUC_lexicon_tags_1 },
    { 0x5173c682, SUC_lexicon_tags_1212 },
    { 0x793c680, SUC_lexicon_tags_47 },
    { 0xd293c685, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe093c689, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb33c68d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3173c68f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe153c694, SUC_lexicon_tags_30 },
    { 0xa133c695, SUC_lexicon_tags_31 },
    { 0x8153c695, SUC_lexicon_tags_25 },
    { 0xff3c697, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdf3c699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b3c6a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f33c6a4, SUC_lexicon_tags_187 },
    { 0x16f3c6a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0f3c6a7, SUC_lexicon_tags_30 },
    { 0x4453c6a8, SUC_lexicon_tags_25 },
    { 0x5833c6a9, SUC_lexicon_tags_136 },
    { 0x5973c6a9, SUC_lexicon_tags_1 },
    { 0x8713c6ab, SUC_lexicon_tags_82 },
    { 0xef13c6ac, SUC_lexicon_tags_7 },
    { 0x2f53c6ad, SUC_lexicon_tags_77 },
    { 0xf6d3c6a8, SUC_lexicon_tags_6 },
    { 0x3553c6af, SUC_lexicon_tags_714 },
    { 0xfef3c6af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40d3c6b2, SUC_lexicon_tags_37 },
    { 0x5773c6b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9933c6b8, SUC_lexicon_tags_37 },
    { 0x5dd3c6b9, SUC_lexicon_tags_0 },
    { 0x15f3c6b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ef3c6bc, SUC_lexicon_tags_12 },
    { 0x4e53c6bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad3c6c4, SUC_lexicon_tags_9 },
    { 0xb353c6c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d3c6cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e93c6d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f3c6d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee33c6db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad3c6de, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa053c6e0, SUC_lexicon_tags_1 },
    { 0x813c6e0, SUC_lexicon_tags_25 },
    { 0xa8b3c6e0, SUC_lexicon_tags_26 },
    { 0xd593c6e3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b3c6e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf413c6ea, SUC_lexicon_tags_60 },
    { 0x5cf3c6eb, SUC_lexicon_tags_136 },
    { 0xddd3c6ec, SUC_lexicon_tags_11 },
    { 0x9a33c6ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f3c6f4, SUC_lexicon_tags_8 },
    { 0xb473c6f5, SUC_lexicon_tags_43 },
    { 0x3453c6f4, SUC_lexicon_tags_38 },
    { 0xc7b3c6f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13c6fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5473c6ff, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf433c703, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5153c708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f53c70a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92f3c70c, SUC_lexicon_tags_1 },
    { 0x4573c70c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f3c710, SUC_lexicon_tags_30 },
    { 0xba93c711, SUC_lexicon_tags_1 },
    { 0xa53c712, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4173c716, SUC_lexicon_tags_37 },
    { 0x70d3c717, SUC_lexicon_tags_23 },
    { 0xcf3c718, SUC_lexicon_tags_25 },
    { 0x46d3c719, SUC_lexicon_tags_47 },
    { 0xb13c717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f73c71d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe093c720, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2173c723, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd3c725, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8613c727, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe613c72a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7413c731, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb3c738, SUC_lexicon_tags_2 },
    { 0x43b3c739, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf13c73c, SUC_lexicon_tags_2 },
    { 0x9553c73d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef93c741, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f53c744, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b3c74a, SUC_lexicon_tags_7 },
    { 0x233c74a, SUC_lexicon_tags_17 },
    { 0x23d3c74c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b3c74f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7493c751, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x753c754, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3c758, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x5733c75a, SUC_lexicon_tags_49 },
    { 0xef53c75b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb933c75d, SUC_lexicon_tags_113 },
    { 0x8d93c75d, SUC_lexicon_tags_15 },
    { 0x7453c75f, SUC_lexicon_tags_31 },
    { 0x2ef3c75e, SUC_lexicon_tags_1 },
    { 0xf2b3c761, SUC_lexicon_tags_37 },
    { 0x2193c762, SUC_lexicon_tags_43 },
    { 0x5533c763, SUC_lexicon_tags_28 },
    { 0xf313c764, SUC_lexicon_tags_37 },
    { 0x1d53c765, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8133c76c, SUC_lexicon_tags_37 },
    { 0xeed3c76d, SUC_lexicon_tags_2 },
    { 0xaf73c76e, SUC_lexicon_tags_66 },
    { 0x8193c76f, SUC_lexicon_tags_12 },
    { 0xd0d3c76f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf3c777, SUC_lexicon_tags_112 },
    { 0xb133c778, SUC_lexicon_tags_59 },
    { 0xa1d3c778, SUC_lexicon_tags_47 },
    { 0xc813c778, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd3c77e, SUC_lexicon_tags_46 },
    { 0xdab3c77f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f33c781, SUC_lexicon_tags_129 },
    { 0x1d53c781, SUC_lexicon_tags_4 },
    { 0xa013c781, SUC_lexicon_tags_25 },
    { 0xd353c784, SUC_lexicon_tags_15 },
    { 0x75b3c785, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd3c788, SUC_lexicon_tags_37 },
    { 0x4393c788, SUC_lexicon_tags_31 },
    { 0x27f3c78a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b3c792, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc173c798, SUC_lexicon_tags_41 },
    { 0x2c93c799, SUC_lexicon_tags_12 },
    { 0x4913c79a, SUC_lexicon_tags_7 },
    { 0x5813c79b, SUC_lexicon_tags_37 },
    { 0x9173c79b, SUC_lexicon_tags_23 },
    { 0xbc93c79d, SUC_lexicon_tags_29 },
    { 0xdd33c79e, SUC_lexicon_tags_33 },
    { 0xe093c79f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3953c7a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d13c7a9, SUC_lexicon_tags_37 },
    { 0x8233c7aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4f13c7ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f3c7af, SUC_lexicon_tags_29 },
    { 0xc9b3c7b0, SUC_lexicon_tags_37 },
    { 0x4033c7b0, SUC_lexicon_tags_38 },
    { 0xd833c7b2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33c7b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b3c7b9, SUC_lexicon_tags_6 },
    { 0x4013c7b9, SUC_lexicon_tags_63 },
    { 0x6233c7bb, SUC_lexicon_tags_23 },
    { 0x79f3c7bc, SUC_lexicon_tags_25 },
    { 0x4513c7ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd993c7bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27b3c7c1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x61b3c7c3, SUC_lexicon_tags_25 },
    { 0xfb33c7c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f3c7ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7e73c7cc, SUC_lexicon_tags_12 },
    { 0x69f3c7cd, SUC_lexicon_tags_45 },
    { 0xb413c7ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93c7d1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb853c7d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8913c7da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5733c7dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf553c7e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9293c7e4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3ad3c7e6, SUC_lexicon_tags_17 },
    { 0x12f3c7e7, SUC_lexicon_tags_12 },
    { 0x5cd3c7e7, SUC_lexicon_tags_0 },
    { 0xb4b3c7e9, SUC_lexicon_tags_25 },
    { 0x2b13c7e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf3c7ed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x473c7ef, SUC_lexicon_tags_0 },
    { 0x8553c7ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf553c7f2, SUC_lexicon_tags_3 },
    { 0xd7d3c7f3, SUC_lexicon_tags_57 },
    { 0x4b53c7f4, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0xde73c7f7, SUC_lexicon_tags_47 },
    { 0x2033c7f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96f3c7fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3933c7fe, SUC_lexicon_tags_47 },
    { 0x5173c7ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d3c804, SUC_lexicon_tags_27 },
    { 0x1f53c805, SUC_lexicon_tags_7 },
    { 0x28f3c806, SUC_lexicon_tags_49 },
    { 0x6bb3c807, SUC_lexicon_tags_25 },
    { 0x9533c808, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f3c80f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3833c811, SUC_lexicon_tags_12 },
    { 0x9353c812, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f3c818, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd3c81c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b3c826, SUC_lexicon_tags_60 },
    { 0x2f13c826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7653c829, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6113c82b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d73c830, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3c833, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b53c835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bb3c837, SUC_lexicon_tags_2 },
    { 0x58d3c838, SUC_lexicon_tags_25 },
    { 0xb693c838, SUC_lexicon_tags_25 },
    { 0x2d73c83a, SUC_lexicon_tags_7 },
    { 0x2e33c83a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4393c83e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9613c844, SUC_lexicon_tags_405 },
    { 0, NULL },
    { 0xe333c846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f93c84a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4633c84c, SUC_lexicon_tags_46 },
    { 0xa953c84d, SUC_lexicon_tags_16 },
    { 0x36d3c84e, SUC_lexicon_tags_25 },
    { 0x6233c84f, SUC_lexicon_tags_9 },
    { 0x4a53c850, SUC_lexicon_tags_21 },
    { 0xcc93c84f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b13c854, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf13c858, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3433c85e, SUC_lexicon_tags_17 },
    { 0x4033c85f, SUC_lexicon_tags_49 },
    { 0xdf93c85f, SUC_lexicon_tags_43 },
    { 0xfbf3c861, SUC_lexicon_tags_1 },
    { 0x77b3c862, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8933c864, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db3c869, SUC_lexicon_tags_17 },
    { 0xdcf3c86a, SUC_lexicon_tags_1 },
    { 0xc53c86a, SUC_lexicon_tags_91 },
    { 0x6073c86c, SUC_lexicon_tags_23 },
    { 0x5eb3c86c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa613c86f, SUC_lexicon_tags_23 },
    { 0x68b3c870, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5413c872, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef3c87a, SUC_lexicon_tags_15 },
    { 0x4673c87a, SUC_lexicon_tags_17 },
    { 0xb513c87c, SUC_lexicon_tags_12 },
    { 0x7013c87d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2573c885, SUC_lexicon_tags_37 },
    { 0x4413c885, SUC_lexicon_tags_15 },
    { 0x91b3c887, SUC_lexicon_tags_9 },
    { 0x64f3c886, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2cd3c88a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd53c88e, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b53c896, SUC_lexicon_tags_7 },
    { 0xe013c897, SUC_lexicon_tags_68 },
    { 0x5233c898, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb013c89a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b3c89f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7713c8a1, SUC_lexicon_tags_23 },
    { 0x3ed3c8a1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8193c8a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd713c8aa, SUC_lexicon_tags_8 },
    { 0x1953c8aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3853c8ad, SUC_lexicon_tags_0 },
    { 0x1e53c8ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x30b3c8b0, SUC_lexicon_tags_22 },
    { 0x9133c8b1, SUC_lexicon_tags_38 },
    { 0x5f33c8b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d3c8b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef3c8b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7333c8be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe573c8c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdd3c8c2, SUC_lexicon_tags_30 },
    { 0xfad3c8c2, SUC_lexicon_tags_37 },
    { 0x85d3c8c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8273c8c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f3c8ca, SUC_lexicon_tags_17 },
    { 0xdbb3c8cb, SUC_lexicon_tags_7 },
    { 0xb073c8ca, SUC_lexicon_tags_47 },
    { 0x4af3c8ca, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3c8d0, SUC_lexicon_tags_19 },
    { 0xe653c8d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd3c8d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa93c8d6, SUC_lexicon_tags_21 },
    { 0xfb33c8d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9373c8e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f3c8e4, SUC_lexicon_tags_19 },
    { 0x7f93c8e4, SUC_lexicon_tags_17 },
    { 0xb273c8e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf673c8e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9b3c8ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6153c8ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2593c8f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa213c8f3, SUC_lexicon_tags_46 },
    { 0x5433c8f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf713c8f7, SUC_lexicon_tags_1 },
    { 0x7cb3c8f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd093c8fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6073c8fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d53c903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f53c905, SUC_lexicon_tags_38 },
    { 0xb273c906, SUC_lexicon_tags_98 },
    { 0x63b3c906, SUC_lexicon_tags_25 },
    { 0xe793c908, SUC_lexicon_tags_14 },
    { 0x3c53c908, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef3c90c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36f3c90e, SUC_lexicon_tags_31 },
    { 0x6e33c90f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3c914, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53c917, SUC_lexicon_tags_77 },
    { 0x99b3c917, SUC_lexicon_tags_12 },
    { 0xc1b3c917, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd033c91c, SUC_lexicon_tags_28 },
    { 0xb53c91d, SUC_lexicon_tags_16 },
    { 0xa653c91e, SUC_lexicon_tags_77 },
    { 0xd4b3c91f, SUC_lexicon_tags_19 },
    { 0x3273c91f, SUC_lexicon_tags_2 },
    { 0x6a93c921, SUC_lexicon_tags_27 },
    { 0xf733c922, SUC_lexicon_tags_27 },
    { 0x34b3c923, SUC_lexicon_tags_17 },
    { 0xfd53c923, SUC_lexicon_tags_6 },
    { 0x673c925, SUC_lexicon_tags_25 },
    { 0x6113c921, SUC_lexicon_tags_23 },
    { 0x5833c927, SUC_lexicon_tags_7 },
    { 0x3d93c925, SUC_lexicon_tags_1 },
    { 0xfff3c921, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b3c92c, SUC_lexicon_tags_10 },
    { 0x4bb3c92d, SUC_lexicon_tags_14 },
    { 0x1953c92e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe473c930, SUC_lexicon_tags_27 },
    { 0xfcb3c931, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x62f3c933, SUC_lexicon_tags_32 },
    { 0x18d3c933, SUC_lexicon_tags_12 },
    { 0x73f3c935, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee53c938, SUC_lexicon_tags_23 },
    { 0x9f13c939, SUC_lexicon_tags_31 },
    { 0x6b13c93a, SUC_lexicon_tags_11 },
    { 0x1ff3c93a, SUC_lexicon_tags_12 },
    { 0xc493c93c, SUC_lexicon_tags_49 },
    { 0xe493c93b, SUC_lexicon_tags_14 },
    { 0x1693c939, SUC_lexicon_tags_7 },
    { 0x1493c93a, SUC_lexicon_tags_11 },
    { 0xa7d3c940, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe633c943, SUC_lexicon_tags_0 },
    { 0x3a93c944, SUC_lexicon_tags_1 },
    { 0x81f3c944, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb633c94a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8233c94c, SUC_lexicon_tags_23 },
    { 0xb313c94d, SUC_lexicon_tags_11 },
    { 0x613c94e, SUC_lexicon_tags_127 },
    { 0xf93c94d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a73c951, SUC_lexicon_tags_29 },
    { 0xd93c952, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8f3c954, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc53c957, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33c95c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8673c961, SUC_lexicon_tags_49 },
    { 0x20f3c962, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6013c967, SUC_lexicon_tags_43 },
    { 0x5e53c967, SUC_lexicon_tags_60 },
    { 0xefb3c968, SUC_lexicon_tags_48 },
    { 0x6813c96a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbad3c96c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db3c96f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xeaf3c971, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b3c97c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa233c981, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc33c987, SUC_lexicon_tags_16 },
    { 0xfd3c987, SUC_lexicon_tags_27 },
    { 0x993c989, SUC_lexicon_tags_8 },
    { 0xd313c98a, SUC_lexicon_tags_4 },
    { 0x4d3c987, SUC_lexicon_tags_25 },
    { 0x1453c98a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d3c991, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13c99a, SUC_lexicon_tags_7 },
    { 0xdb33c99b, SUC_lexicon_tags_46 },
    { 0x2f13c99c, SUC_lexicon_tags_33 },
    { 0x4513c99d, SUC_lexicon_tags_65 },
    { 0xcd13c99e, SUC_lexicon_tags_21 },
    { 0x9873c99e, SUC_lexicon_tags_0 },
    { 0x6773c99d, SUC_lexicon_tags_49 },
    { 0xb3d3c9a1, SUC_lexicon_tags_7 },
    { 0x55b3c9a2, SUC_lexicon_tags_282 },
    { 0x7ed3c9a3, SUC_lexicon_tags_17 },
    { 0xa1b3c9a2, SUC_lexicon_tags_25 },
    { 0x19f3c99c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2f93c9a7, SUC_lexicon_tags_27 },
    { 0x6b93c9a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f3c9ae, SUC_lexicon_tags_10 },
    { 0xdc33c9af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2df3c9b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb93c9b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb673c9ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5793c9bd, SUC_lexicon_tags_49 },
    { 0x2dd3c9be, SUC_lexicon_tags_74 },
    { 0x60d3c9bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f73c9c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d33c9c6, SUC_lexicon_tags_107 },
    { 0x1753c9c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c73c9cc, SUC_lexicon_tags_3 },
    { 0xd1f3c9cd, SUC_lexicon_tags_28 },
    { 0x5313c9ce, SUC_lexicon_tags_0 },
    { 0x1453c9cf, SUC_lexicon_tags_47 },
    { 0xd193c9cf, SUC_lexicon_tags_6 },
    { 0x75f3c9d1, SUC_lexicon_tags_17 },
    { 0x30b3c9d2, SUC_lexicon_tags_37 },
    { 0xa9d3c9d0, SUC_lexicon_tags_47 },
    { 0x6cd3c9cd, SUC_lexicon_tags_1 },
    { 0xf0d3c9cd, SUC_lexicon_tags_11 },
    { 0xcfb3c9d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb053c9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3c9de, SUC_lexicon_tags_7 },
    { 0x2473c9df, SUC_lexicon_tags_0 },
    { 0x2e93c9e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef3c9e3, SUC_lexicon_tags_65 },
    { 0x2c93c9e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93c9e7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f3c9ea, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f3c9ef, SUC_lexicon_tags_11 },
    { 0xd333c9ef, SUC_lexicon_tags_37 },
    { 0x9f33c9ef, SUC_lexicon_tags_30 },
    { 0xe13c9f0, SUC_lexicon_tags_27 },
    { 0x13c9f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe913c9f5, SUC_lexicon_tags_37 },
    { 0x3813c9f5, SUC_lexicon_tags_43 },
    { 0x18f3c9f7, SUC_lexicon_tags_7 },
    { 0x8b3c9f8, SUC_lexicon_tags_60 },
    { 0xa233c9f9, SUC_lexicon_tags_46 },
    { 0xee13c9f5, SUC_lexicon_tags_30 },
    { 0x1273c9fb, SUC_lexicon_tags_12 },
    { 0xd953c9fc, SUC_lexicon_tags_82 },
    { 0x8dd3c9fd, SUC_lexicon_tags_25 },
    { 0x39d3c9fe, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb3ca0b, SUC_lexicon_tags_23 },
    { 0x5db3ca0b, SUC_lexicon_tags_12 },
    { 0x893ca0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f3ca10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6953ca12, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed3ca1a, SUC_lexicon_tags_15 },
    { 0xe6f3ca1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8993ca1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa273ca20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb553ca22, SUC_lexicon_tags_45 },
    { 0xb4f3ca23, SUC_lexicon_tags_16 },
    { 0x8e93ca22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3253ca28, SUC_lexicon_tags_12 },
    { 0x7933ca29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x593ca2c, SUC_lexicon_tags_2 },
    { 0x25d3ca2c, SUC_lexicon_tags_11 },
    { 0x9b53ca2c, SUC_lexicon_tags_2 },
    { 0xeb93ca2f, SUC_lexicon_tags_7 },
    { 0xbdd3ca30, SUC_lexicon_tags_0 },
    { 0x64b3ca30, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33ca35, SUC_lexicon_tags_38 },
    { 0xd9d3ca36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9273ca3a, SUC_lexicon_tags_37 },
    { 0x6a33ca3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc233ca3d, SUC_lexicon_tags_8 },
    { 0x4673ca3e, SUC_lexicon_tags_28 },
    { 0x5213ca3d, SUC_lexicon_tags_25 },
    { 0x90f3ca40, SUC_lexicon_tags_15 },
    { 0x153ca40, SUC_lexicon_tags_85 },
    { 0xa293ca41, SUC_lexicon_tags_37 },
    { 0x2f53ca43, SUC_lexicon_tags_32 },
    { 0xc5b3ca42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7773ca4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a73ca4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1053ca50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bd3ca52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9013ca54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8f3ca56, SUC_lexicon_tags_17 },
    { 0xbd3ca56, SUC_lexicon_tags_35 },
    { 0xfab3ca57, SUC_lexicon_tags_12 },
    { 0xb5f3ca59, SUC_lexicon_tags_33 },
    { 0x4d33ca58, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b73ca5f, SUC_lexicon_tags_45 },
    { 0x5393ca60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd3ca63, SUC_lexicon_tags_68 },
    { 0x3a13ca63, SUC_lexicon_tags_16 },
    { 0x8373ca63, SUC_lexicon_tags_15 },
    { 0xcc13ca66, SUC_lexicon_tags_11 },
    { 0x2b53ca67, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6373ca6a, SUC_lexicon_tags_47 },
    { 0xcef3ca6a, SUC_lexicon_tags_7 },
    { 0xb1d3ca6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x13d3ca6e, SUC_lexicon_tags_42 },
    { 0xc7d3ca6f, SUC_lexicon_tags_32 },
    { 0xf3b3ca70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b3ca7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d3ca7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fd3ca7f, SUC_lexicon_tags_17 },
    { 0xba73ca7f, SUC_lexicon_tags_33 },
    { 0x9273ca81, SUC_lexicon_tags_17 },
    { 0x22d3ca81, SUC_lexicon_tags_6 },
    { 0x3193ca83, SUC_lexicon_tags_15 },
    { 0x7a33ca84, SUC_lexicon_tags_47 },
    { 0xe0b3ca85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d3ca88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2af3ca8a, SUC_lexicon_tags_352 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf3ca9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d3ca9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43f3caa1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f3caa4, SUC_lexicon_tags_65 },
    { 0x8d13caa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2393caa8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8193cab1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb93cab6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb3cabc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3cabf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5713cac8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2713caca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b3cacf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd3cad2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb413cad7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6573cada, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b93cae3, SUC_lexicon_tags_15 },
    { 0xe0d3cae4, SUC_lexicon_tags_1 },
    { 0xceb3cae4, SUC_lexicon_tags_11 },
    { 0xb133cae5, SUC_lexicon_tags_77 },
    { 0xce13cae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6133cae9, SUC_lexicon_tags_77 },
    { 0xec33caea, SUC_lexicon_tags_33 },
    { 0x59b3caea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f3caf0, SUC_lexicon_tags_27 },
    { 0x4673caf1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb73caf6, SUC_lexicon_tags_38 },
    { 0xd3d3caf7, SUC_lexicon_tags_9 },
    { 0xc093caf8, SUC_lexicon_tags_7 },
    { 0x94d3caf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d3cafc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c93caff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd3cb01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3813cb06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0b3cb08, SUC_lexicon_tags_27 },
    { 0xeb93cb09, SUC_lexicon_tags_15 },
    { 0x3213cb08, SUC_lexicon_tags_2 },
    { 0x82d3cb0b, SUC_lexicon_tags_60 },
    { 0x8933cb0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef3cb11, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2db3cb13, SUC_lexicon_tags_16 },
    { 0xbc53cb13, SUC_lexicon_tags_44 },
    { 0x4773cb15, SUC_lexicon_tags_25 },
    { 0x1473cb16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc673cb1b, SUC_lexicon_tags_31 },
    { 0x2053cb1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a73cb1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc13cb20, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd193cb22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc53cb24, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b3cb2c, SUC_lexicon_tags_7 },
    { 0x52b3cb2c, SUC_lexicon_tags_38 },
    { 0x91b3cb2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3413cb38, SUC_lexicon_tags_17 },
    { 0xe13cb39, SUC_lexicon_tags_2 },
    { 0xbc53cb38, SUC_lexicon_tags_45 },
    { 0x2e33cb39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b3cb42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f3cb46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb3cb49, SUC_lexicon_tags_8 },
    { 0x6773cb49, SUC_lexicon_tags_15 },
    { 0xe093cb4b, SUC_lexicon_tags_17 },
    { 0x10b3cb4b, SUC_lexicon_tags_65 },
    { 0x9d13cb49, SUC_lexicon_tags_38 },
    { 0xafb3cb4e, SUC_lexicon_tags_37 },
    { 0x9573cb4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf733cb55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb833cb57, SUC_lexicon_tags_38 },
    { 0x4a33cb58, SUC_lexicon_tags_17 },
    { 0x95b3cb59, SUC_lexicon_tags_11 },
    { 0x7d53cb5a, SUC_lexicon_tags_27 },
    { 0x6f3cb5a, SUC_lexicon_tags_40 },
    { 0x2b53cb59, SUC_lexicon_tags_24 },
    { 0x97f3cb57, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6493cb61, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf8b3cb63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1113cb66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf133cb68, SUC_lexicon_tags_17 },
    { 0xa413cb69, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db3cb76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b3cb79, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8633cb7b, SUC_lexicon_tags_7 },
    { 0xf713cb7b, SUC_lexicon_tags_17 },
    { 0x8473cb7b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7e13cb7f, SUC_lexicon_tags_92 },
    { 0x9393cb80, SUC_lexicon_tags_55 },
    { 0x1173cb80, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd613cb84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7033cb88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d13cb8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xccf3cb8f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xef33cb91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5593cb93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa273cb97, SUC_lexicon_tags_19 },
    { 0xef13cb98, SUC_lexicon_tags_29 },
    { 0xeb93cb99, SUC_lexicon_tags_17 },
    { 0x17b3cb98, SUC_lexicon_tags_15 },
    { 0x4d73cb9a, SUC_lexicon_tags_12 },
    { 0x38b3cb9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x64f3cb9e, SUC_lexicon_tags_14 },
    { 0x52d3cb9e, SUC_lexicon_tags_37 },
    { 0x2d93cba0, SUC_lexicon_tags_17 },
    { 0x2433cba1, SUC_lexicon_tags_60 },
    { 0x6413cba2, SUC_lexicon_tags_23 },
    { 0x67d3cba3, SUC_lexicon_tags_4 },
    { 0x5473cba4, SUC_lexicon_tags_17 },
    { 0x7db3cba5, SUC_lexicon_tags_1 },
    { 0x9233cba2, SUC_lexicon_tags_14 },
    { 0x8cb3cba7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b53cbae, SUC_lexicon_tags_2 },
    { 0x57b3cbaf, SUC_lexicon_tags_17 },
    { 0x78f3cbae, SUC_lexicon_tags_29 },
    { 0xd793cbb0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb3cbb5, SUC_lexicon_tags_70 },
    { 0x7353cbb6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c93cbbd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x98b3cbbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c73cbc6, SUC_lexicon_tags_3 },
    { 0x13b3cbc7, SUC_lexicon_tags_2 },
    { 0x2ff3cbc7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f3cbcd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df3cbd0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3cbd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed3cbdd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb3cbe0, SUC_lexicon_tags_27 },
    { 0x52f3cbe1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5b3cbe3, SUC_lexicon_tags_28 },
    { 0x4a33cbe4, SUC_lexicon_tags_37 },
    { 0xfb93cbe4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9af3cbe7, SUC_lexicon_tags_7 },
    { 0x2d93cbe7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8573cbee, SUC_lexicon_tags_12 },
    { 0xa233cbef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73cbf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513cbf8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f3cbfc, SUC_lexicon_tags_25 },
    { 0x6d13cbfd, SUC_lexicon_tags_35 },
    { 0xcbd3cbfc, SUC_lexicon_tags_17 },
    { 0x4053cbff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc573cc01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef3cc04, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b53cc09, SUC_lexicon_tags_25 },
    { 0x5273cc0a, SUC_lexicon_tags_0 },
    { 0x2853cc0b, SUC_lexicon_tags_7 },
    { 0xa7f3cc0a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x61d3cc0e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xef3cc10, SUC_lexicon_tags_7 },
    { 0x94b3cc10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d3cc14, SUC_lexicon_tags_60 },
    { 0xd273cc15, SUC_lexicon_tags_7 },
    { 0x15f3cc14, SUC_lexicon_tags_7 },
    { 0xd813cc17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf3cc1f, SUC_lexicon_tags_37 },
    { 0xfa93cc20, SUC_lexicon_tags_17 },
    { 0x98b3cc21, SUC_lexicon_tags_3 },
    { 0x4973cc22, SUC_lexicon_tags_2 },
    { 0x1473cc22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d93cc26, SUC_lexicon_tags_9 },
    { 0x9133cc27, SUC_lexicon_tags_63 },
    { 0xf8b3cc27, SUC_lexicon_tags_10 },
    { 0x4d93cc27, SUC_lexicon_tags_15 },
    { 0xfc53cc2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6753cc30, SUC_lexicon_tags_1 },
    { 0xf413cc30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe533cc33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73cc37, SUC_lexicon_tags_12 },
    { 0x9e33cc37, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3cc3c, SUC_lexicon_tags_12 },
    { 0xf293cc3d, SUC_lexicon_tags_29 },
    { 0xfa53cc3c, SUC_lexicon_tags_68 },
    { 0x34b3cc3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb313cc41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd33cc43, SUC_lexicon_tags_44 },
    { 0x10d3cc44, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8833cc46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd893cc4b, SUC_lexicon_tags_2 },
    { 0xfe73cc4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x653cc4e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f3cc53, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc93cc58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3153cc5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8693cc61, SUC_lexicon_tags_31 },
    { 0x55b3cc62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd113cc64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83b3cc66, SUC_lexicon_tags_4 },
    { 0xc9b3cc66, SUC_lexicon_tags_56 },
    { 0x8893cc68, SUC_lexicon_tags_22 },
    { 0x1c93cc69, SUC_lexicon_tags_37 },
    { 0x7df3cc67, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x14b3cc6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf3cc70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa453cc72, SUC_lexicon_tags_38 },
    { 0xae73cc73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f13cc78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b3cc7e, SUC_lexicon_tags_25 },
    { 0x9fb3cc7e, SUC_lexicon_tags_11 },
    { 0x56d3cc80, SUC_lexicon_tags_33 },
    { 0x5873cc81, SUC_lexicon_tags_17 },
    { 0xcad3cc7f, SUC_lexicon_tags_0 },
    { 0xd73cc83, SUC_lexicon_tags_29 },
    { 0xc573cc83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b93cc88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb3cc8b, SUC_lexicon_tags_0 },
    { 0x2233cc8c, SUC_lexicon_tags_80 },
    { 0xd7d3cc8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd3cc98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2b3cc9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e53cc9e, SUC_lexicon_tags_14 },
    { 0xa573cc9e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb73cca7, SUC_lexicon_tags_27 },
    { 0x4153cca8, SUC_lexicon_tags_48 },
    { 0x4c73cca9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc153ccab, SUC_lexicon_tags_0 },
    { 0x8d53ccac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb73ccb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9233ccb4, SUC_lexicon_tags_19 },
    { 0x3bd3ccb4, SUC_lexicon_tags_7 },
    { 0xc953ccb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b3ccbf, SUC_lexicon_tags_31 },
    { 0x8a53ccc0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe6f3ccc2, SUC_lexicon_tags_23 },
    { 0xaf73ccc3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3633ccc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab33ccc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f3cccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e13ccce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5993ccd0, SUC_lexicon_tags_26 },
    { 0xabd3ccd1, SUC_lexicon_tags_0 },
    { 0x7bf3ccd1, SUC_lexicon_tags_19 },
    { 0x1013ccd1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5113ccd9, SUC_lexicon_tags_12 },
    { 0x4cb3ccd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3493cce0, SUC_lexicon_tags_12 },
    { 0xa6b3cce1, SUC_lexicon_tags_68 },
    { 0x8cf3cce2, SUC_lexicon_tags_12 },
    { 0xeb33cce2, SUC_lexicon_tags_17 },
    { 0xe453cce2, SUC_lexicon_tags_38 },
    { 0xd493cce2, SUC_lexicon_tags_15 },
    { 0xd8f3cce6, SUC_lexicon_tags_9 },
    { 0x5bd3cce4, SUC_lexicon_tags_16 },
    { 0x9b3cce8, SUC_lexicon_tags_25 },
    { 0xb493cce8, SUC_lexicon_tags_47 },
    { 0xc0f3cce0, SUC_lexicon_tags_33 },
    { 0x2993cce5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4653cced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d93ccef, SUC_lexicon_tags_23 },
    { 0x71f3ccef, SUC_lexicon_tags_4 },
    { 0xd2d3ccf1, SUC_lexicon_tags_12 },
    { 0x1433ccf2, SUC_lexicon_tags_19 },
    { 0xf493ccf3, SUC_lexicon_tags_15 },
    { 0x3553ccf3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1e73ccf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9593ccf9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd953ccfc, SUC_lexicon_tags_31 },
    { 0x18b3ccfd, SUC_lexicon_tags_9 },
    { 0x3d13ccfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf73cd01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d3cd04, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe33cd06, SUC_lexicon_tags_8 },
    { 0x7b73cd07, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc893cd0b, SUC_lexicon_tags_17 },
    { 0x9bd3cd0b, SUC_lexicon_tags_46 },
    { 0x3393cd0d, SUC_lexicon_tags_12 },
    { 0xdfd3cd0e, SUC_lexicon_tags_30 },
    { 0xdad3cd0f, SUC_lexicon_tags_11 },
    { 0xa6d3cd10, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe473cd15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3193cd1a, SUC_lexicon_tags_12 },
    { 0x9033cd1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa433cd1e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x8eb3cd20, SUC_lexicon_tags_37 },
    { 0x1b53cd21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc113cd23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb93cd26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca93cd28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5bb3cd2a, SUC_lexicon_tags_4 },
    { 0xbef3cd2a, SUC_lexicon_tags_82 },
    { 0xf4b3cd2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa13cd2e, SUC_lexicon_tags_6 },
    { 0xd9f3cd2e, SUC_lexicon_tags_39 },
    { 0xcd73cd2f, SUC_lexicon_tags_13 },
    { 0xd1f3cd31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd3cd3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa753cd44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f73cd4a, SUC_lexicon_tags_7 },
    { 0x9733cd4a, SUC_lexicon_tags_11 },
    { 0xca93cd4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x43b3cd4e, SUC_lexicon_tags_69 },
    { 0xf353cd4e, SUC_lexicon_tags_17 },
    { 0x4b53cd50, SUC_lexicon_tags_1 },
    { 0xa5d3cd4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a53cd57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9173cd59, SUC_lexicon_tags_2 },
    { 0xc813cd59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccd3cd5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f3cd5f, SUC_lexicon_tags_37 },
    { 0x60f3cd5f, SUC_lexicon_tags_25 },
    { 0x7093cd61, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b3cd65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa213cd6b, SUC_lexicon_tags_11 },
    { 0x1d33cd6c, SUC_lexicon_tags_17 },
    { 0xba93cd6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d3cd70, SUC_lexicon_tags_4 },
    { 0x9e53cd71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa113cd78, SUC_lexicon_tags_61 },
    { 0xa0d3cd78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ef3cd7b, SUC_lexicon_tags_11 },
    { 0x9b53cd7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6013cd7e, SUC_lexicon_tags_35 },
    { 0x2ff3cd7f, SUC_lexicon_tags_33 },
    { 0x8bd3cd80, SUC_lexicon_tags_38 },
    { 0x7f13cd81, SUC_lexicon_tags_29 },
    { 0x2d93cd82, SUC_lexicon_tags_37 },
    { 0x913cd83, SUC_lexicon_tags_37 },
    { 0x6013cd83, SUC_lexicon_tags_47 },
    { 0x9b73cd85, SUC_lexicon_tags_15 },
    { 0x533cd84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d3cd8a, SUC_lexicon_tags_44 },
    { 0x38f3cd8b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7353cd92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6953cd94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ad3cd96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bf3cd98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf233cd9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d3cda3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8913cda6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f3cda8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3af3cdab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf3cdae, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc93cdb4, SUC_lexicon_tags_40 },
    { 0xcff3cdb5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa3b3cdb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5753cdb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b3cdbc, SUC_lexicon_tags_29 },
    { 0xaeb3cdbd, SUC_lexicon_tags_9 },
    { 0x7b3cdbd, SUC_lexicon_tags_77 },
    { 0xba13cdbf, SUC_lexicon_tags_14 },
    { 0x8033cdbd, SUC_lexicon_tags_23 },
    { 0x1773cdc0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5d13cdc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3013cdc6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9cf3cdc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d3cdcb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4313cdcd, SUC_lexicon_tags_2 },
    { 0x5d3cdcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6d3cdd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a73cdd5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc013cdd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa273cdde, SUC_lexicon_tags_17 },
    { 0x4693cddf, SUC_lexicon_tags_15 },
    { 0x16f3cde0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a53cde7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd3cdec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd3cdf1, SUC_lexicon_tags_15 },
    { 0x79d3cdf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe13cdf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf253cdfd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec53ce04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8173ce0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16d3ce0c, SUC_lexicon_tags_1 },
    { 0x97b3ce0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99d3ce0f, SUC_lexicon_tags_60 },
    { 0x2713ce0f, SUC_lexicon_tags_27 },
    { 0x2733ce11, SUC_lexicon_tags_85 },
    { 0xc4f3ce0f, SUC_lexicon_tags_19 },
    { 0x8f3ce13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5133ce15, SUC_lexicon_tags_79 },
    { 0xa853ce16, SUC_lexicon_tags_37 },
    { 0xb173ce17, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0xba53ce19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa873ce1c, SUC_lexicon_tags_14 },
    { 0x4c33ce1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b3ce20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3973ce23, SUC_lexicon_tags_2 },
    { 0x84f3ce24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46f3ce26, SUC_lexicon_tags_23 },
    { 0xef33ce26, SUC_lexicon_tags_12 },
    { 0x9b33ce28, SUC_lexicon_tags_12 },
    { 0xf713ce29, SUC_lexicon_tags_11 },
    { 0xd933ce26, SUC_lexicon_tags_27 },
    { 0xc253ce2b, SUC_lexicon_tags_0 },
    { 0x99d3ce2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4d3ce2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc693ce34, SUC_lexicon_tags_174 },
    { 0xb893ce35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2573ce39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf33ce3d, SUC_lexicon_tags_170 },
    { 0x9fb3ce3e, SUC_lexicon_tags_15 },
    { 0xfbb3ce3f, SUC_lexicon_tags_7 },
    { 0x9773ce40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c13ce42, SUC_lexicon_tags_11 },
    { 0xab13ce43, SUC_lexicon_tags_1 },
    { 0xcb3ce44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab33ce46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8733ce48, SUC_lexicon_tags_7 },
    { 0x57d3ce49, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2753ce51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b3ce54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff3ce5d, SUC_lexicon_tags_29 },
    { 0xdc53ce5d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b3ce61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d3ce65, SUC_lexicon_tags_5 },
    { 0x58b3ce65, SUC_lexicon_tags_47 },
    { 0xafb3ce67, SUC_lexicon_tags_23 },
    { 0xbad3ce65, SUC_lexicon_tags_0 },
    { 0x9033ce69, SUC_lexicon_tags_27 },
    { 0xe5b3ce69, SUC_lexicon_tags_1 },
    { 0x8053ce65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13ce6e, SUC_lexicon_tags_28 },
    { 0x2f73ce6f, SUC_lexicon_tags_25 },
    { 0x99f3ce70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf973ce73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8b73ce75, SUC_lexicon_tags_12 },
    { 0xd9d3ce76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8413ce7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f3ce7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7893ce8c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7513ce94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3ce97, SUC_lexicon_tags_33 },
    { 0x2553ce97, SUC_lexicon_tags_79 },
    { 0x6413ce99, SUC_lexicon_tags_38 },
    { 0xbab3ce9a, SUC_lexicon_tags_15 },
    { 0x9393ce9b, SUC_lexicon_tags_55 },
    { 0xbd33ce9a, SUC_lexicon_tags_25 },
    { 0x4313ce9d, SUC_lexicon_tags_17 },
    { 0x65f3ce9e, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f53cea4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba33cea6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3673ceaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe253cead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34d3ceaf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d33ceb3, SUC_lexicon_tags_31 },
    { 0x5c93ceb4, SUC_lexicon_tags_37 },
    { 0x6733ceb5, SUC_lexicon_tags_57 },
    { 0xaeb3ceb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa13ceb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cec2, SUC_lexicon_tags_30 },
    { 0x6713cec2, SUC_lexicon_tags_27 },
    { 0x1e33cec4, SUC_lexicon_tags_3 },
    { 0xf313cec4, SUC_lexicon_tags_17 },
    { 0xf5b3cec5, SUC_lexicon_tags_15 },
    { 0x68b3cec4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d3cecc, SUC_lexicon_tags_10 },
    { 0xfa93cecd, SUC_lexicon_tags_27 },
    { 0xc133cece, SUC_lexicon_tags_15 },
    { 0x58f3cecf, SUC_lexicon_tags_17 },
    { 0x1ef3cecf, SUC_lexicon_tags_16 },
    { 0xa8f3ced1, SUC_lexicon_tags_87 },
    { 0x6bb3ced2, SUC_lexicon_tags_70 },
    { 0x6033cecc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d33ced6, SUC_lexicon_tags_4 },
    { 0xf093ced7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb833cedf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8413cee7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf3ceea, SUC_lexicon_tags_1 },
    { 0xe113ceeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d3cef4, SUC_lexicon_tags_19 },
    { 0x4853cef5, SUC_lexicon_tags_2 },
    { 0x3053cef6, SUC_lexicon_tags_2 },
    { 0xb853cef4, SUC_lexicon_tags_27 },
    { 0xda33cef5, SUC_lexicon_tags_45 },
    { 0xa033cef9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x2093cefc, SUC_lexicon_tags_43 },
    { 0x213cefd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd3ceff, SUC_lexicon_tags_0 },
    { 0x7b93cf00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c33cf03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d3cf06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2493cf08, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x45f3cf0a, SUC_lexicon_tags_83 },
    { 0xe213cf0b, SUC_lexicon_tags_15 },
    { 0xd973cf0b, SUC_lexicon_tags_30 },
    { 0x2c13cf0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a73cf0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb893cf12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61f3cf14, SUC_lexicon_tags_2 },
    { 0xc9f3cf15, SUC_lexicon_tags_60 },
    { 0x2393cf16, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f3cf19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd993cf1b, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0x8cf3cf1d, SUC_lexicon_tags_47 },
    { 0x4433cf1d, SUC_lexicon_tags_7 },
    { 0x6bf3cf1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8633cf26, SUC_lexicon_tags_21 },
    { 0xc5f3cf27, SUC_lexicon_tags_46 },
    { 0x51f3cf27, SUC_lexicon_tags_7 },
    { 0x23d3cf29, SUC_lexicon_tags_37 },
    { 0xda73cf29, SUC_lexicon_tags_9 },
    { 0xa133cf2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d3cf2f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf893cf32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16b3cf34, SUC_lexicon_tags_1 },
    { 0xb893cf35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b3cf37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3cf3a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf673cf3c, SUC_lexicon_tags_12 },
    { 0xa693cf3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3cf41, SUC_lexicon_tags_11 },
    { 0xef13cf42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae13cf4a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xdf73cf4c, SUC_lexicon_tags_12 },
    { 0x6c93cf4d, SUC_lexicon_tags_25 },
    { 0x10d3cf4c, SUC_lexicon_tags_11 },
    { 0x4033cf4f, SUC_lexicon_tags_43 },
    { 0xc8f3cf50, SUC_lexicon_tags_0 },
    { 0x2253cf4f, SUC_lexicon_tags_27 },
    { 0x19f3cf4f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xaf93cf54, SUC_lexicon_tags_66 },
    { 0xbb53cf54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f3cf5a, SUC_lexicon_tags_65 },
    { 0x4533cf5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b33cf65, SUC_lexicon_tags_66 },
    { 0x3933cf66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b3cf6a, SUC_lexicon_tags_23 },
    { 0x1e73cf6b, SUC_lexicon_tags_160 },
    { 0x7773cf6c, SUC_lexicon_tags_15 },
    { 0x2cb3cf6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec13cf6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc173cf71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb913cf74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c33cf76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1313cf79, SUC_lexicon_tags_14 },
    { 0x3b33cf79, SUC_lexicon_tags_68 },
    { 0x1273cf7b, SUC_lexicon_tags_1 },
    { 0x7213cf7c, SUC_lexicon_tags_27 },
    { 0x11d3cf7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f3cf80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa93cf84, SUC_lexicon_tags_37 },
    { 0xe973cf85, SUC_lexicon_tags_37 },
    { 0x6c93cf86, SUC_lexicon_tags_49 },
    { 0xcb3cf84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb3cf94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf3cf96, SUC_lexicon_tags_47 },
    { 0x1833cf96, SUC_lexicon_tags_43 },
    { 0xc693cf98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9153cf9c, SUC_lexicon_tags_764 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d93cfa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6df3cfa5, SUC_lexicon_tags_38 },
    { 0x7ff3cfa6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f3cfae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6173cfb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f3cfb8, SUC_lexicon_tags_11 },
    { 0x5e33cfb8, SUC_lexicon_tags_37 },
    { 0xaa93cfb8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7893cfbc, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb3cfc0, SUC_lexicon_tags_2 },
    { 0xe633cfc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb293cfc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdb3cfc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe753cfca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5433cfcc, SUC_lexicon_tags_7 },
    { 0xb4b3cfcd, SUC_lexicon_tags_1 },
    { 0xf413cfce, SUC_lexicon_tags_11 },
    { 0x9af3cfce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b93cfd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ef3cfd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22d3cfda, SUC_lexicon_tags_0 },
    { 0xd5f3cfdb, SUC_lexicon_tags_17 },
    { 0xe933cfdc, SUC_lexicon_tags_63 },
    { 0x43b3cfdd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d33cfe1, SUC_lexicon_tags_43 },
    { 0xc973cfe2, SUC_lexicon_tags_63 },
    { 0x3c53cfe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf33cfe7, SUC_lexicon_tags_41 },
    { 0x26f3cfe7, SUC_lexicon_tags_1 },
    { 0x5153cfe7, SUC_lexicon_tags_11 },
    { 0xc653cfe8, SUC_lexicon_tags_60 },
    { 0xced3cfe7, SUC_lexicon_tags_48 },
    { 0xf3f3cfe8, SUC_lexicon_tags_62 },
    { 0xe653cfe8, SUC_lexicon_tags_17 },
    { 0xcad3cfee, SUC_lexicon_tags_27 },
    { 0x4c93cfef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d3cff2, SUC_lexicon_tags_18 },
    { 0xcc13cff3, SUC_lexicon_tags_12 },
    { 0xed3cff4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3cff7, SUC_lexicon_tags_11 },
    { 0xe233cff8, SUC_lexicon_tags_38 },
    { 0xbad3cff9, SUC_lexicon_tags_79 },
    { 0x8a73cffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3673cffe, SUC_lexicon_tags_2 },
    { 0xb913cfff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76d3d001, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59f3d003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73d3d005, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf3d00d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4bd3d00f, SUC_lexicon_tags_8 },
    { 0x5eb3d00f, SUC_lexicon_tags_9 },
    { 0x5993d011, SUC_lexicon_tags_14 },
    { 0x79b3d012, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d3d018, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6573d01a, SUC_lexicon_tags_37 },
    { 0x5293d01b, SUC_lexicon_tags_47 },
    { 0xb733d01c, SUC_lexicon_tags_34 },
    { 0x42b3d01a, SUC_lexicon_tags_7 },
    { 0x86b3d01b, SUC_lexicon_tags_43 },
    { 0x9e13d01a, SUC_lexicon_tags_87 },
    { 0x1213d01c, SUC_lexicon_tags_27 },
    { 0x4df3d021, SUC_lexicon_tags_17 },
    { 0xd5d3d01d, SUC_lexicon_tags_43 },
    { 0xd413d01f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d029, SUC_lexicon_tags_43 },
    { 0xc1b3d02a, SUC_lexicon_tags_15 },
    { 0x1ed3d02b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f33d030, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9833d035, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa853d039, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x153d03b, SUC_lexicon_tags_85 },
    { 0x1b53d03b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbaf3d03e, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b3d042, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c33d046, SUC_lexicon_tags_15 },
    { 0xa793d047, SUC_lexicon_tags_27 },
    { 0x4733d048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf33d04a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7193d04e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9c33d050, SUC_lexicon_tags_11 },
    { 0xa673d051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9293d054, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4113d05b, SUC_lexicon_tags_47 },
    { 0xc153d05c, SUC_lexicon_tags_63 },
    { 0x1f93d05c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9c53d05f, SUC_lexicon_tags_39 },
    { 0x3533d060, SUC_lexicon_tags_55 },
    { 0xf93d060, SUC_lexicon_tags_6 },
    { 0x8df3d062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd653d064, SUC_lexicon_tags_17 },
    { 0x6dd3d064, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b53d068, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5353d06a, SUC_lexicon_tags_23 },
    { 0x7613d06a, SUC_lexicon_tags_15 },
    { 0x63f3d06c, SUC_lexicon_tags_11 },
    { 0xdf13d06d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f3d071, SUC_lexicon_tags_37 },
    { 0x5853d071, SUC_lexicon_tags_12 },
    { 0xfcd3d071, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d3d076, SUC_lexicon_tags_160 },
    { 0xabf3d077, SUC_lexicon_tags_1 },
    { 0xe0b3d078, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b3d07c, SUC_lexicon_tags_2 },
    { 0x5c53d07d, SUC_lexicon_tags_11 },
    { 0x7f13d07e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd3d087, SUC_lexicon_tags_8 },
    { 0xb93d088, SUC_lexicon_tags_47 },
    { 0x6193d088, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973d08d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3913d091, SUC_lexicon_tags_35 },
    { 0x2493d092, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0xcb93d094, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f3d09a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7553d09d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc553d09f, SUC_lexicon_tags_27 },
    { 0x373d0a0, SUC_lexicon_tags_9 },
    { 0x7b93d09f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16b3d0a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8873d0a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f3d0a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3f13d0a9, SUC_lexicon_tags_11 },
    { 0xc093d0aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd813d0af, SUC_lexicon_tags_1 },
    { 0xe833d0b0, SUC_lexicon_tags_31 },
    { 0x8733d0b0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf3d0b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c73d0bd, SUC_lexicon_tags_9 },
    { 0x8393d0be, SUC_lexicon_tags_26 },
    { 0xa5b3d0bf, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x62d3d0c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf93d0c4, SUC_lexicon_tags_27 },
    { 0xa8d3d0c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb273d0c7, SUC_lexicon_tags_2 },
    { 0x5033d0c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24f3d0ca, SUC_lexicon_tags_21 },
    { 0xb593d0cb, SUC_lexicon_tags_11 },
    { 0xf4b3d0cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd3d0ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x693d0d0, SUC_lexicon_tags_25 },
    { 0xd073d0d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c93d0d5, SUC_lexicon_tags_1 },
    { 0x4153d0d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3713d0db, SUC_lexicon_tags_7 },
    { 0x8ad3d0dc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5153d0e2, SUC_lexicon_tags_35 },
    { 0x9033d0e2, SUC_lexicon_tags_38 },
    { 0xca93d0e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f3d0ef, SUC_lexicon_tags_17 },
    { 0xba73d0f0, SUC_lexicon_tags_12 },
    { 0x1b13d0ef, SUC_lexicon_tags_46 },
    { 0x6673d0f2, SUC_lexicon_tags_65 },
    { 0x3073d0f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc353d0f8, SUC_lexicon_tags_37 },
    { 0x8593d0f8, SUC_lexicon_tags_1 },
    { 0xbad3d0f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8133d0ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf553d101, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b3d104, SUC_lexicon_tags_17 },
    { 0x7e73d105, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53d10a, SUC_lexicon_tags_107 },
    { 0x1973d10b, SUC_lexicon_tags_37 },
    { 0x5f13d10a, SUC_lexicon_tags_1 },
    { 0x613d10c, SUC_lexicon_tags_25 },
    { 0xcf93d10e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd73d110, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad93d116, SUC_lexicon_tags_60 },
    { 0x27b3d116, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b3d11f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb493d122, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8af3d124, SUC_lexicon_tags_6 },
    { 0x6813d125, SUC_lexicon_tags_6 },
    { 0x31f3d126, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdcb3d128, SUC_lexicon_tags_11 },
    { 0xebb3d129, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af3d12e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64d3d130, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa913d136, SUC_lexicon_tags_17 },
    { 0xfc93d137, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f3d13c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xfd13d13e, SUC_lexicon_tags_37 },
    { 0x8e53d13f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b3d146, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa713d149, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f53d14d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb3d150, SUC_lexicon_tags_1 },
    { 0x64f3d150, SUC_lexicon_tags_11 },
    { 0xf853d152, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9573d154, SUC_lexicon_tags_17 },
    { 0xc693d154, SUC_lexicon_tags_27 },
    { 0xf9f3d156, SUC_lexicon_tags_37 },
    { 0xb93d157, SUC_lexicon_tags_12 },
    { 0x8093d158, SUC_lexicon_tags_118 },
    { 0xa393d157, SUC_lexicon_tags_15 },
    { 0x72f3d154, SUC_lexicon_tags_37 },
    { 0x69f3d158, SUC_lexicon_tags_25 },
    { 0xc233d15c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf233d16b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6113d16d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1773d171, SUC_lexicon_tags_2 },
    { 0xc73d172, SUC_lexicon_tags_17 },
    { 0x8b33d173, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133d179, SUC_lexicon_tags_33 },
    { 0x72b3d17a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb3d17d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6233d181, SUC_lexicon_tags_8 },
    { 0xc813d181, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4153d189, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf793d18c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfdb3d18e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc7f3d190, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc33d192, SUC_lexicon_tags_37 },
    { 0x2113d193, SUC_lexicon_tags_9 },
    { 0xe933d194, SUC_lexicon_tags_43 },
    { 0x4313d195, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb73d198, SUC_lexicon_tags_25 },
    { 0x9f33d199, SUC_lexicon_tags_31 },
    { 0xcbb3d19a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f73d19f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7373d1a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b3d1a4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2953d1a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa273d1ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf853d1b0, SUC_lexicon_tags_25 },
    { 0x9353d1b0, SUC_lexicon_tags_45 },
    { 0xbd73d1b2, SUC_lexicon_tags_46 },
    { 0x3433d1b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7d13d1b5, SUC_lexicon_tags_25 },
    { 0x43b3d1b5, SUC_lexicon_tags_17 },
    { 0xae33d1b5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9dd3d1b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f3d1bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f3d1c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa633d1c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6593d1cc, SUC_lexicon_tags_25 },
    { 0x9433d1cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc33d1d1, SUC_lexicon_tags_25 },
    { 0x3153d1d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d3d1d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdfb3d1d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab3d1dc, SUC_lexicon_tags_29 },
    { 0x1df3d1dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4d3d1df, SUC_lexicon_tags_4 },
    { 0x58b3d1df, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f13d1e3, SUC_lexicon_tags_15 },
    { 0x24f3d1e4, SUC_lexicon_tags_15 },
    { 0x4e93d1e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7013d1ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2333d1ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad3d1f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f93d1f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x64d3d1f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde93d1fd, SUC_lexicon_tags_7 },
    { 0x4053d1fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8a53d200, SUC_lexicon_tags_15 },
    { 0x4d93d201, SUC_lexicon_tags_45 },
    { 0x7493d202, SUC_lexicon_tags_33 },
    { 0x9d53d203, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3093d206, SUC_lexicon_tags_29 },
    { 0xb093d207, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb493d20d, SUC_lexicon_tags_25 },
    { 0x42d3d20e, SUC_lexicon_tags_15 },
    { 0x7fd3d20f, SUC_lexicon_tags_6 },
    { 0xa4f3d20d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1593d214, SUC_lexicon_tags_1 },
    { 0xf0f3d214, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b3d21a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a93d21c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6973d220, SUC_lexicon_tags_14 },
    { 0x35b3d221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b33d223, SUC_lexicon_tags_27 },
    { 0xce73d224, SUC_lexicon_tags_27 },
    { 0x2633d223, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc73d228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb953d22e, SUC_lexicon_tags_9 },
    { 0x9933d22f, SUC_lexicon_tags_2 },
    { 0xbc13d230, SUC_lexicon_tags_56 },
    { 0x78d3d22f, SUC_lexicon_tags_59 },
    { 0x9e93d232, SUC_lexicon_tags_1 },
    { 0x1933d22f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ad3d235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d13d238, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb893d23a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc553d24a, SUC_lexicon_tags_17 },
    { 0x5093d24b, SUC_lexicon_tags_17 },
    { 0x3e33d24b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd3d24f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9f33d251, SUC_lexicon_tags_2 },
    { 0x85b3d252, SUC_lexicon_tags_12 },
    { 0x5ff3d252, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3673d256, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa713d258, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db3d25b, SUC_lexicon_tags_26 },
    { 0xf873d25c, SUC_lexicon_tags_1 },
    { 0x4c73d25d, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a33d260, SUC_lexicon_tags_25 },
    { 0x7893d261, SUC_lexicon_tags_2 },
    { 0x9b3d262, SUC_lexicon_tags_15 },
    { 0xc273d260, SUC_lexicon_tags_12 },
    { 0x2c93d264, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb3d26e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4d3d270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf3d274, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4373d27b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf13d281, SUC_lexicon_tags_26 },
    { 0x89d3d282, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa4f3d284, SUC_lexicon_tags_12 },
    { 0xb793d285, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd93d287, SUC_lexicon_tags_45 },
    { 0x8c33d288, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf53d28b, SUC_lexicon_tags_43 },
    { 0xf193d28c, SUC_lexicon_tags_28 },
    { 0x7573d28b, SUC_lexicon_tags_63 },
    { 0x8ed3d28e, SUC_lexicon_tags_12 },
    { 0xe33d28c, SUC_lexicon_tags_7 },
    { 0x6473d28e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb3d294, SUC_lexicon_tags_197 },
    { 0x8493d295, SUC_lexicon_tags_43 },
    { 0xbab3d294, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d3d29e, SUC_lexicon_tags_11 },
    { 0xfa53d29f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa733d2a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8673d2a4, SUC_lexicon_tags_19 },
    { 0x50b3d2a4, SUC_lexicon_tags_15 },
    { 0x75f3d2a6, SUC_lexicon_tags_68 },
    { 0xf73d2a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b3d2aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf173d2ac, SUC_lexicon_tags_23 },
    { 0xb3d2ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a33d2b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e13d2b3, SUC_lexicon_tags_17 },
    { 0xfa93d2b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3413d2b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f33d2b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2753d2bb, SUC_lexicon_tags_1 },
    { 0xb533d2bc, SUC_lexicon_tags_15 },
    { 0x5073d2bd, SUC_lexicon_tags_25 },
    { 0x21d3d2be, SUC_lexicon_tags_179 },
    { 0xa733d2bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb713d2c2, SUC_lexicon_tags_9 },
    { 0x5fb3d2c2, SUC_lexicon_tags_9 },
    { 0xc1b3d2c4, SUC_lexicon_tags_47 },
    { 0x44f3d2c5, SUC_lexicon_tags_3 },
    { 0xc653d2c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4993d2ca, SUC_lexicon_tags_12 },
    { 0x5d3d2ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea53d2d8, SUC_lexicon_tags_47 },
    { 0xef93d2d9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f73d2de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6953d2e0, SUC_lexicon_tags_15 },
    { 0xba53d2e0, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a13d2ed, SUC_lexicon_tags_25 },
    { 0xbf33d2ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bb3d2f0, SUC_lexicon_tags_83 },
    { 0xa993d2f0, SUC_lexicon_tags_27 },
    { 0x1113d2f2, SUC_lexicon_tags_28 },
    { 0xa9f3d2f3, SUC_lexicon_tags_33 },
    { 0x20b3d2f4, SUC_lexicon_tags_1 },
    { 0xdb73d2f2, SUC_lexicon_tags_2 },
    { 0xce13d2f2, SUC_lexicon_tags_27 },
    { 0x3533d2f4, SUC_lexicon_tags_68 },
    { 0x2593d2f8, SUC_lexicon_tags_10 },
    { 0xffb3d2f5, SUC_lexicon_tags_4 },
    { 0xc8b3d2f5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc233d2fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0d3d2fe, SUC_lexicon_tags_11 },
    { 0x4b13d2fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57f3d301, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed3d309, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe033d30d, SUC_lexicon_tags_43 },
    { 0x24f3d30d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3273d312, SUC_lexicon_tags_87 },
    { 0x3db3d313, SUC_lexicon_tags_2 },
    { 0xa833d314, SUC_lexicon_tags_0 },
    { 0x9d73d315, SUC_lexicon_tags_12 },
    { 0xe8d3d312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d3d318, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab3d31b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f3d320, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee33d325, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2ef3d327, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf3d32b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf93d32d, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb3d330, SUC_lexicon_tags_8 },
    { 0xea93d330, SUC_lexicon_tags_17 },
    { 0x2753d332, SUC_lexicon_tags_26 },
    { 0x4d13d332, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc593d338, SUC_lexicon_tags_8 },
    { 0xd1b3d338, SUC_lexicon_tags_25 },
    { 0x7bd3d33a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb53d342, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf3d347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b3d34a, SUC_lexicon_tags_39 },
    { 0x2f33d34b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d3d351, SUC_lexicon_tags_6 },
    { 0x50f3d352, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b3d357, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6533d359, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d3d361, SUC_lexicon_tags_15 },
    { 0x5733d362, SUC_lexicon_tags_2 },
    { 0x5e13d361, SUC_lexicon_tags_33 },
    { 0x69b3d364, SUC_lexicon_tags_2 },
    { 0xa233d363, SUC_lexicon_tags_39 },
    { 0x2913d366, SUC_lexicon_tags_15 },
    { 0xbd3d366, SUC_lexicon_tags_14 },
    { 0x3cf3d366, SUC_lexicon_tags_7 },
    { 0x9733d369, SUC_lexicon_tags_3 },
    { 0xf573d361, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb373d36e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba93d370, SUC_lexicon_tags_16 },
    { 0xec73d371, SUC_lexicon_tags_126 },
    { 0x5953d372, SUC_lexicon_tags_15 },
    { 0x953d371, SUC_lexicon_tags_27 },
    { 0x1493d373, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4e33d376, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b3d379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51f3d37b, SUC_lexicon_tags_15 },
    { 0xeb93d37c, SUC_lexicon_tags_24 },
    { 0xf593d37b, SUC_lexicon_tags_43 },
    { 0x5213d37e, SUC_lexicon_tags_27 },
    { 0x79b3d37f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5713d382, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc813d386, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd13d38b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d3d390, SUC_lexicon_tags_37 },
    { 0xb433d391, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3d395, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8853d398, SUC_lexicon_tags_38 },
    { 0xcb13d399, SUC_lexicon_tags_1 },
    { 0xa7b3d399, SUC_lexicon_tags_11 },
    { 0x63b3d39a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1153d39d, SUC_lexicon_tags_21 },
    { 0x29b3d39e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7073d3a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdef3d3a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e93d3a9, SUC_lexicon_tags_423 },
    { 0x6e73d3aa, SUC_lexicon_tags_31 },
    { 0x5e13d3ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce13d3ad, SUC_lexicon_tags_11 },
    { 0xd73d3ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe513d3b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b3d3b4, SUC_lexicon_tags_29 },
    { 0x8913d3b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef3d3b8, SUC_lexicon_tags_39 },
    { 0x1a33d3b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd553d3bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3d3c0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6233d3c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5393d3c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0d3d3ca, SUC_lexicon_tags_34 },
    { 0x4373d3cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1f3d3cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf3d3d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1993d3d3, SUC_lexicon_tags_43 },
    { 0x39f3d3d4, SUC_lexicon_tags_11 },
    { 0x1af3d3d5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xceb3d3d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4453d3d9, SUC_lexicon_tags_15 },
    { 0x5433d3da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa833d3df, SUC_lexicon_tags_141 },
    { 0x9a93d3e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab73d3e7, SUC_lexicon_tags_15 },
    { 0x4f13d3e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf3d3eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe953d3ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d3f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x48f3d3f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86d3d3f5, SUC_lexicon_tags_55 },
    { 0x6393d3f6, SUC_lexicon_tags_11 },
    { 0x1473d3f7, SUC_lexicon_tags_7 },
    { 0xa653d3f7, SUC_lexicon_tags_29 },
    { 0xb753d3f7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2713d3fd, SUC_lexicon_tags_14 },
    { 0xa933d3fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9233d404, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xec13d406, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb393d40a, SUC_lexicon_tags_37 },
    { 0x6333d40b, SUC_lexicon_tags_37 },
    { 0x6033d40b, SUC_lexicon_tags_29 },
    { 0x7233d40a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb533d40f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44d3d411, SUC_lexicon_tags_0 },
    { 0x1313d412, SUC_lexicon_tags_33 },
    { 0xf53d413, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed93d417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6333d419, SUC_lexicon_tags_37 },
    { 0x2233d419, SUC_lexicon_tags_7 },
    { 0xf533d41b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a33d420, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf993d423, SUC_lexicon_tags_46 },
    { 0x8333d423, SUC_lexicon_tags_2 },
    { 0xd2b3d425, SUC_lexicon_tags_14 },
    { 0x7313d426, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6d33d428, SUC_lexicon_tags_37 },
    { 0xd8d3d429, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8813d42b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc673d42d, SUC_lexicon_tags_26 },
    { 0x4493d42e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb753d431, SUC_lexicon_tags_37 },
    { 0x4ed3d432, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c73d435, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4b53d437, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff3d43b, SUC_lexicon_tags_2 },
    { 0xe0b3d43c, SUC_lexicon_tags_17 },
    { 0x22b3d43d, SUC_lexicon_tags_15 },
    { 0xe7d3d43e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4853d440, SUC_lexicon_tags_49 },
    { 0xc753d441, SUC_lexicon_tags_9 },
    { 0xd393d441, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf53d444, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d53d447, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db3d44c, SUC_lexicon_tags_0 },
    { 0xe0b3d44d, SUC_lexicon_tags_0 },
    { 0x49f3d44e, SUC_lexicon_tags_63 },
    { 0x45d3d44c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd193d452, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb3d455, SUC_lexicon_tags_27 },
    { 0xcaf3d455, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f3d459, SUC_lexicon_tags_37 },
    { 0xf8b3d459, SUC_lexicon_tags_25 },
    { 0xd433d45b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff3d45e, SUC_lexicon_tags_21 },
    { 0x6053d45f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd373d462, SUC_lexicon_tags_10 },
    { 0xdd93d463, SUC_lexicon_tags_17 },
    { 0xb93d464, SUC_lexicon_tags_1 },
    { 0xc293d464, SUC_lexicon_tags_14 },
    { 0x47d3d464, SUC_lexicon_tags_37 },
    { 0xbaf3d465, SUC_lexicon_tags_38 },
    { 0xb393d464, SUC_lexicon_tags_28 },
    { 0x1933d463, SUC_lexicon_tags_48 },
    { 0x8e73d468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f3d46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb3d470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f53d473, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc73d477, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2233d47c, SUC_lexicon_tags_11 },
    { 0xdc33d47d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf3d480, SUC_lexicon_tags_17 },
    { 0x4613d480, SUC_lexicon_tags_61 },
    { 0x3453d482, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x78d3d484, SUC_lexicon_tags_25 },
    { 0x3fd3d485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2373d488, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe53d48c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b3d490, SUC_lexicon_tags_33 },
    { 0x5593d491, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83d3d493, SUC_lexicon_tags_1 },
    { 0x33f3d493, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd973d49a, SUC_lexicon_tags_0 },
    { 0xe0b3d49b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1613d49e, SUC_lexicon_tags_14 },
    { 0x24d3d49e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f3d4a2, SUC_lexicon_tags_1 },
    { 0xd6d3d4a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f13d4a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b3d4ad, SUC_lexicon_tags_15 },
    { 0x5993d4ae, SUC_lexicon_tags_25 },
    { 0xf093d4af, SUC_lexicon_tags_9 },
    { 0xbef3d4af, SUC_lexicon_tags_10 },
    { 0x7e13d4b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d3d4b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe53d4b8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4953d4ba, SUC_lexicon_tags_15 },
    { 0x82d3d4ba, SUC_lexicon_tags_3 },
    { 0x43b3d4bb, SUC_lexicon_tags_34 },
    { 0xceb3d4bd, SUC_lexicon_tags_25 },
    { 0x9f33d4bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5db3d4c0, SUC_lexicon_tags_12 },
    { 0xde73d4c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62f3d4c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43f3d4c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef73d4c8, SUC_lexicon_tags_60 },
    { 0xca73d4c8, SUC_lexicon_tags_65 },
    { 0x2fb3d4ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8613d4cc, SUC_lexicon_tags_19 },
    { 0x2f73d4cd, SUC_lexicon_tags_37 },
    { 0x9673d4cd, SUC_lexicon_tags_133 },
    { 0xbfb3d4cd, SUC_lexicon_tags_65 },
    { 0xbb13d4d0, SUC_lexicon_tags_7 },
    { 0xac93d4d1, SUC_lexicon_tags_6 },
    { 0xef53d4ce, SUC_lexicon_tags_12 },
    { 0x3d3d4d0, SUC_lexicon_tags_11 },
    { 0x1153d4d4, SUC_lexicon_tags_43 },
    { 0x33b3d4d5, SUC_lexicon_tags_43 },
    { 0x95b3d4d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x313d4d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193d4df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa93d4e2, SUC_lexicon_tags_17 },
    { 0x5d3d4e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf3d4e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7973d4ea, SUC_lexicon_tags_1 },
    { 0xa073d4ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f3d4ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb733d4f4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf933d4f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33d504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5413d50a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8613d50c, SUC_lexicon_tags_15 },
    { 0xfff3d50d, SUC_lexicon_tags_12 },
    { 0x4953d50e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33d3d510, SUC_lexicon_tags_29 },
    { 0x8e13d510, SUC_lexicon_tags_17 },
    { 0x9f93d512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d3d516, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d3d518, SUC_lexicon_tags_25 },
    { 0x6b93d519, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b3d51c, SUC_lexicon_tags_15 },
    { 0xb533d51c, SUC_lexicon_tags_31 },
    { 0xb5f3d51c, SUC_lexicon_tags_17 },
    { 0x753d51f, SUC_lexicon_tags_11 },
    { 0xc7f3d51f, SUC_lexicon_tags_27 },
    { 0x7733d520, SUC_lexicon_tags_8 },
    { 0x88d3d51c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b3d527, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2653d52c, SUC_lexicon_tags_45 },
    { 0x29f3d52c, SUC_lexicon_tags_68 },
    { 0xb653d52e, SUC_lexicon_tags_30 },
    { 0xe253d52c, SUC_lexicon_tags_1 },
    { 0x2733d530, SUC_lexicon_tags_11 },
    { 0x9b33d52c, SUC_lexicon_tags_7 },
    { 0x13f3d532, SUC_lexicon_tags_60 },
    { 0xa73d533, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf3d53d, SUC_lexicon_tags_6 },
    { 0x3253d53d, SUC_lexicon_tags_1 },
    { 0xaed3d53f, SUC_lexicon_tags_15 },
    { 0x1b93d53f, SUC_lexicon_tags_47 },
    { 0xfe73d540, SUC_lexicon_tags_7 },
    { 0x8973d542, SUC_lexicon_tags_37 },
    { 0x9373d543, SUC_lexicon_tags_1 },
    { 0xf833d540, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3d547, SUC_lexicon_tags_14 },
    { 0x36d3d548, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3273d54a, SUC_lexicon_tags_45 },
    { 0xc8d3d54b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb33d551, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe973d556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f3d55a, SUC_lexicon_tags_17 },
    { 0x9c33d55a, SUC_lexicon_tags_26 },
    { 0x60b3d55c, SUC_lexicon_tags_4 },
    { 0x1793d55d, SUC_lexicon_tags_43 },
    { 0x6833d55e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d13d560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f33d563, SUC_lexicon_tags_79 },
    { 0xe273d564, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed3d56b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6213d574, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd573d57a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d3d57f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13d582, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb793d584, SUC_lexicon_tags_17 },
    { 0x533d585, SUC_lexicon_tags_28 },
    { 0xc573d586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f93d58a, SUC_lexicon_tags_14 },
    { 0x8d3d58b, SUC_lexicon_tags_37 },
    { 0xca13d58c, SUC_lexicon_tags_1 },
    { 0x85d3d58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73d593, SUC_lexicon_tags_57 },
    { 0x9a13d593, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2593d598, SUC_lexicon_tags_37 },
    { 0x7b53d599, SUC_lexicon_tags_2 },
    { 0x2b93d598, SUC_lexicon_tags_31 },
    { 0x2393d598, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4873d5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a33d5a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d3d5ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed3d5b0, SUC_lexicon_tags_11 },
    { 0xa173d5b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x313d5b3, SUC_lexicon_tags_15 },
    { 0x853d5b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1733d5b7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a33d5bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f3d5bf, SUC_lexicon_tags_241 },
    { 0, NULL },
    { 0xe753d5c1, SUC_lexicon_tags_65 },
    { 0x2713d5c2, SUC_lexicon_tags_28 },
    { 0xa473d5c3, SUC_lexicon_tags_60 },
    { 0x4e33d5c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd093d5c7, SUC_lexicon_tags_23 },
    { 0xe953d5c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcfb3d5ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f53d5cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4493d5d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5353d5d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9753d5d7, SUC_lexicon_tags_25 },
    { 0xb313d5d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa353d5dd, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5073d5e5, SUC_lexicon_tags_198 },
    { 0x99f3d5e6, SUC_lexicon_tags_11 },
    { 0xd993d5e7, SUC_lexicon_tags_46 },
    { 0xee93d5e8, SUC_lexicon_tags_1 },
    { 0x9253d5e9, SUC_lexicon_tags_27 },
    { 0x49d3d5ea, SUC_lexicon_tags_63 },
    { 0xd633d5e8, SUC_lexicon_tags_47 },
    { 0x1853d5ec, SUC_lexicon_tags_23 },
    { 0xcc13d5e6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b13d5f0, SUC_lexicon_tags_75 },
    { 0xb33d5f1, SUC_lexicon_tags_87 },
    { 0xd293d5f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f3d5fc, SUC_lexicon_tags_7 },
    { 0xc9b3d5fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d602, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b53d607, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc33d60a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab3d611, SUC_lexicon_tags_45 },
    { 0x2e13d612, SUC_lexicon_tags_1 },
    { 0x4113d613, SUC_lexicon_tags_45 },
    { 0xa033d612, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f33d61a, SUC_lexicon_tags_37 },
    { 0x9473d61b, SUC_lexicon_tags_12 },
    { 0xb9f3d61b, SUC_lexicon_tags_12 },
    { 0x8c73d61a, SUC_lexicon_tags_65 },
    { 0x1333d61c, SUC_lexicon_tags_0 },
    { 0xde53d61f, SUC_lexicon_tags_2 },
    { 0xf633d620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf253d625, SUC_lexicon_tags_15 },
    { 0xc2f3d626, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e93d62b, SUC_lexicon_tags_33 },
    { 0xbc53d62c, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0x573d62e, SUC_lexicon_tags_53 },
    { 0xca13d62e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bb3d631, SUC_lexicon_tags_17 },
    { 0xaaf3d632, SUC_lexicon_tags_10 },
    { 0x9ed3d631, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xff13d635, SUC_lexicon_tags_26 },
    { 0x533d636, SUC_lexicon_tags_25 },
    { 0xe173d637, SUC_lexicon_tags_84 },
    { 0x59d3d638, SUC_lexicon_tags_60 },
    { 0x833d636, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd793d63c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b3d640, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd433d642, SUC_lexicon_tags_0 },
    { 0x39f3d642, SUC_lexicon_tags_17 },
    { 0xa5f3d643, SUC_lexicon_tags_9 },
    { 0x2e13d645, SUC_lexicon_tags_14 },
    { 0xadf3d646, SUC_lexicon_tags_12 },
    { 0x4e13d646, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b3d650, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb3d659, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1733d65b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a73d65e, SUC_lexicon_tags_11 },
    { 0xc4f3d65f, SUC_lexicon_tags_66 },
    { 0x7e93d660, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd313d662, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa153d664, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e93d667, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8073d669, SUC_lexicon_tags_1 },
    { 0x98f3d66a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf3d66d, SUC_lexicon_tags_37 },
    { 0xfe73d66e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5113d670, SUC_lexicon_tags_45 },
    { 0xf693d671, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa733d674, SUC_lexicon_tags_63 },
    { 0x2273d675, SUC_lexicon_tags_23 },
    { 0xd173d676, SUC_lexicon_tags_43 },
    { 0x6e33d677, SUC_lexicon_tags_32 },
    { 0x9b73d678, SUC_lexicon_tags_11 },
    { 0xd253d676, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc653d67c, SUC_lexicon_tags_12 },
    { 0x5853d67d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb493d67f, SUC_lexicon_tags_25 },
    { 0x4493d680, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x96d3d682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f3d685, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8813d688, SUC_lexicon_tags_7 },
    { 0x2473d689, SUC_lexicon_tags_1 },
    { 0xc413d68a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b3d68e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b93d692, SUC_lexicon_tags_37 },
    { 0x5673d693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb173d697, SUC_lexicon_tags_2 },
    { 0xb893d697, SUC_lexicon_tags_39 },
    { 0x9c93d699, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8673d69c, SUC_lexicon_tags_17 },
    { 0x8393d69d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe933d6a0, SUC_lexicon_tags_1 },
    { 0x5d3d6a1, SUC_lexicon_tags_11 },
    { 0xc073d6a1, SUC_lexicon_tags_27 },
    { 0xb33d6a3, SUC_lexicon_tags_15 },
    { 0x7433d6a4, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x8113d6a6, SUC_lexicon_tags_37 },
    { 0x90f3d6a7, SUC_lexicon_tags_30 },
    { 0xa2f3d6a8, SUC_lexicon_tags_59 },
    { 0xb913d6a9, SUC_lexicon_tags_17 },
    { 0xe713d6a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f3d6ae, SUC_lexicon_tags_49 },
    { 0xdd93d6af, SUC_lexicon_tags_8 },
    { 0x2e53d6b0, SUC_lexicon_tags_25 },
    { 0x51f3d6b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd73d6b5, SUC_lexicon_tags_68 },
    { 0xfef3d6b6, SUC_lexicon_tags_2 },
    { 0x5fb3d6b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7b3d6b9, SUC_lexicon_tags_30 },
    { 0x6a33d6b9, SUC_lexicon_tags_9 },
    { 0x2033d6b9, SUC_lexicon_tags_37 },
    { 0xa753d6ba, SUC_lexicon_tags_1 },
    { 0x8f3d6bb, SUC_lexicon_tags_2 },
    { 0x8c13d6be, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf3d6c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf33d6c8, SUC_lexicon_tags_87 },
    { 0x8933d6c9, SUC_lexicon_tags_2 },
    { 0x3af3d6c8, SUC_lexicon_tags_12 },
    { 0x9293d6cb, SUC_lexicon_tags_25 },
    { 0x11b3d6cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x573d6ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3513d6d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b3d6d7, SUC_lexicon_tags_21 },
    { 0x3673d6d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb073d6db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f73d6de, SUC_lexicon_tags_46 },
    { 0x6733d6df, SUC_lexicon_tags_37 },
    { 0xd853d6e0, SUC_lexicon_tags_45 },
    { 0x86f3d6e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5153d6e7, SUC_lexicon_tags_68 },
    { 0x3353d6e8, SUC_lexicon_tags_27 },
    { 0xb973d6e9, SUC_lexicon_tags_6 },
    { 0x8333d6ea, SUC_lexicon_tags_11 },
    { 0x57f3d6ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6113d6f0, SUC_lexicon_tags_5 },
    { 0xfc53d6f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x62d3d6f3, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d13d6f6, SUC_lexicon_tags_15 },
    { 0xf153d6f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51b3d6f9, SUC_lexicon_tags_7 },
    { 0x5833d6f9, SUC_lexicon_tags_15 },
    { 0xbff3d6fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e13d700, SUC_lexicon_tags_25 },
    { 0xe833d701, SUC_lexicon_tags_31 },
    { 0x6a93d702, SUC_lexicon_tags_336 },
    { 0xc8b3d701, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2f93d705, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf473d708, SUC_lexicon_tags_172 },
    { 0x34f3d709, SUC_lexicon_tags_43 },
    { 0x73d70a, SUC_lexicon_tags_11 },
    { 0x8473d70a, SUC_lexicon_tags_11 },
    { 0x1733d70c, SUC_lexicon_tags_15 },
    { 0x8533d70d, SUC_lexicon_tags_12 },
    { 0x44f3d70d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef3d711, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f3d715, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x113d718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c53d71a, SUC_lexicon_tags_12 },
    { 0xc2f3d71b, SUC_lexicon_tags_1 },
    { 0x4c93d71a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc713d71f, SUC_lexicon_tags_2 },
    { 0x7153d71f, SUC_lexicon_tags_14 },
    { 0xb893d720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6433d730, SUC_lexicon_tags_255 },
    { 0xd413d731, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4553d740, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb3d749, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c33d74c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bd3d74e, SUC_lexicon_tags_15 },
    { 0x8c53d74f, SUC_lexicon_tags_32 },
    { 0x9293d74e, SUC_lexicon_tags_27 },
    { 0x7013d751, SUC_lexicon_tags_57 },
    { 0x35d3d752, SUC_lexicon_tags_16 },
    { 0xe513d750, SUC_lexicon_tags_60 },
    { 0x5913d754, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad3d758, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd13d75b, SUC_lexicon_tags_11 },
    { 0x9413d75c, SUC_lexicon_tags_37 },
    { 0x5d13d75d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a93d75f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf33d761, SUC_lexicon_tags_85 },
    { 0xd613d762, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1153d765, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbf93d767, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2853d76d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xf513d76f, SUC_lexicon_tags_45 },
    { 0x1f73d76f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda73d777, SUC_lexicon_tags_47 },
    { 0x19d3d778, SUC_lexicon_tags_63 },
    { 0x4e53d779, SUC_lexicon_tags_31 },
    { 0xd213d779, SUC_lexicon_tags_37 },
    { 0x6913d777, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6593d77d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7333d783, SUC_lexicon_tags_23 },
    { 0x8d73d784, SUC_lexicon_tags_11 },
    { 0x393d784, SUC_lexicon_tags_6 },
    { 0xcf13d783, SUC_lexicon_tags_47 },
    { 0x7593d787, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b3d78a, SUC_lexicon_tags_60 },
    { 0x69b3d78b, SUC_lexicon_tags_31 },
    { 0x66b3d78a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7973d790, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d3d794, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc133d799, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9993d79d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a33d7a3, SUC_lexicon_tags_33 },
    { 0xfeb3d7a4, SUC_lexicon_tags_25 },
    { 0x5ed3d7a5, SUC_lexicon_tags_15 },
    { 0xf373d7a4, SUC_lexicon_tags_37 },
    { 0x5353d7a5, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x8f13d7a9, SUC_lexicon_tags_12 },
    { 0xbe73d7aa, SUC_lexicon_tags_14 },
    { 0x8ff3d7ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d3d7ad, SUC_lexicon_tags_11 },
    { 0xa093d7ae, SUC_lexicon_tags_15 },
    { 0xdeb3d7af, SUC_lexicon_tags_15 },
    { 0xf9f3d7b0, SUC_lexicon_tags_68 },
    { 0x10f3d7ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6533d7b5, SUC_lexicon_tags_25 },
    { 0xa33d7b6, SUC_lexicon_tags_37 },
    { 0x7273d7b6, SUC_lexicon_tags_11 },
    { 0x6b3d7b8, SUC_lexicon_tags_60 },
    { 0x3d93d7b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xce53d7bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc733d7be, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf373d7c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af3d7cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8493d7d1, SUC_lexicon_tags_10 },
    { 0xaef3d7d1, SUC_lexicon_tags_37 },
    { 0xf333d7d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd033d7d5, SUC_lexicon_tags_19 },
    { 0x9e73d7d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b3d7da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd0f3d7dc, SUC_lexicon_tags_2 },
    { 0xf493d7dd, SUC_lexicon_tags_2 },
    { 0x1733d7dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb053d7e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x36b3d7e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29b3d7e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5f3d7e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5b3d7e9, SUC_lexicon_tags_49 },
    { 0x9ad3d7ea, SUC_lexicon_tags_43 },
    { 0x2c33d7e9, SUC_lexicon_tags_46 },
    { 0x4c73d7ea, SUC_lexicon_tags_25 },
    { 0x1613d7ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7833d7ef, SUC_lexicon_tags_12 },
    { 0xbf33d7f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7d3d7f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad33d7f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5753d7f7, SUC_lexicon_tags_25 },
    { 0xbf93d7f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b3d7fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d93d803, SUC_lexicon_tags_224 },
    { 0x9133d804, SUC_lexicon_tags_32 },
    { 0x4653d805, SUC_lexicon_tags_17 },
    { 0x9a33d805, SUC_lexicon_tags_27 },
    { 0xa8b3d807, SUC_lexicon_tags_0 },
    { 0xf233d808, SUC_lexicon_tags_47 },
    { 0x1773d809, SUC_lexicon_tags_37 },
    { 0xd533d803, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a73d80f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd953d812, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e73d815, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8913d817, SUC_lexicon_tags_34 },
    { 0xccf3d817, SUC_lexicon_tags_2 },
    { 0xa53d817, SUC_lexicon_tags_11 },
    { 0x8013d81a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11b3d81c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85f3d81e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f53d823, SUC_lexicon_tags_29 },
    { 0xcfd3d824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1933d829, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe673d82c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c13d833, SUC_lexicon_tags_0 },
    { 0x1113d834, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe573d838, SUC_lexicon_tags_11 },
    { 0x7133d838, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3d83d, SUC_lexicon_tags_11 },
    { 0xe7d3d83e, SUC_lexicon_tags_27 },
    { 0xe713d83e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c13d848, SUC_lexicon_tags_11 },
    { 0x9f93d848, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b3d84c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4513d84e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bd3d850, SUC_lexicon_tags_12 },
    { 0x8213d851, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1533d85a, SUC_lexicon_tags_1 },
    { 0xb173d85a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b3d85e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3433d860, SUC_lexicon_tags_41 },
    { 0xff53d860, SUC_lexicon_tags_4 },
    { 0xb893d862, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9753d864, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b93d866, SUC_lexicon_tags_7 },
    { 0xc733d867, SUC_lexicon_tags_28 },
    { 0x5693d867, SUC_lexicon_tags_17 },
    { 0x4693d867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f3d86c, SUC_lexicon_tags_14 },
    { 0xebb3d86c, SUC_lexicon_tags_15 },
    { 0xeb13d86e, SUC_lexicon_tags_68 },
    { 0x8bf3d86f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5453d871, SUC_lexicon_tags_17 },
    { 0xe73d871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6993d874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff3d877, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db3d87a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6d3d87c, SUC_lexicon_tags_38 },
    { 0x62d3d87c, SUC_lexicon_tags_4 },
    { 0xa973d87e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa33d880, SUC_lexicon_tags_0 },
    { 0x2793d881, SUC_lexicon_tags_194 },
    { 0xdb53d882, SUC_lexicon_tags_9 },
    { 0xefd3d883, SUC_lexicon_tags_12 },
    { 0x3833d884, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec13d888, SUC_lexicon_tags_25 },
    { 0x6993d889, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x8133d88b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb3d892, SUC_lexicon_tags_11 },
    { 0x15d3d893, SUC_lexicon_tags_25 },
    { 0x9353d893, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed3d897, SUC_lexicon_tags_11 },
    { 0x5493d898, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x80f3d89a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab3d89e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b3d8a1, SUC_lexicon_tags_0 },
    { 0xec33d8a2, SUC_lexicon_tags_23 },
    { 0x4673d8a2, SUC_lexicon_tags_37 },
    { 0x6e53d8a4, SUC_lexicon_tags_1 },
    { 0x813d8a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd793d8a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf73d8a9, SUC_lexicon_tags_37 },
    { 0x5273d8aa, SUC_lexicon_tags_26 },
    { 0xcc93d8ab, SUC_lexicon_tags_7 },
    { 0xc913d8ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f3d8b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd13d8b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb3d8b7, SUC_lexicon_tags_29 },
    { 0xb833d8b8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x33d8ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f3d8bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f3d8c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c13d8c6, SUC_lexicon_tags_15 },
    { 0x63b3d8c6, SUC_lexicon_tags_11 },
    { 0xf93d8c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633d8cc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5213d8ce, SUC_lexicon_tags_16 },
    { 0x79b3d8cf, SUC_lexicon_tags_30 },
    { 0xcb73d8d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb3d8d4, SUC_lexicon_tags_13 },
    { 0x4633d8d5, SUC_lexicon_tags_33 },
    { 0x71b3d8d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cb3d8d8, SUC_lexicon_tags_21 },
    { 0x2593d8d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9613d8db, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x12f3d8dd, SUC_lexicon_tags_55 },
    { 0x9453d8dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c73d8e1, SUC_lexicon_tags_7 },
    { 0x43b3d8e1, SUC_lexicon_tags_15 },
    { 0x31f3d8e3, SUC_lexicon_tags_11 },
    { 0xab3d8e4, SUC_lexicon_tags_38 },
    { 0xa333d8e5, SUC_lexicon_tags_37 },
    { 0xb9f3d8e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfbb3d8e8, SUC_lexicon_tags_15 },
    { 0xe5f3d8e9, SUC_lexicon_tags_39 },
    { 0x15d3d8e8, SUC_lexicon_tags_11 },
    { 0x1593d8eb, SUC_lexicon_tags_27 },
    { 0x5133d8ec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9ff3d8ee, SUC_lexicon_tags_11 },
    { 0x1d93d8ef, SUC_lexicon_tags_27 },
    { 0xf673d8f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x51f3d8f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f73d8f4, SUC_lexicon_tags_0 },
    { 0xb3d3d8f5, SUC_lexicon_tags_23 },
    { 0xa033d8f6, SUC_lexicon_tags_57 },
    { 0xf593d8f7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa7d3d8f9, SUC_lexicon_tags_6 },
    { 0x1633d8f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2773d900, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b73d907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3053d909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62b3d90b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66d3d90d, SUC_lexicon_tags_5 },
    { 0x7253d90e, SUC_lexicon_tags_11 },
    { 0xc153d90f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9073d911, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa253d913, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f73d91e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d3d926, SUC_lexicon_tags_7 },
    { 0x55d3d927, SUC_lexicon_tags_14 },
    { 0x5eb3d928, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc413d92d, SUC_lexicon_tags_2 },
    { 0x78d3d92e, SUC_lexicon_tags_0 },
    { 0xed53d92f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1d3d931, SUC_lexicon_tags_17 },
    { 0x4e53d932, SUC_lexicon_tags_9 },
    { 0xa633d933, SUC_lexicon_tags_25 },
    { 0xe473d933, SUC_lexicon_tags_38 },
    { 0x8cb3d932, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b3d93b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36d3d93d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb3d942, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa713d945, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1753d94a, SUC_lexicon_tags_11 },
    { 0x853d94b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc73d94e, SUC_lexicon_tags_2 },
    { 0x8933d94f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8473d951, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8953d960, SUC_lexicon_tags_87 },
    { 0x1693d961, SUC_lexicon_tags_24 },
    { 0xc673d962, SUC_lexicon_tags_28 },
    { 0x5893d963, SUC_lexicon_tags_21 },
    { 0x313d962, SUC_lexicon_tags_0 },
    { 0x3133d965, SUC_lexicon_tags_30 },
    { 0x5b93d966, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d3d96b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x30b3d96d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb813d971, SUC_lexicon_tags_27 },
    { 0x2693d972, SUC_lexicon_tags_2 },
    { 0x3713d973, SUC_lexicon_tags_68 },
    { 0xfa33d974, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc93d976, SUC_lexicon_tags_14 },
    { 0xb0d3d977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2773d97a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe73d980, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4673d984, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3f3d986, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a53d98a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c53d98c, SUC_lexicon_tags_37 },
    { 0x34b3d98d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd3d992, SUC_lexicon_tags_7 },
    { 0x7973d993, SUC_lexicon_tags_49 },
    { 0x64f3d992, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29b3d996, SUC_lexicon_tags_7 },
    { 0x2613d997, SUC_lexicon_tags_9 },
    { 0xc8d3d998, SUC_lexicon_tags_47 },
    { 0x8d33d999, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b3d99c, SUC_lexicon_tags_17 },
    { 0xff13d99d, SUC_lexicon_tags_19 },
    { 0x933d99e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x97f3d9a0, SUC_lexicon_tags_12 },
    { 0xfaf3d9a0, SUC_lexicon_tags_33 },
    { 0xb13d9a2, SUC_lexicon_tags_39 },
    { 0x5613d9a3, SUC_lexicon_tags_47 },
    { 0x1973d9a4, SUC_lexicon_tags_38 },
    { 0x8e73d9a4, SUC_lexicon_tags_46 },
    { 0xc53d9a4, SUC_lexicon_tags_74 },
    { 0x4593d9a6, SUC_lexicon_tags_10 },
    { 0xd8d3d9a8, SUC_lexicon_tags_38 },
    { 0x76b3d9a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80b3d9ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd3d9ae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b3d9b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9793d9ba, SUC_lexicon_tags_23 },
    { 0x33b3d9bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb53d9be, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2153d9c2, SUC_lexicon_tags_37 },
    { 0xff93d9c3, SUC_lexicon_tags_2 },
    { 0xd293d9c4, SUC_lexicon_tags_15 },
    { 0xe593d9c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf3d9c9, SUC_lexicon_tags_12 },
    { 0x2613d9ca, SUC_lexicon_tags_45 },
    { 0x8d13d9cb, SUC_lexicon_tags_7 },
    { 0x6ef3d9cc, SUC_lexicon_tags_33 },
    { 0x9193d9cc, SUC_lexicon_tags_35 },
    { 0x9e53d9cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f3d9d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd3d9da, SUC_lexicon_tags_26 },
    { 0x1673d9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2713d9de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2253d9e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1373d9e5, SUC_lexicon_tags_30 },
    { 0x8653d9e6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa253d9e9, SUC_lexicon_tags_12 },
    { 0x5e53d9e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f3d9f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d3d9f4, SUC_lexicon_tags_12 },
    { 0x84b3d9f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87d3d9f7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf133d9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb813d9fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5513da00, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0xce33da02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd3da0a, SUC_lexicon_tags_45 },
    { 0xe7d3da0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f3da0f, SUC_lexicon_tags_44 },
    { 0xff13da10, SUC_lexicon_tags_46 },
    { 0x5293da10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc613da14, SUC_lexicon_tags_133 },
    { 0x74f3da15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1af3da17, SUC_lexicon_tags_95 },
    { 0x5213da18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7113da1f, SUC_lexicon_tags_0 },
    { 0x2b13da1f, SUC_lexicon_tags_27 },
    { 0xd1f3da20, SUC_lexicon_tags_25 },
    { 0xbe33da22, SUC_lexicon_tags_5 },
    { 0xd153da22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5753da26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b3da2b, SUC_lexicon_tags_11 },
    { 0x75f3da2b, SUC_lexicon_tags_23 },
    { 0xf873da2c, SUC_lexicon_tags_1 },
    { 0x693da2e, SUC_lexicon_tags_1 },
    { 0x1f3da2e, SUC_lexicon_tags_11 },
    { 0xcaf3da30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f3da38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2333da3b, SUC_lexicon_tags_17 },
    { 0xb833da3c, SUC_lexicon_tags_7 },
    { 0xcd33da3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf213da43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b13da46, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xda13da49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ab3da4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df3da4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3f3da50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa393da58, SUC_lexicon_tags_43 },
    { 0x9e73da59, SUC_lexicon_tags_0 },
    { 0x76b3da59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7753da61, SUC_lexicon_tags_9 },
    { 0x4eb3da62, SUC_lexicon_tags_27 },
    { 0x14b3da62, SUC_lexicon_tags_263 },
    { 0xfed3da64, SUC_lexicon_tags_30 },
    { 0xe5f3da65, SUC_lexicon_tags_49 },
    { 0xb133da66, SUC_lexicon_tags_13 },
    { 0xe5f3da67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef3da6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd433da6d, SUC_lexicon_tags_37 },
    { 0x9193da6d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef73da72, SUC_lexicon_tags_8 },
    { 0x2713da72, SUC_lexicon_tags_1100 },
    { 0x11d3da72, SUC_lexicon_tags_17 },
    { 0x4873da75, SUC_lexicon_tags_8 },
    { 0xb713da75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b3da7d, SUC_lexicon_tags_33 },
    { 0xf9d3da7e, SUC_lexicon_tags_26 },
    { 0xe313da7f, SUC_lexicon_tags_17 },
    { 0x2df3da7f, SUC_lexicon_tags_1 },
    { 0x9ff3da81, SUC_lexicon_tags_15 },
    { 0xb573da82, SUC_lexicon_tags_37 },
    { 0xd893da80, SUC_lexicon_tags_14 },
    { 0x8db3da84, SUC_lexicon_tags_7 },
    { 0x69d3da85, SUC_lexicon_tags_2 },
    { 0x2e93da7e, SUC_lexicon_tags_17 },
    { 0xed93da87, SUC_lexicon_tags_10 },
    { 0xd373da80, SUC_lexicon_tags_17 },
    { 0x8333da89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf153da90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb3da94, SUC_lexicon_tags_11 },
    { 0x3473da95, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x633da98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe053da9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2873daa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x433daa5, SUC_lexicon_tags_38 },
    { 0x2b13daa5, SUC_lexicon_tags_12 },
    { 0xcb3daa7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd13daac, SUC_lexicon_tags_19 },
    { 0xc13daad, SUC_lexicon_tags_39 },
    { 0xf3b3daac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad3dab2, SUC_lexicon_tags_1 },
    { 0xdc13dab3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f3dab6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d3dabb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f3dabe, SUC_lexicon_tags_45 },
    { 0x2333dabf, SUC_lexicon_tags_0 },
    { 0x46f3dac0, SUC_lexicon_tags_26 },
    { 0xe153dabf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e53dac6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x93d3dac8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e73daca, SUC_lexicon_tags_12 },
    { 0x9bf3dacb, SUC_lexicon_tags_70 },
    { 0x2233dacc, SUC_lexicon_tags_43 },
    { 0x5073dacc, SUC_lexicon_tags_35 },
    { 0xcd93dacd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe053dad0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2193dad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993dad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb253dadc, SUC_lexicon_tags_37 },
    { 0x9493dadc, SUC_lexicon_tags_37 },
    { 0xe833dadd, SUC_lexicon_tags_2 },
    { 0x7d53dade, SUC_lexicon_tags_4 },
    { 0x4e53dadf, SUC_lexicon_tags_15 },
    { 0xaf73dae1, SUC_lexicon_tags_10 },
    { 0xecf3dae1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b13dae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9333dae9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6f3daeb, SUC_lexicon_tags_25 },
    { 0xf033daec, SUC_lexicon_tags_7 },
    { 0xff13daec, SUC_lexicon_tags_15 },
    { 0x1ab3daec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8833daf2, SUC_lexicon_tags_37 },
    { 0x5173daf3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d3daf6, SUC_lexicon_tags_33 },
    { 0x47b3daf7, SUC_lexicon_tags_38 },
    { 0x5c33daf8, SUC_lexicon_tags_1 },
    { 0xeb53daf9, SUC_lexicon_tags_47 },
    { 0xd9b3dafa, SUC_lexicon_tags_40 },
    { 0x6dd3dafb, SUC_lexicon_tags_15 },
    { 0x89f3dafa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b3db01, SUC_lexicon_tags_33 },
    { 0x2a33db01, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e73db08, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1773db0a, SUC_lexicon_tags_14 },
    { 0xadd3db0a, SUC_lexicon_tags_12 },
    { 0x1893db0b, SUC_lexicon_tags_61 },
    { 0x4a93db0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef3db11, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x64f3db13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f3db18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcad3db1a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6613db1c, SUC_lexicon_tags_2 },
    { 0x2d93db1c, SUC_lexicon_tags_12 },
    { 0xc473db1d, SUC_lexicon_tags_38 },
    { 0x5ed3db1f, SUC_lexicon_tags_45 },
    { 0x7613db20, SUC_lexicon_tags_82 },
    { 0x6ed3db21, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf633db27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d3db2b, SUC_lexicon_tags_17 },
    { 0x39d3db2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb3db2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15b3db31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xde33db33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c73db38, SUC_lexicon_tags_17 },
    { 0x4473db39, SUC_lexicon_tags_29 },
    { 0x9153db39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe253db3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe753db3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3db42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb193db48, SUC_lexicon_tags_25 },
    { 0x6293db49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1193db4d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd3db55, SUC_lexicon_tags_37 },
    { 0xfcd3db56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf13db58, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe313db60, SUC_lexicon_tags_31 },
    { 0x1013db61, SUC_lexicon_tags_68 },
    { 0xaf3db61, SUC_lexicon_tags_11 },
    { 0x91f3db63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5613db65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d3db6e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d3db71, SUC_lexicon_tags_27 },
    { 0x6273db71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f3db7b, SUC_lexicon_tags_8 },
    { 0x8df3db7b, SUC_lexicon_tags_37 },
    { 0x8353db7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8a13db7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9053db83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7293db8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xedd3db8f, SUC_lexicon_tags_2 },
    { 0xd173db90, SUC_lexicon_tags_12 },
    { 0xb573db90, SUC_lexicon_tags_7 },
    { 0xb733db92, SUC_lexicon_tags_13 },
    { 0x1493db90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b3db96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x64b3db98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3533db9a, SUC_lexicon_tags_11 },
    { 0xba13db9b, SUC_lexicon_tags_5 },
    { 0x6533db9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3fb3db9e, SUC_lexicon_tags_1 },
    { 0xca93db9f, SUC_lexicon_tags_33 },
    { 0x75f3dba0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb3dba3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3dba6, SUC_lexicon_tags_0 },
    { 0xa9f3dba7, SUC_lexicon_tags_13 },
    { 0xb0d3dba7, SUC_lexicon_tags_47 },
    { 0x9f73dba8, SUC_lexicon_tags_11 },
    { 0xdff3dbaa, SUC_lexicon_tags_7 },
    { 0xa333dbab, SUC_lexicon_tags_9 },
    { 0x3953dbab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xad93dbaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c13dbb4, SUC_lexicon_tags_37 },
    { 0x9213dbb5, SUC_lexicon_tags_1 },
    { 0x3473dbb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59b3dbb8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db3dbbc, SUC_lexicon_tags_12 },
    { 0x6bf3dbbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x553dbbf, SUC_lexicon_tags_9 },
    { 0x76b3dbc0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f13dbc4, SUC_lexicon_tags_27 },
    { 0x52d3dbc5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8193dbc8, SUC_lexicon_tags_15 },
    { 0xe9d3dbc9, SUC_lexicon_tags_33 },
    { 0xe93dbc8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd13dbcd, SUC_lexicon_tags_17 },
    { 0x9f33dbcd, SUC_lexicon_tags_1 },
    { 0x1433dbcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5513dbd1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c93dbd4, SUC_lexicon_tags_45 },
    { 0x2653dbd5, SUC_lexicon_tags_33 },
    { 0xd173dbd6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3dbd9, SUC_lexicon_tags_14 },
    { 0x8133dbd9, SUC_lexicon_tags_1 },
    { 0xb033dbdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa073dbde, SUC_lexicon_tags_106 },
    { 0x61d3dbde, SUC_lexicon_tags_17 },
    { 0x4573dbe0, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xdcf3dbe2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaa53dbe4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8ab3dbe6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2333dbeb, SUC_lexicon_tags_171 },
    { 0xecd3dbec, SUC_lexicon_tags_17 },
    { 0xf6d3dbed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d3dbf2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff3dbf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea53dbff, SUC_lexicon_tags_31 },
    { 0xf2d3dc00, SUC_lexicon_tags_25 },
    { 0x8c73dc01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x333dc03, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e13dc07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ff3dc09, SUC_lexicon_tags_8 },
    { 0x6853dc09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9553dc0e, SUC_lexicon_tags_17 },
    { 0x1473dc0f, SUC_lexicon_tags_25 },
    { 0xabb3dc10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d3dc15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4513dc17, SUC_lexicon_tags_45 },
    { 0x9553dc18, SUC_lexicon_tags_8 },
    { 0xa1d3dc19, SUC_lexicon_tags_38 },
    { 0x72b3dc19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf3dc1f, SUC_lexicon_tags_7 },
    { 0x9bf3dc20, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f3dc23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe413dc2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8173dc36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9d3dc38, SUC_lexicon_tags_8 },
    { 0x5bf3dc39, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f93dc3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9353dc45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5553dc4a, SUC_lexicon_tags_25 },
    { 0x5333dc4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddb3dc4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd233dc53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe733dc59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b3dc5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3dc5f, SUC_lexicon_tags_37 },
    { 0x29f3dc60, SUC_lexicon_tags_33 },
    { 0xf1f3dc61, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa133dc65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4793dc6b, SUC_lexicon_tags_11 },
    { 0xb353dc6b, SUC_lexicon_tags_49 },
    { 0xffb3dc6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc533dc71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdeb3dc73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6993dc78, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5753dc7a, SUC_lexicon_tags_49 },
    { 0xaf73dc7b, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x5f33dc7d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b3dc80, SUC_lexicon_tags_17 },
    { 0x35b3dc80, SUC_lexicon_tags_13 },
    { 0x90b3dc80, SUC_lexicon_tags_8 },
    { 0xe553dc82, SUC_lexicon_tags_87 },
    { 0x9953dc82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b3dc87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa793dc89, SUC_lexicon_tags_17 },
    { 0x8853dc8a, SUC_lexicon_tags_38 },
    { 0xecf3dc8b, SUC_lexicon_tags_28 },
    { 0x4013dc8b, SUC_lexicon_tags_45 },
    { 0xf233dc8d, SUC_lexicon_tags_25 },
    { 0x8833dc8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd013dc90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3913dc92, SUC_lexicon_tags_25 },
    { 0x3bb3dc92, SUC_lexicon_tags_37 },
    { 0xb093dc92, SUC_lexicon_tags_17 },
    { 0x453dc95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd3dc99, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b3dc9e, SUC_lexicon_tags_49 },
    { 0x6fd3dc9e, SUC_lexicon_tags_23 },
    { 0xb733dc9f, SUC_lexicon_tags_47 },
    { 0x34d3dca1, SUC_lexicon_tags_83 },
    { 0x9133dca2, SUC_lexicon_tags_37 },
    { 0x9093dca1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b13dca6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7493dca9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d3dcb3, SUC_lexicon_tags_43 },
    { 0x1613dcb4, SUC_lexicon_tags_1 },
    { 0x2753dcb3, SUC_lexicon_tags_661 },
    { 0x94f3dcb6, SUC_lexicon_tags_8 },
    { 0x173dcb6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4f73dcb9, SUC_lexicon_tags_151 },
    { 0x3b53dcb9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3dcc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b3dcc9, SUC_lexicon_tags_55 },
    { 0x23f3dcc9, SUC_lexicon_tags_104 },
    { 0x2153dccb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb3dcd0, SUC_lexicon_tags_3 },
    { 0xb753dcd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f3dcd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5893dcd7, SUC_lexicon_tags_23 },
    { 0x8153dcd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d3dce2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd3dce6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e13dce9, SUC_lexicon_tags_17 },
    { 0x3cf3dcea, SUC_lexicon_tags_55 },
    { 0x1413dcea, SUC_lexicon_tags_194 },
    { 0x2db3dcec, SUC_lexicon_tags_7 },
    { 0xb713dcec, SUC_lexicon_tags_12 },
    { 0xccf3dced, SUC_lexicon_tags_11 },
    { 0x5773dcec, SUC_lexicon_tags_12 },
    { 0x9fd3dced, SUC_lexicon_tags_25 },
    { 0x18f3dcf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c53dcf6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2213dcfa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3513dcfc, SUC_lexicon_tags_31 },
    { 0x6113dcfd, SUC_lexicon_tags_318 },
    { 0x3973dcfe, SUC_lexicon_tags_17 },
    { 0x7473dcfc, SUC_lexicon_tags_1 },
    { 0x1773dcfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4893dd03, SUC_lexicon_tags_6 },
    { 0xde13dd04, SUC_lexicon_tags_1 },
    { 0x1853dd04, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7f53dd07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9393dd0a, SUC_lexicon_tags_10 },
    { 0x213dd0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf3dd0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1053dd14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9bb3dd16, SUC_lexicon_tags_1 },
    { 0x9833dd17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ff3dd19, SUC_lexicon_tags_8 },
    { 0x1293dd1a, SUC_lexicon_tags_10 },
    { 0x1313dd1b, SUC_lexicon_tags_26 },
    { 0xf1d3dd1a, SUC_lexicon_tags_1 },
    { 0x16f3dd1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9733dd1f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7533dd21, SUC_lexicon_tags_65 },
    { 0x4c93dd21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33dd25, SUC_lexicon_tags_10 },
    { 0x2293dd26, SUC_lexicon_tags_2 },
    { 0xa793dd27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa013dd2a, SUC_lexicon_tags_82 },
    { 0xc5d3dd2b, SUC_lexicon_tags_25 },
    { 0x8f33dd2a, SUC_lexicon_tags_25 },
    { 0x29d3dd2d, SUC_lexicon_tags_63 },
    { 0xb133dd2b, SUC_lexicon_tags_7 },
    { 0x42f3dd2f, SUC_lexicon_tags_14 },
    { 0xa0d3dd30, SUC_lexicon_tags_2 },
    { 0xeb3dd31, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2ff3dd33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc53dd36, SUC_lexicon_tags_37 },
    { 0x9af3dd37, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d93dd3b, SUC_lexicon_tags_37 },
    { 0x7df3dd3b, SUC_lexicon_tags_29 },
    { 0x9333dd3b, SUC_lexicon_tags_77 },
    { 0x273dd3e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda73dd45, SUC_lexicon_tags_1 },
    { 0x2d33dd46, SUC_lexicon_tags_15 },
    { 0x2ff3dd47, SUC_lexicon_tags_11 },
    { 0x70d3dd46, SUC_lexicon_tags_25 },
    { 0x2d3dd49, SUC_lexicon_tags_13 },
    { 0x2a13dd49, SUC_lexicon_tags_2 },
    { 0x5b13dd4a, SUC_lexicon_tags_21 },
    { 0x2273dd4c, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3dd51, SUC_lexicon_tags_17 },
    { 0x95f3dd51, SUC_lexicon_tags_38 },
    { 0xdb93dd53, SUC_lexicon_tags_11 },
    { 0x8433dd54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d3dd57, SUC_lexicon_tags_12 },
    { 0x7293dd58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa633dd5c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc3d3dd5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe733dd65, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd7d3dd67, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5d13dd69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf413dd6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1b3dd6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c93dd73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad33dd76, SUC_lexicon_tags_13 },
    { 0x253dd76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1ad3dd79, SUC_lexicon_tags_23 },
    { 0xbd33dd79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7953dd7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b33dd7e, SUC_lexicon_tags_5 },
    { 0x4c33dd7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673dd83, SUC_lexicon_tags_15 },
    { 0x41b3dd84, SUC_lexicon_tags_11 },
    { 0x3d93dd84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3453dd88, SUC_lexicon_tags_25 },
    { 0xd453dd89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e13dd8e, SUC_lexicon_tags_7 },
    { 0x4173dd8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b3dd94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b3dd97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f3dd99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf3dd9e, SUC_lexicon_tags_177 },
    { 0xdd3dd9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x29f3dda1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe93dda5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6293ddad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb553ddb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d3ddb8, SUC_lexicon_tags_37 },
    { 0xa3d3ddb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad93ddbd, SUC_lexicon_tags_178 },
    { 0x6773ddbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f3ddc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda93ddc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d3ddca, SUC_lexicon_tags_112 },
    { 0xdc53ddcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b3ddd0, SUC_lexicon_tags_11 },
    { 0x5093ddd0, SUC_lexicon_tags_15 },
    { 0xa2f3ddd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b3ddd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3dddb, SUC_lexicon_tags_48 },
    { 0xc133dddb, SUC_lexicon_tags_12 },
    { 0xba93dddd, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253dde2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3dde6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc853dde9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd13ddf0, SUC_lexicon_tags_57 },
    { 0xbe33ddf1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1f3ddf3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed73ddf5, SUC_lexicon_tags_1 },
    { 0x1073ddf6, SUC_lexicon_tags_33 },
    { 0xee13ddf7, SUC_lexicon_tags_68 },
    { 0x8713ddf6, SUC_lexicon_tags_11 },
    { 0x3893ddf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x273ddfb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f3de05, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e93de09, SUC_lexicon_tags_1 },
    { 0x4093de0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf3de12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc953de14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1333de17, SUC_lexicon_tags_1 },
    { 0xfd73de18, SUC_lexicon_tags_7 },
    { 0x3313de19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5193de21, SUC_lexicon_tags_27 },
    { 0x79b3de22, SUC_lexicon_tags_61 },
    { 0x3573de23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa13de25, SUC_lexicon_tags_7 },
    { 0x8a13de26, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2e53de28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa33de2a, SUC_lexicon_tags_38 },
    { 0x89f3de2a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a53de2f, SUC_lexicon_tags_7 },
    { 0x6e93de30, SUC_lexicon_tags_38 },
    { 0x2453de31, SUC_lexicon_tags_63 },
    { 0xffb3de31, SUC_lexicon_tags_2 },
    { 0xe4f3de33, SUC_lexicon_tags_37 },
    { 0x3973de33, SUC_lexicon_tags_35 },
    { 0x3eb3de34, SUC_lexicon_tags_1 },
    { 0x6973de35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xba33de38, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x96b3de3a, SUC_lexicon_tags_23 },
    { 0xc853de3b, SUC_lexicon_tags_10 },
    { 0x4853de3c, SUC_lexicon_tags_6 },
    { 0xb853de3d, SUC_lexicon_tags_19 },
    { 0xa93de3c, SUC_lexicon_tags_0 },
    { 0x8993de3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf3de45, SUC_lexicon_tags_16 },
    { 0x7ed3de45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7913de48, SUC_lexicon_tags_2 },
    { 0x4633de49, SUC_lexicon_tags_29 },
    { 0x37b3de49, SUC_lexicon_tags_19 },
    { 0x6813de4b, SUC_lexicon_tags_87 },
    { 0x3893de4a, SUC_lexicon_tags_8 },
    { 0x4813de4d, SUC_lexicon_tags_0 },
    { 0x3f3de4c, SUC_lexicon_tags_37 },
    { 0x7bf3de4e, SUC_lexicon_tags_63 },
    { 0xcb13de50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7433de52, SUC_lexicon_tags_39 },
    { 0x5af3de53, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2393de55, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf613de59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5953de5b, SUC_lexicon_tags_47 },
    { 0x2ed3de5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3713de61, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa493de64, SUC_lexicon_tags_17 },
    { 0x90d3de64, SUC_lexicon_tags_15 },
    { 0x14f3de64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4273de68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf3de6b, SUC_lexicon_tags_84 },
    { 0x6fd3de6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f93de72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f3de77, SUC_lexicon_tags_25 },
    { 0x4af3de78, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5153de7b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x173de7e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd333de83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48f3de85, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b73de90, SUC_lexicon_tags_33 },
    { 0x8fb3de91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9933de93, SUC_lexicon_tags_27 },
    { 0x6b73de93, SUC_lexicon_tags_32 },
    { 0x8bf3de93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c13de98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8853de9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x433dea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef3dea4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b3dea9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea93deac, SUC_lexicon_tags_2 },
    { 0x6293dead, SUC_lexicon_tags_25 },
    { 0x7173deac, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb893deb7, SUC_lexicon_tags_11 },
    { 0x9233deb7, SUC_lexicon_tags_37 },
    { 0xdf53deb9, SUC_lexicon_tags_2 },
    { 0x13f3deb9, SUC_lexicon_tags_19 },
    { 0xb813debb, SUC_lexicon_tags_60 },
    { 0xdd53debb, SUC_lexicon_tags_0 },
    { 0xf5f3debc, SUC_lexicon_tags_11 },
    { 0x15f3deba, SUC_lexicon_tags_11 },
    { 0x5ad3debf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9993dec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb093dec9, SUC_lexicon_tags_15 },
    { 0xf973deca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9d53decc, SUC_lexicon_tags_37 },
    { 0x2f53decc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b3ded1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c93ded7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2953ded9, SUC_lexicon_tags_46 },
    { 0x4c93deda, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53dedf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd233dee2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d53dee9, SUC_lexicon_tags_33 },
    { 0x14f3deea, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d3def1, SUC_lexicon_tags_1 },
    { 0x21b3def1, SUC_lexicon_tags_114 },
    { 0x4d93def2, SUC_lexicon_tags_4 },
    { 0xe833def1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f3defb, SUC_lexicon_tags_11 },
    { 0x4ab3defc, SUC_lexicon_tags_37 },
    { 0xa073defd, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f3df05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc813df0d, SUC_lexicon_tags_45 },
    { 0x2d53df0d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8a13df10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb3df13, SUC_lexicon_tags_21 },
    { 0x7873df14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd3df17, SUC_lexicon_tags_28 },
    { 0x59b3df18, SUC_lexicon_tags_62 },
    { 0x4233df19, SUC_lexicon_tags_15 },
    { 0xbc33df1a, SUC_lexicon_tags_33 },
    { 0xbfd3df18, SUC_lexicon_tags_11 },
    { 0xa7f3df1a, SUC_lexicon_tags_17 },
    { 0x5553df1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c73df24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6753df29, SUC_lexicon_tags_17 },
    { 0xccb3df29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f33df32, SUC_lexicon_tags_12 },
    { 0x9bb3df33, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3ff3df35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd293df39, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2513df3e, SUC_lexicon_tags_1 },
    { 0x1473df3f, SUC_lexicon_tags_11 },
    { 0xa513df3e, SUC_lexicon_tags_45 },
    { 0xaaf3df3e, SUC_lexicon_tags_60 },
    { 0x4273df3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16f3df44, SUC_lexicon_tags_26 },
    { 0x9173df45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f93df4b, SUC_lexicon_tags_794 },
    { 0x88f3df4b, SUC_lexicon_tags_47 },
    { 0xf713df4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58d3df4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f3df52, SUC_lexicon_tags_6 },
    { 0x61b3df52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff3df56, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef3df59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b93df5e, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4653df62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f93df67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e53df6b, SUC_lexicon_tags_25 },
    { 0x8a93df6c, SUC_lexicon_tags_12 },
    { 0x4d93df6d, SUC_lexicon_tags_23 },
    { 0x7eb3df6b, SUC_lexicon_tags_37 },
    { 0x9073df6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d3df73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45b3df75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3093df78, SUC_lexicon_tags_17 },
    { 0x9e73df78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a93df7d, SUC_lexicon_tags_47 },
    { 0xb633df7e, SUC_lexicon_tags_44 },
    { 0x2c93df7e, SUC_lexicon_tags_37 },
    { 0x7933df7f, SUC_lexicon_tags_16 },
    { 0x97d3df80, SUC_lexicon_tags_29 },
    { 0x6513df82, SUC_lexicon_tags_25 },
    { 0x33d3df7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af3df8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4593df91, SUC_lexicon_tags_37 },
    { 0x8733df92, SUC_lexicon_tags_25 },
    { 0xc1b3df93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf3df95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9353df98, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6913df9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7893dfa1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2e33dfa3, SUC_lexicon_tags_33 },
    { 0x4f93dfa4, SUC_lexicon_tags_1 },
    { 0xf7d3dfa5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb453dfa7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff3dfaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b3dfb4, SUC_lexicon_tags_37 },
    { 0x10f3dfb5, SUC_lexicon_tags_7 },
    { 0x4033dfb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8453dfc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcad3dfc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3d3dfc5, SUC_lexicon_tags_27 },
    { 0x24d3dfc5, SUC_lexicon_tags_37 },
    { 0x8a33dfc7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9433dfcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf133dfd0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4793dfd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb013dfd7, SUC_lexicon_tags_2 },
    { 0xe1b3dfd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4493dfda, SUC_lexicon_tags_21 },
    { 0x41f3dfdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb453dfe1, SUC_lexicon_tags_37 },
    { 0x6653dfe2, SUC_lexicon_tags_37 },
    { 0xca33dfe3, SUC_lexicon_tags_15 },
    { 0xcc13dfe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa193dfe7, SUC_lexicon_tags_201 },
    { 0x3c13dfe8, SUC_lexicon_tags_66 },
    { 0x37b3dfe9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3073dfef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6373dff2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3253dff5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x11d3dff7, SUC_lexicon_tags_8 },
    { 0x1bf3dff7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb3dffe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc13e000, SUC_lexicon_tags_37 },
    { 0x5333e001, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f3e006, SUC_lexicon_tags_33 },
    { 0x5733e007, SUC_lexicon_tags_19 },
    { 0x833e007, SUC_lexicon_tags_17 },
    { 0xcff3e009, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce93e00b, SUC_lexicon_tags_47 },
    { 0x93d3e00b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd0d3e00e, SUC_lexicon_tags_33 },
    { 0xdbf3e00f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413e019, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d3e01f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b3e022, SUC_lexicon_tags_23 },
    { 0x4813e023, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa093e025, SUC_lexicon_tags_11 },
    { 0x7bb3e026, SUC_lexicon_tags_37 },
    { 0x82b3e025, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cd3e029, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d3e02d, SUC_lexicon_tags_26 },
    { 0xc853e02d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b93e030, SUC_lexicon_tags_12 },
    { 0x90f3e031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc93e035, SUC_lexicon_tags_38 },
    { 0x6c73e036, SUC_lexicon_tags_3 },
    { 0xbe93e037, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb113e039, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d3e03d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c73e03f, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5593e043, SUC_lexicon_tags_2 },
    { 0x8c93e044, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d3e04a, SUC_lexicon_tags_47 },
    { 0xaf3e04b, SUC_lexicon_tags_114 },
    { 0xf093e04a, SUC_lexicon_tags_7 },
    { 0xcfd3e04a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9f3e04f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb393e053, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb793e056, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f3e059, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee73e063, SUC_lexicon_tags_0 },
    { 0x2673e064, SUC_lexicon_tags_47 },
    { 0xa333e065, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5953e067, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5493e06a, SUC_lexicon_tags_25 },
    { 0xcc53e06a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5893e06f, SUC_lexicon_tags_69 },
    { 0x3753e070, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb33e073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbd3e075, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e93e07d, SUC_lexicon_tags_29 },
    { 0xf3b3e07d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbad3e080, SUC_lexicon_tags_11 },
    { 0xbd13e081, SUC_lexicon_tags_39 },
    { 0xbc33e081, SUC_lexicon_tags_39 },
    { 0x1533e083, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf73e089, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf213e08b, SUC_lexicon_tags_37 },
    { 0x2633e08c, SUC_lexicon_tags_37 },
    { 0x50b3e08d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb153e093, SUC_lexicon_tags_9 },
    { 0x7e13e094, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf153e097, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4533e09a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabd3e09c, SUC_lexicon_tags_47 },
    { 0x8573e09d, SUC_lexicon_tags_31 },
    { 0xb713e09c, SUC_lexicon_tags_37 },
    { 0xe053e09d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c53e0a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4333e0a5, SUC_lexicon_tags_11 },
    { 0x24f3e0a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53e0a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1d3e0ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73e0b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7933e0b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b93e0ba, SUC_lexicon_tags_55 },
    { 0xd033e0bb, SUC_lexicon_tags_23 },
    { 0xf3d3e0bb, SUC_lexicon_tags_17 },
    { 0x6293e0bc, SUC_lexicon_tags_29 },
    { 0xd73e0be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8753e0c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff3e0c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33e0c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed3e0ce, SUC_lexicon_tags_37 },
    { 0x2633e0ce, SUC_lexicon_tags_12 },
    { 0x4b13e0cf, SUC_lexicon_tags_16 },
    { 0xbf93e0d1, SUC_lexicon_tags_17 },
    { 0xb5b3e0d2, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d3e0d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db3e0db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4053e0de, SUC_lexicon_tags_17 },
    { 0xd213e0df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6353e0ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbed3e0ec, SUC_lexicon_tags_46 },
    { 0x6073e0ed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff3e0f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb553e0f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1233e100, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd3e104, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb3e107, SUC_lexicon_tags_34 },
    { 0x7b73e107, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea13e10b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f3e10d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf973e111, SUC_lexicon_tags_28 },
    { 0xbbf3e112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b3e119, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d3e126, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5233e12b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3073e12d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf93e12f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d73e134, SUC_lexicon_tags_11 },
    { 0xd813e135, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b73e139, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77f3e13b, SUC_lexicon_tags_7 },
    { 0x88b3e13c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd3e141, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa073e144, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8413e146, SUC_lexicon_tags_8 },
    { 0xdff3e147, SUC_lexicon_tags_11 },
    { 0x89f3e148, SUC_lexicon_tags_33 },
    { 0x8733e149, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4233e14d, SUC_lexicon_tags_11 },
    { 0x6dd3e14d, SUC_lexicon_tags_39 },
    { 0x813e14f, SUC_lexicon_tags_4 },
    { 0xbef3e14f, SUC_lexicon_tags_17 },
    { 0x71f3e151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2593e153, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f3e15c, SUC_lexicon_tags_87 },
    { 0x4833e15c, SUC_lexicon_tags_62 },
    { 0xaed3e15e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5cd3e160, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7233e162, SUC_lexicon_tags_85 },
    { 0xfa53e163, SUC_lexicon_tags_21 },
    { 0xc1d3e164, SUC_lexicon_tags_11 },
    { 0x5cd3e163, SUC_lexicon_tags_25 },
    { 0x80f3e166, SUC_lexicon_tags_65 },
    { 0xe413e167, SUC_lexicon_tags_27 },
    { 0xb3f3e167, SUC_lexicon_tags_572 },
    { 0x1273e169, SUC_lexicon_tags_33 },
    { 0x5373e165, SUC_lexicon_tags_55 },
    { 0x7533e16b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a33e16e, SUC_lexicon_tags_8 },
    { 0xdc13e16f, SUC_lexicon_tags_12 },
    { 0x2f93e16e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c53e173, SUC_lexicon_tags_15 },
    { 0xbcf3e174, SUC_lexicon_tags_2 },
    { 0x3453e173, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff3e17a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d3e17e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa953e183, SUC_lexicon_tags_37 },
    { 0xc253e184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4453e186, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6573e190, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8193e196, SUC_lexicon_tags_1 },
    { 0x4733e197, SUC_lexicon_tags_49 },
    { 0xdb93e198, SUC_lexicon_tags_7 },
    { 0x193e196, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8873e19d, SUC_lexicon_tags_37 },
    { 0x6313e19e, SUC_lexicon_tags_16 },
    { 0x6a13e19e, SUC_lexicon_tags_190 },
    { 0x1ff3e1a0, SUC_lexicon_tags_27 },
    { 0x9f13e1a0, SUC_lexicon_tags_19 },
    { 0x9b13e1a2, SUC_lexicon_tags_6 },
    { 0xd173e19e, SUC_lexicon_tags_30 },
    { 0x6913e1a2, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xb833e1a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c13e1ad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa573e1af, SUC_lexicon_tags_37 },
    { 0xacb3e1b0, SUC_lexicon_tags_7 },
    { 0xd153e1af, SUC_lexicon_tags_25 },
    { 0xb833e1af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d93e1b5, SUC_lexicon_tags_0 },
    { 0x3033e1b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a13e1b8, SUC_lexicon_tags_7 },
    { 0xc3b3e1b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb53e1bb, SUC_lexicon_tags_23 },
    { 0x4c73e1bc, SUC_lexicon_tags_37 },
    { 0xfb3e1bb, SUC_lexicon_tags_15 },
    { 0x9cd3e1be, SUC_lexicon_tags_11 },
    { 0xf973e1be, SUC_lexicon_tags_15 },
    { 0xa013e1c0, SUC_lexicon_tags_65 },
    { 0xdeb3e1c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc933e1c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa953e1c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d3e1ca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d3e1cd, SUC_lexicon_tags_45 },
    { 0x57d3e1ce, SUC_lexicon_tags_11 },
    { 0xad33e1cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2613e1d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8653e1d3, SUC_lexicon_tags_43 },
    { 0x1bb3e1d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2193e1d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda93e1db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb3e1df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d3e1e9, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe033e1ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7053e1f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc393e1fd, SUC_lexicon_tags_15 },
    { 0xc413e1fe, SUC_lexicon_tags_8 },
    { 0x6f73e1fe, SUC_lexicon_tags_27 },
    { 0x5473e1ff, SUC_lexicon_tags_12 },
    { 0xe133e1fe, SUC_lexicon_tags_26 },
    { 0x393e1fe, SUC_lexicon_tags_9 },
    { 0xdf53e1fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8933e209, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f3e210, SUC_lexicon_tags_7 },
    { 0x92f3e210, SUC_lexicon_tags_31 },
    { 0x27f3e212, SUC_lexicon_tags_12 },
    { 0xb853e213, SUC_lexicon_tags_12 },
    { 0x4373e214, SUC_lexicon_tags_11 },
    { 0x6b3e215, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f3e219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d3e21d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a13e221, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe033e223, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x81d3e225, SUC_lexicon_tags_19 },
    { 0xe93e226, SUC_lexicon_tags_11 },
    { 0xcad3e227, SUC_lexicon_tags_1 },
    { 0x7213e228, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f3e22b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1313e231, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1193e237, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa613e23a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d3e23e, SUC_lexicon_tags_12 },
    { 0xf413e23f, SUC_lexicon_tags_6 },
    { 0x1e93e240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfef3e242, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f3e247, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf373e24f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd113e252, SUC_lexicon_tags_63 },
    { 0x73b3e253, SUC_lexicon_tags_33 },
    { 0x68f3e253, SUC_lexicon_tags_11 },
    { 0x8a93e255, SUC_lexicon_tags_12 },
    { 0x47b3e256, SUC_lexicon_tags_8 },
    { 0xdf93e257, SUC_lexicon_tags_7 },
    { 0x8473e258, SUC_lexicon_tags_7 },
    { 0xf373e259, SUC_lexicon_tags_6 },
    { 0x89d3e257, SUC_lexicon_tags_13 },
    { 0xbbf3e256, SUC_lexicon_tags_11 },
    { 0x97f3e259, SUC_lexicon_tags_45 },
    { 0xe973e25c, SUC_lexicon_tags_23 },
    { 0x84d3e256, SUC_lexicon_tags_21 },
    { 0xfeb3e25e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e13e262, SUC_lexicon_tags_174 },
    { 0x8c13e263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6573e267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db3e273, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b3e276, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x10d3e278, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb353e27a, SUC_lexicon_tags_19 },
    { 0x4613e27b, SUC_lexicon_tags_60 },
    { 0xc573e27c, SUC_lexicon_tags_28 },
    { 0x55b3e27d, SUC_lexicon_tags_6 },
    { 0x3c73e27e, SUC_lexicon_tags_33 },
    { 0x6433e27e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd3e285, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e33e28a, SUC_lexicon_tags_17 },
    { 0x3973e28b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9f3e28d, SUC_lexicon_tags_7 },
    { 0xfeb3e28d, SUC_lexicon_tags_26 },
    { 0xa013e28d, SUC_lexicon_tags_63 },
    { 0xbb13e290, SUC_lexicon_tags_19 },
    { 0x3573e290, SUC_lexicon_tags_45 },
    { 0x8c13e292, SUC_lexicon_tags_11 },
    { 0xbc33e293, SUC_lexicon_tags_30 },
    { 0x7233e293, SUC_lexicon_tags_37 },
    { 0x8873e293, SUC_lexicon_tags_30 },
    { 0xea13e28e, SUC_lexicon_tags_63 },
    { 0xc1d3e290, SUC_lexicon_tags_2 },
    { 0x51b3e28e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef93e29c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3e2a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7933e2a6, SUC_lexicon_tags_17 },
    { 0xe353e2a7, SUC_lexicon_tags_33 },
    { 0xf073e2a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc113e2aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xff73e2ac, SUC_lexicon_tags_2 },
    { 0x7a13e2ad, SUC_lexicon_tags_11 },
    { 0xd953e2ae, SUC_lexicon_tags_65 },
    { 0x60f3e2af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f3e2b1, SUC_lexicon_tags_1 },
    { 0x7c93e2b1, SUC_lexicon_tags_27 },
    { 0x2273e2b2, SUC_lexicon_tags_37 },
    { 0xb453e2b1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b53e2be, SUC_lexicon_tags_11 },
    { 0x6af3e2bf, SUC_lexicon_tags_38 },
    { 0x2d13e2bf, SUC_lexicon_tags_15 },
    { 0x2653e2c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9013e2c4, SUC_lexicon_tags_1 },
    { 0xda73e2c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d3e2ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6713e2cc, SUC_lexicon_tags_63 },
    { 0x7273e2cd, SUC_lexicon_tags_15 },
    { 0x99b3e2ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c93e2d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf4d3e2d5, SUC_lexicon_tags_15 },
    { 0x4813e2d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff3e2de, SUC_lexicon_tags_10 },
    { 0x2753e2df, SUC_lexicon_tags_17 },
    { 0x54f3e2e0, SUC_lexicon_tags_11 },
    { 0xfbd3e2df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc3f3e2e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2753e2e6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc33e2e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4e33e2eb, SUC_lexicon_tags_48 },
    { 0xe3d3e2ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1d3e2ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1433e2f0, SUC_lexicon_tags_63 },
    { 0x36f3e2f0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c73e2f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x30b3e2f6, SUC_lexicon_tags_37 },
    { 0xe3b3e2f7, SUC_lexicon_tags_38 },
    { 0xde93e2f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db3e2fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa313e300, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec53e30c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1453e30e, SUC_lexicon_tags_38 },
    { 0x8153e30f, SUC_lexicon_tags_1 },
    { 0xe6f3e30f, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9153e315, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf53e318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1353e31a, SUC_lexicon_tags_12 },
    { 0xf1d3e31b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x573e31d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1233e31f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f53e323, SUC_lexicon_tags_10 },
    { 0x78b3e324, SUC_lexicon_tags_23 },
    { 0x2e93e325, SUC_lexicon_tags_10 },
    { 0x11f3e326, SUC_lexicon_tags_26 },
    { 0x2e73e327, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973e32f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d3e334, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1653e33a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c33e33c, SUC_lexicon_tags_17 },
    { 0xcf3e33c, SUC_lexicon_tags_29 },
    { 0xfe33e33e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf13e340, SUC_lexicon_tags_57 },
    { 0x1e73e340, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2873e344, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd3e348, SUC_lexicon_tags_3 },
    { 0xbeb3e348, SUC_lexicon_tags_68 },
    { 0xea33e34a, SUC_lexicon_tags_25 },
    { 0xa0f3e34b, SUC_lexicon_tags_11 },
    { 0x86d3e34c, SUC_lexicon_tags_11 },
    { 0xec13e34d, SUC_lexicon_tags_15 },
    { 0x5f3e34d, SUC_lexicon_tags_1404 },
    { 0xee33e34e, SUC_lexicon_tags_1 },
    { 0xd33e350, SUC_lexicon_tags_48 },
    { 0x80b3e349, SUC_lexicon_tags_29 },
    { 0xc513e352, SUC_lexicon_tags_47 },
    { 0x7053e352, SUC_lexicon_tags_11 },
    { 0xc533e352, SUC_lexicon_tags_43 },
    { 0x513e352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa093e357, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d3e35d, SUC_lexicon_tags_38 },
    { 0xf293e35d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa293e363, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd3e366, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc93e36c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8b33e36e, SUC_lexicon_tags_8 },
    { 0xb53e36f, SUC_lexicon_tags_1 },
    { 0x5633e36f, SUC_lexicon_tags_63 },
    { 0xa4d3e371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b3e374, SUC_lexicon_tags_2 },
    { 0x1953e374, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f3e37b, SUC_lexicon_tags_33 },
    { 0x5d13e37b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfd3e37e, SUC_lexicon_tags_33 },
    { 0xe913e37f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe333e381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d73e384, SUC_lexicon_tags_47 },
    { 0xf1d3e385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7773e38c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3533e391, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af3e396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9093e39c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5993e3a1, SUC_lexicon_tags_160 },
    { 0xd153e3a2, SUC_lexicon_tags_66 },
    { 0xe153e3a3, SUC_lexicon_tags_2 },
    { 0x48d3e3a1, SUC_lexicon_tags_17 },
    { 0xf713e3a5, SUC_lexicon_tags_17 },
    { 0x5313e3a4, SUC_lexicon_tags_12 },
    { 0x4973e3a3, SUC_lexicon_tags_15 },
    { 0x7653e3a7, SUC_lexicon_tags_1 },
    { 0xdd3e3a9, SUC_lexicon_tags_7 },
    { 0x1593e3a2, SUC_lexicon_tags_37 },
    { 0x1593e3ab, SUC_lexicon_tags_19 },
    { 0xef93e3ac, SUC_lexicon_tags_11 },
    { 0x41d3e3ad, SUC_lexicon_tags_43 },
    { 0x14f3e3a1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d3e3b5, SUC_lexicon_tags_2 },
    { 0x7dd3e3b6, SUC_lexicon_tags_11 },
    { 0x8f13e3b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d3e3bc, SUC_lexicon_tags_3 },
    { 0xd2d3e3bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df3e3c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d13e3c5, SUC_lexicon_tags_63 },
    { 0xf6d3e3c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed3e3c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b3e3cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc053e3ce, SUC_lexicon_tags_17 },
    { 0x81f3e3ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b93e3d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x84d3e3d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd573e3d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcdb3e3d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413e3dd, SUC_lexicon_tags_12 },
    { 0xe6d3e3de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d73e3e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d73e3e8, SUC_lexicon_tags_21 },
    { 0x45b3e3e8, SUC_lexicon_tags_27 },
    { 0x80b3e3ea, SUC_lexicon_tags_7 },
    { 0x2513e3e9, SUC_lexicon_tags_45 },
    { 0xd813e3ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc733e3ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f3e3f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f53e3f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf33e3ff, SUC_lexicon_tags_11 },
    { 0x2433e400, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x713e402, SUC_lexicon_tags_37 },
    { 0x813e402, SUC_lexicon_tags_8 },
    { 0xdbf3e404, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8f13e406, SUC_lexicon_tags_252 },
    { 0x6153e406, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb3e40a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdf3e40c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d3e411, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c33e414, SUC_lexicon_tags_47 },
    { 0xe953e415, SUC_lexicon_tags_55 },
    { 0x55f3e416, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b3e41d, SUC_lexicon_tags_7 },
    { 0xb113e41e, SUC_lexicon_tags_1 },
    { 0x1553e41e, SUC_lexicon_tags_0 },
    { 0x45f3e420, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f3e422, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4833e424, SUC_lexicon_tags_17 },
    { 0xd433e425, SUC_lexicon_tags_47 },
    { 0x7893e426, SUC_lexicon_tags_60 },
    { 0x8133e424, SUC_lexicon_tags_47 },
    { 0x69b3e428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c93e42a, SUC_lexicon_tags_35 },
    { 0x1f93e42a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd533e430, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c93e433, SUC_lexicon_tags_0 },
    { 0x6453e434, SUC_lexicon_tags_0 },
    { 0xee53e434, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1553e437, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8893e439, SUC_lexicon_tags_33 },
    { 0x6b13e43a, SUC_lexicon_tags_19 },
    { 0x3a73e439, SUC_lexicon_tags_12 },
    { 0xb793e43c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893e443, SUC_lexicon_tags_14 },
    { 0x83b3e443, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2553e448, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d3e44d, SUC_lexicon_tags_25 },
    { 0xa0f3e44e, SUC_lexicon_tags_11 },
    { 0x5fd3e44f, SUC_lexicon_tags_23 },
    { 0x8133e44d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1593e457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2d3e459, SUC_lexicon_tags_43 },
    { 0x3e33e45a, SUC_lexicon_tags_12 },
    { 0xa7b3e45b, SUC_lexicon_tags_69 },
    { 0x8113e45a, SUC_lexicon_tags_17 },
    { 0x6d73e45d, SUC_lexicon_tags_19 },
    { 0xecd3e45b, SUC_lexicon_tags_14 },
    { 0x5173e45f, SUC_lexicon_tags_40 },
    { 0x93b3e460, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9633e462, SUC_lexicon_tags_46 },
    { 0x2fd3e463, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7893e469, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b3e46d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa2f3e46f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d3e473, SUC_lexicon_tags_1 },
    { 0x4b3e474, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6993e478, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac53e47e, SUC_lexicon_tags_17 },
    { 0xd113e47f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f13e483, SUC_lexicon_tags_21 },
    { 0x9a33e484, SUC_lexicon_tags_55 },
    { 0x6df3e483, SUC_lexicon_tags_12 },
    { 0x1a33e484, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf3e48a, SUC_lexicon_tags_1 },
    { 0xd2b3e48a, SUC_lexicon_tags_17 },
    { 0x2433e48c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d3e490, SUC_lexicon_tags_7 },
    { 0xe3f3e490, SUC_lexicon_tags_39 },
    { 0xbe73e490, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea33e49b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc13e49d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af3e4a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb93e4a9, SUC_lexicon_tags_11 },
    { 0xbb3e4aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf993e4ad, SUC_lexicon_tags_25 },
    { 0x2d13e4ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf433e4bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb013e4bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b3e4c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1133e4c8, SUC_lexicon_tags_25 },
    { 0x7453e4c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7793e4cc, SUC_lexicon_tags_11 },
    { 0x34f3e4cd, SUC_lexicon_tags_15 },
    { 0xd3d3e4ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14f3e4d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xca73e4d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4173e4d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65b3e4d7, SUC_lexicon_tags_23 },
    { 0x5413e4d7, SUC_lexicon_tags_9 },
    { 0xc3d3e4d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f93e4dd, SUC_lexicon_tags_75 },
    { 0x29d3e4de, SUC_lexicon_tags_70 },
    { 0xaf13e4de, SUC_lexicon_tags_1 },
    { 0x5d13e4df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b3e4e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c33e4e5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3553e4ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f3e4f2, SUC_lexicon_tags_11 },
    { 0xe033e4f3, SUC_lexicon_tags_1 },
    { 0x9fb3e4f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33d3e4f6, SUC_lexicon_tags_33 },
    { 0x3d73e4f7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x47d3e4f9, SUC_lexicon_tags_11 },
    { 0x2273e4f9, SUC_lexicon_tags_17 },
    { 0xb113e4fb, SUC_lexicon_tags_32 },
    { 0x6f73e4fc, SUC_lexicon_tags_19 },
    { 0x1953e4fd, SUC_lexicon_tags_38 },
    { 0x2953e4fb, SUC_lexicon_tags_32 },
    { 0x40d3e4fe, SUC_lexicon_tags_11 },
    { 0x71d3e4fd, SUC_lexicon_tags_11 },
    { 0x1533e501, SUC_lexicon_tags_25 },
    { 0x52d3e500, SUC_lexicon_tags_11 },
    { 0x5a33e500, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5813e505, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa373e507, SUC_lexicon_tags_15 },
    { 0x1413e507, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd3e50b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3133e50f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb13e513, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b3e517, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33e51c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8793e521, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b3e525, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96b3e527, SUC_lexicon_tags_15 },
    { 0xf873e527, SUC_lexicon_tags_33 },
    { 0x1a33e528, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67d3e52b, SUC_lexicon_tags_15 },
    { 0xdcb3e52c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20d3e52e, SUC_lexicon_tags_14 },
    { 0x7cf3e52f, SUC_lexicon_tags_43 },
    { 0x8d93e530, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32b3e532, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x20d3e534, SUC_lexicon_tags_63 },
    { 0xc4f3e534, SUC_lexicon_tags_15 },
    { 0xdb73e534, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6733e53a, SUC_lexicon_tags_61 },
    { 0xa933e53b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8993e53d, SUC_lexicon_tags_34 },
    { 0xe6d3e53d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b73e541, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3813e547, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe273e549, SUC_lexicon_tags_27 },
    { 0x9b3e54a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb093e54d, SUC_lexicon_tags_14 },
    { 0x7d73e54e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5e93e550, SUC_lexicon_tags_1085 },
    { 0, NULL },
    { 0xc713e552, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb93e556, SUC_lexicon_tags_48 },
    { 0x34b3e556, SUC_lexicon_tags_29 },
    { 0xc733e558, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2933e55a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea33e55e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4753e560, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2073e563, SUC_lexicon_tags_65 },
    { 0x6c73e564, SUC_lexicon_tags_43 },
    { 0x24f3e563, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2733e567, SUC_lexicon_tags_47 },
    { 0x8513e568, SUC_lexicon_tags_31 },
    { 0x7f33e567, SUC_lexicon_tags_28 },
    { 0x25b3e56a, SUC_lexicon_tags_45 },
    { 0xcff3e56b, SUC_lexicon_tags_33 },
    { 0x8653e56c, SUC_lexicon_tags_12 },
    { 0xb613e56c, SUC_lexicon_tags_84 },
    { 0x913e56e, SUC_lexicon_tags_63 },
    { 0x3293e56e, SUC_lexicon_tags_43 },
    { 0x153e570, SUC_lexicon_tags_1 },
    { 0xca33e571, SUC_lexicon_tags_14 },
    { 0x3813e572, SUC_lexicon_tags_68 },
    { 0x6c53e573, SUC_lexicon_tags_16 },
    { 0x53d3e572, SUC_lexicon_tags_7 },
    { 0x673e574, SUC_lexicon_tags_6 },
    { 0xed3e56d, SUC_lexicon_tags_49 },
    { 0x58f3e577, SUC_lexicon_tags_29 },
    { 0x4a93e577, SUC_lexicon_tags_45 },
    { 0x93d3e579, SUC_lexicon_tags_2 },
    { 0x9bd3e578, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b3e57d, SUC_lexicon_tags_77 },
    { 0xfb33e57d, SUC_lexicon_tags_46 },
    { 0xd6b3e57d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9733e584, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded3e588, SUC_lexicon_tags_7 },
    { 0xe7b3e588, SUC_lexicon_tags_63 },
    { 0xf153e58a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a73e58c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9af3e58e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6d3e590, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353e594, SUC_lexicon_tags_11 },
    { 0x7213e595, SUC_lexicon_tags_25 },
    { 0xb213e595, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b3e599, SUC_lexicon_tags_30 },
    { 0x5853e599, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90b3e59c, SUC_lexicon_tags_37 },
    { 0x14b3e59d, SUC_lexicon_tags_87 },
    { 0xa9b3e59d, SUC_lexicon_tags_17 },
    { 0x6513e59f, SUC_lexicon_tags_0 },
    { 0x5953e5a0, SUC_lexicon_tags_31 },
    { 0xe453e5a1, SUC_lexicon_tags_7 },
    { 0x5db3e5a2, SUC_lexicon_tags_39 },
    { 0xc593e5a1, SUC_lexicon_tags_17 },
    { 0x6a53e5a0, SUC_lexicon_tags_25 },
    { 0xfe93e5a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ff3e5a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6d3e5a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb933e5ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e93e5ae, SUC_lexicon_tags_61 },
    { 0x8193e5af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e13e5b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d3e5b3, SUC_lexicon_tags_30 },
    { 0x2b93e5b3, SUC_lexicon_tags_0 },
    { 0xef73e5b5, SUC_lexicon_tags_68 },
    { 0xa7b3e5b3, SUC_lexicon_tags_60 },
    { 0xcf3e5b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c93e5bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x33d3e5be, SUC_lexicon_tags_0 },
    { 0xe93e5bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf3e5c7, SUC_lexicon_tags_6 },
    { 0xfd73e5c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd3e5cb, SUC_lexicon_tags_25 },
    { 0x7633e5cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe513e5d6, SUC_lexicon_tags_2 },
    { 0x3333e5d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d3e5e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a13e5e4, SUC_lexicon_tags_17 },
    { 0xf653e5e5, SUC_lexicon_tags_12 },
    { 0x2813e5e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b3e5e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa333e5ef, SUC_lexicon_tags_49 },
    { 0x4253e5f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38b3e5f2, SUC_lexicon_tags_12 },
    { 0xe253e5f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433e5fb, SUC_lexicon_tags_84 },
    { 0x2df3e5fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793e600, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6733e602, SUC_lexicon_tags_88 },
    { 0x1c93e603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a53e609, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec93e60b, SUC_lexicon_tags_2 },
    { 0xcad3e60b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f73e611, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6053e615, SUC_lexicon_tags_38 },
    { 0x2ef3e615, SUC_lexicon_tags_63 },
    { 0x5df3e617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc793e619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7893e61c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6713e623, SUC_lexicon_tags_47 },
    { 0x2eb3e623, SUC_lexicon_tags_15 },
    { 0xdfb3e625, SUC_lexicon_tags_11 },
    { 0xf213e625, SUC_lexicon_tags_68 },
    { 0x6c13e627, SUC_lexicon_tags_1 },
    { 0x8bf3e623, SUC_lexicon_tags_37 },
    { 0x95b3e626, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd593e630, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4253e63a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c33e63d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f3e641, SUC_lexicon_tags_12 },
    { 0xa93e641, SUC_lexicon_tags_32 },
    { 0xb6b3e643, SUC_lexicon_tags_17 },
    { 0xfa33e641, SUC_lexicon_tags_49 },
    { 0xeab3e644, SUC_lexicon_tags_32 },
    { 0x5c53e646, SUC_lexicon_tags_38 },
    { 0x2c73e647, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe113e649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d3e64c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53e64f, SUC_lexicon_tags_6 },
    { 0x86d3e650, SUC_lexicon_tags_1 },
    { 0x8213e651, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f3e654, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe233e658, SUC_lexicon_tags_10 },
    { 0xd753e658, SUC_lexicon_tags_8 },
    { 0x4f73e65a, SUC_lexicon_tags_0 },
    { 0xb0d3e65b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933e668, SUC_lexicon_tags_8 },
    { 0x42b3e669, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ed3e66b, SUC_lexicon_tags_61 },
    { 0x8033e66c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa113e66e, SUC_lexicon_tags_25 },
    { 0xeb73e66f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x91b3e671, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3e674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb3e678, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e13e67a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d3e67e, SUC_lexicon_tags_32 },
    { 0x67d3e67e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfd3e681, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4653e68a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf593e68d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a33e690, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5593e69b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x3e33e69d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f93e6a0, SUC_lexicon_tags_17 },
    { 0xd53e6a1, SUC_lexicon_tags_2 },
    { 0xb113e6a1, SUC_lexicon_tags_37 },
    { 0xa673e6a3, SUC_lexicon_tags_11 },
    { 0x4c93e6a1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6733e6a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd833e6a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd93e6aa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf3e6b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa893e6b4, SUC_lexicon_tags_14 },
    { 0x3b93e6b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ef3e6b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4cf3e6b9, SUC_lexicon_tags_2 },
    { 0x6a73e6b9, SUC_lexicon_tags_33 },
    { 0xe413e6bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b3e6bf, SUC_lexicon_tags_38 },
    { 0x393e6c0, SUC_lexicon_tags_62 },
    { 0x73d3e6c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a33e6d0, SUC_lexicon_tags_27 },
    { 0xb113e6d1, SUC_lexicon_tags_15 },
    { 0xdff3e6d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe713e6d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5873e6dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa473e6e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd53e6e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8633e6e5, SUC_lexicon_tags_27 },
    { 0x5213e6e6, SUC_lexicon_tags_37 },
    { 0x5493e6e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7cd3e6e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf273e6ec, SUC_lexicon_tags_7 },
    { 0x5053e6ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xefb3e6ef, SUC_lexicon_tags_0 },
    { 0xfd53e6f0, SUC_lexicon_tags_17 },
    { 0x4a73e6f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f93e6f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd8f3e6f6, SUC_lexicon_tags_28 },
    { 0xaaf3e6f7, SUC_lexicon_tags_316 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1593e702, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc93e704, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a33e709, SUC_lexicon_tags_17 },
    { 0x3f93e70a, SUC_lexicon_tags_7 },
    { 0xcd13e70b, SUC_lexicon_tags_37 },
    { 0x8533e70c, SUC_lexicon_tags_8 },
    { 0x9073e709, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf113e70f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9333e715, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e93e718, SUC_lexicon_tags_12 },
    { 0x5b3e718, SUC_lexicon_tags_1 },
    { 0xfb53e71a, SUC_lexicon_tags_25 },
    { 0xaef3e719, SUC_lexicon_tags_25 },
    { 0x3c53e71c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb353e71e, SUC_lexicon_tags_14 },
    { 0xa573e71f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19f3e721, SUC_lexicon_tags_12 },
    { 0x2113e722, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab3e726, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc793e72f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd513e732, SUC_lexicon_tags_39 },
    { 0x1ab3e732, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c53e737, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef3e73a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6433e73c, SUC_lexicon_tags_7 },
    { 0xeab3e73d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d73e741, SUC_lexicon_tags_14 },
    { 0x76b3e742, SUC_lexicon_tags_11 },
    { 0x4af3e743, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb093e745, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe393e74a, SUC_lexicon_tags_7 },
    { 0x7913e74a, SUC_lexicon_tags_49 },
    { 0x9573e74a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3493e752, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf73e754, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc293e758, SUC_lexicon_tags_33 },
    { 0xf4b3e759, SUC_lexicon_tags_7 },
    { 0xda13e758, SUC_lexicon_tags_6 },
    { 0x5ef3e75b, SUC_lexicon_tags_13 },
    { 0xa353e75c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce33e762, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x60b3e764, SUC_lexicon_tags_38 },
    { 0xc373e765, SUC_lexicon_tags_30 },
    { 0x41b3e766, SUC_lexicon_tags_12 },
    { 0x2f13e766, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa413e76c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5953e776, SUC_lexicon_tags_37 },
    { 0xe0d3e777, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8713e77b, SUC_lexicon_tags_33 },
    { 0xc153e77c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f3e782, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x88b3e784, SUC_lexicon_tags_14 },
    { 0x97f3e785, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8573e787, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb3e790, SUC_lexicon_tags_25 },
    { 0xee93e791, SUC_lexicon_tags_2 },
    { 0x68d3e791, SUC_lexicon_tags_53 },
    { 0x853e793, SUC_lexicon_tags_1 },
    { 0xd013e794, SUC_lexicon_tags_49 },
    { 0x5153e795, SUC_lexicon_tags_19 },
    { 0xc13e795, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e79b, SUC_lexicon_tags_60 },
    { 0xe1b3e79c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe193e79e, SUC_lexicon_tags_25 },
    { 0x76b3e79f, SUC_lexicon_tags_33 },
    { 0xcb73e79f, SUC_lexicon_tags_25 },
    { 0x213e7a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc673e7a4, SUC_lexicon_tags_25 },
    { 0x4a93e7a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb6b3e7a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8573e7a9, SUC_lexicon_tags_17 },
    { 0xa953e7aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13e7ad, SUC_lexicon_tags_38 },
    { 0xd9f3e7ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3693e7b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab3e7b7, SUC_lexicon_tags_49 },
    { 0xec53e7b8, SUC_lexicon_tags_77 },
    { 0xa73e7b8, SUC_lexicon_tags_17 },
    { 0x51f3e7b8, SUC_lexicon_tags_14 },
    { 0xf973e7b9, SUC_lexicon_tags_2 },
    { 0x7e93e7bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb573e7be, SUC_lexicon_tags_17 },
    { 0x9413e7bf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3873e7c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb533e7c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3e7c6, SUC_lexicon_tags_664 },
    { 0xc933e7c6, SUC_lexicon_tags_25 },
    { 0xe9b3e7c8, SUC_lexicon_tags_12 },
    { 0x14b3e7c9, SUC_lexicon_tags_2 },
    { 0x8473e7c8, SUC_lexicon_tags_9 },
    { 0xfe53e7cb, SUC_lexicon_tags_17 },
    { 0x78f3e7ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a53e7ce, SUC_lexicon_tags_25 },
    { 0x5c33e7ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b3e7d4, SUC_lexicon_tags_14 },
    { 0xb0f3e7d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd493e7d8, SUC_lexicon_tags_11 },
    { 0x8433e7d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa113e7dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9233e7de, SUC_lexicon_tags_1 },
    { 0xb993e7de, SUC_lexicon_tags_12 },
    { 0xcd53e7df, SUC_lexicon_tags_7 },
    { 0x7433e7de, SUC_lexicon_tags_26 },
    { 0xceb3e7e2, SUC_lexicon_tags_17 },
    { 0xda73e7e3, SUC_lexicon_tags_12 },
    { 0x4613e7de, SUC_lexicon_tags_39 },
    { 0xc2f3e7e5, SUC_lexicon_tags_16 },
    { 0x5a53e7e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xffd3e7e8, SUC_lexicon_tags_6 },
    { 0x8813e7e9, SUC_lexicon_tags_15 },
    { 0x4eb3e7ea, SUC_lexicon_tags_25 },
    { 0x7ff3e7eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d3e7ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd3e7f2, SUC_lexicon_tags_7 },
    { 0x9373e7f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9653e7f8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4ab3e7fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f3e7fd, SUC_lexicon_tags_45 },
    { 0x1cd3e7fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x72f3e800, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd513e803, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8973e807, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7e33e809, SUC_lexicon_tags_60 },
    { 0x2053e80a, SUC_lexicon_tags_31 },
    { 0xb693e80b, SUC_lexicon_tags_17 },
    { 0xeb73e809, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f3e814, SUC_lexicon_tags_39 },
    { 0x6093e815, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9333e81c, SUC_lexicon_tags_15 },
    { 0x5ed3e81d, SUC_lexicon_tags_2 },
    { 0x1df3e81d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x85b3e820, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d13e824, SUC_lexicon_tags_25 },
    { 0x4533e825, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633e82d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6793e82f, SUC_lexicon_tags_1 },
    { 0xced3e830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf533e834, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x71f3e836, SUC_lexicon_tags_26 },
    { 0x3153e837, SUC_lexicon_tags_11 },
    { 0x4c13e836, SUC_lexicon_tags_14 },
    { 0xae53e839, SUC_lexicon_tags_11 },
    { 0xa6b3e83a, SUC_lexicon_tags_19 },
    { 0x2873e83a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d3e844, SUC_lexicon_tags_56 },
    { 0x4493e844, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f93e848, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5773e84c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b3e84f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b3e854, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a73e85a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1553e85c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff3e860, SUC_lexicon_tags_37 },
    { 0x4e73e861, SUC_lexicon_tags_2 },
    { 0x2dd3e862, SUC_lexicon_tags_37 },
    { 0xbc33e860, SUC_lexicon_tags_26 },
    { 0x6c33e864, SUC_lexicon_tags_9 },
    { 0x1093e860, SUC_lexicon_tags_39 },
    { 0x2333e861, SUC_lexicon_tags_11 },
    { 0x9553e865, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d3e86a, SUC_lexicon_tags_16 },
    { 0x5693e86b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa33e873, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b13e875, SUC_lexicon_tags_17 },
    { 0xc9f3e876, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa793e878, SUC_lexicon_tags_82 },
    { 0xc233e878, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab3e87c, SUC_lexicon_tags_39 },
    { 0x4293e87d, SUC_lexicon_tags_82 },
    { 0x393e87e, SUC_lexicon_tags_4 },
    { 0xb473e87d, SUC_lexicon_tags_63 },
    { 0xdf3e880, SUC_lexicon_tags_27 },
    { 0x3553e881, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xceb3e883, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1e93e885, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe153e888, SUC_lexicon_tags_21 },
    { 0x94f3e889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5853e88b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5753e88e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1893e892, SUC_lexicon_tags_27 },
    { 0x9c73e892, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa53e89b, SUC_lexicon_tags_6 },
    { 0x89d3e89c, SUC_lexicon_tags_26 },
    { 0xb333e89b, SUC_lexicon_tags_11 },
    { 0x1d13e89e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xed73e8a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2433e8a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8d33e8aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff3e8ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7673e8b3, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xf253e8b6, SUC_lexicon_tags_7 },
    { 0x4d93e8b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4d3e8b9, SUC_lexicon_tags_38 },
    { 0x8713e8ba, SUC_lexicon_tags_61 },
    { 0x7953e8bb, SUC_lexicon_tags_37 },
    { 0x59f3e8b9, SUC_lexicon_tags_29 },
    { 0xfcd3e8bb, SUC_lexicon_tags_7 },
    { 0x7173e8bb, SUC_lexicon_tags_25 },
    { 0xf473e8bf, SUC_lexicon_tags_47 },
    { 0xd853e8c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7453e8c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb3e8c5, SUC_lexicon_tags_63 },
    { 0x7253e8c6, SUC_lexicon_tags_19 },
    { 0x3e73e8c7, SUC_lexicon_tags_1 },
    { 0x493e8c7, SUC_lexicon_tags_43 },
    { 0x6493e8c9, SUC_lexicon_tags_2 },
    { 0x8853e8ca, SUC_lexicon_tags_33 },
    { 0xd2d3e8cb, SUC_lexicon_tags_17 },
    { 0xebd3e8cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6133e8d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x593e8d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc113e8d9, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0xa9d3e8db, SUC_lexicon_tags_47 },
    { 0x4a53e8db, SUC_lexicon_tags_31 },
    { 0x9bd3e8dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf3e8e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4d3e8e3, SUC_lexicon_tags_47 },
    { 0x6973e8e4, SUC_lexicon_tags_26 },
    { 0xed33e8e5, SUC_lexicon_tags_37 },
    { 0x1793e8e5, SUC_lexicon_tags_12 },
    { 0xe3f3e8e7, SUC_lexicon_tags_28 },
    { 0x1f33e8e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc53e8ea, SUC_lexicon_tags_1 },
    { 0x4ef3e8eb, SUC_lexicon_tags_47 },
    { 0x4c33e8eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff93e8f3, SUC_lexicon_tags_63 },
    { 0x72b3e8f4, SUC_lexicon_tags_47 },
    { 0x4e73e8f5, SUC_lexicon_tags_45 },
    { 0x4733e8f6, SUC_lexicon_tags_31 },
    { 0x4393e8f7, SUC_lexicon_tags_46 },
    { 0x5cd3e8f8, SUC_lexicon_tags_17 },
    { 0x7833e8f7, SUC_lexicon_tags_53 },
    { 0xb6f3e8fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d3e8ff, SUC_lexicon_tags_23 },
    { 0x5953e900, SUC_lexicon_tags_1 },
    { 0x7d53e901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d3e904, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb173e907, SUC_lexicon_tags_46 },
    { 0xcf3e908, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad3e90c, SUC_lexicon_tags_33 },
    { 0x4053e90d, SUC_lexicon_tags_11 },
    { 0x5673e90e, SUC_lexicon_tags_17 },
    { 0x3773e90f, SUC_lexicon_tags_60 },
    { 0xbd33e910, SUC_lexicon_tags_493 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f3e914, SUC_lexicon_tags_7 },
    { 0x4393e915, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb153e917, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40b3e919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2353e91b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab3e91e, SUC_lexicon_tags_30 },
    { 0x7dd3e91e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd953e922, SUC_lexicon_tags_28 },
    { 0x2bf3e922, SUC_lexicon_tags_26 },
    { 0x213e924, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1f3e926, SUC_lexicon_tags_28 },
    { 0xa2f3e926, SUC_lexicon_tags_498 },
    { 0, NULL },
    { 0xb9b3e929, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4d53e92b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b3e92e, SUC_lexicon_tags_1 },
    { 0x6f3e92f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3513e932, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a33e936, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1393e93c, SUC_lexicon_tags_37 },
    { 0x2bb3e93d, SUC_lexicon_tags_17 },
    { 0xa4d3e93d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e73e940, SUC_lexicon_tags_816 },
    { 0xa393e941, SUC_lexicon_tags_57 },
    { 0x3253e942, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdaf3e944, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1993e947, SUC_lexicon_tags_35 },
    { 0x2ed3e948, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8913e94d, SUC_lexicon_tags_43 },
    { 0xe4b3e94e, SUC_lexicon_tags_6 },
    { 0x5d53e94f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3573e951, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd3e954, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10b3e956, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa693e95a, SUC_lexicon_tags_12 },
    { 0xdb13e95b, SUC_lexicon_tags_8 },
    { 0x58d3e95b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ab3e95e, SUC_lexicon_tags_11 },
    { 0x51d3e95e, SUC_lexicon_tags_25 },
    { 0x24f3e95f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113e965, SUC_lexicon_tags_33 },
    { 0x4493e966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633e96a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb693e96d, SUC_lexicon_tags_0 },
    { 0x9f73e96e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd033e970, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f33e973, SUC_lexicon_tags_2 },
    { 0x9813e973, SUC_lexicon_tags_32 },
    { 0x43f3e973, SUC_lexicon_tags_92 },
    { 0xbaf3e976, SUC_lexicon_tags_17 },
    { 0x16b3e977, SUC_lexicon_tags_15 },
    { 0x90f3e976, SUC_lexicon_tags_47 },
    { 0x96f3e973, SUC_lexicon_tags_12 },
    { 0xc833e97a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf933e97e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f3e983, SUC_lexicon_tags_2 },
    { 0x38d3e983, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b3e98b, SUC_lexicon_tags_17 },
    { 0x1993e98c, SUC_lexicon_tags_37 },
    { 0xe773e98b, SUC_lexicon_tags_15 },
    { 0xb773e98e, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0xa9d3e990, SUC_lexicon_tags_194 },
    { 0xaff3e991, SUC_lexicon_tags_38 },
    { 0xf913e992, SUC_lexicon_tags_47 },
    { 0xf1b3e993, SUC_lexicon_tags_37 },
    { 0x79f3e991, SUC_lexicon_tags_25 },
    { 0x633e995, SUC_lexicon_tags_7 },
    { 0xf13e996, SUC_lexicon_tags_39 },
    { 0xab73e996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe733e999, SUC_lexicon_tags_51 },
    { 0xa1f3e999, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xca73e99d, SUC_lexicon_tags_17 },
    { 0x1ed3e99e, SUC_lexicon_tags_11 },
    { 0x9333e99f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xec73e9a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff13e9a4, SUC_lexicon_tags_47 },
    { 0xf1b3e9a5, SUC_lexicon_tags_4 },
    { 0x5eb3e9a6, SUC_lexicon_tags_47 },
    { 0x3fd3e9a7, SUC_lexicon_tags_38 },
    { 0xbaf3e9a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8673e9ab, SUC_lexicon_tags_16 },
    { 0xa573e9ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3893e9ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6713e9b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a53e9b4, SUC_lexicon_tags_0 },
    { 0xfd3e9b5, SUC_lexicon_tags_21 },
    { 0x8f3e9b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6053e9bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d3e9c1, SUC_lexicon_tags_26 },
    { 0x9473e9c2, SUC_lexicon_tags_17 },
    { 0x83b3e9c2, SUC_lexicon_tags_15 },
    { 0xfd93e9c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d3e9cc, SUC_lexicon_tags_11 },
    { 0x5f3e9cc, SUC_lexicon_tags_12 },
    { 0x2493e9ce, SUC_lexicon_tags_45 },
    { 0x1893e9cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f3e9d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d3e9d5, SUC_lexicon_tags_118 },
    { 0x4073e9d6, SUC_lexicon_tags_25 },
    { 0x16b3e9d6, SUC_lexicon_tags_37 },
    { 0xc9f3e9d7, SUC_lexicon_tags_38 },
    { 0xe93e9d8, SUC_lexicon_tags_25 },
    { 0x9513e9d6, SUC_lexicon_tags_47 },
    { 0xd1b3e9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6433e9e1, SUC_lexicon_tags_17 },
    { 0x9c13e9e2, SUC_lexicon_tags_1 },
    { 0xa093e9e3, SUC_lexicon_tags_9 },
    { 0x9d3e9e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8073e9e8, SUC_lexicon_tags_1 },
    { 0xf33e9e9, SUC_lexicon_tags_138 },
    { 0xdf13e9e9, SUC_lexicon_tags_1 },
    { 0x2d3e9eb, SUC_lexicon_tags_55 },
    { 0x31f3e9ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ad3e9ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf33e9f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde53e9f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5353e9f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f3e9fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6053e9fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7853ea11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda53ea15, SUC_lexicon_tags_49 },
    { 0xfa53ea15, SUC_lexicon_tags_32 },
    { 0xac93ea17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5793ea1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f53ea1f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6633ea21, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d3ea2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa13ea32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d33ea34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5553ea36, SUC_lexicon_tags_12 },
    { 0x38d3ea37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd873ea39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa853ea3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14f3ea3d, SUC_lexicon_tags_37 },
    { 0xcfd3ea3e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf13ea42, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d3ea45, SUC_lexicon_tags_0 },
    { 0x6a73ea45, SUC_lexicon_tags_0 },
    { 0xc6b3ea47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6873ea4a, SUC_lexicon_tags_29 },
    { 0xfef3ea4b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af3ea4e, SUC_lexicon_tags_25 },
    { 0x40b3ea4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde13ea51, SUC_lexicon_tags_11 },
    { 0x2c53ea52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf253ea55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf33ea58, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6973ea5a, SUC_lexicon_tags_23 },
    { 0x1a13ea5b, SUC_lexicon_tags_19 },
    { 0xe73ea5c, SUC_lexicon_tags_37 },
    { 0x88f3ea5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2173ea60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb53ea65, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f3ea6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d33ea6c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff3ea6f, SUC_lexicon_tags_1 },
    { 0x3613ea6f, SUC_lexicon_tags_45 },
    { 0x5a33ea71, SUC_lexicon_tags_0 },
    { 0x2973ea72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1893ea76, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc0d3ea78, SUC_lexicon_tags_25 },
    { 0x4e53ea79, SUC_lexicon_tags_46 },
    { 0xcf33ea79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b13ea7e, SUC_lexicon_tags_2 },
    { 0xd93ea7e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdd53ea81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2153ea83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13ea8b, SUC_lexicon_tags_623 },
    { 0xe133ea8c, SUC_lexicon_tags_2 },
    { 0x68d3ea8b, SUC_lexicon_tags_11 },
    { 0x4573ea8d, SUC_lexicon_tags_11 },
    { 0x34d3ea8d, SUC_lexicon_tags_37 },
    { 0x9d3ea90, SUC_lexicon_tags_1 },
    { 0x5d93ea90, SUC_lexicon_tags_47 },
    { 0x7933ea92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc753ea94, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1673ea97, SUC_lexicon_tags_148 },
    { 0x8433ea98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3293ea9c, SUC_lexicon_tags_31 },
    { 0x4393ea9c, SUC_lexicon_tags_17 },
    { 0x8473ea9e, SUC_lexicon_tags_38 },
    { 0x29d3ea9c, SUC_lexicon_tags_12 },
    { 0x2a93ea9e, SUC_lexicon_tags_11 },
    { 0xb333ea9d, SUC_lexicon_tags_15 },
    { 0x7733eaa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3313eaad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e73eaaf, SUC_lexicon_tags_82 },
    { 0xa0f3eaaf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b53eab3, SUC_lexicon_tags_107 },
    { 0xf833eab3, SUC_lexicon_tags_37 },
    { 0xe8f3eab5, SUC_lexicon_tags_525 },
    { 0x9693eab6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c93eabb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f13eabe, SUC_lexicon_tags_48 },
    { 0x5473eabe, SUC_lexicon_tags_191 },
    { 0xf0b3eabe, SUC_lexicon_tags_49 },
    { 0xfbd3eabe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa913eac5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe533eac8, SUC_lexicon_tags_14 },
    { 0x7553eac8, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xc853eacb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b13ead1, SUC_lexicon_tags_11 },
    { 0x24f3ead2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4593ead5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b33eadc, SUC_lexicon_tags_5 },
    { 0x97f3eadc, SUC_lexicon_tags_8 },
    { 0x8a13eade, SUC_lexicon_tags_6 },
    { 0x52f3eadf, SUC_lexicon_tags_11 },
    { 0xdb3eadf, SUC_lexicon_tags_32 },
    { 0x54b3eae1, SUC_lexicon_tags_55 },
    { 0x4993eae2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb3eaea, SUC_lexicon_tags_7 },
    { 0x4ef3eaeb, SUC_lexicon_tags_11 },
    { 0x2113eaec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f3eaef, SUC_lexicon_tags_47 },
    { 0xf553eaef, SUC_lexicon_tags_37 },
    { 0x4bf3eaf0, SUC_lexicon_tags_56 },
    { 0xe6b3eaf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6313eaf4, SUC_lexicon_tags_11 },
    { 0x33b3eaf5, SUC_lexicon_tags_29 },
    { 0xec33eaf6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd53eaf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5453eafc, SUC_lexicon_tags_65 },
    { 0x9233eafc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x44b3eaff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f3eb03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xae13eb06, SUC_lexicon_tags_15 },
    { 0xae53eb06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb373eb0e, SUC_lexicon_tags_5 },
    { 0x8ed3eb0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa933eb13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb3eb16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc5b3eb18, SUC_lexicon_tags_37 },
    { 0x1a53eb19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b3eb1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8373eb1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb3eb22, SUC_lexicon_tags_9 },
    { 0x5b13eb22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a13eb26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d3eb2b, SUC_lexicon_tags_7 },
    { 0xee53eb2b, SUC_lexicon_tags_7 },
    { 0x8a73eb2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe73eb31, SUC_lexicon_tags_63 },
    { 0x28d3eb31, SUC_lexicon_tags_23 },
    { 0x69b3eb33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9d3eb35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753eb3b, SUC_lexicon_tags_37 },
    { 0x5893eb3c, SUC_lexicon_tags_31 },
    { 0xfb53eb3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd13eb3f, SUC_lexicon_tags_10 },
    { 0x45b3eb40, SUC_lexicon_tags_37 },
    { 0xb453eb41, SUC_lexicon_tags_553 },
    { 0x6cd3eb41, SUC_lexicon_tags_7 },
    { 0x1c53eb42, SUC_lexicon_tags_0 },
    { 0xea53eb42, SUC_lexicon_tags_37 },
    { 0x3993eb40, SUC_lexicon_tags_435 },
    { 0xc73eb46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3eb49, SUC_lexicon_tags_46 },
    { 0xadf3eb4a, SUC_lexicon_tags_17 },
    { 0x27d3eb4b, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f3eb4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea33eb51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9a33eb53, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcf13eb55, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc73eb5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db3eb5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd613eb66, SUC_lexicon_tags_72 },
    { 0x6333eb67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b3eb6b, SUC_lexicon_tags_35 },
    { 0xc193eb6b, SUC_lexicon_tags_7 },
    { 0x2733eb6b, SUC_lexicon_tags_11 },
    { 0x9e33eb6b, SUC_lexicon_tags_60 },
    { 0x7d93eb6f, SUC_lexicon_tags_39 },
    { 0x58b3eb6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd473eb7d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x28b3eb7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c13eb83, SUC_lexicon_tags_25 },
    { 0xd7d3eb83, SUC_lexicon_tags_87 },
    { 0x32f3eb85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe293eb87, SUC_lexicon_tags_25 },
    { 0xbeb3eb88, SUC_lexicon_tags_151 },
    { 0x36f3eb88, SUC_lexicon_tags_55 },
    { 0x54f3eb8a, SUC_lexicon_tags_1 },
    { 0xc6b3eb8b, SUC_lexicon_tags_7 },
    { 0x29d3eb8c, SUC_lexicon_tags_23 },
    { 0x3fd3eb8d, SUC_lexicon_tags_11 },
    { 0x3c73eb8e, SUC_lexicon_tags_26 },
    { 0x9093eb8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d53eb91, SUC_lexicon_tags_17 },
    { 0xf2b3eb92, SUC_lexicon_tags_17 },
    { 0xea93eb92, SUC_lexicon_tags_11 },
    { 0x9a53eb94, SUC_lexicon_tags_45 },
    { 0x1313eb95, SUC_lexicon_tags_17 },
    { 0x8fb3eb94, SUC_lexicon_tags_1 },
    { 0xc053eb91, SUC_lexicon_tags_37 },
    { 0x34f3eb98, SUC_lexicon_tags_25 },
    { 0xdfd3eb99, SUC_lexicon_tags_8 },
    { 0x8013eb99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc913eb9c, SUC_lexicon_tags_10 },
    { 0x9513eb9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b73eba4, SUC_lexicon_tags_37 },
    { 0xa9d3eba5, SUC_lexicon_tags_17 },
    { 0xd073eba5, SUC_lexicon_tags_68 },
    { 0x81f3eba7, SUC_lexicon_tags_53 },
    { 0xb7f3eba7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d3ebab, SUC_lexicon_tags_45 },
    { 0xd7b3ebab, SUC_lexicon_tags_0 },
    { 0x9fd3ebac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9013ebb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca33ebb6, SUC_lexicon_tags_62 },
    { 0x7d13ebb7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xaef3ebb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc53ebbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c33ebc0, SUC_lexicon_tags_38 },
    { 0x9ad3ebc0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1213ebc6, SUC_lexicon_tags_27 },
    { 0x26d3ebc7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xab73ebca, SUC_lexicon_tags_7 },
    { 0xa613ebcb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd573ebcd, SUC_lexicon_tags_27 },
    { 0x4b3ebce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f53ebd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d93ebd5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1ab3ebd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b3ebdd, SUC_lexicon_tags_15 },
    { 0x99b3ebdd, SUC_lexicon_tags_39 },
    { 0xf973ebdd, SUC_lexicon_tags_25 },
    { 0xf233ebde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af3ebe4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7053ebe6, SUC_lexicon_tags_12 },
    { 0x2f73ebe7, SUC_lexicon_tags_12 },
    { 0x633ebe8, SUC_lexicon_tags_15 },
    { 0xa1b3ebe8, SUC_lexicon_tags_7 },
    { 0x53b3ebea, SUC_lexicon_tags_15 },
    { 0x78f3ebeb, SUC_lexicon_tags_27 },
    { 0x95d3ebe8, SUC_lexicon_tags_27 },
    { 0x1e33ebed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb3ebf2, SUC_lexicon_tags_17 },
    { 0x6973ebf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9193ebf7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4693ebfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5833ec01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9db3ec03, SUC_lexicon_tags_63 },
    { 0xf33ec03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d3ec07, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5b3ec09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f53ec13, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433ec18, SUC_lexicon_tags_61 },
    { 0x5413ec19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc833ec1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d3ec20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b3ec23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ed3ec25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f3ec2d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab3ec30, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa33ec32, SUC_lexicon_tags_11 },
    { 0xf5f3ec33, SUC_lexicon_tags_59 },
    { 0xf913ec34, SUC_lexicon_tags_6 },
    { 0xc5d3ec35, SUC_lexicon_tags_55 },
    { 0x9933ec36, SUC_lexicon_tags_23 },
    { 0x29d3ec37, SUC_lexicon_tags_45 },
    { 0xde53ec34, SUC_lexicon_tags_47 },
    { 0x1fd3ec39, SUC_lexicon_tags_15 },
    { 0x3fd3ec3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf3ec41, SUC_lexicon_tags_45 },
    { 0x7b73ec41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c53ec46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf3ec49, SUC_lexicon_tags_39 },
    { 0xbaf3ec4a, SUC_lexicon_tags_25 },
    { 0x2573ec4b, SUC_lexicon_tags_38 },
    { 0xd313ec4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ff3ec4e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb3ec57, SUC_lexicon_tags_15 },
    { 0xed53ec57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc33ec5a, SUC_lexicon_tags_1 },
    { 0x3e13ec5a, SUC_lexicon_tags_29 },
    { 0x3fb3ec5a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd6b3ec5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed33ec6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f3ec70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc733ec72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bf3ec74, SUC_lexicon_tags_11 },
    { 0xb133ec75, SUC_lexicon_tags_17 },
    { 0xb0b3ec75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f3ec7b, SUC_lexicon_tags_1 },
    { 0x39f3ec7b, SUC_lexicon_tags_11 },
    { 0x60f3ec7c, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x3573ec80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffd3ec82, SUC_lexicon_tags_624 },
    { 0xd713ec83, SUC_lexicon_tags_16 },
    { 0xacf3ec84, SUC_lexicon_tags_290 },
    { 0x3a13ec84, SUC_lexicon_tags_25 },
    { 0x7873ec86, SUC_lexicon_tags_14 },
    { 0xb2d3ec87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b3ec8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8973ec91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5113ec93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8153ec97, SUC_lexicon_tags_11 },
    { 0x98b3ec98, SUC_lexicon_tags_9 },
    { 0x893ec98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6353ec9c, SUC_lexicon_tags_7 },
    { 0x26b3ec9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x693eca0, SUC_lexicon_tags_25 },
    { 0x5f3eca1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3573eca4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f3eca8, SUC_lexicon_tags_12 },
    { 0xf953eca9, SUC_lexicon_tags_35 },
    { 0xf913eca8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4273ecad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7793ecaf, SUC_lexicon_tags_11 },
    { 0x73d3ecb0, SUC_lexicon_tags_12 },
    { 0xb2b3ecaf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4293ecb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9493ecb8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d3ecbe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5813ecc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2793ecc5, SUC_lexicon_tags_22 },
    { 0x5653ecc6, SUC_lexicon_tags_48 },
    { 0xd013ecc6, SUC_lexicon_tags_14 },
    { 0xdc13ecc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e93ecd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b3ecdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d73ece1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d3ece4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d3eceb, SUC_lexicon_tags_43 },
    { 0xe7b3eceb, SUC_lexicon_tags_7 },
    { 0x7f53eceb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe93ecf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b3ecf4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2773ecf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6273ecf8, SUC_lexicon_tags_37 },
    { 0x4933ecf8, SUC_lexicon_tags_17 },
    { 0x7053ecfa, SUC_lexicon_tags_1 },
    { 0x5ed3ecfb, SUC_lexicon_tags_15 },
    { 0x5633ecfa, SUC_lexicon_tags_11 },
    { 0xb973ecf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b3ed04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb973ed06, SUC_lexicon_tags_92 },
    { 0x8733ed06, SUC_lexicon_tags_17 },
    { 0xccf3ed08, SUC_lexicon_tags_12 },
    { 0xc933ed07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f3ed10, SUC_lexicon_tags_16 },
    { 0x96d3ed11, SUC_lexicon_tags_11 },
    { 0xebb3ed11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa353ed1d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8273ed1f, SUC_lexicon_tags_65 },
    { 0x7a93ed20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b3ed2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7513ed2f, SUC_lexicon_tags_27 },
    { 0xf1f3ed2f, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xa9b3ed32, SUC_lexicon_tags_32 },
    { 0xbef3ed33, SUC_lexicon_tags_8 },
    { 0x7e53ed34, SUC_lexicon_tags_14 },
    { 0xd3f3ed32, SUC_lexicon_tags_17 },
    { 0x9713ed36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe673ed3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d3ed45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf633ed48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f53ed51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39d3ed53, SUC_lexicon_tags_31 },
    { 0xcdf3ed54, SUC_lexicon_tags_2 },
    { 0x6353ed55, SUC_lexicon_tags_1 },
    { 0x9d93ed55, SUC_lexicon_tags_47 },
    { 0x9373ed56, SUC_lexicon_tags_17 },
    { 0xd213ed58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd3ed5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79f3ed60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec13ed62, SUC_lexicon_tags_10 },
    { 0xcad3ed63, SUC_lexicon_tags_11 },
    { 0xb273ed62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1db3ed66, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ed6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e13ed6e, SUC_lexicon_tags_9 },
    { 0xe2b3ed6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd3ed71, SUC_lexicon_tags_7 },
    { 0x4753ed71, SUC_lexicon_tags_13 },
    { 0x3153ed72, SUC_lexicon_tags_11 },
    { 0x22b3ed74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b3ed7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7753ed7e, SUC_lexicon_tags_25 },
    { 0x4953ed7e, SUC_lexicon_tags_15 },
    { 0xf2b3ed7f, SUC_lexicon_tags_14 },
    { 0x7593ed81, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe313ed83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab3ed8b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd613ed8d, SUC_lexicon_tags_53 },
    { 0xb253ed8e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9e73ed90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f3ed95, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1613ed9b, SUC_lexicon_tags_37 },
    { 0x3eb3ed9c, SUC_lexicon_tags_25 },
    { 0x3c33ed9c, SUC_lexicon_tags_15 },
    { 0x7013ed9e, SUC_lexicon_tags_19 },
    { 0xf133ed9f, SUC_lexicon_tags_47 },
    { 0x8ab3eda0, SUC_lexicon_tags_27 },
    { 0x32b3eda1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f3eda8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d3edac, SUC_lexicon_tags_85 },
    { 0x25b3edad, SUC_lexicon_tags_39 },
    { 0xd6f3edae, SUC_lexicon_tags_2 },
    { 0x2db3edaf, SUC_lexicon_tags_12 },
    { 0xa5d3edae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d3edb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3413edb8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd6d3edba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd533edbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa433edc0, SUC_lexicon_tags_11 },
    { 0xee53edc1, SUC_lexicon_tags_6 },
    { 0xd773edc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf3edcb, SUC_lexicon_tags_9 },
    { 0x1233edcc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec73edd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff3eddb, SUC_lexicon_tags_45 },
    { 0x8ff3eddc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0f3edde, SUC_lexicon_tags_11 },
    { 0x2d13eddf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8013ede5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf913edea, SUC_lexicon_tags_77 },
    { 0x2573edeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce33edee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d3edf1, SUC_lexicon_tags_12 },
    { 0x97b3edf2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6333edf6, SUC_lexicon_tags_25 },
    { 0x8713edf7, SUC_lexicon_tags_14 },
    { 0xbcb3edf7, SUC_lexicon_tags_7 },
    { 0xff93edf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b3edfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c73ee03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x653ee05, SUC_lexicon_tags_49 },
    { 0x5c13ee05, SUC_lexicon_tags_43 },
    { 0x3893ee07, SUC_lexicon_tags_37 },
    { 0xde93ee06, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd213ee0a, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d3ee0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe373ee11, SUC_lexicon_tags_6 },
    { 0xc393ee12, SUC_lexicon_tags_92 },
    { 0x9e33ee11, SUC_lexicon_tags_12 },
    { 0x2c53ee14, SUC_lexicon_tags_8 },
    { 0xdf3ee15, SUC_lexicon_tags_31 },
    { 0xeb73ee11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad3ee1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2893ee1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe273ee22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ff3ee24, SUC_lexicon_tags_30 },
    { 0xa393ee25, SUC_lexicon_tags_53 },
    { 0xacf3ee24, SUC_lexicon_tags_82 },
    { 0x3e33ee27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd3ee2a, SUC_lexicon_tags_37 },
    { 0x5ef3ee2b, SUC_lexicon_tags_15 },
    { 0xecf3ee2a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb393ee32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf253ee35, SUC_lexicon_tags_19 },
    { 0x1c33ee35, SUC_lexicon_tags_7 },
    { 0xf833ee36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb9d3ee39, SUC_lexicon_tags_56 },
    { 0x52f3ee3a, SUC_lexicon_tags_37 },
    { 0x23b3ee3b, SUC_lexicon_tags_45 },
    { 0xf773ee39, SUC_lexicon_tags_12 },
    { 0x21b3ee3c, SUC_lexicon_tags_14 },
    { 0x12f3ee3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b3ee41, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb773ee43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9273ee46, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7793ee48, SUC_lexicon_tags_15 },
    { 0x5533ee49, SUC_lexicon_tags_3 },
    { 0xf813ee4a, SUC_lexicon_tags_17 },
    { 0x20d3ee4b, SUC_lexicon_tags_35 },
    { 0x4f3ee49, SUC_lexicon_tags_17 },
    { 0xa253ee49, SUC_lexicon_tags_32 },
    { 0x4ad3ee4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9673ee50, SUC_lexicon_tags_56 },
    { 0xc8d3ee50, SUC_lexicon_tags_10 },
    { 0x63b3ee52, SUC_lexicon_tags_16 },
    { 0xe0d3ee53, SUC_lexicon_tags_26 },
    { 0xa053ee54, SUC_lexicon_tags_25 },
    { 0xa273ee53, SUC_lexicon_tags_63 },
    { 0x3893ee55, SUC_lexicon_tags_33 },
    { 0xf973ee57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb33ee5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df3ee61, SUC_lexicon_tags_27 },
    { 0x6d3ee62, SUC_lexicon_tags_1 },
    { 0xfed3ee63, SUC_lexicon_tags_8 },
    { 0xa173ee63, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd6b3ee66, SUC_lexicon_tags_2 },
    { 0x5713ee66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6193ee6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3ee6e, SUC_lexicon_tags_30 },
    { 0x5e73ee6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f3ee73, SUC_lexicon_tags_33 },
    { 0x3313ee74, SUC_lexicon_tags_21 },
    { 0x66b3ee73, SUC_lexicon_tags_15 },
    { 0xb153ee76, SUC_lexicon_tags_37 },
    { 0x2d93ee77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b53ee79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5673ee7c, SUC_lexicon_tags_2 },
    { 0x3013ee7d, SUC_lexicon_tags_29 },
    { 0x27b3ee7d, SUC_lexicon_tags_61 },
    { 0xa1b3ee7f, SUC_lexicon_tags_25 },
    { 0xe4f3ee80, SUC_lexicon_tags_11 },
    { 0x5d93ee81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e73ee84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4d3ee86, SUC_lexicon_tags_7 },
    { 0xda53ee87, SUC_lexicon_tags_47 },
    { 0x6413ee86, SUC_lexicon_tags_6 },
    { 0xef93ee89, SUC_lexicon_tags_8 },
    { 0x3e93ee89, SUC_lexicon_tags_12 },
    { 0x5c13ee8b, SUC_lexicon_tags_27 },
    { 0xed53ee8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa913ee90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a33ee93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d3ee98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1753ee9a, SUC_lexicon_tags_53 },
    { 0x3093ee9b, SUC_lexicon_tags_38 },
    { 0x2b53ee9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c13eea0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8813eea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6413eea9, SUC_lexicon_tags_25 },
    { 0xc233eeaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2d3eeac, SUC_lexicon_tags_31 },
    { 0x9633eead, SUC_lexicon_tags_12 },
    { 0xb73eeae, SUC_lexicon_tags_44 },
    { 0xc4f3eeaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb073eeb2, SUC_lexicon_tags_7 },
    { 0x833eeb3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb913eeb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4493eebf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3eec2, SUC_lexicon_tags_39 },
    { 0x8cd3eec3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1c13eec5, SUC_lexicon_tags_11 },
    { 0xf033eec6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x52d3eec8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb3eed4, SUC_lexicon_tags_23 },
    { 0xd0f3eed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bf3eed7, SUC_lexicon_tags_16 },
    { 0xad73eed8, SUC_lexicon_tags_14 },
    { 0x49b3eed8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f53eedd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5793eedf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d33eee2, SUC_lexicon_tags_6 },
    { 0x1973eee3, SUC_lexicon_tags_11 },
    { 0x4b93eee4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd3eeea, SUC_lexicon_tags_11 },
    { 0xc553eeea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa7f3eeed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6133eef1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f3eef5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d3eefb, SUC_lexicon_tags_31 },
    { 0xa3d3eefc, SUC_lexicon_tags_213 },
    { 0x9b93eefd, SUC_lexicon_tags_1317 },
    { 0x68b3eefe, SUC_lexicon_tags_12 },
    { 0x2813eeff, SUC_lexicon_tags_25 },
    { 0xa453ef00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb153ef03, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb2b3ef05, SUC_lexicon_tags_15 },
    { 0xbfd3ef06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad3ef0e, SUC_lexicon_tags_3 },
    { 0xbfd3ef0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x173ef11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f33ef14, SUC_lexicon_tags_15 },
    { 0xef33ef15, SUC_lexicon_tags_25 },
    { 0x7b93ef14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2813ef19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb913ef1c, SUC_lexicon_tags_1 },
    { 0xef73ef1c, SUC_lexicon_tags_138 },
    { 0x1c33ef1c, SUC_lexicon_tags_15 },
    { 0x21b3ef1d, SUC_lexicon_tags_17 },
    { 0xf3b3ef20, SUC_lexicon_tags_25 },
    { 0x4cd3ef1e, SUC_lexicon_tags_7 },
    { 0x3873ef21, SUC_lexicon_tags_23 },
    { 0xebb3ef22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec73ef25, SUC_lexicon_tags_48 },
    { 0x32b3ef25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf853ef29, SUC_lexicon_tags_8 },
    { 0xea93ef2a, SUC_lexicon_tags_49 },
    { 0xc7b3ef2b, SUC_lexicon_tags_14 },
    { 0x6393ef2a, SUC_lexicon_tags_60 },
    { 0x26f3ef2d, SUC_lexicon_tags_64 },
    { 0x5d33ef2e, SUC_lexicon_tags_65 },
    { 0xe4f3ef2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d3ef32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f3ef39, SUC_lexicon_tags_43 },
    { 0x26f3ef3a, SUC_lexicon_tags_28 },
    { 0x8013ef3b, SUC_lexicon_tags_49 },
    { 0xa353ef3c, SUC_lexicon_tags_7 },
    { 0x2693ef3d, SUC_lexicon_tags_2 },
    { 0x5b13ef3e, SUC_lexicon_tags_47 },
    { 0x8b53ef3d, SUC_lexicon_tags_45 },
    { 0x3933ef40, SUC_lexicon_tags_7 },
    { 0xf853ef41, SUC_lexicon_tags_2 },
    { 0xcf33ef42, SUC_lexicon_tags_37 },
    { 0xb3d3ef43, SUC_lexicon_tags_107 },
    { 0x8b53ef44, SUC_lexicon_tags_26 },
    { 0x42b3ef45, SUC_lexicon_tags_47 },
    { 0x2493ef44, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d3ef49, SUC_lexicon_tags_12 },
    { 0x5273ef4a, SUC_lexicon_tags_9 },
    { 0x8513ef49, SUC_lexicon_tags_8 },
    { 0x5173ef4b, SUC_lexicon_tags_65 },
    { 0x7e93ef4d, SUC_lexicon_tags_27 },
    { 0xa9b3ef4d, SUC_lexicon_tags_37 },
    { 0xfdf3ef4f, SUC_lexicon_tags_68 },
    { 0x1733ef50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5473ef52, SUC_lexicon_tags_19 },
    { 0x9533ef52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8313ef5a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a13ef5d, SUC_lexicon_tags_1 },
    { 0x33d3ef5e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe473ef66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb753ef69, SUC_lexicon_tags_70 },
    { 0x26f3ef6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab3ef6e, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53ef74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d3ef79, SUC_lexicon_tags_49 },
    { 0x9e53ef79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcef3ef7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed53ef7e, SUC_lexicon_tags_37 },
    { 0x473ef7e, SUC_lexicon_tags_25 },
    { 0x8213ef7e, SUC_lexicon_tags_19 },
    { 0x8f93ef81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb213ef84, SUC_lexicon_tags_47 },
    { 0x2373ef85, SUC_lexicon_tags_3 },
    { 0xf8b3ef84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b3ef8b, SUC_lexicon_tags_12 },
    { 0xcd53ef8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91d3ef8e, SUC_lexicon_tags_23 },
    { 0x3793ef8f, SUC_lexicon_tags_27 },
    { 0x60d3ef90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb13ef92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa853ef96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc493ef99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5253ef9d, SUC_lexicon_tags_2 },
    { 0x4b53ef9e, SUC_lexicon_tags_19 },
    { 0x23f3ef9f, SUC_lexicon_tags_47 },
    { 0x4233ef9d, SUC_lexicon_tags_29 },
    { 0x5173ef9f, SUC_lexicon_tags_6 },
    { 0x1b73ef9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f3efab, SUC_lexicon_tags_9 },
    { 0x9813efab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa993efb2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa973efb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a93efbb, SUC_lexicon_tags_108 },
    { 0x24d3efbc, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x2693efbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb3efc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf33efc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3efca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f93efda, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd3efdd, SUC_lexicon_tags_64 },
    { 0x38f3efde, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b3efe3, SUC_lexicon_tags_14 },
    { 0x3e73efe4, SUC_lexicon_tags_1 },
    { 0xe793efe3, SUC_lexicon_tags_1 },
    { 0x4f53efe3, SUC_lexicon_tags_18 },
    { 0xe3b3efe7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e53efea, SUC_lexicon_tags_1 },
    { 0x8633efeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4473efed, SUC_lexicon_tags_47 },
    { 0x9693efed, SUC_lexicon_tags_2 },
    { 0x7b13efed, SUC_lexicon_tags_15 },
    { 0xc73eff0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf333eff2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2593eff5, SUC_lexicon_tags_7 },
    { 0x6353eff6, SUC_lexicon_tags_37 },
    { 0x7053eff6, SUC_lexicon_tags_11 },
    { 0xcb73eff6, SUC_lexicon_tags_32 },
    { 0x5613eff5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1f3effb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb753effe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d3f001, SUC_lexicon_tags_12 },
    { 0xd93f002, SUC_lexicon_tags_38 },
    { 0x813f003, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf653f006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb3f010, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3613f015, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe593f01a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd3f023, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6633f025, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe133f027, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6ff3f029, SUC_lexicon_tags_47 },
    { 0x9673f029, SUC_lexicon_tags_15 },
    { 0x4bf3f029, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9c33f02d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3933f02f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x913f032, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83b3f034, SUC_lexicon_tags_16 },
    { 0xe973f034, SUC_lexicon_tags_12 },
    { 0xfa53f036, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2813f038, SUC_lexicon_tags_14 },
    { 0xba73f039, SUC_lexicon_tags_0 },
    { 0x2c73f039, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc353f03c, SUC_lexicon_tags_16 },
    { 0x27d3f03d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f53f03f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd5b3f041, SUC_lexicon_tags_68 },
    { 0x26f3f041, SUC_lexicon_tags_31 },
    { 0x9f53f042, SUC_lexicon_tags_23 },
    { 0x55f3f041, SUC_lexicon_tags_2 },
    { 0x3db3f045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8693f04b, SUC_lexicon_tags_25 },
    { 0xca13f04c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b3f04f, SUC_lexicon_tags_11 },
    { 0x2313f04f, SUC_lexicon_tags_170 },
    { 0xa913f051, SUC_lexicon_tags_43 },
    { 0xb3d3f04f, SUC_lexicon_tags_1 },
    { 0xd553f04f, SUC_lexicon_tags_14 },
    { 0xf093f054, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9833f058, SUC_lexicon_tags_55 },
    { 0x3113f059, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2233f05c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5993f05e, SUC_lexicon_tags_27 },
    { 0x6433f05f, SUC_lexicon_tags_45 },
    { 0xbe13f060, SUC_lexicon_tags_13 },
    { 0xdbf3f061, SUC_lexicon_tags_12 },
    { 0x1a73f062, SUC_lexicon_tags_48 },
    { 0xd473f062, SUC_lexicon_tags_47 },
    { 0x55b3f064, SUC_lexicon_tags_17 },
    { 0x1bb3f061, SUC_lexicon_tags_15 },
    { 0xc693f066, SUC_lexicon_tags_16 },
    { 0x6ab3f067, SUC_lexicon_tags_0 },
    { 0xcc13f05f, SUC_lexicon_tags_65 },
    { 0x6353f067, SUC_lexicon_tags_25 },
    { 0x2b3f06a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff3f06f, SUC_lexicon_tags_2 },
    { 0x7bd3f06f, SUC_lexicon_tags_38 },
    { 0xdeb3f071, SUC_lexicon_tags_2 },
    { 0xbb13f070, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f33f079, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b33f081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa153f083, SUC_lexicon_tags_11 },
    { 0x6bb3f083, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb3f087, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe953f08b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacb3f08d, SUC_lexicon_tags_17 },
    { 0x65b3f08e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d93f091, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ab3f093, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d3f097, SUC_lexicon_tags_43 },
    { 0x48b3f098, SUC_lexicon_tags_12 },
    { 0xafd3f099, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3c73f09b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5d3f09d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfdf3f09f, SUC_lexicon_tags_138 },
    { 0xe373f0a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd193f0a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b3f0a7, SUC_lexicon_tags_37 },
    { 0x4793f0a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe353f0ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x393f0ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9513f0b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc353f0b7, SUC_lexicon_tags_31 },
    { 0xae33f0b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc7f3f0ba, SUC_lexicon_tags_7 },
    { 0xed73f0ba, SUC_lexicon_tags_11 },
    { 0xf713f0bc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc7f3f0be, SUC_lexicon_tags_17 },
    { 0x93d3f0be, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6373f0c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb33f0c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b3f0c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd3f0cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5bf3f0ce, SUC_lexicon_tags_0 },
    { 0x5c33f0ce, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7573f0d6, SUC_lexicon_tags_62 },
    { 0x2c13f0d6, SUC_lexicon_tags_37 },
    { 0x7bd3f0d6, SUC_lexicon_tags_25 },
    { 0xd333f0d9, SUC_lexicon_tags_25 },
    { 0x253f0da, SUC_lexicon_tags_28 },
    { 0x1253f0db, SUC_lexicon_tags_25 },
    { 0x1533f0db, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9573f0df, SUC_lexicon_tags_12 },
    { 0xd853f0df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf433f0e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97d3f0e4, SUC_lexicon_tags_2 },
    { 0xe773f0e4, SUC_lexicon_tags_84 },
    { 0x4d13f0e6, SUC_lexicon_tags_17 },
    { 0xdb53f0e7, SUC_lexicon_tags_1 },
    { 0xb613f0e8, SUC_lexicon_tags_26 },
    { 0x5433f0e8, SUC_lexicon_tags_23 },
    { 0x18d3f0ea, SUC_lexicon_tags_11 },
    { 0xecd3f0eb, SUC_lexicon_tags_49 },
    { 0xaa33f0ec, SUC_lexicon_tags_36 },
    { 0xac53f0ec, SUC_lexicon_tags_15 },
    { 0xb253f0ed, SUC_lexicon_tags_12 },
    { 0x69f3f0ef, SUC_lexicon_tags_35 },
    { 0xda33f0e7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb3d3f0f2, SUC_lexicon_tags_2 },
    { 0xd1d3f0f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf793f0f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9eb3f0f9, SUC_lexicon_tags_25 },
    { 0x3e53f0fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b3f106, SUC_lexicon_tags_25 },
    { 0xc73f107, SUC_lexicon_tags_7 },
    { 0x56b3f108, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7153f10a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb073f110, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93b3f112, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333f117, SUC_lexicon_tags_1 },
    { 0xef93f118, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d3f11b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc5f3f11d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d3f120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb13f124, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc493f12c, SUC_lexicon_tags_2 },
    { 0xef73f12d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc3b3f12f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4773f131, SUC_lexicon_tags_29 },
    { 0x8333f131, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc213f135, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c33f137, SUC_lexicon_tags_25 },
    { 0x8b73f137, SUC_lexicon_tags_1 },
    { 0xbf93f138, SUC_lexicon_tags_47 },
    { 0xc413f137, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5073f13e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb53f147, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b53f14c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d3f151, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa193f156, SUC_lexicon_tags_87 },
    { 0x6bb3f157, SUC_lexicon_tags_38 },
    { 0x2473f158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff3f15d, SUC_lexicon_tags_37 },
    { 0x8133f15e, SUC_lexicon_tags_11 },
    { 0x473f15f, SUC_lexicon_tags_25 },
    { 0xed3f160, SUC_lexicon_tags_1 },
    { 0xcf13f15f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d3f166, SUC_lexicon_tags_2 },
    { 0x6e33f167, SUC_lexicon_tags_60 },
    { 0xf213f167, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b3f171, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe993f175, SUC_lexicon_tags_17 },
    { 0xc993f176, SUC_lexicon_tags_30 },
    { 0x3433f177, SUC_lexicon_tags_35 },
    { 0xdb93f178, SUC_lexicon_tags_8 },
    { 0xb0f3f178, SUC_lexicon_tags_47 },
    { 0x55d3f17a, SUC_lexicon_tags_14 },
    { 0xd393f17b, SUC_lexicon_tags_25 },
    { 0x4853f17b, SUC_lexicon_tags_9 },
    { 0xb5b3f17d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xed13f17f, SUC_lexicon_tags_31 },
    { 0x7753f180, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd333f184, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3553f187, SUC_lexicon_tags_941 },
    { 0xf913f188, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c73f18c, SUC_lexicon_tags_15 },
    { 0xae13f18d, SUC_lexicon_tags_1 },
    { 0xc93f18e, SUC_lexicon_tags_17 },
    { 0xf5f3f18f, SUC_lexicon_tags_16 },
    { 0x8753f190, SUC_lexicon_tags_17 },
    { 0x1f33f190, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3053f198, SUC_lexicon_tags_7 },
    { 0x12f3f198, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad3f19e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb13f1a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf733f1a6, SUC_lexicon_tags_21 },
    { 0xd433f1a7, SUC_lexicon_tags_17 },
    { 0xe2b3f1a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c53f1aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4313f1ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe93f1b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8233f1b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1b3f1b8, SUC_lexicon_tags_25 },
    { 0x79b3f1b9, SUC_lexicon_tags_12 },
    { 0x23b3f1ba, SUC_lexicon_tags_15 },
    { 0x8573f1ba, SUC_lexicon_tags_7 },
    { 0xfad3f1b9, SUC_lexicon_tags_33 },
    { 0xfb3f1bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf573f1c3, SUC_lexicon_tags_37 },
    { 0xf913f1c3, SUC_lexicon_tags_11 },
    { 0x9893f1c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2df3f1c7, SUC_lexicon_tags_39 },
    { 0x6bd3f1c8, SUC_lexicon_tags_43 },
    { 0x433f1c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f3f1cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b53f1d1, SUC_lexicon_tags_1 },
    { 0xb7f3f1d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf333f1d5, SUC_lexicon_tags_49 },
    { 0x8673f1d6, SUC_lexicon_tags_5 },
    { 0x1373f1d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6713f1d9, SUC_lexicon_tags_6 },
    { 0xb73f1d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f93f1dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb3f1e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a93f1e4, SUC_lexicon_tags_37 },
    { 0x3bd3f1e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8013f1e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac13f1ed, SUC_lexicon_tags_47 },
    { 0xad53f1ee, SUC_lexicon_tags_37 },
    { 0x6a33f1ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e53f1f2, SUC_lexicon_tags_38 },
    { 0x45d3f1f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f53f1f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd33f1fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6833f1fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d53f200, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe173f202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1553f205, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x45b3f207, SUC_lexicon_tags_31 },
    { 0xe213f208, SUC_lexicon_tags_6 },
    { 0x1b33f209, SUC_lexicon_tags_37 },
    { 0xd993f20a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf253f20c, SUC_lexicon_tags_4 },
    { 0xb893f20d, SUC_lexicon_tags_2 },
    { 0x79d3f20e, SUC_lexicon_tags_15 },
    { 0xc53f20e, SUC_lexicon_tags_33 },
    { 0x33f3f210, SUC_lexicon_tags_6 },
    { 0xd9b3f211, SUC_lexicon_tags_15 },
    { 0xf773f212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93f215, SUC_lexicon_tags_15 },
    { 0x8fd3f216, SUC_lexicon_tags_17 },
    { 0x8473f217, SUC_lexicon_tags_49 },
    { 0x28f3f218, SUC_lexicon_tags_3 },
    { 0x6b73f219, SUC_lexicon_tags_15 },
    { 0x3d53f219, SUC_lexicon_tags_7 },
    { 0x7653f21b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b3f21e, SUC_lexicon_tags_12 },
    { 0xb1f3f21f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab3f226, SUC_lexicon_tags_38 },
    { 0x1f53f226, SUC_lexicon_tags_68 },
    { 0x1df3f228, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c73f22a, SUC_lexicon_tags_15 },
    { 0xa4b3f22a, SUC_lexicon_tags_55 },
    { 0x1d93f22b, SUC_lexicon_tags_49 },
    { 0xc093f22c, SUC_lexicon_tags_6 },
    { 0x4df3f22e, SUC_lexicon_tags_25 },
    { 0xeff3f22f, SUC_lexicon_tags_38 },
    { 0xf7d3f22f, SUC_lexicon_tags_25 },
    { 0x2a73f22f, SUC_lexicon_tags_57 },
    { 0xb473f232, SUC_lexicon_tags_9 },
    { 0x4db3f233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7353f236, SUC_lexicon_tags_60 },
    { 0xa673f237, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x1513f23a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b3f246, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b33f24a, SUC_lexicon_tags_12 },
    { 0x8633f24b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa833f253, SUC_lexicon_tags_37 },
    { 0x2253f253, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6313f25a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5833f260, SUC_lexicon_tags_17 },
    { 0x15f3f260, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7953f267, SUC_lexicon_tags_11 },
    { 0xf6d3f267, SUC_lexicon_tags_43 },
    { 0xe593f269, SUC_lexicon_tags_49 },
    { 0xcb33f26a, SUC_lexicon_tags_6 },
    { 0x6d13f26a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba33f26d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff3f270, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf53f273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d13f276, SUC_lexicon_tags_48 },
    { 0x92f3f277, SUC_lexicon_tags_1 },
    { 0xad53f278, SUC_lexicon_tags_52 },
    { 0x22f3f279, SUC_lexicon_tags_17 },
    { 0x5273f27a, SUC_lexicon_tags_7 },
    { 0xe093f27a, SUC_lexicon_tags_17 },
    { 0x4633f27c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xba53f27f, SUC_lexicon_tags_19 },
    { 0xaff3f280, SUC_lexicon_tags_17 },
    { 0xc4b3f27f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d93f288, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed13f28a, SUC_lexicon_tags_28 },
    { 0xf873f28b, SUC_lexicon_tags_55 },
    { 0xee33f28c, SUC_lexicon_tags_23 },
    { 0x6353f28d, SUC_lexicon_tags_27 },
    { 0x1f73f28a, SUC_lexicon_tags_1 },
    { 0xb933f28f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a13f291, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4a13f293, SUC_lexicon_tags_17 },
    { 0x2a33f293, SUC_lexicon_tags_48 },
    { 0x453f295, SUC_lexicon_tags_26 },
    { 0x54f3f296, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf3f29c, SUC_lexicon_tags_27 },
    { 0xaa53f29c, SUC_lexicon_tags_42 },
    { 0x1dd3f29d, SUC_lexicon_tags_38 },
    { 0x1d93f29f, SUC_lexicon_tags_47 },
    { 0x3f3f2a0, SUC_lexicon_tags_60 },
    { 0x33d3f2a1, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d3f2a9, SUC_lexicon_tags_401 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f53f2ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b3f2b1, SUC_lexicon_tags_61 },
    { 0x8233f2b1, SUC_lexicon_tags_25 },
    { 0xeeb3f2b1, SUC_lexicon_tags_34 },
    { 0x9233f2b4, SUC_lexicon_tags_114 },
    { 0x3653f2b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b3f2b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced3f2c0, SUC_lexicon_tags_55 },
    { 0xd8f3f2c1, SUC_lexicon_tags_17 },
    { 0xd013f2c0, SUC_lexicon_tags_25 },
    { 0x1a53f2c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4873f2c5, SUC_lexicon_tags_35 },
    { 0x6993f2c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1253f2c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3f2cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2c13f2d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f3f2d6, SUC_lexicon_tags_37 },
    { 0x3a13f2d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb3f2dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24b3f2df, SUC_lexicon_tags_15 },
    { 0x8613f2e0, SUC_lexicon_tags_77 },
    { 0xa493f2e0, SUC_lexicon_tags_37 },
    { 0x7533f2e2, SUC_lexicon_tags_17 },
    { 0x8833f2df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e73f2e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1993f2e9, SUC_lexicon_tags_25 },
    { 0x82d3f2ea, SUC_lexicon_tags_37 },
    { 0x8b33f2e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c53f2f1, SUC_lexicon_tags_14 },
    { 0x7613f2f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3f2f5, SUC_lexicon_tags_7 },
    { 0xd5d3f2f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xab53f2f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb3f2fb, SUC_lexicon_tags_42 },
    { 0xc5b3f2fc, SUC_lexicon_tags_15 },
    { 0x5b3f2fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa673f303, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2833f307, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb3f30a, SUC_lexicon_tags_12 },
    { 0x7533f30b, SUC_lexicon_tags_17 },
    { 0x4d53f30c, SUC_lexicon_tags_45 },
    { 0x2cd3f30d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd833f312, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf0b3f314, SUC_lexicon_tags_12 },
    { 0xadd3f315, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd253f317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49d3f319, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b3f321, SUC_lexicon_tags_32 },
    { 0x86f3f322, SUC_lexicon_tags_14 },
    { 0xf8b3f323, SUC_lexicon_tags_17 },
    { 0x40b3f324, SUC_lexicon_tags_93 },
    { 0x3373f324, SUC_lexicon_tags_26 },
    { 0x1c73f325, SUC_lexicon_tags_31 },
    { 0xfd53f327, SUC_lexicon_tags_39 },
    { 0x57f3f328, SUC_lexicon_tags_2 },
    { 0x44b3f327, SUC_lexicon_tags_33 },
    { 0xe0d3f328, SUC_lexicon_tags_6 },
    { 0x97f3f329, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8593f330, SUC_lexicon_tags_607 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f3f333, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6213f335, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6773f338, SUC_lexicon_tags_18 },
    { 0x8e53f339, SUC_lexicon_tags_0 },
    { 0x5013f339, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c33f33c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5293f33e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3213f341, SUC_lexicon_tags_25 },
    { 0xdc53f341, SUC_lexicon_tags_8 },
    { 0xc473f342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3eb3f345, SUC_lexicon_tags_87 },
    { 0x2a93f346, SUC_lexicon_tags_25 },
    { 0xe793f346, SUC_lexicon_tags_33 },
    { 0x7653f345, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb3f34c, SUC_lexicon_tags_61 },
    { 0x5fd3f34d, SUC_lexicon_tags_30 },
    { 0x50d3f34d, SUC_lexicon_tags_1 },
    { 0x36f3f34e, SUC_lexicon_tags_2 },
    { 0xde93f350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e53f353, SUC_lexicon_tags_33 },
    { 0xa533f353, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53f358, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb3f35b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe433f35d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd873f35f, SUC_lexicon_tags_25 },
    { 0xc413f360, SUC_lexicon_tags_17 },
    { 0x6173f35f, SUC_lexicon_tags_25 },
    { 0xa773f362, SUC_lexicon_tags_12 },
    { 0x4df3f363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67f3f365, SUC_lexicon_tags_0 },
    { 0x72b3f366, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10b3f368, SUC_lexicon_tags_25 },
    { 0xa473f369, SUC_lexicon_tags_9 },
    { 0xcab3f368, SUC_lexicon_tags_8 },
    { 0xdb33f36b, SUC_lexicon_tags_30 },
    { 0xd4b3f36b, SUC_lexicon_tags_17 },
    { 0xf4f3f36a, SUC_lexicon_tags_37 },
    { 0x953f36e, SUC_lexicon_tags_2 },
    { 0xbad3f36e, SUC_lexicon_tags_6 },
    { 0x9833f36a, SUC_lexicon_tags_25 },
    { 0x2093f370, SUC_lexicon_tags_15 },
    { 0x73d3f370, SUC_lexicon_tags_12 },
    { 0xc973f372, SUC_lexicon_tags_45 },
    { 0xb453f36d, SUC_lexicon_tags_14 },
    { 0x6bd3f368, SUC_lexicon_tags_55 },
    { 0xad33f376, SUC_lexicon_tags_38 },
    { 0x9b33f371, SUC_lexicon_tags_103 },
    { 0x5b33f378, SUC_lexicon_tags_36 },
    { 0x2d93f379, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xe1f3f37b, SUC_lexicon_tags_14 },
    { 0x7a33f37c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6033f37e, SUC_lexicon_tags_27 },
    { 0xc913f37f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda33f381, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x4673f384, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d93f386, SUC_lexicon_tags_11 },
    { 0xc8f3f387, SUC_lexicon_tags_17 },
    { 0x41f3f386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f73f38f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb973f392, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa313f394, SUC_lexicon_tags_60 },
    { 0x19b3f394, SUC_lexicon_tags_0 },
    { 0xaf73f395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x913f399, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed73f39c, SUC_lexicon_tags_17 },
    { 0x71b3f39d, SUC_lexicon_tags_54 },
    { 0xd313f39e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5073f3a0, SUC_lexicon_tags_2 },
    { 0x1893f3a1, SUC_lexicon_tags_1 },
    { 0xdb33f3a0, SUC_lexicon_tags_14 },
    { 0x2e13f3a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa313f3a5, SUC_lexicon_tags_14 },
    { 0xa1b3f3a5, SUC_lexicon_tags_45 },
    { 0xc1f3f3a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3413f3a9, SUC_lexicon_tags_1 },
    { 0xb513f3aa, SUC_lexicon_tags_34 },
    { 0x2313f3a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d3f3b2, SUC_lexicon_tags_25 },
    { 0x1f73f3b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdf3f3b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd93f3b7, SUC_lexicon_tags_30 },
    { 0x9693f3b7, SUC_lexicon_tags_26 },
    { 0xdb13f3b9, SUC_lexicon_tags_32 },
    { 0xbfd3f3ba, SUC_lexicon_tags_0 },
    { 0x3bd3f3bb, SUC_lexicon_tags_25 },
    { 0x41b3f3bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd333f3bf, SUC_lexicon_tags_10 },
    { 0xec13f3c0, SUC_lexicon_tags_38 },
    { 0xd973f3c0, SUC_lexicon_tags_14 },
    { 0x8bb3f3c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf3f3c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb3f3cb, SUC_lexicon_tags_28 },
    { 0xbd13f3cc, SUC_lexicon_tags_11 },
    { 0xe033f3cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfb3f3cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5973f3d2, SUC_lexicon_tags_7 },
    { 0x5573f3d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c33f3d9, SUC_lexicon_tags_26 },
    { 0xb73f3da, SUC_lexicon_tags_27 },
    { 0x2db3f3db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3bb3f3dd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f73f3e0, SUC_lexicon_tags_7 },
    { 0xd873f3e1, SUC_lexicon_tags_11 },
    { 0x3673f3e2, SUC_lexicon_tags_16 },
    { 0xe793f3e3, SUC_lexicon_tags_8 },
    { 0xb293f3e4, SUC_lexicon_tags_1 },
    { 0xc393f3e2, SUC_lexicon_tags_17 },
    { 0xeb93f3e5, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b33f3ea, SUC_lexicon_tags_37 },
    { 0xd7d3f3eb, SUC_lexicon_tags_37 },
    { 0xa0b3f3ea, SUC_lexicon_tags_191 },
    { 0xe3f3f3eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f3f3f2, SUC_lexicon_tags_19 },
    { 0x9333f3f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf173f3f5, SUC_lexicon_tags_17 },
    { 0x3ed3f3f6, SUC_lexicon_tags_4 },
    { 0xd213f3f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf573f3fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af3f3ff, SUC_lexicon_tags_12 },
    { 0xb33f3ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9233f402, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253f406, SUC_lexicon_tags_31 },
    { 0x38f3f407, SUC_lexicon_tags_30 },
    { 0x3393f406, SUC_lexicon_tags_10 },
    { 0xb1d3f409, SUC_lexicon_tags_2 },
    { 0x7cb3f40a, SUC_lexicon_tags_9 },
    { 0xe973f40b, SUC_lexicon_tags_7 },
    { 0x7733f40c, SUC_lexicon_tags_7 },
    { 0x23d3f40d, SUC_lexicon_tags_1 },
    { 0x2973f40e, SUC_lexicon_tags_17 },
    { 0x9dd3f40d, SUC_lexicon_tags_15 },
    { 0x5273f410, SUC_lexicon_tags_14 },
    { 0xf873f411, SUC_lexicon_tags_25 },
    { 0x2733f40f, SUC_lexicon_tags_63 },
    { 0xc993f413, SUC_lexicon_tags_7 },
    { 0x78d3f40d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49d3f416, SUC_lexicon_tags_9 },
    { 0x1e73f416, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6953f41d, SUC_lexicon_tags_15 },
    { 0xf353f41d, SUC_lexicon_tags_7 },
    { 0x7333f41f, SUC_lexicon_tags_17 },
    { 0x9c73f420, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f3f424, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x793f426, SUC_lexicon_tags_11 },
    { 0xf713f427, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb433f42a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f53f42d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d3f430, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9893f432, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7b53f434, SUC_lexicon_tags_7 },
    { 0x6bd3f435, SUC_lexicon_tags_13 },
    { 0x9573f435, SUC_lexicon_tags_17 },
    { 0xdf73f437, SUC_lexicon_tags_91 },
    { 0x7d13f438, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8833f43a, SUC_lexicon_tags_43 },
    { 0x6753f43b, SUC_lexicon_tags_47 },
    { 0x27f3f43c, SUC_lexicon_tags_11 },
    { 0x7eb3f43c, SUC_lexicon_tags_37 },
    { 0x1a13f43b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e33f442, SUC_lexicon_tags_39 },
    { 0x4a33f443, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4593f44e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dd3f450, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5ad3f452, SUC_lexicon_tags_15 },
    { 0xc653f453, SUC_lexicon_tags_17 },
    { 0xd053f453, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8bf3f456, SUC_lexicon_tags_43 },
    { 0x1373f456, SUC_lexicon_tags_2 },
    { 0x63f3f458, SUC_lexicon_tags_12 },
    { 0x9a13f458, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8073f45f, SUC_lexicon_tags_1 },
    { 0xbf53f45f, SUC_lexicon_tags_63 },
    { 0x57f3f461, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3073f464, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb3f46b, SUC_lexicon_tags_275 },
    { 0xbb3f46c, SUC_lexicon_tags_17 },
    { 0x73b3f46b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff3f471, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4973f476, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee73f47b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fd3f47d, SUC_lexicon_tags_88 },
    { 0x9d33f47e, SUC_lexicon_tags_47 },
    { 0xd7d3f47f, SUC_lexicon_tags_2 },
    { 0x3a93f480, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2133f482, SUC_lexicon_tags_118 },
    { 0xf593f483, SUC_lexicon_tags_2 },
    { 0x88d3f483, SUC_lexicon_tags_1 },
    { 0xbbb3f483, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3f48d, SUC_lexicon_tags_2 },
    { 0xe5f3f48e, SUC_lexicon_tags_11 },
    { 0x5693f48e, SUC_lexicon_tags_45 },
    { 0x30d3f48d, SUC_lexicon_tags_25 },
    { 0x9b93f491, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb3f496, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc8f3f498, SUC_lexicon_tags_11 },
    { 0xf553f498, SUC_lexicon_tags_46 },
    { 0x13f3f499, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3693f4a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x993f4a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f3f4a8, SUC_lexicon_tags_60 },
    { 0x3b53f4a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf093f4ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x56f3f4af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2273f4b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe373f4ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2193f4bc, SUC_lexicon_tags_9 },
    { 0x77f3f4bd, SUC_lexicon_tags_63 },
    { 0x6df3f4bd, SUC_lexicon_tags_1 },
    { 0xe453f4bf, SUC_lexicon_tags_12 },
    { 0x5373f4bd, SUC_lexicon_tags_41 },
    { 0x38f3f4bd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb3f4c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7233f4c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f3f4cd, SUC_lexicon_tags_33 },
    { 0x9153f4ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6793f4d1, SUC_lexicon_tags_108 },
    { 0x2ad3f4d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe533f4d6, SUC_lexicon_tags_47 },
    { 0x4cd3f4d6, SUC_lexicon_tags_7 },
    { 0xf5f3f4d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db3f4dd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x693f4e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb653f4e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc173f4eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f3f4f1, SUC_lexicon_tags_7 },
    { 0xd173f4f2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d73f4f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8493f4f8, SUC_lexicon_tags_65 },
    { 0x6c53f4f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe313f4ff, SUC_lexicon_tags_37 },
    { 0xba73f500, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc353f503, SUC_lexicon_tags_0 },
    { 0x8453f503, SUC_lexicon_tags_47 },
    { 0xc53f503, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4433f508, SUC_lexicon_tags_30 },
    { 0x7bb3f509, SUC_lexicon_tags_14 },
    { 0x493f509, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c93f50d, SUC_lexicon_tags_1 },
    { 0xcbd3f50d, SUC_lexicon_tags_38 },
    { 0x7b53f50d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9af3f511, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1573f517, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe33f51a, SUC_lexicon_tags_17 },
    { 0xb793f51a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df3f523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab3f52d, SUC_lexicon_tags_11 },
    { 0x533f52d, SUC_lexicon_tags_33 },
    { 0x8273f52f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc13f531, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc93f537, SUC_lexicon_tags_33 },
    { 0x2cb3f538, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d13f53f, SUC_lexicon_tags_7 },
    { 0xecd3f53f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe813f547, SUC_lexicon_tags_37 },
    { 0x5e13f548, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d3f54b, SUC_lexicon_tags_15 },
    { 0xa6b3f54c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe693f54e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4053f556, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7113f559, SUC_lexicon_tags_12 },
    { 0x12d3f55a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd873f562, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d93f565, SUC_lexicon_tags_0 },
    { 0x5013f566, SUC_lexicon_tags_49 },
    { 0xca33f567, SUC_lexicon_tags_7 },
    { 0x3533f568, SUC_lexicon_tags_7 },
    { 0xe413f569, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xf153f56c, SUC_lexicon_tags_0 },
    { 0x6ab3f56d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73f3f56f, SUC_lexicon_tags_8 },
    { 0x2693f570, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e73f577, SUC_lexicon_tags_14 },
    { 0x5f93f577, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f33f57b, SUC_lexicon_tags_15 },
    { 0x5553f57c, SUC_lexicon_tags_25 },
    { 0xd013f57d, SUC_lexicon_tags_32 },
    { 0xd473f57c, SUC_lexicon_tags_11 },
    { 0x7d53f57e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd273f585, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433f58a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6b3f58c, SUC_lexicon_tags_20 },
    { 0x2153f58d, SUC_lexicon_tags_23 },
    { 0xcb13f58d, SUC_lexicon_tags_8 },
    { 0xf7b3f58d, SUC_lexicon_tags_25 },
    { 0xe5d3f58f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xccb3f592, SUC_lexicon_tags_60 },
    { 0xbcf3f593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb3f595, SUC_lexicon_tags_42 },
    { 0xc1f3f596, SUC_lexicon_tags_25 },
    { 0x4233f596, SUC_lexicon_tags_17 },
    { 0x6153f598, SUC_lexicon_tags_19 },
    { 0xebb3f599, SUC_lexicon_tags_11 },
    { 0xb3b3f59a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c53f59c, SUC_lexicon_tags_38 },
    { 0x2e33f59c, SUC_lexicon_tags_60 },
    { 0x4853f59d, SUC_lexicon_tags_60 },
    { 0x97d3f59f, SUC_lexicon_tags_37 },
    { 0x7fb3f5a0, SUC_lexicon_tags_66 },
    { 0x2973f5a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d3f5a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd3f5ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca13f5ad, SUC_lexicon_tags_27 },
    { 0x93f5ae, SUC_lexicon_tags_87 },
    { 0x4f33f5ad, SUC_lexicon_tags_15 },
    { 0xbbb3f5b0, SUC_lexicon_tags_9 },
    { 0x1bf3f5b1, SUC_lexicon_tags_1 },
    { 0xba53f5b1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x3213f5b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f5bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25d3f5bd, SUC_lexicon_tags_63 },
    { 0x42b3f5bd, SUC_lexicon_tags_2 },
    { 0xa513f5be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc33f5c2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xf793f5c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ab3f5c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd93f5cf, SUC_lexicon_tags_11 },
    { 0x8c33f5d0, SUC_lexicon_tags_23 },
    { 0x6833f5cf, SUC_lexicon_tags_17 },
    { 0xd7b3f5cf, SUC_lexicon_tags_10 },
    { 0x4113f5d3, SUC_lexicon_tags_46 },
    { 0x2773f5d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf713f5d6, SUC_lexicon_tags_9 },
    { 0x7e73f5d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb3f5dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc913f5df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e73f5e1, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d93f5e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef73f5ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d73f5ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5193f5ef, SUC_lexicon_tags_46 },
    { 0x6233f5f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4173f5f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea53f5f4, SUC_lexicon_tags_28 },
    { 0xc413f5f5, SUC_lexicon_tags_11 },
    { 0xe4d3f5f5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x11b3f5f8, SUC_lexicon_tags_12 },
    { 0x38d3f5f9, SUC_lexicon_tags_85 },
    { 0x5ff3f5fa, SUC_lexicon_tags_17 },
    { 0x3af3f5fa, SUC_lexicon_tags_7 },
    { 0xcd53f5fc, SUC_lexicon_tags_23 },
    { 0x7cd3f5fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f3f602, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf3f607, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1373f60f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7753f611, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e33f620, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d3f625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1273f629, SUC_lexicon_tags_0 },
    { 0xd373f62a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc013f630, SUC_lexicon_tags_18 },
    { 0x5013f631, SUC_lexicon_tags_11 },
    { 0x72b3f632, SUC_lexicon_tags_7 },
    { 0x8f73f632, SUC_lexicon_tags_29 },
    { 0xa293f631, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa073f636, SUC_lexicon_tags_17 },
    { 0x67b3f636, SUC_lexicon_tags_17 },
    { 0x39f3f637, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0x5e73f63a, SUC_lexicon_tags_60 },
    { 0x65b3f63b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3993f63f, SUC_lexicon_tags_1 },
    { 0xf793f63f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d33f646, SUC_lexicon_tags_25 },
    { 0x1693f647, SUC_lexicon_tags_11 },
    { 0xa1d3f648, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c13f64b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d3f656, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb653f65d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d3f664, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9193f666, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a73f669, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb013f66d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd453f670, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf533f675, SUC_lexicon_tags_11 },
    { 0x1993f676, SUC_lexicon_tags_17 },
    { 0xab13f677, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdc53f679, SUC_lexicon_tags_1 },
    { 0x5db3f679, SUC_lexicon_tags_25 },
    { 0xae93f67b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8073f67d, SUC_lexicon_tags_11 },
    { 0x17b3f67e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8833f681, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfe53f683, SUC_lexicon_tags_7 },
    { 0x1173f684, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb253f687, SUC_lexicon_tags_33 },
    { 0xc3d3f687, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4b3f68a, SUC_lexicon_tags_31 },
    { 0x70d3f68b, SUC_lexicon_tags_19 },
    { 0x92b3f68c, SUC_lexicon_tags_4 },
    { 0xd413f68c, SUC_lexicon_tags_14 },
    { 0xf133f68e, SUC_lexicon_tags_14 },
    { 0x2eb3f68f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa793f693, SUC_lexicon_tags_17 },
    { 0x1873f694, SUC_lexicon_tags_11 },
    { 0xc3b3f695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1353f697, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe33f69a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9653f69f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bd3f6a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f33f6a5, SUC_lexicon_tags_35 },
    { 0x1153f6a6, SUC_lexicon_tags_7 },
    { 0x2973f6a7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1693f6ae, SUC_lexicon_tags_16 },
    { 0xc3b3f6af, SUC_lexicon_tags_12 },
    { 0x20d3f6af, SUC_lexicon_tags_1 },
    { 0x4653f6af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3513f6b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab3f6c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6453f6c6, SUC_lexicon_tags_1 },
    { 0x1773f6c7, SUC_lexicon_tags_37 },
    { 0xb033f6c7, SUC_lexicon_tags_49 },
    { 0xb653f6c8, SUC_lexicon_tags_2 },
    { 0xc793f6ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7973f6ce, SUC_lexicon_tags_31 },
    { 0xb853f6ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1af3f6d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6413f6d3, SUC_lexicon_tags_103 },
    { 0x9e93f6d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f53f6d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb3f6db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc13f6dd, SUC_lexicon_tags_19 },
    { 0x6513f6dd, SUC_lexicon_tags_79 },
    { 0xd273f6df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b3f6e2, SUC_lexicon_tags_9 },
    { 0x7bd3f6e2, SUC_lexicon_tags_31 },
    { 0xdf13f6e4, SUC_lexicon_tags_27 },
    { 0xfb93f6e5, SUC_lexicon_tags_25 },
    { 0x9bb3f6e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f3f6ec, SUC_lexicon_tags_43 },
    { 0x64d3f6ed, SUC_lexicon_tags_37 },
    { 0xab3f6ed, SUC_lexicon_tags_25 },
    { 0xd73f6ef, SUC_lexicon_tags_17 },
    { 0xeeb3f6f0, SUC_lexicon_tags_62 },
    { 0x6493f6f1, SUC_lexicon_tags_23 },
    { 0x57b3f6f2, SUC_lexicon_tags_46 },
    { 0x9213f6f2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d3f6f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa053f6f9, SUC_lexicon_tags_46 },
    { 0xb273f6fa, SUC_lexicon_tags_45 },
    { 0x6013f6fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b3f6ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b73f701, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf833f707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9733f70d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6413f711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f3f714, SUC_lexicon_tags_19 },
    { 0x3e73f715, SUC_lexicon_tags_0 },
    { 0xc8b3f716, SUC_lexicon_tags_12 },
    { 0x4113f716, SUC_lexicon_tags_37 },
    { 0x3a93f718, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ed3f71a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f3f71d, SUC_lexicon_tags_12 },
    { 0xd93f71e, SUC_lexicon_tags_17 },
    { 0x7d13f71f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf793f723, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3993f725, SUC_lexicon_tags_1 },
    { 0x4633f725, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2233f72b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e93f72f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab3f733, SUC_lexicon_tags_10 },
    { 0xd93f734, SUC_lexicon_tags_23 },
    { 0xbd73f733, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3553f738, SUC_lexicon_tags_19 },
    { 0x1ad3f738, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad33f741, SUC_lexicon_tags_61 },
    { 0x3393f742, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x773f744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd513f748, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6993f74a, SUC_lexicon_tags_26 },
    { 0xce13f74b, SUC_lexicon_tags_2 },
    { 0xeb13f74a, SUC_lexicon_tags_15 },
    { 0xcef3f74d, SUC_lexicon_tags_25 },
    { 0x3cb3f74e, SUC_lexicon_tags_87 },
    { 0xfeb3f74f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8b53f751, SUC_lexicon_tags_49 },
    { 0x1933f752, SUC_lexicon_tags_873 },
    { 0x18f3f753, SUC_lexicon_tags_17 },
    { 0x4bf3f754, SUC_lexicon_tags_31 },
    { 0xafb3f755, SUC_lexicon_tags_21 },
    { 0xc9d3f755, SUC_lexicon_tags_66 },
    { 0x9093f757, SUC_lexicon_tags_31 },
    { 0x7a53f755, SUC_lexicon_tags_46 },
    { 0x36b3f759, SUC_lexicon_tags_1 },
    { 0x5453f754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b3f75e, SUC_lexicon_tags_26 },
    { 0x36d3f75e, SUC_lexicon_tags_11 },
    { 0x9e73f75e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3653f768, SUC_lexicon_tags_21 },
    { 0xcc33f769, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x473f76b, SUC_lexicon_tags_2 },
    { 0x86b3f76c, SUC_lexicon_tags_38 },
    { 0xdf73f76c, SUC_lexicon_tags_11 },
    { 0xa4f3f76d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd813f772, SUC_lexicon_tags_37 },
    { 0x6af3f772, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79f3f775, SUC_lexicon_tags_23 },
    { 0xfcb3f776, SUC_lexicon_tags_37 },
    { 0x5a33f776, SUC_lexicon_tags_16 },
    { 0x44f3f776, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc53f77d, SUC_lexicon_tags_13 },
    { 0x95b3f77d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373f782, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3f785, SUC_lexicon_tags_63 },
    { 0xc873f786, SUC_lexicon_tags_1 },
    { 0xcc33f785, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb393f78a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd493f78c, SUC_lexicon_tags_11 },
    { 0xda73f78d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb3f790, SUC_lexicon_tags_17 },
    { 0xa613f791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd3f794, SUC_lexicon_tags_1 },
    { 0x6373f794, SUC_lexicon_tags_11 },
    { 0xa53f796, SUC_lexicon_tags_2 },
    { 0xf093f795, SUC_lexicon_tags_15 },
    { 0x6453f798, SUC_lexicon_tags_65 },
    { 0xacb3f798, SUC_lexicon_tags_37 },
    { 0x1693f796, SUC_lexicon_tags_23 },
    { 0xc053f79b, SUC_lexicon_tags_26 },
    { 0x6af3f79c, SUC_lexicon_tags_87 },
    { 0x7ef3f79c, SUC_lexicon_tags_2 },
    { 0xf993f796, SUC_lexicon_tags_47 },
    { 0xba13f79f, SUC_lexicon_tags_45 },
    { 0xc473f796, SUC_lexicon_tags_0 },
    { 0x633f7a1, SUC_lexicon_tags_2 },
    { 0x633f7a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d3f7a5, SUC_lexicon_tags_11 },
    { 0x5853f7a6, SUC_lexicon_tags_7 },
    { 0x43d3f7a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7353f7ac, SUC_lexicon_tags_15 },
    { 0x4753f7ad, SUC_lexicon_tags_25 },
    { 0xddf3f7ae, SUC_lexicon_tags_7 },
    { 0x3173f7ad, SUC_lexicon_tags_61 },
    { 0x5eb3f7b0, SUC_lexicon_tags_1 },
    { 0xbed3f7af, SUC_lexicon_tags_0 },
    { 0x8853f7b2, SUC_lexicon_tags_75 },
    { 0xfa93f7b3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f3f7b6, SUC_lexicon_tags_21 },
    { 0x16d3f7b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b3f7bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6153f7bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xaa13f7bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xecb3f7c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb4b3f7c3, SUC_lexicon_tags_17 },
    { 0xec93f7c4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d3f7c9, SUC_lexicon_tags_37 },
    { 0x3d33f7ca, SUC_lexicon_tags_15 },
    { 0xc8b3f7cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c93f7cf, SUC_lexicon_tags_11 },
    { 0x3653f7cf, SUC_lexicon_tags_27 },
    { 0x1a53f7d1, SUC_lexicon_tags_47 },
    { 0x3d53f7d2, SUC_lexicon_tags_15 },
    { 0x8373f7d3, SUC_lexicon_tags_30 },
    { 0x4773f7d3, SUC_lexicon_tags_47 },
    { 0xddb3f7d3, SUC_lexicon_tags_15 },
    { 0xc2d3f7d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f3f7d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc4d3f7db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7893f7df, SUC_lexicon_tags_92 },
    { 0x5fb3f7e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc693f7e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df3f7e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b3f7eb, SUC_lexicon_tags_37 },
    { 0xff73f7ec, SUC_lexicon_tags_26 },
    { 0x4f13f7ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f33f7f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93f7f4, SUC_lexicon_tags_17 },
    { 0x88d3f7f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97b3f7f7, SUC_lexicon_tags_37 },
    { 0xfd3f7f7, SUC_lexicon_tags_11 },
    { 0xc33f7f9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef3f7fd, SUC_lexicon_tags_7 },
    { 0x56d3f7fe, SUC_lexicon_tags_15 },
    { 0xd473f7ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b3f802, SUC_lexicon_tags_1 },
    { 0xfb13f803, SUC_lexicon_tags_15 },
    { 0x6cf3f804, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6753f808, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b3f80b, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb93f80f, SUC_lexicon_tags_12 },
    { 0xd5b3f810, SUC_lexicon_tags_27 },
    { 0xb633f811, SUC_lexicon_tags_41 },
    { 0x7b3f811, SUC_lexicon_tags_11 },
    { 0xfe13f80f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3c93f815, SUC_lexicon_tags_12 },
    { 0xc553f816, SUC_lexicon_tags_463 },
    { 0, NULL },
    { 0x5d53f818, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373f81c, SUC_lexicon_tags_25 },
    { 0xbab3f81c, SUC_lexicon_tags_11 },
    { 0x6813f81e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc033f822, SUC_lexicon_tags_6 },
    { 0xa593f823, SUC_lexicon_tags_17 },
    { 0x9313f824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3f82b, SUC_lexicon_tags_37 },
    { 0x9d13f82c, SUC_lexicon_tags_145 },
    { 0x7173f82c, SUC_lexicon_tags_60 },
    { 0xc533f82e, SUC_lexicon_tags_17 },
    { 0xa933f82e, SUC_lexicon_tags_15 },
    { 0x48f3f830, SUC_lexicon_tags_1 },
    { 0x24b3f82f, SUC_lexicon_tags_25 },
    { 0x5d33f832, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1f3f834, SUC_lexicon_tags_235 },
    { 0xa053f835, SUC_lexicon_tags_11 },
    { 0x9ab3f836, SUC_lexicon_tags_47 },
    { 0x3073f835, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8573f83d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b3f840, SUC_lexicon_tags_2 },
    { 0xa5f3f841, SUC_lexicon_tags_28 },
    { 0xfc33f840, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56f3f844, SUC_lexicon_tags_38 },
    { 0x9e93f844, SUC_lexicon_tags_151 },
    { 0xf2b3f846, SUC_lexicon_tags_15 },
    { 0x5233f847, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d3f850, SUC_lexicon_tags_17 },
    { 0xd3b3f851, SUC_lexicon_tags_2 },
    { 0xed3f852, SUC_lexicon_tags_25 },
    { 0xc2b3f852, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b33f855, SUC_lexicon_tags_12 },
    { 0xfb13f856, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa993f858, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8b3f85a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42f3f85c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe973f85e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf3b3f860, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8373f863, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e53f865, SUC_lexicon_tags_17 },
    { 0x7bb3f865, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa433f869, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68d3f86b, SUC_lexicon_tags_15 },
    { 0xb73f86c, SUC_lexicon_tags_12 },
    { 0x22b3f86c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf13f86f, SUC_lexicon_tags_15 },
    { 0xd353f86f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd3f879, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b3f87f, SUC_lexicon_tags_11 },
    { 0xdf93f880, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf213f882, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8753f885, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f53f890, SUC_lexicon_tags_2 },
    { 0xad53f891, SUC_lexicon_tags_17 },
    { 0xbd93f891, SUC_lexicon_tags_103 },
    { 0x10f3f892, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1773f897, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b3f89b, SUC_lexicon_tags_37 },
    { 0x2f73f89b, SUC_lexicon_tags_23 },
    { 0xd493f89d, SUC_lexicon_tags_60 },
    { 0xa13f89e, SUC_lexicon_tags_17 },
    { 0x4473f89f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4473f8a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0d3f8a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f3f8a6, SUC_lexicon_tags_16 },
    { 0x95b3f8a7, SUC_lexicon_tags_13 },
    { 0xe2d3f8a8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9973f8aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3f8b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f3f8b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb3f8bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef13f8c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9633f8c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc213f8c6, SUC_lexicon_tags_82 },
    { 0x6813f8c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a33f8ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xda53f8cd, SUC_lexicon_tags_11 },
    { 0xc7d3f8cd, SUC_lexicon_tags_9 },
    { 0x8a93f8cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf133f8d3, SUC_lexicon_tags_25 },
    { 0xbcf3f8d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b3f8d7, SUC_lexicon_tags_26 },
    { 0xe9d3f8d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2113f8db, SUC_lexicon_tags_103 },
    { 0x88f3f8dc, SUC_lexicon_tags_82 },
    { 0xe593f8dd, SUC_lexicon_tags_63 },
    { 0xac13f8db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd3f8e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd753f8e3, SUC_lexicon_tags_6 },
    { 0x733f8e4, SUC_lexicon_tags_17 },
    { 0x6cf3f8e5, SUC_lexicon_tags_2 },
    { 0x9473f8e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde93f8ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ef3f8ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e53f8f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc753f8f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b3f8f9, SUC_lexicon_tags_34 },
    { 0xadf3f8fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe573f904, SUC_lexicon_tags_17 },
    { 0xfcb3f904, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6513f907, SUC_lexicon_tags_49 },
    { 0xc653f908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe953f90c, SUC_lexicon_tags_145 },
    { 0x82d3f90d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f3f910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4213f913, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8853f916, SUC_lexicon_tags_146 },
    { 0x3f3f916, SUC_lexicon_tags_3 },
    { 0xd433f918, SUC_lexicon_tags_37 },
    { 0x38f3f919, SUC_lexicon_tags_2 },
    { 0xe4d3f91a, SUC_lexicon_tags_55 },
    { 0x1af3f91a, SUC_lexicon_tags_26 },
    { 0x4713f91a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb933f926, SUC_lexicon_tags_57 },
    { 0x7b3f927, SUC_lexicon_tags_11 },
    { 0x1c53f928, SUC_lexicon_tags_17 },
    { 0x2033f929, SUC_lexicon_tags_17 },
    { 0xec13f92a, SUC_lexicon_tags_17 },
    { 0xf8b3f92a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb553f92d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6293f930, SUC_lexicon_tags_37 },
    { 0x6673f931, SUC_lexicon_tags_11 },
    { 0x59f3f931, SUC_lexicon_tags_60 },
    { 0xb0f3f933, SUC_lexicon_tags_11 },
    { 0x7533f934, SUC_lexicon_tags_12 },
    { 0x3553f934, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4453f937, SUC_lexicon_tags_57 },
    { 0x3e53f938, SUC_lexicon_tags_12 },
    { 0xa473f939, SUC_lexicon_tags_23 },
    { 0x1bb3f93a, SUC_lexicon_tags_56 },
    { 0x653f93b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3393f93e, SUC_lexicon_tags_25 },
    { 0x2e73f93f, SUC_lexicon_tags_12 },
    { 0x2c73f940, SUC_lexicon_tags_68 },
    { 0x6b93f941, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8033f943, SUC_lexicon_tags_47 },
    { 0xf933f944, SUC_lexicon_tags_25 },
    { 0x9493f945, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f3f949, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc593f951, SUC_lexicon_tags_14 },
    { 0x23d3f952, SUC_lexicon_tags_37 },
    { 0x9f3f952, SUC_lexicon_tags_45 },
    { 0x81b3f954, SUC_lexicon_tags_37 },
    { 0x57d3f955, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f958, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2213f95a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd3f95d, SUC_lexicon_tags_16 },
    { 0xdfd3f95e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd13f960, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f963, SUC_lexicon_tags_21 },
    { 0x7cd3f964, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b3f96a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe1b3f96c, SUC_lexicon_tags_11 },
    { 0x1ad3f96d, SUC_lexicon_tags_8 },
    { 0xf313f96d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f3f975, SUC_lexicon_tags_2 },
    { 0x1c93f975, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6073f978, SUC_lexicon_tags_15 },
    { 0xd133f979, SUC_lexicon_tags_45 },
    { 0xcc53f978, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f3f97f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb93f988, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b13f98a, SUC_lexicon_tags_15 },
    { 0xef13f98a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc693f995, SUC_lexicon_tags_12 },
    { 0x7e73f996, SUC_lexicon_tags_11 },
    { 0x4e33f997, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153f99b, SUC_lexicon_tags_23 },
    { 0x9cf3f99b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f73f99e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5993f9a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b93f9a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f53f9ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xba33f9b0, SUC_lexicon_tags_17 },
    { 0x22f3f9b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf893f9b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f9c0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd53f9c3, SUC_lexicon_tags_26 },
    { 0x2bd3f9c4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x32f3f9c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb93f9c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f3f9cd, SUC_lexicon_tags_6 },
    { 0xf733f9ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde93f9d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b33f9d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33f9e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd533f9e6, SUC_lexicon_tags_33 },
    { 0x773f9e6, SUC_lexicon_tags_26 },
    { 0xa0f3f9e6, SUC_lexicon_tags_2 },
    { 0x8773f9e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4993f9ef, SUC_lexicon_tags_0 },
    { 0x80d3f9f0, SUC_lexicon_tags_11 },
    { 0xa633f9f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6f3f9f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3053f9fb, SUC_lexicon_tags_45 },
    { 0x4793f9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d3fa00, SUC_lexicon_tags_60 },
    { 0xb113fa00, SUC_lexicon_tags_22 },
    { 0x70f3fa02, SUC_lexicon_tags_60 },
    { 0xcfd3fa03, SUC_lexicon_tags_0 },
    { 0x7433fa04, SUC_lexicon_tags_3 },
    { 0x5ef3fa03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b53fa08, SUC_lexicon_tags_7 },
    { 0x53f3fa08, SUC_lexicon_tags_14 },
    { 0x7513fa0a, SUC_lexicon_tags_85 },
    { 0xc8f3fa0b, SUC_lexicon_tags_42 },
    { 0xb7b3fa0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9493fa12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f3fa18, SUC_lexicon_tags_5 },
    { 0xa993fa19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa513fa21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f3fa25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd93fa27, SUC_lexicon_tags_55 },
    { 0x8d73fa28, SUC_lexicon_tags_37 },
    { 0x4e13fa29, SUC_lexicon_tags_25 },
    { 0x4893fa27, SUC_lexicon_tags_12 },
    { 0xddf3fa2a, SUC_lexicon_tags_28 },
    { 0x8413fa2c, SUC_lexicon_tags_15 },
    { 0x1513fa28, SUC_lexicon_tags_56 },
    { 0x29d3fa2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae33fa30, SUC_lexicon_tags_37 },
    { 0x4753fa30, SUC_lexicon_tags_8 },
    { 0xcbf3fa32, SUC_lexicon_tags_17 },
    { 0x3ff3fa33, SUC_lexicon_tags_15 },
    { 0xf653fa32, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x113fa36, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3653fa39, SUC_lexicon_tags_87 },
    { 0x7953fa3a, SUC_lexicon_tags_16 },
    { 0xfe53fa3b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfab3fa3d, SUC_lexicon_tags_55 },
    { 0xe913fa3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e33fa40, SUC_lexicon_tags_29 },
    { 0x5973fa41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1813fa45, SUC_lexicon_tags_46 },
    { 0xc2f3fa46, SUC_lexicon_tags_116 },
    { 0x9433fa45, SUC_lexicon_tags_7 },
    { 0x6013fa48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2eb3fa4a, SUC_lexicon_tags_31 },
    { 0x64b3fa4b, SUC_lexicon_tags_31 },
    { 0xec13fa4a, SUC_lexicon_tags_10 },
    { 0xc213fa4d, SUC_lexicon_tags_15 },
    { 0xc9f3fa4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf073fa54, SUC_lexicon_tags_60 },
    { 0x1a53fa55, SUC_lexicon_tags_12 },
    { 0x9813fa56, SUC_lexicon_tags_6 },
    { 0x3a33fa54, SUC_lexicon_tags_11 },
    { 0x2af3fa54, SUC_lexicon_tags_7 },
    { 0xa9f3fa55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2413fa60, SUC_lexicon_tags_11 },
    { 0x273fa61, SUC_lexicon_tags_39 },
    { 0x8453fa60, SUC_lexicon_tags_10 },
    { 0x5e93fa62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf13fa65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf253fa6a, SUC_lexicon_tags_63 },
    { 0x8f3fa6b, SUC_lexicon_tags_39 },
    { 0x2053fa6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbaf3fa6e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f13fa72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9633fa74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f3fa7c, SUC_lexicon_tags_19 },
    { 0x5953fa7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf3fa84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f3fa87, SUC_lexicon_tags_17 },
    { 0xe5b3fa88, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e53fa91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd73fa95, SUC_lexicon_tags_4 },
    { 0x54f3fa96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3413fa99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc13fa9c, SUC_lexicon_tags_9 },
    { 0x27f3fa9d, SUC_lexicon_tags_26 },
    { 0xfff3fa9e, SUC_lexicon_tags_37 },
    { 0x5d93fa9e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4393faa6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x11f3faa8, SUC_lexicon_tags_40 },
    { 0x8db3faa8, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f93faad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb613faaf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b3fab4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0d3fab6, SUC_lexicon_tags_15 },
    { 0xa173fab7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce13fab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1273fabd, SUC_lexicon_tags_25 },
    { 0xa373fabd, SUC_lexicon_tags_2 },
    { 0x68f3fabf, SUC_lexicon_tags_33 },
    { 0x91d3fabe, SUC_lexicon_tags_11 },
    { 0xc4f3fabe, SUC_lexicon_tags_1 },
    { 0xcf3fac2, SUC_lexicon_tags_17 },
    { 0xb173fac2, SUC_lexicon_tags_17 },
    { 0x9af3fabe, SUC_lexicon_tags_30 },
    { 0x99f3fac5, SUC_lexicon_tags_1 },
    { 0xd3f3fac6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc173faca, SUC_lexicon_tags_17 },
    { 0x1e53facb, SUC_lexicon_tags_253 },
    { 0xfff3faca, SUC_lexicon_tags_32 },
    { 0x4c73facb, SUC_lexicon_tags_61 },
    { 0x53d3face, SUC_lexicon_tags_31 },
    { 0xdaf3facc, SUC_lexicon_tags_0 },
    { 0x2f93fad0, SUC_lexicon_tags_43 },
    { 0x9bb3fad0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db3fad5, SUC_lexicon_tags_14 },
    { 0x3533fad5, SUC_lexicon_tags_25 },
    { 0x5113fad6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcf33fad9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb113fade, SUC_lexicon_tags_2 },
    { 0xc893fade, SUC_lexicon_tags_30 },
    { 0x69d3fadf, SUC_lexicon_tags_37 },
    { 0xe593fadf, SUC_lexicon_tags_0 },
    { 0x933fae2, SUC_lexicon_tags_46 },
    { 0x9653fae3, SUC_lexicon_tags_9 },
    { 0xfdb3fae1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd13fae6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3533faf2, SUC_lexicon_tags_18 },
    { 0x8693faf3, SUC_lexicon_tags_14 },
    { 0x83f3faf2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd3faf8, SUC_lexicon_tags_11 },
    { 0xb713faf8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5633fafd, SUC_lexicon_tags_37 },
    { 0x1433fafe, SUC_lexicon_tags_37 },
    { 0x8653faff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f3fb02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3133fb05, SUC_lexicon_tags_38 },
    { 0xcd33fb06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e93fb0b, SUC_lexicon_tags_23 },
    { 0xe713fb0c, SUC_lexicon_tags_59 },
    { 0x3e33fb0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f33fb0f, SUC_lexicon_tags_3 },
    { 0x9e73fb10, SUC_lexicon_tags_49 },
    { 0xd093fb10, SUC_lexicon_tags_17 },
    { 0x2753fb12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa733fb16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd73fb1a, SUC_lexicon_tags_54 },
    { 0x4653fb1b, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf93fb20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x44f3fb22, SUC_lexicon_tags_2 },
    { 0xf333fb23, SUC_lexicon_tags_45 },
    { 0xc7d3fb22, SUC_lexicon_tags_2 },
    { 0x8093fb23, SUC_lexicon_tags_15 },
    { 0xbb3fb25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3473fb2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x13fb2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e73fb30, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2813fb33, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1673fb36, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe873fb3c, SUC_lexicon_tags_17 },
    { 0x1733fb3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd3fb41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9873fb44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3713fb47, SUC_lexicon_tags_6 },
    { 0xefd3fb48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd053fb4a, SUC_lexicon_tags_8 },
    { 0xd4d3fb4b, SUC_lexicon_tags_57 },
    { 0xd453fb4c, SUC_lexicon_tags_15 },
    { 0x8633fb4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb3fb51, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd13fb55, SUC_lexicon_tags_60 },
    { 0xce13fb55, SUC_lexicon_tags_37 },
    { 0x8153fb57, SUC_lexicon_tags_45 },
    { 0x3ff3fb58, SUC_lexicon_tags_61 },
    { 0x6593fb56, SUC_lexicon_tags_29 },
    { 0xff93fb58, SUC_lexicon_tags_37 },
    { 0xf373fb5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f3fb60, SUC_lexicon_tags_2 },
    { 0x4c13fb61, SUC_lexicon_tags_33 },
    { 0xed33fb62, SUC_lexicon_tags_25 },
    { 0xfc33fb63, SUC_lexicon_tags_14 },
    { 0x4693fb64, SUC_lexicon_tags_60 },
    { 0x1d13fb63, SUC_lexicon_tags_21 },
    { 0x95b3fb64, SUC_lexicon_tags_2 },
    { 0x2313fb67, SUC_lexicon_tags_468 },
    { 0x4f3fb68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf013fb6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a73fb6e, SUC_lexicon_tags_15 },
    { 0x8c33fb6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af3fb73, SUC_lexicon_tags_17 },
    { 0xf0f3fb73, SUC_lexicon_tags_37 },
    { 0x7873fb73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93fb7a, SUC_lexicon_tags_17 },
    { 0x33f3fb7b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f53fb7e, SUC_lexicon_tags_25 },
    { 0x9ad3fb7f, SUC_lexicon_tags_60 },
    { 0x6613fb80, SUC_lexicon_tags_39 },
    { 0x1f93fb81, SUC_lexicon_tags_45 },
    { 0x5fd3fb81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd3fb8a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9df3fb8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb113fb8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e53fb90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b13fb94, SUC_lexicon_tags_7 },
    { 0xb3f3fb95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8373fb9b, SUC_lexicon_tags_1 },
    { 0x8293fb9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b3fba2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b3fba6, SUC_lexicon_tags_29 },
    { 0xf673fba6, SUC_lexicon_tags_25 },
    { 0x1a13fba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f3fbaf, SUC_lexicon_tags_27 },
    { 0xa933fbb0, SUC_lexicon_tags_39 },
    { 0xe093fbb1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdc73fbb3, SUC_lexicon_tags_11 },
    { 0x1993fbb4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd173fbb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa653fbb8, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d3fbbb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d3fbbe, SUC_lexicon_tags_3 },
    { 0xcf3fbbe, SUC_lexicon_tags_2 },
    { 0xbd93fbc0, SUC_lexicon_tags_10 },
    { 0x3d3fbc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1373fbc4, SUC_lexicon_tags_1 },
    { 0x7473fbc5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f3fbc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1613fbcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0f3fbcd, SUC_lexicon_tags_2 },
    { 0xee53fbce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8333fbd6, SUC_lexicon_tags_25 },
    { 0x7813fbd7, SUC_lexicon_tags_17 },
    { 0x4f33fbd7, SUC_lexicon_tags_47 },
    { 0xee13fbd7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633fbde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7613fbe2, SUC_lexicon_tags_39 },
    { 0x6a33fbe3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc533fbe7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd3fbec, SUC_lexicon_tags_38 },
    { 0x5c73fbed, SUC_lexicon_tags_33 },
    { 0xab3fbee, SUC_lexicon_tags_12 },
    { 0x6cf3fbee, SUC_lexicon_tags_37 },
    { 0xbcf3fbf0, SUC_lexicon_tags_6 },
    { 0x8973fbec, SUC_lexicon_tags_38 },
    { 0xbc73fbef, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7793fbf7, SUC_lexicon_tags_13 },
    { 0xe333fbf8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73fbfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd3fbff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5553fc04, SUC_lexicon_tags_63 },
    { 0x2273fc05, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5d13fc07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd3fc11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90d3fc13, SUC_lexicon_tags_33 },
    { 0xcff3fc14, SUC_lexicon_tags_10 },
    { 0xf933fc15, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8833fc17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb293fc19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff3fc1c, SUC_lexicon_tags_45 },
    { 0x86b3fc1c, SUC_lexicon_tags_1 },
    { 0x2773fc1e, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf393fc22, SUC_lexicon_tags_12 },
    { 0x30f3fc23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9773fc25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee93fc2a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc753fc30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b3fc3b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d3fc40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb773fc47, SUC_lexicon_tags_17 },
    { 0x3233fc48, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb2f3fc4a, SUC_lexicon_tags_37 },
    { 0xc13fc4b, SUC_lexicon_tags_38 },
    { 0x32f3fc4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53fc50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b53fc52, SUC_lexicon_tags_25 },
    { 0x1a13fc53, SUC_lexicon_tags_38 },
    { 0xbb3fc53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a93fc57, SUC_lexicon_tags_13 },
    { 0xa373fc58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d93fc5a, SUC_lexicon_tags_37 },
    { 0xfc93fc5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf3fc5f, SUC_lexicon_tags_1 },
    { 0x3b53fc5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5853fc64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9633fc66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b3fc69, SUC_lexicon_tags_9 },
    { 0xad3fc6a, SUC_lexicon_tags_7 },
    { 0x14f3fc6b, SUC_lexicon_tags_11 },
    { 0xc4d3fc6b, SUC_lexicon_tags_0 },
    { 0x8393fc6d, SUC_lexicon_tags_33 },
    { 0x5593fc6b, SUC_lexicon_tags_11 },
    { 0xabd3fc6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7f3fc71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf33fc73, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1933fc7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e13fc82, SUC_lexicon_tags_92 },
    { 0xd4f3fc83, SUC_lexicon_tags_11 },
    { 0xbf53fc84, SUC_lexicon_tags_14 },
    { 0x25b3fc85, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x43b3fc87, SUC_lexicon_tags_37 },
    { 0xb13fc88, SUC_lexicon_tags_15 },
    { 0x9e53fc87, SUC_lexicon_tags_49 },
    { 0xc7f3fc8a, SUC_lexicon_tags_87 },
    { 0x8053fc8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf393fc90, SUC_lexicon_tags_47 },
    { 0xf473fc91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3893fc94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b93fc96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d73fc99, SUC_lexicon_tags_15 },
    { 0xf493fc9a, SUC_lexicon_tags_25 },
    { 0xf953fc9b, SUC_lexicon_tags_82 },
    { 0x2293fc9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8333fc9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c73fca1, SUC_lexicon_tags_77 },
    { 0xe7d3fca2, SUC_lexicon_tags_62 },
    { 0x1873fca3, SUC_lexicon_tags_37 },
    { 0xd7f3fca4, SUC_lexicon_tags_12 },
    { 0xbd33fca2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3713fca7, SUC_lexicon_tags_37 },
    { 0x9b73fca8, SUC_lexicon_tags_11 },
    { 0x7cb3fca9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9eb3fcab, SUC_lexicon_tags_33 },
    { 0xfb93fcac, SUC_lexicon_tags_15 },
    { 0x87f3fcad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33f3fcaf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4813fcb1, SUC_lexicon_tags_15 },
    { 0x9993fcb1, SUC_lexicon_tags_63 },
    { 0xe053fcb1, SUC_lexicon_tags_26 },
    { 0x3933fcb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d3fcba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1973fcbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d3fcc3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xac73fcc6, SUC_lexicon_tags_846 },
    { 0x8b93fcc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a13fcca, SUC_lexicon_tags_11 },
    { 0x94f3fccb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b3fcd0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853fcd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f3fcd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb5f3fcd9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xda93fcdb, SUC_lexicon_tags_11 },
    { 0x75b3fcdc, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9733fce1, SUC_lexicon_tags_38 },
    { 0x6173fce2, SUC_lexicon_tags_38 },
    { 0x8493fce3, SUC_lexicon_tags_6 },
    { 0x3633fce3, SUC_lexicon_tags_13 },
    { 0x34d3fce5, SUC_lexicon_tags_31 },
    { 0xdd3fce3, SUC_lexicon_tags_37 },
    { 0x1f13fce4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb953fcea, SUC_lexicon_tags_15 },
    { 0xca53fceb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef3fcef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf173fcf3, SUC_lexicon_tags_31 },
    { 0x7653fcf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7573fcf6, SUC_lexicon_tags_47 },
    { 0xb153fcf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d53fcfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d3fcfe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd513fd00, SUC_lexicon_tags_4 },
    { 0x9af3fd00, SUC_lexicon_tags_31 },
    { 0x6013fd00, SUC_lexicon_tags_17 },
    { 0x3113fd02, SUC_lexicon_tags_31 },
    { 0x8753fd04, SUC_lexicon_tags_6 },
    { 0xdd33fd05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9473fd07, SUC_lexicon_tags_1 },
    { 0x14d3fd07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4ab3fd0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9693fd0e, SUC_lexicon_tags_7 },
    { 0x2a93fd0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb3fd18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b3fd1e, SUC_lexicon_tags_7 },
    { 0x9133fd1f, SUC_lexicon_tags_14 },
    { 0x8293fd1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8633fd23, SUC_lexicon_tags_11 },
    { 0x4673fd24, SUC_lexicon_tags_68 },
    { 0x2a33fd25, SUC_lexicon_tags_37 },
    { 0x1e13fd24, SUC_lexicon_tags_34 },
    { 0x6f3fd27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb13fd2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5733fd2f, SUC_lexicon_tags_2 },
    { 0xc7d3fd30, SUC_lexicon_tags_47 },
    { 0xf753fd31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3853fd33, SUC_lexicon_tags_12 },
    { 0x2b93fd34, SUC_lexicon_tags_102 },
    { 0x4953fd33, SUC_lexicon_tags_19 },
    { 0xf5f3fd36, SUC_lexicon_tags_25 },
    { 0xa2d3fd36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30b3fd39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf873fd3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc793fd40, SUC_lexicon_tags_23 },
    { 0x473fd40, SUC_lexicon_tags_7 },
    { 0xadf3fd42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac13fd46, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f33fd49, SUC_lexicon_tags_9 },
    { 0x96d3fd4a, SUC_lexicon_tags_21 },
    { 0x7c73fd4b, SUC_lexicon_tags_14 },
    { 0xb273fd4a, SUC_lexicon_tags_2 },
    { 0xf133fd4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7533fd50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c93fd52, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd133fd58, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5953fd5c, SUC_lexicon_tags_28 },
    { 0x8213fd5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb653fd62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd913fd66, SUC_lexicon_tags_15 },
    { 0x9d73fd67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3233fd69, SUC_lexicon_tags_17 },
    { 0xef33fd6a, SUC_lexicon_tags_27 },
    { 0x52d3fd6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c53fd70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8533fd73, SUC_lexicon_tags_36 },
    { 0x50f3fd74, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd493fd77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6133fd79, SUC_lexicon_tags_14 },
    { 0xd713fd7a, SUC_lexicon_tags_68 },
    { 0x7613fd7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd53fd80, SUC_lexicon_tags_25 },
    { 0xf5d3fd80, SUC_lexicon_tags_37 },
    { 0xcbd3fd82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd73fd84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2d3fd86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb633fd8a, SUC_lexicon_tags_7 },
    { 0x6bd3fd8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f93fd8d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6033fd93, SUC_lexicon_tags_9 },
    { 0x5cd3fd94, SUC_lexicon_tags_29 },
    { 0x9833fd94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd3fd98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d3fd9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1db3fd9f, SUC_lexicon_tags_4 },
    { 0xf5d3fd9f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9333fda7, SUC_lexicon_tags_152 },
    { 0x2e53fda8, SUC_lexicon_tags_25 },
    { 0xee73fda9, SUC_lexicon_tags_11 },
    { 0x9213fdaa, SUC_lexicon_tags_24 },
    { 0x933fdab, SUC_lexicon_tags_30 },
    { 0x30b3fdab, SUC_lexicon_tags_12 },
    { 0xd473fdac, SUC_lexicon_tags_32 },
    { 0x90b3fdaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc93fdb7, SUC_lexicon_tags_15 },
    { 0xff3fdb7, SUC_lexicon_tags_45 },
    { 0x7b53fdb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d53fdbb, SUC_lexicon_tags_38 },
    { 0xe6f3fdbc, SUC_lexicon_tags_3 },
    { 0x8373fdbd, SUC_lexicon_tags_17 },
    { 0xf713fdbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb3fdc1, SUC_lexicon_tags_37 },
    { 0x30d3fdc2, SUC_lexicon_tags_17 },
    { 0x2f73fdc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b3fdc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf313fdca, SUC_lexicon_tags_74 },
    { 0x473fdca, SUC_lexicon_tags_45 },
    { 0x6a13fdcc, SUC_lexicon_tags_17 },
    { 0xca53fdca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a13fdd3, SUC_lexicon_tags_7 },
    { 0x72d3fdd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b3fddc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d3fde0, SUC_lexicon_tags_26 },
    { 0xfe53fde0, SUC_lexicon_tags_31 },
    { 0x3453fde2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb3fde6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f3fdea, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc113fded, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecf3fdef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x713fdf1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xafb3fdf3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a33fdf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4553fdf9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9613fdfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f3fe00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb3fe05, SUC_lexicon_tags_14 },
    { 0x6793fe06, SUC_lexicon_tags_11 },
    { 0x9313fe07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b3fe0a, SUC_lexicon_tags_65 },
    { 0x84b3fe0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9453fe0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa13fe12, SUC_lexicon_tags_2 },
    { 0x2fb3fe12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b3fe17, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe3d3fe19, SUC_lexicon_tags_17 },
    { 0x5ed3fe19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b33fe24, SUC_lexicon_tags_17 },
    { 0xc0b3fe25, SUC_lexicon_tags_1 },
    { 0xa13fe26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf73fe28, SUC_lexicon_tags_28 },
    { 0x88f3fe29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b3fe2e, SUC_lexicon_tags_17 },
    { 0x3a13fe2e, SUC_lexicon_tags_15 },
    { 0xd573fe2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb473fe37, SUC_lexicon_tags_19 },
    { 0x8553fe38, SUC_lexicon_tags_7 },
    { 0xdad3fe37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2a33fe3b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fe40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7093fe43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd93fe49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf3fe4f, SUC_lexicon_tags_12 },
    { 0xa233fe50, SUC_lexicon_tags_43 },
    { 0x2253fe51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb333fe57, SUC_lexicon_tags_34 },
    { 0x5fb3fe58, SUC_lexicon_tags_25 },
    { 0xfed3fe59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5d3fe5b, SUC_lexicon_tags_10 },
    { 0xbd13fe5b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee33fe5e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed3fe65, SUC_lexicon_tags_60 },
    { 0x9c13fe66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6133fe69, SUC_lexicon_tags_10 },
    { 0x78b3fe6a, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xcd53fe6c, SUC_lexicon_tags_66 },
    { 0x5533fe6c, SUC_lexicon_tags_25 },
    { 0x3173fe6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7313fe71, SUC_lexicon_tags_1 },
    { 0x58b3fe71, SUC_lexicon_tags_2 },
    { 0x89b3fe73, SUC_lexicon_tags_33 },
    { 0x1a73fe72, SUC_lexicon_tags_17 },
    { 0xcff3fe75, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53fe7a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x38b3fe7c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe533fe88, SUC_lexicon_tags_27 },
    { 0xd773fe89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f3fe8c, SUC_lexicon_tags_14 },
    { 0x4c73fe8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6513fe8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff3fe92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d3fe98, SUC_lexicon_tags_2 },
    { 0xc0f3fe99, SUC_lexicon_tags_37 },
    { 0x7453fe98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73fea9, SUC_lexicon_tags_127 },
    { 0xdb73feaa, SUC_lexicon_tags_32 },
    { 0x7b3feaa, SUC_lexicon_tags_10 },
    { 0xdef3feac, SUC_lexicon_tags_27 },
    { 0xb7b3fead, SUC_lexicon_tags_37 },
    { 0x9773feac, SUC_lexicon_tags_37 },
    { 0xc2b3fead, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe93feb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8433feb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f33feb8, SUC_lexicon_tags_11 },
    { 0x1dd3feb9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf73febe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513fec2, SUC_lexicon_tags_25 },
    { 0x3e33fec2, SUC_lexicon_tags_17 },
    { 0xf053fec4, SUC_lexicon_tags_25 },
    { 0x93f3fec5, SUC_lexicon_tags_63 },
    { 0xb073fec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb53fec9, SUC_lexicon_tags_1 },
    { 0x7033feca, SUC_lexicon_tags_43 },
    { 0x6fb3fec9, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab3fed3, SUC_lexicon_tags_238 },
    { 0x8953fed4, SUC_lexicon_tags_68 },
    { 0x1df3fed4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xec93fed7, SUC_lexicon_tags_15 },
    { 0xb113fed8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf593feda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed3fede, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x40f3fee0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4e33fee2, SUC_lexicon_tags_12 },
    { 0xc2d3fee3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xed3fee5, SUC_lexicon_tags_47 },
    { 0xc813fee6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb3feea, SUC_lexicon_tags_33 },
    { 0x83b3feea, SUC_lexicon_tags_25 },
    { 0xc8b3feec, SUC_lexicon_tags_63 },
    { 0x6713feed, SUC_lexicon_tags_15 },
    { 0xc2b3feeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f3fef1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacf3fef3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f3fef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedf3fef8, SUC_lexicon_tags_26 },
    { 0x5053fef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8253fefd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e33ff01, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f3ff05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d53ff0c, SUC_lexicon_tags_45 },
    { 0x1733ff0c, SUC_lexicon_tags_47 },
    { 0xc8b3ff0e, SUC_lexicon_tags_27 },
    { 0xd593ff0f, SUC_lexicon_tags_13 },
    { 0xcbb3ff10, SUC_lexicon_tags_1 },
    { 0x93d3ff0f, SUC_lexicon_tags_33 },
    { 0x9f53ff12, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4d13ff14, SUC_lexicon_tags_37 },
    { 0x6b13ff15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe393ff19, SUC_lexicon_tags_11 },
    { 0xff53ff1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd3ff1d, SUC_lexicon_tags_55 },
    { 0xa453ff1e, SUC_lexicon_tags_20 },
    { 0xfd13ff1f, SUC_lexicon_tags_8 },
    { 0x8a53ff1e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5df3ff22, SUC_lexicon_tags_15 },
    { 0xe653ff22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce93ff27, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f3ff2a, SUC_lexicon_tags_33 },
    { 0x94b3ff2b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf73ff2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1d53ff31, SUC_lexicon_tags_11 },
    { 0xded3ff32, SUC_lexicon_tags_282 },
    { 0x51d3ff33, SUC_lexicon_tags_12 },
    { 0x58f3ff34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b3ff36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb3ff3a, SUC_lexicon_tags_7 },
    { 0x4693ff3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f53ff3d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb533ff44, SUC_lexicon_tags_621 },
    { 0x8d3ff45, SUC_lexicon_tags_0 },
    { 0xdad3ff46, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4d53ff48, SUC_lexicon_tags_32 },
    { 0x1153ff49, SUC_lexicon_tags_1 },
    { 0x1273ff4a, SUC_lexicon_tags_0 },
    { 0xe33ff49, SUC_lexicon_tags_2 },
    { 0x6c33ff4c, SUC_lexicon_tags_63 },
    { 0x35b3ff49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f53ff4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde13ff51, SUC_lexicon_tags_12 },
    { 0xd2b3ff52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x96b3ff54, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe273ff5b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5533ff5d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd3ff60, SUC_lexicon_tags_25 },
    { 0xb293ff61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac33ff65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4133ff72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7293ff75, SUC_lexicon_tags_15 },
    { 0x14d3ff75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc473ff7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713ff7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8433ff81, SUC_lexicon_tags_25 },
    { 0x2753ff82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe853ff85, SUC_lexicon_tags_17 },
    { 0xb233ff86, SUC_lexicon_tags_11 },
    { 0x89f3ff87, SUC_lexicon_tags_25 },
    { 0xa73ff88, SUC_lexicon_tags_30 },
    { 0x4973ff88, SUC_lexicon_tags_11 },
    { 0xcf13ff87, SUC_lexicon_tags_14 },
    { 0xfe53ff89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b3ff8e, SUC_lexicon_tags_37 },
    { 0xb133ff8e, SUC_lexicon_tags_7 },
    { 0x6593ff90, SUC_lexicon_tags_32 },
    { 0x4253ff91, SUC_lexicon_tags_12 },
    { 0x86d3ff91, SUC_lexicon_tags_16 },
    { 0x2f33ff92, SUC_lexicon_tags_2 },
    { 0x6953ff94, SUC_lexicon_tags_87 },
    { 0xeb53ff94, SUC_lexicon_tags_28 },
    { 0xf933ff95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a13ff98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb693ff9c, SUC_lexicon_tags_11 },
    { 0x9e73ff9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f3ffa2, SUC_lexicon_tags_17 },
    { 0x9b13ffa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b3ffa8, SUC_lexicon_tags_49 },
    { 0x7b13ffa9, SUC_lexicon_tags_17 },
    { 0xfb13ffa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6453ffac, SUC_lexicon_tags_37 },
    { 0xa833ffad, SUC_lexicon_tags_37 },
    { 0x6a53ffac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b3ffb7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b3ffba, SUC_lexicon_tags_18 },
    { 0xa4f3ffbb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3ffc3, SUC_lexicon_tags_2 },
    { 0xd4b3ffc4, SUC_lexicon_tags_9 },
    { 0xfb53ffc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x133ffc7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc4d3ffc9, SUC_lexicon_tags_17 },
    { 0xc8d3ffca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2213ffce, SUC_lexicon_tags_15 },
    { 0x7713ffce, SUC_lexicon_tags_37 },
    { 0x4a13ffd0, SUC_lexicon_tags_25 },
    { 0xf4f3ffd1, SUC_lexicon_tags_1 },
    { 0x7e13ffd1, SUC_lexicon_tags_7 },
    { 0xdf93ffce, SUC_lexicon_tags_17 },
    { 0x2ff3ffd3, SUC_lexicon_tags_16 },
    { 0x2b13ffcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f3ffd8, SUC_lexicon_tags_63 },
    { 0xb933ffd9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xce93ffdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xedb3ffde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713ffe3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db3ffe7, SUC_lexicon_tags_6 },
    { 0xfeb3ffe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4533ffea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b13fff5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55f3fff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b3fffb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1140002, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x65b40004, SUC_lexicon_tags_45 },
    { 0x5bf40005, SUC_lexicon_tags_21 },
    { 0xad40006, SUC_lexicon_tags_25 },
    { 0xd5d40005, SUC_lexicon_tags_11 },
    { 0xf940008, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd4000d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6540014, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab40017, SUC_lexicon_tags_15 },
    { 0x4d40017, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b4001b, SUC_lexicon_tags_44 },
    { 0x7d54001c, SUC_lexicon_tags_37 },
    { 0x2334001d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68b4001f, SUC_lexicon_tags_37 },
    { 0x8a74001f, SUC_lexicon_tags_23 },
    { 0x36f40021, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x80140023, SUC_lexicon_tags_37 },
    { 0xb2540024, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d40029, SUC_lexicon_tags_8 },
    { 0xd7f4002a, SUC_lexicon_tags_81 },
    { 0x9b4002b, SUC_lexicon_tags_757 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f94002e, SUC_lexicon_tags_148 },
    { 0x8174002e, SUC_lexicon_tags_39 },
    { 0x92d4002f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb40034, SUC_lexicon_tags_37 },
    { 0x92540035, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f4003c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1794003f, SUC_lexicon_tags_28 },
    { 0x5d40040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2740042, SUC_lexicon_tags_40 },
    { 0x1c540043, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b54004a, SUC_lexicon_tags_25 },
    { 0x9674004b, SUC_lexicon_tags_33 },
    { 0x3bd4004c, SUC_lexicon_tags_37 },
    { 0x5114004d, SUC_lexicon_tags_214 },
    { 0x19d4004a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf540054, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f34005c, SUC_lexicon_tags_11 },
    { 0x6514005d, SUC_lexicon_tags_2 },
    { 0x8294005d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61940062, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba940064, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d40072, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7d540074, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35340076, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b54007d, SUC_lexicon_tags_7 },
    { 0xdb54007e, SUC_lexicon_tags_15 },
    { 0xcd54007f, SUC_lexicon_tags_0 },
    { 0xc8540080, SUC_lexicon_tags_16 },
    { 0x70f40081, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28940085, SUC_lexicon_tags_2 },
    { 0x9e140086, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b4008d, SUC_lexicon_tags_11 },
    { 0xd9d4008e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5340093, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b40097, SUC_lexicon_tags_11 },
    { 0x4ed40097, SUC_lexicon_tags_29 },
    { 0x8df40099, SUC_lexicon_tags_1 },
    { 0xf7b4009a, SUC_lexicon_tags_37 },
    { 0x99340097, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x994009d, SUC_lexicon_tags_12 },
    { 0x88f4009e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f400a4, SUC_lexicon_tags_19 },
    { 0x42f400a5, SUC_lexicon_tags_7 },
    { 0x15d400a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcbf400a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd400ac, SUC_lexicon_tags_11 },
    { 0xeb1400ad, SUC_lexicon_tags_38 },
    { 0xf01400ae, SUC_lexicon_tags_11 },
    { 0xae3400ac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b400b4, SUC_lexicon_tags_12 },
    { 0x74f400b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea9400b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73400c0, SUC_lexicon_tags_9 },
    { 0x2e7400c1, SUC_lexicon_tags_23 },
    { 0x8d400c1, SUC_lexicon_tags_14 },
    { 0xd1b400c3, SUC_lexicon_tags_11 },
    { 0x495400c4, SUC_lexicon_tags_16 },
    { 0x505400c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x591400c7, SUC_lexicon_tags_1 },
    { 0x389400c8, SUC_lexicon_tags_17 },
    { 0x29400c7, SUC_lexicon_tags_3 },
    { 0x3c9400ca, SUC_lexicon_tags_30 },
    { 0x4fb400cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9eb400cd, SUC_lexicon_tags_10 },
    { 0x753400ce, SUC_lexicon_tags_29 },
    { 0x3eb400cf, SUC_lexicon_tags_1 },
    { 0x635400d0, SUC_lexicon_tags_1 },
    { 0x3b400d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9400d5, SUC_lexicon_tags_17 },
    { 0x3cb400d6, SUC_lexicon_tags_37 },
    { 0x1c3400d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d400de, SUC_lexicon_tags_37 },
    { 0x8ad400df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcb3400e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19400e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd400ed, SUC_lexicon_tags_7 },
    { 0xcb5400ee, SUC_lexicon_tags_14 },
    { 0xb25400ef, SUC_lexicon_tags_2 },
    { 0x287400ed, SUC_lexicon_tags_33 },
    { 0x70d400f1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75400fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5340101, SUC_lexicon_tags_82 },
    { 0xe7940101, SUC_lexicon_tags_25 },
    { 0x2b340103, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e340106, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46340109, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6894010d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f4010f, SUC_lexicon_tags_28 },
    { 0x7214010f, SUC_lexicon_tags_18 },
    { 0xefb40110, SUC_lexicon_tags_37 },
    { 0xfeb4010f, SUC_lexicon_tags_2 },
    { 0xef740113, SUC_lexicon_tags_17 },
    { 0x10b40113, SUC_lexicon_tags_2 },
    { 0xe6140111, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d4011a, SUC_lexicon_tags_19 },
    { 0xe534011a, SUC_lexicon_tags_11 },
    { 0xac54011a, SUC_lexicon_tags_63 },
    { 0x22f4011d, SUC_lexicon_tags_68 },
    { 0x27b4011a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1940120, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8940123, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab140128, SUC_lexicon_tags_32 },
    { 0x33540129, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8bd4012b, SUC_lexicon_tags_60 },
    { 0x2f94012b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4974012e, SUC_lexicon_tags_12 },
    { 0xca74012f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce140131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cd40133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86340137, SUC_lexicon_tags_37 },
    { 0x48d40138, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a14013f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3140143, SUC_lexicon_tags_23 },
    { 0x87d40144, SUC_lexicon_tags_366 },
    { 0xac740145, SUC_lexicon_tags_26 },
    { 0xa0f40143, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5740149, SUC_lexicon_tags_6 },
    { 0x3554014a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad14014c, SUC_lexicon_tags_82 },
    { 0xd254014d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b40150, SUC_lexicon_tags_45 },
    { 0xf4540151, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b540157, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f54015e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2740164, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d340169, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xab54016c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a34016e, SUC_lexicon_tags_37 },
    { 0x1c14016f, SUC_lexicon_tags_29 },
    { 0x6140170, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b40174, SUC_lexicon_tags_31 },
    { 0xd1f40175, SUC_lexicon_tags_15 },
    { 0x32940176, SUC_lexicon_tags_60 },
    { 0xaeb40177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed4017a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27f4017c, SUC_lexicon_tags_71 },
    { 0xa9d4017c, SUC_lexicon_tags_8 },
    { 0x2674017e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92940188, SUC_lexicon_tags_12 },
    { 0xc1d40188, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5294018e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x74140191, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b540193, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfb40195, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d4019a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd401a2, SUC_lexicon_tags_14 },
    { 0xd5b401a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c3401a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x829401a7, SUC_lexicon_tags_17 },
    { 0xddd401a8, SUC_lexicon_tags_43 },
    { 0x10b401a7, SUC_lexicon_tags_38 },
    { 0x135401aa, SUC_lexicon_tags_17 },
    { 0xb5d401a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9401ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f401b2, SUC_lexicon_tags_1 },
    { 0x63b401b3, SUC_lexicon_tags_6 },
    { 0x725401b4, SUC_lexicon_tags_123 },
    { 0xe15401b5, SUC_lexicon_tags_3 },
    { 0x52b401b5, SUC_lexicon_tags_70 },
    { 0x361401b7, SUC_lexicon_tags_28 },
    { 0x241401b8, SUC_lexicon_tags_15 },
    { 0xcb9401b9, SUC_lexicon_tags_29 },
    { 0x739401b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5b401bc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147401c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1cf401c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d401c5, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xe0d401c7, SUC_lexicon_tags_1 },
    { 0x885401c7, SUC_lexicon_tags_5 },
    { 0xfdf401c9, SUC_lexicon_tags_2 },
    { 0x2d5401ca, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b401cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a9401cf, SUC_lexicon_tags_17 },
    { 0xbbf401d0, SUC_lexicon_tags_17 },
    { 0x785401d0, SUC_lexicon_tags_46 },
    { 0x7ed401d1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbd7401d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b401d8, SUC_lexicon_tags_63 },
    { 0x9d1401d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9401de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959401e9, SUC_lexicon_tags_37 },
    { 0xb65401ea, SUC_lexicon_tags_29 },
    { 0x90d401eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5d1401ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b401f0, SUC_lexicon_tags_25 },
    { 0x965401f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87401fd, SUC_lexicon_tags_15 },
    { 0xb2b401fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e540202, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f40205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b40208, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b4020b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c74020d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed40210, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53340214, SUC_lexicon_tags_33 },
    { 0x92340214, SUC_lexicon_tags_60 },
    { 0x22140216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x414021a, SUC_lexicon_tags_15 },
    { 0x93b4021b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ff4021d, SUC_lexicon_tags_43 },
    { 0x3174021e, SUC_lexicon_tags_42 },
    { 0x8594021f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef340221, SUC_lexicon_tags_16 },
    { 0x82340222, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f40229, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1340232, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe740235, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a340238, SUC_lexicon_tags_57 },
    { 0x89340238, SUC_lexicon_tags_7 },
    { 0x96d40239, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc594023c, SUC_lexicon_tags_11 },
    { 0xe314023d, SUC_lexicon_tags_47 },
    { 0x1df4023e, SUC_lexicon_tags_92 },
    { 0x9eb4023f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3340241, SUC_lexicon_tags_45 },
    { 0x6db40242, SUC_lexicon_tags_1 },
    { 0x4e940241, SUC_lexicon_tags_46 },
    { 0xea340244, SUC_lexicon_tags_11 },
    { 0xd4140243, SUC_lexicon_tags_17 },
    { 0x78f40242, SUC_lexicon_tags_12 },
    { 0xb6b40247, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc574024b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4d14024d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef40250, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea340258, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb774025f, SUC_lexicon_tags_43 },
    { 0xf774025f, SUC_lexicon_tags_37 },
    { 0xfe540261, SUC_lexicon_tags_15 },
    { 0xe2140260, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f740265, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf4f40267, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x95b40269, SUC_lexicon_tags_17 },
    { 0x5574026a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b14026d, SUC_lexicon_tags_63 },
    { 0x12d4026e, SUC_lexicon_tags_12 },
    { 0x174026e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3b40271, SUC_lexicon_tags_60 },
    { 0xb3d40271, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f4027a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f4027d, SUC_lexicon_tags_15 },
    { 0xfcf4027e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39540284, SUC_lexicon_tags_2 },
    { 0x1e540285, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d940288, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54b4028a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2434028c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d4028f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54f40291, SUC_lexicon_tags_11 },
    { 0x7cf40291, SUC_lexicon_tags_108 },
    { 0x8cd40293, SUC_lexicon_tags_12 },
    { 0x22b40292, SUC_lexicon_tags_45 },
    { 0x30f40291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86d40297, SUC_lexicon_tags_15 },
    { 0x75940298, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857402a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687402a4, SUC_lexicon_tags_2 },
    { 0x3db402a5, SUC_lexicon_tags_55 },
    { 0x3d9402a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b402aa, SUC_lexicon_tags_11 },
    { 0x78d402ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3402ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579402b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47402b9, SUC_lexicon_tags_37 },
    { 0x51d402ba, SUC_lexicon_tags_15 },
    { 0xaa5402bb, SUC_lexicon_tags_9 },
    { 0xc51402b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39402c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf402c7, SUC_lexicon_tags_37 },
    { 0xf3b402c7, SUC_lexicon_tags_25 },
    { 0xa4d402c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db402cf, SUC_lexicon_tags_25 },
    { 0x131402d0, SUC_lexicon_tags_27 },
    { 0x1fd402d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d402d5, SUC_lexicon_tags_46 },
    { 0x639402d6, SUC_lexicon_tags_0 },
    { 0xe35402d6, SUC_lexicon_tags_17 },
    { 0x187402d8, SUC_lexicon_tags_60 },
    { 0x4f9402d9, SUC_lexicon_tags_2 },
    { 0x661402d8, SUC_lexicon_tags_17 },
    { 0x6bf402db, SUC_lexicon_tags_14 },
    { 0x357402da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x80d402de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x317402e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0b402e2, SUC_lexicon_tags_10 },
    { 0x20f402e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfab402e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x179402e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1402ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0f402ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb402f5, SUC_lexicon_tags_41 },
    { 0xa5f402f6, SUC_lexicon_tags_0 },
    { 0x2bf402f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x759402fa, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f402fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x98340300, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b40302, SUC_lexicon_tags_15 },
    { 0xe2540303, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ed40305, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf40309, SUC_lexicon_tags_6 },
    { 0x5ab4030a, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad94030e, SUC_lexicon_tags_731 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3940312, SUC_lexicon_tags_37 },
    { 0x43540313, SUC_lexicon_tags_17 },
    { 0xeab40314, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd8140316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7474031b, SUC_lexicon_tags_11 },
    { 0xacb4031c, SUC_lexicon_tags_37 },
    { 0x70d4031c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9140320, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31940322, SUC_lexicon_tags_33 },
    { 0x53540323, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26340328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214032c, SUC_lexicon_tags_38 },
    { 0x46f4032d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd674032f, SUC_lexicon_tags_0 },
    { 0x4a740330, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97740335, SUC_lexicon_tags_43 },
    { 0x3c540335, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f40339, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6f4033b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4474033f, SUC_lexicon_tags_19 },
    { 0x63b40340, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff40344, SUC_lexicon_tags_23 },
    { 0x20140344, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xad540348, SUC_lexicon_tags_37 },
    { 0x6bb40349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b4034d, SUC_lexicon_tags_19 },
    { 0xdbb4034e, SUC_lexicon_tags_39 },
    { 0x9954034d, SUC_lexicon_tags_33 },
    { 0x3514034f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b40353, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4254035a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb834035c, SUC_lexicon_tags_11 },
    { 0xaa54035c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32540360, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3740363, SUC_lexicon_tags_6 },
    { 0xf9140364, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9cf40366, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf254036c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2d54036e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f40371, SUC_lexicon_tags_28 },
    { 0x57f40372, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53940377, SUC_lexicon_tags_14 },
    { 0xfa740378, SUC_lexicon_tags_15 },
    { 0xbe140378, SUC_lexicon_tags_25 },
    { 0x8a740379, SUC_lexicon_tags_17 },
    { 0x76740379, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1dd4037d, SUC_lexicon_tags_15 },
    { 0xd594037d, SUC_lexicon_tags_45 },
    { 0xe5d4037d, SUC_lexicon_tags_46 },
    { 0xb2740380, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x77f40382, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48540384, SUC_lexicon_tags_15 },
    { 0xfb140385, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x32540388, SUC_lexicon_tags_66 },
    { 0x78b40389, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b4038c, SUC_lexicon_tags_57 },
    { 0x6ed4038d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x434038f, SUC_lexicon_tags_1 },
    { 0x3d34038f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x18740393, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f40397, SUC_lexicon_tags_12 },
    { 0x28140398, SUC_lexicon_tags_1 },
    { 0x1eb40398, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d4039c, SUC_lexicon_tags_37 },
    { 0x7034039d, SUC_lexicon_tags_37 },
    { 0xd154039e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1b403a0, SUC_lexicon_tags_47 },
    { 0x819403a1, SUC_lexicon_tags_158 },
    { 0xba7403a2, SUC_lexicon_tags_17 },
    { 0x1403a2, SUC_lexicon_tags_25 },
    { 0x8eb403a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48d403a6, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9403ad, SUC_lexicon_tags_1 },
    { 0xa67403ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb61403b0, SUC_lexicon_tags_27 },
    { 0xd49403b0, SUC_lexicon_tags_55 },
    { 0x1eb403b1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x83b403b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x147403b6, SUC_lexicon_tags_16 },
    { 0x439403b7, SUC_lexicon_tags_15 },
    { 0x71f403b6, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f403be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb21403c0, SUC_lexicon_tags_55 },
    { 0xa7f403c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f403c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe43403c6, SUC_lexicon_tags_40 },
    { 0xab9403c7, SUC_lexicon_tags_37 },
    { 0xceb403c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed403cc, SUC_lexicon_tags_15 },
    { 0x8c3403cd, SUC_lexicon_tags_8 },
    { 0xb1f403cc, SUC_lexicon_tags_46 },
    { 0xef3403ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x84f403d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb403d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5af403d9, SUC_lexicon_tags_15 },
    { 0x22f403da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b403de, SUC_lexicon_tags_1 },
    { 0x86d403df, SUC_lexicon_tags_37 },
    { 0xab403df, SUC_lexicon_tags_25 },
    { 0xb6f403e1, SUC_lexicon_tags_11 },
    { 0xf49403df, SUC_lexicon_tags_23 },
    { 0xe0d403e2, SUC_lexicon_tags_15 },
    { 0xb07403e4, SUC_lexicon_tags_5 },
    { 0xd3403e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf4f403e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d403ea, SUC_lexicon_tags_45 },
    { 0xfaf403ea, SUC_lexicon_tags_114 },
    { 0x8b1403ec, SUC_lexicon_tags_12 },
    { 0x3b9403ec, SUC_lexicon_tags_37 },
    { 0x4c3403ed, SUC_lexicon_tags_15 },
    { 0x3ed403ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad403f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b403f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21403fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf40401, SUC_lexicon_tags_16 },
    { 0xab940402, SUC_lexicon_tags_45 },
    { 0xd0d40403, SUC_lexicon_tags_9 },
    { 0x11b40403, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd4040a, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76940410, SUC_lexicon_tags_47 },
    { 0xa4940410, SUC_lexicon_tags_17 },
    { 0x25340412, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8cb40414, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd494041a, SUC_lexicon_tags_3 },
    { 0x2814041b, SUC_lexicon_tags_4 },
    { 0x2754041c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6854041f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd40429, SUC_lexicon_tags_25 },
    { 0x2fd4042a, SUC_lexicon_tags_25 },
    { 0x44b4042b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1434042e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d40432, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23540438, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f40440, SUC_lexicon_tags_71 },
    { 0xbdb40441, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b940443, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f40449, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd874044c, SUC_lexicon_tags_25 },
    { 0x2034044d, SUC_lexicon_tags_25 },
    { 0xed14044c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f40453, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b540456, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc174045f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47740461, SUC_lexicon_tags_60 },
    { 0xbe540462, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d340464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c34046c, SUC_lexicon_tags_3 },
    { 0x4cd4046c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d34046f, SUC_lexicon_tags_8 },
    { 0xc5940470, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42940472, SUC_lexicon_tags_1 },
    { 0xbcb40473, SUC_lexicon_tags_23 },
    { 0xb3940474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35b40476, SUC_lexicon_tags_92 },
    { 0x83140477, SUC_lexicon_tags_23 },
    { 0x9a140478, SUC_lexicon_tags_25 },
    { 0x61740478, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7014047f, SUC_lexicon_tags_11 },
    { 0xa8940480, SUC_lexicon_tags_45 },
    { 0x8bb40480, SUC_lexicon_tags_29 },
    { 0x19140482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb540487, SUC_lexicon_tags_15 },
    { 0xb0f40487, SUC_lexicon_tags_1 },
    { 0x21b40487, SUC_lexicon_tags_47 },
    { 0xdbd40487, SUC_lexicon_tags_1 },
    { 0x3f94048b, SUC_lexicon_tags_71 },
    { 0xf8b40487, SUC_lexicon_tags_26 },
    { 0x8f54048d, SUC_lexicon_tags_2 },
    { 0x8b540489, SUC_lexicon_tags_27 },
    { 0xe514048e, SUC_lexicon_tags_45 },
    { 0x9914048c, SUC_lexicon_tags_2 },
    { 0xebd40491, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d540497, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c140499, SUC_lexicon_tags_62 },
    { 0x30d4049a, SUC_lexicon_tags_1 },
    { 0x1194049a, SUC_lexicon_tags_43 },
    { 0xf174049a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725404a3, SUC_lexicon_tags_45 },
    { 0x6a1404a4, SUC_lexicon_tags_68 },
    { 0xfb7404a3, SUC_lexicon_tags_17 },
    { 0x6cf404a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b404ab, SUC_lexicon_tags_12 },
    { 0xe03404ac, SUC_lexicon_tags_25 },
    { 0xe1d404ad, SUC_lexicon_tags_10 },
    { 0x389404ac, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad404b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9404be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93404c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x785404c4, SUC_lexicon_tags_25 },
    { 0x83f404c5, SUC_lexicon_tags_57 },
    { 0xbf5404c6, SUC_lexicon_tags_37 },
    { 0x4e1404c7, SUC_lexicon_tags_2 },
    { 0xaa7404c8, SUC_lexicon_tags_37 },
    { 0xbe5404c6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81404ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b404d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdd404d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b404d6, SUC_lexicon_tags_37 },
    { 0xe9d404d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d404e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f404eb, SUC_lexicon_tags_17 },
    { 0x201404ec, SUC_lexicon_tags_864 },
    { 0, NULL },
    { 0x71b404ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f404f2, SUC_lexicon_tags_7 },
    { 0x95f404f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e340500, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb7540502, SUC_lexicon_tags_31 },
    { 0x43d40502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd40507, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3f40509, SUC_lexicon_tags_37 },
    { 0x3a54050a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc6f4050c, SUC_lexicon_tags_29 },
    { 0xdaf4050c, SUC_lexicon_tags_14 },
    { 0x5974050e, SUC_lexicon_tags_27 },
    { 0xd034050d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d40515, SUC_lexicon_tags_31 },
    { 0xb5340516, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd2140518, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d4051a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44940526, SUC_lexicon_tags_37 },
    { 0xcd40527, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd340529, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbdd4052b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24140533, SUC_lexicon_tags_29 },
    { 0xdc940534, SUC_lexicon_tags_1 },
    { 0x6af40533, SUC_lexicon_tags_33 },
    { 0xeb940534, SUC_lexicon_tags_30 },
    { 0x31f40533, SUC_lexicon_tags_15 },
    { 0x21740538, SUC_lexicon_tags_282 },
    { 0xbc740539, SUC_lexicon_tags_27 },
    { 0xf994053a, SUC_lexicon_tags_2 },
    { 0xe4740539, SUC_lexicon_tags_27 },
    { 0xe754053c, SUC_lexicon_tags_6 },
    { 0xac940538, SUC_lexicon_tags_17 },
    { 0x1114053e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x63540540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb40542, SUC_lexicon_tags_49 },
    { 0x8f40542, SUC_lexicon_tags_14 },
    { 0xec940544, SUC_lexicon_tags_7 },
    { 0xec540542, SUC_lexicon_tags_34 },
    { 0x64f40544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5740548, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c54054d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd40558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d4055c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68340562, SUC_lexicon_tags_37 },
    { 0xbcd40563, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x28540566, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9974056c, SUC_lexicon_tags_23 },
    { 0xb1f4056c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xad94056f, SUC_lexicon_tags_4 },
    { 0x22d4056f, SUC_lexicon_tags_33 },
    { 0xc2d4056f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd40574, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f40579, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x714057b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8474057d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b40581, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69b40583, SUC_lexicon_tags_55 },
    { 0x81540583, SUC_lexicon_tags_38 },
    { 0xce140585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc140589, SUC_lexicon_tags_19 },
    { 0x8634058a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7094058e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4cb40590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef540593, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25540598, SUC_lexicon_tags_28 },
    { 0x61740598, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x334059b, SUC_lexicon_tags_3 },
    { 0xcc14059b, SUC_lexicon_tags_30 },
    { 0x7454059d, SUC_lexicon_tags_96 },
    { 0x9e74059e, SUC_lexicon_tags_84 },
    { 0x2894059b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f7405a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x617405a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3b405a5, SUC_lexicon_tags_6 },
    { 0x677405a6, SUC_lexicon_tags_47 },
    { 0x6a1405a7, SUC_lexicon_tags_25 },
    { 0xcd3405a8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b405ad, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe2b405af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b405b2, SUC_lexicon_tags_2 },
    { 0xfcd405b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb405b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x833405ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb45405bc, SUC_lexicon_tags_27 },
    { 0x6c7405bc, SUC_lexicon_tags_3 },
    { 0x49f405bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef405c9, SUC_lexicon_tags_49 },
    { 0xab9405ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3405cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b405d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc1405d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9405db, SUC_lexicon_tags_138 },
    { 0xa4d405dc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xefd405de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53405e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b405e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7405ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87405f2, SUC_lexicon_tags_12 },
    { 0xf85405f3, SUC_lexicon_tags_15 },
    { 0x3c9405f4, SUC_lexicon_tags_15 },
    { 0xa09405f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x155405f7, SUC_lexicon_tags_63 },
    { 0xb41405f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c9405fa, SUC_lexicon_tags_47 },
    { 0x477405fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7405fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3540602, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f40606, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f4060b, SUC_lexicon_tags_8 },
    { 0x52d4060c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6254060e, SUC_lexicon_tags_2 },
    { 0x3574060f, SUC_lexicon_tags_68 },
    { 0xf7940610, SUC_lexicon_tags_37 },
    { 0x4f140611, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e540614, SUC_lexicon_tags_55 },
    { 0xa5740615, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59540619, SUC_lexicon_tags_43 },
    { 0xa114061a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8814061f, SUC_lexicon_tags_11 },
    { 0xa2b40620, SUC_lexicon_tags_9 },
    { 0x57540620, SUC_lexicon_tags_8 },
    { 0x64d40622, SUC_lexicon_tags_2 },
    { 0xb6340621, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83140627, SUC_lexicon_tags_2 },
    { 0xb340628, SUC_lexicon_tags_11 },
    { 0x76740629, SUC_lexicon_tags_14 },
    { 0xe8b40629, SUC_lexicon_tags_17 },
    { 0x62140629, SUC_lexicon_tags_19 },
    { 0x7c34062c, SUC_lexicon_tags_15 },
    { 0xc534062d, SUC_lexicon_tags_60 },
    { 0xc94062e, SUC_lexicon_tags_13 },
    { 0x6e14062d, SUC_lexicon_tags_522 },
    { 0x6794062f, SUC_lexicon_tags_65 },
    { 0xcb140630, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x42740634, SUC_lexicon_tags_60 },
    { 0xecf40634, SUC_lexicon_tags_30 },
    { 0xd1740636, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x71340639, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5154063d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c940640, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2540643, SUC_lexicon_tags_55 },
    { 0xa8940643, SUC_lexicon_tags_11 },
    { 0x51d40645, SUC_lexicon_tags_17 },
    { 0x7c140646, SUC_lexicon_tags_11 },
    { 0x83540647, SUC_lexicon_tags_17 },
    { 0x2140648, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6074064c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb54064e, SUC_lexicon_tags_1 },
    { 0x4234064f, SUC_lexicon_tags_37 },
    { 0x19b40650, SUC_lexicon_tags_11 },
    { 0x17740650, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x64340654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d40657, SUC_lexicon_tags_3 },
    { 0x9740658, SUC_lexicon_tags_6 },
    { 0x42d40659, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c94065e, SUC_lexicon_tags_66 },
    { 0xff94065f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0540661, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67140666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1f40668, SUC_lexicon_tags_17 },
    { 0x55f40669, SUC_lexicon_tags_15 },
    { 0x9e74066a, SUC_lexicon_tags_0 },
    { 0xbc34066b, SUC_lexicon_tags_60 },
    { 0xff94066c, SUC_lexicon_tags_0 },
    { 0x92b4066d, SUC_lexicon_tags_17 },
    { 0xa054066d, SUC_lexicon_tags_2 },
    { 0x6c14066f, SUC_lexicon_tags_12 },
    { 0xdcb40670, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b340673, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb940676, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7054067b, SUC_lexicon_tags_30 },
    { 0xdd34067c, SUC_lexicon_tags_29 },
    { 0x1534067d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9740681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4340684, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7740688, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xef14068a, SUC_lexicon_tags_25 },
    { 0x2274068a, SUC_lexicon_tags_25 },
    { 0x1c74068c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a940691, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3740694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d540696, SUC_lexicon_tags_29 },
    { 0xb1f40696, SUC_lexicon_tags_17 },
    { 0xab340698, SUC_lexicon_tags_10 },
    { 0x83340699, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1db4069b, SUC_lexicon_tags_7 },
    { 0xadd4069c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d406a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x529406a3, SUC_lexicon_tags_29 },
    { 0x831406a3, SUC_lexicon_tags_1 },
    { 0x9a3406a5, SUC_lexicon_tags_14 },
    { 0x6bd406a6, SUC_lexicon_tags_29 },
    { 0xe4d406a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf1406a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1406ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3bf406b0, SUC_lexicon_tags_14 },
    { 0x605406b1, SUC_lexicon_tags_16 },
    { 0x68d406b0, SUC_lexicon_tags_17 },
    { 0xfd7406b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x665406b6, SUC_lexicon_tags_49 },
    { 0xd5d406b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d406ba, SUC_lexicon_tags_1 },
    { 0x391406ba, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37406bf, SUC_lexicon_tags_23 },
    { 0xad406c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8b406c2, SUC_lexicon_tags_19 },
    { 0xf43406c3, SUC_lexicon_tags_32 },
    { 0xc75406c4, SUC_lexicon_tags_2 },
    { 0x4e5406c3, SUC_lexicon_tags_17 },
    { 0x151406c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9406ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b406cf, SUC_lexicon_tags_17 },
    { 0xedb406d0, SUC_lexicon_tags_12 },
    { 0x227406d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f406d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfa9406d7, SUC_lexicon_tags_37 },
    { 0xd3d406d8, SUC_lexicon_tags_47 },
    { 0x3a3406d8, SUC_lexicon_tags_37 },
    { 0xaaf406d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7406de, SUC_lexicon_tags_161 },
    { 0x6e5406df, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f406e2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x553406e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x863406e6, SUC_lexicon_tags_27 },
    { 0xba1406e7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x6db406e9, SUC_lexicon_tags_12 },
    { 0x1f3406ea, SUC_lexicon_tags_63 },
    { 0xf7f406eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7406ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x969406f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x723406f2, SUC_lexicon_tags_2 },
    { 0xb3f406f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb1406f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f406f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56140700, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b40706, SUC_lexicon_tags_68 },
    { 0x91340706, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa174070d, SUC_lexicon_tags_44 },
    { 0xa314070e, SUC_lexicon_tags_19 },
    { 0x80f4070f, SUC_lexicon_tags_1 },
    { 0xc094070f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe540715, SUC_lexicon_tags_26 },
    { 0xbc940716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b40719, SUC_lexicon_tags_43 },
    { 0x72740719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb74071d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea540721, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21140727, SUC_lexicon_tags_103 },
    { 0x2b540728, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad94072a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d14072c, SUC_lexicon_tags_61 },
    { 0x3b4072d, SUC_lexicon_tags_48 },
    { 0xf194072e, SUC_lexicon_tags_27 },
    { 0x9934072f, SUC_lexicon_tags_17 },
    { 0x29740730, SUC_lexicon_tags_72 },
    { 0x7f940731, SUC_lexicon_tags_11 },
    { 0x7dd40730, SUC_lexicon_tags_2 },
    { 0xa9140733, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f4073d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9834073f, SUC_lexicon_tags_191 },
    { 0xd8b40740, SUC_lexicon_tags_38 },
    { 0x54b40741, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90d40743, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb740747, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53740749, SUC_lexicon_tags_15 },
    { 0xc7d40749, SUC_lexicon_tags_28 },
    { 0x8e34074a, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xc834074e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92140751, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb940754, SUC_lexicon_tags_7 },
    { 0x98340754, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1f40757, SUC_lexicon_tags_17 },
    { 0x35140758, SUC_lexicon_tags_46 },
    { 0x5a340759, SUC_lexicon_tags_33 },
    { 0x7474075a, SUC_lexicon_tags_37 },
    { 0x8cb40759, SUC_lexicon_tags_33 },
    { 0xa374075c, SUC_lexicon_tags_37 },
    { 0xf7b4075d, SUC_lexicon_tags_30 },
    { 0xec74075e, SUC_lexicon_tags_15 },
    { 0x6f4075d, SUC_lexicon_tags_7 },
    { 0x9f54075e, SUC_lexicon_tags_57 },
    { 0x80b4075c, SUC_lexicon_tags_1 },
    { 0x1f140757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0140764, SUC_lexicon_tags_15 },
    { 0xa7f40765, SUC_lexicon_tags_26 },
    { 0xf1140766, SUC_lexicon_tags_37 },
    { 0x86b40766, SUC_lexicon_tags_21 },
    { 0xd7940768, SUC_lexicon_tags_19 },
    { 0xbdb40769, SUC_lexicon_tags_33 },
    { 0x9d340766, SUC_lexicon_tags_23 },
    { 0x5234076a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b14076f, SUC_lexicon_tags_31 },
    { 0x1414076f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26940774, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69b40776, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe5d40778, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4077e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdab40780, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4e940782, SUC_lexicon_tags_29 },
    { 0x3c340782, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2b40785, SUC_lexicon_tags_11 },
    { 0x16b40786, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x41740788, SUC_lexicon_tags_25 },
    { 0x62340788, SUC_lexicon_tags_38 },
    { 0x3dd4078a, SUC_lexicon_tags_7 },
    { 0x86f40789, SUC_lexicon_tags_11 },
    { 0x8eb4078c, SUC_lexicon_tags_21 },
    { 0xa194078d, SUC_lexicon_tags_21 },
    { 0x65f4078a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x34b40790, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x44540793, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f40796, SUC_lexicon_tags_26 },
    { 0xd6940797, SUC_lexicon_tags_37 },
    { 0x3540798, SUC_lexicon_tags_31 },
    { 0x5ef40797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xadf4079b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9b4079d, SUC_lexicon_tags_14 },
    { 0x4f54079d, SUC_lexicon_tags_26 },
    { 0x5a14079f, SUC_lexicon_tags_1 },
    { 0xd0f407a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e5407a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x337407a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7407ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b407b2, SUC_lexicon_tags_1 },
    { 0x6e9407b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb9407b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db407ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb35407bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x639407bf, SUC_lexicon_tags_38 },
    { 0x865407c0, SUC_lexicon_tags_17 },
    { 0xea1407c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x699407c4, SUC_lexicon_tags_25 },
    { 0x483407c5, SUC_lexicon_tags_7 },
    { 0x8c7407c6, SUC_lexicon_tags_25 },
    { 0xa73407c6, SUC_lexicon_tags_11 },
    { 0xd0b407c8, SUC_lexicon_tags_25 },
    { 0x235407c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761407cf, SUC_lexicon_tags_30 },
    { 0x4db407d0, SUC_lexicon_tags_60 },
    { 0x251407d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87407d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb7407d8, SUC_lexicon_tags_15 },
    { 0x597407d9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49407e1, SUC_lexicon_tags_45 },
    { 0x407407e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99407e8, SUC_lexicon_tags_11 },
    { 0xe7b407e9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11407ee, SUC_lexicon_tags_11 },
    { 0x691407ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7407f3, SUC_lexicon_tags_5 },
    { 0x899407f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4c1407f6, SUC_lexicon_tags_25 },
    { 0xa7b407f6, SUC_lexicon_tags_37 },
    { 0xb1f407f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x157407fb, SUC_lexicon_tags_27 },
    { 0xecb407fc, SUC_lexicon_tags_1 },
    { 0x8f1407fd, SUC_lexicon_tags_71 },
    { 0x73b407fe, SUC_lexicon_tags_4 },
    { 0x313407fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1540801, SUC_lexicon_tags_1 },
    { 0x3e540802, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f740804, SUC_lexicon_tags_19 },
    { 0xa1140805, SUC_lexicon_tags_37 },
    { 0xd0d40805, SUC_lexicon_tags_2 },
    { 0x95340805, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee94080b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87740811, SUC_lexicon_tags_45 },
    { 0xb0340812, SUC_lexicon_tags_43 },
    { 0xb4b40813, SUC_lexicon_tags_12 },
    { 0x11740814, SUC_lexicon_tags_217 },
    { 0xa6340813, SUC_lexicon_tags_63 },
    { 0x6df40816, SUC_lexicon_tags_1 },
    { 0x16540817, SUC_lexicon_tags_7 },
    { 0x1740813, SUC_lexicon_tags_17 },
    { 0x66b40814, SUC_lexicon_tags_49 },
    { 0xfe140816, SUC_lexicon_tags_1 },
    { 0xb334081b, SUC_lexicon_tags_11 },
    { 0xb994081a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1174081e, SUC_lexicon_tags_8 },
    { 0x8934081f, SUC_lexicon_tags_2 },
    { 0xbc740820, SUC_lexicon_tags_120 },
    { 0x1340821, SUC_lexicon_tags_7 },
    { 0xbb140820, SUC_lexicon_tags_33 },
    { 0xfcb40823, SUC_lexicon_tags_23 },
    { 0xe6b4081f, SUC_lexicon_tags_12 },
    { 0x9334081f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb40829, SUC_lexicon_tags_7 },
    { 0xe154082a, SUC_lexicon_tags_37 },
    { 0xfbf4082a, SUC_lexicon_tags_2 },
    { 0x8f74082c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2254082f, SUC_lexicon_tags_4 },
    { 0x58d4082f, SUC_lexicon_tags_15 },
    { 0x4f340831, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe140835, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c740837, SUC_lexicon_tags_29 },
    { 0x8ed40838, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3c34083a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1f14083c, SUC_lexicon_tags_37 },
    { 0x95d4083d, SUC_lexicon_tags_31 },
    { 0x1394083e, SUC_lexicon_tags_1 },
    { 0x5254083f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x540841, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf40844, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8140847, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19340849, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b4084c, SUC_lexicon_tags_11 },
    { 0x5d74084c, SUC_lexicon_tags_3 },
    { 0xd7d4084e, SUC_lexicon_tags_0 },
    { 0xf8b4084e, SUC_lexicon_tags_25 },
    { 0xb5b40850, SUC_lexicon_tags_49 },
    { 0xb0340850, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d540854, SUC_lexicon_tags_6 },
    { 0xa8140855, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6340859, SUC_lexicon_tags_15 },
    { 0x79b4085a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b4085d, SUC_lexicon_tags_17 },
    { 0xc534085d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b40861, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31140864, SUC_lexicon_tags_43 },
    { 0xb8140864, SUC_lexicon_tags_26 },
    { 0x58140866, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x12740868, SUC_lexicon_tags_19 },
    { 0x99f40868, SUC_lexicon_tags_1 },
    { 0x9d4086a, SUC_lexicon_tags_1 },
    { 0x4394086b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a140874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d40877, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb474087a, SUC_lexicon_tags_11 },
    { 0x7ad4087a, SUC_lexicon_tags_0 },
    { 0x7ab4087a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd934087e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d340881, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f34088a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7db4088c, SUC_lexicon_tags_17 },
    { 0xdbb4088d, SUC_lexicon_tags_29 },
    { 0xb094088e, SUC_lexicon_tags_1 },
    { 0x6174088f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f40894, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e34089a, SUC_lexicon_tags_0 },
    { 0x9c34089b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf4089e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07408a1, SUC_lexicon_tags_19 },
    { 0x37f408a2, SUC_lexicon_tags_7 },
    { 0xd7408a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x199408a6, SUC_lexicon_tags_53 },
    { 0xb0f408a7, SUC_lexicon_tags_17 },
    { 0x4cd408a8, SUC_lexicon_tags_17 },
    { 0xfad408a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783408ad, SUC_lexicon_tags_11 },
    { 0xc01408ad, SUC_lexicon_tags_64 },
    { 0x257408af, SUC_lexicon_tags_47 },
    { 0x751408af, SUC_lexicon_tags_25 },
    { 0x5db408b0, SUC_lexicon_tags_71 },
    { 0x7f9408b2, SUC_lexicon_tags_15 },
    { 0xcbd408b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f408b9, SUC_lexicon_tags_1691 },
    { 0x67408ba, SUC_lexicon_tags_37 },
    { 0xb3d408bb, SUC_lexicon_tags_14 },
    { 0x56f408bc, SUC_lexicon_tags_9 },
    { 0xf1b408bc, SUC_lexicon_tags_0 },
    { 0xaf1408be, SUC_lexicon_tags_28 },
    { 0x747408be, SUC_lexicon_tags_37 },
    { 0xe1408c0, SUC_lexicon_tags_2 },
    { 0xac7408c1, SUC_lexicon_tags_2 },
    { 0xb25408c2, SUC_lexicon_tags_11 },
    { 0x95d408c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f408c7, SUC_lexicon_tags_84 },
    { 0x17d408c8, SUC_lexicon_tags_0 },
    { 0x46d408c9, SUC_lexicon_tags_38 },
    { 0xfa5408ca, SUC_lexicon_tags_11 },
    { 0x1b9408cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x987408cd, SUC_lexicon_tags_43 },
    { 0xaab408cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef408d4, SUC_lexicon_tags_11 },
    { 0xacd408d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x295408d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd408db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x357408de, SUC_lexicon_tags_12 },
    { 0x73408de, SUC_lexicon_tags_23 },
    { 0xd1d408e0, SUC_lexicon_tags_15 },
    { 0x5b5408e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69408e7, SUC_lexicon_tags_19 },
    { 0x5d9408e8, SUC_lexicon_tags_2 },
    { 0x337408e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b408ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df408f0, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x135408f3, SUC_lexicon_tags_16 },
    { 0x6bd408f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4540903, SUC_lexicon_tags_17 },
    { 0xe7f40903, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2934090b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f540912, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32340915, SUC_lexicon_tags_11 },
    { 0x66140916, SUC_lexicon_tags_117 },
    { 0xb7740917, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df4091a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b40920, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd40923, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f40926, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db4092c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1634092e, SUC_lexicon_tags_12 },
    { 0x7ed4092f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d40933, SUC_lexicon_tags_32 },
    { 0xdd940934, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb540936, SUC_lexicon_tags_23 },
    { 0x1f740937, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54093b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df4093f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f40943, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26540945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e740949, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xca74094b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d4094e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70540950, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2b40952, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2f40954, SUC_lexicon_tags_27 },
    { 0x57d40954, SUC_lexicon_tags_25 },
    { 0xb940956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7340959, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa814095e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39740961, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c140964, SUC_lexicon_tags_0 },
    { 0x3ad40965, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf940967, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x98140969, SUC_lexicon_tags_6 },
    { 0xf774096a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0f4096c, SUC_lexicon_tags_61 },
    { 0x3714096c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff140973, SUC_lexicon_tags_33 },
    { 0x8cf40974, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a340977, SUC_lexicon_tags_17 },
    { 0xb8d40977, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd4097b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1340980, SUC_lexicon_tags_14 },
    { 0x9d740981, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4140984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e140988, SUC_lexicon_tags_15 },
    { 0xc8740989, SUC_lexicon_tags_15 },
    { 0x3b34098a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe194098c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea14098e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f40996, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31540998, SUC_lexicon_tags_17 },
    { 0xad740999, SUC_lexicon_tags_27 },
    { 0xfef40999, SUC_lexicon_tags_17 },
    { 0xdd14099b, SUC_lexicon_tags_8 },
    { 0x4e34099c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9409a1, SUC_lexicon_tags_37 },
    { 0x1f409a2, SUC_lexicon_tags_239 },
    { 0, NULL },
    { 0x39b409a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf15409a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69409ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd409b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x121409b4, SUC_lexicon_tags_4 },
    { 0xddd409b5, SUC_lexicon_tags_14 },
    { 0xd37409b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd13409b8, SUC_lexicon_tags_37 },
    { 0x295409b9, SUC_lexicon_tags_179 },
    { 0x395409b9, SUC_lexicon_tags_2 },
    { 0x2ff409b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65409bf, SUC_lexicon_tags_11 },
    { 0xf55409bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x723409c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b9409c4, SUC_lexicon_tags_2 },
    { 0x6c5409c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09409c8, SUC_lexicon_tags_11 },
    { 0xb31409c8, SUC_lexicon_tags_14 },
    { 0x715409ca, SUC_lexicon_tags_7 },
    { 0x8cb409ca, SUC_lexicon_tags_21 },
    { 0x309409cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x281409ce, SUC_lexicon_tags_17 },
    { 0xabd409ce, SUC_lexicon_tags_1 },
    { 0xd4b409ce, SUC_lexicon_tags_7 },
    { 0x33d409d1, SUC_lexicon_tags_33 },
    { 0x53d409d1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb409d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f409dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x419409de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fb409e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65409e3, SUC_lexicon_tags_15 },
    { 0xed409e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77409e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5409eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715409ef, SUC_lexicon_tags_49 },
    { 0xc3f409f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5df409f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53b409f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93409f9, SUC_lexicon_tags_43 },
    { 0xc51409f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27140a02, SUC_lexicon_tags_17 },
    { 0x21340a02, SUC_lexicon_tags_12 },
    { 0xa1940a02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x33740a06, SUC_lexicon_tags_23 },
    { 0xcf540a06, SUC_lexicon_tags_235 },
    { 0x3d340a06, SUC_lexicon_tags_45 },
    { 0xcc540a06, SUC_lexicon_tags_37 },
    { 0x6c740a0a, SUC_lexicon_tags_43 },
    { 0x6bf40a0b, SUC_lexicon_tags_23 },
    { 0x8340a0c, SUC_lexicon_tags_181 },
    { 0xf0740a0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed140a0f, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0x96140a11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1140a16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x40b40a18, SUC_lexicon_tags_9 },
    { 0x59b40a19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40b40a1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7740a26, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x69540a28, SUC_lexicon_tags_23 },
    { 0x25540a29, SUC_lexicon_tags_12 },
    { 0xe2140a29, SUC_lexicon_tags_15 },
    { 0x71b40a2b, SUC_lexicon_tags_7 },
    { 0xb4940a2c, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef40a30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91140a35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f40a40, SUC_lexicon_tags_60 },
    { 0x62d40a40, SUC_lexicon_tags_7 },
    { 0x3a940a42, SUC_lexicon_tags_66 },
    { 0x17940a42, SUC_lexicon_tags_21 },
    { 0x22b40a43, SUC_lexicon_tags_11 },
    { 0x37b40a42, SUC_lexicon_tags_30 },
    { 0xf6140a43, SUC_lexicon_tags_2 },
    { 0xe8b40a40, SUC_lexicon_tags_11 },
    { 0xabf40a48, SUC_lexicon_tags_25 },
    { 0x10140a49, SUC_lexicon_tags_37 },
    { 0x59f40a49, SUC_lexicon_tags_38 },
    { 0x61940a4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc340a4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b340a53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82f40a55, SUC_lexicon_tags_17 },
    { 0x8340a56, SUC_lexicon_tags_15 },
    { 0x60140a56, SUC_lexicon_tags_37 },
    { 0x5c940a57, SUC_lexicon_tags_37 },
    { 0x80340a57, SUC_lexicon_tags_2 },
    { 0x4ed40a5a, SUC_lexicon_tags_48 },
    { 0x62540a56, SUC_lexicon_tags_45 },
    { 0xd3d40a55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9540a5e, SUC_lexicon_tags_42 },
    { 0x34f40a5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6340a62, SUC_lexicon_tags_77 },
    { 0x5f140a63, SUC_lexicon_tags_23 },
    { 0x5cf40a64, SUC_lexicon_tags_15 },
    { 0x28740a64, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf340a68, SUC_lexicon_tags_25 },
    { 0x26540a68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x85740a6c, SUC_lexicon_tags_33 },
    { 0x75b40a6d, SUC_lexicon_tags_11 },
    { 0x74b40a6e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xaa140a70, SUC_lexicon_tags_16 },
    { 0x8cd40a71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d40a75, SUC_lexicon_tags_93 },
    { 0x46140a76, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1ff40a78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d40a80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x14b40a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d40a87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d40a91, SUC_lexicon_tags_2 },
    { 0xedb40a92, SUC_lexicon_tags_12 },
    { 0xffd40a91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb40a99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1940a9c, SUC_lexicon_tags_33 },
    { 0x8d340a9d, SUC_lexicon_tags_17 },
    { 0xbfb40a9d, SUC_lexicon_tags_148 },
    { 0xcbf40a9f, SUC_lexicon_tags_47 },
    { 0xfdb40a9f, SUC_lexicon_tags_19 },
    { 0x72140aa1, SUC_lexicon_tags_19 },
    { 0xff340aa2, SUC_lexicon_tags_11 },
    { 0xb6140aa2, SUC_lexicon_tags_1 },
    { 0x4d140aa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ff40aa6, SUC_lexicon_tags_25 },
    { 0xcb340aa7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd40aaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73140aad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d40aba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd740abc, SUC_lexicon_tags_23 },
    { 0xd4140abc, SUC_lexicon_tags_16 },
    { 0x33740abe, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad40ac1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f40ac7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2740aca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x86940acd, SUC_lexicon_tags_16 },
    { 0x30340ace, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b40ad6, SUC_lexicon_tags_19 },
    { 0x1eb40ad7, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e740add, SUC_lexicon_tags_12 },
    { 0x99140add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe740ae1, SUC_lexicon_tags_15 },
    { 0x16340ae2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99540aeb, SUC_lexicon_tags_17 },
    { 0xd8740aeb, SUC_lexicon_tags_27 },
    { 0xff340aed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb40af9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f940aff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30340b01, SUC_lexicon_tags_47 },
    { 0x940b02, SUC_lexicon_tags_37 },
    { 0xf340b01, SUC_lexicon_tags_9 },
    { 0x1b540b04, SUC_lexicon_tags_23 },
    { 0x2c940b05, SUC_lexicon_tags_2 },
    { 0xb4b40b04, SUC_lexicon_tags_23 },
    { 0xc6d40b05, SUC_lexicon_tags_30 },
    { 0x2140b04, SUC_lexicon_tags_17 },
    { 0x87b40b09, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb540b0b, SUC_lexicon_tags_7 },
    { 0xd9140b0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c140b18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d40b1e, SUC_lexicon_tags_25 },
    { 0x80b40b1f, SUC_lexicon_tags_17 },
    { 0x9b340b20, SUC_lexicon_tags_9 },
    { 0xe6540b21, SUC_lexicon_tags_17 },
    { 0xced40b20, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98340b27, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xd4940b29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f40b31, SUC_lexicon_tags_37 },
    { 0xa4f40b32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df40b38, SUC_lexicon_tags_39 },
    { 0xc7f40b39, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x45340b3b, SUC_lexicon_tags_17 },
    { 0xe5540b3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff40b3f, SUC_lexicon_tags_7 },
    { 0x6c940b40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf940b42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ad40b44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa340b47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86540b49, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa1d40b4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a140b4d, SUC_lexicon_tags_11 },
    { 0x6b940b4e, SUC_lexicon_tags_8 },
    { 0xbfd40b4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3940b54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd540b59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0940b5d, SUC_lexicon_tags_17 },
    { 0x26b40b5e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x10540b61, SUC_lexicon_tags_33 },
    { 0xb1f40b62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6940b64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb40b68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc0940b6a, SUC_lexicon_tags_37 },
    { 0x5340b6b, SUC_lexicon_tags_2 },
    { 0xa1740b6a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed40b71, SUC_lexicon_tags_25 },
    { 0x14f40b72, SUC_lexicon_tags_30 },
    { 0x6d140b73, SUC_lexicon_tags_45 },
    { 0xce740b74, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf1f40b76, SUC_lexicon_tags_33 },
    { 0x3b340b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4940b79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8140b7c, SUC_lexicon_tags_55 },
    { 0x3c540b7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d340b85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff740b87, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b40b8a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d40b8d, SUC_lexicon_tags_12 },
    { 0x16540b8e, SUC_lexicon_tags_15 },
    { 0xcd340b8f, SUC_lexicon_tags_25 },
    { 0xf6340b8e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c140b93, SUC_lexicon_tags_19 },
    { 0x8cf40b93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9540b98, SUC_lexicon_tags_25 },
    { 0x99540b98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40340b9d, SUC_lexicon_tags_1 },
    { 0xe1340b9e, SUC_lexicon_tags_12 },
    { 0x8a940b9f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b40ba2, SUC_lexicon_tags_10 },
    { 0x63f40ba3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a940baa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f740bad, SUC_lexicon_tags_11 },
    { 0xca140bad, SUC_lexicon_tags_9 },
    { 0x99940baf, SUC_lexicon_tags_1 },
    { 0x47540baf, SUC_lexicon_tags_11 },
    { 0x64f40bb1, SUC_lexicon_tags_33 },
    { 0x20f40bb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f140bb6, SUC_lexicon_tags_38 },
    { 0xbcb40bb7, SUC_lexicon_tags_0 },
    { 0xa1540bb8, SUC_lexicon_tags_69 },
    { 0x3c140bb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb40bbd, SUC_lexicon_tags_15 },
    { 0x35340bbe, SUC_lexicon_tags_7 },
    { 0xe4940bbd, SUC_lexicon_tags_15 },
    { 0xb6f40bc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd540bc2, SUC_lexicon_tags_11 },
    { 0xa4b40bc3, SUC_lexicon_tags_43 },
    { 0x75340bc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f40bcf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52340bd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89940bd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3af40bda, SUC_lexicon_tags_7 },
    { 0xb0d40bda, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5bd40bdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb740be0, SUC_lexicon_tags_11 },
    { 0xa5940be1, SUC_lexicon_tags_66 },
    { 0x7340be0, SUC_lexicon_tags_7 },
    { 0x2d40be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b140beb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe140bf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd40bfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29140bfe, SUC_lexicon_tags_15 },
    { 0xd3940bfe, SUC_lexicon_tags_15 },
    { 0x9ef40c00, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b40c04, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x99d40c06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf40c09, SUC_lexicon_tags_10 },
    { 0x9eb40c0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4740c0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6740c0f, SUC_lexicon_tags_11 },
    { 0xc8940c10, SUC_lexicon_tags_27 },
    { 0x41140c11, SUC_lexicon_tags_25 },
    { 0xf2540c12, SUC_lexicon_tags_9 },
    { 0x2f40c0f, SUC_lexicon_tags_46 },
    { 0x4db40c14, SUC_lexicon_tags_37 },
    { 0x7db40c15, SUC_lexicon_tags_4 },
    { 0x11d40c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35140c18, SUC_lexicon_tags_43 },
    { 0xd5d40c19, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0x21740c1b, SUC_lexicon_tags_25 },
    { 0x95740c1b, SUC_lexicon_tags_11 },
    { 0x50d40c1d, SUC_lexicon_tags_37 },
    { 0x20140c1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f40c21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32540c24, SUC_lexicon_tags_43 },
    { 0x8f340c25, SUC_lexicon_tags_12 },
    { 0x14740c24, SUC_lexicon_tags_46 },
    { 0x6f40c26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed140c2a, SUC_lexicon_tags_49 },
    { 0xd7b40c2b, SUC_lexicon_tags_17 },
    { 0x7db40c2c, SUC_lexicon_tags_15 },
    { 0xe9940c2c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde540c32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b40c3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14940c3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b40c42, SUC_lexicon_tags_1 },
    { 0xce540c42, SUC_lexicon_tags_37 },
    { 0x1c540c44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78540c50, SUC_lexicon_tags_1627 },
    { 0xd3540c51, SUC_lexicon_tags_235 },
    { 0xebb40c52, SUC_lexicon_tags_12 },
    { 0x84f40c53, SUC_lexicon_tags_17 },
    { 0x91940c52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55740c56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d40c59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda340c5e, SUC_lexicon_tags_26 },
    { 0xc3140c5f, SUC_lexicon_tags_61 },
    { 0x49740c60, SUC_lexicon_tags_15 },
    { 0xeb40c61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e940c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db40c68, SUC_lexicon_tags_30 },
    { 0xd4540c68, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8140c6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf140c71, SUC_lexicon_tags_12 },
    { 0xab40c72, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b40c7a, SUC_lexicon_tags_84 },
    { 0xf6b40c7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd40c7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed740c83, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e740c88, SUC_lexicon_tags_1 },
    { 0x2af40c89, SUC_lexicon_tags_31 },
    { 0xe2740c88, SUC_lexicon_tags_2 },
    { 0xf0740c8b, SUC_lexicon_tags_37 },
    { 0x73540c88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac140c92, SUC_lexicon_tags_25 },
    { 0xacf40c93, SUC_lexicon_tags_0 },
    { 0xaa140c94, SUC_lexicon_tags_27 },
    { 0x92940c92, SUC_lexicon_tags_61 },
    { 0x9340c96, SUC_lexicon_tags_15 },
    { 0x8340c97, SUC_lexicon_tags_17 },
    { 0x77340c97, SUC_lexicon_tags_54 },
    { 0x6a740c99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf4940c9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d40c9f, SUC_lexicon_tags_23 },
    { 0xab540c9f, SUC_lexicon_tags_2 },
    { 0x12b40ca1, SUC_lexicon_tags_34 },
    { 0xb340ca1, SUC_lexicon_tags_2 },
    { 0x2ab40ca3, SUC_lexicon_tags_2 },
    { 0x99b40ca1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7eb40ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb740ca8, SUC_lexicon_tags_193 },
    { 0x4a740ca9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf740cae, SUC_lexicon_tags_12 },
    { 0x47540cae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f40cb3, SUC_lexicon_tags_25 },
    { 0xf4b40cb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb40cbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a940cbf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x30740cc2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x35f40cc4, SUC_lexicon_tags_37 },
    { 0x3b540cc4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x31140cc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0540cca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf40ccd, SUC_lexicon_tags_2 },
    { 0x75d40ccd, SUC_lexicon_tags_14 },
    { 0xb2d40ccd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75740cd1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb940cd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d40cd7, SUC_lexicon_tags_2 },
    { 0x4cd40cd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5340cdb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x32740cde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff40ce4, SUC_lexicon_tags_2 },
    { 0x99540ce5, SUC_lexicon_tags_37 },
    { 0x8a540ce6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7740cef, SUC_lexicon_tags_11 },
    { 0x3f540cf0, SUC_lexicon_tags_37 },
    { 0xf4140cf1, SUC_lexicon_tags_63 },
    { 0x39340cf2, SUC_lexicon_tags_25 },
    { 0x3cd40cf0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b40cf6, SUC_lexicon_tags_37 },
    { 0x2f140cf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14340cf9, SUC_lexicon_tags_33 },
    { 0x3f940cfa, SUC_lexicon_tags_37 },
    { 0x72340cfa, SUC_lexicon_tags_4 },
    { 0x56d40cfc, SUC_lexicon_tags_31 },
    { 0x1e340cfd, SUC_lexicon_tags_17 },
    { 0x7db40cfa, SUC_lexicon_tags_7 },
    { 0x16b40cff, SUC_lexicon_tags_104 },
    { 0xc6540cfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37740d04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d40d07, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x7ed40d09, SUC_lexicon_tags_28 },
    { 0x84f40d09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f40d0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad40d10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x44740d12, SUC_lexicon_tags_11 },
    { 0x51340d13, SUC_lexicon_tags_12 },
    { 0x55140d12, SUC_lexicon_tags_95 },
    { 0x82f40d15, SUC_lexicon_tags_12 },
    { 0x92540d15, SUC_lexicon_tags_7 },
    { 0x8c740d17, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x97140d19, SUC_lexicon_tags_7 },
    { 0x35940d1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e540d1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98340d28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab40d2d, SUC_lexicon_tags_31 },
    { 0xc1140d2e, SUC_lexicon_tags_47 },
    { 0xf1740d2f, SUC_lexicon_tags_1 },
    { 0x88540d2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2340d33, SUC_lexicon_tags_15 },
    { 0x42740d34, SUC_lexicon_tags_37 },
    { 0x49f40d34, SUC_lexicon_tags_25 },
    { 0x97140d34, SUC_lexicon_tags_25 },
    { 0xbf40d37, SUC_lexicon_tags_33 },
    { 0xcb340d34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b940d3a, SUC_lexicon_tags_17 },
    { 0xf4f40d3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4540d40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16740d42, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1140d44, SUC_lexicon_tags_12 },
    { 0xa1940d44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb940d48, SUC_lexicon_tags_1 },
    { 0xc4140d49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b40d4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x78f40d50, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43940d54, SUC_lexicon_tags_11 },
    { 0xadd40d55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x340d57, SUC_lexicon_tags_23 },
    { 0x76d40d57, SUC_lexicon_tags_37 },
    { 0x25740d59, SUC_lexicon_tags_11 },
    { 0xe8d40d5a, SUC_lexicon_tags_32 },
    { 0x1d540d5b, SUC_lexicon_tags_43 },
    { 0xd3940d58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a740d5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x95740d61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4340d66, SUC_lexicon_tags_43 },
    { 0xd3d40d66, SUC_lexicon_tags_42 },
    { 0x21540d66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e540d6b, SUC_lexicon_tags_31 },
    { 0x99f40d6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58140d6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f40d73, SUC_lexicon_tags_2 },
    { 0x5fd40d74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57940d79, SUC_lexicon_tags_11 },
    { 0x77740d7a, SUC_lexicon_tags_47 },
    { 0x1eb40d7b, SUC_lexicon_tags_25 },
    { 0x40540d7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f40d80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67340d83, SUC_lexicon_tags_2 },
    { 0x10f40d83, SUC_lexicon_tags_25 },
    { 0xf9b40d85, SUC_lexicon_tags_15 },
    { 0xc0340d85, SUC_lexicon_tags_48 },
    { 0xc6140d86, SUC_lexicon_tags_1 },
    { 0xc5140d85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9740d91, SUC_lexicon_tags_12 },
    { 0x17740d92, SUC_lexicon_tags_63 },
    { 0x96340d93, SUC_lexicon_tags_8 },
    { 0x36540d94, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6c740d96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e140da1, SUC_lexicon_tags_84 },
    { 0x7ff40da2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x58940da5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcd540da7, SUC_lexicon_tags_12 },
    { 0x4d340da7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5140dab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1bb40dad, SUC_lexicon_tags_56 },
    { 0x66740dae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd740db1, SUC_lexicon_tags_45 },
    { 0xe4340db2, SUC_lexicon_tags_108 },
    { 0x9740db1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x55740db5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c540db8, SUC_lexicon_tags_1 },
    { 0xc2140db9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44740dc2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x70740dc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf5740dc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b40dca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f40dce, SUC_lexicon_tags_4 },
    { 0x96940dce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x35940dd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d40ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d40de4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef40de8, SUC_lexicon_tags_25 },
    { 0x99340de9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b340dec, SUC_lexicon_tags_19 },
    { 0xfe940ded, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d40df2, SUC_lexicon_tags_7 },
    { 0xbbd40df2, SUC_lexicon_tags_37 },
    { 0x9c140df3, SUC_lexicon_tags_37 },
    { 0x8b40df5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5940dfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a740dfc, SUC_lexicon_tags_33 },
    { 0x3fb40dfd, SUC_lexicon_tags_37 },
    { 0xc5340dfd, SUC_lexicon_tags_8 },
    { 0x47740dfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63940e02, SUC_lexicon_tags_30 },
    { 0x2b40e03, SUC_lexicon_tags_26 },
    { 0x31740e04, SUC_lexicon_tags_49 },
    { 0xd7140e04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99740e0b, SUC_lexicon_tags_84 },
    { 0xc9f40e0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5ef40e0e, SUC_lexicon_tags_17 },
    { 0x44f40e0f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf40e13, SUC_lexicon_tags_0 },
    { 0xb340e14, SUC_lexicon_tags_13 },
    { 0x5cb40e15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6340e1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d40e1f, SUC_lexicon_tags_47 },
    { 0xe5340e20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9d540e22, SUC_lexicon_tags_0 },
    { 0x3a540e23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b40e27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3740e2a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf5740e2c, SUC_lexicon_tags_16 },
    { 0x1740e2d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x95540e2f, SUC_lexicon_tags_23 },
    { 0xd9140e30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58540e32, SUC_lexicon_tags_63 },
    { 0xfd740e33, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf40e36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd40e39, SUC_lexicon_tags_1603 },
    { 0, NULL },
    { 0, NULL },
    { 0x49140e3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3340e41, SUC_lexicon_tags_82 },
    { 0x2a740e41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2140e47, SUC_lexicon_tags_62 },
    { 0x9cd40e48, SUC_lexicon_tags_422 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1340e4b, SUC_lexicon_tags_1 },
    { 0x6d540e4b, SUC_lexicon_tags_19 },
    { 0xb5b40e4c, SUC_lexicon_tags_7 },
    { 0x1fd40e4e, SUC_lexicon_tags_2 },
    { 0x92140e4e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f40e54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xba540e56, SUC_lexicon_tags_41 },
    { 0x2d940e57, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x73340e59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd40e62, SUC_lexicon_tags_14 },
    { 0xf9f40e63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f40e66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd40e6a, SUC_lexicon_tags_7 },
    { 0x94740e6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x79d40e6e, SUC_lexicon_tags_27 },
    { 0x90340e6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c740e76, SUC_lexicon_tags_37 },
    { 0xd4b40e77, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdc140e79, SUC_lexicon_tags_77 },
    { 0x7f940e79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed40e80, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c940e83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x68d40e85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f40e88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb40e8d, SUC_lexicon_tags_7 },
    { 0x73340e8e, SUC_lexicon_tags_12 },
    { 0x18740e8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d40e9a, SUC_lexicon_tags_35 },
    { 0x940e9b, SUC_lexicon_tags_12 },
    { 0xd8740e9c, SUC_lexicon_tags_74 },
    { 0xcbb40e9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d40ea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72940eac, SUC_lexicon_tags_63 },
    { 0x62f40ead, SUC_lexicon_tags_62 },
    { 0x7f140eae, SUC_lexicon_tags_14 },
    { 0xd5140eaf, SUC_lexicon_tags_25 },
    { 0xaa940eac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x61140eb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d540eb9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf40ebd, SUC_lexicon_tags_2 },
    { 0x1dd40ebe, SUC_lexicon_tags_7 },
    { 0x30740ebf, SUC_lexicon_tags_17 },
    { 0xa7d40ebe, SUC_lexicon_tags_62 },
    { 0x96140ec1, SUC_lexicon_tags_15 },
    { 0xc5540ec2, SUC_lexicon_tags_7 },
    { 0x76940ec2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x84340ec5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8340ec7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b40ecb, SUC_lexicon_tags_26 },
    { 0x43140ecc, SUC_lexicon_tags_2 },
    { 0xb3940ecd, SUC_lexicon_tags_39 },
    { 0x16b40ece, SUC_lexicon_tags_37 },
    { 0x8b340ece, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67f40ed1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30940ed4, SUC_lexicon_tags_11 },
    { 0xa4540ed5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a740ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa7d40eda, SUC_lexicon_tags_17 },
    { 0x68b40edb, SUC_lexicon_tags_11 },
    { 0xd3940edc, SUC_lexicon_tags_33 },
    { 0x52740eda, SUC_lexicon_tags_15 },
    { 0xc8b40ede, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f940ee0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x45340ee3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2ff40ee5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b140ee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43940eeb, SUC_lexicon_tags_12 },
    { 0xce540eeb, SUC_lexicon_tags_7 },
    { 0x77140eed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a140ef0, SUC_lexicon_tags_29 },
    { 0x27740ef1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c340ef7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0340efa, SUC_lexicon_tags_25 },
    { 0xef40efa, SUC_lexicon_tags_37 },
    { 0x32140efc, SUC_lexicon_tags_39 },
    { 0xd7940efd, SUC_lexicon_tags_82 },
    { 0xaf540efe, SUC_lexicon_tags_21 },
    { 0xc0540eff, SUC_lexicon_tags_35 },
    { 0x29f40f00, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd540f02, SUC_lexicon_tags_19 },
    { 0x5940f03, SUC_lexicon_tags_37 },
    { 0x68540f04, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xd6540f06, SUC_lexicon_tags_14 },
    { 0x8ff40f06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab40f0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5140f0e, SUC_lexicon_tags_7 },
    { 0x32d40f0f, SUC_lexicon_tags_6 },
    { 0x44d40f10, SUC_lexicon_tags_47 },
    { 0x5b740f0f, SUC_lexicon_tags_29 },
    { 0xbf740f12, SUC_lexicon_tags_11 },
    { 0x6cd40f12, SUC_lexicon_tags_31 },
    { 0x1ab40f14, SUC_lexicon_tags_0 },
    { 0xad540f14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc740f18, SUC_lexicon_tags_14 },
    { 0xc8940f18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9340f1b, SUC_lexicon_tags_920 },
    { 0xbef40f1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf340f23, SUC_lexicon_tags_38 },
    { 0xaeb40f23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff40f27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3740f2a, SUC_lexicon_tags_14 },
    { 0x7d740f2b, SUC_lexicon_tags_213 },
    { 0x8b940f2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed40f36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1140f39, SUC_lexicon_tags_7 },
    { 0xabf40f3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b40f3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bd40f3f, SUC_lexicon_tags_12 },
    { 0xbc140f40, SUC_lexicon_tags_27 },
    { 0x27140f40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc0740f43, SUC_lexicon_tags_63 },
    { 0x83540f43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db40f47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd40f54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d40f62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7db40f64, SUC_lexicon_tags_63 },
    { 0xc4540f64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29b40f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6940f6d, SUC_lexicon_tags_17 },
    { 0x25d40f6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0140f72, SUC_lexicon_tags_46 },
    { 0xbc940f72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb40f75, SUC_lexicon_tags_63 },
    { 0x55340f76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef940f7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b40f82, SUC_lexicon_tags_30 },
    { 0x28d40f82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x44540f86, SUC_lexicon_tags_1 },
    { 0x4a940f87, SUC_lexicon_tags_29 },
    { 0x14940f88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2140f8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd40f93, SUC_lexicon_tags_28 },
    { 0x51f40f93, SUC_lexicon_tags_27 },
    { 0x38d40f93, SUC_lexicon_tags_15 },
    { 0xe4d40f96, SUC_lexicon_tags_1 },
    { 0x4d540f97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23540f9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeeb40f9e, SUC_lexicon_tags_12 },
    { 0x72540f9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63540fa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd940fa9, SUC_lexicon_tags_9 },
    { 0xde340fa9, SUC_lexicon_tags_55 },
    { 0xadd40fab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62540fae, SUC_lexicon_tags_12 },
    { 0x13540faf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde740fb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70140fb4, SUC_lexicon_tags_1 },
    { 0x7ed40fb5, SUC_lexicon_tags_68 },
    { 0xb0940fb6, SUC_lexicon_tags_9 },
    { 0x5fd40fb7, SUC_lexicon_tags_17 },
    { 0x6db40fb6, SUC_lexicon_tags_70 },
    { 0xd2540fb9, SUC_lexicon_tags_28 },
    { 0x9940fba, SUC_lexicon_tags_27 },
    { 0xe9140fbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4940fbd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7940fbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b40fc3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9cd40fc5, SUC_lexicon_tags_8 },
    { 0x22740fc6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83140fcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb40fcf, SUC_lexicon_tags_35 },
    { 0xef340fd0, SUC_lexicon_tags_12 },
    { 0x57d40fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68740fd6, SUC_lexicon_tags_26 },
    { 0x14540fd6, SUC_lexicon_tags_27 },
    { 0xe0940fd6, SUC_lexicon_tags_8 },
    { 0xf5f40fd9, SUC_lexicon_tags_79 },
    { 0xe5140fd9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87340fdc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd540fe1, SUC_lexicon_tags_33 },
    { 0x5db40fe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cf40fe4, SUC_lexicon_tags_35 },
    { 0x58740fe5, SUC_lexicon_tags_17 },
    { 0x26940fe6, SUC_lexicon_tags_2 },
    { 0x72340fe4, SUC_lexicon_tags_54 },
    { 0xf8f40fe8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41b40fea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af40fed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57f40fef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb40ff7, SUC_lexicon_tags_0 },
    { 0x7db40ff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb340ffa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78741002, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8141005, SUC_lexicon_tags_31 },
    { 0x22941006, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84141008, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2b4100a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b4100f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14b41011, SUC_lexicon_tags_2 },
    { 0xd1741012, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b41017, SUC_lexicon_tags_67 },
    { 0x26741018, SUC_lexicon_tags_23 },
    { 0xcd141019, SUC_lexicon_tags_35 },
    { 0x31b4101a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9834101e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27341024, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc0141026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4141028, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57f4102a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b4102e, SUC_lexicon_tags_15 },
    { 0xb4b4102f, SUC_lexicon_tags_7 },
    { 0x3d34102e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5d41032, SUC_lexicon_tags_12 },
    { 0x5f141033, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99541038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a94103e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66541040, SUC_lexicon_tags_1 },
    { 0x15141041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9141043, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6341046, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88b41048, SUC_lexicon_tags_2 },
    { 0x68741049, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x74b4104b, SUC_lexicon_tags_17 },
    { 0xae94104b, SUC_lexicon_tags_17 },
    { 0xb614104d, SUC_lexicon_tags_25 },
    { 0xa6f4104e, SUC_lexicon_tags_25 },
    { 0xa854104d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a741056, SUC_lexicon_tags_15 },
    { 0x66d41057, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3514105a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd41060, SUC_lexicon_tags_15 },
    { 0x64741061, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef941067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91941069, SUC_lexicon_tags_30 },
    { 0x4bd41069, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa794106c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa34106e, SUC_lexicon_tags_19 },
    { 0xee94106f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed41072, SUC_lexicon_tags_31 },
    { 0x9e141073, SUC_lexicon_tags_14 },
    { 0x13d41074, SUC_lexicon_tags_7 },
    { 0x6f541073, SUC_lexicon_tags_13 },
    { 0x5c741076, SUC_lexicon_tags_32 },
    { 0xe0941072, SUC_lexicon_tags_11 },
    { 0x94b41075, SUC_lexicon_tags_29 },
    { 0x83141079, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5574107b, SUC_lexicon_tags_9 },
    { 0xcc94107c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe774107e, SUC_lexicon_tags_69 },
    { 0xfb34107f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f41082, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf541088, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74108d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41090, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x42f41092, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbd741094, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78541098, SUC_lexicon_tags_3 },
    { 0x5541098, SUC_lexicon_tags_17 },
    { 0xbef4109a, SUC_lexicon_tags_37 },
    { 0x89b4109b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd94109f, SUC_lexicon_tags_0 },
    { 0x1d9410a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b410a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9410aa, SUC_lexicon_tags_19 },
    { 0x6e5410ab, SUC_lexicon_tags_14 },
    { 0xb1410ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7a9410ae, SUC_lexicon_tags_2 },
    { 0xbe9410af, SUC_lexicon_tags_23 },
    { 0x8f1410b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe55410b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5410b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9410b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb410bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1410c2, SUC_lexicon_tags_63 },
    { 0x679410c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef410c6, SUC_lexicon_tags_6 },
    { 0xa5b410c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5410ce, SUC_lexicon_tags_30 },
    { 0xd2b410cf, SUC_lexicon_tags_12 },
    { 0xc7410d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9410d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35410d5, SUC_lexicon_tags_43 },
    { 0x39f410d6, SUC_lexicon_tags_68 },
    { 0xc25410d5, SUC_lexicon_tags_38 },
    { 0xc4d410d8, SUC_lexicon_tags_17 },
    { 0xd9410d9, SUC_lexicon_tags_47 },
    { 0x51d410d8, SUC_lexicon_tags_15 },
    { 0x50f410db, SUC_lexicon_tags_11 },
    { 0x601410d6, SUC_lexicon_tags_25 },
    { 0x49b410d7, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d410e8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b410ec, SUC_lexicon_tags_48 },
    { 0x28f410ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3410ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb410f3, SUC_lexicon_tags_7 },
    { 0xea3410f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13410f9, SUC_lexicon_tags_11 },
    { 0xd53410fa, SUC_lexicon_tags_39 },
    { 0x30d410f9, SUC_lexicon_tags_27 },
    { 0x891410fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16341101, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e141104, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd2741106, SUC_lexicon_tags_23 },
    { 0xd4341107, SUC_lexicon_tags_63 },
    { 0x90141107, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xb2b4110a, SUC_lexicon_tags_37 },
    { 0xa1d4110b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2134110e, SUC_lexicon_tags_19 },
    { 0x2794110f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b41111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d41116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb41119, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21b4111b, SUC_lexicon_tags_60 },
    { 0x7d14111c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x754111f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f41124, SUC_lexicon_tags_1 },
    { 0xf9d41125, SUC_lexicon_tags_1 },
    { 0x1eb41126, SUC_lexicon_tags_33 },
    { 0x30941127, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf4112a, SUC_lexicon_tags_16 },
    { 0x474112b, SUC_lexicon_tags_66 },
    { 0x7534112b, SUC_lexicon_tags_17 },
    { 0x5a74112b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x91b4112f, SUC_lexicon_tags_29 },
    { 0xa5941130, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1141134, SUC_lexicon_tags_69 },
    { 0x8f341134, SUC_lexicon_tags_12 },
    { 0x8bb41136, SUC_lexicon_tags_14 },
    { 0xc0d41137, SUC_lexicon_tags_1 },
    { 0xffb41134, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4074113b, SUC_lexicon_tags_15 },
    { 0xb934113c, SUC_lexicon_tags_56 },
    { 0xed54113c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xffb4113f, SUC_lexicon_tags_11 },
    { 0xfe541140, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb41143, SUC_lexicon_tags_25 },
    { 0x5b541144, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d41147, SUC_lexicon_tags_47 },
    { 0x70141147, SUC_lexicon_tags_30 },
    { 0xc1341149, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d4114e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4541150, SUC_lexicon_tags_30 },
    { 0xe2741151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f4115b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b4116a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab4116f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9741175, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b4117f, SUC_lexicon_tags_11 },
    { 0x6e741180, SUC_lexicon_tags_11 },
    { 0xe341181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb41184, SUC_lexicon_tags_15 },
    { 0xc4f41185, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82541187, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xa8b41189, SUC_lexicon_tags_14 },
    { 0x7294118a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4214118c, SUC_lexicon_tags_4 },
    { 0xb034118d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef41190, SUC_lexicon_tags_38 },
    { 0x68d41191, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d4119b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb374119d, SUC_lexicon_tags_38 },
    { 0x55b4119d, SUC_lexicon_tags_0 },
    { 0x1734119f, SUC_lexicon_tags_35 },
    { 0xec7411a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x329411a2, SUC_lexicon_tags_49 },
    { 0x4df411a2, SUC_lexicon_tags_25 },
    { 0x1a5411a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd17411a6, SUC_lexicon_tags_7 },
    { 0x1f9411a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1411ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x971411b0, SUC_lexicon_tags_4 },
    { 0x137411b1, SUC_lexicon_tags_108 },
    { 0x829411b1, SUC_lexicon_tags_23 },
    { 0x895411b3, SUC_lexicon_tags_3 },
    { 0x1ff411b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33411ba, SUC_lexicon_tags_67 },
    { 0x9cf411ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65411bf, SUC_lexicon_tags_10 },
    { 0x227411c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5411c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc5411c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceb411c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3411cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4db411cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb411d2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x597411d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x849411d8, SUC_lexicon_tags_12 },
    { 0xa5b411d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdf3411db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1df411dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7411e3, SUC_lexicon_tags_16 },
    { 0xfdd411e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x83f411e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257411eb, SUC_lexicon_tags_47 },
    { 0x223411ec, SUC_lexicon_tags_17 },
    { 0xbeb411ed, SUC_lexicon_tags_37 },
    { 0xac5411ee, SUC_lexicon_tags_37 },
    { 0x2f3411ef, SUC_lexicon_tags_3 },
    { 0x503411ee, SUC_lexicon_tags_158 },
    { 0x9b411f0, SUC_lexicon_tags_25 },
    { 0x193411f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x379411f5, SUC_lexicon_tags_7 },
    { 0xd8d411f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18f411f8, SUC_lexicon_tags_33 },
    { 0x453411f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c1411fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd5d411fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57d411ff, SUC_lexicon_tags_28 },
    { 0x3a3411ff, SUC_lexicon_tags_13 },
    { 0x33f41201, SUC_lexicon_tags_10 },
    { 0x9d41202, SUC_lexicon_tags_37 },
    { 0xc6341202, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87741205, SUC_lexicon_tags_3 },
    { 0x9b141205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89341208, SUC_lexicon_tags_12 },
    { 0x3dd41209, SUC_lexicon_tags_68 },
    { 0x2c54120a, SUC_lexicon_tags_47 },
    { 0xd8b41209, SUC_lexicon_tags_38 },
    { 0x20d4120c, SUC_lexicon_tags_3 },
    { 0xff14120d, SUC_lexicon_tags_34 },
    { 0x95d4120e, SUC_lexicon_tags_46 },
    { 0x64b4120d, SUC_lexicon_tags_84 },
    { 0x59141210, SUC_lexicon_tags_12 },
    { 0x70b4120e, SUC_lexicon_tags_19 },
    { 0x8f94120d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf41218, SUC_lexicon_tags_47 },
    { 0xf8341219, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d4121d, SUC_lexicon_tags_1 },
    { 0xee14121d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad41223, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce541226, SUC_lexicon_tags_17 },
    { 0x1a541227, SUC_lexicon_tags_87 },
    { 0xd1941227, SUC_lexicon_tags_28 },
    { 0xb4541227, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xe334122b, SUC_lexicon_tags_33 },
    { 0x5174122b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x254122e, SUC_lexicon_tags_28 },
    { 0x5634122f, SUC_lexicon_tags_17 },
    { 0x1054122f, SUC_lexicon_tags_37 },
    { 0xf4122e, SUC_lexicon_tags_68 },
    { 0x9ad4122f, SUC_lexicon_tags_45 },
    { 0xe3541233, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9141235, SUC_lexicon_tags_1 },
    { 0x6fd41236, SUC_lexicon_tags_1 },
    { 0x32541237, SUC_lexicon_tags_21 },
    { 0xb1941238, SUC_lexicon_tags_0 },
    { 0x40941239, SUC_lexicon_tags_27 },
    { 0x22d4123a, SUC_lexicon_tags_25 },
    { 0x13f4123b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f34123f, SUC_lexicon_tags_11 },
    { 0x7b14123f, SUC_lexicon_tags_9 },
    { 0xe6741240, SUC_lexicon_tags_65 },
    { 0x20341240, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46541245, SUC_lexicon_tags_0 },
    { 0xe0541246, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1d41248, SUC_lexicon_tags_27 },
    { 0xe3941248, SUC_lexicon_tags_34 },
    { 0xd234124a, SUC_lexicon_tags_32 },
    { 0x1054124b, SUC_lexicon_tags_23 },
    { 0x14124c, SUC_lexicon_tags_11 },
    { 0xa114124c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf341252, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x19541254, SUC_lexicon_tags_38 },
    { 0x78541255, SUC_lexicon_tags_37 },
    { 0xaab41256, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb3541258, SUC_lexicon_tags_25 },
    { 0xaa741258, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d4125d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d41264, SUC_lexicon_tags_7 },
    { 0x35f41265, SUC_lexicon_tags_17 },
    { 0xf2741266, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1341269, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f74126b, SUC_lexicon_tags_57 },
    { 0xc314126c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6534126f, SUC_lexicon_tags_37 },
    { 0x9af4126f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95741277, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af41280, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfd41282, SUC_lexicon_tags_47 },
    { 0xdf541282, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50f41285, SUC_lexicon_tags_37 },
    { 0xc5941286, SUC_lexicon_tags_26 },
    { 0xbf941287, SUC_lexicon_tags_15 },
    { 0x5cd41288, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd4128e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71141294, SUC_lexicon_tags_6 },
    { 0x2eb41295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b41298, SUC_lexicon_tags_49 },
    { 0x78341299, SUC_lexicon_tags_37 },
    { 0x6f541299, SUC_lexicon_tags_47 },
    { 0x77341298, SUC_lexicon_tags_25 },
    { 0xbd54129c, SUC_lexicon_tags_9 },
    { 0xcc54129d, SUC_lexicon_tags_194 },
    { 0x12b4129a, SUC_lexicon_tags_25 },
    { 0x1e74129f, SUC_lexicon_tags_61 },
    { 0x9ff412a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x753412a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc05412a4, SUC_lexicon_tags_37 },
    { 0x585412a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3ff412a7, SUC_lexicon_tags_1 },
    { 0xe33412a8, SUC_lexicon_tags_118 },
    { 0xf412a9, SUC_lexicon_tags_13 },
    { 0x82b412aa, SUC_lexicon_tags_27 },
    { 0x713412ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f1412ad, SUC_lexicon_tags_15 },
    { 0x1ff412ae, SUC_lexicon_tags_18 },
    { 0xc53412af, SUC_lexicon_tags_14 },
    { 0x889412b0, SUC_lexicon_tags_57 },
    { 0xacd412b1, SUC_lexicon_tags_63 },
    { 0xf9d412b0, SUC_lexicon_tags_37 },
    { 0xb1d412b3, SUC_lexicon_tags_39 },
    { 0x14d412ad, SUC_lexicon_tags_25 },
    { 0x49b412b4, SUC_lexicon_tags_8 },
    { 0x107412b6, SUC_lexicon_tags_3 },
    { 0xf3412b7, SUC_lexicon_tags_7 },
    { 0xc8d412b5, SUC_lexicon_tags_55 },
    { 0xd71412b3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f412bc, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xb7b412be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x633412c0, SUC_lexicon_tags_45 },
    { 0x9e9412c1, SUC_lexicon_tags_30 },
    { 0x45b412c2, SUC_lexicon_tags_1 },
    { 0x195412c3, SUC_lexicon_tags_39 },
    { 0xf07412c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfa3412c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9412cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b412ce, SUC_lexicon_tags_25 },
    { 0xc41412cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601412d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833412da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f412e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649412ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae5412ec, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11412f1, SUC_lexicon_tags_1 },
    { 0x395412f2, SUC_lexicon_tags_37 },
    { 0x84f412f2, SUC_lexicon_tags_1 },
    { 0xb1f412f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279412fa, SUC_lexicon_tags_37 },
    { 0xf8d412fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29412fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83412ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x33741301, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x63341303, SUC_lexicon_tags_6 },
    { 0x1c741303, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xeb341306, SUC_lexicon_tags_11 },
    { 0x86d41307, SUC_lexicon_tags_15 },
    { 0xe1341306, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56f4130a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8894130c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7774130e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb341312, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf554131b, SUC_lexicon_tags_53 },
    { 0x48f4131c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8314131e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4541323, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf541326, SUC_lexicon_tags_17 },
    { 0xdaf41326, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x25141329, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d41330, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f141336, SUC_lexicon_tags_25 },
    { 0x7c541336, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef4133a, SUC_lexicon_tags_26 },
    { 0x61b4133b, SUC_lexicon_tags_37 },
    { 0x35d4133b, SUC_lexicon_tags_11 },
    { 0xe414133d, SUC_lexicon_tags_37 },
    { 0x8694133e, SUC_lexicon_tags_19 },
    { 0xc294133e, SUC_lexicon_tags_100 },
    { 0x8f74133b, SUC_lexicon_tags_15 },
    { 0xed341341, SUC_lexicon_tags_103 },
    { 0x3f541342, SUC_lexicon_tags_1 },
    { 0x25f41342, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5141345, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b41348, SUC_lexicon_tags_11 },
    { 0x82941349, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7674134f, SUC_lexicon_tags_25 },
    { 0x27941350, SUC_lexicon_tags_17 },
    { 0xdf741350, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5614135d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64941360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72741362, SUC_lexicon_tags_11 },
    { 0xbc341362, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec741366, SUC_lexicon_tags_3 },
    { 0x1cb41367, SUC_lexicon_tags_25 },
    { 0x7c541368, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4136e, SUC_lexicon_tags_5 },
    { 0x3c14136e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77741378, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9bd4137a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb574137c, SUC_lexicon_tags_13 },
    { 0xd634137c, SUC_lexicon_tags_37 },
    { 0x1694137e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20541387, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd9341389, SUC_lexicon_tags_25 },
    { 0x76d41389, SUC_lexicon_tags_6 },
    { 0x8c54138b, SUC_lexicon_tags_19 },
    { 0xf1b4138a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11d4138e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb41391, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c341398, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5cf4139a, SUC_lexicon_tags_8 },
    { 0x7934139b, SUC_lexicon_tags_11 },
    { 0x8b34139b, SUC_lexicon_tags_42 },
    { 0xfcb4139d, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3413a2, SUC_lexicon_tags_37 },
    { 0x405413a3, SUC_lexicon_tags_22 },
    { 0x97d413a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401413a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99413ab, SUC_lexicon_tags_35 },
    { 0xd2d413ac, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd2f413ae, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xd4d413b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171413b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735413b9, SUC_lexicon_tags_25 },
    { 0x95413b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3413bd, SUC_lexicon_tags_15 },
    { 0x769413be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01413c2, SUC_lexicon_tags_1 },
    { 0x283413c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295413ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b413d2, SUC_lexicon_tags_25 },
    { 0xe63413d2, SUC_lexicon_tags_55 },
    { 0x4cd413d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31d413d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd413d9, SUC_lexicon_tags_2 },
    { 0xcd1413d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6a5413dc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f413e1, SUC_lexicon_tags_1 },
    { 0x429413e2, SUC_lexicon_tags_63 },
    { 0x2c7413e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd7413e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869413ee, SUC_lexicon_tags_43 },
    { 0x951413ef, SUC_lexicon_tags_39 },
    { 0x90f413ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51413f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc81413f6, SUC_lexicon_tags_4 },
    { 0xfa9413f6, SUC_lexicon_tags_38 },
    { 0xed9413f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37413fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bb413fe, SUC_lexicon_tags_0 },
    { 0xc37413fe, SUC_lexicon_tags_25 },
    { 0x26b413fe, SUC_lexicon_tags_19 },
    { 0xed341401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9341407, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d41410, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64341412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5b41414, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb674141a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd34141c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb4141f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17541427, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b341429, SUC_lexicon_tags_2 },
    { 0x1e74142a, SUC_lexicon_tags_7 },
    { 0x8e34142b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ed4142d, SUC_lexicon_tags_27 },
    { 0x1c94142e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d41432, SUC_lexicon_tags_14 },
    { 0xcd541433, SUC_lexicon_tags_37 },
    { 0x28941434, SUC_lexicon_tags_11 },
    { 0x141435, SUC_lexicon_tags_1 },
    { 0x71141435, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90541439, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6554143e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3341441, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7341443, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d41448, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5dd4144a, SUC_lexicon_tags_47 },
    { 0x2ad4144a, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x4914144d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b54145a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf4145d, SUC_lexicon_tags_66 },
    { 0x2134145e, SUC_lexicon_tags_47 },
    { 0xac34145e, SUC_lexicon_tags_1 },
    { 0x92741460, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6141465, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e941469, SUC_lexicon_tags_21 },
    { 0x61d4146a, SUC_lexicon_tags_39 },
    { 0x42941469, SUC_lexicon_tags_12 },
    { 0x39d4146c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc534146f, SUC_lexicon_tags_11 },
    { 0x1574146f, SUC_lexicon_tags_79 },
    { 0x28141471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82141473, SUC_lexicon_tags_4 },
    { 0x3a741474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bb41476, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf741478, SUC_lexicon_tags_37 },
    { 0x44d41479, SUC_lexicon_tags_37 },
    { 0x17141479, SUC_lexicon_tags_1 },
    { 0xe5d4147b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1541481, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xba941483, SUC_lexicon_tags_306 },
    { 0xa4741483, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df41489, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6bb4148b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f4148f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0141494, SUC_lexicon_tags_37 },
    { 0xc4d41495, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d41498, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa014149a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb34149d, SUC_lexicon_tags_47 },
    { 0x9a74149d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75414a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d414a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b414ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53b414af, SUC_lexicon_tags_65 },
    { 0x7ff414af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x557414b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95414b6, SUC_lexicon_tags_30 },
    { 0xf71414b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x837414b9, SUC_lexicon_tags_79 },
    { 0x243414ba, SUC_lexicon_tags_27 },
    { 0x605414bb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b414bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x915414c2, SUC_lexicon_tags_17 },
    { 0x999414c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777414cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb414d0, SUC_lexicon_tags_38 },
    { 0x327414d0, SUC_lexicon_tags_1 },
    { 0xd11414d0, SUC_lexicon_tags_25 },
    { 0x417414d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x335414d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03414d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x357414dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7414e1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b414e6, SUC_lexicon_tags_12 },
    { 0x151414e7, SUC_lexicon_tags_66 },
    { 0x28b414e7, SUC_lexicon_tags_14 },
    { 0x13b414e9, SUC_lexicon_tags_4 },
    { 0x47d414ea, SUC_lexicon_tags_37 },
    { 0x4cd414e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x78b414ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43414ef, SUC_lexicon_tags_32 },
    { 0x343414f0, SUC_lexicon_tags_25 },
    { 0x52f414f1, SUC_lexicon_tags_1 },
    { 0x21f414f1, SUC_lexicon_tags_53 },
    { 0xfdb414f3, SUC_lexicon_tags_19 },
    { 0x84f414f3, SUC_lexicon_tags_9 },
    { 0x255414f4, SUC_lexicon_tags_27 },
    { 0xf1b414f0, SUC_lexicon_tags_45 },
    { 0x903414f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x993414fa, SUC_lexicon_tags_7 },
    { 0xdcf414fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71141503, SUC_lexicon_tags_47 },
    { 0xceb41504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9341508, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a54150b, SUC_lexicon_tags_27 },
    { 0x6374150c, SUC_lexicon_tags_11 },
    { 0xbab4150b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31541510, SUC_lexicon_tags_15 },
    { 0x5a141510, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0541515, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x59741517, SUC_lexicon_tags_17 },
    { 0x23141517, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4314151c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c141521, SUC_lexicon_tags_11 },
    { 0x23741522, SUC_lexicon_tags_74 },
    { 0x1ef41523, SUC_lexicon_tags_29 },
    { 0xc2b41524, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4741529, SUC_lexicon_tags_47 },
    { 0x8294152a, SUC_lexicon_tags_14 },
    { 0x5e74152b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11941530, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc941533, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f4153d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd54153f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd41542, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0x1f541544, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb41548, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61741550, SUC_lexicon_tags_47 },
    { 0xf9d41551, SUC_lexicon_tags_33 },
    { 0xcb541552, SUC_lexicon_tags_84 },
    { 0x91141551, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52741558, SUC_lexicon_tags_301 },
    { 0x39741559, SUC_lexicon_tags_82 },
    { 0x61b4155a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9b4155c, SUC_lexicon_tags_35 },
    { 0xfc34155d, SUC_lexicon_tags_25 },
    { 0xe914155c, SUC_lexicon_tags_17 },
    { 0xdf14155f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea741564, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1741566, SUC_lexicon_tags_42 },
    { 0xa3741567, SUC_lexicon_tags_23 },
    { 0xd2141567, SUC_lexicon_tags_4 },
    { 0x91141567, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8941570, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf774157c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcd4157e, SUC_lexicon_tags_45 },
    { 0x2654157f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f41583, SUC_lexicon_tags_61 },
    { 0x69741584, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc4341586, SUC_lexicon_tags_33 },
    { 0x58541586, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23741589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f4158d, SUC_lexicon_tags_1 },
    { 0x3494158e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde941590, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5cf41592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8141594, SUC_lexicon_tags_37 },
    { 0xb2341595, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x9e941597, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7141599, SUC_lexicon_tags_2 },
    { 0x5e34159a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe94159f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f415a2, SUC_lexicon_tags_66 },
    { 0x819415a3, SUC_lexicon_tags_37 },
    { 0x49f415a4, SUC_lexicon_tags_6 },
    { 0x3d9415a4, SUC_lexicon_tags_37 },
    { 0xba7415a6, SUC_lexicon_tags_25 },
    { 0x627415a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45415ac, SUC_lexicon_tags_10 },
    { 0x653415ad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51415b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5415b4, SUC_lexicon_tags_25 },
    { 0x997415b5, SUC_lexicon_tags_23 },
    { 0xec9415b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0b415b8, SUC_lexicon_tags_1 },
    { 0x1f415b8, SUC_lexicon_tags_37 },
    { 0xe1f415b8, SUC_lexicon_tags_31 },
    { 0xbeb415bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d415bf, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f415c4, SUC_lexicon_tags_29 },
    { 0x4db415c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x29d415c7, SUC_lexicon_tags_23 },
    { 0xd49415c7, SUC_lexicon_tags_34 },
    { 0xe43415c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35415d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f415d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1415db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61f415dd, SUC_lexicon_tags_1 },
    { 0xdf1415de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d3415e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xced415e2, SUC_lexicon_tags_27 },
    { 0x4f415e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f415e6, SUC_lexicon_tags_17 },
    { 0x7a3415e6, SUC_lexicon_tags_11 },
    { 0xec3415e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919415f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4af415f6, SUC_lexicon_tags_55 },
    { 0x2b1415f7, SUC_lexicon_tags_11 },
    { 0x5ff415f8, SUC_lexicon_tags_1 },
    { 0xd6d415f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xefb415fb, SUC_lexicon_tags_0 },
    { 0x903415fc, SUC_lexicon_tags_7 },
    { 0x86b415fd, SUC_lexicon_tags_25 },
    { 0x1c1415fe, SUC_lexicon_tags_8 },
    { 0x9b3415ff, SUC_lexicon_tags_1 },
    { 0xa5d415fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0941603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51141607, SUC_lexicon_tags_0 },
    { 0xe8741607, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b4160c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6741612, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10541616, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f94161b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6b4161d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1334161f, SUC_lexicon_tags_7 },
    { 0x56941620, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d141625, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb741629, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4e94162b, SUC_lexicon_tags_27 },
    { 0x9d94162c, SUC_lexicon_tags_17 },
    { 0x78d4162d, SUC_lexicon_tags_45 },
    { 0xabb4162c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe3941630, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfd341632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d4163a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf4163e, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b41642, SUC_lexicon_tags_55 },
    { 0xc1941643, SUC_lexicon_tags_68 },
    { 0x2a141644, SUC_lexicon_tags_31 },
    { 0x6a341644, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6034164a, SUC_lexicon_tags_28 },
    { 0x9ad4164b, SUC_lexicon_tags_43 },
    { 0x9b54164b, SUC_lexicon_tags_79 },
    { 0x5974164d, SUC_lexicon_tags_11 },
    { 0xbc54164b, SUC_lexicon_tags_40 },
    { 0xb5b4164e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53541651, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba741653, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d41656, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcff41658, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f54165a, SUC_lexicon_tags_7 },
    { 0x3fd4165b, SUC_lexicon_tags_8 },
    { 0x10b4165c, SUC_lexicon_tags_2 },
    { 0x4534165c, SUC_lexicon_tags_66 },
    { 0x9f14165e, SUC_lexicon_tags_12 },
    { 0x2054165d, SUC_lexicon_tags_11 },
    { 0xf9b41660, SUC_lexicon_tags_37 },
    { 0xb6b4165c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c941664, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d541666, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3254166a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a741670, SUC_lexicon_tags_31 },
    { 0x3df41671, SUC_lexicon_tags_12 },
    { 0xfb41672, SUC_lexicon_tags_8 },
    { 0x6f141673, SUC_lexicon_tags_27 },
    { 0x94941672, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xb8141676, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13941678, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b4167c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x62f4167e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbb41680, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb7341682, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15141686, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd3d41688, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f74168a, SUC_lexicon_tags_15 },
    { 0x26d4168b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x72f4168d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb941692, SUC_lexicon_tags_17 },
    { 0x5ad41693, SUC_lexicon_tags_49 },
    { 0xb9b41694, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8741699, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf4169d, SUC_lexicon_tags_0 },
    { 0x69d4169e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x739416a0, SUC_lexicon_tags_29 },
    { 0xb21416a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x743416a3, SUC_lexicon_tags_11 },
    { 0x67f416a4, SUC_lexicon_tags_13 },
    { 0x9d1416a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x191416a8, SUC_lexicon_tags_28 },
    { 0x5ed416a9, SUC_lexicon_tags_12 },
    { 0xcef416aa, SUC_lexicon_tags_37 },
    { 0x783416ab, SUC_lexicon_tags_7 },
    { 0xe4f416ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14d416ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5416b2, SUC_lexicon_tags_47 },
    { 0x1f416b2, SUC_lexicon_tags_8 },
    { 0xa5416b2, SUC_lexicon_tags_37 },
    { 0x707416b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x75d416b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfab416b9, SUC_lexicon_tags_37 },
    { 0x75b416b9, SUC_lexicon_tags_33 },
    { 0x40d416bb, SUC_lexicon_tags_23 },
    { 0x607416bc, SUC_lexicon_tags_11 },
    { 0x791416bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17416c2, SUC_lexicon_tags_12 },
    { 0x9b416c3, SUC_lexicon_tags_1 },
    { 0x245416c4, SUC_lexicon_tags_12 },
    { 0xe03416c5, SUC_lexicon_tags_37 },
    { 0x62f416c2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf416c9, SUC_lexicon_tags_7 },
    { 0xdf9416ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff416d0, SUC_lexicon_tags_17 },
    { 0xa25416d1, SUC_lexicon_tags_16 },
    { 0x901416d0, SUC_lexicon_tags_25 },
    { 0x791416d1, SUC_lexicon_tags_72 },
    { 0x7f5416d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df416d7, SUC_lexicon_tags_23 },
    { 0x4bf416d7, SUC_lexicon_tags_33 },
    { 0xc21416d9, SUC_lexicon_tags_37 },
    { 0x53f416d8, SUC_lexicon_tags_15 },
    { 0xbff416d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d416de, SUC_lexicon_tags_11 },
    { 0x913416de, SUC_lexicon_tags_16 },
    { 0x32d416df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb31416e2, SUC_lexicon_tags_16 },
    { 0x31416e3, SUC_lexicon_tags_37 },
    { 0x12b416e4, SUC_lexicon_tags_26 },
    { 0x6db416e5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99416ec, SUC_lexicon_tags_11 },
    { 0x39d416ec, SUC_lexicon_tags_7 },
    { 0xe7f416ed, SUC_lexicon_tags_7 },
    { 0x65d416ef, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5416f6, SUC_lexicon_tags_82 },
    { 0x8fb416f6, SUC_lexicon_tags_0 },
    { 0xd49416f7, SUC_lexicon_tags_28 },
    { 0x29416f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7416fd, SUC_lexicon_tags_17 },
    { 0x779416fe, SUC_lexicon_tags_0 },
    { 0xeef416fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb941701, SUC_lexicon_tags_33 },
    { 0x3cd41701, SUC_lexicon_tags_57 },
    { 0xdaf41703, SUC_lexicon_tags_19 },
    { 0xb3741704, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90941708, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2f54170a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad14170d, SUC_lexicon_tags_9 },
    { 0x2d14170e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5341711, SUC_lexicon_tags_12 },
    { 0xcab41712, SUC_lexicon_tags_7 },
    { 0x64341713, SUC_lexicon_tags_17 },
    { 0xbcf41711, SUC_lexicon_tags_60 },
    { 0x41741711, SUC_lexicon_tags_15 },
    { 0x35941712, SUC_lexicon_tags_41 },
    { 0xdd141717, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5874171b, SUC_lexicon_tags_21 },
    { 0xce54171c, SUC_lexicon_tags_0 },
    { 0x6674171d, SUC_lexicon_tags_27 },
    { 0x54171c, SUC_lexicon_tags_2 },
    { 0x5894171f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50541723, SUC_lexicon_tags_45 },
    { 0x93141723, SUC_lexicon_tags_15 },
    { 0xf4541724, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4694172a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4172f, SUC_lexicon_tags_11 },
    { 0xcf94172f, SUC_lexicon_tags_14 },
    { 0xcfb41731, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2141733, SUC_lexicon_tags_32 },
    { 0xaed41734, SUC_lexicon_tags_11 },
    { 0x84b41735, SUC_lexicon_tags_43 },
    { 0x5e341736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47141738, SUC_lexicon_tags_82 },
    { 0x80141739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3694173c, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a141741, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46341744, SUC_lexicon_tags_27 },
    { 0x49941744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea741748, SUC_lexicon_tags_39 },
    { 0xb741748, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c541751, SUC_lexicon_tags_220 },
    { 0xadd41752, SUC_lexicon_tags_60 },
    { 0x8ad41752, SUC_lexicon_tags_23 },
    { 0xcc941753, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0341757, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4175e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf341761, SUC_lexicon_tags_1129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5141765, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60741767, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xa2741769, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4354176b, SUC_lexicon_tags_191 },
    { 0xe634176b, SUC_lexicon_tags_2 },
    { 0xf054176c, SUC_lexicon_tags_15 },
    { 0x4174176e, SUC_lexicon_tags_4 },
    { 0xf0f4176f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd341776, SUC_lexicon_tags_2 },
    { 0x9c141777, SUC_lexicon_tags_71 },
    { 0x58141778, SUC_lexicon_tags_2 },
    { 0x99941778, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc674177d, SUC_lexicon_tags_8 },
    { 0xcf94177e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3541782, SUC_lexicon_tags_23 },
    { 0x22341783, SUC_lexicon_tags_87 },
    { 0x7e341784, SUC_lexicon_tags_12 },
    { 0xb4741785, SUC_lexicon_tags_32 },
    { 0xe7941786, SUC_lexicon_tags_17 },
    { 0x35541786, SUC_lexicon_tags_10 },
    { 0xcab41788, SUC_lexicon_tags_19 },
    { 0x13941788, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e34178c, SUC_lexicon_tags_17 },
    { 0x95f4178d, SUC_lexicon_tags_37 },
    { 0x41d4178e, SUC_lexicon_tags_25 },
    { 0x4534178d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe34179a, SUC_lexicon_tags_29 },
    { 0x6074179a, SUC_lexicon_tags_23 },
    { 0xb654179b, SUC_lexicon_tags_1 },
    { 0xb474179c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd834179f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549417a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb417a8, SUC_lexicon_tags_6 },
    { 0xc23417a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd3417ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97417b1, SUC_lexicon_tags_2 },
    { 0x7c7417b1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d417ba, SUC_lexicon_tags_37 },
    { 0x3ab417bb, SUC_lexicon_tags_7 },
    { 0x735417bc, SUC_lexicon_tags_7 },
    { 0xc85417bd, SUC_lexicon_tags_13 },
    { 0x673417bb, SUC_lexicon_tags_8 },
    { 0x2d1417bd, SUC_lexicon_tags_11 },
    { 0xed1417ba, SUC_lexicon_tags_45 },
    { 0x91b417c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x147417c4, SUC_lexicon_tags_46 },
    { 0x153417c4, SUC_lexicon_tags_37 },
    { 0xbf7417c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07417c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x121417cc, SUC_lexicon_tags_27 },
    { 0x2ab417cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53417d4, SUC_lexicon_tags_23 },
    { 0x80d417d5, SUC_lexicon_tags_24 },
    { 0x875417d6, SUC_lexicon_tags_37 },
    { 0x805417d7, SUC_lexicon_tags_11 },
    { 0x8df417d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf417de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9417e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x591417e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb417e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997417ec, SUC_lexicon_tags_37 },
    { 0x65d417ed, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2b417ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed417f2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb61417f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23417f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5dd417fa, SUC_lexicon_tags_38 },
    { 0xc4d417fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105417ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21541802, SUC_lexicon_tags_9 },
    { 0x72941803, SUC_lexicon_tags_62 },
    { 0xddf41802, SUC_lexicon_tags_25 },
    { 0x4c941805, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4d741807, SUC_lexicon_tags_79 },
    { 0x63741807, SUC_lexicon_tags_4 },
    { 0x8b941808, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3f4180b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd41810, SUC_lexicon_tags_77 },
    { 0x1f41811, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15741813, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f41817, SUC_lexicon_tags_11 },
    { 0xba141818, SUC_lexicon_tags_11 },
    { 0x62741817, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c14181e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34341821, SUC_lexicon_tags_1 },
    { 0xe3d41821, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc141824, SUC_lexicon_tags_15 },
    { 0x5fb41825, SUC_lexicon_tags_37 },
    { 0xb2b41826, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb4182a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8774182d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b41830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b41837, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4934183b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87341841, SUC_lexicon_tags_151 },
    { 0x50541842, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1941845, SUC_lexicon_tags_15 },
    { 0x4ad41846, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x66741849, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd4184c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73741854, SUC_lexicon_tags_15 },
    { 0xd4f41855, SUC_lexicon_tags_47 },
    { 0xaf741856, SUC_lexicon_tags_23 },
    { 0xf1341857, SUC_lexicon_tags_0 },
    { 0x22f41858, SUC_lexicon_tags_10 },
    { 0x64d41857, SUC_lexicon_tags_32 },
    { 0xbdb4185a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d41862, SUC_lexicon_tags_17 },
    { 0x91f41863, SUC_lexicon_tags_60 },
    { 0x46541864, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced41868, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb854186a, SUC_lexicon_tags_87 },
    { 0x5b54186a, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29941871, SUC_lexicon_tags_65 },
    { 0xb741872, SUC_lexicon_tags_37 },
    { 0x5c941872, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d41876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb41882, SUC_lexicon_tags_26 },
    { 0xa141883, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b741887, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f4188a, SUC_lexicon_tags_12 },
    { 0x65d4188b, SUC_lexicon_tags_11 },
    { 0x5514188c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa314188f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9941893, SUC_lexicon_tags_37 },
    { 0x71341894, SUC_lexicon_tags_35 },
    { 0xe1741893, SUC_lexicon_tags_49 },
    { 0xd5941896, SUC_lexicon_tags_0 },
    { 0xcad41897, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3a741899, SUC_lexicon_tags_27 },
    { 0xaaf4189a, SUC_lexicon_tags_91 },
    { 0x2d74189a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c14189d, SUC_lexicon_tags_7 },
    { 0xb054189d, SUC_lexicon_tags_17 },
    { 0x7694189f, SUC_lexicon_tags_10 },
    { 0x885418a0, SUC_lexicon_tags_25 },
    { 0xe89418a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x923418a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f418a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b418ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919418b0, SUC_lexicon_tags_7 },
    { 0xff9418b1, SUC_lexicon_tags_25 },
    { 0x511418b2, SUC_lexicon_tags_65 },
    { 0x5fb418b3, SUC_lexicon_tags_7 },
    { 0x345418b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed418b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe13418ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x331418bc, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x315418be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef418c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9418c6, SUC_lexicon_tags_33 },
    { 0x9c9418c6, SUC_lexicon_tags_10 },
    { 0x73b418c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41418d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5b418d3, SUC_lexicon_tags_23 },
    { 0x4b5418d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b418d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2cd418db, SUC_lexicon_tags_7 },
    { 0x3cb418dc, SUC_lexicon_tags_6 },
    { 0xfe5418dd, SUC_lexicon_tags_7 },
    { 0x93f418db, SUC_lexicon_tags_19 },
    { 0x32b418df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdf7418e1, SUC_lexicon_tags_1 },
    { 0x419418e2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f418e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xff7418e7, SUC_lexicon_tags_15 },
    { 0xae9418e8, SUC_lexicon_tags_35 },
    { 0xd01418e9, SUC_lexicon_tags_17 },
    { 0x765418ea, SUC_lexicon_tags_43 },
    { 0x9f418e8, SUC_lexicon_tags_68 },
    { 0xbd7418eb, SUC_lexicon_tags_11 },
    { 0x6c1418ea, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe4d418ef, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x901418f1, SUC_lexicon_tags_1 },
    { 0x735418f2, SUC_lexicon_tags_21 },
    { 0x1c7418f2, SUC_lexicon_tags_15 },
    { 0x7b1418f4, SUC_lexicon_tags_7 },
    { 0x91f418f5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x845418f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f9418f9, SUC_lexicon_tags_7 },
    { 0xd89418f9, SUC_lexicon_tags_1 },
    { 0x837418fa, SUC_lexicon_tags_26 },
    { 0x415418fc, SUC_lexicon_tags_60 },
    { 0xb6b418fb, SUC_lexicon_tags_11 },
    { 0x85b418fe, SUC_lexicon_tags_32 },
    { 0x437418fe, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x44541901, SUC_lexicon_tags_25 },
    { 0x97941901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9141907, SUC_lexicon_tags_38 },
    { 0x16f41908, SUC_lexicon_tags_107 },
    { 0x5df41909, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7614190b, SUC_lexicon_tags_0 },
    { 0x20b4190c, SUC_lexicon_tags_15 },
    { 0xf094190d, SUC_lexicon_tags_25 },
    { 0x14d4190e, SUC_lexicon_tags_436 },
    { 0xe6f4190c, SUC_lexicon_tags_25 },
    { 0x5b54190e, SUC_lexicon_tags_35 },
    { 0xa8f4190e, SUC_lexicon_tags_18 },
    { 0xe7f41912, SUC_lexicon_tags_2 },
    { 0x66741913, SUC_lexicon_tags_25 },
    { 0x37941913, SUC_lexicon_tags_7 },
    { 0x14341912, SUC_lexicon_tags_4 },
    { 0x70f4190d, SUC_lexicon_tags_38 },
    { 0x19b41917, SUC_lexicon_tags_15 },
    { 0x2e141917, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd494191a, SUC_lexicon_tags_31 },
    { 0xac14191b, SUC_lexicon_tags_28 },
    { 0xc3b4191b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd94191f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3741922, SUC_lexicon_tags_26 },
    { 0x3f541923, SUC_lexicon_tags_60 },
    { 0x7e741924, SUC_lexicon_tags_2 },
    { 0xe8d41925, SUC_lexicon_tags_2 },
    { 0x95741926, SUC_lexicon_tags_32 },
    { 0xabb41925, SUC_lexicon_tags_27 },
    { 0xa7741928, SUC_lexicon_tags_15 },
    { 0x1d541929, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4034192c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6274192e, SUC_lexicon_tags_12 },
    { 0x1d94192f, SUC_lexicon_tags_45 },
    { 0x93941930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf141932, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb541937, SUC_lexicon_tags_11 },
    { 0x6f141937, SUC_lexicon_tags_17 },
    { 0xbf941939, SUC_lexicon_tags_27 },
    { 0x48941938, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8634193c, SUC_lexicon_tags_25 },
    { 0x2734193d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf41940, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d4194b, SUC_lexicon_tags_7 },
    { 0x7d14194c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7141950, SUC_lexicon_tags_27 },
    { 0xb5d41950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44541955, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b41957, SUC_lexicon_tags_25 },
    { 0xbbd41958, SUC_lexicon_tags_21 },
    { 0xa2941959, SUC_lexicon_tags_30 },
    { 0xb954195a, SUC_lexicon_tags_6 },
    { 0x5374195b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51d4195d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd41961, SUC_lexicon_tags_12 },
    { 0x62d41962, SUC_lexicon_tags_1234 },
    { 0, NULL },
    { 0x29b41964, SUC_lexicon_tags_7 },
    { 0xad941965, SUC_lexicon_tags_5 },
    { 0x9c341964, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xaa941968, SUC_lexicon_tags_17 },
    { 0xb5941969, SUC_lexicon_tags_29 },
    { 0x2d74196a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf514196c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99b4196e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49941970, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d41972, SUC_lexicon_tags_27 },
    { 0x64341973, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd41977, SUC_lexicon_tags_1 },
    { 0x47141977, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x79b4197a, SUC_lexicon_tags_1 },
    { 0x334197a, SUC_lexicon_tags_38 },
    { 0x2094197c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4af4197e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf341983, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8341987, SUC_lexicon_tags_45 },
    { 0xed741987, SUC_lexicon_tags_33 },
    { 0x2e941989, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7154198e, SUC_lexicon_tags_8 },
    { 0xd654198e, SUC_lexicon_tags_11 },
    { 0xfed41990, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8bb41992, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0d41994, SUC_lexicon_tags_0 },
    { 0xaf141994, SUC_lexicon_tags_37 },
    { 0x9a341996, SUC_lexicon_tags_264 },
    { 0x91941997, SUC_lexicon_tags_120 },
    { 0x4f941997, SUC_lexicon_tags_27 },
    { 0xb4141997, SUC_lexicon_tags_17 },
    { 0xa9d41999, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d54199d, SUC_lexicon_tags_80 },
    { 0x114199e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299419a3, SUC_lexicon_tags_25 },
    { 0xdc9419a4, SUC_lexicon_tags_31 },
    { 0x84d419a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff5419a7, SUC_lexicon_tags_25 },
    { 0x1e7419a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911419b3, SUC_lexicon_tags_32 },
    { 0x2ef419b4, SUC_lexicon_tags_17 },
    { 0x69f419b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd3419b7, SUC_lexicon_tags_37 },
    { 0xe3f419b8, SUC_lexicon_tags_25 },
    { 0xe5f419b7, SUC_lexicon_tags_16 },
    { 0x99f419b8, SUC_lexicon_tags_7 },
    { 0xd53419bb, SUC_lexicon_tags_234 },
    { 0x9b9419bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd93419be, SUC_lexicon_tags_16 },
    { 0x241419bf, SUC_lexicon_tags_27 },
    { 0x49b419c0, SUC_lexicon_tags_1 },
    { 0x651419c1, SUC_lexicon_tags_2 },
    { 0x401419c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1419c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9419c9, SUC_lexicon_tags_16 },
    { 0x93f419ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9419d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849419d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f419da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93419e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49419e5, SUC_lexicon_tags_79 },
    { 0xfe7419e6, SUC_lexicon_tags_23 },
    { 0xaf9419e6, SUC_lexicon_tags_55 },
    { 0xa65419e5, SUC_lexicon_tags_9 },
    { 0x427419e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a3419eb, SUC_lexicon_tags_163 },
    { 0xc79419ec, SUC_lexicon_tags_11 },
    { 0xfc9419ed, SUC_lexicon_tags_237 },
    { 0x7ab419ee, SUC_lexicon_tags_45 },
    { 0x8d3419ef, SUC_lexicon_tags_7 },
    { 0x597419ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f419f4, SUC_lexicon_tags_70 },
    { 0x3e1419f4, SUC_lexicon_tags_28 },
    { 0x1db419f6, SUC_lexicon_tags_17 },
    { 0xb2d419f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7419fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d5419fe, SUC_lexicon_tags_19 },
    { 0xbe7419ff, SUC_lexicon_tags_29 },
    { 0xe4d41a00, SUC_lexicon_tags_17 },
    { 0xa1741a01, SUC_lexicon_tags_7 },
    { 0x33341a00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1541a08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf41a0b, SUC_lexicon_tags_33 },
    { 0xfc341a0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe2b41a0e, SUC_lexicon_tags_56 },
    { 0x11741a0f, SUC_lexicon_tags_6 },
    { 0x10941a10, SUC_lexicon_tags_11 },
    { 0x12b41a10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64141a21, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb41a26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3941a2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff941a2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf41a31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12741a36, SUC_lexicon_tags_12 },
    { 0xcf341a36, SUC_lexicon_tags_27 },
    { 0xaf141a38, SUC_lexicon_tags_0 },
    { 0x30141a39, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb341a3c, SUC_lexicon_tags_82 },
    { 0xb8b41a3d, SUC_lexicon_tags_25 },
    { 0x37741a3d, SUC_lexicon_tags_7 },
    { 0x8f541a3e, SUC_lexicon_tags_1 },
    { 0x10141a3f, SUC_lexicon_tags_12 },
    { 0x37d41a41, SUC_lexicon_tags_2 },
    { 0xd7d41a41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7741a46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17741a4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e541a53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae541a55, SUC_lexicon_tags_49 },
    { 0x33541a56, SUC_lexicon_tags_28 },
    { 0x5d941a57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a941a5c, SUC_lexicon_tags_37 },
    { 0xda741a5d, SUC_lexicon_tags_25 },
    { 0xe3741a5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb41a61, SUC_lexicon_tags_8 },
    { 0x47141a62, SUC_lexicon_tags_15 },
    { 0x51741a61, SUC_lexicon_tags_25 },
    { 0xebb41a64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b41a67, SUC_lexicon_tags_12 },
    { 0xef341a67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63141a6a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x14741a6d, SUC_lexicon_tags_23 },
    { 0x1c141a6e, SUC_lexicon_tags_29 },
    { 0xce141a6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b41a72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8141a74, SUC_lexicon_tags_34 },
    { 0x1c341a75, SUC_lexicon_tags_25 },
    { 0x3fd41a76, SUC_lexicon_tags_19 },
    { 0x30d41a77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff41a7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b41a7f, SUC_lexicon_tags_25 },
    { 0x53f41a80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a741a83, SUC_lexicon_tags_25 },
    { 0x14d41a84, SUC_lexicon_tags_11 },
    { 0x2cb41a85, SUC_lexicon_tags_14 },
    { 0x9ff41a86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d41a89, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbf41a8b, SUC_lexicon_tags_1 },
    { 0x27741a8b, SUC_lexicon_tags_17 },
    { 0x40141a8d, SUC_lexicon_tags_1 },
    { 0xfc541a8e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2cd41a90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4341a95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0141a97, SUC_lexicon_tags_17 },
    { 0xc0741a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12741a9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cd41a9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7d41a9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0b41aa1, SUC_lexicon_tags_7 },
    { 0xee141aa1, SUC_lexicon_tags_33 },
    { 0x23d41aa1, SUC_lexicon_tags_4 },
    { 0xe9341aa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7df41aa6, SUC_lexicon_tags_14 },
    { 0xc3541aa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6541aab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b41ab0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95341ab4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb41ab7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f341ac4, SUC_lexicon_tags_26 },
    { 0xca141ac5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd741ac8, SUC_lexicon_tags_25 },
    { 0xed541ac9, SUC_lexicon_tags_7 },
    { 0x2e941ac8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91141ad3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e141ad6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xccd41ad8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x26b41ada, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab541ade, SUC_lexicon_tags_11 },
    { 0x23941adf, SUC_lexicon_tags_17 },
    { 0x39d41adf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63941ae5, SUC_lexicon_tags_7 },
    { 0x72141ae6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe3341ae8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x83141aeb, SUC_lexicon_tags_10 },
    { 0x81b41aeb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b41af0, SUC_lexicon_tags_3 },
    { 0x33d41af1, SUC_lexicon_tags_17 },
    { 0xc5d41af2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b341af4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d41af7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x37541afa, SUC_lexicon_tags_38 },
    { 0x9bb41afb, SUC_lexicon_tags_7 },
    { 0x76541afb, SUC_lexicon_tags_10 },
    { 0x8d341afd, SUC_lexicon_tags_11 },
    { 0x41341afe, SUC_lexicon_tags_55 },
    { 0xeb341afd, SUC_lexicon_tags_63 },
    { 0x42341b00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35741b04, SUC_lexicon_tags_49 },
    { 0x5e741b05, SUC_lexicon_tags_10 },
    { 0x45941b06, SUC_lexicon_tags_15 },
    { 0x76d41b07, SUC_lexicon_tags_25 },
    { 0x35541b08, SUC_lexicon_tags_8 },
    { 0x10941b06, SUC_lexicon_tags_2 },
    { 0x1c741b07, SUC_lexicon_tags_7 },
    { 0x68941b0b, SUC_lexicon_tags_31 },
    { 0x16741b0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf3341b0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35141b10, SUC_lexicon_tags_74 },
    { 0x99f41b11, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdd941b13, SUC_lexicon_tags_7 },
    { 0x88341b14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68541b1a, SUC_lexicon_tags_27 },
    { 0xd1f41b1b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x43741b1d, SUC_lexicon_tags_17 },
    { 0xd5541b1e, SUC_lexicon_tags_25 },
    { 0xdd741b1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b41b25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d41b2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2941b31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaaf41b33, SUC_lexicon_tags_11 },
    { 0x74f41b34, SUC_lexicon_tags_15 },
    { 0xa0f41b35, SUC_lexicon_tags_29 },
    { 0xf7941b36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53541b39, SUC_lexicon_tags_23 },
    { 0x7d941b3a, SUC_lexicon_tags_11 },
    { 0x1ff41b3b, SUC_lexicon_tags_47 },
    { 0xbfb41b39, SUC_lexicon_tags_1 },
    { 0x69941b3d, SUC_lexicon_tags_24 },
    { 0xeb741b3d, SUC_lexicon_tags_37 },
    { 0x52741b3f, SUC_lexicon_tags_27 },
    { 0xbe941b40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe1341b42, SUC_lexicon_tags_31 },
    { 0xfbb41b43, SUC_lexicon_tags_63 },
    { 0x79341b42, SUC_lexicon_tags_66 },
    { 0xabf41b44, SUC_lexicon_tags_39 },
    { 0x19541b46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa141b4a, SUC_lexicon_tags_23 },
    { 0x5cf41b4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f41b4e, SUC_lexicon_tags_1 },
    { 0x13141b4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83541b51, SUC_lexicon_tags_37 },
    { 0x14141b52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3341b56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x36741b59, SUC_lexicon_tags_98 },
    { 0x93941b5a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xde341b5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28541b62, SUC_lexicon_tags_31 },
    { 0x1a541b62, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1b741b65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x16d41b67, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf41b6b, SUC_lexicon_tags_11 },
    { 0xe0741b6c, SUC_lexicon_tags_23 },
    { 0x6a941b6d, SUC_lexicon_tags_37 },
    { 0x5ed41b6e, SUC_lexicon_tags_57 },
    { 0xa6341b6f, SUC_lexicon_tags_15 },
    { 0xbeb41b6f, SUC_lexicon_tags_25 },
    { 0xa2541b6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79141b73, SUC_lexicon_tags_11 },
    { 0xb7b41b74, SUC_lexicon_tags_11 },
    { 0xa7941b75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd41b7a, SUC_lexicon_tags_12 },
    { 0x65741b7b, SUC_lexicon_tags_87 },
    { 0x5d341b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a541b7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd541b83, SUC_lexicon_tags_15 },
    { 0x69941b84, SUC_lexicon_tags_7 },
    { 0x6dd41b85, SUC_lexicon_tags_30 },
    { 0x18b41b84, SUC_lexicon_tags_0 },
    { 0x81941b87, SUC_lexicon_tags_44 },
    { 0x8d141b84, SUC_lexicon_tags_33 },
    { 0x7d41b89, SUC_lexicon_tags_48 },
    { 0xe2b41b8a, SUC_lexicon_tags_17 },
    { 0x94b41b8b, SUC_lexicon_tags_10 },
    { 0xad141b8c, SUC_lexicon_tags_21 },
    { 0x9f41b8d, SUC_lexicon_tags_24 },
    { 0x4a541b8e, SUC_lexicon_tags_1 },
    { 0xe1741b8d, SUC_lexicon_tags_56 },
    { 0xec741b90, SUC_lexicon_tags_21 },
    { 0x18141b8f, SUC_lexicon_tags_7 },
    { 0x65d41b91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f41b95, SUC_lexicon_tags_15 },
    { 0xc4141b95, SUC_lexicon_tags_60 },
    { 0xbaf41b96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e941b99, SUC_lexicon_tags_25 },
    { 0x65d41b99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb41b9d, SUC_lexicon_tags_14 },
    { 0x5fd41b9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32941ba2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94141ba5, SUC_lexicon_tags_14 },
    { 0x5e541ba5, SUC_lexicon_tags_25 },
    { 0x13d41ba7, SUC_lexicon_tags_25 },
    { 0x37b41ba8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64541bb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2141bb3, SUC_lexicon_tags_31 },
    { 0x56b41bb4, SUC_lexicon_tags_16 },
    { 0x45741bb4, SUC_lexicon_tags_60 },
    { 0x3b741bb5, SUC_lexicon_tags_25 },
    { 0x96741bb7, SUC_lexicon_tags_45 },
    { 0xbdf41bb6, SUC_lexicon_tags_37 },
    { 0x32d41bb7, SUC_lexicon_tags_31 },
    { 0x9a341bb5, SUC_lexicon_tags_29 },
    { 0xaf941bba, SUC_lexicon_tags_45 },
    { 0xd5d41bb4, SUC_lexicon_tags_1 },
    { 0x8f341bb7, SUC_lexicon_tags_49 },
    { 0xc3341bbe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae541bc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f41bc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd41bc6, SUC_lexicon_tags_49 },
    { 0x7dd41bc7, SUC_lexicon_tags_27 },
    { 0xaeb41bc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x15141bcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e541bcd, SUC_lexicon_tags_1 },
    { 0xceb41bce, SUC_lexicon_tags_29 },
    { 0x56d41bcd, SUC_lexicon_tags_28 },
    { 0x34b41bd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f341bd2, SUC_lexicon_tags_14 },
    { 0xfd41bd3, SUC_lexicon_tags_39 },
    { 0x5e141bd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x67541bd7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b41bda, SUC_lexicon_tags_25 },
    { 0x45141bda, SUC_lexicon_tags_32 },
    { 0xd1341bdc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b41bdf, SUC_lexicon_tags_49 },
    { 0x91341be0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f41be3, SUC_lexicon_tags_65 },
    { 0xc141be4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5741be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86141bec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27f41bee, SUC_lexicon_tags_37 },
    { 0xfaf41bef, SUC_lexicon_tags_2 },
    { 0xb8341bef, SUC_lexicon_tags_1 },
    { 0x21f41bf1, SUC_lexicon_tags_14 },
    { 0xdc341bf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53941bfe, SUC_lexicon_tags_19 },
    { 0xda341bff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c341c05, SUC_lexicon_tags_15 },
    { 0x40f41c06, SUC_lexicon_tags_33 },
    { 0xf0541c06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d741c0c, SUC_lexicon_tags_15 },
    { 0xcb941c0d, SUC_lexicon_tags_19 },
    { 0xcb141c0e, SUC_lexicon_tags_39 },
    { 0x88941c0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35541c15, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xec141c18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb41c1a, SUC_lexicon_tags_37 },
    { 0xcd941c1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe3541c1d, SUC_lexicon_tags_29 },
    { 0x45741c1e, SUC_lexicon_tags_4 },
    { 0xaa941c1f, SUC_lexicon_tags_56 },
    { 0xc3b41c1e, SUC_lexicon_tags_49 },
    { 0x1341c20, SUC_lexicon_tags_12 },
    { 0x341c1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f41c25, SUC_lexicon_tags_9 },
    { 0x3ad41c26, SUC_lexicon_tags_11 },
    { 0xef41c26, SUC_lexicon_tags_179 },
    { 0xa9741c28, SUC_lexicon_tags_7 },
    { 0x73541c25, SUC_lexicon_tags_25 },
    { 0xeb541c26, SUC_lexicon_tags_21 },
    { 0xf9141c2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b41c2e, SUC_lexicon_tags_12 },
    { 0xe5541c2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38941c32, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xb0b41c34, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x62541c36, SUC_lexicon_tags_37 },
    { 0x741c36, SUC_lexicon_tags_65 },
    { 0xfd741c36, SUC_lexicon_tags_60 },
    { 0x48341c38, SUC_lexicon_tags_37 },
    { 0x80541c3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7941c3e, SUC_lexicon_tags_7 },
    { 0xc9b41c3f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x16141c42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce141c44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c941c4d, SUC_lexicon_tags_17 },
    { 0x9541c4d, SUC_lexicon_tags_1 },
    { 0x30941c4d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb41c52, SUC_lexicon_tags_45 },
    { 0xbb41c53, SUC_lexicon_tags_26 },
    { 0x24141c53, SUC_lexicon_tags_70 },
    { 0x15941c55, SUC_lexicon_tags_23 },
    { 0x81941c56, SUC_lexicon_tags_0 },
    { 0xe0341c57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f41c5b, SUC_lexicon_tags_11 },
    { 0x24741c5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3741c5e, SUC_lexicon_tags_10 },
    { 0x57f41c5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b41c62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40141c67, SUC_lexicon_tags_63 },
    { 0xd4941c67, SUC_lexicon_tags_45 },
    { 0x8c941c69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef41c6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d141c72, SUC_lexicon_tags_17 },
    { 0x61941c73, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x18b41c75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5541c78, SUC_lexicon_tags_1624 },
    { 0xdad41c78, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7141c7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3341c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd41c86, SUC_lexicon_tags_17 },
    { 0x6341c87, SUC_lexicon_tags_19 },
    { 0x27f41c88, SUC_lexicon_tags_14 },
    { 0x91341c88, SUC_lexicon_tags_34 },
    { 0xf8141c88, SUC_lexicon_tags_25 },
    { 0x24b41c8b, SUC_lexicon_tags_17 },
    { 0xd8b41c88, SUC_lexicon_tags_15 },
    { 0xe3d41c8d, SUC_lexicon_tags_12 },
    { 0xd1941c8e, SUC_lexicon_tags_12 },
    { 0xdff41c87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f541c94, SUC_lexicon_tags_46 },
    { 0x3c341c94, SUC_lexicon_tags_19 },
    { 0x83d41c95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3b41c98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2a141c9a, SUC_lexicon_tags_26 },
    { 0x19741c9b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db41c9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d41ca5, SUC_lexicon_tags_38 },
    { 0x26341ca5, SUC_lexicon_tags_7 },
    { 0x14741ca7, SUC_lexicon_tags_12 },
    { 0xa3941ca7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a341caa, SUC_lexicon_tags_17 },
    { 0x18341caa, SUC_lexicon_tags_27 },
    { 0xf2941cac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a741cb6, SUC_lexicon_tags_26 },
    { 0xdd141cb6, SUC_lexicon_tags_37 },
    { 0xb2f41cb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1141cc0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b41cc4, SUC_lexicon_tags_35 },
    { 0xd5541cc5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd4f41cc7, SUC_lexicon_tags_4 },
    { 0x68741cc7, SUC_lexicon_tags_15 },
    { 0x5bb41cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62741ccc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1341cd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f41cda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f41cdd, SUC_lexicon_tags_45 },
    { 0x3d741cde, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9741ce7, SUC_lexicon_tags_106 },
    { 0x32f41ce8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d41ced, SUC_lexicon_tags_2 },
    { 0x7bb41cee, SUC_lexicon_tags_28 },
    { 0xd9141cee, SUC_lexicon_tags_28 },
    { 0xe6f41cf0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2bf41cf2, SUC_lexicon_tags_3 },
    { 0x8dd41cf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b41cf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3141cfb, SUC_lexicon_tags_2 },
    { 0x9b741cfb, SUC_lexicon_tags_19 },
    { 0x21741cfd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b41d06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x64541d08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5741d0b, SUC_lexicon_tags_32 },
    { 0xc941d0c, SUC_lexicon_tags_7 },
    { 0x6f941d0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fd41d0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e341d14, SUC_lexicon_tags_60 },
    { 0x5741d15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b41d18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa7741d1a, SUC_lexicon_tags_37 },
    { 0xcc941d1b, SUC_lexicon_tags_15 },
    { 0xa1b41d1c, SUC_lexicon_tags_48 },
    { 0x28341d1d, SUC_lexicon_tags_12 },
    { 0xc7141d1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0341d20, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf541d23, SUC_lexicon_tags_15 },
    { 0x13941d24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f541d26, SUC_lexicon_tags_62 },
    { 0x47341d27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f41d2e, SUC_lexicon_tags_168 },
    { 0x53741d2e, SUC_lexicon_tags_31 },
    { 0xc6141d30, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b41d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb0b41d38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8341d3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60541d3e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95941d46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41d49, SUC_lexicon_tags_7 },
    { 0x12341d4a, SUC_lexicon_tags_25 },
    { 0x1a941d4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8941d51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x92941d54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9941d57, SUC_lexicon_tags_47 },
    { 0x42341d58, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x42341d5a, SUC_lexicon_tags_17 },
    { 0x63f41d5a, SUC_lexicon_tags_7 },
    { 0x141d5a, SUC_lexicon_tags_11 },
    { 0xe9d41d5d, SUC_lexicon_tags_48 },
    { 0xa5d41d5d, SUC_lexicon_tags_10 },
    { 0xfc741d5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d41d69, SUC_lexicon_tags_19 },
    { 0x69f41d69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f41d6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21d41d6f, SUC_lexicon_tags_4 },
    { 0x13d41d70, SUC_lexicon_tags_17 },
    { 0xaad41d71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79541d78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf6541d7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ad41d7c, SUC_lexicon_tags_107 },
    { 0x9a341d7d, SUC_lexicon_tags_38 },
    { 0xec141d7d, SUC_lexicon_tags_80 },
    { 0x52141d7f, SUC_lexicon_tags_19 },
    { 0xb1741d80, SUC_lexicon_tags_0 },
    { 0x1dd41d81, SUC_lexicon_tags_17 },
    { 0x19741d7c, SUC_lexicon_tags_7 },
    { 0xc7141d83, SUC_lexicon_tags_29 },
    { 0x58741d83, SUC_lexicon_tags_66 },
    { 0xa9b41d85, SUC_lexicon_tags_55 },
    { 0xa8541d85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63941d8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21541d91, SUC_lexicon_tags_8 },
    { 0xd5341d91, SUC_lexicon_tags_23 },
    { 0x60341d93, SUC_lexicon_tags_53 },
    { 0xc0741d94, SUC_lexicon_tags_2 },
    { 0xfcd41d95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f141d98, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf41d9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed741da5, SUC_lexicon_tags_23 },
    { 0x24141da6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef41dac, SUC_lexicon_tags_46 },
    { 0x66d41dad, SUC_lexicon_tags_17 },
    { 0xe5b41dae, SUC_lexicon_tags_31 },
    { 0x85341daf, SUC_lexicon_tags_33 },
    { 0x35f41db0, SUC_lexicon_tags_46 },
    { 0xd5d41db1, SUC_lexicon_tags_27 },
    { 0xe8f41dad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f41db7, SUC_lexicon_tags_1 },
    { 0x50341db8, SUC_lexicon_tags_7 },
    { 0x34341db7, SUC_lexicon_tags_13 },
    { 0xdf741dba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77341dc7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99f41dc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b41dcc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25141dd4, SUC_lexicon_tags_53 },
    { 0xfb541dd4, SUC_lexicon_tags_43 },
    { 0x57541dd5, SUC_lexicon_tags_33 },
    { 0x7a741dd7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d141dde, SUC_lexicon_tags_7 },
    { 0x8b341dde, SUC_lexicon_tags_11 },
    { 0x7bf41de0, SUC_lexicon_tags_14 },
    { 0xb4741de1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe941de3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e341deb, SUC_lexicon_tags_63 },
    { 0xbf741dec, SUC_lexicon_tags_13 },
    { 0xe0541ded, SUC_lexicon_tags_49 },
    { 0x76941ded, SUC_lexicon_tags_47 },
    { 0x9141def, SUC_lexicon_tags_2 },
    { 0x42d41df0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdd541df2, SUC_lexicon_tags_79 },
    { 0xf3b41df3, SUC_lexicon_tags_15 },
    { 0xc5741df4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf41df7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1a941df9, SUC_lexicon_tags_27 },
    { 0x6ad41dfa, SUC_lexicon_tags_14 },
    { 0x53f41df9, SUC_lexicon_tags_15 },
    { 0x5ef41dfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd41e01, SUC_lexicon_tags_4 },
    { 0xff541e01, SUC_lexicon_tags_15 },
    { 0x2a341e03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce741e06, SUC_lexicon_tags_37 },
    { 0x49341e06, SUC_lexicon_tags_15 },
    { 0x5c541e07, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc5d41e0a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x77141e0d, SUC_lexicon_tags_63 },
    { 0x56741e0d, SUC_lexicon_tags_25 },
    { 0xea141e0d, SUC_lexicon_tags_7 },
    { 0x82b41e0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67341e14, SUC_lexicon_tags_2 },
    { 0x7341e15, SUC_lexicon_tags_10 },
    { 0x62941e16, SUC_lexicon_tags_25 },
    { 0xc9541e17, SUC_lexicon_tags_2 },
    { 0x3e141e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39d41e1a, SUC_lexicon_tags_27 },
    { 0x24741e1a, SUC_lexicon_tags_12 },
    { 0x75d41e1b, SUC_lexicon_tags_15 },
    { 0x61f41e1b, SUC_lexicon_tags_11 },
    { 0xec741e1e, SUC_lexicon_tags_37 },
    { 0x6d141e1f, SUC_lexicon_tags_17 },
    { 0xbbd41e1f, SUC_lexicon_tags_1 },
    { 0xe7b41e1d, SUC_lexicon_tags_30 },
    { 0xeb541e22, SUC_lexicon_tags_11 },
    { 0xacb41e1e, SUC_lexicon_tags_10 },
    { 0xd5d41e24, SUC_lexicon_tags_30 },
    { 0x63141e25, SUC_lexicon_tags_37 },
    { 0xdc941e21, SUC_lexicon_tags_62 },
    { 0x65941e27, SUC_lexicon_tags_56 },
    { 0x39141e28, SUC_lexicon_tags_17 },
    { 0x55741e29, SUC_lexicon_tags_37 },
    { 0x8df41e2a, SUC_lexicon_tags_15 },
    { 0xda341e22, SUC_lexicon_tags_31 },
    { 0x3741e23, SUC_lexicon_tags_31 },
    { 0xe7f41e2d, SUC_lexicon_tags_91 },
    { 0x71941e23, SUC_lexicon_tags_25 },
    { 0x79541e2f, SUC_lexicon_tags_11 },
    { 0xa6b41e30, SUC_lexicon_tags_37 },
    { 0x9d941e30, SUC_lexicon_tags_77 },
    { 0x98341e30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x75f41e34, SUC_lexicon_tags_32 },
    { 0x9db41e35, SUC_lexicon_tags_37 },
    { 0x87341e34, SUC_lexicon_tags_7 },
    { 0xc7f41e34, SUC_lexicon_tags_1 },
    { 0xe2341e34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde141e3c, SUC_lexicon_tags_12 },
    { 0xe1741e3c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x13b41e3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36341e41, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xcf141e43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f41e46, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb41e4e, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x5e541e50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee541e53, SUC_lexicon_tags_7 },
    { 0x91341e53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35741e58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48341e5a, SUC_lexicon_tags_15 },
    { 0x5d341e5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee141e60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d41e65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf41e69, SUC_lexicon_tags_37 },
    { 0xb4741e69, SUC_lexicon_tags_16 },
    { 0xaa541e6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85741e6d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40141e6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58341e73, SUC_lexicon_tags_25 },
    { 0xb9f41e73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d41e78, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee341e7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9ff41e7e, SUC_lexicon_tags_43 },
    { 0x92d41e7f, SUC_lexicon_tags_17 },
    { 0xdf541e7e, SUC_lexicon_tags_7 },
    { 0x17541e81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf41e86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8941e8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55541e90, SUC_lexicon_tags_21 },
    { 0xc8141e90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x92f41e93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d41e96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78541e9e, SUC_lexicon_tags_2 },
    { 0x27341e9f, SUC_lexicon_tags_2 },
    { 0xeef41e9f, SUC_lexicon_tags_47 },
    { 0x1d741ea1, SUC_lexicon_tags_11 },
    { 0xf1941e9e, SUC_lexicon_tags_23 },
    { 0xf1d41ea1, SUC_lexicon_tags_10 },
    { 0x7941ea4, SUC_lexicon_tags_31 },
    { 0xac941ea5, SUC_lexicon_tags_9 },
    { 0x25541ea5, SUC_lexicon_tags_1 },
    { 0x98b41ea7, SUC_lexicon_tags_2 },
    { 0x4ed41ea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f741eaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c541eaf, SUC_lexicon_tags_43 },
    { 0x7b341eb0, SUC_lexicon_tags_26 },
    { 0x8e741eb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94941eb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc741eb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d41ebb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x59141ebd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb941ec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39b41ec4, SUC_lexicon_tags_39 },
    { 0xbff41ec5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x85741ec8, SUC_lexicon_tags_15 },
    { 0x91141ec9, SUC_lexicon_tags_47 },
    { 0x98b41eca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8741ece, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b41ed1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5141ed4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9341ed9, SUC_lexicon_tags_68 },
    { 0xf1541ed9, SUC_lexicon_tags_38 },
    { 0xf8341ed9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad941ee0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e941ee5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd7541ee7, SUC_lexicon_tags_46 },
    { 0x4b341ee8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54d41eea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43341eee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34941ef0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45741ef5, SUC_lexicon_tags_2 },
    { 0x38141ef6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65141efb, SUC_lexicon_tags_68 },
    { 0x38341efc, SUC_lexicon_tags_12 },
    { 0xcf41efb, SUC_lexicon_tags_17 },
    { 0x89b41efc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60941f04, SUC_lexicon_tags_390 },
    { 0x70d41f04, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b41f08, SUC_lexicon_tags_12 },
    { 0xd1d41f08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d41f0d, SUC_lexicon_tags_25 },
    { 0x38541f0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1541f1a, SUC_lexicon_tags_14 },
    { 0x78f41f1a, SUC_lexicon_tags_14 },
    { 0x43f41f1c, SUC_lexicon_tags_11 },
    { 0x61f41f1a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3141f21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b41f2f, SUC_lexicon_tags_25 },
    { 0x8b941f30, SUC_lexicon_tags_7 },
    { 0x4b941f31, SUC_lexicon_tags_17 },
    { 0x4d941f32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x90341f34, SUC_lexicon_tags_12 },
    { 0x97941f35, SUC_lexicon_tags_15 },
    { 0xbef41f35, SUC_lexicon_tags_1 },
    { 0x50541f37, SUC_lexicon_tags_47 },
    { 0x56d41f37, SUC_lexicon_tags_39 },
    { 0x12341f36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a141f3b, SUC_lexicon_tags_1 },
    { 0xb9f41f3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x52141f3f, SUC_lexicon_tags_19 },
    { 0xda941f3f, SUC_lexicon_tags_7 },
    { 0x2cf41f40, SUC_lexicon_tags_15 },
    { 0x8341f40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f41f47, SUC_lexicon_tags_47 },
    { 0x14341f48, SUC_lexicon_tags_54 },
    { 0xb9541f49, SUC_lexicon_tags_23 },
    { 0xa0d41f49, SUC_lexicon_tags_9 },
    { 0xdb541f4b, SUC_lexicon_tags_81 },
    { 0x8c941f4b, SUC_lexicon_tags_60 },
    { 0x7b341f4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7741f4f, SUC_lexicon_tags_48 },
    { 0x30341f4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b41f52, SUC_lexicon_tags_12 },
    { 0xf2141f52, SUC_lexicon_tags_7 },
    { 0x18741f52, SUC_lexicon_tags_33 },
    { 0xb5341f55, SUC_lexicon_tags_19 },
    { 0x93541f56, SUC_lexicon_tags_14 },
    { 0xee141f55, SUC_lexicon_tags_49 },
    { 0xd9d41f58, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x68541f5b, SUC_lexicon_tags_8 },
    { 0x1ed41f5c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37741f60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b41f67, SUC_lexicon_tags_65 },
    { 0x46541f68, SUC_lexicon_tags_49 },
    { 0xe9741f69, SUC_lexicon_tags_60 },
    { 0xfe141f6a, SUC_lexicon_tags_37 },
    { 0xf7f41f6b, SUC_lexicon_tags_39 },
    { 0xed541f68, SUC_lexicon_tags_25 },
    { 0xd9541f69, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b41f74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1541f77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63141f7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b41f7e, SUC_lexicon_tags_14 },
    { 0x2f41f7f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd41f85, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2c141f87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1c941f89, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x73d41f8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f741f8e, SUC_lexicon_tags_84 },
    { 0xa5941f8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62341f93, SUC_lexicon_tags_37 },
    { 0xb5741f94, SUC_lexicon_tags_11 },
    { 0x28741f95, SUC_lexicon_tags_13 },
    { 0xd5941f95, SUC_lexicon_tags_124 },
    { 0x30341f97, SUC_lexicon_tags_1 },
    { 0x38f41f97, SUC_lexicon_tags_15 },
    { 0xa6f41f99, SUC_lexicon_tags_17 },
    { 0xdb541f97, SUC_lexicon_tags_41 },
    { 0xbfb41f9b, SUC_lexicon_tags_49 },
    { 0xe0d41f9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ad41f9e, SUC_lexicon_tags_37 },
    { 0x58541f9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b941fa5, SUC_lexicon_tags_7 },
    { 0x9c341fa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecf41fa8, SUC_lexicon_tags_37 },
    { 0x9e541fa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0541fab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0941fb2, SUC_lexicon_tags_116 },
    { 0x30141fb3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x27741fb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4941fb8, SUC_lexicon_tags_15 },
    { 0x35b41fb9, SUC_lexicon_tags_7 },
    { 0x5d41fba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d141fbd, SUC_lexicon_tags_25 },
    { 0x3ed41fbe, SUC_lexicon_tags_0 },
    { 0x6b541fbf, SUC_lexicon_tags_32 },
    { 0x97741fc0, SUC_lexicon_tags_63 },
    { 0x8c941fbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde341fc4, SUC_lexicon_tags_25 },
    { 0x50b41fc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f41fd0, SUC_lexicon_tags_27 },
    { 0xe6f41fd1, SUC_lexicon_tags_38 },
    { 0x90d41fd2, SUC_lexicon_tags_17 },
    { 0xabf41fd0, SUC_lexicon_tags_25 },
    { 0x3af41fd1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbdd41fd6, SUC_lexicon_tags_11 },
    { 0xa1941fd7, SUC_lexicon_tags_16 },
    { 0xcbd41fd7, SUC_lexicon_tags_1 },
    { 0xd41fd9, SUC_lexicon_tags_37 },
    { 0xb8341fd8, SUC_lexicon_tags_37 },
    { 0x8141fdb, SUC_lexicon_tags_39 },
    { 0x20941fda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5941fe1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b41fe4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf41fec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca141ff0, SUC_lexicon_tags_17 },
    { 0xb5141ff1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed41ff4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d41ff8, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x21741ffa, SUC_lexicon_tags_49 },
    { 0xebb41ffa, SUC_lexicon_tags_2 },
    { 0x7f141ffc, SUC_lexicon_tags_3 },
    { 0xccf41ffb, SUC_lexicon_tags_25 },
    { 0x29141ffe, SUC_lexicon_tags_49 },
    { 0x7341ffa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1a142001, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0b42003, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa094200b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cf4200d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa494200f, SUC_lexicon_tags_47 },
    { 0xbdd42010, SUC_lexicon_tags_57 },
    { 0xd5b42010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9542013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7342015, SUC_lexicon_tags_8 },
    { 0x36942015, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff42019, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe994201b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f4201f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d42025, SUC_lexicon_tags_37 },
    { 0x9342026, SUC_lexicon_tags_29 },
    { 0x5d342025, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8f942029, SUC_lexicon_tags_1 },
    { 0x68d4202a, SUC_lexicon_tags_37 },
    { 0x9154202b, SUC_lexicon_tags_38 },
    { 0xbd34202b, SUC_lexicon_tags_23 },
    { 0xf0f4202c, SUC_lexicon_tags_32 },
    { 0x9af4202d, SUC_lexicon_tags_47 },
    { 0x8634202f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab42033, SUC_lexicon_tags_11 },
    { 0x5af42034, SUC_lexicon_tags_216 },
    { 0xd5d42035, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8942038, SUC_lexicon_tags_15 },
    { 0x46342038, SUC_lexicon_tags_15 },
    { 0x2a542038, SUC_lexicon_tags_1 },
    { 0x2094203b, SUC_lexicon_tags_2 },
    { 0xbdf4203c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa574203f, SUC_lexicon_tags_1 },
    { 0x6d942040, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdcd42042, SUC_lexicon_tags_9 },
    { 0x9f942042, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f42046, SUC_lexicon_tags_7 },
    { 0x1a342047, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7342049, SUC_lexicon_tags_2 },
    { 0xc074204a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9954204f, SUC_lexicon_tags_37 },
    { 0x3d342050, SUC_lexicon_tags_43 },
    { 0x9ed4204f, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x342056, SUC_lexicon_tags_46 },
    { 0xd2d42056, SUC_lexicon_tags_25 },
    { 0x53542058, SUC_lexicon_tags_30 },
    { 0xe9b42059, SUC_lexicon_tags_170 },
    { 0xddb4205a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f4205e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b42061, SUC_lexicon_tags_33 },
    { 0x3f142061, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22342065, SUC_lexicon_tags_10 },
    { 0x2af42066, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4d42068, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62b4206a, SUC_lexicon_tags_19 },
    { 0x8db4206b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5174206d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a342075, SUC_lexicon_tags_12 },
    { 0x8ab42075, SUC_lexicon_tags_15 },
    { 0xbd942077, SUC_lexicon_tags_65 },
    { 0x32942078, SUC_lexicon_tags_10 },
    { 0xf3542078, SUC_lexicon_tags_1 },
    { 0xa0742079, SUC_lexicon_tags_3 },
    { 0x7a542078, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0d4207d, SUC_lexicon_tags_66 },
    { 0xfe54207e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5414208d, SUC_lexicon_tags_15 },
    { 0xec74208e, SUC_lexicon_tags_17 },
    { 0xa6d4208f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab42094, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaaf42096, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26542098, SUC_lexicon_tags_43 },
    { 0xdfb42098, SUC_lexicon_tags_28 },
    { 0xf6f4209a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c54209c, SUC_lexicon_tags_38 },
    { 0x72d4209d, SUC_lexicon_tags_26 },
    { 0xb394209d, SUC_lexicon_tags_2 },
    { 0xb974209f, SUC_lexicon_tags_37 },
    { 0x7114209d, SUC_lexicon_tags_14 },
    { 0x2d34209f, SUC_lexicon_tags_25 },
    { 0x945420a2, SUC_lexicon_tags_0 },
    { 0xcc3420a3, SUC_lexicon_tags_55 },
    { 0x307420a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d420ae, SUC_lexicon_tags_1 },
    { 0xddb420ae, SUC_lexicon_tags_25 },
    { 0x2d420ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95420b5, SUC_lexicon_tags_15 },
    { 0x855420b6, SUC_lexicon_tags_10 },
    { 0xb1d420b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11d420b9, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x483420bb, SUC_lexicon_tags_19 },
    { 0x7e9420bc, SUC_lexicon_tags_61 },
    { 0xa1d420bd, SUC_lexicon_tags_1160 },
    { 0xfd420be, SUC_lexicon_tags_47 },
    { 0x951420bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e9420c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f420c6, SUC_lexicon_tags_25 },
    { 0xf4d420c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b420ca, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f420d0, SUC_lexicon_tags_12 },
    { 0xa4f420d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x37d420d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f420da, SUC_lexicon_tags_56 },
    { 0xed9420db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d420df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x575420e2, SUC_lexicon_tags_1 },
    { 0x57420e3, SUC_lexicon_tags_15 },
    { 0xd33420e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb420ea, SUC_lexicon_tags_60 },
    { 0xc21420eb, SUC_lexicon_tags_14 },
    { 0x3420ea, SUC_lexicon_tags_12 },
    { 0x91b420ed, SUC_lexicon_tags_9 },
    { 0x809420ee, SUC_lexicon_tags_25 },
    { 0x9a7420ed, SUC_lexicon_tags_27 },
    { 0x5b420f0, SUC_lexicon_tags_4 },
    { 0xf53420f1, SUC_lexicon_tags_46 },
    { 0xbb420f2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2d5420f4, SUC_lexicon_tags_63 },
    { 0xf6f420f4, SUC_lexicon_tags_25 },
    { 0x97420f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d420fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf97420ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x45142101, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76542106, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb54210a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb474210c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3054210e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d42111, SUC_lexicon_tags_12 },
    { 0xccd42111, SUC_lexicon_tags_38 },
    { 0x49542112, SUC_lexicon_tags_37 },
    { 0x3e342114, SUC_lexicon_tags_54 },
    { 0x11342115, SUC_lexicon_tags_14 },
    { 0x1e142115, SUC_lexicon_tags_14 },
    { 0xc5b42114, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb214211a, SUC_lexicon_tags_27 },
    { 0x1414211b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1942122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62942125, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f4212a, SUC_lexicon_tags_26 },
    { 0xc294212a, SUC_lexicon_tags_12 },
    { 0xe174212a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7d4212e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f42131, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce542133, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d42136, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3b342138, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf754213d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf542141, SUC_lexicon_tags_17 },
    { 0x6cb42141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb42145, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7742148, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad42150, SUC_lexicon_tags_38 },
    { 0x4d342151, SUC_lexicon_tags_67 },
    { 0xe0742150, SUC_lexicon_tags_16 },
    { 0xe0342153, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xca342156, SUC_lexicon_tags_6 },
    { 0xba742157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd134215c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb874215e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9542160, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f542165, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d942169, SUC_lexicon_tags_44 },
    { 0x8874216a, SUC_lexicon_tags_63 },
    { 0xda14216a, SUC_lexicon_tags_17 },
    { 0x5634216c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4454216f, SUC_lexicon_tags_26 },
    { 0x1ed42170, SUC_lexicon_tags_11 },
    { 0x4b42170, SUC_lexicon_tags_23 },
    { 0x4742170, SUC_lexicon_tags_1 },
    { 0x5df42170, SUC_lexicon_tags_22 },
    { 0xf4342171, SUC_lexicon_tags_15 },
    { 0xc2942175, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4f42177, SUC_lexicon_tags_36 },
    { 0x6ab42178, SUC_lexicon_tags_12 },
    { 0xef142179, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2dd4217b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd054217e, SUC_lexicon_tags_45 },
    { 0xe34217f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34742181, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x88d42183, SUC_lexicon_tags_25 },
    { 0xed342183, SUC_lexicon_tags_54 },
    { 0xf4142183, SUC_lexicon_tags_3 },
    { 0x57742186, SUC_lexicon_tags_45 },
    { 0x30742187, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43142189, SUC_lexicon_tags_0 },
    { 0x9754218a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5342190, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67f42192, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49942198, SUC_lexicon_tags_37 },
    { 0xcd42199, SUC_lexicon_tags_38 },
    { 0x37b42199, SUC_lexicon_tags_79 },
    { 0x72d42198, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d4219f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x539421a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9421a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f421a7, SUC_lexicon_tags_11 },
    { 0x371421a7, SUC_lexicon_tags_55 },
    { 0x905421a9, SUC_lexicon_tags_11 },
    { 0x38d421aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d421af, SUC_lexicon_tags_15 },
    { 0x26f421b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x587421b2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9421b6, SUC_lexicon_tags_45 },
    { 0xe7d421b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1421bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7421c6, SUC_lexicon_tags_38 },
    { 0x6cd421c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db421cf, SUC_lexicon_tags_1 },
    { 0x43b421cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf49421d2, SUC_lexicon_tags_11 },
    { 0x47f421d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7421d6, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b421db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x451421de, SUC_lexicon_tags_37 },
    { 0x643421df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3421e2, SUC_lexicon_tags_12 },
    { 0xe93421e3, SUC_lexicon_tags_12 },
    { 0x4bb421e3, SUC_lexicon_tags_25 },
    { 0xa8d421e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87f421e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b421f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5421f3, SUC_lexicon_tags_47 },
    { 0x7eb421f4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93421f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d42200, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd42208, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7394220a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d74220d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa342210, SUC_lexicon_tags_11 },
    { 0x82d42210, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb6742213, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56142215, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0342219, SUC_lexicon_tags_146 },
    { 0x3e74221a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc8b4221c, SUC_lexicon_tags_34 },
    { 0x62f4221d, SUC_lexicon_tags_27 },
    { 0xbad4221e, SUC_lexicon_tags_17 },
    { 0xe6b4221f, SUC_lexicon_tags_43 },
    { 0x3a74221e, SUC_lexicon_tags_12 },
    { 0x4454221c, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0x33b42223, SUC_lexicon_tags_46 },
    { 0x74f42224, SUC_lexicon_tags_14 },
    { 0xe5542225, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc342229, SUC_lexicon_tags_37 },
    { 0xca74222a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4222f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2742231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf4223e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7b42240, SUC_lexicon_tags_38 },
    { 0xfe942240, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x82742243, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x91142245, SUC_lexicon_tags_11 },
    { 0x8e942246, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6142248, SUC_lexicon_tags_25 },
    { 0xa0b42248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf54224e, SUC_lexicon_tags_37 },
    { 0x1e54224f, SUC_lexicon_tags_80 },
    { 0x274224f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d42256, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c34225d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc342263, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f4226b, SUC_lexicon_tags_39 },
    { 0x90b4226c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17942271, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47d42273, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b42276, SUC_lexicon_tags_23 },
    { 0x62142276, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d94227b, SUC_lexicon_tags_25 },
    { 0x1db4227c, SUC_lexicon_tags_17 },
    { 0x55f4227c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b942280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81342289, SUC_lexicon_tags_9 },
    { 0xa474228a, SUC_lexicon_tags_15 },
    { 0x2994228b, SUC_lexicon_tags_17 },
    { 0xd4f4228c, SUC_lexicon_tags_15 },
    { 0x2014228d, SUC_lexicon_tags_25 },
    { 0x3494228e, SUC_lexicon_tags_9 },
    { 0x6554228a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b42297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dd42299, SUC_lexicon_tags_108 },
    { 0x2994229a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a34229f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1422a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb422a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa4f422ab, SUC_lexicon_tags_1 },
    { 0x13b422ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb13422ae, SUC_lexicon_tags_12 },
    { 0xdf9422af, SUC_lexicon_tags_17 },
    { 0x2b5422b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58b422b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x82b422b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803422be, SUC_lexicon_tags_17 },
    { 0x12d422be, SUC_lexicon_tags_1 },
    { 0xac9422c0, SUC_lexicon_tags_25 },
    { 0x105422c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73422c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb422cf, SUC_lexicon_tags_31 },
    { 0x521422d0, SUC_lexicon_tags_61 },
    { 0x9a3422d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd422d4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf422db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681422df, SUC_lexicon_tags_1 },
    { 0x81b422e0, SUC_lexicon_tags_37 },
    { 0x937422e0, SUC_lexicon_tags_179 },
    { 0x2a5422e2, SUC_lexicon_tags_47 },
    { 0x63b422e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05422e6, SUC_lexicon_tags_0 },
    { 0x5fd422e7, SUC_lexicon_tags_37 },
    { 0x2df422e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc63422ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b422ed, SUC_lexicon_tags_37 },
    { 0x111422ee, SUC_lexicon_tags_60 },
    { 0xac9422ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d7422f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee9422f3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1422f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1422fa, SUC_lexicon_tags_12 },
    { 0x1a7422fa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7422fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d42304, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x49542306, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0b42308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8294230c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43542310, SUC_lexicon_tags_0 },
    { 0x9d942311, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef42317, SUC_lexicon_tags_4 },
    { 0x2742318, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa134231b, SUC_lexicon_tags_14 },
    { 0xb74231c, SUC_lexicon_tags_37 },
    { 0x6d74231d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f42325, SUC_lexicon_tags_49 },
    { 0xe5342325, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x20742328, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3574232a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x32b4232c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb814232f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf42331, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74742334, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaed42336, SUC_lexicon_tags_1137 },
    { 0x1cd42337, SUC_lexicon_tags_1 },
    { 0x43542338, SUC_lexicon_tags_37 },
    { 0xec142339, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6234233e, SUC_lexicon_tags_15 },
    { 0x3914233f, SUC_lexicon_tags_37 },
    { 0xa794233e, SUC_lexicon_tags_11 },
    { 0xa342340, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb3d42343, SUC_lexicon_tags_12 },
    { 0xfab42344, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8742348, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d4234d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63d4234f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb942352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda74235a, SUC_lexicon_tags_27 },
    { 0x7594235b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab34235f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f542361, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9742369, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a34236e, SUC_lexicon_tags_31 },
    { 0x9f54236e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53542373, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9254237c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74237e, SUC_lexicon_tags_9 },
    { 0xa1d4237e, SUC_lexicon_tags_17 },
    { 0x3294237f, SUC_lexicon_tags_43 },
    { 0x2d94237f, SUC_lexicon_tags_25 },
    { 0xd3f42382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43142385, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1942388, SUC_lexicon_tags_14 },
    { 0xdd542389, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd42390, SUC_lexicon_tags_11 },
    { 0xa542390, SUC_lexicon_tags_7 },
    { 0x53f42390, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc142396, SUC_lexicon_tags_27 },
    { 0xea942397, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b4239b, SUC_lexicon_tags_25 },
    { 0xda34239c, SUC_lexicon_tags_25 },
    { 0xb094239d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbc34239f, SUC_lexicon_tags_27 },
    { 0x457423a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7423a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b423a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x835423a9, SUC_lexicon_tags_2 },
    { 0xe41423aa, SUC_lexicon_tags_15 },
    { 0x705423aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d423ae, SUC_lexicon_tags_15 },
    { 0xc07423af, SUC_lexicon_tags_34 },
    { 0xd7d423b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x83d423b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x497423b4, SUC_lexicon_tags_49 },
    { 0xca1423b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad423ba, SUC_lexicon_tags_1 },
    { 0xffb423bb, SUC_lexicon_tags_25 },
    { 0xfb1423bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69423c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191423cf, SUC_lexicon_tags_37 },
    { 0x36b423cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f9423d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f423d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd423d9, SUC_lexicon_tags_1 },
    { 0x9d1423d9, SUC_lexicon_tags_85 },
    { 0x199423da, SUC_lexicon_tags_15 },
    { 0x867423d9, SUC_lexicon_tags_11 },
    { 0x145423dd, SUC_lexicon_tags_17 },
    { 0x96b423de, SUC_lexicon_tags_43 },
    { 0x8f7423df, SUC_lexicon_tags_8 },
    { 0xdff423d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b423e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97423e8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb423eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a1423ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x561423ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375423f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163423f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2b423fb, SUC_lexicon_tags_23 },
    { 0xfcb423fc, SUC_lexicon_tags_49 },
    { 0x243423fd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f42401, SUC_lexicon_tags_60 },
    { 0x2fb42401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d342406, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd3d42408, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b4240f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5342412, SUC_lexicon_tags_57 },
    { 0x37942412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d42416, SUC_lexicon_tags_7 },
    { 0xafb42417, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b42419, SUC_lexicon_tags_6 },
    { 0x3a94241a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa154241d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66342422, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df42425, SUC_lexicon_tags_7 },
    { 0xc942425, SUC_lexicon_tags_37 },
    { 0x26742427, SUC_lexicon_tags_62 },
    { 0x52942426, SUC_lexicon_tags_7 },
    { 0xd1142429, SUC_lexicon_tags_43 },
    { 0x6cd4242a, SUC_lexicon_tags_35 },
    { 0xfa142428, SUC_lexicon_tags_63 },
    { 0x1c54242c, SUC_lexicon_tags_37 },
    { 0x1d74242a, SUC_lexicon_tags_37 },
    { 0x4bf4242e, SUC_lexicon_tags_17 },
    { 0xb74242f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27142434, SUC_lexicon_tags_33 },
    { 0x88f42434, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4742439, SUC_lexicon_tags_65 },
    { 0x554243a, SUC_lexicon_tags_68 },
    { 0x9194243b, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x5794243e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e542440, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8b42442, SUC_lexicon_tags_39 },
    { 0x7742443, SUC_lexicon_tags_37 },
    { 0x38742443, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7542448, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb4244b, SUC_lexicon_tags_15 },
    { 0xab54244c, SUC_lexicon_tags_15 },
    { 0x4ef4244d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f42452, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37742458, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f74245c, SUC_lexicon_tags_17 },
    { 0xf914245d, SUC_lexicon_tags_146 },
    { 0x5af4245d, SUC_lexicon_tags_17 },
    { 0x9b14245d, SUC_lexicon_tags_2 },
    { 0xb942460, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b42468, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf334246a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f42472, SUC_lexicon_tags_33 },
    { 0xb2f42473, SUC_lexicon_tags_7 },
    { 0xd5742474, SUC_lexicon_tags_15 },
    { 0xd3742475, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x16942477, SUC_lexicon_tags_15 },
    { 0xa7542477, SUC_lexicon_tags_1454 },
    { 0xf2742479, SUC_lexicon_tags_17 },
    { 0x91f4247a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d42485, SUC_lexicon_tags_42 },
    { 0x16142486, SUC_lexicon_tags_37 },
    { 0xd2142487, SUC_lexicon_tags_25 },
    { 0xb8b42488, SUC_lexicon_tags_0 },
    { 0x19742489, SUC_lexicon_tags_70 },
    { 0xb634248a, SUC_lexicon_tags_60 },
    { 0xa554248b, SUC_lexicon_tags_33 },
    { 0x8614248a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff42490, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b42493, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90142496, SUC_lexicon_tags_14 },
    { 0xb9b42497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf4249a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b74249d, SUC_lexicon_tags_2 },
    { 0x2934249e, SUC_lexicon_tags_47 },
    { 0x8e14249f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x573424a1, SUC_lexicon_tags_37 },
    { 0x859424a2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13424a9, SUC_lexicon_tags_60 },
    { 0x211424a9, SUC_lexicon_tags_30 },
    { 0x6b5424a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb424ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x821424b1, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7424b7, SUC_lexicon_tags_47 },
    { 0x565424b8, SUC_lexicon_tags_14 },
    { 0xf2d424b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x569424bc, SUC_lexicon_tags_25 },
    { 0xc7424bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd424c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb424c5, SUC_lexicon_tags_23 },
    { 0xfcb424c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d7424c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7424cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d424d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7424d4, SUC_lexicon_tags_14 },
    { 0xbb9424d4, SUC_lexicon_tags_45 },
    { 0x7cb424d6, SUC_lexicon_tags_9 },
    { 0xb7f424d4, SUC_lexicon_tags_2 },
    { 0x38f424d8, SUC_lexicon_tags_201 },
    { 0x599424d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa77424db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3b3424dd, SUC_lexicon_tags_7 },
    { 0x179424dd, SUC_lexicon_tags_37 },
    { 0xd0f424df, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1424e4, SUC_lexicon_tags_7 },
    { 0xeaf424e5, SUC_lexicon_tags_33 },
    { 0xdd424e6, SUC_lexicon_tags_29 },
    { 0x715424e5, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df424ed, SUC_lexicon_tags_92 },
    { 0x773424ed, SUC_lexicon_tags_7 },
    { 0x84b424ed, SUC_lexicon_tags_17 },
    { 0xe21424f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81424f2, SUC_lexicon_tags_25 },
    { 0xff5424f3, SUC_lexicon_tags_37 },
    { 0xb4b424f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b424f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b424ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20142505, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b4250b, SUC_lexicon_tags_47 },
    { 0x48d4250c, SUC_lexicon_tags_11 },
    { 0x56f4250d, SUC_lexicon_tags_1 },
    { 0xca74250e, SUC_lexicon_tags_37 },
    { 0xd4b4250e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd342511, SUC_lexicon_tags_23 },
    { 0x15542512, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd42516, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e542518, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9014251c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fd4251e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4142521, SUC_lexicon_tags_43 },
    { 0xb3d42521, SUC_lexicon_tags_12 },
    { 0x55d42523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d42527, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2274252a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc794252d, SUC_lexicon_tags_14 },
    { 0x5d74252d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91142532, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f942535, SUC_lexicon_tags_25 },
    { 0x4ff42535, SUC_lexicon_tags_37 },
    { 0xd8342536, SUC_lexicon_tags_28 },
    { 0xcf342535, SUC_lexicon_tags_27 },
    { 0xff342535, SUC_lexicon_tags_63 },
    { 0xfed4253a, SUC_lexicon_tags_82 },
    { 0x70d4253a, SUC_lexicon_tags_7 },
    { 0x55d4253a, SUC_lexicon_tags_25 },
    { 0x26f4253d, SUC_lexicon_tags_11 },
    { 0xa9f4253d, SUC_lexicon_tags_2 },
    { 0x24d4253a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d42542, SUC_lexicon_tags_47 },
    { 0xce542542, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb42547, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31742549, SUC_lexicon_tags_12 },
    { 0xc0b42549, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe734254c, SUC_lexicon_tags_17 },
    { 0x7434254d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d4254f, SUC_lexicon_tags_8 },
    { 0xe254254f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce142554, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f42557, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb4255a, SUC_lexicon_tags_29 },
    { 0x94d4255b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x50b4255d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a54255f, SUC_lexicon_tags_9 },
    { 0xf3b4255f, SUC_lexicon_tags_5 },
    { 0xbf942561, SUC_lexicon_tags_14 },
    { 0x39342560, SUC_lexicon_tags_60 },
    { 0xa0f42563, SUC_lexicon_tags_11 },
    { 0x9cb42564, SUC_lexicon_tags_29 },
    { 0x22742564, SUC_lexicon_tags_0 },
    { 0x6f942566, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa254256d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3914256f, SUC_lexicon_tags_47 },
    { 0x4154256f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e342577, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86d42579, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d4257c, SUC_lexicon_tags_11 },
    { 0xdb74257c, SUC_lexicon_tags_0 },
    { 0x5894257e, SUC_lexicon_tags_7 },
    { 0x8fb4257e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x81b42581, SUC_lexicon_tags_2 },
    { 0x47942582, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcb42584, SUC_lexicon_tags_38 },
    { 0xa6542584, SUC_lexicon_tags_37 },
    { 0x5e142584, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc342588, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5774258b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7342590, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12742594, SUC_lexicon_tags_15 },
    { 0xdf42595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc942597, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa774259e, SUC_lexicon_tags_37 },
    { 0xe934259e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b425a4, SUC_lexicon_tags_37 },
    { 0x96b425a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857425ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x659425ae, SUC_lexicon_tags_68 },
    { 0xeb5425af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3425b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21425b6, SUC_lexicon_tags_37 },
    { 0x33b425b7, SUC_lexicon_tags_65 },
    { 0xb8f425b8, SUC_lexicon_tags_30 },
    { 0x279425b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf425c1, SUC_lexicon_tags_14 },
    { 0x5a9425c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe01425c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe65425c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57425cb, SUC_lexicon_tags_33 },
    { 0xcff425cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x911425ce, SUC_lexicon_tags_25 },
    { 0xf25425ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3425d5, SUC_lexicon_tags_2 },
    { 0x971425d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7425e5, SUC_lexicon_tags_1 },
    { 0xbcb425e6, SUC_lexicon_tags_1 },
    { 0x3d3425e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf7425e9, SUC_lexicon_tags_53 },
    { 0x3d3425ea, SUC_lexicon_tags_12 },
    { 0x875425eb, SUC_lexicon_tags_61 },
    { 0x92b425eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf425ef, SUC_lexicon_tags_17 },
    { 0xf57425f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd91425f2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x515425f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc27425f7, SUC_lexicon_tags_6 },
    { 0xc21425f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x577425fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea3425fd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef42602, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x27942604, SUC_lexicon_tags_37 },
    { 0xd7942605, SUC_lexicon_tags_25 },
    { 0x1f142605, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f42609, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4074260e, SUC_lexicon_tags_38 },
    { 0x5234260f, SUC_lexicon_tags_17 },
    { 0xc9942610, SUC_lexicon_tags_10 },
    { 0xbf42611, SUC_lexicon_tags_68 },
    { 0x3e34260f, SUC_lexicon_tags_23 },
    { 0xde742613, SUC_lexicon_tags_61 },
    { 0x8a142610, SUC_lexicon_tags_0 },
    { 0xb5942614, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b42619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af4261e, SUC_lexicon_tags_7 },
    { 0x6274261e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x31942622, SUC_lexicon_tags_11 },
    { 0xfd542622, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a742628, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x434262a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa14262c, SUC_lexicon_tags_7 },
    { 0xe994262d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2742636, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3742639, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ed4263b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9734263d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84142641, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d42646, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf0142648, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3694264a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5542650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6cd42652, SUC_lexicon_tags_37 },
    { 0x25d42653, SUC_lexicon_tags_14 },
    { 0xa3b42653, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b4265b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b4265e, SUC_lexicon_tags_4 },
    { 0xcd94265f, SUC_lexicon_tags_17 },
    { 0x60542660, SUC_lexicon_tags_26 },
    { 0x19942661, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x67542663, SUC_lexicon_tags_15 },
    { 0x8fb42664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4342667, SUC_lexicon_tags_37 },
    { 0xd0d42668, SUC_lexicon_tags_31 },
    { 0x6f142667, SUC_lexicon_tags_10 },
    { 0x83542668, SUC_lexicon_tags_66 },
    { 0x41d4266a, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b4266f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc342671, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e14267a, SUC_lexicon_tags_33 },
    { 0x82b4267b, SUC_lexicon_tags_17 },
    { 0xe194267b, SUC_lexicon_tags_17 },
    { 0xf334267b, SUC_lexicon_tags_11 },
    { 0xcdb4267b, SUC_lexicon_tags_62 },
    { 0x47f4267f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2c342681, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd42684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb42688, SUC_lexicon_tags_17 },
    { 0xcf542689, SUC_lexicon_tags_37 },
    { 0xbcb42689, SUC_lexicon_tags_27 },
    { 0x5a14268b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x86f4268d, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f42690, SUC_lexicon_tags_45 },
    { 0x1342691, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8742698, SUC_lexicon_tags_53 },
    { 0xcf42699, SUC_lexicon_tags_7 },
    { 0xd394269a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b74269d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997426a1, SUC_lexicon_tags_65 },
    { 0x9df426a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3426a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9f426a9, SUC_lexicon_tags_48 },
    { 0x279426aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x29b426ac, SUC_lexicon_tags_211 },
    { 0x459426ad, SUC_lexicon_tags_37 },
    { 0x79f426ac, SUC_lexicon_tags_27 },
    { 0x21f426ad, SUC_lexicon_tags_14 },
    { 0x77d426b0, SUC_lexicon_tags_26 },
    { 0x4cf426b1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3426b9, SUC_lexicon_tags_25 },
    { 0x509426ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119426c4, SUC_lexicon_tags_19 },
    { 0x5d3426c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273426cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43426d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6fb426d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d426d9, SUC_lexicon_tags_11 },
    { 0x3b7426da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b7426dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb0d426de, SUC_lexicon_tags_37 },
    { 0x4bf426de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x111426e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141426ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5426f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f426f4, SUC_lexicon_tags_29 },
    { 0xdf5426f5, SUC_lexicon_tags_38 },
    { 0xa95426f6, SUC_lexicon_tags_11 },
    { 0x269426f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5426fa, SUC_lexicon_tags_1 },
    { 0x4d9426fa, SUC_lexicon_tags_6 },
    { 0xe4d426fb, SUC_lexicon_tags_25 },
    { 0x635426fc, SUC_lexicon_tags_8 },
    { 0xb39426fb, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xd9942700, SUC_lexicon_tags_6 },
    { 0x23142700, SUC_lexicon_tags_25 },
    { 0xd4942702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70f42704, SUC_lexicon_tags_30 },
    { 0x14b42704, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a94270b, SUC_lexicon_tags_37 },
    { 0x6254270c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6394270e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf4f42710, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5142713, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d942717, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b4271c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc794271e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f42725, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf42728, SUC_lexicon_tags_17 },
    { 0x11342729, SUC_lexicon_tags_15 },
    { 0x8c142729, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d4272e, SUC_lexicon_tags_17 },
    { 0x57d4272f, SUC_lexicon_tags_16 },
    { 0x6a14272e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15542733, SUC_lexicon_tags_30 },
    { 0x93f42734, SUC_lexicon_tags_17 },
    { 0x45942733, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25142737, SUC_lexicon_tags_57 },
    { 0x6bb42738, SUC_lexicon_tags_28 },
    { 0x83d42739, SUC_lexicon_tags_30 },
    { 0xced42738, SUC_lexicon_tags_37 },
    { 0x5634273a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92942743, SUC_lexicon_tags_65 },
    { 0x4342744, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f42747, SUC_lexicon_tags_1 },
    { 0x13d42748, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x54d4274a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda54274c, SUC_lexicon_tags_12 },
    { 0x1cb4274d, SUC_lexicon_tags_16 },
    { 0xc7b4274e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x57342750, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x53542752, SUC_lexicon_tags_10 },
    { 0xeb342753, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8074275a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9934275d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b42761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14942764, SUC_lexicon_tags_37 },
    { 0x86742765, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24142769, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d4276d, SUC_lexicon_tags_25 },
    { 0xba54276d, SUC_lexicon_tags_15 },
    { 0x6f4276f, SUC_lexicon_tags_37 },
    { 0x3614276e, SUC_lexicon_tags_11 },
    { 0xfad4276d, SUC_lexicon_tags_27 },
    { 0xb334276e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2942776, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f42779, SUC_lexicon_tags_28 },
    { 0x1514277a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3114277d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d42783, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba142789, SUC_lexicon_tags_29 },
    { 0xb5542789, SUC_lexicon_tags_27 },
    { 0x4fb42789, SUC_lexicon_tags_37 },
    { 0xfa942789, SUC_lexicon_tags_25 },
    { 0x8d54278d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf4f4278f, SUC_lexicon_tags_17 },
    { 0x45b42790, SUC_lexicon_tags_7 },
    { 0xe7142791, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e542794, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76f42796, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x11b42798, SUC_lexicon_tags_12 },
    { 0x6bb42799, SUC_lexicon_tags_6 },
    { 0xe334279a, SUC_lexicon_tags_11 },
    { 0x33b4279b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb94279e, SUC_lexicon_tags_25 },
    { 0x3a14279e, SUC_lexicon_tags_47 },
    { 0x64b4279f, SUC_lexicon_tags_37 },
    { 0xa75427a1, SUC_lexicon_tags_9 },
    { 0xebd427a2, SUC_lexicon_tags_15 },
    { 0x3994279f, SUC_lexicon_tags_11 },
    { 0xebf427a2, SUC_lexicon_tags_65 },
    { 0x6d427a1, SUC_lexicon_tags_46 },
    { 0x31d427a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f427aa, SUC_lexicon_tags_47 },
    { 0xcb1427aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab427b4, SUC_lexicon_tags_14 },
    { 0x245427b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1427ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3427c2, SUC_lexicon_tags_141 },
    { 0x511427c3, SUC_lexicon_tags_37 },
    { 0xc17427c4, SUC_lexicon_tags_26 },
    { 0x6a7427c5, SUC_lexicon_tags_60 },
    { 0xc7f427c4, SUC_lexicon_tags_2 },
    { 0xa55427c7, SUC_lexicon_tags_12 },
    { 0x9cd427c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7427cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa27427ce, SUC_lexicon_tags_38 },
    { 0x4b7427ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09427d5, SUC_lexicon_tags_56 },
    { 0x147427d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c9427d8, SUC_lexicon_tags_36 },
    { 0x27b427d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff427dc, SUC_lexicon_tags_7 },
    { 0xc8f427dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21427e2, SUC_lexicon_tags_0 },
    { 0x7bf427e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x895427e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5427eb, SUC_lexicon_tags_47 },
    { 0x233427ec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe23427ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05427f2, SUC_lexicon_tags_2 },
    { 0x135427f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1427f6, SUC_lexicon_tags_77 },
    { 0xd77427f7, SUC_lexicon_tags_9 },
    { 0x6df427f7, SUC_lexicon_tags_2 },
    { 0xf85427f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f1427fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9427fe, SUC_lexicon_tags_45 },
    { 0x8b5427ff, SUC_lexicon_tags_25 },
    { 0x3ef42800, SUC_lexicon_tags_47 },
    { 0xad542801, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5154280c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f42811, SUC_lexicon_tags_11 },
    { 0x5bb42812, SUC_lexicon_tags_1 },
    { 0x5bd42811, SUC_lexicon_tags_1 },
    { 0x6ad42811, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb4281f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a942822, SUC_lexicon_tags_42 },
    { 0xc3942823, SUC_lexicon_tags_7 },
    { 0x6e342824, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b42828, SUC_lexicon_tags_17 },
    { 0xa4f42828, SUC_lexicon_tags_2 },
    { 0x2c742829, SUC_lexicon_tags_25 },
    { 0xb254282a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9334282d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1942830, SUC_lexicon_tags_70 },
    { 0x22942831, SUC_lexicon_tags_55 },
    { 0x37d42831, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4542834, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d42838, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc714283c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xddb4283e, SUC_lexicon_tags_12 },
    { 0x9c74283e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc742841, SUC_lexicon_tags_39 },
    { 0x74d42842, SUC_lexicon_tags_12 },
    { 0x27342843, SUC_lexicon_tags_27 },
    { 0xabd42841, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2634284b, SUC_lexicon_tags_11 },
    { 0xcff4284c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b142852, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3d42854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d42859, SUC_lexicon_tags_10 },
    { 0xeb42859, SUC_lexicon_tags_33 },
    { 0xbd942859, SUC_lexicon_tags_37 },
    { 0xa214285c, SUC_lexicon_tags_15 },
    { 0x29f4285d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2942860, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad542863, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff42867, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9114286a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf954286e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5df42870, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10742874, SUC_lexicon_tags_29 },
    { 0x9b42874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22f42877, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedb42879, SUC_lexicon_tags_113 },
    { 0xf6742879, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7074287c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d42882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd42887, SUC_lexicon_tags_28 },
    { 0x2f142888, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c34288c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11142890, SUC_lexicon_tags_47 },
    { 0x25b42891, SUC_lexicon_tags_8 },
    { 0x4ab42892, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58742898, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f54289a, SUC_lexicon_tags_68 },
    { 0x66d4289b, SUC_lexicon_tags_29 },
    { 0xa514289c, SUC_lexicon_tags_37 },
    { 0x3bd4289d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689428a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89428a6, SUC_lexicon_tags_29 },
    { 0x269428a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb428ad, SUC_lexicon_tags_25 },
    { 0xefd428ae, SUC_lexicon_tags_1 },
    { 0xe57428ad, SUC_lexicon_tags_38 },
    { 0x2a9428b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9428b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73d428b5, SUC_lexicon_tags_17 },
    { 0x85d428b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a9428b8, SUC_lexicon_tags_11 },
    { 0xf53428b9, SUC_lexicon_tags_37 },
    { 0x6fb428b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x463428bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f428bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x68d428c1, SUC_lexicon_tags_25 },
    { 0xe95428c2, SUC_lexicon_tags_9 },
    { 0x343428c2, SUC_lexicon_tags_11 },
    { 0x7b7428c4, SUC_lexicon_tags_1 },
    { 0x875428c3, SUC_lexicon_tags_33 },
    { 0x92d428c6, SUC_lexicon_tags_35 },
    { 0x289428c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x737428c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcef428cb, SUC_lexicon_tags_30 },
    { 0xed5428cc, SUC_lexicon_tags_17 },
    { 0xe7b428cc, SUC_lexicon_tags_29 },
    { 0xdd5428ce, SUC_lexicon_tags_26 },
    { 0x543428cd, SUC_lexicon_tags_48 },
    { 0x393428cb, SUC_lexicon_tags_37 },
    { 0xf0f428d1, SUC_lexicon_tags_0 },
    { 0x999428d2, SUC_lexicon_tags_1 },
    { 0xe3d428cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f428d6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x8b7428d8, SUC_lexicon_tags_12 },
    { 0x90d428d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895428dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61428e3, SUC_lexicon_tags_10 },
    { 0xf11428e4, SUC_lexicon_tags_12 },
    { 0x769428e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f428e9, SUC_lexicon_tags_37 },
    { 0x649428ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f428ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e3428ef, SUC_lexicon_tags_43 },
    { 0x53428f0, SUC_lexicon_tags_55 },
    { 0xf03428f0, SUC_lexicon_tags_8 },
    { 0xceb428f1, SUC_lexicon_tags_27 },
    { 0xa0b428ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811428f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3428fb, SUC_lexicon_tags_14 },
    { 0x52d428fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0942901, SUC_lexicon_tags_1 },
    { 0x1dd42902, SUC_lexicon_tags_7 },
    { 0xcc942903, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb42906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0542909, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d4290e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df42912, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcfd42914, SUC_lexicon_tags_17 },
    { 0xf1742915, SUC_lexicon_tags_47 },
    { 0x14d42916, SUC_lexicon_tags_1 },
    { 0xaf42917, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f4291b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31942925, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21342928, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b94292d, SUC_lexicon_tags_6 },
    { 0xee74292e, SUC_lexicon_tags_124 },
    { 0x3dd4292d, SUC_lexicon_tags_45 },
    { 0xb9942930, SUC_lexicon_tags_422 },
    { 0, NULL },
    { 0x142932, SUC_lexicon_tags_7 },
    { 0x1f942933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbd42935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2f42937, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6034293a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb142940, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d42943, SUC_lexicon_tags_7 },
    { 0x8d142944, SUC_lexicon_tags_17 },
    { 0x4f42945, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb42948, SUC_lexicon_tags_63 },
    { 0xf1942948, SUC_lexicon_tags_7 },
    { 0xf4542948, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17d4294c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x76d4294e, SUC_lexicon_tags_5 },
    { 0xe54294f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e342955, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b4295a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x93f4295c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4274295f, SUC_lexicon_tags_12 },
    { 0x4ff42960, SUC_lexicon_tags_16 },
    { 0x65d42960, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x4142963, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7342966, SUC_lexicon_tags_23 },
    { 0xc8942966, SUC_lexicon_tags_39 },
    { 0x15142966, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd4296c, SUC_lexicon_tags_7 },
    { 0xec54296d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa574296f, SUC_lexicon_tags_0 },
    { 0x6c94296f, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1942976, SUC_lexicon_tags_47 },
    { 0x56f42977, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37742979, SUC_lexicon_tags_17 },
    { 0x9a34297a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb4297d, SUC_lexicon_tags_37 },
    { 0xe5b4297e, SUC_lexicon_tags_47 },
    { 0xd594297f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b42982, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f42985, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2e342987, SUC_lexicon_tags_17 },
    { 0x1c342988, SUC_lexicon_tags_2 },
    { 0xa5142988, SUC_lexicon_tags_2 },
    { 0x83542988, SUC_lexicon_tags_82 },
    { 0x7314298b, SUC_lexicon_tags_17 },
    { 0x5a34298c, SUC_lexicon_tags_62 },
    { 0xb594298d, SUC_lexicon_tags_55 },
    { 0x70b4298d, SUC_lexicon_tags_25 },
    { 0x62542987, SUC_lexicon_tags_46 },
    { 0x2bf42990, SUC_lexicon_tags_12 },
    { 0xdeb42991, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdbf42993, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19742997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d4299a, SUC_lexicon_tags_1308 },
    { 0x3b54299b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2d4299d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2714299f, SUC_lexicon_tags_25 },
    { 0x1b429a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b429a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb429b4, SUC_lexicon_tags_87 },
    { 0x433429b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99429b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7f429ba, SUC_lexicon_tags_5 },
    { 0xcb5429bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf11429bd, SUC_lexicon_tags_68 },
    { 0xdd9429be, SUC_lexicon_tags_49 },
    { 0x9e9429bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b429c2, SUC_lexicon_tags_23 },
    { 0x2b429c2, SUC_lexicon_tags_7 },
    { 0x743429c4, SUC_lexicon_tags_33 },
    { 0x5f7429c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3f429c7, SUC_lexicon_tags_19 },
    { 0xc67429c8, SUC_lexicon_tags_49 },
    { 0xc99429c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d429cc, SUC_lexicon_tags_1 },
    { 0x7c7429cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725429d7, SUC_lexicon_tags_47 },
    { 0x367429d8, SUC_lexicon_tags_1 },
    { 0x1c9429d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75429e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1429e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f429ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe91429ee, SUC_lexicon_tags_1 },
    { 0x97429ee, SUC_lexicon_tags_37 },
    { 0x1cb429f0, SUC_lexicon_tags_14 },
    { 0xeb5429f0, SUC_lexicon_tags_39 },
    { 0xc17429f2, SUC_lexicon_tags_37 },
    { 0xd8d429f3, SUC_lexicon_tags_77 },
    { 0x3b429f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7429f9, SUC_lexicon_tags_15 },
    { 0x5bd429f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37429ff, SUC_lexicon_tags_37 },
    { 0x76b42a00, SUC_lexicon_tags_15 },
    { 0x6942a00, SUC_lexicon_tags_17 },
    { 0xabb42a00, SUC_lexicon_tags_6 },
    { 0x67542a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db42a06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5942a08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6342a0a, SUC_lexicon_tags_23 },
    { 0x8ed42a0b, SUC_lexicon_tags_160 },
    { 0xdff42a0b, SUC_lexicon_tags_25 },
    { 0x4af42a0d, SUC_lexicon_tags_15 },
    { 0x21942a0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64142a10, SUC_lexicon_tags_133 },
    { 0x92542a10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab42a14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84142a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0142a1b, SUC_lexicon_tags_23 },
    { 0xcbd42a1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d42a2a, SUC_lexicon_tags_47 },
    { 0x8e142a2a, SUC_lexicon_tags_39 },
    { 0x73542a2a, SUC_lexicon_tags_45 },
    { 0xc2942a2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad42a35, SUC_lexicon_tags_12 },
    { 0x71942a36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca542a3b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x63b42a3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x63342a3f, SUC_lexicon_tags_55 },
    { 0xbcd42a3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb542a43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c342a47, SUC_lexicon_tags_92 },
    { 0x23142a48, SUC_lexicon_tags_25 },
    { 0xa0342a49, SUC_lexicon_tags_79 },
    { 0xe5f42a48, SUC_lexicon_tags_49 },
    { 0x20942a4a, SUC_lexicon_tags_14 },
    { 0x65342a47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90342a52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c742a54, SUC_lexicon_tags_37 },
    { 0x38742a55, SUC_lexicon_tags_30 },
    { 0x2b142a56, SUC_lexicon_tags_25 },
    { 0x87342a55, SUC_lexicon_tags_23 },
    { 0xfb942a57, SUC_lexicon_tags_1213 },
    { 0x76542a56, SUC_lexicon_tags_15 },
    { 0xa8d42a5a, SUC_lexicon_tags_49 },
    { 0x7a542a5b, SUC_lexicon_tags_25 },
    { 0xa5142a5a, SUC_lexicon_tags_10 },
    { 0xd9d42a5a, SUC_lexicon_tags_25 },
    { 0x36f42a5e, SUC_lexicon_tags_85 },
    { 0x1b342a5f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9742a64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x33742a66, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab42a6a, SUC_lexicon_tags_15 },
    { 0xcff42a6b, SUC_lexicon_tags_10 },
    { 0x542a6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70942a6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b742a71, SUC_lexicon_tags_14 },
    { 0x8cf42a72, SUC_lexicon_tags_10 },
    { 0x44142a72, SUC_lexicon_tags_7 },
    { 0x77542a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2942a76, SUC_lexicon_tags_26 },
    { 0x93542a76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9b42a79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7542a7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96942a7f, SUC_lexicon_tags_69 },
    { 0x4ef42a80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af42a84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb42a87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed42a8c, SUC_lexicon_tags_47 },
    { 0xb6742a8d, SUC_lexicon_tags_25 },
    { 0x91b42a8e, SUC_lexicon_tags_37 },
    { 0xc8342a8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96742a91, SUC_lexicon_tags_32 },
    { 0x11342a92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8542a94, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xca742a97, SUC_lexicon_tags_15 },
    { 0x41942a98, SUC_lexicon_tags_17 },
    { 0x98f42a97, SUC_lexicon_tags_12 },
    { 0x14d42a9a, SUC_lexicon_tags_53 },
    { 0x33942a9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49742aa5, SUC_lexicon_tags_47 },
    { 0x56f42aa5, SUC_lexicon_tags_12 },
    { 0xac542aa7, SUC_lexicon_tags_46 },
    { 0x6f342aa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac342aae, SUC_lexicon_tags_27 },
    { 0xe3742aae, SUC_lexicon_tags_23 },
    { 0x4ed42ab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5342ab7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18142aba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f42abd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8742ac4, SUC_lexicon_tags_25 },
    { 0x6a942ac5, SUC_lexicon_tags_47 },
    { 0xf2342ac5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5742acd, SUC_lexicon_tags_7 },
    { 0x9dd42acd, SUC_lexicon_tags_55 },
    { 0x64942acf, SUC_lexicon_tags_45 },
    { 0x34d42ad0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d542ad6, SUC_lexicon_tags_25 },
    { 0x44942ad7, SUC_lexicon_tags_0 },
    { 0x47d42ad8, SUC_lexicon_tags_18 },
    { 0xb1d42ad9, SUC_lexicon_tags_60 },
    { 0xf8342ad7, SUC_lexicon_tags_61 },
    { 0xd6342adb, SUC_lexicon_tags_0 },
    { 0xb1542adc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9742adf, SUC_lexicon_tags_25 },
    { 0x26f42adf, SUC_lexicon_tags_8 },
    { 0xf6942ae1, SUC_lexicon_tags_15 },
    { 0xf2b42ae2, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x47f42ae4, SUC_lexicon_tags_68 },
    { 0x24542ae5, SUC_lexicon_tags_21 },
    { 0x27342ae6, SUC_lexicon_tags_37 },
    { 0xc6f42ae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c142aea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b942af4, SUC_lexicon_tags_23 },
    { 0x51f42af5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b42af9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f542afe, SUC_lexicon_tags_25 },
    { 0x38f42aff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50b42b01, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b42b04, SUC_lexicon_tags_2 },
    { 0x94d42b05, SUC_lexicon_tags_1 },
    { 0x7f542b06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18f42b08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb42b0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f42b1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7e742b1d, SUC_lexicon_tags_23 },
    { 0x52742b1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f42b27, SUC_lexicon_tags_6 },
    { 0x64142b27, SUC_lexicon_tags_26 },
    { 0x3d542b29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47342b2d, SUC_lexicon_tags_1 },
    { 0x5ab42b2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x50f42b30, SUC_lexicon_tags_10 },
    { 0x35b42b31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b42b3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36342b40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db42b43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x55942b45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8342b48, SUC_lexicon_tags_8 },
    { 0x38342b48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41142b4d, SUC_lexicon_tags_948 },
    { 0x3fb42b4e, SUC_lexicon_tags_14 },
    { 0x93d42b4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb142b56, SUC_lexicon_tags_4 },
    { 0x17942b57, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd42b63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff742b69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73142b6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad42b72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d42b76, SUC_lexicon_tags_28 },
    { 0x90342b77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0342b7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66d42b7f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b42b83, SUC_lexicon_tags_37 },
    { 0x67742b83, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe2b42b86, SUC_lexicon_tags_17 },
    { 0x3b542b86, SUC_lexicon_tags_25 },
    { 0x8ff42b88, SUC_lexicon_tags_25 },
    { 0x90d42b88, SUC_lexicon_tags_7 },
    { 0xd0d42b86, SUC_lexicon_tags_7 },
    { 0x7c142b8b, SUC_lexicon_tags_63 },
    { 0x94142b8c, SUC_lexicon_tags_7 },
    { 0x48b42b8c, SUC_lexicon_tags_55 },
    { 0x1a142b8d, SUC_lexicon_tags_29 },
    { 0x34b42b8d, SUC_lexicon_tags_15 },
    { 0xb4b42b8d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f42b94, SUC_lexicon_tags_47 },
    { 0x5b742b95, SUC_lexicon_tags_47 },
    { 0x4cd42b94, SUC_lexicon_tags_37 },
    { 0x48b42b96, SUC_lexicon_tags_70 },
    { 0x23342b98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb942b9c, SUC_lexicon_tags_12 },
    { 0x79b42b9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f42ba2, SUC_lexicon_tags_38 },
    { 0xa8f42ba2, SUC_lexicon_tags_37 },
    { 0xc6142ba3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd5d42ba6, SUC_lexicon_tags_65 },
    { 0x24942ba7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bd42ba9, SUC_lexicon_tags_14 },
    { 0x61342baa, SUC_lexicon_tags_7 },
    { 0x7ff42bab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b42bae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb42bb1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa7342bb3, SUC_lexicon_tags_11 },
    { 0x26142bb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78542bba, SUC_lexicon_tags_7 },
    { 0x41342bbb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12b42bbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5742bc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78942bc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82d42bc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f542bc9, SUC_lexicon_tags_23 },
    { 0xbbb42bc9, SUC_lexicon_tags_30 },
    { 0x4d542bcb, SUC_lexicon_tags_28 },
    { 0x70f42bcc, SUC_lexicon_tags_47 },
    { 0x58742bcb, SUC_lexicon_tags_25 },
    { 0x7d742bcd, SUC_lexicon_tags_151 },
    { 0x9af42bca, SUC_lexicon_tags_37 },
    { 0x4542bc9, SUC_lexicon_tags_60 },
    { 0xe9542bc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2942bd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b42bd7, SUC_lexicon_tags_23 },
    { 0xdc142bd8, SUC_lexicon_tags_38 },
    { 0x23742bd8, SUC_lexicon_tags_26 },
    { 0xd0f42bd9, SUC_lexicon_tags_23 },
    { 0xa0f42bd8, SUC_lexicon_tags_63 },
    { 0x7a742bd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42342bde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19542be3, SUC_lexicon_tags_17 },
    { 0x32342be3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d42be7, SUC_lexicon_tags_9 },
    { 0x75142be8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3f542bea, SUC_lexicon_tags_45 },
    { 0x93742beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf42bee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90942bf0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62142bf6, SUC_lexicon_tags_12 },
    { 0xf2142bf7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x99d42bf9, SUC_lexicon_tags_33 },
    { 0xa4742bf9, SUC_lexicon_tags_5 },
    { 0x742bf9, SUC_lexicon_tags_11 },
    { 0x8f942bfc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b742c00, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c942c03, SUC_lexicon_tags_96 },
    { 0xf2342c04, SUC_lexicon_tags_37 },
    { 0x26542c05, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x95142c07, SUC_lexicon_tags_29 },
    { 0x94f42c08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c742c0b, SUC_lexicon_tags_37 },
    { 0xd1742c0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71742c0e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x76742c11, SUC_lexicon_tags_11 },
    { 0x8cf42c12, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc942c17, SUC_lexicon_tags_53 },
    { 0x79742c18, SUC_lexicon_tags_6 },
    { 0x19b42c18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x98142c1b, SUC_lexicon_tags_1 },
    { 0xb542c1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20f42c1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40b42c20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99342c24, SUC_lexicon_tags_45 },
    { 0x5f942c25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8542c28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b42c30, SUC_lexicon_tags_38 },
    { 0x5dd42c31, SUC_lexicon_tags_60 },
    { 0x47742c32, SUC_lexicon_tags_7 },
    { 0xa3f42c33, SUC_lexicon_tags_12 },
    { 0x6b142c32, SUC_lexicon_tags_28 },
    { 0x30742c35, SUC_lexicon_tags_65 },
    { 0xc5b42c36, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d42c3b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1bd42c3d, SUC_lexicon_tags_15 },
    { 0x40942c3e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67342c42, SUC_lexicon_tags_25 },
    { 0x6d342c42, SUC_lexicon_tags_33 },
    { 0x99142c43, SUC_lexicon_tags_47 },
    { 0x8dd42c42, SUC_lexicon_tags_11 },
    { 0x37f42c46, SUC_lexicon_tags_59 },
    { 0x78d42c44, SUC_lexicon_tags_28 },
    { 0xc0d42c48, SUC_lexicon_tags_63 },
    { 0x1a542c44, SUC_lexicon_tags_77 },
    { 0x38142c42, SUC_lexicon_tags_63 },
    { 0xa6b42c48, SUC_lexicon_tags_84 },
    { 0x8b142c4c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94742c52, SUC_lexicon_tags_5 },
    { 0x39f42c53, SUC_lexicon_tags_2 },
    { 0xf3b42c52, SUC_lexicon_tags_8 },
    { 0x55142c55, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b42c5b, SUC_lexicon_tags_7 },
    { 0xcff42c5c, SUC_lexicon_tags_17 },
    { 0x76342c5d, SUC_lexicon_tags_16 },
    { 0xc9742c5c, SUC_lexicon_tags_2 },
    { 0x7bf42c5b, SUC_lexicon_tags_17 },
    { 0x98542c5b, SUC_lexicon_tags_17 },
    { 0x2ff42c61, SUC_lexicon_tags_12 },
    { 0x87742c62, SUC_lexicon_tags_12 },
    { 0xac342c62, SUC_lexicon_tags_37 },
    { 0xfa942c5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8ff42c66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5542c6b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3342c6f, SUC_lexicon_tags_55 },
    { 0x56342c6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5342c77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58742c79, SUC_lexicon_tags_12 },
    { 0x7f742c7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b542c7d, SUC_lexicon_tags_60 },
    { 0xe1942c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7742c81, SUC_lexicon_tags_69 },
    { 0x2d42c82, SUC_lexicon_tags_37 },
    { 0xe1942c82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b142c87, SUC_lexicon_tags_6 },
    { 0xbf542c88, SUC_lexicon_tags_136 },
    { 0xa7b42c89, SUC_lexicon_tags_77 },
    { 0xe1342c8a, SUC_lexicon_tags_54 },
    { 0xe9942c89, SUC_lexicon_tags_2 },
    { 0x35d42c8c, SUC_lexicon_tags_23 },
    { 0xa9b42c8d, SUC_lexicon_tags_7 },
    { 0xd7542c88, SUC_lexicon_tags_1 },
    { 0xde742c8b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f42c92, SUC_lexicon_tags_27 },
    { 0x53b42c93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb542c95, SUC_lexicon_tags_30 },
    { 0xc3742c96, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xae742c99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e542c9c, SUC_lexicon_tags_17 },
    { 0x25542c9d, SUC_lexicon_tags_2 },
    { 0xeb942c9e, SUC_lexicon_tags_33 },
    { 0x1d542c9f, SUC_lexicon_tags_25 },
    { 0xe2342c9f, SUC_lexicon_tags_9 },
    { 0x4e142c9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4542ca3, SUC_lexicon_tags_21 },
    { 0x3bf42ca4, SUC_lexicon_tags_84 },
    { 0x7d42ca5, SUC_lexicon_tags_7 },
    { 0x7ad42ca5, SUC_lexicon_tags_12 },
    { 0xc6d42ca4, SUC_lexicon_tags_46 },
    { 0xb2742ca4, SUC_lexicon_tags_12 },
    { 0x2e942ca6, SUC_lexicon_tags_17 },
    { 0xe9742ca4, SUC_lexicon_tags_44 },
    { 0x1f742ca4, SUC_lexicon_tags_8 },
    { 0x72942ca4, SUC_lexicon_tags_0 },
    { 0x30342cad, SUC_lexicon_tags_31 },
    { 0xd1142cad, SUC_lexicon_tags_37 },
    { 0x5d342ca3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d542cba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38942cc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f142cc3, SUC_lexicon_tags_25 },
    { 0x80142cc3, SUC_lexicon_tags_1 },
    { 0x1f42cc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49142cc7, SUC_lexicon_tags_37 },
    { 0x1e542cc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b542ccf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26d42cd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5342cd4, SUC_lexicon_tags_14 },
    { 0xe3d42cd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a142cd7, SUC_lexicon_tags_12 },
    { 0x9f942cd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb42cde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb542ce1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b42ce5, SUC_lexicon_tags_37 },
    { 0x74342ce6, SUC_lexicon_tags_15 },
    { 0x3bd42ce6, SUC_lexicon_tags_39 },
    { 0xea742ce6, SUC_lexicon_tags_31 },
    { 0xa8f42ce5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf342ced, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9f42cef, SUC_lexicon_tags_84 },
    { 0xf8742cef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47942cf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa742cf5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5142cf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b42cfc, SUC_lexicon_tags_14 },
    { 0xbd142cfc, SUC_lexicon_tags_37 },
    { 0x61942cfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf542d04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98742d0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5542d10, SUC_lexicon_tags_37 },
    { 0x3b142d11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec742d16, SUC_lexicon_tags_77 },
    { 0xd6542d16, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd42d1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff542d20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2142d25, SUC_lexicon_tags_47 },
    { 0xd1542d26, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe2142d28, SUC_lexicon_tags_26 },
    { 0x32742d29, SUC_lexicon_tags_60 },
    { 0x1af42d2a, SUC_lexicon_tags_30 },
    { 0xf0542d29, SUC_lexicon_tags_38 },
    { 0x1f542d29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe742d2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fb42d30, SUC_lexicon_tags_15 },
    { 0x5ed42d31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13b42d33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb42d35, SUC_lexicon_tags_27 },
    { 0x86742d36, SUC_lexicon_tags_12 },
    { 0x99342d35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe4542d39, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8742d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7942d47, SUC_lexicon_tags_19 },
    { 0xc2f42d48, SUC_lexicon_tags_15 },
    { 0x88342d48, SUC_lexicon_tags_15 },
    { 0x2eb42d4a, SUC_lexicon_tags_7 },
    { 0xebb42d49, SUC_lexicon_tags_43 },
    { 0x37742d4b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1342d4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff42d55, SUC_lexicon_tags_7 },
    { 0x37b42d56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53542d5b, SUC_lexicon_tags_2 },
    { 0x78342d5b, SUC_lexicon_tags_33 },
    { 0x59542d5d, SUC_lexicon_tags_37 },
    { 0x70f42d5d, SUC_lexicon_tags_2 },
    { 0x23142d5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b42d66, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8af42d68, SUC_lexicon_tags_17 },
    { 0x89142d69, SUC_lexicon_tags_7 },
    { 0xa9742d68, SUC_lexicon_tags_15 },
    { 0xfbb42d6b, SUC_lexicon_tags_23 },
    { 0x8b942d6c, SUC_lexicon_tags_37 },
    { 0x3a542d6c, SUC_lexicon_tags_38 },
    { 0x89f42d6e, SUC_lexicon_tags_69 },
    { 0x7d142d6f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed42d7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf942d7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab42d81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4342d83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef42d86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc142d88, SUC_lexicon_tags_19 },
    { 0x16b42d89, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7142d8c, SUC_lexicon_tags_29 },
    { 0x52b42d8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4942d96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1db42d98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22942d9c, SUC_lexicon_tags_60 },
    { 0x8e942d9d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x57742d9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac342da1, SUC_lexicon_tags_72 },
    { 0xf2f42da1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4b942da4, SUC_lexicon_tags_1 },
    { 0x78342da4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d42da8, SUC_lexicon_tags_27 },
    { 0x33b42da9, SUC_lexicon_tags_56 },
    { 0x39142daa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8342db5, SUC_lexicon_tags_0 },
    { 0x3c542db6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5542db8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9342dbe, SUC_lexicon_tags_17 },
    { 0x25142dbf, SUC_lexicon_tags_37 },
    { 0x31342dc0, SUC_lexicon_tags_37 },
    { 0x88b42dc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3ef42dc3, SUC_lexicon_tags_11 },
    { 0x8f142dc4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12342dc8, SUC_lexicon_tags_25 },
    { 0xbb142dc9, SUC_lexicon_tags_25 },
    { 0x4f342dca, SUC_lexicon_tags_11 },
    { 0xfdf42dcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x76d42dcd, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb42dd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81942dd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd942dde, SUC_lexicon_tags_7 },
    { 0xd9d42ddf, SUC_lexicon_tags_17 },
    { 0xb4f42de0, SUC_lexicon_tags_12 },
    { 0x1d742de1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ed42de3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa942dee, SUC_lexicon_tags_15 },
    { 0x64342def, SUC_lexicon_tags_32 },
    { 0x9f142df0, SUC_lexicon_tags_26 },
    { 0x6142def, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x59342df3, SUC_lexicon_tags_25 },
    { 0xd542df4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb142df8, SUC_lexicon_tags_0 },
    { 0x69d42df9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a942dfc, SUC_lexicon_tags_43 },
    { 0xa8b42dfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d42e00, SUC_lexicon_tags_14 },
    { 0xa0742e01, SUC_lexicon_tags_33 },
    { 0x3db42e00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e542e04, SUC_lexicon_tags_43 },
    { 0x56b42e04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77542e0b, SUC_lexicon_tags_158 },
    { 0xf0542e0b, SUC_lexicon_tags_11 },
    { 0x54142e0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6742e16, SUC_lexicon_tags_32 },
    { 0x7c742e16, SUC_lexicon_tags_17 },
    { 0x7c142e18, SUC_lexicon_tags_6 },
    { 0x7d42e19, SUC_lexicon_tags_2 },
    { 0x50542e17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa7342e1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75942e20, SUC_lexicon_tags_17 },
    { 0x81142e20, SUC_lexicon_tags_9 },
    { 0xacd42e20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27342e28, SUC_lexicon_tags_15 },
    { 0xdf342e29, SUC_lexicon_tags_7 },
    { 0x4c142e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41742e2f, SUC_lexicon_tags_57 },
    { 0x14742e30, SUC_lexicon_tags_12 },
    { 0x1b142e31, SUC_lexicon_tags_17 },
    { 0xbe142e31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1742e37, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50942e43, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xea542e45, SUC_lexicon_tags_17 },
    { 0x4e942e46, SUC_lexicon_tags_7 },
    { 0x15942e47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8142e4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d42e5e, SUC_lexicon_tags_25 },
    { 0x19f42e5f, SUC_lexicon_tags_47 },
    { 0xc6342e60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x36342e62, SUC_lexicon_tags_79 },
    { 0x9b342e62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a342e66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3b542e68, SUC_lexicon_tags_28 },
    { 0x3db42e69, SUC_lexicon_tags_46 },
    { 0xd5d42e6a, SUC_lexicon_tags_46 },
    { 0x44942e6b, SUC_lexicon_tags_7 },
    { 0x67142e6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5542e6f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xf142e72, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5a742e74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5742e77, SUC_lexicon_tags_7 },
    { 0xed542e77, SUC_lexicon_tags_77 },
    { 0xc5542e79, SUC_lexicon_tags_10 },
    { 0x76342e78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x65b42e7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db42e83, SUC_lexicon_tags_37 },
    { 0x9d942e84, SUC_lexicon_tags_13 },
    { 0x27f42e85, SUC_lexicon_tags_25 },
    { 0xd8b42e86, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e742e89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa942e8f, SUC_lexicon_tags_12 },
    { 0x9ab42e90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10342e92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49342e94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e942e97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0b42e99, SUC_lexicon_tags_37 },
    { 0xcff42e9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b42ea1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x47f42ea3, SUC_lexicon_tags_47 },
    { 0x82b42ea4, SUC_lexicon_tags_47 },
    { 0x35f42ea4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd42ea8, SUC_lexicon_tags_14 },
    { 0x28142ea8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xca942eab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4742eaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f342eb1, SUC_lexicon_tags_28 },
    { 0x2d942eb1, SUC_lexicon_tags_11 },
    { 0xac542eb3, SUC_lexicon_tags_65 },
    { 0xcd142eb3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f42ebe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46142ec5, SUC_lexicon_tags_21 },
    { 0xab742ec6, SUC_lexicon_tags_27 },
    { 0x98742ec6, SUC_lexicon_tags_79 },
    { 0x75542ec5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaf942eca, SUC_lexicon_tags_11 },
    { 0x32f42ecb, SUC_lexicon_tags_6 },
    { 0x92342eca, SUC_lexicon_tags_16 },
    { 0x1b142eca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d42ed0, SUC_lexicon_tags_23 },
    { 0x5ad42ed0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d42ed5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78742edb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83942ede, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe342ee0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b42ee5, SUC_lexicon_tags_23 },
    { 0xc7742ee5, SUC_lexicon_tags_19 },
    { 0xef542ee7, SUC_lexicon_tags_51 },
    { 0x3c942ee8, SUC_lexicon_tags_51 },
    { 0xb0942ee8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2b42eeb, SUC_lexicon_tags_1 },
    { 0x6e942eeb, SUC_lexicon_tags_37 },
    { 0x8cd42eed, SUC_lexicon_tags_21 },
    { 0x91742eed, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48942ef5, SUC_lexicon_tags_25 },
    { 0xc6342ef6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3b342ef8, SUC_lexicon_tags_25 },
    { 0xe0f42ef9, SUC_lexicon_tags_45 },
    { 0x76342ef9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf9f42efc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8142f04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c742f0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43542f0e, SUC_lexicon_tags_12 },
    { 0xc1142f0f, SUC_lexicon_tags_3 },
    { 0x6342f10, SUC_lexicon_tags_229 },
    { 0xe7d42f0f, SUC_lexicon_tags_15 },
    { 0xe8742f11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb142f14, SUC_lexicon_tags_60 },
    { 0x23942f14, SUC_lexicon_tags_49 },
    { 0x65342f16, SUC_lexicon_tags_46 },
    { 0xb8542f15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d42f1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf542f1e, SUC_lexicon_tags_37 },
    { 0x7af42f1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f342f22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90942f24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c142f26, SUC_lexicon_tags_43 },
    { 0xdf742f27, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0942f30, SUC_lexicon_tags_23 },
    { 0xabf42f30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6342f33, SUC_lexicon_tags_49 },
    { 0x8f342f34, SUC_lexicon_tags_9 },
    { 0xe7f42f34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b42f3b, SUC_lexicon_tags_38 },
    { 0x96f42f3b, SUC_lexicon_tags_32 },
    { 0xdb142f3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3942f3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc942f41, SUC_lexicon_tags_31 },
    { 0xca742f42, SUC_lexicon_tags_7 },
    { 0x2c142f43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44d42f45, SUC_lexicon_tags_69 },
    { 0x99742f46, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4742f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52942f52, SUC_lexicon_tags_27 },
    { 0x67b42f53, SUC_lexicon_tags_37 },
    { 0x64942f53, SUC_lexicon_tags_33 },
    { 0xb942f54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc742f58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84742f5a, SUC_lexicon_tags_12 },
    { 0x81742f5b, SUC_lexicon_tags_10 },
    { 0xc0342f5b, SUC_lexicon_tags_37 },
    { 0x9c742f5d, SUC_lexicon_tags_10 },
    { 0x63942f5d, SUC_lexicon_tags_17 },
    { 0xff142f5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff42f64, SUC_lexicon_tags_9 },
    { 0x66d42f64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e542f6b, SUC_lexicon_tags_32 },
    { 0x51342f6c, SUC_lexicon_tags_12 },
    { 0x4f542f6c, SUC_lexicon_tags_2 },
    { 0xc7142f6e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x75142f70, SUC_lexicon_tags_16 },
    { 0x86f42f70, SUC_lexicon_tags_23 },
    { 0xa2d42f71, SUC_lexicon_tags_68 },
    { 0xbf742f70, SUC_lexicon_tags_28 },
    { 0x3cf42f73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d42f7b, SUC_lexicon_tags_12 },
    { 0xc5942f7c, SUC_lexicon_tags_77 },
    { 0x4ed42f7d, SUC_lexicon_tags_33 },
    { 0xcfd42f7d, SUC_lexicon_tags_16 },
    { 0x79142f7d, SUC_lexicon_tags_30 },
    { 0x15742f80, SUC_lexicon_tags_25 },
    { 0x9f542f7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6542f86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c142f88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9b42f8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74542f8c, SUC_lexicon_tags_7 },
    { 0xa3942f8c, SUC_lexicon_tags_15 },
    { 0x8cd42f8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d42f91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa7b42f93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db42f96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a542f9a, SUC_lexicon_tags_12 },
    { 0xddd42f9b, SUC_lexicon_tags_12 },
    { 0xb8d42f9c, SUC_lexicon_tags_7 },
    { 0xfc742f9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a742fa1, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e742fa5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f142faa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a342fad, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x14f42faf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52942fb3, SUC_lexicon_tags_7 },
    { 0xa2942fb4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x34742fb7, SUC_lexicon_tags_16 },
    { 0x93142fb8, SUC_lexicon_tags_37 },
    { 0x6d542fb9, SUC_lexicon_tags_26 },
    { 0xb8f42fb9, SUC_lexicon_tags_30 },
    { 0xa7142fbb, SUC_lexicon_tags_15 },
    { 0x6d142fbc, SUC_lexicon_tags_12 },
    { 0xc4f42fbd, SUC_lexicon_tags_1 },
    { 0xb2742fbe, SUC_lexicon_tags_7 },
    { 0x63542fbc, SUC_lexicon_tags_25 },
    { 0xca342fba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcf42fc2, SUC_lexicon_tags_43 },
    { 0xfb42fc3, SUC_lexicon_tags_15 },
    { 0x9e742fc3, SUC_lexicon_tags_15 },
    { 0x72142fc5, SUC_lexicon_tags_27 },
    { 0x42f42fc6, SUC_lexicon_tags_17 },
    { 0x11342fc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52942fc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae342fcb, SUC_lexicon_tags_47 },
    { 0x2942fcc, SUC_lexicon_tags_46 },
    { 0x8e942fcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db42fd2, SUC_lexicon_tags_17 },
    { 0x4cb42fd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98942fd7, SUC_lexicon_tags_11 },
    { 0xd4342fd7, SUC_lexicon_tags_12 },
    { 0xa5b42fd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f42fdf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30142fe4, SUC_lexicon_tags_38 },
    { 0x47342fe4, SUC_lexicon_tags_0 },
    { 0x21f42fe6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70742fe8, SUC_lexicon_tags_47 },
    { 0x1c542fe9, SUC_lexicon_tags_16 },
    { 0x3b542fea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xec942fec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68542ff1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b42ff4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a742ffa, SUC_lexicon_tags_84 },
    { 0xbdd42ffb, SUC_lexicon_tags_68 },
    { 0x4542ffb, SUC_lexicon_tags_43 },
    { 0x30342ffb, SUC_lexicon_tags_17 },
    { 0xe3742ffb, SUC_lexicon_tags_29 },
    { 0xf9942fff, SUC_lexicon_tags_1 },
    { 0xe9542fff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff743005, SUC_lexicon_tags_15 },
    { 0x8ff43006, SUC_lexicon_tags_8 },
    { 0x93f43007, SUC_lexicon_tags_8 },
    { 0xb5743007, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e34300b, SUC_lexicon_tags_44 },
    { 0x5a14300b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d143012, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc5143014, SUC_lexicon_tags_23 },
    { 0x4f543014, SUC_lexicon_tags_19 },
    { 0xbc343014, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7994301a, SUC_lexicon_tags_11 },
    { 0xf3b4301b, SUC_lexicon_tags_2 },
    { 0xf8d4301c, SUC_lexicon_tags_25 },
    { 0x2b94301b, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0x8a14301f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf343029, SUC_lexicon_tags_77 },
    { 0x57343029, SUC_lexicon_tags_37 },
    { 0x2c54302b, SUC_lexicon_tags_14 },
    { 0x894302b, SUC_lexicon_tags_30 },
    { 0x1bf4302a, SUC_lexicon_tags_2 },
    { 0xe4d4302e, SUC_lexicon_tags_17 },
    { 0x7794302e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76f43031, SUC_lexicon_tags_1 },
    { 0xdeb43032, SUC_lexicon_tags_2 },
    { 0xbe143032, SUC_lexicon_tags_30 },
    { 0xbbf43034, SUC_lexicon_tags_11 },
    { 0x19943035, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd154303a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2334303e, SUC_lexicon_tags_63 },
    { 0xd034303f, SUC_lexicon_tags_7 },
    { 0xe954303e, SUC_lexicon_tags_15 },
    { 0x41943040, SUC_lexicon_tags_49 },
    { 0xe274303e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f43045, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b43048, SUC_lexicon_tags_45 },
    { 0xa2743049, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5774304b, SUC_lexicon_tags_27 },
    { 0xd114304c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77143052, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x67543054, SUC_lexicon_tags_8 },
    { 0x54543055, SUC_lexicon_tags_193 },
    { 0x1e543054, SUC_lexicon_tags_33 },
    { 0xfe543057, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb4f43059, SUC_lexicon_tags_0 },
    { 0x6234305a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6354305c, SUC_lexicon_tags_17 },
    { 0x2cf4305c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52b4305f, SUC_lexicon_tags_70 },
    { 0x3b143060, SUC_lexicon_tags_1 },
    { 0x79f4305f, SUC_lexicon_tags_57 },
    { 0x77143060, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd234306a, SUC_lexicon_tags_29 },
    { 0x57b4306a, SUC_lexicon_tags_38 },
    { 0x89b4306c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb43070, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b743074, SUC_lexicon_tags_37 },
    { 0x3f343075, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x91543077, SUC_lexicon_tags_25 },
    { 0x52b43078, SUC_lexicon_tags_15 },
    { 0xbdb43077, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6934307b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa874307f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f43085, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e943087, SUC_lexicon_tags_7 },
    { 0x13743088, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b4308d, SUC_lexicon_tags_17 },
    { 0xca94308d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x87943090, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef43093, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x11b43095, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef4309d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87430a5, SUC_lexicon_tags_2 },
    { 0xce9430a6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875430ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3430b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff430b4, SUC_lexicon_tags_14 },
    { 0xc3f430b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1d430b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d430bf, SUC_lexicon_tags_2 },
    { 0xd07430c0, SUC_lexicon_tags_23 },
    { 0x6ff430c1, SUC_lexicon_tags_25 },
    { 0x37f430c2, SUC_lexicon_tags_11 },
    { 0x9ed430c3, SUC_lexicon_tags_29 },
    { 0x2c7430c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3d9430c6, SUC_lexicon_tags_45 },
    { 0xa5f430c7, SUC_lexicon_tags_33 },
    { 0x37f430c7, SUC_lexicon_tags_25 },
    { 0x52f430c6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33430d0, SUC_lexicon_tags_17 },
    { 0x75f430d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17430d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd430db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439430e0, SUC_lexicon_tags_11 },
    { 0x3ad430e0, SUC_lexicon_tags_28 },
    { 0xb9d430e1, SUC_lexicon_tags_1 },
    { 0xd0f430e3, SUC_lexicon_tags_33 },
    { 0xaab430e4, SUC_lexicon_tags_13 },
    { 0xfaf430e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1430eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24f430ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc49430ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd430f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865430f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5430fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d43104, SUC_lexicon_tags_2 },
    { 0x52543105, SUC_lexicon_tags_27 },
    { 0xa0f43106, SUC_lexicon_tags_8 },
    { 0x92b43107, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d4310a, SUC_lexicon_tags_9 },
    { 0xaf74310a, SUC_lexicon_tags_25 },
    { 0x6f14310c, SUC_lexicon_tags_0 },
    { 0x4034310d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b43111, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x93b43113, SUC_lexicon_tags_17 },
    { 0x89b43114, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b43118, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x32b4311a, SUC_lexicon_tags_1 },
    { 0x2314311b, SUC_lexicon_tags_63 },
    { 0x1974311c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca943121, SUC_lexicon_tags_26 },
    { 0x67143121, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe514312b, SUC_lexicon_tags_17 },
    { 0x59f4312c, SUC_lexicon_tags_3 },
    { 0xa794312d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6374312f, SUC_lexicon_tags_2 },
    { 0x80943130, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4b43132, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0343137, SUC_lexicon_tags_32 },
    { 0xd9943138, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f74313a, SUC_lexicon_tags_12 },
    { 0x18b4313a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f14313d, SUC_lexicon_tags_39 },
    { 0x1d94313e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa943140, SUC_lexicon_tags_15 },
    { 0xdd43141, SUC_lexicon_tags_21 },
    { 0x36d43141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b43146, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb4943148, SUC_lexicon_tags_17 },
    { 0xcd943149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd694314b, SUC_lexicon_tags_25 },
    { 0xd2b4314c, SUC_lexicon_tags_25 },
    { 0xb054314c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e14314f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x85143152, SUC_lexicon_tags_39 },
    { 0x45743153, SUC_lexicon_tags_12 },
    { 0xb6743153, SUC_lexicon_tags_43 },
    { 0x8df43153, SUC_lexicon_tags_522 },
    { 0x89943156, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5343158, SUC_lexicon_tags_28 },
    { 0xa943159, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa194315c, SUC_lexicon_tags_1 },
    { 0x1fb4315d, SUC_lexicon_tags_25 },
    { 0x68d4315d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf543164, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd4316b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6254316d, SUC_lexicon_tags_26 },
    { 0x3714316d, SUC_lexicon_tags_32 },
    { 0x48b4316f, SUC_lexicon_tags_60 },
    { 0x77b43170, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7543178, SUC_lexicon_tags_25 },
    { 0xd4943179, SUC_lexicon_tags_37 },
    { 0xf8743178, SUC_lexicon_tags_2 },
    { 0x6074317b, SUC_lexicon_tags_17 },
    { 0x5c14317a, SUC_lexicon_tags_8 },
    { 0xb874317d, SUC_lexicon_tags_38 },
    { 0x8cd4317e, SUC_lexicon_tags_11 },
    { 0x9754317e, SUC_lexicon_tags_10 },
    { 0x30b4317e, SUC_lexicon_tags_15 },
    { 0xb4317f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3543184, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c743186, SUC_lexicon_tags_2 },
    { 0xfb543186, SUC_lexicon_tags_49 },
    { 0xe1343188, SUC_lexicon_tags_49 },
    { 0x50543189, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5194318b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26943193, SUC_lexicon_tags_8 },
    { 0x10f43193, SUC_lexicon_tags_1 },
    { 0xd5543194, SUC_lexicon_tags_12 },
    { 0xd5f43196, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1743199, SUC_lexicon_tags_17 },
    { 0x62d4319a, SUC_lexicon_tags_26 },
    { 0xc914319b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3894319d, SUC_lexicon_tags_17 },
    { 0x1c14319e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23f431a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfb1431a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87431aa, SUC_lexicon_tags_11 },
    { 0xc3431ab, SUC_lexicon_tags_31 },
    { 0x317431ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc61431ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79431b2, SUC_lexicon_tags_37 },
    { 0xcb7431b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1431b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc3431b8, SUC_lexicon_tags_39 },
    { 0x11431b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7431bc, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161431c3, SUC_lexicon_tags_0 },
    { 0x363431c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681431ca, SUC_lexicon_tags_30 },
    { 0x5c9431cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579431d0, SUC_lexicon_tags_25 },
    { 0x4d5431d1, SUC_lexicon_tags_12 },
    { 0x4f3431d2, SUC_lexicon_tags_65 },
    { 0x735431d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x63431d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x835431d7, SUC_lexicon_tags_37 },
    { 0x4d1431d8, SUC_lexicon_tags_37 },
    { 0x8bb431d9, SUC_lexicon_tags_37 },
    { 0xb01431d7, SUC_lexicon_tags_46 },
    { 0xa47431db, SUC_lexicon_tags_37 },
    { 0xde1431dc, SUC_lexicon_tags_2 },
    { 0x825431dc, SUC_lexicon_tags_37 },
    { 0x797431db, SUC_lexicon_tags_65 },
    { 0x29431de, SUC_lexicon_tags_1 },
    { 0x651431e0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x499431e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x275431e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99431ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b431ed, SUC_lexicon_tags_10 },
    { 0xef3431ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7431f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x335431f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7b9431f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409431fc, SUC_lexicon_tags_17 },
    { 0x601431fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b43202, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xaff43204, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb343209, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b4320d, SUC_lexicon_tags_37 },
    { 0x1014320e, SUC_lexicon_tags_25 },
    { 0x9d54320f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdb43211, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb543215, SUC_lexicon_tags_6 },
    { 0xa4f43216, SUC_lexicon_tags_9 },
    { 0xf0743216, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb4321a, SUC_lexicon_tags_87 },
    { 0x61f4321a, SUC_lexicon_tags_37 },
    { 0x38f4321b, SUC_lexicon_tags_48 },
    { 0xcf54321c, SUC_lexicon_tags_19 },
    { 0x6334321e, SUC_lexicon_tags_47 },
    { 0x2ed4321e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b343221, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x36343223, SUC_lexicon_tags_2 },
    { 0x85943224, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd43228, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1514322b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3543231, SUC_lexicon_tags_29 },
    { 0xe6543231, SUC_lexicon_tags_16 },
    { 0xfd143233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2534323a, SUC_lexicon_tags_27 },
    { 0xfdd4323b, SUC_lexicon_tags_47 },
    { 0xdf34323b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d4323f, SUC_lexicon_tags_37 },
    { 0x43143240, SUC_lexicon_tags_19 },
    { 0x9d743241, SUC_lexicon_tags_47 },
    { 0x9cd4323f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b343244, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d43248, SUC_lexicon_tags_49 },
    { 0x9f43249, SUC_lexicon_tags_37 },
    { 0x6743249, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b4324f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa6143251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e343253, SUC_lexicon_tags_2 },
    { 0x4bf43254, SUC_lexicon_tags_79 },
    { 0xd8f43255, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1774325a, SUC_lexicon_tags_60 },
    { 0xe0b4325b, SUC_lexicon_tags_106 },
    { 0x1374325a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1ad4325e, SUC_lexicon_tags_53 },
    { 0x2454325f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87143268, SUC_lexicon_tags_39 },
    { 0x8943268, SUC_lexicon_tags_37 },
    { 0x3a14326a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4994326e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d543276, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2254327a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde143280, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1743286, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc274328d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4143290, SUC_lexicon_tags_60 },
    { 0x9f143290, SUC_lexicon_tags_2 },
    { 0xd0b43290, SUC_lexicon_tags_37 },
    { 0xc1943290, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8b43295, SUC_lexicon_tags_63 },
    { 0xe3b43296, SUC_lexicon_tags_37 },
    { 0xd4b43295, SUC_lexicon_tags_12 },
    { 0x1543298, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa274329e, SUC_lexicon_tags_63 },
    { 0x63b4329e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153432a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b432ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe85432b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5432ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589432be, SUC_lexicon_tags_21 },
    { 0x5d5432be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3f5432c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62d432c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf67432c5, SUC_lexicon_tags_7 },
    { 0xaa7432c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41432cc, SUC_lexicon_tags_15 },
    { 0xb9432cc, SUC_lexicon_tags_17 },
    { 0x791432cc, SUC_lexicon_tags_11 },
    { 0x787432cf, SUC_lexicon_tags_65 },
    { 0x85f432d0, SUC_lexicon_tags_32 },
    { 0x1dd432d0, SUC_lexicon_tags_1 },
    { 0x731432d2, SUC_lexicon_tags_39 },
    { 0x1e1432d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d432d7, SUC_lexicon_tags_9 },
    { 0x5ff432d8, SUC_lexicon_tags_28 },
    { 0x64d432d9, SUC_lexicon_tags_28 },
    { 0x2d3432da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x291432dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd432df, SUC_lexicon_tags_37 },
    { 0xe15432e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x893432e2, SUC_lexicon_tags_14 },
    { 0xab1432e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6b432e5, SUC_lexicon_tags_38 },
    { 0x823432e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297432eb, SUC_lexicon_tags_10 },
    { 0x6bf432ec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b432f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f432f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6fb432f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f432ff, SUC_lexicon_tags_38 },
    { 0x67543300, SUC_lexicon_tags_15 },
    { 0xf3543301, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f43308, SUC_lexicon_tags_37 },
    { 0xc6543308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf274330e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d943312, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf6f43314, SUC_lexicon_tags_47 },
    { 0x2e743315, SUC_lexicon_tags_12 },
    { 0x45f43315, SUC_lexicon_tags_43 },
    { 0xa1943316, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe3b43319, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1594331b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d4331e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45343320, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce543323, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15b43325, SUC_lexicon_tags_43 },
    { 0x1bb43325, SUC_lexicon_tags_27 },
    { 0x29143326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe154332a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfd4332c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad4332f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xac743331, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0743336, SUC_lexicon_tags_2 },
    { 0x12943337, SUC_lexicon_tags_1 },
    { 0xacb43336, SUC_lexicon_tags_384 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b54333c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3174333e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65d43340, SUC_lexicon_tags_19 },
    { 0x53b43341, SUC_lexicon_tags_17 },
    { 0xb143340, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f43345, SUC_lexicon_tags_27 },
    { 0x90743346, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce54334c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3574334e, SUC_lexicon_tags_12 },
    { 0xdbd4334f, SUC_lexicon_tags_15 },
    { 0x28d4334e, SUC_lexicon_tags_1 },
    { 0x7d143351, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaf743353, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1dd43355, SUC_lexicon_tags_17 },
    { 0xd143355, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf6743358, SUC_lexicon_tags_7 },
    { 0xa8543359, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f94335c, SUC_lexicon_tags_9 },
    { 0x94335c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f43360, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x9743363, SUC_lexicon_tags_12 },
    { 0x2cb43363, SUC_lexicon_tags_17 },
    { 0x79343365, SUC_lexicon_tags_7 },
    { 0x51143366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d343369, SUC_lexicon_tags_11 },
    { 0x4a14336a, SUC_lexicon_tags_19 },
    { 0x1694336b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8234336d, SUC_lexicon_tags_29 },
    { 0x3314336e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6b43370, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad43374, SUC_lexicon_tags_8 },
    { 0x5f543375, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d43380, SUC_lexicon_tags_15 },
    { 0x79f43381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdd43383, SUC_lexicon_tags_60 },
    { 0x98143384, SUC_lexicon_tags_30 },
    { 0x26543383, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7b143387, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f4338b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff143390, SUC_lexicon_tags_79 },
    { 0xa5f43391, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5c343393, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0x79d43395, SUC_lexicon_tags_15 },
    { 0xf5b43396, SUC_lexicon_tags_0 },
    { 0x4cd43397, SUC_lexicon_tags_7 },
    { 0x52f43395, SUC_lexicon_tags_65 },
    { 0x54743398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4654339c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97433a4, SUC_lexicon_tags_13 },
    { 0x52f433a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7433aa, SUC_lexicon_tags_17 },
    { 0x165433ab, SUC_lexicon_tags_1 },
    { 0x561433ac, SUC_lexicon_tags_27 },
    { 0xd65433ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x413433b0, SUC_lexicon_tags_19 },
    { 0xfd1433b1, SUC_lexicon_tags_27 },
    { 0x1df433b2, SUC_lexicon_tags_25 },
    { 0xddb433b3, SUC_lexicon_tags_2 },
    { 0xf4b433b4, SUC_lexicon_tags_15 },
    { 0xc2b433b5, SUC_lexicon_tags_7 },
    { 0x5d9433b5, SUC_lexicon_tags_2 },
    { 0xa9433b1, SUC_lexicon_tags_30 },
    { 0x8f7433b8, SUC_lexicon_tags_15 },
    { 0xdad433b9, SUC_lexicon_tags_47 },
    { 0xe8d433b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f433bf, SUC_lexicon_tags_17 },
    { 0xe9433bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f433c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833433ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40b433cc, SUC_lexicon_tags_11 },
    { 0x505433cd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x44b433cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af433d2, SUC_lexicon_tags_32 },
    { 0x191433d2, SUC_lexicon_tags_28 },
    { 0xc71433d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b433db, SUC_lexicon_tags_60 },
    { 0x8c3433dc, SUC_lexicon_tags_60 },
    { 0x471433dd, SUC_lexicon_tags_84 },
    { 0xaf3433de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb433e2, SUC_lexicon_tags_7 },
    { 0xfd1433e3, SUC_lexicon_tags_55 },
    { 0x40f433e3, SUC_lexicon_tags_31 },
    { 0x2cf433e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xebf433e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5433eb, SUC_lexicon_tags_11 },
    { 0x4cb433ec, SUC_lexicon_tags_11 },
    { 0x44d433ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f433f4, SUC_lexicon_tags_1 },
    { 0x29b433f5, SUC_lexicon_tags_35 },
    { 0xaab433f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55433fa, SUC_lexicon_tags_26 },
    { 0xdff433fb, SUC_lexicon_tags_12 },
    { 0xac3433fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef9433fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80943407, SUC_lexicon_tags_19 },
    { 0xbd43408, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa954340b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6374340f, SUC_lexicon_tags_17 },
    { 0x9eb43410, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b43413, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21743417, SUC_lexicon_tags_1 },
    { 0x67b43417, SUC_lexicon_tags_37 },
    { 0x2c343419, SUC_lexicon_tags_27 },
    { 0x3f4341a, SUC_lexicon_tags_33 },
    { 0x5c943419, SUC_lexicon_tags_17 },
    { 0x2c94341a, SUC_lexicon_tags_47 },
    { 0xb8f4341d, SUC_lexicon_tags_49 },
    { 0x5014341e, SUC_lexicon_tags_14 },
    { 0xbad4341f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a143421, SUC_lexicon_tags_37 },
    { 0xf4143422, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95143426, SUC_lexicon_tags_25 },
    { 0xe3543427, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x44543429, SUC_lexicon_tags_1 },
    { 0xaaf43429, SUC_lexicon_tags_65 },
    { 0xcc74342a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d43432, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e743434, SUC_lexicon_tags_60 },
    { 0xbcf43435, SUC_lexicon_tags_17 },
    { 0x1e943435, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9143439, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc194343c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe494343f, SUC_lexicon_tags_8 },
    { 0x26f43440, SUC_lexicon_tags_37 },
    { 0x51543441, SUC_lexicon_tags_19 },
    { 0x56943442, SUC_lexicon_tags_43 },
    { 0xd6b43442, SUC_lexicon_tags_64 },
    { 0xe8d43444, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f343447, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7b43449, SUC_lexicon_tags_15 },
    { 0x9994344a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e74344c, SUC_lexicon_tags_84 },
    { 0x15b4344d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52943451, SUC_lexicon_tags_77 },
    { 0xb543451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd43455, SUC_lexicon_tags_37 },
    { 0xef43456, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34743458, SUC_lexicon_tags_11 },
    { 0x4b743458, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdad4345b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1343460, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b43465, SUC_lexicon_tags_15 },
    { 0x37943466, SUC_lexicon_tags_38 },
    { 0xaa143467, SUC_lexicon_tags_53 },
    { 0xd5343466, SUC_lexicon_tags_37 },
    { 0x32f43466, SUC_lexicon_tags_2 },
    { 0xc6b4346a, SUC_lexicon_tags_8 },
    { 0x9d743469, SUC_lexicon_tags_25 },
    { 0xaf34346c, SUC_lexicon_tags_61 },
    { 0x4fb4346c, SUC_lexicon_tags_33 },
    { 0xfd94346e, SUC_lexicon_tags_15 },
    { 0xcb74346e, SUC_lexicon_tags_47 },
    { 0x63d4346a, SUC_lexicon_tags_25 },
    { 0xe8d43468, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x93743473, SUC_lexicon_tags_11 },
    { 0xde943474, SUC_lexicon_tags_30 },
    { 0xdaf43474, SUC_lexicon_tags_33 },
    { 0xad143475, SUC_lexicon_tags_12 },
    { 0x6543477, SUC_lexicon_tags_46 },
    { 0xce143478, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f43482, SUC_lexicon_tags_77 },
    { 0x84143483, SUC_lexicon_tags_25 },
    { 0xd6b43484, SUC_lexicon_tags_23 },
    { 0xc2943484, SUC_lexicon_tags_19 },
    { 0x9a143486, SUC_lexicon_tags_38 },
    { 0x53f43482, SUC_lexicon_tags_47 },
    { 0x19943488, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf754348a, SUC_lexicon_tags_11 },
    { 0x4c74348b, SUC_lexicon_tags_28 },
    { 0xb814348a, SUC_lexicon_tags_25 },
    { 0x10d4348d, SUC_lexicon_tags_43 },
    { 0xa274348e, SUC_lexicon_tags_7 },
    { 0xa34348d, SUC_lexicon_tags_25 },
    { 0xa0543490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80343493, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb43496, SUC_lexicon_tags_60 },
    { 0xf9f43497, SUC_lexicon_tags_16 },
    { 0xfd543498, SUC_lexicon_tags_27 },
    { 0x8ab43497, SUC_lexicon_tags_37 },
    { 0xee143496, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa74349e, SUC_lexicon_tags_23 },
    { 0x2254349f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3434a2, SUC_lexicon_tags_30 },
    { 0x64d434a2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437434a7, SUC_lexicon_tags_644 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9434ab, SUC_lexicon_tags_6 },
    { 0x8c3434ac, SUC_lexicon_tags_15 },
    { 0x62f434ad, SUC_lexicon_tags_47 },
    { 0x7af434ae, SUC_lexicon_tags_25 },
    { 0xfd3434ab, SUC_lexicon_tags_25 },
    { 0x991434ad, SUC_lexicon_tags_23 },
    { 0xef9434af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7434b6, SUC_lexicon_tags_47 },
    { 0xa6d434b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1434ba, SUC_lexicon_tags_11 },
    { 0xdd5434ba, SUC_lexicon_tags_12 },
    { 0x37d434bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaf434be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xded434c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3434c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9434cd, SUC_lexicon_tags_26 },
    { 0xf95434ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2a1434d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85434d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x785434d7, SUC_lexicon_tags_7 },
    { 0xc1f434d8, SUC_lexicon_tags_11 },
    { 0xee1434d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b434de, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d434e5, SUC_lexicon_tags_60 },
    { 0xe5d434e6, SUC_lexicon_tags_23 },
    { 0xfb7434e7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635434f0, SUC_lexicon_tags_26 },
    { 0x4b5434f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89434f6, SUC_lexicon_tags_11 },
    { 0xb1434f6, SUC_lexicon_tags_45 },
    { 0x4ef434f8, SUC_lexicon_tags_17 },
    { 0x21b434f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55434fb, SUC_lexicon_tags_25 },
    { 0xfb5434fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b43504, SUC_lexicon_tags_26 },
    { 0xd5b43504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f4350b, SUC_lexicon_tags_12 },
    { 0x3794350b, SUC_lexicon_tags_27 },
    { 0x1614350d, SUC_lexicon_tags_17 },
    { 0x8d14350e, SUC_lexicon_tags_60 },
    { 0xcdf4350c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac143515, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10943519, SUC_lexicon_tags_46 },
    { 0xec34351a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6814351e, SUC_lexicon_tags_7 },
    { 0xa594351f, SUC_lexicon_tags_49 },
    { 0x70b43520, SUC_lexicon_tags_49 },
    { 0x4e943521, SUC_lexicon_tags_17 },
    { 0x64d43522, SUC_lexicon_tags_29 },
    { 0xf8f4351e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef4352b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d14352f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88943531, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b143534, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d43538, SUC_lexicon_tags_1237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6343540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b43547, SUC_lexicon_tags_0 },
    { 0x78943548, SUC_lexicon_tags_25 },
    { 0xd5d43548, SUC_lexicon_tags_17 },
    { 0x4034354a, SUC_lexicon_tags_25 },
    { 0xb5d4354b, SUC_lexicon_tags_85 },
    { 0x7694354b, SUC_lexicon_tags_11 },
    { 0xad14354b, SUC_lexicon_tags_2 },
    { 0x52d43547, SUC_lexicon_tags_15 },
    { 0x2e14354d, SUC_lexicon_tags_12 },
    { 0xc743550, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb9f43552, SUC_lexicon_tags_28 },
    { 0xb4b43553, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x36943556, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5154355a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6994355d, SUC_lexicon_tags_1 },
    { 0xd894355d, SUC_lexicon_tags_15 },
    { 0x1b54355f, SUC_lexicon_tags_15 },
    { 0xe3d4355e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc143565, SUC_lexicon_tags_65 },
    { 0xde743565, SUC_lexicon_tags_12 },
    { 0xec343566, SUC_lexicon_tags_17 },
    { 0xad343568, SUC_lexicon_tags_34 },
    { 0x1b943566, SUC_lexicon_tags_12 },
    { 0x71b4356a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7554356e, SUC_lexicon_tags_8 },
    { 0xbe74356f, SUC_lexicon_tags_27 },
    { 0x56143570, SUC_lexicon_tags_25 },
    { 0x8d943571, SUC_lexicon_tags_0 },
    { 0x57743572, SUC_lexicon_tags_27 },
    { 0x99743573, SUC_lexicon_tags_37 },
    { 0x68143573, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xfa143576, SUC_lexicon_tags_37 },
    { 0x39343576, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bd43579, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3614357b, SUC_lexicon_tags_37 },
    { 0xb6f4357b, SUC_lexicon_tags_37 },
    { 0xf374357c, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23743581, SUC_lexicon_tags_29 },
    { 0xb6f43582, SUC_lexicon_tags_124 },
    { 0x58743582, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8743586, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb4358d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef43591, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcdd43593, SUC_lexicon_tags_48 },
    { 0xd7343593, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f43598, SUC_lexicon_tags_12 },
    { 0x79d43599, SUC_lexicon_tags_17 },
    { 0x60543599, SUC_lexicon_tags_14 },
    { 0xd0f4359b, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df435a0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x217435a2, SUC_lexicon_tags_33 },
    { 0xd1d435a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x773435a6, SUC_lexicon_tags_14 },
    { 0x80b435a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x233435aa, SUC_lexicon_tags_46 },
    { 0xd23435ab, SUC_lexicon_tags_8 },
    { 0x401435ac, SUC_lexicon_tags_15 },
    { 0x797435aa, SUC_lexicon_tags_15 },
    { 0x8d3435ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b9435b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x203435b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc7435b4, SUC_lexicon_tags_37 },
    { 0x65b435b5, SUC_lexicon_tags_23 },
    { 0x16d435b6, SUC_lexicon_tags_15 },
    { 0x7ab435b7, SUC_lexicon_tags_25 },
    { 0x1a7435b8, SUC_lexicon_tags_29 },
    { 0x11b435b9, SUC_lexicon_tags_37 },
    { 0x79435ba, SUC_lexicon_tags_43 },
    { 0x6b5435ba, SUC_lexicon_tags_33 },
    { 0x60f435b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699435c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271435c5, SUC_lexicon_tags_30 },
    { 0xd3b435c6, SUC_lexicon_tags_26 },
    { 0x21b435c6, SUC_lexicon_tags_25 },
    { 0xbc5435c8, SUC_lexicon_tags_30 },
    { 0x1a9435c7, SUC_lexicon_tags_27 },
    { 0x81f435c8, SUC_lexicon_tags_2 },
    { 0x92b435ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b435cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7435d2, SUC_lexicon_tags_47 },
    { 0xfbd435d3, SUC_lexicon_tags_29 },
    { 0x209435d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af435d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d435dc, SUC_lexicon_tags_17 },
    { 0xc03435dd, SUC_lexicon_tags_19 },
    { 0x67f435dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1435e1, SUC_lexicon_tags_26 },
    { 0x8f1435e2, SUC_lexicon_tags_11 },
    { 0x9ed435e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29f435e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5435ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d435ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989435f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b435fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89435fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83143606, SUC_lexicon_tags_14 },
    { 0xf3543607, SUC_lexicon_tags_25 },
    { 0xedf43607, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d4360b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf094360d, SUC_lexicon_tags_26 },
    { 0x6054360d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8743610, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xff543612, SUC_lexicon_tags_26 },
    { 0x70143613, SUC_lexicon_tags_25 },
    { 0x5b743613, SUC_lexicon_tags_61 },
    { 0xca943615, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55543617, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb434361b, SUC_lexicon_tags_17 },
    { 0x6854361c, SUC_lexicon_tags_19 },
    { 0xaab4361b, SUC_lexicon_tags_15 },
    { 0x5cd4361d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd43622, SUC_lexicon_tags_19 },
    { 0xa8143622, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36743625, SUC_lexicon_tags_11 },
    { 0x36543626, SUC_lexicon_tags_14 },
    { 0xd543626, SUC_lexicon_tags_1 },
    { 0xb1d43625, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe614362a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f74362e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xed543630, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd43635, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa914363a, SUC_lexicon_tags_9 },
    { 0x2714363b, SUC_lexicon_tags_70 },
    { 0x3254363a, SUC_lexicon_tags_29 },
    { 0xcf94363c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3214363f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b43642, SUC_lexicon_tags_27 },
    { 0xcef43643, SUC_lexicon_tags_19 },
    { 0xc2d43642, SUC_lexicon_tags_14 },
    { 0xc0343645, SUC_lexicon_tags_6 },
    { 0xfeb43645, SUC_lexicon_tags_15 },
    { 0x18f43647, SUC_lexicon_tags_17 },
    { 0x55143648, SUC_lexicon_tags_37 },
    { 0x95143649, SUC_lexicon_tags_14 },
    { 0x95d4364a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xab74364c, SUC_lexicon_tags_45 },
    { 0x1d4364d, SUC_lexicon_tags_28 },
    { 0xd8d4364c, SUC_lexicon_tags_37 },
    { 0x1db4364c, SUC_lexicon_tags_9 },
    { 0xe934364e, SUC_lexicon_tags_63 },
    { 0xc6743651, SUC_lexicon_tags_15 },
    { 0xd914364e, SUC_lexicon_tags_66 },
    { 0xde143653, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x27543656, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0d43658, SUC_lexicon_tags_31 },
    { 0xca943659, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc6b4365b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4ab4365d, SUC_lexicon_tags_12 },
    { 0xdbd4365e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6943663, SUC_lexicon_tags_32 },
    { 0x70343663, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5743669, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4366d, SUC_lexicon_tags_11 },
    { 0xf354366e, SUC_lexicon_tags_29 },
    { 0x5014366f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2543672, SUC_lexicon_tags_9 },
    { 0x4db43673, SUC_lexicon_tags_19 },
    { 0xab343673, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db43679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa594367e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9943682, SUC_lexicon_tags_37 },
    { 0x63d43683, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d143685, SUC_lexicon_tags_15 },
    { 0x1f43686, SUC_lexicon_tags_45 },
    { 0xe5543687, SUC_lexicon_tags_8 },
    { 0x9cf43688, SUC_lexicon_tags_12 },
    { 0xea543689, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2943693, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd943695, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd4543697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf354369a, SUC_lexicon_tags_6 },
    { 0xe974369b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaef4369d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17436a1, SUC_lexicon_tags_17 },
    { 0x1a3436a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487436a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed436ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9ed436ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7436b4, SUC_lexicon_tags_11 },
    { 0x61b436b5, SUC_lexicon_tags_2 },
    { 0x8cf436b6, SUC_lexicon_tags_7 },
    { 0x4fd436b7, SUC_lexicon_tags_2 },
    { 0xc57436b8, SUC_lexicon_tags_28 },
    { 0x98b436b9, SUC_lexicon_tags_46 },
    { 0x925436b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7436bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x457436c0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b436c8, SUC_lexicon_tags_43 },
    { 0xe3f436c9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53436cc, SUC_lexicon_tags_0 },
    { 0xc2b436cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197436d2, SUC_lexicon_tags_28 },
    { 0xfe5436d2, SUC_lexicon_tags_21 },
    { 0xbb9436d3, SUC_lexicon_tags_49 },
    { 0xc8d436d5, SUC_lexicon_tags_25 },
    { 0xc57436d6, SUC_lexicon_tags_14 },
    { 0x859436d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5436dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af436e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af436e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f436e7, SUC_lexicon_tags_6 },
    { 0xf21436e8, SUC_lexicon_tags_49 },
    { 0x739436e8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x10b436eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81f436ed, SUC_lexicon_tags_15 },
    { 0x639436ee, SUC_lexicon_tags_32 },
    { 0xff7436ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9db436f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33d436f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9436f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43436fa, SUC_lexicon_tags_15 },
    { 0x313436fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x947436fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4543706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14b43708, SUC_lexicon_tags_15 },
    { 0x3bb43709, SUC_lexicon_tags_14 },
    { 0xd0343709, SUC_lexicon_tags_12 },
    { 0xf234370b, SUC_lexicon_tags_63 },
    { 0x15b4370b, SUC_lexicon_tags_2 },
    { 0x8754370d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x814370f, SUC_lexicon_tags_30 },
    { 0xbe94370f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23343715, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45943719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad54371c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4943720, SUC_lexicon_tags_31 },
    { 0xd5f43721, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7b943723, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b43728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb894372d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24943731, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xacd43733, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35743735, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x92343737, SUC_lexicon_tags_0 },
    { 0x95343738, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f4373d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b43740, SUC_lexicon_tags_24 },
    { 0xf1b43740, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2143746, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67743748, SUC_lexicon_tags_70 },
    { 0x5b543749, SUC_lexicon_tags_12 },
    { 0x3f143748, SUC_lexicon_tags_0 },
    { 0xf254374b, SUC_lexicon_tags_12 },
    { 0x7c74374c, SUC_lexicon_tags_1 },
    { 0x8114374d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f143750, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cf43752, SUC_lexicon_tags_38 },
    { 0x1af43752, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b43759, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b4375c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45b4375e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e543762, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f4376b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47543773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x743775, SUC_lexicon_tags_0 },
    { 0x8e543776, SUC_lexicon_tags_32 },
    { 0xeb743776, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa034377b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed43781, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac943785, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa3943787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa334378b, SUC_lexicon_tags_0 },
    { 0x54b4378c, SUC_lexicon_tags_9 },
    { 0x1754378d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91f4378f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f43796, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c34379a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07437a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf31437a5, SUC_lexicon_tags_7 },
    { 0x363437a6, SUC_lexicon_tags_8 },
    { 0x3c9437a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49437aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41437ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7b437af, SUC_lexicon_tags_12 },
    { 0x349437af, SUC_lexicon_tags_1 },
    { 0x28d437b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52b437b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec1437b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b437bb, SUC_lexicon_tags_33 },
    { 0xc63437bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b437bf, SUC_lexicon_tags_30 },
    { 0x2b5437c0, SUC_lexicon_tags_49 },
    { 0x489437bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb1437c3, SUC_lexicon_tags_14 },
    { 0x7c7437c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f1437c6, SUC_lexicon_tags_2 },
    { 0x2f437c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df437d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xab437d2, SUC_lexicon_tags_9 },
    { 0xb5d437d3, SUC_lexicon_tags_17 },
    { 0x919437d4, SUC_lexicon_tags_12 },
    { 0xe35437d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23437d9, SUC_lexicon_tags_7 },
    { 0x361437d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe63437dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1437df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49437e2, SUC_lexicon_tags_46 },
    { 0x743437e3, SUC_lexicon_tags_55 },
    { 0x191437e4, SUC_lexicon_tags_23 },
    { 0x41f437e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x621437e8, SUC_lexicon_tags_47 },
    { 0xabf437e8, SUC_lexicon_tags_29 },
    { 0x2ab437ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3f9437ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x963437ee, SUC_lexicon_tags_25 },
    { 0xc07437ef, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb437f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x265437f6, SUC_lexicon_tags_2 },
    { 0x225437f7, SUC_lexicon_tags_11 },
    { 0xbe5437f8, SUC_lexicon_tags_5 },
    { 0x235437f9, SUC_lexicon_tags_9 },
    { 0x34b437fa, SUC_lexicon_tags_84 },
    { 0xda9437fa, SUC_lexicon_tags_3 },
    { 0x8f1437f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5543800, SUC_lexicon_tags_17 },
    { 0x48343800, SUC_lexicon_tags_33 },
    { 0xd8943802, SUC_lexicon_tags_15 },
    { 0xa6943803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30143808, SUC_lexicon_tags_33 },
    { 0xa2d43809, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x1314380c, SUC_lexicon_tags_38 },
    { 0xdfb4380d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3894380f, SUC_lexicon_tags_7 },
    { 0xb3343810, SUC_lexicon_tags_49 },
    { 0xb3543811, SUC_lexicon_tags_1 },
    { 0xfed43812, SUC_lexicon_tags_49 },
    { 0x2cb43813, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0143816, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f4381a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb4381d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45543822, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b43826, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd4382d, SUC_lexicon_tags_17 },
    { 0xdf14382e, SUC_lexicon_tags_11 },
    { 0x60d4382f, SUC_lexicon_tags_15 },
    { 0xb454382d, SUC_lexicon_tags_25 },
    { 0xa634382d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfb943833, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd43836, SUC_lexicon_tags_0 },
    { 0x2bf43836, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa1343839, SUC_lexicon_tags_13 },
    { 0x3b74383a, SUC_lexicon_tags_3 },
    { 0x1db4383a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb4383f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f743844, SUC_lexicon_tags_27 },
    { 0x19943845, SUC_lexicon_tags_37 },
    { 0x83543846, SUC_lexicon_tags_4 },
    { 0xecb43846, SUC_lexicon_tags_738 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d4384b, SUC_lexicon_tags_12 },
    { 0xe4d4384c, SUC_lexicon_tags_32 },
    { 0x2474384d, SUC_lexicon_tags_27 },
    { 0x69b4384e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf143852, SUC_lexicon_tags_2 },
    { 0x69943853, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f43858, SUC_lexicon_tags_30 },
    { 0x78343859, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b4385e, SUC_lexicon_tags_15 },
    { 0x5bb4385e, SUC_lexicon_tags_8 },
    { 0x35143860, SUC_lexicon_tags_7 },
    { 0xd514385f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7f343863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe114386a, SUC_lexicon_tags_17 },
    { 0x95d4386b, SUC_lexicon_tags_8 },
    { 0xa754386b, SUC_lexicon_tags_19 },
    { 0xe794386b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9d94386f, SUC_lexicon_tags_48 },
    { 0x1eb4386f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c943872, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d43876, SUC_lexicon_tags_25 },
    { 0xc2743876, SUC_lexicon_tags_11 },
    { 0x55d43878, SUC_lexicon_tags_11 },
    { 0x3f743879, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4114387b, SUC_lexicon_tags_2 },
    { 0xf74387c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb943881, SUC_lexicon_tags_17 },
    { 0x3fb43882, SUC_lexicon_tags_49 },
    { 0x3e743883, SUC_lexicon_tags_15 },
    { 0x74943884, SUC_lexicon_tags_38 },
    { 0x61943883, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ff43887, SUC_lexicon_tags_149 },
    { 0xe7b43887, SUC_lexicon_tags_37 },
    { 0xfed43889, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xac94388b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8143891, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9143895, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21943897, SUC_lexicon_tags_12 },
    { 0xa9143898, SUC_lexicon_tags_46 },
    { 0xd4943899, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd4389c, SUC_lexicon_tags_2 },
    { 0x4e74389d, SUC_lexicon_tags_47 },
    { 0x6934389e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a3438a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x159438a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57438a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcb9438a7, SUC_lexicon_tags_10 },
    { 0x9f3438a8, SUC_lexicon_tags_15 },
    { 0xdad438a9, SUC_lexicon_tags_25 },
    { 0x20f438aa, SUC_lexicon_tags_60 },
    { 0x443438a9, SUC_lexicon_tags_24 },
    { 0x6bb438a7, SUC_lexicon_tags_27 },
    { 0xb89438ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xced438af, SUC_lexicon_tags_1 },
    { 0x823438b0, SUC_lexicon_tags_47 },
    { 0x34b438b0, SUC_lexicon_tags_30 },
    { 0x6af438b2, SUC_lexicon_tags_38 },
    { 0x119438b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf438b8, SUC_lexicon_tags_37 },
    { 0x7b9438b9, SUC_lexicon_tags_46 },
    { 0xf71438ba, SUC_lexicon_tags_37 },
    { 0x73f438ba, SUC_lexicon_tags_15 },
    { 0xda7438bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd438c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa57438c5, SUC_lexicon_tags_84 },
    { 0x6a3438c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d438c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4b438cb, SUC_lexicon_tags_45 },
    { 0xb09438cc, SUC_lexicon_tags_45 },
    { 0x3f5438cb, SUC_lexicon_tags_37 },
    { 0x829438cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda3438d0, SUC_lexicon_tags_47 },
    { 0xf438d1, SUC_lexicon_tags_8 },
    { 0x659438d2, SUC_lexicon_tags_25 },
    { 0x1e5438d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3438d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57438d9, SUC_lexicon_tags_14 },
    { 0x8b9438da, SUC_lexicon_tags_38 },
    { 0xb33438db, SUC_lexicon_tags_15 },
    { 0x309438dc, SUC_lexicon_tags_16 },
    { 0x9c1438dc, SUC_lexicon_tags_25 },
    { 0x4cf438de, SUC_lexicon_tags_11 },
    { 0xfe7438df, SUC_lexicon_tags_15 },
    { 0x77f438de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9438e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed438ec, SUC_lexicon_tags_1 },
    { 0x785438ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3438f3, SUC_lexicon_tags_165 },
    { 0x31b438f4, SUC_lexicon_tags_17 },
    { 0x885438f5, SUC_lexicon_tags_11 },
    { 0xf65438f5, SUC_lexicon_tags_13 },
    { 0x601438f7, SUC_lexicon_tags_68 },
    { 0xe21438f8, SUC_lexicon_tags_37 },
    { 0x233438f9, SUC_lexicon_tags_31 },
    { 0x4cf438f3, SUC_lexicon_tags_0 },
    { 0x7b5438fb, SUC_lexicon_tags_11 },
    { 0xd19438fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x185438ff, SUC_lexicon_tags_1213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f4390b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeff4390d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3343912, SUC_lexicon_tags_25 },
    { 0x9cd43913, SUC_lexicon_tags_30 },
    { 0xbf943914, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4543921, SUC_lexicon_tags_27 },
    { 0x3b743922, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe6d43924, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf54392c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4934392e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff43931, SUC_lexicon_tags_0 },
    { 0x66343932, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb3943934, SUC_lexicon_tags_19 },
    { 0xe8b43935, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b43939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b34393c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f34393f, SUC_lexicon_tags_37 },
    { 0x7d143940, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfd43942, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6343944, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df4394a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d4394f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13543951, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x97343953, SUC_lexicon_tags_38 },
    { 0x20143954, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd4395b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe794395d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff743962, SUC_lexicon_tags_60 },
    { 0x55743963, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce74396a, SUC_lexicon_tags_17 },
    { 0x7294396a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6734396d, SUC_lexicon_tags_12 },
    { 0x8b4396e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87343970, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce343978, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f4397c, SUC_lexicon_tags_11 },
    { 0xa494397d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a943983, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b43989, SUC_lexicon_tags_17 },
    { 0x1634398a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a14398d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f343990, SUC_lexicon_tags_11 },
    { 0x42b43991, SUC_lexicon_tags_60 },
    { 0xd1d43992, SUC_lexicon_tags_25 },
    { 0x73f43991, SUC_lexicon_tags_28 },
    { 0x71143993, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97f43996, SUC_lexicon_tags_7 },
    { 0x60343997, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7343999, SUC_lexicon_tags_15 },
    { 0xa074399a, SUC_lexicon_tags_37 },
    { 0xc134399a, SUC_lexicon_tags_16 },
    { 0x59f4399b, SUC_lexicon_tags_37 },
    { 0x6ad4399d, SUC_lexicon_tags_26 },
    { 0x7f94399e, SUC_lexicon_tags_7 },
    { 0x6514399f, SUC_lexicon_tags_1 },
    { 0xafb4399c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3439a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f439aa, SUC_lexicon_tags_66 },
    { 0x2af439ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711439b1, SUC_lexicon_tags_27 },
    { 0x2eb439b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f439b6, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xa39439b8, SUC_lexicon_tags_1 },
    { 0x899439b8, SUC_lexicon_tags_10 },
    { 0x8f5439ba, SUC_lexicon_tags_37 },
    { 0x2df439bb, SUC_lexicon_tags_60 },
    { 0xead439bc, SUC_lexicon_tags_23 },
    { 0xe05439bb, SUC_lexicon_tags_37 },
    { 0x113439bc, SUC_lexicon_tags_2 },
    { 0x7f1439bf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6a9439c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b439c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff439c9, SUC_lexicon_tags_47 },
    { 0x823439ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b439ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf49439d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5439d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb439d9, SUC_lexicon_tags_57 },
    { 0x705439da, SUC_lexicon_tags_31 },
    { 0x91439db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f439de, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f439e5, SUC_lexicon_tags_12 },
    { 0xe4f439e6, SUC_lexicon_tags_2 },
    { 0xd11439e5, SUC_lexicon_tags_11 },
    { 0x27b439e8, SUC_lexicon_tags_37 },
    { 0x941439e9, SUC_lexicon_tags_7 },
    { 0x47f439ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611439ef, SUC_lexicon_tags_14 },
    { 0x69b439ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd439f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a7439f5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x13b439f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595439fc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e543a05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40943a08, SUC_lexicon_tags_25 },
    { 0xe143a09, SUC_lexicon_tags_7 },
    { 0xfb43a09, SUC_lexicon_tags_17 },
    { 0xa7143a0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d43a0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d43a11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d43a19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c543a1d, SUC_lexicon_tags_17 },
    { 0x32543a1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4743a20, SUC_lexicon_tags_23 },
    { 0x5f143a21, SUC_lexicon_tags_2 },
    { 0xf8743a22, SUC_lexicon_tags_26 },
    { 0x71543a22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90d43a25, SUC_lexicon_tags_15 },
    { 0x5c943a25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f43a29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf43a2e, SUC_lexicon_tags_28 },
    { 0xfb143a2e, SUC_lexicon_tags_43 },
    { 0x9cd43a30, SUC_lexicon_tags_11 },
    { 0x75743a31, SUC_lexicon_tags_70 },
    { 0x17143a30, SUC_lexicon_tags_68 },
    { 0x2c743a33, SUC_lexicon_tags_2 },
    { 0x9d543a2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8c143a36, SUC_lexicon_tags_12 },
    { 0x3c343a36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49943a3a, SUC_lexicon_tags_10 },
    { 0x83343a3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31143a3f, SUC_lexicon_tags_11 },
    { 0x75143a3f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x89f43a42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88143a44, SUC_lexicon_tags_27 },
    { 0xa7b43a45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0743a47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b343a4b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb9943a4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfd43a4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x83543a52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb43a58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5543a5e, SUC_lexicon_tags_37 },
    { 0xba343a5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d43a62, SUC_lexicon_tags_29 },
    { 0x6d343a63, SUC_lexicon_tags_17 },
    { 0xdbd43a64, SUC_lexicon_tags_45 },
    { 0x81d43a65, SUC_lexicon_tags_6 },
    { 0x30143a66, SUC_lexicon_tags_63 },
    { 0x61f43a64, SUC_lexicon_tags_106 },
    { 0xf3943a68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80943a6c, SUC_lexicon_tags_38 },
    { 0x4c143a6c, SUC_lexicon_tags_2 },
    { 0x943a6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84543a75, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xfbf43a77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83743a7a, SUC_lexicon_tags_63 },
    { 0x31143a7b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d743a7e, SUC_lexicon_tags_7 },
    { 0xe4b43a7e, SUC_lexicon_tags_103 },
    { 0x16343a80, SUC_lexicon_tags_25 },
    { 0xecd43a80, SUC_lexicon_tags_17 },
    { 0x87543a7f, SUC_lexicon_tags_23 },
    { 0x88143a83, SUC_lexicon_tags_6 },
    { 0x5bb43a84, SUC_lexicon_tags_4 },
    { 0x45b43a85, SUC_lexicon_tags_4 },
    { 0xaf743a85, SUC_lexicon_tags_15 },
    { 0x76d43a83, SUC_lexicon_tags_29 },
    { 0xfa343a83, SUC_lexicon_tags_23 },
    { 0x51f43a89, SUC_lexicon_tags_15 },
    { 0xdd143a8a, SUC_lexicon_tags_37 },
    { 0xb6943a88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa343a8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f343a91, SUC_lexicon_tags_37 },
    { 0x12543a92, SUC_lexicon_tags_47 },
    { 0xc4943a92, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b43a98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd43a9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddd43aa1, SUC_lexicon_tags_11 },
    { 0xb2f43aa2, SUC_lexicon_tags_11 },
    { 0xa8d43aa3, SUC_lexicon_tags_1 },
    { 0xe7943aa4, SUC_lexicon_tags_17 },
    { 0x4bb43aa5, SUC_lexicon_tags_70 },
    { 0x78b43aa6, SUC_lexicon_tags_15 },
    { 0xe9b43aa5, SUC_lexicon_tags_2 },
    { 0xdb343aa8, SUC_lexicon_tags_16 },
    { 0x6c543aa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b143aae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28743ab1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b943ab6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x14f43ab8, SUC_lexicon_tags_60 },
    { 0x56943ab8, SUC_lexicon_tags_25 },
    { 0xec143ab9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9d43abc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x93543abe, SUC_lexicon_tags_11 },
    { 0x22943abe, SUC_lexicon_tags_17 },
    { 0x4bb43ac0, SUC_lexicon_tags_65 },
    { 0xc6f43abe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53b43ac3, SUC_lexicon_tags_37 },
    { 0xed343ac4, SUC_lexicon_tags_79 },
    { 0x19f43ac5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc543ac9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x743acb, SUC_lexicon_tags_6 },
    { 0x43b43acb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a143ad2, SUC_lexicon_tags_125 },
    { 0xa5b43ad2, SUC_lexicon_tags_37 },
    { 0x19543ad3, SUC_lexicon_tags_1 },
    { 0xb8143ad5, SUC_lexicon_tags_17 },
    { 0x1c543ad5, SUC_lexicon_tags_9 },
    { 0xce343ad5, SUC_lexicon_tags_12 },
    { 0xdeb43ad7, SUC_lexicon_tags_7 },
    { 0xf2743ad9, SUC_lexicon_tags_11 },
    { 0x3d143ad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4943add, SUC_lexicon_tags_37 },
    { 0x8c543add, SUC_lexicon_tags_25 },
    { 0xb9543add, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x83343ae2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x77943ae4, SUC_lexicon_tags_37 },
    { 0x43d43ae5, SUC_lexicon_tags_12 },
    { 0x7cd43ae5, SUC_lexicon_tags_10 },
    { 0x87b43ae7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b43aef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f43af2, SUC_lexicon_tags_63 },
    { 0xf2b43af2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d43af6, SUC_lexicon_tags_39 },
    { 0x40d43af7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32143afa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b43afd, SUC_lexicon_tags_19 },
    { 0x22f43afd, SUC_lexicon_tags_55 },
    { 0x3e143aff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79943b03, SUC_lexicon_tags_1 },
    { 0x2e143b04, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f43b08, SUC_lexicon_tags_2 },
    { 0x33343b09, SUC_lexicon_tags_11 },
    { 0xe1943b08, SUC_lexicon_tags_47 },
    { 0x18b43b0a, SUC_lexicon_tags_37 },
    { 0x9ed43b0c, SUC_lexicon_tags_23 },
    { 0xbe543b0d, SUC_lexicon_tags_0 },
    { 0xb7543b0e, SUC_lexicon_tags_17 },
    { 0xb4743b0e, SUC_lexicon_tags_8 },
    { 0xfc543b0f, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0x90d43b12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f143b17, SUC_lexicon_tags_87 },
    { 0x4db43b17, SUC_lexicon_tags_27 },
    { 0xe4743b18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff43b1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6bd43b20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5943b22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x45d43b24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab43b27, SUC_lexicon_tags_25 },
    { 0x37543b28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd743b33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x35543b36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8df43b38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f43b3d, SUC_lexicon_tags_12 },
    { 0x7af43b3d, SUC_lexicon_tags_21 },
    { 0x33743b3e, SUC_lexicon_tags_25 },
    { 0x49d43b3f, SUC_lexicon_tags_17 },
    { 0x60d43b3f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25543b45, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb8943b47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d43b49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x66b43b4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x10143b4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6943b54, SUC_lexicon_tags_15 },
    { 0x93343b54, SUC_lexicon_tags_33 },
    { 0x50143b56, SUC_lexicon_tags_12 },
    { 0x66b43b57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b43b5a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaed43b5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e543b5f, SUC_lexicon_tags_10 },
    { 0xb9b43b60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5943b62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b343b67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1343b6a, SUC_lexicon_tags_25 },
    { 0x66943b6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99f43b6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x20743b70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ab43b72, SUC_lexicon_tags_1 },
    { 0x53f43b73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75943b7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f43b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7543b85, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x95143b87, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x66d43b89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf43b8e, SUC_lexicon_tags_37 },
    { 0xcb143b8f, SUC_lexicon_tags_30 },
    { 0xcbf43b8f, SUC_lexicon_tags_37 },
    { 0x8e543b90, SUC_lexicon_tags_10 },
    { 0x5d943b91, SUC_lexicon_tags_23 },
    { 0x5f543b93, SUC_lexicon_tags_17 },
    { 0xd7943b92, SUC_lexicon_tags_37 },
    { 0x28143b95, SUC_lexicon_tags_6 },
    { 0x3e143b8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x69943b99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3943b9d, SUC_lexicon_tags_47 },
    { 0x4b343b9e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a143ba5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21143ba9, SUC_lexicon_tags_12 },
    { 0xe1b43baa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b43bb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e943bb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf43bb9, SUC_lexicon_tags_11 },
    { 0xae343bb9, SUC_lexicon_tags_14 },
    { 0xb5d43bbb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a343bbf, SUC_lexicon_tags_6 },
    { 0xd0743bc0, SUC_lexicon_tags_2 },
    { 0x3a943bc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa943bc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30543bc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d43bcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59543bd6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38143bdb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa743bde, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x16943be1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdd743be3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2943be6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x23543be9, SUC_lexicon_tags_37 },
    { 0x2d543bea, SUC_lexicon_tags_66 },
    { 0x7ed43beb, SUC_lexicon_tags_39 },
    { 0x54d43bec, SUC_lexicon_tags_37 },
    { 0x5543bed, SUC_lexicon_tags_29 },
    { 0x8cb43bee, SUC_lexicon_tags_75 },
    { 0x9eb43bee, SUC_lexicon_tags_7 },
    { 0x60343bf0, SUC_lexicon_tags_19 },
    { 0x6b543be9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d43bf4, SUC_lexicon_tags_45 },
    { 0xe4f43bf4, SUC_lexicon_tags_11 },
    { 0xcb143bf6, SUC_lexicon_tags_15 },
    { 0x2d143bf6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5bb43bf9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f43bff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96943c06, SUC_lexicon_tags_22 },
    { 0x61943c07, SUC_lexicon_tags_37 },
    { 0xd1f43c08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b43c0b, SUC_lexicon_tags_66 },
    { 0x61943c0c, SUC_lexicon_tags_25 },
    { 0xc3543c0d, SUC_lexicon_tags_1 },
    { 0xfe143c0c, SUC_lexicon_tags_7 },
    { 0xcc543c0f, SUC_lexicon_tags_7 },
    { 0x6e343c10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b43c16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5143c19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e143c1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf943c1e, SUC_lexicon_tags_38 },
    { 0x30943c1e, SUC_lexicon_tags_23 },
    { 0xbbb43c1f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d43c23, SUC_lexicon_tags_0 },
    { 0xf6743c24, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda543c29, SUC_lexicon_tags_45 },
    { 0xed43c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb43c3d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a543c41, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf4143c43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f43c47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78d43c49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8543c50, SUC_lexicon_tags_37 },
    { 0x62b43c50, SUC_lexicon_tags_31 },
    { 0x2e743c52, SUC_lexicon_tags_845 },
    { 0xc6d43c52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62143c59, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d43c64, SUC_lexicon_tags_37 },
    { 0x7f943c65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x69543c68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f743c6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e143c74, SUC_lexicon_tags_29 },
    { 0x1d43c75, SUC_lexicon_tags_11 },
    { 0x84f43c76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3143c78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd7b43c7a, SUC_lexicon_tags_63 },
    { 0x96343c7a, SUC_lexicon_tags_11 },
    { 0x19743c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c143c7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5543c81, SUC_lexicon_tags_10 },
    { 0x26143c82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68543c84, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6cb43c86, SUC_lexicon_tags_46 },
    { 0x11143c87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0543c8d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6743c99, SUC_lexicon_tags_33 },
    { 0x8cf43c9a, SUC_lexicon_tags_45 },
    { 0x7bd43c9b, SUC_lexicon_tags_10 },
    { 0x8ab43c9c, SUC_lexicon_tags_12 },
    { 0xc2743c9d, SUC_lexicon_tags_670 },
    { 0x9543c9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b43ca1, SUC_lexicon_tags_25 },
    { 0xb9343ca2, SUC_lexicon_tags_27 },
    { 0x30543ca2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff143ca5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2543ca8, SUC_lexicon_tags_17 },
    { 0x4c743ca9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b43cac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49543cae, SUC_lexicon_tags_30 },
    { 0xedf43caf, SUC_lexicon_tags_11 },
    { 0xa2143cb0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf43cb4, SUC_lexicon_tags_68 },
    { 0xd0543cb5, SUC_lexicon_tags_19 },
    { 0x23143cb6, SUC_lexicon_tags_14 },
    { 0xec143cb5, SUC_lexicon_tags_14 },
    { 0xe9943cb8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79743cc1, SUC_lexicon_tags_30 },
    { 0xfa743cc1, SUC_lexicon_tags_12 },
    { 0x7dd43cc3, SUC_lexicon_tags_11 },
    { 0xe7943cc4, SUC_lexicon_tags_25 },
    { 0xc3f43cc4, SUC_lexicon_tags_16 },
    { 0x74d43cc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79543cca, SUC_lexicon_tags_1 },
    { 0x47743ccb, SUC_lexicon_tags_1 },
    { 0x4f943cca, SUC_lexicon_tags_7 },
    { 0x12543cca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70143cda, SUC_lexicon_tags_12 },
    { 0xf3343cdb, SUC_lexicon_tags_1 },
    { 0xbf943cdb, SUC_lexicon_tags_17 },
    { 0x8db43cdc, SUC_lexicon_tags_37 },
    { 0xefb43cde, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb43ce8, SUC_lexicon_tags_0 },
    { 0x3bf43ce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d43ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89343cf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69143cfb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f43cff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1743d06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d543d08, SUC_lexicon_tags_7 },
    { 0x86543d08, SUC_lexicon_tags_11 },
    { 0xacd43d0a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8d43d0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4343d0e, SUC_lexicon_tags_60 },
    { 0x6df43d0e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x17f43d11, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x30f43d13, SUC_lexicon_tags_87 },
    { 0x2f43d14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb43d17, SUC_lexicon_tags_23 },
    { 0x28543d18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8943d1b, SUC_lexicon_tags_97 },
    { 0xe4343d1c, SUC_lexicon_tags_38 },
    { 0x96543d1d, SUC_lexicon_tags_14 },
    { 0xb9f43d1e, SUC_lexicon_tags_2 },
    { 0xa4343d1f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf543d23, SUC_lexicon_tags_37 },
    { 0x36f43d24, SUC_lexicon_tags_47 },
    { 0xc9743d25, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd8d43d27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7343d2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54543d2e, SUC_lexicon_tags_17 },
    { 0xe3f43d2f, SUC_lexicon_tags_63 },
    { 0x3cd43d30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1743d32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6143d35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd4743d37, SUC_lexicon_tags_14 },
    { 0x8fb43d38, SUC_lexicon_tags_33 },
    { 0x47943d39, SUC_lexicon_tags_43 },
    { 0x9eb43d3a, SUC_lexicon_tags_63 },
    { 0x51343d3b, SUC_lexicon_tags_56 },
    { 0xa6943d37, SUC_lexicon_tags_4 },
    { 0x19543d3d, SUC_lexicon_tags_45 },
    { 0x2eb43d3d, SUC_lexicon_tags_11 },
    { 0xe9f43d3d, SUC_lexicon_tags_1 },
    { 0x76543d40, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd6343d42, SUC_lexicon_tags_14 },
    { 0xd943d42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd43d46, SUC_lexicon_tags_2 },
    { 0x7e143d47, SUC_lexicon_tags_25 },
    { 0x3f543d47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d43d4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df43d4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d43d55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c143d5b, SUC_lexicon_tags_30 },
    { 0xfbb43d5c, SUC_lexicon_tags_19 },
    { 0x4e143d5c, SUC_lexicon_tags_23 },
    { 0xe7f43d5d, SUC_lexicon_tags_63 },
    { 0x6e343d5f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b743d69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6343d6c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57143d71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2143d75, SUC_lexicon_tags_1 },
    { 0x25343d76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb43d79, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f43d7d, SUC_lexicon_tags_15 },
    { 0x73f43d7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9343d81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95143d83, SUC_lexicon_tags_60 },
    { 0xa9743d84, SUC_lexicon_tags_11 },
    { 0xc4543d83, SUC_lexicon_tags_15 },
    { 0xc7743d85, SUC_lexicon_tags_61 },
    { 0x22943d86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1743d89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa343d8d, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75543d91, SUC_lexicon_tags_47 },
    { 0xc0943d92, SUC_lexicon_tags_1 },
    { 0x28f43d92, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b43d98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ff43d9a, SUC_lexicon_tags_27 },
    { 0xc7943d9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d43da0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xff943da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7af43da5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b43daa, SUC_lexicon_tags_179 },
    { 0x10343dab, SUC_lexicon_tags_11 },
    { 0x91743dac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42343dae, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x50543db0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81343db6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x78143db9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d43dc2, SUC_lexicon_tags_47 },
    { 0xb7343dc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b743dc5, SUC_lexicon_tags_2 },
    { 0x77d43dc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc943dc9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b43dcc, SUC_lexicon_tags_6 },
    { 0xdd343dcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b43dd0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a143dd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xceb43dd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa743dd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b43ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3943de1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3f43de3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7943de5, SUC_lexicon_tags_11 },
    { 0xfbf43de5, SUC_lexicon_tags_33 },
    { 0x4eb43de5, SUC_lexicon_tags_11 },
    { 0x6bd43de7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc143dee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbb43df0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b743dfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98f43dfc, SUC_lexicon_tags_17 },
    { 0x33743dfd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f543dff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2943e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f43e0e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x44143e11, SUC_lexicon_tags_7 },
    { 0x4d743e11, SUC_lexicon_tags_30 },
    { 0x1d943e13, SUC_lexicon_tags_15 },
    { 0x84943e13, SUC_lexicon_tags_15 },
    { 0x6b543e14, SUC_lexicon_tags_37 },
    { 0x16743e16, SUC_lexicon_tags_5 },
    { 0x1c343e17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a943e1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5143e25, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb43e28, SUC_lexicon_tags_6 },
    { 0x8b943e29, SUC_lexicon_tags_25 },
    { 0x83943e29, SUC_lexicon_tags_10 },
    { 0x7c343e2b, SUC_lexicon_tags_25 },
    { 0x81143e2b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xba543e2e, SUC_lexicon_tags_7 },
    { 0xa4143e2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14543e35, SUC_lexicon_tags_17 },
    { 0x14b43e36, SUC_lexicon_tags_17 },
    { 0xa9543e37, SUC_lexicon_tags_9 },
    { 0x28943e38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ff43e3a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xabd43e3c, SUC_lexicon_tags_18 },
    { 0x7a743e3d, SUC_lexicon_tags_16 },
    { 0x3543e3e, SUC_lexicon_tags_1 },
    { 0x35d43e3f, SUC_lexicon_tags_10 },
    { 0x5ed43e40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71f43e42, SUC_lexicon_tags_15 },
    { 0x6b743e42, SUC_lexicon_tags_33 },
    { 0x13d43e44, SUC_lexicon_tags_15 },
    { 0xd5143e44, SUC_lexicon_tags_91 },
    { 0x49f43e44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62543e51, SUC_lexicon_tags_32 },
    { 0xeb943e52, SUC_lexicon_tags_37 },
    { 0xa4f43e53, SUC_lexicon_tags_31 },
    { 0x62143e54, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf543e5a, SUC_lexicon_tags_31 },
    { 0x19343e5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd43e5f, SUC_lexicon_tags_26 },
    { 0x20d43e60, SUC_lexicon_tags_15 },
    { 0x15d43e61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x45543e63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f743e6a, SUC_lexicon_tags_37 },
    { 0xdf143e6a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfbf43e6d, SUC_lexicon_tags_46 },
    { 0xaa743e6e, SUC_lexicon_tags_63 },
    { 0xb2943e6f, SUC_lexicon_tags_17 },
    { 0x17f43e70, SUC_lexicon_tags_19 },
    { 0xcdd43e70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73343e73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x69543e75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4143e78, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0xf1f43e7a, SUC_lexicon_tags_12 },
    { 0x4f143e7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8143e83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3343e85, SUC_lexicon_tags_4 },
    { 0x24143e85, SUC_lexicon_tags_21 },
    { 0x2a743e85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75b43e89, SUC_lexicon_tags_29 },
    { 0xb2143e89, SUC_lexicon_tags_35 },
    { 0xd8943e8b, SUC_lexicon_tags_15 },
    { 0x12d43e8b, SUC_lexicon_tags_14 },
    { 0x5d143e8d, SUC_lexicon_tags_48 },
    { 0xd6f43e8d, SUC_lexicon_tags_11 },
    { 0xfbb43e8f, SUC_lexicon_tags_69 },
    { 0x3b143e90, SUC_lexicon_tags_27 },
    { 0xf8343e8d, SUC_lexicon_tags_1 },
    { 0x6df43e92, SUC_lexicon_tags_8 },
    { 0x59d43e93, SUC_lexicon_tags_23 },
    { 0x5a543e93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb943e9d, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e943ea2, SUC_lexicon_tags_61 },
    { 0xb5743ea3, SUC_lexicon_tags_33 },
    { 0x3a943ea4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9943ea9, SUC_lexicon_tags_27 },
    { 0x69b43ea9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd543ead, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x68743eaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb43eb2, SUC_lexicon_tags_1 },
    { 0xe1d43eb3, SUC_lexicon_tags_12 },
    { 0x47d43eb3, SUC_lexicon_tags_37 },
    { 0x6c143eb3, SUC_lexicon_tags_17 },
    { 0xaeb43eb3, SUC_lexicon_tags_11 },
    { 0xb2143eb5, SUC_lexicon_tags_43 },
    { 0x83543eb4, SUC_lexicon_tags_49 },
    { 0x76743eb9, SUC_lexicon_tags_2 },
    { 0x59f43eb4, SUC_lexicon_tags_39 },
    { 0xd7743ebb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2143ebd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab43ec1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67143ec6, SUC_lexicon_tags_22 },
    { 0xec343ec7, SUC_lexicon_tags_25 },
    { 0x86b43ec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b43ecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b43ed1, SUC_lexicon_tags_1 },
    { 0xd2743ed2, SUC_lexicon_tags_19 },
    { 0xa343ed2, SUC_lexicon_tags_111 },
    { 0xd343ed4, SUC_lexicon_tags_8 },
    { 0x5bb43ed5, SUC_lexicon_tags_15 },
    { 0xa9d43ed5, SUC_lexicon_tags_47 },
    { 0x7c143ed7, SUC_lexicon_tags_9 },
    { 0x9f943ed7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae543ee3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x76543ee6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82943ee8, SUC_lexicon_tags_27 },
    { 0x1bb43ee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1db43eeb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb943ef3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61143ef7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb43efa, SUC_lexicon_tags_46 },
    { 0xcb143efb, SUC_lexicon_tags_25 },
    { 0x45343efc, SUC_lexicon_tags_48 },
    { 0x56943efa, SUC_lexicon_tags_1 },
    { 0xb143efe, SUC_lexicon_tags_4 },
    { 0x29543eff, SUC_lexicon_tags_9 },
    { 0x80543eff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc743f03, SUC_lexicon_tags_17 },
    { 0x7743f04, SUC_lexicon_tags_27 },
    { 0x5c943f05, SUC_lexicon_tags_38 },
    { 0xb5143f06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f43f10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd343f14, SUC_lexicon_tags_37 },
    { 0xc3b43f14, SUC_lexicon_tags_25 },
    { 0x38743f16, SUC_lexicon_tags_15 },
    { 0xd7743f17, SUC_lexicon_tags_2 },
    { 0x67343f17, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d43f1b, SUC_lexicon_tags_15 },
    { 0x63b43f1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d43f21, SUC_lexicon_tags_15 },
    { 0xf7743f22, SUC_lexicon_tags_25 },
    { 0x1fb43f23, SUC_lexicon_tags_3 },
    { 0x51543f22, SUC_lexicon_tags_37 },
    { 0xfaf43f21, SUC_lexicon_tags_31 },
    { 0x7a143f22, SUC_lexicon_tags_68 },
    { 0xec343f25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b43f2a, SUC_lexicon_tags_37 },
    { 0x8b943f2b, SUC_lexicon_tags_87 },
    { 0x9343f2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8343f31, SUC_lexicon_tags_63 },
    { 0xb9543f32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a943f35, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef43f38, SUC_lexicon_tags_31 },
    { 0xc6d43f39, SUC_lexicon_tags_10 },
    { 0xf2343f3a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b43f42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6143f46, SUC_lexicon_tags_8 },
    { 0x4dd43f47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a743f4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49143f4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x24f43f51, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd43f55, SUC_lexicon_tags_3 },
    { 0x6d43f56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda143f59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc8343f5b, SUC_lexicon_tags_27 },
    { 0x9cf43f5c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f743f5f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a943f66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab43f70, SUC_lexicon_tags_25 },
    { 0xc4d43f71, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x24b43f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ab43f75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4943f79, SUC_lexicon_tags_31 },
    { 0x9ed43f79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20343f86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff43f8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad943f90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d43f96, SUC_lexicon_tags_17 },
    { 0x90343f97, SUC_lexicon_tags_46 },
    { 0x6d343f96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab43f9b, SUC_lexicon_tags_66 },
    { 0xc2543f9c, SUC_lexicon_tags_63 },
    { 0xd743f9d, SUC_lexicon_tags_15 },
    { 0xa9343f9c, SUC_lexicon_tags_7 },
    { 0xa2543f9e, SUC_lexicon_tags_63 },
    { 0x51343f9d, SUC_lexicon_tags_16 },
    { 0xee143f9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30343fa4, SUC_lexicon_tags_19 },
    { 0x63b43fa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed543fa8, SUC_lexicon_tags_1 },
    { 0xee343fa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f43fae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50f43fb0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdc743fb2, SUC_lexicon_tags_25 },
    { 0x2ad43fb3, SUC_lexicon_tags_80 },
    { 0xba943fb4, SUC_lexicon_tags_1153 },
    { 0xc5b43fb4, SUC_lexicon_tags_28 },
    { 0xa2f43fb6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x78f43fb8, SUC_lexicon_tags_1 },
    { 0x2c343fb9, SUC_lexicon_tags_43 },
    { 0xae943fba, SUC_lexicon_tags_1 },
    { 0x5543fb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45543fc0, SUC_lexicon_tags_11 },
    { 0x64d43fc1, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0xa1d43fc3, SUC_lexicon_tags_29 },
    { 0x88743fc4, SUC_lexicon_tags_170 },
    { 0xa8d43fc4, SUC_lexicon_tags_49 },
    { 0x33f43fc4, SUC_lexicon_tags_25 },
    { 0x6743fc6, SUC_lexicon_tags_2 },
    { 0x7e743fc8, SUC_lexicon_tags_37 },
    { 0xf8743fc9, SUC_lexicon_tags_32 },
    { 0x72543fca, SUC_lexicon_tags_12 },
    { 0x1ed43fcb, SUC_lexicon_tags_70 },
    { 0x28f43fcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0543fcf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd43fd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93543fd4, SUC_lexicon_tags_2 },
    { 0xa4d43fd4, SUC_lexicon_tags_37 },
    { 0xec743fd6, SUC_lexicon_tags_11 },
    { 0xb5743fd7, SUC_lexicon_tags_30 },
    { 0xe4143fd6, SUC_lexicon_tags_60 },
    { 0x9b343fd9, SUC_lexicon_tags_8 },
    { 0xe4743fda, SUC_lexicon_tags_2 },
    { 0x3bb43fdb, SUC_lexicon_tags_84 },
    { 0x9e343fd8, SUC_lexicon_tags_15 },
    { 0xbeb43fdd, SUC_lexicon_tags_29 },
    { 0x22343fd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef43fe2, SUC_lexicon_tags_5 },
    { 0x80743fe3, SUC_lexicon_tags_2 },
    { 0x7f43fe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb43fe7, SUC_lexicon_tags_1 },
    { 0x3fb43fe8, SUC_lexicon_tags_98 },
    { 0x23343fe9, SUC_lexicon_tags_69 },
    { 0xf7d43fe8, SUC_lexicon_tags_39 },
    { 0xd543feb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x79543fee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5fd43ff0, SUC_lexicon_tags_11 },
    { 0x4a343ff1, SUC_lexicon_tags_17 },
    { 0x1ff43ff1, SUC_lexicon_tags_56 },
    { 0xd3d43ff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b43ff7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe9343ff9, SUC_lexicon_tags_23 },
    { 0x29f43ffa, SUC_lexicon_tags_12 },
    { 0xd1343ffa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd44000, SUC_lexicon_tags_27 },
    { 0x47d44001, SUC_lexicon_tags_14 },
    { 0xfe344001, SUC_lexicon_tags_19 },
    { 0x4f44002, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb1f44005, SUC_lexicon_tags_49 },
    { 0xa7344006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d44009, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d4400d, SUC_lexicon_tags_12 },
    { 0x6154400d, SUC_lexicon_tags_25 },
    { 0xe594400f, SUC_lexicon_tags_27 },
    { 0xb254400d, SUC_lexicon_tags_27 },
    { 0x13144011, SUC_lexicon_tags_17 },
    { 0x3d44012, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f344014, SUC_lexicon_tags_13 },
    { 0xdfb44014, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4401a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4134401c, SUC_lexicon_tags_2 },
    { 0x8c94401d, SUC_lexicon_tags_4 },
    { 0x1e94401e, SUC_lexicon_tags_97 },
    { 0x9874401f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1344021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d44025, SUC_lexicon_tags_66 },
    { 0xf7f44026, SUC_lexicon_tags_33 },
    { 0x2c544027, SUC_lexicon_tags_113 },
    { 0x2a944027, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x2254402b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4d94402d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b44030, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d44033, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb4403e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f44042, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90944044, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66144047, SUC_lexicon_tags_1 },
    { 0x6a544047, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe174404d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b44051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6344053, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3944055, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x55944057, SUC_lexicon_tags_9 },
    { 0x84944058, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7334405c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15d4405e, SUC_lexicon_tags_13 },
    { 0x3274405f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8b544061, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3144064, SUC_lexicon_tags_43 },
    { 0xc4f44065, SUC_lexicon_tags_45 },
    { 0xebb44066, SUC_lexicon_tags_21 },
    { 0x4bd44067, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac94406a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8234406c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb744070, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd44073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b44077, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b4407d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d44080, SUC_lexicon_tags_1 },
    { 0xd6b44081, SUC_lexicon_tags_8 },
    { 0x96544081, SUC_lexicon_tags_25 },
    { 0x48144082, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x66744086, SUC_lexicon_tags_14 },
    { 0x97b44087, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd974408a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9674408c, SUC_lexicon_tags_47 },
    { 0xf494408c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd544090, SUC_lexicon_tags_65 },
    { 0x17b44091, SUC_lexicon_tags_44 },
    { 0xd4b44092, SUC_lexicon_tags_7 },
    { 0x1dd44093, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46744095, SUC_lexicon_tags_7 },
    { 0xef144096, SUC_lexicon_tags_37 },
    { 0xa0d44096, SUC_lexicon_tags_77 },
    { 0x9db44096, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af4409b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f94409f, SUC_lexicon_tags_7 },
    { 0x2bf440a0, SUC_lexicon_tags_5 },
    { 0x4eb4409f, SUC_lexicon_tags_38 },
    { 0xd47440a2, SUC_lexicon_tags_17 },
    { 0x65d440a3, SUC_lexicon_tags_11 },
    { 0xc53440a3, SUC_lexicon_tags_7 },
    { 0xc45440a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3440a9, SUC_lexicon_tags_25 },
    { 0x603440a9, SUC_lexicon_tags_37 },
    { 0x477440ab, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad440b0, SUC_lexicon_tags_48 },
    { 0xfc7440b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeef440b3, SUC_lexicon_tags_18 },
    { 0x15440b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x693440b6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d440b9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df440c2, SUC_lexicon_tags_25 },
    { 0x3e5440c2, SUC_lexicon_tags_7 },
    { 0x1b1440c4, SUC_lexicon_tags_82 },
    { 0x5d3440c2, SUC_lexicon_tags_33 },
    { 0xbe7440c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe95440c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77440cc, SUC_lexicon_tags_15 },
    { 0xf4d440cd, SUC_lexicon_tags_15 },
    { 0xb95440ce, SUC_lexicon_tags_45 },
    { 0x5ab440cf, SUC_lexicon_tags_9 },
    { 0xf1f440ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b440d4, SUC_lexicon_tags_58 },
    { 0xe53440d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x791440d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7440da, SUC_lexicon_tags_3 },
    { 0x8a5440db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb440e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc05440e3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3df440e5, SUC_lexicon_tags_32 },
    { 0xee3440e5, SUC_lexicon_tags_61 },
    { 0xf37440e6, SUC_lexicon_tags_15 },
    { 0xd43440e5, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x397440eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x573440ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab440f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3440fd, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x9c9440ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c344102, SUC_lexicon_tags_16 },
    { 0x2a744103, SUC_lexicon_tags_27 },
    { 0x9c944103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f14410b, SUC_lexicon_tags_0 },
    { 0xd4410c, SUC_lexicon_tags_37 },
    { 0x16d4410d, SUC_lexicon_tags_30 },
    { 0x9034410b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x79544111, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf44116, SUC_lexicon_tags_12 },
    { 0x3a144117, SUC_lexicon_tags_46 },
    { 0xf5b44118, SUC_lexicon_tags_10 },
    { 0x9c944118, SUC_lexicon_tags_151 },
    { 0x69944118, SUC_lexicon_tags_37 },
    { 0x4554411b, SUC_lexicon_tags_11 },
    { 0x1e94411c, SUC_lexicon_tags_3 },
    { 0xc94411d, SUC_lexicon_tags_61 },
    { 0x4194411e, SUC_lexicon_tags_12 },
    { 0x54411c, SUC_lexicon_tags_8 },
    { 0x9494411b, SUC_lexicon_tags_31 },
    { 0x44544121, SUC_lexicon_tags_7 },
    { 0x20d44122, SUC_lexicon_tags_1 },
    { 0xf9544117, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99944129, SUC_lexicon_tags_11 },
    { 0x67b44129, SUC_lexicon_tags_11 },
    { 0xb6f4412b, SUC_lexicon_tags_14 },
    { 0x1f144129, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x46b4412e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f44136, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9944138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe874413f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef44143, SUC_lexicon_tags_25 },
    { 0x8e344144, SUC_lexicon_tags_19 },
    { 0x6b344145, SUC_lexicon_tags_33 },
    { 0x5d44145, SUC_lexicon_tags_37 },
    { 0x76744147, SUC_lexicon_tags_8 },
    { 0xf9f44148, SUC_lexicon_tags_15 },
    { 0xdb344149, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3974414d, SUC_lexicon_tags_37 },
    { 0xa394414e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10f44150, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc8144152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33544154, SUC_lexicon_tags_23 },
    { 0xbbf44155, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7544159, SUC_lexicon_tags_0 },
    { 0xe9144159, SUC_lexicon_tags_25 },
    { 0xa2f4415b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb574415d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x12d4415f, SUC_lexicon_tags_10 },
    { 0x42d44160, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb44163, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5894416a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9134416d, SUC_lexicon_tags_33 },
    { 0x3314416e, SUC_lexicon_tags_11 },
    { 0xbf14416d, SUC_lexicon_tags_39 },
    { 0x20144170, SUC_lexicon_tags_25 },
    { 0x43f4416d, SUC_lexicon_tags_15 },
    { 0xa0144172, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x14144174, SUC_lexicon_tags_37 },
    { 0xedb44175, SUC_lexicon_tags_23 },
    { 0x15f44175, SUC_lexicon_tags_37 },
    { 0x5b944176, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3344179, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b44181, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d44185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb44187, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa294418b, SUC_lexicon_tags_15 },
    { 0x3f14418c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f94418e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d44191, SUC_lexicon_tags_60 },
    { 0xa7344192, SUC_lexicon_tags_2 },
    { 0x64544192, SUC_lexicon_tags_23 },
    { 0xc6b44194, SUC_lexicon_tags_26 },
    { 0xffb44195, SUC_lexicon_tags_77 },
    { 0x7c544196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f4419a, SUC_lexicon_tags_11 },
    { 0x23d4419b, SUC_lexicon_tags_15 },
    { 0x3694419c, SUC_lexicon_tags_37 },
    { 0x3534419c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215441a1, SUC_lexicon_tags_12 },
    { 0xe25441a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x477441a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdef441a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd441ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b441af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133441b6, SUC_lexicon_tags_87 },
    { 0xe19441b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b441ba, SUC_lexicon_tags_65 },
    { 0x233441bb, SUC_lexicon_tags_2 },
    { 0xb2f441bb, SUC_lexicon_tags_15 },
    { 0x185441bc, SUC_lexicon_tags_2 },
    { 0x391441bd, SUC_lexicon_tags_23 },
    { 0xf93441bf, SUC_lexicon_tags_1 },
    { 0xd83441bf, SUC_lexicon_tags_25 },
    { 0x197441c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469441ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x845441cc, SUC_lexicon_tags_19 },
    { 0x68b441cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84d441cf, SUC_lexicon_tags_12 },
    { 0x973441d0, SUC_lexicon_tags_17 },
    { 0xedf441d1, SUC_lexicon_tags_42 },
    { 0x589441d2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x937441d5, SUC_lexicon_tags_15 },
    { 0xf81441d6, SUC_lexicon_tags_25 },
    { 0xa61441d6, SUC_lexicon_tags_26 },
    { 0x8df441d7, SUC_lexicon_tags_60 },
    { 0xd3441d9, SUC_lexicon_tags_14 },
    { 0xedb441d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf03441dc, SUC_lexicon_tags_1 },
    { 0x87f441dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc41441df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5441e2, SUC_lexicon_tags_12 },
    { 0x737441e3, SUC_lexicon_tags_7 },
    { 0x269441e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x579441e7, SUC_lexicon_tags_37 },
    { 0x1fd441e7, SUC_lexicon_tags_37 },
    { 0xf65441e9, SUC_lexicon_tags_8 },
    { 0x80d441ea, SUC_lexicon_tags_2 },
    { 0x957441eb, SUC_lexicon_tags_13 },
    { 0x91f441e8, SUC_lexicon_tags_25 },
    { 0x8c7441e8, SUC_lexicon_tags_15 },
    { 0x4a1441ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3441f1, SUC_lexicon_tags_11 },
    { 0xa79441f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f441f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983441fb, SUC_lexicon_tags_60 },
    { 0xe65441fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x74d441fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94b44200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8144202, SUC_lexicon_tags_2 },
    { 0x99744203, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd44206, SUC_lexicon_tags_23 },
    { 0x15744206, SUC_lexicon_tags_43 },
    { 0x58344207, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b4420c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b94420f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95744213, SUC_lexicon_tags_11 },
    { 0xc9144214, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5344217, SUC_lexicon_tags_23 },
    { 0xb1d44218, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37d4421a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15744226, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd44229, SUC_lexicon_tags_8 },
    { 0xef544229, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b4422e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1af44230, SUC_lexicon_tags_25 },
    { 0xec744230, SUC_lexicon_tags_17 },
    { 0x83544232, SUC_lexicon_tags_12 },
    { 0x51f44233, SUC_lexicon_tags_46 },
    { 0x66f44234, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa994423a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30144244, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8744247, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9674424a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d4424d, SUC_lexicon_tags_23 },
    { 0xf934424e, SUC_lexicon_tags_1 },
    { 0x9094424e, SUC_lexicon_tags_15 },
    { 0xb8f44250, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b4425c, SUC_lexicon_tags_11 },
    { 0xe114425d, SUC_lexicon_tags_14 },
    { 0x30b4425e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f44260, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd944267, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7344273, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c744277, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d4427a, SUC_lexicon_tags_29 },
    { 0x1414427b, SUC_lexicon_tags_26 },
    { 0xfb54427c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad34427e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d44281, SUC_lexicon_tags_32 },
    { 0xc9f44282, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc144286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3374428d, SUC_lexicon_tags_12 },
    { 0xff74428e, SUC_lexicon_tags_17 },
    { 0x4254428f, SUC_lexicon_tags_15 },
    { 0x3f744290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0944292, SUC_lexicon_tags_2 },
    { 0x15f44292, SUC_lexicon_tags_7 },
    { 0xbc544293, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53544298, SUC_lexicon_tags_2 },
    { 0x73f44299, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c74429b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xdb54429d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1442a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf442a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd57442aa, SUC_lexicon_tags_12 },
    { 0xac5442ab, SUC_lexicon_tags_37 },
    { 0x4a3442ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd442b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43442b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d442b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x143442ba, SUC_lexicon_tags_13 },
    { 0xfe1442bb, SUC_lexicon_tags_25 },
    { 0x4b1442bc, SUC_lexicon_tags_191 },
    { 0xced442bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff442c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f442c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2f442c8, SUC_lexicon_tags_7 },
    { 0xa41442c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71442cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3442d5, SUC_lexicon_tags_39 },
    { 0xd29442d6, SUC_lexicon_tags_1 },
    { 0x545442d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b442db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f442dd, SUC_lexicon_tags_2 },
    { 0x395442de, SUC_lexicon_tags_17 },
    { 0xcd7442dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35442e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7442ea, SUC_lexicon_tags_15 },
    { 0x79b442eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b442f0, SUC_lexicon_tags_11 },
    { 0x7e5442f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13442f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115442f8, SUC_lexicon_tags_25 },
    { 0xd17442f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf442fc, SUC_lexicon_tags_17 },
    { 0xa67442fd, SUC_lexicon_tags_7 },
    { 0xc93442fe, SUC_lexicon_tags_25 },
    { 0xe9f442ff, SUC_lexicon_tags_17 },
    { 0xc11442ff, SUC_lexicon_tags_27 },
    { 0x3d5442ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f944304, SUC_lexicon_tags_10 },
    { 0xf8744305, SUC_lexicon_tags_11 },
    { 0xcc344306, SUC_lexicon_tags_25 },
    { 0x64744306, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1344309, SUC_lexicon_tags_7 },
    { 0x37d4430a, SUC_lexicon_tags_8 },
    { 0x17944309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b4430e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32344313, SUC_lexicon_tags_49 },
    { 0x94f44313, SUC_lexicon_tags_1 },
    { 0xc9144315, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb44318, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b4431c, SUC_lexicon_tags_7 },
    { 0x1674431d, SUC_lexicon_tags_0 },
    { 0x49b4431e, SUC_lexicon_tags_151 },
    { 0xe214431d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b44323, SUC_lexicon_tags_33 },
    { 0x68944324, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6554432a, SUC_lexicon_tags_37 },
    { 0xb814432b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5144331, SUC_lexicon_tags_29 },
    { 0x90144332, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4544336, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d44339, SUC_lexicon_tags_1 },
    { 0x3944339, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed744340, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7944348, SUC_lexicon_tags_7 },
    { 0x59d44349, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1154434c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd94434f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf2b44351, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa7744353, SUC_lexicon_tags_27 },
    { 0x4e544354, SUC_lexicon_tags_17 },
    { 0xe6144353, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d344357, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a74435b, SUC_lexicon_tags_46 },
    { 0x7f94435c, SUC_lexicon_tags_43 },
    { 0x3e74435d, SUC_lexicon_tags_25 },
    { 0xb914435e, SUC_lexicon_tags_15 },
    { 0x90b4435e, SUC_lexicon_tags_23 },
    { 0xa1d4435e, SUC_lexicon_tags_47 },
    { 0x1234435b, SUC_lexicon_tags_21 },
    { 0x4934435f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xea344364, SUC_lexicon_tags_13 },
    { 0x85344365, SUC_lexicon_tags_37 },
    { 0x27f44365, SUC_lexicon_tags_7 },
    { 0xa9144367, SUC_lexicon_tags_37 },
    { 0x7b44368, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d344378, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6454437a, SUC_lexicon_tags_6 },
    { 0xdbf4437a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f4437e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f44383, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70b44385, SUC_lexicon_tags_11 },
    { 0x92d44386, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa974438b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed44392, SUC_lexicon_tags_15 },
    { 0x21344393, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3744399, SUC_lexicon_tags_25 },
    { 0xdab4439a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b4439d, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d443a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed443a6, SUC_lexicon_tags_37 },
    { 0x48f443a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9443ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd443af, SUC_lexicon_tags_15 },
    { 0xfc5443af, SUC_lexicon_tags_23 },
    { 0x75f443b1, SUC_lexicon_tags_60 },
    { 0xc3f443b2, SUC_lexicon_tags_1 },
    { 0xf9d443b2, SUC_lexicon_tags_11 },
    { 0x9cd443b3, SUC_lexicon_tags_9 },
    { 0x555443b5, SUC_lexicon_tags_29 },
    { 0xc3d443b5, SUC_lexicon_tags_11 },
    { 0x165443b2, SUC_lexicon_tags_23 },
    { 0xd53443b8, SUC_lexicon_tags_19 },
    { 0xb93443b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb7443bb, SUC_lexicon_tags_7 },
    { 0xa05443bb, SUC_lexicon_tags_37 },
    { 0xf75443bb, SUC_lexicon_tags_1661 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7443c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d443c6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7443c9, SUC_lexicon_tags_43 },
    { 0x4ed443ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf443cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d443d1, SUC_lexicon_tags_49 },
    { 0x671443d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabf443d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72d443d6, SUC_lexicon_tags_8 },
    { 0x17b443d7, SUC_lexicon_tags_27 },
    { 0x53443d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37443db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109443e0, SUC_lexicon_tags_29 },
    { 0xcab443e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x231443e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x661443e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9443ea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7f1443ec, SUC_lexicon_tags_37 },
    { 0x5b5443ed, SUC_lexicon_tags_25 },
    { 0x619443ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82f443f0, SUC_lexicon_tags_37 },
    { 0xe07443f1, SUC_lexicon_tags_17 },
    { 0xb17443f2, SUC_lexicon_tags_28 },
    { 0x1b7443f2, SUC_lexicon_tags_60 },
    { 0xb9d443f4, SUC_lexicon_tags_32 },
    { 0x743443f5, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b443f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b443fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb443fe, SUC_lexicon_tags_170 },
    { 0xf0f443ff, SUC_lexicon_tags_45 },
    { 0x3f3443fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb744402, SUC_lexicon_tags_19 },
    { 0xfbb44403, SUC_lexicon_tags_31 },
    { 0x45544404, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed944407, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8354440b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f44417, SUC_lexicon_tags_9 },
    { 0x16d44418, SUC_lexicon_tags_2 },
    { 0xafb44417, SUC_lexicon_tags_7 },
    { 0xf034441a, SUC_lexicon_tags_27 },
    { 0x5a54441a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4441f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41544427, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52b44429, SUC_lexicon_tags_1 },
    { 0x66b4442a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3974442f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31944431, SUC_lexicon_tags_7 },
    { 0x9a344431, SUC_lexicon_tags_26 },
    { 0x93144433, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4d144435, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab4443a, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0xd854443c, SUC_lexicon_tags_37 },
    { 0x33f4443d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f544441, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f144450, SUC_lexicon_tags_7 },
    { 0x17b44450, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91344455, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d44458, SUC_lexicon_tags_2 },
    { 0xc5544458, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5c14445b, SUC_lexicon_tags_19 },
    { 0xc694445b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0344463, SUC_lexicon_tags_47 },
    { 0x1d44464, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29744466, SUC_lexicon_tags_15 },
    { 0x8ed44467, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd3944469, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd44470, SUC_lexicon_tags_17 },
    { 0x7f44470, SUC_lexicon_tags_10 },
    { 0xa2b44472, SUC_lexicon_tags_105 },
    { 0x9df44473, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c344478, SUC_lexicon_tags_30 },
    { 0xb3944478, SUC_lexicon_tags_37 },
    { 0x92f4447a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf954447d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6bf4447f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98744482, SUC_lexicon_tags_15 },
    { 0x3e144483, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5c544485, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f44488, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f4448d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a94448f, SUC_lexicon_tags_27 },
    { 0x7f4448f, SUC_lexicon_tags_30 },
    { 0x89144491, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b44495, SUC_lexicon_tags_372 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a14449a, SUC_lexicon_tags_47 },
    { 0xde54449a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x47d4449d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7134449f, SUC_lexicon_tags_14 },
    { 0x3a9444a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x865444a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d7444a5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01444a9, SUC_lexicon_tags_60 },
    { 0x327444aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab444b3, SUC_lexicon_tags_21 },
    { 0x253444b4, SUC_lexicon_tags_11 },
    { 0xbfd444b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5444ba, SUC_lexicon_tags_17 },
    { 0x47d444bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f444be, SUC_lexicon_tags_45 },
    { 0x4cb444bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x305444c1, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3444c4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227444c9, SUC_lexicon_tags_23 },
    { 0x271444ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff444ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d444d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85b444d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47d444d8, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x743444da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b444e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57f444e6, SUC_lexicon_tags_46 },
    { 0x289444e7, SUC_lexicon_tags_37 },
    { 0x6db444e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe99444ea, SUC_lexicon_tags_27 },
    { 0x753444eb, SUC_lexicon_tags_84 },
    { 0x5dd444ec, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313444f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d444f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x437444fb, SUC_lexicon_tags_4 },
    { 0xbfd444fc, SUC_lexicon_tags_7 },
    { 0x11d444fd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f44503, SUC_lexicon_tags_29 },
    { 0xf1344503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f44507, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1294450b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8694450f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d44513, SUC_lexicon_tags_33 },
    { 0x86544514, SUC_lexicon_tags_43 },
    { 0xff144515, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a144519, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4c14451b, SUC_lexicon_tags_9 },
    { 0x7034451c, SUC_lexicon_tags_33 },
    { 0x5d34451d, SUC_lexicon_tags_19 },
    { 0x20b4451e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75544522, SUC_lexicon_tags_17 },
    { 0xff944523, SUC_lexicon_tags_37 },
    { 0xa4944524, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc294452e, SUC_lexicon_tags_33 },
    { 0xe794452f, SUC_lexicon_tags_5 },
    { 0xc9944530, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3744532, SUC_lexicon_tags_14 },
    { 0x2e544532, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ab44535, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xedf44537, SUC_lexicon_tags_11 },
    { 0x9b944538, SUC_lexicon_tags_23 },
    { 0x95744537, SUC_lexicon_tags_43 },
    { 0x2494453a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7194453c, SUC_lexicon_tags_2 },
    { 0x8d4453c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c344540, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc744548, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc54454e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd44551, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9344554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed544556, SUC_lexicon_tags_179 },
    { 0xb6744557, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb4455a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf554455f, SUC_lexicon_tags_39 },
    { 0xc7344560, SUC_lexicon_tags_11 },
    { 0xf8744561, SUC_lexicon_tags_0 },
    { 0x6c344561, SUC_lexicon_tags_37 },
    { 0xa8d44563, SUC_lexicon_tags_31 },
    { 0x5db44564, SUC_lexicon_tags_29 },
    { 0x90f44564, SUC_lexicon_tags_84 },
    { 0x24d44566, SUC_lexicon_tags_4 },
    { 0xdcb4455f, SUC_lexicon_tags_47 },
    { 0xb0344568, SUC_lexicon_tags_15 },
    { 0x15544569, SUC_lexicon_tags_23 },
    { 0x7d4456a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4bb4456c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa634456e, SUC_lexicon_tags_37 },
    { 0x9274456f, SUC_lexicon_tags_2 },
    { 0x2c744570, SUC_lexicon_tags_66 },
    { 0x7534456f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f44576, SUC_lexicon_tags_37 },
    { 0xb5f44577, SUC_lexicon_tags_33 },
    { 0x86d44578, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3c14457a, SUC_lexicon_tags_2 },
    { 0xa714457b, SUC_lexicon_tags_11 },
    { 0xd654457c, SUC_lexicon_tags_6 },
    { 0x134457d, SUC_lexicon_tags_38 },
    { 0x39f4457a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c344583, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d44586, SUC_lexicon_tags_60 },
    { 0x9fb44587, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd4458d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1874458f, SUC_lexicon_tags_6 },
    { 0xf9944590, SUC_lexicon_tags_2 },
    { 0x9f14458f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b344599, SUC_lexicon_tags_47 },
    { 0x7274459a, SUC_lexicon_tags_25 },
    { 0x50144599, SUC_lexicon_tags_7 },
    { 0xa374459c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975445a2, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d445af, SUC_lexicon_tags_25 },
    { 0x9b1445b0, SUC_lexicon_tags_11 },
    { 0xfc1445af, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3445b5, SUC_lexicon_tags_63 },
    { 0xe4b445b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ad445b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad445ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75445bf, SUC_lexicon_tags_7 },
    { 0x8b1445bf, SUC_lexicon_tags_29 },
    { 0xe69445c1, SUC_lexicon_tags_17 },
    { 0x3b445c2, SUC_lexicon_tags_43 },
    { 0xcc7445c3, SUC_lexicon_tags_34 },
    { 0x4c3445c4, SUC_lexicon_tags_19 },
    { 0xfb5445c5, SUC_lexicon_tags_12 },
    { 0xb93445c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d445cc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbcd445ce, SUC_lexicon_tags_15 },
    { 0x19445cf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1445d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d445d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3445db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03445de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3d7445e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c1445e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d445e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x739445e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d445ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b445f8, SUC_lexicon_tags_28 },
    { 0x9ad445f9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x163445fc, SUC_lexicon_tags_29 },
    { 0x2e3445fd, SUC_lexicon_tags_29 },
    { 0x91f445fc, SUC_lexicon_tags_43 },
    { 0xe5d445ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b44603, SUC_lexicon_tags_1 },
    { 0x16944604, SUC_lexicon_tags_1 },
    { 0x18d44605, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5344607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca74460a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b4460e, SUC_lexicon_tags_1 },
    { 0x2f54460f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10744611, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff744613, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf44618, SUC_lexicon_tags_45 },
    { 0x6cf44618, SUC_lexicon_tags_23 },
    { 0x18744619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c74461d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df44621, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a744628, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb4462a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cd4462c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb44635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa14463a, SUC_lexicon_tags_66 },
    { 0x8534463b, SUC_lexicon_tags_9 },
    { 0x3b94463c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xde94463f, SUC_lexicon_tags_56 },
    { 0x1b74463f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d44649, SUC_lexicon_tags_12 },
    { 0xf944649, SUC_lexicon_tags_36 },
    { 0x19d4464b, SUC_lexicon_tags_19 },
    { 0x5194464c, SUC_lexicon_tags_27 },
    { 0x4b4464a, SUC_lexicon_tags_15 },
    { 0x3a74464e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb44651, SUC_lexicon_tags_15 },
    { 0x91944652, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab44655, SUC_lexicon_tags_2 },
    { 0x41344656, SUC_lexicon_tags_14 },
    { 0x40744655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98744659, SUC_lexicon_tags_25 },
    { 0xc654465a, SUC_lexicon_tags_48 },
    { 0xf894465b, SUC_lexicon_tags_19 },
    { 0xe5b4465c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd144660, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xadb44662, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4466d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7274466f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c144672, SUC_lexicon_tags_25 },
    { 0x144673, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45544677, SUC_lexicon_tags_25 },
    { 0xc0744678, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c34467d, SUC_lexicon_tags_45 },
    { 0xc9d4467d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f344681, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91544685, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbf44687, SUC_lexicon_tags_11 },
    { 0x7f144687, SUC_lexicon_tags_11 },
    { 0xe2d44689, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6734468c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f4468f, SUC_lexicon_tags_23 },
    { 0x4d744690, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b44693, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb944699, SUC_lexicon_tags_7 },
    { 0xae544699, SUC_lexicon_tags_37 },
    { 0x74144699, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x734469d, SUC_lexicon_tags_37 },
    { 0x3054469e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f446a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe49446a5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703446aa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97446af, SUC_lexicon_tags_60 },
    { 0xcab446b0, SUC_lexicon_tags_35 },
    { 0xd3b446b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1446b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaad446ba, SUC_lexicon_tags_11 },
    { 0x625446bb, SUC_lexicon_tags_1 },
    { 0xbc3446bc, SUC_lexicon_tags_15 },
    { 0xc7446bd, SUC_lexicon_tags_9 },
    { 0xed9446be, SUC_lexicon_tags_8 },
    { 0x3c7446bf, SUC_lexicon_tags_27 },
    { 0x723446bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9446c4, SUC_lexicon_tags_66 },
    { 0x29f446c5, SUC_lexicon_tags_43 },
    { 0x2fd446c6, SUC_lexicon_tags_19 },
    { 0x973446c5, SUC_lexicon_tags_0 },
    { 0xf33446c8, SUC_lexicon_tags_1 },
    { 0xec7446c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3446d6, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x37d446d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x925446da, SUC_lexicon_tags_68 },
    { 0x389446da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf7446dd, SUC_lexicon_tags_19 },
    { 0x131446de, SUC_lexicon_tags_37 },
    { 0x95b446df, SUC_lexicon_tags_60 },
    { 0x725446df, SUC_lexicon_tags_12 },
    { 0xfad446e1, SUC_lexicon_tags_11 },
    { 0x56d446e2, SUC_lexicon_tags_2 },
    { 0x90b446e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd446e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767446ed, SUC_lexicon_tags_47 },
    { 0xf05446ee, SUC_lexicon_tags_48 },
    { 0x3ed446ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x341446f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d446f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5bd446f7, SUC_lexicon_tags_49 },
    { 0x3e1446f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1744703, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5fd44705, SUC_lexicon_tags_61 },
    { 0x6744706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71d44708, SUC_lexicon_tags_15 },
    { 0xf7344709, SUC_lexicon_tags_1 },
    { 0xc4f4470a, SUC_lexicon_tags_31 },
    { 0x42544709, SUC_lexicon_tags_30 },
    { 0xa834470c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb44712, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fd44714, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19544716, SUC_lexicon_tags_56 },
    { 0x56f44717, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5344719, SUC_lexicon_tags_7 },
    { 0x32744719, SUC_lexicon_tags_46 },
    { 0x2ab44719, SUC_lexicon_tags_25 },
    { 0x7b74471c, SUC_lexicon_tags_932 },
    { 0xded4471d, SUC_lexicon_tags_37 },
    { 0x77f4471b, SUC_lexicon_tags_2 },
    { 0xb814471f, SUC_lexicon_tags_0 },
    { 0x77b44720, SUC_lexicon_tags_37 },
    { 0xcb14471b, SUC_lexicon_tags_27 },
    { 0xe7344722, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d44728, SUC_lexicon_tags_47 },
    { 0xf5744729, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb9d4472b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b344730, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf44734, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x96b44736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce344738, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2234473a, SUC_lexicon_tags_11 },
    { 0xc0f4473b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9144747, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f144750, SUC_lexicon_tags_47 },
    { 0x21d44751, SUC_lexicon_tags_17 },
    { 0xd3944751, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88744756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce344758, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb944762, SUC_lexicon_tags_7 },
    { 0x1d944762, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad44768, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xea54476b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4476e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99544775, SUC_lexicon_tags_108 },
    { 0xcc744776, SUC_lexicon_tags_16 },
    { 0x5dd44777, SUC_lexicon_tags_63 },
    { 0x1b544775, SUC_lexicon_tags_15 },
    { 0xc3144779, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x9134477b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff44781, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14f44783, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xad944786, SUC_lexicon_tags_5 },
    { 0x52b44786, SUC_lexicon_tags_17 },
    { 0x9ad44786, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f4478c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5344791, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d144794, SUC_lexicon_tags_47 },
    { 0x7bf44795, SUC_lexicon_tags_37 },
    { 0x15944794, SUC_lexicon_tags_17 },
    { 0x90b44797, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df4479d, SUC_lexicon_tags_79 },
    { 0xa994479e, SUC_lexicon_tags_17 },
    { 0xe054479e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x443447a2, SUC_lexicon_tags_8 },
    { 0x7d447a3, SUC_lexicon_tags_0 },
    { 0xb33447a3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf5f447a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495447b0, SUC_lexicon_tags_31 },
    { 0x6f9447b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x471447b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f447b7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5e7447b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f447c0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f447c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335447cb, SUC_lexicon_tags_45 },
    { 0x1fb447cb, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5447d0, SUC_lexicon_tags_0 },
    { 0x391447d1, SUC_lexicon_tags_27 },
    { 0xe2f447d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b447d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5f447da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c9447dc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb447e0, SUC_lexicon_tags_45 },
    { 0x97b447e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f5447e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x563447e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3447ea, SUC_lexicon_tags_11 },
    { 0x8fb447eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa91447ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf3447ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x197447f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x929447f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x619447f6, SUC_lexicon_tags_12 },
    { 0xbcb447f7, SUC_lexicon_tags_7 },
    { 0xc3f447f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf447fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87447ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xae544801, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c144809, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xce74480b, SUC_lexicon_tags_12 },
    { 0x8f4480b, SUC_lexicon_tags_30 },
    { 0x4a94480d, SUC_lexicon_tags_17 },
    { 0x8bb4480d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb44812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c944815, SUC_lexicon_tags_7 },
    { 0xa4744816, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc944819, SUC_lexicon_tags_2 },
    { 0xc3f44819, SUC_lexicon_tags_15 },
    { 0xab14481b, SUC_lexicon_tags_70 },
    { 0xdc34481c, SUC_lexicon_tags_11 },
    { 0xe214481d, SUC_lexicon_tags_43 },
    { 0xc854481b, SUC_lexicon_tags_2 },
    { 0x9b54481f, SUC_lexicon_tags_2 },
    { 0x1a144820, SUC_lexicon_tags_19 },
    { 0x5144821, SUC_lexicon_tags_38 },
    { 0x1b744821, SUC_lexicon_tags_7 },
    { 0xc1d44823, SUC_lexicon_tags_17 },
    { 0x25944824, SUC_lexicon_tags_26 },
    { 0xb494481d, SUC_lexicon_tags_43 },
    { 0x31944826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a944828, SUC_lexicon_tags_12 },
    { 0xfcd44828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1374482b, SUC_lexicon_tags_37 },
    { 0x6594482b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed544834, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86544836, SUC_lexicon_tags_39 },
    { 0xb4944837, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda344839, SUC_lexicon_tags_27 },
    { 0x1df44839, SUC_lexicon_tags_12 },
    { 0x4634483a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9594483e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32b44840, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x23d44842, SUC_lexicon_tags_28 },
    { 0xd3d44843, SUC_lexicon_tags_4 },
    { 0xd9d44843, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b44848, SUC_lexicon_tags_17 },
    { 0xb4d44848, SUC_lexicon_tags_19 },
    { 0x5394484a, SUC_lexicon_tags_12 },
    { 0x44d4484a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4394484d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xab544850, SUC_lexicon_tags_25 },
    { 0x14f44851, SUC_lexicon_tags_37 },
    { 0x8bd44852, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x92144855, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x54944858, SUC_lexicon_tags_60 },
    { 0xa9b44859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccd4485b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd374485d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f44861, SUC_lexicon_tags_54 },
    { 0x8c544861, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed44866, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f4486a, SUC_lexicon_tags_23 },
    { 0x37d4486b, SUC_lexicon_tags_21 },
    { 0x9e14486c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35344872, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f44878, SUC_lexicon_tags_18 },
    { 0xd6f44879, SUC_lexicon_tags_9 },
    { 0xb6744878, SUC_lexicon_tags_37 },
    { 0x6674487b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254487f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42944882, SUC_lexicon_tags_25 },
    { 0xfdd44883, SUC_lexicon_tags_19 },
    { 0x67344883, SUC_lexicon_tags_37 },
    { 0x7ad44885, SUC_lexicon_tags_45 },
    { 0x3d44882, SUC_lexicon_tags_30 },
    { 0x9ed44887, SUC_lexicon_tags_37 },
    { 0x5a744883, SUC_lexicon_tags_37 },
    { 0x1144889, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x88b4488b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33544892, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6df44894, SUC_lexicon_tags_37 },
    { 0x3944895, SUC_lexicon_tags_9 },
    { 0xafd44896, SUC_lexicon_tags_12 },
    { 0xc2144897, SUC_lexicon_tags_15 },
    { 0xb3f44896, SUC_lexicon_tags_14 },
    { 0xfc544899, SUC_lexicon_tags_106 },
    { 0x8a944899, SUC_lexicon_tags_10 },
    { 0x19144897, SUC_lexicon_tags_29 },
    { 0x44f4489c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xc114489f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01448a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1448a7, SUC_lexicon_tags_11 },
    { 0x1f9448a8, SUC_lexicon_tags_47 },
    { 0x649448a7, SUC_lexicon_tags_25 },
    { 0x3df448aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x999448ac, SUC_lexicon_tags_15 },
    { 0x4cf448ad, SUC_lexicon_tags_7 },
    { 0xa55448ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b1448b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7e3448b2, SUC_lexicon_tags_25 },
    { 0x8d7448b2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef448b8, SUC_lexicon_tags_9 },
    { 0x763448b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf448be, SUC_lexicon_tags_31 },
    { 0x8bd448bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb448c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9448cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x377448cf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xac1448d2, SUC_lexicon_tags_17 },
    { 0xb23448d2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81448d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5448dc, SUC_lexicon_tags_11 },
    { 0x59f448dc, SUC_lexicon_tags_37 },
    { 0xe4f448dd, SUC_lexicon_tags_65 },
    { 0x6c3448dc, SUC_lexicon_tags_17 },
    { 0x13448dc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe8b448e2, SUC_lexicon_tags_38 },
    { 0x51448e2, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03448ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f448ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x349448f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905448f8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x497448fb, SUC_lexicon_tags_23 },
    { 0xceb448fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf144900, SUC_lexicon_tags_43 },
    { 0x1c544901, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b44905, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f14490f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd44911, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4014491a, SUC_lexicon_tags_3 },
    { 0x66f4491a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68d4491d, SUC_lexicon_tags_9 },
    { 0x91f4491e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x54344921, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7a944923, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b44928, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fb4492a, SUC_lexicon_tags_12 },
    { 0xc874492a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1744930, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54344935, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15344937, SUC_lexicon_tags_12 },
    { 0xca944938, SUC_lexicon_tags_97 },
    { 0xf7744939, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d94493c, SUC_lexicon_tags_25 },
    { 0xccd4493c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb274493f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12744943, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4a744945, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d4494c, SUC_lexicon_tags_2 },
    { 0x3e74494d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45544950, SUC_lexicon_tags_63 },
    { 0xae144950, SUC_lexicon_tags_1 },
    { 0xca144950, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb1344954, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35344956, SUC_lexicon_tags_15 },
    { 0x72144956, SUC_lexicon_tags_18 },
    { 0x78744958, SUC_lexicon_tags_47 },
    { 0xa1544959, SUC_lexicon_tags_7 },
    { 0xc3744957, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4495f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2744966, SUC_lexicon_tags_3 },
    { 0xfc544967, SUC_lexicon_tags_17 },
    { 0x8c944967, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf894496e, SUC_lexicon_tags_55 },
    { 0x8894496e, SUC_lexicon_tags_29 },
    { 0x41d44970, SUC_lexicon_tags_47 },
    { 0x42b4496f, SUC_lexicon_tags_37 },
    { 0x11744972, SUC_lexicon_tags_43 },
    { 0x8ff44973, SUC_lexicon_tags_15 },
    { 0xb144972, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d44978, SUC_lexicon_tags_25 },
    { 0xcab44979, SUC_lexicon_tags_127 },
    { 0x86d4497a, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe94497e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb44981, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d44986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15b44988, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7c34498a, SUC_lexicon_tags_11 },
    { 0xc4f4498a, SUC_lexicon_tags_15 },
    { 0x34d4498c, SUC_lexicon_tags_17 },
    { 0x1bf4498c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9c94498f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x84544991, SUC_lexicon_tags_15 },
    { 0x94744992, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf714499a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa674499c, SUC_lexicon_tags_25 },
    { 0x2974499c, SUC_lexicon_tags_27 },
    { 0x99f4499c, SUC_lexicon_tags_63 },
    { 0xec94499e, SUC_lexicon_tags_25 },
    { 0xe054499f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57449a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3449a8, SUC_lexicon_tags_38 },
    { 0xdbb449a9, SUC_lexicon_tags_11 },
    { 0xc93449aa, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf5449ac, SUC_lexicon_tags_24 },
    { 0xe33449ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5bd449af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83449b1, SUC_lexicon_tags_1 },
    { 0x199449b2, SUC_lexicon_tags_7 },
    { 0x15d449b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d449b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df449bb, SUC_lexicon_tags_2 },
    { 0x98d449bc, SUC_lexicon_tags_4 },
    { 0x35d449bb, SUC_lexicon_tags_10 },
    { 0xd85449be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5449c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26b449c4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d449c7, SUC_lexicon_tags_38 },
    { 0xf23449c7, SUC_lexicon_tags_1 },
    { 0xe1d449c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e3449cb, SUC_lexicon_tags_11 },
    { 0x673449cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55d449ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f449d2, SUC_lexicon_tags_26 },
    { 0x50b449d3, SUC_lexicon_tags_63 },
    { 0xa8d449d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f449d7, SUC_lexicon_tags_15 },
    { 0x3d5449d8, SUC_lexicon_tags_1 },
    { 0x983449d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501449de, SUC_lexicon_tags_25 },
    { 0x5e3449df, SUC_lexicon_tags_17 },
    { 0xf3b449df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d449e6, SUC_lexicon_tags_15 },
    { 0x37449e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b449eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7449f0, SUC_lexicon_tags_1 },
    { 0x8e1449f1, SUC_lexicon_tags_45 },
    { 0x769449f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b449f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a9449fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2d449fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x169449ff, SUC_lexicon_tags_28 },
    { 0x65344a00, SUC_lexicon_tags_15 },
    { 0xbd344a01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14d44a03, SUC_lexicon_tags_14 },
    { 0xda744a04, SUC_lexicon_tags_1 },
    { 0xf1b44a05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab44a0a, SUC_lexicon_tags_17 },
    { 0xdf44a0a, SUC_lexicon_tags_37 },
    { 0x3c544a0c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb44a0f, SUC_lexicon_tags_65 },
    { 0x28544a10, SUC_lexicon_tags_39 },
    { 0x12b44a11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38344a15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d744a1a, SUC_lexicon_tags_15 },
    { 0x94d44a1b, SUC_lexicon_tags_12 },
    { 0x30f44a1c, SUC_lexicon_tags_87 },
    { 0x80944a1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22744a25, SUC_lexicon_tags_12 },
    { 0xcd144a25, SUC_lexicon_tags_83 },
    { 0xdd944a27, SUC_lexicon_tags_11 },
    { 0x5bd44a28, SUC_lexicon_tags_34 },
    { 0x75544a29, SUC_lexicon_tags_26 },
    { 0x1e544a2a, SUC_lexicon_tags_11 },
    { 0xf9144a26, SUC_lexicon_tags_48 },
    { 0x91944a2c, SUC_lexicon_tags_37 },
    { 0x2b344a2c, SUC_lexicon_tags_2 },
    { 0x86f44a2e, SUC_lexicon_tags_11 },
    { 0x59b44a2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d344a31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde144a35, SUC_lexicon_tags_377 },
    { 0, NULL },
    { 0xcd144a37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4944a3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8744a40, SUC_lexicon_tags_71 },
    { 0x8eb44a40, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1bb44a43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa544a45, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x54944a47, SUC_lexicon_tags_11 },
    { 0x89544a48, SUC_lexicon_tags_12 },
    { 0x17744a48, SUC_lexicon_tags_37 },
    { 0xc4f44a4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6744a4f, SUC_lexicon_tags_47 },
    { 0xfeb44a4f, SUC_lexicon_tags_17 },
    { 0xc5f44a50, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d44a55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b44a58, SUC_lexicon_tags_7 },
    { 0xe7944a59, SUC_lexicon_tags_26 },
    { 0x19f44a58, SUC_lexicon_tags_61 },
    { 0xd9b44a59, SUC_lexicon_tags_60 },
    { 0x54744a5b, SUC_lexicon_tags_25 },
    { 0x43344a5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff44a61, SUC_lexicon_tags_79 },
    { 0xc1744a62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x69944a65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f44a69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4d44a6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf44a74, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd44a78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b44a80, SUC_lexicon_tags_14 },
    { 0x9af44a81, SUC_lexicon_tags_17 },
    { 0xe3544a81, SUC_lexicon_tags_11 },
    { 0xa6f44a81, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x20944a86, SUC_lexicon_tags_26 },
    { 0xdfb44a86, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2344a8c, SUC_lexicon_tags_27 },
    { 0x59b44a8c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36544a92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95144a94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a144a97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf944aa1, SUC_lexicon_tags_1 },
    { 0x97944aa2, SUC_lexicon_tags_25 },
    { 0x47f44aa3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe344aa6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8944aa9, SUC_lexicon_tags_25 },
    { 0xee944aaa, SUC_lexicon_tags_11 },
    { 0xad44aaa, SUC_lexicon_tags_9 },
    { 0x56144aaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df44ab0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c144ab3, SUC_lexicon_tags_11 },
    { 0x63744ab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4144ab6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b44abf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e944ac3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29544ac7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f44acb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11944ad2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38d44ad4, SUC_lexicon_tags_17 },
    { 0x2a744ad4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca744ada, SUC_lexicon_tags_77 },
    { 0x36544ada, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf1344add, SUC_lexicon_tags_30 },
    { 0x79944ade, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3144ae3, SUC_lexicon_tags_11 },
    { 0x2d744ae4, SUC_lexicon_tags_45 },
    { 0x33d44ae5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6ef44ae7, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57744aeb, SUC_lexicon_tags_17 },
    { 0xddb44aec, SUC_lexicon_tags_2 },
    { 0x3544aeb, SUC_lexicon_tags_43 },
    { 0x57d44aec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fb44af0, SUC_lexicon_tags_37 },
    { 0xf3944af0, SUC_lexicon_tags_36 },
    { 0x67f44af2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e544af5, SUC_lexicon_tags_8 },
    { 0xe1b44af6, SUC_lexicon_tags_11 },
    { 0x6c344af7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc544afc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb344afe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23344b03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d44b08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb144b12, SUC_lexicon_tags_45 },
    { 0xcf744b13, SUC_lexicon_tags_37 },
    { 0xbfd44b14, SUC_lexicon_tags_25 },
    { 0x4d344b15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b44b1a, SUC_lexicon_tags_37 },
    { 0xc3544b1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19544b1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47544b21, SUC_lexicon_tags_27 },
    { 0x73b44b22, SUC_lexicon_tags_70 },
    { 0x25d44b23, SUC_lexicon_tags_1 },
    { 0x6144b23, SUC_lexicon_tags_12 },
    { 0x77b44b25, SUC_lexicon_tags_13 },
    { 0x27944b26, SUC_lexicon_tags_11 },
    { 0x73144b21, SUC_lexicon_tags_4 },
    { 0x98d44b28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b44b2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29b44b2f, SUC_lexicon_tags_46 },
    { 0x7c144b30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4144b32, SUC_lexicon_tags_31 },
    { 0x64944b33, SUC_lexicon_tags_107 },
    { 0xf7944b33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb144b3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f44b40, SUC_lexicon_tags_47 },
    { 0xd2544b41, SUC_lexicon_tags_48 },
    { 0x84f44b42, SUC_lexicon_tags_49 },
    { 0x9cd44b42, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b44b4b, SUC_lexicon_tags_15 },
    { 0x4b344b4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26144b4e, SUC_lexicon_tags_37 },
    { 0x6cb44b4f, SUC_lexicon_tags_65 },
    { 0x67f44b4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93d44b52, SUC_lexicon_tags_23 },
    { 0xb8544b53, SUC_lexicon_tags_11 },
    { 0x43d44b54, SUC_lexicon_tags_6 },
    { 0xa6344b55, SUC_lexicon_tags_0 },
    { 0xdf544b52, SUC_lexicon_tags_17 },
    { 0x6f144b57, SUC_lexicon_tags_47 },
    { 0x14544b58, SUC_lexicon_tags_37 },
    { 0x8e944b55, SUC_lexicon_tags_43 },
    { 0x60344b5a, SUC_lexicon_tags_30 },
    { 0x61d44b5b, SUC_lexicon_tags_36 },
    { 0x6cb44b5c, SUC_lexicon_tags_12 },
    { 0xe3d44b5d, SUC_lexicon_tags_31 },
    { 0x49d44b5a, SUC_lexicon_tags_15 },
    { 0xfad44b5f, SUC_lexicon_tags_60 },
    { 0x6f744b54, SUC_lexicon_tags_12 },
    { 0x46944b5e, SUC_lexicon_tags_11 },
    { 0x2c144b55, SUC_lexicon_tags_11 },
    { 0x3e944b63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f144b6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1744b6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b44b72, SUC_lexicon_tags_15 },
    { 0x8cd44b72, SUC_lexicon_tags_49 },
    { 0x2f944b74, SUC_lexicon_tags_45 },
    { 0xb1544b72, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce344b80, SUC_lexicon_tags_27 },
    { 0x20744b81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7544b84, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d744b88, SUC_lexicon_tags_12 },
    { 0xbd144b89, SUC_lexicon_tags_37 },
    { 0x2d744b89, SUC_lexicon_tags_2 },
    { 0x29544b8a, SUC_lexicon_tags_39 },
    { 0x5e544b8c, SUC_lexicon_tags_56 },
    { 0xd8344b8c, SUC_lexicon_tags_689 },
    { 0x15544b8c, SUC_lexicon_tags_38 },
    { 0xbc344b8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74944b98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f44b9d, SUC_lexicon_tags_2 },
    { 0x1744b9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x40144ba1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f44ba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96144ba8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2e544baa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22744bac, SUC_lexicon_tags_12 },
    { 0xd44bad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa3f44baf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39944bb4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe3944bb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb144bbb, SUC_lexicon_tags_107 },
    { 0xe0744bbc, SUC_lexicon_tags_12 },
    { 0x25744bbd, SUC_lexicon_tags_66 },
    { 0x20544bbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2544bc1, SUC_lexicon_tags_15 },
    { 0x49144bc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x36744bc4, SUC_lexicon_tags_17 },
    { 0xc6744bc5, SUC_lexicon_tags_29 },
    { 0x47b44bc4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95344bcf, SUC_lexicon_tags_1 },
    { 0x11744bd0, SUC_lexicon_tags_78 },
    { 0x9b344bd1, SUC_lexicon_tags_37 },
    { 0x62344bd1, SUC_lexicon_tags_26 },
    { 0xb1944bd3, SUC_lexicon_tags_1 },
    { 0xa344bd4, SUC_lexicon_tags_68 },
    { 0xc9d44bd0, SUC_lexicon_tags_45 },
    { 0x79544bd6, SUC_lexicon_tags_2 },
    { 0xcf344bd5, SUC_lexicon_tags_1 },
    { 0x1fb44bd8, SUC_lexicon_tags_7 },
    { 0x96f44bd9, SUC_lexicon_tags_17 },
    { 0x1fd44bda, SUC_lexicon_tags_25 },
    { 0x9d744bdb, SUC_lexicon_tags_39 },
    { 0xdf744bda, SUC_lexicon_tags_7 },
    { 0x7af44bdb, SUC_lexicon_tags_45 },
    { 0x8b544bde, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb44be1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba744be8, SUC_lexicon_tags_15 },
    { 0x2cf44be9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f44bf0, SUC_lexicon_tags_17 },
    { 0xadd44bf1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d44bf4, SUC_lexicon_tags_27 },
    { 0x6c144bf4, SUC_lexicon_tags_12 },
    { 0x65f44bf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d44bf9, SUC_lexicon_tags_1 },
    { 0x42144bfa, SUC_lexicon_tags_25 },
    { 0xdd44bfb, SUC_lexicon_tags_17 },
    { 0x89d44bfb, SUC_lexicon_tags_7 },
    { 0x53f44bfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c144c00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce344c02, SUC_lexicon_tags_14 },
    { 0xda344c03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd44c0a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d44c11, SUC_lexicon_tags_2 },
    { 0xb7344c12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88344c19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1c344c1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b44c1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7144c21, SUC_lexicon_tags_9 },
    { 0x4eb44c22, SUC_lexicon_tags_47 },
    { 0xe6f44c23, SUC_lexicon_tags_1 },
    { 0xd7944c23, SUC_lexicon_tags_28 },
    { 0xaef44c25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36544c2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb44c32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9544c34, SUC_lexicon_tags_66 },
    { 0xdaf44c35, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b44c39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x73744c3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x24944c3e, SUC_lexicon_tags_15 },
    { 0xa544c3f, SUC_lexicon_tags_27 },
    { 0x58744c3e, SUC_lexicon_tags_37 },
    { 0x55944c41, SUC_lexicon_tags_11 },
    { 0x16b44c3e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf8d44c44, SUC_lexicon_tags_60 },
    { 0xf3344c45, SUC_lexicon_tags_25 },
    { 0x19344c44, SUC_lexicon_tags_40 },
    { 0xf0b44c47, SUC_lexicon_tags_1 },
    { 0xec344c48, SUC_lexicon_tags_37 },
    { 0x1d944c48, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf8f44c4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce144c4e, SUC_lexicon_tags_37 },
    { 0x7c144c4e, SUC_lexicon_tags_47 },
    { 0xe7744c50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8144c53, SUC_lexicon_tags_10 },
    { 0x1a544c53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b144c57, SUC_lexicon_tags_49 },
    { 0xb3b44c57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52544c5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e344c61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c344c64, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38744c68, SUC_lexicon_tags_17 },
    { 0x4544c69, SUC_lexicon_tags_15 },
    { 0x7f544c6a, SUC_lexicon_tags_25 },
    { 0x87944c69, SUC_lexicon_tags_12 },
    { 0x2a544c68, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd44c70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92b44c72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55b44c74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b144c77, SUC_lexicon_tags_11 },
    { 0xbeb44c78, SUC_lexicon_tags_7 },
    { 0x21d44c79, SUC_lexicon_tags_118 },
    { 0x1e544c78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34944c7e, SUC_lexicon_tags_12 },
    { 0x8544c7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7544c81, SUC_lexicon_tags_0 },
    { 0xed344c82, SUC_lexicon_tags_63 },
    { 0x6ff44c83, SUC_lexicon_tags_48 },
    { 0x80144c84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13944c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c544c8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b44c8e, SUC_lexicon_tags_15 },
    { 0xa2544c8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83b44c91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c544c95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdb344c97, SUC_lexicon_tags_38 },
    { 0x24f44c98, SUC_lexicon_tags_30 },
    { 0x59344c97, SUC_lexicon_tags_25 },
    { 0x2c744c9a, SUC_lexicon_tags_55 },
    { 0x2d344c9b, SUC_lexicon_tags_14 },
    { 0x85344c99, SUC_lexicon_tags_1 },
    { 0x9e344c9d, SUC_lexicon_tags_11 },
    { 0xd744c9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f544ca3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3944ca6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47944ca8, SUC_lexicon_tags_23 },
    { 0xc9b44ca9, SUC_lexicon_tags_27 },
    { 0xb7944caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5344cb1, SUC_lexicon_tags_92 },
    { 0x75944cb2, SUC_lexicon_tags_7 },
    { 0x8a344cb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7144cb8, SUC_lexicon_tags_74 },
    { 0xbb944cb9, SUC_lexicon_tags_113 },
    { 0xb5744cba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f44cbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9144cc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94944cc3, SUC_lexicon_tags_7 },
    { 0xfbf44cc3, SUC_lexicon_tags_12 },
    { 0x9d44cc3, SUC_lexicon_tags_14 },
    { 0x5e344cc6, SUC_lexicon_tags_25 },
    { 0x1b944cc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6d44cc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4544ccb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f44cce, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x30344cd0, SUC_lexicon_tags_25 },
    { 0xa944cd1, SUC_lexicon_tags_8 },
    { 0xcfd44cd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa344cd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f344cd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f44cda, SUC_lexicon_tags_17 },
    { 0x49544cdb, SUC_lexicon_tags_1 },
    { 0x37744cdc, SUC_lexicon_tags_17 },
    { 0xbd144cdd, SUC_lexicon_tags_10 },
    { 0xded44cdd, SUC_lexicon_tags_17 },
    { 0x35f44cdd, SUC_lexicon_tags_7 },
    { 0xabf44cdf, SUC_lexicon_tags_35 },
    { 0xf7544ce1, SUC_lexicon_tags_65 },
    { 0x35f44ce2, SUC_lexicon_tags_13 },
    { 0xa4144cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4b44ce5, SUC_lexicon_tags_36 },
    { 0xb744ce6, SUC_lexicon_tags_46 },
    { 0xe4144ce5, SUC_lexicon_tags_25 },
    { 0x1df44ce8, SUC_lexicon_tags_18 },
    { 0x81944ce9, SUC_lexicon_tags_11 },
    { 0x57144cea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92144cee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f544cf3, SUC_lexicon_tags_26 },
    { 0xb4d44cf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d44cf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62544cfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef44cff, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f44d05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf44d09, SUC_lexicon_tags_11 },
    { 0x1b944d0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac744d0c, SUC_lexicon_tags_12 },
    { 0x9f144d0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c744d12, SUC_lexicon_tags_100 },
    { 0xbaf44d13, SUC_lexicon_tags_17 },
    { 0x90344d14, SUC_lexicon_tags_33 },
    { 0xecb44d15, SUC_lexicon_tags_7 },
    { 0xcf44d16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x28744d19, SUC_lexicon_tags_19 },
    { 0xfdf44d19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa0f44d1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9344d21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14344d23, SUC_lexicon_tags_38 },
    { 0x87544d24, SUC_lexicon_tags_28 },
    { 0x7144d25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd44d28, SUC_lexicon_tags_25 },
    { 0xabd44d29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9144d2e, SUC_lexicon_tags_17 },
    { 0x53344d2f, SUC_lexicon_tags_1 },
    { 0x42744d30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71544d33, SUC_lexicon_tags_67 },
    { 0xeb944d34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ff44d36, SUC_lexicon_tags_29 },
    { 0x88f44d37, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x72944d3a, SUC_lexicon_tags_38 },
    { 0x62744d3b, SUC_lexicon_tags_29 },
    { 0x94944d3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d44d40, SUC_lexicon_tags_68 },
    { 0xc8544d41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b44d44, SUC_lexicon_tags_25 },
    { 0x19f44d44, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x12944d48, SUC_lexicon_tags_66 },
    { 0xf9144d48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4f44d4b, SUC_lexicon_tags_1 },
    { 0xe5f44d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50144d52, SUC_lexicon_tags_46 },
    { 0xaf344d53, SUC_lexicon_tags_15 },
    { 0x89144d53, SUC_lexicon_tags_37 },
    { 0xc3d44d52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42144d57, SUC_lexicon_tags_25 },
    { 0x3d144d58, SUC_lexicon_tags_17 },
    { 0xcb544d57, SUC_lexicon_tags_48 },
    { 0xb9b44d59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x28544d5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa144d60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53544d65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70b44d67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a344d69, SUC_lexicon_tags_17 },
    { 0xe5944d6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d144d6d, SUC_lexicon_tags_29 },
    { 0xe4544d6e, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98544d77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17744d7b, SUC_lexicon_tags_0 },
    { 0x12d44d7b, SUC_lexicon_tags_64 },
    { 0xf0744d7b, SUC_lexicon_tags_17 },
    { 0xdcb44d7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b144d81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2944d87, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xaa544d89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63f44d8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d44d8f, SUC_lexicon_tags_66 },
    { 0x92d44d90, SUC_lexicon_tags_7 },
    { 0xe1544d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f344d95, SUC_lexicon_tags_17 },
    { 0x76744d95, SUC_lexicon_tags_45 },
    { 0x1db44d97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e144d9c, SUC_lexicon_tags_45 },
    { 0x1f544d9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2b44d9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b44da3, SUC_lexicon_tags_7 },
    { 0xb0744da3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf3f44da6, SUC_lexicon_tags_25 },
    { 0xf1744da7, SUC_lexicon_tags_25 },
    { 0x5e344da8, SUC_lexicon_tags_21 },
    { 0xe8144da8, SUC_lexicon_tags_11 },
    { 0x33544da8, SUC_lexicon_tags_2 },
    { 0xcd144dab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd44daf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b344db4, SUC_lexicon_tags_402 },
    { 0x84344db5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f344db7, SUC_lexicon_tags_8 },
    { 0x51744db8, SUC_lexicon_tags_55 },
    { 0xf2944db9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54944dc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87344dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73144dcd, SUC_lexicon_tags_37 },
    { 0xc1144dce, SUC_lexicon_tags_11 },
    { 0xd8544dcf, SUC_lexicon_tags_7 },
    { 0x76d44dd0, SUC_lexicon_tags_19 },
    { 0x68944dd1, SUC_lexicon_tags_11 },
    { 0x1db44dd2, SUC_lexicon_tags_45 },
    { 0x2b944dd2, SUC_lexicon_tags_82 },
    { 0x28f44dd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b44dd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26b44ddb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa6544ddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0b44ddf, SUC_lexicon_tags_46 },
    { 0x4c544de0, SUC_lexicon_tags_1 },
    { 0xdfb44de1, SUC_lexicon_tags_13 },
    { 0x17944de2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b44de5, SUC_lexicon_tags_30 },
    { 0x91144de6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc544dec, SUC_lexicon_tags_7 },
    { 0xf2144ded, SUC_lexicon_tags_1 },
    { 0xc8744dee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5544df1, SUC_lexicon_tags_2 },
    { 0xe7744df2, SUC_lexicon_tags_11 },
    { 0x57944df2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af44dfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b744dff, SUC_lexicon_tags_4 },
    { 0x11f44dff, SUC_lexicon_tags_1 },
    { 0xa7b44e01, SUC_lexicon_tags_63 },
    { 0x87f44e01, SUC_lexicon_tags_2 },
    { 0xa0d44e03, SUC_lexicon_tags_16 },
    { 0xaf44e04, SUC_lexicon_tags_2 },
    { 0x81b44e02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46144e07, SUC_lexicon_tags_11 },
    { 0x3a944e08, SUC_lexicon_tags_46 },
    { 0x71f44e07, SUC_lexicon_tags_11 },
    { 0xad44e07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a744e0c, SUC_lexicon_tags_37 },
    { 0xb7d44e0d, SUC_lexicon_tags_6 },
    { 0x2f544e0e, SUC_lexicon_tags_37 },
    { 0x60d44e0f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x25f44e11, SUC_lexicon_tags_17 },
    { 0xdd744e11, SUC_lexicon_tags_25 },
    { 0x22544e13, SUC_lexicon_tags_25 },
    { 0x33744e12, SUC_lexicon_tags_27 },
    { 0x84744e14, SUC_lexicon_tags_1 },
    { 0xfc144e16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b44e19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6544e1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d44e22, SUC_lexicon_tags_99 },
    { 0xb5544e22, SUC_lexicon_tags_8 },
    { 0x76d44e23, SUC_lexicon_tags_12 },
    { 0xb2544e25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77944e27, SUC_lexicon_tags_25 },
    { 0xa6f44e28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb44e2b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b344e2e, SUC_lexicon_tags_2 },
    { 0x51344e2f, SUC_lexicon_tags_31 },
    { 0x944e2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42344e33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f44e39, SUC_lexicon_tags_14 },
    { 0x8e944e3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdff44e3c, SUC_lexicon_tags_35 },
    { 0x82f44e3c, SUC_lexicon_tags_12 },
    { 0x1ed44e3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe144e41, SUC_lexicon_tags_17 },
    { 0x68d44e42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65944e48, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b544e4a, SUC_lexicon_tags_11 },
    { 0x1f144e4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96344e4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x97b44e4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f44e52, SUC_lexicon_tags_29 },
    { 0x6cf44e53, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f344e56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf2944e58, SUC_lexicon_tags_19 },
    { 0xf1344e59, SUC_lexicon_tags_7 },
    { 0x78744e58, SUC_lexicon_tags_11 },
    { 0x9ed44e5b, SUC_lexicon_tags_151 },
    { 0x37344e5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a144e60, SUC_lexicon_tags_49 },
    { 0xbbf44e60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a144e65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59b44e67, SUC_lexicon_tags_1 },
    { 0x30f44e68, SUC_lexicon_tags_2 },
    { 0xcb44e67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e744e6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3144e70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x77d44e72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17f44e74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4744e76, SUC_lexicon_tags_30 },
    { 0xfdd44e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab44e7d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1144e81, SUC_lexicon_tags_38 },
    { 0xc0144e81, SUC_lexicon_tags_29 },
    { 0xb7b44e83, SUC_lexicon_tags_49 },
    { 0xba944e82, SUC_lexicon_tags_12 },
    { 0x56744e82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf44e88, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a344e8e, SUC_lexicon_tags_32 },
    { 0x86b44e8f, SUC_lexicon_tags_46 },
    { 0xf5944e90, SUC_lexicon_tags_45 },
    { 0xb2b44e90, SUC_lexicon_tags_38 },
    { 0x7d544e8e, SUC_lexicon_tags_17 },
    { 0xc2f44e91, SUC_lexicon_tags_7 },
    { 0x1dd44e94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1144e97, SUC_lexicon_tags_2 },
    { 0x3b744e97, SUC_lexicon_tags_98 },
    { 0xed44e99, SUC_lexicon_tags_6 },
    { 0xfa344e9a, SUC_lexicon_tags_11 },
    { 0x21544e9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79744e9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f44ea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea144ea5, SUC_lexicon_tags_15 },
    { 0xfd44ea6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e544ea8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7ed44eaa, SUC_lexicon_tags_27 },
    { 0x82b44eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e744eae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b44eb1, SUC_lexicon_tags_11 },
    { 0x5944eb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96544eb5, SUC_lexicon_tags_19 },
    { 0x43144eb6, SUC_lexicon_tags_26 },
    { 0xaef44eb6, SUC_lexicon_tags_11 },
    { 0xbd344eb8, SUC_lexicon_tags_14 },
    { 0x2fb44eb6, SUC_lexicon_tags_2 },
    { 0x30144eba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66144ebe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca144ec2, SUC_lexicon_tags_28 },
    { 0x27d44ec3, SUC_lexicon_tags_37 },
    { 0xc544ec3, SUC_lexicon_tags_7 },
    { 0x69344ec5, SUC_lexicon_tags_1 },
    { 0x8f144ec4, SUC_lexicon_tags_19 },
    { 0x11144ec7, SUC_lexicon_tags_6 },
    { 0xc944ec7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b44ecd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f44ed2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae344ed5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d44edb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b44ee6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd144eea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f44eed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14144ef1, SUC_lexicon_tags_33 },
    { 0x70544ef2, SUC_lexicon_tags_38 },
    { 0x88b44ef3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67944ef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d544efc, SUC_lexicon_tags_367 },
    { 0x9bd44efc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1944f01, SUC_lexicon_tags_4 },
    { 0x66344f02, SUC_lexicon_tags_6 },
    { 0x61944f03, SUC_lexicon_tags_2 },
    { 0x48d44f04, SUC_lexicon_tags_28 },
    { 0xc3f44f05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48b44f07, SUC_lexicon_tags_18 },
    { 0xde744f07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d44f0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b44f11, SUC_lexicon_tags_31 },
    { 0xdb944f12, SUC_lexicon_tags_26 },
    { 0xa0344f12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xddf44f15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0144f22, SUC_lexicon_tags_1000 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52544f27, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e344f2a, SUC_lexicon_tags_15 },
    { 0x68b44f2a, SUC_lexicon_tags_25 },
    { 0x72f44f2c, SUC_lexicon_tags_32 },
    { 0xdf144f2d, SUC_lexicon_tags_11 },
    { 0x4ab44f2b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4f744f30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6344f33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17f44f35, SUC_lexicon_tags_63 },
    { 0x6b944f36, SUC_lexicon_tags_11 },
    { 0x2f544f36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f44f3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75744f3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd44f41, SUC_lexicon_tags_2 },
    { 0x1c144f42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc544f49, SUC_lexicon_tags_19 },
    { 0x40944f4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23944f52, SUC_lexicon_tags_60 },
    { 0x82544f53, SUC_lexicon_tags_25 },
    { 0xb6744f54, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79144f59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14744f5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c144f63, SUC_lexicon_tags_27 },
    { 0x61544f64, SUC_lexicon_tags_17 },
    { 0xe1144f65, SUC_lexicon_tags_32 },
    { 0x92744f66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f44f6a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa4544f6c, SUC_lexicon_tags_19 },
    { 0x89f44f6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xba544f70, SUC_lexicon_tags_17 },
    { 0xe8f44f71, SUC_lexicon_tags_60 },
    { 0x85544f72, SUC_lexicon_tags_7 },
    { 0xb8544f72, SUC_lexicon_tags_37 },
    { 0xdc944f74, SUC_lexicon_tags_13 },
    { 0xef744f71, SUC_lexicon_tags_79 },
    { 0x1944f72, SUC_lexicon_tags_113 },
    { 0x24144f77, SUC_lexicon_tags_1 },
    { 0xe4b44f77, SUC_lexicon_tags_47 },
    { 0x51544f79, SUC_lexicon_tags_45 },
    { 0xd7344f73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae344f7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad44f84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x94544f87, SUC_lexicon_tags_2 },
    { 0xae344f88, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa3f44f8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xed944f8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41144f91, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xac944f94, SUC_lexicon_tags_15 },
    { 0x9e344f95, SUC_lexicon_tags_2 },
    { 0x34d44f95, SUC_lexicon_tags_45 },
    { 0x26b44f97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28744f9d, SUC_lexicon_tags_12 },
    { 0x78d44f9d, SUC_lexicon_tags_7 },
    { 0x1b144f9f, SUC_lexicon_tags_47 },
    { 0x92944fa0, SUC_lexicon_tags_17 },
    { 0xa1944fa0, SUC_lexicon_tags_106 },
    { 0xae944fa2, SUC_lexicon_tags_20 },
    { 0xb344f9d, SUC_lexicon_tags_27 },
    { 0xa9144fa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d744fa9, SUC_lexicon_tags_27 },
    { 0xe4b44faa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb44fad, SUC_lexicon_tags_11 },
    { 0xded44fae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f44fba, SUC_lexicon_tags_38 },
    { 0x7c344fbb, SUC_lexicon_tags_55 },
    { 0xa3944fba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7dd44fbe, SUC_lexicon_tags_25 },
    { 0x93144fbe, SUC_lexicon_tags_60 },
    { 0x51144fbe, SUC_lexicon_tags_37 },
    { 0xbf44fbe, SUC_lexicon_tags_12 },
    { 0x54b44fc2, SUC_lexicon_tags_17 },
    { 0xbc744fc3, SUC_lexicon_tags_15 },
    { 0x7bd44fc3, SUC_lexicon_tags_37 },
    { 0xae344fc5, SUC_lexicon_tags_37 },
    { 0x80b44fc5, SUC_lexicon_tags_15 },
    { 0x58344fc7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73544fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45544fcd, SUC_lexicon_tags_33 },
    { 0xc1d44fce, SUC_lexicon_tags_11 },
    { 0x9b344fce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df44fd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44344fdb, SUC_lexicon_tags_43 },
    { 0x99b44fdb, SUC_lexicon_tags_33 },
    { 0x8eb44fdc, SUC_lexicon_tags_14 },
    { 0x6a344fdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d44fe1, SUC_lexicon_tags_27 },
    { 0x6cd44fe1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92344fe6, SUC_lexicon_tags_12 },
    { 0x52744fe7, SUC_lexicon_tags_10 },
    { 0xaf944fe6, SUC_lexicon_tags_171 },
    { 0x2d544fe6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d344fee, SUC_lexicon_tags_23 },
    { 0xb3b44fef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a744ff2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2344ff4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb6544ff6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93144fff, SUC_lexicon_tags_37 },
    { 0x94f45000, SUC_lexicon_tags_33 },
    { 0xb0545001, SUC_lexicon_tags_45 },
    { 0x79f45002, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbad45004, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42345006, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad145011, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89145017, SUC_lexicon_tags_17 },
    { 0xb4b45018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f14501b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7234501d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31145021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92345024, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x63d45026, SUC_lexicon_tags_1 },
    { 0x98145026, SUC_lexicon_tags_6 },
    { 0xe4b45027, SUC_lexicon_tags_37 },
    { 0x89145026, SUC_lexicon_tags_46 },
    { 0x4a94502a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f34502d, SUC_lexicon_tags_28 },
    { 0x9794502d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2345031, SUC_lexicon_tags_63 },
    { 0xc2d45032, SUC_lexicon_tags_23 },
    { 0x7c145033, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda945037, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d4503a, SUC_lexicon_tags_8 },
    { 0xe3d4503b, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d45042, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df45047, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b94504b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc134504e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d745050, SUC_lexicon_tags_55 },
    { 0xf5745050, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40145055, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd45059, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9a74505b, SUC_lexicon_tags_1 },
    { 0xed54505c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd45063, SUC_lexicon_tags_15 },
    { 0x47145063, SUC_lexicon_tags_48 },
    { 0xec345064, SUC_lexicon_tags_47 },
    { 0xadf45066, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9745068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb74506b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d345070, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22345072, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x37345074, SUC_lexicon_tags_60 },
    { 0x3a545074, SUC_lexicon_tags_204 },
    { 0x53745076, SUC_lexicon_tags_45 },
    { 0x49f45077, SUC_lexicon_tags_25 },
    { 0x345077, SUC_lexicon_tags_2 },
    { 0x8a145079, SUC_lexicon_tags_11 },
    { 0x93b45079, SUC_lexicon_tags_14 },
    { 0x4214507b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16b4507d, SUC_lexicon_tags_68 },
    { 0xb994507e, SUC_lexicon_tags_13 },
    { 0x6614507f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d45082, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8945084, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf145087, SUC_lexicon_tags_92 },
    { 0x75f45088, SUC_lexicon_tags_16 },
    { 0xd1b45087, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f945091, SUC_lexicon_tags_7 },
    { 0xf4145091, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87145097, SUC_lexicon_tags_9 },
    { 0x56b45097, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15450a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3450a5, SUC_lexicon_tags_7 },
    { 0xef7450a6, SUC_lexicon_tags_27 },
    { 0x43d450a6, SUC_lexicon_tags_25 },
    { 0x309450a8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x903450ab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55450ae, SUC_lexicon_tags_29 },
    { 0x55450ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb450b2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9db450b4, SUC_lexicon_tags_7 },
    { 0xaf9450b4, SUC_lexicon_tags_25 },
    { 0x907450b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x237450b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3450bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad450c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd450c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x511450c9, SUC_lexicon_tags_11 },
    { 0xadd450c9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7450cd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc0d450cf, SUC_lexicon_tags_37 },
    { 0x311450d0, SUC_lexicon_tags_50 },
    { 0x4a9450d0, SUC_lexicon_tags_6 },
    { 0xae3450d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5b450d4, SUC_lexicon_tags_9 },
    { 0xc3b450d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x773450d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9450db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x333450dd, SUC_lexicon_tags_63 },
    { 0xe27450de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3fd450e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1d1450e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b450ee, SUC_lexicon_tags_1 },
    { 0x573450ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x987450f1, SUC_lexicon_tags_37 },
    { 0xebf450f2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdb3450f4, SUC_lexicon_tags_70 },
    { 0xe4b450f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fd450f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x457450fa, SUC_lexicon_tags_45 },
    { 0x901450fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53450fe, SUC_lexicon_tags_0 },
    { 0xab3450fe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6945102, SUC_lexicon_tags_47 },
    { 0xc4545103, SUC_lexicon_tags_25 },
    { 0xe0545104, SUC_lexicon_tags_23 },
    { 0x1c945105, SUC_lexicon_tags_7 },
    { 0x61545106, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6545109, SUC_lexicon_tags_25 },
    { 0x4154510a, SUC_lexicon_tags_17 },
    { 0x36b4510b, SUC_lexicon_tags_49 },
    { 0x6494510c, SUC_lexicon_tags_82 },
    { 0x6ed4510d, SUC_lexicon_tags_15 },
    { 0x20b4510d, SUC_lexicon_tags_46 },
    { 0xd874510f, SUC_lexicon_tags_11 },
    { 0xbd94510c, SUC_lexicon_tags_7 },
    { 0xbb4510b, SUC_lexicon_tags_27 },
    { 0xbf345112, SUC_lexicon_tags_19 },
    { 0x994510e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x52945115, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xab745117, SUC_lexicon_tags_2 },
    { 0x69345118, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb114511b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6034511f, SUC_lexicon_tags_16 },
    { 0x4df45120, SUC_lexicon_tags_37 },
    { 0x62d45121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d345125, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x19745127, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xd1b45129, SUC_lexicon_tags_34 },
    { 0xa8d45129, SUC_lexicon_tags_22 },
    { 0x17145129, SUC_lexicon_tags_12 },
    { 0xb1f4512c, SUC_lexicon_tags_53 },
    { 0x68745129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41b4512f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a945134, SUC_lexicon_tags_25 },
    { 0x89345135, SUC_lexicon_tags_28 },
    { 0xbb145136, SUC_lexicon_tags_1 },
    { 0x9d45137, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x31745139, SUC_lexicon_tags_2 },
    { 0xc34513a, SUC_lexicon_tags_8 },
    { 0x33b4513b, SUC_lexicon_tags_7 },
    { 0x3bf4513c, SUC_lexicon_tags_25 },
    { 0x6c94513d, SUC_lexicon_tags_14 },
    { 0x84f4513e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15145142, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78d45144, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65545148, SUC_lexicon_tags_48 },
    { 0x7cd45149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27d4514b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c34514e, SUC_lexicon_tags_37 },
    { 0xb534514f, SUC_lexicon_tags_7 },
    { 0x96345150, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f745154, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe545156, SUC_lexicon_tags_66 },
    { 0x34145156, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cf45159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d4515d, SUC_lexicon_tags_37 },
    { 0x46d4515e, SUC_lexicon_tags_0 },
    { 0x2094515f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b45162, SUC_lexicon_tags_12 },
    { 0xec945163, SUC_lexicon_tags_10 },
    { 0xe7545164, SUC_lexicon_tags_15 },
    { 0x9f745163, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6945168, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b145170, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d745174, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47145179, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x9674517b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9474517f, SUC_lexicon_tags_68 },
    { 0xff54517f, SUC_lexicon_tags_49 },
    { 0xc7d4517f, SUC_lexicon_tags_70 },
    { 0x75945180, SUC_lexicon_tags_4 },
    { 0xdd945181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f145185, SUC_lexicon_tags_39 },
    { 0x85345185, SUC_lexicon_tags_29 },
    { 0xba145187, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b54518c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc994518f, SUC_lexicon_tags_17 },
    { 0x29145190, SUC_lexicon_tags_4 },
    { 0xf4f45191, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4945199, SUC_lexicon_tags_17 },
    { 0x93b4519a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7674519d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231451a2, SUC_lexicon_tags_27 },
    { 0x9a5451a2, SUC_lexicon_tags_12 },
    { 0x271451a4, SUC_lexicon_tags_2 },
    { 0x917451a4, SUC_lexicon_tags_35 },
    { 0x1f3451a6, SUC_lexicon_tags_16 },
    { 0xac5451a7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67451aa, SUC_lexicon_tags_38 },
    { 0xc0d451ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xef1451ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf451b1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb451b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x533451b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41451b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b451bc, SUC_lexicon_tags_23 },
    { 0xe3f451bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817451c2, SUC_lexicon_tags_43 },
    { 0x407451c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9451c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1451cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773451d1, SUC_lexicon_tags_6 },
    { 0x677451d2, SUC_lexicon_tags_17 },
    { 0xcbb451d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75451d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf63451d9, SUC_lexicon_tags_14 },
    { 0xbab451d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6b7451dc, SUC_lexicon_tags_7 },
    { 0xe07451dd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x67d451df, SUC_lexicon_tags_0 },
    { 0xb09451e0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb5451e2, SUC_lexicon_tags_31 },
    { 0x513451e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafb451e5, SUC_lexicon_tags_17 },
    { 0xc7d451e6, SUC_lexicon_tags_9 },
    { 0x19d451e7, SUC_lexicon_tags_12 },
    { 0x243451e7, SUC_lexicon_tags_17 },
    { 0x753451e5, SUC_lexicon_tags_1 },
    { 0xd97451e9, SUC_lexicon_tags_17 },
    { 0x3e7451eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49451ef, SUC_lexicon_tags_26 },
    { 0x2c9451f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x249451f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7d451f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x737451f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3451fd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x54b451ff, SUC_lexicon_tags_12 },
    { 0x7d545200, SUC_lexicon_tags_38 },
    { 0xb2945201, SUC_lexicon_tags_287 },
    { 0x2bb45201, SUC_lexicon_tags_17 },
    { 0x8cf45203, SUC_lexicon_tags_45 },
    { 0xe4545204, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b945209, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbd4520b, SUC_lexicon_tags_27 },
    { 0x2ff4520c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd345211, SUC_lexicon_tags_47 },
    { 0x54945212, SUC_lexicon_tags_17 },
    { 0x1b745213, SUC_lexicon_tags_30 },
    { 0xced45214, SUC_lexicon_tags_12 },
    { 0x8c145215, SUC_lexicon_tags_12 },
    { 0x97345216, SUC_lexicon_tags_49 },
    { 0x77345217, SUC_lexicon_tags_14 },
    { 0x2a545218, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd4521d, SUC_lexicon_tags_25 },
    { 0xcdb4521e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xddf45220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8945224, SUC_lexicon_tags_1345 },
    { 0x82545224, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf945228, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf314522d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd45230, SUC_lexicon_tags_1 },
    { 0x39d45231, SUC_lexicon_tags_11 },
    { 0xc2345232, SUC_lexicon_tags_0 },
    { 0x2f545233, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5d745235, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f4523a, SUC_lexicon_tags_0 },
    { 0x40b4523b, SUC_lexicon_tags_33 },
    { 0x9414523c, SUC_lexicon_tags_48 },
    { 0x53d4523d, SUC_lexicon_tags_9 },
    { 0xd814523d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xfe545240, SUC_lexicon_tags_60 },
    { 0x64945241, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf945243, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d145246, SUC_lexicon_tags_1 },
    { 0x9d745246, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x47345249, SUC_lexicon_tags_17 },
    { 0xf9745249, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db4524f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab545254, SUC_lexicon_tags_9 },
    { 0x6bb45255, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda14525b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x79f4525d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6145261, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa945263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff345265, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb54526a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1154526e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13945273, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe1d45275, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef45279, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8534527b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9594527d, SUC_lexicon_tags_26 },
    { 0x8e14527e, SUC_lexicon_tags_37 },
    { 0xbed4527f, SUC_lexicon_tags_1 },
    { 0x8af45280, SUC_lexicon_tags_17 },
    { 0x4614527f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b145283, SUC_lexicon_tags_55 },
    { 0x97345284, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f45288, SUC_lexicon_tags_13 },
    { 0xcb745289, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d94528b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2694528d, SUC_lexicon_tags_37 },
    { 0xdb34528e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8145291, SUC_lexicon_tags_60 },
    { 0x7d945292, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b345295, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3cb45297, SUC_lexicon_tags_37 },
    { 0xbef45298, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8014529a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf774529d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8734529f, SUC_lexicon_tags_14 },
    { 0xf13452a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd452a7, SUC_lexicon_tags_37 },
    { 0xd1452a8, SUC_lexicon_tags_37 },
    { 0x39452a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45452b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1452bb, SUC_lexicon_tags_33 },
    { 0xf19452bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb452c1, SUC_lexicon_tags_37 },
    { 0x36d452c1, SUC_lexicon_tags_17 },
    { 0x1ff452c1, SUC_lexicon_tags_25 },
    { 0xa77452c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b452c7, SUC_lexicon_tags_43 },
    { 0x8eb452c7, SUC_lexicon_tags_23 },
    { 0x707452c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc17452cb, SUC_lexicon_tags_19 },
    { 0xd91452cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f452d2, SUC_lexicon_tags_29 },
    { 0x2d452d3, SUC_lexicon_tags_12 },
    { 0xeeb452d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb452d7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d452dd, SUC_lexicon_tags_7 },
    { 0xb5b452de, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8af452e0, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b452ef, SUC_lexicon_tags_21 },
    { 0x5b9452f0, SUC_lexicon_tags_19 },
    { 0x171452ef, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f452f4, SUC_lexicon_tags_33 },
    { 0x469452f4, SUC_lexicon_tags_87 },
    { 0x987452f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd452fa, SUC_lexicon_tags_0 },
    { 0x5eb452fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb05452fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa97452ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d145301, SUC_lexicon_tags_29 },
    { 0x57145302, SUC_lexicon_tags_12 },
    { 0x33345301, SUC_lexicon_tags_23 },
    { 0xcab45304, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x24145307, SUC_lexicon_tags_27 },
    { 0x4c345308, SUC_lexicon_tags_148 },
    { 0x1f45308, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9934530b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f45311, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb45314, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f45318, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8454531e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10945325, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f4532a, SUC_lexicon_tags_7 },
    { 0xd974532b, SUC_lexicon_tags_37 },
    { 0x9bf4532c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb514532e, SUC_lexicon_tags_17 },
    { 0xebd4532e, SUC_lexicon_tags_17 },
    { 0xd714532e, SUC_lexicon_tags_15 },
    { 0xd4b45331, SUC_lexicon_tags_11 },
    { 0xa7b45332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2545337, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d4533a, SUC_lexicon_tags_13 },
    { 0x7d54533b, SUC_lexicon_tags_14 },
    { 0x3734533c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d4533f, SUC_lexicon_tags_0 },
    { 0x7745340, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0145343, SUC_lexicon_tags_38 },
    { 0xc2f45344, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47745348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b14534b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf4534e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3945355, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46545358, SUC_lexicon_tags_38 },
    { 0x31345358, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b14535b, SUC_lexicon_tags_15 },
    { 0x17d4535c, SUC_lexicon_tags_11 },
    { 0x8bf4535d, SUC_lexicon_tags_27 },
    { 0xd574535b, SUC_lexicon_tags_29 },
    { 0x48f4535f, SUC_lexicon_tags_25 },
    { 0x83b4535d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0945364, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb545367, SUC_lexicon_tags_75 },
    { 0xb5145368, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8414536a, SUC_lexicon_tags_29 },
    { 0x5974536a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf834536e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d745373, SUC_lexicon_tags_29 },
    { 0x1b745373, SUC_lexicon_tags_17 },
    { 0xaa745375, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92145378, SUC_lexicon_tags_47 },
    { 0x8c345378, SUC_lexicon_tags_30 },
    { 0xdf545378, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4df4537c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd45382, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b45387, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab4538f, SUC_lexicon_tags_37 },
    { 0xf4945390, SUC_lexicon_tags_47 },
    { 0xabd45390, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe745397, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2614539a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfc74539c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd453a0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf3b453a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1453a6, SUC_lexicon_tags_2 },
    { 0x363453a6, SUC_lexicon_tags_26 },
    { 0xf3453a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7453ac, SUC_lexicon_tags_17 },
    { 0xd8b453ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf453b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2d5453b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289453b6, SUC_lexicon_tags_38 },
    { 0xec5453b6, SUC_lexicon_tags_11 },
    { 0xd5b453b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b453c0, SUC_lexicon_tags_33 },
    { 0xa33453c1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353453d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff453d5, SUC_lexicon_tags_10 },
    { 0xedb453d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9453d9, SUC_lexicon_tags_11 },
    { 0x501453da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a9453dc, SUC_lexicon_tags_32 },
    { 0x637453dd, SUC_lexicon_tags_18 },
    { 0x315453de, SUC_lexicon_tags_7 },
    { 0x523453df, SUC_lexicon_tags_14 },
    { 0x72b453df, SUC_lexicon_tags_4 },
    { 0x56b453e1, SUC_lexicon_tags_17 },
    { 0x3ef453e0, SUC_lexicon_tags_66 },
    { 0xe31453e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9453e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3b5453e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d453f1, SUC_lexicon_tags_23 },
    { 0x10b453f1, SUC_lexicon_tags_37 },
    { 0x52d453f1, SUC_lexicon_tags_16 },
    { 0x3c9453f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe95453f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8df453f8, SUC_lexicon_tags_25 },
    { 0xf6d453f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb43453fb, SUC_lexicon_tags_17 },
    { 0x683453fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57145402, SUC_lexicon_tags_37 },
    { 0xaf145403, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4545407, SUC_lexicon_tags_49 },
    { 0x7eb45407, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6d94540a, SUC_lexicon_tags_11 },
    { 0xd3b4540b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5474540f, SUC_lexicon_tags_37 },
    { 0xed145410, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b45414, SUC_lexicon_tags_68 },
    { 0x1f745415, SUC_lexicon_tags_33 },
    { 0x67545416, SUC_lexicon_tags_1 },
    { 0x93d45417, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x12545419, SUC_lexicon_tags_33 },
    { 0x6434541a, SUC_lexicon_tags_37 },
    { 0xe8345419, SUC_lexicon_tags_25 },
    { 0x5654541b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd4541f, SUC_lexicon_tags_7 },
    { 0xe3745420, SUC_lexicon_tags_43 },
    { 0x44f4541f, SUC_lexicon_tags_26 },
    { 0xebf45422, SUC_lexicon_tags_1 },
    { 0xae945423, SUC_lexicon_tags_1 },
    { 0xb4945424, SUC_lexicon_tags_25 },
    { 0xecb45425, SUC_lexicon_tags_25 },
    { 0xebf45426, SUC_lexicon_tags_63 },
    { 0x7a945423, SUC_lexicon_tags_28 },
    { 0x73b45428, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda14542b, SUC_lexicon_tags_17 },
    { 0x7734542c, SUC_lexicon_tags_1 },
    { 0xf834542b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25d4542f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f545432, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6145435, SUC_lexicon_tags_15 },
    { 0x91745436, SUC_lexicon_tags_17 },
    { 0x8c145436, SUC_lexicon_tags_80 },
    { 0xb9145437, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd094543c, SUC_lexicon_tags_11 },
    { 0x99d4543c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c745443, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xea945445, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65345456, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b54545c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5945465, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2345467, SUC_lexicon_tags_15 },
    { 0xea945468, SUC_lexicon_tags_4 },
    { 0xae545468, SUC_lexicon_tags_113 },
    { 0xf9f4546a, SUC_lexicon_tags_11 },
    { 0xcdd4546a, SUC_lexicon_tags_27 },
    { 0x1d34546c, SUC_lexicon_tags_226 },
    { 0x3c94546d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f45473, SUC_lexicon_tags_37 },
    { 0x5af45473, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc0745476, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3f45478, SUC_lexicon_tags_14 },
    { 0xd4145478, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd94547f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xbab45481, SUC_lexicon_tags_54 },
    { 0xafd45481, SUC_lexicon_tags_37 },
    { 0x18345483, SUC_lexicon_tags_37 },
    { 0x4145483, SUC_lexicon_tags_12 },
    { 0x28b45484, SUC_lexicon_tags_14 },
    { 0xd5745486, SUC_lexicon_tags_7 },
    { 0x2bb45486, SUC_lexicon_tags_0 },
    { 0x7ad45486, SUC_lexicon_tags_17 },
    { 0x86d45489, SUC_lexicon_tags_12 },
    { 0x44145484, SUC_lexicon_tags_37 },
    { 0x70b45486, SUC_lexicon_tags_21 },
    { 0x14d4548c, SUC_lexicon_tags_33 },
    { 0xbe74548c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa714548f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3345497, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd454a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28d454a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3454a8, SUC_lexicon_tags_25 },
    { 0x38f454a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7454ae, SUC_lexicon_tags_17 },
    { 0x957454af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x235454b1, SUC_lexicon_tags_87 },
    { 0x1dd454b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3454b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd1454b7, SUC_lexicon_tags_38 },
    { 0x1a1454b8, SUC_lexicon_tags_1 },
    { 0xc27454b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853454bd, SUC_lexicon_tags_25 },
    { 0x2a3454bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc81454c0, SUC_lexicon_tags_69 },
    { 0xd4f454c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65454c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1454c8, SUC_lexicon_tags_25 },
    { 0x2d3454c9, SUC_lexicon_tags_32 },
    { 0x555454c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b3454cc, SUC_lexicon_tags_32 },
    { 0xdc3454cc, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b454d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe31454d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d454d9, SUC_lexicon_tags_1 },
    { 0xf27454da, SUC_lexicon_tags_49 },
    { 0xd19454db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefd454dd, SUC_lexicon_tags_46 },
    { 0x3d7454de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301454e2, SUC_lexicon_tags_259 },
    { 0xdf9454e2, SUC_lexicon_tags_7 },
    { 0xc1d454e3, SUC_lexicon_tags_33 },
    { 0x171454e5, SUC_lexicon_tags_17 },
    { 0x1d3454e6, SUC_lexicon_tags_14 },
    { 0x89b454e7, SUC_lexicon_tags_2 },
    { 0x965454e8, SUC_lexicon_tags_30 },
    { 0x52d454e8, SUC_lexicon_tags_23 },
    { 0x9e3454ea, SUC_lexicon_tags_25 },
    { 0xcf1454eb, SUC_lexicon_tags_7 },
    { 0x3b454e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad454f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x981454f9, SUC_lexicon_tags_14 },
    { 0x2c7454fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xff9454fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af454ff, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a745503, SUC_lexicon_tags_64 },
    { 0x2e345504, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc945506, SUC_lexicon_tags_49 },
    { 0x3c145507, SUC_lexicon_tags_46 },
    { 0xc2d45506, SUC_lexicon_tags_2 },
    { 0x2ff45509, SUC_lexicon_tags_15 },
    { 0x49d45509, SUC_lexicon_tags_60 },
    { 0xbf4550a, SUC_lexicon_tags_37 },
    { 0x34550c, SUC_lexicon_tags_66 },
    { 0xd274550d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeeb4550f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb345512, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f45515, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe74551d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b45522, SUC_lexicon_tags_42 },
    { 0xdf145523, SUC_lexicon_tags_43 },
    { 0xa9545524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46345528, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26b4552a, SUC_lexicon_tags_68 },
    { 0x94552b, SUC_lexicon_tags_11 },
    { 0x8694552c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2545531, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2345534, SUC_lexicon_tags_39 },
    { 0x2d745535, SUC_lexicon_tags_25 },
    { 0xbd45536, SUC_lexicon_tags_27 },
    { 0xcc345536, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb4553b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcb74553d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d45540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8345545, SUC_lexicon_tags_49 },
    { 0xdc945546, SUC_lexicon_tags_29 },
    { 0x8b945546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b4554b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4554e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b45550, SUC_lexicon_tags_33 },
    { 0x16d45551, SUC_lexicon_tags_26 },
    { 0xf1145551, SUC_lexicon_tags_29 },
    { 0xaa745553, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8534555a, SUC_lexicon_tags_2 },
    { 0xf734555b, SUC_lexicon_tags_14 },
    { 0x2c54555c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e54555e, SUC_lexicon_tags_17 },
    { 0xdaf4555e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df45563, SUC_lexicon_tags_114 },
    { 0x98945564, SUC_lexicon_tags_23 },
    { 0xbc745565, SUC_lexicon_tags_25 },
    { 0xc8d45564, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd45569, SUC_lexicon_tags_15 },
    { 0x7a74556a, SUC_lexicon_tags_46 },
    { 0x5fb4556b, SUC_lexicon_tags_37 },
    { 0x57b4556c, SUC_lexicon_tags_87 },
    { 0xd774556d, SUC_lexicon_tags_2 },
    { 0x6bd4556b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef45571, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60945578, SUC_lexicon_tags_7 },
    { 0xdd945579, SUC_lexicon_tags_12 },
    { 0x4ab4557a, SUC_lexicon_tags_6 },
    { 0xe054557b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8174557f, SUC_lexicon_tags_7 },
    { 0x41f4557f, SUC_lexicon_tags_29 },
    { 0x4d945581, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd345586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af45591, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a145593, SUC_lexicon_tags_201 },
    { 0xb7745593, SUC_lexicon_tags_15 },
    { 0x2bd45595, SUC_lexicon_tags_45 },
    { 0xd45596, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd154559b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9514559f, SUC_lexicon_tags_15 },
    { 0x3bd4559f, SUC_lexicon_tags_12 },
    { 0xceb4559f, SUC_lexicon_tags_30 },
    { 0x1b3455a1, SUC_lexicon_tags_25 },
    { 0x1455a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d455a6, SUC_lexicon_tags_17 },
    { 0x57b455a6, SUC_lexicon_tags_30 },
    { 0x1f5455a8, SUC_lexicon_tags_39 },
    { 0x3a1455a9, SUC_lexicon_tags_25 },
    { 0xd1455a9, SUC_lexicon_tags_11 },
    { 0x8df455ab, SUC_lexicon_tags_194 },
    { 0x22f455aa, SUC_lexicon_tags_25 },
    { 0x41f455ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18d455af, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd0b455b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9455b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd05455b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685455b9, SUC_lexicon_tags_17 },
    { 0x4c5455ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9dd455bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b5455be, SUC_lexicon_tags_2 },
    { 0xe89455bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9455c2, SUC_lexicon_tags_11 },
    { 0x28f455c3, SUC_lexicon_tags_13 },
    { 0x47455c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5455c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97455d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d455d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491455dd, SUC_lexicon_tags_43 },
    { 0xeb455dd, SUC_lexicon_tags_25 },
    { 0x807455df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f7455e1, SUC_lexicon_tags_152 },
    { 0x1bb455e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23455e5, SUC_lexicon_tags_15 },
    { 0xc77455e6, SUC_lexicon_tags_9 },
    { 0xca3455e6, SUC_lexicon_tags_48 },
    { 0xe9455e8, SUC_lexicon_tags_7 },
    { 0xcd3455e8, SUC_lexicon_tags_0 },
    { 0x595455e8, SUC_lexicon_tags_11 },
    { 0x10d455ea, SUC_lexicon_tags_6 },
    { 0x3f3455ec, SUC_lexicon_tags_26 },
    { 0x7d1455ed, SUC_lexicon_tags_29 },
    { 0xb2b455ee, SUC_lexicon_tags_14 },
    { 0x495455e5, SUC_lexicon_tags_17 },
    { 0xd0d455f0, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3455f6, SUC_lexicon_tags_2 },
    { 0xc43455f7, SUC_lexicon_tags_1 },
    { 0x7cb455f8, SUC_lexicon_tags_623 },
    { 0, NULL },
    { 0x6b1455fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbdb455fc, SUC_lexicon_tags_1 },
    { 0xd85455fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b45600, SUC_lexicon_tags_7 },
    { 0xc6b45600, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e145604, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfe945606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef45609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b745615, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b745617, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb154561b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb214561d, SUC_lexicon_tags_47 },
    { 0x4fb4561e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5345621, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f45625, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf4562e, SUC_lexicon_tags_7 },
    { 0x2db4562f, SUC_lexicon_tags_37 },
    { 0x73545630, SUC_lexicon_tags_11 },
    { 0x13145630, SUC_lexicon_tags_2 },
    { 0x21b45630, SUC_lexicon_tags_21 },
    { 0x90145633, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4563b, SUC_lexicon_tags_2 },
    { 0x3034563b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb4563f, SUC_lexicon_tags_29 },
    { 0x7d745640, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30145643, SUC_lexicon_tags_1 },
    { 0x1af45644, SUC_lexicon_tags_27 },
    { 0xf9545643, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8ef45647, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b4564b, SUC_lexicon_tags_9 },
    { 0x2354564c, SUC_lexicon_tags_0 },
    { 0x7434564b, SUC_lexicon_tags_23 },
    { 0x6094564d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x15f45650, SUC_lexicon_tags_13 },
    { 0xe1745651, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e545654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf4565a, SUC_lexicon_tags_7 },
    { 0x4e94565b, SUC_lexicon_tags_10 },
    { 0xe534565a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3f4565e, SUC_lexicon_tags_7 },
    { 0x2df4565f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f45664, SUC_lexicon_tags_29 },
    { 0x1e945665, SUC_lexicon_tags_17 },
    { 0xa8345664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9df45668, SUC_lexicon_tags_43 },
    { 0x94945669, SUC_lexicon_tags_5 },
    { 0xac345669, SUC_lexicon_tags_5 },
    { 0x93945669, SUC_lexicon_tags_19 },
    { 0x2f94566c, SUC_lexicon_tags_12 },
    { 0x4eb4566d, SUC_lexicon_tags_1 },
    { 0xcbd4566e, SUC_lexicon_tags_45 },
    { 0x84b4566d, SUC_lexicon_tags_37 },
    { 0x3a145670, SUC_lexicon_tags_14 },
    { 0xe654566a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10945678, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f4567b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7734567d, SUC_lexicon_tags_7 },
    { 0x7b54567e, SUC_lexicon_tags_31 },
    { 0xc254567f, SUC_lexicon_tags_74 },
    { 0xe9b45680, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x58745683, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd114568b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6d34568d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86545692, SUC_lexicon_tags_37 },
    { 0x4e345692, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d45696, SUC_lexicon_tags_37 },
    { 0x47145697, SUC_lexicon_tags_7 },
    { 0xdb145698, SUC_lexicon_tags_37 },
    { 0x54d45699, SUC_lexicon_tags_33 },
    { 0x13f4569a, SUC_lexicon_tags_37 },
    { 0x32945699, SUC_lexicon_tags_15 },
    { 0x20d4569c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38f4569e, SUC_lexicon_tags_30 },
    { 0x10f4569f, SUC_lexicon_tags_31 },
    { 0x3fb4569f, SUC_lexicon_tags_15 },
    { 0x7bd4569f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541456aa, SUC_lexicon_tags_25 },
    { 0xd5d456ab, SUC_lexicon_tags_44 },
    { 0xb01456ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955456b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5df456b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1456bb, SUC_lexicon_tags_127 },
    { 0x487456bc, SUC_lexicon_tags_14 },
    { 0x351456bc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad456c0, SUC_lexicon_tags_12 },
    { 0x8b9456c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x487456c4, SUC_lexicon_tags_32 },
    { 0xffd456c5, SUC_lexicon_tags_79 },
    { 0x321456c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x181456c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21b456cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47456ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6cf456d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf456d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1456d7, SUC_lexicon_tags_12 },
    { 0xf3b456d8, SUC_lexicon_tags_29 },
    { 0x857456d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69456dc, SUC_lexicon_tags_28 },
    { 0x98f456dd, SUC_lexicon_tags_31 },
    { 0x429456dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d456e2, SUC_lexicon_tags_21 },
    { 0x53456e3, SUC_lexicon_tags_47 },
    { 0xa1b456e4, SUC_lexicon_tags_11 },
    { 0xa6b456e5, SUC_lexicon_tags_26 },
    { 0x7fb456e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x571456e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc01456eb, SUC_lexicon_tags_37 },
    { 0x54f456ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb53456ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f456f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb456f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a3456fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f456fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac745700, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44f45702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5db45704, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88f45706, SUC_lexicon_tags_127 },
    { 0xf145707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5534570b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa894570e, SUC_lexicon_tags_47 },
    { 0xd034570f, SUC_lexicon_tags_63 },
    { 0x2e74570e, SUC_lexicon_tags_27 },
    { 0xe7145711, SUC_lexicon_tags_151 },
    { 0x3694570f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf45714, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21b45716, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x67b45718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf834571b, SUC_lexicon_tags_37 },
    { 0xb574571c, SUC_lexicon_tags_12 },
    { 0x36b4571b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe574571f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ab45721, SUC_lexicon_tags_45 },
    { 0xf2545722, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6945728, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f4572d, SUC_lexicon_tags_160 },
    { 0x34d4572e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc345730, SUC_lexicon_tags_2 },
    { 0x6b545730, SUC_lexicon_tags_1 },
    { 0xfa145730, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x19545734, SUC_lexicon_tags_37 },
    { 0x96945734, SUC_lexicon_tags_23 },
    { 0xfdd45736, SUC_lexicon_tags_45 },
    { 0x87145737, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc574573f, SUC_lexicon_tags_8 },
    { 0x31d45740, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbb145742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94145747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8994574b, SUC_lexicon_tags_17 },
    { 0xfaf4574b, SUC_lexicon_tags_37 },
    { 0x1754574d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc945752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d45757, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x63145759, SUC_lexicon_tags_14 },
    { 0x29f4575a, SUC_lexicon_tags_62 },
    { 0xd1b4575a, SUC_lexicon_tags_12 },
    { 0x28d4575c, SUC_lexicon_tags_16 },
    { 0x4a94575b, SUC_lexicon_tags_38 },
    { 0x11b4575c, SUC_lexicon_tags_69 },
    { 0xa54575b, SUC_lexicon_tags_15 },
    { 0x2714575b, SUC_lexicon_tags_1 },
    { 0xe3f45761, SUC_lexicon_tags_2 },
    { 0xd9b45761, SUC_lexicon_tags_79 },
    { 0x14d45763, SUC_lexicon_tags_2 },
    { 0xb8145761, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x5945766, SUC_lexicon_tags_2 },
    { 0xfc345766, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894576d, SUC_lexicon_tags_0 },
    { 0xb514576d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d45771, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xccb45773, SUC_lexicon_tags_26 },
    { 0xd1545774, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed74577a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a34577d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32745780, SUC_lexicon_tags_15 },
    { 0x26345780, SUC_lexicon_tags_29 },
    { 0xad345780, SUC_lexicon_tags_14 },
    { 0x51545780, SUC_lexicon_tags_61 },
    { 0x13345780, SUC_lexicon_tags_38 },
    { 0x37545785, SUC_lexicon_tags_25 },
    { 0xbcd45786, SUC_lexicon_tags_12 },
    { 0x5af45784, SUC_lexicon_tags_1 },
    { 0xe9545788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd34578b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb4578d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b45794, SUC_lexicon_tags_11 },
    { 0x44345795, SUC_lexicon_tags_7 },
    { 0x3ef45795, SUC_lexicon_tags_6 },
    { 0xc7f45797, SUC_lexicon_tags_0 },
    { 0xc1945797, SUC_lexicon_tags_29 },
    { 0x11545799, SUC_lexicon_tags_16 },
    { 0x99f45799, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d4579d, SUC_lexicon_tags_26 },
    { 0xe6b4579d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d457a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289457a8, SUC_lexicon_tags_31 },
    { 0xc19457a9, SUC_lexicon_tags_43 },
    { 0x399457aa, SUC_lexicon_tags_38 },
    { 0x6e7457ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53457b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc51457b5, SUC_lexicon_tags_11 },
    { 0x8fd457b6, SUC_lexicon_tags_17 },
    { 0x1f1457b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3d457b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85457bc, SUC_lexicon_tags_1 },
    { 0xeed457bd, SUC_lexicon_tags_27 },
    { 0x763457bd, SUC_lexicon_tags_49 },
    { 0x5d3457bf, SUC_lexicon_tags_19 },
    { 0x14f457be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x365457c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d457c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f457c9, SUC_lexicon_tags_37 },
    { 0xe29457ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801457d3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1457d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3457da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x817457dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3457e2, SUC_lexicon_tags_27 },
    { 0x833457e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f457ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531457ef, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x457457f2, SUC_lexicon_tags_1 },
    { 0x77b457f2, SUC_lexicon_tags_2 },
    { 0xe8b457f4, SUC_lexicon_tags_2 },
    { 0x41457f3, SUC_lexicon_tags_7 },
    { 0x361457f6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x937457f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9b457fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdb3457fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc7457fe, SUC_lexicon_tags_23 },
    { 0x5b5457fe, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd45802, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4d45804, SUC_lexicon_tags_11 },
    { 0x54f45804, SUC_lexicon_tags_17 },
    { 0xec545806, SUC_lexicon_tags_33 },
    { 0x6e345806, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x58d45809, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd814580b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d94580f, SUC_lexicon_tags_4 },
    { 0xe2f4580f, SUC_lexicon_tags_38 },
    { 0x9a145811, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3945813, SUC_lexicon_tags_55 },
    { 0x6b745813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7194581a, SUC_lexicon_tags_27 },
    { 0x3ad4581b, SUC_lexicon_tags_11 },
    { 0xead4581b, SUC_lexicon_tags_47 },
    { 0x2cf4581a, SUC_lexicon_tags_0 },
    { 0x36d4581a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f45824, SUC_lexicon_tags_46 },
    { 0xc8f45825, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86145829, SUC_lexicon_tags_11 },
    { 0x51d4582a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff45830, SUC_lexicon_tags_2 },
    { 0x29145831, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc745834, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xed545837, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8414583c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32145840, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22745842, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf4584a, SUC_lexicon_tags_17 },
    { 0x35d4584b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8514584d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5c94584f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69545855, SUC_lexicon_tags_37 },
    { 0x1c945856, SUC_lexicon_tags_1 },
    { 0xf6b45856, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4914585b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5034585d, SUC_lexicon_tags_1 },
    { 0x85d4585d, SUC_lexicon_tags_12 },
    { 0x7c94585f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33945861, SUC_lexicon_tags_48 },
    { 0xa1545862, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53945869, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b4586c, SUC_lexicon_tags_27 },
    { 0x7534586d, SUC_lexicon_tags_17 },
    { 0xd714586d, SUC_lexicon_tags_35 },
    { 0x3af4586f, SUC_lexicon_tags_33 },
    { 0x5474586e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd545873, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f745876, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x98145878, SUC_lexicon_tags_2 },
    { 0x72345879, SUC_lexicon_tags_64 },
    { 0x19b4587a, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d74587e, SUC_lexicon_tags_60 },
    { 0xccb4587f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd45882, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcf945884, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94945886, SUC_lexicon_tags_955 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf534588a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd4588e, SUC_lexicon_tags_12 },
    { 0xf554588e, SUC_lexicon_tags_13 },
    { 0xf494588f, SUC_lexicon_tags_17 },
    { 0xf7f45890, SUC_lexicon_tags_43 },
    { 0xbad45892, SUC_lexicon_tags_5 },
    { 0x84d45892, SUC_lexicon_tags_17 },
    { 0xeb54588e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1145897, SUC_lexicon_tags_1 },
    { 0x90345898, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7d4589a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xafd4589c, SUC_lexicon_tags_1 },
    { 0xfff4589d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f458a0, SUC_lexicon_tags_2 },
    { 0x5a7458a1, SUC_lexicon_tags_79 },
    { 0x849458a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe8b458a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35d458a6, SUC_lexicon_tags_40 },
    { 0x9b7458a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5458ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3458b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279458bd, SUC_lexicon_tags_63 },
    { 0x283458be, SUC_lexicon_tags_899 },
    { 0xf63458bf, SUC_lexicon_tags_8 },
    { 0x2f458be, SUC_lexicon_tags_15 },
    { 0x699458c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb1458c3, SUC_lexicon_tags_68 },
    { 0x27d458c4, SUC_lexicon_tags_83 },
    { 0x5a3458c3, SUC_lexicon_tags_9 },
    { 0xc7458c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x407458c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9458cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d458d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4bf458d2, SUC_lexicon_tags_8 },
    { 0x1ab458d3, SUC_lexicon_tags_23 },
    { 0x769458d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71458d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3458dd, SUC_lexicon_tags_3 },
    { 0x5a7458dd, SUC_lexicon_tags_77 },
    { 0xa53458de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x78d458e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b458e6, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267458ea, SUC_lexicon_tags_30 },
    { 0x157458ea, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25458ef, SUC_lexicon_tags_19 },
    { 0x381458f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909458f4, SUC_lexicon_tags_30 },
    { 0x8a3458f5, SUC_lexicon_tags_46 },
    { 0x641458f4, SUC_lexicon_tags_83 },
    { 0x3b5458f7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x85b458f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275458fd, SUC_lexicon_tags_1 },
    { 0x937458fe, SUC_lexicon_tags_87 },
    { 0x695458ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61545905, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb274590c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b14590e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x65345910, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd45913, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b745915, SUC_lexicon_tags_12 },
    { 0x5b45916, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b45919, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3114591e, SUC_lexicon_tags_15 },
    { 0x34591e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62345921, SUC_lexicon_tags_1 },
    { 0xf1745922, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xca345924, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a545926, SUC_lexicon_tags_12 },
    { 0x23745927, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa945929, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4592c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda145930, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde145933, SUC_lexicon_tags_15 },
    { 0x25345934, SUC_lexicon_tags_0 },
    { 0xb8345933, SUC_lexicon_tags_7 },
    { 0x3fd45936, SUC_lexicon_tags_30 },
    { 0x86945937, SUC_lexicon_tags_66 },
    { 0xc0345937, SUC_lexicon_tags_8 },
    { 0x20145933, SUC_lexicon_tags_17 },
    { 0x5854593a, SUC_lexicon_tags_31 },
    { 0xd534593b, SUC_lexicon_tags_38 },
    { 0x4d745938, SUC_lexicon_tags_6 },
    { 0x6914593d, SUC_lexicon_tags_47 },
    { 0x11b4593e, SUC_lexicon_tags_27 },
    { 0x85f4593f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b45942, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd0f45944, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7345946, SUC_lexicon_tags_17 },
    { 0xfab45947, SUC_lexicon_tags_16 },
    { 0x3ef45946, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe914594b, SUC_lexicon_tags_17 },
    { 0x86d4594c, SUC_lexicon_tags_6 },
    { 0x2c54594d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81745954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b45956, SUC_lexicon_tags_27 },
    { 0xd1945957, SUC_lexicon_tags_1 },
    { 0x74b45958, SUC_lexicon_tags_33 },
    { 0x57b45958, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0f4595b, SUC_lexicon_tags_39 },
    { 0xa4b4595b, SUC_lexicon_tags_25 },
    { 0x6f34595d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a345961, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x88b45963, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x54945966, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47945975, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4c745977, SUC_lexicon_tags_38 },
    { 0xc8b45978, SUC_lexicon_tags_37 },
    { 0xbb345979, SUC_lexicon_tags_12 },
    { 0x37d4597a, SUC_lexicon_tags_1 },
    { 0x8094597a, SUC_lexicon_tags_55 },
    { 0x6dd4597c, SUC_lexicon_tags_30 },
    { 0x1c54597c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf45980, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x55545983, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb945986, SUC_lexicon_tags_31 },
    { 0x1e145987, SUC_lexicon_tags_6 },
    { 0x2945986, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f74598b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd914598e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e145991, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcb45993, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeb45995, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe614599a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2314599c, SUC_lexicon_tags_37 },
    { 0x7ff4599c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc77459a0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x56b459a2, SUC_lexicon_tags_23 },
    { 0xf05459a3, SUC_lexicon_tags_17 },
    { 0x7d459a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3459aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1459ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5459b4, SUC_lexicon_tags_37 },
    { 0x2f3459b5, SUC_lexicon_tags_29 },
    { 0xad3459b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdc3459b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc71459ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb459bd, SUC_lexicon_tags_37 },
    { 0xd6f459bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92d459c0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x567459c3, SUC_lexicon_tags_17 },
    { 0xf55459c4, SUC_lexicon_tags_37 },
    { 0x5a7459c4, SUC_lexicon_tags_2 },
    { 0x1b9459c5, SUC_lexicon_tags_37 },
    { 0x451459c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc63459c9, SUC_lexicon_tags_2 },
    { 0xdb7459c9, SUC_lexicon_tags_43 },
    { 0x4bb459c9, SUC_lexicon_tags_33 },
    { 0x6c3459cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf459d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x857459d2, SUC_lexicon_tags_8 },
    { 0xf29459d2, SUC_lexicon_tags_47 },
    { 0x97d459d4, SUC_lexicon_tags_17 },
    { 0xff7459d5, SUC_lexicon_tags_30 },
    { 0x63d459d6, SUC_lexicon_tags_14 },
    { 0xf6b459d7, SUC_lexicon_tags_21 },
    { 0xbad459d4, SUC_lexicon_tags_3 },
    { 0xd03459d5, SUC_lexicon_tags_92 },
    { 0xf3d459da, SUC_lexicon_tags_38 },
    { 0x795459da, SUC_lexicon_tags_56 },
    { 0x5db459d4, SUC_lexicon_tags_10 },
    { 0x46f459db, SUC_lexicon_tags_62 },
    { 0xf5459d4, SUC_lexicon_tags_17 },
    { 0xc7459dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd3459e1, SUC_lexicon_tags_49 },
    { 0xd9459e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b459e5, SUC_lexicon_tags_37 },
    { 0xa5f459e5, SUC_lexicon_tags_1 },
    { 0x157459e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x795459ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff459ed, SUC_lexicon_tags_15 },
    { 0x183459ee, SUC_lexicon_tags_29 },
    { 0x45459ef, SUC_lexicon_tags_25 },
    { 0xca5459ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb459f3, SUC_lexicon_tags_13 },
    { 0x457459f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5459fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b45a04, SUC_lexicon_tags_23 },
    { 0x4c745a05, SUC_lexicon_tags_118 },
    { 0x21345a06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d45a09, SUC_lexicon_tags_49 },
    { 0xe9545a09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89d45a0c, SUC_lexicon_tags_15 },
    { 0xe0f45a0d, SUC_lexicon_tags_15 },
    { 0xfad45a0e, SUC_lexicon_tags_15 },
    { 0x6c945a0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x13745a11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6f45a13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4545a19, SUC_lexicon_tags_23 },
    { 0x74f45a1a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb45a1d, SUC_lexicon_tags_10 },
    { 0x43d45a1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1b45a20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x39b45a22, SUC_lexicon_tags_0 },
    { 0xa6d45a23, SUC_lexicon_tags_1 },
    { 0x73145a23, SUC_lexicon_tags_12 },
    { 0x64b45a23, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d45a29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe6945a2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcc945a2d, SUC_lexicon_tags_15 },
    { 0x90f45a2e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfd345a30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf45a35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92f45a37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80345a3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bd45a3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70545a42, SUC_lexicon_tags_11 },
    { 0xd3f45a43, SUC_lexicon_tags_56 },
    { 0xadf45a44, SUC_lexicon_tags_1 },
    { 0xa0945a45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99745a47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e545a49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d45a4d, SUC_lexicon_tags_37 },
    { 0xe6f45a4d, SUC_lexicon_tags_17 },
    { 0xe5145a4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe7545a51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd4f45a53, SUC_lexicon_tags_104 },
    { 0xd7345a53, SUC_lexicon_tags_7 },
    { 0x2dd45a55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce345a57, SUC_lexicon_tags_37 },
    { 0x73545a58, SUC_lexicon_tags_1 },
    { 0x84d45a59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27545a5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x94945a61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x51745a64, SUC_lexicon_tags_56 },
    { 0xe6145a64, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe145a68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5945a6c, SUC_lexicon_tags_15 },
    { 0x9f145a6d, SUC_lexicon_tags_47 },
    { 0xe0d45a6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb545a71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f345a73, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41945a78, SUC_lexicon_tags_1 },
    { 0xd1545a79, SUC_lexicon_tags_49 },
    { 0x85545a79, SUC_lexicon_tags_2 },
    { 0x95745a7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b45a81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b145a83, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6545a86, SUC_lexicon_tags_37 },
    { 0x23f45a86, SUC_lexicon_tags_2 },
    { 0xd2f45a88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95545a92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x18945a95, SUC_lexicon_tags_11 },
    { 0x9ab45a96, SUC_lexicon_tags_19 },
    { 0xf5745a95, SUC_lexicon_tags_7 },
    { 0xeef45a98, SUC_lexicon_tags_17 },
    { 0x3b345a96, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd0f45a9b, SUC_lexicon_tags_14 },
    { 0xce145a9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf45aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7545aa5, SUC_lexicon_tags_7 },
    { 0x15d45aa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2f45aa8, SUC_lexicon_tags_37 },
    { 0x97f45aa9, SUC_lexicon_tags_29 },
    { 0x9f45aaa, SUC_lexicon_tags_31 },
    { 0x5b945aab, SUC_lexicon_tags_2 },
    { 0x81745aab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab345ab0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f45ab6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d45ab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c345abc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f45abf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9145ac2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3545ac4, SUC_lexicon_tags_27 },
    { 0x9b745ac4, SUC_lexicon_tags_16 },
    { 0x34345ac4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4945ac9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4345acd, SUC_lexicon_tags_12 },
    { 0x60745ace, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77745ad2, SUC_lexicon_tags_2 },
    { 0x8f45ad2, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b45ad6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa145ad8, SUC_lexicon_tags_25 },
    { 0xb5345ad9, SUC_lexicon_tags_17 },
    { 0x29f45ad9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b45add, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a345adf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb945ae2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d45ae9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f145aef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa545af6, SUC_lexicon_tags_45 },
    { 0xc0345af6, SUC_lexicon_tags_25 },
    { 0x8bb45af8, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xf8345afa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b45afd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c745b00, SUC_lexicon_tags_11 },
    { 0x5dd45b00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db45b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe345b08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7945b0b, SUC_lexicon_tags_0 },
    { 0xb945b0c, SUC_lexicon_tags_15 },
    { 0xb3745b0d, SUC_lexicon_tags_1 },
    { 0x2a545b0c, SUC_lexicon_tags_12 },
    { 0xae945b0f, SUC_lexicon_tags_12 },
    { 0x77b45b10, SUC_lexicon_tags_44 },
    { 0x20945b0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64745b14, SUC_lexicon_tags_37 },
    { 0x43745b15, SUC_lexicon_tags_12 },
    { 0x3ef45b16, SUC_lexicon_tags_12 },
    { 0x47145b17, SUC_lexicon_tags_19 },
    { 0x6d945b18, SUC_lexicon_tags_19 },
    { 0x49145b19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c545b20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d45b24, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4b45b26, SUC_lexicon_tags_0 },
    { 0x73345b27, SUC_lexicon_tags_63 },
    { 0x7f745b28, SUC_lexicon_tags_65 },
    { 0xcd345b26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2af45b2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd8d45b2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3745b30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2345b36, SUC_lexicon_tags_63 },
    { 0xc2345b37, SUC_lexicon_tags_25 },
    { 0x32345b38, SUC_lexicon_tags_17 },
    { 0xc1345b39, SUC_lexicon_tags_65 },
    { 0x4cb45b3a, SUC_lexicon_tags_15 },
    { 0x9345b38, SUC_lexicon_tags_16 },
    { 0x1745b3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cd45b3e, SUC_lexicon_tags_7 },
    { 0xa6f45b3f, SUC_lexicon_tags_19 },
    { 0x55545b40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74145b42, SUC_lexicon_tags_45 },
    { 0x6ed45b42, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45745b47, SUC_lexicon_tags_37 },
    { 0x24545b48, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b45b4b, SUC_lexicon_tags_55 },
    { 0x1f545b4c, SUC_lexicon_tags_6 },
    { 0xab545b4b, SUC_lexicon_tags_17 },
    { 0x7cd45b4b, SUC_lexicon_tags_11 },
    { 0x51b45b4f, SUC_lexicon_tags_17 },
    { 0xe2d45b4f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xce145b52, SUC_lexicon_tags_11 },
    { 0x63745b53, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b45b5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d45b5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b45b63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6945b66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf945b69, SUC_lexicon_tags_15 },
    { 0xd6945b6a, SUC_lexicon_tags_2 },
    { 0x78745b69, SUC_lexicon_tags_27 },
    { 0x67745b6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30945b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89d45b74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59d45b76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b45b7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f745b7d, SUC_lexicon_tags_2 },
    { 0xeb45b7e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42745b82, SUC_lexicon_tags_45 },
    { 0xba345b83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c945b85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f45b88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b45b8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98745b90, SUC_lexicon_tags_165 },
    { 0xdbd45b91, SUC_lexicon_tags_14 },
    { 0x3f945b92, SUC_lexicon_tags_31 },
    { 0x13f45b93, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1c345b95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab345b98, SUC_lexicon_tags_47 },
    { 0x7a145b99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x54f45b9b, SUC_lexicon_tags_39 },
    { 0x79345b9b, SUC_lexicon_tags_14 },
    { 0xd5545b9d, SUC_lexicon_tags_26 },
    { 0x3b545b9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b45ba2, SUC_lexicon_tags_33 },
    { 0x70945ba3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7345ba9, SUC_lexicon_tags_15 },
    { 0x32945ba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f45bae, SUC_lexicon_tags_27 },
    { 0x92745bae, SUC_lexicon_tags_23 },
    { 0x9f545bb0, SUC_lexicon_tags_23 },
    { 0x54345bb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d45bbe, SUC_lexicon_tags_45 },
    { 0xddd45bbf, SUC_lexicon_tags_63 },
    { 0x23545bc0, SUC_lexicon_tags_9 },
    { 0x3c145bbe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad45bc4, SUC_lexicon_tags_7 },
    { 0xa145bc4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b145bca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1745bcc, SUC_lexicon_tags_34 },
    { 0x2e345bcd, SUC_lexicon_tags_23 },
    { 0x1bb45bce, SUC_lexicon_tags_68 },
    { 0xb3945bce, SUC_lexicon_tags_49 },
    { 0x2b745bd0, SUC_lexicon_tags_17 },
    { 0xf4345bcf, SUC_lexicon_tags_45 },
    { 0x55f45bd1, SUC_lexicon_tags_17 },
    { 0x5d145bd0, SUC_lexicon_tags_13 },
    { 0x6ad45bd4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d45bd8, SUC_lexicon_tags_63 },
    { 0xfdd45bd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16345bdb, SUC_lexicon_tags_7 },
    { 0xcf145bdc, SUC_lexicon_tags_7 },
    { 0xab345bdd, SUC_lexicon_tags_11 },
    { 0xb0945bde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x745be0, SUC_lexicon_tags_27 },
    { 0xba345be0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb345be5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d45bed, SUC_lexicon_tags_7 },
    { 0x2bb45bed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6745bf0, SUC_lexicon_tags_27 },
    { 0xaad45bf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ed45bf3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5a545bf5, SUC_lexicon_tags_27 },
    { 0x92b45bf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af45bfb, SUC_lexicon_tags_7 },
    { 0xe7345bfc, SUC_lexicon_tags_13 },
    { 0x4cf45bfd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99f45bff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81745c02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa145c05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76945c0b, SUC_lexicon_tags_28 },
    { 0xcb145c0b, SUC_lexicon_tags_38 },
    { 0xbb145c0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79145c11, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xba745c13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7545c18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91945c1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf745c1c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6945c1f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd45c24, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x52545c27, SUC_lexicon_tags_25 },
    { 0x98545c28, SUC_lexicon_tags_47 },
    { 0xecf45c28, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb45c2f, SUC_lexicon_tags_68 },
    { 0xda345c30, SUC_lexicon_tags_16 },
    { 0xd3545c31, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbd45c33, SUC_lexicon_tags_63 },
    { 0xec945c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a345c3c, SUC_lexicon_tags_22 },
    { 0xc5745c3d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2d145c3f, SUC_lexicon_tags_29 },
    { 0xccf45c40, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xc6945c42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b945c45, SUC_lexicon_tags_0 },
    { 0x26f45c46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22345c48, SUC_lexicon_tags_0 },
    { 0x9ef45c49, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d45c4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6345c51, SUC_lexicon_tags_11 },
    { 0x3b945c52, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d45c58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20945c5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1345c61, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x88545c64, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af45c67, SUC_lexicon_tags_15 },
    { 0x90f45c68, SUC_lexicon_tags_2 },
    { 0xf7545c67, SUC_lexicon_tags_12 },
    { 0x52545c69, SUC_lexicon_tags_25 },
    { 0xb0b45c6b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad45c72, SUC_lexicon_tags_37 },
    { 0x7cf45c73, SUC_lexicon_tags_10 },
    { 0xc0d45c74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe945c79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6545c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14945c7f, SUC_lexicon_tags_15 },
    { 0xdc345c80, SUC_lexicon_tags_33 },
    { 0xd2545c7f, SUC_lexicon_tags_19 },
    { 0x9b745c82, SUC_lexicon_tags_1 },
    { 0x60545c83, SUC_lexicon_tags_92 },
    { 0x16145c83, SUC_lexicon_tags_19 },
    { 0xa1145c85, SUC_lexicon_tags_7 },
    { 0x93545c82, SUC_lexicon_tags_33 },
    { 0xea545c84, SUC_lexicon_tags_25 },
    { 0x5ad45c81, SUC_lexicon_tags_12 },
    { 0xeaf45c86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd945c8e, SUC_lexicon_tags_49 },
    { 0xc3d45c8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff45c96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db45c99, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5945c9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xef745c9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78745ca2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d345ca5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf1d45ca7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x30545caa, SUC_lexicon_tags_69 },
    { 0x2fb45cab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed945cad, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab45cb3, SUC_lexicon_tags_11 },
    { 0xea745cb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92b45cb6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x29145cb8, SUC_lexicon_tags_9 },
    { 0x9f545cb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29745cbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x41545cbd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb45cc1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd6545cc3, SUC_lexicon_tags_11 },
    { 0x39945cc4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79545cc6, SUC_lexicon_tags_64 },
    { 0x7db45cc6, SUC_lexicon_tags_45 },
    { 0x66f45cc8, SUC_lexicon_tags_25 },
    { 0x69b45cc9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd45ccf, SUC_lexicon_tags_33 },
    { 0xe2f45ccf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad45cd3, SUC_lexicon_tags_15 },
    { 0x69d45cd4, SUC_lexicon_tags_2 },
    { 0xb1345cd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2945cd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b45cdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f45cde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97945ce4, SUC_lexicon_tags_43 },
    { 0x1745ce4, SUC_lexicon_tags_0 },
    { 0x31545ce5, SUC_lexicon_tags_23 },
    { 0x4c345ce7, SUC_lexicon_tags_37 },
    { 0xd9145ce6, SUC_lexicon_tags_12 },
    { 0x77345ce9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8745ceb, SUC_lexicon_tags_11 },
    { 0xf3745cec, SUC_lexicon_tags_83 },
    { 0x3d945ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d45cf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe545cf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe345cfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88145cfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7df45d00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d545d05, SUC_lexicon_tags_37 },
    { 0x37145d06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d45d0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d45d11, SUC_lexicon_tags_10 },
    { 0x23745d12, SUC_lexicon_tags_17 },
    { 0x6545d12, SUC_lexicon_tags_27 },
    { 0x33b45d14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d45d1b, SUC_lexicon_tags_7 },
    { 0x4cd45d1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2145d1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb345d21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed45d25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9145d27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41945d35, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9745d37, SUC_lexicon_tags_55 },
    { 0xa0545d37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d45d41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b345d47, SUC_lexicon_tags_68 },
    { 0xac545d48, SUC_lexicon_tags_37 },
    { 0x54d45d49, SUC_lexicon_tags_14 },
    { 0x7d545d48, SUC_lexicon_tags_30 },
    { 0x6a745d4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed345d4e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f45d52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca545d5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b45d60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef545d65, SUC_lexicon_tags_17 },
    { 0xb0545d66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff945d6d, SUC_lexicon_tags_12 },
    { 0x42345d6e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf6945d70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d45d73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec745d78, SUC_lexicon_tags_11 },
    { 0x97b45d78, SUC_lexicon_tags_25 },
    { 0xd6745d7a, SUC_lexicon_tags_37 },
    { 0x8f45d78, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4745d80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7745d85, SUC_lexicon_tags_12 },
    { 0x1cb45d86, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xced45d89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1545d8d, SUC_lexicon_tags_14 },
    { 0x7f545d8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75b45d90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5745d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92345d96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a145d9b, SUC_lexicon_tags_254 },
    { 0x7945d9c, SUC_lexicon_tags_15 },
    { 0x82145d9d, SUC_lexicon_tags_26 },
    { 0xc6945d9d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb45da3, SUC_lexicon_tags_47 },
    { 0x1f745da4, SUC_lexicon_tags_37 },
    { 0xb4745da5, SUC_lexicon_tags_37 },
    { 0x31545da3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0545da8, SUC_lexicon_tags_63 },
    { 0x3e945da8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f45dac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f45daf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bd45db1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d545db6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86745db8, SUC_lexicon_tags_0 },
    { 0x27545db9, SUC_lexicon_tags_25 },
    { 0x1bf45dba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90145dbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8145dc2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42745dc6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34945dcb, SUC_lexicon_tags_0 },
    { 0x1745dcb, SUC_lexicon_tags_56 },
    { 0xbf145dcb, SUC_lexicon_tags_10 },
    { 0xca745dcb, SUC_lexicon_tags_27 },
    { 0x15745dcf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41145dd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf545dd6, SUC_lexicon_tags_38 },
    { 0xc7b45dd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x73345dda, SUC_lexicon_tags_17 },
    { 0xa8745ddb, SUC_lexicon_tags_33 },
    { 0x77f45ddc, SUC_lexicon_tags_45 },
    { 0x66345ddd, SUC_lexicon_tags_14 },
    { 0xe0545dde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7545de0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f45de5, SUC_lexicon_tags_17 },
    { 0xb6745de6, SUC_lexicon_tags_28 },
    { 0x40545de6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89345dec, SUC_lexicon_tags_47 },
    { 0x80f45ded, SUC_lexicon_tags_11 },
    { 0x9eb45dee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdaf45df0, SUC_lexicon_tags_23 },
    { 0xc7b45df0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25745df4, SUC_lexicon_tags_11 },
    { 0xe1345df4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63b45df7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f45e07, SUC_lexicon_tags_1 },
    { 0xebd45e08, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1545e0a, SUC_lexicon_tags_7 },
    { 0xc545e0b, SUC_lexicon_tags_33 },
    { 0x58f45e0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d45e13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60d45e15, SUC_lexicon_tags_15 },
    { 0x8f945e16, SUC_lexicon_tags_63 },
    { 0x71d45e16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2345e19, SUC_lexicon_tags_17 },
    { 0x65545e1a, SUC_lexicon_tags_16 },
    { 0x8a145e1a, SUC_lexicon_tags_37 },
    { 0x1b45e1c, SUC_lexicon_tags_15 },
    { 0x52b45e1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f345e20, SUC_lexicon_tags_47 },
    { 0x80f45e20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8d945e23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd45e26, SUC_lexicon_tags_37 },
    { 0x4c145e26, SUC_lexicon_tags_7 },
    { 0xc6545e28, SUC_lexicon_tags_27 },
    { 0x27f45e27, SUC_lexicon_tags_17 },
    { 0xb5545e2a, SUC_lexicon_tags_6 },
    { 0x2c745e2b, SUC_lexicon_tags_7 },
    { 0xf9945e2b, SUC_lexicon_tags_17 },
    { 0x6b545e28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a145e2f, SUC_lexicon_tags_14 },
    { 0xec945e2f, SUC_lexicon_tags_25 },
    { 0x6545e31, SUC_lexicon_tags_25 },
    { 0xdc945e31, SUC_lexicon_tags_17 },
    { 0xfdb45e33, SUC_lexicon_tags_28 },
    { 0xb8d45e33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d45e37, SUC_lexicon_tags_12 },
    { 0xd9b45e38, SUC_lexicon_tags_37 },
    { 0x43745e38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7545e3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d45e41, SUC_lexicon_tags_17 },
    { 0x91745e42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80d45e44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39745e47, SUC_lexicon_tags_68 },
    { 0x3df45e48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc545e4c, SUC_lexicon_tags_4 },
    { 0xc8145e4c, SUC_lexicon_tags_13 },
    { 0xccd45e4e, SUC_lexicon_tags_42 },
    { 0x71345e4f, SUC_lexicon_tags_1 },
    { 0x98345e50, SUC_lexicon_tags_56 },
    { 0x2a945e50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33745e56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab45e58, SUC_lexicon_tags_37 },
    { 0x20945e59, SUC_lexicon_tags_65 },
    { 0x3ed45e5a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad45e5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b45e61, SUC_lexicon_tags_1 },
    { 0x61945e62, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f45e66, SUC_lexicon_tags_9 },
    { 0xe0f45e66, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd45e6b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4cf45e6d, SUC_lexicon_tags_2 },
    { 0x4ad45e6e, SUC_lexicon_tags_6 },
    { 0xd9945e6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1945e72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc145e76, SUC_lexicon_tags_11 },
    { 0x1745e76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f45e79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42945e7e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4bb45e80, SUC_lexicon_tags_11 },
    { 0x75545e80, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa5b45e83, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xca745e86, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd45e8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82745e8d, SUC_lexicon_tags_49 },
    { 0xc0f45e8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b45e9a, SUC_lexicon_tags_17 },
    { 0xaf345e9b, SUC_lexicon_tags_2 },
    { 0xc4b45e9b, SUC_lexicon_tags_11 },
    { 0x28f45e9d, SUC_lexicon_tags_11 },
    { 0x6e745e9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f45ea3, SUC_lexicon_tags_43 },
    { 0xacb45ea4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10545ea8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x81345eaa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6745eae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa945eb1, SUC_lexicon_tags_1 },
    { 0x80d45eb2, SUC_lexicon_tags_16 },
    { 0x9f345eb1, SUC_lexicon_tags_5 },
    { 0x2e945eb2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3345eb8, SUC_lexicon_tags_17 },
    { 0x6b345eb9, SUC_lexicon_tags_19 },
    { 0xefb45eba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff45ebd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95345ebf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc3d45ec1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa945ec4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb45ed0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6945eda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d45ede, SUC_lexicon_tags_16 },
    { 0xe7745edf, SUC_lexicon_tags_14 },
    { 0xe0f45edf, SUC_lexicon_tags_15 },
    { 0x13145ee1, SUC_lexicon_tags_24 },
    { 0xdcf45ede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed545ee6, SUC_lexicon_tags_2 },
    { 0xa9d45ee6, SUC_lexicon_tags_37 },
    { 0x77145ee8, SUC_lexicon_tags_2 },
    { 0x39545ee8, SUC_lexicon_tags_63 },
    { 0xc4d45eea, SUC_lexicon_tags_57 },
    { 0x33145eeb, SUC_lexicon_tags_11 },
    { 0xa3345eea, SUC_lexicon_tags_11 },
    { 0x3df45eec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2cf45eef, SUC_lexicon_tags_37 },
    { 0xbcb45eef, SUC_lexicon_tags_2 },
    { 0xfd745eef, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x18d45ef3, SUC_lexicon_tags_39 },
    { 0xff745ef3, SUC_lexicon_tags_31 },
    { 0xbd545ef5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0145efe, SUC_lexicon_tags_29 },
    { 0x23345eff, SUC_lexicon_tags_12 },
    { 0x15b45f00, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5945f06, SUC_lexicon_tags_11 },
    { 0x4dd45f07, SUC_lexicon_tags_14 },
    { 0xbbd45f08, SUC_lexicon_tags_26 },
    { 0x12f45f08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1345f0b, SUC_lexicon_tags_15 },
    { 0xdb345f0c, SUC_lexicon_tags_25 },
    { 0xd8945f0b, SUC_lexicon_tags_12 },
    { 0xf4945f0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af45f16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f45f1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75145f24, SUC_lexicon_tags_12 },
    { 0xcf45f24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb45f28, SUC_lexicon_tags_9 },
    { 0xb7945f29, SUC_lexicon_tags_32 },
    { 0x45545f29, SUC_lexicon_tags_26 },
    { 0x20f45f2a, SUC_lexicon_tags_57 },
    { 0x82945f2c, SUC_lexicon_tags_19 },
    { 0x40545f2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa4745f2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87745f32, SUC_lexicon_tags_16 },
    { 0x94f45f33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0345f36, SUC_lexicon_tags_28 },
    { 0xbb45f37, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb145f3e, SUC_lexicon_tags_33 },
    { 0x1c745f3f, SUC_lexicon_tags_19 },
    { 0x6c945f40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d945f43, SUC_lexicon_tags_60 },
    { 0xe7d45f44, SUC_lexicon_tags_17 },
    { 0x14945f44, SUC_lexicon_tags_11 },
    { 0x40545f43, SUC_lexicon_tags_55 },
    { 0x4ed45f47, SUC_lexicon_tags_663 },
    { 0xd5f45f46, SUC_lexicon_tags_25 },
    { 0x6b945f49, SUC_lexicon_tags_7 },
    { 0x4e345f44, SUC_lexicon_tags_27 },
    { 0xcd745f4b, SUC_lexicon_tags_37 },
    { 0xa1345f4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9145f52, SUC_lexicon_tags_17 },
    { 0xdfb45f53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f45f59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7545f5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf45f5f, SUC_lexicon_tags_25 },
    { 0xa9f45f60, SUC_lexicon_tags_8 },
    { 0x4e945f61, SUC_lexicon_tags_37 },
    { 0x76945f61, SUC_lexicon_tags_11 },
    { 0x5db45f60, SUC_lexicon_tags_23 },
    { 0x3db45f63, SUC_lexicon_tags_16 },
    { 0xc8145f63, SUC_lexicon_tags_17 },
    { 0x77745f66, SUC_lexicon_tags_0 },
    { 0xf3545f66, SUC_lexicon_tags_14 },
    { 0x20b45f68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb745f71, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4945f74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbfb45f76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b45f7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf545f7c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab45f7f, SUC_lexicon_tags_17 },
    { 0x9145f80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed45f83, SUC_lexicon_tags_56 },
    { 0x76345f84, SUC_lexicon_tags_37 },
    { 0xd1945f85, SUC_lexicon_tags_29 },
    { 0x78745f85, SUC_lexicon_tags_1 },
    { 0xa6b45f85, SUC_lexicon_tags_2 },
    { 0x76145f86, SUC_lexicon_tags_1 },
    { 0x9b745f86, SUC_lexicon_tags_30 },
    { 0xce145f8a, SUC_lexicon_tags_17 },
    { 0x2f345f8a, SUC_lexicon_tags_62 },
    { 0xc3145f84, SUC_lexicon_tags_25 },
    { 0x54945f8d, SUC_lexicon_tags_15 },
    { 0x98b45f8d, SUC_lexicon_tags_61 },
    { 0x35145f8f, SUC_lexicon_tags_15 },
    { 0x98145f90, SUC_lexicon_tags_15 },
    { 0x62f45f90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2545f94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd45f97, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8945f9b, SUC_lexicon_tags_60 },
    { 0x96545f9b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3345f9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8945fa2, SUC_lexicon_tags_6 },
    { 0x11d45fa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69745fa7, SUC_lexicon_tags_45 },
    { 0x81145fa7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6145fab, SUC_lexicon_tags_27 },
    { 0xb5b45fac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c945fae, SUC_lexicon_tags_32 },
    { 0xd1745faf, SUC_lexicon_tags_1 },
    { 0xdd345fb0, SUC_lexicon_tags_17 },
    { 0x2a545fb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ab45fb3, SUC_lexicon_tags_0 },
    { 0x6345fb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb45fb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af45fbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45145fc1, SUC_lexicon_tags_23 },
    { 0x30945fc2, SUC_lexicon_tags_16 },
    { 0x82f45fc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x73b45fc5, SUC_lexicon_tags_70 },
    { 0xe1545fc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df45fc9, SUC_lexicon_tags_37 },
    { 0x97745fc9, SUC_lexicon_tags_23 },
    { 0x34345fcb, SUC_lexicon_tags_43 },
    { 0xe9345fcb, SUC_lexicon_tags_29 },
    { 0xcb345fc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4345fd1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46745fd6, SUC_lexicon_tags_37 },
    { 0x4d745fd6, SUC_lexicon_tags_25 },
    { 0xfe145fd8, SUC_lexicon_tags_62 },
    { 0xcff45fd8, SUC_lexicon_tags_3 },
    { 0xe2945fd9, SUC_lexicon_tags_37 },
    { 0xf5d45fdb, SUC_lexicon_tags_49 },
    { 0x33745fdc, SUC_lexicon_tags_7 },
    { 0x8a345fd7, SUC_lexicon_tags_84 },
    { 0x1ed45fde, SUC_lexicon_tags_17 },
    { 0x33345fdf, SUC_lexicon_tags_1 },
    { 0x7fd45fdc, SUC_lexicon_tags_23 },
    { 0x44f45fe1, SUC_lexicon_tags_13 },
    { 0xfe545fdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67145fe4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc545fe6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x13745fe9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40545fee, SUC_lexicon_tags_43 },
    { 0x48545fee, SUC_lexicon_tags_29 },
    { 0x66d45ff0, SUC_lexicon_tags_45 },
    { 0xaad45ff1, SUC_lexicon_tags_56 },
    { 0x63d45ff0, SUC_lexicon_tags_63 },
    { 0x41745ff3, SUC_lexicon_tags_17 },
    { 0xee945ff4, SUC_lexicon_tags_11 },
    { 0xef745ff0, SUC_lexicon_tags_25 },
    { 0xcf745ff2, SUC_lexicon_tags_0 },
    { 0x91d45ff2, SUC_lexicon_tags_2 },
    { 0x79345ff5, SUC_lexicon_tags_63 },
    { 0x2bf45ff5, SUC_lexicon_tags_0 },
    { 0xe2345ffa, SUC_lexicon_tags_2 },
    { 0x8e345ffa, SUC_lexicon_tags_2 },
    { 0xc4945ffa, SUC_lexicon_tags_23 },
    { 0x11145ff3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfeb45fff, SUC_lexicon_tags_11 },
    { 0xd3f46000, SUC_lexicon_tags_25 },
    { 0x46d46001, SUC_lexicon_tags_31 },
    { 0x6346001, SUC_lexicon_tags_79 },
    { 0x73f46003, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29546013, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3f346015, SUC_lexicon_tags_10 },
    { 0xf2d46016, SUC_lexicon_tags_37 },
    { 0xf746016, SUC_lexicon_tags_15 },
    { 0x17346015, SUC_lexicon_tags_7 },
    { 0xc0d46018, SUC_lexicon_tags_17 },
    { 0x89f4601a, SUC_lexicon_tags_43 },
    { 0x72d46015, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb46021, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f4602f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1146032, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb74603e, SUC_lexicon_tags_37 },
    { 0x3cf4603f, SUC_lexicon_tags_2 },
    { 0xc4146040, SUC_lexicon_tags_1 },
    { 0x24d46040, SUC_lexicon_tags_7 },
    { 0x2e146042, SUC_lexicon_tags_2 },
    { 0x72b46043, SUC_lexicon_tags_0 },
    { 0xaf46044, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75746047, SUC_lexicon_tags_29 },
    { 0x3ed46048, SUC_lexicon_tags_9 },
    { 0xc0f46048, SUC_lexicon_tags_17 },
    { 0x11146047, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xaf94604c, SUC_lexicon_tags_39 },
    { 0xb294604c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab46051, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0x6dd46053, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b46058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb4605b, SUC_lexicon_tags_27 },
    { 0x5254605c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3f4605e, SUC_lexicon_tags_1 },
    { 0x7e54605f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35746062, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30146064, SUC_lexicon_tags_32 },
    { 0x8e346064, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f4606d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc7d4606f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd6b46071, SUC_lexicon_tags_7 },
    { 0xedd46072, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2cb46074, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f46076, SUC_lexicon_tags_11 },
    { 0x7bd46077, SUC_lexicon_tags_8 },
    { 0x6f346077, SUC_lexicon_tags_29 },
    { 0x80146076, SUC_lexicon_tags_21 },
    { 0xc814607a, SUC_lexicon_tags_7 },
    { 0xe274607b, SUC_lexicon_tags_14 },
    { 0xc7b46078, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d4607f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f46084, SUC_lexicon_tags_17 },
    { 0x57d46084, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x31746087, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4734608b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd294608e, SUC_lexicon_tags_1 },
    { 0x7b74608f, SUC_lexicon_tags_49 },
    { 0x82146090, SUC_lexicon_tags_7 },
    { 0x63146091, SUC_lexicon_tags_33 },
    { 0xcb46090, SUC_lexicon_tags_71 },
    { 0x80946091, SUC_lexicon_tags_15 },
    { 0x8d346094, SUC_lexicon_tags_27 },
    { 0x6f146095, SUC_lexicon_tags_2 },
    { 0x82f46095, SUC_lexicon_tags_48 },
    { 0xc3146094, SUC_lexicon_tags_65 },
    { 0xf4b46098, SUC_lexicon_tags_33 },
    { 0xed946095, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xf454609c, SUC_lexicon_tags_9 },
    { 0xdf14609d, SUC_lexicon_tags_32 },
    { 0xf074609e, SUC_lexicon_tags_17 },
    { 0x3fb4609c, SUC_lexicon_tags_15 },
    { 0x47b460a0, SUC_lexicon_tags_37 },
    { 0x53d460a1, SUC_lexicon_tags_4 },
    { 0xc0d460a1, SUC_lexicon_tags_17 },
    { 0xc0d460a0, SUC_lexicon_tags_43 },
    { 0x6914609c, SUC_lexicon_tags_1 },
    { 0x501460a5, SUC_lexicon_tags_11 },
    { 0x8a7460a6, SUC_lexicon_tags_15 },
    { 0x9e1460a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e7460a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761460ad, SUC_lexicon_tags_15 },
    { 0xded460ad, SUC_lexicon_tags_5 },
    { 0x589460ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd460b4, SUC_lexicon_tags_0 },
    { 0x97d460b5, SUC_lexicon_tags_2 },
    { 0xbd460b6, SUC_lexicon_tags_2 },
    { 0x25460b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x749460b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf460bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc9f460be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863460c2, SUC_lexicon_tags_12 },
    { 0x981460c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd460c6, SUC_lexicon_tags_19 },
    { 0x761460c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67460cb, SUC_lexicon_tags_28 },
    { 0x133460cb, SUC_lexicon_tags_11 },
    { 0x83b460cc, SUC_lexicon_tags_17 },
    { 0xf39460ce, SUC_lexicon_tags_1 },
    { 0x3b9460cc, SUC_lexicon_tags_17 },
    { 0xa69460d0, SUC_lexicon_tags_15 },
    { 0xd85460d0, SUC_lexicon_tags_16 },
    { 0xd07460d2, SUC_lexicon_tags_11 },
    { 0x705460d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df460d7, SUC_lexicon_tags_37 },
    { 0x9ff460d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b460dd, SUC_lexicon_tags_55 },
    { 0x27b460de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4d460e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71460e4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1460e8, SUC_lexicon_tags_39 },
    { 0xe0f460e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f460fa, SUC_lexicon_tags_7 },
    { 0x7a7460fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91460fe, SUC_lexicon_tags_12 },
    { 0xf63460ff, SUC_lexicon_tags_37 },
    { 0xfff460ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd46103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72546105, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6774610d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc534610f, SUC_lexicon_tags_34 },
    { 0x31146110, SUC_lexicon_tags_15 },
    { 0x67d46111, SUC_lexicon_tags_66 },
    { 0xc9d4610f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d46115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9b46117, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd7b46119, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x99d4611b, SUC_lexicon_tags_116 },
    { 0x9b54611b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4346120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc746125, SUC_lexicon_tags_34 },
    { 0x2b146126, SUC_lexicon_tags_25 },
    { 0xc946127, SUC_lexicon_tags_25 },
    { 0x38f46128, SUC_lexicon_tags_91 },
    { 0x88d46129, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa3d4612b, SUC_lexicon_tags_57 },
    { 0x7334612c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7946130, SUC_lexicon_tags_25 },
    { 0xbe146131, SUC_lexicon_tags_55 },
    { 0x53f46132, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x99946135, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f4613a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe54613d, SUC_lexicon_tags_17 },
    { 0x7734613e, SUC_lexicon_tags_68 },
    { 0xa094613e, SUC_lexicon_tags_14 },
    { 0xe074613e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3746143, SUC_lexicon_tags_14 },
    { 0x74346144, SUC_lexicon_tags_11 },
    { 0x96146145, SUC_lexicon_tags_7 },
    { 0x3746145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f146148, SUC_lexicon_tags_14 },
    { 0xcd146148, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef4614d, SUC_lexicon_tags_31 },
    { 0xd34614d, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x2b946150, SUC_lexicon_tags_11 },
    { 0x96f46150, SUC_lexicon_tags_2 },
    { 0x18f46150, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x84d46154, SUC_lexicon_tags_17 },
    { 0x92146154, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31346157, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb4746159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd34615b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9154615e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x82146160, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4146164, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c546166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff4616b, SUC_lexicon_tags_30 },
    { 0x2394616b, SUC_lexicon_tags_2 },
    { 0x96b4616d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x40946170, SUC_lexicon_tags_21 },
    { 0xcff46170, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdc346173, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8e146175, SUC_lexicon_tags_6 },
    { 0xc1546176, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3546178, SUC_lexicon_tags_29 },
    { 0x47346178, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f4617d, SUC_lexicon_tags_2 },
    { 0x8654617e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f46183, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d14618f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee546193, SUC_lexicon_tags_12 },
    { 0x8f46194, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87546196, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa014619b, SUC_lexicon_tags_17 },
    { 0xc5b4619b, SUC_lexicon_tags_17 },
    { 0x1294619d, SUC_lexicon_tags_33 },
    { 0xf354619b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb461a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x107461a2, SUC_lexicon_tags_21 },
    { 0xc87461a3, SUC_lexicon_tags_34 },
    { 0x391461a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f461a7, SUC_lexicon_tags_15 },
    { 0xab5461a7, SUC_lexicon_tags_11 },
    { 0x8e3461a7, SUC_lexicon_tags_60 },
    { 0xd43461a9, SUC_lexicon_tags_60 },
    { 0x185461ab, SUC_lexicon_tags_48 },
    { 0x53d461aa, SUC_lexicon_tags_27 },
    { 0xb5b461ab, SUC_lexicon_tags_12 },
    { 0xc0b461ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407461b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f461b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297461be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa6f461c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b461c3, SUC_lexicon_tags_43 },
    { 0x129461c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf461c6, SUC_lexicon_tags_27 },
    { 0xb5461c7, SUC_lexicon_tags_77 },
    { 0x1b9461c6, SUC_lexicon_tags_33 },
    { 0x241461c8, SUC_lexicon_tags_37 },
    { 0xd01461ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89461ce, SUC_lexicon_tags_81 },
    { 0xeb1461cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d461d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d461db, SUC_lexicon_tags_33 },
    { 0x5af461dc, SUC_lexicon_tags_7 },
    { 0x7a7461dd, SUC_lexicon_tags_15 },
    { 0x193461de, SUC_lexicon_tags_25 },
    { 0xd99461df, SUC_lexicon_tags_19 },
    { 0x70b461dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77461e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8e5461e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39461eb, SUC_lexicon_tags_15 },
    { 0xc2d461eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x555461ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af461f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73f461f6, SUC_lexicon_tags_7 },
    { 0xc15461f7, SUC_lexicon_tags_45 },
    { 0x145461f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65461fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x575461fd, SUC_lexicon_tags_63 },
    { 0x37f461fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6946205, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d46208, SUC_lexicon_tags_720 },
    { 0xc7346209, SUC_lexicon_tags_38 },
    { 0x5a14620a, SUC_lexicon_tags_33 },
    { 0xc9b4620b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59746210, SUC_lexicon_tags_79 },
    { 0x83f46211, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed146215, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8fb46217, SUC_lexicon_tags_2 },
    { 0xe4d46217, SUC_lexicon_tags_11 },
    { 0xf1546219, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b4621d, SUC_lexicon_tags_49 },
    { 0x34f4621e, SUC_lexicon_tags_87 },
    { 0x2454621e, SUC_lexicon_tags_25 },
    { 0xbc14621f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f746225, SUC_lexicon_tags_87 },
    { 0x41b46226, SUC_lexicon_tags_17 },
    { 0x71346227, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4622a, SUC_lexicon_tags_17 },
    { 0xfab4622b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35546231, SUC_lexicon_tags_2 },
    { 0xc2946232, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed46235, SUC_lexicon_tags_20 },
    { 0x49946236, SUC_lexicon_tags_11 },
    { 0x49546236, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754623b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23346243, SUC_lexicon_tags_4 },
    { 0xef946244, SUC_lexicon_tags_136 },
    { 0xa5d46243, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d4624b, SUC_lexicon_tags_59 },
    { 0x3454624c, SUC_lexicon_tags_11 },
    { 0x4074624b, SUC_lexicon_tags_65 },
    { 0x23d4624e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x7546250, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf746253, SUC_lexicon_tags_46 },
    { 0xc3746254, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4346259, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d346261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93946264, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55b46266, SUC_lexicon_tags_8 },
    { 0x8df46266, SUC_lexicon_tags_65 },
    { 0x9f346266, SUC_lexicon_tags_37 },
    { 0xb9b46267, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114626e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x95546270, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x17946272, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7b46274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0946277, SUC_lexicon_tags_28 },
    { 0xae946278, SUC_lexicon_tags_60 },
    { 0xf1746278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d4627e, SUC_lexicon_tags_37 },
    { 0x6bb4627f, SUC_lexicon_tags_38 },
    { 0x7b546280, SUC_lexicon_tags_4 },
    { 0xee946281, SUC_lexicon_tags_15 },
    { 0xe3b46280, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab546284, SUC_lexicon_tags_14 },
    { 0xd6f46284, SUC_lexicon_tags_138 },
    { 0x5df46285, SUC_lexicon_tags_7 },
    { 0xb6b46284, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf54628c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d46291, SUC_lexicon_tags_8 },
    { 0xd8b46292, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4546294, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x54f46296, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f4629b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8b4629d, SUC_lexicon_tags_45 },
    { 0xf074629d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7f462a0, SUC_lexicon_tags_64 },
    { 0xce3462a0, SUC_lexicon_tags_30 },
    { 0x619462a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce7462a4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd462a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d462ae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01462b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc79462b4, SUC_lexicon_tags_7 },
    { 0x21f462b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6eb462b7, SUC_lexicon_tags_27 },
    { 0x649462b8, SUC_lexicon_tags_1 },
    { 0x257462b9, SUC_lexicon_tags_15 },
    { 0xf49462ba, SUC_lexicon_tags_3 },
    { 0xa13462ba, SUC_lexicon_tags_37 },
    { 0xd35462bc, SUC_lexicon_tags_57 },
    { 0x423462b9, SUC_lexicon_tags_39 },
    { 0xe61462be, SUC_lexicon_tags_16 },
    { 0x49d462be, SUC_lexicon_tags_12 },
    { 0xd85462ba, SUC_lexicon_tags_14 },
    { 0xb95462ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc3462c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd11462c5, SUC_lexicon_tags_43 },
    { 0xe4d462c6, SUC_lexicon_tags_0 },
    { 0x9cb462c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3462ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec9462cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41462d4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f462d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1f462db, SUC_lexicon_tags_37 },
    { 0x5ff462db, SUC_lexicon_tags_15 },
    { 0xeab462dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255462e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3462eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x471462ed, SUC_lexicon_tags_71 },
    { 0x785462ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd01462f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf462f4, SUC_lexicon_tags_45 },
    { 0xea3462f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3462fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f46302, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee146305, SUC_lexicon_tags_75 },
    { 0xf9d46305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad146309, SUC_lexicon_tags_17 },
    { 0xee34630a, SUC_lexicon_tags_126 },
    { 0x4114630b, SUC_lexicon_tags_14 },
    { 0x3df4630c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b46310, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a146318, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa354631a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb46321, SUC_lexicon_tags_17 },
    { 0xe2b46321, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31b46324, SUC_lexicon_tags_15 },
    { 0x15546325, SUC_lexicon_tags_31 },
    { 0x3cb46326, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d94632b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f146330, SUC_lexicon_tags_37 },
    { 0xef346331, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36146335, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf614633d, SUC_lexicon_tags_37 },
    { 0x4e14633d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xaa146340, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3f146342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4b46344, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15746349, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26146353, SUC_lexicon_tags_33 },
    { 0x67b46354, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2546359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab14635b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d4635e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3146361, SUC_lexicon_tags_29 },
    { 0xa546362, SUC_lexicon_tags_23 },
    { 0x41746363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3946365, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x64546367, SUC_lexicon_tags_12 },
    { 0x1f546367, SUC_lexicon_tags_32 },
    { 0x59546367, SUC_lexicon_tags_11 },
    { 0x1314636a, SUC_lexicon_tags_45 },
    { 0x1c14636b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b46370, SUC_lexicon_tags_15 },
    { 0x37146370, SUC_lexicon_tags_0 },
    { 0x1bb46370, SUC_lexicon_tags_87 },
    { 0x96b46373, SUC_lexicon_tags_47 },
    { 0xb1946373, SUC_lexicon_tags_6 },
    { 0x88746375, SUC_lexicon_tags_17 },
    { 0x84346375, SUC_lexicon_tags_47 },
    { 0x52546374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf4637b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d4637d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4ab4637f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25d46381, SUC_lexicon_tags_33 },
    { 0xbdd46382, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb146385, SUC_lexicon_tags_17 },
    { 0x41d46385, SUC_lexicon_tags_37 },
    { 0x5bf46387, SUC_lexicon_tags_38 },
    { 0xcdf46386, SUC_lexicon_tags_47 },
    { 0x15546389, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd4638d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x28746390, SUC_lexicon_tags_49 },
    { 0x1e746390, SUC_lexicon_tags_25 },
    { 0x4c546392, SUC_lexicon_tags_63 },
    { 0xba546392, SUC_lexicon_tags_2 },
    { 0x24546394, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2a546396, SUC_lexicon_tags_23 },
    { 0x50346397, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf554639b, SUC_lexicon_tags_15 },
    { 0xd214639c, SUC_lexicon_tags_103 },
    { 0xa394639d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf463a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f463b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19463b3, SUC_lexicon_tags_2 },
    { 0xf4d463b4, SUC_lexicon_tags_37 },
    { 0xc53463b5, SUC_lexicon_tags_61 },
    { 0xfcd463b6, SUC_lexicon_tags_33 },
    { 0x2cb463b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x329463b9, SUC_lexicon_tags_37 },
    { 0x35b463ba, SUC_lexicon_tags_16 },
    { 0x7f5463b9, SUC_lexicon_tags_55 },
    { 0xedf463bc, SUC_lexicon_tags_7 },
    { 0xcf9463bd, SUC_lexicon_tags_11 },
    { 0x9b1463be, SUC_lexicon_tags_19 },
    { 0x4cf463bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2b463c1, SUC_lexicon_tags_17 },
    { 0x5a7463c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1463c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f463d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f463d4, SUC_lexicon_tags_1403 },
    { 0xc87463d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b463dc, SUC_lexicon_tags_15 },
    { 0x9f463dc, SUC_lexicon_tags_11 },
    { 0x15f463dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f463e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49463e6, SUC_lexicon_tags_37 },
    { 0x2ad463e6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831463eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367463f8, SUC_lexicon_tags_27 },
    { 0x78b463f8, SUC_lexicon_tags_7 },
    { 0xa69463f8, SUC_lexicon_tags_11 },
    { 0x171463fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f46400, SUC_lexicon_tags_30 },
    { 0xd3346401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13746403, SUC_lexicon_tags_15 },
    { 0x9d46404, SUC_lexicon_tags_37 },
    { 0x36d46404, SUC_lexicon_tags_28 },
    { 0x52d46404, SUC_lexicon_tags_38 },
    { 0xfe746405, SUC_lexicon_tags_30 },
    { 0x17746407, SUC_lexicon_tags_148 },
    { 0xe1946404, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6034640c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d46414, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db46418, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd4641d, SUC_lexicon_tags_1 },
    { 0x2694641e, SUC_lexicon_tags_38 },
    { 0x2994641f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa146425, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7934642b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1154642d, SUC_lexicon_tags_31 },
    { 0x6a74642e, SUC_lexicon_tags_16 },
    { 0x7bd4642f, SUC_lexicon_tags_39 },
    { 0x95b4642d, SUC_lexicon_tags_1 },
    { 0x19946431, SUC_lexicon_tags_1 },
    { 0x6ad4642e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8346439, SUC_lexicon_tags_49 },
    { 0x32f46439, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59146440, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63146442, SUC_lexicon_tags_47 },
    { 0xb5346443, SUC_lexicon_tags_63 },
    { 0xe4f46442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd734644f, SUC_lexicon_tags_14 },
    { 0x2c746450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd46455, SUC_lexicon_tags_15 },
    { 0x34746455, SUC_lexicon_tags_37 },
    { 0xeef46457, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8746459, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1b34645b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d4645e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x16946460, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d46463, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6114646e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x45546470, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3146473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa934647d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2746484, SUC_lexicon_tags_37 },
    { 0xd2b46485, SUC_lexicon_tags_38 },
    { 0x81746485, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d4648b, SUC_lexicon_tags_45 },
    { 0xf894648c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a546491, SUC_lexicon_tags_23 },
    { 0x22746491, SUC_lexicon_tags_11 },
    { 0xeb346493, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb346496, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd46499, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f464a7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x401464a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7464ad, SUC_lexicon_tags_48 },
    { 0x65f464ae, SUC_lexicon_tags_47 },
    { 0xb2d464af, SUC_lexicon_tags_9 },
    { 0x743464b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9464b3, SUC_lexicon_tags_12 },
    { 0x18b464b3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x639464b7, SUC_lexicon_tags_17 },
    { 0x317464b7, SUC_lexicon_tags_12 },
    { 0x53d464b8, SUC_lexicon_tags_5 },
    { 0x477464b9, SUC_lexicon_tags_7 },
    { 0x11464bb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f464c0, SUC_lexicon_tags_37 },
    { 0xd7d464c0, SUC_lexicon_tags_7 },
    { 0x57464c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845464c7, SUC_lexicon_tags_6 },
    { 0x2c5464c8, SUC_lexicon_tags_27 },
    { 0x10b464c9, SUC_lexicon_tags_525 },
    { 0x99f464c8, SUC_lexicon_tags_38 },
    { 0x99464ca, SUC_lexicon_tags_26 },
    { 0xe97464cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5464d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x15b464d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd09464d4, SUC_lexicon_tags_19 },
    { 0x3d9464d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8dd464d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1464dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb1f464de, SUC_lexicon_tags_15 },
    { 0x27d464df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x507464e2, SUC_lexicon_tags_80 },
    { 0x415464e3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbe3464e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9df464e7, SUC_lexicon_tags_2 },
    { 0xe47464e7, SUC_lexicon_tags_17 },
    { 0xddb464e9, SUC_lexicon_tags_37 },
    { 0x277464ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113464ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xceb464f1, SUC_lexicon_tags_38 },
    { 0xd05464f2, SUC_lexicon_tags_38 },
    { 0x211464f3, SUC_lexicon_tags_66 },
    { 0x1e3464f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5464f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d464fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf464fe, SUC_lexicon_tags_9 },
    { 0x447464fe, SUC_lexicon_tags_45 },
    { 0x40f464fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20b46502, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb46515, SUC_lexicon_tags_25 },
    { 0xd3946516, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc214651a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db46522, SUC_lexicon_tags_31 },
    { 0xced46522, SUC_lexicon_tags_31 },
    { 0xb2f46524, SUC_lexicon_tags_14 },
    { 0xad746522, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8d46527, SUC_lexicon_tags_26 },
    { 0xd546528, SUC_lexicon_tags_33 },
    { 0xb9346529, SUC_lexicon_tags_29 },
    { 0xee54652a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f4652d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2394652f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17746533, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b4653b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbe14653d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db46540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5346544, SUC_lexicon_tags_28 },
    { 0x8a546545, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x46546548, SUC_lexicon_tags_26 },
    { 0x11d46548, SUC_lexicon_tags_38 },
    { 0xd8746548, SUC_lexicon_tags_1 },
    { 0x44546549, SUC_lexicon_tags_77 },
    { 0x12d4654b, SUC_lexicon_tags_14 },
    { 0x55b46549, SUC_lexicon_tags_13 },
    { 0x3b74654e, SUC_lexicon_tags_34 },
    { 0xf5b4654e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x15946551, SUC_lexicon_tags_29 },
    { 0x14746552, SUC_lexicon_tags_17 },
    { 0x8d546551, SUC_lexicon_tags_15 },
    { 0x5d46551, SUC_lexicon_tags_11 },
    { 0x92146553, SUC_lexicon_tags_31 },
    { 0x84346551, SUC_lexicon_tags_66 },
    { 0xafd46557, SUC_lexicon_tags_37 },
    { 0x2a946558, SUC_lexicon_tags_60 },
    { 0xa4346559, SUC_lexicon_tags_11 },
    { 0x6514655a, SUC_lexicon_tags_12 },
    { 0xfcf46559, SUC_lexicon_tags_25 },
    { 0x9f346559, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4655f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0746562, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x67546564, SUC_lexicon_tags_9 },
    { 0x81b46565, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e746569, SUC_lexicon_tags_37 },
    { 0xbb94656a, SUC_lexicon_tags_1 },
    { 0x3874656b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x74f4656d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7946570, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62346574, SUC_lexicon_tags_10 },
    { 0x19f46574, SUC_lexicon_tags_24 },
    { 0x50f46576, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x60146578, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb4657e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b546587, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2874658c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc794658e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45d46590, SUC_lexicon_tags_60 },
    { 0xbf46590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff46594, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4659e, SUC_lexicon_tags_14 },
    { 0xa74659f, SUC_lexicon_tags_8 },
    { 0xf11465a0, SUC_lexicon_tags_7 },
    { 0xe2f4659e, SUC_lexicon_tags_12 },
    { 0x739465a0, SUC_lexicon_tags_15 },
    { 0xab14659e, SUC_lexicon_tags_14 },
    { 0x559465a4, SUC_lexicon_tags_158 },
    { 0x995465a5, SUC_lexicon_tags_47 },
    { 0x24d465a4, SUC_lexicon_tags_7 },
    { 0xedb465a4, SUC_lexicon_tags_7 },
    { 0xf8b465a8, SUC_lexicon_tags_2 },
    { 0x815465a8, SUC_lexicon_tags_27 },
    { 0x705465aa, SUC_lexicon_tags_1 },
    { 0x181465a5, SUC_lexicon_tags_39 },
    { 0x2c7465ac, SUC_lexicon_tags_37 },
    { 0x781465ad, SUC_lexicon_tags_0 },
    { 0xefd465ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x277465b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353465b6, SUC_lexicon_tags_37 },
    { 0xda3465b7, SUC_lexicon_tags_16 },
    { 0xc03465b6, SUC_lexicon_tags_15 },
    { 0xb9d465b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3465be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x607465c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe79465c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641465c7, SUC_lexicon_tags_38 },
    { 0xdc7465c7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd03465ca, SUC_lexicon_tags_57 },
    { 0x30f465cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x199465ce, SUC_lexicon_tags_27 },
    { 0x793465cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbaf465d1, SUC_lexicon_tags_23 },
    { 0x50f465d1, SUC_lexicon_tags_9 },
    { 0x14f465d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5465d7, SUC_lexicon_tags_7 },
    { 0x50b465d8, SUC_lexicon_tags_4 },
    { 0x383465d9, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xabf465db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7465df, SUC_lexicon_tags_7 },
    { 0x2b465e0, SUC_lexicon_tags_17 },
    { 0xbe3465df, SUC_lexicon_tags_25 },
    { 0xdc9465e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f465e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d465e8, SUC_lexicon_tags_5 },
    { 0x677465e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x443465eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3465f4, SUC_lexicon_tags_10 },
    { 0xf6f465f5, SUC_lexicon_tags_1 },
    { 0xb6b465f6, SUC_lexicon_tags_15 },
    { 0x23f465f7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595465fe, SUC_lexicon_tags_23 },
    { 0x96d465ff, SUC_lexicon_tags_7 },
    { 0x34546600, SUC_lexicon_tags_38 },
    { 0xebf46600, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4eb46603, SUC_lexicon_tags_7 },
    { 0x93746604, SUC_lexicon_tags_29 },
    { 0xacb46604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1746607, SUC_lexicon_tags_43 },
    { 0x2db46608, SUC_lexicon_tags_17 },
    { 0x5bf46609, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe214660c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf46613, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0346617, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82146619, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e94661c, SUC_lexicon_tags_41 },
    { 0x83f4661c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2746620, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d46627, SUC_lexicon_tags_27 },
    { 0xf9d46628, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae94662f, SUC_lexicon_tags_12 },
    { 0x86b46630, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x78546633, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe146637, SUC_lexicon_tags_16 },
    { 0x60546637, SUC_lexicon_tags_12 },
    { 0x3ad46639, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4234663d, SUC_lexicon_tags_9 },
    { 0xeab4663e, SUC_lexicon_tags_27 },
    { 0xf914663f, SUC_lexicon_tags_12 },
    { 0xa174663f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce946643, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2af46645, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf346649, SUC_lexicon_tags_37 },
    { 0xbf4664a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b46652, SUC_lexicon_tags_12 },
    { 0x15746653, SUC_lexicon_tags_26 },
    { 0xcc746654, SUC_lexicon_tags_15 },
    { 0xbb746654, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdb546657, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8cd46659, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed4665c, SUC_lexicon_tags_37 },
    { 0xdf74665d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73346666, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b46670, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd46673, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a946677, SUC_lexicon_tags_46 },
    { 0xa6146677, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db4667c, SUC_lexicon_tags_37 },
    { 0x5c94667d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd46680, SUC_lexicon_tags_60 },
    { 0x76b46681, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0346685, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa0346687, SUC_lexicon_tags_17 },
    { 0x15d46687, SUC_lexicon_tags_23 },
    { 0x14946687, SUC_lexicon_tags_14 },
    { 0x58746688, SUC_lexicon_tags_38 },
    { 0x7414668b, SUC_lexicon_tags_57 },
    { 0x60f4668c, SUC_lexicon_tags_21 },
    { 0x45d46687, SUC_lexicon_tags_47 },
    { 0x6114668e, SUC_lexicon_tags_27 },
    { 0x5dd4668f, SUC_lexicon_tags_12 },
    { 0x1914668f, SUC_lexicon_tags_19 },
    { 0x89d46691, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2546698, SUC_lexicon_tags_1 },
    { 0x73346698, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf374669b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf54669d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d4669f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d466a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd466a9, SUC_lexicon_tags_1 },
    { 0xd73466aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3466ad, SUC_lexicon_tags_12 },
    { 0x60d466ae, SUC_lexicon_tags_12 },
    { 0x9e3466ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1466b2, SUC_lexicon_tags_14 },
    { 0x3b466b3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85466b8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xba3466ba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x141466bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf9466be, SUC_lexicon_tags_43 },
    { 0x129466bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f466c2, SUC_lexicon_tags_7 },
    { 0xb41466c3, SUC_lexicon_tags_14 },
    { 0x80d466c4, SUC_lexicon_tags_38 },
    { 0xc1f466c3, SUC_lexicon_tags_14 },
    { 0x341466c6, SUC_lexicon_tags_2 },
    { 0xe25466c7, SUC_lexicon_tags_47 },
    { 0xc69466c6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9f5466ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8c3466cc, SUC_lexicon_tags_45 },
    { 0xc17466cd, SUC_lexicon_tags_25 },
    { 0x32b466cd, SUC_lexicon_tags_12 },
    { 0x573466cf, SUC_lexicon_tags_15 },
    { 0xae7466d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b3466d2, SUC_lexicon_tags_25 },
    { 0x9c3466d3, SUC_lexicon_tags_17 },
    { 0x567466d4, SUC_lexicon_tags_12 },
    { 0x933466d5, SUC_lexicon_tags_5 },
    { 0x5ef466d6, SUC_lexicon_tags_25 },
    { 0x3e5466d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7466dc, SUC_lexicon_tags_11 },
    { 0x285466dd, SUC_lexicon_tags_14 },
    { 0x859466dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855466e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd466f0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x127466f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89466f6, SUC_lexicon_tags_81 },
    { 0x56f466f7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x727466fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf9466fc, SUC_lexicon_tags_17 },
    { 0x1cf466fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf946700, SUC_lexicon_tags_26 },
    { 0x5cf46700, SUC_lexicon_tags_15 },
    { 0x9f946702, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc54670a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41d4670c, SUC_lexicon_tags_27 },
    { 0x3ab4670d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49146713, SUC_lexicon_tags_49 },
    { 0x87946714, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb346717, SUC_lexicon_tags_63 },
    { 0xbbb46718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b4671b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c14671d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3614671f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3b46721, SUC_lexicon_tags_13 },
    { 0x81346722, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d46726, SUC_lexicon_tags_27 },
    { 0xcfd46726, SUC_lexicon_tags_0 },
    { 0xb9b46728, SUC_lexicon_tags_42 },
    { 0xbdb46729, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f94672d, SUC_lexicon_tags_25 },
    { 0x6254672e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15146730, SUC_lexicon_tags_37 },
    { 0x62746731, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed94673b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f4673f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b46743, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x4bf46745, SUC_lexicon_tags_1 },
    { 0xd8546746, SUC_lexicon_tags_37 },
    { 0x40d46747, SUC_lexicon_tags_28 },
    { 0x9ed46745, SUC_lexicon_tags_47 },
    { 0x98546749, SUC_lexicon_tags_55 },
    { 0x6c746747, SUC_lexicon_tags_29 },
    { 0x5dd4674b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2134674f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d46755, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b46758, SUC_lexicon_tags_47 },
    { 0xb1146759, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6634675e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd46761, SUC_lexicon_tags_1 },
    { 0xf746762, SUC_lexicon_tags_38 },
    { 0x22746763, SUC_lexicon_tags_45 },
    { 0x39346764, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5b546766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb4676b, SUC_lexicon_tags_37 },
    { 0xcc34676c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f46773, SUC_lexicon_tags_2 },
    { 0x71746774, SUC_lexicon_tags_37 },
    { 0x8cb46775, SUC_lexicon_tags_65 },
    { 0xec946776, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x30946778, SUC_lexicon_tags_25 },
    { 0x6146779, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91b4677b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4f14677d, SUC_lexicon_tags_9 },
    { 0xe494677e, SUC_lexicon_tags_37 },
    { 0x7a74677e, SUC_lexicon_tags_11 },
    { 0x7cb46780, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2fd46782, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32146784, SUC_lexicon_tags_8 },
    { 0xa8546785, SUC_lexicon_tags_57 },
    { 0xcd746786, SUC_lexicon_tags_41 },
    { 0x64d46786, SUC_lexicon_tags_12 },
    { 0x65d46788, SUC_lexicon_tags_60 },
    { 0xa6f46788, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b4678d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf46790, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b46796, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f4679f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x991467a2, SUC_lexicon_tags_15 },
    { 0x717467a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b467a9, SUC_lexicon_tags_26 },
    { 0x563467aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9df467ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0d467ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd467b4, SUC_lexicon_tags_50 },
    { 0xcb7467b5, SUC_lexicon_tags_15 },
    { 0x1fd467b6, SUC_lexicon_tags_27 },
    { 0x9d467b7, SUC_lexicon_tags_38 },
    { 0x859467b8, SUC_lexicon_tags_4 },
    { 0xb89467b9, SUC_lexicon_tags_48 },
    { 0x5d9467ba, SUC_lexicon_tags_37 },
    { 0x923467bb, SUC_lexicon_tags_12 },
    { 0x3e3467bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb467bf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83467c2, SUC_lexicon_tags_37 },
    { 0x12b467c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51467c8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb467cb, SUC_lexicon_tags_0 },
    { 0x653467cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3467cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded467da, SUC_lexicon_tags_158 },
    { 0xb3b467db, SUC_lexicon_tags_182 },
    { 0x291467da, SUC_lexicon_tags_60 },
    { 0x9a7467dd, SUC_lexicon_tags_15 },
    { 0xd6b467db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b467e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97467e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead467eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b467ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad467f1, SUC_lexicon_tags_39 },
    { 0x49f467f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8df467f4, SUC_lexicon_tags_28 },
    { 0x623467f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x947467f7, SUC_lexicon_tags_55 },
    { 0x86f467f7, SUC_lexicon_tags_3 },
    { 0x6c7467f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43467fd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9d3467ff, SUC_lexicon_tags_48 },
    { 0xf83467ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e946803, SUC_lexicon_tags_7 },
    { 0xe8346804, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b46807, SUC_lexicon_tags_7 },
    { 0x5bf46808, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd34680e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f46812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d46815, SUC_lexicon_tags_19 },
    { 0x76746816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c74681a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b14681c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d46820, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d46823, SUC_lexicon_tags_14 },
    { 0x70346823, SUC_lexicon_tags_32 },
    { 0x14d46823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x91746827, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f4682b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5634682f, SUC_lexicon_tags_17 },
    { 0xc514682f, SUC_lexicon_tags_37 },
    { 0x41346831, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb46835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d4683b, SUC_lexicon_tags_9 },
    { 0x5dd4683c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf4683f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63f46841, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf46845, SUC_lexicon_tags_29 },
    { 0x86746846, SUC_lexicon_tags_23 },
    { 0xd546845, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b34684a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e94684d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b346853, SUC_lexicon_tags_32 },
    { 0xda346854, SUC_lexicon_tags_79 },
    { 0xac746854, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2454685c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7b4685e, SUC_lexicon_tags_10 },
    { 0x494685f, SUC_lexicon_tags_25 },
    { 0x35146860, SUC_lexicon_tags_11 },
    { 0xaf746861, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b4686b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5854686e, SUC_lexicon_tags_25 },
    { 0x1eb4686f, SUC_lexicon_tags_11 },
    { 0xa3f4686f, SUC_lexicon_tags_659 },
    { 0x1214686f, SUC_lexicon_tags_25 },
    { 0x3fb46872, SUC_lexicon_tags_87 },
    { 0x82946873, SUC_lexicon_tags_75 },
    { 0x4e946874, SUC_lexicon_tags_11 },
    { 0x21f46873, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3d46877, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x654687c, SUC_lexicon_tags_2 },
    { 0x2754687d, SUC_lexicon_tags_5 },
    { 0xe154687e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff46881, SUC_lexicon_tags_4 },
    { 0x48146881, SUC_lexicon_tags_55 },
    { 0x12346883, SUC_lexicon_tags_44 },
    { 0xf9946884, SUC_lexicon_tags_17 },
    { 0xd5f46885, SUC_lexicon_tags_15 },
    { 0x1ab46884, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4354688f, SUC_lexicon_tags_12 },
    { 0x86146890, SUC_lexicon_tags_14 },
    { 0x63f46890, SUC_lexicon_tags_39 },
    { 0x2c146892, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe946895, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf746897, SUC_lexicon_tags_28 },
    { 0x9c146897, SUC_lexicon_tags_47 },
    { 0x4cb46899, SUC_lexicon_tags_68 },
    { 0x2cd46899, SUC_lexicon_tags_7 },
    { 0x7db46897, SUC_lexicon_tags_38 },
    { 0x69f4689c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1314689e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b468a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49468aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579468ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf51468b0, SUC_lexicon_tags_14 },
    { 0x6d3468b1, SUC_lexicon_tags_0 },
    { 0x317468b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7468b5, SUC_lexicon_tags_27 },
    { 0x9df468b6, SUC_lexicon_tags_47 },
    { 0x107468b6, SUC_lexicon_tags_7 },
    { 0x5c5468b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb468bc, SUC_lexicon_tags_49 },
    { 0xa91468bc, SUC_lexicon_tags_57 },
    { 0x20b468be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889468c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfb468c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb468cb, SUC_lexicon_tags_10 },
    { 0x583468cb, SUC_lexicon_tags_25 },
    { 0xa51468cd, SUC_lexicon_tags_39 },
    { 0x50d468cb, SUC_lexicon_tags_37 },
    { 0x227468cf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47468d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7468d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf5468da, SUC_lexicon_tags_17 },
    { 0x4a1468db, SUC_lexicon_tags_43 },
    { 0x2a7468dc, SUC_lexicon_tags_23 },
    { 0xfd5468dd, SUC_lexicon_tags_49 },
    { 0x1fb468de, SUC_lexicon_tags_37 },
    { 0x299468dd, SUC_lexicon_tags_2 },
    { 0xe03468e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907468ea, SUC_lexicon_tags_1 },
    { 0x52b468eb, SUC_lexicon_tags_12 },
    { 0x85468eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31468f2, SUC_lexicon_tags_35 },
    { 0x929468f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33468f6, SUC_lexicon_tags_33 },
    { 0x7c7468f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x405468f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9468fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d346904, SUC_lexicon_tags_14 },
    { 0x60f46905, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f4690b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb46912, SUC_lexicon_tags_29 },
    { 0x9f746912, SUC_lexicon_tags_15 },
    { 0xbf346914, SUC_lexicon_tags_17 },
    { 0x68f46915, SUC_lexicon_tags_664 },
    { 0xaed46916, SUC_lexicon_tags_65 },
    { 0xe0b46917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x314691d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9346920, SUC_lexicon_tags_61 },
    { 0x59d46921, SUC_lexicon_tags_19 },
    { 0xd4946922, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34746926, SUC_lexicon_tags_17 },
    { 0x6ff46927, SUC_lexicon_tags_100 },
    { 0x6fd46928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf4692e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56346931, SUC_lexicon_tags_1 },
    { 0x4ed46931, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c746936, SUC_lexicon_tags_21 },
    { 0xa4946937, SUC_lexicon_tags_2 },
    { 0x2bb46936, SUC_lexicon_tags_15 },
    { 0xd7346936, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6214693d, SUC_lexicon_tags_82 },
    { 0x4fd4693e, SUC_lexicon_tags_9 },
    { 0xe34693f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b46942, SUC_lexicon_tags_37 },
    { 0x20546943, SUC_lexicon_tags_44 },
    { 0xf5546943, SUC_lexicon_tags_14 },
    { 0x56d46943, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b46948, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd34694a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8774694e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d46952, SUC_lexicon_tags_14 },
    { 0x98b46952, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4146955, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a94695a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf374695e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72746962, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3546966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5f46968, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6894696b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6934696f, SUC_lexicon_tags_77 },
    { 0xa8b46970, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91d46972, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b46976, SUC_lexicon_tags_63 },
    { 0x89d46977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82746979, SUC_lexicon_tags_15 },
    { 0xc74697a, SUC_lexicon_tags_14 },
    { 0xcc946979, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x954697e, SUC_lexicon_tags_7 },
    { 0x35f4697f, SUC_lexicon_tags_103 },
    { 0xddb46980, SUC_lexicon_tags_65 },
    { 0x49f46981, SUC_lexicon_tags_34 },
    { 0x1fd46982, SUC_lexicon_tags_7 },
    { 0x8ef46983, SUC_lexicon_tags_1 },
    { 0x17d46980, SUC_lexicon_tags_63 },
    { 0x17946985, SUC_lexicon_tags_12 },
    { 0xda946986, SUC_lexicon_tags_37 },
    { 0x1db46986, SUC_lexicon_tags_43 },
    { 0xcf946987, SUC_lexicon_tags_17 },
    { 0x68d4697e, SUC_lexicon_tags_10 },
    { 0x75746980, SUC_lexicon_tags_27 },
    { 0xb0d4698b, SUC_lexicon_tags_17 },
    { 0xc1f46986, SUC_lexicon_tags_39 },
    { 0x9a54698d, SUC_lexicon_tags_11 },
    { 0xb6d4698e, SUC_lexicon_tags_17 },
    { 0x9214698f, SUC_lexicon_tags_7 },
    { 0xd4546990, SUC_lexicon_tags_25 },
    { 0xdd746991, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8546993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb546997, SUC_lexicon_tags_37 },
    { 0xb8346998, SUC_lexicon_tags_0 },
    { 0x39546999, SUC_lexicon_tags_37 },
    { 0xe694699a, SUC_lexicon_tags_47 },
    { 0xdb74699a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2994699d, SUC_lexicon_tags_49 },
    { 0xd034699e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793469a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb469aa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3f9469ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917469b0, SUC_lexicon_tags_6 },
    { 0x95d469b0, SUC_lexicon_tags_7 },
    { 0x7af469b2, SUC_lexicon_tags_31 },
    { 0xe2b469b1, SUC_lexicon_tags_11 },
    { 0x563469b4, SUC_lexicon_tags_7 },
    { 0xb9d469b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab469b7, SUC_lexicon_tags_17 },
    { 0xb95469b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x125469bb, SUC_lexicon_tags_63 },
    { 0x5eb469bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe97469be, SUC_lexicon_tags_37 },
    { 0xfa5469bf, SUC_lexicon_tags_7 },
    { 0x4c7469be, SUC_lexicon_tags_17 },
    { 0x5b1469bf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef469c4, SUC_lexicon_tags_26 },
    { 0xe2f469c5, SUC_lexicon_tags_0 },
    { 0xfbb469c4, SUC_lexicon_tags_26 },
    { 0xb35469c7, SUC_lexicon_tags_43 },
    { 0x105469c8, SUC_lexicon_tags_82 },
    { 0xc2d469c9, SUC_lexicon_tags_45 },
    { 0x7d7469ca, SUC_lexicon_tags_24 },
    { 0xcd3469ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477469d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9469d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x903469d5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d469d8, SUC_lexicon_tags_28 },
    { 0xfd5469d9, SUC_lexicon_tags_33 },
    { 0xbf3469da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x621469dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95469e0, SUC_lexicon_tags_6 },
    { 0x4a7469e1, SUC_lexicon_tags_21 },
    { 0x493469e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf469e5, SUC_lexicon_tags_37 },
    { 0xd55469e5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d469e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217469ed, SUC_lexicon_tags_37 },
    { 0x74d469ee, SUC_lexicon_tags_2 },
    { 0xb6d469ee, SUC_lexicon_tags_21 },
    { 0x839469ef, SUC_lexicon_tags_38 },
    { 0x495469ef, SUC_lexicon_tags_33 },
    { 0x535469f2, SUC_lexicon_tags_37 },
    { 0x4bd469f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d469f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7469fa, SUC_lexicon_tags_2 },
    { 0x69469fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x667469fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e746a05, SUC_lexicon_tags_77 },
    { 0xf6746a06, SUC_lexicon_tags_65 },
    { 0x86d46a06, SUC_lexicon_tags_80 },
    { 0x99546a07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8c346a0a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x96946a0d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x87b46a0f, SUC_lexicon_tags_11 },
    { 0xdcd46a10, SUC_lexicon_tags_34 },
    { 0xb1746a10, SUC_lexicon_tags_17 },
    { 0xf5746a12, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46346a22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b346a27, SUC_lexicon_tags_26 },
    { 0xa8546a28, SUC_lexicon_tags_21 },
    { 0x99746a28, SUC_lexicon_tags_39 },
    { 0x98d46a29, SUC_lexicon_tags_25 },
    { 0xf8346a2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e746a2f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf6146a31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcb46a33, SUC_lexicon_tags_12 },
    { 0x8146a34, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x28346a36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd46a3a, SUC_lexicon_tags_37 },
    { 0x5146a3b, SUC_lexicon_tags_1 },
    { 0xd1546a3a, SUC_lexicon_tags_26 },
    { 0xcdd46a3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x35746a3f, SUC_lexicon_tags_1 },
    { 0x79546a40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f46a43, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81146a48, SUC_lexicon_tags_30 },
    { 0x3f746a49, SUC_lexicon_tags_0 },
    { 0x53946a49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7546a4e, SUC_lexicon_tags_15 },
    { 0x2ad46a4e, SUC_lexicon_tags_46 },
    { 0x45f46a50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76746a53, SUC_lexicon_tags_25 },
    { 0x8cf46a54, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af46a5b, SUC_lexicon_tags_30 },
    { 0xfc146a5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6a546a5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff46a61, SUC_lexicon_tags_12 },
    { 0xe3146a62, SUC_lexicon_tags_1 },
    { 0xbf346a63, SUC_lexicon_tags_7 },
    { 0x2dd46a64, SUC_lexicon_tags_16 },
    { 0x42146a65, SUC_lexicon_tags_17 },
    { 0xff346a61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d46a69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa546a6e, SUC_lexicon_tags_1 },
    { 0x80546a6f, SUC_lexicon_tags_25 },
    { 0x52946a70, SUC_lexicon_tags_12 },
    { 0xa0d46a71, SUC_lexicon_tags_36 },
    { 0xf1146a71, SUC_lexicon_tags_32 },
    { 0xf7b46a73, SUC_lexicon_tags_28 },
    { 0x85546a73, SUC_lexicon_tags_49 },
    { 0x3546a73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8f346a77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee346a79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99346a83, SUC_lexicon_tags_1 },
    { 0x3b346a83, SUC_lexicon_tags_17 },
    { 0x42346a84, SUC_lexicon_tags_33 },
    { 0x1b346a86, SUC_lexicon_tags_1 },
    { 0xf4946a87, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x43346a8a, SUC_lexicon_tags_12 },
    { 0xfc346a8b, SUC_lexicon_tags_23 },
    { 0x44546a8b, SUC_lexicon_tags_7 },
    { 0x2ed46a8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41146a94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c946a96, SUC_lexicon_tags_151 },
    { 0x76d46a97, SUC_lexicon_tags_15 },
    { 0x8b46a98, SUC_lexicon_tags_37 },
    { 0x1c346a99, SUC_lexicon_tags_17 },
    { 0xc9746a9a, SUC_lexicon_tags_25 },
    { 0x9d746a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e146a9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b46aa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f346aa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94146aa7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11746aae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xce146ab0, SUC_lexicon_tags_17 },
    { 0xa2546ab0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff46ab4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef46ab8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69f46aba, SUC_lexicon_tags_15 },
    { 0xd5746aba, SUC_lexicon_tags_2 },
    { 0xbb746abb, SUC_lexicon_tags_0 },
    { 0x80d46abd, SUC_lexicon_tags_14 },
    { 0x40b46abb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x38b46ac0, SUC_lexicon_tags_15 },
    { 0x39146ac0, SUC_lexicon_tags_1 },
    { 0xc5d46ac2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35346ac6, SUC_lexicon_tags_12 },
    { 0x1d346ac6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8fd46ac9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6946acd, SUC_lexicon_tags_2 },
    { 0xaa946acd, SUC_lexicon_tags_25 },
    { 0xd1146acd, SUC_lexicon_tags_15 },
    { 0x2546ace, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe746ad2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb46ad5, SUC_lexicon_tags_11 },
    { 0x2b46ad6, SUC_lexicon_tags_27 },
    { 0x24746ad5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5e546ad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2ab46adb, SUC_lexicon_tags_0 },
    { 0x98d46adb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c946ade, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77146ae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd46ae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb46aea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f46af0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xec146af2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf46af5, SUC_lexicon_tags_38 },
    { 0xfd946af6, SUC_lexicon_tags_8 },
    { 0x78146af5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff46afa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af46afd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d46b01, SUC_lexicon_tags_26 },
    { 0xf8d46b01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d46b06, SUC_lexicon_tags_25 },
    { 0xbd346b07, SUC_lexicon_tags_2 },
    { 0x56346b08, SUC_lexicon_tags_11 },
    { 0xef946b09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x88946b0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e746b0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26946b15, SUC_lexicon_tags_33 },
    { 0x63946b15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e146b1a, SUC_lexicon_tags_25 },
    { 0x4146b1b, SUC_lexicon_tags_17 },
    { 0x25946b1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb946b20, SUC_lexicon_tags_23 },
    { 0x96b46b20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28346b26, SUC_lexicon_tags_45 },
    { 0x4e746b27, SUC_lexicon_tags_56 },
    { 0x84b46b28, SUC_lexicon_tags_25 },
    { 0x7b146b29, SUC_lexicon_tags_2 },
    { 0xc9b46b2a, SUC_lexicon_tags_8 },
    { 0xbdd46b2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13546b2f, SUC_lexicon_tags_16 },
    { 0x63346b2f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x41546b32, SUC_lexicon_tags_6 },
    { 0x9f46b32, SUC_lexicon_tags_15 },
    { 0x77946b34, SUC_lexicon_tags_17 },
    { 0xde346b35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d46b3a, SUC_lexicon_tags_7 },
    { 0x1cd46b3b, SUC_lexicon_tags_77 },
    { 0xc2746b3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea346b41, SUC_lexicon_tags_29 },
    { 0x40f46b42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87746b48, SUC_lexicon_tags_2 },
    { 0x31946b49, SUC_lexicon_tags_16 },
    { 0x92b46b4a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcb546b4c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb7346b4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f46b52, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe4b46b54, SUC_lexicon_tags_26 },
    { 0x7ad46b55, SUC_lexicon_tags_48 },
    { 0xbbf46b54, SUC_lexicon_tags_11 },
    { 0x35946b54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35946b59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x79146b5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22d46b5e, SUC_lexicon_tags_12 },
    { 0x2f546b5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12946b63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f46b69, SUC_lexicon_tags_39 },
    { 0xea546b69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61746b6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b546b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3746b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32546b79, SUC_lexicon_tags_10 },
    { 0x35146b7a, SUC_lexicon_tags_45 },
    { 0x6ef46b7a, SUC_lexicon_tags_33 },
    { 0x5f346b7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71946b7e, SUC_lexicon_tags_25 },
    { 0xe9346b7e, SUC_lexicon_tags_0 },
    { 0xec746b80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c546b83, SUC_lexicon_tags_172 },
    { 0xf2f46b84, SUC_lexicon_tags_2 },
    { 0xc2546b85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df46b89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f46b8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f746b8e, SUC_lexicon_tags_19 },
    { 0x14b46b8f, SUC_lexicon_tags_26 },
    { 0x64d46b90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x85946b92, SUC_lexicon_tags_17 },
    { 0x87946b93, SUC_lexicon_tags_28 },
    { 0x94746b92, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9eb46b96, SUC_lexicon_tags_47 },
    { 0xf8946b97, SUC_lexicon_tags_26 },
    { 0x35346b98, SUC_lexicon_tags_15 },
    { 0xe3b46b99, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xba946b9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7746b9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62946ba1, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x88346ba3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b46ba7, SUC_lexicon_tags_1 },
    { 0xed946ba8, SUC_lexicon_tags_25 },
    { 0xb2d46ba9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57146bae, SUC_lexicon_tags_7 },
    { 0x13546baf, SUC_lexicon_tags_4 },
    { 0x8a546bb0, SUC_lexicon_tags_33 },
    { 0x8b746bb1, SUC_lexicon_tags_14 },
    { 0x4d346bb2, SUC_lexicon_tags_27 },
    { 0x73d46bb3, SUC_lexicon_tags_10 },
    { 0x9b146bb1, SUC_lexicon_tags_47 },
    { 0xf1b46bb5, SUC_lexicon_tags_12 },
    { 0x71746bb5, SUC_lexicon_tags_43 },
    { 0xd5d46bb3, SUC_lexicon_tags_45 },
    { 0x75146baf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3e146bba, SUC_lexicon_tags_68 },
    { 0xa6746bbb, SUC_lexicon_tags_59 },
    { 0x81f46bbc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad46bbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x81946bc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50746bc3, SUC_lexicon_tags_19 },
    { 0x1d946bc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3146bc7, SUC_lexicon_tags_17 },
    { 0x34946bc8, SUC_lexicon_tags_75 },
    { 0x6a946bc8, SUC_lexicon_tags_0 },
    { 0xf8746bca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x66d46bcc, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf746bd0, SUC_lexicon_tags_82 },
    { 0xa5b46bd1, SUC_lexicon_tags_27 },
    { 0x38346bd2, SUC_lexicon_tags_25 },
    { 0xda946bd3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6746bd6, SUC_lexicon_tags_28 },
    { 0xa0d46bd6, SUC_lexicon_tags_61 },
    { 0x18146bd7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xed946bda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc346bdf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x35946be2, SUC_lexicon_tags_31 },
    { 0xa2d46be3, SUC_lexicon_tags_32 },
    { 0x3bf46be3, SUC_lexicon_tags_39 },
    { 0xa8346be5, SUC_lexicon_tags_37 },
    { 0xbeb46be6, SUC_lexicon_tags_10 },
    { 0x2df46be6, SUC_lexicon_tags_37 },
    { 0xe3746be8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b46bef, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc6b46bf1, SUC_lexicon_tags_25 },
    { 0x1746bf2, SUC_lexicon_tags_6 },
    { 0xa2546bf2, SUC_lexicon_tags_31 },
    { 0xb7146bf1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x10146bf7, SUC_lexicon_tags_68 },
    { 0x6346bf8, SUC_lexicon_tags_7 },
    { 0xa1346bf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7146bfb, SUC_lexicon_tags_23 },
    { 0xe9d46bfc, SUC_lexicon_tags_27 },
    { 0x6a946bfc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6c146bff, SUC_lexicon_tags_10 },
    { 0x14b46bff, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b46c03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe2546c05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f346c08, SUC_lexicon_tags_1 },
    { 0x4c946c08, SUC_lexicon_tags_15 },
    { 0x37d46c0a, SUC_lexicon_tags_3 },
    { 0x4cb46c0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba546c0d, SUC_lexicon_tags_49 },
    { 0x3a946c0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8746c12, SUC_lexicon_tags_37 },
    { 0x16146c13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6946c17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d46c1a, SUC_lexicon_tags_17 },
    { 0x3c346c1a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5fb46c1d, SUC_lexicon_tags_33 },
    { 0x87746c1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95146c23, SUC_lexicon_tags_25 },
    { 0x84746c24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9346c27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8d146c29, SUC_lexicon_tags_38 },
    { 0x70546c29, SUC_lexicon_tags_0 },
    { 0x37d46c2b, SUC_lexicon_tags_46 },
    { 0xfcb46c2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df46c31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46546c36, SUC_lexicon_tags_37 },
    { 0x18146c37, SUC_lexicon_tags_11 },
    { 0x16f46c37, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x61346c3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12346c3e, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2546c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x946c48, SUC_lexicon_tags_17 },
    { 0xe0346c49, SUC_lexicon_tags_15 },
    { 0x4b546c4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9546c4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b46c53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b46c56, SUC_lexicon_tags_2 },
    { 0x6e946c57, SUC_lexicon_tags_87 },
    { 0xbdd46c58, SUC_lexicon_tags_8 },
    { 0x4f546c59, SUC_lexicon_tags_14 },
    { 0xec346c57, SUC_lexicon_tags_25 },
    { 0x71b46c58, SUC_lexicon_tags_43 },
    { 0x1a946c56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22146c5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f46c65, SUC_lexicon_tags_11 },
    { 0xfc746c65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde946c68, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d46c70, SUC_lexicon_tags_23 },
    { 0x12146c70, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b746c79, SUC_lexicon_tags_25 },
    { 0x19746c79, SUC_lexicon_tags_14 },
    { 0x55746c79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x59946c7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ef46c7f, SUC_lexicon_tags_17 },
    { 0x36146c7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a546c82, SUC_lexicon_tags_27 },
    { 0x6c146c83, SUC_lexicon_tags_37 },
    { 0x53f46c84, SUC_lexicon_tags_12 },
    { 0xe7146c84, SUC_lexicon_tags_25 },
    { 0x9cf46c86, SUC_lexicon_tags_29 },
    { 0x8b146c87, SUC_lexicon_tags_46 },
    { 0x26b46c84, SUC_lexicon_tags_29 },
    { 0x6af46c89, SUC_lexicon_tags_12 },
    { 0x3f46c85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf346c8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3ed46c8e, SUC_lexicon_tags_12 },
    { 0x1c946c8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0946c91, SUC_lexicon_tags_151 },
    { 0x51d46c91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48346c94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd46c98, SUC_lexicon_tags_4 },
    { 0x94546c99, SUC_lexicon_tags_17 },
    { 0x80146c98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb346c9d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81146ca7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f46caa, SUC_lexicon_tags_29 },
    { 0x8c946cab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f746cad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x25946cb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf46cb4, SUC_lexicon_tags_29 },
    { 0x8146cb5, SUC_lexicon_tags_11 },
    { 0x72f46cb6, SUC_lexicon_tags_12 },
    { 0x6a746cb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f946cc4, SUC_lexicon_tags_2 },
    { 0x50546cc5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad46cc9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0546ccd, SUC_lexicon_tags_11 },
    { 0x74d46cce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb46cd2, SUC_lexicon_tags_37 },
    { 0xce746cd2, SUC_lexicon_tags_15 },
    { 0x8bf46cd2, SUC_lexicon_tags_4 },
    { 0xd3346cd2, SUC_lexicon_tags_15 },
    { 0xbc546cd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99746cda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b46cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53546ce1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ff46ce3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f46ce7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x73546cea, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1c346cec, SUC_lexicon_tags_38 },
    { 0x2b746ced, SUC_lexicon_tags_2 },
    { 0x82d46cec, SUC_lexicon_tags_17 },
    { 0xf9346cef, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xf2146cf1, SUC_lexicon_tags_6 },
    { 0x5bb46cf1, SUC_lexicon_tags_33 },
    { 0xee746cf1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d46cf8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe546cfb, SUC_lexicon_tags_11 },
    { 0x346cfc, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d46cff, SUC_lexicon_tags_12 },
    { 0x6f946d00, SUC_lexicon_tags_17 },
    { 0xed346d00, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1546d04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3b46d06, SUC_lexicon_tags_13 },
    { 0xb9f46d07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52946d09, SUC_lexicon_tags_10 },
    { 0x98146d0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34f46d0c, SUC_lexicon_tags_43 },
    { 0xad46d0c, SUC_lexicon_tags_16 },
    { 0x59746d0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f46d12, SUC_lexicon_tags_79 },
    { 0xf146d13, SUC_lexicon_tags_123 },
    { 0xc2d46d13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f46d19, SUC_lexicon_tags_68 },
    { 0x5eb46d19, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af46d1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52146d24, SUC_lexicon_tags_6 },
    { 0x9cf46d25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c946d28, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9746d2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0546d2c, SUC_lexicon_tags_15 },
    { 0x51b46d2d, SUC_lexicon_tags_15 },
    { 0x1a546d2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0946d30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6c146d32, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x60546d35, SUC_lexicon_tags_1 },
    { 0x44346d35, SUC_lexicon_tags_2 },
    { 0x1146d37, SUC_lexicon_tags_33 },
    { 0x55f46d38, SUC_lexicon_tags_4 },
    { 0xe4546d35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24f46d3b, SUC_lexicon_tags_11 },
    { 0x4ff46d3c, SUC_lexicon_tags_46 },
    { 0xd9546d3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97f46d3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec546d46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8f46d48, SUC_lexicon_tags_66 },
    { 0x95746d49, SUC_lexicon_tags_1 },
    { 0x37f46d4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf746d4d, SUC_lexicon_tags_37 },
    { 0x14546d4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4eb46d50, SUC_lexicon_tags_11 },
    { 0x75546d51, SUC_lexicon_tags_14 },
    { 0xf5946d52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26546d54, SUC_lexicon_tags_11 },
    { 0x44d46d54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4946d5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51746d62, SUC_lexicon_tags_24 },
    { 0xe0f46d62, SUC_lexicon_tags_11 },
    { 0x8e746d64, SUC_lexicon_tags_38 },
    { 0x6cd46d65, SUC_lexicon_tags_17 },
    { 0x42f46d66, SUC_lexicon_tags_15 },
    { 0xfaf46d67, SUC_lexicon_tags_37 },
    { 0x5a746d68, SUC_lexicon_tags_23 },
    { 0xe6d46d66, SUC_lexicon_tags_15 },
    { 0xcbf46d65, SUC_lexicon_tags_42 },
    { 0xd1346d68, SUC_lexicon_tags_38 },
    { 0x15546d6c, SUC_lexicon_tags_12 },
    { 0x4b546d64, SUC_lexicon_tags_32 },
    { 0x81d46d6e, SUC_lexicon_tags_11 },
    { 0x8346d6f, SUC_lexicon_tags_19 },
    { 0xead46d70, SUC_lexicon_tags_45 },
    { 0x8a746d71, SUC_lexicon_tags_43 },
    { 0xa4b46d6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a946d74, SUC_lexicon_tags_17 },
    { 0x14d46d75, SUC_lexicon_tags_7 },
    { 0x13f46d75, SUC_lexicon_tags_19 },
    { 0x63b46d75, SUC_lexicon_tags_11 },
    { 0xd6746d77, SUC_lexicon_tags_2 },
    { 0x5bd46d79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1d46d7b, SUC_lexicon_tags_47 },
    { 0x95746d7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12746d87, SUC_lexicon_tags_13 },
    { 0x9346d87, SUC_lexicon_tags_133 },
    { 0xc7b46d89, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b46d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1146d99, SUC_lexicon_tags_60 },
    { 0xed546d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f46da0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a146da3, SUC_lexicon_tags_23 },
    { 0xc8546da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31346da7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc346daa, SUC_lexicon_tags_30 },
    { 0xdbb46daa, SUC_lexicon_tags_17 },
    { 0xf1346dac, SUC_lexicon_tags_1 },
    { 0xe4946dac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c746db0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x78946db2, SUC_lexicon_tags_16 },
    { 0x45b46db3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5746db8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c146dbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x68946dbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f46dc0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b46dc3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf5946dc5, SUC_lexicon_tags_19 },
    { 0x37b46dc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d46dcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f46dcf, SUC_lexicon_tags_2 },
    { 0x56346dcf, SUC_lexicon_tags_37 },
    { 0x4ed46dd1, SUC_lexicon_tags_1 },
    { 0x32346dd2, SUC_lexicon_tags_7 },
    { 0x1cf46dd3, SUC_lexicon_tags_14 },
    { 0xca946dd4, SUC_lexicon_tags_21 },
    { 0x1b746dd3, SUC_lexicon_tags_2 },
    { 0x20946dd6, SUC_lexicon_tags_25 },
    { 0x24b46dd7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b46dda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cf46ddc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24346de2, SUC_lexicon_tags_17 },
    { 0x53f46de3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d46de6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d46ded, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd746df0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x65146df3, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92946dfb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa6746dfd, SUC_lexicon_tags_34 },
    { 0xea546dfe, SUC_lexicon_tags_14 },
    { 0x37d46dfd, SUC_lexicon_tags_11 },
    { 0x64d46e00, SUC_lexicon_tags_27 },
    { 0x41946dfd, SUC_lexicon_tags_60 },
    { 0x5bb46e00, SUC_lexicon_tags_69 },
    { 0xf0b46e03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8346e05, SUC_lexicon_tags_66 },
    { 0xf3546e06, SUC_lexicon_tags_66 },
    { 0xa2f46e06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3f546e09, SUC_lexicon_tags_45 },
    { 0xa6746e0a, SUC_lexicon_tags_37 },
    { 0xb946e0b, SUC_lexicon_tags_60 },
    { 0x3c146e0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec346e0e, SUC_lexicon_tags_44 },
    { 0xe0346e0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7746e14, SUC_lexicon_tags_63 },
    { 0xffb46e15, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xca146e18, SUC_lexicon_tags_301 },
    { 0x12f46e19, SUC_lexicon_tags_23 },
    { 0x14546e18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa346e20, SUC_lexicon_tags_84 },
    { 0x67d46e21, SUC_lexicon_tags_66 },
    { 0xdab46e22, SUC_lexicon_tags_15 },
    { 0x6c546e22, SUC_lexicon_tags_0 },
    { 0xfd546e23, SUC_lexicon_tags_43 },
    { 0x60146e23, SUC_lexicon_tags_7 },
    { 0x28946e25, SUC_lexicon_tags_35 },
    { 0x33746e27, SUC_lexicon_tags_49 },
    { 0x49f46e28, SUC_lexicon_tags_30 },
    { 0xca746e29, SUC_lexicon_tags_25 },
    { 0xa9746e22, SUC_lexicon_tags_15 },
    { 0xf0d46e23, SUC_lexicon_tags_17 },
    { 0xe0146e2c, SUC_lexicon_tags_63 },
    { 0xa5f46e25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61d46e2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff46e32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51346e35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff46e38, SUC_lexicon_tags_57 },
    { 0x5a346e39, SUC_lexicon_tags_25 },
    { 0xc3346e3a, SUC_lexicon_tags_41 },
    { 0x61f46e3a, SUC_lexicon_tags_14 },
    { 0x24946e3c, SUC_lexicon_tags_23 },
    { 0xb3146e3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3946e44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39346e48, SUC_lexicon_tags_25 },
    { 0x8fb46e49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3346e4c, SUC_lexicon_tags_7 },
    { 0xd1146e4d, SUC_lexicon_tags_6 },
    { 0xc7346e4d, SUC_lexicon_tags_12 },
    { 0xa3d46e4f, SUC_lexicon_tags_27 },
    { 0x53146e4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf746e56, SUC_lexicon_tags_27 },
    { 0xcfd46e57, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d946e62, SUC_lexicon_tags_29 },
    { 0x50146e63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47546e68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1546e6d, SUC_lexicon_tags_43 },
    { 0xdb46e6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e146e70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d546e79, SUC_lexicon_tags_129 },
    { 0xbff46e7a, SUC_lexicon_tags_17 },
    { 0x71746e7b, SUC_lexicon_tags_63 },
    { 0x3bf46e7b, SUC_lexicon_tags_27 },
    { 0x18d46e79, SUC_lexicon_tags_17 },
    { 0x97146e7e, SUC_lexicon_tags_28 },
    { 0x42546e7e, SUC_lexicon_tags_38 },
    { 0x49546e80, SUC_lexicon_tags_3 },
    { 0x3f46e81, SUC_lexicon_tags_17 },
    { 0x8eb46e7e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38746e84, SUC_lexicon_tags_11 },
    { 0xf8346e84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d46e88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9b46e8a, SUC_lexicon_tags_68 },
    { 0x46f46e8b, SUC_lexicon_tags_9 },
    { 0x65146e8c, SUC_lexicon_tags_26 },
    { 0xe7946e8b, SUC_lexicon_tags_66 },
    { 0x7b146e8d, SUC_lexicon_tags_318 },
    { 0xa7946e8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb946e93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a946e95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d546e97, SUC_lexicon_tags_47 },
    { 0x39746e98, SUC_lexicon_tags_15 },
    { 0xea146e98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xee946e9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c746ea0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66746ea4, SUC_lexicon_tags_23 },
    { 0xa7146ea5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x34b46ea7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b46eb0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xe9946eb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f46eb7, SUC_lexicon_tags_11 },
    { 0xdb546eb8, SUC_lexicon_tags_57 },
    { 0xd2d46eb7, SUC_lexicon_tags_39 },
    { 0x3ed46eba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f546ebe, SUC_lexicon_tags_2 },
    { 0xedb46ebf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b46ec2, SUC_lexicon_tags_49 },
    { 0x7d346ec2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3946ec7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63b46ec9, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0946ed1, SUC_lexicon_tags_19 },
    { 0x9146ed1, SUC_lexicon_tags_0 },
    { 0x1c746ed3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f146ed6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b46edf, SUC_lexicon_tags_23 },
    { 0x2346ee0, SUC_lexicon_tags_60 },
    { 0xc1946ee0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9946ee9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f46eee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb46ef4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67546ef9, SUC_lexicon_tags_45 },
    { 0x11d46ef9, SUC_lexicon_tags_23 },
    { 0xa3546efb, SUC_lexicon_tags_33 },
    { 0x79d46efc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0746efe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f746f03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6346f08, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8546f0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42946f12, SUC_lexicon_tags_11 },
    { 0xea546f13, SUC_lexicon_tags_37 },
    { 0x9546f13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb546f1b, SUC_lexicon_tags_17 },
    { 0x5b146f1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1346f1e, SUC_lexicon_tags_60 },
    { 0x6d546f1e, SUC_lexicon_tags_2 },
    { 0x6a346f20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb46f23, SUC_lexicon_tags_2 },
    { 0x82b46f24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd1346f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f346f2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d46f31, SUC_lexicon_tags_29 },
    { 0x8af46f31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b146f34, SUC_lexicon_tags_13 },
    { 0x2a146f35, SUC_lexicon_tags_11 },
    { 0x37546f36, SUC_lexicon_tags_7 },
    { 0x3f746f37, SUC_lexicon_tags_3 },
    { 0xb1946f38, SUC_lexicon_tags_63 },
    { 0x81746f38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4b46f3b, SUC_lexicon_tags_11 },
    { 0x38546f3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0146f41, SUC_lexicon_tags_2 },
    { 0xe7746f41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1ef46f44, SUC_lexicon_tags_25 },
    { 0x8fb46f44, SUC_lexicon_tags_55 },
    { 0x6e746f46, SUC_lexicon_tags_18 },
    { 0x3af46f47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d46f4b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3746f51, SUC_lexicon_tags_28 },
    { 0xa4946f52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5346f5a, SUC_lexicon_tags_12 },
    { 0xae946f5b, SUC_lexicon_tags_63 },
    { 0x26f46f5c, SUC_lexicon_tags_15 },
    { 0xaf146f5d, SUC_lexicon_tags_11 },
    { 0xdc746f5a, SUC_lexicon_tags_14 },
    { 0x3ff46f5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f46f67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf46f6b, SUC_lexicon_tags_17 },
    { 0x58b46f6b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc346f6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd46f71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d146f76, SUC_lexicon_tags_2 },
    { 0xdb146f77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa5946f79, SUC_lexicon_tags_37 },
    { 0xaf146f7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd46f7e, SUC_lexicon_tags_11 },
    { 0xaa346f7f, SUC_lexicon_tags_57 },
    { 0x2d46f7e, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65146f84, SUC_lexicon_tags_28 },
    { 0x29746f85, SUC_lexicon_tags_23 },
    { 0x56746f84, SUC_lexicon_tags_23 },
    { 0x6346f85, SUC_lexicon_tags_31 },
    { 0x46546f85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c746f8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f46f8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71346f8f, SUC_lexicon_tags_1 },
    { 0x79146f8f, SUC_lexicon_tags_37 },
    { 0x9dd46f90, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead46f95, SUC_lexicon_tags_7 },
    { 0x2a346f96, SUC_lexicon_tags_11 },
    { 0xc5546f97, SUC_lexicon_tags_32 },
    { 0xaff46f97, SUC_lexicon_tags_47 },
    { 0xa8d46f99, SUC_lexicon_tags_25 },
    { 0x1f746f99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb46fa1, SUC_lexicon_tags_11 },
    { 0xb9b46fa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49146fa4, SUC_lexicon_tags_21 },
    { 0xa3d46fa5, SUC_lexicon_tags_11 },
    { 0xf8146fa4, SUC_lexicon_tags_39 },
    { 0xd4746fa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc146fa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50146fb0, SUC_lexicon_tags_37 },
    { 0x80d46fb1, SUC_lexicon_tags_21 },
    { 0xaa346fb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49546fb5, SUC_lexicon_tags_66 },
    { 0x95f46fb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d746fc2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc746fcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c946fcd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd5d46fcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f46fd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc546fd5, SUC_lexicon_tags_74 },
    { 0xc3b46fd6, SUC_lexicon_tags_0 },
    { 0x1b946fd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b346fd9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f46fdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d46fe3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db46fe9, SUC_lexicon_tags_17 },
    { 0x4e146fe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27546fec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94546ff0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf946ff3, SUC_lexicon_tags_66 },
    { 0xeaf46ff4, SUC_lexicon_tags_27 },
    { 0x9f946ff5, SUC_lexicon_tags_11 },
    { 0xee346ff6, SUC_lexicon_tags_13 },
    { 0x50146ff7, SUC_lexicon_tags_612 },
    { 0x26546ff8, SUC_lexicon_tags_7 },
    { 0x6d346ff5, SUC_lexicon_tags_17 },
    { 0x57946ffa, SUC_lexicon_tags_38 },
    { 0x86b46ffb, SUC_lexicon_tags_15 },
    { 0xc0746ff3, SUC_lexicon_tags_37 },
    { 0xea946ffd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57947001, SUC_lexicon_tags_11 },
    { 0xc0947002, SUC_lexicon_tags_7 },
    { 0xb747002, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66547009, SUC_lexicon_tags_32 },
    { 0xeab4700a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb47011, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc1747013, SUC_lexicon_tags_25 },
    { 0x72f47014, SUC_lexicon_tags_7 },
    { 0x6dd47014, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57d47017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b94701d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f547022, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b4702b, SUC_lexicon_tags_1 },
    { 0x34b4702c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d94702f, SUC_lexicon_tags_28 },
    { 0xa1147030, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e347033, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf347039, SUC_lexicon_tags_47 },
    { 0x8cb4703a, SUC_lexicon_tags_191 },
    { 0x5cb4703b, SUC_lexicon_tags_27 },
    { 0x8d74703a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xddf4703e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c947041, SUC_lexicon_tags_26 },
    { 0x88f47041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4cf47044, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d47048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf94704d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6547056, SUC_lexicon_tags_19 },
    { 0x82f47057, SUC_lexicon_tags_17 },
    { 0x5af47058, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f4705c, SUC_lexicon_tags_1 },
    { 0x32b4705c, SUC_lexicon_tags_2 },
    { 0x5754705d, SUC_lexicon_tags_60 },
    { 0xe194705c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x93347061, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd47064, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d47067, SUC_lexicon_tags_11 },
    { 0x45f47067, SUC_lexicon_tags_47 },
    { 0x5e147067, SUC_lexicon_tags_11 },
    { 0x12f4706a, SUC_lexicon_tags_26 },
    { 0x4c14706b, SUC_lexicon_tags_12 },
    { 0x7234706c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad747070, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4547072, SUC_lexicon_tags_63 },
    { 0x6d147073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b47076, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a947079, SUC_lexicon_tags_27 },
    { 0xdcd4707a, SUC_lexicon_tags_14 },
    { 0xdd14707b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea747083, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3cd47085, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x174708a, SUC_lexicon_tags_13 },
    { 0x8e54708b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe414708f, SUC_lexicon_tags_87 },
    { 0xcab47090, SUC_lexicon_tags_30 },
    { 0xe6147091, SUC_lexicon_tags_17 },
    { 0x9f947092, SUC_lexicon_tags_37 },
    { 0x8df47093, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47547096, SUC_lexicon_tags_37 },
    { 0xc9547096, SUC_lexicon_tags_174 },
    { 0x5cd47097, SUC_lexicon_tags_26 },
    { 0x62f47099, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdf14709b, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f470a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd9b470a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a3470a7, SUC_lexicon_tags_30 },
    { 0xe11470a7, SUC_lexicon_tags_23 },
    { 0xb1f470a9, SUC_lexicon_tags_7 },
    { 0xf1470a9, SUC_lexicon_tags_8 },
    { 0x781470ab, SUC_lexicon_tags_7 },
    { 0x4e7470a7, SUC_lexicon_tags_38 },
    { 0x6d5470ab, SUC_lexicon_tags_33 },
    { 0xfd5470a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3470b1, SUC_lexicon_tags_1 },
    { 0x26b470b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cb470b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5470b8, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xda7470ba, SUC_lexicon_tags_79 },
    { 0x6d9470bb, SUC_lexicon_tags_37 },
    { 0x7cf470bc, SUC_lexicon_tags_9 },
    { 0x787470bc, SUC_lexicon_tags_25 },
    { 0x665470be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab470c1, SUC_lexicon_tags_28 },
    { 0x8bd470c2, SUC_lexicon_tags_207 },
    { 0x847470c3, SUC_lexicon_tags_51 },
    { 0x191470c2, SUC_lexicon_tags_17 },
    { 0x8af470c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5470c8, SUC_lexicon_tags_47 },
    { 0xc39470c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x355470cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f470d2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61470d5, SUC_lexicon_tags_1 },
    { 0x87d470d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17d470d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x733470da, SUC_lexicon_tags_11 },
    { 0x9b9470db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3470e0, SUC_lexicon_tags_24 },
    { 0x833470e0, SUC_lexicon_tags_69 },
    { 0xc0d470e2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x65b470e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb9d470e6, SUC_lexicon_tags_45 },
    { 0xa09470e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df470ec, SUC_lexicon_tags_29 },
    { 0xa4d470ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x233470f0, SUC_lexicon_tags_19 },
    { 0x6df470f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b470f6, SUC_lexicon_tags_15 },
    { 0x917470f7, SUC_lexicon_tags_19 },
    { 0xa2b470f6, SUC_lexicon_tags_11 },
    { 0x1d7470f9, SUC_lexicon_tags_37 },
    { 0x13d470f9, SUC_lexicon_tags_28 },
    { 0x97f470fa, SUC_lexicon_tags_15 },
    { 0x889470fc, SUC_lexicon_tags_27 },
    { 0x805470fd, SUC_lexicon_tags_5 },
    { 0xf6d470fd, SUC_lexicon_tags_60 },
    { 0x50b470ff, SUC_lexicon_tags_37 },
    { 0xa93470ff, SUC_lexicon_tags_27 },
    { 0x3e347100, SUC_lexicon_tags_27 },
    { 0x11470ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b47105, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a747107, SUC_lexicon_tags_63 },
    { 0x9e347108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb4710d, SUC_lexicon_tags_11 },
    { 0xf354710d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x31747111, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5e947113, SUC_lexicon_tags_1 },
    { 0xdc947113, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e147119, SUC_lexicon_tags_15 },
    { 0x6034711a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6f94711c, SUC_lexicon_tags_27 },
    { 0xcff4711c, SUC_lexicon_tags_7 },
    { 0x43f4711e, SUC_lexicon_tags_37 },
    { 0xbab4711d, SUC_lexicon_tags_1 },
    { 0x27947120, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc4b47122, SUC_lexicon_tags_29 },
    { 0x73b47123, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x46147126, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe6b47128, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb4712c, SUC_lexicon_tags_69 },
    { 0xbab4712c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d4712f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90747138, SUC_lexicon_tags_60 },
    { 0x83f47139, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d47141, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b47145, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff147147, SUC_lexicon_tags_37 },
    { 0xec347148, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6094714c, SUC_lexicon_tags_46 },
    { 0x8e94714d, SUC_lexicon_tags_29 },
    { 0x3df4714d, SUC_lexicon_tags_7 },
    { 0xa5f4714f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7547152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e147154, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x68347157, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x17f47159, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce94715b, SUC_lexicon_tags_4 },
    { 0x8d4715c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a947161, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5b47163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd547165, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5674716c, SUC_lexicon_tags_15 },
    { 0x4074716d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f147171, SUC_lexicon_tags_37 },
    { 0xf0347172, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36547177, SUC_lexicon_tags_91 },
    { 0x37d47178, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf794717b, SUC_lexicon_tags_45 },
    { 0xfb14717c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d47183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d547185, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9947188, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3694718d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa74718f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d47195, SUC_lexicon_tags_44 },
    { 0x3d747196, SUC_lexicon_tags_7 },
    { 0x8c547197, SUC_lexicon_tags_27 },
    { 0x46547198, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e94719d, SUC_lexicon_tags_37 },
    { 0x6654719e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247471a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8cd471a6, SUC_lexicon_tags_28 },
    { 0xd8d471a7, SUC_lexicon_tags_2 },
    { 0x6e3471a7, SUC_lexicon_tags_7 },
    { 0x5f471a9, SUC_lexicon_tags_37 },
    { 0x271471aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d7471ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xacf471ae, SUC_lexicon_tags_17 },
    { 0x195471af, SUC_lexicon_tags_33 },
    { 0x617471af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64b471b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x701471b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661471ba, SUC_lexicon_tags_37 },
    { 0xf59471bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21471c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x825471c2, SUC_lexicon_tags_31 },
    { 0x16b471c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07471c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x543471cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x537471ce, SUC_lexicon_tags_16 },
    { 0xe09471cf, SUC_lexicon_tags_25 },
    { 0xbe5471cf, SUC_lexicon_tags_1 },
    { 0xe91471d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b471d6, SUC_lexicon_tags_7 },
    { 0x239471d6, SUC_lexicon_tags_30 },
    { 0x78b471d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf6f471da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3471dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8f471df, SUC_lexicon_tags_37 },
    { 0x51f471df, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7471e6, SUC_lexicon_tags_0 },
    { 0x48b471e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9bd471e9, SUC_lexicon_tags_2 },
    { 0x627471ea, SUC_lexicon_tags_47 },
    { 0x8f3471e9, SUC_lexicon_tags_15 },
    { 0x633471ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x113471ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78d471f0, SUC_lexicon_tags_15 },
    { 0xe11471f1, SUC_lexicon_tags_47 },
    { 0x98d471f1, SUC_lexicon_tags_0 },
    { 0xc2b471f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd471fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41471fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7747200, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff47204, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd747206, SUC_lexicon_tags_25 },
    { 0xf7f47206, SUC_lexicon_tags_37 },
    { 0x5f947208, SUC_lexicon_tags_47 },
    { 0x35947208, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb9d4720b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf6d4720d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d547212, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9147219, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8634721b, SUC_lexicon_tags_39 },
    { 0x154721b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7414721f, SUC_lexicon_tags_7 },
    { 0xc1b4721f, SUC_lexicon_tags_27 },
    { 0x12d4721f, SUC_lexicon_tags_12 },
    { 0x8d147222, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd47228, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb8b4722a, SUC_lexicon_tags_47 },
    { 0x4554722b, SUC_lexicon_tags_31 },
    { 0xaf34722c, SUC_lexicon_tags_85 },
    { 0xefb4722d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91547232, SUC_lexicon_tags_48 },
    { 0x3bd47232, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x75547236, SUC_lexicon_tags_37 },
    { 0x74547237, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b74723a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0747241, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7f47243, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17b47245, SUC_lexicon_tags_12 },
    { 0xe8547246, SUC_lexicon_tags_63 },
    { 0x78b47246, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x53147249, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8614724c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3074724f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d47252, SUC_lexicon_tags_1 },
    { 0x68747253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe147255, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f47258, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48747261, SUC_lexicon_tags_12 },
    { 0x7b47262, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab347268, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4726f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf747271, SUC_lexicon_tags_0 },
    { 0x14747272, SUC_lexicon_tags_15 },
    { 0x7bb47273, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3947278, SUC_lexicon_tags_45 },
    { 0x20347279, SUC_lexicon_tags_11 },
    { 0x67547279, SUC_lexicon_tags_19 },
    { 0x5a54727a, SUC_lexicon_tags_38 },
    { 0xffb47279, SUC_lexicon_tags_15 },
    { 0x6134727d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd747280, SUC_lexicon_tags_55 },
    { 0x19f47281, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21547283, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xde347286, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd47289, SUC_lexicon_tags_26 },
    { 0xa7747289, SUC_lexicon_tags_4 },
    { 0xb8147289, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb4728e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7ef47290, SUC_lexicon_tags_2 },
    { 0x3ef47290, SUC_lexicon_tags_15 },
    { 0x2c147290, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x94147295, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d34729c, SUC_lexicon_tags_37 },
    { 0x8214729d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa374729f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ab472a1, SUC_lexicon_tags_37 },
    { 0xabf472a2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xe3f472a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9472a7, SUC_lexicon_tags_14 },
    { 0xb97472a7, SUC_lexicon_tags_27 },
    { 0x819472a8, SUC_lexicon_tags_15 },
    { 0xb1f472aa, SUC_lexicon_tags_15 },
    { 0x3d7472aa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x529472ad, SUC_lexicon_tags_37 },
    { 0xd15472ae, SUC_lexicon_tags_62 },
    { 0xc51472ad, SUC_lexicon_tags_19 },
    { 0x715472b0, SUC_lexicon_tags_404 },
    { 0, NULL },
    { 0x4c1472b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedf472b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d472b8, SUC_lexicon_tags_19 },
    { 0x673472b8, SUC_lexicon_tags_47 },
    { 0x6df472ba, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55472c0, SUC_lexicon_tags_49 },
    { 0x8ad472c1, SUC_lexicon_tags_3 },
    { 0x149472c2, SUC_lexicon_tags_17 },
    { 0x55d472c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9472c8, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd472cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95472d5, SUC_lexicon_tags_19 },
    { 0x357472d5, SUC_lexicon_tags_0 },
    { 0xd472d5, SUC_lexicon_tags_1 },
    { 0x131472d8, SUC_lexicon_tags_47 },
    { 0x805472d9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87472e2, SUC_lexicon_tags_8 },
    { 0xa31472e2, SUC_lexicon_tags_12 },
    { 0xe01472e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af472ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f9472f1, SUC_lexicon_tags_32 },
    { 0x6eb472f1, SUC_lexicon_tags_3 },
    { 0x123472f3, SUC_lexicon_tags_7 },
    { 0x6b3472f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa547300, SUC_lexicon_tags_12 },
    { 0x23147301, SUC_lexicon_tags_20 },
    { 0x4c147302, SUC_lexicon_tags_14 },
    { 0x97747303, SUC_lexicon_tags_25 },
    { 0x5cd47304, SUC_lexicon_tags_0 },
    { 0xb3947305, SUC_lexicon_tags_1 },
    { 0x60f47305, SUC_lexicon_tags_33 },
    { 0xe3d47307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b54730a, SUC_lexicon_tags_15 },
    { 0xe74730b, SUC_lexicon_tags_10 },
    { 0x7254730c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5d4730e, SUC_lexicon_tags_43 },
    { 0x5794730f, SUC_lexicon_tags_87 },
    { 0x3e747310, SUC_lexicon_tags_65 },
    { 0xb3f47311, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f47315, SUC_lexicon_tags_26 },
    { 0x84547316, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26b47318, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f4731d, SUC_lexicon_tags_38 },
    { 0xbad4731e, SUC_lexicon_tags_25 },
    { 0x4854731d, SUC_lexicon_tags_29 },
    { 0xddf4731d, SUC_lexicon_tags_45 },
    { 0x8694731f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11f47323, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf147332, SUC_lexicon_tags_23 },
    { 0x9c547333, SUC_lexicon_tags_7 },
    { 0xedd47334, SUC_lexicon_tags_11 },
    { 0x6ff47335, SUC_lexicon_tags_12 },
    { 0xd3547335, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab47339, SUC_lexicon_tags_11 },
    { 0x1f74733a, SUC_lexicon_tags_17 },
    { 0x9674733b, SUC_lexicon_tags_19 },
    { 0x87b4733c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5af4733e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x67f47340, SUC_lexicon_tags_32 },
    { 0x9c347340, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a547344, SUC_lexicon_tags_7 },
    { 0x8b147345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9147349, SUC_lexicon_tags_27 },
    { 0x1394734a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3874734d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb47350, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74747353, SUC_lexicon_tags_26 },
    { 0x12547354, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd14735a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe914735e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9147362, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14736a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed34736f, SUC_lexicon_tags_74 },
    { 0x57947370, SUC_lexicon_tags_30 },
    { 0xcf47370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f547373, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc947378, SUC_lexicon_tags_7 },
    { 0xae347378, SUC_lexicon_tags_2 },
    { 0x1414737a, SUC_lexicon_tags_11 },
    { 0x8374737b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb894737f, SUC_lexicon_tags_8 },
    { 0x24d47380, SUC_lexicon_tags_37 },
    { 0xeaf47381, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc347383, SUC_lexicon_tags_55 },
    { 0xdb147384, SUC_lexicon_tags_47 },
    { 0x27f47385, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d47388, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5894738b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10d4738d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f47394, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb354739c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bb4739e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777473a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339473a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe99473aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff473ae, SUC_lexicon_tags_1 },
    { 0x3bd473ae, SUC_lexicon_tags_23 },
    { 0x751473b0, SUC_lexicon_tags_17 },
    { 0xc8f473b1, SUC_lexicon_tags_8 },
    { 0xfd3473b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b473b5, SUC_lexicon_tags_1 },
    { 0xea7473b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925473ba, SUC_lexicon_tags_33 },
    { 0xa39473ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9ad473bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75473c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc5473c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x50d473c5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xa7b473c7, SUC_lexicon_tags_17 },
    { 0x7eb473c8, SUC_lexicon_tags_28 },
    { 0x875473c9, SUC_lexicon_tags_15 },
    { 0xd07473ca, SUC_lexicon_tags_17 },
    { 0x887473c7, SUC_lexicon_tags_91 },
    { 0xd93473c8, SUC_lexicon_tags_70 },
    { 0x5b473cd, SUC_lexicon_tags_29 },
    { 0xdb3473cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x77473d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3473d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1473d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4b473da, SUC_lexicon_tags_17 },
    { 0xc8b473db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c3473dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfb5473df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab473e4, SUC_lexicon_tags_45 },
    { 0x247473e4, SUC_lexicon_tags_47 },
    { 0x2e5473e6, SUC_lexicon_tags_26 },
    { 0x58b473e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5473ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xebf473ec, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab473ef, SUC_lexicon_tags_25 },
    { 0x549473ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7473f4, SUC_lexicon_tags_13 },
    { 0x60f473f5, SUC_lexicon_tags_87 },
    { 0x38d473f5, SUC_lexicon_tags_3 },
    { 0xc97473f7, SUC_lexicon_tags_0 },
    { 0xee3473f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b473fb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xba1473fd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96547402, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9347406, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4740c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1514740e, SUC_lexicon_tags_25 },
    { 0x4834740f, SUC_lexicon_tags_29 },
    { 0x3dd47410, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50147412, SUC_lexicon_tags_11 },
    { 0x6f47413, SUC_lexicon_tags_18 },
    { 0x2d147414, SUC_lexicon_tags_343 },
    { 0xb947412, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf4741d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7747421, SUC_lexicon_tags_45 },
    { 0xb4747422, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7eb47424, SUC_lexicon_tags_23 },
    { 0x9f947425, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d4742d, SUC_lexicon_tags_7 },
    { 0x10f4742e, SUC_lexicon_tags_34 },
    { 0x2534742e, SUC_lexicon_tags_1 },
    { 0xba747430, SUC_lexicon_tags_62 },
    { 0xbd74742e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd2d47433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf347435, SUC_lexicon_tags_60 },
    { 0xa8547435, SUC_lexicon_tags_7 },
    { 0x4c547437, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49b47439, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f34743f, SUC_lexicon_tags_25 },
    { 0x2b34743f, SUC_lexicon_tags_32 },
    { 0x7d94743f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88947445, SUC_lexicon_tags_234 },
    { 0x59747446, SUC_lexicon_tags_82 },
    { 0x5947446, SUC_lexicon_tags_55 },
    { 0x4d147448, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed94744f, SUC_lexicon_tags_39 },
    { 0xaf747450, SUC_lexicon_tags_43 },
    { 0x29747451, SUC_lexicon_tags_54 },
    { 0x21f47450, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x94b47454, SUC_lexicon_tags_92 },
    { 0xbf47455, SUC_lexicon_tags_62 },
    { 0x38347455, SUC_lexicon_tags_47 },
    { 0xa3d47457, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa234745a, SUC_lexicon_tags_25 },
    { 0x41d4745b, SUC_lexicon_tags_17 },
    { 0x9d14745b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa8d4745e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd347460, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d47465, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0d47467, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc547469, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3674746e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf947477, SUC_lexicon_tags_10 },
    { 0xaff47478, SUC_lexicon_tags_1 },
    { 0xabb47479, SUC_lexicon_tags_26 },
    { 0x8fd47479, SUC_lexicon_tags_30 },
    { 0x2c947479, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db4747e, SUC_lexicon_tags_37 },
    { 0xff74747e, SUC_lexicon_tags_27 },
    { 0x33147480, SUC_lexicon_tags_25 },
    { 0x9d14747e, SUC_lexicon_tags_45 },
    { 0x2cd47482, SUC_lexicon_tags_57 },
    { 0x42547480, SUC_lexicon_tags_37 },
    { 0xed34747e, SUC_lexicon_tags_6 },
    { 0x2b47485, SUC_lexicon_tags_47 },
    { 0xba347486, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f147489, SUC_lexicon_tags_11 },
    { 0x88947489, SUC_lexicon_tags_35 },
    { 0x9834748a, SUC_lexicon_tags_17 },
    { 0xca34748c, SUC_lexicon_tags_28 },
    { 0xc4d4748d, SUC_lexicon_tags_46 },
    { 0x8bb4748c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33947491, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57347496, SUC_lexicon_tags_1 },
    { 0x1347497, SUC_lexicon_tags_11 },
    { 0x53f47498, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb4749b, SUC_lexicon_tags_17 },
    { 0x3f74749b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf34749f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d474a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x193474a9, SUC_lexicon_tags_30 },
    { 0xd19474aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7474b1, SUC_lexicon_tags_11 },
    { 0xac7474b1, SUC_lexicon_tags_37 },
    { 0x3f9474b3, SUC_lexicon_tags_17 },
    { 0x8df474b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af474b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293474be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b474c2, SUC_lexicon_tags_11 },
    { 0x84b474c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa79474c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57474cb, SUC_lexicon_tags_6 },
    { 0x8f5474cc, SUC_lexicon_tags_0 },
    { 0x4c5474cd, SUC_lexicon_tags_8 },
    { 0xe99474ce, SUC_lexicon_tags_47 },
    { 0x7ef474cb, SUC_lexicon_tags_32 },
    { 0xe93474ce, SUC_lexicon_tags_2 },
    { 0x5cd474cb, SUC_lexicon_tags_1 },
    { 0xe95474cf, SUC_lexicon_tags_27 },
    { 0x31f474d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f474d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b474dc, SUC_lexicon_tags_1 },
    { 0x3d7474dc, SUC_lexicon_tags_37 },
    { 0x5a9474de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcf9474e0, SUC_lexicon_tags_25 },
    { 0x67d474e1, SUC_lexicon_tags_28 },
    { 0xbd3474e2, SUC_lexicon_tags_15 },
    { 0xecb474e1, SUC_lexicon_tags_45 },
    { 0xf51474e2, SUC_lexicon_tags_120 },
    { 0x1ed474e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b474e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9474ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7474ef, SUC_lexicon_tags_60 },
    { 0xcb5474f0, SUC_lexicon_tags_30 },
    { 0x545474f0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1474f4, SUC_lexicon_tags_47 },
    { 0xb6f474f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cf474f7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191474fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc6d474ff, SUC_lexicon_tags_38 },
    { 0x85347500, SUC_lexicon_tags_23 },
    { 0xda1474ff, SUC_lexicon_tags_37 },
    { 0xeb147502, SUC_lexicon_tags_38 },
    { 0x2ed47503, SUC_lexicon_tags_129 },
    { 0xdf47503, SUC_lexicon_tags_37 },
    { 0xb75474ff, SUC_lexicon_tags_68 },
    { 0x2b747504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac14750f, SUC_lexicon_tags_17 },
    { 0x5b147510, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b47517, SUC_lexicon_tags_37 },
    { 0x49f47518, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5414751b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5754751f, SUC_lexicon_tags_7 },
    { 0x414751f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d747524, SUC_lexicon_tags_1 },
    { 0x8347524, SUC_lexicon_tags_17 },
    { 0x76d47526, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d4752a, SUC_lexicon_tags_9 },
    { 0xde14752a, SUC_lexicon_tags_26 },
    { 0xd974752c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd4752f, SUC_lexicon_tags_19 },
    { 0xdc347530, SUC_lexicon_tags_65 },
    { 0xcc947531, SUC_lexicon_tags_26 },
    { 0xc0347530, SUC_lexicon_tags_2 },
    { 0x1274752f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14947535, SUC_lexicon_tags_8 },
    { 0x90347536, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1c547538, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4b4753a, SUC_lexicon_tags_17 },
    { 0x38f4753b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca74753f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef47543, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2147546, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d47549, SUC_lexicon_tags_23 },
    { 0xb974754a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4754754c, SUC_lexicon_tags_15 },
    { 0xcc74754c, SUC_lexicon_tags_25 },
    { 0x4b34754e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd47552, SUC_lexicon_tags_4 },
    { 0xb6d47553, SUC_lexicon_tags_25 },
    { 0x23547554, SUC_lexicon_tags_55 },
    { 0x7d47555, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2394755c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6934755e, SUC_lexicon_tags_1 },
    { 0x71f4755f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5747563, SUC_lexicon_tags_68 },
    { 0x33947563, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa6347566, SUC_lexicon_tags_1 },
    { 0x40947566, SUC_lexicon_tags_2 },
    { 0x25747568, SUC_lexicon_tags_23 },
    { 0x49f47568, SUC_lexicon_tags_37 },
    { 0x86747567, SUC_lexicon_tags_4 },
    { 0x91f4756b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb574756d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b747571, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb47574, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x28f47576, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x51147578, SUC_lexicon_tags_7 },
    { 0xe4347579, SUC_lexicon_tags_19 },
    { 0xb547578, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b4757d, SUC_lexicon_tags_30 },
    { 0x5af4757e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd47585, SUC_lexicon_tags_9 },
    { 0x1c347586, SUC_lexicon_tags_1 },
    { 0x4d347586, SUC_lexicon_tags_3 },
    { 0x58b47588, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9194758d, SUC_lexicon_tags_11 },
    { 0x9194758e, SUC_lexicon_tags_35 },
    { 0x6334758e, SUC_lexicon_tags_38 },
    { 0xf0747590, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6f547592, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80547594, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f4759a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0f4759c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef74759e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x793475a1, SUC_lexicon_tags_25 },
    { 0x33475a2, SUC_lexicon_tags_38 },
    { 0x10f475a3, SUC_lexicon_tags_14 },
    { 0xc0b475a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x353475a7, SUC_lexicon_tags_61 },
    { 0x283475a7, SUC_lexicon_tags_57 },
    { 0x277475a9, SUC_lexicon_tags_37 },
    { 0x845475aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9475ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281475b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f475b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a9475b7, SUC_lexicon_tags_28 },
    { 0xbd5475b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3475c0, SUC_lexicon_tags_21 },
    { 0x705475c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1475c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc83475c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x15475c8, SUC_lexicon_tags_35 },
    { 0x35d475c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x625475cb, SUC_lexicon_tags_21 },
    { 0x9f475cc, SUC_lexicon_tags_68 },
    { 0x77f475cc, SUC_lexicon_tags_37 },
    { 0x3b1475cb, SUC_lexicon_tags_28 },
    { 0xf03475cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db475d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29475d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x269475d7, SUC_lexicon_tags_29 },
    { 0xc8d475d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa9475da, SUC_lexicon_tags_25 },
    { 0x87475da, SUC_lexicon_tags_11 },
    { 0x45f475dc, SUC_lexicon_tags_12 },
    { 0x407475db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa7475df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1d475e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71475e3, SUC_lexicon_tags_11 },
    { 0xff3475e4, SUC_lexicon_tags_38 },
    { 0x8d1475e3, SUC_lexicon_tags_17 },
    { 0x337475e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98b475e8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x171475ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb475ee, SUC_lexicon_tags_33 },
    { 0x6ef475ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xef7475f1, SUC_lexicon_tags_60 },
    { 0xd0b475f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28147603, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x55f47605, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73147607, SUC_lexicon_tags_9 },
    { 0x9b747607, SUC_lexicon_tags_37 },
    { 0x37147608, SUC_lexicon_tags_17 },
    { 0x9c547607, SUC_lexicon_tags_14 },
    { 0x17f4760b, SUC_lexicon_tags_15 },
    { 0x4b34760c, SUC_lexicon_tags_2 },
    { 0x9554760b, SUC_lexicon_tags_38 },
    { 0xad74760e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x95547611, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b47614, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcb347616, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95947618, SUC_lexicon_tags_55 },
    { 0x4e347619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a74761f, SUC_lexicon_tags_66 },
    { 0x7a547620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad547626, SUC_lexicon_tags_148 },
    { 0x40147626, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8854762a, SUC_lexicon_tags_23 },
    { 0x1cb4762b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc747630, SUC_lexicon_tags_1 },
    { 0xed347631, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70747634, SUC_lexicon_tags_7 },
    { 0xe0947635, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe034763b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34747640, SUC_lexicon_tags_12 },
    { 0x43f47641, SUC_lexicon_tags_47 },
    { 0xecb47642, SUC_lexicon_tags_28 },
    { 0x6df47642, SUC_lexicon_tags_6 },
    { 0x67d47644, SUC_lexicon_tags_6 },
    { 0xbcf47645, SUC_lexicon_tags_27 },
    { 0x51947646, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa947648, SUC_lexicon_tags_1 },
    { 0x34d47648, SUC_lexicon_tags_43 },
    { 0x4a34764a, SUC_lexicon_tags_12 },
    { 0xfb14764a, SUC_lexicon_tags_57 },
    { 0xc94764b, SUC_lexicon_tags_38 },
    { 0x4714764a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f147652, SUC_lexicon_tags_25 },
    { 0x37147653, SUC_lexicon_tags_16 },
    { 0x2d947654, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd147656, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4765c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf4765f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d47662, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8747664, SUC_lexicon_tags_16 },
    { 0x43f47665, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2d147667, SUC_lexicon_tags_28 },
    { 0x41d47667, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd774766b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6e74766d, SUC_lexicon_tags_16 },
    { 0xfc14766d, SUC_lexicon_tags_13 },
    { 0x5974766f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb1347671, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51547673, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f747677, SUC_lexicon_tags_27 },
    { 0x96147678, SUC_lexicon_tags_38 },
    { 0x25147678, SUC_lexicon_tags_46 },
    { 0x4174767a, SUC_lexicon_tags_80 },
    { 0xf4b47679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9594767d, SUC_lexicon_tags_15 },
    { 0x1af4767e, SUC_lexicon_tags_44 },
    { 0xf594767d, SUC_lexicon_tags_37 },
    { 0xcbf47680, SUC_lexicon_tags_1 },
    { 0xd4547681, SUC_lexicon_tags_37 },
    { 0xb8f47681, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96147687, SUC_lexicon_tags_8 },
    { 0x66b47688, SUC_lexicon_tags_23 },
    { 0x56347688, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0b4768b, SUC_lexicon_tags_12 },
    { 0x7634768b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f4768f, SUC_lexicon_tags_0 },
    { 0x1214768f, SUC_lexicon_tags_11 },
    { 0x7eb47691, SUC_lexicon_tags_25 },
    { 0xb3f47690, SUC_lexicon_tags_27 },
    { 0x37347693, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x70947696, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc434769a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7476a0, SUC_lexicon_tags_2 },
    { 0xc5d476a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd476a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09476a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d476ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x547476ae, SUC_lexicon_tags_37 },
    { 0xb41476af, SUC_lexicon_tags_2 },
    { 0xbf9476af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7476b3, SUC_lexicon_tags_4 },
    { 0x14f476b3, SUC_lexicon_tags_15 },
    { 0x127476b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x809476b7, SUC_lexicon_tags_30 },
    { 0x11f476b8, SUC_lexicon_tags_17 },
    { 0x927476b9, SUC_lexicon_tags_64 },
    { 0x379476ba, SUC_lexicon_tags_84 },
    { 0xdf5476bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x435476be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16d476c0, SUC_lexicon_tags_15 },
    { 0x6cb476c0, SUC_lexicon_tags_27 },
    { 0xc8f476c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d476ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd476cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5476d3, SUC_lexicon_tags_37 },
    { 0x827476d4, SUC_lexicon_tags_1 },
    { 0x37476d4, SUC_lexicon_tags_15 },
    { 0x243476d5, SUC_lexicon_tags_1 },
    { 0x789476d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9476da, SUC_lexicon_tags_65 },
    { 0xcd476db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadd476dd, SUC_lexicon_tags_48 },
    { 0x3e1476de, SUC_lexicon_tags_17 },
    { 0xa13476df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb476e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd476e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x607476eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943476f0, SUC_lexicon_tags_25 },
    { 0x587476f1, SUC_lexicon_tags_31 },
    { 0xe5f476f1, SUC_lexicon_tags_66 },
    { 0xf9d476f2, SUC_lexicon_tags_37 },
    { 0xd69476f0, SUC_lexicon_tags_17 },
    { 0xab5476f2, SUC_lexicon_tags_60 },
    { 0x9476f6, SUC_lexicon_tags_19 },
    { 0xacf476f6, SUC_lexicon_tags_37 },
    { 0x561476f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963476fc, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc21476fe, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb547705, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff4770b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d94770f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd47713, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x84947715, SUC_lexicon_tags_38 },
    { 0x45b47715, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9914771c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb4771f, SUC_lexicon_tags_47 },
    { 0xcd947720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75347723, SUC_lexicon_tags_25 },
    { 0x90d47724, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9547727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd074772a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3547731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8b47733, SUC_lexicon_tags_25 },
    { 0x70747734, SUC_lexicon_tags_11 },
    { 0xa347734, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25347737, SUC_lexicon_tags_7 },
    { 0x90f47737, SUC_lexicon_tags_16 },
    { 0xe6947738, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe814773c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed947740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb547743, SUC_lexicon_tags_25 },
    { 0x72747744, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34774a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ad4774c, SUC_lexicon_tags_25 },
    { 0xd454774d, SUC_lexicon_tags_7 },
    { 0x62f4774d, SUC_lexicon_tags_8 },
    { 0x45b4774d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe4747751, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe6547753, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3947757, SUC_lexicon_tags_2 },
    { 0x78347758, SUC_lexicon_tags_23 },
    { 0x42f47759, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb74775d, SUC_lexicon_tags_2 },
    { 0xdb94775e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6147761, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x5a747763, SUC_lexicon_tags_7 },
    { 0xccb47763, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa347768, SUC_lexicon_tags_2 },
    { 0x4a147769, SUC_lexicon_tags_23 },
    { 0x42d4776a, SUC_lexicon_tags_10 },
    { 0x7c34776b, SUC_lexicon_tags_1 },
    { 0x76f4776a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf134776e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d47771, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48d47773, SUC_lexicon_tags_3 },
    { 0x9eb47774, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9354777c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2747780, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b47782, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3f47784, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51947786, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5d47788, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3254778b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2b4778d, SUC_lexicon_tags_27 },
    { 0xe734778d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74f47790, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xba747793, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d47796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90b47798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd4779d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f477a2, SUC_lexicon_tags_33 },
    { 0x279477a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d477a7, SUC_lexicon_tags_23 },
    { 0xcc3477a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d477ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb477b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x413477b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd477ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x725477bd, SUC_lexicon_tags_17 },
    { 0x6d1477be, SUC_lexicon_tags_37 },
    { 0x9e5477bf, SUC_lexicon_tags_17 },
    { 0x641477c0, SUC_lexicon_tags_29 },
    { 0x835477bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c3477c3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6ef477c5, SUC_lexicon_tags_65 },
    { 0x65f477c6, SUC_lexicon_tags_0 },
    { 0xe69477c5, SUC_lexicon_tags_27 },
    { 0x6b3477c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f477cf, SUC_lexicon_tags_38 },
    { 0x7c3477cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf477d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x311477d7, SUC_lexicon_tags_23 },
    { 0x9e3477d8, SUC_lexicon_tags_14 },
    { 0x557477d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745477df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207477e9, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x505477eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf1477ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1477f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d477f4, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x357477f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899477fc, SUC_lexicon_tags_25 },
    { 0x535477fd, SUC_lexicon_tags_47 },
    { 0x4a1477fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x91347800, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81747802, SUC_lexicon_tags_0 },
    { 0x7c747803, SUC_lexicon_tags_794 },
    { 0xb1547804, SUC_lexicon_tags_29 },
    { 0xcd547804, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a34780b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf374780e, SUC_lexicon_tags_56 },
    { 0xcc54780e, SUC_lexicon_tags_3 },
    { 0x63b47810, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42b47812, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7094781b, SUC_lexicon_tags_2 },
    { 0xd134781c, SUC_lexicon_tags_14 },
    { 0xbc74781b, SUC_lexicon_tags_37 },
    { 0xa774781b, SUC_lexicon_tags_7 },
    { 0x3c54781f, SUC_lexicon_tags_220 },
    { 0xc1b47820, SUC_lexicon_tags_15 },
    { 0xc5d47821, SUC_lexicon_tags_19 },
    { 0xd1947821, SUC_lexicon_tags_2 },
    { 0x91d47822, SUC_lexicon_tags_1 },
    { 0xae347823, SUC_lexicon_tags_55 },
    { 0xf7347824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34947828, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6747833, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f47837, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff547839, SUC_lexicon_tags_11 },
    { 0xc814783a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2234783c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e347840, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9947842, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd2947844, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1547847, SUC_lexicon_tags_7 },
    { 0x44947848, SUC_lexicon_tags_19 },
    { 0xd9947849, SUC_lexicon_tags_15 },
    { 0xfc147847, SUC_lexicon_tags_92 },
    { 0xc4547848, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa534784f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce147852, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb547855, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d47859, SUC_lexicon_tags_49 },
    { 0xfe74785a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d4785c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d47860, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x53147862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0f47864, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x81347866, SUC_lexicon_tags_10 },
    { 0xfbf47866, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x44b47869, SUC_lexicon_tags_33 },
    { 0x75b47869, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b4786e, SUC_lexicon_tags_11 },
    { 0xb154786f, SUC_lexicon_tags_37 },
    { 0x6d74786f, SUC_lexicon_tags_39 },
    { 0x45b47871, SUC_lexicon_tags_9 },
    { 0x41f47871, SUC_lexicon_tags_25 },
    { 0x5dd47873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad47877, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf854787a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea14787f, SUC_lexicon_tags_59 },
    { 0xd747880, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x59747882, SUC_lexicon_tags_25 },
    { 0x94b47882, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x4b947885, SUC_lexicon_tags_3 },
    { 0x96f47886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39747889, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c14788c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf47892, SUC_lexicon_tags_66 },
    { 0x20347892, SUC_lexicon_tags_37 },
    { 0xd147892, SUC_lexicon_tags_797 },
    { 0xff947895, SUC_lexicon_tags_16 },
    { 0xdab47895, SUC_lexicon_tags_37 },
    { 0x31147896, SUC_lexicon_tags_39 },
    { 0x4c547896, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f54789a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc794789c, SUC_lexicon_tags_37 },
    { 0x5654789d, SUC_lexicon_tags_18 },
    { 0xb5f4789e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7478a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaad478a7, SUC_lexicon_tags_49 },
    { 0x9a3478a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad478ad, SUC_lexicon_tags_15 },
    { 0xf61478ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb478b0, SUC_lexicon_tags_37 },
    { 0xf6d478b0, SUC_lexicon_tags_15 },
    { 0x671478b2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f478b7, SUC_lexicon_tags_14 },
    { 0x50f478b8, SUC_lexicon_tags_55 },
    { 0x64f478b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b478bc, SUC_lexicon_tags_7 },
    { 0xaad478bc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9478c4, SUC_lexicon_tags_60 },
    { 0xc5d478c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb478ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x729478cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe77478ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x269478d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805478d5, SUC_lexicon_tags_92 },
    { 0x601478d6, SUC_lexicon_tags_15 },
    { 0xe2f478d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75478da, SUC_lexicon_tags_28 },
    { 0x70b478db, SUC_lexicon_tags_46 },
    { 0x9ab478db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3ab478de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873478e2, SUC_lexicon_tags_31 },
    { 0x9e9478e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b478e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x235478eb, SUC_lexicon_tags_37 },
    { 0xed9478ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183478f6, SUC_lexicon_tags_17 },
    { 0xd5f478f7, SUC_lexicon_tags_77 },
    { 0xfd5478f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b47901, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65d47903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e747909, SUC_lexicon_tags_17 },
    { 0xa3d4790a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x654790d, SUC_lexicon_tags_37 },
    { 0x9274790d, SUC_lexicon_tags_24 },
    { 0xdfd4790f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8fd47911, SUC_lexicon_tags_19 },
    { 0xa7147912, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe147914, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe4347916, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x15747918, SUC_lexicon_tags_37 },
    { 0xb6747919, SUC_lexicon_tags_7 },
    { 0xdf947919, SUC_lexicon_tags_39 },
    { 0xbcf4791b, SUC_lexicon_tags_42 },
    { 0xd814791c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcfb4791e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xca147920, SUC_lexicon_tags_45 },
    { 0x80b47921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8547924, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48f47926, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29547928, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2634792c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1214792f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47747931, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d747933, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6d47935, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5947938, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaf4793a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc154793f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d747943, SUC_lexicon_tags_13 },
    { 0x55747944, SUC_lexicon_tags_32 },
    { 0x74947943, SUC_lexicon_tags_6 },
    { 0x27d47944, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb4794c, SUC_lexicon_tags_12 },
    { 0x3794794d, SUC_lexicon_tags_37 },
    { 0x3f74794c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b47951, SUC_lexicon_tags_37 },
    { 0x4e547952, SUC_lexicon_tags_37 },
    { 0xeb347951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7347957, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaff47959, SUC_lexicon_tags_12 },
    { 0xe2d4795a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5114795c, SUC_lexicon_tags_60 },
    { 0x67f4795d, SUC_lexicon_tags_15 },
    { 0xc854795e, SUC_lexicon_tags_12 },
    { 0xfef4795e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b47962, SUC_lexicon_tags_38 },
    { 0xe7d47963, SUC_lexicon_tags_65 },
    { 0xab547964, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d47967, SUC_lexicon_tags_19 },
    { 0xea347968, SUC_lexicon_tags_1 },
    { 0x54947969, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d4796c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb47971, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x99947973, SUC_lexicon_tags_1 },
    { 0xb4347974, SUC_lexicon_tags_25 },
    { 0x36f47975, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd1f47977, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f54797b, SUC_lexicon_tags_25 },
    { 0x1394797b, SUC_lexicon_tags_15 },
    { 0xf8b4797d, SUC_lexicon_tags_63 },
    { 0x1294797d, SUC_lexicon_tags_12 },
    { 0x1a74797b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf47984, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d47987, SUC_lexicon_tags_6 },
    { 0x7f147987, SUC_lexicon_tags_31 },
    { 0xe3747989, SUC_lexicon_tags_194 },
    { 0x8414798a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd4f4798c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe14798e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef947990, SUC_lexicon_tags_2 },
    { 0x8e747990, SUC_lexicon_tags_37 },
    { 0x6af47991, SUC_lexicon_tags_7 },
    { 0x60347993, SUC_lexicon_tags_12 },
    { 0x1a547994, SUC_lexicon_tags_47 },
    { 0x78347993, SUC_lexicon_tags_17 },
    { 0xc8d47993, SUC_lexicon_tags_15 },
    { 0xe4747993, SUC_lexicon_tags_0 },
    { 0x1f47998, SUC_lexicon_tags_2 },
    { 0x92147996, SUC_lexicon_tags_37 },
    { 0xf7f4799a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6074799d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xacd4799f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x537479a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d479a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x179479a8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x2a7479aa, SUC_lexicon_tags_15 },
    { 0x715479ab, SUC_lexicon_tags_8 },
    { 0xe7479ab, SUC_lexicon_tags_23 },
    { 0x90d479ad, SUC_lexicon_tags_25 },
    { 0xd99479ae, SUC_lexicon_tags_60 },
    { 0x383479af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xded479b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b479b5, SUC_lexicon_tags_17 },
    { 0x7a3479b6, SUC_lexicon_tags_12 },
    { 0x27b479b5, SUC_lexicon_tags_17 },
    { 0xfe5479b5, SUC_lexicon_tags_37 },
    { 0x757479b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc17479bb, SUC_lexicon_tags_48 },
    { 0x5df479bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d479bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbc7479c1, SUC_lexicon_tags_63 },
    { 0x9d1479c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b479c5, SUC_lexicon_tags_1 },
    { 0x99479c6, SUC_lexicon_tags_29 },
    { 0xd45479c7, SUC_lexicon_tags_25 },
    { 0x89479c8, SUC_lexicon_tags_31 },
    { 0x79b479c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b3479cb, SUC_lexicon_tags_43 },
    { 0x961479cc, SUC_lexicon_tags_47 },
    { 0x4bb479cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0d479cf, SUC_lexicon_tags_28 },
    { 0xe39479d0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9479d5, SUC_lexicon_tags_17 },
    { 0x59f479d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5479da, SUC_lexicon_tags_27 },
    { 0x7cf479db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9cb479dd, SUC_lexicon_tags_1 },
    { 0x9479dd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761479e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b479f1, SUC_lexicon_tags_1 },
    { 0x197479f2, SUC_lexicon_tags_27 },
    { 0x5e7479f3, SUC_lexicon_tags_160 },
    { 0x365479f4, SUC_lexicon_tags_53 },
    { 0x599479f1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb479fa, SUC_lexicon_tags_28 },
    { 0xd31479fb, SUC_lexicon_tags_65 },
    { 0x24f479fc, SUC_lexicon_tags_30 },
    { 0xe51479fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f47a04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3747a0b, SUC_lexicon_tags_79 },
    { 0x74147a0c, SUC_lexicon_tags_7 },
    { 0xa6947a0d, SUC_lexicon_tags_47 },
    { 0x32b47a0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5947a16, SUC_lexicon_tags_37 },
    { 0xf8547a16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d47a1b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db47a1f, SUC_lexicon_tags_25 },
    { 0x15147a1f, SUC_lexicon_tags_8 },
    { 0xf0347a20, SUC_lexicon_tags_6 },
    { 0x12147a1f, SUC_lexicon_tags_13 },
    { 0xe6f47a20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b47a27, SUC_lexicon_tags_33 },
    { 0xfdd47a28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51747a2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d47a2d, SUC_lexicon_tags_17 },
    { 0xf6b47a2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0147a31, SUC_lexicon_tags_2 },
    { 0x53f47a32, SUC_lexicon_tags_37 },
    { 0xecd47a33, SUC_lexicon_tags_12 },
    { 0xc5547a34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x21f47a36, SUC_lexicon_tags_5 },
    { 0xda947a36, SUC_lexicon_tags_25 },
    { 0xa947a37, SUC_lexicon_tags_25 },
    { 0x6d347a38, SUC_lexicon_tags_49 },
    { 0xae147a3a, SUC_lexicon_tags_30 },
    { 0xa1f47a3b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3147a3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d47a43, SUC_lexicon_tags_21 },
    { 0xcd47a43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b47a4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44947a4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6b947a51, SUC_lexicon_tags_39 },
    { 0x2b347a52, SUC_lexicon_tags_0 },
    { 0xaf147a53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x35947a56, SUC_lexicon_tags_31 },
    { 0x86147a57, SUC_lexicon_tags_12 },
    { 0x3547a58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b47a61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5947a68, SUC_lexicon_tags_25 },
    { 0x3d147a68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc947a6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1947a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2747a76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x78547a79, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x75147a7c, SUC_lexicon_tags_9 },
    { 0x1e947a7d, SUC_lexicon_tags_2 },
    { 0xd7147a7c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb547a82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b47a87, SUC_lexicon_tags_4 },
    { 0x81947a87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x51b47a8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b47a8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc747a90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79547a94, SUC_lexicon_tags_42 },
    { 0x11947a95, SUC_lexicon_tags_15 },
    { 0x67947a95, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x26547a98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d47a9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0f47a9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdeb47a9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f47aa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3147aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86347aad, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab47ab1, SUC_lexicon_tags_107 },
    { 0x20947ab1, SUC_lexicon_tags_53 },
    { 0xcc747ab3, SUC_lexicon_tags_37 },
    { 0x9a147ab3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4347abd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f47ac6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af47ac9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4a547acb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca147acd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa347ad0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf47ad3, SUC_lexicon_tags_15 },
    { 0x5cd47ad4, SUC_lexicon_tags_25 },
    { 0x13d47ad5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65547ad9, SUC_lexicon_tags_9 },
    { 0x96747ada, SUC_lexicon_tags_2 },
    { 0xe5d47ad9, SUC_lexicon_tags_28 },
    { 0x41547ada, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x15547ade, SUC_lexicon_tags_8 },
    { 0xf7747adf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7547ae2, SUC_lexicon_tags_1 },
    { 0xefd47ae3, SUC_lexicon_tags_48 },
    { 0x79347ae4, SUC_lexicon_tags_45 },
    { 0x8ff47ae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec347ae7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb147aee, SUC_lexicon_tags_30 },
    { 0xdd547aef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd1b47af1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5347af4, SUC_lexicon_tags_11 },
    { 0xa2147af4, SUC_lexicon_tags_15 },
    { 0xec347af4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab47af9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66947afe, SUC_lexicon_tags_9 },
    { 0x5d947aff, SUC_lexicon_tags_17 },
    { 0x68f47afe, SUC_lexicon_tags_43 },
    { 0xc1f47afe, SUC_lexicon_tags_12 },
    { 0xaf347b00, SUC_lexicon_tags_28 },
    { 0x23747b03, SUC_lexicon_tags_8 },
    { 0xb5147b03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77147b08, SUC_lexicon_tags_37 },
    { 0x46347b09, SUC_lexicon_tags_8 },
    { 0xba947b08, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5e947b0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a147b13, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d47b19, SUC_lexicon_tags_12 },
    { 0x61d47b1a, SUC_lexicon_tags_33 },
    { 0xf2f47b1b, SUC_lexicon_tags_49 },
    { 0xc8547b1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab47b20, SUC_lexicon_tags_37 },
    { 0x72547b21, SUC_lexicon_tags_66 },
    { 0x34347b21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb4d47b24, SUC_lexicon_tags_33 },
    { 0x2947b25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc347b28, SUC_lexicon_tags_8 },
    { 0x58f47b29, SUC_lexicon_tags_25 },
    { 0x22947b29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41547b30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d47b41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x94147b44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30347b48, SUC_lexicon_tags_30 },
    { 0x86b47b49, SUC_lexicon_tags_0 },
    { 0x19147b4a, SUC_lexicon_tags_0 },
    { 0x99747b4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25f47b4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11147b50, SUC_lexicon_tags_25 },
    { 0xf2147b51, SUC_lexicon_tags_57 },
    { 0x71747b50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f47b56, SUC_lexicon_tags_21 },
    { 0x3eb47b57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef47b5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1147b5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2547b60, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6147b62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc8147b64, SUC_lexicon_tags_13 },
    { 0x39947b65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b547b6b, SUC_lexicon_tags_29 },
    { 0x44f47b6c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8947b75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b47b7d, SUC_lexicon_tags_9 },
    { 0xf2947b7e, SUC_lexicon_tags_10 },
    { 0xbbf47b7e, SUC_lexicon_tags_28 },
    { 0x13d47b80, SUC_lexicon_tags_12 },
    { 0x6f547b81, SUC_lexicon_tags_60 },
    { 0xb2547b80, SUC_lexicon_tags_15 },
    { 0xb4947b81, SUC_lexicon_tags_37 },
    { 0x9947b7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64f47b86, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f47b8b, SUC_lexicon_tags_8 },
    { 0xd0547b8c, SUC_lexicon_tags_45 },
    { 0x9e147b8d, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd47b93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd47b97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e547b99, SUC_lexicon_tags_12 },
    { 0xff47b9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10547b9c, SUC_lexicon_tags_2 },
    { 0x71147b9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb47ba0, SUC_lexicon_tags_2 },
    { 0xfa347ba1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa747ba7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a347bad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc347bb0, SUC_lexicon_tags_2 },
    { 0x98347bb0, SUC_lexicon_tags_33 },
    { 0x93947bb2, SUC_lexicon_tags_29 },
    { 0x38147bb2, SUC_lexicon_tags_43 },
    { 0x7747bb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22747bb7, SUC_lexicon_tags_28 },
    { 0x77147bb7, SUC_lexicon_tags_45 },
    { 0xa3d47bb8, SUC_lexicon_tags_25 },
    { 0x8fd47bba, SUC_lexicon_tags_49 },
    { 0x43f47bb8, SUC_lexicon_tags_38 },
    { 0xd5147bbc, SUC_lexicon_tags_2 },
    { 0xa9d47bbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f47bc0, SUC_lexicon_tags_15 },
    { 0x1eb47bc0, SUC_lexicon_tags_2 },
    { 0x9fb47bc1, SUC_lexicon_tags_12 },
    { 0x9d547bc3, SUC_lexicon_tags_29 },
    { 0xe7347bc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb47bca, SUC_lexicon_tags_37 },
    { 0x18f47bcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6347bce, SUC_lexicon_tags_28 },
    { 0x1f47bcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb47bd4, SUC_lexicon_tags_14 },
    { 0x99547bd5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86147bdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c747be2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92147be7, SUC_lexicon_tags_27 },
    { 0xbb347be7, SUC_lexicon_tags_9 },
    { 0xbad47be9, SUC_lexicon_tags_37 },
    { 0xc6347be8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6347bef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x67747bf1, SUC_lexicon_tags_43 },
    { 0xb6f47bf2, SUC_lexicon_tags_33 },
    { 0xdf47bf3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x81147bf6, SUC_lexicon_tags_1 },
    { 0xd5347bf7, SUC_lexicon_tags_43 },
    { 0x40f47bf7, SUC_lexicon_tags_60 },
    { 0xac947bf7, SUC_lexicon_tags_14 },
    { 0xfd547bf7, SUC_lexicon_tags_11 },
    { 0x67d47bfb, SUC_lexicon_tags_7 },
    { 0x21147bfb, SUC_lexicon_tags_15 },
    { 0x4ef47bf7, SUC_lexicon_tags_19 },
    { 0xc747bf8, SUC_lexicon_tags_17 },
    { 0xaeb47bf6, SUC_lexicon_tags_11 },
    { 0x80f47c00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4147c04, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67747c0a, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b47c11, SUC_lexicon_tags_37 },
    { 0x4dd47c12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e547c15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x96d47c17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96d47c19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f747c1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f47c21, SUC_lexicon_tags_28 },
    { 0xfc947c22, SUC_lexicon_tags_7 },
    { 0xfd47c23, SUC_lexicon_tags_28 },
    { 0xc3747c24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3947c29, SUC_lexicon_tags_9 },
    { 0xdf747c29, SUC_lexicon_tags_14 },
    { 0xb8347c2b, SUC_lexicon_tags_63 },
    { 0xa5147c2a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b347c2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58547c33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61b47c35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35747c37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x90b47c39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4347c3b, SUC_lexicon_tags_37 },
    { 0xc1547c3c, SUC_lexicon_tags_11 },
    { 0xa8347c3d, SUC_lexicon_tags_7 },
    { 0x73d47c3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8747c44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb147c46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6147c48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17947c4b, SUC_lexicon_tags_18 },
    { 0x20547c4c, SUC_lexicon_tags_79 },
    { 0x7a747c4d, SUC_lexicon_tags_3 },
    { 0x31147c4e, SUC_lexicon_tags_30 },
    { 0x74547c4d, SUC_lexicon_tags_3 },
    { 0x29147c50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x50947c53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94947c57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4747c59, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf47c60, SUC_lexicon_tags_151 },
    { 0x8ed47c60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe8d47c63, SUC_lexicon_tags_12 },
    { 0x4a547c64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcf147c66, SUC_lexicon_tags_2 },
    { 0xef147c67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75747c6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff47c75, SUC_lexicon_tags_14 },
    { 0xae947c75, SUC_lexicon_tags_7 },
    { 0x35747c77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8347c7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x41547c7f, SUC_lexicon_tags_49 },
    { 0x7bb47c80, SUC_lexicon_tags_77 },
    { 0xd2747c81, SUC_lexicon_tags_19 },
    { 0xaa747c82, SUC_lexicon_tags_151 },
    { 0x6a947c80, SUC_lexicon_tags_8 },
    { 0x24b47c82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b547c86, SUC_lexicon_tags_7 },
    { 0xddd47c87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a147c89, SUC_lexicon_tags_37 },
    { 0x78f47c89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f47c8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65347c94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8147c96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87347c99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab47c9f, SUC_lexicon_tags_2 },
    { 0x7e347c9f, SUC_lexicon_tags_45 },
    { 0xf6b47ca1, SUC_lexicon_tags_6 },
    { 0xaf147c9f, SUC_lexicon_tags_12 },
    { 0x8e147ca3, SUC_lexicon_tags_49 },
    { 0x6b547ca3, SUC_lexicon_tags_6 },
    { 0x49b47ca0, SUC_lexicon_tags_2 },
    { 0xa8b47ca5, SUC_lexicon_tags_1 },
    { 0x60d47ca7, SUC_lexicon_tags_47 },
    { 0x73b47ca7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ef47caa, SUC_lexicon_tags_37 },
    { 0xfb47caa, SUC_lexicon_tags_15 },
    { 0x76347cab, SUC_lexicon_tags_6 },
    { 0xe0747cad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed47cb0, SUC_lexicon_tags_15 },
    { 0xb2547cb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb947cb3, SUC_lexicon_tags_40 },
    { 0x46d47cb4, SUC_lexicon_tags_46 },
    { 0x16147cb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb47cb8, SUC_lexicon_tags_45 },
    { 0x82f47cb9, SUC_lexicon_tags_37 },
    { 0x5a147cb8, SUC_lexicon_tags_27 },
    { 0xa547cbb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58947cc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9947cc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d47ccc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd47cd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd47cd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed947cd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d47cdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x13147cde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe2547ce0, SUC_lexicon_tags_17 },
    { 0xec947ce0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb47ce8, SUC_lexicon_tags_37 },
    { 0x10347ce8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b47ced, SUC_lexicon_tags_27 },
    { 0xd6b47cee, SUC_lexicon_tags_1 },
    { 0x6ef47cee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cd47cf1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91b47cf3, SUC_lexicon_tags_14 },
    { 0xfd147cf4, SUC_lexicon_tags_9 },
    { 0x93547cf5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd47cf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa147cff, SUC_lexicon_tags_19 },
    { 0x7cf47d00, SUC_lexicon_tags_47 },
    { 0x7ed47cff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x93547d03, SUC_lexicon_tags_11 },
    { 0xe6547d04, SUC_lexicon_tags_82 },
    { 0xcc547d04, SUC_lexicon_tags_33 },
    { 0x4cb47d04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fb47d08, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff347d0c, SUC_lexicon_tags_25 },
    { 0x27d47d0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d47d10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d47d14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b147d16, SUC_lexicon_tags_14 },
    { 0xb947d16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d47d1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c747d1f, SUC_lexicon_tags_21 },
    { 0x26147d20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8547d27, SUC_lexicon_tags_27 },
    { 0xbfb47d27, SUC_lexicon_tags_48 },
    { 0x18547d29, SUC_lexicon_tags_7 },
    { 0xad747d29, SUC_lexicon_tags_142 },
    { 0xf2d47d28, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f47d2e, SUC_lexicon_tags_25 },
    { 0x4e947d2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6347d31, SUC_lexicon_tags_15 },
    { 0x49547d31, SUC_lexicon_tags_45 },
    { 0xc6747d33, SUC_lexicon_tags_23 },
    { 0x5cb47d34, SUC_lexicon_tags_25 },
    { 0xe4b47d35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17547d38, SUC_lexicon_tags_17 },
    { 0x49347d39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5947d3e, SUC_lexicon_tags_23 },
    { 0x15d47d3f, SUC_lexicon_tags_15 },
    { 0x82747d40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d47d45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43b47d47, SUC_lexicon_tags_37 },
    { 0x1c547d47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d47d4e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4f947d50, SUC_lexicon_tags_47 },
    { 0xa4747d51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f47d54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9347d5c, SUC_lexicon_tags_26 },
    { 0xd147d5d, SUC_lexicon_tags_38 },
    { 0x14747d5d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf47d64, SUC_lexicon_tags_46 },
    { 0x52d47d64, SUC_lexicon_tags_19 },
    { 0xc1d47d66, SUC_lexicon_tags_0 },
    { 0x4d47d67, SUC_lexicon_tags_27 },
    { 0xf1147d67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbd47d6a, SUC_lexicon_tags_2 },
    { 0x1147d6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26147d78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70347d7d, SUC_lexicon_tags_7 },
    { 0x96747d7e, SUC_lexicon_tags_15 },
    { 0x7f947d7f, SUC_lexicon_tags_2 },
    { 0x4b47d7e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b47d83, SUC_lexicon_tags_26 },
    { 0xdb947d84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c147d86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed47d8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdd747d8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d47d91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b47d95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x19d47d97, SUC_lexicon_tags_1 },
    { 0x7df47d97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc6f47d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81d47d9c, SUC_lexicon_tags_1 },
    { 0x5e147d9d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2747d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d47da3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c147da6, SUC_lexicon_tags_37 },
    { 0x2e547da6, SUC_lexicon_tags_124 },
    { 0x93147da8, SUC_lexicon_tags_29 },
    { 0xcf547da9, SUC_lexicon_tags_19 },
    { 0xac147da9, SUC_lexicon_tags_37 },
    { 0xbed47dab, SUC_lexicon_tags_60 },
    { 0xb747dab, SUC_lexicon_tags_12 },
    { 0x71547da8, SUC_lexicon_tags_9 },
    { 0xa3b47dae, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x89747db1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61747db3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb47db8, SUC_lexicon_tags_45 },
    { 0x16547db9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb47dbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x51547dc1, SUC_lexicon_tags_37 },
    { 0x66947dc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfff47dc4, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d747dc8, SUC_lexicon_tags_37 },
    { 0xc7547dc8, SUC_lexicon_tags_25 },
    { 0xd3f47dca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f947dd0, SUC_lexicon_tags_38 },
    { 0xfdd47dd1, SUC_lexicon_tags_65 },
    { 0x1f747dd0, SUC_lexicon_tags_8 },
    { 0x1d747dd3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b547dda, SUC_lexicon_tags_27 },
    { 0xb0547ddb, SUC_lexicon_tags_2 },
    { 0xb4747dda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55147de3, SUC_lexicon_tags_0 },
    { 0x83d47de4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d147de6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7747dea, SUC_lexicon_tags_17 },
    { 0xfc347deb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b47df6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf2947df8, SUC_lexicon_tags_28 },
    { 0x27147df9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6c347dfb, SUC_lexicon_tags_45 },
    { 0xbc147dfb, SUC_lexicon_tags_11 },
    { 0x12347dfc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x62947dff, SUC_lexicon_tags_10 },
    { 0x43747dff, SUC_lexicon_tags_9 },
    { 0x2dd47e00, SUC_lexicon_tags_7 },
    { 0x28b47e02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b47e0a, SUC_lexicon_tags_7 },
    { 0x4fd47e0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47347e0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x46547e10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd747e13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e347e16, SUC_lexicon_tags_27 },
    { 0xf7d47e16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f947e1d, SUC_lexicon_tags_66 },
    { 0xc5147e1d, SUC_lexicon_tags_2 },
    { 0xa5d47e1f, SUC_lexicon_tags_121 },
    { 0x76347e1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b547e23, SUC_lexicon_tags_66 },
    { 0x1747e24, SUC_lexicon_tags_17 },
    { 0x77947e25, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3747e27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f47e2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1d47e2c, SUC_lexicon_tags_1 },
    { 0x6b547e2d, SUC_lexicon_tags_17 },
    { 0x4147e2e, SUC_lexicon_tags_17 },
    { 0xa4547e2e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe0d47e31, SUC_lexicon_tags_16 },
    { 0xaf347e32, SUC_lexicon_tags_28 },
    { 0x2347e32, SUC_lexicon_tags_48 },
    { 0x23547e32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f47e37, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd747e3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7a947e3c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d47e40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x62147e42, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd747e48, SUC_lexicon_tags_17 },
    { 0x53747e48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e147e52, SUC_lexicon_tags_23 },
    { 0xe2747e53, SUC_lexicon_tags_38 },
    { 0x20f47e54, SUC_lexicon_tags_38 },
    { 0xe7347e55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc347e57, SUC_lexicon_tags_17 },
    { 0xbb47e58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f947e5b, SUC_lexicon_tags_68 },
    { 0xd6747e5c, SUC_lexicon_tags_11 },
    { 0x45547e5d, SUC_lexicon_tags_4 },
    { 0xf8747e5d, SUC_lexicon_tags_37 },
    { 0x1f147e5c, SUC_lexicon_tags_11 },
    { 0xb7b47e5b, SUC_lexicon_tags_6 },
    { 0x40347e5d, SUC_lexicon_tags_56 },
    { 0x84547e5d, SUC_lexicon_tags_15 },
    { 0x6e747e5b, SUC_lexicon_tags_26 },
    { 0x15547e64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5347e68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5547e6a, SUC_lexicon_tags_11 },
    { 0xf0347e6b, SUC_lexicon_tags_14 },
    { 0xe2547e6c, SUC_lexicon_tags_49 },
    { 0xf8d47e6b, SUC_lexicon_tags_12 },
    { 0x8e547e6c, SUC_lexicon_tags_65 },
    { 0xbb347e6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68b47e71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb47e75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f47e77, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d47e7c, SUC_lexicon_tags_22 },
    { 0xb147e7c, SUC_lexicon_tags_1 },
    { 0x34f47e7e, SUC_lexicon_tags_17 },
    { 0x53d47e7c, SUC_lexicon_tags_27 },
    { 0xd8f47e80, SUC_lexicon_tags_49 },
    { 0xec347e81, SUC_lexicon_tags_15 },
    { 0xdeb47e82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb747e85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2547e8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b47e8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38947e96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc547e9a, SUC_lexicon_tags_0 },
    { 0x5e547e9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f47e9f, SUC_lexicon_tags_7 },
    { 0x35147ea0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x47947ea2, SUC_lexicon_tags_15 },
    { 0xd3747ea3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74f47ea5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa347eaa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14147eb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c347eb6, SUC_lexicon_tags_85 },
    { 0x44347eb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x36747eba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaef47ebc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa147ec0, SUC_lexicon_tags_6 },
    { 0xc2d47ec0, SUC_lexicon_tags_1 },
    { 0xd1b47ec0, SUC_lexicon_tags_10 },
    { 0xcd747ec2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d47ec6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9c747ec8, SUC_lexicon_tags_1 },
    { 0x5347ec8, SUC_lexicon_tags_2 },
    { 0xf1b47eca, SUC_lexicon_tags_2 },
    { 0x7f147ecb, SUC_lexicon_tags_25 },
    { 0x48d47ecc, SUC_lexicon_tags_23 },
    { 0xac147ecb, SUC_lexicon_tags_12 },
    { 0x4747ece, SUC_lexicon_tags_12 },
    { 0xf1347ecf, SUC_lexicon_tags_27 },
    { 0x49147ece, SUC_lexicon_tags_23 },
    { 0xd3747eca, SUC_lexicon_tags_37 },
    { 0x1e147ed2, SUC_lexicon_tags_11 },
    { 0x40d47ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47147edb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab547edf, SUC_lexicon_tags_2 },
    { 0xd2347ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9d47ee2, SUC_lexicon_tags_5 },
    { 0xa7f47ee3, SUC_lexicon_tags_11 },
    { 0x76947ee4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x37947ee7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88347ef4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc147ef7, SUC_lexicon_tags_25 },
    { 0xa9147ef7, SUC_lexicon_tags_38 },
    { 0x10747ef9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x57347efc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94747f00, SUC_lexicon_tags_11 },
    { 0x7b747f01, SUC_lexicon_tags_68 },
    { 0xd0747f02, SUC_lexicon_tags_66 },
    { 0xb8947f03, SUC_lexicon_tags_2 },
    { 0x91f47f04, SUC_lexicon_tags_11 },
    { 0x97747f03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd147f07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb47f0d, SUC_lexicon_tags_11 },
    { 0x5a347f0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x26147f11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80f47f13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d747f16, SUC_lexicon_tags_28 },
    { 0xba947f16, SUC_lexicon_tags_7 },
    { 0x84547f18, SUC_lexicon_tags_39 },
    { 0x83347f19, SUC_lexicon_tags_14 },
    { 0xf8347f19, SUC_lexicon_tags_1 },
    { 0x55947f16, SUC_lexicon_tags_17 },
    { 0xb1947f1c, SUC_lexicon_tags_33 },
    { 0xfdb47f16, SUC_lexicon_tags_10 },
    { 0x13947f1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8547f22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab547f24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69547f29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x83347f2b, SUC_lexicon_tags_17 },
    { 0x84747f2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50f47f2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65b47f30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7547f33, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xec347f36, SUC_lexicon_tags_14 },
    { 0x6d47f37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b47f3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3347f41, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73547f49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43947f4c, SUC_lexicon_tags_37 },
    { 0x24747f4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f547f50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed47f64, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d47f6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb947f71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6547f76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x44547f79, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4d147f7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf47f7d, SUC_lexicon_tags_25 },
    { 0x3e347f7e, SUC_lexicon_tags_26 },
    { 0x78547f7f, SUC_lexicon_tags_0 },
    { 0x5ad47f80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d47f86, SUC_lexicon_tags_45 },
    { 0x70147f87, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c747f8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb47f90, SUC_lexicon_tags_8 },
    { 0x26547f90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb47f96, SUC_lexicon_tags_60 },
    { 0x78f47f97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2547f99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f47f9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8147fa0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x19147fa2, SUC_lexicon_tags_46 },
    { 0xfeb47fa3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f147fa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2147faa, SUC_lexicon_tags_15 },
    { 0xf8947fab, SUC_lexicon_tags_12 },
    { 0xcbb47fab, SUC_lexicon_tags_25 },
    { 0xd9d47fac, SUC_lexicon_tags_22 },
    { 0xa7f47fae, SUC_lexicon_tags_12 },
    { 0xe6147faf, SUC_lexicon_tags_1 },
    { 0x28147fb0, SUC_lexicon_tags_6 },
    { 0xe7547fae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd47fbd, SUC_lexicon_tags_262 },
    { 0x2bf47fbe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91547fc4, SUC_lexicon_tags_8 },
    { 0x8f147fc5, SUC_lexicon_tags_2 },
    { 0xeb747fc5, SUC_lexicon_tags_45 },
    { 0x5ad47fc7, SUC_lexicon_tags_9 },
    { 0x94d47fc8, SUC_lexicon_tags_4 },
    { 0x547fc9, SUC_lexicon_tags_17 },
    { 0xe6347fc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf47fcd, SUC_lexicon_tags_1 },
    { 0x99d47fce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf8747fd0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf47fd4, SUC_lexicon_tags_16 },
    { 0x7cf47fd4, SUC_lexicon_tags_10 },
    { 0x83347fd6, SUC_lexicon_tags_25 },
    { 0x41f47fd4, SUC_lexicon_tags_10 },
    { 0x7bb47fd5, SUC_lexicon_tags_11 },
    { 0xa5747fd9, SUC_lexicon_tags_25 },
    { 0xc0347fda, SUC_lexicon_tags_17 },
    { 0xb3747fda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2cb47fdd, SUC_lexicon_tags_11 },
    { 0xb47fde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28547fe3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41347fea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df47fed, SUC_lexicon_tags_15 },
    { 0x2147fee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62347ff1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3ab47ff3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e147ff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8547ff9, SUC_lexicon_tags_55 },
    { 0x23f47ffa, SUC_lexicon_tags_108 },
    { 0xf5347ff9, SUC_lexicon_tags_29 },
    { 0x75747ffc, SUC_lexicon_tags_49 },
    { 0x59947ffd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61748002, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x76b48004, SUC_lexicon_tags_1 },
    { 0x71948005, SUC_lexicon_tags_65 },
    { 0x72f48006, SUC_lexicon_tags_23 },
    { 0x5ef48006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd314800c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b34800f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8348011, SUC_lexicon_tags_70 },
    { 0xb7148011, SUC_lexicon_tags_7 },
    { 0x8f548013, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba748017, SUC_lexicon_tags_9 },
    { 0x62948018, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6634801f, SUC_lexicon_tags_17 },
    { 0x1ad4801f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3fb48022, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9d348024, SUC_lexicon_tags_25 },
    { 0x70348024, SUC_lexicon_tags_26 },
    { 0x9c148026, SUC_lexicon_tags_0 },
    { 0x15548027, SUC_lexicon_tags_23 },
    { 0x1f948028, SUC_lexicon_tags_1 },
    { 0x31d48029, SUC_lexicon_tags_26 },
    { 0xb0148029, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9348035, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2934803a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbbf4803c, SUC_lexicon_tags_62 },
    { 0xa014803c, SUC_lexicon_tags_26 },
    { 0x53b4803c, SUC_lexicon_tags_31 },
    { 0xbf94803f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55948041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb348044, SUC_lexicon_tags_55 },
    { 0xc2148044, SUC_lexicon_tags_117 },
    { 0x71748046, SUC_lexicon_tags_47 },
    { 0x81148047, SUC_lexicon_tags_11 },
    { 0x1ed48048, SUC_lexicon_tags_33 },
    { 0x91948045, SUC_lexicon_tags_6 },
    { 0xad548048, SUC_lexicon_tags_17 },
    { 0xfcf4804b, SUC_lexicon_tags_8 },
    { 0x3e34804b, SUC_lexicon_tags_10 },
    { 0x4994804c, SUC_lexicon_tags_0 },
    { 0x9d4804e, SUC_lexicon_tags_25 },
    { 0x2e94804b, SUC_lexicon_tags_37 },
    { 0x148050, SUC_lexicon_tags_23 },
    { 0xfff48048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47948057, SUC_lexicon_tags_25 },
    { 0x14548058, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa7d4805a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6548061, SUC_lexicon_tags_36 },
    { 0x48f48062, SUC_lexicon_tags_12 },
    { 0xfb548063, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63748067, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d4806b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa948070, SUC_lexicon_tags_42 },
    { 0x2f348070, SUC_lexicon_tags_37 },
    { 0x3e748072, SUC_lexicon_tags_7 },
    { 0x14948072, SUC_lexicon_tags_63 },
    { 0xab548074, SUC_lexicon_tags_40 },
    { 0x27948072, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a948078, SUC_lexicon_tags_30 },
    { 0x83f48079, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8a54807b, SUC_lexicon_tags_11 },
    { 0x9c74807b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2214807f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36548085, SUC_lexicon_tags_25 },
    { 0xbc548086, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f4808a, SUC_lexicon_tags_1 },
    { 0xba94808a, SUC_lexicon_tags_7 },
    { 0x314808c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a348091, SUC_lexicon_tags_25 },
    { 0x1a948091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f48096, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d4809f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb71480a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd7d480a3, SUC_lexicon_tags_7 },
    { 0xaeb480a4, SUC_lexicon_tags_0 },
    { 0xf97480a4, SUC_lexicon_tags_23 },
    { 0x4bd480a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9480aa, SUC_lexicon_tags_38 },
    { 0x325480ab, SUC_lexicon_tags_47 },
    { 0x6cf480aa, SUC_lexicon_tags_0 },
    { 0x98f480ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441480b3, SUC_lexicon_tags_82 },
    { 0xbc9480b3, SUC_lexicon_tags_17 },
    { 0xcb3480b3, SUC_lexicon_tags_15 },
    { 0xc0d480b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f480b9, SUC_lexicon_tags_11 },
    { 0x957480b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db480be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db480c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd480c9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf09480cb, SUC_lexicon_tags_160 },
    { 0x6f7480cc, SUC_lexicon_tags_113 },
    { 0x6f9480cc, SUC_lexicon_tags_8 },
    { 0xec1480ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83d480d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x153480d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac7480d4, SUC_lexicon_tags_7 },
    { 0x593480d4, SUC_lexicon_tags_23 },
    { 0x467480d6, SUC_lexicon_tags_42 },
    { 0xd39480d5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f480db, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9480e1, SUC_lexicon_tags_25 },
    { 0x8eb480e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b480ea, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe99480ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9480ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x133480f2, SUC_lexicon_tags_7 },
    { 0x6ef480f2, SUC_lexicon_tags_6 },
    { 0xa7480f4, SUC_lexicon_tags_7 },
    { 0x7af480f5, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1480fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69480ff, SUC_lexicon_tags_12 },
    { 0x7cb48100, SUC_lexicon_tags_160 },
    { 0xd5548100, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20d48103, SUC_lexicon_tags_9 },
    { 0x1b548104, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ad48106, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b48110, SUC_lexicon_tags_26 },
    { 0xfc348111, SUC_lexicon_tags_47 },
    { 0x75348112, SUC_lexicon_tags_17 },
    { 0xaab48111, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5748115, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d48119, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb54811b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4948120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d48123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb4812a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4948130, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6754813a, SUC_lexicon_tags_12 },
    { 0x9454813b, SUC_lexicon_tags_17 },
    { 0x7c74813b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6148140, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14348149, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf714814b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b4814f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7948153, SUC_lexicon_tags_2 },
    { 0x34f48154, SUC_lexicon_tags_37 },
    { 0xe9748155, SUC_lexicon_tags_35 },
    { 0x83b48156, SUC_lexicon_tags_33 },
    { 0x44748157, SUC_lexicon_tags_8 },
    { 0xca948155, SUC_lexicon_tags_12 },
    { 0x49f48156, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe094815b, SUC_lexicon_tags_47 },
    { 0x3dd4815c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b48161, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed548163, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b48168, SUC_lexicon_tags_57 },
    { 0x65348168, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7874816f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d48172, SUC_lexicon_tags_31 },
    { 0xc9748172, SUC_lexicon_tags_12 },
    { 0xb3348174, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f948176, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb274817d, SUC_lexicon_tags_17 },
    { 0x6e34817e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b48185, SUC_lexicon_tags_2 },
    { 0xa6748186, SUC_lexicon_tags_60 },
    { 0x1ab48186, SUC_lexicon_tags_47 },
    { 0x43948186, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb4818a, SUC_lexicon_tags_6 },
    { 0x9234818a, SUC_lexicon_tags_116 },
    { 0xeeb4818c, SUC_lexicon_tags_39 },
    { 0x3434818b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bf4818f, SUC_lexicon_tags_11 },
    { 0x99948190, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f48198, SUC_lexicon_tags_49 },
    { 0x5af48199, SUC_lexicon_tags_4 },
    { 0x48b4819a, SUC_lexicon_tags_38 },
    { 0x9414819b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d481a7, SUC_lexicon_tags_12 },
    { 0x7f481a8, SUC_lexicon_tags_35 },
    { 0x621481a7, SUC_lexicon_tags_1 },
    { 0xfdb481aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85b481ac, SUC_lexicon_tags_1 },
    { 0xa57481ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b481b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff481bc, SUC_lexicon_tags_29 },
    { 0x4b7481bc, SUC_lexicon_tags_1 },
    { 0xd33481bd, SUC_lexicon_tags_47 },
    { 0xb47481bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f481c2, SUC_lexicon_tags_2 },
    { 0x315481c3, SUC_lexicon_tags_37 },
    { 0x2f3481c4, SUC_lexicon_tags_47 },
    { 0x925481c4, SUC_lexicon_tags_7 },
    { 0xb8b481c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1481ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d481cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d481d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395481d7, SUC_lexicon_tags_15 },
    { 0x143481d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16d481da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x665481dd, SUC_lexicon_tags_25 },
    { 0xdeb481de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85481e1, SUC_lexicon_tags_46 },
    { 0xe99481e2, SUC_lexicon_tags_15 },
    { 0x9c3481e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23481e7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9481eb, SUC_lexicon_tags_9 },
    { 0x19f481ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x559481ee, SUC_lexicon_tags_45 },
    { 0x7e1481ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97481f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf481f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e7481f6, SUC_lexicon_tags_27 },
    { 0xbd7481f6, SUC_lexicon_tags_45 },
    { 0x3d9481f6, SUC_lexicon_tags_14 },
    { 0xaf7481f9, SUC_lexicon_tags_5 },
    { 0xc69481f9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69481fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82748200, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5948204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b48208, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f48212, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df4821c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2148221, SUC_lexicon_tags_1 },
    { 0xbd48222, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d48227, SUC_lexicon_tags_37 },
    { 0x9a148228, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd54822a, SUC_lexicon_tags_48 },
    { 0xab4822a, SUC_lexicon_tags_47 },
    { 0x3c54822c, SUC_lexicon_tags_8 },
    { 0xe834822d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb48234, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f48237, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d4823d, SUC_lexicon_tags_37 },
    { 0x5ad4823d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeb548240, SUC_lexicon_tags_10 },
    { 0xa3348241, SUC_lexicon_tags_4 },
    { 0xd7748240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaef48244, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e548246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb834824a, SUC_lexicon_tags_1574 },
    { 0xced4824b, SUC_lexicon_tags_43 },
    { 0x314824c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa734824e, SUC_lexicon_tags_39 },
    { 0x2314824f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1548254, SUC_lexicon_tags_68 },
    { 0x7bb48254, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe548257, SUC_lexicon_tags_31 },
    { 0x98148258, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6074825d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x81148260, SUC_lexicon_tags_15 },
    { 0xc3948260, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f148265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a14826b, SUC_lexicon_tags_25 },
    { 0xfff4826c, SUC_lexicon_tags_11 },
    { 0x9354826c, SUC_lexicon_tags_17 },
    { 0xacd4826e, SUC_lexicon_tags_9 },
    { 0x5834826f, SUC_lexicon_tags_17 },
    { 0x52d48270, SUC_lexicon_tags_16 },
    { 0x1994826f, SUC_lexicon_tags_47 },
    { 0xe7f48272, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94748276, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5af48278, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b4827c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x93b4827e, SUC_lexicon_tags_15 },
    { 0x6a14827e, SUC_lexicon_tags_908 },
    { 0x2ef48280, SUC_lexicon_tags_11 },
    { 0x25d48281, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b348286, SUC_lexicon_tags_42 },
    { 0x7df48287, SUC_lexicon_tags_33 },
    { 0x37548287, SUC_lexicon_tags_2 },
    { 0x67748286, SUC_lexicon_tags_60 },
    { 0xaaf4828a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14828e, SUC_lexicon_tags_45 },
    { 0x2cb4828f, SUC_lexicon_tags_18 },
    { 0x96148290, SUC_lexicon_tags_14 },
    { 0x4ab48291, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47548296, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd4829c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95f4829e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb482a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9482a6, SUC_lexicon_tags_8 },
    { 0x453482a6, SUC_lexicon_tags_26 },
    { 0x865482a8, SUC_lexicon_tags_179 },
    { 0x58d482a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab482ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731482b3, SUC_lexicon_tags_9 },
    { 0xc15482b4, SUC_lexicon_tags_25 },
    { 0x1ed482b5, SUC_lexicon_tags_39 },
    { 0xecd482b5, SUC_lexicon_tags_37 },
    { 0x4b5482b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd33482b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49482bd, SUC_lexicon_tags_2 },
    { 0x915482be, SUC_lexicon_tags_63 },
    { 0x35482be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x317482c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x50b482c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d5482c5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d482cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43b482cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455482d6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa31482d8, SUC_lexicon_tags_25 },
    { 0xbfb482d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b482de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7482e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609482e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77482e9, SUC_lexicon_tags_32 },
    { 0x98f482ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9482f2, SUC_lexicon_tags_27 },
    { 0x9d7482f2, SUC_lexicon_tags_287 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57482f7, SUC_lexicon_tags_25 },
    { 0xfb3482f7, SUC_lexicon_tags_43 },
    { 0xdbb482f9, SUC_lexicon_tags_7 },
    { 0xd03482fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6948302, SUC_lexicon_tags_6 },
    { 0xb3748302, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52948305, SUC_lexicon_tags_8 },
    { 0xb1b48305, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58b48308, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b4830c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c14830e, SUC_lexicon_tags_49 },
    { 0xd174830e, SUC_lexicon_tags_8 },
    { 0x85348310, SUC_lexicon_tags_12 },
    { 0xa5f48311, SUC_lexicon_tags_37 },
    { 0xd314830f, SUC_lexicon_tags_80 },
    { 0xe5948313, SUC_lexicon_tags_17 },
    { 0x1d348314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79748318, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf14831a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc174831c, SUC_lexicon_tags_49 },
    { 0xe8d4831d, SUC_lexicon_tags_11 },
    { 0x5f94831c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7f48320, SUC_lexicon_tags_19 },
    { 0x17b48320, SUC_lexicon_tags_40 },
    { 0x15948322, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3bb48324, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xde748326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a148329, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18148332, SUC_lexicon_tags_2 },
    { 0x64948333, SUC_lexicon_tags_7 },
    { 0x65548334, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92348338, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b4833b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d4833f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47748341, SUC_lexicon_tags_57 },
    { 0x72748342, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71148345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee748348, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9794834a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b4834e, SUC_lexicon_tags_107 },
    { 0xa8f4834f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36748358, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93948361, SUC_lexicon_tags_17 },
    { 0xb7b48362, SUC_lexicon_tags_95 },
    { 0xe2b48363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45548365, SUC_lexicon_tags_1 },
    { 0x68548366, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80748368, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2d4836a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f4836d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2348370, SUC_lexicon_tags_1 },
    { 0xdb748371, SUC_lexicon_tags_39 },
    { 0x76548372, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71948378, SUC_lexicon_tags_7 },
    { 0x7db48378, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7354837f, SUC_lexicon_tags_7 },
    { 0x8e74837f, SUC_lexicon_tags_12 },
    { 0x21b48381, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3d48383, SUC_lexicon_tags_38 },
    { 0x9d48383, SUC_lexicon_tags_77 },
    { 0xc3948384, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8548387, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34838c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9854838f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x148392, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf4839a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50d4839c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07483a1, SUC_lexicon_tags_77 },
    { 0xfed483a2, SUC_lexicon_tags_15 },
    { 0xf9f483a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa9f483a5, SUC_lexicon_tags_47 },
    { 0x203483a5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989483aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47483af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7483b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe95483b4, SUC_lexicon_tags_92 },
    { 0x607483b5, SUC_lexicon_tags_10 },
    { 0x97f483b4, SUC_lexicon_tags_25 },
    { 0x51b483b5, SUC_lexicon_tags_14 },
    { 0xe1d483b8, SUC_lexicon_tags_63 },
    { 0xed9483b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x31d483bb, SUC_lexicon_tags_17 },
    { 0x575483bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x705483be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611483c4, SUC_lexicon_tags_13 },
    { 0x8af483c4, SUC_lexicon_tags_11 },
    { 0x5e3483c5, SUC_lexicon_tags_11 },
    { 0xa67483c4, SUC_lexicon_tags_15 },
    { 0x1d1483c8, SUC_lexicon_tags_17 },
    { 0x91d483c4, SUC_lexicon_tags_26 },
    { 0x347483c6, SUC_lexicon_tags_21 },
    { 0xad5483cb, SUC_lexicon_tags_45 },
    { 0xc0b483cc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5483cf, SUC_lexicon_tags_30 },
    { 0x475483d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b7483d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3483d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x833483da, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f483dd, SUC_lexicon_tags_34 },
    { 0xdd9483de, SUC_lexicon_tags_26 },
    { 0xbc3483df, SUC_lexicon_tags_26 },
    { 0x105483de, SUC_lexicon_tags_17 },
    { 0x927483dd, SUC_lexicon_tags_19 },
    { 0x643483e2, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x281483e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03483e9, SUC_lexicon_tags_43 },
    { 0xedf483ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7483ed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5483f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9483f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315483fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9483ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf748402, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x79948404, SUC_lexicon_tags_37 },
    { 0x8d148405, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b4840b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe874840f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b148414, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0748419, SUC_lexicon_tags_8 },
    { 0xb294841a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed14841f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a348424, SUC_lexicon_tags_27 },
    { 0xad748425, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f948429, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b4842c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8948432, SUC_lexicon_tags_19 },
    { 0xc8348433, SUC_lexicon_tags_17 },
    { 0x89f48432, SUC_lexicon_tags_28 },
    { 0xaa348435, SUC_lexicon_tags_47 },
    { 0xb4348436, SUC_lexicon_tags_11 },
    { 0x36948437, SUC_lexicon_tags_37 },
    { 0x82548438, SUC_lexicon_tags_2 },
    { 0xf2548439, SUC_lexicon_tags_11 },
    { 0xcf14843a, SUC_lexicon_tags_45 },
    { 0x8754843b, SUC_lexicon_tags_33 },
    { 0xae748433, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4748441, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d48444, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43948446, SUC_lexicon_tags_23 },
    { 0xba148447, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47d48449, SUC_lexicon_tags_15 },
    { 0x2454844a, SUC_lexicon_tags_14 },
    { 0x3634844b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4844f, SUC_lexicon_tags_15 },
    { 0x35d48450, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31348454, SUC_lexicon_tags_25 },
    { 0x3db48455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe154845a, SUC_lexicon_tags_42 },
    { 0x6594845a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5574845f, SUC_lexicon_tags_68 },
    { 0xe4148460, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c548467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f4846a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f4846f, SUC_lexicon_tags_47 },
    { 0x2748470, SUC_lexicon_tags_31 },
    { 0x5748471, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d48477, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1434847e, SUC_lexicon_tags_14 },
    { 0x3774847f, SUC_lexicon_tags_320 },
    { 0x74748480, SUC_lexicon_tags_1 },
    { 0x7d48481, SUC_lexicon_tags_34 },
    { 0xb9748482, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c348486, SUC_lexicon_tags_36 },
    { 0xa9b48487, SUC_lexicon_tags_17 },
    { 0x76d48487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5154848b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c94848f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e148495, SUC_lexicon_tags_38 },
    { 0xcc148496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8174849b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6634849d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3614849f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f484a2, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0x423484a5, SUC_lexicon_tags_48 },
    { 0x51b484a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x455484a8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff484ab, SUC_lexicon_tags_12 },
    { 0xa8d484ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15484b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x639484b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd484b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x385484b7, SUC_lexicon_tags_59 },
    { 0xaad484b8, SUC_lexicon_tags_33 },
    { 0xb2d484b9, SUC_lexicon_tags_15 },
    { 0xbeb484ba, SUC_lexicon_tags_12 },
    { 0x401484bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61484bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdf484c1, SUC_lexicon_tags_23 },
    { 0x847484c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27484c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857484cd, SUC_lexicon_tags_38 },
    { 0xa4d484cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x697484d0, SUC_lexicon_tags_14 },
    { 0xb2f484d1, SUC_lexicon_tags_29 },
    { 0xe9b484d2, SUC_lexicon_tags_37 },
    { 0xa59484d3, SUC_lexicon_tags_37 },
    { 0x77b484d4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd7484d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x609484d8, SUC_lexicon_tags_34 },
    { 0xa67484d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1484dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9484e0, SUC_lexicon_tags_95 },
    { 0x2c7484e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9484e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b7484e7, SUC_lexicon_tags_1054 },
    { 0, NULL },
    { 0xf53484e9, SUC_lexicon_tags_17 },
    { 0xf25484ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47484ed, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29484f1, SUC_lexicon_tags_15 },
    { 0xd21484f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd484fb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf67484fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77348501, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d948504, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f48509, SUC_lexicon_tags_45 },
    { 0x1234850a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f54850f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa3f48511, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d48515, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2b48517, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef74851b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc548523, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a948528, SUC_lexicon_tags_5 },
    { 0x21348528, SUC_lexicon_tags_47 },
    { 0xe9d4852a, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa554852e, SUC_lexicon_tags_15 },
    { 0x7274852e, SUC_lexicon_tags_68 },
    { 0xfad48530, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xff748533, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe348536, SUC_lexicon_tags_27 },
    { 0x34748537, SUC_lexicon_tags_0 },
    { 0xc6f48536, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8074853e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x61348541, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x72f48543, SUC_lexicon_tags_3 },
    { 0x97348544, SUC_lexicon_tags_26 },
    { 0xd7b48545, SUC_lexicon_tags_27 },
    { 0x3148545, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc634854d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3314854f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1748552, SUC_lexicon_tags_0 },
    { 0xaab48553, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f748558, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad94855b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa414855d, SUC_lexicon_tags_31 },
    { 0x27f4855d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d48561, SUC_lexicon_tags_2 },
    { 0xe4f48562, SUC_lexicon_tags_37 },
    { 0x88d48563, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cd48565, SUC_lexicon_tags_2 },
    { 0xc8148566, SUC_lexicon_tags_48 },
    { 0x1748566, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba34856d, SUC_lexicon_tags_25 },
    { 0x5b94856e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x34b48570, SUC_lexicon_tags_37 },
    { 0x45348571, SUC_lexicon_tags_0 },
    { 0x89548572, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35b48574, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34857e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f148581, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15348586, SUC_lexicon_tags_25 },
    { 0x1f948586, SUC_lexicon_tags_1 },
    { 0xf4548588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2834858a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2794858f, SUC_lexicon_tags_37 },
    { 0x91f48590, SUC_lexicon_tags_25 },
    { 0x78b48590, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52948595, SUC_lexicon_tags_11 },
    { 0x35d48596, SUC_lexicon_tags_7 },
    { 0x89b48595, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xca94859a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4594859f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7485a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495485ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x549485ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8a3485b0, SUC_lexicon_tags_37 },
    { 0x2a3485b1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693485b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x953485b7, SUC_lexicon_tags_7 },
    { 0xaf5485b8, SUC_lexicon_tags_9 },
    { 0xcd3485b9, SUC_lexicon_tags_45 },
    { 0x127485ba, SUC_lexicon_tags_17 },
    { 0x8d3485b9, SUC_lexicon_tags_23 },
    { 0xb01485b7, SUC_lexicon_tags_23 },
    { 0x78d485bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x275485bf, SUC_lexicon_tags_2 },
    { 0x7ef485bf, SUC_lexicon_tags_37 },
    { 0xd13485c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb485c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99485ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5485cd, SUC_lexicon_tags_40 },
    { 0xd17485cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1485d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c9485d3, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef485dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf485e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7485e5, SUC_lexicon_tags_32 },
    { 0x731485e5, SUC_lexicon_tags_55 },
    { 0x7e7485e6, SUC_lexicon_tags_25 },
    { 0x1a5485e8, SUC_lexicon_tags_27 },
    { 0x96b485e9, SUC_lexicon_tags_8 },
    { 0xad3485e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b485ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdd485ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9485f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3485f6, SUC_lexicon_tags_17 },
    { 0x91f485f7, SUC_lexicon_tags_13 },
    { 0x3d7485f6, SUC_lexicon_tags_15 },
    { 0x9fb485f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1485ff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3148602, SUC_lexicon_tags_8 },
    { 0x14748603, SUC_lexicon_tags_5 },
    { 0x63b48603, SUC_lexicon_tags_25 },
    { 0x9348603, SUC_lexicon_tags_7 },
    { 0x1f748606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2948612, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2cb48614, SUC_lexicon_tags_12 },
    { 0xcfb48615, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d148618, SUC_lexicon_tags_12 },
    { 0xe8748618, SUC_lexicon_tags_37 },
    { 0x6c74861a, SUC_lexicon_tags_11 },
    { 0x70d4861a, SUC_lexicon_tags_1 },
    { 0xf074861c, SUC_lexicon_tags_31 },
    { 0x16748618, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7234861f, SUC_lexicon_tags_37 },
    { 0xe7148620, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5f48622, SUC_lexicon_tags_23 },
    { 0xe148623, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf948625, SUC_lexicon_tags_47 },
    { 0xab548625, SUC_lexicon_tags_31 },
    { 0xd7948626, SUC_lexicon_tags_25 },
    { 0x65548627, SUC_lexicon_tags_13 },
    { 0x80f48629, SUC_lexicon_tags_7 },
    { 0xd48628, SUC_lexicon_tags_15 },
    { 0x56348629, SUC_lexicon_tags_13 },
    { 0x7b34862c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f548632, SUC_lexicon_tags_12 },
    { 0xecd48632, SUC_lexicon_tags_160 },
    { 0xd5548634, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb48639, SUC_lexicon_tags_33 },
    { 0x9954863a, SUC_lexicon_tags_15 },
    { 0x2b74863a, SUC_lexicon_tags_1575 },
    { 0x5b4863c, SUC_lexicon_tags_1 },
    { 0xb0b4863c, SUC_lexicon_tags_11 },
    { 0x6154863e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56548640, SUC_lexicon_tags_37 },
    { 0x9e948641, SUC_lexicon_tags_38 },
    { 0x18d48641, SUC_lexicon_tags_60 },
    { 0x32748642, SUC_lexicon_tags_14 },
    { 0x49948644, SUC_lexicon_tags_39 },
    { 0xed948645, SUC_lexicon_tags_48 },
    { 0x7d948645, SUC_lexicon_tags_8 },
    { 0xe0748646, SUC_lexicon_tags_26 },
    { 0x6548643, SUC_lexicon_tags_15 },
    { 0x15348649, SUC_lexicon_tags_20 },
    { 0xe2148642, SUC_lexicon_tags_11 },
    { 0x34d4864b, SUC_lexicon_tags_39 },
    { 0x6294864c, SUC_lexicon_tags_37 },
    { 0x1234864d, SUC_lexicon_tags_12 },
    { 0x5f94864c, SUC_lexicon_tags_23 },
    { 0xd5b4864f, SUC_lexicon_tags_31 },
    { 0x794864d, SUC_lexicon_tags_31 },
    { 0x33748651, SUC_lexicon_tags_0 },
    { 0x10948652, SUC_lexicon_tags_25 },
    { 0xc9148653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb948656, SUC_lexicon_tags_17 },
    { 0xd9748657, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd0b48659, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5274865d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ad4865f, SUC_lexicon_tags_38 },
    { 0xc9348660, SUC_lexicon_tags_15 },
    { 0xfcb48661, SUC_lexicon_tags_60 },
    { 0x66f48662, SUC_lexicon_tags_8 },
    { 0xbcb48660, SUC_lexicon_tags_45 },
    { 0x58348664, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c948666, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a548668, SUC_lexicon_tags_1 },
    { 0x5a748669, SUC_lexicon_tags_25 },
    { 0x6574866a, SUC_lexicon_tags_37 },
    { 0x46f4866b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e34866d, SUC_lexicon_tags_816 },
    { 0, NULL },
    { 0x2654866f, SUC_lexicon_tags_8 },
    { 0x8d4866f, SUC_lexicon_tags_15 },
    { 0x44f48670, SUC_lexicon_tags_2 },
    { 0x34f48672, SUC_lexicon_tags_25 },
    { 0x1d748673, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4348676, SUC_lexicon_tags_60 },
    { 0x18348676, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc854867c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b148680, SUC_lexicon_tags_37 },
    { 0xb9b48681, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72948683, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd348686, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8148689, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7654868d, SUC_lexicon_tags_30 },
    { 0xeb34868e, SUC_lexicon_tags_68 },
    { 0xfd74868f, SUC_lexicon_tags_39 },
    { 0x95b4868e, SUC_lexicon_tags_112 },
    { 0x4394868e, SUC_lexicon_tags_33 },
    { 0x45b4868d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14748696, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9394869b, SUC_lexicon_tags_0 },
    { 0x2fb4869b, SUC_lexicon_tags_28 },
    { 0x4b54869c, SUC_lexicon_tags_27 },
    { 0x4d94869e, SUC_lexicon_tags_25 },
    { 0xc9d4869f, SUC_lexicon_tags_1 },
    { 0xc314869f, SUC_lexicon_tags_37 },
    { 0xa85486a0, SUC_lexicon_tags_129 },
    { 0xfef486a2, SUC_lexicon_tags_1 },
    { 0x587486a1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1486a6, SUC_lexicon_tags_43 },
    { 0xd7486a7, SUC_lexicon_tags_27 },
    { 0x805486a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb486ac, SUC_lexicon_tags_11 },
    { 0x83b486ad, SUC_lexicon_tags_1 },
    { 0xdd486ae, SUC_lexicon_tags_0 },
    { 0x531486af, SUC_lexicon_tags_12 },
    { 0x26d486ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce9486b2, SUC_lexicon_tags_19 },
    { 0x4d5486b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1486b8, SUC_lexicon_tags_11 },
    { 0x995486b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f486be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863486cb, SUC_lexicon_tags_15 },
    { 0xa53486cc, SUC_lexicon_tags_28 },
    { 0xa6d486cc, SUC_lexicon_tags_23 },
    { 0x31486cd, SUC_lexicon_tags_2 },
    { 0x701486cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x109486d2, SUC_lexicon_tags_55 },
    { 0x989486d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b486d9, SUC_lexicon_tags_18 },
    { 0xa7b486da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101486e3, SUC_lexicon_tags_12 },
    { 0x34d486e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3486e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x331486e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f486ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc43486f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b486fb, SUC_lexicon_tags_1 },
    { 0xc0d486fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x455486ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe348701, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xedd48703, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1eb48705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa694870b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa114870e, SUC_lexicon_tags_15 },
    { 0xf594870e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f48712, SUC_lexicon_tags_17 },
    { 0x93b48713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d348715, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef48718, SUC_lexicon_tags_62 },
    { 0x4c748718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24948722, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xdc948724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db48729, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f74872d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45748733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51548736, SUC_lexicon_tags_46 },
    { 0x39748737, SUC_lexicon_tags_2 },
    { 0x6a548738, SUC_lexicon_tags_32 },
    { 0x84f48739, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9594873c, SUC_lexicon_tags_87 },
    { 0x514873d, SUC_lexicon_tags_66 },
    { 0xf834873e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3fd48740, SUC_lexicon_tags_23 },
    { 0xb3148741, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd4874a, SUC_lexicon_tags_14 },
    { 0x214874b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5734874f, SUC_lexicon_tags_23 },
    { 0xc14874f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbb348752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd548754, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d48756, SUC_lexicon_tags_2 },
    { 0x78748756, SUC_lexicon_tags_1 },
    { 0x39948758, SUC_lexicon_tags_9 },
    { 0x56f48759, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f4875c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x454875e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e748762, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3c348764, SUC_lexicon_tags_49 },
    { 0xc5b48764, SUC_lexicon_tags_23 },
    { 0xe1d48766, SUC_lexicon_tags_11 },
    { 0x54148766, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b74876c, SUC_lexicon_tags_234 },
    { 0xaa74876c, SUC_lexicon_tags_37 },
    { 0xc994876e, SUC_lexicon_tags_19 },
    { 0x16b4876e, SUC_lexicon_tags_30 },
    { 0x63b4876f, SUC_lexicon_tags_23 },
    { 0xa8948771, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x96548774, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19348776, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c94877b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb54877d, SUC_lexicon_tags_7 },
    { 0xeef4877d, SUC_lexicon_tags_4 },
    { 0x62d4877f, SUC_lexicon_tags_15 },
    { 0x1b548780, SUC_lexicon_tags_11 },
    { 0x78748780, SUC_lexicon_tags_7 },
    { 0x97748782, SUC_lexicon_tags_25 },
    { 0x8f548783, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b4878e, SUC_lexicon_tags_0 },
    { 0x99b4878f, SUC_lexicon_tags_38 },
    { 0x19b48790, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef48797, SUC_lexicon_tags_11 },
    { 0x4a748798, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5994879b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58d4879d, SUC_lexicon_tags_37 },
    { 0xf314879e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1487a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af487a6, SUC_lexicon_tags_34 },
    { 0x169487a6, SUC_lexicon_tags_25 },
    { 0x347487a8, SUC_lexicon_tags_9 },
    { 0xabb487a8, SUC_lexicon_tags_37 },
    { 0x77487a9, SUC_lexicon_tags_3 },
    { 0xc99487ab, SUC_lexicon_tags_12 },
    { 0x913487ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f487af, SUC_lexicon_tags_38 },
    { 0x5a3487af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233487b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3487b8, SUC_lexicon_tags_38 },
    { 0x31f487b9, SUC_lexicon_tags_38 },
    { 0x64b487ba, SUC_lexicon_tags_25 },
    { 0xdb5487bb, SUC_lexicon_tags_49 },
    { 0x501487bc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf03487be, SUC_lexicon_tags_15 },
    { 0x48f487bf, SUC_lexicon_tags_10 },
    { 0x76f487bf, SUC_lexicon_tags_47 },
    { 0x59d487c1, SUC_lexicon_tags_49 },
    { 0xbb9487c1, SUC_lexicon_tags_66 },
    { 0xc35487c2, SUC_lexicon_tags_33 },
    { 0x2f5487c1, SUC_lexicon_tags_63 },
    { 0x303487be, SUC_lexicon_tags_8 },
    { 0xa39487c6, SUC_lexicon_tags_1 },
    { 0xd65487be, SUC_lexicon_tags_194 },
    { 0x181487c3, SUC_lexicon_tags_17 },
    { 0xb8d487c9, SUC_lexicon_tags_30 },
    { 0xbc1487bf, SUC_lexicon_tags_31 },
    { 0xa45487c3, SUC_lexicon_tags_0 },
    { 0xfd3487cc, SUC_lexicon_tags_28 },
    { 0xa79487cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x771487d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x981487d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x635487d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9487da, SUC_lexicon_tags_2 },
    { 0xacb487da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d487df, SUC_lexicon_tags_11 },
    { 0xe15487e0, SUC_lexicon_tags_17 },
    { 0xe4d487e1, SUC_lexicon_tags_11 },
    { 0xda1487e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x991487e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235487ea, SUC_lexicon_tags_37 },
    { 0x6e5487ea, SUC_lexicon_tags_31 },
    { 0x761487ec, SUC_lexicon_tags_38 },
    { 0x87d487ec, SUC_lexicon_tags_37 },
    { 0x17b487ea, SUC_lexicon_tags_69 },
    { 0x65487ec, SUC_lexicon_tags_1453 },
    { 0x585487ea, SUC_lexicon_tags_16 },
    { 0xe99487eb, SUC_lexicon_tags_3 },
    { 0x529487f2, SUC_lexicon_tags_0 },
    { 0x413487ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1487f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99487fe, SUC_lexicon_tags_31 },
    { 0xba7487fe, SUC_lexicon_tags_25 },
    { 0xba148800, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d48807, SUC_lexicon_tags_1 },
    { 0x35748808, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63948815, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb4881b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeed4881d, SUC_lexicon_tags_10 },
    { 0x6c94881e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46548826, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x98d48828, SUC_lexicon_tags_8 },
    { 0x57548828, SUC_lexicon_tags_9 },
    { 0xc214882a, SUC_lexicon_tags_37 },
    { 0x7748829, SUC_lexicon_tags_27 },
    { 0x1274882c, SUC_lexicon_tags_60 },
    { 0xb154882d, SUC_lexicon_tags_29 },
    { 0x6354882c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe148835, SUC_lexicon_tags_48 },
    { 0xeb748836, SUC_lexicon_tags_31 },
    { 0xd2f48836, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67f48839, SUC_lexicon_tags_27 },
    { 0x6014883a, SUC_lexicon_tags_11 },
    { 0xe614883b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7774883d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa548843, SUC_lexicon_tags_8 },
    { 0x4a148843, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10748847, SUC_lexicon_tags_28 },
    { 0x99b48847, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x154884a, SUC_lexicon_tags_47 },
    { 0x3a34884a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a148851, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc054885a, SUC_lexicon_tags_7 },
    { 0xe934885a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db4885f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d48862, SUC_lexicon_tags_2 },
    { 0x19548862, SUC_lexicon_tags_29 },
    { 0xb3748864, SUC_lexicon_tags_7 },
    { 0x64548863, SUC_lexicon_tags_37 },
    { 0x5fd48866, SUC_lexicon_tags_46 },
    { 0x6dd48864, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4886a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa814886f, SUC_lexicon_tags_1 },
    { 0xfe348870, SUC_lexicon_tags_39 },
    { 0xcb548870, SUC_lexicon_tags_45 },
    { 0x29348872, SUC_lexicon_tags_7 },
    { 0x1948873, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb8948875, SUC_lexicon_tags_46 },
    { 0x39f48876, SUC_lexicon_tags_25 },
    { 0x3348877, SUC_lexicon_tags_0 },
    { 0xb4148875, SUC_lexicon_tags_12 },
    { 0xcbb48879, SUC_lexicon_tags_15 },
    { 0xeb748877, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44948886, SUC_lexicon_tags_19 },
    { 0x6a948887, SUC_lexicon_tags_7 },
    { 0xca148886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6234888f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f748898, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe74889d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b488a2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x88f488a4, SUC_lexicon_tags_1 },
    { 0x92d488a5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x211488a8, SUC_lexicon_tags_38 },
    { 0xdb5488a9, SUC_lexicon_tags_39 },
    { 0xb01488aa, SUC_lexicon_tags_14 },
    { 0x19d488ab, SUC_lexicon_tags_47 },
    { 0xb9f488ac, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x709488ae, SUC_lexicon_tags_37 },
    { 0x441488af, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b488b4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3488b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf07488b9, SUC_lexicon_tags_15 },
    { 0x49b488ba, SUC_lexicon_tags_47 },
    { 0x763488b9, SUC_lexicon_tags_19 },
    { 0xe61488bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c1488be, SUC_lexicon_tags_32 },
    { 0x5f7488bf, SUC_lexicon_tags_63 },
    { 0x3b3488c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1488c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575488c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd488ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d488d1, SUC_lexicon_tags_89 },
    { 0x6f9488d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa7488d4, SUC_lexicon_tags_15 },
    { 0x85488d5, SUC_lexicon_tags_87 },
    { 0x2d3488d5, SUC_lexicon_tags_37 },
    { 0xb1b488d6, SUC_lexicon_tags_40 },
    { 0xc43488d7, SUC_lexicon_tags_12 },
    { 0x11f488d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x27488dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd488e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb57488e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b488e7, SUC_lexicon_tags_264 },
    { 0xd61488e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d488ed, SUC_lexicon_tags_27 },
    { 0x245488ed, SUC_lexicon_tags_30 },
    { 0xc01488ee, SUC_lexicon_tags_26 },
    { 0xecf488f0, SUC_lexicon_tags_27 },
    { 0xbff488ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5488f5, SUC_lexicon_tags_1 },
    { 0x45b488f6, SUC_lexicon_tags_79 },
    { 0xacb488f7, SUC_lexicon_tags_11 },
    { 0x9fb488f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9b488fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf69488fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65f488fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1948900, SUC_lexicon_tags_71 },
    { 0x2c348901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5748905, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4890d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42348910, SUC_lexicon_tags_12 },
    { 0xae748910, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2548913, SUC_lexicon_tags_65 },
    { 0x8fb48914, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c348918, SUC_lexicon_tags_65 },
    { 0x1ab48919, SUC_lexicon_tags_64 },
    { 0xc8d4891a, SUC_lexicon_tags_47 },
    { 0x8894891a, SUC_lexicon_tags_32 },
    { 0x46f4891c, SUC_lexicon_tags_23 },
    { 0x1f14891d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f148922, SUC_lexicon_tags_55 },
    { 0x97948923, SUC_lexicon_tags_25 },
    { 0xef148924, SUC_lexicon_tags_37 },
    { 0x3f148925, SUC_lexicon_tags_39 },
    { 0xf4148925, SUC_lexicon_tags_45 },
    { 0x6dd48927, SUC_lexicon_tags_12 },
    { 0x1a148922, SUC_lexicon_tags_4 },
    { 0xc0748926, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7614892c, SUC_lexicon_tags_45 },
    { 0x9874892d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70148932, SUC_lexicon_tags_1 },
    { 0x86948933, SUC_lexicon_tags_25 },
    { 0x61548932, SUC_lexicon_tags_9 },
    { 0x93348933, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead4893c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23748940, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58748942, SUC_lexicon_tags_11 },
    { 0x5b748943, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c348946, SUC_lexicon_tags_48 },
    { 0x2c948946, SUC_lexicon_tags_126 },
    { 0xc4b48948, SUC_lexicon_tags_5 },
    { 0xeed48948, SUC_lexicon_tags_25 },
    { 0x2b54894a, SUC_lexicon_tags_26 },
    { 0x5154894a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c54894f, SUC_lexicon_tags_23 },
    { 0xc0b48950, SUC_lexicon_tags_11 },
    { 0x76548951, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc1148953, SUC_lexicon_tags_2 },
    { 0x87948953, SUC_lexicon_tags_19 },
    { 0xe548955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1948959, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xab74895b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa54895f, SUC_lexicon_tags_8 },
    { 0xc1348960, SUC_lexicon_tags_11 },
    { 0xa3748961, SUC_lexicon_tags_7 },
    { 0x3cd48960, SUC_lexicon_tags_15 },
    { 0x38b48960, SUC_lexicon_tags_1 },
    { 0x20348964, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x91748966, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef48969, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba74896b, SUC_lexicon_tags_38 },
    { 0xed74896b, SUC_lexicon_tags_28 },
    { 0x7c74896c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x32548970, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd48975, SUC_lexicon_tags_37 },
    { 0x14748976, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd274897b, SUC_lexicon_tags_6 },
    { 0x36d4897c, SUC_lexicon_tags_29 },
    { 0xec54897d, SUC_lexicon_tags_38 },
    { 0xd2f4897d, SUC_lexicon_tags_8 },
    { 0x4eb4897f, SUC_lexicon_tags_11 },
    { 0xdc54897f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x85948983, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37348986, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a34898b, SUC_lexicon_tags_12 },
    { 0xd194898c, SUC_lexicon_tags_65 },
    { 0xd494898b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x3db4898f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x49948991, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x80948993, SUC_lexicon_tags_47 },
    { 0x7b948993, SUC_lexicon_tags_25 },
    { 0xb1d48994, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b48998, SUC_lexicon_tags_28 },
    { 0x23748999, SUC_lexicon_tags_47 },
    { 0x21148999, SUC_lexicon_tags_37 },
    { 0x9f54899b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd294899d, SUC_lexicon_tags_7 },
    { 0x4a94899e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183489a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9d489a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1f9489a8, SUC_lexicon_tags_27 },
    { 0x473489a9, SUC_lexicon_tags_37 },
    { 0x2f489a9, SUC_lexicon_tags_25 },
    { 0xb95489aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b5489ad, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa53489af, SUC_lexicon_tags_49 },
    { 0x787489af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x41b489b2, SUC_lexicon_tags_1 },
    { 0xe07489b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183489b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d489bb, SUC_lexicon_tags_46 },
    { 0x28f489bc, SUC_lexicon_tags_60 },
    { 0x39f489bd, SUC_lexicon_tags_17 },
    { 0xe99489be, SUC_lexicon_tags_47 },
    { 0x35d489be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd95489c1, SUC_lexicon_tags_12 },
    { 0xc81489c2, SUC_lexicon_tags_57 },
    { 0x3cd489c3, SUC_lexicon_tags_39 },
    { 0x589489c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb489c7, SUC_lexicon_tags_33 },
    { 0x2e5489c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93489cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54f489cd, SUC_lexicon_tags_10 },
    { 0x8df489ce, SUC_lexicon_tags_43 },
    { 0x1fd489ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3489d5, SUC_lexicon_tags_133 },
    { 0xab3489d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1489d9, SUC_lexicon_tags_34 },
    { 0x683489da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413489df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287489e4, SUC_lexicon_tags_45 },
    { 0x75489e5, SUC_lexicon_tags_1 },
    { 0xdb9489e6, SUC_lexicon_tags_8 },
    { 0x8ad489e7, SUC_lexicon_tags_7 },
    { 0x11489e8, SUC_lexicon_tags_17 },
    { 0x73b489e9, SUC_lexicon_tags_37 },
    { 0xf57489ea, SUC_lexicon_tags_17 },
    { 0xc57489eb, SUC_lexicon_tags_15 },
    { 0xa37489e9, SUC_lexicon_tags_80 },
    { 0xd8d489ed, SUC_lexicon_tags_2 },
    { 0xfb7489ed, SUC_lexicon_tags_28 },
    { 0x451489ee, SUC_lexicon_tags_8 },
    { 0x87d489f0, SUC_lexicon_tags_60 },
    { 0xf6f489f1, SUC_lexicon_tags_37 },
    { 0x437489ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39489f9, SUC_lexicon_tags_12 },
    { 0x8c3489fa, SUC_lexicon_tags_0 },
    { 0xc4b489fa, SUC_lexicon_tags_25 },
    { 0xadd489f9, SUC_lexicon_tags_6 },
    { 0x993489fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f48a03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc548a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x98548a09, SUC_lexicon_tags_11 },
    { 0xe0b48a09, SUC_lexicon_tags_1 },
    { 0x26f48a0b, SUC_lexicon_tags_4 },
    { 0x59748a0c, SUC_lexicon_tags_37 },
    { 0xa6748a0d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xb5348a0f, SUC_lexicon_tags_124 },
    { 0xc6f48a10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb48a17, SUC_lexicon_tags_17 },
    { 0x548a18, SUC_lexicon_tags_2 },
    { 0x1ed48a19, SUC_lexicon_tags_15 },
    { 0x1c748a18, SUC_lexicon_tags_9 },
    { 0xcf348a19, SUC_lexicon_tags_15 },
    { 0x13d48a18, SUC_lexicon_tags_37 },
    { 0xb8948a1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb48a22, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3e148a24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e348a28, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3d348a2a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94348a2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d348a33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3948a3a, SUC_lexicon_tags_60 },
    { 0x19148a3b, SUC_lexicon_tags_37 },
    { 0xd9d48a3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab548a3e, SUC_lexicon_tags_14 },
    { 0x97948a3f, SUC_lexicon_tags_37 },
    { 0x99148a3f, SUC_lexicon_tags_14 },
    { 0x14148a41, SUC_lexicon_tags_23 },
    { 0xe4748a42, SUC_lexicon_tags_27 },
    { 0xabf48a43, SUC_lexicon_tags_25 },
    { 0x55d48a44, SUC_lexicon_tags_87 },
    { 0x58b48a45, SUC_lexicon_tags_12 },
    { 0x28148a44, SUC_lexicon_tags_30 },
    { 0x80548a47, SUC_lexicon_tags_30 },
    { 0xc7d48a46, SUC_lexicon_tags_17 },
    { 0x8d348a44, SUC_lexicon_tags_14 },
    { 0x6e348a46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1948a4e, SUC_lexicon_tags_6 },
    { 0x84f48a4f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x71748a51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d148a54, SUC_lexicon_tags_35 },
    { 0x15b48a55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94148a57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb48a5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22748a5e, SUC_lexicon_tags_1 },
    { 0xd3b48a5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda748a69, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x68948a6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf48a6e, SUC_lexicon_tags_66 },
    { 0x9a348a6f, SUC_lexicon_tags_38 },
    { 0xfb148a6e, SUC_lexicon_tags_14 },
    { 0x4c948a71, SUC_lexicon_tags_136 },
    { 0xf2348a70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb48a77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b48a81, SUC_lexicon_tags_39 },
    { 0xec748a81, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d48a86, SUC_lexicon_tags_69 },
    { 0x6d148a86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed48a8a, SUC_lexicon_tags_38 },
    { 0x2ed48a8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc748a90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x65348a92, SUC_lexicon_tags_28 },
    { 0x9fd48a93, SUC_lexicon_tags_7 },
    { 0xb8948a93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xded48a96, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdf148a98, SUC_lexicon_tags_25 },
    { 0x35b48a98, SUC_lexicon_tags_46 },
    { 0x11948a99, SUC_lexicon_tags_15 },
    { 0x82d48a9a, SUC_lexicon_tags_33 },
    { 0x73348a99, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x97148a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16548aa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb48aa3, SUC_lexicon_tags_37 },
    { 0x73748aa4, SUC_lexicon_tags_17 },
    { 0x5e348aa4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95548aab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa548aad, SUC_lexicon_tags_15 },
    { 0x7148aae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f748ab5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x34748ab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5d48ab9, SUC_lexicon_tags_13 },
    { 0xfc148aba, SUC_lexicon_tags_0 },
    { 0x72548abb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b48ac2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda148ac5, SUC_lexicon_tags_29 },
    { 0xd1348ac6, SUC_lexicon_tags_2 },
    { 0xe348ac7, SUC_lexicon_tags_17 },
    { 0xadd48ac8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f48acb, SUC_lexicon_tags_28 },
    { 0x2eb48acb, SUC_lexicon_tags_7 },
    { 0x78548acd, SUC_lexicon_tags_47 },
    { 0x9eb48acb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfeb48ad0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x81d48ad2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x76548ad4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48548ad6, SUC_lexicon_tags_17 },
    { 0x8b148ad7, SUC_lexicon_tags_63 },
    { 0x63348ad7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d48adb, SUC_lexicon_tags_19 },
    { 0x63b48adb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3af48ade, SUC_lexicon_tags_7 },
    { 0x5b48adf, SUC_lexicon_tags_45 },
    { 0x2d148ae0, SUC_lexicon_tags_39 },
    { 0xacf48ae0, SUC_lexicon_tags_2 },
    { 0x68948ae2, SUC_lexicon_tags_55 },
    { 0x1fb48ae3, SUC_lexicon_tags_25 },
    { 0xacf48ae0, SUC_lexicon_tags_7 },
    { 0xa1548ae3, SUC_lexicon_tags_14 },
    { 0xea548ae6, SUC_lexicon_tags_48 },
    { 0x83748ae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb48aeb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xad348aee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7748af1, SUC_lexicon_tags_60 },
    { 0x93b48af1, SUC_lexicon_tags_10 },
    { 0x76f48af3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8748afa, SUC_lexicon_tags_11 },
    { 0x26b48afb, SUC_lexicon_tags_2 },
    { 0xa7d48afc, SUC_lexicon_tags_2 },
    { 0x2e548afd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d48b02, SUC_lexicon_tags_0 },
    { 0x2f148b03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6948b05, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x18348b07, SUC_lexicon_tags_26 },
    { 0xae348b08, SUC_lexicon_tags_91 },
    { 0xa4f48b07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd48b0f, SUC_lexicon_tags_1 },
    { 0x81d48b10, SUC_lexicon_tags_84 },
    { 0xfaf48b11, SUC_lexicon_tags_14 },
    { 0xfa948b12, SUC_lexicon_tags_7 },
    { 0xf1548b13, SUC_lexicon_tags_12 },
    { 0x1e148b11, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f48b17, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x2c348b19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8348b1b, SUC_lexicon_tags_47 },
    { 0xebb48b1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53748b1f, SUC_lexicon_tags_21 },
    { 0x5948b20, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xab548b22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97d48b24, SUC_lexicon_tags_6 },
    { 0xeef48b24, SUC_lexicon_tags_26 },
    { 0x46548b26, SUC_lexicon_tags_33 },
    { 0x86548b24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7548b29, SUC_lexicon_tags_11 },
    { 0x2af48b2a, SUC_lexicon_tags_11 },
    { 0x5c348b2a, SUC_lexicon_tags_505 },
    { 0xdb748b2a, SUC_lexicon_tags_2 },
    { 0x8a748b2a, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0x48548b2f, SUC_lexicon_tags_37 },
    { 0x3bf48b30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x28548b32, SUC_lexicon_tags_29 },
    { 0xf5948b33, SUC_lexicon_tags_63 },
    { 0x26348b34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3148b36, SUC_lexicon_tags_1 },
    { 0xd2f48b36, SUC_lexicon_tags_7 },
    { 0xc4548b38, SUC_lexicon_tags_2 },
    { 0xfa748b38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61548b3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9548b41, SUC_lexicon_tags_84 },
    { 0x9ad48b42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15348b48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f48b4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff48b4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c548b52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14948b59, SUC_lexicon_tags_7 },
    { 0xb2748b5a, SUC_lexicon_tags_29 },
    { 0x40d48b5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab48b5f, SUC_lexicon_tags_37 },
    { 0xbd48b60, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31148b67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1548b6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59548b70, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x78b48b72, SUC_lexicon_tags_12 },
    { 0x24b48b73, SUC_lexicon_tags_25 },
    { 0xaa548b74, SUC_lexicon_tags_13 },
    { 0xf0148b72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a548b77, SUC_lexicon_tags_108 },
    { 0xf0d48b77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69948b7a, SUC_lexicon_tags_11 },
    { 0x18d48b7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec148b7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8148b7f, SUC_lexicon_tags_2 },
    { 0x8f948b80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab48b83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44748b85, SUC_lexicon_tags_9 },
    { 0x18948b85, SUC_lexicon_tags_11 },
    { 0xc1b48b87, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d348b8d, SUC_lexicon_tags_2 },
    { 0xb4348b8e, SUC_lexicon_tags_2 },
    { 0x3a148b8f, SUC_lexicon_tags_2 },
    { 0x77148b8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe148b95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf48b9b, SUC_lexicon_tags_46 },
    { 0x8f548b9c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db48ba0, SUC_lexicon_tags_11 },
    { 0x71348ba1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f548ba4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x45548ba6, SUC_lexicon_tags_2 },
    { 0x4cf48ba7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2748bad, SUC_lexicon_tags_33 },
    { 0x52948bad, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc548bb1, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x50348bb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d48bb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f48bba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d48bbe, SUC_lexicon_tags_17 },
    { 0x56348bbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb48bc2, SUC_lexicon_tags_21 },
    { 0x75548bc3, SUC_lexicon_tags_37 },
    { 0x38548bc4, SUC_lexicon_tags_43 },
    { 0xfff48bc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbb48bc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29748bc9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x21b48bcb, SUC_lexicon_tags_47 },
    { 0xf2748bcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd48bd0, SUC_lexicon_tags_7 },
    { 0xbab48bd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6148bd7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f748bdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xedb48bde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b48be8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d48beb, SUC_lexicon_tags_37 },
    { 0xb8f48bec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed48bef, SUC_lexicon_tags_1 },
    { 0x27548bef, SUC_lexicon_tags_6 },
    { 0xe4148bef, SUC_lexicon_tags_7 },
    { 0xa6148bf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36148bf6, SUC_lexicon_tags_7 },
    { 0x73548bf7, SUC_lexicon_tags_12 },
    { 0xf5148bf8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44148bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb48c02, SUC_lexicon_tags_4 },
    { 0xdff48c03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28748c07, SUC_lexicon_tags_14 },
    { 0x23b48c08, SUC_lexicon_tags_31 },
    { 0x8ab48c09, SUC_lexicon_tags_28 },
    { 0x37348c0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98948c0d, SUC_lexicon_tags_16 },
    { 0x9cd48c0d, SUC_lexicon_tags_37 },
    { 0xc3b48c0e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4948c15, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20148c19, SUC_lexicon_tags_2 },
    { 0xdf948c1a, SUC_lexicon_tags_1 },
    { 0xd2148c1b, SUC_lexicon_tags_22 },
    { 0x3a348c1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71348c1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3348c21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70948c27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94948c2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb48c2e, SUC_lexicon_tags_11 },
    { 0xdc348c2f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf5d48c31, SUC_lexicon_tags_29 },
    { 0xa1348c32, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0948c35, SUC_lexicon_tags_14 },
    { 0x92748c36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ff48c38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c348c3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa148c40, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75748c49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd48c4c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b48c55, SUC_lexicon_tags_31 },
    { 0x80148c56, SUC_lexicon_tags_39 },
    { 0xdff48c55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd48c5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f948c60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6cd48c62, SUC_lexicon_tags_11 },
    { 0xe7d48c63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f48c6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb48c6f, SUC_lexicon_tags_8 },
    { 0x24348c70, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xab948c73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc548c76, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6c748c78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90948c7b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10148c82, SUC_lexicon_tags_7 },
    { 0xfc748c83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d48c87, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x25b48c89, SUC_lexicon_tags_2 },
    { 0x27148c89, SUC_lexicon_tags_30 },
    { 0x95548c8b, SUC_lexicon_tags_23 },
    { 0x1c748c8c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xae948c8f, SUC_lexicon_tags_62 },
    { 0xa0348c90, SUC_lexicon_tags_2 },
    { 0x96d48c91, SUC_lexicon_tags_67 },
    { 0xa5d48c92, SUC_lexicon_tags_12 },
    { 0xc1148c93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6148c97, SUC_lexicon_tags_1 },
    { 0x44d48c98, SUC_lexicon_tags_11 },
    { 0x20948c97, SUC_lexicon_tags_15 },
    { 0xe8348c9a, SUC_lexicon_tags_68 },
    { 0x62f48c9a, SUC_lexicon_tags_11 },
    { 0x34948c9a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xbaf48c9e, SUC_lexicon_tags_33 },
    { 0x4a348c9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93948ca1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28548ca6, SUC_lexicon_tags_40 },
    { 0xfc748ca7, SUC_lexicon_tags_39 },
    { 0x54b48ca8, SUC_lexicon_tags_37 },
    { 0x4b48ca6, SUC_lexicon_tags_45 },
    { 0xd948caa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c148cac, SUC_lexicon_tags_2 },
    { 0x2b748cad, SUC_lexicon_tags_0 },
    { 0x43148cad, SUC_lexicon_tags_26 },
    { 0xde548cae, SUC_lexicon_tags_56 },
    { 0x9f148caf, SUC_lexicon_tags_2 },
    { 0x61b48cb1, SUC_lexicon_tags_39 },
    { 0xc7148cb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf3b48cb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64b48cb6, SUC_lexicon_tags_12 },
    { 0x1c548cb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3548cbd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e348cc0, SUC_lexicon_tags_49 },
    { 0x31948cc1, SUC_lexicon_tags_60 },
    { 0xfff48cc1, SUC_lexicon_tags_66 },
    { 0x16748cc3, SUC_lexicon_tags_47 },
    { 0xa8b48cc4, SUC_lexicon_tags_6 },
    { 0x49f48cc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3148cc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d48ccc, SUC_lexicon_tags_25 },
    { 0x7b48ccd, SUC_lexicon_tags_60 },
    { 0x70d48cce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe2748cd0, SUC_lexicon_tags_12 },
    { 0x91b48cd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97548cd4, SUC_lexicon_tags_6 },
    { 0xc2348cd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61348cd7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8ef48cd9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xad348cdc, SUC_lexicon_tags_4 },
    { 0x36348cdd, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba748ce8, SUC_lexicon_tags_1 },
    { 0x2d348ce9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48548cec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87148cf2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f48cf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d548cfb, SUC_lexicon_tags_56 },
    { 0x70148cfc, SUC_lexicon_tags_27 },
    { 0x7c948cfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea748d02, SUC_lexicon_tags_23 },
    { 0xae748d03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1748d07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23148d17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5948d19, SUC_lexicon_tags_42 },
    { 0x36748d19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45d48d1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3c548d1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3548d20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2b48d22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52148d26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a348d28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9748d2b, SUC_lexicon_tags_37 },
    { 0x29f48d2b, SUC_lexicon_tags_28 },
    { 0x81f48d2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf48d38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe748d3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10548d42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5548d44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59d48d46, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdd348d48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72948d4b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe5348d4d, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0x49948d4f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x32b48d51, SUC_lexicon_tags_49 },
    { 0x7748d52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f348d57, SUC_lexicon_tags_11 },
    { 0x7548d58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14748d5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9ab48d5c, SUC_lexicon_tags_37 },
    { 0x97348d5d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab948d61, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2f48d63, SUC_lexicon_tags_15 },
    { 0xb5548d64, SUC_lexicon_tags_17 },
    { 0x56748d65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb48d69, SUC_lexicon_tags_46 },
    { 0xe148d6a, SUC_lexicon_tags_37 },
    { 0x77948d6a, SUC_lexicon_tags_15 },
    { 0xe2b48d6c, SUC_lexicon_tags_1 },
    { 0x5a748d6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcb48d6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x68348d71, SUC_lexicon_tags_37 },
    { 0xda748d72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f148d7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75948d7d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x48f48d7f, SUC_lexicon_tags_47 },
    { 0x21348d80, SUC_lexicon_tags_49 },
    { 0xa0d48d80, SUC_lexicon_tags_11 },
    { 0x63948d7f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6348d87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdb348d89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92948d8b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x38148d8d, SUC_lexicon_tags_32 },
    { 0x5f948d8d, SUC_lexicon_tags_17 },
    { 0xa6b48d8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4d548d91, SUC_lexicon_tags_12 },
    { 0xbf948d92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3748d96, SUC_lexicon_tags_7 },
    { 0x9ad48d97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8748d9c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x17148d9e, SUC_lexicon_tags_7 },
    { 0x40d48d9f, SUC_lexicon_tags_46 },
    { 0xd0d48da0, SUC_lexicon_tags_32 },
    { 0xbbd48da0, SUC_lexicon_tags_19 },
    { 0x68348da2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34948dab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b48dae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad948db8, SUC_lexicon_tags_17 },
    { 0x12d48db9, SUC_lexicon_tags_66 },
    { 0xbd48db9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b148dbd, SUC_lexicon_tags_2 },
    { 0x3af48dbe, SUC_lexicon_tags_25 },
    { 0xf2148dbf, SUC_lexicon_tags_27 },
    { 0x3c148dbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdb348dc2, SUC_lexicon_tags_23 },
    { 0x97948dc3, SUC_lexicon_tags_27 },
    { 0xaf548dc4, SUC_lexicon_tags_15 },
    { 0xd5148dc5, SUC_lexicon_tags_15 },
    { 0xa1948dc6, SUC_lexicon_tags_47 },
    { 0xe2b48dc7, SUC_lexicon_tags_17 },
    { 0xb1f48dc8, SUC_lexicon_tags_87 },
    { 0xa2f48dc7, SUC_lexicon_tags_15 },
    { 0x2b48dc8, SUC_lexicon_tags_37 },
    { 0x96b48dcb, SUC_lexicon_tags_140 },
    { 0xa6148dc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf48dd0, SUC_lexicon_tags_23 },
    { 0xe2548dd1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x30348dd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecf48dd5, SUC_lexicon_tags_4 },
    { 0x69948dd6, SUC_lexicon_tags_13 },
    { 0x5c748dd7, SUC_lexicon_tags_37 },
    { 0xf7348dd8, SUC_lexicon_tags_16 },
    { 0x5ab48dd8, SUC_lexicon_tags_9 },
    { 0xfbb48dd9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd7f48ddc, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6348de3, SUC_lexicon_tags_5 },
    { 0xd1748de4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c348dec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56148df0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x15b48df2, SUC_lexicon_tags_48 },
    { 0x60948df3, SUC_lexicon_tags_31 },
    { 0x8cd48df4, SUC_lexicon_tags_25 },
    { 0xb9548df5, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed48df9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x65b48dfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51148dfd, SUC_lexicon_tags_7 },
    { 0x54d48dfe, SUC_lexicon_tags_1 },
    { 0x548dfe, SUC_lexicon_tags_23 },
    { 0x51548dfe, SUC_lexicon_tags_47 },
    { 0xead48dfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfff48e03, SUC_lexicon_tags_10 },
    { 0x88548e04, SUC_lexicon_tags_5 },
    { 0xf7348e03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57548e07, SUC_lexicon_tags_7 },
    { 0x27d48e08, SUC_lexicon_tags_9 },
    { 0x12d48e08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x65348e0b, SUC_lexicon_tags_47 },
    { 0xc8f48e0b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d48e11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9d48e13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc548e15, SUC_lexicon_tags_7 },
    { 0x3cd48e16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cd48e18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf48e1b, SUC_lexicon_tags_47 },
    { 0x41748e1b, SUC_lexicon_tags_12 },
    { 0x43548e1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b48e20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83948e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38748e26, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2548e2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2948e2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab48e2f, SUC_lexicon_tags_42 },
    { 0x88148e2f, SUC_lexicon_tags_15 },
    { 0x13948e31, SUC_lexicon_tags_6 },
    { 0xca148e2f, SUC_lexicon_tags_14 },
    { 0x85148e33, SUC_lexicon_tags_1352 },
    { 0, NULL },
    { 0x1c948e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39348e3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9548e3c, SUC_lexicon_tags_1 },
    { 0xa4d48e3d, SUC_lexicon_tags_37 },
    { 0x1f948e3d, SUC_lexicon_tags_11 },
    { 0xeaf48e3d, SUC_lexicon_tags_3 },
    { 0x6c748e3f, SUC_lexicon_tags_7 },
    { 0xa7948e3e, SUC_lexicon_tags_17 },
    { 0x20f48e42, SUC_lexicon_tags_1 },
    { 0x56148e3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6348e48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5148e4a, SUC_lexicon_tags_84 },
    { 0xc5b48e4a, SUC_lexicon_tags_43 },
    { 0xd1f48e4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8748e52, SUC_lexicon_tags_17 },
    { 0x548e53, SUC_lexicon_tags_29 },
    { 0x9df48e52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d48e58, SUC_lexicon_tags_25 },
    { 0x9b948e59, SUC_lexicon_tags_56 },
    { 0x77348e58, SUC_lexicon_tags_63 },
    { 0x7a548e5a, SUC_lexicon_tags_37 },
    { 0xd0148e5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b748e5f, SUC_lexicon_tags_15 },
    { 0xdb48e5f, SUC_lexicon_tags_66 },
    { 0xeff48e61, SUC_lexicon_tags_6 },
    { 0x748e60, SUC_lexicon_tags_48 },
    { 0x5b748e5f, SUC_lexicon_tags_37 },
    { 0x50348e64, SUC_lexicon_tags_43 },
    { 0x5a748e64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15348e69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa0948e6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2148e70, SUC_lexicon_tags_49 },
    { 0x9d348e70, SUC_lexicon_tags_26 },
    { 0xea348e70, SUC_lexicon_tags_12 },
    { 0x15948e72, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xdc148e75, SUC_lexicon_tags_30 },
    { 0x42f48e75, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x52d48e78, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6548e7c, SUC_lexicon_tags_15 },
    { 0x7bd48e7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd48e80, SUC_lexicon_tags_230 },
    { 0xc1948e80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4948e84, SUC_lexicon_tags_26 },
    { 0x80148e85, SUC_lexicon_tags_1 },
    { 0xa148e85, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f748e89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d48e8c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbb948e8e, SUC_lexicon_tags_15 },
    { 0x3bf48e8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47548e92, SUC_lexicon_tags_38 },
    { 0x2fb48e93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1548e97, SUC_lexicon_tags_33 },
    { 0x1b148e98, SUC_lexicon_tags_46 },
    { 0x50148e98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb48e9f, SUC_lexicon_tags_3 },
    { 0x2b48ea0, SUC_lexicon_tags_49 },
    { 0xb3d48ea0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38548ea5, SUC_lexicon_tags_2 },
    { 0x42148ea6, SUC_lexicon_tags_11 },
    { 0xeb48ea7, SUC_lexicon_tags_11 },
    { 0xc2f48ea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2b48eaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb748eac, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xd9748eae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f48eb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef48eb5, SUC_lexicon_tags_47 },
    { 0x8ad48eb6, SUC_lexicon_tags_14 },
    { 0x52948eb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8148eb9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd48ebc, SUC_lexicon_tags_17 },
    { 0xb0f48ebd, SUC_lexicon_tags_63 },
    { 0x11548ebc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed348ec2, SUC_lexicon_tags_158 },
    { 0x6ed48ec3, SUC_lexicon_tags_25 },
    { 0x35348ec4, SUC_lexicon_tags_37 },
    { 0x6e748ec4, SUC_lexicon_tags_13 },
    { 0x91b48ec6, SUC_lexicon_tags_12 },
    { 0x1f948ec4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b48eca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad48ecd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f48ed3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25748ed6, SUC_lexicon_tags_47 },
    { 0xfa348ed7, SUC_lexicon_tags_6 },
    { 0x53348ed8, SUC_lexicon_tags_49 },
    { 0x2548ed6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41948edb, SUC_lexicon_tags_21 },
    { 0x72b48edc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x94d48ede, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16948ee5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3f948ee7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4d48ee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7948ef3, SUC_lexicon_tags_23 },
    { 0x98b48ef4, SUC_lexicon_tags_30 },
    { 0xe7748ef5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f48efa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d48efd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33948f03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a948f05, SUC_lexicon_tags_37 },
    { 0x69948f06, SUC_lexicon_tags_33 },
    { 0x29348f07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50f48f09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed48f0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f48f11, SUC_lexicon_tags_57 },
    { 0x2eb48f12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2348f16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b148f1c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x55b48f1e, SUC_lexicon_tags_11 },
    { 0xa4948f1f, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xa9d48f21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b48f25, SUC_lexicon_tags_33 },
    { 0x7c548f25, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f48f29, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89548f2d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d48f30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d48f33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90f48f35, SUC_lexicon_tags_7 },
    { 0x10d48f36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30748f3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3748f3e, SUC_lexicon_tags_11 },
    { 0xf2748f3e, SUC_lexicon_tags_25 },
    { 0xf5b48f3f, SUC_lexicon_tags_11 },
    { 0xa948f3e, SUC_lexicon_tags_37 },
    { 0x94148f42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f48f46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d548f49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d48f4d, SUC_lexicon_tags_27 },
    { 0xd3d48f4e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f48f54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b48f5a, SUC_lexicon_tags_7 },
    { 0xcdd48f5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d48f63, SUC_lexicon_tags_29 },
    { 0x8ff48f64, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8948f68, SUC_lexicon_tags_47 },
    { 0x20748f69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b148f70, SUC_lexicon_tags_15 },
    { 0x7b348f70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79148f78, SUC_lexicon_tags_1 },
    { 0x9f48f78, SUC_lexicon_tags_8 },
    { 0x7f748f78, SUC_lexicon_tags_7 },
    { 0xb6748f7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d48f7f, SUC_lexicon_tags_1 },
    { 0xc9b48f80, SUC_lexicon_tags_12 },
    { 0xe6d48f80, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9348f85, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d48f89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda348f8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcf48f8d, SUC_lexicon_tags_112 },
    { 0x27d48f8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1748f91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d148f95, SUC_lexicon_tags_37 },
    { 0xa9148f95, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d48f99, SUC_lexicon_tags_17 },
    { 0x90748f9a, SUC_lexicon_tags_8 },
    { 0x75b48f9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3748f9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2548fa2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7bb48fa4, SUC_lexicon_tags_7 },
    { 0x98348fa5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b548faa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd48fad, SUC_lexicon_tags_194 },
    { 0x1c748fad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x79f48fb0, SUC_lexicon_tags_15 },
    { 0x4c948fb1, SUC_lexicon_tags_13 },
    { 0x80d48fb0, SUC_lexicon_tags_4 },
    { 0xe148fb1, SUC_lexicon_tags_2 },
    { 0xfd48fb4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x87f48fb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b748fc3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x86148fc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6948fc9, SUC_lexicon_tags_25 },
    { 0x85548fca, SUC_lexicon_tags_27 },
    { 0xa4f48fcb, SUC_lexicon_tags_61 },
    { 0xd548fca, SUC_lexicon_tags_0 },
    { 0x93d48fcb, SUC_lexicon_tags_25 },
    { 0x10548fce, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b48fda, SUC_lexicon_tags_30 },
    { 0x33748fda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cf48fdd, SUC_lexicon_tags_15 },
    { 0xb8748fdd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8548fe2, SUC_lexicon_tags_15 },
    { 0xb4948fe2, SUC_lexicon_tags_49 },
    { 0xa3d48fe4, SUC_lexicon_tags_37 },
    { 0x9548fe5, SUC_lexicon_tags_47 },
    { 0x2c148fe4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xba948fe8, SUC_lexicon_tags_55 },
    { 0xd7748fe8, SUC_lexicon_tags_30 },
    { 0x73f48fea, SUC_lexicon_tags_15 },
    { 0x31148feb, SUC_lexicon_tags_2 },
    { 0xeaf48fe8, SUC_lexicon_tags_2 },
    { 0x4d748fed, SUC_lexicon_tags_37 },
    { 0x6b148feb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16948ff5, SUC_lexicon_tags_37 },
    { 0x48548ff6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d48ffc, SUC_lexicon_tags_28 },
    { 0x40f48ffd, SUC_lexicon_tags_2 },
    { 0x7a148ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x30149001, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98d49003, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b49006, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff34900a, SUC_lexicon_tags_38 },
    { 0x554900b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d54900e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96349012, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a54901b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b74901d, SUC_lexicon_tags_7 },
    { 0xdf34901e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3749025, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1f49027, SUC_lexicon_tags_39 },
    { 0x2c949027, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b949035, SUC_lexicon_tags_21 },
    { 0xb4f49036, SUC_lexicon_tags_19 },
    { 0x1349037, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b49039, SUC_lexicon_tags_25 },
    { 0x3114903a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38d4903c, SUC_lexicon_tags_11 },
    { 0xf2f4903d, SUC_lexicon_tags_45 },
    { 0x33b4903e, SUC_lexicon_tags_21 },
    { 0xe594903d, SUC_lexicon_tags_27 },
    { 0x5db49040, SUC_lexicon_tags_37 },
    { 0xe7d4903d, SUC_lexicon_tags_12 },
    { 0x9d94903e, SUC_lexicon_tags_0 },
    { 0x99f49043, SUC_lexicon_tags_45 },
    { 0xe314903d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6549048, SUC_lexicon_tags_11 },
    { 0x70549049, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2774904c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2994904e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf49051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ef49053, SUC_lexicon_tags_7 },
    { 0x9f949054, SUC_lexicon_tags_39 },
    { 0xa1d49055, SUC_lexicon_tags_11 },
    { 0xd8549056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbd49058, SUC_lexicon_tags_29 },
    { 0xeb49059, SUC_lexicon_tags_17 },
    { 0xc594905a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b14905c, SUC_lexicon_tags_46 },
    { 0xa9d4905c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32349061, SUC_lexicon_tags_37 },
    { 0xfdd49061, SUC_lexicon_tags_43 },
    { 0x80749063, SUC_lexicon_tags_27 },
    { 0xa0749064, SUC_lexicon_tags_37 },
    { 0x3c549065, SUC_lexicon_tags_37 },
    { 0xac349064, SUC_lexicon_tags_6 },
    { 0x6f749065, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f749069, SUC_lexicon_tags_8 },
    { 0x7f4906a, SUC_lexicon_tags_47 },
    { 0xcd74906b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc14906d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45749071, SUC_lexicon_tags_11 },
    { 0x8f949072, SUC_lexicon_tags_12 },
    { 0xf1749073, SUC_lexicon_tags_37 },
    { 0xde149074, SUC_lexicon_tags_38 },
    { 0x1bf49073, SUC_lexicon_tags_11 },
    { 0x1cd49073, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73549079, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6054907b, SUC_lexicon_tags_50 },
    { 0x7014907b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f49080, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0549082, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a34908c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfcd4908e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd549094, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb949097, SUC_lexicon_tags_9 },
    { 0xd2549097, SUC_lexicon_tags_15 },
    { 0xf7149099, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7490a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b490a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf83490a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb15490a9, SUC_lexicon_tags_14 },
    { 0x41490a9, SUC_lexicon_tags_37 },
    { 0xdb7490ab, SUC_lexicon_tags_15 },
    { 0x13490a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37490af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9490b3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49490b8, SUC_lexicon_tags_8 },
    { 0xd7d490b8, SUC_lexicon_tags_7 },
    { 0xdad490ba, SUC_lexicon_tags_17 },
    { 0x84f490ba, SUC_lexicon_tags_38 },
    { 0x83b490bb, SUC_lexicon_tags_7 },
    { 0x8a3490bd, SUC_lexicon_tags_1 },
    { 0xcd490be, SUC_lexicon_tags_11 },
    { 0x775490bc, SUC_lexicon_tags_10 },
    { 0xd13490ba, SUC_lexicon_tags_27 },
    { 0x8b1490c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x393490c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab490c9, SUC_lexicon_tags_17 },
    { 0x81f490ca, SUC_lexicon_tags_6 },
    { 0x6b1490cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd7490cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x57490cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84b490d1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x113490d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1490da, SUC_lexicon_tags_8 },
    { 0x5d9490db, SUC_lexicon_tags_12 },
    { 0x587490da, SUC_lexicon_tags_17 },
    { 0xac3490dd, SUC_lexicon_tags_3 },
    { 0x1f3490da, SUC_lexicon_tags_23 },
    { 0x703490df, SUC_lexicon_tags_103 },
    { 0x3d7490dd, SUC_lexicon_tags_77 },
    { 0xea1490e1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1490e6, SUC_lexicon_tags_25 },
    { 0xb23490e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x173490e9, SUC_lexicon_tags_12 },
    { 0x29b490ea, SUC_lexicon_tags_17 },
    { 0xb6d490eb, SUC_lexicon_tags_116 },
    { 0x8c9490eb, SUC_lexicon_tags_15 },
    { 0x177490ea, SUC_lexicon_tags_49 },
    { 0xb65490ea, SUC_lexicon_tags_16 },
    { 0x39d490ef, SUC_lexicon_tags_11 },
    { 0xdf7490ef, SUC_lexicon_tags_9 },
    { 0x77b490ed, SUC_lexicon_tags_10 },
    { 0x81d490f2, SUC_lexicon_tags_15 },
    { 0x90d490ea, SUC_lexicon_tags_31 },
    { 0xc23490f4, SUC_lexicon_tags_12 },
    { 0x53490f5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3490f9, SUC_lexicon_tags_84 },
    { 0x691490fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d490fd, SUC_lexicon_tags_5 },
    { 0xfed490fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df49101, SUC_lexicon_tags_37 },
    { 0x2a149102, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f49106, SUC_lexicon_tags_37 },
    { 0x39f49107, SUC_lexicon_tags_1 },
    { 0xf3f49108, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb74910a, SUC_lexicon_tags_17 },
    { 0xb214910b, SUC_lexicon_tags_25 },
    { 0x6bd4910c, SUC_lexicon_tags_463 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab4910f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0749113, SUC_lexicon_tags_8 },
    { 0x32d49113, SUC_lexicon_tags_12 },
    { 0x76749114, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3b49117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc434911a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7d4911c, SUC_lexicon_tags_25 },
    { 0x54d4911d, SUC_lexicon_tags_47 },
    { 0x9614911e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d49124, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe8349126, SUC_lexicon_tags_12 },
    { 0xa549126, SUC_lexicon_tags_19 },
    { 0x27949128, SUC_lexicon_tags_2 },
    { 0xf0349128, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f4912e, SUC_lexicon_tags_10 },
    { 0xb374912f, SUC_lexicon_tags_3 },
    { 0x60349130, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c949134, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b49139, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa074913d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65b4913f, SUC_lexicon_tags_45 },
    { 0x43749140, SUC_lexicon_tags_19 },
    { 0x1a149140, SUC_lexicon_tags_15 },
    { 0x85949142, SUC_lexicon_tags_63 },
    { 0xc149143, SUC_lexicon_tags_23 },
    { 0x7bb49141, SUC_lexicon_tags_101 },
    { 0xf0f49145, SUC_lexicon_tags_15 },
    { 0x17749146, SUC_lexicon_tags_19 },
    { 0x32549147, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb49150, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4149153, SUC_lexicon_tags_74 },
    { 0xd9f49153, SUC_lexicon_tags_6 },
    { 0x9ad49155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2414915a, SUC_lexicon_tags_17 },
    { 0x6614915b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b4915e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b49163, SUC_lexicon_tags_25 },
    { 0xf3149164, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb534916f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4749172, SUC_lexicon_tags_11 },
    { 0xc9749173, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc549178, SUC_lexicon_tags_16 },
    { 0xb0149179, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2949183, SUC_lexicon_tags_11 },
    { 0xb5949184, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc149186, SUC_lexicon_tags_11 },
    { 0x55949186, SUC_lexicon_tags_1 },
    { 0xe2b49188, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5954918a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfab4918c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd494918e, SUC_lexicon_tags_11 },
    { 0xb9d4918f, SUC_lexicon_tags_17 },
    { 0xbb749190, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d49193, SUC_lexicon_tags_29 },
    { 0x85349194, SUC_lexicon_tags_11 },
    { 0x5c749195, SUC_lexicon_tags_27 },
    { 0x28149193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb154919c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf491a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49f491a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x707491a4, SUC_lexicon_tags_12 },
    { 0xcd491a5, SUC_lexicon_tags_234 },
    { 0x823491a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c5491a8, SUC_lexicon_tags_7 },
    { 0x7d1491a9, SUC_lexicon_tags_40 },
    { 0xa8d491a9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x995491ac, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01491af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9491b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9491b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf9491ba, SUC_lexicon_tags_64 },
    { 0x5af491ba, SUC_lexicon_tags_16 },
    { 0xa9f491bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5491c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9491c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d491c7, SUC_lexicon_tags_7 },
    { 0xc99491c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab491cc, SUC_lexicon_tags_53 },
    { 0x947491cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03491d1, SUC_lexicon_tags_17 },
    { 0x7a7491d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f491d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4d491d7, SUC_lexicon_tags_25 },
    { 0xda1491d8, SUC_lexicon_tags_7 },
    { 0xbdb491d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3491dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7491e4, SUC_lexicon_tags_1 },
    { 0xad9491e5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x49491e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8d491e9, SUC_lexicon_tags_25 },
    { 0x583491ea, SUC_lexicon_tags_10 },
    { 0xb49491eb, SUC_lexicon_tags_23 },
    { 0x6cf491ec, SUC_lexicon_tags_2 },
    { 0xaf3491eb, SUC_lexicon_tags_13 },
    { 0xdd7491ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f491f3, SUC_lexicon_tags_8 },
    { 0x1ff491f4, SUC_lexicon_tags_23 },
    { 0x3f1491f4, SUC_lexicon_tags_11 },
    { 0xa3b491f6, SUC_lexicon_tags_14 },
    { 0x9f1491f7, SUC_lexicon_tags_34 },
    { 0x99d491f8, SUC_lexicon_tags_1 },
    { 0xe41491f8, SUC_lexicon_tags_7 },
    { 0x743491fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f491fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x71749200, SUC_lexicon_tags_11 },
    { 0x68349200, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4bb49203, SUC_lexicon_tags_25 },
    { 0x29b49204, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead4920b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea54920d, SUC_lexicon_tags_2 },
    { 0x2234920d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb149212, SUC_lexicon_tags_19 },
    { 0x12149212, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7349216, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c749219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3cd4921b, SUC_lexicon_tags_15 },
    { 0x75f4921c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa149222, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d349226, SUC_lexicon_tags_51 },
    { 0x73549227, SUC_lexicon_tags_33 },
    { 0xa2f49228, SUC_lexicon_tags_14 },
    { 0xbd949226, SUC_lexicon_tags_45 },
    { 0xe0f4922a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a14922f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef49236, SUC_lexicon_tags_1206 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc434923a, SUC_lexicon_tags_9 },
    { 0xaed4923a, SUC_lexicon_tags_11 },
    { 0x4934923c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8914924a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49f4924c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ad4924e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2549254, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1334925a, SUC_lexicon_tags_65 },
    { 0x8454925b, SUC_lexicon_tags_2 },
    { 0x3b14925a, SUC_lexicon_tags_8 },
    { 0x1d14925d, SUC_lexicon_tags_1 },
    { 0x6b74925e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97349262, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2949266, SUC_lexicon_tags_24 },
    { 0x97d49267, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d4926a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f34926f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6f49271, SUC_lexicon_tags_17 },
    { 0x87749272, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8749276, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4927a, SUC_lexicon_tags_36 },
    { 0xdd34927b, SUC_lexicon_tags_14 },
    { 0xdf54927c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9254927f, SUC_lexicon_tags_15 },
    { 0xe2749280, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2549283, SUC_lexicon_tags_17 },
    { 0x78d49284, SUC_lexicon_tags_49 },
    { 0xcbd49284, SUC_lexicon_tags_43 },
    { 0xec349286, SUC_lexicon_tags_63 },
    { 0x98349287, SUC_lexicon_tags_25 },
    { 0xbc149288, SUC_lexicon_tags_7 },
    { 0x8bf49284, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b74928f, SUC_lexicon_tags_30 },
    { 0xf8f49290, SUC_lexicon_tags_1 },
    { 0x1d349291, SUC_lexicon_tags_0 },
    { 0x88149291, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82749297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d4929e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cb492a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cb492a2, SUC_lexicon_tags_11 },
    { 0x141492a3, SUC_lexicon_tags_16 },
    { 0x425492a4, SUC_lexicon_tags_12 },
    { 0x3ed492a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d492a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7492ab, SUC_lexicon_tags_39 },
    { 0x815492ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd492b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9492b6, SUC_lexicon_tags_17 },
    { 0xa3492b7, SUC_lexicon_tags_49 },
    { 0xdd9492b8, SUC_lexicon_tags_13 },
    { 0x37f492b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125492bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f5492bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25492c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341492c6, SUC_lexicon_tags_27 },
    { 0x9ef492c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43492cb, SUC_lexicon_tags_25 },
    { 0xd8d492cc, SUC_lexicon_tags_30 },
    { 0x325492cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51d492cf, SUC_lexicon_tags_37 },
    { 0x10b492cf, SUC_lexicon_tags_7 },
    { 0x409492d1, SUC_lexicon_tags_1 },
    { 0x95d492d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x96b492d4, SUC_lexicon_tags_1 },
    { 0xb47492d5, SUC_lexicon_tags_92 },
    { 0x3d1492d4, SUC_lexicon_tags_17 },
    { 0xce5492d7, SUC_lexicon_tags_17 },
    { 0x22f492d4, SUC_lexicon_tags_1 },
    { 0x9d9492d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef492de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45492e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x533492e4, SUC_lexicon_tags_27 },
    { 0x209492e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49492e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d492ec, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0xad9492ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a3492f0, SUC_lexicon_tags_7 },
    { 0x96b492f1, SUC_lexicon_tags_37 },
    { 0x9b7492f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x117492f5, SUC_lexicon_tags_17 },
    { 0xb09492f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf492f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe2d492fa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe59492fc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2d5492fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f49302, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b49305, SUC_lexicon_tags_9 },
    { 0xab949306, SUC_lexicon_tags_17 },
    { 0x95949307, SUC_lexicon_tags_17 },
    { 0xec349308, SUC_lexicon_tags_1 },
    { 0xa4749306, SUC_lexicon_tags_11 },
    { 0x93d49307, SUC_lexicon_tags_2 },
    { 0xabb4930b, SUC_lexicon_tags_63 },
    { 0x1f94930c, SUC_lexicon_tags_15 },
    { 0x3b34930d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d34930f, SUC_lexicon_tags_11 },
    { 0xdfb49310, SUC_lexicon_tags_15 },
    { 0x4b349310, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x47549314, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f49317, SUC_lexicon_tags_12 },
    { 0x22549318, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b49321, SUC_lexicon_tags_38 },
    { 0xb7549322, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a149325, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f34932b, SUC_lexicon_tags_37 },
    { 0x7cd4932c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc434932e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33749333, SUC_lexicon_tags_7 },
    { 0x6b949334, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb0349336, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc1f49338, SUC_lexicon_tags_30 },
    { 0x91d49339, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb4933d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa174933f, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d549342, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb8349344, SUC_lexicon_tags_8 },
    { 0xea549344, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f4934c, SUC_lexicon_tags_45 },
    { 0x9934934d, SUC_lexicon_tags_66 },
    { 0x3b74934d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x80b49350, SUC_lexicon_tags_33 },
    { 0xfab49351, SUC_lexicon_tags_29 },
    { 0x82d49352, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d49355, SUC_lexicon_tags_15 },
    { 0x32b49355, SUC_lexicon_tags_1 },
    { 0x8e149355, SUC_lexicon_tags_6 },
    { 0x1bd49355, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5814935c, SUC_lexicon_tags_17 },
    { 0x27d4935c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd49360, SUC_lexicon_tags_0 },
    { 0x35d49360, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6549365, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe614936d, SUC_lexicon_tags_17 },
    { 0x4f74936d, SUC_lexicon_tags_9 },
    { 0xb2b4936d, SUC_lexicon_tags_8 },
    { 0xccb4936d, SUC_lexicon_tags_1 },
    { 0xcbb49370, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb49379, SUC_lexicon_tags_27 },
    { 0x1514937a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa854937d, SUC_lexicon_tags_12 },
    { 0xac14937d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e149380, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec549386, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac74938a, SUC_lexicon_tags_63 },
    { 0x6a74938b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9094938d, SUC_lexicon_tags_37 },
    { 0x9bf4938d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c149396, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3254939b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd2d4939d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1a74939f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109493a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d493a7, SUC_lexicon_tags_21 },
    { 0xb39493a8, SUC_lexicon_tags_12 },
    { 0x915493a9, SUC_lexicon_tags_55 },
    { 0x321493a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x80f493ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20b493ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc07493b0, SUC_lexicon_tags_45 },
    { 0x6dd493b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x163493b4, SUC_lexicon_tags_0 },
    { 0x5bd493b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x427493b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25493ba, SUC_lexicon_tags_37 },
    { 0x593493bb, SUC_lexicon_tags_29 },
    { 0x7d5493bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01493c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ef493c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37493c8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x69493cb, SUC_lexicon_tags_13 },
    { 0x567493cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc27493ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b493da, SUC_lexicon_tags_17 },
    { 0x497493db, SUC_lexicon_tags_17 },
    { 0xdb3493db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd493df, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb493e7, SUC_lexicon_tags_10 },
    { 0x435493e7, SUC_lexicon_tags_7 },
    { 0x5b5493e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647493f0, SUC_lexicon_tags_49 },
    { 0x36d493f0, SUC_lexicon_tags_224 },
    { 0x133493f1, SUC_lexicon_tags_9 },
    { 0x77493f3, SUC_lexicon_tags_10 },
    { 0x6b5493f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9493f9, SUC_lexicon_tags_7 },
    { 0x313493f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d493fe, SUC_lexicon_tags_15 },
    { 0x65493fe, SUC_lexicon_tags_19 },
    { 0x52949400, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe149403, SUC_lexicon_tags_30 },
    { 0xe149404, SUC_lexicon_tags_2 },
    { 0x58949405, SUC_lexicon_tags_37 },
    { 0xd5949406, SUC_lexicon_tags_28 },
    { 0x83b49406, SUC_lexicon_tags_17 },
    { 0x59949406, SUC_lexicon_tags_141 },
    { 0x65149408, SUC_lexicon_tags_37 },
    { 0x14b4940a, SUC_lexicon_tags_2 },
    { 0x31f49409, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x8634940e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b49412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d49415, SUC_lexicon_tags_6 },
    { 0x4ed49415, SUC_lexicon_tags_1 },
    { 0xed549416, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x66b49419, SUC_lexicon_tags_0 },
    { 0x7fb4941a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ff4941c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe14941e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x1cd49420, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f49423, SUC_lexicon_tags_7 },
    { 0x4f949424, SUC_lexicon_tags_14 },
    { 0x61749424, SUC_lexicon_tags_33 },
    { 0xb949426, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb54942a, SUC_lexicon_tags_2 },
    { 0xdab4942a, SUC_lexicon_tags_25 },
    { 0x7274942c, SUC_lexicon_tags_25 },
    { 0xa134942c, SUC_lexicon_tags_31 },
    { 0x99b4942c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf49432, SUC_lexicon_tags_1 },
    { 0xc1749433, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4a749435, SUC_lexicon_tags_19 },
    { 0x3149436, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x45149438, SUC_lexicon_tags_16 },
    { 0x45949439, SUC_lexicon_tags_15 },
    { 0x27f4943a, SUC_lexicon_tags_37 },
    { 0x8534943b, SUC_lexicon_tags_33 },
    { 0x5f14943a, SUC_lexicon_tags_12 },
    { 0x3d54943d, SUC_lexicon_tags_25 },
    { 0xeaf4943c, SUC_lexicon_tags_15 },
    { 0xe3b4943a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27349443, SUC_lexicon_tags_0 },
    { 0x39549444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b49447, SUC_lexicon_tags_11 },
    { 0xa2349448, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6f4944a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7554944d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b34944f, SUC_lexicon_tags_38 },
    { 0x8ad49450, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa749454, SUC_lexicon_tags_14 },
    { 0xb9949454, SUC_lexicon_tags_56 },
    { 0xfc349456, SUC_lexicon_tags_25 },
    { 0x58549455, SUC_lexicon_tags_4 },
    { 0x4c149457, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xdd14945a, SUC_lexicon_tags_7 },
    { 0x64f4945b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f4945e, SUC_lexicon_tags_69 },
    { 0x4114945e, SUC_lexicon_tags_65 },
    { 0x7854945f, SUC_lexicon_tags_6 },
    { 0x9074945f, SUC_lexicon_tags_15 },
    { 0xfd49462, SUC_lexicon_tags_68 },
    { 0xbc149462, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b149465, SUC_lexicon_tags_17 },
    { 0xe1f49465, SUC_lexicon_tags_12 },
    { 0xb0f49467, SUC_lexicon_tags_17 },
    { 0xf1349468, SUC_lexicon_tags_16 },
    { 0xfdd49467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd414946b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1454946d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd49473, SUC_lexicon_tags_25 },
    { 0x43149474, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf949476, SUC_lexicon_tags_15 },
    { 0x9f349477, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe74947e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7ef49480, SUC_lexicon_tags_6 },
    { 0x1b749481, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19349485, SUC_lexicon_tags_2 },
    { 0xa1749486, SUC_lexicon_tags_37 },
    { 0xc5749487, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe454948a, SUC_lexicon_tags_12 },
    { 0x3d54948b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe934948d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x10d4948f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x60f49491, SUC_lexicon_tags_1 },
    { 0x81b49491, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8d49494, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad49498, SUC_lexicon_tags_25 },
    { 0xd3549498, SUC_lexicon_tags_45 },
    { 0xc614949a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3554949c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb494a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3b494a3, SUC_lexicon_tags_17 },
    { 0x9a9494a3, SUC_lexicon_tags_0 },
    { 0xbe5494a4, SUC_lexicon_tags_106 },
    { 0xa53494a6, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x287494a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d494ae, SUC_lexicon_tags_11 },
    { 0x351494af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7494b2, SUC_lexicon_tags_37 },
    { 0x76f494b3, SUC_lexicon_tags_33 },
    { 0x607494b3, SUC_lexicon_tags_79 },
    { 0xe5f494b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41494b9, SUC_lexicon_tags_37 },
    { 0xad1494ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a7494bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87494c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73b494c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573494cb, SUC_lexicon_tags_7 },
    { 0x309494cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1494d2, SUC_lexicon_tags_11 },
    { 0xf49494d2, SUC_lexicon_tags_35 },
    { 0x40b494d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f494db, SUC_lexicon_tags_12 },
    { 0x57494dc, SUC_lexicon_tags_61 },
    { 0xf75494dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfaf494df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8f494e1, SUC_lexicon_tags_9 },
    { 0x541494e2, SUC_lexicon_tags_1 },
    { 0x9fb494e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb494f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd494f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc749503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f349509, SUC_lexicon_tags_17 },
    { 0xdb149509, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6314950e, SUC_lexicon_tags_16 },
    { 0x5354950f, SUC_lexicon_tags_60 },
    { 0xdf149510, SUC_lexicon_tags_37 },
    { 0x22149511, SUC_lexicon_tags_87 },
    { 0xf9349511, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d49516, SUC_lexicon_tags_39 },
    { 0x87549517, SUC_lexicon_tags_25 },
    { 0xb1149517, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf74951c, SUC_lexicon_tags_45 },
    { 0xdb4951c, SUC_lexicon_tags_15 },
    { 0xb6f4951c, SUC_lexicon_tags_12 },
    { 0x2bf4951d, SUC_lexicon_tags_17 },
    { 0x42749520, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f949524, SUC_lexicon_tags_27 },
    { 0x50d49525, SUC_lexicon_tags_21 },
    { 0x6a549526, SUC_lexicon_tags_2 },
    { 0x92b49526, SUC_lexicon_tags_28 },
    { 0x5ef49526, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e34952a, SUC_lexicon_tags_12 },
    { 0xe694952a, SUC_lexicon_tags_3 },
    { 0xa5d4952c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe054952e, SUC_lexicon_tags_2 },
    { 0xe9f4952f, SUC_lexicon_tags_25 },
    { 0x43749530, SUC_lexicon_tags_27 },
    { 0x23749531, SUC_lexicon_tags_14 },
    { 0x42d49532, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9549536, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db4953a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1349544, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x16b49546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1949548, SUC_lexicon_tags_45 },
    { 0x25f49549, SUC_lexicon_tags_82 },
    { 0x6b349548, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3814954f, SUC_lexicon_tags_5 },
    { 0xb234954f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97149558, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa874955c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89349560, SUC_lexicon_tags_0 },
    { 0x51d49560, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85349564, SUC_lexicon_tags_12 },
    { 0x5bd49565, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc54956d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x14b4956f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b149572, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b349577, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa094957c, SUC_lexicon_tags_56 },
    { 0xd554957c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x41149580, SUC_lexicon_tags_17 },
    { 0x40b49581, SUC_lexicon_tags_10 },
    { 0x39549581, SUC_lexicon_tags_12 },
    { 0xb7d49581, SUC_lexicon_tags_12 },
    { 0x28f49581, SUC_lexicon_tags_1 },
    { 0xf6949585, SUC_lexicon_tags_10 },
    { 0xe0549585, SUC_lexicon_tags_15 },
    { 0x4dd49587, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b4958b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1d4958d, SUC_lexicon_tags_2 },
    { 0x254958e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d949592, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x60549595, SUC_lexicon_tags_13 },
    { 0x8f349595, SUC_lexicon_tags_1 },
    { 0xc1749597, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8214959a, SUC_lexicon_tags_12 },
    { 0xac54959b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11495a1, SUC_lexicon_tags_17 },
    { 0x9b1495a2, SUC_lexicon_tags_43 },
    { 0x269495a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x201495a5, SUC_lexicon_tags_11 },
    { 0x687495a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f495ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987495b0, SUC_lexicon_tags_47 },
    { 0x799495b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf7495b3, SUC_lexicon_tags_45 },
    { 0x7bf495b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b495b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1495bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd495c5, SUC_lexicon_tags_33 },
    { 0xc03495c6, SUC_lexicon_tags_1 },
    { 0x9d9495c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4f3495c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9495d1, SUC_lexicon_tags_38 },
    { 0x7a1495d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55495d8, SUC_lexicon_tags_86 },
    { 0xc77495d9, SUC_lexicon_tags_14 },
    { 0x905495d9, SUC_lexicon_tags_9 },
    { 0x307495d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7fd495dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9495e0, SUC_lexicon_tags_37 },
    { 0xcc7495e0, SUC_lexicon_tags_25 },
    { 0x33495e2, SUC_lexicon_tags_37 },
    { 0xa01495e1, SUC_lexicon_tags_37 },
    { 0xd6f495e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd495e7, SUC_lexicon_tags_14 },
    { 0xf29495e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f495ec, SUC_lexicon_tags_25 },
    { 0x681495ec, SUC_lexicon_tags_6 },
    { 0xcf5495ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88b495f0, SUC_lexicon_tags_25 },
    { 0x20d495f1, SUC_lexicon_tags_5 },
    { 0x7c3495f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f495f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1495fa, SUC_lexicon_tags_7 },
    { 0x7f5495fb, SUC_lexicon_tags_15 },
    { 0xd87495fa, SUC_lexicon_tags_65 },
    { 0x8b1495fa, SUC_lexicon_tags_11 },
    { 0x127495fe, SUC_lexicon_tags_37 },
    { 0x7e7495ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25549602, SUC_lexicon_tags_11 },
    { 0x9db49603, SUC_lexicon_tags_29 },
    { 0xcf749602, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f949608, SUC_lexicon_tags_37 },
    { 0x11749609, SUC_lexicon_tags_12 },
    { 0xdd149609, SUC_lexicon_tags_12 },
    { 0xc074960b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b49612, SUC_lexicon_tags_7 },
    { 0x5749612, SUC_lexicon_tags_17 },
    { 0xf49613, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9349617, SUC_lexicon_tags_15 },
    { 0x11549617, SUC_lexicon_tags_37 },
    { 0x28b49617, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4974961b, SUC_lexicon_tags_66 },
    { 0xadf4961c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad149620, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0349626, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4962a, SUC_lexicon_tags_1 },
    { 0x9714962a, SUC_lexicon_tags_65 },
    { 0x90b4962c, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x1034962f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd9549631, SUC_lexicon_tags_8 },
    { 0xf3549632, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x36549634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28349637, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29349640, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f349645, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9034964a, SUC_lexicon_tags_17 },
    { 0x4b4964b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef74964f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f49655, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab49658, SUC_lexicon_tags_7 },
    { 0x71b49658, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5694965d, SUC_lexicon_tags_17 },
    { 0xb0b4965e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d949660, SUC_lexicon_tags_17 },
    { 0x1b49661, SUC_lexicon_tags_60 },
    { 0xb0549662, SUC_lexicon_tags_7 },
    { 0x34949663, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc749667, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f14966a, SUC_lexicon_tags_43 },
    { 0x1bf4966b, SUC_lexicon_tags_12 },
    { 0xf4b4966c, SUC_lexicon_tags_85 },
    { 0x63d4966d, SUC_lexicon_tags_30 },
    { 0x8b4966c, SUC_lexicon_tags_29 },
    { 0x9d54966d, SUC_lexicon_tags_23 },
    { 0xef34966b, SUC_lexicon_tags_12 },
    { 0x91749670, SUC_lexicon_tags_23 },
    { 0xb5949672, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5749677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2334967c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa434967f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46949682, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4968a, SUC_lexicon_tags_2 },
    { 0xbdb4968b, SUC_lexicon_tags_23 },
    { 0x9314968c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76949691, SUC_lexicon_tags_23 },
    { 0x23f49691, SUC_lexicon_tags_40 },
    { 0x84349692, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xb4f49695, SUC_lexicon_tags_12 },
    { 0x67b49696, SUC_lexicon_tags_49 },
    { 0x60d49697, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d54969a, SUC_lexicon_tags_32 },
    { 0x4234969b, SUC_lexicon_tags_19 },
    { 0xcd34969b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xe454969f, SUC_lexicon_tags_49 },
    { 0x3b9496a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967496a4, SUC_lexicon_tags_437 },
    { 0x537496a5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71496ae, SUC_lexicon_tags_63 },
    { 0xe79496ae, SUC_lexicon_tags_19 },
    { 0xfad496b0, SUC_lexicon_tags_39 },
    { 0xd71496b1, SUC_lexicon_tags_25 },
    { 0x79f496af, SUC_lexicon_tags_81 },
    { 0x915496b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x749496b5, SUC_lexicon_tags_37 },
    { 0x9cb496b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x267496b8, SUC_lexicon_tags_63 },
    { 0x51b496b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7496bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd496c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1496c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d496c9, SUC_lexicon_tags_25 },
    { 0x571496ca, SUC_lexicon_tags_26 },
    { 0x9bf496c9, SUC_lexicon_tags_14 },
    { 0x9bb496cc, SUC_lexicon_tags_2 },
    { 0x22b496cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf496d3, SUC_lexicon_tags_47 },
    { 0x1df496d3, SUC_lexicon_tags_15 },
    { 0xa43496d5, SUC_lexicon_tags_30 },
    { 0xad9496d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd3496d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d496de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c3496e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf09496e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b496e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b496e8, SUC_lexicon_tags_63 },
    { 0xacf496e9, SUC_lexicon_tags_26 },
    { 0x833496e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1496ed, SUC_lexicon_tags_30 },
    { 0x14b496ed, SUC_lexicon_tags_11 },
    { 0xdad496ed, SUC_lexicon_tags_45 },
    { 0xcf9496f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x353496f2, SUC_lexicon_tags_1 },
    { 0x535496f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa5496f5, SUC_lexicon_tags_19 },
    { 0x283496f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1496fa, SUC_lexicon_tags_12 },
    { 0x269496fa, SUC_lexicon_tags_3 },
    { 0xb65496fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6d496fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd49705, SUC_lexicon_tags_14 },
    { 0xda949706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83b49708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7894970a, SUC_lexicon_tags_25 },
    { 0x2414970b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1d54970d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83349711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fb49713, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe8749715, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x71549717, SUC_lexicon_tags_66 },
    { 0x41d49717, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f74971d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x86d4971f, SUC_lexicon_tags_0 },
    { 0x99f49720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb549722, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad49725, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd49728, SUC_lexicon_tags_11 },
    { 0xb5b49728, SUC_lexicon_tags_4 },
    { 0xadd49728, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d4972f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f349735, SUC_lexicon_tags_12 },
    { 0xb6f49736, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf4973d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f49745, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f4974d, SUC_lexicon_tags_14 },
    { 0x7ef4974d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64749750, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x36349753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d49757, SUC_lexicon_tags_21 },
    { 0x3e349758, SUC_lexicon_tags_1 },
    { 0xf8349759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa174975b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8194975e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d49766, SUC_lexicon_tags_33 },
    { 0x71149767, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd94976b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b4976e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xecf49770, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3af49772, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3949775, SUC_lexicon_tags_11 },
    { 0xb6949775, SUC_lexicon_tags_26 },
    { 0x45349775, SUC_lexicon_tags_15 },
    { 0x26b49778, SUC_lexicon_tags_11 },
    { 0x549779, SUC_lexicon_tags_12 },
    { 0xe9b4977a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e54977f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3549782, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b49786, SUC_lexicon_tags_62 },
    { 0xf2349787, SUC_lexicon_tags_65 },
    { 0xb6f49786, SUC_lexicon_tags_15 },
    { 0x48b49789, SUC_lexicon_tags_25 },
    { 0x2894978a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81949791, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b949797, SUC_lexicon_tags_26 },
    { 0x2fd49797, SUC_lexicon_tags_98 },
    { 0xe2b49799, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd4979c, SUC_lexicon_tags_7 },
    { 0x2dd4979c, SUC_lexicon_tags_6 },
    { 0xa34979e, SUC_lexicon_tags_29 },
    { 0x7d54979f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b497a2, SUC_lexicon_tags_31 },
    { 0xfef497a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59497a6, SUC_lexicon_tags_7 },
    { 0x5eb497a7, SUC_lexicon_tags_11 },
    { 0xae1497a8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd59497aa, SUC_lexicon_tags_11 },
    { 0x6a5497aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56d497ad, SUC_lexicon_tags_33 },
    { 0xf21497ae, SUC_lexicon_tags_11 },
    { 0xa11497ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87497ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265497bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39497c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3497cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x385497cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43d497cf, SUC_lexicon_tags_66 },
    { 0x665497d0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315497d4, SUC_lexicon_tags_11 },
    { 0x4d497d5, SUC_lexicon_tags_4 },
    { 0xca7497d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b497d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7fb497db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d497de, SUC_lexicon_tags_27 },
    { 0x48f497df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1497e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5f497e5, SUC_lexicon_tags_63 },
    { 0x577497e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d497ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db497ed, SUC_lexicon_tags_66 },
    { 0x1c7497ee, SUC_lexicon_tags_1 },
    { 0x5a7497ef, SUC_lexicon_tags_26 },
    { 0x9df497ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb497f4, SUC_lexicon_tags_14 },
    { 0x3f1497f5, SUC_lexicon_tags_55 },
    { 0xbf1497f6, SUC_lexicon_tags_890 },
    { 0, NULL },
    { 0, NULL },
    { 0x56f497f9, SUC_lexicon_tags_33 },
    { 0xa89497fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25d497fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf49497fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b49801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e949807, SUC_lexicon_tags_11 },
    { 0x76d49807, SUC_lexicon_tags_8 },
    { 0xc549809, SUC_lexicon_tags_124 },
    { 0x7f4980a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8a14980c, SUC_lexicon_tags_15 },
    { 0x44b4980c, SUC_lexicon_tags_7 },
    { 0xe474980e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc549814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f49819, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x15f4981b, SUC_lexicon_tags_35 },
    { 0xd3f4981c, SUC_lexicon_tags_11 },
    { 0x5cb4981c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d349821, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2149827, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f4982f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0949833, SUC_lexicon_tags_27 },
    { 0x37149834, SUC_lexicon_tags_17 },
    { 0x5a149834, SUC_lexicon_tags_9 },
    { 0x9349834, SUC_lexicon_tags_15 },
    { 0xc7749837, SUC_lexicon_tags_31 },
    { 0x75f49838, SUC_lexicon_tags_17 },
    { 0x36349837, SUC_lexicon_tags_17 },
    { 0xae74983a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x56f4983c, SUC_lexicon_tags_25 },
    { 0xef14983d, SUC_lexicon_tags_19 },
    { 0xff54983e, SUC_lexicon_tags_68 },
    { 0x114983d, SUC_lexicon_tags_43 },
    { 0x7bd49840, SUC_lexicon_tags_5 },
    { 0x81b49841, SUC_lexicon_tags_47 },
    { 0x8df49842, SUC_lexicon_tags_25 },
    { 0xa0d49843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8949846, SUC_lexicon_tags_17 },
    { 0xdb949847, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b4984b, SUC_lexicon_tags_28 },
    { 0x1ab4984b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1054984f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b49854, SUC_lexicon_tags_12 },
    { 0x89549854, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46149859, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d4985d, SUC_lexicon_tags_15 },
    { 0x6d4985e, SUC_lexicon_tags_30 },
    { 0xda94985e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f949864, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8334986e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a549877, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b4987a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b49880, SUC_lexicon_tags_60 },
    { 0x91b49881, SUC_lexicon_tags_27 },
    { 0x11f49881, SUC_lexicon_tags_71 },
    { 0xdcd49882, SUC_lexicon_tags_11 },
    { 0xf5149884, SUC_lexicon_tags_11 },
    { 0xb1349881, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af49889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a94988f, SUC_lexicon_tags_28 },
    { 0x71749890, SUC_lexicon_tags_25 },
    { 0xf1f49891, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d49896, SUC_lexicon_tags_7 },
    { 0x96b49897, SUC_lexicon_tags_82 },
    { 0xe9949898, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe254989a, SUC_lexicon_tags_27 },
    { 0xef54989b, SUC_lexicon_tags_7 },
    { 0xb494989a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe14989e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb13498a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x30b498a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f498a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b498b3, SUC_lexicon_tags_32 },
    { 0x5df498b4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5498b8, SUC_lexicon_tags_39 },
    { 0x869498b8, SUC_lexicon_tags_39 },
    { 0xce9498ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff498bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf9498bf, SUC_lexicon_tags_7 },
    { 0xce9498c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9498c5, SUC_lexicon_tags_11 },
    { 0xe55498c6, SUC_lexicon_tags_45 },
    { 0xc3d498c7, SUC_lexicon_tags_56 },
    { 0xf7498c6, SUC_lexicon_tags_8 },
    { 0xe01498c8, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xe05498cb, SUC_lexicon_tags_8 },
    { 0xf87498cb, SUC_lexicon_tags_21 },
    { 0x97f498cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff498d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x983498d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a5498d5, SUC_lexicon_tags_15 },
    { 0xecb498d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65498d9, SUC_lexicon_tags_14 },
    { 0xaf498d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfff498dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfc9498de, SUC_lexicon_tags_11 },
    { 0x2b498df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3498e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x901498e5, SUC_lexicon_tags_12 },
    { 0x76f498e6, SUC_lexicon_tags_2 },
    { 0x585498e6, SUC_lexicon_tags_61 },
    { 0xb03498e8, SUC_lexicon_tags_15 },
    { 0x711498e9, SUC_lexicon_tags_33 },
    { 0x1b498e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33498ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af498f2, SUC_lexicon_tags_118 },
    { 0xab1498f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d498fd, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf49902, SUC_lexicon_tags_15 },
    { 0x46749902, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9b349905, SUC_lexicon_tags_1 },
    { 0x55f49906, SUC_lexicon_tags_14 },
    { 0xa2949907, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b14990a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab74990e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80949911, SUC_lexicon_tags_7 },
    { 0xb0d49912, SUC_lexicon_tags_12 },
    { 0x2d49913, SUC_lexicon_tags_38 },
    { 0x95749914, SUC_lexicon_tags_9 },
    { 0xce549913, SUC_lexicon_tags_7 },
    { 0xec149916, SUC_lexicon_tags_30 },
    { 0x39f49917, SUC_lexicon_tags_12 },
    { 0x74949911, SUC_lexicon_tags_11 },
    { 0xfa349919, SUC_lexicon_tags_23 },
    { 0xf974991a, SUC_lexicon_tags_23 },
    { 0x6f34991b, SUC_lexicon_tags_47 },
    { 0x1cd4991c, SUC_lexicon_tags_2 },
    { 0x6f14991c, SUC_lexicon_tags_55 },
    { 0x4db4991c, SUC_lexicon_tags_38 },
    { 0xd054991e, SUC_lexicon_tags_2 },
    { 0x8a549920, SUC_lexicon_tags_34 },
    { 0xfa149921, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x67d49923, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc3749925, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d149928, SUC_lexicon_tags_63 },
    { 0x86349929, SUC_lexicon_tags_12 },
    { 0xc0549928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56749930, SUC_lexicon_tags_27 },
    { 0xdfb49931, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f549935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94f49937, SUC_lexicon_tags_7 },
    { 0xcfb49937, SUC_lexicon_tags_12 },
    { 0x24349938, SUC_lexicon_tags_30 },
    { 0xcad49938, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x34b4993c, SUC_lexicon_tags_27 },
    { 0x2dd4993d, SUC_lexicon_tags_47 },
    { 0xb3f4993c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd49947, SUC_lexicon_tags_9 },
    { 0x5cf49947, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8814994a, SUC_lexicon_tags_31 },
    { 0x4c74994b, SUC_lexicon_tags_23 },
    { 0x28b4994c, SUC_lexicon_tags_37 },
    { 0x60d4994d, SUC_lexicon_tags_11 },
    { 0xab74994b, SUC_lexicon_tags_54 },
    { 0xd5f4994b, SUC_lexicon_tags_39 },
    { 0x5d949950, SUC_lexicon_tags_11 },
    { 0xa6349951, SUC_lexicon_tags_15 },
    { 0xcbf4994e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6349954, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf549956, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f4995b, SUC_lexicon_tags_28 },
    { 0x7c14995b, SUC_lexicon_tags_37 },
    { 0x8c94995b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd749960, SUC_lexicon_tags_25 },
    { 0x92f49961, SUC_lexicon_tags_1 },
    { 0x51549961, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af49966, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab4996a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2149972, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc654997e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d49984, SUC_lexicon_tags_53 },
    { 0x50549985, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd4998b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f549990, SUC_lexicon_tags_17 },
    { 0x88349991, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd49994, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x70149997, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cb49999, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda14999b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997499a1, SUC_lexicon_tags_17 },
    { 0x5bd499a2, SUC_lexicon_tags_20 },
    { 0xbf9499a3, SUC_lexicon_tags_17 },
    { 0x21499a2, SUC_lexicon_tags_17 },
    { 0x5b499a5, SUC_lexicon_tags_31 },
    { 0xdb1499a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b499ab, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2b499ad, SUC_lexicon_tags_17 },
    { 0xfbf499ad, SUC_lexicon_tags_14 },
    { 0x48d499af, SUC_lexicon_tags_1 },
    { 0x5499af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95499b5, SUC_lexicon_tags_43 },
    { 0x303499b6, SUC_lexicon_tags_57 },
    { 0xfb499b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf499bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1499c0, SUC_lexicon_tags_25 },
    { 0xbe9499c1, SUC_lexicon_tags_37 },
    { 0x879499c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57499c7, SUC_lexicon_tags_4 },
    { 0xdcd499c8, SUC_lexicon_tags_23 },
    { 0x33b499c8, SUC_lexicon_tags_45 },
    { 0x92b499c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95499cd, SUC_lexicon_tags_151 },
    { 0xdf5499cd, SUC_lexicon_tags_49 },
    { 0xbc5499ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f499dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d499e2, SUC_lexicon_tags_7 },
    { 0xc55499e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd499ea, SUC_lexicon_tags_1 },
    { 0x3a5499eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad3499ed, SUC_lexicon_tags_37 },
    { 0xf0f499ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4b499f0, SUC_lexicon_tags_37 },
    { 0xb5d499f0, SUC_lexicon_tags_25 },
    { 0x8f499f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85499f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d499fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x843499fe, SUC_lexicon_tags_23 },
    { 0x279499ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf149a06, SUC_lexicon_tags_15 },
    { 0xebb49a07, SUC_lexicon_tags_26 },
    { 0x98149a07, SUC_lexicon_tags_2 },
    { 0x49149a09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a949a18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22349a1e, SUC_lexicon_tags_230 },
    { 0x83d49a1e, SUC_lexicon_tags_60 },
    { 0x76f49a20, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac349a25, SUC_lexicon_tags_7 },
    { 0x97549a26, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5f949a28, SUC_lexicon_tags_29 },
    { 0xa749a29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29349a31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc949a34, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db49a37, SUC_lexicon_tags_17 },
    { 0x89749a37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86549a3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83149a3e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d49a42, SUC_lexicon_tags_25 },
    { 0x75349a43, SUC_lexicon_tags_25 },
    { 0x16349a44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad49a51, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db49a54, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6349a5a, SUC_lexicon_tags_7 },
    { 0xcfd49a5a, SUC_lexicon_tags_2 },
    { 0x59d49a5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d49a63, SUC_lexicon_tags_43 },
    { 0xa3949a63, SUC_lexicon_tags_37 },
    { 0x36749a65, SUC_lexicon_tags_12 },
    { 0x29549a66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab49a6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x89b49a6c, SUC_lexicon_tags_7 },
    { 0xac149a6d, SUC_lexicon_tags_12 },
    { 0x81f49a6c, SUC_lexicon_tags_45 },
    { 0x94f49a6f, SUC_lexicon_tags_12 },
    { 0xbf549a70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x77749a73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb49a76, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa9749a78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa1149a7a, SUC_lexicon_tags_29 },
    { 0x9ff49a7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b349a7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf49a86, SUC_lexicon_tags_2 },
    { 0xa3549a86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53549a8b, SUC_lexicon_tags_27 },
    { 0x21f49a8c, SUC_lexicon_tags_13 },
    { 0xd6349a8d, SUC_lexicon_tags_31 },
    { 0x10b49a8d, SUC_lexicon_tags_25 },
    { 0xcf949a8f, SUC_lexicon_tags_11 },
    { 0x8f749a90, SUC_lexicon_tags_15 },
    { 0x17349a91, SUC_lexicon_tags_10 },
    { 0x75949a91, SUC_lexicon_tags_8 },
    { 0x90b49a91, SUC_lexicon_tags_11 },
    { 0x3b549a94, SUC_lexicon_tags_10 },
    { 0x29949a95, SUC_lexicon_tags_14 },
    { 0x10549a93, SUC_lexicon_tags_1 },
    { 0xfb349a8b, SUC_lexicon_tags_11 },
    { 0x2749a98, SUC_lexicon_tags_63 },
    { 0x3ab49a98, SUC_lexicon_tags_56 },
    { 0xbd349a8e, SUC_lexicon_tags_37 },
    { 0xf8949a92, SUC_lexicon_tags_31 },
    { 0x25d49a96, SUC_lexicon_tags_15 },
    { 0x29949a99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87549a9f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x29b49aa1, SUC_lexicon_tags_0 },
    { 0x6cf49aa1, SUC_lexicon_tags_61 },
    { 0xd8749aa3, SUC_lexicon_tags_17 },
    { 0x93149aa2, SUC_lexicon_tags_65 },
    { 0xda149aa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1949aa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9349aab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e149aaf, SUC_lexicon_tags_37 },
    { 0x1cf49ab0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda349ab4, SUC_lexicon_tags_46 },
    { 0x19349ab4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a149ab7, SUC_lexicon_tags_27 },
    { 0x90149ab7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6349ac1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f49ac4, SUC_lexicon_tags_14 },
    { 0x9b749ac5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x72f49ac7, SUC_lexicon_tags_10 },
    { 0x43f49ac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b49ad2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9149ad6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78149adb, SUC_lexicon_tags_25 },
    { 0x9d49adb, SUC_lexicon_tags_7 },
    { 0xcdb49add, SUC_lexicon_tags_7 },
    { 0x80f49add, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11549ae0, SUC_lexicon_tags_7 },
    { 0xaf349ae1, SUC_lexicon_tags_18 },
    { 0xae349ae2, SUC_lexicon_tags_1 },
    { 0xc2b49ae3, SUC_lexicon_tags_7 },
    { 0xc3f49ae4, SUC_lexicon_tags_45 },
    { 0xf7b49ae5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x87b49ae7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e749aee, SUC_lexicon_tags_9 },
    { 0xe1749aee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7bf49af1, SUC_lexicon_tags_11 },
    { 0x36d49af2, SUC_lexicon_tags_47 },
    { 0xc1149af3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96149af8, SUC_lexicon_tags_19 },
    { 0x66f49af8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82549afc, SUC_lexicon_tags_23 },
    { 0x16149afd, SUC_lexicon_tags_57 },
    { 0x2d49afc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0949b06, SUC_lexicon_tags_87 },
    { 0x6ef49b06, SUC_lexicon_tags_17 },
    { 0x3dd49b08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f49b0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd49b10, SUC_lexicon_tags_55 },
    { 0xb9749b10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e149b15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa949b18, SUC_lexicon_tags_10 },
    { 0xce349b19, SUC_lexicon_tags_63 },
    { 0xa2549b19, SUC_lexicon_tags_23 },
    { 0xebf49b1b, SUC_lexicon_tags_11 },
    { 0x97f49b1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47f49b1e, SUC_lexicon_tags_9 },
    { 0x13549b1e, SUC_lexicon_tags_1 },
    { 0x49b49b20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55749b24, SUC_lexicon_tags_26 },
    { 0xb9349b25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa149b29, SUC_lexicon_tags_17 },
    { 0x5bd49b2a, SUC_lexicon_tags_17 },
    { 0x21549b2b, SUC_lexicon_tags_84 },
    { 0xb5149b29, SUC_lexicon_tags_37 },
    { 0x25d49b2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73b49b2f, SUC_lexicon_tags_63 },
    { 0x99749b30, SUC_lexicon_tags_23 },
    { 0x36749b31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d49b34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78549b36, SUC_lexicon_tags_99 },
    { 0x8b749b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcef49b39, SUC_lexicon_tags_68 },
    { 0x45b49b39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4349b3d, SUC_lexicon_tags_61 },
    { 0xef149b3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc949b42, SUC_lexicon_tags_6 },
    { 0x77749b42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d49b47, SUC_lexicon_tags_8 },
    { 0x749b47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61149b4b, SUC_lexicon_tags_2 },
    { 0xac749b4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35549b4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x63f49b51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c549b54, SUC_lexicon_tags_49 },
    { 0x6c949b54, SUC_lexicon_tags_19 },
    { 0xfdd49b54, SUC_lexicon_tags_6 },
    { 0xb3d49b54, SUC_lexicon_tags_12 },
    { 0xa7149b58, SUC_lexicon_tags_39 },
    { 0x31149b59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8549b60, SUC_lexicon_tags_7 },
    { 0x86949b61, SUC_lexicon_tags_2 },
    { 0xa1549b61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d749b64, SUC_lexicon_tags_1 },
    { 0x42749b64, SUC_lexicon_tags_84 },
    { 0xf1b49b64, SUC_lexicon_tags_136 },
    { 0x2dd49b65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b49b6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a949b73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93d49b75, SUC_lexicon_tags_43 },
    { 0x64749b76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf749b78, SUC_lexicon_tags_11 },
    { 0xf9949b78, SUC_lexicon_tags_25 },
    { 0x42749b7a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xc549b7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77b49b7f, SUC_lexicon_tags_14 },
    { 0x4eb49b80, SUC_lexicon_tags_38 },
    { 0x64749b80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb549b85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50749b8a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa149b8e, SUC_lexicon_tags_17 },
    { 0x67149b8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc549b92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a749b96, SUC_lexicon_tags_0 },
    { 0xfd349b96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1349b9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d49b9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9349ba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76949ba8, SUC_lexicon_tags_27 },
    { 0xa4f49ba9, SUC_lexicon_tags_12 },
    { 0xb2149ba8, SUC_lexicon_tags_12 },
    { 0x6c749ba8, SUC_lexicon_tags_15 },
    { 0xa9b49bac, SUC_lexicon_tags_40 },
    { 0xa3d49bad, SUC_lexicon_tags_17 },
    { 0x97f49bae, SUC_lexicon_tags_2 },
    { 0x19b49ba9, SUC_lexicon_tags_37 },
    { 0x22b49bae, SUC_lexicon_tags_37 },
    { 0x54349bac, SUC_lexicon_tags_25 },
    { 0xa949bad, SUC_lexicon_tags_46 },
    { 0xb8349bad, SUC_lexicon_tags_45 },
    { 0x5e149baf, SUC_lexicon_tags_2 },
    { 0xa0749bb1, SUC_lexicon_tags_12 },
    { 0xa8549bb6, SUC_lexicon_tags_43 },
    { 0xfef49bb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3db49bb9, SUC_lexicon_tags_17 },
    { 0xc0f49bba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80349bbe, SUC_lexicon_tags_37 },
    { 0xbdb49bbf, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f49bc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77949bcb, SUC_lexicon_tags_68 },
    { 0xe9149bcc, SUC_lexicon_tags_25 },
    { 0x20b49bcd, SUC_lexicon_tags_14 },
    { 0xfcb49bce, SUC_lexicon_tags_11 },
    { 0x3cb49bcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5749bd1, SUC_lexicon_tags_1 },
    { 0x6dd49bd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47549bd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf49bd7, SUC_lexicon_tags_37 },
    { 0x3db49bd8, SUC_lexicon_tags_46 },
    { 0x32749bd9, SUC_lexicon_tags_17 },
    { 0xbc349bda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d49be1, SUC_lexicon_tags_33 },
    { 0x3d149be2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0749be8, SUC_lexicon_tags_7 },
    { 0xd7749be8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad749beb, SUC_lexicon_tags_31 },
    { 0xdb549bec, SUC_lexicon_tags_4 },
    { 0xa0d49bec, SUC_lexicon_tags_38 },
    { 0xce749bed, SUC_lexicon_tags_12 },
    { 0xff349bef, SUC_lexicon_tags_38 },
    { 0xddb49bf0, SUC_lexicon_tags_2 },
    { 0xd9349bf1, SUC_lexicon_tags_15 },
    { 0xc2749bee, SUC_lexicon_tags_31 },
    { 0xf0b49bf3, SUC_lexicon_tags_19 },
    { 0x10949bef, SUC_lexicon_tags_34 },
    { 0x13749bef, SUC_lexicon_tags_1 },
    { 0x1cd49bed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe549bf9, SUC_lexicon_tags_37 },
    { 0x94749bf9, SUC_lexicon_tags_15 },
    { 0x7cd49bf9, SUC_lexicon_tags_7 },
    { 0x94f49bfa, SUC_lexicon_tags_45 },
    { 0x45b49bfd, SUC_lexicon_tags_15 },
    { 0x8d149bfc, SUC_lexicon_tags_42 },
    { 0x9fb49bfb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7d49c01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b49c04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f49c0c, SUC_lexicon_tags_48 },
    { 0x47949c0c, SUC_lexicon_tags_45 },
    { 0x6f549c0d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3349c14, SUC_lexicon_tags_11 },
    { 0xb5149c15, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x53149c18, SUC_lexicon_tags_27 },
    { 0x7149c19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60b49c1b, SUC_lexicon_tags_11 },
    { 0x1af49c1b, SUC_lexicon_tags_16 },
    { 0x84749c1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1149c21, SUC_lexicon_tags_65 },
    { 0x14549c22, SUC_lexicon_tags_19 },
    { 0xe6d49c21, SUC_lexicon_tags_37 },
    { 0x1e749c24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b149c26, SUC_lexicon_tags_37 },
    { 0xc8149c27, SUC_lexicon_tags_31 },
    { 0xe8f49c28, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x31949c2b, SUC_lexicon_tags_45 },
    { 0xdc149c2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60d49c2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b49c33, SUC_lexicon_tags_14 },
    { 0xc1749c34, SUC_lexicon_tags_29 },
    { 0xb2749c35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdb49c37, SUC_lexicon_tags_25 },
    { 0xd6f49c38, SUC_lexicon_tags_12 },
    { 0x85b49c39, SUC_lexicon_tags_15 },
    { 0x25f49c38, SUC_lexicon_tags_13 },
    { 0xb9349c3b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6fd49c3d, SUC_lexicon_tags_25 },
    { 0xb5149c3e, SUC_lexicon_tags_27 },
    { 0x80149c3f, SUC_lexicon_tags_55 },
    { 0x7749c3f, SUC_lexicon_tags_14 },
    { 0x52d49c40, SUC_lexicon_tags_17 },
    { 0x42f49c40, SUC_lexicon_tags_60 },
    { 0x37549c41, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe349c4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95749c4e, SUC_lexicon_tags_37 },
    { 0x67d49c4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e349c52, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2149c55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xab549c58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b49c5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd49c60, SUC_lexicon_tags_38 },
    { 0xb6b49c61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f49c66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46d49c68, SUC_lexicon_tags_351 },
    { 0x91949c68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f49c6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f49c73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7bf49c75, SUC_lexicon_tags_11 },
    { 0xb6549c76, SUC_lexicon_tags_230 },
    { 0x17749c77, SUC_lexicon_tags_113 },
    { 0x34149c78, SUC_lexicon_tags_15 },
    { 0x7949c76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbed49c7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed49c80, SUC_lexicon_tags_25 },
    { 0x89149c80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949c8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f49c92, SUC_lexicon_tags_87 },
    { 0xf549c92, SUC_lexicon_tags_11 },
    { 0x1b349c93, SUC_lexicon_tags_15 },
    { 0x94549c94, SUC_lexicon_tags_33 },
    { 0x3149c96, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x79149c99, SUC_lexicon_tags_37 },
    { 0x60149c99, SUC_lexicon_tags_17 },
    { 0x24749c99, SUC_lexicon_tags_8 },
    { 0xd2149c9c, SUC_lexicon_tags_55 },
    { 0x27749c9c, SUC_lexicon_tags_2 },
    { 0xed749c9d, SUC_lexicon_tags_84 },
    { 0x98749c9f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9549ca6, SUC_lexicon_tags_1 },
    { 0x94d49ca6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd49caa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5749cb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80549cbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5949cc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b149cc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6b549cca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b49ccf, SUC_lexicon_tags_25 },
    { 0x2f349cd0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa149cd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3549cd6, SUC_lexicon_tags_36 },
    { 0x2b349cd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1349cda, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c149cdc, SUC_lexicon_tags_19 },
    { 0x31349cdd, SUC_lexicon_tags_7 },
    { 0x99749cde, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35149ce3, SUC_lexicon_tags_6 },
    { 0x85d49ce3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea349ce9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8549ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f49cf2, SUC_lexicon_tags_15 },
    { 0x3a749cf3, SUC_lexicon_tags_11 },
    { 0x84949cf2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b49cf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4549cfc, SUC_lexicon_tags_17 },
    { 0x7c749cfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef349d00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28d49d02, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb149d05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3549d09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbdd49d0b, SUC_lexicon_tags_1 },
    { 0x9ed49d0c, SUC_lexicon_tags_60 },
    { 0xa0b49d0d, SUC_lexicon_tags_37 },
    { 0xac349d0e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xecb49d10, SUC_lexicon_tags_2 },
    { 0xf549d10, SUC_lexicon_tags_1 },
    { 0xfe549d11, SUC_lexicon_tags_55 },
    { 0xd4d49d11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0b49d15, SUC_lexicon_tags_55 },
    { 0xb7f49d15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43349d18, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f49d1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x68949d1d, SUC_lexicon_tags_8 },
    { 0x14949d1e, SUC_lexicon_tags_17 },
    { 0xccd49d1f, SUC_lexicon_tags_695 },
    { 0x9d149d1e, SUC_lexicon_tags_17 },
    { 0x8f349d21, SUC_lexicon_tags_45 },
    { 0xd9d49d22, SUC_lexicon_tags_2 },
    { 0x7e749d23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6749d27, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfd749d29, SUC_lexicon_tags_1 },
    { 0x5d349d2a, SUC_lexicon_tags_3 },
    { 0xe7949d2b, SUC_lexicon_tags_43 },
    { 0xfb749d2c, SUC_lexicon_tags_28 },
    { 0x75549d2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff749d2f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x48f49d31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53549d34, SUC_lexicon_tags_15 },
    { 0x3bd49d35, SUC_lexicon_tags_31 },
    { 0x6f149d36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b49d39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89149d3b, SUC_lexicon_tags_2 },
    { 0x7df49d3c, SUC_lexicon_tags_47 },
    { 0x74149d3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd49d41, SUC_lexicon_tags_25 },
    { 0xdb549d42, SUC_lexicon_tags_0 },
    { 0x2749d43, SUC_lexicon_tags_27 },
    { 0x7f149d43, SUC_lexicon_tags_47 },
    { 0xefb49d45, SUC_lexicon_tags_29 },
    { 0x10949d44, SUC_lexicon_tags_47 },
    { 0x18949d41, SUC_lexicon_tags_13 },
    { 0x84349d48, SUC_lexicon_tags_1 },
    { 0xb4d49d49, SUC_lexicon_tags_37 },
    { 0x6db49d48, SUC_lexicon_tags_6 },
    { 0x2bb49d47, SUC_lexicon_tags_47 },
    { 0x8ff49d4c, SUC_lexicon_tags_0 },
    { 0x99749d4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d49d50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4549d54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b49d5b, SUC_lexicon_tags_40 },
    { 0xa4149d5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a549d5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0349d60, SUC_lexicon_tags_63 },
    { 0xc6149d61, SUC_lexicon_tags_16 },
    { 0x73d49d60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5ed49d64, SUC_lexicon_tags_47 },
    { 0xcbb49d65, SUC_lexicon_tags_2 },
    { 0x10d49d66, SUC_lexicon_tags_25 },
    { 0xdc749d67, SUC_lexicon_tags_7 },
    { 0x60549d68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb5d49d6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb149d6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x43d49d70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea349d73, SUC_lexicon_tags_6 },
    { 0x7c949d74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9349d78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb49d7a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x27149d7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f49d83, SUC_lexicon_tags_8 },
    { 0x61949d84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96349d87, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x28349d89, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc749d8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7749d8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a349d90, SUC_lexicon_tags_63 },
    { 0xe1b49d91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9349d98, SUC_lexicon_tags_19 },
    { 0xcc949d98, SUC_lexicon_tags_17 },
    { 0x71549d9a, SUC_lexicon_tags_29 },
    { 0x4749d98, SUC_lexicon_tags_8 },
    { 0x17349d9a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x31d49d9e, SUC_lexicon_tags_11 },
    { 0x8a349d9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8349da8, SUC_lexicon_tags_25 },
    { 0x77149da9, SUC_lexicon_tags_12 },
    { 0xff349da9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd49daf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5549db4, SUC_lexicon_tags_31 },
    { 0xde149db5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7a349db7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb949dba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x80949dbd, SUC_lexicon_tags_17 },
    { 0xa7b49dbe, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a749dc1, SUC_lexicon_tags_37 },
    { 0x12349dc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb749dc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60949dcd, SUC_lexicon_tags_15 },
    { 0xaf749dcd, SUC_lexicon_tags_7 },
    { 0xbff49dcf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5b749dd1, SUC_lexicon_tags_10 },
    { 0x2149dd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20d49dd4, SUC_lexicon_tags_87 },
    { 0x1e949dd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d49dd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba549dda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f49ddf, SUC_lexicon_tags_82 },
    { 0x26d49de0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c349de2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b49de5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7149ded, SUC_lexicon_tags_12 },
    { 0xb3f49dee, SUC_lexicon_tags_10 },
    { 0x44949def, SUC_lexicon_tags_27 },
    { 0xdbb49def, SUC_lexicon_tags_45 },
    { 0xa6149df1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76d49df3, SUC_lexicon_tags_37 },
    { 0xced49df3, SUC_lexicon_tags_49 },
    { 0x95f49df5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4949df7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdab49df9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e749e01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf49e08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x52b49e0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16d49e0c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x95549e0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa949e18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5b349e1a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x16349e1c, SUC_lexicon_tags_56 },
    { 0x70949e1c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb49e20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd49e24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf49e27, SUC_lexicon_tags_3 },
    { 0x16b49e28, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc749e2c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f49e2f, SUC_lexicon_tags_23 },
    { 0x4df49e2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3149e38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97f49e3a, SUC_lexicon_tags_2 },
    { 0x7e549e3a, SUC_lexicon_tags_12 },
    { 0xa4949e3a, SUC_lexicon_tags_23 },
    { 0xa8b49e3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc349e3f, SUC_lexicon_tags_56 },
    { 0x70f49e3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d49e44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8d49e46, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72749e4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9549e4c, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x38949e4f, SUC_lexicon_tags_12 },
    { 0xba949e50, SUC_lexicon_tags_13 },
    { 0x3af49e51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3549e53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb549e5d, SUC_lexicon_tags_60 },
    { 0x67549e5d, SUC_lexicon_tags_65 },
    { 0x13349e5f, SUC_lexicon_tags_70 },
    { 0x67b49e60, SUC_lexicon_tags_25 },
    { 0xca549e5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40149e66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2949e6a, SUC_lexicon_tags_29 },
    { 0x8ff49e6b, SUC_lexicon_tags_25 },
    { 0x55d49e6c, SUC_lexicon_tags_16 },
    { 0x6dd49e6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d49e72, SUC_lexicon_tags_1 },
    { 0xf1749e72, SUC_lexicon_tags_37 },
    { 0x1fd49e74, SUC_lexicon_tags_46 },
    { 0x3ab49e74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x21549e78, SUC_lexicon_tags_17 },
    { 0x3eb49e78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb49e7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb349e81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5b49e83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7db49e85, SUC_lexicon_tags_11 },
    { 0xf0749e86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa549e89, SUC_lexicon_tags_23 },
    { 0xaed49e8a, SUC_lexicon_tags_14 },
    { 0x1bb49e8b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x41549e8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12549e92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x55349e95, SUC_lexicon_tags_201 },
    { 0x1d749e95, SUC_lexicon_tags_2 },
    { 0x26d49e96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77549e9a, SUC_lexicon_tags_15 },
    { 0x2f349e9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f349ea1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7d949ea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93949ea5, SUC_lexicon_tags_17 },
    { 0xe1149ea5, SUC_lexicon_tags_13 },
    { 0xce749ea5, SUC_lexicon_tags_15 },
    { 0x19549ea7, SUC_lexicon_tags_23 },
    { 0x15149ea5, SUC_lexicon_tags_23 },
    { 0x52f49eaa, SUC_lexicon_tags_11 },
    { 0x2a549eaa, SUC_lexicon_tags_16 },
    { 0x76949ea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b49eb2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d49eb6, SUC_lexicon_tags_6 },
    { 0x4e749eb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f49ebb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8349ebf, SUC_lexicon_tags_7 },
    { 0xdf49ec0, SUC_lexicon_tags_7 },
    { 0x52b49ec1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5549ec4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa5d49ec6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14349ec8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1749ecc, SUC_lexicon_tags_13 },
    { 0xc6149ecd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49ed1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d149ed3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc349eda, SUC_lexicon_tags_0 },
    { 0x57349eda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6549edd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1949edf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb49ee1, SUC_lexicon_tags_30 },
    { 0x63149ee2, SUC_lexicon_tags_17 },
    { 0xbc549ee2, SUC_lexicon_tags_7 },
    { 0x1b49ee3, SUC_lexicon_tags_19 },
    { 0xccb49ee4, SUC_lexicon_tags_65 },
    { 0x83d49ee3, SUC_lexicon_tags_98 },
    { 0xe5949ee7, SUC_lexicon_tags_87 },
    { 0xa7149ee8, SUC_lexicon_tags_29 },
    { 0xfeb49ee8, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea149eed, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xdfd49eef, SUC_lexicon_tags_14 },
    { 0x1f949ef0, SUC_lexicon_tags_38 },
    { 0x25149ef1, SUC_lexicon_tags_38 },
    { 0x11f49ef2, SUC_lexicon_tags_37 },
    { 0x36949ef3, SUC_lexicon_tags_380 },
    { 0xe0b49ef3, SUC_lexicon_tags_28 },
    { 0x59149ef5, SUC_lexicon_tags_38 },
    { 0x36349ef1, SUC_lexicon_tags_12 },
    { 0xdc749ef5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f149efb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeb149efd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd49f00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12749f05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5549f10, SUC_lexicon_tags_17 },
    { 0x58349f11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14d49f13, SUC_lexicon_tags_19 },
    { 0xec949f14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef49f17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b749f1a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82949f1e, SUC_lexicon_tags_12 },
    { 0x61349f1f, SUC_lexicon_tags_7 },
    { 0xd6d49f1f, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed549f28, SUC_lexicon_tags_38 },
    { 0x11749f29, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x53749f2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf749f2e, SUC_lexicon_tags_646 },
    { 0x46549f2e, SUC_lexicon_tags_35 },
    { 0x35549f30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d49f3a, SUC_lexicon_tags_45 },
    { 0xafd49f3b, SUC_lexicon_tags_60 },
    { 0x6af49f3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4149f44, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x26749f47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16349f4d, SUC_lexicon_tags_17 },
    { 0xe5f49f4e, SUC_lexicon_tags_8 },
    { 0x4df49f4d, SUC_lexicon_tags_47 },
    { 0x7d49f4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67549f52, SUC_lexicon_tags_53 },
    { 0x19f49f53, SUC_lexicon_tags_37 },
    { 0x25d49f54, SUC_lexicon_tags_37 },
    { 0x11149f55, SUC_lexicon_tags_39 },
    { 0xdc349f56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed49f5a, SUC_lexicon_tags_1 },
    { 0xa0549f5a, SUC_lexicon_tags_45 },
    { 0x69149f5c, SUC_lexicon_tags_17 },
    { 0x9a749f5d, SUC_lexicon_tags_11 },
    { 0x6a749f5e, SUC_lexicon_tags_12 },
    { 0xa6b49f5f, SUC_lexicon_tags_17 },
    { 0x8c549f60, SUC_lexicon_tags_37 },
    { 0x83749f60, SUC_lexicon_tags_15 },
    { 0xc0749f62, SUC_lexicon_tags_12 },
    { 0x3b749f60, SUC_lexicon_tags_17 },
    { 0x2d349f64, SUC_lexicon_tags_7 },
    { 0xc5949f61, SUC_lexicon_tags_25 },
    { 0xa0f49f61, SUC_lexicon_tags_37 },
    { 0x69f49f62, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0149f6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe3b49f6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b49f6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f49f7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cf49f7e, SUC_lexicon_tags_2 },
    { 0x4149f7f, SUC_lexicon_tags_29 },
    { 0xad149f7f, SUC_lexicon_tags_28 },
    { 0x90549f80, SUC_lexicon_tags_63 },
    { 0x46349f82, SUC_lexicon_tags_23 },
    { 0xac549f83, SUC_lexicon_tags_7 },
    { 0xeb149f7f, SUC_lexicon_tags_33 },
    { 0x2fd49f85, SUC_lexicon_tags_0 },
    { 0x61549f85, SUC_lexicon_tags_65 },
    { 0x9b349f85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x16149f89, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x68349f8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f49f8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f49f93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2fd49f95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79749f99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb49fa0, SUC_lexicon_tags_25 },
    { 0xbb549fa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b49fab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa1949fad, SUC_lexicon_tags_17 },
    { 0xccd49fae, SUC_lexicon_tags_46 },
    { 0x7e149faf, SUC_lexicon_tags_48 },
    { 0xcf349fb0, SUC_lexicon_tags_63 },
    { 0x3b149faf, SUC_lexicon_tags_107 },
    { 0x1549fb2, SUC_lexicon_tags_6 },
    { 0xc5549fb3, SUC_lexicon_tags_12 },
    { 0x94949fb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bb49fb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c949fbf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x36949fc1, SUC_lexicon_tags_28 },
    { 0x62149fc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15149fc5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x57949fc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af49fcb, SUC_lexicon_tags_48 },
    { 0x39149fcb, SUC_lexicon_tags_25 },
    { 0xd6949fcd, SUC_lexicon_tags_11 },
    { 0xdbd49fce, SUC_lexicon_tags_37 },
    { 0xa2b49fce, SUC_lexicon_tags_11 },
    { 0xf949fd0, SUC_lexicon_tags_48 },
    { 0xe7d49fce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf49fd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe6f49fd9, SUC_lexicon_tags_15 },
    { 0xf5749fd9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd49fe3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc1949fe5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x42f49fe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77149fea, SUC_lexicon_tags_2 },
    { 0x70749feb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde549fef, SUC_lexicon_tags_17 },
    { 0xf0949fef, SUC_lexicon_tags_39 },
    { 0x20149ff1, SUC_lexicon_tags_43 },
    { 0xde349fef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef49ffb, SUC_lexicon_tags_87 },
    { 0xfc549ffc, SUC_lexicon_tags_15 },
    { 0xdbd49ffd, SUC_lexicon_tags_31 },
    { 0x26549ffc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b34a004, SUC_lexicon_tags_1 },
    { 0x9394a004, SUC_lexicon_tags_43 },
    { 0xfcf4a006, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd054a00c, SUC_lexicon_tags_45 },
    { 0x6e74a00d, SUC_lexicon_tags_16 },
    { 0x2234a00e, SUC_lexicon_tags_8 },
    { 0xb754a00d, SUC_lexicon_tags_15 },
    { 0xad4a010, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xaaf4a012, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1674a016, SUC_lexicon_tags_62 },
    { 0x9754a017, SUC_lexicon_tags_62 },
    { 0xb574a018, SUC_lexicon_tags_2 },
    { 0x5254a018, SUC_lexicon_tags_23 },
    { 0x5474a01a, SUC_lexicon_tags_47 },
    { 0x3c94a018, SUC_lexicon_tags_39 },
    { 0x7f14a01c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f34a01e, SUC_lexicon_tags_9 },
    { 0x4a54a01f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe934a021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d4a026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9614a02b, SUC_lexicon_tags_47 },
    { 0x3ef4a02c, SUC_lexicon_tags_3 },
    { 0xc294a02c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f34a02f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2534a031, SUC_lexicon_tags_14 },
    { 0x21f4a031, SUC_lexicon_tags_26 },
    { 0x5774a032, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8f74a035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1614a037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b54a03f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6054a041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf74a043, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf4a04a, SUC_lexicon_tags_6 },
    { 0xf474a04a, SUC_lexicon_tags_17 },
    { 0x4314a04a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd14a04f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef94a051, SUC_lexicon_tags_1 },
    { 0x7534a052, SUC_lexicon_tags_11 },
    { 0x6e94a051, SUC_lexicon_tags_7 },
    { 0x74d4a054, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c14a056, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5914a059, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b4a060, SUC_lexicon_tags_5 },
    { 0xbf34a061, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b4a065, SUC_lexicon_tags_29 },
    { 0x7f14a066, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x9d4a068, SUC_lexicon_tags_17 },
    { 0xf014a068, SUC_lexicon_tags_7 },
    { 0x9994a069, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f14a070, SUC_lexicon_tags_25 },
    { 0xe94a071, SUC_lexicon_tags_1 },
    { 0xe894a071, SUC_lexicon_tags_12 },
    { 0xd14a073, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb94a076, SUC_lexicon_tags_34 },
    { 0x61b4a076, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54b4a079, SUC_lexicon_tags_49 },
    { 0x594a07a, SUC_lexicon_tags_48 },
    { 0xe94a079, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd4a080, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xec34a082, SUC_lexicon_tags_37 },
    { 0x7bd4a083, SUC_lexicon_tags_37 },
    { 0xa1f4a084, SUC_lexicon_tags_49 },
    { 0xf4f4a085, SUC_lexicon_tags_43 },
    { 0xfc34a082, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5074a088, SUC_lexicon_tags_15 },
    { 0xae94a089, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc914a094, SUC_lexicon_tags_27 },
    { 0x3334a095, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b4a09b, SUC_lexicon_tags_12 },
    { 0xbfb4a09b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9714a09f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3bb4a0a1, SUC_lexicon_tags_12 },
    { 0xafb4a0a2, SUC_lexicon_tags_2 },
    { 0x38f4a0a2, SUC_lexicon_tags_12 },
    { 0xb174a0a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd34a0a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9794a0aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d34a0ac, SUC_lexicon_tags_15 },
    { 0x3654a0ad, SUC_lexicon_tags_17 },
    { 0xfb34a0ae, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d4a0b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa674a0b7, SUC_lexicon_tags_17 },
    { 0xe7d4a0b8, SUC_lexicon_tags_26 },
    { 0x4574a0b8, SUC_lexicon_tags_0 },
    { 0x5254a0ba, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x994a0bc, SUC_lexicon_tags_7 },
    { 0x9d34a0bd, SUC_lexicon_tags_25 },
    { 0x6514a0be, SUC_lexicon_tags_15 },
    { 0x4a74a0bf, SUC_lexicon_tags_48 },
    { 0x3a94a0bf, SUC_lexicon_tags_26 },
    { 0x6a34a0bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e74a0c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b4a0c6, SUC_lexicon_tags_14 },
    { 0x6dd4a0c7, SUC_lexicon_tags_5 },
    { 0x4f14a0c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc574a0ca, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f4a0cf, SUC_lexicon_tags_6 },
    { 0x8614a0cf, SUC_lexicon_tags_58 },
    { 0xbcf4a0d1, SUC_lexicon_tags_37 },
    { 0x6594a0d2, SUC_lexicon_tags_38 },
    { 0xfe54a0d2, SUC_lexicon_tags_25 },
    { 0x3d54a0d3, SUC_lexicon_tags_63 },
    { 0xc5f4a0d5, SUC_lexicon_tags_15 },
    { 0x88b4a0d6, SUC_lexicon_tags_2 },
    { 0x2e74a0d4, SUC_lexicon_tags_15 },
    { 0xdff4a0d3, SUC_lexicon_tags_17 },
    { 0x6ab4a0d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x794a0dc, SUC_lexicon_tags_8 },
    { 0x31d4a0dd, SUC_lexicon_tags_37 },
    { 0x60d4a0de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81b4a0e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7834a0e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bb4a0e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4a0f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdd4a0f2, SUC_lexicon_tags_69 },
    { 0xf734a0f3, SUC_lexicon_tags_1 },
    { 0x9314a0f4, SUC_lexicon_tags_17 },
    { 0xd934a0f5, SUC_lexicon_tags_9 },
    { 0xf8f4a0f4, SUC_lexicon_tags_47 },
    { 0xc834a0f7, SUC_lexicon_tags_1 },
    { 0xee94a0f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44b4a0fa, SUC_lexicon_tags_7 },
    { 0x51d4a0fa, SUC_lexicon_tags_8 },
    { 0x72b4a0fa, SUC_lexicon_tags_12 },
    { 0x3774a0fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc654a0ff, SUC_lexicon_tags_45 },
    { 0x61d4a100, SUC_lexicon_tags_37 },
    { 0x24b4a101, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb074a105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf554a10b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x394a10d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa314a112, SUC_lexicon_tags_0 },
    { 0xd234a113, SUC_lexicon_tags_31 },
    { 0xe674a113, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b4a118, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b34a11a, SUC_lexicon_tags_23 },
    { 0xaf54a11b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f74a11e, SUC_lexicon_tags_37 },
    { 0xb6d4a11f, SUC_lexicon_tags_2 },
    { 0xf154a120, SUC_lexicon_tags_63 },
    { 0x1d54a120, SUC_lexicon_tags_8 },
    { 0xc6f4a122, SUC_lexicon_tags_7 },
    { 0xd8d4a11f, SUC_lexicon_tags_27 },
    { 0x30b4a11e, SUC_lexicon_tags_25 },
    { 0x9594a125, SUC_lexicon_tags_2 },
    { 0x6374a125, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c34a129, SUC_lexicon_tags_37 },
    { 0x69f4a12a, SUC_lexicon_tags_11 },
    { 0xc4b4a12a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3994a12d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa0b4a12f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5034a132, SUC_lexicon_tags_25 },
    { 0x954a133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb4a138, SUC_lexicon_tags_26 },
    { 0x3fd4a139, SUC_lexicon_tags_84 },
    { 0x7694a138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f74a13e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c94a144, SUC_lexicon_tags_43 },
    { 0xf434a145, SUC_lexicon_tags_25 },
    { 0x4294a146, SUC_lexicon_tags_25 },
    { 0xa094a146, SUC_lexicon_tags_37 },
    { 0x5a34a148, SUC_lexicon_tags_21 },
    { 0x1194a148, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xef94a14c, SUC_lexicon_tags_7 },
    { 0xc714a14d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd34a152, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2714a156, SUC_lexicon_tags_47 },
    { 0xa2d4a157, SUC_lexicon_tags_13 },
    { 0x6914a158, SUC_lexicon_tags_65 },
    { 0xa54a159, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb974a15b, SUC_lexicon_tags_1 },
    { 0xeb54a15b, SUC_lexicon_tags_7 },
    { 0x114a15b, SUC_lexicon_tags_12 },
    { 0x7774a15e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db4a162, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c74a165, SUC_lexicon_tags_14 },
    { 0xbbf4a166, SUC_lexicon_tags_9 },
    { 0xd5f4a167, SUC_lexicon_tags_0 },
    { 0x98b4a165, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb714a16b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe854a16f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4a17d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b4a183, SUC_lexicon_tags_6 },
    { 0x6fb4a183, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10d4a186, SUC_lexicon_tags_17 },
    { 0x6b34a186, SUC_lexicon_tags_15 },
    { 0x8b34a188, SUC_lexicon_tags_25 },
    { 0xe8f4a189, SUC_lexicon_tags_23 },
    { 0x53f4a186, SUC_lexicon_tags_8 },
    { 0x4dd4a18b, SUC_lexicon_tags_17 },
    { 0x1f14a18c, SUC_lexicon_tags_37 },
    { 0xd774a18d, SUC_lexicon_tags_3 },
    { 0x9d54a18e, SUC_lexicon_tags_15 },
    { 0x8c34a18e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa174a191, SUC_lexicon_tags_31 },
    { 0x7ff4a192, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b4a196, SUC_lexicon_tags_19 },
    { 0xfc74a197, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae54a19c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3954a1a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbd54a1a2, SUC_lexicon_tags_25 },
    { 0xe674a1a2, SUC_lexicon_tags_118 },
    { 0x1af4a1a4, SUC_lexicon_tags_47 },
    { 0xc994a1a5, SUC_lexicon_tags_23 },
    { 0x5b14a1a5, SUC_lexicon_tags_13 },
    { 0x5014a1a7, SUC_lexicon_tags_1 },
    { 0xe334a1a6, SUC_lexicon_tags_48 },
    { 0x6d4a1a5, SUC_lexicon_tags_1 },
    { 0x8834a1a3, SUC_lexicon_tags_2 },
    { 0x96d4a1a3, SUC_lexicon_tags_45 },
    { 0x6254a1ac, SUC_lexicon_tags_15 },
    { 0x5094a1a9, SUC_lexicon_tags_25 },
    { 0x854a1ae, SUC_lexicon_tags_7 },
    { 0x5814a1af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4a1b7, SUC_lexicon_tags_47 },
    { 0x63b4a1b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d4a1bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f4a1c2, SUC_lexicon_tags_26 },
    { 0xf7b4a1c3, SUC_lexicon_tags_25 },
    { 0x6b54a1c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xffd4a1c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa014a1c9, SUC_lexicon_tags_1 },
    { 0xe594a1ca, SUC_lexicon_tags_34 },
    { 0x42f4a1ca, SUC_lexicon_tags_37 },
    { 0x7d74a1cc, SUC_lexicon_tags_7 },
    { 0x8634a1cd, SUC_lexicon_tags_37 },
    { 0xb5f4a1cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf914a1d4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7894a1da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x64b4a1dc, SUC_lexicon_tags_37 },
    { 0x2f94a1dd, SUC_lexicon_tags_12 },
    { 0x76b4a1dd, SUC_lexicon_tags_17 },
    { 0x4a94a1df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9754a1e1, SUC_lexicon_tags_17 },
    { 0x1ef4a1e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d34a1e6, SUC_lexicon_tags_19 },
    { 0x8cb4a1e7, SUC_lexicon_tags_47 },
    { 0xcc74a1e8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdeb4a1ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6174a1ed, SUC_lexicon_tags_11 },
    { 0xb94a1ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d54a1f2, SUC_lexicon_tags_25 },
    { 0xcef4a1f3, SUC_lexicon_tags_31 },
    { 0xdfb4a1f4, SUC_lexicon_tags_15 },
    { 0xdd14a1f5, SUC_lexicon_tags_37 },
    { 0x7c94a1f6, SUC_lexicon_tags_11 },
    { 0xe634a1f5, SUC_lexicon_tags_43 },
    { 0x4174a1f8, SUC_lexicon_tags_82 },
    { 0x4094a1f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b14a1fb, SUC_lexicon_tags_25 },
    { 0xc8b4a1fb, SUC_lexicon_tags_6 },
    { 0x4314a1fb, SUC_lexicon_tags_28 },
    { 0x3974a1fe, SUC_lexicon_tags_25 },
    { 0x92f4a1ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b54a206, SUC_lexicon_tags_37 },
    { 0xb834a206, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb4a20d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf4a211, SUC_lexicon_tags_21 },
    { 0x1534a212, SUC_lexicon_tags_7 },
    { 0x2a94a212, SUC_lexicon_tags_82 },
    { 0xac94a212, SUC_lexicon_tags_47 },
    { 0xf314a211, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d4a218, SUC_lexicon_tags_37 },
    { 0x7694a219, SUC_lexicon_tags_2 },
    { 0xc374a21a, SUC_lexicon_tags_43 },
    { 0x3c94a21a, SUC_lexicon_tags_15 },
    { 0x5954a21c, SUC_lexicon_tags_46 },
    { 0xff4a21a, SUC_lexicon_tags_17 },
    { 0xf774a21b, SUC_lexicon_tags_47 },
    { 0xe394a21e, SUC_lexicon_tags_2 },
    { 0x674a220, SUC_lexicon_tags_11 },
    { 0x6eb4a21e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b4a225, SUC_lexicon_tags_11 },
    { 0xcf4a225, SUC_lexicon_tags_8 },
    { 0x1d74a227, SUC_lexicon_tags_25 },
    { 0xe554a225, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3774a22f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9414a231, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc2b4a233, SUC_lexicon_tags_25 },
    { 0xd3b4a234, SUC_lexicon_tags_2 },
    { 0xfe14a235, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad74a23f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaef4a241, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe494a243, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9014a245, SUC_lexicon_tags_12 },
    { 0xa634a246, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa214a248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e34a24e, SUC_lexicon_tags_25 },
    { 0xb794a24f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe014a257, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2234a259, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbab4a25b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974a25f, SUC_lexicon_tags_37 },
    { 0xb4a260, SUC_lexicon_tags_25 },
    { 0xac94a261, SUC_lexicon_tags_17 },
    { 0xaf34a260, SUC_lexicon_tags_15 },
    { 0xf4b4a25f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1894a266, SUC_lexicon_tags_10 },
    { 0x4494a266, SUC_lexicon_tags_19 },
    { 0x9194a267, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d14a26f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf994a271, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbeb4a273, SUC_lexicon_tags_26 },
    { 0x23b4a274, SUC_lexicon_tags_63 },
    { 0x3d34a273, SUC_lexicon_tags_43 },
    { 0x1354a276, SUC_lexicon_tags_37 },
    { 0x8df4a275, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf74a279, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9474a27c, SUC_lexicon_tags_14 },
    { 0x4bf4a27d, SUC_lexicon_tags_85 },
    { 0x7d94a27e, SUC_lexicon_tags_0 },
    { 0xd5f4a27d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2834a281, SUC_lexicon_tags_43 },
    { 0x1634a282, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6154a284, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4a288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20f4a28a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2994a290, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b4a298, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6474a29e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc014a2a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a14a2ab, SUC_lexicon_tags_12 },
    { 0x12f4a2ac, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5394a2ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f4a2b1, SUC_lexicon_tags_10 },
    { 0x4834a2b2, SUC_lexicon_tags_24 },
    { 0x2cb4a2b2, SUC_lexicon_tags_12 },
    { 0xf1f4a2b4, SUC_lexicon_tags_26 },
    { 0xeff4a2b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc294a2b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd774a2bb, SUC_lexicon_tags_25 },
    { 0x5df4a2bc, SUC_lexicon_tags_34 },
    { 0xc554a2bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1714a2c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d14a2c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cd4a2c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8174a2cb, SUC_lexicon_tags_37 },
    { 0x94b4a2cc, SUC_lexicon_tags_63 },
    { 0xe294a2cd, SUC_lexicon_tags_17 },
    { 0xe34a2ce, SUC_lexicon_tags_17 },
    { 0xab4a2cd, SUC_lexicon_tags_6 },
    { 0xc754a2d0, SUC_lexicon_tags_30 },
    { 0x7cb4a2d1, SUC_lexicon_tags_17 },
    { 0xa334a2d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4ff4a2d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d4a2dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe54a2df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74d4a2e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x69f4a2e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d4a2ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd4a2f3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad4a2f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda14a2f8, SUC_lexicon_tags_7 },
    { 0x6434a2f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb594a2fe, SUC_lexicon_tags_21 },
    { 0xbbb4a2ff, SUC_lexicon_tags_17 },
    { 0x47b4a2fe, SUC_lexicon_tags_8 },
    { 0xa714a301, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b4a305, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbad4a307, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd714a309, SUC_lexicon_tags_59 },
    { 0xe1b4a309, SUC_lexicon_tags_15 },
    { 0x7db4a30a, SUC_lexicon_tags_30 },
    { 0xebd4a30a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4014a311, SUC_lexicon_tags_11 },
    { 0x9cb4a312, SUC_lexicon_tags_38 },
    { 0xb294a312, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd774a318, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd4a322, SUC_lexicon_tags_6 },
    { 0x7bf4a323, SUC_lexicon_tags_66 },
    { 0x2a74a323, SUC_lexicon_tags_25 },
    { 0x2c74a322, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3674a329, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7414a32b, SUC_lexicon_tags_48 },
    { 0x36b4a32c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa214a331, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d4a335, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb4a339, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeed4a33b, SUC_lexicon_tags_9 },
    { 0x43b4a33c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9cf4a33e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb4a346, SUC_lexicon_tags_14 },
    { 0xc14a347, SUC_lexicon_tags_29 },
    { 0x26f4a348, SUC_lexicon_tags_37 },
    { 0x6874a349, SUC_lexicon_tags_11 },
    { 0x4294a349, SUC_lexicon_tags_15 },
    { 0x38d4a34b, SUC_lexicon_tags_12 },
    { 0x9094a348, SUC_lexicon_tags_17 },
    { 0x9794a34d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e14a34f, SUC_lexicon_tags_12 },
    { 0x96f4a350, SUC_lexicon_tags_77 },
    { 0x9e94a34f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b4a354, SUC_lexicon_tags_8 },
    { 0xac74a355, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b4a358, SUC_lexicon_tags_27 },
    { 0xa8b4a359, SUC_lexicon_tags_28 },
    { 0x3874a35a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a54a35c, SUC_lexicon_tags_63 },
    { 0x2114a35d, SUC_lexicon_tags_38 },
    { 0xcb94a35d, SUC_lexicon_tags_25 },
    { 0x5154a35f, SUC_lexicon_tags_63 },
    { 0xb054a360, SUC_lexicon_tags_7 },
    { 0x2d34a361, SUC_lexicon_tags_1 },
    { 0x9b34a362, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca14a366, SUC_lexicon_tags_47 },
    { 0x16b4a367, SUC_lexicon_tags_25 },
    { 0x8534a366, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff4a36c, SUC_lexicon_tags_14 },
    { 0xbe14a36d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc134a36f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d14a371, SUC_lexicon_tags_17 },
    { 0x57b4a371, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7754a374, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5754a376, SUC_lexicon_tags_30 },
    { 0xdc54a377, SUC_lexicon_tags_37 },
    { 0x5fb4a378, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe954a37d, SUC_lexicon_tags_17 },
    { 0x6ad4a37e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd634a381, SUC_lexicon_tags_25 },
    { 0x6ab4a382, SUC_lexicon_tags_2 },
    { 0xce54a381, SUC_lexicon_tags_59 },
    { 0xde14a384, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d4a388, SUC_lexicon_tags_235 },
    { 0xecd4a388, SUC_lexicon_tags_68 },
    { 0xad4a38a, SUC_lexicon_tags_12 },
    { 0x21b4a38b, SUC_lexicon_tags_33 },
    { 0x7974a38b, SUC_lexicon_tags_12 },
    { 0x2f74a389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c14a390, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xddd4a392, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe74a396, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4a39c, SUC_lexicon_tags_45 },
    { 0x97b4a39d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af4a3a1, SUC_lexicon_tags_31 },
    { 0x9f14a3a2, SUC_lexicon_tags_38 },
    { 0xa0d4a3a3, SUC_lexicon_tags_4 },
    { 0x1d14a3a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d4a3ab, SUC_lexicon_tags_27 },
    { 0x6854a3ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794a3b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d74a3b4, SUC_lexicon_tags_37 },
    { 0xf554a3b5, SUC_lexicon_tags_37 },
    { 0xefd4a3b5, SUC_lexicon_tags_37 },
    { 0x1274a3b7, SUC_lexicon_tags_7 },
    { 0xfa74a3b8, SUC_lexicon_tags_17 },
    { 0x68f4a3b9, SUC_lexicon_tags_46 },
    { 0x6634a3b5, SUC_lexicon_tags_0 },
    { 0x494a3bb, SUC_lexicon_tags_7 },
    { 0x6a14a3b4, SUC_lexicon_tags_37 },
    { 0x5cd4a3ba, SUC_lexicon_tags_17 },
    { 0x894a3be, SUC_lexicon_tags_27 },
    { 0x2714a3bf, SUC_lexicon_tags_25 },
    { 0x75b4a3bf, SUC_lexicon_tags_7 },
    { 0x7b4a3be, SUC_lexicon_tags_29 },
    { 0x92f4a3be, SUC_lexicon_tags_81 },
    { 0xfb14a3c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85b4a3c5, SUC_lexicon_tags_21 },
    { 0x6954a3c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1d4a3c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f4a3d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4a3da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab4a3dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4a3e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f4a3e7, SUC_lexicon_tags_9 },
    { 0x3954a3e8, SUC_lexicon_tags_60 },
    { 0xaad4a3e9, SUC_lexicon_tags_12 },
    { 0xf234a3ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a94a3ec, SUC_lexicon_tags_17 },
    { 0x3b54a3ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa314a3ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6014a3f1, SUC_lexicon_tags_15 },
    { 0x114a3f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3874a3f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f14a3fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6734a3ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf214a402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d4a413, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe974a41a, SUC_lexicon_tags_19 },
    { 0x1234a41a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3914a41d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f4a422, SUC_lexicon_tags_0 },
    { 0x30b4a422, SUC_lexicon_tags_12 },
    { 0x6b94a424, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3734a428, SUC_lexicon_tags_21 },
    { 0xdf54a429, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd4a42f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb4a438, SUC_lexicon_tags_33 },
    { 0xa694a439, SUC_lexicon_tags_4 },
    { 0x5fd4a43a, SUC_lexicon_tags_60 },
    { 0x6db4a438, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf894a441, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x73b4a443, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa94a446, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6b4a448, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d94a44b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4594a451, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b74a454, SUC_lexicon_tags_60 },
    { 0xd334a455, SUC_lexicon_tags_0 },
    { 0xadb4a456, SUC_lexicon_tags_23 },
    { 0x5b94a456, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4a45b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda54a45e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe754a466, SUC_lexicon_tags_3 },
    { 0xb9b4a466, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd4a471, SUC_lexicon_tags_65 },
    { 0xc754a472, SUC_lexicon_tags_31 },
    { 0x6a54a472, SUC_lexicon_tags_29 },
    { 0xbef4a474, SUC_lexicon_tags_9 },
    { 0x11d4a475, SUC_lexicon_tags_11 },
    { 0xc754a475, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x474a47a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b54a47c, SUC_lexicon_tags_11 },
    { 0xcd94a47c, SUC_lexicon_tags_11 },
    { 0xf4b4a47e, SUC_lexicon_tags_15 },
    { 0xf874a47c, SUC_lexicon_tags_46 },
    { 0xdf14a47d, SUC_lexicon_tags_2 },
    { 0x5d34a47c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb534a488, SUC_lexicon_tags_4 },
    { 0x3094a489, SUC_lexicon_tags_63 },
    { 0xf574a48a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3714a48c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3794a48e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xab94a490, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8314a499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc434a49b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b14a49e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc434a4a1, SUC_lexicon_tags_17 },
    { 0x6b34a4a2, SUC_lexicon_tags_47 },
    { 0x70d4a4a2, SUC_lexicon_tags_17 },
    { 0x5dd4a4a4, SUC_lexicon_tags_7 },
    { 0x1754a4a2, SUC_lexicon_tags_12 },
    { 0xbd54a4a4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb894a4aa, SUC_lexicon_tags_69 },
    { 0x3a34a4aa, SUC_lexicon_tags_7 },
    { 0xe094a4ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f4a4b3, SUC_lexicon_tags_27 },
    { 0x374a4b4, SUC_lexicon_tags_77 },
    { 0xf4f4a4b3, SUC_lexicon_tags_14 },
    { 0xa74a4b6, SUC_lexicon_tags_66 },
    { 0xd454a4b6, SUC_lexicon_tags_27 },
    { 0x1774a4b7, SUC_lexicon_tags_31 },
    { 0xb174a4b9, SUC_lexicon_tags_15 },
    { 0x8894a4ba, SUC_lexicon_tags_27 },
    { 0x29f4a4bb, SUC_lexicon_tags_40 },
    { 0x8454a4bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2a54a4be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d74a4c3, SUC_lexicon_tags_84 },
    { 0xe7f4a4c4, SUC_lexicon_tags_25 },
    { 0xb5f4a4c5, SUC_lexicon_tags_61 },
    { 0x60b4a4c6, SUC_lexicon_tags_82 },
    { 0x1614a4c7, SUC_lexicon_tags_1 },
    { 0x1d34a4c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf34a4ca, SUC_lexicon_tags_12 },
    { 0xa514a4cb, SUC_lexicon_tags_1 },
    { 0x8eb4a4cc, SUC_lexicon_tags_66 },
    { 0x9434a4cd, SUC_lexicon_tags_62 },
    { 0xb7d4a4ce, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7034a4d2, SUC_lexicon_tags_37 },
    { 0xd174a4d2, SUC_lexicon_tags_14 },
    { 0x1e54a4d4, SUC_lexicon_tags_23 },
    { 0x6174a4d5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb914a4d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d4a4dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcd74a4de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c14a4e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf34a4e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b4a4ee, SUC_lexicon_tags_141 },
    { 0xabf4a4ef, SUC_lexicon_tags_17 },
    { 0xefd4a4ef, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b34a4f6, SUC_lexicon_tags_37 },
    { 0x6c14a4f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1f4a4f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6134a4fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3d4a4fd, SUC_lexicon_tags_25 },
    { 0x3034a4fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa314a505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f4a509, SUC_lexicon_tags_23 },
    { 0xaed4a50a, SUC_lexicon_tags_95 },
    { 0x3834a509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc474a50d, SUC_lexicon_tags_48 },
    { 0x1eb4a50e, SUC_lexicon_tags_12 },
    { 0x21f4a50f, SUC_lexicon_tags_45 },
    { 0x1c14a50e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd5b4a512, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f4a517, SUC_lexicon_tags_15 },
    { 0xe0f4a518, SUC_lexicon_tags_1216 },
    { 0xff14a518, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7074a51f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdf4a521, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x974a528, SUC_lexicon_tags_15 },
    { 0x1394a528, SUC_lexicon_tags_2 },
    { 0x4354a52a, SUC_lexicon_tags_573 },
    { 0, NULL },
    { 0x1b74a52c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab4a52f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f4a535, SUC_lexicon_tags_68 },
    { 0x7934a536, SUC_lexicon_tags_25 },
    { 0xe034a535, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xcb54a539, SUC_lexicon_tags_1 },
    { 0xf4b4a53a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x73b4a53c, SUC_lexicon_tags_7 },
    { 0x8234a53d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d4a540, SUC_lexicon_tags_62 },
    { 0x97b4a541, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb4a546, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d4a549, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x33b4a54b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2074a550, SUC_lexicon_tags_7 },
    { 0xe634a550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa394a556, SUC_lexicon_tags_11 },
    { 0x14b4a556, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe54a55a, SUC_lexicon_tags_11 },
    { 0xbad4a55a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9ed4a55d, SUC_lexicon_tags_11 },
    { 0xa14a55e, SUC_lexicon_tags_4 },
    { 0x5f74a55e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2334a561, SUC_lexicon_tags_2 },
    { 0x15f4a562, SUC_lexicon_tags_37 },
    { 0x34f4a562, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9194a565, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6714a568, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d4a56b, SUC_lexicon_tags_4 },
    { 0x61f4a56b, SUC_lexicon_tags_63 },
    { 0xf774a56d, SUC_lexicon_tags_11 },
    { 0xd674a56d, SUC_lexicon_tags_7 },
    { 0x5854a56e, SUC_lexicon_tags_15 },
    { 0x35d4a570, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbe14a572, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x82f4a574, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5334a578, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d34a57d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd4a581, SUC_lexicon_tags_15 },
    { 0xd914a582, SUC_lexicon_tags_17 },
    { 0x6174a582, SUC_lexicon_tags_1 },
    { 0x1234a581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x694a587, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc174a58a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9274a598, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a74a59b, SUC_lexicon_tags_118 },
    { 0x7bb4a59c, SUC_lexicon_tags_29 },
    { 0x2874a59b, SUC_lexicon_tags_33 },
    { 0x12d4a59e, SUC_lexicon_tags_8 },
    { 0x18d4a59c, SUC_lexicon_tags_37 },
    { 0xbdb4a5a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3434a5a2, SUC_lexicon_tags_49 },
    { 0x5b14a5a3, SUC_lexicon_tags_38 },
    { 0x8cd4a5a3, SUC_lexicon_tags_27 },
    { 0x8a34a5a3, SUC_lexicon_tags_33 },
    { 0x32d4a5a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf4a5aa, SUC_lexicon_tags_84 },
    { 0x5f74a5aa, SUC_lexicon_tags_37 },
    { 0x4db4a5ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb074a5b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb54a5b8, SUC_lexicon_tags_7 },
    { 0x8194a5b9, SUC_lexicon_tags_14 },
    { 0x2a54a5ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb7d4a5bc, SUC_lexicon_tags_17 },
    { 0xf334a5bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4534a5c0, SUC_lexicon_tags_11 },
    { 0xa974a5c0, SUC_lexicon_tags_12 },
    { 0x13d4a5c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f4a5c7, SUC_lexicon_tags_47 },
    { 0x4134a5c7, SUC_lexicon_tags_15 },
    { 0xb574a5c9, SUC_lexicon_tags_38 },
    { 0x4314a5ca, SUC_lexicon_tags_17 },
    { 0x78b4a5cb, SUC_lexicon_tags_12 },
    { 0xcd14a5cb, SUC_lexicon_tags_25 },
    { 0x1d54a5cb, SUC_lexicon_tags_19 },
    { 0xc214a5c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d94a5d0, SUC_lexicon_tags_14 },
    { 0x2f4a5d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2cd4a5d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32d4a5d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf1d4a5d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7574a5d9, SUC_lexicon_tags_37 },
    { 0xbeb4a5d9, SUC_lexicon_tags_3 },
    { 0x97f4a5d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2874a5df, SUC_lexicon_tags_69 },
    { 0x3434a5e0, SUC_lexicon_tags_15 },
    { 0x2714a5e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbff4a5e3, SUC_lexicon_tags_49 },
    { 0xa174a5e4, SUC_lexicon_tags_26 },
    { 0x8b34a5e5, SUC_lexicon_tags_65 },
    { 0x30f4a5e6, SUC_lexicon_tags_68 },
    { 0xac14a5e5, SUC_lexicon_tags_15 },
    { 0xd9d4a5e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb314a5eb, SUC_lexicon_tags_63 },
    { 0x75d4a5ec, SUC_lexicon_tags_23 },
    { 0x7954a5ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60f4a5ef, SUC_lexicon_tags_45 },
    { 0xbfb4a5f0, SUC_lexicon_tags_61 },
    { 0xcf34a5f0, SUC_lexicon_tags_56 },
    { 0x5614a5f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41f4a5f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1034a5f6, SUC_lexicon_tags_15 },
    { 0x1474a5f6, SUC_lexicon_tags_25 },
    { 0x9934a5f8, SUC_lexicon_tags_105 },
    { 0x18d4a5f8, SUC_lexicon_tags_91 },
    { 0xdb54a5f6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5c94a5fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3af4a5fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7974a602, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd14a607, SUC_lexicon_tags_37 },
    { 0xecf4a608, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6214a60f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xac54a612, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7694a614, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f54a61b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff4a624, SUC_lexicon_tags_11 },
    { 0xdd4a625, SUC_lexicon_tags_21 },
    { 0x47b4a626, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd774a62a, SUC_lexicon_tags_12 },
    { 0x9d54a62b, SUC_lexicon_tags_12 },
    { 0x4c74a62c, SUC_lexicon_tags_49 },
    { 0xc594a62a, SUC_lexicon_tags_17 },
    { 0x45d4a62e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe5f4a630, SUC_lexicon_tags_21 },
    { 0xb4a631, SUC_lexicon_tags_77 },
    { 0xbcf4a632, SUC_lexicon_tags_50 },
    { 0x4b94a630, SUC_lexicon_tags_91 },
    { 0x30f4a634, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f4a638, SUC_lexicon_tags_2 },
    { 0x8e34a638, SUC_lexicon_tags_33 },
    { 0x9a14a63a, SUC_lexicon_tags_16 },
    { 0x7594a63b, SUC_lexicon_tags_61 },
    { 0xe9d4a63c, SUC_lexicon_tags_37 },
    { 0xac14a63c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed34a642, SUC_lexicon_tags_60 },
    { 0x2254a642, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd674a649, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb4a64c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef4a64f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4a656, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x97b4a658, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4794a65c, SUC_lexicon_tags_0 },
    { 0xb334a65d, SUC_lexicon_tags_14 },
    { 0x8b34a65e, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c54a661, SUC_lexicon_tags_1 },
    { 0xa934a662, SUC_lexicon_tags_12 },
    { 0xd034a663, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a54a669, SUC_lexicon_tags_13 },
    { 0xe34a66a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd34a66c, SUC_lexicon_tags_25 },
    { 0xb274a66c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f4a672, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf194a674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e94a67a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa014a682, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5114a684, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb4a68c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33d4a68e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd4a691, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b4a695, SUC_lexicon_tags_11 },
    { 0x4d34a696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9894a698, SUC_lexicon_tags_47 },
    { 0x2614a699, SUC_lexicon_tags_3 },
    { 0x414a699, SUC_lexicon_tags_0 },
    { 0x1b14a699, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac14a69f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7194a6a2, SUC_lexicon_tags_65 },
    { 0x5cb4a6a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ef4a6a5, SUC_lexicon_tags_43 },
    { 0x44f4a6a5, SUC_lexicon_tags_15 },
    { 0xe9d4a6a5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d4a6aa, SUC_lexicon_tags_19 },
    { 0xb254a6ab, SUC_lexicon_tags_0 },
    { 0xb9f4a6aa, SUC_lexicon_tags_11 },
    { 0x7634a6ad, SUC_lexicon_tags_15 },
    { 0x4734a6ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefb4a6b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc414a6b7, SUC_lexicon_tags_6 },
    { 0xa954a6b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1874a6bd, SUC_lexicon_tags_11 },
    { 0xabd4a6bd, SUC_lexicon_tags_25 },
    { 0xa9d4a6bd, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae54a6c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b4a6cd, SUC_lexicon_tags_17 },
    { 0x2ed4a6ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3774a6d3, SUC_lexicon_tags_7 },
    { 0x5894a6d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6074a6d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa574a6dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f4a6df, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xf174a6e1, SUC_lexicon_tags_15 },
    { 0x5954a6e2, SUC_lexicon_tags_2 },
    { 0x14d4a6e2, SUC_lexicon_tags_0 },
    { 0xe9d4a6e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4374a6e7, SUC_lexicon_tags_15 },
    { 0x75f4a6e8, SUC_lexicon_tags_25 },
    { 0xb094a6e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd54a6eb, SUC_lexicon_tags_14 },
    { 0x68d4a6ec, SUC_lexicon_tags_29 },
    { 0x9314a6ec, SUC_lexicon_tags_37 },
    { 0xbd14a6ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3054a6f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634a6f7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd74a6f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1274a6fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe34a701, SUC_lexicon_tags_37 },
    { 0x43b4a701, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36b4a704, SUC_lexicon_tags_7 },
    { 0x4074a705, SUC_lexicon_tags_77 },
    { 0x3354a705, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde34a70b, SUC_lexicon_tags_29 },
    { 0xe834a70c, SUC_lexicon_tags_37 },
    { 0x9674a70c, SUC_lexicon_tags_16 },
    { 0xef4a70d, SUC_lexicon_tags_33 },
    { 0xd294a70f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f94a712, SUC_lexicon_tags_45 },
    { 0x9dd4a712, SUC_lexicon_tags_11 },
    { 0x8df4a712, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4a71e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6794a723, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef4a72a, SUC_lexicon_tags_2 },
    { 0xd9f4a72a, SUC_lexicon_tags_39 },
    { 0x4a34a72a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb4a72f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b4a733, SUC_lexicon_tags_1 },
    { 0xf354a733, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdb4a736, SUC_lexicon_tags_12 },
    { 0xaf14a736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2df4a739, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f4a73e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6034a744, SUC_lexicon_tags_27 },
    { 0xdab4a744, SUC_lexicon_tags_12 },
    { 0x65f4a744, SUC_lexicon_tags_34 },
    { 0xbdb4a744, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ef4a749, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1614a74b, SUC_lexicon_tags_10 },
    { 0x3834a74c, SUC_lexicon_tags_7 },
    { 0xf594a74d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa074a750, SUC_lexicon_tags_43 },
    { 0xf9d4a750, SUC_lexicon_tags_15 },
    { 0xbd54a750, SUC_lexicon_tags_66 },
    { 0xc5b4a752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc794a755, SUC_lexicon_tags_25 },
    { 0x5fb4a756, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd4a767, SUC_lexicon_tags_68 },
    { 0xa234a768, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d74a76b, SUC_lexicon_tags_16 },
    { 0xc734a76b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7514a76e, SUC_lexicon_tags_39 },
    { 0x7ef4a76f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1514a775, SUC_lexicon_tags_8 },
    { 0xc454a776, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49d4a778, SUC_lexicon_tags_1 },
    { 0x5f54a778, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b4a77c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f54a77e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe474a781, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d4a785, SUC_lexicon_tags_1 },
    { 0x61d4a786, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b14a78c, SUC_lexicon_tags_48 },
    { 0x5254a78d, SUC_lexicon_tags_6 },
    { 0x5e54a78e, SUC_lexicon_tags_17 },
    { 0x50d4a78e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5234a792, SUC_lexicon_tags_11 },
    { 0xd074a793, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3d4a795, SUC_lexicon_tags_15 },
    { 0xd874a795, SUC_lexicon_tags_38 },
    { 0x8114a797, SUC_lexicon_tags_38 },
    { 0x4034a798, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe094a79c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b14a7a1, SUC_lexicon_tags_8 },
    { 0x3c14a7a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc114a7a6, SUC_lexicon_tags_77 },
    { 0xaeb4a7a6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f4a7aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe94a7ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf4a7af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff4a7b2, SUC_lexicon_tags_7 },
    { 0xa134a7b2, SUC_lexicon_tags_17 },
    { 0x4c94a7b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f4a7b9, SUC_lexicon_tags_79 },
    { 0xf8b4a7b9, SUC_lexicon_tags_25 },
    { 0x9cf4a7b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab4a7be, SUC_lexicon_tags_9 },
    { 0xfd4a7bf, SUC_lexicon_tags_13 },
    { 0x2774a7bf, SUC_lexicon_tags_62 },
    { 0x4a34a7c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4174a7c3, SUC_lexicon_tags_15 },
    { 0x9734a7c4, SUC_lexicon_tags_0 },
    { 0x46f4a7c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb34a7c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4a7cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba94a7cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf4a7d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f4a7d7, SUC_lexicon_tags_2 },
    { 0x9914a7d8, SUC_lexicon_tags_17 },
    { 0x3154a7d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3574a7dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6554a7df, SUC_lexicon_tags_25 },
    { 0xe8d4a7df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d4a7e4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e74a7e7, SUC_lexicon_tags_19 },
    { 0xa8d4a7e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5f14a7ea, SUC_lexicon_tags_15 },
    { 0xa034a7ea, SUC_lexicon_tags_15 },
    { 0xa3f4a7ec, SUC_lexicon_tags_87 },
    { 0x2554a7ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4a7f0, SUC_lexicon_tags_2 },
    { 0x5c54a7f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc214a7f3, SUC_lexicon_tags_47 },
    { 0xa694a7f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde54a7fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b4a7fe, SUC_lexicon_tags_48 },
    { 0xc0b4a7ff, SUC_lexicon_tags_66 },
    { 0xf5d4a800, SUC_lexicon_tags_83 },
    { 0x4cf4a801, SUC_lexicon_tags_27 },
    { 0x1d4a802, SUC_lexicon_tags_13 },
    { 0xf9d4a7ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f4a809, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b4a80d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d94a80f, SUC_lexicon_tags_11 },
    { 0xcaf4a80f, SUC_lexicon_tags_16 },
    { 0xb5b4a811, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc214a817, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b34a81a, SUC_lexicon_tags_1 },
    { 0x18b4a81b, SUC_lexicon_tags_25 },
    { 0x7b4a81a, SUC_lexicon_tags_46 },
    { 0x15f4a81d, SUC_lexicon_tags_151 },
    { 0xfff4a81e, SUC_lexicon_tags_25 },
    { 0x7bb4a81f, SUC_lexicon_tags_14 },
    { 0x5654a81f, SUC_lexicon_tags_17 },
    { 0x3314a821, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf74a824, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd14a827, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f4a82f, SUC_lexicon_tags_17 },
    { 0x3954a82f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f74a832, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb4a836, SUC_lexicon_tags_12 },
    { 0xdef4a837, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98b4a839, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9994a83b, SUC_lexicon_tags_6 },
    { 0x8a74a83c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d14a83f, SUC_lexicon_tags_11 },
    { 0x6874a840, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa934a843, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6534a845, SUC_lexicon_tags_2 },
    { 0x77b4a846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa274a84d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf034a851, SUC_lexicon_tags_25 },
    { 0x8994a852, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d4a855, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8434a857, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4114a85a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x61f4a85c, SUC_lexicon_tags_25 },
    { 0x3b14a85d, SUC_lexicon_tags_15 },
    { 0x47d4a85e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x65f4a860, SUC_lexicon_tags_64 },
    { 0xedf4a860, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7894a868, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74a86d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd954a86f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d4a872, SUC_lexicon_tags_1 },
    { 0xe814a873, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f4a87f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b4a885, SUC_lexicon_tags_18 },
    { 0x8ab4a886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4894a889, SUC_lexicon_tags_57 },
    { 0x1494a88a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f34a891, SUC_lexicon_tags_15 },
    { 0xfb14a892, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3b74a894, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4354a896, SUC_lexicon_tags_18 },
    { 0x3194a897, SUC_lexicon_tags_45 },
    { 0x6354a898, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb954a89b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f4a89e, SUC_lexicon_tags_2 },
    { 0x2074a89f, SUC_lexicon_tags_8 },
    { 0x7594a89f, SUC_lexicon_tags_23 },
    { 0x12b4a8a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeeb4a8a3, SUC_lexicon_tags_30 },
    { 0x84b4a8a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f4a8a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b4a8aa, SUC_lexicon_tags_17 },
    { 0xa0f4a8ab, SUC_lexicon_tags_19 },
    { 0xd0f4a8ac, SUC_lexicon_tags_37 },
    { 0x7114a8ad, SUC_lexicon_tags_12 },
    { 0x4f14a8ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9694a8b0, SUC_lexicon_tags_37 },
    { 0xa374a8b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x61f4a8b3, SUC_lexicon_tags_14 },
    { 0xa9f4a8b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf714a8b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3314a8be, SUC_lexicon_tags_16 },
    { 0xf674a8bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b4a8c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9f4a8c4, SUC_lexicon_tags_12 },
    { 0xf614a8c5, SUC_lexicon_tags_47 },
    { 0x2754a8c6, SUC_lexicon_tags_28 },
    { 0xe254a8c4, SUC_lexicon_tags_7 },
    { 0xff54a8c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4cf4a8ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7914a8cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b54a8d0, SUC_lexicon_tags_2 },
    { 0x3934a8d1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b74a8d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a54a8d6, SUC_lexicon_tags_37 },
    { 0x4e94a8d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf274a8da, SUC_lexicon_tags_14 },
    { 0x2134a8db, SUC_lexicon_tags_1 },
    { 0xf474a8dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9a94a8de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb94a8e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67d4a8e3, SUC_lexicon_tags_28 },
    { 0xfdb4a8e3, SUC_lexicon_tags_25 },
    { 0x5d34a8e3, SUC_lexicon_tags_15 },
    { 0x58b4a8e6, SUC_lexicon_tags_11 },
    { 0x45d4a8e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d74a8ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc414a8ef, SUC_lexicon_tags_15 },
    { 0xa634a8ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c74a8f6, SUC_lexicon_tags_60 },
    { 0x15b4a8f7, SUC_lexicon_tags_65 },
    { 0x4334a8f6, SUC_lexicon_tags_25 },
    { 0x8d94a8f9, SUC_lexicon_tags_31 },
    { 0x65f4a8f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df4a8fd, SUC_lexicon_tags_7 },
    { 0xd214a8fd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f4a901, SUC_lexicon_tags_14 },
    { 0x84d4a902, SUC_lexicon_tags_33 },
    { 0x5cd4a903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe694a906, SUC_lexicon_tags_49 },
    { 0xd034a906, SUC_lexicon_tags_17 },
    { 0x4ed4a907, SUC_lexicon_tags_28 },
    { 0x7794a909, SUC_lexicon_tags_38 },
    { 0x8f94a90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff4a912, SUC_lexicon_tags_45 },
    { 0xbe14a913, SUC_lexicon_tags_2 },
    { 0x2734a912, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5974a916, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa514a918, SUC_lexicon_tags_12 },
    { 0x3d94a919, SUC_lexicon_tags_47 },
    { 0x76b4a918, SUC_lexicon_tags_17 },
    { 0xdb94a91a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1db4a91d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7254a91f, SUC_lexicon_tags_29 },
    { 0x6494a91f, SUC_lexicon_tags_108 },
    { 0x32f4a920, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd54a927, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9254a931, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8394a936, SUC_lexicon_tags_80 },
    { 0xa54a936, SUC_lexicon_tags_11 },
    { 0xbd4a937, SUC_lexicon_tags_0 },
    { 0x77b4a939, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa914a93d, SUC_lexicon_tags_60 },
    { 0xad54a93e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6af4a940, SUC_lexicon_tags_37 },
    { 0x7b94a941, SUC_lexicon_tags_15 },
    { 0xa594a940, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xd3d4a944, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcad4a946, SUC_lexicon_tags_25 },
    { 0x16f4a947, SUC_lexicon_tags_0 },
    { 0x2c14a948, SUC_lexicon_tags_21 },
    { 0xc294a947, SUC_lexicon_tags_0 },
    { 0xdff4a94a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1374a94e, SUC_lexicon_tags_21 },
    { 0x5e74a94f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc794a951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22f4a953, SUC_lexicon_tags_26 },
    { 0xae94a954, SUC_lexicon_tags_37 },
    { 0x9974a954, SUC_lexicon_tags_15 },
    { 0xf354a956, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b4a95b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c54a95f, SUC_lexicon_tags_11 },
    { 0xc494a960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd54a963, SUC_lexicon_tags_12 },
    { 0x7f34a964, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5f4a966, SUC_lexicon_tags_12 },
    { 0xd4d4a967, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e74a96b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f34a96f, SUC_lexicon_tags_17 },
    { 0xa34a96f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2634a973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xedd4a975, SUC_lexicon_tags_1 },
    { 0xa0d4a976, SUC_lexicon_tags_25 },
    { 0xa634a977, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b14a97b, SUC_lexicon_tags_405 },
    { 0, NULL },
    { 0xf7f4a97d, SUC_lexicon_tags_27 },
    { 0xec94a97e, SUC_lexicon_tags_17 },
    { 0xabb4a97d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd494a986, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f4a98d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1894a995, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf894a998, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa714a99e, SUC_lexicon_tags_87 },
    { 0x8b14a99f, SUC_lexicon_tags_17 },
    { 0x2194a99f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xbc14a9a2, SUC_lexicon_tags_48 },
    { 0x23b4a9a3, SUC_lexicon_tags_12 },
    { 0x9754a9a4, SUC_lexicon_tags_15 },
    { 0x6c34a9a5, SUC_lexicon_tags_14 },
    { 0xfff4a9a6, SUC_lexicon_tags_77 },
    { 0xfd14a9a6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad4a9aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a34a9ad, SUC_lexicon_tags_14 },
    { 0x2574a9ad, SUC_lexicon_tags_11 },
    { 0xacf4a9af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6f4a9b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd4a9b4, SUC_lexicon_tags_34 },
    { 0x3314a9b5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xedb4a9b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72d4a9b9, SUC_lexicon_tags_17 },
    { 0x1394a9ba, SUC_lexicon_tags_7 },
    { 0xdfd4a9bb, SUC_lexicon_tags_17 },
    { 0x76b4a9bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1794a9c5, SUC_lexicon_tags_16 },
    { 0x43f4a9c6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f74a9c9, SUC_lexicon_tags_27 },
    { 0xcad4a9c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32d4a9cc, SUC_lexicon_tags_15 },
    { 0x89f4a9cd, SUC_lexicon_tags_11 },
    { 0xf6f4a9ce, SUC_lexicon_tags_26 },
    { 0x7794a9cf, SUC_lexicon_tags_37 },
    { 0xf2b4a9d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b4a9d3, SUC_lexicon_tags_43 },
    { 0xf3f4a9d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4714a9d6, SUC_lexicon_tags_15 },
    { 0x194a9d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f14a9d9, SUC_lexicon_tags_49 },
    { 0xcb74a9da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xca94a9dd, SUC_lexicon_tags_37 },
    { 0x2b74a9de, SUC_lexicon_tags_47 },
    { 0x7ef4a9de, SUC_lexicon_tags_17 },
    { 0x9e54a9df, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc434a9e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc7b4a9e5, SUC_lexicon_tags_14 },
    { 0xc654a9e6, SUC_lexicon_tags_25 },
    { 0x2234a9e7, SUC_lexicon_tags_38 },
    { 0x8354a9e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b4a9eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b4a9ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d4a9f1, SUC_lexicon_tags_2 },
    { 0xb6b4a9f1, SUC_lexicon_tags_11 },
    { 0xd0f4a9f1, SUC_lexicon_tags_27 },
    { 0x54f4a9f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f4a9f7, SUC_lexicon_tags_2 },
    { 0x18d4a9f8, SUC_lexicon_tags_9 },
    { 0xf354a9f9, SUC_lexicon_tags_39 },
    { 0x1794a9f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8934a9fc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d4aa00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d4aa03, SUC_lexicon_tags_26 },
    { 0x88f4aa04, SUC_lexicon_tags_37 },
    { 0xced4aa04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9614aa0a, SUC_lexicon_tags_60 },
    { 0x1db4aa0b, SUC_lexicon_tags_1 },
    { 0xb6f4aa0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6114aa0f, SUC_lexicon_tags_37 },
    { 0x1594aa0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b4aa13, SUC_lexicon_tags_1 },
    { 0xc094aa14, SUC_lexicon_tags_25 },
    { 0xab74aa15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1994aa17, SUC_lexicon_tags_17 },
    { 0xdcd4aa18, SUC_lexicon_tags_275 },
    { 0x2c74aa18, SUC_lexicon_tags_65 },
    { 0xfc14aa1a, SUC_lexicon_tags_0 },
    { 0x7314aa1a, SUC_lexicon_tags_0 },
    { 0x1274aa1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14aa1f, SUC_lexicon_tags_104 },
    { 0xa5d4aa20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd4aa22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef4aa25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d4aa2e, SUC_lexicon_tags_19 },
    { 0x74f4aa2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3dd4aa31, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5db4aa33, SUC_lexicon_tags_87 },
    { 0x8234aa33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ef4aa36, SUC_lexicon_tags_30 },
    { 0xf7d4aa36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a14aa3a, SUC_lexicon_tags_23 },
    { 0xddd4aa3b, SUC_lexicon_tags_38 },
    { 0x5674aa3c, SUC_lexicon_tags_11 },
    { 0xf034aa3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead4aa42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc534aa44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f4aa49, SUC_lexicon_tags_55 },
    { 0x1f74aa49, SUC_lexicon_tags_11 },
    { 0x1a94aa4a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2154aa4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4aa52, SUC_lexicon_tags_11 },
    { 0xbff4aa53, SUC_lexicon_tags_68 },
    { 0x3a74aa53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f74aa59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f4aa5c, SUC_lexicon_tags_45 },
    { 0xa54aa5d, SUC_lexicon_tags_10 },
    { 0xcef4aa5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1234aa65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6b4aa67, SUC_lexicon_tags_19 },
    { 0xd074aa68, SUC_lexicon_tags_35 },
    { 0x4b14aa68, SUC_lexicon_tags_19 },
    { 0x7754aa6a, SUC_lexicon_tags_14 },
    { 0xadd4aa6a, SUC_lexicon_tags_11 },
    { 0x4df4aa6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d14aa6f, SUC_lexicon_tags_46 },
    { 0xa394aa6f, SUC_lexicon_tags_11 },
    { 0x9474aa71, SUC_lexicon_tags_9 },
    { 0x38d4aa71, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6634aa75, SUC_lexicon_tags_15 },
    { 0xb514aa76, SUC_lexicon_tags_15 },
    { 0x9f54aa77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa554aa79, SUC_lexicon_tags_13 },
    { 0xa274aa7a, SUC_lexicon_tags_7 },
    { 0xce14aa7a, SUC_lexicon_tags_37 },
    { 0xb494aa7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4614aa7f, SUC_lexicon_tags_70 },
    { 0xd974aa80, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8054aa83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d34aa86, SUC_lexicon_tags_37 },
    { 0x5dd4aa86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb54aa89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2254aa8f, SUC_lexicon_tags_25 },
    { 0xe774aa8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf994aa94, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4114aa97, SUC_lexicon_tags_2 },
    { 0x1174aa97, SUC_lexicon_tags_12 },
    { 0x3d14aa99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x334aa9c, SUC_lexicon_tags_27 },
    { 0x1ab4aa9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5054aa9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb4aaa5, SUC_lexicon_tags_33 },
    { 0x1bf4aaa6, SUC_lexicon_tags_33 },
    { 0xf8b4aaa5, SUC_lexicon_tags_38 },
    { 0xff74aaa8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e94aaac, SUC_lexicon_tags_11 },
    { 0x2254aaad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb254aab2, SUC_lexicon_tags_18 },
    { 0x7174aab2, SUC_lexicon_tags_11 },
    { 0xff94aab4, SUC_lexicon_tags_634 },
    { 0x2274aab5, SUC_lexicon_tags_13 },
    { 0xc9b4aab6, SUC_lexicon_tags_2 },
    { 0xe894aab7, SUC_lexicon_tags_15 },
    { 0xa074aab8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x16d4aaba, SUC_lexicon_tags_11 },
    { 0x4574aabb, SUC_lexicon_tags_26 },
    { 0xdcd4aaba, SUC_lexicon_tags_8 },
    { 0xad54aaba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4aac0, SUC_lexicon_tags_66 },
    { 0xb394aac1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc894aac3, SUC_lexicon_tags_29 },
    { 0xfa94aac4, SUC_lexicon_tags_60 },
    { 0x834aac5, SUC_lexicon_tags_49 },
    { 0x3f14aac6, SUC_lexicon_tags_45 },
    { 0x31b4aac7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x414aac9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4114aacd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x28f4aacf, SUC_lexicon_tags_32 },
    { 0xcd14aacf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d14aad5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb4aadf, SUC_lexicon_tags_25 },
    { 0x8d74aae0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b4aae4, SUC_lexicon_tags_14 },
    { 0xd74aae5, SUC_lexicon_tags_11 },
    { 0xba94aae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a34aae9, SUC_lexicon_tags_17 },
    { 0x6854aaea, SUC_lexicon_tags_77 },
    { 0x9994aaea, SUC_lexicon_tags_11 },
    { 0xd54aaec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2094aaef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4754aaf8, SUC_lexicon_tags_29 },
    { 0x5654aaf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb914aafb, SUC_lexicon_tags_38 },
    { 0x8df4aafc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2d4aafe, SUC_lexicon_tags_14 },
    { 0xfff4aaff, SUC_lexicon_tags_15 },
    { 0x2854ab00, SUC_lexicon_tags_25 },
    { 0x73d4ab01, SUC_lexicon_tags_14 },
    { 0x674ab01, SUC_lexicon_tags_37 },
    { 0xe654ab01, SUC_lexicon_tags_37 },
    { 0xf554ab03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1954ab06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d94ab08, SUC_lexicon_tags_14 },
    { 0xad14ab09, SUC_lexicon_tags_7 },
    { 0x54d4ab09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x974ab0c, SUC_lexicon_tags_28 },
    { 0xdef4ab0d, SUC_lexicon_tags_82 },
    { 0x1774ab0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af4ab12, SUC_lexicon_tags_11 },
    { 0x91f4ab12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x43b4ab15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f14ab18, SUC_lexicon_tags_11 },
    { 0x9c54ab19, SUC_lexicon_tags_14 },
    { 0xb4b4ab1a, SUC_lexicon_tags_77 },
    { 0xa814ab1a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f34ab21, SUC_lexicon_tags_23 },
    { 0x70b4ab22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa34ab25, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab94ab2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa934ab31, SUC_lexicon_tags_37 },
    { 0xe854ab32, SUC_lexicon_tags_39 },
    { 0x5ab4ab33, SUC_lexicon_tags_17 },
    { 0x5bb4ab33, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb4ab39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd4ab3c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef4ab40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ef4ab42, SUC_lexicon_tags_118 },
    { 0x4f54ab42, SUC_lexicon_tags_46 },
    { 0xccb4ab44, SUC_lexicon_tags_29 },
    { 0x4934ab42, SUC_lexicon_tags_65 },
    { 0xfcd4ab46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe814ab49, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4694ab4f, SUC_lexicon_tags_17 },
    { 0x6c94ab50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b4ab56, SUC_lexicon_tags_11 },
    { 0x93f4ab56, SUC_lexicon_tags_1 },
    { 0x34d4ab56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9574ab5c, SUC_lexicon_tags_15 },
    { 0x4354ab5d, SUC_lexicon_tags_82 },
    { 0x63b4ab5e, SUC_lexicon_tags_40 },
    { 0x41b4ab5f, SUC_lexicon_tags_47 },
    { 0xb834ab60, SUC_lexicon_tags_7 },
    { 0x4394ab5f, SUC_lexicon_tags_17 },
    { 0xa354ab62, SUC_lexicon_tags_37 },
    { 0x6cf4ab5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe634ab67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f14ab69, SUC_lexicon_tags_65 },
    { 0x75d4ab6a, SUC_lexicon_tags_37 },
    { 0x2534ab69, SUC_lexicon_tags_25 },
    { 0xaed4ab6c, SUC_lexicon_tags_160 },
    { 0x72b4ab6c, SUC_lexicon_tags_60 },
    { 0x8514ab6e, SUC_lexicon_tags_46 },
    { 0xe414ab69, SUC_lexicon_tags_11 },
    { 0xc494ab6f, SUC_lexicon_tags_10 },
    { 0x794ab71, SUC_lexicon_tags_111 },
    { 0xf4b4ab72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x794ab78, SUC_lexicon_tags_12 },
    { 0xcb4ab79, SUC_lexicon_tags_39 },
    { 0x9314ab7a, SUC_lexicon_tags_2 },
    { 0x35f4ab7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6894ab7e, SUC_lexicon_tags_7 },
    { 0x31f4ab7f, SUC_lexicon_tags_16 },
    { 0x42d4ab7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34ab8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1974ab8d, SUC_lexicon_tags_27 },
    { 0x16b4ab8e, SUC_lexicon_tags_62 },
    { 0x2d4ab8f, SUC_lexicon_tags_60 },
    { 0xbcb4ab90, SUC_lexicon_tags_0 },
    { 0xa754ab8f, SUC_lexicon_tags_25 },
    { 0xbbf4ab91, SUC_lexicon_tags_21 },
    { 0xfa34ab93, SUC_lexicon_tags_43 },
    { 0x68b4ab94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb654ab98, SUC_lexicon_tags_45 },
    { 0x10d4ab98, SUC_lexicon_tags_2 },
    { 0xc3d4ab9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e34aba2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe634aba4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb4abac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7c54abae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d4abb2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4abba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc1b4abbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xef34abbf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d94abc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf174abcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3b4abcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa994abd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc794abdc, SUC_lexicon_tags_28 },
    { 0xfd94abdc, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xbe14abdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d4abe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd4abe9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18d4abeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2b4abed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d94abef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2734abf1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f4abf4, SUC_lexicon_tags_498 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34abf9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f94abfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9134abfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e34ac04, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x134ac07, SUC_lexicon_tags_2 },
    { 0xc2b4ac07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4ac10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d4ac14, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x85d4ac16, SUC_lexicon_tags_37 },
    { 0xb9d4ac16, SUC_lexicon_tags_1275 },
    { 0x38b4ac18, SUC_lexicon_tags_10 },
    { 0x7354ac19, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5134ac1c, SUC_lexicon_tags_39 },
    { 0x5c74ac1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d4ac20, SUC_lexicon_tags_15 },
    { 0xe454ac21, SUC_lexicon_tags_27 },
    { 0x10b4ac22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f4ac26, SUC_lexicon_tags_27 },
    { 0x65d4ac27, SUC_lexicon_tags_38 },
    { 0xcbd4ac27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8634ac2d, SUC_lexicon_tags_12 },
    { 0x2814ac2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7554ac30, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcfd4ac32, SUC_lexicon_tags_3 },
    { 0x23b4ac33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd094ac36, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4ac3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb94ac40, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4d14ac42, SUC_lexicon_tags_37 },
    { 0xf354ac43, SUC_lexicon_tags_11 },
    { 0xabf4ac44, SUC_lexicon_tags_45 },
    { 0xb1f4ac44, SUC_lexicon_tags_25 },
    { 0xf714ac43, SUC_lexicon_tags_29 },
    { 0x94ac44, SUC_lexicon_tags_11 },
    { 0x3654ac44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe54ac50, SUC_lexicon_tags_12 },
    { 0x87d4ac51, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d4ac56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfb34ac58, SUC_lexicon_tags_25 },
    { 0x8c34ac58, SUC_lexicon_tags_30 },
    { 0xb2d4ac5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6b4ac5c, SUC_lexicon_tags_7 },
    { 0xe54ac5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb4ac60, SUC_lexicon_tags_12 },
    { 0x74b4ac60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaf14ac63, SUC_lexicon_tags_8 },
    { 0xd214ac63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1174ac67, SUC_lexicon_tags_11 },
    { 0xb0f4ac68, SUC_lexicon_tags_17 },
    { 0x4d94ac67, SUC_lexicon_tags_46 },
    { 0x2814ac68, SUC_lexicon_tags_87 },
    { 0xaf54ac6b, SUC_lexicon_tags_37 },
    { 0xc614ac6a, SUC_lexicon_tags_12 },
    { 0x62d4ac68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2774ac70, SUC_lexicon_tags_47 },
    { 0x5e54ac71, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcaf4ac73, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a74ac79, SUC_lexicon_tags_3 },
    { 0x8494ac7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb7d4ac7c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6d74ac7e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2ab4ac80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f4ac83, SUC_lexicon_tags_114 },
    { 0xba54ac84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f4ac8a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xe914ac8c, SUC_lexicon_tags_15 },
    { 0xff34ac8d, SUC_lexicon_tags_9 },
    { 0x5494ac8d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b4ac92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9d4ac94, SUC_lexicon_tags_17 },
    { 0x5cd4ac94, SUC_lexicon_tags_19 },
    { 0x9d14ac94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd554ac99, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f4aca2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb714aca4, SUC_lexicon_tags_28 },
    { 0xa254aca5, SUC_lexicon_tags_63 },
    { 0xcc14aca6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e54aca8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd94acaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b4acad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f4acb0, SUC_lexicon_tags_25 },
    { 0x6d14acb1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf954acb7, SUC_lexicon_tags_47 },
    { 0xdf74acb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee34acc2, SUC_lexicon_tags_19 },
    { 0xdc94acc2, SUC_lexicon_tags_7 },
    { 0x38d4acc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee74acc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16f4acc8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef54acca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1914accd, SUC_lexicon_tags_79 },
    { 0xaa54acce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe854acd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b4acd5, SUC_lexicon_tags_15 },
    { 0x7594acd6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad4acdd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1934ace4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d4ace8, SUC_lexicon_tags_84 },
    { 0x9614ace9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4c14aceb, SUC_lexicon_tags_14 },
    { 0xe34acec, SUC_lexicon_tags_47 },
    { 0xe94aceb, SUC_lexicon_tags_25 },
    { 0x3254acee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7994acf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9574acf8, SUC_lexicon_tags_1 },
    { 0x7ef4acf9, SUC_lexicon_tags_7 },
    { 0xc6b4acf8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe714acfe, SUC_lexicon_tags_7 },
    { 0x7b34acfe, SUC_lexicon_tags_17 },
    { 0x45d4ad00, SUC_lexicon_tags_84 },
    { 0xbd94ad01, SUC_lexicon_tags_7 },
    { 0xa4b4ad01, SUC_lexicon_tags_19 },
    { 0x8474acff, SUC_lexicon_tags_4 },
    { 0xa4d4ad04, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd454ad0b, SUC_lexicon_tags_60 },
    { 0xbe34ad0c, SUC_lexicon_tags_11 },
    { 0x1af4ad0d, SUC_lexicon_tags_1 },
    { 0xbb94ad0e, SUC_lexicon_tags_49 },
    { 0x5c54ad0f, SUC_lexicon_tags_17 },
    { 0x13f4ad0f, SUC_lexicon_tags_27 },
    { 0xb834ad11, SUC_lexicon_tags_7 },
    { 0xf594ad11, SUC_lexicon_tags_104 },
    { 0x5f54ad11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7b4ad15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a94ad1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d74ad21, SUC_lexicon_tags_83 },
    { 0xdc14ad22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb034ad24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454ad2c, SUC_lexicon_tags_46 },
    { 0xd9f4ad2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d4ad32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81d4ad34, SUC_lexicon_tags_32 },
    { 0x854ad35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8354ad3a, SUC_lexicon_tags_29 },
    { 0x8c94ad3a, SUC_lexicon_tags_37 },
    { 0xc254ad3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c34ad3e, SUC_lexicon_tags_13 },
    { 0xe7d4ad3e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6d14ad41, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f4ad49, SUC_lexicon_tags_38 },
    { 0xc914ad4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb194ad4e, SUC_lexicon_tags_47 },
    { 0xb394ad4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe454ad51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b4ad55, SUC_lexicon_tags_8 },
    { 0x3e14ad55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd674ad59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc574ad5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f54ad5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x154ad61, SUC_lexicon_tags_14 },
    { 0xa6d4ad62, SUC_lexicon_tags_37 },
    { 0x21b4ad63, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f14ad68, SUC_lexicon_tags_11 },
    { 0x20f4ad69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa454ad6c, SUC_lexicon_tags_15 },
    { 0xd9f4ad6c, SUC_lexicon_tags_7 },
    { 0xd354ad6c, SUC_lexicon_tags_23 },
    { 0x3894ad6d, SUC_lexicon_tags_37 },
    { 0x6874ad70, SUC_lexicon_tags_44 },
    { 0x1274ad71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d14ad73, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6694ad76, SUC_lexicon_tags_12 },
    { 0xb494ad77, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c34ad7d, SUC_lexicon_tags_29 },
    { 0x36d4ad7e, SUC_lexicon_tags_8 },
    { 0x6134ad7f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb34ad8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f4ad8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e34ad95, SUC_lexicon_tags_12 },
    { 0x9534ad96, SUC_lexicon_tags_1 },
    { 0x7034ad97, SUC_lexicon_tags_29 },
    { 0x6854ad97, SUC_lexicon_tags_414 },
    { 0x514ad96, SUC_lexicon_tags_14 },
    { 0x1794ad95, SUC_lexicon_tags_17 },
    { 0x55d4ad9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa454ada0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab4ada6, SUC_lexicon_tags_2 },
    { 0xc574ada7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xba54adaa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb4adb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb4adb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9654adbd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc794adbf, SUC_lexicon_tags_46 },
    { 0x5a34adc0, SUC_lexicon_tags_25 },
    { 0x28d4adc1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda94adc5, SUC_lexicon_tags_40 },
    { 0x8894adc6, SUC_lexicon_tags_15 },
    { 0x3314adc6, SUC_lexicon_tags_53 },
    { 0x8534adc8, SUC_lexicon_tags_17 },
    { 0x1e54adc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1354adcb, SUC_lexicon_tags_84 },
    { 0xdbb4adcc, SUC_lexicon_tags_26 },
    { 0x8b4adcd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf6b4adcf, SUC_lexicon_tags_57 },
    { 0x57d4adcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef54add2, SUC_lexicon_tags_47 },
    { 0x3114add3, SUC_lexicon_tags_29 },
    { 0x6814add3, SUC_lexicon_tags_7 },
    { 0x6b94add3, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x2234add8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4534adde, SUC_lexicon_tags_17 },
    { 0x1c14addf, SUC_lexicon_tags_2 },
    { 0xd634ade0, SUC_lexicon_tags_63 },
    { 0xabb4ade0, SUC_lexicon_tags_30 },
    { 0x1c94ade2, SUC_lexicon_tags_68 },
    { 0x7334addf, SUC_lexicon_tags_46 },
    { 0x99f4ade1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x354ade7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd074ade9, SUC_lexicon_tags_8 },
    { 0xfb14adea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2794adf0, SUC_lexicon_tags_37 },
    { 0x43f4adf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x134adf3, SUC_lexicon_tags_12 },
    { 0x5c74adf4, SUC_lexicon_tags_68 },
    { 0x8c14adf4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa194adfc, SUC_lexicon_tags_63 },
    { 0xe874adfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b4ae09, SUC_lexicon_tags_33 },
    { 0x2dd4ae0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54ae0e, SUC_lexicon_tags_37 },
    { 0xc154ae0f, SUC_lexicon_tags_11 },
    { 0x4b14ae10, SUC_lexicon_tags_26 },
    { 0xc34ae0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42d4ae13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb994ae1b, SUC_lexicon_tags_27 },
    { 0x9294ae1c, SUC_lexicon_tags_49 },
    { 0xaf4ae1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8994ae21, SUC_lexicon_tags_37 },
    { 0x1b94ae21, SUC_lexicon_tags_11 },
    { 0xa6d4ae23, SUC_lexicon_tags_12 },
    { 0xff34ae24, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f4ae2b, SUC_lexicon_tags_7 },
    { 0x30b4ae2c, SUC_lexicon_tags_61 },
    { 0x7594ae2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad4ae31, SUC_lexicon_tags_70 },
    { 0x3cb4ae31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed4ae37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab34ae42, SUC_lexicon_tags_10 },
    { 0xcb14ae43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc514ae4f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf14ae52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d4ae55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe614ae5a, SUC_lexicon_tags_25 },
    { 0x1394ae5b, SUC_lexicon_tags_14 },
    { 0x80d4ae5c, SUC_lexicon_tags_0 },
    { 0x5754ae5c, SUC_lexicon_tags_63 },
    { 0x74d4ae5e, SUC_lexicon_tags_1 },
    { 0x5e14ae5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f4ae63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d4ae66, SUC_lexicon_tags_25 },
    { 0x35d4ae66, SUC_lexicon_tags_26 },
    { 0x9b34ae68, SUC_lexicon_tags_15 },
    { 0xfed4ae68, SUC_lexicon_tags_28 },
    { 0x5074ae67, SUC_lexicon_tags_7 },
    { 0xb3f4ae6b, SUC_lexicon_tags_8 },
    { 0x8014ae6b, SUC_lexicon_tags_25 },
    { 0x98b4ae6b, SUC_lexicon_tags_33 },
    { 0x4174ae6e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe674ae70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa634ae73, SUC_lexicon_tags_45 },
    { 0x12d4ae74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d14ae76, SUC_lexicon_tags_11 },
    { 0xc754ae76, SUC_lexicon_tags_11 },
    { 0x9434ae76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3dd4ae7a, SUC_lexicon_tags_29 },
    { 0xfd34ae7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf94ae80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc54ae82, SUC_lexicon_tags_37 },
    { 0xdd14ae83, SUC_lexicon_tags_0 },
    { 0x174ae84, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd4ae88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d4ae8b, SUC_lexicon_tags_45 },
    { 0xe734ae8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1254ae8f, SUC_lexicon_tags_6 },
    { 0x3cd4ae90, SUC_lexicon_tags_25 },
    { 0xdf54ae8f, SUC_lexicon_tags_25 },
    { 0x5274ae92, SUC_lexicon_tags_48 },
    { 0xbad4ae93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1554ae95, SUC_lexicon_tags_43 },
    { 0xc1f4ae95, SUC_lexicon_tags_11 },
    { 0x7f14ae95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae94ae9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb894ae9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae94aea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f94aea3, SUC_lexicon_tags_15 },
    { 0x4634aea4, SUC_lexicon_tags_15 },
    { 0xdef4aea4, SUC_lexicon_tags_21 },
    { 0x9f54aea4, SUC_lexicon_tags_7 },
    { 0x2274aea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa94aeac, SUC_lexicon_tags_25 },
    { 0x9ef4aead, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b4aeb0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5174aeb6, SUC_lexicon_tags_21 },
    { 0xfc94aeb7, SUC_lexicon_tags_15 },
    { 0x4d14aeb8, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x8e34aeba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3574aebc, SUC_lexicon_tags_27 },
    { 0x5674aebc, SUC_lexicon_tags_47 },
    { 0x5274aebe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf4aec1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb4aec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab4aec8, SUC_lexicon_tags_7 },
    { 0x91d4aec9, SUC_lexicon_tags_12 },
    { 0x6d74aeca, SUC_lexicon_tags_12 },
    { 0xe454aec9, SUC_lexicon_tags_25 },
    { 0x2754aecc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3074aed1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xca34aed3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb4aed9, SUC_lexicon_tags_31 },
    { 0x2bf4aeda, SUC_lexicon_tags_7 },
    { 0x8b54aeda, SUC_lexicon_tags_49 },
    { 0x9794aedc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5394aee0, SUC_lexicon_tags_17 },
    { 0xe9b4aee0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4214aee3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1654aeea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e34aeef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1f4aef1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff4aef4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87d4aef6, SUC_lexicon_tags_30 },
    { 0xd794aef7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8174aefa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ad4aefc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb34aeff, SUC_lexicon_tags_12 },
    { 0x6dd4af00, SUC_lexicon_tags_31 },
    { 0x1754af01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a54af04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15d4af06, SUC_lexicon_tags_23 },
    { 0x9ed4af06, SUC_lexicon_tags_37 },
    { 0xc514af06, SUC_lexicon_tags_15 },
    { 0xa094af09, SUC_lexicon_tags_26 },
    { 0x9f74af0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4af0d, SUC_lexicon_tags_1 },
    { 0xf594af0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82b4af10, SUC_lexicon_tags_31 },
    { 0xf8f4af11, SUC_lexicon_tags_48 },
    { 0xe7b4af10, SUC_lexicon_tags_14 },
    { 0x4a34af13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb74af15, SUC_lexicon_tags_37 },
    { 0x2e74af16, SUC_lexicon_tags_17 },
    { 0x7e14af17, SUC_lexicon_tags_12 },
    { 0x1d14af16, SUC_lexicon_tags_37 },
    { 0x2e14af19, SUC_lexicon_tags_51 },
    { 0x2d74af19, SUC_lexicon_tags_37 },
    { 0xf1f4af1b, SUC_lexicon_tags_16 },
    { 0xd634af19, SUC_lexicon_tags_19 },
    { 0x29d4af16, SUC_lexicon_tags_37 },
    { 0xcc94af1e, SUC_lexicon_tags_17 },
    { 0xf734af1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b4af24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf114af28, SUC_lexicon_tags_30 },
    { 0x6554af29, SUC_lexicon_tags_37 },
    { 0xbbf4af29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5f4af2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f4af2f, SUC_lexicon_tags_33 },
    { 0x334af2f, SUC_lexicon_tags_45 },
    { 0x9814af31, SUC_lexicon_tags_12 },
    { 0xc674af32, SUC_lexicon_tags_29 },
    { 0x2d94af33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b4af39, SUC_lexicon_tags_15 },
    { 0x9974af3a, SUC_lexicon_tags_4 },
    { 0x7654af39, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b4af45, SUC_lexicon_tags_39 },
    { 0x1474af46, SUC_lexicon_tags_1 },
    { 0x1614af46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab4af4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc354af4f, SUC_lexicon_tags_13 },
    { 0xdb94af50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe214af54, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4654af66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c14af69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2134af77, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a74af7e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf294af80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c74af82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50d4af84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d4af8a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2034af8d, SUC_lexicon_tags_38 },
    { 0xa374af8e, SUC_lexicon_tags_11 },
    { 0x4c14af8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x73b4af91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b4af9d, SUC_lexicon_tags_77 },
    { 0x6194af9d, SUC_lexicon_tags_15 },
    { 0xf8b4af9e, SUC_lexicon_tags_30 },
    { 0x22d4af9e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4afa5, SUC_lexicon_tags_31 },
    { 0xd9b4afa5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f4afad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a74afb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9154afb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7894afb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ff4afb8, SUC_lexicon_tags_30 },
    { 0x3dd4afb8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x13f4afbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f94afbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff4afc3, SUC_lexicon_tags_2 },
    { 0x6bf4afc4, SUC_lexicon_tags_11 },
    { 0xe694afc3, SUC_lexicon_tags_66 },
    { 0xbfd4afc6, SUC_lexicon_tags_55 },
    { 0x1f4afc7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x30d4afc9, SUC_lexicon_tags_25 },
    { 0xc454afc9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b34afd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e34afd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b14afd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e94afda, SUC_lexicon_tags_37 },
    { 0x5a34afdb, SUC_lexicon_tags_31 },
    { 0xb774afdb, SUC_lexicon_tags_1 },
    { 0xb054afdd, SUC_lexicon_tags_0 },
    { 0x3634afdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7374afe2, SUC_lexicon_tags_33 },
    { 0x1f4afe2, SUC_lexicon_tags_17 },
    { 0x94afe4, SUC_lexicon_tags_1 },
    { 0xb0d4afe5, SUC_lexicon_tags_46 },
    { 0x92f4afe6, SUC_lexicon_tags_29 },
    { 0xcdd4afe5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9674afeb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf74aff0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad14aff9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6614affb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7354affe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2f4b000, SUC_lexicon_tags_37 },
    { 0x1714b000, SUC_lexicon_tags_11 },
    { 0xd834b002, SUC_lexicon_tags_8 },
    { 0x8434b000, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2134b00a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4814b00c, SUC_lexicon_tags_29 },
    { 0xcff4b00d, SUC_lexicon_tags_45 },
    { 0x3cd4b00e, SUC_lexicon_tags_68 },
    { 0x694b00f, SUC_lexicon_tags_30 },
    { 0x2394b00f, SUC_lexicon_tags_17 },
    { 0x9a34b00d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94b017, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c14b01b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x3c34b01d, SUC_lexicon_tags_31 },
    { 0xa594b01e, SUC_lexicon_tags_6 },
    { 0xff54b01f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd4b022, SUC_lexicon_tags_7 },
    { 0x6654b023, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f14b026, SUC_lexicon_tags_27 },
    { 0xab4b027, SUC_lexicon_tags_2 },
    { 0xb354b028, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef14b02a, SUC_lexicon_tags_1 },
    { 0x6b4b02b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a74b02e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2794b030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb4b035, SUC_lexicon_tags_7 },
    { 0x6a54b036, SUC_lexicon_tags_17 },
    { 0xabf4b036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b94b039, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b4b03f, SUC_lexicon_tags_97 },
    { 0x2314b040, SUC_lexicon_tags_8 },
    { 0x1674b041, SUC_lexicon_tags_17 },
    { 0x1af4b03f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf4b046, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f4b049, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b4b050, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf4b053, SUC_lexicon_tags_24 },
    { 0x4df4b054, SUC_lexicon_tags_63 },
    { 0x9314b054, SUC_lexicon_tags_11 },
    { 0xfd54b056, SUC_lexicon_tags_23 },
    { 0x4d94b057, SUC_lexicon_tags_9 },
    { 0x8354b054, SUC_lexicon_tags_0 },
    { 0xd6f4b059, SUC_lexicon_tags_17 },
    { 0xa2d4b05a, SUC_lexicon_tags_44 },
    { 0x8f14b055, SUC_lexicon_tags_2 },
    { 0x33f4b05c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd74b05f, SUC_lexicon_tags_38 },
    { 0x5494b05f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d4b064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d4b067, SUC_lexicon_tags_15 },
    { 0x5334b068, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe0d4b06a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb4b06d, SUC_lexicon_tags_42 },
    { 0xa2f4b06d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14b075, SUC_lexicon_tags_0 },
    { 0x6194b075, SUC_lexicon_tags_65 },
    { 0x534b077, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef4b07d, SUC_lexicon_tags_37 },
    { 0x4414b07e, SUC_lexicon_tags_11 },
    { 0x5614b07f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf774b085, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbdd4b087, SUC_lexicon_tags_25 },
    { 0xb994b087, SUC_lexicon_tags_38 },
    { 0x1d94b089, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc494b08b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b4b090, SUC_lexicon_tags_1 },
    { 0x6354b091, SUC_lexicon_tags_1 },
    { 0x80b4b090, SUC_lexicon_tags_68 },
    { 0x80f4b090, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f4b096, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6134b09a, SUC_lexicon_tags_15 },
    { 0xcc74b09b, SUC_lexicon_tags_1 },
    { 0x9914b09b, SUC_lexicon_tags_47 },
    { 0xec34b09d, SUC_lexicon_tags_46 },
    { 0x63f4b09e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a14b0a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f4b0a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9094b0ab, SUC_lexicon_tags_19 },
    { 0x1fd4b0ab, SUC_lexicon_tags_25 },
    { 0x7694b0ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe5b4b0af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4b0b6, SUC_lexicon_tags_65 },
    { 0xd9b4b0b7, SUC_lexicon_tags_17 },
    { 0x1ab4b0b8, SUC_lexicon_tags_15 },
    { 0x10d4b0b9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf4b0c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd4b0c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x99f4b0c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc334b0cb, SUC_lexicon_tags_15 },
    { 0x7554b0cc, SUC_lexicon_tags_87 },
    { 0x8cb4b0cd, SUC_lexicon_tags_1 },
    { 0x4794b0ce, SUC_lexicon_tags_17 },
    { 0xd8b4b0cd, SUC_lexicon_tags_154 },
    { 0x74b0cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f4b0d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b34b0d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4814b0d9, SUC_lexicon_tags_2 },
    { 0x1b4b0da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6374b0dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3534b0e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa34b0eb, SUC_lexicon_tags_23 },
    { 0xdbf4b0ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf874b0ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b74b0f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd054b0f3, SUC_lexicon_tags_17 },
    { 0x3c14b0f4, SUC_lexicon_tags_65 },
    { 0xb94b0f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd214b0f8, SUC_lexicon_tags_13 },
    { 0x1074b0f9, SUC_lexicon_tags_25 },
    { 0x694b0f9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd774b0ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d4b10a, SUC_lexicon_tags_1 },
    { 0xf494b10b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d4b10e, SUC_lexicon_tags_79 },
    { 0x1e14b10f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f4b112, SUC_lexicon_tags_27 },
    { 0xc254b113, SUC_lexicon_tags_10 },
    { 0x90b4b114, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x114b117, SUC_lexicon_tags_17 },
    { 0x2e94b118, SUC_lexicon_tags_82 },
    { 0xf854b119, SUC_lexicon_tags_27 },
    { 0x9454b119, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af4b11d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb854b120, SUC_lexicon_tags_37 },
    { 0xa8f4b121, SUC_lexicon_tags_11 },
    { 0x3754b121, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3294b125, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d4b12b, SUC_lexicon_tags_11 },
    { 0x9834b12b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e14b138, SUC_lexicon_tags_65 },
    { 0x4a34b138, SUC_lexicon_tags_49 },
    { 0x91b4b13a, SUC_lexicon_tags_23 },
    { 0x8814b138, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2714b140, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd354b149, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xa1d4b14b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7d4b14d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd934b14f, SUC_lexicon_tags_49 },
    { 0x1814b150, SUC_lexicon_tags_17 },
    { 0x674b14f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6454b154, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd4b161, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb914b164, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb634b169, SUC_lexicon_tags_315 },
    { 0, NULL },
    { 0, NULL },
    { 0x7434b16c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf154b16e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe634b179, SUC_lexicon_tags_38 },
    { 0x2c54b17a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf34b180, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf934b183, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x374b187, SUC_lexicon_tags_17 },
    { 0x7a54b188, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xacd4b18a, SUC_lexicon_tags_12 },
    { 0xe74b18b, SUC_lexicon_tags_13 },
    { 0xba74b18b, SUC_lexicon_tags_15 },
    { 0xd374b18d, SUC_lexicon_tags_12 },
    { 0x6714b18a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa034b191, SUC_lexicon_tags_1 },
    { 0xddd4b192, SUC_lexicon_tags_19 },
    { 0xe7f4b193, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e34b19a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d94b19c, SUC_lexicon_tags_21 },
    { 0xcbd4b19c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd4b1a3, SUC_lexicon_tags_2 },
    { 0x7554b1a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2b4b1a6, SUC_lexicon_tags_38 },
    { 0xe0b4b1a7, SUC_lexicon_tags_38 },
    { 0x24d4b1a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8414b1ad, SUC_lexicon_tags_15 },
    { 0x4914b1ae, SUC_lexicon_tags_45 },
    { 0xd254b1ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8714b1b4, SUC_lexicon_tags_17 },
    { 0x7134b1b4, SUC_lexicon_tags_15 },
    { 0x6ff4b1b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a54b1b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd4b1bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8174b1bf, SUC_lexicon_tags_10 },
    { 0x9054b1c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc034b1c4, SUC_lexicon_tags_9 },
    { 0xcdf4b1c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d4b1c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d4b1cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cf4b1cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa194b1d5, SUC_lexicon_tags_87 },
    { 0xf254b1d6, SUC_lexicon_tags_298 },
    { 0x1df4b1d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4b1da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4b1df, SUC_lexicon_tags_29 },
    { 0x4f54b1e0, SUC_lexicon_tags_27 },
    { 0xd7f4b1e1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6994b1e4, SUC_lexicon_tags_9 },
    { 0xf274b1e4, SUC_lexicon_tags_39 },
    { 0x3474b1e6, SUC_lexicon_tags_17 },
    { 0xa3d4b1e6, SUC_lexicon_tags_39 },
    { 0x80d4b1e4, SUC_lexicon_tags_25 },
    { 0x37b4b1e9, SUC_lexicon_tags_2 },
    { 0x2374b1ea, SUC_lexicon_tags_12 },
    { 0xee54b1eb, SUC_lexicon_tags_1 },
    { 0xdf94b1ec, SUC_lexicon_tags_23 },
    { 0xd054b1e7, SUC_lexicon_tags_45 },
    { 0xa8b4b1ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f14b1f4, SUC_lexicon_tags_7 },
    { 0x97f4b1f5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb574b1f7, SUC_lexicon_tags_15 },
    { 0xe874b1f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d14b200, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x22b4b202, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6774b204, SUC_lexicon_tags_1 },
    { 0xe114b205, SUC_lexicon_tags_65 },
    { 0xe074b204, SUC_lexicon_tags_6 },
    { 0x97f4b206, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c94b20a, SUC_lexicon_tags_5 },
    { 0x6274b20a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af4b20e, SUC_lexicon_tags_37 },
    { 0x23f4b20f, SUC_lexicon_tags_27 },
    { 0xe874b210, SUC_lexicon_tags_12 },
    { 0x8554b210, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe574b214, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2474b216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddf4b218, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb374b21e, SUC_lexicon_tags_35 },
    { 0xa4b4b21e, SUC_lexicon_tags_31 },
    { 0x3ed4b220, SUC_lexicon_tags_8 },
    { 0x4874b221, SUC_lexicon_tags_63 },
    { 0xd5b4b21e, SUC_lexicon_tags_33 },
    { 0xdd54b223, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4b226, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe4f4b228, SUC_lexicon_tags_37 },
    { 0xa6d4b228, SUC_lexicon_tags_32 },
    { 0xcb94b22a, SUC_lexicon_tags_47 },
    { 0x1ff4b22b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a74b22e, SUC_lexicon_tags_15 },
    { 0x8a34b22e, SUC_lexicon_tags_8 },
    { 0xb7d4b230, SUC_lexicon_tags_3 },
    { 0x1d14b22e, SUC_lexicon_tags_33 },
    { 0xb14b22e, SUC_lexicon_tags_129 },
    { 0x53b4b233, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa494b235, SUC_lexicon_tags_8 },
    { 0xb5f4b236, SUC_lexicon_tags_11 },
    { 0xe2f4b237, SUC_lexicon_tags_45 },
    { 0x5c74b238, SUC_lexicon_tags_43 },
    { 0x3094b235, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe054b23b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9254b23d, SUC_lexicon_tags_69 },
    { 0x1014b23d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8514b241, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa094b244, SUC_lexicon_tags_63 },
    { 0x7194b244, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3994b24f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15d4b251, SUC_lexicon_tags_45 },
    { 0xd534b251, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b4b255, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74b25c, SUC_lexicon_tags_27 },
    { 0xea54b25c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb14b261, SUC_lexicon_tags_16 },
    { 0xe4b4b262, SUC_lexicon_tags_37 },
    { 0xd4d4b263, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc14b267, SUC_lexicon_tags_8 },
    { 0x5f74b268, SUC_lexicon_tags_17 },
    { 0x3754b269, SUC_lexicon_tags_6 },
    { 0x2b14b269, SUC_lexicon_tags_14 },
    { 0x6634b267, SUC_lexicon_tags_12 },
    { 0xbdf4b26c, SUC_lexicon_tags_43 },
    { 0xe9d4b26d, SUC_lexicon_tags_37 },
    { 0xa074b26e, SUC_lexicon_tags_23 },
    { 0xedf4b26c, SUC_lexicon_tags_39 },
    { 0x66b4b270, SUC_lexicon_tags_1 },
    { 0x6174b270, SUC_lexicon_tags_11 },
    { 0xbd4b267, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab4b275, SUC_lexicon_tags_34 },
    { 0x8974b276, SUC_lexicon_tags_31 },
    { 0xbf14b276, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x574b280, SUC_lexicon_tags_19 },
    { 0x8d54b281, SUC_lexicon_tags_37 },
    { 0xc634b282, SUC_lexicon_tags_11 },
    { 0xe214b282, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f4b287, SUC_lexicon_tags_2 },
    { 0x5074b288, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b4b28f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x5974b291, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3154b294, SUC_lexicon_tags_25 },
    { 0x3374b295, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2674b29a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d4b29c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1494b2a7, SUC_lexicon_tags_161 },
    { 0x37b4b2a8, SUC_lexicon_tags_11 },
    { 0xba34b2a8, SUC_lexicon_tags_46 },
    { 0x20b4b2a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd54b2ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6074b2b0, SUC_lexicon_tags_30 },
    { 0x3a14b2b0, SUC_lexicon_tags_8 },
    { 0xd5f4b2b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9174b2b7, SUC_lexicon_tags_17 },
    { 0x7f4b2b7, SUC_lexicon_tags_18 },
    { 0x40f4b2b7, SUC_lexicon_tags_11 },
    { 0xf0f4b2ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e14b2bf, SUC_lexicon_tags_23 },
    { 0x70f4b2c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb134b2c2, SUC_lexicon_tags_47 },
    { 0xdc34b2c2, SUC_lexicon_tags_2 },
    { 0xc7d4b2c4, SUC_lexicon_tags_11 },
    { 0x1654b2c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f4b2c7, SUC_lexicon_tags_44 },
    { 0x1434b2c7, SUC_lexicon_tags_33 },
    { 0x3fb4b2c7, SUC_lexicon_tags_12 },
    { 0x9794b2ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9354b2cc, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb4b2d4, SUC_lexicon_tags_43 },
    { 0x394b2d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf4b2df, SUC_lexicon_tags_25 },
    { 0x9354b2df, SUC_lexicon_tags_4 },
    { 0x9f54b2e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fd4b2e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2034b2ea, SUC_lexicon_tags_25 },
    { 0xf674b2eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4254b2f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f4b2f3, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x5a74b2f5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc154b2fa, SUC_lexicon_tags_28 },
    { 0xb9b4b2fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd9d4b2fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e74b300, SUC_lexicon_tags_16 },
    { 0x3694b301, SUC_lexicon_tags_66 },
    { 0x2b74b300, SUC_lexicon_tags_12 },
    { 0xb8d4b300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa434b305, SUC_lexicon_tags_18 },
    { 0x5b4b306, SUC_lexicon_tags_12 },
    { 0xc394b307, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72f4b309, SUC_lexicon_tags_63 },
    { 0x4834b309, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8654b30e, SUC_lexicon_tags_37 },
    { 0x97b4b30e, SUC_lexicon_tags_47 },
    { 0x55d4b30f, SUC_lexicon_tags_44 },
    { 0x2954b311, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd4b318, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d4b31b, SUC_lexicon_tags_7 },
    { 0xe414b31b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca54b320, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f4b323, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d4b326, SUC_lexicon_tags_29 },
    { 0xa9d4b326, SUC_lexicon_tags_8 },
    { 0xa9d4b328, SUC_lexicon_tags_12 },
    { 0xdbf4b328, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a54b32b, SUC_lexicon_tags_29 },
    { 0x2e14b32c, SUC_lexicon_tags_2 },
    { 0x35f4b32d, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0xd154b330, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b4b333, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8e74b335, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc74b339, SUC_lexicon_tags_43 },
    { 0x634b33a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d4b342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4714b344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4b349, SUC_lexicon_tags_15 },
    { 0xe394b34a, SUC_lexicon_tags_3 },
    { 0x50f4b34a, SUC_lexicon_tags_12 },
    { 0xd4f4b34a, SUC_lexicon_tags_2 },
    { 0xaa54b34d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcff4b34f, SUC_lexicon_tags_25 },
    { 0x7d34b34f, SUC_lexicon_tags_37 },
    { 0x1d74b34f, SUC_lexicon_tags_17 },
    { 0xf7d4b34f, SUC_lexicon_tags_1 },
    { 0x3414b353, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b4b357, SUC_lexicon_tags_25 },
    { 0xc154b358, SUC_lexicon_tags_37 },
    { 0xce14b358, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb4b360, SUC_lexicon_tags_37 },
    { 0x1ab4b361, SUC_lexicon_tags_45 },
    { 0xda74b360, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1754b366, SUC_lexicon_tags_12 },
    { 0xfdf4b366, SUC_lexicon_tags_0 },
    { 0x7934b366, SUC_lexicon_tags_8 },
    { 0x4014b366, SUC_lexicon_tags_17 },
    { 0x8534b367, SUC_lexicon_tags_23 },
    { 0x5094b366, SUC_lexicon_tags_43 },
    { 0xd14b36c, SUC_lexicon_tags_25 },
    { 0x2e14b368, SUC_lexicon_tags_12 },
    { 0xce54b36e, SUC_lexicon_tags_29 },
    { 0x6db4b36c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4b378, SUC_lexicon_tags_8 },
    { 0x1a14b379, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7794b37b, SUC_lexicon_tags_33 },
    { 0xb7b4b37c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x514b37f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f4b387, SUC_lexicon_tags_14 },
    { 0x4f14b388, SUC_lexicon_tags_37 },
    { 0xeff4b389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d4b38d, SUC_lexicon_tags_25 },
    { 0x2c34b38e, SUC_lexicon_tags_25 },
    { 0x7b14b38d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb354b397, SUC_lexicon_tags_25 },
    { 0x6174b398, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df4b3a0, SUC_lexicon_tags_11 },
    { 0xf074b3a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d4b3ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e94b3af, SUC_lexicon_tags_38 },
    { 0xfc74b3b0, SUC_lexicon_tags_32 },
    { 0xd6d4b3b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7d4b3b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b4b3b9, SUC_lexicon_tags_7 },
    { 0xe734b3ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4734b3bc, SUC_lexicon_tags_37 },
    { 0x8674b3bd, SUC_lexicon_tags_7 },
    { 0x2af4b3be, SUC_lexicon_tags_29 },
    { 0x4af4b3bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4b3c3, SUC_lexicon_tags_28 },
    { 0xe154b3c4, SUC_lexicon_tags_63 },
    { 0xc5b4b3c5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b4b3cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d4b3ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b4b3d2, SUC_lexicon_tags_47 },
    { 0x3514b3d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xded4b3d5, SUC_lexicon_tags_29 },
    { 0x9694b3d5, SUC_lexicon_tags_4 },
    { 0xf0f4b3d7, SUC_lexicon_tags_17 },
    { 0xd094b3d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4454b3df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e14b3e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f4b3e7, SUC_lexicon_tags_2 },
    { 0xd474b3e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6bb4b3ea, SUC_lexicon_tags_51 },
    { 0xe4d4b3eb, SUC_lexicon_tags_92 },
    { 0x54b3ea, SUC_lexicon_tags_46 },
    { 0x2394b3ea, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e94b3f5, SUC_lexicon_tags_46 },
    { 0x67b4b3f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf294b3fa, SUC_lexicon_tags_6 },
    { 0x79b4b3fb, SUC_lexicon_tags_11 },
    { 0x9f94b3fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d4b3ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b4b409, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6274b40d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb254b410, SUC_lexicon_tags_17 },
    { 0x2974b410, SUC_lexicon_tags_11 },
    { 0x8bd4b412, SUC_lexicon_tags_37 },
    { 0xa1d4b413, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5eb4b415, SUC_lexicon_tags_81 },
    { 0x7c34b416, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x454b418, SUC_lexicon_tags_30 },
    { 0x8754b419, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e34b41b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88d4b41d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f4b421, SUC_lexicon_tags_12 },
    { 0x4714b421, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7b4b424, SUC_lexicon_tags_17 },
    { 0x2b34b425, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa194b427, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5594b42c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3dd4b42e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b94b432, SUC_lexicon_tags_41 },
    { 0xc114b433, SUC_lexicon_tags_14 },
    { 0x5df4b433, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8254b436, SUC_lexicon_tags_6 },
    { 0xaa74b436, SUC_lexicon_tags_10 },
    { 0xd054b438, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19f4b43a, SUC_lexicon_tags_6 },
    { 0xe894b43b, SUC_lexicon_tags_37 },
    { 0x46d4b43c, SUC_lexicon_tags_33 },
    { 0x5d14b43d, SUC_lexicon_tags_8 },
    { 0x7f14b43c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1274b440, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe774b442, SUC_lexicon_tags_11 },
    { 0x2a74b442, SUC_lexicon_tags_12 },
    { 0x6e74b444, SUC_lexicon_tags_25 },
    { 0x2094b444, SUC_lexicon_tags_30 },
    { 0x5e54b446, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c54b449, SUC_lexicon_tags_7 },
    { 0xd494b44a, SUC_lexicon_tags_17 },
    { 0xe8b4b44b, SUC_lexicon_tags_19 },
    { 0x1174b44c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x794b44e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe714b451, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb74b458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba74b45a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc974b461, SUC_lexicon_tags_14 },
    { 0x70b4b462, SUC_lexicon_tags_2 },
    { 0x6574b461, SUC_lexicon_tags_17 },
    { 0xa274b461, SUC_lexicon_tags_15 },
    { 0xad14b465, SUC_lexicon_tags_2 },
    { 0x7174b466, SUC_lexicon_tags_15 },
    { 0xdf74b465, SUC_lexicon_tags_15 },
    { 0x1394b463, SUC_lexicon_tags_2 },
    { 0x9854b469, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe014b46c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d4b471, SUC_lexicon_tags_37 },
    { 0x7534b471, SUC_lexicon_tags_17 },
    { 0x1994b471, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8f14b475, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b94b479, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74b47e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45d4b480, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b4b484, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c74b486, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f4b48a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd714b48e, SUC_lexicon_tags_58 },
    { 0xa534b48f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b4b492, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd094b494, SUC_lexicon_tags_26 },
    { 0xc54b495, SUC_lexicon_tags_68 },
    { 0x2ab4b496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8034b4a1, SUC_lexicon_tags_15 },
    { 0xdd54b4a2, SUC_lexicon_tags_7 },
    { 0xde14b4a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x59b4b4a5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4454b4a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b4b4ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0f4b4ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4834b4b2, SUC_lexicon_tags_27 },
    { 0x5554b4b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe074b4b6, SUC_lexicon_tags_87 },
    { 0xfd4b4b6, SUC_lexicon_tags_81 },
    { 0x3754b4b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb134b4ba, SUC_lexicon_tags_12 },
    { 0xd174b4bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff4b4be, SUC_lexicon_tags_1 },
    { 0xda94b4bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3274b4ce, SUC_lexicon_tags_15 },
    { 0x5694b4cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x434b4d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc914b4d3, SUC_lexicon_tags_19 },
    { 0xb8b4b4d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac94b4d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x74d4b4da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d4b4de, SUC_lexicon_tags_15 },
    { 0xf714b4df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d4b4e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b34b4e6, SUC_lexicon_tags_8 },
    { 0xea54b4e7, SUC_lexicon_tags_7 },
    { 0x7654b4e8, SUC_lexicon_tags_15 },
    { 0x5b54b4e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1254b4fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9cf4b4fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec54b4fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc34b50e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4b511, SUC_lexicon_tags_46 },
    { 0x1f14b512, SUC_lexicon_tags_13 },
    { 0x234b513, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8854b515, SUC_lexicon_tags_46 },
    { 0x5eb4b515, SUC_lexicon_tags_63 },
    { 0x4e54b516, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfef4b519, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a54b51b, SUC_lexicon_tags_17 },
    { 0xfcd4b51c, SUC_lexicon_tags_31 },
    { 0xf0d4b51d, SUC_lexicon_tags_28 },
    { 0xc1f4b51d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcd4b520, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe94b527, SUC_lexicon_tags_8 },
    { 0xa2f4b528, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf4b52c, SUC_lexicon_tags_12 },
    { 0x21f4b52d, SUC_lexicon_tags_55 },
    { 0x3d34b52e, SUC_lexicon_tags_29 },
    { 0x2054b52e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b4b532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f4b535, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1fb4b537, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b4b539, SUC_lexicon_tags_38 },
    { 0x6334b53a, SUC_lexicon_tags_16 },
    { 0x9d54b53b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe974b53f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa974b543, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d14b546, SUC_lexicon_tags_47 },
    { 0x2f94b547, SUC_lexicon_tags_11 },
    { 0x5b54b546, SUC_lexicon_tags_268 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca54b54d, SUC_lexicon_tags_16 },
    { 0x9d74b54d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc374b550, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x99d4b552, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4074b554, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2a94b556, SUC_lexicon_tags_19 },
    { 0x6354b557, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e54b55f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4b565, SUC_lexicon_tags_17 },
    { 0x2eb4b566, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b4b56c, SUC_lexicon_tags_11 },
    { 0x2074b56d, SUC_lexicon_tags_2 },
    { 0xab4b56c, SUC_lexicon_tags_12 },
    { 0x2c54b56c, SUC_lexicon_tags_15 },
    { 0x55d4b570, SUC_lexicon_tags_2 },
    { 0x1634b571, SUC_lexicon_tags_2 },
    { 0x54d4b56c, SUC_lexicon_tags_49 },
    { 0x4b4b573, SUC_lexicon_tags_42 },
    { 0x9d4b574, SUC_lexicon_tags_37 },
    { 0xdfb4b574, SUC_lexicon_tags_17 },
    { 0x3134b576, SUC_lexicon_tags_11 },
    { 0x7c14b575, SUC_lexicon_tags_47 },
    { 0xa0b4b578, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854b580, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe474b586, SUC_lexicon_tags_25 },
    { 0xdbd4b587, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb94b58c, SUC_lexicon_tags_1 },
    { 0xfef4b58c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9674b58f, SUC_lexicon_tags_1 },
    { 0x4954b58f, SUC_lexicon_tags_19 },
    { 0xa6b4b591, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5bb4b593, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5514b597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc214b599, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72b4b59b, SUC_lexicon_tags_15 },
    { 0xa854b59c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a34b59e, SUC_lexicon_tags_23 },
    { 0xba74b59f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe234b5a1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa934b5a4, SUC_lexicon_tags_13 },
    { 0x3df4b5a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd074b5a7, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab4b5aa, SUC_lexicon_tags_15 },
    { 0x5374b5ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a54b5af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b4b5b5, SUC_lexicon_tags_25 },
    { 0x7bf4b5b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9774b5bb, SUC_lexicon_tags_17 },
    { 0x2e94b5bc, SUC_lexicon_tags_11 },
    { 0x8314b5bd, SUC_lexicon_tags_17 },
    { 0x8a94b5bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4614b5c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb4b5c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9474b5cc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x27f4b5ce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d14b5d2, SUC_lexicon_tags_17 },
    { 0xffd4b5d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70f4b5d5, SUC_lexicon_tags_25 },
    { 0x8814b5d6, SUC_lexicon_tags_12 },
    { 0xc294b5d6, SUC_lexicon_tags_12 },
    { 0x7054b5d7, SUC_lexicon_tags_38 },
    { 0x3734b5d8, SUC_lexicon_tags_25 },
    { 0xac54b5da, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x6834b5dd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d4b5e3, SUC_lexicon_tags_6 },
    { 0x4014b5e4, SUC_lexicon_tags_60 },
    { 0x4674b5e5, SUC_lexicon_tags_6 },
    { 0x1f14b5e3, SUC_lexicon_tags_17 },
    { 0x95b4b5e7, SUC_lexicon_tags_15 },
    { 0xdf74b5e8, SUC_lexicon_tags_17 },
    { 0x2df4b5e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6194b5ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f4b5f4, SUC_lexicon_tags_15 },
    { 0x2254b5f5, SUC_lexicon_tags_84 },
    { 0x7f4b5f6, SUC_lexicon_tags_37 },
    { 0x2f94b5f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe734b5f9, SUC_lexicon_tags_55 },
    { 0x3494b5f9, SUC_lexicon_tags_17 },
    { 0x9a94b5fb, SUC_lexicon_tags_30 },
    { 0x31b4b5fc, SUC_lexicon_tags_2 },
    { 0xc7f4b5f9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb14b602, SUC_lexicon_tags_37 },
    { 0x72f4b603, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d4b606, SUC_lexicon_tags_19 },
    { 0x9354b607, SUC_lexicon_tags_21 },
    { 0xf174b608, SUC_lexicon_tags_47 },
    { 0xe3d4b607, SUC_lexicon_tags_11 },
    { 0x2cb4b60a, SUC_lexicon_tags_23 },
    { 0x25f4b60b, SUC_lexicon_tags_55 },
    { 0xb4d4b60b, SUC_lexicon_tags_43 },
    { 0x72b4b60b, SUC_lexicon_tags_19 },
    { 0x5a34b60c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b4b617, SUC_lexicon_tags_5 },
    { 0xc934b617, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7274b61d, SUC_lexicon_tags_47 },
    { 0x1f4b61e, SUC_lexicon_tags_29 },
    { 0x86d4b61f, SUC_lexicon_tags_62 },
    { 0x6054b620, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d4b623, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514b62f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4b633, SUC_lexicon_tags_1 },
    { 0x5a74b634, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc094b639, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2014b63b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff4b63f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3594b643, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc74b645, SUC_lexicon_tags_36 },
    { 0xe954b646, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb074b648, SUC_lexicon_tags_25 },
    { 0xfe54b649, SUC_lexicon_tags_88 },
    { 0xea34b64a, SUC_lexicon_tags_33 },
    { 0x1bb4b64a, SUC_lexicon_tags_33 },
    { 0xcf74b648, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb34b64e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x674b651, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c54b654, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c74b657, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b4b661, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5dd4b663, SUC_lexicon_tags_57 },
    { 0x5734b664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3974b666, SUC_lexicon_tags_37 },
    { 0xadf4b667, SUC_lexicon_tags_25 },
    { 0x7694b667, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf4b66b, SUC_lexicon_tags_7 },
    { 0x8f74b66c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5534b66f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d4b674, SUC_lexicon_tags_2 },
    { 0xbc34b675, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1814b67b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c94b67f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe134b683, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb54b685, SUC_lexicon_tags_60 },
    { 0xecf4b685, SUC_lexicon_tags_37 },
    { 0x6b4b687, SUC_lexicon_tags_28 },
    { 0x5374b688, SUC_lexicon_tags_2 },
    { 0xe174b687, SUC_lexicon_tags_201 },
    { 0xc834b68a, SUC_lexicon_tags_0 },
    { 0x7a94b686, SUC_lexicon_tags_295 },
    { 0xe894b68c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe434b695, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5654b699, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef4b69c, SUC_lexicon_tags_39 },
    { 0x1ad4b69d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce54b69f, SUC_lexicon_tags_8 },
    { 0x3254b6a0, SUC_lexicon_tags_77 },
    { 0xadd4b6a1, SUC_lexicon_tags_46 },
    { 0x3434b6a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6294b6a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc734b6aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2854b6b5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3214b6bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b4b6bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xccd4b6c1, SUC_lexicon_tags_15 },
    { 0x70f4b6c2, SUC_lexicon_tags_0 },
    { 0x5b54b6c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d94b6c5, SUC_lexicon_tags_18 },
    { 0xeb4b6c6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2bb4b6c8, SUC_lexicon_tags_37 },
    { 0xad94b6c9, SUC_lexicon_tags_17 },
    { 0x95f4b6c9, SUC_lexicon_tags_37 },
    { 0xb474b6cb, SUC_lexicon_tags_37 },
    { 0x4114b6cc, SUC_lexicon_tags_16 },
    { 0x4874b6cd, SUC_lexicon_tags_6 },
    { 0x994b6ce, SUC_lexicon_tags_2 },
    { 0x18b4b6cd, SUC_lexicon_tags_0 },
    { 0x9e54b6cd, SUC_lexicon_tags_46 },
    { 0xe974b6c9, SUC_lexicon_tags_62 },
    { 0x1dd4b6d0, SUC_lexicon_tags_54 },
    { 0x26d4b6c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c74b6d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd4b6da, SUC_lexicon_tags_37 },
    { 0xd0b4b6db, SUC_lexicon_tags_37 },
    { 0x8a74b6da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6314b6de, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce94b6e4, SUC_lexicon_tags_37 },
    { 0x1894b6e4, SUC_lexicon_tags_2 },
    { 0x7854b6e4, SUC_lexicon_tags_66 },
    { 0xac94b6e7, SUC_lexicon_tags_74 },
    { 0x65f4b6e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9d34b6ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a94b6f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d4b6f4, SUC_lexicon_tags_2 },
    { 0xc874b6f5, SUC_lexicon_tags_65 },
    { 0x6dd4b6f6, SUC_lexicon_tags_11 },
    { 0x6694b6f7, SUC_lexicon_tags_43 },
    { 0xafd4b6f8, SUC_lexicon_tags_7 },
    { 0x7c34b6f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8414b6fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5d34b6ff, SUC_lexicon_tags_69 },
    { 0x3e94b6ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef4b707, SUC_lexicon_tags_37 },
    { 0xc6d4b708, SUC_lexicon_tags_87 },
    { 0x634b709, SUC_lexicon_tags_7 },
    { 0xe334b70a, SUC_lexicon_tags_11 },
    { 0xcfb4b70b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1334b70e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1bf4b710, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f4b714, SUC_lexicon_tags_12 },
    { 0x8774b715, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b4b718, SUC_lexicon_tags_31 },
    { 0x3f94b719, SUC_lexicon_tags_27 },
    { 0x6b14b71a, SUC_lexicon_tags_17 },
    { 0x79d4b71b, SUC_lexicon_tags_1 },
    { 0x6394b719, SUC_lexicon_tags_23 },
    { 0x634b71c, SUC_lexicon_tags_12 },
    { 0xbe14b71e, SUC_lexicon_tags_92 },
    { 0x7bb4b71e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb4b722, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad4b727, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd54b72b, SUC_lexicon_tags_87 },
    { 0xc274b72c, SUC_lexicon_tags_1 },
    { 0xf974b72b, SUC_lexicon_tags_82 },
    { 0x6494b72e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9714b731, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd4b734, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1b74b736, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f4b739, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1fb4b73b, SUC_lexicon_tags_17 },
    { 0x1674b73c, SUC_lexicon_tags_26 },
    { 0x6c14b73d, SUC_lexicon_tags_17 },
    { 0x89d4b73d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf4b743, SUC_lexicon_tags_28 },
    { 0x6ff4b743, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3714b747, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec14b749, SUC_lexicon_tags_11 },
    { 0x80b4b749, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa7b4b74c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b4b751, SUC_lexicon_tags_17 },
    { 0x76d4b752, SUC_lexicon_tags_37 },
    { 0x87b4b752, SUC_lexicon_tags_11 },
    { 0x574b752, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7494b760, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a94b769, SUC_lexicon_tags_12 },
    { 0x9614b769, SUC_lexicon_tags_17 },
    { 0x1df4b76b, SUC_lexicon_tags_68 },
    { 0x1414b769, SUC_lexicon_tags_25 },
    { 0xcb54b76d, SUC_lexicon_tags_12 },
    { 0xef34b76c, SUC_lexicon_tags_17 },
    { 0xc574b76f, SUC_lexicon_tags_0 },
    { 0xdc94b770, SUC_lexicon_tags_9 },
    { 0xd974b771, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa734b777, SUC_lexicon_tags_32 },
    { 0x17f4b778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6874b77b, SUC_lexicon_tags_20 },
    { 0xc3f4b77c, SUC_lexicon_tags_12 },
    { 0x9374b77c, SUC_lexicon_tags_4 },
    { 0x2374b77b, SUC_lexicon_tags_1 },
    { 0xe674b77f, SUC_lexicon_tags_29 },
    { 0x5d34b77e, SUC_lexicon_tags_37 },
    { 0x7634b77d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfd14b783, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ff4b785, SUC_lexicon_tags_30 },
    { 0xb414b786, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd374b788, SUC_lexicon_tags_49 },
    { 0x63d4b788, SUC_lexicon_tags_7 },
    { 0xd554b78a, SUC_lexicon_tags_80 },
    { 0x1774b788, SUC_lexicon_tags_49 },
    { 0x7cd4b78c, SUC_lexicon_tags_3 },
    { 0x8d34b78d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x334b78f, SUC_lexicon_tags_26 },
    { 0x1bd4b790, SUC_lexicon_tags_7 },
    { 0xc994b791, SUC_lexicon_tags_1 },
    { 0x32d4b792, SUC_lexicon_tags_17 },
    { 0x7dd4b793, SUC_lexicon_tags_45 },
    { 0x51d4b793, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd6d4b796, SUC_lexicon_tags_25 },
    { 0x2d54b797, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb534b79b, SUC_lexicon_tags_15 },
    { 0x69d4b79c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4394b7a1, SUC_lexicon_tags_7 },
    { 0xeaf4b7a1, SUC_lexicon_tags_84 },
    { 0x9a54b7a1, SUC_lexicon_tags_19 },
    { 0xd374b7a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x374b7a6, SUC_lexicon_tags_8 },
    { 0x4794b7a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5314b7a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b4b7ad, SUC_lexicon_tags_13 },
    { 0x25f4b7ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd754b7b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7114b7b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4b7b9, SUC_lexicon_tags_37 },
    { 0xa734b7b9, SUC_lexicon_tags_12 },
    { 0x1774b7bb, SUC_lexicon_tags_16 },
    { 0x1714b7bc, SUC_lexicon_tags_37 },
    { 0x7ff4b7bd, SUC_lexicon_tags_25 },
    { 0x47b4b7b9, SUC_lexicon_tags_2 },
    { 0x8274b7bf, SUC_lexicon_tags_10 },
    { 0x4bb4b7bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4474b7c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e54b7c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d74b7c9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf4b7d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b94b7d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d14b7da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b4b7e0, SUC_lexicon_tags_15 },
    { 0x6a34b7e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x634b7e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b4b7eb, SUC_lexicon_tags_11 },
    { 0x9094b7ec, SUC_lexicon_tags_33 },
    { 0x2274b7eb, SUC_lexicon_tags_25 },
    { 0xdcd4b7ee, SUC_lexicon_tags_15 },
    { 0xa2f4b7ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b4b7f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf94b7f5, SUC_lexicon_tags_12 },
    { 0x3f4b7f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd274b7fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf4b802, SUC_lexicon_tags_151 },
    { 0x5494b803, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3394b807, SUC_lexicon_tags_18 },
    { 0xa274b808, SUC_lexicon_tags_48 },
    { 0x6e74b809, SUC_lexicon_tags_1 },
    { 0xf414b808, SUC_lexicon_tags_63 },
    { 0x4c34b808, SUC_lexicon_tags_45 },
    { 0xac94b80a, SUC_lexicon_tags_7 },
    { 0x9354b80d, SUC_lexicon_tags_2 },
    { 0xc854b80d, SUC_lexicon_tags_15 },
    { 0xb6f4b80f, SUC_lexicon_tags_7 },
    { 0x20d4b80d, SUC_lexicon_tags_82 },
    { 0x6034b811, SUC_lexicon_tags_29 },
    { 0xc194b812, SUC_lexicon_tags_7 },
    { 0x54f4b80f, SUC_lexicon_tags_21 },
    { 0xdb4b810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6554b819, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6014b81e, SUC_lexicon_tags_15 },
    { 0x6214b81f, SUC_lexicon_tags_12 },
    { 0x6934b820, SUC_lexicon_tags_19 },
    { 0xf6b4b81f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d34b823, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf4b827, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9534b82b, SUC_lexicon_tags_11 },
    { 0x59f4b82b, SUC_lexicon_tags_31 },
    { 0x9a34b82d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb4b830, SUC_lexicon_tags_47 },
    { 0x5174b831, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc4f4b833, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9234b839, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2714b83c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb4b83f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa414b842, SUC_lexicon_tags_17 },
    { 0x7254b843, SUC_lexicon_tags_8 },
    { 0xcfb4b844, SUC_lexicon_tags_17 },
    { 0x91d4b844, SUC_lexicon_tags_24 },
    { 0x9394b846, SUC_lexicon_tags_25 },
    { 0xdbf4b844, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd4b84a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf214b84d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8534b850, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4114b853, SUC_lexicon_tags_30 },
    { 0xc714b854, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e74b858, SUC_lexicon_tags_11 },
    { 0x7ed4b858, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb734b85b, SUC_lexicon_tags_37 },
    { 0x9ed4b85c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd4b85e, SUC_lexicon_tags_15 },
    { 0x7674b85f, SUC_lexicon_tags_26 },
    { 0xf574b85f, SUC_lexicon_tags_2 },
    { 0x1e34b85e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b34b863, SUC_lexicon_tags_7 },
    { 0x2034b864, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3314b86b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd434b86d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc94b870, SUC_lexicon_tags_25 },
    { 0x3f54b870, SUC_lexicon_tags_33 },
    { 0x2e14b872, SUC_lexicon_tags_12 },
    { 0x514b873, SUC_lexicon_tags_37 },
    { 0x8134b874, SUC_lexicon_tags_26 },
    { 0x7474b870, SUC_lexicon_tags_22 },
    { 0x6454b870, SUC_lexicon_tags_15 },
    { 0x8af4b872, SUC_lexicon_tags_17 },
    { 0xf194b878, SUC_lexicon_tags_25 },
    { 0x7734b879, SUC_lexicon_tags_14 },
    { 0x7ef4b871, SUC_lexicon_tags_11 },
    { 0xef34b87a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8314b880, SUC_lexicon_tags_57 },
    { 0xea14b881, SUC_lexicon_tags_2 },
    { 0x1554b882, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x354b884, SUC_lexicon_tags_11 },
    { 0x9ed4b885, SUC_lexicon_tags_68 },
    { 0xe294b886, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4254b888, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8554b88a, SUC_lexicon_tags_7 },
    { 0xd9d4b88b, SUC_lexicon_tags_21 },
    { 0x12d4b88c, SUC_lexicon_tags_6 },
    { 0xf4d4b88d, SUC_lexicon_tags_17 },
    { 0xb574b88e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae14b890, SUC_lexicon_tags_11 },
    { 0x5874b891, SUC_lexicon_tags_44 },
    { 0x3194b891, SUC_lexicon_tags_9 },
    { 0xf394b893, SUC_lexicon_tags_47 },
    { 0x97f4b894, SUC_lexicon_tags_12 },
    { 0x4f74b894, SUC_lexicon_tags_25 },
    { 0x96f4b893, SUC_lexicon_tags_12 },
    { 0x2014b897, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b54b89b, SUC_lexicon_tags_25 },
    { 0xe5d4b89b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ff4b89e, SUC_lexicon_tags_37 },
    { 0x8b34b89e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7254b8a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e94b8a6, SUC_lexicon_tags_15 },
    { 0x8254b8a7, SUC_lexicon_tags_11 },
    { 0x4eb4b8a7, SUC_lexicon_tags_2 },
    { 0x4cf4b8a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x37f4b8ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e94b8ad, SUC_lexicon_tags_709 },
    { 0x1df4b8ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf4b8b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae74b8bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xea54b8bf, SUC_lexicon_tags_11 },
    { 0xf054b8c0, SUC_lexicon_tags_46 },
    { 0x5d14b8bf, SUC_lexicon_tags_3 },
    { 0xa654b8c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab4b8c5, SUC_lexicon_tags_27 },
    { 0x9c54b8c5, SUC_lexicon_tags_26 },
    { 0xdc54b8c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe34b8ca, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb694b8cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa214b8ce, SUC_lexicon_tags_43 },
    { 0x2814b8cf, SUC_lexicon_tags_63 },
    { 0xd334b8d0, SUC_lexicon_tags_138 },
    { 0x16b4b8d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd34b8db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec74b8e0, SUC_lexicon_tags_60 },
    { 0x2274b8e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb74b8e7, SUC_lexicon_tags_16 },
    { 0xf354b8e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2654b8ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6774b8ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x77d4b8f1, SUC_lexicon_tags_4 },
    { 0x5b94b8f1, SUC_lexicon_tags_11 },
    { 0xdc74b8f1, SUC_lexicon_tags_31 },
    { 0xee94b8f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xddb4b8f6, SUC_lexicon_tags_12 },
    { 0x3254b8f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff4b8fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ab4b8fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7094b901, SUC_lexicon_tags_9 },
    { 0xac74b902, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a74b905, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb4b90b, SUC_lexicon_tags_11 },
    { 0x5634b90c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe14b910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6974b919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf4b922, SUC_lexicon_tags_0 },
    { 0x5a34b923, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc534b925, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2b4b927, SUC_lexicon_tags_49 },
    { 0x5794b928, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1674b930, SUC_lexicon_tags_60 },
    { 0x3bd4b931, SUC_lexicon_tags_32 },
    { 0x1214b932, SUC_lexicon_tags_17 },
    { 0x8474b932, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf334b93a, SUC_lexicon_tags_26 },
    { 0x94d4b93b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8714b941, SUC_lexicon_tags_12 },
    { 0xa814b942, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa054b944, SUC_lexicon_tags_45 },
    { 0xcd74b945, SUC_lexicon_tags_26 },
    { 0xb574b944, SUC_lexicon_tags_1 },
    { 0x3f4b947, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad4b94b, SUC_lexicon_tags_25 },
    { 0xd634b94c, SUC_lexicon_tags_11 },
    { 0x8234b94c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad4b950, SUC_lexicon_tags_29 },
    { 0x3414b950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4234b959, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b14b95c, SUC_lexicon_tags_68 },
    { 0x1554b95d, SUC_lexicon_tags_481 },
    { 0x2bd4b95c, SUC_lexicon_tags_47 },
    { 0x7014b95f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f4b962, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8354b96c, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa754b971, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d4b975, SUC_lexicon_tags_55 },
    { 0xa114b976, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4b97e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e94b980, SUC_lexicon_tags_15 },
    { 0xe0f4b981, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfe94b983, SUC_lexicon_tags_17 },
    { 0x3174b984, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f4b988, SUC_lexicon_tags_1 },
    { 0x5ff4b988, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1374b98b, SUC_lexicon_tags_15 },
    { 0x6194b98c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f34b98e, SUC_lexicon_tags_34 },
    { 0x8f14b98f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb4b992, SUC_lexicon_tags_37 },
    { 0xe5d4b993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bb4b995, SUC_lexicon_tags_3 },
    { 0x4af4b995, SUC_lexicon_tags_17 },
    { 0x7514b995, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe274b99b, SUC_lexicon_tags_19 },
    { 0x1354b99b, SUC_lexicon_tags_23 },
    { 0x7e34b99b, SUC_lexicon_tags_11 },
    { 0xa6b4b99e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab4b9a3, SUC_lexicon_tags_12 },
    { 0xd8d4b9a4, SUC_lexicon_tags_17 },
    { 0x15f4b9a4, SUC_lexicon_tags_47 },
    { 0x5a34b9a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb474b9a8, SUC_lexicon_tags_12 },
    { 0xe7b4b9a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab4b9ba, SUC_lexicon_tags_0 },
    { 0x6ed4b9bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf94b9c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f4b9c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3af4b9c7, SUC_lexicon_tags_7 },
    { 0x214b9c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea54b9cd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbb34b9cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29d4b9d1, SUC_lexicon_tags_7 },
    { 0x7ad4b9d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad94b9d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab74b9dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8074b9df, SUC_lexicon_tags_49 },
    { 0xf134b9e0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7654b9e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x64f4b9e7, SUC_lexicon_tags_62 },
    { 0xefb4b9e7, SUC_lexicon_tags_28 },
    { 0x3534b9e9, SUC_lexicon_tags_60 },
    { 0x2494b9e8, SUC_lexicon_tags_2 },
    { 0x1bd4b9e8, SUC_lexicon_tags_45 },
    { 0x9f34b9ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff34b9f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e74b9f7, SUC_lexicon_tags_63 },
    { 0xe014b9f8, SUC_lexicon_tags_44 },
    { 0x6cf4b9f8, SUC_lexicon_tags_64 },
    { 0x8c34b9f9, SUC_lexicon_tags_65 },
    { 0x9ad4b9f9, SUC_lexicon_tags_68 },
    { 0x2ef4b9fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb774b9fe, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2394ba04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf454ba08, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594ba0f, SUC_lexicon_tags_63 },
    { 0x1854ba10, SUC_lexicon_tags_47 },
    { 0x7b14ba0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4ba15, SUC_lexicon_tags_84 },
    { 0xc394ba16, SUC_lexicon_tags_11 },
    { 0x8a34ba17, SUC_lexicon_tags_6 },
    { 0x3bf4ba17, SUC_lexicon_tags_1 },
    { 0xe814ba15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9b4ba1b, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd054ba20, SUC_lexicon_tags_25 },
    { 0x7d94ba21, SUC_lexicon_tags_26 },
    { 0x62b4ba22, SUC_lexicon_tags_151 },
    { 0xe674ba22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f4ba25, SUC_lexicon_tags_31 },
    { 0x9354ba26, SUC_lexicon_tags_6 },
    { 0x8a54ba25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1834ba2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbff4ba2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b4ba36, SUC_lexicon_tags_33 },
    { 0x2a14ba36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa074ba39, SUC_lexicon_tags_25 },
    { 0xcd14ba39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4494ba3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdb4ba3f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2e74ba41, SUC_lexicon_tags_2 },
    { 0xfc14ba42, SUC_lexicon_tags_6 },
    { 0x8814ba42, SUC_lexicon_tags_15 },
    { 0xb034ba44, SUC_lexicon_tags_1 },
    { 0x8914ba45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd34ba4f, SUC_lexicon_tags_23 },
    { 0x64d4ba50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf4ba53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8bf4ba55, SUC_lexicon_tags_0 },
    { 0x34d4ba56, SUC_lexicon_tags_3 },
    { 0x9c14ba56, SUC_lexicon_tags_82 },
    { 0x32f4ba57, SUC_lexicon_tags_32 },
    { 0xce14ba57, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe494ba5b, SUC_lexicon_tags_17 },
    { 0x1534ba5c, SUC_lexicon_tags_3 },
    { 0x3654ba5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc194ba62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f4ba66, SUC_lexicon_tags_8 },
    { 0x2c54ba67, SUC_lexicon_tags_91 },
    { 0xce14ba66, SUC_lexicon_tags_0 },
    { 0xfbb4ba67, SUC_lexicon_tags_63 },
    { 0x2914ba6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd594ba70, SUC_lexicon_tags_17 },
    { 0xca14ba71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd894ba75, SUC_lexicon_tags_49 },
    { 0x6914ba76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1054ba7c, SUC_lexicon_tags_15 },
    { 0x8e94ba7d, SUC_lexicon_tags_29 },
    { 0x4334ba7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d74ba81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd4ba86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed4ba89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc274ba8b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f4ba93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd434ba98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f4ba9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9534baa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe174baac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4374baae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29b4bab0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7034bab6, SUC_lexicon_tags_14 },
    { 0xca94bab7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaf14bab9, SUC_lexicon_tags_1 },
    { 0x31b4baba, SUC_lexicon_tags_61 },
    { 0x49d4babb, SUC_lexicon_tags_60 },
    { 0x5e34babc, SUC_lexicon_tags_37 },
    { 0x4f74babc, SUC_lexicon_tags_25 },
    { 0x2f94babc, SUC_lexicon_tags_33 },
    { 0x1234babd, SUC_lexicon_tags_92 },
    { 0xd754bac0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d74bacb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7b4bacd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1d54bacf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3554bad3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c54bad6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8f4bad8, SUC_lexicon_tags_9 },
    { 0x7bb4bad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb374badd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8774bae3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd4bae6, SUC_lexicon_tags_11 },
    { 0xed54bae7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d4baea, SUC_lexicon_tags_25 },
    { 0x40b4baeb, SUC_lexicon_tags_0 },
    { 0xac54baeb, SUC_lexicon_tags_12 },
    { 0x49d4baed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf734baef, SUC_lexicon_tags_17 },
    { 0xb34baf0, SUC_lexicon_tags_27 },
    { 0x4234baf1, SUC_lexicon_tags_46 },
    { 0x9674baf2, SUC_lexicon_tags_87 },
    { 0x8bd4baf1, SUC_lexicon_tags_25 },
    { 0x9694baf2, SUC_lexicon_tags_7 },
    { 0xc8b4baf4, SUC_lexicon_tags_17 },
    { 0x3f54baf0, SUC_lexicon_tags_26 },
    { 0x4cb4baf7, SUC_lexicon_tags_37 },
    { 0xf034baf0, SUC_lexicon_tags_37 },
    { 0x7a54baf9, SUC_lexicon_tags_28 },
    { 0xe194baf9, SUC_lexicon_tags_92 },
    { 0x44b4bafb, SUC_lexicon_tags_7 },
    { 0xf374bafc, SUC_lexicon_tags_10 },
    { 0x55f4bafc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4bb04, SUC_lexicon_tags_15 },
    { 0x1cb4bb05, SUC_lexicon_tags_1 },
    { 0xb934bb06, SUC_lexicon_tags_38 },
    { 0x1774bb07, SUC_lexicon_tags_25 },
    { 0xd394bb06, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0x16b4bb0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4af4bb0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef74bb11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1f4bb13, SUC_lexicon_tags_45 },
    { 0x2df4bb14, SUC_lexicon_tags_23 },
    { 0x7994bb15, SUC_lexicon_tags_25 },
    { 0x6454bb15, SUC_lexicon_tags_15 },
    { 0xe14bb15, SUC_lexicon_tags_23 },
    { 0x1674bb14, SUC_lexicon_tags_2 },
    { 0x6974bb18, SUC_lexicon_tags_17 },
    { 0x5594bb14, SUC_lexicon_tags_26 },
    { 0x9b54bb1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4714bb1d, SUC_lexicon_tags_25 },
    { 0xf414bb1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x254bb28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe14bb2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3974bb30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e74bb32, SUC_lexicon_tags_25 },
    { 0xfbb4bb32, SUC_lexicon_tags_11 },
    { 0x7234bb34, SUC_lexicon_tags_17 },
    { 0x6bd4bb35, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad4bb38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f4bb3c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4654bb3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b74bb44, SUC_lexicon_tags_14 },
    { 0x1ed4bb45, SUC_lexicon_tags_84 },
    { 0x4bd4bb46, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7214bb4d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3694bb50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce54bb57, SUC_lexicon_tags_25 },
    { 0x8094bb58, SUC_lexicon_tags_11 },
    { 0x9a54bb59, SUC_lexicon_tags_12 },
    { 0x2594bb5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2e14bb5c, SUC_lexicon_tags_87 },
    { 0xd34bb5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e94bb63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2154bb65, SUC_lexicon_tags_1 },
    { 0xe594bb66, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2494bb6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2894bb6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb54bb6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e54bb74, SUC_lexicon_tags_38 },
    { 0xf954bb75, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5354bb78, SUC_lexicon_tags_46 },
    { 0xc034bb78, SUC_lexicon_tags_55 },
    { 0x2d94bb78, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe054bb7d, SUC_lexicon_tags_15 },
    { 0xdb14bb7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8274bb80, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7614bb83, SUC_lexicon_tags_38 },
    { 0x82b4bb83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x54b4bb86, SUC_lexicon_tags_47 },
    { 0xced4bb87, SUC_lexicon_tags_11 },
    { 0x94f4bb86, SUC_lexicon_tags_0 },
    { 0x5e14bb86, SUC_lexicon_tags_25 },
    { 0x5214bb8a, SUC_lexicon_tags_68 },
    { 0x7af4bb86, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc294bb8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc14bb8f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb34bb99, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb534bb9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x994bbaa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d4bbb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc34bbb4, SUC_lexicon_tags_21 },
    { 0x9d34bbb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1154bbba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce94bbbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb714bbc1, SUC_lexicon_tags_37 },
    { 0xecd4bbc1, SUC_lexicon_tags_25 },
    { 0x1374bbc1, SUC_lexicon_tags_26 },
    { 0xfc34bbc4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d4bbc7, SUC_lexicon_tags_25 },
    { 0x65b4bbc8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23b4bbca, SUC_lexicon_tags_33 },
    { 0x14d4bbcb, SUC_lexicon_tags_43 },
    { 0x1bf4bbcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9214bbd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe394bbda, SUC_lexicon_tags_25 },
    { 0x26b4bbdb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3174bbde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5674bbe2, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f14bbe5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xce94bbe8, SUC_lexicon_tags_60 },
    { 0xd714bbe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c74bbeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2514bbed, SUC_lexicon_tags_60 },
    { 0xf574bbee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef4bbf0, SUC_lexicon_tags_38 },
    { 0x8154bbf0, SUC_lexicon_tags_27 },
    { 0xfe14bbf2, SUC_lexicon_tags_66 },
    { 0x2b34bbf3, SUC_lexicon_tags_47 },
    { 0x9554bbf0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa834bbf6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9eb4bbf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b74bc04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe374bc07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f4bc0a, SUC_lexicon_tags_12 },
    { 0x2e74bc0b, SUC_lexicon_tags_37 },
    { 0x1f34bc0c, SUC_lexicon_tags_15 },
    { 0x35d4bc0d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc534bc0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4994bc15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf674bc1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5254bc22, SUC_lexicon_tags_15 },
    { 0xd874bc23, SUC_lexicon_tags_47 },
    { 0xf3f4bc24, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed4bc29, SUC_lexicon_tags_11 },
    { 0xe3f4bc29, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xceb4bc2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcff4bc2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb474bc35, SUC_lexicon_tags_6 },
    { 0x1b34bc36, SUC_lexicon_tags_34 },
    { 0xc814bc37, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3dd4bc39, SUC_lexicon_tags_33 },
    { 0xdf14bc3a, SUC_lexicon_tags_21 },
    { 0x4854bc3a, SUC_lexicon_tags_25 },
    { 0x8d94bc3c, SUC_lexicon_tags_17 },
    { 0xa774bc3b, SUC_lexicon_tags_12 },
    { 0x214bc3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af4bc41, SUC_lexicon_tags_151 },
    { 0xc7f4bc42, SUC_lexicon_tags_23 },
    { 0x8d54bc41, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554bc47, SUC_lexicon_tags_171 },
    { 0x3294bc48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1674bc4d, SUC_lexicon_tags_49 },
    { 0xda94bc4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4214bc52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xde54bc54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcb4bc56, SUC_lexicon_tags_29 },
    { 0x7e34bc57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4bc5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0d4bc5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af4bc69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12d4bc6b, SUC_lexicon_tags_25 },
    { 0x95d4bc6b, SUC_lexicon_tags_449 },
    { 0xf754bc6d, SUC_lexicon_tags_41 },
    { 0xc014bc6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9634bc71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1934bc75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4114bc78, SUC_lexicon_tags_25 },
    { 0x2374bc78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad4bc7c, SUC_lexicon_tags_11 },
    { 0x8214bc7d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7294bc7f, SUC_lexicon_tags_0 },
    { 0xaff4bc80, SUC_lexicon_tags_6 },
    { 0x5eb4bc7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x214bc89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8114bc8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x17b4bc8d, SUC_lexicon_tags_14 },
    { 0x2994bc8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb4bc94, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6854bc98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb734bc9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab4bc9f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc34bca2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5754bca5, SUC_lexicon_tags_17 },
    { 0x1314bca5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd434bca9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb434bcad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8794bcb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1154bcb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf354bcb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfb4bcbb, SUC_lexicon_tags_37 },
    { 0x3b34bcbb, SUC_lexicon_tags_37 },
    { 0x6f34bcbc, SUC_lexicon_tags_23 },
    { 0xedf4bcbe, SUC_lexicon_tags_6 },
    { 0x6a94bcbd, SUC_lexicon_tags_14 },
    { 0xecd4bcc0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d4bcc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b4bcc8, SUC_lexicon_tags_11 },
    { 0x3834bcc9, SUC_lexicon_tags_20 },
    { 0xf2d4bcca, SUC_lexicon_tags_40 },
    { 0x334bcca, SUC_lexicon_tags_31 },
    { 0x9414bcca, SUC_lexicon_tags_0 },
    { 0xf8f4bccd, SUC_lexicon_tags_11 },
    { 0x2f54bccb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd834bcd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f4bcd6, SUC_lexicon_tags_25 },
    { 0x3894bcd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf74bcd9, SUC_lexicon_tags_23 },
    { 0xbeb4bcda, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x94d4bcdc, SUC_lexicon_tags_49 },
    { 0x5a14bcdd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab4bce2, SUC_lexicon_tags_17 },
    { 0x6af4bce2, SUC_lexicon_tags_38 },
    { 0xf8d4bce4, SUC_lexicon_tags_21 },
    { 0xcf34bce5, SUC_lexicon_tags_11 },
    { 0x934bce6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f4bce9, SUC_lexicon_tags_26 },
    { 0xc54bcea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb094bcef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d4bcf2, SUC_lexicon_tags_25 },
    { 0x1314bcf2, SUC_lexicon_tags_6 },
    { 0x73f4bcf3, SUC_lexicon_tags_27 },
    { 0x8a54bcf5, SUC_lexicon_tags_33 },
    { 0x7794bcf5, SUC_lexicon_tags_14 },
    { 0xc234bcf3, SUC_lexicon_tags_15 },
    { 0xf4d4bcf8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa514bcfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf334bcfd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd4bd02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f34bd04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b4bd09, SUC_lexicon_tags_91 },
    { 0x8534bd0a, SUC_lexicon_tags_37 },
    { 0xaa14bd09, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf14bd0d, SUC_lexicon_tags_11 },
    { 0x9114bd0e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d34bd12, SUC_lexicon_tags_87 },
    { 0xe1f4bd12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e54bd15, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5ff4bd17, SUC_lexicon_tags_17 },
    { 0x5534bd18, SUC_lexicon_tags_43 },
    { 0xa1d4bd19, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xad74bd1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4374bd1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb74bd26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454bd2f, SUC_lexicon_tags_40 },
    { 0xeb54bd30, SUC_lexicon_tags_25 },
    { 0xaef4bd30, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bd34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xced4bd37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3194bd39, SUC_lexicon_tags_10 },
    { 0x7f4bd3a, SUC_lexicon_tags_1508 },
    { 0, NULL },
    { 0x4334bd3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d4bd41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5854bd4b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4bd4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd054bd57, SUC_lexicon_tags_60 },
    { 0x4bb4bd58, SUC_lexicon_tags_2 },
    { 0xfab4bd59, SUC_lexicon_tags_47 },
    { 0x7054bd5a, SUC_lexicon_tags_11 },
    { 0x1574bd5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9014bd64, SUC_lexicon_tags_37 },
    { 0xeed4bd65, SUC_lexicon_tags_66 },
    { 0x74bd65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcf4bd68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf4bd6b, SUC_lexicon_tags_15 },
    { 0x2f14bd6c, SUC_lexicon_tags_19 },
    { 0x1f74bd6d, SUC_lexicon_tags_18 },
    { 0x70d4bd6e, SUC_lexicon_tags_63 },
    { 0xaf94bd6f, SUC_lexicon_tags_4 },
    { 0x9ef4bd6c, SUC_lexicon_tags_15 },
    { 0x27d4bd71, SUC_lexicon_tags_39 },
    { 0x40b4bd6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd094bd74, SUC_lexicon_tags_104 },
    { 0x89f4bd75, SUC_lexicon_tags_11 },
    { 0xee14bd74, SUC_lexicon_tags_40 },
    { 0x9114bd75, SUC_lexicon_tags_37 },
    { 0xffb4bd75, SUC_lexicon_tags_17 },
    { 0xdd34bd79, SUC_lexicon_tags_32 },
    { 0x754bd74, SUC_lexicon_tags_17 },
    { 0x2d54bd7b, SUC_lexicon_tags_27 },
    { 0xad4bd77, SUC_lexicon_tags_25 },
    { 0x6dd4bd7d, SUC_lexicon_tags_27 },
    { 0x4534bd7e, SUC_lexicon_tags_7 },
    { 0xc114bd7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5014bd81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa674bd83, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f4bd87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea14bd93, SUC_lexicon_tags_15 },
    { 0x69f4bd94, SUC_lexicon_tags_0 },
    { 0xe74bd95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5634bd97, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8714bd9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7c74bd9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d4bd9f, SUC_lexicon_tags_11 },
    { 0xff94bda0, SUC_lexicon_tags_30 },
    { 0x3874bda0, SUC_lexicon_tags_35 },
    { 0xf34bda1, SUC_lexicon_tags_2 },
    { 0xe474bda1, SUC_lexicon_tags_15 },
    { 0x30f4bd9f, SUC_lexicon_tags_38 },
    { 0xf8d4bda5, SUC_lexicon_tags_15 },
    { 0x2994bda1, SUC_lexicon_tags_44 },
    { 0xbc14bda4, SUC_lexicon_tags_11 },
    { 0xc414bda8, SUC_lexicon_tags_33 },
    { 0x574bda9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9974bdae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc994bdb1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x6cb4bdb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f4bdb8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x33d4bdba, SUC_lexicon_tags_17 },
    { 0xa894bdbb, SUC_lexicon_tags_33 },
    { 0xd3d4bdbb, SUC_lexicon_tags_37 },
    { 0x9c74bdbd, SUC_lexicon_tags_25 },
    { 0xc34bdbe, SUC_lexicon_tags_0 },
    { 0xd094bdbb, SUC_lexicon_tags_17 },
    { 0x6d54bdc0, SUC_lexicon_tags_37 },
    { 0x9c54bdc1, SUC_lexicon_tags_65 },
    { 0x3054bdc1, SUC_lexicon_tags_64 },
    { 0xf9f4bdbe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb54bdc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f14bdc8, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xa594bdca, SUC_lexicon_tags_12 },
    { 0x6c54bdcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa254bdcd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f4bdd2, SUC_lexicon_tags_212 },
    { 0x9054bdd3, SUC_lexicon_tags_56 },
    { 0xbb94bdd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d4bddb, SUC_lexicon_tags_37 },
    { 0x2354bddc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff4bdde, SUC_lexicon_tags_15 },
    { 0x7234bddf, SUC_lexicon_tags_84 },
    { 0xdab4bde0, SUC_lexicon_tags_8 },
    { 0x1e14bde1, SUC_lexicon_tags_39 },
    { 0x75b4bde1, SUC_lexicon_tags_2 },
    { 0xfe74bde3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b4bde6, SUC_lexicon_tags_37 },
    { 0x48f4bde7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6054bdea, SUC_lexicon_tags_27 },
    { 0x5754bdea, SUC_lexicon_tags_14 },
    { 0xf414bdec, SUC_lexicon_tags_38 },
    { 0x83b4bded, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe274bdf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd314bdfa, SUC_lexicon_tags_37 },
    { 0xc5b4bdfa, SUC_lexicon_tags_37 },
    { 0xec14bdfb, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x354bdfe, SUC_lexicon_tags_19 },
    { 0x5074bdfe, SUC_lexicon_tags_7 },
    { 0x1f14bdff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5254be02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf194be04, SUC_lexicon_tags_62 },
    { 0xead4be05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb474be0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b4be12, SUC_lexicon_tags_37 },
    { 0x84d4be13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd354be15, SUC_lexicon_tags_37 },
    { 0x6fd4be16, SUC_lexicon_tags_14 },
    { 0xe514be17, SUC_lexicon_tags_12 },
    { 0x66b4be18, SUC_lexicon_tags_17 },
    { 0x50f4be15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e14be1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8474be1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b4be21, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4754be28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99d4be2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b4be2d, SUC_lexicon_tags_37 },
    { 0xeb54be2d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x774be32, SUC_lexicon_tags_2 },
    { 0xd3d4be33, SUC_lexicon_tags_17 },
    { 0x2874be34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6b4be36, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf94be38, SUC_lexicon_tags_12 },
    { 0xa294be39, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6534be3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf834be42, SUC_lexicon_tags_38 },
    { 0x8eb4be43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7074be45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc74be4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa214be4f, SUC_lexicon_tags_12 },
    { 0xcbf4be50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe14be53, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe334be55, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b4be58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5334be5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf334be5e, SUC_lexicon_tags_12 },
    { 0x2f14be5e, SUC_lexicon_tags_37 },
    { 0x6474be60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd554be62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d4be66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f4be68, SUC_lexicon_tags_23 },
    { 0x6134be69, SUC_lexicon_tags_6 },
    { 0x29d4be6a, SUC_lexicon_tags_23 },
    { 0x4d34be69, SUC_lexicon_tags_11 },
    { 0x1fb4be69, SUC_lexicon_tags_2 },
    { 0xdff4be6d, SUC_lexicon_tags_2 },
    { 0xdb54be6e, SUC_lexicon_tags_12 },
    { 0xd3f4be6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7694be73, SUC_lexicon_tags_1442 },
    { 0x4a14be74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c14be76, SUC_lexicon_tags_26 },
    { 0x8494be77, SUC_lexicon_tags_8 },
    { 0x94d4be78, SUC_lexicon_tags_7 },
    { 0xb0b4be79, SUC_lexicon_tags_15 },
    { 0xd8b4be7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8994be81, SUC_lexicon_tags_17 },
    { 0xbc94be82, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d4be86, SUC_lexicon_tags_17 },
    { 0xa9d4be87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f4be8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe734be8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b4be94, SUC_lexicon_tags_49 },
    { 0x7354be95, SUC_lexicon_tags_9 },
    { 0x85d4be96, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf94be99, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f4bea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4254bea6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4014beaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5414beac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4654beb0, SUC_lexicon_tags_80 },
    { 0xb114beb0, SUC_lexicon_tags_37 },
    { 0x33d4beb0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b4beb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6174bebe, SUC_lexicon_tags_25 },
    { 0xfbf4bebe, SUC_lexicon_tags_25 },
    { 0x5234bebf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf4bec3, SUC_lexicon_tags_0 },
    { 0x2eb4bec3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91d4bec6, SUC_lexicon_tags_11 },
    { 0x6234bec6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b4becb, SUC_lexicon_tags_12 },
    { 0x1a94becb, SUC_lexicon_tags_826 },
    { 0, NULL },
    { 0, NULL },
    { 0xa154becf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf054bed2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddf4bed4, SUC_lexicon_tags_14 },
    { 0xefd4bed5, SUC_lexicon_tags_0 },
    { 0xfc54bed4, SUC_lexicon_tags_23 },
    { 0x92f4bed7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f4bede, SUC_lexicon_tags_37 },
    { 0x6554bede, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1274bee9, SUC_lexicon_tags_47 },
    { 0x35f4beea, SUC_lexicon_tags_1 },
    { 0xfab4beeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1094beef, SUC_lexicon_tags_1 },
    { 0xd494bef0, SUC_lexicon_tags_17 },
    { 0x9b94bef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe474bef7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x76f4bef9, SUC_lexicon_tags_25 },
    { 0xf8b4befa, SUC_lexicon_tags_62 },
    { 0x5e74befb, SUC_lexicon_tags_6 },
    { 0x514befc, SUC_lexicon_tags_47 },
    { 0x7154befc, SUC_lexicon_tags_38 },
    { 0x7f4befc, SUC_lexicon_tags_64 },
    { 0x4654beff, SUC_lexicon_tags_12 },
    { 0x17f4bf00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92d4bf02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8b4bf04, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c34bf07, SUC_lexicon_tags_25 },
    { 0xd6f4bf07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d4bf0c, SUC_lexicon_tags_26 },
    { 0x8f74bf0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb614bf16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b4bf1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0b4bf1d, SUC_lexicon_tags_265 },
    { 0xf5d4bf1d, SUC_lexicon_tags_11 },
    { 0x36d4bf1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8954bf26, SUC_lexicon_tags_26 },
    { 0x9b4bf27, SUC_lexicon_tags_61 },
    { 0x80b4bf28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49d4bf2a, SUC_lexicon_tags_19 },
    { 0xbf54bf2b, SUC_lexicon_tags_7 },
    { 0x2314bf2c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b4bf2f, SUC_lexicon_tags_47 },
    { 0x30b4bf30, SUC_lexicon_tags_14 },
    { 0x5714bf30, SUC_lexicon_tags_40 },
    { 0xc914bf31, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x71f4bf34, SUC_lexicon_tags_33 },
    { 0xb4f4bf35, SUC_lexicon_tags_1 },
    { 0xd794bf34, SUC_lexicon_tags_27 },
    { 0x7b14bf37, SUC_lexicon_tags_7 },
    { 0x8794bf37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ef4bf3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4bf3d, SUC_lexicon_tags_15 },
    { 0x62b4bf3e, SUC_lexicon_tags_49 },
    { 0x83f4bf3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a74bf41, SUC_lexicon_tags_27 },
    { 0xfbf4bf42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5074bf46, SUC_lexicon_tags_65 },
    { 0xe3d4bf47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f14bf49, SUC_lexicon_tags_4 },
    { 0xf614bf4a, SUC_lexicon_tags_21 },
    { 0x3a34bf49, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f4bf50, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x3634bf53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b4bf57, SUC_lexicon_tags_12 },
    { 0x5f74bf57, SUC_lexicon_tags_12 },
    { 0xc594bf59, SUC_lexicon_tags_274 },
    { 0x53b4bf5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee94bf64, SUC_lexicon_tags_7 },
    { 0x5a34bf64, SUC_lexicon_tags_28 },
    { 0xdcf4bf66, SUC_lexicon_tags_37 },
    { 0x16d4bf67, SUC_lexicon_tags_7 },
    { 0x9bb4bf68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d4bf6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5fb4bf70, SUC_lexicon_tags_11 },
    { 0x4514bf70, SUC_lexicon_tags_1 },
    { 0x1814bf72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9794bf75, SUC_lexicon_tags_10 },
    { 0x6974bf75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f4bf7c, SUC_lexicon_tags_37 },
    { 0x8a14bf7d, SUC_lexicon_tags_12 },
    { 0xe94bf7e, SUC_lexicon_tags_17 },
    { 0xaf34bf7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed4bf82, SUC_lexicon_tags_30 },
    { 0xcbd4bf82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf254bf86, SUC_lexicon_tags_17 },
    { 0x22b4bf87, SUC_lexicon_tags_77 },
    { 0xbcb4bf87, SUC_lexicon_tags_34 },
    { 0xffb4bf89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4834bf8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b4bf8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaf4bf90, SUC_lexicon_tags_11 },
    { 0x8514bf90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec34bf96, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1834bf9a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c54bfa0, SUC_lexicon_tags_0 },
    { 0x4bf4bfa1, SUC_lexicon_tags_15 },
    { 0x85f4bfa0, SUC_lexicon_tags_26 },
    { 0x7934bfa3, SUC_lexicon_tags_353 },
    { 0x3994bfa4, SUC_lexicon_tags_37 },
    { 0x26f4bfa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4214bfa9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3754bfae, SUC_lexicon_tags_12 },
    { 0x4674bfaf, SUC_lexicon_tags_899 },
    { 0, NULL },
    { 0x7834bfb1, SUC_lexicon_tags_27 },
    { 0x79f4bfb2, SUC_lexicon_tags_2 },
    { 0x10b4bfb3, SUC_lexicon_tags_12 },
    { 0xccd4bfb3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f4bfb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb914bfb9, SUC_lexicon_tags_29 },
    { 0x3874bfba, SUC_lexicon_tags_49 },
    { 0xd494bfbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d4bfbe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7094bfc1, SUC_lexicon_tags_37 },
    { 0x2a74bfc2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114bfc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x114bfcb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d34bfce, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1db4bfd0, SUC_lexicon_tags_17 },
    { 0xd6d4bfd1, SUC_lexicon_tags_11 },
    { 0xc4b4bfd1, SUC_lexicon_tags_37 },
    { 0xfb34bfd1, SUC_lexicon_tags_27 },
    { 0x7f54bfd2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x62d4bfd6, SUC_lexicon_tags_17 },
    { 0x8af4bfd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe914bfdf, SUC_lexicon_tags_1 },
    { 0xe7b4bfe0, SUC_lexicon_tags_1 },
    { 0x9154bfe1, SUC_lexicon_tags_37 },
    { 0x7f54bfe1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ff4bfe4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7014bfe6, SUC_lexicon_tags_37 },
    { 0x88d4bfe7, SUC_lexicon_tags_23 },
    { 0x7cf4bfe6, SUC_lexicon_tags_42 },
    { 0xd4bfe9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b4bfee, SUC_lexicon_tags_11 },
    { 0x1f94bfee, SUC_lexicon_tags_29 },
    { 0xb374bfef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4734bff7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874bffc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9654bffe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa394c000, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x86d4c002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac94c004, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe94c007, SUC_lexicon_tags_1 },
    { 0x3374c008, SUC_lexicon_tags_0 },
    { 0x9174c009, SUC_lexicon_tags_1 },
    { 0xd774c008, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3b4c00c, SUC_lexicon_tags_12 },
    { 0x1a94c00c, SUC_lexicon_tags_10 },
    { 0x6074c00e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d4c012, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60f4c014, SUC_lexicon_tags_6 },
    { 0x154c015, SUC_lexicon_tags_38 },
    { 0x1a74c016, SUC_lexicon_tags_45 },
    { 0x4f94c016, SUC_lexicon_tags_18 },
    { 0x7214c018, SUC_lexicon_tags_25 },
    { 0xa5b4c018, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x89b4c01b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc194c01d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa034c01f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e34c024, SUC_lexicon_tags_43 },
    { 0xb914c024, SUC_lexicon_tags_2 },
    { 0xda34c026, SUC_lexicon_tags_31 },
    { 0xe2d4c027, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe934c02c, SUC_lexicon_tags_60 },
    { 0x7914c02d, SUC_lexicon_tags_37 },
    { 0xf14c02e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e74c031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6654c035, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd4c039, SUC_lexicon_tags_32 },
    { 0x47f4c03a, SUC_lexicon_tags_0 },
    { 0xa254c03b, SUC_lexicon_tags_65 },
    { 0x7894c03a, SUC_lexicon_tags_29 },
    { 0x6594c03a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b4c046, SUC_lexicon_tags_8 },
    { 0x9634c047, SUC_lexicon_tags_11 },
    { 0xe054c048, SUC_lexicon_tags_12 },
    { 0x85b4c047, SUC_lexicon_tags_1 },
    { 0xc74c04a, SUC_lexicon_tags_62 },
    { 0x9b34c04a, SUC_lexicon_tags_85 },
    { 0x4054c047, SUC_lexicon_tags_13 },
    { 0xef94c04d, SUC_lexicon_tags_2 },
    { 0xbf4c04a, SUC_lexicon_tags_2 },
    { 0xd574c046, SUC_lexicon_tags_4 },
    { 0xaf14c050, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d4c057, SUC_lexicon_tags_116 },
    { 0xecb4c058, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3054c05a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ed4c05c, SUC_lexicon_tags_8 },
    { 0xe9f4c05c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2094c05f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26d4c061, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc014c063, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d4c068, SUC_lexicon_tags_1 },
    { 0x7fd4c069, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f74c077, SUC_lexicon_tags_8 },
    { 0x8ed4c077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x414c07a, SUC_lexicon_tags_0 },
    { 0x2b4c07b, SUC_lexicon_tags_62 },
    { 0xa894c07c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34c082, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd4c086, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc174c088, SUC_lexicon_tags_25 },
    { 0xe174c089, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9814c08e, SUC_lexicon_tags_26 },
    { 0xb354c08f, SUC_lexicon_tags_8 },
    { 0x1194c08f, SUC_lexicon_tags_37 },
    { 0x52d4c091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc94c094, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef4c099, SUC_lexicon_tags_12 },
    { 0xc0f4c09a, SUC_lexicon_tags_46 },
    { 0xc9b4c099, SUC_lexicon_tags_25 },
    { 0x1654c09c, SUC_lexicon_tags_17 },
    { 0x26d4c09b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9794c0a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5974c0a2, SUC_lexicon_tags_13 },
    { 0x71f4c0a2, SUC_lexicon_tags_1 },
    { 0x7e34c0a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x33f4c0a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe74c0a9, SUC_lexicon_tags_15 },
    { 0xd514c0aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4c0b1, SUC_lexicon_tags_25 },
    { 0x5614c0b1, SUC_lexicon_tags_23 },
    { 0x6c94c0b3, SUC_lexicon_tags_26 },
    { 0x7d4c0b4, SUC_lexicon_tags_38 },
    { 0x1e14c0b5, SUC_lexicon_tags_43 },
    { 0xe554c0b5, SUC_lexicon_tags_31 },
    { 0x4d54c0b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xad74c0ba, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd214c0c0, SUC_lexicon_tags_37 },
    { 0xf234c0c1, SUC_lexicon_tags_816 },
    { 0x34b4c0c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2454c0c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4214c0c7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2954c0c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8414c0cb, SUC_lexicon_tags_12 },
    { 0x8834c0cc, SUC_lexicon_tags_8 },
    { 0xc634c0cd, SUC_lexicon_tags_15 },
    { 0x2514c0cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4d4c0d0, SUC_lexicon_tags_82 },
    { 0x2f74c0d1, SUC_lexicon_tags_8 },
    { 0x69b4c0d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa954c0da, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa514c0dc, SUC_lexicon_tags_15 },
    { 0x11b4c0dc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7bf4c0df, SUC_lexicon_tags_15 },
    { 0x5f34c0e0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3974c0ea, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe34c0f1, SUC_lexicon_tags_37 },
    { 0x2654c0f2, SUC_lexicon_tags_7 },
    { 0xbb14c0f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54c0f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd74c0fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d4c0ff, SUC_lexicon_tags_14 },
    { 0xbb4c0ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8f4c102, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x41f4c104, SUC_lexicon_tags_19 },
    { 0x76d4c105, SUC_lexicon_tags_17 },
    { 0x25b4c105, SUC_lexicon_tags_0 },
    { 0x1614c107, SUC_lexicon_tags_64 },
    { 0x6ab4c107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8eb4c10a, SUC_lexicon_tags_34 },
    { 0x1434c10a, SUC_lexicon_tags_12 },
    { 0x3cb4c10b, SUC_lexicon_tags_47 },
    { 0xc554c10c, SUC_lexicon_tags_2 },
    { 0x1554c10b, SUC_lexicon_tags_11 },
    { 0x5d4c10f, SUC_lexicon_tags_126 },
    { 0xb274c110, SUC_lexicon_tags_0 },
    { 0x7b34c111, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5714c114, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7434c11c, SUC_lexicon_tags_23 },
    { 0x794c11d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d4c121, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea54c126, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7ab4c128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d54c12d, SUC_lexicon_tags_11 },
    { 0x6a14c12d, SUC_lexicon_tags_37 },
    { 0x3874c12f, SUC_lexicon_tags_70 },
    { 0x5614c130, SUC_lexicon_tags_9 },
    { 0x4354c12f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2034c133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd374c135, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc414c137, SUC_lexicon_tags_62 },
    { 0x4e54c138, SUC_lexicon_tags_45 },
    { 0x1e74c139, SUC_lexicon_tags_61 },
    { 0xc154c13a, SUC_lexicon_tags_15 },
    { 0xed14c13b, SUC_lexicon_tags_1 },
    { 0xbb4c13a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b4c144, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd4c147, SUC_lexicon_tags_11 },
    { 0x2814c147, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb9d4c14a, SUC_lexicon_tags_12 },
    { 0x9b14c14b, SUC_lexicon_tags_43 },
    { 0x8b54c14b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2454c14f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1774c151, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2074c153, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d4c156, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e74c158, SUC_lexicon_tags_22 },
    { 0xf5b4c159, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd5b4c15b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb4c15e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b4c161, SUC_lexicon_tags_1 },
    { 0xb14c162, SUC_lexicon_tags_77 },
    { 0x9ef4c161, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bd4c165, SUC_lexicon_tags_37 },
    { 0xf3b4c166, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa334c168, SUC_lexicon_tags_34 },
    { 0xe7d4c169, SUC_lexicon_tags_22 },
    { 0xc214c16a, SUC_lexicon_tags_7 },
    { 0xe6b4c16b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f4c16e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfd4c170, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc234c17b, SUC_lexicon_tags_11 },
    { 0x63f4c17c, SUC_lexicon_tags_49 },
    { 0xb2f4c17b, SUC_lexicon_tags_6 },
    { 0x1134c17e, SUC_lexicon_tags_1 },
    { 0x76f4c17f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9814c185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab34c187, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b54c189, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd4c18e, SUC_lexicon_tags_43 },
    { 0x3534c18f, SUC_lexicon_tags_25 },
    { 0x7f4c190, SUC_lexicon_tags_19 },
    { 0xc834c191, SUC_lexicon_tags_26 },
    { 0x7ff4c192, SUC_lexicon_tags_9 },
    { 0x1b4c192, SUC_lexicon_tags_7 },
    { 0x45d4c194, SUC_lexicon_tags_26 },
    { 0xeaf4c195, SUC_lexicon_tags_6 },
    { 0xfbf4c18e, SUC_lexicon_tags_12 },
    { 0x8c54c18e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd4c19b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3034c19d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe834c1a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9614c1ae, SUC_lexicon_tags_37 },
    { 0x76d4c1af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd4c1b3, SUC_lexicon_tags_1 },
    { 0x7154c1b3, SUC_lexicon_tags_49 },
    { 0xd354c1b5, SUC_lexicon_tags_8 },
    { 0x4f54c1b5, SUC_lexicon_tags_47 },
    { 0x57f4c1b7, SUC_lexicon_tags_151 },
    { 0xf374c1b8, SUC_lexicon_tags_37 },
    { 0xb74c1b3, SUC_lexicon_tags_17 },
    { 0xaf34c1b4, SUC_lexicon_tags_39 },
    { 0xe374c1bb, SUC_lexicon_tags_23 },
    { 0x8c74c1bc, SUC_lexicon_tags_32 },
    { 0xb74c1bd, SUC_lexicon_tags_16 },
    { 0x8af4c1be, SUC_lexicon_tags_60 },
    { 0xd2f4c1b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c54c1c3, SUC_lexicon_tags_43 },
    { 0xdf14c1c4, SUC_lexicon_tags_2 },
    { 0x5754c1c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd4c1c8, SUC_lexicon_tags_37 },
    { 0x95d4c1c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9694c1cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1494c1d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7234c1d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3434c1da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f14c1dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe54c1de, SUC_lexicon_tags_37 },
    { 0xceb4c1df, SUC_lexicon_tags_23 },
    { 0xd074c1e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22d4c1e2, SUC_lexicon_tags_2 },
    { 0x44d4c1e3, SUC_lexicon_tags_12 },
    { 0x14d4c1e4, SUC_lexicon_tags_60 },
    { 0xa2f4c1e3, SUC_lexicon_tags_17 },
    { 0x71b4c1e6, SUC_lexicon_tags_17 },
    { 0x61f4c1e4, SUC_lexicon_tags_85 },
    { 0xb3f4c1e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db4c1eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b74c1ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae94c1f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6114c1f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7254c1f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf14c1fa, SUC_lexicon_tags_48 },
    { 0xe314c1fb, SUC_lexicon_tags_49 },
    { 0xa474c1fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9774c1ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce94c203, SUC_lexicon_tags_7 },
    { 0x7454c204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd94c208, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ab4c20a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8f4c20c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f54c20e, SUC_lexicon_tags_2 },
    { 0x1d4c20f, SUC_lexicon_tags_19 },
    { 0x2b94c210, SUC_lexicon_tags_35 },
    { 0x7954c20e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6214c214, SUC_lexicon_tags_37 },
    { 0x8dd4c215, SUC_lexicon_tags_37 },
    { 0xda14c215, SUC_lexicon_tags_17 },
    { 0xaa14c215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf4c21c, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb4c220, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2394c226, SUC_lexicon_tags_11 },
    { 0x81f4c227, SUC_lexicon_tags_68 },
    { 0x59b4c228, SUC_lexicon_tags_1 },
    { 0x5794c229, SUC_lexicon_tags_37 },
    { 0x9fd4c22a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7294c22c, SUC_lexicon_tags_1 },
    { 0xaed4c22d, SUC_lexicon_tags_9 },
    { 0xb614c22e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaed4c230, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc54c23a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6314c23d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d4c247, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f4c24f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7fb4c251, SUC_lexicon_tags_11 },
    { 0xb134c252, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd514c258, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefb4c25a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x9a74c25c, SUC_lexicon_tags_31 },
    { 0x41d4c25d, SUC_lexicon_tags_37 },
    { 0x7614c25e, SUC_lexicon_tags_25 },
    { 0xc214c25e, SUC_lexicon_tags_12 },
    { 0x4f34c260, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec34c265, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa14c26c, SUC_lexicon_tags_26 },
    { 0xb3b4c26c, SUC_lexicon_tags_15 },
    { 0x5394c26e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f4c273, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e94c277, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb4c27a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc594c27d, SUC_lexicon_tags_39 },
    { 0x16d4c27d, SUC_lexicon_tags_0 },
    { 0xcbf4c27d, SUC_lexicon_tags_7 },
    { 0x77f4c27d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc734c282, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb494c288, SUC_lexicon_tags_7 },
    { 0x9794c288, SUC_lexicon_tags_23 },
    { 0xa994c288, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d4c28f, SUC_lexicon_tags_25 },
    { 0x18f4c290, SUC_lexicon_tags_12 },
    { 0xf494c290, SUC_lexicon_tags_9 },
    { 0x574c292, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f4c295, SUC_lexicon_tags_7 },
    { 0xdf34c295, SUC_lexicon_tags_9 },
    { 0xabb4c296, SUC_lexicon_tags_25 },
    { 0x5014c298, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c54c29b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd4c2a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1834c2a7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xef54c2aa, SUC_lexicon_tags_60 },
    { 0x6454c2ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf014c2ad, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x1674c2af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca74c2b3, SUC_lexicon_tags_0 },
    { 0xe114c2b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6514c2b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe354c2bb, SUC_lexicon_tags_17 },
    { 0xd9f4c2bc, SUC_lexicon_tags_63 },
    { 0x8d54c2bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf4c2c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f4c2c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9334c2cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d4c2d3, SUC_lexicon_tags_7 },
    { 0x1554c2d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4374c2d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d4c2dd, SUC_lexicon_tags_33 },
    { 0x45b4c2dd, SUC_lexicon_tags_55 },
    { 0x2294c2df, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9634c2e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb934c2ed, SUC_lexicon_tags_11 },
    { 0x2f34c2ee, SUC_lexicon_tags_25 },
    { 0x7654c2ef, SUC_lexicon_tags_15 },
    { 0xacd4c2ef, SUC_lexicon_tags_224 },
    { 0xf354c2f0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d34c2f4, SUC_lexicon_tags_14 },
    { 0x8e34c2f4, SUC_lexicon_tags_11 },
    { 0xaef4c2f5, SUC_lexicon_tags_11 },
    { 0x84f4c2f7, SUC_lexicon_tags_27 },
    { 0x5d4c2f6, SUC_lexicon_tags_60 },
    { 0xa3d4c2f9, SUC_lexicon_tags_65 },
    { 0xa8d4c2f4, SUC_lexicon_tags_2 },
    { 0x8e54c2f5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe694c2fd, SUC_lexicon_tags_6 },
    { 0x62b4c2fe, SUC_lexicon_tags_17 },
    { 0xe894c2fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab4c304, SUC_lexicon_tags_68 },
    { 0x6534c305, SUC_lexicon_tags_32 },
    { 0xef74c306, SUC_lexicon_tags_7 },
    { 0xa074c306, SUC_lexicon_tags_15 },
    { 0x2114c308, SUC_lexicon_tags_68 },
    { 0xd254c308, SUC_lexicon_tags_47 },
    { 0xca34c30a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc414c30e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa14c311, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25f4c313, SUC_lexicon_tags_39 },
    { 0xc514c313, SUC_lexicon_tags_17 },
    { 0x2314c315, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6154c31b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc354c320, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfcf4c322, SUC_lexicon_tags_31 },
    { 0x90f4c323, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4574c325, SUC_lexicon_tags_33 },
    { 0x8814c325, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb4b4c328, SUC_lexicon_tags_11 },
    { 0xb0f4c328, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d4c32c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d4c336, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7ad4c338, SUC_lexicon_tags_37 },
    { 0x19f4c339, SUC_lexicon_tags_45 },
    { 0x27b4c33a, SUC_lexicon_tags_1 },
    { 0xff4c339, SUC_lexicon_tags_26 },
    { 0x1e94c33c, SUC_lexicon_tags_2 },
    { 0x5b4c33c, SUC_lexicon_tags_8 },
    { 0x8f4c33c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecd4c340, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b4c346, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b4c349, SUC_lexicon_tags_15 },
    { 0xb834c349, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa774c354, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3cf4c356, SUC_lexicon_tags_6 },
    { 0xc8f4c357, SUC_lexicon_tags_63 },
    { 0x6534c358, SUC_lexicon_tags_12 },
    { 0xa714c356, SUC_lexicon_tags_38 },
    { 0x2ad4c35a, SUC_lexicon_tags_43 },
    { 0x3c14c35b, SUC_lexicon_tags_39 },
    { 0x1c34c35b, SUC_lexicon_tags_15 },
    { 0x79f4c35d, SUC_lexicon_tags_25 },
    { 0x294c358, SUC_lexicon_tags_25 },
    { 0x9754c35f, SUC_lexicon_tags_12 },
    { 0x7fb4c357, SUC_lexicon_tags_37 },
    { 0x16f4c361, SUC_lexicon_tags_22 },
    { 0xa974c35d, SUC_lexicon_tags_17 },
    { 0xc2f4c362, SUC_lexicon_tags_47 },
    { 0xe094c364, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3254c366, SUC_lexicon_tags_19 },
    { 0xc8f4c367, SUC_lexicon_tags_7 },
    { 0x6df4c367, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8cf4c36a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded4c36e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb4c373, SUC_lexicon_tags_43 },
    { 0x7fb4c374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c34c376, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd74c378, SUC_lexicon_tags_43 },
    { 0x1b94c378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf154c37e, SUC_lexicon_tags_11 },
    { 0xd7d4c37e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6754c381, SUC_lexicon_tags_49 },
    { 0xea34c382, SUC_lexicon_tags_14 },
    { 0x50b4c382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f4c386, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x3374c389, SUC_lexicon_tags_12 },
    { 0x16d4c38a, SUC_lexicon_tags_63 },
    { 0xde94c38b, SUC_lexicon_tags_27 },
    { 0x1474c38b, SUC_lexicon_tags_25 },
    { 0xd294c38d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7754c38f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb4c392, SUC_lexicon_tags_19 },
    { 0xaf54c393, SUC_lexicon_tags_27 },
    { 0xcf54c392, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e74c398, SUC_lexicon_tags_61 },
    { 0xe514c399, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf654c39d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba74c39f, SUC_lexicon_tags_37 },
    { 0x8134c39f, SUC_lexicon_tags_45 },
    { 0xc14c3a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdad4c3a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e54c3a7, SUC_lexicon_tags_15 },
    { 0x7d34c3a8, SUC_lexicon_tags_23 },
    { 0xcd14c3a9, SUC_lexicon_tags_16 },
    { 0x5f34c3a8, SUC_lexicon_tags_11 },
    { 0xda74c3ab, SUC_lexicon_tags_64 },
    { 0xc3d4c3ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4c74c3ae, SUC_lexicon_tags_15 },
    { 0xe494c3af, SUC_lexicon_tags_12 },
    { 0x3b34c3b0, SUC_lexicon_tags_12 },
    { 0x9334c3b1, SUC_lexicon_tags_38 },
    { 0xe814c3b1, SUC_lexicon_tags_25 },
    { 0x94f4c3b3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3994c3b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8d4c3b7, SUC_lexicon_tags_11 },
    { 0xbe74c3b8, SUC_lexicon_tags_21 },
    { 0xd114c3b9, SUC_lexicon_tags_30 },
    { 0xc94c3ba, SUC_lexicon_tags_26 },
    { 0x6674c3bb, SUC_lexicon_tags_816 },
    { 0x2cb4c3b8, SUC_lexicon_tags_19 },
    { 0x74f4c3bb, SUC_lexicon_tags_25 },
    { 0x9174c3b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb4c3c1, SUC_lexicon_tags_7 },
    { 0xbd4c3c2, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x9834c3c5, SUC_lexicon_tags_11 },
    { 0x5fb4c3c6, SUC_lexicon_tags_17 },
    { 0x70d4c3c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7b4c3c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc254c3cd, SUC_lexicon_tags_30 },
    { 0x5934c3ce, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb54c3d1, SUC_lexicon_tags_63 },
    { 0xa54c3d2, SUC_lexicon_tags_23 },
    { 0x3494c3d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2074c3d5, SUC_lexicon_tags_47 },
    { 0xe614c3d5, SUC_lexicon_tags_4 },
    { 0xca94c3d7, SUC_lexicon_tags_25 },
    { 0xb314c3d8, SUC_lexicon_tags_14 },
    { 0x35b4c3d5, SUC_lexicon_tags_54 },
    { 0x13f4c3d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3074c3df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b4c3e2, SUC_lexicon_tags_30 },
    { 0x9ef4c3e3, SUC_lexicon_tags_2 },
    { 0x4114c3e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1954c3e7, SUC_lexicon_tags_11 },
    { 0x84d4c3e8, SUC_lexicon_tags_37 },
    { 0x1454c3e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5194c3ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab54c3ee, SUC_lexicon_tags_33 },
    { 0x69f4c3ee, SUC_lexicon_tags_15 },
    { 0xeb14c3ef, SUC_lexicon_tags_68 },
    { 0xe934c3f1, SUC_lexicon_tags_34 },
    { 0xab4c3f2, SUC_lexicon_tags_2 },
    { 0xc134c3f3, SUC_lexicon_tags_15 },
    { 0x2794c3ee, SUC_lexicon_tags_11 },
    { 0xfbb4c3f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b4c3fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa114c400, SUC_lexicon_tags_8 },
    { 0xff94c401, SUC_lexicon_tags_37 },
    { 0x8114c402, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x614c404, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f94c408, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x51f4c40a, SUC_lexicon_tags_4 },
    { 0xab14c40a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5014c40d, SUC_lexicon_tags_17 },
    { 0xecf4c40e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2094c410, SUC_lexicon_tags_46 },
    { 0xf3b4c410, SUC_lexicon_tags_48 },
    { 0x83b4c411, SUC_lexicon_tags_0 },
    { 0x5854c413, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b54c417, SUC_lexicon_tags_45 },
    { 0xf114c418, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed4c41d, SUC_lexicon_tags_24 },
    { 0x7cf4c41d, SUC_lexicon_tags_17 },
    { 0xb4f4c41f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb4c424, SUC_lexicon_tags_11 },
    { 0xcf74c424, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6f4c427, SUC_lexicon_tags_28 },
    { 0x9a94c428, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xef4c42b, SUC_lexicon_tags_25 },
    { 0xde74c42c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6074c42e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d4c431, SUC_lexicon_tags_11 },
    { 0x8d54c432, SUC_lexicon_tags_37 },
    { 0x59d4c431, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf114c438, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff4c43b, SUC_lexicon_tags_19 },
    { 0xb454c43b, SUC_lexicon_tags_91 },
    { 0xd0b4c43d, SUC_lexicon_tags_0 },
    { 0x9694c43d, SUC_lexicon_tags_1 },
    { 0xf774c43f, SUC_lexicon_tags_12 },
    { 0xd3d4c43d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5114c442, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad34c446, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f4c449, SUC_lexicon_tags_30 },
    { 0xf714c44a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb854c44c, SUC_lexicon_tags_8 },
    { 0x54d4c44c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17d4c44f, SUC_lexicon_tags_50 },
    { 0x8df4c450, SUC_lexicon_tags_15 },
    { 0xf54c451, SUC_lexicon_tags_45 },
    { 0x5e74c452, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f54c456, SUC_lexicon_tags_27 },
    { 0x8154c456, SUC_lexicon_tags_11 },
    { 0xc574c458, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b4c45d, SUC_lexicon_tags_17 },
    { 0x8d14c45e, SUC_lexicon_tags_0 },
    { 0x8b94c45f, SUC_lexicon_tags_91 },
    { 0x7b54c460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c74c465, SUC_lexicon_tags_17 },
    { 0xa1b4c466, SUC_lexicon_tags_31 },
    { 0x54b4c467, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2a54c469, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4514c46e, SUC_lexicon_tags_67 },
    { 0x1294c46e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18b4c471, SUC_lexicon_tags_23 },
    { 0xa554c472, SUC_lexicon_tags_60 },
    { 0x8114c473, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x28b4c475, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d4c477, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b4c47a, SUC_lexicon_tags_13 },
    { 0x6894c47a, SUC_lexicon_tags_45 },
    { 0xff34c47a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b74c47e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe74c481, SUC_lexicon_tags_43 },
    { 0x2db4c482, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3754c484, SUC_lexicon_tags_63 },
    { 0xf3d4c485, SUC_lexicon_tags_66 },
    { 0xfef4c486, SUC_lexicon_tags_23 },
    { 0xd534c485, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb4c48c, SUC_lexicon_tags_17 },
    { 0x4994c48d, SUC_lexicon_tags_35 },
    { 0xd734c48e, SUC_lexicon_tags_38 },
    { 0x694c48e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f4c494, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7794c497, SUC_lexicon_tags_19 },
    { 0x13f4c498, SUC_lexicon_tags_7 },
    { 0xe494c499, SUC_lexicon_tags_23 },
    { 0x7cf4c499, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf494c49e, SUC_lexicon_tags_15 },
    { 0x6274c49e, SUC_lexicon_tags_7 },
    { 0xc94c49f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5474c4ab, SUC_lexicon_tags_30 },
    { 0xc6d4c4ac, SUC_lexicon_tags_33 },
    { 0xe3f4c4ad, SUC_lexicon_tags_2 },
    { 0xf554c4ae, SUC_lexicon_tags_33 },
    { 0xf394c4ad, SUC_lexicon_tags_63 },
    { 0xaf4c4ac, SUC_lexicon_tags_25 },
    { 0xe0f4c4b0, SUC_lexicon_tags_11 },
    { 0x8274c4b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74c4b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5474c4b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d4c4bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5634c4c5, SUC_lexicon_tags_47 },
    { 0x9414c4c6, SUC_lexicon_tags_27 },
    { 0xc7d4c4c7, SUC_lexicon_tags_25 },
    { 0x7af4c4c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc994c4cb, SUC_lexicon_tags_39 },
    { 0x2f74c4cc, SUC_lexicon_tags_19 },
    { 0x6f4c4cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf4c4d1, SUC_lexicon_tags_14 },
    { 0xbf74c4d2, SUC_lexicon_tags_11 },
    { 0xfa54c4d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1d4c4d5, SUC_lexicon_tags_31 },
    { 0x6974c4d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9594c4d9, SUC_lexicon_tags_82 },
    { 0xe754c4da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c94c4de, SUC_lexicon_tags_27 },
    { 0xaa94c4de, SUC_lexicon_tags_55 },
    { 0xead4c4e0, SUC_lexicon_tags_17 },
    { 0x9ed4c4e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5114c4e4, SUC_lexicon_tags_16 },
    { 0x2254c4e5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xb114c4e8, SUC_lexicon_tags_37 },
    { 0x5214c4e8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e94c4f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f4c4f4, SUC_lexicon_tags_8 },
    { 0xcf14c4f4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a74c4fa, SUC_lexicon_tags_49 },
    { 0xe5d4c4fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d74c4ff, SUC_lexicon_tags_64 },
    { 0x35b4c500, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b4c503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6494c507, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd374c50f, SUC_lexicon_tags_0 },
    { 0xdff4c510, SUC_lexicon_tags_23 },
    { 0xc4f4c511, SUC_lexicon_tags_38 },
    { 0xf9b4c511, SUC_lexicon_tags_7 },
    { 0x7494c513, SUC_lexicon_tags_31 },
    { 0x5134c510, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1794c518, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e54c51d, SUC_lexicon_tags_47 },
    { 0xfb54c51d, SUC_lexicon_tags_7 },
    { 0xa6d4c51e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b4c523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4994c528, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4934c52d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7254c52f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef4c534, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d4c539, SUC_lexicon_tags_17 },
    { 0xdf14c539, SUC_lexicon_tags_2 },
    { 0x5a94c53a, SUC_lexicon_tags_17 },
    { 0x79f4c53b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf094c53e, SUC_lexicon_tags_7 },
    { 0xa4d4c53e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b4c544, SUC_lexicon_tags_87 },
    { 0x18b4c545, SUC_lexicon_tags_45 },
    { 0x8174c545, SUC_lexicon_tags_70 },
    { 0xe3d4c547, SUC_lexicon_tags_46 },
    { 0xfed4c546, SUC_lexicon_tags_37 },
    { 0x9b14c549, SUC_lexicon_tags_29 },
    { 0xd5b4c548, SUC_lexicon_tags_15 },
    { 0x5434c54b, SUC_lexicon_tags_11 },
    { 0x4594c548, SUC_lexicon_tags_77 },
    { 0xd0b4c54a, SUC_lexicon_tags_65 },
    { 0x1ff4c548, SUC_lexicon_tags_45 },
    { 0xc114c548, SUC_lexicon_tags_47 },
    { 0x5c54c547, SUC_lexicon_tags_29 },
    { 0x97f4c551, SUC_lexicon_tags_1 },
    { 0x794c552, SUC_lexicon_tags_25 },
    { 0xbd34c546, SUC_lexicon_tags_31 },
    { 0x63b4c554, SUC_lexicon_tags_17 },
    { 0x4df4c554, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d4c559, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ef4c55b, SUC_lexicon_tags_201 },
    { 0x2d34c55c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x17d4c55e, SUC_lexicon_tags_60 },
    { 0x5d14c55f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d4c563, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e74c567, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc854c56b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b4c56e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9034c571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f14c579, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf394c57b, SUC_lexicon_tags_15 },
    { 0xd834c57b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3034c582, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x334c586, SUC_lexicon_tags_11 },
    { 0x1f94c586, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8354c58a, SUC_lexicon_tags_17 },
    { 0x5294c58b, SUC_lexicon_tags_8 },
    { 0xe214c58b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14c592, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf1d4c594, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2694c596, SUC_lexicon_tags_85 },
    { 0xca74c597, SUC_lexicon_tags_2 },
    { 0x38f4c598, SUC_lexicon_tags_3 },
    { 0xe874c599, SUC_lexicon_tags_47 },
    { 0xe1f4c59a, SUC_lexicon_tags_23 },
    { 0x3134c59b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x914c59d, SUC_lexicon_tags_11 },
    { 0x37b4c59e, SUC_lexicon_tags_2 },
    { 0x74d4c59f, SUC_lexicon_tags_60 },
    { 0xe8f4c5a0, SUC_lexicon_tags_14 },
    { 0xd2b4c5a1, SUC_lexicon_tags_38 },
    { 0x9114c5a2, SUC_lexicon_tags_49 },
    { 0xd674c5a3, SUC_lexicon_tags_33 },
    { 0x4934c5a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b4c5ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ad4c5ad, SUC_lexicon_tags_25 },
    { 0x1d34c5ad, SUC_lexicon_tags_11 },
    { 0xc134c5af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2734c5b6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe894c5c4, SUC_lexicon_tags_0 },
    { 0xa4f4c5c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b54c5c7, SUC_lexicon_tags_7 },
    { 0xc0b4c5c8, SUC_lexicon_tags_15 },
    { 0x594c5c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d4c5cc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x73b4c5ce, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6954c5d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef4c5d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x87f4c5da, SUC_lexicon_tags_0 },
    { 0xad4c5db, SUC_lexicon_tags_38 },
    { 0x1554c5da, SUC_lexicon_tags_49 },
    { 0xa914c5da, SUC_lexicon_tags_37 },
    { 0xab4c5de, SUC_lexicon_tags_17 },
    { 0xab74c5dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89f4c5e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3234c5e6, SUC_lexicon_tags_23 },
    { 0x9874c5e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d4c5ec, SUC_lexicon_tags_25 },
    { 0xb394c5ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7174c5f0, SUC_lexicon_tags_17 },
    { 0x5ed4c5f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b4c5f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9714c5f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7194c5fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb0d4c600, SUC_lexicon_tags_26 },
    { 0x30d4c601, SUC_lexicon_tags_47 },
    { 0x3d54c601, SUC_lexicon_tags_11 },
    { 0x4a74c603, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f14c606, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x574c609, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3274c60b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd534c60f, SUC_lexicon_tags_56 },
    { 0x6454c610, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9014c613, SUC_lexicon_tags_1 },
    { 0xee34c614, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c74c616, SUC_lexicon_tags_201 },
    { 0x4f54c617, SUC_lexicon_tags_4 },
    { 0x7a74c618, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff4c61b, SUC_lexicon_tags_37 },
    { 0xc514c61c, SUC_lexicon_tags_7 },
    { 0x9a34c61c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf74c620, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe534c624, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4c627, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf34c629, SUC_lexicon_tags_11 },
    { 0x9fd4c62a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd974c62c, SUC_lexicon_tags_17 },
    { 0x8514c62d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b4c634, SUC_lexicon_tags_14 },
    { 0x8674c634, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b4c63c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6574c643, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb4c651, SUC_lexicon_tags_33 },
    { 0x4694c652, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d4c655, SUC_lexicon_tags_25 },
    { 0xa5f4c656, SUC_lexicon_tags_2 },
    { 0xb214c657, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6774c659, SUC_lexicon_tags_29 },
    { 0xeb94c65a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce54c65c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3d4c65e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff4c663, SUC_lexicon_tags_61 },
    { 0xe394c664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd454c666, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4554c66b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b4c673, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f14c676, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f14c67c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f54c68e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53d4c690, SUC_lexicon_tags_25 },
    { 0xab4c691, SUC_lexicon_tags_10 },
    { 0xa34c692, SUC_lexicon_tags_0 },
    { 0x9374c693, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc94c696, SUC_lexicon_tags_1 },
    { 0xc054c696, SUC_lexicon_tags_12 },
    { 0x8d34c698, SUC_lexicon_tags_38 },
    { 0x3d14c698, SUC_lexicon_tags_11 },
    { 0xeb34c69a, SUC_lexicon_tags_56 },
    { 0x86d4c697, SUC_lexicon_tags_14 },
    { 0xabf4c69c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d4c6a4, SUC_lexicon_tags_55 },
    { 0xcd54c6a4, SUC_lexicon_tags_63 },
    { 0x9434c6a6, SUC_lexicon_tags_12 },
    { 0xfa54c6a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f4c6aa, SUC_lexicon_tags_19 },
    { 0x3034c6ab, SUC_lexicon_tags_9 },
    { 0xab4c6ac, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8834c6b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d94c6b5, SUC_lexicon_tags_60 },
    { 0xf974c6b6, SUC_lexicon_tags_25 },
    { 0x6074c6b7, SUC_lexicon_tags_1 },
    { 0x5474c6b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6eb4c6ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e74c6be, SUC_lexicon_tags_15 },
    { 0x79f4c6bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d4c6c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc314c6c9, SUC_lexicon_tags_38 },
    { 0x4e14c6ca, SUC_lexicon_tags_45 },
    { 0xd934c6cb, SUC_lexicon_tags_37 },
    { 0x6074c6cc, SUC_lexicon_tags_14 },
    { 0x5a94c6c9, SUC_lexicon_tags_11 },
    { 0xa094c6c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3714c6de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb534c6e0, SUC_lexicon_tags_87 },
    { 0xebd4c6e1, SUC_lexicon_tags_47 },
    { 0xe14c6e2, SUC_lexicon_tags_49 },
    { 0xc834c6e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab4c6e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4c6ea, SUC_lexicon_tags_38 },
    { 0xd9f4c6eb, SUC_lexicon_tags_37 },
    { 0xd3f4c6ec, SUC_lexicon_tags_37 },
    { 0x9bd4c6ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df4c6f0, SUC_lexicon_tags_17 },
    { 0xf514c6f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdfd4c6f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d4c6f7, SUC_lexicon_tags_1 },
    { 0x94d4c6f8, SUC_lexicon_tags_38 },
    { 0x2a34c6f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3394c6fe, SUC_lexicon_tags_49 },
    { 0x4b34c6ff, SUC_lexicon_tags_27 },
    { 0xfef4c700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe014c703, SUC_lexicon_tags_27 },
    { 0x38b4c704, SUC_lexicon_tags_15 },
    { 0xef4c703, SUC_lexicon_tags_15 },
    { 0x3e34c704, SUC_lexicon_tags_37 },
    { 0xcab4c707, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e94c70a, SUC_lexicon_tags_61 },
    { 0x6354c70b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2934c70d, SUC_lexicon_tags_39 },
    { 0x5af4c70d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b4c714, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda94c718, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2694c71a, SUC_lexicon_tags_38 },
    { 0x31d4c71b, SUC_lexicon_tags_15 },
    { 0xe0d4c71a, SUC_lexicon_tags_17 },
    { 0x6534c71b, SUC_lexicon_tags_2 },
    { 0x10d4c71e, SUC_lexicon_tags_38 },
    { 0x8634c71f, SUC_lexicon_tags_191 },
    { 0x9554c71e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b4c723, SUC_lexicon_tags_1 },
    { 0x1d54c723, SUC_lexicon_tags_25 },
    { 0xca74c725, SUC_lexicon_tags_21 },
    { 0x67d4c725, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f74c729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c74c72e, SUC_lexicon_tags_25 },
    { 0x8774c72e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b4c733, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3894c735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2474c73c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce74c741, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbf4c743, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc34c745, SUC_lexicon_tags_60 },
    { 0x33d4c746, SUC_lexicon_tags_6 },
    { 0x5674c747, SUC_lexicon_tags_34 },
    { 0x3234c747, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4c74f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d4c755, SUC_lexicon_tags_19 },
    { 0x7fb4c756, SUC_lexicon_tags_17 },
    { 0xdc34c756, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5034c75f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5914c762, SUC_lexicon_tags_84 },
    { 0xb5d4c763, SUC_lexicon_tags_10 },
    { 0x4c14c764, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaed4c766, SUC_lexicon_tags_9 },
    { 0x20d4c767, SUC_lexicon_tags_39 },
    { 0xb094c768, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3494c76a, SUC_lexicon_tags_31 },
    { 0x52b4c76b, SUC_lexicon_tags_37 },
    { 0xf714c76a, SUC_lexicon_tags_1 },
    { 0xc4b4c76a, SUC_lexicon_tags_46 },
    { 0x4754c76e, SUC_lexicon_tags_37 },
    { 0xb434c76f, SUC_lexicon_tags_37 },
    { 0x1db4c76d, SUC_lexicon_tags_15 },
    { 0xc394c76a, SUC_lexicon_tags_12 },
    { 0x43f4c76f, SUC_lexicon_tags_47 },
    { 0x10b4c76c, SUC_lexicon_tags_10 },
    { 0x73d4c774, SUC_lexicon_tags_1 },
    { 0xc634c76e, SUC_lexicon_tags_2 },
    { 0xfcd4c76f, SUC_lexicon_tags_30 },
    { 0xfcb4c777, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b4c77a, SUC_lexicon_tags_2 },
    { 0x3d54c77b, SUC_lexicon_tags_1 },
    { 0xffd4c77c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf994c77f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd154c783, SUC_lexicon_tags_23 },
    { 0xe494c783, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc14c78a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf514c78d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x474c78f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce54c791, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa194c798, SUC_lexicon_tags_37 },
    { 0xff74c798, SUC_lexicon_tags_30 },
    { 0x4754c799, SUC_lexicon_tags_17 },
    { 0x3154c799, SUC_lexicon_tags_11 },
    { 0xee54c798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce14c7a3, SUC_lexicon_tags_9 },
    { 0xa594c7a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4c7a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f4c7ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd4c7b1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa414c7b3, SUC_lexicon_tags_27 },
    { 0xd294c7b4, SUC_lexicon_tags_65 },
    { 0x5af4c7b5, SUC_lexicon_tags_7 },
    { 0xb5b4c7b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9054c7b9, SUC_lexicon_tags_1 },
    { 0x93d4c7ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc94c7bd, SUC_lexicon_tags_49 },
    { 0xfed4c7be, SUC_lexicon_tags_30 },
    { 0x8f4c7bd, SUC_lexicon_tags_47 },
    { 0x76b4c7c0, SUC_lexicon_tags_15 },
    { 0x7774c7c0, SUC_lexicon_tags_33 },
    { 0xe7b4c7c2, SUC_lexicon_tags_23 },
    { 0x6774c7bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b4c7c6, SUC_lexicon_tags_0 },
    { 0x5a74c7c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd4c7cb, SUC_lexicon_tags_7 },
    { 0x3134c7cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa214c7d2, SUC_lexicon_tags_15 },
    { 0xb3f4c7d3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff4c7d7, SUC_lexicon_tags_9 },
    { 0xecb4c7d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2454c7da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f4c7dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe14c7e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5234c7e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8874c7e6, SUC_lexicon_tags_14 },
    { 0x75d4c7e7, SUC_lexicon_tags_12 },
    { 0x8e74c7e7, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xf94c7eb, SUC_lexicon_tags_33 },
    { 0x2774c7ec, SUC_lexicon_tags_65 },
    { 0x7e74c7eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b4c7f0, SUC_lexicon_tags_1 },
    { 0xc854c7f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcc94c7f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb214c7f6, SUC_lexicon_tags_2 },
    { 0x25f4c7f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8f4c7f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b74c7fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38d4c7fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7554c802, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b54c806, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98b4c808, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe434c80a, SUC_lexicon_tags_1 },
    { 0x97b4c80b, SUC_lexicon_tags_25 },
    { 0xaf54c80c, SUC_lexicon_tags_37 },
    { 0x2614c80d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4194c810, SUC_lexicon_tags_12 },
    { 0xe014c811, SUC_lexicon_tags_17 },
    { 0x8db4c812, SUC_lexicon_tags_6 },
    { 0x2af4c812, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3574c816, SUC_lexicon_tags_7 },
    { 0xab14c817, SUC_lexicon_tags_1 },
    { 0x8254c818, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f54c81c, SUC_lexicon_tags_1 },
    { 0xd0d4c81d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d54c820, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf914c82a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f4c82e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef4c831, SUC_lexicon_tags_63 },
    { 0xdcd4c831, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2754c837, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x614c839, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x69f4c83b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd314c83e, SUC_lexicon_tags_9 },
    { 0x5874c83f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec14c845, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xceb4c847, SUC_lexicon_tags_6 },
    { 0x5794c847, SUC_lexicon_tags_29 },
    { 0xca14c847, SUC_lexicon_tags_11 },
    { 0x4414c84a, SUC_lexicon_tags_30 },
    { 0xc294c84b, SUC_lexicon_tags_253 },
    { 0x7334c84c, SUC_lexicon_tags_201 },
    { 0x1b4c84d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe34c84f, SUC_lexicon_tags_470 },
    { 0xfe14c850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a34c853, SUC_lexicon_tags_15 },
    { 0x4534c854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1814c856, SUC_lexicon_tags_37 },
    { 0xbd54c856, SUC_lexicon_tags_37 },
    { 0x8f74c858, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa434c85b, SUC_lexicon_tags_70 },
    { 0xfaf4c85b, SUC_lexicon_tags_69 },
    { 0xb4b4c85b, SUC_lexicon_tags_37 },
    { 0x11b4c85e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea54c860, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc14c862, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb14c867, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb4c86a, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xab94c86c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3594c86e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a74c871, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad4c875, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14c87d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1db4c87f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b4c882, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d4c885, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x294c887, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8034c88c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e94c88f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e14c892, SUC_lexicon_tags_17 },
    { 0x79d4c893, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40f4c895, SUC_lexicon_tags_8 },
    { 0xcc34c896, SUC_lexicon_tags_43 },
    { 0x11b4c896, SUC_lexicon_tags_33 },
    { 0x5594c897, SUC_lexicon_tags_51 },
    { 0x8e74c898, SUC_lexicon_tags_38 },
    { 0x54f4c896, SUC_lexicon_tags_12 },
    { 0xff54c89a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ef4c89d, SUC_lexicon_tags_33 },
    { 0x18f4c89e, SUC_lexicon_tags_42 },
    { 0x8914c89f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb4c8a1, SUC_lexicon_tags_11 },
    { 0xc794c8a2, SUC_lexicon_tags_27 },
    { 0x64d4c8a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2ab4c8a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9b34c8a7, SUC_lexicon_tags_60 },
    { 0x7554c8a7, SUC_lexicon_tags_53 },
    { 0x5234c8a8, SUC_lexicon_tags_43 },
    { 0x1954c8aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b4c8ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe794c8b1, SUC_lexicon_tags_14 },
    { 0xe5d4c8b1, SUC_lexicon_tags_4 },
    { 0xa354c8b3, SUC_lexicon_tags_63 },
    { 0xa7b4c8b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f4c8b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd14c8c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f54c8c4, SUC_lexicon_tags_47 },
    { 0x2bf4c8c4, SUC_lexicon_tags_37 },
    { 0xfaf4c8c4, SUC_lexicon_tags_3 },
    { 0x7934c8c7, SUC_lexicon_tags_2 },
    { 0xed94c8c7, SUC_lexicon_tags_1 },
    { 0xdd74c8c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb74c8cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af4c8d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb4c8d6, SUC_lexicon_tags_40 },
    { 0xbeb4c8d7, SUC_lexicon_tags_27 },
    { 0xaaf4c8d7, SUC_lexicon_tags_17 },
    { 0xc0b4c8d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2814c8dc, SUC_lexicon_tags_15 },
    { 0x6bd4c8dd, SUC_lexicon_tags_37 },
    { 0xa6b4c8dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb74c8e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e34c8e9, SUC_lexicon_tags_40 },
    { 0x23b4c8e9, SUC_lexicon_tags_10 },
    { 0x9834c8ea, SUC_lexicon_tags_29 },
    { 0x934c8ea, SUC_lexicon_tags_1 },
    { 0xef14c8ed, SUC_lexicon_tags_37 },
    { 0x91f4c8ea, SUC_lexicon_tags_38 },
    { 0x6614c8ef, SUC_lexicon_tags_61 },
    { 0xe994c8ef, SUC_lexicon_tags_11 },
    { 0x4f94c8f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf874c8f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4834c8f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2194c900, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4214c906, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdb34c908, SUC_lexicon_tags_14 },
    { 0x7db4c908, SUC_lexicon_tags_11 },
    { 0x3f74c90a, SUC_lexicon_tags_45 },
    { 0x86f4c90b, SUC_lexicon_tags_2 },
    { 0x8354c90c, SUC_lexicon_tags_17 },
    { 0xaa34c90b, SUC_lexicon_tags_19 },
    { 0xa0b4c90e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db4c911, SUC_lexicon_tags_25 },
    { 0x9db4c911, SUC_lexicon_tags_17 },
    { 0xedd4c911, SUC_lexicon_tags_19 },
    { 0x6a94c914, SUC_lexicon_tags_17 },
    { 0xb094c913, SUC_lexicon_tags_2 },
    { 0x6dd4c916, SUC_lexicon_tags_2 },
    { 0x5974c917, SUC_lexicon_tags_37 },
    { 0xc614c918, SUC_lexicon_tags_2 },
    { 0x7014c913, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd514c91c, SUC_lexicon_tags_15 },
    { 0x1994c91c, SUC_lexicon_tags_15 },
    { 0xf554c91d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3a14c920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4474c923, SUC_lexicon_tags_8 },
    { 0x7034c924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9094c926, SUC_lexicon_tags_14 },
    { 0xa534c927, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7754c92f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b4c938, SUC_lexicon_tags_49 },
    { 0xdaf4c939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f54c93c, SUC_lexicon_tags_63 },
    { 0xdbd4c93c, SUC_lexicon_tags_10 },
    { 0x3e94c93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9334c940, SUC_lexicon_tags_15 },
    { 0xca54c940, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe014c943, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1034c945, SUC_lexicon_tags_16 },
    { 0xcad4c946, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9fd4c948, SUC_lexicon_tags_7 },
    { 0x574c949, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ef4c94b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3074c952, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d4c957, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b4c95a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed4c964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f4c971, SUC_lexicon_tags_17 },
    { 0xdbf4c971, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4b4c974, SUC_lexicon_tags_654 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b4c978, SUC_lexicon_tags_61 },
    { 0xc1d4c979, SUC_lexicon_tags_60 },
    { 0xc134c979, SUC_lexicon_tags_25 },
    { 0xc6b4c97b, SUC_lexicon_tags_31 },
    { 0xe734c97c, SUC_lexicon_tags_2 },
    { 0x30b4c97b, SUC_lexicon_tags_133 },
    { 0x18b4c97d, SUC_lexicon_tags_15 },
    { 0x6474c979, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d4c98a, SUC_lexicon_tags_19 },
    { 0x70d4c98b, SUC_lexicon_tags_31 },
    { 0xf4c98c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8394c98e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf974c990, SUC_lexicon_tags_61 },
    { 0xbe14c991, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1154c995, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7034c999, SUC_lexicon_tags_9 },
    { 0x2614c99a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea94c99c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe34c9a1, SUC_lexicon_tags_15 },
    { 0xf874c9a2, SUC_lexicon_tags_25 },
    { 0xb2d4c9a3, SUC_lexicon_tags_25 },
    { 0xdcf4c9a1, SUC_lexicon_tags_2 },
    { 0x1094c9a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc494c9a8, SUC_lexicon_tags_21 },
    { 0xfc74c9a9, SUC_lexicon_tags_66 },
    { 0xad54c9aa, SUC_lexicon_tags_49 },
    { 0xfb4c9aa, SUC_lexicon_tags_38 },
    { 0x6a54c9a9, SUC_lexicon_tags_43 },
    { 0xe3d4c9ad, SUC_lexicon_tags_34 },
    { 0x9d74c9ae, SUC_lexicon_tags_37 },
    { 0xdff4c9ae, SUC_lexicon_tags_75 },
    { 0x4ff4c9af, SUC_lexicon_tags_14 },
    { 0x3694c9ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa514c9b6, SUC_lexicon_tags_15 },
    { 0xa734c9b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b94c9ba, SUC_lexicon_tags_38 },
    { 0xd094c9bb, SUC_lexicon_tags_53 },
    { 0x6bf4c9bc, SUC_lexicon_tags_37 },
    { 0x1d54c9ba, SUC_lexicon_tags_29 },
    { 0x72d4c9be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb734c9c0, SUC_lexicon_tags_11 },
    { 0x7df4c9c0, SUC_lexicon_tags_15 },
    { 0x454c9c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c14c9c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb4c9ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5754c9cc, SUC_lexicon_tags_37 },
    { 0x2fb4c9cd, SUC_lexicon_tags_37 },
    { 0xe2b4c9cc, SUC_lexicon_tags_25 },
    { 0xa5f4c9cf, SUC_lexicon_tags_45 },
    { 0x5e34c9d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2e34c9d2, SUC_lexicon_tags_37 },
    { 0xf194c9d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3414c9d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d4c9db, SUC_lexicon_tags_62 },
    { 0x75d4c9dc, SUC_lexicon_tags_38 },
    { 0xf8b4c9dd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2074c9df, SUC_lexicon_tags_2 },
    { 0xf5f4c9e0, SUC_lexicon_tags_0 },
    { 0x6934c9e1, SUC_lexicon_tags_9 },
    { 0x23d4c9e1, SUC_lexicon_tags_25 },
    { 0x4f94c9e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd0d4c9e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed14c9e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae54c9ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7214c9f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd494c9f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf514c9f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b4c9fe, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6894ca00, SUC_lexicon_tags_38 },
    { 0xcf34ca01, SUC_lexicon_tags_46 },
    { 0xf534ca00, SUC_lexicon_tags_1 },
    { 0xe374ca03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91b4ca05, SUC_lexicon_tags_7 },
    { 0x5814ca06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x874ca0b, SUC_lexicon_tags_14 },
    { 0x1674ca0c, SUC_lexicon_tags_12 },
    { 0x8614ca0c, SUC_lexicon_tags_17 },
    { 0xb1f4ca0c, SUC_lexicon_tags_38 },
    { 0xf574ca0f, SUC_lexicon_tags_6 },
    { 0xa234ca10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d4ca13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa614ca16, SUC_lexicon_tags_15 },
    { 0x4094ca16, SUC_lexicon_tags_60 },
    { 0xf774ca18, SUC_lexicon_tags_12 },
    { 0xf0b4ca19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8274ca1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4534ca22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa34ca2b, SUC_lexicon_tags_47 },
    { 0x2ff4ca2b, SUC_lexicon_tags_39 },
    { 0xa4b4ca2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6914ca32, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xe5d4ca34, SUC_lexicon_tags_12 },
    { 0x26b4ca35, SUC_lexicon_tags_12 },
    { 0x2674ca35, SUC_lexicon_tags_87 },
    { 0x7494ca35, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d4ca3f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1074ca47, SUC_lexicon_tags_37 },
    { 0x1374ca48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb94ca4b, SUC_lexicon_tags_70 },
    { 0x3214ca4c, SUC_lexicon_tags_25 },
    { 0x3154ca4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa854ca53, SUC_lexicon_tags_11 },
    { 0x6d74ca53, SUC_lexicon_tags_17 },
    { 0x1f94ca55, SUC_lexicon_tags_11 },
    { 0xb494ca56, SUC_lexicon_tags_37 },
    { 0x9e94ca55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a74ca5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb54ca60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a94ca62, SUC_lexicon_tags_14 },
    { 0xbd94ca63, SUC_lexicon_tags_25 },
    { 0x48d4ca64, SUC_lexicon_tags_11 },
    { 0xfd4ca65, SUC_lexicon_tags_141 },
    { 0xb1d4ca62, SUC_lexicon_tags_40 },
    { 0x2b14ca63, SUC_lexicon_tags_25 },
    { 0x52f4ca67, SUC_lexicon_tags_8 },
    { 0x5b34ca69, SUC_lexicon_tags_37 },
    { 0x3814ca6a, SUC_lexicon_tags_26 },
    { 0x4654ca6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d74ca6f, SUC_lexicon_tags_25 },
    { 0xd814ca70, SUC_lexicon_tags_15 },
    { 0x3c34ca71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b54ca77, SUC_lexicon_tags_43 },
    { 0x4b74ca78, SUC_lexicon_tags_11 },
    { 0xdf34ca78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f4ca7c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x58f4ca7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd14ca81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd534ca88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4ca8d, SUC_lexicon_tags_30 },
    { 0x1134ca8e, SUC_lexicon_tags_25 },
    { 0x9414ca8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9154ca94, SUC_lexicon_tags_1 },
    { 0xbc54ca94, SUC_lexicon_tags_179 },
    { 0xc034ca94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e94ca98, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2594ca9a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb4ca9e, SUC_lexicon_tags_29 },
    { 0xb2b4ca9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b4caa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbb4caa7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4e74caa9, SUC_lexicon_tags_1 },
    { 0xf3b4caaa, SUC_lexicon_tags_37 },
    { 0x7214caab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7134cab2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3af4cab4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd4cab9, SUC_lexicon_tags_1 },
    { 0x1754caba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe4f4cabc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa454cabe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8b4cac0, SUC_lexicon_tags_39 },
    { 0x4674cac0, SUC_lexicon_tags_26 },
    { 0xa774cac0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f4cac5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee94cacc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81d4cace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb4cad4, SUC_lexicon_tags_15 },
    { 0xf2f4cad4, SUC_lexicon_tags_17 },
    { 0x1554cad5, SUC_lexicon_tags_52 },
    { 0x99f4cad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc114cada, SUC_lexicon_tags_2 },
    { 0xd354cada, SUC_lexicon_tags_10 },
    { 0xd394cadc, SUC_lexicon_tags_27 },
    { 0xf934cadd, SUC_lexicon_tags_11 },
    { 0x6ed4cadd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb594cae0, SUC_lexicon_tags_47 },
    { 0x3914cae0, SUC_lexicon_tags_9 },
    { 0x12d4cae2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd74cae6, SUC_lexicon_tags_25 },
    { 0x3c74cae7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d4caed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd694caf8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd74cb01, SUC_lexicon_tags_27 },
    { 0x95b4cb02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa14cb05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd4cb0e, SUC_lexicon_tags_1 },
    { 0x2274cb0e, SUC_lexicon_tags_12 },
    { 0x6cb4cb10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1894cb12, SUC_lexicon_tags_0 },
    { 0x22d4cb12, SUC_lexicon_tags_32 },
    { 0xf234cb14, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d54cb18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd34cb21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1654cb24, SUC_lexicon_tags_37 },
    { 0x6634cb25, SUC_lexicon_tags_7 },
    { 0xaa14cb26, SUC_lexicon_tags_37 },
    { 0xd3f4cb27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41d4cb29, SUC_lexicon_tags_8 },
    { 0x2e34cb2a, SUC_lexicon_tags_2 },
    { 0xe894cb2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef74cb2d, SUC_lexicon_tags_6 },
    { 0xb114cb2d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5214cb33, SUC_lexicon_tags_60 },
    { 0xff34cb34, SUC_lexicon_tags_390 },
    { 0x6174cb35, SUC_lexicon_tags_66 },
    { 0x3274cb35, SUC_lexicon_tags_0 },
    { 0x7914cb37, SUC_lexicon_tags_31 },
    { 0xe3d4cb35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbe54cb3a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4cb44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb4cb4a, SUC_lexicon_tags_37 },
    { 0xdc74cb4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7594cb4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb4cb53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5614cb55, SUC_lexicon_tags_25 },
    { 0x9f74cb55, SUC_lexicon_tags_60 },
    { 0x4534cb55, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4434cb5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b4cb60, SUC_lexicon_tags_11 },
    { 0x89f4cb61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b4cb64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d4cb67, SUC_lexicon_tags_25 },
    { 0x9f34cb68, SUC_lexicon_tags_33 },
    { 0x1354cb68, SUC_lexicon_tags_68 },
    { 0x7414cb68, SUC_lexicon_tags_30 },
    { 0xe2d4cb6b, SUC_lexicon_tags_14 },
    { 0x6af4cb6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe54cb6f, SUC_lexicon_tags_15 },
    { 0xc534cb70, SUC_lexicon_tags_27 },
    { 0x7914cb71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4494cb73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6474cb75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ff4cb77, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9014cb7a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd54cb7e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf114cb80, SUC_lexicon_tags_14 },
    { 0x8734cb80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f94cb83, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd3d4cb85, SUC_lexicon_tags_33 },
    { 0xaf14cb86, SUC_lexicon_tags_12 },
    { 0x53f4cb87, SUC_lexicon_tags_11 },
    { 0xaf74cb88, SUC_lexicon_tags_0 },
    { 0xe1b4cb89, SUC_lexicon_tags_37 },
    { 0xbd74cb8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced4cb90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7694cb92, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b4cb95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa9b4cb97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5914cb9c, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xbe74cb9e, SUC_lexicon_tags_11 },
    { 0x1514cb9f, SUC_lexicon_tags_11 },
    { 0xd614cba0, SUC_lexicon_tags_47 },
    { 0x2234cba0, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d74cba4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5534cba8, SUC_lexicon_tags_25 },
    { 0x4ff4cba9, SUC_lexicon_tags_7 },
    { 0x6ad4cbaa, SUC_lexicon_tags_17 },
    { 0xc4f4cbab, SUC_lexicon_tags_36 },
    { 0x6d94cba8, SUC_lexicon_tags_16 },
    { 0xc714cbad, SUC_lexicon_tags_172 },
    { 0xeaf4cbab, SUC_lexicon_tags_14 },
    { 0x57b4cbad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd534cbb3, SUC_lexicon_tags_27 },
    { 0x34b4cbb4, SUC_lexicon_tags_97 },
    { 0xe74cbb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d4cbb9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1214cbbb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7314cbc1, SUC_lexicon_tags_63 },
    { 0xda34cbc1, SUC_lexicon_tags_29 },
    { 0xbf34cbc1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca34cbc7, SUC_lexicon_tags_8 },
    { 0x7f14cbc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab4cbcb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd4cbce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc134cbd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3d4cbd4, SUC_lexicon_tags_118 },
    { 0x1a74cbd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d4cbd8, SUC_lexicon_tags_41 },
    { 0xe454cbd9, SUC_lexicon_tags_37 },
    { 0x1694cbda, SUC_lexicon_tags_25 },
    { 0xbeb4cbdb, SUC_lexicon_tags_17 },
    { 0x8254cbd8, SUC_lexicon_tags_12 },
    { 0x8ad4cbdc, SUC_lexicon_tags_44 },
    { 0x1374cbd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb934cbe0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x254cbe2, SUC_lexicon_tags_25 },
    { 0x9b4cbe2, SUC_lexicon_tags_43 },
    { 0x15d4cbe4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa634cbea, SUC_lexicon_tags_14 },
    { 0x61d4cbeb, SUC_lexicon_tags_11 },
    { 0x54f4cbeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b4cbfa, SUC_lexicon_tags_21 },
    { 0x6354cbfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f4cc02, SUC_lexicon_tags_8 },
    { 0x4374cc02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab4cc08, SUC_lexicon_tags_11 },
    { 0x9174cc09, SUC_lexicon_tags_26 },
    { 0x9274cc0a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4ff4cc0c, SUC_lexicon_tags_42 },
    { 0x4b54cc0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x38b4cc0f, SUC_lexicon_tags_275 },
    { 0xe734cc10, SUC_lexicon_tags_1 },
    { 0xd4f4cc0f, SUC_lexicon_tags_27 },
    { 0x3db4cc12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6974cc14, SUC_lexicon_tags_10 },
    { 0xb854cc15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1694cc1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa74cc1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa9b4cc1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x494cc22, SUC_lexicon_tags_16 },
    { 0xdf94cc23, SUC_lexicon_tags_15 },
    { 0xaf94cc23, SUC_lexicon_tags_47 },
    { 0xe8d4cc25, SUC_lexicon_tags_47 },
    { 0x2f74cc25, SUC_lexicon_tags_11 },
    { 0x9914cc25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4954cc29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce94cc2b, SUC_lexicon_tags_30 },
    { 0x77f4cc2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4cc30, SUC_lexicon_tags_27 },
    { 0xbff4cc30, SUC_lexicon_tags_324 },
    { 0, NULL },
    { 0xf774cc33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5474cc35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc654cc39, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd714cc3c, SUC_lexicon_tags_37 },
    { 0xa8d4cc3d, SUC_lexicon_tags_31 },
    { 0xb514cc3c, SUC_lexicon_tags_204 },
    { 0xb554cc3f, SUC_lexicon_tags_25 },
    { 0x85d4cc40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1034cc44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6994cc46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c94cc48, SUC_lexicon_tags_17 },
    { 0xfd14cc48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb4cc4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14cc4f, SUC_lexicon_tags_23 },
    { 0xc734cc4f, SUC_lexicon_tags_27 },
    { 0xa434cc50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6754cc54, SUC_lexicon_tags_1 },
    { 0xb754cc54, SUC_lexicon_tags_13 },
    { 0x334cc56, SUC_lexicon_tags_47 },
    { 0xe854cc57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d4cc5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd4cc64, SUC_lexicon_tags_28 },
    { 0xda54cc65, SUC_lexicon_tags_6 },
    { 0x5d74cc66, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d4cc69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b54cc70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1814cc72, SUC_lexicon_tags_12 },
    { 0x64b4cc72, SUC_lexicon_tags_14 },
    { 0xde54cc73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb194cc77, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a14cc7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4274cc83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9834cc8f, SUC_lexicon_tags_37 },
    { 0xf114cc90, SUC_lexicon_tags_56 },
    { 0x9f4cc90, SUC_lexicon_tags_38 },
    { 0x5794cc91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62d4cc94, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1ab4cc96, SUC_lexicon_tags_31 },
    { 0xc514cc96, SUC_lexicon_tags_37 },
    { 0xef54cc97, SUC_lexicon_tags_37 },
    { 0xef54cc99, SUC_lexicon_tags_30 },
    { 0x9434cc97, SUC_lexicon_tags_26 },
    { 0xc4b4cc9b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af4cca2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b54cca6, SUC_lexicon_tags_2 },
    { 0xfc74cca6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd554cca9, SUC_lexicon_tags_30 },
    { 0xbe14ccaa, SUC_lexicon_tags_83 },
    { 0xcfd4ccab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7434ccad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdc14ccaf, SUC_lexicon_tags_65 },
    { 0x4e54ccb0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb4ccb5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb334ccb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2314ccb9, SUC_lexicon_tags_44 },
    { 0x8014ccba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdad4ccbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda74ccbf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9114ccc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad94ccc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf574ccce, SUC_lexicon_tags_572 },
    { 0xbff4ccce, SUC_lexicon_tags_0 },
    { 0xbb54ccd0, SUC_lexicon_tags_14 },
    { 0xe454ccd1, SUC_lexicon_tags_15 },
    { 0xa194ccd2, SUC_lexicon_tags_11 },
    { 0x9ab4ccd3, SUC_lexicon_tags_12 },
    { 0x4194ccd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854ccd9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf614ccdb, SUC_lexicon_tags_27 },
    { 0xaf54ccdc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f4cce1, SUC_lexicon_tags_2 },
    { 0x954cce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4cceb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x48f4cced, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8114ccef, SUC_lexicon_tags_25 },
    { 0x5574ccef, SUC_lexicon_tags_15 },
    { 0x7ef4ccf1, SUC_lexicon_tags_2 },
    { 0xcb94ccf0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd434ccf4, SUC_lexicon_tags_15 },
    { 0x7934ccf5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb734ccf8, SUC_lexicon_tags_41 },
    { 0xc5d4ccf9, SUC_lexicon_tags_6 },
    { 0xe074ccfa, SUC_lexicon_tags_37 },
    { 0x3ad4ccfb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1694ccfd, SUC_lexicon_tags_14 },
    { 0x38f4ccfe, SUC_lexicon_tags_45 },
    { 0x5394ccff, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb1b4cd01, SUC_lexicon_tags_17 },
    { 0xa0d4cd02, SUC_lexicon_tags_46 },
    { 0xd614cd03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b4cd07, SUC_lexicon_tags_35 },
    { 0x4f34cd08, SUC_lexicon_tags_7 },
    { 0x3614cd09, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3234cd0b, SUC_lexicon_tags_0 },
    { 0x7d74cd0b, SUC_lexicon_tags_11 },
    { 0x82f4cd0c, SUC_lexicon_tags_25 },
    { 0x70f4cd0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xce14cd11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1654cd16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3454cd1c, SUC_lexicon_tags_25 },
    { 0xd2d4cd1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6694cd20, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb4cd31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfe94cd33, SUC_lexicon_tags_28 },
    { 0x9434cd34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb854cd37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd394cd3e, SUC_lexicon_tags_56 },
    { 0x49d4cd3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c34cd41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1614cd43, SUC_lexicon_tags_77 },
    { 0xfb34cd44, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6eb4cd46, SUC_lexicon_tags_27 },
    { 0xf5d4cd46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d4cd4b, SUC_lexicon_tags_11 },
    { 0x1b14cd4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4d4cd4e, SUC_lexicon_tags_1 },
    { 0x9af4cd4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab4cd54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe34cd58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a74cd5c, SUC_lexicon_tags_1 },
    { 0x3cd4cd5d, SUC_lexicon_tags_46 },
    { 0x6bd4cd5e, SUC_lexicon_tags_2 },
    { 0x1714cd5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b14cd61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaa34cd63, SUC_lexicon_tags_12 },
    { 0xd0f4cd64, SUC_lexicon_tags_17 },
    { 0xb214cd63, SUC_lexicon_tags_25 },
    { 0xc1f4cd64, SUC_lexicon_tags_10 },
    { 0xabb4cd63, SUC_lexicon_tags_27 },
    { 0xef94cd68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3614cd6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4cd73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf4cd7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4cd81, SUC_lexicon_tags_42 },
    { 0x9354cd82, SUC_lexicon_tags_37 },
    { 0x2f54cd83, SUC_lexicon_tags_19 },
    { 0x1f14cd84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4cd89, SUC_lexicon_tags_25 },
    { 0x7374cd89, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd254cd8e, SUC_lexicon_tags_12 },
    { 0x7294cd8f, SUC_lexicon_tags_19 },
    { 0xe114cd90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3294cd94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4214cd97, SUC_lexicon_tags_45 },
    { 0xc954cd98, SUC_lexicon_tags_56 },
    { 0x72d4cd99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde94cd9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b4cda0, SUC_lexicon_tags_11 },
    { 0x8374cda0, SUC_lexicon_tags_14 },
    { 0xa1b4cda1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e94cda7, SUC_lexicon_tags_10 },
    { 0x71d4cda8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce14cdab, SUC_lexicon_tags_23 },
    { 0xd134cdac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7154cdb2, SUC_lexicon_tags_45 },
    { 0x8894cdb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa94cdb7, SUC_lexicon_tags_13 },
    { 0x9534cdb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32b4cdba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8014cdc0, SUC_lexicon_tags_8 },
    { 0x4c14cdc1, SUC_lexicon_tags_1 },
    { 0x44b4cdc1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6cf4cdc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x854cdc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x44d4cdc9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x554cdcb, SUC_lexicon_tags_3 },
    { 0xb1f4cdcb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd4cdd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9694cdd9, SUC_lexicon_tags_39 },
    { 0x30d4cdda, SUC_lexicon_tags_9 },
    { 0xf6b4cddb, SUC_lexicon_tags_11 },
    { 0x4914cddb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d4cddf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9434cde6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd34cdeb, SUC_lexicon_tags_8 },
    { 0x8a94cdec, SUC_lexicon_tags_15 },
    { 0x78f4cded, SUC_lexicon_tags_11 },
    { 0x874cdee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b4cdf1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6174cdf3, SUC_lexicon_tags_27 },
    { 0x6c94cdf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b4cdfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x274cdfd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8294ce02, SUC_lexicon_tags_7 },
    { 0x3574ce02, SUC_lexicon_tags_29 },
    { 0x7bf4ce03, SUC_lexicon_tags_60 },
    { 0x4b94ce05, SUC_lexicon_tags_31 },
    { 0x9b14ce06, SUC_lexicon_tags_11 },
    { 0x6b14ce07, SUC_lexicon_tags_46 },
    { 0xfe94ce08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb4ce0f, SUC_lexicon_tags_33 },
    { 0xb0d4ce10, SUC_lexicon_tags_113 },
    { 0x6f54ce11, SUC_lexicon_tags_96 },
    { 0x10d4ce10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7834ce1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d54ce1d, SUC_lexicon_tags_1 },
    { 0x74ce1d, SUC_lexicon_tags_1 },
    { 0x5094ce1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4874ce23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdab4ce25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe774ce27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5214ce29, SUC_lexicon_tags_17 },
    { 0xadb4ce2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b74ce2f, SUC_lexicon_tags_17 },
    { 0x6e54ce30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe14ce32, SUC_lexicon_tags_95 },
    { 0xa854ce33, SUC_lexicon_tags_211 },
    { 0xbe34ce33, SUC_lexicon_tags_60 },
    { 0x1934ce33, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3494ce3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc774ce3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a74ce45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f14ce48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4814ce4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8394ce5d, SUC_lexicon_tags_37 },
    { 0x3ef4ce5d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b4ce62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa514ce64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4994ce67, SUC_lexicon_tags_11 },
    { 0x1434ce68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a34ce6f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f4ce72, SUC_lexicon_tags_37 },
    { 0xe674ce73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f94ce79, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b4ce7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb4ce7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b4ce83, SUC_lexicon_tags_17 },
    { 0x6314ce84, SUC_lexicon_tags_25 },
    { 0x2d54ce85, SUC_lexicon_tags_25 },
    { 0x10f4ce84, SUC_lexicon_tags_25 },
    { 0x6bd4ce84, SUC_lexicon_tags_107 },
    { 0xa034ce88, SUC_lexicon_tags_25 },
    { 0xc474ce88, SUC_lexicon_tags_11 },
    { 0xbeb4ce8a, SUC_lexicon_tags_17 },
    { 0xad94ce8b, SUC_lexicon_tags_29 },
    { 0xe434ce8a, SUC_lexicon_tags_17 },
    { 0x68d4ce8d, SUC_lexicon_tags_15 },
    { 0x3974ce8e, SUC_lexicon_tags_65 },
    { 0x4c34ce88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6d4ce91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8694ce93, SUC_lexicon_tags_14 },
    { 0x49f4ce93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b4ce98, SUC_lexicon_tags_1 },
    { 0xded4ce98, SUC_lexicon_tags_7 },
    { 0x1154ce99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d4cea4, SUC_lexicon_tags_17 },
    { 0xa614cea5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcc34cea7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x514cea9, SUC_lexicon_tags_1 },
    { 0xa974ceaa, SUC_lexicon_tags_27 },
    { 0x3a74cea9, SUC_lexicon_tags_17 },
    { 0xa9b4ceac, SUC_lexicon_tags_26 },
    { 0x5074ceab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a14ceb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5234ceb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdc74ceb4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f4cebc, SUC_lexicon_tags_15 },
    { 0x82b4cebd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7274cebf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9234cec6, SUC_lexicon_tags_7 },
    { 0xb794cec7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4ceca, SUC_lexicon_tags_14 },
    { 0xb254cecb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b54ced1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca74ced6, SUC_lexicon_tags_14 },
    { 0x6a94ced7, SUC_lexicon_tags_28 },
    { 0x2a94ced6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1334ceda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14cede, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfa94cee0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb814cee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd4cee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cee8, SUC_lexicon_tags_1 },
    { 0x3174cee9, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df4ceee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa074cef3, SUC_lexicon_tags_6 },
    { 0xac54cef4, SUC_lexicon_tags_45 },
    { 0xe2f4cef5, SUC_lexicon_tags_13 },
    { 0xfcd4cef6, SUC_lexicon_tags_50 },
    { 0xc174cef7, SUC_lexicon_tags_15 },
    { 0x8514cef6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x45f4cefa, SUC_lexicon_tags_63 },
    { 0xb174cefb, SUC_lexicon_tags_10 },
    { 0xb454cefc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5694cf01, SUC_lexicon_tags_37 },
    { 0x49d4cf02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc734cf0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c34cf0f, SUC_lexicon_tags_49 },
    { 0x21b4cf10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79b4cf12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd4cf15, SUC_lexicon_tags_30 },
    { 0xcd74cf16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f4cf1f, SUC_lexicon_tags_14 },
    { 0xc414cf20, SUC_lexicon_tags_1 },
    { 0x94d4cf20, SUC_lexicon_tags_7 },
    { 0xc354cf21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc474cf25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5314cf2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4074cf3a, SUC_lexicon_tags_23 },
    { 0x9fd4cf3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5114cf40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed74cf44, SUC_lexicon_tags_2 },
    { 0x39b4cf45, SUC_lexicon_tags_33 },
    { 0xaa34cf46, SUC_lexicon_tags_17 },
    { 0x5c14cf46, SUC_lexicon_tags_25 },
    { 0xd94cf45, SUC_lexicon_tags_25 },
    { 0x4e14cf49, SUC_lexicon_tags_17 },
    { 0xfd74cf4a, SUC_lexicon_tags_11 },
    { 0xbcd4cf4a, SUC_lexicon_tags_1 },
    { 0x9094cf4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf54cf4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4cf51, SUC_lexicon_tags_37 },
    { 0x9a74cf52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2d4cf54, SUC_lexicon_tags_63 },
    { 0x8f94cf55, SUC_lexicon_tags_28 },
    { 0x14f4cf56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6014cf58, SUC_lexicon_tags_17 },
    { 0x15b4cf59, SUC_lexicon_tags_7 },
    { 0x2c54cf5a, SUC_lexicon_tags_25 },
    { 0x7a14cf5b, SUC_lexicon_tags_60 },
    { 0xa934cf5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xafb4cf5e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3974cf61, SUC_lexicon_tags_2 },
    { 0xb8f4cf62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3b4cf64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ed4cf66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd034cf68, SUC_lexicon_tags_30 },
    { 0xe954cf69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd974cf6b, SUC_lexicon_tags_63 },
    { 0x35f4cf6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb574cf70, SUC_lexicon_tags_15 },
    { 0xbb54cf71, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb4cf74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d4cf7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad4cf82, SUC_lexicon_tags_23 },
    { 0x3974cf82, SUC_lexicon_tags_68 },
    { 0x9854cf84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3094cf86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x65b4cf88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8614cf8d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc014cf92, SUC_lexicon_tags_11 },
    { 0xe614cf93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd4cf96, SUC_lexicon_tags_38 },
    { 0x4b94cf97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb4cf9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9294cf9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d14cfa5, SUC_lexicon_tags_11 },
    { 0xb6f4cfa6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52f4cfa8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd754cfac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf354cfae, SUC_lexicon_tags_65 },
    { 0x9734cfaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a54cfb1, SUC_lexicon_tags_12 },
    { 0x8294cfb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8954cfb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2934cfb9, SUC_lexicon_tags_7 },
    { 0xd874cfb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b74cfbd, SUC_lexicon_tags_6 },
    { 0x6b4cfbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f4cfc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6234cfc5, SUC_lexicon_tags_37 },
    { 0xa234cfc6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f4cfd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7394cfd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd974cfd5, SUC_lexicon_tags_46 },
    { 0x9ed4cfd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b4cfde, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d94cfe8, SUC_lexicon_tags_29 },
    { 0x3014cfe8, SUC_lexicon_tags_15 },
    { 0xc3f4cfea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa474cfef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5454cff2, SUC_lexicon_tags_37 },
    { 0x6674cff3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x51f4cff5, SUC_lexicon_tags_37 },
    { 0x5174cff6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcff4cff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d4cffe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14f4d000, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b4d003, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f4d006, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b4d009, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f54d00c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9454d015, SUC_lexicon_tags_15 },
    { 0xb54d016, SUC_lexicon_tags_28 },
    { 0xe4f4d016, SUC_lexicon_tags_17 },
    { 0xcb74d016, SUC_lexicon_tags_25 },
    { 0xdcb4d019, SUC_lexicon_tags_15 },
    { 0x3374d01a, SUC_lexicon_tags_10 },
    { 0x3ed4d016, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9414d01d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe14d020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc94d022, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb4d028, SUC_lexicon_tags_29 },
    { 0x8954d028, SUC_lexicon_tags_2 },
    { 0x6154d02a, SUC_lexicon_tags_29 },
    { 0x3494d02b, SUC_lexicon_tags_17 },
    { 0xeff4d02b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1294d030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x234d038, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc354d03a, SUC_lexicon_tags_47 },
    { 0xbf14d03b, SUC_lexicon_tags_17 },
    { 0x8f14d03a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f54d040, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b34d043, SUC_lexicon_tags_6 },
    { 0x5c94d043, SUC_lexicon_tags_1 },
    { 0x7dd4d045, SUC_lexicon_tags_2 },
    { 0x1e94d046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e94d048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3954d04b, SUC_lexicon_tags_31 },
    { 0xdd4d04c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6774d04e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d4d051, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6034d055, SUC_lexicon_tags_1 },
    { 0x394d056, SUC_lexicon_tags_4 },
    { 0x4b4d056, SUC_lexicon_tags_44 },
    { 0x56b4d056, SUC_lexicon_tags_37 },
    { 0xe8d4d059, SUC_lexicon_tags_37 },
    { 0x9274d059, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b4d05d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f14d061, SUC_lexicon_tags_18 },
    { 0xac94d062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f4d066, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d4d06a, SUC_lexicon_tags_7 },
    { 0xf4d4d06b, SUC_lexicon_tags_6 },
    { 0xf94d06a, SUC_lexicon_tags_62 },
    { 0x9d94d06b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d14d070, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c14d073, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27f4d075, SUC_lexicon_tags_25 },
    { 0x67d4d075, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd4d079, SUC_lexicon_tags_25 },
    { 0x57b4d07a, SUC_lexicon_tags_151 },
    { 0x2874d079, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe634d07e, SUC_lexicon_tags_11 },
    { 0x61f4d07f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc14d081, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bf4d083, SUC_lexicon_tags_23 },
    { 0x37d4d083, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454d08f, SUC_lexicon_tags_15 },
    { 0xd234d090, SUC_lexicon_tags_45 },
    { 0x5854d090, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5734d097, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4794d09b, SUC_lexicon_tags_33 },
    { 0x6e14d09c, SUC_lexicon_tags_25 },
    { 0x9374d09d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e14d09f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf434d0a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf914d0b2, SUC_lexicon_tags_0 },
    { 0xa794d0b3, SUC_lexicon_tags_37 },
    { 0xb854d0b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1354d0b7, SUC_lexicon_tags_18 },
    { 0x6f54d0b8, SUC_lexicon_tags_33 },
    { 0xc494d0b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3814d0bb, SUC_lexicon_tags_37 },
    { 0x9bb4d0bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b4d0c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1694d0c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6634d0c5, SUC_lexicon_tags_8 },
    { 0xfc54d0c6, SUC_lexicon_tags_0 },
    { 0xee54d0c5, SUC_lexicon_tags_25 },
    { 0xc0b4d0c6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6414d0cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88d4d0ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x194d0d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb054d0d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf694d0d7, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5134d0dd, SUC_lexicon_tags_39 },
    { 0xa1b4d0dd, SUC_lexicon_tags_25 },
    { 0x3cd4d0de, SUC_lexicon_tags_84 },
    { 0x44b4d0dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5614d0e3, SUC_lexicon_tags_15 },
    { 0x8074d0e4, SUC_lexicon_tags_47 },
    { 0xebf4d0e5, SUC_lexicon_tags_15 },
    { 0x7074d0e4, SUC_lexicon_tags_11 },
    { 0x8814d0e7, SUC_lexicon_tags_37 },
    { 0xdb34d0e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d4d0ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5bf4d0f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d34d0f3, SUC_lexicon_tags_1 },
    { 0xfc34d0f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87f4d0f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8794d0f8, SUC_lexicon_tags_3 },
    { 0x9914d0f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3ff4d0fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc54d101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f4d108, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89b4d10a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc674d10d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd4d110, SUC_lexicon_tags_30 },
    { 0x81f4d110, SUC_lexicon_tags_37 },
    { 0x1cb4d112, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f4d115, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4674d117, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd734d11c, SUC_lexicon_tags_48 },
    { 0xbb94d11d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x57b4d11f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3db4d121, SUC_lexicon_tags_11 },
    { 0x3534d122, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc954d124, SUC_lexicon_tags_25 },
    { 0x3d4d124, SUC_lexicon_tags_15 },
    { 0xcfd4d124, SUC_lexicon_tags_7 },
    { 0x97f4d126, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d4d12a, SUC_lexicon_tags_28 },
    { 0xeed4d12a, SUC_lexicon_tags_19 },
    { 0x71f4d12b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4014d131, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d94d134, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf774d136, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b14d138, SUC_lexicon_tags_57 },
    { 0x5e14d139, SUC_lexicon_tags_1 },
    { 0x9214d13a, SUC_lexicon_tags_17 },
    { 0x35b4d13b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d4d141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df4d144, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d4d148, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1774d14b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcb54d14d, SUC_lexicon_tags_12 },
    { 0xb1b4d14d, SUC_lexicon_tags_64 },
    { 0x4614d14d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc314d153, SUC_lexicon_tags_46 },
    { 0x3cb4d154, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcd34d156, SUC_lexicon_tags_43 },
    { 0xf214d157, SUC_lexicon_tags_48 },
    { 0x63d4d157, SUC_lexicon_tags_15 },
    { 0x4814d158, SUC_lexicon_tags_82 },
    { 0xe494d15a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b94d15c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd614d15e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4494d161, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6fd4d163, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf54d167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b94d16e, SUC_lexicon_tags_1 },
    { 0xe274d16e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e74d173, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2eb4d175, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b4d17a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd494d180, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab34d185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f4d192, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d14d198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc34d19b, SUC_lexicon_tags_38 },
    { 0xc2d4d19b, SUC_lexicon_tags_55 },
    { 0xf8d4d19d, SUC_lexicon_tags_37 },
    { 0x7d34d19e, SUC_lexicon_tags_17 },
    { 0xa74d19b, SUC_lexicon_tags_19 },
    { 0x72f4d19f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4d1a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5794d1a9, SUC_lexicon_tags_12 },
    { 0x37d4d1aa, SUC_lexicon_tags_0 },
    { 0xccf4d1ab, SUC_lexicon_tags_33 },
    { 0xb5b4d1ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6394d1ae, SUC_lexicon_tags_11 },
    { 0x61d4d1af, SUC_lexicon_tags_172 },
    { 0xe6b4d1b0, SUC_lexicon_tags_11 },
    { 0x77d4d1b1, SUC_lexicon_tags_38 },
    { 0x47b4d1b0, SUC_lexicon_tags_60 },
    { 0xb14d1b3, SUC_lexicon_tags_2 },
    { 0xdb34d1b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd4d1b9, SUC_lexicon_tags_47 },
    { 0x2a14d1b9, SUC_lexicon_tags_23 },
    { 0xdcf4d1bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5fb4d1bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e74d1c0, SUC_lexicon_tags_15 },
    { 0x34d4d1c1, SUC_lexicon_tags_601 },
    { 0x9a14d1c2, SUC_lexicon_tags_39 },
    { 0xd0b4d1c2, SUC_lexicon_tags_6 },
    { 0xf8b4d1c4, SUC_lexicon_tags_6 },
    { 0x93b4d1c1, SUC_lexicon_tags_1 },
    { 0xf6b4d1c4, SUC_lexicon_tags_47 },
    { 0xf3f4d1c7, SUC_lexicon_tags_88 },
    { 0x32f4d1c4, SUC_lexicon_tags_10 },
    { 0x1bf4d1c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af4d1cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd4d1d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2ed4d1d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8034d1e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf4d1e9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed4d1ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf4d1f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3a14d1f6, SUC_lexicon_tags_9 },
    { 0x7054d1f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9154d1fa, SUC_lexicon_tags_29 },
    { 0x2954d1fa, SUC_lexicon_tags_6 },
    { 0x7cf4d1fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d1ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d4d208, SUC_lexicon_tags_27 },
    { 0xbd54d209, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53d4d20b, SUC_lexicon_tags_15 },
    { 0xd374d20c, SUC_lexicon_tags_37 },
    { 0x2d34d20d, SUC_lexicon_tags_17 },
    { 0x45f4d20e, SUC_lexicon_tags_0 },
    { 0x84b4d20f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f14d215, SUC_lexicon_tags_37 },
    { 0x4b94d216, SUC_lexicon_tags_3 },
    { 0x134d216, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b4d21c, SUC_lexicon_tags_0 },
    { 0x7f14d21d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x43d4d21f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d34d223, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed4d22c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b4d230, SUC_lexicon_tags_7 },
    { 0xfef4d231, SUC_lexicon_tags_65 },
    { 0xa274d232, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4274d236, SUC_lexicon_tags_1 },
    { 0xff54d237, SUC_lexicon_tags_25 },
    { 0x9dd4d237, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f4d240, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6214d243, SUC_lexicon_tags_23 },
    { 0x6674d244, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec54d247, SUC_lexicon_tags_17 },
    { 0x134d247, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcd4d24a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xaef4d24c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda34d251, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb4d254, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe3f4d256, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde34d258, SUC_lexicon_tags_11 },
    { 0xa734d259, SUC_lexicon_tags_23 },
    { 0x8e54d258, SUC_lexicon_tags_47 },
    { 0x3fb4d25b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4914d25f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb4d262, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d4d267, SUC_lexicon_tags_160 },
    { 0xfd94d268, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf34d26f, SUC_lexicon_tags_47 },
    { 0x52d4d26f, SUC_lexicon_tags_185 },
    { 0xc14d271, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b94d274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5854d276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf4d279, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa894d27d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee54d282, SUC_lexicon_tags_29 },
    { 0x5294d282, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf4d28a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9f54d28c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf5b4d28e, SUC_lexicon_tags_4 },
    { 0x5f94d28f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd4d292, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3754d295, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ff4d297, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2ad4d299, SUC_lexicon_tags_37 },
    { 0x57b4d29a, SUC_lexicon_tags_60 },
    { 0x8dd4d29b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x61d4d29d, SUC_lexicon_tags_27 },
    { 0x7eb4d29d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x93f4d2a0, SUC_lexicon_tags_109 },
    { 0x9b94d2a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd4d2a4, SUC_lexicon_tags_37 },
    { 0xbab4d2a5, SUC_lexicon_tags_11 },
    { 0x7d14d2a6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b4d2ad, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f4d2b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff4d2b4, SUC_lexicon_tags_1 },
    { 0x2614d2b5, SUC_lexicon_tags_33 },
    { 0x31f4d2b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e94d2be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf74d2c2, SUC_lexicon_tags_11 },
    { 0x4254d2c3, SUC_lexicon_tags_38 },
    { 0x38f4d2c4, SUC_lexicon_tags_34 },
    { 0x3f74d2c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb674d2c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f4d2cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6094d2cf, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc74d2d3, SUC_lexicon_tags_27 },
    { 0x8c14d2d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4214d2db, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc9d4d2dd, SUC_lexicon_tags_29 },
    { 0xf7b4d2de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e14d2e3, SUC_lexicon_tags_56 },
    { 0x5094d2e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd394d2e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd494d2eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b14d2f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6af4d2f2, SUC_lexicon_tags_12 },
    { 0x3d14d2f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d94d2f5, SUC_lexicon_tags_15 },
    { 0xc14d2f6, SUC_lexicon_tags_17 },
    { 0x43f4d2f7, SUC_lexicon_tags_19 },
    { 0x1314d2f8, SUC_lexicon_tags_11 },
    { 0xb3f4d2f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d2fd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xad94d300, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd94d303, SUC_lexicon_tags_38 },
    { 0x6df4d304, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3254d307, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a94d30b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf74d30e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d4d311, SUC_lexicon_tags_55 },
    { 0x2074d312, SUC_lexicon_tags_2 },
    { 0xfcf4d313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa994d31a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf74d31e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf274d320, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7354d323, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x294d32c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2914d32e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d4d333, SUC_lexicon_tags_0 },
    { 0x46b4d333, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f4d338, SUC_lexicon_tags_56 },
    { 0x5294d338, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd634d33c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xad74d33e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb34d340, SUC_lexicon_tags_80 },
    { 0xe834d341, SUC_lexicon_tags_37 },
    { 0xa574d342, SUC_lexicon_tags_26 },
    { 0xb5f4d340, SUC_lexicon_tags_11 },
    { 0x7db4d344, SUC_lexicon_tags_28 },
    { 0xf914d344, SUC_lexicon_tags_11 },
    { 0xe574d345, SUC_lexicon_tags_16 },
    { 0xed94d347, SUC_lexicon_tags_49 },
    { 0x9534d346, SUC_lexicon_tags_38 },
    { 0x2214d344, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x374d34b, SUC_lexicon_tags_47 },
    { 0x64b4d34c, SUC_lexicon_tags_23 },
    { 0x58d4d34c, SUC_lexicon_tags_65 },
    { 0x4d14d34c, SUC_lexicon_tags_834 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c54d354, SUC_lexicon_tags_1 },
    { 0xf4f4d355, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3054d35a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d4d360, SUC_lexicon_tags_2 },
    { 0x8c14d361, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf4d366, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca54d36a, SUC_lexicon_tags_2 },
    { 0x8014d36b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b4d370, SUC_lexicon_tags_11 },
    { 0x17b4d371, SUC_lexicon_tags_29 },
    { 0x9d74d372, SUC_lexicon_tags_26 },
    { 0x9774d373, SUC_lexicon_tags_45 },
    { 0x1954d371, SUC_lexicon_tags_49 },
    { 0x3454d374, SUC_lexicon_tags_27 },
    { 0xc574d376, SUC_lexicon_tags_25 },
    { 0x8dd4d377, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8554d37b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9794d37e, SUC_lexicon_tags_1 },
    { 0xc554d37e, SUC_lexicon_tags_11 },
    { 0xce74d380, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b34d388, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f4d38c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab4d397, SUC_lexicon_tags_1 },
    { 0xb6b4d397, SUC_lexicon_tags_37 },
    { 0xf1d4d399, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7f4d39b, SUC_lexicon_tags_63 },
    { 0xadf4d39b, SUC_lexicon_tags_19 },
    { 0xde14d39d, SUC_lexicon_tags_126 },
    { 0x41f4d39e, SUC_lexicon_tags_77 },
    { 0xa654d39f, SUC_lexicon_tags_17 },
    { 0xd3b4d3a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f4d3a6, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0xe054d3a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf814d3ac, SUC_lexicon_tags_25 },
    { 0x1a54d3ad, SUC_lexicon_tags_37 },
    { 0xc0f4d3ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d4d3b1, SUC_lexicon_tags_7 },
    { 0x6974d3b1, SUC_lexicon_tags_38 },
    { 0x28d4d3b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef4d3b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x86d4d3ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f4d3bf, SUC_lexicon_tags_23 },
    { 0x9354d3c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a34d3c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f4d3cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b4d3d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9634d3e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb94d3e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f34d3ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f4d3ee, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e14d3f3, SUC_lexicon_tags_0 },
    { 0xb874d3f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46f4d3f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad4d3f9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc9f4d3fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e14d3ff, SUC_lexicon_tags_17 },
    { 0xc854d3ff, SUC_lexicon_tags_7 },
    { 0x2674d400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d4d408, SUC_lexicon_tags_1 },
    { 0xee74d409, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3054d40c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3b94d40e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa294d414, SUC_lexicon_tags_47 },
    { 0xb674d414, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b54d41b, SUC_lexicon_tags_31 },
    { 0x394d41c, SUC_lexicon_tags_45 },
    { 0x98d4d41c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f4d421, SUC_lexicon_tags_10 },
    { 0x6414d422, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd474d425, SUC_lexicon_tags_1 },
    { 0xb8d4d425, SUC_lexicon_tags_27 },
    { 0x7294d426, SUC_lexicon_tags_1 },
    { 0x4434d428, SUC_lexicon_tags_82 },
    { 0x3054d428, SUC_lexicon_tags_7 },
    { 0xb954d42a, SUC_lexicon_tags_14 },
    { 0x9274d42b, SUC_lexicon_tags_37 },
    { 0x80b4d42c, SUC_lexicon_tags_7 },
    { 0x2694d42d, SUC_lexicon_tags_2 },
    { 0x6774d427, SUC_lexicon_tags_60 },
    { 0xdfd4d42e, SUC_lexicon_tags_33 },
    { 0x88b4d430, SUC_lexicon_tags_7 },
    { 0xa754d42b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbef4d433, SUC_lexicon_tags_17 },
    { 0xf854d434, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf014d438, SUC_lexicon_tags_23 },
    { 0x954d439, SUC_lexicon_tags_28 },
    { 0x6014d439, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6b4d43c, SUC_lexicon_tags_39 },
    { 0x5ff4d43c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a94d440, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7954d442, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f4d447, SUC_lexicon_tags_65 },
    { 0x4d74d448, SUC_lexicon_tags_12 },
    { 0x21d4d449, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc74d44b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd4d44f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7314d452, SUC_lexicon_tags_11 },
    { 0x9934d453, SUC_lexicon_tags_37 },
    { 0xa914d452, SUC_lexicon_tags_14 },
    { 0xb634d455, SUC_lexicon_tags_37 },
    { 0xa474d456, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb4d45b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b4d465, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe294d468, SUC_lexicon_tags_37 },
    { 0x6df4d469, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5d4d46b, SUC_lexicon_tags_15 },
    { 0xfc54d46b, SUC_lexicon_tags_37 },
    { 0x5c34d46d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf974d470, SUC_lexicon_tags_11 },
    { 0x3bf4d471, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3414d478, SUC_lexicon_tags_21 },
    { 0xea54d479, SUC_lexicon_tags_17 },
    { 0x1cd4d47a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3434d47c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb534d482, SUC_lexicon_tags_9 },
    { 0x74f4d483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b4d485, SUC_lexicon_tags_37 },
    { 0x6234d486, SUC_lexicon_tags_12 },
    { 0xc714d486, SUC_lexicon_tags_7 },
    { 0xd354d486, SUC_lexicon_tags_48 },
    { 0x6654d486, SUC_lexicon_tags_84 },
    { 0xb3f4d48a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2394d48c, SUC_lexicon_tags_79 },
    { 0x2f74d48c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e54d490, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad94d494, SUC_lexicon_tags_45 },
    { 0xa494d495, SUC_lexicon_tags_17 },
    { 0xb6d4d495, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb114d498, SUC_lexicon_tags_25 },
    { 0xddf4d498, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf4d49d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e34d4a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf54d4a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1254d4a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf54d4ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ab4d4ad, SUC_lexicon_tags_26 },
    { 0x2dd4d4ae, SUC_lexicon_tags_12 },
    { 0x2894d4ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b74d4b1, SUC_lexicon_tags_27 },
    { 0xebb4d4b1, SUC_lexicon_tags_30 },
    { 0x4994d4b3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1314d4b6, SUC_lexicon_tags_25 },
    { 0x8374d4b6, SUC_lexicon_tags_15 },
    { 0xbc14d4b7, SUC_lexicon_tags_39 },
    { 0x8254d4b6, SUC_lexicon_tags_12 },
    { 0xbcf4d4b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f4d4c3, SUC_lexicon_tags_11 },
    { 0xf274d4c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b4d4cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b4d4d2, SUC_lexicon_tags_16 },
    { 0x16d4d4d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f4d4d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c34d4da, SUC_lexicon_tags_17 },
    { 0x594d4db, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b14d4e4, SUC_lexicon_tags_93 },
    { 0x9a54d4e5, SUC_lexicon_tags_7 },
    { 0x4eb4d4e6, SUC_lexicon_tags_45 },
    { 0xe174d4e7, SUC_lexicon_tags_8 },
    { 0x93d4d4e6, SUC_lexicon_tags_69 },
    { 0xf4b4d4e6, SUC_lexicon_tags_11 },
    { 0x5d94d4ea, SUC_lexicon_tags_37 },
    { 0x9ed4d4e5, SUC_lexicon_tags_10 },
    { 0x9494d4ec, SUC_lexicon_tags_74 },
    { 0x8bf4d4ed, SUC_lexicon_tags_6 },
    { 0xe794d4e9, SUC_lexicon_tags_15 },
    { 0xfd34d4e6, SUC_lexicon_tags_37 },
    { 0xcfd4d4e6, SUC_lexicon_tags_2 },
    { 0x3594d4e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4634d4f6, SUC_lexicon_tags_11 },
    { 0xd2b4d4f7, SUC_lexicon_tags_40 },
    { 0x934d4f6, SUC_lexicon_tags_11 },
    { 0xfcd4d4f9, SUC_lexicon_tags_1 },
    { 0x1914d4fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd834d4fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae14d503, SUC_lexicon_tags_25 },
    { 0xbd14d503, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2574d506, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb814d509, SUC_lexicon_tags_43 },
    { 0x1ab4d50a, SUC_lexicon_tags_1 },
    { 0x80b4d50b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2cf4d50d, SUC_lexicon_tags_7 },
    { 0xd254d50e, SUC_lexicon_tags_25 },
    { 0x7f34d50f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7294d513, SUC_lexicon_tags_2 },
    { 0x7834d513, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bb4d516, SUC_lexicon_tags_60 },
    { 0xd7f4d517, SUC_lexicon_tags_49 },
    { 0x88d4d516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8174d51b, SUC_lexicon_tags_7 },
    { 0xdd74d51c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb4d51f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2354d526, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf4d52b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d4d52f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75f4d531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5794d534, SUC_lexicon_tags_30 },
    { 0xff94d535, SUC_lexicon_tags_37 },
    { 0x4254d536, SUC_lexicon_tags_17 },
    { 0x294d537, SUC_lexicon_tags_19 },
    { 0x2c34d537, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee54d53a, SUC_lexicon_tags_48 },
    { 0xec74d53a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d4d542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xead4d544, SUC_lexicon_tags_25 },
    { 0xa954d545, SUC_lexicon_tags_11 },
    { 0x2d94d546, SUC_lexicon_tags_25 },
    { 0xa754d547, SUC_lexicon_tags_25 },
    { 0x3fb4d547, SUC_lexicon_tags_25 },
    { 0x9554d546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ab4d54b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9074d54d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee54d54f, SUC_lexicon_tags_2 },
    { 0x5394d550, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e94d556, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xb354d558, SUC_lexicon_tags_37 },
    { 0x4b14d559, SUC_lexicon_tags_15 },
    { 0x2114d55a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d4d55d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x13b4d55f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7714d562, SUC_lexicon_tags_2 },
    { 0xad4d563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af4d566, SUC_lexicon_tags_12 },
    { 0xdd94d567, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd4d569, SUC_lexicon_tags_11 },
    { 0xfb74d56a, SUC_lexicon_tags_14 },
    { 0x4354d569, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdfd4d56d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3754d571, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe74d573, SUC_lexicon_tags_27 },
    { 0xb6f4d574, SUC_lexicon_tags_1 },
    { 0x4a14d575, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa714d57a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2674d583, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d4d589, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcd74d58b, SUC_lexicon_tags_25 },
    { 0x1db4d58c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd234d58f, SUC_lexicon_tags_56 },
    { 0x6474d590, SUC_lexicon_tags_21 },
    { 0x3a74d591, SUC_lexicon_tags_7 },
    { 0x5a54d592, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3114d594, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b4d597, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1074d59b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb4d5a0, SUC_lexicon_tags_29 },
    { 0x4a14d5a1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5774d5a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d4d5ab, SUC_lexicon_tags_10 },
    { 0xa2d4d5ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3494d5af, SUC_lexicon_tags_29 },
    { 0x8594d5b0, SUC_lexicon_tags_2 },
    { 0x7594d5b1, SUC_lexicon_tags_7 },
    { 0x90b4d5b1, SUC_lexicon_tags_7 },
    { 0xa094d5af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a94d5b5, SUC_lexicon_tags_23 },
    { 0x1d74d5b6, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x7174d5b8, SUC_lexicon_tags_15 },
    { 0xb974d5b9, SUC_lexicon_tags_2 },
    { 0x5494d5ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d4d5be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe494d5c1, SUC_lexicon_tags_8 },
    { 0xf454d5c2, SUC_lexicon_tags_47 },
    { 0x4134d5c3, SUC_lexicon_tags_4 },
    { 0x4af4d5c3, SUC_lexicon_tags_37 },
    { 0x6374d5c5, SUC_lexicon_tags_22 },
    { 0x93d4d5c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c14d5c9, SUC_lexicon_tags_21 },
    { 0x5fb4d5ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc174d5ce, SUC_lexicon_tags_11 },
    { 0x71d4d5cf, SUC_lexicon_tags_25 },
    { 0x9b34d5d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2114d5d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d4d5da, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc54d5dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4634d5df, SUC_lexicon_tags_7 },
    { 0x1e74d5e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1814d5e5, SUC_lexicon_tags_65 },
    { 0x64f4d5e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d5ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b4d5ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd974d5f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d4d5f4, SUC_lexicon_tags_17 },
    { 0x9a34d5f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd4d5f9, SUC_lexicon_tags_23 },
    { 0x98f4d5f9, SUC_lexicon_tags_28 },
    { 0xd674d5fb, SUC_lexicon_tags_25 },
    { 0x77d4d5fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e14d5ff, SUC_lexicon_tags_38 },
    { 0x6034d600, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf734d602, SUC_lexicon_tags_17 },
    { 0xa3f4d603, SUC_lexicon_tags_26 },
    { 0xfed4d603, SUC_lexicon_tags_27 },
    { 0x9394d604, SUC_lexicon_tags_1 },
    { 0x4e94d606, SUC_lexicon_tags_25 },
    { 0x4614d606, SUC_lexicon_tags_60 },
    { 0x4c14d606, SUC_lexicon_tags_17 },
    { 0x6c34d608, SUC_lexicon_tags_2 },
    { 0xf514d606, SUC_lexicon_tags_27 },
    { 0x9874d60b, SUC_lexicon_tags_7 },
    { 0xf394d60c, SUC_lexicon_tags_32 },
    { 0xe234d60c, SUC_lexicon_tags_28 },
    { 0x5234d604, SUC_lexicon_tags_7 },
    { 0x52f4d60f, SUC_lexicon_tags_25 },
    { 0x6c14d60f, SUC_lexicon_tags_1 },
    { 0x7094d611, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63b4d613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc674d616, SUC_lexicon_tags_14 },
    { 0x4bb4d617, SUC_lexicon_tags_11 },
    { 0x3294d618, SUC_lexicon_tags_17 },
    { 0x6334d618, SUC_lexicon_tags_1 },
    { 0x21b4d61a, SUC_lexicon_tags_14 },
    { 0xc894d61b, SUC_lexicon_tags_87 },
    { 0x8134d61c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc634d61f, SUC_lexicon_tags_15 },
    { 0xf5d4d620, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4494d622, SUC_lexicon_tags_68 },
    { 0x96d4d623, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4b4d625, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d4d62a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93f4d62c, SUC_lexicon_tags_17 },
    { 0x7114d62d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb4d634, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe554d637, SUC_lexicon_tags_1 },
    { 0xddd4d637, SUC_lexicon_tags_113 },
    { 0x6f34d639, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1634d63b, SUC_lexicon_tags_0 },
    { 0xc9b4d63c, SUC_lexicon_tags_1 },
    { 0xadb4d63d, SUC_lexicon_tags_17 },
    { 0x60f4d63e, SUC_lexicon_tags_17 },
    { 0x9294d63d, SUC_lexicon_tags_14 },
    { 0xa0b4d640, SUC_lexicon_tags_14 },
    { 0x6654d63f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad4d647, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea74d64d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ad4d64f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1854d65b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff4d65e, SUC_lexicon_tags_27 },
    { 0x6394d65f, SUC_lexicon_tags_63 },
    { 0x40b4d660, SUC_lexicon_tags_25 },
    { 0x8334d661, SUC_lexicon_tags_17 },
    { 0xf154d662, SUC_lexicon_tags_12 },
    { 0x3c34d663, SUC_lexicon_tags_30 },
    { 0x154d661, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c34d66c, SUC_lexicon_tags_27 },
    { 0xed94d66d, SUC_lexicon_tags_17 },
    { 0xe1b4d66d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8614d675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb94d678, SUC_lexicon_tags_57 },
    { 0xde34d679, SUC_lexicon_tags_26 },
    { 0x7274d67a, SUC_lexicon_tags_37 },
    { 0x5294d67b, SUC_lexicon_tags_15 },
    { 0x9774d67c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2254d67f, SUC_lexicon_tags_48 },
    { 0xb514d67f, SUC_lexicon_tags_8 },
    { 0x7794d681, SUC_lexicon_tags_14 },
    { 0xba34d67f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe354d686, SUC_lexicon_tags_23 },
    { 0x6cb4d687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f4d68c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4954d68f, SUC_lexicon_tags_63 },
    { 0x7ab4d68f, SUC_lexicon_tags_25 },
    { 0x1994d691, SUC_lexicon_tags_37 },
    { 0xe774d692, SUC_lexicon_tags_7 },
    { 0x534d693, SUC_lexicon_tags_27 },
    { 0xc5b4d693, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd4d69a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd794d69c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa474d6a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd454d6af, SUC_lexicon_tags_23 },
    { 0xa954d6af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1914d6b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f4d6b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7754d6bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b4d6c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6b4d6c3, SUC_lexicon_tags_25 },
    { 0xb9d4d6c4, SUC_lexicon_tags_37 },
    { 0x43d4d6c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb194d6d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75f4d6d2, SUC_lexicon_tags_11 },
    { 0x1074d6d3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc34d6d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f74d6dc, SUC_lexicon_tags_37 },
    { 0xaaf4d6dd, SUC_lexicon_tags_17 },
    { 0x514d6de, SUC_lexicon_tags_17 },
    { 0xcd14d6de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f4d6e3, SUC_lexicon_tags_10 },
    { 0x9954d6e3, SUC_lexicon_tags_14 },
    { 0x4eb4d6e5, SUC_lexicon_tags_60 },
    { 0x4ff4d6e6, SUC_lexicon_tags_68 },
    { 0xf6b4d6e7, SUC_lexicon_tags_25 },
    { 0x6f74d6e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf4d6eb, SUC_lexicon_tags_82 },
    { 0x4f14d6eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9534d6f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6014d6f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb394d6f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b54d6f8, SUC_lexicon_tags_23 },
    { 0x6754d6f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7814d6fb, SUC_lexicon_tags_10 },
    { 0x3954d6fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf4d6ff, SUC_lexicon_tags_23 },
    { 0x4474d700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6294d702, SUC_lexicon_tags_32 },
    { 0x66b4d702, SUC_lexicon_tags_140 },
    { 0xaeb4d704, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4314d706, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2db4d708, SUC_lexicon_tags_1 },
    { 0xed4d709, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb4d70c, SUC_lexicon_tags_1 },
    { 0x3fb4d70d, SUC_lexicon_tags_11 },
    { 0x7eb4d70e, SUC_lexicon_tags_60 },
    { 0xbc94d70f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd74d713, SUC_lexicon_tags_2 },
    { 0x43b4d714, SUC_lexicon_tags_7 },
    { 0x64d4d714, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb294d71a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x314d71c, SUC_lexicon_tags_69 },
    { 0xd754d71d, SUC_lexicon_tags_12 },
    { 0x10f4d71e, SUC_lexicon_tags_27 },
    { 0xdb54d71c, SUC_lexicon_tags_15 },
    { 0x9c74d720, SUC_lexicon_tags_37 },
    { 0xd014d71e, SUC_lexicon_tags_37 },
    { 0x514d71c, SUC_lexicon_tags_19 },
    { 0xa114d721, SUC_lexicon_tags_60 },
    { 0xbe74d722, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4d726, SUC_lexicon_tags_79 },
    { 0xc454d726, SUC_lexicon_tags_53 },
    { 0xc5d4d727, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b54d72b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc274d72e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff14d732, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd54d736, SUC_lexicon_tags_77 },
    { 0x1e74d737, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4c94d739, SUC_lexicon_tags_4 },
    { 0x7b34d739, SUC_lexicon_tags_17 },
    { 0xaa54d73b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cd4d73d, SUC_lexicon_tags_13 },
    { 0x8bb4d73e, SUC_lexicon_tags_28 },
    { 0x2674d73f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b4d742, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe114d744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb94d746, SUC_lexicon_tags_7 },
    { 0x61f4d746, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd4d74a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e94d74d, SUC_lexicon_tags_15 },
    { 0x2134d74e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27d4d750, SUC_lexicon_tags_37 },
    { 0x8334d750, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25f4d753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b4d756, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d4d75c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2374d75e, SUC_lexicon_tags_25 },
    { 0x1454d75f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf374d762, SUC_lexicon_tags_11 },
    { 0x32f4d763, SUC_lexicon_tags_17 },
    { 0x6d14d764, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b74d767, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78b4d769, SUC_lexicon_tags_17 },
    { 0x70b4d769, SUC_lexicon_tags_47 },
    { 0x9c54d76b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b4d76f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd54d772, SUC_lexicon_tags_11 },
    { 0x93f4d773, SUC_lexicon_tags_27 },
    { 0x8c14d774, SUC_lexicon_tags_15 },
    { 0xfdb4d773, SUC_lexicon_tags_0 },
    { 0x24d4d776, SUC_lexicon_tags_8 },
    { 0x1814d777, SUC_lexicon_tags_25 },
    { 0x35b4d776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9974d77d, SUC_lexicon_tags_320 },
    { 0xf254d77d, SUC_lexicon_tags_2 },
    { 0x1334d77f, SUC_lexicon_tags_2 },
    { 0x87d4d780, SUC_lexicon_tags_17 },
    { 0x6b4d77f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab4d786, SUC_lexicon_tags_60 },
    { 0x7b54d786, SUC_lexicon_tags_87 },
    { 0xdbb4d788, SUC_lexicon_tags_11 },
    { 0x1774d789, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4d78d, SUC_lexicon_tags_7 },
    { 0xafd4d78e, SUC_lexicon_tags_26 },
    { 0xe1b4d78f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6774d793, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7314d797, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8ef4d799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea14d79b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b4d7a0, SUC_lexicon_tags_60 },
    { 0xf834d7a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab94d7a3, SUC_lexicon_tags_27 },
    { 0xa1d4d7a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5774d7a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f4d7ad, SUC_lexicon_tags_7 },
    { 0x7674d7ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8194d7b5, SUC_lexicon_tags_43 },
    { 0x7434d7b5, SUC_lexicon_tags_31 },
    { 0x454d7b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd4d7bb, SUC_lexicon_tags_12 },
    { 0xe334d7bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3414d7c2, SUC_lexicon_tags_49 },
    { 0x32f4d7c2, SUC_lexicon_tags_66 },
    { 0x62d4d7c4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb4d7ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad4d7cd, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd94d7d4, SUC_lexicon_tags_25 },
    { 0xe454d7d5, SUC_lexicon_tags_25 },
    { 0x9f74d7d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f34d7d8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa74d7da, SUC_lexicon_tags_87 },
    { 0x7fb4d7da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb94d7dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9674d7df, SUC_lexicon_tags_12 },
    { 0x4234d7e0, SUC_lexicon_tags_19 },
    { 0x90f4d7e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa14d7ea, SUC_lexicon_tags_1 },
    { 0x8974d7ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d34d7f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2114d7f7, SUC_lexicon_tags_11 },
    { 0x3e54d7f8, SUC_lexicon_tags_10 },
    { 0x2494d7f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf174d7fc, SUC_lexicon_tags_17 },
    { 0xc474d7fc, SUC_lexicon_tags_79 },
    { 0xee74d7fe, SUC_lexicon_tags_19 },
    { 0x1bb4d7ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d4d802, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2534d806, SUC_lexicon_tags_25 },
    { 0x9bb4d806, SUC_lexicon_tags_9 },
    { 0x8854d807, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89d4d80a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b94d80d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f4d810, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xddb4d812, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbf4d814, SUC_lexicon_tags_37 },
    { 0xbaf4d815, SUC_lexicon_tags_11 },
    { 0x6c94d816, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f4d81a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6d4d81c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b34d81f, SUC_lexicon_tags_15 },
    { 0x7234d820, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a34d827, SUC_lexicon_tags_38 },
    { 0x57b4d828, SUC_lexicon_tags_97 },
    { 0x3174d829, SUC_lexicon_tags_7 },
    { 0x8174d829, SUC_lexicon_tags_25 },
    { 0x92f4d829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef4d82e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e14d830, SUC_lexicon_tags_84 },
    { 0x46b4d830, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb014d837, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf94d83d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x414d83f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x38f4d841, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb614d844, SUC_lexicon_tags_15 },
    { 0x5d74d844, SUC_lexicon_tags_9 },
    { 0x6394d846, SUC_lexicon_tags_11 },
    { 0x3f34d847, SUC_lexicon_tags_77 },
    { 0xf8f4d847, SUC_lexicon_tags_45 },
    { 0xb754d849, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf4d850, SUC_lexicon_tags_15 },
    { 0x67d4d850, SUC_lexicon_tags_77 },
    { 0x1e54d852, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1ff4d854, SUC_lexicon_tags_26 },
    { 0xf5b4d855, SUC_lexicon_tags_47 },
    { 0x2994d856, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd934d858, SUC_lexicon_tags_32 },
    { 0x7914d859, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b4d861, SUC_lexicon_tags_19 },
    { 0x7d34d862, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a34d864, SUC_lexicon_tags_37 },
    { 0x6db4d864, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d4d86b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac94d870, SUC_lexicon_tags_19 },
    { 0x5914d871, SUC_lexicon_tags_31 },
    { 0xcf34d872, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x1e94d874, SUC_lexicon_tags_7 },
    { 0xd2b4d875, SUC_lexicon_tags_53 },
    { 0x67d4d876, SUC_lexicon_tags_37 },
    { 0xc554d877, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd4d87a, SUC_lexicon_tags_15 },
    { 0x8874d87b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb4d87f, SUC_lexicon_tags_37 },
    { 0xc394d880, SUC_lexicon_tags_43 },
    { 0x7c94d881, SUC_lexicon_tags_25 },
    { 0xe394d882, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7234d884, SUC_lexicon_tags_25 },
    { 0xc3d4d885, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d4d889, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf4d891, SUC_lexicon_tags_43 },
    { 0x3f54d892, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f4d896, SUC_lexicon_tags_17 },
    { 0x2854d897, SUC_lexicon_tags_39 },
    { 0x2c74d897, SUC_lexicon_tags_7 },
    { 0x7014d898, SUC_lexicon_tags_60 },
    { 0xf154d899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa574d89e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc934d8a0, SUC_lexicon_tags_15 },
    { 0x5714d8a0, SUC_lexicon_tags_25 },
    { 0x4a94d8a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4894d8a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5214d8a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf994d8a8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f4d8ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8874d8ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf4d8b0, SUC_lexicon_tags_2 },
    { 0x14f4d8b1, SUC_lexicon_tags_23 },
    { 0xdb34d8b2, SUC_lexicon_tags_38 },
    { 0x2b54d8b3, SUC_lexicon_tags_12 },
    { 0xecf4d8b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9034d8b9, SUC_lexicon_tags_44 },
    { 0x7774d8ba, SUC_lexicon_tags_127 },
    { 0xfad4d8ba, SUC_lexicon_tags_26 },
    { 0x42f4d8ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f4d8c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c54d8c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6254d8cb, SUC_lexicon_tags_11 },
    { 0x5c74d8cc, SUC_lexicon_tags_7 },
    { 0xb5f4d8cc, SUC_lexicon_tags_29 },
    { 0xca14d8ce, SUC_lexicon_tags_4 },
    { 0x6db4d8ce, SUC_lexicon_tags_25 },
    { 0x8b74d8ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9554d8d2, SUC_lexicon_tags_28 },
    { 0xe34d8d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7534d8d6, SUC_lexicon_tags_6 },
    { 0x80b4d8d6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xff94d8da, SUC_lexicon_tags_6 },
    { 0x8b4d8db, SUC_lexicon_tags_30 },
    { 0x7734d8da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x914d8df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x86d4d8e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f4d8e4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc94d8ec, SUC_lexicon_tags_17 },
    { 0xf534d8ec, SUC_lexicon_tags_31 },
    { 0xfd54d8ee, SUC_lexicon_tags_48 },
    { 0x8cb4d8ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc034d8f3, SUC_lexicon_tags_11 },
    { 0x9174d8f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60d4d8f6, SUC_lexicon_tags_29 },
    { 0x74b4d8f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb614d8f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5934d901, SUC_lexicon_tags_15 },
    { 0x7834d902, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac94d908, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf534d90c, SUC_lexicon_tags_2 },
    { 0x5834d90c, SUC_lexicon_tags_14 },
    { 0x64b4d90d, SUC_lexicon_tags_15 },
    { 0xc754d90c, SUC_lexicon_tags_11 },
    { 0xb614d90d, SUC_lexicon_tags_7 },
    { 0x9654d911, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb54d914, SUC_lexicon_tags_17 },
    { 0x3ed4d915, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf794d917, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7694d91b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b4d91f, SUC_lexicon_tags_1 },
    { 0xf0b4d91f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9834d923, SUC_lexicon_tags_8 },
    { 0x1bf4d923, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c54d927, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f4d92a, SUC_lexicon_tags_7 },
    { 0x21d4d92b, SUC_lexicon_tags_51 },
    { 0x9df4d92b, SUC_lexicon_tags_11 },
    { 0x9214d92b, SUC_lexicon_tags_43 },
    { 0xdc34d92b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa874d932, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb4d935, SUC_lexicon_tags_37 },
    { 0x6ed4d936, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4974d938, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ab4d93a, SUC_lexicon_tags_3 },
    { 0xfe54d93b, SUC_lexicon_tags_26 },
    { 0xc8d4d93b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9714d940, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb4d944, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f4d94a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc594d94d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd94d954, SUC_lexicon_tags_38 },
    { 0xffb4d954, SUC_lexicon_tags_49 },
    { 0x54d956, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf274d958, SUC_lexicon_tags_11 },
    { 0xfdd4d959, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6b4d95b, SUC_lexicon_tags_9 },
    { 0x55b4d95c, SUC_lexicon_tags_15 },
    { 0x1b94d95c, SUC_lexicon_tags_7 },
    { 0xaf4d95d, SUC_lexicon_tags_15 },
    { 0x1fd4d95f, SUC_lexicon_tags_25 },
    { 0x5914d95f, SUC_lexicon_tags_112 },
    { 0xa6b4d95c, SUC_lexicon_tags_2 },
    { 0x6dd4d962, SUC_lexicon_tags_47 },
    { 0x5a74d963, SUC_lexicon_tags_2 },
    { 0x7654d963, SUC_lexicon_tags_47 },
    { 0xa354d965, SUC_lexicon_tags_9 },
    { 0x71b4d966, SUC_lexicon_tags_12 },
    { 0xfb14d962, SUC_lexicon_tags_25 },
    { 0x7554d968, SUC_lexicon_tags_2 },
    { 0x98b4d969, SUC_lexicon_tags_27 },
    { 0xaf74d96a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfab4d96c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe014d96f, SUC_lexicon_tags_11 },
    { 0x3cb4d970, SUC_lexicon_tags_38 },
    { 0x4234d971, SUC_lexicon_tags_60 },
    { 0x1594d970, SUC_lexicon_tags_17 },
    { 0xe814d970, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ad4d975, SUC_lexicon_tags_14 },
    { 0xfbd4d976, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b74d979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6314d97c, SUC_lexicon_tags_11 },
    { 0x76f4d97c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1b4d97f, SUC_lexicon_tags_63 },
    { 0xe854d97f, SUC_lexicon_tags_37 },
    { 0xe134d97f, SUC_lexicon_tags_29 },
    { 0x1bb4d97f, SUC_lexicon_tags_32 },
    { 0x9f74d983, SUC_lexicon_tags_15 },
    { 0x64f4d983, SUC_lexicon_tags_41 },
    { 0x1874d983, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ad4d987, SUC_lexicon_tags_23 },
    { 0x3314d987, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b4d990, SUC_lexicon_tags_66 },
    { 0x64f4d991, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5014d993, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4d997, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a34d99a, SUC_lexicon_tags_49 },
    { 0xffb4d99b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35d4d99d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4054d9ac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d4d9b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa714d9b4, SUC_lexicon_tags_15 },
    { 0x8bf4d9b5, SUC_lexicon_tags_0 },
    { 0xc414d9b6, SUC_lexicon_tags_31 },
    { 0x4f54d9b7, SUC_lexicon_tags_6 },
    { 0x6494d9b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b4d9bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2734d9bf, SUC_lexicon_tags_152 },
    { 0x31b4d9bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe054d9c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7f4d9c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd14d9c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdef4d9c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff4d9cd, SUC_lexicon_tags_2 },
    { 0x94f4d9ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f4d9d1, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe154d9d7, SUC_lexicon_tags_1 },
    { 0xe6b4d9d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xea34d9da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd54d9df, SUC_lexicon_tags_19 },
    { 0xe894d9df, SUC_lexicon_tags_17 },
    { 0x78b4d9e1, SUC_lexicon_tags_12 },
    { 0x4df4d9e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b54d9e8, SUC_lexicon_tags_35 },
    { 0x9594d9e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3cf4d9eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f4d9f0, SUC_lexicon_tags_23 },
    { 0x834d9f1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c94d9f4, SUC_lexicon_tags_45 },
    { 0x9b4d9f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f4d9f9, SUC_lexicon_tags_38 },
    { 0x274d9fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d4d9fd, SUC_lexicon_tags_44 },
    { 0xbed4d9fe, SUC_lexicon_tags_6 },
    { 0xb1f4d9fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ff4da01, SUC_lexicon_tags_25 },
    { 0x48b4da01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd4da04, SUC_lexicon_tags_1 },
    { 0xde94da05, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xae34da07, SUC_lexicon_tags_57 },
    { 0x88b4da07, SUC_lexicon_tags_63 },
    { 0xb74da09, SUC_lexicon_tags_77 },
    { 0xe314da09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d4da0e, SUC_lexicon_tags_30 },
    { 0x4d54da0f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xba54da11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe534da14, SUC_lexicon_tags_7 },
    { 0xd234da15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf274da18, SUC_lexicon_tags_45 },
    { 0xce54da19, SUC_lexicon_tags_60 },
    { 0xf214da19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3714da21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a34da23, SUC_lexicon_tags_18 },
    { 0xd554da23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e74da27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e54da2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa874da2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a34da30, SUC_lexicon_tags_17 },
    { 0x6754da30, SUC_lexicon_tags_45 },
    { 0xbad4da32, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3014da35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x97b4da37, SUC_lexicon_tags_28 },
    { 0x9214da37, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf294da3b, SUC_lexicon_tags_15 },
    { 0x60d4da3c, SUC_lexicon_tags_63 },
    { 0x3f34da3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7654da3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b4da41, SUC_lexicon_tags_37 },
    { 0xd534da42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb954da46, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x754da48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8194da4c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1314da4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb4da53, SUC_lexicon_tags_25 },
    { 0x80d4da54, SUC_lexicon_tags_45 },
    { 0x9234da54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf4da57, SUC_lexicon_tags_17 },
    { 0xc8f4da57, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef4da5b, SUC_lexicon_tags_25 },
    { 0x9394da5c, SUC_lexicon_tags_68 },
    { 0xaa14da5d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf7f4da5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d4da63, SUC_lexicon_tags_37 },
    { 0xdcf4da63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa954da69, SUC_lexicon_tags_39 },
    { 0xe2d4da6a, SUC_lexicon_tags_7 },
    { 0x4234da6b, SUC_lexicon_tags_60 },
    { 0x7a94da6b, SUC_lexicon_tags_23 },
    { 0x254da6d, SUC_lexicon_tags_39 },
    { 0xdef4da6e, SUC_lexicon_tags_14 },
    { 0x3cf4da6e, SUC_lexicon_tags_60 },
    { 0xcf74da70, SUC_lexicon_tags_87 },
    { 0xcd14da6c, SUC_lexicon_tags_12 },
    { 0xb1b4da6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c14da75, SUC_lexicon_tags_3 },
    { 0xd4d4da76, SUC_lexicon_tags_11 },
    { 0x3dd4da77, SUC_lexicon_tags_31 },
    { 0x3474da78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2694da7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3434da7c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x93d4da7e, SUC_lexicon_tags_68 },
    { 0x434da7f, SUC_lexicon_tags_1 },
    { 0x1ad4da7f, SUC_lexicon_tags_30 },
    { 0x52f4da7f, SUC_lexicon_tags_25 },
    { 0x2a54da80, SUC_lexicon_tags_1 },
    { 0xfd34da83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f4da8a, SUC_lexicon_tags_37 },
    { 0x9094da8b, SUC_lexicon_tags_47 },
    { 0x2674da8c, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf14da8f, SUC_lexicon_tags_46 },
    { 0x5a54da90, SUC_lexicon_tags_23 },
    { 0x8974da90, SUC_lexicon_tags_48 },
    { 0x5bd4da92, SUC_lexicon_tags_31 },
    { 0x7d34da90, SUC_lexicon_tags_7 },
    { 0xde14da8f, SUC_lexicon_tags_47 },
    { 0x86b4da95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf254da97, SUC_lexicon_tags_56 },
    { 0xe654da98, SUC_lexicon_tags_17 },
    { 0x88d4da97, SUC_lexicon_tags_33 },
    { 0x71d4da98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3114da9e, SUC_lexicon_tags_19 },
    { 0x9e54da9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe574daa2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7934daa4, SUC_lexicon_tags_7 },
    { 0x5b94daa4, SUC_lexicon_tags_15 },
    { 0x7bf4daa6, SUC_lexicon_tags_206 },
    { 0x3674daa6, SUC_lexicon_tags_25 },
    { 0xc8f4daa7, SUC_lexicon_tags_4 },
    { 0x614daa6, SUC_lexicon_tags_37 },
    { 0xabb4daaa, SUC_lexicon_tags_11 },
    { 0x214daab, SUC_lexicon_tags_12 },
    { 0xb334daa7, SUC_lexicon_tags_7 },
    { 0x67f4daad, SUC_lexicon_tags_38 },
    { 0x9cf4daae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9754dab2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f4dab6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf834dab8, SUC_lexicon_tags_11 },
    { 0xdc94dab9, SUC_lexicon_tags_2 },
    { 0x4454dab9, SUC_lexicon_tags_1 },
    { 0x5194dabb, SUC_lexicon_tags_21 },
    { 0x6fd4dabc, SUC_lexicon_tags_1 },
    { 0xd334dabc, SUC_lexicon_tags_33 },
    { 0x56d4dabe, SUC_lexicon_tags_34 },
    { 0x4c14dabe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7bd4dac1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x53d4dac3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2994dacd, SUC_lexicon_tags_45 },
    { 0x74dacd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc094dad0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x1d34dad2, SUC_lexicon_tags_38 },
    { 0x3734dad3, SUC_lexicon_tags_30 },
    { 0xa2b4dad3, SUC_lexicon_tags_25 },
    { 0xd0f4dad5, SUC_lexicon_tags_9 },
    { 0x5674dad6, SUC_lexicon_tags_30 },
    { 0x1934dad2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2854dad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x434dae4, SUC_lexicon_tags_23 },
    { 0xe2b4dae4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xed74dae8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5434daed, SUC_lexicon_tags_66 },
    { 0xfb4daee, SUC_lexicon_tags_5 },
    { 0x64f4daef, SUC_lexicon_tags_35 },
    { 0xa914daed, SUC_lexicon_tags_992 },
    { 0xa9d4daee, SUC_lexicon_tags_37 },
    { 0x6d14daf2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4754daf4, SUC_lexicon_tags_37 },
    { 0x3394daf4, SUC_lexicon_tags_28 },
    { 0x6394daf5, SUC_lexicon_tags_17 },
    { 0x6f14daf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e14dafb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x92d4dafd, SUC_lexicon_tags_56 },
    { 0x7df4dafd, SUC_lexicon_tags_17 },
    { 0x4554daff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b4db03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca74db06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26d4db08, SUC_lexicon_tags_67 },
    { 0xeef4db09, SUC_lexicon_tags_15 },
    { 0xf6f4db09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9834db0c, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x2674db0f, SUC_lexicon_tags_38 },
    { 0xb74db10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b54db12, SUC_lexicon_tags_47 },
    { 0x9d54db13, SUC_lexicon_tags_14 },
    { 0xc0d4db12, SUC_lexicon_tags_65 },
    { 0x7e14db13, SUC_lexicon_tags_24 },
    { 0x29f4db13, SUC_lexicon_tags_66 },
    { 0x914db17, SUC_lexicon_tags_9 },
    { 0x82b4db17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cd4db1a, SUC_lexicon_tags_33 },
    { 0xb4db1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f4db2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed94db2e, SUC_lexicon_tags_27 },
    { 0x5dd4db2f, SUC_lexicon_tags_61 },
    { 0xdcd4db2e, SUC_lexicon_tags_15 },
    { 0x21f4db31, SUC_lexicon_tags_207 },
    { 0xbb94db32, SUC_lexicon_tags_63 },
    { 0x3fb4db33, SUC_lexicon_tags_12 },
    { 0xcb54db34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff34db36, SUC_lexicon_tags_9 },
    { 0x3174db37, SUC_lexicon_tags_11 },
    { 0xda34db37, SUC_lexicon_tags_15 },
    { 0x67d4db39, SUC_lexicon_tags_6 },
    { 0x1f74db3a, SUC_lexicon_tags_18 },
    { 0xf114db3b, SUC_lexicon_tags_1 },
    { 0xa5f4db3c, SUC_lexicon_tags_16 },
    { 0xfdd4db3b, SUC_lexicon_tags_12 },
    { 0x1d74db3a, SUC_lexicon_tags_63 },
    { 0x1214db3f, SUC_lexicon_tags_37 },
    { 0x1d74db3a, SUC_lexicon_tags_15 },
    { 0xc654db40, SUC_lexicon_tags_30 },
    { 0xe4f4db39, SUC_lexicon_tags_38 },
    { 0xcf74db3c, SUC_lexicon_tags_47 },
    { 0x9bb4db44, SUC_lexicon_tags_19 },
    { 0x4d74db45, SUC_lexicon_tags_17 },
    { 0x1914db45, SUC_lexicon_tags_12 },
    { 0x3734db43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7614db4b, SUC_lexicon_tags_37 },
    { 0x93f4db4b, SUC_lexicon_tags_7 },
    { 0x6754db4c, SUC_lexicon_tags_12 },
    { 0x7c34db4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5674db50, SUC_lexicon_tags_4 },
    { 0x14d4db51, SUC_lexicon_tags_25 },
    { 0x7874db52, SUC_lexicon_tags_38 },
    { 0xe1d4db51, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x12b4db55, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe334db57, SUC_lexicon_tags_2 },
    { 0xe694db58, SUC_lexicon_tags_33 },
    { 0x7c94db59, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5014db5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c34db61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7994db63, SUC_lexicon_tags_17 },
    { 0x8934db63, SUC_lexicon_tags_37 },
    { 0xd8d4db65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6d34db67, SUC_lexicon_tags_1 },
    { 0x8d94db68, SUC_lexicon_tags_7 },
    { 0x3554db68, SUC_lexicon_tags_57 },
    { 0xad34db6a, SUC_lexicon_tags_30 },
    { 0x8594db6a, SUC_lexicon_tags_12 },
    { 0x57d4db6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6074db6f, SUC_lexicon_tags_23 },
    { 0xf7d4db70, SUC_lexicon_tags_16 },
    { 0x9574db71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f94db75, SUC_lexicon_tags_82 },
    { 0x6414db76, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd4d4db78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ab4db7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf114db81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb354db87, SUC_lexicon_tags_39 },
    { 0x5a34db88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe214db8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e34db8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad4db92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3774db96, SUC_lexicon_tags_7 },
    { 0x73d4db97, SUC_lexicon_tags_1 },
    { 0x9ad4db96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b34db9a, SUC_lexicon_tags_15 },
    { 0x4334db9a, SUC_lexicon_tags_63 },
    { 0x2154db9c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e74dba2, SUC_lexicon_tags_37 },
    { 0x5d34dba3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf334dba5, SUC_lexicon_tags_27 },
    { 0x4f4dba5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d4dba9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8734dbac, SUC_lexicon_tags_33 },
    { 0x16b4dbad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b4dbb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b4dbb3, SUC_lexicon_tags_27 },
    { 0x3614dbb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa834dbba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b4dbbf, SUC_lexicon_tags_39 },
    { 0xb974dbc0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab4dbc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6054dbc8, SUC_lexicon_tags_14 },
    { 0xb014dbc9, SUC_lexicon_tags_8 },
    { 0xc894dbc9, SUC_lexicon_tags_11 },
    { 0x4ff4dbcb, SUC_lexicon_tags_17 },
    { 0x8754dbc9, SUC_lexicon_tags_37 },
    { 0xb1f4dbcd, SUC_lexicon_tags_2 },
    { 0xe494dbc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d54dbd6, SUC_lexicon_tags_43 },
    { 0x14f4dbd7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x82b4dbd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4dbdd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc874dbe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d4dbe4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2974dbef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f4dbf2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xed4dbf4, SUC_lexicon_tags_14 },
    { 0x7e14dbf5, SUC_lexicon_tags_1 },
    { 0x49d4dbf6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b74dbf8, SUC_lexicon_tags_29 },
    { 0x5474dbf9, SUC_lexicon_tags_49 },
    { 0xc4b4dbf9, SUC_lexicon_tags_2 },
    { 0xd3f4dbfb, SUC_lexicon_tags_3 },
    { 0xc234dbfc, SUC_lexicon_tags_46 },
    { 0xef54dbfc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db4dc00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a54dc03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4dc05, SUC_lexicon_tags_45 },
    { 0x2454dc06, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2e54dc08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d4dc0a, SUC_lexicon_tags_15 },
    { 0x6014dc0b, SUC_lexicon_tags_1 },
    { 0x8154dc0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f94dc0e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f4dc11, SUC_lexicon_tags_44 },
    { 0xe4d4dc12, SUC_lexicon_tags_25 },
    { 0xcef4dc13, SUC_lexicon_tags_17 },
    { 0x7674dc12, SUC_lexicon_tags_38 },
    { 0x9154dc15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f74dc1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6214dc1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6894dc21, SUC_lexicon_tags_7 },
    { 0x6dd4dc22, SUC_lexicon_tags_17 },
    { 0x63f4dc23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac34dc28, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x61f4dc2a, SUC_lexicon_tags_12 },
    { 0x9874dc2b, SUC_lexicon_tags_7 },
    { 0x4fb4dc2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f4dc31, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0x6334dc33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b14dc39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda54dc3b, SUC_lexicon_tags_15 },
    { 0xa1b4dc3b, SUC_lexicon_tags_33 },
    { 0x2e14dc3d, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b4dc41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c54dc45, SUC_lexicon_tags_11 },
    { 0x7374dc45, SUC_lexicon_tags_18 },
    { 0x94b4dc47, SUC_lexicon_tags_1 },
    { 0xf114dc47, SUC_lexicon_tags_45 },
    { 0x6334dc49, SUC_lexicon_tags_7 },
    { 0xedd4dc47, SUC_lexicon_tags_29 },
    { 0x10d4dc4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1594dc4f, SUC_lexicon_tags_4 },
    { 0x7f4dc4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe774dc53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d4dc59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd0f4dc5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1214dc5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1594dc60, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd14dc65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8654dc67, SUC_lexicon_tags_79 },
    { 0x9e14dc68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbf4dc6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8734dc6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd4dc74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xced4dc77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7954dc79, SUC_lexicon_tags_47 },
    { 0xd794dc7a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1c54dc7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bb4dc7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d14dc85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4534dc87, SUC_lexicon_tags_14 },
    { 0xff4dc88, SUC_lexicon_tags_25 },
    { 0xf6d4dc88, SUC_lexicon_tags_7 },
    { 0x8494dc88, SUC_lexicon_tags_42 },
    { 0xd4b4dc88, SUC_lexicon_tags_15 },
    { 0x90b4dc89, SUC_lexicon_tags_17 },
    { 0xdc14dc89, SUC_lexicon_tags_21 },
    { 0x24f4dc8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbb74dc90, SUC_lexicon_tags_11 },
    { 0xb594dc91, SUC_lexicon_tags_16 },
    { 0x1934dc92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f4dc97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f4dc9b, SUC_lexicon_tags_4 },
    { 0xfc74dc9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf134dc9e, SUC_lexicon_tags_40 },
    { 0xfbf4dc9f, SUC_lexicon_tags_7 },
    { 0x7734dca0, SUC_lexicon_tags_37 },
    { 0x4af4dca0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba14dca6, SUC_lexicon_tags_31 },
    { 0xf834dca6, SUC_lexicon_tags_38 },
    { 0x8f94dca8, SUC_lexicon_tags_17 },
    { 0x6934dca9, SUC_lexicon_tags_15 },
    { 0x24d4dcaa, SUC_lexicon_tags_87 },
    { 0x5e74dcab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5d4dcad, SUC_lexicon_tags_31 },
    { 0x35d4dcae, SUC_lexicon_tags_7 },
    { 0xec54dcaf, SUC_lexicon_tags_6 },
    { 0xa9b4dcaf, SUC_lexicon_tags_2 },
    { 0xc334dcaf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6514dcb7, SUC_lexicon_tags_27 },
    { 0xe54dcb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a54dcbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf034dcc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb754dcc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d54dcc6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c34dccf, SUC_lexicon_tags_60 },
    { 0xcad4dcd0, SUC_lexicon_tags_2 },
    { 0xc614dcd0, SUC_lexicon_tags_27 },
    { 0xe7f4dcd2, SUC_lexicon_tags_39 },
    { 0xb894dcd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2634dcd5, SUC_lexicon_tags_11 },
    { 0x2a34dcd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd4dcd9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4dcdd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d4dce1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90f4dce3, SUC_lexicon_tags_43 },
    { 0xebb4dce3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2cb4dce6, SUC_lexicon_tags_9 },
    { 0xaaf4dce7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8ed4dce9, SUC_lexicon_tags_25 },
    { 0xc514dcea, SUC_lexicon_tags_46 },
    { 0xa754dcea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd34dcee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b4dcf1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9554dcf3, SUC_lexicon_tags_37 },
    { 0xf354dcf3, SUC_lexicon_tags_43 },
    { 0x754dcf3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4734dcfd, SUC_lexicon_tags_14 },
    { 0x25d4dcfe, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4dd01, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9734dd04, SUC_lexicon_tags_30 },
    { 0x4dd4dd05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7434dd07, SUC_lexicon_tags_19 },
    { 0xe294dd08, SUC_lexicon_tags_12 },
    { 0x9e74dd09, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f4dd0d, SUC_lexicon_tags_11 },
    { 0x27d4dd0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af4dd12, SUC_lexicon_tags_37 },
    { 0xaa14dd13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4614dd15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae14dd17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf114dd1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab14dd1e, SUC_lexicon_tags_1 },
    { 0x2a14dd1f, SUC_lexicon_tags_38 },
    { 0x9b34dd20, SUC_lexicon_tags_48 },
    { 0xa4d4dd21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2294dd28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x674dd2a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c54dd30, SUC_lexicon_tags_14 },
    { 0x6914dd31, SUC_lexicon_tags_17 },
    { 0xbdd4dd32, SUC_lexicon_tags_27 },
    { 0xe9d4dd30, SUC_lexicon_tags_7 },
    { 0x4dd4dd34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8914dd37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc134dd3a, SUC_lexicon_tags_19 },
    { 0x6834dd3b, SUC_lexicon_tags_16 },
    { 0xd534dd3b, SUC_lexicon_tags_37 },
    { 0x3794dd3b, SUC_lexicon_tags_9 },
    { 0xfd14dd3d, SUC_lexicon_tags_21 },
    { 0x38f4dd3f, SUC_lexicon_tags_60 },
    { 0x1254dd3d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x19b4dd42, SUC_lexicon_tags_0 },
    { 0xcc34dd42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5514dd46, SUC_lexicon_tags_11 },
    { 0x4654dd47, SUC_lexicon_tags_15 },
    { 0x8714dd47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4354dd4b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5634dd52, SUC_lexicon_tags_11 },
    { 0x60d4dd53, SUC_lexicon_tags_1 },
    { 0x6b34dd53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7054dd56, SUC_lexicon_tags_10 },
    { 0x9234dd56, SUC_lexicon_tags_2 },
    { 0xccd4dd58, SUC_lexicon_tags_17 },
    { 0x1674dd56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b54dd5d, SUC_lexicon_tags_38 },
    { 0x4c34dd5d, SUC_lexicon_tags_35 },
    { 0x2d54dd5f, SUC_lexicon_tags_4 },
    { 0xcd14dd5f, SUC_lexicon_tags_7 },
    { 0x70d4dd61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4dd6b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x3374dd6d, SUC_lexicon_tags_2 },
    { 0xe54dd6e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c34dd73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa994dd75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf54dd7b, SUC_lexicon_tags_37 },
    { 0xf434dd7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0f4dd7e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f4dd83, SUC_lexicon_tags_1 },
    { 0x6f54dd84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc6d4dd86, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4dd89, SUC_lexicon_tags_39 },
    { 0x2fd4dd8a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b4dd91, SUC_lexicon_tags_14 },
    { 0x8e54dd92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5774dd97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b4dd9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa314dd9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6594dda1, SUC_lexicon_tags_15 },
    { 0x9074dda1, SUC_lexicon_tags_143 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b4dda5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf114dda7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d4ddad, SUC_lexicon_tags_49 },
    { 0x7fb4ddae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd4ddb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf54ddb8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd4ddbb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4534ddbd, SUC_lexicon_tags_15 },
    { 0x8a14ddbd, SUC_lexicon_tags_13 },
    { 0x11b4ddbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b54ddc2, SUC_lexicon_tags_12 },
    { 0xf514ddc3, SUC_lexicon_tags_49 },
    { 0xc494ddc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5494ddc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7014ddcd, SUC_lexicon_tags_7 },
    { 0x3c94ddcd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb434ddd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b4ddd6, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x7bd4ddd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd494dde2, SUC_lexicon_tags_12 },
    { 0xf3b4dde3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe414dde5, SUC_lexicon_tags_35 },
    { 0x6334dde5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad4ddee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3734ddf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef94ddf9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4754ddfd, SUC_lexicon_tags_87 },
    { 0x8d54ddfd, SUC_lexicon_tags_1 },
    { 0x23d4ddff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf374de01, SUC_lexicon_tags_30 },
    { 0xd354de01, SUC_lexicon_tags_47 },
    { 0xc114de01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114de0d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x914de10, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4554de12, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd1b4de14, SUC_lexicon_tags_30 },
    { 0x1494de14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3174de17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a94de1e, SUC_lexicon_tags_40 },
    { 0x2674de1e, SUC_lexicon_tags_47 },
    { 0xc9f4de20, SUC_lexicon_tags_63 },
    { 0xf134de20, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb54de26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2974de2b, SUC_lexicon_tags_15 },
    { 0x3d14de2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf134de2f, SUC_lexicon_tags_3 },
    { 0x1514de30, SUC_lexicon_tags_56 },
    { 0x2374de30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd4de38, SUC_lexicon_tags_9 },
    { 0xfa94de38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d34de3b, SUC_lexicon_tags_16 },
    { 0x23d4de3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9814de3f, SUC_lexicon_tags_12 },
    { 0x1554de40, SUC_lexicon_tags_68 },
    { 0xa834de41, SUC_lexicon_tags_57 },
    { 0xf34de3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3994de44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b74de46, SUC_lexicon_tags_60 },
    { 0x57f4de46, SUC_lexicon_tags_47 },
    { 0xf8f4de46, SUC_lexicon_tags_12 },
    { 0xff4de47, SUC_lexicon_tags_7 },
    { 0x37b4de4a, SUC_lexicon_tags_25 },
    { 0xf754de49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c34de4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9394de50, SUC_lexicon_tags_7 },
    { 0xd194de51, SUC_lexicon_tags_4 },
    { 0x40f4de51, SUC_lexicon_tags_30 },
    { 0x52b4de51, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab4de56, SUC_lexicon_tags_15 },
    { 0xe94de57, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c94de5a, SUC_lexicon_tags_21 },
    { 0xc754de5b, SUC_lexicon_tags_37 },
    { 0xb6b4de5c, SUC_lexicon_tags_28 },
    { 0x1b74de5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4454de5f, SUC_lexicon_tags_37 },
    { 0x1e94de60, SUC_lexicon_tags_37 },
    { 0x60d4de61, SUC_lexicon_tags_10 },
    { 0xde34de61, SUC_lexicon_tags_46 },
    { 0x4e14de63, SUC_lexicon_tags_1 },
    { 0xb914de62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3814de66, SUC_lexicon_tags_25 },
    { 0xb1f4de67, SUC_lexicon_tags_37 },
    { 0x2e94de66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f4de6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d34de70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe614de73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd34de78, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3354de7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8434de7f, SUC_lexicon_tags_13 },
    { 0x5294de80, SUC_lexicon_tags_43 },
    { 0x1e94de80, SUC_lexicon_tags_10 },
    { 0x8f4de82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7694de86, SUC_lexicon_tags_11 },
    { 0x17f4de86, SUC_lexicon_tags_17 },
    { 0x2534de88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e54de8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf894de8d, SUC_lexicon_tags_6 },
    { 0x17b4de8d, SUC_lexicon_tags_17 },
    { 0xb34de8f, SUC_lexicon_tags_12 },
    { 0xb0b4de8d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbe14de92, SUC_lexicon_tags_25 },
    { 0xd854de93, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb74de98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9454de9a, SUC_lexicon_tags_13 },
    { 0x2df4de9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2534de9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9a14dea0, SUC_lexicon_tags_15 },
    { 0x77b4dea0, SUC_lexicon_tags_45 },
    { 0x9f4dea2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8954dea4, SUC_lexicon_tags_84 },
    { 0x1034dea5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf4dea8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f4deaf, SUC_lexicon_tags_9 },
    { 0x4a14deaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5954deb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9974deb5, SUC_lexicon_tags_5 },
    { 0xe334deb6, SUC_lexicon_tags_6 },
    { 0x1874deb7, SUC_lexicon_tags_11 },
    { 0xbb14deb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b34debb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1014debe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x434dec0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f4dec7, SUC_lexicon_tags_33 },
    { 0x82b4dec8, SUC_lexicon_tags_12 },
    { 0x4594dec8, SUC_lexicon_tags_16 },
    { 0xb714dec9, SUC_lexicon_tags_2 },
    { 0x13b4decb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4654dece, SUC_lexicon_tags_14 },
    { 0xb3f4decf, SUC_lexicon_tags_21 },
    { 0xdc14ded0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb534ded2, SUC_lexicon_tags_14 },
    { 0x9f4ded3, SUC_lexicon_tags_34 },
    { 0x7874ded4, SUC_lexicon_tags_1 },
    { 0x7eb4ded5, SUC_lexicon_tags_19 },
    { 0x2d94ded6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe574deda, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b4dedd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61d4dedf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff34dee4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d4dee8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe494deea, SUC_lexicon_tags_1 },
    { 0xb14deeb, SUC_lexicon_tags_62 },
    { 0xf9b4deec, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x97b4deee, SUC_lexicon_tags_65 },
    { 0x90b4deef, SUC_lexicon_tags_61 },
    { 0x8fd4deef, SUC_lexicon_tags_25 },
    { 0x51b4def1, SUC_lexicon_tags_11 },
    { 0xf114deef, SUC_lexicon_tags_38 },
    { 0x5954def3, SUC_lexicon_tags_56 },
    { 0x19b4deef, SUC_lexicon_tags_2 },
    { 0xe8b4def2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf274defa, SUC_lexicon_tags_77 },
    { 0xd294defb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa294defe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a14df01, SUC_lexicon_tags_10 },
    { 0x19d4df02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb4df05, SUC_lexicon_tags_6 },
    { 0xe254df06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af4df09, SUC_lexicon_tags_38 },
    { 0xd114df0a, SUC_lexicon_tags_37 },
    { 0xcd34df0a, SUC_lexicon_tags_25 },
    { 0x27d4df0c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4df10, SUC_lexicon_tags_60 },
    { 0xf614df11, SUC_lexicon_tags_2 },
    { 0xe7f4df12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b4df17, SUC_lexicon_tags_7 },
    { 0xb1b4df18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcd54df1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4834df1e, SUC_lexicon_tags_106 },
    { 0x5814df1f, SUC_lexicon_tags_25 },
    { 0x4414df20, SUC_lexicon_tags_55 },
    { 0x2154df21, SUC_lexicon_tags_17 },
    { 0x2db4df20, SUC_lexicon_tags_17 },
    { 0xf434df22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5d4df25, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa914df29, SUC_lexicon_tags_65 },
    { 0x2154df2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd4df2f, SUC_lexicon_tags_19 },
    { 0x3af4df30, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8994df35, SUC_lexicon_tags_65 },
    { 0xf474df36, SUC_lexicon_tags_10 },
    { 0x7834df37, SUC_lexicon_tags_14 },
    { 0x1094df38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x494df3f, SUC_lexicon_tags_15 },
    { 0xc4d4df40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd4df42, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xd514df45, SUC_lexicon_tags_30 },
    { 0xaaf4df46, SUC_lexicon_tags_2 },
    { 0xda74df47, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8454df49, SUC_lexicon_tags_11 },
    { 0x1014df4a, SUC_lexicon_tags_11 },
    { 0x5494df49, SUC_lexicon_tags_49 },
    { 0x8cb4df4c, SUC_lexicon_tags_17 },
    { 0x6e54df4d, SUC_lexicon_tags_47 },
    { 0x3614df4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5814df50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fb4df52, SUC_lexicon_tags_2 },
    { 0x534df53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb174df55, SUC_lexicon_tags_7 },
    { 0x80b4df56, SUC_lexicon_tags_25 },
    { 0x5414df55, SUC_lexicon_tags_92 },
    { 0xddf4df56, SUC_lexicon_tags_38 },
    { 0x31d4df59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa674df5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3674df5e, SUC_lexicon_tags_10 },
    { 0xfb14df5f, SUC_lexicon_tags_28 },
    { 0x67b4df60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8594df62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff4df65, SUC_lexicon_tags_8 },
    { 0x5234df66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b14df68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b34df6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe334df6f, SUC_lexicon_tags_31 },
    { 0xeb14df70, SUC_lexicon_tags_9 },
    { 0x7b94df70, SUC_lexicon_tags_7 },
    { 0x54df72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d14df74, SUC_lexicon_tags_43 },
    { 0x2534df75, SUC_lexicon_tags_28 },
    { 0x61f4df76, SUC_lexicon_tags_23 },
    { 0xc154df76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd634df79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0f4df7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3494df7e, SUC_lexicon_tags_12 },
    { 0x78d4df7f, SUC_lexicon_tags_7 },
    { 0x8794df7e, SUC_lexicon_tags_61 },
    { 0x7b4df81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9394df83, SUC_lexicon_tags_17 },
    { 0xdd54df84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8614df86, SUC_lexicon_tags_33 },
    { 0xb774df87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf4df8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7434df8d, SUC_lexicon_tags_1 },
    { 0xb454df8e, SUC_lexicon_tags_6 },
    { 0x3274df8d, SUC_lexicon_tags_3 },
    { 0x3e14df90, SUC_lexicon_tags_23 },
    { 0x8ab4df8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7134df94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa074df99, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4734df9c, SUC_lexicon_tags_3 },
    { 0xab94df9c, SUC_lexicon_tags_15 },
    { 0x4c74df9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf14dfa6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd994dfab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f4dfaf, SUC_lexicon_tags_25 },
    { 0xce14dfb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa654dfb3, SUC_lexicon_tags_15 },
    { 0x5154dfb3, SUC_lexicon_tags_17 },
    { 0x7bb4dfb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba74dfba, SUC_lexicon_tags_25 },
    { 0xc174dfbb, SUC_lexicon_tags_15 },
    { 0x6b34dfbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d14dfbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33b4dfc0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb4f4dfc2, SUC_lexicon_tags_15 },
    { 0xc794dfc3, SUC_lexicon_tags_25 },
    { 0x1594dfc3, SUC_lexicon_tags_66 },
    { 0x14d4dfc5, SUC_lexicon_tags_37 },
    { 0x1814dfc6, SUC_lexicon_tags_27 },
    { 0x4154dfc3, SUC_lexicon_tags_25 },
    { 0xff94dfc4, SUC_lexicon_tags_17 },
    { 0xb5d4dfc9, SUC_lexicon_tags_29 },
    { 0xb8f4dfc2, SUC_lexicon_tags_30 },
    { 0xa754dfc9, SUC_lexicon_tags_11 },
    { 0x9394dfcc, SUC_lexicon_tags_37 },
    { 0xbad4dfcc, SUC_lexicon_tags_12 },
    { 0xfb94dfcc, SUC_lexicon_tags_7 },
    { 0x52b4dfcf, SUC_lexicon_tags_49 },
    { 0xd214dfcd, SUC_lexicon_tags_9 },
    { 0x8414dfd1, SUC_lexicon_tags_28 },
    { 0x6054dfc3, SUC_lexicon_tags_38 },
    { 0x5274dfcf, SUC_lexicon_tags_37 },
    { 0x8194dfd4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x91b4dfd6, SUC_lexicon_tags_7 },
    { 0xf2f4dfd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b74dfd9, SUC_lexicon_tags_57 },
    { 0x50d4dfd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3dd4dfdc, SUC_lexicon_tags_38 },
    { 0x3694dfdd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4194dfdf, SUC_lexicon_tags_60 },
    { 0x5214dfe0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf34dfe2, SUC_lexicon_tags_37 },
    { 0x4a94dfe3, SUC_lexicon_tags_27 },
    { 0x37b4dfe3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb774dfe8, SUC_lexicon_tags_7 },
    { 0x14f4dfe9, SUC_lexicon_tags_49 },
    { 0xd5b4dfe8, SUC_lexicon_tags_11 },
    { 0x2ab4dfeb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5794dff3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd174dff6, SUC_lexicon_tags_15 },
    { 0x3294dff7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1834dffa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9014dffc, SUC_lexicon_tags_12 },
    { 0xaef4dffd, SUC_lexicon_tags_11 },
    { 0xf694dffe, SUC_lexicon_tags_201 },
    { 0xcbb4dfff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c94e002, SUC_lexicon_tags_45 },
    { 0x8d14e003, SUC_lexicon_tags_12 },
    { 0x7c74e004, SUC_lexicon_tags_17 },
    { 0x8934e004, SUC_lexicon_tags_77 },
    { 0x9af4e003, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d4e00a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b4e00c, SUC_lexicon_tags_37 },
    { 0x4374e00d, SUC_lexicon_tags_37 },
    { 0xb34e00e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d34e011, SUC_lexicon_tags_65 },
    { 0xad54e012, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd94e017, SUC_lexicon_tags_66 },
    { 0x1654e018, SUC_lexicon_tags_17 },
    { 0xe614e019, SUC_lexicon_tags_11 },
    { 0x3434e01a, SUC_lexicon_tags_12 },
    { 0xe834e01b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5314e01d, SUC_lexicon_tags_37 },
    { 0xa494e01e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccd4e020, SUC_lexicon_tags_1 },
    { 0x7794e021, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9814e023, SUC_lexicon_tags_26 },
    { 0xa9d4e024, SUC_lexicon_tags_27 },
    { 0x16d4e025, SUC_lexicon_tags_2 },
    { 0x9eb4e025, SUC_lexicon_tags_27 },
    { 0xae74e025, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b4e02a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd4e02f, SUC_lexicon_tags_15 },
    { 0xd934e030, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d4e033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b4e038, SUC_lexicon_tags_87 },
    { 0x30f4e038, SUC_lexicon_tags_7 },
    { 0xb114e03a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x27d4e03c, SUC_lexicon_tags_9 },
    { 0xa474e03c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b4e041, SUC_lexicon_tags_7 },
    { 0xab14e042, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d14e046, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd4e04a, SUC_lexicon_tags_11 },
    { 0xdf74e04b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x99d4e04d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c94e04f, SUC_lexicon_tags_7 },
    { 0x86f4e050, SUC_lexicon_tags_37 },
    { 0x4a14e051, SUC_lexicon_tags_85 },
    { 0xf94e050, SUC_lexicon_tags_43 },
    { 0xabd4e052, SUC_lexicon_tags_6 },
    { 0x9af4e051, SUC_lexicon_tags_17 },
    { 0x9e74e055, SUC_lexicon_tags_92 },
    { 0x1514e056, SUC_lexicon_tags_2 },
    { 0xc1d4e057, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4734e059, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4874e05d, SUC_lexicon_tags_6 },
    { 0xc6b4e05d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x894e061, SUC_lexicon_tags_5 },
    { 0xf814e062, SUC_lexicon_tags_26 },
    { 0xf754e063, SUC_lexicon_tags_38 },
    { 0x6eb4e064, SUC_lexicon_tags_12 },
    { 0x99f4e065, SUC_lexicon_tags_11 },
    { 0x9354e066, SUC_lexicon_tags_7 },
    { 0x754e066, SUC_lexicon_tags_15 },
    { 0x6954e068, SUC_lexicon_tags_27 },
    { 0x3934e069, SUC_lexicon_tags_37 },
    { 0xa4b4e06a, SUC_lexicon_tags_1 },
    { 0xbad4e067, SUC_lexicon_tags_33 },
    { 0xe754e06c, SUC_lexicon_tags_15 },
    { 0x4ff4e06c, SUC_lexicon_tags_26 },
    { 0x4a74e06d, SUC_lexicon_tags_17 },
    { 0xc874e06e, SUC_lexicon_tags_26 },
    { 0xd7f4e068, SUC_lexicon_tags_2 },
    { 0xff74e06e, SUC_lexicon_tags_11 },
    { 0xfeb4e072, SUC_lexicon_tags_49 },
    { 0x6714e072, SUC_lexicon_tags_79 },
    { 0x8ef4e072, SUC_lexicon_tags_39 },
    { 0xf5d4e074, SUC_lexicon_tags_17 },
    { 0x79d4e070, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd4e07a, SUC_lexicon_tags_124 },
    { 0x8db4e07b, SUC_lexicon_tags_17 },
    { 0xca34e07b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4274e083, SUC_lexicon_tags_0 },
    { 0x6534e084, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1494e086, SUC_lexicon_tags_16 },
    { 0x1674e086, SUC_lexicon_tags_38 },
    { 0x4d4e088, SUC_lexicon_tags_43 },
    { 0x5214e088, SUC_lexicon_tags_43 },
    { 0xee74e08a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb4e08f, SUC_lexicon_tags_65 },
    { 0x1334e090, SUC_lexicon_tags_7 },
    { 0x6f14e090, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6b4e093, SUC_lexicon_tags_252 },
    { 0xc614e094, SUC_lexicon_tags_65 },
    { 0x5e34e095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e34e099, SUC_lexicon_tags_7 },
    { 0xb4b4e09a, SUC_lexicon_tags_17 },
    { 0x1554e099, SUC_lexicon_tags_43 },
    { 0x2474e09c, SUC_lexicon_tags_25 },
    { 0x2c54e09d, SUC_lexicon_tags_27 },
    { 0xc094e09d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a94e0a1, SUC_lexicon_tags_2 },
    { 0xe2f4e0a2, SUC_lexicon_tags_2 },
    { 0xb6d4e0a1, SUC_lexicon_tags_25 },
    { 0xed4e0a3, SUC_lexicon_tags_29 },
    { 0xda94e0a5, SUC_lexicon_tags_2 },
    { 0xbeb4e0a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd594e0a8, SUC_lexicon_tags_47 },
    { 0xdc94e0a8, SUC_lexicon_tags_31 },
    { 0x3bb4e0aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd4e0ac, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4e0af, SUC_lexicon_tags_37 },
    { 0x56f4e0b0, SUC_lexicon_tags_28 },
    { 0x6c14e0b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554e0b5, SUC_lexicon_tags_49 },
    { 0xe8f4e0b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e94e0b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac94e0ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f4e0bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a34e0c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa314e0c3, SUC_lexicon_tags_15 },
    { 0xf094e0c4, SUC_lexicon_tags_1 },
    { 0x35d4e0c3, SUC_lexicon_tags_49 },
    { 0x1bd4e0c6, SUC_lexicon_tags_39 },
    { 0x89f4e0c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9114e0cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f4e0ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1054e0d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xddb4e0d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc994e0d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc34e0db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x59b4e0dd, SUC_lexicon_tags_25 },
    { 0xe634e0de, SUC_lexicon_tags_31 },
    { 0xa94e0df, SUC_lexicon_tags_21 },
    { 0x1d14e0df, SUC_lexicon_tags_32 },
    { 0x7314e0de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1d4e0e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaff4e0e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9894e0e8, SUC_lexicon_tags_26 },
    { 0x9634e0e9, SUC_lexicon_tags_40 },
    { 0x8934e0ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe194e0f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2214e0fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa494e0fd, SUC_lexicon_tags_61 },
    { 0x1e34e0fe, SUC_lexicon_tags_47 },
    { 0x86f4e0ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1954e101, SUC_lexicon_tags_17 },
    { 0xfa74e101, SUC_lexicon_tags_33 },
    { 0x5f74e102, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d4e108, SUC_lexicon_tags_30 },
    { 0x7074e109, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3a14e10b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa8f4e10d, SUC_lexicon_tags_37 },
    { 0xd234e10e, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0xb754e111, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c74e118, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb614e11d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe594e121, SUC_lexicon_tags_42 },
    { 0xa974e122, SUC_lexicon_tags_31 },
    { 0xed34e123, SUC_lexicon_tags_12 },
    { 0x3154e123, SUC_lexicon_tags_43 },
    { 0xf054e125, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4b4e127, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb954e129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd094e12b, SUC_lexicon_tags_57 },
    { 0x4234e12c, SUC_lexicon_tags_11 },
    { 0x8f74e12c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1df4e12f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd714e131, SUC_lexicon_tags_2 },
    { 0x7ab4e132, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x614e139, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1414e13c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e94e13f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d4e147, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f4e14a, SUC_lexicon_tags_68 },
    { 0xef14e14a, SUC_lexicon_tags_12 },
    { 0x25d4e14c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f4e14f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc434e151, SUC_lexicon_tags_16 },
    { 0xdad4e152, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1954e15a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4e160, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f34e163, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8b4e165, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db4e16a, SUC_lexicon_tags_1 },
    { 0x24b4e16b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2414e16e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x8014e170, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3114e173, SUC_lexicon_tags_17 },
    { 0xbbb4e174, SUC_lexicon_tags_11 },
    { 0x1414e175, SUC_lexicon_tags_77 },
    { 0xae74e176, SUC_lexicon_tags_12 },
    { 0x3594e174, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7794e17b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb4e181, SUC_lexicon_tags_7 },
    { 0x9074e182, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa674e184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda54e189, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5354e18e, SUC_lexicon_tags_37 },
    { 0x40f4e18e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f4e192, SUC_lexicon_tags_23 },
    { 0x8db4e193, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6734e19b, SUC_lexicon_tags_17 },
    { 0xc6f4e19c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22f4e19e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd9d4e1a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5954e1a4, SUC_lexicon_tags_675 },
    { 0xf7d4e1a4, SUC_lexicon_tags_16 },
    { 0x8354e1a6, SUC_lexicon_tags_87 },
    { 0xb354e1a7, SUC_lexicon_tags_25 },
    { 0x6974e1a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f4e1ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd914e1ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7df4e1af, SUC_lexicon_tags_25 },
    { 0x57b4e1af, SUC_lexicon_tags_12 },
    { 0x6c54e1b1, SUC_lexicon_tags_27 },
    { 0xf194e1b1, SUC_lexicon_tags_2 },
    { 0xdb54e1b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa134e1b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7994e1b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2114e1ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5db4e1bc, SUC_lexicon_tags_46 },
    { 0x4bb4e1bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe634e1c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfbd4e1c3, SUC_lexicon_tags_35 },
    { 0x5774e1c3, SUC_lexicon_tags_47 },
    { 0xffd4e1c5, SUC_lexicon_tags_11 },
    { 0xc034e1c6, SUC_lexicon_tags_2 },
    { 0x4534e1c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x13b4e1c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a54e1cd, SUC_lexicon_tags_43 },
    { 0xe9d4e1cd, SUC_lexicon_tags_49 },
    { 0x3274e1cf, SUC_lexicon_tags_12 },
    { 0x8f74e1d0, SUC_lexicon_tags_60 },
    { 0xd314e1d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8634e1d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc54e1d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0b4e1d8, SUC_lexicon_tags_7 },
    { 0xbf34e1d8, SUC_lexicon_tags_29 },
    { 0x98b4e1da, SUC_lexicon_tags_27 },
    { 0x9f14e1db, SUC_lexicon_tags_20 },
    { 0x3754e1dc, SUC_lexicon_tags_0 },
    { 0xeb14e1db, SUC_lexicon_tags_17 },
    { 0xd134e1de, SUC_lexicon_tags_28 },
    { 0xac54e1de, SUC_lexicon_tags_17 },
    { 0xe234e1e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b54e1e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb6f4e1e5, SUC_lexicon_tags_80 },
    { 0x58d4e1e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4294e1e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f54e1ea, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x80b4e1ec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8f94e1ee, SUC_lexicon_tags_0 },
    { 0xf674e1ef, SUC_lexicon_tags_23 },
    { 0x83b4e1ef, SUC_lexicon_tags_17 },
    { 0x7ed4e1f0, SUC_lexicon_tags_11 },
    { 0x7db4e1f2, SUC_lexicon_tags_15 },
    { 0x7d14e1f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8494e1f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e94e1f7, SUC_lexicon_tags_160 },
    { 0xfd54e1f8, SUC_lexicon_tags_65 },
    { 0x174e1f9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f4e1fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef4e207, SUC_lexicon_tags_32 },
    { 0xc894e208, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c94e20b, SUC_lexicon_tags_17 },
    { 0x95b4e20c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b4e213, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8114e216, SUC_lexicon_tags_9 },
    { 0xa7f4e217, SUC_lexicon_tags_11 },
    { 0xbf94e218, SUC_lexicon_tags_29 },
    { 0x62b4e217, SUC_lexicon_tags_11 },
    { 0x7594e217, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa554e21e, SUC_lexicon_tags_31 },
    { 0x19f4e21f, SUC_lexicon_tags_25 },
    { 0x5414e220, SUC_lexicon_tags_7 },
    { 0xaf34e220, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b4e225, SUC_lexicon_tags_37 },
    { 0x9f4e226, SUC_lexicon_tags_37 },
    { 0x9f94e225, SUC_lexicon_tags_17 },
    { 0xa4b4e226, SUC_lexicon_tags_14 },
    { 0x2274e229, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe834e22d, SUC_lexicon_tags_27 },
    { 0x6114e22e, SUC_lexicon_tags_25 },
    { 0xdbb4e22f, SUC_lexicon_tags_55 },
    { 0xd134e230, SUC_lexicon_tags_114 },
    { 0xe874e22e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xabf4e233, SUC_lexicon_tags_49 },
    { 0x1c74e234, SUC_lexicon_tags_6 },
    { 0x9234e235, SUC_lexicon_tags_45 },
    { 0x9274e236, SUC_lexicon_tags_1 },
    { 0x9ef4e237, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8034e23a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c34e240, SUC_lexicon_tags_38 },
    { 0x6534e240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf4e243, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x2534e245, SUC_lexicon_tags_9 },
    { 0xa394e246, SUC_lexicon_tags_7 },
    { 0x7934e245, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d34e24a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d4e24f, SUC_lexicon_tags_25 },
    { 0x1494e250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x494e253, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6734e259, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d4e25c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d4e262, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd4e268, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xac34e26b, SUC_lexicon_tags_27 },
    { 0xe134e26c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb14e26f, SUC_lexicon_tags_15 },
    { 0x4f14e26f, SUC_lexicon_tags_11 },
    { 0x1034e271, SUC_lexicon_tags_29 },
    { 0x7a94e271, SUC_lexicon_tags_37 },
    { 0x9614e273, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x574e276, SUC_lexicon_tags_4 },
    { 0xe1d4e276, SUC_lexicon_tags_107 },
    { 0xa2f4e277, SUC_lexicon_tags_12 },
    { 0x10d4e279, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaf34e27b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4054e27f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf4e282, SUC_lexicon_tags_12 },
    { 0x374e283, SUC_lexicon_tags_63 },
    { 0xa0b4e282, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfbd4e286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b94e28b, SUC_lexicon_tags_12 },
    { 0x794e28b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc034e28e, SUC_lexicon_tags_34 },
    { 0xef34e28f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d4e295, SUC_lexicon_tags_44 },
    { 0x9754e296, SUC_lexicon_tags_1 },
    { 0xc514e297, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b4e29b, SUC_lexicon_tags_61 },
    { 0x2794e29c, SUC_lexicon_tags_37 },
    { 0xf934e29b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b94e29f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6014e2a1, SUC_lexicon_tags_47 },
    { 0x1974e2a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f4e2a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ab4e2aa, SUC_lexicon_tags_37 },
    { 0x8d4e2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa734e2ae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c14e2b1, SUC_lexicon_tags_15 },
    { 0xa2b4e2b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d4e2b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f4e2b9, SUC_lexicon_tags_17 },
    { 0x54f4e2ba, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b4e2c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb734e2c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd434e2ca, SUC_lexicon_tags_12 },
    { 0xd5d4e2cb, SUC_lexicon_tags_2 },
    { 0x2414e2cc, SUC_lexicon_tags_47 },
    { 0xc1d4e2cc, SUC_lexicon_tags_38 },
    { 0x2f4e2cd, SUC_lexicon_tags_46 },
    { 0x1594e2cc, SUC_lexicon_tags_12 },
    { 0x3794e2cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8094e2d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf4e2da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6574e2dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2054e2e2, SUC_lexicon_tags_11 },
    { 0x75f4e2e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f4e2ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaeb4e2ef, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x754e2f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8574e2f4, SUC_lexicon_tags_1 },
    { 0x3634e2f4, SUC_lexicon_tags_2 },
    { 0x2e74e2f6, SUC_lexicon_tags_37 },
    { 0x6ef4e2f5, SUC_lexicon_tags_11 },
    { 0x6b94e2f4, SUC_lexicon_tags_30 },
    { 0x9654e2f9, SUC_lexicon_tags_37 },
    { 0x3714e2fa, SUC_lexicon_tags_7 },
    { 0x2794e2f6, SUC_lexicon_tags_17 },
    { 0x434e2f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8874e2fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe794e301, SUC_lexicon_tags_17 },
    { 0xc654e302, SUC_lexicon_tags_25 },
    { 0x5ef4e303, SUC_lexicon_tags_15 },
    { 0x77b4e302, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c94e306, SUC_lexicon_tags_25 },
    { 0xebb4e306, SUC_lexicon_tags_60 },
    { 0x8b14e308, SUC_lexicon_tags_27 },
    { 0xae74e308, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f4e30c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x854e313, SUC_lexicon_tags_15 },
    { 0xcc94e314, SUC_lexicon_tags_17 },
    { 0x32b4e314, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7cf4e317, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9df4e319, SUC_lexicon_tags_12 },
    { 0xc3f4e319, SUC_lexicon_tags_6 },
    { 0x5894e31a, SUC_lexicon_tags_65 },
    { 0xb554e31c, SUC_lexicon_tags_2 },
    { 0x1974e31c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3954e322, SUC_lexicon_tags_11 },
    { 0x8854e323, SUC_lexicon_tags_47 },
    { 0x8654e324, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb514e327, SUC_lexicon_tags_37 },
    { 0x8d54e328, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1094e32b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18d4e32d, SUC_lexicon_tags_28 },
    { 0x3ed4e32d, SUC_lexicon_tags_47 },
    { 0x8d4e32f, SUC_lexicon_tags_91 },
    { 0xf8d4e330, SUC_lexicon_tags_25 },
    { 0xa074e32f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x42f4e333, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7754e342, SUC_lexicon_tags_37 },
    { 0xb6b4e343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2014e345, SUC_lexicon_tags_47 },
    { 0x1674e346, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3974e34f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3634e352, SUC_lexicon_tags_37 },
    { 0x2dd4e353, SUC_lexicon_tags_0 },
    { 0x3494e354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff4e356, SUC_lexicon_tags_60 },
    { 0xed94e357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b34e35c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34d4e35e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab4e364, SUC_lexicon_tags_37 },
    { 0xbbb4e364, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd4e369, SUC_lexicon_tags_80 },
    { 0x474e369, SUC_lexicon_tags_11 },
    { 0xe554e36b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8c34e36d, SUC_lexicon_tags_1 },
    { 0x4f4e36e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d4e374, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2714e377, SUC_lexicon_tags_14 },
    { 0x9554e378, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb114e37a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b4e37f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef34e385, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9654e38a, SUC_lexicon_tags_1 },
    { 0x90f4e38a, SUC_lexicon_tags_12 },
    { 0x20b4e38c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b4e38f, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb4e392, SUC_lexicon_tags_68 },
    { 0x4b74e393, SUC_lexicon_tags_28 },
    { 0x1e94e394, SUC_lexicon_tags_7 },
    { 0x1774e395, SUC_lexicon_tags_22 },
    { 0x8574e395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa14e399, SUC_lexicon_tags_37 },
    { 0xc8f4e39a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd174e3a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x594e3a8, SUC_lexicon_tags_33 },
    { 0x7194e3a8, SUC_lexicon_tags_47 },
    { 0xd94e3aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef4e3ae, SUC_lexicon_tags_11 },
    { 0x97f4e3af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1674e3b1, SUC_lexicon_tags_37 },
    { 0xcfb4e3b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ad4e3b4, SUC_lexicon_tags_15 },
    { 0xaa94e3b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d4e3b8, SUC_lexicon_tags_47 },
    { 0x3094e3b8, SUC_lexicon_tags_47 },
    { 0x3af4e3ba, SUC_lexicon_tags_56 },
    { 0xb9b4e3b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4914e3bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b4e3c1, SUC_lexicon_tags_15 },
    { 0xda14e3c2, SUC_lexicon_tags_27 },
    { 0x6b34e3c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4494e3cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65d4e3cd, SUC_lexicon_tags_19 },
    { 0xa3b4e3cd, SUC_lexicon_tags_45 },
    { 0x69d4e3cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd4e3d2, SUC_lexicon_tags_31 },
    { 0x52b4e3d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8654e3d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfd4e3d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d4e3e1, SUC_lexicon_tags_7 },
    { 0xb594e3e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa894e3e8, SUC_lexicon_tags_12 },
    { 0xf174e3e9, SUC_lexicon_tags_48 },
    { 0x5654e3e9, SUC_lexicon_tags_37 },
    { 0x1374e3ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3034e3ed, SUC_lexicon_tags_105 },
    { 0x9fd4e3ed, SUC_lexicon_tags_61 },
    { 0x9d4e3ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc94e3f4, SUC_lexicon_tags_0 },
    { 0x2bb4e3f5, SUC_lexicon_tags_37 },
    { 0xbf54e3f5, SUC_lexicon_tags_17 },
    { 0x73f4e3f7, SUC_lexicon_tags_108 },
    { 0x8a74e3f5, SUC_lexicon_tags_17 },
    { 0x5554e3f6, SUC_lexicon_tags_12 },
    { 0xc2f4e3f9, SUC_lexicon_tags_15 },
    { 0x8454e3fb, SUC_lexicon_tags_11 },
    { 0x25f4e3fc, SUC_lexicon_tags_43 },
    { 0xf0b4e3fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x454e403, SUC_lexicon_tags_15 },
    { 0x60b4e403, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f54e406, SUC_lexicon_tags_37 },
    { 0x2134e407, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd4e40a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf4e40e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec54e410, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6694e412, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd994e414, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a34e418, SUC_lexicon_tags_25 },
    { 0x2634e418, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd054e41f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9214e421, SUC_lexicon_tags_47 },
    { 0x2954e422, SUC_lexicon_tags_8 },
    { 0xaeb4e421, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6494e427, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf954e42a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1894e42f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb554e433, SUC_lexicon_tags_37 },
    { 0x3d74e433, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3054e436, SUC_lexicon_tags_87 },
    { 0xc034e437, SUC_lexicon_tags_37 },
    { 0x46f4e436, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f34e43a, SUC_lexicon_tags_68 },
    { 0x6874e43b, SUC_lexicon_tags_1 },
    { 0x70b4e43c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5154e445, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d4e44b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae94e44d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d4e450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6394e453, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8d4e455, SUC_lexicon_tags_23 },
    { 0x7e14e456, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b4e45c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c34e463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d4e468, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3274e46a, SUC_lexicon_tags_37 },
    { 0x6814e46b, SUC_lexicon_tags_26 },
    { 0x73d4e46c, SUC_lexicon_tags_27 },
    { 0xedd4e46d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b54e470, SUC_lexicon_tags_30 },
    { 0xf834e471, SUC_lexicon_tags_12 },
    { 0xe314e472, SUC_lexicon_tags_723 },
    { 0x7674e473, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x554e476, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb4e479, SUC_lexicon_tags_14 },
    { 0xe394e47a, SUC_lexicon_tags_12 },
    { 0x9f74e47b, SUC_lexicon_tags_25 },
    { 0x9694e47b, SUC_lexicon_tags_45 },
    { 0x9614e47c, SUC_lexicon_tags_31 },
    { 0x8714e47e, SUC_lexicon_tags_33 },
    { 0x26b4e47f, SUC_lexicon_tags_11 },
    { 0xbd94e480, SUC_lexicon_tags_11 },
    { 0x3d74e480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d4e485, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x694e488, SUC_lexicon_tags_15 },
    { 0x4ab4e488, SUC_lexicon_tags_27 },
    { 0xbbd4e489, SUC_lexicon_tags_25 },
    { 0xeb94e488, SUC_lexicon_tags_12 },
    { 0x3434e48c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1074e48e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb54e491, SUC_lexicon_tags_23 },
    { 0x2a94e492, SUC_lexicon_tags_65 },
    { 0xefb4e491, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b94e495, SUC_lexicon_tags_37 },
    { 0x4114e496, SUC_lexicon_tags_1 },
    { 0x89b4e497, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7d14e499, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbf4e49b, SUC_lexicon_tags_30 },
    { 0x89b4e49c, SUC_lexicon_tags_63 },
    { 0x9394e49d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b4e4a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f4e4a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6014e4a8, SUC_lexicon_tags_87 },
    { 0xb734e4a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb4e4b3, SUC_lexicon_tags_63 },
    { 0x5234e4b3, SUC_lexicon_tags_21 },
    { 0xcb4e4b4, SUC_lexicon_tags_1 },
    { 0x8434e4b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee54e4b9, SUC_lexicon_tags_43 },
    { 0x95b4e4b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1814e4be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b4e4c3, SUC_lexicon_tags_32 },
    { 0x2014e4c4, SUC_lexicon_tags_12 },
    { 0x36f4e4c5, SUC_lexicon_tags_37 },
    { 0x5e14e4c6, SUC_lexicon_tags_40 },
    { 0xa234e4c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7994e4cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3cb4e4cf, SUC_lexicon_tags_63 },
    { 0x9914e4d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d4e4d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa634e4dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb014e4e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7534e4e4, SUC_lexicon_tags_31 },
    { 0x47d4e4e5, SUC_lexicon_tags_15 },
    { 0x854e4e6, SUC_lexicon_tags_17 },
    { 0x4d4e4e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7834e4e9, SUC_lexicon_tags_33 },
    { 0x1574e4e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f14e4f2, SUC_lexicon_tags_39 },
    { 0xdad4e4f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd294e4f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4e4fa, SUC_lexicon_tags_104 },
    { 0x9594e4fa, SUC_lexicon_tags_21 },
    { 0x8934e4fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe634e4fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd694e500, SUC_lexicon_tags_47 },
    { 0x53f4e501, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab4e507, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf34e50c, SUC_lexicon_tags_17 },
    { 0xa094e50c, SUC_lexicon_tags_28 },
    { 0x394e50e, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b54e511, SUC_lexicon_tags_12 },
    { 0x8f14e511, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52f4e514, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd314e517, SUC_lexicon_tags_17 },
    { 0x9c94e518, SUC_lexicon_tags_26 },
    { 0xf7f4e518, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2074e51b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5334e51e, SUC_lexicon_tags_27 },
    { 0xdcd4e51f, SUC_lexicon_tags_7 },
    { 0xcdb4e520, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d4e528, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4954e52b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb014e52f, SUC_lexicon_tags_6 },
    { 0x1674e52f, SUC_lexicon_tags_29 },
    { 0x53f4e530, SUC_lexicon_tags_12 },
    { 0x2694e531, SUC_lexicon_tags_29 },
    { 0x7ff4e533, SUC_lexicon_tags_30 },
    { 0xa3b4e534, SUC_lexicon_tags_113 },
    { 0xadf4e534, SUC_lexicon_tags_14 },
    { 0xeb54e536, SUC_lexicon_tags_68 },
    { 0xc9f4e536, SUC_lexicon_tags_26 },
    { 0x8174e52f, SUC_lexicon_tags_37 },
    { 0x38f4e539, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7354e53e, SUC_lexicon_tags_16 },
    { 0xef4e53e, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x7774e541, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3034e543, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d4e546, SUC_lexicon_tags_11 },
    { 0xeb4e547, SUC_lexicon_tags_11 },
    { 0x8014e548, SUC_lexicon_tags_56 },
    { 0x63d4e548, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd114e54b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe174e553, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9594e555, SUC_lexicon_tags_17 },
    { 0xe994e556, SUC_lexicon_tags_19 },
    { 0x254e557, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6274e55c, SUC_lexicon_tags_17 },
    { 0x7a74e55d, SUC_lexicon_tags_37 },
    { 0xadb4e55e, SUC_lexicon_tags_25 },
    { 0xb54e55c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8134e563, SUC_lexicon_tags_10 },
    { 0x9634e564, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb94e567, SUC_lexicon_tags_37 },
    { 0x9b94e568, SUC_lexicon_tags_7 },
    { 0x7a54e568, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd134e56b, SUC_lexicon_tags_42 },
    { 0xb114e56c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d4e572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf274e575, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d14e578, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd154e57a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c94e57d, SUC_lexicon_tags_37 },
    { 0x1c54e57e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f4e585, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6114e588, SUC_lexicon_tags_15 },
    { 0xe1b4e588, SUC_lexicon_tags_27 },
    { 0xae54e58a, SUC_lexicon_tags_32 },
    { 0x7f34e589, SUC_lexicon_tags_25 },
    { 0xc174e58c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfdd4e58e, SUC_lexicon_tags_33 },
    { 0xd7f4e58f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4314e591, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d54e594, SUC_lexicon_tags_23 },
    { 0xfff4e595, SUC_lexicon_tags_46 },
    { 0xae94e594, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda94e599, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc054e59b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x7754e59e, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2994e5a3, SUC_lexicon_tags_37 },
    { 0x4df4e5a3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2414e5a8, SUC_lexicon_tags_23 },
    { 0x5bf4e5a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f34e5ae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4994e5b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c94e5b4, SUC_lexicon_tags_56 },
    { 0xcf54e5b4, SUC_lexicon_tags_29 },
    { 0x1bf4e5b6, SUC_lexicon_tags_15 },
    { 0xb4e5b6, SUC_lexicon_tags_25 },
    { 0x22f4e5b4, SUC_lexicon_tags_47 },
    { 0xfbb4e5b7, SUC_lexicon_tags_27 },
    { 0xb1f4e5ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4914e5c1, SUC_lexicon_tags_11 },
    { 0x83f4e5c1, SUC_lexicon_tags_86 },
    { 0xc1b4e5c3, SUC_lexicon_tags_25 },
    { 0x3ab4e5c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6554e5d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc474e5d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6394e5d9, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b54e5dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad4e5e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb94e5e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb14e5e7, SUC_lexicon_tags_66 },
    { 0x2794e5e8, SUC_lexicon_tags_44 },
    { 0xcab4e5e9, SUC_lexicon_tags_30 },
    { 0xaf74e5ea, SUC_lexicon_tags_47 },
    { 0xdc94e5eb, SUC_lexicon_tags_25 },
    { 0x7cd4e5ec, SUC_lexicon_tags_49 },
    { 0xc4b4e5ed, SUC_lexicon_tags_63 },
    { 0x7054e5ee, SUC_lexicon_tags_29 },
    { 0x3b14e5ed, SUC_lexicon_tags_12 },
    { 0x8094e5ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c94e5f6, SUC_lexicon_tags_19 },
    { 0x4a54e5f7, SUC_lexicon_tags_37 },
    { 0x6974e5f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3ab4e5fa, SUC_lexicon_tags_28 },
    { 0x8cf4e5fb, SUC_lexicon_tags_17 },
    { 0x5d14e5fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f4e600, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b4e606, SUC_lexicon_tags_47 },
    { 0xa14e607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa314e60e, SUC_lexicon_tags_49 },
    { 0x6e34e60f, SUC_lexicon_tags_33 },
    { 0xbf74e60f, SUC_lexicon_tags_11 },
    { 0xd4d4e60e, SUC_lexicon_tags_47 },
    { 0x80d4e612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d4e618, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x87d4e61a, SUC_lexicon_tags_11 },
    { 0x4f94e61b, SUC_lexicon_tags_7 },
    { 0xd9b4e61b, SUC_lexicon_tags_34 },
    { 0x4454e61a, SUC_lexicon_tags_2 },
    { 0x8354e61e, SUC_lexicon_tags_26 },
    { 0x9134e61f, SUC_lexicon_tags_37 },
    { 0x6c14e61d, SUC_lexicon_tags_12 },
    { 0x2ef4e621, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf4e625, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd4e628, SUC_lexicon_tags_48 },
    { 0xaaf4e629, SUC_lexicon_tags_47 },
    { 0xa374e629, SUC_lexicon_tags_28 },
    { 0xde54e62b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f94e633, SUC_lexicon_tags_15 },
    { 0x7af4e634, SUC_lexicon_tags_9 },
    { 0x6db4e635, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d4e638, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc554e63f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e74e641, SUC_lexicon_tags_15 },
    { 0x9c74e641, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4d4e644, SUC_lexicon_tags_47 },
    { 0x1734e644, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2194e647, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f74e64b, SUC_lexicon_tags_12 },
    { 0xe0d4e64c, SUC_lexicon_tags_11 },
    { 0xa654e64d, SUC_lexicon_tags_31 },
    { 0x7fb4e64b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f34e651, SUC_lexicon_tags_6 },
    { 0xc114e651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8274e65b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f4e660, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad94e662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb954e667, SUC_lexicon_tags_2 },
    { 0x9834e668, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec54e66b, SUC_lexicon_tags_17 },
    { 0x5d4e66c, SUC_lexicon_tags_15 },
    { 0x5954e66d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb4e674, SUC_lexicon_tags_22 },
    { 0x7bb4e674, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe474e678, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb34e67d, SUC_lexicon_tags_8 },
    { 0x8cf4e67e, SUC_lexicon_tags_43 },
    { 0x98f4e67d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6274e681, SUC_lexicon_tags_33 },
    { 0xc274e682, SUC_lexicon_tags_23 },
    { 0xf0f4e682, SUC_lexicon_tags_33 },
    { 0x60d4e684, SUC_lexicon_tags_27 },
    { 0xf574e681, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f4e68c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3754e691, SUC_lexicon_tags_2 },
    { 0xfcf4e692, SUC_lexicon_tags_72 },
    { 0x1f4e693, SUC_lexicon_tags_63 },
    { 0x4f14e694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe434e697, SUC_lexicon_tags_39 },
    { 0x5fd4e697, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f4e69c, SUC_lexicon_tags_17 },
    { 0xae14e69c, SUC_lexicon_tags_60 },
    { 0x6af4e69e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a54e6a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75f4e6a6, SUC_lexicon_tags_14 },
    { 0x3714e6a6, SUC_lexicon_tags_25 },
    { 0xa214e6a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d34e6ac, SUC_lexicon_tags_79 },
    { 0xb434e6ad, SUC_lexicon_tags_31 },
    { 0xe6b4e6ae, SUC_lexicon_tags_55 },
    { 0x6954e6af, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5554e6b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a54e6b7, SUC_lexicon_tags_27 },
    { 0xf3f4e6b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf4e6bd, SUC_lexicon_tags_159 },
    { 0x4954e6be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd4e6c1, SUC_lexicon_tags_12 },
    { 0xd114e6c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a94e6c4, SUC_lexicon_tags_11 },
    { 0x2e94e6c4, SUC_lexicon_tags_1 },
    { 0xcc34e6c4, SUC_lexicon_tags_7 },
    { 0xca94e6c5, SUC_lexicon_tags_23 },
    { 0xabf4e6c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef4e6cb, SUC_lexicon_tags_60 },
    { 0xef34e6cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd4e6d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5594e6d2, SUC_lexicon_tags_79 },
    { 0x28f4e6d3, SUC_lexicon_tags_45 },
    { 0x66f4e6d3, SUC_lexicon_tags_32 },
    { 0x5694e6d5, SUC_lexicon_tags_39 },
    { 0x6074e6d5, SUC_lexicon_tags_26 },
    { 0x6ad4e6d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d4e6df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf474e6e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f4e6e7, SUC_lexicon_tags_33 },
    { 0x5274e6e8, SUC_lexicon_tags_2 },
    { 0x5c74e6e9, SUC_lexicon_tags_11 },
    { 0xddb4e6ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d54e6ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d4e6f2, SUC_lexicon_tags_43 },
    { 0xf3b4e6f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7614e6f5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4714e6fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ad4e6fc, SUC_lexicon_tags_17 },
    { 0xfc54e6fc, SUC_lexicon_tags_12 },
    { 0xc6f4e6fe, SUC_lexicon_tags_7 },
    { 0xe614e6ff, SUC_lexicon_tags_17 },
    { 0x6054e6fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x754e706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b94e709, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b4e70c, SUC_lexicon_tags_25 },
    { 0xe7b4e70d, SUC_lexicon_tags_2 },
    { 0x5a94e70d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d54e710, SUC_lexicon_tags_25 },
    { 0x2a94e710, SUC_lexicon_tags_7 },
    { 0x63d4e711, SUC_lexicon_tags_12 },
    { 0xe774e711, SUC_lexicon_tags_12 },
    { 0xac54e712, SUC_lexicon_tags_17 },
    { 0x1194e715, SUC_lexicon_tags_28 },
    { 0x5154e715, SUC_lexicon_tags_47 },
    { 0xe7f4e714, SUC_lexicon_tags_19 },
    { 0x10f4e718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea34e71b, SUC_lexicon_tags_17 },
    { 0x47d4e71c, SUC_lexicon_tags_16 },
    { 0xce14e71c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd4e720, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ed4e722, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f34e726, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2534e728, SUC_lexicon_tags_15 },
    { 0xde54e728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d14e732, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c54e737, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf054e73c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff4e743, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9094e748, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6a54e74a, SUC_lexicon_tags_37 },
    { 0xdd4e74a, SUC_lexicon_tags_28 },
    { 0x32b4e74c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb74e750, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94e757, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e54e75c, SUC_lexicon_tags_15 },
    { 0x7734e75d, SUC_lexicon_tags_2 },
    { 0xe94e75e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3834e763, SUC_lexicon_tags_1 },
    { 0x1454e764, SUC_lexicon_tags_11 },
    { 0x7994e765, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d94e767, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5894e76e, SUC_lexicon_tags_14 },
    { 0x3374e76e, SUC_lexicon_tags_15 },
    { 0x9c54e76e, SUC_lexicon_tags_6 },
    { 0x6df4e76f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f94e775, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea94e777, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7914e77a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda14e77c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3214e77e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ff4e780, SUC_lexicon_tags_46 },
    { 0xfeb4e780, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa994e785, SUC_lexicon_tags_17 },
    { 0xa014e786, SUC_lexicon_tags_17 },
    { 0xa3d4e787, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82b4e789, SUC_lexicon_tags_38 },
    { 0x4d54e78a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5134e78c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df4e796, SUC_lexicon_tags_13 },
    { 0x48f4e796, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9d4e799, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6834e79b, SUC_lexicon_tags_59 },
    { 0x994e79c, SUC_lexicon_tags_25 },
    { 0xa294e79d, SUC_lexicon_tags_54 },
    { 0xb294e79e, SUC_lexicon_tags_62 },
    { 0xfe34e79d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d4e7a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6614e7ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6374e7af, SUC_lexicon_tags_21 },
    { 0xa194e7b0, SUC_lexicon_tags_30 },
    { 0x1eb4e7b1, SUC_lexicon_tags_8 },
    { 0xd3b4e7b0, SUC_lexicon_tags_70 },
    { 0x6d14e7b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d54e7b6, SUC_lexicon_tags_11 },
    { 0x43d4e7b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8314e7bf, SUC_lexicon_tags_23 },
    { 0x98f4e7bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4074e7c3, SUC_lexicon_tags_15 },
    { 0x2674e7c3, SUC_lexicon_tags_9 },
    { 0x7054e7c5, SUC_lexicon_tags_15 },
    { 0xd4f4e7c6, SUC_lexicon_tags_29 },
    { 0x9eb4e7c6, SUC_lexicon_tags_37 },
    { 0x5394e7c4, SUC_lexicon_tags_12 },
    { 0xb994e7c8, SUC_lexicon_tags_17 },
    { 0x23b4e7ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc474e7cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd4e7d5, SUC_lexicon_tags_19 },
    { 0xf8f4e7d5, SUC_lexicon_tags_35 },
    { 0xacf4e7d7, SUC_lexicon_tags_26 },
    { 0xf7d4e7d8, SUC_lexicon_tags_12 },
    { 0xd0d4e7d8, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b74e7e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb314e7e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62b4e7e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xec14e7e9, SUC_lexicon_tags_30 },
    { 0x7374e7ea, SUC_lexicon_tags_2 },
    { 0xa734e7e9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x2234e7ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1514e7ef, SUC_lexicon_tags_17 },
    { 0x2ed4e7ef, SUC_lexicon_tags_14 },
    { 0x86d4e7ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4234e7f8, SUC_lexicon_tags_26 },
    { 0x7354e7f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1814e7fc, SUC_lexicon_tags_13 },
    { 0xab4e7fd, SUC_lexicon_tags_12 },
    { 0x7434e7fe, SUC_lexicon_tags_15 },
    { 0x69d4e7ff, SUC_lexicon_tags_70 },
    { 0x30d4e7ff, SUC_lexicon_tags_12 },
    { 0xf314e801, SUC_lexicon_tags_37 },
    { 0x5914e7ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f4e809, SUC_lexicon_tags_30 },
    { 0x894e80a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb4e80d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7374e80f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6d4e811, SUC_lexicon_tags_37 },
    { 0x1494e812, SUC_lexicon_tags_16 },
    { 0x9394e812, SUC_lexicon_tags_11 },
    { 0xa9d4e812, SUC_lexicon_tags_39 },
    { 0x1a14e815, SUC_lexicon_tags_140 },
    { 0xccd4e816, SUC_lexicon_tags_22 },
    { 0x85b4e815, SUC_lexicon_tags_8 },
    { 0x1294e818, SUC_lexicon_tags_472 },
    { 0xa8d4e817, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb4e821, SUC_lexicon_tags_15 },
    { 0x1554e821, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef4e825, SUC_lexicon_tags_17 },
    { 0xf1f4e826, SUC_lexicon_tags_15 },
    { 0x37b4e827, SUC_lexicon_tags_8 },
    { 0xe4b4e828, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xad14e82b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4314e834, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb494e836, SUC_lexicon_tags_0 },
    { 0xe254e837, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1db4e839, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x4634e83b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa114e840, SUC_lexicon_tags_33 },
    { 0x8f54e840, SUC_lexicon_tags_37 },
    { 0x2614e841, SUC_lexicon_tags_37 },
    { 0x3cd4e843, SUC_lexicon_tags_15 },
    { 0x1094e841, SUC_lexicon_tags_7 },
    { 0xa274e843, SUC_lexicon_tags_25 },
    { 0x7d14e840, SUC_lexicon_tags_14 },
    { 0x3434e847, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d74e84a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5ff4e84c, SUC_lexicon_tags_33 },
    { 0x974e84c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe274e851, SUC_lexicon_tags_17 },
    { 0x5754e851, SUC_lexicon_tags_39 },
    { 0x6b94e853, SUC_lexicon_tags_43 },
    { 0x3574e853, SUC_lexicon_tags_127 },
    { 0xb534e855, SUC_lexicon_tags_8 },
    { 0x3cb4e856, SUC_lexicon_tags_60 },
    { 0x9bd4e857, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3314e85a, SUC_lexicon_tags_65 },
    { 0xcc34e85a, SUC_lexicon_tags_8 },
    { 0x9d34e85c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe14e868, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3894e86a, SUC_lexicon_tags_11 },
    { 0xe634e86a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c74e86f, SUC_lexicon_tags_15 },
    { 0x65f4e870, SUC_lexicon_tags_25 },
    { 0xf5d4e86f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2754e87a, SUC_lexicon_tags_60 },
    { 0x8834e87b, SUC_lexicon_tags_27 },
    { 0x4254e87b, SUC_lexicon_tags_29 },
    { 0x79d4e87d, SUC_lexicon_tags_172 },
    { 0xe7d4e87e, SUC_lexicon_tags_25 },
    { 0x65d4e87f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf4e883, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d4e889, SUC_lexicon_tags_11 },
    { 0x1934e88a, SUC_lexicon_tags_7 },
    { 0xa5d4e889, SUC_lexicon_tags_27 },
    { 0x5c34e88a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfa54e88e, SUC_lexicon_tags_14 },
    { 0x1194e88f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2154e892, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6934e895, SUC_lexicon_tags_1 },
    { 0x7854e896, SUC_lexicon_tags_11 },
    { 0x7434e896, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4574e89f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85f4e8a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a14e8a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd34e8a8, SUC_lexicon_tags_37 },
    { 0x9454e8a9, SUC_lexicon_tags_61 },
    { 0x7b54e8aa, SUC_lexicon_tags_38 },
    { 0xfd34e8aa, SUC_lexicon_tags_25 },
    { 0xfa14e8aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad4e8b2, SUC_lexicon_tags_4 },
    { 0xc194e8b3, SUC_lexicon_tags_5 },
    { 0x8774e8b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b4e8b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd314e8ba, SUC_lexicon_tags_27 },
    { 0xbd14e8bb, SUC_lexicon_tags_29 },
    { 0xb674e8bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f4e8bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4314e8c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf794e8ca, SUC_lexicon_tags_11 },
    { 0xc5b4e8cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab4e8d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4834e8da, SUC_lexicon_tags_1 },
    { 0xc874e8db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce14e8dd, SUC_lexicon_tags_1 },
    { 0x7c54e8de, SUC_lexicon_tags_37 },
    { 0x7a34e8df, SUC_lexicon_tags_11 },
    { 0xe314e8e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe754e8e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc954e8e4, SUC_lexicon_tags_27 },
    { 0x55d4e8e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc74e8e9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef4e8ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f4e8f0, SUC_lexicon_tags_7 },
    { 0xafb4e8f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b4e8f5, SUC_lexicon_tags_30 },
    { 0xc334e8f5, SUC_lexicon_tags_30 },
    { 0x6654e8f7, SUC_lexicon_tags_2 },
    { 0x8114e8f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5994e900, SUC_lexicon_tags_46 },
    { 0xc5f4e901, SUC_lexicon_tags_19 },
    { 0xc9d4e900, SUC_lexicon_tags_1 },
    { 0x3bb4e900, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f4e906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f14e908, SUC_lexicon_tags_11 },
    { 0x5854e909, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6314e90c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8654e910, SUC_lexicon_tags_30 },
    { 0x2434e910, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a14e914, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa094e916, SUC_lexicon_tags_25 },
    { 0x5ff4e916, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95d4e919, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe994e91b, SUC_lexicon_tags_11 },
    { 0x8734e91c, SUC_lexicon_tags_11 },
    { 0x3694e91b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f4e920, SUC_lexicon_tags_25 },
    { 0x90f4e921, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8094e926, SUC_lexicon_tags_8 },
    { 0x21d4e927, SUC_lexicon_tags_84 },
    { 0xf594e927, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf874e92a, SUC_lexicon_tags_25 },
    { 0x9794e92b, SUC_lexicon_tags_22 },
    { 0x8d94e92b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d4e92f, SUC_lexicon_tags_56 },
    { 0x8b74e92f, SUC_lexicon_tags_15 },
    { 0x1a74e930, SUC_lexicon_tags_15 },
    { 0x1f94e932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d4e937, SUC_lexicon_tags_17 },
    { 0x594e937, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b4e93e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf514e948, SUC_lexicon_tags_44 },
    { 0xd0f4e949, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe854e94b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb174e952, SUC_lexicon_tags_37 },
    { 0xe54e953, SUC_lexicon_tags_23 },
    { 0x8794e953, SUC_lexicon_tags_30 },
    { 0x3ef4e952, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd414e95c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c74e960, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2814e962, SUC_lexicon_tags_31 },
    { 0xa9b4e963, SUC_lexicon_tags_19 },
    { 0x3b74e963, SUC_lexicon_tags_2 },
    { 0x83b4e965, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8134e96a, SUC_lexicon_tags_68 },
    { 0x6b4e96a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e14e96e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3254e970, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4834e979, SUC_lexicon_tags_11 },
    { 0x3434e97a, SUC_lexicon_tags_7 },
    { 0xd5b4e97a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d4e97e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9254e983, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba14e985, SUC_lexicon_tags_37 },
    { 0x5c34e986, SUC_lexicon_tags_8 },
    { 0xad34e987, SUC_lexicon_tags_25 },
    { 0x9294e988, SUC_lexicon_tags_16 },
    { 0x9074e985, SUC_lexicon_tags_29 },
    { 0xfc74e989, SUC_lexicon_tags_11 },
    { 0xb414e987, SUC_lexicon_tags_16 },
    { 0x9314e98c, SUC_lexicon_tags_12 },
    { 0x83d4e98c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa554e990, SUC_lexicon_tags_25 },
    { 0x4974e991, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa394e993, SUC_lexicon_tags_38 },
    { 0x5d4e993, SUC_lexicon_tags_25 },
    { 0x8d34e993, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7394e997, SUC_lexicon_tags_31 },
    { 0xc34e998, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8334e99e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b4e9a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5114e9a4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd4e9aa, SUC_lexicon_tags_11 },
    { 0x7994e9ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58b4e9ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe914e9af, SUC_lexicon_tags_19 },
    { 0x7654e9b0, SUC_lexicon_tags_36 },
    { 0x27d4e9b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f4e9b4, SUC_lexicon_tags_27 },
    { 0x1554e9b5, SUC_lexicon_tags_7 },
    { 0xa814e9b6, SUC_lexicon_tags_27 },
    { 0x9ad4e9b7, SUC_lexicon_tags_47 },
    { 0xb954e9b8, SUC_lexicon_tags_1 },
    { 0x8574e9b9, SUC_lexicon_tags_431 },
    { 0xe354e9b5, SUC_lexicon_tags_14 },
    { 0x3ff4e9b6, SUC_lexicon_tags_19 },
    { 0x5e34e9bb, SUC_lexicon_tags_12 },
    { 0xf354e9b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe054e9c0, SUC_lexicon_tags_25 },
    { 0x80d4e9c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d4e9c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d4e9c9, SUC_lexicon_tags_38 },
    { 0xa994e9ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe954e9d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d4e9d5, SUC_lexicon_tags_15 },
    { 0x8c74e9d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1154e9d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1934e9da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9814e9e2, SUC_lexicon_tags_7 },
    { 0x1eb4e9e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf4e9e6, SUC_lexicon_tags_60 },
    { 0xb874e9e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf434e9e9, SUC_lexicon_tags_65 },
    { 0x274e9ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d4e9ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b4e9f2, SUC_lexicon_tags_39 },
    { 0x66b4e9f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b4e9f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4e9fc, SUC_lexicon_tags_29 },
    { 0xaf34e9fd, SUC_lexicon_tags_8 },
    { 0xd94e9fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb414ea03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd4ea11, SUC_lexicon_tags_38 },
    { 0x90f4ea12, SUC_lexicon_tags_38 },
    { 0x4154ea13, SUC_lexicon_tags_29 },
    { 0xf194ea14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9874ea16, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3134ea1c, SUC_lexicon_tags_11 },
    { 0x5a34ea1d, SUC_lexicon_tags_37 },
    { 0x7474ea1d, SUC_lexicon_tags_37 },
    { 0x16f4ea1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf4ea21, SUC_lexicon_tags_3 },
    { 0xd1b4ea21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3214ea25, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b4ea28, SUC_lexicon_tags_14 },
    { 0xc0d4ea29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74ea2b, SUC_lexicon_tags_2 },
    { 0x5814ea2c, SUC_lexicon_tags_7 },
    { 0x7294ea2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb274ea2f, SUC_lexicon_tags_17 },
    { 0x1774ea30, SUC_lexicon_tags_26 },
    { 0xaf34ea31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4734ea35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4394ea3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d74ea40, SUC_lexicon_tags_38 },
    { 0x5e74ea41, SUC_lexicon_tags_9 },
    { 0xbd4ea40, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b4ea49, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab4ea4f, SUC_lexicon_tags_28 },
    { 0x8794ea4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xed94ea52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1434ea54, SUC_lexicon_tags_7 },
    { 0xc994ea55, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda34ea5a, SUC_lexicon_tags_30 },
    { 0x78d4ea5b, SUC_lexicon_tags_33 },
    { 0xd034ea5c, SUC_lexicon_tags_10 },
    { 0x8654ea5c, SUC_lexicon_tags_66 },
    { 0xe314ea5d, SUC_lexicon_tags_61 },
    { 0x5f14ea5f, SUC_lexicon_tags_47 },
    { 0x7c74ea5e, SUC_lexicon_tags_29 },
    { 0xeb14ea5d, SUC_lexicon_tags_7 },
    { 0x27f4ea62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c74ea64, SUC_lexicon_tags_16 },
    { 0xc3b4ea65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa14ea68, SUC_lexicon_tags_124 },
    { 0x1614ea69, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef4ea70, SUC_lexicon_tags_17 },
    { 0xa874ea71, SUC_lexicon_tags_25 },
    { 0x9cd4ea72, SUC_lexicon_tags_25 },
    { 0xdb94ea72, SUC_lexicon_tags_12 },
    { 0x7354ea72, SUC_lexicon_tags_0 },
    { 0x71b4ea75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf74ea79, SUC_lexicon_tags_68 },
    { 0xc814ea7a, SUC_lexicon_tags_29 },
    { 0x29f4ea79, SUC_lexicon_tags_11 },
    { 0x2c14ea7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5eb4ea7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3554ea80, SUC_lexicon_tags_27 },
    { 0xa974ea81, SUC_lexicon_tags_45 },
    { 0x39b4ea80, SUC_lexicon_tags_0 },
    { 0x8a74ea80, SUC_lexicon_tags_26 },
    { 0x99b4ea84, SUC_lexicon_tags_6 },
    { 0xe5f4ea85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb674ea8a, SUC_lexicon_tags_172 },
    { 0xbd94ea8a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1e54ea8d, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xee94ea8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd34ea91, SUC_lexicon_tags_65 },
    { 0x9b4ea91, SUC_lexicon_tags_25 },
    { 0x4e54ea93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd94ea96, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8e74ea98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb4ea9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1b4ea9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f34eaa4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f4eaa7, SUC_lexicon_tags_63 },
    { 0xb174eaa8, SUC_lexicon_tags_47 },
    { 0x894eaa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5714eaad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd874eab0, SUC_lexicon_tags_37 },
    { 0x4654eab1, SUC_lexicon_tags_14 },
    { 0xa094eab1, SUC_lexicon_tags_23 },
    { 0x2f4eab3, SUC_lexicon_tags_12 },
    { 0x7c94eab3, SUC_lexicon_tags_62 },
    { 0x12d4eab5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff4eab8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb874eabc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf554eabf, SUC_lexicon_tags_46 },
    { 0x9194eac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e34eac3, SUC_lexicon_tags_0 },
    { 0xe0b4eac4, SUC_lexicon_tags_25 },
    { 0x1194eac4, SUC_lexicon_tags_48 },
    { 0xab94eac5, SUC_lexicon_tags_11 },
    { 0x1fb4eac6, SUC_lexicon_tags_91 },
    { 0x5514eac3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4354ead0, SUC_lexicon_tags_30 },
    { 0x99b4ead1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb14ead4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9b14ead6, SUC_lexicon_tags_4 },
    { 0xe334ead7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf714eadb, SUC_lexicon_tags_19 },
    { 0xb594eadc, SUC_lexicon_tags_23 },
    { 0x5cd4eadb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3414eae1, SUC_lexicon_tags_31 },
    { 0x8374eae2, SUC_lexicon_tags_37 },
    { 0x4f94eae2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1854eae6, SUC_lexicon_tags_12 },
    { 0xd154eae7, SUC_lexicon_tags_29 },
    { 0x6834eae7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8214eaec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1274eaf0, SUC_lexicon_tags_17 },
    { 0xd7b4eaf1, SUC_lexicon_tags_68 },
    { 0xecb4eaf0, SUC_lexicon_tags_38 },
    { 0xe514eaf3, SUC_lexicon_tags_26 },
    { 0x7a74eaf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x554eaf8, SUC_lexicon_tags_14 },
    { 0x6f54eaf9, SUC_lexicon_tags_37 },
    { 0x95b4eafa, SUC_lexicon_tags_47 },
    { 0xbe74eaf8, SUC_lexicon_tags_12 },
    { 0x22b4eafc, SUC_lexicon_tags_7 },
    { 0xccf4eafd, SUC_lexicon_tags_37 },
    { 0xf454eafe, SUC_lexicon_tags_1 },
    { 0xb534eaff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd914eb04, SUC_lexicon_tags_1 },
    { 0xe094eb05, SUC_lexicon_tags_7 },
    { 0xca34eb06, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af4eb09, SUC_lexicon_tags_37 },
    { 0x98f4eb0a, SUC_lexicon_tags_45 },
    { 0xed34eb0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x894eb10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb4eb13, SUC_lexicon_tags_207 },
    { 0x8654eb14, SUC_lexicon_tags_49 },
    { 0xb4b4eb15, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f4eb19, SUC_lexicon_tags_8 },
    { 0xea94eb1a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1614eb1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d4eb21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4594eb25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2d14eb27, SUC_lexicon_tags_60 },
    { 0x2a94eb28, SUC_lexicon_tags_19 },
    { 0xc2d4eb29, SUC_lexicon_tags_1 },
    { 0x4f74eb29, SUC_lexicon_tags_12 },
    { 0xdeb4eb2b, SUC_lexicon_tags_37 },
    { 0x1af4eb2c, SUC_lexicon_tags_12 },
    { 0x92d4eb2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b4eb31, SUC_lexicon_tags_47 },
    { 0x8594eb32, SUC_lexicon_tags_29 },
    { 0x4f94eb32, SUC_lexicon_tags_43 },
    { 0x81d4eb32, SUC_lexicon_tags_26 },
    { 0xad34eb32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f4eb3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f94eb3f, SUC_lexicon_tags_95 },
    { 0x5c54eb3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e94eb44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4274eb48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53d4eb4a, SUC_lexicon_tags_17 },
    { 0x3fd4eb4b, SUC_lexicon_tags_103 },
    { 0xc174eb4c, SUC_lexicon_tags_15 },
    { 0xe734eb4a, SUC_lexicon_tags_25 },
    { 0x2d94eb4e, SUC_lexicon_tags_45 },
    { 0xd194eb4c, SUC_lexicon_tags_11 },
    { 0x6994eb4c, SUC_lexicon_tags_63 },
    { 0x29f4eb51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8234eb53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38b4eb55, SUC_lexicon_tags_25 },
    { 0x99f4eb56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xddd4eb58, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f94eb5c, SUC_lexicon_tags_1 },
    { 0x9d34eb5d, SUC_lexicon_tags_25 },
    { 0x5494eb5d, SUC_lexicon_tags_17 },
    { 0xf8f4eb5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b4eb62, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8734eb69, SUC_lexicon_tags_17 },
    { 0xf9d4eb6a, SUC_lexicon_tags_15 },
    { 0x4914eb6a, SUC_lexicon_tags_45 },
    { 0xa0d4eb6b, SUC_lexicon_tags_66 },
    { 0xae74eb6d, SUC_lexicon_tags_61 },
    { 0x4054eb6e, SUC_lexicon_tags_28 },
    { 0x7634eb6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b4eb72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d4eb75, SUC_lexicon_tags_2 },
    { 0x3974eb76, SUC_lexicon_tags_12 },
    { 0xe534eb76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf14eb79, SUC_lexicon_tags_25 },
    { 0xfed4eb7a, SUC_lexicon_tags_60 },
    { 0x13f4eb7b, SUC_lexicon_tags_8 },
    { 0xee14eb7c, SUC_lexicon_tags_1 },
    { 0x5494eb7d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc934eb7f, SUC_lexicon_tags_19 },
    { 0x1b74eb7f, SUC_lexicon_tags_25 },
    { 0x8494eb81, SUC_lexicon_tags_7 },
    { 0x69b4eb82, SUC_lexicon_tags_60 },
    { 0x7714eb83, SUC_lexicon_tags_43 },
    { 0xec14eb84, SUC_lexicon_tags_65 },
    { 0xdbb4eb85, SUC_lexicon_tags_8 },
    { 0xf614eb85, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe934eb88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2d4eb8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2634eb8c, SUC_lexicon_tags_39 },
    { 0x2314eb8c, SUC_lexicon_tags_7 },
    { 0xecf4eb8e, SUC_lexicon_tags_2 },
    { 0x1714eb8f, SUC_lexicon_tags_32 },
    { 0x5d54eb8e, SUC_lexicon_tags_37 },
    { 0xacf4eb91, SUC_lexicon_tags_15 },
    { 0xdf14eb90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f74eb96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa514eb98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15f4eb9a, SUC_lexicon_tags_1 },
    { 0x7d74eb9a, SUC_lexicon_tags_23 },
    { 0x6e14eb9c, SUC_lexicon_tags_1 },
    { 0xd074eb9d, SUC_lexicon_tags_60 },
    { 0xb7b4eb9e, SUC_lexicon_tags_15 },
    { 0x4374eb9f, SUC_lexicon_tags_2 },
    { 0x5df4eb9f, SUC_lexicon_tags_15 },
    { 0x40f4eba0, SUC_lexicon_tags_6 },
    { 0x59d4eba0, SUC_lexicon_tags_7 },
    { 0xa7d4eba3, SUC_lexicon_tags_17 },
    { 0x3614eba4, SUC_lexicon_tags_14 },
    { 0xbc14eb9c, SUC_lexicon_tags_63 },
    { 0xbd94eba6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f4ebab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1174ebad, SUC_lexicon_tags_25 },
    { 0xc3f4ebae, SUC_lexicon_tags_26 },
    { 0x1a14ebaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x494ebb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67f4ebb3, SUC_lexicon_tags_48 },
    { 0x2b94ebb4, SUC_lexicon_tags_17 },
    { 0xd594ebb3, SUC_lexicon_tags_8 },
    { 0x11b4ebb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde34ebbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce94ebbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b4ebc2, SUC_lexicon_tags_37 },
    { 0xb6f4ebc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd4ebc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b4ebcb, SUC_lexicon_tags_33 },
    { 0xa614ebcc, SUC_lexicon_tags_27 },
    { 0xe5f4ebcc, SUC_lexicon_tags_49 },
    { 0xe954ebcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xac14ebd1, SUC_lexicon_tags_27 },
    { 0x1614ebd2, SUC_lexicon_tags_17 },
    { 0x3454ebd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21f4ebd5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7874ebda, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe894ebdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f4ebe1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf934ebe3, SUC_lexicon_tags_28 },
    { 0xd574ebe3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe534ebe7, SUC_lexicon_tags_1 },
    { 0xd674ebe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f4ebf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f4ebfb, SUC_lexicon_tags_60 },
    { 0xf694ebfc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb394ec00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f14ec03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe14ec05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb74ec07, SUC_lexicon_tags_37 },
    { 0x8b14ec08, SUC_lexicon_tags_1 },
    { 0x19f4ec09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4954ec0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x114ec0f, SUC_lexicon_tags_0 },
    { 0xba74ec10, SUC_lexicon_tags_8 },
    { 0x6af4ec11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdd4ec13, SUC_lexicon_tags_17 },
    { 0x4334ec14, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6834ec1a, SUC_lexicon_tags_15 },
    { 0x86f4ec1b, SUC_lexicon_tags_31 },
    { 0xcc34ec1a, SUC_lexicon_tags_12 },
    { 0xfab4ec1b, SUC_lexicon_tags_37 },
    { 0x8774ec1e, SUC_lexicon_tags_21 },
    { 0x7a54ec1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd4ec25, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa494ec2f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1d74ec31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf34ec34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8834ec3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x254ec3c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd4ec3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f74ec41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd654ec45, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d4ec49, SUC_lexicon_tags_44 },
    { 0x1bb4ec4a, SUC_lexicon_tags_8 },
    { 0x54b4ec4b, SUC_lexicon_tags_63 },
    { 0x2f4ec4a, SUC_lexicon_tags_60 },
    { 0x7d14ec4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f34ec52, SUC_lexicon_tags_57 },
    { 0x7414ec53, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xafd4ec55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe94ec58, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc774ec5c, SUC_lexicon_tags_3 },
    { 0xc034ec5c, SUC_lexicon_tags_39 },
    { 0x8f4ec5d, SUC_lexicon_tags_23 },
    { 0x49f4ec5c, SUC_lexicon_tags_17 },
    { 0x40f4ec60, SUC_lexicon_tags_14 },
    { 0xd1f4ec5e, SUC_lexicon_tags_15 },
    { 0x2574ec5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d4ec68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d94ec71, SUC_lexicon_tags_11 },
    { 0xfb4ec71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6474ec75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f4ec78, SUC_lexicon_tags_39 },
    { 0x2334ec79, SUC_lexicon_tags_13 },
    { 0x8d34ec79, SUC_lexicon_tags_12 },
    { 0xea14ec78, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb94ec7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x674ec80, SUC_lexicon_tags_9 },
    { 0xb774ec81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63d4ec83, SUC_lexicon_tags_57 },
    { 0x2dd4ec84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe454ec86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf4ec8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ef4ec8f, SUC_lexicon_tags_11 },
    { 0xced4ec90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf354ec93, SUC_lexicon_tags_37 },
    { 0xd294ec94, SUC_lexicon_tags_56 },
    { 0xd3b4ec94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc734ec98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb4ec9d, SUC_lexicon_tags_2 },
    { 0xfc54ec9e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a34eca1, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c54eca6, SUC_lexicon_tags_37 },
    { 0xb234eca6, SUC_lexicon_tags_25 },
    { 0xc6f4eca7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd74ecab, SUC_lexicon_tags_25 },
    { 0xb3d4ecac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b4ecaf, SUC_lexicon_tags_14 },
    { 0x49d4ecb0, SUC_lexicon_tags_21 },
    { 0xb6d4ecaf, SUC_lexicon_tags_21 },
    { 0xcaf4ecb0, SUC_lexicon_tags_21 },
    { 0x21f4ecb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab14ecb5, SUC_lexicon_tags_28 },
    { 0xe434ecb6, SUC_lexicon_tags_12 },
    { 0xb0d4ecb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40d4ecb9, SUC_lexicon_tags_8 },
    { 0xe3d4ecba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d4ecbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf654ecbf, SUC_lexicon_tags_27 },
    { 0x4d74ecc0, SUC_lexicon_tags_38 },
    { 0x4534ecc0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf4ecc5, SUC_lexicon_tags_15 },
    { 0x4234ecc6, SUC_lexicon_tags_11 },
    { 0xb414ecc6, SUC_lexicon_tags_4 },
    { 0xeb94ecc8, SUC_lexicon_tags_37 },
    { 0x3474ecc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20f4eccb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4ecd2, SUC_lexicon_tags_19 },
    { 0xce14ecd3, SUC_lexicon_tags_1 },
    { 0x9074ecd3, SUC_lexicon_tags_82 },
    { 0x994ecd2, SUC_lexicon_tags_14 },
    { 0x98f4ecd6, SUC_lexicon_tags_12 },
    { 0x3974ecd7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8194ecdc, SUC_lexicon_tags_1 },
    { 0x8f14ecdd, SUC_lexicon_tags_34 },
    { 0x3974ecdd, SUC_lexicon_tags_26 },
    { 0x9214ecdf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3774ece1, SUC_lexicon_tags_2 },
    { 0x8fd4ece1, SUC_lexicon_tags_21 },
    { 0x1cb4ece3, SUC_lexicon_tags_38 },
    { 0x2374ece1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c54ece7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb214ecf0, SUC_lexicon_tags_25 },
    { 0xd9f4ecf0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f4ecf4, SUC_lexicon_tags_38 },
    { 0x154ecf5, SUC_lexicon_tags_68 },
    { 0xf5d4ecf6, SUC_lexicon_tags_43 },
    { 0x7a74ecf7, SUC_lexicon_tags_16 },
    { 0xcbd4ecf8, SUC_lexicon_tags_12 },
    { 0xd174ecf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f4ecfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe474ed03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef4ed07, SUC_lexicon_tags_15 },
    { 0x694ed07, SUC_lexicon_tags_28 },
    { 0x6514ed07, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d4ed0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2f74ed0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb94ed11, SUC_lexicon_tags_1 },
    { 0x7cf4ed11, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5874ed14, SUC_lexicon_tags_31 },
    { 0x6a34ed14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6974ed17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd254ed19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc74ed1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa074ed23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f4ed26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54ed2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f4ed2f, SUC_lexicon_tags_4 },
    { 0x7894ed2f, SUC_lexicon_tags_11 },
    { 0x3a54ed31, SUC_lexicon_tags_2 },
    { 0xc54ed2f, SUC_lexicon_tags_9 },
    { 0x5db4ed31, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f4ed36, SUC_lexicon_tags_37 },
    { 0x4bd4ed37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c34ed39, SUC_lexicon_tags_17 },
    { 0xf4ed3a, SUC_lexicon_tags_21 },
    { 0xa34ed3b, SUC_lexicon_tags_11 },
    { 0xe974ed3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c34ed41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce34ed43, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1654ed4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3ef4ed4d, SUC_lexicon_tags_14 },
    { 0x4954ed4e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3d74ed50, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c14ed55, SUC_lexicon_tags_8 },
    { 0xa774ed55, SUC_lexicon_tags_7 },
    { 0xb7b4ed56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9314ed59, SUC_lexicon_tags_17 },
    { 0x1054ed59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf994ed5d, SUC_lexicon_tags_29 },
    { 0x15f4ed5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x54f4ed60, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa814ed62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf034ed64, SUC_lexicon_tags_25 },
    { 0x8914ed64, SUC_lexicon_tags_37 },
    { 0x48b4ed64, SUC_lexicon_tags_1 },
    { 0x9ed4ed67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6374ed6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d4ed6f, SUC_lexicon_tags_11 },
    { 0x8c94ed6f, SUC_lexicon_tags_7 },
    { 0xca14ed6f, SUC_lexicon_tags_70 },
    { 0x74ed72, SUC_lexicon_tags_15 },
    { 0x55b4ed73, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac14ed78, SUC_lexicon_tags_25 },
    { 0x8974ed78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa634ed7d, SUC_lexicon_tags_56 },
    { 0x3b94ed7e, SUC_lexicon_tags_63 },
    { 0xef14ed7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f4ed84, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe34ed86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a14ed8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d4ed8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d4ed94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8734ed97, SUC_lexicon_tags_1 },
    { 0xc6d4ed98, SUC_lexicon_tags_1 },
    { 0x5a94ed98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd14ed9d, SUC_lexicon_tags_1 },
    { 0x3df4ed9d, SUC_lexicon_tags_17 },
    { 0x9ff4ed9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7314eda2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f4eda4, SUC_lexicon_tags_37 },
    { 0x814eda5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb94edaa, SUC_lexicon_tags_17 },
    { 0x7354edab, SUC_lexicon_tags_12 },
    { 0x3034edac, SUC_lexicon_tags_37 },
    { 0xfc34edad, SUC_lexicon_tags_15 },
    { 0xd1b4edab, SUC_lexicon_tags_37 },
    { 0xff14edaf, SUC_lexicon_tags_61 },
    { 0x4494edac, SUC_lexicon_tags_45 },
    { 0xee34edae, SUC_lexicon_tags_1 },
    { 0x3fb4edb2, SUC_lexicon_tags_32 },
    { 0x18d4edb3, SUC_lexicon_tags_65 },
    { 0x3fd4edb2, SUC_lexicon_tags_1 },
    { 0xa0d4edb3, SUC_lexicon_tags_1 },
    { 0x54edb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf534edbb, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d94edbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x634edc2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf574edc5, SUC_lexicon_tags_1 },
    { 0xc414edc6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb4edcc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0b4edce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d4edd4, SUC_lexicon_tags_0 },
    { 0x29d4edd4, SUC_lexicon_tags_65 },
    { 0x71b4edd6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd794edd8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6714eddd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe854ede0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3634ede2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd354ede6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4ede9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84b4edeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f4edf2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf4edf8, SUC_lexicon_tags_2 },
    { 0x38f4edf8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1f54edfb, SUC_lexicon_tags_11 },
    { 0xca54edfc, SUC_lexicon_tags_55 },
    { 0x93f4edfd, SUC_lexicon_tags_28 },
    { 0xc74edfd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d94ee03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd334ee0b, SUC_lexicon_tags_39 },
    { 0xac54ee0c, SUC_lexicon_tags_25 },
    { 0xd934ee0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb4ee12, SUC_lexicon_tags_47 },
    { 0x15b4ee13, SUC_lexicon_tags_11 },
    { 0xd8f4ee12, SUC_lexicon_tags_2 },
    { 0xc934ee12, SUC_lexicon_tags_401 },
    { 0, NULL },
    { 0, NULL },
    { 0xf834ee18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9554ee1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f34ee1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4394ee26, SUC_lexicon_tags_17 },
    { 0xc2f4ee26, SUC_lexicon_tags_31 },
    { 0xdb14ee28, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x16d4ee2a, SUC_lexicon_tags_37 },
    { 0xf5b4ee2a, SUC_lexicon_tags_39 },
    { 0xf414ee2b, SUC_lexicon_tags_43 },
    { 0x4bf4ee2d, SUC_lexicon_tags_49 },
    { 0x7af4ee2a, SUC_lexicon_tags_27 },
    { 0x78d4ee2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1294ee32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd494ee37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2374ee3b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b4ee3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c94ee41, SUC_lexicon_tags_38 },
    { 0x66d4ee42, SUC_lexicon_tags_49 },
    { 0xd1b4ee43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f4ee46, SUC_lexicon_tags_7 },
    { 0x8d54ee47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe7b4ee49, SUC_lexicon_tags_49 },
    { 0xf5f4ee4a, SUC_lexicon_tags_19 },
    { 0x76d4ee4b, SUC_lexicon_tags_5 },
    { 0x1b94ee4b, SUC_lexicon_tags_84 },
    { 0x7454ee49, SUC_lexicon_tags_39 },
    { 0x8654ee4b, SUC_lexicon_tags_0 },
    { 0x9cf4ee4f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc74ee54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9434ee57, SUC_lexicon_tags_26 },
    { 0x6b14ee57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd4ee5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd094ee60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3f4ee62, SUC_lexicon_tags_49 },
    { 0x5c14ee62, SUC_lexicon_tags_45 },
    { 0x7134ee64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa034ee66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb4ee6a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb4ee6d, SUC_lexicon_tags_0 },
    { 0xab34ee6d, SUC_lexicon_tags_84 },
    { 0x3994ee6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4334ee74, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df4ee77, SUC_lexicon_tags_7 },
    { 0x5934ee78, SUC_lexicon_tags_2 },
    { 0xa9b4ee78, SUC_lexicon_tags_4 },
    { 0xbef4ee79, SUC_lexicon_tags_6 },
    { 0x7034ee7b, SUC_lexicon_tags_33 },
    { 0x74f4ee7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd74ee7e, SUC_lexicon_tags_19 },
    { 0xa9d4ee7e, SUC_lexicon_tags_15 },
    { 0x5794ee80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d4ee84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f4ee87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd4ee8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd4ee8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad4ee91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe214ee93, SUC_lexicon_tags_29 },
    { 0xeeb4ee94, SUC_lexicon_tags_17 },
    { 0x45f4ee93, SUC_lexicon_tags_37 },
    { 0x6214ee96, SUC_lexicon_tags_8 },
    { 0x3ab4ee97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb074ee99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d34ee9b, SUC_lexicon_tags_29 },
    { 0xaf4ee9c, SUC_lexicon_tags_27 },
    { 0x6774ee9d, SUC_lexicon_tags_27 },
    { 0xeff4ee9b, SUC_lexicon_tags_47 },
    { 0x8234ee9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1914eea4, SUC_lexicon_tags_37 },
    { 0x3534eea4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df4eea9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb54eeac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbeb4eeae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb54eeb0, SUC_lexicon_tags_11 },
    { 0x8fd4eeb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c94eeb8, SUC_lexicon_tags_17 },
    { 0x9d14eeb8, SUC_lexicon_tags_46 },
    { 0xb494eeb9, SUC_lexicon_tags_17 },
    { 0x2b4eebb, SUC_lexicon_tags_25 },
    { 0x48d4eebc, SUC_lexicon_tags_37 },
    { 0x9d14eebc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b4eec1, SUC_lexicon_tags_0 },
    { 0x1c34eec2, SUC_lexicon_tags_27 },
    { 0x4334eec2, SUC_lexicon_tags_14 },
    { 0xde54eec4, SUC_lexicon_tags_61 },
    { 0xba54eec5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d4eec9, SUC_lexicon_tags_9 },
    { 0xedd4eeca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b4eecd, SUC_lexicon_tags_61 },
    { 0x7794eece, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d74eed1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc174eed8, SUC_lexicon_tags_17 },
    { 0x7a94eed9, SUC_lexicon_tags_14 },
    { 0x2274eeda, SUC_lexicon_tags_60 },
    { 0x7d74eedb, SUC_lexicon_tags_33 },
    { 0xbb54eedc, SUC_lexicon_tags_13 },
    { 0xb4d4eedb, SUC_lexicon_tags_41 },
    { 0x9a34eedd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7d94eee0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe614eee4, SUC_lexicon_tags_6 },
    { 0x5674eee5, SUC_lexicon_tags_15 },
    { 0xc554eee6, SUC_lexicon_tags_0 },
    { 0xeb4eee7, SUC_lexicon_tags_15 },
    { 0xf754eee8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc474eeea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb94eeed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf34eef4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa614eef9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4154eefd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x114ef00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b4ef04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca34ef09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b4ef0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb74ef0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4754ef11, SUC_lexicon_tags_121 },
    { 0x5334ef12, SUC_lexicon_tags_37 },
    { 0x5dd4ef13, SUC_lexicon_tags_58 },
    { 0x9034ef13, SUC_lexicon_tags_1 },
    { 0xb8b4ef15, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c54ef1a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa194ef1c, SUC_lexicon_tags_17 },
    { 0x1b4ef1d, SUC_lexicon_tags_1 },
    { 0x1074ef1c, SUC_lexicon_tags_103 },
    { 0x6794ef1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd74ef23, SUC_lexicon_tags_37 },
    { 0x65b4ef24, SUC_lexicon_tags_25 },
    { 0x68f4ef25, SUC_lexicon_tags_39 },
    { 0x5b74ef26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf094ef2b, SUC_lexicon_tags_26 },
    { 0x49f4ef2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1934ef2e, SUC_lexicon_tags_30 },
    { 0x4574ef2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7654ef31, SUC_lexicon_tags_34 },
    { 0x7954ef31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90d4ef34, SUC_lexicon_tags_63 },
    { 0x5ff4ef34, SUC_lexicon_tags_2 },
    { 0x8e14ef36, SUC_lexicon_tags_28 },
    { 0x9f74ef37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6714ef39, SUC_lexicon_tags_29 },
    { 0xda14ef3a, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b4ef3d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3f94ef3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc894ef42, SUC_lexicon_tags_11 },
    { 0x8734ef43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9674ef48, SUC_lexicon_tags_11 },
    { 0xb0b4ef49, SUC_lexicon_tags_9 },
    { 0xed4ef4a, SUC_lexicon_tags_27 },
    { 0x33f4ef4b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3794ef4e, SUC_lexicon_tags_11 },
    { 0x31f4ef4f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4b94ef51, SUC_lexicon_tags_15 },
    { 0xbfd4ef51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b4ef55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab4ef58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xde14ef5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d4ef5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2b4ef5f, SUC_lexicon_tags_65 },
    { 0x5b34ef60, SUC_lexicon_tags_2 },
    { 0xff54ef61, SUC_lexicon_tags_9 },
    { 0x3f34ef61, SUC_lexicon_tags_30 },
    { 0xdb34ef60, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x87d4ef65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f14ef68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab4ef70, SUC_lexicon_tags_12 },
    { 0xbcd4ef71, SUC_lexicon_tags_17 },
    { 0xddf4ef72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2d4ef74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3854ef76, SUC_lexicon_tags_37 },
    { 0xf774ef76, SUC_lexicon_tags_6 },
    { 0xaf74ef78, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d4ef7d, SUC_lexicon_tags_27 },
    { 0xfe94ef7e, SUC_lexicon_tags_47 },
    { 0x914ef7f, SUC_lexicon_tags_63 },
    { 0xe314ef80, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xb134ef82, SUC_lexicon_tags_55 },
    { 0x2234ef83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf974ef85, SUC_lexicon_tags_28 },
    { 0x8174ef86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe074ef8f, SUC_lexicon_tags_28 },
    { 0x4554ef8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1994ef92, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f4ef99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b54ef9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc94efa2, SUC_lexicon_tags_2 },
    { 0x4b14efa3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1034efa6, SUC_lexicon_tags_43 },
    { 0xc154efa7, SUC_lexicon_tags_19 },
    { 0x4e34efa8, SUC_lexicon_tags_85 },
    { 0x3434efa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d4efae, SUC_lexicon_tags_47 },
    { 0xaff4efae, SUC_lexicon_tags_6 },
    { 0x8d74efae, SUC_lexicon_tags_91 },
    { 0x88f4efb1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d4efb4, SUC_lexicon_tags_110 },
    { 0xa34efb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd4efbb, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0xc534efbe, SUC_lexicon_tags_12 },
    { 0x1ed4efbe, SUC_lexicon_tags_7 },
    { 0x4d74efc0, SUC_lexicon_tags_85 },
    { 0xab74efc0, SUC_lexicon_tags_14 },
    { 0xd074efc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f4efc5, SUC_lexicon_tags_61 },
    { 0xce74efc6, SUC_lexicon_tags_66 },
    { 0xb6f4efc7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f4efd0, SUC_lexicon_tags_37 },
    { 0xc654efd0, SUC_lexicon_tags_2 },
    { 0x1494efd2, SUC_lexicon_tags_5 },
    { 0x43f4efd3, SUC_lexicon_tags_91 },
    { 0x1ff4efd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf94efd6, SUC_lexicon_tags_32 },
    { 0x8254efd6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f4efda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf074efdc, SUC_lexicon_tags_56 },
    { 0xdeb4efdc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1934efe6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d74efe9, SUC_lexicon_tags_63 },
    { 0xf334efe9, SUC_lexicon_tags_11 },
    { 0x96b4efeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4fb4efed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaff4efef, SUC_lexicon_tags_1 },
    { 0x5294eff0, SUC_lexicon_tags_1 },
    { 0xc374eff1, SUC_lexicon_tags_107 },
    { 0xab54eff2, SUC_lexicon_tags_17 },
    { 0xd134eff3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2434eff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe154eff8, SUC_lexicon_tags_1 },
    { 0x9c94eff8, SUC_lexicon_tags_61 },
    { 0xbd94effa, SUC_lexicon_tags_11 },
    { 0xcbb4eff8, SUC_lexicon_tags_12 },
    { 0x9214effc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x87b4effe, SUC_lexicon_tags_11 },
    { 0xb3b4efff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa54f001, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd914f003, SUC_lexicon_tags_38 },
    { 0xa974f004, SUC_lexicon_tags_11 },
    { 0xd894f004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1cb4f007, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b4f00a, SUC_lexicon_tags_60 },
    { 0x8c34f00b, SUC_lexicon_tags_12 },
    { 0x41b4f00c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd4f00f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6514f011, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4354f016, SUC_lexicon_tags_47 },
    { 0x9094f016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd014f01b, SUC_lexicon_tags_6 },
    { 0x2554f01c, SUC_lexicon_tags_45 },
    { 0x4a34f01c, SUC_lexicon_tags_7 },
    { 0xe2f4f01d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2534f021, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1054f024, SUC_lexicon_tags_13 },
    { 0xab14f024, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf4f028, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5074f02a, SUC_lexicon_tags_19 },
    { 0xf774f02a, SUC_lexicon_tags_64 },
    { 0xed54f02a, SUC_lexicon_tags_27 },
    { 0x49f4f02d, SUC_lexicon_tags_49 },
    { 0x25f4f02d, SUC_lexicon_tags_7 },
    { 0x48f4f02f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd4f034, SUC_lexicon_tags_12 },
    { 0xf474f035, SUC_lexicon_tags_33 },
    { 0x5eb4f034, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f4f039, SUC_lexicon_tags_11 },
    { 0x894f03a, SUC_lexicon_tags_8 },
    { 0xd554f03b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1434f041, SUC_lexicon_tags_47 },
    { 0x9c54f042, SUC_lexicon_tags_11 },
    { 0x7bd4f043, SUC_lexicon_tags_39 },
    { 0xb5d4f044, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf694f046, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ff4f048, SUC_lexicon_tags_23 },
    { 0xfc54f049, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf14f04c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2314f04f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f4f052, SUC_lexicon_tags_25 },
    { 0x4794f052, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x28d4f055, SUC_lexicon_tags_59 },
    { 0xe9b4f055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe574f05a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5174f062, SUC_lexicon_tags_37 },
    { 0xdd14f063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb14f065, SUC_lexicon_tags_12 },
    { 0x3cb4f065, SUC_lexicon_tags_37 },
    { 0xf6d4f067, SUC_lexicon_tags_11 },
    { 0xfed4f068, SUC_lexicon_tags_23 },
    { 0xf2d4f069, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7514f070, SUC_lexicon_tags_7 },
    { 0x7a74f070, SUC_lexicon_tags_17 },
    { 0xf334f072, SUC_lexicon_tags_30 },
    { 0x71b4f073, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe954f07b, SUC_lexicon_tags_30 },
    { 0x5954f07c, SUC_lexicon_tags_17 },
    { 0xb14f07d, SUC_lexicon_tags_37 },
    { 0x99b4f07d, SUC_lexicon_tags_11 },
    { 0x5774f07f, SUC_lexicon_tags_150 },
    { 0x49b4f07f, SUC_lexicon_tags_28 },
    { 0x1374f081, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2a54f083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf34f085, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a94f087, SUC_lexicon_tags_2 },
    { 0xd34f088, SUC_lexicon_tags_39 },
    { 0x6e34f088, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab74f08c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b4f090, SUC_lexicon_tags_61 },
    { 0xf194f091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad54f093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb4f097, SUC_lexicon_tags_63 },
    { 0x1594f098, SUC_lexicon_tags_29 },
    { 0x6fb4f099, SUC_lexicon_tags_27 },
    { 0x78d4f099, SUC_lexicon_tags_5 },
    { 0x6e74f09b, SUC_lexicon_tags_6 },
    { 0x6d94f09a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5514f0a1, SUC_lexicon_tags_48 },
    { 0x6a34f0a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d4f0a8, SUC_lexicon_tags_46 },
    { 0x1d34f0a9, SUC_lexicon_tags_37 },
    { 0x6b74f0aa, SUC_lexicon_tags_14 },
    { 0x55f4f0a8, SUC_lexicon_tags_37 },
    { 0x1154f0aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1274f0b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf474f0b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc094f0b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4f0c0, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x514f0c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb14f0c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2774f0c8, SUC_lexicon_tags_2 },
    { 0x7554f0c8, SUC_lexicon_tags_31 },
    { 0xed14f0ca, SUC_lexicon_tags_63 },
    { 0x7af4f0cb, SUC_lexicon_tags_7 },
    { 0x3554f0cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d4f0d1, SUC_lexicon_tags_9 },
    { 0xa054f0d2, SUC_lexicon_tags_26 },
    { 0x22f4f0d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d4f0d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f4f0e0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d4f0e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd4f0e7, SUC_lexicon_tags_27 },
    { 0x6bb4f0e8, SUC_lexicon_tags_38 },
    { 0xb294f0e8, SUC_lexicon_tags_29 },
    { 0x7374f0e8, SUC_lexicon_tags_27 },
    { 0x4894f0e9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe554f0ee, SUC_lexicon_tags_27 },
    { 0xe874f0ee, SUC_lexicon_tags_8 },
    { 0xea54f0ef, SUC_lexicon_tags_8 },
    { 0x5a54f0f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cb4f0f3, SUC_lexicon_tags_298 },
    { 0xe9d4f0f3, SUC_lexicon_tags_0 },
    { 0x2394f0f5, SUC_lexicon_tags_12 },
    { 0x9934f0f4, SUC_lexicon_tags_37 },
    { 0x1f34f0f7, SUC_lexicon_tags_62 },
    { 0x4654f0f8, SUC_lexicon_tags_1 },
    { 0x1034f0f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4774f0fd, SUC_lexicon_tags_23 },
    { 0x2af4f0fd, SUC_lexicon_tags_2 },
    { 0x6a94f0ff, SUC_lexicon_tags_3 },
    { 0xd294f100, SUC_lexicon_tags_25 },
    { 0x1054f100, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d4f104, SUC_lexicon_tags_63 },
    { 0x654f105, SUC_lexicon_tags_25 },
    { 0xf834f106, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2754f108, SUC_lexicon_tags_12 },
    { 0xc34f109, SUC_lexicon_tags_14 },
    { 0x4174f10a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe14f10f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6134f112, SUC_lexicon_tags_33 },
    { 0xcdd4f112, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e34f118, SUC_lexicon_tags_53 },
    { 0xbed4f119, SUC_lexicon_tags_11 },
    { 0x1b74f11a, SUC_lexicon_tags_17 },
    { 0xc434f11b, SUC_lexicon_tags_0 },
    { 0xd4f11b, SUC_lexicon_tags_23 },
    { 0x6574f118, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6474f121, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xef94f124, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f4f12b, SUC_lexicon_tags_15 },
    { 0xe2f4f12c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xb414f12f, SUC_lexicon_tags_25 },
    { 0x6cd4f130, SUC_lexicon_tags_15 },
    { 0x9834f12f, SUC_lexicon_tags_2 },
    { 0xe9b4f132, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf4f13c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f4f140, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b4f145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9054f148, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8f74f14a, SUC_lexicon_tags_37 },
    { 0x80b4f14b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b4f14e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f4f151, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb494f153, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5814f155, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x314f157, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c74f15c, SUC_lexicon_tags_12 },
    { 0x71f4f15d, SUC_lexicon_tags_7 },
    { 0x24f4f15c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc554f161, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x81b4f163, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x8354f166, SUC_lexicon_tags_17 },
    { 0xfb34f167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa314f16a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee34f16d, SUC_lexicon_tags_49 },
    { 0xaa94f16e, SUC_lexicon_tags_11 },
    { 0x6254f16f, SUC_lexicon_tags_12 },
    { 0xde94f16f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf4f175, SUC_lexicon_tags_23 },
    { 0xde94f176, SUC_lexicon_tags_11 },
    { 0x87b4f176, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3414f17b, SUC_lexicon_tags_8 },
    { 0xab34f17b, SUC_lexicon_tags_28 },
    { 0x4bd4f17d, SUC_lexicon_tags_30 },
    { 0x7c74f17d, SUC_lexicon_tags_27 },
    { 0x8174f17d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd894f184, SUC_lexicon_tags_25 },
    { 0x3e94f185, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f4f18e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f4f191, SUC_lexicon_tags_7 },
    { 0x3754f192, SUC_lexicon_tags_17 },
    { 0x64f4f193, SUC_lexicon_tags_17 },
    { 0xe254f194, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7f4f196, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2254f198, SUC_lexicon_tags_45 },
    { 0xe0b4f198, SUC_lexicon_tags_23 },
    { 0xdfb4f198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4114f19d, SUC_lexicon_tags_37 },
    { 0x7874f19e, SUC_lexicon_tags_38 },
    { 0xb274f19f, SUC_lexicon_tags_23 },
    { 0x8354f1a0, SUC_lexicon_tags_63 },
    { 0x28b4f1a0, SUC_lexicon_tags_0 },
    { 0x9b14f1a2, SUC_lexicon_tags_37 },
    { 0x14f4f1a2, SUC_lexicon_tags_25 },
    { 0xf354f1a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c54f1aa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x594f1ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4f1ae, SUC_lexicon_tags_11 },
    { 0x6094f1af, SUC_lexicon_tags_26 },
    { 0x6874f1b0, SUC_lexicon_tags_30 },
    { 0x7df4f1b1, SUC_lexicon_tags_46 },
    { 0xf394f1b2, SUC_lexicon_tags_77 },
    { 0x49f4f1b3, SUC_lexicon_tags_17 },
    { 0x2d74f1b4, SUC_lexicon_tags_2 },
    { 0x97b4f1b5, SUC_lexicon_tags_60 },
    { 0xe7f4f1b5, SUC_lexicon_tags_9 },
    { 0x5a14f1b3, SUC_lexicon_tags_7 },
    { 0xbe34f1b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7334f1bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad4f1c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b4f1c4, SUC_lexicon_tags_25 },
    { 0x5a74f1c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b74f1c7, SUC_lexicon_tags_37 },
    { 0xaa74f1c8, SUC_lexicon_tags_43 },
    { 0xc6f4f1c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd4f1cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc154f1ce, SUC_lexicon_tags_30 },
    { 0x7974f1ce, SUC_lexicon_tags_2 },
    { 0x7174f1d0, SUC_lexicon_tags_49 },
    { 0xf7b4f1d1, SUC_lexicon_tags_43 },
    { 0x40f4f1ce, SUC_lexicon_tags_11 },
    { 0xc714f1d3, SUC_lexicon_tags_25 },
    { 0x8114f1d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b4f1d8, SUC_lexicon_tags_146 },
    { 0x7614f1d9, SUC_lexicon_tags_27 },
    { 0xaa14f1d8, SUC_lexicon_tags_12 },
    { 0x614f1db, SUC_lexicon_tags_37 },
    { 0x354f1db, SUC_lexicon_tags_37 },
    { 0x5274f1d9, SUC_lexicon_tags_4 },
    { 0xf9d4f1de, SUC_lexicon_tags_7 },
    { 0x5d74f1de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc294f1e1, SUC_lexicon_tags_15 },
    { 0x65b4f1e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b74f1e6, SUC_lexicon_tags_47 },
    { 0x7a14f1e6, SUC_lexicon_tags_11 },
    { 0xa294f1e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe494f1f2, SUC_lexicon_tags_7 },
    { 0xaef4f1f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xded4f1f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b4f1f8, SUC_lexicon_tags_65 },
    { 0x4e94f1f9, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x2814f1fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8694f1fe, SUC_lexicon_tags_60 },
    { 0xe234f1fe, SUC_lexicon_tags_57 },
    { 0x5934f200, SUC_lexicon_tags_12 },
    { 0xe3b4f201, SUC_lexicon_tags_12 },
    { 0xe294f202, SUC_lexicon_tags_10 },
    { 0xba74f1ff, SUC_lexicon_tags_27 },
    { 0xcef4f204, SUC_lexicon_tags_2 },
    { 0x2534f204, SUC_lexicon_tags_23 },
    { 0xab54f206, SUC_lexicon_tags_15 },
    { 0x7974f207, SUC_lexicon_tags_60 },
    { 0xb6b4f207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x914f20d, SUC_lexicon_tags_1 },
    { 0x77f4f20e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcdf4f210, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d4f215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa874f218, SUC_lexicon_tags_37 },
    { 0x334f219, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb074f21d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec94f21f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe74f222, SUC_lexicon_tags_53 },
    { 0x2bd4f222, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5b4f225, SUC_lexicon_tags_30 },
    { 0x4054f225, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e14f22b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f4f22e, SUC_lexicon_tags_27 },
    { 0x4834f22e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd54f236, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a74f239, SUC_lexicon_tags_47 },
    { 0xca54f23a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb14f23d, SUC_lexicon_tags_10 },
    { 0x1af4f23e, SUC_lexicon_tags_29 },
    { 0xdad4f23e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc34f243, SUC_lexicon_tags_45 },
    { 0x53f4f243, SUC_lexicon_tags_15 },
    { 0x5214f243, SUC_lexicon_tags_7 },
    { 0x9734f245, SUC_lexicon_tags_4 },
    { 0xc274f247, SUC_lexicon_tags_25 },
    { 0x8b34f243, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3274f24a, SUC_lexicon_tags_25 },
    { 0xcf74f24a, SUC_lexicon_tags_27 },
    { 0x3f74f24c, SUC_lexicon_tags_1 },
    { 0xda34f24d, SUC_lexicon_tags_7 },
    { 0x5194f24c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd4f250, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f4f25a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c14f265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f54f267, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc54f26a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa494f26c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d4f274, SUC_lexicon_tags_57 },
    { 0x1454f275, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7674f27a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd4f27d, SUC_lexicon_tags_151 },
    { 0x26d4f27e, SUC_lexicon_tags_19 },
    { 0xf994f27f, SUC_lexicon_tags_6 },
    { 0x4534f27e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa314f283, SUC_lexicon_tags_35 },
    { 0xfeb4f284, SUC_lexicon_tags_30 },
    { 0x5354f285, SUC_lexicon_tags_25 },
    { 0xc934f286, SUC_lexicon_tags_87 },
    { 0xd0f4f287, SUC_lexicon_tags_45 },
    { 0xa6d4f288, SUC_lexicon_tags_63 },
    { 0xb914f286, SUC_lexicon_tags_17 },
    { 0x4e14f287, SUC_lexicon_tags_17 },
    { 0x5e94f286, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7274f28d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b4f292, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f74f294, SUC_lexicon_tags_11 },
    { 0xdbd4f295, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e54f297, SUC_lexicon_tags_38 },
    { 0x9654f298, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x6bf4f29a, SUC_lexicon_tags_2 },
    { 0x3934f29a, SUC_lexicon_tags_37 },
    { 0x3994f29c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e94f29f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dd4f2a1, SUC_lexicon_tags_1 },
    { 0x5c54f2a1, SUC_lexicon_tags_12 },
    { 0x13b4f2a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf0f4f2a5, SUC_lexicon_tags_2 },
    { 0xebf4f2a6, SUC_lexicon_tags_1 },
    { 0x7334f2a7, SUC_lexicon_tags_7 },
    { 0xf594f2a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xedf4f2aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d94f2ad, SUC_lexicon_tags_39 },
    { 0xf454f2ad, SUC_lexicon_tags_37 },
    { 0x7a74f2ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb774f2b8, SUC_lexicon_tags_21 },
    { 0x794f2b8, SUC_lexicon_tags_17 },
    { 0xf754f2b9, SUC_lexicon_tags_11 },
    { 0x8734f2bb, SUC_lexicon_tags_14 },
    { 0x68b4f2bc, SUC_lexicon_tags_19 },
    { 0xc34f2bc, SUC_lexicon_tags_19 },
    { 0xb154f2be, SUC_lexicon_tags_15 },
    { 0x8994f2bf, SUC_lexicon_tags_25 },
    { 0xe694f2bd, SUC_lexicon_tags_11 },
    { 0x7874f2bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4674f2c4, SUC_lexicon_tags_1 },
    { 0xfa74f2c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41b4f2c7, SUC_lexicon_tags_8 },
    { 0xdf54f2c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3494f2ca, SUC_lexicon_tags_60 },
    { 0xaf94f2cb, SUC_lexicon_tags_11 },
    { 0xd974f2cc, SUC_lexicon_tags_21 },
    { 0xf454f2cd, SUC_lexicon_tags_12 },
    { 0xb6d4f2cc, SUC_lexicon_tags_11 },
    { 0xc5f4f2cf, SUC_lexicon_tags_92 },
    { 0xb674f2d0, SUC_lexicon_tags_25 },
    { 0xb014f2d1, SUC_lexicon_tags_56 },
    { 0x7b4f2d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa014f2d4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f4f2d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf4f2db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae74f2dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f54f2e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f94f2e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb4f2e9, SUC_lexicon_tags_37 },
    { 0x5e54f2e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3314f2f1, SUC_lexicon_tags_2 },
    { 0x9114f2f2, SUC_lexicon_tags_4 },
    { 0xc0b4f2f3, SUC_lexicon_tags_37 },
    { 0xa414f2f4, SUC_lexicon_tags_37 },
    { 0xf194f2f1, SUC_lexicon_tags_16 },
    { 0x16b4f2f6, SUC_lexicon_tags_2 },
    { 0x9134f2f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7034f2fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd94f2fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe074f303, SUC_lexicon_tags_17 },
    { 0xf834f304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a94f309, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd014f30c, SUC_lexicon_tags_87 },
    { 0x42d4f30d, SUC_lexicon_tags_7 },
    { 0x7694f30e, SUC_lexicon_tags_15 },
    { 0x9554f30e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5c54f311, SUC_lexicon_tags_1 },
    { 0x91d4f311, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6974f315, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9bd4f317, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6254f319, SUC_lexicon_tags_7 },
    { 0x7294f31a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1794f31c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x3214f31f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4474f325, SUC_lexicon_tags_27 },
    { 0xc634f325, SUC_lexicon_tags_2 },
    { 0x3c54f325, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b4f329, SUC_lexicon_tags_47 },
    { 0xf834f329, SUC_lexicon_tags_91 },
    { 0x13f4f32b, SUC_lexicon_tags_16 },
    { 0x7194f32c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4814f32e, SUC_lexicon_tags_15 },
    { 0x5d14f32f, SUC_lexicon_tags_11 },
    { 0xa54f330, SUC_lexicon_tags_17 },
    { 0x8fd4f331, SUC_lexicon_tags_4 },
    { 0xf334f32f, SUC_lexicon_tags_29 },
    { 0x9ab4f333, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed4f338, SUC_lexicon_tags_66 },
    { 0x8894f338, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4f33d, SUC_lexicon_tags_30 },
    { 0xa7b4f33e, SUC_lexicon_tags_1 },
    { 0x6314f33f, SUC_lexicon_tags_1 },
    { 0x88b4f33f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4df4f342, SUC_lexicon_tags_79 },
    { 0x4154f343, SUC_lexicon_tags_15 },
    { 0xcf4f343, SUC_lexicon_tags_3 },
    { 0xc674f345, SUC_lexicon_tags_17 },
    { 0xcf54f346, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x11f4f348, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb94f34e, SUC_lexicon_tags_11 },
    { 0x4b54f34e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc74f353, SUC_lexicon_tags_37 },
    { 0xd494f354, SUC_lexicon_tags_87 },
    { 0xe634f355, SUC_lexicon_tags_29 },
    { 0x9194f354, SUC_lexicon_tags_31 },
    { 0x8cd4f354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2654f359, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9214f35b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4594f360, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c94f364, SUC_lexicon_tags_17 },
    { 0x4834f365, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd674f36a, SUC_lexicon_tags_43 },
    { 0x8d74f36a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2b14f36d, SUC_lexicon_tags_25 },
    { 0xb9f4f36e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x974f370, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad4f37a, SUC_lexicon_tags_37 },
    { 0x1834f37b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb594f37d, SUC_lexicon_tags_11 },
    { 0x4974f37d, SUC_lexicon_tags_11 },
    { 0xf3d4f37f, SUC_lexicon_tags_6 },
    { 0x94f4f380, SUC_lexicon_tags_19 },
    { 0x3ed4f37d, SUC_lexicon_tags_60 },
    { 0x61d4f382, SUC_lexicon_tags_74 },
    { 0xa474f382, SUC_lexicon_tags_32 },
    { 0x6114f37f, SUC_lexicon_tags_2 },
    { 0x7e54f385, SUC_lexicon_tags_8 },
    { 0x3ab4f385, SUC_lexicon_tags_11 },
    { 0xe5f4f37f, SUC_lexicon_tags_11 },
    { 0xbbf4f386, SUC_lexicon_tags_12 },
    { 0xfe34f389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b4f38d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69d4f38f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd94f391, SUC_lexicon_tags_21 },
    { 0x1434f392, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7814f394, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d4f398, SUC_lexicon_tags_63 },
    { 0x44b4f399, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a34f39f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1174f3a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7174f3ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb4f3b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf174f3b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f4f3b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1f4f3ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe574f3be, SUC_lexicon_tags_31 },
    { 0x474f3be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a14f3c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e34f3c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d4f3cd, SUC_lexicon_tags_88 },
    { 0xb2d4f3ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb74f3d1, SUC_lexicon_tags_38 },
    { 0x2ab4f3d2, SUC_lexicon_tags_1 },
    { 0xc2b4f3d1, SUC_lexicon_tags_32 },
    { 0x7214f3d1, SUC_lexicon_tags_9 },
    { 0x5c94f3d5, SUC_lexicon_tags_21 },
    { 0xc3b4f3d6, SUC_lexicon_tags_31 },
    { 0xf7d4f3d7, SUC_lexicon_tags_8 },
    { 0x4514f3d7, SUC_lexicon_tags_47 },
    { 0x7334f3d3, SUC_lexicon_tags_17 },
    { 0x9454f3d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd514f3e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3c34f3e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d4f3e5, SUC_lexicon_tags_34 },
    { 0x8214f3e5, SUC_lexicon_tags_11 },
    { 0x3994f3e7, SUC_lexicon_tags_47 },
    { 0x5174f3e8, SUC_lexicon_tags_1 },
    { 0x5b14f3e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd834f3ec, SUC_lexicon_tags_74 },
    { 0x8f34f3ed, SUC_lexicon_tags_15 },
    { 0x2074f3ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df4f3f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbb4f3f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f74f3f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8714f3ff, SUC_lexicon_tags_34 },
    { 0x8b4f400, SUC_lexicon_tags_1 },
    { 0xbd94f401, SUC_lexicon_tags_77 },
    { 0x3894f402, SUC_lexicon_tags_37 },
    { 0x69b4f403, SUC_lexicon_tags_56 },
    { 0xeeb4f403, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x73d4f406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b4f40b, SUC_lexicon_tags_47 },
    { 0x2834f40c, SUC_lexicon_tags_1 },
    { 0x96f4f40d, SUC_lexicon_tags_55 },
    { 0x3ff4f40e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1694f418, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd294f41a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7df4f41c, SUC_lexicon_tags_11 },
    { 0xe1b4f41d, SUC_lexicon_tags_65 },
    { 0x1ff4f41c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa174f424, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f4f427, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37d4f429, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfd54f42b, SUC_lexicon_tags_14 },
    { 0xfcf4f42c, SUC_lexicon_tags_1 },
    { 0x42d4f42d, SUC_lexicon_tags_1 },
    { 0xa8f4f42d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xef14f431, SUC_lexicon_tags_46 },
    { 0x3ad4f432, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2034f437, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x53f4f439, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6034f43d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63b4f43f, SUC_lexicon_tags_23 },
    { 0x10d4f43f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd914f444, SUC_lexicon_tags_27 },
    { 0x1834f445, SUC_lexicon_tags_1 },
    { 0xe614f446, SUC_lexicon_tags_1 },
    { 0x1154f446, SUC_lexicon_tags_12 },
    { 0x1f94f448, SUC_lexicon_tags_1 },
    { 0xf214f449, SUC_lexicon_tags_9 },
    { 0x1794f449, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa114f44d, SUC_lexicon_tags_63 },
    { 0x6054f44e, SUC_lexicon_tags_14 },
    { 0xc334f44e, SUC_lexicon_tags_7 },
    { 0xe634f44f, SUC_lexicon_tags_1 },
    { 0x1374f44d, SUC_lexicon_tags_11 },
    { 0x1914f452, SUC_lexicon_tags_15 },
    { 0x6d34f44e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6874f457, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b4f45a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab94f45d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f4f461, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc814f463, SUC_lexicon_tags_23 },
    { 0x5bf4f464, SUC_lexicon_tags_112 },
    { 0xe294f463, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb394f467, SUC_lexicon_tags_14 },
    { 0x6f14f468, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a74f46e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f4f471, SUC_lexicon_tags_11 },
    { 0x7334f472, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd4f477, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e54f47d, SUC_lexicon_tags_103 },
    { 0xe874f47d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b4f482, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf14f489, SUC_lexicon_tags_82 },
    { 0xa7f4f48a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe354f48d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d4f490, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f4f493, SUC_lexicon_tags_9 },
    { 0x25b4f494, SUC_lexicon_tags_8 },
    { 0x7c74f493, SUC_lexicon_tags_33 },
    { 0x4634f496, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41f4f498, SUC_lexicon_tags_19 },
    { 0x8834f499, SUC_lexicon_tags_12 },
    { 0xd6d4f49a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4f49e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92f4f4a0, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b4f4a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff14f4ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd294f4b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa474f4b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f4f4ba, SUC_lexicon_tags_39 },
    { 0x3ad4f4bb, SUC_lexicon_tags_11 },
    { 0x3d4f4bc, SUC_lexicon_tags_152 },
    { 0xb334f4bd, SUC_lexicon_tags_9 },
    { 0xd374f4ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1c94f4c0, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb4f4c3, SUC_lexicon_tags_12 },
    { 0xc2d4f4c3, SUC_lexicon_tags_26 },
    { 0x794f4c5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2894f4c9, SUC_lexicon_tags_37 },
    { 0xcb14f4c9, SUC_lexicon_tags_38 },
    { 0x1cb4f4ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40b4f4cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e14f4cf, SUC_lexicon_tags_17 },
    { 0x8d34f4d0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa854f4d2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8674f4d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ef4f4d9, SUC_lexicon_tags_25 },
    { 0x6294f4da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf154f4de, SUC_lexicon_tags_35 },
    { 0xfd74f4df, SUC_lexicon_tags_47 },
    { 0xed54f4e0, SUC_lexicon_tags_77 },
    { 0x16b4f4e1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f34f4e5, SUC_lexicon_tags_25 },
    { 0xd9b4f4e6, SUC_lexicon_tags_6 },
    { 0x74b4f4e7, SUC_lexicon_tags_17 },
    { 0x474f4e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3754f4f4, SUC_lexicon_tags_80 },
    { 0x1814f4f4, SUC_lexicon_tags_6 },
    { 0x6b34f4f5, SUC_lexicon_tags_60 },
    { 0x13d4f4f7, SUC_lexicon_tags_25 },
    { 0xd754f4f8, SUC_lexicon_tags_4 },
    { 0xaa94f4f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc254f4fc, SUC_lexicon_tags_46 },
    { 0x5b34f4fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb074f503, SUC_lexicon_tags_25 },
    { 0x98b4f503, SUC_lexicon_tags_38 },
    { 0x8194f505, SUC_lexicon_tags_11 },
    { 0xf0f4f505, SUC_lexicon_tags_43 },
    { 0x5654f507, SUC_lexicon_tags_25 },
    { 0xe8d4f508, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f94f510, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa094f512, SUC_lexicon_tags_17 },
    { 0x3314f512, SUC_lexicon_tags_29 },
    { 0x80b4f514, SUC_lexicon_tags_1 },
    { 0xc214f515, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1c14f517, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac54f51f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7014f522, SUC_lexicon_tags_86 },
    { 0xb314f523, SUC_lexicon_tags_15 },
    { 0x5214f524, SUC_lexicon_tags_37 },
    { 0x6d4f525, SUC_lexicon_tags_10 },
    { 0x7694f526, SUC_lexicon_tags_44 },
    { 0x3354f527, SUC_lexicon_tags_48 },
    { 0x85d4f525, SUC_lexicon_tags_44 },
    { 0xf174f529, SUC_lexicon_tags_12 },
    { 0x1c34f52a, SUC_lexicon_tags_7 },
    { 0x2614f52b, SUC_lexicon_tags_29 },
    { 0x6f54f52c, SUC_lexicon_tags_2 },
    { 0x7234f52a, SUC_lexicon_tags_27 },
    { 0xab4f52d, SUC_lexicon_tags_25 },
    { 0x6d14f52f, SUC_lexicon_tags_60 },
    { 0x10f4f530, SUC_lexicon_tags_48 },
    { 0xa7d4f531, SUC_lexicon_tags_38 },
    { 0x5554f531, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd4f537, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3534f53d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf74f543, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d4f548, SUC_lexicon_tags_15 },
    { 0xff74f549, SUC_lexicon_tags_265 },
    { 0xeb94f548, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3234f54c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7734f54f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc034f553, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6714f557, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd794f55b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec94f56b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c34f56e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed4f572, SUC_lexicon_tags_49 },
    { 0xfb94f573, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f74f577, SUC_lexicon_tags_2 },
    { 0x72f4f578, SUC_lexicon_tags_19 },
    { 0xa894f579, SUC_lexicon_tags_49 },
    { 0x8d74f57a, SUC_lexicon_tags_14 },
    { 0x7b74f57a, SUC_lexicon_tags_14 },
    { 0x91b4f57c, SUC_lexicon_tags_37 },
    { 0xce74f57c, SUC_lexicon_tags_15 },
    { 0x9454f578, SUC_lexicon_tags_25 },
    { 0x42b4f57f, SUC_lexicon_tags_38 },
    { 0x4bb4f57f, SUC_lexicon_tags_65 },
    { 0x98f4f581, SUC_lexicon_tags_79 },
    { 0xbd94f581, SUC_lexicon_tags_14 },
    { 0xbff4f579, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d4f587, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6294f589, SUC_lexicon_tags_23 },
    { 0x4f14f589, SUC_lexicon_tags_161 },
    { 0x2694f58b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1974f58d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42d4f58f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1cb4f591, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d4f594, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a14f596, SUC_lexicon_tags_85 },
    { 0x94b4f597, SUC_lexicon_tags_11 },
    { 0x87d4f596, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b4f59d, SUC_lexicon_tags_33 },
    { 0x1334f59d, SUC_lexicon_tags_37 },
    { 0xae54f59f, SUC_lexicon_tags_87 },
    { 0xf554f5a0, SUC_lexicon_tags_66 },
    { 0x8694f5a0, SUC_lexicon_tags_37 },
    { 0x4294f5a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3734f5a4, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xe874f5a7, SUC_lexicon_tags_39 },
    { 0xd374f5a7, SUC_lexicon_tags_11 },
    { 0x2d54f5a9, SUC_lexicon_tags_1 },
    { 0x3cb4f5a9, SUC_lexicon_tags_12 },
    { 0x10d4f5aa, SUC_lexicon_tags_23 },
    { 0x2fb4f5a8, SUC_lexicon_tags_8 },
    { 0xc414f5ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d34f5b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f4f5b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b4f5b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2574f5bc, SUC_lexicon_tags_65 },
    { 0x96b4f5bc, SUC_lexicon_tags_57 },
    { 0x2b34f5bc, SUC_lexicon_tags_15 },
    { 0x3354f5be, SUC_lexicon_tags_2 },
    { 0x9274f5c0, SUC_lexicon_tags_17 },
    { 0xcf4f5c1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf74f5c4, SUC_lexicon_tags_17 },
    { 0x30f4f5c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb614f5c9, SUC_lexicon_tags_34 },
    { 0xf234f5c9, SUC_lexicon_tags_87 },
    { 0x5f4f5ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd4f5cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2994f5d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb4f5da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7a54f5dc, SUC_lexicon_tags_7 },
    { 0xaa34f5dd, SUC_lexicon_tags_65 },
    { 0xdd34f5dc, SUC_lexicon_tags_64 },
    { 0x8cd4f5df, SUC_lexicon_tags_37 },
    { 0x134f5e0, SUC_lexicon_tags_14 },
    { 0xed4f5e0, SUC_lexicon_tags_49 },
    { 0x18f4f5e0, SUC_lexicon_tags_23 },
    { 0xfa14f5e3, SUC_lexicon_tags_25 },
    { 0xe454f5dd, SUC_lexicon_tags_2 },
    { 0xfdf4f5e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5394f5e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3a74f5e9, SUC_lexicon_tags_1 },
    { 0x8a14f5e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa6d4f5ec, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x54b4f5ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d4f5f4, SUC_lexicon_tags_17 },
    { 0xd634f5f5, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf94f5ff, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xabb4f601, SUC_lexicon_tags_47 },
    { 0x19b4f602, SUC_lexicon_tags_25 },
    { 0x6414f603, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf094f605, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa534f608, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8d4f60a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4374f60e, SUC_lexicon_tags_30 },
    { 0x2074f60f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f14f612, SUC_lexicon_tags_80 },
    { 0xa774f612, SUC_lexicon_tags_38 },
    { 0x8c54f614, SUC_lexicon_tags_14 },
    { 0x6df4f615, SUC_lexicon_tags_5 },
    { 0xd214f614, SUC_lexicon_tags_2 },
    { 0xc374f615, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b4f61b, SUC_lexicon_tags_43 },
    { 0x5554f61c, SUC_lexicon_tags_38 },
    { 0x3fb4f61c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x634f61f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8294f623, SUC_lexicon_tags_39 },
    { 0xee34f623, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc754f627, SUC_lexicon_tags_7 },
    { 0xb7b4f627, SUC_lexicon_tags_29 },
    { 0xf854f629, SUC_lexicon_tags_36 },
    { 0x7994f629, SUC_lexicon_tags_0 },
    { 0xa5b4f629, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef4f62f, SUC_lexicon_tags_16 },
    { 0x8f4f630, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b4f63b, SUC_lexicon_tags_8 },
    { 0x7e34f63b, SUC_lexicon_tags_3 },
    { 0xaa14f63d, SUC_lexicon_tags_15 },
    { 0xd234f63e, SUC_lexicon_tags_25 },
    { 0xdfd4f63b, SUC_lexicon_tags_25 },
    { 0xb034f640, SUC_lexicon_tags_9 },
    { 0x8894f63e, SUC_lexicon_tags_26 },
    { 0x6434f642, SUC_lexicon_tags_0 },
    { 0xa954f642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1394f646, SUC_lexicon_tags_32 },
    { 0x34f4f646, SUC_lexicon_tags_17 },
    { 0xdb34f648, SUC_lexicon_tags_12 },
    { 0x8514f648, SUC_lexicon_tags_14 },
    { 0xb514f647, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd9d4f64c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2994f650, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d4f658, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc14f65c, SUC_lexicon_tags_37 },
    { 0xdb94f65d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d74f664, SUC_lexicon_tags_23 },
    { 0x9094f664, SUC_lexicon_tags_11 },
    { 0xe9b4f666, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d4f66b, SUC_lexicon_tags_49 },
    { 0x9bd4f66c, SUC_lexicon_tags_41 },
    { 0x1a14f66d, SUC_lexicon_tags_32 },
    { 0x29d4f66d, SUC_lexicon_tags_36 },
    { 0x6f14f66f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f4f674, SUC_lexicon_tags_7 },
    { 0x40d4f674, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcf4f677, SUC_lexicon_tags_1 },
    { 0x5e54f678, SUC_lexicon_tags_17 },
    { 0xbbd4f679, SUC_lexicon_tags_40 },
    { 0x9e34f67a, SUC_lexicon_tags_25 },
    { 0x8d4f67a, SUC_lexicon_tags_23 },
    { 0x2e14f67a, SUC_lexicon_tags_26 },
    { 0x89f4f67c, SUC_lexicon_tags_17 },
    { 0xc474f67e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4974f683, SUC_lexicon_tags_17 },
    { 0x20f4f683, SUC_lexicon_tags_37 },
    { 0x3834f685, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x95b4f687, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1534f68c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3654f693, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x35f4f695, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db4f698, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef94f69a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b4f69e, SUC_lexicon_tags_15 },
    { 0x6b4f69f, SUC_lexicon_tags_37 },
    { 0x4714f69e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe814f6a7, SUC_lexicon_tags_7 },
    { 0x84d4f6a7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b4f6ab, SUC_lexicon_tags_25 },
    { 0x90b4f6ac, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f4f6af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4f6b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b4f6bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1734f6c2, SUC_lexicon_tags_2 },
    { 0x1834f6c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc94f6c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c14f6ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d54f6d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f4f6db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d4f6df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9a74f6e1, SUC_lexicon_tags_15 },
    { 0xfff4f6e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3154f6e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb3d4f6ea, SUC_lexicon_tags_25 },
    { 0xd694f6ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c94f6ee, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1594f6f5, SUC_lexicon_tags_32 },
    { 0x59f4f6f6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2734f6f8, SUC_lexicon_tags_68 },
    { 0x2754f6f9, SUC_lexicon_tags_12 },
    { 0x534f6fa, SUC_lexicon_tags_23 },
    { 0xa954f6fb, SUC_lexicon_tags_11 },
    { 0x2014f6fc, SUC_lexicon_tags_31 },
    { 0x3474f6fd, SUC_lexicon_tags_0 },
    { 0x2074f6fe, SUC_lexicon_tags_7 },
    { 0x5e94f6ff, SUC_lexicon_tags_2 },
    { 0x7db4f6fe, SUC_lexicon_tags_32 },
    { 0xfcf4f701, SUC_lexicon_tags_28 },
    { 0x6b74f701, SUC_lexicon_tags_2 },
    { 0x2174f703, SUC_lexicon_tags_63 },
    { 0x9ef4f6fb, SUC_lexicon_tags_16 },
    { 0x1e74f705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd654f709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8894f70b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8a14f70d, SUC_lexicon_tags_6 },
    { 0x9b4f70e, SUC_lexicon_tags_46 },
    { 0x12d4f70e, SUC_lexicon_tags_42 },
    { 0xbc54f710, SUC_lexicon_tags_66 },
    { 0xb4f70d, SUC_lexicon_tags_23 },
    { 0xdd4f712, SUC_lexicon_tags_9 },
    { 0x82b4f713, SUC_lexicon_tags_49 },
    { 0xd534f714, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc374f71d, SUC_lexicon_tags_24 },
    { 0xd014f71d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d54f723, SUC_lexicon_tags_15 },
    { 0x7674f724, SUC_lexicon_tags_37 },
    { 0x2214f725, SUC_lexicon_tags_38 },
    { 0x6f74f726, SUC_lexicon_tags_7 },
    { 0x2fd4f723, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x8db4f729, SUC_lexicon_tags_10 },
    { 0xf94f72a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d14f72c, SUC_lexicon_tags_25 },
    { 0xd5d4f72d, SUC_lexicon_tags_127 },
    { 0x9714f72d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x55f4f730, SUC_lexicon_tags_14 },
    { 0xf314f731, SUC_lexicon_tags_7 },
    { 0x79f4f732, SUC_lexicon_tags_37 },
    { 0x57f4f730, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6174f735, SUC_lexicon_tags_25 },
    { 0xb754f736, SUC_lexicon_tags_4 },
    { 0xcb94f735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b4f73b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b4f741, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8dd4f743, SUC_lexicon_tags_11 },
    { 0x1d54f743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f4f747, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x9374f74a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b4f74e, SUC_lexicon_tags_37 },
    { 0xb394f74e, SUC_lexicon_tags_11 },
    { 0xe354f74f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb4f758, SUC_lexicon_tags_87 },
    { 0x8034f758, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf614f75b, SUC_lexicon_tags_37 },
    { 0x6234f75c, SUC_lexicon_tags_16 },
    { 0x4e54f75d, SUC_lexicon_tags_45 },
    { 0xc494f75d, SUC_lexicon_tags_25 },
    { 0x8c74f75f, SUC_lexicon_tags_14 },
    { 0x6234f760, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfab4f762, SUC_lexicon_tags_1 },
    { 0xf834f763, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4294f765, SUC_lexicon_tags_28 },
    { 0x6014f765, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8934f76a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1114f76e, SUC_lexicon_tags_0 },
    { 0xc114f76e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4414f771, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf394f773, SUC_lexicon_tags_1 },
    { 0xf094f773, SUC_lexicon_tags_57 },
    { 0x4614f775, SUC_lexicon_tags_2 },
    { 0x9194f776, SUC_lexicon_tags_40 },
    { 0x1a74f776, SUC_lexicon_tags_25 },
    { 0x1b74f774, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5af4f77a, SUC_lexicon_tags_82 },
    { 0x2ad4f77b, SUC_lexicon_tags_23 },
    { 0x88f4f77c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3754f782, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b4f785, SUC_lexicon_tags_11 },
    { 0x5b54f786, SUC_lexicon_tags_25 },
    { 0x9eb4f786, SUC_lexicon_tags_37 },
    { 0x1cd4f786, SUC_lexicon_tags_1 },
    { 0xa7f4f789, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86f4f78b, SUC_lexicon_tags_6 },
    { 0xf5f4f78b, SUC_lexicon_tags_17 },
    { 0x28f4f78d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5434f78f, SUC_lexicon_tags_49 },
    { 0x1b94f790, SUC_lexicon_tags_63 },
    { 0x9cd4f78f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf414f794, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbf4f796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc494f79a, SUC_lexicon_tags_2 },
    { 0x5014f79b, SUC_lexicon_tags_11 },
    { 0x82d4f79c, SUC_lexicon_tags_17 },
    { 0x1c14f79c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d4f7a3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a74f7a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf4f7a8, SUC_lexicon_tags_6 },
    { 0xd634f7a9, SUC_lexicon_tags_15 },
    { 0xe754f7a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xebb4f7ac, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d4f7af, SUC_lexicon_tags_112 },
    { 0x9574f7af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8934f7b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd814f7b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7154f7b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab94f7c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e14f7c7, SUC_lexicon_tags_44 },
    { 0xbfb4f7c8, SUC_lexicon_tags_2 },
    { 0x72b4f7c9, SUC_lexicon_tags_11 },
    { 0x7854f7ca, SUC_lexicon_tags_27 },
    { 0x3d54f7cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f94f7cf, SUC_lexicon_tags_17 },
    { 0xa534f7d0, SUC_lexicon_tags_14 },
    { 0xa1b4f7d1, SUC_lexicon_tags_15 },
    { 0x854f7d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf94f7db, SUC_lexicon_tags_57 },
    { 0xabf4f7dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3b4f7de, SUC_lexicon_tags_12 },
    { 0x3714f7de, SUC_lexicon_tags_13 },
    { 0xcbd4f7e0, SUC_lexicon_tags_43 },
    { 0x4ed4f7e1, SUC_lexicon_tags_28 },
    { 0x9ed4f7e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af4f7e6, SUC_lexicon_tags_7 },
    { 0x9df4f7e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b54f7ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b4f7ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x35f4f7ef, SUC_lexicon_tags_17 },
    { 0xd234f7ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd4f7f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e14f7fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15d4f7fe, SUC_lexicon_tags_25 },
    { 0x4494f7ff, SUC_lexicon_tags_12 },
    { 0x9f14f800, SUC_lexicon_tags_37 },
    { 0xeb4f801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1814f804, SUC_lexicon_tags_1 },
    { 0x9bb4f805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb4f809, SUC_lexicon_tags_17 },
    { 0xd914f80a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x97b4f80c, SUC_lexicon_tags_23 },
    { 0x88d4f80d, SUC_lexicon_tags_19 },
    { 0xbdb4f80d, SUC_lexicon_tags_4 },
    { 0x7d74f80f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d4f813, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb734f817, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8074f81c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac54f81e, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2974f824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab4f82d, SUC_lexicon_tags_7 },
    { 0xa1d4f82d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfcf4f830, SUC_lexicon_tags_19 },
    { 0x1534f831, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f14f835, SUC_lexicon_tags_37 },
    { 0x42b4f836, SUC_lexicon_tags_25 },
    { 0xf0b4f836, SUC_lexicon_tags_61 },
    { 0xafb4f836, SUC_lexicon_tags_25 },
    { 0x3714f839, SUC_lexicon_tags_14 },
    { 0x3ad4f839, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a14f83d, SUC_lexicon_tags_85 },
    { 0x53d4f83e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d4f841, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb4f844, SUC_lexicon_tags_32 },
    { 0xf074f845, SUC_lexicon_tags_12 },
    { 0x7b14f846, SUC_lexicon_tags_38 },
    { 0xc694f847, SUC_lexicon_tags_31 },
    { 0xf1b4f846, SUC_lexicon_tags_6 },
    { 0x6994f849, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab4f84c, SUC_lexicon_tags_9 },
    { 0xbdb4f84d, SUC_lexicon_tags_37 },
    { 0xa254f84c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f4f851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d4f85b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9554f85e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd54f860, SUC_lexicon_tags_15 },
    { 0x6834f860, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d4f867, SUC_lexicon_tags_2 },
    { 0xeed4f867, SUC_lexicon_tags_15 },
    { 0x8c94f867, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe14f86e, SUC_lexicon_tags_17 },
    { 0x9334f86f, SUC_lexicon_tags_19 },
    { 0x54d4f870, SUC_lexicon_tags_27 },
    { 0x6c14f870, SUC_lexicon_tags_12 },
    { 0x41b4f870, SUC_lexicon_tags_66 },
    { 0x4b74f86f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b14f876, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d4f879, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb14f87d, SUC_lexicon_tags_11 },
    { 0x7bd4f87e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d34f880, SUC_lexicon_tags_28 },
    { 0xf7d4f880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa954f889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bd4f88b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9334f894, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce34f899, SUC_lexicon_tags_31 },
    { 0x2994f89a, SUC_lexicon_tags_8 },
    { 0xfdd4f89a, SUC_lexicon_tags_31 },
    { 0x4e94f89b, SUC_lexicon_tags_0 },
    { 0xd2f4f89d, SUC_lexicon_tags_46 },
    { 0xca34f89b, SUC_lexicon_tags_15 },
    { 0xa554f89d, SUC_lexicon_tags_27 },
    { 0x72d4f8a0, SUC_lexicon_tags_17 },
    { 0xf194f8a1, SUC_lexicon_tags_37 },
    { 0xcb94f8a2, SUC_lexicon_tags_12 },
    { 0x48f4f8a2, SUC_lexicon_tags_29 },
    { 0xdbd4f8a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b34f8aa, SUC_lexicon_tags_8 },
    { 0x3694f8ab, SUC_lexicon_tags_27 },
    { 0x2334f8aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d4f8b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f8b3, SUC_lexicon_tags_12 },
    { 0xd2b4f8b4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc634f8b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f4f8b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa014f8bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c34f8c1, SUC_lexicon_tags_11 },
    { 0xb454f8c2, SUC_lexicon_tags_0 },
    { 0xea14f8c3, SUC_lexicon_tags_25 },
    { 0xbd74f8c3, SUC_lexicon_tags_1 },
    { 0x5914f8c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d4f8c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9474f8cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7854f8d3, SUC_lexicon_tags_12 },
    { 0xe8f4f8d4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf34f8dc, SUC_lexicon_tags_16 },
    { 0xb634f8dc, SUC_lexicon_tags_23 },
    { 0xa914f8de, SUC_lexicon_tags_17 },
    { 0xae94f8df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x514f8e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d4f8ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3634f8ee, SUC_lexicon_tags_0 },
    { 0xf794f8ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ab4f8f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d4f8f6, SUC_lexicon_tags_13 },
    { 0x4ab4f8f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b4f8fa, SUC_lexicon_tags_6 },
    { 0x15b4f8fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e54f903, SUC_lexicon_tags_14 },
    { 0x9d14f904, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b4f908, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab4f90f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f4f912, SUC_lexicon_tags_28 },
    { 0x2994f913, SUC_lexicon_tags_11 },
    { 0x4d94f913, SUC_lexicon_tags_33 },
    { 0xb14f915, SUC_lexicon_tags_17 },
    { 0x4b54f916, SUC_lexicon_tags_17 },
    { 0x73d4f915, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f91a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85d4f91c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd354f920, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xffd4f922, SUC_lexicon_tags_45 },
    { 0xe254f923, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7854f927, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd74f929, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff4f92d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ad4f92f, SUC_lexicon_tags_12 },
    { 0xee94f92f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd114f932, SUC_lexicon_tags_37 },
    { 0x574f933, SUC_lexicon_tags_15 },
    { 0x9fb4f933, SUC_lexicon_tags_9 },
    { 0x2914f934, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb4f93c, SUC_lexicon_tags_49 },
    { 0x5054f93d, SUC_lexicon_tags_7 },
    { 0x9b14f93d, SUC_lexicon_tags_7 },
    { 0xfcf4f93f, SUC_lexicon_tags_14 },
    { 0xe1f4f93d, SUC_lexicon_tags_10 },
    { 0xbbf4f93d, SUC_lexicon_tags_8 },
    { 0x2994f942, SUC_lexicon_tags_3 },
    { 0x3c34f942, SUC_lexicon_tags_7 },
    { 0xcc54f942, SUC_lexicon_tags_37 },
    { 0x9bb4f945, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6534f947, SUC_lexicon_tags_64 },
    { 0x6574f948, SUC_lexicon_tags_17 },
    { 0x5254f947, SUC_lexicon_tags_11 },
    { 0xc4f4f94a, SUC_lexicon_tags_41 },
    { 0xb714f94b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57d4f94d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x6734f94f, SUC_lexicon_tags_63 },
    { 0x6f54f94f, SUC_lexicon_tags_2 },
    { 0x8af4f951, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf4f958, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76f4f95a, SUC_lexicon_tags_14 },
    { 0xd014f95b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9614f95f, SUC_lexicon_tags_29 },
    { 0x2934f95f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f34f963, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b34f965, SUC_lexicon_tags_35 },
    { 0x7834f965, SUC_lexicon_tags_20 },
    { 0xae14f965, SUC_lexicon_tags_439 },
    { 0x334f968, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd4f96c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf4f96f, SUC_lexicon_tags_16 },
    { 0xb514f970, SUC_lexicon_tags_9 },
    { 0xf34f970, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaf94f973, SUC_lexicon_tags_29 },
    { 0x6454f974, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d54f977, SUC_lexicon_tags_17 },
    { 0x3274f977, SUC_lexicon_tags_11 },
    { 0xb374f979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f74f985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25d4f987, SUC_lexicon_tags_14 },
    { 0x1c94f987, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc0d4f98a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c94f990, SUC_lexicon_tags_37 },
    { 0xced4f990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a54f997, SUC_lexicon_tags_49 },
    { 0xa654f998, SUC_lexicon_tags_27 },
    { 0xf2d4f997, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd114f99c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d4f99e, SUC_lexicon_tags_47 },
    { 0x42f4f99e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x354f9a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d74f9a5, SUC_lexicon_tags_15 },
    { 0xc7f4f9a6, SUC_lexicon_tags_27 },
    { 0x9db4f9a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5d4f9a9, SUC_lexicon_tags_35 },
    { 0xb1f4f9aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e54f9ac, SUC_lexicon_tags_38 },
    { 0x3ed4f9ad, SUC_lexicon_tags_25 },
    { 0xddf4f9ae, SUC_lexicon_tags_46 },
    { 0x14b4f9ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b94f9b6, SUC_lexicon_tags_1 },
    { 0xee14f9b7, SUC_lexicon_tags_25 },
    { 0x1c74f9b8, SUC_lexicon_tags_12 },
    { 0xbef4f9b8, SUC_lexicon_tags_10 },
    { 0x3db4f9b9, SUC_lexicon_tags_17 },
    { 0x99f4f9b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b4f9bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df4f9c3, SUC_lexicon_tags_12 },
    { 0xfa34f9c4, SUC_lexicon_tags_21 },
    { 0x9974f9c5, SUC_lexicon_tags_37 },
    { 0xd394f9c3, SUC_lexicon_tags_26 },
    { 0x8f14f9c4, SUC_lexicon_tags_61 },
    { 0x4ed4f9c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e54f9cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3734f9d2, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf114f9d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd994f9db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8234f9df, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4054f9e2, SUC_lexicon_tags_7 },
    { 0x31f4f9e3, SUC_lexicon_tags_23 },
    { 0x8174f9e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf674f9e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4814f9eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a54f9ef, SUC_lexicon_tags_0 },
    { 0xa1d4f9f0, SUC_lexicon_tags_27 },
    { 0x9654f9f1, SUC_lexicon_tags_29 },
    { 0x4cb4f9f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x714f9f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd4f9fc, SUC_lexicon_tags_60 },
    { 0x57d4f9fd, SUC_lexicon_tags_45 },
    { 0x9c34f9fd, SUC_lexicon_tags_25 },
    { 0x3af4f9ff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xff94fa02, SUC_lexicon_tags_60 },
    { 0x15f4fa03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1914fa06, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd894fa08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa714fa0c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6834fa0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7754fa15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee54fa1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf4fa22, SUC_lexicon_tags_11 },
    { 0x6794fa23, SUC_lexicon_tags_17 },
    { 0xfcf4fa23, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad4fa27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf674fa29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef4fa30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc54fa36, SUC_lexicon_tags_27 },
    { 0x5994fa37, SUC_lexicon_tags_29 },
    { 0x7fd4fa38, SUC_lexicon_tags_37 },
    { 0x19b4fa39, SUC_lexicon_tags_40 },
    { 0x3014fa39, SUC_lexicon_tags_1 },
    { 0xa434fa37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xab34fa3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9174fa43, SUC_lexicon_tags_17 },
    { 0x9d34fa43, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b14fa49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf814fa4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab4fa50, SUC_lexicon_tags_13 },
    { 0x3394fa51, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1f34fa53, SUC_lexicon_tags_28 },
    { 0x42f4fa53, SUC_lexicon_tags_23 },
    { 0x97d4fa55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb5f4fa57, SUC_lexicon_tags_15 },
    { 0xce54fa58, SUC_lexicon_tags_37 },
    { 0xf1b4fa57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6474fa5c, SUC_lexicon_tags_12 },
    { 0x7fd4fa5d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa14fa62, SUC_lexicon_tags_47 },
    { 0x3d94fa62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe294fa66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf4fa6a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb514fa6f, SUC_lexicon_tags_37 },
    { 0xc8b4fa6f, SUC_lexicon_tags_7 },
    { 0x8ef4fa71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71f4fa73, SUC_lexicon_tags_1 },
    { 0x9494fa74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaed4fa76, SUC_lexicon_tags_64 },
    { 0x13d4fa76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbed4fa79, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaf74fa7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b4fa7f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3cb4fa81, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f4fa87, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b4fa8a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f4fa8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe7d4fa90, SUC_lexicon_tags_11 },
    { 0xadb4fa91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf94fa96, SUC_lexicon_tags_7 },
    { 0xef4fa96, SUC_lexicon_tags_11 },
    { 0xf014fa97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f54fa9e, SUC_lexicon_tags_25 },
    { 0x3e74fa9e, SUC_lexicon_tags_18 },
    { 0x72f4fa9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f4faa6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d34faab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b4fab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f4fab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7634faba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47b4fabc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b4fac5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf74faca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9974face, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f4fad2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f4fadc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b34fadf, SUC_lexicon_tags_2 },
    { 0x5674fae0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb394fae2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f4fae6, SUC_lexicon_tags_25 },
    { 0x6c34fae6, SUC_lexicon_tags_15 },
    { 0x3b94fae6, SUC_lexicon_tags_29 },
    { 0x5ad4fae9, SUC_lexicon_tags_20 },
    { 0xb6d4fae9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b4faed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf4faf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b4faf6, SUC_lexicon_tags_27 },
    { 0x8794faf7, SUC_lexicon_tags_10 },
    { 0xb274faf8, SUC_lexicon_tags_7 },
    { 0x87f4faf7, SUC_lexicon_tags_0 },
    { 0x1db4faf9, SUC_lexicon_tags_2 },
    { 0x9eb4fafb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d4faff, SUC_lexicon_tags_39 },
    { 0x1f94fb00, SUC_lexicon_tags_2 },
    { 0x5ff4fb01, SUC_lexicon_tags_230 },
    { 0xe614fb02, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xba74fb04, SUC_lexicon_tags_11 },
    { 0xecb4fb04, SUC_lexicon_tags_43 },
    { 0xf494fb06, SUC_lexicon_tags_19 },
    { 0xeb34fb07, SUC_lexicon_tags_14 },
    { 0x2d94fb06, SUC_lexicon_tags_1 },
    { 0xf374fb09, SUC_lexicon_tags_19 },
    { 0x81d4fb0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda74fb0d, SUC_lexicon_tags_27 },
    { 0x6054fb0e, SUC_lexicon_tags_17 },
    { 0x4354fb0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8d54fb11, SUC_lexicon_tags_26 },
    { 0xfc94fb11, SUC_lexicon_tags_7 },
    { 0x8f14fb13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5834fb17, SUC_lexicon_tags_4 },
    { 0x7074fb17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd054fb1a, SUC_lexicon_tags_10 },
    { 0x7394fb1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9014fb21, SUC_lexicon_tags_17 },
    { 0xc334fb22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf4fb28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2314fb2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa34fb2e, SUC_lexicon_tags_27 },
    { 0x90f4fb2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1954fb31, SUC_lexicon_tags_2 },
    { 0x5154fb32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6494fb38, SUC_lexicon_tags_65 },
    { 0x99d4fb39, SUC_lexicon_tags_12 },
    { 0xb894fb3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda94fb3e, SUC_lexicon_tags_20 },
    { 0x57b4fb3f, SUC_lexicon_tags_992 },
    { 0x7e34fb40, SUC_lexicon_tags_1 },
    { 0xc834fb3f, SUC_lexicon_tags_27 },
    { 0x15f4fb41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b4fb45, SUC_lexicon_tags_6 },
    { 0xb5b4fb46, SUC_lexicon_tags_25 },
    { 0x6794fb45, SUC_lexicon_tags_31 },
    { 0xfcd4fb48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f34fb4a, SUC_lexicon_tags_29 },
    { 0xc254fb4b, SUC_lexicon_tags_6 },
    { 0xc254fb4b, SUC_lexicon_tags_45 },
    { 0x83d4fb4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9754fb4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f4fb55, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf2f4fb57, SUC_lexicon_tags_27 },
    { 0x9c54fb57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd574fb5b, SUC_lexicon_tags_26 },
    { 0xb874fb5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9494fb5e, SUC_lexicon_tags_82 },
    { 0xa5d4fb5f, SUC_lexicon_tags_29 },
    { 0xb934fb60, SUC_lexicon_tags_29 },
    { 0xde74fb5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9774fb63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7234fb65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc634fb68, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab14fb6f, SUC_lexicon_tags_6 },
    { 0x8014fb70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff4fb76, SUC_lexicon_tags_17 },
    { 0xb0d4fb76, SUC_lexicon_tags_2 },
    { 0xdd74fb78, SUC_lexicon_tags_45 },
    { 0xa874fb78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c74fb7c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d4fb84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7714fb86, SUC_lexicon_tags_47 },
    { 0x2254fb86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbf4fb89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6814fb8e, SUC_lexicon_tags_25 },
    { 0x1a34fb8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2214fb94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb334fb96, SUC_lexicon_tags_63 },
    { 0xe4b4fb96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc234fb9a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4754fb9c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xea94fb9e, SUC_lexicon_tags_19 },
    { 0xfa34fb9f, SUC_lexicon_tags_7 },
    { 0xee14fb9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe314fba5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a94fba7, SUC_lexicon_tags_82 },
    { 0xa394fba8, SUC_lexicon_tags_60 },
    { 0xeeb4fba9, SUC_lexicon_tags_17 },
    { 0xbed4fba9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff34fbae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6094fbb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd34fbb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7714fbb6, SUC_lexicon_tags_97 },
    { 0x48b4fbb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf074fbbb, SUC_lexicon_tags_6 },
    { 0xb834fbbc, SUC_lexicon_tags_87 },
    { 0x9214fbbc, SUC_lexicon_tags_21 },
    { 0x5614fbbe, SUC_lexicon_tags_48 },
    { 0xa3b4fbbf, SUC_lexicon_tags_18 },
    { 0x93d4fbc0, SUC_lexicon_tags_25 },
    { 0x7a94fbc0, SUC_lexicon_tags_63 },
    { 0x9b94fbbc, SUC_lexicon_tags_19 },
    { 0x8cb4fbc3, SUC_lexicon_tags_11 },
    { 0x9774fbc4, SUC_lexicon_tags_17 },
    { 0x8794fbc5, SUC_lexicon_tags_12 },
    { 0x2874fbc6, SUC_lexicon_tags_43 },
    { 0xa134fbbe, SUC_lexicon_tags_11 },
    { 0x7a14fbc8, SUC_lexicon_tags_15 },
    { 0x3294fbc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e14fbcc, SUC_lexicon_tags_82 },
    { 0x7e14fbcd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc554fbcf, SUC_lexicon_tags_15 },
    { 0x6ab4fbd0, SUC_lexicon_tags_16 },
    { 0x60d4fbcf, SUC_lexicon_tags_2 },
    { 0xc214fbd1, SUC_lexicon_tags_3 },
    { 0xb334fbd3, SUC_lexicon_tags_17 },
    { 0x714fbd3, SUC_lexicon_tags_26 },
    { 0xf914fbd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f4fbd9, SUC_lexicon_tags_31 },
    { 0xccb4fbd9, SUC_lexicon_tags_25 },
    { 0x11d4fbdb, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff4fbe2, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x8174fbe5, SUC_lexicon_tags_11 },
    { 0x7f74fbe6, SUC_lexicon_tags_43 },
    { 0x5f14fbe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc254fbea, SUC_lexicon_tags_17 },
    { 0xa994fbeb, SUC_lexicon_tags_15 },
    { 0xda34fbec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6b4fbee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5f4fbf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3774fbf3, SUC_lexicon_tags_7 },
    { 0x1eb4fbf4, SUC_lexicon_tags_33 },
    { 0xcbb4fbf3, SUC_lexicon_tags_47 },
    { 0x5c34fbf3, SUC_lexicon_tags_12 },
    { 0x2ef4fbf3, SUC_lexicon_tags_55 },
    { 0x4254fbf3, SUC_lexicon_tags_19 },
    { 0xbd94fbf9, SUC_lexicon_tags_25 },
    { 0xa7b4fbf9, SUC_lexicon_tags_55 },
    { 0xec34fbfb, SUC_lexicon_tags_37 },
    { 0x1014fbfa, SUC_lexicon_tags_25 },
    { 0x8cf4fbf9, SUC_lexicon_tags_2 },
    { 0x3d74fbfe, SUC_lexicon_tags_27 },
    { 0x5bf4fbff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d4fc03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3874fc09, SUC_lexicon_tags_1 },
    { 0x5754fc0a, SUC_lexicon_tags_1 },
    { 0x7bd4fc0b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf14fc12, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d4fc15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f4fc19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f4fc1f, SUC_lexicon_tags_11 },
    { 0xb4fc20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d4fc25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f34fc2f, SUC_lexicon_tags_12 },
    { 0x3fb4fc30, SUC_lexicon_tags_37 },
    { 0x5e14fc2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdf4fc33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1454fc36, SUC_lexicon_tags_30 },
    { 0x2834fc36, SUC_lexicon_tags_12 },
    { 0xbd14fc38, SUC_lexicon_tags_7 },
    { 0xb374fc39, SUC_lexicon_tags_79 },
    { 0xb434fc38, SUC_lexicon_tags_2 },
    { 0x1f4fc38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff4fc45, SUC_lexicon_tags_82 },
    { 0xbcd4fc46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc914fc4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d4fc4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a94fc51, SUC_lexicon_tags_25 },
    { 0xc9d4fc52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2b4fc54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3394fc56, SUC_lexicon_tags_17 },
    { 0x4fd4fc56, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcf74fc59, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad4fc5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa914fc61, SUC_lexicon_tags_1479 },
    { 0, NULL },
    { 0, NULL },
    { 0xb074fc64, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df4fc67, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd4fc6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff74fc71, SUC_lexicon_tags_2 },
    { 0x33d4fc72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad4fc75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d4fc79, SUC_lexicon_tags_56 },
    { 0x9714fc7a, SUC_lexicon_tags_37 },
    { 0x6434fc7b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b34fc7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d4fc85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d4fc88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8574fc8a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x30f4fc8c, SUC_lexicon_tags_28 },
    { 0x4494fc8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c74fc8f, SUC_lexicon_tags_15 },
    { 0x6c14fc8f, SUC_lexicon_tags_27 },
    { 0xf014fc91, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb314fc99, SUC_lexicon_tags_26 },
    { 0xdfd4fc99, SUC_lexicon_tags_16 },
    { 0xad74fc9b, SUC_lexicon_tags_29 },
    { 0x3694fc9c, SUC_lexicon_tags_38 },
    { 0xab34fc9d, SUC_lexicon_tags_11 },
    { 0xa0d4fc9c, SUC_lexicon_tags_45 },
    { 0xae94fc9e, SUC_lexicon_tags_7 },
    { 0x774fc9d, SUC_lexicon_tags_12 },
    { 0xd714fc9d, SUC_lexicon_tags_7 },
    { 0x7374fca2, SUC_lexicon_tags_85 },
    { 0x68b4fca3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3094fca5, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb4fca9, SUC_lexicon_tags_17 },
    { 0xfa74fcaa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe454fcac, SUC_lexicon_tags_7 },
    { 0xbef4fcad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4734fcb1, SUC_lexicon_tags_43 },
    { 0x7b74fcb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c14fcb4, SUC_lexicon_tags_12 },
    { 0x5ff4fcb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd4fcb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6754fcbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee34fcc6, SUC_lexicon_tags_11 },
    { 0xb854fcc7, SUC_lexicon_tags_15 },
    { 0x134fcc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6bd4fcca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1934fccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe194fcd2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc434fcd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43b4fcda, SUC_lexicon_tags_25 },
    { 0xfcb4fcdb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbc14fcdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd634fce0, SUC_lexicon_tags_3 },
    { 0x9094fce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7494fce7, SUC_lexicon_tags_7 },
    { 0x2b4fce8, SUC_lexicon_tags_17 },
    { 0x18d4fce8, SUC_lexicon_tags_1 },
    { 0x9654fcea, SUC_lexicon_tags_37 },
    { 0x2bd4fceb, SUC_lexicon_tags_28 },
    { 0xbb34fcec, SUC_lexicon_tags_1 },
    { 0x69b4fcec, SUC_lexicon_tags_14 },
    { 0x8614fced, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a74fcf4, SUC_lexicon_tags_19 },
    { 0x6874fcf5, SUC_lexicon_tags_34 },
    { 0x93d4fcf5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39d4fcf8, SUC_lexicon_tags_2 },
    { 0xad74fcf9, SUC_lexicon_tags_7 },
    { 0x3f14fcf8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd814fcff, SUC_lexicon_tags_63 },
    { 0x9594fd00, SUC_lexicon_tags_7 },
    { 0xe9f4fd01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf4fd06, SUC_lexicon_tags_17 },
    { 0x9d94fd07, SUC_lexicon_tags_25 },
    { 0xb8d4fd08, SUC_lexicon_tags_65 },
    { 0xdcf4fd07, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa114fd0b, SUC_lexicon_tags_47 },
    { 0xd054fd0c, SUC_lexicon_tags_39 },
    { 0xc914fd0d, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0x46d4fd0f, SUC_lexicon_tags_68 },
    { 0xeb54fd10, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xdc94fd12, SUC_lexicon_tags_27 },
    { 0x1c14fd12, SUC_lexicon_tags_6 },
    { 0xa6b4fd13, SUC_lexicon_tags_39 },
    { 0xe6d4fd15, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6c14fd17, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd894fd19, SUC_lexicon_tags_14 },
    { 0x1634fd19, SUC_lexicon_tags_49 },
    { 0x6cb4fd1b, SUC_lexicon_tags_2 },
    { 0xe7f4fd1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b4fd1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72f4fd21, SUC_lexicon_tags_45 },
    { 0x2214fd22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a54fd24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd194fd2a, SUC_lexicon_tags_19 },
    { 0xc614fd2b, SUC_lexicon_tags_32 },
    { 0xa74fd2c, SUC_lexicon_tags_37 },
    { 0x4b94fd2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b4fd32, SUC_lexicon_tags_22 },
    { 0x7db4fd33, SUC_lexicon_tags_25 },
    { 0x8354fd32, SUC_lexicon_tags_1 },
    { 0xd4b4fd35, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1954fd37, SUC_lexicon_tags_43 },
    { 0xf474fd38, SUC_lexicon_tags_17 },
    { 0x6a94fd39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb54fd3e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d4fd41, SUC_lexicon_tags_29 },
    { 0x5d4fd42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0d4fd44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd94fd47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5114fd51, SUC_lexicon_tags_0 },
    { 0xd794fd52, SUC_lexicon_tags_69 },
    { 0xe934fd51, SUC_lexicon_tags_17 },
    { 0xc654fd54, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x77d4fd56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d4fd59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3014fd5b, SUC_lexicon_tags_61 },
    { 0x91d4fd5c, SUC_lexicon_tags_1 },
    { 0x78d4fd5b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4014fd62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2294fd64, SUC_lexicon_tags_17 },
    { 0x4c14fd64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc714fd67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b4fd6e, SUC_lexicon_tags_82 },
    { 0x3614fd6f, SUC_lexicon_tags_25 },
    { 0x9954fd70, SUC_lexicon_tags_7 },
    { 0xa594fd6f, SUC_lexicon_tags_17 },
    { 0x6034fd72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29d4fd74, SUC_lexicon_tags_6 },
    { 0x70f4fd75, SUC_lexicon_tags_38 },
    { 0x3c94fd75, SUC_lexicon_tags_19 },
    { 0x2994fd77, SUC_lexicon_tags_17 },
    { 0x10d4fd74, SUC_lexicon_tags_29 },
    { 0xf0d4fd76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb4fd7d, SUC_lexicon_tags_0 },
    { 0xe214fd7e, SUC_lexicon_tags_43 },
    { 0xa994fd7f, SUC_lexicon_tags_38 },
    { 0xb94fd80, SUC_lexicon_tags_38 },
    { 0x3954fd81, SUC_lexicon_tags_15 },
    { 0x5a34fd7f, SUC_lexicon_tags_2 },
    { 0x5fd4fd83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9474fd85, SUC_lexicon_tags_11 },
    { 0xa014fd86, SUC_lexicon_tags_1 },
    { 0x3ef4fd87, SUC_lexicon_tags_16 },
    { 0x1874fd87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1dd4fd8a, SUC_lexicon_tags_25 },
    { 0xd0f4fd8b, SUC_lexicon_tags_11 },
    { 0xb794fd8a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf494fd91, SUC_lexicon_tags_31 },
    { 0x89d4fd91, SUC_lexicon_tags_11 },
    { 0x9bf4fd91, SUC_lexicon_tags_68 },
    { 0xe134fd94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4774fd96, SUC_lexicon_tags_17 },
    { 0x2794fd96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db4fd9e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf4fda1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ef4fda3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb4fda7, SUC_lexicon_tags_27 },
    { 0xf8f4fda8, SUC_lexicon_tags_9 },
    { 0xc374fda7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e14fdae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5674fdb0, SUC_lexicon_tags_17 },
    { 0x58b4fdb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f4fdb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d4fdb7, SUC_lexicon_tags_60 },
    { 0x13d4fdb8, SUC_lexicon_tags_63 },
    { 0x7bf4fdb8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4554fdbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb694fdc8, SUC_lexicon_tags_37 },
    { 0x72d4fdc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf114fdcb, SUC_lexicon_tags_15 },
    { 0xe5b4fdcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdad4fdce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad4fdd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f4fdd4, SUC_lexicon_tags_585 },
    { 0, NULL },
    { 0xc834fdd6, SUC_lexicon_tags_23 },
    { 0xf814fdd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x134fdda, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8534fddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b4fde3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a14fde6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc54fdec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9694fdee, SUC_lexicon_tags_26 },
    { 0x4b54fdee, SUC_lexicon_tags_25 },
    { 0x3174fdf0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9354fdf2, SUC_lexicon_tags_14 },
    { 0xf1f4fdf2, SUC_lexicon_tags_37 },
    { 0xb234fdf4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf474fdf7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5954fdfb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3874fe00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4094fe07, SUC_lexicon_tags_11 },
    { 0x9014fe08, SUC_lexicon_tags_25 },
    { 0xc9d4fe09, SUC_lexicon_tags_6 },
    { 0xba94fe07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1134fe0c, SUC_lexicon_tags_26 },
    { 0x14fe0c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d34fe12, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a94fe17, SUC_lexicon_tags_1 },
    { 0x2cf4fe18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa394fe1c, SUC_lexicon_tags_49 },
    { 0x63d4fe1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3eb4fe1f, SUC_lexicon_tags_25 },
    { 0xaf54fe20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9734fe22, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd834fe26, SUC_lexicon_tags_37 },
    { 0x4a54fe27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd534fe29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe234fe2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4414fe2e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6f54fe30, SUC_lexicon_tags_61 },
    { 0xf374fe30, SUC_lexicon_tags_14 },
    { 0x7674fe32, SUC_lexicon_tags_17 },
    { 0x51b4fe33, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc074fe35, SUC_lexicon_tags_38 },
    { 0xc7d4fe36, SUC_lexicon_tags_12 },
    { 0x2834fe36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6f4fe39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e14fe3f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7394fe43, SUC_lexicon_tags_15 },
    { 0x97b4fe43, SUC_lexicon_tags_28 },
    { 0x2234fe44, SUC_lexicon_tags_26 },
    { 0x14b4fe45, SUC_lexicon_tags_2 },
    { 0x10f4fe46, SUC_lexicon_tags_31 },
    { 0xc994fe48, SUC_lexicon_tags_35 },
    { 0xc1d4fe48, SUC_lexicon_tags_37 },
    { 0x1434fe43, SUC_lexicon_tags_0 },
    { 0x96b4fe4b, SUC_lexicon_tags_15 },
    { 0x3774fe47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc274fe52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d4fe55, SUC_lexicon_tags_1 },
    { 0xe814fe56, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4c54fe58, SUC_lexicon_tags_37 },
    { 0xb0d4fe59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb4fe5d, SUC_lexicon_tags_0 },
    { 0xed34fe5e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xa34fe60, SUC_lexicon_tags_11 },
    { 0x1374fe61, SUC_lexicon_tags_37 },
    { 0x9a34fe61, SUC_lexicon_tags_14 },
    { 0x1eb4fe63, SUC_lexicon_tags_65 },
    { 0xe6f4fe64, SUC_lexicon_tags_37 },
    { 0xe6f4fe65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfd4fe67, SUC_lexicon_tags_7 },
    { 0x3934fe68, SUC_lexicon_tags_0 },
    { 0x3e54fe69, SUC_lexicon_tags_17 },
    { 0xac34fe6a, SUC_lexicon_tags_12 },
    { 0x1c74fe68, SUC_lexicon_tags_39 },
    { 0x7f34fe6c, SUC_lexicon_tags_17 },
    { 0xcdd4fe6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2154fe6f, SUC_lexicon_tags_37 },
    { 0xf3b4fe70, SUC_lexicon_tags_7 },
    { 0x814fe71, SUC_lexicon_tags_47 },
    { 0x3334fe72, SUC_lexicon_tags_27 },
    { 0x80f4fe72, SUC_lexicon_tags_26 },
    { 0xc154fe74, SUC_lexicon_tags_23 },
    { 0xf5b4fe75, SUC_lexicon_tags_37 },
    { 0x9f94fe75, SUC_lexicon_tags_14 },
    { 0x84d4fe77, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x2954fe79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4034fe7e, SUC_lexicon_tags_17 },
    { 0x3df4fe7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21d4fe81, SUC_lexicon_tags_47 },
    { 0x83b4fe81, SUC_lexicon_tags_27 },
    { 0xc214fe83, SUC_lexicon_tags_12 },
    { 0x9f14fe83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf194fe86, SUC_lexicon_tags_6 },
    { 0x7154fe87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc734fe8c, SUC_lexicon_tags_60 },
    { 0x37f4fe8c, SUC_lexicon_tags_26 },
    { 0x5834fe8c, SUC_lexicon_tags_8 },
    { 0x5eb4fe8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f4fe93, SUC_lexicon_tags_26 },
    { 0x4094fe94, SUC_lexicon_tags_26 },
    { 0xc314fe95, SUC_lexicon_tags_25 },
    { 0x8fb4fe95, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xc5d4fe98, SUC_lexicon_tags_56 },
    { 0x4d74fe99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3014fe9b, SUC_lexicon_tags_43 },
    { 0x6c94fe9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x834fe9f, SUC_lexicon_tags_15 },
    { 0xfbd4fea0, SUC_lexicon_tags_2 },
    { 0x82d4fea1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8514fea5, SUC_lexicon_tags_11 },
    { 0xe734fea6, SUC_lexicon_tags_27 },
    { 0xe874fea7, SUC_lexicon_tags_25 },
    { 0x9f94fea8, SUC_lexicon_tags_63 },
    { 0x9a94fea9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xad94feab, SUC_lexicon_tags_87 },
    { 0xe0b4feac, SUC_lexicon_tags_33 },
    { 0x45d4fead, SUC_lexicon_tags_63 },
    { 0x4a34feae, SUC_lexicon_tags_83 },
    { 0xdfd4fead, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d4feb4, SUC_lexicon_tags_10 },
    { 0x6d34feb4, SUC_lexicon_tags_148 },
    { 0x8ed4feb6, SUC_lexicon_tags_27 },
    { 0xb3d4feb4, SUC_lexicon_tags_14 },
    { 0xa214feb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb854feba, SUC_lexicon_tags_17 },
    { 0x30d4febb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9b74febd, SUC_lexicon_tags_7 },
    { 0x4154febe, SUC_lexicon_tags_8 },
    { 0x7694febe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a14fec1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ab4fec3, SUC_lexicon_tags_17 },
    { 0xa034fec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f94fecb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba14fecd, SUC_lexicon_tags_19 },
    { 0x1b74fece, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6594fed2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe654fed4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7914fed8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd054fee0, SUC_lexicon_tags_12 },
    { 0x6ff4fee1, SUC_lexicon_tags_39 },
    { 0xfe74fee1, SUC_lexicon_tags_60 },
    { 0xed34fee3, SUC_lexicon_tags_43 },
    { 0x6f54fee3, SUC_lexicon_tags_11 },
    { 0xda94fee5, SUC_lexicon_tags_8 },
    { 0xbe54fee4, SUC_lexicon_tags_17 },
    { 0x3df4fee4, SUC_lexicon_tags_63 },
    { 0xe694fee5, SUC_lexicon_tags_25 },
    { 0xfa14fee6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd974fef0, SUC_lexicon_tags_1 },
    { 0x2754fef0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8af4fef3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3f4fef5, SUC_lexicon_tags_11 },
    { 0x44f4fef6, SUC_lexicon_tags_27 },
    { 0x6b54fef7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8eb4fef9, SUC_lexicon_tags_7 },
    { 0x3614fefa, SUC_lexicon_tags_184 },
    { 0x3954fefb, SUC_lexicon_tags_17 },
    { 0x3bb4fefa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b34fefe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7214ff00, SUC_lexicon_tags_27 },
    { 0xbbf4ff01, SUC_lexicon_tags_55 },
    { 0x8c14ff02, SUC_lexicon_tags_23 },
    { 0xd634ff00, SUC_lexicon_tags_7 },
    { 0xde54ff04, SUC_lexicon_tags_49 },
    { 0xd234ff05, SUC_lexicon_tags_14 },
    { 0x46f4ff06, SUC_lexicon_tags_15 },
    { 0x72f4ff07, SUC_lexicon_tags_31 },
    { 0xc874ff08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d4ff0c, SUC_lexicon_tags_28 },
    { 0xdbd4ff0d, SUC_lexicon_tags_37 },
    { 0xbe14ff0c, SUC_lexicon_tags_899 },
    { 0xcfb4ff0f, SUC_lexicon_tags_38 },
    { 0xf3f4ff0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfc74ff12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc734ff1d, SUC_lexicon_tags_10 },
    { 0x52b4ff1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f4ff21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe354ff26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c74ff28, SUC_lexicon_tags_11 },
    { 0xde74ff29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae14ff2c, SUC_lexicon_tags_109 },
    { 0x634ff2c, SUC_lexicon_tags_17 },
    { 0x1bb4ff2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d14ff30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7654ff34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9894ff36, SUC_lexicon_tags_12 },
    { 0xf654ff36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3d4ff39, SUC_lexicon_tags_17 },
    { 0xd9b4ff3a, SUC_lexicon_tags_15 },
    { 0x76d4ff3b, SUC_lexicon_tags_27 },
    { 0x1694ff3c, SUC_lexicon_tags_11 },
    { 0x25b4ff3d, SUC_lexicon_tags_65 },
    { 0x9394ff3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd994ff41, SUC_lexicon_tags_15 },
    { 0x79d4ff41, SUC_lexicon_tags_2 },
    { 0xed94ff43, SUC_lexicon_tags_37 },
    { 0x5434ff42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d4ff48, SUC_lexicon_tags_22 },
    { 0xd334ff49, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d4ff4c, SUC_lexicon_tags_15 },
    { 0x1f14ff4c, SUC_lexicon_tags_27 },
    { 0x92d4ff4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc14ff53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdcb4ff55, SUC_lexicon_tags_12 },
    { 0xa9b4ff56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f4ff5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x834ff5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9014ff63, SUC_lexicon_tags_13 },
    { 0x7f4ff63, SUC_lexicon_tags_63 },
    { 0x2d54ff65, SUC_lexicon_tags_17 },
    { 0xa714ff66, SUC_lexicon_tags_14 },
    { 0x16f4ff67, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x75b4ff69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa314ff6b, SUC_lexicon_tags_28 },
    { 0x71d4ff6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9614ff6e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd254ff71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x234ff73, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc614ff7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf74ff84, SUC_lexicon_tags_19 },
    { 0x5b14ff85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f4ff88, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf4ff8b, SUC_lexicon_tags_9 },
    { 0xb2f4ff8c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2414ff8f, SUC_lexicon_tags_1 },
    { 0x6f34ff90, SUC_lexicon_tags_11 },
    { 0xd314ff91, SUC_lexicon_tags_17 },
    { 0x46f4ff90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f4ff97, SUC_lexicon_tags_15 },
    { 0xd454ff97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd4ff9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x92b4ff9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7dd4ff9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a54ffa1, SUC_lexicon_tags_31 },
    { 0x1894ffa2, SUC_lexicon_tags_2 },
    { 0x9e34ffa1, SUC_lexicon_tags_43 },
    { 0x4d54ffa1, SUC_lexicon_tags_17 },
    { 0x42b4ffa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf4ffac, SUC_lexicon_tags_65 },
    { 0xed54ffad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e34ffb0, SUC_lexicon_tags_12 },
    { 0x19b4ffb1, SUC_lexicon_tags_26 },
    { 0xab34ffb1, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x6754ffb4, SUC_lexicon_tags_27 },
    { 0xeb74ffb4, SUC_lexicon_tags_33 },
    { 0x33b4ffb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe94ffba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe874ffbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a14ffbf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4614ffc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75f4ffc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36f4ffcb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7ab4ffcd, SUC_lexicon_tags_25 },
    { 0x29b4ffcd, SUC_lexicon_tags_25 },
    { 0x6874ffce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc854ffd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e54ffd8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x93f4ffda, SUC_lexicon_tags_11 },
    { 0xfcd4ffda, SUC_lexicon_tags_45 },
    { 0xa6d4ffdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe814ffde, SUC_lexicon_tags_37 },
    { 0xcd94ffdf, SUC_lexicon_tags_11 },
    { 0xbcf4ffe0, SUC_lexicon_tags_16 },
    { 0xfdf4ffe1, SUC_lexicon_tags_29 },
    { 0x7634ffde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ab4ffe4, SUC_lexicon_tags_60 },
    { 0x8f54ffe5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfcd4ffe7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2174ffe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5094ffec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9814ffee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd74fff3, SUC_lexicon_tags_37 },
    { 0xcd4fff3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe34fff8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb4fffd, SUC_lexicon_tags_11 },
    { 0x24f4fffe, SUC_lexicon_tags_26 },
    { 0x8dd4fffe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd750001, SUC_lexicon_tags_8 },
    { 0xa9550001, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b150005, SUC_lexicon_tags_15 },
    { 0x50950006, SUC_lexicon_tags_2 },
    { 0xebb50007, SUC_lexicon_tags_46 },
    { 0xb5b50006, SUC_lexicon_tags_35 },
    { 0x4df50008, SUC_lexicon_tags_8 },
    { 0xf5d50008, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a35000c, SUC_lexicon_tags_1076 },
    { 0xde75000d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d50011, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a950015, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36150018, SUC_lexicon_tags_7 },
    { 0xf2350019, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa915001f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c950022, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2550025, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f5002c, SUC_lexicon_tags_23 },
    { 0xfaf5002d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf50030, SUC_lexicon_tags_12 },
    { 0x78f50031, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x41550033, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20b50035, SUC_lexicon_tags_17 },
    { 0xc3f50036, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4150039, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb95003b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf695003d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d50042, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b5004e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9950051, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a15005a, SUC_lexicon_tags_21 },
    { 0x3b95005b, SUC_lexicon_tags_39 },
    { 0x1175005a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1b5005e, SUC_lexicon_tags_33 },
    { 0x9a95005f, SUC_lexicon_tags_17 },
    { 0x90950060, SUC_lexicon_tags_11 },
    { 0x18f5005f, SUC_lexicon_tags_12 },
    { 0xc1b50062, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x16150064, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab50067, SUC_lexicon_tags_49 },
    { 0xe2150067, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a35006c, SUC_lexicon_tags_37 },
    { 0xbf35006d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4150070, SUC_lexicon_tags_49 },
    { 0xf2950070, SUC_lexicon_tags_15 },
    { 0x4e550070, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b50077, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce15007b, SUC_lexicon_tags_1323 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf315007f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47350081, SUC_lexicon_tags_7 },
    { 0x20b50081, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4cf50084, SUC_lexicon_tags_11 },
    { 0x54550084, SUC_lexicon_tags_0 },
    { 0x7150085, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xd7f50088, SUC_lexicon_tags_46 },
    { 0xe3d50089, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6155008d, SUC_lexicon_tags_1 },
    { 0xb0d5008d, SUC_lexicon_tags_29 },
    { 0x98d5008d, SUC_lexicon_tags_68 },
    { 0x55950090, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x76b50092, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e550095, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf5b50097, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5350099, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe535009b, SUC_lexicon_tags_37 },
    { 0x8475009c, SUC_lexicon_tags_37 },
    { 0x6d35009c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d500a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x303500a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xec5500a7, SUC_lexicon_tags_25 },
    { 0xfff500a8, SUC_lexicon_tags_11 },
    { 0x18f500a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3500ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f500b0, SUC_lexicon_tags_66 },
    { 0x13b500b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447500b7, SUC_lexicon_tags_39 },
    { 0xca7500b7, SUC_lexicon_tags_37 },
    { 0xcd1500b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c7500bb, SUC_lexicon_tags_12 },
    { 0xde3500bc, SUC_lexicon_tags_2 },
    { 0x2c3500bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xddf500bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x613500c1, SUC_lexicon_tags_12 },
    { 0xb85500c1, SUC_lexicon_tags_37 },
    { 0x753500c3, SUC_lexicon_tags_2 },
    { 0xd17500c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9500c8, SUC_lexicon_tags_19 },
    { 0x95b500c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x169500cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1500ce, SUC_lexicon_tags_25 },
    { 0xaa5500cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25500d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd500da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x821500dc, SUC_lexicon_tags_37 },
    { 0xf81500dc, SUC_lexicon_tags_17 },
    { 0x613500de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff500e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895500ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x223500ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd99500ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4e7500f0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d500f3, SUC_lexicon_tags_146 },
    { 0x5f9500f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b500f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1500fd, SUC_lexicon_tags_29 },
    { 0xb4b500fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25150102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f50104, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c35010c, SUC_lexicon_tags_15 },
    { 0x7795010c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x22950110, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68150114, SUC_lexicon_tags_43 },
    { 0x6f350115, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b50119, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e15011d, SUC_lexicon_tags_25 },
    { 0x5f15011e, SUC_lexicon_tags_43 },
    { 0x97b5011e, SUC_lexicon_tags_2 },
    { 0x3d35011e, SUC_lexicon_tags_77 },
    { 0x9f750121, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2150123, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4550126, SUC_lexicon_tags_1 },
    { 0x80150127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f35012a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xebd5012c, SUC_lexicon_tags_28 },
    { 0xabf5012c, SUC_lexicon_tags_23 },
    { 0x5915012e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc3b50130, SUC_lexicon_tags_47 },
    { 0x40550131, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f50134, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42950137, SUC_lexicon_tags_87 },
    { 0xc8950137, SUC_lexicon_tags_63 },
    { 0x1e950139, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed50141, SUC_lexicon_tags_12 },
    { 0x73f50142, SUC_lexicon_tags_37 },
    { 0xe3350143, SUC_lexicon_tags_26 },
    { 0xe950144, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d50147, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa6550149, SUC_lexicon_tags_38 },
    { 0x6c35014a, SUC_lexicon_tags_63 },
    { 0x85b5014a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5995014f, SUC_lexicon_tags_68 },
    { 0x1e950150, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47350152, SUC_lexicon_tags_7 },
    { 0x36d50153, SUC_lexicon_tags_12 },
    { 0x7d550153, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3750157, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe8750159, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf5015c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6015016b, SUC_lexicon_tags_12 },
    { 0xabb5016b, SUC_lexicon_tags_55 },
    { 0x6015016b, SUC_lexicon_tags_27 },
    { 0xfd75016b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b350170, SUC_lexicon_tags_1 },
    { 0xf0150171, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6550175, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb350177, SUC_lexicon_tags_7 },
    { 0xc9d50178, SUC_lexicon_tags_17 },
    { 0xbcf50179, SUC_lexicon_tags_34 },
    { 0x39d5017a, SUC_lexicon_tags_46 },
    { 0x98d5017a, SUC_lexicon_tags_28 },
    { 0x8595017c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4550180, SUC_lexicon_tags_33 },
    { 0xac950181, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5550183, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x72f50185, SUC_lexicon_tags_38 },
    { 0x5bd50185, SUC_lexicon_tags_25 },
    { 0xabf50186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b5018b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe575018f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8750192, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c350194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c350198, SUC_lexicon_tags_37 },
    { 0xd5550199, SUC_lexicon_tags_12 },
    { 0xdd550199, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9f5019c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4b5019e, SUC_lexicon_tags_48 },
    { 0xb2f5019e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f501a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x775501a8, SUC_lexicon_tags_43 },
    { 0x477501a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x40d501ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x73d501ad, SUC_lexicon_tags_63 },
    { 0xb11501ae, SUC_lexicon_tags_14 },
    { 0x9cf501ae, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3501b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169501bc, SUC_lexicon_tags_12 },
    { 0xcc5501bd, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3501c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75501c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5501c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81501cf, SUC_lexicon_tags_15 },
    { 0xf53501cf, SUC_lexicon_tags_25 },
    { 0x957501d0, SUC_lexicon_tags_42 },
    { 0x3e9501d1, SUC_lexicon_tags_17 },
    { 0x13d501d2, SUC_lexicon_tags_25 },
    { 0x20d501d4, SUC_lexicon_tags_47 },
    { 0x785501d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529501da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd45501dc, SUC_lexicon_tags_15 },
    { 0x29501dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d501e1, SUC_lexicon_tags_43 },
    { 0x8bd501e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x707501e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ed501e6, SUC_lexicon_tags_6 },
    { 0x345501e6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8a1501e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa1f501eb, SUC_lexicon_tags_7 },
    { 0x93b501ec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x41b501ee, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03501f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443501f7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f501fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d3501fc, SUC_lexicon_tags_2 },
    { 0x6dd501fd, SUC_lexicon_tags_32 },
    { 0x9ed501fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd350207, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xcdf50209, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe495020b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64750216, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f50219, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7255021d, SUC_lexicon_tags_60 },
    { 0x9f95021e, SUC_lexicon_tags_60 },
    { 0xe895021f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b50225, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855022c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa515022f, SUC_lexicon_tags_10 },
    { 0x5f95022f, SUC_lexicon_tags_23 },
    { 0xab50231, SUC_lexicon_tags_8 },
    { 0xcad50232, SUC_lexicon_tags_15 },
    { 0x91b50231, SUC_lexicon_tags_14 },
    { 0x71950234, SUC_lexicon_tags_11 },
    { 0x44750235, SUC_lexicon_tags_150 },
    { 0x3dd50236, SUC_lexicon_tags_118 },
    { 0xf3f50237, SUC_lexicon_tags_37 },
    { 0x6db50234, SUC_lexicon_tags_7 },
    { 0xca150239, SUC_lexicon_tags_26 },
    { 0xe055023a, SUC_lexicon_tags_15 },
    { 0x9cb5023b, SUC_lexicon_tags_65 },
    { 0xe975023c, SUC_lexicon_tags_15 },
    { 0x7135023b, SUC_lexicon_tags_56 },
    { 0x5c75023e, SUC_lexicon_tags_77 },
    { 0x3835023b, SUC_lexicon_tags_12 },
    { 0xa4150240, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe375024a, SUC_lexicon_tags_0 },
    { 0x7455024a, SUC_lexicon_tags_11 },
    { 0x49d5024b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bd5024e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefb50250, SUC_lexicon_tags_26 },
    { 0x5d50251, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64750258, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b35025a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a15025d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6cb5025f, SUC_lexicon_tags_47 },
    { 0xb4d50260, SUC_lexicon_tags_47 },
    { 0xbdb5025f, SUC_lexicon_tags_1 },
    { 0xa4f50262, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3350264, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60950267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d350269, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f95026d, SUC_lexicon_tags_49 },
    { 0xed5026e, SUC_lexicon_tags_12 },
    { 0x3ed5026f, SUC_lexicon_tags_17 },
    { 0xac350270, SUC_lexicon_tags_11 },
    { 0xa3d50271, SUC_lexicon_tags_49 },
    { 0xa5b50270, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34350277, SUC_lexicon_tags_17 },
    { 0x6f350277, SUC_lexicon_tags_12 },
    { 0xd3550279, SUC_lexicon_tags_61 },
    { 0xbff50277, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xba95027d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f95027f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e350281, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b50284, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99950286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64550289, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8bf5028b, SUC_lexicon_tags_33 },
    { 0xb455028b, SUC_lexicon_tags_23 },
    { 0x88b5028b, SUC_lexicon_tags_1 },
    { 0xf975028b, SUC_lexicon_tags_17 },
    { 0x8c15028f, SUC_lexicon_tags_5 },
    { 0x36150290, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf150294, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72950296, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f502a1, SUC_lexicon_tags_15 },
    { 0xbcd502a1, SUC_lexicon_tags_12 },
    { 0x165502a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb502aa, SUC_lexicon_tags_33 },
    { 0x43502aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74f502ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15502b3, SUC_lexicon_tags_27 },
    { 0x9f9502b4, SUC_lexicon_tags_0 },
    { 0x7d502b3, SUC_lexicon_tags_38 },
    { 0xd61502b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55502bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab502be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681502c5, SUC_lexicon_tags_61 },
    { 0x6e3502c6, SUC_lexicon_tags_37 },
    { 0x389502c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x139502ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c1502cc, SUC_lexicon_tags_25 },
    { 0xddf502cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x631502cf, SUC_lexicon_tags_15 },
    { 0x62d502d0, SUC_lexicon_tags_1 },
    { 0x96b502d1, SUC_lexicon_tags_46 },
    { 0x5cd502d2, SUC_lexicon_tags_27 },
    { 0xc3502d3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7502e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73502e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9502ec, SUC_lexicon_tags_37 },
    { 0x757502ed, SUC_lexicon_tags_27 },
    { 0x5ff502ee, SUC_lexicon_tags_7 },
    { 0xef5502ee, SUC_lexicon_tags_26 },
    { 0xf4d502f0, SUC_lexicon_tags_14 },
    { 0xb83502ed, SUC_lexicon_tags_118 },
    { 0x707502ee, SUC_lexicon_tags_37 },
    { 0x127502f3, SUC_lexicon_tags_25 },
    { 0x787502f0, SUC_lexicon_tags_12 },
    { 0xf23502ed, SUC_lexicon_tags_31 },
    { 0xea9502ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x463502f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5502fe, SUC_lexicon_tags_60 },
    { 0xd83502ff, SUC_lexicon_tags_49 },
    { 0x8cd502ff, SUC_lexicon_tags_45 },
    { 0xaf50300, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9150304, SUC_lexicon_tags_25 },
    { 0xf8d50304, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x73b50307, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b150309, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe295030d, SUC_lexicon_tags_19 },
    { 0x59b5030e, SUC_lexicon_tags_33 },
    { 0x6e95030f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb950311, SUC_lexicon_tags_33 },
    { 0x1d550312, SUC_lexicon_tags_30 },
    { 0x1e750311, SUC_lexicon_tags_60 },
    { 0x47350314, SUC_lexicon_tags_12 },
    { 0x79550315, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc350322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f750324, SUC_lexicon_tags_25 },
    { 0x45f50324, SUC_lexicon_tags_37 },
    { 0x21750324, SUC_lexicon_tags_31 },
    { 0x3bd50327, SUC_lexicon_tags_102 },
    { 0x2df50328, SUC_lexicon_tags_0 },
    { 0xf7150329, SUC_lexicon_tags_25 },
    { 0xf8b5032a, SUC_lexicon_tags_45 },
    { 0x4e350329, SUC_lexicon_tags_47 },
    { 0x6595032c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b350331, SUC_lexicon_tags_27 },
    { 0xb0f50332, SUC_lexicon_tags_28 },
    { 0xdfd50333, SUC_lexicon_tags_9 },
    { 0xbaf50334, SUC_lexicon_tags_7 },
    { 0xb1750335, SUC_lexicon_tags_1 },
    { 0x1c950332, SUC_lexicon_tags_37 },
    { 0x37f50337, SUC_lexicon_tags_46 },
    { 0x5b50331, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc835033c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9d5033e, SUC_lexicon_tags_1 },
    { 0x2955033e, SUC_lexicon_tags_46 },
    { 0x2ad5033e, SUC_lexicon_tags_25 },
    { 0x6fb5033f, SUC_lexicon_tags_25 },
    { 0xf935033e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab50347, SUC_lexicon_tags_21 },
    { 0xd2f50347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f150355, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b50359, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d50365, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8150369, SUC_lexicon_tags_11 },
    { 0x1f55036a, SUC_lexicon_tags_8 },
    { 0x9375036b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f50370, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62150372, SUC_lexicon_tags_863 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7950376, SUC_lexicon_tags_36 },
    { 0xbff50376, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa550379, SUC_lexicon_tags_15 },
    { 0xeab5037a, SUC_lexicon_tags_29 },
    { 0xc9f5037b, SUC_lexicon_tags_31 },
    { 0x75950379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b5037e, SUC_lexicon_tags_12 },
    { 0x99b5037f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d50382, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf50385, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255038a, SUC_lexicon_tags_19 },
    { 0x4b55038b, SUC_lexicon_tags_45 },
    { 0x5e35038c, SUC_lexicon_tags_34 },
    { 0x5835038d, SUC_lexicon_tags_188 },
    { 0xd115038c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8950392, SUC_lexicon_tags_29 },
    { 0xa0350393, SUC_lexicon_tags_27 },
    { 0xc2f50392, SUC_lexicon_tags_15 },
    { 0x34d50394, SUC_lexicon_tags_10 },
    { 0x6f950396, SUC_lexicon_tags_27 },
    { 0xb950395, SUC_lexicon_tags_60 },
    { 0xb4350394, SUC_lexicon_tags_23 },
    { 0x55f50395, SUC_lexicon_tags_25 },
    { 0xfe15039a, SUC_lexicon_tags_31 },
    { 0xdad5039b, SUC_lexicon_tags_57 },
    { 0xa675039c, SUC_lexicon_tags_23 },
    { 0x1155039d, SUC_lexicon_tags_39 },
    { 0x9bd5039e, SUC_lexicon_tags_25 },
    { 0xa235039b, SUC_lexicon_tags_11 },
    { 0xe275039b, SUC_lexicon_tags_28 },
    { 0xf3d50399, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23503a6, SUC_lexicon_tags_25 },
    { 0x845503a7, SUC_lexicon_tags_60 },
    { 0xe7503a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb65503aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xedf503ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf503af, SUC_lexicon_tags_68 },
    { 0xaf5503b0, SUC_lexicon_tags_17 },
    { 0xabd503b1, SUC_lexicon_tags_17 },
    { 0xafb503b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd503ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1503c0, SUC_lexicon_tags_11 },
    { 0xd3f503c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x393503c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35503c7, SUC_lexicon_tags_1 },
    { 0xa3b503c8, SUC_lexicon_tags_2 },
    { 0xb33503c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843503d0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435503d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf503d8, SUC_lexicon_tags_15 },
    { 0x829503d9, SUC_lexicon_tags_29 },
    { 0x40b503da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x417503dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9503df, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5b1503e1, SUC_lexicon_tags_11 },
    { 0x319503e2, SUC_lexicon_tags_43 },
    { 0xf59503e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b503e6, SUC_lexicon_tags_47 },
    { 0xd5503e7, SUC_lexicon_tags_2 },
    { 0xe81503e7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb503ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5503f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf81503fa, SUC_lexicon_tags_15 },
    { 0xf9b503fa, SUC_lexicon_tags_60 },
    { 0x6fd503fc, SUC_lexicon_tags_47 },
    { 0xc23503fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13350401, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2550406, SUC_lexicon_tags_10 },
    { 0x2dd50406, SUC_lexicon_tags_1 },
    { 0xdbd50408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab750410, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x99d50412, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f950414, SUC_lexicon_tags_94 },
    { 0x6f150415, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22d50417, SUC_lexicon_tags_49 },
    { 0x4ff50418, SUC_lexicon_tags_14 },
    { 0x24550418, SUC_lexicon_tags_25 },
    { 0x7c95041a, SUC_lexicon_tags_60 },
    { 0xd315041a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a15041d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c750423, SUC_lexicon_tags_37 },
    { 0x53150423, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b50429, SUC_lexicon_tags_25 },
    { 0xdeb50429, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f5042e, SUC_lexicon_tags_1 },
    { 0x5755042f, SUC_lexicon_tags_17 },
    { 0x6995042f, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91750436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92350442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf550449, SUC_lexicon_tags_11 },
    { 0xa5d5044a, SUC_lexicon_tags_23 },
    { 0xcbf5044b, SUC_lexicon_tags_25 },
    { 0x8bd5044c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31550451, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4d50453, SUC_lexicon_tags_37 },
    { 0xf9f50454, SUC_lexicon_tags_19 },
    { 0xed50455, SUC_lexicon_tags_27 },
    { 0x38950456, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd8f50458, SUC_lexicon_tags_10 },
    { 0xce950459, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0d5045b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f5045e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67750463, SUC_lexicon_tags_15 },
    { 0x74150464, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xead50466, SUC_lexicon_tags_17 },
    { 0x3a350466, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95046b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b75046f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf50474, SUC_lexicon_tags_293 },
    { 0xb9f50474, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2750478, SUC_lexicon_tags_9 },
    { 0xc7750478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8d5047b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe815047d, SUC_lexicon_tags_15 },
    { 0xf955047e, SUC_lexicon_tags_25 },
    { 0x6ab5047f, SUC_lexicon_tags_25 },
    { 0x8cf50480, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b5048d, SUC_lexicon_tags_8 },
    { 0xa0d5048e, SUC_lexicon_tags_60 },
    { 0x9c55048d, SUC_lexicon_tags_45 },
    { 0x1e55048e, SUC_lexicon_tags_37 },
    { 0x24350491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf550497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x750499, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x175049c, SUC_lexicon_tags_37 },
    { 0x6d55049c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9504a0, SUC_lexicon_tags_2 },
    { 0xbc3504a1, SUC_lexicon_tags_2 },
    { 0xa4d504a2, SUC_lexicon_tags_43 },
    { 0x72b504a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a3504a5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb504ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x29504ad, SUC_lexicon_tags_60 },
    { 0xb11504ae, SUC_lexicon_tags_7 },
    { 0xb3d504af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917504b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe6d504b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x245504b7, SUC_lexicon_tags_7 },
    { 0x923504b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb59504ba, SUC_lexicon_tags_14 },
    { 0x7c9504bb, SUC_lexicon_tags_17 },
    { 0x649504bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df504c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab504c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5504cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf23504d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a1504d3, SUC_lexicon_tags_30 },
    { 0xa9504d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x659504d6, SUC_lexicon_tags_10 },
    { 0x431504d7, SUC_lexicon_tags_14 },
    { 0x861504d8, SUC_lexicon_tags_25 },
    { 0xd31504d8, SUC_lexicon_tags_31 },
    { 0xd05504d9, SUC_lexicon_tags_11 },
    { 0x14b504da, SUC_lexicon_tags_6 },
    { 0x7c3504d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5504df, SUC_lexicon_tags_68 },
    { 0xd95504e0, SUC_lexicon_tags_15 },
    { 0x18b504e1, SUC_lexicon_tags_11 },
    { 0x349504e2, SUC_lexicon_tags_15 },
    { 0xa5504e0, SUC_lexicon_tags_66 },
    { 0x10d504e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d504ea, SUC_lexicon_tags_38 },
    { 0x4a3504ea, SUC_lexicon_tags_7 },
    { 0xc8b504ec, SUC_lexicon_tags_14 },
    { 0x94d504ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05504f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b7504f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1504f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5550500, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1950503, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd395050b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7855050f, SUC_lexicon_tags_27 },
    { 0x18550510, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d50513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc950515, SUC_lexicon_tags_12 },
    { 0x49d50516, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87950519, SUC_lexicon_tags_11 },
    { 0x9c55051a, SUC_lexicon_tags_11 },
    { 0x95f5051a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3755051e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b550524, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf855052c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d50530, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x76750533, SUC_lexicon_tags_37 },
    { 0x55f50533, SUC_lexicon_tags_11 },
    { 0xc1f50535, SUC_lexicon_tags_46 },
    { 0x1ad50535, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x23350539, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b5053c, SUC_lexicon_tags_39 },
    { 0x7155053d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d50544, SUC_lexicon_tags_7 },
    { 0x71350545, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc350548, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb50551, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe150558, SUC_lexicon_tags_7 },
    { 0x1ff50559, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f5055f, SUC_lexicon_tags_15 },
    { 0x2235055f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f50567, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ed50569, SUC_lexicon_tags_63 },
    { 0x11750569, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb735056c, SUC_lexicon_tags_57 },
    { 0x5f35056d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af50571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc950574, SUC_lexicon_tags_34 },
    { 0xaa150575, SUC_lexicon_tags_25 },
    { 0xd4f50575, SUC_lexicon_tags_7 },
    { 0x5f750577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa195057a, SUC_lexicon_tags_2 },
    { 0x9cd5057a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf50580, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f50583, SUC_lexicon_tags_31 },
    { 0xcef50584, SUC_lexicon_tags_11 },
    { 0xe1b50585, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x10d50587, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b5058d, SUC_lexicon_tags_11 },
    { 0x9515058d, SUC_lexicon_tags_2 },
    { 0x5075058e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef50596, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1350598, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5715059a, SUC_lexicon_tags_2 },
    { 0x56b5059b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb505a1, SUC_lexicon_tags_27 },
    { 0xe01505a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5cb505a4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d505a8, SUC_lexicon_tags_12 },
    { 0xac5505a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5505ac, SUC_lexicon_tags_47 },
    { 0x75b505ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf505b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1505b5, SUC_lexicon_tags_14 },
    { 0x7b3505b6, SUC_lexicon_tags_14 },
    { 0xfa1505b7, SUC_lexicon_tags_11 },
    { 0xce5505b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25505bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x263505be, SUC_lexicon_tags_17 },
    { 0x65d505be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba1505c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12d505c3, SUC_lexicon_tags_1 },
    { 0x8fd505c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef505c8, SUC_lexicon_tags_66 },
    { 0x2d505c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xceb505cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35505cf, SUC_lexicon_tags_38 },
    { 0xc89505d0, SUC_lexicon_tags_27 },
    { 0x661505cf, SUC_lexicon_tags_14 },
    { 0xbe3505cf, SUC_lexicon_tags_23 },
    { 0x34f505d3, SUC_lexicon_tags_39 },
    { 0x181505cf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb505d8, SUC_lexicon_tags_11 },
    { 0xe8b505d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43505dd, SUC_lexicon_tags_49 },
    { 0xd79505de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff505e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x779505e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf505e8, SUC_lexicon_tags_1 },
    { 0xe6d505e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ab505eb, SUC_lexicon_tags_1 },
    { 0x2b1505ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d505ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6f505f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9505f3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7505f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97505fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x177505fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5950602, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c150604, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc150607, SUC_lexicon_tags_45 },
    { 0x2f750607, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4155060b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d50610, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdf50612, SUC_lexicon_tags_63 },
    { 0x2c750612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b50619, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b95061b, SUC_lexicon_tags_34 },
    { 0x4515061c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d50624, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f50628, SUC_lexicon_tags_37 },
    { 0x23150629, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d5062c, SUC_lexicon_tags_19 },
    { 0xbb95062c, SUC_lexicon_tags_14 },
    { 0xcf15062e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea750639, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc55063f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d50642, SUC_lexicon_tags_48 },
    { 0x74b50643, SUC_lexicon_tags_25 },
    { 0x47350643, SUC_lexicon_tags_33 },
    { 0x5950645, SUC_lexicon_tags_5 },
    { 0x63b50644, SUC_lexicon_tags_17 },
    { 0x83f50645, SUC_lexicon_tags_29 },
    { 0xdb550648, SUC_lexicon_tags_8 },
    { 0xef350649, SUC_lexicon_tags_3 },
    { 0x5f50647, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9055064e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x98350650, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe150654, SUC_lexicon_tags_489 },
    { 0x61550655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94b50657, SUC_lexicon_tags_87 },
    { 0xdcd50657, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa735065b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd95065d, SUC_lexicon_tags_21 },
    { 0xd1b5065e, SUC_lexicon_tags_47 },
    { 0x4b35065e, SUC_lexicon_tags_4 },
    { 0xa535065e, SUC_lexicon_tags_63 },
    { 0x52d5065e, SUC_lexicon_tags_0 },
    { 0x9ef50662, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9350664, SUC_lexicon_tags_1 },
    { 0x88750664, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f5066a, SUC_lexicon_tags_1 },
    { 0x4b15066a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95066d, SUC_lexicon_tags_37 },
    { 0x4935066d, SUC_lexicon_tags_1 },
    { 0x615066e, SUC_lexicon_tags_23 },
    { 0x3b550670, SUC_lexicon_tags_45 },
    { 0x70950671, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f50674, SUC_lexicon_tags_81 },
    { 0xf7350675, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31350678, SUC_lexicon_tags_33 },
    { 0x61350678, SUC_lexicon_tags_15 },
    { 0xf2750679, SUC_lexicon_tags_1 },
    { 0xcd55067b, SUC_lexicon_tags_7 },
    { 0x3c95067c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d5067f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x78550681, SUC_lexicon_tags_87 },
    { 0x5fb50681, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca150685, SUC_lexicon_tags_37 },
    { 0xc7b50686, SUC_lexicon_tags_27 },
    { 0xdf750687, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf5068b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd5068f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad750695, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2715069f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167506a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869506ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x603506af, SUC_lexicon_tags_19 },
    { 0xe05506b0, SUC_lexicon_tags_63 },
    { 0x69f506af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x671506b3, SUC_lexicon_tags_11 },
    { 0x26d506b4, SUC_lexicon_tags_37 },
    { 0xbb1506b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91506ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x277506bd, SUC_lexicon_tags_61 },
    { 0xaa7506bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405506c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f506cb, SUC_lexicon_tags_54 },
    { 0xe29506cb, SUC_lexicon_tags_30 },
    { 0xd4d506cd, SUC_lexicon_tags_27 },
    { 0xc3506cc, SUC_lexicon_tags_66 },
    { 0x3b506cf, SUC_lexicon_tags_9 },
    { 0x259506cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb506d8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487506dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd506e1, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d506e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xad506e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa53506ea, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f506f0, SUC_lexicon_tags_14 },
    { 0xb2d506f1, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf506f5, SUC_lexicon_tags_11 },
    { 0xb55506f5, SUC_lexicon_tags_22 },
    { 0xea5506f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b5506f9, SUC_lexicon_tags_46 },
    { 0x985506f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb506ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b50705, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7350708, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f5070c, SUC_lexicon_tags_47 },
    { 0xc7d5070d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc3d5070f, SUC_lexicon_tags_0 },
    { 0x37150710, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df50719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x675071b, SUC_lexicon_tags_16 },
    { 0x6e55071c, SUC_lexicon_tags_37 },
    { 0x67f5071d, SUC_lexicon_tags_0 },
    { 0xd95071c, SUC_lexicon_tags_37 },
    { 0xd1d5071f, SUC_lexicon_tags_1 },
    { 0x99150720, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a950729, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3375072b, SUC_lexicon_tags_1 },
    { 0xb0b5072c, SUC_lexicon_tags_19 },
    { 0xead5072d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa2b5072f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d50733, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbef50735, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f50738, SUC_lexicon_tags_11 },
    { 0x4db50738, SUC_lexicon_tags_37 },
    { 0xef50738, SUC_lexicon_tags_19 },
    { 0x2d15073b, SUC_lexicon_tags_26 },
    { 0x91d5073b, SUC_lexicon_tags_47 },
    { 0x67b5073b, SUC_lexicon_tags_17 },
    { 0x1d35073e, SUC_lexicon_tags_25 },
    { 0x4855073e, SUC_lexicon_tags_14 },
    { 0xcbf50740, SUC_lexicon_tags_26 },
    { 0xd55073a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc350752, SUC_lexicon_tags_7 },
    { 0xc5350753, SUC_lexicon_tags_43 },
    { 0x65750754, SUC_lexicon_tags_26 },
    { 0xd7750753, SUC_lexicon_tags_16 },
    { 0x3b50756, SUC_lexicon_tags_17 },
    { 0xccb50757, SUC_lexicon_tags_15 },
    { 0xd8750752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6835075c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5550760, SUC_lexicon_tags_27 },
    { 0x9ef50761, SUC_lexicon_tags_37 },
    { 0xbd750762, SUC_lexicon_tags_25 },
    { 0x2d550763, SUC_lexicon_tags_17 },
    { 0xc8b50763, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcbd50766, SUC_lexicon_tags_37 },
    { 0x35750767, SUC_lexicon_tags_84 },
    { 0xdc150768, SUC_lexicon_tags_17 },
    { 0x2150766, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d5076c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd50771, SUC_lexicon_tags_23 },
    { 0x78350772, SUC_lexicon_tags_37 },
    { 0x3af50771, SUC_lexicon_tags_33 },
    { 0x61f50774, SUC_lexicon_tags_25 },
    { 0xa4950775, SUC_lexicon_tags_6 },
    { 0xc750773, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x34b50778, SUC_lexicon_tags_1 },
    { 0x8ff50779, SUC_lexicon_tags_25 },
    { 0x8b15077a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b550782, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x34d50784, SUC_lexicon_tags_15 },
    { 0x97950785, SUC_lexicon_tags_264 },
    { 0x4a550784, SUC_lexicon_tags_26 },
    { 0xb7150787, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f5078a, SUC_lexicon_tags_39 },
    { 0xce15078a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d5078f, SUC_lexicon_tags_60 },
    { 0x7835078f, SUC_lexicon_tags_87 },
    { 0x67350791, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbc950793, SUC_lexicon_tags_57 },
    { 0xaaf50793, SUC_lexicon_tags_28 },
    { 0x85150795, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94f50797, SUC_lexicon_tags_87 },
    { 0xa4b50797, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d507a0, SUC_lexicon_tags_12 },
    { 0xdb507a1, SUC_lexicon_tags_158 },
    { 0xa01507a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d7507a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x877507a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67507ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x641507ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x341507af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453507b4, SUC_lexicon_tags_61 },
    { 0x4af507b5, SUC_lexicon_tags_8 },
    { 0x543507b6, SUC_lexicon_tags_0 },
    { 0xfe7507b5, SUC_lexicon_tags_17 },
    { 0xa21507b5, SUC_lexicon_tags_17 },
    { 0x4eb507b8, SUC_lexicon_tags_33 },
    { 0x3f9507b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57507bf, SUC_lexicon_tags_27 },
    { 0x55f507c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b507ca, SUC_lexicon_tags_42 },
    { 0xdb3507cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33507cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5c1507d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3507d7, SUC_lexicon_tags_37 },
    { 0x29b507d7, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889507dd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b507e2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x441507e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb507e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db507ec, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31507f1, SUC_lexicon_tags_10 },
    { 0x739507f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09507f7, SUC_lexicon_tags_27 },
    { 0x2eb507f7, SUC_lexicon_tags_47 },
    { 0xdf507f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8350801, SUC_lexicon_tags_25 },
    { 0xa9d50802, SUC_lexicon_tags_3 },
    { 0x1e350802, SUC_lexicon_tags_12 },
    { 0xf9550804, SUC_lexicon_tags_8 },
    { 0xb7150805, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf50809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6235080e, SUC_lexicon_tags_15 },
    { 0xf0d5080f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea950814, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xe5b50816, SUC_lexicon_tags_17 },
    { 0x85d50817, SUC_lexicon_tags_37 },
    { 0x95b50818, SUC_lexicon_tags_1 },
    { 0xd4750817, SUC_lexicon_tags_17 },
    { 0x9995081a, SUC_lexicon_tags_82 },
    { 0xe235081a, SUC_lexicon_tags_17 },
    { 0xbb95081c, SUC_lexicon_tags_14 },
    { 0x68350818, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5cd5081f, SUC_lexicon_tags_25 },
    { 0xf2d50820, SUC_lexicon_tags_15 },
    { 0xa7550821, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b50825, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b550828, SUC_lexicon_tags_87 },
    { 0x27350829, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab55082e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af50836, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3c750838, SUC_lexicon_tags_18 },
    { 0x83150839, SUC_lexicon_tags_17 },
    { 0xe4750839, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33d5083c, SUC_lexicon_tags_3 },
    { 0x3cd5083d, SUC_lexicon_tags_108 },
    { 0x98d5083e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6350841, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b50845, SUC_lexicon_tags_12 },
    { 0x8150845, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b75084a, SUC_lexicon_tags_7 },
    { 0xd3f5084b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4bf5084d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80150851, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d350854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b350858, SUC_lexicon_tags_1 },
    { 0x39f50858, SUC_lexicon_tags_15 },
    { 0xa2150859, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d50860, SUC_lexicon_tags_7 },
    { 0x28750861, SUC_lexicon_tags_17 },
    { 0x89950860, SUC_lexicon_tags_14 },
    { 0xe8750861, SUC_lexicon_tags_7 },
    { 0x2b750861, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5350866, SUC_lexicon_tags_15 },
    { 0xc2750867, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd955086a, SUC_lexicon_tags_2 },
    { 0x54f5086b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e350871, SUC_lexicon_tags_17 },
    { 0x6d150872, SUC_lexicon_tags_46 },
    { 0x29350872, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc1550875, SUC_lexicon_tags_11 },
    { 0x92150875, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf15087d, SUC_lexicon_tags_12 },
    { 0x8275087e, SUC_lexicon_tags_48 },
    { 0x40d5087d, SUC_lexicon_tags_26 },
    { 0x9495087e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d50885, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x67d50887, SUC_lexicon_tags_37 },
    { 0xea750887, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5775088b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9115088d, SUC_lexicon_tags_12 },
    { 0xb0f5088e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18150893, SUC_lexicon_tags_14 },
    { 0x4b550894, SUC_lexicon_tags_47 },
    { 0xda950894, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe1350897, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8635089f, SUC_lexicon_tags_12 },
    { 0xd4f5089f, SUC_lexicon_tags_11 },
    { 0x4f3508a1, SUC_lexicon_tags_25 },
    { 0x9415089f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb508a6, SUC_lexicon_tags_45 },
    { 0xf2b508a6, SUC_lexicon_tags_29 },
    { 0xe2d508a7, SUC_lexicon_tags_14 },
    { 0x7fb508a9, SUC_lexicon_tags_11 },
    { 0x9d9508a9, SUC_lexicon_tags_15 },
    { 0xb2d508ab, SUC_lexicon_tags_7 },
    { 0x9a5508ac, SUC_lexicon_tags_1 },
    { 0xa25508a8, SUC_lexicon_tags_11 },
    { 0x105508ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x173508b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f508b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c7508b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd71508ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d9508bc, SUC_lexicon_tags_2 },
    { 0x24f508bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf508c1, SUC_lexicon_tags_47 },
    { 0x7d1508c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb71508c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297508cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7508cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d508d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x913508d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97508df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3508e6, SUC_lexicon_tags_15 },
    { 0xaa3508e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f508f2, SUC_lexicon_tags_49 },
    { 0xffd508f3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x497508f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x45d508f7, SUC_lexicon_tags_12 },
    { 0xaef508f8, SUC_lexicon_tags_23 },
    { 0x5b3508f9, SUC_lexicon_tags_31 },
    { 0xea5508f8, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79508fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c150905, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf750907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a750909, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef5090d, SUC_lexicon_tags_38 },
    { 0x3055090d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52550910, SUC_lexicon_tags_7 },
    { 0x29750910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93f50913, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3150917, SUC_lexicon_tags_129 },
    { 0x14350918, SUC_lexicon_tags_32 },
    { 0x25350919, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa455091c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df50920, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f50925, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e950928, SUC_lexicon_tags_23 },
    { 0x9750929, SUC_lexicon_tags_17 },
    { 0xbeb50928, SUC_lexicon_tags_1 },
    { 0xfa150929, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2750934, SUC_lexicon_tags_17 },
    { 0x25350935, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd950939, SUC_lexicon_tags_11 },
    { 0x6b5093a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x335093d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d50940, SUC_lexicon_tags_61 },
    { 0x61b50940, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33b50943, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c35094a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62350950, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c550957, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b5095e, SUC_lexicon_tags_15 },
    { 0x2495095f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fd50961, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x95150963, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8150966, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb535096d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb195096f, SUC_lexicon_tags_14 },
    { 0x1b750970, SUC_lexicon_tags_63 },
    { 0x80350971, SUC_lexicon_tags_39 },
    { 0x14950972, SUC_lexicon_tags_47 },
    { 0x2b150972, SUC_lexicon_tags_14 },
    { 0xcb350970, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x63950977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeed50979, SUC_lexicon_tags_11 },
    { 0x1f950979, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd15097c, SUC_lexicon_tags_11 },
    { 0x5cd5097d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a750983, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x69950986, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x44d50988, SUC_lexicon_tags_14 },
    { 0x5c150989, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7ff5098b, SUC_lexicon_tags_60 },
    { 0xcff5098c, SUC_lexicon_tags_47 },
    { 0xaad5098c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f50990, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd150995, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde150999, SUC_lexicon_tags_11 },
    { 0x9495099a, SUC_lexicon_tags_35 },
    { 0x3035099b, SUC_lexicon_tags_25 },
    { 0x51d5099c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xebd5099e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x301509a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83509a5, SUC_lexicon_tags_9 },
    { 0x86f509a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb509ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7509ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b5509b0, SUC_lexicon_tags_12 },
    { 0x97b509b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df509b5, SUC_lexicon_tags_30 },
    { 0xc77509b5, SUC_lexicon_tags_63 },
    { 0xc65509b7, SUC_lexicon_tags_45 },
    { 0xf95509b8, SUC_lexicon_tags_2 },
    { 0x663509b8, SUC_lexicon_tags_11 },
    { 0x9b509b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c7509bc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xafd509be, SUC_lexicon_tags_37 },
    { 0xf73509be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3cb509c1, SUC_lexicon_tags_17 },
    { 0x32d509c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f509c5, SUC_lexicon_tags_15 },
    { 0x4c5509c6, SUC_lexicon_tags_56 },
    { 0xba7509c6, SUC_lexicon_tags_63 },
    { 0x2cf509c8, SUC_lexicon_tags_9 },
    { 0x36d509c8, SUC_lexicon_tags_30 },
    { 0xf7f509ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x443509cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81509cf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5509d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeeb509d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d509d7, SUC_lexicon_tags_1 },
    { 0x67509d7, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xd01509da, SUC_lexicon_tags_21 },
    { 0x5a3509db, SUC_lexicon_tags_34 },
    { 0x741509dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f509df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7509e3, SUC_lexicon_tags_12 },
    { 0xc9f509e3, SUC_lexicon_tags_21 },
    { 0xb9f509e5, SUC_lexicon_tags_1 },
    { 0x19b509e5, SUC_lexicon_tags_33 },
    { 0x8eb509e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xae9509e9, SUC_lexicon_tags_25 },
    { 0xa4b509ea, SUC_lexicon_tags_17 },
    { 0x3e9509ea, SUC_lexicon_tags_15 },
    { 0x3f7509eb, SUC_lexicon_tags_62 },
    { 0x2ab509ea, SUC_lexicon_tags_57 },
    { 0x2bf509ee, SUC_lexicon_tags_12 },
    { 0x741509ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x271509f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa5d509f3, SUC_lexicon_tags_49 },
    { 0xcd5509f3, SUC_lexicon_tags_25 },
    { 0x51509f3, SUC_lexicon_tags_17 },
    { 0x7b7509f6, SUC_lexicon_tags_25 },
    { 0x365509f3, SUC_lexicon_tags_17 },
    { 0xbe5509f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb4d509fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf85509fc, SUC_lexicon_tags_3 },
    { 0x359509fd, SUC_lexicon_tags_11 },
    { 0xeff509fe, SUC_lexicon_tags_47 },
    { 0xb09509fd, SUC_lexicon_tags_148 },
    { 0x9e9509ff, SUC_lexicon_tags_1 },
    { 0xfd150a01, SUC_lexicon_tags_17 },
    { 0xd3b50a02, SUC_lexicon_tags_56 },
    { 0x4fb509fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x79d50a06, SUC_lexicon_tags_38 },
    { 0x10550a07, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f50a0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf50a0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c750a15, SUC_lexicon_tags_30 },
    { 0xcdd50a15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48550a1d, SUC_lexicon_tags_37 },
    { 0x73950a1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5550a21, SUC_lexicon_tags_46 },
    { 0x1cb50a22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d50a27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec750a2a, SUC_lexicon_tags_10 },
    { 0xd2f50a2b, SUC_lexicon_tags_6 },
    { 0x97b50a2c, SUC_lexicon_tags_15 },
    { 0xe4550a2b, SUC_lexicon_tags_9 },
    { 0x91d50a2e, SUC_lexicon_tags_17 },
    { 0xdd50a2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8b50a31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b550a35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10150a38, SUC_lexicon_tags_68 },
    { 0x56950a39, SUC_lexicon_tags_35 },
    { 0x2bd50a39, SUC_lexicon_tags_2 },
    { 0x8f550a39, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaa150a3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81b50a3f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd5f50a41, SUC_lexicon_tags_32 },
    { 0x37b50a42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd50a46, SUC_lexicon_tags_13 },
    { 0xf9d50a47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed50a4a, SUC_lexicon_tags_11 },
    { 0x63750a4a, SUC_lexicon_tags_1 },
    { 0xd0f50a4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f50a50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83750a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78150a60, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x27750a62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4150a65, SUC_lexicon_tags_37 },
    { 0xdb950a65, SUC_lexicon_tags_15 },
    { 0x22b50a67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e950a6b, SUC_lexicon_tags_14 },
    { 0xad950a6c, SUC_lexicon_tags_55 },
    { 0x22150a6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed50a74, SUC_lexicon_tags_16 },
    { 0x57d50a74, SUC_lexicon_tags_2 },
    { 0x76750a76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef50a7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2150a80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9bd50a82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f550a84, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d550a8c, SUC_lexicon_tags_38 },
    { 0x53d50a8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb50a90, SUC_lexicon_tags_7 },
    { 0x5750a91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b50a94, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d950a9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93950a9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4350a9e, SUC_lexicon_tags_33 },
    { 0x5d150a9f, SUC_lexicon_tags_20 },
    { 0x58150a9f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff50aa8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc7f50aaa, SUC_lexicon_tags_65 },
    { 0xeab50aab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b750aad, SUC_lexicon_tags_77 },
    { 0xc8b50aae, SUC_lexicon_tags_17 },
    { 0x6f750aaf, SUC_lexicon_tags_1 },
    { 0xdb50aaf, SUC_lexicon_tags_2 },
    { 0xd3950aad, SUC_lexicon_tags_49 },
    { 0x93150aad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd50ab6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d50aba, SUC_lexicon_tags_77 },
    { 0x71d50aba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef50abe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b950ac1, SUC_lexicon_tags_85 },
    { 0x98950ac2, SUC_lexicon_tags_15 },
    { 0x4c150ac3, SUC_lexicon_tags_87 },
    { 0x59150ac2, SUC_lexicon_tags_59 },
    { 0xf3b50ac4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb50ac8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2750aca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8e550acc, SUC_lexicon_tags_45 },
    { 0x98550acc, SUC_lexicon_tags_23 },
    { 0x65950ace, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1150ad0, SUC_lexicon_tags_47 },
    { 0x2a350ad1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61550ada, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcff50adc, SUC_lexicon_tags_14 },
    { 0x66350add, SUC_lexicon_tags_57 },
    { 0x3950ade, SUC_lexicon_tags_31 },
    { 0xda750adf, SUC_lexicon_tags_26 },
    { 0xb5150ae0, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af50ae3, SUC_lexicon_tags_15 },
    { 0x9b350ae3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x52350ae6, SUC_lexicon_tags_47 },
    { 0x93750ae7, SUC_lexicon_tags_12 },
    { 0xb6750ae7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa150aea, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xddb50aec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf50aee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f50af2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xef950af5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab50af8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd550afc, SUC_lexicon_tags_5 },
    { 0x9b550afd, SUC_lexicon_tags_35 },
    { 0x9ef50afe, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x88f50b00, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db50b03, SUC_lexicon_tags_6 },
    { 0xa5750b03, SUC_lexicon_tags_29 },
    { 0x8ad50b05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47950b07, SUC_lexicon_tags_2 },
    { 0xdbf50b08, SUC_lexicon_tags_2 },
    { 0x9d150b07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff150b0b, SUC_lexicon_tags_8 },
    { 0x65f50b0b, SUC_lexicon_tags_12 },
    { 0xdb750b0b, SUC_lexicon_tags_1 },
    { 0x6b350b0e, SUC_lexicon_tags_29 },
    { 0xa2b50b0d, SUC_lexicon_tags_17 },
    { 0xb7d50b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6750b16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb50b1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9150b1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f550b21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18150b23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b750b26, SUC_lexicon_tags_7 },
    { 0x38b50b27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x72150b2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcbd50b2c, SUC_lexicon_tags_38 },
    { 0x83750b2d, SUC_lexicon_tags_47 },
    { 0xa6150b2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58550b30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb50b34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd2350b36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96750b38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30d50b3a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74750b3f, SUC_lexicon_tags_47 },
    { 0x81550b3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd150b46, SUC_lexicon_tags_23 },
    { 0xd950b47, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x69950b49, SUC_lexicon_tags_65 },
    { 0x5bf50b4a, SUC_lexicon_tags_10 },
    { 0x4d950b49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd950b4f, SUC_lexicon_tags_30 },
    { 0x63550b50, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x93750b52, SUC_lexicon_tags_15 },
    { 0x78150b53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf150b57, SUC_lexicon_tags_23 },
    { 0x44950b58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed750b5d, SUC_lexicon_tags_25 },
    { 0x3d550b5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c150b60, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11350b65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff350b69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d50b70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f50b73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86750b79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1950b7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b350b83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f50b86, SUC_lexicon_tags_45 },
    { 0xa8950b87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67350b8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c550b8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49350b8f, SUC_lexicon_tags_17 },
    { 0x51350b8f, SUC_lexicon_tags_29 },
    { 0x60d50b91, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xdd950b93, SUC_lexicon_tags_25 },
    { 0x7af50b94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35b50b96, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ab50b98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1e950b9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79950b9d, SUC_lexicon_tags_27 },
    { 0x3d950b9e, SUC_lexicon_tags_45 },
    { 0x97b50b9f, SUC_lexicon_tags_11 },
    { 0x27350b9d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e350ba6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d550ba9, SUC_lexicon_tags_43 },
    { 0xe2350ba9, SUC_lexicon_tags_55 },
    { 0x5ed50bab, SUC_lexicon_tags_2 },
    { 0xe5950ba9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f950bae, SUC_lexicon_tags_27 },
    { 0xe6350bae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3750bb3, SUC_lexicon_tags_43 },
    { 0x21550bb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9550bb9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9fd50bbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e750bbe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xccb50bc0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1950bc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8150bca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d50bd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8750bd2, SUC_lexicon_tags_25 },
    { 0x1d350bd3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74750bd9, SUC_lexicon_tags_3 },
    { 0x1eb50bda, SUC_lexicon_tags_15 },
    { 0xf9950bdb, SUC_lexicon_tags_45 },
    { 0x21d50bdc, SUC_lexicon_tags_229 },
    { 0xca950bda, SUC_lexicon_tags_15 },
    { 0x93550bde, SUC_lexicon_tags_37 },
    { 0x2ff50bdf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2950be6, SUC_lexicon_tags_31 },
    { 0xe6150be6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38750beb, SUC_lexicon_tags_2 },
    { 0x7f50bec, SUC_lexicon_tags_14 },
    { 0xba350bed, SUC_lexicon_tags_15 },
    { 0xd3550bee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b50bf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b150bf5, SUC_lexicon_tags_17 },
    { 0xf750bf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef50bf9, SUC_lexicon_tags_37 },
    { 0xb1350bfa, SUC_lexicon_tags_4 },
    { 0x28b50bfb, SUC_lexicon_tags_32 },
    { 0x7e550bf9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3750c01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe750c04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4350c0a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b550c0e, SUC_lexicon_tags_68 },
    { 0x10b50c0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d50c15, SUC_lexicon_tags_19 },
    { 0x54150c15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f50c1a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe2550c1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef50c20, SUC_lexicon_tags_31 },
    { 0xadd50c21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f50c25, SUC_lexicon_tags_1 },
    { 0x51150c25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b50c2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8cb50c2d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0150c34, SUC_lexicon_tags_26 },
    { 0x9e950c35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cf50c37, SUC_lexicon_tags_8 },
    { 0x41750c38, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99750c3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e550c3e, SUC_lexicon_tags_15 },
    { 0x9750c3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd750c43, SUC_lexicon_tags_14 },
    { 0x6ef50c44, SUC_lexicon_tags_1 },
    { 0x58950c44, SUC_lexicon_tags_7 },
    { 0xf3950c46, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92350c4a, SUC_lexicon_tags_25 },
    { 0x2c950c4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd8d50c4d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d50c55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0350c58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe550c5b, SUC_lexicon_tags_63 },
    { 0x57f50c5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81d50c5e, SUC_lexicon_tags_25 },
    { 0xdef50c5f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b50c62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd50c65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b50c68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3950c6e, SUC_lexicon_tags_28 },
    { 0x3a950c6f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x60b50c71, SUC_lexicon_tags_19 },
    { 0x6c150c71, SUC_lexicon_tags_17 },
    { 0x34550c73, SUC_lexicon_tags_152 },
    { 0xd4950c74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93f50c76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb50c7c, SUC_lexicon_tags_68 },
    { 0x2af50c7d, SUC_lexicon_tags_148 },
    { 0xced50c7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc950c80, SUC_lexicon_tags_7 },
    { 0x10750c81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98950c83, SUC_lexicon_tags_607 },
    { 0xc6350c84, SUC_lexicon_tags_43 },
    { 0x7ad50c85, SUC_lexicon_tags_0 },
    { 0x64950c84, SUC_lexicon_tags_0 },
    { 0x64950c84, SUC_lexicon_tags_49 },
    { 0x34750c84, SUC_lexicon_tags_17 },
    { 0xf3750c89, SUC_lexicon_tags_9 },
    { 0xfe350c8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d50c8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff150c8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f50c94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6350c96, SUC_lexicon_tags_12 },
    { 0x43550c96, SUC_lexicon_tags_37 },
    { 0xa8b50c98, SUC_lexicon_tags_0 },
    { 0x15b50c96, SUC_lexicon_tags_15 },
    { 0x75b50c99, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91350c9e, SUC_lexicon_tags_23 },
    { 0x5f350c9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b50ca4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5cf50ca6, SUC_lexicon_tags_7 },
    { 0x8a150ca7, SUC_lexicon_tags_17 },
    { 0xa6550ca7, SUC_lexicon_tags_1 },
    { 0x51750ca9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4150cad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f50cb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b50cbb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf3350cbd, SUC_lexicon_tags_32 },
    { 0x6c950cbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93f50cc0, SUC_lexicon_tags_7 },
    { 0x1bd50cc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51550cc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b950ccb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a750ccd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea150cd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cb50cd3, SUC_lexicon_tags_178 },
    { 0xa5f50cd4, SUC_lexicon_tags_40 },
    { 0x81f50cd3, SUC_lexicon_tags_20 },
    { 0x70950cd6, SUC_lexicon_tags_17 },
    { 0xb8d50cd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47550cdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cf50cdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe750cea, SUC_lexicon_tags_19 },
    { 0x66350ceb, SUC_lexicon_tags_2 },
    { 0x42750cec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8150cef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f350cf1, SUC_lexicon_tags_0 },
    { 0xcf150cf2, SUC_lexicon_tags_15 },
    { 0x64150cf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d50cf7, SUC_lexicon_tags_77 },
    { 0x93b50cf8, SUC_lexicon_tags_37 },
    { 0x53f50cf8, SUC_lexicon_tags_141 },
    { 0x77550cfa, SUC_lexicon_tags_37 },
    { 0x20f50cfb, SUC_lexicon_tags_32 },
    { 0xdb350cfc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x49350cff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd50d04, SUC_lexicon_tags_117 },
    { 0x4df50d05, SUC_lexicon_tags_0 },
    { 0x90550d06, SUC_lexicon_tags_14 },
    { 0x93750d07, SUC_lexicon_tags_47 },
    { 0x50150d05, SUC_lexicon_tags_38 },
    { 0x4e950d07, SUC_lexicon_tags_14 },
    { 0xc3950d08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x23150d0c, SUC_lexicon_tags_63 },
    { 0xdb150d0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce550d0f, SUC_lexicon_tags_27 },
    { 0x69950d10, SUC_lexicon_tags_17 },
    { 0x3ff50d0f, SUC_lexicon_tags_7 },
    { 0x16150d12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5750d15, SUC_lexicon_tags_11 },
    { 0x31150d16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc750d18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80350d1b, SUC_lexicon_tags_11 },
    { 0x7750d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f50d23, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1750d27, SUC_lexicon_tags_25 },
    { 0xb5550d28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f50d2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd50d35, SUC_lexicon_tags_27 },
    { 0x6a750d35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a950d38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a950d3b, SUC_lexicon_tags_7 },
    { 0x8df50d3b, SUC_lexicon_tags_27 },
    { 0xa0b50d3d, SUC_lexicon_tags_17 },
    { 0xd2550d3e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd50d44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd550d46, SUC_lexicon_tags_920 },
    { 0x6f950d47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31950d4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b750d51, SUC_lexicon_tags_7 },
    { 0xd0f50d52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f950d56, SUC_lexicon_tags_11 },
    { 0x80550d57, SUC_lexicon_tags_46 },
    { 0xa9f50d58, SUC_lexicon_tags_55 },
    { 0x77b50d58, SUC_lexicon_tags_160 },
    { 0x3fb50d5a, SUC_lexicon_tags_17 },
    { 0xcad50d58, SUC_lexicon_tags_23 },
    { 0xfd150d5a, SUC_lexicon_tags_17 },
    { 0x72f50d5d, SUC_lexicon_tags_22 },
    { 0xd2350d5e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24950d62, SUC_lexicon_tags_12 },
    { 0x3d950d62, SUC_lexicon_tags_16 },
    { 0x70750d63, SUC_lexicon_tags_25 },
    { 0xd750d65, SUC_lexicon_tags_17 },
    { 0x3af50d65, SUC_lexicon_tags_1 },
    { 0x7a150d66, SUC_lexicon_tags_8 },
    { 0xd7750d68, SUC_lexicon_tags_65 },
    { 0x98550d69, SUC_lexicon_tags_1 },
    { 0xf3350d69, SUC_lexicon_tags_21 },
    { 0x59350d68, SUC_lexicon_tags_61 },
    { 0xdb150d67, SUC_lexicon_tags_37 },
    { 0xdc150d62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef50d71, SUC_lexicon_tags_2 },
    { 0xe8350d72, SUC_lexicon_tags_0 },
    { 0x2ed50d73, SUC_lexicon_tags_49 },
    { 0x6f50d72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2b950d76, SUC_lexicon_tags_7 },
    { 0xfc550d77, SUC_lexicon_tags_7 },
    { 0xda350d78, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b50d7b, SUC_lexicon_tags_35 },
    { 0xc4750d7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x16750d7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b50d81, SUC_lexicon_tags_40 },
    { 0xba750d82, SUC_lexicon_tags_32 },
    { 0xe4550d82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9350d8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80750d90, SUC_lexicon_tags_26 },
    { 0xc7150d90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73150d98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f750d9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1350da0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f50da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3350da5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f50da8, SUC_lexicon_tags_20 },
    { 0x5fd50da9, SUC_lexicon_tags_15 },
    { 0x7ed50da9, SUC_lexicon_tags_26 },
    { 0x87350daa, SUC_lexicon_tags_37 },
    { 0x58350dac, SUC_lexicon_tags_17 },
    { 0xa5550dac, SUC_lexicon_tags_15 },
    { 0x59750dae, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1d50db0, SUC_lexicon_tags_65 },
    { 0xaed50db0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67150db3, SUC_lexicon_tags_11 },
    { 0xfdb50db4, SUC_lexicon_tags_2 },
    { 0xb1d50db5, SUC_lexicon_tags_53 },
    { 0x13350db3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84150dbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a550dbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23350dc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28150dc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7950dc9, SUC_lexicon_tags_14 },
    { 0x10150dca, SUC_lexicon_tags_63 },
    { 0xc0f50dc9, SUC_lexicon_tags_7 },
    { 0xa7350dca, SUC_lexicon_tags_34 },
    { 0x43f50dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60150dd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0150dd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e150ddc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x16550dde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb50de0, SUC_lexicon_tags_30 },
    { 0x42150de0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd0550de3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x31150de5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab950dea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3350def, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x13350df1, SUC_lexicon_tags_12 },
    { 0x19f50df2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df50df7, SUC_lexicon_tags_27 },
    { 0xbf750df8, SUC_lexicon_tags_45 },
    { 0x9a350df9, SUC_lexicon_tags_37 },
    { 0xb3550dfa, SUC_lexicon_tags_10 },
    { 0x75d50dfb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x50950dfd, SUC_lexicon_tags_81 },
    { 0x2350dfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53350e01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x60f50e03, SUC_lexicon_tags_7 },
    { 0xd9350e04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23350e06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef50e0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b50e17, SUC_lexicon_tags_12 },
    { 0x34750e18, SUC_lexicon_tags_63 },
    { 0x9eb50e18, SUC_lexicon_tags_11 },
    { 0xe6550e1a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaaf50e1c, SUC_lexicon_tags_47 },
    { 0x93b50e1c, SUC_lexicon_tags_49 },
    { 0x8ff50e1e, SUC_lexicon_tags_49 },
    { 0x82b50e1e, SUC_lexicon_tags_57 },
    { 0x66550e20, SUC_lexicon_tags_0 },
    { 0x42550e21, SUC_lexicon_tags_33 },
    { 0xc7f50e22, SUC_lexicon_tags_31 },
    { 0xdf550e23, SUC_lexicon_tags_21 },
    { 0xaf550e23, SUC_lexicon_tags_17 },
    { 0xbf50e25, SUC_lexicon_tags_25 },
    { 0xd5950e24, SUC_lexicon_tags_6 },
    { 0xb0b50e25, SUC_lexicon_tags_17 },
    { 0xe8550e28, SUC_lexicon_tags_1 },
    { 0x89d50e29, SUC_lexicon_tags_30 },
    { 0xdf150e2a, SUC_lexicon_tags_40 },
    { 0x74950e2b, SUC_lexicon_tags_11 },
    { 0x2b150e2c, SUC_lexicon_tags_49 },
    { 0xb6d50e2d, SUC_lexicon_tags_37 },
    { 0x6b150e2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54350e30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40550e32, SUC_lexicon_tags_27 },
    { 0x91150e33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b50e36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7150e38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e350e3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76750e41, SUC_lexicon_tags_7 },
    { 0xe0350e41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d50e47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x94750e4a, SUC_lexicon_tags_37 },
    { 0x1ff50e4b, SUC_lexicon_tags_11 },
    { 0xf3d50e4c, SUC_lexicon_tags_67 },
    { 0xbef50e4a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9950e55, SUC_lexicon_tags_9 },
    { 0x1550e56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf3550e58, SUC_lexicon_tags_14 },
    { 0x6cb50e59, SUC_lexicon_tags_31 },
    { 0xdef50e58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0150e5c, SUC_lexicon_tags_34 },
    { 0x58f50e5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2150e60, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b50e66, SUC_lexicon_tags_70 },
    { 0xaab50e67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18150e6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17350e6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba150e6e, SUC_lexicon_tags_25 },
    { 0x9750e6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5950e72, SUC_lexicon_tags_37 },
    { 0xbbb50e73, SUC_lexicon_tags_172 },
    { 0x5d350e74, SUC_lexicon_tags_37 },
    { 0x85150e74, SUC_lexicon_tags_17 },
    { 0x6950e72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81750e79, SUC_lexicon_tags_12 },
    { 0x37950e79, SUC_lexicon_tags_28 },
    { 0x93f50e7b, SUC_lexicon_tags_19 },
    { 0x97550e7c, SUC_lexicon_tags_20 },
    { 0xc0b50e7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37950e80, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd50e84, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46750e8b, SUC_lexicon_tags_25 },
    { 0x2b350e8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b50e8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a950e94, SUC_lexicon_tags_11 },
    { 0xcef50e95, SUC_lexicon_tags_12 },
    { 0xdd350e94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54150e99, SUC_lexicon_tags_118 },
    { 0x44750e99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e750ea2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x10b50ea4, SUC_lexicon_tags_25 },
    { 0xf8b50ea5, SUC_lexicon_tags_14 },
    { 0xc9f50ea6, SUC_lexicon_tags_26 },
    { 0x42950ea7, SUC_lexicon_tags_33 },
    { 0xcdf50ea8, SUC_lexicon_tags_7 },
    { 0x3f550ea9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x39750eac, SUC_lexicon_tags_2 },
    { 0xb0550ead, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdb50eaf, SUC_lexicon_tags_47 },
    { 0x94b50eaf, SUC_lexicon_tags_19 },
    { 0x55550eb1, SUC_lexicon_tags_84 },
    { 0x3f150eaf, SUC_lexicon_tags_7 },
    { 0x44750eb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82550eb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x23550eb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd50ebc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa350ec3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2d50ec5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8df50ec7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85350ec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6350ed1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3350ed4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4f50ed6, SUC_lexicon_tags_25 },
    { 0xee750ed7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98950eda, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec150ee0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b50ee4, SUC_lexicon_tags_15 },
    { 0x9bf50ee5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf950ee7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4150eef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x62750ef1, SUC_lexicon_tags_33 },
    { 0x88350ef2, SUC_lexicon_tags_13 },
    { 0xb3550ef3, SUC_lexicon_tags_14 },
    { 0x38950ef4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5950ef6, SUC_lexicon_tags_25 },
    { 0x45b50ef7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75150efe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f50f03, SUC_lexicon_tags_8 },
    { 0x72b50f03, SUC_lexicon_tags_87 },
    { 0x3f50f03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea950f0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9750f1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd50f1e, SUC_lexicon_tags_7 },
    { 0xb950f1f, SUC_lexicon_tags_1 },
    { 0xff150f1e, SUC_lexicon_tags_17 },
    { 0xdad50f20, SUC_lexicon_tags_11 },
    { 0xa0550f22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc550f27, SUC_lexicon_tags_32 },
    { 0x53350f28, SUC_lexicon_tags_2 },
    { 0xf3d50f28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5350f2c, SUC_lexicon_tags_11 },
    { 0x28550f2d, SUC_lexicon_tags_26 },
    { 0x4e950f2e, SUC_lexicon_tags_31 },
    { 0x61f50f2c, SUC_lexicon_tags_11 },
    { 0xadd50f30, SUC_lexicon_tags_57 },
    { 0x2550f31, SUC_lexicon_tags_8 },
    { 0x1af50f32, SUC_lexicon_tags_11 },
    { 0xca950f32, SUC_lexicon_tags_63 },
    { 0xef50f31, SUC_lexicon_tags_26 },
    { 0x5db50f2c, SUC_lexicon_tags_49 },
    { 0xc5d50f32, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x32f50f38, SUC_lexicon_tags_30 },
    { 0xd3f50f38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf50f3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd50f42, SUC_lexicon_tags_47 },
    { 0xf0150f43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49150f45, SUC_lexicon_tags_11 },
    { 0x8b750f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63750f4a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x24150f4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62750f52, SUC_lexicon_tags_11 },
    { 0x44950f52, SUC_lexicon_tags_34 },
    { 0x28950f54, SUC_lexicon_tags_56 },
    { 0xd350f55, SUC_lexicon_tags_60 },
    { 0x22750f55, SUC_lexicon_tags_11 },
    { 0xb0b50f57, SUC_lexicon_tags_1 },
    { 0x7d350f57, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab50f5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab50f5e, SUC_lexicon_tags_47 },
    { 0x3ff50f5f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x21950f61, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x16950f64, SUC_lexicon_tags_7 },
    { 0x73750f64, SUC_lexicon_tags_15 },
    { 0xbc150f64, SUC_lexicon_tags_21 },
    { 0x4ef50f67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xffb50f69, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87350f6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3550f73, SUC_lexicon_tags_27 },
    { 0x98750f74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26550f76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0750f78, SUC_lexicon_tags_34 },
    { 0x59350f79, SUC_lexicon_tags_19 },
    { 0x99d50f78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87950f82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9750f85, SUC_lexicon_tags_2 },
    { 0x79d50f85, SUC_lexicon_tags_25 },
    { 0x70b50f86, SUC_lexicon_tags_15 },
    { 0x1e950f86, SUC_lexicon_tags_26 },
    { 0x44550f89, SUC_lexicon_tags_1 },
    { 0xc9950f8a, SUC_lexicon_tags_37 },
    { 0x9eb50f8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x38550f8e, SUC_lexicon_tags_11 },
    { 0x79d50f8f, SUC_lexicon_tags_60 },
    { 0x5f750f90, SUC_lexicon_tags_25 },
    { 0xe9150f90, SUC_lexicon_tags_27 },
    { 0xf6b50f92, SUC_lexicon_tags_10 },
    { 0x30f50f93, SUC_lexicon_tags_37 },
    { 0x29950f92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb150f97, SUC_lexicon_tags_61 },
    { 0x80d50f98, SUC_lexicon_tags_11 },
    { 0x52150f99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b150f9b, SUC_lexicon_tags_17 },
    { 0x9ef50f9b, SUC_lexicon_tags_66 },
    { 0x3d150f9d, SUC_lexicon_tags_37 },
    { 0x29150f9d, SUC_lexicon_tags_11 },
    { 0x7cb50f9b, SUC_lexicon_tags_12 },
    { 0xfcb50f9e, SUC_lexicon_tags_11 },
    { 0xd3b50f9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f50fa7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x51750faa, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9d950fac, SUC_lexicon_tags_14 },
    { 0x19350fac, SUC_lexicon_tags_17 },
    { 0x1b350fae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43550fb1, SUC_lexicon_tags_38 },
    { 0x3d350fb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9150fb5, SUC_lexicon_tags_42 },
    { 0xfaf50fb5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2fd50fb8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34550fbf, SUC_lexicon_tags_2 },
    { 0xc3f50fc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0950fc2, SUC_lexicon_tags_0 },
    { 0xdef50fc2, SUC_lexicon_tags_8 },
    { 0x67d50fc4, SUC_lexicon_tags_45 },
    { 0xf50fc5, SUC_lexicon_tags_26 },
    { 0xe6d50fc5, SUC_lexicon_tags_25 },
    { 0x4e750fc6, SUC_lexicon_tags_47 },
    { 0x81b50fc6, SUC_lexicon_tags_63 },
    { 0x4350fc9, SUC_lexicon_tags_25 },
    { 0xcc350fc3, SUC_lexicon_tags_15 },
    { 0x60750fcb, SUC_lexicon_tags_0 },
    { 0xd4750fcb, SUC_lexicon_tags_37 },
    { 0x8c550fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f550fcf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac950fd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d50fe2, SUC_lexicon_tags_12 },
    { 0x4ff50fe3, SUC_lexicon_tags_25 },
    { 0x37750fe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cb50fe6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe1f50fe8, SUC_lexicon_tags_112 },
    { 0x4b550fe9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7f750feb, SUC_lexicon_tags_4 },
    { 0x2a350feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35750fee, SUC_lexicon_tags_27 },
    { 0xa6150fef, SUC_lexicon_tags_17 },
    { 0x91d50ff0, SUC_lexicon_tags_7 },
    { 0xcf750ff1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4d50ff3, SUC_lexicon_tags_2 },
    { 0x2ab50ff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed50ffd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf51001, SUC_lexicon_tags_17 },
    { 0x51751002, SUC_lexicon_tags_7 },
    { 0x58f51003, SUC_lexicon_tags_55 },
    { 0x3bf51004, SUC_lexicon_tags_27 },
    { 0x33b51005, SUC_lexicon_tags_37 },
    { 0xd2d51003, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1115100b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32751011, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7751013, SUC_lexicon_tags_19 },
    { 0xd3951014, SUC_lexicon_tags_43 },
    { 0xe3d51013, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4b51017, SUC_lexicon_tags_1 },
    { 0xed51018, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8675101b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc175101d, SUC_lexicon_tags_0 },
    { 0xfad5101e, SUC_lexicon_tags_15 },
    { 0xfa35101d, SUC_lexicon_tags_22 },
    { 0xd5b51020, SUC_lexicon_tags_4 },
    { 0xc0551021, SUC_lexicon_tags_28 },
    { 0xde351021, SUC_lexicon_tags_31 },
    { 0xad951023, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1351027, SUC_lexicon_tags_37 },
    { 0xf0b51028, SUC_lexicon_tags_43 },
    { 0x96151029, SUC_lexicon_tags_87 },
    { 0xb951027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f5102e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3f51030, SUC_lexicon_tags_27 },
    { 0xc9d51031, SUC_lexicon_tags_15 },
    { 0x6e151030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7d51034, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0xd5951036, SUC_lexicon_tags_37 },
    { 0x7ab51037, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56b51039, SUC_lexicon_tags_2 },
    { 0x7f951039, SUC_lexicon_tags_55 },
    { 0x15f5103b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e95103e, SUC_lexicon_tags_7 },
    { 0xfc15103f, SUC_lexicon_tags_1 },
    { 0xf7d5103e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f51046, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee751049, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa595104e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd51055, SUC_lexicon_tags_45 },
    { 0x14751056, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf51059, SUC_lexicon_tags_18 },
    { 0xcc35105a, SUC_lexicon_tags_1 },
    { 0x44f5105b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f5105f, SUC_lexicon_tags_0 },
    { 0x2b151060, SUC_lexicon_tags_12 },
    { 0xf2351060, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b351067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cd51069, SUC_lexicon_tags_11 },
    { 0x82d5106a, SUC_lexicon_tags_1 },
    { 0x7a95106b, SUC_lexicon_tags_23 },
    { 0xe935106c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22551071, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b351073, SUC_lexicon_tags_15 },
    { 0xf4d51073, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x3b551076, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc3d51078, SUC_lexicon_tags_15 },
    { 0xf4151078, SUC_lexicon_tags_23 },
    { 0xbb551078, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97f5107c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa775107e, SUC_lexicon_tags_35 },
    { 0x4975107f, SUC_lexicon_tags_31 },
    { 0x3755107f, SUC_lexicon_tags_45 },
    { 0xb8f51081, SUC_lexicon_tags_30 },
    { 0x10751082, SUC_lexicon_tags_14 },
    { 0x9d151083, SUC_lexicon_tags_15 },
    { 0x1b751084, SUC_lexicon_tags_2 },
    { 0xa6351081, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2695108b, SUC_lexicon_tags_11 },
    { 0x5395108b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd815108f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85351091, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e351093, SUC_lexicon_tags_17 },
    { 0x21f51094, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6351099, SUC_lexicon_tags_23 },
    { 0xfc551099, SUC_lexicon_tags_12 },
    { 0x9a75109b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f5109f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1510a4, SUC_lexicon_tags_17 },
    { 0x785510a5, SUC_lexicon_tags_30 },
    { 0xbc3510a6, SUC_lexicon_tags_30 },
    { 0xf15510a6, SUC_lexicon_tags_23 },
    { 0x669510a5, SUC_lexicon_tags_56 },
    { 0x76f510a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53510ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647510b0, SUC_lexicon_tags_7 },
    { 0x6cd510b1, SUC_lexicon_tags_11 },
    { 0x13b510b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99510b9, SUC_lexicon_tags_37 },
    { 0x65b510ba, SUC_lexicon_tags_29 },
    { 0x841510bb, SUC_lexicon_tags_43 },
    { 0x3c5510b9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861510c3, SUC_lexicon_tags_1 },
    { 0x103510c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95510cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943510d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b510d9, SUC_lexicon_tags_33 },
    { 0x2ff510da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951510e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb510e8, SUC_lexicon_tags_7 },
    { 0x469510e8, SUC_lexicon_tags_33 },
    { 0x389510e8, SUC_lexicon_tags_68 },
    { 0x3ed510ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8db510ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc51510ef, SUC_lexicon_tags_60 },
    { 0xd9d510ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d510f3, SUC_lexicon_tags_49 },
    { 0x3db510f4, SUC_lexicon_tags_18 },
    { 0xc41510f5, SUC_lexicon_tags_7 },
    { 0xd6f510f5, SUC_lexicon_tags_0 },
    { 0x72f510f7, SUC_lexicon_tags_23 },
    { 0x167510f7, SUC_lexicon_tags_17 },
    { 0x2fb510f9, SUC_lexicon_tags_10 },
    { 0xc75510f3, SUC_lexicon_tags_49 },
    { 0x643510f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b51101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0751103, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65551107, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd15110c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd51111, SUC_lexicon_tags_31 },
    { 0xaa951112, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x90b51114, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7bd51116, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26551119, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4555111b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f51121, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3175112b, SUC_lexicon_tags_32 },
    { 0x6215112c, SUC_lexicon_tags_1 },
    { 0x1855112d, SUC_lexicon_tags_39 },
    { 0xee55112e, SUC_lexicon_tags_60 },
    { 0xb6b5112c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8951133, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef51137, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2c551139, SUC_lexicon_tags_25 },
    { 0xe355113a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe975113c, SUC_lexicon_tags_12 },
    { 0x1755113c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f351140, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57551147, SUC_lexicon_tags_25 },
    { 0x34951148, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d5114d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37751153, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb9351155, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc5151157, SUC_lexicon_tags_37 },
    { 0xd51157, SUC_lexicon_tags_47 },
    { 0x97151159, SUC_lexicon_tags_4 },
    { 0xe1b5115a, SUC_lexicon_tags_64 },
    { 0x6595115b, SUC_lexicon_tags_17 },
    { 0xf1b5115c, SUC_lexicon_tags_6 },
    { 0x1f55115d, SUC_lexicon_tags_39 },
    { 0x24b5115d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b51167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a351169, SUC_lexicon_tags_63 },
    { 0xd2751169, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5595116e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98951172, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d51176, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2551178, SUC_lexicon_tags_17 },
    { 0xc951179, SUC_lexicon_tags_12 },
    { 0x4bf5117a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf5117e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x69951181, SUC_lexicon_tags_129 },
    { 0x67751182, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d51185, SUC_lexicon_tags_11 },
    { 0xab151185, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c55118a, SUC_lexicon_tags_23 },
    { 0x555118b, SUC_lexicon_tags_60 },
    { 0x13d5118b, SUC_lexicon_tags_10 },
    { 0x1215118d, SUC_lexicon_tags_28 },
    { 0xc295118e, SUC_lexicon_tags_56 },
    { 0x82b5118f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf4151191, SUC_lexicon_tags_29 },
    { 0xdb551192, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9551196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b5119d, SUC_lexicon_tags_37 },
    { 0x6855119d, SUC_lexicon_tags_8 },
    { 0x1755119e, SUC_lexicon_tags_25 },
    { 0xac55119f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9511a3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x685511a5, SUC_lexicon_tags_60 },
    { 0x3db511a5, SUC_lexicon_tags_8 },
    { 0xa53511a7, SUC_lexicon_tags_17 },
    { 0x233511a7, SUC_lexicon_tags_1650 },
    { 0x903511a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf5511ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605511b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11511ba, SUC_lexicon_tags_11 },
    { 0x71d511bb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7cb511bd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdbd511bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x651511c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa41511c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb55511c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7511d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf511d5, SUC_lexicon_tags_27 },
    { 0xa01511d6, SUC_lexicon_tags_2 },
    { 0xc13511d6, SUC_lexicon_tags_27 },
    { 0xa9f511d8, SUC_lexicon_tags_9 },
    { 0xfdd511d9, SUC_lexicon_tags_201 },
    { 0x8f511d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93511df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b511e4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5511e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39511eb, SUC_lexicon_tags_1 },
    { 0x159511ec, SUC_lexicon_tags_7 },
    { 0x295511ed, SUC_lexicon_tags_35 },
    { 0x583511ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19511f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x365511fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b511ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4551203, SUC_lexicon_tags_7 },
    { 0x70951204, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed51207, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4275120b, SUC_lexicon_tags_37 },
    { 0xc7d5120c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2315120e, SUC_lexicon_tags_15 },
    { 0xdc35120f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c151212, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a951216, SUC_lexicon_tags_43 },
    { 0x9e351216, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db5121a, SUC_lexicon_tags_23 },
    { 0x49b5121b, SUC_lexicon_tags_14 },
    { 0x4c55121c, SUC_lexicon_tags_46 },
    { 0xea15121d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f51220, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c951224, SUC_lexicon_tags_27 },
    { 0xd2b51225, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x91151228, SUC_lexicon_tags_27 },
    { 0xc9551229, SUC_lexicon_tags_28 },
    { 0xa1d5122a, SUC_lexicon_tags_30 },
    { 0x3a35122b, SUC_lexicon_tags_37 },
    { 0xfb5122b, SUC_lexicon_tags_18 },
    { 0x67f5122d, SUC_lexicon_tags_11 },
    { 0x7215122e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57151233, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b5123d, SUC_lexicon_tags_38 },
    { 0x3b75123d, SUC_lexicon_tags_11 },
    { 0x72f5123f, SUC_lexicon_tags_25 },
    { 0xfb35123f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3ef51242, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d51245, SUC_lexicon_tags_25 },
    { 0x3c751246, SUC_lexicon_tags_11 },
    { 0xb0951245, SUC_lexicon_tags_19 },
    { 0x1e351246, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed51250, SUC_lexicon_tags_37 },
    { 0xdd751251, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a951254, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62151258, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6595125a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9675125f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x54951262, SUC_lexicon_tags_15 },
    { 0x9cf51262, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7d51265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8951267, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10351276, SUC_lexicon_tags_17 },
    { 0xe8d51277, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13b51279, SUC_lexicon_tags_15 },
    { 0xe735127a, SUC_lexicon_tags_10 },
    { 0x1495127b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba751285, SUC_lexicon_tags_43 },
    { 0x93f51286, SUC_lexicon_tags_14 },
    { 0xbd951285, SUC_lexicon_tags_1 },
    { 0x99d51288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbd5128a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3935128e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5751292, SUC_lexicon_tags_23 },
    { 0x11b51293, SUC_lexicon_tags_25 },
    { 0xda351294, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37351298, SUC_lexicon_tags_0 },
    { 0xe7751299, SUC_lexicon_tags_46 },
    { 0xf2151299, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc355129e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d512a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80b512a9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81512af, SUC_lexicon_tags_17 },
    { 0x5e5512b0, SUC_lexicon_tags_14 },
    { 0x859512b1, SUC_lexicon_tags_17 },
    { 0xdc7512b0, SUC_lexicon_tags_47 },
    { 0x843512b3, SUC_lexicon_tags_2 },
    { 0xb37512b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb99512b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3512bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x683512bd, SUC_lexicon_tags_30 },
    { 0x5cb512be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3f9512c0, SUC_lexicon_tags_45 },
    { 0xe07512c1, SUC_lexicon_tags_6 },
    { 0x3d9512c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49512c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883512cb, SUC_lexicon_tags_15 },
    { 0xf5512cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447512d1, SUC_lexicon_tags_15 },
    { 0x4fd512d2, SUC_lexicon_tags_15 },
    { 0xf09512d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x927512d6, SUC_lexicon_tags_45 },
    { 0x1eb512d7, SUC_lexicon_tags_37 },
    { 0xd71512d8, SUC_lexicon_tags_2 },
    { 0xc7512d8, SUC_lexicon_tags_17 },
    { 0x267512d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b512e0, SUC_lexicon_tags_32 },
    { 0x2b9512e0, SUC_lexicon_tags_45 },
    { 0xda5512e1, SUC_lexicon_tags_63 },
    { 0xcc1512e1, SUC_lexicon_tags_43 },
    { 0x837512e4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1512e8, SUC_lexicon_tags_57 },
    { 0x685512e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3512ec, SUC_lexicon_tags_17 },
    { 0x1fd512ed, SUC_lexicon_tags_26 },
    { 0x601512ee, SUC_lexicon_tags_28 },
    { 0xcd5512ef, SUC_lexicon_tags_11 },
    { 0x647512f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x473512f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f512f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac151300, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87151302, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xde351304, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x1e551306, SUC_lexicon_tags_37 },
    { 0x80f51307, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d5130a, SUC_lexicon_tags_56 },
    { 0xd475130b, SUC_lexicon_tags_11 },
    { 0x85d5130c, SUC_lexicon_tags_37 },
    { 0xf735130a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b51313, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f51319, SUC_lexicon_tags_12 },
    { 0xa295131a, SUC_lexicon_tags_37 },
    { 0x7835131a, SUC_lexicon_tags_10 },
    { 0xf635131c, SUC_lexicon_tags_30 },
    { 0xf5f5131d, SUC_lexicon_tags_13 },
    { 0x5d35131d, SUC_lexicon_tags_2 },
    { 0x1b75131f, SUC_lexicon_tags_55 },
    { 0xc855131e, SUC_lexicon_tags_17 },
    { 0x56351321, SUC_lexicon_tags_7 },
    { 0xc0b51321, SUC_lexicon_tags_3 },
    { 0xb7151323, SUC_lexicon_tags_37 },
    { 0xf3f51319, SUC_lexicon_tags_25 },
    { 0xeb951325, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc015132a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd635132c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f51333, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1595133a, SUC_lexicon_tags_17 },
    { 0x8835133a, SUC_lexicon_tags_1 },
    { 0xe0f5133a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd5133f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b51343, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb51347, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5134a, SUC_lexicon_tags_15 },
    { 0x6935134b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d51353, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x50751355, SUC_lexicon_tags_17 },
    { 0x21d51356, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc95135a, SUC_lexicon_tags_25 },
    { 0x4555135a, SUC_lexicon_tags_7 },
    { 0x9495135c, SUC_lexicon_tags_66 },
    { 0xc575135d, SUC_lexicon_tags_17 },
    { 0xd3b5135e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde151361, SUC_lexicon_tags_82 },
    { 0xba151361, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x25151365, SUC_lexicon_tags_77 },
    { 0xa3151366, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8db51368, SUC_lexicon_tags_48 },
    { 0xd1351369, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6195136b, SUC_lexicon_tags_37 },
    { 0x7d95136c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24751374, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x81151377, SUC_lexicon_tags_39 },
    { 0x6cf51378, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4655137a, SUC_lexicon_tags_31 },
    { 0x8bf5137a, SUC_lexicon_tags_25 },
    { 0x8715137a, SUC_lexicon_tags_39 },
    { 0x2c15137a, SUC_lexicon_tags_61 },
    { 0x7015137b, SUC_lexicon_tags_1 },
    { 0x3075137e, SUC_lexicon_tags_35 },
    { 0xf575137e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c951384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40351387, SUC_lexicon_tags_1312 },
    { 0x6f551388, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8555138b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd5138f, SUC_lexicon_tags_28 },
    { 0xf755138f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd51393, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c551397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e551399, SUC_lexicon_tags_35 },
    { 0x255139a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3513a0, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb513a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33513a7, SUC_lexicon_tags_27 },
    { 0x7ab513a8, SUC_lexicon_tags_41 },
    { 0xd53513a8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3513ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x795513ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8b513b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86f513b2, SUC_lexicon_tags_37 },
    { 0xec5513b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529513b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7513bb, SUC_lexicon_tags_17 },
    { 0xc7d513bb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xc59513be, SUC_lexicon_tags_2 },
    { 0x8e7513bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f513c3, SUC_lexicon_tags_3 },
    { 0xd89513c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7513cb, SUC_lexicon_tags_28 },
    { 0xa3b513cc, SUC_lexicon_tags_60 },
    { 0x3f7513cc, SUC_lexicon_tags_11 },
    { 0x5e3513ce, SUC_lexicon_tags_462 },
    { 0x191513ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa9513d1, SUC_lexicon_tags_33 },
    { 0xe13513d2, SUC_lexicon_tags_95 },
    { 0x18d513d1, SUC_lexicon_tags_26 },
    { 0x839513d1, SUC_lexicon_tags_11 },
    { 0x861513d5, SUC_lexicon_tags_2 },
    { 0x5b5513d3, SUC_lexicon_tags_170 },
    { 0x74b513d6, SUC_lexicon_tags_46 },
    { 0x12f513d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf513db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x551513dd, SUC_lexicon_tags_23 },
    { 0xd97513de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x977513e0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd513e4, SUC_lexicon_tags_27 },
    { 0xdd1513e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d513e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb1513eb, SUC_lexicon_tags_45 },
    { 0x507513ec, SUC_lexicon_tags_12 },
    { 0xd3d513ec, SUC_lexicon_tags_11 },
    { 0x419513ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b513f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601513fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x17513ff, SUC_lexicon_tags_11 },
    { 0x179513ff, SUC_lexicon_tags_15 },
    { 0x24b51401, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b351405, SUC_lexicon_tags_29 },
    { 0x70951405, SUC_lexicon_tags_15 },
    { 0x91151407, SUC_lexicon_tags_2 },
    { 0x551408, SUC_lexicon_tags_12 },
    { 0xaa751409, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa2f5140b, SUC_lexicon_tags_2 },
    { 0xa435140c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb5140f, SUC_lexicon_tags_60 },
    { 0xcbf51410, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5351414, SUC_lexicon_tags_17 },
    { 0xd4d51415, SUC_lexicon_tags_8 },
    { 0xaa351414, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5551418, SUC_lexicon_tags_7 },
    { 0x5f151419, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed95141e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88751427, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e151429, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd5142c, SUC_lexicon_tags_30 },
    { 0x98d5142d, SUC_lexicon_tags_12 },
    { 0x6d15142c, SUC_lexicon_tags_7 },
    { 0x15142f, SUC_lexicon_tags_1 },
    { 0x555142f, SUC_lexicon_tags_25 },
    { 0x62b51431, SUC_lexicon_tags_282 },
    { 0xcd351432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbd51434, SUC_lexicon_tags_39 },
    { 0x85351434, SUC_lexicon_tags_43 },
    { 0x5c751436, SUC_lexicon_tags_37 },
    { 0xfb351437, SUC_lexicon_tags_1271 },
    { 0x91d51437, SUC_lexicon_tags_68 },
    { 0xaef51439, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85b5143b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c951441, SUC_lexicon_tags_25 },
    { 0xc5151442, SUC_lexicon_tags_1 },
    { 0xf8951441, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f551446, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5175144c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f51456, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb95145b, SUC_lexicon_tags_6 },
    { 0x6535145c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83b5145e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64351460, SUC_lexicon_tags_27 },
    { 0x7a951461, SUC_lexicon_tags_11 },
    { 0x8df51462, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed951466, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x24d51468, SUC_lexicon_tags_17 },
    { 0x41f51469, SUC_lexicon_tags_170 },
    { 0xab95146a, SUC_lexicon_tags_39 },
    { 0xe875146a, SUC_lexicon_tags_38 },
    { 0xa3b5146a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9951471, SUC_lexicon_tags_7 },
    { 0x9c351472, SUC_lexicon_tags_12 },
    { 0xe1751472, SUC_lexicon_tags_17 },
    { 0x3ef51474, SUC_lexicon_tags_23 },
    { 0x5ef51475, SUC_lexicon_tags_30 },
    { 0x1951475, SUC_lexicon_tags_49 },
    { 0xb8351473, SUC_lexicon_tags_65 },
    { 0x3d951478, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd35147c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0b5147e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5551482, SUC_lexicon_tags_45 },
    { 0x24f51483, SUC_lexicon_tags_15 },
    { 0x9d551484, SUC_lexicon_tags_38 },
    { 0xdf51484, SUC_lexicon_tags_17 },
    { 0xe9951484, SUC_lexicon_tags_1 },
    { 0xb9951487, SUC_lexicon_tags_11 },
    { 0x8e751486, SUC_lexicon_tags_37 },
    { 0xcb951489, SUC_lexicon_tags_77 },
    { 0x8a351489, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x17d5148c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd51490, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb51498, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7815149c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d5149f, SUC_lexicon_tags_8 },
    { 0x671514a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfef514a2, SUC_lexicon_tags_4 },
    { 0xfd1514a3, SUC_lexicon_tags_14 },
    { 0xccf514a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x187514a7, SUC_lexicon_tags_11 },
    { 0xdf5514a7, SUC_lexicon_tags_25 },
    { 0xcaf514a9, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x35b514ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333514b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b514b5, SUC_lexicon_tags_116 },
    { 0x7d514b6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77514b9, SUC_lexicon_tags_17 },
    { 0xd81514b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77514bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d514c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d514c8, SUC_lexicon_tags_77 },
    { 0x259514c9, SUC_lexicon_tags_2 },
    { 0xfa3514c9, SUC_lexicon_tags_37 },
    { 0x979514ca, SUC_lexicon_tags_13 },
    { 0x277514cc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b514cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb514d1, SUC_lexicon_tags_25 },
    { 0x18d514d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f514d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb514df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x755514e1, SUC_lexicon_tags_16 },
    { 0xae3514e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555514e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc75514ea, SUC_lexicon_tags_14 },
    { 0x4c7514ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683514f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71514f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1e5514f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfd514fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907514ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1dd51501, SUC_lexicon_tags_2 },
    { 0xa5151501, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23d51504, SUC_lexicon_tags_65 },
    { 0xf7751504, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f51508, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa395150b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4551513, SUC_lexicon_tags_141 },
    { 0xfcb51514, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc751517, SUC_lexicon_tags_27 },
    { 0x5b951518, SUC_lexicon_tags_12 },
    { 0xb951517, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6555151b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc35151d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x75d5151f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f751523, SUC_lexicon_tags_16 },
    { 0x8c751523, SUC_lexicon_tags_45 },
    { 0x79351525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f5152a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe915152c, SUC_lexicon_tags_37 },
    { 0x1395152d, SUC_lexicon_tags_37 },
    { 0x2a95152d, SUC_lexicon_tags_15 },
    { 0x62b5152f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa475153a, SUC_lexicon_tags_11 },
    { 0xf675153b, SUC_lexicon_tags_47 },
    { 0x40f5153b, SUC_lexicon_tags_82 },
    { 0xb9b5153d, SUC_lexicon_tags_39 },
    { 0xd455153e, SUC_lexicon_tags_26 },
    { 0x32f5153e, SUC_lexicon_tags_37 },
    { 0xffb51540, SUC_lexicon_tags_88 },
    { 0x3195153e, SUC_lexicon_tags_12 },
    { 0x5ad51540, SUC_lexicon_tags_56 },
    { 0xc2351543, SUC_lexicon_tags_47 },
    { 0xea751544, SUC_lexicon_tags_6 },
    { 0x95751545, SUC_lexicon_tags_7 },
    { 0xef351543, SUC_lexicon_tags_47 },
    { 0xd2151544, SUC_lexicon_tags_91 },
    { 0xd351548, SUC_lexicon_tags_1 },
    { 0x2fd51546, SUC_lexicon_tags_37 },
    { 0x67f5154a, SUC_lexicon_tags_2 },
    { 0x6b5154b, SUC_lexicon_tags_49 },
    { 0x515154c, SUC_lexicon_tags_49 },
    { 0xecb5154d, SUC_lexicon_tags_19 },
    { 0xf215154e, SUC_lexicon_tags_1 },
    { 0x5875154b, SUC_lexicon_tags_60 },
    { 0xa895154d, SUC_lexicon_tags_17 },
    { 0x8c151551, SUC_lexicon_tags_0 },
    { 0x4c351552, SUC_lexicon_tags_13 },
    { 0xac951553, SUC_lexicon_tags_47 },
    { 0x42d51552, SUC_lexicon_tags_3 },
    { 0x5551555, SUC_lexicon_tags_60 },
    { 0x5675154d, SUC_lexicon_tags_49 },
    { 0xebd5154f, SUC_lexicon_tags_19 },
    { 0xebb51558, SUC_lexicon_tags_15 },
    { 0x4695153a, SUC_lexicon_tags_1 },
    { 0x87f51548, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc6b5155c, SUC_lexicon_tags_37 },
    { 0xddb5155c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4295155f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc751563, SUC_lexicon_tags_18 },
    { 0xd551564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3351566, SUC_lexicon_tags_19 },
    { 0xf9351566, SUC_lexicon_tags_15 },
    { 0x36b51568, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2955156e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6df51570, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b51573, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f51577, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d5157a, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xddd5157c, SUC_lexicon_tags_0 },
    { 0x7855157d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb551587, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d51589, SUC_lexicon_tags_45 },
    { 0xbd15158a, SUC_lexicon_tags_60 },
    { 0x4cf5158a, SUC_lexicon_tags_17 },
    { 0x2015158c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd51591, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e751593, SUC_lexicon_tags_25 },
    { 0xce751594, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf51597, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11f51599, SUC_lexicon_tags_30 },
    { 0xca151599, SUC_lexicon_tags_11 },
    { 0x9d15159b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd5159e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa19515a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65515a2, SUC_lexicon_tags_14 },
    { 0x395515a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f515a6, SUC_lexicon_tags_28 },
    { 0xfcd515a7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69515ac, SUC_lexicon_tags_1 },
    { 0x5ff515ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae5515af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x643515b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101515b8, SUC_lexicon_tags_2 },
    { 0x92d515b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3515bc, SUC_lexicon_tags_7 },
    { 0x315515bd, SUC_lexicon_tags_8 },
    { 0xddf515be, SUC_lexicon_tags_28 },
    { 0xe71515bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89515c2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x31515c4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517515c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff515cf, SUC_lexicon_tags_43 },
    { 0xe85515d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301515d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d515dd, SUC_lexicon_tags_47 },
    { 0x11515dd, SUC_lexicon_tags_5 },
    { 0xf9515df, SUC_lexicon_tags_25 },
    { 0x7e9515df, SUC_lexicon_tags_108 },
    { 0x1a7515e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817515e7, SUC_lexicon_tags_37 },
    { 0xfc5515e8, SUC_lexicon_tags_37 },
    { 0x721515e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f515ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b515ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65515f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed515fc, SUC_lexicon_tags_7 },
    { 0x7d9515fc, SUC_lexicon_tags_0 },
    { 0xac1515fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xda751601, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c751604, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9151606, SUC_lexicon_tags_29 },
    { 0x6c951607, SUC_lexicon_tags_7 },
    { 0x1351606, SUC_lexicon_tags_15 },
    { 0x29f51609, SUC_lexicon_tags_12 },
    { 0x6ef51607, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc335160d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6f75160f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab551614, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf951616, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x13351618, SUC_lexicon_tags_3 },
    { 0x6d751618, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5815161b, SUC_lexicon_tags_63 },
    { 0xb415161b, SUC_lexicon_tags_16 },
    { 0xb15161d, SUC_lexicon_tags_0 },
    { 0x8d35161e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df51621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe751623, SUC_lexicon_tags_27 },
    { 0xb5151623, SUC_lexicon_tags_17 },
    { 0x66f51625, SUC_lexicon_tags_2 },
    { 0x52751624, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd75162b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8435162d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac95162f, SUC_lexicon_tags_46 },
    { 0xc4d51630, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d51633, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61751635, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e551638, SUC_lexicon_tags_45 },
    { 0x5ab51639, SUC_lexicon_tags_11 },
    { 0xc4951639, SUC_lexicon_tags_49 },
    { 0x6d35163b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36d5163d, SUC_lexicon_tags_25 },
    { 0xa4d5163e, SUC_lexicon_tags_49 },
    { 0x1f55163e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff51642, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b51645, SUC_lexicon_tags_69 },
    { 0x4d751645, SUC_lexicon_tags_25 },
    { 0xf351646, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d5164a, SUC_lexicon_tags_27 },
    { 0x2435164b, SUC_lexicon_tags_17 },
    { 0xc7b5164c, SUC_lexicon_tags_11 },
    { 0x66b5164c, SUC_lexicon_tags_1 },
    { 0xc9f5164e, SUC_lexicon_tags_28 },
    { 0x55d5164e, SUC_lexicon_tags_33 },
    { 0xa3f51650, SUC_lexicon_tags_1 },
    { 0xf4d5164d, SUC_lexicon_tags_15 },
    { 0xf0751652, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd51655, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71551657, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af5165e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe551660, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d51664, SUC_lexicon_tags_100 },
    { 0x40151665, SUC_lexicon_tags_25 },
    { 0xc9751666, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef51669, SUC_lexicon_tags_12 },
    { 0x2c15166a, SUC_lexicon_tags_15 },
    { 0xa2f5166a, SUC_lexicon_tags_1 },
    { 0x46f5166c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2075166f, SUC_lexicon_tags_15 },
    { 0x50951670, SUC_lexicon_tags_0 },
    { 0x5cf51670, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe9d51673, SUC_lexicon_tags_2 },
    { 0x62551674, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd5167a, SUC_lexicon_tags_11 },
    { 0x9235167b, SUC_lexicon_tags_5 },
    { 0xee15167b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1175167e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x25151681, SUC_lexicon_tags_31 },
    { 0x6f151682, SUC_lexicon_tags_47 },
    { 0x8ed51681, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d5168b, SUC_lexicon_tags_11 },
    { 0xf35168b, SUC_lexicon_tags_39 },
    { 0xc915168c, SUC_lexicon_tags_17 },
    { 0xd095168e, SUC_lexicon_tags_26 },
    { 0x4df5168f, SUC_lexicon_tags_16 },
    { 0xacb5168d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10751698, SUC_lexicon_tags_38 },
    { 0xa1551698, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25d5169b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3516a4, SUC_lexicon_tags_10 },
    { 0xba5516a5, SUC_lexicon_tags_3 },
    { 0x7b5516a6, SUC_lexicon_tags_17 },
    { 0x33b516a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d516aa, SUC_lexicon_tags_45 },
    { 0x90f516ab, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcfb516ad, SUC_lexicon_tags_19 },
    { 0xa07516ae, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd516b7, SUC_lexicon_tags_33 },
    { 0xf67516b8, SUC_lexicon_tags_2 },
    { 0x9df516b7, SUC_lexicon_tags_2 },
    { 0x7e7516b7, SUC_lexicon_tags_38 },
    { 0xa51516b7, SUC_lexicon_tags_12 },
    { 0x4a3516bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33516bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7516c4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb516cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab516d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263516d6, SUC_lexicon_tags_6 },
    { 0xff3516d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3516de, SUC_lexicon_tags_1 },
    { 0x855516de, SUC_lexicon_tags_11 },
    { 0x1c7516de, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569516e4, SUC_lexicon_tags_15 },
    { 0x277516e5, SUC_lexicon_tags_1 },
    { 0xb93516e6, SUC_lexicon_tags_25 },
    { 0xa45516e7, SUC_lexicon_tags_12 },
    { 0x441516e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63f516ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b516f4, SUC_lexicon_tags_48 },
    { 0xb2d516f5, SUC_lexicon_tags_2 },
    { 0x79516f6, SUC_lexicon_tags_27 },
    { 0x3d5516f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2b516f9, SUC_lexicon_tags_32 },
    { 0x565516f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf516fd, SUC_lexicon_tags_37 },
    { 0xe93516fe, SUC_lexicon_tags_1 },
    { 0x7b5516ff, SUC_lexicon_tags_43 },
    { 0xb4b516ff, SUC_lexicon_tags_7 },
    { 0xa73516fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe9551703, SUC_lexicon_tags_15 },
    { 0xe1751704, SUC_lexicon_tags_17 },
    { 0x79151705, SUC_lexicon_tags_17 },
    { 0x85751706, SUC_lexicon_tags_87 },
    { 0x9b751706, SUC_lexicon_tags_12 },
    { 0xd5551707, SUC_lexicon_tags_0 },
    { 0x24351709, SUC_lexicon_tags_31 },
    { 0x8e15170a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1955170c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7855170e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb51711, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x22151713, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x66151715, SUC_lexicon_tags_17 },
    { 0x2b551715, SUC_lexicon_tags_25 },
    { 0x35951715, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4a151719, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f5171f, SUC_lexicon_tags_34 },
    { 0xaa751720, SUC_lexicon_tags_43 },
    { 0x4715171f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9551724, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x18551726, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f51733, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b51737, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfed51739, SUC_lexicon_tags_10 },
    { 0xe915173a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b5173d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60d5173f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x32951742, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff51747, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55151749, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2975174b, SUC_lexicon_tags_221 },
    { 0xc855174b, SUC_lexicon_tags_68 },
    { 0x5075174d, SUC_lexicon_tags_2 },
    { 0x83d5174d, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c551752, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x66551755, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6951759, SUC_lexicon_tags_38 },
    { 0x16f51759, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d5175e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f51761, SUC_lexicon_tags_11 },
    { 0xbf951762, SUC_lexicon_tags_42 },
    { 0x28151761, SUC_lexicon_tags_15 },
    { 0x1751764, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x40551767, SUC_lexicon_tags_8 },
    { 0xfa351768, SUC_lexicon_tags_1 },
    { 0x97f51769, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae951770, SUC_lexicon_tags_31 },
    { 0xc151770, SUC_lexicon_tags_12 },
    { 0x12151770, SUC_lexicon_tags_37 },
    { 0x12551773, SUC_lexicon_tags_49 },
    { 0xb8f51774, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61951776, SUC_lexicon_tags_7 },
    { 0x1c151776, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9d51779, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f5177c, SUC_lexicon_tags_25 },
    { 0x71b5177d, SUC_lexicon_tags_48 },
    { 0x94b5177e, SUC_lexicon_tags_12 },
    { 0x5395177e, SUC_lexicon_tags_25 },
    { 0x7a75177e, SUC_lexicon_tags_15 },
    { 0x3351781, SUC_lexicon_tags_7 },
    { 0xabb51781, SUC_lexicon_tags_15 },
    { 0xf4d51783, SUC_lexicon_tags_23 },
    { 0x71d51783, SUC_lexicon_tags_49 },
    { 0x48151783, SUC_lexicon_tags_7 },
    { 0x1b151786, SUC_lexicon_tags_47 },
    { 0xf6351787, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f5178c, SUC_lexicon_tags_37 },
    { 0xc7d5178d, SUC_lexicon_tags_11 },
    { 0xf5b5178d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7551790, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e551797, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ef51799, SUC_lexicon_tags_15 },
    { 0xf3d5179a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8815179f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedb517a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c7517a3, SUC_lexicon_tags_7 },
    { 0x2ab517a4, SUC_lexicon_tags_21 },
    { 0xee7517a3, SUC_lexicon_tags_47 },
    { 0x3c9517a6, SUC_lexicon_tags_12 },
    { 0x20b517a6, SUC_lexicon_tags_6 },
    { 0x335517a7, SUC_lexicon_tags_68 },
    { 0xfb9517a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b517ae, SUC_lexicon_tags_21 },
    { 0x81f517af, SUC_lexicon_tags_0 },
    { 0xdb517af, SUC_lexicon_tags_21 },
    { 0x973517ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175517b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d517bd, SUC_lexicon_tags_15 },
    { 0xe19517be, SUC_lexicon_tags_37 },
    { 0x4ad517bf, SUC_lexicon_tags_5 },
    { 0x2a7517c0, SUC_lexicon_tags_14 },
    { 0xc53517c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef517c7, SUC_lexicon_tags_11 },
    { 0x149517c8, SUC_lexicon_tags_17 },
    { 0x7d1517c9, SUC_lexicon_tags_0 },
    { 0x8a3517ca, SUC_lexicon_tags_179 },
    { 0x819517cb, SUC_lexicon_tags_1 },
    { 0xdb1517cc, SUC_lexicon_tags_66 },
    { 0x481517c9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b517d0, SUC_lexicon_tags_1 },
    { 0x69f517d0, SUC_lexicon_tags_33 },
    { 0x3ad517d2, SUC_lexicon_tags_7 },
    { 0x66f517d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b517d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5517da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb517dd, SUC_lexicon_tags_25 },
    { 0x1a3517dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab517e1, SUC_lexicon_tags_45 },
    { 0x5cd517e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7517e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f517ee, SUC_lexicon_tags_25 },
    { 0xca5517ee, SUC_lexicon_tags_27 },
    { 0x90d517ee, SUC_lexicon_tags_40 },
    { 0x5517ee, SUC_lexicon_tags_2 },
    { 0xbd1517f2, SUC_lexicon_tags_11 },
    { 0xe1b517f2, SUC_lexicon_tags_14 },
    { 0xe9d517ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b5517f6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x591517f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b517fd, SUC_lexicon_tags_2 },
    { 0xc69517fe, SUC_lexicon_tags_7 },
    { 0x567517fd, SUC_lexicon_tags_46 },
    { 0x885517ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31951808, SUC_lexicon_tags_12 },
    { 0xe8151808, SUC_lexicon_tags_11 },
    { 0x25d5180a, SUC_lexicon_tags_27 },
    { 0xf895180a, SUC_lexicon_tags_30 },
    { 0xbe55180c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8435180e, SUC_lexicon_tags_942 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb51813, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b51818, SUC_lexicon_tags_47 },
    { 0xbab51818, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55181c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30751820, SUC_lexicon_tags_1 },
    { 0xc3b51821, SUC_lexicon_tags_63 },
    { 0xd0951821, SUC_lexicon_tags_29 },
    { 0x9b751823, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45d51825, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x30751828, SUC_lexicon_tags_17 },
    { 0xc3551828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44d5182b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1e95182d, SUC_lexicon_tags_2 },
    { 0x8935182e, SUC_lexicon_tags_33 },
    { 0x2b95182f, SUC_lexicon_tags_7 },
    { 0xabf5182e, SUC_lexicon_tags_37 },
    { 0x9cf5182f, SUC_lexicon_tags_45 },
    { 0xff55182e, SUC_lexicon_tags_7 },
    { 0x32f51831, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0f51835, SUC_lexicon_tags_0 },
    { 0x53f51835, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x46351838, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8d5183a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdbb5183c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb75183f, SUC_lexicon_tags_37 },
    { 0xb9f51840, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47951848, SUC_lexicon_tags_156 },
    { 0x64951849, SUC_lexicon_tags_12 },
    { 0x38f5184a, SUC_lexicon_tags_37 },
    { 0xe835184b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfbf5184d, SUC_lexicon_tags_28 },
    { 0x9b95184e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa5b51850, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd151853, SUC_lexicon_tags_9 },
    { 0xc3b51853, SUC_lexicon_tags_12 },
    { 0xb5751855, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca75185d, SUC_lexicon_tags_1170 },
    { 0xafd5185e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db51863, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2951867, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f151869, SUC_lexicon_tags_15 },
    { 0xd3b5186a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa455186c, SUC_lexicon_tags_1 },
    { 0xa75186d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeef5186f, SUC_lexicon_tags_8 },
    { 0xae551870, SUC_lexicon_tags_15 },
    { 0x6ad5186f, SUC_lexicon_tags_39 },
    { 0x8d951872, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85351874, SUC_lexicon_tags_1 },
    { 0x1b51875, SUC_lexicon_tags_12 },
    { 0x7a351874, SUC_lexicon_tags_4 },
    { 0xf6551876, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5187a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9951880, SUC_lexicon_tags_25 },
    { 0x84b51880, SUC_lexicon_tags_0 },
    { 0x51951880, SUC_lexicon_tags_11 },
    { 0xbf751883, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x7a551885, SUC_lexicon_tags_30 },
    { 0x6b151886, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5495188b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b551892, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x10d51894, SUC_lexicon_tags_25 },
    { 0xc2b51895, SUC_lexicon_tags_17 },
    { 0x2a751896, SUC_lexicon_tags_15 },
    { 0xd0f51897, SUC_lexicon_tags_51 },
    { 0xa4751896, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3155189a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9055189c, SUC_lexicon_tags_26 },
    { 0xab75189d, SUC_lexicon_tags_12 },
    { 0xfbd5189e, SUC_lexicon_tags_26 },
    { 0x4735189e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x183518a2, SUC_lexicon_tags_13 },
    { 0x2ab518a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x369518a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9518a9, SUC_lexicon_tags_25 },
    { 0xc85518a9, SUC_lexicon_tags_37 },
    { 0xdc5518aa, SUC_lexicon_tags_25 },
    { 0xaab518aa, SUC_lexicon_tags_46 },
    { 0xbaf518ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x915518b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d518b3, SUC_lexicon_tags_17 },
    { 0x4b3518b3, SUC_lexicon_tags_65 },
    { 0xb5518b5, SUC_lexicon_tags_25 },
    { 0xf3b518b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49518bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x395518be, SUC_lexicon_tags_61 },
    { 0x733518bf, SUC_lexicon_tags_1 },
    { 0xdbb518c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf518c6, SUC_lexicon_tags_37 },
    { 0xc5d518c6, SUC_lexicon_tags_7 },
    { 0xce9518c7, SUC_lexicon_tags_27 },
    { 0xb25518c9, SUC_lexicon_tags_30 },
    { 0x9fb518ca, SUC_lexicon_tags_28 },
    { 0x1c9518cb, SUC_lexicon_tags_2 },
    { 0x75518cb, SUC_lexicon_tags_21 },
    { 0xd47518c8, SUC_lexicon_tags_43 },
    { 0x9cd518ce, SUC_lexicon_tags_43 },
    { 0x89b518cf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f518d3, SUC_lexicon_tags_37 },
    { 0x26f518d4, SUC_lexicon_tags_61 },
    { 0xb83518d4, SUC_lexicon_tags_33 },
    { 0xe43518d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f518d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ad518db, SUC_lexicon_tags_25 },
    { 0xb3f518dc, SUC_lexicon_tags_60 },
    { 0xe8b518db, SUC_lexicon_tags_25 },
    { 0xa47518de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75518ea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x347518ed, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db518f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7f518f3, SUC_lexicon_tags_17 },
    { 0x721518f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbc5518f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb518fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef518ff, SUC_lexicon_tags_29 },
    { 0x94351900, SUC_lexicon_tags_38 },
    { 0x20f51901, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa351904, SUC_lexicon_tags_26 },
    { 0xb6551905, SUC_lexicon_tags_45 },
    { 0x25951905, SUC_lexicon_tags_32 },
    { 0xd4551906, SUC_lexicon_tags_25 },
    { 0xfd951905, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c95190c, SUC_lexicon_tags_15 },
    { 0xd3b5190d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8135190f, SUC_lexicon_tags_15 },
    { 0xcc951910, SUC_lexicon_tags_25 },
    { 0x7df51911, SUC_lexicon_tags_72 },
    { 0x19551912, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x81351914, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa351919, SUC_lexicon_tags_2 },
    { 0x2375191a, SUC_lexicon_tags_12 },
    { 0x8d751919, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb15191d, SUC_lexicon_tags_0 },
    { 0xdf5191e, SUC_lexicon_tags_25 },
    { 0xe2f5191e, SUC_lexicon_tags_37 },
    { 0x3eb51920, SUC_lexicon_tags_42 },
    { 0xf6151920, SUC_lexicon_tags_7 },
    { 0x3f51922, SUC_lexicon_tags_31 },
    { 0x81b5191d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b151925, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb51927, SUC_lexicon_tags_74 },
    { 0x36b51927, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75192a, SUC_lexicon_tags_19 },
    { 0xca55192b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df5192e, SUC_lexicon_tags_0 },
    { 0x1455192f, SUC_lexicon_tags_37 },
    { 0x4555192e, SUC_lexicon_tags_38 },
    { 0x15351931, SUC_lexicon_tags_49 },
    { 0xd4351932, SUC_lexicon_tags_12 },
    { 0xd775192f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f5193a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7d5193c, SUC_lexicon_tags_12 },
    { 0x55f5193d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x26351940, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c351945, SUC_lexicon_tags_37 },
    { 0x12351946, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d5194e, SUC_lexicon_tags_1 },
    { 0x4f55194f, SUC_lexicon_tags_33 },
    { 0x27f5194f, SUC_lexicon_tags_17 },
    { 0xc8d51951, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34351954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0351956, SUC_lexicon_tags_11 },
    { 0xaaf51957, SUC_lexicon_tags_12 },
    { 0x94551957, SUC_lexicon_tags_37 },
    { 0xb51958, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa815195e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b51961, SUC_lexicon_tags_19 },
    { 0x33b51961, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef51967, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e151969, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d5196c, SUC_lexicon_tags_66 },
    { 0x5c35196c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf555196f, SUC_lexicon_tags_25 },
    { 0x5ad51970, SUC_lexicon_tags_32 },
    { 0xa4551971, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbbf51973, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d51976, SUC_lexicon_tags_65 },
    { 0x36751977, SUC_lexicon_tags_55 },
    { 0xb8b51978, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff951982, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa1151984, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29951987, SUC_lexicon_tags_1 },
    { 0x7a751987, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e15198e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b51995, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26951997, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d5199a, SUC_lexicon_tags_17 },
    { 0xcb75199a, SUC_lexicon_tags_15 },
    { 0x8d75199b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x38d5199e, SUC_lexicon_tags_31 },
    { 0x35f5199e, SUC_lexicon_tags_82 },
    { 0x79f519a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50f519a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1cd519a4, SUC_lexicon_tags_1 },
    { 0x615519a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf519a8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced519ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5519b0, SUC_lexicon_tags_179 },
    { 0x9cb519b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b519b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96b519b6, SUC_lexicon_tags_11 },
    { 0xcb519b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b519bc, SUC_lexicon_tags_49 },
    { 0x82f519bc, SUC_lexicon_tags_1 },
    { 0x9eb519bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f519c5, SUC_lexicon_tags_15 },
    { 0x85519c6, SUC_lexicon_tags_68 },
    { 0xe7b519c7, SUC_lexicon_tags_2 },
    { 0x683519c8, SUC_lexicon_tags_33 },
    { 0xae9519c6, SUC_lexicon_tags_4 },
    { 0x57519c9, SUC_lexicon_tags_43 },
    { 0x3d5519cb, SUC_lexicon_tags_38 },
    { 0xf25519cb, SUC_lexicon_tags_7 },
    { 0xacb519cd, SUC_lexicon_tags_12 },
    { 0xa1f519ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x177519d1, SUC_lexicon_tags_11 },
    { 0x305519d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89519d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc65519da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33519e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff519e3, SUC_lexicon_tags_60 },
    { 0x9ed519e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed519f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409519f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9519f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa01519fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x83b519fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d51a00, SUC_lexicon_tags_43 },
    { 0x49351a01, SUC_lexicon_tags_38 },
    { 0xb6951a01, SUC_lexicon_tags_60 },
    { 0xdbf51a00, SUC_lexicon_tags_29 },
    { 0x8ad51a01, SUC_lexicon_tags_32 },
    { 0xd7551a05, SUC_lexicon_tags_108 },
    { 0x1751a06, SUC_lexicon_tags_21 },
    { 0x21d51a01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d51a0a, SUC_lexicon_tags_16 },
    { 0xeb551a0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2751a0e, SUC_lexicon_tags_87 },
    { 0x6d951a0f, SUC_lexicon_tags_60 },
    { 0xf2951a10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f551a15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f51a1a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x78951a1c, SUC_lexicon_tags_65 },
    { 0x1cb51a1d, SUC_lexicon_tags_15 },
    { 0x15d51a1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42151a20, SUC_lexicon_tags_17 },
    { 0x78351a20, SUC_lexicon_tags_51 },
    { 0x1e751a22, SUC_lexicon_tags_37 },
    { 0xdff51a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d51a27, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa2d51a29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd51a2f, SUC_lexicon_tags_0 },
    { 0x29351a2f, SUC_lexicon_tags_3 },
    { 0x8f351a31, SUC_lexicon_tags_12 },
    { 0x1a551a32, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa9551a34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9351a37, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b951a3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36151a42, SUC_lexicon_tags_37 },
    { 0x74351a43, SUC_lexicon_tags_26 },
    { 0xc8951a44, SUC_lexicon_tags_8 },
    { 0x6cf51a44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15351a4a, SUC_lexicon_tags_25 },
    { 0xe9551a4a, SUC_lexicon_tags_1 },
    { 0x2551a4c, SUC_lexicon_tags_68 },
    { 0xc4551a4d, SUC_lexicon_tags_25 },
    { 0x23b51a4e, SUC_lexicon_tags_37 },
    { 0x1cb51a4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf51a52, SUC_lexicon_tags_24 },
    { 0x77751a52, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x40951a55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd351a5a, SUC_lexicon_tags_29 },
    { 0x1c551a5b, SUC_lexicon_tags_15 },
    { 0x4dd51a5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96151a60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b51a65, SUC_lexicon_tags_6 },
    { 0xaf151a65, SUC_lexicon_tags_10 },
    { 0x751a67, SUC_lexicon_tags_11 },
    { 0x83351a66, SUC_lexicon_tags_15 },
    { 0x69951a69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b51a6c, SUC_lexicon_tags_11 },
    { 0xffb51a6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b51a70, SUC_lexicon_tags_26 },
    { 0x2d751a71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3f351a73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x351a76, SUC_lexicon_tags_0 },
    { 0xc151a77, SUC_lexicon_tags_17 },
    { 0x11b51a77, SUC_lexicon_tags_37 },
    { 0xf1351a79, SUC_lexicon_tags_47 },
    { 0xf5551a7a, SUC_lexicon_tags_63 },
    { 0x92b51a79, SUC_lexicon_tags_7 },
    { 0x1ab51a77, SUC_lexicon_tags_37 },
    { 0x36351a7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7551a82, SUC_lexicon_tags_11 },
    { 0xa4d51a83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d51a89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d51a93, SUC_lexicon_tags_37 },
    { 0xd5951a94, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a951a98, SUC_lexicon_tags_12 },
    { 0x1ab51a99, SUC_lexicon_tags_15 },
    { 0x3bd51a98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37551aa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f51aa7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96551aae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf3751ab0, SUC_lexicon_tags_7 },
    { 0xfd551ab0, SUC_lexicon_tags_1 },
    { 0x4d351ab2, SUC_lexicon_tags_29 },
    { 0x94351ab2, SUC_lexicon_tags_23 },
    { 0x3d51ab4, SUC_lexicon_tags_12 },
    { 0x13351ab5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x91351ab7, SUC_lexicon_tags_25 },
    { 0x85d51ab8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3bd51aba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18951ac0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf51ac7, SUC_lexicon_tags_30 },
    { 0x2151ac8, SUC_lexicon_tags_19 },
    { 0xe2951ac8, SUC_lexicon_tags_31 },
    { 0x46751aca, SUC_lexicon_tags_0 },
    { 0x3a351aca, SUC_lexicon_tags_66 },
    { 0x72b51acc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x55d51ace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ef51ad0, SUC_lexicon_tags_25 },
    { 0xbd951ad1, SUC_lexicon_tags_11 },
    { 0xe9351ad2, SUC_lexicon_tags_15 },
    { 0x37151ad1, SUC_lexicon_tags_48 },
    { 0x6a951ad4, SUC_lexicon_tags_33 },
    { 0xd2751ad1, SUC_lexicon_tags_49 },
    { 0x19d51ad3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x40151ad8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f51add, SUC_lexicon_tags_12 },
    { 0x1bb51ade, SUC_lexicon_tags_77 },
    { 0xa7d51ade, SUC_lexicon_tags_30 },
    { 0xb2151adf, SUC_lexicon_tags_1 },
    { 0xb3b51add, SUC_lexicon_tags_38 },
    { 0x7751ae2, SUC_lexicon_tags_4 },
    { 0xa6751ae3, SUC_lexicon_tags_14 },
    { 0x5ad51ae2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82351aec, SUC_lexicon_tags_77 },
    { 0xa2d51aed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb51af1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb51af5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98351afb, SUC_lexicon_tags_11 },
    { 0x9b151afc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd551afe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf6b51b00, SUC_lexicon_tags_29 },
    { 0x3f51b01, SUC_lexicon_tags_29 },
    { 0x1c751b02, SUC_lexicon_tags_14 },
    { 0xe6b51b03, SUC_lexicon_tags_11 },
    { 0x93151b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfdd51b06, SUC_lexicon_tags_37 },
    { 0x18151b07, SUC_lexicon_tags_39 },
    { 0xbfd51b08, SUC_lexicon_tags_45 },
    { 0x23d51b08, SUC_lexicon_tags_19 },
    { 0x51751b0a, SUC_lexicon_tags_25 },
    { 0x6951b08, SUC_lexicon_tags_47 },
    { 0xc9951b0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb51b15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db51b19, SUC_lexicon_tags_37 },
    { 0x27551b1a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59951b29, SUC_lexicon_tags_14 },
    { 0x54951b29, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x56b51b2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a551b2e, SUC_lexicon_tags_31 },
    { 0x39751b2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef351b36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f51b3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb751b46, SUC_lexicon_tags_63 },
    { 0xc151b46, SUC_lexicon_tags_29 },
    { 0xc0b51b46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x79f51b4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9351b4d, SUC_lexicon_tags_11 },
    { 0x5ed51b4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f51b58, SUC_lexicon_tags_17 },
    { 0x44151b58, SUC_lexicon_tags_47 },
    { 0x31751b5a, SUC_lexicon_tags_60 },
    { 0xf6b51b5b, SUC_lexicon_tags_14 },
    { 0x72151b5a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x26551b5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4cf51b60, SUC_lexicon_tags_15 },
    { 0xa7151b60, SUC_lexicon_tags_27 },
    { 0x2e351b62, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7cf51b64, SUC_lexicon_tags_21 },
    { 0xb2f51b65, SUC_lexicon_tags_15 },
    { 0x99f51b66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x52951b68, SUC_lexicon_tags_79 },
    { 0x39f51b68, SUC_lexicon_tags_8 },
    { 0x85351b68, SUC_lexicon_tags_26 },
    { 0x4a951b6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64951b6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xacd51b71, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4951b74, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe2351b76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42151b78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8351b7b, SUC_lexicon_tags_39 },
    { 0x30f51b7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15151b80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c951b87, SUC_lexicon_tags_15 },
    { 0xd9151b87, SUC_lexicon_tags_14 },
    { 0xf3b51b89, SUC_lexicon_tags_2 },
    { 0x4a751b8a, SUC_lexicon_tags_23 },
    { 0xc7f51b8b, SUC_lexicon_tags_60 },
    { 0x60151b89, SUC_lexicon_tags_2 },
    { 0x56551b8d, SUC_lexicon_tags_17 },
    { 0xad551b8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde351b91, SUC_lexicon_tags_87 },
    { 0xf3551b92, SUC_lexicon_tags_31 },
    { 0x18351b93, SUC_lexicon_tags_30 },
    { 0xb4351b93, SUC_lexicon_tags_2 },
    { 0x6cd51b95, SUC_lexicon_tags_11 },
    { 0x7b751b96, SUC_lexicon_tags_17 },
    { 0x60951b96, SUC_lexicon_tags_33 },
    { 0x7bf51b96, SUC_lexicon_tags_2 },
    { 0xf1351b99, SUC_lexicon_tags_14 },
    { 0x98551b96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5151b9c, SUC_lexicon_tags_2 },
    { 0x49351b9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80751ba0, SUC_lexicon_tags_0 },
    { 0xc4d51ba0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4551ba5, SUC_lexicon_tags_27 },
    { 0xe9551ba6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f51baa, SUC_lexicon_tags_7 },
    { 0xf0151bab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67751bb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2b51bb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23d51bb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b351bb9, SUC_lexicon_tags_25 },
    { 0xb8f51bb9, SUC_lexicon_tags_14 },
    { 0x58151bb9, SUC_lexicon_tags_29 },
    { 0xb951bbc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca751bc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55951bcb, SUC_lexicon_tags_15 },
    { 0xac551bcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8951bce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd6551bd0, SUC_lexicon_tags_6 },
    { 0xcd351bd1, SUC_lexicon_tags_63 },
    { 0xb3751bd1, SUC_lexicon_tags_53 },
    { 0x69d51bd1, SUC_lexicon_tags_1167 },
    { 0x68351bd2, SUC_lexicon_tags_12 },
    { 0x35351bd5, SUC_lexicon_tags_12 },
    { 0x1751bd1, SUC_lexicon_tags_2 },
    { 0xf8751bd7, SUC_lexicon_tags_81 },
    { 0xf5751bd6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73b51bda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba751bdd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9cd51bdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdf51be1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d151be4, SUC_lexicon_tags_2 },
    { 0x8751be5, SUC_lexicon_tags_55 },
    { 0xb2551be6, SUC_lexicon_tags_87 },
    { 0xe1f51be4, SUC_lexicon_tags_47 },
    { 0xca151be8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea751bea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8d51bec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28d51bee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d51bf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52351bf7, SUC_lexicon_tags_19 },
    { 0x62151bf8, SUC_lexicon_tags_12 },
    { 0xad151bf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d51bfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20551c00, SUC_lexicon_tags_28 },
    { 0xf0351c00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12b51c03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x58351c05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43151c08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a751c0b, SUC_lexicon_tags_7 },
    { 0xea351c0c, SUC_lexicon_tags_12 },
    { 0xacb51c0c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb51c11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d51c15, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xbef51c17, SUC_lexicon_tags_1 },
    { 0xf0151c17, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a751c20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7951c22, SUC_lexicon_tags_38 },
    { 0x61f51c23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x36f51c25, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xea151c28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdeb51c2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda751c2c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3751c2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44351c32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc551c34, SUC_lexicon_tags_7 },
    { 0x9a151c35, SUC_lexicon_tags_31 },
    { 0x7fd51c36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22151c3a, SUC_lexicon_tags_25 },
    { 0x2f551c3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd51c3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d351c41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f751c46, SUC_lexicon_tags_29 },
    { 0x79151c47, SUC_lexicon_tags_46 },
    { 0x8c351c48, SUC_lexicon_tags_14 },
    { 0x9b751c49, SUC_lexicon_tags_4 },
    { 0x1ed51c49, SUC_lexicon_tags_31 },
    { 0x32f51c4b, SUC_lexicon_tags_7 },
    { 0xbc751c48, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b51c51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2eb51c53, SUC_lexicon_tags_37 },
    { 0xe5351c54, SUC_lexicon_tags_450 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d151c57, SUC_lexicon_tags_11 },
    { 0x72b51c58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59951c5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d351c60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25351c62, SUC_lexicon_tags_17 },
    { 0x15951c63, SUC_lexicon_tags_28 },
    { 0xc3551c63, SUC_lexicon_tags_19 },
    { 0xdf351c65, SUC_lexicon_tags_31 },
    { 0x7f951c66, SUC_lexicon_tags_0 },
    { 0x4a551c65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf51c6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20751c70, SUC_lexicon_tags_29 },
    { 0x33b51c70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd151c75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7f51c77, SUC_lexicon_tags_12 },
    { 0x7af51c78, SUC_lexicon_tags_25 },
    { 0xe9b51c78, SUC_lexicon_tags_23 },
    { 0xb7b51c7a, SUC_lexicon_tags_15 },
    { 0xe3151c7b, SUC_lexicon_tags_48 },
    { 0x4c951c7b, SUC_lexicon_tags_7 },
    { 0xbf51c7b, SUC_lexicon_tags_37 },
    { 0x7751c7e, SUC_lexicon_tags_1 },
    { 0x1a951c7f, SUC_lexicon_tags_1 },
    { 0xb2f51c80, SUC_lexicon_tags_23 },
    { 0x7e551c81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6d51c83, SUC_lexicon_tags_85 },
    { 0x4a951c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f551c87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30151c8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d551c8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49551c90, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfa551c92, SUC_lexicon_tags_49 },
    { 0xa0d51c92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a551c96, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39951c9e, SUC_lexicon_tags_7 },
    { 0x85951c9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d951ca4, SUC_lexicon_tags_31 },
    { 0xb9151ca4, SUC_lexicon_tags_25 },
    { 0xad551ca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62d51ca8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb751caa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6d351cac, SUC_lexicon_tags_17 },
    { 0x96751cac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce551caf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14951cbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8351cbe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab51cc4, SUC_lexicon_tags_49 },
    { 0xd51cc5, SUC_lexicon_tags_27 },
    { 0x70d51cc5, SUC_lexicon_tags_38 },
    { 0xcdb51cc7, SUC_lexicon_tags_17 },
    { 0x26951cc8, SUC_lexicon_tags_17 },
    { 0xe3d51cc9, SUC_lexicon_tags_16 },
    { 0x99b51cca, SUC_lexicon_tags_8 },
    { 0x1ff51ccb, SUC_lexicon_tags_1 },
    { 0xfa951ccc, SUC_lexicon_tags_6 },
    { 0x65351cc9, SUC_lexicon_tags_118 },
    { 0x32551cc7, SUC_lexicon_tags_12 },
    { 0x29751ccf, SUC_lexicon_tags_47 },
    { 0x17951cd0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f151cd6, SUC_lexicon_tags_15 },
    { 0x87951cd6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f51cdb, SUC_lexicon_tags_11 },
    { 0xbb751cdc, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7751ce2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdd51ce4, SUC_lexicon_tags_7 },
    { 0xd5951ce5, SUC_lexicon_tags_1 },
    { 0x1ab51ce5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4df51ce8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b51ceb, SUC_lexicon_tags_25 },
    { 0xb7951cec, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf351cfa, SUC_lexicon_tags_37 },
    { 0x5fd51cfb, SUC_lexicon_tags_29 },
    { 0xe0351cfb, SUC_lexicon_tags_1 },
    { 0x8f951cfb, SUC_lexicon_tags_129 },
    { 0x19d51cfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff51d03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45351d06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f51d09, SUC_lexicon_tags_17 },
    { 0xe151d0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33b51d0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd151d0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b51d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d51d17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d51d1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x93751d20, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b51d24, SUC_lexicon_tags_26 },
    { 0x22151d25, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3751d28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd951d2a, SUC_lexicon_tags_12 },
    { 0x9e751d2a, SUC_lexicon_tags_12 },
    { 0x17351d2c, SUC_lexicon_tags_34 },
    { 0xb3f51d2a, SUC_lexicon_tags_1 },
    { 0x65551d2c, SUC_lexicon_tags_9 },
    { 0x97951d2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a951d31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2351d35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc751d3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b51d44, SUC_lexicon_tags_12 },
    { 0xcff51d44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0b51d47, SUC_lexicon_tags_2 },
    { 0xd6351d48, SUC_lexicon_tags_38 },
    { 0x37b51d49, SUC_lexicon_tags_37 },
    { 0xf8151d48, SUC_lexicon_tags_45 },
    { 0xa4b51d4b, SUC_lexicon_tags_23 },
    { 0x3df51d48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6151d54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb51d5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b51d5e, SUC_lexicon_tags_15 },
    { 0xd5751d5e, SUC_lexicon_tags_0 },
    { 0x3f951d60, SUC_lexicon_tags_17 },
    { 0x84b51d61, SUC_lexicon_tags_65 },
    { 0x3151d61, SUC_lexicon_tags_15 },
    { 0x40b51d60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3951d68, SUC_lexicon_tags_12 },
    { 0xb3d51d69, SUC_lexicon_tags_84 },
    { 0x16351d69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f951d6c, SUC_lexicon_tags_1 },
    { 0x8c751d6c, SUC_lexicon_tags_2 },
    { 0x48b51d6c, SUC_lexicon_tags_68 },
    { 0xd2f51d6f, SUC_lexicon_tags_15 },
    { 0x8c351d6c, SUC_lexicon_tags_1 },
    { 0x5d51d71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12951d76, SUC_lexicon_tags_12 },
    { 0x3db51d77, SUC_lexicon_tags_1 },
    { 0x9b751d76, SUC_lexicon_tags_7 },
    { 0x85751d79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf951d7c, SUC_lexicon_tags_37 },
    { 0xc1b51d7c, SUC_lexicon_tags_45 },
    { 0xf4551d7e, SUC_lexicon_tags_39 },
    { 0xbfb51d7d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x65f51d81, SUC_lexicon_tags_15 },
    { 0x92151d81, SUC_lexicon_tags_7 },
    { 0x7551d81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b51d87, SUC_lexicon_tags_47 },
    { 0x751d88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60351d8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdad51d8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1551d90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7351d93, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x41551d95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41751d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4751d9c, SUC_lexicon_tags_1 },
    { 0xe8351d9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0d51d9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7351da4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d51da7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b551da9, SUC_lexicon_tags_55 },
    { 0x89351da9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x21751dac, SUC_lexicon_tags_2 },
    { 0x69951dad, SUC_lexicon_tags_46 },
    { 0x8e951dae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d351db1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x68551db3, SUC_lexicon_tags_45 },
    { 0x10b51db3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2551dbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb51dbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27351dc1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x66551dc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53b51dc6, SUC_lexicon_tags_37 },
    { 0x97b51dc7, SUC_lexicon_tags_37 },
    { 0xf2551dc8, SUC_lexicon_tags_33 },
    { 0x6b351dc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42151dcb, SUC_lexicon_tags_46 },
    { 0xb9151dcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcdd51dce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd51dd4, SUC_lexicon_tags_61 },
    { 0xe5151dd5, SUC_lexicon_tags_36 },
    { 0xd7551dd6, SUC_lexicon_tags_7 },
    { 0x41b51dd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d51ddc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3ff51dde, SUC_lexicon_tags_0 },
    { 0x60351dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4751de2, SUC_lexicon_tags_7 },
    { 0xb9d51de3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3551de7, SUC_lexicon_tags_25 },
    { 0x7351de7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef951deb, SUC_lexicon_tags_12 },
    { 0x751dec, SUC_lexicon_tags_12 },
    { 0x9a551dec, SUC_lexicon_tags_0 },
    { 0x71d51dee, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac551df3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42751df8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced51e03, SUC_lexicon_tags_0 },
    { 0x24151e04, SUC_lexicon_tags_14 },
    { 0x7e751e04, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d51e08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed51e0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c951e0c, SUC_lexicon_tags_30 },
    { 0xa9151e0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f751e12, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb51e17, SUC_lexicon_tags_49 },
    { 0x2fd51e18, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7551e1b, SUC_lexicon_tags_9 },
    { 0xed751e1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89b51e1e, SUC_lexicon_tags_60 },
    { 0x9c951e1e, SUC_lexicon_tags_37 },
    { 0x31f51e20, SUC_lexicon_tags_37 },
    { 0x74151e21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75551e23, SUC_lexicon_tags_11 },
    { 0x5d951e24, SUC_lexicon_tags_18 },
    { 0x5b951e25, SUC_lexicon_tags_37 },
    { 0xd7951e26, SUC_lexicon_tags_25 },
    { 0xe6d51e26, SUC_lexicon_tags_15 },
    { 0x58d51e25, SUC_lexicon_tags_27 },
    { 0xb8551e29, SUC_lexicon_tags_63 },
    { 0x54b51e2a, SUC_lexicon_tags_29 },
    { 0x29b51e2b, SUC_lexicon_tags_2 },
    { 0x38151e2c, SUC_lexicon_tags_25 },
    { 0xe3d51e2d, SUC_lexicon_tags_17 },
    { 0x12151e2d, SUC_lexicon_tags_11 },
    { 0x8bf51e2f, SUC_lexicon_tags_23 },
    { 0x71351e30, SUC_lexicon_tags_23 },
    { 0x8bf51e31, SUC_lexicon_tags_351 },
    { 0x74751e32, SUC_lexicon_tags_30 },
    { 0xbaf51e32, SUC_lexicon_tags_37 },
    { 0x43351e2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb51e36, SUC_lexicon_tags_9 },
    { 0x6d351e36, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x4f751e39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79951e3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0d51e3d, SUC_lexicon_tags_11 },
    { 0x9e351e3d, SUC_lexicon_tags_15 },
    { 0x3a351e3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f951e44, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa751e4a, SUC_lexicon_tags_17 },
    { 0x5e951e4b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x59351e4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd951e51, SUC_lexicon_tags_25 },
    { 0xe5f51e52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e351e54, SUC_lexicon_tags_14 },
    { 0x83551e55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f351e5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1351e5d, SUC_lexicon_tags_20 },
    { 0x27751e5d, SUC_lexicon_tags_30 },
    { 0x23951e5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf51e63, SUC_lexicon_tags_30 },
    { 0xc4151e63, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x66351e66, SUC_lexicon_tags_21 },
    { 0xdb151e66, SUC_lexicon_tags_17 },
    { 0x6b751e68, SUC_lexicon_tags_14 },
    { 0xb8f51e66, SUC_lexicon_tags_224 },
    { 0x5a551e69, SUC_lexicon_tags_25 },
    { 0x60951e6b, SUC_lexicon_tags_37 },
    { 0x43551e6c, SUC_lexicon_tags_3 },
    { 0x8f551e6c, SUC_lexicon_tags_0 },
    { 0x72b51e6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf951e77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89351e7b, SUC_lexicon_tags_37 },
    { 0xb6d51e7c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe1951e7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2751e80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc551e85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90551e8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59351e90, SUC_lexicon_tags_23 },
    { 0xa3151e91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3351e95, SUC_lexicon_tags_4 },
    { 0x4e751e95, SUC_lexicon_tags_1 },
    { 0x25d51e95, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd51e9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30b51ea0, SUC_lexicon_tags_27 },
    { 0x45d51ea1, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb51ea4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a951ea9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf51eac, SUC_lexicon_tags_69 },
    { 0x7b51ead, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40551eb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f51eb5, SUC_lexicon_tags_0 },
    { 0xdc751eb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3eb51eb8, SUC_lexicon_tags_2 },
    { 0x35951eb8, SUC_lexicon_tags_11 },
    { 0xc5151eb8, SUC_lexicon_tags_37 },
    { 0xe6b51ebb, SUC_lexicon_tags_2 },
    { 0x3cd51ebb, SUC_lexicon_tags_37 },
    { 0x99751eba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34751ec0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb1551ec2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7151ec9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b51ecc, SUC_lexicon_tags_2 },
    { 0xe1f51ecd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xcff51ecf, SUC_lexicon_tags_2 },
    { 0x88f51ed0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f51ed6, SUC_lexicon_tags_29 },
    { 0xee951ed7, SUC_lexicon_tags_23 },
    { 0xe0351ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f51edd, SUC_lexicon_tags_37 },
    { 0xcf951edd, SUC_lexicon_tags_37 },
    { 0xbd951edd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5df51ee1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x951ee4, SUC_lexicon_tags_12 },
    { 0x3a351ee4, SUC_lexicon_tags_66 },
    { 0xa4151ee4, SUC_lexicon_tags_17 },
    { 0xdbd51ee7, SUC_lexicon_tags_2 },
    { 0x8c551ee6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb551eed, SUC_lexicon_tags_68 },
    { 0x22f51eee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50551ef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2551ef6, SUC_lexicon_tags_16 },
    { 0x45f51ef7, SUC_lexicon_tags_17 },
    { 0xea751ef6, SUC_lexicon_tags_1 },
    { 0xf4f51ef9, SUC_lexicon_tags_8 },
    { 0xf3951efa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb51efe, SUC_lexicon_tags_14 },
    { 0xa8151eff, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e751f05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b51f08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b351f0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f51f10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4fb51f12, SUC_lexicon_tags_25 },
    { 0x50751f13, SUC_lexicon_tags_42 },
    { 0xbcd51f13, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0xe0951f16, SUC_lexicon_tags_2 },
    { 0xf2751f16, SUC_lexicon_tags_1 },
    { 0xbe951f18, SUC_lexicon_tags_26 },
    { 0xdbd51f19, SUC_lexicon_tags_60 },
    { 0x52751f19, SUC_lexicon_tags_4 },
    { 0xb6d51f16, SUC_lexicon_tags_47 },
    { 0x32551f1b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x44551f1e, SUC_lexicon_tags_1 },
    { 0xc5b51f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67151f24, SUC_lexicon_tags_17 },
    { 0xa2d51f24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9dd51f27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78751f2b, SUC_lexicon_tags_11 },
    { 0x61551f2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f51f30, SUC_lexicon_tags_57 },
    { 0xc5351f30, SUC_lexicon_tags_37 },
    { 0xb5151f32, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xca751f34, SUC_lexicon_tags_53 },
    { 0x79b51f35, SUC_lexicon_tags_17 },
    { 0x8cb51f36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38751f38, SUC_lexicon_tags_27 },
    { 0x55d51f38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4751f40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99351f46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91751f4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb951f4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30751f52, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3c751f54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x95151f57, SUC_lexicon_tags_15 },
    { 0x9df51f58, SUC_lexicon_tags_15 },
    { 0x58d51f59, SUC_lexicon_tags_168 },
    { 0xf8d51f57, SUC_lexicon_tags_37 },
    { 0x6d951f58, SUC_lexicon_tags_11 },
    { 0x5f951f57, SUC_lexicon_tags_17 },
    { 0xa1351f5d, SUC_lexicon_tags_32 },
    { 0x1cd51f5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd51f62, SUC_lexicon_tags_11 },
    { 0x9b951f63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98351f65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e751f6b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26f51f6d, SUC_lexicon_tags_26 },
    { 0xb7551f6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3e551f70, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed51f78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9fb51f7a, SUC_lexicon_tags_35 },
    { 0x5b351f7b, SUC_lexicon_tags_57 },
    { 0x8c951f7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac151f7e, SUC_lexicon_tags_28 },
    { 0xeb51f7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e951f87, SUC_lexicon_tags_11 },
    { 0xaf151f88, SUC_lexicon_tags_60 },
    { 0xedd51f89, SUC_lexicon_tags_6 },
    { 0x47b51f8a, SUC_lexicon_tags_63 },
    { 0x4a151f8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d51f8e, SUC_lexicon_tags_4 },
    { 0x59751f8f, SUC_lexicon_tags_12 },
    { 0xb0551f90, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f51f93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf151f98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87951f9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd351f9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb4551f9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3951fa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26151fa7, SUC_lexicon_tags_9 },
    { 0x24551fa8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3351faa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36751fac, SUC_lexicon_tags_16 },
    { 0xe8d51fad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14151fb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x51751fb4, SUC_lexicon_tags_46 },
    { 0x3e551fb5, SUC_lexicon_tags_12 },
    { 0x41551fb6, SUC_lexicon_tags_10 },
    { 0x79351fb6, SUC_lexicon_tags_4 },
    { 0xcf951fb5, SUC_lexicon_tags_15 },
    { 0xa951fb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5751fbd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9951fc0, SUC_lexicon_tags_4 },
    { 0x56351fc0, SUC_lexicon_tags_1 },
    { 0x10b51fc2, SUC_lexicon_tags_34 },
    { 0x4d751fc2, SUC_lexicon_tags_37 },
    { 0x99151fc2, SUC_lexicon_tags_9 },
    { 0x4551fc5, SUC_lexicon_tags_8 },
    { 0x43351fc6, SUC_lexicon_tags_45 },
    { 0x80751fc5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb1351fc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83351fcf, SUC_lexicon_tags_37 },
    { 0x15751fd0, SUC_lexicon_tags_47 },
    { 0x2d551fd1, SUC_lexicon_tags_30 },
    { 0x93151fd2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a951fd6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x951fd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd151fdb, SUC_lexicon_tags_872 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0551fdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f51fe2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f51fe5, SUC_lexicon_tags_14 },
    { 0x3cd51fe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed51fe9, SUC_lexicon_tags_12 },
    { 0x74751fea, SUC_lexicon_tags_12 },
    { 0xb6f51feb, SUC_lexicon_tags_28 },
    { 0x36d51fea, SUC_lexicon_tags_17 },
    { 0x5e751fe9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc751ff0, SUC_lexicon_tags_12 },
    { 0xf4551ff0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d51ff4, SUC_lexicon_tags_45 },
    { 0x2bf51ff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf351ff9, SUC_lexicon_tags_17 },
    { 0x65751ffa, SUC_lexicon_tags_17 },
    { 0x7b751ffb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb51fff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x98f52001, SUC_lexicon_tags_17 },
    { 0x22552002, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb1b52004, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a952006, SUC_lexicon_tags_7 },
    { 0x5b152007, SUC_lexicon_tags_5 },
    { 0xd4d52007, SUC_lexicon_tags_25 },
    { 0xf1d52008, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9375200c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf52011, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x92152014, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b752017, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6d52019, SUC_lexicon_tags_40 },
    { 0x1115201a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b5201d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b52022, SUC_lexicon_tags_25 },
    { 0x80d52022, SUC_lexicon_tags_65 },
    { 0x8d152024, SUC_lexicon_tags_77 },
    { 0x5752025, SUC_lexicon_tags_14 },
    { 0x27952026, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54952029, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9475202b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8d55202d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd615202f, SUC_lexicon_tags_33 },
    { 0x8c35202f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d52034, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23352039, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc735203c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7d35203e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82552043, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8d352045, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f5204b, SUC_lexicon_tags_37 },
    { 0xb0f5204b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7552050, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa952054, SUC_lexicon_tags_11 },
    { 0xf0752055, SUC_lexicon_tags_2 },
    { 0x5d952056, SUC_lexicon_tags_17 },
    { 0xd952056, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2195205a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f5205e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6152066, SUC_lexicon_tags_4 },
    { 0xba952067, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd75206c, SUC_lexicon_tags_55 },
    { 0xbbd5206d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f52073, SUC_lexicon_tags_49 },
    { 0xb1d52074, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0x81152077, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff5207b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6335207d, SUC_lexicon_tags_11 },
    { 0x99b5207e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b52084, SUC_lexicon_tags_11 },
    { 0x9cd52085, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9352087, SUC_lexicon_tags_1 },
    { 0x67952087, SUC_lexicon_tags_52 },
    { 0x82f52089, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2915208f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa6b52091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f152096, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x83f52098, SUC_lexicon_tags_60 },
    { 0x63952099, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0x8355209b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6195209f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9a1520a1, SUC_lexicon_tags_84 },
    { 0xf77520a1, SUC_lexicon_tags_37 },
    { 0x2b3520a3, SUC_lexicon_tags_34 },
    { 0xb05520a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf2f520a6, SUC_lexicon_tags_7 },
    { 0x3c7520a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5520ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa93520ae, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25520b2, SUC_lexicon_tags_37 },
    { 0x6f3520b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc39520b5, SUC_lexicon_tags_32 },
    { 0x685520b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31520ba, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd520be, SUC_lexicon_tags_25 },
    { 0x6b1520bf, SUC_lexicon_tags_1 },
    { 0xf87520c0, SUC_lexicon_tags_26 },
    { 0xe61520c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7520c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1520c8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x487520cb, SUC_lexicon_tags_87 },
    { 0xdf9520cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f520cf, SUC_lexicon_tags_7 },
    { 0x7b3520d0, SUC_lexicon_tags_34 },
    { 0x77f520d1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7ab520d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813520d7, SUC_lexicon_tags_30 },
    { 0x925520d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb73520da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df520e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183520e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x487520e8, SUC_lexicon_tags_87 },
    { 0x18d520e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab520f0, SUC_lexicon_tags_60 },
    { 0xb1b520f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8d520f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b520f6, SUC_lexicon_tags_2 },
    { 0xd1d520f7, SUC_lexicon_tags_8 },
    { 0x303520f8, SUC_lexicon_tags_87 },
    { 0x727520f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11520fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d52100, SUC_lexicon_tags_21 },
    { 0xd52101, SUC_lexicon_tags_11 },
    { 0xa8352102, SUC_lexicon_tags_77 },
    { 0x1b352103, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcc152105, SUC_lexicon_tags_11 },
    { 0x8d952105, SUC_lexicon_tags_71 },
    { 0x20552107, SUC_lexicon_tags_1 },
    { 0x69152108, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb275210f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e752111, SUC_lexicon_tags_4 },
    { 0xe5d52111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebd52114, SUC_lexicon_tags_1 },
    { 0xf9b52115, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86552119, SUC_lexicon_tags_47 },
    { 0x1695211a, SUC_lexicon_tags_25 },
    { 0xc0752119, SUC_lexicon_tags_11 },
    { 0xc515211a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b52121, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9352123, SUC_lexicon_tags_17 },
    { 0x57f52124, SUC_lexicon_tags_64 },
    { 0x58b52124, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6875212f, SUC_lexicon_tags_1 },
    { 0xb0352130, SUC_lexicon_tags_45 },
    { 0x69552130, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d52134, SUC_lexicon_tags_15 },
    { 0xe9d52135, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0152139, SUC_lexicon_tags_19 },
    { 0xfad5213a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xcbd5213c, SUC_lexicon_tags_14 },
    { 0x7635213d, SUC_lexicon_tags_60 },
    { 0x6815213c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f52142, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b952146, SUC_lexicon_tags_25 },
    { 0x21952147, SUC_lexicon_tags_39 },
    { 0xc1152146, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4175214c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34952150, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6e952152, SUC_lexicon_tags_15 },
    { 0xe0152152, SUC_lexicon_tags_15 },
    { 0x65952154, SUC_lexicon_tags_11 },
    { 0x66752152, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a352157, SUC_lexicon_tags_8 },
    { 0x95b52158, SUC_lexicon_tags_11 },
    { 0x69b52159, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4675215c, SUC_lexicon_tags_17 },
    { 0xb1b5215c, SUC_lexicon_tags_37 },
    { 0x8075215e, SUC_lexicon_tags_49 },
    { 0x1355215e, SUC_lexicon_tags_10 },
    { 0xd035215e, SUC_lexicon_tags_0 },
    { 0x52b52161, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb52164, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0752168, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x58f5216a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8715216d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffd5216f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb52172, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81952176, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c95217a, SUC_lexicon_tags_22 },
    { 0x9515217b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1d5217d, SUC_lexicon_tags_15 },
    { 0xae35217e, SUC_lexicon_tags_12 },
    { 0x9895217f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d52185, SUC_lexicon_tags_26 },
    { 0x60752186, SUC_lexicon_tags_25 },
    { 0xf3352186, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd5218e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef52190, SUC_lexicon_tags_26 },
    { 0x2af52190, SUC_lexicon_tags_6 },
    { 0xc8d52192, SUC_lexicon_tags_3 },
    { 0x25152192, SUC_lexicon_tags_7 },
    { 0xe7f52194, SUC_lexicon_tags_37 },
    { 0x1bb52193, SUC_lexicon_tags_25 },
    { 0xb8752196, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35952198, SUC_lexicon_tags_15 },
    { 0xd7552199, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda95219c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3521a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4cb521a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x32b521a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b3521a8, SUC_lexicon_tags_14 },
    { 0x7b1521a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187521ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55521b8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb75521ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x495521bd, SUC_lexicon_tags_11 },
    { 0xe77521bd, SUC_lexicon_tags_15 },
    { 0x591521bf, SUC_lexicon_tags_250 },
    { 0x879521be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x543521c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb521c8, SUC_lexicon_tags_30 },
    { 0x809521c9, SUC_lexicon_tags_11 },
    { 0x9b3521ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243521d0, SUC_lexicon_tags_14 },
    { 0x813521d0, SUC_lexicon_tags_4 },
    { 0x8e5521d2, SUC_lexicon_tags_7 },
    { 0x6b521d3, SUC_lexicon_tags_33 },
    { 0xd2b521d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1521d8, SUC_lexicon_tags_14 },
    { 0x5bd521d8, SUC_lexicon_tags_11 },
    { 0x755521da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x293521dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b521e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1521e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb521eb, SUC_lexicon_tags_27 },
    { 0xc71521ec, SUC_lexicon_tags_7 },
    { 0x72b521ec, SUC_lexicon_tags_16 },
    { 0x639521ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x13f521f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295521f4, SUC_lexicon_tags_15 },
    { 0xb7f521f4, SUC_lexicon_tags_25 },
    { 0xcbf521f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfb5521f8, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d521ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab52203, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x19152206, SUC_lexicon_tags_2 },
    { 0x86b52207, SUC_lexicon_tags_65 },
    { 0x22152207, SUC_lexicon_tags_28 },
    { 0x73152208, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b5220e, SUC_lexicon_tags_47 },
    { 0xdc55220f, SUC_lexicon_tags_80 },
    { 0xedb52210, SUC_lexicon_tags_38 },
    { 0xa8952211, SUC_lexicon_tags_2 },
    { 0x65952212, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d52216, SUC_lexicon_tags_17 },
    { 0xfbf52217, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12f52219, SUC_lexicon_tags_7 },
    { 0x25b5221a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9152223, SUC_lexicon_tags_25 },
    { 0xfc552223, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xed352226, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7555222e, SUC_lexicon_tags_7 },
    { 0x3a55222e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b52237, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e55223a, SUC_lexicon_tags_7 },
    { 0xc735223b, SUC_lexicon_tags_48 },
    { 0xd0b5223c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf52245, SUC_lexicon_tags_38 },
    { 0x62552246, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf7f52248, SUC_lexicon_tags_4 },
    { 0xf9d52249, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x22d5224b, SUC_lexicon_tags_27 },
    { 0xabd5224c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b752251, SUC_lexicon_tags_32 },
    { 0xa2952252, SUC_lexicon_tags_15 },
    { 0x81752251, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0952257, SUC_lexicon_tags_25 },
    { 0x74152257, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7815225a, SUC_lexicon_tags_11 },
    { 0x3dd5225b, SUC_lexicon_tags_45 },
    { 0x4b5225b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b5225f, SUC_lexicon_tags_12 },
    { 0x57352260, SUC_lexicon_tags_1 },
    { 0x6e352260, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xced52264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf152266, SUC_lexicon_tags_7 },
    { 0x2f752267, SUC_lexicon_tags_11 },
    { 0x5152268, SUC_lexicon_tags_29 },
    { 0x31352269, SUC_lexicon_tags_27 },
    { 0x75f52269, SUC_lexicon_tags_19 },
    { 0x6595226b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d95226e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa7952270, SUC_lexicon_tags_11 },
    { 0x88152270, SUC_lexicon_tags_40 },
    { 0x15352271, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b5227a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde15227e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df52285, SUC_lexicon_tags_11 },
    { 0x57b52286, SUC_lexicon_tags_19 },
    { 0x44552286, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55228f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40752294, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd52297, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3f52299, SUC_lexicon_tags_7 },
    { 0x5c75229a, SUC_lexicon_tags_37 },
    { 0x2bb52299, SUC_lexicon_tags_3 },
    { 0xe995229b, SUC_lexicon_tags_37 },
    { 0x35f5229d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9522a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff522a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4c9522a7, SUC_lexicon_tags_37 },
    { 0x24d522a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d522ac, SUC_lexicon_tags_37 },
    { 0x813522ad, SUC_lexicon_tags_121 },
    { 0x1a3522ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb41522b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5522c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1522c5, SUC_lexicon_tags_46 },
    { 0xa03522c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd522ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7522cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15522d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5d522d4, SUC_lexicon_tags_224 },
    { 0x96d522d4, SUC_lexicon_tags_21 },
    { 0x19522d6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x24d522d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f522dc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7522e0, SUC_lexicon_tags_0 },
    { 0xa21522e1, SUC_lexicon_tags_32 },
    { 0x373522e2, SUC_lexicon_tags_47 },
    { 0x1f522e3, SUC_lexicon_tags_30 },
    { 0x755522e4, SUC_lexicon_tags_46 },
    { 0xd1d522e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed522e9, SUC_lexicon_tags_68 },
    { 0x7c1522e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xae1522ec, SUC_lexicon_tags_26 },
    { 0x66b522ed, SUC_lexicon_tags_11 },
    { 0xa0f522ee, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163522f2, SUC_lexicon_tags_26 },
    { 0x239522f3, SUC_lexicon_tags_14 },
    { 0x4c7522f4, SUC_lexicon_tags_33 },
    { 0x58d522f4, SUC_lexicon_tags_60 },
    { 0x1b5522f6, SUC_lexicon_tags_30 },
    { 0x959522f7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf37522f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5522fd, SUC_lexicon_tags_37 },
    { 0xfb9522fd, SUC_lexicon_tags_33 },
    { 0x759522fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f52303, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6152306, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3b52308, SUC_lexicon_tags_14 },
    { 0x71952308, SUC_lexicon_tags_26 },
    { 0x17b5230a, SUC_lexicon_tags_0 },
    { 0xb555230a, SUC_lexicon_tags_366 },
    { 0x64552309, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25552310, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18752312, SUC_lexicon_tags_27 },
    { 0xb7f52313, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d52317, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f5231f, SUC_lexicon_tags_11 },
    { 0x5f95231f, SUC_lexicon_tags_2 },
    { 0xed952321, SUC_lexicon_tags_7 },
    { 0xe4352320, SUC_lexicon_tags_37 },
    { 0x69f52323, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe352325, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x12152327, SUC_lexicon_tags_25 },
    { 0xa8152328, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f55232b, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9152331, SUC_lexicon_tags_30 },
    { 0x5dd52331, SUC_lexicon_tags_45 },
    { 0x7d152333, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x15152335, SUC_lexicon_tags_25 },
    { 0x33952335, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x62f52338, SUC_lexicon_tags_15 },
    { 0xeb152339, SUC_lexicon_tags_11 },
    { 0x90b52338, SUC_lexicon_tags_9 },
    { 0xc3f5233b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d752342, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb52347, SUC_lexicon_tags_15 },
    { 0x23d52347, SUC_lexicon_tags_2 },
    { 0x97352348, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f52350, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5352353, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f52358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66f5235a, SUC_lexicon_tags_37 },
    { 0x25f5235a, SUC_lexicon_tags_39 },
    { 0x1fb5235c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xef5235e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe0f52360, SUC_lexicon_tags_12 },
    { 0x82152360, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65352363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd5236a, SUC_lexicon_tags_2 },
    { 0xf255236b, SUC_lexicon_tags_0 },
    { 0xb15236b, SUC_lexicon_tags_12 },
    { 0x4775236b, SUC_lexicon_tags_15 },
    { 0x8f95236e, SUC_lexicon_tags_47 },
    { 0x8475236f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d952374, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb152379, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x97d5237b, SUC_lexicon_tags_12 },
    { 0xaef5237c, SUC_lexicon_tags_35 },
    { 0x4e15237c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7f35237f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7552383, SUC_lexicon_tags_37 },
    { 0xa0352384, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf152388, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54f5238a, SUC_lexicon_tags_11 },
    { 0xfef5238b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0552396, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf5239a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f35239d, SUC_lexicon_tags_37 },
    { 0x8f75239d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x945523a1, SUC_lexicon_tags_11 },
    { 0xdc1523a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3523a5, SUC_lexicon_tags_49 },
    { 0x637523a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ef523a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeef523aa, SUC_lexicon_tags_17 },
    { 0xc47523aa, SUC_lexicon_tags_25 },
    { 0xf09523aa, SUC_lexicon_tags_126 },
    { 0x2e5523ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9523b2, SUC_lexicon_tags_14 },
    { 0xa5d523b3, SUC_lexicon_tags_27 },
    { 0x407523b3, SUC_lexicon_tags_8 },
    { 0xb39523b5, SUC_lexicon_tags_12 },
    { 0x5a3523b5, SUC_lexicon_tags_11 },
    { 0x465523b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb523ba, SUC_lexicon_tags_36 },
    { 0xc75523bb, SUC_lexicon_tags_19 },
    { 0xaff523bc, SUC_lexicon_tags_14 },
    { 0x18d523bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeab523bf, SUC_lexicon_tags_38 },
    { 0xa65523c0, SUC_lexicon_tags_37 },
    { 0xde5523c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5523c9, SUC_lexicon_tags_2 },
    { 0xc99523ca, SUC_lexicon_tags_43 },
    { 0xb15523ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85f523cd, SUC_lexicon_tags_37 },
    { 0xf03523ce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7523d4, SUC_lexicon_tags_32 },
    { 0x7db523d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef523d9, SUC_lexicon_tags_17 },
    { 0xcc7523da, SUC_lexicon_tags_6 },
    { 0x7cd523db, SUC_lexicon_tags_12 },
    { 0xe3d523dc, SUC_lexicon_tags_87 },
    { 0x35523dc, SUC_lexicon_tags_49 },
    { 0xbf523dd, SUC_lexicon_tags_35 },
    { 0x389523de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d523e2, SUC_lexicon_tags_37 },
    { 0xd9b523e2, SUC_lexicon_tags_26 },
    { 0x5c9523e3, SUC_lexicon_tags_23 },
    { 0xc35523e5, SUC_lexicon_tags_9 },
    { 0x21d523e3, SUC_lexicon_tags_15 },
    { 0xf39523e7, SUC_lexicon_tags_38 },
    { 0x3e5523e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x19523eb, SUC_lexicon_tags_2 },
    { 0x2cd523ec, SUC_lexicon_tags_37 },
    { 0xb7d523ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f523f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x795523f3, SUC_lexicon_tags_2 },
    { 0xee1523f3, SUC_lexicon_tags_60 },
    { 0xb23523f3, SUC_lexicon_tags_47 },
    { 0x7c5523f6, SUC_lexicon_tags_7 },
    { 0xab1523f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d523fc, SUC_lexicon_tags_37 },
    { 0x2d1523fd, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9552406, SUC_lexicon_tags_77 },
    { 0xf9f52406, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb955240e, SUC_lexicon_tags_6 },
    { 0x60f5240e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39552411, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7fb52413, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x65152416, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12b52418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5241c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6355241e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xacd52420, SUC_lexicon_tags_15 },
    { 0x88d52420, SUC_lexicon_tags_26 },
    { 0x70352422, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32152427, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5242a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10352433, SUC_lexicon_tags_3 },
    { 0x2952434, SUC_lexicon_tags_7 },
    { 0x8f352435, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d5243a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d15243d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd4f5243f, SUC_lexicon_tags_0 },
    { 0x77752440, SUC_lexicon_tags_236 },
    { 0x98352441, SUC_lexicon_tags_17 },
    { 0x30952442, SUC_lexicon_tags_17 },
    { 0x2a352442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73152446, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d52449, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f5244d, SUC_lexicon_tags_12 },
    { 0x7f15244e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9152450, SUC_lexicon_tags_25 },
    { 0xd5752451, SUC_lexicon_tags_39 },
    { 0x91552451, SUC_lexicon_tags_37 },
    { 0x38d52453, SUC_lexicon_tags_17 },
    { 0x77152453, SUC_lexicon_tags_37 },
    { 0x15d52455, SUC_lexicon_tags_17 },
    { 0xa3152452, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d5245a, SUC_lexicon_tags_4 },
    { 0x6ed5245b, SUC_lexicon_tags_19 },
    { 0xbb15245c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d952463, SUC_lexicon_tags_84 },
    { 0xcc552464, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d52468, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4495246e, SUC_lexicon_tags_7 },
    { 0x6775246f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9952471, SUC_lexicon_tags_25 },
    { 0x56952472, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfaf52474, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87352476, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5755247b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb55247d, SUC_lexicon_tags_25 },
    { 0xb315247e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3552480, SUC_lexicon_tags_87 },
    { 0x7c952481, SUC_lexicon_tags_39 },
    { 0x64152482, SUC_lexicon_tags_15 },
    { 0xb3352482, SUC_lexicon_tags_26 },
    { 0x91352484, SUC_lexicon_tags_48 },
    { 0xbcf52481, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6675248f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd52493, SUC_lexicon_tags_2 },
    { 0x5eb52494, SUC_lexicon_tags_11 },
    { 0x3ef52495, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9f952497, SUC_lexicon_tags_27 },
    { 0xd4352498, SUC_lexicon_tags_13 },
    { 0xd8d52499, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff524a0, SUC_lexicon_tags_61 },
    { 0xd85524a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535524a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3524b2, SUC_lexicon_tags_64 },
    { 0xb71524b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597524b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145524bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x845524c2, SUC_lexicon_tags_25 },
    { 0x363524c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d524c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef524d1, SUC_lexicon_tags_66 },
    { 0x4db524d2, SUC_lexicon_tags_7 },
    { 0x6a7524d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaab524d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x779524d7, SUC_lexicon_tags_8 },
    { 0xbe5524d8, SUC_lexicon_tags_45 },
    { 0xa6f524d9, SUC_lexicon_tags_37 },
    { 0xe75524da, SUC_lexicon_tags_49 },
    { 0x511524db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x649524dd, SUC_lexicon_tags_35 },
    { 0xb25524dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9d524e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7524e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x937524e7, SUC_lexicon_tags_25 },
    { 0xdf7524e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6d524ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x949524ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x339524ef, SUC_lexicon_tags_12 },
    { 0x17b524ef, SUC_lexicon_tags_16 },
    { 0x2cb524f1, SUC_lexicon_tags_23 },
    { 0xfff524f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x681524f5, SUC_lexicon_tags_60 },
    { 0xda9524f6, SUC_lexicon_tags_37 },
    { 0x29b524f6, SUC_lexicon_tags_37 },
    { 0x1a7524f8, SUC_lexicon_tags_29 },
    { 0x63d524f8, SUC_lexicon_tags_12 },
    { 0xc6b524fa, SUC_lexicon_tags_12 },
    { 0x549524fa, SUC_lexicon_tags_11 },
    { 0x41b524fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82952501, SUC_lexicon_tags_39 },
    { 0x63f52502, SUC_lexicon_tags_18 },
    { 0xe9752503, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8752505, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0752509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc95250b, SUC_lexicon_tags_2 },
    { 0xe755250c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa95250e, SUC_lexicon_tags_55 },
    { 0xca75250f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b52513, SUC_lexicon_tags_47 },
    { 0xf3152514, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x40952516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf5251a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x46f5251c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8b5251e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d52522, SUC_lexicon_tags_11 },
    { 0x65352522, SUC_lexicon_tags_25 },
    { 0xba152524, SUC_lexicon_tags_9 },
    { 0x64f52525, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbab52527, SUC_lexicon_tags_18 },
    { 0x90752527, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15252b, SUC_lexicon_tags_29 },
    { 0xdfb5252b, SUC_lexicon_tags_8 },
    { 0xbbd5252c, SUC_lexicon_tags_11 },
    { 0x3015252d, SUC_lexicon_tags_0 },
    { 0xbbf5252e, SUC_lexicon_tags_27 },
    { 0x52352530, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc552536, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50d52538, SUC_lexicon_tags_39 },
    { 0x73752538, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35253c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a752540, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf8952542, SUC_lexicon_tags_19 },
    { 0xd3752543, SUC_lexicon_tags_9 },
    { 0xf4352543, SUC_lexicon_tags_17 },
    { 0xa4b52543, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe9152547, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf5254b, SUC_lexicon_tags_11 },
    { 0x3c95254c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfef5254e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6b52550, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5152557, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e352559, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b52562, SUC_lexicon_tags_15 },
    { 0x79152563, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x73752565, SUC_lexicon_tags_17 },
    { 0xc2152566, SUC_lexicon_tags_736 },
    { 0x4af52566, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8675256a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7755256d, SUC_lexicon_tags_11 },
    { 0x7f55256e, SUC_lexicon_tags_64 },
    { 0x1af5256f, SUC_lexicon_tags_60 },
    { 0x5e352570, SUC_lexicon_tags_87 },
    { 0x8b752570, SUC_lexicon_tags_27 },
    { 0x69952570, SUC_lexicon_tags_1 },
    { 0xc9952571, SUC_lexicon_tags_1 },
    { 0xab52574, SUC_lexicon_tags_56 },
    { 0x4952575, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x61552578, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6795257b, SUC_lexicon_tags_37 },
    { 0xa8d5257c, SUC_lexicon_tags_26 },
    { 0x8415257d, SUC_lexicon_tags_11 },
    { 0x9275257c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61352583, SUC_lexicon_tags_21 },
    { 0x80952584, SUC_lexicon_tags_25 },
    { 0x2f752584, SUC_lexicon_tags_25 },
    { 0x1952584, SUC_lexicon_tags_39 },
    { 0x65b52587, SUC_lexicon_tags_9 },
    { 0x65752587, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x5775258a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b52591, SUC_lexicon_tags_68 },
    { 0x47b52592, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f52597, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21525a1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3525a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x807525a7, SUC_lexicon_tags_34 },
    { 0xac9525a7, SUC_lexicon_tags_15 },
    { 0x973525a9, SUC_lexicon_tags_15 },
    { 0xb3f525a9, SUC_lexicon_tags_27 },
    { 0xf9d525ab, SUC_lexicon_tags_7 },
    { 0x161525ab, SUC_lexicon_tags_11 },
    { 0xc1d525ab, SUC_lexicon_tags_12 },
    { 0x725525ae, SUC_lexicon_tags_37 },
    { 0x61525a8, SUC_lexicon_tags_11 },
    { 0x90f525ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5f9525b2, SUC_lexicon_tags_23 },
    { 0x637525b3, SUC_lexicon_tags_7 },
    { 0x747525b4, SUC_lexicon_tags_2 },
    { 0x16b525b4, SUC_lexicon_tags_69 },
    { 0x52b525b6, SUC_lexicon_tags_37 },
    { 0x5d525b6, SUC_lexicon_tags_7 },
    { 0x959525b7, SUC_lexicon_tags_10 },
    { 0x79525b4, SUC_lexicon_tags_47 },
    { 0xe17525b4, SUC_lexicon_tags_14 },
    { 0xc75525b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b525c3, SUC_lexicon_tags_0 },
    { 0x4a3525c3, SUC_lexicon_tags_31 },
    { 0xa25525c5, SUC_lexicon_tags_11 },
    { 0x905525c3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x18d525c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb53525ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7d525cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f525d1, SUC_lexicon_tags_27 },
    { 0x2a1525d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8b525d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a3525d6, SUC_lexicon_tags_77 },
    { 0x24d525d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1525dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95525df, SUC_lexicon_tags_38 },
    { 0xee5525e0, SUC_lexicon_tags_63 },
    { 0xb8b525e1, SUC_lexicon_tags_11 },
    { 0x1c1525e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b525e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e7525e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd525f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9525f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ab525f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x651525fb, SUC_lexicon_tags_32 },
    { 0x613525fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3525ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x97352601, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8e952603, SUC_lexicon_tags_47 },
    { 0x68552604, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xf952606, SUC_lexicon_tags_37 },
    { 0x3e952607, SUC_lexicon_tags_6 },
    { 0xeb752606, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b5260c, SUC_lexicon_tags_30 },
    { 0x6db5260d, SUC_lexicon_tags_15 },
    { 0xe035260d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d52612, SUC_lexicon_tags_43 },
    { 0x2bb52612, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14752618, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5335261a, SUC_lexicon_tags_23 },
    { 0xf575261a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b35261e, SUC_lexicon_tags_17 },
    { 0xfad5261e, SUC_lexicon_tags_7 },
    { 0xa7352620, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd52623, SUC_lexicon_tags_39 },
    { 0xf1752623, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba752626, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f5262c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c752632, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e152634, SUC_lexicon_tags_12 },
    { 0x8c752635, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa352638, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6d5263a, SUC_lexicon_tags_37 },
    { 0xa3d5263a, SUC_lexicon_tags_35 },
    { 0xfd15263a, SUC_lexicon_tags_1 },
    { 0xed35263a, SUC_lexicon_tags_29 },
    { 0xc1f5263e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb52643, SUC_lexicon_tags_48 },
    { 0xfd752643, SUC_lexicon_tags_1252 },
    { 0, NULL },
    { 0x9df52646, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb52649, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb5264c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5175264e, SUC_lexicon_tags_15 },
    { 0xc6b5264e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3152651, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4a352653, SUC_lexicon_tags_25 },
    { 0x99d52653, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac752659, SUC_lexicon_tags_12 },
    { 0xa8b52659, SUC_lexicon_tags_2 },
    { 0xf6f5265b, SUC_lexicon_tags_21 },
    { 0x7015265a, SUC_lexicon_tags_38 },
    { 0x8ad5265a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd52660, SUC_lexicon_tags_11 },
    { 0x7af52661, SUC_lexicon_tags_1 },
    { 0xde152661, SUC_lexicon_tags_15 },
    { 0xbcb52660, SUC_lexicon_tags_12 },
    { 0x4eb52664, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22752667, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af5266a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfb5266c, SUC_lexicon_tags_37 },
    { 0x2dd5266d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3752674, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6815267d, SUC_lexicon_tags_27 },
    { 0x5215267e, SUC_lexicon_tags_37 },
    { 0x6295267e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c552684, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f952687, SUC_lexicon_tags_35 },
    { 0x8a152688, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1b5268a, SUC_lexicon_tags_46 },
    { 0x435268b, SUC_lexicon_tags_14 },
    { 0xf495268c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2952692, SUC_lexicon_tags_6 },
    { 0xd9552692, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54152697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec352699, SUC_lexicon_tags_38 },
    { 0xe8352699, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf5269e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7b526a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57526a4, SUC_lexicon_tags_23 },
    { 0x91526a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90d526a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77526aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa09526ac, SUC_lexicon_tags_1 },
    { 0xd39526ad, SUC_lexicon_tags_7 },
    { 0x4bd526ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa89526b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe3b526b2, SUC_lexicon_tags_10 },
    { 0x835526b3, SUC_lexicon_tags_7 },
    { 0xcdb526b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87526c4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7cd526c6, SUC_lexicon_tags_25 },
    { 0xe4f526c6, SUC_lexicon_tags_17 },
    { 0xb4f526c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb27526ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129526cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97526d3, SUC_lexicon_tags_12 },
    { 0xc25526d4, SUC_lexicon_tags_19 },
    { 0x389526d4, SUC_lexicon_tags_25 },
    { 0xfdd526d6, SUC_lexicon_tags_10 },
    { 0xb3526d6, SUC_lexicon_tags_30 },
    { 0x2cb526d6, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad526db, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa9526dd, SUC_lexicon_tags_61 },
    { 0xf23526de, SUC_lexicon_tags_11 },
    { 0x79f526df, SUC_lexicon_tags_60 },
    { 0xb07526e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf526e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1bf526e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f526e9, SUC_lexicon_tags_17 },
    { 0xeb7526ea, SUC_lexicon_tags_80 },
    { 0x6dd526ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f526f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3526fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ab526ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e352706, SUC_lexicon_tags_30 },
    { 0x42552707, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf835270a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x7995270c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4f5270e, SUC_lexicon_tags_12 },
    { 0x26f5270e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf552711, SUC_lexicon_tags_50 },
    { 0xed752711, SUC_lexicon_tags_37 },
    { 0x33352713, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfbb52715, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5e352717, SUC_lexicon_tags_1 },
    { 0x46d52717, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc675271e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af52722, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebf52724, SUC_lexicon_tags_68 },
    { 0x53f52725, SUC_lexicon_tags_2 },
    { 0x3d552724, SUC_lexicon_tags_37 },
    { 0x96752727, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x49352729, SUC_lexicon_tags_23 },
    { 0x8ab52729, SUC_lexicon_tags_2 },
    { 0x3c55272b, SUC_lexicon_tags_18 },
    { 0x4335272a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d75272f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64552734, SUC_lexicon_tags_14 },
    { 0x74152734, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd52739, SUC_lexicon_tags_15 },
    { 0x7e552739, SUC_lexicon_tags_2 },
    { 0x635273a, SUC_lexicon_tags_92 },
    { 0xac15273c, SUC_lexicon_tags_82 },
    { 0x3c15273d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f352745, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8952749, SUC_lexicon_tags_17 },
    { 0x82d5274a, SUC_lexicon_tags_12 },
    { 0x7515274b, SUC_lexicon_tags_83 },
    { 0x47f5274c, SUC_lexicon_tags_29 },
    { 0x235274d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f152750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda152752, SUC_lexicon_tags_28 },
    { 0x90352752, SUC_lexicon_tags_49 },
    { 0x3c952754, SUC_lexicon_tags_14 },
    { 0x7a952754, SUC_lexicon_tags_38 },
    { 0x23752754, SUC_lexicon_tags_47 },
    { 0x2ed52754, SUC_lexicon_tags_14 },
    { 0x77d52755, SUC_lexicon_tags_2 },
    { 0xd9f52758, SUC_lexicon_tags_48 },
    { 0x2bb52752, SUC_lexicon_tags_85 },
    { 0x635275b, SUC_lexicon_tags_17 },
    { 0x2935275c, SUC_lexicon_tags_7 },
    { 0x6e15275d, SUC_lexicon_tags_38 },
    { 0x6f75275b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x58d52760, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd552762, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x46152765, SUC_lexicon_tags_25 },
    { 0xf1f52766, SUC_lexicon_tags_63 },
    { 0xd3152766, SUC_lexicon_tags_31 },
    { 0xfc152768, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa995276b, SUC_lexicon_tags_23 },
    { 0x1d55276b, SUC_lexicon_tags_1 },
    { 0x6275276d, SUC_lexicon_tags_27 },
    { 0xab95276e, SUC_lexicon_tags_47 },
    { 0xa835276f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x8b352771, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0x57d52773, SUC_lexicon_tags_37 },
    { 0x14b52773, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4352776, SUC_lexicon_tags_46 },
    { 0x28752777, SUC_lexicon_tags_37 },
    { 0xb9f52778, SUC_lexicon_tags_37 },
    { 0x3bf52779, SUC_lexicon_tags_37 },
    { 0x6115277a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab5277d, SUC_lexicon_tags_14 },
    { 0x4b75277d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37352782, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d52789, SUC_lexicon_tags_23 },
    { 0x64b5278a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4e35278c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15752793, SUC_lexicon_tags_26 },
    { 0x76152794, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef52798, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd55279c, SUC_lexicon_tags_61 },
    { 0x8b35279d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4175279f, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741527a5, SUC_lexicon_tags_47 },
    { 0xd2f527a6, SUC_lexicon_tags_15 },
    { 0xbe5527a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9527ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0b527af, SUC_lexicon_tags_43 },
    { 0xd0b527b0, SUC_lexicon_tags_12 },
    { 0x101527b1, SUC_lexicon_tags_15 },
    { 0xb49527b2, SUC_lexicon_tags_19 },
    { 0xc3f527b1, SUC_lexicon_tags_1 },
    { 0xc83527b4, SUC_lexicon_tags_1 },
    { 0x3f7527b1, SUC_lexicon_tags_60 },
    { 0x4af527b5, SUC_lexicon_tags_1 },
    { 0x5d527b6, SUC_lexicon_tags_38 },
    { 0xd7d527b8, SUC_lexicon_tags_7 },
    { 0x1d5527b8, SUC_lexicon_tags_37 },
    { 0xd13527b8, SUC_lexicon_tags_12 },
    { 0x98b527bb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11527bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63527c3, SUC_lexicon_tags_15 },
    { 0xec5527c3, SUC_lexicon_tags_11 },
    { 0x939527c3, SUC_lexicon_tags_27 },
    { 0x285527c3, SUC_lexicon_tags_1 },
    { 0x795527c7, SUC_lexicon_tags_49 },
    { 0xb2b527c7, SUC_lexicon_tags_43 },
    { 0x4a9527c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x963527cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303527d3, SUC_lexicon_tags_14 },
    { 0xcb3527d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb83527d6, SUC_lexicon_tags_12 },
    { 0x223527d6, SUC_lexicon_tags_26 },
    { 0x72f527d8, SUC_lexicon_tags_6 },
    { 0x601527d9, SUC_lexicon_tags_56 },
    { 0xc15527da, SUC_lexicon_tags_39 },
    { 0x8fd527d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33f527dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x351527df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331527e3, SUC_lexicon_tags_113 },
    { 0xf69527e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x371527e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1527e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db527f1, SUC_lexicon_tags_13 },
    { 0x191527f2, SUC_lexicon_tags_25 },
    { 0x1dd527f2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d527f6, SUC_lexicon_tags_43 },
    { 0x25f527f7, SUC_lexicon_tags_48 },
    { 0x931527f7, SUC_lexicon_tags_63 },
    { 0x7cb527f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe97527fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541527ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf52806, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ab52808, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8d75280a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34752812, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21152816, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d5281a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab5281e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c752820, SUC_lexicon_tags_7 },
    { 0xeb552821, SUC_lexicon_tags_0 },
    { 0x38152822, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xef352824, SUC_lexicon_tags_127 },
    { 0xb1352824, SUC_lexicon_tags_25 },
    { 0x37f52826, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc4952828, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b5282c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9952830, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a75283c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2575283e, SUC_lexicon_tags_30 },
    { 0x4935283f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90352841, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xd952843, SUC_lexicon_tags_15 },
    { 0x90752844, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24152848, SUC_lexicon_tags_194 },
    { 0xb6752848, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed5284f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c752851, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb52856, SUC_lexicon_tags_56 },
    { 0x22d52857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb435285b, SUC_lexicon_tags_30 },
    { 0x4815285b, SUC_lexicon_tags_38 },
    { 0x475285b, SUC_lexicon_tags_25 },
    { 0x2735285d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x57752860, SUC_lexicon_tags_37 },
    { 0xdc952861, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x95752864, SUC_lexicon_tags_14 },
    { 0xbc952864, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca152867, SUC_lexicon_tags_23 },
    { 0x9952868, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b5286c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98752872, SUC_lexicon_tags_15 },
    { 0x86552873, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde552877, SUC_lexicon_tags_38 },
    { 0xd5d52878, SUC_lexicon_tags_15 },
    { 0x23352879, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b5287e, SUC_lexicon_tags_12 },
    { 0xf355287f, SUC_lexicon_tags_27 },
    { 0x4ef52880, SUC_lexicon_tags_43 },
    { 0x73152880, SUC_lexicon_tags_15 },
    { 0xedd52882, SUC_lexicon_tags_77 },
    { 0x55f52882, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe6152885, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x21752888, SUC_lexicon_tags_30 },
    { 0xadb52889, SUC_lexicon_tags_14 },
    { 0x6352889, SUC_lexicon_tags_37 },
    { 0xc515288b, SUC_lexicon_tags_60 },
    { 0x2f55288b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7db5288e, SUC_lexicon_tags_44 },
    { 0x7d35288f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd152892, SUC_lexicon_tags_1 },
    { 0x6cd52893, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8552896, SUC_lexicon_tags_79 },
    { 0xe5b52896, SUC_lexicon_tags_12 },
    { 0xa0152896, SUC_lexicon_tags_0 },
    { 0xacd52897, SUC_lexicon_tags_74 },
    { 0x7ad52896, SUC_lexicon_tags_19 },
    { 0xbf75289b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5289f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x329528a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5528a3, SUC_lexicon_tags_7 },
    { 0xeab528a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x313528a6, SUC_lexicon_tags_38 },
    { 0x9eb528a7, SUC_lexicon_tags_25 },
    { 0xaeb528a8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9528ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x119528ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5528b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd9528b3, SUC_lexicon_tags_7 },
    { 0x979528b4, SUC_lexicon_tags_60 },
    { 0xfed528b5, SUC_lexicon_tags_16 },
    { 0x601528b6, SUC_lexicon_tags_1 },
    { 0x82b528b7, SUC_lexicon_tags_43 },
    { 0x54d528b7, SUC_lexicon_tags_2 },
    { 0x9e7528b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb5528bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d528be, SUC_lexicon_tags_14 },
    { 0x689528be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7528c3, SUC_lexicon_tags_11 },
    { 0x9b3528c3, SUC_lexicon_tags_15 },
    { 0xe0d528c5, SUC_lexicon_tags_25 },
    { 0xed1528c6, SUC_lexicon_tags_87 },
    { 0x4cb528c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x257528ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x295528cc, SUC_lexicon_tags_31 },
    { 0x345528cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1528d0, SUC_lexicon_tags_0 },
    { 0xec9528d0, SUC_lexicon_tags_9 },
    { 0xdbb528d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62f528d4, SUC_lexicon_tags_79 },
    { 0x437528d5, SUC_lexicon_tags_39 },
    { 0xac9528d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4af528d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b528de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72f528e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x28b528e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4bd528e4, SUC_lexicon_tags_7 },
    { 0x687528e4, SUC_lexicon_tags_37 },
    { 0x2f5528e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df528ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeab528f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9528f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639528fd, SUC_lexicon_tags_14 },
    { 0x925528fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1952900, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d52903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f552907, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24152909, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d5290d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x75f5290f, SUC_lexicon_tags_47 },
    { 0x90752910, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x17552913, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f5291a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x97d5291c, SUC_lexicon_tags_49 },
    { 0x4ff5291d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7752921, SUC_lexicon_tags_33 },
    { 0x48552921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e952935, SUC_lexicon_tags_11 },
    { 0x33152936, SUC_lexicon_tags_14 },
    { 0x41552937, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b5293a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e55293e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x95152940, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84152946, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df52949, SUC_lexicon_tags_7 },
    { 0xdd5294a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f52952, SUC_lexicon_tags_27 },
    { 0x3d352953, SUC_lexicon_tags_37 },
    { 0xabd52952, SUC_lexicon_tags_23 },
    { 0xaa752954, SUC_lexicon_tags_19 },
    { 0xdd552956, SUC_lexicon_tags_29 },
    { 0x78952953, SUC_lexicon_tags_11 },
    { 0x4352955, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f15295b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb752963, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1df52965, SUC_lexicon_tags_39 },
    { 0x31352966, SUC_lexicon_tags_31 },
    { 0x70952967, SUC_lexicon_tags_1 },
    { 0x90352967, SUC_lexicon_tags_38 },
    { 0xb6152968, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b52970, SUC_lexicon_tags_19 },
    { 0x62752971, SUC_lexicon_tags_37 },
    { 0xbdd52972, SUC_lexicon_tags_1 },
    { 0x67152971, SUC_lexicon_tags_11 },
    { 0x22d52974, SUC_lexicon_tags_26 },
    { 0x65152973, SUC_lexicon_tags_25 },
    { 0xca952972, SUC_lexicon_tags_21 },
    { 0x61d52977, SUC_lexicon_tags_25 },
    { 0x51152977, SUC_lexicon_tags_27 },
    { 0x5dd52975, SUC_lexicon_tags_11 },
    { 0xb4352976, SUC_lexicon_tags_15 },
    { 0x5ef52971, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575297f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4752985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68352988, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a35298d, SUC_lexicon_tags_47 },
    { 0x9475298e, SUC_lexicon_tags_17 },
    { 0x8eb5298d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e152992, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6552996, SUC_lexicon_tags_63 },
    { 0xce552997, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xec152999, SUC_lexicon_tags_4 },
    { 0x73552999, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f529a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917529af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x745529b1, SUC_lexicon_tags_25 },
    { 0x95b529b2, SUC_lexicon_tags_2 },
    { 0x79b529b3, SUC_lexicon_tags_6 },
    { 0xd07529b4, SUC_lexicon_tags_11 },
    { 0x7f5529b5, SUC_lexicon_tags_37 },
    { 0x3cd529b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537529ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389529be, SUC_lexicon_tags_17 },
    { 0xfef529be, SUC_lexicon_tags_37 },
    { 0xa5b529c0, SUC_lexicon_tags_30 },
    { 0xb8d529c1, SUC_lexicon_tags_13 },
    { 0x7d9529c2, SUC_lexicon_tags_23 },
    { 0x171529c3, SUC_lexicon_tags_23 },
    { 0xa6b529c2, SUC_lexicon_tags_26 },
    { 0x31529c2, SUC_lexicon_tags_37 },
    { 0x1d3529c6, SUC_lexicon_tags_7 },
    { 0xb9d529c6, SUC_lexicon_tags_37 },
    { 0x33529c8, SUC_lexicon_tags_27 },
    { 0x6d5529c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x367529cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615529d4, SUC_lexicon_tags_7 },
    { 0x621529d4, SUC_lexicon_tags_65 },
    { 0x5f9529d6, SUC_lexicon_tags_1 },
    { 0x9c7529d4, SUC_lexicon_tags_264 },
    { 0x6af529d7, SUC_lexicon_tags_31 },
    { 0x909529d8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59529dd, SUC_lexicon_tags_12 },
    { 0x299529dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x945529e0, SUC_lexicon_tags_27 },
    { 0x54d529e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d529e8, SUC_lexicon_tags_10 },
    { 0x245529e8, SUC_lexicon_tags_31 },
    { 0x40d529e8, SUC_lexicon_tags_11 },
    { 0xdcd529eb, SUC_lexicon_tags_45 },
    { 0x1cb529ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf7529ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f529f6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x707529f9, SUC_lexicon_tags_38 },
    { 0xb57529fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef52a01, SUC_lexicon_tags_10 },
    { 0x47352a02, SUC_lexicon_tags_15 },
    { 0x1fd52a02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab52a06, SUC_lexicon_tags_14 },
    { 0x6d552a07, SUC_lexicon_tags_33 },
    { 0xd0f52a06, SUC_lexicon_tags_7 },
    { 0xe2752a07, SUC_lexicon_tags_43 },
    { 0xcd52a0a, SUC_lexicon_tags_8 },
    { 0x75752a0a, SUC_lexicon_tags_7 },
    { 0xb3d52a0c, SUC_lexicon_tags_17 },
    { 0x4552a0d, SUC_lexicon_tags_7 },
    { 0xe8352a0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99952a17, SUC_lexicon_tags_62 },
    { 0x6d352a17, SUC_lexicon_tags_12 },
    { 0x30352a18, SUC_lexicon_tags_30 },
    { 0x66f52a1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x79952a1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89952a1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ab52a20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d52a23, SUC_lexicon_tags_12 },
    { 0x70352a23, SUC_lexicon_tags_48 },
    { 0xf5952a24, SUC_lexicon_tags_25 },
    { 0xc552a26, SUC_lexicon_tags_11 },
    { 0xc2352a27, SUC_lexicon_tags_25 },
    { 0xb3952a26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58d52a2a, SUC_lexicon_tags_25 },
    { 0x1bd52a2b, SUC_lexicon_tags_1 },
    { 0xf3352a2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba552a36, SUC_lexicon_tags_7 },
    { 0x3d752a36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f52a3b, SUC_lexicon_tags_33 },
    { 0x61352a3b, SUC_lexicon_tags_43 },
    { 0xa2352a3d, SUC_lexicon_tags_14 },
    { 0xa6b52a3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf6152a40, SUC_lexicon_tags_12 },
    { 0x5552a41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1152a44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c952a4d, SUC_lexicon_tags_7 },
    { 0x7b952a4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf52a57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe552a5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12752a5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0552a5f, SUC_lexicon_tags_1 },
    { 0x86152a60, SUC_lexicon_tags_14 },
    { 0xaef52a60, SUC_lexicon_tags_2 },
    { 0xea952a62, SUC_lexicon_tags_9 },
    { 0xb1952a63, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e152a69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1152a6c, SUC_lexicon_tags_19 },
    { 0x60352a6c, SUC_lexicon_tags_6 },
    { 0x63152a6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f52a71, SUC_lexicon_tags_80 },
    { 0x34952a71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1552a75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54552a78, SUC_lexicon_tags_10 },
    { 0xa5152a79, SUC_lexicon_tags_47 },
    { 0xfb552a7a, SUC_lexicon_tags_63 },
    { 0xaa352a7a, SUC_lexicon_tags_17 },
    { 0x5352a7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38552a80, SUC_lexicon_tags_11 },
    { 0x8e152a81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d352a83, SUC_lexicon_tags_22 },
    { 0x16f52a83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4352a86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf52a8a, SUC_lexicon_tags_43 },
    { 0x77352a8b, SUC_lexicon_tags_2 },
    { 0x52352a8c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff952a8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa952a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf152a95, SUC_lexicon_tags_19 },
    { 0x2b952a96, SUC_lexicon_tags_21 },
    { 0x65d52a96, SUC_lexicon_tags_45 },
    { 0x71952a98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb152a9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86f52a9e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb52aa2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd52aa6, SUC_lexicon_tags_27 },
    { 0xe952aa6, SUC_lexicon_tags_37 },
    { 0x1bd52aa6, SUC_lexicon_tags_11 },
    { 0xe9b52aa7, SUC_lexicon_tags_16 },
    { 0x8af52aaa, SUC_lexicon_tags_37 },
    { 0x3f552aab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f52ab0, SUC_lexicon_tags_43 },
    { 0xef552ab1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48152ab6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5152abc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff52ac3, SUC_lexicon_tags_28 },
    { 0x3bd52ac4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x79152ac7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68552acc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8152ad5, SUC_lexicon_tags_37 },
    { 0x90952ad5, SUC_lexicon_tags_12 },
    { 0x41752ad7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35752ada, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee752adf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3152ae8, SUC_lexicon_tags_6 },
    { 0xa1952ae9, SUC_lexicon_tags_0 },
    { 0x33552ae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd52af2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab352af5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb52b05, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc3352b07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccb52b09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77952b0c, SUC_lexicon_tags_37 },
    { 0x89752b0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x51552b0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa2b52b11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4152b18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5352b1b, SUC_lexicon_tags_359 },
    { 0x87152b1c, SUC_lexicon_tags_296 },
    { 0x17352b1b, SUC_lexicon_tags_38 },
    { 0xa3952b1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad52b22, SUC_lexicon_tags_114 },
    { 0xa9f52b23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b52b28, SUC_lexicon_tags_27 },
    { 0x45952b28, SUC_lexicon_tags_12 },
    { 0x70152b29, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x56552b2c, SUC_lexicon_tags_17 },
    { 0x8a952b2d, SUC_lexicon_tags_47 },
    { 0xed752b2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xee952b31, SUC_lexicon_tags_37 },
    { 0xf2d52b32, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x86752b35, SUC_lexicon_tags_27 },
    { 0xe152b35, SUC_lexicon_tags_37 },
    { 0x59b52b36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad352b3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c952b3d, SUC_lexicon_tags_16 },
    { 0x31752b3d, SUC_lexicon_tags_11 },
    { 0xaab52b3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd52b43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f552b45, SUC_lexicon_tags_17 },
    { 0xa2752b46, SUC_lexicon_tags_14 },
    { 0x44552b46, SUC_lexicon_tags_30 },
    { 0x11152b47, SUC_lexicon_tags_37 },
    { 0xc5952b47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x49b52b4b, SUC_lexicon_tags_1 },
    { 0xa7b52b4b, SUC_lexicon_tags_17 },
    { 0x4f952b4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc952b51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95752b57, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9152b5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20752b65, SUC_lexicon_tags_2 },
    { 0xb1152b65, SUC_lexicon_tags_47 },
    { 0xdb52b67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7752b6c, SUC_lexicon_tags_17 },
    { 0x18b52b6d, SUC_lexicon_tags_12 },
    { 0x41752b6e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b52b77, SUC_lexicon_tags_15 },
    { 0x37552b78, SUC_lexicon_tags_31 },
    { 0xd7752b78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf352b7e, SUC_lexicon_tags_37 },
    { 0xf8152b7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f52b84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6152b86, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc4d52b88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f52b8b, SUC_lexicon_tags_3 },
    { 0x5ef52b8c, SUC_lexicon_tags_25 },
    { 0xff352b8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b752b92, SUC_lexicon_tags_13 },
    { 0xc3552b93, SUC_lexicon_tags_7 },
    { 0x91d52b94, SUC_lexicon_tags_27 },
    { 0x7f152b94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe352b99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab52b9e, SUC_lexicon_tags_15 },
    { 0x8ad52b9f, SUC_lexicon_tags_37 },
    { 0x5c352ba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28352ba2, SUC_lexicon_tags_30 },
    { 0xf2f52ba2, SUC_lexicon_tags_29 },
    { 0x81952ba4, SUC_lexicon_tags_65 },
    { 0x7ff52ba4, SUC_lexicon_tags_28 },
    { 0xa1152ba4, SUC_lexicon_tags_10 },
    { 0xe8152ba7, SUC_lexicon_tags_11 },
    { 0x4b552ba4, SUC_lexicon_tags_25 },
    { 0x7b52ba9, SUC_lexicon_tags_11 },
    { 0xdc152ba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5152bad, SUC_lexicon_tags_34 },
    { 0x10f52bae, SUC_lexicon_tags_25 },
    { 0xd7152baf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85952bb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ff52bb5, SUC_lexicon_tags_43 },
    { 0xed952bb6, SUC_lexicon_tags_12 },
    { 0x68152bb6, SUC_lexicon_tags_54 },
    { 0x8e352bb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80552bba, SUC_lexicon_tags_17 },
    { 0xd2f52bba, SUC_lexicon_tags_11 },
    { 0x42f52bba, SUC_lexicon_tags_87 },
    { 0x21552bbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5352bbf, SUC_lexicon_tags_174 },
    { 0x7cd52bc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f52bc5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x96752bc7, SUC_lexicon_tags_80 },
    { 0xa6b52bc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97352bca, SUC_lexicon_tags_33 },
    { 0x27152bcb, SUC_lexicon_tags_30 },
    { 0x17752bcb, SUC_lexicon_tags_2 },
    { 0x79752bcc, SUC_lexicon_tags_17 },
    { 0xf8552bce, SUC_lexicon_tags_12 },
    { 0x67952bcf, SUC_lexicon_tags_27 },
    { 0xb2552bd0, SUC_lexicon_tags_15 },
    { 0x37f52bcf, SUC_lexicon_tags_107 },
    { 0x9d152bd2, SUC_lexicon_tags_55 },
    { 0xbe352bd3, SUC_lexicon_tags_33 },
    { 0xeaf52bcc, SUC_lexicon_tags_10 },
    { 0x2bd52bce, SUC_lexicon_tags_23 },
    { 0x5bb52bd6, SUC_lexicon_tags_60 },
    { 0x3a152bd6, SUC_lexicon_tags_17 },
    { 0x47f52bd5, SUC_lexicon_tags_25 },
    { 0x80352bcb, SUC_lexicon_tags_37 },
    { 0xc152bda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb352bdc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbb152bde, SUC_lexicon_tags_178 },
    { 0x34d52bdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb52be2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29d52be4, SUC_lexicon_tags_17 },
    { 0x63f52be4, SUC_lexicon_tags_25 },
    { 0x66352be4, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d52bea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9352bee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7152bf2, SUC_lexicon_tags_91 },
    { 0x7c552bf3, SUC_lexicon_tags_1 },
    { 0xf2552bf2, SUC_lexicon_tags_25 },
    { 0x22b52bf5, SUC_lexicon_tags_39 },
    { 0xa2552bf6, SUC_lexicon_tags_1 },
    { 0x4b752bf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91f52bf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f52c00, SUC_lexicon_tags_25 },
    { 0x26552c00, SUC_lexicon_tags_61 },
    { 0x3dd52c00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85152c04, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4152c09, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30352c10, SUC_lexicon_tags_14 },
    { 0x2db52c10, SUC_lexicon_tags_47 },
    { 0x71152c11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76752c1f, SUC_lexicon_tags_23 },
    { 0x2b952c1f, SUC_lexicon_tags_17 },
    { 0xa0352c21, SUC_lexicon_tags_37 },
    { 0x6f552c22, SUC_lexicon_tags_23 },
    { 0x42952c23, SUC_lexicon_tags_25 },
    { 0x53952c24, SUC_lexicon_tags_37 },
    { 0xf3752c25, SUC_lexicon_tags_38 },
    { 0x7df52c22, SUC_lexicon_tags_11 },
    { 0x29152c27, SUC_lexicon_tags_12 },
    { 0xc8f52c28, SUC_lexicon_tags_25 },
    { 0x152c27, SUC_lexicon_tags_12 },
    { 0xc1352c2a, SUC_lexicon_tags_38 },
    { 0x3552c2b, SUC_lexicon_tags_55 },
    { 0xfdb52c2c, SUC_lexicon_tags_60 },
    { 0xd1752c2d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd52c31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9952c34, SUC_lexicon_tags_11 },
    { 0x88152c34, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x60552c37, SUC_lexicon_tags_60 },
    { 0x1a752c38, SUC_lexicon_tags_13 },
    { 0x30952c38, SUC_lexicon_tags_17 },
    { 0x69752c3a, SUC_lexicon_tags_60 },
    { 0xff752c3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb52c3f, SUC_lexicon_tags_14 },
    { 0x7ef52c40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7752c43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb52c46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd152c4c, SUC_lexicon_tags_31 },
    { 0x74152c4d, SUC_lexicon_tags_25 },
    { 0xc9952c4c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc552c52, SUC_lexicon_tags_30 },
    { 0x79952c52, SUC_lexicon_tags_26 },
    { 0xee152c52, SUC_lexicon_tags_0 },
    { 0x89152c55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98552c5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd4152c61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0352c64, SUC_lexicon_tags_37 },
    { 0xe152c65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d952c69, SUC_lexicon_tags_15 },
    { 0x7952c69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0952c6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x87352c70, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f552c72, SUC_lexicon_tags_47 },
    { 0x85f52c72, SUC_lexicon_tags_17 },
    { 0xbfd52c72, SUC_lexicon_tags_28 },
    { 0xf4552c75, SUC_lexicon_tags_1 },
    { 0x7b152c73, SUC_lexicon_tags_17 },
    { 0x80352c77, SUC_lexicon_tags_12 },
    { 0x59552c77, SUC_lexicon_tags_25 },
    { 0xa6b52c79, SUC_lexicon_tags_60 },
    { 0x82952c7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b52c7f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x66552c82, SUC_lexicon_tags_14 },
    { 0xc2352c83, SUC_lexicon_tags_7 },
    { 0xf5b52c84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f52c87, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe3152c89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82752c8b, SUC_lexicon_tags_12 },
    { 0x36552c8b, SUC_lexicon_tags_25 },
    { 0x4b752c8c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14952c93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f752c95, SUC_lexicon_tags_12 },
    { 0x5ff52c95, SUC_lexicon_tags_29 },
    { 0x5bd52c97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeaf52c99, SUC_lexicon_tags_1 },
    { 0x60152c9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93352c9c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79352ca1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc0752ca3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2752ca8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8152cab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b752cae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29152cb4, SUC_lexicon_tags_27 },
    { 0x97d52cb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x79b52cb7, SUC_lexicon_tags_46 },
    { 0x22b52cb8, SUC_lexicon_tags_7 },
    { 0x57b52cb8, SUC_lexicon_tags_39 },
    { 0xf7752cb7, SUC_lexicon_tags_26 },
    { 0x66752cbb, SUC_lexicon_tags_15 },
    { 0x4e552cbc, SUC_lexicon_tags_26 },
    { 0xbd552cbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf52cc0, SUC_lexicon_tags_1 },
    { 0xa7552cc0, SUC_lexicon_tags_16 },
    { 0xc952cc2, SUC_lexicon_tags_9 },
    { 0xeff52cc3, SUC_lexicon_tags_6 },
    { 0xe4b52cc4, SUC_lexicon_tags_14 },
    { 0xda152cc4, SUC_lexicon_tags_37 },
    { 0x74f52cc6, SUC_lexicon_tags_25 },
    { 0xc2b52cc7, SUC_lexicon_tags_11 },
    { 0x8f952cc7, SUC_lexicon_tags_15 },
    { 0x4c752cc6, SUC_lexicon_tags_15 },
    { 0xefb52cca, SUC_lexicon_tags_25 },
    { 0xc2352cc4, SUC_lexicon_tags_63 },
    { 0x55f52ccc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39752cd0, SUC_lexicon_tags_60 },
    { 0x9ff52cd1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa7152cd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb52cdc, SUC_lexicon_tags_17 },
    { 0x35352cdc, SUC_lexicon_tags_11 },
    { 0x7df52cde, SUC_lexicon_tags_1 },
    { 0x4c952cdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22952ce1, SUC_lexicon_tags_28 },
    { 0xf2f52ce1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98952ce9, SUC_lexicon_tags_15 },
    { 0xe8152ce9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd52cec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8d52cee, SUC_lexicon_tags_25 },
    { 0x58d52cee, SUC_lexicon_tags_30 },
    { 0xeed52cf0, SUC_lexicon_tags_12 },
    { 0x4dd52cf1, SUC_lexicon_tags_10 },
    { 0x3e152cf0, SUC_lexicon_tags_27 },
    { 0xf6b52cf3, SUC_lexicon_tags_56 },
    { 0x47752cf3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5c352cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d52cfa, SUC_lexicon_tags_10 },
    { 0x37752cfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87552cfe, SUC_lexicon_tags_23 },
    { 0x9e352cfe, SUC_lexicon_tags_8 },
    { 0x51b52d00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d52d03, SUC_lexicon_tags_124 },
    { 0x61f52d04, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b52d08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62552d0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41352d0e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d52d11, SUC_lexicon_tags_25 },
    { 0xa9d52d11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff52d17, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6fd52d19, SUC_lexicon_tags_25 },
    { 0xc3152d1a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd952d20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde552d23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80352d25, SUC_lexicon_tags_17 },
    { 0xb4752d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f552d2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b952d36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62752d3c, SUC_lexicon_tags_6 },
    { 0x2e152d3d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x61f52d3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a752d48, SUC_lexicon_tags_30 },
    { 0x8c752d48, SUC_lexicon_tags_17 },
    { 0xa1f52d4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3352d4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5752d54, SUC_lexicon_tags_17 },
    { 0x20352d55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2552d58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9c352d5a, SUC_lexicon_tags_15 },
    { 0xed552d5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf52d5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d152d62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd5752d64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c352d66, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d52d69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c752d6b, SUC_lexicon_tags_2 },
    { 0x45352d6c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf352d70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69352d72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78952d79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af52d7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6752d81, SUC_lexicon_tags_11 },
    { 0x3fb52d82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b52d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a752d87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb52d96, SUC_lexicon_tags_1 },
    { 0x19352d97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x12952d99, SUC_lexicon_tags_25 },
    { 0x3b952d9a, SUC_lexicon_tags_38 },
    { 0x1a152d9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc952d9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cf52da0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4152da2, SUC_lexicon_tags_6 },
    { 0xf9352da2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x75b52da5, SUC_lexicon_tags_37 },
    { 0xb2f52da6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb6d52da8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37552db1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf6152db3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b52db7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4952dbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21952dbd, SUC_lexicon_tags_29 },
    { 0xedf52dbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31952dc3, SUC_lexicon_tags_7 },
    { 0xd2952dc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5752dc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1b52dc8, SUC_lexicon_tags_11 },
    { 0x81b52dc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5152dd1, SUC_lexicon_tags_17 },
    { 0x83752dd2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1152dd6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c352de0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa552de2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x17752de5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1152de8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d952df3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2952dfb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb52e00, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f52e04, SUC_lexicon_tags_28 },
    { 0x5b552e04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x48752e07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3752e0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d52e10, SUC_lexicon_tags_49 },
    { 0x29552e10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b52e14, SUC_lexicon_tags_66 },
    { 0x30152e14, SUC_lexicon_tags_15 },
    { 0x99f52e16, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1352e1b, SUC_lexicon_tags_68 },
    { 0x82b52e1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93352e20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef752e22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18352e24, SUC_lexicon_tags_56 },
    { 0x24352e25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f552e2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa752e2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55d52e2e, SUC_lexicon_tags_16 },
    { 0xd2d52e2e, SUC_lexicon_tags_65 },
    { 0xd5152e2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a352e34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xac152e36, SUC_lexicon_tags_7 },
    { 0x81152e37, SUC_lexicon_tags_15 },
    { 0x95152e37, SUC_lexicon_tags_37 },
    { 0xe5752e39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b952e42, SUC_lexicon_tags_793 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb152e47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4d552e49, SUC_lexicon_tags_37 },
    { 0x30b52e4a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3b552e4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb352e50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefd52e52, SUC_lexicon_tags_45 },
    { 0xb4952e53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f552e56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58552e5f, SUC_lexicon_tags_190 },
    { 0x34d52e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46952e65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f52e68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd952e6a, SUC_lexicon_tags_25 },
    { 0x9e552e6a, SUC_lexicon_tags_37 },
    { 0x6952e6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1c752e6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1752e70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff52e7e, SUC_lexicon_tags_7 },
    { 0x2dd52e7f, SUC_lexicon_tags_36 },
    { 0x94552e7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d552e84, SUC_lexicon_tags_68 },
    { 0xa4d52e85, SUC_lexicon_tags_15 },
    { 0x53f52e85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f952e88, SUC_lexicon_tags_1 },
    { 0xaeb52e89, SUC_lexicon_tags_17 },
    { 0x16752e89, SUC_lexicon_tags_2 },
    { 0x41152e88, SUC_lexicon_tags_19 },
    { 0xc3f52e89, SUC_lexicon_tags_30 },
    { 0x2e352e88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d52e91, SUC_lexicon_tags_17 },
    { 0xa4552e92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xad352e95, SUC_lexicon_tags_17 },
    { 0x83152e96, SUC_lexicon_tags_2 },
    { 0x35f52e95, SUC_lexicon_tags_17 },
    { 0xab552e95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f52e9e, SUC_lexicon_tags_85 },
    { 0xcad52e9f, SUC_lexicon_tags_82 },
    { 0x7c952e9f, SUC_lexicon_tags_25 },
    { 0xc3d52e9e, SUC_lexicon_tags_11 },
    { 0xccb52ea2, SUC_lexicon_tags_14 },
    { 0x2e152e9e, SUC_lexicon_tags_29 },
    { 0xa752e9f, SUC_lexicon_tags_17 },
    { 0x24552ea4, SUC_lexicon_tags_37 },
    { 0xb7b52ea6, SUC_lexicon_tags_14 },
    { 0xcb752ea3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9752ea9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5c152eab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b552eb2, SUC_lexicon_tags_17 },
    { 0x25b52eb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b52eb6, SUC_lexicon_tags_45 },
    { 0xcb352eb7, SUC_lexicon_tags_0 },
    { 0x6f152eb8, SUC_lexicon_tags_19 },
    { 0x60f52eb8, SUC_lexicon_tags_25 },
    { 0xcb152eb6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5752ebd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55f52ebf, SUC_lexicon_tags_47 },
    { 0x4d52ebf, SUC_lexicon_tags_2 },
    { 0x5a152ec1, SUC_lexicon_tags_19 },
    { 0x7b152ec1, SUC_lexicon_tags_15 },
    { 0xd2552ec2, SUC_lexicon_tags_2 },
    { 0x82b52ec4, SUC_lexicon_tags_23 },
    { 0xd1752ec3, SUC_lexicon_tags_17 },
    { 0xb7952ec6, SUC_lexicon_tags_63 },
    { 0x89352ebf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b52ed0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a752ed3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8db52ed5, SUC_lexicon_tags_7 },
    { 0xe9952ed6, SUC_lexicon_tags_37 },
    { 0xa1352ed7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1352ed9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5152edc, SUC_lexicon_tags_17 },
    { 0xd8152edd, SUC_lexicon_tags_7 },
    { 0x9e352ede, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x17f52ee0, SUC_lexicon_tags_25 },
    { 0xbab52ee1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3552ee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4952eea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a152ef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d52efc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4352f00, SUC_lexicon_tags_47 },
    { 0x12d52f00, SUC_lexicon_tags_56 },
    { 0x45d52f02, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x46552f04, SUC_lexicon_tags_12 },
    { 0xcdb52f05, SUC_lexicon_tags_11 },
    { 0xaf152f06, SUC_lexicon_tags_25 },
    { 0xec552f07, SUC_lexicon_tags_45 },
    { 0x53352f07, SUC_lexicon_tags_39 },
    { 0x49352f09, SUC_lexicon_tags_43 },
    { 0x7eb52f05, SUC_lexicon_tags_2 },
    { 0x2e552f0b, SUC_lexicon_tags_31 },
    { 0x7c752f05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb952f0e, SUC_lexicon_tags_60 },
    { 0xc0552f0f, SUC_lexicon_tags_17 },
    { 0xdb352f10, SUC_lexicon_tags_27 },
    { 0x57352f11, SUC_lexicon_tags_37 },
    { 0x52b52f12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x90352f14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd752f16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd52f1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91152f1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7952f23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19352f2f, SUC_lexicon_tags_54 },
    { 0x90f52f2f, SUC_lexicon_tags_23 },
    { 0x8f552f31, SUC_lexicon_tags_30 },
    { 0x29352f30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48752f35, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2152f39, SUC_lexicon_tags_0 },
    { 0x83352f39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f52f3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b52f40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6152f45, SUC_lexicon_tags_11 },
    { 0x32f52f45, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29552f4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24952f4f, SUC_lexicon_tags_9 },
    { 0xfd952f4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30b52f52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x76952f54, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24352f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81152f5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f52f5e, SUC_lexicon_tags_53 },
    { 0xaeb52f5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5352f62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61552f65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbef52f67, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x83552f6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f952f6c, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xc9352f6e, SUC_lexicon_tags_25 },
    { 0xcf152f6e, SUC_lexicon_tags_17 },
    { 0x4ab52f70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24352f77, SUC_lexicon_tags_25 },
    { 0x64952f77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x96352f7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93352f7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d152f80, SUC_lexicon_tags_65 },
    { 0x44f52f81, SUC_lexicon_tags_6 },
    { 0x39152f81, SUC_lexicon_tags_66 },
    { 0x3e552f83, SUC_lexicon_tags_27 },
    { 0xbcd52f83, SUC_lexicon_tags_1 },
    { 0xfcd52f85, SUC_lexicon_tags_25 },
    { 0xd9d52f80, SUC_lexicon_tags_17 },
    { 0xb7b52f81, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b52f8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9152f8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x96b52f90, SUC_lexicon_tags_2 },
    { 0x85752f90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81552f93, SUC_lexicon_tags_160 },
    { 0x81d52f94, SUC_lexicon_tags_43 },
    { 0x4f152f95, SUC_lexicon_tags_12 },
    { 0x7f552f96, SUC_lexicon_tags_6 },
    { 0x952f96, SUC_lexicon_tags_29 },
    { 0xbf752f94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x58552f9b, SUC_lexicon_tags_2 },
    { 0xda352f9c, SUC_lexicon_tags_62 },
    { 0x21952f9d, SUC_lexicon_tags_56 },
    { 0xe0352f9b, SUC_lexicon_tags_25 },
    { 0xf8552f9c, SUC_lexicon_tags_15 },
    { 0x74152fa0, SUC_lexicon_tags_21 },
    { 0xc6152fa1, SUC_lexicon_tags_14 },
    { 0xab52fa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c552fa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xed52fa7, SUC_lexicon_tags_16 },
    { 0x35552fa7, SUC_lexicon_tags_26 },
    { 0xebd52fa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdf52fab, SUC_lexicon_tags_23 },
    { 0x36752fac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b52faf, SUC_lexicon_tags_9 },
    { 0x5a352faf, SUC_lexicon_tags_70 },
    { 0x60752fb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed52fb4, SUC_lexicon_tags_25 },
    { 0xc0352fb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41552fb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed52fbd, SUC_lexicon_tags_0 },
    { 0x46952fbe, SUC_lexicon_tags_0 },
    { 0x50752fbf, SUC_lexicon_tags_1 },
    { 0x8c352fc0, SUC_lexicon_tags_22 },
    { 0xcf952fbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x14752fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a752fc6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf552fcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87152fcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3552fcf, SUC_lexicon_tags_26 },
    { 0x5c352fd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x42952fd3, SUC_lexicon_tags_17 },
    { 0xbe152fd3, SUC_lexicon_tags_599 },
    { 0x85152fd5, SUC_lexicon_tags_37 },
    { 0x4e752fd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c152fda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2552fde, SUC_lexicon_tags_23 },
    { 0xdf952fdf, SUC_lexicon_tags_68 },
    { 0xc3152fdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e152fe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd952fe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92552fea, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3352fee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe1b52ff0, SUC_lexicon_tags_84 },
    { 0xb3f52ff1, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4fd52ff3, SUC_lexicon_tags_7 },
    { 0xcad52ff4, SUC_lexicon_tags_38 },
    { 0xe5952ff5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90552ff7, SUC_lexicon_tags_27 },
    { 0xcc352ff7, SUC_lexicon_tags_47 },
    { 0xa4152ff9, SUC_lexicon_tags_45 },
    { 0x26152ffa, SUC_lexicon_tags_38 },
    { 0xb7352ff9, SUC_lexicon_tags_15 },
    { 0x94352ff9, SUC_lexicon_tags_47 },
    { 0x4ef52ff9, SUC_lexicon_tags_60 },
    { 0x5f752ff9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11753006, SUC_lexicon_tags_23 },
    { 0x5d553007, SUC_lexicon_tags_37 },
    { 0x4753008, SUC_lexicon_tags_51 },
    { 0xcd553009, SUC_lexicon_tags_2 },
    { 0xf5553009, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1475300e, SUC_lexicon_tags_98 },
    { 0xb5b5300e, SUC_lexicon_tags_33 },
    { 0x99f53010, SUC_lexicon_tags_55 },
    { 0xecd53011, SUC_lexicon_tags_1 },
    { 0xa7d5300f, SUC_lexicon_tags_2 },
    { 0xf2953013, SUC_lexicon_tags_25 },
    { 0x55553013, SUC_lexicon_tags_63 },
    { 0x7ad53014, SUC_lexicon_tags_56 },
    { 0xedd5300e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d53019, SUC_lexicon_tags_14 },
    { 0xe315301a, SUC_lexicon_tags_15 },
    { 0xed15301b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc7b5301d, SUC_lexicon_tags_11 },
    { 0x6535301e, SUC_lexicon_tags_627 },
    { 0, NULL },
    { 0x5eb53020, SUC_lexicon_tags_60 },
    { 0xa1953021, SUC_lexicon_tags_27 },
    { 0x5e153022, SUC_lexicon_tags_7 },
    { 0xd953023, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14b53025, SUC_lexicon_tags_32 },
    { 0xcf353025, SUC_lexicon_tags_17 },
    { 0xea153025, SUC_lexicon_tags_11 },
    { 0x3d553027, SUC_lexicon_tags_68 },
    { 0x78553029, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f53030, SUC_lexicon_tags_14 },
    { 0x3953031, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38153033, SUC_lexicon_tags_23 },
    { 0x3cd53034, SUC_lexicon_tags_25 },
    { 0x8b953035, SUC_lexicon_tags_23 },
    { 0x31f53036, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f75303b, SUC_lexicon_tags_27 },
    { 0xf255303c, SUC_lexicon_tags_2 },
    { 0xfe35303d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f53041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x19553044, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba953046, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115304e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0753050, SUC_lexicon_tags_8 },
    { 0x2c153051, SUC_lexicon_tags_16 },
    { 0x1f353052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65553055, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab53058, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ef5305a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc375305c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f153060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f153062, SUC_lexicon_tags_37 },
    { 0x61353063, SUC_lexicon_tags_17 },
    { 0x30553064, SUC_lexicon_tags_26 },
    { 0x8ab53065, SUC_lexicon_tags_2 },
    { 0xa4153066, SUC_lexicon_tags_37 },
    { 0x74553067, SUC_lexicon_tags_45 },
    { 0x1d153068, SUC_lexicon_tags_2 },
    { 0x49b53069, SUC_lexicon_tags_1 },
    { 0xdc15306a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd5306d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7d5306f, SUC_lexicon_tags_31 },
    { 0xc9b53070, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf153074, SUC_lexicon_tags_27 },
    { 0xfd153075, SUC_lexicon_tags_38 },
    { 0xf9953076, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d5307d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a95307f, SUC_lexicon_tags_31 },
    { 0x69153080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66753086, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab5308b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea95308d, SUC_lexicon_tags_32 },
    { 0x355308e, SUC_lexicon_tags_33 },
    { 0x9d35308f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64353092, SUC_lexicon_tags_28 },
    { 0xedb53093, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b553097, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b5309d, SUC_lexicon_tags_15 },
    { 0x1435309e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119530a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5530a9, SUC_lexicon_tags_7 },
    { 0xa1b530aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db530ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65530b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b530b9, SUC_lexicon_tags_26 },
    { 0xab7530b9, SUC_lexicon_tags_27 },
    { 0x645530b9, SUC_lexicon_tags_11 },
    { 0xeb530b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7530c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d530c4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1f1530c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5530cb, SUC_lexicon_tags_7 },
    { 0x371530cc, SUC_lexicon_tags_0 },
    { 0xd9530cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1530d2, SUC_lexicon_tags_63 },
    { 0x23530d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4cd530d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd530de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d530e2, SUC_lexicon_tags_25 },
    { 0x74f530e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d530e6, SUC_lexicon_tags_37 },
    { 0x13530e7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe69530e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11f530eb, SUC_lexicon_tags_25 },
    { 0x60f530ec, SUC_lexicon_tags_15 },
    { 0x54b530eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d530f1, SUC_lexicon_tags_77 },
    { 0x845530f2, SUC_lexicon_tags_38 },
    { 0x3e7530f3, SUC_lexicon_tags_28 },
    { 0x7a1530f3, SUC_lexicon_tags_7 },
    { 0xf3530f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95530fa, SUC_lexicon_tags_37 },
    { 0x7fb530fa, SUC_lexicon_tags_37 },
    { 0x879530fb, SUC_lexicon_tags_39 },
    { 0x6db530fd, SUC_lexicon_tags_19 },
    { 0xf19530fd, SUC_lexicon_tags_37 },
    { 0xc3530fe, SUC_lexicon_tags_14 },
    { 0xe69530fe, SUC_lexicon_tags_2 },
    { 0x141530fe, SUC_lexicon_tags_15 },
    { 0xee353100, SUC_lexicon_tags_9 },
    { 0xc3f530fa, SUC_lexicon_tags_57 },
    { 0xe5153104, SUC_lexicon_tags_32 },
    { 0xd8f53104, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5d553107, SUC_lexicon_tags_17 },
    { 0xbd953107, SUC_lexicon_tags_36 },
    { 0x76553109, SUC_lexicon_tags_1421 },
    { 0xef35310a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8b5310c, SUC_lexicon_tags_37 },
    { 0x85d5310d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xddf5310f, SUC_lexicon_tags_60 },
    { 0xc9d53110, SUC_lexicon_tags_27 },
    { 0x57953110, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f553113, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6f153115, SUC_lexicon_tags_37 },
    { 0xd2b53115, SUC_lexicon_tags_2 },
    { 0x2c553117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e55311a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc215311e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad53122, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f53126, SUC_lexicon_tags_6 },
    { 0xfb753126, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x45b53129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f55312c, SUC_lexicon_tags_2 },
    { 0x4935312d, SUC_lexicon_tags_31 },
    { 0x9495312e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1b53130, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6553134, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef53138, SUC_lexicon_tags_47 },
    { 0xc9f53138, SUC_lexicon_tags_63 },
    { 0xbef53138, SUC_lexicon_tags_28 },
    { 0x75313b, SUC_lexicon_tags_12 },
    { 0x55d5313a, SUC_lexicon_tags_17 },
    { 0xeed5313a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f53142, SUC_lexicon_tags_2 },
    { 0xea153142, SUC_lexicon_tags_34 },
    { 0x1353144, SUC_lexicon_tags_146 },
    { 0x32d53144, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xc1553147, SUC_lexicon_tags_15 },
    { 0x4e753148, SUC_lexicon_tags_9 },
    { 0x6f153149, SUC_lexicon_tags_2 },
    { 0xd3353148, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f5314e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x67553150, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b553154, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40d53156, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e35315e, SUC_lexicon_tags_17 },
    { 0xe1b5315e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa6f53161, SUC_lexicon_tags_781 },
    { 0, NULL },
    { 0xde753163, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d353165, SUC_lexicon_tags_32 },
    { 0x61753165, SUC_lexicon_tags_63 },
    { 0x56753167, SUC_lexicon_tags_14 },
    { 0xefb53167, SUC_lexicon_tags_11 },
    { 0xd0b53168, SUC_lexicon_tags_82 },
    { 0x4075316a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b5316e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9953171, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3935317d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3095317f, SUC_lexicon_tags_19 },
    { 0x71353180, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5753184, SUC_lexicon_tags_14 },
    { 0xccb53185, SUC_lexicon_tags_32 },
    { 0x19d53184, SUC_lexicon_tags_12 },
    { 0x8a353187, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa555318a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f95318d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d153191, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed53196, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3db53198, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1595319c, SUC_lexicon_tags_7 },
    { 0xbb55319d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd531a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25d531a2, SUC_lexicon_tags_3 },
    { 0x40f531a3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x267531a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b531ae, SUC_lexicon_tags_26 },
    { 0x60d531af, SUC_lexicon_tags_33 },
    { 0x957531b0, SUC_lexicon_tags_0 },
    { 0x8a7531b0, SUC_lexicon_tags_12 },
    { 0xdc1531b2, SUC_lexicon_tags_2 },
    { 0x21531b3, SUC_lexicon_tags_14 },
    { 0x2b531b3, SUC_lexicon_tags_43 },
    { 0x30b531b2, SUC_lexicon_tags_8 },
    { 0x6b531b6, SUC_lexicon_tags_7 },
    { 0x9e1531b5, SUC_lexicon_tags_39 },
    { 0xfaf531b6, SUC_lexicon_tags_47 },
    { 0xd2f531b9, SUC_lexicon_tags_1 },
    { 0x12f531b8, SUC_lexicon_tags_29 },
    { 0x703531b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7dd531bd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbab531bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05531c4, SUC_lexicon_tags_32 },
    { 0x739531c5, SUC_lexicon_tags_11 },
    { 0xb39531c6, SUC_lexicon_tags_25 },
    { 0x4a9531c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719531cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b531d1, SUC_lexicon_tags_27 },
    { 0x97531d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73531db, SUC_lexicon_tags_15 },
    { 0xa2f531db, SUC_lexicon_tags_136 },
    { 0x94b531dd, SUC_lexicon_tags_14 },
    { 0x9fb531dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f531e3, SUC_lexicon_tags_11 },
    { 0x835531e3, SUC_lexicon_tags_45 },
    { 0x66b531e5, SUC_lexicon_tags_8 },
    { 0x4f531e6, SUC_lexicon_tags_49 },
    { 0x14b531e5, SUC_lexicon_tags_2 },
    { 0xbcd531e8, SUC_lexicon_tags_47 },
    { 0xa4f531e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f531ed, SUC_lexicon_tags_38 },
    { 0x6cd531ee, SUC_lexicon_tags_17 },
    { 0xae5531ef, SUC_lexicon_tags_37 },
    { 0x2dd531ef, SUC_lexicon_tags_29 },
    { 0x1c9531f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b531f4, SUC_lexicon_tags_26 },
    { 0x8e3531f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x92b531f7, SUC_lexicon_tags_28 },
    { 0x96d531f8, SUC_lexicon_tags_12 },
    { 0xe75531f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x59b531fb, SUC_lexicon_tags_17 },
    { 0x619531fc, SUC_lexicon_tags_37 },
    { 0xb4f531fd, SUC_lexicon_tags_37 },
    { 0x40f531fe, SUC_lexicon_tags_17 },
    { 0x73b531ff, SUC_lexicon_tags_9 },
    { 0x857531fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4353203, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50153210, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x40953213, SUC_lexicon_tags_83 },
    { 0x18f53214, SUC_lexicon_tags_2 },
    { 0xb3753215, SUC_lexicon_tags_17 },
    { 0xc5d53213, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78b53218, SUC_lexicon_tags_25 },
    { 0x9f553218, SUC_lexicon_tags_23 },
    { 0x80153219, SUC_lexicon_tags_17 },
    { 0xe5b5321b, SUC_lexicon_tags_28 },
    { 0x3fb5321a, SUC_lexicon_tags_47 },
    { 0x2ed5321d, SUC_lexicon_tags_29 },
    { 0x9b15321e, SUC_lexicon_tags_27 },
    { 0x11b5321e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd5322c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fb5322e, SUC_lexicon_tags_27 },
    { 0x20b5322f, SUC_lexicon_tags_2 },
    { 0xb8753230, SUC_lexicon_tags_12 },
    { 0x41f53230, SUC_lexicon_tags_57 },
    { 0x69753232, SUC_lexicon_tags_37 },
    { 0xd4753233, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e353235, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x58353237, SUC_lexicon_tags_27 },
    { 0xfbb53237, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6e35323a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe415323c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f53246, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbb753248, SUC_lexicon_tags_45 },
    { 0xbeb53249, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfab5324b, SUC_lexicon_tags_33 },
    { 0x3555324b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f53252, SUC_lexicon_tags_27 },
    { 0x6a153253, SUC_lexicon_tags_32 },
    { 0x57953254, SUC_lexicon_tags_26 },
    { 0x16553253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76153257, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x51953259, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a55325f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9953261, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff53265, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa9353267, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b5326c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43553270, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc753276, SUC_lexicon_tags_28 },
    { 0xcb553276, SUC_lexicon_tags_25 },
    { 0x79d53276, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab53283, SUC_lexicon_tags_37 },
    { 0x85f53284, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0x18f53286, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x85753288, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7995328a, SUC_lexicon_tags_24 },
    { 0xc475328b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2355328f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94753292, SUC_lexicon_tags_17 },
    { 0xbc953292, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7353296, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5753299, SUC_lexicon_tags_7 },
    { 0x3c15329a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95532a0, SUC_lexicon_tags_61 },
    { 0x5d1532a1, SUC_lexicon_tags_25 },
    { 0x72b532a2, SUC_lexicon_tags_25 },
    { 0xa95532a3, SUC_lexicon_tags_1 },
    { 0x91b532a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bb532a6, SUC_lexicon_tags_11 },
    { 0x605532a7, SUC_lexicon_tags_6 },
    { 0xa89532a8, SUC_lexicon_tags_53 },
    { 0xad5532a7, SUC_lexicon_tags_66 },
    { 0x347532a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x779532ac, SUC_lexicon_tags_15 },
    { 0xeed532ad, SUC_lexicon_tags_19 },
    { 0xb39532ad, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x283532b0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295532b6, SUC_lexicon_tags_37 },
    { 0xa61532b6, SUC_lexicon_tags_1 },
    { 0xe89532b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649532bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5532c0, SUC_lexicon_tags_15 },
    { 0xac3532c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b532ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf7532d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f532d3, SUC_lexicon_tags_22 },
    { 0x731532d3, SUC_lexicon_tags_11 },
    { 0x4d5532d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5532d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x783532db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd7f532dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed532e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f532e5, SUC_lexicon_tags_23 },
    { 0x86d532e6, SUC_lexicon_tags_43 },
    { 0x379532e7, SUC_lexicon_tags_21 },
    { 0xfef532e8, SUC_lexicon_tags_39 },
    { 0x9a9532e8, SUC_lexicon_tags_12 },
    { 0x293532e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x201532ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3532f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13532f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23532fb, SUC_lexicon_tags_28 },
    { 0xb3f532fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf53300, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e953303, SUC_lexicon_tags_23 },
    { 0xdcb53303, SUC_lexicon_tags_21 },
    { 0xef553305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d95330b, SUC_lexicon_tags_28 },
    { 0x9af5330b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa075330f, SUC_lexicon_tags_298 },
    { 0x9b553310, SUC_lexicon_tags_12 },
    { 0x4ab53311, SUC_lexicon_tags_69 },
    { 0xaa153312, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b753315, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7635331b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3cd5331d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b53320, SUC_lexicon_tags_15 },
    { 0xd7753321, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0553326, SUC_lexicon_tags_25 },
    { 0xc6f53327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1553329, SUC_lexicon_tags_25 },
    { 0x1195332a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87753334, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d53337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd5333e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd153340, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6953342, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d53348, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f75334b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8d35334d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb53351, SUC_lexicon_tags_47 },
    { 0x3b953352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd153364, SUC_lexicon_tags_25 },
    { 0xb5b53365, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb5336c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5c75336e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d953374, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715337a, SUC_lexicon_tags_1 },
    { 0xd5f5337b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b153380, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31753383, SUC_lexicon_tags_0 },
    { 0x4753384, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6d53386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84553389, SUC_lexicon_tags_0 },
    { 0x28b53389, SUC_lexicon_tags_28 },
    { 0xb2153389, SUC_lexicon_tags_40 },
    { 0xf975338c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b353391, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x86553393, SUC_lexicon_tags_124 },
    { 0x57d53393, SUC_lexicon_tags_25 },
    { 0x5dd53394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d53399, SUC_lexicon_tags_0 },
    { 0xbb35339a, SUC_lexicon_tags_25 },
    { 0x5295339a, SUC_lexicon_tags_9 },
    { 0xbbd5339c, SUC_lexicon_tags_31 },
    { 0x3dd5339d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1533a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21533a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdc1533a6, SUC_lexicon_tags_12 },
    { 0x20f533a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ef533a9, SUC_lexicon_tags_25 },
    { 0x127533aa, SUC_lexicon_tags_32 },
    { 0x2d5533ab, SUC_lexicon_tags_13 },
    { 0x79533a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69533b3, SUC_lexicon_tags_29 },
    { 0xb09533b4, SUC_lexicon_tags_6 },
    { 0x6a5533b3, SUC_lexicon_tags_17 },
    { 0x1c7533b6, SUC_lexicon_tags_25 },
    { 0xbbd533b3, SUC_lexicon_tags_8 },
    { 0xce5533b8, SUC_lexicon_tags_10 },
    { 0xf5f533b9, SUC_lexicon_tags_37 },
    { 0x6c3533b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1b533bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x929533be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17533c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4d533c4, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x819533c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35533ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x621533cc, SUC_lexicon_tags_17 },
    { 0xf533cc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x349533cf, SUC_lexicon_tags_1 },
    { 0x491533d0, SUC_lexicon_tags_15 },
    { 0x599533d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d533d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb03533d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce3533db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d533e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x121533e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1533e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xccf533e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d533ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x211533ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2ed533f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3533fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x233533fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b53404, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f753406, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7753409, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d5340e, SUC_lexicon_tags_37 },
    { 0x88f5340f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b53412, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ef53414, SUC_lexicon_tags_45 },
    { 0x10753415, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf255341d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7595341f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7153424, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x18753426, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d5342c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9095342e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb53431, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27953434, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2f953436, SUC_lexicon_tags_4 },
    { 0x97d53437, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d5343e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d53442, SUC_lexicon_tags_7 },
    { 0x4353442, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78353445, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db53449, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbcd5344b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d5344f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x94b53451, SUC_lexicon_tags_37 },
    { 0x99353452, SUC_lexicon_tags_1 },
    { 0xfdd53451, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x97353456, SUC_lexicon_tags_2 },
    { 0x39153457, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42153459, SUC_lexicon_tags_14 },
    { 0x753459, SUC_lexicon_tags_1 },
    { 0xff55345b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf53461, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3315346d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7615346f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c553472, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb53476, SUC_lexicon_tags_281 },
    { 0xb9353477, SUC_lexicon_tags_15 },
    { 0x7a353477, SUC_lexicon_tags_30 },
    { 0x51353479, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f53482, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f53487, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb553489, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb275348e, SUC_lexicon_tags_25 },
    { 0xb835348f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x20b53491, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x51153493, SUC_lexicon_tags_37 },
    { 0x12b53494, SUC_lexicon_tags_45 },
    { 0xe953495, SUC_lexicon_tags_12 },
    { 0x69d53496, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdab53498, SUC_lexicon_tags_17 },
    { 0x8d753498, SUC_lexicon_tags_37 },
    { 0x7c353499, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f35349d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa275349f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1534a4, SUC_lexicon_tags_2 },
    { 0xa79534a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf05534a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d7534a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x361534ac, SUC_lexicon_tags_9 },
    { 0x5e1534ac, SUC_lexicon_tags_2 },
    { 0xaaf534ae, SUC_lexicon_tags_25 },
    { 0xa47534ae, SUC_lexicon_tags_26 },
    { 0xba1534b0, SUC_lexicon_tags_47 },
    { 0xc6f534b1, SUC_lexicon_tags_9 },
    { 0xe05534b2, SUC_lexicon_tags_11 },
    { 0x723534ad, SUC_lexicon_tags_7 },
    { 0x6a9534b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f534b7, SUC_lexicon_tags_32 },
    { 0xd6b534b7, SUC_lexicon_tags_17 },
    { 0xe41534b9, SUC_lexicon_tags_8 },
    { 0xa3b534b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x703534bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd534c0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f534c3, SUC_lexicon_tags_25 },
    { 0x2d9534c4, SUC_lexicon_tags_37 },
    { 0x683534c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbd534c7, SUC_lexicon_tags_15 },
    { 0x429534c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x95f534ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155534d1, SUC_lexicon_tags_12 },
    { 0x9b1534d1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33534da, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f534dd, SUC_lexicon_tags_1 },
    { 0x1ed534dd, SUC_lexicon_tags_25 },
    { 0x793534df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77d534e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55534e4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef534e8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb534ec, SUC_lexicon_tags_10 },
    { 0x755534ec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2b9534ef, SUC_lexicon_tags_157 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f534f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9534f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1534f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3534fb, SUC_lexicon_tags_35 },
    { 0xc3534fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f53502, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7915350a, SUC_lexicon_tags_37 },
    { 0xedd5350b, SUC_lexicon_tags_23 },
    { 0x9d35350c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc4b5350e, SUC_lexicon_tags_8 },
    { 0x3a55350e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4153513, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf53518, SUC_lexicon_tags_82 },
    { 0x13753519, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195351e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b53522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2f53524, SUC_lexicon_tags_27 },
    { 0x4fd53524, SUC_lexicon_tags_11 },
    { 0x7d353524, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc15352a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb1f5352c, SUC_lexicon_tags_12 },
    { 0x5035352d, SUC_lexicon_tags_30 },
    { 0xb9f5352e, SUC_lexicon_tags_0 },
    { 0x5a95352f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2753532, SUC_lexicon_tags_40 },
    { 0x21153533, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4953537, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x975353a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x275353d, SUC_lexicon_tags_47 },
    { 0x8635353e, SUC_lexicon_tags_17 },
    { 0x45b5353f, SUC_lexicon_tags_2 },
    { 0xad95353f, SUC_lexicon_tags_10 },
    { 0x7775353d, SUC_lexicon_tags_37 },
    { 0x94d53542, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95354a, SUC_lexicon_tags_7 },
    { 0x97b5354a, SUC_lexicon_tags_1 },
    { 0xcbd5354a, SUC_lexicon_tags_11 },
    { 0x3f95354b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2553551, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7153555, SUC_lexicon_tags_12 },
    { 0xdd553555, SUC_lexicon_tags_15 },
    { 0x7a753556, SUC_lexicon_tags_1 },
    { 0x45753558, SUC_lexicon_tags_2 },
    { 0x7fd53556, SUC_lexicon_tags_0 },
    { 0xb5f53558, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff53560, SUC_lexicon_tags_12 },
    { 0x3753561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61953563, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4395356a, SUC_lexicon_tags_31 },
    { 0xe675356b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcb95356d, SUC_lexicon_tags_47 },
    { 0x6115356e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30953570, SUC_lexicon_tags_15 },
    { 0x38d53571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53578, SUC_lexicon_tags_1 },
    { 0x2d953579, SUC_lexicon_tags_2 },
    { 0x50553578, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb555357c, SUC_lexicon_tags_37 },
    { 0x13d5357d, SUC_lexicon_tags_2 },
    { 0xc415357e, SUC_lexicon_tags_1 },
    { 0x5a95357e, SUC_lexicon_tags_63 },
    { 0x51f53580, SUC_lexicon_tags_1 },
    { 0xcd53580, SUC_lexicon_tags_15 },
    { 0xe6753580, SUC_lexicon_tags_25 },
    { 0xf153583, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcdd53585, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdeb53587, SUC_lexicon_tags_16 },
    { 0xe1153588, SUC_lexicon_tags_2 },
    { 0x1ff53588, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6675358b, SUC_lexicon_tags_14 },
    { 0x7d15358c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45353593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f75359b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa15359d, SUC_lexicon_tags_33 },
    { 0x7175359e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5535a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad535a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21535aa, SUC_lexicon_tags_15 },
    { 0x199535ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7535ae, SUC_lexicon_tags_11 },
    { 0xadd535af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73535b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205535b7, SUC_lexicon_tags_15 },
    { 0x87b535b7, SUC_lexicon_tags_1 },
    { 0x3a7535b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x157535bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f535be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x551535c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b535c7, SUC_lexicon_tags_7 },
    { 0x83b535c8, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5535cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25535d0, SUC_lexicon_tags_0 },
    { 0x75535d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x20b535d3, SUC_lexicon_tags_37 },
    { 0x84d535d3, SUC_lexicon_tags_11 },
    { 0xe49535d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971535db, SUC_lexicon_tags_1 },
    { 0xecd535dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xef7535de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfe9535e0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x733535e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f3535e4, SUC_lexicon_tags_31 },
    { 0xe59535e5, SUC_lexicon_tags_14 },
    { 0x1db535e5, SUC_lexicon_tags_11 },
    { 0x75d535e5, SUC_lexicon_tags_26 },
    { 0x695535e8, SUC_lexicon_tags_23 },
    { 0xcf7535e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61535ec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf91535ee, SUC_lexicon_tags_7 },
    { 0xf41535ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1535f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x207535f7, SUC_lexicon_tags_17 },
    { 0xdc9535f8, SUC_lexicon_tags_27 },
    { 0xf15535f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x749535fc, SUC_lexicon_tags_31 },
    { 0x2e5535fc, SUC_lexicon_tags_12 },
    { 0x2eb535fd, SUC_lexicon_tags_17 },
    { 0x8ad535fc, SUC_lexicon_tags_547 },
    { 0xe2553600, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5353603, SUC_lexicon_tags_14 },
    { 0xe5953604, SUC_lexicon_tags_1 },
    { 0x8cf53604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5360a, SUC_lexicon_tags_71 },
    { 0xb475360b, SUC_lexicon_tags_25 },
    { 0x6f5360c, SUC_lexicon_tags_0 },
    { 0x455360d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6353610, SUC_lexicon_tags_22 },
    { 0xf4353611, SUC_lexicon_tags_26 },
    { 0xb2b53612, SUC_lexicon_tags_65 },
    { 0x32f53611, SUC_lexicon_tags_33 },
    { 0x4a153612, SUC_lexicon_tags_19 },
    { 0x57753612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9db53617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9435361a, SUC_lexicon_tags_19 },
    { 0x1555361b, SUC_lexicon_tags_43 },
    { 0xb015361c, SUC_lexicon_tags_92 },
    { 0x4075361d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf35361f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f53623, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78353625, SUC_lexicon_tags_37 },
    { 0xaf353625, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf53629, SUC_lexicon_tags_7 },
    { 0x4b5362a, SUC_lexicon_tags_7 },
    { 0x60f5362b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7af5362d, SUC_lexicon_tags_11 },
    { 0x4135362e, SUC_lexicon_tags_37 },
    { 0x4955362e, SUC_lexicon_tags_43 },
    { 0x2855362f, SUC_lexicon_tags_17 },
    { 0xcc35362d, SUC_lexicon_tags_19 },
    { 0xd5553631, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b53636, SUC_lexicon_tags_37 },
    { 0xbeb53637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7953639, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb5363c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d53641, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8db53643, SUC_lexicon_tags_17 },
    { 0x8b953644, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2095364a, SUC_lexicon_tags_17 },
    { 0xe5d5364b, SUC_lexicon_tags_32 },
    { 0x7815364b, SUC_lexicon_tags_43 },
    { 0x1a75364b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd53650, SUC_lexicon_tags_32 },
    { 0xb8f53651, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa953655, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc8153657, SUC_lexicon_tags_12 },
    { 0x26553657, SUC_lexicon_tags_25 },
    { 0xb0953658, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe935365e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x93553660, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbb553662, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x32153664, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd53668, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf655366d, SUC_lexicon_tags_44 },
    { 0x59b5366e, SUC_lexicon_tags_31 },
    { 0x9475366f, SUC_lexicon_tags_15 },
    { 0xab5366e, SUC_lexicon_tags_47 },
    { 0xb0353671, SUC_lexicon_tags_63 },
    { 0x41953672, SUC_lexicon_tags_37 },
    { 0xe815366d, SUC_lexicon_tags_45 },
    { 0xa4f53674, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f53679, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7e35367b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54d5367d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x44f5367f, SUC_lexicon_tags_17 },
    { 0x79153680, SUC_lexicon_tags_7 },
    { 0x23b53680, SUC_lexicon_tags_49 },
    { 0x6af53681, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x79953685, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf53689, SUC_lexicon_tags_19 },
    { 0xab75368a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf5368f, SUC_lexicon_tags_1 },
    { 0x9a753690, SUC_lexicon_tags_37 },
    { 0x23f53690, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3f53693, SUC_lexicon_tags_146 },
    { 0x6a753694, SUC_lexicon_tags_15 },
    { 0x21753695, SUC_lexicon_tags_15 },
    { 0xd1f53696, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd5369c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25536a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x123536a2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873536ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b536af, SUC_lexicon_tags_30 },
    { 0xff5536b0, SUC_lexicon_tags_2 },
    { 0x7ed536b1, SUC_lexicon_tags_21 },
    { 0x841536b1, SUC_lexicon_tags_37 },
    { 0x6d5536b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4f536b5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d536b9, SUC_lexicon_tags_2 },
    { 0x4e9536b9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9536c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac7536c3, SUC_lexicon_tags_38 },
    { 0x13536c4, SUC_lexicon_tags_2 },
    { 0x38d536c5, SUC_lexicon_tags_19 },
    { 0xc39536c4, SUC_lexicon_tags_48 },
    { 0xda3536c7, SUC_lexicon_tags_17 },
    { 0xc81536c7, SUC_lexicon_tags_43 },
    { 0x24d536c9, SUC_lexicon_tags_9 },
    { 0x111536ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd3536cc, SUC_lexicon_tags_37 },
    { 0xc95536cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d536d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15536dc, SUC_lexicon_tags_7 },
    { 0x1bb536dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5536e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x773536e3, SUC_lexicon_tags_11 },
    { 0xb0b536e4, SUC_lexicon_tags_14 },
    { 0x913536e4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xa9f536e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd536f6, SUC_lexicon_tags_12 },
    { 0x39536f6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xff3536f9, SUC_lexicon_tags_68 },
    { 0x19d536fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dd536fc, SUC_lexicon_tags_56 },
    { 0xf536fc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7353700, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b53705, SUC_lexicon_tags_25 },
    { 0x89b53706, SUC_lexicon_tags_29 },
    { 0xf553705, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91953709, SUC_lexicon_tags_37 },
    { 0xc3b53709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab5370d, SUC_lexicon_tags_223 },
    { 0, NULL },
    { 0xe675370f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x56953712, SUC_lexicon_tags_49 },
    { 0xd1353712, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6753717, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd5371d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d53722, SUC_lexicon_tags_9 },
    { 0x25353723, SUC_lexicon_tags_187 },
    { 0x6553723, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b353728, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c95372b, SUC_lexicon_tags_38 },
    { 0xcbd5372c, SUC_lexicon_tags_84 },
    { 0x9735372c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9055372f, SUC_lexicon_tags_37 },
    { 0x3d953730, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3153736, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8c553738, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb975373b, SUC_lexicon_tags_47 },
    { 0x2d15373c, SUC_lexicon_tags_7 },
    { 0xdc75373d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65f5373f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c153744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4753748, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d5374c, SUC_lexicon_tags_6 },
    { 0x1bf5374c, SUC_lexicon_tags_23 },
    { 0x4ef5374c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60953756, SUC_lexicon_tags_21 },
    { 0xc8f53756, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf3153759, SUC_lexicon_tags_15 },
    { 0x71d5375a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f53761, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b53766, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7cb53768, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e35376a, SUC_lexicon_tags_2 },
    { 0x75d5376b, SUC_lexicon_tags_26 },
    { 0xa155376b, SUC_lexicon_tags_38 },
    { 0x1775376c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f53770, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f553772, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5f53774, SUC_lexicon_tags_38 },
    { 0x62753775, SUC_lexicon_tags_46 },
    { 0xd2953776, SUC_lexicon_tags_12 },
    { 0xf7753777, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe9953779, SUC_lexicon_tags_55 },
    { 0xa875377a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69d5377c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd395377f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61753787, SUC_lexicon_tags_11 },
    { 0xa6553788, SUC_lexicon_tags_54 },
    { 0xeb953789, SUC_lexicon_tags_60 },
    { 0xaf15378a, SUC_lexicon_tags_43 },
    { 0x7895378b, SUC_lexicon_tags_45 },
    { 0xb855378b, SUC_lexicon_tags_37 },
    { 0xdf95378c, SUC_lexicon_tags_30 },
    { 0x2b35378d, SUC_lexicon_tags_31 },
    { 0x3d15378f, SUC_lexicon_tags_37 },
    { 0xd6b53790, SUC_lexicon_tags_9 },
    { 0xb6153791, SUC_lexicon_tags_15 },
    { 0x62f53792, SUC_lexicon_tags_1 },
    { 0x66553790, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87d53795, SUC_lexicon_tags_2 },
    { 0x8d153795, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7753798, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf575379a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657537a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3537a7, SUC_lexicon_tags_45 },
    { 0x23d537a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7537ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f537b0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763537b8, SUC_lexicon_tags_510 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713537bc, SUC_lexicon_tags_0 },
    { 0xf93537bd, SUC_lexicon_tags_9 },
    { 0xe99537bc, SUC_lexicon_tags_26 },
    { 0x481537bf, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61537c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6d537c7, SUC_lexicon_tags_49 },
    { 0xb9b537c8, SUC_lexicon_tags_1 },
    { 0x239537c7, SUC_lexicon_tags_1 },
    { 0xef9537c8, SUC_lexicon_tags_17 },
    { 0x97b537c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191537cf, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f537d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5537d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd537dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f537e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5d7537e6, SUC_lexicon_tags_1 },
    { 0x99537e7, SUC_lexicon_tags_45 },
    { 0xef9537e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f537ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1537f1, SUC_lexicon_tags_47 },
    { 0x27f537f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b537fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79537ff, SUC_lexicon_tags_12 },
    { 0xced53800, SUC_lexicon_tags_38 },
    { 0xfa953800, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8953805, SUC_lexicon_tags_1 },
    { 0xa5553806, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a553809, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb375380b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33553811, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x73353813, SUC_lexicon_tags_91 },
    { 0xbf53813, SUC_lexicon_tags_208 },
    { 0x6b53813, SUC_lexicon_tags_12 },
    { 0xb5553816, SUC_lexicon_tags_37 },
    { 0x65d53813, SUC_lexicon_tags_201 },
    { 0x7553818, SUC_lexicon_tags_2 },
    { 0x35d53818, SUC_lexicon_tags_2 },
    { 0x1e35381a, SUC_lexicon_tags_17 },
    { 0xcb35381b, SUC_lexicon_tags_11 },
    { 0x3315381b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f53830, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97553833, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d53838, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2215383b, SUC_lexicon_tags_43 },
    { 0x3095383b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2d5383e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1ef53840, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5eb53842, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f53845, SUC_lexicon_tags_15 },
    { 0xb0d53845, SUC_lexicon_tags_2 },
    { 0xf4753846, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd35384e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x63553850, SUC_lexicon_tags_17 },
    { 0x97153851, SUC_lexicon_tags_149 },
    { 0x6fd53852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb53856, SUC_lexicon_tags_26 },
    { 0x7e953856, SUC_lexicon_tags_38 },
    { 0xac153858, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b35385a, SUC_lexicon_tags_70 },
    { 0xa4d5385b, SUC_lexicon_tags_42 },
    { 0x9bb5385a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb895385e, SUC_lexicon_tags_8 },
    { 0xb375385f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x59153861, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb753863, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d53866, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85953868, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xceb5386a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd953871, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6b553873, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3353876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d53879, SUC_lexicon_tags_37 },
    { 0xa035387a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73d5387c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f5387f, SUC_lexicon_tags_31 },
    { 0xb5f5387f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7953884, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b553888, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d55388a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f53891, SUC_lexicon_tags_37 },
    { 0x2d553892, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b53896, SUC_lexicon_tags_29 },
    { 0xcf153897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b75389a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb715389c, SUC_lexicon_tags_2 },
    { 0x68d5389d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17538a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637538a8, SUC_lexicon_tags_29 },
    { 0x95f538a9, SUC_lexicon_tags_15 },
    { 0xf3538a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f538ae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb538b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x221538b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd39538b8, SUC_lexicon_tags_91 },
    { 0x6a1538b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd538bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d538c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2e9538c4, SUC_lexicon_tags_31 },
    { 0x635538c5, SUC_lexicon_tags_8 },
    { 0x215538c6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5a7538c8, SUC_lexicon_tags_7 },
    { 0xc29538c9, SUC_lexicon_tags_11 },
    { 0xb0d538ca, SUC_lexicon_tags_14 },
    { 0xa93538cb, SUC_lexicon_tags_47 },
    { 0x569538cb, SUC_lexicon_tags_11 },
    { 0x331538cd, SUC_lexicon_tags_31 },
    { 0x293538cc, SUC_lexicon_tags_30 },
    { 0x8e9538cf, SUC_lexicon_tags_25 },
    { 0xc09538d0, SUC_lexicon_tags_35 },
    { 0x531538d1, SUC_lexicon_tags_29 },
    { 0x869538d1, SUC_lexicon_tags_21 },
    { 0x7cd538ce, SUC_lexicon_tags_0 },
    { 0x575538cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75538d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1cf538d9, SUC_lexicon_tags_15 },
    { 0x837538da, SUC_lexicon_tags_25 },
    { 0x893538db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef538e2, SUC_lexicon_tags_713 },
    { 0x943538e3, SUC_lexicon_tags_15 },
    { 0xbcd538e3, SUC_lexicon_tags_17 },
    { 0xf81538e3, SUC_lexicon_tags_7 },
    { 0x9a9538e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d538e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759538ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b538f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137538fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1538fe, SUC_lexicon_tags_38 },
    { 0x77538ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfe353901, SUC_lexicon_tags_13 },
    { 0xfdf53901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74153906, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8715390b, SUC_lexicon_tags_17 },
    { 0x6a15390c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8495390e, SUC_lexicon_tags_38 },
    { 0x32b5390f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d5391a, SUC_lexicon_tags_43 },
    { 0x8f95391a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a15391e, SUC_lexicon_tags_45 },
    { 0x6835391f, SUC_lexicon_tags_11 },
    { 0xbbb53920, SUC_lexicon_tags_31 },
    { 0xd8553921, SUC_lexicon_tags_30 },
    { 0x7d553921, SUC_lexicon_tags_43 },
    { 0xc5753921, SUC_lexicon_tags_14 },
    { 0x22d53921, SUC_lexicon_tags_45 },
    { 0xc6953923, SUC_lexicon_tags_7 },
    { 0xabd53926, SUC_lexicon_tags_27 },
    { 0xe3353927, SUC_lexicon_tags_47 },
    { 0xad353920, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6735392a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b5392d, SUC_lexicon_tags_11 },
    { 0xb855392e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b753931, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x36753933, SUC_lexicon_tags_11 },
    { 0x5b53934, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df53937, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70553939, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2455393c, SUC_lexicon_tags_0 },
    { 0x4f15393d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab53940, SUC_lexicon_tags_15 },
    { 0x88153941, SUC_lexicon_tags_30 },
    { 0xd5553942, SUC_lexicon_tags_7 },
    { 0x17953943, SUC_lexicon_tags_2 },
    { 0x13553944, SUC_lexicon_tags_37 },
    { 0x5b153941, SUC_lexicon_tags_92 },
    { 0x1e353943, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x27b53948, SUC_lexicon_tags_2 },
    { 0x5af53949, SUC_lexicon_tags_11 },
    { 0xbf35394a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc855394d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7c35394f, SUC_lexicon_tags_276 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed953953, SUC_lexicon_tags_2 },
    { 0x83d53953, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54153957, SUC_lexicon_tags_17 },
    { 0x9df53958, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3b5395a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c75395d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c35395f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e753964, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd5396c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10353979, SUC_lexicon_tags_87 },
    { 0xb2f53979, SUC_lexicon_tags_9 },
    { 0x47153979, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x295397d, SUC_lexicon_tags_14 },
    { 0x49b5397e, SUC_lexicon_tags_47 },
    { 0xb7d5397f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f53982, SUC_lexicon_tags_7 },
    { 0x29b53983, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f53986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74553990, SUC_lexicon_tags_17 },
    { 0x24553991, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b53994, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad753998, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29539a3, SUC_lexicon_tags_33 },
    { 0xc23539a4, SUC_lexicon_tags_29 },
    { 0xfaf539a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f539a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x317539aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x565539ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3df539af, SUC_lexicon_tags_37 },
    { 0x29539b0, SUC_lexicon_tags_30 },
    { 0x93539b1, SUC_lexicon_tags_17 },
    { 0x615539b0, SUC_lexicon_tags_39 },
    { 0x1c7539b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7539ba, SUC_lexicon_tags_14 },
    { 0xf9d539bb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995539c3, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xf4b539c5, SUC_lexicon_tags_25 },
    { 0xad3539c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9539cd, SUC_lexicon_tags_29 },
    { 0x2e3539ce, SUC_lexicon_tags_32 },
    { 0xd1f539ce, SUC_lexicon_tags_201 },
    { 0x6c5539ce, SUC_lexicon_tags_30 },
    { 0x655539d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x973539d4, SUC_lexicon_tags_11 },
    { 0xee9539d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85539d8, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xb9d539da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f539dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f539e0, SUC_lexicon_tags_19 },
    { 0x1dd539e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf539e4, SUC_lexicon_tags_14 },
    { 0x1ef539e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x613539e7, SUC_lexicon_tags_12 },
    { 0xac1539e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7539ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b539ef, SUC_lexicon_tags_45 },
    { 0x3c3539ef, SUC_lexicon_tags_43 },
    { 0x537539f1, SUC_lexicon_tags_1 },
    { 0x605539ef, SUC_lexicon_tags_47 },
    { 0x765539f2, SUC_lexicon_tags_11 },
    { 0x835539f4, SUC_lexicon_tags_19 },
    { 0x2ff539ef, SUC_lexicon_tags_36 },
    { 0x411539f6, SUC_lexicon_tags_56 },
    { 0x7c5539f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b539fa, SUC_lexicon_tags_11 },
    { 0xea1539fb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9539ff, SUC_lexicon_tags_25 },
    { 0x2a7539ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d53a03, SUC_lexicon_tags_2 },
    { 0x4a353a04, SUC_lexicon_tags_13 },
    { 0x28553a05, SUC_lexicon_tags_25 },
    { 0xaf53a05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3753a09, SUC_lexicon_tags_26 },
    { 0xdb753a0a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfed53a0c, SUC_lexicon_tags_11 },
    { 0x4a553a0d, SUC_lexicon_tags_14 },
    { 0x44d53a0e, SUC_lexicon_tags_34 },
    { 0x5b53a0e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6dd53a11, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x14953a13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb753a16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87d53a18, SUC_lexicon_tags_2 },
    { 0x4ef53a19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83153a1c, SUC_lexicon_tags_17 },
    { 0xb753a1d, SUC_lexicon_tags_17 },
    { 0xf9f53a1e, SUC_lexicon_tags_6 },
    { 0xed353a1e, SUC_lexicon_tags_6 },
    { 0xbbf53a1f, SUC_lexicon_tags_26 },
    { 0x6d353a1f, SUC_lexicon_tags_12 },
    { 0xfc553a21, SUC_lexicon_tags_49 },
    { 0x23953a21, SUC_lexicon_tags_15 },
    { 0xe2153a24, SUC_lexicon_tags_10 },
    { 0x11b53a24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x64153a27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7153a2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2153a2e, SUC_lexicon_tags_47 },
    { 0x45353a2e, SUC_lexicon_tags_8 },
    { 0x9b953a30, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x75953a33, SUC_lexicon_tags_28 },
    { 0xc6b53a34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24953a40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff953a44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e553a49, SUC_lexicon_tags_25 },
    { 0x76553a4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fb53a4c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c553a50, SUC_lexicon_tags_35 },
    { 0xc7753a50, SUC_lexicon_tags_9 },
    { 0x90553a50, SUC_lexicon_tags_14 },
    { 0x23753a51, SUC_lexicon_tags_0 },
    { 0x31f53a51, SUC_lexicon_tags_14 },
    { 0x3553a52, SUC_lexicon_tags_15 },
    { 0x6b553a50, SUC_lexicon_tags_15 },
    { 0xc7753a53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9353a59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30953a64, SUC_lexicon_tags_33 },
    { 0xad953a65, SUC_lexicon_tags_12 },
    { 0xdb953a65, SUC_lexicon_tags_30 },
    { 0xfab53a64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9753a69, SUC_lexicon_tags_12 },
    { 0x8f53a69, SUC_lexicon_tags_66 },
    { 0x9e953a6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1b53a6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f53a71, SUC_lexicon_tags_25 },
    { 0xfd53a72, SUC_lexicon_tags_37 },
    { 0x7dd53a72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98953a77, SUC_lexicon_tags_14 },
    { 0x5d153a78, SUC_lexicon_tags_47 },
    { 0xb9953a78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c353a83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x55f53a85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b53a8b, SUC_lexicon_tags_48 },
    { 0xb0f53a8b, SUC_lexicon_tags_17 },
    { 0xa9753a8b, SUC_lexicon_tags_27 },
    { 0x1b153a8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd53a91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc753a94, SUC_lexicon_tags_8 },
    { 0x81953a94, SUC_lexicon_tags_14 },
    { 0x4c153a94, SUC_lexicon_tags_19 },
    { 0x153a97, SUC_lexicon_tags_826 },
    { 0x24d53a98, SUC_lexicon_tags_33 },
    { 0x97353a99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6bd53a9b, SUC_lexicon_tags_45 },
    { 0xf5f53a9c, SUC_lexicon_tags_17 },
    { 0x2ef53a9d, SUC_lexicon_tags_11 },
    { 0x41f53a9d, SUC_lexicon_tags_23 },
    { 0x85353a9f, SUC_lexicon_tags_27 },
    { 0xf53aa0, SUC_lexicon_tags_47 },
    { 0xbb153aa0, SUC_lexicon_tags_1 },
    { 0x89153aa2, SUC_lexicon_tags_60 },
    { 0x94953aa2, SUC_lexicon_tags_23 },
    { 0x9a753aa1, SUC_lexicon_tags_11 },
    { 0x4b353a9e, SUC_lexicon_tags_28 },
    { 0x1153aa6, SUC_lexicon_tags_37 },
    { 0xd353aa6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x88153aa9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4753aad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32b53aaf, SUC_lexicon_tags_15 },
    { 0xb4b53aaf, SUC_lexicon_tags_17 },
    { 0x20353ab1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99753ab6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc153ab9, SUC_lexicon_tags_45 },
    { 0x16153aba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f53ac2, SUC_lexicon_tags_17 },
    { 0xb9953ac3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6df53ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d553ac7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x66f53ac9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9553acd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6753ad4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78f53ad6, SUC_lexicon_tags_33 },
    { 0xac553ad7, SUC_lexicon_tags_1 },
    { 0x21d53ad8, SUC_lexicon_tags_9 },
    { 0x1ed53ad8, SUC_lexicon_tags_65 },
    { 0x72553ad8, SUC_lexicon_tags_15 },
    { 0xcd753ad6, SUC_lexicon_tags_49 },
    { 0xbad53ada, SUC_lexicon_tags_6 },
    { 0xe8153ad6, SUC_lexicon_tags_19 },
    { 0xf1553ad9, SUC_lexicon_tags_191 },
    { 0x94d53adf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5553ae5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf53ae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1153aeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63553aee, SUC_lexicon_tags_38 },
    { 0x90f53aef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x80553af1, SUC_lexicon_tags_26 },
    { 0x68f53af2, SUC_lexicon_tags_14 },
    { 0x7bf53af1, SUC_lexicon_tags_11 },
    { 0x23553af4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d53af7, SUC_lexicon_tags_43 },
    { 0x2e553af7, SUC_lexicon_tags_25 },
    { 0x91d53af8, SUC_lexicon_tags_15 },
    { 0xe1353af7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x58353afd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f353aff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b153b06, SUC_lexicon_tags_11 },
    { 0x2c153b07, SUC_lexicon_tags_4 },
    { 0xccf53b06, SUC_lexicon_tags_25 },
    { 0xdcf53b09, SUC_lexicon_tags_7 },
    { 0x17153b0a, SUC_lexicon_tags_4 },
    { 0x753b0b, SUC_lexicon_tags_63 },
    { 0x42153b0c, SUC_lexicon_tags_26 },
    { 0xbdd53b0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x19353b10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56753b12, SUC_lexicon_tags_33 },
    { 0xb7353b12, SUC_lexicon_tags_7 },
    { 0x7a353b14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fb53b16, SUC_lexicon_tags_11 },
    { 0x93353b16, SUC_lexicon_tags_49 },
    { 0x9b53b16, SUC_lexicon_tags_8 },
    { 0xb0b53b18, SUC_lexicon_tags_11 },
    { 0x35b53b1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c953b1c, SUC_lexicon_tags_6 },
    { 0xf2f53b1d, SUC_lexicon_tags_49 },
    { 0x1f753b1c, SUC_lexicon_tags_11 },
    { 0x6b353b1f, SUC_lexicon_tags_11 },
    { 0x6b153b20, SUC_lexicon_tags_15 },
    { 0x19b53b1d, SUC_lexicon_tags_37 },
    { 0xf3353b22, SUC_lexicon_tags_11 },
    { 0x1cd53b23, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf153b27, SUC_lexicon_tags_25 },
    { 0x78d53b28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31553b32, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7cd53b34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8d53b36, SUC_lexicon_tags_15 },
    { 0x78753b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e953b39, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d553b3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1753b3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90b53b40, SUC_lexicon_tags_2 },
    { 0x4ed53b40, SUC_lexicon_tags_282 },
    { 0xdcb53b42, SUC_lexicon_tags_37 },
    { 0xa7153b42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e553b47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f53b53, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x12353b55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd553b5c, SUC_lexicon_tags_11 },
    { 0xc3353b5d, SUC_lexicon_tags_141 },
    { 0x8f153b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11153b60, SUC_lexicon_tags_48 },
    { 0x37953b60, SUC_lexicon_tags_7 },
    { 0x27753b62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac953b68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b153b6b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8d953b6d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99953b71, SUC_lexicon_tags_37 },
    { 0x79d53b72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b53b75, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d553b79, SUC_lexicon_tags_1 },
    { 0x50953b7a, SUC_lexicon_tags_11 },
    { 0xa953b7a, SUC_lexicon_tags_84 },
    { 0xca753b7c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd53b83, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa3553b85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b953b87, SUC_lexicon_tags_7 },
    { 0xfa553b88, SUC_lexicon_tags_31 },
    { 0xd5f53b89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f753b8b, SUC_lexicon_tags_7 },
    { 0x9d753b8c, SUC_lexicon_tags_33 },
    { 0xb5553b8b, SUC_lexicon_tags_11 },
    { 0x49f53b8e, SUC_lexicon_tags_25 },
    { 0x37f53b8c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb53b92, SUC_lexicon_tags_17 },
    { 0x67d53b93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b953b97, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1153b99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26353ba0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87553ba9, SUC_lexicon_tags_13 },
    { 0x9f353baa, SUC_lexicon_tags_7 },
    { 0x90d53ba9, SUC_lexicon_tags_1 },
    { 0xb6353bab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8f53bae, SUC_lexicon_tags_7 },
    { 0x8a753baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66553bb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92d53bb6, SUC_lexicon_tags_27 },
    { 0xf5353bb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31d53bb9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab53bbc, SUC_lexicon_tags_19 },
    { 0x9b753bbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b953bc1, SUC_lexicon_tags_9 },
    { 0xca553bc2, SUC_lexicon_tags_3 },
    { 0x19753bc2, SUC_lexicon_tags_2 },
    { 0x85d53bc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf53bc7, SUC_lexicon_tags_25 },
    { 0x74f53bc8, SUC_lexicon_tags_63 },
    { 0xf5953bc8, SUC_lexicon_tags_59 },
    { 0x5a753bca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f53bce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa4953bd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a153bd3, SUC_lexicon_tags_13 },
    { 0x5a353bd4, SUC_lexicon_tags_1 },
    { 0x63f53bd5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b953bd8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e953bdb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x74153bde, SUC_lexicon_tags_7 },
    { 0xcc153bde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a953be3, SUC_lexicon_tags_1 },
    { 0x32f53be4, SUC_lexicon_tags_0 },
    { 0xa0353be4, SUC_lexicon_tags_63 },
    { 0x19953be3, SUC_lexicon_tags_31 },
    { 0xa9153be7, SUC_lexicon_tags_31 },
    { 0x72d53be4, SUC_lexicon_tags_27 },
    { 0xbdb53be9, SUC_lexicon_tags_37 },
    { 0xc2f53be5, SUC_lexicon_tags_15 },
    { 0xe9753beb, SUC_lexicon_tags_33 },
    { 0x2ab53beb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b53bef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f753bf5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa953bf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b53bfe, SUC_lexicon_tags_31 },
    { 0x4e553bfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb153c02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19b53c04, SUC_lexicon_tags_33 },
    { 0x32153c05, SUC_lexicon_tags_29 },
    { 0x96153c05, SUC_lexicon_tags_3 },
    { 0x63553c06, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x73953c0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4df53c0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55953c0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27753c13, SUC_lexicon_tags_11 },
    { 0x2353c14, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbdd53c16, SUC_lexicon_tags_1 },
    { 0x39153c16, SUC_lexicon_tags_21 },
    { 0x83f53c17, SUC_lexicon_tags_1 },
    { 0xe4f53c17, SUC_lexicon_tags_7 },
    { 0xb1753c1a, SUC_lexicon_tags_7 },
    { 0x80153c1b, SUC_lexicon_tags_37 },
    { 0x1b953c1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45953c22, SUC_lexicon_tags_23 },
    { 0x7f953c23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb53c29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12753c2c, SUC_lexicon_tags_4 },
    { 0x77153c2d, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c753c31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9d53c33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d53c37, SUC_lexicon_tags_37 },
    { 0xf5553c37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf53c3c, SUC_lexicon_tags_27 },
    { 0x2db53c3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6753c42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x27353c44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f53c49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x12753c4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d753c4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x47353c4f, SUC_lexicon_tags_60 },
    { 0x5df53c50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd753c53, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x13753c55, SUC_lexicon_tags_0 },
    { 0x17f53c55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1153c58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x52953c5a, SUC_lexicon_tags_77 },
    { 0x9953c5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92153c5d, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb53c65, SUC_lexicon_tags_33 },
    { 0xb9153c66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf53c69, SUC_lexicon_tags_32 },
    { 0x7a353c69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b53c6d, SUC_lexicon_tags_25 },
    { 0x34b53c6e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1b953c70, SUC_lexicon_tags_38 },
    { 0xd1d53c71, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab53c77, SUC_lexicon_tags_26 },
    { 0xa5953c78, SUC_lexicon_tags_2 },
    { 0xf0753c77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56353c7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b53c82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd153c85, SUC_lexicon_tags_7 },
    { 0x4dd53c86, SUC_lexicon_tags_2 },
    { 0xb4f53c87, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x60153c89, SUC_lexicon_tags_7 },
    { 0x25353c89, SUC_lexicon_tags_37 },
    { 0xae753c8b, SUC_lexicon_tags_15 },
    { 0x2cf53c8c, SUC_lexicon_tags_0 },
    { 0xcd353c8d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d53c96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4b353c98, SUC_lexicon_tags_11 },
    { 0x14953c99, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4753c9c, SUC_lexicon_tags_33 },
    { 0xcc753c9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9353ca6, SUC_lexicon_tags_2 },
    { 0xb1153ca7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d53cac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf53caf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f53cb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf53cb8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d153cbb, SUC_lexicon_tags_2 },
    { 0x3753cbc, SUC_lexicon_tags_37 },
    { 0x90153cbd, SUC_lexicon_tags_12 },
    { 0x51d53cbb, SUC_lexicon_tags_11 },
    { 0x9353cbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af53cc3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc9b53cc5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a553cd2, SUC_lexicon_tags_37 },
    { 0xbb153cd3, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0x62753cd6, SUC_lexicon_tags_3 },
    { 0x1ef53cd6, SUC_lexicon_tags_8 },
    { 0x46d53cd8, SUC_lexicon_tags_25 },
    { 0x23353cd9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9753cdd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87153ce1, SUC_lexicon_tags_0 },
    { 0xf1f53ce2, SUC_lexicon_tags_19 },
    { 0x26553ce2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5153ce5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xab153ce8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x64553ceb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe753cf4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x29953cf7, SUC_lexicon_tags_65 },
    { 0x8153cf8, SUC_lexicon_tags_26 },
    { 0x6fd53cf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62953cfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab53cff, SUC_lexicon_tags_28 },
    { 0x69d53cff, SUC_lexicon_tags_1 },
    { 0xb7753cff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d53d07, SUC_lexicon_tags_2 },
    { 0x42b53d08, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76353d0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbdd53d0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46753d14, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c153d1b, SUC_lexicon_tags_25 },
    { 0x80753d1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe0b53d1e, SUC_lexicon_tags_79 },
    { 0x2b353d1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x57553d21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc4153d23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90953d25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xea353d27, SUC_lexicon_tags_17 },
    { 0xc353d28, SUC_lexicon_tags_17 },
    { 0x4a153d27, SUC_lexicon_tags_113 },
    { 0xb0753d28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd0753d2c, SUC_lexicon_tags_17 },
    { 0x73753d2d, SUC_lexicon_tags_17 },
    { 0x66153d2e, SUC_lexicon_tags_66 },
    { 0x153d2f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b53d34, SUC_lexicon_tags_57 },
    { 0x7ab53d35, SUC_lexicon_tags_47 },
    { 0x46353d35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc753d3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae153d3d, SUC_lexicon_tags_0 },
    { 0xd553d3e, SUC_lexicon_tags_31 },
    { 0x6eb53d3f, SUC_lexicon_tags_1 },
    { 0xc7553d3e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d53d43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22f53d45, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c953d4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f953d50, SUC_lexicon_tags_11 },
    { 0x71353d50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62553d55, SUC_lexicon_tags_25 },
    { 0xed953d56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb153d5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a153d5e, SUC_lexicon_tags_12 },
    { 0xf4d53d5f, SUC_lexicon_tags_14 },
    { 0xa2f53d60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e553d67, SUC_lexicon_tags_79 },
    { 0xd8b53d68, SUC_lexicon_tags_1 },
    { 0xa4553d68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b153d6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80953d6f, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3153d72, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbff53d74, SUC_lexicon_tags_15 },
    { 0x61753d75, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3ad53d77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xde753d7a, SUC_lexicon_tags_43 },
    { 0xa7f53d7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf53d7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad53d83, SUC_lexicon_tags_25 },
    { 0x4f553d84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b53d89, SUC_lexicon_tags_14 },
    { 0x59d53d8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d353d8c, SUC_lexicon_tags_1 },
    { 0x2d353d8d, SUC_lexicon_tags_57 },
    { 0xead53d8d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff53d93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4753d99, SUC_lexicon_tags_11 },
    { 0x4f53d9a, SUC_lexicon_tags_30 },
    { 0x86753d9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb53d9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28553da6, SUC_lexicon_tags_3 },
    { 0xad53da7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x48953da9, SUC_lexicon_tags_47 },
    { 0x5e753daa, SUC_lexicon_tags_25 },
    { 0x96f53dab, SUC_lexicon_tags_11 },
    { 0xfd553dac, SUC_lexicon_tags_30 },
    { 0xce553dab, SUC_lexicon_tags_47 },
    { 0x2d553dae, SUC_lexicon_tags_45 },
    { 0x5db53dab, SUC_lexicon_tags_14 },
    { 0xa8153dad, SUC_lexicon_tags_29 },
    { 0xa6753dae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f53db5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b953db9, SUC_lexicon_tags_2 },
    { 0xbbd53dba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe2553dbc, SUC_lexicon_tags_17 },
    { 0x2fb53dbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63753dbf, SUC_lexicon_tags_68 },
    { 0x93353dbf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfb153dc2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9953dc5, SUC_lexicon_tags_29 },
    { 0x6d553dc5, SUC_lexicon_tags_2 },
    { 0x52353dc7, SUC_lexicon_tags_17 },
    { 0x92d53dc8, SUC_lexicon_tags_17 },
    { 0x11753dc9, SUC_lexicon_tags_0 },
    { 0xee153dc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c953dcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d353dce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x93153dd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64353dd2, SUC_lexicon_tags_37 },
    { 0x34953dd3, SUC_lexicon_tags_15 },
    { 0x32f53dd4, SUC_lexicon_tags_63 },
    { 0x6b953dd4, SUC_lexicon_tags_12 },
    { 0x8a953dd5, SUC_lexicon_tags_179 },
    { 0xd6f53dd7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb53de4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6353de7, SUC_lexicon_tags_5 },
    { 0xe0153de8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32353ded, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8153df1, SUC_lexicon_tags_12 },
    { 0xda553df2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b53df6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8f53df8, SUC_lexicon_tags_49 },
    { 0x89d53df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16f53dfb, SUC_lexicon_tags_68 },
    { 0x68553dfb, SUC_lexicon_tags_3 },
    { 0xb7d53dfd, SUC_lexicon_tags_11 },
    { 0x24553dfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1af53e00, SUC_lexicon_tags_43 },
    { 0xb8553e00, SUC_lexicon_tags_43 },
    { 0x9d553e02, SUC_lexicon_tags_25 },
    { 0x86353e03, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xfbf53e05, SUC_lexicon_tags_16 },
    { 0x8b753e05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d53e09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf53e0c, SUC_lexicon_tags_7 },
    { 0x7ff53e0d, SUC_lexicon_tags_27 },
    { 0xef953e0d, SUC_lexicon_tags_29 },
    { 0x9353e0e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf53e13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f753e17, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2753e1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb153e22, SUC_lexicon_tags_2 },
    { 0x6cf53e23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b53e28, SUC_lexicon_tags_65 },
    { 0xf2553e29, SUC_lexicon_tags_25 },
    { 0x8753e29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a553e33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21b53e35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63d53e37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0353e3d, SUC_lexicon_tags_9 },
    { 0x80b53e3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48153e40, SUC_lexicon_tags_33 },
    { 0xf0953e40, SUC_lexicon_tags_17 },
    { 0xfc153e42, SUC_lexicon_tags_19 },
    { 0xc2953e43, SUC_lexicon_tags_31 },
    { 0xc0f53e43, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf953e47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96f53e49, SUC_lexicon_tags_7 },
    { 0xc3f53e4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b553e4f, SUC_lexicon_tags_33 },
    { 0xc1753e50, SUC_lexicon_tags_9 },
    { 0x1a553e51, SUC_lexicon_tags_37 },
    { 0x21d53e52, SUC_lexicon_tags_11 },
    { 0xea953e53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7bb53e55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60553e59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf953e5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84753e5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e553e62, SUC_lexicon_tags_31 },
    { 0x9ef53e63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b553e68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xef353e6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa153e71, SUC_lexicon_tags_37 },
    { 0x17b53e72, SUC_lexicon_tags_45 },
    { 0x92553e73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf353e78, SUC_lexicon_tags_41 },
    { 0xb8553e79, SUC_lexicon_tags_37 },
    { 0x50b53e79, SUC_lexicon_tags_1 },
    { 0x8c953e7b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa6953e7d, SUC_lexicon_tags_12 },
    { 0x2c153e7e, SUC_lexicon_tags_1 },
    { 0xb8153e7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b53e82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9353e88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58153e8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb553e8d, SUC_lexicon_tags_15 },
    { 0x2a553e8e, SUC_lexicon_tags_15 },
    { 0xe4753e8d, SUC_lexicon_tags_42 },
    { 0x72b53e90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc953e92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40153e96, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7f953e98, SUC_lexicon_tags_19 },
    { 0xe4b53e99, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x33153e9b, SUC_lexicon_tags_25 },
    { 0x37753e9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd553e9e, SUC_lexicon_tags_13 },
    { 0x17b53e9f, SUC_lexicon_tags_13 },
    { 0xde153e9f, SUC_lexicon_tags_38 },
    { 0xe3953ea1, SUC_lexicon_tags_47 },
    { 0xb0153ea2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33553ea5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b53eab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af53eb7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc153ebc, SUC_lexicon_tags_44 },
    { 0xfbf53ebc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6b53ebf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31753ec4, SUC_lexicon_tags_1 },
    { 0xb7d53ec5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa2353ec7, SUC_lexicon_tags_26 },
    { 0x7c353ec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26353eca, SUC_lexicon_tags_17 },
    { 0xfa553eca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf353ece, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc2553ed0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb53ed3, SUC_lexicon_tags_60 },
    { 0xbcf53ed4, SUC_lexicon_tags_0 },
    { 0x68953ed5, SUC_lexicon_tags_15 },
    { 0xc2553ed6, SUC_lexicon_tags_11 },
    { 0x7fd53ed7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db53edd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5553ee5, SUC_lexicon_tags_38 },
    { 0xbd953ee6, SUC_lexicon_tags_63 },
    { 0x33b53ee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77553ee9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5c353eeb, SUC_lexicon_tags_47 },
    { 0x3db53eec, SUC_lexicon_tags_75 },
    { 0x62953eed, SUC_lexicon_tags_14 },
    { 0xad553eed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x11b53ef0, SUC_lexicon_tags_55 },
    { 0xd4f53ef1, SUC_lexicon_tags_37 },
    { 0xbef53ef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78f53ef4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a553ef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf53eff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d53f02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81553f04, SUC_lexicon_tags_25 },
    { 0xa6353f04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dd53f07, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec753f0b, SUC_lexicon_tags_63 },
    { 0xf1b53f0c, SUC_lexicon_tags_25 },
    { 0xef53f0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2753f0f, SUC_lexicon_tags_0 },
    { 0x95953f0f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5153f13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c153f16, SUC_lexicon_tags_1 },
    { 0x10153f16, SUC_lexicon_tags_194 },
    { 0x62953f17, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x75953f1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb553f1e, SUC_lexicon_tags_65 },
    { 0x35953f1f, SUC_lexicon_tags_75 },
    { 0x37f53f20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e953f26, SUC_lexicon_tags_11 },
    { 0x4df53f27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f753f29, SUC_lexicon_tags_25 },
    { 0x25353f2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5153f2e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c353f31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73153f33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b53f37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34553f39, SUC_lexicon_tags_37 },
    { 0x8dd53f3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4153f3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f153f46, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x91353f48, SUC_lexicon_tags_60 },
    { 0x3a753f48, SUC_lexicon_tags_49 },
    { 0xb353f4a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd0353f4c, SUC_lexicon_tags_17 },
    { 0x753f4c, SUC_lexicon_tags_37 },
    { 0x3353f4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81753f52, SUC_lexicon_tags_2 },
    { 0x8db53f53, SUC_lexicon_tags_7 },
    { 0x76753f54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a153f56, SUC_lexicon_tags_14 },
    { 0xbf353f56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e553f59, SUC_lexicon_tags_32 },
    { 0xe7553f5a, SUC_lexicon_tags_47 },
    { 0xe8953f59, SUC_lexicon_tags_23 },
    { 0xce753f5c, SUC_lexicon_tags_1 },
    { 0xb1b53f5d, SUC_lexicon_tags_11 },
    { 0xd0b53f5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d53f64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x62b53f66, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8d953f68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b53f70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b753f72, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0153f77, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1753f7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x58f53f7c, SUC_lexicon_tags_9 },
    { 0xec753f7d, SUC_lexicon_tags_2 },
    { 0x8b353f7e, SUC_lexicon_tags_27 },
    { 0x9d753f7f, SUC_lexicon_tags_48 },
    { 0xfb553f80, SUC_lexicon_tags_11 },
    { 0x24753f81, SUC_lexicon_tags_2 },
    { 0xa2353f82, SUC_lexicon_tags_7 },
    { 0x4bf53f83, SUC_lexicon_tags_30 },
    { 0x85553f80, SUC_lexicon_tags_11 },
    { 0x87353f85, SUC_lexicon_tags_80 },
    { 0x5953f7d, SUC_lexicon_tags_39 },
    { 0x9e553f87, SUC_lexicon_tags_11 },
    { 0x61353f7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf53f8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd53f8e, SUC_lexicon_tags_7 },
    { 0x36353f8f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb5d53f91, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x30553f93, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6f153f95, SUC_lexicon_tags_43 },
    { 0x83753f95, SUC_lexicon_tags_341 },
    { 0x31d53f97, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe4953f99, SUC_lexicon_tags_23 },
    { 0x79553f99, SUC_lexicon_tags_2 },
    { 0x7e153f9b, SUC_lexicon_tags_43 },
    { 0xbf953f99, SUC_lexicon_tags_45 },
    { 0xb0b53f9d, SUC_lexicon_tags_15 },
    { 0x9f753f99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x84553fa0, SUC_lexicon_tags_96 },
    { 0x87153fa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35b53fa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb553fa5, SUC_lexicon_tags_39 },
    { 0x92753fa5, SUC_lexicon_tags_1 },
    { 0xb6353fa7, SUC_lexicon_tags_17 },
    { 0xa6753fa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d53fab, SUC_lexicon_tags_7 },
    { 0x98553fac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b153fae, SUC_lexicon_tags_17 },
    { 0xf2353fae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9353fb3, SUC_lexicon_tags_14 },
    { 0xc5153fb4, SUC_lexicon_tags_15 },
    { 0x6ef53fb5, SUC_lexicon_tags_1 },
    { 0x74953fb5, SUC_lexicon_tags_15 },
    { 0x93b53fb5, SUC_lexicon_tags_16 },
    { 0x9953fb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b53fbb, SUC_lexicon_tags_66 },
    { 0x99553fbb, SUC_lexicon_tags_37 },
    { 0x91d53fbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf53fc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x44953fc3, SUC_lexicon_tags_33 },
    { 0x6b953fc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d53fca, SUC_lexicon_tags_68 },
    { 0x84553fcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d53fcf, SUC_lexicon_tags_11 },
    { 0x48353fd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3fb53fd2, SUC_lexicon_tags_0 },
    { 0x4153fd2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd153fd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce553fda, SUC_lexicon_tags_82 },
    { 0x3ab53fdb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f53fe3, SUC_lexicon_tags_25 },
    { 0x9153fe4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6ef53fe6, SUC_lexicon_tags_27 },
    { 0xc0353fe7, SUC_lexicon_tags_25 },
    { 0x7b753fe8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5d53fea, SUC_lexicon_tags_11 },
    { 0xae153fea, SUC_lexicon_tags_25 },
    { 0xa3153fea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f53fef, SUC_lexicon_tags_2 },
    { 0x2a753ff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb53ff3, SUC_lexicon_tags_19 },
    { 0x67753ff3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36553ffd, SUC_lexicon_tags_15 },
    { 0x353ffe, SUC_lexicon_tags_15 },
    { 0x11f53fff, SUC_lexicon_tags_959 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd54003, SUC_lexicon_tags_34 },
    { 0x67154004, SUC_lexicon_tags_33 },
    { 0x78754003, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb54008, SUC_lexicon_tags_10 },
    { 0x4e754009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5400c, SUC_lexicon_tags_1 },
    { 0x90f5400d, SUC_lexicon_tags_37 },
    { 0x5a95400e, SUC_lexicon_tags_11 },
    { 0x2115400e, SUC_lexicon_tags_463 },
    { 0x1d95400e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45954012, SUC_lexicon_tags_17 },
    { 0x6f954013, SUC_lexicon_tags_25 },
    { 0x3b754014, SUC_lexicon_tags_29 },
    { 0x59f54015, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96754019, SUC_lexicon_tags_17 },
    { 0xdd554019, SUC_lexicon_tags_1 },
    { 0xce95401b, SUC_lexicon_tags_400 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b95401e, SUC_lexicon_tags_29 },
    { 0xccf5401e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59f54021, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e354028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x875402a, SUC_lexicon_tags_1 },
    { 0xa5f5402b, SUC_lexicon_tags_1 },
    { 0x1c35402c, SUC_lexicon_tags_29 },
    { 0x1215402b, SUC_lexicon_tags_2 },
    { 0x2a35402e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b54033, SUC_lexicon_tags_452 },
    { 0x23f54033, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff54037, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e95403c, SUC_lexicon_tags_60 },
    { 0xf35403d, SUC_lexicon_tags_47 },
    { 0x3ad5403d, SUC_lexicon_tags_15 },
    { 0x29f5403f, SUC_lexicon_tags_19 },
    { 0xb55403c, SUC_lexicon_tags_12 },
    { 0xfcd5403e, SUC_lexicon_tags_60 },
    { 0xde554042, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x85f54044, SUC_lexicon_tags_11 },
    { 0x47554044, SUC_lexicon_tags_581 },
    { 0x12b54045, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d5404d, SUC_lexicon_tags_29 },
    { 0x55b5404e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c354050, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5154053, SUC_lexicon_tags_11 },
    { 0x62d54054, SUC_lexicon_tags_8 },
    { 0x24354054, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b95405e, SUC_lexicon_tags_17 },
    { 0x5435405f, SUC_lexicon_tags_17 },
    { 0x1eb5405e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d554062, SUC_lexicon_tags_23 },
    { 0x49554063, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e554065, SUC_lexicon_tags_38 },
    { 0xf1354065, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc4354068, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf735406d, SUC_lexicon_tags_16 },
    { 0xb075406d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f954071, SUC_lexicon_tags_47 },
    { 0xa4554071, SUC_lexicon_tags_11 },
    { 0x89754073, SUC_lexicon_tags_25 },
    { 0xc2d54073, SUC_lexicon_tags_25 },
    { 0xd2954075, SUC_lexicon_tags_56 },
    { 0x1754076, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1154078, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad5407b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xda75407d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd54082, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf554085, SUC_lexicon_tags_17 },
    { 0x63154086, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d554089, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3075408b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb615408d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa175408f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd554091, SUC_lexicon_tags_66 },
    { 0x71154091, SUC_lexicon_tags_14 },
    { 0x76354093, SUC_lexicon_tags_70 },
    { 0xa4554091, SUC_lexicon_tags_37 },
    { 0x19f54095, SUC_lexicon_tags_25 },
    { 0x4e554095, SUC_lexicon_tags_37 },
    { 0x37d54094, SUC_lexicon_tags_48 },
    { 0x39554095, SUC_lexicon_tags_8 },
    { 0x59354098, SUC_lexicon_tags_45 },
    { 0x7bf54092, SUC_lexicon_tags_21 },
    { 0x62354093, SUC_lexicon_tags_46 },
    { 0xca15409b, SUC_lexicon_tags_2 },
    { 0xafb54096, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd540a0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdd3540a2, SUC_lexicon_tags_35 },
    { 0x465540a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb83540a5, SUC_lexicon_tags_34 },
    { 0xc1540a6, SUC_lexicon_tags_1 },
    { 0x1eb540a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x903540aa, SUC_lexicon_tags_34 },
    { 0x9f3540ab, SUC_lexicon_tags_37 },
    { 0x377540aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b540b0, SUC_lexicon_tags_37 },
    { 0x6d540b1, SUC_lexicon_tags_49 },
    { 0xeb9540b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295540b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb540b9, SUC_lexicon_tags_25 },
    { 0xb0b540b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x581540bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad540c8, SUC_lexicon_tags_9 },
    { 0xc23540c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681540cf, SUC_lexicon_tags_2 },
    { 0x421540cf, SUC_lexicon_tags_43 },
    { 0x265540d1, SUC_lexicon_tags_31 },
    { 0xbbd540d2, SUC_lexicon_tags_15 },
    { 0xbd3540d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363540de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe1540e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd75540e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f540e5, SUC_lexicon_tags_4 },
    { 0x967540e6, SUC_lexicon_tags_37 },
    { 0x39540e7, SUC_lexicon_tags_16 },
    { 0x385540e5, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05540f0, SUC_lexicon_tags_27 },
    { 0xf79540f1, SUC_lexicon_tags_31 },
    { 0xd3b540f1, SUC_lexicon_tags_17 },
    { 0x86d540f3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1540f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc51540f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7540fe, SUC_lexicon_tags_37 },
    { 0x1c7540ff, SUC_lexicon_tags_25 },
    { 0x877540fe, SUC_lexicon_tags_7 },
    { 0x3c7540ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa754106, SUC_lexicon_tags_32 },
    { 0x20754107, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x16154109, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70354111, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb754114, SUC_lexicon_tags_6 },
    { 0xf6954115, SUC_lexicon_tags_32 },
    { 0x3cf54115, SUC_lexicon_tags_37 },
    { 0x14154117, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b5411e, SUC_lexicon_tags_17 },
    { 0x63b5411f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x51354121, SUC_lexicon_tags_49 },
    { 0x28154122, SUC_lexicon_tags_9 },
    { 0xdab54123, SUC_lexicon_tags_38 },
    { 0xf4954124, SUC_lexicon_tags_11 },
    { 0x8db54125, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d54128, SUC_lexicon_tags_37 },
    { 0x2fd54129, SUC_lexicon_tags_0 },
    { 0x2c95412a, SUC_lexicon_tags_79 },
    { 0xf7154128, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9135412d, SUC_lexicon_tags_6 },
    { 0xca95412e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8354133, SUC_lexicon_tags_25 },
    { 0x5df54134, SUC_lexicon_tags_19 },
    { 0x8c954135, SUC_lexicon_tags_17 },
    { 0x10354136, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87f54138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c75413d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f54142, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b54148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e95414b, SUC_lexicon_tags_12 },
    { 0x5e55414c, SUC_lexicon_tags_84 },
    { 0x6715414c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb554150, SUC_lexicon_tags_14 },
    { 0x63f54151, SUC_lexicon_tags_14 },
    { 0x9a154152, SUC_lexicon_tags_1 },
    { 0x54f54153, SUC_lexicon_tags_1 },
    { 0xf2354154, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8754158, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb5415b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb354163, SUC_lexicon_tags_9 },
    { 0xa0754164, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d54167, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f5416a, SUC_lexicon_tags_37 },
    { 0x7ad5416a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b95416e, SUC_lexicon_tags_45 },
    { 0xc015416f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x98954171, SUC_lexicon_tags_60 },
    { 0x29354171, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71554174, SUC_lexicon_tags_27 },
    { 0x7c954175, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x32754178, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb55417b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b5417f, SUC_lexicon_tags_12 },
    { 0x94554180, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7154185, SUC_lexicon_tags_1 },
    { 0xe0d54185, SUC_lexicon_tags_536 },
    { 0x55554187, SUC_lexicon_tags_2 },
    { 0x4b354188, SUC_lexicon_tags_28 },
    { 0x28954189, SUC_lexicon_tags_1 },
    { 0x4e954187, SUC_lexicon_tags_7 },
    { 0x6b35418b, SUC_lexicon_tags_7 },
    { 0xa355418b, SUC_lexicon_tags_27 },
    { 0x1895418d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1855418f, SUC_lexicon_tags_15 },
    { 0xe4154190, SUC_lexicon_tags_12 },
    { 0x7f354191, SUC_lexicon_tags_17 },
    { 0x28554191, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0354194, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64154199, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d55419d, SUC_lexicon_tags_2 },
    { 0xc9d5419e, SUC_lexicon_tags_21 },
    { 0x9115419f, SUC_lexicon_tags_27 },
    { 0x405541a0, SUC_lexicon_tags_7 },
    { 0xe93541a1, SUC_lexicon_tags_17 },
    { 0x91541a1, SUC_lexicon_tags_27 },
    { 0x9fb541a3, SUC_lexicon_tags_11 },
    { 0x849541a4, SUC_lexicon_tags_10 },
    { 0x62b541a1, SUC_lexicon_tags_47 },
    { 0x605541a6, SUC_lexicon_tags_49 },
    { 0xb87541a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3541ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea3541ad, SUC_lexicon_tags_9 },
    { 0x245541ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9541b6, SUC_lexicon_tags_38 },
    { 0xbb1541b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3d1541b9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff541bc, SUC_lexicon_tags_1 },
    { 0xeb541bd, SUC_lexicon_tags_17 },
    { 0xd2b541be, SUC_lexicon_tags_17 },
    { 0xd6b541bf, SUC_lexicon_tags_1 },
    { 0x247541bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3541c8, SUC_lexicon_tags_12 },
    { 0x669541c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f541cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe4d541cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1541d0, SUC_lexicon_tags_37 },
    { 0x9f3541d1, SUC_lexicon_tags_17 },
    { 0x3b7541d2, SUC_lexicon_tags_37 },
    { 0x48d541d3, SUC_lexicon_tags_37 },
    { 0x48d541d3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3541df, SUC_lexicon_tags_15 },
    { 0xf1541e0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605541e4, SUC_lexicon_tags_63 },
    { 0xc69541e5, SUC_lexicon_tags_57 },
    { 0xf9f541e6, SUC_lexicon_tags_2 },
    { 0xd5b541e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59d541e9, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d541ef, SUC_lexicon_tags_66 },
    { 0x68d541f0, SUC_lexicon_tags_1 },
    { 0x183541f0, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d541f4, SUC_lexicon_tags_57 },
    { 0x535541f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ef541f7, SUC_lexicon_tags_92 },
    { 0xc9d541f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1d541fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d541fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb54201, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf3554203, SUC_lexicon_tags_4 },
    { 0x60d54204, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5b554206, SUC_lexicon_tags_33 },
    { 0x7f154206, SUC_lexicon_tags_17 },
    { 0x47d54208, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x915420b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1335420e, SUC_lexicon_tags_2 },
    { 0x7215420f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee954216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72554218, SUC_lexicon_tags_2 },
    { 0xf5154219, SUC_lexicon_tags_37 },
    { 0x84b5421a, SUC_lexicon_tags_37 },
    { 0xd9b5421a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9275421d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd54220, SUC_lexicon_tags_77 },
    { 0x9754221, SUC_lexicon_tags_2 },
    { 0xd4d54220, SUC_lexicon_tags_12 },
    { 0xa9b54223, SUC_lexicon_tags_27 },
    { 0x75754224, SUC_lexicon_tags_11 },
    { 0xbc354221, SUC_lexicon_tags_17 },
    { 0xacf54224, SUC_lexicon_tags_0 },
    { 0xee354227, SUC_lexicon_tags_4 },
    { 0x8e154226, SUC_lexicon_tags_43 },
    { 0xf2b54223, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7415422d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4954233, SUC_lexicon_tags_1 },
    { 0xd0f54234, SUC_lexicon_tags_84 },
    { 0xb0754235, SUC_lexicon_tags_15 },
    { 0x6d754236, SUC_lexicon_tags_15 },
    { 0x27f54237, SUC_lexicon_tags_14 },
    { 0x72554233, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5195423c, SUC_lexicon_tags_2 },
    { 0x55423d, SUC_lexicon_tags_83 },
    { 0x855423e, SUC_lexicon_tags_12 },
    { 0x9d35423f, SUC_lexicon_tags_10 },
    { 0x3df5423f, SUC_lexicon_tags_1 },
    { 0x20754241, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f754245, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf5424c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5835424f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5754252, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf154254, SUC_lexicon_tags_37 },
    { 0xb6d54255, SUC_lexicon_tags_4 },
    { 0xc5d54256, SUC_lexicon_tags_7 },
    { 0x7f754256, SUC_lexicon_tags_15 },
    { 0xa0154258, SUC_lexicon_tags_6 },
    { 0xe754258, SUC_lexicon_tags_37 },
    { 0xb0954256, SUC_lexicon_tags_12 },
    { 0x9ff5425b, SUC_lexicon_tags_69 },
    { 0xa1b5425c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8295425e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f154264, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x12154267, SUC_lexicon_tags_15 },
    { 0xa7954267, SUC_lexicon_tags_32 },
    { 0x3eb54269, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c75426d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b54270, SUC_lexicon_tags_28 },
    { 0xf9354271, SUC_lexicon_tags_14 },
    { 0x94b54272, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb54275, SUC_lexicon_tags_31 },
    { 0x1e554276, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d54279, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf5427e, SUC_lexicon_tags_38 },
    { 0x6015427f, SUC_lexicon_tags_6 },
    { 0x51f54280, SUC_lexicon_tags_33 },
    { 0x41354281, SUC_lexicon_tags_14 },
    { 0x46954280, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f554287, SUC_lexicon_tags_47 },
    { 0x28b54287, SUC_lexicon_tags_61 },
    { 0xdbd54289, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf754290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x26154292, SUC_lexicon_tags_17 },
    { 0x4c554293, SUC_lexicon_tags_15 },
    { 0x2e954294, SUC_lexicon_tags_66 },
    { 0x3e154295, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d5429b, SUC_lexicon_tags_42 },
    { 0xf8b5429c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3315429f, SUC_lexicon_tags_7 },
    { 0x109542a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x501542a3, SUC_lexicon_tags_34 },
    { 0xe7b542a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b542ab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x959542ad, SUC_lexicon_tags_12 },
    { 0x299542ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d542b2, SUC_lexicon_tags_15 },
    { 0x8f7542b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x799542b5, SUC_lexicon_tags_15 },
    { 0xab7542b6, SUC_lexicon_tags_60 },
    { 0x9d542b6, SUC_lexicon_tags_7 },
    { 0xd7f542b8, SUC_lexicon_tags_45 },
    { 0x4dd542b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665542be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7542c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9542c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7542cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1df542ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1542d5, SUC_lexicon_tags_60 },
    { 0x603542d6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573542e1, SUC_lexicon_tags_7 },
    { 0x5f542e1, SUC_lexicon_tags_37 },
    { 0xfa7542e3, SUC_lexicon_tags_138 },
    { 0xc43542e3, SUC_lexicon_tags_86 },
    { 0x59542e5, SUC_lexicon_tags_17 },
    { 0x5b542e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f542eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d542ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x977542f1, SUC_lexicon_tags_1 },
    { 0xced542f2, SUC_lexicon_tags_12 },
    { 0x2af542f3, SUC_lexicon_tags_12 },
    { 0x881542f3, SUC_lexicon_tags_47 },
    { 0x523542f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5542f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e754300, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x90b54302, SUC_lexicon_tags_63 },
    { 0x47b54303, SUC_lexicon_tags_14 },
    { 0x3c954303, SUC_lexicon_tags_83 },
    { 0x92f54305, SUC_lexicon_tags_7 },
    { 0xef754305, SUC_lexicon_tags_17 },
    { 0x4d954307, SUC_lexicon_tags_61 },
    { 0x8f554305, SUC_lexicon_tags_18 },
    { 0x52954305, SUC_lexicon_tags_11 },
    { 0x5e954307, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f54314, SUC_lexicon_tags_7 },
    { 0x13554315, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92754317, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4095431a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d54321, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd8354323, SUC_lexicon_tags_1 },
    { 0x2d354323, SUC_lexicon_tags_14 },
    { 0x4d554325, SUC_lexicon_tags_92 },
    { 0xcfb54326, SUC_lexicon_tags_2 },
    { 0x39154323, SUC_lexicon_tags_47 },
    { 0xbb154328, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d5432b, SUC_lexicon_tags_25 },
    { 0x6795432b, SUC_lexicon_tags_63 },
    { 0xb1f5432c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf754330, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d54334, SUC_lexicon_tags_7 },
    { 0xe1354334, SUC_lexicon_tags_37 },
    { 0xf4154336, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f5433d, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xf5f5433f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb755434a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1975434c, SUC_lexicon_tags_19 },
    { 0xe4d5434d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4754354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5dd54356, SUC_lexicon_tags_63 },
    { 0x12954357, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac154359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56f5435b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd275435d, SUC_lexicon_tags_37 },
    { 0x6555435e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5754361, SUC_lexicon_tags_30 },
    { 0x94354362, SUC_lexicon_tags_37 },
    { 0x2cb54362, SUC_lexicon_tags_27 },
    { 0xd7154362, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b54367, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5595436b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4cd5436d, SUC_lexicon_tags_1 },
    { 0x9435436d, SUC_lexicon_tags_7 },
    { 0xdc95436f, SUC_lexicon_tags_15 },
    { 0x61154370, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575437d, SUC_lexicon_tags_17 },
    { 0x29b5437e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f954385, SUC_lexicon_tags_28 },
    { 0xecf54385, SUC_lexicon_tags_536 },
    { 0xcc354387, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9515438a, SUC_lexicon_tags_30 },
    { 0xaf55438b, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75438e, SUC_lexicon_tags_0 },
    { 0x1115438f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda554393, SUC_lexicon_tags_60 },
    { 0x4d54394, SUC_lexicon_tags_17 },
    { 0xc4754395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac554397, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83f54399, SUC_lexicon_tags_25 },
    { 0x6bf54399, SUC_lexicon_tags_62 },
    { 0x6c15439a, SUC_lexicon_tags_10 },
    { 0x5f75439c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d5439f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53543a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf543aa, SUC_lexicon_tags_151 },
    { 0xb33543ab, SUC_lexicon_tags_23 },
    { 0xe77543ac, SUC_lexicon_tags_613 },
    { 0xaf9543ac, SUC_lexicon_tags_68 },
    { 0xd83543ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7543b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07543b5, SUC_lexicon_tags_47 },
    { 0xcfb543b6, SUC_lexicon_tags_47 },
    { 0x3e5543b7, SUC_lexicon_tags_7 },
    { 0xe2f543b8, SUC_lexicon_tags_25 },
    { 0xcff543b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x207543bb, SUC_lexicon_tags_25 },
    { 0x65b543bc, SUC_lexicon_tags_28 },
    { 0x477543bd, SUC_lexicon_tags_7 },
    { 0xc47543be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca5543c0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb543c3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627543c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2f543c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf543cf, SUC_lexicon_tags_11 },
    { 0x6b3543d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1543d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x733543d6, SUC_lexicon_tags_135 },
    { 0, NULL },
    { 0x233543d8, SUC_lexicon_tags_25 },
    { 0x951543d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47543de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59543e5, SUC_lexicon_tags_12 },
    { 0x1a9543e6, SUC_lexicon_tags_19 },
    { 0x477543e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5543eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef543ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b7543f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d543f3, SUC_lexicon_tags_35 },
    { 0x7b9543f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x94b543f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9543fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7f543ff, SUC_lexicon_tags_68 },
    { 0x40b543ff, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x87954403, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30154408, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b5441d, SUC_lexicon_tags_26 },
    { 0x2c55441d, SUC_lexicon_tags_19 },
    { 0xbdf5441f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda354421, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6bd54423, SUC_lexicon_tags_33 },
    { 0x24f54424, SUC_lexicon_tags_11 },
    { 0xebf54424, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f54428, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe4b5442a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3855442c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5895442f, SUC_lexicon_tags_43 },
    { 0x12b5442f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76554434, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x84554437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa755443b, SUC_lexicon_tags_57 },
    { 0x7ff5443c, SUC_lexicon_tags_38 },
    { 0x7875443c, SUC_lexicon_tags_27 },
    { 0xb735443e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f54445, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d54448, SUC_lexicon_tags_12 },
    { 0xfa754449, SUC_lexicon_tags_19 },
    { 0x20d5444a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e55444f, SUC_lexicon_tags_79 },
    { 0x25554450, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f154452, SUC_lexicon_tags_53 },
    { 0x63354453, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x73d54455, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b554458, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d54461, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcab54463, SUC_lexicon_tags_7 },
    { 0x82f54464, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89954466, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a95446a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f5446d, SUC_lexicon_tags_53 },
    { 0x3b55446d, SUC_lexicon_tags_14 },
    { 0x40f5446f, SUC_lexicon_tags_38 },
    { 0xa6354470, SUC_lexicon_tags_1 },
    { 0xfe154471, SUC_lexicon_tags_46 },
    { 0x96b5446d, SUC_lexicon_tags_38 },
    { 0x8ff54473, SUC_lexicon_tags_45 },
    { 0xd1954474, SUC_lexicon_tags_15 },
    { 0xae554474, SUC_lexicon_tags_11 },
    { 0x3fb54476, SUC_lexicon_tags_2 },
    { 0x24554471, SUC_lexicon_tags_47 },
    { 0xe2b54471, SUC_lexicon_tags_84 },
    { 0x2b35446d, SUC_lexicon_tags_4 },
    { 0x8f55447a, SUC_lexicon_tags_1 },
    { 0x4975447a, SUC_lexicon_tags_37 },
    { 0xc435446d, SUC_lexicon_tags_29 },
    { 0x1775447d, SUC_lexicon_tags_63 },
    { 0x2895446d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed154481, SUC_lexicon_tags_4 },
    { 0xc6754482, SUC_lexicon_tags_15 },
    { 0x19754482, SUC_lexicon_tags_25 },
    { 0xc5b54484, SUC_lexicon_tags_2 },
    { 0x48354484, SUC_lexicon_tags_46 },
    { 0xbb354486, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x71554489, SUC_lexicon_tags_15 },
    { 0x81f5448a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b5448d, SUC_lexicon_tags_4 },
    { 0xe395448d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33954490, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e754493, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1f154495, SUC_lexicon_tags_1 },
    { 0x9754496, SUC_lexicon_tags_12 },
    { 0x10f54497, SUC_lexicon_tags_45 },
    { 0xa3754497, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15449d, SUC_lexicon_tags_48 },
    { 0xd315449e, SUC_lexicon_tags_1 },
    { 0x5e95449f, SUC_lexicon_tags_37 },
    { 0x3595449e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x25f544a2, SUC_lexicon_tags_24 },
    { 0x4e7544a2, SUC_lexicon_tags_17 },
    { 0xddf544a4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x827544a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e1544a9, SUC_lexicon_tags_0 },
    { 0x751544aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81544af, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01544b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135544b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb544ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7544bd, SUC_lexicon_tags_27 },
    { 0x7544be, SUC_lexicon_tags_7 },
    { 0x85b544bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb7b544c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x631544c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x153544c5, SUC_lexicon_tags_46 },
    { 0xaf5544c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf544cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x513544ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b544d1, SUC_lexicon_tags_37 },
    { 0xa9b544d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b544d9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363544de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x299544e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ed544e2, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x63f544e4, SUC_lexicon_tags_0 },
    { 0xa1d544e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5544e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf09544ea, SUC_lexicon_tags_6 },
    { 0x7ff544eb, SUC_lexicon_tags_12 },
    { 0x9d9544eb, SUC_lexicon_tags_62 },
    { 0xecd544ec, SUC_lexicon_tags_37 },
    { 0x7cf544ec, SUC_lexicon_tags_65 },
    { 0xda9544eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x655544f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74b544f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x661544f7, SUC_lexicon_tags_46 },
    { 0x4fb544f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ff544fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf544fd, SUC_lexicon_tags_25 },
    { 0xc8d544fd, SUC_lexicon_tags_26 },
    { 0x6d3544fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8554504, SUC_lexicon_tags_47 },
    { 0x7c554505, SUC_lexicon_tags_65 },
    { 0x1c954504, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30954508, SUC_lexicon_tags_49 },
    { 0x9b154508, SUC_lexicon_tags_25 },
    { 0xaef54508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31b5450c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa75450e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a154510, SUC_lexicon_tags_13 },
    { 0xb8154511, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd54514, SUC_lexicon_tags_17 },
    { 0x20754515, SUC_lexicon_tags_8 },
    { 0xcfd54516, SUC_lexicon_tags_15 },
    { 0x42954517, SUC_lexicon_tags_9 },
    { 0x6d54518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83b5451a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8035451f, SUC_lexicon_tags_11 },
    { 0x71d54520, SUC_lexicon_tags_9 },
    { 0xc4d54521, SUC_lexicon_tags_39 },
    { 0x18154522, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a754524, SUC_lexicon_tags_68 },
    { 0x7f954525, SUC_lexicon_tags_8 },
    { 0xdfb54526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8b54528, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb2d5452a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe195452d, SUC_lexicon_tags_15 },
    { 0x9675452d, SUC_lexicon_tags_7 },
    { 0x77d5452f, SUC_lexicon_tags_17 },
    { 0xfef54530, SUC_lexicon_tags_1 },
    { 0x1354530, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f54538, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4475453c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf8b5453e, SUC_lexicon_tags_14 },
    { 0xa195453f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92154541, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42d54543, SUC_lexicon_tags_19 },
    { 0x6db54543, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb8b54546, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8315454a, SUC_lexicon_tags_0 },
    { 0xf3d5454b, SUC_lexicon_tags_12 },
    { 0x6db5454c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x63b5454e, SUC_lexicon_tags_25 },
    { 0x6295454e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc4b54551, SUC_lexicon_tags_43 },
    { 0x25554552, SUC_lexicon_tags_25 },
    { 0x99f54553, SUC_lexicon_tags_29 },
    { 0x32554552, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x38b54556, SUC_lexicon_tags_0 },
    { 0x8c754557, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc354559, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9ef5455b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95455f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f954562, SUC_lexicon_tags_55 },
    { 0xb4d54563, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10954568, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c15456a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5456e, SUC_lexicon_tags_15 },
    { 0x5b95456f, SUC_lexicon_tags_7 },
    { 0xb9154570, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd54574, SUC_lexicon_tags_19 },
    { 0xa7154575, SUC_lexicon_tags_55 },
    { 0x65554574, SUC_lexicon_tags_4 },
    { 0x40354577, SUC_lexicon_tags_43 },
    { 0xb0b54575, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f95457b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x435457d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb154581, SUC_lexicon_tags_37 },
    { 0x26f54581, SUC_lexicon_tags_27 },
    { 0x18354582, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99954585, SUC_lexicon_tags_32 },
    { 0x97554586, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd5458c, SUC_lexicon_tags_26 },
    { 0x62f5458c, SUC_lexicon_tags_25 },
    { 0xd1f5458e, SUC_lexicon_tags_14 },
    { 0x1955458d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31754592, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2154597, SUC_lexicon_tags_1 },
    { 0xa4154598, SUC_lexicon_tags_45 },
    { 0xbd54599, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6875459c, SUC_lexicon_tags_79 },
    { 0x3155459c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf475459f, SUC_lexicon_tags_12 },
    { 0x8295459f, SUC_lexicon_tags_14 },
    { 0x1e5545a1, SUC_lexicon_tags_37 },
    { 0x7545a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x411545a4, SUC_lexicon_tags_12 },
    { 0xb99545a5, SUC_lexicon_tags_63 },
    { 0x7cf545a5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9545ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f7545b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d545b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde3545b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x667545b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d545c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d545c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529545cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26d545ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97b545d0, SUC_lexicon_tags_32 },
    { 0x32b545d1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d545d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15545d9, SUC_lexicon_tags_24 },
    { 0x979545da, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675545e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1545e4, SUC_lexicon_tags_26 },
    { 0x38d545e5, SUC_lexicon_tags_28 },
    { 0x405545e6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6b545e8, SUC_lexicon_tags_113 },
    { 0xd91545e9, SUC_lexicon_tags_47 },
    { 0x1d545ea, SUC_lexicon_tags_17 },
    { 0xfb5545ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf545ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d545f5, SUC_lexicon_tags_15 },
    { 0x5a3545f6, SUC_lexicon_tags_11 },
    { 0xcbf545f7, SUC_lexicon_tags_10 },
    { 0x6dd545f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39545fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x455545fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43b54600, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4b54602, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b54607, SUC_lexicon_tags_14 },
    { 0x42354607, SUC_lexicon_tags_37 },
    { 0xb6154609, SUC_lexicon_tags_15 },
    { 0x8e354607, SUC_lexicon_tags_17 },
    { 0xb9b5460b, SUC_lexicon_tags_19 },
    { 0x7695460a, SUC_lexicon_tags_0 },
    { 0x2675460d, SUC_lexicon_tags_17 },
    { 0xf815460d, SUC_lexicon_tags_56 },
    { 0x8215460f, SUC_lexicon_tags_49 },
    { 0x3675460d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfb54612, SUC_lexicon_tags_7 },
    { 0x69d54612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e554616, SUC_lexicon_tags_15 },
    { 0x58f54617, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd154619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d754622, SUC_lexicon_tags_25 },
    { 0x23f54623, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45754627, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x52154629, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6335462b, SUC_lexicon_tags_29 },
    { 0x2bd5462c, SUC_lexicon_tags_27 },
    { 0xd955462d, SUC_lexicon_tags_0 },
    { 0x4075462e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x95154631, SUC_lexicon_tags_28 },
    { 0x98754632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f754634, SUC_lexicon_tags_3 },
    { 0xbe754634, SUC_lexicon_tags_47 },
    { 0x37b54636, SUC_lexicon_tags_270 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2895463c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b75463e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f54641, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0b54643, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8375464c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b54650, SUC_lexicon_tags_10 },
    { 0x85754651, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56754656, SUC_lexicon_tags_32 },
    { 0xc1b54657, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9855465b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a154660, SUC_lexicon_tags_0 },
    { 0x14554660, SUC_lexicon_tags_12 },
    { 0x27b54661, SUC_lexicon_tags_11 },
    { 0x69754661, SUC_lexicon_tags_1 },
    { 0x54754664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30754666, SUC_lexicon_tags_39 },
    { 0x9f154667, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8035466e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b54672, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9554676, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b54679, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7895467b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb5467e, SUC_lexicon_tags_15 },
    { 0xe6b5467f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x98554682, SUC_lexicon_tags_27 },
    { 0xc7954682, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7fb54685, SUC_lexicon_tags_12 },
    { 0x82354686, SUC_lexicon_tags_68 },
    { 0x2bf54687, SUC_lexicon_tags_37 },
    { 0xd7354686, SUC_lexicon_tags_2 },
    { 0x38b54688, SUC_lexicon_tags_15 },
    { 0x39f54689, SUC_lexicon_tags_12 },
    { 0xf9754686, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x37f5468d, SUC_lexicon_tags_15 },
    { 0x7435468e, SUC_lexicon_tags_25 },
    { 0xfe55468e, SUC_lexicon_tags_13 },
    { 0x85b5468e, SUC_lexicon_tags_15 },
    { 0x3b354691, SUC_lexicon_tags_65 },
    { 0xa4954692, SUC_lexicon_tags_37 },
    { 0xc535468f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf9754695, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f5469a, SUC_lexicon_tags_37 },
    { 0x3155469b, SUC_lexicon_tags_25 },
    { 0xa955469c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xceb5469e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef546a1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x321546a4, SUC_lexicon_tags_11 },
    { 0xe5f546a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a7546a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x879546a9, SUC_lexicon_tags_4 },
    { 0xdaf546aa, SUC_lexicon_tags_37 },
    { 0x905546ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f546ae, SUC_lexicon_tags_14 },
    { 0xc71546af, SUC_lexicon_tags_82 },
    { 0x15d546b0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977546b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9546b9, SUC_lexicon_tags_2 },
    { 0x699546ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb2d546bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69546bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd546c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d546c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0d546c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61546cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f546ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19546d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d546d5, SUC_lexicon_tags_65 },
    { 0x431546d6, SUC_lexicon_tags_12 },
    { 0x85f546d6, SUC_lexicon_tags_7 },
    { 0x9f546d5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8ed546da, SUC_lexicon_tags_17 },
    { 0xca9546db, SUC_lexicon_tags_28 },
    { 0x39546db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3546e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb546e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5546ea, SUC_lexicon_tags_4 },
    { 0x90f546eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7546ef, SUC_lexicon_tags_25 },
    { 0x355546ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b546f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb546f8, SUC_lexicon_tags_2 },
    { 0x233546f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db54701, SUC_lexicon_tags_11 },
    { 0xa8554701, SUC_lexicon_tags_21 },
    { 0xf1954703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59154706, SUC_lexicon_tags_12 },
    { 0xde954707, SUC_lexicon_tags_14 },
    { 0x7cd54706, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5470f, SUC_lexicon_tags_1 },
    { 0x31d5470f, SUC_lexicon_tags_13 },
    { 0xb5754711, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5471b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a55471d, SUC_lexicon_tags_37 },
    { 0x9b35471e, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b54721, SUC_lexicon_tags_175 },
    { 0xb5954722, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c754725, SUC_lexicon_tags_12 },
    { 0x98154725, SUC_lexicon_tags_17 },
    { 0xc6954727, SUC_lexicon_tags_38 },
    { 0x33b54726, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f5472c, SUC_lexicon_tags_2 },
    { 0x3355472d, SUC_lexicon_tags_25 },
    { 0xa935472e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda154730, SUC_lexicon_tags_10 },
    { 0x28754731, SUC_lexicon_tags_25 },
    { 0xd5954732, SUC_lexicon_tags_34 },
    { 0x2354733, SUC_lexicon_tags_25 },
    { 0xb54731, SUC_lexicon_tags_1 },
    { 0xadf54735, SUC_lexicon_tags_32 },
    { 0xacb54735, SUC_lexicon_tags_12 },
    { 0x1eb54737, SUC_lexicon_tags_7 },
    { 0x72d54735, SUC_lexicon_tags_13 },
    { 0x29b54734, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b35473b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3935473f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef54745, SUC_lexicon_tags_21 },
    { 0xceb54745, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f75474c, SUC_lexicon_tags_37 },
    { 0x74d5474c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x19b5474f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae954755, SUC_lexicon_tags_37 },
    { 0x6a354756, SUC_lexicon_tags_15 },
    { 0xe7554757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1715475a, SUC_lexicon_tags_25 },
    { 0xb335475b, SUC_lexicon_tags_68 },
    { 0xc695475c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc1b5475e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0754760, SUC_lexicon_tags_15 },
    { 0x9c554761, SUC_lexicon_tags_12 },
    { 0x51354762, SUC_lexicon_tags_11 },
    { 0x49154760, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d54769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5455476b, SUC_lexicon_tags_17 },
    { 0x7915476c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f5476f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8d54771, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1354773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94354775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8154779, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8975477c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d35477f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5755478a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc0b5478c, SUC_lexicon_tags_37 },
    { 0x495478c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38554793, SUC_lexicon_tags_0 },
    { 0xa5f54794, SUC_lexicon_tags_37 },
    { 0x24f54793, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb215479a, SUC_lexicon_tags_11 },
    { 0x71b5479b, SUC_lexicon_tags_17 },
    { 0x8135479b, SUC_lexicon_tags_11 },
    { 0x5215479c, SUC_lexicon_tags_11 },
    { 0x3e15479c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57547a1, SUC_lexicon_tags_12 },
    { 0xd23547a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7547a5, SUC_lexicon_tags_9 },
    { 0x105547a6, SUC_lexicon_tags_55 },
    { 0xef5547a7, SUC_lexicon_tags_619 },
    { 0x17b547a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7547ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3547af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433547b3, SUC_lexicon_tags_16 },
    { 0x675547b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4f9547b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf83547b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75f547ba, SUC_lexicon_tags_10 },
    { 0xeb5547bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07547c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d547c4, SUC_lexicon_tags_2 },
    { 0x9e9547c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x693547c7, SUC_lexicon_tags_2 },
    { 0x371547c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd55547ca, SUC_lexicon_tags_37 },
    { 0xff3547cb, SUC_lexicon_tags_15 },
    { 0x77f547cb, SUC_lexicon_tags_29 },
    { 0x7eb547cd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb547d0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27547da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d547dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b547e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4b547e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df547e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb1547ea, SUC_lexicon_tags_27 },
    { 0x8f1547eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d547ee, SUC_lexicon_tags_25 },
    { 0xdb1547ef, SUC_lexicon_tags_79 },
    { 0x1d1547ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1547f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb99547f5, SUC_lexicon_tags_17 },
    { 0x31d547f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x60b547f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85547fc, SUC_lexicon_tags_1 },
    { 0x551547fc, SUC_lexicon_tags_10 },
    { 0xb55547fd, SUC_lexicon_tags_19 },
    { 0x18f547fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4554801, SUC_lexicon_tags_31 },
    { 0xd6554802, SUC_lexicon_tags_47 },
    { 0x73154801, SUC_lexicon_tags_37 },
    { 0x8d954803, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xddf54806, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b5480e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x62554810, SUC_lexicon_tags_60 },
    { 0xb9354811, SUC_lexicon_tags_47 },
    { 0x26954812, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6354816, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb154820, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf354822, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40154824, SUC_lexicon_tags_30 },
    { 0x4f554824, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced54829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5835482d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d54830, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b54833, SUC_lexicon_tags_7 },
    { 0x8cf54833, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f54837, SUC_lexicon_tags_324 },
    { 0x5d354838, SUC_lexicon_tags_23 },
    { 0x99b54838, SUC_lexicon_tags_37 },
    { 0x3dd5483a, SUC_lexicon_tags_45 },
    { 0x6a35483b, SUC_lexicon_tags_11 },
    { 0x7395483b, SUC_lexicon_tags_63 },
    { 0x5d75483a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18354845, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2354849, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb835484b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4554850, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c754852, SUC_lexicon_tags_25 },
    { 0x70754853, SUC_lexicon_tags_29 },
    { 0x7ab54853, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99754858, SUC_lexicon_tags_12 },
    { 0x7b354859, SUC_lexicon_tags_27 },
    { 0x6e15485a, SUC_lexicon_tags_19 },
    { 0xf175485b, SUC_lexicon_tags_25 },
    { 0xac554858, SUC_lexicon_tags_23 },
    { 0x43954859, SUC_lexicon_tags_17 },
    { 0x2a15485a, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xfb154860, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb354864, SUC_lexicon_tags_0 },
    { 0x4a554864, SUC_lexicon_tags_16 },
    { 0x31f54865, SUC_lexicon_tags_11 },
    { 0xd9754867, SUC_lexicon_tags_11 },
    { 0x1d954865, SUC_lexicon_tags_0 },
    { 0xc9554869, SUC_lexicon_tags_30 },
    { 0x8a554869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a75486f, SUC_lexicon_tags_37 },
    { 0x28954870, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d754873, SUC_lexicon_tags_27 },
    { 0x42354874, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x43154877, SUC_lexicon_tags_12 },
    { 0xfe354878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc735487c, SUC_lexicon_tags_23 },
    { 0xdcb5487c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1415487f, SUC_lexicon_tags_37 },
    { 0xf0f54880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37754882, SUC_lexicon_tags_14 },
    { 0x2f154883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97154885, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebb54887, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa875488a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb5488f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b54892, SUC_lexicon_tags_2 },
    { 0xe7154893, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f54895, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x23154898, SUC_lexicon_tags_17 },
    { 0x4eb54899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef5489e, SUC_lexicon_tags_15 },
    { 0xb15489f, SUC_lexicon_tags_37 },
    { 0xef95489f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57b548a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615548a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25548ac, SUC_lexicon_tags_3 },
    { 0xea7548ad, SUC_lexicon_tags_23 },
    { 0x555548ae, SUC_lexicon_tags_3 },
    { 0xf0f548af, SUC_lexicon_tags_11 },
    { 0xd09548b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a9548b2, SUC_lexicon_tags_37 },
    { 0x20d548b2, SUC_lexicon_tags_55 },
    { 0x7b5548b4, SUC_lexicon_tags_37 },
    { 0xf0b548b2, SUC_lexicon_tags_33 },
    { 0xae7548b6, SUC_lexicon_tags_1 },
    { 0x4c1548b6, SUC_lexicon_tags_37 },
    { 0x73548b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4af548ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x893548bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bd548be, SUC_lexicon_tags_7 },
    { 0x6bd548bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe3548c1, SUC_lexicon_tags_14 },
    { 0x63548c2, SUC_lexicon_tags_11 },
    { 0xc8d548c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913548c9, SUC_lexicon_tags_23 },
    { 0x2fb548ca, SUC_lexicon_tags_2 },
    { 0xbbb548cb, SUC_lexicon_tags_87 },
    { 0x4eb548cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3548d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3bd548d2, SUC_lexicon_tags_14 },
    { 0xc3b548d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3548d9, SUC_lexicon_tags_15 },
    { 0xe5d548da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d548dd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5548e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xead548e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9548e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181548eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x557548ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8f548ef, SUC_lexicon_tags_32 },
    { 0x7f7548f0, SUC_lexicon_tags_2 },
    { 0xb35548f1, SUC_lexicon_tags_19 },
    { 0xa5f548f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x919548f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707548f8, SUC_lexicon_tags_4 },
    { 0xbb5548f9, SUC_lexicon_tags_1 },
    { 0xc11548f9, SUC_lexicon_tags_33 },
    { 0xa23548f8, SUC_lexicon_tags_49 },
    { 0x113548fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed548ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd5490a, SUC_lexicon_tags_15 },
    { 0xaa35490a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab54910, SUC_lexicon_tags_17 },
    { 0x71954911, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82754917, SUC_lexicon_tags_8 },
    { 0x48954918, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b5491c, SUC_lexicon_tags_15 },
    { 0x4035491d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4954920, SUC_lexicon_tags_12 },
    { 0x87b54920, SUC_lexicon_tags_63 },
    { 0x38f54920, SUC_lexicon_tags_38 },
    { 0x64154922, SUC_lexicon_tags_43 },
    { 0xd4954920, SUC_lexicon_tags_23 },
    { 0x70d54923, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf354927, SUC_lexicon_tags_12 },
    { 0xeeb54927, SUC_lexicon_tags_8 },
    { 0x80b54929, SUC_lexicon_tags_2 },
    { 0x8f554929, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5135492e, SUC_lexicon_tags_3 },
    { 0x5f35492f, SUC_lexicon_tags_15 },
    { 0x72b54930, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65354935, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f95493d, SUC_lexicon_tags_45 },
    { 0xb375493e, SUC_lexicon_tags_26 },
    { 0x6a15493d, SUC_lexicon_tags_77 },
    { 0xef354940, SUC_lexicon_tags_47 },
    { 0xe135493e, SUC_lexicon_tags_8 },
    { 0xdcd54941, SUC_lexicon_tags_29 },
    { 0x2ef54943, SUC_lexicon_tags_7 },
    { 0xdb954944, SUC_lexicon_tags_19 },
    { 0xafd54945, SUC_lexicon_tags_2 },
    { 0x77d54946, SUC_lexicon_tags_38 },
    { 0x31754947, SUC_lexicon_tags_15 },
    { 0x4cf54944, SUC_lexicon_tags_55 },
    { 0xd0f54944, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1755494b, SUC_lexicon_tags_17 },
    { 0x2ed5494b, SUC_lexicon_tags_11 },
    { 0xa5f5494d, SUC_lexicon_tags_12 },
    { 0xd55494e, SUC_lexicon_tags_33 },
    { 0xc4b5494c, SUC_lexicon_tags_12 },
    { 0xd5d5494b, SUC_lexicon_tags_38 },
    { 0xf875494f, SUC_lexicon_tags_57 },
    { 0xe2754952, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc055495e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0554960, SUC_lexicon_tags_9 },
    { 0xcad54960, SUC_lexicon_tags_47 },
    { 0xc4954962, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67d54964, SUC_lexicon_tags_11 },
    { 0x52354965, SUC_lexicon_tags_60 },
    { 0x2d354964, SUC_lexicon_tags_2 },
    { 0x29154967, SUC_lexicon_tags_40 },
    { 0x6cf54967, SUC_lexicon_tags_15 },
    { 0x88354969, SUC_lexicon_tags_33 },
    { 0x9c754968, SUC_lexicon_tags_7 },
    { 0x8dd5496b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb54970, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50754978, SUC_lexicon_tags_25 },
    { 0x14554979, SUC_lexicon_tags_36 },
    { 0xfc354979, SUC_lexicon_tags_38 },
    { 0xadd5497b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3cf5497d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f54981, SUC_lexicon_tags_10 },
    { 0x98554982, SUC_lexicon_tags_25 },
    { 0x5f154983, SUC_lexicon_tags_12 },
    { 0x13d54984, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21554987, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5154989, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb9f5498b, SUC_lexicon_tags_37 },
    { 0xe975498b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf35498e, SUC_lexicon_tags_0 },
    { 0xbef5498f, SUC_lexicon_tags_37 },
    { 0xc8f5498f, SUC_lexicon_tags_48 },
    { 0x43b54991, SUC_lexicon_tags_39 },
    { 0xd2954990, SUC_lexicon_tags_2 },
    { 0x78154993, SUC_lexicon_tags_49 },
    { 0xf7b54991, SUC_lexicon_tags_37 },
    { 0x96954995, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81154997, SUC_lexicon_tags_48 },
    { 0x8154997, SUC_lexicon_tags_27 },
    { 0xd4154997, SUC_lexicon_tags_0 },
    { 0x9554999, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x3175499c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf549a4, SUC_lexicon_tags_15 },
    { 0x8d9549a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x403549a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d549a9, SUC_lexicon_tags_19 },
    { 0xd1549aa, SUC_lexicon_tags_82 },
    { 0xde7549ab, SUC_lexicon_tags_45 },
    { 0x351549ab, SUC_lexicon_tags_47 },
    { 0x6f5549ad, SUC_lexicon_tags_60 },
    { 0x4f5549ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe31549b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1549b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd549b9, SUC_lexicon_tags_11 },
    { 0x1b5549b9, SUC_lexicon_tags_8 },
    { 0x2f3549bb, SUC_lexicon_tags_45 },
    { 0x737549bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b549bf, SUC_lexicon_tags_25 },
    { 0x7f3549bf, SUC_lexicon_tags_60 },
    { 0x2f9549bf, SUC_lexicon_tags_1 },
    { 0x7b9549c2, SUC_lexicon_tags_14 },
    { 0xecf549c2, SUC_lexicon_tags_63 },
    { 0x18f549c3, SUC_lexicon_tags_37 },
    { 0xdcf549c5, SUC_lexicon_tags_2 },
    { 0x379549c6, SUC_lexicon_tags_32 },
    { 0xb83549c5, SUC_lexicon_tags_47 },
    { 0x85549bf, SUC_lexicon_tags_7 },
    { 0x9a7549c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb57549cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf2b549cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9549d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f549d9, SUC_lexicon_tags_33 },
    { 0xed1549d9, SUC_lexicon_tags_2 },
    { 0x205549db, SUC_lexicon_tags_15 },
    { 0x5ef549db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e9549de, SUC_lexicon_tags_49 },
    { 0xc07549df, SUC_lexicon_tags_7 },
    { 0x8bd549e0, SUC_lexicon_tags_37 },
    { 0xe97549e1, SUC_lexicon_tags_1 },
    { 0x4b7549e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117549e7, SUC_lexicon_tags_1 },
    { 0xc97549e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703549ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9549f0, SUC_lexicon_tags_2 },
    { 0x5e3549f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x569549f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859549f7, SUC_lexicon_tags_47 },
    { 0x1e1549f8, SUC_lexicon_tags_1 },
    { 0xf3d549f9, SUC_lexicon_tags_32 },
    { 0x9b549fa, SUC_lexicon_tags_17 },
    { 0xd77549fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d549fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb54a00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf54a03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92354a0d, SUC_lexicon_tags_60 },
    { 0x6e154a0d, SUC_lexicon_tags_43 },
    { 0x38554a0f, SUC_lexicon_tags_2 },
    { 0x2ff54a10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x12354a12, SUC_lexicon_tags_25 },
    { 0x2c354a13, SUC_lexicon_tags_25 },
    { 0xe0754a12, SUC_lexicon_tags_23 },
    { 0x2c954a14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d54a19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf54a1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b554a1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf54a26, SUC_lexicon_tags_17 },
    { 0x3154a26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd54a2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c354a2c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x95b54a2e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x60554a30, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab54a33, SUC_lexicon_tags_17 },
    { 0x7ff54a34, SUC_lexicon_tags_39 },
    { 0x23154a33, SUC_lexicon_tags_37 },
    { 0xc3754a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f54a38, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9354a3d, SUC_lexicon_tags_616 },
    { 0, NULL },
    { 0xd9354a3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67954a45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a754a47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3554a4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc0754a4f, SUC_lexicon_tags_11 },
    { 0x9c754a50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66754a52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3754a56, SUC_lexicon_tags_11 },
    { 0x6bb54a56, SUC_lexicon_tags_28 },
    { 0x25954a58, SUC_lexicon_tags_11 },
    { 0xe5154a58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79154a5f, SUC_lexicon_tags_2 },
    { 0x4f754a60, SUC_lexicon_tags_0 },
    { 0x4d554a60, SUC_lexicon_tags_7 },
    { 0x64d54a62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x43b54a64, SUC_lexicon_tags_10 },
    { 0x8e154a65, SUC_lexicon_tags_14 },
    { 0xa2554a66, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf154a6d, SUC_lexicon_tags_7 },
    { 0x21954a6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27154a75, SUC_lexicon_tags_1 },
    { 0xaef54a76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25154a79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f554a7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x44154a7f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x27b54a81, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22354a86, SUC_lexicon_tags_87 },
    { 0x3fd54a87, SUC_lexicon_tags_23 },
    { 0x66154a87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d54a8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f754a8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b54a95, SUC_lexicon_tags_33 },
    { 0x3b554a95, SUC_lexicon_tags_27 },
    { 0x7eb54a95, SUC_lexicon_tags_6 },
    { 0x6f354a95, SUC_lexicon_tags_17 },
    { 0x1af54a99, SUC_lexicon_tags_14 },
    { 0x72554a9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33154a9c, SUC_lexicon_tags_0 },
    { 0x9ff54a9c, SUC_lexicon_tags_37 },
    { 0x4954a9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2a554aa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2154aa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b54aa7, SUC_lexicon_tags_15 },
    { 0x4e154aa8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66554aac, SUC_lexicon_tags_37 },
    { 0x4f754aad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7954ab2, SUC_lexicon_tags_2 },
    { 0xd2954ab3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8554ab7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3554abc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a554ac3, SUC_lexicon_tags_37 },
    { 0x9c754ac4, SUC_lexicon_tags_33 },
    { 0xfdd54ac5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb7b54ac7, SUC_lexicon_tags_12 },
    { 0x39f54ac8, SUC_lexicon_tags_1 },
    { 0x17554ac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0954acd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc954ad5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8154ad8, SUC_lexicon_tags_7 },
    { 0x59554ad9, SUC_lexicon_tags_37 },
    { 0x12d54ada, SUC_lexicon_tags_29 },
    { 0x70d54ad8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9154add, SUC_lexicon_tags_7 },
    { 0x57d54ade, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab54ae4, SUC_lexicon_tags_7 },
    { 0x87354ae5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd54ae9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87954aeb, SUC_lexicon_tags_37 },
    { 0x16354aec, SUC_lexicon_tags_37 },
    { 0x7bb54aed, SUC_lexicon_tags_47 },
    { 0x25954aec, SUC_lexicon_tags_1 },
    { 0xe7f54aed, SUC_lexicon_tags_30 },
    { 0xe3f54aec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb54af6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15954af9, SUC_lexicon_tags_15 },
    { 0x67754afa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0554afd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d54b00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1d154b02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c154b05, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x46f54b07, SUC_lexicon_tags_37 },
    { 0x89554b08, SUC_lexicon_tags_33 },
    { 0xd5d54b09, SUC_lexicon_tags_37 },
    { 0x2b554b0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d954b0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3554b10, SUC_lexicon_tags_63 },
    { 0x94554b11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0754b15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63554b18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde754b1f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc6d54b21, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed54b24, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c954b27, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8954b2b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5cb54b2d, SUC_lexicon_tags_30 },
    { 0x53554b2d, SUC_lexicon_tags_34 },
    { 0xc7354b2f, SUC_lexicon_tags_7 },
    { 0xefb54b30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f354b36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbff54b38, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d54b3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf8154b3f, SUC_lexicon_tags_14 },
    { 0x2d154b40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64754b47, SUC_lexicon_tags_21 },
    { 0xe7554b47, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xfb54b4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea954b50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed54b55, SUC_lexicon_tags_12 },
    { 0x5e354b56, SUC_lexicon_tags_15 },
    { 0xa1754b55, SUC_lexicon_tags_12 },
    { 0x5d754b57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62954b5b, SUC_lexicon_tags_15 },
    { 0x9e354b5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75554b65, SUC_lexicon_tags_32 },
    { 0x21354b65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a354b68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1554b6c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x41154b6e, SUC_lexicon_tags_31 },
    { 0xb7f54b6e, SUC_lexicon_tags_7 },
    { 0xe9754b6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e354b77, SUC_lexicon_tags_6 },
    { 0x9d954b78, SUC_lexicon_tags_13 },
    { 0xa9f54b77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf7954b7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf554b7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2354b82, SUC_lexicon_tags_15 },
    { 0x82b54b83, SUC_lexicon_tags_26 },
    { 0xcfd54b82, SUC_lexicon_tags_39 },
    { 0xc6954b83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96954b89, SUC_lexicon_tags_48 },
    { 0x86b54b8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe2154b8c, SUC_lexicon_tags_11 },
    { 0x99d54b8d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f54b93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5954b9a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x14954b9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57154b9e, SUC_lexicon_tags_15 },
    { 0xdd954b9f, SUC_lexicon_tags_30 },
    { 0x6d554ba0, SUC_lexicon_tags_12 },
    { 0xd2f54ba0, SUC_lexicon_tags_33 },
    { 0x9a554ba2, SUC_lexicon_tags_11 },
    { 0x3bf54ba2, SUC_lexicon_tags_11 },
    { 0x8c954b9e, SUC_lexicon_tags_7 },
    { 0x1b54ba5, SUC_lexicon_tags_136 },
    { 0xf4f54ba6, SUC_lexicon_tags_47 },
    { 0xe5354ba5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed54ba9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f54bac, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3b354bae, SUC_lexicon_tags_2 },
    { 0xacd54baf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x354bb3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf5d54bb5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd754bba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f54bbe, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x7bf54bc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xab754bc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f954bc8, SUC_lexicon_tags_39 },
    { 0xc2b54bc8, SUC_lexicon_tags_25 },
    { 0x29554bca, SUC_lexicon_tags_92 },
    { 0x38f54bca, SUC_lexicon_tags_27 },
    { 0xde954bcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d54bd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7354bd5, SUC_lexicon_tags_11 },
    { 0x11754bd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d154bd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb54bdc, SUC_lexicon_tags_0 },
    { 0x85154bdd, SUC_lexicon_tags_0 },
    { 0x58754bdd, SUC_lexicon_tags_63 },
    { 0x8db54bdd, SUC_lexicon_tags_23 },
    { 0xe4d54be0, SUC_lexicon_tags_7 },
    { 0x5bf54bde, SUC_lexicon_tags_26 },
    { 0xa5354be2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b754be6, SUC_lexicon_tags_37 },
    { 0xdf54be7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x79354bea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd954bed, SUC_lexicon_tags_32 },
    { 0x9fd54bee, SUC_lexicon_tags_60 },
    { 0x3cb54bee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc554bf1, SUC_lexicon_tags_37 },
    { 0xd8954bf1, SUC_lexicon_tags_12 },
    { 0x47954bf3, SUC_lexicon_tags_47 },
    { 0x6f154bf2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x40f54bf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80954bfa, SUC_lexicon_tags_37 },
    { 0x97154bfb, SUC_lexicon_tags_17 },
    { 0xca154bfb, SUC_lexicon_tags_1 },
    { 0xd9754bfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f54c01, SUC_lexicon_tags_11 },
    { 0xdff54c02, SUC_lexicon_tags_32 },
    { 0x74554c03, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f54c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea554c0c, SUC_lexicon_tags_19 },
    { 0xabb54c0d, SUC_lexicon_tags_43 },
    { 0x5c554c0e, SUC_lexicon_tags_37 },
    { 0xd4754c0e, SUC_lexicon_tags_17 },
    { 0x87554c10, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x99954c13, SUC_lexicon_tags_17 },
    { 0xa9b54c13, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4554c16, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x20354c18, SUC_lexicon_tags_15 },
    { 0x41154c19, SUC_lexicon_tags_28 },
    { 0x8f554c19, SUC_lexicon_tags_38 },
    { 0x354c19, SUC_lexicon_tags_14 },
    { 0x63154c1c, SUC_lexicon_tags_32 },
    { 0x86f54c1a, SUC_lexicon_tags_11 },
    { 0xded54c1e, SUC_lexicon_tags_17 },
    { 0xe3d54c1a, SUC_lexicon_tags_27 },
    { 0xbdd54c20, SUC_lexicon_tags_15 },
    { 0x1eb54c21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3754c24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85754c27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b54c2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd54c34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d54c38, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9754c3b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d54c3f, SUC_lexicon_tags_11 },
    { 0xa8554c3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b54c44, SUC_lexicon_tags_334 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f54c49, SUC_lexicon_tags_14 },
    { 0xab354c4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d54c4f, SUC_lexicon_tags_45 },
    { 0x28954c50, SUC_lexicon_tags_39 },
    { 0xaf754c51, SUC_lexicon_tags_6 },
    { 0x33354c4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb154c56, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa3f54c58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe154c5d, SUC_lexicon_tags_124 },
    { 0x9fb54c5e, SUC_lexicon_tags_6 },
    { 0x5d954c5f, SUC_lexicon_tags_1024 },
    { 0xdb554c60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf754c62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62154c64, SUC_lexicon_tags_91 },
    { 0xb2d54c65, SUC_lexicon_tags_12 },
    { 0x30f54c66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8554c68, SUC_lexicon_tags_0 },
    { 0xc1554c69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0754c6d, SUC_lexicon_tags_37 },
    { 0x44954c6d, SUC_lexicon_tags_23 },
    { 0x3a354c6d, SUC_lexicon_tags_17 },
    { 0xb6554c6e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x15354c72, SUC_lexicon_tags_45 },
    { 0x1e354c73, SUC_lexicon_tags_17 },
    { 0x66b54c74, SUC_lexicon_tags_29 },
    { 0xea554c73, SUC_lexicon_tags_7 },
    { 0x13b54c76, SUC_lexicon_tags_38 },
    { 0xe0b54c74, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b754c7c, SUC_lexicon_tags_33 },
    { 0xa2d54c7c, SUC_lexicon_tags_33 },
    { 0x28554c7e, SUC_lexicon_tags_12 },
    { 0x34554c7f, SUC_lexicon_tags_15 },
    { 0x7d554c80, SUC_lexicon_tags_40 },
    { 0xb2b54c81, SUC_lexicon_tags_151 },
    { 0xed554c82, SUC_lexicon_tags_77 },
    { 0xa9754c82, SUC_lexicon_tags_1 },
    { 0x67b54c7e, SUC_lexicon_tags_46 },
    { 0x25f54c7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7754c8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe4754c8c, SUC_lexicon_tags_6 },
    { 0x8df54c8d, SUC_lexicon_tags_31 },
    { 0x66354c8d, SUC_lexicon_tags_31 },
    { 0xfb354c8f, SUC_lexicon_tags_57 },
    { 0x8e954c8f, SUC_lexicon_tags_4 },
    { 0xa9554c91, SUC_lexicon_tags_15 },
    { 0xe9b54c91, SUC_lexicon_tags_47 },
    { 0x1d154c92, SUC_lexicon_tags_17 },
    { 0xb9954c94, SUC_lexicon_tags_37 },
    { 0xaf754c95, SUC_lexicon_tags_2 },
    { 0x96754c93, SUC_lexicon_tags_37 },
    { 0xc6954c93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e954c9b, SUC_lexicon_tags_37 },
    { 0xb2754c9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x67d54c9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8f954ca0, SUC_lexicon_tags_60 },
    { 0xbe754ca1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7d54ca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8554ca8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa954cb9, SUC_lexicon_tags_56 },
    { 0xf5754cb9, SUC_lexicon_tags_29 },
    { 0x8db54cbb, SUC_lexicon_tags_17 },
    { 0x19f54cbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4954cc0, SUC_lexicon_tags_38 },
    { 0xc6154cc0, SUC_lexicon_tags_0 },
    { 0xb8b54cc2, SUC_lexicon_tags_0 },
    { 0x5e554cc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x47954cc5, SUC_lexicon_tags_30 },
    { 0x8f54cc6, SUC_lexicon_tags_43 },
    { 0x47f54cc6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x60554cc9, SUC_lexicon_tags_37 },
    { 0x38354cc9, SUC_lexicon_tags_2 },
    { 0x29154ccb, SUC_lexicon_tags_3 },
    { 0x4d54cc9, SUC_lexicon_tags_12 },
    { 0x2db54ccc, SUC_lexicon_tags_39 },
    { 0xe6154cc9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b54cd4, SUC_lexicon_tags_11 },
    { 0x12554cd5, SUC_lexicon_tags_16 },
    { 0xd1f54cd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0354cd9, SUC_lexicon_tags_33 },
    { 0x4cd54cd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e154cdd, SUC_lexicon_tags_1 },
    { 0xc6154cde, SUC_lexicon_tags_11 },
    { 0x41b54cde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10354ce5, SUC_lexicon_tags_7 },
    { 0x6d554ce6, SUC_lexicon_tags_7 },
    { 0x9d354ce6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d554cec, SUC_lexicon_tags_55 },
    { 0x9cf54ced, SUC_lexicon_tags_15 },
    { 0x40b54cee, SUC_lexicon_tags_11 },
    { 0xb5354cef, SUC_lexicon_tags_68 },
    { 0x19554cf0, SUC_lexicon_tags_30 },
    { 0x9eb54cf1, SUC_lexicon_tags_7 },
    { 0x9d754cf2, SUC_lexicon_tags_37 },
    { 0xac754cf3, SUC_lexicon_tags_79 },
    { 0x99154cf3, SUC_lexicon_tags_37 },
    { 0x52d54cf5, SUC_lexicon_tags_12 },
    { 0xd1754cf4, SUC_lexicon_tags_29 },
    { 0x49154cf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89754cfb, SUC_lexicon_tags_15 },
    { 0x1fd54cfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a554d00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b554d06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef54d0b, SUC_lexicon_tags_30 },
    { 0x4dd54d0c, SUC_lexicon_tags_31 },
    { 0x4fd54d0b, SUC_lexicon_tags_21 },
    { 0xfb354d0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b54d11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92754d13, SUC_lexicon_tags_12 },
    { 0x97954d13, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0354d17, SUC_lexicon_tags_47 },
    { 0x54354d18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57f54d1a, SUC_lexicon_tags_35 },
    { 0x56f54d1b, SUC_lexicon_tags_34 },
    { 0x7cb54d1c, SUC_lexicon_tags_10 },
    { 0x94354d1c, SUC_lexicon_tags_45 },
    { 0x4ef54d1b, SUC_lexicon_tags_33 },
    { 0x75554d1b, SUC_lexicon_tags_12 },
    { 0xb5b54d20, SUC_lexicon_tags_13 },
    { 0x6dd54d1d, SUC_lexicon_tags_47 },
    { 0x7f554d22, SUC_lexicon_tags_11 },
    { 0xb1154d23, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe2754d25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26154d28, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf954d2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb54d2e, SUC_lexicon_tags_16 },
    { 0x25554d2f, SUC_lexicon_tags_1 },
    { 0xd4f54d30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92354d32, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xb9554d34, SUC_lexicon_tags_15 },
    { 0x87354d35, SUC_lexicon_tags_25 },
    { 0x8d354d36, SUC_lexicon_tags_17 },
    { 0x30754d37, SUC_lexicon_tags_37 },
    { 0x76154d38, SUC_lexicon_tags_37 },
    { 0x2bd54d39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f54d3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7154d42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5954d44, SUC_lexicon_tags_103 },
    { 0x6dd54d45, SUC_lexicon_tags_87 },
    { 0x70d54d46, SUC_lexicon_tags_7 },
    { 0x1a554d47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb54d4a, SUC_lexicon_tags_1 },
    { 0xf3f54d4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff54d51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf54d54, SUC_lexicon_tags_7 },
    { 0x60354d54, SUC_lexicon_tags_37 },
    { 0xc1f54d56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xea554d59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47954d61, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad554d66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcfd54d68, SUC_lexicon_tags_19 },
    { 0x39354d69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d354d6c, SUC_lexicon_tags_25 },
    { 0x87154d6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f54d70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21754d75, SUC_lexicon_tags_60 },
    { 0x70f54d76, SUC_lexicon_tags_17 },
    { 0x58d54d77, SUC_lexicon_tags_17 },
    { 0x59754d76, SUC_lexicon_tags_15 },
    { 0x86554d78, SUC_lexicon_tags_8 },
    { 0x43354d7a, SUC_lexicon_tags_60 },
    { 0xb6954d7a, SUC_lexicon_tags_26 },
    { 0x3c354d77, SUC_lexicon_tags_12 },
    { 0xf0754d7d, SUC_lexicon_tags_65 },
    { 0xf3754d7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54154d84, SUC_lexicon_tags_7 },
    { 0x37f54d85, SUC_lexicon_tags_17 },
    { 0x86954d86, SUC_lexicon_tags_11 },
    { 0x1a754d86, SUC_lexicon_tags_38 },
    { 0x77354d88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f54d92, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4954d99, SUC_lexicon_tags_25 },
    { 0x73954d9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d354d9c, SUC_lexicon_tags_17 },
    { 0x40954d9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7cf54d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4954da4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xae754da6, SUC_lexicon_tags_65 },
    { 0xfb54da7, SUC_lexicon_tags_26 },
    { 0xfaf54da8, SUC_lexicon_tags_2 },
    { 0xa6754da9, SUC_lexicon_tags_29 },
    { 0xcbf54daa, SUC_lexicon_tags_26 },
    { 0x55754da7, SUC_lexicon_tags_11 },
    { 0x25f54dac, SUC_lexicon_tags_47 },
    { 0xdd154dac, SUC_lexicon_tags_1 },
    { 0x12754dae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd154db5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb54dba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x57954dbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b54dc1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x49554dc3, SUC_lexicon_tags_61 },
    { 0x12f54dc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f54dcd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x78554dd0, SUC_lexicon_tags_15 },
    { 0x65f54dd1, SUC_lexicon_tags_6 },
    { 0xb354dd2, SUC_lexicon_tags_85 },
    { 0x84f54dd1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66154dd8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x75754dda, SUC_lexicon_tags_1 },
    { 0xe354ddb, SUC_lexicon_tags_33 },
    { 0x49954dda, SUC_lexicon_tags_21 },
    { 0xb7154ddd, SUC_lexicon_tags_60 },
    { 0xbd754ddd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd54de7, SUC_lexicon_tags_253 },
    { 0x64b54de8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4554dea, SUC_lexicon_tags_27 },
    { 0xaa154deb, SUC_lexicon_tags_68 },
    { 0xc8754dec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40354dee, SUC_lexicon_tags_27 },
    { 0x9af54def, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x33954df1, SUC_lexicon_tags_7 },
    { 0x59154df1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x17954df4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53154df7, SUC_lexicon_tags_30 },
    { 0xd154df8, SUC_lexicon_tags_12 },
    { 0x48154df7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f554dfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48d54dfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9954e02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51554e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5754e0b, SUC_lexicon_tags_0 },
    { 0x4e754e0c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa554e12, SUC_lexicon_tags_17 },
    { 0xb8954e13, SUC_lexicon_tags_15 },
    { 0xa1d54e14, SUC_lexicon_tags_16 },
    { 0x82554e15, SUC_lexicon_tags_375 },
    { 0x5d554e16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x50f54e18, SUC_lexicon_tags_60 },
    { 0x6d54e18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff54e1d, SUC_lexicon_tags_37 },
    { 0x98f54e1d, SUC_lexicon_tags_37 },
    { 0x15554e1f, SUC_lexicon_tags_4 },
    { 0xa9354e20, SUC_lexicon_tags_17 },
    { 0x82f54e21, SUC_lexicon_tags_27 },
    { 0x83d54e1f, SUC_lexicon_tags_44 },
    { 0x5ab54e23, SUC_lexicon_tags_84 },
    { 0xcbd54e24, SUC_lexicon_tags_37 },
    { 0xb8354e25, SUC_lexicon_tags_9 },
    { 0xddf54e21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef54e29, SUC_lexicon_tags_69 },
    { 0x32b54e29, SUC_lexicon_tags_27 },
    { 0x9b954e2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f54e2f, SUC_lexicon_tags_13 },
    { 0x36354e30, SUC_lexicon_tags_60 },
    { 0x75b54e2f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b54e34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5354e37, SUC_lexicon_tags_348 },
    { 0x18b54e37, SUC_lexicon_tags_9 },
    { 0xc7354e37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67f54e3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17154e42, SUC_lexicon_tags_16 },
    { 0x26954e42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4554e45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6a554e47, SUC_lexicon_tags_25 },
    { 0x84f54e48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1354e51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x49354e54, SUC_lexicon_tags_6 },
    { 0x2f954e55, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39154e59, SUC_lexicon_tags_10 },
    { 0x26b54e5a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f54e61, SUC_lexicon_tags_47 },
    { 0x98354e62, SUC_lexicon_tags_15 },
    { 0x82d54e63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f54e67, SUC_lexicon_tags_9 },
    { 0xbd754e67, SUC_lexicon_tags_17 },
    { 0x3db54e68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b954e6c, SUC_lexicon_tags_65 },
    { 0xb9954e6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2bf54e6f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b54e72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb5954e74, SUC_lexicon_tags_25 },
    { 0x49954e74, SUC_lexicon_tags_49 },
    { 0x73f54e76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ab54e78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdab54e7a, SUC_lexicon_tags_32 },
    { 0xc8154e7b, SUC_lexicon_tags_11 },
    { 0xd954e7c, SUC_lexicon_tags_11 },
    { 0x4c154e7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x41154e7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc4154e81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c154e83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdd54e85, SUC_lexicon_tags_44 },
    { 0xa554e86, SUC_lexicon_tags_17 },
    { 0xf5354e85, SUC_lexicon_tags_318 },
    { 0x2d554e86, SUC_lexicon_tags_37 },
    { 0x16554e88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbad54e8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df54e90, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2954e96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef54e99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88554ea1, SUC_lexicon_tags_17 },
    { 0xacb54ea1, SUC_lexicon_tags_27 },
    { 0x4e354ea3, SUC_lexicon_tags_29 },
    { 0xc7d54ea3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x154ea7, SUC_lexicon_tags_19 },
    { 0x24554ea8, SUC_lexicon_tags_29 },
    { 0xf5b54ea7, SUC_lexicon_tags_15 },
    { 0x27b54eaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea354eb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf954eb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc954eba, SUC_lexicon_tags_9 },
    { 0xbb754eba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f54ebe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xad54ec0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69d54ec2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5354ec5, SUC_lexicon_tags_23 },
    { 0xf1d54ec6, SUC_lexicon_tags_55 },
    { 0x3f554ec7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b54eca, SUC_lexicon_tags_15 },
    { 0x9af54eca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2754ece, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d54ed2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd5554ed4, SUC_lexicon_tags_45 },
    { 0xd5b54ed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6f54ed7, SUC_lexicon_tags_1 },
    { 0x64954ed8, SUC_lexicon_tags_17 },
    { 0xf3554ed7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69354ede, SUC_lexicon_tags_7 },
    { 0xccd54ede, SUC_lexicon_tags_2 },
    { 0x5db54edf, SUC_lexicon_tags_19 },
    { 0x57554ee1, SUC_lexicon_tags_28 },
    { 0xcbf54ee2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d54ee8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57354eeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9754eed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9554eef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x65154ef2, SUC_lexicon_tags_39 },
    { 0x78554ef2, SUC_lexicon_tags_17 },
    { 0x3f354ef4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5754ef7, SUC_lexicon_tags_69 },
    { 0x10154ef8, SUC_lexicon_tags_7 },
    { 0x76954ef9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b954efb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50b54efd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90754eff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b54f03, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd1b54f05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b754f09, SUC_lexicon_tags_39 },
    { 0x67154f09, SUC_lexicon_tags_14 },
    { 0xd5754f0b, SUC_lexicon_tags_1 },
    { 0x36f54f0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92954f11, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf4f54f13, SUC_lexicon_tags_45 },
    { 0x38b54f13, SUC_lexicon_tags_12 },
    { 0xdcf54f13, SUC_lexicon_tags_37 },
    { 0xe3f54f15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10154f18, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b54f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e154f24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57954f26, SUC_lexicon_tags_7 },
    { 0x86954f27, SUC_lexicon_tags_39 },
    { 0x5db54f28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47354f2f, SUC_lexicon_tags_27 },
    { 0x6b754f2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x18154f32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe9154f34, SUC_lexicon_tags_17 },
    { 0x8a954f35, SUC_lexicon_tags_2 },
    { 0xce954f36, SUC_lexicon_tags_27 },
    { 0xa2754f37, SUC_lexicon_tags_14 },
    { 0x2e354f38, SUC_lexicon_tags_25 },
    { 0xf9f54f38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfef54f3b, SUC_lexicon_tags_15 },
    { 0x154f3c, SUC_lexicon_tags_62 },
    { 0x7fd54f3d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3954f43, SUC_lexicon_tags_10 },
    { 0x92b54f44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd54f4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b54f50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43754f55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb54f5a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d154f5d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6c354f5f, SUC_lexicon_tags_17 },
    { 0x954f60, SUC_lexicon_tags_8 },
    { 0x26f54f61, SUC_lexicon_tags_17 },
    { 0xa8554f5f, SUC_lexicon_tags_16 },
    { 0x89154f63, SUC_lexicon_tags_35 },
    { 0xc8d54f64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18354f68, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3554f6b, SUC_lexicon_tags_66 },
    { 0x13354f6b, SUC_lexicon_tags_0 },
    { 0xb5554f6d, SUC_lexicon_tags_1 },
    { 0x14b54f6d, SUC_lexicon_tags_38 },
    { 0x46754f6f, SUC_lexicon_tags_77 },
    { 0x83954f6e, SUC_lexicon_tags_91 },
    { 0xfb554f71, SUC_lexicon_tags_7 },
    { 0x76554f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86354f74, SUC_lexicon_tags_11 },
    { 0xa8554f74, SUC_lexicon_tags_1 },
    { 0x2cd54f74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af54f7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b754f7d, SUC_lexicon_tags_9 },
    { 0xe8754f7d, SUC_lexicon_tags_1 },
    { 0xdeb54f7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d54f83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d54f8a, SUC_lexicon_tags_40 },
    { 0xdd154f8b, SUC_lexicon_tags_27 },
    { 0x6c954f8c, SUC_lexicon_tags_17 },
    { 0xf7f54f8c, SUC_lexicon_tags_87 },
    { 0x2d754f8e, SUC_lexicon_tags_0 },
    { 0xf7354f8f, SUC_lexicon_tags_11 },
    { 0xecb54f90, SUC_lexicon_tags_27 },
    { 0x97954f90, SUC_lexicon_tags_80 },
    { 0x9b754f8f, SUC_lexicon_tags_60 },
    { 0x41f54f93, SUC_lexicon_tags_11 },
    { 0xc6154f8a, SUC_lexicon_tags_25 },
    { 0xc1754f8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe754f9a, SUC_lexicon_tags_81 },
    { 0x77f54f9a, SUC_lexicon_tags_2 },
    { 0x8af54f9c, SUC_lexicon_tags_7 },
    { 0x7d54f9a, SUC_lexicon_tags_32 },
    { 0x76754f9e, SUC_lexicon_tags_14 },
    { 0xc9754f9e, SUC_lexicon_tags_65 },
    { 0x28154fa0, SUC_lexicon_tags_60 },
    { 0xa4d54f9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bf54fa3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc3d54fa5, SUC_lexicon_tags_11 },
    { 0xe8954fa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf354fa8, SUC_lexicon_tags_17 },
    { 0xfc754fa9, SUC_lexicon_tags_2 },
    { 0x87b54faa, SUC_lexicon_tags_7 },
    { 0x8e154fab, SUC_lexicon_tags_92 },
    { 0xc9954fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10554fb1, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0154fb8, SUC_lexicon_tags_47 },
    { 0x93d54fb9, SUC_lexicon_tags_1 },
    { 0xfdb54fba, SUC_lexicon_tags_37 },
    { 0x84554fb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88154fbf, SUC_lexicon_tags_7 },
    { 0xc0554fc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6154fc2, SUC_lexicon_tags_11 },
    { 0xe954fc3, SUC_lexicon_tags_37 },
    { 0xa5754fc4, SUC_lexicon_tags_38 },
    { 0x7154fc5, SUC_lexicon_tags_19 },
    { 0xb3354fc2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b354fc9, SUC_lexicon_tags_37 },
    { 0xfc754fca, SUC_lexicon_tags_45 },
    { 0xcaf54fc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83754fce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b54fd2, SUC_lexicon_tags_29 },
    { 0x7c754fd2, SUC_lexicon_tags_37 },
    { 0x6ab54fd4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x55554fd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f354fda, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x68954fdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed54fe3, SUC_lexicon_tags_17 },
    { 0x30554fe3, SUC_lexicon_tags_6 },
    { 0x12554fe5, SUC_lexicon_tags_1 },
    { 0xe6354fe4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x71354fe8, SUC_lexicon_tags_49 },
    { 0x54954fe8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f154fed, SUC_lexicon_tags_15 },
    { 0x2cd54fee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d354ff2, SUC_lexicon_tags_14 },
    { 0xc7954ff3, SUC_lexicon_tags_25 },
    { 0xdbd54ff4, SUC_lexicon_tags_34 },
    { 0x87b54ff5, SUC_lexicon_tags_2 },
    { 0xf8754ff6, SUC_lexicon_tags_25 },
    { 0xc4154ff6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d54ffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81555004, SUC_lexicon_tags_49 },
    { 0x3d555005, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d55008, SUC_lexicon_tags_1713 },
    { 0, NULL },
    { 0xd155500a, SUC_lexicon_tags_55 },
    { 0x7af5500b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8755012, SUC_lexicon_tags_19 },
    { 0xfdb55013, SUC_lexicon_tags_19 },
    { 0xe355013, SUC_lexicon_tags_13 },
    { 0x31d55013, SUC_lexicon_tags_12 },
    { 0xd1955016, SUC_lexicon_tags_17 },
    { 0x1bd55017, SUC_lexicon_tags_6 },
    { 0x44955018, SUC_lexicon_tags_8 },
    { 0x11f55016, SUC_lexicon_tags_1 },
    { 0x83355018, SUC_lexicon_tags_9 },
    { 0x6e15501a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2e15501d, SUC_lexicon_tags_8 },
    { 0x2b15501e, SUC_lexicon_tags_0 },
    { 0x435501d, SUC_lexicon_tags_63 },
    { 0xcdf55020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31955022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56555024, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8955028, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d5502c, SUC_lexicon_tags_47 },
    { 0x655502d, SUC_lexicon_tags_49 },
    { 0x6a35502d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31155030, SUC_lexicon_tags_25 },
    { 0x80f55031, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1755038, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5835503a, SUC_lexicon_tags_1 },
    { 0xc9f5503b, SUC_lexicon_tags_38 },
    { 0x81b5503b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3655503f, SUC_lexicon_tags_7 },
    { 0xc8d5503f, SUC_lexicon_tags_0 },
    { 0x1ef55040, SUC_lexicon_tags_35 },
    { 0x3cb5503f, SUC_lexicon_tags_17 },
    { 0xae155040, SUC_lexicon_tags_12 },
    { 0xd4d55043, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6cf55046, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb1b55048, SUC_lexicon_tags_16 },
    { 0xb3755048, SUC_lexicon_tags_30 },
    { 0x5f55504a, SUC_lexicon_tags_17 },
    { 0xff75504b, SUC_lexicon_tags_7 },
    { 0x78b5504c, SUC_lexicon_tags_0 },
    { 0xf015504a, SUC_lexicon_tags_1 },
    { 0x2035504e, SUC_lexicon_tags_7 },
    { 0x47f55049, SUC_lexicon_tags_37 },
    { 0xefd55050, SUC_lexicon_tags_15 },
    { 0x1055504a, SUC_lexicon_tags_3 },
    { 0xa2d5504d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x78955055, SUC_lexicon_tags_37 },
    { 0x5a155056, SUC_lexicon_tags_37 },
    { 0xa955057, SUC_lexicon_tags_1 },
    { 0xbc555058, SUC_lexicon_tags_7 },
    { 0x72f55059, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d15505e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb55067, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x97755069, SUC_lexicon_tags_2 },
    { 0xd5b5506a, SUC_lexicon_tags_31 },
    { 0x5d75506a, SUC_lexicon_tags_45 },
    { 0x7735506c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9d5506e, SUC_lexicon_tags_12 },
    { 0xa275506e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89155072, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d355076, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa95507a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f95507d, SUC_lexicon_tags_30 },
    { 0xac75507e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xb4d55080, SUC_lexicon_tags_0 },
    { 0x22555081, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x99b55083, SUC_lexicon_tags_19 },
    { 0x7ab55084, SUC_lexicon_tags_12 },
    { 0x93555085, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd5508a, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56155091, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81755095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2555097, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5275509a, SUC_lexicon_tags_28 },
    { 0xbb15509b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f5509e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad550a6, SUC_lexicon_tags_38 },
    { 0x69d550a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x309550a9, SUC_lexicon_tags_17 },
    { 0x66b550a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x915550ad, SUC_lexicon_tags_15 },
    { 0x9d3550ae, SUC_lexicon_tags_17 },
    { 0x3bf550af, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbd550b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc7d550b3, SUC_lexicon_tags_16 },
    { 0x8fd550b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7550ba, SUC_lexicon_tags_63 },
    { 0x543550bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd3550bd, SUC_lexicon_tags_16 },
    { 0x29d550bd, SUC_lexicon_tags_6 },
    { 0x8fd550bf, SUC_lexicon_tags_32 },
    { 0x52b550be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x593550c2, SUC_lexicon_tags_15 },
    { 0x4ab550c3, SUC_lexicon_tags_7 },
    { 0xb550c2, SUC_lexicon_tags_46 },
    { 0x593550c3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x163550c7, SUC_lexicon_tags_15 },
    { 0xf91550c8, SUC_lexicon_tags_63 },
    { 0x649550c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1df550cb, SUC_lexicon_tags_23 },
    { 0x7b7550cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f550d6, SUC_lexicon_tags_46 },
    { 0x39d550d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfad550d9, SUC_lexicon_tags_56 },
    { 0x129550d9, SUC_lexicon_tags_12 },
    { 0xdc7550db, SUC_lexicon_tags_18 },
    { 0xe43550dc, SUC_lexicon_tags_17 },
    { 0xc49550dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4db550df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3550e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3550e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7550ea, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xeb550ec, SUC_lexicon_tags_17 },
    { 0xfb1550ed, SUC_lexicon_tags_6 },
    { 0x91f550ed, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3550f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x133550f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3550fd, SUC_lexicon_tags_40 },
    { 0x96f550fe, SUC_lexicon_tags_37 },
    { 0xbf550fd, SUC_lexicon_tags_10 },
    { 0xf1955100, SUC_lexicon_tags_0 },
    { 0xa3b55101, SUC_lexicon_tags_47 },
    { 0xb7355101, SUC_lexicon_tags_38 },
    { 0x33f55103, SUC_lexicon_tags_19 },
    { 0x51f55104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37955106, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb75510c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb55110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d155114, SUC_lexicon_tags_4 },
    { 0x37d55115, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f55118, SUC_lexicon_tags_19 },
    { 0x11555119, SUC_lexicon_tags_12 },
    { 0xc455511a, SUC_lexicon_tags_9 },
    { 0x43f5511a, SUC_lexicon_tags_12 },
    { 0x9975511c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f55125, SUC_lexicon_tags_29 },
    { 0xfd955125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef5512a, SUC_lexicon_tags_1328 },
    { 0x1c55512a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf5512e, SUC_lexicon_tags_33 },
    { 0xac75512e, SUC_lexicon_tags_17 },
    { 0xab5512f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27755132, SUC_lexicon_tags_25 },
    { 0xad155133, SUC_lexicon_tags_37 },
    { 0xd0955132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b55139, SUC_lexicon_tags_38 },
    { 0xcc75513a, SUC_lexicon_tags_23 },
    { 0xdf5513b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe75513e, SUC_lexicon_tags_33 },
    { 0x5435513f, SUC_lexicon_tags_14 },
    { 0x4575513f, SUC_lexicon_tags_7 },
    { 0x49d55141, SUC_lexicon_tags_25 },
    { 0x1b75513f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2355144, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f5514b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d5514e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2955157, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d5515f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c755162, SUC_lexicon_tags_17 },
    { 0x57f55163, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f55516a, SUC_lexicon_tags_17 },
    { 0x1875516b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xced5516d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af55171, SUC_lexicon_tags_1 },
    { 0x50355171, SUC_lexicon_tags_4 },
    { 0x74155172, SUC_lexicon_tags_15 },
    { 0x2d555174, SUC_lexicon_tags_12 },
    { 0x3d755175, SUC_lexicon_tags_919 },
    { 0x91555175, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3955179, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335517d, SUC_lexicon_tags_30 },
    { 0xa1d5517e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fd55180, SUC_lexicon_tags_30 },
    { 0x2ab55181, SUC_lexicon_tags_7 },
    { 0x6ad55181, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f55185, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbff55187, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d5518b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe355190, SUC_lexicon_tags_29 },
    { 0x4fb55191, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7b55193, SUC_lexicon_tags_15 },
    { 0xc2f55194, SUC_lexicon_tags_37 },
    { 0xa0f55193, SUC_lexicon_tags_11 },
    { 0x40f55196, SUC_lexicon_tags_37 },
    { 0x8f755196, SUC_lexicon_tags_0 },
    { 0x18d55196, SUC_lexicon_tags_27 },
    { 0x94155198, SUC_lexicon_tags_84 },
    { 0x6c95519a, SUC_lexicon_tags_91 },
    { 0x45555197, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf235519e, SUC_lexicon_tags_27 },
    { 0xeb95519e, SUC_lexicon_tags_11 },
    { 0xad75519f, SUC_lexicon_tags_37 },
    { 0xda1551a1, SUC_lexicon_tags_7 },
    { 0x26b551a2, SUC_lexicon_tags_60 },
    { 0x263551a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1551ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe37551ae, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737551b5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65551bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581551bf, SUC_lexicon_tags_61 },
    { 0xd9551bf, SUC_lexicon_tags_25 },
    { 0x621551c0, SUC_lexicon_tags_37 },
    { 0x7a9551c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1551c5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f551c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501551d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f551d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1551da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7551dd, SUC_lexicon_tags_19 },
    { 0x3e9551dd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf551e1, SUC_lexicon_tags_7 },
    { 0xa57551e1, SUC_lexicon_tags_48 },
    { 0xf63551e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195551ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b551f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d551f7, SUC_lexicon_tags_14 },
    { 0xce9551f7, SUC_lexicon_tags_7 },
    { 0x689551f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d355200, SUC_lexicon_tags_56 },
    { 0xc6b55201, SUC_lexicon_tags_34 },
    { 0xa2f55202, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ad55204, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4755209, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x275520b, SUC_lexicon_tags_56 },
    { 0x7a95520c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b15520f, SUC_lexicon_tags_26 },
    { 0x23555210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d55213, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cf55215, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb955219, SUC_lexicon_tags_17 },
    { 0x9195521a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa575521c, SUC_lexicon_tags_71 },
    { 0xc775521d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb355220, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff555222, SUC_lexicon_tags_7 },
    { 0xaad55223, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8755227, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8355229, SUC_lexicon_tags_45 },
    { 0x3a55522a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d5522e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87955231, SUC_lexicon_tags_37 },
    { 0xdd955231, SUC_lexicon_tags_33 },
    { 0x25d55232, SUC_lexicon_tags_10 },
    { 0x2bb55234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38f55236, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab15523d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17155244, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68955246, SUC_lexicon_tags_19 },
    { 0x20955247, SUC_lexicon_tags_65 },
    { 0xa3155248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b15524a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1d5524c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c95524e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x37555251, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x99755254, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe155256, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b55259, SUC_lexicon_tags_18 },
    { 0x1235525a, SUC_lexicon_tags_16 },
    { 0xe5d5525a, SUC_lexicon_tags_37 },
    { 0xa375525c, SUC_lexicon_tags_14 },
    { 0x6e75525a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa155260, SUC_lexicon_tags_46 },
    { 0x10755261, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde955268, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe615526f, SUC_lexicon_tags_49 },
    { 0xdb55270, SUC_lexicon_tags_15 },
    { 0xf3b55271, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d55276, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74155278, SUC_lexicon_tags_129 },
    { 0x16155278, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d5527c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d5527f, SUC_lexicon_tags_17 },
    { 0x95555280, SUC_lexicon_tags_29 },
    { 0xc75527f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6955286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e35528d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f355290, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x67155292, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a955294, SUC_lexicon_tags_38 },
    { 0xd3955295, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb3555297, SUC_lexicon_tags_8 },
    { 0x78f55297, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf5529b, SUC_lexicon_tags_15 },
    { 0xc55529b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5575529e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3a1552a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39552a4, SUC_lexicon_tags_43 },
    { 0xb6d552a5, SUC_lexicon_tags_7 },
    { 0x73f552a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1552ad, SUC_lexicon_tags_61 },
    { 0x83d552ae, SUC_lexicon_tags_17 },
    { 0x147552af, SUC_lexicon_tags_8 },
    { 0xa6d552ad, SUC_lexicon_tags_1 },
    { 0xc0b552af, SUC_lexicon_tags_40 },
    { 0x105552ae, SUC_lexicon_tags_17 },
    { 0x30d552b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf552b6, SUC_lexicon_tags_15 },
    { 0xbb9552b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x415552b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x585552bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x531552bd, SUC_lexicon_tags_1 },
    { 0xa5552bd, SUC_lexicon_tags_47 },
    { 0xd43552bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfe1552c1, SUC_lexicon_tags_7 },
    { 0xc37552c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x211552c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x393552c7, SUC_lexicon_tags_1 },
    { 0x287552c7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9552cd, SUC_lexicon_tags_23 },
    { 0x96b552ce, SUC_lexicon_tags_0 },
    { 0xb09552cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3552d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x79d552d4, SUC_lexicon_tags_23 },
    { 0xc5d552d5, SUC_lexicon_tags_2 },
    { 0xb9552d5, SUC_lexicon_tags_18 },
    { 0x22f552d7, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1552da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd552e7, SUC_lexicon_tags_238 },
    { 0xb2d552e8, SUC_lexicon_tags_25 },
    { 0xedd552e9, SUC_lexicon_tags_77 },
    { 0x321552ea, SUC_lexicon_tags_2 },
    { 0xe85552ea, SUC_lexicon_tags_29 },
    { 0xb3f552eb, SUC_lexicon_tags_8 },
    { 0x34b552e9, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87552f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d552f5, SUC_lexicon_tags_8 },
    { 0xd67552f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c755301, SUC_lexicon_tags_15 },
    { 0x80f55302, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d55308, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b5530b, SUC_lexicon_tags_17 },
    { 0x2bd5530b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf55313, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9755318, SUC_lexicon_tags_27 },
    { 0xa6b55319, SUC_lexicon_tags_11 },
    { 0x7f355319, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d5531d, SUC_lexicon_tags_37 },
    { 0x30f5531e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x74f55320, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d55323, SUC_lexicon_tags_66 },
    { 0x80155324, SUC_lexicon_tags_12 },
    { 0x60355325, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67755328, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4815532d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a155333, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9955337, SUC_lexicon_tags_60 },
    { 0x2755338, SUC_lexicon_tags_43 },
    { 0x88b55339, SUC_lexicon_tags_42 },
    { 0xbc35533a, SUC_lexicon_tags_77 },
    { 0xe5f55338, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d55533e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdfd55340, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x48d55342, SUC_lexicon_tags_13 },
    { 0x70155343, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20555349, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd275534b, SUC_lexicon_tags_29 },
    { 0x8835534c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b55356, SUC_lexicon_tags_15 },
    { 0xcd55357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb5535a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe895535e, SUC_lexicon_tags_1 },
    { 0xe915535e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb55369, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf215536d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcab5536f, SUC_lexicon_tags_37 },
    { 0x50955370, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1155374, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b555376, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9955378, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2935537a, SUC_lexicon_tags_47 },
    { 0x5e35537b, SUC_lexicon_tags_1 },
    { 0x4975537c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0755382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3755384, SUC_lexicon_tags_11 },
    { 0xaa555385, SUC_lexicon_tags_40 },
    { 0x76d55385, SUC_lexicon_tags_63 },
    { 0x35955387, SUC_lexicon_tags_46 },
    { 0xa3d55387, SUC_lexicon_tags_37 },
    { 0x19955388, SUC_lexicon_tags_47 },
    { 0x5035538a, SUC_lexicon_tags_24 },
    { 0x6d55386, SUC_lexicon_tags_23 },
    { 0x31755386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee155391, SUC_lexicon_tags_17 },
    { 0x94d55391, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db55396, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd135539b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8495539d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f5539f, SUC_lexicon_tags_55 },
    { 0xba75539f, SUC_lexicon_tags_11 },
    { 0x1e1553a1, SUC_lexicon_tags_17 },
    { 0x69b553a2, SUC_lexicon_tags_15 },
    { 0xe87553a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x659553a5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43553a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f553ad, SUC_lexicon_tags_12 },
    { 0x7a5553ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb553b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3553b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d553b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x147553ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b553bf, SUC_lexicon_tags_14 },
    { 0x26d553bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x637553c2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x29f553c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77b553c6, SUC_lexicon_tags_2 },
    { 0xee3553c6, SUC_lexicon_tags_1 },
    { 0x3cd553c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67553d0, SUC_lexicon_tags_12 },
    { 0xc5b553d1, SUC_lexicon_tags_12 },
    { 0xa89553d2, SUC_lexicon_tags_63 },
    { 0x1fd553d3, SUC_lexicon_tags_25 },
    { 0xc8f553d4, SUC_lexicon_tags_15 },
    { 0x589553d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x473553d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x553553d9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf553de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f553e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d553e9, SUC_lexicon_tags_17 },
    { 0x653553e9, SUC_lexicon_tags_2 },
    { 0xa63553ea, SUC_lexicon_tags_5 },
    { 0xb8d553ec, SUC_lexicon_tags_23 },
    { 0x97f553eb, SUC_lexicon_tags_25 },
    { 0x8b5553e9, SUC_lexicon_tags_15 },
    { 0x807553ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3553f3, SUC_lexicon_tags_12 },
    { 0xbad553f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x38f553f6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x729553f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb553fb, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x321553fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f55403, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed5540e, SUC_lexicon_tags_65 },
    { 0x30f5540f, SUC_lexicon_tags_2 },
    { 0xd2155410, SUC_lexicon_tags_7 },
    { 0x3cf55411, SUC_lexicon_tags_10 },
    { 0xf8555412, SUC_lexicon_tags_7 },
    { 0xfc955412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255541a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb295541c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x4135541e, SUC_lexicon_tags_19 },
    { 0x2695541e, SUC_lexicon_tags_1 },
    { 0x1b555420, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0955423, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e355428, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae955430, SUC_lexicon_tags_12 },
    { 0xb1355431, SUC_lexicon_tags_53 },
    { 0x47955432, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x28755435, SUC_lexicon_tags_19 },
    { 0x63555436, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb5543b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dd5543d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b55543f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81755446, SUC_lexicon_tags_43 },
    { 0x50755447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b5544b, SUC_lexicon_tags_57 },
    { 0xe435544c, SUC_lexicon_tags_23 },
    { 0xdd95544c, SUC_lexicon_tags_63 },
    { 0x1c95544e, SUC_lexicon_tags_12 },
    { 0x9bd5544f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74755453, SUC_lexicon_tags_27 },
    { 0x4a555453, SUC_lexicon_tags_27 },
    { 0xf1355455, SUC_lexicon_tags_12 },
    { 0xcef55455, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7755458, SUC_lexicon_tags_6 },
    { 0x4b155459, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c15545b, SUC_lexicon_tags_15 },
    { 0x9255545c, SUC_lexicon_tags_26 },
    { 0x7515545c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77555463, SUC_lexicon_tags_2 },
    { 0x69f55464, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc35546f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d555473, SUC_lexicon_tags_11 },
    { 0xfe155474, SUC_lexicon_tags_4 },
    { 0x5f355474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df55479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d95547e, SUC_lexicon_tags_28 },
    { 0x9e35547e, SUC_lexicon_tags_12 },
    { 0x74355480, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3755486, SUC_lexicon_tags_2 },
    { 0x3f355487, SUC_lexicon_tags_34 },
    { 0xae555487, SUC_lexicon_tags_2 },
    { 0xbef55489, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e75548b, SUC_lexicon_tags_47 },
    { 0x1015548b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d5548f, SUC_lexicon_tags_11 },
    { 0xd0d55490, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c355494, SUC_lexicon_tags_7 },
    { 0x2d755494, SUC_lexicon_tags_49 },
    { 0xb0955496, SUC_lexicon_tags_12 },
    { 0x80555496, SUC_lexicon_tags_25 },
    { 0xed155498, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5175549b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f5549e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85554a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d554a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x433554a9, SUC_lexicon_tags_26 },
    { 0xb21554aa, SUC_lexicon_tags_2 },
    { 0x2a7554ab, SUC_lexicon_tags_12 },
    { 0x64f554ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f554b1, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7554b5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309554ba, SUC_lexicon_tags_62 },
    { 0xc11554bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x43554bd, SUC_lexicon_tags_0 },
    { 0xa43554be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x837554c0, SUC_lexicon_tags_29 },
    { 0x7d1554c1, SUC_lexicon_tags_1 },
    { 0xfcb554c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x773554c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f554c7, SUC_lexicon_tags_0 },
    { 0xd93554c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb554cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851554d7, SUC_lexicon_tags_27 },
    { 0x309554d8, SUC_lexicon_tags_38 },
    { 0x1ad554d9, SUC_lexicon_tags_0 },
    { 0x849554d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x103554dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa25554de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f554e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad3554e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97554ec, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x561554ee, SUC_lexicon_tags_66 },
    { 0xbd9554ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5554f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55554f9, SUC_lexicon_tags_55 },
    { 0x5c7554fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf554fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x159554ff, SUC_lexicon_tags_7 },
    { 0xa2155500, SUC_lexicon_tags_39 },
    { 0xd155501, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x54355503, SUC_lexicon_tags_47 },
    { 0x5bb55504, SUC_lexicon_tags_63 },
    { 0xc7955505, SUC_lexicon_tags_21 },
    { 0x99b55505, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd755510, SUC_lexicon_tags_30 },
    { 0x89955511, SUC_lexicon_tags_29 },
    { 0xe1b55512, SUC_lexicon_tags_7 },
    { 0x14155513, SUC_lexicon_tags_43 },
    { 0x1fd55512, SUC_lexicon_tags_26 },
    { 0xb2d55515, SUC_lexicon_tags_48 },
    { 0x29355516, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1315551c, SUC_lexicon_tags_66 },
    { 0x43b5551d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f55522, SUC_lexicon_tags_1 },
    { 0x3b55523, SUC_lexicon_tags_37 },
    { 0xf6f55524, SUC_lexicon_tags_45 },
    { 0xf5155522, SUC_lexicon_tags_30 },
    { 0xd2b55525, SUC_lexicon_tags_10 },
    { 0xc5955523, SUC_lexicon_tags_8 },
    { 0x5d355528, SUC_lexicon_tags_80 },
    { 0xa0d55529, SUC_lexicon_tags_1 },
    { 0x6e35552a, SUC_lexicon_tags_56 },
    { 0xf195552b, SUC_lexicon_tags_37 },
    { 0x7af5552c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9815552e, SUC_lexicon_tags_15 },
    { 0x1d75552f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4555532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95155536, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9215553b, SUC_lexicon_tags_17 },
    { 0x23d5553b, SUC_lexicon_tags_60 },
    { 0x5c95553c, SUC_lexicon_tags_931 },
    { 0x6d35553d, SUC_lexicon_tags_3 },
    { 0x635553f, SUC_lexicon_tags_7 },
    { 0x74755540, SUC_lexicon_tags_43 },
    { 0xbc755541, SUC_lexicon_tags_9 },
    { 0x70955542, SUC_lexicon_tags_26 },
    { 0x35b55543, SUC_lexicon_tags_15 },
    { 0x4055553b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33755547, SUC_lexicon_tags_44 },
    { 0xcd55548, SUC_lexicon_tags_43 },
    { 0x38555547, SUC_lexicon_tags_11 },
    { 0x56f55547, SUC_lexicon_tags_19 },
    { 0x2a95554b, SUC_lexicon_tags_77 },
    { 0xd7f5554c, SUC_lexicon_tags_12 },
    { 0xea15554d, SUC_lexicon_tags_26 },
    { 0xe9955548, SUC_lexicon_tags_7 },
    { 0xb5554f, SUC_lexicon_tags_45 },
    { 0x4d955550, SUC_lexicon_tags_108 },
    { 0x10155551, SUC_lexicon_tags_11 },
    { 0xf555554e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd155557, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe975555a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2f95555c, SUC_lexicon_tags_170 },
    { 0x4195555d, SUC_lexicon_tags_2 },
    { 0x7ab5555e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3955566, SUC_lexicon_tags_3 },
    { 0xdd755566, SUC_lexicon_tags_33 },
    { 0xba555567, SUC_lexicon_tags_47 },
    { 0xe5555569, SUC_lexicon_tags_68 },
    { 0x7a555569, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf75556d, SUC_lexicon_tags_85 },
    { 0xe155556e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4555570, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd4f55572, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbcf55574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b5557f, SUC_lexicon_tags_11 },
    { 0xeff5557f, SUC_lexicon_tags_16 },
    { 0xd6355581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc155587, SUC_lexicon_tags_10 },
    { 0xdad55588, SUC_lexicon_tags_45 },
    { 0x50155588, SUC_lexicon_tags_18 },
    { 0x2ad55588, SUC_lexicon_tags_29 },
    { 0xe9f5558b, SUC_lexicon_tags_3 },
    { 0x6b5558c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d955596, SUC_lexicon_tags_7 },
    { 0xfef55597, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d5559a, SUC_lexicon_tags_0 },
    { 0xde55559b, SUC_lexicon_tags_10 },
    { 0xfbd5559b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb5559f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d5555a1, SUC_lexicon_tags_37 },
    { 0x643555a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf97555a4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb87555a6, SUC_lexicon_tags_37 },
    { 0xf45555a6, SUC_lexicon_tags_9 },
    { 0x3a1555a8, SUC_lexicon_tags_15 },
    { 0xd69555a9, SUC_lexicon_tags_37 },
    { 0xad7555a8, SUC_lexicon_tags_37 },
    { 0x101555aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad555b0, SUC_lexicon_tags_14 },
    { 0x76b555b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x457555b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc6d555b5, SUC_lexicon_tags_19 },
    { 0x1b9555b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9555b9, SUC_lexicon_tags_40 },
    { 0xa09555ba, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb555bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x385555c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf555c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x223555c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9555cc, SUC_lexicon_tags_11 },
    { 0xfbb555cd, SUC_lexicon_tags_37 },
    { 0xbbf555ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d555de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3555e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd555e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1555e8, SUC_lexicon_tags_37 },
    { 0x9bd555e9, SUC_lexicon_tags_25 },
    { 0x3eb555e8, SUC_lexicon_tags_0 },
    { 0x7ad555e9, SUC_lexicon_tags_17 },
    { 0x161555ec, SUC_lexicon_tags_9 },
    { 0xbbd555ec, SUC_lexicon_tags_7 },
    { 0x145555e9, SUC_lexicon_tags_29 },
    { 0xe95555ef, SUC_lexicon_tags_11 },
    { 0x6d555e8, SUC_lexicon_tags_37 },
    { 0xad555ef, SUC_lexicon_tags_47 },
    { 0xa8b555f0, SUC_lexicon_tags_37 },
    { 0x515555f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9555f6, SUC_lexicon_tags_15 },
    { 0x8c7555f7, SUC_lexicon_tags_9 },
    { 0xee7555f7, SUC_lexicon_tags_14 },
    { 0xeef555f7, SUC_lexicon_tags_37 },
    { 0x20b555fa, SUC_lexicon_tags_17 },
    { 0x8b1555fa, SUC_lexicon_tags_17 },
    { 0xf45555f9, SUC_lexicon_tags_2 },
    { 0x3d1555fd, SUC_lexicon_tags_10 },
    { 0xcb9555fd, SUC_lexicon_tags_37 },
    { 0xa9f555ff, SUC_lexicon_tags_14 },
    { 0x37555600, SUC_lexicon_tags_55 },
    { 0x69d555ff, SUC_lexicon_tags_27 },
    { 0x5e555602, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9155605, SUC_lexicon_tags_25 },
    { 0x35f55605, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2d755608, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7b5560a, SUC_lexicon_tags_30 },
    { 0x9b95560b, SUC_lexicon_tags_37 },
    { 0x8e15560a, SUC_lexicon_tags_15 },
    { 0x7635560d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1695560f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74555613, SUC_lexicon_tags_15 },
    { 0xd7f55614, SUC_lexicon_tags_25 },
    { 0x17b55614, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9f555617, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x92955619, SUC_lexicon_tags_13 },
    { 0x79f5561a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8f5561c, SUC_lexicon_tags_11 },
    { 0x97f5561d, SUC_lexicon_tags_11 },
    { 0xd7d5561c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b55621, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xdd955623, SUC_lexicon_tags_23 },
    { 0x99b55624, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e955626, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xae955628, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xa275562b, SUC_lexicon_tags_43 },
    { 0x395562c, SUC_lexicon_tags_23 },
    { 0x8b55562c, SUC_lexicon_tags_49 },
    { 0x4095562e, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4155631, SUC_lexicon_tags_47 },
    { 0x51755632, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d55637, SUC_lexicon_tags_120 },
    { 0xf6955638, SUC_lexicon_tags_37 },
    { 0xd7f55637, SUC_lexicon_tags_48 },
    { 0xae155638, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf755640, SUC_lexicon_tags_31 },
    { 0x76b55641, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fb55643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41555646, SUC_lexicon_tags_62 },
    { 0x89f55647, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c15564a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b55651, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x77755653, SUC_lexicon_tags_6 },
    { 0xbbd55654, SUC_lexicon_tags_17 },
    { 0x64555654, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42555658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5095565a, SUC_lexicon_tags_17 },
    { 0x6d75565b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81b5565d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6815565f, SUC_lexicon_tags_17 },
    { 0x78955660, SUC_lexicon_tags_1 },
    { 0x6f755661, SUC_lexicon_tags_43 },
    { 0x58555661, SUC_lexicon_tags_37 },
    { 0xaf155663, SUC_lexicon_tags_37 },
    { 0xdd55664, SUC_lexicon_tags_87 },
    { 0xb955665, SUC_lexicon_tags_123 },
    { 0x5c955666, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f55669, SUC_lexicon_tags_19 },
    { 0xda35566a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c75566f, SUC_lexicon_tags_23 },
    { 0x51555670, SUC_lexicon_tags_7 },
    { 0x48b5566f, SUC_lexicon_tags_14 },
    { 0x7b955672, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b55675, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd55678, SUC_lexicon_tags_2 },
    { 0x36155679, SUC_lexicon_tags_0 },
    { 0xc6f5567a, SUC_lexicon_tags_28 },
    { 0xb795567a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc755567f, SUC_lexicon_tags_15 },
    { 0xae75567f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf55685, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b555687, SUC_lexicon_tags_12 },
    { 0x59f55688, SUC_lexicon_tags_25 },
    { 0xd5355687, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2195568b, SUC_lexicon_tags_45 },
    { 0x6cf5568c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff55696, SUC_lexicon_tags_7 },
    { 0x1e355697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb5569a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xccb5569c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b556a3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89556a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647556ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17556b0, SUC_lexicon_tags_25 },
    { 0x2f7556b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa3556b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5556b6, SUC_lexicon_tags_12 },
    { 0xad7556b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa7d556b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33b556bb, SUC_lexicon_tags_1 },
    { 0x6f5556bb, SUC_lexicon_tags_17 },
    { 0xcbb556bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851556c1, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xd11556c3, SUC_lexicon_tags_42 },
    { 0x371556c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375556c8, SUC_lexicon_tags_45 },
    { 0x68f556c9, SUC_lexicon_tags_14 },
    { 0x549556ca, SUC_lexicon_tags_2 },
    { 0x17556cb, SUC_lexicon_tags_2 },
    { 0xe33556cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f556d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617556d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2d556d9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc89556db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f556df, SUC_lexicon_tags_45 },
    { 0x7b7556e0, SUC_lexicon_tags_27 },
    { 0xaa7556df, SUC_lexicon_tags_27 },
    { 0x563556e2, SUC_lexicon_tags_19 },
    { 0x503556e2, SUC_lexicon_tags_25 },
    { 0xcab556e2, SUC_lexicon_tags_14 },
    { 0xff1556e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b556ec, SUC_lexicon_tags_11 },
    { 0x571556ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15b556ef, SUC_lexicon_tags_25 },
    { 0x205556f0, SUC_lexicon_tags_34 },
    { 0x62f556f0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121556f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73556fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x513556fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x53355700, SUC_lexicon_tags_11 },
    { 0xb7555701, SUC_lexicon_tags_14 },
    { 0x97f55702, SUC_lexicon_tags_47 },
    { 0xd8d55702, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72355705, SUC_lexicon_tags_2 },
    { 0xebd55706, SUC_lexicon_tags_32 },
    { 0xa4355706, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc355709, SUC_lexicon_tags_7 },
    { 0x93d5570a, SUC_lexicon_tags_42 },
    { 0xa15570b, SUC_lexicon_tags_60 },
    { 0x5bd5570a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa955570e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b755716, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10555718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e15571a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f55722, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60f55724, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x85955726, SUC_lexicon_tags_45 },
    { 0xb955727, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe95572d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb55730, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f755732, SUC_lexicon_tags_38 },
    { 0xdfd55733, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4fb55735, SUC_lexicon_tags_25 },
    { 0xce355735, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x38755738, SUC_lexicon_tags_26 },
    { 0x6e355738, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66155741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa355743, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54155747, SUC_lexicon_tags_38 },
    { 0x7ab55748, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa295574a, SUC_lexicon_tags_28 },
    { 0x72b5574b, SUC_lexicon_tags_37 },
    { 0x19b5574c, SUC_lexicon_tags_38 },
    { 0xab5574d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdb55574f, SUC_lexicon_tags_39 },
    { 0x2ed55750, SUC_lexicon_tags_1 },
    { 0x3b955751, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad955755, SUC_lexicon_tags_9 },
    { 0x82d55756, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3375575a, SUC_lexicon_tags_37 },
    { 0x97d5575b, SUC_lexicon_tags_9 },
    { 0xb055575b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x16d5575e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10355761, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7515576a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f95576c, SUC_lexicon_tags_32 },
    { 0x5a35576c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cb5576f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x97155772, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f55775, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5075577b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb5577f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f355782, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3555784, SUC_lexicon_tags_14 },
    { 0x73b55785, SUC_lexicon_tags_1 },
    { 0xf0355786, SUC_lexicon_tags_0 },
    { 0xd3355787, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf135578b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa95578d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb55791, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f55794, SUC_lexicon_tags_4 },
    { 0xf6b55795, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8755798, SUC_lexicon_tags_60 },
    { 0x12b55799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f5579c, SUC_lexicon_tags_0 },
    { 0xd095579d, SUC_lexicon_tags_30 },
    { 0x18b5579e, SUC_lexicon_tags_6 },
    { 0xcc35579f, SUC_lexicon_tags_2 },
    { 0x945557a0, SUC_lexicon_tags_39 },
    { 0xd21557a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff557a5, SUC_lexicon_tags_32 },
    { 0x2c9557a5, SUC_lexicon_tags_37 },
    { 0x5b557a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf27557a9, SUC_lexicon_tags_46 },
    { 0x7a3557aa, SUC_lexicon_tags_43 },
    { 0xbe7557aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a1557ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b557b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43557b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3557bc, SUC_lexicon_tags_19 },
    { 0x8fd557bd, SUC_lexicon_tags_7 },
    { 0x96b557be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d557c2, SUC_lexicon_tags_14 },
    { 0x137557c2, SUC_lexicon_tags_31 },
    { 0x851557c4, SUC_lexicon_tags_14 },
    { 0xbb557c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1f557c7, SUC_lexicon_tags_8 },
    { 0xf67557c8, SUC_lexicon_tags_15 },
    { 0x621557c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7557cd, SUC_lexicon_tags_34 },
    { 0xbdb557cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b557d3, SUC_lexicon_tags_15 },
    { 0x483557d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e7557d6, SUC_lexicon_tags_11 },
    { 0x7cb557d6, SUC_lexicon_tags_22 },
    { 0xb27557d8, SUC_lexicon_tags_2 },
    { 0xc91557d9, SUC_lexicon_tags_15 },
    { 0x8a7557d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x919557dd, SUC_lexicon_tags_11 },
    { 0xff1557de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5557e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad557e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc99557e9, SUC_lexicon_tags_695 },
    { 0x79557ea, SUC_lexicon_tags_25 },
    { 0x78d557eb, SUC_lexicon_tags_60 },
    { 0xaf3557ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1557ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1f7557f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa5f557f3, SUC_lexicon_tags_26 },
    { 0x855557f3, SUC_lexicon_tags_17 },
    { 0x67f557f5, SUC_lexicon_tags_6 },
    { 0x66b557f6, SUC_lexicon_tags_2 },
    { 0x685557f3, SUC_lexicon_tags_11 },
    { 0x92d557f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x41557fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7557ff, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x81955801, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b55805, SUC_lexicon_tags_7 },
    { 0x56b55806, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7755808, SUC_lexicon_tags_33 },
    { 0xbc955809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d5580d, SUC_lexicon_tags_13 },
    { 0x74d5580d, SUC_lexicon_tags_38 },
    { 0x3ab5580d, SUC_lexicon_tags_15 },
    { 0x5a955810, SUC_lexicon_tags_1 },
    { 0xe0d5580e, SUC_lexicon_tags_2 },
    { 0xf475580f, SUC_lexicon_tags_15 },
    { 0xbf5580d, SUC_lexicon_tags_37 },
    { 0x5a555811, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee555816, SUC_lexicon_tags_62 },
    { 0x7db55816, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a15581b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d5581f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf3955821, SUC_lexicon_tags_87 },
    { 0x2d55822, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47155829, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf8d5582b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4d5582d, SUC_lexicon_tags_7 },
    { 0xf735582e, SUC_lexicon_tags_15 },
    { 0xdff5582f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa755832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32155834, SUC_lexicon_tags_4 },
    { 0x19955834, SUC_lexicon_tags_11 },
    { 0x1ed55836, SUC_lexicon_tags_11 },
    { 0xce355837, SUC_lexicon_tags_0 },
    { 0xd1555837, SUC_lexicon_tags_33 },
    { 0xd7d55839, SUC_lexicon_tags_25 },
    { 0xff155839, SUC_lexicon_tags_17 },
    { 0xa315583b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5583e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16755842, SUC_lexicon_tags_32 },
    { 0xb8d55843, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22f55845, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d95584a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b75584e, SUC_lexicon_tags_7 },
    { 0x95584f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8eb55851, SUC_lexicon_tags_2 },
    { 0x51d55851, SUC_lexicon_tags_81 },
    { 0x64f55851, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd555856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6115585a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8815585c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51555860, SUC_lexicon_tags_30 },
    { 0xe0555860, SUC_lexicon_tags_9 },
    { 0xf9d55860, SUC_lexicon_tags_43 },
    { 0x3d155863, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24355865, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb5586b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71755874, SUC_lexicon_tags_7 },
    { 0xe9555875, SUC_lexicon_tags_17 },
    { 0x24b55876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61955879, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef5587c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16d5587e, SUC_lexicon_tags_46 },
    { 0x6d75587e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65d55881, SUC_lexicon_tags_15 },
    { 0x44355881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3955884, SUC_lexicon_tags_49 },
    { 0xa9755885, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb6955887, SUC_lexicon_tags_25 },
    { 0x4f55888, SUC_lexicon_tags_37 },
    { 0x83555887, SUC_lexicon_tags_37 },
    { 0x10d55888, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf5588d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e755891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1355893, SUC_lexicon_tags_2 },
    { 0x8a755894, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x20555896, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9595589a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd5589d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1558a8, SUC_lexicon_tags_14 },
    { 0x3b7558a9, SUC_lexicon_tags_29 },
    { 0x343558a8, SUC_lexicon_tags_1 },
    { 0x857558a9, SUC_lexicon_tags_38 },
    { 0x843558a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1558af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f558b7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59558ba, SUC_lexicon_tags_66 },
    { 0x2c5558bb, SUC_lexicon_tags_18 },
    { 0xf23558bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x565558be, SUC_lexicon_tags_2 },
    { 0x28b558bf, SUC_lexicon_tags_57 },
    { 0x1b9558c0, SUC_lexicon_tags_7 },
    { 0x433558c1, SUC_lexicon_tags_2 },
    { 0x59f558c2, SUC_lexicon_tags_25 },
    { 0x7e7558c3, SUC_lexicon_tags_0 },
    { 0xcd1558c2, SUC_lexicon_tags_15 },
    { 0x49d558c1, SUC_lexicon_tags_1 },
    { 0xe2f558c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03558cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9558d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b558d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x355558d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb558db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f558e0, SUC_lexicon_tags_0 },
    { 0x927558e1, SUC_lexicon_tags_2 },
    { 0x519558e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d5558e4, SUC_lexicon_tags_17 },
    { 0xae3558e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1558eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af558f0, SUC_lexicon_tags_24 },
    { 0xf71558f0, SUC_lexicon_tags_45 },
    { 0x5a9558f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b558f5, SUC_lexicon_tags_63 },
    { 0x775558f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25558f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963558fd, SUC_lexicon_tags_37 },
    { 0xcad558fe, SUC_lexicon_tags_49 },
    { 0x21b558fe, SUC_lexicon_tags_30 },
    { 0x63555900, SUC_lexicon_tags_25 },
    { 0xdf5558ff, SUC_lexicon_tags_12 },
    { 0xfa3558fd, SUC_lexicon_tags_17 },
    { 0x8755900, SUC_lexicon_tags_45 },
    { 0x13d55904, SUC_lexicon_tags_37 },
    { 0x72955905, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc35590a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5095590e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7155914, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff55919, SUC_lexicon_tags_7 },
    { 0xe6b5591a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f5591d, SUC_lexicon_tags_60 },
    { 0xe3f5591e, SUC_lexicon_tags_7 },
    { 0x7235591f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc955921, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64555925, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f55592c, SUC_lexicon_tags_19 },
    { 0x7255592d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df55932, SUC_lexicon_tags_17 },
    { 0xc7955933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb55937, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x55555939, SUC_lexicon_tags_11 },
    { 0x92755939, SUC_lexicon_tags_10 },
    { 0x4e35593b, SUC_lexicon_tags_7 },
    { 0x8e35593b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f55943, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66955947, SUC_lexicon_tags_39 },
    { 0xe1b55948, SUC_lexicon_tags_37 },
    { 0x32755949, SUC_lexicon_tags_15 },
    { 0xe9f5594a, SUC_lexicon_tags_12 },
    { 0xdeb55949, SUC_lexicon_tags_15 },
    { 0xf6d5594c, SUC_lexicon_tags_33 },
    { 0xfa15594d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0155950, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x99755953, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x90755956, SUC_lexicon_tags_3 },
    { 0x8ff55957, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc95595b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d5595e, SUC_lexicon_tags_19 },
    { 0x72f5595e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf55966, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7575596d, SUC_lexicon_tags_38 },
    { 0x3c75596e, SUC_lexicon_tags_12 },
    { 0x9fd5596e, SUC_lexicon_tags_11 },
    { 0x13155970, SUC_lexicon_tags_22 },
    { 0x76355971, SUC_lexicon_tags_0 },
    { 0x7f5596f, SUC_lexicon_tags_27 },
    { 0x20155973, SUC_lexicon_tags_2 },
    { 0xf9d55973, SUC_lexicon_tags_16 },
    { 0x33f55975, SUC_lexicon_tags_48 },
    { 0x4155976, SUC_lexicon_tags_25 },
    { 0x83955973, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4955980, SUC_lexicon_tags_37 },
    { 0x39155981, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e155987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd5598f, SUC_lexicon_tags_2 },
    { 0x67f5598f, SUC_lexicon_tags_23 },
    { 0x6b955991, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f559a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd7559a2, SUC_lexicon_tags_17 },
    { 0xed1559a2, SUC_lexicon_tags_15 },
    { 0xefb559a4, SUC_lexicon_tags_1 },
    { 0xd97559a5, SUC_lexicon_tags_9 },
    { 0xb89559a4, SUC_lexicon_tags_30 },
    { 0x2d5559a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e1559a9, SUC_lexicon_tags_4 },
    { 0xd61559aa, SUC_lexicon_tags_37 },
    { 0x87559a9, SUC_lexicon_tags_1 },
    { 0xf7559aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x841559ae, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d559b3, SUC_lexicon_tags_19 },
    { 0xf9559b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x757559b6, SUC_lexicon_tags_18 },
    { 0xbe5559b7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x139559ba, SUC_lexicon_tags_35 },
    { 0xa0b559ba, SUC_lexicon_tags_2 },
    { 0x3f3559bc, SUC_lexicon_tags_16 },
    { 0x1cf559bd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x747559c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9559c3, SUC_lexicon_tags_7 },
    { 0xfa7559c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683559c8, SUC_lexicon_tags_25 },
    { 0xf13559c8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x8b559cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1559cf, SUC_lexicon_tags_14 },
    { 0xa6d559cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x941559d2, SUC_lexicon_tags_49 },
    { 0x9f3559d3, SUC_lexicon_tags_17 },
    { 0xfb3559d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x313559d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9559d9, SUC_lexicon_tags_30 },
    { 0xc99559d9, SUC_lexicon_tags_56 },
    { 0x88f559da, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95559de, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3559e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c1559e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5559ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221559ee, SUC_lexicon_tags_7 },
    { 0x221559ef, SUC_lexicon_tags_30 },
    { 0x8ef559ee, SUC_lexicon_tags_0 },
    { 0x937559ef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9559f4, SUC_lexicon_tags_28 },
    { 0x543559f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff559f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7559fd, SUC_lexicon_tags_60 },
    { 0xcff559fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d55a06, SUC_lexicon_tags_17 },
    { 0xbf355a07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a955a09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28955a0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d55a11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa955a13, SUC_lexicon_tags_10 },
    { 0xcdf55a14, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9355a1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d555a1d, SUC_lexicon_tags_19 },
    { 0x6bf55a1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f555a23, SUC_lexicon_tags_37 },
    { 0xe4d55a24, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d55a28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1355a2b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9955a2e, SUC_lexicon_tags_21 },
    { 0x2d55a2f, SUC_lexicon_tags_82 },
    { 0x1ab55a30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66d55a32, SUC_lexicon_tags_74 },
    { 0xcb55a32, SUC_lexicon_tags_15 },
    { 0xf155a33, SUC_lexicon_tags_19 },
    { 0x11d55a34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd55a39, SUC_lexicon_tags_16 },
    { 0xdc955a3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f55a43, SUC_lexicon_tags_61 },
    { 0xc6155a44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28355a46, SUC_lexicon_tags_25 },
    { 0x40f55a46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f55a4b, SUC_lexicon_tags_27 },
    { 0x83d55a4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9955a51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe6d55a53, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd55a56, SUC_lexicon_tags_47 },
    { 0xa3555a56, SUC_lexicon_tags_55 },
    { 0x7d355a58, SUC_lexicon_tags_17 },
    { 0x20555a58, SUC_lexicon_tags_7 },
    { 0x5dd55a5a, SUC_lexicon_tags_30 },
    { 0x8355a5b, SUC_lexicon_tags_37 },
    { 0xd8955a58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd55a64, SUC_lexicon_tags_65 },
    { 0x2d555a64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f55a6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b55a6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc755a70, SUC_lexicon_tags_2 },
    { 0x53355a71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd55a75, SUC_lexicon_tags_19 },
    { 0xaff55a76, SUC_lexicon_tags_47 },
    { 0x1f555a75, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1fd55a79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb155a7b, SUC_lexicon_tags_7 },
    { 0x13155a7c, SUC_lexicon_tags_27 },
    { 0x2155a7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5355a80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92955a87, SUC_lexicon_tags_31 },
    { 0xe5555a87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2155a8a, SUC_lexicon_tags_15 },
    { 0xf9b55a8b, SUC_lexicon_tags_6 },
    { 0xdc955a8c, SUC_lexicon_tags_44 },
    { 0xe1f55a8d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae955a8f, SUC_lexicon_tags_25 },
    { 0x6f955a8f, SUC_lexicon_tags_43 },
    { 0x32755a8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d55a99, SUC_lexicon_tags_15 },
    { 0xaaf55a9a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f55a9e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d55aa1, SUC_lexicon_tags_12 },
    { 0xe5555aa2, SUC_lexicon_tags_92 },
    { 0xa3355aa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43b55aa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32555aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39955aaa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x59955aac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b55ab0, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d55abd, SUC_lexicon_tags_16 },
    { 0x62f55abe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3555ac4, SUC_lexicon_tags_0 },
    { 0x21d55ac5, SUC_lexicon_tags_14 },
    { 0xa7755ac6, SUC_lexicon_tags_12 },
    { 0x6ff55ac7, SUC_lexicon_tags_37 },
    { 0x1b755ac4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32b55aca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b55ace, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab55ad1, SUC_lexicon_tags_6 },
    { 0xf9f55ad2, SUC_lexicon_tags_29 },
    { 0x56155ad2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde555ad6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe6355ad8, SUC_lexicon_tags_7 },
    { 0x30d55ad9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45555adb, SUC_lexicon_tags_28 },
    { 0xf0f55adc, SUC_lexicon_tags_0 },
    { 0x48955adc, SUC_lexicon_tags_43 },
    { 0x9e355ade, SUC_lexicon_tags_43 },
    { 0xa1955ade, SUC_lexicon_tags_15 },
    { 0xe555ae0, SUC_lexicon_tags_85 },
    { 0xdcb55adf, SUC_lexicon_tags_27 },
    { 0x44555ae2, SUC_lexicon_tags_17 },
    { 0x42f55ae3, SUC_lexicon_tags_98 },
    { 0x58f55ae1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17f55ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb555ae9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x88955aeb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d55aee, SUC_lexicon_tags_26 },
    { 0xeed55aee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c955af7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37155af9, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x14955afb, SUC_lexicon_tags_2 },
    { 0x35555afb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f55b00, SUC_lexicon_tags_7 },
    { 0xa4f55b01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4755b03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x35155b05, SUC_lexicon_tags_48 },
    { 0x4ad55b05, SUC_lexicon_tags_11 },
    { 0x7b755b07, SUC_lexicon_tags_38 },
    { 0x19f55b07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d55b0d, SUC_lexicon_tags_0 },
    { 0x14f55b0e, SUC_lexicon_tags_27 },
    { 0x54d55b0f, SUC_lexicon_tags_40 },
    { 0x6c755b0f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x91755b12, SUC_lexicon_tags_45 },
    { 0xc2955b13, SUC_lexicon_tags_14 },
    { 0x22955b14, SUC_lexicon_tags_23 },
    { 0xc3555b15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71155b1d, SUC_lexicon_tags_33 },
    { 0xfdf55b1e, SUC_lexicon_tags_17 },
    { 0xc7555b1e, SUC_lexicon_tags_77 },
    { 0x7b55b20, SUC_lexicon_tags_12 },
    { 0x6c355b20, SUC_lexicon_tags_6 },
    { 0xc0955b22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8355b26, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd55b2a, SUC_lexicon_tags_12 },
    { 0xf955b2a, SUC_lexicon_tags_37 },
    { 0x57355b2c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x53755b2e, SUC_lexicon_tags_30 },
    { 0x71355b2f, SUC_lexicon_tags_27 },
    { 0xea555b30, SUC_lexicon_tags_37 },
    { 0xad955b30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb355b37, SUC_lexicon_tags_55 },
    { 0xf9155b38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d755b3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7555b3f, SUC_lexicon_tags_12 },
    { 0x12f55b40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e555b46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90355b48, SUC_lexicon_tags_63 },
    { 0x57955b49, SUC_lexicon_tags_25 },
    { 0x41155b4a, SUC_lexicon_tags_30 },
    { 0x43955b4a, SUC_lexicon_tags_11 },
    { 0x82755b4a, SUC_lexicon_tags_32 },
    { 0x2bb55b4c, SUC_lexicon_tags_7 },
    { 0x66b55b49, SUC_lexicon_tags_30 },
    { 0x8d955b48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f955b52, SUC_lexicon_tags_7 },
    { 0xecd55b52, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f55b58, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c155b5b, SUC_lexicon_tags_25 },
    { 0x9d755b5b, SUC_lexicon_tags_14 },
    { 0xd755b5c, SUC_lexicon_tags_12 },
    { 0x2cf55b5b, SUC_lexicon_tags_37 },
    { 0xb9755b5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4955b62, SUC_lexicon_tags_37 },
    { 0x92b55b63, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda955b67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8155b6a, SUC_lexicon_tags_11 },
    { 0x40755b6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a555b6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f55b74, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9ed55b76, SUC_lexicon_tags_17 },
    { 0xdc955b77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d55b7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3555b80, SUC_lexicon_tags_15 },
    { 0xb8f55b81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd955b89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3155b8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb55b99, SUC_lexicon_tags_48 },
    { 0xff55b99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe0555b9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2d55b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd55ba4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5155bab, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf55bb6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb55bb9, SUC_lexicon_tags_37 },
    { 0x67f55bba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b555bbc, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x81955bbe, SUC_lexicon_tags_31 },
    { 0x1b355bbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb955bc5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb55bc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b55bcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11355bd0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb555bdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55f55bdd, SUC_lexicon_tags_61 },
    { 0xe1355bde, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3bd55be0, SUC_lexicon_tags_17 },
    { 0xcdb55be0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1755be4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x48155be6, SUC_lexicon_tags_33 },
    { 0x36555be7, SUC_lexicon_tags_26 },
    { 0x7cb55be7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0355bee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33755bf2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x72f55bf4, SUC_lexicon_tags_727 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa155bfa, SUC_lexicon_tags_11 },
    { 0xce355bfb, SUC_lexicon_tags_37 },
    { 0x71355bfb, SUC_lexicon_tags_12 },
    { 0x30955bfc, SUC_lexicon_tags_14 },
    { 0xe955bfe, SUC_lexicon_tags_16 },
    { 0x8d755bff, SUC_lexicon_tags_15 },
    { 0xd9155bfc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9155c08, SUC_lexicon_tags_60 },
    { 0x40b55c08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f55c0f, SUC_lexicon_tags_11 },
    { 0x47d55c0f, SUC_lexicon_tags_49 },
    { 0x60555c11, SUC_lexicon_tags_31 },
    { 0x9c755c12, SUC_lexicon_tags_15 },
    { 0xc4955c13, SUC_lexicon_tags_15 },
    { 0xbd355c14, SUC_lexicon_tags_19 },
    { 0x1f55c14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x89555c18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf55c1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe555c1d, SUC_lexicon_tags_17 },
    { 0xe2b55c1e, SUC_lexicon_tags_2 },
    { 0x9f955c1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd55c25, SUC_lexicon_tags_17 },
    { 0xe7155c26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f55c2a, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6155c2f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd6b55c31, SUC_lexicon_tags_17 },
    { 0xf7f55c31, SUC_lexicon_tags_46 },
    { 0x1f155c32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ab55c35, SUC_lexicon_tags_45 },
    { 0xa3755c36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81955c3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4955c41, SUC_lexicon_tags_45 },
    { 0xd4d55c41, SUC_lexicon_tags_1 },
    { 0x16f55c41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4555c47, SUC_lexicon_tags_14 },
    { 0x3ff55c48, SUC_lexicon_tags_11 },
    { 0xcb355c49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5355c4e, SUC_lexicon_tags_28 },
    { 0x55955c4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7555c58, SUC_lexicon_tags_19 },
    { 0xb4555c59, SUC_lexicon_tags_7 },
    { 0x2f555c59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda555c5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e555c63, SUC_lexicon_tags_19 },
    { 0x4755c64, SUC_lexicon_tags_7 },
    { 0x6d755c65, SUC_lexicon_tags_35 },
    { 0xd7355c66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70755c69, SUC_lexicon_tags_1 },
    { 0xc2355c69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92b55c6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8155c6e, SUC_lexicon_tags_6 },
    { 0xfa955c6f, SUC_lexicon_tags_68 },
    { 0x28d55c70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b955c77, SUC_lexicon_tags_43 },
    { 0xca355c78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98155c7f, SUC_lexicon_tags_29 },
    { 0xde155c7f, SUC_lexicon_tags_14 },
    { 0x7ff55c80, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e155c87, SUC_lexicon_tags_306 },
    { 0x58355c88, SUC_lexicon_tags_2 },
    { 0xc3955c89, SUC_lexicon_tags_14 },
    { 0x40d55c87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0755c8f, SUC_lexicon_tags_63 },
    { 0x7e355c90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47955c92, SUC_lexicon_tags_0 },
    { 0x79555c92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb55c96, SUC_lexicon_tags_38 },
    { 0xfe355c96, SUC_lexicon_tags_6 },
    { 0x66755c98, SUC_lexicon_tags_25 },
    { 0x1fb55c99, SUC_lexicon_tags_10 },
    { 0x55155c96, SUC_lexicon_tags_0 },
    { 0x87755c9b, SUC_lexicon_tags_27 },
    { 0xa1355c9c, SUC_lexicon_tags_39 },
    { 0x3355c98, SUC_lexicon_tags_2 },
    { 0x50755c9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d55ca4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f55caa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87155caf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5c755cb1, SUC_lexicon_tags_12 },
    { 0xcbb55cb2, SUC_lexicon_tags_37 },
    { 0x19555cb2, SUC_lexicon_tags_28 },
    { 0xd8955cb4, SUC_lexicon_tags_31 },
    { 0x8355cb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d55cbd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26355cbf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd755cc3, SUC_lexicon_tags_0 },
    { 0x3c355cc4, SUC_lexicon_tags_2 },
    { 0x5355cc5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e955cc7, SUC_lexicon_tags_2 },
    { 0xf2155cc7, SUC_lexicon_tags_2 },
    { 0xb955cc7, SUC_lexicon_tags_15 },
    { 0x50955cc9, SUC_lexicon_tags_15 },
    { 0xfeb55ccb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f555cce, SUC_lexicon_tags_7 },
    { 0x95155ccf, SUC_lexicon_tags_84 },
    { 0xcf355cd0, SUC_lexicon_tags_7 },
    { 0x48b55cd1, SUC_lexicon_tags_19 },
    { 0x73f55cd2, SUC_lexicon_tags_5 },
    { 0x20b55cce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36755cd8, SUC_lexicon_tags_1 },
    { 0x8955cd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7955cdb, SUC_lexicon_tags_11 },
    { 0xf2f55cdc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b55cdf, SUC_lexicon_tags_27 },
    { 0xe3d55ce0, SUC_lexicon_tags_1 },
    { 0x8d955cdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f55ce5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b55ce8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4355cee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26955cf2, SUC_lexicon_tags_27 },
    { 0x9e755cf3, SUC_lexicon_tags_45 },
    { 0x49955cf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8555cf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa755cfc, SUC_lexicon_tags_27 },
    { 0xac555cfc, SUC_lexicon_tags_15 },
    { 0xaeb55cfc, SUC_lexicon_tags_61 },
    { 0x4f355cfd, SUC_lexicon_tags_12 },
    { 0x1f755d00, SUC_lexicon_tags_0 },
    { 0xf5155d00, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc4555d03, SUC_lexicon_tags_11 },
    { 0x52d55d03, SUC_lexicon_tags_38 },
    { 0x25b55d05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x60355d08, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60155d0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb555d12, SUC_lexicon_tags_7 },
    { 0xf0555d13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f55d18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45955d1b, SUC_lexicon_tags_1 },
    { 0x1d55d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed55d20, SUC_lexicon_tags_16 },
    { 0x41955d21, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x84355d24, SUC_lexicon_tags_12 },
    { 0xd0955d25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f55d28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf755d2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3555d30, SUC_lexicon_tags_17 },
    { 0x8fb55d31, SUC_lexicon_tags_82 },
    { 0x88155d30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b755d35, SUC_lexicon_tags_7 },
    { 0x6b155d35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e555d38, SUC_lexicon_tags_15 },
    { 0xaa355d39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9555d3c, SUC_lexicon_tags_15 },
    { 0xf3155d3d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b55d40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76955d43, SUC_lexicon_tags_48 },
    { 0xa2555d44, SUC_lexicon_tags_65 },
    { 0xe7155d44, SUC_lexicon_tags_27 },
    { 0xbc955d45, SUC_lexicon_tags_47 },
    { 0x67d55d47, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x69955d4a, SUC_lexicon_tags_26 },
    { 0xf4155d4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d155d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f55d5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa5555d5c, SUC_lexicon_tags_7 },
    { 0x5d555d5d, SUC_lexicon_tags_33 },
    { 0x3cd55d5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf55d67, SUC_lexicon_tags_17 },
    { 0x4ff55d68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf55d6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77755d70, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4d155d72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7955d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaff55d78, SUC_lexicon_tags_21 },
    { 0xb6555d79, SUC_lexicon_tags_15 },
    { 0x80555d79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59355d81, SUC_lexicon_tags_25 },
    { 0x90f55d81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4155d84, SUC_lexicon_tags_15 },
    { 0x8f55d84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29955d87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1fd55d89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff155d8d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa8155d8f, SUC_lexicon_tags_37 },
    { 0xdf755d8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc155d93, SUC_lexicon_tags_29 },
    { 0xd8955d94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d55d97, SUC_lexicon_tags_12 },
    { 0x77155d98, SUC_lexicon_tags_45 },
    { 0x6ed55d98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb555d9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x64955da1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9755da6, SUC_lexicon_tags_60 },
    { 0xcf955da7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7955dab, SUC_lexicon_tags_31 },
    { 0x25955dac, SUC_lexicon_tags_17 },
    { 0xd6555dad, SUC_lexicon_tags_43 },
    { 0x18755dae, SUC_lexicon_tags_45 },
    { 0x75755daf, SUC_lexicon_tags_33 },
    { 0x5b55db0, SUC_lexicon_tags_32 },
    { 0x2e955dad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15755dbd, SUC_lexicon_tags_9 },
    { 0xe4955dbd, SUC_lexicon_tags_37 },
    { 0xcf555dbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5755dc5, SUC_lexicon_tags_33 },
    { 0x6bb55dc6, SUC_lexicon_tags_1 },
    { 0xaab55dc5, SUC_lexicon_tags_33 },
    { 0x26b55dc8, SUC_lexicon_tags_25 },
    { 0x17b55dc9, SUC_lexicon_tags_7 },
    { 0x8d355dc9, SUC_lexicon_tags_68 },
    { 0x3dd55dcb, SUC_lexicon_tags_7 },
    { 0x58755dcb, SUC_lexicon_tags_97 },
    { 0x3e155dcd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97555dcf, SUC_lexicon_tags_15 },
    { 0x46355dcf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4555dd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5af55dd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12955dd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x46b55dda, SUC_lexicon_tags_4 },
    { 0x55f55ddb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x92b55ddd, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f55de4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdcf55de6, SUC_lexicon_tags_17 },
    { 0x95d55de6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23955de9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4d755deb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ed55ded, SUC_lexicon_tags_12 },
    { 0x9d155dee, SUC_lexicon_tags_60 },
    { 0x8c155dee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xda555df1, SUC_lexicon_tags_17 },
    { 0x1955df2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d755df5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f55dfc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92955e01, SUC_lexicon_tags_7 },
    { 0xe9555e02, SUC_lexicon_tags_2 },
    { 0x81355e03, SUC_lexicon_tags_23 },
    { 0xa6d55e04, SUC_lexicon_tags_158 },
    { 0x1a755e05, SUC_lexicon_tags_25 },
    { 0x9cd55e06, SUC_lexicon_tags_2 },
    { 0x4b555e05, SUC_lexicon_tags_7 },
    { 0x32155e08, SUC_lexicon_tags_15 },
    { 0x65155e09, SUC_lexicon_tags_37 },
    { 0x17b55e07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef55e0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f55e10, SUC_lexicon_tags_25 },
    { 0x9ab55e11, SUC_lexicon_tags_23 },
    { 0x9a955e11, SUC_lexicon_tags_15 },
    { 0x22555e13, SUC_lexicon_tags_37 },
    { 0x53b55e14, SUC_lexicon_tags_14 },
    { 0x3a755e15, SUC_lexicon_tags_17 },
    { 0x36d55e15, SUC_lexicon_tags_10 },
    { 0x56b55e17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7955e1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5355e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e955e28, SUC_lexicon_tags_32 },
    { 0x76f55e28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26755e2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cb55e2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf155e31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f55e34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41355e38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c355e41, SUC_lexicon_tags_45 },
    { 0xde955e41, SUC_lexicon_tags_5 },
    { 0xf0b55e42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b55e47, SUC_lexicon_tags_47 },
    { 0xca955e48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17555e4a, SUC_lexicon_tags_7 },
    { 0xb6d55e4b, SUC_lexicon_tags_25 },
    { 0xed155e4b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90955e50, SUC_lexicon_tags_1 },
    { 0x97955e51, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd55e55, SUC_lexicon_tags_37 },
    { 0x9ef55e56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xff355e58, SUC_lexicon_tags_0 },
    { 0x3bf55e58, SUC_lexicon_tags_49 },
    { 0x10355e5a, SUC_lexicon_tags_25 },
    { 0x88355e5b, SUC_lexicon_tags_106 },
    { 0x38b55e5c, SUC_lexicon_tags_1 },
    { 0x5af55e5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2555e63, SUC_lexicon_tags_12 },
    { 0xe1355e63, SUC_lexicon_tags_37 },
    { 0x8ed55e65, SUC_lexicon_tags_66 },
    { 0xc1155e66, SUC_lexicon_tags_27 },
    { 0xc5d55e66, SUC_lexicon_tags_25 },
    { 0x4a355e68, SUC_lexicon_tags_28 },
    { 0xefd55e63, SUC_lexicon_tags_23 },
    { 0xe1355e66, SUC_lexicon_tags_12 },
    { 0x2d155e6b, SUC_lexicon_tags_6 },
    { 0x28d55e6b, SUC_lexicon_tags_11 },
    { 0xe6d55e6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x87355e70, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db55e79, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6755e7d, SUC_lexicon_tags_1522 },
    { 0x52355e7e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaff55e80, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf55e85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d55e89, SUC_lexicon_tags_14 },
    { 0x67755e89, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f55e8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21955e93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c155e9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b55ea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x61755ea4, SUC_lexicon_tags_32 },
    { 0x9f155ea4, SUC_lexicon_tags_25 },
    { 0x7c755ea6, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf55eab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9d55ead, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6755eb3, SUC_lexicon_tags_11 },
    { 0x7ed55eb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af55eb7, SUC_lexicon_tags_47 },
    { 0x1b55eb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c955ebc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f55ec1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xab755ec4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53955ec6, SUC_lexicon_tags_55 },
    { 0x4d555ec6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d55ecc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9355ecf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x52355ed1, SUC_lexicon_tags_23 },
    { 0x63355ed1, SUC_lexicon_tags_261 },
    { 0xff755ed1, SUC_lexicon_tags_7 },
    { 0x93755ed4, SUC_lexicon_tags_30 },
    { 0x28355ed5, SUC_lexicon_tags_1 },
    { 0xe4d55ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91355ed8, SUC_lexicon_tags_14 },
    { 0xbc755ed8, SUC_lexicon_tags_11 },
    { 0x8a555ed9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37f55edc, SUC_lexicon_tags_25 },
    { 0x46d55edc, SUC_lexicon_tags_45 },
    { 0xa9d55edd, SUC_lexicon_tags_4 },
    { 0xeef55edf, SUC_lexicon_tags_0 },
    { 0x7b55edf, SUC_lexicon_tags_23 },
    { 0x94355ee1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83955ee3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x75755ee5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9755eeb, SUC_lexicon_tags_0 },
    { 0xc0555eec, SUC_lexicon_tags_23 },
    { 0x78f55eec, SUC_lexicon_tags_19 },
    { 0xabb55eee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc7755ef0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55ef6, SUC_lexicon_tags_27 },
    { 0x40f55ef7, SUC_lexicon_tags_49 },
    { 0x7ad55ef8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe355efe, SUC_lexicon_tags_11 },
    { 0xbe755efe, SUC_lexicon_tags_17 },
    { 0xfa155efe, SUC_lexicon_tags_1 },
    { 0xb5355f01, SUC_lexicon_tags_17 },
    { 0x19f55f02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbe755f04, SUC_lexicon_tags_8 },
    { 0x88755f05, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xce555f07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9d55f09, SUC_lexicon_tags_1 },
    { 0x40155f09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7555f0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59755f11, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfaf55f13, SUC_lexicon_tags_4 },
    { 0xe8f55f13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8155f17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6555f1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b55f1d, SUC_lexicon_tags_43 },
    { 0x9bb55f1e, SUC_lexicon_tags_7 },
    { 0x1dd55f1e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x87955f21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea155f23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcd55f25, SUC_lexicon_tags_2 },
    { 0x8a755f25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a755f28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f55f30, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3355f32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbf55f34, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b55f37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16f55f39, SUC_lexicon_tags_29 },
    { 0x38955f3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d55f3c, SUC_lexicon_tags_77 },
    { 0xcf755f3d, SUC_lexicon_tags_37 },
    { 0x96f55f3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49155f40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b55f44, SUC_lexicon_tags_7 },
    { 0x6c355f44, SUC_lexicon_tags_74 },
    { 0xfad55f46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd55f49, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcb155f4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86755f50, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f55f54, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87755f59, SUC_lexicon_tags_17 },
    { 0x51f55f5a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d55f5e, SUC_lexicon_tags_16 },
    { 0x91155f5e, SUC_lexicon_tags_33 },
    { 0x68555f60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84955f65, SUC_lexicon_tags_49 },
    { 0x56155f66, SUC_lexicon_tags_37 },
    { 0xedd55f67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d55f6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc0d55f6d, SUC_lexicon_tags_63 },
    { 0x7d755f6e, SUC_lexicon_tags_27 },
    { 0x7f55f6f, SUC_lexicon_tags_65 },
    { 0x53155f70, SUC_lexicon_tags_16 },
    { 0x4fb55f71, SUC_lexicon_tags_47 },
    { 0x8ff55f6f, SUC_lexicon_tags_14 },
    { 0x7f555f6e, SUC_lexicon_tags_48 },
    { 0xa3555f71, SUC_lexicon_tags_2 },
    { 0x22555f72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x21955f78, SUC_lexicon_tags_0 },
    { 0x82755f78, SUC_lexicon_tags_56 },
    { 0xabb55f7a, SUC_lexicon_tags_1 },
    { 0x71b55f7b, SUC_lexicon_tags_182 },
    { 0x6ad55f7a, SUC_lexicon_tags_43 },
    { 0x4d555f7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4fd55f7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbcd55f81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a955f85, SUC_lexicon_tags_11 },
    { 0xed755f85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88555f88, SUC_lexicon_tags_15 },
    { 0xc2555f88, SUC_lexicon_tags_12 },
    { 0x87d55f8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75755f8f, SUC_lexicon_tags_49 },
    { 0xbf955f90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x78755f93, SUC_lexicon_tags_1 },
    { 0x4555f93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39555f96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18955f99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd55f9d, SUC_lexicon_tags_26 },
    { 0x77355f9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d755fa0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb955fa2, SUC_lexicon_tags_1 },
    { 0x5f55fa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef755fa9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeeb55fab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd55fb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x59555fb4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6155fb7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97955fbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b55fc1, SUC_lexicon_tags_1 },
    { 0x50f55fc1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5755fca, SUC_lexicon_tags_23 },
    { 0xf2555fcb, SUC_lexicon_tags_49 },
    { 0x14555fcc, SUC_lexicon_tags_15 },
    { 0xe755fcb, SUC_lexicon_tags_36 },
    { 0x80355fcd, SUC_lexicon_tags_27 },
    { 0x22b55fcf, SUC_lexicon_tags_19 },
    { 0x7e955fcf, SUC_lexicon_tags_0 },
    { 0x45b55fd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d55fd5, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69755fd9, SUC_lexicon_tags_37 },
    { 0x74355fda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58355fde, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed55fe4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe1155fe6, SUC_lexicon_tags_1 },
    { 0x60b55fe7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xab55fea, SUC_lexicon_tags_23 },
    { 0x7ef55feb, SUC_lexicon_tags_38 },
    { 0xe4d55fec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb55ff6, SUC_lexicon_tags_33 },
    { 0x5e555ff7, SUC_lexicon_tags_0 },
    { 0xe8d55ff8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75555ffa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20155ffe, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd1d56000, SUC_lexicon_tags_37 },
    { 0x54b56001, SUC_lexicon_tags_66 },
    { 0xc5756001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bf56004, SUC_lexicon_tags_1 },
    { 0xdd956004, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab56008, SUC_lexicon_tags_3 },
    { 0xc2556009, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7835600d, SUC_lexicon_tags_1 },
    { 0x2335600e, SUC_lexicon_tags_5 },
    { 0x42d5600f, SUC_lexicon_tags_25 },
    { 0x5855600d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79956015, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb156018, SUC_lexicon_tags_15 },
    { 0xbc556019, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc635601c, SUC_lexicon_tags_6 },
    { 0x3b35601d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xab156020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcb56022, SUC_lexicon_tags_47 },
    { 0xf7956023, SUC_lexicon_tags_17 },
    { 0x79d56022, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d56028, SUC_lexicon_tags_15 },
    { 0x35356028, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba55602b, SUC_lexicon_tags_30 },
    { 0xa0b5602c, SUC_lexicon_tags_37 },
    { 0x18d5602c, SUC_lexicon_tags_47 },
    { 0x7bd5602e, SUC_lexicon_tags_39 },
    { 0x9655602f, SUC_lexicon_tags_45 },
    { 0x81b56030, SUC_lexicon_tags_54 },
    { 0x2dd56031, SUC_lexicon_tags_6 },
    { 0x83156032, SUC_lexicon_tags_12 },
    { 0xee15602f, SUC_lexicon_tags_11 },
    { 0xd0556034, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf56038, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9995603e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d56046, SUC_lexicon_tags_12 },
    { 0xfe156046, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b5604a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f5604d, SUC_lexicon_tags_9 },
    { 0x4255604e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xc7556050, SUC_lexicon_tags_19 },
    { 0x5d356051, SUC_lexicon_tags_33 },
    { 0x56b56052, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d56055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a356057, SUC_lexicon_tags_25 },
    { 0x92556058, SUC_lexicon_tags_55 },
    { 0x61556058, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a75605c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f5605e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66756062, SUC_lexicon_tags_25 },
    { 0xec956063, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf95606c, SUC_lexicon_tags_55 },
    { 0x7055606d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b35606f, SUC_lexicon_tags_14 },
    { 0xfe75606f, SUC_lexicon_tags_1 },
    { 0x4d356071, SUC_lexicon_tags_1 },
    { 0xe3756072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f756074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7395607a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc535607c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd5607f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d956082, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5356087, SUC_lexicon_tags_9 },
    { 0x8156088, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd5608b, SUC_lexicon_tags_21 },
    { 0x63b5608c, SUC_lexicon_tags_60 },
    { 0x3d75608c, SUC_lexicon_tags_37 },
    { 0xf255608e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c356090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67756095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5609b, SUC_lexicon_tags_2 },
    { 0x39b5609b, SUC_lexicon_tags_65 },
    { 0x2515609d, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x259560a0, SUC_lexicon_tags_12 },
    { 0xf19560a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43560a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef560ad, SUC_lexicon_tags_34 },
    { 0xdb1560ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f3560b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ed560b2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb560b9, SUC_lexicon_tags_1 },
    { 0x8b9560ba, SUC_lexicon_tags_15 },
    { 0x9df560bb, SUC_lexicon_tags_7 },
    { 0x127560bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5560c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x635560c2, SUC_lexicon_tags_38 },
    { 0xa13560c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49560d5, SUC_lexicon_tags_9 },
    { 0x593560d5, SUC_lexicon_tags_1 },
    { 0x951560d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdf560d9, SUC_lexicon_tags_250 },
    { 0x3e9560d9, SUC_lexicon_tags_15 },
    { 0x3c7560db, SUC_lexicon_tags_2 },
    { 0x1e3560dc, SUC_lexicon_tags_25 },
    { 0x7a5560dc, SUC_lexicon_tags_28 },
    { 0x743560dd, SUC_lexicon_tags_40 },
    { 0x65f560de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1560e3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85560e6, SUC_lexicon_tags_2 },
    { 0x5cb560e7, SUC_lexicon_tags_37 },
    { 0x273560e6, SUC_lexicon_tags_57 },
    { 0x86d560e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x665560ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f560f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26f560f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715560f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd7560f9, SUC_lexicon_tags_13 },
    { 0x841560fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b56100, SUC_lexicon_tags_27 },
    { 0x91156101, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65356106, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa195610e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4356111, SUC_lexicon_tags_12 },
    { 0x9356111, SUC_lexicon_tags_17 },
    { 0xd4156111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7356118, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x56f5611a, SUC_lexicon_tags_11 },
    { 0xb695611b, SUC_lexicon_tags_17 },
    { 0x7955611c, SUC_lexicon_tags_1 },
    { 0x37b5611d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11756122, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xff56125, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36356127, SUC_lexicon_tags_2 },
    { 0xe8156128, SUC_lexicon_tags_25 },
    { 0x13156129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb56130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0b56132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb56135, SUC_lexicon_tags_11 },
    { 0x89556136, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc15613a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf15613c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe215613e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x10156141, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad356144, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55156147, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f5614b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b55614d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa195614f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f756154, SUC_lexicon_tags_262 },
    { 0xdab56155, SUC_lexicon_tags_37 },
    { 0xcb756155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a75615a, SUC_lexicon_tags_47 },
    { 0x5015615b, SUC_lexicon_tags_49 },
    { 0xb195615c, SUC_lexicon_tags_32 },
    { 0x2d75615d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ad5615f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b56163, SUC_lexicon_tags_17 },
    { 0x7f356164, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27156167, SUC_lexicon_tags_47 },
    { 0x16f56167, SUC_lexicon_tags_15 },
    { 0xa8d56169, SUC_lexicon_tags_15 },
    { 0x2895616a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaeb5616c, SUC_lexicon_tags_14 },
    { 0x7e95616d, SUC_lexicon_tags_15 },
    { 0x97d5616d, SUC_lexicon_tags_17 },
    { 0xe3f5616f, SUC_lexicon_tags_14 },
    { 0xfb5616f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a556174, SUC_lexicon_tags_5 },
    { 0xec956175, SUC_lexicon_tags_48 },
    { 0xc0356175, SUC_lexicon_tags_5 },
    { 0x84d56177, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b5617b, SUC_lexicon_tags_14 },
    { 0xbc75617b, SUC_lexicon_tags_46 },
    { 0x58f5617c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a556182, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6c156184, SUC_lexicon_tags_59 },
    { 0xb9956185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d56189, SUC_lexicon_tags_37 },
    { 0xc6556189, SUC_lexicon_tags_2 },
    { 0xcc95618b, SUC_lexicon_tags_1 },
    { 0x78156189, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd695618e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x86b56190, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x18156192, SUC_lexicon_tags_1 },
    { 0x61156193, SUC_lexicon_tags_43 },
    { 0xf3756193, SUC_lexicon_tags_49 },
    { 0x87d56195, SUC_lexicon_tags_9 },
    { 0x6f56193, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34556198, SUC_lexicon_tags_11 },
    { 0xd756199, SUC_lexicon_tags_11 },
    { 0x54b5619a, SUC_lexicon_tags_47 },
    { 0x71756199, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7295619d, SUC_lexicon_tags_11 },
    { 0x4a15619d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x929561a0, SUC_lexicon_tags_38 },
    { 0x3dd561a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f561a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xedb561ab, SUC_lexicon_tags_23 },
    { 0x307561ac, SUC_lexicon_tags_11 },
    { 0x54f561ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d561b0, SUC_lexicon_tags_14 },
    { 0x4e7561b1, SUC_lexicon_tags_66 },
    { 0xa79561b2, SUC_lexicon_tags_37 },
    { 0x255561b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79561b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x433561ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f9561bc, SUC_lexicon_tags_12 },
    { 0x43561bc, SUC_lexicon_tags_31 },
    { 0x95b561bc, SUC_lexicon_tags_2 },
    { 0xfb1561bd, SUC_lexicon_tags_15 },
    { 0xb71561c0, SUC_lexicon_tags_1 },
    { 0xe01561bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763561c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b561cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3b561ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc81561d0, SUC_lexicon_tags_37 },
    { 0x675561d1, SUC_lexicon_tags_31 },
    { 0x96f561d2, SUC_lexicon_tags_43 },
    { 0x10d561d2, SUC_lexicon_tags_48 },
    { 0xb19561d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b561d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93b561d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4b9561db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1561df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25561e3, SUC_lexicon_tags_57 },
    { 0xef3561e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x32b561e6, SUC_lexicon_tags_28 },
    { 0x2ad561e6, SUC_lexicon_tags_19 },
    { 0xfc3561e6, SUC_lexicon_tags_15 },
    { 0xbc1561e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd561eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9561ee, SUC_lexicon_tags_77 },
    { 0xfbb561ef, SUC_lexicon_tags_25 },
    { 0x481561ee, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb2f561f2, SUC_lexicon_tags_33 },
    { 0xfd7561f3, SUC_lexicon_tags_33 },
    { 0x4c1561f3, SUC_lexicon_tags_10 },
    { 0x4ef561f5, SUC_lexicon_tags_47 },
    { 0x6eb561f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2a3561f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f561fe, SUC_lexicon_tags_9 },
    { 0xd39561ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f156202, SUC_lexicon_tags_12 },
    { 0x7e756203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73556207, SUC_lexicon_tags_25 },
    { 0xb356208, SUC_lexicon_tags_2 },
    { 0xc3f56209, SUC_lexicon_tags_31 },
    { 0x15b56209, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c95620c, SUC_lexicon_tags_7 },
    { 0x34b5620d, SUC_lexicon_tags_60 },
    { 0x6515620d, SUC_lexicon_tags_28 },
    { 0x8155620e, SUC_lexicon_tags_34 },
    { 0xfc35620e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x44756212, SUC_lexicon_tags_23 },
    { 0x82b56213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f756217, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5156219, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9556221, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f56224, SUC_lexicon_tags_12 },
    { 0x9356225, SUC_lexicon_tags_25 },
    { 0xf4556226, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b5622a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc75622e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1156233, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa756235, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9756239, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b5623d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69556242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54f56244, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb56248, SUC_lexicon_tags_60 },
    { 0x67556248, SUC_lexicon_tags_7 },
    { 0x1f5624a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17756251, SUC_lexicon_tags_37 },
    { 0x6cf56252, SUC_lexicon_tags_12 },
    { 0xb4f56253, SUC_lexicon_tags_37 },
    { 0xa8956253, SUC_lexicon_tags_255 },
    { 0x2d956255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc356259, SUC_lexicon_tags_4 },
    { 0x1615625a, SUC_lexicon_tags_17 },
    { 0x5df5625b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b5625e, SUC_lexicon_tags_17 },
    { 0x6455625e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e156263, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b56266, SUC_lexicon_tags_2 },
    { 0xbe156266, SUC_lexicon_tags_15 },
    { 0xfb756266, SUC_lexicon_tags_55 },
    { 0x54d56269, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x14b5626b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b75626d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2cf5626f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd9956271, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfa156273, SUC_lexicon_tags_30 },
    { 0xd0556274, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe0556276, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf635627a, SUC_lexicon_tags_15 },
    { 0x6495627a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f56281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59956285, SUC_lexicon_tags_12 },
    { 0xad356286, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215628c, SUC_lexicon_tags_31 },
    { 0x2f15628c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb56290, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd56293, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f756295, SUC_lexicon_tags_12 },
    { 0xc6f56296, SUC_lexicon_tags_11 },
    { 0x1b556296, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92556299, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12b5629b, SUC_lexicon_tags_15 },
    { 0x2955629c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa715629f, SUC_lexicon_tags_12 },
    { 0x18d5629f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdef562a2, SUC_lexicon_tags_34 },
    { 0x8d3562a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x513562a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587562aa, SUC_lexicon_tags_1 },
    { 0xf71562ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7562ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45562b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x86d562b8, SUC_lexicon_tags_1 },
    { 0x9d7562b9, SUC_lexicon_tags_11 },
    { 0x9d7562ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60b562bc, SUC_lexicon_tags_7 },
    { 0xa03562bd, SUC_lexicon_tags_63 },
    { 0xd89562be, SUC_lexicon_tags_26 },
    { 0x6bf562bf, SUC_lexicon_tags_60 },
    { 0x603562bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5eb562c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb77562c4, SUC_lexicon_tags_11 },
    { 0xc1d562c5, SUC_lexicon_tags_11 },
    { 0xf5562c6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x403562c9, SUC_lexicon_tags_81 },
    { 0xebd562ca, SUC_lexicon_tags_52 },
    { 0x2dd562cb, SUC_lexicon_tags_12 },
    { 0x3ed562cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb562cf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5c3562d1, SUC_lexicon_tags_28 },
    { 0xa6d562d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf7b562d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd562d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5562dd, SUC_lexicon_tags_65 },
    { 0xc9562dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a9562e0, SUC_lexicon_tags_2 },
    { 0xcd5562e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d562e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71562e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x483562e9, SUC_lexicon_tags_37 },
    { 0x29b562ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca9562ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bb562ee, SUC_lexicon_tags_12 },
    { 0xaed562ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69562f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3562f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bd562fb, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3562fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b356305, SUC_lexicon_tags_38 },
    { 0x69756306, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe156309, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab5630c, SUC_lexicon_tags_9 },
    { 0xaff5630d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7235630f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23956311, SUC_lexicon_tags_45 },
    { 0x14556312, SUC_lexicon_tags_37 },
    { 0xd2d56313, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40756319, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7095631c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4415631e, SUC_lexicon_tags_29 },
    { 0x63f5631f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd2556321, SUC_lexicon_tags_60 },
    { 0xd3556322, SUC_lexicon_tags_47 },
    { 0x5a356322, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22d56325, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc135632f, SUC_lexicon_tags_12 },
    { 0xea756330, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3156333, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46b56335, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x10556338, SUC_lexicon_tags_9 },
    { 0xc1956338, SUC_lexicon_tags_48 },
    { 0x6bb56338, SUC_lexicon_tags_46 },
    { 0x5355633a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b56342, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0, NULL },
    { 0x92356345, SUC_lexicon_tags_12 },
    { 0x356346, SUC_lexicon_tags_11 },
    { 0xe8f56347, SUC_lexicon_tags_15 },
    { 0x66556347, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e75634c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c75634f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbab56351, SUC_lexicon_tags_7 },
    { 0xd5956352, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd956354, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f56357, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf895635a, SUC_lexicon_tags_27 },
    { 0x4bf5635b, SUC_lexicon_tags_39 },
    { 0xd1f5635c, SUC_lexicon_tags_47 },
    { 0x5475635b, SUC_lexicon_tags_29 },
    { 0xd675635b, SUC_lexicon_tags_37 },
    { 0x8bd5635a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b56366, SUC_lexicon_tags_87 },
    { 0x2e356366, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88556369, SUC_lexicon_tags_25 },
    { 0xfb5636a, SUC_lexicon_tags_40 },
    { 0xab5636b, SUC_lexicon_tags_25 },
    { 0x28b56369, SUC_lexicon_tags_55 },
    { 0x5ad5636d, SUC_lexicon_tags_33 },
    { 0x6915636d, SUC_lexicon_tags_1 },
    { 0x6515636f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d56373, SUC_lexicon_tags_0 },
    { 0x47356374, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75f56376, SUC_lexicon_tags_1 },
    { 0xc3d56377, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93956379, SUC_lexicon_tags_26 },
    { 0x2e756379, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8435637e, SUC_lexicon_tags_25 },
    { 0x5f75637e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56356381, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x92156384, SUC_lexicon_tags_52 },
    { 0x70d56385, SUC_lexicon_tags_25 },
    { 0x28756386, SUC_lexicon_tags_25 },
    { 0x3e156387, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x38756389, SUC_lexicon_tags_33 },
    { 0x9af56389, SUC_lexicon_tags_14 },
    { 0x3215638b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7335638f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52356391, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f56394, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa715639d, SUC_lexicon_tags_25 },
    { 0x6355639d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x79d563a0, SUC_lexicon_tags_49 },
    { 0x147563a1, SUC_lexicon_tags_2 },
    { 0xc55563a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5563a4, SUC_lexicon_tags_11 },
    { 0xf01563a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf65563a7, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7563ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7563b3, SUC_lexicon_tags_33 },
    { 0x6d563b4, SUC_lexicon_tags_30 },
    { 0x7ef563b4, SUC_lexicon_tags_37 },
    { 0xd3b563b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x899563b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5563bb, SUC_lexicon_tags_37 },
    { 0x5d3563bc, SUC_lexicon_tags_27 },
    { 0xb75563bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d563c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1563c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1563cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09563d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7563d5, SUC_lexicon_tags_28 },
    { 0x141563d6, SUC_lexicon_tags_56 },
    { 0x989563d6, SUC_lexicon_tags_38 },
    { 0x819563d8, SUC_lexicon_tags_47 },
    { 0x4ed563d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f563dc, SUC_lexicon_tags_66 },
    { 0x149563dd, SUC_lexicon_tags_9 },
    { 0x15f563de, SUC_lexicon_tags_9 },
    { 0x1563de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283563e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2b563e6, SUC_lexicon_tags_63 },
    { 0xb17563e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117563ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67563f1, SUC_lexicon_tags_34 },
    { 0xf9b563f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2eb563f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa5b563f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f563f9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd59563fb, SUC_lexicon_tags_25 },
    { 0x8b9563fc, SUC_lexicon_tags_72 },
    { 0x855563fd, SUC_lexicon_tags_11 },
    { 0x201563fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26356400, SUC_lexicon_tags_38 },
    { 0x1f356401, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b56405, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x14556408, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd755640a, SUC_lexicon_tags_33 },
    { 0xd695640b, SUC_lexicon_tags_1 },
    { 0x5db5640c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab5640f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4956412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2356418, SUC_lexicon_tags_16 },
    { 0x74d56419, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3095641b, SUC_lexicon_tags_17 },
    { 0xf235641c, SUC_lexicon_tags_33 },
    { 0xba75641c, SUC_lexicon_tags_47 },
    { 0x3335641c, SUC_lexicon_tags_12 },
    { 0x9595641f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f56423, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x61956426, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9395642a, SUC_lexicon_tags_63 },
    { 0xddb5642b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6935642d, SUC_lexicon_tags_43 },
    { 0xabf5642e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad956437, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe795643a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87756442, SUC_lexicon_tags_15 },
    { 0x19156443, SUC_lexicon_tags_37 },
    { 0xd0156442, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f5644a, SUC_lexicon_tags_23 },
    { 0x4695644b, SUC_lexicon_tags_60 },
    { 0xa5d5644b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f56450, SUC_lexicon_tags_37 },
    { 0x49f56451, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d56454, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd56459, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4fb5645b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b56460, SUC_lexicon_tags_11 },
    { 0xdaf56461, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d56464, SUC_lexicon_tags_27 },
    { 0xff956465, SUC_lexicon_tags_11 },
    { 0xb9156465, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c956468, SUC_lexicon_tags_83 },
    { 0x99956469, SUC_lexicon_tags_0 },
    { 0x7bf56469, SUC_lexicon_tags_1 },
    { 0x32556469, SUC_lexicon_tags_37 },
    { 0xf935646b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9675646e, SUC_lexicon_tags_37 },
    { 0x955646f, SUC_lexicon_tags_14 },
    { 0x2e95646f, SUC_lexicon_tags_39 },
    { 0x3275646e, SUC_lexicon_tags_29 },
    { 0xc8356472, SUC_lexicon_tags_25 },
    { 0x15b56472, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8956475, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb56478, SUC_lexicon_tags_0 },
    { 0x3b956478, SUC_lexicon_tags_4 },
    { 0x1f35647a, SUC_lexicon_tags_55 },
    { 0x955647a, SUC_lexicon_tags_17 },
    { 0xa895647c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f56480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d56485, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x19556487, SUC_lexicon_tags_14 },
    { 0x84d56488, SUC_lexicon_tags_12 },
    { 0x54756488, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61b5648b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e95648d, SUC_lexicon_tags_29 },
    { 0xcd15648e, SUC_lexicon_tags_12 },
    { 0x5975648f, SUC_lexicon_tags_23 },
    { 0xa1b5648f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64556494, SUC_lexicon_tags_56 },
    { 0x3e956495, SUC_lexicon_tags_12 },
    { 0x7a956494, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b56499, SUC_lexicon_tags_65 },
    { 0x70d56499, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71f5649c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe195649f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9564a2, SUC_lexicon_tags_16 },
    { 0xe31564a2, SUC_lexicon_tags_5 },
    { 0x733564a4, SUC_lexicon_tags_7 },
    { 0x45f564a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85564a8, SUC_lexicon_tags_1 },
    { 0x2fd564a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171564ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5564b1, SUC_lexicon_tags_15 },
    { 0x683564b1, SUC_lexicon_tags_7 },
    { 0xcb5564b3, SUC_lexicon_tags_11 },
    { 0x195564b4, SUC_lexicon_tags_1 },
    { 0x4d5564b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x109564b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf564bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf564be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5564c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f564c4, SUC_lexicon_tags_11 },
    { 0x72d564c5, SUC_lexicon_tags_17 },
    { 0xd71564c6, SUC_lexicon_tags_8 },
    { 0x9d7564c7, SUC_lexicon_tags_19 },
    { 0x371564c8, SUC_lexicon_tags_7 },
    { 0x361564c8, SUC_lexicon_tags_47 },
    { 0xa9d564ca, SUC_lexicon_tags_14 },
    { 0xf55564cb, SUC_lexicon_tags_32 },
    { 0xc5f564cb, SUC_lexicon_tags_25 },
    { 0x50b564c7, SUC_lexicon_tags_9 },
    { 0x917564ce, SUC_lexicon_tags_46 },
    { 0x8c1564cf, SUC_lexicon_tags_15 },
    { 0xe3b564d0, SUC_lexicon_tags_1 },
    { 0x98d564c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x30b564d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x331564d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435564db, SUC_lexicon_tags_1 },
    { 0x495564db, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69564df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe99564e1, SUC_lexicon_tags_26 },
    { 0xf71564e2, SUC_lexicon_tags_48 },
    { 0x835564e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d564e7, SUC_lexicon_tags_60 },
    { 0x95b564e8, SUC_lexicon_tags_11 },
    { 0xff7564e9, SUC_lexicon_tags_0 },
    { 0x69b564e9, SUC_lexicon_tags_37 },
    { 0xa9d564ea, SUC_lexicon_tags_12 },
    { 0x1f1564ec, SUC_lexicon_tags_25 },
    { 0x417564ec, SUC_lexicon_tags_15 },
    { 0xfd9564ee, SUC_lexicon_tags_37 },
    { 0x4ff564ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf564f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d564fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x179564fe, SUC_lexicon_tags_12 },
    { 0xd9f564ff, SUC_lexicon_tags_38 },
    { 0xf73564fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3556503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11156507, SUC_lexicon_tags_29 },
    { 0xa9756507, SUC_lexicon_tags_8 },
    { 0x3c156508, SUC_lexicon_tags_47 },
    { 0xd15650a, SUC_lexicon_tags_25 },
    { 0x52956509, SUC_lexicon_tags_45 },
    { 0x3835650c, SUC_lexicon_tags_17 },
    { 0xa615650a, SUC_lexicon_tags_16 },
    { 0x89d5650e, SUC_lexicon_tags_27 },
    { 0x56b5650f, SUC_lexicon_tags_4 },
    { 0xdbb56510, SUC_lexicon_tags_34 },
    { 0x99d5650f, SUC_lexicon_tags_37 },
    { 0xc1356507, SUC_lexicon_tags_12 },
    { 0x5d56513, SUC_lexicon_tags_37 },
    { 0x65b56514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7956517, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x34756519, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf415651e, SUC_lexicon_tags_19 },
    { 0x2135651e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb56528, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e55652b, SUC_lexicon_tags_25 },
    { 0x5c95652b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b5652f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f756531, SUC_lexicon_tags_37 },
    { 0x99756531, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0156534, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2155653a, SUC_lexicon_tags_25 },
    { 0xe315653b, SUC_lexicon_tags_17 },
    { 0xb0d5653b, SUC_lexicon_tags_15 },
    { 0x2b55653d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd055653f, SUC_lexicon_tags_14 },
    { 0x51756540, SUC_lexicon_tags_39 },
    { 0x61d56540, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b756545, SUC_lexicon_tags_14 },
    { 0xa2156545, SUC_lexicon_tags_38 },
    { 0xd8d56547, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5654b, SUC_lexicon_tags_2 },
    { 0xa555654c, SUC_lexicon_tags_66 },
    { 0x4ff5654d, SUC_lexicon_tags_1 },
    { 0x995654e, SUC_lexicon_tags_80 },
    { 0x50b5654d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb56556, SUC_lexicon_tags_37 },
    { 0x5a156557, SUC_lexicon_tags_25 },
    { 0xbf756556, SUC_lexicon_tags_29 },
    { 0xfbb56557, SUC_lexicon_tags_12 },
    { 0xb5b56558, SUC_lexicon_tags_37 },
    { 0xb295655b, SUC_lexicon_tags_46 },
    { 0x65b5655b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe1b5655e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc4d56560, SUC_lexicon_tags_45 },
    { 0xe0356561, SUC_lexicon_tags_47 },
    { 0x10156560, SUC_lexicon_tags_37 },
    { 0x3b156563, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56568, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf95656a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e35656c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfdb5656e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad56571, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36356574, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad5657a, SUC_lexicon_tags_9 },
    { 0x72d5657a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7356582, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b556584, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae956587, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c55658a, SUC_lexicon_tags_27 },
    { 0x48b5658a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b56592, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d156595, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93756599, SUC_lexicon_tags_25 },
    { 0xdbb56599, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d565a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9565ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd83565ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfed565af, SUC_lexicon_tags_12 },
    { 0xf51565b0, SUC_lexicon_tags_37 },
    { 0x8f565b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775565b5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x909565b7, SUC_lexicon_tags_6 },
    { 0x87f565b8, SUC_lexicon_tags_27 },
    { 0x55f565b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc1565bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b1565bd, SUC_lexicon_tags_37 },
    { 0x96b565be, SUC_lexicon_tags_45 },
    { 0x2bb565bf, SUC_lexicon_tags_47 },
    { 0xef9565bf, SUC_lexicon_tags_25 },
    { 0x895565bf, SUC_lexicon_tags_0 },
    { 0xbd565c2, SUC_lexicon_tags_27 },
    { 0x1a5565c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d5565c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131565ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x95d565cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9565d2, SUC_lexicon_tags_17 },
    { 0x6a9565d3, SUC_lexicon_tags_12 },
    { 0xa3d565d4, SUC_lexicon_tags_49 },
    { 0x2e9565d5, SUC_lexicon_tags_37 },
    { 0xcd1565d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x13b565d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef565dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe81565de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25565e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc75565e3, SUC_lexicon_tags_30 },
    { 0xfe9565e4, SUC_lexicon_tags_14 },
    { 0x18d565e5, SUC_lexicon_tags_16 },
    { 0x341565e6, SUC_lexicon_tags_0 },
    { 0xe49565e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb71565e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379565ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce1565ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989565f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ff565f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9565fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b56601, SUC_lexicon_tags_30 },
    { 0x9bd56602, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b56605, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d5660b, SUC_lexicon_tags_37 },
    { 0xf6d5660c, SUC_lexicon_tags_17 },
    { 0xfd15660c, SUC_lexicon_tags_65 },
    { 0x2155660e, SUC_lexicon_tags_15 },
    { 0xe175660b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb56613, SUC_lexicon_tags_231 },
    { 0xbb756614, SUC_lexicon_tags_39 },
    { 0x3d356615, SUC_lexicon_tags_37 },
    { 0x3e956614, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0356619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc975661f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9f56621, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f756624, SUC_lexicon_tags_25 },
    { 0x94b56624, SUC_lexicon_tags_45 },
    { 0x2f756625, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa475662c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2db5662e, SUC_lexicon_tags_25 },
    { 0x1f75662f, SUC_lexicon_tags_1 },
    { 0x6d756630, SUC_lexicon_tags_46 },
    { 0x89756631, SUC_lexicon_tags_15 },
    { 0xbcf56630, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b556636, SUC_lexicon_tags_17 },
    { 0xf7d56637, SUC_lexicon_tags_21 },
    { 0xd6d56638, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d55663b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4295663f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89556643, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x50f56645, SUC_lexicon_tags_1 },
    { 0x5d956646, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd75664c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe156650, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b56655, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8756659, SUC_lexicon_tags_23 },
    { 0x3355665a, SUC_lexicon_tags_65 },
    { 0xeb55665b, SUC_lexicon_tags_29 },
    { 0xa815665c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaf15665e, SUC_lexicon_tags_94 },
    { 0x5dd5665f, SUC_lexicon_tags_14 },
    { 0xb8f56660, SUC_lexicon_tags_17 },
    { 0xc7f5665f, SUC_lexicon_tags_36 },
    { 0x47556661, SUC_lexicon_tags_1 },
    { 0xa515665f, SUC_lexicon_tags_15 },
    { 0x13b56660, SUC_lexicon_tags_17 },
    { 0xf556660, SUC_lexicon_tags_10 },
    { 0x7b56666, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x41556668, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9615666a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b15666c, SUC_lexicon_tags_83 },
    { 0x6495666d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0356672, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef56675, SUC_lexicon_tags_37 },
    { 0x75556676, SUC_lexicon_tags_2 },
    { 0xe0b56676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ab56679, SUC_lexicon_tags_60 },
    { 0x29b56679, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d56683, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x85d56685, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a356687, SUC_lexicon_tags_27 },
    { 0xc8756688, SUC_lexicon_tags_25 },
    { 0x3f56689, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c55668b, SUC_lexicon_tags_37 },
    { 0xb335668c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf56693, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x12956695, SUC_lexicon_tags_23 },
    { 0xc9d56696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b56699, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xabd5669b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd575669f, SUC_lexicon_tags_7 },
    { 0x597566a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e7566a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f566a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x525566ab, SUC_lexicon_tags_40 },
    { 0x757566ac, SUC_lexicon_tags_25 },
    { 0xfcb566ad, SUC_lexicon_tags_2 },
    { 0x61f566ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab566b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff566b5, SUC_lexicon_tags_23 },
    { 0x883566b5, SUC_lexicon_tags_6 },
    { 0x8c3566b7, SUC_lexicon_tags_8 },
    { 0xfdf566b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3566be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf73566c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb49566c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687566d0, SUC_lexicon_tags_25 },
    { 0xd5566d1, SUC_lexicon_tags_18 },
    { 0x817566d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf89566d4, SUC_lexicon_tags_11 },
    { 0x64b566d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3566de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d566e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89566e4, SUC_lexicon_tags_27 },
    { 0x3fb566e5, SUC_lexicon_tags_23 },
    { 0xf33566e6, SUC_lexicon_tags_27 },
    { 0x9b3566e5, SUC_lexicon_tags_37 },
    { 0xbdd566e7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x529566eb, SUC_lexicon_tags_17 },
    { 0x6f7566ec, SUC_lexicon_tags_49 },
    { 0x9a3566eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625566f1, SUC_lexicon_tags_11 },
    { 0x4b566f2, SUC_lexicon_tags_11 },
    { 0x3b9566f3, SUC_lexicon_tags_1422 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459566f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc7f566f9, SUC_lexicon_tags_7 },
    { 0x4d5566fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b566fe, SUC_lexicon_tags_47 },
    { 0x60b566fe, SUC_lexicon_tags_15 },
    { 0x41d56700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf56703, SUC_lexicon_tags_37 },
    { 0xcc156704, SUC_lexicon_tags_25 },
    { 0x8c756704, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d56708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d5670e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c156712, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d956714, SUC_lexicon_tags_25 },
    { 0xd2f56715, SUC_lexicon_tags_7 },
    { 0xbdd56716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb56719, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab5671c, SUC_lexicon_tags_37 },
    { 0x49f5671d, SUC_lexicon_tags_12 },
    { 0xe4f5671d, SUC_lexicon_tags_10 },
    { 0x6235671e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e356726, SUC_lexicon_tags_60 },
    { 0xb1556727, SUC_lexicon_tags_11 },
    { 0x73156727, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9f5672a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa175672e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e756731, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b56737, SUC_lexicon_tags_66 },
    { 0x4a956738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe015673c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd756740, SUC_lexicon_tags_25 },
    { 0x83956740, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6a356743, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f5674d, SUC_lexicon_tags_1 },
    { 0xd0d5674e, SUC_lexicon_tags_13 },
    { 0x70f5674e, SUC_lexicon_tags_13 },
    { 0x47f56750, SUC_lexicon_tags_11 },
    { 0x14b56751, SUC_lexicon_tags_16 },
    { 0xab556752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ab56754, SUC_lexicon_tags_25 },
    { 0x27156754, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6756757, SUC_lexicon_tags_12 },
    { 0xebb56757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd95675d, SUC_lexicon_tags_12 },
    { 0x78f5675d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3956761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf556763, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b56769, SUC_lexicon_tags_33 },
    { 0x57f56769, SUC_lexicon_tags_25 },
    { 0x3b56769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec15676f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x65956771, SUC_lexicon_tags_11 },
    { 0xa7756772, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7756774, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2556776, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb756779, SUC_lexicon_tags_17 },
    { 0xb0b5677a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7115677d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb56786, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b5678a, SUC_lexicon_tags_12 },
    { 0x9675678a, SUC_lexicon_tags_9 },
    { 0x71f5678b, SUC_lexicon_tags_37 },
    { 0xa035678a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x11d5678f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6956792, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e156794, SUC_lexicon_tags_25 },
    { 0x57756795, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b5679b, SUC_lexicon_tags_11 },
    { 0xcfb5679c, SUC_lexicon_tags_25 },
    { 0x1b35679c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99567a1, SUC_lexicon_tags_36 },
    { 0xd77567a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd567a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37567aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509567af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b567b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741567bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65567bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95567c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391567c6, SUC_lexicon_tags_1 },
    { 0x845567c7, SUC_lexicon_tags_108 },
    { 0x2d9567c8, SUC_lexicon_tags_116 },
    { 0xee1567c9, SUC_lexicon_tags_38 },
    { 0xef567c9, SUC_lexicon_tags_25 },
    { 0x6d1567c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d567cf, SUC_lexicon_tags_11 },
    { 0x1af567d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb567d6, SUC_lexicon_tags_25 },
    { 0x497567d6, SUC_lexicon_tags_92 },
    { 0xef3567d8, SUC_lexicon_tags_33 },
    { 0xd3d567d9, SUC_lexicon_tags_37 },
    { 0x4cd567da, SUC_lexicon_tags_37 },
    { 0xd77567d9, SUC_lexicon_tags_7 },
    { 0xe5f567db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29567eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f9567ed, SUC_lexicon_tags_82 },
    { 0x8c3567ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3567f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1567f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1567fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21567ff, SUC_lexicon_tags_7 },
    { 0x51b567ff, SUC_lexicon_tags_37 },
    { 0x30f56801, SUC_lexicon_tags_37 },
    { 0xdbf56802, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d156804, SUC_lexicon_tags_25 },
    { 0xccd56805, SUC_lexicon_tags_21 },
    { 0x7ff56806, SUC_lexicon_tags_255 },
    { 0x3cd56807, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd56810, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x94556813, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf56818, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef556821, SUC_lexicon_tags_3 },
    { 0x4d156822, SUC_lexicon_tags_17 },
    { 0xe6956822, SUC_lexicon_tags_27 },
    { 0x24f56824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f56827, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f15682a, SUC_lexicon_tags_15 },
    { 0x9555682b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf135682f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d5683b, SUC_lexicon_tags_49 },
    { 0x64f5683c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd935683e, SUC_lexicon_tags_59 },
    { 0xc635683f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa8f56841, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1156844, SUC_lexicon_tags_15 },
    { 0x17956845, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39356847, SUC_lexicon_tags_23 },
    { 0xef156848, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b5684d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e75684f, SUC_lexicon_tags_38 },
    { 0xbc75684f, SUC_lexicon_tags_14 },
    { 0x97756850, SUC_lexicon_tags_2 },
    { 0x9a756852, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9f56854, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1fd56856, SUC_lexicon_tags_35 },
    { 0xe6b56856, SUC_lexicon_tags_27 },
    { 0xb3356858, SUC_lexicon_tags_47 },
    { 0xc3b56858, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b5685d, SUC_lexicon_tags_33 },
    { 0xb775685e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f956861, SUC_lexicon_tags_7 },
    { 0xefb56861, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e156864, SUC_lexicon_tags_45 },
    { 0x1956864, SUC_lexicon_tags_26 },
    { 0xb7556866, SUC_lexicon_tags_6 },
    { 0x41b56864, SUC_lexicon_tags_33 },
    { 0xb7b56867, SUC_lexicon_tags_32 },
    { 0xe0356869, SUC_lexicon_tags_26 },
    { 0xbcf5686a, SUC_lexicon_tags_88 },
    { 0x4255686b, SUC_lexicon_tags_2 },
    { 0xacd5686c, SUC_lexicon_tags_1 },
    { 0x8d15686a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b5687f, SUC_lexicon_tags_37 },
    { 0x4e15687f, SUC_lexicon_tags_77 },
    { 0x65f56881, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec356884, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x78356887, SUC_lexicon_tags_56 },
    { 0x1b156888, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb5b5688a, SUC_lexicon_tags_39 },
    { 0x8335688a, SUC_lexicon_tags_32 },
    { 0xa335688c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f56891, SUC_lexicon_tags_15 },
    { 0xfbf56892, SUC_lexicon_tags_1 },
    { 0x16956892, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f356896, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99f56898, SUC_lexicon_tags_11 },
    { 0x4e356899, SUC_lexicon_tags_61 },
    { 0x6055689a, SUC_lexicon_tags_23 },
    { 0x8035689b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955568a1, SUC_lexicon_tags_53 },
    { 0xd57568a2, SUC_lexicon_tags_15 },
    { 0x869568a3, SUC_lexicon_tags_1 },
    { 0x59d568a4, SUC_lexicon_tags_27 },
    { 0xf3568a4, SUC_lexicon_tags_47 },
    { 0x411568a2, SUC_lexicon_tags_25 },
    { 0xfed568a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf568ad, SUC_lexicon_tags_2 },
    { 0x6ad568ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89568b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x409568b4, SUC_lexicon_tags_14 },
    { 0x615568b5, SUC_lexicon_tags_19 },
    { 0x523568b6, SUC_lexicon_tags_3 },
    { 0xc71568b6, SUC_lexicon_tags_13 },
    { 0x82b568b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe9f568ba, SUC_lexicon_tags_9 },
    { 0xa5f568bb, SUC_lexicon_tags_28 },
    { 0x791568bc, SUC_lexicon_tags_49 },
    { 0x7f568bd, SUC_lexicon_tags_14 },
    { 0xf93568bd, SUC_lexicon_tags_37 },
    { 0xf07568bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f568c3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d568c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc17568c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd47568ca, SUC_lexicon_tags_17 },
    { 0x899568cb, SUC_lexicon_tags_7 },
    { 0xa2d568cc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1568cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b568dd, SUC_lexicon_tags_27 },
    { 0x175568de, SUC_lexicon_tags_11 },
    { 0xdfd568df, SUC_lexicon_tags_32 },
    { 0x42b568de, SUC_lexicon_tags_66 },
    { 0xd41568e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x447568e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f568e9, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f568ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b568f1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b568f6, SUC_lexicon_tags_12 },
    { 0xe79568f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf11568f9, SUC_lexicon_tags_25 },
    { 0xdf3568f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81568fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc91568ff, SUC_lexicon_tags_30 },
    { 0xf0756900, SUC_lexicon_tags_32 },
    { 0x19956901, SUC_lexicon_tags_65 },
    { 0x98956900, SUC_lexicon_tags_12 },
    { 0x1b556903, SUC_lexicon_tags_2 },
    { 0x43756902, SUC_lexicon_tags_62 },
    { 0x45f56905, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x17956908, SUC_lexicon_tags_49 },
    { 0x32156909, SUC_lexicon_tags_53 },
    { 0x7b556909, SUC_lexicon_tags_35 },
    { 0xee55690b, SUC_lexicon_tags_25 },
    { 0xc2f5690b, SUC_lexicon_tags_26 },
    { 0xdcf5690d, SUC_lexicon_tags_25 },
    { 0x3d75690e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b56915, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d5691a, SUC_lexicon_tags_9 },
    { 0x4b75691b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7975691d, SUC_lexicon_tags_7 },
    { 0xd015691e, SUC_lexicon_tags_10 },
    { 0x675691d, SUC_lexicon_tags_15 },
    { 0xdf356920, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3156923, SUC_lexicon_tags_3 },
    { 0x2c156924, SUC_lexicon_tags_63 },
    { 0x9c956923, SUC_lexicon_tags_1 },
    { 0xcd356926, SUC_lexicon_tags_17 },
    { 0x43f56926, SUC_lexicon_tags_11 },
    { 0x6d56928, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd5692d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d956930, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fb56932, SUC_lexicon_tags_33 },
    { 0xcff56933, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db56937, SUC_lexicon_tags_158 },
    { 0x62f56937, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe135693a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d55693c, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23956943, SUC_lexicon_tags_17 },
    { 0x3d756944, SUC_lexicon_tags_17 },
    { 0x74d56945, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd56949, SUC_lexicon_tags_37 },
    { 0xd0d56949, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d56950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9f56952, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf56955, SUC_lexicon_tags_13 },
    { 0x2d356955, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb56959, SUC_lexicon_tags_1244 },
    { 0x2935695a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3375695c, SUC_lexicon_tags_25 },
    { 0x3cf5695d, SUC_lexicon_tags_6 },
    { 0xe535695d, SUC_lexicon_tags_11 },
    { 0xb55695f, SUC_lexicon_tags_15 },
    { 0x93f56960, SUC_lexicon_tags_17 },
    { 0xa3556961, SUC_lexicon_tags_33 },
    { 0x5315695f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd56968, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1775696e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf56971, SUC_lexicon_tags_9 },
    { 0x98356972, SUC_lexicon_tags_37 },
    { 0x19f56972, SUC_lexicon_tags_1 },
    { 0x46b56971, SUC_lexicon_tags_26 },
    { 0x76b56975, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b156978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a75697b, SUC_lexicon_tags_38 },
    { 0x2935697b, SUC_lexicon_tags_0 },
    { 0x17d5697d, SUC_lexicon_tags_7 },
    { 0x5ab5697e, SUC_lexicon_tags_25 },
    { 0x4bd5697f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c156982, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54356984, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x21156986, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f95698c, SUC_lexicon_tags_29 },
    { 0xe035698d, SUC_lexicon_tags_17 },
    { 0xfb5698c, SUC_lexicon_tags_25 },
    { 0x55698f, SUC_lexicon_tags_1 },
    { 0x68f5698e, SUC_lexicon_tags_1 },
    { 0x5fd5698e, SUC_lexicon_tags_0 },
    { 0x65156992, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17956994, SUC_lexicon_tags_17 },
    { 0xabd56994, SUC_lexicon_tags_28 },
    { 0xf4556996, SUC_lexicon_tags_37 },
    { 0xee756996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5699d, SUC_lexicon_tags_15 },
    { 0x7bb5699d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b569a1, SUC_lexicon_tags_2 },
    { 0x6f5569a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf569a5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x9f9569a7, SUC_lexicon_tags_2 },
    { 0xa99569a8, SUC_lexicon_tags_14 },
    { 0x8cb569a8, SUC_lexicon_tags_87 },
    { 0x8c5569a7, SUC_lexicon_tags_38 },
    { 0xf7b569ab, SUC_lexicon_tags_10 },
    { 0x4a9569ac, SUC_lexicon_tags_7 },
    { 0x8d3569ad, SUC_lexicon_tags_25 },
    { 0x889569ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11569b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577569ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589569c2, SUC_lexicon_tags_28 },
    { 0x9b9569c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3569c7, SUC_lexicon_tags_11 },
    { 0xac5569c8, SUC_lexicon_tags_9 },
    { 0x55569c9, SUC_lexicon_tags_12 },
    { 0x79b569ca, SUC_lexicon_tags_43 },
    { 0xef7569ca, SUC_lexicon_tags_0 },
    { 0x1e5569ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5569d1, SUC_lexicon_tags_10 },
    { 0xaab569d2, SUC_lexicon_tags_46 },
    { 0xed9569d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65569d6, SUC_lexicon_tags_25 },
    { 0x4af569d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9569da, SUC_lexicon_tags_37 },
    { 0x907569db, SUC_lexicon_tags_15 },
    { 0x887569dc, SUC_lexicon_tags_7 },
    { 0x701569dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09569e0, SUC_lexicon_tags_15 },
    { 0xfd1569e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xec9569e3, SUC_lexicon_tags_11 },
    { 0x851569e3, SUC_lexicon_tags_17 },
    { 0xb7569e5, SUC_lexicon_tags_11 },
    { 0xaa3569e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f569ea, SUC_lexicon_tags_25 },
    { 0x27f569eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5569ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73f569f1, SUC_lexicon_tags_17 },
    { 0x9d3569f1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeb5569f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc1569f6, SUC_lexicon_tags_65 },
    { 0xa4b569f6, SUC_lexicon_tags_27 },
    { 0xf61569f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x859569fb, SUC_lexicon_tags_65 },
    { 0x43569fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x921569fe, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6556a01, SUC_lexicon_tags_7 },
    { 0xdc556a01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb56a05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91356a0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf56a0d, SUC_lexicon_tags_66 },
    { 0x72156a0e, SUC_lexicon_tags_0 },
    { 0x69956a0f, SUC_lexicon_tags_2 },
    { 0xca556a10, SUC_lexicon_tags_10 },
    { 0x52356a0d, SUC_lexicon_tags_27 },
    { 0x38f56a10, SUC_lexicon_tags_1 },
    { 0xacd56a13, SUC_lexicon_tags_19 },
    { 0x90f56a14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd56a17, SUC_lexicon_tags_39 },
    { 0xbcd56a18, SUC_lexicon_tags_37 },
    { 0xb7d56a18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3756a1c, SUC_lexicon_tags_15 },
    { 0xb3756a1d, SUC_lexicon_tags_37 },
    { 0x2956a1e, SUC_lexicon_tags_17 },
    { 0x756a1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda556a24, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd56a28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c956a2a, SUC_lexicon_tags_28 },
    { 0xae756a2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x78f56a2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d56a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f56a39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x24956a3c, SUC_lexicon_tags_61 },
    { 0xd356a3d, SUC_lexicon_tags_15 },
    { 0x50356a3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f56a44, SUC_lexicon_tags_37 },
    { 0xb9f56a45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6756a49, SUC_lexicon_tags_30 },
    { 0x22b56a49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd56a4e, SUC_lexicon_tags_12 },
    { 0x3e756a4f, SUC_lexicon_tags_12 },
    { 0xec356a50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f56a55, SUC_lexicon_tags_63 },
    { 0x84d56a55, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83556a5c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x25356a5e, SUC_lexicon_tags_11 },
    { 0xa0956a5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0556a61, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x40b56a63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e356a6b, SUC_lexicon_tags_12 },
    { 0x5fd56a6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4556a71, SUC_lexicon_tags_60 },
    { 0x7b156a72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd56a77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7356a79, SUC_lexicon_tags_10 },
    { 0x4f956a7a, SUC_lexicon_tags_77 },
    { 0xa8d56a79, SUC_lexicon_tags_17 },
    { 0x81156a7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27156a8e, SUC_lexicon_tags_1 },
    { 0xc556a8f, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c756a96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2156a9b, SUC_lexicon_tags_37 },
    { 0x9f156a9c, SUC_lexicon_tags_65 },
    { 0x1bd56a9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4156aa3, SUC_lexicon_tags_7 },
    { 0x6b156aa3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7bd56aa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99956aa8, SUC_lexicon_tags_12 },
    { 0x70356aa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71756aac, SUC_lexicon_tags_46 },
    { 0x1a356aad, SUC_lexicon_tags_25 },
    { 0xd7156aac, SUC_lexicon_tags_37 },
    { 0x3a356aaf, SUC_lexicon_tags_40 },
    { 0xbaf56aae, SUC_lexicon_tags_11 },
    { 0x84f56aad, SUC_lexicon_tags_17 },
    { 0xcff56ab2, SUC_lexicon_tags_14 },
    { 0x18156aae, SUC_lexicon_tags_68 },
    { 0x98f56ab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f156ab9, SUC_lexicon_tags_1 },
    { 0x73d56ab9, SUC_lexicon_tags_55 },
    { 0x9f356abb, SUC_lexicon_tags_11 },
    { 0x1d756abc, SUC_lexicon_tags_43 },
    { 0x51b56abc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e556ac5, SUC_lexicon_tags_11 },
    { 0x68756ac6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b356acb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd956ace, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee556ad1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59756ad3, SUC_lexicon_tags_45 },
    { 0xabb56ad4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1bf56ad6, SUC_lexicon_tags_31 },
    { 0x23356ad6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd56adf, SUC_lexicon_tags_15 },
    { 0x14956ae0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd56ae4, SUC_lexicon_tags_17 },
    { 0x3ed56ae5, SUC_lexicon_tags_14 },
    { 0xa9b56ae6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3156aed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d56af0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d56af4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde956af7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b56afb, SUC_lexicon_tags_43 },
    { 0xacd56afc, SUC_lexicon_tags_6 },
    { 0x1a156afc, SUC_lexicon_tags_2 },
    { 0x73756afe, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d756b01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d556b06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d56b09, SUC_lexicon_tags_49 },
    { 0xcf156b0a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c556b12, SUC_lexicon_tags_7 },
    { 0x11f56b13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb56b1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e956b1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b556b2a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4556b2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d56b30, SUC_lexicon_tags_15 },
    { 0x36756b31, SUC_lexicon_tags_17 },
    { 0xd1b56b31, SUC_lexicon_tags_39 },
    { 0x58956b30, SUC_lexicon_tags_25 },
    { 0xced56b34, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b56b3c, SUC_lexicon_tags_2 },
    { 0x9af56b3c, SUC_lexicon_tags_250 },
    { 0x5e756b3e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2956b43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8756b46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff56b50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7f56b52, SUC_lexicon_tags_60 },
    { 0x71f56b53, SUC_lexicon_tags_39 },
    { 0xb556b54, SUC_lexicon_tags_138 },
    { 0x9ef56b55, SUC_lexicon_tags_1 },
    { 0xc7356b55, SUC_lexicon_tags_21 },
    { 0xf6156b53, SUC_lexicon_tags_25 },
    { 0xe1d56b56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb756b60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af56b66, SUC_lexicon_tags_12 },
    { 0xd4756b67, SUC_lexicon_tags_32 },
    { 0xc9556b67, SUC_lexicon_tags_19 },
    { 0x97b56b69, SUC_lexicon_tags_85 },
    { 0xc3f56b68, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x85d56b6c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a156b72, SUC_lexicon_tags_2 },
    { 0x55f56b73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xde556b76, SUC_lexicon_tags_43 },
    { 0xc9956b76, SUC_lexicon_tags_28 },
    { 0x3c356b76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8156b85, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b56b8a, SUC_lexicon_tags_37 },
    { 0x3fb56b8b, SUC_lexicon_tags_15 },
    { 0x73b56b8c, SUC_lexicon_tags_17 },
    { 0xc4956b8d, SUC_lexicon_tags_55 },
    { 0xb3f56b8d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7b156b90, SUC_lexicon_tags_5 },
    { 0xdad56b91, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa9756b93, SUC_lexicon_tags_2 },
    { 0x1b356b94, SUC_lexicon_tags_15 },
    { 0x32b56b93, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17556b9b, SUC_lexicon_tags_27 },
    { 0x2f556b9c, SUC_lexicon_tags_26 },
    { 0x5756b9c, SUC_lexicon_tags_26 },
    { 0x5df56b9e, SUC_lexicon_tags_26 },
    { 0x87356b9e, SUC_lexicon_tags_25 },
    { 0x8e356ba0, SUC_lexicon_tags_160 },
    { 0x3ed56ba0, SUC_lexicon_tags_28 },
    { 0x2fd56b9d, SUC_lexicon_tags_11 },
    { 0x1f56b9c, SUC_lexicon_tags_3 },
    { 0x93f56ba4, SUC_lexicon_tags_11 },
    { 0x4fb56ba0, SUC_lexicon_tags_7 },
    { 0xdf156ba6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b356bab, SUC_lexicon_tags_2 },
    { 0x1a956bac, SUC_lexicon_tags_10 },
    { 0x88d56bad, SUC_lexicon_tags_11 },
    { 0x5eb56bae, SUC_lexicon_tags_7 },
    { 0xe3b56bae, SUC_lexicon_tags_49 },
    { 0xedd56bab, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x6ff56bb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84156bb7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf5f56bb9, SUC_lexicon_tags_25 },
    { 0xeed56bba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab356bbe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb56bc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d556bc3, SUC_lexicon_tags_25 },
    { 0x8f956bc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd56bc8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd56bd2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f56bd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66956bdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf356bde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a356be2, SUC_lexicon_tags_82 },
    { 0xd3f56be3, SUC_lexicon_tags_20 },
    { 0x3a156be3, SUC_lexicon_tags_64 },
    { 0x38956be5, SUC_lexicon_tags_12 },
    { 0x53156be2, SUC_lexicon_tags_7 },
    { 0x43d56be7, SUC_lexicon_tags_63 },
    { 0x3cb56be8, SUC_lexicon_tags_26 },
    { 0x73956be9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b56bec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe356bef, SUC_lexicon_tags_77 },
    { 0x4a356bf0, SUC_lexicon_tags_27 },
    { 0xe5f56bf1, SUC_lexicon_tags_91 },
    { 0x16156bf2, SUC_lexicon_tags_37 },
    { 0x40356bf3, SUC_lexicon_tags_37 },
    { 0x18b56bf4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d56bf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b56bfb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56356c01, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e556c06, SUC_lexicon_tags_37 },
    { 0xd7d56c07, SUC_lexicon_tags_47 },
    { 0xacb56c07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x61d56c0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4756c0c, SUC_lexicon_tags_26 },
    { 0xfe356c0d, SUC_lexicon_tags_7 },
    { 0xa756c0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x40556c11, SUC_lexicon_tags_48 },
    { 0x59956c12, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59556c17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8556c1c, SUC_lexicon_tags_15 },
    { 0x77d56c1d, SUC_lexicon_tags_32 },
    { 0xa7956c1e, SUC_lexicon_tags_60 },
    { 0x19556c1f, SUC_lexicon_tags_8 },
    { 0x77356c1d, SUC_lexicon_tags_11 },
    { 0xbd556c21, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1b756c23, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc956c27, SUC_lexicon_tags_17 },
    { 0x99756c28, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d56c2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3156c33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b56c3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91956c3e, SUC_lexicon_tags_28 },
    { 0x7956c3f, SUC_lexicon_tags_4 },
    { 0x4db56c3e, SUC_lexicon_tags_26 },
    { 0xd8d56c3e, SUC_lexicon_tags_92 },
    { 0x51d56c42, SUC_lexicon_tags_37 },
    { 0xa5f56c43, SUC_lexicon_tags_11 },
    { 0x48756c43, SUC_lexicon_tags_26 },
    { 0x58556c3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb756c4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89756c51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe5156c53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1b56c55, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83156c59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5e956c5b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xba356c5d, SUC_lexicon_tags_12 },
    { 0xdab56c5e, SUC_lexicon_tags_4 },
    { 0x8b956c5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a556c62, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x48156c64, SUC_lexicon_tags_3 },
    { 0xcf556c64, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbf156c67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5356c69, SUC_lexicon_tags_37 },
    { 0x80556c6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74756c71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0756c74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c156c76, SUC_lexicon_tags_0 },
    { 0x7ef56c77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df56c7c, SUC_lexicon_tags_47 },
    { 0xd956c7d, SUC_lexicon_tags_27 },
    { 0xd6956c7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf56c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7356c88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd756c8d, SUC_lexicon_tags_14 },
    { 0x59756c8d, SUC_lexicon_tags_17 },
    { 0xcdd56c8f, SUC_lexicon_tags_11 },
    { 0x50956c8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8f756c92, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x48556c94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59156c99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0b56c9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a556c9e, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf56ca3, SUC_lexicon_tags_3 },
    { 0xa1356ca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90b56ca6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf956ca9, SUC_lexicon_tags_61 },
    { 0xfab56caa, SUC_lexicon_tags_37 },
    { 0x1956caa, SUC_lexicon_tags_11 },
    { 0x4f756caa, SUC_lexicon_tags_2 },
    { 0xec156ca9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87356cb2, SUC_lexicon_tags_170 },
    { 0x59f56cb3, SUC_lexicon_tags_35 },
    { 0xa0d56cb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7156cb7, SUC_lexicon_tags_8 },
    { 0x7d556cb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b356cbb, SUC_lexicon_tags_17 },
    { 0x57756cbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b56cbe, SUC_lexicon_tags_25 },
    { 0x15756cbf, SUC_lexicon_tags_14 },
    { 0x2f956cbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f56cc7, SUC_lexicon_tags_37 },
    { 0x87556cc8, SUC_lexicon_tags_11 },
    { 0xa4556cc9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x17156ccb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81556ccd, SUC_lexicon_tags_17 },
    { 0xb9756ccd, SUC_lexicon_tags_17 },
    { 0x90d56cce, SUC_lexicon_tags_37 },
    { 0xd9356ccd, SUC_lexicon_tags_25 },
    { 0x5c756cd1, SUC_lexicon_tags_33 },
    { 0xf8d56cd1, SUC_lexicon_tags_27 },
    { 0x40f56cce, SUC_lexicon_tags_60 },
    { 0xf9556cd4, SUC_lexicon_tags_1 },
    { 0xba956cd5, SUC_lexicon_tags_84 },
    { 0xadd56cd5, SUC_lexicon_tags_45 },
    { 0xd0356cd5, SUC_lexicon_tags_34 },
    { 0x8c356cd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d56cdd, SUC_lexicon_tags_7 },
    { 0x11d56cde, SUC_lexicon_tags_11 },
    { 0x78b56cdf, SUC_lexicon_tags_12 },
    { 0x99356ce0, SUC_lexicon_tags_2 },
    { 0x5c756cdd, SUC_lexicon_tags_37 },
    { 0x20556ce2, SUC_lexicon_tags_7 },
    { 0xed356ce1, SUC_lexicon_tags_11 },
    { 0xf5356cdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x72556ce6, SUC_lexicon_tags_12 },
    { 0xb2f56ce6, SUC_lexicon_tags_27 },
    { 0x2cd56ce8, SUC_lexicon_tags_2 },
    { 0x9ff56ce9, SUC_lexicon_tags_37 },
    { 0x67756cea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e756cf2, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0x4bf56cf4, SUC_lexicon_tags_15 },
    { 0x52356cf5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x5c356cf7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e156cfa, SUC_lexicon_tags_27 },
    { 0xbd56cfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef956d03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69156d05, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2356d0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f56d0d, SUC_lexicon_tags_5 },
    { 0xf7956d0e, SUC_lexicon_tags_80 },
    { 0x71f56d0f, SUC_lexicon_tags_6 },
    { 0x7e356d10, SUC_lexicon_tags_43 },
    { 0xd556d11, SUC_lexicon_tags_82 },
    { 0xeb756d12, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2c756d14, SUC_lexicon_tags_12 },
    { 0x98556d15, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0356d19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9956d1b, SUC_lexicon_tags_16 },
    { 0x44156d1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f156d1f, SUC_lexicon_tags_28 },
    { 0x81756d20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38356d22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd56d2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9eb56d2e, SUC_lexicon_tags_160 },
    { 0xe2756d2e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc3f56d31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86756d36, SUC_lexicon_tags_12 },
    { 0x98956d37, SUC_lexicon_tags_11 },
    { 0x50156d38, SUC_lexicon_tags_2 },
    { 0x5f356d39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1356d48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf56d4b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9556d55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1556d57, SUC_lexicon_tags_39 },
    { 0x19756d58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7db56d5a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xba356d5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67356d5f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1556d64, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34156d69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68756d6d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b56d70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73f56d73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xab556d76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bd56d78, SUC_lexicon_tags_6 },
    { 0x6dd56d79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61956d80, SUC_lexicon_tags_2 },
    { 0x8a356d81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24556d85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84556d8a, SUC_lexicon_tags_47 },
    { 0xadb56d8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af56d93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2356d95, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b56d9c, SUC_lexicon_tags_15 },
    { 0xc6756d9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b56da3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf56daa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf0f56dac, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd156daf, SUC_lexicon_tags_2 },
    { 0x44f56db0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94556db5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f56dbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8f56dbd, SUC_lexicon_tags_17 },
    { 0x7e156dbe, SUC_lexicon_tags_60 },
    { 0xcfb56dbf, SUC_lexicon_tags_14 },
    { 0x1cb56dc0, SUC_lexicon_tags_26 },
    { 0x14956dc0, SUC_lexicon_tags_35 },
    { 0xecf56dc2, SUC_lexicon_tags_37 },
    { 0x4e956dc0, SUC_lexicon_tags_37 },
    { 0x48756dc3, SUC_lexicon_tags_19 },
    { 0x39f56dbf, SUC_lexicon_tags_7 },
    { 0xe4f56dc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb956dc9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef56dce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb56dd3, SUC_lexicon_tags_31 },
    { 0x78156dd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2556dd6, SUC_lexicon_tags_82 },
    { 0x3d556dd7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd56ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f956dde, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb4156de0, SUC_lexicon_tags_7 },
    { 0xd0f56de1, SUC_lexicon_tags_32 },
    { 0x88756de2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf56de8, SUC_lexicon_tags_26 },
    { 0x5df56de9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff56df3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ab56df5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad956df7, SUC_lexicon_tags_17 },
    { 0xbf756df8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f56dfb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b56dfe, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f56e04, SUC_lexicon_tags_25 },
    { 0xaa956e04, SUC_lexicon_tags_43 },
    { 0xa7f56e05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3b56e08, SUC_lexicon_tags_336 },
    { 0xb6556e09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df56e0c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf56e0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee556e11, SUC_lexicon_tags_63 },
    { 0xa7556e11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14556e16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x82d56e18, SUC_lexicon_tags_9 },
    { 0xd1756e18, SUC_lexicon_tags_12 },
    { 0x2a756e1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b56e1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33556e22, SUC_lexicon_tags_32 },
    { 0xcbd56e22, SUC_lexicon_tags_19 },
    { 0x5e156e24, SUC_lexicon_tags_25 },
    { 0xded56e23, SUC_lexicon_tags_19 },
    { 0xf5756e25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51756e3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d756e3e, SUC_lexicon_tags_4 },
    { 0x44556e3f, SUC_lexicon_tags_22 },
    { 0xb5956e3e, SUC_lexicon_tags_109 },
    { 0xa8956e41, SUC_lexicon_tags_37 },
    { 0x17756e41, SUC_lexicon_tags_11 },
    { 0xef356e43, SUC_lexicon_tags_15 },
    { 0x63f56e44, SUC_lexicon_tags_65 },
    { 0x50156e45, SUC_lexicon_tags_43 },
    { 0x2c556e45, SUC_lexicon_tags_18 },
    { 0xe2156e43, SUC_lexicon_tags_6 },
    { 0x34b56e43, SUC_lexicon_tags_13 },
    { 0x34756e41, SUC_lexicon_tags_56 },
    { 0xec556e4a, SUC_lexicon_tags_26 },
    { 0x18d56e49, SUC_lexicon_tags_17 },
    { 0xaf956e4c, SUC_lexicon_tags_16 },
    { 0x3db56e4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48756e51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x79156e53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc156e5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68156e5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa5356e5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d56e62, SUC_lexicon_tags_37 },
    { 0x9a156e63, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2956e68, SUC_lexicon_tags_12 },
    { 0x79f56e69, SUC_lexicon_tags_6 },
    { 0x38556e69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82556e6e, SUC_lexicon_tags_31 },
    { 0x82b56e6f, SUC_lexicon_tags_1 },
    { 0x2cd56e6e, SUC_lexicon_tags_2 },
    { 0x38d56e70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c556e73, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xda356e75, SUC_lexicon_tags_15 },
    { 0x43756e76, SUC_lexicon_tags_15 },
    { 0xed956e77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcd56e79, SUC_lexicon_tags_25 },
    { 0xfcf56e79, SUC_lexicon_tags_0 },
    { 0xfc156e7b, SUC_lexicon_tags_37 },
    { 0x22b56e7b, SUC_lexicon_tags_23 },
    { 0x83756e7d, SUC_lexicon_tags_48 },
    { 0x2ad56e7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfc956e80, SUC_lexicon_tags_6 },
    { 0x96556e81, SUC_lexicon_tags_11 },
    { 0xecd56e80, SUC_lexicon_tags_37 },
    { 0x26956e83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4db56e85, SUC_lexicon_tags_11 },
    { 0x31556e85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6356e88, SUC_lexicon_tags_25 },
    { 0x51956e89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d556e92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b56e97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d56e9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af56ea3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6b56ea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc756eaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5556eae, SUC_lexicon_tags_29 },
    { 0xfc956eaf, SUC_lexicon_tags_33 },
    { 0x4356eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd56eb6, SUC_lexicon_tags_49 },
    { 0x3d756eb7, SUC_lexicon_tags_17 },
    { 0xa0156eb8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4156ebb, SUC_lexicon_tags_1 },
    { 0x40956ebb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x41156ebf, SUC_lexicon_tags_6 },
    { 0x6e556ebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2356ec5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xff956ec8, SUC_lexicon_tags_47 },
    { 0x80356ec9, SUC_lexicon_tags_2 },
    { 0xe2b56eca, SUC_lexicon_tags_64 },
    { 0x79756eca, SUC_lexicon_tags_1 },
    { 0xdfd56ecc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73556ece, SUC_lexicon_tags_30 },
    { 0x11356ecf, SUC_lexicon_tags_11 },
    { 0x1f156ece, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda156ed5, SUC_lexicon_tags_10 },
    { 0x38d56ed6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d956ed9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c956edc, SUC_lexicon_tags_15 },
    { 0x36d56edd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba356edf, SUC_lexicon_tags_12 },
    { 0x39d56edf, SUC_lexicon_tags_68 },
    { 0x1b556edf, SUC_lexicon_tags_2 },
    { 0x9bb56ee2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x36356ee4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcb756ee6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfe756ee8, SUC_lexicon_tags_11 },
    { 0x10156ee9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f56eed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9956ef0, SUC_lexicon_tags_71 },
    { 0xb8156ef0, SUC_lexicon_tags_29 },
    { 0x78156ef1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x37356ef4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff56ef8, SUC_lexicon_tags_15 },
    { 0x85156ef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d56efc, SUC_lexicon_tags_136 },
    { 0x49356efd, SUC_lexicon_tags_37 },
    { 0x51b56efe, SUC_lexicon_tags_25 },
    { 0xaff56eff, SUC_lexicon_tags_10 },
    { 0x96156efc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3956f04, SUC_lexicon_tags_43 },
    { 0x28156f05, SUC_lexicon_tags_17 },
    { 0xeb956f06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d356f0b, SUC_lexicon_tags_37 },
    { 0xc5f56f0c, SUC_lexicon_tags_26 },
    { 0x54356f0c, SUC_lexicon_tags_243 },
    { 0xa7d56f0e, SUC_lexicon_tags_15 },
    { 0x41956f0f, SUC_lexicon_tags_27 },
    { 0x64156f0e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x156f14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b156f18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a556f1a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6ff56f1c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x27356f1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b556f22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f56f27, SUC_lexicon_tags_17 },
    { 0xe9f56f27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb556f2b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56f2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad156f31, SUC_lexicon_tags_32 },
    { 0xd4b56f31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61356f38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c756f3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e956f42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93156f45, SUC_lexicon_tags_0 },
    { 0x9df56f46, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf56f4a, SUC_lexicon_tags_160 },
    { 0xd4d56f4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf56f50, SUC_lexicon_tags_25 },
    { 0xe5556f51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc556f53, SUC_lexicon_tags_26 },
    { 0x97d56f53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62556f58, SUC_lexicon_tags_43 },
    { 0xd7956f58, SUC_lexicon_tags_28 },
    { 0xa0d56f5a, SUC_lexicon_tags_17 },
    { 0xb2156f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d56f60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29556f65, SUC_lexicon_tags_25 },
    { 0x51f56f66, SUC_lexicon_tags_2 },
    { 0xa2156f67, SUC_lexicon_tags_31 },
    { 0x6d556f68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2356f6a, SUC_lexicon_tags_11 },
    { 0xe8556f6b, SUC_lexicon_tags_60 },
    { 0x67956f6b, SUC_lexicon_tags_26 },
    { 0xe8756f6d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe3956f6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f56f75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3156f7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6156f7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f156f86, SUC_lexicon_tags_25 },
    { 0xb2f56f86, SUC_lexicon_tags_33 },
    { 0x13d56f88, SUC_lexicon_tags_14 },
    { 0xd0b56f89, SUC_lexicon_tags_172 },
    { 0xb2b56f8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49556f8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87756f8e, SUC_lexicon_tags_21 },
    { 0x52f56f8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e356f98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5556fa3, SUC_lexicon_tags_233 },
    { 0x4f556fa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3356fa6, SUC_lexicon_tags_1 },
    { 0x9956fa6, SUC_lexicon_tags_17 },
    { 0x14d56fa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd756fb0, SUC_lexicon_tags_32 },
    { 0x54b56fb1, SUC_lexicon_tags_37 },
    { 0xf1556fb0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa6356fb4, SUC_lexicon_tags_30 },
    { 0x21356fb5, SUC_lexicon_tags_15 },
    { 0x50f56fb5, SUC_lexicon_tags_2 },
    { 0xbc756fb4, SUC_lexicon_tags_11 },
    { 0x6f56fb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4556fbb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9156fbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b56fc1, SUC_lexicon_tags_63 },
    { 0x2d956fc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7756fc4, SUC_lexicon_tags_57 },
    { 0xf9d56fc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b56fc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f56fce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42d56fd0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58356fd5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d56fd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x53956fdb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a156fe1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9756fe7, SUC_lexicon_tags_35 },
    { 0x9dd56fe8, SUC_lexicon_tags_37 },
    { 0x58156fe8, SUC_lexicon_tags_29 },
    { 0x87f56fea, SUC_lexicon_tags_37 },
    { 0xdeb56fe7, SUC_lexicon_tags_11 },
    { 0x64b56fec, SUC_lexicon_tags_15 },
    { 0x20156fea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d56ff1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc356ff3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x22756ff5, SUC_lexicon_tags_15 },
    { 0x42956ff6, SUC_lexicon_tags_26 },
    { 0xc3756ff7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb57007, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e15700a, SUC_lexicon_tags_25 },
    { 0xaab5700b, SUC_lexicon_tags_7 },
    { 0xf595700c, SUC_lexicon_tags_14 },
    { 0x575700b, SUC_lexicon_tags_27 },
    { 0x5d5700b, SUC_lexicon_tags_15 },
    { 0x4935700f, SUC_lexicon_tags_33 },
    { 0x2c357010, SUC_lexicon_tags_25 },
    { 0xbf357011, SUC_lexicon_tags_17 },
    { 0x7555700d, SUC_lexicon_tags_19 },
    { 0x6775700d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90d57015, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b57021, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90357024, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f57028, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f57030, SUC_lexicon_tags_40 },
    { 0xfc557030, SUC_lexicon_tags_47 },
    { 0x95957032, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4157034, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b57039, SUC_lexicon_tags_21 },
    { 0xe6b5703a, SUC_lexicon_tags_9 },
    { 0xb3d5703b, SUC_lexicon_tags_11 },
    { 0x8bd5703a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafb5703e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc157040, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd57043, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b57049, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b5704c, SUC_lexicon_tags_23 },
    { 0x435704c, SUC_lexicon_tags_11 },
    { 0x40d5704e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xba757050, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6f57052, SUC_lexicon_tags_15 },
    { 0x89757053, SUC_lexicon_tags_49 },
    { 0x7cb57054, SUC_lexicon_tags_12 },
    { 0x1b57055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6935705b, SUC_lexicon_tags_25 },
    { 0x9f35705b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc075705f, SUC_lexicon_tags_11 },
    { 0x33357060, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56157064, SUC_lexicon_tags_37 },
    { 0x1f157065, SUC_lexicon_tags_55 },
    { 0x98957066, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8875706a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd57079, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f5707f, SUC_lexicon_tags_7 },
    { 0x66757080, SUC_lexicon_tags_15 },
    { 0x52f57080, SUC_lexicon_tags_6 },
    { 0x7315707f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d57086, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xda557088, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f5708c, SUC_lexicon_tags_47 },
    { 0xd7b5708c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f157090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad57093, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b57096, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf75709a, SUC_lexicon_tags_4 },
    { 0xce35709b, SUC_lexicon_tags_17 },
    { 0xa8b5709b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445570a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9570a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103570ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b570b0, SUC_lexicon_tags_27 },
    { 0xc93570b1, SUC_lexicon_tags_38 },
    { 0x2cf570b1, SUC_lexicon_tags_25 },
    { 0xab1570b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87570bb, SUC_lexicon_tags_15 },
    { 0x951570bc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7f9570be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f570c1, SUC_lexicon_tags_25 },
    { 0xb8b570c1, SUC_lexicon_tags_484 },
    { 0x237570c1, SUC_lexicon_tags_25 },
    { 0xb8f570c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5570c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5b570c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b7570cb, SUC_lexicon_tags_30 },
    { 0x3a1570cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95570d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef570d5, SUC_lexicon_tags_33 },
    { 0x10f570d6, SUC_lexicon_tags_60 },
    { 0x6b5570d7, SUC_lexicon_tags_23 },
    { 0xb35570d8, SUC_lexicon_tags_29 },
    { 0x119570d9, SUC_lexicon_tags_25 },
    { 0xed9570d6, SUC_lexicon_tags_37 },
    { 0x317570d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf570de, SUC_lexicon_tags_12 },
    { 0x4ad570de, SUC_lexicon_tags_11 },
    { 0xbe7570e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d570e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff570ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d570f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41570f5, SUC_lexicon_tags_33 },
    { 0xff9570f6, SUC_lexicon_tags_8 },
    { 0x3bb570f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115570fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x595570fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d757100, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc757103, SUC_lexicon_tags_33 },
    { 0x30557104, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9af57106, SUC_lexicon_tags_12 },
    { 0xa3f57106, SUC_lexicon_tags_25 },
    { 0xd9f57108, SUC_lexicon_tags_25 },
    { 0xf1757109, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xce55710c, SUC_lexicon_tags_79 },
    { 0x9dd5710d, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0xa635710f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x79557111, SUC_lexicon_tags_11 },
    { 0x8ad57111, SUC_lexicon_tags_0 },
    { 0x1e357113, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb57116, SUC_lexicon_tags_7 },
    { 0xb4757117, SUC_lexicon_tags_37 },
    { 0x92957116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b5711b, SUC_lexicon_tags_15 },
    { 0x4c95711c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5955711e, SUC_lexicon_tags_57 },
    { 0xd535711f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f57122, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ab57124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9157128, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad557132, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9157136, SUC_lexicon_tags_45 },
    { 0xd8f57137, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d75713d, SUC_lexicon_tags_21 },
    { 0x7955713e, SUC_lexicon_tags_11 },
    { 0x8d55713d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb0f57141, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe2357143, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42157145, SUC_lexicon_tags_11 },
    { 0x7bf57146, SUC_lexicon_tags_7 },
    { 0xbeb57147, SUC_lexicon_tags_37 },
    { 0x157147, SUC_lexicon_tags_37 },
    { 0x66157147, SUC_lexicon_tags_15 },
    { 0x16d5714a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c15714e, SUC_lexicon_tags_43 },
    { 0x9a55714e, SUC_lexicon_tags_38 },
    { 0x15b57150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87b57152, SUC_lexicon_tags_0 },
    { 0x24f57153, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x11957155, SUC_lexicon_tags_15 },
    { 0x8c357156, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f5715f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d57165, SUC_lexicon_tags_1 },
    { 0x5ed57166, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87957170, SUC_lexicon_tags_17 },
    { 0x12157171, SUC_lexicon_tags_55 },
    { 0xbf357172, SUC_lexicon_tags_17 },
    { 0xbef57173, SUC_lexicon_tags_17 },
    { 0x7b357170, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x17357176, SUC_lexicon_tags_9 },
    { 0xc5957177, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34557182, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd157188, SUC_lexicon_tags_12 },
    { 0xd7957188, SUC_lexicon_tags_17 },
    { 0x6055718a, SUC_lexicon_tags_23 },
    { 0x41f5718b, SUC_lexicon_tags_11 },
    { 0x46b5718c, SUC_lexicon_tags_63 },
    { 0x4ff5718b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb57191, SUC_lexicon_tags_133 },
    { 0x21157192, SUC_lexicon_tags_84 },
    { 0x71b57193, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xee157196, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x40757198, SUC_lexicon_tags_31 },
    { 0xd4157199, SUC_lexicon_tags_81 },
    { 0x3f95719a, SUC_lexicon_tags_47 },
    { 0x6b35719b, SUC_lexicon_tags_59 },
    { 0xdab5719c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3035719f, SUC_lexicon_tags_14 },
    { 0x217571a0, SUC_lexicon_tags_63 },
    { 0x3395719f, SUC_lexicon_tags_87 },
    { 0x4e3571a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x281571a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1571a9, SUC_lexicon_tags_37 },
    { 0x72d571aa, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213571ae, SUC_lexicon_tags_0 },
    { 0x2ad571ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601571b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe95571b5, SUC_lexicon_tags_12 },
    { 0xc59571b5, SUC_lexicon_tags_37 },
    { 0x1a5571b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8db571b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389571bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75571c3, SUC_lexicon_tags_17 },
    { 0xda5571c4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211571c8, SUC_lexicon_tags_37 },
    { 0x9d571c9, SUC_lexicon_tags_673 },
    { 0x2e3571ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9571d1, SUC_lexicon_tags_66 },
    { 0x559571d2, SUC_lexicon_tags_15 },
    { 0x119571d2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5571d7, SUC_lexicon_tags_31 },
    { 0xb3f571d7, SUC_lexicon_tags_7 },
    { 0xc53571d9, SUC_lexicon_tags_25 },
    { 0xb4f571da, SUC_lexicon_tags_14 },
    { 0xccb571d8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc5571dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x689571df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x617571e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55571ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x153571ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169571f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45571fa, SUC_lexicon_tags_1 },
    { 0xe1571fb, SUC_lexicon_tags_26 },
    { 0xa9f571fb, SUC_lexicon_tags_12 },
    { 0x2ab571fc, SUC_lexicon_tags_23 },
    { 0xbf1571fe, SUC_lexicon_tags_33 },
    { 0xa9b571fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d357201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db57209, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb757213, SUC_lexicon_tags_17 },
    { 0x1dd57214, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b57217, SUC_lexicon_tags_8 },
    { 0x3b557217, SUC_lexicon_tags_45 },
    { 0x78b57218, SUC_lexicon_tags_17 },
    { 0xc7157219, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b57221, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f557226, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3975722d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaad5722f, SUC_lexicon_tags_30 },
    { 0xe495722f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f57235, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x12f57237, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x30d57239, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5723d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79157240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b57243, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6a957245, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8357249, SUC_lexicon_tags_65 },
    { 0x7d15724a, SUC_lexicon_tags_56 },
    { 0x78b57249, SUC_lexicon_tags_25 },
    { 0x5cd57249, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbfb5724e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb57256, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9975725c, SUC_lexicon_tags_15 },
    { 0x7715725d, SUC_lexicon_tags_2 },
    { 0xd155725e, SUC_lexicon_tags_33 },
    { 0x3c15725d, SUC_lexicon_tags_7 },
    { 0x4cb5725f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b57269, SUC_lexicon_tags_11 },
    { 0x72957269, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f5726d, SUC_lexicon_tags_6 },
    { 0x78d5726e, SUC_lexicon_tags_49 },
    { 0xc35726e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c957272, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23957275, SUC_lexicon_tags_70 },
    { 0x9b557275, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3157278, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf15727a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71f5727c, SUC_lexicon_tags_29 },
    { 0xeb75727d, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4357280, SUC_lexicon_tags_14 },
    { 0x69f57281, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12957286, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9b57288, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9295728e, SUC_lexicon_tags_25 },
    { 0x14f5728f, SUC_lexicon_tags_63 },
    { 0x90157290, SUC_lexicon_tags_43 },
    { 0x1c957291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f57294, SUC_lexicon_tags_2 },
    { 0x52957295, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b57298, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b5729e, SUC_lexicon_tags_1 },
    { 0xe515729f, SUC_lexicon_tags_0 },
    { 0x1035729f, SUC_lexicon_tags_60 },
    { 0x3195729f, SUC_lexicon_tags_12 },
    { 0xb6b572a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2df572a4, SUC_lexicon_tags_2 },
    { 0xbe1572a5, SUC_lexicon_tags_11 },
    { 0xef3572a6, SUC_lexicon_tags_17 },
    { 0x69f572a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x161572a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1cf572ab, SUC_lexicon_tags_33 },
    { 0x3a1572ab, SUC_lexicon_tags_26 },
    { 0x26b572ad, SUC_lexicon_tags_55 },
    { 0x337572ae, SUC_lexicon_tags_11 },
    { 0x6df572ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e9572b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x917572b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9b3572b6, SUC_lexicon_tags_38 },
    { 0x373572b7, SUC_lexicon_tags_46 },
    { 0x809572b7, SUC_lexicon_tags_415 },
    { 0xaeb572b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f572bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x747572c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d572c7, SUC_lexicon_tags_37 },
    { 0xbbb572c8, SUC_lexicon_tags_12 },
    { 0x297572c9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x38b572cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3572cf, SUC_lexicon_tags_17 },
    { 0x761572d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b572d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775572dd, SUC_lexicon_tags_6 },
    { 0xa13572de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1572e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb572e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287572e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f572f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5572f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6b572f8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x79d572fa, SUC_lexicon_tags_13 },
    { 0x3af572fb, SUC_lexicon_tags_17 },
    { 0x371572fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe5d572fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e557303, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdb957305, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xceb57307, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d5730a, SUC_lexicon_tags_37 },
    { 0x9395730b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6715730e, SUC_lexicon_tags_25 },
    { 0x1615730f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60557311, SUC_lexicon_tags_41 },
    { 0x97f57312, SUC_lexicon_tags_58 },
    { 0xaa357312, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x14757315, SUC_lexicon_tags_17 },
    { 0xec557316, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x21d57318, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a75731b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c357324, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d957326, SUC_lexicon_tags_15 },
    { 0x9df57327, SUC_lexicon_tags_9 },
    { 0xb0157328, SUC_lexicon_tags_25 },
    { 0x6c157329, SUC_lexicon_tags_33 },
    { 0x6335732a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc55732c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc515732f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8557331, SUC_lexicon_tags_389 },
    { 0xc9357332, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb357334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacf57336, SUC_lexicon_tags_25 },
    { 0xfa557337, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5f157339, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c75733c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe757342, SUC_lexicon_tags_1 },
    { 0xd1557343, SUC_lexicon_tags_49 },
    { 0x79157342, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x44357346, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5875734c, SUC_lexicon_tags_11 },
    { 0xde95734d, SUC_lexicon_tags_7 },
    { 0xab55734e, SUC_lexicon_tags_14 },
    { 0xcd95734f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10d57351, SUC_lexicon_tags_47 },
    { 0xe6357352, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d57357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2d57359, SUC_lexicon_tags_60 },
    { 0x3cd5735a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29357361, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xebd57363, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b57368, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b95736a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8855736d, SUC_lexicon_tags_238 },
    { 0x4915736e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11357376, SUC_lexicon_tags_87 },
    { 0xf9357377, SUC_lexicon_tags_7 },
    { 0xd9557378, SUC_lexicon_tags_16 },
    { 0xe4b57379, SUC_lexicon_tags_15 },
    { 0xaed57377, SUC_lexicon_tags_12 },
    { 0xd9f5737a, SUC_lexicon_tags_7 },
    { 0x1d75737b, SUC_lexicon_tags_10 },
    { 0x4dd5737d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc557380, SUC_lexicon_tags_25 },
    { 0xe957381, SUC_lexicon_tags_2 },
    { 0xb3557382, SUC_lexicon_tags_45 },
    { 0x25b57383, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc557386, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f5738a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b5738f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7d57391, SUC_lexicon_tags_60 },
    { 0x53557391, SUC_lexicon_tags_15 },
    { 0xe9557393, SUC_lexicon_tags_17 },
    { 0x7157391, SUC_lexicon_tags_1 },
    { 0x79157393, SUC_lexicon_tags_46 },
    { 0x50f57393, SUC_lexicon_tags_37 },
    { 0x65f57397, SUC_lexicon_tags_87 },
    { 0x15f57398, SUC_lexicon_tags_17 },
    { 0xe2d57399, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a55739c, SUC_lexicon_tags_28 },
    { 0x1db5739c, SUC_lexicon_tags_7 },
    { 0x5635739e, SUC_lexicon_tags_60 },
    { 0xf615739c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d573a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b3573a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1573ab, SUC_lexicon_tags_47 },
    { 0x7b9573ab, SUC_lexicon_tags_17 },
    { 0x7d3573ad, SUC_lexicon_tags_15 },
    { 0xf89573ae, SUC_lexicon_tags_15 },
    { 0xd5f573af, SUC_lexicon_tags_8 },
    { 0xed7573af, SUC_lexicon_tags_69 },
    { 0xb11573b1, SUC_lexicon_tags_17 },
    { 0x5f7573b2, SUC_lexicon_tags_46 },
    { 0x483573b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90b573b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f573b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1573bc, SUC_lexicon_tags_28 },
    { 0x845573bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x795573bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x197573c2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b573c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd573c9, SUC_lexicon_tags_33 },
    { 0xb7b573ca, SUC_lexicon_tags_16 },
    { 0x507573cb, SUC_lexicon_tags_15 },
    { 0x4c1573c9, SUC_lexicon_tags_65 },
    { 0x919573cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff573d0, SUC_lexicon_tags_7 },
    { 0x99f573d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x647573d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d573d7, SUC_lexicon_tags_17 },
    { 0xea3573d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x11d573da, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb9d573dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe23573de, SUC_lexicon_tags_12 },
    { 0x3dd573df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f573ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279573ee, SUC_lexicon_tags_37 },
    { 0x777573ef, SUC_lexicon_tags_7 },
    { 0x3cf573ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc55573f2, SUC_lexicon_tags_30 },
    { 0xe39573f3, SUC_lexicon_tags_23 },
    { 0xb3d573f3, SUC_lexicon_tags_7 },
    { 0xe59573f5, SUC_lexicon_tags_15 },
    { 0x631573f5, SUC_lexicon_tags_33 },
    { 0x65b573f4, SUC_lexicon_tags_14 },
    { 0x897573f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x523573fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9573fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d57400, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb557403, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x42957406, SUC_lexicon_tags_12 },
    { 0xa0d57407, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd935740a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2455740c, SUC_lexicon_tags_0 },
    { 0xde15740c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1f5740f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6fb57411, SUC_lexicon_tags_37 },
    { 0x6e757412, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d57415, SUC_lexicon_tags_0 },
    { 0x8157416, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3957418, SUC_lexicon_tags_77 },
    { 0xa6757419, SUC_lexicon_tags_49 },
    { 0x3835741a, SUC_lexicon_tags_149 },
    { 0x88d5741b, SUC_lexicon_tags_37 },
    { 0xe1f5741b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f57421, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e357427, SUC_lexicon_tags_29 },
    { 0x95b57428, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef5742b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5742f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37357434, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd757436, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78957440, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6757444, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfe757446, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2075744e, SUC_lexicon_tags_60 },
    { 0x1915744f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5d757451, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9357455, SUC_lexicon_tags_63 },
    { 0x5a757456, SUC_lexicon_tags_14 },
    { 0x5f957457, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f5745c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68757461, SUC_lexicon_tags_10 },
    { 0xeb957462, SUC_lexicon_tags_11 },
    { 0x98d57462, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19957467, SUC_lexicon_tags_29 },
    { 0x1e557468, SUC_lexicon_tags_17 },
    { 0xbe957468, SUC_lexicon_tags_12 },
    { 0x87157469, SUC_lexicon_tags_1 },
    { 0xec157468, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x83f5746d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7757475, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e557479, SUC_lexicon_tags_47 },
    { 0x86357479, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xef35747c, SUC_lexicon_tags_1 },
    { 0xe8f5747d, SUC_lexicon_tags_37 },
    { 0x6d15747c, SUC_lexicon_tags_29 },
    { 0xc375747f, SUC_lexicon_tags_61 },
    { 0x2a55747f, SUC_lexicon_tags_0 },
    { 0xb6d5747d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73f57484, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf5748d, SUC_lexicon_tags_43 },
    { 0x4e95748d, SUC_lexicon_tags_28 },
    { 0xb8d5748f, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81757499, SUC_lexicon_tags_19 },
    { 0x7035749a, SUC_lexicon_tags_1 },
    { 0xecb5749b, SUC_lexicon_tags_79 },
    { 0x44b5749b, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39574a0, SUC_lexicon_tags_129 },
    { 0xa9f574a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee3574a3, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f574a6, SUC_lexicon_tags_38 },
    { 0x15d574a7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b574aa, SUC_lexicon_tags_7 },
    { 0xab574ab, SUC_lexicon_tags_2 },
    { 0xdd5574ac, SUC_lexicon_tags_57 },
    { 0x5a5574ac, SUC_lexicon_tags_11 },
    { 0x2d5574ae, SUC_lexicon_tags_11 },
    { 0x1b1574aa, SUC_lexicon_tags_38 },
    { 0x405574ae, SUC_lexicon_tags_47 },
    { 0x637574ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d574c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69574c6, SUC_lexicon_tags_15 },
    { 0xfb9574c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7574cd, SUC_lexicon_tags_2 },
    { 0xc13574cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11574d1, SUC_lexicon_tags_1 },
    { 0x9c3574d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbe7574d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899574d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57574dc, SUC_lexicon_tags_26 },
    { 0xb91574dc, SUC_lexicon_tags_2 },
    { 0x9e3574de, SUC_lexicon_tags_37 },
    { 0xd85574dc, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817574e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1c9574e5, SUC_lexicon_tags_15 },
    { 0xb91574e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ff574e8, SUC_lexicon_tags_200 },
    { 0, NULL },
    { 0xecf574ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab574ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x13574f1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xbfd574f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325574fa, SUC_lexicon_tags_160 },
    { 0x7ab574fb, SUC_lexicon_tags_60 },
    { 0x4d1574fb, SUC_lexicon_tags_7 },
    { 0x651574fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x93574ff, SUC_lexicon_tags_42 },
    { 0xf75574ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67957502, SUC_lexicon_tags_37 },
    { 0xa8357503, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe557505, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf157508, SUC_lexicon_tags_9 },
    { 0x3c157509, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf4d5750b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd615750d, SUC_lexicon_tags_11 },
    { 0xc175750d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5157515, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95f57517, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6357519, SUC_lexicon_tags_31 },
    { 0x25f5751a, SUC_lexicon_tags_2 },
    { 0xe755751b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98557520, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46157525, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e557527, SUC_lexicon_tags_15 },
    { 0x39357528, SUC_lexicon_tags_39 },
    { 0x89957529, SUC_lexicon_tags_2 },
    { 0x9555752a, SUC_lexicon_tags_11 },
    { 0xd375752b, SUC_lexicon_tags_15 },
    { 0xca15752c, SUC_lexicon_tags_11 },
    { 0x2d35752d, SUC_lexicon_tags_17 },
    { 0xc9d5752a, SUC_lexicon_tags_1 },
    { 0xc1b5752f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf57531, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xced57533, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2b57535, SUC_lexicon_tags_27 },
    { 0xcc757535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b5753f, SUC_lexicon_tags_60 },
    { 0x6a95753f, SUC_lexicon_tags_12 },
    { 0xcf957541, SUC_lexicon_tags_28 },
    { 0x58f5753f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcdf57544, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa357548, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6995754a, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x6935754c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64157550, SUC_lexicon_tags_13 },
    { 0x9d557551, SUC_lexicon_tags_29 },
    { 0x7df57552, SUC_lexicon_tags_1 },
    { 0x8f957553, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d757556, SUC_lexicon_tags_43 },
    { 0x2f757557, SUC_lexicon_tags_62 },
    { 0xd5957558, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f5755c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3155755f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeed57561, SUC_lexicon_tags_23 },
    { 0x20757562, SUC_lexicon_tags_30 },
    { 0x3cf57561, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xca357566, SUC_lexicon_tags_33 },
    { 0xa3157566, SUC_lexicon_tags_45 },
    { 0x8cf57567, SUC_lexicon_tags_11 },
    { 0xacb57566, SUC_lexicon_tags_61 },
    { 0x8757567, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef57572, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f5757a, SUC_lexicon_tags_425 },
    { 0xa935757b, SUC_lexicon_tags_15 },
    { 0xd715757c, SUC_lexicon_tags_2 },
    { 0x98f5757c, SUC_lexicon_tags_74 },
    { 0xceb5757e, SUC_lexicon_tags_74 },
    { 0x8ff5757c, SUC_lexicon_tags_25 },
    { 0xfe75757c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b57583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b157588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd5758b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd095758e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19757592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd957595, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d357598, SUC_lexicon_tags_12 },
    { 0xa4d57598, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0, NULL },
    { 0xd855759c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d5759f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x869575a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1575aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3575ad, SUC_lexicon_tags_23 },
    { 0xf67575ae, SUC_lexicon_tags_14 },
    { 0x437575ae, SUC_lexicon_tags_14 },
    { 0x5fb575ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b575b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x229575b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x505575bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7575be, SUC_lexicon_tags_198 },
    { 0x2c1575be, SUC_lexicon_tags_8 },
    { 0xed3575c0, SUC_lexicon_tags_23 },
    { 0xbd5575be, SUC_lexicon_tags_33 },
    { 0x219575c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f575c7, SUC_lexicon_tags_15 },
    { 0x3d5575c7, SUC_lexicon_tags_2 },
    { 0x5b7575c9, SUC_lexicon_tags_65 },
    { 0x27b575c8, SUC_lexicon_tags_38 },
    { 0x66d575c7, SUC_lexicon_tags_11 },
    { 0xef9575cc, SUC_lexicon_tags_15 },
    { 0xb9575cd, SUC_lexicon_tags_60 },
    { 0x64f575c8, SUC_lexicon_tags_60 },
    { 0x9ef575cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1d3575d1, SUC_lexicon_tags_85 },
    { 0xefd575d2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47575dc, SUC_lexicon_tags_23 },
    { 0x719575dd, SUC_lexicon_tags_197 },
    { 0xf57575de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43575e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17575e9, SUC_lexicon_tags_45 },
    { 0xa75575e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d575ee, SUC_lexicon_tags_8 },
    { 0x7a3575ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6ef575f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15575f6, SUC_lexicon_tags_13 },
    { 0xa1d575f7, SUC_lexicon_tags_25 },
    { 0xa9575f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x437575fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x781575fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b557600, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee357603, SUC_lexicon_tags_160 },
    { 0xe6f57604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e357606, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcc557608, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c357610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd157614, SUC_lexicon_tags_25 },
    { 0x1d157615, SUC_lexicon_tags_12 },
    { 0x6f357616, SUC_lexicon_tags_37 },
    { 0x4b57616, SUC_lexicon_tags_38 },
    { 0x51d57616, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8b5761a, SUC_lexicon_tags_1 },
    { 0x4d75761a, SUC_lexicon_tags_27 },
    { 0xadf5761c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a957621, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb157625, SUC_lexicon_tags_11 },
    { 0x83557626, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x46957628, SUC_lexicon_tags_20 },
    { 0xce157628, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d5762c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd255762f, SUC_lexicon_tags_8 },
    { 0xb835762f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x47757632, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd57637, SUC_lexicon_tags_12 },
    { 0x52557637, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d5763b, SUC_lexicon_tags_17 },
    { 0x88f5763c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe357642, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf57645, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c75764e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x37357651, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x78157653, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d57657, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d5765c, SUC_lexicon_tags_38 },
    { 0x9a75765d, SUC_lexicon_tags_0 },
    { 0xa835765e, SUC_lexicon_tags_14 },
    { 0x6cd5765f, SUC_lexicon_tags_12 },
    { 0xf155765f, SUC_lexicon_tags_29 },
    { 0x7c15765e, SUC_lexicon_tags_55 },
    { 0x9b35765e, SUC_lexicon_tags_12 },
    { 0x5f157663, SUC_lexicon_tags_38 },
    { 0x2af57664, SUC_lexicon_tags_39 },
    { 0x84757665, SUC_lexicon_tags_13 },
    { 0xb2f57666, SUC_lexicon_tags_1 },
    { 0x71157667, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4635766d, SUC_lexicon_tags_15 },
    { 0x9515766e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1157670, SUC_lexicon_tags_33 },
    { 0x3b757671, SUC_lexicon_tags_87 },
    { 0x3f157672, SUC_lexicon_tags_25 },
    { 0x83157673, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35757678, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b5767b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d5767e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e157682, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29f57684, SUC_lexicon_tags_17 },
    { 0x6e557685, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4d57687, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d5768b, SUC_lexicon_tags_48 },
    { 0xacf5768c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf835768e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f57691, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f57695, SUC_lexicon_tags_31 },
    { 0x5d957696, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e35769a, SUC_lexicon_tags_17 },
    { 0xcdd5769b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5576a3, SUC_lexicon_tags_12 },
    { 0xe1b576a4, SUC_lexicon_tags_46 },
    { 0x863576a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x331576a7, SUC_lexicon_tags_63 },
    { 0xeb1576a8, SUC_lexicon_tags_9 },
    { 0x719576a8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x745576ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737576b2, SUC_lexicon_tags_25 },
    { 0xa67576b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3d576b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51b576b7, SUC_lexicon_tags_43 },
    { 0xb53576b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1576bd, SUC_lexicon_tags_0 },
    { 0x71f576be, SUC_lexicon_tags_12 },
    { 0x12f576bf, SUC_lexicon_tags_33 },
    { 0xb39576c0, SUC_lexicon_tags_65 },
    { 0xe8f576be, SUC_lexicon_tags_49 },
    { 0x66b576c2, SUC_lexicon_tags_12 },
    { 0xf4b576c3, SUC_lexicon_tags_46 },
    { 0xacb576c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d576c8, SUC_lexicon_tags_37 },
    { 0xb3576c9, SUC_lexicon_tags_87 },
    { 0x9db576ca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9576ce, SUC_lexicon_tags_66 },
    { 0x44f576cf, SUC_lexicon_tags_15 },
    { 0xf3b576d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe51576d2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x733576d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33576db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959576e0, SUC_lexicon_tags_7 },
    { 0xb89576e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x573576e4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7576e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d576e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad576ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f576f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3576f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x451576fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b576fd, SUC_lexicon_tags_49 },
    { 0x72d576fe, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x93357701, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd57704, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x44d57706, SUC_lexicon_tags_8 },
    { 0xc7d57706, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2995770b, SUC_lexicon_tags_36 },
    { 0x3c55770b, SUC_lexicon_tags_7 },
    { 0x3f35770d, SUC_lexicon_tags_47 },
    { 0x4815770e, SUC_lexicon_tags_19 },
    { 0xc8d5770f, SUC_lexicon_tags_37 },
    { 0x98b57710, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe1b57712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8957717, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f5771a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6bf5771c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb8d5771e, SUC_lexicon_tags_77 },
    { 0xb0b5771f, SUC_lexicon_tags_68 },
    { 0x5b357720, SUC_lexicon_tags_45 },
    { 0x7b157720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef57728, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4b5772a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfd5772c, SUC_lexicon_tags_12 },
    { 0x86d5772d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab95772f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36b57731, SUC_lexicon_tags_4 },
    { 0x1a357732, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaf357734, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x19357737, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3715773a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb5773d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6835773f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x68d57741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb5774a, SUC_lexicon_tags_34 },
    { 0x3675774a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d95774e, SUC_lexicon_tags_43 },
    { 0xe0d5774e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x59357751, SUC_lexicon_tags_12 },
    { 0x26b57752, SUC_lexicon_tags_8 },
    { 0x3157751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1357755, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3f557757, SUC_lexicon_tags_33 },
    { 0xfd957757, SUC_lexicon_tags_12 },
    { 0x45557759, SUC_lexicon_tags_37 },
    { 0x9dd5775a, SUC_lexicon_tags_12 },
    { 0x38b57759, SUC_lexicon_tags_47 },
    { 0x98d5775c, SUC_lexicon_tags_2 },
    { 0x6b5775d, SUC_lexicon_tags_17 },
    { 0x4635775d, SUC_lexicon_tags_38 },
    { 0x9b75775e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1f57761, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d57764, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65b57766, SUC_lexicon_tags_43 },
    { 0xce357766, SUC_lexicon_tags_15 },
    { 0x4357768, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9575776b, SUC_lexicon_tags_17 },
    { 0x80f5776c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x4095776e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7757773, SUC_lexicon_tags_29 },
    { 0xee357774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb57778, SUC_lexicon_tags_19 },
    { 0xf7f57778, SUC_lexicon_tags_21 },
    { 0xfdb57778, SUC_lexicon_tags_82 },
    { 0xcab5777a, SUC_lexicon_tags_8 },
    { 0xce55777c, SUC_lexicon_tags_2 },
    { 0x5655777c, SUC_lexicon_tags_11 },
    { 0x3535777e, SUC_lexicon_tags_26 },
    { 0xaa55777b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3957781, SUC_lexicon_tags_30 },
    { 0xe9b57781, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38957785, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a957787, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7475778a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f75778d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f95778f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99157792, SUC_lexicon_tags_1 },
    { 0x56f57792, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b57799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e95779c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81577a1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59577a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719577ae, SUC_lexicon_tags_2 },
    { 0x2e3577af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83f577b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xadb577b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f577b7, SUC_lexicon_tags_39 },
    { 0x275577b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff577bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11577c1, SUC_lexicon_tags_37 },
    { 0xb8b577c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe63577c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69577c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd99577c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d577cd, SUC_lexicon_tags_25 },
    { 0xbf7577cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f577d2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x673577d4, SUC_lexicon_tags_31 },
    { 0x9f9577d4, SUC_lexicon_tags_45 },
    { 0x9cd577d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757577dd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3577e1, SUC_lexicon_tags_27 },
    { 0x525577e2, SUC_lexicon_tags_46 },
    { 0x637577e3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3577e7, SUC_lexicon_tags_8 },
    { 0xe8f577e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd29577ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x683577ec, SUC_lexicon_tags_15 },
    { 0x253577ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797577f1, SUC_lexicon_tags_37 },
    { 0x17577f1, SUC_lexicon_tags_11 },
    { 0xbd9577f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7577f6, SUC_lexicon_tags_15 },
    { 0x727577f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973577fd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x66f577ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5557803, SUC_lexicon_tags_37 },
    { 0xc4357804, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b157806, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9757808, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8215780b, SUC_lexicon_tags_38 },
    { 0x7855780c, SUC_lexicon_tags_11 },
    { 0xfef5780d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66957816, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x98157818, SUC_lexicon_tags_37 },
    { 0x15557819, SUC_lexicon_tags_38 },
    { 0x8bb57819, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d35781c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58b5781e, SUC_lexicon_tags_11 },
    { 0x7fd5781f, SUC_lexicon_tags_17 },
    { 0x86757820, SUC_lexicon_tags_43 },
    { 0x56357821, SUC_lexicon_tags_30 },
    { 0x4fb57821, SUC_lexicon_tags_7 },
    { 0xb4d57823, SUC_lexicon_tags_11 },
    { 0x16157823, SUC_lexicon_tags_15 },
    { 0x7ad57825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20b57827, SUC_lexicon_tags_17 },
    { 0xad357827, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a95782e, SUC_lexicon_tags_38 },
    { 0x3035782f, SUC_lexicon_tags_1 },
    { 0xe3957830, SUC_lexicon_tags_37 },
    { 0x2e157831, SUC_lexicon_tags_15 },
    { 0xb3b5782e, SUC_lexicon_tags_6 },
    { 0x50157833, SUC_lexicon_tags_16 },
    { 0x91957832, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52357839, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2575783e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae157844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce557847, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95157849, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa235784c, SUC_lexicon_tags_19 },
    { 0xf135784d, SUC_lexicon_tags_9 },
    { 0x575784c, SUC_lexicon_tags_14 },
    { 0xb4f5784f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2035785d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd57860, SUC_lexicon_tags_1 },
    { 0x94d57861, SUC_lexicon_tags_16 },
    { 0xe6d57861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5957865, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3357867, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd5786a, SUC_lexicon_tags_26 },
    { 0xd915786b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71557871, SUC_lexicon_tags_37 },
    { 0xbd557872, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f55787e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34757880, SUC_lexicon_tags_11 },
    { 0xef757881, SUC_lexicon_tags_15 },
    { 0xff557882, SUC_lexicon_tags_2 },
    { 0xa157882, SUC_lexicon_tags_1 },
    { 0x1fb57882, SUC_lexicon_tags_11 },
    { 0x3c957880, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xaa757887, SUC_lexicon_tags_28 },
    { 0x8757888, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43357890, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb557895, SUC_lexicon_tags_14 },
    { 0x5357896, SUC_lexicon_tags_2 },
    { 0x62357897, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4157899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x615789b, SUC_lexicon_tags_108 },
    { 0x475789b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5578a5, SUC_lexicon_tags_29 },
    { 0x907578a6, SUC_lexicon_tags_8 },
    { 0x783578a7, SUC_lexicon_tags_65 },
    { 0x7578a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d578ab, SUC_lexicon_tags_1 },
    { 0xe4b578ab, SUC_lexicon_tags_25 },
    { 0x775578ad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f578b0, SUC_lexicon_tags_16 },
    { 0x679578b0, SUC_lexicon_tags_44 },
    { 0xe49578b2, SUC_lexicon_tags_16 },
    { 0x3b9578b1, SUC_lexicon_tags_8 },
    { 0xda9578b4, SUC_lexicon_tags_39 },
    { 0xf85578b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6b578b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x497578ba, SUC_lexicon_tags_8 },
    { 0x7bf578bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cf578bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x727578c0, SUC_lexicon_tags_45 },
    { 0x5eb578c1, SUC_lexicon_tags_17 },
    { 0x443578c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa69578c4, SUC_lexicon_tags_16 },
    { 0xe33578c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d578ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x243578d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b578d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3f1578d6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x329578d8, SUC_lexicon_tags_62 },
    { 0xa05578d9, SUC_lexicon_tags_39 },
    { 0x25f578da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x485578dd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f578e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb578e6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d578e9, SUC_lexicon_tags_49 },
    { 0xc9578e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b578ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59f578f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a7578f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9578f5, SUC_lexicon_tags_7 },
    { 0x24d578f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42b578f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd578fb, SUC_lexicon_tags_49 },
    { 0x545578fc, SUC_lexicon_tags_38 },
    { 0x16f578fb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d57900, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e357904, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6195790a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1f5790c, SUC_lexicon_tags_12 },
    { 0xe0d5790d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d157910, SUC_lexicon_tags_11 },
    { 0x2e157911, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf135791a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94f5791c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f5791f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4157922, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc4f57924, SUC_lexicon_tags_37 },
    { 0x4db57924, SUC_lexicon_tags_7 },
    { 0x2ad57926, SUC_lexicon_tags_12 },
    { 0x35757927, SUC_lexicon_tags_17 },
    { 0x2f357928, SUC_lexicon_tags_25 },
    { 0xbb557929, SUC_lexicon_tags_37 },
    { 0x4f757926, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd95792c, SUC_lexicon_tags_25 },
    { 0xcd15792c, SUC_lexicon_tags_17 },
    { 0xe855792e, SUC_lexicon_tags_0 },
    { 0xe9f5792f, SUC_lexicon_tags_25 },
    { 0xda357930, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac957932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88957934, SUC_lexicon_tags_10 },
    { 0x21f57935, SUC_lexicon_tags_7 },
    { 0xb2557935, SUC_lexicon_tags_5 },
    { 0x2bf57937, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b95793a, SUC_lexicon_tags_14 },
    { 0x9335793b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f57940, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f757943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f57947, SUC_lexicon_tags_11 },
    { 0x9a357947, SUC_lexicon_tags_43 },
    { 0x37d57948, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8335794c, SUC_lexicon_tags_66 },
    { 0xee95794d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7157950, SUC_lexicon_tags_7 },
    { 0xf57951, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21557955, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf9557957, SUC_lexicon_tags_25 },
    { 0x98b57957, SUC_lexicon_tags_0 },
    { 0xd8d57958, SUC_lexicon_tags_32 },
    { 0xdb75795a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe35795e, SUC_lexicon_tags_63 },
    { 0x1fd5795f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f357963, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c557967, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b357969, SUC_lexicon_tags_2 },
    { 0x2b55796a, SUC_lexicon_tags_11 },
    { 0xd775796b, SUC_lexicon_tags_10 },
    { 0x8e55796c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82b5796e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b757974, SUC_lexicon_tags_27 },
    { 0x7d557975, SUC_lexicon_tags_26 },
    { 0xf57975, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5797b, SUC_lexicon_tags_7 },
    { 0xb655797c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd357984, SUC_lexicon_tags_28 },
    { 0x5b957985, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x60b57987, SUC_lexicon_tags_11 },
    { 0x33f57987, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b5798b, SUC_lexicon_tags_31 },
    { 0x79f5798c, SUC_lexicon_tags_37 },
    { 0x77f5798c, SUC_lexicon_tags_25 },
    { 0x57b5798e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3357990, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b157994, SUC_lexicon_tags_31 },
    { 0xe7b57995, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e957998, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b35799c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21579a3, SUC_lexicon_tags_11 },
    { 0x767579a4, SUC_lexicon_tags_33 },
    { 0x579579a5, SUC_lexicon_tags_11 },
    { 0xa3579a6, SUC_lexicon_tags_65 },
    { 0x381579a7, SUC_lexicon_tags_60 },
    { 0xddf579a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x505579aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x25d579ac, SUC_lexicon_tags_25 },
    { 0x6fb579ad, SUC_lexicon_tags_5 },
    { 0xc57579ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989579b3, SUC_lexicon_tags_60 },
    { 0x45b579b3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801579b8, SUC_lexicon_tags_23 },
    { 0x6a7579b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939579be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91579c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2f579c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5579c6, SUC_lexicon_tags_17 },
    { 0xfd1579c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9579d1, SUC_lexicon_tags_30 },
    { 0xf9b579d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b579d5, SUC_lexicon_tags_15 },
    { 0x24d579d6, SUC_lexicon_tags_1 },
    { 0x29b579d7, SUC_lexicon_tags_7 },
    { 0xd4d579d8, SUC_lexicon_tags_37 },
    { 0x309579d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3eb579db, SUC_lexicon_tags_1694 },
    { 0x953579dc, SUC_lexicon_tags_17 },
    { 0xe65579dc, SUC_lexicon_tags_37 },
    { 0x545579dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff579e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55579e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb1579e7, SUC_lexicon_tags_30 },
    { 0x24d579e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb579f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x575579f2, SUC_lexicon_tags_1 },
    { 0x5c7579f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401579f7, SUC_lexicon_tags_14 },
    { 0x99b579f7, SUC_lexicon_tags_11 },
    { 0x979579f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14157a00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e557a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8f57a05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1157a07, SUC_lexicon_tags_31 },
    { 0x10557a07, SUC_lexicon_tags_7 },
    { 0xb957a09, SUC_lexicon_tags_11 },
    { 0x69157a0a, SUC_lexicon_tags_25 },
    { 0xb9b57a0a, SUC_lexicon_tags_49 },
    { 0xbdf57a09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57157a10, SUC_lexicon_tags_26 },
    { 0x5b757a11, SUC_lexicon_tags_48 },
    { 0x5b757a12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd157a14, SUC_lexicon_tags_45 },
    { 0xae357a15, SUC_lexicon_tags_37 },
    { 0xadd57a14, SUC_lexicon_tags_1 },
    { 0x3af57a15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea157a1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d757a1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64357a20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d757a22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b57a27, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4557a34, SUC_lexicon_tags_38 },
    { 0x91f57a34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b757a39, SUC_lexicon_tags_42 },
    { 0x3e757a3a, SUC_lexicon_tags_17 },
    { 0xd557a3b, SUC_lexicon_tags_14 },
    { 0x66d57a3b, SUC_lexicon_tags_1 },
    { 0x59357a3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f57a41, SUC_lexicon_tags_14 },
    { 0x79b57a41, SUC_lexicon_tags_49 },
    { 0x77d57a43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2d57a45, SUC_lexicon_tags_38 },
    { 0x5ab57a46, SUC_lexicon_tags_1 },
    { 0xf2d57a47, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9d157a49, SUC_lexicon_tags_33 },
    { 0x69357a4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af57a51, SUC_lexicon_tags_17 },
    { 0x53d57a52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11157a54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf57a57, SUC_lexicon_tags_30 },
    { 0xbe357a57, SUC_lexicon_tags_280 },
    { 0xeaf57a57, SUC_lexicon_tags_11 },
    { 0x8bb57a58, SUC_lexicon_tags_17 },
    { 0x87357a5a, SUC_lexicon_tags_25 },
    { 0x61157a5c, SUC_lexicon_tags_14 },
    { 0xcb557a5c, SUC_lexicon_tags_17 },
    { 0xfe757a5e, SUC_lexicon_tags_79 },
    { 0x4e157a5e, SUC_lexicon_tags_35 },
    { 0x41557a5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbaf57a62, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa957a64, SUC_lexicon_tags_11 },
    { 0xfb57a64, SUC_lexicon_tags_37 },
    { 0x8f57a66, SUC_lexicon_tags_9 },
    { 0xaef57a67, SUC_lexicon_tags_17 },
    { 0x55357a68, SUC_lexicon_tags_31 },
    { 0x77757a69, SUC_lexicon_tags_9 },
    { 0x14157a66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66157a75, SUC_lexicon_tags_17 },
    { 0xfab57a76, SUC_lexicon_tags_7 },
    { 0x71b57a76, SUC_lexicon_tags_15 },
    { 0xeb157a78, SUC_lexicon_tags_6 },
    { 0x2f157a75, SUC_lexicon_tags_82 },
    { 0xd3357a7a, SUC_lexicon_tags_33 },
    { 0x38b57a7b, SUC_lexicon_tags_2 },
    { 0x5c757a7b, SUC_lexicon_tags_37 },
    { 0x1f57a7b, SUC_lexicon_tags_25 },
    { 0x95357a7e, SUC_lexicon_tags_12 },
    { 0xb9b57a7e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd57a83, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x72f57a85, SUC_lexicon_tags_118 },
    { 0x64557a86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed57a89, SUC_lexicon_tags_28 },
    { 0x83f57a89, SUC_lexicon_tags_17 },
    { 0x47357a8a, SUC_lexicon_tags_6 },
    { 0xdb757a8c, SUC_lexicon_tags_35 },
    { 0x77157a8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f57a92, SUC_lexicon_tags_17 },
    { 0x33b57a93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f57a9c, SUC_lexicon_tags_25 },
    { 0x1bb57a9d, SUC_lexicon_tags_25 },
    { 0xc5b57a9c, SUC_lexicon_tags_12 },
    { 0x9f557a9c, SUC_lexicon_tags_34 },
    { 0xad357aa0, SUC_lexicon_tags_2 },
    { 0x68357aa0, SUC_lexicon_tags_14 },
    { 0xb0157a9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdf57aa4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb57aa7, SUC_lexicon_tags_27 },
    { 0xd0f57aa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde357aaa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d357aaf, SUC_lexicon_tags_82 },
    { 0x38357ab0, SUC_lexicon_tags_63 },
    { 0x90f57ab1, SUC_lexicon_tags_37 },
    { 0x73b57ab2, SUC_lexicon_tags_88 },
    { 0x29557ab3, SUC_lexicon_tags_2 },
    { 0xd5357ab2, SUC_lexicon_tags_27 },
    { 0x52d57ab5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12f57ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5157abd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84157ac5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb557ac7, SUC_lexicon_tags_25 },
    { 0xb7357ac8, SUC_lexicon_tags_1 },
    { 0xb6557ac9, SUC_lexicon_tags_53 },
    { 0x32757aca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed57ace, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d357ad1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x35357ad3, SUC_lexicon_tags_48 },
    { 0x56757ad3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d57ad9, SUC_lexicon_tags_32 },
    { 0xad957ada, SUC_lexicon_tags_17 },
    { 0x79d57adb, SUC_lexicon_tags_27 },
    { 0x29557ad9, SUC_lexicon_tags_15 },
    { 0x80f57adc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b57ae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1557ae4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9557aea, SUC_lexicon_tags_47 },
    { 0x2b57aea, SUC_lexicon_tags_1 },
    { 0x36b57aec, SUC_lexicon_tags_63 },
    { 0x1b157aed, SUC_lexicon_tags_28 },
    { 0x4fd57aee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d57af2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90557af7, SUC_lexicon_tags_49 },
    { 0xac157af8, SUC_lexicon_tags_29 },
    { 0x48b57af7, SUC_lexicon_tags_0 },
    { 0x74357afa, SUC_lexicon_tags_26 },
    { 0x51357af8, SUC_lexicon_tags_16 },
    { 0xba557afc, SUC_lexicon_tags_2 },
    { 0xa7757afc, SUC_lexicon_tags_43 },
    { 0xd8d57afe, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xded57b01, SUC_lexicon_tags_14 },
    { 0xd5357b02, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d57b0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36957b17, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b57b1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1557b1d, SUC_lexicon_tags_45 },
    { 0x4cf57b1e, SUC_lexicon_tags_23 },
    { 0x11357b1e, SUC_lexicon_tags_17 },
    { 0x96957b1d, SUC_lexicon_tags_28 },
    { 0x59357b21, SUC_lexicon_tags_25 },
    { 0x56157b1e, SUC_lexicon_tags_6 },
    { 0x5dd57b23, SUC_lexicon_tags_37 },
    { 0xad57b24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f57b27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd57b2a, SUC_lexicon_tags_26 },
    { 0xc7d57b2a, SUC_lexicon_tags_47 },
    { 0x29157b2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa557b2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9957b31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd757b34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b557b36, SUC_lexicon_tags_1 },
    { 0xdf957b37, SUC_lexicon_tags_34 },
    { 0xd8b57b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2957b3b, SUC_lexicon_tags_60 },
    { 0x7f157b3c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed57b42, SUC_lexicon_tags_4 },
    { 0x42757b43, SUC_lexicon_tags_15 },
    { 0x90957b43, SUC_lexicon_tags_38 },
    { 0x34f57b45, SUC_lexicon_tags_62 },
    { 0x7dd57b43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a757b49, SUC_lexicon_tags_4 },
    { 0x5ed57b4a, SUC_lexicon_tags_23 },
    { 0x3f157b4b, SUC_lexicon_tags_25 },
    { 0x27b57b4c, SUC_lexicon_tags_37 },
    { 0x32357b4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca757b4f, SUC_lexicon_tags_2 },
    { 0xc8b57b50, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5957b55, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d57b5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed57b61, SUC_lexicon_tags_11 },
    { 0x9557b62, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc8757b64, SUC_lexicon_tags_17 },
    { 0xc8f57b65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc7957b67, SUC_lexicon_tags_1 },
    { 0x28157b68, SUC_lexicon_tags_25 },
    { 0x2e557b69, SUC_lexicon_tags_69 },
    { 0x91957b68, SUC_lexicon_tags_25 },
    { 0xec557b69, SUC_lexicon_tags_17 },
    { 0xd2b57b6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0357b72, SUC_lexicon_tags_310 },
    { 0x10957b73, SUC_lexicon_tags_2 },
    { 0xee557b73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2557b78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1b57b7a, SUC_lexicon_tags_7 },
    { 0xb5357b7a, SUC_lexicon_tags_17 },
    { 0x7b957b7a, SUC_lexicon_tags_11 },
    { 0x9f557b7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d57b84, SUC_lexicon_tags_487 },
    { 0x23357b85, SUC_lexicon_tags_11 },
    { 0x9a357b85, SUC_lexicon_tags_65 },
    { 0x2e557b85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db57b8c, SUC_lexicon_tags_15 },
    { 0xae957b8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x64957b8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f57b93, SUC_lexicon_tags_25 },
    { 0x33357b93, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x72157b97, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b57b9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfd57b9c, SUC_lexicon_tags_103 },
    { 0x11f57b9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79357ba4, SUC_lexicon_tags_25 },
    { 0x20d57ba4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d57ba8, SUC_lexicon_tags_17 },
    { 0x6af57ba8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73357bab, SUC_lexicon_tags_47 },
    { 0xbc957bac, SUC_lexicon_tags_7 },
    { 0x41757bad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe157baf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ef57bb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68557bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b57bb8, SUC_lexicon_tags_17 },
    { 0x1d57bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2357bbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b57bc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3757bc5, SUC_lexicon_tags_60 },
    { 0xd1557bc5, SUC_lexicon_tags_17 },
    { 0x5b157bc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xac557bc9, SUC_lexicon_tags_7 },
    { 0xc3f57bca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b57bce, SUC_lexicon_tags_13 },
    { 0x69b57bcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca757bd1, SUC_lexicon_tags_7 },
    { 0xdd57bd2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x16157bd5, SUC_lexicon_tags_19 },
    { 0x81957bd6, SUC_lexicon_tags_8 },
    { 0x95557bd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7957bd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee957bdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab357bde, SUC_lexicon_tags_1 },
    { 0x9b157bdf, SUC_lexicon_tags_14 },
    { 0x71157be0, SUC_lexicon_tags_8 },
    { 0x78f57be0, SUC_lexicon_tags_19 },
    { 0x8d57be0, SUC_lexicon_tags_7 },
    { 0x6b57be3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf57be6, SUC_lexicon_tags_1 },
    { 0x58757be6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb5357be9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe9557beb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ff57bed, SUC_lexicon_tags_7 },
    { 0xfbf57bee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cd57bf0, SUC_lexicon_tags_11 },
    { 0xf757bf0, SUC_lexicon_tags_9 },
    { 0xf4f57bf1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8557bf5, SUC_lexicon_tags_1 },
    { 0xdeb57bf6, SUC_lexicon_tags_47 },
    { 0xfdb57bf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c557bfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49157c00, SUC_lexicon_tags_37 },
    { 0x28d57c01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48757c04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95157c09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41557c0d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xb157c0f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xed57c11, SUC_lexicon_tags_7 },
    { 0x4b357c12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d57c17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef57c1c, SUC_lexicon_tags_32 },
    { 0xf6757c1d, SUC_lexicon_tags_29 },
    { 0x84957c1d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd57c22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d57c26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81957c2a, SUC_lexicon_tags_41 },
    { 0xa8157c2b, SUC_lexicon_tags_11 },
    { 0xe7d57c2c, SUC_lexicon_tags_15 },
    { 0x3ad57c2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cd57c2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb57c32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x26757c35, SUC_lexicon_tags_0 },
    { 0xab57c36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d357c38, SUC_lexicon_tags_17 },
    { 0x61f57c39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f57c3c, SUC_lexicon_tags_15 },
    { 0xddb57c3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2757c41, SUC_lexicon_tags_141 },
    { 0x80f57c42, SUC_lexicon_tags_54 },
    { 0x2b957c41, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b57c47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3957c4a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1cf57c4c, SUC_lexicon_tags_4 },
    { 0x89357c4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f57c52, SUC_lexicon_tags_2 },
    { 0x69557c53, SUC_lexicon_tags_60 },
    { 0x65d57c53, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef57c57, SUC_lexicon_tags_37 },
    { 0xe3757c58, SUC_lexicon_tags_0 },
    { 0xe4957c58, SUC_lexicon_tags_17 },
    { 0x9cf57c59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5557c60, SUC_lexicon_tags_2 },
    { 0xea957c60, SUC_lexicon_tags_12 },
    { 0x7cb57c62, SUC_lexicon_tags_31 },
    { 0x3fd57c62, SUC_lexicon_tags_49 },
    { 0xd5b57c62, SUC_lexicon_tags_2 },
    { 0x5d757c65, SUC_lexicon_tags_17 },
    { 0x50f57c61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10b57c68, SUC_lexicon_tags_2 },
    { 0x4f157c68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91b57c6b, SUC_lexicon_tags_15 },
    { 0x82957c6c, SUC_lexicon_tags_84 },
    { 0xe0557c6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78357c71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf57c7a, SUC_lexicon_tags_43 },
    { 0xfed57c7b, SUC_lexicon_tags_49 },
    { 0xf7357c7a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x48357c7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47557c82, SUC_lexicon_tags_7 },
    { 0xee357c83, SUC_lexicon_tags_47 },
    { 0x32157c82, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9557c88, SUC_lexicon_tags_33 },
    { 0x4f57c89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3557c8d, SUC_lexicon_tags_40 },
    { 0x4df57c8e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4eb57c90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fb57c92, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f57c9a, SUC_lexicon_tags_61 },
    { 0xd2557c9b, SUC_lexicon_tags_2 },
    { 0x59557c9a, SUC_lexicon_tags_46 },
    { 0x75b57c9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c157ca4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa0557ca6, SUC_lexicon_tags_31 },
    { 0xf0d57ca6, SUC_lexicon_tags_17 },
    { 0x27b57ca8, SUC_lexicon_tags_15 },
    { 0x58f57ca9, SUC_lexicon_tags_15 },
    { 0x7bd57ca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48957cac, SUC_lexicon_tags_6 },
    { 0xf2f57cac, SUC_lexicon_tags_193 },
    { 0xe6b57cad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4357cb2, SUC_lexicon_tags_11 },
    { 0x2f557cb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7957cbd, SUC_lexicon_tags_6 },
    { 0xb1557cbe, SUC_lexicon_tags_17 },
    { 0xa5757cbf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f57cc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d57cc6, SUC_lexicon_tags_13 },
    { 0xcc157cc7, SUC_lexicon_tags_7 },
    { 0xc5757cc8, SUC_lexicon_tags_66 },
    { 0x6157cc7, SUC_lexicon_tags_26 },
    { 0xaeb57cca, SUC_lexicon_tags_49 },
    { 0x71757ccb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd57cd0, SUC_lexicon_tags_49 },
    { 0xfa157cd1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f57cd7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c157cda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27157cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d57cdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d57ce3, SUC_lexicon_tags_87 },
    { 0x5d757ce4, SUC_lexicon_tags_37 },
    { 0x6cd57ce5, SUC_lexicon_tags_17 },
    { 0x17357ce5, SUC_lexicon_tags_25 },
    { 0x56157ce7, SUC_lexicon_tags_1 },
    { 0xf8f57ce8, SUC_lexicon_tags_27 },
    { 0xce357ce5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd757cec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f57cef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32757cf3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87557cfb, SUC_lexicon_tags_37 },
    { 0x6db57cfc, SUC_lexicon_tags_6 },
    { 0x43157cfd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f57d00, SUC_lexicon_tags_25 },
    { 0xf9157d01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a357d0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc957d0e, SUC_lexicon_tags_1 },
    { 0x1ed57d0f, SUC_lexicon_tags_57 },
    { 0xbe157d10, SUC_lexicon_tags_37 },
    { 0x43357d11, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb57d1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdaf57d1d, SUC_lexicon_tags_37 },
    { 0x26357d1e, SUC_lexicon_tags_27 },
    { 0x6ed57d1e, SUC_lexicon_tags_2 },
    { 0xb6957d20, SUC_lexicon_tags_38 },
    { 0x4e757d1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f57d29, SUC_lexicon_tags_1 },
    { 0x98157d2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c357d2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b57d32, SUC_lexicon_tags_11 },
    { 0xaff57d33, SUC_lexicon_tags_1 },
    { 0x4d57d33, SUC_lexicon_tags_61 },
    { 0xd6957d35, SUC_lexicon_tags_17 },
    { 0x84157d35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd2357d38, SUC_lexicon_tags_48 },
    { 0xc2b57d39, SUC_lexicon_tags_48 },
    { 0x34357d39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a957d3d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80557d42, SUC_lexicon_tags_5 },
    { 0x53d57d43, SUC_lexicon_tags_17 },
    { 0x7c157d43, SUC_lexicon_tags_33 },
    { 0x9f157d45, SUC_lexicon_tags_10 },
    { 0x46b57d43, SUC_lexicon_tags_19 },
    { 0x4b957d46, SUC_lexicon_tags_0 },
    { 0x7e157d48, SUC_lexicon_tags_7 },
    { 0x28157d49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32957d4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19d57d4e, SUC_lexicon_tags_7 },
    { 0x10357d4f, SUC_lexicon_tags_12 },
    { 0x43357d50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3757d52, SUC_lexicon_tags_4 },
    { 0xcf157d52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc757d56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5e557d58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x82357d5b, SUC_lexicon_tags_4 },
    { 0xd957d5c, SUC_lexicon_tags_37 },
    { 0x17d57d5d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81957d5f, SUC_lexicon_tags_31 },
    { 0x1cd57d60, SUC_lexicon_tags_0 },
    { 0x9c357d60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72157d63, SUC_lexicon_tags_51 },
    { 0xbdf57d64, SUC_lexicon_tags_55 },
    { 0xcf157d63, SUC_lexicon_tags_15 },
    { 0x35357d63, SUC_lexicon_tags_17 },
    { 0xb0957d67, SUC_lexicon_tags_37 },
    { 0xb5557d68, SUC_lexicon_tags_35 },
    { 0x88b57d69, SUC_lexicon_tags_17 },
    { 0xf0f57d68, SUC_lexicon_tags_60 },
    { 0xcf157d67, SUC_lexicon_tags_31 },
    { 0x86d57d69, SUC_lexicon_tags_68 },
    { 0x53757d6d, SUC_lexicon_tags_12 },
    { 0xa357d6d, SUC_lexicon_tags_29 },
    { 0xe357d68, SUC_lexicon_tags_38 },
    { 0xfc757d6b, SUC_lexicon_tags_33 },
    { 0x61357d71, SUC_lexicon_tags_37 },
    { 0x31757d71, SUC_lexicon_tags_28 },
    { 0xef157d6d, SUC_lexicon_tags_74 },
    { 0x36d57d74, SUC_lexicon_tags_37 },
    { 0x7b957d68, SUC_lexicon_tags_11 },
    { 0x54f57d76, SUC_lexicon_tags_2 },
    { 0x99b57d77, SUC_lexicon_tags_23 },
    { 0x39157d78, SUC_lexicon_tags_49 },
    { 0x3f757d79, SUC_lexicon_tags_31 },
    { 0xd3f57d7a, SUC_lexicon_tags_9 },
    { 0xe7957d7a, SUC_lexicon_tags_19 },
    { 0x58757d7a, SUC_lexicon_tags_18 },
    { 0x157d7a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbf357d7f, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f557d87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10357d91, SUC_lexicon_tags_11 },
    { 0xd4d57d92, SUC_lexicon_tags_11 },
    { 0xd0757d93, SUC_lexicon_tags_39 },
    { 0x3bb57d94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b757d96, SUC_lexicon_tags_38 },
    { 0x82157d97, SUC_lexicon_tags_49 },
    { 0x26957d98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54757da3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x55357da6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8e157da8, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d57dad, SUC_lexicon_tags_25 },
    { 0xcad57dae, SUC_lexicon_tags_37 },
    { 0xc3357dad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x51757db2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4e157db4, SUC_lexicon_tags_14 },
    { 0xa8157db4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62757db9, SUC_lexicon_tags_2 },
    { 0xa957dba, SUC_lexicon_tags_35 },
    { 0xa1f57dbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b57dc0, SUC_lexicon_tags_12 },
    { 0xde357dc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4557dc3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x85357dc6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x85757dc8, SUC_lexicon_tags_0 },
    { 0x97757dc9, SUC_lexicon_tags_30 },
    { 0x5a357dc9, SUC_lexicon_tags_25 },
    { 0x89b57dc9, SUC_lexicon_tags_17 },
    { 0x17757dc8, SUC_lexicon_tags_17 },
    { 0x49f57dc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12957dd1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af57dda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b157ddc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb57ddf, SUC_lexicon_tags_25 },
    { 0xf6757de0, SUC_lexicon_tags_1 },
    { 0x8c357de1, SUC_lexicon_tags_87 },
    { 0x5f557ddf, SUC_lexicon_tags_2 },
    { 0x84d57ddf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x38757de5, SUC_lexicon_tags_17 },
    { 0xf7d57de5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95d57de8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x68157deb, SUC_lexicon_tags_11 },
    { 0x98b57dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf57def, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1157df2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b57df8, SUC_lexicon_tags_94 },
    { 0xaaf57df9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15357dfd, SUC_lexicon_tags_45 },
    { 0xf3757dfd, SUC_lexicon_tags_30 },
    { 0x35157dff, SUC_lexicon_tags_23 },
    { 0xcc957dfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20957e05, SUC_lexicon_tags_15 },
    { 0x69357e06, SUC_lexicon_tags_66 },
    { 0x36157e07, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a557e0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39757e12, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f57e18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe157e1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf557e1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f57e22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb157e35, SUC_lexicon_tags_65 },
    { 0x61557e35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb6157e38, SUC_lexicon_tags_36 },
    { 0x5957e38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb57e40, SUC_lexicon_tags_15 },
    { 0xd7157e41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f57e44, SUC_lexicon_tags_2 },
    { 0xe5d57e45, SUC_lexicon_tags_25 },
    { 0x1d757e45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84957e48, SUC_lexicon_tags_49 },
    { 0xb6357e49, SUC_lexicon_tags_14 },
    { 0x5b57e49, SUC_lexicon_tags_17 },
    { 0xa6557e49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6557e50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53357e56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bf57e58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54557e5a, SUC_lexicon_tags_25 },
    { 0xc9157e5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x30157e5e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d557e61, SUC_lexicon_tags_235 },
    { 0xc5557e62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b57e65, SUC_lexicon_tags_37 },
    { 0xcfb57e66, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0xf3757e68, SUC_lexicon_tags_26 },
    { 0x78357e69, SUC_lexicon_tags_25 },
    { 0x48357e6a, SUC_lexicon_tags_11 },
    { 0x50d57e69, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9557e6e, SUC_lexicon_tags_11 },
    { 0x99957e6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xadf57e71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f57e75, SUC_lexicon_tags_14 },
    { 0xd6557e76, SUC_lexicon_tags_31 },
    { 0x10757e77, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb57e7a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b57e7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d57e83, SUC_lexicon_tags_23 },
    { 0xcc357e84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f557e86, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b57e8a, SUC_lexicon_tags_13 },
    { 0x67757e8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x90557e8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1957e91, SUC_lexicon_tags_8 },
    { 0xee757e92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6df57e94, SUC_lexicon_tags_15 },
    { 0xce157e94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3b57e97, SUC_lexicon_tags_39 },
    { 0x98157e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1357ea0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98157ea2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67f57ea4, SUC_lexicon_tags_23 },
    { 0x29957ea4, SUC_lexicon_tags_34 },
    { 0xd0157ea6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d557eb1, SUC_lexicon_tags_25 },
    { 0x1b557eb2, SUC_lexicon_tags_35 },
    { 0x68957eb1, SUC_lexicon_tags_12 },
    { 0x70d57eb4, SUC_lexicon_tags_138 },
    { 0xe0357eb5, SUC_lexicon_tags_8 },
    { 0x94157eb5, SUC_lexicon_tags_16 },
    { 0x3357eb7, SUC_lexicon_tags_11 },
    { 0x58b57eb7, SUC_lexicon_tags_38 },
    { 0xd8157eb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8757ebe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d57ec4, SUC_lexicon_tags_37 },
    { 0xf4957ec5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x41557ec8, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x3c757eca, SUC_lexicon_tags_10 },
    { 0x4d357ecb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff57ecf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f57ed2, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57557ed6, SUC_lexicon_tags_11 },
    { 0x41557ed6, SUC_lexicon_tags_17 },
    { 0x91357ed8, SUC_lexicon_tags_2 },
    { 0x9d157ed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba357edb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f57ede, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd57ee3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d57ee6, SUC_lexicon_tags_14 },
    { 0xc557ee7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b57eed, SUC_lexicon_tags_26 },
    { 0x78f57eed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b57ef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d757ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9957eff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfd57f01, SUC_lexicon_tags_47 },
    { 0x75357f01, SUC_lexicon_tags_25 },
    { 0xd8f57f03, SUC_lexicon_tags_33 },
    { 0x9c357f04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93f57f06, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d757f09, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf6d57f0b, SUC_lexicon_tags_45 },
    { 0x79d57f0b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77357f10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x34957f13, SUC_lexicon_tags_15 },
    { 0xf57f14, SUC_lexicon_tags_15 },
    { 0xb9f57f15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f57f19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f57f1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25557f1e, SUC_lexicon_tags_45 },
    { 0x14757f1e, SUC_lexicon_tags_17 },
    { 0x48b57f20, SUC_lexicon_tags_17 },
    { 0x7c757f21, SUC_lexicon_tags_6 },
    { 0x13757f21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4957f2c, SUC_lexicon_tags_7 },
    { 0x34357f2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63557f31, SUC_lexicon_tags_32 },
    { 0xcd357f32, SUC_lexicon_tags_17 },
    { 0xcd957f31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea157f38, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85357f3d, SUC_lexicon_tags_0 },
    { 0x70d57f3d, SUC_lexicon_tags_15 },
    { 0xa6d57f3d, SUC_lexicon_tags_6 },
    { 0x41f57f40, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c557f43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e757f45, SUC_lexicon_tags_26 },
    { 0x60357f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc757f48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc7157f4a, SUC_lexicon_tags_25 },
    { 0x3a957f4b, SUC_lexicon_tags_7 },
    { 0x47357f4c, SUC_lexicon_tags_38 },
    { 0xcdb57f4d, SUC_lexicon_tags_7 },
    { 0xc1d57f4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23557f50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xccf57f52, SUC_lexicon_tags_6 },
    { 0x65f57f52, SUC_lexicon_tags_33 },
    { 0xeb157f54, SUC_lexicon_tags_1 },
    { 0x1b757f52, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f57f5b, SUC_lexicon_tags_2 },
    { 0xdb557f5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x67b57f5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d57f62, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa6757f64, SUC_lexicon_tags_40 },
    { 0x3fb57f65, SUC_lexicon_tags_32 },
    { 0x13557f64, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x86757f68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b157f6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3157f6f, SUC_lexicon_tags_15 },
    { 0xe1f57f70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c157f75, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xea157f78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccf57f7a, SUC_lexicon_tags_44 },
    { 0x51757f7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda357f7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9357f83, SUC_lexicon_tags_1 },
    { 0xe9957f84, SUC_lexicon_tags_14 },
    { 0xe4f57f83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b57f88, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x80957f8a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2357f91, SUC_lexicon_tags_17 },
    { 0x93557f92, SUC_lexicon_tags_10 },
    { 0x1f157f93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51357f98, SUC_lexicon_tags_82 },
    { 0xacb57f99, SUC_lexicon_tags_38 },
    { 0x89b57f98, SUC_lexicon_tags_30 },
    { 0x47557f9a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b57fa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21957fa9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf57fae, SUC_lexicon_tags_32 },
    { 0x4157faf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14157fb2, SUC_lexicon_tags_148 },
    { 0xed757fb3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc557fb7, SUC_lexicon_tags_12 },
    { 0x13357fb8, SUC_lexicon_tags_69 },
    { 0xa4157fb9, SUC_lexicon_tags_29 },
    { 0x29d57fba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d757fc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6d57fc2, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x31357fc4, SUC_lexicon_tags_87 },
    { 0x9af57fc5, SUC_lexicon_tags_56 },
    { 0x9957fc4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d957fcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c357fcf, SUC_lexicon_tags_14 },
    { 0x30757fd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3957fd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0557fd6, SUC_lexicon_tags_65 },
    { 0x50b57fd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29757fdb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d57fde, SUC_lexicon_tags_25 },
    { 0x11f57fdf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6c157fe1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1557fe3, SUC_lexicon_tags_2 },
    { 0x48f57fe4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f57fe7, SUC_lexicon_tags_15 },
    { 0x9f757fe7, SUC_lexicon_tags_15 },
    { 0x7ab57fe8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83357fef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b57ff7, SUC_lexicon_tags_11 },
    { 0xfb57ff8, SUC_lexicon_tags_17 },
    { 0x20b57ff8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f957ffc, SUC_lexicon_tags_30 },
    { 0xa8b57ffd, SUC_lexicon_tags_23 },
    { 0x4e157ffe, SUC_lexicon_tags_38 },
    { 0xecd57fff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2758002, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b58005, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1558007, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f95800a, SUC_lexicon_tags_8 },
    { 0xe5f5800b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f5800e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bd58010, SUC_lexicon_tags_25 },
    { 0x84758011, SUC_lexicon_tags_2 },
    { 0x33f58012, SUC_lexicon_tags_8 },
    { 0xbd558011, SUC_lexicon_tags_17 },
    { 0x8d358012, SUC_lexicon_tags_7 },
    { 0x7ed58015, SUC_lexicon_tags_63 },
    { 0xc6158016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b58019, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf475801c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe495801f, SUC_lexicon_tags_37 },
    { 0x64f58020, SUC_lexicon_tags_31 },
    { 0xe558021, SUC_lexicon_tags_21 },
    { 0x43758020, SUC_lexicon_tags_91 },
    { 0x9bf58020, SUC_lexicon_tags_37 },
    { 0xcff58021, SUC_lexicon_tags_15 },
    { 0xe9d58024, SUC_lexicon_tags_37 },
    { 0x3e958022, SUC_lexicon_tags_35 },
    { 0x6cf58027, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28d58029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a95802e, SUC_lexicon_tags_136 },
    { 0x2255802f, SUC_lexicon_tags_47 },
    { 0x13958030, SUC_lexicon_tags_25 },
    { 0x7355802e, SUC_lexicon_tags_37 },
    { 0x8a958032, SUC_lexicon_tags_62 },
    { 0x28958033, SUC_lexicon_tags_7 },
    { 0x69b58031, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe358036, SUC_lexicon_tags_21 },
    { 0xe4358037, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa758039, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed5803c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf58040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7158046, SUC_lexicon_tags_68 },
    { 0xd2f58047, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f5804a, SUC_lexicon_tags_37 },
    { 0x6bd5804b, SUC_lexicon_tags_1 },
    { 0xe915804a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7958050, SUC_lexicon_tags_26 },
    { 0xd6358050, SUC_lexicon_tags_1 },
    { 0xbef58052, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7158055, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b58059, SUC_lexicon_tags_37 },
    { 0x9d15805a, SUC_lexicon_tags_32 },
    { 0xc8f5805b, SUC_lexicon_tags_11 },
    { 0x4475805c, SUC_lexicon_tags_17 },
    { 0x5555805c, SUC_lexicon_tags_15 },
    { 0x3295805b, SUC_lexicon_tags_27 },
    { 0xcc95805c, SUC_lexicon_tags_15 },
    { 0xedb5805b, SUC_lexicon_tags_17 },
    { 0xe495805e, SUC_lexicon_tags_19 },
    { 0xd4958062, SUC_lexicon_tags_7 },
    { 0x30f58063, SUC_lexicon_tags_14 },
    { 0x71b5805d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b958067, SUC_lexicon_tags_160 },
    { 0x6ff58067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf35806c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x675806e, SUC_lexicon_tags_17 },
    { 0x1535806f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d58076, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd5807b, SUC_lexicon_tags_60 },
    { 0xbcb5807c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4af5807e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51358080, SUC_lexicon_tags_15 },
    { 0x10b58081, SUC_lexicon_tags_1 },
    { 0x73558082, SUC_lexicon_tags_15 },
    { 0xc958083, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ad58085, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf3158087, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d5808e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x84d58090, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26958092, SUC_lexicon_tags_84 },
    { 0x15b58093, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25158098, SUC_lexicon_tags_47 },
    { 0x9c358099, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5809f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3580a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4af580a5, SUC_lexicon_tags_17 },
    { 0xab9580a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b580a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b580ae, SUC_lexicon_tags_17 },
    { 0x45b580ae, SUC_lexicon_tags_38 },
    { 0x175580b0, SUC_lexicon_tags_33 },
    { 0x8f3580af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463580b6, SUC_lexicon_tags_37 },
    { 0xe4d580b7, SUC_lexicon_tags_87 },
    { 0xe7b580b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3580c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x483580c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed580c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d580ca, SUC_lexicon_tags_25 },
    { 0xd4d580cb, SUC_lexicon_tags_27 },
    { 0x185580cc, SUC_lexicon_tags_37 },
    { 0x3e7580cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979580d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3ff580d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685580da, SUC_lexicon_tags_11 },
    { 0x175580db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd47580dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967580e1, SUC_lexicon_tags_11 },
    { 0x36b580e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1d580e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d580ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21580f3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc9b580f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x505580f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64d580fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x801580fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb1580ff, SUC_lexicon_tags_11 },
    { 0x3a558100, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf58103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee758105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3455810a, SUC_lexicon_tags_15 },
    { 0x2835810b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66558112, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d758114, SUC_lexicon_tags_31 },
    { 0x4df58115, SUC_lexicon_tags_37 },
    { 0x88358116, SUC_lexicon_tags_19 },
    { 0xa5358115, SUC_lexicon_tags_28 },
    { 0xc0d58118, SUC_lexicon_tags_42 },
    { 0x56358115, SUC_lexicon_tags_37 },
    { 0x55b5811a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd5811e, SUC_lexicon_tags_12 },
    { 0x2d5811f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d958121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82558123, SUC_lexicon_tags_47 },
    { 0x8158124, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25558128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5275812f, SUC_lexicon_tags_17 },
    { 0x92358130, SUC_lexicon_tags_31 },
    { 0x3cf58131, SUC_lexicon_tags_32 },
    { 0x5b758130, SUC_lexicon_tags_25 },
    { 0x4e358133, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86758139, SUC_lexicon_tags_19 },
    { 0xf2f5813a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6395813d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x33958140, SUC_lexicon_tags_37 },
    { 0xfd158140, SUC_lexicon_tags_7 },
    { 0x7d958142, SUC_lexicon_tags_47 },
    { 0xcc958143, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36758148, SUC_lexicon_tags_0 },
    { 0x78158149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95814d, SUC_lexicon_tags_0 },
    { 0x1b95814e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47558153, SUC_lexicon_tags_33 },
    { 0xa3358154, SUC_lexicon_tags_17 },
    { 0x9d158155, SUC_lexicon_tags_36 },
    { 0xea558156, SUC_lexicon_tags_7 },
    { 0x5d358157, SUC_lexicon_tags_103 },
    { 0x4a358157, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaaf5815a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4d5815c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xb095815e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b758163, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xee358166, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb675816c, SUC_lexicon_tags_10 },
    { 0xe215816c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8158172, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0558177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd5817b, SUC_lexicon_tags_2 },
    { 0xc95817c, SUC_lexicon_tags_11 },
    { 0x335817d, SUC_lexicon_tags_6 },
    { 0x3a35817e, SUC_lexicon_tags_47 },
    { 0x4f15817e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48758185, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f5818b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x77f5818d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26958195, SUC_lexicon_tags_7 },
    { 0xf9358196, SUC_lexicon_tags_11 },
    { 0xf9f58197, SUC_lexicon_tags_0 },
    { 0x15b58198, SUC_lexicon_tags_16 },
    { 0x14358198, SUC_lexicon_tags_25 },
    { 0x7655819a, SUC_lexicon_tags_32 },
    { 0x64b58198, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf315819f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb4b581a1, SUC_lexicon_tags_11 },
    { 0xe95581a2, SUC_lexicon_tags_28 },
    { 0x23b581a3, SUC_lexicon_tags_25 },
    { 0x1a7581a4, SUC_lexicon_tags_14 },
    { 0x45f581a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x215581a7, SUC_lexicon_tags_7 },
    { 0xfb3581a8, SUC_lexicon_tags_12 },
    { 0x8f9581a9, SUC_lexicon_tags_17 },
    { 0x8a3581a9, SUC_lexicon_tags_43 },
    { 0xb21581a7, SUC_lexicon_tags_7 },
    { 0x341581ac, SUC_lexicon_tags_43 },
    { 0xa43581ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7581b4, SUC_lexicon_tags_25 },
    { 0x653581b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5581b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7581bb, SUC_lexicon_tags_14 },
    { 0xee5581bc, SUC_lexicon_tags_11 },
    { 0xb7f581bc, SUC_lexicon_tags_15 },
    { 0x61f581be, SUC_lexicon_tags_25 },
    { 0x5d1581be, SUC_lexicon_tags_2 },
    { 0xe03581c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc45581c2, SUC_lexicon_tags_3 },
    { 0x8ad581c3, SUC_lexicon_tags_38 },
    { 0x6fb581c2, SUC_lexicon_tags_28 },
    { 0x4f5581c2, SUC_lexicon_tags_2 },
    { 0x3d3581c6, SUC_lexicon_tags_28 },
    { 0x45b581c6, SUC_lexicon_tags_17 },
    { 0x785581c8, SUC_lexicon_tags_11 },
    { 0x6e5581c9, SUC_lexicon_tags_12 },
    { 0xb81581c9, SUC_lexicon_tags_11 },
    { 0x9cd581cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb581ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d581d3, SUC_lexicon_tags_1 },
    { 0x2d7581d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5581dd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdf7581df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7581e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d581ea, SUC_lexicon_tags_30 },
    { 0x80f581eb, SUC_lexicon_tags_87 },
    { 0xef7581ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19581f0, SUC_lexicon_tags_17 },
    { 0x967581f1, SUC_lexicon_tags_15 },
    { 0xde5581f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41581fc, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c558200, SUC_lexicon_tags_23 },
    { 0x94b58201, SUC_lexicon_tags_77 },
    { 0xfd758201, SUC_lexicon_tags_34 },
    { 0xcef58203, SUC_lexicon_tags_49 },
    { 0x2c958204, SUC_lexicon_tags_11 },
    { 0xc9f58200, SUC_lexicon_tags_12 },
    { 0x89f58206, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b5820b, SUC_lexicon_tags_28 },
    { 0xc1d5820c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ef5820e, SUC_lexicon_tags_15 },
    { 0xd935820e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8158213, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa8f58215, SUC_lexicon_tags_25 },
    { 0xc4f58216, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f5821a, SUC_lexicon_tags_31 },
    { 0x1ad5821b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8595821d, SUC_lexicon_tags_15 },
    { 0x82b5821e, SUC_lexicon_tags_37 },
    { 0x92f5821d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f58222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d558224, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e558229, SUC_lexicon_tags_31 },
    { 0xbcd58229, SUC_lexicon_tags_1 },
    { 0xbd5822b, SUC_lexicon_tags_46 },
    { 0x41b58229, SUC_lexicon_tags_37 },
    { 0x2315822b, SUC_lexicon_tags_8 },
    { 0x7ff5822d, SUC_lexicon_tags_12 },
    { 0xc515822f, SUC_lexicon_tags_37 },
    { 0x50f5822f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x54358232, SUC_lexicon_tags_16 },
    { 0x8ef58233, SUC_lexicon_tags_17 },
    { 0xe2d58232, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x60158236, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7bf58238, SUC_lexicon_tags_37 },
    { 0xfab58238, SUC_lexicon_tags_2 },
    { 0xc095823a, SUC_lexicon_tags_23 },
    { 0x78f58238, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0xc8d5823d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d75824a, SUC_lexicon_tags_25 },
    { 0x175824b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15824f, SUC_lexicon_tags_230 },
    { 0xe8358250, SUC_lexicon_tags_17 },
    { 0xd3358250, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7a558253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7758259, SUC_lexicon_tags_13 },
    { 0x7955825a, SUC_lexicon_tags_60 },
    { 0x4415825b, SUC_lexicon_tags_19 },
    { 0xf035825a, SUC_lexicon_tags_25 },
    { 0x9975825d, SUC_lexicon_tags_68 },
    { 0x1bb5825c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7158265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c95826a, SUC_lexicon_tags_43 },
    { 0xc0f5826b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7158275, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae15827b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b35827f, SUC_lexicon_tags_17 },
    { 0x23958280, SUC_lexicon_tags_11 },
    { 0xe6d58281, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xca158283, SUC_lexicon_tags_7 },
    { 0xc8958284, SUC_lexicon_tags_71 },
    { 0x36358284, SUC_lexicon_tags_23 },
    { 0x7158284, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd58289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf515828b, SUC_lexicon_tags_47 },
    { 0xf5d5828c, SUC_lexicon_tags_1 },
    { 0x3915828d, SUC_lexicon_tags_39 },
    { 0xc595828d, SUC_lexicon_tags_21 },
    { 0xfd95828c, SUC_lexicon_tags_2 },
    { 0xb9b5828f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f158295, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb2758297, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb755829a, SUC_lexicon_tags_7 },
    { 0xab5829b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21582a1, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xe8f582a3, SUC_lexicon_tags_11 },
    { 0xc3f582a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b582a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad7582ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1b582ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d582b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2bf582b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d582be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x715582c0, SUC_lexicon_tags_91 },
    { 0x89d582c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17582c4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x799582c6, SUC_lexicon_tags_17 },
    { 0x7c5582c7, SUC_lexicon_tags_17 },
    { 0x251582c7, SUC_lexicon_tags_12 },
    { 0xa35582c9, SUC_lexicon_tags_25 },
    { 0xc95582c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x367582cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b5582cf, SUC_lexicon_tags_12 },
    { 0xee7582cf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965582d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5f582d7, SUC_lexicon_tags_22 },
    { 0x823582d8, SUC_lexicon_tags_47 },
    { 0xbcb582d8, SUC_lexicon_tags_49 },
    { 0x277582d9, SUC_lexicon_tags_45 },
    { 0x3a5582d7, SUC_lexicon_tags_2 },
    { 0x7f5582dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b582e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc69582e3, SUC_lexicon_tags_6 },
    { 0x659582e4, SUC_lexicon_tags_65 },
    { 0xc35582e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9582ed, SUC_lexicon_tags_11 },
    { 0x43b582ed, SUC_lexicon_tags_11 },
    { 0xab582ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd582f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x333582f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd09582f7, SUC_lexicon_tags_35 },
    { 0x4b1582f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9582fe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3558303, SUC_lexicon_tags_37 },
    { 0x6df58303, SUC_lexicon_tags_28 },
    { 0xe5d58305, SUC_lexicon_tags_32 },
    { 0x6f758305, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4a958308, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f5830c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32958315, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d58318, SUC_lexicon_tags_403 },
    { 0xb6758319, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6758321, SUC_lexicon_tags_11 },
    { 0x63758322, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb58326, SUC_lexicon_tags_11 },
    { 0x14d58327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef958329, SUC_lexicon_tags_60 },
    { 0x64158329, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70f5832c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2bf5832e, SUC_lexicon_tags_9 },
    { 0x8dd5832f, SUC_lexicon_tags_23 },
    { 0x54158330, SUC_lexicon_tags_0 },
    { 0xa6b58331, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f5833b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f58340, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2558343, SUC_lexicon_tags_25 },
    { 0xb3358344, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda758348, SUC_lexicon_tags_17 },
    { 0x1b158348, SUC_lexicon_tags_2 },
    { 0x2515834a, SUC_lexicon_tags_84 },
    { 0x8e35834b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f5834e, SUC_lexicon_tags_15 },
    { 0x6cf5834e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed58352, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b58359, SUC_lexicon_tags_31 },
    { 0xd1d5835a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7558362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b358365, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb6758367, SUC_lexicon_tags_2 },
    { 0xe5b58367, SUC_lexicon_tags_37 },
    { 0xdfb58367, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7915836b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3e35836d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee958371, SUC_lexicon_tags_9 },
    { 0xdcf58371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd58377, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df5837c, SUC_lexicon_tags_37 },
    { 0x5595837d, SUC_lexicon_tags_11 },
    { 0xb635837c, SUC_lexicon_tags_87 },
    { 0xf6b5837d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84358384, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7558387, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9195838c, SUC_lexicon_tags_37 },
    { 0x6df5838c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0558390, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d58394, SUC_lexicon_tags_4 },
    { 0x6f58394, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17558397, SUC_lexicon_tags_15 },
    { 0x16758398, SUC_lexicon_tags_9 },
    { 0x73758398, SUC_lexicon_tags_1 },
    { 0x75839a, SUC_lexicon_tags_37 },
    { 0x53b5839b, SUC_lexicon_tags_23 },
    { 0xcbf5839c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f5839f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43583a4, SUC_lexicon_tags_33 },
    { 0x789583a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad583a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5dd583aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb583b0, SUC_lexicon_tags_63 },
    { 0xf5f583b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b1583b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x203583b5, SUC_lexicon_tags_3 },
    { 0x8cf583b6, SUC_lexicon_tags_25 },
    { 0x4af583b6, SUC_lexicon_tags_37 },
    { 0x71f583b5, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7583bd, SUC_lexicon_tags_15 },
    { 0xe15583be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1583c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33583ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9583da, SUC_lexicon_tags_7 },
    { 0xc3d583db, SUC_lexicon_tags_1 },
    { 0x32d583dc, SUC_lexicon_tags_17 },
    { 0xc67583db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d583e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5583e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5583ee, SUC_lexicon_tags_37 },
    { 0xde9583ee, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7583f3, SUC_lexicon_tags_45 },
    { 0xfb5583f4, SUC_lexicon_tags_11 },
    { 0x44f583f5, SUC_lexicon_tags_33 },
    { 0xb93583f6, SUC_lexicon_tags_1 },
    { 0x52b583f5, SUC_lexicon_tags_39 },
    { 0xc49583f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x157583fa, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xe41583fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff3583fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41158401, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f58405, SUC_lexicon_tags_25 },
    { 0xe6558406, SUC_lexicon_tags_15 },
    { 0x7ab58407, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a35840b, SUC_lexicon_tags_1 },
    { 0xf015840c, SUC_lexicon_tags_40 },
    { 0xc4d5840d, SUC_lexicon_tags_17 },
    { 0xb455840c, SUC_lexicon_tags_30 },
    { 0x3a95840d, SUC_lexicon_tags_11 },
    { 0xbe558410, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f58416, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b58419, SUC_lexicon_tags_7 },
    { 0x8b75841a, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d75841f, SUC_lexicon_tags_30 },
    { 0x49958420, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb358423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cb58425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28758429, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af5842f, SUC_lexicon_tags_15 },
    { 0x7d358430, SUC_lexicon_tags_55 },
    { 0x65358431, SUC_lexicon_tags_14 },
    { 0xdc758431, SUC_lexicon_tags_37 },
    { 0x34f58433, SUC_lexicon_tags_19 },
    { 0xf7558434, SUC_lexicon_tags_0 },
    { 0x51558431, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6558437, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e95843a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e95843e, SUC_lexicon_tags_1 },
    { 0xcab5843f, SUC_lexicon_tags_1 },
    { 0x74b5843f, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb58443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e558449, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d758452, SUC_lexicon_tags_17 },
    { 0x67b58452, SUC_lexicon_tags_26 },
    { 0x3f158453, SUC_lexicon_tags_9 },
    { 0xdd158455, SUC_lexicon_tags_1 },
    { 0x4df58455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e358459, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f5845e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd58464, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb358467, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f5846c, SUC_lexicon_tags_17 },
    { 0x6eb5846d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf158470, SUC_lexicon_tags_43 },
    { 0x89958471, SUC_lexicon_tags_25 },
    { 0x1ad58472, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0958475, SUC_lexicon_tags_25 },
    { 0xcf758476, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd58479, SUC_lexicon_tags_25 },
    { 0xff558479, SUC_lexicon_tags_37 },
    { 0x1095847b, SUC_lexicon_tags_12 },
    { 0x7835847c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5847f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef58481, SUC_lexicon_tags_15 },
    { 0x90d58482, SUC_lexicon_tags_47 },
    { 0xf2358483, SUC_lexicon_tags_25 },
    { 0x2b958483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40d58486, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x23d58488, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77358491, SUC_lexicon_tags_9 },
    { 0x3558492, SUC_lexicon_tags_13 },
    { 0x84758493, SUC_lexicon_tags_65 },
    { 0xc6b58494, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x72558497, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5e158499, SUC_lexicon_tags_87 },
    { 0xaf55849a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe195849d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d584a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d584a6, SUC_lexicon_tags_55 },
    { 0xcdd584a7, SUC_lexicon_tags_25 },
    { 0xb15584a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf99584aa, SUC_lexicon_tags_16 },
    { 0x885584aa, SUC_lexicon_tags_37 },
    { 0x623584ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d584af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff584b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb584b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1584bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23584be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05584c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7c3584c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x889584c9, SUC_lexicon_tags_7 },
    { 0xc17584ca, SUC_lexicon_tags_32 },
    { 0xc8d584cb, SUC_lexicon_tags_9 },
    { 0x459584cc, SUC_lexicon_tags_12 },
    { 0xeb1584cd, SUC_lexicon_tags_37 },
    { 0x3cf584cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0f584d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x311584d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf584da, SUC_lexicon_tags_7 },
    { 0x8bd584da, SUC_lexicon_tags_25 },
    { 0x37d584dc, SUC_lexicon_tags_5 },
    { 0x6df584dc, SUC_lexicon_tags_17 },
    { 0xf81584de, SUC_lexicon_tags_25 },
    { 0xa47584de, SUC_lexicon_tags_63 },
    { 0x5b584de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x713584e2, SUC_lexicon_tags_255 },
    { 0x2d1584e3, SUC_lexicon_tags_0 },
    { 0x1ff584e3, SUC_lexicon_tags_3 },
    { 0xb59584e5, SUC_lexicon_tags_21 },
    { 0x9fb584e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7584e9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503584ed, SUC_lexicon_tags_25 },
    { 0x19584ed, SUC_lexicon_tags_36 },
    { 0xa81584ef, SUC_lexicon_tags_37 },
    { 0xa39584ef, SUC_lexicon_tags_9 },
    { 0xe5584ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7584f7, SUC_lexicon_tags_17 },
    { 0x1f7584f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f584fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e1584ff, SUC_lexicon_tags_12 },
    { 0x88b584ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7af58502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e758504, SUC_lexicon_tags_31 },
    { 0x50d58505, SUC_lexicon_tags_12 },
    { 0x6f358506, SUC_lexicon_tags_37 },
    { 0x6958505, SUC_lexicon_tags_15 },
    { 0x64b58505, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67f5850a, SUC_lexicon_tags_10 },
    { 0xaff5850a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d5850e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40758514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99958519, SUC_lexicon_tags_0 },
    { 0xc6b5851a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc675851e, SUC_lexicon_tags_11 },
    { 0xec55851e, SUC_lexicon_tags_96 },
    { 0xd6f5851e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c358524, SUC_lexicon_tags_25 },
    { 0xa0f58525, SUC_lexicon_tags_17 },
    { 0x68f58525, SUC_lexicon_tags_29 },
    { 0xbc158527, SUC_lexicon_tags_7 },
    { 0x18958528, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe935852f, SUC_lexicon_tags_14 },
    { 0xbd58530, SUC_lexicon_tags_23 },
    { 0x7b758531, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3db58533, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf58538, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x47b5853a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dd5853c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33558546, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c558549, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18d5854b, SUC_lexicon_tags_154 },
    { 0x9f35854c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d58550, SUC_lexicon_tags_23 },
    { 0x66958551, SUC_lexicon_tags_29 },
    { 0x73f58551, SUC_lexicon_tags_11 },
    { 0xc6d58552, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc4f58555, SUC_lexicon_tags_2 },
    { 0x88f58556, SUC_lexicon_tags_25 },
    { 0x72158557, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xed95855a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a15855d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf375855f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe5358561, SUC_lexicon_tags_1 },
    { 0x94358562, SUC_lexicon_tags_15 },
    { 0xa8758562, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xca358565, SUC_lexicon_tags_19 },
    { 0x97d58565, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7958568, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3435856b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd5856e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad958573, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb715857a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a55857c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5857f, SUC_lexicon_tags_28 },
    { 0xb1158580, SUC_lexicon_tags_2 },
    { 0xfc358580, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd58584, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3a358586, SUC_lexicon_tags_1 },
    { 0x2ef58586, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x67158589, SUC_lexicon_tags_31 },
    { 0x96558589, SUC_lexicon_tags_4 },
    { 0x53158589, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b58590, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e158595, SUC_lexicon_tags_15 },
    { 0xe3b58595, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf835859a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ab5859c, SUC_lexicon_tags_17 },
    { 0xb8f5859d, SUC_lexicon_tags_23 },
    { 0xfb5859e, SUC_lexicon_tags_17 },
    { 0xa375859e, SUC_lexicon_tags_25 },
    { 0x555585a0, SUC_lexicon_tags_10 },
    { 0xe79585a1, SUC_lexicon_tags_66 },
    { 0xddb5859e, SUC_lexicon_tags_7 },
    { 0x7a7585a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x127585a6, SUC_lexicon_tags_23 },
    { 0x801585a7, SUC_lexicon_tags_33 },
    { 0xa0d585a8, SUC_lexicon_tags_12 },
    { 0xe89585a9, SUC_lexicon_tags_29 },
    { 0xf33585a7, SUC_lexicon_tags_15 },
    { 0x83b585ab, SUC_lexicon_tags_32 },
    { 0x825585ac, SUC_lexicon_tags_27 },
    { 0x5fd585ac, SUC_lexicon_tags_17 },
    { 0x971585ae, SUC_lexicon_tags_11 },
    { 0xf585af, SUC_lexicon_tags_7 },
    { 0x753585af, SUC_lexicon_tags_17 },
    { 0xb77585b1, SUC_lexicon_tags_9 },
    { 0xd9585b2, SUC_lexicon_tags_47 },
    { 0x3e3585b3, SUC_lexicon_tags_1 },
    { 0x217585b4, SUC_lexicon_tags_19 },
    { 0x3af585af, SUC_lexicon_tags_40 },
    { 0xa0b585b2, SUC_lexicon_tags_0 },
    { 0x5d9585a7, SUC_lexicon_tags_30 },
    { 0x763585ac, SUC_lexicon_tags_43 },
    { 0x519585b9, SUC_lexicon_tags_27 },
    { 0x301585b1, SUC_lexicon_tags_4 },
    { 0xc4d585b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5585c1, SUC_lexicon_tags_11 },
    { 0xf99585c1, SUC_lexicon_tags_11 },
    { 0xe83585c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3585c7, SUC_lexicon_tags_71 },
    { 0xab5585c8, SUC_lexicon_tags_16 },
    { 0x315585c9, SUC_lexicon_tags_29 },
    { 0x851585c7, SUC_lexicon_tags_19 },
    { 0x3a7585cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8a3585cd, SUC_lexicon_tags_15 },
    { 0xf45585ce, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xd49585d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b585d5, SUC_lexicon_tags_8 },
    { 0x801585d6, SUC_lexicon_tags_2 },
    { 0xbfb585d5, SUC_lexicon_tags_2 },
    { 0x1cd585d8, SUC_lexicon_tags_87 },
    { 0x673585d5, SUC_lexicon_tags_15 },
    { 0xfc5585da, SUC_lexicon_tags_339 },
    { 0x7e7585d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f3585dd, SUC_lexicon_tags_17 },
    { 0x629585de, SUC_lexicon_tags_2 },
    { 0xfb5585dd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5585e4, SUC_lexicon_tags_29 },
    { 0xdb5585e5, SUC_lexicon_tags_68 },
    { 0xc73585e6, SUC_lexicon_tags_21 },
    { 0x711585e7, SUC_lexicon_tags_48 },
    { 0xbe9585e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d585ea, SUC_lexicon_tags_23 },
    { 0x851585eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x135585ee, SUC_lexicon_tags_2 },
    { 0x90b585ef, SUC_lexicon_tags_14 },
    { 0x107585ee, SUC_lexicon_tags_33 },
    { 0x641585f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d585f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad585f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa47585f9, SUC_lexicon_tags_37 },
    { 0xaef585fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c7585fc, SUC_lexicon_tags_55 },
    { 0xc05585fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f58601, SUC_lexicon_tags_31 },
    { 0xdbb58601, SUC_lexicon_tags_17 },
    { 0x7558601, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7958607, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe455860a, SUC_lexicon_tags_4 },
    { 0x6c75860a, SUC_lexicon_tags_25 },
    { 0xb4b5860c, SUC_lexicon_tags_1 },
    { 0xb9f5860a, SUC_lexicon_tags_25 },
    { 0x82b5860e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf58611, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x7d58613, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa2958615, SUC_lexicon_tags_25 },
    { 0x38158616, SUC_lexicon_tags_33 },
    { 0x17d58616, SUC_lexicon_tags_1 },
    { 0x4a958616, SUC_lexicon_tags_2 },
    { 0xe6b58619, SUC_lexicon_tags_17 },
    { 0x1155861a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5158620, SUC_lexicon_tags_2 },
    { 0xc358621, SUC_lexicon_tags_11 },
    { 0x73d58622, SUC_lexicon_tags_49 },
    { 0xabd58623, SUC_lexicon_tags_17 },
    { 0x67758624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8558626, SUC_lexicon_tags_62 },
    { 0xb958627, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb9958629, SUC_lexicon_tags_21 },
    { 0xd6f5862a, SUC_lexicon_tags_11 },
    { 0x86158629, SUC_lexicon_tags_91 },
    { 0x5515862c, SUC_lexicon_tags_29 },
    { 0x82b5862a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbd5862f, SUC_lexicon_tags_37 },
    { 0x9c158630, SUC_lexicon_tags_47 },
    { 0x76158631, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc6358633, SUC_lexicon_tags_1 },
    { 0xbe558633, SUC_lexicon_tags_46 },
    { 0xcdb58635, SUC_lexicon_tags_5 },
    { 0x4b358633, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64158640, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b58647, SUC_lexicon_tags_1 },
    { 0x57158648, SUC_lexicon_tags_1 },
    { 0x82158649, SUC_lexicon_tags_24 },
    { 0x13158648, SUC_lexicon_tags_15 },
    { 0xa315864a, SUC_lexicon_tags_33 },
    { 0xb635864c, SUC_lexicon_tags_12 },
    { 0x7e55864c, SUC_lexicon_tags_37 },
    { 0x2e75864c, SUC_lexicon_tags_8 },
    { 0xa1b5864a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71958651, SUC_lexicon_tags_14 },
    { 0x7e558652, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11758656, SUC_lexicon_tags_12 },
    { 0xf8758657, SUC_lexicon_tags_7 },
    { 0xe2958656, SUC_lexicon_tags_43 },
    { 0x3f958658, SUC_lexicon_tags_31 },
    { 0x7535865a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4158661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7958667, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b5866c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec358670, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a558672, SUC_lexicon_tags_38 },
    { 0x9758673, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9f958675, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f58679, SUC_lexicon_tags_33 },
    { 0x72b58679, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92758682, SUC_lexicon_tags_16 },
    { 0x7cd58682, SUC_lexicon_tags_37 },
    { 0x9dd58683, SUC_lexicon_tags_17 },
    { 0x24d58685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb5868b, SUC_lexicon_tags_48 },
    { 0x7a95868b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14b5868e, SUC_lexicon_tags_2 },
    { 0x4435868f, SUC_lexicon_tags_1 },
    { 0xd4358690, SUC_lexicon_tags_16 },
    { 0x1955868e, SUC_lexicon_tags_23 },
    { 0x6fd58690, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x88358694, SUC_lexicon_tags_47 },
    { 0xa5b58695, SUC_lexicon_tags_7 },
    { 0x57758696, SUC_lexicon_tags_14 },
    { 0xb358697, SUC_lexicon_tags_29 },
    { 0xc4358698, SUC_lexicon_tags_16 },
    { 0x67d58698, SUC_lexicon_tags_1 },
    { 0x73f58696, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a55869e, SUC_lexicon_tags_2 },
    { 0x7895869f, SUC_lexicon_tags_15 },
    { 0x7a7586a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891586a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e1586a7, SUC_lexicon_tags_17 },
    { 0x51586a8, SUC_lexicon_tags_38 },
    { 0x5ab586a7, SUC_lexicon_tags_27 },
    { 0xd95586aa, SUC_lexicon_tags_11 },
    { 0x4a3586ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935586af, SUC_lexicon_tags_10 },
    { 0x4ed586b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19b586b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1586b5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf2f586b7, SUC_lexicon_tags_15 },
    { 0x431586b8, SUC_lexicon_tags_4 },
    { 0x24b586b9, SUC_lexicon_tags_38 },
    { 0x12b586b9, SUC_lexicon_tags_15 },
    { 0x9a5586ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84d586bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x861586bf, SUC_lexicon_tags_55 },
    { 0xe27586c0, SUC_lexicon_tags_17 },
    { 0x279586c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c7586c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43586c6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xba9586c8, SUC_lexicon_tags_8 },
    { 0x67d586c9, SUC_lexicon_tags_55 },
    { 0x155586c8, SUC_lexicon_tags_11 },
    { 0x715586c9, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b586d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365586da, SUC_lexicon_tags_37 },
    { 0x4a9586db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9586e0, SUC_lexicon_tags_66 },
    { 0xf1f586e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09586ec, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9586ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9586f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf586f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d58700, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5958704, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf0758706, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe2558708, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf5870e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90358712, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99958718, SUC_lexicon_tags_30 },
    { 0x4b558719, SUC_lexicon_tags_17 },
    { 0x24558719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a75871c, SUC_lexicon_tags_31 },
    { 0x7955871c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc15871f, SUC_lexicon_tags_2 },
    { 0x93d5871f, SUC_lexicon_tags_12 },
    { 0x2e558720, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x64758723, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x92f58725, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8df58727, SUC_lexicon_tags_19 },
    { 0x7af58728, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf335872d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f58730, SUC_lexicon_tags_47 },
    { 0xe7358730, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b758734, SUC_lexicon_tags_13 },
    { 0x52358734, SUC_lexicon_tags_2 },
    { 0xda958734, SUC_lexicon_tags_100 },
    { 0x4a758737, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b5873a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d5873e, SUC_lexicon_tags_49 },
    { 0xfa15873e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfef58741, SUC_lexicon_tags_14 },
    { 0x28358742, SUC_lexicon_tags_15 },
    { 0x4e358743, SUC_lexicon_tags_1 },
    { 0x72358743, SUC_lexicon_tags_7 },
    { 0xcc958745, SUC_lexicon_tags_124 },
    { 0xa9158741, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x99358749, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb235874b, SUC_lexicon_tags_6 },
    { 0x695874c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x78d5874e, SUC_lexicon_tags_19 },
    { 0xe935874e, SUC_lexicon_tags_6 },
    { 0xb9f58750, SUC_lexicon_tags_13 },
    { 0x27758751, SUC_lexicon_tags_16 },
    { 0x47b58752, SUC_lexicon_tags_17 },
    { 0x6df58752, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3a158755, SUC_lexicon_tags_25 },
    { 0xe1b58755, SUC_lexicon_tags_68 },
    { 0xe7558757, SUC_lexicon_tags_37 },
    { 0xa5158758, SUC_lexicon_tags_0 },
    { 0x4b358756, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf975875b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f75875d, SUC_lexicon_tags_19 },
    { 0xbef5875e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc958772, SUC_lexicon_tags_79 },
    { 0xa0f58773, SUC_lexicon_tags_35 },
    { 0x2758773, SUC_lexicon_tags_23 },
    { 0x31b58775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97358779, SUC_lexicon_tags_1 },
    { 0xf175877a, SUC_lexicon_tags_43 },
    { 0xf155877b, SUC_lexicon_tags_160 },
    { 0xb9d5877c, SUC_lexicon_tags_27 },
    { 0x4175877a, SUC_lexicon_tags_37 },
    { 0xe3f5877e, SUC_lexicon_tags_62 },
    { 0x79d5877c, SUC_lexicon_tags_11 },
    { 0xc7758780, SUC_lexicon_tags_17 },
    { 0x4d758781, SUC_lexicon_tags_37 },
    { 0xeef58782, SUC_lexicon_tags_25 },
    { 0xa2d5877d, SUC_lexicon_tags_37 },
    { 0xf0b58779, SUC_lexicon_tags_37 },
    { 0xae958785, SUC_lexicon_tags_2 },
    { 0x5c15877a, SUC_lexicon_tags_15 },
    { 0xb6f58779, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32158790, SUC_lexicon_tags_7 },
    { 0x43558790, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x51b58793, SUC_lexicon_tags_1 },
    { 0x85f58794, SUC_lexicon_tags_43 },
    { 0x2f558795, SUC_lexicon_tags_1 },
    { 0x28d58796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd58799, SUC_lexicon_tags_46 },
    { 0x3cf5879a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd935879c, SUC_lexicon_tags_28 },
    { 0xefd5879d, SUC_lexicon_tags_15 },
    { 0xe935879d, SUC_lexicon_tags_24 },
    { 0xadf5879c, SUC_lexicon_tags_27 },
    { 0x4bb587a0, SUC_lexicon_tags_1 },
    { 0x5b5587a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31587a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1587ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93587b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503587bf, SUC_lexicon_tags_61 },
    { 0xf75587bf, SUC_lexicon_tags_37 },
    { 0x943587c1, SUC_lexicon_tags_17 },
    { 0x39d587c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09587c5, SUC_lexicon_tags_6 },
    { 0xcf587c6, SUC_lexicon_tags_37 },
    { 0xe43587c7, SUC_lexicon_tags_37 },
    { 0xff3587c6, SUC_lexicon_tags_12 },
    { 0x74f587c9, SUC_lexicon_tags_34 },
    { 0xf53587c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11587cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1587d7, SUC_lexicon_tags_47 },
    { 0x7e1587d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13587dd, SUC_lexicon_tags_0 },
    { 0xf09587de, SUC_lexicon_tags_45 },
    { 0x441587de, SUC_lexicon_tags_43 },
    { 0x873587dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3587e4, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1587e9, SUC_lexicon_tags_1 },
    { 0xce5587ea, SUC_lexicon_tags_57 },
    { 0x73b587ea, SUC_lexicon_tags_80 },
    { 0x2fd587ec, SUC_lexicon_tags_11 },
    { 0xa97587ed, SUC_lexicon_tags_17 },
    { 0x335587ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d587f1, SUC_lexicon_tags_3 },
    { 0xa4d587f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89587f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb77587fb, SUC_lexicon_tags_17 },
    { 0x75d587fc, SUC_lexicon_tags_11 },
    { 0xd0d587fd, SUC_lexicon_tags_23 },
    { 0xca1587fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f58802, SUC_lexicon_tags_46 },
    { 0xe6f58802, SUC_lexicon_tags_16 },
    { 0x94358804, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2958807, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55880a, SUC_lexicon_tags_49 },
    { 0xd795880b, SUC_lexicon_tags_37 },
    { 0x1315880a, SUC_lexicon_tags_14 },
    { 0x7f55880c, SUC_lexicon_tags_45 },
    { 0x9435880e, SUC_lexicon_tags_29 },
    { 0xb6f5880f, SUC_lexicon_tags_43 },
    { 0xe1b5880f, SUC_lexicon_tags_66 },
    { 0xa635880e, SUC_lexicon_tags_1 },
    { 0x7655880e, SUC_lexicon_tags_26 },
    { 0x3b75880d, SUC_lexicon_tags_15 },
    { 0xa8758814, SUC_lexicon_tags_10 },
    { 0xc1158814, SUC_lexicon_tags_8 },
    { 0x6935880c, SUC_lexicon_tags_11 },
    { 0xb2358817, SUC_lexicon_tags_12 },
    { 0x2fd58817, SUC_lexicon_tags_49 },
    { 0x61758817, SUC_lexicon_tags_5 },
    { 0x3515881a, SUC_lexicon_tags_109 },
    { 0xa215881a, SUC_lexicon_tags_16 },
    { 0xa9958818, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2c75881e, SUC_lexicon_tags_11 },
    { 0xd0f5881e, SUC_lexicon_tags_47 },
    { 0xf9358820, SUC_lexicon_tags_34 },
    { 0x2455881f, SUC_lexicon_tags_46 },
    { 0x99158821, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df58827, SUC_lexicon_tags_125 },
    { 0x29958828, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80f5882a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf15882d, SUC_lexicon_tags_7 },
    { 0xeb95882e, SUC_lexicon_tags_22 },
    { 0xbeb5882f, SUC_lexicon_tags_8 },
    { 0x25f58830, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd58835, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35883a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x155883d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67558841, SUC_lexicon_tags_12 },
    { 0xea358841, SUC_lexicon_tags_45 },
    { 0x44558843, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7558849, SUC_lexicon_tags_13 },
    { 0xc655884a, SUC_lexicon_tags_28 },
    { 0xef95884b, SUC_lexicon_tags_11 },
    { 0x1a55884c, SUC_lexicon_tags_49 },
    { 0xfd15884d, SUC_lexicon_tags_31 },
    { 0x8ad58849, SUC_lexicon_tags_37 },
    { 0xaad5884d, SUC_lexicon_tags_45 },
    { 0xed558849, SUC_lexicon_tags_38 },
    { 0x2c55884c, SUC_lexicon_tags_21 },
    { 0xa1158852, SUC_lexicon_tags_17 },
    { 0x2cb58852, SUC_lexicon_tags_1 },
    { 0x67758854, SUC_lexicon_tags_473 },
    { 0xc815884a, SUC_lexicon_tags_60 },
    { 0x99f5884d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f58859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf58864, SUC_lexicon_tags_1 },
    { 0xe7158864, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7558868, SUC_lexicon_tags_47 },
    { 0x80158869, SUC_lexicon_tags_17 },
    { 0x7a158869, SUC_lexicon_tags_63 },
    { 0xc1b5886b, SUC_lexicon_tags_30 },
    { 0x4015886b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76558870, SUC_lexicon_tags_17 },
    { 0xf7b58871, SUC_lexicon_tags_60 },
    { 0x93958871, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b358875, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe315887b, SUC_lexicon_tags_68 },
    { 0xe255887c, SUC_lexicon_tags_37 },
    { 0x3bf5887c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e95887f, SUC_lexicon_tags_7 },
    { 0x49f5887f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93f58882, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b58887, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2415888a, SUC_lexicon_tags_25 },
    { 0x2675888b, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f5888e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44158890, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b58897, SUC_lexicon_tags_60 },
    { 0x7b158898, SUC_lexicon_tags_11 },
    { 0x96558897, SUC_lexicon_tags_2 },
    { 0xf4f5889a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce55889c, SUC_lexicon_tags_14 },
    { 0x8315889c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f588a4, SUC_lexicon_tags_29 },
    { 0x2f3588a5, SUC_lexicon_tags_15 },
    { 0x2d7588a6, SUC_lexicon_tags_33 },
    { 0xbdf588a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ad588a9, SUC_lexicon_tags_15 },
    { 0x537588aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x985588ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb588b0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b588b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa93588b5, SUC_lexicon_tags_1 },
    { 0xc6f588b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d588b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97588bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x45b588bf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b588c2, SUC_lexicon_tags_11 },
    { 0xa59588c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f588c7, SUC_lexicon_tags_63 },
    { 0x47588c8, SUC_lexicon_tags_27 },
    { 0x309588c9, SUC_lexicon_tags_39 },
    { 0x7bb588c9, SUC_lexicon_tags_2 },
    { 0x327588ca, SUC_lexicon_tags_31 },
    { 0x11b588cc, SUC_lexicon_tags_55 },
    { 0xfa3588ca, SUC_lexicon_tags_27 },
    { 0x42d588cb, SUC_lexicon_tags_17 },
    { 0xb99588cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8b9588d1, SUC_lexicon_tags_11 },
    { 0x9f1588d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8a9588d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8ab588d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9588dc, SUC_lexicon_tags_37 },
    { 0xeb9588dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c5588df, SUC_lexicon_tags_4 },
    { 0x5b9588df, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399588e8, SUC_lexicon_tags_15 },
    { 0x9b7588e9, SUC_lexicon_tags_23 },
    { 0xc4f588e9, SUC_lexicon_tags_1 },
    { 0x731588e8, SUC_lexicon_tags_11 },
    { 0xf99588eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93588f0, SUC_lexicon_tags_46 },
    { 0xdc7588f1, SUC_lexicon_tags_15 },
    { 0xafd588f2, SUC_lexicon_tags_146 },
    { 0x8e3588f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65588f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x43f588f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b58904, SUC_lexicon_tags_11 },
    { 0x25358904, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa6958907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb58910, SUC_lexicon_tags_8 },
    { 0xfa358911, SUC_lexicon_tags_37 },
    { 0xdc358912, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71558914, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38558916, SUC_lexicon_tags_45 },
    { 0xa0158916, SUC_lexicon_tags_19 },
    { 0xc4158918, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8335891d, SUC_lexicon_tags_17 },
    { 0x92b5891d, SUC_lexicon_tags_47 },
    { 0xba75891f, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6958924, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf5892a, SUC_lexicon_tags_4 },
    { 0xeaf5892a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9615892f, SUC_lexicon_tags_14 },
    { 0x95558930, SUC_lexicon_tags_20 },
    { 0x6e958930, SUC_lexicon_tags_27 },
    { 0xd7d58930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3f58934, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x69358936, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15893f, SUC_lexicon_tags_37 },
    { 0xdf58940, SUC_lexicon_tags_14 },
    { 0xc5d58941, SUC_lexicon_tags_1 },
    { 0x54f58942, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd958945, SUC_lexicon_tags_37 },
    { 0x71558945, SUC_lexicon_tags_60 },
    { 0x6e558947, SUC_lexicon_tags_0 },
    { 0xcdd58948, SUC_lexicon_tags_0 },
    { 0x84d58949, SUC_lexicon_tags_25 },
    { 0xcd558948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa275894c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf58957, SUC_lexicon_tags_60 },
    { 0x10358958, SUC_lexicon_tags_25 },
    { 0x4c958958, SUC_lexicon_tags_17 },
    { 0xdfb5895a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7fb5895c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16f5895e, SUC_lexicon_tags_14 },
    { 0x9f15895f, SUC_lexicon_tags_107 },
    { 0x76f58960, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47558962, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3958965, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6ab58967, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5958969, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa335896c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d958970, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf558975, SUC_lexicon_tags_57 },
    { 0x50158975, SUC_lexicon_tags_25 },
    { 0xb1f58977, SUC_lexicon_tags_37 },
    { 0xf9d58976, SUC_lexicon_tags_37 },
    { 0x5df58979, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3955897b, SUC_lexicon_tags_56 },
    { 0x3d15897c, SUC_lexicon_tags_27 },
    { 0xea95897d, SUC_lexicon_tags_11 },
    { 0x6215897e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e158984, SUC_lexicon_tags_46 },
    { 0xe758985, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b5898b, SUC_lexicon_tags_11 },
    { 0x9a75898c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb58994, SUC_lexicon_tags_39 },
    { 0x81758995, SUC_lexicon_tags_7 },
    { 0x47758995, SUC_lexicon_tags_25 },
    { 0xf2f58997, SUC_lexicon_tags_26 },
    { 0x4158997, SUC_lexicon_tags_1 },
    { 0xec158996, SUC_lexicon_tags_63 },
    { 0x51d58995, SUC_lexicon_tags_15 },
    { 0x9eb5899b, SUC_lexicon_tags_2 },
    { 0xb9b5899b, SUC_lexicon_tags_33 },
    { 0xcfb5899d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x12b5899f, SUC_lexicon_tags_9 },
    { 0x617589a0, SUC_lexicon_tags_19 },
    { 0xa4f5899f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf589a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1589aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x385589ad, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9589b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x23589b5, SUC_lexicon_tags_11 },
    { 0x4b1589b6, SUC_lexicon_tags_6 },
    { 0xa41589b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b589ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf8f589bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9eb589be, SUC_lexicon_tags_63 },
    { 0x2f589be, SUC_lexicon_tags_1 },
    { 0x8fd589c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cb589c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x649589c5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77589c9, SUC_lexicon_tags_15 },
    { 0x7e9589ca, SUC_lexicon_tags_4 },
    { 0x16d589cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x441589cd, SUC_lexicon_tags_29 },
    { 0x927589ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9589d0, SUC_lexicon_tags_45 },
    { 0x7e1589d1, SUC_lexicon_tags_2 },
    { 0xd3589d0, SUC_lexicon_tags_65 },
    { 0x847589d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf89589d5, SUC_lexicon_tags_30 },
    { 0xcad589d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43589db, SUC_lexicon_tags_25 },
    { 0xe83589dc, SUC_lexicon_tags_25 },
    { 0x33d589dd, SUC_lexicon_tags_1 },
    { 0x45f589dd, SUC_lexicon_tags_31 },
    { 0x6c7589df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb589e2, SUC_lexicon_tags_12 },
    { 0x307589e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453589ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf75589ec, SUC_lexicon_tags_0 },
    { 0x6a7589ed, SUC_lexicon_tags_25 },
    { 0x2b589ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfb3589f0, SUC_lexicon_tags_9 },
    { 0x673589f0, SUC_lexicon_tags_32 },
    { 0x629589f2, SUC_lexicon_tags_33 },
    { 0x347589f1, SUC_lexicon_tags_7 },
    { 0x589589f4, SUC_lexicon_tags_37 },
    { 0xdff589f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x35f589f7, SUC_lexicon_tags_43 },
    { 0xb95589f8, SUC_lexicon_tags_27 },
    { 0xf3589f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfe1589fb, SUC_lexicon_tags_14 },
    { 0x843589fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x50b589fe, SUC_lexicon_tags_17 },
    { 0xec1589ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea358a01, SUC_lexicon_tags_17 },
    { 0x87d58a02, SUC_lexicon_tags_24 },
    { 0xf1b58a02, SUC_lexicon_tags_17 },
    { 0x54758a04, SUC_lexicon_tags_8 },
    { 0x30158a05, SUC_lexicon_tags_38 },
    { 0x86558a06, SUC_lexicon_tags_12 },
    { 0xa1358a07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b58a0a, SUC_lexicon_tags_17 },
    { 0x8cf58a0b, SUC_lexicon_tags_48 },
    { 0x8e158a0c, SUC_lexicon_tags_194 },
    { 0x93958a0a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7a358a0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab958a14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd758a16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1958a19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0158a1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53158a22, SUC_lexicon_tags_15 },
    { 0x2fb58a22, SUC_lexicon_tags_19 },
    { 0x1c558a24, SUC_lexicon_tags_30 },
    { 0xe7d58a24, SUC_lexicon_tags_26 },
    { 0x3db58a26, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x52958a28, SUC_lexicon_tags_87 },
    { 0x63f58a28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6758a2e, SUC_lexicon_tags_14 },
    { 0xa6558a2f, SUC_lexicon_tags_25 },
    { 0x3a758a30, SUC_lexicon_tags_17 },
    { 0x40958a30, SUC_lexicon_tags_16 },
    { 0x15758a32, SUC_lexicon_tags_60 },
    { 0x8db58a33, SUC_lexicon_tags_27 },
    { 0x96d58a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8558a37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84358a3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea758a42, SUC_lexicon_tags_15 },
    { 0x23158a43, SUC_lexicon_tags_26 },
    { 0xee758a43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x91f58a46, SUC_lexicon_tags_15 },
    { 0xf9d58a47, SUC_lexicon_tags_5 },
    { 0x51158a47, SUC_lexicon_tags_2 },
    { 0x9c958a47, SUC_lexicon_tags_55 },
    { 0xa8958a47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd558a4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff58a4e, SUC_lexicon_tags_56 },
    { 0xc8b58a4f, SUC_lexicon_tags_23 },
    { 0xda158a4f, SUC_lexicon_tags_160 },
    { 0xdeb58a4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xceb58a53, SUC_lexicon_tags_207 },
    { 0xf2358a54, SUC_lexicon_tags_31 },
    { 0xdab58a54, SUC_lexicon_tags_27 },
    { 0xcd558a56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2758a59, SUC_lexicon_tags_26 },
    { 0xeef58a5a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b58a5e, SUC_lexicon_tags_23 },
    { 0x26558a5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9558a61, SUC_lexicon_tags_390 },
    { 0x58358a62, SUC_lexicon_tags_16 },
    { 0x2d158a63, SUC_lexicon_tags_77 },
    { 0xbf158a63, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc5f58a66, SUC_lexicon_tags_19 },
    { 0xfb158a67, SUC_lexicon_tags_26 },
    { 0xc7f58a67, SUC_lexicon_tags_66 },
    { 0xd358a69, SUC_lexicon_tags_23 },
    { 0x1558a6a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x91d58a6c, SUC_lexicon_tags_25 },
    { 0xb8758a6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d58a74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7f358a76, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x93958a78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24758a7a, SUC_lexicon_tags_9 },
    { 0x91d58a7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69358a80, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a358a8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97958a8c, SUC_lexicon_tags_12 },
    { 0x87758a8c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99958a92, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a958aa0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x57358aa2, SUC_lexicon_tags_10 },
    { 0x81f58aa3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x87f58aa5, SUC_lexicon_tags_36 },
    { 0x33d58aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67158aac, SUC_lexicon_tags_9 },
    { 0x27358aac, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe5158aaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63758ab2, SUC_lexicon_tags_49 },
    { 0x15158ab3, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x7a758ab5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f58abb, SUC_lexicon_tags_17 },
    { 0xf3558abb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2ad58abe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef58ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d58ac9, SUC_lexicon_tags_10 },
    { 0xfbb58aca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75158ace, SUC_lexicon_tags_45 },
    { 0xb8758acf, SUC_lexicon_tags_2 },
    { 0x4cb58ad0, SUC_lexicon_tags_13 },
    { 0x6a558acf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2958ad4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d58ada, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2f58adc, SUC_lexicon_tags_70 },
    { 0xe0758add, SUC_lexicon_tags_45 },
    { 0xde558ade, SUC_lexicon_tags_30 },
    { 0xad358adf, SUC_lexicon_tags_11 },
    { 0x5cb58ade, SUC_lexicon_tags_49 },
    { 0xdd58ae1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4b58ae3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0f58ae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c758ae8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb6158aea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1958aee, SUC_lexicon_tags_2 },
    { 0x6f558aef, SUC_lexicon_tags_43 },
    { 0x52b58af0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53158af4, SUC_lexicon_tags_25 },
    { 0x6e958af4, SUC_lexicon_tags_15 },
    { 0x20d58af6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4f58af8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf158afa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x59758afc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x53158aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b58b03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40f58b05, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e558b09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf9b58b0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb758b10, SUC_lexicon_tags_38 },
    { 0x7c158b11, SUC_lexicon_tags_2 },
    { 0xe7b58b12, SUC_lexicon_tags_33 },
    { 0x31b58b13, SUC_lexicon_tags_12 },
    { 0x10758b13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62158b1c, SUC_lexicon_tags_11 },
    { 0xc2558b1d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca558b24, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x95b58b26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb58b29, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb58b30, SUC_lexicon_tags_65 },
    { 0x79358b31, SUC_lexicon_tags_62 },
    { 0x49358b31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb58b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d58b3a, SUC_lexicon_tags_14 },
    { 0x36158b3b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x71558b3d, SUC_lexicon_tags_65 },
    { 0x4f558b3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb58b42, SUC_lexicon_tags_47 },
    { 0x5e158b43, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65758b4a, SUC_lexicon_tags_761 },
    { 0xa4558b4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b58b4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f758b52, SUC_lexicon_tags_1 },
    { 0x59158b53, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbc758b55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f58b58, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde758b5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7158b61, SUC_lexicon_tags_9 },
    { 0xbd58b62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7558b64, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b558b6b, SUC_lexicon_tags_17 },
    { 0x8af58b6c, SUC_lexicon_tags_28 },
    { 0x2c558b6d, SUC_lexicon_tags_7 },
    { 0xdcb58b6d, SUC_lexicon_tags_27 },
    { 0xeff58b6d, SUC_lexicon_tags_63 },
    { 0x19d58b6b, SUC_lexicon_tags_38 },
    { 0xa7b58b6d, SUC_lexicon_tags_7 },
    { 0x9f558b6c, SUC_lexicon_tags_12 },
    { 0x6ed58b71, SUC_lexicon_tags_8 },
    { 0xa9558b72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d58b7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x13958b7d, SUC_lexicon_tags_7 },
    { 0xdb958b7e, SUC_lexicon_tags_43 },
    { 0x13958b7e, SUC_lexicon_tags_104 },
    { 0x31d58b7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21758b82, SUC_lexicon_tags_46 },
    { 0x53f58b82, SUC_lexicon_tags_61 },
    { 0x39d58b84, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x8ef58b86, SUC_lexicon_tags_15 },
    { 0x51758b86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9758b89, SUC_lexicon_tags_26 },
    { 0xbff58b8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd358b90, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec558b95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf358b99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x358b9b, SUC_lexicon_tags_15 },
    { 0xe5158b9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6958b9e, SUC_lexicon_tags_12 },
    { 0x90358b9e, SUC_lexicon_tags_28 },
    { 0xc5758b9e, SUC_lexicon_tags_37 },
    { 0x4eb58ba1, SUC_lexicon_tags_47 },
    { 0xa1358ba2, SUC_lexicon_tags_12 },
    { 0xa9558ba3, SUC_lexicon_tags_25 },
    { 0x6f158ba4, SUC_lexicon_tags_55 },
    { 0xd7758ba4, SUC_lexicon_tags_47 },
    { 0x99f58ba2, SUC_lexicon_tags_37 },
    { 0x3ab58ba4, SUC_lexicon_tags_31 },
    { 0x33158ba8, SUC_lexicon_tags_11 },
    { 0x1d158ba4, SUC_lexicon_tags_15 },
    { 0x4e958ba1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd58bad, SUC_lexicon_tags_15 },
    { 0xb7d58bae, SUC_lexicon_tags_27 },
    { 0x7158bae, SUC_lexicon_tags_12 },
    { 0xeb758bb0, SUC_lexicon_tags_17 },
    { 0xac158bb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a758bb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf158bb9, SUC_lexicon_tags_11 },
    { 0x49d58bba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce358bbc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9758bc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9b58bc7, SUC_lexicon_tags_37 },
    { 0x2b758bc8, SUC_lexicon_tags_19 },
    { 0xde358bc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70358bcb, SUC_lexicon_tags_14 },
    { 0x24b58bcb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc358bd0, SUC_lexicon_tags_0 },
    { 0xe7358bd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1558bd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x96158bd7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa558bdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3358be3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d158be6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf0158be8, SUC_lexicon_tags_56 },
    { 0x33558be9, SUC_lexicon_tags_12 },
    { 0x3e958bea, SUC_lexicon_tags_6 },
    { 0xae158beb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d58bef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af58bfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29158bfc, SUC_lexicon_tags_28 },
    { 0x59758bfd, SUC_lexicon_tags_43 },
    { 0xab58bfe, SUC_lexicon_tags_27 },
    { 0x2c758bfe, SUC_lexicon_tags_7 },
    { 0x2c158c00, SUC_lexicon_tags_43 },
    { 0x6f58bff, SUC_lexicon_tags_315 },
    { 0x61558bfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89358c04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc558c06, SUC_lexicon_tags_0 },
    { 0x21d58c07, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x63158c0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xce758c0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b58c11, SUC_lexicon_tags_28 },
    { 0xe5158c11, SUC_lexicon_tags_45 },
    { 0x11f58c13, SUC_lexicon_tags_45 },
    { 0x1d158c13, SUC_lexicon_tags_87 },
    { 0x53958c11, SUC_lexicon_tags_25 },
    { 0xaf358c16, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x70158c18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1af58c1a, SUC_lexicon_tags_23 },
    { 0xea158c1b, SUC_lexicon_tags_15 },
    { 0xaa758c1c, SUC_lexicon_tags_106 },
    { 0xf0f58c1d, SUC_lexicon_tags_39 },
    { 0x43558c1d, SUC_lexicon_tags_0 },
    { 0x4d358c1f, SUC_lexicon_tags_43 },
    { 0xf4558c20, SUC_lexicon_tags_11 },
    { 0xf0d58c21, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x75558c23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7d58c25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24358c2c, SUC_lexicon_tags_47 },
    { 0x4b58c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c158c32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41358c37, SUC_lexicon_tags_56 },
    { 0xa0d58c38, SUC_lexicon_tags_37 },
    { 0x4e958c39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b958c3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb58c3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b958c42, SUC_lexicon_tags_1 },
    { 0x47758c43, SUC_lexicon_tags_14 },
    { 0x80758c42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4958c48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b58c4c, SUC_lexicon_tags_45 },
    { 0x6ab58c4d, SUC_lexicon_tags_27 },
    { 0x93d58c4d, SUC_lexicon_tags_17 },
    { 0x97d58c4f, SUC_lexicon_tags_55 },
    { 0x9b758c50, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb6558c52, SUC_lexicon_tags_9 },
    { 0x74558c53, SUC_lexicon_tags_11 },
    { 0x97f58c54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25158c58, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7f958c5a, SUC_lexicon_tags_27 },
    { 0x1ad58c5a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2158c5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b758c64, SUC_lexicon_tags_162 },
    { 0x91f58c65, SUC_lexicon_tags_30 },
    { 0x5e958c65, SUC_lexicon_tags_11 },
    { 0x9358c67, SUC_lexicon_tags_38 },
    { 0xc9358c68, SUC_lexicon_tags_39 },
    { 0x84b58c69, SUC_lexicon_tags_25 },
    { 0xce758c69, SUC_lexicon_tags_7 },
    { 0xab558c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99358c73, SUC_lexicon_tags_65 },
    { 0xa3758c74, SUC_lexicon_tags_201 },
    { 0x7b758c73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9758c78, SUC_lexicon_tags_19 },
    { 0x12158c79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88758c7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a758c81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb58c88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b58c8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b58c92, SUC_lexicon_tags_61 },
    { 0x1cd58c93, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x43f58c95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e958c97, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4558c9d, SUC_lexicon_tags_37 },
    { 0x80d58c9e, SUC_lexicon_tags_62 },
    { 0x1a558c9f, SUC_lexicon_tags_38 },
    { 0x5b158ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6358ca3, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d58ca8, SUC_lexicon_tags_15 },
    { 0x9d558ca9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e558cac, SUC_lexicon_tags_30 },
    { 0xd4158cad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb558cb2, SUC_lexicon_tags_47 },
    { 0xf558cb3, SUC_lexicon_tags_15 },
    { 0x59558cb4, SUC_lexicon_tags_17 },
    { 0x43d58cb5, SUC_lexicon_tags_11 },
    { 0x4cf58cb3, SUC_lexicon_tags_25 },
    { 0xfa558cb2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b58cba, SUC_lexicon_tags_7 },
    { 0xa6158cbb, SUC_lexicon_tags_38 },
    { 0xe1758cbc, SUC_lexicon_tags_7 },
    { 0xa9758cbd, SUC_lexicon_tags_37 },
    { 0xfa758cbe, SUC_lexicon_tags_63 },
    { 0x4cf58cbf, SUC_lexicon_tags_45 },
    { 0x4c558cbf, SUC_lexicon_tags_27 },
    { 0xb0158cba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9758cc3, SUC_lexicon_tags_65 },
    { 0xa6558cc4, SUC_lexicon_tags_77 },
    { 0x6bf58cc4, SUC_lexicon_tags_84 },
    { 0xf1758cc6, SUC_lexicon_tags_17 },
    { 0x6c158cc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf758ccc, SUC_lexicon_tags_9 },
    { 0x16158ccd, SUC_lexicon_tags_23 },
    { 0x59558cce, SUC_lexicon_tags_27 },
    { 0x65358ccc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5558cd3, SUC_lexicon_tags_11 },
    { 0x69758cd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23358cd6, SUC_lexicon_tags_47 },
    { 0xb9958cd7, SUC_lexicon_tags_88 },
    { 0x7bd58cd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f958cdb, SUC_lexicon_tags_0 },
    { 0x57758cdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec958cdf, SUC_lexicon_tags_47 },
    { 0x28b58cdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29158ce2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a958ce7, SUC_lexicon_tags_16 },
    { 0x23d58ce8, SUC_lexicon_tags_0 },
    { 0xb58ce9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd158ced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9558cf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bd58cf3, SUC_lexicon_tags_464 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88758cf8, SUC_lexicon_tags_0 },
    { 0xb758cf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78358cfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94958d01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcab58d03, SUC_lexicon_tags_10 },
    { 0xc758d04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6558d06, SUC_lexicon_tags_25 },
    { 0xfab58d07, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe558d0b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed58d11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed758d18, SUC_lexicon_tags_1 },
    { 0x98d58d19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b758d23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a958d26, SUC_lexicon_tags_33 },
    { 0x73f58d27, SUC_lexicon_tags_25 },
    { 0x958d28, SUC_lexicon_tags_2 },
    { 0xb7f58d29, SUC_lexicon_tags_48 },
    { 0x55358d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b58d2e, SUC_lexicon_tags_8 },
    { 0x6d558d2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89b58d31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb58d34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86358d39, SUC_lexicon_tags_43 },
    { 0x12558d39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b758d3c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40758d40, SUC_lexicon_tags_17 },
    { 0x1f358d41, SUC_lexicon_tags_39 },
    { 0x4cf58d42, SUC_lexicon_tags_12 },
    { 0xa6b58d43, SUC_lexicon_tags_11 },
    { 0x9d958d43, SUC_lexicon_tags_27 },
    { 0x7c958d44, SUC_lexicon_tags_29 },
    { 0x4a558d46, SUC_lexicon_tags_15 },
    { 0xfd558d47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3cd58d49, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x22158d4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8958d4d, SUC_lexicon_tags_37 },
    { 0xb4958d4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33f58d50, SUC_lexicon_tags_63 },
    { 0x3dd58d51, SUC_lexicon_tags_11 },
    { 0xe2158d52, SUC_lexicon_tags_17 },
    { 0x60158d53, SUC_lexicon_tags_61 },
    { 0xa5d58d52, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x39b58d56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e758d58, SUC_lexicon_tags_15 },
    { 0xa4b58d58, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39958d5f, SUC_lexicon_tags_1 },
    { 0x39f58d60, SUC_lexicon_tags_25 },
    { 0x55158d61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d58d68, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d58d6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6758d70, SUC_lexicon_tags_0 },
    { 0x37358d71, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e758d77, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xe8d58d79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1158d7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f58d84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3358d88, SUC_lexicon_tags_25 },
    { 0x71b58d89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9f58d8b, SUC_lexicon_tags_33 },
    { 0x82f58d8b, SUC_lexicon_tags_28 },
    { 0x7758d8d, SUC_lexicon_tags_2 },
    { 0xe1558d8e, SUC_lexicon_tags_23 },
    { 0x51b58d8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0b58d91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd958d95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55958d98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b958d9a, SUC_lexicon_tags_31 },
    { 0xbc358d9b, SUC_lexicon_tags_10 },
    { 0x5f158d9a, SUC_lexicon_tags_37 },
    { 0x26158d9c, SUC_lexicon_tags_45 },
    { 0x4a758d9e, SUC_lexicon_tags_37 },
    { 0xe0f58d9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32158da1, SUC_lexicon_tags_45 },
    { 0x60158da1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1958da6, SUC_lexicon_tags_7 },
    { 0x11d58da7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4558dac, SUC_lexicon_tags_12 },
    { 0x1d558dac, SUC_lexicon_tags_45 },
    { 0x8e758dac, SUC_lexicon_tags_15 },
    { 0xbfb58dae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd58db2, SUC_lexicon_tags_31 },
    { 0xaf958db3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc7958db5, SUC_lexicon_tags_4 },
    { 0x37558db5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f758db8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2fd58dba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3758dbf, SUC_lexicon_tags_1 },
    { 0x9c758dc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d58dc3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa358dc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe758dc9, SUC_lexicon_tags_11 },
    { 0x70d58dca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e958dcd, SUC_lexicon_tags_48 },
    { 0x36f58dcd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd58dd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd358dd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e158dda, SUC_lexicon_tags_2 },
    { 0x6f958dda, SUC_lexicon_tags_19 },
    { 0xd7d58ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14d58dde, SUC_lexicon_tags_48 },
    { 0x8cf58dde, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94758de5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23158dee, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d58df2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc158df4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b58dfb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b58dfe, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x47758e00, SUC_lexicon_tags_17 },
    { 0x83d58e00, SUC_lexicon_tags_1 },
    { 0xe3358e00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3758e06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf58e0a, SUC_lexicon_tags_47 },
    { 0x7c558e0b, SUC_lexicon_tags_26 },
    { 0xe3758e0c, SUC_lexicon_tags_30 },
    { 0xfcf58e0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb58e11, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2758e15, SUC_lexicon_tags_77 },
    { 0xb3358e16, SUC_lexicon_tags_11 },
    { 0x13758e17, SUC_lexicon_tags_12 },
    { 0x4958e15, SUC_lexicon_tags_45 },
    { 0xd7f58e19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb158e1b, SUC_lexicon_tags_47 },
    { 0xf5758e1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4958e1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b758e26, SUC_lexicon_tags_43 },
    { 0x8ff58e27, SUC_lexicon_tags_25 },
    { 0x28158e27, SUC_lexicon_tags_7 },
    { 0xd8958e28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6558e2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc4558e2e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb58e33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b58e36, SUC_lexicon_tags_15 },
    { 0xe9358e37, SUC_lexicon_tags_148 },
    { 0xee958e37, SUC_lexicon_tags_62 },
    { 0xfd558e39, SUC_lexicon_tags_60 },
    { 0xa158e39, SUC_lexicon_tags_2 },
    { 0x66158e36, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d558e40, SUC_lexicon_tags_46 },
    { 0xb1d58e41, SUC_lexicon_tags_12 },
    { 0x8f958e41, SUC_lexicon_tags_25 },
    { 0x34d58e43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e358e47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46558e4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29558e52, SUC_lexicon_tags_25 },
    { 0x2d558e52, SUC_lexicon_tags_36 },
    { 0xceb58e53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55558e56, SUC_lexicon_tags_30 },
    { 0xd0158e57, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5c758e59, SUC_lexicon_tags_14 },
    { 0xbf958e59, SUC_lexicon_tags_17 },
    { 0x97f58e5b, SUC_lexicon_tags_9 },
    { 0x94b58e59, SUC_lexicon_tags_9 },
    { 0x55d58e5d, SUC_lexicon_tags_9 },
    { 0x4af58e5e, SUC_lexicon_tags_39 },
    { 0xde758e5e, SUC_lexicon_tags_534 },
    { 0x22158e5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f358e63, SUC_lexicon_tags_12 },
    { 0x24358e63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f58e68, SUC_lexicon_tags_15 },
    { 0x1eb58e69, SUC_lexicon_tags_15 },
    { 0x67358e69, SUC_lexicon_tags_3 },
    { 0x94f58e69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff58e70, SUC_lexicon_tags_38 },
    { 0x32b58e70, SUC_lexicon_tags_37 },
    { 0x22758e71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab58e75, SUC_lexicon_tags_6 },
    { 0x62958e76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea358e7a, SUC_lexicon_tags_29 },
    { 0xfff58e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57d58e7d, SUC_lexicon_tags_11 },
    { 0xb0f58e7e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb58e86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba358e8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb0758e8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b58e91, SUC_lexicon_tags_8 },
    { 0x66b58e92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3958e94, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70158e9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b58e9f, SUC_lexicon_tags_37 },
    { 0x7cb58ea0, SUC_lexicon_tags_12 },
    { 0x3d358e9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16958ea3, SUC_lexicon_tags_1 },
    { 0xf7158ea4, SUC_lexicon_tags_28 },
    { 0x5a558ea5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd5358ea7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc758ea9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf958eab, SUC_lexicon_tags_23 },
    { 0x97558eab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d58eb4, SUC_lexicon_tags_6 },
    { 0x37958eb5, SUC_lexicon_tags_37 },
    { 0xc1758eb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11558ebb, SUC_lexicon_tags_1 },
    { 0xebf58ebc, SUC_lexicon_tags_38 },
    { 0x3cb58ebd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94758ec1, SUC_lexicon_tags_15 },
    { 0x4b158ec1, SUC_lexicon_tags_11 },
    { 0x4cf58ec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ff58ec5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d358ec9, SUC_lexicon_tags_15 },
    { 0x9bd58eca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdbb58ecc, SUC_lexicon_tags_19 },
    { 0x23958ecd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f58ed0, SUC_lexicon_tags_15 },
    { 0x95b58ed1, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d58ed6, SUC_lexicon_tags_38 },
    { 0x9b58ed7, SUC_lexicon_tags_17 },
    { 0xaed58ed7, SUC_lexicon_tags_17 },
    { 0xc4958ed6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xde358edc, SUC_lexicon_tags_6 },
    { 0xe6358edd, SUC_lexicon_tags_0 },
    { 0x5ad58ede, SUC_lexicon_tags_6 },
    { 0x73358edf, SUC_lexicon_tags_35 },
    { 0x82958ee0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xc5b58ee2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0f58ee4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f58ee7, SUC_lexicon_tags_45 },
    { 0x2ed58ee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83758ef7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad758efe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd158f02, SUC_lexicon_tags_7 },
    { 0xa1758f02, SUC_lexicon_tags_12 },
    { 0x9ab58f03, SUC_lexicon_tags_1 },
    { 0xa9758f05, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56758f12, SUC_lexicon_tags_12 },
    { 0x54f58f12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed58f15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d58f19, SUC_lexicon_tags_60 },
    { 0x9e758f1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59358f20, SUC_lexicon_tags_26 },
    { 0x52758f21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5bb58f23, SUC_lexicon_tags_66 },
    { 0x16758f24, SUC_lexicon_tags_17 },
    { 0x82158f23, SUC_lexicon_tags_27 },
    { 0xfdd58f26, SUC_lexicon_tags_11 },
    { 0x78b58f27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63158f29, SUC_lexicon_tags_30 },
    { 0x76758f29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x86158f2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf158f34, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x77158f36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca158f39, SUC_lexicon_tags_29 },
    { 0xe6358f3a, SUC_lexicon_tags_45 },
    { 0x63758f3b, SUC_lexicon_tags_77 },
    { 0xfef58f3c, SUC_lexicon_tags_38 },
    { 0x56358f3c, SUC_lexicon_tags_29 },
    { 0xf0b58f3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f58f46, SUC_lexicon_tags_79 },
    { 0xed58f46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d58f4a, SUC_lexicon_tags_2 },
    { 0x65b58f4b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x91358f4e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d58f51, SUC_lexicon_tags_33 },
    { 0xcb358f52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30958f54, SUC_lexicon_tags_37 },
    { 0x58358f55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44558f5a, SUC_lexicon_tags_49 },
    { 0x7eb58f5b, SUC_lexicon_tags_1 },
    { 0x7f958f5c, SUC_lexicon_tags_19 },
    { 0x6b758f5d, SUC_lexicon_tags_47 },
    { 0x9958f5e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x89358f61, SUC_lexicon_tags_91 },
    { 0x7d958f62, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f958f65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d58f68, SUC_lexicon_tags_8 },
    { 0x1b758f68, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe9958f6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8558f6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6358f72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c358f74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x13b58f76, SUC_lexicon_tags_19 },
    { 0xa4358f77, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x47b58f79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1358f7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b758f80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36d58f82, SUC_lexicon_tags_39 },
    { 0x46d58f83, SUC_lexicon_tags_25 },
    { 0xe4358f83, SUC_lexicon_tags_63 },
    { 0xd5f58f85, SUC_lexicon_tags_31 },
    { 0x3bb58f86, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee158f8a, SUC_lexicon_tags_505 },
    { 0, NULL },
    { 0x5f358f8c, SUC_lexicon_tags_14 },
    { 0xe0358f8d, SUC_lexicon_tags_12 },
    { 0x9cf58f8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38958f93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69958f95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41158f98, SUC_lexicon_tags_8 },
    { 0xbe358f98, SUC_lexicon_tags_12 },
    { 0x7f558f99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff58f9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e358fa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a558fa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34358fa8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3958fae, SUC_lexicon_tags_23 },
    { 0xf8758fae, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xd1358fb1, SUC_lexicon_tags_12 },
    { 0xe4b58fb1, SUC_lexicon_tags_19 },
    { 0x1c158fb3, SUC_lexicon_tags_0 },
    { 0xfab58fb3, SUC_lexicon_tags_33 },
    { 0x73758fb1, SUC_lexicon_tags_59 },
    { 0x3bf58fb6, SUC_lexicon_tags_47 },
    { 0x48558fb7, SUC_lexicon_tags_7 },
    { 0x67158fb7, SUC_lexicon_tags_11 },
    { 0x9eb58fb9, SUC_lexicon_tags_25 },
    { 0xa9558fba, SUC_lexicon_tags_14 },
    { 0xbc158fbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7d58fbd, SUC_lexicon_tags_37 },
    { 0xe5158fbe, SUC_lexicon_tags_23 },
    { 0xdbf58fbf, SUC_lexicon_tags_14 },
    { 0xb0358fbf, SUC_lexicon_tags_23 },
    { 0x8c958fbe, SUC_lexicon_tags_2 },
    { 0xeb158fbf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32358fca, SUC_lexicon_tags_11 },
    { 0xb6558fcb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x58158fcd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1158fd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd58fd3, SUC_lexicon_tags_12 },
    { 0x8a758fd4, SUC_lexicon_tags_7 },
    { 0x6bf58fd5, SUC_lexicon_tags_37 },
    { 0xceb58fd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1758fd9, SUC_lexicon_tags_92 },
    { 0x20358fd9, SUC_lexicon_tags_11 },
    { 0x63358fdb, SUC_lexicon_tags_5 },
    { 0x98158fdc, SUC_lexicon_tags_13 },
    { 0x17158fdd, SUC_lexicon_tags_38 },
    { 0xafb58fdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df58fe2, SUC_lexicon_tags_77 },
    { 0x2b58fe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cd58fe5, SUC_lexicon_tags_49 },
    { 0xc7b58fe6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b58ff0, SUC_lexicon_tags_15 },
    { 0xc9d58ff0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3958ff3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10758ff7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf58ffc, SUC_lexicon_tags_2 },
    { 0x34158ffd, SUC_lexicon_tags_17 },
    { 0xc3b58ffe, SUC_lexicon_tags_49 },
    { 0xf0758fff, SUC_lexicon_tags_7 },
    { 0x5b759000, SUC_lexicon_tags_11 },
    { 0x77f59000, SUC_lexicon_tags_8 },
    { 0xcaf58ffc, SUC_lexicon_tags_23 },
    { 0x32b59003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9159008, SUC_lexicon_tags_15 },
    { 0xdc559009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf675900b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe75900e, SUC_lexicon_tags_7 },
    { 0x5335900e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x80159011, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd59016, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f559029, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b59032, SUC_lexicon_tags_91 },
    { 0x19359032, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c159037, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f559039, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x64f5903b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10959040, SUC_lexicon_tags_39 },
    { 0xfd559041, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5759044, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d59049, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaf95904b, SUC_lexicon_tags_77 },
    { 0xecd5904b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32f5904e, SUC_lexicon_tags_60 },
    { 0x73f5904e, SUC_lexicon_tags_45 },
    { 0x56959050, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb6359052, SUC_lexicon_tags_27 },
    { 0x88d59053, SUC_lexicon_tags_17 },
    { 0xd9b59054, SUC_lexicon_tags_11 },
    { 0x4b159055, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b59059, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e95905d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9435905f, SUC_lexicon_tags_63 },
    { 0xa4759060, SUC_lexicon_tags_39 },
    { 0xefb59060, SUC_lexicon_tags_65 },
    { 0xfb759060, SUC_lexicon_tags_27 },
    { 0xf8f59061, SUC_lexicon_tags_47 },
    { 0x2dd59060, SUC_lexicon_tags_46 },
    { 0xba759060, SUC_lexicon_tags_43 },
    { 0x5c359066, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9c759068, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d5906d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xdc95906f, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f59072, SUC_lexicon_tags_37 },
    { 0x55b59073, SUC_lexicon_tags_91 },
    { 0x41159074, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xa8759076, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f15907a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b15907c, SUC_lexicon_tags_12 },
    { 0x1f75907d, SUC_lexicon_tags_17 },
    { 0x4515907d, SUC_lexicon_tags_14 },
    { 0xbb5907f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b59082, SUC_lexicon_tags_11 },
    { 0x1b559083, SUC_lexicon_tags_27 },
    { 0x2a359084, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff59087, SUC_lexicon_tags_2 },
    { 0xcf159087, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d5908b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72359090, SUC_lexicon_tags_25 },
    { 0x90b59091, SUC_lexicon_tags_9 },
    { 0x35159092, SUC_lexicon_tags_79 },
    { 0x7d959093, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb559096, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa835909f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x43590a1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd590a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d590aa, SUC_lexicon_tags_38 },
    { 0xffb590aa, SUC_lexicon_tags_47 },
    { 0x32d590ac, SUC_lexicon_tags_46 },
    { 0x31f590ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961590b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f590b8, SUC_lexicon_tags_79 },
    { 0x517590b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xddd590bb, SUC_lexicon_tags_30 },
    { 0x5bb590bc, SUC_lexicon_tags_17 },
    { 0x69f590bc, SUC_lexicon_tags_37 },
    { 0x455590bd, SUC_lexicon_tags_2 },
    { 0x8bf590be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d590c2, SUC_lexicon_tags_30 },
    { 0x26d590c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb590c7, SUC_lexicon_tags_25 },
    { 0x73590c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac5590ca, SUC_lexicon_tags_23 },
    { 0xe55590cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49590ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e7590d0, SUC_lexicon_tags_15 },
    { 0x32d590d1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x729590d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x987590d6, SUC_lexicon_tags_33 },
    { 0x82f590d6, SUC_lexicon_tags_50 },
    { 0xd81590d8, SUC_lexicon_tags_2 },
    { 0x48b590d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7590dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7d590de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b590e2, SUC_lexicon_tags_2 },
    { 0xf7590e3, SUC_lexicon_tags_37 },
    { 0x903590e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd590f3, SUC_lexicon_tags_14 },
    { 0x6db590f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b590f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b590fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x635590fd, SUC_lexicon_tags_30 },
    { 0xbab590fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56d59100, SUC_lexicon_tags_25 },
    { 0x5e159100, SUC_lexicon_tags_6 },
    { 0x72f59101, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e759105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad5910a, SUC_lexicon_tags_69 },
    { 0x9995910b, SUC_lexicon_tags_49 },
    { 0xd0b5910c, SUC_lexicon_tags_55 },
    { 0xdc75910d, SUC_lexicon_tags_43 },
    { 0xdc55910d, SUC_lexicon_tags_15 },
    { 0x4e35910e, SUC_lexicon_tags_170 },
    { 0x4d959110, SUC_lexicon_tags_9 },
    { 0x2ad5910b, SUC_lexicon_tags_55 },
    { 0x5ad59112, SUC_lexicon_tags_15 },
    { 0x5fd59113, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d59117, SUC_lexicon_tags_2 },
    { 0x1e559117, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe135911a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42559120, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f59124, SUC_lexicon_tags_37 },
    { 0x19f59124, SUC_lexicon_tags_47 },
    { 0x46b59126, SUC_lexicon_tags_82 },
    { 0x4cf59126, SUC_lexicon_tags_33 },
    { 0x42d59126, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe3f5912a, SUC_lexicon_tags_2 },
    { 0xd655912b, SUC_lexicon_tags_0 },
    { 0xa555912c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1a15912e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e559134, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa1759136, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x26159138, SUC_lexicon_tags_55 },
    { 0x29f59139, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30b5913b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f59141, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42959145, SUC_lexicon_tags_47 },
    { 0xfa759145, SUC_lexicon_tags_1 },
    { 0xce159147, SUC_lexicon_tags_10 },
    { 0x3bb59148, SUC_lexicon_tags_11 },
    { 0xaa959149, SUC_lexicon_tags_2 },
    { 0x6d959149, SUC_lexicon_tags_141 },
    { 0x9c75914b, SUC_lexicon_tags_7 },
    { 0xb795914b, SUC_lexicon_tags_17 },
    { 0x2f15914d, SUC_lexicon_tags_23 },
    { 0x4ab59147, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x56959150, SUC_lexicon_tags_8 },
    { 0xfd559150, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff5915c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5915f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41959164, SUC_lexicon_tags_15 },
    { 0xfaf59165, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99359167, SUC_lexicon_tags_63 },
    { 0x95359167, SUC_lexicon_tags_50 },
    { 0x97359167, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf95916e, SUC_lexicon_tags_1 },
    { 0xa475916f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80559171, SUC_lexicon_tags_25 },
    { 0x3db59172, SUC_lexicon_tags_39 },
    { 0xb6159173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b559176, SUC_lexicon_tags_64 },
    { 0x2d959176, SUC_lexicon_tags_45 },
    { 0xf9b59178, SUC_lexicon_tags_48 },
    { 0x21f59179, SUC_lexicon_tags_12 },
    { 0x9ed59178, SUC_lexicon_tags_45 },
    { 0xfad5917b, SUC_lexicon_tags_16 },
    { 0xcc95917c, SUC_lexicon_tags_17 },
    { 0xdf59178, SUC_lexicon_tags_27 },
    { 0x14d5917a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd7d59180, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98359182, SUC_lexicon_tags_17 },
    { 0x25159183, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e359185, SUC_lexicon_tags_37 },
    { 0x74f59185, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e95918a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58d5918c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a35918f, SUC_lexicon_tags_25 },
    { 0x2075918f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77b59192, SUC_lexicon_tags_3 },
    { 0x4a759193, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86d59195, SUC_lexicon_tags_4 },
    { 0x8c959196, SUC_lexicon_tags_6 },
    { 0x18959197, SUC_lexicon_tags_25 },
    { 0x4f759198, SUC_lexicon_tags_15 },
    { 0x6d759197, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf655919b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3595919d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa535919f, SUC_lexicon_tags_60 },
    { 0xac9591a0, SUC_lexicon_tags_25 },
    { 0xfe3591a1, SUC_lexicon_tags_23 },
    { 0x115591a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x277591a4, SUC_lexicon_tags_11 },
    { 0x9e7591a5, SUC_lexicon_tags_45 },
    { 0xff3591a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3591ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf591b4, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xf7f591b6, SUC_lexicon_tags_39 },
    { 0x4bf591b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34d591b9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x549591bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3591c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7591c3, SUC_lexicon_tags_32 },
    { 0xf29591c4, SUC_lexicon_tags_12 },
    { 0xd71591c5, SUC_lexicon_tags_37 },
    { 0x2fb591c5, SUC_lexicon_tags_54 },
    { 0x16d591c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3591cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd19591ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b7591d0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1591d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1591db, SUC_lexicon_tags_63 },
    { 0x3f1591dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f591e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35591e3, SUC_lexicon_tags_37 },
    { 0x805591e3, SUC_lexicon_tags_27 },
    { 0x8a3591e5, SUC_lexicon_tags_2 },
    { 0x401591e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b591eb, SUC_lexicon_tags_33 },
    { 0xccd591ec, SUC_lexicon_tags_17 },
    { 0x547591ed, SUC_lexicon_tags_37 },
    { 0xc69591ed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb89591f0, SUC_lexicon_tags_2 },
    { 0x437591f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5f591f3, SUC_lexicon_tags_15 },
    { 0x641591f3, SUC_lexicon_tags_11 },
    { 0xc33591f5, SUC_lexicon_tags_12 },
    { 0x5d1591f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcab591f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf59206, SUC_lexicon_tags_37 },
    { 0x9fb59206, SUC_lexicon_tags_21 },
    { 0x15d59208, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b5920c, SUC_lexicon_tags_37 },
    { 0xb3d5920c, SUC_lexicon_tags_17 },
    { 0xd4d5920e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa559210, SUC_lexicon_tags_47 },
    { 0x7c959211, SUC_lexicon_tags_19 },
    { 0x87b59212, SUC_lexicon_tags_37 },
    { 0x68959211, SUC_lexicon_tags_11 },
    { 0x4fd59214, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26759218, SUC_lexicon_tags_25 },
    { 0xf1159219, SUC_lexicon_tags_25 },
    { 0x3c75921a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d59220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18559228, SUC_lexicon_tags_31 },
    { 0x99959228, SUC_lexicon_tags_16 },
    { 0x5a159228, SUC_lexicon_tags_15 },
    { 0x74b5922b, SUC_lexicon_tags_49 },
    { 0xb4b5922b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x655922f, SUC_lexicon_tags_26 },
    { 0x2ab59230, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7759232, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc235923c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe095923e, SUC_lexicon_tags_25 },
    { 0x86d5923f, SUC_lexicon_tags_37 },
    { 0x89b5923f, SUC_lexicon_tags_17 },
    { 0x36959241, SUC_lexicon_tags_6 },
    { 0x6c159242, SUC_lexicon_tags_19 },
    { 0xe8359242, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df59247, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a55924d, SUC_lexicon_tags_31 },
    { 0xea95924e, SUC_lexicon_tags_17 },
    { 0x52b5924f, SUC_lexicon_tags_7 },
    { 0x7f75924e, SUC_lexicon_tags_79 },
    { 0x4115924e, SUC_lexicon_tags_12 },
    { 0xccb5924e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73f59255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb759257, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x11d59259, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7975925d, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61759262, SUC_lexicon_tags_47 },
    { 0xd4f59262, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f59269, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7975926b, SUC_lexicon_tags_38 },
    { 0x8315926b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d59272, SUC_lexicon_tags_293 },
    { 0xb0d59272, SUC_lexicon_tags_11 },
    { 0x48559274, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62559276, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db5927b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4355927f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42159284, SUC_lexicon_tags_23 },
    { 0xb7959285, SUC_lexicon_tags_92 },
    { 0x49f59286, SUC_lexicon_tags_37 },
    { 0xacd59286, SUC_lexicon_tags_8 },
    { 0xd8b59287, SUC_lexicon_tags_47 },
    { 0xdd759286, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835928e, SUC_lexicon_tags_4 },
    { 0x44b5928f, SUC_lexicon_tags_43 },
    { 0xa4159290, SUC_lexicon_tags_37 },
    { 0xbb959290, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58159295, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b59297, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3159299, SUC_lexicon_tags_2 },
    { 0x5b5929a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x34d5929c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401592a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x443592a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x873592a4, SUC_lexicon_tags_66 },
    { 0xd5d592a5, SUC_lexicon_tags_73 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb592ab, SUC_lexicon_tags_11 },
    { 0xa13592ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x987592ae, SUC_lexicon_tags_14 },
    { 0xdaf592ae, SUC_lexicon_tags_25 },
    { 0x78d592ae, SUC_lexicon_tags_25 },
    { 0x49b592b1, SUC_lexicon_tags_12 },
    { 0x27d592ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe51592b4, SUC_lexicon_tags_17 },
    { 0xc7592b4, SUC_lexicon_tags_7 },
    { 0xe41592b4, SUC_lexicon_tags_11 },
    { 0xc37592b7, SUC_lexicon_tags_16 },
    { 0xff9592b7, SUC_lexicon_tags_42 },
    { 0x117592b9, SUC_lexicon_tags_38 },
    { 0x95d592b6, SUC_lexicon_tags_11 },
    { 0x703592bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bb592bd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x649592c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d592c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d592c7, SUC_lexicon_tags_4 },
    { 0x42b592c8, SUC_lexicon_tags_7 },
    { 0x821592c7, SUC_lexicon_tags_9 },
    { 0x4b592ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0b592cc, SUC_lexicon_tags_17 },
    { 0x4a1592cd, SUC_lexicon_tags_23 },
    { 0xdb7592ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ab592d0, SUC_lexicon_tags_37 },
    { 0x45d592d1, SUC_lexicon_tags_8 },
    { 0x339592d1, SUC_lexicon_tags_27 },
    { 0x49b592d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f592da, SUC_lexicon_tags_48 },
    { 0x227592db, SUC_lexicon_tags_26 },
    { 0xf77592dc, SUC_lexicon_tags_39 },
    { 0x7bd592dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa01592df, SUC_lexicon_tags_2 },
    { 0x47592df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967592e6, SUC_lexicon_tags_0 },
    { 0xe15592e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879592ed, SUC_lexicon_tags_26 },
    { 0x8b9592ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x245592f0, SUC_lexicon_tags_33 },
    { 0xf99592f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431592fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41959302, SUC_lexicon_tags_45 },
    { 0x5b559303, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98959306, SUC_lexicon_tags_37 },
    { 0xd1b59307, SUC_lexicon_tags_77 },
    { 0xa0159307, SUC_lexicon_tags_23 },
    { 0x76559307, SUC_lexicon_tags_17 },
    { 0x3e959308, SUC_lexicon_tags_10 },
    { 0x2f75930b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x14d5930d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2d5930f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17959311, SUC_lexicon_tags_56 },
    { 0xf359312, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba359316, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99559318, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x84d5931a, SUC_lexicon_tags_2 },
    { 0x4335931a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0959322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c759327, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc759329, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2975932b, SUC_lexicon_tags_57 },
    { 0x8c95932b, SUC_lexicon_tags_37 },
    { 0x5e35932d, SUC_lexicon_tags_114 },
    { 0x1195932e, SUC_lexicon_tags_1 },
    { 0x4f15932e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db59333, SUC_lexicon_tags_13 },
    { 0x6db59334, SUC_lexicon_tags_46 },
    { 0x8e959333, SUC_lexicon_tags_6 },
    { 0x9f159334, SUC_lexicon_tags_65 },
    { 0x48d59333, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb415933f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xfe959341, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8159349, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b5934c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc159353, SUC_lexicon_tags_37 },
    { 0x49159353, SUC_lexicon_tags_61 },
    { 0xd5b59355, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78559357, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b5935a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bd5935c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2755935f, SUC_lexicon_tags_15 },
    { 0xdcd59360, SUC_lexicon_tags_1 },
    { 0xb835935f, SUC_lexicon_tags_33 },
    { 0xb8959362, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5ad59364, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf159367, SUC_lexicon_tags_14 },
    { 0x2959368, SUC_lexicon_tags_37 },
    { 0xca359368, SUC_lexicon_tags_23 },
    { 0x8375936a, SUC_lexicon_tags_14 },
    { 0x6f5936b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d15936d, SUC_lexicon_tags_79 },
    { 0x9095936e, SUC_lexicon_tags_15 },
    { 0x9535936f, SUC_lexicon_tags_25 },
    { 0x35d59370, SUC_lexicon_tags_37 },
    { 0x20b59371, SUC_lexicon_tags_87 },
    { 0x8f359372, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x48759375, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5395937d, SUC_lexicon_tags_14 },
    { 0x3cf5937e, SUC_lexicon_tags_9 },
    { 0x7095937e, SUC_lexicon_tags_33 },
    { 0xb6d59380, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b59383, SUC_lexicon_tags_2 },
    { 0x11359384, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b5938a, SUC_lexicon_tags_2 },
    { 0x5ff5938a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b5938e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a359393, SUC_lexicon_tags_27 },
    { 0xacf59393, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f59397, SUC_lexicon_tags_68 },
    { 0x31759398, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe35939b, SUC_lexicon_tags_12 },
    { 0x1b55939c, SUC_lexicon_tags_49 },
    { 0x1e95939b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bf5939f, SUC_lexicon_tags_71 },
    { 0x505593a0, SUC_lexicon_tags_11 },
    { 0xab7593a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3593a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8df593a7, SUC_lexicon_tags_47 },
    { 0x9a3593a7, SUC_lexicon_tags_12 },
    { 0xbd9593a9, SUC_lexicon_tags_7 },
    { 0xdd7593a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3593ad, SUC_lexicon_tags_49 },
    { 0x2b9593ae, SUC_lexicon_tags_25 },
    { 0x4cf593ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x127593b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5ef593b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x503593b5, SUC_lexicon_tags_0 },
    { 0xa89593b6, SUC_lexicon_tags_40 },
    { 0xbf9593b6, SUC_lexicon_tags_63 },
    { 0x389593b8, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x97d593ba, SUC_lexicon_tags_39 },
    { 0xd33593bb, SUC_lexicon_tags_66 },
    { 0x767593bc, SUC_lexicon_tags_7 },
    { 0xca1593bd, SUC_lexicon_tags_11 },
    { 0xd57593be, SUC_lexicon_tags_12 },
    { 0x685593be, SUC_lexicon_tags_194 },
    { 0x8a7593be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb7593c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643593c6, SUC_lexicon_tags_25 },
    { 0xa77593c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851593cf, SUC_lexicon_tags_37 },
    { 0xe4b593cf, SUC_lexicon_tags_65 },
    { 0xf0d593d1, SUC_lexicon_tags_17 },
    { 0x629593d1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7593d7, SUC_lexicon_tags_87 },
    { 0xc39593d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61593df, SUC_lexicon_tags_0 },
    { 0x7b9593e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d593e3, SUC_lexicon_tags_12 },
    { 0xaf9593e3, SUC_lexicon_tags_47 },
    { 0x23d593e4, SUC_lexicon_tags_27 },
    { 0xfc3593e4, SUC_lexicon_tags_1 },
    { 0x6c3593e5, SUC_lexicon_tags_109 },
    { 0x1d9593e8, SUC_lexicon_tags_17 },
    { 0xb27593e8, SUC_lexicon_tags_26 },
    { 0x97b593ea, SUC_lexicon_tags_17 },
    { 0xe8f593e6, SUC_lexicon_tags_109 },
    { 0x8f593ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x249593ef, SUC_lexicon_tags_0 },
    { 0x123593ef, SUC_lexicon_tags_12 },
    { 0xb7b593f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf593f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65593fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8bb593fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf25593ff, SUC_lexicon_tags_31 },
    { 0x43359400, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27d59402, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc559404, SUC_lexicon_tags_55 },
    { 0x7b59405, SUC_lexicon_tags_60 },
    { 0x45f59406, SUC_lexicon_tags_2 },
    { 0x13359405, SUC_lexicon_tags_8 },
    { 0xf6559408, SUC_lexicon_tags_15 },
    { 0x24759406, SUC_lexicon_tags_37 },
    { 0x8f15940a, SUC_lexicon_tags_12 },
    { 0x8f35940a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2d5940d, SUC_lexicon_tags_7 },
    { 0x30b5940d, SUC_lexicon_tags_37 },
    { 0xddb5940f, SUC_lexicon_tags_63 },
    { 0xec359410, SUC_lexicon_tags_7 },
    { 0xde559411, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ef59413, SUC_lexicon_tags_19 },
    { 0xa4959413, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb59417, SUC_lexicon_tags_9 },
    { 0x53959418, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf155941a, SUC_lexicon_tags_25 },
    { 0x4cf5941b, SUC_lexicon_tags_14 },
    { 0xfe35941b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d5941f, SUC_lexicon_tags_47 },
    { 0x67f5941f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b359424, SUC_lexicon_tags_16 },
    { 0x70d59425, SUC_lexicon_tags_12 },
    { 0xdad59424, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4935942a, SUC_lexicon_tags_17 },
    { 0x4db5942b, SUC_lexicon_tags_23 },
    { 0x42d5942b, SUC_lexicon_tags_38 },
    { 0x5595942d, SUC_lexicon_tags_37 },
    { 0x4d5942e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf4759430, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xed959433, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaef59435, SUC_lexicon_tags_37 },
    { 0x41559435, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d5943a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd35943c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4415943e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad759444, SUC_lexicon_tags_63 },
    { 0x76d59445, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe6f59447, SUC_lexicon_tags_9 },
    { 0x7cb59448, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7f5944a, SUC_lexicon_tags_38 },
    { 0xab75944b, SUC_lexicon_tags_12 },
    { 0xa3f5944c, SUC_lexicon_tags_33 },
    { 0xb715944d, SUC_lexicon_tags_29 },
    { 0x2ad5944b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3559451, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8d59453, SUC_lexicon_tags_57 },
    { 0xa7159454, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb559456, SUC_lexicon_tags_27 },
    { 0x9f959457, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4275945c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b5945f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21b59461, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91359463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33559466, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdad59468, SUC_lexicon_tags_7 },
    { 0x7c159468, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb395946b, SUC_lexicon_tags_84 },
    { 0xd4f5946c, SUC_lexicon_tags_12 },
    { 0x1995946d, SUC_lexicon_tags_15 },
    { 0xde15946e, SUC_lexicon_tags_7 },
    { 0x2d95946f, SUC_lexicon_tags_15 },
    { 0x8815946d, SUC_lexicon_tags_39 },
    { 0xa359470, SUC_lexicon_tags_27 },
    { 0x61b59472, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b59474, SUC_lexicon_tags_23 },
    { 0x20359474, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x41359477, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d5947a, SUC_lexicon_tags_25 },
    { 0xb7f5947b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee35947e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3359480, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x59b59482, SUC_lexicon_tags_37 },
    { 0xeb359483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b59489, SUC_lexicon_tags_9 },
    { 0xbf759489, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9395948f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ab59491, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x44959493, SUC_lexicon_tags_17 },
    { 0xed359494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63759496, SUC_lexicon_tags_32 },
    { 0x6b59497, SUC_lexicon_tags_55 },
    { 0x6d559497, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ef5949a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1215949e, SUC_lexicon_tags_60 },
    { 0x9295949f, SUC_lexicon_tags_35 },
    { 0x5a3594a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x167594a3, SUC_lexicon_tags_27 },
    { 0xfc5594a4, SUC_lexicon_tags_19 },
    { 0x77594a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17594a8, SUC_lexicon_tags_35 },
    { 0xbad594a9, SUC_lexicon_tags_38 },
    { 0xa0f594a9, SUC_lexicon_tags_23 },
    { 0xe79594ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x883594ae, SUC_lexicon_tags_65 },
    { 0x537594af, SUC_lexicon_tags_23 },
    { 0x9cb594ae, SUC_lexicon_tags_25 },
    { 0xc01594af, SUC_lexicon_tags_1 },
    { 0x2df594b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1594b7, SUC_lexicon_tags_2 },
    { 0x167594b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x699594ba, SUC_lexicon_tags_66 },
    { 0x6a5594bb, SUC_lexicon_tags_7 },
    { 0xb67594bc, SUC_lexicon_tags_14 },
    { 0xab1594bd, SUC_lexicon_tags_25 },
    { 0x85b594be, SUC_lexicon_tags_6 },
    { 0x153594bf, SUC_lexicon_tags_19 },
    { 0x1b5594c0, SUC_lexicon_tags_3 },
    { 0xf1b594c1, SUC_lexicon_tags_7 },
    { 0xa5b594bf, SUC_lexicon_tags_0 },
    { 0xf61594bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89594ca, SUC_lexicon_tags_26 },
    { 0xc9b594cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3594cf, SUC_lexicon_tags_63 },
    { 0x1c9594d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d594d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd594d9, SUC_lexicon_tags_47 },
    { 0x88f594d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xddb594dc, SUC_lexicon_tags_7 },
    { 0x92d594dc, SUC_lexicon_tags_12 },
    { 0x39b594dd, SUC_lexicon_tags_71 },
    { 0xbfd594dd, SUC_lexicon_tags_2 },
    { 0x86f594e0, SUC_lexicon_tags_19 },
    { 0x44f594e1, SUC_lexicon_tags_43 },
    { 0x71f594e1, SUC_lexicon_tags_12 },
    { 0xee3594e3, SUC_lexicon_tags_1 },
    { 0xf57594e3, SUC_lexicon_tags_15 },
    { 0xa5b594e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x925594e8, SUC_lexicon_tags_14 },
    { 0x743594e9, SUC_lexicon_tags_17 },
    { 0xa2d594ea, SUC_lexicon_tags_8 },
    { 0x52f594eb, SUC_lexicon_tags_23 },
    { 0xa35594ec, SUC_lexicon_tags_6 },
    { 0x19f594ea, SUC_lexicon_tags_2 },
    { 0xa9f594ea, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x183594f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae9594f2, SUC_lexicon_tags_46 },
    { 0x24b594f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89594f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x283594f9, SUC_lexicon_tags_37 },
    { 0x895594f9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1d7594fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x33d594fe, SUC_lexicon_tags_85 },
    { 0x1f3594ff, SUC_lexicon_tags_11 },
    { 0xdcb594ff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2359504, SUC_lexicon_tags_13 },
    { 0x7ed59505, SUC_lexicon_tags_51 },
    { 0x9759505, SUC_lexicon_tags_1 },
    { 0x68d59504, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb159509, SUC_lexicon_tags_37 },
    { 0x4a95950a, SUC_lexicon_tags_2 },
    { 0xfc559509, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2255950f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9359512, SUC_lexicon_tags_11 },
    { 0x1e159512, SUC_lexicon_tags_36 },
    { 0xf4559514, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5655951a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2675951f, SUC_lexicon_tags_17 },
    { 0x53559520, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64559529, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab59530, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60159534, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ed59536, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x19359538, SUC_lexicon_tags_27 },
    { 0x91f59539, SUC_lexicon_tags_9 },
    { 0x4935953a, SUC_lexicon_tags_25 },
    { 0x31f5953a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9275953d, SUC_lexicon_tags_43 },
    { 0x5ab5953e, SUC_lexicon_tags_12 },
    { 0xd5b5953f, SUC_lexicon_tags_53 },
    { 0xdbf59540, SUC_lexicon_tags_0 },
    { 0xc0b59540, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3095954c, SUC_lexicon_tags_7 },
    { 0x67f5954d, SUC_lexicon_tags_23 },
    { 0x6d95954e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeb59550, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc9f59552, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b559555, SUC_lexicon_tags_1 },
    { 0x8f559555, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1159559, SUC_lexicon_tags_27 },
    { 0x4bf5955a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ad5955c, SUC_lexicon_tags_23 },
    { 0xe575955d, SUC_lexicon_tags_12 },
    { 0xbc55955e, SUC_lexicon_tags_53 },
    { 0x3235955f, SUC_lexicon_tags_25 },
    { 0xa15955c, SUC_lexicon_tags_37 },
    { 0x82759561, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f59566, SUC_lexicon_tags_10 },
    { 0xa5b59567, SUC_lexicon_tags_11 },
    { 0x2b959567, SUC_lexicon_tags_33 },
    { 0x81959569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d55956d, SUC_lexicon_tags_6 },
    { 0xddb5956e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf59575, SUC_lexicon_tags_26 },
    { 0x6e559576, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7095957a, SUC_lexicon_tags_23 },
    { 0x9995957b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d5957d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f15957f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xda159582, SUC_lexicon_tags_15 },
    { 0x8eb59583, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6e959585, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbc359587, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xab559589, SUC_lexicon_tags_17 },
    { 0x14359589, SUC_lexicon_tags_2 },
    { 0x6dd5958b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b559591, SUC_lexicon_tags_37 },
    { 0xd9b59592, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x45959594, SUC_lexicon_tags_13 },
    { 0x6a359595, SUC_lexicon_tags_2 },
    { 0x16959596, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc0159598, SUC_lexicon_tags_23 },
    { 0x6d959598, SUC_lexicon_tags_1 },
    { 0x1a759599, SUC_lexicon_tags_63 },
    { 0xf9b5959b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229595a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab595ad, SUC_lexicon_tags_25 },
    { 0x113595ae, SUC_lexicon_tags_15 },
    { 0x7eb595ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d595b7, SUC_lexicon_tags_28 },
    { 0xd7b595b8, SUC_lexicon_tags_219 },
    { 0xced595b7, SUC_lexicon_tags_38 },
    { 0xce5595b7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231595c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x945595c5, SUC_lexicon_tags_47 },
    { 0x6cf595c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb595cd, SUC_lexicon_tags_15 },
    { 0xa59595cd, SUC_lexicon_tags_23 },
    { 0xecf595cd, SUC_lexicon_tags_45 },
    { 0xf75595d0, SUC_lexicon_tags_11 },
    { 0x7b595d0, SUC_lexicon_tags_6 },
    { 0x927595cd, SUC_lexicon_tags_37 },
    { 0x771595d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d7595d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x911595d7, SUC_lexicon_tags_17 },
    { 0xbdd595d8, SUC_lexicon_tags_37 },
    { 0xcd595d9, SUC_lexicon_tags_0 },
    { 0x779595da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47595dd, SUC_lexicon_tags_63 },
    { 0x9a3595de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b595e2, SUC_lexicon_tags_68 },
    { 0x9f5595e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x591595e5, SUC_lexicon_tags_43 },
    { 0xf3b595e5, SUC_lexicon_tags_15 },
    { 0x3b3595e6, SUC_lexicon_tags_11 },
    { 0xa61595e6, SUC_lexicon_tags_11 },
    { 0xf17595e9, SUC_lexicon_tags_11 },
    { 0x25595ea, SUC_lexicon_tags_35 },
    { 0xd09595eb, SUC_lexicon_tags_9 },
    { 0xad5595ec, SUC_lexicon_tags_57 },
    { 0xcc3595e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd71595ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x511595f2, SUC_lexicon_tags_12 },
    { 0x809595f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6ab595f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3595fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b595fc, SUC_lexicon_tags_72 },
    { 0x901595fd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81559601, SUC_lexicon_tags_14 },
    { 0x75559601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebf59604, SUC_lexicon_tags_2 },
    { 0x37b59605, SUC_lexicon_tags_4 },
    { 0xdb559605, SUC_lexicon_tags_11 },
    { 0x21d59607, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd5960a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc2b5960c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76b5960e, SUC_lexicon_tags_11 },
    { 0x8fd5960f, SUC_lexicon_tags_0 },
    { 0x4455960e, SUC_lexicon_tags_47 },
    { 0x58b59611, SUC_lexicon_tags_14 },
    { 0x67559610, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99359615, SUC_lexicon_tags_11 },
    { 0xf4559615, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x8a959618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xed55961b, SUC_lexicon_tags_19 },
    { 0x5975961b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa755961f, SUC_lexicon_tags_38 },
    { 0x54559620, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d759622, SUC_lexicon_tags_7 },
    { 0xdc159623, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f59626, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2855962a, SUC_lexicon_tags_22 },
    { 0x20b5962a, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf59632, SUC_lexicon_tags_1 },
    { 0x2b959633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f959639, SUC_lexicon_tags_28 },
    { 0xfa75963a, SUC_lexicon_tags_19 },
    { 0xec959639, SUC_lexicon_tags_48 },
    { 0x56d5963c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78f5963e, SUC_lexicon_tags_16 },
    { 0x42d5963f, SUC_lexicon_tags_33 },
    { 0xac559640, SUC_lexicon_tags_45 },
    { 0x7c959641, SUC_lexicon_tags_25 },
    { 0x79359640, SUC_lexicon_tags_17 },
    { 0xbb159643, SUC_lexicon_tags_9 },
    { 0xe9d59643, SUC_lexicon_tags_44 },
    { 0xd7f59644, SUC_lexicon_tags_8 },
    { 0x55759644, SUC_lexicon_tags_74 },
    { 0x21959645, SUC_lexicon_tags_47 },
    { 0xf759648, SUC_lexicon_tags_43 },
    { 0xf5159648, SUC_lexicon_tags_37 },
    { 0xdab59645, SUC_lexicon_tags_37 },
    { 0xc9f5964b, SUC_lexicon_tags_37 },
    { 0x8b15964c, SUC_lexicon_tags_15 },
    { 0x2195964c, SUC_lexicon_tags_1 },
    { 0x28d5964e, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba159655, SUC_lexicon_tags_31 },
    { 0xfdd59656, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd2d59658, SUC_lexicon_tags_37 },
    { 0x59559659, SUC_lexicon_tags_0 },
    { 0x11b59658, SUC_lexicon_tags_11 },
    { 0x75f5965b, SUC_lexicon_tags_17 },
    { 0x67559659, SUC_lexicon_tags_8 },
    { 0xe9f5965d, SUC_lexicon_tags_7 },
    { 0x1035965e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x96559661, SUC_lexicon_tags_37 },
    { 0x61159662, SUC_lexicon_tags_31 },
    { 0x54159663, SUC_lexicon_tags_7 },
    { 0x66359662, SUC_lexicon_tags_11 },
    { 0xd4b59665, SUC_lexicon_tags_49 },
    { 0x89b59666, SUC_lexicon_tags_29 },
    { 0x73f59666, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf1159669, SUC_lexicon_tags_11 },
    { 0xd395966a, SUC_lexicon_tags_15 },
    { 0x3855966b, SUC_lexicon_tags_17 },
    { 0x4415966a, SUC_lexicon_tags_31 },
    { 0x1a15966d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78b5966f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e959673, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b159677, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd5967c, SUC_lexicon_tags_60 },
    { 0x32b5967d, SUC_lexicon_tags_15 },
    { 0xb075967e, SUC_lexicon_tags_114 },
    { 0x4ab5967d, SUC_lexicon_tags_31 },
    { 0x8895967c, SUC_lexicon_tags_1 },
    { 0x46d5967c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46f59683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a759685, SUC_lexicon_tags_37 },
    { 0xa6959686, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2415968a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b5968e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1759693, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b359695, SUC_lexicon_tags_43 },
    { 0x22359695, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255596a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5596a6, SUC_lexicon_tags_39 },
    { 0xd5d596a7, SUC_lexicon_tags_1 },
    { 0x3b7596a7, SUC_lexicon_tags_15 },
    { 0x443596a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd596ac, SUC_lexicon_tags_1 },
    { 0x91d596ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c5596af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913596b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5596b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff596bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x343596bf, SUC_lexicon_tags_13 },
    { 0x67b596c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79596c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4df596c8, SUC_lexicon_tags_8 },
    { 0xdbd596c8, SUC_lexicon_tags_2 },
    { 0x89b596c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2d3596cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1c9596ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8b596d0, SUC_lexicon_tags_54 },
    { 0x5b3596d0, SUC_lexicon_tags_9 },
    { 0xa7f596d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9596d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d596d8, SUC_lexicon_tags_49 },
    { 0x631596d9, SUC_lexicon_tags_77 },
    { 0x597596d8, SUC_lexicon_tags_39 },
    { 0x32d596d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf596e3, SUC_lexicon_tags_45 },
    { 0x841596e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223596ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6d596ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343596f0, SUC_lexicon_tags_43 },
    { 0x279596f1, SUC_lexicon_tags_17 },
    { 0xad9596f2, SUC_lexicon_tags_1558 },
    { 0x293596f3, SUC_lexicon_tags_27 },
    { 0xbe3596f3, SUC_lexicon_tags_12 },
    { 0x4af596f3, SUC_lexicon_tags_25 },
    { 0x165596f6, SUC_lexicon_tags_4 },
    { 0xe33596f7, SUC_lexicon_tags_26 },
    { 0x74b596f1, SUC_lexicon_tags_45 },
    { 0x399596f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7d7596fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd41596fd, SUC_lexicon_tags_2 },
    { 0x7a9596fe, SUC_lexicon_tags_12 },
    { 0xa59596fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87559702, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28159707, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x735970a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d759711, SUC_lexicon_tags_12 },
    { 0x28d59712, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc559717, SUC_lexicon_tags_14 },
    { 0x39159717, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d5971e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b759722, SUC_lexicon_tags_1 },
    { 0x11d59723, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b5972b, SUC_lexicon_tags_2 },
    { 0x1255972c, SUC_lexicon_tags_2 },
    { 0xa195972c, SUC_lexicon_tags_2 },
    { 0x9035972d, SUC_lexicon_tags_37 },
    { 0x4bd5972d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x32559731, SUC_lexicon_tags_11 },
    { 0xb7959732, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe959737, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b5973e, SUC_lexicon_tags_29 },
    { 0x5f55973f, SUC_lexicon_tags_35 },
    { 0x5d359740, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7959745, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb135974c, SUC_lexicon_tags_178 },
    { 0x2b95974c, SUC_lexicon_tags_17 },
    { 0xa175974e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3159751, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x17159753, SUC_lexicon_tags_27 },
    { 0x66159754, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd2159756, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f359759, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe155975b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4135975e, SUC_lexicon_tags_17 },
    { 0xaa75975f, SUC_lexicon_tags_15 },
    { 0xb7d59760, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x13759763, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf59765, SUC_lexicon_tags_18 },
    { 0x5f159766, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b59769, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d5976c, SUC_lexicon_tags_7 },
    { 0x575976d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb359771, SUC_lexicon_tags_9 },
    { 0x26d59772, SUC_lexicon_tags_12 },
    { 0xc1d59772, SUC_lexicon_tags_17 },
    { 0x29159774, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60159779, SUC_lexicon_tags_569 },
    { 0x1775977a, SUC_lexicon_tags_19 },
    { 0xa7f5977a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5455977d, SUC_lexicon_tags_7 },
    { 0x7755977e, SUC_lexicon_tags_47 },
    { 0xae75977e, SUC_lexicon_tags_25 },
    { 0x5a55977f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa9559782, SUC_lexicon_tags_14 },
    { 0x25759783, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6675978b, SUC_lexicon_tags_14 },
    { 0x915978c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9815978f, SUC_lexicon_tags_33 },
    { 0x2195978f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab59793, SUC_lexicon_tags_17 },
    { 0xf3f59794, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b5979b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5635979d, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31597a1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb597a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21597aa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d597b0, SUC_lexicon_tags_60 },
    { 0xc1d597b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45597b6, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0x67b597b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1597bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661597c1, SUC_lexicon_tags_27 },
    { 0x2b1597c2, SUC_lexicon_tags_82 },
    { 0x2af597c3, SUC_lexicon_tags_29 },
    { 0xf49597c4, SUC_lexicon_tags_25 },
    { 0x71b597c1, SUC_lexicon_tags_63 },
    { 0xed1597c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd3597c8, SUC_lexicon_tags_2 },
    { 0xdd9597c8, SUC_lexicon_tags_4 },
    { 0x885597ca, SUC_lexicon_tags_11 },
    { 0xc0b597c9, SUC_lexicon_tags_39 },
    { 0xc5b597cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f597d0, SUC_lexicon_tags_11 },
    { 0x41597d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x597597d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x269597d7, SUC_lexicon_tags_56 },
    { 0x5e5597d8, SUC_lexicon_tags_37 },
    { 0xa91597d9, SUC_lexicon_tags_45 },
    { 0x547597da, SUC_lexicon_tags_49 },
    { 0xea7597d8, SUC_lexicon_tags_7 },
    { 0x4d9597dc, SUC_lexicon_tags_25 },
    { 0x385597dc, SUC_lexicon_tags_1 },
    { 0x35b597d7, SUC_lexicon_tags_17 },
    { 0x459597d9, SUC_lexicon_tags_15 },
    { 0x21597e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1597e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc75597e5, SUC_lexicon_tags_1 },
    { 0xb21597e6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb597e9, SUC_lexicon_tags_43 },
    { 0x865597ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783597f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7597f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x667597fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73959802, SUC_lexicon_tags_7 },
    { 0x3ad59803, SUC_lexicon_tags_264 },
    { 0x86d59804, SUC_lexicon_tags_15 },
    { 0x3af59803, SUC_lexicon_tags_66 },
    { 0x2e959806, SUC_lexicon_tags_25 },
    { 0x18959806, SUC_lexicon_tags_1 },
    { 0x95759808, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc475980a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5980c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6359810, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5d59812, SUC_lexicon_tags_22 },
    { 0xd0159812, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81759817, SUC_lexicon_tags_26 },
    { 0x44359818, SUC_lexicon_tags_33 },
    { 0x57359819, SUC_lexicon_tags_17 },
    { 0x15559818, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af5981d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2559820, SUC_lexicon_tags_38 },
    { 0x44759821, SUC_lexicon_tags_6 },
    { 0xfab59822, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd59824, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78959828, SUC_lexicon_tags_11 },
    { 0xda759829, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3695982c, SUC_lexicon_tags_11 },
    { 0x74b5982c, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x67d5982f, SUC_lexicon_tags_7 },
    { 0x33559830, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab59833, SUC_lexicon_tags_37 },
    { 0x5bd59833, SUC_lexicon_tags_11 },
    { 0x5c159835, SUC_lexicon_tags_17 },
    { 0x81159836, SUC_lexicon_tags_17 },
    { 0xfe559836, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2435983f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3cd59841, SUC_lexicon_tags_12 },
    { 0xdab59841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f59846, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5984a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea959851, SUC_lexicon_tags_11 },
    { 0x98359852, SUC_lexicon_tags_1 },
    { 0x9e559851, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x53959856, SUC_lexicon_tags_7 },
    { 0xa5d59857, SUC_lexicon_tags_66 },
    { 0xa3759856, SUC_lexicon_tags_47 },
    { 0x88f59859, SUC_lexicon_tags_25 },
    { 0xed95985a, SUC_lexicon_tags_7 },
    { 0xb5b5985b, SUC_lexicon_tags_38 },
    { 0x93f59856, SUC_lexicon_tags_1 },
    { 0xa115985d, SUC_lexicon_tags_31 },
    { 0x3d95985e, SUC_lexicon_tags_17 },
    { 0x3815985d, SUC_lexicon_tags_0 },
    { 0x7cd59857, SUC_lexicon_tags_11 },
    { 0x1e359861, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d59866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3775986a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5215986e, SUC_lexicon_tags_19 },
    { 0xd8d5986f, SUC_lexicon_tags_7 },
    { 0xa55986f, SUC_lexicon_tags_16 },
    { 0x50359871, SUC_lexicon_tags_37 },
    { 0x3035986e, SUC_lexicon_tags_63 },
    { 0x8c15986f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f59876, SUC_lexicon_tags_30 },
    { 0x1c759876, SUC_lexicon_tags_63 },
    { 0x64159878, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc395987e, SUC_lexicon_tags_14 },
    { 0xad35987f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad59884, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb59888, SUC_lexicon_tags_12 },
    { 0xdf59889, SUC_lexicon_tags_23 },
    { 0x7675988a, SUC_lexicon_tags_100 },
    { 0x53d5988b, SUC_lexicon_tags_17 },
    { 0x16d5988b, SUC_lexicon_tags_28 },
    { 0x3e15988d, SUC_lexicon_tags_26 },
    { 0x9db5988b, SUC_lexicon_tags_10 },
    { 0x4c75988f, SUC_lexicon_tags_68 },
    { 0xb6b59890, SUC_lexicon_tags_79 },
    { 0x90f59891, SUC_lexicon_tags_343 },
    { 0x48d59888, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4559896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf5989a, SUC_lexicon_tags_2 },
    { 0x5995989b, SUC_lexicon_tags_11 },
    { 0x32f5989a, SUC_lexicon_tags_40 },
    { 0x4d15989c, SUC_lexicon_tags_12 },
    { 0xe9d5989a, SUC_lexicon_tags_38 },
    { 0xfcb5989d, SUC_lexicon_tags_7 },
    { 0xb87598a0, SUC_lexicon_tags_47 },
    { 0xfc5598a0, SUC_lexicon_tags_12 },
    { 0x695598a0, SUC_lexicon_tags_17 },
    { 0xbc7598a3, SUC_lexicon_tags_29 },
    { 0x511598a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61598b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1d598b2, SUC_lexicon_tags_39 },
    { 0x85f598b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d598b7, SUC_lexicon_tags_63 },
    { 0xb5d598b8, SUC_lexicon_tags_17 },
    { 0x827598b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcbd598bb, SUC_lexicon_tags_17 },
    { 0x45d598bc, SUC_lexicon_tags_9 },
    { 0xce9598bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f598c0, SUC_lexicon_tags_19 },
    { 0x41f598c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b598c4, SUC_lexicon_tags_11 },
    { 0xc55598c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef5598c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadf598c9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01598ce, SUC_lexicon_tags_37 },
    { 0x89d598ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5598d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff598d5, SUC_lexicon_tags_19 },
    { 0x969598d5, SUC_lexicon_tags_71 },
    { 0xfa1598d7, SUC_lexicon_tags_0 },
    { 0xe0d598d8, SUC_lexicon_tags_17 },
    { 0x8f9598d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x737598dc, SUC_lexicon_tags_47 },
    { 0x497598dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdb9598df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd9598e1, SUC_lexicon_tags_37 },
    { 0xe3598e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x89598e5, SUC_lexicon_tags_12 },
    { 0x119598e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41598ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d598ef, SUC_lexicon_tags_780 },
    { 0xbef598ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5598f5, SUC_lexicon_tags_65 },
    { 0xb2b598f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57598fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39359901, SUC_lexicon_tags_23 },
    { 0x66359902, SUC_lexicon_tags_37 },
    { 0x47959903, SUC_lexicon_tags_6 },
    { 0x7159904, SUC_lexicon_tags_14 },
    { 0x41159902, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30359907, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe3159909, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5975990e, SUC_lexicon_tags_234 },
    { 0xf4f5990f, SUC_lexicon_tags_341 },
    { 0x6e35990f, SUC_lexicon_tags_58 },
    { 0x16b5990e, SUC_lexicon_tags_15 },
    { 0xce359912, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2559914, SUC_lexicon_tags_37 },
    { 0x2f159914, SUC_lexicon_tags_15 },
    { 0x55359914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53559918, SUC_lexicon_tags_9 },
    { 0x22159919, SUC_lexicon_tags_7 },
    { 0xaa159918, SUC_lexicon_tags_1 },
    { 0x2075991b, SUC_lexicon_tags_11 },
    { 0x9f55991a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f5991f, SUC_lexicon_tags_11 },
    { 0xf5959920, SUC_lexicon_tags_23 },
    { 0x85959921, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf59926, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e55992b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9495992f, SUC_lexicon_tags_1 },
    { 0x8df59930, SUC_lexicon_tags_14 },
    { 0x27959931, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x69559933, SUC_lexicon_tags_25 },
    { 0x89159934, SUC_lexicon_tags_25 },
    { 0x54559933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81b59937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb959939, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa895993c, SUC_lexicon_tags_17 },
    { 0x7d5993d, SUC_lexicon_tags_7 },
    { 0x37d5993d, SUC_lexicon_tags_6 },
    { 0xac15993f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb59949, SUC_lexicon_tags_30 },
    { 0x1dd5994a, SUC_lexicon_tags_25 },
    { 0x7155994a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c15994e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d59951, SUC_lexicon_tags_43 },
    { 0x99959952, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65959959, SUC_lexicon_tags_6 },
    { 0x75f5995a, SUC_lexicon_tags_15 },
    { 0x92f5995b, SUC_lexicon_tags_28 },
    { 0xe255995c, SUC_lexicon_tags_15 },
    { 0x97959959, SUC_lexicon_tags_38 },
    { 0x5f75995e, SUC_lexicon_tags_4 },
    { 0x84f5995f, SUC_lexicon_tags_17 },
    { 0xb4759960, SUC_lexicon_tags_25 },
    { 0xb0d5995f, SUC_lexicon_tags_11 },
    { 0xb755995e, SUC_lexicon_tags_25 },
    { 0xf6b59963, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ab59965, SUC_lexicon_tags_0 },
    { 0x43f59966, SUC_lexicon_tags_14 },
    { 0xe4b59967, SUC_lexicon_tags_8 },
    { 0x74959966, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d5996c, SUC_lexicon_tags_63 },
    { 0xc6d5996d, SUC_lexicon_tags_16 },
    { 0x4d55996e, SUC_lexicon_tags_37 },
    { 0x6c55996d, SUC_lexicon_tags_11 },
    { 0x14559970, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x89159972, SUC_lexicon_tags_17 },
    { 0xd0f59972, SUC_lexicon_tags_47 },
    { 0x3f559974, SUC_lexicon_tags_19 },
    { 0x9ed59975, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8935997a, SUC_lexicon_tags_11 },
    { 0x73b5997b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad95998a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x55b5998c, SUC_lexicon_tags_0 },
    { 0xb9d5998d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd175998f, SUC_lexicon_tags_57 },
    { 0x8235998f, SUC_lexicon_tags_7 },
    { 0x71359991, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1159994, SUC_lexicon_tags_46 },
    { 0x6f759995, SUC_lexicon_tags_23 },
    { 0xab559996, SUC_lexicon_tags_29 },
    { 0x7c759995, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f5999c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd5999f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47599a5, SUC_lexicon_tags_12 },
    { 0xb85599a6, SUC_lexicon_tags_14 },
    { 0xb51599a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47f599a9, SUC_lexicon_tags_37 },
    { 0x411599aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d599b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a3599b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5599b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b599bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa43599bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261599c6, SUC_lexicon_tags_7 },
    { 0xebb599c7, SUC_lexicon_tags_45 },
    { 0x1f1599c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf599cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b599d2, SUC_lexicon_tags_28 },
    { 0x62d599d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57599d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c1599d8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd599dc, SUC_lexicon_tags_30 },
    { 0xa17599dc, SUC_lexicon_tags_7 },
    { 0xc85599dd, SUC_lexicon_tags_27 },
    { 0x461599dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x25599e2, SUC_lexicon_tags_15 },
    { 0xfcb599e3, SUC_lexicon_tags_28 },
    { 0x3fd599e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7599e7, SUC_lexicon_tags_0 },
    { 0x31599e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x191599eb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9599ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc21599f0, SUC_lexicon_tags_7 },
    { 0xecb599f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5599f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5599fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x30f599fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fd599fe, SUC_lexicon_tags_37 },
    { 0x8c7599ff, SUC_lexicon_tags_47 },
    { 0x8d159a00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa959a03, SUC_lexicon_tags_30 },
    { 0xf3559a03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d59a0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a559a16, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff59a21, SUC_lexicon_tags_14 },
    { 0x97d59a22, SUC_lexicon_tags_15 },
    { 0x87559a21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f59a29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b59a2c, SUC_lexicon_tags_2 },
    { 0xc2359a2d, SUC_lexicon_tags_12 },
    { 0x8a959a2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d59a30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76559a33, SUC_lexicon_tags_25 },
    { 0xa3f59a34, SUC_lexicon_tags_7 },
    { 0x7959a35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d59a39, SUC_lexicon_tags_14 },
    { 0x83559a3a, SUC_lexicon_tags_15 },
    { 0x97f59a3b, SUC_lexicon_tags_1 },
    { 0x6a559a3c, SUC_lexicon_tags_29 },
    { 0x8bf59a3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x53759a3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21959a41, SUC_lexicon_tags_541 },
    { 0x15b59a41, SUC_lexicon_tags_15 },
    { 0x3dd59a43, SUC_lexicon_tags_29 },
    { 0xe4559a43, SUC_lexicon_tags_2 },
    { 0x4bf59a45, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x37759a48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca359a4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf59a53, SUC_lexicon_tags_2 },
    { 0xd5359a53, SUC_lexicon_tags_14 },
    { 0x59d59a55, SUC_lexicon_tags_11 },
    { 0x20f59a56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x11b59a58, SUC_lexicon_tags_15 },
    { 0xfed59a58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72359a5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63b59a61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab59a64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x10d59a66, SUC_lexicon_tags_39 },
    { 0x80559a67, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xe3759a69, SUC_lexicon_tags_38 },
    { 0x59f59a69, SUC_lexicon_tags_17 },
    { 0x5d959a6a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d59a6f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b759a73, SUC_lexicon_tags_31 },
    { 0x88159a74, SUC_lexicon_tags_25 },
    { 0x10b59a74, SUC_lexicon_tags_12 },
    { 0xc1759a76, SUC_lexicon_tags_25 },
    { 0xbb959a74, SUC_lexicon_tags_25 },
    { 0x26359a77, SUC_lexicon_tags_1 },
    { 0xd6159a76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x90359a7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71159a80, SUC_lexicon_tags_9 },
    { 0x17f59a81, SUC_lexicon_tags_87 },
    { 0x67959a82, SUC_lexicon_tags_17 },
    { 0x8a559a83, SUC_lexicon_tags_2 },
    { 0xe0759a81, SUC_lexicon_tags_26 },
    { 0x1e359a84, SUC_lexicon_tags_12 },
    { 0xed59a80, SUC_lexicon_tags_12 },
    { 0x2a159a84, SUC_lexicon_tags_30 },
    { 0x63d59a86, SUC_lexicon_tags_37 },
    { 0xeaf59a82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5959a8c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af59a8f, SUC_lexicon_tags_37 },
    { 0xf6159a90, SUC_lexicon_tags_9 },
    { 0xc9f59a91, SUC_lexicon_tags_29 },
    { 0x51759a92, SUC_lexicon_tags_56 },
    { 0x21f59a92, SUC_lexicon_tags_12 },
    { 0x95159a94, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x29359a97, SUC_lexicon_tags_17 },
    { 0x8fb59a98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73959a9a, SUC_lexicon_tags_33 },
    { 0x6ff59a9b, SUC_lexicon_tags_12 },
    { 0x2c359a9c, SUC_lexicon_tags_25 },
    { 0x80959a9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d59aa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b59aa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda959aa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f59ab3, SUC_lexicon_tags_37 },
    { 0xde359ab4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf59ab7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x98159ab9, SUC_lexicon_tags_1 },
    { 0x18359aba, SUC_lexicon_tags_70 },
    { 0xc0159abb, SUC_lexicon_tags_75 },
    { 0x64b59abb, SUC_lexicon_tags_1 },
    { 0x98f59abd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f59ac5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcaf59ac7, SUC_lexicon_tags_55 },
    { 0x3c559ac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47159aca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d59acd, SUC_lexicon_tags_114 },
    { 0x10359ace, SUC_lexicon_tags_1 },
    { 0xf2f59ace, SUC_lexicon_tags_9 },
    { 0x68959acf, SUC_lexicon_tags_31 },
    { 0xe0559ad1, SUC_lexicon_tags_1 },
    { 0x9159ad2, SUC_lexicon_tags_17 },
    { 0xb3759ad2, SUC_lexicon_tags_37 },
    { 0xcab59acf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6959ad6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed59ada, SUC_lexicon_tags_4 },
    { 0xe7759adb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d59ae1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xba559ae3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b59ae7, SUC_lexicon_tags_16 },
    { 0xd8559ae8, SUC_lexicon_tags_31 },
    { 0x6a559ae9, SUC_lexicon_tags_12 },
    { 0x16d59ae8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f59af3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcad59af5, SUC_lexicon_tags_9 },
    { 0xfbb59af5, SUC_lexicon_tags_47 },
    { 0x88359af7, SUC_lexicon_tags_12 },
    { 0xf1759af8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d59aff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b59b02, SUC_lexicon_tags_42 },
    { 0x18559b03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3759b05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f759b07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d59b0e, SUC_lexicon_tags_25 },
    { 0x58359b0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93359b11, SUC_lexicon_tags_12 },
    { 0x3e759b12, SUC_lexicon_tags_37 },
    { 0xafb59b13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54159b1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x30959b1c, SUC_lexicon_tags_23 },
    { 0xffd59b1d, SUC_lexicon_tags_29 },
    { 0x71d59b1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff59b21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba159b2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0959b2d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x13359b2f, SUC_lexicon_tags_22 },
    { 0x8cd59b30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83759b34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb59b3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75759b41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66959b46, SUC_lexicon_tags_27 },
    { 0x2ad59b47, SUC_lexicon_tags_34 },
    { 0xc2b59b48, SUC_lexicon_tags_25 },
    { 0x6db59b49, SUC_lexicon_tags_55 },
    { 0xdc359b49, SUC_lexicon_tags_7 },
    { 0xd2f59b47, SUC_lexicon_tags_29 },
    { 0x79d59b4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95959b50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c759b57, SUC_lexicon_tags_17 },
    { 0xb4b59b57, SUC_lexicon_tags_108 },
    { 0xc8359b59, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x82d59b5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f59b63, SUC_lexicon_tags_30 },
    { 0x20559b63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d359b68, SUC_lexicon_tags_68 },
    { 0x40d59b68, SUC_lexicon_tags_17 },
    { 0x48759b68, SUC_lexicon_tags_26 },
    { 0x32f59b68, SUC_lexicon_tags_15 },
    { 0xa5d59b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dd59b6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8959b72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb759b75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d59b78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d59b7b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f59b7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18559b82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9d59b84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b359b86, SUC_lexicon_tags_61 },
    { 0xf8b59b87, SUC_lexicon_tags_17 },
    { 0xe3959b88, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe1959b8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x16b59b8c, SUC_lexicon_tags_17 },
    { 0x90359b8c, SUC_lexicon_tags_1 },
    { 0xecd59b8c, SUC_lexicon_tags_11 },
    { 0x62959b8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24959b93, SUC_lexicon_tags_7 },
    { 0xadb59b94, SUC_lexicon_tags_17 },
    { 0xc7d59b95, SUC_lexicon_tags_12 },
    { 0x9f559b95, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b59b99, SUC_lexicon_tags_2 },
    { 0xf5959b9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77759b9d, SUC_lexicon_tags_1 },
    { 0xe0959b9e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d59ba1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xef559ba3, SUC_lexicon_tags_37 },
    { 0x1d359ba3, SUC_lexicon_tags_178 },
    { 0x3c959ba5, SUC_lexicon_tags_23 },
    { 0x34f59ba5, SUC_lexicon_tags_6 },
    { 0x86b59ba5, SUC_lexicon_tags_13 },
    { 0xc9f59ba8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78359bac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb759bb2, SUC_lexicon_tags_17 },
    { 0x2d59bb3, SUC_lexicon_tags_25 },
    { 0x88d59bb4, SUC_lexicon_tags_38 },
    { 0xbf559bb2, SUC_lexicon_tags_8 },
    { 0x29759bb4, SUC_lexicon_tags_58 },
    { 0x6c159bb5, SUC_lexicon_tags_12 },
    { 0x1db59bb8, SUC_lexicon_tags_30 },
    { 0x8c959bb9, SUC_lexicon_tags_17 },
    { 0x5bd59bb8, SUC_lexicon_tags_11 },
    { 0x1e559bb3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6159bbe, SUC_lexicon_tags_37 },
    { 0x26b59bbe, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x4b159bc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27159bc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a759bc7, SUC_lexicon_tags_15 },
    { 0xbc559bc8, SUC_lexicon_tags_1 },
    { 0xd7759bc7, SUC_lexicon_tags_30 },
    { 0x24b59bc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8759bcd, SUC_lexicon_tags_66 },
    { 0xc3d59bcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6959bd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa7159bd5, SUC_lexicon_tags_14 },
    { 0x2ab59bd5, SUC_lexicon_tags_11 },
    { 0xf8559bd7, SUC_lexicon_tags_29 },
    { 0xeb759bd7, SUC_lexicon_tags_12 },
    { 0x49959bd9, SUC_lexicon_tags_1 },
    { 0xeed59bd9, SUC_lexicon_tags_138 },
    { 0x28f59bd9, SUC_lexicon_tags_61 },
    { 0x4b559bdc, SUC_lexicon_tags_61 },
    { 0x6c559bdd, SUC_lexicon_tags_25 },
    { 0x75d59bde, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5759be1, SUC_lexicon_tags_17 },
    { 0x3159be2, SUC_lexicon_tags_13 },
    { 0x83559be2, SUC_lexicon_tags_15 },
    { 0x60759be3, SUC_lexicon_tags_11 },
    { 0x87159be5, SUC_lexicon_tags_45 },
    { 0x2ed59be6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4959be8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559bec, SUC_lexicon_tags_26 },
    { 0x95359bed, SUC_lexicon_tags_19 },
    { 0xf7f59bed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8959bf5, SUC_lexicon_tags_11 },
    { 0xa5359bf6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb359bfd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeb959bff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x66359c01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb159c04, SUC_lexicon_tags_8 },
    { 0xf5559c04, SUC_lexicon_tags_45 },
    { 0xef559c05, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xed59c09, SUC_lexicon_tags_37 },
    { 0xf8d59c0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b59c0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed59c10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6159c16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd59c1d, SUC_lexicon_tags_27 },
    { 0xe4f59c1d, SUC_lexicon_tags_30 },
    { 0x8159c1f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x41159c22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50f59c24, SUC_lexicon_tags_65 },
    { 0xd6359c25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb959c28, SUC_lexicon_tags_15 },
    { 0x19359c29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b59c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f59c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf59c33, SUC_lexicon_tags_8 },
    { 0x56959c34, SUC_lexicon_tags_12 },
    { 0xb6f59c35, SUC_lexicon_tags_4 },
    { 0x4a559c36, SUC_lexicon_tags_15 },
    { 0x49f59c36, SUC_lexicon_tags_60 },
    { 0x4d59c36, SUC_lexicon_tags_19 },
    { 0x23559c39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15359c46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1159c48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8759c4b, SUC_lexicon_tags_39 },
    { 0x71f59c4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85759c4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe359c52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x14559c54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x93759c57, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b59c60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbe559c62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d59c65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59159c69, SUC_lexicon_tags_15 },
    { 0x2df59c6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b59c6e, SUC_lexicon_tags_37 },
    { 0xbb159c6f, SUC_lexicon_tags_15 },
    { 0x4759c6f, SUC_lexicon_tags_28 },
    { 0x14759c71, SUC_lexicon_tags_17 },
    { 0x5f359c71, SUC_lexicon_tags_17 },
    { 0x1d59c71, SUC_lexicon_tags_65 },
    { 0xd2959c74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1759c79, SUC_lexicon_tags_63 },
    { 0xa7b59c7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d559c7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99359c80, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b59c83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3159c8a, SUC_lexicon_tags_37 },
    { 0x84559c8b, SUC_lexicon_tags_255 },
    { 0xf1959c8b, SUC_lexicon_tags_7 },
    { 0x77d59c8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62d59c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee759c93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2eb59c95, SUC_lexicon_tags_17 },
    { 0x8b759c96, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d59c9a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d59c9e, SUC_lexicon_tags_17 },
    { 0x66959c9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32759ca4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d59caf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b59cbe, SUC_lexicon_tags_25 },
    { 0xb9759cbf, SUC_lexicon_tags_14 },
    { 0xfd359cbf, SUC_lexicon_tags_37 },
    { 0xd7559cc1, SUC_lexicon_tags_39 },
    { 0xf4d59cc2, SUC_lexicon_tags_11 },
    { 0x5f559cc3, SUC_lexicon_tags_11 },
    { 0x4d59cc4, SUC_lexicon_tags_31 },
    { 0x75b59cc3, SUC_lexicon_tags_27 },
    { 0xb5d59cc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac559cc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97759ccf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1359cd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24159cd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a559ce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb59ce5, SUC_lexicon_tags_17 },
    { 0x89759ce6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f159ce9, SUC_lexicon_tags_30 },
    { 0x4a759ce9, SUC_lexicon_tags_8 },
    { 0xf6359ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f959ced, SUC_lexicon_tags_34 },
    { 0x19f59cee, SUC_lexicon_tags_1 },
    { 0x98359cef, SUC_lexicon_tags_87 },
    { 0xbaf59cf0, SUC_lexicon_tags_60 },
    { 0xbc159cf1, SUC_lexicon_tags_15 },
    { 0x2f59cef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e759cf4, SUC_lexicon_tags_19 },
    { 0xc959cf5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26b59cf7, SUC_lexicon_tags_25 },
    { 0xc5d59cf8, SUC_lexicon_tags_17 },
    { 0x35759cf9, SUC_lexicon_tags_148 },
    { 0xd3d59cfa, SUC_lexicon_tags_61 },
    { 0x9c359cfa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfb559cfd, SUC_lexicon_tags_25 },
    { 0x59d59cfe, SUC_lexicon_tags_1 },
    { 0xfbf59cff, SUC_lexicon_tags_17 },
    { 0x95759cff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13759d03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad59d0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2b59d0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68d59d0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d59d12, SUC_lexicon_tags_31 },
    { 0xb3f59d12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d159d15, SUC_lexicon_tags_1 },
    { 0x6a159d15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x39759d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1959d1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18759d1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d59d22, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f59d28, SUC_lexicon_tags_1 },
    { 0x5fb59d29, SUC_lexicon_tags_29 },
    { 0x53d59d2a, SUC_lexicon_tags_34 },
    { 0x54559d2b, SUC_lexicon_tags_14 },
    { 0x77959d2c, SUC_lexicon_tags_29 },
    { 0x23159d2d, SUC_lexicon_tags_2 },
    { 0x88959d2d, SUC_lexicon_tags_9 },
    { 0xba959d2f, SUC_lexicon_tags_60 },
    { 0xf1b59d30, SUC_lexicon_tags_57 },
    { 0x21359d28, SUC_lexicon_tags_17 },
    { 0x8db59d32, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef59d37, SUC_lexicon_tags_29 },
    { 0x66159d38, SUC_lexicon_tags_49 },
    { 0xb9959d38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8759d3c, SUC_lexicon_tags_47 },
    { 0xaaf59d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb59d40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85959d45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b559d50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1b59d52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57159d55, SUC_lexicon_tags_32 },
    { 0xbc759d56, SUC_lexicon_tags_1 },
    { 0x65759d57, SUC_lexicon_tags_12 },
    { 0xc8559d56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfa759d5a, SUC_lexicon_tags_138 },
    { 0xe5359d5a, SUC_lexicon_tags_1 },
    { 0xe8159d5c, SUC_lexicon_tags_7 },
    { 0x82b59d5d, SUC_lexicon_tags_27 },
    { 0x38359d5c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x7f959d60, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8559d65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5759d68, SUC_lexicon_tags_11 },
    { 0x3b559d69, SUC_lexicon_tags_17 },
    { 0xc1b59d6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a959d76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8159d7f, SUC_lexicon_tags_35 },
    { 0x6ed59d7f, SUC_lexicon_tags_25 },
    { 0xe1b59d7f, SUC_lexicon_tags_1 },
    { 0x16f59d7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af59d85, SUC_lexicon_tags_25 },
    { 0x78959d85, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8959d88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0359d8a, SUC_lexicon_tags_14 },
    { 0x34359d8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63359d8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4359d96, SUC_lexicon_tags_37 },
    { 0xb0759d96, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3559d9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd59d9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d59da2, SUC_lexicon_tags_2 },
    { 0xc559da3, SUC_lexicon_tags_17 },
    { 0x47359da4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5359da6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c959dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f59dae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x14359db0, SUC_lexicon_tags_37 },
    { 0x78d59db1, SUC_lexicon_tags_15 },
    { 0x62559db2, SUC_lexicon_tags_25 },
    { 0x68359db2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab59db7, SUC_lexicon_tags_468 },
    { 0x2f559db8, SUC_lexicon_tags_33 },
    { 0x4a359db8, SUC_lexicon_tags_37 },
    { 0x38759dba, SUC_lexicon_tags_30 },
    { 0xa8359db8, SUC_lexicon_tags_42 },
    { 0x35959dbc, SUC_lexicon_tags_136 },
    { 0x28759dbc, SUC_lexicon_tags_0 },
    { 0x51d59dbe, SUC_lexicon_tags_66 },
    { 0x18f59dbd, SUC_lexicon_tags_65 },
    { 0x95d59dc0, SUC_lexicon_tags_15 },
    { 0xbd359dc1, SUC_lexicon_tags_11 },
    { 0xa0d59dbf, SUC_lexicon_tags_0 },
    { 0xe4b59dc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66359dcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5959dce, SUC_lexicon_tags_15 },
    { 0xb2b59dcf, SUC_lexicon_tags_17 },
    { 0xc5359dcf, SUC_lexicon_tags_65 },
    { 0x1fd59dd1, SUC_lexicon_tags_47 },
    { 0x7559dd1, SUC_lexicon_tags_10 },
    { 0xed959dd3, SUC_lexicon_tags_17 },
    { 0x59d59dd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98759dd6, SUC_lexicon_tags_2 },
    { 0xbf759dd6, SUC_lexicon_tags_11 },
    { 0xa759dd8, SUC_lexicon_tags_106 },
    { 0x61759dd7, SUC_lexicon_tags_15 },
    { 0xb6b59dda, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xcdf59ddc, SUC_lexicon_tags_2 },
    { 0xa7b59ddd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f59de0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92759de4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x46959de6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9d359de8, SUC_lexicon_tags_14 },
    { 0xfd959de8, SUC_lexicon_tags_15 },
    { 0x59d59de8, SUC_lexicon_tags_31 },
    { 0x71559dea, SUC_lexicon_tags_37 },
    { 0xa5359dec, SUC_lexicon_tags_57 },
    { 0xb7d59ded, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd4359def, SUC_lexicon_tags_16 },
    { 0xed59df0, SUC_lexicon_tags_11 },
    { 0x59959df1, SUC_lexicon_tags_12 },
    { 0xf959def, SUC_lexicon_tags_15 },
    { 0xecd59df3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f59df6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b159dfa, SUC_lexicon_tags_1 },
    { 0x24559dfa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8b559dfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17159e01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8559e03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3559e07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f59e0a, SUC_lexicon_tags_45 },
    { 0x93159e0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c559e0e, SUC_lexicon_tags_69 },
    { 0x1a359e0f, SUC_lexicon_tags_15 },
    { 0x3c159e10, SUC_lexicon_tags_37 },
    { 0x98159e11, SUC_lexicon_tags_26 },
    { 0x9dd59e12, SUC_lexicon_tags_43 },
    { 0xa9159e0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd59e18, SUC_lexicon_tags_6 },
    { 0xe0d59e19, SUC_lexicon_tags_77 },
    { 0xb5f59e1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf359e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb59e2f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2359e32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53759e38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf959e3a, SUC_lexicon_tags_31 },
    { 0xc5f59e3a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3759e42, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f59e46, SUC_lexicon_tags_142 },
    { 0x51959e47, SUC_lexicon_tags_45 },
    { 0xb7b59e48, SUC_lexicon_tags_46 },
    { 0xa3f59e48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf59e4e, SUC_lexicon_tags_14 },
    { 0x87d59e4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1959e52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed59e58, SUC_lexicon_tags_26 },
    { 0x92b59e59, SUC_lexicon_tags_17 },
    { 0xe3959e59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c159e5d, SUC_lexicon_tags_37 },
    { 0x23359e5d, SUC_lexicon_tags_2 },
    { 0x51159e5e, SUC_lexicon_tags_32 },
    { 0x1c959e5f, SUC_lexicon_tags_14 },
    { 0xd3559e61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x41b59e63, SUC_lexicon_tags_35 },
    { 0x4959e63, SUC_lexicon_tags_44 },
    { 0x73359e64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41359e6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13559e6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f359e70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f59e75, SUC_lexicon_tags_17 },
    { 0xbe959e76, SUC_lexicon_tags_59 },
    { 0x5e759e75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f759e81, SUC_lexicon_tags_11 },
    { 0xc5359e82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60559e93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab359e96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82359e98, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaf959e9a, SUC_lexicon_tags_11 },
    { 0x9f759e9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2159e9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f59ea3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x91759ea5, SUC_lexicon_tags_0 },
    { 0x4cb59ea5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x43559ea9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26759eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92b59ead, SUC_lexicon_tags_37 },
    { 0xf9759eae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d59eb6, SUC_lexicon_tags_38 },
    { 0x28159eb6, SUC_lexicon_tags_56 },
    { 0xe4f59eb8, SUC_lexicon_tags_8 },
    { 0xd6b59eb7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1359ebb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb59ec0, SUC_lexicon_tags_9 },
    { 0xe9f59ec1, SUC_lexicon_tags_2 },
    { 0xa5359ec2, SUC_lexicon_tags_2 },
    { 0x31759ec3, SUC_lexicon_tags_15 },
    { 0xca559ec1, SUC_lexicon_tags_26 },
    { 0x59359ec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27559ec7, SUC_lexicon_tags_17 },
    { 0xd159ec8, SUC_lexicon_tags_46 },
    { 0x1cd59ec9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaed59ecb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab59ece, SUC_lexicon_tags_37 },
    { 0x92d59ecf, SUC_lexicon_tags_37 },
    { 0x2cf59ecf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccd59ed2, SUC_lexicon_tags_43 },
    { 0xbf959ed2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d59ed6, SUC_lexicon_tags_8 },
    { 0x5d559ed7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81d59ed9, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x99159edb, SUC_lexicon_tags_37 },
    { 0x7a359edc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4359edf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77559ee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82159ee4, SUC_lexicon_tags_38 },
    { 0x32159ee5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78959eef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c759ef1, SUC_lexicon_tags_761 },
    { 0x4f559ef2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5f59ef4, SUC_lexicon_tags_33 },
    { 0x4e759ef5, SUC_lexicon_tags_27 },
    { 0x94d59ef5, SUC_lexicon_tags_14 },
    { 0xdb559ef7, SUC_lexicon_tags_63 },
    { 0xe3559ef6, SUC_lexicon_tags_45 },
    { 0xb3159ef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70559efd, SUC_lexicon_tags_11 },
    { 0xfaf59efe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db59f01, SUC_lexicon_tags_1 },
    { 0x8c159f02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x75d59f04, SUC_lexicon_tags_11 },
    { 0xefb59f05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76759f09, SUC_lexicon_tags_25 },
    { 0x37d59f0a, SUC_lexicon_tags_2 },
    { 0xc4759f0b, SUC_lexicon_tags_49 },
    { 0xa2759f0c, SUC_lexicon_tags_47 },
    { 0x8a959f0a, SUC_lexicon_tags_27 },
    { 0xb1959f0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1b59f10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x26f59f12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2159f15, SUC_lexicon_tags_2 },
    { 0x60b59f16, SUC_lexicon_tags_4 },
    { 0x80b59f16, SUC_lexicon_tags_15 },
    { 0xf6d59f18, SUC_lexicon_tags_2 },
    { 0xe0f59f19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5159f1b, SUC_lexicon_tags_11 },
    { 0x57d59f1c, SUC_lexicon_tags_46 },
    { 0xa4359f1d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0559f26, SUC_lexicon_tags_23 },
    { 0x4f159f26, SUC_lexicon_tags_12 },
    { 0xf5f59f28, SUC_lexicon_tags_15 },
    { 0x66559f28, SUC_lexicon_tags_2 },
    { 0xe2159f2a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb3159f2c, SUC_lexicon_tags_25 },
    { 0x88359f2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9f59f2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab59f35, SUC_lexicon_tags_7 },
    { 0x8a959f36, SUC_lexicon_tags_79 },
    { 0xee559f37, SUC_lexicon_tags_26 },
    { 0xdff59f36, SUC_lexicon_tags_8 },
    { 0xcfd59f39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77b59f3b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdaf59f3d, SUC_lexicon_tags_103 },
    { 0x81f59f3e, SUC_lexicon_tags_45 },
    { 0xf8f59f3e, SUC_lexicon_tags_2 },
    { 0x93159f3f, SUC_lexicon_tags_11 },
    { 0xa9559f41, SUC_lexicon_tags_17 },
    { 0x47d59f41, SUC_lexicon_tags_68 },
    { 0xfd759f43, SUC_lexicon_tags_9 },
    { 0x2e959f3e, SUC_lexicon_tags_170 },
    { 0x58d59f45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b59f4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c559f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72d59f51, SUC_lexicon_tags_17 },
    { 0xcc759f51, SUC_lexicon_tags_29 },
    { 0xc0b59f51, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xde759f56, SUC_lexicon_tags_24 },
    { 0x76559f56, SUC_lexicon_tags_10 },
    { 0x9d959f56, SUC_lexicon_tags_127 },
    { 0x6e359f57, SUC_lexicon_tags_37 },
    { 0xf8159f58, SUC_lexicon_tags_23 },
    { 0x51559f5b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1959f61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f959f63, SUC_lexicon_tags_47 },
    { 0x27159f63, SUC_lexicon_tags_15 },
    { 0xdc559f64, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae359f6a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f559f73, SUC_lexicon_tags_12 },
    { 0xc5359f74, SUC_lexicon_tags_25 },
    { 0x74159f75, SUC_lexicon_tags_25 },
    { 0xddd59f76, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0559f7a, SUC_lexicon_tags_37 },
    { 0x41559f7b, SUC_lexicon_tags_47 },
    { 0xfa159f7c, SUC_lexicon_tags_1 },
    { 0x2b959f7c, SUC_lexicon_tags_15 },
    { 0xe2559f7c, SUC_lexicon_tags_6 },
    { 0x3f559f7f, SUC_lexicon_tags_38 },
    { 0xcb59f7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b759f82, SUC_lexicon_tags_17 },
    { 0x73759f83, SUC_lexicon_tags_23 },
    { 0xabb59f82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32959f86, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d359f8a, SUC_lexicon_tags_2 },
    { 0xbbd59f8b, SUC_lexicon_tags_9 },
    { 0xa4159f8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x75d59f8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51959f90, SUC_lexicon_tags_43 },
    { 0xbe959f91, SUC_lexicon_tags_49 },
    { 0xdc759f92, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x97959f94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe159f97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c359fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeff59fa3, SUC_lexicon_tags_64 },
    { 0x29959fa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54559fa6, SUC_lexicon_tags_79 },
    { 0x4159fa7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a359faa, SUC_lexicon_tags_2 },
    { 0x1ad59faa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b959faf, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x33959fb1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7559fb3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48559fb5, SUC_lexicon_tags_47 },
    { 0xd4359fb6, SUC_lexicon_tags_49 },
    { 0xaf559fb6, SUC_lexicon_tags_16 },
    { 0xa5159fb8, SUC_lexicon_tags_37 },
    { 0x2f159fb8, SUC_lexicon_tags_47 },
    { 0x76759fba, SUC_lexicon_tags_37 },
    { 0xeef59fb6, SUC_lexicon_tags_63 },
    { 0x6ab59fbc, SUC_lexicon_tags_25 },
    { 0xb2159fbd, SUC_lexicon_tags_17 },
    { 0xf7f59fbe, SUC_lexicon_tags_12 },
    { 0x85159fbf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7359fc2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaf959fc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e959fc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x19959fca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf759fcf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f59fd2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x34159fd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad59fd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f359fda, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f59fe0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76759fe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48159fec, SUC_lexicon_tags_37 },
    { 0x16b59fed, SUC_lexicon_tags_17 },
    { 0x93559fed, SUC_lexicon_tags_7 },
    { 0xe8b59fee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9bf59ff1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90559ff8, SUC_lexicon_tags_207 },
    { 0xb359ff9, SUC_lexicon_tags_25 },
    { 0x24959ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9759ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33359ffe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55a004, SUC_lexicon_tags_14 },
    { 0x2175a005, SUC_lexicon_tags_61 },
    { 0x235a004, SUC_lexicon_tags_15 },
    { 0x9975a005, SUC_lexicon_tags_26 },
    { 0x2d95a008, SUC_lexicon_tags_27 },
    { 0x7515a008, SUC_lexicon_tags_14 },
    { 0x2975a008, SUC_lexicon_tags_11 },
    { 0xc75a00b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b15a00e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f35a018, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb5a01b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b15a01f, SUC_lexicon_tags_10 },
    { 0x2cf5a01f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea75a024, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b35a029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf35a02b, SUC_lexicon_tags_15 },
    { 0x7f35a02b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b75a035, SUC_lexicon_tags_37 },
    { 0x9b15a036, SUC_lexicon_tags_37 },
    { 0x9b15a037, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d95a04b, SUC_lexicon_tags_0 },
    { 0x2fb5a04c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd815a04e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6675a058, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2435a05b, SUC_lexicon_tags_40 },
    { 0xb3b5a05c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbcf5a05e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xd395a061, SUC_lexicon_tags_33 },
    { 0x5835a062, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5ad5a064, SUC_lexicon_tags_34 },
    { 0x5cd5a064, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3815a068, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b5a077, SUC_lexicon_tags_116 },
    { 0x32f5a078, SUC_lexicon_tags_12 },
    { 0xe715a078, SUC_lexicon_tags_46 },
    { 0x7d75a077, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8255a07d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x8a75a07f, SUC_lexicon_tags_27 },
    { 0xc715a080, SUC_lexicon_tags_37 },
    { 0x35f5a081, SUC_lexicon_tags_25 },
    { 0x1695a082, SUC_lexicon_tags_19 },
    { 0x3a75a083, SUC_lexicon_tags_19 },
    { 0xe035a084, SUC_lexicon_tags_11 },
    { 0x8075a084, SUC_lexicon_tags_25 },
    { 0xfaf5a086, SUC_lexicon_tags_12 },
    { 0x28d5a084, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd995a089, SUC_lexicon_tags_58 },
    { 0x1b55a08a, SUC_lexicon_tags_17 },
    { 0xb6f5a08a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b5a08f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f5a092, SUC_lexicon_tags_1 },
    { 0x8d75a092, SUC_lexicon_tags_17 },
    { 0xc5d5a094, SUC_lexicon_tags_11 },
    { 0x8795a094, SUC_lexicon_tags_37 },
    { 0x5015a096, SUC_lexicon_tags_9 },
    { 0x1f75a093, SUC_lexicon_tags_29 },
    { 0x6275a098, SUC_lexicon_tags_12 },
    { 0x2355a098, SUC_lexicon_tags_23 },
    { 0x5235a093, SUC_lexicon_tags_17 },
    { 0xbeb5a09b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b5a09f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b15a0a3, SUC_lexicon_tags_23 },
    { 0x7055a0a4, SUC_lexicon_tags_1 },
    { 0xdcd5a0a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd5a0a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75a0ab, SUC_lexicon_tags_25 },
    { 0xa035a0ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75a0af, SUC_lexicon_tags_37 },
    { 0xdc95a0b0, SUC_lexicon_tags_118 },
    { 0x1535a0b0, SUC_lexicon_tags_17 },
    { 0x2735a0b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b55a0b9, SUC_lexicon_tags_359 },
    { 0xe2b5a0ba, SUC_lexicon_tags_26 },
    { 0x6e35a0ba, SUC_lexicon_tags_23 },
    { 0x7035a0bc, SUC_lexicon_tags_17 },
    { 0xe055a0bd, SUC_lexicon_tags_14 },
    { 0xee95a0bc, SUC_lexicon_tags_11 },
    { 0xdd75a0b9, SUC_lexicon_tags_45 },
    { 0x99b5a0c0, SUC_lexicon_tags_12 },
    { 0x7335a0c1, SUC_lexicon_tags_29 },
    { 0x1755a0c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135a0c7, SUC_lexicon_tags_27 },
    { 0x8835a0c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb5a0cd, SUC_lexicon_tags_17 },
    { 0xc715a0cd, SUC_lexicon_tags_17 },
    { 0x3f75a0cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2095a0d3, SUC_lexicon_tags_64 },
    { 0xa95a0d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b5a0db, SUC_lexicon_tags_17 },
    { 0x3f15a0dc, SUC_lexicon_tags_30 },
    { 0x95b5a0dd, SUC_lexicon_tags_15 },
    { 0xc1f5a0db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255a0e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd935a0e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e95a0ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb95a0f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbd5a0f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e15a0fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8435a0ff, SUC_lexicon_tags_66 },
    { 0xc075a0ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5875a103, SUC_lexicon_tags_136 },
    { 0x9b5a104, SUC_lexicon_tags_1 },
    { 0x8a75a105, SUC_lexicon_tags_13 },
    { 0xcd15a103, SUC_lexicon_tags_21 },
    { 0x2df5a104, SUC_lexicon_tags_60 },
    { 0x9a95a107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8355a111, SUC_lexicon_tags_27 },
    { 0x8055a112, SUC_lexicon_tags_60 },
    { 0x8d55a113, SUC_lexicon_tags_27 },
    { 0x1bb5a113, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f55a117, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbeb5a119, SUC_lexicon_tags_7 },
    { 0xf6b5a11a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa815a11d, SUC_lexicon_tags_0 },
    { 0xa275a11e, SUC_lexicon_tags_302 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc975a124, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad15a129, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5a131, SUC_lexicon_tags_15 },
    { 0x5655a132, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc1b5a134, SUC_lexicon_tags_11 },
    { 0xc0f5a134, SUC_lexicon_tags_28 },
    { 0xcc35a135, SUC_lexicon_tags_23 },
    { 0x69d5a135, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f5a13a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2315a13c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d5a145, SUC_lexicon_tags_26 },
    { 0x6435a146, SUC_lexicon_tags_30 },
    { 0x7095a146, SUC_lexicon_tags_1 },
    { 0x3895a147, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd75a14a, SUC_lexicon_tags_6 },
    { 0x1c35a14b, SUC_lexicon_tags_63 },
    { 0xd975a14c, SUC_lexicon_tags_7 },
    { 0x1cf5a14c, SUC_lexicon_tags_7 },
    { 0x6555a14e, SUC_lexicon_tags_17 },
    { 0xe755a14f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc795a151, SUC_lexicon_tags_31 },
    { 0xca95a152, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb5a157, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x235a159, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee55a15b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d35a15e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab55a161, SUC_lexicon_tags_1 },
    { 0xb3b5a162, SUC_lexicon_tags_37 },
    { 0x1c15a162, SUC_lexicon_tags_27 },
    { 0x13b5a164, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1615a169, SUC_lexicon_tags_57 },
    { 0x6e95a16a, SUC_lexicon_tags_28 },
    { 0x4195a16b, SUC_lexicon_tags_31 },
    { 0x2815a16a, SUC_lexicon_tags_1 },
    { 0x5fb5a16d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4795a16f, SUC_lexicon_tags_12 },
    { 0xf055a170, SUC_lexicon_tags_11 },
    { 0x34f5a171, SUC_lexicon_tags_15 },
    { 0x5115a172, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f5a175, SUC_lexicon_tags_56 },
    { 0xe095a175, SUC_lexicon_tags_7 },
    { 0x8f55a177, SUC_lexicon_tags_82 },
    { 0x8a95a176, SUC_lexicon_tags_13 },
    { 0x8615a179, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5935a17b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7555a17d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a75a183, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d5a188, SUC_lexicon_tags_37 },
    { 0xdb5a189, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xedd5a18b, SUC_lexicon_tags_45 },
    { 0x615a18c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7755a194, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5ad5a196, SUC_lexicon_tags_27 },
    { 0x6315a196, SUC_lexicon_tags_17 },
    { 0x8815a198, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe695a19e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d35a1a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb5a1a6, SUC_lexicon_tags_14 },
    { 0x8e95a1a7, SUC_lexicon_tags_16 },
    { 0x6d15a1a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7475a1ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa995a1b4, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x7195a1b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1955a1bf, SUC_lexicon_tags_30 },
    { 0x2a55a1bf, SUC_lexicon_tags_762 },
    { 0xf335a1c1, SUC_lexicon_tags_13 },
    { 0x2af5a1c1, SUC_lexicon_tags_0 },
    { 0xe995a1c3, SUC_lexicon_tags_19 },
    { 0x5935a1c2, SUC_lexicon_tags_34 },
    { 0x49f5a1c5, SUC_lexicon_tags_25 },
    { 0xb935a1c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x875a1c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40f5a1cb, SUC_lexicon_tags_11 },
    { 0xe715a1cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x175a1cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6615a1d2, SUC_lexicon_tags_2 },
    { 0xc915a1d2, SUC_lexicon_tags_15 },
    { 0x6bb5a1d4, SUC_lexicon_tags_25 },
    { 0xc2d5a1d5, SUC_lexicon_tags_12 },
    { 0x9055a1d6, SUC_lexicon_tags_1 },
    { 0x44f5a1d7, SUC_lexicon_tags_17 },
    { 0xd875a1d7, SUC_lexicon_tags_33 },
    { 0x8855a1d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x915a1db, SUC_lexicon_tags_7 },
    { 0xed75a1dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc855a1de, SUC_lexicon_tags_43 },
    { 0xc7d5a1df, SUC_lexicon_tags_17 },
    { 0x85f5a1de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a35a1e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b5a1ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f5a1f0, SUC_lexicon_tags_37 },
    { 0x83b5a1f1, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xc535a1f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf5a1f8, SUC_lexicon_tags_15 },
    { 0x62d5a1f9, SUC_lexicon_tags_27 },
    { 0x48d5a1fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd5a1fd, SUC_lexicon_tags_25 },
    { 0xb3d5a1fd, SUC_lexicon_tags_37 },
    { 0xf2b5a1fe, SUC_lexicon_tags_15 },
    { 0x16f5a1fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa395a204, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6395a208, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd855a20b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc035a20d, SUC_lexicon_tags_23 },
    { 0x27d5a20e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3555a215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd875a21b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f95a222, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xde35a225, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4f5a227, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f5a22c, SUC_lexicon_tags_25 },
    { 0xe855a22d, SUC_lexicon_tags_25 },
    { 0xd475a22e, SUC_lexicon_tags_14 },
    { 0xa855a22e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd115a232, SUC_lexicon_tags_12 },
    { 0x9875a233, SUC_lexicon_tags_27 },
    { 0x94b5a234, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe455a239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8015a23f, SUC_lexicon_tags_84 },
    { 0x9f55a240, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e55a243, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc255a245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c35a24a, SUC_lexicon_tags_7 },
    { 0x95b5a24a, SUC_lexicon_tags_32 },
    { 0xff15a24c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab5a24e, SUC_lexicon_tags_43 },
    { 0x7535a24f, SUC_lexicon_tags_37 },
    { 0xb235a24e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5b5a252, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f75a255, SUC_lexicon_tags_10 },
    { 0xcf15a255, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d55a25a, SUC_lexicon_tags_47 },
    { 0x82f5a25a, SUC_lexicon_tags_17 },
    { 0xc0f5a25c, SUC_lexicon_tags_25 },
    { 0xdbf5a25d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9c55a25f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6055a261, SUC_lexicon_tags_17 },
    { 0xb815a262, SUC_lexicon_tags_60 },
    { 0xf0b5a263, SUC_lexicon_tags_56 },
    { 0x6bf5a262, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc75a26a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b55a26c, SUC_lexicon_tags_12 },
    { 0xd715a26d, SUC_lexicon_tags_28 },
    { 0x7195a26d, SUC_lexicon_tags_15 },
    { 0x4355a26c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6475a271, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b55a274, SUC_lexicon_tags_28 },
    { 0xbe35a275, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575a279, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc415a27c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e35a280, SUC_lexicon_tags_29 },
    { 0x275a280, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb5a283, SUC_lexicon_tags_39 },
    { 0xd955a284, SUC_lexicon_tags_48 },
    { 0x7475a284, SUC_lexicon_tags_37 },
    { 0xa875a285, SUC_lexicon_tags_33 },
    { 0xa75a286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1315a28c, SUC_lexicon_tags_38 },
    { 0x4d5a28d, SUC_lexicon_tags_23 },
    { 0x5435a28c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc55a291, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2d5a293, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6535a295, SUC_lexicon_tags_17 },
    { 0x8175a296, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3395a298, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2975a29e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b5a2a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24b5a2a3, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf5a2a6, SUC_lexicon_tags_2 },
    { 0x6135a2a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e55a2ac, SUC_lexicon_tags_12 },
    { 0x6935a2ad, SUC_lexicon_tags_47 },
    { 0xe735a2ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad5a2b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f5a2bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd415a2c2, SUC_lexicon_tags_19 },
    { 0x6675a2c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5a2c9, SUC_lexicon_tags_7 },
    { 0xabb5a2ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd5a2cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe55a2d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fd5a2d5, SUC_lexicon_tags_66 },
    { 0xd135a2d6, SUC_lexicon_tags_1 },
    { 0x9995a2d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9015a2d9, SUC_lexicon_tags_65 },
    { 0x8195a2da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8975a2de, SUC_lexicon_tags_66 },
    { 0xe355a2df, SUC_lexicon_tags_11 },
    { 0x7915a2e0, SUC_lexicon_tags_60 },
    { 0x49f5a2e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab75a2e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b15a2ed, SUC_lexicon_tags_15 },
    { 0x26f5a2ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5a2f2, SUC_lexicon_tags_15 },
    { 0x935a2f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f15a2f8, SUC_lexicon_tags_25 },
    { 0xf215a2f9, SUC_lexicon_tags_25 },
    { 0x15b5a2fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd595a2fc, SUC_lexicon_tags_47 },
    { 0xcf75a2fc, SUC_lexicon_tags_63 },
    { 0xa755a2fe, SUC_lexicon_tags_19 },
    { 0x2015a2fe, SUC_lexicon_tags_82 },
    { 0x5095a2fe, SUC_lexicon_tags_32 },
    { 0x21f5a301, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf835a305, SUC_lexicon_tags_8 },
    { 0x38d5a306, SUC_lexicon_tags_40 },
    { 0x41b5a305, SUC_lexicon_tags_37 },
    { 0xd415a308, SUC_lexicon_tags_82 },
    { 0xdad5a308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a95a30b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ed5a30d, SUC_lexicon_tags_25 },
    { 0xcbb5a30e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac35a313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15f5a315, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc55a317, SUC_lexicon_tags_7 },
    { 0x7a35a317, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c75a320, SUC_lexicon_tags_25 },
    { 0xdf55a320, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69f5a323, SUC_lexicon_tags_1 },
    { 0x6a55a323, SUC_lexicon_tags_7 },
    { 0xddd5a325, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf335a328, SUC_lexicon_tags_79 },
    { 0x5535a329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4615a32d, SUC_lexicon_tags_47 },
    { 0x9375a32e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5a332, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb35a338, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xfe35a33a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe735a33c, SUC_lexicon_tags_6 },
    { 0x4b95a33d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6435a33f, SUC_lexicon_tags_23 },
    { 0x1e55a340, SUC_lexicon_tags_12 },
    { 0x4415a33f, SUC_lexicon_tags_17 },
    { 0xca35a342, SUC_lexicon_tags_65 },
    { 0xb415a343, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe215a345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f5a348, SUC_lexicon_tags_1 },
    { 0x85b5a349, SUC_lexicon_tags_14 },
    { 0x2a75a34a, SUC_lexicon_tags_11 },
    { 0xc675a348, SUC_lexicon_tags_19 },
    { 0x4af5a34c, SUC_lexicon_tags_136 },
    { 0xd3b5a34b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9015a34f, SUC_lexicon_tags_60 },
    { 0x6bd5a350, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1eb5a352, SUC_lexicon_tags_17 },
    { 0xf7f5a352, SUC_lexicon_tags_27 },
    { 0xa0f5a353, SUC_lexicon_tags_23 },
    { 0x9c55a354, SUC_lexicon_tags_35 },
    { 0x395a356, SUC_lexicon_tags_9 },
    { 0x43b5a357, SUC_lexicon_tags_11 },
    { 0x6a35a358, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b5a35e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed5a361, SUC_lexicon_tags_38 },
    { 0xdd75a362, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf515a364, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15a367, SUC_lexicon_tags_27 },
    { 0xaf55a367, SUC_lexicon_tags_35 },
    { 0x2eb5a369, SUC_lexicon_tags_2 },
    { 0x9e15a36a, SUC_lexicon_tags_17 },
    { 0x8d95a369, SUC_lexicon_tags_16 },
    { 0x7615a36c, SUC_lexicon_tags_33 },
    { 0x38b5a36c, SUC_lexicon_tags_60 },
    { 0x9815a36d, SUC_lexicon_tags_1 },
    { 0x1e55a36f, SUC_lexicon_tags_9 },
    { 0x1e15a370, SUC_lexicon_tags_19 },
    { 0xa35a370, SUC_lexicon_tags_17 },
    { 0x7635a36e, SUC_lexicon_tags_2 },
    { 0xbd55a36b, SUC_lexicon_tags_25 },
    { 0xd015a374, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd355a378, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd4b5a37a, SUC_lexicon_tags_9 },
    { 0xd995a37b, SUC_lexicon_tags_45 },
    { 0x8d75a37c, SUC_lexicon_tags_31 },
    { 0x7255a37a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d5a381, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb95a385, SUC_lexicon_tags_2 },
    { 0xa2d5a386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c35a38a, SUC_lexicon_tags_15 },
    { 0xa895a38a, SUC_lexicon_tags_11 },
    { 0x1ab5a38c, SUC_lexicon_tags_11 },
    { 0xba15a38d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b5a393, SUC_lexicon_tags_8 },
    { 0x7f15a394, SUC_lexicon_tags_68 },
    { 0x65b5a395, SUC_lexicon_tags_6 },
    { 0x5675a396, SUC_lexicon_tags_11 },
    { 0xe7b5a397, SUC_lexicon_tags_45 },
    { 0x675a398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2515a3a2, SUC_lexicon_tags_43 },
    { 0x4f55a3a3, SUC_lexicon_tags_11 },
    { 0x8fd5a3a3, SUC_lexicon_tags_45 },
    { 0x1755a3a5, SUC_lexicon_tags_0 },
    { 0x1b35a3a2, SUC_lexicon_tags_7 },
    { 0x295a3a5, SUC_lexicon_tags_11 },
    { 0x2775a3a8, SUC_lexicon_tags_0 },
    { 0xb595a3a8, SUC_lexicon_tags_37 },
    { 0xc4b5a3aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e55a3b0, SUC_lexicon_tags_63 },
    { 0xa795a3b1, SUC_lexicon_tags_25 },
    { 0x62b5a3b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b5a3b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b5a3bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5015a3be, SUC_lexicon_tags_14 },
    { 0xb495a3bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a75a3c3, SUC_lexicon_tags_49 },
    { 0x3fb5a3c3, SUC_lexicon_tags_2 },
    { 0xfb5a3c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f5a3ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2695a3cd, SUC_lexicon_tags_46 },
    { 0x82d5a3ce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9435a3d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5115a3d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b5a3d9, SUC_lexicon_tags_44 },
    { 0x6615a3d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb695a3dc, SUC_lexicon_tags_2 },
    { 0xbef5a3dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9115a3df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1515a3e5, SUC_lexicon_tags_15 },
    { 0xa695a3e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc55a3e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5a3ec, SUC_lexicon_tags_25 },
    { 0x5275a3ed, SUC_lexicon_tags_11 },
    { 0xdbb5a3ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a75a3f0, SUC_lexicon_tags_15 },
    { 0xc675a3f1, SUC_lexicon_tags_15 },
    { 0x3d95a3f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd5a3f7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb55a3fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe175a402, SUC_lexicon_tags_7 },
    { 0xf8f5a403, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce15a405, SUC_lexicon_tags_10 },
    { 0x41d5a406, SUC_lexicon_tags_21 },
    { 0x6cb5a407, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5a40c, SUC_lexicon_tags_32 },
    { 0xffb5a40d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5a413, SUC_lexicon_tags_9 },
    { 0x9f15a413, SUC_lexicon_tags_15 },
    { 0x72d5a414, SUC_lexicon_tags_0 },
    { 0x39f5a416, SUC_lexicon_tags_15 },
    { 0xed5a417, SUC_lexicon_tags_0 },
    { 0x56d5a418, SUC_lexicon_tags_12 },
    { 0x7b95a419, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2715a426, SUC_lexicon_tags_11 },
    { 0x5915a427, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4295a42b, SUC_lexicon_tags_17 },
    { 0x9a35a42c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2e15a42e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5795a437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d35a43b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b5a440, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b5a445, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6615a448, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc35a44e, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b5a457, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f5a45b, SUC_lexicon_tags_124 },
    { 0x4fb5a45b, SUC_lexicon_tags_2 },
    { 0x2eb5a45c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed15a45f, SUC_lexicon_tags_43 },
    { 0x9595a460, SUC_lexicon_tags_17 },
    { 0xe3b5a461, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb5a46a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a75a46f, SUC_lexicon_tags_17 },
    { 0x4bb5a46f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd635a476, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20b5a478, SUC_lexicon_tags_2 },
    { 0xc675a479, SUC_lexicon_tags_1 },
    { 0x1e55a479, SUC_lexicon_tags_25 },
    { 0x63f5a479, SUC_lexicon_tags_16 },
    { 0xf5f5a47c, SUC_lexicon_tags_6 },
    { 0xd235a47d, SUC_lexicon_tags_3 },
    { 0x135a478, SUC_lexicon_tags_36 },
    { 0x1cd5a47f, SUC_lexicon_tags_7 },
    { 0xf95a480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f5a485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d35a487, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9475a489, SUC_lexicon_tags_4 },
    { 0x1f35a48a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4295a48c, SUC_lexicon_tags_37 },
    { 0x9db5a48c, SUC_lexicon_tags_51 },
    { 0xe15a48d, SUC_lexicon_tags_47 },
    { 0x23f5a48f, SUC_lexicon_tags_7 },
    { 0x4ed5a48f, SUC_lexicon_tags_6 },
    { 0xc775a491, SUC_lexicon_tags_47 },
    { 0x2795a48d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5a495, SUC_lexicon_tags_12 },
    { 0x9835a496, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd5a49c, SUC_lexicon_tags_11 },
    { 0x2f55a49d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a75a49f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b5a4a3, SUC_lexicon_tags_63 },
    { 0xc855a4a4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2175a4a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb135a4aa, SUC_lexicon_tags_47 },
    { 0xe715a4ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb1b5a4ad, SUC_lexicon_tags_17 },
    { 0x2255a4ae, SUC_lexicon_tags_49 },
    { 0x7795a4af, SUC_lexicon_tags_49 },
    { 0x10b5a4b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d5a4b4, SUC_lexicon_tags_15 },
    { 0x5b75a4b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed5a4b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7855a4be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f5a4c3, SUC_lexicon_tags_2 },
    { 0x375a4c3, SUC_lexicon_tags_62 },
    { 0xb9f5a4c4, SUC_lexicon_tags_15 },
    { 0xc6d5a4c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b5a4ca, SUC_lexicon_tags_25 },
    { 0xa515a4cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e15a4ce, SUC_lexicon_tags_60 },
    { 0x67f5a4ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed5a4d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ff5a4d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf5a4d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f5a4df, SUC_lexicon_tags_27 },
    { 0xe055a4df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df5a4e4, SUC_lexicon_tags_19 },
    { 0x3695a4e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ff5a4e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e75a4e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xccf5a4eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa935a4ef, SUC_lexicon_tags_25 },
    { 0xfad5a4f0, SUC_lexicon_tags_9 },
    { 0xdc95a4f0, SUC_lexicon_tags_5 },
    { 0x5055a4ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e15a4f4, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x2995a4f6, SUC_lexicon_tags_19 },
    { 0x1195a4f7, SUC_lexicon_tags_33 },
    { 0x8b75a4f8, SUC_lexicon_tags_37 },
    { 0xcff5a4f7, SUC_lexicon_tags_60 },
    { 0x18b5a4f8, SUC_lexicon_tags_37 },
    { 0xf295a4f7, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9295a4ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7355a501, SUC_lexicon_tags_1 },
    { 0x2a75a502, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe995a504, SUC_lexicon_tags_87 },
    { 0xcf75a504, SUC_lexicon_tags_22 },
    { 0x8f75a506, SUC_lexicon_tags_25 },
    { 0xb695a507, SUC_lexicon_tags_12 },
    { 0xff5a508, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf5a50c, SUC_lexicon_tags_17 },
    { 0xf155a50d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc975a514, SUC_lexicon_tags_15 },
    { 0x1375a515, SUC_lexicon_tags_32 },
    { 0x83f5a516, SUC_lexicon_tags_37 },
    { 0x355a516, SUC_lexicon_tags_1 },
    { 0xf135a518, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d75a51c, SUC_lexicon_tags_8 },
    { 0xfff5a51d, SUC_lexicon_tags_22 },
    { 0x2bd5a51e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd075a520, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb155a524, SUC_lexicon_tags_46 },
    { 0x7fd5a524, SUC_lexicon_tags_17 },
    { 0x8715a526, SUC_lexicon_tags_63 },
    { 0x4d95a527, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9295a52c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe95a52e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccd5a530, SUC_lexicon_tags_23 },
    { 0x6235a531, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa235a535, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8755a539, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b5a53d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7315a540, SUC_lexicon_tags_47 },
    { 0xb055a540, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6755a546, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb995a548, SUC_lexicon_tags_37 },
    { 0x4d55a549, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5795a54b, SUC_lexicon_tags_7 },
    { 0xf955a54b, SUC_lexicon_tags_30 },
    { 0x5715a54d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcbf5a54f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3195a558, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5455a55c, SUC_lexicon_tags_17 },
    { 0x16f5a55c, SUC_lexicon_tags_33 },
    { 0x66f5a55e, SUC_lexicon_tags_23 },
    { 0xa6b5a55e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd375a562, SUC_lexicon_tags_47 },
    { 0x84b5a563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6195a565, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e55a567, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd5a56a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b55a570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa815a572, SUC_lexicon_tags_12 },
    { 0xf455a572, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed5a577, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb35a579, SUC_lexicon_tags_11 },
    { 0xb675a57a, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6775a57f, SUC_lexicon_tags_23 },
    { 0x9b35a580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4515a585, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd5a589, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaad5a58b, SUC_lexicon_tags_37 },
    { 0x4a95a58c, SUC_lexicon_tags_7 },
    { 0xda35a58d, SUC_lexicon_tags_12 },
    { 0x4475a58d, SUC_lexicon_tags_0 },
    { 0xe575a58e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd5a592, SUC_lexicon_tags_14 },
    { 0xba55a593, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f35a598, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675a59c, SUC_lexicon_tags_34 },
    { 0xc555a59c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4035a5a0, SUC_lexicon_tags_48 },
    { 0xf4b5a5a0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db5a5a7, SUC_lexicon_tags_28 },
    { 0x3895a5a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce55a5aa, SUC_lexicon_tags_61 },
    { 0x3415a5ab, SUC_lexicon_tags_14 },
    { 0xfe75a5ab, SUC_lexicon_tags_2 },
    { 0x8e55a5ad, SUC_lexicon_tags_37 },
    { 0x53b5a5ae, SUC_lexicon_tags_14 },
    { 0xaf15a5ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23b5a5b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a15a5b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf75a5b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b55a5ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5235a5bc, SUC_lexicon_tags_15 },
    { 0x3ff5a5bd, SUC_lexicon_tags_1 },
    { 0x9ed5a5be, SUC_lexicon_tags_9 },
    { 0xf0f5a5bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac55a5c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6835a5d2, SUC_lexicon_tags_32 },
    { 0x2d35a5d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5a5d6, SUC_lexicon_tags_17 },
    { 0xadf5a5d7, SUC_lexicon_tags_29 },
    { 0x2ed5a5d8, SUC_lexicon_tags_12 },
    { 0x9555a5d6, SUC_lexicon_tags_25 },
    { 0x2bb5a5d8, SUC_lexicon_tags_63 },
    { 0x91f5a5d7, SUC_lexicon_tags_1 },
    { 0x1f55a5dc, SUC_lexicon_tags_2 },
    { 0x16b5a5dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5a5e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75d5a5e4, SUC_lexicon_tags_16 },
    { 0xb1f5a5e4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e55a5ee, SUC_lexicon_tags_8 },
    { 0xbe55a5ef, SUC_lexicon_tags_25 },
    { 0xe375a5f0, SUC_lexicon_tags_25 },
    { 0x8795a5f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c55a5f4, SUC_lexicon_tags_38 },
    { 0x9e95a5f5, SUC_lexicon_tags_12 },
    { 0xc2b5a5f5, SUC_lexicon_tags_11 },
    { 0xac95a5f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4355a5ff, SUC_lexicon_tags_28 },
    { 0xaa75a600, SUC_lexicon_tags_25 },
    { 0x3235a601, SUC_lexicon_tags_17 },
    { 0xa075a602, SUC_lexicon_tags_23 },
    { 0x4d75a603, SUC_lexicon_tags_43 },
    { 0x6d75a604, SUC_lexicon_tags_12 },
    { 0xd8b5a605, SUC_lexicon_tags_11 },
    { 0x44d5a602, SUC_lexicon_tags_2 },
    { 0x2915a604, SUC_lexicon_tags_6 },
    { 0xefd5a608, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf855a60a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8435a60e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed75a612, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf755a614, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab5a618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba55a61a, SUC_lexicon_tags_10 },
    { 0x80d5a61a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1815a61d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf035a61f, SUC_lexicon_tags_25 },
    { 0x4dd5a620, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd5a623, SUC_lexicon_tags_23 },
    { 0x9cb5a624, SUC_lexicon_tags_63 },
    { 0x2c35a624, SUC_lexicon_tags_2 },
    { 0xc215a626, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b5a62a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9495a62f, SUC_lexicon_tags_1 },
    { 0x4055a630, SUC_lexicon_tags_14 },
    { 0x5a15a62f, SUC_lexicon_tags_12 },
    { 0xbff5a632, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9875a634, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4ef5a636, SUC_lexicon_tags_37 },
    { 0xd2b5a636, SUC_lexicon_tags_17 },
    { 0x5455a638, SUC_lexicon_tags_158 },
    { 0xfbf5a639, SUC_lexicon_tags_15 },
    { 0x4135a638, SUC_lexicon_tags_37 },
    { 0x89b5a63b, SUC_lexicon_tags_7 },
    { 0x4d75a63c, SUC_lexicon_tags_2 },
    { 0x8b75a63d, SUC_lexicon_tags_0 },
    { 0x3f15a63c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb5b5a640, SUC_lexicon_tags_17 },
    { 0xa895a641, SUC_lexicon_tags_17 },
    { 0xd215a642, SUC_lexicon_tags_63 },
    { 0xe9b5a643, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5a651, SUC_lexicon_tags_32 },
    { 0x64b5a651, SUC_lexicon_tags_46 },
    { 0x2555a651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af5a656, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b5a65a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7775a65e, SUC_lexicon_tags_9 },
    { 0xc235a65f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xb615a661, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x5435a663, SUC_lexicon_tags_17 },
    { 0xb295a664, SUC_lexicon_tags_25 },
    { 0x6775a665, SUC_lexicon_tags_6 },
    { 0x36f5a665, SUC_lexicon_tags_63 },
    { 0xfbf5a667, SUC_lexicon_tags_37 },
    { 0x8bb5a668, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8015a66c, SUC_lexicon_tags_46 },
    { 0x1655a66d, SUC_lexicon_tags_39 },
    { 0x7a95a66e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d5a671, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc35a674, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d5a679, SUC_lexicon_tags_26 },
    { 0xc415a67a, SUC_lexicon_tags_31 },
    { 0xf875a67b, SUC_lexicon_tags_37 },
    { 0xb195a679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c75a67e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53b5a680, SUC_lexicon_tags_133 },
    { 0x3af5a681, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9db5a683, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f5a68c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3695a691, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd855a693, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xb535a696, SUC_lexicon_tags_29 },
    { 0x6035a696, SUC_lexicon_tags_17 },
    { 0x355a697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5095a69a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e55a69d, SUC_lexicon_tags_38 },
    { 0x3055a69e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd095a6a3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7755a6a5, SUC_lexicon_tags_45 },
    { 0xed15a6a6, SUC_lexicon_tags_23 },
    { 0x2515a6a6, SUC_lexicon_tags_8 },
    { 0x55b5a6a7, SUC_lexicon_tags_6 },
    { 0x9115a6a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e95a6b2, SUC_lexicon_tags_1 },
    { 0x81d5a6b3, SUC_lexicon_tags_47 },
    { 0x6ab5a6b3, SUC_lexicon_tags_30 },
    { 0x8355a6b2, SUC_lexicon_tags_15 },
    { 0x8d5a6b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa95a6b8, SUC_lexicon_tags_16 },
    { 0xc635a6b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d15a6bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf755a6bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc255a6c2, SUC_lexicon_tags_37 },
    { 0xc55a6c3, SUC_lexicon_tags_2 },
    { 0x1795a6c2, SUC_lexicon_tags_25 },
    { 0x3695a6c5, SUC_lexicon_tags_4 },
    { 0x2d5a6c6, SUC_lexicon_tags_2 },
    { 0xb855a6c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d5a6d0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8c75a6d2, SUC_lexicon_tags_82 },
    { 0xe815a6d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4155a6d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd75a6d9, SUC_lexicon_tags_9 },
    { 0x835a6da, SUC_lexicon_tags_25 },
    { 0x1b95a6da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7415a6dd, SUC_lexicon_tags_11 },
    { 0xcd55a6de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa55a6e1, SUC_lexicon_tags_2 },
    { 0x5f35a6e1, SUC_lexicon_tags_49 },
    { 0xbcb5a6e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f5a6e5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2d35a6e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd475a6ed, SUC_lexicon_tags_37 },
    { 0x2bb5a6ee, SUC_lexicon_tags_27 },
    { 0x8575a6ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd15a6f5, SUC_lexicon_tags_7 },
    { 0xbbb5a6f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87f5a6f8, SUC_lexicon_tags_57 },
    { 0x6835a6f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7635a6fb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa115a700, SUC_lexicon_tags_2 },
    { 0xdd15a701, SUC_lexicon_tags_12 },
    { 0x24b5a701, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15a70c, SUC_lexicon_tags_30 },
    { 0xd0b5a70c, SUC_lexicon_tags_15 },
    { 0x7ab5a70e, SUC_lexicon_tags_7 },
    { 0xc995a70f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b5a717, SUC_lexicon_tags_29 },
    { 0x3775a717, SUC_lexicon_tags_11 },
    { 0xa35a718, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a75a71e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9895a723, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc15a727, SUC_lexicon_tags_61 },
    { 0xe3d5a727, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3875a72b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d75a72d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3315a730, SUC_lexicon_tags_2 },
    { 0xcbb5a731, SUC_lexicon_tags_2 },
    { 0x5315a732, SUC_lexicon_tags_37 },
    { 0xb575a733, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35a73a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6955a73c, SUC_lexicon_tags_14 },
    { 0xd615a73d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c35a741, SUC_lexicon_tags_37 },
    { 0x2e95a742, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f75a745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b5a750, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac55a755, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb5a759, SUC_lexicon_tags_14 },
    { 0x655a75a, SUC_lexicon_tags_37 },
    { 0x4355a75a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a75a763, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2c35a765, SUC_lexicon_tags_37 },
    { 0x895a766, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc715a769, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xda35a76b, SUC_lexicon_tags_17 },
    { 0xc9b5a76b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3435a772, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f15a775, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2c95a777, SUC_lexicon_tags_46 },
    { 0xcfb5a778, SUC_lexicon_tags_25 },
    { 0x65d5a779, SUC_lexicon_tags_45 },
    { 0x48d5a779, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb15a77e, SUC_lexicon_tags_1 },
    { 0xf55a77e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xb4f5a781, SUC_lexicon_tags_1 },
    { 0xb755a782, SUC_lexicon_tags_19 },
    { 0x34b5a783, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b55a791, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555a797, SUC_lexicon_tags_49 },
    { 0x15b5a798, SUC_lexicon_tags_11 },
    { 0x1c75a798, SUC_lexicon_tags_15 },
    { 0xffb5a79a, SUC_lexicon_tags_45 },
    { 0xe175a79b, SUC_lexicon_tags_37 },
    { 0xd35a79a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2635a79e, SUC_lexicon_tags_1 },
    { 0x2d15a79e, SUC_lexicon_tags_37 },
    { 0xe715a79f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc355a7a3, SUC_lexicon_tags_30 },
    { 0x5b75a7a3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3835a7a6, SUC_lexicon_tags_3 },
    { 0x3515a7a7, SUC_lexicon_tags_0 },
    { 0xd5d5a7a7, SUC_lexicon_tags_38 },
    { 0xdbf5a7a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc0f5a7ab, SUC_lexicon_tags_37 },
    { 0x8035a7ac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef55a7b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc675a7b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d5a7b7, SUC_lexicon_tags_7 },
    { 0xddd5a7b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46b5a7ba, SUC_lexicon_tags_15 },
    { 0x135a7ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b5a7bf, SUC_lexicon_tags_47 },
    { 0xbd15a7bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf75a7c3, SUC_lexicon_tags_17 },
    { 0xef15a7c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4d5a7c6, SUC_lexicon_tags_17 },
    { 0x8255a7c7, SUC_lexicon_tags_11 },
    { 0xa7d5a7c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d5a7cb, SUC_lexicon_tags_15 },
    { 0xb8d5a7cc, SUC_lexicon_tags_38 },
    { 0xe435a7cd, SUC_lexicon_tags_3 },
    { 0x175a7cb, SUC_lexicon_tags_0 },
    { 0x4795a7cb, SUC_lexicon_tags_45 },
    { 0xe475a7cd, SUC_lexicon_tags_30 },
    { 0xaa95a7cb, SUC_lexicon_tags_27 },
    { 0x2335a7d2, SUC_lexicon_tags_37 },
    { 0x1135a7d0, SUC_lexicon_tags_37 },
    { 0xfab5a7d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d15a7d7, SUC_lexicon_tags_48 },
    { 0x61b5a7d8, SUC_lexicon_tags_39 },
    { 0xe9d5a7d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1675a7de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa455a7e2, SUC_lexicon_tags_14 },
    { 0x14f5a7e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af5a7e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d75a7e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55a7ee, SUC_lexicon_tags_37 },
    { 0xf3d5a7ef, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1955a7f1, SUC_lexicon_tags_38 },
    { 0x9235a7f2, SUC_lexicon_tags_27 },
    { 0xc255a7f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6255a7f6, SUC_lexicon_tags_15 },
    { 0xcb75a7f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f5a7fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb5a801, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa435a804, SUC_lexicon_tags_6 },
    { 0xc8d5a805, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5a808, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xfbb5a80a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb5a80e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc515a815, SUC_lexicon_tags_23 },
    { 0xb5d5a816, SUC_lexicon_tags_4 },
    { 0x9895a817, SUC_lexicon_tags_17 },
    { 0x1195a817, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2455a81e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf35a821, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7995a825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb15a827, SUC_lexicon_tags_7 },
    { 0xfe75a828, SUC_lexicon_tags_17 },
    { 0x5775a829, SUC_lexicon_tags_12 },
    { 0xb395a82a, SUC_lexicon_tags_38 },
    { 0xd875a828, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19f5a82d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f5a830, SUC_lexicon_tags_27 },
    { 0x3395a831, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x995a833, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb715a837, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f5a83c, SUC_lexicon_tags_2 },
    { 0x4af5a83d, SUC_lexicon_tags_11 },
    { 0xf795a83e, SUC_lexicon_tags_15 },
    { 0x8a55a83f, SUC_lexicon_tags_37 },
    { 0x1ff5a840, SUC_lexicon_tags_37 },
    { 0xbef5a83e, SUC_lexicon_tags_26 },
    { 0xe2f5a841, SUC_lexicon_tags_15 },
    { 0x4d55a843, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6295a845, SUC_lexicon_tags_12 },
    { 0xeb35a846, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5535a84c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x97d5a84e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e75a851, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5895a853, SUC_lexicon_tags_15 },
    { 0x44f5a853, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d5a859, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd815a85d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf075a860, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb75a868, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac35a86a, SUC_lexicon_tags_69 },
    { 0x5f5a86b, SUC_lexicon_tags_37 },
    { 0x7a15a86b, SUC_lexicon_tags_42 },
    { 0xad5a86c, SUC_lexicon_tags_2 },
    { 0x27f5a86d, SUC_lexicon_tags_1 },
    { 0x5d75a86f, SUC_lexicon_tags_38 },
    { 0x6935a870, SUC_lexicon_tags_60 },
    { 0xf535a871, SUC_lexicon_tags_19 },
    { 0x1e15a872, SUC_lexicon_tags_23 },
    { 0xaeb5a873, SUC_lexicon_tags_12 },
    { 0x5b15a872, SUC_lexicon_tags_25 },
    { 0xad35a874, SUC_lexicon_tags_3 },
    { 0x2195a876, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5695a878, SUC_lexicon_tags_60 },
    { 0x8375a879, SUC_lexicon_tags_17 },
    { 0x2f55a879, SUC_lexicon_tags_25 },
    { 0xf6f5a878, SUC_lexicon_tags_17 },
    { 0xd075a878, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af5a881, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8835a884, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5a887, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15a889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb5a88e, SUC_lexicon_tags_63 },
    { 0x2175a88f, SUC_lexicon_tags_11 },
    { 0x1095a88f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe075a895, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f5a89a, SUC_lexicon_tags_15 },
    { 0xccb5a89b, SUC_lexicon_tags_25 },
    { 0xe5b5a89b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c15a8a2, SUC_lexicon_tags_42 },
    { 0x22b5a8a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7555a8a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b5a8a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5255a8ac, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb5a8b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2155a8b3, SUC_lexicon_tags_12 },
    { 0x5355a8b4, SUC_lexicon_tags_17 },
    { 0x1df5a8b4, SUC_lexicon_tags_0 },
    { 0xfcd5a8b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2395a8b9, SUC_lexicon_tags_19 },
    { 0xb3b5a8ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce35a8bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c15a8c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf5a8c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d5a8c8, SUC_lexicon_tags_77 },
    { 0x9095a8c8, SUC_lexicon_tags_27 },
    { 0x31d5a8ca, SUC_lexicon_tags_17 },
    { 0x895a8c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5715a8cd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe735a8d3, SUC_lexicon_tags_91 },
    { 0x1a15a8d4, SUC_lexicon_tags_719 },
    { 0xd3d5a8d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9815a8d7, SUC_lexicon_tags_12 },
    { 0xf295a8d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd555a8db, SUC_lexicon_tags_18 },
    { 0x95a8dc, SUC_lexicon_tags_65 },
    { 0x2ed5a8dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9075a8df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe675a8e1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad5a8e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd975a8ed, SUC_lexicon_tags_31 },
    { 0xab35a8ee, SUC_lexicon_tags_12 },
    { 0x2055a8ef, SUC_lexicon_tags_17 },
    { 0x3f95a8f0, SUC_lexicon_tags_17 },
    { 0x9d15a8f1, SUC_lexicon_tags_140 },
    { 0xf415a8ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad35a8f6, SUC_lexicon_tags_25 },
    { 0xc75a8f7, SUC_lexicon_tags_10 },
    { 0x9cb5a8f7, SUC_lexicon_tags_11 },
    { 0x8015a8f7, SUC_lexicon_tags_47 },
    { 0x1595a8fa, SUC_lexicon_tags_27 },
    { 0x32b5a8fa, SUC_lexicon_tags_11 },
    { 0xef35a8fc, SUC_lexicon_tags_257 },
    { 0xf5a8f6, SUC_lexicon_tags_23 },
    { 0x78d5a8fe, SUC_lexicon_tags_43 },
    { 0x57d5a8fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe15a908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a75a90b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d5a90f, SUC_lexicon_tags_37 },
    { 0x6975a910, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f5a914, SUC_lexicon_tags_37 },
    { 0xfa75a915, SUC_lexicon_tags_12 },
    { 0xae35a916, SUC_lexicon_tags_13 },
    { 0x26b5a917, SUC_lexicon_tags_2 },
    { 0x1d15a917, SUC_lexicon_tags_25 },
    { 0xe195a918, SUC_lexicon_tags_31 },
    { 0xf495a919, SUC_lexicon_tags_92 },
    { 0x8935a91a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a75a91e, SUC_lexicon_tags_7 },
    { 0xa2d5a91e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff5a926, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7135a92a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45b5a92c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9215a92e, SUC_lexicon_tags_25 },
    { 0x2375a92f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6a75a931, SUC_lexicon_tags_7 },
    { 0x8555a932, SUC_lexicon_tags_56 },
    { 0x4af5a933, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3475a938, SUC_lexicon_tags_25 },
    { 0xa775a938, SUC_lexicon_tags_45 },
    { 0x6c15a93a, SUC_lexicon_tags_14 },
    { 0x3b35a93b, SUC_lexicon_tags_55 },
    { 0x75a93b, SUC_lexicon_tags_14 },
    { 0xf455a938, SUC_lexicon_tags_26 },
    { 0x2135a93e, SUC_lexicon_tags_17 },
    { 0x50f5a93f, SUC_lexicon_tags_97 },
    { 0xcb75a93e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa895a94a, SUC_lexicon_tags_37 },
    { 0xc555a94b, SUC_lexicon_tags_11 },
    { 0xde75a94c, SUC_lexicon_tags_46 },
    { 0xa7b5a94c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x4e55a94f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c55a953, SUC_lexicon_tags_3 },
    { 0x16f5a954, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f95a95a, SUC_lexicon_tags_23 },
    { 0x4b35a95a, SUC_lexicon_tags_61 },
    { 0x9e55a95a, SUC_lexicon_tags_15 },
    { 0xd815a95a, SUC_lexicon_tags_28 },
    { 0x8415a95e, SUC_lexicon_tags_6 },
    { 0xb415a95f, SUC_lexicon_tags_7 },
    { 0x4ad5a960, SUC_lexicon_tags_72 },
    { 0xb4b5a95c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d5a964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1955a966, SUC_lexicon_tags_45 },
    { 0x6c75a966, SUC_lexicon_tags_17 },
    { 0x3c15a968, SUC_lexicon_tags_23 },
    { 0xf995a969, SUC_lexicon_tags_2 },
    { 0x77d5a96a, SUC_lexicon_tags_25 },
    { 0xc9d5a968, SUC_lexicon_tags_68 },
    { 0x8035a96c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde75a96e, SUC_lexicon_tags_65 },
    { 0xbe35a96e, SUC_lexicon_tags_37 },
    { 0xca35a96f, SUC_lexicon_tags_0 },
    { 0xd9b5a971, SUC_lexicon_tags_27 },
    { 0xe9b5a972, SUC_lexicon_tags_177 },
    { 0xe215a970, SUC_lexicon_tags_12 },
    { 0x6055a972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4f5a976, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x955a978, SUC_lexicon_tags_3 },
    { 0xf235a978, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1935a97c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9555a97e, SUC_lexicon_tags_15 },
    { 0xb9d5a97f, SUC_lexicon_tags_39 },
    { 0x6bf5a980, SUC_lexicon_tags_1 },
    { 0x8e15a980, SUC_lexicon_tags_39 },
    { 0x17f5a97e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd15a985, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f5a988, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6095a98e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df5a992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae75a994, SUC_lexicon_tags_15 },
    { 0xd515a994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd5a998, SUC_lexicon_tags_49 },
    { 0xef55a999, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d5a99e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255a9a7, SUC_lexicon_tags_60 },
    { 0x63b5a9a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34d5a9aa, SUC_lexicon_tags_60 },
    { 0xa935a9ab, SUC_lexicon_tags_47 },
    { 0x5715a9ab, SUC_lexicon_tags_9 },
    { 0xf6b5a9ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa975a9af, SUC_lexicon_tags_55 },
    { 0xb7f5a9b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b55a9b2, SUC_lexicon_tags_1 },
    { 0x8af5a9b3, SUC_lexicon_tags_7 },
    { 0xf635a9b4, SUC_lexicon_tags_35 },
    { 0x1e75a9b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1755a9ba, SUC_lexicon_tags_38 },
    { 0xf455a9bb, SUC_lexicon_tags_82 },
    { 0x3635a9bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc155a9be, SUC_lexicon_tags_11 },
    { 0xdf15a9be, SUC_lexicon_tags_25 },
    { 0xaf15a9c0, SUC_lexicon_tags_63 },
    { 0x8b15a9c1, SUC_lexicon_tags_7 },
    { 0x6ed5a9be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8395a9c9, SUC_lexicon_tags_7 },
    { 0xd835a9ca, SUC_lexicon_tags_7 },
    { 0x5595a9ca, SUC_lexicon_tags_11 },
    { 0xe155a9cc, SUC_lexicon_tags_14 },
    { 0xc035a9cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef5a9d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b5a9da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdb5a9dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad55a9de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7d5a9e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe15a9e3, SUC_lexicon_tags_27 },
    { 0x4c75a9e3, SUC_lexicon_tags_26 },
    { 0x2a75a9e5, SUC_lexicon_tags_25 },
    { 0x32d5a9e6, SUC_lexicon_tags_0 },
    { 0xd5f5a9e3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4855a9eb, SUC_lexicon_tags_29 },
    { 0x8655a9eb, SUC_lexicon_tags_37 },
    { 0x8775a9eb, SUC_lexicon_tags_52 },
    { 0xe195a9ee, SUC_lexicon_tags_47 },
    { 0x46d5a9ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3315a9f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc075a9f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed5a9f8, SUC_lexicon_tags_15 },
    { 0xd835a9f9, SUC_lexicon_tags_0 },
    { 0x47d5a9f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f5a9fe, SUC_lexicon_tags_38 },
    { 0x78d5a9ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d5aa01, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad5aa07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f5aa0b, SUC_lexicon_tags_1 },
    { 0x2e15aa0c, SUC_lexicon_tags_37 },
    { 0x7b75aa0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6f5aa0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5415aa12, SUC_lexicon_tags_25 },
    { 0x675aa13, SUC_lexicon_tags_47 },
    { 0xf4f5aa14, SUC_lexicon_tags_25 },
    { 0xea95aa14, SUC_lexicon_tags_31 },
    { 0x8e35aa16, SUC_lexicon_tags_1 },
    { 0x9bd5aa17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2835aa19, SUC_lexicon_tags_27 },
    { 0x8f35aa19, SUC_lexicon_tags_12 },
    { 0xd795aa1b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb5aa23, SUC_lexicon_tags_48 },
    { 0x5915aa23, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1795aa28, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35aa2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd55aa2f, SUC_lexicon_tags_37 },
    { 0x93f5aa2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde75aa37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7555aa3a, SUC_lexicon_tags_80 },
    { 0x9f95aa3b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7835aa41, SUC_lexicon_tags_6 },
    { 0x7a15aa41, SUC_lexicon_tags_23 },
    { 0x24d5aa43, SUC_lexicon_tags_1 },
    { 0x10d5aa43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x71d5aa46, SUC_lexicon_tags_37 },
    { 0x48d5aa47, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced5aa51, SUC_lexicon_tags_60 },
    { 0xdcd5aa52, SUC_lexicon_tags_37 },
    { 0x92b5aa53, SUC_lexicon_tags_87 },
    { 0xa975aa54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b15aa5c, SUC_lexicon_tags_27 },
    { 0x9d35aa5d, SUC_lexicon_tags_1 },
    { 0x1af5aa5e, SUC_lexicon_tags_47 },
    { 0x6f15aa5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff35aa61, SUC_lexicon_tags_17 },
    { 0x9cf5aa62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa775aa64, SUC_lexicon_tags_7 },
    { 0xce75aa64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb175aa69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5c95aa6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x475aa6d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd5aa71, SUC_lexicon_tags_17 },
    { 0x8175aa72, SUC_lexicon_tags_11 },
    { 0xa75aa72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3635aa78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5055aa7a, SUC_lexicon_tags_1 },
    { 0xd195aa7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf5aa7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x19d5aa80, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc795aa82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5015aa84, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb735aa86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe0b5aa88, SUC_lexicon_tags_0 },
    { 0x7075aa88, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x6f75aa8b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8e75aa8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2f5aa8f, SUC_lexicon_tags_14 },
    { 0x8795aa90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d5aa93, SUC_lexicon_tags_12 },
    { 0xdb55aa94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895aa9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3455aaa0, SUC_lexicon_tags_25 },
    { 0xd175aaa0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb5aaa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3635aaa7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa35aaaa, SUC_lexicon_tags_80 },
    { 0x34d5aaaa, SUC_lexicon_tags_14 },
    { 0xaf5aaaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbb5aaae, SUC_lexicon_tags_37 },
    { 0x4f95aaaf, SUC_lexicon_tags_11 },
    { 0x65b5aab0, SUC_lexicon_tags_43 },
    { 0x22d5aab0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff55aac0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb5aac8, SUC_lexicon_tags_16 },
    { 0x5eb5aac9, SUC_lexicon_tags_25 },
    { 0x86f5aac8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2555aacd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1835aad1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f5aad5, SUC_lexicon_tags_55 },
    { 0xeb75aad6, SUC_lexicon_tags_31 },
    { 0x8c75aad6, SUC_lexicon_tags_37 },
    { 0xb635aad6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa95aadc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4ab5aade, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e55aae0, SUC_lexicon_tags_15 },
    { 0x1af5aae1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf5aae4, SUC_lexicon_tags_25 },
    { 0xc875aae5, SUC_lexicon_tags_6 },
    { 0x5b55aae6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf375aae9, SUC_lexicon_tags_11 },
    { 0x8615aaea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a75aaee, SUC_lexicon_tags_87 },
    { 0x16b5aaee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5295aaf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5aaf6, SUC_lexicon_tags_170 },
    { 0x5cd5aaf6, SUC_lexicon_tags_77 },
    { 0xc6b5aaf6, SUC_lexicon_tags_11 },
    { 0xca95aaf9, SUC_lexicon_tags_38 },
    { 0x8695aafa, SUC_lexicon_tags_17 },
    { 0xd8f5aafb, SUC_lexicon_tags_23 },
    { 0xde75aafb, SUC_lexicon_tags_49 },
    { 0x2615aafd, SUC_lexicon_tags_46 },
    { 0x67b5aafe, SUC_lexicon_tags_49 },
    { 0x4835aaff, SUC_lexicon_tags_65 },
    { 0x9b5aaf8, SUC_lexicon_tags_77 },
    { 0xf495ab01, SUC_lexicon_tags_28 },
    { 0x22b5ab02, SUC_lexicon_tags_25 },
    { 0xbb55ab03, SUC_lexicon_tags_37 },
    { 0x3f75ab02, SUC_lexicon_tags_2 },
    { 0xefb5ab05, SUC_lexicon_tags_11 },
    { 0x3655ab06, SUC_lexicon_tags_47 },
    { 0xf515ab07, SUC_lexicon_tags_3 },
    { 0x4775ab08, SUC_lexicon_tags_11 },
    { 0x1075ab09, SUC_lexicon_tags_57 },
    { 0xe2b5aafd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef5ab12, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7af5ab14, SUC_lexicon_tags_30 },
    { 0x25b5ab15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf5ab18, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad5ab1d, SUC_lexicon_tags_29 },
    { 0x6cf5ab1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f5ab21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x67d5ab23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b95ab2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0f5ab2c, SUC_lexicon_tags_17 },
    { 0x7815ab2c, SUC_lexicon_tags_37 },
    { 0xf6d5ab2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5195ab30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a75ab34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e75ab36, SUC_lexicon_tags_38 },
    { 0xa795ab37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe75ab3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc255ab3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b5ab4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb35ab4c, SUC_lexicon_tags_25 },
    { 0x1c15ab4d, SUC_lexicon_tags_9 },
    { 0x8115ab4d, SUC_lexicon_tags_7 },
    { 0x655ab4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb5ab54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7335ab5b, SUC_lexicon_tags_17 },
    { 0xadf5ab5c, SUC_lexicon_tags_82 },
    { 0xee15ab5d, SUC_lexicon_tags_17 },
    { 0x1955ab5b, SUC_lexicon_tags_59 },
    { 0xb235ab5f, SUC_lexicon_tags_7 },
    { 0x95b5ab60, SUC_lexicon_tags_9 },
    { 0x3f55ab61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff5ab64, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f5ab6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f5ab72, SUC_lexicon_tags_63 },
    { 0x46d5ab73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf5ab76, SUC_lexicon_tags_27 },
    { 0x8c15ab77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d5ab7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3175ab7f, SUC_lexicon_tags_37 },
    { 0x5af5ab7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78d5ab82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb575ab87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75ab89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f95ab8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b5ab90, SUC_lexicon_tags_47 },
    { 0xb5d5ab91, SUC_lexicon_tags_25 },
    { 0xed15ab92, SUC_lexicon_tags_2 },
    { 0xf9f5ab93, SUC_lexicon_tags_49 },
    { 0xb655ab92, SUC_lexicon_tags_63 },
    { 0xb995ab95, SUC_lexicon_tags_60 },
    { 0x1ad5ab96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5715ab98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d5ab9a, SUC_lexicon_tags_14 },
    { 0xa615ab9b, SUC_lexicon_tags_30 },
    { 0xa835ab9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdbd5ab9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ef5aba0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6155aba4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1095aba8, SUC_lexicon_tags_8 },
    { 0x5b75aba8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5975abab, SUC_lexicon_tags_1 },
    { 0xc635abac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5abb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8695abb6, SUC_lexicon_tags_27 },
    { 0x2235abb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde95abbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbd5abbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd035abc1, SUC_lexicon_tags_8 },
    { 0xebb5abc1, SUC_lexicon_tags_15 },
    { 0x1e75abc1, SUC_lexicon_tags_65 },
    { 0x5c95abc4, SUC_lexicon_tags_21 },
    { 0x5235abc5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1f5abc7, SUC_lexicon_tags_25 },
    { 0xfe15abc8, SUC_lexicon_tags_2 },
    { 0x19d5abc9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f5abcc, SUC_lexicon_tags_31 },
    { 0x2555abcc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8895abcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b5abd3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xddb5abd5, SUC_lexicon_tags_108 },
    { 0x7595abd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7295abda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab75abdf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f5abe5, SUC_lexicon_tags_27 },
    { 0x7a75abe5, SUC_lexicon_tags_25 },
    { 0x1595abe7, SUC_lexicon_tags_37 },
    { 0x1895abe8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeb5abea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8af5abec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5355abef, SUC_lexicon_tags_7 },
    { 0x57d5abf0, SUC_lexicon_tags_12 },
    { 0x7e35abf1, SUC_lexicon_tags_37 },
    { 0x7d55abef, SUC_lexicon_tags_12 },
    { 0xf3f5abf1, SUC_lexicon_tags_1 },
    { 0x4555abf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e15abf7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2795ac02, SUC_lexicon_tags_13 },
    { 0x15d5ac03, SUC_lexicon_tags_11 },
    { 0x14d5ac04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xced5ac06, SUC_lexicon_tags_2 },
    { 0x47b5ac07, SUC_lexicon_tags_23 },
    { 0xc295ac08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfa15ac0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d5ac12, SUC_lexicon_tags_11 },
    { 0xb435ac12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa155ac17, SUC_lexicon_tags_1 },
    { 0xce55ac17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b5ac1b, SUC_lexicon_tags_38 },
    { 0x33b5ac1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcf5ac1e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe655ac20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3195ac22, SUC_lexicon_tags_29 },
    { 0xc715ac23, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc8f5ac25, SUC_lexicon_tags_47 },
    { 0x5755ac26, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b5ac2a, SUC_lexicon_tags_12 },
    { 0xf3b5ac2b, SUC_lexicon_tags_45 },
    { 0x4f75ac2c, SUC_lexicon_tags_8 },
    { 0x11d5ac2b, SUC_lexicon_tags_10 },
    { 0x3bd5ac2e, SUC_lexicon_tags_197 },
    { 0x5d5ac2c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe175ac37, SUC_lexicon_tags_7 },
    { 0x4515ac38, SUC_lexicon_tags_27 },
    { 0x6c35ac39, SUC_lexicon_tags_17 },
    { 0x9c55ac37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df5ac3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf195ac41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13b5ac43, SUC_lexicon_tags_1 },
    { 0x18f5ac44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c55ac49, SUC_lexicon_tags_11 },
    { 0x97b5ac49, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8475ac4c, SUC_lexicon_tags_3 },
    { 0xfa15ac4d, SUC_lexicon_tags_34 },
    { 0xf1b5ac4c, SUC_lexicon_tags_6 },
    { 0xbab5ac4c, SUC_lexicon_tags_45 },
    { 0x4e35ac4c, SUC_lexicon_tags_25 },
    { 0x7935ac4f, SUC_lexicon_tags_25 },
    { 0x79d5ac52, SUC_lexicon_tags_17 },
    { 0x52f5ac53, SUC_lexicon_tags_27 },
    { 0x8ab5ac53, SUC_lexicon_tags_6 },
    { 0xedf5ac53, SUC_lexicon_tags_31 },
    { 0xc55ac54, SUC_lexicon_tags_17 },
    { 0x7535ac57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa035ac5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb095ac5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d5ac66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xedf5ac68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5ac6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5575ac71, SUC_lexicon_tags_27 },
    { 0xacd5ac72, SUC_lexicon_tags_11 },
    { 0xd135ac72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff5ac76, SUC_lexicon_tags_37 },
    { 0x8795ac77, SUC_lexicon_tags_66 },
    { 0xf075ac77, SUC_lexicon_tags_15 },
    { 0x9f55ac79, SUC_lexicon_tags_14 },
    { 0x9c95ac7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d5ac7e, SUC_lexicon_tags_25 },
    { 0x3e75ac7f, SUC_lexicon_tags_0 },
    { 0x6e75ac80, SUC_lexicon_tags_11 },
    { 0x7135ac7e, SUC_lexicon_tags_23 },
    { 0x2b35ac81, SUC_lexicon_tags_55 },
    { 0x3dd5ac80, SUC_lexicon_tags_39 },
    { 0x23d5ac84, SUC_lexicon_tags_17 },
    { 0xfd35ac7f, SUC_lexicon_tags_27 },
    { 0x79f5ac86, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f5ac8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f5ac8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5ac91, SUC_lexicon_tags_17 },
    { 0x4cb5ac92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb235ac97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc415ac99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd875ac9b, SUC_lexicon_tags_37 },
    { 0x75f5ac9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d5aca0, SUC_lexicon_tags_31 },
    { 0xfb95aca0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd555aca3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b5aca7, SUC_lexicon_tags_17 },
    { 0x2f35aca7, SUC_lexicon_tags_33 },
    { 0x34d5aca9, SUC_lexicon_tags_10 },
    { 0xf3d5aca7, SUC_lexicon_tags_37 },
    { 0x8b95acab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf415acae, SUC_lexicon_tags_32 },
    { 0x5f95acaf, SUC_lexicon_tags_11 },
    { 0x1b35acb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5acb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5095acb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c95acba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe655acbd, SUC_lexicon_tags_17 },
    { 0xfe55acbd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b5acc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5acc5, SUC_lexicon_tags_7 },
    { 0x5dd5acc6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9935acce, SUC_lexicon_tags_1 },
    { 0x3255accf, SUC_lexicon_tags_25 },
    { 0xd895acce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x215acd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe855acd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb975acd7, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff35acdd, SUC_lexicon_tags_7 },
    { 0xd9b5acde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e15ace0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe215ace3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b5ace6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835acf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c75acf8, SUC_lexicon_tags_14 },
    { 0x3b5acf8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfaf5acfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5135acfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b5ad03, SUC_lexicon_tags_7 },
    { 0x33d5ad04, SUC_lexicon_tags_21 },
    { 0xa5f5ad04, SUC_lexicon_tags_37 },
    { 0x3b35ad06, SUC_lexicon_tags_46 },
    { 0xab75ad04, SUC_lexicon_tags_1 },
    { 0xe575ad04, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc595ad0a, SUC_lexicon_tags_12 },
    { 0x71b5ad0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f55ad0d, SUC_lexicon_tags_17 },
    { 0x7f15ad0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa255ad14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2495ad16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe255ad19, SUC_lexicon_tags_11 },
    { 0xdd5ad1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1135ad1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b5ad1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba75ad21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5d15ad23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1515ad25, SUC_lexicon_tags_65 },
    { 0x6b5ad26, SUC_lexicon_tags_25 },
    { 0xe475ad26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf55ad2a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xeed5ad2c, SUC_lexicon_tags_37 },
    { 0xddf5ad2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd2b5ad2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa035ad31, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef5ad35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5ad3d, SUC_lexicon_tags_17 },
    { 0xb155ad3e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc995ad40, SUC_lexicon_tags_14 },
    { 0x7ab5ad40, SUC_lexicon_tags_11 },
    { 0x5df5ad42, SUC_lexicon_tags_25 },
    { 0xa1b5ad43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3975ad47, SUC_lexicon_tags_10 },
    { 0x12d5ad47, SUC_lexicon_tags_3 },
    { 0x7f5ad47, SUC_lexicon_tags_33 },
    { 0xe935ad49, SUC_lexicon_tags_37 },
    { 0x915ad48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd55ad4e, SUC_lexicon_tags_11 },
    { 0x1555ad4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a35ad53, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x195ad56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5815ad59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d5ad5e, SUC_lexicon_tags_7 },
    { 0xbe75ad5f, SUC_lexicon_tags_37 },
    { 0xe735ad5e, SUC_lexicon_tags_2 },
    { 0xb7b5ad61, SUC_lexicon_tags_31 },
    { 0x54b5ad61, SUC_lexicon_tags_25 },
    { 0xd835ad61, SUC_lexicon_tags_1 },
    { 0xf4d5ad63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb5ad68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb895ad6a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e75ad70, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xea95ad72, SUC_lexicon_tags_47 },
    { 0x6a15ad72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f5ad7b, SUC_lexicon_tags_37 },
    { 0x76b5ad7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b55ad7f, SUC_lexicon_tags_15 },
    { 0xe555ad80, SUC_lexicon_tags_17 },
    { 0x6db5ad81, SUC_lexicon_tags_15 },
    { 0xbb15ad81, SUC_lexicon_tags_17 },
    { 0xe3d5ad82, SUC_lexicon_tags_39 },
    { 0xf375ad84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95ad87, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3035ad8b, SUC_lexicon_tags_4 },
    { 0xa5b5ad8b, SUC_lexicon_tags_25 },
    { 0x6895ad8d, SUC_lexicon_tags_25 },
    { 0xb5b5ad8e, SUC_lexicon_tags_25 },
    { 0x14b5ad8f, SUC_lexicon_tags_27 },
    { 0x4495ad8f, SUC_lexicon_tags_15 },
    { 0xd1b5ad91, SUC_lexicon_tags_0 },
    { 0x7ef5ad91, SUC_lexicon_tags_9 },
    { 0x5ed5ad92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ed5ad95, SUC_lexicon_tags_48 },
    { 0xdc15ad96, SUC_lexicon_tags_12 },
    { 0x3d55ad96, SUC_lexicon_tags_2 },
    { 0xd635ad95, SUC_lexicon_tags_16 },
    { 0x8a15ad98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4355ad9c, SUC_lexicon_tags_45 },
    { 0xbe75ad9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1235ad9f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf915ada3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d5ada9, SUC_lexicon_tags_63 },
    { 0x18f5adaa, SUC_lexicon_tags_33 },
    { 0x2015adab, SUC_lexicon_tags_1 },
    { 0xc6b5adab, SUC_lexicon_tags_0 },
    { 0xb6b5adad, SUC_lexicon_tags_32 },
    { 0xf635adae, SUC_lexicon_tags_87 },
    { 0x5915adac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f5adb2, SUC_lexicon_tags_12 },
    { 0x3735adb3, SUC_lexicon_tags_25 },
    { 0x1795adb2, SUC_lexicon_tags_7 },
    { 0x9735adb5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb75adba, SUC_lexicon_tags_43 },
    { 0x5675adbb, SUC_lexicon_tags_33 },
    { 0xc75adbc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8c75adbe, SUC_lexicon_tags_1 },
    { 0x1375adbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd5adc2, SUC_lexicon_tags_21 },
    { 0xcf55adc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2155adc5, SUC_lexicon_tags_1 },
    { 0xbf5adc6, SUC_lexicon_tags_31 },
    { 0x3395adc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b75adca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4315adcc, SUC_lexicon_tags_30 },
    { 0xb095adcd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe055add0, SUC_lexicon_tags_37 },
    { 0xeeb5add1, SUC_lexicon_tags_11 },
    { 0x1b15add2, SUC_lexicon_tags_11 },
    { 0xc0f5add3, SUC_lexicon_tags_25 },
    { 0xe995add4, SUC_lexicon_tags_45 },
    { 0xbcf5add2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd5addd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb5ade2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7435ade6, SUC_lexicon_tags_14 },
    { 0x3b5ade6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8215ade9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1155aded, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e95adef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbfd5adf1, SUC_lexicon_tags_15 },
    { 0xdbb5adf2, SUC_lexicon_tags_14 },
    { 0x8675adf1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6335adf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf155adf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d5ae00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a55ae03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6235ae0e, SUC_lexicon_tags_28 },
    { 0x1635ae0f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d5ae13, SUC_lexicon_tags_376 },
    { 0x40d5ae14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1895ae17, SUC_lexicon_tags_23 },
    { 0xc555ae18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab15ae23, SUC_lexicon_tags_47 },
    { 0x2615ae24, SUC_lexicon_tags_60 },
    { 0x7315ae25, SUC_lexicon_tags_30 },
    { 0xddf5ae26, SUC_lexicon_tags_19 },
    { 0x3255ae27, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7115ae2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5495ae2d, SUC_lexicon_tags_37 },
    { 0x7935ae2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8595ae31, SUC_lexicon_tags_12 },
    { 0xf435ae31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbf5ae34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1895ae3b, SUC_lexicon_tags_26 },
    { 0xbdf5ae3c, SUC_lexicon_tags_9 },
    { 0xbf5ae3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd5ae40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4f5ae42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb5ae4f, SUC_lexicon_tags_38 },
    { 0x9ff5ae50, SUC_lexicon_tags_68 },
    { 0xaed5ae4f, SUC_lexicon_tags_6 },
    { 0x7a95ae4f, SUC_lexicon_tags_65 },
    { 0xd655ae53, SUC_lexicon_tags_37 },
    { 0x12b5ae51, SUC_lexicon_tags_14 },
    { 0x12b5ae54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c35ae58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb855ae5a, SUC_lexicon_tags_12 },
    { 0xda35ae5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7855ae5f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9955ae63, SUC_lexicon_tags_17 },
    { 0x2055ae64, SUC_lexicon_tags_37 },
    { 0x2135ae65, SUC_lexicon_tags_29 },
    { 0x89b5ae66, SUC_lexicon_tags_34 },
    { 0x9695ae67, SUC_lexicon_tags_66 },
    { 0x7255ae68, SUC_lexicon_tags_61 },
    { 0xfbb5ae69, SUC_lexicon_tags_7 },
    { 0xf0f5ae69, SUC_lexicon_tags_17 },
    { 0x17f5ae6a, SUC_lexicon_tags_70 },
    { 0x6195ae6b, SUC_lexicon_tags_25 },
    { 0xa0f5ae6d, SUC_lexicon_tags_37 },
    { 0x48b5ae6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbad5ae70, SUC_lexicon_tags_15 },
    { 0xcc75ae71, SUC_lexicon_tags_37 },
    { 0x7e95ae72, SUC_lexicon_tags_47 },
    { 0xa775ae73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3f5ae75, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5975ae7d, SUC_lexicon_tags_33 },
    { 0x7775ae7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe275ae84, SUC_lexicon_tags_31 },
    { 0x1495ae85, SUC_lexicon_tags_49 },
    { 0x7dd5ae85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd855ae8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3715ae8d, SUC_lexicon_tags_17 },
    { 0x9e15ae8d, SUC_lexicon_tags_6 },
    { 0xe055ae8f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4d5ae91, SUC_lexicon_tags_27 },
    { 0xcb35ae92, SUC_lexicon_tags_26 },
    { 0x6455ae93, SUC_lexicon_tags_23 },
    { 0x97d5ae94, SUC_lexicon_tags_21 },
    { 0xd7f5ae94, SUC_lexicon_tags_37 },
    { 0x9e55ae93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6015ae9b, SUC_lexicon_tags_7 },
    { 0x7235ae9c, SUC_lexicon_tags_7 },
    { 0xefb5ae9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f75aea3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9a35aea5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca15aeaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe195aead, SUC_lexicon_tags_6 },
    { 0x5715aead, SUC_lexicon_tags_37 },
    { 0x9cf5aeae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd35aeb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df5aeb9, SUC_lexicon_tags_23 },
    { 0x1775aeba, SUC_lexicon_tags_1 },
    { 0x9d75aeba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78f5aebd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2075aec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9295aec2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3795aec8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7615aecf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea75aed1, SUC_lexicon_tags_56 },
    { 0x1115aed1, SUC_lexicon_tags_14 },
    { 0x9f55aed3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8855aede, SUC_lexicon_tags_4 },
    { 0x6055aedf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4875aee4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9c15aee6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf095aee8, SUC_lexicon_tags_37 },
    { 0x1835aee8, SUC_lexicon_tags_11 },
    { 0x48f5aee8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc595aeed, SUC_lexicon_tags_6 },
    { 0xfb35aeee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98b5aef0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38b5aef2, SUC_lexicon_tags_63 },
    { 0x675aef3, SUC_lexicon_tags_25 },
    { 0x3b35aef4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5395aefb, SUC_lexicon_tags_17 },
    { 0x6f75aefc, SUC_lexicon_tags_25 },
    { 0x2715aefb, SUC_lexicon_tags_35 },
    { 0x4915aefe, SUC_lexicon_tags_17 },
    { 0xb075aeff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1415af01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46d5af03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd8b5af05, SUC_lexicon_tags_25 },
    { 0xea55af06, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5af0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b95af0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55f5af0f, SUC_lexicon_tags_14 },
    { 0x2d75af0f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b5af13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1f5af15, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x9f55af17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f5af1a, SUC_lexicon_tags_17 },
    { 0x2115af1a, SUC_lexicon_tags_105 },
    { 0x6115af1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa295af1e, SUC_lexicon_tags_12 },
    { 0x5b95af1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa955af21, SUC_lexicon_tags_63 },
    { 0xd715af22, SUC_lexicon_tags_10 },
    { 0x1955af22, SUC_lexicon_tags_63 },
    { 0xc015af21, SUC_lexicon_tags_31 },
    { 0x1f95af25, SUC_lexicon_tags_19 },
    { 0x9575af26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb195af2d, SUC_lexicon_tags_81 },
    { 0x6a35af2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d5af31, SUC_lexicon_tags_45 },
    { 0x9215af32, SUC_lexicon_tags_25 },
    { 0x7455af32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5395af35, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b75af39, SUC_lexicon_tags_11 },
    { 0x3d35af39, SUC_lexicon_tags_47 },
    { 0xe8d5af3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3335af3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf95af41, SUC_lexicon_tags_63 },
    { 0xddf5af42, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3215af48, SUC_lexicon_tags_30 },
    { 0xfa95af49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f15af4e, SUC_lexicon_tags_2 },
    { 0x6d35af4f, SUC_lexicon_tags_14 },
    { 0x6a15af50, SUC_lexicon_tags_37 },
    { 0x4535af51, SUC_lexicon_tags_32 },
    { 0x3cb5af52, SUC_lexicon_tags_13 },
    { 0xee15af53, SUC_lexicon_tags_60 },
    { 0xec35af4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5af57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a35af59, SUC_lexicon_tags_37 },
    { 0x4bd5af5a, SUC_lexicon_tags_27 },
    { 0x63d5af5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x93f5af5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf595af60, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xab5af62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7d15af64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaad5af66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb5af6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d5af6d, SUC_lexicon_tags_2 },
    { 0x5715af6d, SUC_lexicon_tags_14 },
    { 0xe8b5af6f, SUC_lexicon_tags_3 },
    { 0xf4d5af70, SUC_lexicon_tags_50 },
    { 0x7a95af70, SUC_lexicon_tags_1 },
    { 0x56f5af6f, SUC_lexicon_tags_3 },
    { 0xf6d5af73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6f5af75, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6275af77, SUC_lexicon_tags_15 },
    { 0xeff5af78, SUC_lexicon_tags_19 },
    { 0x15f5af79, SUC_lexicon_tags_30 },
    { 0xead5af77, SUC_lexicon_tags_21 },
    { 0xac75af7b, SUC_lexicon_tags_37 },
    { 0xf935af7c, SUC_lexicon_tags_19 },
    { 0xd9f5af7d, SUC_lexicon_tags_45 },
    { 0xa595af7e, SUC_lexicon_tags_12 },
    { 0x82d5af7f, SUC_lexicon_tags_106 },
    { 0xbe55af7c, SUC_lexicon_tags_8 },
    { 0x6135af79, SUC_lexicon_tags_11 },
    { 0x6175af82, SUC_lexicon_tags_7 },
    { 0x7a15af7e, SUC_lexicon_tags_4 },
    { 0xbfd5af84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5715af87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c75af8b, SUC_lexicon_tags_318 },
    { 0x9215af8c, SUC_lexicon_tags_17 },
    { 0xce35af8c, SUC_lexicon_tags_82 },
    { 0x275af8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b75af93, SUC_lexicon_tags_14 },
    { 0xc955af93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaa75af96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f5af99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e15af9d, SUC_lexicon_tags_1 },
    { 0x3d95af9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff5afa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb5afae, SUC_lexicon_tags_82 },
    { 0x8095afae, SUC_lexicon_tags_7 },
    { 0xf175afb0, SUC_lexicon_tags_60 },
    { 0xde35afb0, SUC_lexicon_tags_46 },
    { 0x3d75afb0, SUC_lexicon_tags_11 },
    { 0x9f5afb3, SUC_lexicon_tags_25 },
    { 0xb115afb4, SUC_lexicon_tags_49 },
    { 0x6275afb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef5afbe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf735afc7, SUC_lexicon_tags_45 },
    { 0x5ad5afc7, SUC_lexicon_tags_45 },
    { 0x4c75afc9, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1555afd0, SUC_lexicon_tags_82 },
    { 0xb9f5afd1, SUC_lexicon_tags_25 },
    { 0xadf5afd2, SUC_lexicon_tags_6 },
    { 0xdb55afd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8815afd5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a15afd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc895afdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3675afdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55afe2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf015afed, SUC_lexicon_tags_15 },
    { 0x5695afed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d5aff2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a75aff6, SUC_lexicon_tags_37 },
    { 0x70b5aff7, SUC_lexicon_tags_9 },
    { 0x8b75aff7, SUC_lexicon_tags_33 },
    { 0x1655aff7, SUC_lexicon_tags_2 },
    { 0x3555affa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5a35affc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4575b004, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xca75b006, SUC_lexicon_tags_25 },
    { 0xc75b007, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5e75b009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d5b00c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5075b010, SUC_lexicon_tags_19 },
    { 0xc2f5b011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9315b015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb675b01d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd475b01f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb875b021, SUC_lexicon_tags_11 },
    { 0x91f5b021, SUC_lexicon_tags_9 },
    { 0xad55b021, SUC_lexicon_tags_25 },
    { 0x3895b024, SUC_lexicon_tags_0 },
    { 0x32f5b023, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9695b02c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb275b02f, SUC_lexicon_tags_37 },
    { 0x8975b030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc015b034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3715b037, SUC_lexicon_tags_17 },
    { 0x6215b038, SUC_lexicon_tags_12 },
    { 0xb8f5b037, SUC_lexicon_tags_63 },
    { 0x7755b03a, SUC_lexicon_tags_17 },
    { 0xe7d5b03b, SUC_lexicon_tags_38 },
    { 0xd8d5b03c, SUC_lexicon_tags_25 },
    { 0xf775b03c, SUC_lexicon_tags_1 },
    { 0xe7f5b03c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x435b041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2995b046, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db5b04a, SUC_lexicon_tags_1 },
    { 0x9215b04b, SUC_lexicon_tags_9 },
    { 0x5235b04c, SUC_lexicon_tags_13 },
    { 0x935b04d, SUC_lexicon_tags_37 },
    { 0x8bd5b04e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc95b051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb5b054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e95b058, SUC_lexicon_tags_28 },
    { 0xf075b059, SUC_lexicon_tags_7 },
    { 0xb0f5b058, SUC_lexicon_tags_37 },
    { 0xb575b05b, SUC_lexicon_tags_31 },
    { 0xc935b058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5b05f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a95b064, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed35b067, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b15b069, SUC_lexicon_tags_37 },
    { 0xec55b06a, SUC_lexicon_tags_23 },
    { 0x9595b06b, SUC_lexicon_tags_28 },
    { 0xe6d5b069, SUC_lexicon_tags_31 },
    { 0x7895b06d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd75b070, SUC_lexicon_tags_32 },
    { 0xb655b071, SUC_lexicon_tags_0 },
    { 0xb9d5b072, SUC_lexicon_tags_1 },
    { 0x2bf5b071, SUC_lexicon_tags_60 },
    { 0x3e15b074, SUC_lexicon_tags_34 },
    { 0xf75b072, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355b07c, SUC_lexicon_tags_234 },
    { 0xf55b07d, SUC_lexicon_tags_15 },
    { 0x5935b07e, SUC_lexicon_tags_38 },
    { 0x4695b07f, SUC_lexicon_tags_35 },
    { 0x25d5b080, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff5b083, SUC_lexicon_tags_15 },
    { 0x7015b084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb55b086, SUC_lexicon_tags_33 },
    { 0xf115b087, SUC_lexicon_tags_79 },
    { 0xcbb5b087, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad5b08b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d35b098, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b09d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2835b09f, SUC_lexicon_tags_30 },
    { 0xcf55b0a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd75b0a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b5b0a8, SUC_lexicon_tags_19 },
    { 0x1535b0a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d5b0ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce35b0b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc435b0bd, SUC_lexicon_tags_8 },
    { 0x41d5b0bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f5b0c2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8155b0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x875b0c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf935b0cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb5b0d3, SUC_lexicon_tags_34 },
    { 0x49d5b0d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf395b0d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc495b0d9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb495b0df, SUC_lexicon_tags_14 },
    { 0x4f5b0e0, SUC_lexicon_tags_12 },
    { 0x3b5b0df, SUC_lexicon_tags_14 },
    { 0xabb5b0e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf215b0e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d5b0e9, SUC_lexicon_tags_1 },
    { 0xac35b0e9, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0xd175b0ec, SUC_lexicon_tags_17 },
    { 0x2d95b0ec, SUC_lexicon_tags_65 },
    { 0xba55b0ee, SUC_lexicon_tags_28 },
    { 0xc6b5b0ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f55b0f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8155b0f3, SUC_lexicon_tags_37 },
    { 0xce95b0f4, SUC_lexicon_tags_25 },
    { 0x9ff5b0f5, SUC_lexicon_tags_27 },
    { 0x5575b0f5, SUC_lexicon_tags_25 },
    { 0x5195b0f6, SUC_lexicon_tags_37 },
    { 0x4d15b0f5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e15b101, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4855b103, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf95b106, SUC_lexicon_tags_63 },
    { 0x5bd5b107, SUC_lexicon_tags_43 },
    { 0x1655b107, SUC_lexicon_tags_17 },
    { 0x8835b109, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa255b10b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2155b10f, SUC_lexicon_tags_307 },
    { 0xaa75b10f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe555b114, SUC_lexicon_tags_1 },
    { 0x94f5b115, SUC_lexicon_tags_15 },
    { 0x8875b116, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2335b118, SUC_lexicon_tags_25 },
    { 0x7d55b119, SUC_lexicon_tags_65 },
    { 0x6295b11a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c95b11d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c95b121, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b5b12c, SUC_lexicon_tags_27 },
    { 0xcff5b12c, SUC_lexicon_tags_37 },
    { 0xa675b12e, SUC_lexicon_tags_43 },
    { 0xe395b12f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d5b133, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb675b135, SUC_lexicon_tags_38 },
    { 0x2e15b136, SUC_lexicon_tags_8 },
    { 0x3035b136, SUC_lexicon_tags_63 },
    { 0xb55b138, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa675b13a, SUC_lexicon_tags_11 },
    { 0x79f5b13b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x28d5b13d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd695b141, SUC_lexicon_tags_12 },
    { 0x22f5b142, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc55b147, SUC_lexicon_tags_25 },
    { 0xfc15b148, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5eb5b14a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa875b14c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd455b151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd5b158, SUC_lexicon_tags_120 },
    { 0xdbb5b158, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2355b161, SUC_lexicon_tags_65 },
    { 0x2e55b162, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8135b164, SUC_lexicon_tags_17 },
    { 0x3b35b165, SUC_lexicon_tags_45 },
    { 0xaa55b164, SUC_lexicon_tags_13 },
    { 0xb375b165, SUC_lexicon_tags_26 },
    { 0x9875b168, SUC_lexicon_tags_16 },
    { 0xe215b169, SUC_lexicon_tags_28 },
    { 0x87b5b168, SUC_lexicon_tags_26 },
    { 0x87d5b16a, SUC_lexicon_tags_19 },
    { 0x1fd5b16c, SUC_lexicon_tags_9 },
    { 0xf3f5b168, SUC_lexicon_tags_5 },
    { 0x1c95b16e, SUC_lexicon_tags_17 },
    { 0xc975b165, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d5b174, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d5b179, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b182, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44d5b184, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e75b187, SUC_lexicon_tags_1 },
    { 0x81b5b188, SUC_lexicon_tags_7 },
    { 0xc5f5b189, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6215b18c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5b194, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe155b197, SUC_lexicon_tags_61 },
    { 0x9d95b198, SUC_lexicon_tags_12 },
    { 0x5615b199, SUC_lexicon_tags_49 },
    { 0xce75b19a, SUC_lexicon_tags_11 },
    { 0x5855b19b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af5b19e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6235b1a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb115b1a2, SUC_lexicon_tags_26 },
    { 0x8ad5b1a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xce35b1a5, SUC_lexicon_tags_66 },
    { 0xf935b1a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b5b1ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8915b1b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d5b1b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d5b1ba, SUC_lexicon_tags_1 },
    { 0xa9b5b1bb, SUC_lexicon_tags_43 },
    { 0x4f55b1ba, SUC_lexicon_tags_21 },
    { 0xb175b1bb, SUC_lexicon_tags_55 },
    { 0x6eb5b1be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7415b1c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6715b1ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf55b1ce, SUC_lexicon_tags_25 },
    { 0x7215b1cf, SUC_lexicon_tags_27 },
    { 0x9f15b1d0, SUC_lexicon_tags_33 },
    { 0xb075b1ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8755b1d3, SUC_lexicon_tags_45 },
    { 0xcf5b1d4, SUC_lexicon_tags_25 },
    { 0xd5d5b1d5, SUC_lexicon_tags_49 },
    { 0x8dd5b1d5, SUC_lexicon_tags_47 },
    { 0xe2b5b1d7, SUC_lexicon_tags_47 },
    { 0x5b75b1d6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3375b1dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e55b1e0, SUC_lexicon_tags_14 },
    { 0xf3f5b1e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc5d5b1e3, SUC_lexicon_tags_13 },
    { 0x20d5b1e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b55b1e7, SUC_lexicon_tags_8 },
    { 0xd8b5b1e7, SUC_lexicon_tags_41 },
    { 0x1955b1e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80f5b1eb, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a55b1ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4f5b1f1, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xe535b1f3, SUC_lexicon_tags_60 },
    { 0x6795b1f4, SUC_lexicon_tags_7 },
    { 0x9d5b1f5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f15b1fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15f5b1fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d5b206, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa4f5b208, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b20c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8b5b20e, SUC_lexicon_tags_23 },
    { 0xf75b20e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5b214, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315b21e, SUC_lexicon_tags_48 },
    { 0x7915b21f, SUC_lexicon_tags_8 },
    { 0x3415b220, SUC_lexicon_tags_11 },
    { 0xf435b221, SUC_lexicon_tags_38 },
    { 0xe915b222, SUC_lexicon_tags_15 },
    { 0x5995b223, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5b226, SUC_lexicon_tags_18 },
    { 0x31b5b226, SUC_lexicon_tags_25 },
    { 0xcfb5b228, SUC_lexicon_tags_84 },
    { 0xcf55b228, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f75b22d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9095b22f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b55b234, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d5b239, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd475b23e, SUC_lexicon_tags_15 },
    { 0x87f5b23f, SUC_lexicon_tags_15 },
    { 0xaff5b240, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b55b242, SUC_lexicon_tags_39 },
    { 0x81b5b243, SUC_lexicon_tags_0 },
    { 0x8055b243, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbbf5b246, SUC_lexicon_tags_25 },
    { 0xb315b246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe55b249, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa695b24c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f5b24f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb235b251, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d75b25a, SUC_lexicon_tags_55 },
    { 0xf875b25b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6375b25d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6e55b25f, SUC_lexicon_tags_1 },
    { 0xc095b260, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2875b262, SUC_lexicon_tags_45 },
    { 0xaf5b262, SUC_lexicon_tags_17 },
    { 0xed15b262, SUC_lexicon_tags_68 },
    { 0xc7f5b263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf55b26b, SUC_lexicon_tags_4 },
    { 0xf335b26b, SUC_lexicon_tags_15 },
    { 0x4935b26c, SUC_lexicon_tags_38 },
    { 0x5415b26e, SUC_lexicon_tags_198 },
    { 0x1555b26e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb0d5b271, SUC_lexicon_tags_11 },
    { 0x5975b272, SUC_lexicon_tags_25 },
    { 0x9ef5b273, SUC_lexicon_tags_37 },
    { 0x6755b273, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc175b276, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde75b279, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d5b27c, SUC_lexicon_tags_49 },
    { 0x7a55b27d, SUC_lexicon_tags_14 },
    { 0x51b5b27e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9255b280, SUC_lexicon_tags_1 },
    { 0x98d5b281, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3435b285, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8955b288, SUC_lexicon_tags_17 },
    { 0xb0b5b289, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x275b28b, SUC_lexicon_tags_104 },
    { 0xcd15b28b, SUC_lexicon_tags_47 },
    { 0xc355b28c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7315b293, SUC_lexicon_tags_60 },
    { 0x75b5b294, SUC_lexicon_tags_45 },
    { 0xcf5b294, SUC_lexicon_tags_34 },
    { 0xe515b296, SUC_lexicon_tags_12 },
    { 0xc2b5b296, SUC_lexicon_tags_65 },
    { 0xce55b298, SUC_lexicon_tags_25 },
    { 0x5855b298, SUC_lexicon_tags_25 },
    { 0x9ad5b29a, SUC_lexicon_tags_87 },
    { 0xf515b29b, SUC_lexicon_tags_14 },
    { 0xcc55b29b, SUC_lexicon_tags_92 },
    { 0x2875b29d, SUC_lexicon_tags_0 },
    { 0x7495b29b, SUC_lexicon_tags_11 },
    { 0x2c15b29f, SUC_lexicon_tags_11 },
    { 0xebf5b297, SUC_lexicon_tags_17 },
    { 0xac75b2a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe695b2a3, SUC_lexicon_tags_6 },
    { 0x92d5b2a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c15b2a7, SUC_lexicon_tags_30 },
    { 0x23f5b2a8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d5b2ab, SUC_lexicon_tags_31 },
    { 0x47f5b2ac, SUC_lexicon_tags_17 },
    { 0xeff5b2ad, SUC_lexicon_tags_29 },
    { 0x1355b2ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d5b2b4, SUC_lexicon_tags_31 },
    { 0xd355b2b5, SUC_lexicon_tags_6 },
    { 0xe455b2b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x49d5b2b8, SUC_lexicon_tags_9 },
    { 0xc875b2b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9575b2bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x575b2c0, SUC_lexicon_tags_45 },
    { 0xfb35b2c1, SUC_lexicon_tags_34 },
    { 0x9335b2c2, SUC_lexicon_tags_35 },
    { 0xed5b2c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3455b2c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa235b2c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8275b2ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d5b2d0, SUC_lexicon_tags_40 },
    { 0xb895b2d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf075b2d3, SUC_lexicon_tags_2 },
    { 0xbc75b2d4, SUC_lexicon_tags_27 },
    { 0x2335b2d4, SUC_lexicon_tags_6 },
    { 0x49b5b2d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9775b2d8, SUC_lexicon_tags_7 },
    { 0x70b5b2d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b55b2de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b5b2e3, SUC_lexicon_tags_0 },
    { 0x46f5b2e4, SUC_lexicon_tags_7 },
    { 0xb755b2e4, SUC_lexicon_tags_35 },
    { 0x675b2e3, SUC_lexicon_tags_25 },
    { 0xb175b2e3, SUC_lexicon_tags_25 },
    { 0xd075b2e8, SUC_lexicon_tags_25 },
    { 0xb695b2e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b5b2ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1455b2f6, SUC_lexicon_tags_11 },
    { 0x98d5b2f7, SUC_lexicon_tags_12 },
    { 0x995b2f8, SUC_lexicon_tags_6 },
    { 0xfa15b2f9, SUC_lexicon_tags_30 },
    { 0xbf75b2f8, SUC_lexicon_tags_66 },
    { 0x2935b2f9, SUC_lexicon_tags_47 },
    { 0x58d5b2fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d5b300, SUC_lexicon_tags_0 },
    { 0x43f5b301, SUC_lexicon_tags_19 },
    { 0xcf15b300, SUC_lexicon_tags_25 },
    { 0x1635b303, SUC_lexicon_tags_48 },
    { 0x5895b304, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5b307, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb5b30e, SUC_lexicon_tags_21 },
    { 0x7f75b30f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a95b315, SUC_lexicon_tags_44 },
    { 0xbb95b315, SUC_lexicon_tags_23 },
    { 0x6495b316, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d35b31b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd475b31f, SUC_lexicon_tags_63 },
    { 0x8b5b31f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3235b324, SUC_lexicon_tags_17 },
    { 0xc115b325, SUC_lexicon_tags_29 },
    { 0x715b326, SUC_lexicon_tags_7 },
    { 0x3e15b326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f5b32a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b15b32d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d75b330, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f35b332, SUC_lexicon_tags_57 },
    { 0xda75b332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6075b336, SUC_lexicon_tags_7 },
    { 0x84d5b336, SUC_lexicon_tags_9 },
    { 0x9635b336, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf555b33a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf515b33f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa435b341, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba75b346, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd5b34a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb5b34f, SUC_lexicon_tags_30 },
    { 0x9055b34f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3135b352, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3135b357, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfcf5b359, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1335b35e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63f5b360, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed5b365, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3115b367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde35b369, SUC_lexicon_tags_17 },
    { 0xeb55b36a, SUC_lexicon_tags_63 },
    { 0x63d5b36a, SUC_lexicon_tags_28 },
    { 0x4d95b36c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa55b376, SUC_lexicon_tags_12 },
    { 0xc115b377, SUC_lexicon_tags_60 },
    { 0x57f5b378, SUC_lexicon_tags_37 },
    { 0xf415b379, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf575b37b, SUC_lexicon_tags_25 },
    { 0xeeb5b37c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d55b383, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe535b385, SUC_lexicon_tags_48 },
    { 0xa735b386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bb5b388, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2015b38a, SUC_lexicon_tags_87 },
    { 0xdeb5b38a, SUC_lexicon_tags_27 },
    { 0xb995b38b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5a35b38e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5b391, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa235b393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15b5b395, SUC_lexicon_tags_17 },
    { 0x7255b396, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ad5b398, SUC_lexicon_tags_38 },
    { 0xff15b399, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b15b39d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5375b3a0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad5b3a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bb5b3a6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6d75b3a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb95b3ac, SUC_lexicon_tags_31 },
    { 0xed95b3ad, SUC_lexicon_tags_37 },
    { 0xbe55b3ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f5b3b2, SUC_lexicon_tags_7 },
    { 0xc7b5b3b3, SUC_lexicon_tags_19 },
    { 0xb1f5b3b3, SUC_lexicon_tags_14 },
    { 0x86f5b3b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe495b3b7, SUC_lexicon_tags_7 },
    { 0xcbd5b3b8, SUC_lexicon_tags_25 },
    { 0xd455b3b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6e15b3bb, SUC_lexicon_tags_2 },
    { 0x2635b3bc, SUC_lexicon_tags_4 },
    { 0xc375b3bb, SUC_lexicon_tags_33 },
    { 0xa175b3be, SUC_lexicon_tags_14 },
    { 0x76b5b3bf, SUC_lexicon_tags_33 },
    { 0xea35b3c0, SUC_lexicon_tags_27 },
    { 0xe015b3c1, SUC_lexicon_tags_13 },
    { 0x81b5b3c2, SUC_lexicon_tags_10 },
    { 0xbb35b3c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9555b3c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x935b3c9, SUC_lexicon_tags_57 },
    { 0xe7b5b3c9, SUC_lexicon_tags_25 },
    { 0x2f5b3cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b5b3d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f5b3d3, SUC_lexicon_tags_35 },
    { 0x12d5b3d3, SUC_lexicon_tags_33 },
    { 0x8b95b3d5, SUC_lexicon_tags_66 },
    { 0xe015b3d6, SUC_lexicon_tags_14 },
    { 0x6555b3d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5975b3d9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd915b3df, SUC_lexicon_tags_25 },
    { 0xdc75b3e0, SUC_lexicon_tags_15 },
    { 0x7315b3e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa755b3e4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed5b3e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c35b3f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4055b3f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9535b3f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a15b3f8, SUC_lexicon_tags_79 },
    { 0x6d15b3f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b5b404, SUC_lexicon_tags_17 },
    { 0xf535b404, SUC_lexicon_tags_25 },
    { 0xdbf5b405, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7335b409, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1f5b40b, SUC_lexicon_tags_47 },
    { 0x6e95b40c, SUC_lexicon_tags_47 },
    { 0x1f15b40b, SUC_lexicon_tags_7 },
    { 0xee75b40c, SUC_lexicon_tags_4 },
    { 0xc795b40f, SUC_lexicon_tags_11 },
    { 0x59d5b40c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc35b414, SUC_lexicon_tags_25 },
    { 0x3f55b415, SUC_lexicon_tags_29 },
    { 0xff75b416, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf995b41b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef35b422, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4575b424, SUC_lexicon_tags_70 },
    { 0x3ef5b425, SUC_lexicon_tags_9 },
    { 0x4095b425, SUC_lexicon_tags_35 },
    { 0x4b75b426, SUC_lexicon_tags_60 },
    { 0x4ad5b428, SUC_lexicon_tags_25 },
    { 0x6275b429, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5b430, SUC_lexicon_tags_17 },
    { 0x17d5b431, SUC_lexicon_tags_25 },
    { 0xd3b5b432, SUC_lexicon_tags_30 },
    { 0x4395b432, SUC_lexicon_tags_25 },
    { 0xccd5b430, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55b437, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe735b439, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc635b43f, SUC_lexicon_tags_37 },
    { 0x1715b440, SUC_lexicon_tags_1 },
    { 0x6135b441, SUC_lexicon_tags_12 },
    { 0x7a75b440, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa075b446, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2675b448, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d5b450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2eb5b452, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xd155b454, SUC_lexicon_tags_27 },
    { 0x3615b455, SUC_lexicon_tags_12 },
    { 0x9435b456, SUC_lexicon_tags_25 },
    { 0xe835b457, SUC_lexicon_tags_17 },
    { 0x4fb5b457, SUC_lexicon_tags_60 },
    { 0x9ef5b457, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf755b45b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d55b45d, SUC_lexicon_tags_55 },
    { 0xe475b45e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf55b461, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4835b466, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ad5b468, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8895b46a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af5b46d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe295b46f, SUC_lexicon_tags_0 },
    { 0x9475b470, SUC_lexicon_tags_11 },
    { 0x1355b471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb5b474, SUC_lexicon_tags_82 },
    { 0xb8d5b475, SUC_lexicon_tags_25 },
    { 0x8b95b476, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5715b479, SUC_lexicon_tags_49 },
    { 0xcbd5b47a, SUC_lexicon_tags_45 },
    { 0xa8d5b47a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4df5b47d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf5b487, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f5b48a, SUC_lexicon_tags_4 },
    { 0x3395b48a, SUC_lexicon_tags_46 },
    { 0x90f5b48c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b95b48f, SUC_lexicon_tags_63 },
    { 0x3f55b490, SUC_lexicon_tags_29 },
    { 0x8d75b490, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcbd5b493, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac95b497, SUC_lexicon_tags_9 },
    { 0x1155b498, SUC_lexicon_tags_57 },
    { 0xf115b497, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc55b49d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x5455b4a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe75b4a3, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0xb775b4a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b5b4ab, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x2f15b4ad, SUC_lexicon_tags_37 },
    { 0x2b35b4ad, SUC_lexicon_tags_15 },
    { 0x1355b4af, SUC_lexicon_tags_11 },
    { 0xa975b4b0, SUC_lexicon_tags_3 },
    { 0xf655b4b1, SUC_lexicon_tags_7 },
    { 0x3135b4b1, SUC_lexicon_tags_32 },
    { 0x815b4b1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xab15b4b5, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x9295b4b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe95b4bc, SUC_lexicon_tags_39 },
    { 0xf135b4bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b15b4c0, SUC_lexicon_tags_33 },
    { 0x79b5b4c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac75b4c4, SUC_lexicon_tags_17 },
    { 0xa155b4c5, SUC_lexicon_tags_16 },
    { 0xbc15b4c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9975b4ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c15b4d0, SUC_lexicon_tags_10 },
    { 0xe095b4d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc815b4d5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9475b4d7, SUC_lexicon_tags_124 },
    { 0x2495b4d7, SUC_lexicon_tags_8 },
    { 0x54d5b4d9, SUC_lexicon_tags_62 },
    { 0x7815b4da, SUC_lexicon_tags_35 },
    { 0x76d5b4d9, SUC_lexicon_tags_17 },
    { 0xd855b4da, SUC_lexicon_tags_1 },
    { 0xb555b4dc, SUC_lexicon_tags_49 },
    { 0xfd35b4de, SUC_lexicon_tags_43 },
    { 0x2795b4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd5b4e3, SUC_lexicon_tags_30 },
    { 0x7ed5b4e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8315b4e7, SUC_lexicon_tags_30 },
    { 0x74f5b4e8, SUC_lexicon_tags_43 },
    { 0x3235b4e9, SUC_lexicon_tags_12 },
    { 0x3b95b4ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2435b4f4, SUC_lexicon_tags_12 },
    { 0xb855b4f5, SUC_lexicon_tags_37 },
    { 0x8ff5b4f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9315b4fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb5b502, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd95b507, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e55b509, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b5b50c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b5b511, SUC_lexicon_tags_79 },
    { 0xd755b511, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7635b514, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4295b518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6435b524, SUC_lexicon_tags_1 },
    { 0x795b525, SUC_lexicon_tags_11 },
    { 0x635b526, SUC_lexicon_tags_37 },
    { 0x2755b527, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa055b52c, SUC_lexicon_tags_30 },
    { 0xc6f5b52d, SUC_lexicon_tags_28 },
    { 0x1075b52e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa035b532, SUC_lexicon_tags_31 },
    { 0x77f5b533, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7915b539, SUC_lexicon_tags_31 },
    { 0x3575b53a, SUC_lexicon_tags_11 },
    { 0xa3f5b53b, SUC_lexicon_tags_47 },
    { 0xd475b53a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af5b53f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec15b541, SUC_lexicon_tags_48 },
    { 0x44f5b542, SUC_lexicon_tags_11 },
    { 0xc815b543, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa415b548, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2915b54f, SUC_lexicon_tags_60 },
    { 0x94d5b54f, SUC_lexicon_tags_29 },
    { 0x5d75b551, SUC_lexicon_tags_22 },
    { 0x9975b552, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf5b558, SUC_lexicon_tags_4 },
    { 0xfdf5b558, SUC_lexicon_tags_37 },
    { 0x7d35b55a, SUC_lexicon_tags_56 },
    { 0xd415b55a, SUC_lexicon_tags_12 },
    { 0xc2b5b55a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe095b55e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x795b561, SUC_lexicon_tags_27 },
    { 0xc435b562, SUC_lexicon_tags_7 },
    { 0x4055b563, SUC_lexicon_tags_2 },
    { 0xde35b564, SUC_lexicon_tags_45 },
    { 0x8f5b561, SUC_lexicon_tags_37 },
    { 0xed75b565, SUC_lexicon_tags_1 },
    { 0x85d5b567, SUC_lexicon_tags_7 },
    { 0xed55b568, SUC_lexicon_tags_37 },
    { 0xeff5b569, SUC_lexicon_tags_47 },
    { 0x2f55b565, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f5b574, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b577, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa355b581, SUC_lexicon_tags_37 },
    { 0xcdd5b581, SUC_lexicon_tags_37 },
    { 0x9eb5b583, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1e15b585, SUC_lexicon_tags_17 },
    { 0xc0b5b586, SUC_lexicon_tags_40 },
    { 0xa755b587, SUC_lexicon_tags_43 },
    { 0x76d5b588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2755b58a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x1995b58d, SUC_lexicon_tags_14 },
    { 0x4275b58e, SUC_lexicon_tags_38 },
    { 0xd5b58f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9f5b591, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd475b599, SUC_lexicon_tags_40 },
    { 0x4615b59a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d75b59e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c95b5a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb695b5a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b5b5a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2875b5ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2575b5ae, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1555b5b0, SUC_lexicon_tags_60 },
    { 0x1a55b5b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa595b5b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5515b5b5, SUC_lexicon_tags_9 },
    { 0x1bb5b5b5, SUC_lexicon_tags_2 },
    { 0x175b5b7, SUC_lexicon_tags_84 },
    { 0xc235b5b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad5b5bb, SUC_lexicon_tags_17 },
    { 0x6a35b5bb, SUC_lexicon_tags_14 },
    { 0x26d5b5bb, SUC_lexicon_tags_37 },
    { 0x8695b5be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8515b5c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd9f5b5c2, SUC_lexicon_tags_37 },
    { 0xc8d5b5c3, SUC_lexicon_tags_69 },
    { 0xff55b5c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b5b5ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b5b5d2, SUC_lexicon_tags_4 },
    { 0x38d5b5d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50d5b5d5, SUC_lexicon_tags_17 },
    { 0xd0f5b5d6, SUC_lexicon_tags_15 },
    { 0xe595b5d7, SUC_lexicon_tags_49 },
    { 0x4ab5b5d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b5b5dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa135b5e1, SUC_lexicon_tags_1 },
    { 0x9855b5e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3895b5e4, SUC_lexicon_tags_62 },
    { 0xd695b5e4, SUC_lexicon_tags_16 },
    { 0x8bd5b5e6, SUC_lexicon_tags_27 },
    { 0xfa15b5e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd55b5ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b95b5ec, SUC_lexicon_tags_25 },
    { 0x3515b5ec, SUC_lexicon_tags_55 },
    { 0x4bf5b5ec, SUC_lexicon_tags_1 },
    { 0x60d5b5ec, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b15b5f4, SUC_lexicon_tags_14 },
    { 0x52b5b5f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd15b5f7, SUC_lexicon_tags_43 },
    { 0x7f55b5f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6d5b5fa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x675b5fd, SUC_lexicon_tags_255 },
    { 0x8f35b5fe, SUC_lexicon_tags_11 },
    { 0xcbf5b5ff, SUC_lexicon_tags_43 },
    { 0xe1b5b5ff, SUC_lexicon_tags_10 },
    { 0x8f55b601, SUC_lexicon_tags_44 },
    { 0x5155b600, SUC_lexicon_tags_31 },
    { 0x72f5b602, SUC_lexicon_tags_8 },
    { 0xdbd5b604, SUC_lexicon_tags_29 },
    { 0xbcd5b604, SUC_lexicon_tags_2 },
    { 0xa235b5ff, SUC_lexicon_tags_27 },
    { 0x56b5b5fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3775b60a, SUC_lexicon_tags_10 },
    { 0x2a15b60b, SUC_lexicon_tags_63 },
    { 0xe835b60a, SUC_lexicon_tags_17 },
    { 0x2615b60d, SUC_lexicon_tags_27 },
    { 0x5535b60d, SUC_lexicon_tags_48 },
    { 0x6595b60c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc335b614, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad5b61e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7375b622, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x455b625, SUC_lexicon_tags_12 },
    { 0x4b15b626, SUC_lexicon_tags_25 },
    { 0xd055b627, SUC_lexicon_tags_66 },
    { 0x91f5b627, SUC_lexicon_tags_30 },
    { 0x415b625, SUC_lexicon_tags_4 },
    { 0x5395b628, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f5b62d, SUC_lexicon_tags_60 },
    { 0xe235b62d, SUC_lexicon_tags_31 },
    { 0x95b62f, SUC_lexicon_tags_25 },
    { 0xda75b62d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f75b633, SUC_lexicon_tags_47 },
    { 0xf5d5b634, SUC_lexicon_tags_44 },
    { 0xe315b635, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f5b63a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb295b63c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e75b641, SUC_lexicon_tags_70 },
    { 0x27d5b642, SUC_lexicon_tags_61 },
    { 0x4695b642, SUC_lexicon_tags_65 },
    { 0x14f5b644, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e35b647, SUC_lexicon_tags_43 },
    { 0x3315b647, SUC_lexicon_tags_11 },
    { 0x3575b649, SUC_lexicon_tags_25 },
    { 0x3c35b64a, SUC_lexicon_tags_11 },
    { 0x3af5b64a, SUC_lexicon_tags_48 },
    { 0x895b64b, SUC_lexicon_tags_11 },
    { 0xee55b647, SUC_lexicon_tags_15 },
    { 0x23d5b64e, SUC_lexicon_tags_2 },
    { 0x9f55b64f, SUC_lexicon_tags_11 },
    { 0xde55b64f, SUC_lexicon_tags_49 },
    { 0x8175b647, SUC_lexicon_tags_37 },
    { 0x4d35b652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55b655, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f5b65c, SUC_lexicon_tags_37 },
    { 0x63b5b65d, SUC_lexicon_tags_1 },
    { 0x55b5b65d, SUC_lexicon_tags_7 },
    { 0xacf5b65f, SUC_lexicon_tags_84 },
    { 0xa855b65e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3df5b662, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1835b665, SUC_lexicon_tags_17 },
    { 0x84d5b665, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8835b66b, SUC_lexicon_tags_27 },
    { 0xfdd5b66c, SUC_lexicon_tags_262 },
    { 0xfaf5b66d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f5b671, SUC_lexicon_tags_38 },
    { 0xa135b672, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35b677, SUC_lexicon_tags_45 },
    { 0xbed5b678, SUC_lexicon_tags_17 },
    { 0xe9d5b679, SUC_lexicon_tags_25 },
    { 0x24d5b677, SUC_lexicon_tags_1 },
    { 0xacd5b67b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b15b67d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f5b681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b5b684, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55b5b686, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7db5b688, SUC_lexicon_tags_33 },
    { 0x2335b689, SUC_lexicon_tags_17 },
    { 0x85d5b689, SUC_lexicon_tags_13 },
    { 0x7e35b68b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d5b693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec35b697, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9555b699, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf95b69b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff5b69e, SUC_lexicon_tags_55 },
    { 0xc955b69e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6075b6a1, SUC_lexicon_tags_1360 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d95b6a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe175b6a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfcf5b6ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c15b6ad, SUC_lexicon_tags_17 },
    { 0xab95b6ae, SUC_lexicon_tags_13 },
    { 0xdb75b6ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7735b6b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf075b6b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4475b6bd, SUC_lexicon_tags_40 },
    { 0xf595b6be, SUC_lexicon_tags_48 },
    { 0xf175b6bf, SUC_lexicon_tags_27 },
    { 0x4e55b6be, SUC_lexicon_tags_25 },
    { 0x35d5b6c1, SUC_lexicon_tags_25 },
    { 0x3a75b6c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa355b6c6, SUC_lexicon_tags_11 },
    { 0x5f35b6c6, SUC_lexicon_tags_7 },
    { 0x6195b6c8, SUC_lexicon_tags_17 },
    { 0xc275b6c7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f5b6cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3d5b6cf, SUC_lexicon_tags_7 },
    { 0xbb95b6d0, SUC_lexicon_tags_74 },
    { 0x4915b6d1, SUC_lexicon_tags_1 },
    { 0x8d75b6d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f15b6da, SUC_lexicon_tags_25 },
    { 0x8d5b6db, SUC_lexicon_tags_11 },
    { 0x9595b6db, SUC_lexicon_tags_69 },
    { 0x9c35b6dc, SUC_lexicon_tags_57 },
    { 0xca55b6db, SUC_lexicon_tags_2 },
    { 0x2835b6df, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x6355b6e1, SUC_lexicon_tags_30 },
    { 0x9355b6e2, SUC_lexicon_tags_16 },
    { 0xa75b6e2, SUC_lexicon_tags_19 },
    { 0x8675b6e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b95b6e6, SUC_lexicon_tags_25 },
    { 0xe6b5b6e7, SUC_lexicon_tags_15 },
    { 0xc215b6e7, SUC_lexicon_tags_33 },
    { 0x55b5b6e9, SUC_lexicon_tags_4 },
    { 0xbe95b6ea, SUC_lexicon_tags_12 },
    { 0xfc55b6eb, SUC_lexicon_tags_27 },
    { 0xb5d5b6e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95b6f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae55b6f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe375b6fb, SUC_lexicon_tags_12 },
    { 0x78f5b6fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd955b700, SUC_lexicon_tags_17 },
    { 0x1e55b701, SUC_lexicon_tags_63 },
    { 0xf1b5b702, SUC_lexicon_tags_68 },
    { 0xdf55b703, SUC_lexicon_tags_17 },
    { 0xe3f5b701, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x9855b707, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a95b70d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5f35b70f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7475b711, SUC_lexicon_tags_17 },
    { 0x2ff5b712, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb15b714, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb5b719, SUC_lexicon_tags_32 },
    { 0x9335b719, SUC_lexicon_tags_17 },
    { 0x7215b71a, SUC_lexicon_tags_31 },
    { 0x1a95b71c, SUC_lexicon_tags_11 },
    { 0x4675b71b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6675b71f, SUC_lexicon_tags_30 },
    { 0x9915b720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bb5b722, SUC_lexicon_tags_38 },
    { 0x9995b723, SUC_lexicon_tags_57 },
    { 0xcbb5b723, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc675b729, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f5b72e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfe95b730, SUC_lexicon_tags_37 },
    { 0x5f35b731, SUC_lexicon_tags_12 },
    { 0xdfb5b730, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4b95b734, SUC_lexicon_tags_15 },
    { 0x21b5b735, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f5b738, SUC_lexicon_tags_11 },
    { 0x8a55b739, SUC_lexicon_tags_15 },
    { 0x5d5b73a, SUC_lexicon_tags_37 },
    { 0xa895b738, SUC_lexicon_tags_37 },
    { 0x6555b739, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f5b741, SUC_lexicon_tags_11 },
    { 0x8615b742, SUC_lexicon_tags_11 },
    { 0x3d15b743, SUC_lexicon_tags_28 },
    { 0x4735b744, SUC_lexicon_tags_15 },
    { 0xc7d5b744, SUC_lexicon_tags_22 },
    { 0x4d5b742, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf95b748, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d15b74c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6955b74e, SUC_lexicon_tags_43 },
    { 0xc815b74e, SUC_lexicon_tags_2 },
    { 0x3775b74e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9715b752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7315b755, SUC_lexicon_tags_19 },
    { 0xf215b756, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc655b758, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d5b75b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3255b75d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc95b762, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d5b766, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5055b768, SUC_lexicon_tags_2 },
    { 0xf815b769, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x60f5b76b, SUC_lexicon_tags_33 },
    { 0x8175b76c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f5b76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f5b774, SUC_lexicon_tags_28 },
    { 0xce35b774, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xed35b778, SUC_lexicon_tags_19 },
    { 0xdaf5b779, SUC_lexicon_tags_12 },
    { 0x6d5b77a, SUC_lexicon_tags_15 },
    { 0x34d5b779, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2315b780, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd395b786, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a35b789, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebd5b78b, SUC_lexicon_tags_19 },
    { 0x6f5b78c, SUC_lexicon_tags_7 },
    { 0x6eb5b78b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735b796, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb735b799, SUC_lexicon_tags_25 },
    { 0x3195b799, SUC_lexicon_tags_11 },
    { 0x7b55b79b, SUC_lexicon_tags_17 },
    { 0x9875b799, SUC_lexicon_tags_692 },
    { 0xb115b79d, SUC_lexicon_tags_37 },
    { 0x1c35b79d, SUC_lexicon_tags_27 },
    { 0x76d5b799, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd75b7a2, SUC_lexicon_tags_4 },
    { 0x76b5b7a3, SUC_lexicon_tags_63 },
    { 0x86f5b7a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255b7a8, SUC_lexicon_tags_87 },
    { 0x7a55b7a9, SUC_lexicon_tags_1 },
    { 0x30b5b7aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf15b7ac, SUC_lexicon_tags_1 },
    { 0xac75b7ad, SUC_lexicon_tags_17 },
    { 0xd9f5b7ae, SUC_lexicon_tags_151 },
    { 0x1875b7af, SUC_lexicon_tags_3 },
    { 0x4e95b7b0, SUC_lexicon_tags_65 },
    { 0x6b55b7b0, SUC_lexicon_tags_44 },
    { 0x6115b7b2, SUC_lexicon_tags_17 },
    { 0x20f5b7ae, SUC_lexicon_tags_37 },
    { 0xab35b7b3, SUC_lexicon_tags_47 },
    { 0x7655b7b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2875b7b7, SUC_lexicon_tags_61 },
    { 0x9155b7b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b5b7bb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe155b7bd, SUC_lexicon_tags_6 },
    { 0xc535b7bd, SUC_lexicon_tags_2 },
    { 0x55f5b7bf, SUC_lexicon_tags_10 },
    { 0xf2f5b7bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5b7c5, SUC_lexicon_tags_1 },
    { 0xef35b7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3015b7c9, SUC_lexicon_tags_3 },
    { 0x42d5b7ca, SUC_lexicon_tags_25 },
    { 0x3c35b7cb, SUC_lexicon_tags_49 },
    { 0x3655b7cc, SUC_lexicon_tags_16 },
    { 0x415b7ca, SUC_lexicon_tags_63 },
    { 0xc275b7cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a35b7d0, SUC_lexicon_tags_9 },
    { 0x5455b7d1, SUC_lexicon_tags_6 },
    { 0x67b5b7d2, SUC_lexicon_tags_60 },
    { 0x6675b7d2, SUC_lexicon_tags_37 },
    { 0x2435b7d4, SUC_lexicon_tags_25 },
    { 0x4595b7d1, SUC_lexicon_tags_17 },
    { 0x8d55b7d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7635b7db, SUC_lexicon_tags_15 },
    { 0x9535b7db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b7e0, SUC_lexicon_tags_0 },
    { 0xf3b5b7e1, SUC_lexicon_tags_34 },
    { 0x55d5b7e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3d5b7e4, SUC_lexicon_tags_25 },
    { 0x3255b7e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83d5b7e7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb995b7e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6075b7f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b35b7fc, SUC_lexicon_tags_12 },
    { 0x6f75b7fd, SUC_lexicon_tags_117 },
    { 0x1735b7fc, SUC_lexicon_tags_15 },
    { 0x2955b7fd, SUC_lexicon_tags_27 },
    { 0x2315b800, SUC_lexicon_tags_33 },
    { 0x1c75b801, SUC_lexicon_tags_12 },
    { 0xe015b801, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5415b804, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f5b809, SUC_lexicon_tags_103 },
    { 0x9275b80a, SUC_lexicon_tags_12 },
    { 0x2e35b809, SUC_lexicon_tags_91 },
    { 0x79f5b809, SUC_lexicon_tags_12 },
    { 0x8915b80d, SUC_lexicon_tags_11 },
    { 0x5f95b809, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2995b811, SUC_lexicon_tags_45 },
    { 0xcb75b811, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c95b814, SUC_lexicon_tags_34 },
    { 0x36d5b815, SUC_lexicon_tags_10 },
    { 0xcb5b816, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc015b818, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f75b81c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf9d5b81e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9cb5b820, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa55b823, SUC_lexicon_tags_11 },
    { 0x1855b824, SUC_lexicon_tags_3 },
    { 0xbf5b824, SUC_lexicon_tags_37 },
    { 0x18b5b824, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xff55b828, SUC_lexicon_tags_25 },
    { 0x7f15b828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f75b82e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d5b839, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc275b83c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d5b840, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4615b842, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9415b845, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d5b848, SUC_lexicon_tags_13 },
    { 0x34b5b849, SUC_lexicon_tags_38 },
    { 0xe4f5b84a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x53d5b84c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb5b850, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f95b854, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe15b856, SUC_lexicon_tags_11 },
    { 0x3715b857, SUC_lexicon_tags_15 },
    { 0xce55b858, SUC_lexicon_tags_63 },
    { 0x8335b857, SUC_lexicon_tags_29 },
    { 0x8095b85a, SUC_lexicon_tags_8 },
    { 0xd4f5b85b, SUC_lexicon_tags_23 },
    { 0xb075b85c, SUC_lexicon_tags_30 },
    { 0x9f75b85c, SUC_lexicon_tags_8 },
    { 0xa7f5b857, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed5b861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f55b863, SUC_lexicon_tags_7 },
    { 0x6fb5b863, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb215b868, SUC_lexicon_tags_49 },
    { 0xf895b868, SUC_lexicon_tags_21 },
    { 0x44b5b86a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e95b86c, SUC_lexicon_tags_17 },
    { 0x1275b86c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8955b86f, SUC_lexicon_tags_30 },
    { 0xbf55b86f, SUC_lexicon_tags_2 },
    { 0x415b871, SUC_lexicon_tags_2 },
    { 0x15b5b872, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb155b875, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b35b879, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b87d, SUC_lexicon_tags_11 },
    { 0x8e35b87e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4875b886, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x8315b889, SUC_lexicon_tags_17 },
    { 0x36f5b88a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1755b88c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafd5b88e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c75b890, SUC_lexicon_tags_16 },
    { 0x2a95b891, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb895b893, SUC_lexicon_tags_27 },
    { 0x9995b894, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd935b89b, SUC_lexicon_tags_43 },
    { 0xb6d5b89c, SUC_lexicon_tags_45 },
    { 0xb6d5b89b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa075b89f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a55b8a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8235b8aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3275b8ac, SUC_lexicon_tags_17 },
    { 0x43f5b8ad, SUC_lexicon_tags_30 },
    { 0x6755b8ae, SUC_lexicon_tags_11 },
    { 0x2055b8ad, SUC_lexicon_tags_7 },
    { 0xd935b8b0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb935b8b6, SUC_lexicon_tags_36 },
    { 0xc955b8b6, SUC_lexicon_tags_26 },
    { 0x3755b8b8, SUC_lexicon_tags_0 },
    { 0xd1f5b8b9, SUC_lexicon_tags_61 },
    { 0x7d5b8b8, SUC_lexicon_tags_1 },
    { 0x9915b8bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6c55b8bd, SUC_lexicon_tags_23 },
    { 0x23f5b8be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xab5b8c0, SUC_lexicon_tags_11 },
    { 0x32d5b8c0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2935b8c3, SUC_lexicon_tags_7 },
    { 0xe3f5b8c4, SUC_lexicon_tags_27 },
    { 0xf2d5b8c5, SUC_lexicon_tags_21 },
    { 0x1f15b8c3, SUC_lexicon_tags_1 },
    { 0x2415b8c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff15b8cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb55b8d4, SUC_lexicon_tags_1 },
    { 0x8a75b8d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d35b8d8, SUC_lexicon_tags_25 },
    { 0xfc95b8d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x83d5b8db, SUC_lexicon_tags_26 },
    { 0x9cf5b8dc, SUC_lexicon_tags_45 },
    { 0x4395b8dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9095b8df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895b8e4, SUC_lexicon_tags_6 },
    { 0xf255b8e4, SUC_lexicon_tags_62 },
    { 0xbf95b8e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc915b8eb, SUC_lexicon_tags_30 },
    { 0xb2d5b8eb, SUC_lexicon_tags_7 },
    { 0xbb5b8ec, SUC_lexicon_tags_48 },
    { 0x2715b8ee, SUC_lexicon_tags_11 },
    { 0x4ef5b8ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc295b8f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc075b8f3, SUC_lexicon_tags_37 },
    { 0x3ab5b8f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8dd5b8f6, SUC_lexicon_tags_1 },
    { 0xda15b8f7, SUC_lexicon_tags_0 },
    { 0xa235b8f7, SUC_lexicon_tags_37 },
    { 0x4955b8f8, SUC_lexicon_tags_17 },
    { 0x7855b8f8, SUC_lexicon_tags_29 },
    { 0xbfb5b8fb, SUC_lexicon_tags_87 },
    { 0x27d5b8fc, SUC_lexicon_tags_15 },
    { 0x8195b8fc, SUC_lexicon_tags_11 },
    { 0xd4f5b8fe, SUC_lexicon_tags_2 },
    { 0x53d5b8ff, SUC_lexicon_tags_65 },
    { 0x9af5b8fc, SUC_lexicon_tags_6 },
    { 0x4fb5b901, SUC_lexicon_tags_17 },
    { 0x4f5b8fd, SUC_lexicon_tags_1 },
    { 0x6ed5b8ff, SUC_lexicon_tags_7 },
    { 0x52b5b904, SUC_lexicon_tags_47 },
    { 0xc675b905, SUC_lexicon_tags_7 },
    { 0xb6f5b906, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d5b909, SUC_lexicon_tags_37 },
    { 0xf4b5b90a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e75b910, SUC_lexicon_tags_25 },
    { 0xf95b911, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c75b918, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd5b91b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1775b91e, SUC_lexicon_tags_37 },
    { 0x48b5b91f, SUC_lexicon_tags_27 },
    { 0x5c15b91e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5275b922, SUC_lexicon_tags_10 },
    { 0x66b5b922, SUC_lexicon_tags_27 },
    { 0x6595b924, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b5b929, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f5b930, SUC_lexicon_tags_7 },
    { 0x5495b930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1735b934, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc55b936, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb515b938, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8235b943, SUC_lexicon_tags_32 },
    { 0x1195b944, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd5b948, SUC_lexicon_tags_32 },
    { 0xe035b949, SUC_lexicon_tags_1 },
    { 0x6435b94a, SUC_lexicon_tags_2 },
    { 0x73f5b94b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa895b94e, SUC_lexicon_tags_12 },
    { 0x4235b94f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9295b954, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1075b956, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf655b958, SUC_lexicon_tags_33 },
    { 0x5175b958, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd555b95c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xef95b95e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff5b964, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a95b96c, SUC_lexicon_tags_198 },
    { 0xa2b5b96d, SUC_lexicon_tags_49 },
    { 0x1ab5b96e, SUC_lexicon_tags_60 },
    { 0x2ab5b96e, SUC_lexicon_tags_2 },
    { 0x1855b970, SUC_lexicon_tags_9 },
    { 0x1d35b971, SUC_lexicon_tags_2 },
    { 0xac15b972, SUC_lexicon_tags_0 },
    { 0x5f75b972, SUC_lexicon_tags_17 },
    { 0x5175b974, SUC_lexicon_tags_6 },
    { 0x28f5b975, SUC_lexicon_tags_8 },
    { 0x3b55b976, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb655b97d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88b5b97f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9255b982, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f75b985, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd535b989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3df5b98b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2015b98d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27d5b98f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9195b994, SUC_lexicon_tags_11 },
    { 0xa455b995, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5f55b997, SUC_lexicon_tags_17 },
    { 0x21f5b997, SUC_lexicon_tags_5 },
    { 0xb955b999, SUC_lexicon_tags_17 },
    { 0xe275b997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8735b99e, SUC_lexicon_tags_17 },
    { 0xd655b99f, SUC_lexicon_tags_12 },
    { 0xd3d5b99f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1695b9a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2195b9ad, SUC_lexicon_tags_48 },
    { 0xe335b9ae, SUC_lexicon_tags_37 },
    { 0xa8d5b9af, SUC_lexicon_tags_19 },
    { 0x7215b9b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1575b9bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1475b9c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b15b9c7, SUC_lexicon_tags_62 },
    { 0xc135b9c8, SUC_lexicon_tags_12 },
    { 0x8b5b9c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc155b9cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41f5b9cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf675b9cf, SUC_lexicon_tags_33 },
    { 0xf0d5b9cf, SUC_lexicon_tags_6 },
    { 0xb1b5b9d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5235b9d5, SUC_lexicon_tags_1 },
    { 0x8955b9d6, SUC_lexicon_tags_37 },
    { 0xb095b9d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5b9dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1455b9e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d5b9e9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1095b9eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb95b9ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa295b9f0, SUC_lexicon_tags_27 },
    { 0xf775b9f1, SUC_lexicon_tags_25 },
    { 0x475b9f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f5b9f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3d35b9f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83b5b9f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef5b9fc, SUC_lexicon_tags_47 },
    { 0x1b5b9fd, SUC_lexicon_tags_11 },
    { 0x5075b9fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fb5ba00, SUC_lexicon_tags_23 },
    { 0x7cf5ba01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f35ba0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2915ba0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a75ba11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d15ba13, SUC_lexicon_tags_17 },
    { 0xfe35ba14, SUC_lexicon_tags_37 },
    { 0x5e35ba15, SUC_lexicon_tags_7 },
    { 0xae35ba16, SUC_lexicon_tags_39 },
    { 0x87f5ba17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d5ba1b, SUC_lexicon_tags_38 },
    { 0xb035ba1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe675ba22, SUC_lexicon_tags_37 },
    { 0x7715ba23, SUC_lexicon_tags_14 },
    { 0xff35ba24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5ba27, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb6f5ba29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f5ba2c, SUC_lexicon_tags_25 },
    { 0x5c95ba2d, SUC_lexicon_tags_12 },
    { 0xc235ba2d, SUC_lexicon_tags_26 },
    { 0xd6f5ba2d, SUC_lexicon_tags_60 },
    { 0x1db5ba2d, SUC_lexicon_tags_12 },
    { 0x91d5ba31, SUC_lexicon_tags_16 },
    { 0x3735ba32, SUC_lexicon_tags_11 },
    { 0xfa75ba33, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d5ba36, SUC_lexicon_tags_11 },
    { 0xbb5ba36, SUC_lexicon_tags_17 },
    { 0x92f5ba38, SUC_lexicon_tags_12 },
    { 0x6e35ba39, SUC_lexicon_tags_79 },
    { 0x7f55ba3a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc115ba3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xae95ba3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc835ba42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495ba48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e35ba4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b5ba50, SUC_lexicon_tags_37 },
    { 0x4ed5ba50, SUC_lexicon_tags_14 },
    { 0x5c75ba51, SUC_lexicon_tags_60 },
    { 0xd055ba53, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd075ba58, SUC_lexicon_tags_7 },
    { 0x73d5ba58, SUC_lexicon_tags_15 },
    { 0x5bf5ba5a, SUC_lexicon_tags_19 },
    { 0x2135ba59, SUC_lexicon_tags_37 },
    { 0x1275ba59, SUC_lexicon_tags_25 },
    { 0x7f35ba5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd5ba60, SUC_lexicon_tags_37 },
    { 0x13f5ba61, SUC_lexicon_tags_66 },
    { 0xc1f5ba62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1235ba65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d5ba6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d5ba6e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xaa15ba70, SUC_lexicon_tags_30 },
    { 0x55ba71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe8d5ba73, SUC_lexicon_tags_17 },
    { 0x2975ba74, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f5ba77, SUC_lexicon_tags_11 },
    { 0x3d5ba77, SUC_lexicon_tags_25 },
    { 0x5d5ba77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b5ba7c, SUC_lexicon_tags_19 },
    { 0xac55ba7d, SUC_lexicon_tags_81 },
    { 0xd135ba7e, SUC_lexicon_tags_27 },
    { 0x2c15ba7f, SUC_lexicon_tags_15 },
    { 0x3bd5ba7f, SUC_lexicon_tags_2 },
    { 0x3495ba81, SUC_lexicon_tags_37 },
    { 0x4135ba82, SUC_lexicon_tags_28 },
    { 0x9995ba80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a55ba86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb5ba89, SUC_lexicon_tags_68 },
    { 0x2b15ba8a, SUC_lexicon_tags_2 },
    { 0xd415ba8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f5ba93, SUC_lexicon_tags_154 },
    { 0x5535ba94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb5ba97, SUC_lexicon_tags_112 },
    { 0xe7d5ba97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb395ba9a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad75baa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb5baa6, SUC_lexicon_tags_17 },
    { 0x4295baa7, SUC_lexicon_tags_0 },
    { 0x8895baa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395baac, SUC_lexicon_tags_11 },
    { 0x8775baad, SUC_lexicon_tags_30 },
    { 0x5ff5baae, SUC_lexicon_tags_43 },
    { 0x7f35baac, SUC_lexicon_tags_15 },
    { 0x9bb5baac, SUC_lexicon_tags_30 },
    { 0x9915bab1, SUC_lexicon_tags_1 },
    { 0x2cf5bab2, SUC_lexicon_tags_57 },
    { 0x39f5bab3, SUC_lexicon_tags_1 },
    { 0x1695bab4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b5bab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1795baba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd75babc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6715babe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1935bac1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6835bac4, SUC_lexicon_tags_2 },
    { 0x6f5bac4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9235baca, SUC_lexicon_tags_47 },
    { 0x2495bacb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9315bad2, SUC_lexicon_tags_0 },
    { 0xfb75bad3, SUC_lexicon_tags_63 },
    { 0x3495bad4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b5bad9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b15badb, SUC_lexicon_tags_25 },
    { 0x9475badb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc655badf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb475bae2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa415bae5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555baea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc755baef, SUC_lexicon_tags_25 },
    { 0x3d95baf0, SUC_lexicon_tags_2 },
    { 0x12d5baef, SUC_lexicon_tags_37 },
    { 0xe955baf0, SUC_lexicon_tags_25 },
    { 0x2255baf3, SUC_lexicon_tags_28 },
    { 0x1e55baf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb5baf6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8335baf9, SUC_lexicon_tags_47 },
    { 0xc5f5baf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9935baff, SUC_lexicon_tags_2 },
    { 0x2295bb00, SUC_lexicon_tags_9 },
    { 0x72d5bb01, SUC_lexicon_tags_0 },
    { 0xc095bb00, SUC_lexicon_tags_25 },
    { 0xc9f5bb00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd5bb07, SUC_lexicon_tags_30 },
    { 0xa915bb08, SUC_lexicon_tags_68 },
    { 0xd215bb09, SUC_lexicon_tags_17 },
    { 0x6c75bb08, SUC_lexicon_tags_10 },
    { 0x8535bb08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe395bb0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd655bb0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f5bb13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2ad5bb15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d75bb17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd715bb1e, SUC_lexicon_tags_1 },
    { 0x5af5bb1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f5bb23, SUC_lexicon_tags_17 },
    { 0xdd95bb24, SUC_lexicon_tags_49 },
    { 0x9d75bb25, SUC_lexicon_tags_14 },
    { 0x1c75bb26, SUC_lexicon_tags_55 },
    { 0xba95bb24, SUC_lexicon_tags_151 },
    { 0xbd55bb28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69b5bb2a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5bb31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb5bb35, SUC_lexicon_tags_28 },
    { 0xc795bb35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5b5bb38, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0xd5d5bb3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe175bb3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71d5bb3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5255bb41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe195bb43, SUC_lexicon_tags_29 },
    { 0x5a75bb44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb655bb46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa415bb4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa455bb4f, SUC_lexicon_tags_23 },
    { 0xd1b5bb4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6695bb53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x855bb55, SUC_lexicon_tags_37 },
    { 0x75bb56, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9295bb5b, SUC_lexicon_tags_15 },
    { 0x9c95bb5b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3435bb5e, SUC_lexicon_tags_32 },
    { 0x4515bb5e, SUC_lexicon_tags_25 },
    { 0x4395bb60, SUC_lexicon_tags_27 },
    { 0x6775bb60, SUC_lexicon_tags_12 },
    { 0xf0f5bb62, SUC_lexicon_tags_2 },
    { 0x2e15bb60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f5bb67, SUC_lexicon_tags_1 },
    { 0xc7b5bb68, SUC_lexicon_tags_27 },
    { 0x48d5bb69, SUC_lexicon_tags_19 },
    { 0x495bb6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d75bb6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5d15bb6e, SUC_lexicon_tags_6 },
    { 0x6cf5bb6e, SUC_lexicon_tags_37 },
    { 0xa375bb6f, SUC_lexicon_tags_1 },
    { 0xf1f5bb71, SUC_lexicon_tags_2 },
    { 0xe35bb72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf5bb7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b95bb80, SUC_lexicon_tags_7 },
    { 0xbd95bb80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d5bb87, SUC_lexicon_tags_25 },
    { 0xcc55bb88, SUC_lexicon_tags_11 },
    { 0xd875bb88, SUC_lexicon_tags_117 },
    { 0xe7b5bb88, SUC_lexicon_tags_17 },
    { 0x5f15bb87, SUC_lexicon_tags_37 },
    { 0xca35bb89, SUC_lexicon_tags_82 },
    { 0x9bd5bb8d, SUC_lexicon_tags_2 },
    { 0x6fb5bb8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2835bb92, SUC_lexicon_tags_17 },
    { 0xccb5bb92, SUC_lexicon_tags_40 },
    { 0xb075bb94, SUC_lexicon_tags_81 },
    { 0x7e55bb94, SUC_lexicon_tags_37 },
    { 0x3735bb96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7815bb98, SUC_lexicon_tags_2 },
    { 0x5875bb99, SUC_lexicon_tags_37 },
    { 0x7a55bb98, SUC_lexicon_tags_35 },
    { 0x5d35bb9b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3995bb9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd755bba0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a35bba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5bba9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xabd5bbab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc555bbae, SUC_lexicon_tags_11 },
    { 0xf755bbae, SUC_lexicon_tags_55 },
    { 0x9695bbb0, SUC_lexicon_tags_11 },
    { 0x36b5bbb1, SUC_lexicon_tags_27 },
    { 0x395bbb1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f5bbb7, SUC_lexicon_tags_31 },
    { 0x49f5bbb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf95bbbd, SUC_lexicon_tags_12 },
    { 0x9d15bbbe, SUC_lexicon_tags_11 },
    { 0xd095bbbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd95bbc1, SUC_lexicon_tags_33 },
    { 0x3c15bbc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b5bbc7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb155bbc9, SUC_lexicon_tags_7 },
    { 0x85f5bbca, SUC_lexicon_tags_15 },
    { 0xaf5bbca, SUC_lexicon_tags_22 },
    { 0xc675bbcb, SUC_lexicon_tags_15 },
    { 0x7335bbcb, SUC_lexicon_tags_29 },
    { 0xb6b5bbc9, SUC_lexicon_tags_27 },
    { 0xfcb5bbcf, SUC_lexicon_tags_68 },
    { 0x3255bbd0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa1d5bbd2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa175bbd9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa955bbe1, SUC_lexicon_tags_43 },
    { 0x46f5bbe1, SUC_lexicon_tags_1 },
    { 0x2d5bbe1, SUC_lexicon_tags_19 },
    { 0xad55bbe2, SUC_lexicon_tags_18 },
    { 0xf1b5bbe4, SUC_lexicon_tags_51 },
    { 0xfe75bbe3, SUC_lexicon_tags_39 },
    { 0x4855bbe7, SUC_lexicon_tags_7 },
    { 0x98b5bbe8, SUC_lexicon_tags_49 },
    { 0x9995bbe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955bbed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd715bbf0, SUC_lexicon_tags_11 },
    { 0xcc75bbf1, SUC_lexicon_tags_87 },
    { 0x6e35bbf1, SUC_lexicon_tags_37 },
    { 0xd8d5bbf3, SUC_lexicon_tags_11 },
    { 0xec75bbf4, SUC_lexicon_tags_6 },
    { 0x7ab5bbf4, SUC_lexicon_tags_11 },
    { 0x1cb5bbf5, SUC_lexicon_tags_15 },
    { 0xa835bbf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f5bbfe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb5bc03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2eb5bc05, SUC_lexicon_tags_29 },
    { 0xed55bc06, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0x8fd5bc08, SUC_lexicon_tags_25 },
    { 0x37f5bc08, SUC_lexicon_tags_19 },
    { 0xfe35bc08, SUC_lexicon_tags_60 },
    { 0x5cb5bc0b, SUC_lexicon_tags_49 },
    { 0x5b35bc0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9475bc12, SUC_lexicon_tags_6 },
    { 0x59b5bc13, SUC_lexicon_tags_64 },
    { 0x22f5bc13, SUC_lexicon_tags_12 },
    { 0x9755bc15, SUC_lexicon_tags_11 },
    { 0x62d5bc16, SUC_lexicon_tags_1 },
    { 0x6ad5bc17, SUC_lexicon_tags_25 },
    { 0x76f5bc15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d55bc1a, SUC_lexicon_tags_27 },
    { 0x6b15bc1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5bc20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4235bc25, SUC_lexicon_tags_49 },
    { 0x5f5bc26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b5bc2f, SUC_lexicon_tags_74 },
    { 0x6d5bc30, SUC_lexicon_tags_11 },
    { 0xeed5bc31, SUC_lexicon_tags_7 },
    { 0xdaf5bc32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3875bc34, SUC_lexicon_tags_19 },
    { 0x7c95bc35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab5bc39, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f5bc41, SUC_lexicon_tags_3 },
    { 0xe595bc42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb935bc49, SUC_lexicon_tags_25 },
    { 0x15d5bc4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6435bc4d, SUC_lexicon_tags_11 },
    { 0x7735bc4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db5bc58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86d5bc5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295bc5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa075bc63, SUC_lexicon_tags_33 },
    { 0x2b75bc63, SUC_lexicon_tags_37 },
    { 0xe955bc65, SUC_lexicon_tags_14 },
    { 0x3f5bc64, SUC_lexicon_tags_27 },
    { 0x7215bc67, SUC_lexicon_tags_12 },
    { 0x2115bc64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f5bc6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b55bc6d, SUC_lexicon_tags_5 },
    { 0x2995bc6e, SUC_lexicon_tags_47 },
    { 0xf235bc6f, SUC_lexicon_tags_15 },
    { 0xd715bc6d, SUC_lexicon_tags_15 },
    { 0xe595bc71, SUC_lexicon_tags_30 },
    { 0x8a15bc72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb5bc7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9035bc7d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7275bc7f, SUC_lexicon_tags_11 },
    { 0xbd5bc7f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf615bc86, SUC_lexicon_tags_43 },
    { 0xae15bc87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf235bc8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbcb5bc8d, SUC_lexicon_tags_12 },
    { 0x5935bc8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdcb5bc90, SUC_lexicon_tags_37 },
    { 0x2b15bc90, SUC_lexicon_tags_44 },
    { 0x3d95bc92, SUC_lexicon_tags_35 },
    { 0x67d5bc92, SUC_lexicon_tags_11 },
    { 0xac35bc94, SUC_lexicon_tags_30 },
    { 0x14b5bc94, SUC_lexicon_tags_39 },
    { 0xa415bc95, SUC_lexicon_tags_3 },
    { 0x11f5bc97, SUC_lexicon_tags_47 },
    { 0x7fb5bc98, SUC_lexicon_tags_49 },
    { 0x36f5bc98, SUC_lexicon_tags_38 },
    { 0x5495bc99, SUC_lexicon_tags_19 },
    { 0x8ab5bc9b, SUC_lexicon_tags_38 },
    { 0xdb55bc9c, SUC_lexicon_tags_12 },
    { 0xda95bc95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5855bca0, SUC_lexicon_tags_12 },
    { 0xd5d5bca0, SUC_lexicon_tags_66 },
    { 0xc575bca2, SUC_lexicon_tags_45 },
    { 0x8975bca3, SUC_lexicon_tags_17 },
    { 0xc9b5bca4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6035bca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf5bca9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b75bcad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1d5bcaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd5bcb4, SUC_lexicon_tags_17 },
    { 0x3b35bcb4, SUC_lexicon_tags_63 },
    { 0x1935bcb5, SUC_lexicon_tags_15 },
    { 0xae15bcb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff5bcba, SUC_lexicon_tags_1 },
    { 0x47f5bcbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf635bcc3, SUC_lexicon_tags_63 },
    { 0x1075bcc4, SUC_lexicon_tags_94 },
    { 0x95b5bcc5, SUC_lexicon_tags_7 },
    { 0x7595bcc5, SUC_lexicon_tags_42 },
    { 0x29d5bcc4, SUC_lexicon_tags_38 },
    { 0x57d5bcc8, SUC_lexicon_tags_25 },
    { 0xfd75bcc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb5bccd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42f5bccf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f5bcd2, SUC_lexicon_tags_7 },
    { 0xa875bcd2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3495bcd5, SUC_lexicon_tags_33 },
    { 0x2a15bcd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92f5bcd8, SUC_lexicon_tags_0 },
    { 0xe955bcd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e75bcdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d5bce0, SUC_lexicon_tags_2 },
    { 0x4575bce1, SUC_lexicon_tags_26 },
    { 0x8b15bce1, SUC_lexicon_tags_29 },
    { 0xaed5bce3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db5bce7, SUC_lexicon_tags_43 },
    { 0x2bb5bce8, SUC_lexicon_tags_9 },
    { 0x70f5bce9, SUC_lexicon_tags_45 },
    { 0x99d5bcea, SUC_lexicon_tags_45 },
    { 0x9695bceb, SUC_lexicon_tags_63 },
    { 0xa215bcec, SUC_lexicon_tags_68 },
    { 0xc9d5bce9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b5bcf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f5bcf5, SUC_lexicon_tags_33 },
    { 0x7ef5bcf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d55bcf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b5bcfb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbb5bcfd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd5bd01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd635bd03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f5bd08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f15bd0f, SUC_lexicon_tags_11 },
    { 0xee95bd10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb895bd13, SUC_lexicon_tags_37 },
    { 0x8055bd14, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b5bd1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc75bd1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2715bd22, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8655bd25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8295bd27, SUC_lexicon_tags_69 },
    { 0x255bd27, SUC_lexicon_tags_25 },
    { 0xfe35bd29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd75bd2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab5bd31, SUC_lexicon_tags_9 },
    { 0xcad5bd32, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed5bd39, SUC_lexicon_tags_41 },
    { 0x6435bd39, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb375bd3c, SUC_lexicon_tags_17 },
    { 0x36b5bd3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d75bd3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3515bd41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b5bd48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64b5bd4a, SUC_lexicon_tags_55 },
    { 0x175bd4a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2635bd4e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xca35bd50, SUC_lexicon_tags_7 },
    { 0x7b75bd51, SUC_lexicon_tags_17 },
    { 0x81b5bd51, SUC_lexicon_tags_11 },
    { 0x34b5bd53, SUC_lexicon_tags_23 },
    { 0xd775bd53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa215bd58, SUC_lexicon_tags_8 },
    { 0x8335bd59, SUC_lexicon_tags_11 },
    { 0x5095bd5a, SUC_lexicon_tags_27 },
    { 0xe295bd5a, SUC_lexicon_tags_47 },
    { 0x37b5bd5c, SUC_lexicon_tags_37 },
    { 0xebd5bd5d, SUC_lexicon_tags_12 },
    { 0x6d15bd5d, SUC_lexicon_tags_11 },
    { 0xca35bd5f, SUC_lexicon_tags_2 },
    { 0x3a55bd60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe275bd63, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd5bd66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe095bd69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5075bd70, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x82d5bd72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4635bd74, SUC_lexicon_tags_13 },
    { 0x4715bd75, SUC_lexicon_tags_14 },
    { 0x7b15bd74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb775bd79, SUC_lexicon_tags_19 },
    { 0x71d5bd79, SUC_lexicon_tags_55 },
    { 0xa375bd7b, SUC_lexicon_tags_7 },
    { 0x2ff5bd7a, SUC_lexicon_tags_49 },
    { 0xe015bd7d, SUC_lexicon_tags_15 },
    { 0xa175bd79, SUC_lexicon_tags_27 },
    { 0xf815bd7f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x5235bd81, SUC_lexicon_tags_43 },
    { 0x62f5bd81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b5bd85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8215bd8b, SUC_lexicon_tags_34 },
    { 0x4495bd8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5bd8f, SUC_lexicon_tags_15 },
    { 0x8eb5bd90, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe075bd94, SUC_lexicon_tags_15 },
    { 0x20d5bd94, SUC_lexicon_tags_46 },
    { 0x69f5bd96, SUC_lexicon_tags_8 },
    { 0x33b5bd94, SUC_lexicon_tags_60 },
    { 0xac15bd97, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab5bd9d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd815bd9f, SUC_lexicon_tags_12 },
    { 0xbef5bda0, SUC_lexicon_tags_19 },
    { 0xdf55bda0, SUC_lexicon_tags_2 },
    { 0xe2f5bda2, SUC_lexicon_tags_17 },
    { 0x5555bda2, SUC_lexicon_tags_68 },
    { 0x4d75bda4, SUC_lexicon_tags_7 },
    { 0xe2f5bda5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16d5bda7, SUC_lexicon_tags_47 },
    { 0x70d5bda8, SUC_lexicon_tags_243 },
    { 0xbb35bda8, SUC_lexicon_tags_26 },
    { 0xde35bdaa, SUC_lexicon_tags_7 },
    { 0xc555bda8, SUC_lexicon_tags_23 },
    { 0xdfd5bdac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62d5bdae, SUC_lexicon_tags_31 },
    { 0xefd5bdaf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1555bdb3, SUC_lexicon_tags_15 },
    { 0xc9b5bdb4, SUC_lexicon_tags_23 },
    { 0x2995bdb5, SUC_lexicon_tags_47 },
    { 0x7a95bdb6, SUC_lexicon_tags_30 },
    { 0x4795bdb7, SUC_lexicon_tags_63 },
    { 0xe375bdb8, SUC_lexicon_tags_2 },
    { 0x4b75bdb9, SUC_lexicon_tags_1 },
    { 0xe8d5bdb9, SUC_lexicon_tags_1 },
    { 0xdfd5bdb8, SUC_lexicon_tags_14 },
    { 0x6c15bdbc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd5bdc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b5bdc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b5bdcb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b5bdce, SUC_lexicon_tags_11 },
    { 0x31f5bdce, SUC_lexicon_tags_31 },
    { 0x11f5bdd0, SUC_lexicon_tags_28 },
    { 0x2cb5bdd0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb5bdd4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a95bdd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bf5bdda, SUC_lexicon_tags_1 },
    { 0x3715bddb, SUC_lexicon_tags_14 },
    { 0x3f55bddc, SUC_lexicon_tags_11 },
    { 0x615bddd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa195bde3, SUC_lexicon_tags_7 },
    { 0xfc55bde3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb995bde6, SUC_lexicon_tags_47 },
    { 0x4775bde7, SUC_lexicon_tags_7 },
    { 0x8c55bde8, SUC_lexicon_tags_25 },
    { 0x94b5bde8, SUC_lexicon_tags_87 },
    { 0xcf75bdea, SUC_lexicon_tags_69 },
    { 0x7c95bde9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c75bdf4, SUC_lexicon_tags_15 },
    { 0xcab5bdf5, SUC_lexicon_tags_12 },
    { 0x8d55bdf5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6355be00, SUC_lexicon_tags_7 },
    { 0x3635be01, SUC_lexicon_tags_30 },
    { 0x7ff5be01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b5be06, SUC_lexicon_tags_63 },
    { 0x8435be07, SUC_lexicon_tags_29 },
    { 0xe255be06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b5be0b, SUC_lexicon_tags_1 },
    { 0x57f5be0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b5be0e, SUC_lexicon_tags_25 },
    { 0x3c15be0f, SUC_lexicon_tags_10 },
    { 0xb075be10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd795be14, SUC_lexicon_tags_23 },
    { 0x3635be15, SUC_lexicon_tags_37 },
    { 0xd055be16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d5be1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6915be21, SUC_lexicon_tags_37 },
    { 0x6c95be21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5635be25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c15be28, SUC_lexicon_tags_11 },
    { 0x97b5be28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d5be2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5be32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc875be3a, SUC_lexicon_tags_87 },
    { 0x8775be3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3235be40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f5be45, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdd35be47, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4275be4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f5be4e, SUC_lexicon_tags_37 },
    { 0x5355be4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c35be59, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6095be5b, SUC_lexicon_tags_8 },
    { 0xbff5be5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d5be5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed5be61, SUC_lexicon_tags_15 },
    { 0x9cf5be62, SUC_lexicon_tags_15 },
    { 0xde95be62, SUC_lexicon_tags_23 },
    { 0xb1b5be62, SUC_lexicon_tags_2 },
    { 0xdfd5be65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3975be6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x58f5be6e, SUC_lexicon_tags_7 },
    { 0x1195be6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbd5be71, SUC_lexicon_tags_43 },
    { 0xb075be71, SUC_lexicon_tags_19 },
    { 0xeb55be71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2355be77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7695be7c, SUC_lexicon_tags_20 },
    { 0x64f5be7d, SUC_lexicon_tags_29 },
    { 0x1a55be7e, SUC_lexicon_tags_1 },
    { 0x9b35be7c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9675be83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5be89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b5be8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b95be8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735be96, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d5be99, SUC_lexicon_tags_1 },
    { 0x5215be99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d5be9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5bea2, SUC_lexicon_tags_17 },
    { 0xa175bea3, SUC_lexicon_tags_11 },
    { 0x5af5bea2, SUC_lexicon_tags_15 },
    { 0x295bea5, SUC_lexicon_tags_33 },
    { 0x3515bea4, SUC_lexicon_tags_37 },
    { 0xdf15bea3, SUC_lexicon_tags_2 },
    { 0xdbd5bea8, SUC_lexicon_tags_0 },
    { 0xfc35bea8, SUC_lexicon_tags_0 },
    { 0x5cf5bea3, SUC_lexicon_tags_21 },
    { 0x5595beab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd5bead, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6135beb2, SUC_lexicon_tags_47 },
    { 0xe4b5beb3, SUC_lexicon_tags_39 },
    { 0x82b5beb4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8515beb7, SUC_lexicon_tags_37 },
    { 0xaab5beb7, SUC_lexicon_tags_37 },
    { 0x50b5beb7, SUC_lexicon_tags_1 },
    { 0xe775beba, SUC_lexicon_tags_27 },
    { 0x36b5bebb, SUC_lexicon_tags_0 },
    { 0x1f35beb9, SUC_lexicon_tags_63 },
    { 0xacb5bebb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb535bec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb75bec6, SUC_lexicon_tags_2 },
    { 0xac95bec6, SUC_lexicon_tags_17 },
    { 0xf1d5bec8, SUC_lexicon_tags_29 },
    { 0xb9d5bec9, SUC_lexicon_tags_21 },
    { 0x2eb5bec8, SUC_lexicon_tags_1 },
    { 0x3db5beca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab55bed0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15bed4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2b5bed6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d5bed9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b5bedd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd935bedf, SUC_lexicon_tags_37 },
    { 0xab75bee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2b5bee2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf5bee6, SUC_lexicon_tags_19 },
    { 0xc535bee7, SUC_lexicon_tags_11 },
    { 0x9415bee7, SUC_lexicon_tags_37 },
    { 0xee35bee9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc8d5beeb, SUC_lexicon_tags_2 },
    { 0xffb5beec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb175beee, SUC_lexicon_tags_28 },
    { 0xaad5beef, SUC_lexicon_tags_38 },
    { 0xc15bef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b5bef3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f5bef7, SUC_lexicon_tags_17 },
    { 0xa755bef7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495beff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34b5bf01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f5bf06, SUC_lexicon_tags_87 },
    { 0x8d15bf07, SUC_lexicon_tags_37 },
    { 0x1375bf08, SUC_lexicon_tags_19 },
    { 0x7cb5bf08, SUC_lexicon_tags_33 },
    { 0x5095bf0a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfb15bf0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe75bf16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb5bf1f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9915bf26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d5bf2c, SUC_lexicon_tags_4 },
    { 0xaef5bf2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9675bf30, SUC_lexicon_tags_19 },
    { 0xf2f5bf30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b5bf35, SUC_lexicon_tags_80 },
    { 0xe3b5bf36, SUC_lexicon_tags_37 },
    { 0xd0d5bf36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b55bf3f, SUC_lexicon_tags_11 },
    { 0xa5b5bf40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa615bf49, SUC_lexicon_tags_27 },
    { 0x9db5bf4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db5bf4d, SUC_lexicon_tags_47 },
    { 0x5795bf4e, SUC_lexicon_tags_11 },
    { 0x89f5bf4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe55bf51, SUC_lexicon_tags_6 },
    { 0x2995bf52, SUC_lexicon_tags_4 },
    { 0x1795bf53, SUC_lexicon_tags_32 },
    { 0xb1f5bf54, SUC_lexicon_tags_21 },
    { 0x7d35bf54, SUC_lexicon_tags_13 },
    { 0xfc75bf56, SUC_lexicon_tags_7 },
    { 0xc395bf51, SUC_lexicon_tags_27 },
    { 0x9155bf54, SUC_lexicon_tags_7 },
    { 0xa5f5bf52, SUC_lexicon_tags_31 },
    { 0x85d5bf55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6995bf5c, SUC_lexicon_tags_38 },
    { 0xdbf5bf5d, SUC_lexicon_tags_38 },
    { 0xaf5bf5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af5bf61, SUC_lexicon_tags_9 },
    { 0xd5b5bf61, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x295bf64, SUC_lexicon_tags_2 },
    { 0xff55bf65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a95bf68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875bf6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb5bf77, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1c15bf79, SUC_lexicon_tags_45 },
    { 0x97f5bf7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe275bf7c, SUC_lexicon_tags_15 },
    { 0xd255bf7c, SUC_lexicon_tags_2 },
    { 0xc3f5bf7e, SUC_lexicon_tags_27 },
    { 0xa7b5bf7f, SUC_lexicon_tags_9 },
    { 0x8455bf7f, SUC_lexicon_tags_11 },
    { 0x2915bf7f, SUC_lexicon_tags_17 },
    { 0xcc95bf7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af5bf86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf055bf8c, SUC_lexicon_tags_25 },
    { 0x7ed5bf8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed5bf90, SUC_lexicon_tags_19 },
    { 0x8835bf91, SUC_lexicon_tags_17 },
    { 0x1035bf91, SUC_lexicon_tags_32 },
    { 0x8f95bf93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc135bf96, SUC_lexicon_tags_17 },
    { 0xe355bf96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f15bf9b, SUC_lexicon_tags_27 },
    { 0x8d75bf9b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef5bf9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f15bfa7, SUC_lexicon_tags_25 },
    { 0x7935bfa7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd375bfaa, SUC_lexicon_tags_2 },
    { 0x2595bfab, SUC_lexicon_tags_6 },
    { 0xefd5bfac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f15bfaf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f5bfba, SUC_lexicon_tags_31 },
    { 0xc275bfba, SUC_lexicon_tags_17 },
    { 0x9d35bfbc, SUC_lexicon_tags_33 },
    { 0x6a75bfbc, SUC_lexicon_tags_33 },
    { 0xce95bfbe, SUC_lexicon_tags_25 },
    { 0x19b5bfbf, SUC_lexicon_tags_1 },
    { 0x3155bfbf, SUC_lexicon_tags_79 },
    { 0x2295bfbd, SUC_lexicon_tags_9 },
    { 0x3e55bfc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d95bfc7, SUC_lexicon_tags_49 },
    { 0x65b5bfc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fb5bfca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4215bfcd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xffb5bfcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7675bfd1, SUC_lexicon_tags_15 },
    { 0x1835bfd2, SUC_lexicon_tags_25 },
    { 0x3535bfd2, SUC_lexicon_tags_1 },
    { 0x9e35bfd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75bfdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb5bfe8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbc95bfea, SUC_lexicon_tags_15 },
    { 0xe915bfeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3b5bfed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9b5bfef, SUC_lexicon_tags_45 },
    { 0x4cf5bff0, SUC_lexicon_tags_63 },
    { 0xe775bfef, SUC_lexicon_tags_8 },
    { 0x3455bff2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7535bff5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5bffb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7ff5bffd, SUC_lexicon_tags_46 },
    { 0xae15bffe, SUC_lexicon_tags_2 },
    { 0x7835bfff, SUC_lexicon_tags_25 },
    { 0x1b95c000, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a75c004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xead5c006, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0x4355c009, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9b95c00b, SUC_lexicon_tags_23 },
    { 0x7895c00b, SUC_lexicon_tags_37 },
    { 0x8c35c00d, SUC_lexicon_tags_23 },
    { 0x3f35c00e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefb5c010, SUC_lexicon_tags_19 },
    { 0x43b5c011, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe855c014, SUC_lexicon_tags_55 },
    { 0x3cd5c014, SUC_lexicon_tags_20 },
    { 0xabd5c015, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9f5c018, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc955c01a, SUC_lexicon_tags_15 },
    { 0x8935c01b, SUC_lexicon_tags_9 },
    { 0x2eb5c01c, SUC_lexicon_tags_9 },
    { 0xf2d5c01d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb5c022, SUC_lexicon_tags_13 },
    { 0xdcd5c023, SUC_lexicon_tags_9 },
    { 0x86d5c024, SUC_lexicon_tags_13 },
    { 0x3555c022, SUC_lexicon_tags_416 },
    { 0xecd5c025, SUC_lexicon_tags_2 },
    { 0x9d55c026, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x6075c029, SUC_lexicon_tags_6 },
    { 0x8755c029, SUC_lexicon_tags_152 },
    { 0x6535c029, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb035c02f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9515c032, SUC_lexicon_tags_1 },
    { 0xedf5c032, SUC_lexicon_tags_17 },
    { 0xf455c033, SUC_lexicon_tags_8 },
    { 0x32f5c035, SUC_lexicon_tags_2 },
    { 0x6595c036, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd875c038, SUC_lexicon_tags_21 },
    { 0xb295c039, SUC_lexicon_tags_17 },
    { 0xac15c038, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc775c03d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab5c042, SUC_lexicon_tags_13 },
    { 0x3695c043, SUC_lexicon_tags_15 },
    { 0xf935c044, SUC_lexicon_tags_9 },
    { 0x5cd5c045, SUC_lexicon_tags_7 },
    { 0xc835c045, SUC_lexicon_tags_14 },
    { 0x6c75c047, SUC_lexicon_tags_182 },
    { 0x155c043, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5d95c04a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef5c050, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ed5c052, SUC_lexicon_tags_23 },
    { 0xe215c052, SUC_lexicon_tags_2 },
    { 0xf815c054, SUC_lexicon_tags_301 },
    { 0x63b5c054, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9b5c057, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9855c05b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6f5c05d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5c060, SUC_lexicon_tags_25 },
    { 0xeef5c060, SUC_lexicon_tags_30 },
    { 0xe2f5c062, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95c066, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e35c069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11b5c06b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b5c073, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe215c075, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b55c078, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x5495c07a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5175c07f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5c088, SUC_lexicon_tags_84 },
    { 0xad55c088, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc35c08d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a95c08f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d75c095, SUC_lexicon_tags_27 },
    { 0x9395c096, SUC_lexicon_tags_38 },
    { 0x1af5c096, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2215c09c, SUC_lexicon_tags_25 },
    { 0x2f75c09d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155c0a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe75c0a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8015c0a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e55c0ab, SUC_lexicon_tags_10 },
    { 0x5975c0ac, SUC_lexicon_tags_37 },
    { 0x1715c0ad, SUC_lexicon_tags_14 },
    { 0xb595c0ac, SUC_lexicon_tags_2 },
    { 0x9975c0af, SUC_lexicon_tags_1 },
    { 0xd495c0af, SUC_lexicon_tags_1 },
    { 0x2ef5c0ab, SUC_lexicon_tags_149 },
    { 0xf2f5c0b2, SUC_lexicon_tags_0 },
    { 0x5b55c0b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xed35c0b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d75c0b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8d5c0b9, SUC_lexicon_tags_50 },
    { 0xbbd5c0ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5335c0be, SUC_lexicon_tags_84 },
    { 0x96d5c0bf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f15c0c2, SUC_lexicon_tags_6 },
    { 0xad35c0c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfd5c0c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94f5c0c7, SUC_lexicon_tags_15 },
    { 0x3f35c0c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe715c0ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c15c0ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d5c0d5, SUC_lexicon_tags_47 },
    { 0x6895c0d6, SUC_lexicon_tags_17 },
    { 0xe355c0d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d75c0e2, SUC_lexicon_tags_5 },
    { 0x79d5c0e3, SUC_lexicon_tags_15 },
    { 0x2c35c0e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d75c0e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5c0eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc895c0f4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d5c0f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe975c0fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7075c0fc, SUC_lexicon_tags_11 },
    { 0x5eb5c0fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9035c0ff, SUC_lexicon_tags_69 },
    { 0xa575c100, SUC_lexicon_tags_11 },
    { 0xf2b5c0ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d75c103, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b5c10d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc455c10f, SUC_lexicon_tags_8 },
    { 0x3955c110, SUC_lexicon_tags_60 },
    { 0x86b5c111, SUC_lexicon_tags_7 },
    { 0xb1b5c112, SUC_lexicon_tags_37 },
    { 0x8195c113, SUC_lexicon_tags_17 },
    { 0x4395c114, SUC_lexicon_tags_0 },
    { 0x31d5c110, SUC_lexicon_tags_2 },
    { 0x2e75c114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d5c11a, SUC_lexicon_tags_1 },
    { 0x3515c11b, SUC_lexicon_tags_47 },
    { 0xce55c11c, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0x9295c11e, SUC_lexicon_tags_11 },
    { 0xcf75c11e, SUC_lexicon_tags_34 },
    { 0xc135c11e, SUC_lexicon_tags_1 },
    { 0x8855c121, SUC_lexicon_tags_15 },
    { 0xcc75c122, SUC_lexicon_tags_38 },
    { 0xf335c11f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2835c128, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7435c12a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc795c12c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c35c131, SUC_lexicon_tags_35 },
    { 0x3615c132, SUC_lexicon_tags_16 },
    { 0x3d5c131, SUC_lexicon_tags_19 },
    { 0x24f5c131, SUC_lexicon_tags_19 },
    { 0x1835c134, SUC_lexicon_tags_2 },
    { 0x61b5c133, SUC_lexicon_tags_25 },
    { 0x9535c137, SUC_lexicon_tags_25 },
    { 0x1175c138, SUC_lexicon_tags_61 },
    { 0x2615c139, SUC_lexicon_tags_37 },
    { 0x2415c139, SUC_lexicon_tags_19 },
    { 0x6bb5c138, SUC_lexicon_tags_47 },
    { 0x4a15c13b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9575c13e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e75c143, SUC_lexicon_tags_25 },
    { 0x1a15c144, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad5c14a, SUC_lexicon_tags_38 },
    { 0x62d5c14a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa695c150, SUC_lexicon_tags_19 },
    { 0xe275c151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4835c155, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe3b5c157, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f5c15d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf5c160, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa135c163, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b5c16b, SUC_lexicon_tags_46 },
    { 0xbfb5c16c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba35c16e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa35c174, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf875c177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef5c17a, SUC_lexicon_tags_45 },
    { 0xc515c17b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd635c185, SUC_lexicon_tags_11 },
    { 0x7215c185, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f5c18d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7075c196, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd75c198, SUC_lexicon_tags_23 },
    { 0xffb5c198, SUC_lexicon_tags_31 },
    { 0x54f5c199, SUC_lexicon_tags_91 },
    { 0x6dd5c19b, SUC_lexicon_tags_11 },
    { 0x5755c19b, SUC_lexicon_tags_27 },
    { 0x13b5c19b, SUC_lexicon_tags_37 },
    { 0xc3d5c19e, SUC_lexicon_tags_43 },
    { 0x5f15c198, SUC_lexicon_tags_32 },
    { 0x4655c19b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc995c1a4, SUC_lexicon_tags_1 },
    { 0x6155c1a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf935c1a8, SUC_lexicon_tags_0 },
    { 0xe755c1a9, SUC_lexicon_tags_7 },
    { 0x8895c1aa, SUC_lexicon_tags_9 },
    { 0x4375c1a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa75c1b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79f5c1b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed95c1b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf775c1c0, SUC_lexicon_tags_37 },
    { 0x29d5c1c0, SUC_lexicon_tags_11 },
    { 0x3f95c1c2, SUC_lexicon_tags_37 },
    { 0xa8d5c1c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7015c1c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4035c1c8, SUC_lexicon_tags_22 },
    { 0xc035c1c8, SUC_lexicon_tags_63 },
    { 0xe1f5c1ca, SUC_lexicon_tags_57 },
    { 0x9bd5c1cb, SUC_lexicon_tags_19 },
    { 0x29b5c1cc, SUC_lexicon_tags_2 },
    { 0xd4d5c1cc, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b5c1d1, SUC_lexicon_tags_11 },
    { 0xf195c1d2, SUC_lexicon_tags_37 },
    { 0x23b5c1d1, SUC_lexicon_tags_47 },
    { 0xed35c1d4, SUC_lexicon_tags_29 },
    { 0xccd5c1d5, SUC_lexicon_tags_16 },
    { 0xc415c1d3, SUC_lexicon_tags_39 },
    { 0x65f5c1d7, SUC_lexicon_tags_11 },
    { 0xc195c1d5, SUC_lexicon_tags_9 },
    { 0x7db5c1d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5c1dd, SUC_lexicon_tags_17 },
    { 0xd935c1de, SUC_lexicon_tags_87 },
    { 0xa055c1dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6315c1e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f75c1ea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b5c1ed, SUC_lexicon_tags_25 },
    { 0x29d5c1ee, SUC_lexicon_tags_37 },
    { 0xdb15c1ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1255c1f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb895c1f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d5c1fa, SUC_lexicon_tags_12 },
    { 0x8815c1fa, SUC_lexicon_tags_17 },
    { 0xb675c1fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1655c200, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x4435c203, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f35c208, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c95c20a, SUC_lexicon_tags_44 },
    { 0xc1b5c20b, SUC_lexicon_tags_14 },
    { 0x45d5c20a, SUC_lexicon_tags_7 },
    { 0x74f5c20a, SUC_lexicon_tags_12 },
    { 0xc915c20c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xec95c211, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f5c214, SUC_lexicon_tags_7 },
    { 0x7ab5c214, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8315c218, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xda75c21a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6155c21f, SUC_lexicon_tags_12 },
    { 0x355c220, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd5c226, SUC_lexicon_tags_9 },
    { 0x6d75c226, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf595c22e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x8875c231, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5275c236, SUC_lexicon_tags_1 },
    { 0x2c15c237, SUC_lexicon_tags_37 },
    { 0x9ab5c237, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc355c23c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8495c23e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1555c240, SUC_lexicon_tags_0 },
    { 0xe775c240, SUC_lexicon_tags_45 },
    { 0x9fd5c242, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d5c246, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc95c249, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d35c24d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2d5c24f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f55c251, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa355c255, SUC_lexicon_tags_10 },
    { 0x71d5c255, SUC_lexicon_tags_37 },
    { 0x4e95c256, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b5c25c, SUC_lexicon_tags_56 },
    { 0xd9d5c25d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd955c25f, SUC_lexicon_tags_9 },
    { 0x1235c260, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15c263, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd5c267, SUC_lexicon_tags_2 },
    { 0x115c267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f35c26a, SUC_lexicon_tags_2 },
    { 0x2c55c26a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af5c26f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f5c273, SUC_lexicon_tags_11 },
    { 0x4e55c274, SUC_lexicon_tags_60 },
    { 0xb995c275, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc675c278, SUC_lexicon_tags_8 },
    { 0x88b5c279, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb5c27d, SUC_lexicon_tags_28 },
    { 0x3cd5c27e, SUC_lexicon_tags_61 },
    { 0x9875c27f, SUC_lexicon_tags_29 },
    { 0xdbb5c280, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff5c285, SUC_lexicon_tags_7 },
    { 0x4d15c286, SUC_lexicon_tags_37 },
    { 0x3255c287, SUC_lexicon_tags_25 },
    { 0x46b5c287, SUC_lexicon_tags_15 },
    { 0x715c287, SUC_lexicon_tags_43 },
    { 0x25b5c28a, SUC_lexicon_tags_12 },
    { 0x7235c289, SUC_lexicon_tags_15 },
    { 0xfab5c287, SUC_lexicon_tags_57 },
    { 0x1135c28d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa115c28f, SUC_lexicon_tags_12 },
    { 0x2835c28f, SUC_lexicon_tags_7 },
    { 0xb195c28f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d5c298, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe95c29f, SUC_lexicon_tags_37 },
    { 0xa715c2a0, SUC_lexicon_tags_17 },
    { 0xad35c2a1, SUC_lexicon_tags_62 },
    { 0xf0d5c2a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdf15c2a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x15d5c2a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1095c2ac, SUC_lexicon_tags_63 },
    { 0x5b15c2ac, SUC_lexicon_tags_29 },
    { 0x5c55c2ae, SUC_lexicon_tags_0 },
    { 0xbfd5c2af, SUC_lexicon_tags_9 },
    { 0xe975c2af, SUC_lexicon_tags_10 },
    { 0xe0f5c2ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd5c2b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd5c2ba, SUC_lexicon_tags_45 },
    { 0x2195c2bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53f5c2bd, SUC_lexicon_tags_25 },
    { 0x8295c2be, SUC_lexicon_tags_1 },
    { 0x9f95c2be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef95c2c3, SUC_lexicon_tags_45 },
    { 0xfdd5c2c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc55c2c6, SUC_lexicon_tags_11 },
    { 0x9e35c2c7, SUC_lexicon_tags_81 },
    { 0x8df5c2c8, SUC_lexicon_tags_7 },
    { 0xaf15c2c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5c2cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe75c2cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd5c2d6, SUC_lexicon_tags_25 },
    { 0xa555c2d6, SUC_lexicon_tags_45 },
    { 0x2095c2d8, SUC_lexicon_tags_114 },
    { 0x54d5c2d9, SUC_lexicon_tags_1 },
    { 0x5175c2da, SUC_lexicon_tags_45 },
    { 0x4e15c2d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xacf5c2dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d5c2e0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d5c2e3, SUC_lexicon_tags_9 },
    { 0xc75c2e4, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x2835c2e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7835c2e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d5c2ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5235c2f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7215c2f3, SUC_lexicon_tags_17 },
    { 0x94d5c2f3, SUC_lexicon_tags_1 },
    { 0xebb5c2f3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d5c2fc, SUC_lexicon_tags_8 },
    { 0x515c2fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a55c302, SUC_lexicon_tags_11 },
    { 0x1f55c302, SUC_lexicon_tags_47 },
    { 0xb355c303, SUC_lexicon_tags_17 },
    { 0xa355c305, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9c35c307, SUC_lexicon_tags_47 },
    { 0xd8d5c307, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b5c30b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x32f5c30d, SUC_lexicon_tags_30 },
    { 0xea15c30d, SUC_lexicon_tags_37 },
    { 0x4d75c30f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8995c314, SUC_lexicon_tags_26 },
    { 0x6d15c315, SUC_lexicon_tags_37 },
    { 0x7a95c316, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d5c31f, SUC_lexicon_tags_68 },
    { 0x2335c320, SUC_lexicon_tags_17 },
    { 0x1275c321, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf35c324, SUC_lexicon_tags_8 },
    { 0xecf5c324, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5915c32b, SUC_lexicon_tags_12 },
    { 0xb955c32c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5835c332, SUC_lexicon_tags_37 },
    { 0x2ff5c333, SUC_lexicon_tags_33 },
    { 0xf935c334, SUC_lexicon_tags_7 },
    { 0x8cb5c335, SUC_lexicon_tags_4 },
    { 0x4e15c335, SUC_lexicon_tags_37 },
    { 0xfef5c337, SUC_lexicon_tags_113 },
    { 0x43b5c333, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7b5c33a, SUC_lexicon_tags_12 },
    { 0xbd5c33b, SUC_lexicon_tags_27 },
    { 0xb15c33c, SUC_lexicon_tags_46 },
    { 0x13d5c33d, SUC_lexicon_tags_19 },
    { 0x7a15c33b, SUC_lexicon_tags_45 },
    { 0x7ed5c33e, SUC_lexicon_tags_15 },
    { 0xa175c33d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6695c344, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe9f5c346, SUC_lexicon_tags_8 },
    { 0x6555c347, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7435c349, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba95c34c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b5c34e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95d5c350, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2555c356, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb815c35a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xde95c35d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f35c363, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2055c366, SUC_lexicon_tags_27 },
    { 0x35c367, SUC_lexicon_tags_37 },
    { 0xd7d5c368, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xef55c36b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db5c376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55c37b, SUC_lexicon_tags_2 },
    { 0xd835c37b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaef5c37e, SUC_lexicon_tags_60 },
    { 0xd995c37e, SUC_lexicon_tags_14 },
    { 0x1275c37e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e55c391, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4275c393, SUC_lexicon_tags_40 },
    { 0xc435c394, SUC_lexicon_tags_27 },
    { 0x28b5c394, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fd5c397, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf5c39a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3055c39d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3d15c39f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b5c3a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7bb5c3a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd215c3ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x48b5c3ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5775c3b3, SUC_lexicon_tags_25 },
    { 0x37d5c3b4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3335c3b6, SUC_lexicon_tags_23 },
    { 0xc0b5c3b7, SUC_lexicon_tags_15 },
    { 0xa3d5c3b8, SUC_lexicon_tags_38 },
    { 0xd715c3b6, SUC_lexicon_tags_9 },
    { 0x8635c3ba, SUC_lexicon_tags_27 },
    { 0x935c3bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1255c3bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1595c3c7, SUC_lexicon_tags_3 },
    { 0x7195c3c7, SUC_lexicon_tags_7 },
    { 0x615c3c9, SUC_lexicon_tags_6 },
    { 0x93b5c3c9, SUC_lexicon_tags_27 },
    { 0x1515c3cb, SUC_lexicon_tags_92 },
    { 0xf215c3cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c75c3cf, SUC_lexicon_tags_39 },
    { 0x8d5c3d0, SUC_lexicon_tags_45 },
    { 0xe9d5c3d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2795c3d3, SUC_lexicon_tags_15 },
    { 0x60f5c3d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0f5c3d6, SUC_lexicon_tags_31 },
    { 0x4795c3d7, SUC_lexicon_tags_920 },
    { 0x12b5c3d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb2b5c3da, SUC_lexicon_tags_1 },
    { 0xa555c3da, SUC_lexicon_tags_25 },
    { 0x3e95c3dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd5c3df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf3d5c3e1, SUC_lexicon_tags_2 },
    { 0x2a35c3e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1495c3e5, SUC_lexicon_tags_47 },
    { 0xa595c3e6, SUC_lexicon_tags_77 },
    { 0x5cf5c3e5, SUC_lexicon_tags_7 },
    { 0xba75c3e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c35c3eb, SUC_lexicon_tags_1 },
    { 0x2835c3eb, SUC_lexicon_tags_25 },
    { 0xb815c3eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd5c3f0, SUC_lexicon_tags_17 },
    { 0x215c3f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xef95c3f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6b5c3f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b75c3fb, SUC_lexicon_tags_1 },
    { 0x52b5c3fc, SUC_lexicon_tags_14 },
    { 0x22b5c3fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75c400, SUC_lexicon_tags_1 },
    { 0x7c35c401, SUC_lexicon_tags_6 },
    { 0x1bd5c402, SUC_lexicon_tags_8 },
    { 0x8fd5c401, SUC_lexicon_tags_46 },
    { 0x4ab5c404, SUC_lexicon_tags_1 },
    { 0x3015c405, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f5c409, SUC_lexicon_tags_14 },
    { 0x46b5c40a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc555c40c, SUC_lexicon_tags_27 },
    { 0x99b5c40c, SUC_lexicon_tags_8 },
    { 0x9495c40e, SUC_lexicon_tags_43 },
    { 0xb55c40f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d5c412, SUC_lexicon_tags_31 },
    { 0xa395c413, SUC_lexicon_tags_33 },
    { 0x7cb5c414, SUC_lexicon_tags_7 },
    { 0xb515c415, SUC_lexicon_tags_37 },
    { 0x5fb5c416, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed5c41a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39d5c41c, SUC_lexicon_tags_1 },
    { 0xd735c41d, SUC_lexicon_tags_45 },
    { 0x90f5c41e, SUC_lexicon_tags_65 },
    { 0x6675c41e, SUC_lexicon_tags_15 },
    { 0x7395c420, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce95c422, SUC_lexicon_tags_6 },
    { 0x1b35c423, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x7295c426, SUC_lexicon_tags_17 },
    { 0x3775c426, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9075c42c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff5c42f, SUC_lexicon_tags_15 },
    { 0x1115c430, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6775c432, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db5c435, SUC_lexicon_tags_44 },
    { 0x455c435, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb735c438, SUC_lexicon_tags_26 },
    { 0x1f5c439, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5475c43d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb5c444, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbd5c446, SUC_lexicon_tags_17 },
    { 0x4d35c447, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6255c451, SUC_lexicon_tags_11 },
    { 0xd95c451, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2db5c454, SUC_lexicon_tags_62 },
    { 0xcd55c455, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa715c458, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa315c45a, SUC_lexicon_tags_1 },
    { 0xa915c45a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde15c45d, SUC_lexicon_tags_9 },
    { 0xffb5c45e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6af5c460, SUC_lexicon_tags_15 },
    { 0x995c461, SUC_lexicon_tags_37 },
    { 0x3735c462, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf15c464, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7fd5c466, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb5c46c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd5c470, SUC_lexicon_tags_25 },
    { 0x5515c471, SUC_lexicon_tags_13 },
    { 0xcc15c471, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df5c477, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5435c47a, SUC_lexicon_tags_29 },
    { 0x8735c47a, SUC_lexicon_tags_15 },
    { 0x9455c47c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22d5c47e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e35c481, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xa9f5c483, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5a35c485, SUC_lexicon_tags_2 },
    { 0x39f5c486, SUC_lexicon_tags_1 },
    { 0x92d5c486, SUC_lexicon_tags_12 },
    { 0xd675c488, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba75c48d, SUC_lexicon_tags_19 },
    { 0xbfb5c48e, SUC_lexicon_tags_26 },
    { 0xad5c48e, SUC_lexicon_tags_25 },
    { 0xf615c48f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c15c492, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f5c499, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff5c49e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f5c4a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3795c4ac, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x28b5c4ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7835c4b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a75c4b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd715c4b6, SUC_lexicon_tags_8 },
    { 0x17b5c4b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a55c4ba, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0x7695c4bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd295c4c2, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xa955c4c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23b5c4c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c95c4cf, SUC_lexicon_tags_37 },
    { 0xb195c4d0, SUC_lexicon_tags_15 },
    { 0x4e35c4d1, SUC_lexicon_tags_11 },
    { 0x8295c4d1, SUC_lexicon_tags_63 },
    { 0x27f5c4d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d5c4d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3335c4d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db5c4e4, SUC_lexicon_tags_14 },
    { 0xdcf5c4e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd75c4e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f5c4ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3d75c4f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c55c4f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe4f5c4f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1cb5c4fb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f5c4fe, SUC_lexicon_tags_16 },
    { 0xecb5c4ff, SUC_lexicon_tags_33 },
    { 0xc435c500, SUC_lexicon_tags_12 },
    { 0x7195c501, SUC_lexicon_tags_15 },
    { 0x5f95c501, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d55c504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6955c506, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d5c50f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc75c511, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b5c514, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9375c519, SUC_lexicon_tags_15 },
    { 0xa895c51a, SUC_lexicon_tags_37 },
    { 0x33d5c51b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe075c51d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x55c51f, SUC_lexicon_tags_31 },
    { 0x9555c520, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7935c524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c75c528, SUC_lexicon_tags_38 },
    { 0xb655c529, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4715c52d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69f5c52f, SUC_lexicon_tags_14 },
    { 0xa9d5c530, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c535, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d5c53a, SUC_lexicon_tags_17 },
    { 0xc5b5c53b, SUC_lexicon_tags_12 },
    { 0x22f5c53b, SUC_lexicon_tags_13 },
    { 0xf135c53d, SUC_lexicon_tags_33 },
    { 0xd475c53e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xacb5c540, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc35c543, SUC_lexicon_tags_2 },
    { 0xded5c544, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2775c548, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac75c552, SUC_lexicon_tags_6 },
    { 0x5715c553, SUC_lexicon_tags_15 },
    { 0x8b75c554, SUC_lexicon_tags_33 },
    { 0x4c35c554, SUC_lexicon_tags_12 },
    { 0xebf5c555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2175c561, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2735c568, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df5c570, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f5c578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0f5c57a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3b5c57c, SUC_lexicon_tags_30 },
    { 0x9b75c57d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4595c583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9215c585, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb75c589, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f5c58c, SUC_lexicon_tags_37 },
    { 0xb215c58d, SUC_lexicon_tags_7 },
    { 0xb455c58e, SUC_lexicon_tags_17 },
    { 0x3495c58f, SUC_lexicon_tags_2 },
    { 0x6a55c58e, SUC_lexicon_tags_11 },
    { 0xb3f5c58f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2535c595, SUC_lexicon_tags_37 },
    { 0xab5c596, SUC_lexicon_tags_4 },
    { 0x3015c596, SUC_lexicon_tags_12 },
    { 0xe615c598, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xaff5c59a, SUC_lexicon_tags_49 },
    { 0x45f5c59b, SUC_lexicon_tags_27 },
    { 0xe835c59a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3995c59e, SUC_lexicon_tags_1 },
    { 0xb715c59f, SUC_lexicon_tags_63 },
    { 0xfe75c5a0, SUC_lexicon_tags_1 },
    { 0x8835c5a1, SUC_lexicon_tags_17 },
    { 0xdc75c5a2, SUC_lexicon_tags_11 },
    { 0x7155c5a3, SUC_lexicon_tags_29 },
    { 0x5555c5a1, SUC_lexicon_tags_336 },
    { 0x5c35c5a4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c35c5a9, SUC_lexicon_tags_220 },
    { 0x6ed5c5aa, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xd8b5c5ac, SUC_lexicon_tags_7 },
    { 0x79d5c5ac, SUC_lexicon_tags_33 },
    { 0xb5c5ae, SUC_lexicon_tags_31 },
    { 0x775c5ae, SUC_lexicon_tags_1 },
    { 0x4f55c5af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5c5b4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x7e75c5b6, SUC_lexicon_tags_27 },
    { 0x8975c5b6, SUC_lexicon_tags_13 },
    { 0x5475c5b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x235c5bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb5c5be, SUC_lexicon_tags_507 },
    { 0x5f75c5bf, SUC_lexicon_tags_17 },
    { 0x3975c5bf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8855c5c2, SUC_lexicon_tags_12 },
    { 0xf875c5c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8635c5c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2955c5c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed15c5ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bd5c5cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6455c5ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295c5d5, SUC_lexicon_tags_15 },
    { 0xde15c5d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b5c5de, SUC_lexicon_tags_7 },
    { 0x2435c5de, SUC_lexicon_tags_15 },
    { 0x1a75c5df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35c5ea, SUC_lexicon_tags_26 },
    { 0x77b5c5eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6d5c5ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3955c5ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab5c5f4, SUC_lexicon_tags_32 },
    { 0xead5c5f5, SUC_lexicon_tags_12 },
    { 0xf895c5f6, SUC_lexicon_tags_33 },
    { 0x91f5c5f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf5c5fc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f75c605, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5735c60a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6295c60c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2095c60e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf35c610, SUC_lexicon_tags_45 },
    { 0x5c15c611, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e95c615, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f5c61a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4f5c61c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf375c620, SUC_lexicon_tags_31 },
    { 0xf175c621, SUC_lexicon_tags_35 },
    { 0x3315c621, SUC_lexicon_tags_38 },
    { 0xe8f5c622, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8b5c625, SUC_lexicon_tags_47 },
    { 0x4b95c626, SUC_lexicon_tags_23 },
    { 0x9055c627, SUC_lexicon_tags_11 },
    { 0x5ab5c626, SUC_lexicon_tags_37 },
    { 0xc135c629, SUC_lexicon_tags_1 },
    { 0x8775c626, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x79d5c62c, SUC_lexicon_tags_26 },
    { 0x3835c62d, SUC_lexicon_tags_11 },
    { 0xb915c62e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3055c633, SUC_lexicon_tags_2 },
    { 0xe535c633, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb5c639, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd375c63b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5395c640, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf6b5c642, SUC_lexicon_tags_27 },
    { 0xab95c642, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdf35c645, SUC_lexicon_tags_56 },
    { 0x4d35c646, SUC_lexicon_tags_23 },
    { 0x1d15c646, SUC_lexicon_tags_15 },
    { 0x1475c648, SUC_lexicon_tags_19 },
    { 0x4ed5c648, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdcb5c64b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd735c652, SUC_lexicon_tags_0 },
    { 0x2295c653, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af5c656, SUC_lexicon_tags_63 },
    { 0x215c657, SUC_lexicon_tags_17 },
    { 0x9315c658, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf75c65a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35c65e, SUC_lexicon_tags_11 },
    { 0xc795c65e, SUC_lexicon_tags_26 },
    { 0x7d5c65f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6b15c662, SUC_lexicon_tags_2 },
    { 0x2ad5c663, SUC_lexicon_tags_1 },
    { 0x8015c663, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc555c667, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f5c66a, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3175c66e, SUC_lexicon_tags_37 },
    { 0xd3d5c66f, SUC_lexicon_tags_30 },
    { 0xd935c66e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd295c674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd355c678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d5c67d, SUC_lexicon_tags_25 },
    { 0xa15c67e, SUC_lexicon_tags_11 },
    { 0x9f55c67f, SUC_lexicon_tags_23 },
    { 0x6795c680, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6c35c682, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7ad5c684, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4075c688, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d15c68a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1275c68d, SUC_lexicon_tags_25 },
    { 0xc2d5c68e, SUC_lexicon_tags_26 },
    { 0xeb55c68f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f95c692, SUC_lexicon_tags_12 },
    { 0x6f35c692, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6675c697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f5c69c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe255c6a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb035c6a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf695c6a9, SUC_lexicon_tags_37 },
    { 0xe875c6a9, SUC_lexicon_tags_57 },
    { 0x1d15c6a9, SUC_lexicon_tags_31 },
    { 0x45f5c6aa, SUC_lexicon_tags_51 },
    { 0xfb75c6ad, SUC_lexicon_tags_193 },
    { 0x1255c6ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63b5c6b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c75c6b3, SUC_lexicon_tags_37 },
    { 0x5cd5c6b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbcb5c6b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf335c6b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38d5c6ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf515c6bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3935c6be, SUC_lexicon_tags_28 },
    { 0x5f55c6bf, SUC_lexicon_tags_12 },
    { 0xce95c6c0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe35c6c3, SUC_lexicon_tags_38 },
    { 0x28f5c6c3, SUC_lexicon_tags_37 },
    { 0x1095c6c4, SUC_lexicon_tags_551 },
    { 0xd195c6c4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x5635c6c9, SUC_lexicon_tags_37 },
    { 0x9bf5c6ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c35c6cc, SUC_lexicon_tags_25 },
    { 0xaf95c6cc, SUC_lexicon_tags_106 },
    { 0x2fb5c6cd, SUC_lexicon_tags_7 },
    { 0xc3f5c6cc, SUC_lexicon_tags_33 },
    { 0x2c15c6cc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f5c6d3, SUC_lexicon_tags_34 },
    { 0xafd5c6d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f5c6d7, SUC_lexicon_tags_23 },
    { 0xf7d5c6d8, SUC_lexicon_tags_17 },
    { 0x3f15c6d7, SUC_lexicon_tags_6 },
    { 0xeff5c6d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f55c6de, SUC_lexicon_tags_10 },
    { 0x25b5c6df, SUC_lexicon_tags_17 },
    { 0xbf35c6de, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7855c6e5, SUC_lexicon_tags_37 },
    { 0x6335c6e6, SUC_lexicon_tags_12 },
    { 0xf755c6e6, SUC_lexicon_tags_27 },
    { 0x5a95c6e6, SUC_lexicon_tags_15 },
    { 0x5f95c6e8, SUC_lexicon_tags_23 },
    { 0x4095c6e7, SUC_lexicon_tags_11 },
    { 0xa735c6eb, SUC_lexicon_tags_9 },
    { 0xdd5c6eb, SUC_lexicon_tags_38 },
    { 0x3d55c6ec, SUC_lexicon_tags_7 },
    { 0x12f5c6ee, SUC_lexicon_tags_16 },
    { 0xec55c6ef, SUC_lexicon_tags_9 },
    { 0xe755c6f0, SUC_lexicon_tags_2 },
    { 0x9bf5c6ef, SUC_lexicon_tags_12 },
    { 0x9b75c6f2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b5c6fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed5c705, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d55c709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5c70c, SUC_lexicon_tags_46 },
    { 0x12f5c70c, SUC_lexicon_tags_7 },
    { 0x3475c70e, SUC_lexicon_tags_25 },
    { 0x3015c70c, SUC_lexicon_tags_15 },
    { 0x6535c710, SUC_lexicon_tags_46 },
    { 0x4275c711, SUC_lexicon_tags_67 },
    { 0xda55c711, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba95c714, SUC_lexicon_tags_17 },
    { 0xda75c715, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d15c718, SUC_lexicon_tags_46 },
    { 0x5b75c719, SUC_lexicon_tags_60 },
    { 0x85d5c719, SUC_lexicon_tags_15 },
    { 0xdd95c71a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a55c71d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x86b5c71f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf255c723, SUC_lexicon_tags_7 },
    { 0xf655c724, SUC_lexicon_tags_37 },
    { 0x85b5c724, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb5c728, SUC_lexicon_tags_23 },
    { 0x60b5c728, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf55c72f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d5c733, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4935c736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa75c738, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cf5c73a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9195c73e, SUC_lexicon_tags_1641 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95c741, SUC_lexicon_tags_15 },
    { 0x1ab5c742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa255c747, SUC_lexicon_tags_6 },
    { 0xdc15c747, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f5c74b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd5c74e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe95c750, SUC_lexicon_tags_8 },
    { 0xd6f5c750, SUC_lexicon_tags_37 },
    { 0x67f5c750, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f55c756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee95c758, SUC_lexicon_tags_11 },
    { 0xa595c759, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef5c75c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f75c75e, SUC_lexicon_tags_28 },
    { 0x75f5c75e, SUC_lexicon_tags_46 },
    { 0xebf5c75e, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c15c766, SUC_lexicon_tags_0 },
    { 0x7095c767, SUC_lexicon_tags_25 },
    { 0xaf75c768, SUC_lexicon_tags_112 },
    { 0xa815c768, SUC_lexicon_tags_37 },
    { 0x2b95c76a, SUC_lexicon_tags_28 },
    { 0x875c76b, SUC_lexicon_tags_61 },
    { 0x8955c76c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1895c775, SUC_lexicon_tags_1 },
    { 0xe255c776, SUC_lexicon_tags_11 },
    { 0x1835c777, SUC_lexicon_tags_17 },
    { 0x8fd5c778, SUC_lexicon_tags_9 },
    { 0xc0d5c779, SUC_lexicon_tags_38 },
    { 0x44f5c777, SUC_lexicon_tags_14 },
    { 0xe3b5c77b, SUC_lexicon_tags_37 },
    { 0x96b5c77c, SUC_lexicon_tags_29 },
    { 0xfd35c77d, SUC_lexicon_tags_0 },
    { 0xf415c77b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8095c780, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b5c784, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d5c789, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b5c78f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a95c792, SUC_lexicon_tags_11 },
    { 0x3575c793, SUC_lexicon_tags_65 },
    { 0x1fd5c793, SUC_lexicon_tags_7 },
    { 0x45b5c794, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b5c799, SUC_lexicon_tags_46 },
    { 0x91d5c79a, SUC_lexicon_tags_60 },
    { 0x6515c79b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2735c79d, SUC_lexicon_tags_12 },
    { 0xb4d5c79e, SUC_lexicon_tags_33 },
    { 0x6b75c79e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf775c7a5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd355c7ab, SUC_lexicon_tags_15 },
    { 0x2f95c7ac, SUC_lexicon_tags_9 },
    { 0x7bd5c7ad, SUC_lexicon_tags_7 },
    { 0x55f5c7ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfef5c7b0, SUC_lexicon_tags_11 },
    { 0x9215c7b1, SUC_lexicon_tags_2 },
    { 0xae35c7b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc975c7b4, SUC_lexicon_tags_12 },
    { 0x84d5c7b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x88d5c7b7, SUC_lexicon_tags_17 },
    { 0x7455c7b7, SUC_lexicon_tags_40 },
    { 0x2935c7b7, SUC_lexicon_tags_12 },
    { 0xe895c7b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b5c7bd, SUC_lexicon_tags_25 },
    { 0xb0d5c7be, SUC_lexicon_tags_1 },
    { 0xc5f5c7bf, SUC_lexicon_tags_23 },
    { 0x9f95c7bf, SUC_lexicon_tags_44 },
    { 0xc035c7bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a95c7c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f5c7c8, SUC_lexicon_tags_129 },
    { 0xb655c7c9, SUC_lexicon_tags_65 },
    { 0x4115c7ca, SUC_lexicon_tags_15 },
    { 0x5635c7cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f15c7ce, SUC_lexicon_tags_37 },
    { 0xe3d5c7cf, SUC_lexicon_tags_37 },
    { 0x23b5c7d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4415c7d3, SUC_lexicon_tags_37 },
    { 0x935c7d4, SUC_lexicon_tags_2 },
    { 0xb1d5c7d4, SUC_lexicon_tags_11 },
    { 0x2a55c7d6, SUC_lexicon_tags_15 },
    { 0xa755c7d7, SUC_lexicon_tags_68 },
    { 0x31b5c7d8, SUC_lexicon_tags_14 },
    { 0x8bb5c7d9, SUC_lexicon_tags_17 },
    { 0xc95c7d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f95c7dc, SUC_lexicon_tags_19 },
    { 0xc5f5c7dd, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd55c7e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda75c7e5, SUC_lexicon_tags_37 },
    { 0xe9b5c7e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd715c7ea, SUC_lexicon_tags_25 },
    { 0x4c75c7ea, SUC_lexicon_tags_49 },
    { 0x2d95c7ea, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x30d5c7ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f5c7f3, SUC_lexicon_tags_45 },
    { 0x1db5c7f4, SUC_lexicon_tags_15 },
    { 0x7735c7f5, SUC_lexicon_tags_82 },
    { 0x7095c7f3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9d35c7f8, SUC_lexicon_tags_11 },
    { 0xaad5c7f9, SUC_lexicon_tags_2 },
    { 0xaff5c7fa, SUC_lexicon_tags_25 },
    { 0xd1b5c7fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c7fe, SUC_lexicon_tags_29 },
    { 0xf015c7ff, SUC_lexicon_tags_37 },
    { 0x3495c7fe, SUC_lexicon_tags_10 },
    { 0xd355c800, SUC_lexicon_tags_38 },
    { 0xe795c802, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc3b5c804, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab5c807, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d5c80d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44b5c80f, SUC_lexicon_tags_16 },
    { 0xe75c810, SUC_lexicon_tags_2 },
    { 0x88d5c811, SUC_lexicon_tags_23 },
    { 0x58f5c810, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf475c817, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d5c81a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb5c825, SUC_lexicon_tags_25 },
    { 0x9bb5c826, SUC_lexicon_tags_31 },
    { 0xdd95c827, SUC_lexicon_tags_8 },
    { 0x2dd5c828, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x1af5c82a, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x7975c82c, SUC_lexicon_tags_12 },
    { 0xb4f5c82d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5c832, SUC_lexicon_tags_15 },
    { 0xb175c833, SUC_lexicon_tags_26 },
    { 0xa5f5c834, SUC_lexicon_tags_35 },
    { 0xe3f5c835, SUC_lexicon_tags_14 },
    { 0x3815c835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4b5c838, SUC_lexicon_tags_10 },
    { 0x2f35c839, SUC_lexicon_tags_25 },
    { 0x94f5c839, SUC_lexicon_tags_12 },
    { 0xbaf5c83b, SUC_lexicon_tags_6 },
    { 0x11f5c839, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b75c83f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7c35c841, SUC_lexicon_tags_25 },
    { 0x1eb5c842, SUC_lexicon_tags_17 },
    { 0x2395c843, SUC_lexicon_tags_38 },
    { 0xe375c844, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4315c848, SUC_lexicon_tags_1 },
    { 0xc6d5c849, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5c84c, SUC_lexicon_tags_15 },
    { 0xa55c84d, SUC_lexicon_tags_19 },
    { 0x7175c84e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b15c851, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f75c856, SUC_lexicon_tags_47 },
    { 0x3815c857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef5c85a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef75c85e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf55c869, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x1bf5c86b, SUC_lexicon_tags_19 },
    { 0x83f5c86b, SUC_lexicon_tags_30 },
    { 0x5495c86b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3395c87a, SUC_lexicon_tags_2 },
    { 0xb4b5c87a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9155c87f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3175c883, SUC_lexicon_tags_107 },
    { 0x5a55c883, SUC_lexicon_tags_13 },
    { 0x4935c885, SUC_lexicon_tags_30 },
    { 0x7435c886, SUC_lexicon_tags_8 },
    { 0x4835c884, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8235c889, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5255c88d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf935c88f, SUC_lexicon_tags_2 },
    { 0xead5c890, SUC_lexicon_tags_25 },
    { 0x6ef5c891, SUC_lexicon_tags_25 },
    { 0x595c892, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd615c898, SUC_lexicon_tags_15 },
    { 0xad35c899, SUC_lexicon_tags_28 },
    { 0x3db5c89a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3f15c89c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a75c8a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb955c8a5, SUC_lexicon_tags_7 },
    { 0xe955c8a6, SUC_lexicon_tags_1 },
    { 0x25d5c8a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe675c8aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff5c8b5, SUC_lexicon_tags_48 },
    { 0xb735c8b6, SUC_lexicon_tags_14 },
    { 0x3455c8b7, SUC_lexicon_tags_11 },
    { 0x15f5c8b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f35c8bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af5c8c3, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0x3195c8c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2995c8cb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb5c8ce, SUC_lexicon_tags_104 },
    { 0xa0b5c8ce, SUC_lexicon_tags_12 },
    { 0x2bd5c8d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3275c8d7, SUC_lexicon_tags_7 },
    { 0x8e75c8d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1695c8da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d5c8e4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8d55c8e6, SUC_lexicon_tags_1 },
    { 0x4915c8e7, SUC_lexicon_tags_17 },
    { 0xf5f5c8e8, SUC_lexicon_tags_28 },
    { 0x6055c8e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc475c8eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bb5c8ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd5c8f3, SUC_lexicon_tags_842 },
    { 0x2275c8f4, SUC_lexicon_tags_10 },
    { 0xa7f5c8f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7115c8fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc255c901, SUC_lexicon_tags_55 },
    { 0x31f5c902, SUC_lexicon_tags_15 },
    { 0x9ff5c903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab5c907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60b5c909, SUC_lexicon_tags_11 },
    { 0x4935c90a, SUC_lexicon_tags_47 },
    { 0x33d5c90b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe735c913, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xed55c915, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb5c919, SUC_lexicon_tags_17 },
    { 0xaf35c919, SUC_lexicon_tags_11 },
    { 0xa315c91a, SUC_lexicon_tags_1 },
    { 0x895c91c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b5c91f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5c922, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4f95c924, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab5c92c, SUC_lexicon_tags_27 },
    { 0x4df5c92c, SUC_lexicon_tags_1 },
    { 0xfd55c92d, SUC_lexicon_tags_11 },
    { 0x4195c92e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c55c932, SUC_lexicon_tags_15 },
    { 0x8175c933, SUC_lexicon_tags_68 },
    { 0x155c933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5c938, SUC_lexicon_tags_60 },
    { 0xf795c938, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d5c93e, SUC_lexicon_tags_2 },
    { 0xced5c93f, SUC_lexicon_tags_17 },
    { 0x3b35c940, SUC_lexicon_tags_47 },
    { 0xd0f5c941, SUC_lexicon_tags_37 },
    { 0x7895c942, SUC_lexicon_tags_12 },
    { 0xa995c943, SUC_lexicon_tags_12 },
    { 0x4db5c943, SUC_lexicon_tags_277 },
    { 0x4875c93f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e55c947, SUC_lexicon_tags_10 },
    { 0x3355c947, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xef35c94b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1575c94d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e55c94f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5815c95a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbd75c95c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b5c960, SUC_lexicon_tags_19 },
    { 0x7535c961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe935c966, SUC_lexicon_tags_63 },
    { 0x1895c966, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d55c969, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2835c96d, SUC_lexicon_tags_28 },
    { 0x97b5c96e, SUC_lexicon_tags_2 },
    { 0x5295c96f, SUC_lexicon_tags_2 },
    { 0x2515c96f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9275c974, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x52b5c976, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf35c978, SUC_lexicon_tags_2 },
    { 0x6d35c979, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xa015c97b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2375c97d, SUC_lexicon_tags_17 },
    { 0xced5c97e, SUC_lexicon_tags_7 },
    { 0xcf55c97d, SUC_lexicon_tags_0 },
    { 0x4695c980, SUC_lexicon_tags_37 },
    { 0x31b5c980, SUC_lexicon_tags_1 },
    { 0x5df5c982, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x25b5c984, SUC_lexicon_tags_14 },
    { 0x4f75c984, SUC_lexicon_tags_7 },
    { 0x1c15c986, SUC_lexicon_tags_25 },
    { 0x6f95c986, SUC_lexicon_tags_11 },
    { 0x3255c987, SUC_lexicon_tags_14 },
    { 0xb495c989, SUC_lexicon_tags_11 },
    { 0x2955c984, SUC_lexicon_tags_15 },
    { 0xe2d5c985, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f5c98e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3f5c990, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2875c992, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf75c994, SUC_lexicon_tags_43 },
    { 0x10f5c995, SUC_lexicon_tags_2 },
    { 0x4475c996, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc15c998, SUC_lexicon_tags_43 },
    { 0x2bb5c998, SUC_lexicon_tags_19 },
    { 0x7c35c998, SUC_lexicon_tags_15 },
    { 0xf9b5c998, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6c55c99d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58f5c99f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355c9a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad15c9aa, SUC_lexicon_tags_63 },
    { 0x7c35c9aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b5c9b2, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xf535c9b4, SUC_lexicon_tags_15 },
    { 0x1ed5c9b5, SUC_lexicon_tags_26 },
    { 0x40b5c9b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1655c9b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe395c9bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8875c9bf, SUC_lexicon_tags_33 },
    { 0xee15c9c0, SUC_lexicon_tags_65 },
    { 0x9d5c9c1, SUC_lexicon_tags_25 },
    { 0xda35c9c2, SUC_lexicon_tags_45 },
    { 0x3115c9c1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x95b5c9c5, SUC_lexicon_tags_79 },
    { 0x7215c9c5, SUC_lexicon_tags_1 },
    { 0x49f5c9c6, SUC_lexicon_tags_27 },
    { 0x5555c9c8, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1075c9d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad5c9d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67b5c9d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e35c9e0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9df5c9e2, SUC_lexicon_tags_8 },
    { 0x36f5c9e2, SUC_lexicon_tags_47 },
    { 0xc955c9e4, SUC_lexicon_tags_7 },
    { 0x6615c9e5, SUC_lexicon_tags_0 },
    { 0xb2d5c9e5, SUC_lexicon_tags_2 },
    { 0x9995c9e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd15c9e9, SUC_lexicon_tags_1 },
    { 0x9c15c9ea, SUC_lexicon_tags_29 },
    { 0xfff5c9ea, SUC_lexicon_tags_11 },
    { 0xa115c9ec, SUC_lexicon_tags_28 },
    { 0xe15c9eb, SUC_lexicon_tags_158 },
    { 0xaad5c9eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7d5c9f0, SUC_lexicon_tags_6 },
    { 0x4bd5c9f1, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6675c9fc, SUC_lexicon_tags_15 },
    { 0xc675c9fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb5ca03, SUC_lexicon_tags_19 },
    { 0xda95ca03, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe095ca09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d5ca0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdbf5ca0f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4795ca12, SUC_lexicon_tags_10 },
    { 0xa615ca13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef5ca17, SUC_lexicon_tags_2 },
    { 0xf695ca18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d5ca1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf5ca22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a55ca27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8535ca29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaf95ca2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d5ca32, SUC_lexicon_tags_15 },
    { 0x8cd5ca33, SUC_lexicon_tags_11 },
    { 0xc475ca34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb875ca36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f5ca39, SUC_lexicon_tags_9 },
    { 0x4695ca39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc35ca3e, SUC_lexicon_tags_60 },
    { 0x4f35ca3f, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x9495ca41, SUC_lexicon_tags_76 },
    { 0x78b5ca41, SUC_lexicon_tags_37 },
    { 0x1a95ca42, SUC_lexicon_tags_7 },
    { 0x13b5ca41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58b5ca46, SUC_lexicon_tags_11 },
    { 0x8af5ca47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd5ca4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6735ca4f, SUC_lexicon_tags_25 },
    { 0x40f5ca50, SUC_lexicon_tags_27 },
    { 0x63d5ca51, SUC_lexicon_tags_207 },
    { 0x95ca51, SUC_lexicon_tags_2 },
    { 0x32d5ca52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a15ca55, SUC_lexicon_tags_27 },
    { 0xe475ca56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d5ca5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf75ca62, SUC_lexicon_tags_0 },
    { 0xe655ca63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2375ca6a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1295ca6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d35ca70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6195ca72, SUC_lexicon_tags_7 },
    { 0x1375ca72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d5ca77, SUC_lexicon_tags_46 },
    { 0x1df5ca78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc35ca7a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe0b5ca7c, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xee95ca7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd15ca80, SUC_lexicon_tags_1 },
    { 0xb6d5ca80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd5ca85, SUC_lexicon_tags_7 },
    { 0xc8d5ca86, SUC_lexicon_tags_37 },
    { 0xb7d5ca87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf475ca89, SUC_lexicon_tags_17 },
    { 0x9895ca8a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d5ca8e, SUC_lexicon_tags_1 },
    { 0x8435ca8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf935ca91, SUC_lexicon_tags_112 },
    { 0xf935ca92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5ca96, SUC_lexicon_tags_1 },
    { 0xb355ca97, SUC_lexicon_tags_37 },
    { 0x7575ca98, SUC_lexicon_tags_37 },
    { 0xa275ca99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d55ca9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x895ca9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cb5caa1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9835caa3, SUC_lexicon_tags_3 },
    { 0x72f5caa4, SUC_lexicon_tags_17 },
    { 0xf175caa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa315caa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4295caab, SUC_lexicon_tags_6 },
    { 0x1135caab, SUC_lexicon_tags_108 },
    { 0xf7f5caad, SUC_lexicon_tags_37 },
    { 0xf1f5caae, SUC_lexicon_tags_47 },
    { 0x4535caae, SUC_lexicon_tags_11 },
    { 0x8995caae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b5cab3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2475cab9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x355cabc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8995cac0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2735cac4, SUC_lexicon_tags_66 },
    { 0xa2f5cac5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe955cac7, SUC_lexicon_tags_14 },
    { 0x2f5cac8, SUC_lexicon_tags_26 },
    { 0x8e75cac8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb55cacd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe35cad1, SUC_lexicon_tags_11 },
    { 0x66f5cad1, SUC_lexicon_tags_1 },
    { 0xf3b5cad3, SUC_lexicon_tags_12 },
    { 0x155cad1, SUC_lexicon_tags_14 },
    { 0x2815cad2, SUC_lexicon_tags_29 },
    { 0xf715cad6, SUC_lexicon_tags_17 },
    { 0x9735cad7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa175cae1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb515cae4, SUC_lexicon_tags_12 },
    { 0x7575cae5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5cae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf35caeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d55caf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b5caf5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4eb5caf7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3a55caf9, SUC_lexicon_tags_61 },
    { 0xfc55cafa, SUC_lexicon_tags_60 },
    { 0x92d5cafa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb495caff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb55cb02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb535cb04, SUC_lexicon_tags_45 },
    { 0x28f5cb05, SUC_lexicon_tags_57 },
    { 0x44f5cb05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68f5cb08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c95cb0a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xad15cb0d, SUC_lexicon_tags_2 },
    { 0x3875cb0d, SUC_lexicon_tags_31 },
    { 0x4eb5cb0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc1d5cb11, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2475cb13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9755cb19, SUC_lexicon_tags_2 },
    { 0xd495cb1a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f5cb1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7235cb21, SUC_lexicon_tags_17 },
    { 0x5dd5cb22, SUC_lexicon_tags_13 },
    { 0x7d5cb21, SUC_lexicon_tags_21 },
    { 0x40b5cb24, SUC_lexicon_tags_84 },
    { 0x2ef5cb25, SUC_lexicon_tags_23 },
    { 0xac75cb26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c55cb2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xba75cb2f, SUC_lexicon_tags_13 },
    { 0x9b35cb30, SUC_lexicon_tags_17 },
    { 0x5f35cb30, SUC_lexicon_tags_38 },
    { 0xbb15cb31, SUC_lexicon_tags_30 },
    { 0x3235cb33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6255cb38, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x335cb3a, SUC_lexicon_tags_68 },
    { 0x1e95cb3b, SUC_lexicon_tags_14 },
    { 0xe515cb3c, SUC_lexicon_tags_87 },
    { 0xc955cb3b, SUC_lexicon_tags_2 },
    { 0xc635cb3c, SUC_lexicon_tags_37 },
    { 0xb535cb3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8655cb42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8a75cb44, SUC_lexicon_tags_17 },
    { 0x3075cb45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a35cb49, SUC_lexicon_tags_59 },
    { 0x3635cb4a, SUC_lexicon_tags_66 },
    { 0xc0b5cb4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f5cb51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d5cb56, SUC_lexicon_tags_12 },
    { 0xc9b5cb57, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b55cb5a, SUC_lexicon_tags_12 },
    { 0x30b5cb5b, SUC_lexicon_tags_28 },
    { 0xd0f5cb5c, SUC_lexicon_tags_28 },
    { 0xf5d5cb5d, SUC_lexicon_tags_29 },
    { 0xdbf5cb5e, SUC_lexicon_tags_37 },
    { 0x4ff5cb5f, SUC_lexicon_tags_7 },
    { 0x4195cb5f, SUC_lexicon_tags_7 },
    { 0xe495cb5f, SUC_lexicon_tags_2 },
    { 0x7a35cb62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b5cb65, SUC_lexicon_tags_15 },
    { 0x9135cb66, SUC_lexicon_tags_11 },
    { 0x5cf5cb67, SUC_lexicon_tags_49 },
    { 0x5255cb67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c35cb6e, SUC_lexicon_tags_68 },
    { 0x3d35cb6e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xa555cb72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1115cb74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5cb7d, SUC_lexicon_tags_37 },
    { 0xeed5cb7e, SUC_lexicon_tags_14 },
    { 0x9515cb7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb35cb83, SUC_lexicon_tags_9 },
    { 0x7775cb84, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xff15cb86, SUC_lexicon_tags_27 },
    { 0x2a55cb87, SUC_lexicon_tags_1 },
    { 0x7735cb86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1035cb8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf535cb8e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2875cb90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeeb5cb92, SUC_lexicon_tags_11 },
    { 0x71b5cb92, SUC_lexicon_tags_17 },
    { 0x4395cb94, SUC_lexicon_tags_25 },
    { 0x4715cb95, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfa95cb97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6855cb9b, SUC_lexicon_tags_117 },
    { 0xe115cb9c, SUC_lexicon_tags_60 },
    { 0xc0f5cb9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa995cba3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd5cba7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd95cbaa, SUC_lexicon_tags_2 },
    { 0xd975cbaa, SUC_lexicon_tags_25 },
    { 0xc1d5cbac, SUC_lexicon_tags_0 },
    { 0x5975cbac, SUC_lexicon_tags_84 },
    { 0x2eb5cbae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73f5cbb0, SUC_lexicon_tags_2 },
    { 0xd3b5cbb1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5035cbb4, SUC_lexicon_tags_87 },
    { 0x26b5cbb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2015cbba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f5cbc6, SUC_lexicon_tags_7 },
    { 0x275cbc7, SUC_lexicon_tags_11 },
    { 0x7d15cbc8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5655cbca, SUC_lexicon_tags_21 },
    { 0xdb75cbca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d35cbcf, SUC_lexicon_tags_37 },
    { 0x47f5cbcf, SUC_lexicon_tags_37 },
    { 0x7735cbcf, SUC_lexicon_tags_12 },
    { 0x5c75cbd1, SUC_lexicon_tags_40 },
    { 0x7595cbcf, SUC_lexicon_tags_14 },
    { 0x2e75cbd4, SUC_lexicon_tags_32 },
    { 0xfdd5cbd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b5cbda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2335cbe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb055cbe5, SUC_lexicon_tags_17 },
    { 0xc9f5cbe5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6555cbe9, SUC_lexicon_tags_7 },
    { 0x2f95cbea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb495cbec, SUC_lexicon_tags_12 },
    { 0xd6f5cbed, SUC_lexicon_tags_17 },
    { 0xc8b5cbed, SUC_lexicon_tags_17 },
    { 0x48f5cbef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd955cbf1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb235cc00, SUC_lexicon_tags_1 },
    { 0x7815cc01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d5cc04, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd815cc09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae55cc0c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xde35cc0f, SUC_lexicon_tags_56 },
    { 0x8875cc10, SUC_lexicon_tags_26 },
    { 0x12f5cc10, SUC_lexicon_tags_17 },
    { 0x9635cc10, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb035cc16, SUC_lexicon_tags_29 },
    { 0xc95cc17, SUC_lexicon_tags_12 },
    { 0x1795cc17, SUC_lexicon_tags_11 },
    { 0x6cb5cc19, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b5cc1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa855cc21, SUC_lexicon_tags_7 },
    { 0xb0f5cc22, SUC_lexicon_tags_37 },
    { 0x7b95cc22, SUC_lexicon_tags_7 },
    { 0xf4d5cc24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3455cc26, SUC_lexicon_tags_34 },
    { 0x2495cc27, SUC_lexicon_tags_8 },
    { 0x22f5cc28, SUC_lexicon_tags_878 },
    { 0x2675cc29, SUC_lexicon_tags_45 },
    { 0xe5d5cc28, SUC_lexicon_tags_15 },
    { 0xe2d5cc2b, SUC_lexicon_tags_45 },
    { 0x88b5cc2c, SUC_lexicon_tags_43 },
    { 0x78b5cc2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe435cc37, SUC_lexicon_tags_11 },
    { 0xc2b5cc37, SUC_lexicon_tags_48 },
    { 0xbad5cc39, SUC_lexicon_tags_197 },
    { 0xddd5cc37, SUC_lexicon_tags_46 },
    { 0x4c35cc3b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d5cc3e, SUC_lexicon_tags_14 },
    { 0xa135cc3f, SUC_lexicon_tags_17 },
    { 0xf4f5cc3f, SUC_lexicon_tags_2 },
    { 0x9ed5cc41, SUC_lexicon_tags_136 },
    { 0x9d5cc41, SUC_lexicon_tags_37 },
    { 0x94d5cc43, SUC_lexicon_tags_42 },
    { 0xf335cc41, SUC_lexicon_tags_12 },
    { 0x6c15cc45, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf5cc4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce35cc57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f5cc5f, SUC_lexicon_tags_11 },
    { 0x7a15cc60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9b5cc62, SUC_lexicon_tags_25 },
    { 0x2915cc62, SUC_lexicon_tags_19 },
    { 0xbe55cc63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a95cc67, SUC_lexicon_tags_32 },
    { 0xbc35cc68, SUC_lexicon_tags_2 },
    { 0x7095cc69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18f5cc6b, SUC_lexicon_tags_14 },
    { 0x8855cc6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b55cc6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f95cc71, SUC_lexicon_tags_38 },
    { 0xdcd5cc71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe835cc75, SUC_lexicon_tags_2 },
    { 0xf1f5cc76, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa315cc78, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6ab5cc7a, SUC_lexicon_tags_61 },
    { 0x8875cc7b, SUC_lexicon_tags_201 },
    { 0x5a55cc7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a55cc80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b5cc83, SUC_lexicon_tags_25 },
    { 0x6935cc83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb15cc86, SUC_lexicon_tags_112 },
    { 0xc435cc86, SUC_lexicon_tags_2 },
    { 0xf995cc88, SUC_lexicon_tags_104 },
    { 0xa5f5cc89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f5cc8c, SUC_lexicon_tags_47 },
    { 0xa5f5cc8d, SUC_lexicon_tags_40 },
    { 0x17f5cc8c, SUC_lexicon_tags_8 },
    { 0x6915cc8f, SUC_lexicon_tags_17 },
    { 0x1e15cc90, SUC_lexicon_tags_19 },
    { 0x7ad5cc91, SUC_lexicon_tags_11 },
    { 0x6df5cc92, SUC_lexicon_tags_1 },
    { 0x2955cc92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2f55cc95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3315cc99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e75cc9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b15cca3, SUC_lexicon_tags_24 },
    { 0xf575cca4, SUC_lexicon_tags_103 },
    { 0x9df5cca4, SUC_lexicon_tags_38 },
    { 0xfcb5cca3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4275cca9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b5ccac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe95ccae, SUC_lexicon_tags_230 },
    { 0x11b5ccaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe195ccb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf5ccb5, SUC_lexicon_tags_6 },
    { 0x71f5ccb6, SUC_lexicon_tags_60 },
    { 0xd475ccb7, SUC_lexicon_tags_7 },
    { 0x2fb5ccb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd35ccbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6975ccbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9235ccc3, SUC_lexicon_tags_25 },
    { 0x5695ccc4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d35ccc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5ccca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb5cccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9df5cccf, SUC_lexicon_tags_11 },
    { 0x2b95cccf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1395ccd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f95ccd7, SUC_lexicon_tags_25 },
    { 0xf9b5ccd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1615ccdc, SUC_lexicon_tags_17 },
    { 0xc575ccdd, SUC_lexicon_tags_8 },
    { 0xfe75ccde, SUC_lexicon_tags_37 },
    { 0xc375ccdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa315cce1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe595cce3, SUC_lexicon_tags_30 },
    { 0x9cb5cce4, SUC_lexicon_tags_37 },
    { 0x36f5cce5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675ccea, SUC_lexicon_tags_19 },
    { 0x3275cceb, SUC_lexicon_tags_17 },
    { 0xd175ccec, SUC_lexicon_tags_33 },
    { 0xc155cceb, SUC_lexicon_tags_11 },
    { 0x9a75cceb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc395ccf0, SUC_lexicon_tags_12 },
    { 0x3d75ccf1, SUC_lexicon_tags_1 },
    { 0x4035ccf2, SUC_lexicon_tags_33 },
    { 0x2b75ccf3, SUC_lexicon_tags_38 },
    { 0xec35ccf1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4455ccf8, SUC_lexicon_tags_7 },
    { 0x155ccf9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d5ccfd, SUC_lexicon_tags_29 },
    { 0xdd75ccfd, SUC_lexicon_tags_37 },
    { 0x53f5ccff, SUC_lexicon_tags_17 },
    { 0x7815ccff, SUC_lexicon_tags_27 },
    { 0x3d35cd01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b5cd04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b5cd09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd5cd0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b5cd10, SUC_lexicon_tags_39 },
    { 0x2335cd11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b5cd15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad35cd19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb5cd1c, SUC_lexicon_tags_17 },
    { 0x2695cd1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f5cd20, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5cd26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b55cd28, SUC_lexicon_tags_23 },
    { 0x1355cd29, SUC_lexicon_tags_7 },
    { 0x4255cd2a, SUC_lexicon_tags_25 },
    { 0xc055cd2b, SUC_lexicon_tags_47 },
    { 0xdaf5cd2a, SUC_lexicon_tags_17 },
    { 0xb0f5cd2d, SUC_lexicon_tags_39 },
    { 0x12f5cd2d, SUC_lexicon_tags_7 },
    { 0xe0d5cd2e, SUC_lexicon_tags_99 },
    { 0x3a35cd30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1695cd36, SUC_lexicon_tags_35 },
    { 0x16f5cd37, SUC_lexicon_tags_63 },
    { 0xcf5cd37, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe795cd3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4675cd3d, SUC_lexicon_tags_2 },
    { 0xc435cd3e, SUC_lexicon_tags_1 },
    { 0xb415cd3f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8295cd41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb595cd44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cb5cd46, SUC_lexicon_tags_4 },
    { 0x6715cd47, SUC_lexicon_tags_82 },
    { 0x31b5cd47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f5cd4d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2995cd50, SUC_lexicon_tags_68 },
    { 0xf8d5cd50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2135cd53, SUC_lexicon_tags_23 },
    { 0x7075cd53, SUC_lexicon_tags_7 },
    { 0x3af5cd55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7435cd59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c35cd5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab5cd5e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c95cd62, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7855cd6c, SUC_lexicon_tags_12 },
    { 0xc2b5cd6d, SUC_lexicon_tags_82 },
    { 0xbcb5cd6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4435cd72, SUC_lexicon_tags_11 },
    { 0x6a35cd73, SUC_lexicon_tags_44 },
    { 0x56b5cd72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5355cd77, SUC_lexicon_tags_31 },
    { 0xdf35cd77, SUC_lexicon_tags_17 },
    { 0xab35cd79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9855cd7e, SUC_lexicon_tags_1 },
    { 0xab55cd7f, SUC_lexicon_tags_49 },
    { 0xc4d5cd80, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe755cd82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f5cd8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d5cd90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd035cd95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d35cd97, SUC_lexicon_tags_6 },
    { 0x6cb5cd97, SUC_lexicon_tags_30 },
    { 0xf175cd99, SUC_lexicon_tags_7 },
    { 0xa795cd98, SUC_lexicon_tags_12 },
    { 0xad5cd9b, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0xdfd5cd9d, SUC_lexicon_tags_70 },
    { 0xdd35cd9e, SUC_lexicon_tags_11 },
    { 0x1f95cd9e, SUC_lexicon_tags_0 },
    { 0xe8f5cd9e, SUC_lexicon_tags_25 },
    { 0x7ed5cda1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5cda6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9575cda9, SUC_lexicon_tags_43 },
    { 0xec95cdaa, SUC_lexicon_tags_69 },
    { 0x9275cdab, SUC_lexicon_tags_87 },
    { 0xf35cdac, SUC_lexicon_tags_9 },
    { 0x25b5cdad, SUC_lexicon_tags_15 },
    { 0xbd15cdac, SUC_lexicon_tags_37 },
    { 0xd615cdaf, SUC_lexicon_tags_15 },
    { 0x76b5cdae, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a75cdb3, SUC_lexicon_tags_15 },
    { 0xea15cdb4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c75cdbf, SUC_lexicon_tags_14 },
    { 0x6235cdc0, SUC_lexicon_tags_0 },
    { 0xf7d5cdc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7435cdc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b15cdce, SUC_lexicon_tags_12 },
    { 0xa35cdcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f35cdd4, SUC_lexicon_tags_31 },
    { 0x3d75cdd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f5cddd, SUC_lexicon_tags_19 },
    { 0x6fb5cdde, SUC_lexicon_tags_32 },
    { 0xf4d5cddd, SUC_lexicon_tags_15 },
    { 0x4c15cddd, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x7835cde2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5cde6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91f5cde8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbb75cdea, SUC_lexicon_tags_15 },
    { 0x335cdeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f5cdf5, SUC_lexicon_tags_26 },
    { 0x3b95cdf6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4635cdf9, SUC_lexicon_tags_11 },
    { 0x3d75cdf9, SUC_lexicon_tags_11 },
    { 0x3155cdfa, SUC_lexicon_tags_11 },
    { 0xf975cdfb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd435cdff, SUC_lexicon_tags_4 },
    { 0x7095cdff, SUC_lexicon_tags_282 },
    { 0x415ce01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc15ce0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f55ce11, SUC_lexicon_tags_31 },
    { 0x1f55ce12, SUC_lexicon_tags_37 },
    { 0x4d95ce13, SUC_lexicon_tags_1 },
    { 0xe455ce13, SUC_lexicon_tags_37 },
    { 0xe035ce15, SUC_lexicon_tags_63 },
    { 0x8e35ce16, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df5ce19, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf655ce1c, SUC_lexicon_tags_7 },
    { 0xde55ce1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9895ce1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4775ce21, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f5ce2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6015ce2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4715ce31, SUC_lexicon_tags_11 },
    { 0xa35ce32, SUC_lexicon_tags_2 },
    { 0x5a35ce33, SUC_lexicon_tags_26 },
    { 0x1b5ce33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf495ce3e, SUC_lexicon_tags_12 },
    { 0x5cd5ce3f, SUC_lexicon_tags_1 },
    { 0x15b5ce40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81b5ce42, SUC_lexicon_tags_11 },
    { 0xda95ce43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb455ce45, SUC_lexicon_tags_32 },
    { 0xb4f5ce46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c35ce48, SUC_lexicon_tags_33 },
    { 0x7915ce48, SUC_lexicon_tags_23 },
    { 0xead5ce4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83b5ce4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb5ce50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc755ce52, SUC_lexicon_tags_38 },
    { 0x1bf5ce53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c15ce55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b5ce5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14f5ce5c, SUC_lexicon_tags_22 },
    { 0x4b55ce5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa1b5ce5f, SUC_lexicon_tags_28 },
    { 0x4eb5ce5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba75ce63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4575ce68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7755ce6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbd55ce6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f95ce6e, SUC_lexicon_tags_63 },
    { 0xf835ce6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc135ce71, SUC_lexicon_tags_7 },
    { 0x915ce71, SUC_lexicon_tags_11 },
    { 0xcb55ce73, SUC_lexicon_tags_31 },
    { 0xf355ce71, SUC_lexicon_tags_15 },
    { 0xadf5ce75, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0x5295ce78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b5ce7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8295ce7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4415ce81, SUC_lexicon_tags_9 },
    { 0xb335ce82, SUC_lexicon_tags_12 },
    { 0x8bf5ce83, SUC_lexicon_tags_63 },
    { 0x37d5ce84, SUC_lexicon_tags_93 },
    { 0xcb15ce85, SUC_lexicon_tags_63 },
    { 0xf0b5ce86, SUC_lexicon_tags_1 },
    { 0x8095ce83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2595ce8a, SUC_lexicon_tags_37 },
    { 0xd035ce8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa75ce8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e15ce90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f5ce9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b5ce9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7155cea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd555cea8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x955ceaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc835ceb0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef5ceb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f5ceb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b5ceba, SUC_lexicon_tags_45 },
    { 0xd3b5cebb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd5cebe, SUC_lexicon_tags_7 },
    { 0x4b75cebf, SUC_lexicon_tags_14 },
    { 0xae35cec0, SUC_lexicon_tags_47 },
    { 0xab75cebf, SUC_lexicon_tags_5 },
    { 0x92d5cec1, SUC_lexicon_tags_10 },
    { 0x9a35cec3, SUC_lexicon_tags_23 },
    { 0x9975cec2, SUC_lexicon_tags_1 },
    { 0xccb5cec1, SUC_lexicon_tags_19 },
    { 0xfcf5cec3, SUC_lexicon_tags_7 },
    { 0xfa95cec7, SUC_lexicon_tags_4 },
    { 0x4375cec8, SUC_lexicon_tags_25 },
    { 0xcc15cec8, SUC_lexicon_tags_29 },
    { 0x2e55ceca, SUC_lexicon_tags_25 },
    { 0x2535cec8, SUC_lexicon_tags_48 },
    { 0xd515ceca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc55ced0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575ced6, SUC_lexicon_tags_16 },
    { 0x9295ced7, SUC_lexicon_tags_1 },
    { 0x9bf5ced8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa955cee1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc15ceeb, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f5cef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1635cef5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4395cefd, SUC_lexicon_tags_37 },
    { 0x3db5cefe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7975cf05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeaf5cf07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f5cf0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2915cf0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b5cf10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f5cf13, SUC_lexicon_tags_14 },
    { 0xc155cf14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef5cf1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef5cf21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d75cf23, SUC_lexicon_tags_11 },
    { 0x2c55cf23, SUC_lexicon_tags_5 },
    { 0xa95cf24, SUC_lexicon_tags_1 },
    { 0x40d5cf26, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab5cf2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa555cf2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95cf33, SUC_lexicon_tags_87 },
    { 0xa795cf34, SUC_lexicon_tags_81 },
    { 0x6815cf35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b5cf39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2f55cf3b, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f5cf3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f15cf4f, SUC_lexicon_tags_37 },
    { 0x39b5cf4f, SUC_lexicon_tags_14 },
    { 0xa7d5cf51, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b15cf54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b5cf5b, SUC_lexicon_tags_33 },
    { 0x8e35cf5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1035cf63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f35cf6a, SUC_lexicon_tags_14 },
    { 0xdb75cf6b, SUC_lexicon_tags_16 },
    { 0x11f5cf6c, SUC_lexicon_tags_25 },
    { 0xa8f5cf6c, SUC_lexicon_tags_17 },
    { 0x7575cf6d, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d5cf71, SUC_lexicon_tags_48 },
    { 0xac15cf72, SUC_lexicon_tags_31 },
    { 0xc435cf73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd15cf7c, SUC_lexicon_tags_16 },
    { 0xe4d5cf7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4555cf7f, SUC_lexicon_tags_38 },
    { 0xb5b5cf80, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b5cf89, SUC_lexicon_tags_6 },
    { 0xdf75cf8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf15cf8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c75cf92, SUC_lexicon_tags_17 },
    { 0x7bf5cf93, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d5cf96, SUC_lexicon_tags_25 },
    { 0x1b35cf96, SUC_lexicon_tags_45 },
    { 0xf975cf98, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d5cf9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d75cfa3, SUC_lexicon_tags_25 },
    { 0x1db5cfa4, SUC_lexicon_tags_11 },
    { 0xc7f5cfa3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1735cfa7, SUC_lexicon_tags_3 },
    { 0x7575cfa8, SUC_lexicon_tags_48 },
    { 0x3c35cfa8, SUC_lexicon_tags_63 },
    { 0x4855cfaa, SUC_lexicon_tags_30 },
    { 0xe6b5cfab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3455cfad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c55cfb5, SUC_lexicon_tags_19 },
    { 0x7035cfb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6735cfb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf5cfbf, SUC_lexicon_tags_7 },
    { 0xe9b5cfc0, SUC_lexicon_tags_14 },
    { 0xbd95cfc1, SUC_lexicon_tags_25 },
    { 0x9235cfc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb15cfc4, SUC_lexicon_tags_1 },
    { 0x56f5cfc5, SUC_lexicon_tags_63 },
    { 0x55d5cfc6, SUC_lexicon_tags_60 },
    { 0x46b5cfc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf5cfce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2915cfd4, SUC_lexicon_tags_15 },
    { 0xb15cfd5, SUC_lexicon_tags_613 },
    { 0, NULL },
    { 0xb815cfd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbcf5cfd9, SUC_lexicon_tags_14 },
    { 0xa115cfda, SUC_lexicon_tags_38 },
    { 0x90d5cfdb, SUC_lexicon_tags_45 },
    { 0x4d75cfdc, SUC_lexicon_tags_5 },
    { 0xc2b5cfdd, SUC_lexicon_tags_1 },
    { 0x2e55cfdd, SUC_lexicon_tags_66 },
    { 0xdf75cfdf, SUC_lexicon_tags_2 },
    { 0x2035cfdd, SUC_lexicon_tags_49 },
    { 0x955cfda, SUC_lexicon_tags_6 },
    { 0xac15cfe2, SUC_lexicon_tags_7 },
    { 0xbf5cfe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64f5cfe5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6955cfe8, SUC_lexicon_tags_37 },
    { 0x2495cfe9, SUC_lexicon_tags_31 },
    { 0x5395cfe9, SUC_lexicon_tags_87 },
    { 0xd295cfeb, SUC_lexicon_tags_20 },
    { 0xaad5cfeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f5cff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf5cff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3f5cff6, SUC_lexicon_tags_113 },
    { 0xda35cff6, SUC_lexicon_tags_224 },
    { 0xa095cff7, SUC_lexicon_tags_14 },
    { 0x4c15cff9, SUC_lexicon_tags_2 },
    { 0xe9d5cffa, SUC_lexicon_tags_60 },
    { 0x43f5cff6, SUC_lexicon_tags_4 },
    { 0x9d15cffc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98d5cffe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbcb5d000, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf715d003, SUC_lexicon_tags_37 },
    { 0xd75d004, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb595d006, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d55d009, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf95d00c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x39f5d00e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x855d011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b5d014, SUC_lexicon_tags_27 },
    { 0x9475d015, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6615d017, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d5d024, SUC_lexicon_tags_81 },
    { 0x9335d024, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2755d02d, SUC_lexicon_tags_24 },
    { 0x22b5d02d, SUC_lexicon_tags_18 },
    { 0x9415d02f, SUC_lexicon_tags_15 },
    { 0x6155d030, SUC_lexicon_tags_39 },
    { 0x4715d030, SUC_lexicon_tags_30 },
    { 0x9215d030, SUC_lexicon_tags_85 },
    { 0x6775d02e, SUC_lexicon_tags_12 },
    { 0x2795d034, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc355d038, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf395d03d, SUC_lexicon_tags_1 },
    { 0xce75d03e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2475d040, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef5d045, SUC_lexicon_tags_37 },
    { 0x5755d046, SUC_lexicon_tags_1 },
    { 0x4eb5d047, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd175d04d, SUC_lexicon_tags_14 },
    { 0xac35d04d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6615d051, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6b95d053, SUC_lexicon_tags_66 },
    { 0x1bd5d054, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd675d057, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d15d061, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf835d064, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec75d067, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d06a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b75d06d, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x3655d06f, SUC_lexicon_tags_15 },
    { 0x5495d06f, SUC_lexicon_tags_39 },
    { 0x1135d070, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f5d07c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a55d07e, SUC_lexicon_tags_43 },
    { 0xe695d07f, SUC_lexicon_tags_0 },
    { 0x935d07f, SUC_lexicon_tags_49 },
    { 0xc975d081, SUC_lexicon_tags_9 },
    { 0x9435d082, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x81f5d084, SUC_lexicon_tags_17 },
    { 0x2bd5d084, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3495d087, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef5d08a, SUC_lexicon_tags_12 },
    { 0xeb5d08b, SUC_lexicon_tags_25 },
    { 0x735d08b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9835d098, SUC_lexicon_tags_11 },
    { 0xa635d098, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfd5d09b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff55d09f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f5d0a3, SUC_lexicon_tags_27 },
    { 0xcd15d0a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d5d0a8, SUC_lexicon_tags_14 },
    { 0xb5d0a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b5d0b0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc475d0b5, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xe435d0b7, SUC_lexicon_tags_7 },
    { 0x1e35d0b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae75d0c1, SUC_lexicon_tags_38 },
    { 0x7e75d0c2, SUC_lexicon_tags_17 },
    { 0x2c35d0c3, SUC_lexicon_tags_33 },
    { 0x7e15d0c2, SUC_lexicon_tags_28 },
    { 0xc475d0c5, SUC_lexicon_tags_2 },
    { 0x8a55d0c5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd9d5d0c8, SUC_lexicon_tags_14 },
    { 0x9255d0c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5d0cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd475d0d1, SUC_lexicon_tags_9 },
    { 0xdfb5d0d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb235d0d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c55d0db, SUC_lexicon_tags_1 },
    { 0x7715d0dc, SUC_lexicon_tags_26 },
    { 0xd8b5d0dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb5d0e2, SUC_lexicon_tags_35 },
    { 0xf075d0e3, SUC_lexicon_tags_15 },
    { 0xeeb5d0e3, SUC_lexicon_tags_25 },
    { 0xfb35d0e5, SUC_lexicon_tags_15 },
    { 0xaf95d0e5, SUC_lexicon_tags_1 },
    { 0xe455d0e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf855d0e9, SUC_lexicon_tags_2 },
    { 0x50d5d0e9, SUC_lexicon_tags_56 },
    { 0xbd15d0ea, SUC_lexicon_tags_15 },
    { 0xf0d5d0eb, SUC_lexicon_tags_1 },
    { 0x4cb5d0ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc495d0f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe15d0f4, SUC_lexicon_tags_1 },
    { 0x1975d0f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255d0f9, SUC_lexicon_tags_15 },
    { 0xf835d0fa, SUC_lexicon_tags_37 },
    { 0xea75d0fb, SUC_lexicon_tags_16 },
    { 0xb915d0fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f5d0ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9735d102, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x575d104, SUC_lexicon_tags_12 },
    { 0x5e55d105, SUC_lexicon_tags_56 },
    { 0x8d55d104, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d5d10a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1df5d10c, SUC_lexicon_tags_60 },
    { 0x5275d10c, SUC_lexicon_tags_47 },
    { 0xd815d10d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6175d110, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d5d119, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7255d11f, SUC_lexicon_tags_27 },
    { 0x53b5d120, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8335d122, SUC_lexicon_tags_23 },
    { 0x2215d123, SUC_lexicon_tags_2 },
    { 0x7bd5d123, SUC_lexicon_tags_15 },
    { 0x9815d125, SUC_lexicon_tags_49 },
    { 0x3095d125, SUC_lexicon_tags_15 },
    { 0xfbb5d125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7675d12c, SUC_lexicon_tags_34 },
    { 0x47d5d12d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd5d134, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b5d13a, SUC_lexicon_tags_15 },
    { 0x2595d13b, SUC_lexicon_tags_10 },
    { 0x555d13a, SUC_lexicon_tags_15 },
    { 0xfc75d13a, SUC_lexicon_tags_20 },
    { 0x1f35d13d, SUC_lexicon_tags_27 },
    { 0x9875d13f, SUC_lexicon_tags_563 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf5d145, SUC_lexicon_tags_25 },
    { 0xb195d146, SUC_lexicon_tags_32 },
    { 0x10f5d147, SUC_lexicon_tags_1 },
    { 0xa2f5d146, SUC_lexicon_tags_45 },
    { 0xf4f5d149, SUC_lexicon_tags_25 },
    { 0x64d5d14a, SUC_lexicon_tags_2 },
    { 0x3295d14a, SUC_lexicon_tags_63 },
    { 0xd3d5d14c, SUC_lexicon_tags_55 },
    { 0xce55d14d, SUC_lexicon_tags_47 },
    { 0xaed5d14d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f5d152, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d5d156, SUC_lexicon_tags_12 },
    { 0x82f5d156, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd5d15d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5635d15f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb35d161, SUC_lexicon_tags_55 },
    { 0x275d161, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x33d5d164, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd595d16a, SUC_lexicon_tags_11 },
    { 0xfff5d16b, SUC_lexicon_tags_1 },
    { 0x2e35d16c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd935d16f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f75d172, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb5d175, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3435d17a, SUC_lexicon_tags_25 },
    { 0x8eb5d17a, SUC_lexicon_tags_1 },
    { 0xc4b5d17b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6115d17e, SUC_lexicon_tags_12 },
    { 0x9d75d17f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd55d186, SUC_lexicon_tags_25 },
    { 0xf415d186, SUC_lexicon_tags_17 },
    { 0x2e75d188, SUC_lexicon_tags_79 },
    { 0x7b75d187, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2715d18d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5795d190, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2695d193, SUC_lexicon_tags_57 },
    { 0xce35d194, SUC_lexicon_tags_15 },
    { 0x8255d195, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x9595d198, SUC_lexicon_tags_17 },
    { 0xa995d199, SUC_lexicon_tags_60 },
    { 0x8795d19a, SUC_lexicon_tags_2 },
    { 0xf355d19a, SUC_lexicon_tags_11 },
    { 0x675d199, SUC_lexicon_tags_11 },
    { 0xf6d5d19d, SUC_lexicon_tags_55 },
    { 0x6eb5d19d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef35d1ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5375d1b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd95d1b8, SUC_lexicon_tags_17 },
    { 0xdc75d1b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d5d1bc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe655d1c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d5d1c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d75d1c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d5d1d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb5d1d4, SUC_lexicon_tags_29 },
    { 0xaab5d1d4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a35d1db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6355d1de, SUC_lexicon_tags_140 },
    { 0xf2d5d1de, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa175d1e1, SUC_lexicon_tags_26 },
    { 0x4fb5d1e2, SUC_lexicon_tags_1 },
    { 0x14b5d1e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb5d1e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d5d1e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd5d1eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa615d1ed, SUC_lexicon_tags_39 },
    { 0x74f5d1ed, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775d1f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10f5d1f6, SUC_lexicon_tags_31 },
    { 0xd4f5d1f6, SUC_lexicon_tags_1 },
    { 0xb835d1f8, SUC_lexicon_tags_12 },
    { 0xc055d1f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35d1ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5775d201, SUC_lexicon_tags_37 },
    { 0x4cd5d202, SUC_lexicon_tags_1 },
    { 0x2f5d203, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d75d206, SUC_lexicon_tags_14 },
    { 0xfbd5d206, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4395d20f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3615d213, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7155d215, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb5d21a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db5d21d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3f55d21f, SUC_lexicon_tags_17 },
    { 0x87b5d21f, SUC_lexicon_tags_25 },
    { 0xb195d221, SUC_lexicon_tags_2 },
    { 0xe035d21f, SUC_lexicon_tags_63 },
    { 0x85d5d223, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b5d229, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b75d22c, SUC_lexicon_tags_32 },
    { 0x975d22d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe415d230, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc475d232, SUC_lexicon_tags_30 },
    { 0x9095d233, SUC_lexicon_tags_1 },
    { 0x54d5d232, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5675d236, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb35d23c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f5d23e, SUC_lexicon_tags_11 },
    { 0xf115d23e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4315d244, SUC_lexicon_tags_1 },
    { 0xff75d245, SUC_lexicon_tags_7 },
    { 0xda55d246, SUC_lexicon_tags_35 },
    { 0x6f55d247, SUC_lexicon_tags_79 },
    { 0x4895d248, SUC_lexicon_tags_17 },
    { 0x83d5d249, SUC_lexicon_tags_64 },
    { 0x8f95d248, SUC_lexicon_tags_2 },
    { 0xf6f5d245, SUC_lexicon_tags_1 },
    { 0xa655d24a, SUC_lexicon_tags_4 },
    { 0xd6f5d247, SUC_lexicon_tags_15 },
    { 0x27b5d247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde55d250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f55d253, SUC_lexicon_tags_7 },
    { 0xd375d253, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c35d257, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfe15d259, SUC_lexicon_tags_11 },
    { 0x6335d259, SUC_lexicon_tags_48 },
    { 0x7c55d25a, SUC_lexicon_tags_39 },
    { 0x59d5d259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9335d25e, SUC_lexicon_tags_9 },
    { 0x7275d25f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d5d268, SUC_lexicon_tags_47 },
    { 0x6a55d268, SUC_lexicon_tags_17 },
    { 0xa4b5d26a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x97d5d26c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x39f5d26e, SUC_lexicon_tags_21 },
    { 0x33f5d26e, SUC_lexicon_tags_38 },
    { 0xd135d270, SUC_lexicon_tags_9 },
    { 0x2dd5d271, SUC_lexicon_tags_37 },
    { 0x4655d272, SUC_lexicon_tags_60 },
    { 0xa5d5d271, SUC_lexicon_tags_25 },
    { 0xe535d26e, SUC_lexicon_tags_17 },
    { 0xd755d275, SUC_lexicon_tags_2 },
    { 0xcf95d276, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a75d279, SUC_lexicon_tags_19 },
    { 0xcd75d27a, SUC_lexicon_tags_38 },
    { 0xf4d5d27b, SUC_lexicon_tags_15 },
    { 0xa4b5d27b, SUC_lexicon_tags_31 },
    { 0x3c75d27c, SUC_lexicon_tags_250 },
    { 0x8375d27e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed55d283, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d5d286, SUC_lexicon_tags_23 },
    { 0x6cb5d287, SUC_lexicon_tags_2 },
    { 0x4775d287, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6075d28a, SUC_lexicon_tags_25 },
    { 0x2115d28b, SUC_lexicon_tags_11 },
    { 0x6355d28c, SUC_lexicon_tags_12 },
    { 0xd2f5d28d, SUC_lexicon_tags_112 },
    { 0x9655d28e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb5d293, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d5d296, SUC_lexicon_tags_8 },
    { 0xa4d5d296, SUC_lexicon_tags_12 },
    { 0x4b55d296, SUC_lexicon_tags_35 },
    { 0xf6d5d299, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf5d29c, SUC_lexicon_tags_27 },
    { 0x1d15d29c, SUC_lexicon_tags_25 },
    { 0x44d5d29e, SUC_lexicon_tags_7 },
    { 0x5315d29d, SUC_lexicon_tags_17 },
    { 0xf535d2a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x75f5d2a2, SUC_lexicon_tags_29 },
    { 0x9935d2a3, SUC_lexicon_tags_15 },
    { 0x81d5d2a4, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d5d2a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5255d2a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9235d2ad, SUC_lexicon_tags_63 },
    { 0x2755d2ae, SUC_lexicon_tags_17 },
    { 0x3275d2af, SUC_lexicon_tags_12 },
    { 0x7ed5d2b0, SUC_lexicon_tags_31 },
    { 0xad15d2ad, SUC_lexicon_tags_15 },
    { 0x69b5d2b1, SUC_lexicon_tags_43 },
    { 0x22d5d2b3, SUC_lexicon_tags_79 },
    { 0x8535d2ae, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e55d2b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5075d2ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb5d2c0, SUC_lexicon_tags_11 },
    { 0x8b15d2c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d5d2c4, SUC_lexicon_tags_7 },
    { 0x98d5d2c4, SUC_lexicon_tags_29 },
    { 0xbc75d2c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2415d2cc, SUC_lexicon_tags_25 },
    { 0x6b15d2cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd5d2d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa375d2d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f5d2da, SUC_lexicon_tags_21 },
    { 0x58d5d2da, SUC_lexicon_tags_37 },
    { 0xc575d2dc, SUC_lexicon_tags_17 },
    { 0xcfd5d2dc, SUC_lexicon_tags_10 },
    { 0xad5d2de, SUC_lexicon_tags_23 },
    { 0x3e15d2de, SUC_lexicon_tags_13 },
    { 0xb5f5d2e0, SUC_lexicon_tags_7 },
    { 0x1075d2dd, SUC_lexicon_tags_15 },
    { 0x2775d2e1, SUC_lexicon_tags_27 },
    { 0x4f75d2e3, SUC_lexicon_tags_17 },
    { 0x3f15d2da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb335d2ea, SUC_lexicon_tags_29 },
    { 0x5735d2eb, SUC_lexicon_tags_16 },
    { 0x1595d2ec, SUC_lexicon_tags_15 },
    { 0x55b5d2ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d5d2f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x18f5d2f5, SUC_lexicon_tags_25 },
    { 0x8a15d2f6, SUC_lexicon_tags_0 },
    { 0xbbd5d2f7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x4175d2f9, SUC_lexicon_tags_34 },
    { 0xf755d2fa, SUC_lexicon_tags_84 },
    { 0x1bf5d2fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6275d2fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2935d302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb5d304, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c75d307, SUC_lexicon_tags_33 },
    { 0xaa55d307, SUC_lexicon_tags_17 },
    { 0xcab5d309, SUC_lexicon_tags_15 },
    { 0x5c35d30a, SUC_lexicon_tags_1 },
    { 0x9b35d308, SUC_lexicon_tags_17 },
    { 0x18d5d30c, SUC_lexicon_tags_7 },
    { 0x6535d30c, SUC_lexicon_tags_65 },
    { 0xb615d30d, SUC_lexicon_tags_10 },
    { 0xf1f5d30f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf795d314, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb375d316, SUC_lexicon_tags_25 },
    { 0x5975d317, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f5d31b, SUC_lexicon_tags_21 },
    { 0x2535d31b, SUC_lexicon_tags_70 },
    { 0xc9d5d31b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3235d323, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd615d326, SUC_lexicon_tags_60 },
    { 0x8695d326, SUC_lexicon_tags_17 },
    { 0xa635d327, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cb5d32a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af5d332, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdb5d334, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa595d337, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed95d33d, SUC_lexicon_tags_37 },
    { 0xbb75d33e, SUC_lexicon_tags_48 },
    { 0x95d33f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a75d342, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xebd5d344, SUC_lexicon_tags_37 },
    { 0x8ed5d345, SUC_lexicon_tags_23 },
    { 0x9875d346, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb495d34c, SUC_lexicon_tags_108 },
    { 0x8435d34d, SUC_lexicon_tags_37 },
    { 0xd295d34e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3655d353, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc975d355, SUC_lexicon_tags_37 },
    { 0xfbf5d355, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af5d35a, SUC_lexicon_tags_4 },
    { 0x6875d35a, SUC_lexicon_tags_35 },
    { 0x4e35d35c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab5d363, SUC_lexicon_tags_12 },
    { 0xcf95d364, SUC_lexicon_tags_12 },
    { 0x7295d364, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a15d368, SUC_lexicon_tags_15 },
    { 0x7755d368, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5d36d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef5d370, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb5d374, SUC_lexicon_tags_12 },
    { 0x7055d374, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef5d378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d55d37e, SUC_lexicon_tags_63 },
    { 0xfcb5d37f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d5d385, SUC_lexicon_tags_17 },
    { 0x97b5d386, SUC_lexicon_tags_19 },
    { 0xf8d5d386, SUC_lexicon_tags_0 },
    { 0x8cb5d388, SUC_lexicon_tags_68 },
    { 0x9795d389, SUC_lexicon_tags_15 },
    { 0x8975d38a, SUC_lexicon_tags_10 },
    { 0x44f5d38a, SUC_lexicon_tags_37 },
    { 0xab15d38c, SUC_lexicon_tags_25 },
    { 0x5295d38a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff55d390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe035d392, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf795d397, SUC_lexicon_tags_2 },
    { 0x7b5d397, SUC_lexicon_tags_9 },
    { 0x9c55d398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa315d39c, SUC_lexicon_tags_11 },
    { 0x8795d39c, SUC_lexicon_tags_51 },
    { 0xfd75d39d, SUC_lexicon_tags_752 },
    { 0x7f5d39c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed5d3a2, SUC_lexicon_tags_23 },
    { 0x3835d3a2, SUC_lexicon_tags_11 },
    { 0xf775d3a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f5d3a8, SUC_lexicon_tags_27 },
    { 0x7bf5d3a9, SUC_lexicon_tags_25 },
    { 0x50f5d3a9, SUC_lexicon_tags_87 },
    { 0x9d15d3ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795d3b1, SUC_lexicon_tags_11 },
    { 0xe135d3b1, SUC_lexicon_tags_21 },
    { 0x97f5d3b2, SUC_lexicon_tags_1 },
    { 0xbd55d3b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a35d3bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4995d3bf, SUC_lexicon_tags_15 },
    { 0x1295d3c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4b95d3c2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1d75d3c4, SUC_lexicon_tags_27 },
    { 0x6ff5d3c5, SUC_lexicon_tags_29 },
    { 0xa415d3c6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xee75d3c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db5d3cc, SUC_lexicon_tags_26 },
    { 0x87b5d3cd, SUC_lexicon_tags_68 },
    { 0x1815d3ce, SUC_lexicon_tags_25 },
    { 0x7f75d3ce, SUC_lexicon_tags_34 },
    { 0x4715d3d0, SUC_lexicon_tags_23 },
    { 0xd6f5d3ce, SUC_lexicon_tags_1 },
    { 0xf375d3cc, SUC_lexicon_tags_7 },
    { 0x9c75d3d3, SUC_lexicon_tags_33 },
    { 0x5755d3cf, SUC_lexicon_tags_21 },
    { 0x5f35d3d5, SUC_lexicon_tags_17 },
    { 0x76f5d3d6, SUC_lexicon_tags_15 },
    { 0x7e95d3d7, SUC_lexicon_tags_274 },
    { 0x7e95d3d8, SUC_lexicon_tags_1 },
    { 0xc15d3d7, SUC_lexicon_tags_4 },
    { 0x4fb5d3d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5735d3e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f5d3e2, SUC_lexicon_tags_7 },
    { 0xebf5d3e3, SUC_lexicon_tags_17 },
    { 0xea95d3e4, SUC_lexicon_tags_23 },
    { 0xee35d3e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb455d3e8, SUC_lexicon_tags_11 },
    { 0x3f5d3e9, SUC_lexicon_tags_45 },
    { 0x79d5d3ea, SUC_lexicon_tags_14 },
    { 0x2635d3ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaaf5d3ed, SUC_lexicon_tags_37 },
    { 0x48b5d3ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3cb5d3f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb5d3f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48f5d3f6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f5d3fa, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7495d402, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb5d408, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x32d5d40a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb15d40c, SUC_lexicon_tags_54 },
    { 0x5ef5d40c, SUC_lexicon_tags_25 },
    { 0xb495d40e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8875d410, SUC_lexicon_tags_17 },
    { 0x1b75d411, SUC_lexicon_tags_25 },
    { 0xa735d412, SUC_lexicon_tags_17 },
    { 0x7a15d413, SUC_lexicon_tags_60 },
    { 0x2e35d412, SUC_lexicon_tags_25 },
    { 0x4815d411, SUC_lexicon_tags_25 },
    { 0x5d5d416, SUC_lexicon_tags_25 },
    { 0x59f5d410, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5f5d419, SUC_lexicon_tags_37 },
    { 0x32f5d41a, SUC_lexicon_tags_46 },
    { 0xdd15d41b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xba55d41d, SUC_lexicon_tags_15 },
    { 0x94b5d41d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e75d420, SUC_lexicon_tags_49 },
    { 0x26d5d421, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb055d423, SUC_lexicon_tags_0 },
    { 0x43f5d424, SUC_lexicon_tags_23 },
    { 0x2e55d424, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d5d42c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5595d42e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda35d431, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f15d435, SUC_lexicon_tags_47 },
    { 0xbbb5d435, SUC_lexicon_tags_15 },
    { 0x7af5d436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c35d43b, SUC_lexicon_tags_201 },
    { 0x4a55d43c, SUC_lexicon_tags_28 },
    { 0xaf35d43d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d5d442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db5d445, SUC_lexicon_tags_7 },
    { 0x3515d446, SUC_lexicon_tags_11 },
    { 0x4315d446, SUC_lexicon_tags_0 },
    { 0x6475d448, SUC_lexicon_tags_11 },
    { 0x1615d449, SUC_lexicon_tags_75 },
    { 0xb715d44a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30d5d44c, SUC_lexicon_tags_12 },
    { 0xe075d44c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf5d452, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd475d454, SUC_lexicon_tags_14 },
    { 0x5f35d454, SUC_lexicon_tags_7 },
    { 0x2fb5d454, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad35d45a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea95d45c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86b5d45e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa015d461, SUC_lexicon_tags_2 },
    { 0xc1b5d462, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3d5d464, SUC_lexicon_tags_17 },
    { 0xc0f5d465, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2eb5d467, SUC_lexicon_tags_7 },
    { 0x24d5d468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53b5d46a, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0xf75d46c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb5b5d46e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d5d471, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf135d475, SUC_lexicon_tags_28 },
    { 0x8115d476, SUC_lexicon_tags_12 },
    { 0xe1f5d477, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6335d47b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d5d480, SUC_lexicon_tags_11 },
    { 0x29f5d481, SUC_lexicon_tags_11 },
    { 0xe395d481, SUC_lexicon_tags_25 },
    { 0x2a35d483, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb95d487, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3455d48a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x72f5d48c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7875d48e, SUC_lexicon_tags_47 },
    { 0xafb5d48e, SUC_lexicon_tags_15 },
    { 0x1b55d490, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b5d495, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6055d497, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4c15d499, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5d49d, SUC_lexicon_tags_0 },
    { 0x5055d49d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf15d4a2, SUC_lexicon_tags_37 },
    { 0xc15d4a2, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0xcb5d4a5, SUC_lexicon_tags_39 },
    { 0xc1b5d4a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9f5d4a8, SUC_lexicon_tags_28 },
    { 0x54d5d4a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfb75d4ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255d4b0, SUC_lexicon_tags_27 },
    { 0xfc55d4b0, SUC_lexicon_tags_33 },
    { 0x2c75d4b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe75d4b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd5d4b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae75d4ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55d4bf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255d4c3, SUC_lexicon_tags_30 },
    { 0x43d5d4c4, SUC_lexicon_tags_56 },
    { 0x3535d4c5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x5e35d4c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe555d4ca, SUC_lexicon_tags_28 },
    { 0xdf95d4ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc155d4ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6555d4d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae35d4d2, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xd295d4d4, SUC_lexicon_tags_17 },
    { 0xc2b5d4d5, SUC_lexicon_tags_33 },
    { 0x18d5d4d4, SUC_lexicon_tags_33 },
    { 0x3315d4d7, SUC_lexicon_tags_29 },
    { 0xf1b5d4d8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6795d4da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d5d4dd, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf675d4e5, SUC_lexicon_tags_27 },
    { 0xd035d4e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f75d4ea, SUC_lexicon_tags_11 },
    { 0xc675d4eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9415d4ed, SUC_lexicon_tags_1189 },
    { 0, NULL },
    { 0, NULL },
    { 0x7115d4f0, SUC_lexicon_tags_46 },
    { 0xb175d4f1, SUC_lexicon_tags_17 },
    { 0x20d5d4f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d5d4f5, SUC_lexicon_tags_1 },
    { 0xa595d4f6, SUC_lexicon_tags_0 },
    { 0xe7d5d4f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed5d4fc, SUC_lexicon_tags_12 },
    { 0x2a35d4fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5f5d4ff, SUC_lexicon_tags_39 },
    { 0x7d15d4ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe935d502, SUC_lexicon_tags_26 },
    { 0x2fb5d502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41f5d505, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2df5d507, SUC_lexicon_tags_58 },
    { 0x1ff5d508, SUC_lexicon_tags_48 },
    { 0xe675d507, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7235d50e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc335d513, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b5d518, SUC_lexicon_tags_17 },
    { 0xedf5d519, SUC_lexicon_tags_49 },
    { 0xa6d5d519, SUC_lexicon_tags_9 },
    { 0x9575d51b, SUC_lexicon_tags_12 },
    { 0x9015d519, SUC_lexicon_tags_33 },
    { 0x2d95d518, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4955d525, SUC_lexicon_tags_60 },
    { 0xf5d526, SUC_lexicon_tags_28 },
    { 0xc355d525, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd5d52c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9015d531, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2ef5d533, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d55d537, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4715d53a, SUC_lexicon_tags_113 },
    { 0x3115d53a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9d5d53d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9855d542, SUC_lexicon_tags_17 },
    { 0x5bf5d543, SUC_lexicon_tags_27 },
    { 0x735d543, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e15d548, SUC_lexicon_tags_11 },
    { 0x5875d548, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d5d551, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9455d553, SUC_lexicon_tags_15 },
    { 0xf735d553, SUC_lexicon_tags_27 },
    { 0x7455d555, SUC_lexicon_tags_25 },
    { 0x99d5d554, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b5d559, SUC_lexicon_tags_66 },
    { 0x4a95d55a, SUC_lexicon_tags_27 },
    { 0x4e15d55b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac75d55f, SUC_lexicon_tags_37 },
    { 0x8615d55f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe1d5d562, SUC_lexicon_tags_1 },
    { 0x3255d562, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb675d568, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f5d56b, SUC_lexicon_tags_15 },
    { 0xd075d56c, SUC_lexicon_tags_30 },
    { 0x4bd5d56d, SUC_lexicon_tags_66 },
    { 0x175d56e, SUC_lexicon_tags_37 },
    { 0x5495d56f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed55d575, SUC_lexicon_tags_37 },
    { 0x175d575, SUC_lexicon_tags_34 },
    { 0xc6f5d577, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xce35d57a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb355d57c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x975d57e, SUC_lexicon_tags_9 },
    { 0x7eb5d57f, SUC_lexicon_tags_14 },
    { 0x71f5d580, SUC_lexicon_tags_11 },
    { 0x9d35d57f, SUC_lexicon_tags_12 },
    { 0x3d35d582, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7875d586, SUC_lexicon_tags_56 },
    { 0x8af5d587, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xab75d589, SUC_lexicon_tags_62 },
    { 0x5935d58a, SUC_lexicon_tags_37 },
    { 0xd0f5d58b, SUC_lexicon_tags_15 },
    { 0x4675d58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b5d592, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa475d594, SUC_lexicon_tags_10 },
    { 0x9415d594, SUC_lexicon_tags_63 },
    { 0xa0d5d596, SUC_lexicon_tags_47 },
    { 0xc315d596, SUC_lexicon_tags_54 },
    { 0xbab5d595, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca95d59f, SUC_lexicon_tags_11 },
    { 0x5575d5a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe015d5a2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b75d5a5, SUC_lexicon_tags_26 },
    { 0xcbf5d5a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f5d5aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d5d5ad, SUC_lexicon_tags_7 },
    { 0x7b5d5ae, SUC_lexicon_tags_27 },
    { 0x8e35d5ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7975d5b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e35d5b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f95d5b9, SUC_lexicon_tags_63 },
    { 0xac95d5ba, SUC_lexicon_tags_1 },
    { 0x5495d5bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2655d5be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5295d5c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb5d5c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd795d5c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd15d5cc, SUC_lexicon_tags_4 },
    { 0x3315d5cc, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x71b5d5cf, SUC_lexicon_tags_38 },
    { 0x6ad5d5cf, SUC_lexicon_tags_4 },
    { 0xadb5d5d0, SUC_lexicon_tags_56 },
    { 0xa295d5d2, SUC_lexicon_tags_46 },
    { 0xa895d5d3, SUC_lexicon_tags_47 },
    { 0x5235d5d3, SUC_lexicon_tags_79 },
    { 0x3195d5d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6655d5da, SUC_lexicon_tags_23 },
    { 0xe155d5db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb15d5dd, SUC_lexicon_tags_12 },
    { 0x1955d5de, SUC_lexicon_tags_43 },
    { 0x3895d5de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3075d5e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x475d5e3, SUC_lexicon_tags_10 },
    { 0x8ad5d5e4, SUC_lexicon_tags_23 },
    { 0xf595d5e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ef5d5e7, SUC_lexicon_tags_25 },
    { 0xff35d5e7, SUC_lexicon_tags_43 },
    { 0x7f75d5e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa5f5d5eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875d5f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd5d5f7, SUC_lexicon_tags_6 },
    { 0xd775d5f7, SUC_lexicon_tags_25 },
    { 0x6f5d5f9, SUC_lexicon_tags_17 },
    { 0xddb5d5fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6015d5ff, SUC_lexicon_tags_8 },
    { 0x1fb5d5ff, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc795d602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa15d609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd55d60b, SUC_lexicon_tags_160 },
    { 0x4d95d60b, SUC_lexicon_tags_27 },
    { 0xc475d60d, SUC_lexicon_tags_4 },
    { 0x8b95d60c, SUC_lexicon_tags_25 },
    { 0xf0b5d60e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7395d61a, SUC_lexicon_tags_25 },
    { 0xd815d61a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x56b5d61d, SUC_lexicon_tags_14 },
    { 0xd7d5d61e, SUC_lexicon_tags_45 },
    { 0xf215d61f, SUC_lexicon_tags_47 },
    { 0xaf55d61f, SUC_lexicon_tags_87 },
    { 0x6895d621, SUC_lexicon_tags_6 },
    { 0x7df5d61e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b5d627, SUC_lexicon_tags_2 },
    { 0x54f5d628, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef55d62a, SUC_lexicon_tags_44 },
    { 0x8915d62a, SUC_lexicon_tags_92 },
    { 0xf1d5d62c, SUC_lexicon_tags_15 },
    { 0x7e15d62b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf35d634, SUC_lexicon_tags_12 },
    { 0x96f5d634, SUC_lexicon_tags_17 },
    { 0xc6f5d634, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x555d638, SUC_lexicon_tags_6 },
    { 0xa575d639, SUC_lexicon_tags_23 },
    { 0xebd5d639, SUC_lexicon_tags_87 },
    { 0x3875d63b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf5d63e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d5d645, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4855d647, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc9d5d649, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b5d650, SUC_lexicon_tags_22 },
    { 0xf9b5d651, SUC_lexicon_tags_68 },
    { 0x6215d652, SUC_lexicon_tags_17 },
    { 0x5815d650, SUC_lexicon_tags_3 },
    { 0x4555d651, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d5d657, SUC_lexicon_tags_37 },
    { 0xd415d657, SUC_lexicon_tags_9 },
    { 0x9e15d659, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5d65e, SUC_lexicon_tags_39 },
    { 0x575d65f, SUC_lexicon_tags_38 },
    { 0x4f55d660, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x50f5d662, SUC_lexicon_tags_68 },
    { 0x41b5d663, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b75d665, SUC_lexicon_tags_98 },
    { 0xec35d665, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d5d669, SUC_lexicon_tags_27 },
    { 0xad95d669, SUC_lexicon_tags_1 },
    { 0xd615d66b, SUC_lexicon_tags_16 },
    { 0x2235d66a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b75d670, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ed5d672, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f5d675, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xeaf5d677, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x15d679, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d5d67d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8655d681, SUC_lexicon_tags_29 },
    { 0x6cf5d682, SUC_lexicon_tags_27 },
    { 0xefd5d683, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7315d686, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa55d688, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa55d68a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd975d690, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa95d696, SUC_lexicon_tags_56 },
    { 0xef75d697, SUC_lexicon_tags_62 },
    { 0x39b5d698, SUC_lexicon_tags_1 },
    { 0x4875d697, SUC_lexicon_tags_47 },
    { 0xdbb5d69a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d5d6a2, SUC_lexicon_tags_13 },
    { 0x5d35d6a3, SUC_lexicon_tags_60 },
    { 0x3a95d6a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86d5d6a6, SUC_lexicon_tags_25 },
    { 0xde55d6a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd235d6ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef5d6af, SUC_lexicon_tags_23 },
    { 0x92f5d6b0, SUC_lexicon_tags_19 },
    { 0x7775d6b0, SUC_lexicon_tags_9 },
    { 0x7075d6af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52b5d6b4, SUC_lexicon_tags_12 },
    { 0x9af5d6b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9435d6b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa475d6bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac35d6bd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x7775d6bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf855d6c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf295d6c6, SUC_lexicon_tags_37 },
    { 0x4015d6c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b5d6c9, SUC_lexicon_tags_17 },
    { 0x6675d6c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe915d6cc, SUC_lexicon_tags_37 },
    { 0xd8f5d6cd, SUC_lexicon_tags_11 },
    { 0xcaf5d6cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe615d6d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f5d6d5, SUC_lexicon_tags_17 },
    { 0xfb95d6d5, SUC_lexicon_tags_28 },
    { 0x4d55d6d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2b5d6d9, SUC_lexicon_tags_12 },
    { 0x4375d6da, SUC_lexicon_tags_47 },
    { 0xfcf5d6db, SUC_lexicon_tags_2 },
    { 0xb955d6dc, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa295d6e3, SUC_lexicon_tags_63 },
    { 0x83d5d6e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb5d6eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb35d6f2, SUC_lexicon_tags_49 },
    { 0x5355d6f3, SUC_lexicon_tags_60 },
    { 0x4875d6f2, SUC_lexicon_tags_14 },
    { 0xe75d6f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6435d6f7, SUC_lexicon_tags_8 },
    { 0x6715d6f8, SUC_lexicon_tags_17 },
    { 0x3195d6f8, SUC_lexicon_tags_8 },
    { 0xb715d6fa, SUC_lexicon_tags_42 },
    { 0xe7b5d6f9, SUC_lexicon_tags_17 },
    { 0x3ed5d6f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e55d704, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb5d708, SUC_lexicon_tags_82 },
    { 0x18f5d709, SUC_lexicon_tags_81 },
    { 0xdd5d70a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x815d70d, SUC_lexicon_tags_15 },
    { 0x6335d70e, SUC_lexicon_tags_17 },
    { 0x6d95d70f, SUC_lexicon_tags_12 },
    { 0xb975d710, SUC_lexicon_tags_25 },
    { 0xc2b5d70f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee35d715, SUC_lexicon_tags_60 },
    { 0xf635d716, SUC_lexicon_tags_10 },
    { 0x6f5d717, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f5d71b, SUC_lexicon_tags_30 },
    { 0x8215d71c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b5d71f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b5d723, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb795d728, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba75d72a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6015d72f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe115d732, SUC_lexicon_tags_37 },
    { 0x7d15d733, SUC_lexicon_tags_25 },
    { 0xfb15d733, SUC_lexicon_tags_63 },
    { 0xcdd5d732, SUC_lexicon_tags_49 },
    { 0x9bb5d735, SUC_lexicon_tags_31 },
    { 0xc795d737, SUC_lexicon_tags_49 },
    { 0x9fd5d738, SUC_lexicon_tags_49 },
    { 0xb375d735, SUC_lexicon_tags_66 },
    { 0xea35d739, SUC_lexicon_tags_25 },
    { 0xef15d73b, SUC_lexicon_tags_87 },
    { 0x3895d73b, SUC_lexicon_tags_11 },
    { 0x3e95d73d, SUC_lexicon_tags_0 },
    { 0x9115d73d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc095d740, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe255d743, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a95d747, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbad5d749, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5835d74b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e55d74f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3d35d751, SUC_lexicon_tags_7 },
    { 0x85f5d752, SUC_lexicon_tags_48 },
    { 0x9cd5d752, SUC_lexicon_tags_69 },
    { 0x11f5d753, SUC_lexicon_tags_25 },
    { 0xb835d755, SUC_lexicon_tags_5 },
    { 0x7715d753, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b55d759, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5615d75b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255d760, SUC_lexicon_tags_3 },
    { 0xc495d760, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2755d763, SUC_lexicon_tags_39 },
    { 0x2675d763, SUC_lexicon_tags_55 },
    { 0x8515d765, SUC_lexicon_tags_7 },
    { 0x8d35d763, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xda35d768, SUC_lexicon_tags_182 },
    { 0x435d769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5295d76b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1b95d76d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf5d770, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0xb9f5d772, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f55d774, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9275d776, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb75d778, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc635d780, SUC_lexicon_tags_30 },
    { 0x4ed5d781, SUC_lexicon_tags_47 },
    { 0x8235d782, SUC_lexicon_tags_106 },
    { 0x53b5d783, SUC_lexicon_tags_23 },
    { 0xa535d783, SUC_lexicon_tags_37 },
    { 0x57b5d785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb75d789, SUC_lexicon_tags_0 },
    { 0x8af5d78a, SUC_lexicon_tags_25 },
    { 0x2755d789, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xedf5d78d, SUC_lexicon_tags_33 },
    { 0xc975d78e, SUC_lexicon_tags_7 },
    { 0xba75d78f, SUC_lexicon_tags_27 },
    { 0x9335d78f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5015d792, SUC_lexicon_tags_39 },
    { 0xecb5d793, SUC_lexicon_tags_63 },
    { 0xd775d794, SUC_lexicon_tags_12 },
    { 0xa515d795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe395d79b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5955d79e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd875d7a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd5d7a6, SUC_lexicon_tags_63 },
    { 0x1515d7a7, SUC_lexicon_tags_10 },
    { 0xbab5d7a8, SUC_lexicon_tags_25 },
    { 0x2bd5d7a9, SUC_lexicon_tags_17 },
    { 0x42d5d7a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7555d7ad, SUC_lexicon_tags_56 },
    { 0xa255d7ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7055d7b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf435d7b4, SUC_lexicon_tags_29 },
    { 0x4db5d7b5, SUC_lexicon_tags_15 },
    { 0xa5f5d7b6, SUC_lexicon_tags_9 },
    { 0x7855d7b5, SUC_lexicon_tags_25 },
    { 0xd35d7b5, SUC_lexicon_tags_38 },
    { 0x4835d7b9, SUC_lexicon_tags_125 },
    { 0x7095d7ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1f55d7bc, SUC_lexicon_tags_47 },
    { 0xc835d7bd, SUC_lexicon_tags_0 },
    { 0xe155d7bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5d7c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f15d7c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad5d7c8, SUC_lexicon_tags_11 },
    { 0x5395d7c9, SUC_lexicon_tags_37 },
    { 0xd6d5d7ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f5d7d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4355d7d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7155d7d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5575d7dc, SUC_lexicon_tags_25 },
    { 0x23d5d7dd, SUC_lexicon_tags_37 },
    { 0xb855d7dc, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef5d7e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb55d7ed, SUC_lexicon_tags_10 },
    { 0x25b5d7ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf755d7f4, SUC_lexicon_tags_37 },
    { 0x8d5d7f5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7735d7f9, SUC_lexicon_tags_7 },
    { 0x1f55d7fa, SUC_lexicon_tags_45 },
    { 0x16f5d7f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f75d7fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd295d804, SUC_lexicon_tags_1 },
    { 0xa35d805, SUC_lexicon_tags_47 },
    { 0x83f5d806, SUC_lexicon_tags_46 },
    { 0xd955d806, SUC_lexicon_tags_15 },
    { 0x2875d808, SUC_lexicon_tags_47 },
    { 0x9515d809, SUC_lexicon_tags_25 },
    { 0x9655d805, SUC_lexicon_tags_11 },
    { 0x9795d804, SUC_lexicon_tags_19 },
    { 0xe55d805, SUC_lexicon_tags_29 },
    { 0x35d80c, SUC_lexicon_tags_25 },
    { 0x3795d80e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd95d810, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dd5d812, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df5d817, SUC_lexicon_tags_27 },
    { 0xe9b5d818, SUC_lexicon_tags_12 },
    { 0x4c55d819, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d5d81f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b15d821, SUC_lexicon_tags_27 },
    { 0x1ed5d822, SUC_lexicon_tags_31 },
    { 0xaa15d823, SUC_lexicon_tags_68 },
    { 0xfa55d823, SUC_lexicon_tags_8 },
    { 0xc3b5d825, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5195d827, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab5d82e, SUC_lexicon_tags_1 },
    { 0x3375d82f, SUC_lexicon_tags_17 },
    { 0x7335d82f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c35d833, SUC_lexicon_tags_17 },
    { 0xafd5d834, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a75d839, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2515d83b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2975d83d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xca95d840, SUC_lexicon_tags_0 },
    { 0x7d55d841, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa055d844, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf5d849, SUC_lexicon_tags_56 },
    { 0x9195d849, SUC_lexicon_tags_37 },
    { 0xe175d84b, SUC_lexicon_tags_10 },
    { 0x1bd5d84c, SUC_lexicon_tags_14 },
    { 0x3cb5d84d, SUC_lexicon_tags_68 },
    { 0x9855d84e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4db5d850, SUC_lexicon_tags_45 },
    { 0xc6f5d850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdb5d853, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee95d85b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b75d85e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8735d860, SUC_lexicon_tags_7 },
    { 0xced5d861, SUC_lexicon_tags_7 },
    { 0x9895d861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4855d864, SUC_lexicon_tags_12 },
    { 0x14b5d865, SUC_lexicon_tags_28 },
    { 0xf855d866, SUC_lexicon_tags_2 },
    { 0x58f5d866, SUC_lexicon_tags_2 },
    { 0x6035d865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e95d86a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c75d86e, SUC_lexicon_tags_46 },
    { 0xedf5d86f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f35d872, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9995d879, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7855d87b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d55d881, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f5d888, SUC_lexicon_tags_33 },
    { 0xd4d5d889, SUC_lexicon_tags_26 },
    { 0x9335d88a, SUC_lexicon_tags_17 },
    { 0x315d88b, SUC_lexicon_tags_6 },
    { 0x9ad5d88c, SUC_lexicon_tags_12 },
    { 0x92f5d88b, SUC_lexicon_tags_66 },
    { 0x7a95d88e, SUC_lexicon_tags_12 },
    { 0x8f5d88f, SUC_lexicon_tags_17 },
    { 0xf775d890, SUC_lexicon_tags_26 },
    { 0x5515d891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe615d893, SUC_lexicon_tags_47 },
    { 0x8a55d894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38f5d896, SUC_lexicon_tags_79 },
    { 0xd9b5d897, SUC_lexicon_tags_47 },
    { 0xf75d898, SUC_lexicon_tags_60 },
    { 0x5655d897, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2f5d89b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcd75d89d, SUC_lexicon_tags_0 },
    { 0xbb95d89e, SUC_lexicon_tags_40 },
    { 0xa575d89e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd5d8a2, SUC_lexicon_tags_1 },
    { 0x9b15d8a3, SUC_lexicon_tags_14 },
    { 0xc5f5d8a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f15d8a7, SUC_lexicon_tags_26 },
    { 0x3e75d8a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7355d8b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff75d8b4, SUC_lexicon_tags_17 },
    { 0xea75d8b5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf55d8bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2415d8bf, SUC_lexicon_tags_44 },
    { 0x2535d8c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1735d8c7, SUC_lexicon_tags_68 },
    { 0xdf95d8c8, SUC_lexicon_tags_31 },
    { 0x4095d8c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb875d8cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d5d8cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f55d8d3, SUC_lexicon_tags_38 },
    { 0x9cf5d8d4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d75d8d7, SUC_lexicon_tags_12 },
    { 0xc415d8d8, SUC_lexicon_tags_37 },
    { 0x9235d8d8, SUC_lexicon_tags_68 },
    { 0x9055d8da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd95d8dc, SUC_lexicon_tags_27 },
    { 0x7055d8dd, SUC_lexicon_tags_31 },
    { 0xba95d8dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf115d8e0, SUC_lexicon_tags_12 },
    { 0xa615d8e1, SUC_lexicon_tags_17 },
    { 0x295d8e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xad75d8e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4215d8e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb5d8ea, SUC_lexicon_tags_12 },
    { 0x6835d8eb, SUC_lexicon_tags_7 },
    { 0x70f5d8ec, SUC_lexicon_tags_33 },
    { 0xe5d5d8ed, SUC_lexicon_tags_7 },
    { 0x37f5d8ea, SUC_lexicon_tags_29 },
    { 0x90d5d8ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f75d8f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d75d8f8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd5d8fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f15d900, SUC_lexicon_tags_25 },
    { 0x5915d900, SUC_lexicon_tags_10 },
    { 0x8bd5d902, SUC_lexicon_tags_37 },
    { 0x35d5d901, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d5d906, SUC_lexicon_tags_14 },
    { 0xfd75d907, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xda55d909, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d35d90c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd215d914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a55d917, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d5d91e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9995d920, SUC_lexicon_tags_37 },
    { 0xe2b5d920, SUC_lexicon_tags_37 },
    { 0x7495d920, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295d926, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd175d92e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a15d934, SUC_lexicon_tags_49 },
    { 0xab95d935, SUC_lexicon_tags_7 },
    { 0xe15d934, SUC_lexicon_tags_21 },
    { 0xf5b5d936, SUC_lexicon_tags_7 },
    { 0x7735d937, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd5d93b, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0xebb5d93d, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f5d941, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5d944, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b5d947, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf155d94a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf5d94d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc795d950, SUC_lexicon_tags_17 },
    { 0x87b5d950, SUC_lexicon_tags_49 },
    { 0x98f5d952, SUC_lexicon_tags_68 },
    { 0xae15d951, SUC_lexicon_tags_3 },
    { 0x39f5d954, SUC_lexicon_tags_47 },
    { 0xd155d955, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf395d957, SUC_lexicon_tags_18 },
    { 0xd0d5d957, SUC_lexicon_tags_55 },
    { 0x13f5d958, SUC_lexicon_tags_37 },
    { 0xafd5d95a, SUC_lexicon_tags_43 },
    { 0x38b5d957, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f35d95e, SUC_lexicon_tags_25 },
    { 0x3075d95e, SUC_lexicon_tags_10 },
    { 0x8bd5d960, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb5d963, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6295d966, SUC_lexicon_tags_48 },
    { 0x5395d966, SUC_lexicon_tags_7 },
    { 0x1675d968, SUC_lexicon_tags_8 },
    { 0xda55d969, SUC_lexicon_tags_28 },
    { 0x2b5d969, SUC_lexicon_tags_11 },
    { 0xf9f5d96b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd5d970, SUC_lexicon_tags_25 },
    { 0x20d5d970, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a75d974, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb9d5d976, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1575d979, SUC_lexicon_tags_39 },
    { 0xb435d97a, SUC_lexicon_tags_45 },
    { 0xb595d979, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe635d97f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd6b5d981, SUC_lexicon_tags_30 },
    { 0xe895d982, SUC_lexicon_tags_11 },
    { 0x8db5d982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f5d987, SUC_lexicon_tags_28 },
    { 0x6ff5d988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xda15d98b, SUC_lexicon_tags_68 },
    { 0x99b5d98c, SUC_lexicon_tags_47 },
    { 0xafd5d98d, SUC_lexicon_tags_15 },
    { 0x1fd5d98e, SUC_lexicon_tags_3 },
    { 0xdb55d98e, SUC_lexicon_tags_33 },
    { 0x8395d98c, SUC_lexicon_tags_17 },
    { 0x9155d98d, SUC_lexicon_tags_33 },
    { 0x1cd5d992, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf935d994, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd975d99c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d75d9a1, SUC_lexicon_tags_69 },
    { 0xd655d9a2, SUC_lexicon_tags_151 },
    { 0xb0b5d9a2, SUC_lexicon_tags_30 },
    { 0x8275d9a1, SUC_lexicon_tags_7 },
    { 0x14f5d9a1, SUC_lexicon_tags_60 },
    { 0x8595d9a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2495d9ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf5d9b4, SUC_lexicon_tags_11 },
    { 0xdfd5d9b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc75d9b8, SUC_lexicon_tags_1 },
    { 0x6f5d9b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb5d9bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef55d9c1, SUC_lexicon_tags_43 },
    { 0x1a15d9c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea15d9c7, SUC_lexicon_tags_34 },
    { 0x64b5d9c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb315d9cb, SUC_lexicon_tags_68 },
    { 0xf115d9cb, SUC_lexicon_tags_17 },
    { 0x6355d9cd, SUC_lexicon_tags_19 },
    { 0x75d9cc, SUC_lexicon_tags_87 },
    { 0xac75d9cf, SUC_lexicon_tags_12 },
    { 0x6135d9cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e15d9d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b5d9d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2475d9dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb5d9df, SUC_lexicon_tags_553 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba75d9e3, SUC_lexicon_tags_27 },
    { 0xcc95d9e4, SUC_lexicon_tags_7 },
    { 0x46d5d9e5, SUC_lexicon_tags_15 },
    { 0x74f5d9e6, SUC_lexicon_tags_1 },
    { 0x4395d9e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a35d9ef, SUC_lexicon_tags_28 },
    { 0x4915d9ef, SUC_lexicon_tags_41 },
    { 0xfa95d9f0, SUC_lexicon_tags_12 },
    { 0x2075d9f2, SUC_lexicon_tags_12 },
    { 0xccb5d9f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf5d9f7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3715d9fd, SUC_lexicon_tags_37 },
    { 0x60b5d9fe, SUC_lexicon_tags_12 },
    { 0x1055d9ff, SUC_lexicon_tags_27 },
    { 0x3455da00, SUC_lexicon_tags_17 },
    { 0xbe95da00, SUC_lexicon_tags_25 },
    { 0xcf95d9fe, SUC_lexicon_tags_49 },
    { 0x63d5da03, SUC_lexicon_tags_17 },
    { 0x1095da02, SUC_lexicon_tags_816 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4615da0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8655da0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfb5da10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff5da13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d5da19, SUC_lexicon_tags_1 },
    { 0x1b35da19, SUC_lexicon_tags_1 },
    { 0x8ef5da1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba55da1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f5da22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b5da28, SUC_lexicon_tags_15 },
    { 0x9e55da28, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xec15da2b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b55da31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf55da35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d95da38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5da3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84d5da3e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf35da46, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed5da49, SUC_lexicon_tags_38 },
    { 0x1975da4a, SUC_lexicon_tags_7 },
    { 0x89f5da4b, SUC_lexicon_tags_19 },
    { 0x1b35da4c, SUC_lexicon_tags_37 },
    { 0x8fd5da4c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f5da56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5335da5b, SUC_lexicon_tags_1 },
    { 0x5d75da5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d5da60, SUC_lexicon_tags_56 },
    { 0x7d75da61, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd5da64, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d5da67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd5da6c, SUC_lexicon_tags_37 },
    { 0x2b35da6c, SUC_lexicon_tags_55 },
    { 0xfe55da6e, SUC_lexicon_tags_15 },
    { 0x81b5da6f, SUC_lexicon_tags_28 },
    { 0x5415da6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf5f5da72, SUC_lexicon_tags_37 },
    { 0xec15da72, SUC_lexicon_tags_19 },
    { 0x6415da74, SUC_lexicon_tags_33 },
    { 0x8fd5da74, SUC_lexicon_tags_32 },
    { 0x70f5da74, SUC_lexicon_tags_38 },
    { 0xcbb5da77, SUC_lexicon_tags_12 },
    { 0xfa15da72, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d5da7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53d5da7f, SUC_lexicon_tags_7 },
    { 0x60f5da80, SUC_lexicon_tags_8 },
    { 0xc355da81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5d5da83, SUC_lexicon_tags_12 },
    { 0xbdd5da83, SUC_lexicon_tags_12 },
    { 0x4355da84, SUC_lexicon_tags_0 },
    { 0x1b35da86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1915da8b, SUC_lexicon_tags_14 },
    { 0x5735da8c, SUC_lexicon_tags_11 },
    { 0xdcf5da8b, SUC_lexicon_tags_11 },
    { 0x3215da8e, SUC_lexicon_tags_9 },
    { 0x2715da8b, SUC_lexicon_tags_6 },
    { 0x9715da90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9455da93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb5da96, SUC_lexicon_tags_25 },
    { 0x2955da97, SUC_lexicon_tags_45 },
    { 0xcdd5da96, SUC_lexicon_tags_7 },
    { 0xe2b5da99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9055da9c, SUC_lexicon_tags_21 },
    { 0xc2d5da9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b5daa0, SUC_lexicon_tags_61 },
    { 0xadf5daa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc675daa4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa695daa7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa355dab1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe5d5dab3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8315dab5, SUC_lexicon_tags_15 },
    { 0x7115dab6, SUC_lexicon_tags_37 },
    { 0xbb75dab7, SUC_lexicon_tags_25 },
    { 0x5555dab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5235daba, SUC_lexicon_tags_68 },
    { 0x1075daba, SUC_lexicon_tags_15 },
    { 0xf3f5dabc, SUC_lexicon_tags_17 },
    { 0xcbd5dabb, SUC_lexicon_tags_60 },
    { 0x9ef5dabe, SUC_lexicon_tags_29 },
    { 0xbc55dabe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc435dac2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5215dac8, SUC_lexicon_tags_0 },
    { 0xe415dac9, SUC_lexicon_tags_38 },
    { 0xe575dac9, SUC_lexicon_tags_12 },
    { 0x2f35dacb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d5dacf, SUC_lexicon_tags_64 },
    { 0x4495dad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb5dad5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb75dada, SUC_lexicon_tags_38 },
    { 0x695dadb, SUC_lexicon_tags_823 },
    { 0x4a55dadc, SUC_lexicon_tags_26 },
    { 0x9255dadd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d35dae1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb5dae4, SUC_lexicon_tags_37 },
    { 0x44f5dae5, SUC_lexicon_tags_129 },
    { 0xd2d5dae5, SUC_lexicon_tags_17 },
    { 0xae75dae5, SUC_lexicon_tags_7 },
    { 0x35d5dae8, SUC_lexicon_tags_15 },
    { 0x5f75dae9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd35daec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b5daf4, SUC_lexicon_tags_30 },
    { 0x9055daf4, SUC_lexicon_tags_12 },
    { 0x7415daf6, SUC_lexicon_tags_16 },
    { 0x2515daf7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab75dafe, SUC_lexicon_tags_63 },
    { 0xe775daff, SUC_lexicon_tags_47 },
    { 0x19f5dafe, SUC_lexicon_tags_37 },
    { 0x1c95daff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa015db07, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5015db09, SUC_lexicon_tags_15 },
    { 0x6ab5db09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x615db0c, SUC_lexicon_tags_68 },
    { 0xf3d5db0d, SUC_lexicon_tags_14 },
    { 0xb835db0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa115db10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xee95db12, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e35db15, SUC_lexicon_tags_33 },
    { 0x4955db16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6515db1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd395db1d, SUC_lexicon_tags_170 },
    { 0x75f5db1e, SUC_lexicon_tags_37 },
    { 0x2975db1d, SUC_lexicon_tags_46 },
    { 0xd195db20, SUC_lexicon_tags_15 },
    { 0x2535db21, SUC_lexicon_tags_37 },
    { 0x9fb5db22, SUC_lexicon_tags_39 },
    { 0x1f15db22, SUC_lexicon_tags_15 },
    { 0x2ad5db22, SUC_lexicon_tags_17 },
    { 0xf355db21, SUC_lexicon_tags_63 },
    { 0x7095db26, SUC_lexicon_tags_62 },
    { 0x7375db1f, SUC_lexicon_tags_66 },
    { 0x1e55db28, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7135db2c, SUC_lexicon_tags_101 },
    { 0x3a75db2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c55db31, SUC_lexicon_tags_37 },
    { 0x9835db32, SUC_lexicon_tags_29 },
    { 0x6235db33, SUC_lexicon_tags_1 },
    { 0x2575db34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2815db37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e15db3e, SUC_lexicon_tags_47 },
    { 0x5df5db3e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1735db41, SUC_lexicon_tags_12 },
    { 0x9ad5db41, SUC_lexicon_tags_17 },
    { 0xb075db42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d5db47, SUC_lexicon_tags_25 },
    { 0x4695db48, SUC_lexicon_tags_17 },
    { 0xa5b5db49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b5db4e, SUC_lexicon_tags_112 },
    { 0x8155db4f, SUC_lexicon_tags_2 },
    { 0x4695db4f, SUC_lexicon_tags_15 },
    { 0x435db51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd155db55, SUC_lexicon_tags_1 },
    { 0x8df5db55, SUC_lexicon_tags_11 },
    { 0x83b5db56, SUC_lexicon_tags_8 },
    { 0x8a35db58, SUC_lexicon_tags_30 },
    { 0x2d5db59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa015db5c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb5db65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d75db69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4595db6d, SUC_lexicon_tags_79 },
    { 0x2015db6e, SUC_lexicon_tags_37 },
    { 0x92d5db6f, SUC_lexicon_tags_317 },
    { 0x6375db70, SUC_lexicon_tags_70 },
    { 0x2715db71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb5db75, SUC_lexicon_tags_630 },
    { 0, NULL },
    { 0, NULL },
    { 0x215db78, SUC_lexicon_tags_17 },
    { 0xb635db79, SUC_lexicon_tags_15 },
    { 0x20d5db7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc515db83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb255db86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8515db88, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4255db8a, SUC_lexicon_tags_8 },
    { 0x6af5db8a, SUC_lexicon_tags_15 },
    { 0x9795db8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3455db92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa6b5db94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9975db97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8615db9a, SUC_lexicon_tags_60 },
    { 0x86d5db9b, SUC_lexicon_tags_27 },
    { 0x8bf5db9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe655db9e, SUC_lexicon_tags_11 },
    { 0x4e35db9f, SUC_lexicon_tags_15 },
    { 0x875db9f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d5dba3, SUC_lexicon_tags_17 },
    { 0xfb35dba3, SUC_lexicon_tags_37 },
    { 0xa535dba5, SUC_lexicon_tags_12 },
    { 0x2595dba3, SUC_lexicon_tags_15 },
    { 0x7bb5dba7, SUC_lexicon_tags_15 },
    { 0x72d5dba8, SUC_lexicon_tags_5 },
    { 0x19b5dba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d35dbab, SUC_lexicon_tags_37 },
    { 0x4615dbac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b5dbb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4135dbb3, SUC_lexicon_tags_10 },
    { 0x6075dbb3, SUC_lexicon_tags_38 },
    { 0x3e35dbb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8775dbbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2055dbc7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7875dbcb, SUC_lexicon_tags_62 },
    { 0x2395dbcb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c75dbd2, SUC_lexicon_tags_25 },
    { 0x7d95dbd2, SUC_lexicon_tags_25 },
    { 0xba55dbd4, SUC_lexicon_tags_37 },
    { 0x975dbd5, SUC_lexicon_tags_12 },
    { 0x1e95dbd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd055dbd9, SUC_lexicon_tags_25 },
    { 0xea95dbda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ab5dbdc, SUC_lexicon_tags_34 },
    { 0x975dbdd, SUC_lexicon_tags_37 },
    { 0x6a35dbdd, SUC_lexicon_tags_11 },
    { 0x9f35dbde, SUC_lexicon_tags_7 },
    { 0x14b5dbe0, SUC_lexicon_tags_177 },
    { 0x7135dbe0, SUC_lexicon_tags_6 },
    { 0xacb5dbe0, SUC_lexicon_tags_28 },
    { 0x235dbe3, SUC_lexicon_tags_47 },
    { 0xf95dbe3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95dbe9, SUC_lexicon_tags_37 },
    { 0xa3f5dbe9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb5dbee, SUC_lexicon_tags_10 },
    { 0xcff5dbef, SUC_lexicon_tags_19 },
    { 0xf195dbf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd435dbf2, SUC_lexicon_tags_25 },
    { 0xd2b5dbf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecf5dbf5, SUC_lexicon_tags_29 },
    { 0x22f5dbf5, SUC_lexicon_tags_45 },
    { 0xb5b5dbf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d5dbfa, SUC_lexicon_tags_17 },
    { 0xe215dbfa, SUC_lexicon_tags_35 },
    { 0x80b5dbfc, SUC_lexicon_tags_33 },
    { 0x5255dbfc, SUC_lexicon_tags_25 },
    { 0x5075dbfe, SUC_lexicon_tags_27 },
    { 0xc635dbfa, SUC_lexicon_tags_376 },
    { 0xb815dc00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f5dc08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1355dc0c, SUC_lexicon_tags_37 },
    { 0x94f5dc0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2475dc12, SUC_lexicon_tags_15 },
    { 0xbe35dc12, SUC_lexicon_tags_6 },
    { 0xb375dc13, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xd035dc17, SUC_lexicon_tags_25 },
    { 0x2895dc18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa815dc1c, SUC_lexicon_tags_23 },
    { 0x25f5dc1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75dc21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b5dc25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27b5dc27, SUC_lexicon_tags_19 },
    { 0xc4b5dc27, SUC_lexicon_tags_25 },
    { 0x495dc29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b5dc2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x18d5dc2f, SUC_lexicon_tags_2 },
    { 0x3975dc30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6195dc3a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef5dc40, SUC_lexicon_tags_15 },
    { 0x54b5dc41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b5dc44, SUC_lexicon_tags_23 },
    { 0x2cf5dc44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7735dc47, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb5dc4e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc2f5dc50, SUC_lexicon_tags_38 },
    { 0xbed5dc51, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d5dc55, SUC_lexicon_tags_27 },
    { 0x4635dc56, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7a55dc58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3995dc61, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb695dc64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a35dc66, SUC_lexicon_tags_29 },
    { 0x2fb5dc67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ef5dc69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5855dc6b, SUC_lexicon_tags_12 },
    { 0xcbf5dc6c, SUC_lexicon_tags_25 },
    { 0x5135dc6d, SUC_lexicon_tags_15 },
    { 0xb0b5dc6d, SUC_lexicon_tags_33 },
    { 0xbf35dc6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3495dc71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9415dc78, SUC_lexicon_tags_65 },
    { 0xc5f5dc78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6815dc7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d5dc82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7735dc84, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad5dc8b, SUC_lexicon_tags_35 },
    { 0xe8f5dc8c, SUC_lexicon_tags_12 },
    { 0x6295dc8d, SUC_lexicon_tags_11 },
    { 0x48b5dc8d, SUC_lexicon_tags_38 },
    { 0x3095dc8d, SUC_lexicon_tags_17 },
    { 0x595dc90, SUC_lexicon_tags_37 },
    { 0xead5dc91, SUC_lexicon_tags_2 },
    { 0xadb5dc92, SUC_lexicon_tags_17 },
    { 0x3835dc8c, SUC_lexicon_tags_65 },
    { 0x835dc93, SUC_lexicon_tags_2 },
    { 0xff55dc95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccf5dc97, SUC_lexicon_tags_47 },
    { 0xc515dc97, SUC_lexicon_tags_29 },
    { 0x3a55dc99, SUC_lexicon_tags_60 },
    { 0xbdd5dc99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf5dc9d, SUC_lexicon_tags_30 },
    { 0xea35dc9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd475dca0, SUC_lexicon_tags_6 },
    { 0xa6f5dca1, SUC_lexicon_tags_8 },
    { 0x1495dca2, SUC_lexicon_tags_21 },
    { 0x22f5dca3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf5dca6, SUC_lexicon_tags_87 },
    { 0x53d5dca7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ad5dca9, SUC_lexicon_tags_65 },
    { 0x58d5dcaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55dcad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6695dcaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95b5dcb1, SUC_lexicon_tags_15 },
    { 0xcad5dcb2, SUC_lexicon_tags_21 },
    { 0xe415dcb3, SUC_lexicon_tags_23 },
    { 0x6255dcb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd5dcb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1395dcbb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1575dcbd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf235dcc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f95dcce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac95dcd3, SUC_lexicon_tags_15 },
    { 0xb075dcd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd455dce1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1c15dce3, SUC_lexicon_tags_11 },
    { 0x43d5dce3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d5dce7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f5dcea, SUC_lexicon_tags_17 },
    { 0xdaf5dcea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xebf5dced, SUC_lexicon_tags_26 },
    { 0x4015dced, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a15dcf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f5dcf7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d5dcfb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe595dcff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d95dd02, SUC_lexicon_tags_1 },
    { 0xe175dd02, SUC_lexicon_tags_6 },
    { 0x5a75dd04, SUC_lexicon_tags_37 },
    { 0x3b5dd05, SUC_lexicon_tags_12 },
    { 0xc135dd06, SUC_lexicon_tags_49 },
    { 0x39b5dd07, SUC_lexicon_tags_15 },
    { 0x2fb5dd04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6115dd0c, SUC_lexicon_tags_17 },
    { 0x8575dd0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd95dd11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9eb5dd13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b5dd16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9f75dd18, SUC_lexicon_tags_25 },
    { 0xa7f5dd19, SUC_lexicon_tags_17 },
    { 0xf155dd19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bf5dd1c, SUC_lexicon_tags_1 },
    { 0x6415dd1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a15dd23, SUC_lexicon_tags_25 },
    { 0x8175dd24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f5dd27, SUC_lexicon_tags_25 },
    { 0x515dd28, SUC_lexicon_tags_68 },
    { 0xc0b5dd29, SUC_lexicon_tags_23 },
    { 0x835dd29, SUC_lexicon_tags_31 },
    { 0xfb35dd2b, SUC_lexicon_tags_1 },
    { 0x62f5dd28, SUC_lexicon_tags_40 },
    { 0x24b5dd28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d5dd36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3455dd3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5575dd3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbc15dd3e, SUC_lexicon_tags_15 },
    { 0x5ed5dd3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3135dd46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc15dd4a, SUC_lexicon_tags_35 },
    { 0xf5dd4b, SUC_lexicon_tags_63 },
    { 0x6795dd4b, SUC_lexicon_tags_25 },
    { 0x70d5dd4b, SUC_lexicon_tags_25 },
    { 0xbfd5dd4d, SUC_lexicon_tags_25 },
    { 0x5c75dd4f, SUC_lexicon_tags_25 },
    { 0x24d5dd4c, SUC_lexicon_tags_47 },
    { 0x1d15dd4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef5dd56, SUC_lexicon_tags_11 },
    { 0x69b5dd57, SUC_lexicon_tags_15 },
    { 0x575dd57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8315dd5a, SUC_lexicon_tags_45 },
    { 0xf5b5dd5b, SUC_lexicon_tags_11 },
    { 0xe675dd5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d5dd61, SUC_lexicon_tags_174 },
    { 0xda15dd62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec15dd67, SUC_lexicon_tags_17 },
    { 0xbb95dd68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3235dd6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc895dd6f, SUC_lexicon_tags_37 },
    { 0xc135dd70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf5dd73, SUC_lexicon_tags_6 },
    { 0xccf5dd73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4315dd79, SUC_lexicon_tags_28 },
    { 0x3cd5dd79, SUC_lexicon_tags_8 },
    { 0xed95dd7b, SUC_lexicon_tags_11 },
    { 0x7355dd7c, SUC_lexicon_tags_25 },
    { 0x7cb5dd7d, SUC_lexicon_tags_38 },
    { 0xd3f5dd7c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab5dd88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb5dd8b, SUC_lexicon_tags_37 },
    { 0xca55dd8c, SUC_lexicon_tags_25 },
    { 0x3195dd8d, SUC_lexicon_tags_17 },
    { 0x20d5dd8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3115dd90, SUC_lexicon_tags_9 },
    { 0x115dd91, SUC_lexicon_tags_11 },
    { 0xf7f5dd91, SUC_lexicon_tags_47 },
    { 0x6b75dd93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bd5dd95, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d5dd98, SUC_lexicon_tags_12 },
    { 0xcd55dd99, SUC_lexicon_tags_47 },
    { 0xefd5dd98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1615ddb2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe0b5ddb4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcb95ddb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f5ddba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5c75ddbc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc115ddbf, SUC_lexicon_tags_37 },
    { 0xdb55ddc0, SUC_lexicon_tags_15 },
    { 0xed15ddbf, SUC_lexicon_tags_27 },
    { 0x4d75ddc0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe595ddcc, SUC_lexicon_tags_39 },
    { 0x1395ddcc, SUC_lexicon_tags_35 },
    { 0x3315ddce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db5ddd2, SUC_lexicon_tags_2 },
    { 0x2ef5ddd3, SUC_lexicon_tags_15 },
    { 0xa5d5ddd3, SUC_lexicon_tags_10 },
    { 0x92d5ddd5, SUC_lexicon_tags_27 },
    { 0xaab5ddd6, SUC_lexicon_tags_17 },
    { 0x55ddd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7db5ddd9, SUC_lexicon_tags_0 },
    { 0x2975ddda, SUC_lexicon_tags_68 },
    { 0xc095dddb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c95ddde, SUC_lexicon_tags_1 },
    { 0x3a75dddf, SUC_lexicon_tags_9 },
    { 0x41b5dde0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f35dde5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d5ddeb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c95ddf5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb595ddf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b5ddfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f5de00, SUC_lexicon_tags_38 },
    { 0x4895de00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb5de04, SUC_lexicon_tags_23 },
    { 0x1f55de05, SUC_lexicon_tags_17 },
    { 0xad35de06, SUC_lexicon_tags_25 },
    { 0xb495de06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4975de0a, SUC_lexicon_tags_47 },
    { 0x1ad5de0a, SUC_lexicon_tags_79 },
    { 0x8d35de0a, SUC_lexicon_tags_19 },
    { 0x2375de0d, SUC_lexicon_tags_13 },
    { 0x75b5de0e, SUC_lexicon_tags_12 },
    { 0xce55de0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf495de11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f5de1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1895de1c, SUC_lexicon_tags_11 },
    { 0x4515de1c, SUC_lexicon_tags_79 },
    { 0x7d75de1e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2c95de20, SUC_lexicon_tags_15 },
    { 0xa5d5de21, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd5de24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb35de26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d5de29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4015de2b, SUC_lexicon_tags_38 },
    { 0x1875de2c, SUC_lexicon_tags_25 },
    { 0xfbf5de2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b5de31, SUC_lexicon_tags_7 },
    { 0xa7f5de32, SUC_lexicon_tags_49 },
    { 0xf035de31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8775de38, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x26b5de3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8995de3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb5de42, SUC_lexicon_tags_38 },
    { 0x5c75de43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1595de4a, SUC_lexicon_tags_33 },
    { 0x7e75de4b, SUC_lexicon_tags_1 },
    { 0xa415de4b, SUC_lexicon_tags_44 },
    { 0x2f75de4b, SUC_lexicon_tags_17 },
    { 0x2855de4b, SUC_lexicon_tags_4 },
    { 0xce35de4f, SUC_lexicon_tags_28 },
    { 0x2ef5de4a, SUC_lexicon_tags_43 },
    { 0x1175de4a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b5de54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1655de58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x575de5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2df5de5d, SUC_lexicon_tags_6 },
    { 0x9295de5e, SUC_lexicon_tags_43 },
    { 0xfb15de5f, SUC_lexicon_tags_47 },
    { 0xa215de60, SUC_lexicon_tags_0 },
    { 0xd895de60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955de65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2955de69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed95de74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615de78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe995de7a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2695de84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d95de88, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d5de8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21f5de8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd655de93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b75de97, SUC_lexicon_tags_17 },
    { 0x8735de97, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1355dea0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf635dea3, SUC_lexicon_tags_0 },
    { 0x7355dea3, SUC_lexicon_tags_37 },
    { 0x4c15dea5, SUC_lexicon_tags_27 },
    { 0xbfb5dea3, SUC_lexicon_tags_7 },
    { 0x4fd5dea7, SUC_lexicon_tags_25 },
    { 0xb875dea8, SUC_lexicon_tags_26 },
    { 0x51b5dea9, SUC_lexicon_tags_160 },
    { 0xe095deaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e75dead, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c55deaf, SUC_lexicon_tags_60 },
    { 0x9095deb0, SUC_lexicon_tags_2 },
    { 0x3275deb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9215deb3, SUC_lexicon_tags_37 },
    { 0x2ab5deb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb5deba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb5debe, SUC_lexicon_tags_15 },
    { 0x675debe, SUC_lexicon_tags_17 },
    { 0xf815debf, SUC_lexicon_tags_11 },
    { 0x5035dec1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc275dec3, SUC_lexicon_tags_29 },
    { 0x9715dec3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c95deca, SUC_lexicon_tags_47 },
    { 0x435decb, SUC_lexicon_tags_37 },
    { 0xa775deca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb175dece, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb675ded0, SUC_lexicon_tags_29 },
    { 0x5b55ded0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3915ded4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe375ded6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d5ded9, SUC_lexicon_tags_17 },
    { 0x8015deda, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3395dedd, SUC_lexicon_tags_33 },
    { 0x9db5dedd, SUC_lexicon_tags_11 },
    { 0xad35dedf, SUC_lexicon_tags_46 },
    { 0xbe55dedf, SUC_lexicon_tags_60 },
    { 0xde55dee1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd475dee5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc4b5dee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa55dee9, SUC_lexicon_tags_38 },
    { 0x47b5deea, SUC_lexicon_tags_2 },
    { 0x7855deea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1ab5deed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d95def3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d5def7, SUC_lexicon_tags_28 },
    { 0xa935def8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa615defb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d5defe, SUC_lexicon_tags_17 },
    { 0xfdd5defe, SUC_lexicon_tags_27 },
    { 0x7cf5defe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef5df04, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb5df09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f5df0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2055df0f, SUC_lexicon_tags_485 },
    { 0x75df10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb735df16, SUC_lexicon_tags_43 },
    { 0x9095df17, SUC_lexicon_tags_29 },
    { 0xc15df18, SUC_lexicon_tags_61 },
    { 0xe4f5df19, SUC_lexicon_tags_45 },
    { 0x8995df1a, SUC_lexicon_tags_12 },
    { 0xd3d5df1b, SUC_lexicon_tags_27 },
    { 0xa8f5df1a, SUC_lexicon_tags_79 },
    { 0x4d5df1d, SUC_lexicon_tags_1 },
    { 0x3eb5df18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8495df26, SUC_lexicon_tags_32 },
    { 0xa6f5df26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5df30, SUC_lexicon_tags_45 },
    { 0xdc15df31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf155df33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5055df36, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b5df3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf0f5df3c, SUC_lexicon_tags_37 },
    { 0x5b15df3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef75df41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd55df43, SUC_lexicon_tags_3 },
    { 0x8435df43, SUC_lexicon_tags_37 },
    { 0xedf5df45, SUC_lexicon_tags_63 },
    { 0x55f5df44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf015df4d, SUC_lexicon_tags_12 },
    { 0xb0f5df4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a55df55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc955df57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4d95df59, SUC_lexicon_tags_37 },
    { 0xd715df5a, SUC_lexicon_tags_2 },
    { 0xdd95df5b, SUC_lexicon_tags_1041 },
    { 0xe7d5df5c, SUC_lexicon_tags_47 },
    { 0x8bf5df5a, SUC_lexicon_tags_16 },
    { 0x5055df5e, SUC_lexicon_tags_37 },
    { 0x1035df5e, SUC_lexicon_tags_2 },
    { 0x9d75df60, SUC_lexicon_tags_14 },
    { 0x26d5df61, SUC_lexicon_tags_4 },
    { 0xceb5df62, SUC_lexicon_tags_1 },
    { 0xc8b5df5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f5df66, SUC_lexicon_tags_32 },
    { 0x9b5df66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1495df6b, SUC_lexicon_tags_37 },
    { 0x2555df6c, SUC_lexicon_tags_23 },
    { 0xfe55df6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5df72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb55df74, SUC_lexicon_tags_15 },
    { 0x8bd5df75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b5df7a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b5df7e, SUC_lexicon_tags_7 },
    { 0x95b5df7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695df83, SUC_lexicon_tags_8 },
    { 0x1e15df84, SUC_lexicon_tags_10 },
    { 0x7375df85, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf8d5df87, SUC_lexicon_tags_125 },
    { 0xe815df88, SUC_lexicon_tags_25 },
    { 0x5675df87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c15df8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc875df92, SUC_lexicon_tags_674 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff5df97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7395df99, SUC_lexicon_tags_0 },
    { 0x1e55df9a, SUC_lexicon_tags_740 },
    { 0x3415df99, SUC_lexicon_tags_27 },
    { 0xe1d5df9b, SUC_lexicon_tags_65 },
    { 0x70f5df9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9315dfa0, SUC_lexicon_tags_17 },
    { 0x69b5dfa0, SUC_lexicon_tags_26 },
    { 0xc075dfa2, SUC_lexicon_tags_31 },
    { 0x5715dfa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b5dfa9, SUC_lexicon_tags_1 },
    { 0x995dfaa, SUC_lexicon_tags_6 },
    { 0xa375dfab, SUC_lexicon_tags_47 },
    { 0xc495dfac, SUC_lexicon_tags_17 },
    { 0x3d55dfad, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75dfb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3035dfba, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b5dfbd, SUC_lexicon_tags_37 },
    { 0x6f35dfbe, SUC_lexicon_tags_11 },
    { 0x6775dfbe, SUC_lexicon_tags_17 },
    { 0x5f5dfbf, SUC_lexicon_tags_25 },
    { 0xa3d5dfc1, SUC_lexicon_tags_47 },
    { 0x25d5dfbd, SUC_lexicon_tags_61 },
    { 0x59b5dfc2, SUC_lexicon_tags_10 },
    { 0xab75dfc4, SUC_lexicon_tags_7 },
    { 0x9535dfc5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d5dfca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ef5dfcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf8d5dfce, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4d55dfd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d5dfd3, SUC_lexicon_tags_43 },
    { 0xb975dfd4, SUC_lexicon_tags_25 },
    { 0x7575dfd3, SUC_lexicon_tags_63 },
    { 0xfcf5dfd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1075dfdd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd5dfe0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff5dfe8, SUC_lexicon_tags_33 },
    { 0x3015dfe9, SUC_lexicon_tags_27 },
    { 0xccd5dfe9, SUC_lexicon_tags_11 },
    { 0xb275dfeb, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df5dfee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f5dff2, SUC_lexicon_tags_49 },
    { 0x875dff2, SUC_lexicon_tags_15 },
    { 0xd015dff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b95dff7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb35dffd, SUC_lexicon_tags_37 },
    { 0xc435dffe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe215e001, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3235e006, SUC_lexicon_tags_39 },
    { 0x8195e006, SUC_lexicon_tags_7 },
    { 0xdb5e008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4b5e00a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd455e010, SUC_lexicon_tags_27 },
    { 0x94b5e010, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0, NULL },
    { 0x155e014, SUC_lexicon_tags_33 },
    { 0x6055e015, SUC_lexicon_tags_33 },
    { 0xc8d5e015, SUC_lexicon_tags_37 },
    { 0x88f5e017, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe55e019, SUC_lexicon_tags_17 },
    { 0x7915e01a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2975e01d, SUC_lexicon_tags_14 },
    { 0x7295e01e, SUC_lexicon_tags_2 },
    { 0x4d95e01f, SUC_lexicon_tags_58 },
    { 0x2e95e020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93b5e022, SUC_lexicon_tags_91 },
    { 0x3535e023, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb5e029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb5e02d, SUC_lexicon_tags_17 },
    { 0x48b5e02e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9855e030, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5e033, SUC_lexicon_tags_28 },
    { 0xb655e034, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb595e037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1995e039, SUC_lexicon_tags_12 },
    { 0xf495e039, SUC_lexicon_tags_1 },
    { 0xbad5e03b, SUC_lexicon_tags_0 },
    { 0x1195e03c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc055e03e, SUC_lexicon_tags_29 },
    { 0x7435e03e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43b5e041, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e55e044, SUC_lexicon_tags_11 },
    { 0x56b5e044, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x11b5e047, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd5e04a, SUC_lexicon_tags_27 },
    { 0x26b5e04b, SUC_lexicon_tags_83 },
    { 0x2a95e04c, SUC_lexicon_tags_63 },
    { 0x7f55e04b, SUC_lexicon_tags_11 },
    { 0xbf95e04d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1515e051, SUC_lexicon_tags_25 },
    { 0xa9f5e051, SUC_lexicon_tags_8 },
    { 0x6cd5e053, SUC_lexicon_tags_30 },
    { 0x1795e054, SUC_lexicon_tags_37 },
    { 0x575e055, SUC_lexicon_tags_2 },
    { 0xe935e055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37f5e058, SUC_lexicon_tags_22 },
    { 0x5b15e058, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xaef5e05b, SUC_lexicon_tags_4 },
    { 0x715e05c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb355e060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3195e064, SUC_lexicon_tags_4 },
    { 0x6775e064, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7135e067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f5e06d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf5e071, SUC_lexicon_tags_17 },
    { 0xd375e072, SUC_lexicon_tags_1 },
    { 0xe435e072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e95e078, SUC_lexicon_tags_84 },
    { 0x67b5e078, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6635e084, SUC_lexicon_tags_160 },
    { 0x255e085, SUC_lexicon_tags_12 },
    { 0xc495e086, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7df5e088, SUC_lexicon_tags_15 },
    { 0xe995e088, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcb35e08b, SUC_lexicon_tags_7 },
    { 0xf15e08c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b5e090, SUC_lexicon_tags_11 },
    { 0x6a75e090, SUC_lexicon_tags_15 },
    { 0x7535e092, SUC_lexicon_tags_7 },
    { 0x5ab5e093, SUC_lexicon_tags_62 },
    { 0x8df5e094, SUC_lexicon_tags_30 },
    { 0x7bd5e095, SUC_lexicon_tags_7 },
    { 0x43f5e093, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc35e098, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc55e09a, SUC_lexicon_tags_26 },
    { 0xe415e09a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c15e09e, SUC_lexicon_tags_34 },
    { 0xffb5e09f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3095e0a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d5e0aa, SUC_lexicon_tags_60 },
    { 0x74d5e0aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d5e0b0, SUC_lexicon_tags_27 },
    { 0xf4f5e0b1, SUC_lexicon_tags_61 },
    { 0xa815e0b0, SUC_lexicon_tags_14 },
    { 0x4675e0b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d5e0b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9155e0b9, SUC_lexicon_tags_2 },
    { 0xdc35e0b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff5e0bf, SUC_lexicon_tags_25 },
    { 0x1e95e0c0, SUC_lexicon_tags_13 },
    { 0xb2f5e0c1, SUC_lexicon_tags_45 },
    { 0x6cb5e0c2, SUC_lexicon_tags_25 },
    { 0x43d5e0c3, SUC_lexicon_tags_25 },
    { 0xbf35e0c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd5e0c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa975e0ca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe135e0cc, SUC_lexicon_tags_194 },
    { 0xc5d5e0cd, SUC_lexicon_tags_63 },
    { 0xf715e0cd, SUC_lexicon_tags_35 },
    { 0x34f5e0ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1475e0d3, SUC_lexicon_tags_11 },
    { 0xf215e0d4, SUC_lexicon_tags_17 },
    { 0xe35e0d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c15e0dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab5e0e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad5e0eb, SUC_lexicon_tags_68 },
    { 0x7cf5e0ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb455e0f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d5e0fc, SUC_lexicon_tags_0 },
    { 0x6515e0fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d95e104, SUC_lexicon_tags_0 },
    { 0x3dd5e104, SUC_lexicon_tags_53 },
    { 0x7215e106, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c55e109, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd75e10c, SUC_lexicon_tags_12 },
    { 0x1435e10d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db5e116, SUC_lexicon_tags_11 },
    { 0xc355e116, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd5e11c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6175e11e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2675e120, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe815e122, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d5e126, SUC_lexicon_tags_47 },
    { 0xedd5e127, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1495e12e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af5e132, SUC_lexicon_tags_15 },
    { 0xaa55e133, SUC_lexicon_tags_146 },
    { 0xeff5e133, SUC_lexicon_tags_17 },
    { 0xc9d5e135, SUC_lexicon_tags_2 },
    { 0x49d5e136, SUC_lexicon_tags_0 },
    { 0xcd35e137, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5055e139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8df5e13b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb875e13d, SUC_lexicon_tags_26 },
    { 0x27d5e13e, SUC_lexicon_tags_37 },
    { 0xa455e13f, SUC_lexicon_tags_6 },
    { 0xc3f5e140, SUC_lexicon_tags_28 },
    { 0x3cb5e141, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9b5e143, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555e14e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x875e150, SUC_lexicon_tags_1 },
    { 0xd55e151, SUC_lexicon_tags_43 },
    { 0xca95e152, SUC_lexicon_tags_28 },
    { 0xb95e151, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd5d5e155, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa915e158, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x9e55e15a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe6d5e15c, SUC_lexicon_tags_7 },
    { 0xcb75e15c, SUC_lexicon_tags_1 },
    { 0xd595e15e, SUC_lexicon_tags_46 },
    { 0x5b15e15f, SUC_lexicon_tags_1 },
    { 0xa0b5e15d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded5e166, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5e16d, SUC_lexicon_tags_15 },
    { 0xb4b5e16e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d5e171, SUC_lexicon_tags_11 },
    { 0x3b55e172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa115e17c, SUC_lexicon_tags_37 },
    { 0x4215e17c, SUC_lexicon_tags_2 },
    { 0xffd5e17d, SUC_lexicon_tags_18 },
    { 0xdd95e17f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3b5e181, SUC_lexicon_tags_11 },
    { 0xf2f5e182, SUC_lexicon_tags_12 },
    { 0x6235e181, SUC_lexicon_tags_65 },
    { 0x83d5e184, SUC_lexicon_tags_25 },
    { 0x955e185, SUC_lexicon_tags_25 },
    { 0x4255e181, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd195e190, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a35e195, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5e19c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6bf5e19e, SUC_lexicon_tags_60 },
    { 0x1895e19f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf15e1a1, SUC_lexicon_tags_28 },
    { 0xaa75e1a2, SUC_lexicon_tags_65 },
    { 0xe015e1a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb5e1aa, SUC_lexicon_tags_37 },
    { 0x29d5e1aa, SUC_lexicon_tags_14 },
    { 0xdd15e1aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ab5e1ae, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd5e1b1, SUC_lexicon_tags_17 },
    { 0x475e1b2, SUC_lexicon_tags_1 },
    { 0xa4f5e1b1, SUC_lexicon_tags_12 },
    { 0x2fd5e1b4, SUC_lexicon_tags_2 },
    { 0x71f5e1b4, SUC_lexicon_tags_49 },
    { 0xca55e1b6, SUC_lexicon_tags_39 },
    { 0xa435e1b7, SUC_lexicon_tags_15 },
    { 0xdd95e1b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b5e1be, SUC_lexicon_tags_13 },
    { 0xd695e1bf, SUC_lexicon_tags_91 },
    { 0x7eb5e1c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f5e1c4, SUC_lexicon_tags_17 },
    { 0xd055e1c4, SUC_lexicon_tags_27 },
    { 0xb15e1c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb615e1c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe095e1d0, SUC_lexicon_tags_27 },
    { 0x235e1d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d5e1d4, SUC_lexicon_tags_23 },
    { 0x74f5e1d5, SUC_lexicon_tags_31 },
    { 0xf635e1d6, SUC_lexicon_tags_25 },
    { 0xb815e1d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb95e1db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff5e1de, SUC_lexicon_tags_31 },
    { 0x4db5e1de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebf5e1e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f5e1e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf315e1e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf95e1f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc095e1f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb75e1fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfeb5e1fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3d15e1fe, SUC_lexicon_tags_12 },
    { 0x5295e1ff, SUC_lexicon_tags_21 },
    { 0xbed5e1fe, SUC_lexicon_tags_15 },
    { 0x19d5e1fe, SUC_lexicon_tags_37 },
    { 0x7f95e1ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf5e205, SUC_lexicon_tags_49 },
    { 0x8095e206, SUC_lexicon_tags_11 },
    { 0xcc55e207, SUC_lexicon_tags_22 },
    { 0x72f5e207, SUC_lexicon_tags_25 },
    { 0xda55e205, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1355e20d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb5e20f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a75e211, SUC_lexicon_tags_37 },
    { 0x8f5e212, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4335e215, SUC_lexicon_tags_37 },
    { 0x7ad5e216, SUC_lexicon_tags_66 },
    { 0xca75e217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9395e21c, SUC_lexicon_tags_12 },
    { 0x1835e21d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ad5e21f, SUC_lexicon_tags_25 },
    { 0xf2f5e220, SUC_lexicon_tags_84 },
    { 0xb935e221, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d5e225, SUC_lexicon_tags_47 },
    { 0x3cd5e225, SUC_lexicon_tags_1 },
    { 0xa015e227, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd35e22a, SUC_lexicon_tags_15 },
    { 0x4e15e22b, SUC_lexicon_tags_63 },
    { 0x3c35e22b, SUC_lexicon_tags_8 },
    { 0xa135e22d, SUC_lexicon_tags_12 },
    { 0x19d5e22d, SUC_lexicon_tags_55 },
    { 0x2f5e22f, SUC_lexicon_tags_12 },
    { 0x8a35e230, SUC_lexicon_tags_2 },
    { 0x64f5e231, SUC_lexicon_tags_37 },
    { 0x27f5e231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb515e234, SUC_lexicon_tags_11 },
    { 0x75e235, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3315e239, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f5e23e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa875e249, SUC_lexicon_tags_12 },
    { 0x77b5e249, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5e24f, SUC_lexicon_tags_11 },
    { 0x6e75e250, SUC_lexicon_tags_37 },
    { 0xc8f5e24f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5895e25c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc75e261, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b5e264, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2435e26b, SUC_lexicon_tags_11 },
    { 0xaf95e26c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7715e26e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d5e275, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29d5e277, SUC_lexicon_tags_44 },
    { 0x6c75e277, SUC_lexicon_tags_46 },
    { 0xfd75e278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa755e27f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc095e281, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc135e283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5e287, SUC_lexicon_tags_36 },
    { 0x59b5e288, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3915e28b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56b5e28d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7f5e28f, SUC_lexicon_tags_38 },
    { 0x3475e28f, SUC_lexicon_tags_61 },
    { 0xd155e291, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9195e293, SUC_lexicon_tags_4 },
    { 0xab35e293, SUC_lexicon_tags_882 },
    { 0xaab5e295, SUC_lexicon_tags_1 },
    { 0x38b5e294, SUC_lexicon_tags_11 },
    { 0x1735e295, SUC_lexicon_tags_10 },
    { 0x17d5e298, SUC_lexicon_tags_6 },
    { 0xd435e298, SUC_lexicon_tags_14 },
    { 0x8eb5e29a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc95e2a1, SUC_lexicon_tags_10 },
    { 0x3c35e2a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b5e2a6, SUC_lexicon_tags_2 },
    { 0xe435e2a7, SUC_lexicon_tags_12 },
    { 0x6535e2a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb415e2ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3335e2b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b5e2b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2055e2b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1035e2bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d5e2c1, SUC_lexicon_tags_27 },
    { 0x9bf5e2c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1595e2c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb5e2c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa75e2ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c35e2cc, SUC_lexicon_tags_49 },
    { 0xca75e2cd, SUC_lexicon_tags_2 },
    { 0x1cf5e2ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b5e2dc, SUC_lexicon_tags_39 },
    { 0x32f5e2dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b5e2e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22f5e2e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd5e2e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f5e2ed, SUC_lexicon_tags_2 },
    { 0x7575e2ed, SUC_lexicon_tags_26 },
    { 0x5d75e2ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f75e2f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2315e2f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd55e2fb, SUC_lexicon_tags_37 },
    { 0x2cb5e2fc, SUC_lexicon_tags_80 },
    { 0x9e75e2fd, SUC_lexicon_tags_45 },
    { 0xaf95e2fe, SUC_lexicon_tags_46 },
    { 0x9195e2ff, SUC_lexicon_tags_65 },
    { 0x2f95e2fb, SUC_lexicon_tags_68 },
    { 0x20b5e2fd, SUC_lexicon_tags_14 },
    { 0x22b5e300, SUC_lexicon_tags_15 },
    { 0x1115e2fb, SUC_lexicon_tags_55 },
    { 0x8c75e302, SUC_lexicon_tags_2 },
    { 0xeff5e301, SUC_lexicon_tags_37 },
    { 0x4535e306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b5e30e, SUC_lexicon_tags_1 },
    { 0xd195e30e, SUC_lexicon_tags_7 },
    { 0x8715e310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb5e314, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b75e316, SUC_lexicon_tags_7 },
    { 0x7df5e316, SUC_lexicon_tags_2 },
    { 0x5995e318, SUC_lexicon_tags_0 },
    { 0x9e15e319, SUC_lexicon_tags_17 },
    { 0x7eb5e318, SUC_lexicon_tags_66 },
    { 0xba95e31b, SUC_lexicon_tags_12 },
    { 0xb0b5e31c, SUC_lexicon_tags_28 },
    { 0x76b5e31d, SUC_lexicon_tags_46 },
    { 0xac75e31b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xf655e321, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x7335e324, SUC_lexicon_tags_63 },
    { 0xcef5e325, SUC_lexicon_tags_12 },
    { 0x1ef5e326, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6135e32b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b5e32e, SUC_lexicon_tags_11 },
    { 0x7b35e32e, SUC_lexicon_tags_15 },
    { 0xd8d5e32e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f5e336, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa695e338, SUC_lexicon_tags_37 },
    { 0x43b5e338, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3395e33d, SUC_lexicon_tags_57 },
    { 0x6f5e33e, SUC_lexicon_tags_29 },
    { 0x5315e33f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf595e341, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55e343, SUC_lexicon_tags_15 },
    { 0x8435e343, SUC_lexicon_tags_12 },
    { 0x1d5e343, SUC_lexicon_tags_48 },
    { 0x2635e346, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdc55e348, SUC_lexicon_tags_25 },
    { 0x135e349, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6b35e34b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4615e34d, SUC_lexicon_tags_37 },
    { 0xfa75e34e, SUC_lexicon_tags_63 },
    { 0xec15e34f, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x1695e351, SUC_lexicon_tags_25 },
    { 0x13f5e352, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5e358, SUC_lexicon_tags_27 },
    { 0x22f5e359, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4995e35d, SUC_lexicon_tags_91 },
    { 0x9af5e35d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa415e361, SUC_lexicon_tags_1 },
    { 0x4a15e361, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x615e365, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b5e36d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7f5e36f, SUC_lexicon_tags_56 },
    { 0x6a75e36f, SUC_lexicon_tags_37 },
    { 0xe2d5e371, SUC_lexicon_tags_19 },
    { 0xcef5e372, SUC_lexicon_tags_23 },
    { 0x94b5e370, SUC_lexicon_tags_23 },
    { 0xb5b5e373, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd75e376, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb215e378, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5875e37b, SUC_lexicon_tags_12 },
    { 0xc5f5e37c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b5e381, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef55e384, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf5e387, SUC_lexicon_tags_29 },
    { 0x1c35e388, SUC_lexicon_tags_0 },
    { 0xf095e388, SUC_lexicon_tags_7 },
    { 0x4cd5e38a, SUC_lexicon_tags_19 },
    { 0x1715e387, SUC_lexicon_tags_11 },
    { 0xf5e38b, SUC_lexicon_tags_40 },
    { 0x3c15e38b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d5e390, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf875e392, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb755e394, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe55e39b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3095e3a0, SUC_lexicon_tags_25 },
    { 0xe115e3a1, SUC_lexicon_tags_8 },
    { 0x9935e3a2, SUC_lexicon_tags_11 },
    { 0xbb55e3a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d5e3ab, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5915e3b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5b5e3b3, SUC_lexicon_tags_2 },
    { 0x4015e3b3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb235e3b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa275e3bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf15e3bf, SUC_lexicon_tags_23 },
    { 0x5c35e3c0, SUC_lexicon_tags_49 },
    { 0x5675e3c0, SUC_lexicon_tags_17 },
    { 0xbb95e3bf, SUC_lexicon_tags_0 },
    { 0x4eb5e3c3, SUC_lexicon_tags_0 },
    { 0xe135e3c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d15e3ca, SUC_lexicon_tags_77 },
    { 0x4d95e3ca, SUC_lexicon_tags_7 },
    { 0x5795e3cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9b5e3ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc775e3d2, SUC_lexicon_tags_26 },
    { 0xbf5e3d3, SUC_lexicon_tags_7 },
    { 0x555e3d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a95e3da, SUC_lexicon_tags_57 },
    { 0x87f5e3db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd5e3e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3875e3e4, SUC_lexicon_tags_118 },
    { 0x7b75e3e5, SUC_lexicon_tags_11 },
    { 0x3a35e3e6, SUC_lexicon_tags_23 },
    { 0x1855e3e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd035e3ea, SUC_lexicon_tags_25 },
    { 0x9e35e3eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2475e3ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26f5e3ef, SUC_lexicon_tags_1 },
    { 0xa5b5e3ef, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b5e3f3, SUC_lexicon_tags_11 },
    { 0x2055e3f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b95e3f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7475e3f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9735e3fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa815e3fd, SUC_lexicon_tags_2 },
    { 0x6575e3fd, SUC_lexicon_tags_17 },
    { 0x8a75e3ff, SUC_lexicon_tags_7 },
    { 0xd855e400, SUC_lexicon_tags_1 },
    { 0xec95e401, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1095e403, SUC_lexicon_tags_12 },
    { 0x3815e404, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cb5e406, SUC_lexicon_tags_48 },
    { 0x2ed5e407, SUC_lexicon_tags_15 },
    { 0x1e55e407, SUC_lexicon_tags_38 },
    { 0x2fd5e409, SUC_lexicon_tags_25 },
    { 0xdf75e409, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe095e40c, SUC_lexicon_tags_0 },
    { 0x3cd5e40d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd275e40f, SUC_lexicon_tags_15 },
    { 0x87d5e40f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdb95e412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb55e414, SUC_lexicon_tags_25 },
    { 0xe715e415, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d5e41f, SUC_lexicon_tags_23 },
    { 0x25f5e420, SUC_lexicon_tags_25 },
    { 0xc395e421, SUC_lexicon_tags_11 },
    { 0xb1f5e420, SUC_lexicon_tags_38 },
    { 0xb055e421, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe835e428, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e55e42e, SUC_lexicon_tags_11 },
    { 0x2435e42e, SUC_lexicon_tags_30 },
    { 0x99d5e42e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xced5e433, SUC_lexicon_tags_7 },
    { 0x6755e433, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3455e439, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc515e43e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab5e441, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe695e447, SUC_lexicon_tags_17 },
    { 0xfe95e448, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xda95e44b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d15e450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f35e452, SUC_lexicon_tags_43 },
    { 0x4b35e452, SUC_lexicon_tags_25 },
    { 0xf2b5e452, SUC_lexicon_tags_2 },
    { 0xb1d5e455, SUC_lexicon_tags_12 },
    { 0xcb95e454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f5e463, SUC_lexicon_tags_27 },
    { 0xf8d5e464, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b5e468, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef5e46b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3e55e46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b5e471, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e35e474, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa7b5e476, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6595e483, SUC_lexicon_tags_81 },
    { 0x3235e484, SUC_lexicon_tags_11 },
    { 0x4b15e485, SUC_lexicon_tags_7 },
    { 0xc3b5e486, SUC_lexicon_tags_43 },
    { 0x45b5e485, SUC_lexicon_tags_68 },
    { 0xd3d5e486, SUC_lexicon_tags_25 },
    { 0x7d35e488, SUC_lexicon_tags_60 },
    { 0xaf75e487, SUC_lexicon_tags_28 },
    { 0x45b5e488, SUC_lexicon_tags_17 },
    { 0x5a75e48c, SUC_lexicon_tags_7 },
    { 0xda55e488, SUC_lexicon_tags_15 },
    { 0x12f5e48e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a15e490, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db5e493, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f5e498, SUC_lexicon_tags_37 },
    { 0x9855e499, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6395e49b, SUC_lexicon_tags_9 },
    { 0x7395e49c, SUC_lexicon_tags_17 },
    { 0xca15e49d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f5e4a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf815e4a5, SUC_lexicon_tags_19 },
    { 0xad15e4a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f5e4a9, SUC_lexicon_tags_12 },
    { 0xfad5e4aa, SUC_lexicon_tags_68 },
    { 0x5915e4aa, SUC_lexicon_tags_28 },
    { 0x6995e4ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe235e4b6, SUC_lexicon_tags_12 },
    { 0x1715e4b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca95e4ba, SUC_lexicon_tags_14 },
    { 0x9a35e4bb, SUC_lexicon_tags_30 },
    { 0xb9d5e4bc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad5e4bf, SUC_lexicon_tags_52 },
    { 0x1ad5e4bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf5e4c3, SUC_lexicon_tags_1 },
    { 0xf3d5e4c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad5e4c9, SUC_lexicon_tags_6 },
    { 0x9695e4ca, SUC_lexicon_tags_21 },
    { 0xde35e4c9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xfe75e4cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa455e4d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f35e4dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b5e4e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc175e4e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed5e4ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd75e4f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb5e4f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f35e4ff, SUC_lexicon_tags_0 },
    { 0xf995e500, SUC_lexicon_tags_29 },
    { 0xd5d5e4ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6035e504, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfd35e506, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2315e50a, SUC_lexicon_tags_66 },
    { 0xd315e50b, SUC_lexicon_tags_2 },
    { 0x4ef5e50c, SUC_lexicon_tags_39 },
    { 0xa435e50c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ad5e50f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6415e516, SUC_lexicon_tags_2 },
    { 0x3c95e517, SUC_lexicon_tags_6 },
    { 0xdb15e516, SUC_lexicon_tags_31 },
    { 0xd1b5e517, SUC_lexicon_tags_43 },
    { 0x6e15e519, SUC_lexicon_tags_25 },
    { 0xafd5e516, SUC_lexicon_tags_9 },
    { 0x1535e51c, SUC_lexicon_tags_70 },
    { 0xda95e51d, SUC_lexicon_tags_11 },
    { 0xf7f5e51e, SUC_lexicon_tags_6 },
    { 0x5e35e51f, SUC_lexicon_tags_17 },
    { 0x4095e51d, SUC_lexicon_tags_28 },
    { 0x4fd5e521, SUC_lexicon_tags_9 },
    { 0xde55e522, SUC_lexicon_tags_26 },
    { 0x6815e523, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f5e529, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x3bd5e52b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x1875e52e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8355e530, SUC_lexicon_tags_65 },
    { 0xa955e530, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd5e535, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa1b5e537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb975e539, SUC_lexicon_tags_23 },
    { 0x8775e539, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b55e53d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd5e542, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf75e544, SUC_lexicon_tags_3 },
    { 0xe8d5e544, SUC_lexicon_tags_38 },
    { 0xc0d5e546, SUC_lexicon_tags_29 },
    { 0xa35e547, SUC_lexicon_tags_48 },
    { 0xd4d5e545, SUC_lexicon_tags_2 },
    { 0xe935e548, SUC_lexicon_tags_63 },
    { 0x3195e546, SUC_lexicon_tags_49 },
    { 0x4255e54b, SUC_lexicon_tags_17 },
    { 0xa75e54c, SUC_lexicon_tags_21 },
    { 0xee15e54d, SUC_lexicon_tags_27 },
    { 0x415e54e, SUC_lexicon_tags_63 },
    { 0x5e75e545, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d5e559, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf635e55b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2755e561, SUC_lexicon_tags_36 },
    { 0x2735e561, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f5e566, SUC_lexicon_tags_12 },
    { 0x1a55e566, SUC_lexicon_tags_17 },
    { 0x5915e568, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4255e56a, SUC_lexicon_tags_25 },
    { 0xed5e56a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf5e56f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bf5e571, SUC_lexicon_tags_104 },
    { 0x8a15e572, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f5e576, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc295e579, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbf5e57b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b5e580, SUC_lexicon_tags_69 },
    { 0x1135e580, SUC_lexicon_tags_15 },
    { 0x93d5e580, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4eb5e584, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d55e589, SUC_lexicon_tags_21 },
    { 0x1b95e58a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d5e58d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4135e58f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d5e592, SUC_lexicon_tags_47 },
    { 0xcd15e592, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x2115e596, SUC_lexicon_tags_2 },
    { 0xa855e596, SUC_lexicon_tags_1 },
    { 0xdcb5e598, SUC_lexicon_tags_55 },
    { 0xaa15e598, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3595e59b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x235e59d, SUC_lexicon_tags_65 },
    { 0x3a35e59e, SUC_lexicon_tags_46 },
    { 0x10b5e59f, SUC_lexicon_tags_57 },
    { 0xaad5e5a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab5e5a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6335e5af, SUC_lexicon_tags_2 },
    { 0xc075e5af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b15e5b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x995e5b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6375e5bd, SUC_lexicon_tags_15 },
    { 0x6855e5be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda55e5c5, SUC_lexicon_tags_1 },
    { 0xce75e5c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x80d5e5c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ed5e5ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7275e5cd, SUC_lexicon_tags_48 },
    { 0x7995e5cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f95e5d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd835e5d4, SUC_lexicon_tags_0 },
    { 0x7b15e5d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b95e5d8, SUC_lexicon_tags_23 },
    { 0xf6d5e5d9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b35e5dd, SUC_lexicon_tags_11 },
    { 0xaf15e5de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa815e5e0, SUC_lexicon_tags_31 },
    { 0xa815e5e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b5e5e5, SUC_lexicon_tags_17 },
    { 0x3695e5e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d5e5eb, SUC_lexicon_tags_12 },
    { 0x3f15e5eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5e5f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5575e5fa, SUC_lexicon_tags_8 },
    { 0xeab5e5fa, SUC_lexicon_tags_116 },
    { 0xe4d5e5fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1275e5fe, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xd4f5e600, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa975e604, SUC_lexicon_tags_2 },
    { 0x8475e604, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d5e608, SUC_lexicon_tags_37 },
    { 0x28f5e608, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b35e60e, SUC_lexicon_tags_26 },
    { 0x32b5e60f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45d5e611, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe355e614, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95d5e616, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2af5e618, SUC_lexicon_tags_12 },
    { 0x315e619, SUC_lexicon_tags_30 },
    { 0x5475e61a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd5e61e, SUC_lexicon_tags_11 },
    { 0xb975e61e, SUC_lexicon_tags_138 },
    { 0x53b5e620, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd535e623, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595e62c, SUC_lexicon_tags_37 },
    { 0xc795e62d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3675e62f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc75e632, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b75e635, SUC_lexicon_tags_17 },
    { 0xdcf5e636, SUC_lexicon_tags_12 },
    { 0x6055e637, SUC_lexicon_tags_3 },
    { 0xda55e637, SUC_lexicon_tags_32 },
    { 0x4895e637, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8555e63c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55e63f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d5e642, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1e55e644, SUC_lexicon_tags_17 },
    { 0xd315e644, SUC_lexicon_tags_8 },
    { 0x9495e645, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c95e648, SUC_lexicon_tags_37 },
    { 0x8595e649, SUC_lexicon_tags_9 },
    { 0x6c95e64a, SUC_lexicon_tags_33 },
    { 0x3775e64b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37f5e64d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9f5e64f, SUC_lexicon_tags_7 },
    { 0x9e55e650, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa75e652, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x22f5e654, SUC_lexicon_tags_49 },
    { 0x21f5e655, SUC_lexicon_tags_38 },
    { 0x40d5e656, SUC_lexicon_tags_77 },
    { 0xd2d5e656, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d5e65c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafb5e65e, SUC_lexicon_tags_17 },
    { 0xa2f5e65e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe95e661, SUC_lexicon_tags_12 },
    { 0x8675e662, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf5e66b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda75e671, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf055e674, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8855e67c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd5e67f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6895e681, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa935e683, SUC_lexicon_tags_61 },
    { 0x63b5e684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f5e688, SUC_lexicon_tags_11 },
    { 0x1315e689, SUC_lexicon_tags_37 },
    { 0xe6d5e68a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d5e68c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d5e692, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3255e695, SUC_lexicon_tags_27 },
    { 0xa755e696, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef5e69b, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f5e69f, SUC_lexicon_tags_7 },
    { 0x1555e69f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f5e6a9, SUC_lexicon_tags_37 },
    { 0xf4b5e6a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d5e6b0, SUC_lexicon_tags_17 },
    { 0xfe15e6b0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4c95e6b3, SUC_lexicon_tags_4 },
    { 0x2a35e6b4, SUC_lexicon_tags_12 },
    { 0xacd5e6b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3195e6b7, SUC_lexicon_tags_38 },
    { 0xef75e6b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7b5e6ba, SUC_lexicon_tags_37 },
    { 0xd6b5e6bb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d5e6c8, SUC_lexicon_tags_31 },
    { 0xb355e6c9, SUC_lexicon_tags_11 },
    { 0x3cd5e6c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7115e6cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb375e6d0, SUC_lexicon_tags_37 },
    { 0xf115e6d0, SUC_lexicon_tags_1329 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d5e6d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b5e6da, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d5e6df, SUC_lexicon_tags_7 },
    { 0x8d5e6df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b5e6e3, SUC_lexicon_tags_11 },
    { 0xa235e6e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb15e6ef, SUC_lexicon_tags_37 },
    { 0xf395e6f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x50b5e6f2, SUC_lexicon_tags_7 },
    { 0x115e6f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe155e6f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75e6fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd775e6fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc95e701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd195e703, SUC_lexicon_tags_23 },
    { 0x7ad5e704, SUC_lexicon_tags_0 },
    { 0xfcd5e704, SUC_lexicon_tags_7 },
    { 0xbe95e706, SUC_lexicon_tags_23 },
    { 0xb55e705, SUC_lexicon_tags_55 },
    { 0xb95e704, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe955e70b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb35e70d, SUC_lexicon_tags_33 },
    { 0xfb55e70e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad5e713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe775e718, SUC_lexicon_tags_7 },
    { 0xead5e719, SUC_lexicon_tags_8 },
    { 0xff75e71a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc15e71e, SUC_lexicon_tags_16 },
    { 0xe835e71f, SUC_lexicon_tags_7 },
    { 0x8135e720, SUC_lexicon_tags_45 },
    { 0x5075e71e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b5e724, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x6955e726, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe055e729, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa775e72d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf815e734, SUC_lexicon_tags_15 },
    { 0x94f5e735, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d5e739, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef5e73d, SUC_lexicon_tags_25 },
    { 0x66b5e73e, SUC_lexicon_tags_15 },
    { 0xc615e73f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf5e745, SUC_lexicon_tags_7 },
    { 0x1c95e746, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9415e74b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1b15e74d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd955e750, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe515e752, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x8195e754, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f35e75a, SUC_lexicon_tags_15 },
    { 0x4ad5e75b, SUC_lexicon_tags_2 },
    { 0xf035e75b, SUC_lexicon_tags_88 },
    { 0x38b5e75a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4255e75f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d5e763, SUC_lexicon_tags_11 },
    { 0x4a15e763, SUC_lexicon_tags_47 },
    { 0xfe35e764, SUC_lexicon_tags_75 },
    { 0xab95e764, SUC_lexicon_tags_49 },
    { 0x2b95e767, SUC_lexicon_tags_47 },
    { 0xaa15e766, SUC_lexicon_tags_28 },
    { 0x8a95e769, SUC_lexicon_tags_77 },
    { 0x2ab5e765, SUC_lexicon_tags_11 },
    { 0xddf5e767, SUC_lexicon_tags_48 },
    { 0x30f5e766, SUC_lexicon_tags_16 },
    { 0xf815e76d, SUC_lexicon_tags_47 },
    { 0xc9b5e76e, SUC_lexicon_tags_11 },
    { 0xda55e76f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f5e774, SUC_lexicon_tags_11 },
    { 0x28d5e774, SUC_lexicon_tags_32 },
    { 0x5e35e774, SUC_lexicon_tags_11 },
    { 0xfe35e777, SUC_lexicon_tags_1 },
    { 0x25d5e774, SUC_lexicon_tags_5 },
    { 0x3735e774, SUC_lexicon_tags_1 },
    { 0xc295e77a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbab5e77c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1375e781, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6875e784, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d15e787, SUC_lexicon_tags_12 },
    { 0x6bb5e788, SUC_lexicon_tags_65 },
    { 0xbab5e788, SUC_lexicon_tags_47 },
    { 0x42d5e788, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f15e78c, SUC_lexicon_tags_11 },
    { 0x2715e78c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff5e790, SUC_lexicon_tags_25 },
    { 0x8875e791, SUC_lexicon_tags_2 },
    { 0xf135e791, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3795e795, SUC_lexicon_tags_15 },
    { 0x2dd5e796, SUC_lexicon_tags_46 },
    { 0x3af5e797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b95e799, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8795e79b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8015e79f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf575e7a1, SUC_lexicon_tags_32 },
    { 0x8815e7a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa275e7a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda75e7a9, SUC_lexicon_tags_25 },
    { 0x8e95e7aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7835e7b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8955e7b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf915e7b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7255e7b9, SUC_lexicon_tags_25 },
    { 0x9ad5e7ba, SUC_lexicon_tags_37 },
    { 0xa215e7bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf55e7bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd75e7c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f15e7c6, SUC_lexicon_tags_17 },
    { 0xbb55e7c7, SUC_lexicon_tags_15 },
    { 0xe735e7c8, SUC_lexicon_tags_53 },
    { 0xa215e7c9, SUC_lexicon_tags_66 },
    { 0x1635e7c6, SUC_lexicon_tags_17 },
    { 0x8015e7cb, SUC_lexicon_tags_12 },
    { 0x1095e7cc, SUC_lexicon_tags_11 },
    { 0x1735e7cd, SUC_lexicon_tags_66 },
    { 0xa495e7cd, SUC_lexicon_tags_63 },
    { 0x4f35e7cf, SUC_lexicon_tags_43 },
    { 0xe9b5e7cf, SUC_lexicon_tags_30 },
    { 0xd315e7d1, SUC_lexicon_tags_15 },
    { 0xd4f5e7d2, SUC_lexicon_tags_14 },
    { 0xce55e7d1, SUC_lexicon_tags_60 },
    { 0x3f15e7d1, SUC_lexicon_tags_27 },
    { 0xfc35e7d5, SUC_lexicon_tags_2 },
    { 0xfa55e7ce, SUC_lexicon_tags_10 },
    { 0xd35e7d7, SUC_lexicon_tags_25 },
    { 0xf575e7d7, SUC_lexicon_tags_12 },
    { 0x9055e7d9, SUC_lexicon_tags_12 },
    { 0xd795e7da, SUC_lexicon_tags_15 },
    { 0xb0d5e7cb, SUC_lexicon_tags_10 },
    { 0x2095e7ce, SUC_lexicon_tags_1 },
    { 0xd235e7d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49f5e7df, SUC_lexicon_tags_29 },
    { 0x1555e7e0, SUC_lexicon_tags_12 },
    { 0x1455e7e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x475e7e3, SUC_lexicon_tags_11 },
    { 0x7475e7e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bd5e7e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ff5e7e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d5e7ec, SUC_lexicon_tags_11 },
    { 0x8375e7ed, SUC_lexicon_tags_39 },
    { 0xf335e7ee, SUC_lexicon_tags_2 },
    { 0xa0f5e7ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa135e7f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5315e7f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf55e7f5, SUC_lexicon_tags_17 },
    { 0x8835e7f6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1615e7fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb5e801, SUC_lexicon_tags_15 },
    { 0xf935e802, SUC_lexicon_tags_11 },
    { 0x99d5e803, SUC_lexicon_tags_1 },
    { 0xb415e803, SUC_lexicon_tags_37 },
    { 0x4bd5e802, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2195e807, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe655e80a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3895e80e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd5e816, SUC_lexicon_tags_12 },
    { 0xdfb5e816, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e75e81d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd635e821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8d5e823, SUC_lexicon_tags_60 },
    { 0x8f15e824, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6575e827, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc275e82d, SUC_lexicon_tags_43 },
    { 0xf3b5e82e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3f5e830, SUC_lexicon_tags_15 },
    { 0xb9b5e830, SUC_lexicon_tags_13 },
    { 0xbf35e832, SUC_lexicon_tags_28 },
    { 0xfed5e833, SUC_lexicon_tags_2 },
    { 0x1a75e834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d5e839, SUC_lexicon_tags_13 },
    { 0x1b15e83a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1735e83c, SUC_lexicon_tags_33 },
    { 0xc515e83d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b5e84c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e55e84e, SUC_lexicon_tags_6 },
    { 0x6015e84f, SUC_lexicon_tags_39 },
    { 0x8415e850, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b5e85d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf15e862, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9515e866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7035e86a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7395e86f, SUC_lexicon_tags_9 },
    { 0x75f5e870, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4055e872, SUC_lexicon_tags_15 },
    { 0x9ad5e873, SUC_lexicon_tags_39 },
    { 0xb4f5e874, SUC_lexicon_tags_12 },
    { 0x80d5e874, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1015e877, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x60b5e879, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb5e87f, SUC_lexicon_tags_45 },
    { 0xa0f5e880, SUC_lexicon_tags_11 },
    { 0xdab5e881, SUC_lexicon_tags_46 },
    { 0xdb95e87f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d75e885, SUC_lexicon_tags_31 },
    { 0xc775e886, SUC_lexicon_tags_28 },
    { 0x2ed5e887, SUC_lexicon_tags_45 },
    { 0xef35e886, SUC_lexicon_tags_639 },
    { 0x3095e889, SUC_lexicon_tags_3 },
    { 0x2c15e88a, SUC_lexicon_tags_17 },
    { 0xb295e88b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f5e892, SUC_lexicon_tags_25 },
    { 0xdd15e892, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d5e89b, SUC_lexicon_tags_17 },
    { 0x1115e89c, SUC_lexicon_tags_14 },
    { 0xd3b5e89d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35e8a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a55e8a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d5e8ae, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf195e8b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6395e8bd, SUC_lexicon_tags_15 },
    { 0x6835e8bd, SUC_lexicon_tags_33 },
    { 0xbed5e8bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc595e8c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44f5e8c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73f5e8c6, SUC_lexicon_tags_17 },
    { 0x8555e8c6, SUC_lexicon_tags_45 },
    { 0xcbd5e8c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3755e8ca, SUC_lexicon_tags_13 },
    { 0x66d5e8cb, SUC_lexicon_tags_25 },
    { 0x3a95e8cc, SUC_lexicon_tags_29 },
    { 0xdcb5e8cd, SUC_lexicon_tags_25 },
    { 0x5b95e8ce, SUC_lexicon_tags_17 },
    { 0x66b5e8cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb755e8d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b5e8d6, SUC_lexicon_tags_12 },
    { 0x2f95e8d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2115e8da, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f5e8e1, SUC_lexicon_tags_1 },
    { 0xc55e8e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b5e8e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ab5e8e9, SUC_lexicon_tags_37 },
    { 0x47f5e8ea, SUC_lexicon_tags_0 },
    { 0xf635e8eb, SUC_lexicon_tags_38 },
    { 0x155e8ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5495e8f1, SUC_lexicon_tags_9 },
    { 0xa3d5e8f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b35e8f8, SUC_lexicon_tags_63 },
    { 0xdd5e8f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e95e8fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6355e8ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad35e901, SUC_lexicon_tags_37 },
    { 0x94d5e902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5475e90c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3095e90e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfc15e910, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b5e91c, SUC_lexicon_tags_11 },
    { 0x9d95e91d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f5e920, SUC_lexicon_tags_43 },
    { 0xf5f5e921, SUC_lexicon_tags_37 },
    { 0xea15e922, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa455e92a, SUC_lexicon_tags_21 },
    { 0xe5b5e92a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcd5e92d, SUC_lexicon_tags_25 },
    { 0xaf5e92e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb5e934, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a35e937, SUC_lexicon_tags_39 },
    { 0x3c95e937, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7095e941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3155e943, SUC_lexicon_tags_47 },
    { 0x82b5e944, SUC_lexicon_tags_1 },
    { 0x8575e945, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc295e948, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b5e94b, SUC_lexicon_tags_7 },
    { 0x2c55e94c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2495e952, SUC_lexicon_tags_80 },
    { 0x5895e953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15e957, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f5e95a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e75e964, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6e95e966, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac35e96a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b5e972, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf035e974, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b5e97a, SUC_lexicon_tags_14 },
    { 0x5155e97b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb535e97d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d5e97f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd35e981, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3815e985, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed15e989, SUC_lexicon_tags_87 },
    { 0xb2d5e98a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3175e996, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7995e99a, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x3235e99c, SUC_lexicon_tags_27 },
    { 0x3bb5e99d, SUC_lexicon_tags_16 },
    { 0x4915e99e, SUC_lexicon_tags_1 },
    { 0x6175e99e, SUC_lexicon_tags_12 },
    { 0xc3f5e9a0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7395e9ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f5e9b2, SUC_lexicon_tags_43 },
    { 0xc275e9b3, SUC_lexicon_tags_29 },
    { 0x5f35e9b4, SUC_lexicon_tags_25 },
    { 0xc095e9b5, SUC_lexicon_tags_37 },
    { 0xf235e9b6, SUC_lexicon_tags_37 },
    { 0x8cb5e9b4, SUC_lexicon_tags_23 },
    { 0x2455e9b3, SUC_lexicon_tags_17 },
    { 0xfb35e9b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18f5e9bb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x8eb5e9bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f5e9c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b35e9cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2515e9ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f5e9d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f75e9d5, SUC_lexicon_tags_40 },
    { 0xfef5e9d5, SUC_lexicon_tags_11 },
    { 0xddb5e9d5, SUC_lexicon_tags_45 },
    { 0x7a75e9d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b5e9dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5275e9df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f15e9e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd955e9e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa35e9e9, SUC_lexicon_tags_34 },
    { 0x6815e9ea, SUC_lexicon_tags_25 },
    { 0x8b55e9eb, SUC_lexicon_tags_11 },
    { 0x75e9e9, SUC_lexicon_tags_43 },
    { 0x6455e9e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf615e9f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7575e9f4, SUC_lexicon_tags_2 },
    { 0x5915e9f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4675e9f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f5e9fb, SUC_lexicon_tags_25 },
    { 0xc7f5e9fb, SUC_lexicon_tags_56 },
    { 0xdc55e9fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75ea01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f5ea04, SUC_lexicon_tags_37 },
    { 0xded5ea05, SUC_lexicon_tags_28 },
    { 0xad15ea06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x96d5ea08, SUC_lexicon_tags_15 },
    { 0x77f5ea09, SUC_lexicon_tags_25 },
    { 0xbfb5ea09, SUC_lexicon_tags_17 },
    { 0xad15ea0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3175ea0d, SUC_lexicon_tags_485 },
    { 0xfe15ea0e, SUC_lexicon_tags_37 },
    { 0x1755ea0f, SUC_lexicon_tags_0 },
    { 0xd955ea10, SUC_lexicon_tags_14 },
    { 0x7555ea0f, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f5ea16, SUC_lexicon_tags_12 },
    { 0xc755ea17, SUC_lexicon_tags_2 },
    { 0x6cd5ea17, SUC_lexicon_tags_39 },
    { 0xdbd5ea16, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad5ea1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4ef5ea1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c55ea23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1eb5ea25, SUC_lexicon_tags_23 },
    { 0xa7b5ea26, SUC_lexicon_tags_23 },
    { 0xdbb5ea26, SUC_lexicon_tags_35 },
    { 0x1295ea26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5315ea2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b5ea2e, SUC_lexicon_tags_37 },
    { 0x24b5ea2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575ea33, SUC_lexicon_tags_12 },
    { 0x3e95ea34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb635ea37, SUC_lexicon_tags_25 },
    { 0x56d5ea37, SUC_lexicon_tags_23 },
    { 0x6635ea39, SUC_lexicon_tags_47 },
    { 0x4815ea38, SUC_lexicon_tags_14 },
    { 0x2395ea3b, SUC_lexicon_tags_65 },
    { 0x8a35ea3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec15ea3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc115ea40, SUC_lexicon_tags_37 },
    { 0x6075ea40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b35ea43, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b5ea46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa95ea4d, SUC_lexicon_tags_7 },
    { 0x1db5ea4e, SUC_lexicon_tags_28 },
    { 0x5115ea4e, SUC_lexicon_tags_29 },
    { 0x4315ea4d, SUC_lexicon_tags_1 },
    { 0xc0d5ea51, SUC_lexicon_tags_47 },
    { 0x6df5ea52, SUC_lexicon_tags_33 },
    { 0xc5d5ea53, SUC_lexicon_tags_170 },
    { 0xd775ea53, SUC_lexicon_tags_60 },
    { 0xdbd5ea55, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b5ea59, SUC_lexicon_tags_16 },
    { 0xa595ea59, SUC_lexicon_tags_7 },
    { 0xf075ea5a, SUC_lexicon_tags_959 },
    { 0x80f5ea59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe35ea5e, SUC_lexicon_tags_16 },
    { 0x7ab5ea5e, SUC_lexicon_tags_7 },
    { 0x4575ea60, SUC_lexicon_tags_2 },
    { 0xdf95ea61, SUC_lexicon_tags_11 },
    { 0x2b35ea62, SUC_lexicon_tags_14 },
    { 0xc1f5ea63, SUC_lexicon_tags_15 },
    { 0xc255ea64, SUC_lexicon_tags_33 },
    { 0x50f5ea65, SUC_lexicon_tags_0 },
    { 0x34f5ea61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc415ea68, SUC_lexicon_tags_31 },
    { 0xbd55ea69, SUC_lexicon_tags_65 },
    { 0x6e15ea6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7795ea6e, SUC_lexicon_tags_878 },
    { 0xc3f5ea6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d5ea78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77b5ea7a, SUC_lexicon_tags_114 },
    { 0xae35ea7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d5ea7e, SUC_lexicon_tags_14 },
    { 0x5dd5ea7f, SUC_lexicon_tags_35 },
    { 0xfd15ea7f, SUC_lexicon_tags_15 },
    { 0x5255ea81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d5ea85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8855ea89, SUC_lexicon_tags_46 },
    { 0x5735ea8a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf95ea8e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa635ea91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c95ea93, SUC_lexicon_tags_14 },
    { 0x76f5ea93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe015ea99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c75ea9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9715eaa0, SUC_lexicon_tags_9 },
    { 0x29d5eaa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a35eaa4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7415eaa7, SUC_lexicon_tags_15 },
    { 0xec95eaa8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde55eaaa, SUC_lexicon_tags_17 },
    { 0x9075eaaa, SUC_lexicon_tags_28 },
    { 0xee35eaac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xee55eaaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e35eab2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82f5eab4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d5eaba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b35eabf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b5eac4, SUC_lexicon_tags_11 },
    { 0x4d35eac4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4655eac7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d5eaca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcaf5eacc, SUC_lexicon_tags_11 },
    { 0xfbd5eacd, SUC_lexicon_tags_57 },
    { 0xaa15eace, SUC_lexicon_tags_30 },
    { 0xcf5eacd, SUC_lexicon_tags_14 },
    { 0x4bd5eace, SUC_lexicon_tags_23 },
    { 0xbf5ead1, SUC_lexicon_tags_47 },
    { 0x8415ead2, SUC_lexicon_tags_47 },
    { 0x20d5ead3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfd5ead5, SUC_lexicon_tags_47 },
    { 0x95d5ead5, SUC_lexicon_tags_47 },
    { 0x4d95ead6, SUC_lexicon_tags_37 },
    { 0xb155ead8, SUC_lexicon_tags_57 },
    { 0x4db5ead9, SUC_lexicon_tags_27 },
    { 0x6875ead9, SUC_lexicon_tags_46 },
    { 0xefb5eadb, SUC_lexicon_tags_0 },
    { 0x8895ead5, SUC_lexicon_tags_25 },
    { 0xa3b5eadd, SUC_lexicon_tags_48 },
    { 0xc3f5eade, SUC_lexicon_tags_12 },
    { 0xcd55eadf, SUC_lexicon_tags_27 },
    { 0xbeb5eadc, SUC_lexicon_tags_1368 },
    { 0xc95eae1, SUC_lexicon_tags_17 },
    { 0x5d95eae2, SUC_lexicon_tags_28 },
    { 0xb435eae1, SUC_lexicon_tags_23 },
    { 0x6075eae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd5eae8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16d5eaea, SUC_lexicon_tags_38 },
    { 0x4b35eaeb, SUC_lexicon_tags_9 },
    { 0xb15eaec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c55eaef, SUC_lexicon_tags_62 },
    { 0xbf5eaf0, SUC_lexicon_tags_17 },
    { 0x8d15eaef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa075eaf4, SUC_lexicon_tags_0 },
    { 0xae35eaf4, SUC_lexicon_tags_2 },
    { 0x235eaf6, SUC_lexicon_tags_113 },
    { 0x37d5eaf7, SUC_lexicon_tags_37 },
    { 0x2eb5eaf6, SUC_lexicon_tags_60 },
    { 0x75b5eaf9, SUC_lexicon_tags_15 },
    { 0xa795eaf6, SUC_lexicon_tags_5 },
    { 0x4455eaf9, SUC_lexicon_tags_1 },
    { 0xc595eafc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c95eb00, SUC_lexicon_tags_19 },
    { 0x62b5eb01, SUC_lexicon_tags_29 },
    { 0x6295eb01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf175eb08, SUC_lexicon_tags_68 },
    { 0x7ed5eb09, SUC_lexicon_tags_37 },
    { 0x3d75eb09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5835eb0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8235eb10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98d5eb12, SUC_lexicon_tags_7 },
    { 0x18d5eb13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2375eb19, SUC_lexicon_tags_11 },
    { 0x1d75eb1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1b5eb1c, SUC_lexicon_tags_11 },
    { 0xa515eb1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6335eb23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d55eb26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7035eb28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d35eb2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115eb33, SUC_lexicon_tags_35 },
    { 0xebf5eb34, SUC_lexicon_tags_29 },
    { 0x1015eb35, SUC_lexicon_tags_19 },
    { 0x3735eb33, SUC_lexicon_tags_63 },
    { 0x7835eb37, SUC_lexicon_tags_1 },
    { 0x1355eb38, SUC_lexicon_tags_12 },
    { 0xb375eb39, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2295eb3b, SUC_lexicon_tags_29 },
    { 0x2b35eb3c, SUC_lexicon_tags_47 },
    { 0x4a55eb3c, SUC_lexicon_tags_1 },
    { 0x9e15eb3e, SUC_lexicon_tags_103 },
    { 0x5f75eb3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc695eb47, SUC_lexicon_tags_1 },
    { 0xf255eb47, SUC_lexicon_tags_1 },
    { 0x7495eb48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe275eb4c, SUC_lexicon_tags_63 },
    { 0x3f75eb4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x90f5eb4f, SUC_lexicon_tags_29 },
    { 0x3cf5eb50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b55eb52, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2475eb54, SUC_lexicon_tags_27 },
    { 0xb675eb55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3255eb57, SUC_lexicon_tags_63 },
    { 0x3175eb58, SUC_lexicon_tags_57 },
    { 0xa2f5eb59, SUC_lexicon_tags_25 },
    { 0x9855eb5a, SUC_lexicon_tags_25 },
    { 0xd835eb5b, SUC_lexicon_tags_6 },
    { 0x9595eb58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd5eb5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe995eb63, SUC_lexicon_tags_17 },
    { 0x54d5eb64, SUC_lexicon_tags_17 },
    { 0x2295eb65, SUC_lexicon_tags_11 },
    { 0xa3d5eb64, SUC_lexicon_tags_11 },
    { 0x99d5eb64, SUC_lexicon_tags_3 },
    { 0x5935eb68, SUC_lexicon_tags_17 },
    { 0x7f95eb64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5835eb71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9055eb73, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8c15eb75, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc35eb7b, SUC_lexicon_tags_96 },
    { 0xbdd5eb7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd5eb7e, SUC_lexicon_tags_25 },
    { 0x47d5eb7f, SUC_lexicon_tags_26 },
    { 0xd095eb80, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab5eb88, SUC_lexicon_tags_1 },
    { 0xbd55eb89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b5eb8e, SUC_lexicon_tags_7 },
    { 0x89d5eb8f, SUC_lexicon_tags_15 },
    { 0x4255eb90, SUC_lexicon_tags_2 },
    { 0x58d5eb91, SUC_lexicon_tags_33 },
    { 0x6d5eb91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6575eb94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd5eb97, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3075eb9a, SUC_lexicon_tags_57 },
    { 0x2395eb9b, SUC_lexicon_tags_27 },
    { 0xb3f5eb9b, SUC_lexicon_tags_26 },
    { 0xcbb5eb9b, SUC_lexicon_tags_17 },
    { 0xe1b5eb9b, SUC_lexicon_tags_47 },
    { 0x55d5eb9d, SUC_lexicon_tags_45 },
    { 0x5ff5eba0, SUC_lexicon_tags_16 },
    { 0x6c75eba1, SUC_lexicon_tags_1 },
    { 0x1f5eb9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba15eba6, SUC_lexicon_tags_43 },
    { 0xce15eba7, SUC_lexicon_tags_17 },
    { 0xbb55eba8, SUC_lexicon_tags_12 },
    { 0xc9f5eba9, SUC_lexicon_tags_27 },
    { 0xc775eba9, SUC_lexicon_tags_33 },
    { 0xe315ebab, SUC_lexicon_tags_15 },
    { 0xea15ebac, SUC_lexicon_tags_47 },
    { 0xa615eba8, SUC_lexicon_tags_17 },
    { 0xf855ebae, SUC_lexicon_tags_30 },
    { 0xc3b5eba8, SUC_lexicon_tags_63 },
    { 0xb255ebae, SUC_lexicon_tags_11 },
    { 0x9255eba6, SUC_lexicon_tags_47 },
    { 0x7f95ebb2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7075ebb6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2c35ebb8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4435ebbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe595ebbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb5ebc4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5f75ebc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2035ebc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd835ebd0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfb95ebd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435ebd6, SUC_lexicon_tags_12 },
    { 0x9af5ebd7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e95ebda, SUC_lexicon_tags_4 },
    { 0xecf5ebda, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd5ebde, SUC_lexicon_tags_17 },
    { 0xfc95ebdf, SUC_lexicon_tags_12 },
    { 0x78b5ebe0, SUC_lexicon_tags_26 },
    { 0xf075ebe0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5ebe3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28f5ebe5, SUC_lexicon_tags_31 },
    { 0x5995ebe6, SUC_lexicon_tags_5 },
    { 0x3d75ebe6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdf15ebe9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa4d5ebeb, SUC_lexicon_tags_17 },
    { 0xf555ebec, SUC_lexicon_tags_14 },
    { 0xc815ebed, SUC_lexicon_tags_17 },
    { 0xec75ebec, SUC_lexicon_tags_25 },
    { 0x7475ebeb, SUC_lexicon_tags_48 },
    { 0x5615ebf0, SUC_lexicon_tags_25 },
    { 0xadd5ebf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4695ebf9, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead5ebfe, SUC_lexicon_tags_154 },
    { 0xc935ebff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8415ec04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f5ec08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1795ec13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab5ec15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe75ec19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e55ec1b, SUC_lexicon_tags_12 },
    { 0x8815ec1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd535ec22, SUC_lexicon_tags_31 },
    { 0xc595ec22, SUC_lexicon_tags_17 },
    { 0xfdd5ec23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ec2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d5ec32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d5ec36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9735ec3a, SUC_lexicon_tags_12 },
    { 0xe535ec3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b5ec3f, SUC_lexicon_tags_48 },
    { 0x5d5ec3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd9d5ec42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7d55ec44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed5ec4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea75ec4e, SUC_lexicon_tags_27 },
    { 0x6495ec4f, SUC_lexicon_tags_39 },
    { 0x595ec50, SUC_lexicon_tags_14 },
    { 0xf3b5ec51, SUC_lexicon_tags_63 },
    { 0x16b5ec52, SUC_lexicon_tags_1 },
    { 0xad15ec52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c35ec5b, SUC_lexicon_tags_38 },
    { 0x2e15ec5c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe35ec5f, SUC_lexicon_tags_37 },
    { 0xfb5ec5f, SUC_lexicon_tags_27 },
    { 0x715ec60, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5ec64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba95ec66, SUC_lexicon_tags_12 },
    { 0x2d55ec67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2655ec6c, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a15ec70, SUC_lexicon_tags_63 },
    { 0xd115ec71, SUC_lexicon_tags_8 },
    { 0x26d5ec71, SUC_lexicon_tags_27 },
    { 0x26d5ec71, SUC_lexicon_tags_12 },
    { 0xfe95ec71, SUC_lexicon_tags_19 },
    { 0x2bd5ec70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9555ec77, SUC_lexicon_tags_47 },
    { 0x5e75ec78, SUC_lexicon_tags_12 },
    { 0xbff5ec78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c15ec7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d75ec83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xba35ec86, SUC_lexicon_tags_7 },
    { 0x7b15ec87, SUC_lexicon_tags_48 },
    { 0x3095ec88, SUC_lexicon_tags_14 },
    { 0xcc35ec89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b5ec8f, SUC_lexicon_tags_15 },
    { 0xf3d5ec90, SUC_lexicon_tags_43 },
    { 0xe875ec91, SUC_lexicon_tags_25 },
    { 0x4ed5ec90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd95ec94, SUC_lexicon_tags_25 },
    { 0x4c35ec94, SUC_lexicon_tags_45 },
    { 0xeb95ec96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb5ec99, SUC_lexicon_tags_25 },
    { 0x2255ec9a, SUC_lexicon_tags_15 },
    { 0x6f55ec9b, SUC_lexicon_tags_6 },
    { 0xd015ec9a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b5eca5, SUC_lexicon_tags_22 },
    { 0xe375eca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6775eca8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4655ecae, SUC_lexicon_tags_0 },
    { 0x3f55ecaf, SUC_lexicon_tags_30 },
    { 0x4b55ecae, SUC_lexicon_tags_31 },
    { 0xb7b5ecb1, SUC_lexicon_tags_1 },
    { 0xc1f5ecb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c95ecb6, SUC_lexicon_tags_60 },
    { 0x3775ecb6, SUC_lexicon_tags_17 },
    { 0xdb35ecb7, SUC_lexicon_tags_11 },
    { 0x5ff5ecb8, SUC_lexicon_tags_57 },
    { 0x1bd5ecb7, SUC_lexicon_tags_1 },
    { 0x71d5ecb7, SUC_lexicon_tags_11 },
    { 0x6a95ecbc, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d5ecc0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6555ecc2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f5ecc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa2b5ecc9, SUC_lexicon_tags_60 },
    { 0x7355ecca, SUC_lexicon_tags_8 },
    { 0xad15ecc9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f5ecce, SUC_lexicon_tags_114 },
    { 0xa455eccf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef55ecd1, SUC_lexicon_tags_45 },
    { 0x3695ecd2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e75ecd4, SUC_lexicon_tags_37 },
    { 0x6c35ecd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcd5ecd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5b5ecd9, SUC_lexicon_tags_66 },
    { 0x79b5ecd9, SUC_lexicon_tags_68 },
    { 0xa195ecdb, SUC_lexicon_tags_11 },
    { 0x6935ecdc, SUC_lexicon_tags_1 },
    { 0x5435ecdd, SUC_lexicon_tags_33 },
    { 0x96b5ecda, SUC_lexicon_tags_12 },
    { 0xad5ecda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b5ece3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f75ece5, SUC_lexicon_tags_11 },
    { 0x4575ece5, SUC_lexicon_tags_16 },
    { 0x8355ece6, SUC_lexicon_tags_23 },
    { 0xa575ece8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d5eced, SUC_lexicon_tags_8 },
    { 0xa655ecee, SUC_lexicon_tags_25 },
    { 0x1475ecee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45f5ecf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x715ecf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4075ecfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5595ecfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fb5ecff, SUC_lexicon_tags_82 },
    { 0x10d5ecff, SUC_lexicon_tags_37 },
    { 0x7db5ed01, SUC_lexicon_tags_6 },
    { 0x7f5ed02, SUC_lexicon_tags_23 },
    { 0x12d5ecff, SUC_lexicon_tags_15 },
    { 0x6415ed04, SUC_lexicon_tags_12 },
    { 0x35d5ecff, SUC_lexicon_tags_34 },
    { 0x5a55ed06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5ed09, SUC_lexicon_tags_5 },
    { 0xed35ed09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f5ed14, SUC_lexicon_tags_15 },
    { 0x84f5ed15, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7b75ed17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b55ed1c, SUC_lexicon_tags_9 },
    { 0x5035ed1c, SUC_lexicon_tags_12 },
    { 0xb075ed1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc375ed20, SUC_lexicon_tags_8 },
    { 0xa675ed20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d5ed2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f5ed34, SUC_lexicon_tags_11 },
    { 0xa135ed35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4f5ed37, SUC_lexicon_tags_15 },
    { 0x51d5ed37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa615ed3a, SUC_lexicon_tags_37 },
    { 0x7c35ed3a, SUC_lexicon_tags_27 },
    { 0x9975ed3b, SUC_lexicon_tags_3 },
    { 0x30d5ed3d, SUC_lexicon_tags_63 },
    { 0x9595ed3c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d5ed43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7b5ed45, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9675ed4a, SUC_lexicon_tags_605 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf5ed4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7075ed51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf675ed56, SUC_lexicon_tags_35 },
    { 0xe655ed57, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f5ed5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb5ed64, SUC_lexicon_tags_15 },
    { 0x8cf5ed64, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375ed6b, SUC_lexicon_tags_17 },
    { 0x47d5ed6b, SUC_lexicon_tags_6 },
    { 0x11f5ed6c, SUC_lexicon_tags_37 },
    { 0x10d5ed6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd5ed73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1b5ed75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b5ed78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce75ed7e, SUC_lexicon_tags_48 },
    { 0x3315ed7f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d5ed84, SUC_lexicon_tags_0 },
    { 0xb8f5ed85, SUC_lexicon_tags_14 },
    { 0xa0d5ed86, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4015ed8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd635ed96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4895ed98, SUC_lexicon_tags_184 },
    { 0xd635ed99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x995ed9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd95eda0, SUC_lexicon_tags_34 },
    { 0xdad5eda1, SUC_lexicon_tags_2 },
    { 0x2a55eda2, SUC_lexicon_tags_8 },
    { 0x6255eda3, SUC_lexicon_tags_37 },
    { 0xb1b5eda3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f5eda9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c55edab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b5edaf, SUC_lexicon_tags_38 },
    { 0x1e75edaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2375edb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8855edb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe675edb8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2f95edba, SUC_lexicon_tags_1 },
    { 0xd6d5edbb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9515edbd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa15edc3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9075edca, SUC_lexicon_tags_45 },
    { 0x9ad5edcb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x81d5edcd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d5edd0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f5edd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1915edd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5af5edd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b35edda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6bb5eddc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455ede1, SUC_lexicon_tags_17 },
    { 0x9135ede1, SUC_lexicon_tags_15 },
    { 0x2ff5ede2, SUC_lexicon_tags_28 },
    { 0xfcb5ede4, SUC_lexicon_tags_30 },
    { 0x4395ede4, SUC_lexicon_tags_462 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b5ede9, SUC_lexicon_tags_35 },
    { 0x1e75ede9, SUC_lexicon_tags_7 },
    { 0x4f75edeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7195eded, SUC_lexicon_tags_45 },
    { 0xa835eded, SUC_lexicon_tags_11 },
    { 0x8c35edef, SUC_lexicon_tags_42 },
    { 0x6e75edf0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf15edf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5edf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f5edfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad5ee01, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd275ee03, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8415ee08, SUC_lexicon_tags_39 },
    { 0x4c15ee09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3135ee0f, SUC_lexicon_tags_17 },
    { 0x5655ee0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8055ee16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb915ee19, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xd5b5ee1b, SUC_lexicon_tags_7 },
    { 0xf515ee1b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x3435ee1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f35ee21, SUC_lexicon_tags_37 },
    { 0x5d5ee22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f15ee28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x92d5ee2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd5ee2e, SUC_lexicon_tags_19 },
    { 0x9835ee2e, SUC_lexicon_tags_17 },
    { 0x30f5ee30, SUC_lexicon_tags_11 },
    { 0x8b75ee31, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7355ee35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e35ee38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd15ee3d, SUC_lexicon_tags_14 },
    { 0x7f35ee3e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a15ee42, SUC_lexicon_tags_25 },
    { 0xb975ee42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8875ee46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a75ee48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf415ee4c, SUC_lexicon_tags_12 },
    { 0x1d5ee4c, SUC_lexicon_tags_23 },
    { 0x8e95ee4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d5ee52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d35ee57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc7f5ee59, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca55ee5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f5ee60, SUC_lexicon_tags_11 },
    { 0xebd5ee61, SUC_lexicon_tags_25 },
    { 0xd215ee62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4615ee67, SUC_lexicon_tags_7 },
    { 0x7715ee68, SUC_lexicon_tags_31 },
    { 0xd235ee68, SUC_lexicon_tags_11 },
    { 0x1795ee68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc95ee6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5ee71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde75ee74, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7435ee78, SUC_lexicon_tags_12 },
    { 0x6535ee79, SUC_lexicon_tags_13 },
    { 0x7615ee78, SUC_lexicon_tags_45 },
    { 0x5835ee7b, SUC_lexicon_tags_7 },
    { 0xac75ee7c, SUC_lexicon_tags_29 },
    { 0xf275ee7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c15ee81, SUC_lexicon_tags_37 },
    { 0x2735ee81, SUC_lexicon_tags_13 },
    { 0x1715ee83, SUC_lexicon_tags_15 },
    { 0x75ee84, SUC_lexicon_tags_12 },
    { 0x8c75ee85, SUC_lexicon_tags_19 },
    { 0xaf75ee85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c55ee89, SUC_lexicon_tags_60 },
    { 0x9a15ee89, SUC_lexicon_tags_45 },
    { 0xccf5ee89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1875ee90, SUC_lexicon_tags_7 },
    { 0xf435ee90, SUC_lexicon_tags_15 },
    { 0xa95ee92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c35ee94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe95ee97, SUC_lexicon_tags_47 },
    { 0xa235ee97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11d5ee9a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8e75ee9c, SUC_lexicon_tags_7 },
    { 0x7b75ee9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f5eea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c55eea5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d35eeab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5d15eead, SUC_lexicon_tags_39 },
    { 0x1d35eeae, SUC_lexicon_tags_7 },
    { 0x7635eeaf, SUC_lexicon_tags_17 },
    { 0x25b5eeb0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6295eeb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b5eeb7, SUC_lexicon_tags_8 },
    { 0x56f5eeb8, SUC_lexicon_tags_3 },
    { 0xef5eeb8, SUC_lexicon_tags_1 },
    { 0x75eeb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd5eebd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd5eec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d5eec5, SUC_lexicon_tags_12 },
    { 0xfaf5eec5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb5eeca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7435eecd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff5eed0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3315eed4, SUC_lexicon_tags_29 },
    { 0xc7f5eed5, SUC_lexicon_tags_30 },
    { 0xfd15eed6, SUC_lexicon_tags_47 },
    { 0xeef5eed6, SUC_lexicon_tags_9 },
    { 0x2635eed8, SUC_lexicon_tags_37 },
    { 0xbe75eed9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e35eedb, SUC_lexicon_tags_49 },
    { 0x5ad5eedb, SUC_lexicon_tags_68 },
    { 0xa495eedb, SUC_lexicon_tags_70 },
    { 0xbd75eede, SUC_lexicon_tags_1 },
    { 0x92d5eedf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f5eee6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b15eee9, SUC_lexicon_tags_11 },
    { 0xaf15eeea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc95eeee, SUC_lexicon_tags_8 },
    { 0xc255eeee, SUC_lexicon_tags_26 },
    { 0x8675eeee, SUC_lexicon_tags_11 },
    { 0xf055eef1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd1b5eef3, SUC_lexicon_tags_2 },
    { 0x7275eef4, SUC_lexicon_tags_87 },
    { 0xcc55eef5, SUC_lexicon_tags_14 },
    { 0xdc75eef5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8695eef9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8815eeff, SUC_lexicon_tags_8 },
    { 0x4d5ef00, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc7f5ef02, SUC_lexicon_tags_49 },
    { 0x1335ef03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f5ef08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19d5ef0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab75ef12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcab5ef14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f75ef1b, SUC_lexicon_tags_45 },
    { 0x6c95ef1c, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea95ef23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c95ef26, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b5ef2a, SUC_lexicon_tags_25 },
    { 0x6c95ef2a, SUC_lexicon_tags_9 },
    { 0x4455ef2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef55ef2e, SUC_lexicon_tags_23 },
    { 0xf9b5ef2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7695ef32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5615ef34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d5ef37, SUC_lexicon_tags_11 },
    { 0x9bd5ef38, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x77b5ef3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a35ef3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd495ef3f, SUC_lexicon_tags_25 },
    { 0x6f55ef40, SUC_lexicon_tags_7 },
    { 0x9995ef41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75ef4b, SUC_lexicon_tags_15 },
    { 0xa1f5ef4b, SUC_lexicon_tags_38 },
    { 0x6f95ef4d, SUC_lexicon_tags_28 },
    { 0xdb75ef4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2675ef50, SUC_lexicon_tags_47 },
    { 0x2995ef50, SUC_lexicon_tags_7 },
    { 0xb0f5ef52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2915ef59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6315ef5d, SUC_lexicon_tags_47 },
    { 0x23b5ef5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7475ef60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3e55ef62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d5ef65, SUC_lexicon_tags_60 },
    { 0xbe75ef65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd5ef6a, SUC_lexicon_tags_23 },
    { 0x9c35ef6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd35ef72, SUC_lexicon_tags_17 },
    { 0xe975ef73, SUC_lexicon_tags_10 },
    { 0x4175ef74, SUC_lexicon_tags_12 },
    { 0xccf5ef75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea15ef78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf035ef7a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6235ef7d, SUC_lexicon_tags_14 },
    { 0x7975ef7e, SUC_lexicon_tags_46 },
    { 0x64d5ef7f, SUC_lexicon_tags_234 },
    { 0x1c55ef80, SUC_lexicon_tags_7 },
    { 0x7055ef7e, SUC_lexicon_tags_8 },
    { 0xd7f5ef82, SUC_lexicon_tags_23 },
    { 0x8b95ef83, SUC_lexicon_tags_47 },
    { 0x4875ef84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8555ef86, SUC_lexicon_tags_60 },
    { 0x40d5ef87, SUC_lexicon_tags_324 },
    { 0x51f5ef88, SUC_lexicon_tags_7 },
    { 0x95b5ef89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f5ef8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9ef5ef91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5555ef93, SUC_lexicon_tags_17 },
    { 0x27d5ef94, SUC_lexicon_tags_37 },
    { 0x6bf5ef93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d5ef97, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4575ef99, SUC_lexicon_tags_12 },
    { 0xd355ef9a, SUC_lexicon_tags_14 },
    { 0x16f5ef99, SUC_lexicon_tags_4 },
    { 0xd495ef9c, SUC_lexicon_tags_25 },
    { 0x4f55ef9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95efa1, SUC_lexicon_tags_37 },
    { 0x4315efa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35f5efa4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2955efad, SUC_lexicon_tags_95 },
    { 0x3cd5efad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb675efb1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55efb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb95efbc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8595efbe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2055efc2, SUC_lexicon_tags_84 },
    { 0x7ad5efc3, SUC_lexicon_tags_11 },
    { 0xe335efc2, SUC_lexicon_tags_7 },
    { 0x7055efc5, SUC_lexicon_tags_4 },
    { 0x2d95efc6, SUC_lexicon_tags_19 },
    { 0x48f5efc3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3535efca, SUC_lexicon_tags_45 },
    { 0x98b5efcb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf15efce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe5d5efd0, SUC_lexicon_tags_7 },
    { 0x90d5efd0, SUC_lexicon_tags_1 },
    { 0x1f55efd0, SUC_lexicon_tags_68 },
    { 0x3a15efd1, SUC_lexicon_tags_17 },
    { 0x695efd2, SUC_lexicon_tags_7 },
    { 0xccd5efd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5215efde, SUC_lexicon_tags_47 },
    { 0xe75efdf, SUC_lexicon_tags_43 },
    { 0xae15efe0, SUC_lexicon_tags_33 },
    { 0x6395efe0, SUC_lexicon_tags_33 },
    { 0x7e95efe0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf75efeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9915efed, SUC_lexicon_tags_33 },
    { 0x9cf5efee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79d5eff0, SUC_lexicon_tags_19 },
    { 0xcf5eff1, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5eff8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb35effb, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb5effe, SUC_lexicon_tags_25 },
    { 0x3455efff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715f003, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8df5f005, SUC_lexicon_tags_19 },
    { 0x95f5f005, SUC_lexicon_tags_25 },
    { 0xbd75f007, SUC_lexicon_tags_31 },
    { 0x2835f007, SUC_lexicon_tags_11 },
    { 0x1bf5f007, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe695f00c, SUC_lexicon_tags_37 },
    { 0xa135f00d, SUC_lexicon_tags_12 },
    { 0xdf55f00d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d5f011, SUC_lexicon_tags_0 },
    { 0x3e75f012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc475f016, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc015f019, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1515f01f, SUC_lexicon_tags_1 },
    { 0xb795f01f, SUC_lexicon_tags_12 },
    { 0xb0d5f021, SUC_lexicon_tags_31 },
    { 0x8075f022, SUC_lexicon_tags_11 },
    { 0x7b95f01f, SUC_lexicon_tags_12 },
    { 0xd875f024, SUC_lexicon_tags_7 },
    { 0x7095f022, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd835f02d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3b5f02f, SUC_lexicon_tags_19 },
    { 0xb755f02f, SUC_lexicon_tags_29 },
    { 0x6615f030, SUC_lexicon_tags_8 },
    { 0x7c35f032, SUC_lexicon_tags_25 },
    { 0x6bf5f033, SUC_lexicon_tags_85 },
    { 0x9875f034, SUC_lexicon_tags_2 },
    { 0x3bf5f032, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a35f03b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b75f03d, SUC_lexicon_tags_0 },
    { 0x7595f03d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4455f043, SUC_lexicon_tags_11 },
    { 0x1e95f044, SUC_lexicon_tags_12 },
    { 0x6d15f043, SUC_lexicon_tags_37 },
    { 0x6cf5f045, SUC_lexicon_tags_1 },
    { 0x9bd5f047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b5f04b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb95f04f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2335f054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x135f056, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d5f05a, SUC_lexicon_tags_9 },
    { 0x6475f05b, SUC_lexicon_tags_37 },
    { 0xad75f05c, SUC_lexicon_tags_19 },
    { 0xd415f05d, SUC_lexicon_tags_17 },
    { 0x3155f05c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xf475f061, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x575f064, SUC_lexicon_tags_12 },
    { 0xfc95f065, SUC_lexicon_tags_4 },
    { 0x7715f066, SUC_lexicon_tags_12 },
    { 0xd955f066, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a55f06b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d5f06e, SUC_lexicon_tags_2 },
    { 0x9255f06f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad5f072, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8575f07a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2855f07f, SUC_lexicon_tags_14 },
    { 0x7715f080, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa55f086, SUC_lexicon_tags_47 },
    { 0x2bb5f087, SUC_lexicon_tags_60 },
    { 0xbc55f088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f95f08b, SUC_lexicon_tags_12 },
    { 0xff55f08c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde15f090, SUC_lexicon_tags_37 },
    { 0x3d55f091, SUC_lexicon_tags_6 },
    { 0x9a75f091, SUC_lexicon_tags_7 },
    { 0x9295f093, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf5f09a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdbd5f09c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed5f0a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8cb5f0a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb35f0a5, SUC_lexicon_tags_17 },
    { 0xdff5f0a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc415f0ad, SUC_lexicon_tags_27 },
    { 0x9d75f0ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf975f0b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7795f0b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc155f0b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f35f0ba, SUC_lexicon_tags_29 },
    { 0x9a15f0bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d5f0be, SUC_lexicon_tags_7 },
    { 0x9a75f0bf, SUC_lexicon_tags_27 },
    { 0xea75f0bf, SUC_lexicon_tags_11 },
    { 0xd715f0be, SUC_lexicon_tags_23 },
    { 0x9355f0bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe395f0c4, SUC_lexicon_tags_45 },
    { 0xce95f0c4, SUC_lexicon_tags_34 },
    { 0x2875f0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd5f0ca, SUC_lexicon_tags_27 },
    { 0xfc35f0cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf455f0d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb15f0d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7a55f0d7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x62d5f0d9, SUC_lexicon_tags_43 },
    { 0xd7d5f0da, SUC_lexicon_tags_65 },
    { 0x13d5f0da, SUC_lexicon_tags_14 },
    { 0x4055f0dc, SUC_lexicon_tags_23 },
    { 0x7ed5f0dd, SUC_lexicon_tags_29 },
    { 0x1435f0dc, SUC_lexicon_tags_37 },
    { 0xfa75f0df, SUC_lexicon_tags_55 },
    { 0xcb95f0df, SUC_lexicon_tags_37 },
    { 0x3515f0e0, SUC_lexicon_tags_2 },
    { 0x5995f0df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e95f0e5, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b5f0eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b75f0ee, SUC_lexicon_tags_47 },
    { 0x6935f0ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6955f0f2, SUC_lexicon_tags_37 },
    { 0x3555f0f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd895f0f5, SUC_lexicon_tags_1 },
    { 0x6415f0f6, SUC_lexicon_tags_32 },
    { 0xec95f0f6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9315f0f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa995f0fc, SUC_lexicon_tags_7 },
    { 0x6695f0fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd35f104, SUC_lexicon_tags_17 },
    { 0xa495f105, SUC_lexicon_tags_15 },
    { 0x8695f105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa775f108, SUC_lexicon_tags_14 },
    { 0xbbd5f109, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f75f10f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f5f113, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf835f118, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a95f11b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3375f11f, SUC_lexicon_tags_0 },
    { 0x5cd5f120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc915f124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df5f128, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc795f12a, SUC_lexicon_tags_7 },
    { 0x79f5f12a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4975f12f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e75f131, SUC_lexicon_tags_16 },
    { 0xad95f132, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf775f134, SUC_lexicon_tags_11 },
    { 0x54b5f134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c75f137, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7515f13a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb5f13d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f144, SUC_lexicon_tags_15 },
    { 0x4755f145, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x915f147, SUC_lexicon_tags_15 },
    { 0x3b5f147, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb5f14c, SUC_lexicon_tags_25 },
    { 0xf6b5f14d, SUC_lexicon_tags_17 },
    { 0x5ff5f14d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1435f151, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3975f154, SUC_lexicon_tags_11 },
    { 0x6355f155, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1e55f157, SUC_lexicon_tags_39 },
    { 0x7515f158, SUC_lexicon_tags_26 },
    { 0xd4d5f159, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9355f15b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbeb5f15d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3815f164, SUC_lexicon_tags_11 },
    { 0x4055f165, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b5f16b, SUC_lexicon_tags_66 },
    { 0x3555f16c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7255f16e, SUC_lexicon_tags_37 },
    { 0x48f5f16f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f5f17a, SUC_lexicon_tags_12 },
    { 0x5875f17b, SUC_lexicon_tags_13 },
    { 0x3b55f17b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee75f180, SUC_lexicon_tags_2 },
    { 0xd4d5f180, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6cb5f183, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f5f186, SUC_lexicon_tags_87 },
    { 0x7255f186, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd55f18b, SUC_lexicon_tags_25 },
    { 0xc615f18b, SUC_lexicon_tags_47 },
    { 0x9115f18d, SUC_lexicon_tags_12 },
    { 0xed5f18d, SUC_lexicon_tags_11 },
    { 0x63f5f18b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb5f193, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1675f195, SUC_lexicon_tags_11 },
    { 0x5535f196, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae35f1a3, SUC_lexicon_tags_17 },
    { 0x7fd5f1a3, SUC_lexicon_tags_35 },
    { 0x30b5f1a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a35f1ae, SUC_lexicon_tags_43 },
    { 0xf2d5f1ae, SUC_lexicon_tags_11 },
    { 0xbb5f1b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d5f1b6, SUC_lexicon_tags_37 },
    { 0xdd55f1b7, SUC_lexicon_tags_19 },
    { 0x9b15f1b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f5f1bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5f1be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d15f1c0, SUC_lexicon_tags_85 },
    { 0x4c35f1c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf315f1ca, SUC_lexicon_tags_17 },
    { 0xf035f1cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8655f1cd, SUC_lexicon_tags_37 },
    { 0xef35f1ce, SUC_lexicon_tags_82 },
    { 0x4955f1cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9c95f1d1, SUC_lexicon_tags_6 },
    { 0xae35f1d1, SUC_lexicon_tags_11 },
    { 0x6975f1d3, SUC_lexicon_tags_13 },
    { 0xa9b5f1d4, SUC_lexicon_tags_38 },
    { 0xe235f1d5, SUC_lexicon_tags_2 },
    { 0x2515f1d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c35f1dc, SUC_lexicon_tags_7 },
    { 0x8755f1dd, SUC_lexicon_tags_4 },
    { 0xe495f1de, SUC_lexicon_tags_17 },
    { 0x7715f1dd, SUC_lexicon_tags_15 },
    { 0xde75f1de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5f1e4, SUC_lexicon_tags_25 },
    { 0x1635f1e5, SUC_lexicon_tags_37 },
    { 0x71f5f1e6, SUC_lexicon_tags_45 },
    { 0x90b5f1e6, SUC_lexicon_tags_21 },
    { 0x3695f1e8, SUC_lexicon_tags_38 },
    { 0x4895f1e9, SUC_lexicon_tags_1 },
    { 0x8b35f1e4, SUC_lexicon_tags_28 },
    { 0x63b5f1eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x47d5f1ed, SUC_lexicon_tags_23 },
    { 0xb9b5f1ee, SUC_lexicon_tags_35 },
    { 0x9995f1ed, SUC_lexicon_tags_63 },
    { 0x10b5f1ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc495f1f5, SUC_lexicon_tags_8 },
    { 0x6ef5f1f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa75f1fa, SUC_lexicon_tags_49 },
    { 0xe455f1fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa675f1fe, SUC_lexicon_tags_1533 },
    { 0x7e95f1ff, SUC_lexicon_tags_19 },
    { 0x6555f1ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x515f203, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95f206, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b5f209, SUC_lexicon_tags_7 },
    { 0x49f5f20a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x57d5f20c, SUC_lexicon_tags_28 },
    { 0x3d75f20d, SUC_lexicon_tags_51 },
    { 0xf135f20c, SUC_lexicon_tags_60 },
    { 0xf4f5f20f, SUC_lexicon_tags_5 },
    { 0xe415f20f, SUC_lexicon_tags_47 },
    { 0x63b5f20f, SUC_lexicon_tags_11 },
    { 0xa655f212, SUC_lexicon_tags_7 },
    { 0x3ef5f212, SUC_lexicon_tags_27 },
    { 0x5535f214, SUC_lexicon_tags_23 },
    { 0xb8f5f213, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f5f218, SUC_lexicon_tags_47 },
    { 0xff5f219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9855f21b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6d5f21d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xda55f21f, SUC_lexicon_tags_37 },
    { 0xe4b5f21f, SUC_lexicon_tags_64 },
    { 0xe835f221, SUC_lexicon_tags_30 },
    { 0x1f15f222, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8d5f224, SUC_lexicon_tags_39 },
    { 0xcad5f224, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15f228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8795f22a, SUC_lexicon_tags_37 },
    { 0xf415f22a, SUC_lexicon_tags_47 },
    { 0x7d95f22c, SUC_lexicon_tags_0 },
    { 0xf1b5f22d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf15f231, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe515f236, SUC_lexicon_tags_12 },
    { 0x7335f236, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf195f239, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e15f23f, SUC_lexicon_tags_2 },
    { 0x9795f23f, SUC_lexicon_tags_66 },
    { 0xd975f240, SUC_lexicon_tags_25 },
    { 0xeb15f242, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f5f245, SUC_lexicon_tags_43 },
    { 0xd555f246, SUC_lexicon_tags_37 },
    { 0x2435f247, SUC_lexicon_tags_25 },
    { 0x1875f248, SUC_lexicon_tags_37 },
    { 0x1755f247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9db5f24b, SUC_lexicon_tags_21 },
    { 0xb4d5f24c, SUC_lexicon_tags_47 },
    { 0x21f5f24c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad5f252, SUC_lexicon_tags_87 },
    { 0x7195f253, SUC_lexicon_tags_23 },
    { 0x8715f254, SUC_lexicon_tags_8 },
    { 0x32b5f255, SUC_lexicon_tags_11 },
    { 0x30f5f256, SUC_lexicon_tags_15 },
    { 0x1535f253, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b5f25c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd75f25e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8075f268, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf5f26d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe635f26f, SUC_lexicon_tags_38 },
    { 0xd655f26f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7655f274, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4155f276, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6435f278, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x84d5f27a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37b5f27c, SUC_lexicon_tags_15 },
    { 0xc9d5f27d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8195f27f, SUC_lexicon_tags_34 },
    { 0x77b5f280, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6d5f282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b55f285, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b5f288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb5f28b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad5f28f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4dd5f291, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a75f294, SUC_lexicon_tags_1 },
    { 0x4815f294, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c55f298, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d5f29a, SUC_lexicon_tags_83 },
    { 0xa3b5f29b, SUC_lexicon_tags_10 },
    { 0x43d5f29b, SUC_lexicon_tags_16 },
    { 0x5f5f29b, SUC_lexicon_tags_17 },
    { 0xd055f29e, SUC_lexicon_tags_17 },
    { 0x3bb5f29a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbb5f2a1, SUC_lexicon_tags_17 },
    { 0xf2b5f2a1, SUC_lexicon_tags_70 },
    { 0xdb95f2a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d5f2a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d5f2ab, SUC_lexicon_tags_29 },
    { 0x1dd5f2ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf495f2af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd315f2b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1f5f2b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8535f2b7, SUC_lexicon_tags_38 },
    { 0x9195f2b8, SUC_lexicon_tags_11 },
    { 0x1115f2b9, SUC_lexicon_tags_17 },
    { 0xbc95f2ba, SUC_lexicon_tags_13 },
    { 0xedb5f2ba, SUC_lexicon_tags_60 },
    { 0xc8b5f2b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf5f2c7, SUC_lexicon_tags_4 },
    { 0xb3d5f2c7, SUC_lexicon_tags_28 },
    { 0xd875f2c9, SUC_lexicon_tags_1 },
    { 0xe895f2c7, SUC_lexicon_tags_6 },
    { 0x735f2cb, SUC_lexicon_tags_17 },
    { 0xac35f2ca, SUC_lexicon_tags_3 },
    { 0xd395f2cd, SUC_lexicon_tags_113 },
    { 0x9c95f2ce, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb475f2d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d5f2d4, SUC_lexicon_tags_9 },
    { 0x7e75f2d4, SUC_lexicon_tags_33 },
    { 0x56d5f2d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3015f2d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb595f2db, SUC_lexicon_tags_37 },
    { 0x7215f2dc, SUC_lexicon_tags_0 },
    { 0x33f5f2dc, SUC_lexicon_tags_11 },
    { 0xc055f2dd, SUC_lexicon_tags_2 },
    { 0xe495f2df, SUC_lexicon_tags_16 },
    { 0xb315f2df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe55f2e2, SUC_lexicon_tags_47 },
    { 0x5c15f2e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f5f2e9, SUC_lexicon_tags_2 },
    { 0x9315f2ea, SUC_lexicon_tags_45 },
    { 0x2e75f2eb, SUC_lexicon_tags_60 },
    { 0xc575f2eb, SUC_lexicon_tags_17 },
    { 0xa555f2ea, SUC_lexicon_tags_10 },
    { 0x3295f2ec, SUC_lexicon_tags_15 },
    { 0xb3d5f2ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a35f2f2, SUC_lexicon_tags_10 },
    { 0xeff5f2f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e95f2f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6275f2f7, SUC_lexicon_tags_0 },
    { 0xfa15f2f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1275f2fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xee35f2fe, SUC_lexicon_tags_11 },
    { 0x6515f2ff, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9215f30b, SUC_lexicon_tags_11 },
    { 0xeab5f30c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3575f30f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d35f311, SUC_lexicon_tags_37 },
    { 0x5735f311, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbeb5f314, SUC_lexicon_tags_19 },
    { 0x4c15f314, SUC_lexicon_tags_37 },
    { 0xebd5f316, SUC_lexicon_tags_0 },
    { 0x1675f317, SUC_lexicon_tags_2 },
    { 0x4715f314, SUC_lexicon_tags_26 },
    { 0x6dd5f316, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3bf5f31b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b5f31f, SUC_lexicon_tags_37 },
    { 0xef75f320, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd835f323, SUC_lexicon_tags_12 },
    { 0xced5f324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a55f326, SUC_lexicon_tags_7 },
    { 0x2655f326, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cd5f329, SUC_lexicon_tags_15 },
    { 0x39d5f32a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38f5f32c, SUC_lexicon_tags_30 },
    { 0x5415f32d, SUC_lexicon_tags_19 },
    { 0x8e75f32e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5d75f330, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2f75f332, SUC_lexicon_tags_6 },
    { 0xa1b5f332, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb755f33e, SUC_lexicon_tags_11 },
    { 0xc915f33f, SUC_lexicon_tags_17 },
    { 0x9935f340, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d5f343, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d5f34b, SUC_lexicon_tags_68 },
    { 0x7f95f34c, SUC_lexicon_tags_17 },
    { 0x8975f34c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f95f354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff5f357, SUC_lexicon_tags_17 },
    { 0xea15f358, SUC_lexicon_tags_25 },
    { 0x6a55f359, SUC_lexicon_tags_37 },
    { 0xd7f5f35a, SUC_lexicon_tags_11 },
    { 0x94f5f35a, SUC_lexicon_tags_34 },
    { 0x70f5f35c, SUC_lexicon_tags_47 },
    { 0xb355f35a, SUC_lexicon_tags_49 },
    { 0x4515f35c, SUC_lexicon_tags_33 },
    { 0xf895f358, SUC_lexicon_tags_25 },
    { 0x3735f35b, SUC_lexicon_tags_7 },
    { 0x8975f361, SUC_lexicon_tags_30 },
    { 0x735f362, SUC_lexicon_tags_8 },
    { 0x1795f362, SUC_lexicon_tags_1 },
    { 0x6375f364, SUC_lexicon_tags_158 },
    { 0x65f5f365, SUC_lexicon_tags_1 },
    { 0x49b5f366, SUC_lexicon_tags_47 },
    { 0x3f75f364, SUC_lexicon_tags_17 },
    { 0x7a55f35b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb5f36b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14f5f36d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbb5f36f, SUC_lexicon_tags_17 },
    { 0xfef5f370, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda75f373, SUC_lexicon_tags_0 },
    { 0xe595f373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5d5f376, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb135f379, SUC_lexicon_tags_7 },
    { 0x6a75f37a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd3b5f37c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35f37f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe555f383, SUC_lexicon_tags_6 },
    { 0xb515f384, SUC_lexicon_tags_10 },
    { 0xcc55f385, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc95f38a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef15f38c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb5f38f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59d5f391, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4035f394, SUC_lexicon_tags_47 },
    { 0xe95f394, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d5f39d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaab5f39f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x52d5f3a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3035f3a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d35f3a7, SUC_lexicon_tags_23 },
    { 0xfff5f3a8, SUC_lexicon_tags_43 },
    { 0xfed5f3a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b5f3af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c15f3ba, SUC_lexicon_tags_17 },
    { 0x2055f3bb, SUC_lexicon_tags_12 },
    { 0xbbf5f3bb, SUC_lexicon_tags_15 },
    { 0x1f95f3ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecd5f3bf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb1b5f3c1, SUC_lexicon_tags_65 },
    { 0x6eb5f3c1, SUC_lexicon_tags_47 },
    { 0xd9f5f3c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed5f3c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6095f3ce, SUC_lexicon_tags_55 },
    { 0xc695f3cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1195f3d3, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e75f3d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc495f3d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf5f3dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79b5f3df, SUC_lexicon_tags_27 },
    { 0x9f35f3e0, SUC_lexicon_tags_44 },
    { 0x1c75f3e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd615f3e4, SUC_lexicon_tags_27 },
    { 0xe8d5f3e4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5795f3e8, SUC_lexicon_tags_32 },
    { 0xcb15f3e9, SUC_lexicon_tags_3 },
    { 0xe0f5f3e9, SUC_lexicon_tags_21 },
    { 0x1515f3e9, SUC_lexicon_tags_19 },
    { 0x6335f3ec, SUC_lexicon_tags_37 },
    { 0xfd55f3ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd675f3f4, SUC_lexicon_tags_17 },
    { 0xf535f3f4, SUC_lexicon_tags_1 },
    { 0xe195f3f6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3435f3fa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8055f3fd, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0x3c75f3ff, SUC_lexicon_tags_25 },
    { 0xf235f400, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x64d5f402, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa275f406, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8715f408, SUC_lexicon_tags_1 },
    { 0xc5d5f409, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc935f40c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e15f410, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff5f414, SUC_lexicon_tags_165 },
    { 0x2795f415, SUC_lexicon_tags_16 },
    { 0xcc35f415, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9815f418, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1515f41a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf5f41d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b5f421, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2935f42b, SUC_lexicon_tags_2 },
    { 0x2915f42c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf5f431, SUC_lexicon_tags_12 },
    { 0x2895f432, SUC_lexicon_tags_27 },
    { 0xbff5f433, SUC_lexicon_tags_25 },
    { 0x83d5f432, SUC_lexicon_tags_33 },
    { 0xddf5f435, SUC_lexicon_tags_11 },
    { 0x5695f436, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5275f438, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8875f43c, SUC_lexicon_tags_23 },
    { 0x9ff5f43c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9975f43f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4c75f441, SUC_lexicon_tags_1 },
    { 0xb1b5f441, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x9175f444, SUC_lexicon_tags_65 },
    { 0xe975f444, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xca55f447, SUC_lexicon_tags_17 },
    { 0xdbf5f447, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175f44e, SUC_lexicon_tags_28 },
    { 0x3855f44f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2435f453, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x875f455, SUC_lexicon_tags_27 },
    { 0x32f5f456, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdb5f458, SUC_lexicon_tags_12 },
    { 0x475f459, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9735f45d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf935f45f, SUC_lexicon_tags_7 },
    { 0x6c55f460, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7935f462, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5f465, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d5f46a, SUC_lexicon_tags_19 },
    { 0x8335f46a, SUC_lexicon_tags_47 },
    { 0xfc15f46a, SUC_lexicon_tags_84 },
    { 0xd6f5f46b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b35f46f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b95f472, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc9b5f474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe695f476, SUC_lexicon_tags_12 },
    { 0xf715f477, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc535f47b, SUC_lexicon_tags_38 },
    { 0xb7b5f47b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c95f480, SUC_lexicon_tags_64 },
    { 0xea95f481, SUC_lexicon_tags_1 },
    { 0x1e15f481, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4255f484, SUC_lexicon_tags_10 },
    { 0x755f485, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3255f488, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f5f48b, SUC_lexicon_tags_43 },
    { 0x2675f48c, SUC_lexicon_tags_17 },
    { 0xdcf5f48c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd575f491, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6295f494, SUC_lexicon_tags_12 },
    { 0x27b5f495, SUC_lexicon_tags_63 },
    { 0x7855f494, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f5f49d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd495f49f, SUC_lexicon_tags_11 },
    { 0x4cf5f4a0, SUC_lexicon_tags_46 },
    { 0xed15f49f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7075f4a3, SUC_lexicon_tags_2 },
    { 0x6135f4a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7855f4a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d5f4a9, SUC_lexicon_tags_0 },
    { 0x4895f4a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6835f4ac, SUC_lexicon_tags_7 },
    { 0x53f5f4ac, SUC_lexicon_tags_1 },
    { 0xf8b5f4ad, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3375f4b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc975f4b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b35f4bc, SUC_lexicon_tags_57 },
    { 0x4b95f4bd, SUC_lexicon_tags_34 },
    { 0x2475f4bc, SUC_lexicon_tags_47 },
    { 0xcbf5f4bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f75f4c3, SUC_lexicon_tags_4 },
    { 0xecb5f4c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d5f4c9, SUC_lexicon_tags_45 },
    { 0xc8b5f4ca, SUC_lexicon_tags_57 },
    { 0x2355f4c9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5255f4cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f55f4cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd5f4d4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d55f4d7, SUC_lexicon_tags_19 },
    { 0xf595f4d7, SUC_lexicon_tags_37 },
    { 0xa755f4d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb535f4db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5115f4e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d5f4e4, SUC_lexicon_tags_12 },
    { 0x9ab5f4e4, SUC_lexicon_tags_40 },
    { 0x99d5f4e6, SUC_lexicon_tags_17 },
    { 0xe9b5f4e7, SUC_lexicon_tags_25 },
    { 0x9575f4e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2635f4ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa535f4f2, SUC_lexicon_tags_7 },
    { 0x4c75f4f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f35f4fb, SUC_lexicon_tags_39 },
    { 0x56f5f4fc, SUC_lexicon_tags_30 },
    { 0x8f5f4fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd535f507, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f5f50f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d5f512, SUC_lexicon_tags_15 },
    { 0x80f5f513, SUC_lexicon_tags_7 },
    { 0x6bd5f514, SUC_lexicon_tags_29 },
    { 0x3c75f513, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe135f517, SUC_lexicon_tags_33 },
    { 0x8af5f517, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f5f51b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x695f51e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c15f521, SUC_lexicon_tags_37 },
    { 0xfa35f521, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf075f529, SUC_lexicon_tags_112 },
    { 0xe1d5f52a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d5f52e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f35f531, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb535f534, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd55f538, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d5f53b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd5f53f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2815f542, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9335f548, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ff5f54a, SUC_lexicon_tags_39 },
    { 0xc35f54b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce15f54e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d5f555, SUC_lexicon_tags_11 },
    { 0x8415f556, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6455f55b, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6435f560, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf5f56a, SUC_lexicon_tags_14 },
    { 0x29f5f56b, SUC_lexicon_tags_25 },
    { 0x8095f56a, SUC_lexicon_tags_15 },
    { 0x4bd5f56d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff15f56f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x575f572, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb95f577, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x8655f579, SUC_lexicon_tags_6 },
    { 0xe5b5f57a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbbd5f57c, SUC_lexicon_tags_3 },
    { 0xf775f57d, SUC_lexicon_tags_37 },
    { 0x4ef5f57d, SUC_lexicon_tags_107 },
    { 0xd195f57f, SUC_lexicon_tags_39 },
    { 0x3df5f57e, SUC_lexicon_tags_5 },
    { 0x2195f581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2355f583, SUC_lexicon_tags_0 },
    { 0xed35f584, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde35f586, SUC_lexicon_tags_11 },
    { 0x7055f587, SUC_lexicon_tags_27 },
    { 0x2d15f588, SUC_lexicon_tags_45 },
    { 0x3b95f589, SUC_lexicon_tags_12 },
    { 0x3fb5f58a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa675f58c, SUC_lexicon_tags_2 },
    { 0xe6b5f58c, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd5f593, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f5f598, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d5f59b, SUC_lexicon_tags_17 },
    { 0xf3f5f59c, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xe1f5f59e, SUC_lexicon_tags_7 },
    { 0xea75f59f, SUC_lexicon_tags_1 },
    { 0xe975f59e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x26b5f5a2, SUC_lexicon_tags_32 },
    { 0x7b35f5a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7415f5a7, SUC_lexicon_tags_10 },
    { 0x7dd5f5a7, SUC_lexicon_tags_37 },
    { 0x6635f5a7, SUC_lexicon_tags_496 },
    { 0x8ff5f5aa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc895f5ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5995f5b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7355f5b5, SUC_lexicon_tags_8 },
    { 0x3d95f5b6, SUC_lexicon_tags_37 },
    { 0xa8b5f5b7, SUC_lexicon_tags_4 },
    { 0x58b5f5b7, SUC_lexicon_tags_7 },
    { 0xdc75f5b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf435f5bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf255f5c2, SUC_lexicon_tags_15 },
    { 0x6975f5c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xda75f5c5, SUC_lexicon_tags_12 },
    { 0x82f5f5c6, SUC_lexicon_tags_25 },
    { 0xf7b5f5c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5655f5c9, SUC_lexicon_tags_16 },
    { 0x155f5ca, SUC_lexicon_tags_26 },
    { 0x19f5f5cb, SUC_lexicon_tags_45 },
    { 0x6835f5c9, SUC_lexicon_tags_8 },
    { 0xd8b5f5cb, SUC_lexicon_tags_27 },
    { 0x8075f5ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc975f5d1, SUC_lexicon_tags_55 },
    { 0x9bd5f5d2, SUC_lexicon_tags_9 },
    { 0xff75f5d2, SUC_lexicon_tags_37 },
    { 0x1715f5d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f5f5dc, SUC_lexicon_tags_15 },
    { 0xdeb5f5dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad5f5e0, SUC_lexicon_tags_64 },
    { 0x1e75f5e1, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x11f5f5e3, SUC_lexicon_tags_17 },
    { 0x90d5f5e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5715f5e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd5f5eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5055f5ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9395f5f5, SUC_lexicon_tags_48 },
    { 0x7db5f5f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7815f5f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9375f5fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7235f601, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b5f604, SUC_lexicon_tags_37 },
    { 0x7135f605, SUC_lexicon_tags_7 },
    { 0xc955f606, SUC_lexicon_tags_12 },
    { 0xce55f604, SUC_lexicon_tags_37 },
    { 0xc2f5f608, SUC_lexicon_tags_69 },
    { 0x3a35f608, SUC_lexicon_tags_19 },
    { 0x2d15f609, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x30d5f60c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f5f612, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f5f615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6555f619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc55f61b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b55f61f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd8b5f621, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbd95f623, SUC_lexicon_tags_15 },
    { 0xc4f5f624, SUC_lexicon_tags_37 },
    { 0xee35f623, SUC_lexicon_tags_19 },
    { 0xe855f623, SUC_lexicon_tags_25 },
    { 0xf415f627, SUC_lexicon_tags_17 },
    { 0x3a75f628, SUC_lexicon_tags_7 },
    { 0x1a35f629, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf195f62d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d5f630, SUC_lexicon_tags_15 },
    { 0x7275f630, SUC_lexicon_tags_8 },
    { 0xd035f630, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa335f636, SUC_lexicon_tags_11 },
    { 0x1fb5f637, SUC_lexicon_tags_12 },
    { 0x86d5f637, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc095f63c, SUC_lexicon_tags_25 },
    { 0xa0f5f63d, SUC_lexicon_tags_11 },
    { 0x4dd5f63d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4255f642, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb735f645, SUC_lexicon_tags_37 },
    { 0xd315f646, SUC_lexicon_tags_26 },
    { 0x65b5f647, SUC_lexicon_tags_23 },
    { 0x8e95f647, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3215f64b, SUC_lexicon_tags_15 },
    { 0x23b5f64b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa915f650, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x455f652, SUC_lexicon_tags_15 },
    { 0x2195f653, SUC_lexicon_tags_47 },
    { 0x82f5f654, SUC_lexicon_tags_55 },
    { 0xa2b5f655, SUC_lexicon_tags_25 },
    { 0x7155f656, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f5f659, SUC_lexicon_tags_26 },
    { 0x8255f65a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd395f65d, SUC_lexicon_tags_49 },
    { 0x5015f65e, SUC_lexicon_tags_11 },
    { 0x88f5f65f, SUC_lexicon_tags_14 },
    { 0x1515f65f, SUC_lexicon_tags_7 },
    { 0x4ed5f661, SUC_lexicon_tags_47 },
    { 0xac35f65f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7215f665, SUC_lexicon_tags_15 },
    { 0xf2f5f666, SUC_lexicon_tags_33 },
    { 0x5075f667, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa315f669, SUC_lexicon_tags_11 },
    { 0xcb75f66a, SUC_lexicon_tags_2 },
    { 0x5975f66a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x93f5f66d, SUC_lexicon_tags_14 },
    { 0xd535f66e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f95f673, SUC_lexicon_tags_11 },
    { 0xe3b5f674, SUC_lexicon_tags_25 },
    { 0x335f673, SUC_lexicon_tags_12 },
    { 0x215f673, SUC_lexicon_tags_15 },
    { 0x4df5f673, SUC_lexicon_tags_25 },
    { 0xd9d5f678, SUC_lexicon_tags_8 },
    { 0xd635f679, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f5f67c, SUC_lexicon_tags_27 },
    { 0x2055f67d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c75f67f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4755f682, SUC_lexicon_tags_45 },
    { 0x4ed5f682, SUC_lexicon_tags_39 },
    { 0x41f5f684, SUC_lexicon_tags_62 },
    { 0x3a55f684, SUC_lexicon_tags_1 },
    { 0x9295f682, SUC_lexicon_tags_27 },
    { 0x7f55f683, SUC_lexicon_tags_11 },
    { 0x27f5f682, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4b75f68a, SUC_lexicon_tags_7 },
    { 0x1c35f68b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xec95f68e, SUC_lexicon_tags_17 },
    { 0x6955f68f, SUC_lexicon_tags_359 },
    { 0xab35f68e, SUC_lexicon_tags_4 },
    { 0x79b5f691, SUC_lexicon_tags_29 },
    { 0x2675f691, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1f5f694, SUC_lexicon_tags_38 },
    { 0xa735f695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd75f69a, SUC_lexicon_tags_62 },
    { 0x6bd5f69a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7675f69e, SUC_lexicon_tags_38 },
    { 0x6fb5f69f, SUC_lexicon_tags_17 },
    { 0xb755f69e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6375f6a2, SUC_lexicon_tags_30 },
    { 0xead5f6a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1175f6a5, SUC_lexicon_tags_21 },
    { 0xc235f6a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f5f6ab, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d55f6b0, SUC_lexicon_tags_1 },
    { 0xa095f6b1, SUC_lexicon_tags_43 },
    { 0x9a35f6b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff5f6b8, SUC_lexicon_tags_2 },
    { 0x76b5f6b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c75f6bf, SUC_lexicon_tags_2 },
    { 0x1f75f6c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc295f6c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf5f6c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7115f6d1, SUC_lexicon_tags_17 },
    { 0xb755f6d1, SUC_lexicon_tags_1 },
    { 0x5895f6d1, SUC_lexicon_tags_17 },
    { 0x9575f6d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf6b5f6d6, SUC_lexicon_tags_2 },
    { 0x61d5f6d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5055f6e0, SUC_lexicon_tags_4 },
    { 0xa015f6e0, SUC_lexicon_tags_7 },
    { 0xcd5f6e2, SUC_lexicon_tags_30 },
    { 0xa455f6e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda55f6e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b5f6ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f35f6f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f5f6f4, SUC_lexicon_tags_43 },
    { 0xf755f6f4, SUC_lexicon_tags_37 },
    { 0xedd5f6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d5f6fa, SUC_lexicon_tags_2 },
    { 0xb75f6fa, SUC_lexicon_tags_94 },
    { 0xf0b5f6fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5335f6fe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6355f700, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3875f704, SUC_lexicon_tags_25 },
    { 0x24f5f704, SUC_lexicon_tags_2 },
    { 0x8f15f706, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad95f70c, SUC_lexicon_tags_7 },
    { 0x62d5f70d, SUC_lexicon_tags_108 },
    { 0x6e95f70e, SUC_lexicon_tags_49 },
    { 0x2935f70e, SUC_lexicon_tags_45 },
    { 0x1235f70e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9375f712, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f5f716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b5f71d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb075f71f, SUC_lexicon_tags_17 },
    { 0xe5b5f71f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf115f725, SUC_lexicon_tags_11 },
    { 0x49b5f725, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2135f72a, SUC_lexicon_tags_37 },
    { 0x68d5f72b, SUC_lexicon_tags_1 },
    { 0xce55f72c, SUC_lexicon_tags_15 },
    { 0x15f5f72d, SUC_lexicon_tags_6 },
    { 0xe375f72c, SUC_lexicon_tags_7 },
    { 0xf0d5f72f, SUC_lexicon_tags_11 },
    { 0xee75f72f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b5f733, SUC_lexicon_tags_92 },
    { 0x7e35f734, SUC_lexicon_tags_32 },
    { 0x96b5f733, SUC_lexicon_tags_48 },
    { 0x8c95f733, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89b5f738, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7555f73c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd275f73e, SUC_lexicon_tags_211 },
    { 0x6e95f73e, SUC_lexicon_tags_29 },
    { 0x215f740, SUC_lexicon_tags_107 },
    { 0x9315f740, SUC_lexicon_tags_0 },
    { 0xb695f742, SUC_lexicon_tags_43 },
    { 0x5fd5f743, SUC_lexicon_tags_191 },
    { 0x8b5f73f, SUC_lexicon_tags_7 },
    { 0x24f5f745, SUC_lexicon_tags_14 },
    { 0x9f15f73f, SUC_lexicon_tags_28 },
    { 0x8455f745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d75f74e, SUC_lexicon_tags_77 },
    { 0x9135f74f, SUC_lexicon_tags_18 },
    { 0x6275f750, SUC_lexicon_tags_19 },
    { 0xe195f750, SUC_lexicon_tags_47 },
    { 0x8b35f74f, SUC_lexicon_tags_11 },
    { 0x1df5f752, SUC_lexicon_tags_85 },
    { 0xa9b5f751, SUC_lexicon_tags_69 },
    { 0x6ef5f755, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb5f759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b5f75c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab5f763, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6515f76a, SUC_lexicon_tags_2 },
    { 0xcfd5f76b, SUC_lexicon_tags_37 },
    { 0xc815f76c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5595f777, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5415f779, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3355f77b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5f782, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde15f786, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa535f78a, SUC_lexicon_tags_6 },
    { 0x7375f78a, SUC_lexicon_tags_16 },
    { 0xa655f78c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xead5f78e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x97b5f790, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe8f5f792, SUC_lexicon_tags_47 },
    { 0x70d5f793, SUC_lexicon_tags_37 },
    { 0x9b15f794, SUC_lexicon_tags_77 },
    { 0x4f55f795, SUC_lexicon_tags_47 },
    { 0x8e95f796, SUC_lexicon_tags_12 },
    { 0xa115f795, SUC_lexicon_tags_29 },
    { 0x7a35f798, SUC_lexicon_tags_12 },
    { 0xf255f799, SUC_lexicon_tags_21 },
    { 0xab35f792, SUC_lexicon_tags_45 },
    { 0xae75f79b, SUC_lexicon_tags_7 },
    { 0xce15f79b, SUC_lexicon_tags_39 },
    { 0x3935f79d, SUC_lexicon_tags_11 },
    { 0xeaf5f79d, SUC_lexicon_tags_31 },
    { 0xf6d5f79f, SUC_lexicon_tags_82 },
    { 0x4a15f79d, SUC_lexicon_tags_15 },
    { 0xda75f7a1, SUC_lexicon_tags_17 },
    { 0x5335f7a2, SUC_lexicon_tags_45 },
    { 0xf795f7a3, SUC_lexicon_tags_82 },
    { 0x8415f7a4, SUC_lexicon_tags_79 },
    { 0xf7f5f7a1, SUC_lexicon_tags_46 },
    { 0x9db5f7a6, SUC_lexicon_tags_37 },
    { 0x9475f7a6, SUC_lexicon_tags_0 },
    { 0x8075f7a3, SUC_lexicon_tags_1 },
    { 0x3b95f794, SUC_lexicon_tags_2 },
    { 0x3855f7aa, SUC_lexicon_tags_17 },
    { 0x75d5f7aa, SUC_lexicon_tags_2 },
    { 0x22f5f794, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e75f7b0, SUC_lexicon_tags_390 },
    { 0x7195f7b1, SUC_lexicon_tags_38 },
    { 0xd5f5f7b2, SUC_lexicon_tags_48 },
    { 0xa775f7b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a75f7b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae75f7bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4515f7bf, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x1055f7c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8395f7c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b15f7cd, SUC_lexicon_tags_1 },
    { 0xce75f7ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa995f7d0, SUC_lexicon_tags_13 },
    { 0x2bf5f7d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf5f7d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe515f7d8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b5f7dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d5f7e3, SUC_lexicon_tags_23 },
    { 0xff35f7e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab5f7e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b5f7ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac35f7f6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf95f7fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x91b5f7fd, SUC_lexicon_tags_37 },
    { 0x1155f7fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe95f806, SUC_lexicon_tags_27 },
    { 0x49f5f806, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3155f809, SUC_lexicon_tags_26 },
    { 0xd2f5f80a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b5f80d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef5f811, SUC_lexicon_tags_1 },
    { 0xab35f812, SUC_lexicon_tags_25 },
    { 0x10f5f812, SUC_lexicon_tags_17 },
    { 0x79b5f814, SUC_lexicon_tags_12 },
    { 0x8015f812, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f75f81c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f5f824, SUC_lexicon_tags_7 },
    { 0xa975f824, SUC_lexicon_tags_15 },
    { 0xc215f826, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf735f828, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa695f82a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db5f82d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b5f830, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75f836, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c15f83a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5f841, SUC_lexicon_tags_9 },
    { 0x8a35f842, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c15f845, SUC_lexicon_tags_11 },
    { 0x9235f845, SUC_lexicon_tags_32 },
    { 0xce75f847, SUC_lexicon_tags_63 },
    { 0x7395f848, SUC_lexicon_tags_14 },
    { 0x5e55f849, SUC_lexicon_tags_27 },
    { 0xa935f84a, SUC_lexicon_tags_12 },
    { 0x5b5f84a, SUC_lexicon_tags_7 },
    { 0x1b35f84a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7635f851, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f5f858, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe575f85d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa475f85f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76b5f861, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8635f864, SUC_lexicon_tags_17 },
    { 0x68b5f865, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94f5f867, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe535f86b, SUC_lexicon_tags_30 },
    { 0xc195f86b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f5f870, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc415f874, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x3d55f876, SUC_lexicon_tags_9 },
    { 0x4915f877, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7895f87f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef5f885, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce75f889, SUC_lexicon_tags_14 },
    { 0x8f35f88a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8255f88d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x64b5f88f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b5f897, SUC_lexicon_tags_10 },
    { 0xaa55f897, SUC_lexicon_tags_47 },
    { 0xa855f897, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5335f89b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b5f8a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f5f8aa, SUC_lexicon_tags_27 },
    { 0xb3d5f8aa, SUC_lexicon_tags_14 },
    { 0x6d35f8ac, SUC_lexicon_tags_15 },
    { 0x6b55f8ad, SUC_lexicon_tags_38 },
    { 0x1655f8ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75f8b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2595f8ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a55f8c0, SUC_lexicon_tags_2 },
    { 0x83b5f8c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35f8c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f15f8ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b15f8cc, SUC_lexicon_tags_104 },
    { 0x99b5f8cc, SUC_lexicon_tags_25 },
    { 0x6c15f8cc, SUC_lexicon_tags_1 },
    { 0x4fd5f8cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8315f8d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa915f8d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2c55f8d5, SUC_lexicon_tags_12 },
    { 0x1555f8d6, SUC_lexicon_tags_29 },
    { 0x1d75f8d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3df5f8d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb635f8dc, SUC_lexicon_tags_17 },
    { 0x7795f8dd, SUC_lexicon_tags_0 },
    { 0xbc35f8de, SUC_lexicon_tags_11 },
    { 0xe015f8de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d5f8e2, SUC_lexicon_tags_1 },
    { 0x32d5f8e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b5f8ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc475f8ef, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b5f8f2, SUC_lexicon_tags_14 },
    { 0x8ab5f8f3, SUC_lexicon_tags_878 },
    { 0, NULL },
    { 0xbcd5f8f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6eb5f8f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x995f8f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe255f8fb, SUC_lexicon_tags_2 },
    { 0xd7b5f8fc, SUC_lexicon_tags_37 },
    { 0xab15f8fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d5f900, SUC_lexicon_tags_26 },
    { 0x2cf5f901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd335f905, SUC_lexicon_tags_17 },
    { 0x2f75f906, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6e95f908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f5f90b, SUC_lexicon_tags_2 },
    { 0xaff5f90b, SUC_lexicon_tags_71 },
    { 0x4195f90c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a35f914, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd95f91d, SUC_lexicon_tags_38 },
    { 0x79b5f91d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2015f920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa035f92a, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd5f92f, SUC_lexicon_tags_17 },
    { 0xedf5f92f, SUC_lexicon_tags_11 },
    { 0xe6d5f930, SUC_lexicon_tags_4 },
    { 0xc9d5f92f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd5f936, SUC_lexicon_tags_17 },
    { 0x8ad5f936, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f5f93a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc35f93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c75f93e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3355f943, SUC_lexicon_tags_11 },
    { 0xd695f943, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f5f94a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d5f94e, SUC_lexicon_tags_17 },
    { 0xe1f5f94f, SUC_lexicon_tags_49 },
    { 0x315f950, SUC_lexicon_tags_43 },
    { 0xb115f951, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde55f959, SUC_lexicon_tags_48 },
    { 0x7535f95a, SUC_lexicon_tags_17 },
    { 0xdb5f95a, SUC_lexicon_tags_0 },
    { 0xc2d5f95c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fd5f95e, SUC_lexicon_tags_15 },
    { 0x2355f95e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2995f962, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1015f964, SUC_lexicon_tags_45 },
    { 0xa075f965, SUC_lexicon_tags_29 },
    { 0x32d5f966, SUC_lexicon_tags_12 },
    { 0x2af5f967, SUC_lexicon_tags_3 },
    { 0x1db5f968, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9dd5f96a, SUC_lexicon_tags_63 },
    { 0x6855f96a, SUC_lexicon_tags_46 },
    { 0x1cf5f96c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa55f96e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d5f976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73d5f978, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb5f97e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d5f98a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80d5f98c, SUC_lexicon_tags_27 },
    { 0xd315f98d, SUC_lexicon_tags_21 },
    { 0xd315f98e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b5f991, SUC_lexicon_tags_27 },
    { 0x97f5f991, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd75f999, SUC_lexicon_tags_46 },
    { 0x7d95f99a, SUC_lexicon_tags_13 },
    { 0xe35f99b, SUC_lexicon_tags_27 },
    { 0xf6b5f99c, SUC_lexicon_tags_37 },
    { 0x2e75f99d, SUC_lexicon_tags_24 },
    { 0x7695f99e, SUC_lexicon_tags_25 },
    { 0x4c55f99b, SUC_lexicon_tags_63 },
    { 0x8af5f9a0, SUC_lexicon_tags_63 },
    { 0x17f5f99d, SUC_lexicon_tags_15 },
    { 0x7c75f9a2, SUC_lexicon_tags_15 },
    { 0xd055f99d, SUC_lexicon_tags_25 },
    { 0x75f99e, SUC_lexicon_tags_158 },
    { 0xe635f9a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f5f9ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff15f9ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3455f9af, SUC_lexicon_tags_28 },
    { 0xe135f9b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b5f9b6, SUC_lexicon_tags_45 },
    { 0x41b5f9b6, SUC_lexicon_tags_11 },
    { 0x4ed5f9b8, SUC_lexicon_tags_27 },
    { 0x71b5f9b9, SUC_lexicon_tags_12 },
    { 0x2fb5f9b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b15f9bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff5f9c0, SUC_lexicon_tags_1 },
    { 0x7b35f9c1, SUC_lexicon_tags_37 },
    { 0x95d5f9c1, SUC_lexicon_tags_15 },
    { 0xdb15f9c3, SUC_lexicon_tags_35 },
    { 0xfeb5f9c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd95f9c7, SUC_lexicon_tags_65 },
    { 0xd3b5f9c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb5f9cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d5f9d0, SUC_lexicon_tags_37 },
    { 0x8935f9d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af5f9d4, SUC_lexicon_tags_8 },
    { 0x99b5f9d5, SUC_lexicon_tags_29 },
    { 0xd8d5f9d6, SUC_lexicon_tags_25 },
    { 0x8e15f9d5, SUC_lexicon_tags_65 },
    { 0xeff5f9d5, SUC_lexicon_tags_37 },
    { 0x11f5f9d9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf5f9e2, SUC_lexicon_tags_6 },
    { 0x8d5f9e3, SUC_lexicon_tags_63 },
    { 0x5295f9e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc755f9e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf595f9e9, SUC_lexicon_tags_8 },
    { 0xdf5f9ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf175f9ec, SUC_lexicon_tags_15 },
    { 0xf7d5f9ed, SUC_lexicon_tags_17 },
    { 0x9135f9ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b5f9f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b5f9f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b5f9f8, SUC_lexicon_tags_8 },
    { 0x7a55f9f9, SUC_lexicon_tags_60 },
    { 0x5395f9fa, SUC_lexicon_tags_17 },
    { 0x255f9fb, SUC_lexicon_tags_38 },
    { 0x335f9fb, SUC_lexicon_tags_47 },
    { 0xde95f9fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65b5f9ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8515fa03, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd475fa06, SUC_lexicon_tags_30 },
    { 0xa5d5fa07, SUC_lexicon_tags_1 },
    { 0xb6b5fa06, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9075fa11, SUC_lexicon_tags_37 },
    { 0xa2b5fa11, SUC_lexicon_tags_46 },
    { 0xcd5fa11, SUC_lexicon_tags_17 },
    { 0xc035fa14, SUC_lexicon_tags_47 },
    { 0x2f35fa14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e75fa19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1075fa1c, SUC_lexicon_tags_28 },
    { 0xb9f5fa1d, SUC_lexicon_tags_15 },
    { 0x5575fa1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa435fa21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b5fa27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b5fa2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde55fa2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b5fa32, SUC_lexicon_tags_63 },
    { 0xda55fa33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f5fa38, SUC_lexicon_tags_6 },
    { 0xaf75fa39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d5fa40, SUC_lexicon_tags_38 },
    { 0x4ab5fa41, SUC_lexicon_tags_47 },
    { 0xd2b5fa42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc75fa46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa175fa49, SUC_lexicon_tags_28 },
    { 0xfc55fa4a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d5fa50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8295fa52, SUC_lexicon_tags_1 },
    { 0x3175fa53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5b5fa55, SUC_lexicon_tags_37 },
    { 0xa3f5fa55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc955fa5b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d5fa5e, SUC_lexicon_tags_7 },
    { 0x9ff5fa5e, SUC_lexicon_tags_15 },
    { 0xe435fa60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd15fa63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb5fa66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9b5fa68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a15fa6b, SUC_lexicon_tags_9 },
    { 0x3375fa6b, SUC_lexicon_tags_7 },
    { 0x3f35fa6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b15fa72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d75fa7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd955fa7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd775fa81, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5755fa83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7615fa87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6335fa8a, SUC_lexicon_tags_11 },
    { 0xce15fa8b, SUC_lexicon_tags_1 },
    { 0xbf55fa8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae55fa90, SUC_lexicon_tags_17 },
    { 0x6e95fa91, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d5fa94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d5fa97, SUC_lexicon_tags_1 },
    { 0xe2b5fa98, SUC_lexicon_tags_12 },
    { 0x50d5fa99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe55fa9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f5faa0, SUC_lexicon_tags_1 },
    { 0xe015faa1, SUC_lexicon_tags_47 },
    { 0x1cb5faa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca15faa4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd15faa7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd195faa9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xdf5faab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa655fab1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd115faba, SUC_lexicon_tags_1 },
    { 0x9b55fabb, SUC_lexicon_tags_7 },
    { 0x44b5fabb, SUC_lexicon_tags_1 },
    { 0xdb95fabd, SUC_lexicon_tags_17 },
    { 0x42d5fabe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe495fac0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa035fac4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e95fac6, SUC_lexicon_tags_43 },
    { 0x5c95fac6, SUC_lexicon_tags_17 },
    { 0xf75fac8, SUC_lexicon_tags_11 },
    { 0xbad5fac9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f15facc, SUC_lexicon_tags_45 },
    { 0xf0b5facd, SUC_lexicon_tags_11 },
    { 0x9fd5face, SUC_lexicon_tags_15 },
    { 0x955facf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf75fad2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2575fad5, SUC_lexicon_tags_11 },
    { 0xc735fad5, SUC_lexicon_tags_26 },
    { 0x9775fad7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f5fada, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd5fadf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1455fae1, SUC_lexicon_tags_23 },
    { 0xcf15fae2, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f5fae9, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xafd5faeb, SUC_lexicon_tags_46 },
    { 0x4455faeb, SUC_lexicon_tags_0 },
    { 0xfc95faed, SUC_lexicon_tags_15 },
    { 0x61d5faee, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b5faf5, SUC_lexicon_tags_0 },
    { 0x6b5faf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b5fafb, SUC_lexicon_tags_25 },
    { 0x835fafb, SUC_lexicon_tags_25 },
    { 0xd8b5fafd, SUC_lexicon_tags_8 },
    { 0x8c15fafe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8955fb00, SUC_lexicon_tags_11 },
    { 0xb435fb01, SUC_lexicon_tags_19 },
    { 0x8ef5fb02, SUC_lexicon_tags_11 },
    { 0x7d55fb00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc175fb05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x675fb07, SUC_lexicon_tags_23 },
    { 0x25d5fb08, SUC_lexicon_tags_15 },
    { 0x5a95fb09, SUC_lexicon_tags_14 },
    { 0x6295fb07, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e35fb13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d5fb16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc35fb18, SUC_lexicon_tags_1 },
    { 0x11d5fb19, SUC_lexicon_tags_25 },
    { 0xd935fb18, SUC_lexicon_tags_16 },
    { 0x2a35fb1b, SUC_lexicon_tags_11 },
    { 0xe975fb1c, SUC_lexicon_tags_25 },
    { 0x9515fb1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c75fb20, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd695fb22, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca75fb26, SUC_lexicon_tags_17 },
    { 0xea55fb26, SUC_lexicon_tags_37 },
    { 0x36b5fb28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e75fb2c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbdd5fb2e, SUC_lexicon_tags_17 },
    { 0x62b5fb2f, SUC_lexicon_tags_160 },
    { 0x6d5fb30, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd775fb32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7315fb34, SUC_lexicon_tags_46 },
    { 0x5dd5fb35, SUC_lexicon_tags_897 },
    { 0x8075fb36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8075fb3d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5fb48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b95fb4c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4eb5fb4e, SUC_lexicon_tags_33 },
    { 0x675fb4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf615fb51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9b5fb53, SUC_lexicon_tags_37 },
    { 0x2155fb54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6235fb59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa795fb5e, SUC_lexicon_tags_18 },
    { 0xe675fb5f, SUC_lexicon_tags_38 },
    { 0xb135fb5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa875fb68, SUC_lexicon_tags_60 },
    { 0x3815fb69, SUC_lexicon_tags_37 },
    { 0xfcb5fb68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfd5fb6c, SUC_lexicon_tags_11 },
    { 0x44d5fb6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b5fb75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd5fb7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9f5fb7e, SUC_lexicon_tags_60 },
    { 0x3695fb7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe335fb83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7575fb85, SUC_lexicon_tags_37 },
    { 0xdd75fb85, SUC_lexicon_tags_11 },
    { 0xf955fb87, SUC_lexicon_tags_2 },
    { 0xe0b5fb85, SUC_lexicon_tags_1 },
    { 0xcd55fb89, SUC_lexicon_tags_31 },
    { 0x1475fb89, SUC_lexicon_tags_65 },
    { 0xb5fb8b, SUC_lexicon_tags_13 },
    { 0xc095fb8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3535fb91, SUC_lexicon_tags_28 },
    { 0x7155fb92, SUC_lexicon_tags_7 },
    { 0x9035fb91, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9435fb96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fb9b, SUC_lexicon_tags_25 },
    { 0x92d5fb9b, SUC_lexicon_tags_12 },
    { 0xf3b5fb9c, SUC_lexicon_tags_38 },
    { 0x2f95fb9e, SUC_lexicon_tags_11 },
    { 0x1ff5fb9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d5fba1, SUC_lexicon_tags_85 },
    { 0xa615fba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d5fbab, SUC_lexicon_tags_11 },
    { 0xef5fbac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f5fbb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa675fbbd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2655fbc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b5fbc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5595fbc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x855fbcb, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6975fbd1, SUC_lexicon_tags_15 },
    { 0x1675fbd1, SUC_lexicon_tags_7 },
    { 0xe295fbd1, SUC_lexicon_tags_30 },
    { 0xfad5fbd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5575fbdc, SUC_lexicon_tags_17 },
    { 0xf355fbdd, SUC_lexicon_tags_7 },
    { 0xb495fbdd, SUC_lexicon_tags_25 },
    { 0xda75fbde, SUC_lexicon_tags_7 },
    { 0x3b75fbe0, SUC_lexicon_tags_30 },
    { 0x89d5fbe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55fbe8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb975fbec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7195fbef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2195fbf9, SUC_lexicon_tags_14 },
    { 0x1a15fbfa, SUC_lexicon_tags_13 },
    { 0xf995fbf9, SUC_lexicon_tags_60 },
    { 0x3a95fbfb, SUC_lexicon_tags_1 },
    { 0x9a35fbfd, SUC_lexicon_tags_14 },
    { 0xa6b5fbfe, SUC_lexicon_tags_2 },
    { 0x4db5fbfe, SUC_lexicon_tags_57 },
    { 0xb135fc00, SUC_lexicon_tags_5 },
    { 0xebb5fc01, SUC_lexicon_tags_15 },
    { 0xc675fc02, SUC_lexicon_tags_415 },
    { 0x3c95fbfe, SUC_lexicon_tags_33 },
    { 0x6355fbfe, SUC_lexicon_tags_7 },
    { 0x1df5fbfe, SUC_lexicon_tags_11 },
    { 0x62d5fc06, SUC_lexicon_tags_38 },
    { 0xd4f5fc06, SUC_lexicon_tags_1 },
    { 0xc375fc08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9975fc0e, SUC_lexicon_tags_47 },
    { 0x1fb5fc0f, SUC_lexicon_tags_118 },
    { 0x6ad5fc0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf5fc13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b5fc16, SUC_lexicon_tags_7 },
    { 0x1115fc17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b5fc1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8175fc20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d55fc23, SUC_lexicon_tags_12 },
    { 0x5f35fc23, SUC_lexicon_tags_11 },
    { 0xee75fc25, SUC_lexicon_tags_15 },
    { 0xfb35fc26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf255fc29, SUC_lexicon_tags_8 },
    { 0xa35fc29, SUC_lexicon_tags_14 },
    { 0x67b5fc29, SUC_lexicon_tags_42 },
    { 0x48b5fc2a, SUC_lexicon_tags_17 },
    { 0xb5b5fc2b, SUC_lexicon_tags_12 },
    { 0x2b55fc2e, SUC_lexicon_tags_25 },
    { 0xc0f5fc29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8d5fc31, SUC_lexicon_tags_15 },
    { 0x6955fc31, SUC_lexicon_tags_29 },
    { 0x1dd5fc33, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed5fc37, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb5fc3d, SUC_lexicon_tags_31 },
    { 0x2b35fc3d, SUC_lexicon_tags_49 },
    { 0x9b5fc3f, SUC_lexicon_tags_9 },
    { 0xed15fc40, SUC_lexicon_tags_0 },
    { 0xdab5fc41, SUC_lexicon_tags_77 },
    { 0x3e75fc41, SUC_lexicon_tags_19 },
    { 0x7a15fc43, SUC_lexicon_tags_1 },
    { 0x6f5fc44, SUC_lexicon_tags_7 },
    { 0x4675fc44, SUC_lexicon_tags_37 },
    { 0x3975fc46, SUC_lexicon_tags_63 },
    { 0xf0f5fc46, SUC_lexicon_tags_7 },
    { 0xc4b5fc48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d5fc4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b5fc51, SUC_lexicon_tags_11 },
    { 0x6735fc51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4455fc59, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4055fc5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5f5fc5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf5fc65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91b5fc67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf515fc7e, SUC_lexicon_tags_46 },
    { 0x9575fc7e, SUC_lexicon_tags_47 },
    { 0x6635fc7e, SUC_lexicon_tags_11 },
    { 0x5175fc81, SUC_lexicon_tags_2 },
    { 0x2675fc82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcef5fc84, SUC_lexicon_tags_17 },
    { 0xd195fc85, SUC_lexicon_tags_17 },
    { 0x4175fc84, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8ed5fc88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b95fc8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d5fc91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb655fc94, SUC_lexicon_tags_3 },
    { 0x6dd5fc94, SUC_lexicon_tags_31 },
    { 0xc895fc95, SUC_lexicon_tags_1 },
    { 0x3b95fc97, SUC_lexicon_tags_23 },
    { 0x5415fc97, SUC_lexicon_tags_0 },
    { 0x97f5fc99, SUC_lexicon_tags_21 },
    { 0xfdd5fc96, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa055fc9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d5fca4, SUC_lexicon_tags_65 },
    { 0x7cf5fca4, SUC_lexicon_tags_29 },
    { 0x4af5fca5, SUC_lexicon_tags_29 },
    { 0xbb5fca7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf375fcab, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe295fcb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d5fcb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d5fcbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3f5fcbe, SUC_lexicon_tags_39 },
    { 0xa095fcbe, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b5fcc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c95fccc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fcd3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x37f5fcd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a15fcd7, SUC_lexicon_tags_12 },
    { 0x2df5fcd8, SUC_lexicon_tags_60 },
    { 0x2fd5fcd7, SUC_lexicon_tags_15 },
    { 0x57b5fcd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c15fcde, SUC_lexicon_tags_77 },
    { 0x1935fcdf, SUC_lexicon_tags_88 },
    { 0xad35fcdf, SUC_lexicon_tags_2 },
    { 0xd315fcde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55fce7, SUC_lexicon_tags_42 },
    { 0x8a75fce7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc695fcea, SUC_lexicon_tags_16 },
    { 0xb835fceb, SUC_lexicon_tags_17 },
    { 0xc755fcea, SUC_lexicon_tags_48 },
    { 0x3795fced, SUC_lexicon_tags_15 },
    { 0xc055fcee, SUC_lexicon_tags_32 },
    { 0x17b5fcef, SUC_lexicon_tags_35 },
    { 0xcb55fcec, SUC_lexicon_tags_64 },
    { 0x5815fcec, SUC_lexicon_tags_27 },
    { 0xc9f5fcf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf5fcf7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b35fcfc, SUC_lexicon_tags_37 },
    { 0xe0b5fcfc, SUC_lexicon_tags_7 },
    { 0x4af5fcfc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b5fd04, SUC_lexicon_tags_61 },
    { 0x4355fd04, SUC_lexicon_tags_19 },
    { 0x9eb5fd06, SUC_lexicon_tags_26 },
    { 0x24b5fd07, SUC_lexicon_tags_26 },
    { 0xd2f5fd07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b5fd0a, SUC_lexicon_tags_15 },
    { 0x1715fd0a, SUC_lexicon_tags_49 },
    { 0xc315fd0c, SUC_lexicon_tags_43 },
    { 0x4715fd0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d5fd12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf255fd14, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a55fd1a, SUC_lexicon_tags_2 },
    { 0xaf35fd1b, SUC_lexicon_tags_37 },
    { 0x8575fd1c, SUC_lexicon_tags_48 },
    { 0x6f95fd1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb115fd1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b5fd25, SUC_lexicon_tags_1 },
    { 0x1275fd25, SUC_lexicon_tags_17 },
    { 0x6ff5fd27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b55fd2c, SUC_lexicon_tags_25 },
    { 0x8955fd2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d5fd30, SUC_lexicon_tags_2 },
    { 0xbc55fd31, SUC_lexicon_tags_105 },
    { 0x63f5fd32, SUC_lexicon_tags_46 },
    { 0xfad5fd33, SUC_lexicon_tags_4 },
    { 0x6035fd32, SUC_lexicon_tags_19 },
    { 0x8355fd34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9db5fd37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdfd5fd39, SUC_lexicon_tags_11 },
    { 0xcc55fd3a, SUC_lexicon_tags_11 },
    { 0x535fd3b, SUC_lexicon_tags_17 },
    { 0xa6b5fd3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe215fd3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db5fd41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff5fd47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb35fd4b, SUC_lexicon_tags_29 },
    { 0xb55fd4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3395fd4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x15b5fd51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa015fd53, SUC_lexicon_tags_9 },
    { 0x9b35fd54, SUC_lexicon_tags_87 },
    { 0x4195fd53, SUC_lexicon_tags_25 },
    { 0x9fd5fd56, SUC_lexicon_tags_7 },
    { 0x41d5fd57, SUC_lexicon_tags_66 },
    { 0xd0d5fd55, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x2615fd5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26f5fd5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda55fd61, SUC_lexicon_tags_17 },
    { 0xac95fd61, SUC_lexicon_tags_37 },
    { 0x81f5fd62, SUC_lexicon_tags_47 },
    { 0x9c35fd63, SUC_lexicon_tags_37 },
    { 0x6bd5fd65, SUC_lexicon_tags_29 },
    { 0xa975fd66, SUC_lexicon_tags_26 },
    { 0x61d5fd67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dd5fd69, SUC_lexicon_tags_1 },
    { 0x7915fd6a, SUC_lexicon_tags_6 },
    { 0xa7f5fd6b, SUC_lexicon_tags_23 },
    { 0x8875fd6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed5fd6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18d5fd71, SUC_lexicon_tags_7 },
    { 0xff55fd71, SUC_lexicon_tags_37 },
    { 0xf275fd71, SUC_lexicon_tags_34 },
    { 0x2bf5fd74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f5fd78, SUC_lexicon_tags_9 },
    { 0xd35fd79, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b5fd80, SUC_lexicon_tags_17 },
    { 0xe5b5fd80, SUC_lexicon_tags_19 },
    { 0x32d5fd82, SUC_lexicon_tags_38 },
    { 0xf155fd83, SUC_lexicon_tags_8 },
    { 0xff75fd84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa215fd87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7915fd8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf5d5fd8d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d5fd90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xed15fd92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff75fd98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x70f5fd9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7075fd9d, SUC_lexicon_tags_0 },
    { 0xcd15fd9e, SUC_lexicon_tags_38 },
    { 0xa855fd9f, SUC_lexicon_tags_65 },
    { 0xc315fda0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1535fda3, SUC_lexicon_tags_11 },
    { 0x58b5fda4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4215fdaa, SUC_lexicon_tags_33 },
    { 0xe295fdab, SUC_lexicon_tags_49 },
    { 0x93b5fdac, SUC_lexicon_tags_16 },
    { 0xa255fdac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d5fdb0, SUC_lexicon_tags_68 },
    { 0xdd15fdb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd355fdb3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4055fdbe, SUC_lexicon_tags_23 },
    { 0xb9f5fdbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db5fdc3, SUC_lexicon_tags_23 },
    { 0x9555fdc4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a15fdc7, SUC_lexicon_tags_7 },
    { 0x1eb5fdc8, SUC_lexicon_tags_19 },
    { 0x18b5fdc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd715fdce, SUC_lexicon_tags_37 },
    { 0x62b5fdcf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xac35fdd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfed5fdd3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5fdda, SUC_lexicon_tags_15 },
    { 0xbb15fddb, SUC_lexicon_tags_37 },
    { 0x9e15fddc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5df5fdde, SUC_lexicon_tags_4 },
    { 0xefb5fdde, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b5fde5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb15fde8, SUC_lexicon_tags_136 },
    { 0x8295fde9, SUC_lexicon_tags_44 },
    { 0xca95fdea, SUC_lexicon_tags_38 },
    { 0x3a95fdeb, SUC_lexicon_tags_26 },
    { 0x4ed5fdec, SUC_lexicon_tags_32 },
    { 0x1875fded, SUC_lexicon_tags_37 },
    { 0xcf75fdee, SUC_lexicon_tags_19 },
    { 0x8135fdef, SUC_lexicon_tags_12 },
    { 0x8215fded, SUC_lexicon_tags_201 },
    { 0x73d5fdf1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xf835fdf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5fdfa, SUC_lexicon_tags_17 },
    { 0xf055fdfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa935fdfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c95fdff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc775fe02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7f5fe04, SUC_lexicon_tags_31 },
    { 0xe1b5fe05, SUC_lexicon_tags_45 },
    { 0x895fe06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5fe0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad35fe12, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f5fe17, SUC_lexicon_tags_48 },
    { 0x4115fe18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a55fe1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6375fe1d, SUC_lexicon_tags_0 },
    { 0x2495fe1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8255fe21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc055fe23, SUC_lexicon_tags_3 },
    { 0xcf35fe24, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d5fe28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xab15fe2a, SUC_lexicon_tags_14 },
    { 0x6db5fe2a, SUC_lexicon_tags_11 },
    { 0x5cb5fe2c, SUC_lexicon_tags_11 },
    { 0x1b75fe2c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf5fe30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x23f5fe32, SUC_lexicon_tags_15 },
    { 0x1f35fe33, SUC_lexicon_tags_19 },
    { 0xe4b5fe34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe595fe37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb5fe3d, SUC_lexicon_tags_27 },
    { 0x4955fe3e, SUC_lexicon_tags_38 },
    { 0x9915fe3f, SUC_lexicon_tags_2 },
    { 0x8635fe40, SUC_lexicon_tags_55 },
    { 0x70b5fe3f, SUC_lexicon_tags_15 },
    { 0xb6b5fe3f, SUC_lexicon_tags_9 },
    { 0xf8d5fe42, SUC_lexicon_tags_29 },
    { 0x8175fe40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2515fe49, SUC_lexicon_tags_14 },
    { 0x34f5fe4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8635fe4f, SUC_lexicon_tags_12 },
    { 0x8415fe50, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xccf5fe52, SUC_lexicon_tags_28 },
    { 0x2d75fe53, SUC_lexicon_tags_45 },
    { 0x7fd5fe54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2755fe56, SUC_lexicon_tags_37 },
    { 0x8af5fe57, SUC_lexicon_tags_38 },
    { 0x2795fe57, SUC_lexicon_tags_75 },
    { 0xb9d5fe59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1075fe5b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb5fe5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af5fe64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab5fe69, SUC_lexicon_tags_47 },
    { 0xa7f5fe6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e35fe72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5fe76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb975fe7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a95fe7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f5fe85, SUC_lexicon_tags_42 },
    { 0x9d15fe86, SUC_lexicon_tags_17 },
    { 0x7655fe85, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe655fe89, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2d5fe8b, SUC_lexicon_tags_7 },
    { 0x6535fe8b, SUC_lexicon_tags_25 },
    { 0x1f75fe8d, SUC_lexicon_tags_13 },
    { 0xb655fe8d, SUC_lexicon_tags_38 },
    { 0xef5fe8d, SUC_lexicon_tags_13 },
    { 0xf2d5fe90, SUC_lexicon_tags_14 },
    { 0xe595fe91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb55fe95, SUC_lexicon_tags_21 },
    { 0xcd15fe95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc275fea0, SUC_lexicon_tags_12 },
    { 0xbbb5fea1, SUC_lexicon_tags_13 },
    { 0xff55fea2, SUC_lexicon_tags_13 },
    { 0x2ff5fea1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1df5fea5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8875fea7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x415fea9, SUC_lexicon_tags_25 },
    { 0x175fea9, SUC_lexicon_tags_87 },
    { 0xb8d5feab, SUC_lexicon_tags_35 },
    { 0x6bb5fea9, SUC_lexicon_tags_7 },
    { 0x135fea9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4095feb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5feb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f5fec1, SUC_lexicon_tags_63 },
    { 0x1fb5fec2, SUC_lexicon_tags_29 },
    { 0xa835fec3, SUC_lexicon_tags_26 },
    { 0x7475fec3, SUC_lexicon_tags_4 },
    { 0xe435fec1, SUC_lexicon_tags_61 },
    { 0xc2b5fec4, SUC_lexicon_tags_17 },
    { 0xc6b5fec1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc235fec9, SUC_lexicon_tags_37 },
    { 0xceb5feca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e15fecc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1af5fece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d95fed0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc9f5fed2, SUC_lexicon_tags_2 },
    { 0x4b55fed3, SUC_lexicon_tags_2 },
    { 0x6d95fed4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9915fedb, SUC_lexicon_tags_47 },
    { 0x7dd5fedc, SUC_lexicon_tags_59 },
    { 0xccd5fedc, SUC_lexicon_tags_16 },
    { 0x7215fedc, SUC_lexicon_tags_84 },
    { 0x9f55fedc, SUC_lexicon_tags_28 },
    { 0x2b15fee0, SUC_lexicon_tags_17 },
    { 0x2555fee1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5d55fee3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa675fee8, SUC_lexicon_tags_38 },
    { 0xbbf5fee9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e15feee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcc55fef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb935fef2, SUC_lexicon_tags_47 },
    { 0xb855fef2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b5fef6, SUC_lexicon_tags_11 },
    { 0x2eb5fef7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf5fefa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a35ff00, SUC_lexicon_tags_9 },
    { 0x5c55ff01, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d5ff04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa75ff06, SUC_lexicon_tags_55 },
    { 0x1f35ff06, SUC_lexicon_tags_47 },
    { 0xb735ff08, SUC_lexicon_tags_2 },
    { 0x70f5ff08, SUC_lexicon_tags_31 },
    { 0x9a75ff0a, SUC_lexicon_tags_28 },
    { 0x3c15ff0a, SUC_lexicon_tags_2 },
    { 0xb995ff0c, SUC_lexicon_tags_37 },
    { 0xf675ff0a, SUC_lexicon_tags_7 },
    { 0x7cb5ff0a, SUC_lexicon_tags_14 },
    { 0xfe15ff0f, SUC_lexicon_tags_28 },
    { 0xc735ff0a, SUC_lexicon_tags_48 },
    { 0xf0d5ff0b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x55b5ff13, SUC_lexicon_tags_2 },
    { 0xf755ff13, SUC_lexicon_tags_63 },
    { 0x6275ff15, SUC_lexicon_tags_39 },
    { 0x25f5ff13, SUC_lexicon_tags_0 },
    { 0xb675ff17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ff1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc3f5ff1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae95ff22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f95ff26, SUC_lexicon_tags_25 },
    { 0x2275ff27, SUC_lexicon_tags_14 },
    { 0xf0d5ff28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4695ff2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe295ff2f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x8c15ff31, SUC_lexicon_tags_60 },
    { 0xac15ff32, SUC_lexicon_tags_3 },
    { 0x6215ff33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa955ff36, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3b75ff38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24b5ff3a, SUC_lexicon_tags_55 },
    { 0x57f5ff3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b15ff4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ff4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f5ff51, SUC_lexicon_tags_17 },
    { 0x5535ff52, SUC_lexicon_tags_4 },
    { 0xb15ff53, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb75ff59, SUC_lexicon_tags_4 },
    { 0x8df5ff5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d95ff5c, SUC_lexicon_tags_60 },
    { 0x4fb5ff5d, SUC_lexicon_tags_1 },
    { 0xdd15ff5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc75ff64, SUC_lexicon_tags_43 },
    { 0x4a75ff65, SUC_lexicon_tags_11 },
    { 0xd695ff65, SUC_lexicon_tags_22 },
    { 0xedf5ff67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db5ff6a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc15ff6c, SUC_lexicon_tags_29 },
    { 0xac55ff6d, SUC_lexicon_tags_1 },
    { 0xca75ff6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55ff72, SUC_lexicon_tags_14 },
    { 0x5f5ff73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b5ff7b, SUC_lexicon_tags_47 },
    { 0x40f5ff7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9015ff7e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2955ff82, SUC_lexicon_tags_46 },
    { 0x1655ff83, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df5ff88, SUC_lexicon_tags_15 },
    { 0xc6f5ff89, SUC_lexicon_tags_1 },
    { 0xab95ff8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7915ff8e, SUC_lexicon_tags_56 },
    { 0x7e15ff8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb5ff92, SUC_lexicon_tags_33 },
    { 0x8835ff92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9755ff96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb935ff9a, SUC_lexicon_tags_7 },
    { 0xf075ff9a, SUC_lexicon_tags_17 },
    { 0x77d5ff9c, SUC_lexicon_tags_12 },
    { 0xa215ff9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f5ffa2, SUC_lexicon_tags_7 },
    { 0xcf5ffa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b5ffa7, SUC_lexicon_tags_49 },
    { 0x2a15ffa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe035ffad, SUC_lexicon_tags_25 },
    { 0xc6d5ffae, SUC_lexicon_tags_12 },
    { 0xfc55ffae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d5ffb4, SUC_lexicon_tags_28 },
    { 0x2f35ffb4, SUC_lexicon_tags_44 },
    { 0x4e75ffb6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa15ffb8, SUC_lexicon_tags_9 },
    { 0x50d5ffb9, SUC_lexicon_tags_11 },
    { 0x4355ffb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f5ffbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a15ffbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3235ffc6, SUC_lexicon_tags_37 },
    { 0xf5d5ffc7, SUC_lexicon_tags_47 },
    { 0x76d5ffc8, SUC_lexicon_tags_17 },
    { 0xc335ffc8, SUC_lexicon_tags_2 },
    { 0xc875ffc8, SUC_lexicon_tags_47 },
    { 0x2455ffc7, SUC_lexicon_tags_16 },
    { 0x7ed5ffca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f5ffcf, SUC_lexicon_tags_21 },
    { 0xbe55ffd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b5ffd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5ffd9, SUC_lexicon_tags_6 },
    { 0xe495ffd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x295ffdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb595ffdf, SUC_lexicon_tags_10 },
    { 0x1475ffe0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9215ffe2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb595ffe6, SUC_lexicon_tags_25 },
    { 0x5c75ffe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a35ffe9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc695fff1, SUC_lexicon_tags_33 },
    { 0x1975fff2, SUC_lexicon_tags_11 },
    { 0x1ef5fff2, SUC_lexicon_tags_14 },
    { 0xc195fff2, SUC_lexicon_tags_8 },
    { 0x23f5fff5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2375fff8, SUC_lexicon_tags_26 },
    { 0x7275fff8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc915ffff, SUC_lexicon_tags_45 },
    { 0x10560000, SUC_lexicon_tags_37 },
    { 0xc4b60001, SUC_lexicon_tags_22 },
    { 0x7d760002, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xef360005, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a560009, SUC_lexicon_tags_37 },
    { 0x22960009, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b6000d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c760013, SUC_lexicon_tags_39 },
    { 0x68f60014, SUC_lexicon_tags_17 },
    { 0xf7560013, SUC_lexicon_tags_0 },
    { 0x23f60016, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7176001b, SUC_lexicon_tags_12 },
    { 0x9cf6001c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e56001e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff60021, SUC_lexicon_tags_23 },
    { 0x12760022, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e560025, SUC_lexicon_tags_0 },
    { 0xbdf60026, SUC_lexicon_tags_10 },
    { 0x4dd60026, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e16002d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb60039, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf956003b, SUC_lexicon_tags_9 },
    { 0x9856003c, SUC_lexicon_tags_7 },
    { 0x4836003b, SUC_lexicon_tags_12 },
    { 0xe736003e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe360040, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71960043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f60046, SUC_lexicon_tags_31 },
    { 0x90360047, SUC_lexicon_tags_1 },
    { 0x32160048, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa76004f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41960052, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x83360055, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70360057, SUC_lexicon_tags_28 },
    { 0x9df60058, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2596005a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a16005d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92360061, SUC_lexicon_tags_43 },
    { 0x94d60061, SUC_lexicon_tags_17 },
    { 0x31360063, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b560066, SUC_lexicon_tags_7 },
    { 0xc1160067, SUC_lexicon_tags_8 },
    { 0xed160068, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab60071, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff60075, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e160077, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed160079, SUC_lexicon_tags_1 },
    { 0x96360079, SUC_lexicon_tags_7 },
    { 0x94760079, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed6007e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeb960080, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15760084, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3eb60086, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x59360089, SUC_lexicon_tags_5 },
    { 0xbdb60089, SUC_lexicon_tags_1 },
    { 0x9f6008b, SUC_lexicon_tags_12 },
    { 0x2cb6008c, SUC_lexicon_tags_1 },
    { 0x7096008c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb656009b, SUC_lexicon_tags_15 },
    { 0xa6b6009c, SUC_lexicon_tags_30 },
    { 0x78b6009d, SUC_lexicon_tags_250 },
    { 0xafb6009e, SUC_lexicon_tags_1 },
    { 0x1ad6009f, SUC_lexicon_tags_2 },
    { 0x2c1600a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e7600a2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4d5600a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f600a8, SUC_lexicon_tags_57 },
    { 0xcef600a8, SUC_lexicon_tags_1 },
    { 0x9a5600a8, SUC_lexicon_tags_11 },
    { 0xb8f600ab, SUC_lexicon_tags_7 },
    { 0xd69600ac, SUC_lexicon_tags_30 },
    { 0x161600a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99600b1, SUC_lexicon_tags_9 },
    { 0x62f600b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629600b6, SUC_lexicon_tags_27 },
    { 0xed3600b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x709600ba, SUC_lexicon_tags_14 },
    { 0x82d600bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x545600bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b600c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa5d600c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccb600c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7600d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe3f600d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877600d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e9600da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x973600dc, SUC_lexicon_tags_11 },
    { 0x23d600dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79600e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x81b600e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81600e8, SUC_lexicon_tags_17 },
    { 0x725600e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83600ee, SUC_lexicon_tags_15 },
    { 0xa35600ef, SUC_lexicon_tags_7 },
    { 0xe4d600ee, SUC_lexicon_tags_17 },
    { 0xc77600f1, SUC_lexicon_tags_55 },
    { 0xae1600f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91600f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91600fa, SUC_lexicon_tags_46 },
    { 0xcd5600fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf600fe, SUC_lexicon_tags_15 },
    { 0xf51600ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36560106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde96010b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb60111, SUC_lexicon_tags_491 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf60114, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c160117, SUC_lexicon_tags_91 },
    { 0xde560118, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b6011d, SUC_lexicon_tags_15 },
    { 0x4a96011e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f960122, SUC_lexicon_tags_33 },
    { 0x1af60122, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b6012c, SUC_lexicon_tags_27 },
    { 0x7796012d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58960132, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6d560134, SUC_lexicon_tags_170 },
    { 0x64960134, SUC_lexicon_tags_53 },
    { 0x2f560136, SUC_lexicon_tags_17 },
    { 0x54760136, SUC_lexicon_tags_12 },
    { 0x45960138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55160141, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5360144, SUC_lexicon_tags_12 },
    { 0x54160145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3df60147, SUC_lexicon_tags_8 },
    { 0x7c560148, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8560150, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f60152, SUC_lexicon_tags_7 },
    { 0x69560152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf60156, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3396015a, SUC_lexicon_tags_48 },
    { 0x416015b, SUC_lexicon_tags_12 },
    { 0x38b6015c, SUC_lexicon_tags_14 },
    { 0x6696015b, SUC_lexicon_tags_19 },
    { 0xcdd6015c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd60163, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f360168, SUC_lexicon_tags_23 },
    { 0xb4b60169, SUC_lexicon_tags_29 },
    { 0x87760169, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b6016d, SUC_lexicon_tags_37 },
    { 0xf0b6016e, SUC_lexicon_tags_37 },
    { 0xeb16016d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25560173, SUC_lexicon_tags_8 },
    { 0x58d60174, SUC_lexicon_tags_252 },
    { 0x8f160173, SUC_lexicon_tags_11 },
    { 0x18b60176, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b60179, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8d36017b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2b6017d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7af6017f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c960185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2216018e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xab760190, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1360192, SUC_lexicon_tags_63 },
    { 0xdf60193, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83960197, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6019b, SUC_lexicon_tags_33 },
    { 0x8a36019b, SUC_lexicon_tags_1 },
    { 0xe696019d, SUC_lexicon_tags_1 },
    { 0x6fb6019e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab601a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaa5601a3, SUC_lexicon_tags_56 },
    { 0xdd5601a3, SUC_lexicon_tags_177 },
    { 0x7ad601a5, SUC_lexicon_tags_11 },
    { 0x2bf601a6, SUC_lexicon_tags_8 },
    { 0x481601a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb1d601a9, SUC_lexicon_tags_15 },
    { 0xe69601aa, SUC_lexicon_tags_27 },
    { 0xe31601ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777601af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31601b6, SUC_lexicon_tags_10 },
    { 0x193601b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23601ba, SUC_lexicon_tags_25 },
    { 0xe69601bb, SUC_lexicon_tags_25 },
    { 0xd19601bb, SUC_lexicon_tags_0 },
    { 0x155601bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9601c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f601ce, SUC_lexicon_tags_48 },
    { 0x54b601cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5601d4, SUC_lexicon_tags_7 },
    { 0x563601d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d601da, SUC_lexicon_tags_2 },
    { 0xccd601db, SUC_lexicon_tags_11 },
    { 0x24d601db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d601e0, SUC_lexicon_tags_27 },
    { 0xa83601e0, SUC_lexicon_tags_19 },
    { 0xe5601e2, SUC_lexicon_tags_7 },
    { 0x5c3601e2, SUC_lexicon_tags_47 },
    { 0xd89601e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3601e9, SUC_lexicon_tags_50 },
    { 0xb89601ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5d601ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39d601ee, SUC_lexicon_tags_15 },
    { 0xad5601ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef601f2, SUC_lexicon_tags_49 },
    { 0xcd9601f2, SUC_lexicon_tags_23 },
    { 0x8a9601f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd93601f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab601fd, SUC_lexicon_tags_37 },
    { 0xa5601fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb160202, SUC_lexicon_tags_29 },
    { 0x65960202, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe960205, SUC_lexicon_tags_43 },
    { 0xe960206, SUC_lexicon_tags_23 },
    { 0xef760207, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce160209, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7076020b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b56020e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeed60210, SUC_lexicon_tags_11 },
    { 0x74360211, SUC_lexicon_tags_3 },
    { 0x80760212, SUC_lexicon_tags_15 },
    { 0xacf60212, SUC_lexicon_tags_16 },
    { 0x8a360211, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a360216, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5160219, SUC_lexicon_tags_27 },
    { 0x95760219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9976021c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b160222, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af60228, SUC_lexicon_tags_12 },
    { 0xfe560229, SUC_lexicon_tags_17 },
    { 0xe216022a, SUC_lexicon_tags_8 },
    { 0x22d60228, SUC_lexicon_tags_23 },
    { 0x94360229, SUC_lexicon_tags_1 },
    { 0x5f6022a, SUC_lexicon_tags_15 },
    { 0x3696022e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf60232, SUC_lexicon_tags_57 },
    { 0xb4360232, SUC_lexicon_tags_35 },
    { 0x64760233, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad60239, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3360243, SUC_lexicon_tags_10 },
    { 0x4d60244, SUC_lexicon_tags_87 },
    { 0x87b60245, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df6024b, SUC_lexicon_tags_14 },
    { 0x1216024c, SUC_lexicon_tags_36 },
    { 0xbad6024b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f60252, SUC_lexicon_tags_30 },
    { 0x5e360253, SUC_lexicon_tags_7 },
    { 0x16d60253, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0160257, SUC_lexicon_tags_25 },
    { 0x29d60258, SUC_lexicon_tags_23 },
    { 0x3360259, SUC_lexicon_tags_182 },
    { 0x4ed6025a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8136025c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c360260, SUC_lexicon_tags_65 },
    { 0x9f60261, SUC_lexicon_tags_11 },
    { 0x53d60261, SUC_lexicon_tags_25 },
    { 0x61360263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2160269, SUC_lexicon_tags_8 },
    { 0x4216026a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebf6026c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71560272, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5d560274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f60277, SUC_lexicon_tags_11 },
    { 0x1fb60277, SUC_lexicon_tags_33 },
    { 0x5f360278, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd6027d, SUC_lexicon_tags_17 },
    { 0x21f6027e, SUC_lexicon_tags_15 },
    { 0x9ed6027f, SUC_lexicon_tags_161 },
    { 0xd2960280, SUC_lexicon_tags_33 },
    { 0x2560281, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d60286, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f60289, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db6028d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4560290, SUC_lexicon_tags_7 },
    { 0xd4d60290, SUC_lexicon_tags_14 },
    { 0x7eb60292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2960295, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa6f60297, SUC_lexicon_tags_13 },
    { 0xfe960298, SUC_lexicon_tags_23 },
    { 0x8bd60298, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07602a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5602a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef1602a7, SUC_lexicon_tags_32 },
    { 0x987602a8, SUC_lexicon_tags_9 },
    { 0x5f5602a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3602ac, SUC_lexicon_tags_98 },
    { 0x525602ad, SUC_lexicon_tags_33 },
    { 0xc21602ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf09602b0, SUC_lexicon_tags_63 },
    { 0x43b602b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1602b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87b602b8, SUC_lexicon_tags_8 },
    { 0x7d602b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x629602bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5602bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1602c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xadd602c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9602c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e5602cb, SUC_lexicon_tags_17 },
    { 0x821602cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f602d2, SUC_lexicon_tags_6 },
    { 0x12f602d3, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed602d7, SUC_lexicon_tags_13 },
    { 0x865602d7, SUC_lexicon_tags_15 },
    { 0x52b602d8, SUC_lexicon_tags_2 },
    { 0x705602da, SUC_lexicon_tags_22 },
    { 0x197602da, SUC_lexicon_tags_39 },
    { 0x7a5602d8, SUC_lexicon_tags_11 },
    { 0xb89602d7, SUC_lexicon_tags_15 },
    { 0xbdd602de, SUC_lexicon_tags_1 },
    { 0x24b602dd, SUC_lexicon_tags_48 },
    { 0x8ab602e0, SUC_lexicon_tags_32 },
    { 0xe9602e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19602e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89602e8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x30b602ea, SUC_lexicon_tags_29 },
    { 0x11d602ea, SUC_lexicon_tags_14 },
    { 0x711602ec, SUC_lexicon_tags_15 },
    { 0xce9602eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb602f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf29602f3, SUC_lexicon_tags_35 },
    { 0xe3d602f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5602fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xff9602fc, SUC_lexicon_tags_14 },
    { 0x97d602fd, SUC_lexicon_tags_1 },
    { 0xfb3602fc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5760302, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79960305, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefd60307, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22360312, SUC_lexicon_tags_12 },
    { 0x4c960312, SUC_lexicon_tags_2 },
    { 0x8560314, SUC_lexicon_tags_26 },
    { 0xbf360315, SUC_lexicon_tags_87 },
    { 0xdbd60316, SUC_lexicon_tags_11 },
    { 0xd6f60315, SUC_lexicon_tags_2 },
    { 0x82160317, SUC_lexicon_tags_49 },
    { 0x70d60319, SUC_lexicon_tags_30 },
    { 0x52560319, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x836031c, SUC_lexicon_tags_61 },
    { 0xcfd6031d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d60321, SUC_lexicon_tags_1 },
    { 0xe5f60322, SUC_lexicon_tags_41 },
    { 0x9bb60323, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4160327, SUC_lexicon_tags_21 },
    { 0x5960327, SUC_lexicon_tags_11 },
    { 0x93d60329, SUC_lexicon_tags_37 },
    { 0x7c360327, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf96032e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f960330, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34560334, SUC_lexicon_tags_1 },
    { 0x27b60335, SUC_lexicon_tags_37 },
    { 0xcc960336, SUC_lexicon_tags_42 },
    { 0xb3b60335, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96033b, SUC_lexicon_tags_11 },
    { 0x3056033c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5160343, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3db60345, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf60348, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56034f, SUC_lexicon_tags_8 },
    { 0x5ff6034f, SUC_lexicon_tags_32 },
    { 0x21b60350, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6360354, SUC_lexicon_tags_49 },
    { 0xe8560355, SUC_lexicon_tags_124 },
    { 0x77f60356, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3f60358, SUC_lexicon_tags_11 },
    { 0xe1160359, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb760361, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d160365, SUC_lexicon_tags_2 },
    { 0xd3360366, SUC_lexicon_tags_37 },
    { 0x21960367, SUC_lexicon_tags_1 },
    { 0xa360366, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb6036b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8496036f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb6d60371, SUC_lexicon_tags_15 },
    { 0xba360372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2760374, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb960376, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9fd60378, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d6037e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b60387, SUC_lexicon_tags_33 },
    { 0xf2560388, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa76038d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c36038f, SUC_lexicon_tags_25 },
    { 0x2356038f, SUC_lexicon_tags_15 },
    { 0xb1b60390, SUC_lexicon_tags_10 },
    { 0x9160392, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3160395, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce360399, SUC_lexicon_tags_31 },
    { 0x63d6039a, SUC_lexicon_tags_7 },
    { 0x5296039b, SUC_lexicon_tags_12 },
    { 0xb456039c, SUC_lexicon_tags_49 },
    { 0x82b6039d, SUC_lexicon_tags_118 },
    { 0x60b6039c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f603a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcd5603a3, SUC_lexicon_tags_5 },
    { 0x69603a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd603a7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9603aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97603af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53603b1, SUC_lexicon_tags_65 },
    { 0x78d603b2, SUC_lexicon_tags_23 },
    { 0x201603b2, SUC_lexicon_tags_65 },
    { 0x5cf603b2, SUC_lexicon_tags_2 },
    { 0x98d603b5, SUC_lexicon_tags_25 },
    { 0x90d603b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1603bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd603c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5603c8, SUC_lexicon_tags_25 },
    { 0xc29603c9, SUC_lexicon_tags_38 },
    { 0xd8b603ca, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165603d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207603d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f603da, SUC_lexicon_tags_11 },
    { 0x103603da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1603de, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb603e2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847603e9, SUC_lexicon_tags_8 },
    { 0x2cf603ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c3603ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cf603ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x283603f0, SUC_lexicon_tags_25 },
    { 0x73f603f1, SUC_lexicon_tags_17 },
    { 0x46d603f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x197603f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313603f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x665603fb, SUC_lexicon_tags_15 },
    { 0x9df603fb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d60401, SUC_lexicon_tags_27 },
    { 0xcb60402, SUC_lexicon_tags_63 },
    { 0xab960402, SUC_lexicon_tags_6 },
    { 0x16360401, SUC_lexicon_tags_27 },
    { 0xd8d60404, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f160407, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2476040c, SUC_lexicon_tags_33 },
    { 0x1336040c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd60413, SUC_lexicon_tags_66 },
    { 0x69f60413, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91760419, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bd6041b, SUC_lexicon_tags_37 },
    { 0xa996041b, SUC_lexicon_tags_17 },
    { 0xd9d6041c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf60422, SUC_lexicon_tags_9 },
    { 0x96960423, SUC_lexicon_tags_37 },
    { 0xba160423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47360429, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed6042d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3760434, SUC_lexicon_tags_4 },
    { 0xd6960435, SUC_lexicon_tags_33 },
    { 0xc0360434, SUC_lexicon_tags_17 },
    { 0x88d60437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd6043a, SUC_lexicon_tags_17 },
    { 0xaed6043a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd516043e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf3760440, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1af60442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17160446, SUC_lexicon_tags_2 },
    { 0xe8b60446, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1816044b, SUC_lexicon_tags_29 },
    { 0x5056044c, SUC_lexicon_tags_37 },
    { 0xcaf6044d, SUC_lexicon_tags_2 },
    { 0x276044c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d60452, SUC_lexicon_tags_10 },
    { 0x21560453, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54760458, SUC_lexicon_tags_2 },
    { 0x96d60459, SUC_lexicon_tags_2 },
    { 0x3c56045a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81b6045c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcdd6045e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49f60460, SUC_lexicon_tags_2 },
    { 0x9d560460, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a960463, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdb160465, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8160467, SUC_lexicon_tags_1 },
    { 0xa0360467, SUC_lexicon_tags_17 },
    { 0x17560469, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x60d6046b, SUC_lexicon_tags_53 },
    { 0x3296046b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b6047a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d60483, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25760488, SUC_lexicon_tags_23 },
    { 0xc3360489, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf36048b, SUC_lexicon_tags_87 },
    { 0x90b6048c, SUC_lexicon_tags_17 },
    { 0xda16048d, SUC_lexicon_tags_2 },
    { 0x4fd6048e, SUC_lexicon_tags_37 },
    { 0x2316048f, SUC_lexicon_tags_30 },
    { 0x44160490, SUC_lexicon_tags_27 },
    { 0x3d360491, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b6049a, SUC_lexicon_tags_6 },
    { 0x23d6049a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91604a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe37604a3, SUC_lexicon_tags_47 },
    { 0x3b1604a3, SUC_lexicon_tags_47 },
    { 0x65f604a5, SUC_lexicon_tags_7 },
    { 0x76d604a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7604a9, SUC_lexicon_tags_37 },
    { 0x6df604aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b604b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ab604b4, SUC_lexicon_tags_60 },
    { 0x337604b5, SUC_lexicon_tags_13 },
    { 0x3d1604b4, SUC_lexicon_tags_39 },
    { 0x4bf604b7, SUC_lexicon_tags_7 },
    { 0x1dd604b6, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xa77604ba, SUC_lexicon_tags_47 },
    { 0xff7604bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3604c0, SUC_lexicon_tags_39 },
    { 0xe45604c0, SUC_lexicon_tags_1 },
    { 0xb39604c0, SUC_lexicon_tags_25 },
    { 0xa1d604c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x52b604c5, SUC_lexicon_tags_7 },
    { 0x7f7604c6, SUC_lexicon_tags_11 },
    { 0x63d604c7, SUC_lexicon_tags_1 },
    { 0x363604c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b604d2, SUC_lexicon_tags_8 },
    { 0xf15604d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d604d6, SUC_lexicon_tags_11 },
    { 0x799604d7, SUC_lexicon_tags_29 },
    { 0xbe7604d8, SUC_lexicon_tags_12 },
    { 0x27b604d9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05604dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff9604de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181604e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab604e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b604e8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59604ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693604f4, SUC_lexicon_tags_52 },
    { 0xf17604f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x341604f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x717604f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9604fd, SUC_lexicon_tags_435 },
    { 0xe3d604fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd3960500, SUC_lexicon_tags_261 },
    { 0x84160501, SUC_lexicon_tags_17 },
    { 0xfcf60500, SUC_lexicon_tags_37 },
    { 0xacf60502, SUC_lexicon_tags_17 },
    { 0x24d60504, SUC_lexicon_tags_23 },
    { 0x49360502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff16050a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf9b6050c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16960511, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae360515, SUC_lexicon_tags_31 },
    { 0x8b60515, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b60519, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce96051d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88560522, SUC_lexicon_tags_63 },
    { 0x3b760522, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c360526, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x48b60528, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0xf1b6052a, SUC_lexicon_tags_30 },
    { 0x436052b, SUC_lexicon_tags_87 },
    { 0x6b16052b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d560536, SUC_lexicon_tags_1 },
    { 0x1c960537, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f760539, SUC_lexicon_tags_9 },
    { 0xcf96053a, SUC_lexicon_tags_12 },
    { 0xe9b6053a, SUC_lexicon_tags_15 },
    { 0xf2f6053c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2816054d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b60554, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ad60556, SUC_lexicon_tags_0 },
    { 0x7cd60556, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b6055d, SUC_lexicon_tags_1 },
    { 0x93b6055e, SUC_lexicon_tags_17 },
    { 0x9976055d, SUC_lexicon_tags_216 },
    { 0x3c96055e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x560565, SUC_lexicon_tags_25 },
    { 0x24560566, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbb60568, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b16056b, SUC_lexicon_tags_23 },
    { 0x8376056c, SUC_lexicon_tags_0 },
    { 0xa776056c, SUC_lexicon_tags_37 },
    { 0x976056d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3d60570, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x29360573, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf60576, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac96057c, SUC_lexicon_tags_19 },
    { 0x4d6057d, SUC_lexicon_tags_12 },
    { 0x75b6057e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd360582, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd60585, SUC_lexicon_tags_10 },
    { 0xc9360585, SUC_lexicon_tags_11 },
    { 0x95760585, SUC_lexicon_tags_82 },
    { 0xb1f60588, SUC_lexicon_tags_39 },
    { 0xb4760585, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6160591, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff6059b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8316059e, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x445605a0, SUC_lexicon_tags_49 },
    { 0x661605a1, SUC_lexicon_tags_15 },
    { 0x9fb605a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9605a5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807605ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa1605ae, SUC_lexicon_tags_82 },
    { 0x9b1605ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd605b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21605b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1d605b9, SUC_lexicon_tags_25 },
    { 0xcc9605ba, SUC_lexicon_tags_1 },
    { 0xfdf605bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281605c0, SUC_lexicon_tags_17 },
    { 0x44d605c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef605cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7605dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85605e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3605e4, SUC_lexicon_tags_8 },
    { 0x3605e5, SUC_lexicon_tags_2 },
    { 0x809605e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d605e9, SUC_lexicon_tags_17 },
    { 0xe2b605ea, SUC_lexicon_tags_49 },
    { 0x4ef605ea, SUC_lexicon_tags_0 },
    { 0x721605ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b605ef, SUC_lexicon_tags_70 },
    { 0xeaf605f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f605f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1605f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9605fa, SUC_lexicon_tags_11 },
    { 0x275605fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9605fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3960602, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f60606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf60609, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1856060c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c360610, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6960613, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x22b60615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f360619, SUC_lexicon_tags_1 },
    { 0x6676061a, SUC_lexicon_tags_25 },
    { 0xe136061a, SUC_lexicon_tags_13 },
    { 0xf1560619, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf6061f, SUC_lexicon_tags_61 },
    { 0x83960620, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84f60622, SUC_lexicon_tags_14 },
    { 0x29760622, SUC_lexicon_tags_23 },
    { 0xebf60624, SUC_lexicon_tags_14 },
    { 0xd1760623, SUC_lexicon_tags_48 },
    { 0xa960626, SUC_lexicon_tags_43 },
    { 0xce960627, SUC_lexicon_tags_17 },
    { 0x54760626, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb136062b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x39d6062d, SUC_lexicon_tags_31 },
    { 0x9f76062e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc360631, SUC_lexicon_tags_17 },
    { 0x87360631, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8760636, SUC_lexicon_tags_24 },
    { 0x6c760637, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f6063a, SUC_lexicon_tags_47 },
    { 0xcaf6063b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf6063f, SUC_lexicon_tags_38 },
    { 0x6a160640, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb60646, SUC_lexicon_tags_27 },
    { 0xe2960646, SUC_lexicon_tags_40 },
    { 0x97160646, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd6064e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x99d60650, SUC_lexicon_tags_15 },
    { 0x97360650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c160655, SUC_lexicon_tags_17 },
    { 0x6e760656, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef6065a, SUC_lexicon_tags_8 },
    { 0xba76065b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad6065e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x42360661, SUC_lexicon_tags_17 },
    { 0xd9f60662, SUC_lexicon_tags_15 },
    { 0x29d60663, SUC_lexicon_tags_13 },
    { 0xd1760664, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13560668, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa996066a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c36066f, SUC_lexicon_tags_66 },
    { 0xb2160670, SUC_lexicon_tags_30 },
    { 0x19360670, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef360676, SUC_lexicon_tags_27 },
    { 0x23160676, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28960681, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa2d60683, SUC_lexicon_tags_12 },
    { 0x47360683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9760689, SUC_lexicon_tags_5 },
    { 0x3f360689, SUC_lexicon_tags_12 },
    { 0x7436068b, SUC_lexicon_tags_37 },
    { 0x8ed6068c, SUC_lexicon_tags_37 },
    { 0xd476068c, SUC_lexicon_tags_65 },
    { 0xbc36068e, SUC_lexicon_tags_29 },
    { 0x5016068f, SUC_lexicon_tags_47 },
    { 0xc8360690, SUC_lexicon_tags_15 },
    { 0x63360690, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af60695, SUC_lexicon_tags_23 },
    { 0x9f760696, SUC_lexicon_tags_14 },
    { 0x1f60697, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b6069f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689606a5, SUC_lexicon_tags_25 },
    { 0xc7606a6, SUC_lexicon_tags_47 },
    { 0xb59606a7, SUC_lexicon_tags_10 },
    { 0x2ef606a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9606b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46b606bb, SUC_lexicon_tags_17 },
    { 0x4f1606bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96b606be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75606c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2af606ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd606d6, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d606db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x645606de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x403606e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45606e4, SUC_lexicon_tags_45 },
    { 0x6a3606e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799606ed, SUC_lexicon_tags_37 },
    { 0xa71606ee, SUC_lexicon_tags_23 },
    { 0x909606ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db606f3, SUC_lexicon_tags_65 },
    { 0x49d606f4, SUC_lexicon_tags_3 },
    { 0xbfd606f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357606f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab60702, SUC_lexicon_tags_10 },
    { 0x23d60703, SUC_lexicon_tags_2 },
    { 0xb9b60704, SUC_lexicon_tags_11 },
    { 0x4b160705, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb6070a, SUC_lexicon_tags_8 },
    { 0xa276070a, SUC_lexicon_tags_12 },
    { 0xe956070a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa456070e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46360710, SUC_lexicon_tags_47 },
    { 0x40760711, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d60715, SUC_lexicon_tags_37 },
    { 0xde360715, SUC_lexicon_tags_63 },
    { 0xc7560717, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d6071b, SUC_lexicon_tags_27 },
    { 0xcf96071c, SUC_lexicon_tags_15 },
    { 0x8596071d, SUC_lexicon_tags_2 },
    { 0x9376071d, SUC_lexicon_tags_0 },
    { 0x13b6071f, SUC_lexicon_tags_15 },
    { 0xc9b60720, SUC_lexicon_tags_2 },
    { 0xf6560720, SUC_lexicon_tags_12 },
    { 0xcc560722, SUC_lexicon_tags_47 },
    { 0xf9f60723, SUC_lexicon_tags_25 },
    { 0xe4d60724, SUC_lexicon_tags_37 },
    { 0x80b60725, SUC_lexicon_tags_25 },
    { 0x5a160721, SUC_lexicon_tags_87 },
    { 0x85b60723, SUC_lexicon_tags_46 },
    { 0xd1760728, SUC_lexicon_tags_12 },
    { 0xde760726, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7636072e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c560731, SUC_lexicon_tags_33 },
    { 0x50b60732, SUC_lexicon_tags_63 },
    { 0xc5d60733, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xc1560735, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e76073d, SUC_lexicon_tags_7 },
    { 0xa716073e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b60743, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86160747, SUC_lexicon_tags_11 },
    { 0xc960748, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a76074b, SUC_lexicon_tags_19 },
    { 0x2eb6074c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3436074f, SUC_lexicon_tags_7 },
    { 0x6bf60750, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36b60752, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xc7f60754, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd160757, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf256075c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b560761, SUC_lexicon_tags_25 },
    { 0x1db60761, SUC_lexicon_tags_37 },
    { 0xf6160763, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3f60765, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d60769, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8bf6076b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc160772, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4960775, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbad60777, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf56077d, SUC_lexicon_tags_2 },
    { 0x8056077d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44160784, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xc9560786, SUC_lexicon_tags_63 },
    { 0xecf60787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f6078a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x696078d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc760790, SUC_lexicon_tags_38 },
    { 0xf4d60791, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc8d60793, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773607a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x311607aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd607ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xadd607af, SUC_lexicon_tags_17 },
    { 0xc57607af, SUC_lexicon_tags_17 },
    { 0x5e3607b0, SUC_lexicon_tags_18 },
    { 0x96d607b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b607b8, SUC_lexicon_tags_2 },
    { 0x729607b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f7607bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57607be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839607c6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc31607c8, SUC_lexicon_tags_15 },
    { 0x6e9607c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7607cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f607d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd29607d6, SUC_lexicon_tags_30 },
    { 0xd11607d7, SUC_lexicon_tags_6 },
    { 0x791607d8, SUC_lexicon_tags_38 },
    { 0x8e3607d8, SUC_lexicon_tags_14 },
    { 0xcb3607d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21607dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xddd607df, SUC_lexicon_tags_27 },
    { 0x561607e0, SUC_lexicon_tags_6 },
    { 0xf43607e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x91607e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad607ec, SUC_lexicon_tags_68 },
    { 0xcd7607ed, SUC_lexicon_tags_11 },
    { 0xfbd607ed, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe87607f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32b607f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6b607f4, SUC_lexicon_tags_8 },
    { 0xb7f607f5, SUC_lexicon_tags_8 },
    { 0x5f7607f6, SUC_lexicon_tags_57 },
    { 0x563607f7, SUC_lexicon_tags_23 },
    { 0xbc3607f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b607fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af60800, SUC_lexicon_tags_11 },
    { 0xda960801, SUC_lexicon_tags_16 },
    { 0xd5960802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73d60804, SUC_lexicon_tags_7 },
    { 0xecf60805, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf760807, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7236080f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b60813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2760818, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6456081a, SUC_lexicon_tags_31 },
    { 0xec96081b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5936081e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5560827, SUC_lexicon_tags_47 },
    { 0x93b60828, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3ad6082a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d60831, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb7b60833, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df60836, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f560839, SUC_lexicon_tags_49 },
    { 0x8116083a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa936083d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb376083f, SUC_lexicon_tags_33 },
    { 0x1f60840, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x88560842, SUC_lexicon_tags_17 },
    { 0x60760842, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f60849, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d6084e, SUC_lexicon_tags_37 },
    { 0xba36084e, SUC_lexicon_tags_37 },
    { 0xf5d60850, SUC_lexicon_tags_25 },
    { 0xc0360851, SUC_lexicon_tags_37 },
    { 0xe9160850, SUC_lexicon_tags_27 },
    { 0x87b60853, SUC_lexicon_tags_1 },
    { 0x6d60854, SUC_lexicon_tags_85 },
    { 0x44360855, SUC_lexicon_tags_1 },
    { 0xbfd60855, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc5960858, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xe596085a, SUC_lexicon_tags_37 },
    { 0xfbf6085b, SUC_lexicon_tags_47 },
    { 0xf156085a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ed6085e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb5760860, SUC_lexicon_tags_38 },
    { 0x14560861, SUC_lexicon_tags_37 },
    { 0xfdf60860, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d60865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a160868, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b6086b, SUC_lexicon_tags_37 },
    { 0xc496086c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e560870, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6360875, SUC_lexicon_tags_17 },
    { 0x5af60876, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd60879, SUC_lexicon_tags_1 },
    { 0xd0360879, SUC_lexicon_tags_7 },
    { 0xca760879, SUC_lexicon_tags_6 },
    { 0xbff6087b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2396087e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70160881, SUC_lexicon_tags_25 },
    { 0xcf360881, SUC_lexicon_tags_0 },
    { 0x29360882, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3560889, SUC_lexicon_tags_65 },
    { 0x88960889, SUC_lexicon_tags_27 },
    { 0xaf6088b, SUC_lexicon_tags_37 },
    { 0x276088c, SUC_lexicon_tags_37 },
    { 0xdc56088c, SUC_lexicon_tags_30 },
    { 0x1796088e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb560895, SUC_lexicon_tags_25 },
    { 0xf0360896, SUC_lexicon_tags_37 },
    { 0x3bf60895, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa316089a, SUC_lexicon_tags_29 },
    { 0x43b6089b, SUC_lexicon_tags_0 },
    { 0x9316089c, SUC_lexicon_tags_37 },
    { 0xd1d6089d, SUC_lexicon_tags_1 },
    { 0x6236089e, SUC_lexicon_tags_2 },
    { 0x7fd6089f, SUC_lexicon_tags_15 },
    { 0x5ff6089e, SUC_lexicon_tags_7 },
    { 0x1316089f, SUC_lexicon_tags_4 },
    { 0x96089f, SUC_lexicon_tags_15 },
    { 0x347608a3, SUC_lexicon_tags_37 },
    { 0x719608a4, SUC_lexicon_tags_7 },
    { 0x5db608a5, SUC_lexicon_tags_11 },
    { 0x75608a0, SUC_lexicon_tags_25 },
    { 0xa03608a5, SUC_lexicon_tags_26 },
    { 0xb61608a6, SUC_lexicon_tags_15 },
    { 0xd8b608a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d9608ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1608b0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b608b3, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367608ba, SUC_lexicon_tags_15 },
    { 0x13608ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f9608bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf608c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcad608c4, SUC_lexicon_tags_7 },
    { 0x74f608c5, SUC_lexicon_tags_15 },
    { 0x235608c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x225608c9, SUC_lexicon_tags_66 },
    { 0xf79608ca, SUC_lexicon_tags_11 },
    { 0xfc7608cb, SUC_lexicon_tags_14 },
    { 0x231608cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547608d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x537608d2, SUC_lexicon_tags_15 },
    { 0xbfd608d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1f608d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9608d7, SUC_lexicon_tags_11 },
    { 0x9c7608d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535608dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71d608df, SUC_lexicon_tags_11 },
    { 0x651608e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97608e9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7cb608eb, SUC_lexicon_tags_15 },
    { 0xc6b608eb, SUC_lexicon_tags_2 },
    { 0xfb5608ed, SUC_lexicon_tags_27 },
    { 0x6f3608ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd29608f0, SUC_lexicon_tags_30 },
    { 0xaf3608f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f608f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf6d608f6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f608fb, SUC_lexicon_tags_25 },
    { 0x3cb608fc, SUC_lexicon_tags_28 },
    { 0xd81608fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80d608ff, SUC_lexicon_tags_30 },
    { 0x38360900, SUC_lexicon_tags_47 },
    { 0xb4d608ff, SUC_lexicon_tags_34 },
    { 0xb81608ff, SUC_lexicon_tags_17 },
    { 0xf2b60901, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb360907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3096090a, SUC_lexicon_tags_15 },
    { 0x4cb6090b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d6090e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83760912, SUC_lexicon_tags_234 },
    { 0x54b60913, SUC_lexicon_tags_23 },
    { 0x88760912, SUC_lexicon_tags_12 },
    { 0x10f60915, SUC_lexicon_tags_85 },
    { 0x6c960916, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ab60918, SUC_lexicon_tags_25 },
    { 0xa4d60919, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f76091d, SUC_lexicon_tags_25 },
    { 0x8796091d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d60921, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71360923, SUC_lexicon_tags_108 },
    { 0x14360923, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4f60926, SUC_lexicon_tags_14 },
    { 0xe5960926, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e96092c, SUC_lexicon_tags_4 },
    { 0x7316092c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe856092f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e360931, SUC_lexicon_tags_7 },
    { 0x62760932, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee960934, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8160936, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe776093b, SUC_lexicon_tags_23 },
    { 0x1c96093c, SUC_lexicon_tags_12 },
    { 0xd5d6093b, SUC_lexicon_tags_17 },
    { 0x53d6093c, SUC_lexicon_tags_17 },
    { 0x9b36093f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d60944, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38560949, SUC_lexicon_tags_25 },
    { 0x75f6094a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f96094d, SUC_lexicon_tags_38 },
    { 0x9df6094d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c760950, SUC_lexicon_tags_15 },
    { 0x67960951, SUC_lexicon_tags_18 },
    { 0x15560952, SUC_lexicon_tags_37 },
    { 0xa1360953, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f160959, SUC_lexicon_tags_12 },
    { 0x116095a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa396095c, SUC_lexicon_tags_60 },
    { 0x8bd6095d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb216095f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5960962, SUC_lexicon_tags_15 },
    { 0x95360962, SUC_lexicon_tags_57 },
    { 0xe3d60963, SUC_lexicon_tags_16 },
    { 0xbc960965, SUC_lexicon_tags_17 },
    { 0xd3d60965, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20360974, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xde560976, SUC_lexicon_tags_5 },
    { 0x3fb60977, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cb60979, SUC_lexicon_tags_0 },
    { 0xa3760979, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b56097c, SUC_lexicon_tags_15 },
    { 0xa276097d, SUC_lexicon_tags_11 },
    { 0x2c36097e, SUC_lexicon_tags_1 },
    { 0x8b36097f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60960981, SUC_lexicon_tags_7 },
    { 0xb5960982, SUC_lexicon_tags_63 },
    { 0xaad60983, SUC_lexicon_tags_84 },
    { 0xec960983, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d6098e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b60991, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x19360993, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a160997, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd6099b, SUC_lexicon_tags_23 },
    { 0xb536099b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1716099f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3609a2, SUC_lexicon_tags_19 },
    { 0xd1f609a3, SUC_lexicon_tags_19 },
    { 0x52f609a3, SUC_lexicon_tags_1 },
    { 0x3ed609a5, SUC_lexicon_tags_88 },
    { 0x593609a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07609aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b609ad, SUC_lexicon_tags_68 },
    { 0x349609ae, SUC_lexicon_tags_25 },
    { 0xc41609af, SUC_lexicon_tags_18 },
    { 0x759609ae, SUC_lexicon_tags_28 },
    { 0x4fb609b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b609b4, SUC_lexicon_tags_82 },
    { 0x39b609b5, SUC_lexicon_tags_27 },
    { 0xe11609b6, SUC_lexicon_tags_30 },
    { 0x313609b6, SUC_lexicon_tags_0 },
    { 0xdcf609b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42d609ba, SUC_lexicon_tags_30 },
    { 0x9ef609bb, SUC_lexicon_tags_0 },
    { 0xc61609bc, SUC_lexicon_tags_33 },
    { 0x391609bd, SUC_lexicon_tags_17 },
    { 0x49b609bd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x923609c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d609cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x537609ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f609d2, SUC_lexicon_tags_1 },
    { 0x919609d2, SUC_lexicon_tags_17 },
    { 0x2df609d2, SUC_lexicon_tags_38 },
    { 0x9c3609d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x647609d7, SUC_lexicon_tags_7 },
    { 0x841609d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x617609db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa43609dd, SUC_lexicon_tags_66 },
    { 0x6ed609dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9609e3, SUC_lexicon_tags_25 },
    { 0xe9f609e3, SUC_lexicon_tags_1 },
    { 0x8c3609e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33609e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xceb609eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b609f0, SUC_lexicon_tags_15 },
    { 0x909609f1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb609f6, SUC_lexicon_tags_37 },
    { 0xafb609f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713609fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7609ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb560a01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a960a09, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e960a0f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe360a14, SUC_lexicon_tags_6 },
    { 0x19360a14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a360a18, SUC_lexicon_tags_12 },
    { 0x9a760a18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d60a1c, SUC_lexicon_tags_48 },
    { 0xfad60a1d, SUC_lexicon_tags_29 },
    { 0x4d960a1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2560a24, SUC_lexicon_tags_1 },
    { 0x11560a25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d60a28, SUC_lexicon_tags_37 },
    { 0x50160a28, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1c160a2b, SUC_lexicon_tags_10 },
    { 0xbfb60a2b, SUC_lexicon_tags_1 },
    { 0xced60a2d, SUC_lexicon_tags_63 },
    { 0x7b760a2e, SUC_lexicon_tags_2 },
    { 0xc2f60a2e, SUC_lexicon_tags_17 },
    { 0x56960a2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2b160a32, SUC_lexicon_tags_37 },
    { 0x51d60a32, SUC_lexicon_tags_37 },
    { 0xe7f60a34, SUC_lexicon_tags_31 },
    { 0xeb760a35, SUC_lexicon_tags_11 },
    { 0x8e360a35, SUC_lexicon_tags_27 },
    { 0xd2960a36, SUC_lexicon_tags_37 },
    { 0x38360a32, SUC_lexicon_tags_38 },
    { 0x37f60a39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b960a3b, SUC_lexicon_tags_84 },
    { 0x83160a3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b560a3f, SUC_lexicon_tags_17 },
    { 0xfe560a3f, SUC_lexicon_tags_12 },
    { 0xe6760a3f, SUC_lexicon_tags_30 },
    { 0x22760a41, SUC_lexicon_tags_17 },
    { 0x9b60a40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f60a48, SUC_lexicon_tags_27 },
    { 0x92d60a49, SUC_lexicon_tags_2 },
    { 0x4af60a4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b360a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb960a4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93960a51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f60a55, SUC_lexicon_tags_68 },
    { 0xffd60a55, SUC_lexicon_tags_65 },
    { 0x22960a57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf60a5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf760a62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc360a65, SUC_lexicon_tags_19 },
    { 0x98d60a65, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4af60a68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b60a6c, SUC_lexicon_tags_37 },
    { 0xc3160a6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe560a71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x21760a74, SUC_lexicon_tags_46 },
    { 0x360a74, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6360a7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48760a7e, SUC_lexicon_tags_9 },
    { 0x39360a7f, SUC_lexicon_tags_2 },
    { 0x4d60a80, SUC_lexicon_tags_37 },
    { 0x28960a7f, SUC_lexicon_tags_11 },
    { 0xef60a82, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0560a88, SUC_lexicon_tags_25 },
    { 0xcf760a89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ed60a8b, SUC_lexicon_tags_85 },
    { 0x4bb60a8c, SUC_lexicon_tags_87 },
    { 0x43760a8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d160a93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2160a98, SUC_lexicon_tags_29 },
    { 0x77360a99, SUC_lexicon_tags_26 },
    { 0x6ef60a99, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d60a9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b560aa1, SUC_lexicon_tags_47 },
    { 0x12360aa1, SUC_lexicon_tags_11 },
    { 0x5e560aa2, SUC_lexicon_tags_25 },
    { 0xaff60aa4, SUC_lexicon_tags_1 },
    { 0x6a960aa3, SUC_lexicon_tags_56 },
    { 0xaeb60aa6, SUC_lexicon_tags_49 },
    { 0xcdd60aa3, SUC_lexicon_tags_11 },
    { 0xa8360aa8, SUC_lexicon_tags_30 },
    { 0x9b60aa8, SUC_lexicon_tags_47 },
    { 0x7b760aaa, SUC_lexicon_tags_12 },
    { 0x8eb60aa6, SUC_lexicon_tags_17 },
    { 0xd6960aa7, SUC_lexicon_tags_47 },
    { 0xbd760aad, SUC_lexicon_tags_23 },
    { 0x4c160aa4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42560ab2, SUC_lexicon_tags_33 },
    { 0xa3960ab3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf60ab6, SUC_lexicon_tags_25 },
    { 0x4f960ab7, SUC_lexicon_tags_7 },
    { 0x3f360ab7, SUC_lexicon_tags_19 },
    { 0x43560ab9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x94b60abb, SUC_lexicon_tags_11 },
    { 0xec760abc, SUC_lexicon_tags_12 },
    { 0xd5760abd, SUC_lexicon_tags_2 },
    { 0x39160abe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95560ac3, SUC_lexicon_tags_43 },
    { 0x99960ac4, SUC_lexicon_tags_1 },
    { 0x46160ac5, SUC_lexicon_tags_13 },
    { 0xdc160ac5, SUC_lexicon_tags_15 },
    { 0x33f60ac7, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83760acc, SUC_lexicon_tags_46 },
    { 0xeeb60acd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5960acf, SUC_lexicon_tags_11 },
    { 0x13160ad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd60ad3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf2d60ad5, SUC_lexicon_tags_4 },
    { 0x23360ad6, SUC_lexicon_tags_33 },
    { 0xd3b60ad6, SUC_lexicon_tags_8 },
    { 0xfb960ad8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a560adf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f60ae4, SUC_lexicon_tags_31 },
    { 0x7b560ae5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10560ae9, SUC_lexicon_tags_2 },
    { 0x22360aea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c360aed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad60af1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73d60af3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6160af8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4cd60afa, SUC_lexicon_tags_7 },
    { 0x96f60afa, SUC_lexicon_tags_1 },
    { 0xd5d60afc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48960b00, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45560b04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb60b0f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf60b12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16760b19, SUC_lexicon_tags_2 },
    { 0xf160b1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b60b24, SUC_lexicon_tags_2 },
    { 0xd8b60b25, SUC_lexicon_tags_80 },
    { 0x3d960b25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x71360b28, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d60b2b, SUC_lexicon_tags_19 },
    { 0x97b60b2c, SUC_lexicon_tags_11 },
    { 0x3e960b2c, SUC_lexicon_tags_11 },
    { 0xc2d60b2e, SUC_lexicon_tags_17 },
    { 0xd9560b2e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x40160b32, SUC_lexicon_tags_17 },
    { 0x57560b33, SUC_lexicon_tags_45 },
    { 0x5bb60b34, SUC_lexicon_tags_2 },
    { 0x59560b35, SUC_lexicon_tags_1 },
    { 0x1ef60b33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a560b39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeaf60b3b, SUC_lexicon_tags_47 },
    { 0x78b60b3c, SUC_lexicon_tags_63 },
    { 0x92b60b3b, SUC_lexicon_tags_37 },
    { 0x52160b3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5760b41, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9ad60b43, SUC_lexicon_tags_40 },
    { 0x10f60b44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd60b49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec160b4c, SUC_lexicon_tags_12 },
    { 0xd5b60b4d, SUC_lexicon_tags_16 },
    { 0x76160b4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b60b52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x160b57, SUC_lexicon_tags_48 },
    { 0x4e160b57, SUC_lexicon_tags_92 },
    { 0xa9d60b59, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xce560b5b, SUC_lexicon_tags_277 },
    { 0xc1960b5c, SUC_lexicon_tags_39 },
    { 0xc5360b5d, SUC_lexicon_tags_17 },
    { 0x12b60b5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85360b60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf60b64, SUC_lexicon_tags_17 },
    { 0x3db60b65, SUC_lexicon_tags_60 },
    { 0x64b60b65, SUC_lexicon_tags_17 },
    { 0x64b60b67, SUC_lexicon_tags_31 },
    { 0x1b360b68, SUC_lexicon_tags_12 },
    { 0x30160b69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xda960b6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd60b70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1560b74, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x8e160b76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9360b7c, SUC_lexicon_tags_8 },
    { 0x7c360b7d, SUC_lexicon_tags_28 },
    { 0x49d60b7d, SUC_lexicon_tags_2 },
    { 0x2c760b7d, SUC_lexicon_tags_38 },
    { 0x5d560b80, SUC_lexicon_tags_49 },
    { 0xc0160b81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93760b84, SUC_lexicon_tags_29 },
    { 0x57160b85, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7960b8b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5160b90, SUC_lexicon_tags_0 },
    { 0xd6360b90, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5dd60b93, SUC_lexicon_tags_16 },
    { 0xced60b93, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x39b60b96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd60b99, SUC_lexicon_tags_39 },
    { 0x42560b9a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf60ba2, SUC_lexicon_tags_46 },
    { 0xe6960ba3, SUC_lexicon_tags_37 },
    { 0x2d360ba4, SUC_lexicon_tags_25 },
    { 0xd1960ba4, SUC_lexicon_tags_37 },
    { 0x6c960ba5, SUC_lexicon_tags_37 },
    { 0xaf560ba5, SUC_lexicon_tags_30 },
    { 0xfe560ba3, SUC_lexicon_tags_17 },
    { 0xf1f60ba8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x76360bab, SUC_lexicon_tags_31 },
    { 0x72160bab, SUC_lexicon_tags_11 },
    { 0xab560bad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e360bb2, SUC_lexicon_tags_2 },
    { 0x62960bb3, SUC_lexicon_tags_35 },
    { 0x1e760bb4, SUC_lexicon_tags_2 },
    { 0x45160bb3, SUC_lexicon_tags_19 },
    { 0x79160bb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd60bbc, SUC_lexicon_tags_63 },
    { 0x97b60bbd, SUC_lexicon_tags_26 },
    { 0xa0f60bbe, SUC_lexicon_tags_12 },
    { 0x99b60bbe, SUC_lexicon_tags_17 },
    { 0x9af60bc0, SUC_lexicon_tags_2 },
    { 0x4b160bc1, SUC_lexicon_tags_17 },
    { 0x95960bc1, SUC_lexicon_tags_38 },
    { 0x8ed60bc2, SUC_lexicon_tags_167 },
    { 0x13560bc4, SUC_lexicon_tags_33 },
    { 0x22d60bc4, SUC_lexicon_tags_1 },
    { 0x9ff60bc6, SUC_lexicon_tags_1 },
    { 0x9d360bc1, SUC_lexicon_tags_17 },
    { 0x3e160bc4, SUC_lexicon_tags_1 },
    { 0x97d60bc9, SUC_lexicon_tags_15 },
    { 0x74960bbd, SUC_lexicon_tags_12 },
    { 0x6f60bbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18160bce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf60bd1, SUC_lexicon_tags_0 },
    { 0xf3b60bd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d360bd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1360bd6, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0xe8360bd8, SUC_lexicon_tags_14 },
    { 0x8c560bd9, SUC_lexicon_tags_6 },
    { 0x1a360bda, SUC_lexicon_tags_12 },
    { 0xe8f60bdb, SUC_lexicon_tags_38 },
    { 0x59f60bdc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b60bdf, SUC_lexicon_tags_12 },
    { 0x34d60bdf, SUC_lexicon_tags_27 },
    { 0x82760be0, SUC_lexicon_tags_1 },
    { 0x87160be0, SUC_lexicon_tags_2 },
    { 0xf9b60be0, SUC_lexicon_tags_61 },
    { 0x2d960be4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66160bea, SUC_lexicon_tags_151 },
    { 0x32760bea, SUC_lexicon_tags_17 },
    { 0x4dd60bec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdad60bee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca960bf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3760bf6, SUC_lexicon_tags_19 },
    { 0xfb760bf7, SUC_lexicon_tags_43 },
    { 0xaf160bf8, SUC_lexicon_tags_17 },
    { 0xdfb60bf7, SUC_lexicon_tags_43 },
    { 0x35160bf6, SUC_lexicon_tags_37 },
    { 0xd1f60bfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46360c01, SUC_lexicon_tags_8 },
    { 0x45160c02, SUC_lexicon_tags_27 },
    { 0xd0760c02, SUC_lexicon_tags_37 },
    { 0xc4160c04, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58760c0e, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x94760c10, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x28560c13, SUC_lexicon_tags_17 },
    { 0x59b60c14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f760c18, SUC_lexicon_tags_10 },
    { 0x46160c19, SUC_lexicon_tags_9 },
    { 0x78f60c1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d960c1c, SUC_lexicon_tags_17 },
    { 0x36360c1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f60c20, SUC_lexicon_tags_55 },
    { 0xd0b60c21, SUC_lexicon_tags_38 },
    { 0xf6960c20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe7960c24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdb60c26, SUC_lexicon_tags_11 },
    { 0x45360c26, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e360c2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d560c32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43560c35, SUC_lexicon_tags_4 },
    { 0x73360c36, SUC_lexicon_tags_37 },
    { 0x2fd60c37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b60c3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31760c42, SUC_lexicon_tags_17 },
    { 0x77160c43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d60c47, SUC_lexicon_tags_37 },
    { 0xbaf60c48, SUC_lexicon_tags_33 },
    { 0xbdd60c49, SUC_lexicon_tags_1 },
    { 0x9d760c49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81160c4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c960c51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f60c54, SUC_lexicon_tags_17 },
    { 0x54f60c54, SUC_lexicon_tags_17 },
    { 0x61960c54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73960c5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xec960c5f, SUC_lexicon_tags_15 },
    { 0x7a360c60, SUC_lexicon_tags_14 },
    { 0xb7160c61, SUC_lexicon_tags_27 },
    { 0x4b760c61, SUC_lexicon_tags_12 },
    { 0x72f60c63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13760c6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbb60c6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd60c73, SUC_lexicon_tags_87 },
    { 0x2e560c73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x74160c77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa0560c79, SUC_lexicon_tags_138 },
    { 0x3e960c7a, SUC_lexicon_tags_25 },
    { 0x1c560c7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb960c7e, SUC_lexicon_tags_6 },
    { 0x2b60c7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf60c87, SUC_lexicon_tags_25 },
    { 0x36b60c87, SUC_lexicon_tags_1 },
    { 0x5bf60c89, SUC_lexicon_tags_1 },
    { 0xd9160c8a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x66f60c8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af60c91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5bb60c93, SUC_lexicon_tags_30 },
    { 0x5b960c93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7d60c96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1160c98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f60c9c, SUC_lexicon_tags_35 },
    { 0xabd60c9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x14760c9f, SUC_lexicon_tags_14 },
    { 0xf1b60c9f, SUC_lexicon_tags_45 },
    { 0xe4760ca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b60ca4, SUC_lexicon_tags_31 },
    { 0xfe560ca5, SUC_lexicon_tags_17 },
    { 0x12360ca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d60caa, SUC_lexicon_tags_38 },
    { 0xfa760cab, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb760cb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39960cb2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1960cb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c760cb9, SUC_lexicon_tags_82 },
    { 0xc8160cb9, SUC_lexicon_tags_8 },
    { 0xf2f60cbb, SUC_lexicon_tags_57 },
    { 0x72160cba, SUC_lexicon_tags_7 },
    { 0x9cb60cb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3960cc0, SUC_lexicon_tags_25 },
    { 0xbc560cc1, SUC_lexicon_tags_0 },
    { 0x2cf60cc2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcc560cc4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4db60cc6, SUC_lexicon_tags_82 },
    { 0x6360cc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f560cca, SUC_lexicon_tags_17 },
    { 0xc2d60cca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38760cce, SUC_lexicon_tags_37 },
    { 0x99960ccf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2360cd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92960ce2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb60ce6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63160ce8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2e160cea, SUC_lexicon_tags_69 },
    { 0xa9760cea, SUC_lexicon_tags_56 },
    { 0xa2560cea, SUC_lexicon_tags_55 },
    { 0x35160ced, SUC_lexicon_tags_17 },
    { 0xba160cea, SUC_lexicon_tags_3 },
    { 0xf9b60cef, SUC_lexicon_tags_25 },
    { 0xa1f60cf0, SUC_lexicon_tags_25 },
    { 0x4e760cf1, SUC_lexicon_tags_2 },
    { 0xc2d60cf2, SUC_lexicon_tags_7 },
    { 0x37360cf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xded60cf5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf560cf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77160cf9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6ef60cfb, SUC_lexicon_tags_49 },
    { 0x23960cfb, SUC_lexicon_tags_29 },
    { 0x80560cfd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xead60d00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd60d03, SUC_lexicon_tags_49 },
    { 0x86b60d04, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba960d09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89360d0b, SUC_lexicon_tags_107 },
    { 0x5f760d0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b60d0f, SUC_lexicon_tags_25 },
    { 0x89760d0f, SUC_lexicon_tags_66 },
    { 0xf9160d10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f360d13, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb60d18, SUC_lexicon_tags_15 },
    { 0x5360d19, SUC_lexicon_tags_37 },
    { 0xb5160d1a, SUC_lexicon_tags_27 },
    { 0x6f160d1b, SUC_lexicon_tags_11 },
    { 0x2df60d1b, SUC_lexicon_tags_0 },
    { 0x80160d1d, SUC_lexicon_tags_7 },
    { 0x66560d1b, SUC_lexicon_tags_17 },
    { 0x98f60d1f, SUC_lexicon_tags_16 },
    { 0x42360d20, SUC_lexicon_tags_15 },
    { 0x92560d20, SUC_lexicon_tags_28 },
    { 0x6a160d22, SUC_lexicon_tags_30 },
    { 0xa3960d22, SUC_lexicon_tags_47 },
    { 0xcad60d24, SUC_lexicon_tags_47 },
    { 0xb3d60d23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbe960d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14160d2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e160d2f, SUC_lexicon_tags_37 },
    { 0xb9f60d2f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd60d36, SUC_lexicon_tags_230 },
    { 0xc5960d37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6760d3b, SUC_lexicon_tags_6 },
    { 0xa9360d3c, SUC_lexicon_tags_12 },
    { 0x6ef60d3d, SUC_lexicon_tags_12 },
    { 0xabd60d3d, SUC_lexicon_tags_8 },
    { 0x3af60d3b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc9360d41, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef60d44, SUC_lexicon_tags_13 },
    { 0x5e160d45, SUC_lexicon_tags_11 },
    { 0x73d60d45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa360d48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d160d4b, SUC_lexicon_tags_12 },
    { 0x22f60d4c, SUC_lexicon_tags_12 },
    { 0xbd360d4d, SUC_lexicon_tags_37 },
    { 0xffd60d4e, SUC_lexicon_tags_17 },
    { 0xf6760d4d, SUC_lexicon_tags_14 },
    { 0x2a360d4d, SUC_lexicon_tags_74 },
    { 0x69760d51, SUC_lexicon_tags_49 },
    { 0xf8560d4e, SUC_lexicon_tags_25 },
    { 0xb1560d52, SUC_lexicon_tags_2 },
    { 0xc9560d4e, SUC_lexicon_tags_15 },
    { 0xbfb60d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e560d59, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf4d60d5b, SUC_lexicon_tags_47 },
    { 0x18560d5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed60d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6760d66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18960d73, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f60d79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1760d7c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51160d84, SUC_lexicon_tags_16 },
    { 0xc6160d84, SUC_lexicon_tags_14 },
    { 0xd5960d85, SUC_lexicon_tags_7 },
    { 0x34f60d87, SUC_lexicon_tags_17 },
    { 0xf9d60d87, SUC_lexicon_tags_26 },
    { 0x16160d89, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef60d8e, SUC_lexicon_tags_84 },
    { 0xfcd60d8f, SUC_lexicon_tags_3 },
    { 0x48b60d90, SUC_lexicon_tags_37 },
    { 0xded60d8f, SUC_lexicon_tags_1 },
    { 0x21b60d92, SUC_lexicon_tags_56 },
    { 0x51d60d93, SUC_lexicon_tags_7 },
    { 0xd5d60d8f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0960d98, SUC_lexicon_tags_17 },
    { 0x2f360d99, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64960d9f, SUC_lexicon_tags_48 },
    { 0x50d60d9f, SUC_lexicon_tags_11 },
    { 0x9a760d9f, SUC_lexicon_tags_43 },
    { 0x44b60da2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cf60da4, SUC_lexicon_tags_63 },
    { 0xa5b60da4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d60da8, SUC_lexicon_tags_542 },
    { 0xa7960da9, SUC_lexicon_tags_2 },
    { 0x7a360daa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89760dad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30160db2, SUC_lexicon_tags_29 },
    { 0xc7560db3, SUC_lexicon_tags_120 },
    { 0xe4560db4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc360db6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded60dbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64160dca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2bf60dcc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2c560dce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb60dd0, SUC_lexicon_tags_131 },
    { 0x7960dd1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae160dd5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde760dd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0960ddd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d60de3, SUC_lexicon_tags_14 },
    { 0x3960de4, SUC_lexicon_tags_13 },
    { 0x4a560de5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf560dec, SUC_lexicon_tags_11 },
    { 0xcd560dec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f60df0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37160df3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61760dfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79560dfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb60e04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b60e07, SUC_lexicon_tags_11 },
    { 0x2bd60e07, SUC_lexicon_tags_46 },
    { 0x19160e09, SUC_lexicon_tags_29 },
    { 0x23b60e07, SUC_lexicon_tags_11 },
    { 0x82f60e0b, SUC_lexicon_tags_17 },
    { 0x6f760e0b, SUC_lexicon_tags_37 },
    { 0xf4d60e0d, SUC_lexicon_tags_26 },
    { 0x14f60e07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ed60e10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca160e13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd760e17, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29960e1b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd60e1e, SUC_lexicon_tags_47 },
    { 0xb9360e1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b60e27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ab60e29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47960e2d, SUC_lexicon_tags_25 },
    { 0x99560e2e, SUC_lexicon_tags_24 },
    { 0xf4760e2f, SUC_lexicon_tags_15 },
    { 0xb0560e2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x22d60e32, SUC_lexicon_tags_65 },
    { 0xa7b60e33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db60e36, SUC_lexicon_tags_19 },
    { 0x7af60e37, SUC_lexicon_tags_11 },
    { 0xa5960e38, SUC_lexicon_tags_29 },
    { 0xed60e39, SUC_lexicon_tags_48 },
    { 0xb4f60e3a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f960e3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x11160e41, SUC_lexicon_tags_12 },
    { 0x8cf60e41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cf60e44, SUC_lexicon_tags_9 },
    { 0x27160e45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f60e48, SUC_lexicon_tags_506 },
    { 0, NULL },
    { 0x9960e4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8360e4f, SUC_lexicon_tags_35 },
    { 0x74d60e4f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x16d60e52, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9560e55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf560e58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d60e5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f760e5d, SUC_lexicon_tags_14 },
    { 0x45960e5e, SUC_lexicon_tags_7 },
    { 0x4e560e5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21360e68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb7160e6a, SUC_lexicon_tags_7 },
    { 0xda560e6a, SUC_lexicon_tags_3 },
    { 0x70960e6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54560e6f, SUC_lexicon_tags_38 },
    { 0x22f60e6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e160e74, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x39360e76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8160e7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffb60e7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb60e83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b60e89, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a160e90, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb360e96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6960e99, SUC_lexicon_tags_63 },
    { 0x5f360e9a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e760ea0, SUC_lexicon_tags_44 },
    { 0xb9d60ea0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51160eb1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7d60eb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f60eb7, SUC_lexicon_tags_0 },
    { 0xda760eb7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x65160eba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4b960ebc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x78160ebe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d60ec4, SUC_lexicon_tags_14 },
    { 0x1d60ec4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d760eca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xab560ecd, SUC_lexicon_tags_7 },
    { 0x35960ecd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38160ed7, SUC_lexicon_tags_7 },
    { 0x38560ed8, SUC_lexicon_tags_55 },
    { 0xa6d60ed9, SUC_lexicon_tags_10 },
    { 0x6f160eda, SUC_lexicon_tags_7 },
    { 0xfe560eda, SUC_lexicon_tags_37 },
    { 0xce760edb, SUC_lexicon_tags_7 },
    { 0x91760edd, SUC_lexicon_tags_12 },
    { 0x70760edd, SUC_lexicon_tags_37 },
    { 0x22f60edf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x35560ee1, SUC_lexicon_tags_29 },
    { 0x59160ee2, SUC_lexicon_tags_0 },
    { 0xceb60ee3, SUC_lexicon_tags_37 },
    { 0xb9760ee2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b60ee9, SUC_lexicon_tags_37 },
    { 0xce760ee9, SUC_lexicon_tags_2 },
    { 0x9760eeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd60ef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3360ef4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7560efa, SUC_lexicon_tags_38 },
    { 0x80160efb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f60efe, SUC_lexicon_tags_1 },
    { 0x7d560eff, SUC_lexicon_tags_37 },
    { 0x99360eff, SUC_lexicon_tags_11 },
    { 0xa8560f01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa560f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48760f05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58760f07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb160f0a, SUC_lexicon_tags_47 },
    { 0x8f960f0b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1960f0e, SUC_lexicon_tags_31 },
    { 0x35960f0f, SUC_lexicon_tags_66 },
    { 0xfb60f0f, SUC_lexicon_tags_25 },
    { 0xcf360f0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf60f15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc160f1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d60f1f, SUC_lexicon_tags_60 },
    { 0x1cd60f1f, SUC_lexicon_tags_12 },
    { 0xa0760f21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57560f23, SUC_lexicon_tags_11 },
    { 0xec760f24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x83960f27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6160f31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d60f35, SUC_lexicon_tags_673 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8960f38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d60f3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b560f3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43160f3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6360f43, SUC_lexicon_tags_15 },
    { 0x36b60f43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b60f47, SUC_lexicon_tags_23 },
    { 0x14760f48, SUC_lexicon_tags_7 },
    { 0x5a760f48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed60f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d960f4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62360f52, SUC_lexicon_tags_10 },
    { 0xf2960f53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb2760f55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2360f59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60760f5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x65560f60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee160f64, SUC_lexicon_tags_7 },
    { 0xe1960f65, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81760f69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c160f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b60f73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e960f79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f60f7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85360f80, SUC_lexicon_tags_47 },
    { 0xde560f81, SUC_lexicon_tags_8 },
    { 0x85360f82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f60f87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0360f8a, SUC_lexicon_tags_14 },
    { 0x8cd60f8b, SUC_lexicon_tags_37 },
    { 0xfaf60f8b, SUC_lexicon_tags_15 },
    { 0x8fb60f8d, SUC_lexicon_tags_17 },
    { 0x97f60f8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63760f92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14560f95, SUC_lexicon_tags_27 },
    { 0x66160f96, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f60f9b, SUC_lexicon_tags_4 },
    { 0x26b60f9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x44960f9e, SUC_lexicon_tags_10 },
    { 0x6560f9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32560fab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b60fb1, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0x87b60fb3, SUC_lexicon_tags_27 },
    { 0x57160fb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e760fb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b60fbc, SUC_lexicon_tags_23 },
    { 0xc6360fbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25960fc4, SUC_lexicon_tags_25 },
    { 0xbad60fc5, SUC_lexicon_tags_27 },
    { 0x94f60fc4, SUC_lexicon_tags_25 },
    { 0x4f60fc7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd60fca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd760fcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29560fd1, SUC_lexicon_tags_44 },
    { 0x6bf60fd2, SUC_lexicon_tags_30 },
    { 0xf2d60fd3, SUC_lexicon_tags_0 },
    { 0xbaf60fd4, SUC_lexicon_tags_18 },
    { 0x99160fd3, SUC_lexicon_tags_43 },
    { 0xa9560fd4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x83d60fd8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d60fdb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a160fe2, SUC_lexicon_tags_33 },
    { 0xbcb60fe3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97960fe7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef60ff4, SUC_lexicon_tags_48 },
    { 0x2db60ff5, SUC_lexicon_tags_47 },
    { 0x82560ff5, SUC_lexicon_tags_37 },
    { 0xf3b60ff4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0960ffa, SUC_lexicon_tags_0 },
    { 0x22160ffb, SUC_lexicon_tags_25 },
    { 0x52560ffb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f360ffe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d61004, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9361009, SUC_lexicon_tags_7 },
    { 0xe9961009, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc196100c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe416100e, SUC_lexicon_tags_2 },
    { 0x416100f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d61012, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd61016, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc16101b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f6101f, SUC_lexicon_tags_17 },
    { 0xf056101f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f61023, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25561028, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf436102b, SUC_lexicon_tags_27 },
    { 0xd136102b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9361030, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20561032, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x99f61034, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29561037, SUC_lexicon_tags_37 },
    { 0x89761038, SUC_lexicon_tags_10 },
    { 0x52d61037, SUC_lexicon_tags_45 },
    { 0x716103a, SUC_lexicon_tags_63 },
    { 0x1df6103a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b361043, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd961047, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7576104a, SUC_lexicon_tags_15 },
    { 0xef76104b, SUC_lexicon_tags_25 },
    { 0xeb36104b, SUC_lexicon_tags_11 },
    { 0x2cb6104b, SUC_lexicon_tags_30 },
    { 0xe156104b, SUC_lexicon_tags_0 },
    { 0x5f16104f, SUC_lexicon_tags_37 },
    { 0x31561050, SUC_lexicon_tags_26 },
    { 0xf836104e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6fb61053, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3cf61055, SUC_lexicon_tags_1 },
    { 0x69161055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce161059, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b6105c, SUC_lexicon_tags_47 },
    { 0x6e16105c, SUC_lexicon_tags_2 },
    { 0xf776105e, SUC_lexicon_tags_25 },
    { 0x5ef6105f, SUC_lexicon_tags_2 },
    { 0xf3b61060, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4361063, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f61066, SUC_lexicon_tags_49 },
    { 0xbe361067, SUC_lexicon_tags_27 },
    { 0xec161066, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8596106a, SUC_lexicon_tags_15 },
    { 0x65f6106b, SUC_lexicon_tags_2 },
    { 0xd9b6106c, SUC_lexicon_tags_32 },
    { 0x3a96106d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d56106f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf61072, SUC_lexicon_tags_11 },
    { 0x3f561073, SUC_lexicon_tags_6 },
    { 0x56d61074, SUC_lexicon_tags_1 },
    { 0xc3b61073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc256107a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb6107d, SUC_lexicon_tags_87 },
    { 0xf3b6107e, SUC_lexicon_tags_37 },
    { 0xb916107e, SUC_lexicon_tags_12 },
    { 0x27b61080, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b361082, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef61085, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf96108f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f61093, SUC_lexicon_tags_1 },
    { 0xe4761093, SUC_lexicon_tags_1 },
    { 0xa9361093, SUC_lexicon_tags_17 },
    { 0xa7561096, SUC_lexicon_tags_12 },
    { 0x20d61096, SUC_lexicon_tags_31 },
    { 0xd5361097, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f16109d, SUC_lexicon_tags_17 },
    { 0x3ed6109e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b610a7, SUC_lexicon_tags_75 },
    { 0xb31610a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa9f610aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b610ad, SUC_lexicon_tags_0 },
    { 0x2af610ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x829610b0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4f5610b2, SUC_lexicon_tags_32 },
    { 0xaf610b3, SUC_lexicon_tags_37 },
    { 0xfef610b4, SUC_lexicon_tags_17 },
    { 0x239610b3, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369610bc, SUC_lexicon_tags_4 },
    { 0x8fd610bd, SUC_lexicon_tags_126 },
    { 0x929610be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501610c3, SUC_lexicon_tags_17 },
    { 0x267610c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9610cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37610cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f610d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b610d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d610d9, SUC_lexicon_tags_66 },
    { 0xc41610d9, SUC_lexicon_tags_25 },
    { 0xab3610db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35610de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769610e7, SUC_lexicon_tags_2 },
    { 0xf0f610e8, SUC_lexicon_tags_11 },
    { 0x1d1610e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df610ec, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f610f5, SUC_lexicon_tags_10 },
    { 0x63f610f6, SUC_lexicon_tags_47 },
    { 0xe4f610f7, SUC_lexicon_tags_56 },
    { 0x38f610f6, SUC_lexicon_tags_28 },
    { 0xe69610f9, SUC_lexicon_tags_10 },
    { 0x25610fa, SUC_lexicon_tags_37 },
    { 0xdc1610f5, SUC_lexicon_tags_42 },
    { 0x99f610f9, SUC_lexicon_tags_17 },
    { 0x7cf610fd, SUC_lexicon_tags_12 },
    { 0x4c1610fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd961105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f61109, SUC_lexicon_tags_2 },
    { 0x3476110a, SUC_lexicon_tags_45 },
    { 0x36561109, SUC_lexicon_tags_2 },
    { 0xe3961109, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8476110e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb61110, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a161117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3256111b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6296111e, SUC_lexicon_tags_1 },
    { 0x2036111f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bf61121, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b61124, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1836112b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d61130, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d561135, SUC_lexicon_tags_17 },
    { 0x53b61136, SUC_lexicon_tags_29 },
    { 0xb8361136, SUC_lexicon_tags_2 },
    { 0x34361136, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc936113b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56b6113d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdeb6113f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80561141, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1561143, SUC_lexicon_tags_38 },
    { 0x35f61144, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b161149, SUC_lexicon_tags_25 },
    { 0xa6f6114a, SUC_lexicon_tags_25 },
    { 0x2c561149, SUC_lexicon_tags_77 },
    { 0x776114c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8561155, SUC_lexicon_tags_32 },
    { 0x1a161155, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x30561159, SUC_lexicon_tags_1 },
    { 0xe0f61159, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x90f6115c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b61160, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16161162, SUC_lexicon_tags_12 },
    { 0xaa161163, SUC_lexicon_tags_37 },
    { 0xc9961162, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x36361167, SUC_lexicon_tags_11 },
    { 0x43161168, SUC_lexicon_tags_30 },
    { 0x7a761169, SUC_lexicon_tags_11 },
    { 0x5fb61169, SUC_lexicon_tags_12 },
    { 0xbf56116b, SUC_lexicon_tags_31 },
    { 0x13561169, SUC_lexicon_tags_45 },
    { 0x7736116d, SUC_lexicon_tags_31 },
    { 0xe416116e, SUC_lexicon_tags_12 },
    { 0xee76116f, SUC_lexicon_tags_12 },
    { 0x6876116f, SUC_lexicon_tags_92 },
    { 0xae561171, SUC_lexicon_tags_33 },
    { 0xe7961172, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b761175, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e361178, SUC_lexicon_tags_17 },
    { 0x9ed61178, SUC_lexicon_tags_1 },
    { 0x96117a, SUC_lexicon_tags_3 },
    { 0xb216117a, SUC_lexicon_tags_91 },
    { 0x6f161179, SUC_lexicon_tags_1 },
    { 0x2cb6117b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7296117f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b961184, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79161186, SUC_lexicon_tags_19 },
    { 0x62361187, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1556118b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x96118d, SUC_lexicon_tags_11 },
    { 0xa3b6118e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f61195, SUC_lexicon_tags_11 },
    { 0xd1561196, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb1b61198, SUC_lexicon_tags_54 },
    { 0xb2361199, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f36119d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca76119f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b611a2, SUC_lexicon_tags_60 },
    { 0xfbd611a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05611a7, SUC_lexicon_tags_25 },
    { 0x8bd611a8, SUC_lexicon_tags_7 },
    { 0xa3611a9, SUC_lexicon_tags_15 },
    { 0x4f611aa, SUC_lexicon_tags_23 },
    { 0xf5f611a7, SUC_lexicon_tags_61 },
    { 0x3d7611a7, SUC_lexicon_tags_25 },
    { 0x363611ad, SUC_lexicon_tags_1 },
    { 0x375611ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55611b1, SUC_lexicon_tags_12 },
    { 0x853611b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423611b6, SUC_lexicon_tags_37 },
    { 0xb1b611b7, SUC_lexicon_tags_37 },
    { 0x83b611b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdf5611ba, SUC_lexicon_tags_2 },
    { 0x5eb611bb, SUC_lexicon_tags_28 },
    { 0x7fd611bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd83611be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27611c2, SUC_lexicon_tags_25 },
    { 0x857611c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43611c7, SUC_lexicon_tags_61 },
    { 0x67f611c8, SUC_lexicon_tags_12 },
    { 0xeb1611c7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x901611cc, SUC_lexicon_tags_1 },
    { 0x2e9611cd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5611d1, SUC_lexicon_tags_25 },
    { 0x353611d2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x587611d5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe2b611d7, SUC_lexicon_tags_2 },
    { 0xdd1611d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7611dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7611df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b1611e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535611e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d611e9, SUC_lexicon_tags_9 },
    { 0x1e9611e9, SUC_lexicon_tags_4 },
    { 0xbfd611eb, SUC_lexicon_tags_1 },
    { 0x33f611e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x591611ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3611f3, SUC_lexicon_tags_37 },
    { 0x433611f4, SUC_lexicon_tags_23 },
    { 0x95611f5, SUC_lexicon_tags_17 },
    { 0xa8d611f4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2ad611f8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5611ff, SUC_lexicon_tags_19 },
    { 0xffb61200, SUC_lexicon_tags_23 },
    { 0xd2b61200, SUC_lexicon_tags_37 },
    { 0xe761202, SUC_lexicon_tags_11 },
    { 0x35161203, SUC_lexicon_tags_37 },
    { 0xfcf61201, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3f61206, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f6120a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f6120d, SUC_lexicon_tags_25 },
    { 0x76120d, SUC_lexicon_tags_2 },
    { 0xce56120f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b61214, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3df61216, SUC_lexicon_tags_2 },
    { 0xaab61217, SUC_lexicon_tags_8 },
    { 0x4761218, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f6121d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b56121f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b961223, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd6122f, SUC_lexicon_tags_38 },
    { 0xe1f61230, SUC_lexicon_tags_6 },
    { 0xf1761231, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2561239, SUC_lexicon_tags_2 },
    { 0x8e56123a, SUC_lexicon_tags_22 },
    { 0xa76123b, SUC_lexicon_tags_27 },
    { 0x1bd6123c, SUC_lexicon_tags_38 },
    { 0x7856123d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb3b6123f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed61242, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd61245, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c961251, SUC_lexicon_tags_65 },
    { 0x1c361252, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d361258, SUC_lexicon_tags_79 },
    { 0xfd761259, SUC_lexicon_tags_12 },
    { 0x6e36125a, SUC_lexicon_tags_29 },
    { 0x7f96125b, SUC_lexicon_tags_11 },
    { 0x1196125c, SUC_lexicon_tags_16 },
    { 0xc936125c, SUC_lexicon_tags_8 },
    { 0x6fd61258, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6361262, SUC_lexicon_tags_0 },
    { 0xec161262, SUC_lexicon_tags_29 },
    { 0xced61263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f61267, SUC_lexicon_tags_14 },
    { 0x79561268, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8f56126a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x18b6126c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x71f6126e, SUC_lexicon_tags_207 },
    { 0x1bd6126f, SUC_lexicon_tags_15 },
    { 0x24161270, SUC_lexicon_tags_33 },
    { 0x7a761270, SUC_lexicon_tags_26 },
    { 0xa8d6126e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6161276, SUC_lexicon_tags_19 },
    { 0xabd61277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f361279, SUC_lexicon_tags_2 },
    { 0x8a36127a, SUC_lexicon_tags_21 },
    { 0x27d6127a, SUC_lexicon_tags_17 },
    { 0x9936127c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed6127f, SUC_lexicon_tags_37 },
    { 0x44161280, SUC_lexicon_tags_63 },
    { 0xe2f61281, SUC_lexicon_tags_7 },
    { 0x4bf61282, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e961288, SUC_lexicon_tags_28 },
    { 0xaa561289, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5436128b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb236128d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f61291, SUC_lexicon_tags_0 },
    { 0x34561292, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db61296, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x69161299, SUC_lexicon_tags_27 },
    { 0x44b6129a, SUC_lexicon_tags_25 },
    { 0x9df6129a, SUC_lexicon_tags_14 },
    { 0x7d76129c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3612a6, SUC_lexicon_tags_151 },
    { 0x8ef612a6, SUC_lexicon_tags_15 },
    { 0x3ef612a8, SUC_lexicon_tags_37 },
    { 0x541612a8, SUC_lexicon_tags_11 },
    { 0x983612aa, SUC_lexicon_tags_11 },
    { 0xed1612aa, SUC_lexicon_tags_25 },
    { 0xa25612ac, SUC_lexicon_tags_65 },
    { 0x561612a6, SUC_lexicon_tags_31 },
    { 0xc79612ae, SUC_lexicon_tags_17 },
    { 0x1a1612af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565612b6, SUC_lexicon_tags_0 },
    { 0x2f3612b7, SUC_lexicon_tags_7 },
    { 0x7d7612b8, SUC_lexicon_tags_11 },
    { 0x633612b7, SUC_lexicon_tags_8 },
    { 0x3c9612b8, SUC_lexicon_tags_11 },
    { 0xbdf612b7, SUC_lexicon_tags_12 },
    { 0x675612bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x63d612be, SUC_lexicon_tags_32 },
    { 0x7e9612bf, SUC_lexicon_tags_12 },
    { 0xf13612bf, SUC_lexicon_tags_1 },
    { 0x2f7612be, SUC_lexicon_tags_11 },
    { 0x8b612c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x73612c4, SUC_lexicon_tags_21 },
    { 0x67d612c5, SUC_lexicon_tags_19 },
    { 0xaa7612c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac5612c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263612cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd612cf, SUC_lexicon_tags_17 },
    { 0x66d612cf, SUC_lexicon_tags_7 },
    { 0xfe5612d1, SUC_lexicon_tags_37 },
    { 0xce1612d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b612d9, SUC_lexicon_tags_39 },
    { 0x43f612da, SUC_lexicon_tags_6 },
    { 0xab3612db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x173612dd, SUC_lexicon_tags_17 },
    { 0x6d7612dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31612e0, SUC_lexicon_tags_30 },
    { 0x689612e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179612ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x335612ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767612f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243612f6, SUC_lexicon_tags_36 },
    { 0xecf612f7, SUC_lexicon_tags_6 },
    { 0x965612f8, SUC_lexicon_tags_0 },
    { 0xc93612f7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x127612fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb39612fd, SUC_lexicon_tags_38 },
    { 0xbf7612fe, SUC_lexicon_tags_2 },
    { 0x771612fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16761308, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b6130b, SUC_lexicon_tags_2 },
    { 0x9096130b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa456130e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76561314, SUC_lexicon_tags_27 },
    { 0x81361315, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3d361317, SUC_lexicon_tags_57 },
    { 0x80761318, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb776131b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5476131f, SUC_lexicon_tags_12 },
    { 0x3056131f, SUC_lexicon_tags_29 },
    { 0xacf61321, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x35561323, SUC_lexicon_tags_7 },
    { 0x52161323, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95561327, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65b61329, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb6132f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83161335, SUC_lexicon_tags_53 },
    { 0x31761336, SUC_lexicon_tags_25 },
    { 0xe3b61337, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5076133e, SUC_lexicon_tags_37 },
    { 0x43b6133f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49561344, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5a761346, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af61349, SUC_lexicon_tags_31 },
    { 0x6036134a, SUC_lexicon_tags_92 },
    { 0x1a96134b, SUC_lexicon_tags_193 },
    { 0xee56134c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe761352, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b61355, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f6135b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xafb6135d, SUC_lexicon_tags_49 },
    { 0x9b76135d, SUC_lexicon_tags_4 },
    { 0x9356135e, SUC_lexicon_tags_12 },
    { 0x66961360, SUC_lexicon_tags_49 },
    { 0x66161361, SUC_lexicon_tags_19 },
    { 0xe016135d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5161365, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c161367, SUC_lexicon_tags_3 },
    { 0x9d761368, SUC_lexicon_tags_69 },
    { 0xff61369, SUC_lexicon_tags_47 },
    { 0x46f6136a, SUC_lexicon_tags_113 },
    { 0xed161369, SUC_lexicon_tags_29 },
    { 0x58b61369, SUC_lexicon_tags_6 },
    { 0x7a36136d, SUC_lexicon_tags_1 },
    { 0x8f76136d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd361372, SUC_lexicon_tags_7 },
    { 0x1c961373, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x68161376, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b61379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c76137b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e161382, SUC_lexicon_tags_87 },
    { 0x2a561382, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9eb61385, SUC_lexicon_tags_17 },
    { 0xd1361385, SUC_lexicon_tags_17 },
    { 0x29761387, SUC_lexicon_tags_17 },
    { 0x34961385, SUC_lexicon_tags_11 },
    { 0xc4761386, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e96138e, SUC_lexicon_tags_25 },
    { 0x3b56138e, SUC_lexicon_tags_16 },
    { 0xa2361390, SUC_lexicon_tags_25 },
    { 0xf996138f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d961395, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f6139a, SUC_lexicon_tags_11 },
    { 0x6796139b, SUC_lexicon_tags_47 },
    { 0x1b36139c, SUC_lexicon_tags_65 },
    { 0x21b6139d, SUC_lexicon_tags_74 },
    { 0x3736139d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa73613a0, SUC_lexicon_tags_65 },
    { 0x805613a0, SUC_lexicon_tags_8 },
    { 0x271613a0, SUC_lexicon_tags_63 },
    { 0xf3f613a1, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d613a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x325613a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21613ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f613b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x797613b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3c9613b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd613be, SUC_lexicon_tags_46 },
    { 0xfc5613bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f613c2, SUC_lexicon_tags_7 },
    { 0x601613c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd613c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbff613c9, SUC_lexicon_tags_31 },
    { 0x739613ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x911613cd, SUC_lexicon_tags_17 },
    { 0xa4b613ce, SUC_lexicon_tags_25 },
    { 0xa49613cf, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9613d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f613d7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115613db, SUC_lexicon_tags_42 },
    { 0xf73613dc, SUC_lexicon_tags_15 },
    { 0x111613dd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xe87613df, SUC_lexicon_tags_11 },
    { 0x4d613e0, SUC_lexicon_tags_0 },
    { 0xa63613e0, SUC_lexicon_tags_2 },
    { 0x447613e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af613e8, SUC_lexicon_tags_0 },
    { 0xf3f613e8, SUC_lexicon_tags_37 },
    { 0x401613ea, SUC_lexicon_tags_60 },
    { 0x70b613eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3613ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c7613f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23b613f3, SUC_lexicon_tags_9 },
    { 0x613613f4, SUC_lexicon_tags_68 },
    { 0x415613f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9613f8, SUC_lexicon_tags_47 },
    { 0xb25613f8, SUC_lexicon_tags_15 },
    { 0x887613f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3613fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9dd613ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90161405, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a361407, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416140c, SUC_lexicon_tags_18 },
    { 0x8d96140d, SUC_lexicon_tags_33 },
    { 0x9e36140e, SUC_lexicon_tags_161 },
    { 0xfff6140c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b61412, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2bf61414, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b6141b, SUC_lexicon_tags_37 },
    { 0x2756141c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f61422, SUC_lexicon_tags_95 },
    { 0x8e361423, SUC_lexicon_tags_1 },
    { 0xf4161424, SUC_lexicon_tags_19 },
    { 0x37961425, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x69761428, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe036142c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb76142f, SUC_lexicon_tags_7 },
    { 0x28561430, SUC_lexicon_tags_12 },
    { 0x79561430, SUC_lexicon_tags_2 },
    { 0x5f361432, SUC_lexicon_tags_1 },
    { 0xcc561432, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15b61435, SUC_lexicon_tags_27 },
    { 0x4bf61436, SUC_lexicon_tags_42 },
    { 0xb6361437, SUC_lexicon_tags_8 },
    { 0xa5361438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f6143b, SUC_lexicon_tags_11 },
    { 0xdb76143b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b61440, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3961442, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1161447, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76144b, SUC_lexicon_tags_37 },
    { 0xc976144c, SUC_lexicon_tags_37 },
    { 0xf9f6144d, SUC_lexicon_tags_33 },
    { 0x3f6144e, SUC_lexicon_tags_11 },
    { 0x9456144f, SUC_lexicon_tags_25 },
    { 0x8776144c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a961453, SUC_lexicon_tags_87 },
    { 0x16161454, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef6145a, SUC_lexicon_tags_14 },
    { 0x64b6145a, SUC_lexicon_tags_37 },
    { 0x2196145c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e161460, SUC_lexicon_tags_26 },
    { 0x49361461, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a561466, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d6146e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed61473, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db61476, SUC_lexicon_tags_32 },
    { 0xf1b61477, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d6147d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d61480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5561483, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb61487, SUC_lexicon_tags_17 },
    { 0x7a561488, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11b6148a, SUC_lexicon_tags_0 },
    { 0xff6148b, SUC_lexicon_tags_37 },
    { 0x52b6148a, SUC_lexicon_tags_11 },
    { 0x1bf6148d, SUC_lexicon_tags_0 },
    { 0xb5f6148c, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x7df61490, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b61492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1361498, SUC_lexicon_tags_11 },
    { 0x3a761499, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20b6149b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96149d, SUC_lexicon_tags_2 },
    { 0x5cd6149d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x633614a1, SUC_lexicon_tags_26 },
    { 0x163614a2, SUC_lexicon_tags_17 },
    { 0x59d614a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63b614a5, SUC_lexicon_tags_15 },
    { 0x101614a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf614a9, SUC_lexicon_tags_32 },
    { 0xeb7614a9, SUC_lexicon_tags_15 },
    { 0xc43614aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21614af, SUC_lexicon_tags_15 },
    { 0x653614b0, SUC_lexicon_tags_0 },
    { 0x74f614b1, SUC_lexicon_tags_17 },
    { 0x1e7614b1, SUC_lexicon_tags_28 },
    { 0x499614b3, SUC_lexicon_tags_25 },
    { 0xe9b614b3, SUC_lexicon_tags_11 },
    { 0xe09614b2, SUC_lexicon_tags_11 },
    { 0x633614b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43614b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85d614bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf95614bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe29614bf, SUC_lexicon_tags_11 },
    { 0xff7614c0, SUC_lexicon_tags_15 },
    { 0xbed614bf, SUC_lexicon_tags_8 },
    { 0x36d614c2, SUC_lexicon_tags_43 },
    { 0xd67614c3, SUC_lexicon_tags_27 },
    { 0xa7d614c4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3614ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65614ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7f614d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8f9614d2, SUC_lexicon_tags_12 },
    { 0x175614d3, SUC_lexicon_tags_38 },
    { 0x22f614d2, SUC_lexicon_tags_25 },
    { 0x1614d5, SUC_lexicon_tags_49 },
    { 0x36f614d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b614dc, SUC_lexicon_tags_2 },
    { 0x67f614dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x723614e0, SUC_lexicon_tags_7 },
    { 0xcf9614e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5b614e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x573614e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbd614e7, SUC_lexicon_tags_37 },
    { 0x927614e7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11614ee, SUC_lexicon_tags_12 },
    { 0x8e3614ef, SUC_lexicon_tags_23 },
    { 0xa37614f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x247614f2, SUC_lexicon_tags_38 },
    { 0xc85614f3, SUC_lexicon_tags_27 },
    { 0x4eb614f4, SUC_lexicon_tags_37 },
    { 0x513614f5, SUC_lexicon_tags_48 },
    { 0xae3614f2, SUC_lexicon_tags_275 },
    { 0x98b614f5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd614fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb61500, SUC_lexicon_tags_45 },
    { 0x26161501, SUC_lexicon_tags_45 },
    { 0xe8761501, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6150b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb6150f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b61512, SUC_lexicon_tags_17 },
    { 0x78d61513, SUC_lexicon_tags_14 },
    { 0xc5961514, SUC_lexicon_tags_28 },
    { 0xeff61514, SUC_lexicon_tags_8 },
    { 0x48561516, SUC_lexicon_tags_104 },
    { 0x92761514, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1016151a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7176151f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7361524, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f61527, SUC_lexicon_tags_12 },
    { 0x4a561528, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d6152b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd96152d, SUC_lexicon_tags_5 },
    { 0x18f6152e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb2161530, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6b61532, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x31361535, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7696153d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63361540, SUC_lexicon_tags_4 },
    { 0xac361541, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x77b61543, SUC_lexicon_tags_14 },
    { 0xf7961544, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd561547, SUC_lexicon_tags_37 },
    { 0xbaf61548, SUC_lexicon_tags_0 },
    { 0xbf761548, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x43f6154b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7eb6154d, SUC_lexicon_tags_38 },
    { 0x3556154e, SUC_lexicon_tags_21 },
    { 0xeed6154e, SUC_lexicon_tags_12 },
    { 0x6516154d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d361552, SUC_lexicon_tags_47 },
    { 0xb5561553, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db61556, SUC_lexicon_tags_7 },
    { 0x32b61557, SUC_lexicon_tags_33 },
    { 0xbef61558, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8496155c, SUC_lexicon_tags_17 },
    { 0xba56155d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b61561, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce361563, SUC_lexicon_tags_27 },
    { 0xf2761564, SUC_lexicon_tags_2 },
    { 0x35b61565, SUC_lexicon_tags_12 },
    { 0xfc761566, SUC_lexicon_tags_38 },
    { 0x97161564, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2316156c, SUC_lexicon_tags_60 },
    { 0x5756156c, SUC_lexicon_tags_19 },
    { 0x7056156e, SUC_lexicon_tags_12 },
    { 0x9a36156e, SUC_lexicon_tags_47 },
    { 0x88d61570, SUC_lexicon_tags_12 },
    { 0x2a361570, SUC_lexicon_tags_66 },
    { 0x1d161571, SUC_lexicon_tags_16 },
    { 0xc161570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9961575, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b6157a, SUC_lexicon_tags_12 },
    { 0xfef6157a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4161582, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce761584, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16961588, SUC_lexicon_tags_0 },
    { 0x79f61589, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29b6158b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59961590, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd761593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64d61595, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75b61597, SUC_lexicon_tags_1 },
    { 0xef361597, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe896159b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85615a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xed5615a8, SUC_lexicon_tags_27 },
    { 0x61615a8, SUC_lexicon_tags_37 },
    { 0x187615aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7615ad, SUC_lexicon_tags_15 },
    { 0xfbf615ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5615b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc9615b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3615b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe8d615b9, SUC_lexicon_tags_14 },
    { 0x393615b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb1615bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc83615be, SUC_lexicon_tags_10 },
    { 0x465615be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x691615c2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d615c7, SUC_lexicon_tags_25 },
    { 0x98b615c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cd615ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd615ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cb615d0, SUC_lexicon_tags_49 },
    { 0x819615d0, SUC_lexicon_tags_17 },
    { 0x545615d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629615d8, SUC_lexicon_tags_10 },
    { 0x7ab615d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b615dc, SUC_lexicon_tags_34 },
    { 0x9f615dd, SUC_lexicon_tags_14 },
    { 0xb47615de, SUC_lexicon_tags_19 },
    { 0xa55615de, SUC_lexicon_tags_17 },
    { 0x31d615e0, SUC_lexicon_tags_37 },
    { 0x33b615e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d615e4, SUC_lexicon_tags_17 },
    { 0x68f615e5, SUC_lexicon_tags_25 },
    { 0x777615e5, SUC_lexicon_tags_2 },
    { 0x23f615e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25615ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bf615ee, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7615f3, SUC_lexicon_tags_10 },
    { 0x9f615f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x621615f6, SUC_lexicon_tags_71 },
    { 0xbf1615f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f615fa, SUC_lexicon_tags_11 },
    { 0x99615fb, SUC_lexicon_tags_9 },
    { 0xf09615fc, SUC_lexicon_tags_30 },
    { 0x6a9615fc, SUC_lexicon_tags_17 },
    { 0xffb615fe, SUC_lexicon_tags_7 },
    { 0x3c1615fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76361601, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x61361603, SUC_lexicon_tags_28 },
    { 0x1361604, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5961606, SUC_lexicon_tags_11 },
    { 0x7db61606, SUC_lexicon_tags_25 },
    { 0x48d61608, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7156160a, SUC_lexicon_tags_27 },
    { 0xd9d6160b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c16160d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d61618, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa76161a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d96161c, SUC_lexicon_tags_42 },
    { 0xcf36161c, SUC_lexicon_tags_70 },
    { 0x4e56161e, SUC_lexicon_tags_113 },
    { 0xd36161f, SUC_lexicon_tags_12 },
    { 0x6bf6161c, SUC_lexicon_tags_1 },
    { 0x19961620, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51561623, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd61626, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5916162b, SUC_lexicon_tags_10 },
    { 0x7a36162b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6876162e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd361639, SUC_lexicon_tags_17 },
    { 0x42f6163a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc276163f, SUC_lexicon_tags_25 },
    { 0xa9f61640, SUC_lexicon_tags_34 },
    { 0x17361641, SUC_lexicon_tags_17 },
    { 0x21f61641, SUC_lexicon_tags_1 },
    { 0x57f61643, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84561647, SUC_lexicon_tags_33 },
    { 0xa9961648, SUC_lexicon_tags_26 },
    { 0xf361649, SUC_lexicon_tags_12 },
    { 0xcf761649, SUC_lexicon_tags_35 },
    { 0xd976164b, SUC_lexicon_tags_1 },
    { 0x4c56164b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d16164e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2961650, SUC_lexicon_tags_4 },
    { 0xf61650, SUC_lexicon_tags_6 },
    { 0xd7f61652, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x39161654, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x59161656, SUC_lexicon_tags_31 },
    { 0x81f61656, SUC_lexicon_tags_11 },
    { 0x79f61658, SUC_lexicon_tags_25 },
    { 0x7f961657, SUC_lexicon_tags_16 },
    { 0x72b61658, SUC_lexicon_tags_17 },
    { 0xad16165b, SUC_lexicon_tags_37 },
    { 0x16761658, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x45d6165e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d361663, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f61667, SUC_lexicon_tags_15 },
    { 0x1f61668, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a76166a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb376166f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbdd61671, SUC_lexicon_tags_21 },
    { 0xd1561672, SUC_lexicon_tags_14 },
    { 0x38d61672, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a961676, SUC_lexicon_tags_17 },
    { 0x8cb61677, SUC_lexicon_tags_23 },
    { 0xde361677, SUC_lexicon_tags_47 },
    { 0xfb361679, SUC_lexicon_tags_9 },
    { 0x86761676, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab6167d, SUC_lexicon_tags_60 },
    { 0xe36167e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad61683, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2761687, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70961689, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd8f6168b, SUC_lexicon_tags_37 },
    { 0x28f6168c, SUC_lexicon_tags_43 },
    { 0xb016168d, SUC_lexicon_tags_118 },
    { 0x58b6168c, SUC_lexicon_tags_27 },
    { 0xf016168f, SUC_lexicon_tags_17 },
    { 0x80b6168d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19b61692, SUC_lexicon_tags_14 },
    { 0x74f61693, SUC_lexicon_tags_6 },
    { 0xfcd61694, SUC_lexicon_tags_23 },
    { 0x5761693, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe761697, SUC_lexicon_tags_46 },
    { 0xab61698, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x16169a, SUC_lexicon_tags_14 },
    { 0x1736169b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2af6169d, SUC_lexicon_tags_15 },
    { 0x7d96169d, SUC_lexicon_tags_49 },
    { 0x4916169f, SUC_lexicon_tags_57 },
    { 0xf9d616a0, SUC_lexicon_tags_17 },
    { 0x687616a0, SUC_lexicon_tags_16 },
    { 0x1c36169e, SUC_lexicon_tags_37 },
    { 0xe35616a3, SUC_lexicon_tags_1 },
    { 0x6d9616a4, SUC_lexicon_tags_49 },
    { 0x5cb616a5, SUC_lexicon_tags_8 },
    { 0x39b6169d, SUC_lexicon_tags_13 },
    { 0x55b616a2, SUC_lexicon_tags_12 },
    { 0x8b7616a6, SUC_lexicon_tags_11 },
    { 0x3a7616a9, SUC_lexicon_tags_2 },
    { 0xa99616a9, SUC_lexicon_tags_30 },
    { 0xbd7616aa, SUC_lexicon_tags_46 },
    { 0xb0f616a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x581616af, SUC_lexicon_tags_2 },
    { 0x973616b0, SUC_lexicon_tags_9 },
    { 0x5d7616b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x661616b3, SUC_lexicon_tags_29 },
    { 0xa75616b4, SUC_lexicon_tags_37 },
    { 0x5cb616b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923616b9, SUC_lexicon_tags_25 },
    { 0xc71616ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c3616bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x645616be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a5616c0, SUC_lexicon_tags_47 },
    { 0x57616c1, SUC_lexicon_tags_45 },
    { 0x229616c2, SUC_lexicon_tags_28 },
    { 0xce3616c3, SUC_lexicon_tags_12 },
    { 0xff3616c4, SUC_lexicon_tags_32 },
    { 0x9ab616c3, SUC_lexicon_tags_17 },
    { 0xd91616c1, SUC_lexicon_tags_118 },
    { 0x433616c7, SUC_lexicon_tags_1 },
    { 0x1b7616c4, SUC_lexicon_tags_1 },
    { 0xc0f616c9, SUC_lexicon_tags_97 },
    { 0xb95616c3, SUC_lexicon_tags_15 },
    { 0x731616c4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x847616ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3616d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7616d9, SUC_lexicon_tags_26 },
    { 0xa4f616da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5616df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05616e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9616e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x615616e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x335616eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x651616ed, SUC_lexicon_tags_40 },
    { 0x8ff616ee, SUC_lexicon_tags_25 },
    { 0x34b616ef, SUC_lexicon_tags_1 },
    { 0x847616f0, SUC_lexicon_tags_15 },
    { 0xaab616f0, SUC_lexicon_tags_43 },
    { 0x8b5616f2, SUC_lexicon_tags_7 },
    { 0xec7616f2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3616f6, SUC_lexicon_tags_15 },
    { 0x5b9616f7, SUC_lexicon_tags_53 },
    { 0x7cd616f8, SUC_lexicon_tags_4 },
    { 0xc05616f7, SUC_lexicon_tags_49 },
    { 0x5ef616fa, SUC_lexicon_tags_11 },
    { 0x9e7616f7, SUC_lexicon_tags_63 },
    { 0x5bf616f8, SUC_lexicon_tags_49 },
    { 0xad7616fd, SUC_lexicon_tags_14 },
    { 0x75616fe, SUC_lexicon_tags_0 },
    { 0xef7616fd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa3961701, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x70561704, SUC_lexicon_tags_34 },
    { 0x6ad61705, SUC_lexicon_tags_2 },
    { 0xcf761706, SUC_lexicon_tags_12 },
    { 0xb8b61706, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d6170a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8361713, SUC_lexicon_tags_19 },
    { 0x7761713, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52b61716, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb4b61718, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb16171c, SUC_lexicon_tags_10 },
    { 0xe196171d, SUC_lexicon_tags_8 },
    { 0x36f6171c, SUC_lexicon_tags_38 },
    { 0x7156171c, SUC_lexicon_tags_7 },
    { 0x316171f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd61726, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21161728, SUC_lexicon_tags_16 },
    { 0x1fb61729, SUC_lexicon_tags_15 },
    { 0x7216172a, SUC_lexicon_tags_37 },
    { 0x5e36172b, SUC_lexicon_tags_25 },
    { 0xf8d6172c, SUC_lexicon_tags_47 },
    { 0x6b16172d, SUC_lexicon_tags_1 },
    { 0x7416172e, SUC_lexicon_tags_68 },
    { 0xd2f6172c, SUC_lexicon_tags_37 },
    { 0x5361730, SUC_lexicon_tags_31 },
    { 0x7e76172b, SUC_lexicon_tags_43 },
    { 0x29f61731, SUC_lexicon_tags_46 },
    { 0x5ef6172f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7961735, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcdb61737, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b6173e, SUC_lexicon_tags_11 },
    { 0xdd16173f, SUC_lexicon_tags_17 },
    { 0xded61740, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x99561742, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87361744, SUC_lexicon_tags_37 },
    { 0x3b61745, SUC_lexicon_tags_63 },
    { 0xc6f61744, SUC_lexicon_tags_27 },
    { 0xc0161744, SUC_lexicon_tags_31 },
    { 0x27761748, SUC_lexicon_tags_11 },
    { 0xadd61749, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2896174b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d61755, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xead61757, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a76175b, SUC_lexicon_tags_25 },
    { 0x6436175b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba761760, SUC_lexicon_tags_17 },
    { 0x3761761, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a961766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8276176f, SUC_lexicon_tags_82 },
    { 0x4b16176f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9d61772, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0xc1961774, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x36161776, SUC_lexicon_tags_25 },
    { 0x92361777, SUC_lexicon_tags_7 },
    { 0x29b61776, SUC_lexicon_tags_81 },
    { 0x69f61778, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b6177e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47361783, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe3961785, SUC_lexicon_tags_45 },
    { 0xcef61785, SUC_lexicon_tags_77 },
    { 0x22b61786, SUC_lexicon_tags_14 },
    { 0x9bb61787, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x90b6178a, SUC_lexicon_tags_60 },
    { 0x2256178a, SUC_lexicon_tags_30 },
    { 0x1b76178c, SUC_lexicon_tags_31 },
    { 0xc036178d, SUC_lexicon_tags_11 },
    { 0xfef6178e, SUC_lexicon_tags_37 },
    { 0x9e36178f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b61792, SUC_lexicon_tags_25 },
    { 0xb9d61793, SUC_lexicon_tags_49 },
    { 0xc3d61792, SUC_lexicon_tags_1 },
    { 0xc9b61795, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab961797, SUC_lexicon_tags_45 },
    { 0x69361797, SUC_lexicon_tags_16 },
    { 0xb7f61797, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3136179f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x63b617a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f617a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5617a8, SUC_lexicon_tags_45 },
    { 0x85b617a8, SUC_lexicon_tags_12 },
    { 0xc41617aa, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b617af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93617b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed9617b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3617b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdef617ba, SUC_lexicon_tags_1 },
    { 0x3c9617bb, SUC_lexicon_tags_87 },
    { 0xa51617bc, SUC_lexicon_tags_79 },
    { 0xdc5617ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe5b617bf, SUC_lexicon_tags_17 },
    { 0xb27617c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11617c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431617d1, SUC_lexicon_tags_37 },
    { 0xfc1617d2, SUC_lexicon_tags_7 },
    { 0x9e5617d2, SUC_lexicon_tags_14 },
    { 0x88f617d2, SUC_lexicon_tags_34 },
    { 0xdb3617d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb617d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5f617db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82b617dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd617e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d617eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77617ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x319617f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b617f8, SUC_lexicon_tags_27 },
    { 0xf99617f8, SUC_lexicon_tags_8 },
    { 0x6fb617fa, SUC_lexicon_tags_26 },
    { 0xe4d617fb, SUC_lexicon_tags_6 },
    { 0x255617fc, SUC_lexicon_tags_7 },
    { 0xc5617f8, SUC_lexicon_tags_1 },
    { 0xb1617fe, SUC_lexicon_tags_37 },
    { 0xd7617ff, SUC_lexicon_tags_4 },
    { 0xdc3617fe, SUC_lexicon_tags_37 },
    { 0x95961801, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3961803, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9d61805, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b6180a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2876180c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab56181a, SUC_lexicon_tags_29 },
    { 0xbfb6181a, SUC_lexicon_tags_15 },
    { 0xf016181a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4996181f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8961822, SUC_lexicon_tags_8 },
    { 0xad161823, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe096182c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f361830, SUC_lexicon_tags_19 },
    { 0xff561831, SUC_lexicon_tags_7 },
    { 0xda161832, SUC_lexicon_tags_63 },
    { 0x20561833, SUC_lexicon_tags_17 },
    { 0xeed61834, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4856183b, SUC_lexicon_tags_39 },
    { 0x7556183b, SUC_lexicon_tags_7 },
    { 0xcdf6183d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb61840, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8961842, SUC_lexicon_tags_37 },
    { 0xfe561842, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3856184a, SUC_lexicon_tags_37 },
    { 0x4296184a, SUC_lexicon_tags_12 },
    { 0xae56184c, SUC_lexicon_tags_4 },
    { 0x9cd6184c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9696184f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8361855, SUC_lexicon_tags_39 },
    { 0xf0761856, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81361858, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x376185a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf916185c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd61860, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb61865, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b6186c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1136186f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x92f61871, SUC_lexicon_tags_47 },
    { 0x7b961871, SUC_lexicon_tags_64 },
    { 0xf2961873, SUC_lexicon_tags_17 },
    { 0x41f61873, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad61877, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d6187d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf576187f, SUC_lexicon_tags_38 },
    { 0x19761880, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d61887, SUC_lexicon_tags_7 },
    { 0xb4561887, SUC_lexicon_tags_38 },
    { 0xc7761889, SUC_lexicon_tags_13 },
    { 0xdcb61889, SUC_lexicon_tags_15 },
    { 0x2316188b, SUC_lexicon_tags_17 },
    { 0xc7d6188c, SUC_lexicon_tags_25 },
    { 0xff61888, SUC_lexicon_tags_39 },
    { 0xf176188d, SUC_lexicon_tags_25 },
    { 0xea96188f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8961892, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe161895, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f61899, SUC_lexicon_tags_396 },
    { 0x1d961899, SUC_lexicon_tags_15 },
    { 0x6656189b, SUC_lexicon_tags_44 },
    { 0xe16189c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7618a1, SUC_lexicon_tags_17 },
    { 0xb19618a2, SUC_lexicon_tags_94 },
    { 0x38f618a3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247618a7, SUC_lexicon_tags_83 },
    { 0x239618a7, SUC_lexicon_tags_12 },
    { 0x913618a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1618ab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d618b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d618b9, SUC_lexicon_tags_31 },
    { 0x477618b9, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x2e5618bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd618c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69618c8, SUC_lexicon_tags_33 },
    { 0x62b618c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d618d4, SUC_lexicon_tags_30 },
    { 0xbab618d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27618d7, SUC_lexicon_tags_10 },
    { 0x4f618d7, SUC_lexicon_tags_4 },
    { 0x7f7618d9, SUC_lexicon_tags_2 },
    { 0xcb3618d9, SUC_lexicon_tags_47 },
    { 0x533618db, SUC_lexicon_tags_0 },
    { 0x837618dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f618df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25618e4, SUC_lexicon_tags_33 },
    { 0x4a5618e4, SUC_lexicon_tags_25 },
    { 0xb01618e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaa3618e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f618ec, SUC_lexicon_tags_37 },
    { 0x433618ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x973618f0, SUC_lexicon_tags_2 },
    { 0x5a7618f1, SUC_lexicon_tags_299 },
    { 0xa37618f2, SUC_lexicon_tags_55 },
    { 0xf83618f3, SUC_lexicon_tags_23 },
    { 0xc9618f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f1618f6, SUC_lexicon_tags_917 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d618fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1b618fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a761901, SUC_lexicon_tags_33 },
    { 0x7d161902, SUC_lexicon_tags_12 },
    { 0x7bd61902, SUC_lexicon_tags_16 },
    { 0x31161901, SUC_lexicon_tags_14 },
    { 0xe7d61905, SUC_lexicon_tags_12 },
    { 0x4fd61905, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c36190d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54161919, SUC_lexicon_tags_6 },
    { 0x9c561919, SUC_lexicon_tags_11 },
    { 0xe136191b, SUC_lexicon_tags_37 },
    { 0xd156191c, SUC_lexicon_tags_37 },
    { 0xcdf6191c, SUC_lexicon_tags_15 },
    { 0x8396191e, SUC_lexicon_tags_27 },
    { 0x3516191f, SUC_lexicon_tags_17 },
    { 0x1556191e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff61924, SUC_lexicon_tags_7 },
    { 0xf5b61925, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9761928, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc36192d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14961932, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2b361934, SUC_lexicon_tags_12 },
    { 0x86961935, SUC_lexicon_tags_1 },
    { 0xaf761934, SUC_lexicon_tags_14 },
    { 0xfb561937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2796193e, SUC_lexicon_tags_2 },
    { 0xc76193f, SUC_lexicon_tags_2 },
    { 0x23761940, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x95d61942, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2961945, SUC_lexicon_tags_25 },
    { 0x62161946, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc161949, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8f6194b, SUC_lexicon_tags_17 },
    { 0x5756194b, SUC_lexicon_tags_11 },
    { 0x37f6194b, SUC_lexicon_tags_10 },
    { 0x1eb6194e, SUC_lexicon_tags_103 },
    { 0xe856194f, SUC_lexicon_tags_6 },
    { 0x1356194c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9b61952, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9761956, SUC_lexicon_tags_11 },
    { 0xef761957, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xc056195a, SUC_lexicon_tags_1 },
    { 0xbf56195b, SUC_lexicon_tags_38 },
    { 0x1516195c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9196195f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79361963, SUC_lexicon_tags_13 },
    { 0x97361963, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb61967, SUC_lexicon_tags_45 },
    { 0x94b61968, SUC_lexicon_tags_2 },
    { 0xe2761968, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e56196b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd561970, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0961973, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92561975, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82161978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef6197d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d61980, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf361982, SUC_lexicon_tags_45 },
    { 0x21b61983, SUC_lexicon_tags_47 },
    { 0xdcb61984, SUC_lexicon_tags_5 },
    { 0x4e761985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a361989, SUC_lexicon_tags_29 },
    { 0x26f6198a, SUC_lexicon_tags_9 },
    { 0xd0f6198b, SUC_lexicon_tags_19 },
    { 0x98d61989, SUC_lexicon_tags_2 },
    { 0x9bd6198d, SUC_lexicon_tags_6 },
    { 0x1f76198e, SUC_lexicon_tags_52 },
    { 0x4a76198f, SUC_lexicon_tags_92 },
    { 0xed961990, SUC_lexicon_tags_174 },
    { 0xe776198f, SUC_lexicon_tags_14 },
    { 0x7c16198e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a56199b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b6199f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107619a5, SUC_lexicon_tags_11 },
    { 0x98b619a5, SUC_lexicon_tags_11 },
    { 0xed7619a7, SUC_lexicon_tags_5 },
    { 0x6f5619a8, SUC_lexicon_tags_66 },
    { 0x20f619a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf619ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385619b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf01619b4, SUC_lexicon_tags_68 },
    { 0x4f1619b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x797619b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d619c3, SUC_lexicon_tags_47 },
    { 0x6d9619c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb619c7, SUC_lexicon_tags_12 },
    { 0xfa7619c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7619cc, SUC_lexicon_tags_39 },
    { 0x143619cd, SUC_lexicon_tags_13 },
    { 0x4d7619cc, SUC_lexicon_tags_4 },
    { 0x12b619cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5619d2, SUC_lexicon_tags_48 },
    { 0x807619d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xff619d5, SUC_lexicon_tags_19 },
    { 0xa4f619d5, SUC_lexicon_tags_37 },
    { 0xed7619d7, SUC_lexicon_tags_107 },
    { 0xe0d619d5, SUC_lexicon_tags_43 },
    { 0x84b619d9, SUC_lexicon_tags_17 },
    { 0x81619d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99619e3, SUC_lexicon_tags_418 },
    { 0x29f619e4, SUC_lexicon_tags_25 },
    { 0x527619e5, SUC_lexicon_tags_6 },
    { 0xde5619e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145619eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7619ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747619f5, SUC_lexicon_tags_47 },
    { 0xb13619f6, SUC_lexicon_tags_160 },
    { 0x227619f6, SUC_lexicon_tags_11 },
    { 0x80b619f7, SUC_lexicon_tags_2 },
    { 0x9bd619f8, SUC_lexicon_tags_11 },
    { 0x2d619fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc961a03, SUC_lexicon_tags_4 },
    { 0x51d61a04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49161a07, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x39761a0a, SUC_lexicon_tags_17 },
    { 0x99f61a0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14561a14, SUC_lexicon_tags_2 },
    { 0xb6361a15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf61a1a, SUC_lexicon_tags_23 },
    { 0x4cb61a1a, SUC_lexicon_tags_17 },
    { 0x4e961a1a, SUC_lexicon_tags_43 },
    { 0xd2b61a1d, SUC_lexicon_tags_37 },
    { 0x4bf61a1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f61a21, SUC_lexicon_tags_15 },
    { 0x8b961a22, SUC_lexicon_tags_14 },
    { 0x67b61a23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d61a27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b61a2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5561a2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12561a32, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff61a37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40161a3c, SUC_lexicon_tags_32 },
    { 0x82361a3d, SUC_lexicon_tags_60 },
    { 0xccd61a3d, SUC_lexicon_tags_87 },
    { 0xcfd61a3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d761a43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x66761a46, SUC_lexicon_tags_15 },
    { 0xac361a46, SUC_lexicon_tags_11 },
    { 0x3b761a48, SUC_lexicon_tags_40 },
    { 0x99761a49, SUC_lexicon_tags_28 },
    { 0xaa761a49, SUC_lexicon_tags_37 },
    { 0x8b961a4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d61a4f, SUC_lexicon_tags_8 },
    { 0xee361a50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52f61a52, SUC_lexicon_tags_6 },
    { 0x93761a52, SUC_lexicon_tags_23 },
    { 0xd161a53, SUC_lexicon_tags_33 },
    { 0x1f161a55, SUC_lexicon_tags_26 },
    { 0xd6d61a56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0161a5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefb61a5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83361a61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47f61a63, SUC_lexicon_tags_1 },
    { 0xfe761a63, SUC_lexicon_tags_15 },
    { 0xb7961a65, SUC_lexicon_tags_17 },
    { 0x15d61a66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1361a68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b61a6c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2561a6f, SUC_lexicon_tags_66 },
    { 0xd0b61a70, SUC_lexicon_tags_18 },
    { 0x78961a71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57561a73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x58961a75, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x64761a77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95561a7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c561a7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6fb61a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c561a85, SUC_lexicon_tags_17 },
    { 0x38f61a85, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d61a8d, SUC_lexicon_tags_17 },
    { 0x15761a8e, SUC_lexicon_tags_60 },
    { 0x34f61a8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27161a93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4b361a95, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a961a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d961a99, SUC_lexicon_tags_49 },
    { 0x53761a99, SUC_lexicon_tags_149 },
    { 0xbd961a9b, SUC_lexicon_tags_8 },
    { 0x18561a9c, SUC_lexicon_tags_33 },
    { 0x8561a9a, SUC_lexicon_tags_25 },
    { 0xe5d61a9e, SUC_lexicon_tags_14 },
    { 0xc0961a9f, SUC_lexicon_tags_12 },
    { 0x20361aa0, SUC_lexicon_tags_7 },
    { 0x59b61a99, SUC_lexicon_tags_15 },
    { 0xd4361a9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43961aaf, SUC_lexicon_tags_45 },
    { 0xd6d61ab0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe2d61ab2, SUC_lexicon_tags_15 },
    { 0x89761ab3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f761ab7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31961ac0, SUC_lexicon_tags_2 },
    { 0x69f61ac1, SUC_lexicon_tags_16 },
    { 0x22d61ac2, SUC_lexicon_tags_279 },
    { 0x2f361ac2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x18961ac5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b61ac8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f61ace, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7ef61ad0, SUC_lexicon_tags_31 },
    { 0x71361ad1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ad61ad3, SUC_lexicon_tags_0 },
    { 0x66961ad3, SUC_lexicon_tags_0 },
    { 0x55561ad5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf761ad9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x2d361adb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f761ae0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab61ae5, SUC_lexicon_tags_42 },
    { 0xdfd61ae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xffb61ae8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d61aed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c761af0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6961af2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x14561af4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x53f61af6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82761afb, SUC_lexicon_tags_27 },
    { 0x65361afb, SUC_lexicon_tags_25 },
    { 0x24561afc, SUC_lexicon_tags_25 },
    { 0x2ed61afe, SUC_lexicon_tags_31 },
    { 0xbcd61aff, SUC_lexicon_tags_11 },
    { 0x70f61aff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3ef61b02, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd9961b04, SUC_lexicon_tags_26 },
    { 0x5f761b05, SUC_lexicon_tags_47 },
    { 0xf1361b05, SUC_lexicon_tags_37 },
    { 0xb0561b07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0961b0c, SUC_lexicon_tags_11 },
    { 0xc161b0c, SUC_lexicon_tags_61 },
    { 0x2f961b0d, SUC_lexicon_tags_6 },
    { 0x34361b0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29161b13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3361b16, SUC_lexicon_tags_46 },
    { 0x54161b16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc961b1b, SUC_lexicon_tags_37 },
    { 0x4b761b1b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x15b61b1e, SUC_lexicon_tags_15 },
    { 0xeaf61b1f, SUC_lexicon_tags_21 },
    { 0xe9d61b1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3761b23, SUC_lexicon_tags_12 },
    { 0xdf561b23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b161b35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b61b3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b61b42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5561b45, SUC_lexicon_tags_65 },
    { 0x54161b45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8561b4e, SUC_lexicon_tags_17 },
    { 0xd961b4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15161b52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c561b5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x99361b5e, SUC_lexicon_tags_15 },
    { 0x99d61b5f, SUC_lexicon_tags_30 },
    { 0xbc961b5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af61b63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb61b66, SUC_lexicon_tags_47 },
    { 0x72361b67, SUC_lexicon_tags_0 },
    { 0x8f61b68, SUC_lexicon_tags_27 },
    { 0x59d61b69, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0x2e361b6b, SUC_lexicon_tags_19 },
    { 0xfe161b6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f61b6f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab61b72, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5361b74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9ad61b76, SUC_lexicon_tags_19 },
    { 0x95961b76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4961b79, SUC_lexicon_tags_38 },
    { 0xf5f61b7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92961b7c, SUC_lexicon_tags_64 },
    { 0x9db61b7d, SUC_lexicon_tags_37 },
    { 0x31f61b7c, SUC_lexicon_tags_9 },
    { 0x1c161b7d, SUC_lexicon_tags_46 },
    { 0x3eb61b7d, SUC_lexicon_tags_27 },
    { 0xbc161b81, SUC_lexicon_tags_10 },
    { 0x57361b82, SUC_lexicon_tags_34 },
    { 0x6db61b83, SUC_lexicon_tags_4 },
    { 0x1d61b84, SUC_lexicon_tags_7 },
    { 0xb6d61b82, SUC_lexicon_tags_43 },
    { 0x8fb61b86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb2d61b88, SUC_lexicon_tags_6 },
    { 0x10561b88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d61b8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd961b93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6d61b95, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d61b9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1bd61ba0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d61ba3, SUC_lexicon_tags_15 },
    { 0xf4561ba4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f61ba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f761bac, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0xc5161bae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb61bb3, SUC_lexicon_tags_9 },
    { 0xeef61bb4, SUC_lexicon_tags_17 },
    { 0x92561bb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75561bb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8761bbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c761bbd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee761bbf, SUC_lexicon_tags_12 },
    { 0x1fd61bc0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc561bc3, SUC_lexicon_tags_103 },
    { 0xed161bc4, SUC_lexicon_tags_37 },
    { 0x4d561bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cb61bc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc961bc9, SUC_lexicon_tags_17 },
    { 0xe761bc9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f61bce, SUC_lexicon_tags_19 },
    { 0xe9f61bcf, SUC_lexicon_tags_11 },
    { 0xc8d61bcf, SUC_lexicon_tags_12 },
    { 0x6f561bd1, SUC_lexicon_tags_15 },
    { 0x2b961bd2, SUC_lexicon_tags_65 },
    { 0xd6961bd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfd61bd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d61bd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x761bda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d61bde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b161be0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9f61be2, SUC_lexicon_tags_15 },
    { 0xbe161be2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcf61be5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe9961be7, SUC_lexicon_tags_2 },
    { 0x60b61be8, SUC_lexicon_tags_39 },
    { 0x34761be9, SUC_lexicon_tags_23 },
    { 0xc7b61be9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9561bf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9761bf2, SUC_lexicon_tags_7 },
    { 0x7ad61bf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10b61bf5, SUC_lexicon_tags_30 },
    { 0xf6961bf6, SUC_lexicon_tags_17 },
    { 0xc5961bf7, SUC_lexicon_tags_37 },
    { 0xad61bf8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf6161bfa, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7361bfd, SUC_lexicon_tags_1 },
    { 0x24761bfe, SUC_lexicon_tags_14 },
    { 0xa2f61bfe, SUC_lexicon_tags_30 },
    { 0x78761c00, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3761c02, SUC_lexicon_tags_1 },
    { 0xdc361c03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f61c08, SUC_lexicon_tags_37 },
    { 0x87961c09, SUC_lexicon_tags_45 },
    { 0xf6f61c0a, SUC_lexicon_tags_11 },
    { 0x40f61c09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef61c0e, SUC_lexicon_tags_60 },
    { 0xd6761c0e, SUC_lexicon_tags_1 },
    { 0x15b61c0e, SUC_lexicon_tags_21 },
    { 0x9d761c11, SUC_lexicon_tags_8 },
    { 0x8b61c12, SUC_lexicon_tags_45 },
    { 0xf1d61c11, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd61c19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40361c1b, SUC_lexicon_tags_2 },
    { 0x46f61c1b, SUC_lexicon_tags_29 },
    { 0xc761c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35561c24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8761c28, SUC_lexicon_tags_15 },
    { 0x1e561c29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43f61c2b, SUC_lexicon_tags_12 },
    { 0x82561c2c, SUC_lexicon_tags_66 },
    { 0x3ef61c2d, SUC_lexicon_tags_47 },
    { 0x1eb61c2b, SUC_lexicon_tags_29 },
    { 0x50161c2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b61c33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf561c37, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa9f61c39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd561c3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2d61c3d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x94761c40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52361c42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa761c44, SUC_lexicon_tags_1 },
    { 0xe2361c45, SUC_lexicon_tags_23 },
    { 0x9fb61c46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde561c48, SUC_lexicon_tags_27 },
    { 0x6b761c48, SUC_lexicon_tags_2 },
    { 0x24b61c4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfe361c4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d61c57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac161c5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf361c61, SUC_lexicon_tags_37 },
    { 0xfe561c61, SUC_lexicon_tags_0 },
    { 0xe2361c63, SUC_lexicon_tags_39 },
    { 0x62961c64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x16161c66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9961c6d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x41961c70, SUC_lexicon_tags_45 },
    { 0xa8161c70, SUC_lexicon_tags_26 },
    { 0x17161c72, SUC_lexicon_tags_6 },
    { 0xeef61c73, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23961c77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e561c7a, SUC_lexicon_tags_47 },
    { 0xc0561c7b, SUC_lexicon_tags_11 },
    { 0x8561c7c, SUC_lexicon_tags_45 },
    { 0xf2561c7b, SUC_lexicon_tags_15 },
    { 0x40561c7d, SUC_lexicon_tags_28 },
    { 0xeb961c7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64761c89, SUC_lexicon_tags_17 },
    { 0x2f161c89, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d61c8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd361c92, SUC_lexicon_tags_8 },
    { 0x31561c93, SUC_lexicon_tags_6 },
    { 0x99161c94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11f61c96, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x83161c98, SUC_lexicon_tags_13 },
    { 0xfa361c99, SUC_lexicon_tags_153 },
    { 0xa7f61c9a, SUC_lexicon_tags_49 },
    { 0xafd61c9b, SUC_lexicon_tags_11 },
    { 0xd3d61c99, SUC_lexicon_tags_27 },
    { 0xc6361c9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfd61c9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf561ca1, SUC_lexicon_tags_1 },
    { 0xa3b61ca2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8f61ca4, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d561ca8, SUC_lexicon_tags_0 },
    { 0x1e561ca9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f61cae, SUC_lexicon_tags_63 },
    { 0xc2161caf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3961cb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4961cb7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf9161cb9, SUC_lexicon_tags_12 },
    { 0xe1361cba, SUC_lexicon_tags_60 },
    { 0xf361cbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1961cbd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x83961cbf, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb761cc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f61ccc, SUC_lexicon_tags_40 },
    { 0xf2961ccd, SUC_lexicon_tags_17 },
    { 0x96561cce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8561cd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24761cd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb61cdc, SUC_lexicon_tags_56 },
    { 0x9f561cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc8561cdf, SUC_lexicon_tags_13 },
    { 0x24f61cdf, SUC_lexicon_tags_2 },
    { 0x38361cdf, SUC_lexicon_tags_2 },
    { 0x1a561ce1, SUC_lexicon_tags_43 },
    { 0x55b61ce1, SUC_lexicon_tags_17 },
    { 0xbad61ce4, SUC_lexicon_tags_12 },
    { 0x90361ce5, SUC_lexicon_tags_25 },
    { 0x1d761cdf, SUC_lexicon_tags_12 },
    { 0xe361cdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5961ce9, SUC_lexicon_tags_0 },
    { 0x64f61ce9, SUC_lexicon_tags_37 },
    { 0x46f61ceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61cf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f61cfa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df61d00, SUC_lexicon_tags_14 },
    { 0x21561d01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b361d04, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x4161d07, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d561d0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20b61d0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a161d0f, SUC_lexicon_tags_55 },
    { 0x81761d10, SUC_lexicon_tags_237 },
    { 0x8fb61d11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10961d13, SUC_lexicon_tags_2 },
    { 0x7dd61d14, SUC_lexicon_tags_15 },
    { 0x84761d15, SUC_lexicon_tags_25 },
    { 0x2b561d16, SUC_lexicon_tags_11 },
    { 0xc7b61d16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66161d1b, SUC_lexicon_tags_27 },
    { 0x50761d1c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x5f761d1e, SUC_lexicon_tags_26 },
    { 0x53761d1f, SUC_lexicon_tags_12 },
    { 0xd361d20, SUC_lexicon_tags_19 },
    { 0x52561d20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7dd61d23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d361d2b, SUC_lexicon_tags_11 },
    { 0x50f61d2c, SUC_lexicon_tags_1 },
    { 0x18361d2b, SUC_lexicon_tags_68 },
    { 0xf7961d2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49161d36, SUC_lexicon_tags_68 },
    { 0x61b61d37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54f61d39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xab561d3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f61d41, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf761d44, SUC_lexicon_tags_15 },
    { 0xf3361d45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea361d4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x46161d4e, SUC_lexicon_tags_43 },
    { 0x47361d4e, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xce361d51, SUC_lexicon_tags_2 },
    { 0xc7d61d52, SUC_lexicon_tags_17 },
    { 0x24961d53, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b61d56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd961d58, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52161d5a, SUC_lexicon_tags_11 },
    { 0xac561d5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bb61d5d, SUC_lexicon_tags_65 },
    { 0x77b61d5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa361d60, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37361d65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd3f61d67, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x73161d69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd61d6e, SUC_lexicon_tags_25 },
    { 0xbd761d6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c161d71, SUC_lexicon_tags_15 },
    { 0xea361d71, SUC_lexicon_tags_23 },
    { 0x1b61d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82961d76, SUC_lexicon_tags_15 },
    { 0x18f61d77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb361d79, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d61d7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x79561d80, SUC_lexicon_tags_9 },
    { 0x89961d80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59b61d83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f61d87, SUC_lexicon_tags_12 },
    { 0x27561d87, SUC_lexicon_tags_35 },
    { 0xd6d61d89, SUC_lexicon_tags_23 },
    { 0x4d561d89, SUC_lexicon_tags_23 },
    { 0x89b61d8b, SUC_lexicon_tags_439 },
    { 0x57961d8c, SUC_lexicon_tags_11 },
    { 0xf9561d8c, SUC_lexicon_tags_2 },
    { 0x6a361d8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce161d91, SUC_lexicon_tags_26 },
    { 0xab161d91, SUC_lexicon_tags_2 },
    { 0x5a761d92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62f61d95, SUC_lexicon_tags_1 },
    { 0x79961d96, SUC_lexicon_tags_37 },
    { 0x89b61d95, SUC_lexicon_tags_15 },
    { 0xbe961d96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7d61d9a, SUC_lexicon_tags_11 },
    { 0x8cb61d9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c361d9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad61da0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94961da2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59361da6, SUC_lexicon_tags_25 },
    { 0x30d61da7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5b61da9, SUC_lexicon_tags_16 },
    { 0xa361da9, SUC_lexicon_tags_17 },
    { 0x55361daa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf61dae, SUC_lexicon_tags_33 },
    { 0x60b61daf, SUC_lexicon_tags_47 },
    { 0x10d61db0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4161db2, SUC_lexicon_tags_11 },
    { 0x3161db3, SUC_lexicon_tags_27 },
    { 0x30961db3, SUC_lexicon_tags_29 },
    { 0xdb961db3, SUC_lexicon_tags_7 },
    { 0x91761db6, SUC_lexicon_tags_114 },
    { 0x48761db7, SUC_lexicon_tags_25 },
    { 0x7e561db7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b61dbb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d361dbf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdcd61dc1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16761dc5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc1b61dc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58561dcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d61dd3, SUC_lexicon_tags_12 },
    { 0xf4161dd4, SUC_lexicon_tags_27 },
    { 0x55561dd5, SUC_lexicon_tags_49 },
    { 0xa2d61dd6, SUC_lexicon_tags_29 },
    { 0xe4f61dd7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf61dda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b61ddd, SUC_lexicon_tags_12 },
    { 0x3f761dde, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2961de4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3561de6, SUC_lexicon_tags_1 },
    { 0x15161de6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b61df3, SUC_lexicon_tags_17 },
    { 0x69b61df3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4df61df6, SUC_lexicon_tags_27 },
    { 0x51161df7, SUC_lexicon_tags_17 },
    { 0x2f161df8, SUC_lexicon_tags_0 },
    { 0x92f61df9, SUC_lexicon_tags_68 },
    { 0x2b561df8, SUC_lexicon_tags_0 },
    { 0x17561dfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa561dfd, SUC_lexicon_tags_23 },
    { 0x73f61dfe, SUC_lexicon_tags_7 },
    { 0x9961dff, SUC_lexicon_tags_15 },
    { 0xc7561dfd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb4b61e02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3bf61e04, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49161e07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc161e0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x90161e12, SUC_lexicon_tags_45 },
    { 0x14b61e13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad761e16, SUC_lexicon_tags_2 },
    { 0x69f61e17, SUC_lexicon_tags_87 },
    { 0x44961e18, SUC_lexicon_tags_37 },
    { 0xb8761e19, SUC_lexicon_tags_17 },
    { 0xc0d61e1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x34561e1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d161e25, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f161e2b, SUC_lexicon_tags_45 },
    { 0x4b761e2b, SUC_lexicon_tags_12 },
    { 0xe2961e2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f61e30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e561e34, SUC_lexicon_tags_19 },
    { 0xc9161e34, SUC_lexicon_tags_19 },
    { 0xef761e34, SUC_lexicon_tags_68 },
    { 0x91d61e36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f61e3c, SUC_lexicon_tags_1 },
    { 0x6e561e3d, SUC_lexicon_tags_1 },
    { 0xdb161e3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x64b61e40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7d61e42, SUC_lexicon_tags_37 },
    { 0xb5961e43, SUC_lexicon_tags_11 },
    { 0x48b61e44, SUC_lexicon_tags_138 },
    { 0xf1561e45, SUC_lexicon_tags_55 },
    { 0xaff61e46, SUC_lexicon_tags_32 },
    { 0xe3f61e47, SUC_lexicon_tags_7 },
    { 0x1a361e47, SUC_lexicon_tags_45 },
    { 0x5d561e49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e961e4c, SUC_lexicon_tags_7 },
    { 0xef361e4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19361e50, SUC_lexicon_tags_12 },
    { 0xe9961e51, SUC_lexicon_tags_23 },
    { 0x93761e52, SUC_lexicon_tags_37 },
    { 0xccd61e52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b61e56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b161e5d, SUC_lexicon_tags_17 },
    { 0x4f761e5e, SUC_lexicon_tags_103 },
    { 0xb0161e5e, SUC_lexicon_tags_2 },
    { 0x35761e60, SUC_lexicon_tags_25 },
    { 0x76761e5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66161e67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d761e6a, SUC_lexicon_tags_26 },
    { 0x4b61e6b, SUC_lexicon_tags_21 },
    { 0xb6561e6c, SUC_lexicon_tags_17 },
    { 0x23761e6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b61e70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9d361e72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x88361e74, SUC_lexicon_tags_25 },
    { 0x24761e75, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb61e7c, SUC_lexicon_tags_17 },
    { 0x1b161e7d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf61e80, SUC_lexicon_tags_46 },
    { 0xd3161e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ff61e83, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x69961e86, SUC_lexicon_tags_39 },
    { 0xc961e87, SUC_lexicon_tags_2 },
    { 0x53561e87, SUC_lexicon_tags_188 },
    { 0xfff61e89, SUC_lexicon_tags_8 },
    { 0x6d561e89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf561e8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdb61e8f, SUC_lexicon_tags_37 },
    { 0xb4161e8f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d61e93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9bf61e95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25761e97, SUC_lexicon_tags_63 },
    { 0x9a761e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1761e9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d61ea0, SUC_lexicon_tags_17 },
    { 0x3e161ea1, SUC_lexicon_tags_174 },
    { 0xa6561ea0, SUC_lexicon_tags_15 },
    { 0x56361ea3, SUC_lexicon_tags_15 },
    { 0xb9161ea3, SUC_lexicon_tags_2 },
    { 0x9f961ea2, SUC_lexicon_tags_49 },
    { 0x13f61ea1, SUC_lexicon_tags_25 },
    { 0x58b61ea5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e361eab, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf61eae, SUC_lexicon_tags_49 },
    { 0xe7161eaf, SUC_lexicon_tags_60 },
    { 0x67b61eb0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xa6761eb2, SUC_lexicon_tags_34 },
    { 0x25361eb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b61eb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0361eb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cb61eba, SUC_lexicon_tags_12 },
    { 0x99561ebb, SUC_lexicon_tags_25 },
    { 0x84161ebc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f61ec1, SUC_lexicon_tags_87 },
    { 0x5bd61ec1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6fd61ec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68961ec6, SUC_lexicon_tags_12 },
    { 0x13d61ec7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f61ecb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f61ecf, SUC_lexicon_tags_60 },
    { 0xbe761ed0, SUC_lexicon_tags_17 },
    { 0x49361ed1, SUC_lexicon_tags_11 },
    { 0x77b61ed2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d61ed7, SUC_lexicon_tags_2 },
    { 0x75d61ed7, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xbed61eda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b61edd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4961edf, SUC_lexicon_tags_31 },
    { 0xd6561edf, SUC_lexicon_tags_194 },
    { 0xfbd61edf, SUC_lexicon_tags_1252 },
    { 0xaf161ee2, SUC_lexicon_tags_48 },
    { 0x3d161ee2, SUC_lexicon_tags_2 },
    { 0xb2761ee4, SUC_lexicon_tags_4 },
    { 0x4cf61ee5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14361eeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85961eee, SUC_lexicon_tags_37 },
    { 0xfab61eef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe561ef2, SUC_lexicon_tags_11 },
    { 0x9561ef3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b961ef6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db61efb, SUC_lexicon_tags_25 },
    { 0xef361efb, SUC_lexicon_tags_14 },
    { 0x72361efb, SUC_lexicon_tags_28 },
    { 0x26961efe, SUC_lexicon_tags_7 },
    { 0x4d761efe, SUC_lexicon_tags_14 },
    { 0x95d61efd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x93b61f02, SUC_lexicon_tags_25 },
    { 0x5fb61f03, SUC_lexicon_tags_26 },
    { 0x3f61f02, SUC_lexicon_tags_2 },
    { 0xb9961f05, SUC_lexicon_tags_7 },
    { 0x33b61f05, SUC_lexicon_tags_37 },
    { 0xdb161f07, SUC_lexicon_tags_47 },
    { 0xbf361f05, SUC_lexicon_tags_61 },
    { 0x1f161f09, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2761f0d, SUC_lexicon_tags_27 },
    { 0xcd561f0e, SUC_lexicon_tags_84 },
    { 0x14d61f0f, SUC_lexicon_tags_11 },
    { 0x57f61f10, SUC_lexicon_tags_11 },
    { 0xcdd61f11, SUC_lexicon_tags_63 },
    { 0xa3761f11, SUC_lexicon_tags_68 },
    { 0xf3561f13, SUC_lexicon_tags_37 },
    { 0xa3161f0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47361f1a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x70161f1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f61f24, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc8361f26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d61f2c, SUC_lexicon_tags_45 },
    { 0xb2d61f2d, SUC_lexicon_tags_30 },
    { 0x96161f2d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed61f33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d61f38, SUC_lexicon_tags_17 },
    { 0x77761f39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc4161f3b, SUC_lexicon_tags_77 },
    { 0xf6b61f3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1361f3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36361f43, SUC_lexicon_tags_45 },
    { 0xaeb61f43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c961f49, SUC_lexicon_tags_28 },
    { 0x76561f4a, SUC_lexicon_tags_25 },
    { 0xdd161f49, SUC_lexicon_tags_25 },
    { 0x1af61f4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf3561f4e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b61f51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf161f57, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2561f59, SUC_lexicon_tags_12 },
    { 0x15b61f5a, SUC_lexicon_tags_15 },
    { 0xe9561f59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc161f5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b61f61, SUC_lexicon_tags_8 },
    { 0xa2961f61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd561f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0761f6d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8561f6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x39361f71, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb6f61f73, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x79d61f75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61f7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c561f7c, SUC_lexicon_tags_15 },
    { 0x79961f7d, SUC_lexicon_tags_45 },
    { 0x7e561f7d, SUC_lexicon_tags_11 },
    { 0x7161f7f, SUC_lexicon_tags_17 },
    { 0x68161f7d, SUC_lexicon_tags_1 },
    { 0xdcd61f81, SUC_lexicon_tags_33 },
    { 0xc7361f82, SUC_lexicon_tags_136 },
    { 0xb3361f83, SUC_lexicon_tags_15 },
    { 0x2b961f83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2761f86, SUC_lexicon_tags_37 },
    { 0x8cf61f87, SUC_lexicon_tags_49 },
    { 0x50961f88, SUC_lexicon_tags_12 },
    { 0xed161f88, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63761f8f, SUC_lexicon_tags_26 },
    { 0x88d61f90, SUC_lexicon_tags_2 },
    { 0xd6161f91, SUC_lexicon_tags_1 },
    { 0xc6d61f91, SUC_lexicon_tags_55 },
    { 0xa0161f93, SUC_lexicon_tags_60 },
    { 0xc9f61f94, SUC_lexicon_tags_2 },
    { 0xbb61f95, SUC_lexicon_tags_41 },
    { 0x77761f96, SUC_lexicon_tags_6 },
    { 0x37561f94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1ed61f99, SUC_lexicon_tags_27 },
    { 0xb7961f9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad61f9f, SUC_lexicon_tags_25 },
    { 0x23f61fa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb161fa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf61fa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21361faa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f61fb1, SUC_lexicon_tags_49 },
    { 0xccb61fb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6b61fb4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2f61fb6, SUC_lexicon_tags_28 },
    { 0x9d61fb6, SUC_lexicon_tags_45 },
    { 0xc5361fb8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x161fbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c361fbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9961fc1, SUC_lexicon_tags_6 },
    { 0xb8961fc2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b61fcb, SUC_lexicon_tags_6 },
    { 0xc9d61fcb, SUC_lexicon_tags_17 },
    { 0xf3d61fcd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd61fd2, SUC_lexicon_tags_18 },
    { 0xd7f61fd3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x67961fd5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b61fdb, SUC_lexicon_tags_77 },
    { 0xf7b61fdc, SUC_lexicon_tags_60 },
    { 0xd961fdd, SUC_lexicon_tags_26 },
    { 0x5e761fdb, SUC_lexicon_tags_29 },
    { 0xaa961fdd, SUC_lexicon_tags_15 },
    { 0x4b61fe0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc761fe5, SUC_lexicon_tags_65 },
    { 0xc1761fe6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6961fe9, SUC_lexicon_tags_1439 },
    { 0, NULL },
    { 0xc7361feb, SUC_lexicon_tags_10 },
    { 0x22561feb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa7361fee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81361ff0, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1161ff6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6761ff9, SUC_lexicon_tags_47 },
    { 0x25361ff9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76962001, SUC_lexicon_tags_15 },
    { 0x44562001, SUC_lexicon_tags_31 },
    { 0x69b62003, SUC_lexicon_tags_6 },
    { 0xff762004, SUC_lexicon_tags_16 },
    { 0x8ad62003, SUC_lexicon_tags_72 },
    { 0xa0362006, SUC_lexicon_tags_7 },
    { 0x84562007, SUC_lexicon_tags_31 },
    { 0x4f362008, SUC_lexicon_tags_26 },
    { 0xabd62008, SUC_lexicon_tags_12 },
    { 0x6e762007, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b6200d, SUC_lexicon_tags_19 },
    { 0x7256200e, SUC_lexicon_tags_45 },
    { 0xf3d6200d, SUC_lexicon_tags_1 },
    { 0x17f62010, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5362013, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0962017, SUC_lexicon_tags_37 },
    { 0x9df62017, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf36201f, SUC_lexicon_tags_65 },
    { 0x1736201f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed762026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f62032, SUC_lexicon_tags_45 },
    { 0x46f62033, SUC_lexicon_tags_719 },
    { 0, NULL },
    { 0x26962035, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b76203a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd016203f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad62042, SUC_lexicon_tags_43 },
    { 0xda562042, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d62048, SUC_lexicon_tags_103 },
    { 0x51962049, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff62051, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa162054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8f62056, SUC_lexicon_tags_14 },
    { 0x39962056, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f36205e, SUC_lexicon_tags_66 },
    { 0x3e56205f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdd762061, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd562063, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3e562065, SUC_lexicon_tags_4 },
    { 0x56b62065, SUC_lexicon_tags_43 },
    { 0x68562066, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8436206b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae962071, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82962076, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa416207b, SUC_lexicon_tags_151 },
    { 0xafb6207c, SUC_lexicon_tags_15 },
    { 0xe6b6207d, SUC_lexicon_tags_12 },
    { 0x3996207d, SUC_lexicon_tags_160 },
    { 0x3f56207f, SUC_lexicon_tags_14 },
    { 0x4996207f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b62083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81762085, SUC_lexicon_tags_12 },
    { 0xa4562085, SUC_lexicon_tags_27 },
    { 0x14362085, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50162093, SUC_lexicon_tags_34 },
    { 0xb6d62094, SUC_lexicon_tags_37 },
    { 0xfed62095, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d6209a, SUC_lexicon_tags_17 },
    { 0xf596209b, SUC_lexicon_tags_27 },
    { 0xc236209b, SUC_lexicon_tags_12 },
    { 0x8f56209d, SUC_lexicon_tags_29 },
    { 0x3e96209d, SUC_lexicon_tags_25 },
    { 0x4cf6209f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8af620a1, SUC_lexicon_tags_37 },
    { 0xfc1620a1, SUC_lexicon_tags_1 },
    { 0xe0d620a3, SUC_lexicon_tags_47 },
    { 0x8cf620a1, SUC_lexicon_tags_12 },
    { 0x9fd620a5, SUC_lexicon_tags_10 },
    { 0xf5f620a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x175620a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x941620ab, SUC_lexicon_tags_38 },
    { 0x87f620ac, SUC_lexicon_tags_9 },
    { 0x723620ad, SUC_lexicon_tags_42 },
    { 0xce9620ad, SUC_lexicon_tags_19 },
    { 0x507620ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb43620b1, SUC_lexicon_tags_19 },
    { 0x70d620b1, SUC_lexicon_tags_2 },
    { 0x11f620b3, SUC_lexicon_tags_17 },
    { 0xfb620b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93620bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x711620bf, SUC_lexicon_tags_10 },
    { 0x23b620c0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa69620c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3620c5, SUC_lexicon_tags_1 },
    { 0x2cf620c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681620cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49620d2, SUC_lexicon_tags_11 },
    { 0x2ef620d2, SUC_lexicon_tags_7 },
    { 0x489620d4, SUC_lexicon_tags_25 },
    { 0x7f7620d5, SUC_lexicon_tags_33 },
    { 0xc47620d6, SUC_lexicon_tags_49 },
    { 0xc61620d7, SUC_lexicon_tags_14 },
    { 0x849620d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7620db, SUC_lexicon_tags_16 },
    { 0xe4d620dc, SUC_lexicon_tags_7 },
    { 0xed9620db, SUC_lexicon_tags_141 },
    { 0x9ef620de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361620e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7620e5, SUC_lexicon_tags_8 },
    { 0x761620e6, SUC_lexicon_tags_17 },
    { 0xe47620e6, SUC_lexicon_tags_47 },
    { 0xa29620e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x581620ea, SUC_lexicon_tags_11 },
    { 0xbdd620eb, SUC_lexicon_tags_31 },
    { 0xcff620ec, SUC_lexicon_tags_15 },
    { 0x6d7620ea, SUC_lexicon_tags_49 },
    { 0xacd620ea, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x2f5620f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c5620f2, SUC_lexicon_tags_15 },
    { 0x75b620f3, SUC_lexicon_tags_38 },
    { 0x7d620f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d620f7, SUC_lexicon_tags_33 },
    { 0x84d620f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3620fb, SUC_lexicon_tags_37 },
    { 0x673620fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb62103, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb62107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f6210a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b16210c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa636210e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4e962110, SUC_lexicon_tags_25 },
    { 0x80f62110, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb62114, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf536211b, SUC_lexicon_tags_63 },
    { 0xc856211c, SUC_lexicon_tags_39 },
    { 0x6636211d, SUC_lexicon_tags_47 },
    { 0xbaf6211e, SUC_lexicon_tags_27 },
    { 0x1556211c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75562126, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad62129, SUC_lexicon_tags_3 },
    { 0x4db62129, SUC_lexicon_tags_47 },
    { 0x37b62129, SUC_lexicon_tags_7 },
    { 0xe4d62129, SUC_lexicon_tags_28 },
    { 0x4f56212d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62562131, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0162133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b62136, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8456213e, SUC_lexicon_tags_37 },
    { 0xdb36213f, SUC_lexicon_tags_33 },
    { 0x496213f, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x1ed62142, SUC_lexicon_tags_12 },
    { 0xf9b62142, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b762149, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd76214b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc56214f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed62153, SUC_lexicon_tags_15 },
    { 0x26f62154, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d36215b, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af62160, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4b62162, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df62166, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd296216a, SUC_lexicon_tags_38 },
    { 0x2c56216b, SUC_lexicon_tags_9 },
    { 0x38d6216c, SUC_lexicon_tags_151 },
    { 0x2d96216d, SUC_lexicon_tags_60 },
    { 0x2036216a, SUC_lexicon_tags_21 },
    { 0xa196216f, SUC_lexicon_tags_15 },
    { 0xe7d6216a, SUC_lexicon_tags_20 },
    { 0x3c362171, SUC_lexicon_tags_17 },
    { 0x45162172, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b62175, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40362179, SUC_lexicon_tags_105 },
    { 0xa3d62179, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefd6217c, SUC_lexicon_tags_5 },
    { 0x2fd6217c, SUC_lexicon_tags_11 },
    { 0xacf6217e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1162182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68d62184, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x51162187, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd16218a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf616218e, SUC_lexicon_tags_25 },
    { 0x4f56218f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d62192, SUC_lexicon_tags_33 },
    { 0x94362193, SUC_lexicon_tags_2 },
    { 0x98f62194, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f62198, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe56219c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad621a1, SUC_lexicon_tags_17 },
    { 0x2f1621a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af621a7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769621ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15621b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d1621b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x31b621ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x699621bd, SUC_lexicon_tags_43 },
    { 0x547621be, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1621c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd621d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15621dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7621df, SUC_lexicon_tags_60 },
    { 0x75621e0, SUC_lexicon_tags_1 },
    { 0xaf621df, SUC_lexicon_tags_1 },
    { 0xa3621e2, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507621e6, SUC_lexicon_tags_27 },
    { 0xe37621e7, SUC_lexicon_tags_37 },
    { 0x811621e8, SUC_lexicon_tags_11 },
    { 0x13621e9, SUC_lexicon_tags_7 },
    { 0xf07621e6, SUC_lexicon_tags_112 },
    { 0x7f3621e8, SUC_lexicon_tags_15 },
    { 0x6e5621ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b621ef, SUC_lexicon_tags_11 },
    { 0xfa1621f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d621f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5ab621f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x587621fb, SUC_lexicon_tags_69 },
    { 0x785621fb, SUC_lexicon_tags_31 },
    { 0x379621fd, SUC_lexicon_tags_12 },
    { 0x791621fb, SUC_lexicon_tags_23 },
    { 0xe13621fe, SUC_lexicon_tags_781 },
    { 0x87b621fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b962203, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff62206, SUC_lexicon_tags_11 },
    { 0xb6162206, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a762209, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d56220c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7996220e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb9762210, SUC_lexicon_tags_11 },
    { 0x2f62211, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd6221a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5ed6221c, SUC_lexicon_tags_11 },
    { 0xf996221c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a76221f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x2762221, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a362224, SUC_lexicon_tags_0 },
    { 0x94362224, SUC_lexicon_tags_65 },
    { 0xbb162226, SUC_lexicon_tags_47 },
    { 0xf8f62227, SUC_lexicon_tags_16 },
    { 0xcf962227, SUC_lexicon_tags_43 },
    { 0x7c762228, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8616222b, SUC_lexicon_tags_32 },
    { 0xc3d6222c, SUC_lexicon_tags_11 },
    { 0xa136222c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb576222f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3f62231, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d962235, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b6223c, SUC_lexicon_tags_0 },
    { 0x256223d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1296223f, SUC_lexicon_tags_55 },
    { 0x8a962240, SUC_lexicon_tags_11 },
    { 0xf916223f, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0x6f362243, SUC_lexicon_tags_15 },
    { 0xba962243, SUC_lexicon_tags_2 },
    { 0x63162244, SUC_lexicon_tags_12 },
    { 0x7f362245, SUC_lexicon_tags_77 },
    { 0x63562246, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd6224d, SUC_lexicon_tags_13 },
    { 0x3536224e, SUC_lexicon_tags_68 },
    { 0xb4f6224f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3e562251, SUC_lexicon_tags_33 },
    { 0x95d62252, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97762258, SUC_lexicon_tags_12 },
    { 0x44b62259, SUC_lexicon_tags_37 },
    { 0x5a962258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c56225f, SUC_lexicon_tags_39 },
    { 0x1b62260, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9562264, SUC_lexicon_tags_15 },
    { 0x66162264, SUC_lexicon_tags_160 },
    { 0x55b62264, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3962269, SUC_lexicon_tags_7 },
    { 0xfd56226a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f362270, SUC_lexicon_tags_15 },
    { 0x54b62271, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d762273, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f62276, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f6227e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57362280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f762284, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x55962287, SUC_lexicon_tags_29 },
    { 0x54962288, SUC_lexicon_tags_9 },
    { 0x4e362288, SUC_lexicon_tags_46 },
    { 0x79d62287, SUC_lexicon_tags_17 },
    { 0x2e56228b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4b6228d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b562292, SUC_lexicon_tags_82 },
    { 0x32762293, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x562295, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb62298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579622a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x763622a5, SUC_lexicon_tags_11 },
    { 0xc53622a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ed622a8, SUC_lexicon_tags_10 },
    { 0x26b622a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb622ac, SUC_lexicon_tags_11 },
    { 0x5db622ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x639622af, SUC_lexicon_tags_11 },
    { 0xcb622b0, SUC_lexicon_tags_8 },
    { 0xf13622b1, SUC_lexicon_tags_23 },
    { 0x5ed622b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11622b6, SUC_lexicon_tags_17 },
    { 0x799622b6, SUC_lexicon_tags_12 },
    { 0x19b622b8, SUC_lexicon_tags_23 },
    { 0x519622b8, SUC_lexicon_tags_63 },
    { 0x70b622b7, SUC_lexicon_tags_9 },
    { 0xe2d622bb, SUC_lexicon_tags_17 },
    { 0xbc1622bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575622c1, SUC_lexicon_tags_49 },
    { 0xc59622c2, SUC_lexicon_tags_17 },
    { 0x36b622c2, SUC_lexicon_tags_39 },
    { 0xea9622c4, SUC_lexicon_tags_47 },
    { 0x5df622c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9622c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153622cd, SUC_lexicon_tags_6 },
    { 0x5d3622ce, SUC_lexicon_tags_64 },
    { 0xa67622ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab622d4, SUC_lexicon_tags_2 },
    { 0x96f622d5, SUC_lexicon_tags_38 },
    { 0xfa1622d5, SUC_lexicon_tags_17 },
    { 0xf65622d5, SUC_lexicon_tags_35 },
    { 0x303622d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb622dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b622df, SUC_lexicon_tags_15 },
    { 0x397622df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185622e4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9622ea, SUC_lexicon_tags_2 },
    { 0x62b622eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x433622ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92b622f0, SUC_lexicon_tags_6 },
    { 0x4b622f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f622f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143622f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4db622fb, SUC_lexicon_tags_23 },
    { 0x331622fc, SUC_lexicon_tags_33 },
    { 0x84b622fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e962305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43d62307, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d162310, SUC_lexicon_tags_27 },
    { 0x7df62311, SUC_lexicon_tags_0 },
    { 0xc2b62312, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c162317, SUC_lexicon_tags_16 },
    { 0xff562318, SUC_lexicon_tags_30 },
    { 0x6ed62319, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6996231b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd562322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22962325, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db6232a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73962331, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5962339, SUC_lexicon_tags_26 },
    { 0x7a56233a, SUC_lexicon_tags_12 },
    { 0xce362339, SUC_lexicon_tags_25 },
    { 0xcd96233c, SUC_lexicon_tags_15 },
    { 0x1cf6233c, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d62344, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe3962346, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5162349, SUC_lexicon_tags_14 },
    { 0x20d6234a, SUC_lexicon_tags_61 },
    { 0xef562349, SUC_lexicon_tags_27 },
    { 0x3756234c, SUC_lexicon_tags_39 },
    { 0x7e36234d, SUC_lexicon_tags_61 },
    { 0xaf6234d, SUC_lexicon_tags_37 },
    { 0x56234b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcef62351, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f562353, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f62358, SUC_lexicon_tags_2 },
    { 0xbf762358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d62363, SUC_lexicon_tags_1 },
    { 0x10b62364, SUC_lexicon_tags_26 },
    { 0x82362365, SUC_lexicon_tags_15 },
    { 0xb1562366, SUC_lexicon_tags_82 },
    { 0xf3362367, SUC_lexicon_tags_48 },
    { 0xf1f62366, SUC_lexicon_tags_10 },
    { 0xd1962369, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa456236d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60762370, SUC_lexicon_tags_17 },
    { 0x57162371, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8562374, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e962376, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x98362378, SUC_lexicon_tags_17 },
    { 0x1fd62378, SUC_lexicon_tags_23 },
    { 0x47d62378, SUC_lexicon_tags_63 },
    { 0x13b6237b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46f6237d, SUC_lexicon_tags_9 },
    { 0xb256237e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81762381, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9762385, SUC_lexicon_tags_17 },
    { 0x70762386, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x70362388, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa736238d, SUC_lexicon_tags_37 },
    { 0xbe96238e, SUC_lexicon_tags_37 },
    { 0xab76238f, SUC_lexicon_tags_32 },
    { 0xe976238f, SUC_lexicon_tags_7 },
    { 0x68b62391, SUC_lexicon_tags_12 },
    { 0x69d6238f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf562394, SUC_lexicon_tags_49 },
    { 0xadb62394, SUC_lexicon_tags_46 },
    { 0xa6162394, SUC_lexicon_tags_25 },
    { 0x44362395, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf836239b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941623a0, SUC_lexicon_tags_29 },
    { 0x48f623a1, SUC_lexicon_tags_12 },
    { 0x811623a0, SUC_lexicon_tags_29 },
    { 0x223623a3, SUC_lexicon_tags_45 },
    { 0xf85623a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1623a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3623ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1e3623ad, SUC_lexicon_tags_14 },
    { 0x67d623ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3623b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7623b6, SUC_lexicon_tags_37 },
    { 0x151623b7, SUC_lexicon_tags_77 },
    { 0x611623b8, SUC_lexicon_tags_27 },
    { 0x181623b9, SUC_lexicon_tags_34 },
    { 0x971623ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad623c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d623cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb623d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5623d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa2f623d6, SUC_lexicon_tags_25 },
    { 0x5b9623d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfeb623d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5623dc, SUC_lexicon_tags_25 },
    { 0xd93623dc, SUC_lexicon_tags_14 },
    { 0xb33623de, SUC_lexicon_tags_38 },
    { 0xc9623de, SUC_lexicon_tags_27 },
    { 0xc03623df, SUC_lexicon_tags_12 },
    { 0xfed623e1, SUC_lexicon_tags_53 },
    { 0x25d623e2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27623e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb85623e8, SUC_lexicon_tags_2 },
    { 0x67d623e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f623ec, SUC_lexicon_tags_17 },
    { 0xb79623ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb81623ef, SUC_lexicon_tags_43 },
    { 0x1a3623f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff623f3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x487623f5, SUC_lexicon_tags_65 },
    { 0xe5f623f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7623fa, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x1b623fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d623ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe562405, SUC_lexicon_tags_15 },
    { 0x3f962405, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbd62408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8096240a, SUC_lexicon_tags_198 },
    { 0xeef6240b, SUC_lexicon_tags_33 },
    { 0xb8f6240c, SUC_lexicon_tags_2 },
    { 0x33f6240d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16b6240f, SUC_lexicon_tags_11 },
    { 0x24762410, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf562413, SUC_lexicon_tags_11 },
    { 0x1a162414, SUC_lexicon_tags_2 },
    { 0x2df62415, SUC_lexicon_tags_37 },
    { 0x4cb62415, SUC_lexicon_tags_48 },
    { 0xee362417, SUC_lexicon_tags_10 },
    { 0xbf762418, SUC_lexicon_tags_2 },
    { 0xc0762413, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb896241f, SUC_lexicon_tags_17 },
    { 0x55562420, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1362422, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d962425, SUC_lexicon_tags_32 },
    { 0xebd62426, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9336242b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e96242e, SUC_lexicon_tags_28 },
    { 0x6b36242f, SUC_lexicon_tags_160 },
    { 0xa5362430, SUC_lexicon_tags_43 },
    { 0xea362431, SUC_lexicon_tags_3 },
    { 0x8bd6242f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb562437, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf556243b, SUC_lexicon_tags_17 },
    { 0x56f6243c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee96243f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde962443, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5562447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97362449, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe956244c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f62453, SUC_lexicon_tags_2 },
    { 0xb7162454, SUC_lexicon_tags_0 },
    { 0x94f62454, SUC_lexicon_tags_6 },
    { 0x7fb62456, SUC_lexicon_tags_7 },
    { 0xecb62457, SUC_lexicon_tags_19 },
    { 0x90d62454, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7776245a, SUC_lexicon_tags_49 },
    { 0x2956245a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5016245d, SUC_lexicon_tags_9 },
    { 0x5eb6245e, SUC_lexicon_tags_12 },
    { 0x3f6245e, SUC_lexicon_tags_12 },
    { 0x8c76245f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e362463, SUC_lexicon_tags_26 },
    { 0xfeb62464, SUC_lexicon_tags_25 },
    { 0x9a562465, SUC_lexicon_tags_27 },
    { 0xcf562464, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb62469, SUC_lexicon_tags_1 },
    { 0x4636246a, SUC_lexicon_tags_12 },
    { 0xdf362469, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2716246d, SUC_lexicon_tags_45 },
    { 0x10d6246e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f62473, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6f62475, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee36247e, SUC_lexicon_tags_1 },
    { 0xbad6247f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0362485, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x816248b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda162492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0562496, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab6249d, SUC_lexicon_tags_19 },
    { 0xfef6249d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x555624a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9624a4, SUC_lexicon_tags_11 },
    { 0xd7d624a5, SUC_lexicon_tags_43 },
    { 0xe95624a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29f624a8, SUC_lexicon_tags_28 },
    { 0x243624a9, SUC_lexicon_tags_15 },
    { 0x717624a9, SUC_lexicon_tags_38 },
    { 0xf2f624a9, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b624af, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x713624b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33624b7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd624c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x911624c5, SUC_lexicon_tags_1 },
    { 0x881624c6, SUC_lexicon_tags_39 },
    { 0xa7b624c6, SUC_lexicon_tags_65 },
    { 0xad1624c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363624d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1624d6, SUC_lexicon_tags_2 },
    { 0x293624d7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x101624d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86f624db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691624e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e3624e2, SUC_lexicon_tags_17 },
    { 0xfef624e2, SUC_lexicon_tags_54 },
    { 0xb45624e2, SUC_lexicon_tags_27 },
    { 0x213624e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b624e9, SUC_lexicon_tags_14 },
    { 0xedf624e9, SUC_lexicon_tags_48 },
    { 0x77624eb, SUC_lexicon_tags_25 },
    { 0xf624ec, SUC_lexicon_tags_37 },
    { 0x6bf624ed, SUC_lexicon_tags_797 },
    { 0, NULL },
    { 0xcc3624ef, SUC_lexicon_tags_17 },
    { 0x101624f0, SUC_lexicon_tags_9 },
    { 0x325624f1, SUC_lexicon_tags_2 },
    { 0x5fb624f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x945624f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0d624f7, SUC_lexicon_tags_43 },
    { 0xe5b624f8, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07624fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x945624ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b362503, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed962506, SUC_lexicon_tags_39 },
    { 0xc8962506, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46962509, SUC_lexicon_tags_30 },
    { 0xdc762509, SUC_lexicon_tags_299 },
    { 0x4762509, SUC_lexicon_tags_7 },
    { 0xe0f6250a, SUC_lexicon_tags_37 },
    { 0xf96250d, SUC_lexicon_tags_15 },
    { 0x4916250e, SUC_lexicon_tags_28 },
    { 0x3716250f, SUC_lexicon_tags_2 },
    { 0x92f6250f, SUC_lexicon_tags_47 },
    { 0x44d6250d, SUC_lexicon_tags_30 },
    { 0xdc76250b, SUC_lexicon_tags_37 },
    { 0x5a96250b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd96251b, SUC_lexicon_tags_16 },
    { 0xe2b6251c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xe556251e, SUC_lexicon_tags_15 },
    { 0x7456251f, SUC_lexicon_tags_37 },
    { 0x94962520, SUC_lexicon_tags_25 },
    { 0x8b162521, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf762524, SUC_lexicon_tags_65 },
    { 0x73962524, SUC_lexicon_tags_15 },
    { 0xa7b62526, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xc362529, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b6252d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc636252f, SUC_lexicon_tags_11 },
    { 0xcff6252f, SUC_lexicon_tags_60 },
    { 0x1e162531, SUC_lexicon_tags_30 },
    { 0xe8562532, SUC_lexicon_tags_12 },
    { 0xd3d6252f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b62536, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7d62538, SUC_lexicon_tags_19 },
    { 0x37962539, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf016253b, SUC_lexicon_tags_4 },
    { 0xd776253c, SUC_lexicon_tags_21 },
    { 0x58b6253d, SUC_lexicon_tags_3 },
    { 0x9596253d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7962543, SUC_lexicon_tags_21 },
    { 0x33962544, SUC_lexicon_tags_60 },
    { 0x52362545, SUC_lexicon_tags_37 },
    { 0x37362543, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58b62548, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f56254b, SUC_lexicon_tags_11 },
    { 0x5516254c, SUC_lexicon_tags_17 },
    { 0xfaf6254d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb16254f, SUC_lexicon_tags_26 },
    { 0x3e962550, SUC_lexicon_tags_0 },
    { 0x7d762551, SUC_lexicon_tags_17 },
    { 0x7e162550, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad56255e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b62562, SUC_lexicon_tags_47 },
    { 0x9c562563, SUC_lexicon_tags_21 },
    { 0x68f62564, SUC_lexicon_tags_63 },
    { 0xf3d62565, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d62568, SUC_lexicon_tags_45 },
    { 0xedb62568, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4db6256b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db6256e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5162574, SUC_lexicon_tags_15 },
    { 0x13762575, SUC_lexicon_tags_63 },
    { 0xc0362576, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x34b62578, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5556257a, SUC_lexicon_tags_49 },
    { 0xd296257a, SUC_lexicon_tags_28 },
    { 0x2ed6257b, SUC_lexicon_tags_7 },
    { 0xd1b6257d, SUC_lexicon_tags_141 },
    { 0xbf96257e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf762580, SUC_lexicon_tags_25 },
    { 0x6ab62581, SUC_lexicon_tags_14 },
    { 0xa7b62582, SUC_lexicon_tags_11 },
    { 0xf562581, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3162586, SUC_lexicon_tags_19 },
    { 0x12562587, SUC_lexicon_tags_11 },
    { 0xeeb62588, SUC_lexicon_tags_11 },
    { 0x19362588, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b6258c, SUC_lexicon_tags_11 },
    { 0xa276258c, SUC_lexicon_tags_27 },
    { 0x86f6258d, SUC_lexicon_tags_136 },
    { 0xc8b6258f, SUC_lexicon_tags_7 },
    { 0x4a16258c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x15562592, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27962598, SUC_lexicon_tags_11 },
    { 0xebb62599, SUC_lexicon_tags_27 },
    { 0x77d62599, SUC_lexicon_tags_82 },
    { 0xccd6259b, SUC_lexicon_tags_28 },
    { 0x3d76259c, SUC_lexicon_tags_15 },
    { 0xfe962599, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x451625a0, SUC_lexicon_tags_46 },
    { 0xf625a1, SUC_lexicon_tags_21 },
    { 0x52b625a2, SUC_lexicon_tags_46 },
    { 0xbb1625a3, SUC_lexicon_tags_7 },
    { 0xfdf625a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f625b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdaf625b7, SUC_lexicon_tags_6 },
    { 0xf01625b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe95625ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5b625bc, SUC_lexicon_tags_60 },
    { 0x155625bd, SUC_lexicon_tags_17 },
    { 0xac5625bd, SUC_lexicon_tags_60 },
    { 0x575625bf, SUC_lexicon_tags_17 },
    { 0x6d3625c0, SUC_lexicon_tags_43 },
    { 0x675625bf, SUC_lexicon_tags_17 },
    { 0xba9625bd, SUC_lexicon_tags_28 },
    { 0xd1b625c3, SUC_lexicon_tags_37 },
    { 0x29625c4, SUC_lexicon_tags_11 },
    { 0x789625c4, SUC_lexicon_tags_37 },
    { 0xbab625c5, SUC_lexicon_tags_25 },
    { 0x6b1625bd, SUC_lexicon_tags_63 },
    { 0x17f625c8, SUC_lexicon_tags_37 },
    { 0xeb5625c6, SUC_lexicon_tags_29 },
    { 0xae9625c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6b625cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf71625ce, SUC_lexicon_tags_17 },
    { 0x37625cf, SUC_lexicon_tags_27 },
    { 0x2c3625ce, SUC_lexicon_tags_6 },
    { 0xa2d625d0, SUC_lexicon_tags_17 },
    { 0x1cb625d1, SUC_lexicon_tags_14 },
    { 0x59f625d3, SUC_lexicon_tags_43 },
    { 0x2e1625d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9625d7, SUC_lexicon_tags_15 },
    { 0x6a3625d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc19625da, SUC_lexicon_tags_28 },
    { 0x999625db, SUC_lexicon_tags_17 },
    { 0x7dd625db, SUC_lexicon_tags_16 },
    { 0xc13625dd, SUC_lexicon_tags_2 },
    { 0x605625de, SUC_lexicon_tags_12 },
    { 0x4f5625de, SUC_lexicon_tags_14 },
    { 0xbd1625e0, SUC_lexicon_tags_1 },
    { 0x429625e0, SUC_lexicon_tags_37 },
    { 0x75f625e2, SUC_lexicon_tags_43 },
    { 0x831625e1, SUC_lexicon_tags_11 },
    { 0x493625dc, SUC_lexicon_tags_19 },
    { 0xfa7625e5, SUC_lexicon_tags_33 },
    { 0xd27625e0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d625e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe63625eb, SUC_lexicon_tags_11 },
    { 0xc41625eb, SUC_lexicon_tags_37 },
    { 0x1e7625ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3625f7, SUC_lexicon_tags_31 },
    { 0xe45625f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7625fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b625fd, SUC_lexicon_tags_49 },
    { 0xb19625fe, SUC_lexicon_tags_12 },
    { 0xaf5625fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab62605, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd62608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12b6260a, SUC_lexicon_tags_43 },
    { 0xa316260b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x41b6260d, SUC_lexicon_tags_6 },
    { 0x6576260d, SUC_lexicon_tags_15 },
    { 0xefb6260d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9562611, SUC_lexicon_tags_37 },
    { 0xe9362612, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x37162614, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x53162617, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6261a, SUC_lexicon_tags_68 },
    { 0x1f36261a, SUC_lexicon_tags_2 },
    { 0xaf76261b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf776261f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf562627, SUC_lexicon_tags_0 },
    { 0x92362628, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7616262d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7362630, SUC_lexicon_tags_3 },
    { 0xabf62630, SUC_lexicon_tags_17 },
    { 0xfe562631, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa1562634, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b62637, SUC_lexicon_tags_15 },
    { 0x1b562637, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x27d6263a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7256263d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf162643, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e562645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd6264d, SUC_lexicon_tags_54 },
    { 0x99f6264d, SUC_lexicon_tags_27 },
    { 0x21b6264f, SUC_lexicon_tags_72 },
    { 0x2a56264f, SUC_lexicon_tags_60 },
    { 0x8c762651, SUC_lexicon_tags_14 },
    { 0xe076264f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe2562654, SUC_lexicon_tags_11 },
    { 0xeef62655, SUC_lexicon_tags_48 },
    { 0x11762656, SUC_lexicon_tags_15 },
    { 0x16b62654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f6265b, SUC_lexicon_tags_22 },
    { 0x2336265c, SUC_lexicon_tags_12 },
    { 0xae36265c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5936265f, SUC_lexicon_tags_1 },
    { 0xa6f62660, SUC_lexicon_tags_37 },
    { 0x5f562661, SUC_lexicon_tags_37 },
    { 0xb9f62661, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x79362664, SUC_lexicon_tags_25 },
    { 0xe3562665, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f162668, SUC_lexicon_tags_11 },
    { 0x8db62668, SUC_lexicon_tags_37 },
    { 0x49b6266a, SUC_lexicon_tags_2 },
    { 0xab36266b, SUC_lexicon_tags_34 },
    { 0x65b6266c, SUC_lexicon_tags_1 },
    { 0x82b6266c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9962670, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x34762673, SUC_lexicon_tags_2 },
    { 0x48962674, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f62679, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7fd6267b, SUC_lexicon_tags_55 },
    { 0xe756267b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3362686, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd336268c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe96268f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefb62691, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10362694, SUC_lexicon_tags_26 },
    { 0x59d62695, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c962698, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf36269d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9bf6269f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d626a2, SUC_lexicon_tags_45 },
    { 0x6b1626a3, SUC_lexicon_tags_11 },
    { 0x965626a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x551626a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35626ac, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0x7e9626ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f626b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b626b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9626bd, SUC_lexicon_tags_25 },
    { 0x6f7626be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53626c3, SUC_lexicon_tags_37 },
    { 0xe2d626c3, SUC_lexicon_tags_31 },
    { 0x559626c5, SUC_lexicon_tags_7 },
    { 0x1ed626c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13626ca, SUC_lexicon_tags_1 },
    { 0xc5f626ca, SUC_lexicon_tags_1 },
    { 0xab1626cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41626d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59f626d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35626da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb626e2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x53626e5, SUC_lexicon_tags_12 },
    { 0x6ab626e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957626ea, SUC_lexicon_tags_27 },
    { 0x313626ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x347626ed, SUC_lexicon_tags_1 },
    { 0x997626ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1626f4, SUC_lexicon_tags_15 },
    { 0x9db626f4, SUC_lexicon_tags_28 },
    { 0x139626f6, SUC_lexicon_tags_7 },
    { 0xc89626f7, SUC_lexicon_tags_1 },
    { 0x5e7626f6, SUC_lexicon_tags_2 },
    { 0x601626f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd1b626fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf626ff, SUC_lexicon_tags_37 },
    { 0xa1162700, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c162703, SUC_lexicon_tags_13 },
    { 0x21762704, SUC_lexicon_tags_30 },
    { 0xda762705, SUC_lexicon_tags_2 },
    { 0x1f62703, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f62709, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee36270e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd62715, SUC_lexicon_tags_12 },
    { 0xdd762715, SUC_lexicon_tags_40 },
    { 0xb562717, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd236271e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaeb62720, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d62729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1776272c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfbb6272e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca962735, SUC_lexicon_tags_79 },
    { 0x27f62736, SUC_lexicon_tags_1041 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff62739, SUC_lexicon_tags_37 },
    { 0x6cb6273a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6562744, SUC_lexicon_tags_47 },
    { 0xd0f62745, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f62748, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa9d6274a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b6274d, SUC_lexicon_tags_84 },
    { 0x81f6274e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b62751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c562753, SUC_lexicon_tags_19 },
    { 0xb8f62754, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xe5d62756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa362758, SUC_lexicon_tags_38 },
    { 0x8e962759, SUC_lexicon_tags_15 },
    { 0x2f76275a, SUC_lexicon_tags_49 },
    { 0xea162758, SUC_lexicon_tags_8 },
    { 0xf496275a, SUC_lexicon_tags_12 },
    { 0x8c76275d, SUC_lexicon_tags_55 },
    { 0x4e96275e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe362765, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d62768, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6276c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b6276f, SUC_lexicon_tags_25 },
    { 0xab362770, SUC_lexicon_tags_23 },
    { 0xaf962771, SUC_lexicon_tags_33 },
    { 0xf2d62771, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d62776, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1562778, SUC_lexicon_tags_31 },
    { 0x88962778, SUC_lexicon_tags_6 },
    { 0x82362779, SUC_lexicon_tags_17 },
    { 0xd8f6277b, SUC_lexicon_tags_25 },
    { 0xe2d6277c, SUC_lexicon_tags_63 },
    { 0xb1b6277b, SUC_lexicon_tags_7 },
    { 0x2a162779, SUC_lexicon_tags_27 },
    { 0xfd6277f, SUC_lexicon_tags_11 },
    { 0xcd162780, SUC_lexicon_tags_19 },
    { 0x5636277a, SUC_lexicon_tags_27 },
    { 0xe136277e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xaf962784, SUC_lexicon_tags_11 },
    { 0xe762784, SUC_lexicon_tags_11 },
    { 0x56762786, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x22962788, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6278b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d6278e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf962793, SUC_lexicon_tags_19 },
    { 0x28162794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b6279a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2fd6279c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4136279f, SUC_lexicon_tags_19 },
    { 0xbaf6279f, SUC_lexicon_tags_1 },
    { 0x4af627a1, SUC_lexicon_tags_16 },
    { 0x77f627a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd49627a4, SUC_lexicon_tags_13 },
    { 0x847627a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d627a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3627b5, SUC_lexicon_tags_1 },
    { 0x369627b6, SUC_lexicon_tags_7 },
    { 0x62d627b7, SUC_lexicon_tags_60 },
    { 0x31627b8, SUC_lexicon_tags_43 },
    { 0xfdd627b8, SUC_lexicon_tags_61 },
    { 0x73b627b8, SUC_lexicon_tags_11 },
    { 0x7ef627bb, SUC_lexicon_tags_31 },
    { 0x8df627bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9627bf, SUC_lexicon_tags_47 },
    { 0xaff627bf, SUC_lexicon_tags_11 },
    { 0x7d1627c1, SUC_lexicon_tags_0 },
    { 0xf13627c2, SUC_lexicon_tags_83 },
    { 0x58d627c3, SUC_lexicon_tags_25 },
    { 0x935627c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8b627c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xba3627c8, SUC_lexicon_tags_82 },
    { 0xf7b627c8, SUC_lexicon_tags_0 },
    { 0x1fd627c8, SUC_lexicon_tags_11 },
    { 0xffb627cb, SUC_lexicon_tags_82 },
    { 0x4cb627cc, SUC_lexicon_tags_16 },
    { 0x9e3627cc, SUC_lexicon_tags_37 },
    { 0x161627cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5627d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x48f627d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd8f627d7, SUC_lexicon_tags_25 },
    { 0x86b627d8, SUC_lexicon_tags_30 },
    { 0xbc1627d8, SUC_lexicon_tags_37 },
    { 0xc37627d7, SUC_lexicon_tags_11 },
    { 0x24b627db, SUC_lexicon_tags_41 },
    { 0x52b627d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9627e1, SUC_lexicon_tags_7 },
    { 0x841627e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9627e5, SUC_lexicon_tags_11 },
    { 0x3bb627e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9627eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b5627ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3627f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313627f7, SUC_lexicon_tags_13 },
    { 0x229627f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56362802, SUC_lexicon_tags_37 },
    { 0xbcd62803, SUC_lexicon_tags_28 },
    { 0x86f62803, SUC_lexicon_tags_6 },
    { 0x2b962805, SUC_lexicon_tags_15 },
    { 0x56562805, SUC_lexicon_tags_14 },
    { 0xa6d62807, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x2136280a, SUC_lexicon_tags_47 },
    { 0xced6280a, SUC_lexicon_tags_25 },
    { 0xf176280c, SUC_lexicon_tags_23 },
    { 0x9b6280d, SUC_lexicon_tags_27 },
    { 0xf276280d, SUC_lexicon_tags_37 },
    { 0x2076280e, SUC_lexicon_tags_13 },
    { 0xe76280a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xee162812, SUC_lexicon_tags_60 },
    { 0x88362813, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9962815, SUC_lexicon_tags_6 },
    { 0xb7362816, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b6281d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5762821, SUC_lexicon_tags_6 },
    { 0x9db62822, SUC_lexicon_tags_37 },
    { 0xe0962822, SUC_lexicon_tags_21 },
    { 0x40d62823, SUC_lexicon_tags_25 },
    { 0x13b62824, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3df62827, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb356282a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e56282d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9162830, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd362833, SUC_lexicon_tags_9 },
    { 0xb3762834, SUC_lexicon_tags_37 },
    { 0xa6162835, SUC_lexicon_tags_37 },
    { 0x8f162836, SUC_lexicon_tags_25 },
    { 0xe2b62835, SUC_lexicon_tags_31 },
    { 0xef162836, SUC_lexicon_tags_26 },
    { 0x10f62839, SUC_lexicon_tags_15 },
    { 0xa4b62834, SUC_lexicon_tags_7 },
    { 0xbe76283b, SUC_lexicon_tags_15 },
    { 0x3e36283b, SUC_lexicon_tags_60 },
    { 0x20d6283c, SUC_lexicon_tags_23 },
    { 0x6256283e, SUC_lexicon_tags_19 },
    { 0x7bb6283e, SUC_lexicon_tags_15 },
    { 0xbc762840, SUC_lexicon_tags_25 },
    { 0x61962841, SUC_lexicon_tags_49 },
    { 0x17b62833, SUC_lexicon_tags_21 },
    { 0x98762840, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf1b62845, SUC_lexicon_tags_25 },
    { 0x4c962846, SUC_lexicon_tags_12 },
    { 0xf3d62847, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x45d62849, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8796284c, SUC_lexicon_tags_1 },
    { 0x24d6284c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4762852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8362857, SUC_lexicon_tags_43 },
    { 0xedf62858, SUC_lexicon_tags_2 },
    { 0x7ab62859, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb36285f, SUC_lexicon_tags_2 },
    { 0x63b6285f, SUC_lexicon_tags_1 },
    { 0x46d6285f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d6286a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95362871, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf62875, SUC_lexicon_tags_15 },
    { 0x1f162876, SUC_lexicon_tags_19 },
    { 0xd6962876, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5736287c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1956287e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb562880, SUC_lexicon_tags_15 },
    { 0xe8562881, SUC_lexicon_tags_2 },
    { 0x89d62881, SUC_lexicon_tags_23 },
    { 0xeb562882, SUC_lexicon_tags_15 },
    { 0x87962881, SUC_lexicon_tags_33 },
    { 0xf0562882, SUC_lexicon_tags_12 },
    { 0x63762883, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb6288f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe0b62891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9f62893, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af6289a, SUC_lexicon_tags_25 },
    { 0xb7d6289b, SUC_lexicon_tags_27 },
    { 0xb0f6289c, SUC_lexicon_tags_11 },
    { 0xc836289d, SUC_lexicon_tags_57 },
    { 0x716289a, SUC_lexicon_tags_39 },
    { 0x7e56289f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x987628a2, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xf628a4, SUC_lexicon_tags_57 },
    { 0x8dd628a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x25b628a7, SUC_lexicon_tags_27 },
    { 0xc6f628a7, SUC_lexicon_tags_71 },
    { 0xa73628a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96f628ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x861628ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b9628af, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41628b5, SUC_lexicon_tags_25 },
    { 0x37f628b6, SUC_lexicon_tags_12 },
    { 0xc3b628b5, SUC_lexicon_tags_14 },
    { 0x8ef628b8, SUC_lexicon_tags_37 },
    { 0xdd7628b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd99628bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b628be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681628c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3d628c6, SUC_lexicon_tags_15 },
    { 0x457628c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfeb628c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d628d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa35628d3, SUC_lexicon_tags_11 },
    { 0x79d628d3, SUC_lexicon_tags_11 },
    { 0x66f628d5, SUC_lexicon_tags_0 },
    { 0x98b628d5, SUC_lexicon_tags_11 },
    { 0x9f628d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x627628da, SUC_lexicon_tags_2 },
    { 0x34d628db, SUC_lexicon_tags_25 },
    { 0x303628dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1d628de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7628e2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x549628e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b628eb, SUC_lexicon_tags_45 },
    { 0x285628ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d628f6, SUC_lexicon_tags_31 },
    { 0x94f628f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7628fc, SUC_lexicon_tags_29 },
    { 0xe4d628fd, SUC_lexicon_tags_14 },
    { 0x205628fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82b62900, SUC_lexicon_tags_14 },
    { 0x13b62901, SUC_lexicon_tags_31 },
    { 0xa5362902, SUC_lexicon_tags_19 },
    { 0x75962903, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b62906, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd596290f, SUC_lexicon_tags_26 },
    { 0xb6562910, SUC_lexicon_tags_12 },
    { 0x1f762910, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef62918, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc376291a, SUC_lexicon_tags_12 },
    { 0xfb76291b, SUC_lexicon_tags_47 },
    { 0xc3d6291a, SUC_lexicon_tags_1 },
    { 0xb3d6291d, SUC_lexicon_tags_17 },
    { 0xd376291a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x30162921, SUC_lexicon_tags_37 },
    { 0x9ab62922, SUC_lexicon_tags_37 },
    { 0x27962921, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f562925, SUC_lexicon_tags_37 },
    { 0xcd762926, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc356292a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae56292d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81962931, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4562933, SUC_lexicon_tags_1 },
    { 0xc962934, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb162937, SUC_lexicon_tags_0 },
    { 0xc8b62937, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5516293a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3696293c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4776293f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5ed62941, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb962947, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3456294c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab62953, SUC_lexicon_tags_29 },
    { 0x1762954, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f62957, SUC_lexicon_tags_9 },
    { 0x1d962957, SUC_lexicon_tags_15 },
    { 0xf762959, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3b36295b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55962961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80362964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a962967, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7996296c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d6296f, SUC_lexicon_tags_45 },
    { 0xc4d6296f, SUC_lexicon_tags_82 },
    { 0xf3d62971, SUC_lexicon_tags_8 },
    { 0x5a762970, SUC_lexicon_tags_37 },
    { 0x59362971, SUC_lexicon_tags_17 },
    { 0xa5f62972, SUC_lexicon_tags_25 },
    { 0x4516296f, SUC_lexicon_tags_14 },
    { 0x6b762973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18362979, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb436297c, SUC_lexicon_tags_2 },
    { 0x1396297d, SUC_lexicon_tags_2 },
    { 0x7db6297c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb962986, SUC_lexicon_tags_15 },
    { 0x7ad62986, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x97162989, SUC_lexicon_tags_7 },
    { 0x8616298a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8616298c, SUC_lexicon_tags_7 },
    { 0xe76298d, SUC_lexicon_tags_2 },
    { 0x9916298e, SUC_lexicon_tags_37 },
    { 0x71d6298f, SUC_lexicon_tags_27 },
    { 0xc2b6298f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e76299c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f629a0, SUC_lexicon_tags_37 },
    { 0xccf629a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf629a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703629b2, SUC_lexicon_tags_10 },
    { 0x61d629b2, SUC_lexicon_tags_49 },
    { 0x3e9629b4, SUC_lexicon_tags_29 },
    { 0xe0d629b5, SUC_lexicon_tags_17 },
    { 0x689629b2, SUC_lexicon_tags_11 },
    { 0x7f7629b7, SUC_lexicon_tags_17 },
    { 0x21629b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x645629ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x575629bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65629c1, SUC_lexicon_tags_0 },
    { 0x645629c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511629c7, SUC_lexicon_tags_17 },
    { 0x2cd629c8, SUC_lexicon_tags_7 },
    { 0x18f629c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x161629cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175629d0, SUC_lexicon_tags_33 },
    { 0x6bd629d1, SUC_lexicon_tags_29 },
    { 0x2d629d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f629d6, SUC_lexicon_tags_1 },
    { 0x18f629d6, SUC_lexicon_tags_4 },
    { 0xec7629d6, SUC_lexicon_tags_301 },
    { 0x1fd629d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe99629db, SUC_lexicon_tags_19 },
    { 0xc97629dc, SUC_lexicon_tags_15 },
    { 0xcf5629dd, SUC_lexicon_tags_47 },
    { 0x2d9629de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed629e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1629ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x64b629ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f629f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21629ff, SUC_lexicon_tags_0 },
    { 0x7d5629ff, SUC_lexicon_tags_25 },
    { 0x877629ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd962a04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b62a07, SUC_lexicon_tags_23 },
    { 0x62962a08, SUC_lexicon_tags_18 },
    { 0xf6d62a08, SUC_lexicon_tags_15 },
    { 0xa8d62a09, SUC_lexicon_tags_38 },
    { 0x52362a09, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf362a11, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbeb62a13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66562a18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e962a1b, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xee562a1d, SUC_lexicon_tags_25 },
    { 0x42562a1e, SUC_lexicon_tags_13 },
    { 0xc6b62a1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1bd62a21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b962a24, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x33762a27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf962a2a, SUC_lexicon_tags_11 },
    { 0x68762a2b, SUC_lexicon_tags_63 },
    { 0xc2962a2c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x24962a2e, SUC_lexicon_tags_7 },
    { 0xddd62a2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26d62a31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4562a33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf6b62a35, SUC_lexicon_tags_31 },
    { 0x6dd62a36, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98762a3d, SUC_lexicon_tags_60 },
    { 0xa7f62a3e, SUC_lexicon_tags_2 },
    { 0xab562a3f, SUC_lexicon_tags_37 },
    { 0x38362a3f, SUC_lexicon_tags_1 },
    { 0x46962a40, SUC_lexicon_tags_33 },
    { 0xebd62a42, SUC_lexicon_tags_63 },
    { 0x7cb62a43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42762a4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb62a50, SUC_lexicon_tags_55 },
    { 0x4cf62a51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ab62a53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xee962a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d62a59, SUC_lexicon_tags_47 },
    { 0xb7b62a59, SUC_lexicon_tags_2 },
    { 0x98162a5a, SUC_lexicon_tags_28 },
    { 0xc0162a5b, SUC_lexicon_tags_17 },
    { 0x5d962a5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8362a5f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xff562a61, SUC_lexicon_tags_33 },
    { 0x92162a62, SUC_lexicon_tags_80 },
    { 0x83162a61, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e162a6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f62a6e, SUC_lexicon_tags_11 },
    { 0xefb62a6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65162a75, SUC_lexicon_tags_37 },
    { 0x83562a76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf62a7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7762a7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f562a81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cd62a83, SUC_lexicon_tags_45 },
    { 0x52b62a83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d362a87, SUC_lexicon_tags_16 },
    { 0xa8762a88, SUC_lexicon_tags_7 },
    { 0xe6b62a89, SUC_lexicon_tags_7 },
    { 0x97b62a8a, SUC_lexicon_tags_8 },
    { 0x9cf62a8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3d62a8d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f62a92, SUC_lexicon_tags_23 },
    { 0x70f62a92, SUC_lexicon_tags_15 },
    { 0xcc362a94, SUC_lexicon_tags_15 },
    { 0x92362a95, SUC_lexicon_tags_57 },
    { 0xcf962a95, SUC_lexicon_tags_25 },
    { 0xf5b62a97, SUC_lexicon_tags_3 },
    { 0xe5362a98, SUC_lexicon_tags_1 },
    { 0x74b62a99, SUC_lexicon_tags_68 },
    { 0x32b62a9a, SUC_lexicon_tags_2 },
    { 0xec562a99, SUC_lexicon_tags_7 },
    { 0xcfd62a9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba562a9e, SUC_lexicon_tags_12 },
    { 0x15162a9f, SUC_lexicon_tags_14 },
    { 0x29562aa0, SUC_lexicon_tags_11 },
    { 0xf9762aa1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x84162aa3, SUC_lexicon_tags_0 },
    { 0xf0362aa3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x85762aa6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8562aa9, SUC_lexicon_tags_71 },
    { 0x93b62aaa, SUC_lexicon_tags_1 },
    { 0x1d562aaa, SUC_lexicon_tags_15 },
    { 0x4162aac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e762aaf, SUC_lexicon_tags_17 },
    { 0x8d62ab0, SUC_lexicon_tags_17 },
    { 0x9df62aaf, SUC_lexicon_tags_29 },
    { 0xa3b62ab2, SUC_lexicon_tags_7 },
    { 0x88f62ab3, SUC_lexicon_tags_32 },
    { 0xa8f62ab2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e962abe, SUC_lexicon_tags_37 },
    { 0xa5d62abf, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb62ac3, SUC_lexicon_tags_45 },
    { 0xec362ac4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81162acc, SUC_lexicon_tags_38 },
    { 0x7a762acd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9362ad0, SUC_lexicon_tags_25 },
    { 0xa2d62ad1, SUC_lexicon_tags_37 },
    { 0x14762ad1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4762ad6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d62ad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67d62adb, SUC_lexicon_tags_19 },
    { 0xaa162adb, SUC_lexicon_tags_282 },
    { 0x73f62adb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2162ae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4962aec, SUC_lexicon_tags_15 },
    { 0x1e762aed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f62af5, SUC_lexicon_tags_28 },
    { 0x35962af6, SUC_lexicon_tags_28 },
    { 0x39962af7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef162b00, SUC_lexicon_tags_21 },
    { 0xf1762b01, SUC_lexicon_tags_45 },
    { 0xd4762b02, SUC_lexicon_tags_19 },
    { 0x6a762b02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1162b05, SUC_lexicon_tags_65 },
    { 0x75d62b05, SUC_lexicon_tags_17 },
    { 0xa162b06, SUC_lexicon_tags_7 },
    { 0x8e562b05, SUC_lexicon_tags_11 },
    { 0xf9762b09, SUC_lexicon_tags_47 },
    { 0x92962b0a, SUC_lexicon_tags_6 },
    { 0xd7962b08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd62b0e, SUC_lexicon_tags_45 },
    { 0xd62b0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0562b14, SUC_lexicon_tags_1 },
    { 0xc3f62b14, SUC_lexicon_tags_47 },
    { 0xc3b62b16, SUC_lexicon_tags_17 },
    { 0x95162b17, SUC_lexicon_tags_0 },
    { 0xcbf62b18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f62b1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e162b20, SUC_lexicon_tags_0 },
    { 0x3bf62b20, SUC_lexicon_tags_113 },
    { 0x76562b20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x69162b24, SUC_lexicon_tags_7 },
    { 0x4cf62b25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9f62b27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82362b29, SUC_lexicon_tags_11 },
    { 0x25962b29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab62b2e, SUC_lexicon_tags_6 },
    { 0xae162b2e, SUC_lexicon_tags_37 },
    { 0x56d62b30, SUC_lexicon_tags_2 },
    { 0x54362b31, SUC_lexicon_tags_9 },
    { 0xd2b62b2e, SUC_lexicon_tags_48 },
    { 0x55f62b33, SUC_lexicon_tags_12 },
    { 0x40b62b2f, SUC_lexicon_tags_38 },
    { 0x91562b31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed962b37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2b62b39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x16f62b3b, SUC_lexicon_tags_17 },
    { 0xf2362b3c, SUC_lexicon_tags_2 },
    { 0xc8362b3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d62b45, SUC_lexicon_tags_60 },
    { 0xaff62b46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f962b49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68162b4f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52762b55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d62b60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66362b65, SUC_lexicon_tags_47 },
    { 0x96762b65, SUC_lexicon_tags_38 },
    { 0xc9962b67, SUC_lexicon_tags_0 },
    { 0x40b62b68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b762b6f, SUC_lexicon_tags_0 },
    { 0x4dd62b70, SUC_lexicon_tags_43 },
    { 0xe6962b71, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7562b73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c162b7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4562b87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41162b93, SUC_lexicon_tags_15 },
    { 0xffb62b94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f62b97, SUC_lexicon_tags_30 },
    { 0xa8b62b98, SUC_lexicon_tags_25 },
    { 0xbd162b99, SUC_lexicon_tags_49 },
    { 0x9d162b9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x13362b9c, SUC_lexicon_tags_19 },
    { 0xca562b9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bd62b9f, SUC_lexicon_tags_17 },
    { 0xaf62b9f, SUC_lexicon_tags_25 },
    { 0x22562ba1, SUC_lexicon_tags_37 },
    { 0x58f62ba0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b962ba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e162bab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21762baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c562bb1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f62bb7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2af62bb9, SUC_lexicon_tags_3 },
    { 0xd2d62bb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bd62bbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b62bbf, SUC_lexicon_tags_33 },
    { 0x1c562bbf, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xabd62bc2, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xc562bc4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99162bc8, SUC_lexicon_tags_4 },
    { 0x78f62bc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65762bcb, SUC_lexicon_tags_1 },
    { 0xae162bcc, SUC_lexicon_tags_25 },
    { 0xa8162bcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8162bd3, SUC_lexicon_tags_133 },
    { 0x5d162bd3, SUC_lexicon_tags_65 },
    { 0x76f62bd4, SUC_lexicon_tags_11 },
    { 0xcef62bd6, SUC_lexicon_tags_35 },
    { 0xc0f62bd3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x18362bd9, SUC_lexicon_tags_188 },
    { 0x3a362bd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b62bdc, SUC_lexicon_tags_37 },
    { 0x4eb62bdd, SUC_lexicon_tags_39 },
    { 0xf6362bdc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xf3362be0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1762be6, SUC_lexicon_tags_45 },
    { 0x45962be7, SUC_lexicon_tags_9 },
    { 0x52962be8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe762beb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f62bf4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x78962bf6, SUC_lexicon_tags_46 },
    { 0x6d62bf6, SUC_lexicon_tags_2 },
    { 0xd1162bf6, SUC_lexicon_tags_7 },
    { 0x7f362bf8, SUC_lexicon_tags_37 },
    { 0x6f762bfa, SUC_lexicon_tags_25 },
    { 0x8df62bf7, SUC_lexicon_tags_60 },
    { 0x3ef62bfc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x52962bff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d562c02, SUC_lexicon_tags_64 },
    { 0x4f562c03, SUC_lexicon_tags_1 },
    { 0xea562c03, SUC_lexicon_tags_2 },
    { 0x55d62c02, SUC_lexicon_tags_17 },
    { 0x98362c05, SUC_lexicon_tags_2 },
    { 0x69b62c07, SUC_lexicon_tags_37 },
    { 0xc8d62c07, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3962c0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d62c11, SUC_lexicon_tags_11 },
    { 0xb3162c11, SUC_lexicon_tags_30 },
    { 0x48362c13, SUC_lexicon_tags_28 },
    { 0xa6d62c13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d62c1a, SUC_lexicon_tags_15 },
    { 0xe0d62c1b, SUC_lexicon_tags_17 },
    { 0x62962c1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62162c1e, SUC_lexicon_tags_43 },
    { 0xaf62c1e, SUC_lexicon_tags_77 },
    { 0xe6d62c1e, SUC_lexicon_tags_25 },
    { 0xc0f62c20, SUC_lexicon_tags_33 },
    { 0x99162c20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af62c27, SUC_lexicon_tags_39 },
    { 0x8e762c28, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd762c2c, SUC_lexicon_tags_2 },
    { 0x1f562c2d, SUC_lexicon_tags_12 },
    { 0xa5d62c2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee562c31, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d62c34, SUC_lexicon_tags_7 },
    { 0xdc962c35, SUC_lexicon_tags_11 },
    { 0xba762c35, SUC_lexicon_tags_21 },
    { 0x40562c34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbdd62c39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27962c3c, SUC_lexicon_tags_15 },
    { 0x5e362c3d, SUC_lexicon_tags_37 },
    { 0x2562c3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec162c40, SUC_lexicon_tags_34 },
    { 0xa3f62c41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55762c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b62c4a, SUC_lexicon_tags_25 },
    { 0x14962c4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11b62c4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xad562c50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41962c52, SUC_lexicon_tags_17 },
    { 0x2e362c53, SUC_lexicon_tags_8 },
    { 0xefd62c52, SUC_lexicon_tags_15 },
    { 0xc6162c53, SUC_lexicon_tags_21 },
    { 0x89b62c56, SUC_lexicon_tags_17 },
    { 0x29b62c54, SUC_lexicon_tags_38 },
    { 0xc3762c58, SUC_lexicon_tags_47 },
    { 0x6ed62c59, SUC_lexicon_tags_37 },
    { 0xb8562c55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0562c5e, SUC_lexicon_tags_1399 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48362c63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f62c66, SUC_lexicon_tags_26 },
    { 0x2d62c66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3a762c69, SUC_lexicon_tags_711 },
    { 0x51962c6a, SUC_lexicon_tags_12 },
    { 0x34762c69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8762c6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b62c72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47162c77, SUC_lexicon_tags_2 },
    { 0xd0b62c78, SUC_lexicon_tags_6 },
    { 0x9d962c78, SUC_lexicon_tags_47 },
    { 0x93962c7a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x62162c7d, SUC_lexicon_tags_11 },
    { 0xa7162c7e, SUC_lexicon_tags_47 },
    { 0x1bd62c7f, SUC_lexicon_tags_19 },
    { 0xe8562c80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16162c83, SUC_lexicon_tags_17 },
    { 0xcb62c84, SUC_lexicon_tags_37 },
    { 0x3e562c85, SUC_lexicon_tags_43 },
    { 0x69b62c85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd962c89, SUC_lexicon_tags_27 },
    { 0x24762c89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xda562c8d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2362c90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5562c92, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfa762c94, SUC_lexicon_tags_21 },
    { 0x2d362c95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbd62c97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96962ca9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x98562cab, SUC_lexicon_tags_6 },
    { 0x42f62cab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4362cb2, SUC_lexicon_tags_1 },
    { 0xa0362cb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d62cb8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f162cbd, SUC_lexicon_tags_47 },
    { 0x4db62cbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f62cc4, SUC_lexicon_tags_77 },
    { 0xb4962cc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43562cc8, SUC_lexicon_tags_47 },
    { 0xa8f62cc9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0162ccb, SUC_lexicon_tags_15 },
    { 0x77562ccc, SUC_lexicon_tags_10 },
    { 0x16962ccd, SUC_lexicon_tags_23 },
    { 0xaab62cce, SUC_lexicon_tags_14 },
    { 0xd0762cce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f962cd4, SUC_lexicon_tags_6 },
    { 0xb562cd5, SUC_lexicon_tags_64 },
    { 0xa1b62cd6, SUC_lexicon_tags_37 },
    { 0x3f762cd7, SUC_lexicon_tags_43 },
    { 0x39b62cd8, SUC_lexicon_tags_37 },
    { 0x49b62cd7, SUC_lexicon_tags_30 },
    { 0x50162cda, SUC_lexicon_tags_37 },
    { 0xd4962cdb, SUC_lexicon_tags_38 },
    { 0x7962cdc, SUC_lexicon_tags_14 },
    { 0xc3562cd9, SUC_lexicon_tags_26 },
    { 0xcdf62cda, SUC_lexicon_tags_32 },
    { 0xe0562cdd, SUC_lexicon_tags_141 },
    { 0x5cf62ce0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd62ce4, SUC_lexicon_tags_37 },
    { 0x6d362ce5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f62ce9, SUC_lexicon_tags_17 },
    { 0xc8962ce9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f62cef, SUC_lexicon_tags_7 },
    { 0x94362cf0, SUC_lexicon_tags_12 },
    { 0x3bf62cf0, SUC_lexicon_tags_37 },
    { 0xfa962cef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x65362cf4, SUC_lexicon_tags_163 },
    { 0xb5962cf4, SUC_lexicon_tags_31 },
    { 0xc4b62cf4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f762cf8, SUC_lexicon_tags_7 },
    { 0x53962cf9, SUC_lexicon_tags_17 },
    { 0xc8562cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51362cfc, SUC_lexicon_tags_37 },
    { 0x60d62cfd, SUC_lexicon_tags_19 },
    { 0xd9962cfe, SUC_lexicon_tags_38 },
    { 0x77962cff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b62d02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d362d04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe762d06, SUC_lexicon_tags_33 },
    { 0x3ff62d07, SUC_lexicon_tags_32 },
    { 0x24f62d08, SUC_lexicon_tags_38 },
    { 0xe9d62d07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7962d0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97962d0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31162d0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5562d13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32962d15, SUC_lexicon_tags_1 },
    { 0xe962d16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf62d1a, SUC_lexicon_tags_12 },
    { 0x89f62d1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3762d1d, SUC_lexicon_tags_1 },
    { 0x65d62d1d, SUC_lexicon_tags_14 },
    { 0xb6962d1f, SUC_lexicon_tags_215 },
    { 0xed562d20, SUC_lexicon_tags_37 },
    { 0x6cb62d21, SUC_lexicon_tags_66 },
    { 0xaf162d22, SUC_lexicon_tags_1 },
    { 0xe8362d22, SUC_lexicon_tags_37 },
    { 0x60f62d24, SUC_lexicon_tags_8 },
    { 0x9fd62d25, SUC_lexicon_tags_38 },
    { 0x86762d1d, SUC_lexicon_tags_47 },
    { 0xdd762d27, SUC_lexicon_tags_37 },
    { 0xabb62d22, SUC_lexicon_tags_84 },
    { 0x5d162d29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67362d2b, SUC_lexicon_tags_7 },
    { 0xf9362d2c, SUC_lexicon_tags_7 },
    { 0x57362d2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f62d30, SUC_lexicon_tags_25 },
    { 0xe4562d31, SUC_lexicon_tags_29 },
    { 0x42d62d32, SUC_lexicon_tags_17 },
    { 0x14b62d33, SUC_lexicon_tags_1 },
    { 0xfc762d33, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f62d38, SUC_lexicon_tags_60 },
    { 0x10562d39, SUC_lexicon_tags_37 },
    { 0x2a562d38, SUC_lexicon_tags_23 },
    { 0xe2b62d3b, SUC_lexicon_tags_38 },
    { 0xb7962d3c, SUC_lexicon_tags_33 },
    { 0x63762d3d, SUC_lexicon_tags_39 },
    { 0xc8f62d3e, SUC_lexicon_tags_5 },
    { 0xb9362d3f, SUC_lexicon_tags_2 },
    { 0xeb362d3f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6c762d42, SUC_lexicon_tags_17 },
    { 0x42562d43, SUC_lexicon_tags_29 },
    { 0xbc362d42, SUC_lexicon_tags_63 },
    { 0xf7562d44, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x30d62d47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9562d49, SUC_lexicon_tags_37 },
    { 0xfa762d4a, SUC_lexicon_tags_31 },
    { 0x54562d4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x70b62d4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99762d50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb562d59, SUC_lexicon_tags_37 },
    { 0x63162d5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f62d5f, SUC_lexicon_tags_29 },
    { 0xe3162d5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57762d62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e362d65, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced62d6a, SUC_lexicon_tags_57 },
    { 0xb9d62d6b, SUC_lexicon_tags_7 },
    { 0x1fb62d6c, SUC_lexicon_tags_13 },
    { 0x7ab62d6c, SUC_lexicon_tags_7 },
    { 0x22f62d6e, SUC_lexicon_tags_12 },
    { 0x7b62d6d, SUC_lexicon_tags_1 },
    { 0x41d62d6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x75f62d72, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71962d76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32162d78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad962d7d, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff562d84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f162d86, SUC_lexicon_tags_17 },
    { 0xe1962d86, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x2d362d89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xac962d8b, SUC_lexicon_tags_7 },
    { 0x1a562d8c, SUC_lexicon_tags_47 },
    { 0x98562d8d, SUC_lexicon_tags_49 },
    { 0x66762d8e, SUC_lexicon_tags_17 },
    { 0x1bd62d8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe0f62d91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd62d97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x27562d9a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x43d62d9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5362d9f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc362da2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f62da5, SUC_lexicon_tags_63 },
    { 0xe8162da5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd5b62da8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xaa762daa, SUC_lexicon_tags_25 },
    { 0xe0562daa, SUC_lexicon_tags_17 },
    { 0x76162dac, SUC_lexicon_tags_15 },
    { 0xf962dad, SUC_lexicon_tags_63 },
    { 0x59962dae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b562db0, SUC_lexicon_tags_23 },
    { 0xfbf62db1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4962db7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42f62db9, SUC_lexicon_tags_65 },
    { 0xb7f62dba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b62dbe, SUC_lexicon_tags_39 },
    { 0x5b162dbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefd62dc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x99d62dc3, SUC_lexicon_tags_33 },
    { 0xc2162dc3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9962dc9, SUC_lexicon_tags_19 },
    { 0x26362dca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f62dcd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af62dd3, SUC_lexicon_tags_87 },
    { 0x20d62dd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46562dd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6362dd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97b62dda, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35362de3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43562de7, SUC_lexicon_tags_55 },
    { 0x9e962de8, SUC_lexicon_tags_148 },
    { 0x2b162de9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33762ded, SUC_lexicon_tags_13 },
    { 0x9162dee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6b62df0, SUC_lexicon_tags_7 },
    { 0x6b762df1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b62df4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b62df8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5562dfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x66562dfd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85362e01, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad62e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d762e0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8af62e0c, SUC_lexicon_tags_19 },
    { 0x14f62e0c, SUC_lexicon_tags_17 },
    { 0x39962e0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8362e10, SUC_lexicon_tags_91 },
    { 0xd3362e11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b62e1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b762e20, SUC_lexicon_tags_29 },
    { 0x4e762e20, SUC_lexicon_tags_47 },
    { 0x77f62e22, SUC_lexicon_tags_17 },
    { 0xc5f62e22, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1962e29, SUC_lexicon_tags_8 },
    { 0x23162e2a, SUC_lexicon_tags_31 },
    { 0xe6b62e2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25362e2f, SUC_lexicon_tags_17 },
    { 0xbdf62e30, SUC_lexicon_tags_15 },
    { 0xa562e30, SUC_lexicon_tags_12 },
    { 0xad962e31, SUC_lexicon_tags_25 },
    { 0x44762e32, SUC_lexicon_tags_17 },
    { 0x75162e31, SUC_lexicon_tags_12 },
    { 0x83762e35, SUC_lexicon_tags_17 },
    { 0x56162e36, SUC_lexicon_tags_27 },
    { 0xaa962e36, SUC_lexicon_tags_28 },
    { 0x58962e37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f62e3e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x15362e40, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f62e43, SUC_lexicon_tags_10 },
    { 0x74762e44, SUC_lexicon_tags_12 },
    { 0xd2b62e43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b62e4c, SUC_lexicon_tags_14 },
    { 0xabf62e4d, SUC_lexicon_tags_1 },
    { 0x43b62e4d, SUC_lexicon_tags_107 },
    { 0x4f962e4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47b62e51, SUC_lexicon_tags_37 },
    { 0x43b62e52, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37962e5a, SUC_lexicon_tags_14 },
    { 0x54762e5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a162e5d, SUC_lexicon_tags_14 },
    { 0x74b62e5d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5362e65, SUC_lexicon_tags_0 },
    { 0x7eb62e65, SUC_lexicon_tags_11 },
    { 0x4a162e67, SUC_lexicon_tags_84 },
    { 0x26d62e68, SUC_lexicon_tags_29 },
    { 0x93162e69, SUC_lexicon_tags_17 },
    { 0x3ed62e6a, SUC_lexicon_tags_11 },
    { 0xfa762e6b, SUC_lexicon_tags_6 },
    { 0x84d62e66, SUC_lexicon_tags_37 },
    { 0x3b562e6d, SUC_lexicon_tags_11 },
    { 0xdd762e6e, SUC_lexicon_tags_37 },
    { 0x2a762e6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10f62e71, SUC_lexicon_tags_45 },
    { 0x2e962e72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db62e75, SUC_lexicon_tags_11 },
    { 0xac362e76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e562e79, SUC_lexicon_tags_15 },
    { 0x8d362e7a, SUC_lexicon_tags_18 },
    { 0xb2b62e7a, SUC_lexicon_tags_22 },
    { 0x9ad62e7c, SUC_lexicon_tags_14 },
    { 0x63762e7d, SUC_lexicon_tags_49 },
    { 0xaf762e7e, SUC_lexicon_tags_25 },
    { 0x1a962e7f, SUC_lexicon_tags_21 },
    { 0x15762e7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29762e85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b62e92, SUC_lexicon_tags_17 },
    { 0x6d962e93, SUC_lexicon_tags_25 },
    { 0xac562e93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfbf62e96, SUC_lexicon_tags_0 },
    { 0x7b162e97, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc162e9b, SUC_lexicon_tags_1 },
    { 0x4db62e9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1962e9e, SUC_lexicon_tags_0 },
    { 0x48b62e9f, SUC_lexicon_tags_25 },
    { 0xe6562ea0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19962ea4, SUC_lexicon_tags_69 },
    { 0x91d62ea4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f62ea8, SUC_lexicon_tags_11 },
    { 0xb0762ea9, SUC_lexicon_tags_56 },
    { 0xdfd62ea8, SUC_lexicon_tags_14 },
    { 0x5b162eab, SUC_lexicon_tags_15 },
    { 0x19b62eac, SUC_lexicon_tags_7 },
    { 0xb4562eaa, SUC_lexicon_tags_45 },
    { 0x2df62ead, SUC_lexicon_tags_14 },
    { 0xc7162eaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f62eb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f62eb7, SUC_lexicon_tags_17 },
    { 0x73f62eb8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdd62eba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b62ebe, SUC_lexicon_tags_37 },
    { 0x4f562ebf, SUC_lexicon_tags_12 },
    { 0x1f162ec0, SUC_lexicon_tags_30 },
    { 0x49162ec1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67162ec8, SUC_lexicon_tags_37 },
    { 0x91962ec9, SUC_lexicon_tags_7 },
    { 0x29b62ec8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x91362ecc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x76162ecf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e962ed1, SUC_lexicon_tags_7 },
    { 0xe4b62ed2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab62ed7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x47d62ed9, SUC_lexicon_tags_1 },
    { 0x59162ed9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7762edc, SUC_lexicon_tags_45 },
    { 0x99762edd, SUC_lexicon_tags_2 },
    { 0xb4962ede, SUC_lexicon_tags_8 },
    { 0x75362edc, SUC_lexicon_tags_69 },
    { 0x5c962edd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb362ee4, SUC_lexicon_tags_6 },
    { 0x2e762ee5, SUC_lexicon_tags_33 },
    { 0xe362ee6, SUC_lexicon_tags_37 },
    { 0x6e362ee7, SUC_lexicon_tags_17 },
    { 0x13b62ee7, SUC_lexicon_tags_2 },
    { 0x97d62ee7, SUC_lexicon_tags_30 },
    { 0x2b562eea, SUC_lexicon_tags_37 },
    { 0x36562eeb, SUC_lexicon_tags_37 },
    { 0x23162eec, SUC_lexicon_tags_30 },
    { 0x42762eeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x65d62eef, SUC_lexicon_tags_30 },
    { 0x67b62ef0, SUC_lexicon_tags_37 },
    { 0x6a362ef0, SUC_lexicon_tags_15 },
    { 0x41b62ef1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f62ef5, SUC_lexicon_tags_7 },
    { 0x17f62ef6, SUC_lexicon_tags_17 },
    { 0x65562ef7, SUC_lexicon_tags_30 },
    { 0x2f762ef8, SUC_lexicon_tags_1 },
    { 0x86b62ef6, SUC_lexicon_tags_7 },
    { 0x28362efa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d762efc, SUC_lexicon_tags_2 },
    { 0xdc162efd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b562f00, SUC_lexicon_tags_49 },
    { 0xbb62f01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x71162f04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7962f09, SUC_lexicon_tags_27 },
    { 0x34b62f0a, SUC_lexicon_tags_39 },
    { 0x1ef62f0b, SUC_lexicon_tags_1 },
    { 0x1b962f0c, SUC_lexicon_tags_107 },
    { 0x9e562f0d, SUC_lexicon_tags_23 },
    { 0x56962f0e, SUC_lexicon_tags_11 },
    { 0x762f0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc162f15, SUC_lexicon_tags_23 },
    { 0x9c562f15, SUC_lexicon_tags_84 },
    { 0x6f162f16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b362f1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe162f1e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9962f20, SUC_lexicon_tags_46 },
    { 0xf2762f20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2562f25, SUC_lexicon_tags_12 },
    { 0x69762f26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf62f2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61962f36, SUC_lexicon_tags_11 },
    { 0x8dd62f37, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b362f43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d62f46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5762f4b, SUC_lexicon_tags_2 },
    { 0x5f62f4c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b62f4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69362f54, SUC_lexicon_tags_4 },
    { 0x11d62f54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0162f5d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d162f62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50162f67, SUC_lexicon_tags_33 },
    { 0xad362f68, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6362f6b, SUC_lexicon_tags_60 },
    { 0x7b962f6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec762f73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86962f78, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe5362f7a, SUC_lexicon_tags_15 },
    { 0x4cf62f7b, SUC_lexicon_tags_23 },
    { 0x49362f7c, SUC_lexicon_tags_28 },
    { 0x9c362f7d, SUC_lexicon_tags_17 },
    { 0xb2162f7c, SUC_lexicon_tags_17 },
    { 0x9d162f7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdd762f81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f62f85, SUC_lexicon_tags_7 },
    { 0x86162f86, SUC_lexicon_tags_2 },
    { 0x3c962f87, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d62f8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27f62f8d, SUC_lexicon_tags_27 },
    { 0x40962f8e, SUC_lexicon_tags_0 },
    { 0x62d62f8f, SUC_lexicon_tags_15 },
    { 0xc8162f8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7162f92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66962f94, SUC_lexicon_tags_31 },
    { 0xcf362f94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6362f97, SUC_lexicon_tags_14 },
    { 0xb4162f98, SUC_lexicon_tags_2 },
    { 0xe3d62f98, SUC_lexicon_tags_84 },
    { 0x1f562f9a, SUC_lexicon_tags_60 },
    { 0x26162f9b, SUC_lexicon_tags_25 },
    { 0xf1d62f9c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x34362f9e, SUC_lexicon_tags_259 },
    { 0x9e962f9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d62fa5, SUC_lexicon_tags_23 },
    { 0x8ad62fa5, SUC_lexicon_tags_2 },
    { 0x89962fa6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc362fab, SUC_lexicon_tags_14 },
    { 0x8f762fac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5f62fae, SUC_lexicon_tags_2 },
    { 0x5b562faf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf62fb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f62fb7, SUC_lexicon_tags_39 },
    { 0x8e762fb8, SUC_lexicon_tags_25 },
    { 0x7fb62fb7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa162fbc, SUC_lexicon_tags_15 },
    { 0xac762fbd, SUC_lexicon_tags_37 },
    { 0xa5d62fbd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x95962fc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87b62fc2, SUC_lexicon_tags_33 },
    { 0x59762fc3, SUC_lexicon_tags_60 },
    { 0xbad62fc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6f62fc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45162fc9, SUC_lexicon_tags_55 },
    { 0x48b62fca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b62fce, SUC_lexicon_tags_11 },
    { 0xb9162fcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f62fd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e162fd4, SUC_lexicon_tags_55 },
    { 0x64f62fd5, SUC_lexicon_tags_11 },
    { 0xdf962fd6, SUC_lexicon_tags_11 },
    { 0x69d62fd6, SUC_lexicon_tags_1 },
    { 0xb9762fd7, SUC_lexicon_tags_25 },
    { 0x7f762fd9, SUC_lexicon_tags_7 },
    { 0x76362fda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b62fdd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x53962fe0, SUC_lexicon_tags_12 },
    { 0x5cd62fe0, SUC_lexicon_tags_17 },
    { 0xd4d62fe2, SUC_lexicon_tags_14 },
    { 0x12f62fe3, SUC_lexicon_tags_37 },
    { 0x29f62fe4, SUC_lexicon_tags_15 },
    { 0xd0b62fe5, SUC_lexicon_tags_8 },
    { 0xcc362fe6, SUC_lexicon_tags_32 },
    { 0x5b362fe7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf62ff0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1762ff6, SUC_lexicon_tags_17 },
    { 0x2a762ff7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d62ff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d363000, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd4363002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa163004, SUC_lexicon_tags_26 },
    { 0x54363005, SUC_lexicon_tags_10 },
    { 0xf5963006, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a363009, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a96300d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xd963010, SUC_lexicon_tags_19 },
    { 0x30b63011, SUC_lexicon_tags_108 },
    { 0x45163012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b63015, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf6301a, SUC_lexicon_tags_4 },
    { 0xffb6301b, SUC_lexicon_tags_3 },
    { 0x97d6301b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf496301f, SUC_lexicon_tags_197 },
    { 0x76f6301f, SUC_lexicon_tags_24 },
    { 0x1716301f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b63025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d6302a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd76302c, SUC_lexicon_tags_37 },
    { 0x7636302c, SUC_lexicon_tags_37 },
    { 0x1656302e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66163030, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0163038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6303c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9163040, SUC_lexicon_tags_11 },
    { 0xa2363041, SUC_lexicon_tags_25 },
    { 0x8db63041, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd63045, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x58763047, SUC_lexicon_tags_17 },
    { 0xec163048, SUC_lexicon_tags_106 },
    { 0x93b63047, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d6304f, SUC_lexicon_tags_12 },
    { 0xed96304f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d63053, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1563056, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa016305a, SUC_lexicon_tags_15 },
    { 0x9636305a, SUC_lexicon_tags_15 },
    { 0x71f6305b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf36305f, SUC_lexicon_tags_7 },
    { 0x8c363060, SUC_lexicon_tags_21 },
    { 0x48163061, SUC_lexicon_tags_8 },
    { 0xad63062, SUC_lexicon_tags_11 },
    { 0x6e763063, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d63066, SUC_lexicon_tags_63 },
    { 0xd0f63067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b563069, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f96306b, SUC_lexicon_tags_60 },
    { 0x5756306c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb6306f, SUC_lexicon_tags_19 },
    { 0x76763070, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed63073, SUC_lexicon_tags_25 },
    { 0xe363073, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25563079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb816307c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa136307f, SUC_lexicon_tags_23 },
    { 0xa56307f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea563088, SUC_lexicon_tags_19 },
    { 0x36b63088, SUC_lexicon_tags_24 },
    { 0xfef63088, SUC_lexicon_tags_11 },
    { 0x4a163089, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x33f6308d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6363090, SUC_lexicon_tags_7 },
    { 0x3f363091, SUC_lexicon_tags_16 },
    { 0xecf63092, SUC_lexicon_tags_37 },
    { 0xa9363093, SUC_lexicon_tags_25 },
    { 0xae763093, SUC_lexicon_tags_12 },
    { 0xf1563092, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd63099, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9e96309b, SUC_lexicon_tags_7 },
    { 0x9cb6309c, SUC_lexicon_tags_36 },
    { 0x43b6309d, SUC_lexicon_tags_1 },
    { 0x77f6309b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x711630a1, SUC_lexicon_tags_0 },
    { 0x917630a1, SUC_lexicon_tags_70 },
    { 0x147630a3, SUC_lexicon_tags_17 },
    { 0x92b630a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x883630a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b630aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47630b1, SUC_lexicon_tags_32 },
    { 0x387630b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3630b5, SUC_lexicon_tags_38 },
    { 0x129630b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6f630b8, SUC_lexicon_tags_37 },
    { 0x445630b8, SUC_lexicon_tags_84 },
    { 0xadb630b9, SUC_lexicon_tags_31 },
    { 0x845630b8, SUC_lexicon_tags_45 },
    { 0x4b3630bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1a3630be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x575630c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b630c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92b630c6, SUC_lexicon_tags_1 },
    { 0xc9630c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1630d0, SUC_lexicon_tags_33 },
    { 0x485630d1, SUC_lexicon_tags_17 },
    { 0xff630d2, SUC_lexicon_tags_2 },
    { 0xd49630d0, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45630d7, SUC_lexicon_tags_15 },
    { 0x9db630d8, SUC_lexicon_tags_55 },
    { 0x137630d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19630de, SUC_lexicon_tags_37 },
    { 0xe59630df, SUC_lexicon_tags_57 },
    { 0x3d7630e0, SUC_lexicon_tags_36 },
    { 0x863630e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71630e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f630eb, SUC_lexicon_tags_47 },
    { 0xa9d630ec, SUC_lexicon_tags_27 },
    { 0x93d630ed, SUC_lexicon_tags_17 },
    { 0x4db630ee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x687630f1, SUC_lexicon_tags_29 },
    { 0xe81630f1, SUC_lexicon_tags_47 },
    { 0x9b630f3, SUC_lexicon_tags_2 },
    { 0x56f630f4, SUC_lexicon_tags_22 },
    { 0xb630f5, SUC_lexicon_tags_37 },
    { 0x6ff630f4, SUC_lexicon_tags_108 },
    { 0x179630f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe163100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc163102, SUC_lexicon_tags_1 },
    { 0xaef63103, SUC_lexicon_tags_25 },
    { 0xe4363104, SUC_lexicon_tags_25 },
    { 0x97f63103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb63108, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7716310b, SUC_lexicon_tags_37 },
    { 0x9596310c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2116310e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cf63110, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22763112, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69963115, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84563118, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476311e, SUC_lexicon_tags_43 },
    { 0x6d16311f, SUC_lexicon_tags_10 },
    { 0xa2d63120, SUC_lexicon_tags_6 },
    { 0x93163121, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4963124, SUC_lexicon_tags_9 },
    { 0x33563124, SUC_lexicon_tags_14 },
    { 0x40f63126, SUC_lexicon_tags_1 },
    { 0x84163127, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3ff63129, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3676312c, SUC_lexicon_tags_31 },
    { 0x11f6312d, SUC_lexicon_tags_61 },
    { 0xa4d6312d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef63132, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3563134, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b6313a, SUC_lexicon_tags_11 },
    { 0x99b6313b, SUC_lexicon_tags_15 },
    { 0xd056313c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40363140, SUC_lexicon_tags_71 },
    { 0xc8d63141, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20f63143, SUC_lexicon_tags_25 },
    { 0x13963144, SUC_lexicon_tags_37 },
    { 0xff763145, SUC_lexicon_tags_63 },
    { 0xbe763146, SUC_lexicon_tags_1 },
    { 0x52d63145, SUC_lexicon_tags_9 },
    { 0x85b63148, SUC_lexicon_tags_25 },
    { 0x48f63143, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbdb6314b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe396314e, SUC_lexicon_tags_37 },
    { 0xa7d6314f, SUC_lexicon_tags_12 },
    { 0x18b6314f, SUC_lexicon_tags_65 },
    { 0xec763150, SUC_lexicon_tags_14 },
    { 0x3ff63151, SUC_lexicon_tags_23 },
    { 0x9f96314f, SUC_lexicon_tags_12 },
    { 0xa16314f, SUC_lexicon_tags_14 },
    { 0x2076314f, SUC_lexicon_tags_2 },
    { 0x74b63156, SUC_lexicon_tags_3 },
    { 0xaa763154, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2976315a, SUC_lexicon_tags_21 },
    { 0x7d96315b, SUC_lexicon_tags_37 },
    { 0x46d6315c, SUC_lexicon_tags_15 },
    { 0xea76315d, SUC_lexicon_tags_11 },
    { 0xa9d6315d, SUC_lexicon_tags_53 },
    { 0xc476315f, SUC_lexicon_tags_85 },
    { 0xf496315d, SUC_lexicon_tags_60 },
    { 0x9ef63160, SUC_lexicon_tags_6 },
    { 0xe9963162, SUC_lexicon_tags_12 },
    { 0x736315d, SUC_lexicon_tags_15 },
    { 0xaa763163, SUC_lexicon_tags_56 },
    { 0xaa763163, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x11163168, SUC_lexicon_tags_6 },
    { 0xabb63168, SUC_lexicon_tags_17 },
    { 0xc2963169, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1076316c, SUC_lexicon_tags_26 },
    { 0xdc56316c, SUC_lexicon_tags_2 },
    { 0x9136316e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77563174, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4963176, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d6317f, SUC_lexicon_tags_12 },
    { 0xd2563180, SUC_lexicon_tags_1 },
    { 0xe336317f, SUC_lexicon_tags_423 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d163187, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2376318a, SUC_lexicon_tags_33 },
    { 0xb896318b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad163190, SUC_lexicon_tags_8 },
    { 0x53163191, SUC_lexicon_tags_17 },
    { 0x56763192, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfbb63194, SUC_lexicon_tags_37 },
    { 0xa9d63195, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76319c, SUC_lexicon_tags_27 },
    { 0x1a56319c, SUC_lexicon_tags_7 },
    { 0x71d6319d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f631a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347631a8, SUC_lexicon_tags_44 },
    { 0x157631a9, SUC_lexicon_tags_68 },
    { 0x221631aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b631b2, SUC_lexicon_tags_87 },
    { 0xbfd631b3, SUC_lexicon_tags_37 },
    { 0x4b1631b4, SUC_lexicon_tags_49 },
    { 0x437631b3, SUC_lexicon_tags_25 },
    { 0x2ab631b2, SUC_lexicon_tags_7 },
    { 0x429631b7, SUC_lexicon_tags_32 },
    { 0x369631b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x407631ba, SUC_lexicon_tags_107 },
    { 0xd67631bb, SUC_lexicon_tags_61 },
    { 0x9d3631bb, SUC_lexicon_tags_2 },
    { 0xdff631bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99f631bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d631c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db631ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc13631cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731631d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65631d3, SUC_lexicon_tags_7 },
    { 0x8fd631d4, SUC_lexicon_tags_25 },
    { 0x477631d4, SUC_lexicon_tags_1 },
    { 0x98b631d6, SUC_lexicon_tags_11 },
    { 0xe71631d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9bf631d9, SUC_lexicon_tags_8 },
    { 0xe51631da, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb631dd, SUC_lexicon_tags_11 },
    { 0xc15631de, SUC_lexicon_tags_9 },
    { 0x8bf631de, SUC_lexicon_tags_25 },
    { 0xf67631e0, SUC_lexicon_tags_24 },
    { 0xa75631df, SUC_lexicon_tags_12 },
    { 0xd53631e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x285631e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xccb631e6, SUC_lexicon_tags_12 },
    { 0x467631e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b631f1, SUC_lexicon_tags_11 },
    { 0x6f631f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x587631f5, SUC_lexicon_tags_7 },
    { 0x773631f6, SUC_lexicon_tags_30 },
    { 0xed7631f7, SUC_lexicon_tags_23 },
    { 0x899631f7, SUC_lexicon_tags_30 },
    { 0x81631f9, SUC_lexicon_tags_15 },
    { 0x943631f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b63200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88763205, SUC_lexicon_tags_71 },
    { 0x4af63205, SUC_lexicon_tags_87 },
    { 0x92f63207, SUC_lexicon_tags_4 },
    { 0xb1763208, SUC_lexicon_tags_11 },
    { 0x3c763207, SUC_lexicon_tags_2 },
    { 0x70963207, SUC_lexicon_tags_63 },
    { 0xbcf63207, SUC_lexicon_tags_64 },
    { 0x4e16320c, SUC_lexicon_tags_15 },
    { 0x66b6320d, SUC_lexicon_tags_16 },
    { 0xdbd63208, SUC_lexicon_tags_31 },
    { 0x3536320f, SUC_lexicon_tags_38 },
    { 0xc5d63210, SUC_lexicon_tags_11 },
    { 0xee76320e, SUC_lexicon_tags_17 },
    { 0xe7d63212, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xed563214, SUC_lexicon_tags_171 },
    { 0x78363214, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5963219, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2756321f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdfb63221, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x19b63223, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d63226, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f363229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94b6322b, SUC_lexicon_tags_14 },
    { 0x9f36322c, SUC_lexicon_tags_12 },
    { 0xd956322c, SUC_lexicon_tags_21 },
    { 0x96b6322d, SUC_lexicon_tags_25 },
    { 0x24b6322f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa1763231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c963237, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f6323e, SUC_lexicon_tags_65 },
    { 0xe7b6323e, SUC_lexicon_tags_65 },
    { 0x6f163240, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8963242, SUC_lexicon_tags_60 },
    { 0xb5763242, SUC_lexicon_tags_12 },
    { 0x7c563242, SUC_lexicon_tags_17 },
    { 0x96763242, SUC_lexicon_tags_25 },
    { 0x9db63242, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3f63248, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b6324e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70763258, SUC_lexicon_tags_1 },
    { 0x4dd63259, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd6325c, SUC_lexicon_tags_17 },
    { 0x99d6325d, SUC_lexicon_tags_7 },
    { 0xbe96325c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf163263, SUC_lexicon_tags_1 },
    { 0xb9763263, SUC_lexicon_tags_11 },
    { 0x6ff63265, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc4763267, SUC_lexicon_tags_37 },
    { 0x26763268, SUC_lexicon_tags_7 },
    { 0xfab63269, SUC_lexicon_tags_11 },
    { 0x6d563268, SUC_lexicon_tags_7 },
    { 0x69b63267, SUC_lexicon_tags_2 },
    { 0x716326c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1856326e, SUC_lexicon_tags_25 },
    { 0x7e16326f, SUC_lexicon_tags_0 },
    { 0xbe363270, SUC_lexicon_tags_15 },
    { 0xbef6326e, SUC_lexicon_tags_24 },
    { 0x4e563272, SUC_lexicon_tags_66 },
    { 0x3eb63270, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18763278, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab6327e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b63280, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f63285, SUC_lexicon_tags_5 },
    { 0x43b63285, SUC_lexicon_tags_2 },
    { 0xa2363287, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4196328b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8eb6328d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa156328f, SUC_lexicon_tags_9 },
    { 0x8356328f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31163294, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb36329e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6d5632a0, SUC_lexicon_tags_14 },
    { 0x2f632a1, SUC_lexicon_tags_27 },
    { 0x183632a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61f632a4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df632a9, SUC_lexicon_tags_11 },
    { 0x9ff632aa, SUC_lexicon_tags_17 },
    { 0x8c1632a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419632af, SUC_lexicon_tags_19 },
    { 0x519632b0, SUC_lexicon_tags_18 },
    { 0x961632b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723632bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x30b632bf, SUC_lexicon_tags_25 },
    { 0xf65632bf, SUC_lexicon_tags_12 },
    { 0xafd632c1, SUC_lexicon_tags_11 },
    { 0x3a9632c2, SUC_lexicon_tags_31 },
    { 0xc49632c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f632c6, SUC_lexicon_tags_1 },
    { 0xcb3632c6, SUC_lexicon_tags_11 },
    { 0xd7632c6, SUC_lexicon_tags_49 },
    { 0xd6b632c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19632cf, SUC_lexicon_tags_5 },
    { 0xff632d0, SUC_lexicon_tags_25 },
    { 0xb41632d0, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xc85632d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79632d6, SUC_lexicon_tags_17 },
    { 0xf07632d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3f3632d9, SUC_lexicon_tags_37 },
    { 0x741632da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327632de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd632e1, SUC_lexicon_tags_7 },
    { 0x897632e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd632e5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5632e8, SUC_lexicon_tags_26 },
    { 0xc05632e9, SUC_lexicon_tags_17 },
    { 0x927632e9, SUC_lexicon_tags_349 },
    { 0x227632e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69632ef, SUC_lexicon_tags_11 },
    { 0xd3d632f0, SUC_lexicon_tags_60 },
    { 0x889632f1, SUC_lexicon_tags_27 },
    { 0x113632f2, SUC_lexicon_tags_0 },
    { 0x4f1632f3, SUC_lexicon_tags_54 },
    { 0xb11632f4, SUC_lexicon_tags_15 },
    { 0x6e9632f5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21632f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64d632fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73632ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb63304, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1963306, SUC_lexicon_tags_43 },
    { 0x6d563307, SUC_lexicon_tags_2 },
    { 0x6f63307, SUC_lexicon_tags_23 },
    { 0x77d63307, SUC_lexicon_tags_17 },
    { 0xe176330a, SUC_lexicon_tags_15 },
    { 0x8e563309, SUC_lexicon_tags_15 },
    { 0xce16330c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe796330f, SUC_lexicon_tags_37 },
    { 0xcf56330f, SUC_lexicon_tags_37 },
    { 0x15763311, SUC_lexicon_tags_7 },
    { 0x32563312, SUC_lexicon_tags_5 },
    { 0x8f63311, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24163318, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f6331d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c163322, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d563324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4763326, SUC_lexicon_tags_47 },
    { 0x52963326, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44f63329, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab6332f, SUC_lexicon_tags_11 },
    { 0xb3963330, SUC_lexicon_tags_151 },
    { 0x5d163331, SUC_lexicon_tags_37 },
    { 0x89763332, SUC_lexicon_tags_37 },
    { 0x3e36332f, SUC_lexicon_tags_19 },
    { 0xd9763334, SUC_lexicon_tags_20 },
    { 0x81563335, SUC_lexicon_tags_17 },
    { 0x1d63335, SUC_lexicon_tags_2 },
    { 0x2dd63335, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b6333a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe656333e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x49b63340, SUC_lexicon_tags_26 },
    { 0x84b63340, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f963344, SUC_lexicon_tags_47 },
    { 0x44763345, SUC_lexicon_tags_19 },
    { 0x19b63346, SUC_lexicon_tags_37 },
    { 0x26363347, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd63352, SUC_lexicon_tags_33 },
    { 0x9a163353, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b763355, SUC_lexicon_tags_37 },
    { 0xbd163356, SUC_lexicon_tags_26 },
    { 0x20563355, SUC_lexicon_tags_25 },
    { 0x60b63358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff6335b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa696335d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8163362, SUC_lexicon_tags_39 },
    { 0xd8d63363, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b63369, SUC_lexicon_tags_12 },
    { 0x3356336a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7bf6336c, SUC_lexicon_tags_237 },
    { 0xb2b6336c, SUC_lexicon_tags_87 },
    { 0x4c16336e, SUC_lexicon_tags_61 },
    { 0x8056336f, SUC_lexicon_tags_11 },
    { 0x1d76336f, SUC_lexicon_tags_6 },
    { 0x4a363371, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6c763373, SUC_lexicon_tags_1607 },
    { 0, NULL },
    { 0x81d63375, SUC_lexicon_tags_25 },
    { 0x6cf63376, SUC_lexicon_tags_17 },
    { 0xa4d63377, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d6337b, SUC_lexicon_tags_17 },
    { 0x8496337b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43f6337e, SUC_lexicon_tags_27 },
    { 0x4716337f, SUC_lexicon_tags_7 },
    { 0x1eb63380, SUC_lexicon_tags_66 },
    { 0x78363381, SUC_lexicon_tags_0 },
    { 0xcad63382, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af63387, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a36338b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6936338e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f63392, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d6339a, SUC_lexicon_tags_55 },
    { 0x67b6339a, SUC_lexicon_tags_4 },
    { 0xf216339a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2896339e, SUC_lexicon_tags_15 },
    { 0x7a36339e, SUC_lexicon_tags_17 },
    { 0xef9633a0, SUC_lexicon_tags_25 },
    { 0x2cf6339e, SUC_lexicon_tags_2 },
    { 0xb87633a1, SUC_lexicon_tags_37 },
    { 0x549633a3, SUC_lexicon_tags_17 },
    { 0x14f633a4, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df633a7, SUC_lexicon_tags_79 },
    { 0x88b633a7, SUC_lexicon_tags_10 },
    { 0x95b633a9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb6f633ab, SUC_lexicon_tags_27 },
    { 0xca3633ac, SUC_lexicon_tags_12 },
    { 0xb25633ab, SUC_lexicon_tags_60 },
    { 0x3bd633ab, SUC_lexicon_tags_34 },
    { 0x815633ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74b633b1, SUC_lexicon_tags_3 },
    { 0xf3d633b2, SUC_lexicon_tags_33 },
    { 0x741633b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17633b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d633bc, SUC_lexicon_tags_25 },
    { 0xa25633bd, SUC_lexicon_tags_27 },
    { 0x853633be, SUC_lexicon_tags_45 },
    { 0xced633bf, SUC_lexicon_tags_38 },
    { 0xdbb633c0, SUC_lexicon_tags_21 },
    { 0xead633be, SUC_lexicon_tags_11 },
    { 0x3bb633c1, SUC_lexicon_tags_65 },
    { 0x11f633c3, SUC_lexicon_tags_87 },
    { 0x9ad633c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xccf633c6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6bb633c8, SUC_lexicon_tags_9 },
    { 0x19b633c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df633d3, SUC_lexicon_tags_16 },
    { 0xea9633d4, SUC_lexicon_tags_23 },
    { 0x923633d5, SUC_lexicon_tags_40 },
    { 0x435633d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217633dc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa1633de, SUC_lexicon_tags_32 },
    { 0x849633df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183633e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f633ea, SUC_lexicon_tags_12 },
    { 0x4f7633ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01633ef, SUC_lexicon_tags_26 },
    { 0x105633ef, SUC_lexicon_tags_6 },
    { 0x4e9633f1, SUC_lexicon_tags_62 },
    { 0x1633f2, SUC_lexicon_tags_17 },
    { 0xfb3633f3, SUC_lexicon_tags_11 },
    { 0x50f633f2, SUC_lexicon_tags_34 },
    { 0x4c1633f3, SUC_lexicon_tags_11 },
    { 0x32d633f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcb7633f8, SUC_lexicon_tags_60 },
    { 0x9db633f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d633fc, SUC_lexicon_tags_31 },
    { 0x98f633fd, SUC_lexicon_tags_1 },
    { 0x9cf633fe, SUC_lexicon_tags_7 },
    { 0xd79633fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b63403, SUC_lexicon_tags_29 },
    { 0xdbb63404, SUC_lexicon_tags_2 },
    { 0x63563404, SUC_lexicon_tags_15 },
    { 0x24d63406, SUC_lexicon_tags_11 },
    { 0x13763407, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34363409, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb56340b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d6340e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ed63410, SUC_lexicon_tags_63 },
    { 0xbc563410, SUC_lexicon_tags_1411 },
    { 0x1bf63412, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2763414, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f63419, SUC_lexicon_tags_39 },
    { 0x3856341a, SUC_lexicon_tags_83 },
    { 0x1f16341b, SUC_lexicon_tags_1 },
    { 0x2f76341c, SUC_lexicon_tags_7 },
    { 0x77b6341d, SUC_lexicon_tags_7 },
    { 0x2d96341d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1d63420, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcd363422, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d63429, SUC_lexicon_tags_146 },
    { 0x7896342a, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x5356342d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f63434, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4236343a, SUC_lexicon_tags_25 },
    { 0x8a96343b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x516343e, SUC_lexicon_tags_1 },
    { 0xc056343f, SUC_lexicon_tags_33 },
    { 0x7cd6343f, SUC_lexicon_tags_4 },
    { 0x5763440, SUC_lexicon_tags_37 },
    { 0xf163442, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbdd63444, SUC_lexicon_tags_14 },
    { 0x52763445, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6344a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d96344d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f63450, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x93963452, SUC_lexicon_tags_40 },
    { 0xb2163453, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb63458, SUC_lexicon_tags_9 },
    { 0x2563458, SUC_lexicon_tags_17 },
    { 0x7496345a, SUC_lexicon_tags_12 },
    { 0x50763458, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93763468, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b6346c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54963470, SUC_lexicon_tags_15 },
    { 0xf6b63471, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e163473, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8363476, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb6347b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf763482, SUC_lexicon_tags_14 },
    { 0x7eb63483, SUC_lexicon_tags_37 },
    { 0x22f63484, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbe363486, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6db63488, SUC_lexicon_tags_12 },
    { 0xbcd63489, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x896348c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe256348e, SUC_lexicon_tags_38 },
    { 0x69d6348e, SUC_lexicon_tags_0 },
    { 0x95363490, SUC_lexicon_tags_37 },
    { 0x4c963491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f16349a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa296349e, SUC_lexicon_tags_7 },
    { 0xad76349e, SUC_lexicon_tags_1 },
    { 0x9c1634a0, SUC_lexicon_tags_25 },
    { 0x8ed634a1, SUC_lexicon_tags_12 },
    { 0x90f634a1, SUC_lexicon_tags_14 },
    { 0x9d3634a1, SUC_lexicon_tags_63 },
    { 0xe9b634a3, SUC_lexicon_tags_13 },
    { 0x3bd634a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f634a8, SUC_lexicon_tags_33 },
    { 0x97f634a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f634ac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363634b1, SUC_lexicon_tags_17 },
    { 0x4a1634b1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817634b9, SUC_lexicon_tags_47 },
    { 0xa6f634ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17634c3, SUC_lexicon_tags_3 },
    { 0xe5634c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbc7634c6, SUC_lexicon_tags_66 },
    { 0x2d5634c6, SUC_lexicon_tags_11 },
    { 0x203634c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673634cf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xda1634d1, SUC_lexicon_tags_10 },
    { 0x2ff634d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3fb634d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1634d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7634db, SUC_lexicon_tags_19 },
    { 0xee3634dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d634e5, SUC_lexicon_tags_37 },
    { 0x601634e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d634ed, SUC_lexicon_tags_43 },
    { 0xf6b634ee, SUC_lexicon_tags_17 },
    { 0x4ff634ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1634f2, SUC_lexicon_tags_84 },
    { 0x439634f2, SUC_lexicon_tags_10 },
    { 0x1c3634f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x487634f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x715634f9, SUC_lexicon_tags_4 },
    { 0xeff634fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45634ff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67763505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73963507, SUC_lexicon_tags_1 },
    { 0xcb963508, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b6350d, SUC_lexicon_tags_12 },
    { 0x7696350e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48363510, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d63516, SUC_lexicon_tags_17 },
    { 0x39363517, SUC_lexicon_tags_17 },
    { 0xc5363518, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f6351b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x3116351e, SUC_lexicon_tags_0 },
    { 0x3a96351e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1763525, SUC_lexicon_tags_12 },
    { 0x64f63526, SUC_lexicon_tags_86 },
    { 0x79163525, SUC_lexicon_tags_37 },
    { 0x25563526, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f63535, SUC_lexicon_tags_37 },
    { 0x50963535, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb216353b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6816353f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x21163541, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb63544, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x80163547, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x87563549, SUC_lexicon_tags_16 },
    { 0x72b6354a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a36354d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b63552, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x31763554, SUC_lexicon_tags_25 },
    { 0xd5f63555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90163562, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf9363564, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd63567, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5056356e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe563574, SUC_lexicon_tags_25 },
    { 0xa1b63574, SUC_lexicon_tags_1 },
    { 0xcad63576, SUC_lexicon_tags_12 },
    { 0xafd63575, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56357b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b63580, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed563583, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e163589, SUC_lexicon_tags_2 },
    { 0x84f6358a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1763590, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56363592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd63596, SUC_lexicon_tags_39 },
    { 0xebb63597, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c96359e, SUC_lexicon_tags_7 },
    { 0xdd96359e, SUC_lexicon_tags_11 },
    { 0xfbf635a0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51635a4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x34d635a6, SUC_lexicon_tags_2 },
    { 0x17f635a7, SUC_lexicon_tags_2 },
    { 0xddf635a6, SUC_lexicon_tags_25 },
    { 0x4e7635a9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf69635ab, SUC_lexicon_tags_11 },
    { 0x70b635ac, SUC_lexicon_tags_47 },
    { 0xdd1635ad, SUC_lexicon_tags_30 },
    { 0x657635ae, SUC_lexicon_tags_27 },
    { 0x94f635af, SUC_lexicon_tags_46 },
    { 0x725635af, SUC_lexicon_tags_47 },
    { 0xcc3635ab, SUC_lexicon_tags_30 },
    { 0xc19635ad, SUC_lexicon_tags_7 },
    { 0xa5b635ab, SUC_lexicon_tags_16 },
    { 0xc55635b4, SUC_lexicon_tags_1 },
    { 0x22d635b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d635b9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc5635bb, SUC_lexicon_tags_12 },
    { 0x935635bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5635c0, SUC_lexicon_tags_11 },
    { 0x521635c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7635c5, SUC_lexicon_tags_27 },
    { 0x8bd635c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb03635c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9635cb, SUC_lexicon_tags_50 },
    { 0xeb635cc, SUC_lexicon_tags_17 },
    { 0x4bd635cc, SUC_lexicon_tags_63 },
    { 0xbc5635ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427635d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f635d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x599635d8, SUC_lexicon_tags_26 },
    { 0xbed635d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc53635db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8d635dd, SUC_lexicon_tags_25 },
    { 0x85635de, SUC_lexicon_tags_42 },
    { 0x5b5635df, SUC_lexicon_tags_25 },
    { 0x32d635de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd635e3, SUC_lexicon_tags_6 },
    { 0xf7635e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x381635e6, SUC_lexicon_tags_38 },
    { 0x7635e7, SUC_lexicon_tags_15 },
    { 0xe55635e8, SUC_lexicon_tags_2 },
    { 0x471635e7, SUC_lexicon_tags_2 },
    { 0x519635e6, SUC_lexicon_tags_25 },
    { 0x8b1635eb, SUC_lexicon_tags_17 },
    { 0xdc3635ec, SUC_lexicon_tags_11 },
    { 0x4cb635eb, SUC_lexicon_tags_2 },
    { 0xe29635ec, SUC_lexicon_tags_30 },
    { 0xe87635ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9635f3, SUC_lexicon_tags_45 },
    { 0x37d635f4, SUC_lexicon_tags_37 },
    { 0x93b635f5, SUC_lexicon_tags_12 },
    { 0x42f635f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a3635f8, SUC_lexicon_tags_4 },
    { 0x359635f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af635ff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f63602, SUC_lexicon_tags_7 },
    { 0xe9963603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad6360d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb63610, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f63613, SUC_lexicon_tags_15 },
    { 0x3bd63613, SUC_lexicon_tags_19 },
    { 0x48563615, SUC_lexicon_tags_1 },
    { 0x77163615, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x37763619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7056361d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85763621, SUC_lexicon_tags_17 },
    { 0x33763621, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5563625, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x92563627, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7256362a, SUC_lexicon_tags_33 },
    { 0x3176362b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f6362e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb763631, SUC_lexicon_tags_1093 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75363635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60363638, SUC_lexicon_tags_2 },
    { 0x85563639, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f6363d, SUC_lexicon_tags_0 },
    { 0x1056363d, SUC_lexicon_tags_14 },
    { 0x8996363f, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf63642, SUC_lexicon_tags_43 },
    { 0x4bf63642, SUC_lexicon_tags_48 },
    { 0x7d763644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85f63646, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab6364e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe6f63650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41363656, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf916365a, SUC_lexicon_tags_45 },
    { 0x18d6365b, SUC_lexicon_tags_47 },
    { 0x8736365a, SUC_lexicon_tags_35 },
    { 0x92b6365c, SUC_lexicon_tags_45 },
    { 0x6cf6365d, SUC_lexicon_tags_70 },
    { 0xee36365c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af63662, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf63665, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10763668, SUC_lexicon_tags_27 },
    { 0x28363669, SUC_lexicon_tags_4 },
    { 0x3e96366a, SUC_lexicon_tags_43 },
    { 0x3d36366a, SUC_lexicon_tags_13 },
    { 0x4bd6366c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1936366e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0763673, SUC_lexicon_tags_37 },
    { 0x5bd63674, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7763677, SUC_lexicon_tags_25 },
    { 0xab363678, SUC_lexicon_tags_29 },
    { 0x79963677, SUC_lexicon_tags_9 },
    { 0x6ab6367a, SUC_lexicon_tags_37 },
    { 0x416367b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb6367f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf63686, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x79763689, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d56368b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad6368f, SUC_lexicon_tags_26 },
    { 0x73d63690, SUC_lexicon_tags_1 },
    { 0x7b963690, SUC_lexicon_tags_17 },
    { 0xdd63691, SUC_lexicon_tags_35 },
    { 0x48f63693, SUC_lexicon_tags_37 },
    { 0x86363693, SUC_lexicon_tags_30 },
    { 0xa7563694, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d63698, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f56369c, SUC_lexicon_tags_87 },
    { 0x9b76369d, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f636a1, SUC_lexicon_tags_86 },
    { 0x8eb636a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa77636a4, SUC_lexicon_tags_3 },
    { 0xd41636a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d7636a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb636ac, SUC_lexicon_tags_2 },
    { 0x42f636ac, SUC_lexicon_tags_29 },
    { 0xc87636ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c3636b0, SUC_lexicon_tags_25 },
    { 0x3e1636b1, SUC_lexicon_tags_43 },
    { 0x3d1636b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d636b5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x563636b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x625636bb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9636c4, SUC_lexicon_tags_17 },
    { 0x445636c4, SUC_lexicon_tags_37 },
    { 0x8f5636c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb9d636c8, SUC_lexicon_tags_11 },
    { 0x931636c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5636d0, SUC_lexicon_tags_37 },
    { 0xd45636d1, SUC_lexicon_tags_11 },
    { 0x669636d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x197636d4, SUC_lexicon_tags_47 },
    { 0x335636d5, SUC_lexicon_tags_1 },
    { 0x4e9636d6, SUC_lexicon_tags_25 },
    { 0xc33636d4, SUC_lexicon_tags_11 },
    { 0x789636d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331636dc, SUC_lexicon_tags_25 },
    { 0x87d636dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc81636df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d636e3, SUC_lexicon_tags_11 },
    { 0x6e9636e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbd9636e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x887636e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03636f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9636fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38163700, SUC_lexicon_tags_2 },
    { 0x13563700, SUC_lexicon_tags_28 },
    { 0xd7d63702, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xa9963704, SUC_lexicon_tags_47 },
    { 0xeff63705, SUC_lexicon_tags_11 },
    { 0x80b63706, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc56370b, SUC_lexicon_tags_15 },
    { 0xb6f6370c, SUC_lexicon_tags_11 },
    { 0x6b6370d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20163712, SUC_lexicon_tags_42 },
    { 0xf3563713, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d63718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d96371e, SUC_lexicon_tags_37 },
    { 0xb8b6371f, SUC_lexicon_tags_37 },
    { 0x82563720, SUC_lexicon_tags_65 },
    { 0xe4d63721, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4363725, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f63729, SUC_lexicon_tags_8 },
    { 0xed6372a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1896372e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb63733, SUC_lexicon_tags_15 },
    { 0xc8363734, SUC_lexicon_tags_40 },
    { 0xb4763734, SUC_lexicon_tags_35 },
    { 0x23b63736, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c76373b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc936373e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b63746, SUC_lexicon_tags_2 },
    { 0x4363746, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86d63749, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xec16374c, SUC_lexicon_tags_14 },
    { 0xd536374c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8963750, SUC_lexicon_tags_25 },
    { 0x4ef63751, SUC_lexicon_tags_17 },
    { 0x87763751, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76163754, SUC_lexicon_tags_1 },
    { 0xdb563755, SUC_lexicon_tags_12 },
    { 0x82763755, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xae363759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x316375c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b6375f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe163761, SUC_lexicon_tags_11 },
    { 0x8e563762, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb63769, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b36376b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6db6376d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3556376f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x55363772, SUC_lexicon_tags_15 },
    { 0x11d63773, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9b63775, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab6377a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9163780, SUC_lexicon_tags_80 },
    { 0x4ad63780, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6136378b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7db6378d, SUC_lexicon_tags_1 },
    { 0x3976378d, SUC_lexicon_tags_12 },
    { 0x9016378f, SUC_lexicon_tags_22 },
    { 0xd96378f, SUC_lexicon_tags_25 },
    { 0x8e16378f, SUC_lexicon_tags_26 },
    { 0xc3563790, SUC_lexicon_tags_12 },
    { 0x74963793, SUC_lexicon_tags_14 },
    { 0x48963794, SUC_lexicon_tags_25 },
    { 0xdf36378d, SUC_lexicon_tags_45 },
    { 0x59f63794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x363798, SUC_lexicon_tags_4 },
    { 0xec363799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf396379d, SUC_lexicon_tags_6 },
    { 0x1c56379e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x333637a1, SUC_lexicon_tags_37 },
    { 0xf1f637a2, SUC_lexicon_tags_7 },
    { 0x781637a3, SUC_lexicon_tags_49 },
    { 0xd77637a2, SUC_lexicon_tags_33 },
    { 0x87f637a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x179637a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45637ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d637af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd637b7, SUC_lexicon_tags_26 },
    { 0x805637b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d637bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db637bf, SUC_lexicon_tags_17 },
    { 0x5ed637c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43637c5, SUC_lexicon_tags_68 },
    { 0x813637c5, SUC_lexicon_tags_29 },
    { 0xe19637c7, SUC_lexicon_tags_1 },
    { 0x633637c7, SUC_lexicon_tags_27 },
    { 0xfb5637c7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f637cc, SUC_lexicon_tags_29 },
    { 0xc31637cc, SUC_lexicon_tags_2 },
    { 0xfd1637cd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9637d0, SUC_lexicon_tags_7 },
    { 0x731637d1, SUC_lexicon_tags_37 },
    { 0x7fb637d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd637d6, SUC_lexicon_tags_2 },
    { 0x4fb637d7, SUC_lexicon_tags_26 },
    { 0x321637d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557637df, SUC_lexicon_tags_11 },
    { 0xb0b637df, SUC_lexicon_tags_57 },
    { 0x847637e0, SUC_lexicon_tags_7 },
    { 0xd3f637e2, SUC_lexicon_tags_33 },
    { 0x64b637e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77637e7, SUC_lexicon_tags_17 },
    { 0xafb637e8, SUC_lexicon_tags_48 },
    { 0x4dd637e9, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9637f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b637f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb637fc, SUC_lexicon_tags_124 },
    { 0x4e7637fd, SUC_lexicon_tags_30 },
    { 0x7d637fe, SUC_lexicon_tags_29 },
    { 0xdad637fc, SUC_lexicon_tags_6 },
    { 0x865637fe, SUC_lexicon_tags_27 },
    { 0x73563801, SUC_lexicon_tags_47 },
    { 0x92763801, SUC_lexicon_tags_1 },
    { 0x64563803, SUC_lexicon_tags_28 },
    { 0xf8563804, SUC_lexicon_tags_23 },
    { 0xc9363805, SUC_lexicon_tags_23 },
    { 0xd2963804, SUC_lexicon_tags_25 },
    { 0x28b63806, SUC_lexicon_tags_49 },
    { 0x7a963808, SUC_lexicon_tags_25 },
    { 0x66f63806, SUC_lexicon_tags_30 },
    { 0xd9b6380a, SUC_lexicon_tags_10 },
    { 0x3196380b, SUC_lexicon_tags_15 },
    { 0x3d6380c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7f36380e, SUC_lexicon_tags_8 },
    { 0x816380f, SUC_lexicon_tags_18 },
    { 0x3596380f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88763816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0363818, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3896381a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e56381c, SUC_lexicon_tags_32 },
    { 0xc276381c, SUC_lexicon_tags_7 },
    { 0x7356381e, SUC_lexicon_tags_17 },
    { 0xd3b6381c, SUC_lexicon_tags_109 },
    { 0x9076381c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3e163822, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba963827, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d363830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11763832, SUC_lexicon_tags_1 },
    { 0xd1163833, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcfd63835, SUC_lexicon_tags_15 },
    { 0x28d63836, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb796383c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9176383f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89763842, SUC_lexicon_tags_29 },
    { 0x15b63843, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d56384a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x896384d, SUC_lexicon_tags_30 },
    { 0xa196384e, SUC_lexicon_tags_46 },
    { 0xd56384f, SUC_lexicon_tags_45 },
    { 0x6736384e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9b63852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5963855, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x68b63857, SUC_lexicon_tags_37 },
    { 0x13f63857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f16385e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf516386b, SUC_lexicon_tags_77 },
    { 0xd4d6386c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdd56386e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4163871, SUC_lexicon_tags_9 },
    { 0x97d63872, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37563879, SUC_lexicon_tags_14 },
    { 0xbaf6387a, SUC_lexicon_tags_70 },
    { 0x76f6387b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9856387d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5936387f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52963887, SUC_lexicon_tags_2 },
    { 0x1af63888, SUC_lexicon_tags_39 },
    { 0xa8f63888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d6388e, SUC_lexicon_tags_37 },
    { 0xc8f6388f, SUC_lexicon_tags_11 },
    { 0xcf63890, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe363893, SUC_lexicon_tags_16 },
    { 0x7f963893, SUC_lexicon_tags_15 },
    { 0x81963894, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x53763897, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xfef63899, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2636389c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c56389e, SUC_lexicon_tags_62 },
    { 0x68d6389f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x635638a2, SUC_lexicon_tags_42 },
    { 0x295638a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f638a6, SUC_lexicon_tags_2 },
    { 0x481638a7, SUC_lexicon_tags_35 },
    { 0x299638a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b638ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41638ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbed638b0, SUC_lexicon_tags_2 },
    { 0xc5b638b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x143638b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x265638b5, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x43f638b7, SUC_lexicon_tags_11 },
    { 0x4a1638b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d3638ba, SUC_lexicon_tags_15 },
    { 0xca5638bb, SUC_lexicon_tags_7 },
    { 0xe81638bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3c9638be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c1638c0, SUC_lexicon_tags_6 },
    { 0xb03638c1, SUC_lexicon_tags_11 },
    { 0x151638c2, SUC_lexicon_tags_34 },
    { 0x633638c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95f638c5, SUC_lexicon_tags_37 },
    { 0xbf638c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x723638c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef638cc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe39638ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad5638d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9638d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5638d7, SUC_lexicon_tags_11 },
    { 0xaef638d7, SUC_lexicon_tags_37 },
    { 0xf9d638d9, SUC_lexicon_tags_37 },
    { 0x80f638da, SUC_lexicon_tags_9 },
    { 0xe9d638da, SUC_lexicon_tags_30 },
    { 0xf6f638dc, SUC_lexicon_tags_13 },
    { 0xa71638dd, SUC_lexicon_tags_66 },
    { 0x6bd638de, SUC_lexicon_tags_28 },
    { 0x597638df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9bb638e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3638e7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3638f1, SUC_lexicon_tags_38 },
    { 0xaab638f2, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x699638f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef638f7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247638fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbff638fe, SUC_lexicon_tags_37 },
    { 0xf65638ff, SUC_lexicon_tags_37 },
    { 0xe9163900, SUC_lexicon_tags_3 },
    { 0x759638ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40f63903, SUC_lexicon_tags_182 },
    { 0xc5b63904, SUC_lexicon_tags_25 },
    { 0x44163904, SUC_lexicon_tags_12 },
    { 0x62763906, SUC_lexicon_tags_0 },
    { 0x48563906, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb6390b, SUC_lexicon_tags_63 },
    { 0xe7f6390c, SUC_lexicon_tags_82 },
    { 0x3bf6390d, SUC_lexicon_tags_85 },
    { 0x5556390e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5363914, SUC_lexicon_tags_2 },
    { 0x95563915, SUC_lexicon_tags_25 },
    { 0xd9163916, SUC_lexicon_tags_13 },
    { 0x7a163916, SUC_lexicon_tags_63 },
    { 0x9f563915, SUC_lexicon_tags_63 },
    { 0xa2f63918, SUC_lexicon_tags_25 },
    { 0x5a16391a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1f6391c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb763922, SUC_lexicon_tags_15 },
    { 0x12f63923, SUC_lexicon_tags_15 },
    { 0xadf63924, SUC_lexicon_tags_2 },
    { 0x80b63923, SUC_lexicon_tags_11 },
    { 0x83d63924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd963929, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf6392e, SUC_lexicon_tags_49 },
    { 0xdb96392f, SUC_lexicon_tags_2 },
    { 0xb756392f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ed63932, SUC_lexicon_tags_23 },
    { 0x1e563933, SUC_lexicon_tags_44 },
    { 0x9eb63934, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8163936, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d563939, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f96393d, SUC_lexicon_tags_2 },
    { 0xe716393d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d63942, SUC_lexicon_tags_37 },
    { 0x48b63943, SUC_lexicon_tags_1 },
    { 0x51363943, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc163948, SUC_lexicon_tags_6 },
    { 0x1b563948, SUC_lexicon_tags_33 },
    { 0x6156394a, SUC_lexicon_tags_14 },
    { 0xb0b6394b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9476394f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf1d63951, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b763954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf63957, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b36395a, SUC_lexicon_tags_37 },
    { 0x7e96395a, SUC_lexicon_tags_33 },
    { 0xf376395c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43763961, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f563964, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x88363967, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f56396a, SUC_lexicon_tags_45 },
    { 0x5d16396b, SUC_lexicon_tags_27 },
    { 0x1db6396c, SUC_lexicon_tags_14 },
    { 0x7fd6396b, SUC_lexicon_tags_43 },
    { 0x9c96396e, SUC_lexicon_tags_17 },
    { 0x5236396e, SUC_lexicon_tags_1 },
    { 0xf763970, SUC_lexicon_tags_37 },
    { 0x42363971, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cd63973, SUC_lexicon_tags_42 },
    { 0x14b63974, SUC_lexicon_tags_8 },
    { 0x57563975, SUC_lexicon_tags_14 },
    { 0x7b963975, SUC_lexicon_tags_19 },
    { 0x78763977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3976397f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5ed63981, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f63984, SUC_lexicon_tags_17 },
    { 0xb5563984, SUC_lexicon_tags_37 },
    { 0x84163985, SUC_lexicon_tags_105 },
    { 0x6ed63987, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a563989, SUC_lexicon_tags_25 },
    { 0x7736398a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x596398d, SUC_lexicon_tags_12 },
    { 0x9876398e, SUC_lexicon_tags_47 },
    { 0x5dd6398f, SUC_lexicon_tags_31 },
    { 0x9316398d, SUC_lexicon_tags_28 },
    { 0x763991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c763994, SUC_lexicon_tags_19 },
    { 0xccb63994, SUC_lexicon_tags_37 },
    { 0x42563996, SUC_lexicon_tags_57 },
    { 0xaf763996, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7639a2, SUC_lexicon_tags_33 },
    { 0x10d639a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x823639a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7639a9, SUC_lexicon_tags_25 },
    { 0xd53639a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b5639ac, SUC_lexicon_tags_38 },
    { 0x1dd639ad, SUC_lexicon_tags_11 },
    { 0xe1639ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x141639b1, SUC_lexicon_tags_63 },
    { 0x8ad639b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc03639b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b639b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x289639bb, SUC_lexicon_tags_44 },
    { 0x467639bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x123639be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9639c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69639cb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89639ce, SUC_lexicon_tags_37 },
    { 0xcf3639cf, SUC_lexicon_tags_0 },
    { 0x395639ce, SUC_lexicon_tags_17 },
    { 0xe21639d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53639d7, SUC_lexicon_tags_0 },
    { 0xe5b639d8, SUC_lexicon_tags_2 },
    { 0x94d639d8, SUC_lexicon_tags_11 },
    { 0x693639da, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed639df, SUC_lexicon_tags_46 },
    { 0xb01639e0, SUC_lexicon_tags_26 },
    { 0x933639e0, SUC_lexicon_tags_47 },
    { 0xa79639e1, SUC_lexicon_tags_25 },
    { 0xc2d639e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b639e6, SUC_lexicon_tags_39 },
    { 0x667639e6, SUC_lexicon_tags_6 },
    { 0x9f9639e8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f639ec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671639f1, SUC_lexicon_tags_10 },
    { 0x4d1639f2, SUC_lexicon_tags_19 },
    { 0xae5639f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819639f7, SUC_lexicon_tags_27 },
    { 0x687639f8, SUC_lexicon_tags_37 },
    { 0x379639f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x875639fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a563a03, SUC_lexicon_tags_7 },
    { 0x30163a04, SUC_lexicon_tags_37 },
    { 0x4b763a05, SUC_lexicon_tags_37 },
    { 0x90363a04, SUC_lexicon_tags_52 },
    { 0xbed63a06, SUC_lexicon_tags_1 },
    { 0xe2563a05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x40f63a0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc763a0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa363a11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8563a19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd3163a1b, SUC_lexicon_tags_2 },
    { 0xbd63a1b, SUC_lexicon_tags_47 },
    { 0xf3d63a1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53b63a1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86163a23, SUC_lexicon_tags_11 },
    { 0x51d63a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34763a26, SUC_lexicon_tags_129 },
    { 0x81d63a27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8563a2b, SUC_lexicon_tags_23 },
    { 0xc8f63a2b, SUC_lexicon_tags_289 },
    { 0xe2763a2d, SUC_lexicon_tags_61 },
    { 0x95163a2e, SUC_lexicon_tags_26 },
    { 0x51d63a2b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46963a33, SUC_lexicon_tags_11 },
    { 0x18763a34, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b963a36, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b63a39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb363a3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70363a43, SUC_lexicon_tags_12 },
    { 0xd363a44, SUC_lexicon_tags_25 },
    { 0xd4363a45, SUC_lexicon_tags_53 },
    { 0xb3163a45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef63a49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b63a4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72163a52, SUC_lexicon_tags_47 },
    { 0x94363a52, SUC_lexicon_tags_11 },
    { 0xdf363a54, SUC_lexicon_tags_11 },
    { 0xf2363a53, SUC_lexicon_tags_1 },
    { 0x4f763a56, SUC_lexicon_tags_7 },
    { 0xf4d63a57, SUC_lexicon_tags_12 },
    { 0x95963a58, SUC_lexicon_tags_45 },
    { 0xb2563a57, SUC_lexicon_tags_27 },
    { 0xc6163a59, SUC_lexicon_tags_15 },
    { 0xdf963a5b, SUC_lexicon_tags_63 },
    { 0x65f63a5c, SUC_lexicon_tags_56 },
    { 0x2f363a58, SUC_lexicon_tags_7 },
    { 0x57163a5e, SUC_lexicon_tags_29 },
    { 0x7f63a5f, SUC_lexicon_tags_146 },
    { 0xd2f63a5a, SUC_lexicon_tags_37 },
    { 0x42b63a5d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x24563a64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa0f63a66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78563a6b, SUC_lexicon_tags_38 },
    { 0xf7b63a6b, SUC_lexicon_tags_7 },
    { 0x9ef63a6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x72f63a6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f363a71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5b63a73, SUC_lexicon_tags_26 },
    { 0x9963a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d63a7a, SUC_lexicon_tags_1 },
    { 0xbf63a7a, SUC_lexicon_tags_15 },
    { 0x4563a7b, SUC_lexicon_tags_23 },
    { 0xe7563a7c, SUC_lexicon_tags_12 },
    { 0xdbd63a7a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b763a81, SUC_lexicon_tags_39 },
    { 0x86d63a82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f763a84, SUC_lexicon_tags_7 },
    { 0x9d563a85, SUC_lexicon_tags_4 },
    { 0xa363a86, SUC_lexicon_tags_60 },
    { 0x7963a87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe6163a89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe0d63a8b, SUC_lexicon_tags_15 },
    { 0x58b63a8c, SUC_lexicon_tags_39 },
    { 0xe3d63a8d, SUC_lexicon_tags_4 },
    { 0xbaf63a8d, SUC_lexicon_tags_15 },
    { 0xf0163a8c, SUC_lexicon_tags_7 },
    { 0xe2763a90, SUC_lexicon_tags_38 },
    { 0xda163a8c, SUC_lexicon_tags_43 },
    { 0xe0f63a8c, SUC_lexicon_tags_35 },
    { 0x9b63a93, SUC_lexicon_tags_17 },
    { 0x47763a8f, SUC_lexicon_tags_25 },
    { 0xbc363a95, SUC_lexicon_tags_7 },
    { 0x6cf63a96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x22363a99, SUC_lexicon_tags_18 },
    { 0x80363a9a, SUC_lexicon_tags_12 },
    { 0x5b563a9a, SUC_lexicon_tags_7 },
    { 0x4cf63a9c, SUC_lexicon_tags_25 },
    { 0xdd963a9b, SUC_lexicon_tags_57 },
    { 0x3e963a9c, SUC_lexicon_tags_47 },
    { 0xc2363a9f, SUC_lexicon_tags_1 },
    { 0x53763aa0, SUC_lexicon_tags_1 },
    { 0x6f63aa0, SUC_lexicon_tags_31 },
    { 0x85763aa2, SUC_lexicon_tags_25 },
    { 0x4c163aa2, SUC_lexicon_tags_0 },
    { 0xa4f63aa3, SUC_lexicon_tags_57 },
    { 0x48363aa5, SUC_lexicon_tags_21 },
    { 0xcbd63aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89363aa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a563aaa, SUC_lexicon_tags_16 },
    { 0xa5963aaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d63ab0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa763ab8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8763abc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f63abf, SUC_lexicon_tags_25 },
    { 0x8f563ac0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f63ac3, SUC_lexicon_tags_28 },
    { 0x35563ac4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e363ac8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1b63aca, SUC_lexicon_tags_37 },
    { 0xcd163acb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff63ad1, SUC_lexicon_tags_12 },
    { 0x78163ad2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x49963ad4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5563ad6, SUC_lexicon_tags_1032 },
    { 0, NULL },
    { 0xf4963ad8, SUC_lexicon_tags_37 },
    { 0xa5d63ad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4163adb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e163adf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec763ae7, SUC_lexicon_tags_11 },
    { 0x1df63ae8, SUC_lexicon_tags_31 },
    { 0xf3963ae9, SUC_lexicon_tags_2 },
    { 0x5a963ae9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb63aed, SUC_lexicon_tags_37 },
    { 0x9f163aee, SUC_lexicon_tags_28 },
    { 0xdcf63aee, SUC_lexicon_tags_19 },
    { 0x59563af0, SUC_lexicon_tags_2 },
    { 0xa5763aee, SUC_lexicon_tags_11 },
    { 0xb6d63aee, SUC_lexicon_tags_15 },
    { 0xb3b63af3, SUC_lexicon_tags_12 },
    { 0x4f563aed, SUC_lexicon_tags_48 },
    { 0x7e963af5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f63af8, SUC_lexicon_tags_29 },
    { 0x7a563af9, SUC_lexicon_tags_2 },
    { 0x7ad63afa, SUC_lexicon_tags_47 },
    { 0xd5f63afb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4363afd, SUC_lexicon_tags_7 },
    { 0x99b63afd, SUC_lexicon_tags_43 },
    { 0xbff63aff, SUC_lexicon_tags_63 },
    { 0x19763b00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b63b07, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1a163b09, SUC_lexicon_tags_19 },
    { 0x91f63b0a, SUC_lexicon_tags_21 },
    { 0x6ff63b09, SUC_lexicon_tags_62 },
    { 0xbb563b09, SUC_lexicon_tags_1 },
    { 0xb6363b0c, SUC_lexicon_tags_1 },
    { 0x63163b0e, SUC_lexicon_tags_47 },
    { 0xec363b0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7563b12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42563b15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50563b1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81963b1d, SUC_lexicon_tags_40 },
    { 0xed63b1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0763b21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde563b26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20563b2d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3963b31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20f63b33, SUC_lexicon_tags_7 },
    { 0x41763b33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d63b38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5b363b3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x55d63b3c, SUC_lexicon_tags_21 },
    { 0xb1763b3d, SUC_lexicon_tags_32 },
    { 0x22763b3e, SUC_lexicon_tags_26 },
    { 0x79363b3f, SUC_lexicon_tags_112 },
    { 0x41f63b40, SUC_lexicon_tags_11 },
    { 0xd2363b3c, SUC_lexicon_tags_38 },
    { 0x52963b42, SUC_lexicon_tags_17 },
    { 0xbed63b43, SUC_lexicon_tags_0 },
    { 0x75563b43, SUC_lexicon_tags_48 },
    { 0xcb163b42, SUC_lexicon_tags_15 },
    { 0xd763b46, SUC_lexicon_tags_27 },
    { 0x2fd63b44, SUC_lexicon_tags_14 },
    { 0x94d63b48, SUC_lexicon_tags_27 },
    { 0xdf363b49, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63b4e, SUC_lexicon_tags_1 },
    { 0x26963b4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9363b51, SUC_lexicon_tags_17 },
    { 0x88d63b52, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xc4563b54, SUC_lexicon_tags_26 },
    { 0x75563b55, SUC_lexicon_tags_70 },
    { 0xf5f63b56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea963b5c, SUC_lexicon_tags_15 },
    { 0xe9363b5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d63b60, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x11363b63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2563b6e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f63b71, SUC_lexicon_tags_15 },
    { 0xa8363b72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17b63b74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17563b76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19363b7c, SUC_lexicon_tags_45 },
    { 0x9a763b7d, SUC_lexicon_tags_39 },
    { 0x8ad63b7d, SUC_lexicon_tags_38 },
    { 0xa5b63b7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8963b83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c963b85, SUC_lexicon_tags_17 },
    { 0x7d763b85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68163b8b, SUC_lexicon_tags_17 },
    { 0x25563b8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b63b90, SUC_lexicon_tags_25 },
    { 0xde763b91, SUC_lexicon_tags_77 },
    { 0x7b563b91, SUC_lexicon_tags_14 },
    { 0x74563b93, SUC_lexicon_tags_31 },
    { 0x52f63b92, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f63b98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xefd63b9a, SUC_lexicon_tags_0 },
    { 0x46b63b9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec963ba0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53d63ba2, SUC_lexicon_tags_12 },
    { 0x45363ba3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f963bac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58163baf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x51963bb2, SUC_lexicon_tags_18 },
    { 0x3c963bb2, SUC_lexicon_tags_37 },
    { 0x38763bb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd63bb7, SUC_lexicon_tags_45 },
    { 0x8ad63bb8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8963bba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d363bbc, SUC_lexicon_tags_38 },
    { 0x9db63bbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x21563bc0, SUC_lexicon_tags_25 },
    { 0x8b363bc1, SUC_lexicon_tags_23 },
    { 0xf8763bc1, SUC_lexicon_tags_21 },
    { 0x20763bc1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c363bc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6563bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f63bcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c163bd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad63bd8, SUC_lexicon_tags_57 },
    { 0xd5163bd9, SUC_lexicon_tags_7 },
    { 0x80563bda, SUC_lexicon_tags_60 },
    { 0xb2b63bda, SUC_lexicon_tags_37 },
    { 0x98163bdc, SUC_lexicon_tags_14 },
    { 0x56363bdb, SUC_lexicon_tags_3 },
    { 0x2563bdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc163be0, SUC_lexicon_tags_68 },
    { 0xe6f63be0, SUC_lexicon_tags_45 },
    { 0x5df63be2, SUC_lexicon_tags_37 },
    { 0x9d363be3, SUC_lexicon_tags_37 },
    { 0xe3763be4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf8163be6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a763be9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbd963beb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x68b63bed, SUC_lexicon_tags_77 },
    { 0x7d763bee, SUC_lexicon_tags_10 },
    { 0xd0363bef, SUC_lexicon_tags_62 },
    { 0xce363bf0, SUC_lexicon_tags_25 },
    { 0xf7363bf0, SUC_lexicon_tags_25 },
    { 0xc7b63bf2, SUC_lexicon_tags_9 },
    { 0x50163bf2, SUC_lexicon_tags_37 },
    { 0xea163bed, SUC_lexicon_tags_2 },
    { 0x7ef63bf5, SUC_lexicon_tags_37 },
    { 0x163bf0, SUC_lexicon_tags_15 },
    { 0xb5d63bf7, SUC_lexicon_tags_12 },
    { 0x26963bf7, SUC_lexicon_tags_21 },
    { 0x14563bf7, SUC_lexicon_tags_27 },
    { 0x92b63bfa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb563bfe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x99563c00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1e963c02, SUC_lexicon_tags_34 },
    { 0x82b63c03, SUC_lexicon_tags_592 },
    { 0xce163c04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e563c08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f63c0b, SUC_lexicon_tags_25 },
    { 0x77b63c0b, SUC_lexicon_tags_11 },
    { 0x7c563c0b, SUC_lexicon_tags_65 },
    { 0x8d963c0c, SUC_lexicon_tags_17 },
    { 0x3d63c0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf1963c11, SUC_lexicon_tags_47 },
    { 0xf5d63c11, SUC_lexicon_tags_1 },
    { 0x33b63c12, SUC_lexicon_tags_29 },
    { 0x59f63c14, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x43963c16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1c963c18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d63c1f, SUC_lexicon_tags_1 },
    { 0xeef63c20, SUC_lexicon_tags_26 },
    { 0x80363c21, SUC_lexicon_tags_26 },
    { 0xd1f63c20, SUC_lexicon_tags_15 },
    { 0xfb963c23, SUC_lexicon_tags_4 },
    { 0xebb63c24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0363c26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x14763c28, SUC_lexicon_tags_27 },
    { 0x63363c28, SUC_lexicon_tags_39 },
    { 0xcc763c28, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf63c2d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f63c32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb63c35, SUC_lexicon_tags_7 },
    { 0x3f763c36, SUC_lexicon_tags_15 },
    { 0x50d63c37, SUC_lexicon_tags_19 },
    { 0x73f63c37, SUC_lexicon_tags_37 },
    { 0xf0d63c36, SUC_lexicon_tags_25 },
    { 0x24563c3a, SUC_lexicon_tags_9 },
    { 0xd4163c3b, SUC_lexicon_tags_2 },
    { 0x1b963c3b, SUC_lexicon_tags_22 },
    { 0x52763c3d, SUC_lexicon_tags_26 },
    { 0x72163c3e, SUC_lexicon_tags_33 },
    { 0x52763c36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf63c43, SUC_lexicon_tags_25 },
    { 0x33d63c44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f63c47, SUC_lexicon_tags_25 },
    { 0xb8563c48, SUC_lexicon_tags_12 },
    { 0xbb563c48, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe5b63c4b, SUC_lexicon_tags_27 },
    { 0x97f63c4b, SUC_lexicon_tags_14 },
    { 0x5d763c4d, SUC_lexicon_tags_14 },
    { 0x57b63c4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27963c55, SUC_lexicon_tags_0 },
    { 0x3d163c56, SUC_lexicon_tags_7 },
    { 0xe1163c55, SUC_lexicon_tags_60 },
    { 0x93d63c55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a963c5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf963c5f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x66b63c61, SUC_lexicon_tags_25 },
    { 0xf1d63c61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f63c65, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x80163c68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16163c6d, SUC_lexicon_tags_2 },
    { 0x18163c6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcab63c70, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9563c7b, SUC_lexicon_tags_25 },
    { 0x51d63c7c, SUC_lexicon_tags_37 },
    { 0x39363c7b, SUC_lexicon_tags_3 },
    { 0x29963c7e, SUC_lexicon_tags_23 },
    { 0x1d963c7b, SUC_lexicon_tags_34 },
    { 0x28d63c80, SUC_lexicon_tags_56 },
    { 0xff63c7e, SUC_lexicon_tags_12 },
    { 0xb7f63c82, SUC_lexicon_tags_43 },
    { 0x93f63c83, SUC_lexicon_tags_0 },
    { 0xe7563c84, SUC_lexicon_tags_77 },
    { 0x68d63c83, SUC_lexicon_tags_60 },
    { 0x45b63c84, SUC_lexicon_tags_25 },
    { 0x99563c86, SUC_lexicon_tags_12 },
    { 0x51963c88, SUC_lexicon_tags_25 },
    { 0x45f63c89, SUC_lexicon_tags_25 },
    { 0xd7f63c8a, SUC_lexicon_tags_13 },
    { 0xb1363c7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db63c8e, SUC_lexicon_tags_11 },
    { 0xa2563c8f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8fb63c91, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a563c97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd63c9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72163ca0, SUC_lexicon_tags_11 },
    { 0x82563ca0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db63ca4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xec963ca7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf963cab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc163cb0, SUC_lexicon_tags_12 },
    { 0xaf63cb0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2bb63cb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcaf63cb5, SUC_lexicon_tags_30 },
    { 0x12763cb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10963cba, SUC_lexicon_tags_68 },
    { 0x61b63cbb, SUC_lexicon_tags_23 },
    { 0xac963cbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff63cc1, SUC_lexicon_tags_11 },
    { 0xd7763cc2, SUC_lexicon_tags_15 },
    { 0x72363cc1, SUC_lexicon_tags_65 },
    { 0x17763cc1, SUC_lexicon_tags_43 },
    { 0x31d63cc5, SUC_lexicon_tags_7 },
    { 0x13b63cc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87363cc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x43963ccb, SUC_lexicon_tags_14 },
    { 0x3c163ccb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7563cd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7363cd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcfb63cd9, SUC_lexicon_tags_11 },
    { 0xb1963cd9, SUC_lexicon_tags_23 },
    { 0x57163cdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8563cdf, SUC_lexicon_tags_22 },
    { 0xfd363ce0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7ab63ce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54563ce4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36763ce6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x45f63ce8, SUC_lexicon_tags_19 },
    { 0x93363ce9, SUC_lexicon_tags_7 },
    { 0xee363ce9, SUC_lexicon_tags_45 },
    { 0xdbd63ceb, SUC_lexicon_tags_21 },
    { 0x22963ce8, SUC_lexicon_tags_12 },
    { 0x3a163ced, SUC_lexicon_tags_1 },
    { 0xd5763ced, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a963cf1, SUC_lexicon_tags_47 },
    { 0x63b63cf2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8363cf4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f63cf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd63cfb, SUC_lexicon_tags_47 },
    { 0xa5963cfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97963d02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x52763d05, SUC_lexicon_tags_37 },
    { 0xed963d05, SUC_lexicon_tags_6 },
    { 0x35163d07, SUC_lexicon_tags_63 },
    { 0x67563d08, SUC_lexicon_tags_56 },
    { 0x1f563d06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf63d0d, SUC_lexicon_tags_34 },
    { 0xd8563d0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11163d18, SUC_lexicon_tags_26 },
    { 0xe3163d19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f63d1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb63d20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd0163d22, SUC_lexicon_tags_30 },
    { 0xa1163d23, SUC_lexicon_tags_30 },
    { 0xdf163d22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37163d28, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b63d2e, SUC_lexicon_tags_34 },
    { 0x9d363d2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb163d33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff63d36, SUC_lexicon_tags_17 },
    { 0xbf63d37, SUC_lexicon_tags_2 },
    { 0xe763d38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54363d3d, SUC_lexicon_tags_68 },
    { 0xddb63d3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb63d46, SUC_lexicon_tags_45 },
    { 0xa0763d46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b63d4e, SUC_lexicon_tags_37 },
    { 0x4ff63d4f, SUC_lexicon_tags_29 },
    { 0xb6d63d50, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf63d53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6163d56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xac763d59, SUC_lexicon_tags_6 },
    { 0x2ab63d59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f63d5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32963d5f, SUC_lexicon_tags_2 },
    { 0x4db63d60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d963d64, SUC_lexicon_tags_12 },
    { 0x86763d65, SUC_lexicon_tags_17 },
    { 0xc8163d64, SUC_lexicon_tags_74 },
    { 0x4cf63d67, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x44f63d69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf163d6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d563d6d, SUC_lexicon_tags_11 },
    { 0x7ad63d6e, SUC_lexicon_tags_25 },
    { 0xfc363d6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94163d72, SUC_lexicon_tags_11 },
    { 0x20563d73, SUC_lexicon_tags_79 },
    { 0xb5163d72, SUC_lexicon_tags_9 },
    { 0xb9963d75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d563d79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1163d7b, SUC_lexicon_tags_37 },
    { 0x24963d7c, SUC_lexicon_tags_16 },
    { 0x25563d7d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4963d8c, SUC_lexicon_tags_30 },
    { 0xa3763d8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85563d91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6163d95, SUC_lexicon_tags_61 },
    { 0xd7363d96, SUC_lexicon_tags_33 },
    { 0xa3163d97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef63d9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9563d9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b363dac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f963db1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d63db8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb363dbd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced63dc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20d63dc3, SUC_lexicon_tags_28 },
    { 0xce963dc3, SUC_lexicon_tags_23 },
    { 0x29763dc5, SUC_lexicon_tags_30 },
    { 0xf3563dc6, SUC_lexicon_tags_12 },
    { 0xa7d63dc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b63dca, SUC_lexicon_tags_15 },
    { 0xa7363dca, SUC_lexicon_tags_60 },
    { 0xd2f63dca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef63dd0, SUC_lexicon_tags_2 },
    { 0x19f63dd1, SUC_lexicon_tags_60 },
    { 0x79363dd2, SUC_lexicon_tags_28 },
    { 0xf7f63dd2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71163dd5, SUC_lexicon_tags_1 },
    { 0x28b63dd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d63ddc, SUC_lexicon_tags_25 },
    { 0x43763ddd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf63de1, SUC_lexicon_tags_14 },
    { 0x7963de2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54963de4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd963de7, SUC_lexicon_tags_63 },
    { 0xd163de8, SUC_lexicon_tags_25 },
    { 0xaef63de8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd63dee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb563df4, SUC_lexicon_tags_3 },
    { 0x50363df4, SUC_lexicon_tags_17 },
    { 0xed763df6, SUC_lexicon_tags_15 },
    { 0xee763df6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e363dfb, SUC_lexicon_tags_26 },
    { 0xdbd63dfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0163e00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x79363e02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c163e08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d363e0a, SUC_lexicon_tags_6 },
    { 0xd1563e0a, SUC_lexicon_tags_1 },
    { 0x3bd63e0c, SUC_lexicon_tags_7 },
    { 0x52963e0d, SUC_lexicon_tags_17 },
    { 0x68f63e0e, SUC_lexicon_tags_23 },
    { 0xdc763e0d, SUC_lexicon_tags_43 },
    { 0x15963e10, SUC_lexicon_tags_14 },
    { 0xf8d63e10, SUC_lexicon_tags_18 },
    { 0x4bf63e12, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x23163e15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b63e1c, SUC_lexicon_tags_7 },
    { 0xb8763e1d, SUC_lexicon_tags_10 },
    { 0x8df63e1e, SUC_lexicon_tags_60 },
    { 0xd0163e1f, SUC_lexicon_tags_7 },
    { 0x8bd63e20, SUC_lexicon_tags_33 },
    { 0x9b763e1f, SUC_lexicon_tags_29 },
    { 0x72763e1f, SUC_lexicon_tags_15 },
    { 0x7db63e23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57163e26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0163e28, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2363e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd63e32, SUC_lexicon_tags_43 },
    { 0x2b963e32, SUC_lexicon_tags_12 },
    { 0x6b563e34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6963e36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2963e41, SUC_lexicon_tags_1 },
    { 0x1cd63e42, SUC_lexicon_tags_83 },
    { 0xf5363e43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8963e45, SUC_lexicon_tags_11 },
    { 0x6f563e46, SUC_lexicon_tags_16 },
    { 0x94b63e47, SUC_lexicon_tags_17 },
    { 0x36d63e46, SUC_lexicon_tags_19 },
    { 0x11363e49, SUC_lexicon_tags_65 },
    { 0xf4b63e4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4763e50, SUC_lexicon_tags_37 },
    { 0x22363e50, SUC_lexicon_tags_9 },
    { 0xf0b63e50, SUC_lexicon_tags_7 },
    { 0x25b63e52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff63e5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22363e63, SUC_lexicon_tags_0 },
    { 0xdbf63e64, SUC_lexicon_tags_2 },
    { 0x21363e64, SUC_lexicon_tags_17 },
    { 0x7ab63e64, SUC_lexicon_tags_56 },
    { 0xa5363e65, SUC_lexicon_tags_26 },
    { 0x39963e65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed63e6f, SUC_lexicon_tags_25 },
    { 0x95b63e70, SUC_lexicon_tags_27 },
    { 0xc8163e71, SUC_lexicon_tags_2 },
    { 0x3c763e71, SUC_lexicon_tags_45 },
    { 0xde363e73, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed63e77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d63e7a, SUC_lexicon_tags_37 },
    { 0x81163e7b, SUC_lexicon_tags_1 },
    { 0x53b63e7b, SUC_lexicon_tags_38 },
    { 0x1d363e7c, SUC_lexicon_tags_8 },
    { 0xe0f63e7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93d63e80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a563e82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9363e84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15163e86, SUC_lexicon_tags_17 },
    { 0xad63e87, SUC_lexicon_tags_26 },
    { 0x8b163e86, SUC_lexicon_tags_25 },
    { 0x4d963e89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53763e8b, SUC_lexicon_tags_23 },
    { 0xb2163e8c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b363e8e, SUC_lexicon_tags_12 },
    { 0xbc963e8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab163e95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6963e97, SUC_lexicon_tags_12 },
    { 0x5b363e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ad63e9a, SUC_lexicon_tags_37 },
    { 0xa4163e9b, SUC_lexicon_tags_8 },
    { 0x4f363e9c, SUC_lexicon_tags_0 },
    { 0x2af63e9a, SUC_lexicon_tags_37 },
    { 0x17963e9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b63ea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3163ea9, SUC_lexicon_tags_25 },
    { 0x4b563eaa, SUC_lexicon_tags_26 },
    { 0x3c563eab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23763ead, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24363eb1, SUC_lexicon_tags_11 },
    { 0xfdd63eb2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf9b63eb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc963eb9, SUC_lexicon_tags_2 },
    { 0x61363eb9, SUC_lexicon_tags_26 },
    { 0xa5163ebb, SUC_lexicon_tags_7 },
    { 0x40163ebc, SUC_lexicon_tags_60 },
    { 0x86d63ebc, SUC_lexicon_tags_14 },
    { 0x9cf63ebd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5963ec1, SUC_lexicon_tags_37 },
    { 0x1b763ec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36763ec4, SUC_lexicon_tags_23 },
    { 0x1a563ec5, SUC_lexicon_tags_22 },
    { 0xa0563ec5, SUC_lexicon_tags_7 },
    { 0xc9d63ec7, SUC_lexicon_tags_28 },
    { 0x8b163ec5, SUC_lexicon_tags_7 },
    { 0xcfd63ec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb163ed4, SUC_lexicon_tags_49 },
    { 0x21763ed5, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xac363ed7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2c163ed9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d63edc, SUC_lexicon_tags_17 },
    { 0xb9563edd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f63ee0, SUC_lexicon_tags_23 },
    { 0x12b63ee1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab63ee5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd63ee7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f63eec, SUC_lexicon_tags_10 },
    { 0x9a363eed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65163ef1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87963ef6, SUC_lexicon_tags_12 },
    { 0x9f963ef7, SUC_lexicon_tags_57 },
    { 0xdcb63ef8, SUC_lexicon_tags_62 },
    { 0xc1963ef6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b563efb, SUC_lexicon_tags_734 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b63f02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f63f05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f763f07, SUC_lexicon_tags_7 },
    { 0x1cd63f08, SUC_lexicon_tags_17 },
    { 0x65363f08, SUC_lexicon_tags_0 },
    { 0xacb63f08, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x27f63f0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f763f11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1363f15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97b63f17, SUC_lexicon_tags_2 },
    { 0x88d63f18, SUC_lexicon_tags_38 },
    { 0xb763f19, SUC_lexicon_tags_16 },
    { 0x63b63f1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf63f1f, SUC_lexicon_tags_15 },
    { 0x3f163f20, SUC_lexicon_tags_31 },
    { 0xf2363f20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b63f24, SUC_lexicon_tags_63 },
    { 0xd6963f24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc963f28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8363f31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99363f36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc563f39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf763f3c, SUC_lexicon_tags_7 },
    { 0x8963f3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b563f40, SUC_lexicon_tags_15 },
    { 0xa763f41, SUC_lexicon_tags_37 },
    { 0xa9163f42, SUC_lexicon_tags_11 },
    { 0x45f63f41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cf63f45, SUC_lexicon_tags_118 },
    { 0xad963f46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x19363f49, SUC_lexicon_tags_37 },
    { 0x53d63f4a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d563f4f, SUC_lexicon_tags_17 },
    { 0x30b63f4f, SUC_lexicon_tags_1 },
    { 0x52563f51, SUC_lexicon_tags_65 },
    { 0xb8763f51, SUC_lexicon_tags_49 },
    { 0x4f563f53, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b63f57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51b63f59, SUC_lexicon_tags_48 },
    { 0x81f63f5a, SUC_lexicon_tags_19 },
    { 0xe5563f5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb63f64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b63f69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x34d63f6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d63f6e, SUC_lexicon_tags_17 },
    { 0xd0363f6f, SUC_lexicon_tags_39 },
    { 0xc8363f70, SUC_lexicon_tags_26 },
    { 0xacd63f71, SUC_lexicon_tags_21 },
    { 0x26d63f72, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f63f78, SUC_lexicon_tags_42 },
    { 0xe2963f79, SUC_lexicon_tags_38 },
    { 0xa2b63f78, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d63f7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a163f84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd963f89, SUC_lexicon_tags_44 },
    { 0xc8163f8a, SUC_lexicon_tags_87 },
    { 0xeff63f8b, SUC_lexicon_tags_1 },
    { 0xd6963f8c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff163f92, SUC_lexicon_tags_29 },
    { 0xb3363f93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef963f97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52963f9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d63f9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3763fa2, SUC_lexicon_tags_0 },
    { 0xf5b63fa3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3163faa, SUC_lexicon_tags_12 },
    { 0xfbf63fab, SUC_lexicon_tags_34 },
    { 0x5b363fab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xca963fae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf63fb0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4c963fb2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5b63fb4, SUC_lexicon_tags_63 },
    { 0x94363fb4, SUC_lexicon_tags_5 },
    { 0xb9f63fb5, SUC_lexicon_tags_6 },
    { 0xb2363fb7, SUC_lexicon_tags_16 },
    { 0xac763fb8, SUC_lexicon_tags_25 },
    { 0xff763fb8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6f763fbb, SUC_lexicon_tags_46 },
    { 0xff163fbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeff63fbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9963fc1, SUC_lexicon_tags_19 },
    { 0x3f63fc2, SUC_lexicon_tags_27 },
    { 0x10d63fc3, SUC_lexicon_tags_25 },
    { 0x73563fc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16363fc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65163fc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fb63fca, SUC_lexicon_tags_13 },
    { 0x77363fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7763fce, SUC_lexicon_tags_63 },
    { 0x87163fcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cf63fd1, SUC_lexicon_tags_20 },
    { 0x96b63fd2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3cf63fd4, SUC_lexicon_tags_60 },
    { 0x96963fd5, SUC_lexicon_tags_14 },
    { 0xedf63fd6, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8563fd9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6363fde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72763fe3, SUC_lexicon_tags_109 },
    { 0x2e363fe3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86563fe7, SUC_lexicon_tags_1 },
    { 0xd563fe8, SUC_lexicon_tags_21 },
    { 0x30d63fe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e963feb, SUC_lexicon_tags_37 },
    { 0x5c963fec, SUC_lexicon_tags_31 },
    { 0xf4f63fed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f63ff2, SUC_lexicon_tags_19 },
    { 0x5a763ff3, SUC_lexicon_tags_7 },
    { 0x42563ff4, SUC_lexicon_tags_32 },
    { 0x53763ff4, SUC_lexicon_tags_26 },
    { 0xab63ff6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7563ffa, SUC_lexicon_tags_37 },
    { 0x9c963ffa, SUC_lexicon_tags_26 },
    { 0x99b63ffa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4164001, SUC_lexicon_tags_12 },
    { 0x4d364001, SUC_lexicon_tags_47 },
    { 0x75b64003, SUC_lexicon_tags_11 },
    { 0xed64003, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x4f64006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38964009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf6400f, SUC_lexicon_tags_47 },
    { 0x5c164010, SUC_lexicon_tags_37 },
    { 0x70d64011, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d64015, SUC_lexicon_tags_38 },
    { 0x8e564015, SUC_lexicon_tags_2 },
    { 0xf0d64017, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56401b, SUC_lexicon_tags_1 },
    { 0x46d6401c, SUC_lexicon_tags_0 },
    { 0x1c16401d, SUC_lexicon_tags_30 },
    { 0xbc16401e, SUC_lexicon_tags_65 },
    { 0xed16401f, SUC_lexicon_tags_15 },
    { 0x6c76401e, SUC_lexicon_tags_33 },
    { 0x95d6401c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x19b64023, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4364025, SUC_lexicon_tags_25 },
    { 0xb8964026, SUC_lexicon_tags_37 },
    { 0xb764026, SUC_lexicon_tags_12 },
    { 0x87f64027, SUC_lexicon_tags_37 },
    { 0x48564027, SUC_lexicon_tags_27 },
    { 0x25564027, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ed6402c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x996402f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0564031, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb764035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb6403a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x27f6403c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3af6403f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x69764041, SUC_lexicon_tags_26 },
    { 0xb5364042, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0964044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8364046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3364048, SUC_lexicon_tags_31 },
    { 0x37d64048, SUC_lexicon_tags_30 },
    { 0x34d6404a, SUC_lexicon_tags_15 },
    { 0x9d564048, SUC_lexicon_tags_57 },
    { 0xbf76404c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19764053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83764057, SUC_lexicon_tags_63 },
    { 0x75b64058, SUC_lexicon_tags_1 },
    { 0x1b364059, SUC_lexicon_tags_46 },
    { 0xe6764058, SUC_lexicon_tags_37 },
    { 0x5ff6405b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8964062, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28564064, SUC_lexicon_tags_14 },
    { 0x7bb64065, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf176406c, SUC_lexicon_tags_25 },
    { 0xf5f6406d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7564077, SUC_lexicon_tags_26 },
    { 0x2b564078, SUC_lexicon_tags_11 },
    { 0x4f564079, SUC_lexicon_tags_29 },
    { 0x3fb64078, SUC_lexicon_tags_26 },
    { 0x2196407b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66d6407d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed64082, SUC_lexicon_tags_25 },
    { 0xea164083, SUC_lexicon_tags_37 },
    { 0x1d64084, SUC_lexicon_tags_13 },
    { 0x90b64085, SUC_lexicon_tags_27 },
    { 0x7c564085, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef364088, SUC_lexicon_tags_17 },
    { 0xa0f64089, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa076408c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa564092, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a364099, SUC_lexicon_tags_34 },
    { 0xfb96409a, SUC_lexicon_tags_11 },
    { 0x1176409b, SUC_lexicon_tags_37 },
    { 0x3b6409c, SUC_lexicon_tags_37 },
    { 0x8a36409d, SUC_lexicon_tags_37 },
    { 0x17d6409e, SUC_lexicon_tags_151 },
    { 0xcd6409b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x157640a2, SUC_lexicon_tags_37 },
    { 0xf5f640a3, SUC_lexicon_tags_23 },
    { 0xa5f640a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51640ab, SUC_lexicon_tags_66 },
    { 0xfd9640ac, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd640b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x271640b6, SUC_lexicon_tags_46 },
    { 0xf0d640b7, SUC_lexicon_tags_17 },
    { 0xed7640b8, SUC_lexicon_tags_14 },
    { 0x983640b9, SUC_lexicon_tags_26 },
    { 0xdb1640b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb640bf, SUC_lexicon_tags_2 },
    { 0x8b9640c0, SUC_lexicon_tags_25 },
    { 0x741640c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e5640c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d1640c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63b640c7, SUC_lexicon_tags_15 },
    { 0x5a5640c8, SUC_lexicon_tags_23 },
    { 0x26b640c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd640cd, SUC_lexicon_tags_15 },
    { 0x8d3640ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf5640d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03640d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215640db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd2f640dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1640e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x641640e3, SUC_lexicon_tags_25 },
    { 0xd19640e4, SUC_lexicon_tags_49 },
    { 0xe7d640e5, SUC_lexicon_tags_23 },
    { 0x39640e5, SUC_lexicon_tags_27 },
    { 0xd49640e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef640f2, SUC_lexicon_tags_31 },
    { 0x13d640f2, SUC_lexicon_tags_4 },
    { 0xdd9640f4, SUC_lexicon_tags_17 },
    { 0x4a5640f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa27640f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd640fa, SUC_lexicon_tags_16 },
    { 0xf6b640fb, SUC_lexicon_tags_27 },
    { 0x399640fc, SUC_lexicon_tags_23 },
    { 0xc3640fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x68564100, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf64103, SUC_lexicon_tags_28 },
    { 0x3fd64104, SUC_lexicon_tags_12 },
    { 0xb2d64103, SUC_lexicon_tags_17 },
    { 0xdef64106, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9356410c, SUC_lexicon_tags_5 },
    { 0x3b6410c, SUC_lexicon_tags_79 },
    { 0x9456410c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x17964111, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3564114, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x12964116, SUC_lexicon_tags_16 },
    { 0xf364117, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76411a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cb6411c, SUC_lexicon_tags_25 },
    { 0x70f6411d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x164125, SUC_lexicon_tags_27 },
    { 0x9ed64126, SUC_lexicon_tags_47 },
    { 0x5cb64126, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f96412a, SUC_lexicon_tags_21 },
    { 0xf216412b, SUC_lexicon_tags_60 },
    { 0x12b6412a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b36412e, SUC_lexicon_tags_31 },
    { 0x9976412f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ab64131, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd64134, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d64138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d56413d, SUC_lexicon_tags_47 },
    { 0x31f6413e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9df64140, SUC_lexicon_tags_11 },
    { 0xf5b64141, SUC_lexicon_tags_32 },
    { 0xdb364142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d64145, SUC_lexicon_tags_25 },
    { 0x30164146, SUC_lexicon_tags_33 },
    { 0xf4b64146, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f6414b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa416414d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81364151, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f64155, SUC_lexicon_tags_70 },
    { 0x6764155, SUC_lexicon_tags_15 },
    { 0x84364157, SUC_lexicon_tags_17 },
    { 0x13164157, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4f6415a, SUC_lexicon_tags_25 },
    { 0x2916415b, SUC_lexicon_tags_390 },
    { 0x1df6415c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2f6415e, SUC_lexicon_tags_37 },
    { 0xb716415f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b64167, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5156416d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f64170, SUC_lexicon_tags_38 },
    { 0x82b64171, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d64174, SUC_lexicon_tags_38 },
    { 0x9f564174, SUC_lexicon_tags_37 },
    { 0x3f964175, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0964178, SUC_lexicon_tags_7 },
    { 0x66364178, SUC_lexicon_tags_42 },
    { 0x1696417a, SUC_lexicon_tags_37 },
    { 0x7076417b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd6418a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef64192, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25364196, SUC_lexicon_tags_103 },
    { 0x71564197, SUC_lexicon_tags_34 },
    { 0xfe764198, SUC_lexicon_tags_49 },
    { 0x53964199, SUC_lexicon_tags_68 },
    { 0x2736419a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x996419c, SUC_lexicon_tags_6 },
    { 0xf956419d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a96419f, SUC_lexicon_tags_30 },
    { 0x11b6419f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83641a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf15641a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5641a8, SUC_lexicon_tags_17 },
    { 0x699641a8, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59641ac, SUC_lexicon_tags_33 },
    { 0xced641ad, SUC_lexicon_tags_77 },
    { 0x89641ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f641b4, SUC_lexicon_tags_1 },
    { 0x929641b4, SUC_lexicon_tags_2 },
    { 0x9cd641b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x33b641b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9641bc, SUC_lexicon_tags_2 },
    { 0xc09641bc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497641c2, SUC_lexicon_tags_15 },
    { 0x9b3641c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607641cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d641d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf641d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25d641d8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9641db, SUC_lexicon_tags_23 },
    { 0xa51641dc, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19641f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4b641f5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x949641f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267641fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed9641fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b64201, SUC_lexicon_tags_25 },
    { 0x32964202, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x82764204, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6164208, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3964210, SUC_lexicon_tags_47 },
    { 0x1b764211, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d64215, SUC_lexicon_tags_37 },
    { 0xccf64215, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f76421a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69764220, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd9164222, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2964227, SUC_lexicon_tags_2 },
    { 0xa1764228, SUC_lexicon_tags_17 },
    { 0xcb164228, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf6422f, SUC_lexicon_tags_19 },
    { 0x88564230, SUC_lexicon_tags_0 },
    { 0xd2b6422f, SUC_lexicon_tags_29 },
    { 0x35364232, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d64237, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99964239, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7996423c, SUC_lexicon_tags_15 },
    { 0xa536423d, SUC_lexicon_tags_2 },
    { 0x8476423e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94164240, SUC_lexicon_tags_123 },
    { 0x81b64241, SUC_lexicon_tags_7 },
    { 0x98964242, SUC_lexicon_tags_66 },
    { 0xad164242, SUC_lexicon_tags_47 },
    { 0x68b64242, SUC_lexicon_tags_1400 },
    { 0x75b64245, SUC_lexicon_tags_12 },
    { 0x764241, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f6424e, SUC_lexicon_tags_38 },
    { 0xbb96424f, SUC_lexicon_tags_14 },
    { 0xba56424e, SUC_lexicon_tags_63 },
    { 0x9b764251, SUC_lexicon_tags_37 },
    { 0x83b64252, SUC_lexicon_tags_79 },
    { 0xa4764252, SUC_lexicon_tags_15 },
    { 0xb696424e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d64257, SUC_lexicon_tags_28 },
    { 0xaed64257, SUC_lexicon_tags_25 },
    { 0x85f64259, SUC_lexicon_tags_26 },
    { 0x8b96425a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d6425e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7bb64260, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea564264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95964266, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d364268, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2096426a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe36426c, SUC_lexicon_tags_15 },
    { 0x3176426d, SUC_lexicon_tags_2 },
    { 0x7836426e, SUC_lexicon_tags_16 },
    { 0xb316426e, SUC_lexicon_tags_15 },
    { 0x69f6426e, SUC_lexicon_tags_12 },
    { 0x2164271, SUC_lexicon_tags_38 },
    { 0x8236426f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3df64274, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5d64276, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d6427b, SUC_lexicon_tags_10 },
    { 0x80f6427c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f64283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d364289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed6428c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e16428f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3d64291, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x21364293, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92764297, SUC_lexicon_tags_29 },
    { 0xf8564298, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd576429c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895642a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf57642a2, SUC_lexicon_tags_234 },
    { 0xdf5642a3, SUC_lexicon_tags_65 },
    { 0x9cf642a3, SUC_lexicon_tags_27 },
    { 0xd1642a3, SUC_lexicon_tags_25 },
    { 0xa2d642a6, SUC_lexicon_tags_9 },
    { 0x63b642a7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x803642aa, SUC_lexicon_tags_11 },
    { 0xc01642ab, SUC_lexicon_tags_0 },
    { 0xf2f642ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0d642ae, SUC_lexicon_tags_63 },
    { 0x111642ae, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x7a9642b1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51642b5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc05642b7, SUC_lexicon_tags_14 },
    { 0x6ff642b7, SUC_lexicon_tags_25 },
    { 0x71d642b8, SUC_lexicon_tags_11 },
    { 0xf3642b9, SUC_lexicon_tags_65 },
    { 0x27642b9, SUC_lexicon_tags_7 },
    { 0xe67642bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d642bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5642c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8f642c4, SUC_lexicon_tags_22 },
    { 0xb11642c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b642ca, SUC_lexicon_tags_2 },
    { 0xbe5642cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeff642cd, SUC_lexicon_tags_46 },
    { 0x6eb642ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1642d1, SUC_lexicon_tags_35 },
    { 0x8e1642d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1642d5, SUC_lexicon_tags_2 },
    { 0xd6b642d6, SUC_lexicon_tags_14 },
    { 0xfc9642d7, SUC_lexicon_tags_15 },
    { 0x4c1642d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdc9642da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f642e6, SUC_lexicon_tags_9 },
    { 0xf59642e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed642ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b642f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1642f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf55642fa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c164301, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8564305, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696430d, SUC_lexicon_tags_0 },
    { 0x1eb6430d, SUC_lexicon_tags_60 },
    { 0xbab6430e, SUC_lexicon_tags_11 },
    { 0xcb964310, SUC_lexicon_tags_7 },
    { 0xe4564310, SUC_lexicon_tags_12 },
    { 0xf4764311, SUC_lexicon_tags_11 },
    { 0x55564313, SUC_lexicon_tags_21 },
    { 0xd1364313, SUC_lexicon_tags_14 },
    { 0xd7564315, SUC_lexicon_tags_14 },
    { 0x55364316, SUC_lexicon_tags_12 },
    { 0x22d64317, SUC_lexicon_tags_12 },
    { 0x2f764316, SUC_lexicon_tags_37 },
    { 0xdc36430d, SUC_lexicon_tags_17 },
    { 0xb64312, SUC_lexicon_tags_7 },
    { 0xdf6431b, SUC_lexicon_tags_25 },
    { 0xddb6431c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f64321, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd64324, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc764326, SUC_lexicon_tags_15 },
    { 0x7dd64326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f564329, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d6432e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d64331, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd3d64333, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd64336, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9d64338, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6433f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac564341, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x41564343, SUC_lexicon_tags_17 },
    { 0x9ff64343, SUC_lexicon_tags_12 },
    { 0x8f164345, SUC_lexicon_tags_10 },
    { 0xfaf64345, SUC_lexicon_tags_2 },
    { 0x17564345, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc756434b, SUC_lexicon_tags_14 },
    { 0x80b6434c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb36434f, SUC_lexicon_tags_19 },
    { 0x8bd64350, SUC_lexicon_tags_29 },
    { 0x7f64350, SUC_lexicon_tags_11 },
    { 0xb7564352, SUC_lexicon_tags_0 },
    { 0xef164353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23164356, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x33364358, SUC_lexicon_tags_7 },
    { 0xf3364358, SUC_lexicon_tags_2 },
    { 0xa5164359, SUC_lexicon_tags_0 },
    { 0xc4f64358, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7436435e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3164360, SUC_lexicon_tags_37 },
    { 0xea964361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f64364, SUC_lexicon_tags_14 },
    { 0x1364364, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6876436c, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f64373, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x12164375, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d64379, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe056437c, SUC_lexicon_tags_84 },
    { 0xf736437d, SUC_lexicon_tags_38 },
    { 0x4df6437e, SUC_lexicon_tags_9 },
    { 0xbcf6437e, SUC_lexicon_tags_66 },
    { 0xa7b6437e, SUC_lexicon_tags_17 },
    { 0x22764381, SUC_lexicon_tags_16 },
    { 0xdcd64382, SUC_lexicon_tags_37 },
    { 0xa896437e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39b64385, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196438c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3f6438e, SUC_lexicon_tags_37 },
    { 0x4476438f, SUC_lexicon_tags_68 },
    { 0x99b6438f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x80964393, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf64397, SUC_lexicon_tags_15 },
    { 0x67364398, SUC_lexicon_tags_1 },
    { 0x30d64398, SUC_lexicon_tags_17 },
    { 0x15764397, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xbb96439c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb643a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d7643a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd643a8, SUC_lexicon_tags_37 },
    { 0x26b643a9, SUC_lexicon_tags_25 },
    { 0x735643aa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3643ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f643af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x63f643b1, SUC_lexicon_tags_37 },
    { 0x6d5643b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7b643b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d643b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9643bd, SUC_lexicon_tags_7 },
    { 0x433643bd, SUC_lexicon_tags_0 },
    { 0xc87643bd, SUC_lexicon_tags_9 },
    { 0xa7643c0, SUC_lexicon_tags_11 },
    { 0x4b643c1, SUC_lexicon_tags_2 },
    { 0xfd643c0, SUC_lexicon_tags_27 },
    { 0xe55643c0, SUC_lexicon_tags_16 },
    { 0xa19643c3, SUC_lexicon_tags_11 },
    { 0x30f643c5, SUC_lexicon_tags_37 },
    { 0xbf3643c6, SUC_lexicon_tags_37 },
    { 0xd49643c6, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751643cf, SUC_lexicon_tags_37 },
    { 0xd4f643d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19643d3, SUC_lexicon_tags_7 },
    { 0xd643d4, SUC_lexicon_tags_8 },
    { 0xdb7643d5, SUC_lexicon_tags_82 },
    { 0x10f643d3, SUC_lexicon_tags_11 },
    { 0x883643d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x76f643d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x281643db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3643e3, SUC_lexicon_tags_11 },
    { 0xf15643e4, SUC_lexicon_tags_11 },
    { 0xfb643e4, SUC_lexicon_tags_59 },
    { 0xa91643e4, SUC_lexicon_tags_39 },
    { 0x751643e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff643ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6eb643ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x523643f1, SUC_lexicon_tags_77 },
    { 0xc6b643f2, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0xb69643f4, SUC_lexicon_tags_11 },
    { 0x539643f4, SUC_lexicon_tags_43 },
    { 0xeff643f5, SUC_lexicon_tags_15 },
    { 0x25b643f7, SUC_lexicon_tags_37 },
    { 0x521643f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d643fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3764400, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae364406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88964408, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5416440a, SUC_lexicon_tags_33 },
    { 0x6f76440b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17364411, SUC_lexicon_tags_17 },
    { 0x5cb64412, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a764414, SUC_lexicon_tags_7 },
    { 0xb6b64415, SUC_lexicon_tags_23 },
    { 0x3fd64416, SUC_lexicon_tags_11 },
    { 0x72564417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ad64419, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d6441d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df64421, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f64425, SUC_lexicon_tags_2 },
    { 0xe1364426, SUC_lexicon_tags_15 },
    { 0x2f164426, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b6442a, SUC_lexicon_tags_15 },
    { 0xa416442b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b76442f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf564436, SUC_lexicon_tags_9 },
    { 0x55f64437, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f6443b, SUC_lexicon_tags_1 },
    { 0x3ab6443c, SUC_lexicon_tags_17 },
    { 0x3676443c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a164444, SUC_lexicon_tags_8 },
    { 0x63d64445, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd5564447, SUC_lexicon_tags_136 },
    { 0x9d764448, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x396444b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f964454, SUC_lexicon_tags_9 },
    { 0xceb64455, SUC_lexicon_tags_35 },
    { 0x3c964455, SUC_lexicon_tags_148 },
    { 0x2b964457, SUC_lexicon_tags_36 },
    { 0x7d564456, SUC_lexicon_tags_15 },
    { 0x11d64455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29b6445b, SUC_lexicon_tags_45 },
    { 0xde56445c, SUC_lexicon_tags_5 },
    { 0xc2b6445d, SUC_lexicon_tags_53 },
    { 0x1436445e, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3164461, SUC_lexicon_tags_27 },
    { 0xdcf64462, SUC_lexicon_tags_11 },
    { 0x90b64462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71b64465, SUC_lexicon_tags_33 },
    { 0x75d64465, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x95d64468, SUC_lexicon_tags_8 },
    { 0xdb64469, SUC_lexicon_tags_12 },
    { 0x4f76446a, SUC_lexicon_tags_1 },
    { 0x6736446b, SUC_lexicon_tags_12 },
    { 0xc0f6446c, SUC_lexicon_tags_6 },
    { 0x48d6446a, SUC_lexicon_tags_23 },
    { 0x9e964468, SUC_lexicon_tags_55 },
    { 0x1676446c, SUC_lexicon_tags_13 },
    { 0x20564470, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1564472, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3164478, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff6447e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb64485, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2764488, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1416448a, SUC_lexicon_tags_37 },
    { 0x27f6448b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8e96448d, SUC_lexicon_tags_7 },
    { 0xe096448d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f64491, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26764499, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45644a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x771644a3, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9644ab, SUC_lexicon_tags_15 },
    { 0xf3d644ac, SUC_lexicon_tags_2 },
    { 0xc27644ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb644b1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe49644b3, SUC_lexicon_tags_14 },
    { 0xfdb644b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaef644b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b644bc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723644c0, SUC_lexicon_tags_12 },
    { 0x9cd644c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb644c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x151644c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x205644ca, SUC_lexicon_tags_31 },
    { 0xd37644cb, SUC_lexicon_tags_25 },
    { 0x11f644cc, SUC_lexicon_tags_28 },
    { 0x669644cd, SUC_lexicon_tags_11 },
    { 0x1b3644ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9644d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd644db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9644de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e1644e0, SUC_lexicon_tags_23 },
    { 0xa41644e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed644e4, SUC_lexicon_tags_11 },
    { 0x58b644e5, SUC_lexicon_tags_1 },
    { 0x251644e6, SUC_lexicon_tags_38 },
    { 0x529644e7, SUC_lexicon_tags_60 },
    { 0x6f5644e8, SUC_lexicon_tags_55 },
    { 0xb4d644e9, SUC_lexicon_tags_32 },
    { 0xc33644e9, SUC_lexicon_tags_12 },
    { 0x2c7644e8, SUC_lexicon_tags_6 },
    { 0x339644ec, SUC_lexicon_tags_17 },
    { 0xfb9644ec, SUC_lexicon_tags_1 },
    { 0xf5644e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75b644f0, SUC_lexicon_tags_37 },
    { 0x20f644f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369644fa, SUC_lexicon_tags_17 },
    { 0xd59644fa, SUC_lexicon_tags_1 },
    { 0x4d1644fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5644ff, SUC_lexicon_tags_33 },
    { 0x55b64500, SUC_lexicon_tags_19 },
    { 0x749644ff, SUC_lexicon_tags_11 },
    { 0xe9564502, SUC_lexicon_tags_60 },
    { 0xb8564502, SUC_lexicon_tags_108 },
    { 0xc6964504, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70764509, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xca76450c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbbf6450e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1564511, SUC_lexicon_tags_29 },
    { 0x41f64511, SUC_lexicon_tags_12 },
    { 0xd3f64513, SUC_lexicon_tags_23 },
    { 0xd0564513, SUC_lexicon_tags_25 },
    { 0x50f64515, SUC_lexicon_tags_0 },
    { 0xdcd64516, SUC_lexicon_tags_17 },
    { 0xf7d64517, SUC_lexicon_tags_15 },
    { 0x73f64517, SUC_lexicon_tags_37 },
    { 0x2e164515, SUC_lexicon_tags_37 },
    { 0x1ab6451a, SUC_lexicon_tags_87 },
    { 0x5db6451b, SUC_lexicon_tags_30 },
    { 0x4636451a, SUC_lexicon_tags_32 },
    { 0xc2d6451d, SUC_lexicon_tags_2 },
    { 0x2c36451a, SUC_lexicon_tags_7 },
    { 0xc496451f, SUC_lexicon_tags_33 },
    { 0x8c164511, SUC_lexicon_tags_8 },
    { 0xdf764515, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74964526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf6452a, SUC_lexicon_tags_45 },
    { 0x3976452b, SUC_lexicon_tags_69 },
    { 0x46b6452c, SUC_lexicon_tags_69 },
    { 0x2636452b, SUC_lexicon_tags_25 },
    { 0x2f76452b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x69d64530, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b64538, SUC_lexicon_tags_463 },
    { 0, NULL },
    { 0x1a56453a, SUC_lexicon_tags_30 },
    { 0x6036453a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a36453e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf764541, SUC_lexicon_tags_11 },
    { 0xf4b64542, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b64545, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5276454b, SUC_lexicon_tags_2 },
    { 0x74d6454c, SUC_lexicon_tags_27 },
    { 0x8256454d, SUC_lexicon_tags_46 },
    { 0x4456454e, SUC_lexicon_tags_23 },
    { 0xf8f6454f, SUC_lexicon_tags_4 },
    { 0xc4f6454b, SUC_lexicon_tags_15 },
    { 0x2876454e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b64559, SUC_lexicon_tags_25 },
    { 0x4cf6455a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa056455e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88f64560, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf964565, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8164568, SUC_lexicon_tags_15 },
    { 0x3c364569, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8616456b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbd16456d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd56456f, SUC_lexicon_tags_37 },
    { 0x60b64570, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b64575, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d6457b, SUC_lexicon_tags_17 },
    { 0x61b6457c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf64580, SUC_lexicon_tags_65 },
    { 0x23d64580, SUC_lexicon_tags_7 },
    { 0xed564581, SUC_lexicon_tags_11 },
    { 0x27564582, SUC_lexicon_tags_29 },
    { 0xc964584, SUC_lexicon_tags_12 },
    { 0x8d964585, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39564588, SUC_lexicon_tags_23 },
    { 0x67564589, SUC_lexicon_tags_17 },
    { 0x51564589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb64590, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab364594, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc364598, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b56459d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa836459f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfe7645a1, SUC_lexicon_tags_7 },
    { 0x6a5645a1, SUC_lexicon_tags_11 },
    { 0x93645a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x209645a5, SUC_lexicon_tags_33 },
    { 0x8ff645a5, SUC_lexicon_tags_27 },
    { 0xb2f645a6, SUC_lexicon_tags_7 },
    { 0x79645a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d645b2, SUC_lexicon_tags_46 },
    { 0xb1d645b3, SUC_lexicon_tags_26 },
    { 0xecb645b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b645ba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b645bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x487645c0, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0x323645c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53645c7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b645cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad645d4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119645d8, SUC_lexicon_tags_37 },
    { 0xb79645d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfe7645db, SUC_lexicon_tags_25 },
    { 0xfa7645dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f645e0, SUC_lexicon_tags_49 },
    { 0x21d645e1, SUC_lexicon_tags_66 },
    { 0x6e9645e2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa89645e4, SUC_lexicon_tags_25 },
    { 0x261645e5, SUC_lexicon_tags_43 },
    { 0xaef645e5, SUC_lexicon_tags_25 },
    { 0xe7645e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x615645ea, SUC_lexicon_tags_7 },
    { 0xa1d645ea, SUC_lexicon_tags_33 },
    { 0x519645ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61645ef, SUC_lexicon_tags_56 },
    { 0xb1b645f0, SUC_lexicon_tags_15 },
    { 0x32d645f0, SUC_lexicon_tags_48 },
    { 0x56b645ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331645f6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd2b645f8, SUC_lexicon_tags_25 },
    { 0xb55645f8, SUC_lexicon_tags_1 },
    { 0x8eb645f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fb645fc, SUC_lexicon_tags_37 },
    { 0xbf9645fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7764602, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c964606, SUC_lexicon_tags_25 },
    { 0x21564606, SUC_lexicon_tags_39 },
    { 0x55964606, SUC_lexicon_tags_2 },
    { 0x3d564609, SUC_lexicon_tags_60 },
    { 0xf7d64606, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b6460d, SUC_lexicon_tags_39 },
    { 0x38d6460d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4964617, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d6461a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5716461e, SUC_lexicon_tags_33 },
    { 0x4bd6461f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8b64621, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd64624, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c764628, SUC_lexicon_tags_1 },
    { 0xb7f64629, SUC_lexicon_tags_37 },
    { 0x85164629, SUC_lexicon_tags_71 },
    { 0x2996462b, SUC_lexicon_tags_1 },
    { 0x1256462b, SUC_lexicon_tags_2 },
    { 0x3b56462a, SUC_lexicon_tags_25 },
    { 0xf616462b, SUC_lexicon_tags_92 },
    { 0x60d6462e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f64633, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3c964635, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab64640, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb296464c, SUC_lexicon_tags_0 },
    { 0xe796464c, SUC_lexicon_tags_11 },
    { 0x86b6464e, SUC_lexicon_tags_38 },
    { 0xf4b6464f, SUC_lexicon_tags_47 },
    { 0x1b96464f, SUC_lexicon_tags_47 },
    { 0xb4b64651, SUC_lexicon_tags_26 },
    { 0x21964652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f64655, SUC_lexicon_tags_0 },
    { 0xcaf64656, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b64659, SUC_lexicon_tags_15 },
    { 0x47f6465a, SUC_lexicon_tags_0 },
    { 0x71f6465a, SUC_lexicon_tags_61 },
    { 0x416465c, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d64660, SUC_lexicon_tags_27 },
    { 0x5d164661, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6964663, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x38364665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e564667, SUC_lexicon_tags_15 },
    { 0x27b64668, SUC_lexicon_tags_23 },
    { 0x2c364667, SUC_lexicon_tags_7 },
    { 0x5b36466a, SUC_lexicon_tags_7 },
    { 0xfb56466b, SUC_lexicon_tags_2 },
    { 0xf2364667, SUC_lexicon_tags_17 },
    { 0x1076466b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f64670, SUC_lexicon_tags_16 },
    { 0xa3964671, SUC_lexicon_tags_68 },
    { 0xa9f64672, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1564675, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f64678, SUC_lexicon_tags_11 },
    { 0x2c764679, SUC_lexicon_tags_11 },
    { 0xfc56467a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7936467d, SUC_lexicon_tags_10 },
    { 0x8bf6467d, SUC_lexicon_tags_45 },
    { 0xce36467f, SUC_lexicon_tags_11 },
    { 0x30f64680, SUC_lexicon_tags_17 },
    { 0xc3d6467f, SUC_lexicon_tags_7 },
    { 0xd2b64681, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58d64684, SUC_lexicon_tags_2 },
    { 0xd1564685, SUC_lexicon_tags_11 },
    { 0x7ed64686, SUC_lexicon_tags_7 },
    { 0xc9f64687, SUC_lexicon_tags_1 },
    { 0xe9764688, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb656468e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f64690, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f64693, SUC_lexicon_tags_2 },
    { 0x50f64694, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3564696, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77164699, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3196469c, SUC_lexicon_tags_25 },
    { 0xc6d6469d, SUC_lexicon_tags_15 },
    { 0x46b6469c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51646a2, SUC_lexicon_tags_11 },
    { 0x971646a3, SUC_lexicon_tags_43 },
    { 0x9b9646a4, SUC_lexicon_tags_23 },
    { 0xf6d646a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799646a9, SUC_lexicon_tags_107 },
    { 0x267646aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1646ac, SUC_lexicon_tags_116 },
    { 0xea9646ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5646b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f646b5, SUC_lexicon_tags_14 },
    { 0x819646b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3646bd, SUC_lexicon_tags_21 },
    { 0x339646be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7eb646c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79646c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd646cd, SUC_lexicon_tags_17 },
    { 0x4f7646cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f646d1, SUC_lexicon_tags_15 },
    { 0xca3646d2, SUC_lexicon_tags_1 },
    { 0x6fd646d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc33646d5, SUC_lexicon_tags_1 },
    { 0x2e5646d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfb646d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd646dc, SUC_lexicon_tags_17 },
    { 0xc4b646dd, SUC_lexicon_tags_11 },
    { 0x27646dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd01646e0, SUC_lexicon_tags_13 },
    { 0x983646e0, SUC_lexicon_tags_47 },
    { 0xdf646e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9d646e4, SUC_lexicon_tags_25 },
    { 0x36f646e5, SUC_lexicon_tags_28 },
    { 0xdb3646e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b646ec, SUC_lexicon_tags_92 },
    { 0x129646ed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x637646ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f646f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21d646f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9646fa, SUC_lexicon_tags_77 },
    { 0x155646fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98964701, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x46564703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd164708, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd6470f, SUC_lexicon_tags_87 },
    { 0xe5b6470f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdeb64712, SUC_lexicon_tags_8 },
    { 0x18b64713, SUC_lexicon_tags_1 },
    { 0xa1164714, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdfd64716, SUC_lexicon_tags_47 },
    { 0x98564716, SUC_lexicon_tags_3 },
    { 0xbb764718, SUC_lexicon_tags_43 },
    { 0x4f64717, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9356471b, SUC_lexicon_tags_7 },
    { 0x45d6471c, SUC_lexicon_tags_2 },
    { 0x58b6471d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x71b6471f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35964726, SUC_lexicon_tags_37 },
    { 0x48564727, SUC_lexicon_tags_30 },
    { 0x1d164726, SUC_lexicon_tags_27 },
    { 0xae964729, SUC_lexicon_tags_7 },
    { 0xff764729, SUC_lexicon_tags_56 },
    { 0x7d164729, SUC_lexicon_tags_15 },
    { 0xd9d6472b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda764730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52764735, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd564737, SUC_lexicon_tags_2 },
    { 0xccd64738, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5256473b, SUC_lexicon_tags_63 },
    { 0x7bb6473c, SUC_lexicon_tags_7 },
    { 0xd7b6473d, SUC_lexicon_tags_63 },
    { 0x7af6473e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d64743, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b64746, SUC_lexicon_tags_12 },
    { 0x26764747, SUC_lexicon_tags_1 },
    { 0x64764748, SUC_lexicon_tags_17 },
    { 0x7364749, SUC_lexicon_tags_34 },
    { 0xce964746, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb64750, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d964753, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80f64755, SUC_lexicon_tags_1 },
    { 0xa1b64756, SUC_lexicon_tags_15 },
    { 0x2b764756, SUC_lexicon_tags_37 },
    { 0x80164758, SUC_lexicon_tags_4 },
    { 0x1ad64759, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe96475d, SUC_lexicon_tags_1 },
    { 0x19b6475e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7bd64760, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9ff64762, SUC_lexicon_tags_84 },
    { 0xa4164762, SUC_lexicon_tags_17 },
    { 0x9a364763, SUC_lexicon_tags_14 },
    { 0x34964765, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab64768, SUC_lexicon_tags_48 },
    { 0x25364768, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91564773, SUC_lexicon_tags_17 },
    { 0x22d64774, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1936477a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8364785, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x66f64787, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa1564789, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d64791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d164795, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb64799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdff6479b, SUC_lexicon_tags_49 },
    { 0xb176479b, SUC_lexicon_tags_3 },
    { 0xd396479c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x643647a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3647a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x27f647a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x657647a7, SUC_lexicon_tags_8 },
    { 0xfc1647a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x681647aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b647b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d647b5, SUC_lexicon_tags_2 },
    { 0x8cf647b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x821647b9, SUC_lexicon_tags_38 },
    { 0xe5d647ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff647be, SUC_lexicon_tags_16 },
    { 0xecf647bf, SUC_lexicon_tags_43 },
    { 0xe7b647be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8b1647c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631647c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbd9647c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7647cb, SUC_lexicon_tags_1 },
    { 0xcff647cb, SUC_lexicon_tags_11 },
    { 0xf69647cd, SUC_lexicon_tags_68 },
    { 0xbe5647cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde9647d0, SUC_lexicon_tags_37 },
    { 0x9a7647d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07647d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55647d7, SUC_lexicon_tags_7 },
    { 0x319647d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1647dc, SUC_lexicon_tags_48 },
    { 0xc2b647dd, SUC_lexicon_tags_25 },
    { 0x36d647dd, SUC_lexicon_tags_29 },
    { 0x6f9647de, SUC_lexicon_tags_11 },
    { 0x7db647e0, SUC_lexicon_tags_33 },
    { 0xac1647e1, SUC_lexicon_tags_15 },
    { 0x163647e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x585647e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869647e9, SUC_lexicon_tags_34 },
    { 0x39f647ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd647ed, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x863647ef, SUC_lexicon_tags_25 },
    { 0xd43647f0, SUC_lexicon_tags_27 },
    { 0x5bb647f1, SUC_lexicon_tags_19 },
    { 0x71647f1, SUC_lexicon_tags_15 },
    { 0x22d647f0, SUC_lexicon_tags_32 },
    { 0x707647ef, SUC_lexicon_tags_29 },
    { 0xbd7647f0, SUC_lexicon_tags_60 },
    { 0x3eb647f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf3647f8, SUC_lexicon_tags_85 },
    { 0xbb7647f8, SUC_lexicon_tags_38 },
    { 0x605647fa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xec3647fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d64806, SUC_lexicon_tags_2 },
    { 0x4f964806, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b6480a, SUC_lexicon_tags_11 },
    { 0xce36480b, SUC_lexicon_tags_37 },
    { 0x54b6480c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d64816, SUC_lexicon_tags_37 },
    { 0xb5564817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c164819, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a76481d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9516481f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb64822, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x30764825, SUC_lexicon_tags_12 },
    { 0x15964826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4b64828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6482c, SUC_lexicon_tags_19 },
    { 0xb6482d, SUC_lexicon_tags_15 },
    { 0xf456482e, SUC_lexicon_tags_17 },
    { 0xffd6482d, SUC_lexicon_tags_12 },
    { 0x5256482d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55564833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c764839, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb6483c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a16483e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e964846, SUC_lexicon_tags_248 },
    { 0, NULL },
    { 0x1364848, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eb6484a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896484e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb964851, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0564854, SUC_lexicon_tags_15 },
    { 0x64964855, SUC_lexicon_tags_56 },
    { 0x1d364856, SUC_lexicon_tags_14 },
    { 0xdd364857, SUC_lexicon_tags_46 },
    { 0x10d64856, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea16485a, SUC_lexicon_tags_43 },
    { 0x31b6485b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6eb6485d, SUC_lexicon_tags_45 },
    { 0xa96485d, SUC_lexicon_tags_74 },
    { 0x7736485f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f64862, SUC_lexicon_tags_17 },
    { 0xd2f64862, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d64866, SUC_lexicon_tags_11 },
    { 0x59564866, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98b64869, SUC_lexicon_tags_63 },
    { 0xc576486a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5016486d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff364871, SUC_lexicon_tags_11 },
    { 0x7c164872, SUC_lexicon_tags_25 },
    { 0xb3964873, SUC_lexicon_tags_37 },
    { 0x56164873, SUC_lexicon_tags_12 },
    { 0x73f64875, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35d64877, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27364879, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56487e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb764884, SUC_lexicon_tags_7 },
    { 0xb7764884, SUC_lexicon_tags_6 },
    { 0x49164886, SUC_lexicon_tags_15 },
    { 0xc164886, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d6488c, SUC_lexicon_tags_2 },
    { 0x7b56488d, SUC_lexicon_tags_35 },
    { 0xfe56488e, SUC_lexicon_tags_38 },
    { 0x3836488e, SUC_lexicon_tags_87 },
    { 0x14d6488f, SUC_lexicon_tags_12 },
    { 0x25d64891, SUC_lexicon_tags_42 },
    { 0x1156488e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1964896, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1816489b, SUC_lexicon_tags_38 },
    { 0x3db6489b, SUC_lexicon_tags_47 },
    { 0x8636489c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1648a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45648aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89648ad, SUC_lexicon_tags_47 },
    { 0x38f648ae, SUC_lexicon_tags_16 },
    { 0xffd648af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7648b5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf7f648b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3648ba, SUC_lexicon_tags_23 },
    { 0xe77648ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db648c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65b648c3, SUC_lexicon_tags_39 },
    { 0xa69648c4, SUC_lexicon_tags_45 },
    { 0xf69648c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df648c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6bf648cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f648d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713648d7, SUC_lexicon_tags_15 },
    { 0x7db648d7, SUC_lexicon_tags_2 },
    { 0xc53648d7, SUC_lexicon_tags_18 },
    { 0x23b648d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591648df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x441648e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d648e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1648f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x399648f7, SUC_lexicon_tags_17 },
    { 0x9af648f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28764901, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x7564904, SUC_lexicon_tags_25 },
    { 0x52f64905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47164908, SUC_lexicon_tags_45 },
    { 0xde164909, SUC_lexicon_tags_11 },
    { 0xd2764909, SUC_lexicon_tags_68 },
    { 0x556490a, SUC_lexicon_tags_38 },
    { 0xa2364908, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f6490f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2564912, SUC_lexicon_tags_11 },
    { 0xd9764913, SUC_lexicon_tags_45 },
    { 0xe0d64914, SUC_lexicon_tags_46 },
    { 0xa9f64914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf64918, SUC_lexicon_tags_2 },
    { 0x89564919, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f64923, SUC_lexicon_tags_7 },
    { 0xe1d64924, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd64927, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b96492a, SUC_lexicon_tags_11 },
    { 0xc196492b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb64931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad164933, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32964937, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xd964939, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d96493c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7216493e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb764942, SUC_lexicon_tags_2 },
    { 0x29764942, SUC_lexicon_tags_66 },
    { 0x6e164944, SUC_lexicon_tags_17 },
    { 0x87d64944, SUC_lexicon_tags_37 },
    { 0x89764945, SUC_lexicon_tags_62 },
    { 0xfa764947, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b6494a, SUC_lexicon_tags_17 },
    { 0x2d16494b, SUC_lexicon_tags_18 },
    { 0x4296494a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87564950, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad64953, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x64f64955, SUC_lexicon_tags_7 },
    { 0xd5564955, SUC_lexicon_tags_28 },
    { 0xf2f64957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99964963, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f64968, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa556496b, SUC_lexicon_tags_2 },
    { 0x8af6496c, SUC_lexicon_tags_27 },
    { 0xeff6496d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51364976, SUC_lexicon_tags_9 },
    { 0xde364976, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf6964979, SUC_lexicon_tags_37 },
    { 0xdb16497a, SUC_lexicon_tags_61 },
    { 0x33d6497b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x29f6497d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4964988, SUC_lexicon_tags_37 },
    { 0x2e964989, SUC_lexicon_tags_48 },
    { 0x1164989, SUC_lexicon_tags_32 },
    { 0x84d64989, SUC_lexicon_tags_19 },
    { 0x58b6498c, SUC_lexicon_tags_17 },
    { 0x1536498d, SUC_lexicon_tags_55 },
    { 0x52964989, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43764991, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x78564994, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x88764996, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd6499a, SUC_lexicon_tags_38 },
    { 0xfbd6499b, SUC_lexicon_tags_81 },
    { 0x7ad6499b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd649a4, SUC_lexicon_tags_38 },
    { 0x95649a5, SUC_lexicon_tags_25 },
    { 0xb31649a6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4e1649a8, SUC_lexicon_tags_37 },
    { 0x27649a9, SUC_lexicon_tags_13 },
    { 0xc5f649aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd7649ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5649af, SUC_lexicon_tags_14 },
    { 0xdaf649b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3649b3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9649b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x429649b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x265649ba, SUC_lexicon_tags_17 },
    { 0x6fb649ba, SUC_lexicon_tags_49 },
    { 0x2c1649ba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9d9649be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619649c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f649cc, SUC_lexicon_tags_17 },
    { 0x3f9649cc, SUC_lexicon_tags_37 },
    { 0x11d649cc, SUC_lexicon_tags_12 },
    { 0x2b649cd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad649d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73f649d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x621649d9, SUC_lexicon_tags_7 },
    { 0x1c3649da, SUC_lexicon_tags_17 },
    { 0xf31649d9, SUC_lexicon_tags_8 },
    { 0x99649db, SUC_lexicon_tags_12 },
    { 0x68f649da, SUC_lexicon_tags_74 },
    { 0x63f649de, SUC_lexicon_tags_11 },
    { 0x767649df, SUC_lexicon_tags_0 },
    { 0xcc5649dd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x30b649e2, SUC_lexicon_tags_37 },
    { 0xdf7649e2, SUC_lexicon_tags_29 },
    { 0x5eb649e2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x329649e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb75649e8, SUC_lexicon_tags_191 },
    { 0xe1d649e9, SUC_lexicon_tags_17 },
    { 0xfb5649e8, SUC_lexicon_tags_15 },
    { 0xead649eb, SUC_lexicon_tags_7 },
    { 0x5fd649ec, SUC_lexicon_tags_10 },
    { 0xbfd649ec, SUC_lexicon_tags_17 },
    { 0x3f5649eb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf4f649f0, SUC_lexicon_tags_77 },
    { 0x353649f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7649f6, SUC_lexicon_tags_30 },
    { 0x3fb649f7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4b3649f9, SUC_lexicon_tags_2 },
    { 0x97d649f9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x307649fd, SUC_lexicon_tags_87 },
    { 0x5bd649fe, SUC_lexicon_tags_2 },
    { 0x49649fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb64a03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d364a07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ef64a09, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd64a0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67564a13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb64a18, SUC_lexicon_tags_21 },
    { 0x43964a19, SUC_lexicon_tags_37 },
    { 0x48564a19, SUC_lexicon_tags_1 },
    { 0x1364a1b, SUC_lexicon_tags_37 },
    { 0xa5764a1c, SUC_lexicon_tags_30 },
    { 0x21d64a1a, SUC_lexicon_tags_38 },
    { 0x78964a1e, SUC_lexicon_tags_55 },
    { 0xdb564a1f, SUC_lexicon_tags_37 },
    { 0x24764a1c, SUC_lexicon_tags_0 },
    { 0x32d64a21, SUC_lexicon_tags_46 },
    { 0xda764a22, SUC_lexicon_tags_1 },
    { 0xb0d64a1a, SUC_lexicon_tags_11 },
    { 0x82f64a1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1b64a26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d64a2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7964a30, SUC_lexicon_tags_23 },
    { 0xd9164a31, SUC_lexicon_tags_25 },
    { 0xea564a31, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b364a36, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25364a42, SUC_lexicon_tags_15 },
    { 0x60764a42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f64a46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23b64a48, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdbf64a4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7964a4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x764a52, SUC_lexicon_tags_17 },
    { 0xe0164a52, SUC_lexicon_tags_10 },
    { 0x8d564a54, SUC_lexicon_tags_2 },
    { 0x8964a55, SUC_lexicon_tags_12 },
    { 0x95b64a56, SUC_lexicon_tags_17 },
    { 0xb9b64a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef64a59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x56964a5b, SUC_lexicon_tags_11 },
    { 0x69d64a5c, SUC_lexicon_tags_1 },
    { 0x4eb64a5d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xef364a5f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84164a64, SUC_lexicon_tags_148 },
    { 0xa2164a64, SUC_lexicon_tags_14 },
    { 0xc5b64a65, SUC_lexicon_tags_23 },
    { 0x4f64a67, SUC_lexicon_tags_14 },
    { 0xcd64a68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ef64a6a, SUC_lexicon_tags_17 },
    { 0x98f64a6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3564a6f, SUC_lexicon_tags_12 },
    { 0x90164a70, SUC_lexicon_tags_12 },
    { 0x1ef64a71, SUC_lexicon_tags_15 },
    { 0x67b64a72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3764a75, SUC_lexicon_tags_9 },
    { 0x5f364a76, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b64a7d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb164a7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f964a82, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb64a88, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x53164a8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x54f64a8d, SUC_lexicon_tags_29 },
    { 0xa4564a8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a164a94, SUC_lexicon_tags_12 },
    { 0x45364a95, SUC_lexicon_tags_19 },
    { 0x8d564a96, SUC_lexicon_tags_28 },
    { 0xb5364a95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11764a99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17964a9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47364aa0, SUC_lexicon_tags_12 },
    { 0xdf64aa1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac364aa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad364ab7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4b764ab9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2364abb, SUC_lexicon_tags_134 },
    { 0xdc564abc, SUC_lexicon_tags_37 },
    { 0x1ad64abc, SUC_lexicon_tags_11 },
    { 0xb3b64abe, SUC_lexicon_tags_9 },
    { 0x77d64abc, SUC_lexicon_tags_12 },
    { 0x35564ac0, SUC_lexicon_tags_7 },
    { 0xe3764ac0, SUC_lexicon_tags_33 },
    { 0x1a564ac2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62564acb, SUC_lexicon_tags_2 },
    { 0xfad64acb, SUC_lexicon_tags_25 },
    { 0x1c764acd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8164ad0, SUC_lexicon_tags_14 },
    { 0xf6d64ad1, SUC_lexicon_tags_1 },
    { 0xdd764ad1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf64ad7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3964adc, SUC_lexicon_tags_21 },
    { 0x50d64adc, SUC_lexicon_tags_112 },
    { 0xbf64add, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x2df64ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69964ae3, SUC_lexicon_tags_45 },
    { 0xfa164ae4, SUC_lexicon_tags_11 },
    { 0xfb564ae5, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0x2364ae7, SUC_lexicon_tags_27 },
    { 0x80b64ae7, SUC_lexicon_tags_37 },
    { 0x3c764ae8, SUC_lexicon_tags_80 },
    { 0x4f564ae8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51b64aec, SUC_lexicon_tags_15 },
    { 0x3f564aed, SUC_lexicon_tags_27 },
    { 0x32b64aee, SUC_lexicon_tags_15 },
    { 0xc4964aee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xccb64af1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c764afb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b764aff, SUC_lexicon_tags_13 },
    { 0xd6364aff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb64b0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9364b0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf764b10, SUC_lexicon_tags_47 },
    { 0x48564b11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf64b17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa764b1f, SUC_lexicon_tags_7 },
    { 0x3a764b20, SUC_lexicon_tags_25 },
    { 0xb4364b21, SUC_lexicon_tags_23 },
    { 0x70964b22, SUC_lexicon_tags_11 },
    { 0x40964b22, SUC_lexicon_tags_0 },
    { 0x1a364b24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8564b27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cf64b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d964b30, SUC_lexicon_tags_23 },
    { 0x2a364b30, SUC_lexicon_tags_25 },
    { 0xb564b32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd364b36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca564b3a, SUC_lexicon_tags_11 },
    { 0xd5364b3b, SUC_lexicon_tags_47 },
    { 0x6f564b3b, SUC_lexicon_tags_55 },
    { 0x5764b3b, SUC_lexicon_tags_17 },
    { 0xa6b64b3e, SUC_lexicon_tags_12 },
    { 0xa9564b3c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x82f64b41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab64b45, SUC_lexicon_tags_17 },
    { 0x2f764b46, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde764b4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3764b50, SUC_lexicon_tags_11 },
    { 0x78364b51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f64b54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f64b59, SUC_lexicon_tags_12 },
    { 0x3cf64b59, SUC_lexicon_tags_15 },
    { 0xb9b64b5b, SUC_lexicon_tags_17 },
    { 0xb0f64b5b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2964b64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x91b64b66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x65364b69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b64b6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f64b74, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x67164b77, SUC_lexicon_tags_44 },
    { 0xb2164b77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb64b7b, SUC_lexicon_tags_21 },
    { 0xb9364b7b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57964b84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc364b8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf964b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11364b96, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb64b99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x18764b9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2164b9d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8c964b9f, SUC_lexicon_tags_25 },
    { 0x6cb64b9f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x75764ba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8164ba6, SUC_lexicon_tags_25 },
    { 0xae164ba7, SUC_lexicon_tags_17 },
    { 0x7f764ba8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b64bac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd64baf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5364bb2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x29564bb4, SUC_lexicon_tags_1 },
    { 0x48d64bb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b64bbb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c764bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a564bc5, SUC_lexicon_tags_65 },
    { 0xc1164bc6, SUC_lexicon_tags_30 },
    { 0xed964bc7, SUC_lexicon_tags_789 },
    { 0, NULL },
    { 0, NULL },
    { 0xe764bca, SUC_lexicon_tags_15 },
    { 0x2f164bcb, SUC_lexicon_tags_2 },
    { 0xdf64bcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe764bcf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e164bd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb164bda, SUC_lexicon_tags_7 },
    { 0xaf964bda, SUC_lexicon_tags_15 },
    { 0x3db64bda, SUC_lexicon_tags_0 },
    { 0xd6764bdd, SUC_lexicon_tags_87 },
    { 0xedd64bda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7db64be0, SUC_lexicon_tags_7 },
    { 0x8bb64be0, SUC_lexicon_tags_1 },
    { 0x75764be0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xeab64be4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd64be7, SUC_lexicon_tags_25 },
    { 0xe8564be8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe364bec, SUC_lexicon_tags_37 },
    { 0x4a764bed, SUC_lexicon_tags_17 },
    { 0xba964bee, SUC_lexicon_tags_9 },
    { 0x64564bec, SUC_lexicon_tags_6 },
    { 0x43f64bf0, SUC_lexicon_tags_14 },
    { 0x6eb64bf1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f64bfa, SUC_lexicon_tags_17 },
    { 0x1fd64bfb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xba764bfd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4964bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8764c02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74964c05, SUC_lexicon_tags_25 },
    { 0xaad64c06, SUC_lexicon_tags_47 },
    { 0xe5764c06, SUC_lexicon_tags_16 },
    { 0x16964c08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x23764c0b, SUC_lexicon_tags_8 },
    { 0xab964c0c, SUC_lexicon_tags_47 },
    { 0x7cf64c0d, SUC_lexicon_tags_10 },
    { 0xd2f64c0b, SUC_lexicon_tags_17 },
    { 0xa5b64c0f, SUC_lexicon_tags_7 },
    { 0x4eb64c10, SUC_lexicon_tags_16 },
    { 0x2bd64c10, SUC_lexicon_tags_10 },
    { 0x2a964c12, SUC_lexicon_tags_15 },
    { 0xe5564c13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3eb64c15, SUC_lexicon_tags_15 },
    { 0x38964c15, SUC_lexicon_tags_1272 },
    { 0x48164c16, SUC_lexicon_tags_17 },
    { 0x8fb64c18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7b64c1a, SUC_lexicon_tags_17 },
    { 0x76b64c1a, SUC_lexicon_tags_25 },
    { 0x65364c1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ed64c1e, SUC_lexicon_tags_15 },
    { 0xdfd64c1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22564c23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6d64c25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x10d64c27, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3f564c29, SUC_lexicon_tags_17 },
    { 0x4b564c2a, SUC_lexicon_tags_100 },
    { 0xcef64c29, SUC_lexicon_tags_45 },
    { 0xc2964c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f64c33, SUC_lexicon_tags_55 },
    { 0xd2564c34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4b64c36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6d64c38, SUC_lexicon_tags_39 },
    { 0x13764c39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x97964c3c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f64c40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4764c43, SUC_lexicon_tags_25 },
    { 0x94564c44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7964c46, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32564c4d, SUC_lexicon_tags_45 },
    { 0x27364c4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4564c51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f364c54, SUC_lexicon_tags_11 },
    { 0x59f64c55, SUC_lexicon_tags_7 },
    { 0x7ad64c55, SUC_lexicon_tags_60 },
    { 0xdff64c57, SUC_lexicon_tags_37 },
    { 0x4cb64c58, SUC_lexicon_tags_12 },
    { 0xc4764c57, SUC_lexicon_tags_0 },
    { 0xcd964c5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5b364c5c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3e564c5e, SUC_lexicon_tags_37 },
    { 0xd0f64c5f, SUC_lexicon_tags_25 },
    { 0xcd64c60, SUC_lexicon_tags_48 },
    { 0x9f164c61, SUC_lexicon_tags_43 },
    { 0x1ad64c60, SUC_lexicon_tags_17 },
    { 0x16364c60, SUC_lexicon_tags_65 },
    { 0x9fd64c64, SUC_lexicon_tags_2 },
    { 0x1bd64c64, SUC_lexicon_tags_10 },
    { 0xc8d64c5e, SUC_lexicon_tags_23 },
    { 0xb0164c60, SUC_lexicon_tags_2 },
    { 0xfd764c5f, SUC_lexicon_tags_37 },
    { 0x76164c69, SUC_lexicon_tags_32 },
    { 0xc1d64c69, SUC_lexicon_tags_47 },
    { 0xfc764c6b, SUC_lexicon_tags_13 },
    { 0x71764c6c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x11164c6f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab764c76, SUC_lexicon_tags_15 },
    { 0x10d64c76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f64c7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9164c7e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46364c84, SUC_lexicon_tags_12 },
    { 0xa7d64c84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5764c89, SUC_lexicon_tags_52 },
    { 0xd7364c89, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd6964c8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0364c90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef64c94, SUC_lexicon_tags_4 },
    { 0x98564c95, SUC_lexicon_tags_17 },
    { 0xd0764c96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38764c9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef64ca0, SUC_lexicon_tags_1 },
    { 0xe6364ca1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x50564ca4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x96964ca7, SUC_lexicon_tags_68 },
    { 0x7e364ca8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf364cac, SUC_lexicon_tags_14 },
    { 0x74764cad, SUC_lexicon_tags_29 },
    { 0x7a164cae, SUC_lexicon_tags_1 },
    { 0x91964cae, SUC_lexicon_tags_33 },
    { 0x28564cb0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b64cb4, SUC_lexicon_tags_17 },
    { 0x6cd64cb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf64cb8, SUC_lexicon_tags_23 },
    { 0x9e764cb9, SUC_lexicon_tags_14 },
    { 0xc4b64cb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26f64cbc, SUC_lexicon_tags_33 },
    { 0x35f64cbc, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x45364cc0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b64cc3, SUC_lexicon_tags_92 },
    { 0xcb564cc4, SUC_lexicon_tags_15 },
    { 0x55f64cc4, SUC_lexicon_tags_15 },
    { 0xbcb64cc4, SUC_lexicon_tags_13 },
    { 0x25964cc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9564ccb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc164ccf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd964cd3, SUC_lexicon_tags_17 },
    { 0x65b64cd4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xadb64cd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd2764cd8, SUC_lexicon_tags_14 },
    { 0xfa564cd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77b64cdb, SUC_lexicon_tags_2 },
    { 0xb9b64cdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1b64cde, SUC_lexicon_tags_37 },
    { 0x40764cdf, SUC_lexicon_tags_2 },
    { 0x35d64cde, SUC_lexicon_tags_63 },
    { 0x11564ce1, SUC_lexicon_tags_12 },
    { 0x7364ce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f64ce8, SUC_lexicon_tags_36 },
    { 0x68564ce9, SUC_lexicon_tags_7 },
    { 0x90564ce9, SUC_lexicon_tags_31 },
    { 0xf5164cea, SUC_lexicon_tags_11 },
    { 0x7b764cec, SUC_lexicon_tags_9 },
    { 0xd9b64cea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78364cf1, SUC_lexicon_tags_2 },
    { 0xf1164cf2, SUC_lexicon_tags_12 },
    { 0x58f64cf1, SUC_lexicon_tags_17 },
    { 0xa3964cf2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b564cf7, SUC_lexicon_tags_37 },
    { 0x7db64cf8, SUC_lexicon_tags_11 },
    { 0x3fb64cf9, SUC_lexicon_tags_60 },
    { 0x9f364cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef64d04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75764d08, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x92564d0a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b64d11, SUC_lexicon_tags_7 },
    { 0xb64d12, SUC_lexicon_tags_37 },
    { 0xf6564d11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7364d16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b64d1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d64d1f, SUC_lexicon_tags_40 },
    { 0xb9564d1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12164d23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc964d27, SUC_lexicon_tags_17 },
    { 0xea164d28, SUC_lexicon_tags_46 },
    { 0x4a364d29, SUC_lexicon_tags_25 },
    { 0x6eb64d27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab964d2c, SUC_lexicon_tags_15 },
    { 0xd2d64d2d, SUC_lexicon_tags_8 },
    { 0x26564d2c, SUC_lexicon_tags_17 },
    { 0xe9764d2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f764d34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99564d3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x54764d3d, SUC_lexicon_tags_12 },
    { 0xaaf64d3e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5b964d40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaff64d42, SUC_lexicon_tags_15 },
    { 0x89364d43, SUC_lexicon_tags_12 },
    { 0xbf64d43, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x31764d46, SUC_lexicon_tags_11 },
    { 0x31f64d47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x14764d4a, SUC_lexicon_tags_16 },
    { 0x63164d4b, SUC_lexicon_tags_8 },
    { 0xfad64d4b, SUC_lexicon_tags_1 },
    { 0x2a564d4a, SUC_lexicon_tags_1 },
    { 0xe7164d4b, SUC_lexicon_tags_43 },
    { 0xa3b64d4f, SUC_lexicon_tags_66 },
    { 0x3ff64d4f, SUC_lexicon_tags_37 },
    { 0x88164d51, SUC_lexicon_tags_21 },
    { 0x9db64d51, SUC_lexicon_tags_47 },
    { 0xbd564d51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28f64d55, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b964d5b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfe164d5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d64d60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5164d65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f64d68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11764d6f, SUC_lexicon_tags_19 },
    { 0xb5b64d6f, SUC_lexicon_tags_55 },
    { 0xa6f64d70, SUC_lexicon_tags_1 },
    { 0x26364d72, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x59b64d74, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2964d7b, SUC_lexicon_tags_2 },
    { 0xbb364d7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9564d7e, SUC_lexicon_tags_33 },
    { 0xd4364d7f, SUC_lexicon_tags_1 },
    { 0x3b764d7f, SUC_lexicon_tags_0 },
    { 0x77164d81, SUC_lexicon_tags_60 },
    { 0xed764d82, SUC_lexicon_tags_42 },
    { 0x9564d81, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f64d87, SUC_lexicon_tags_12 },
    { 0x3e164d87, SUC_lexicon_tags_63 },
    { 0x52564d89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeab64d8b, SUC_lexicon_tags_29 },
    { 0x30364d8b, SUC_lexicon_tags_7 },
    { 0xb9364d8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb564d8f, SUC_lexicon_tags_12 },
    { 0xccd64d8f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d64d95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d764d97, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d64d9b, SUC_lexicon_tags_12 },
    { 0x34964d9b, SUC_lexicon_tags_13 },
    { 0xb1964d9d, SUC_lexicon_tags_40 },
    { 0xf8564d9d, SUC_lexicon_tags_7 },
    { 0x6ff64d9f, SUC_lexicon_tags_63 },
    { 0x9d164da0, SUC_lexicon_tags_0 },
    { 0x53f64d9e, SUC_lexicon_tags_65 },
    { 0x38f64da2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f64da6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37564da9, SUC_lexicon_tags_12 },
    { 0x98764daa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75164dad, SUC_lexicon_tags_31 },
    { 0xbf564dae, SUC_lexicon_tags_37 },
    { 0xf8964daf, SUC_lexicon_tags_59 },
    { 0xac364db0, SUC_lexicon_tags_2 },
    { 0xe0364db0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c164dba, SUC_lexicon_tags_56 },
    { 0x9a564dba, SUC_lexicon_tags_39 },
    { 0x80f64dbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ff64dbe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfed64dc0, SUC_lexicon_tags_37 },
    { 0xa1964dc1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x31964dc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d764dca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef64dcf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d64dd6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82964dde, SUC_lexicon_tags_30 },
    { 0x63f64ddf, SUC_lexicon_tags_2 },
    { 0xe3764de0, SUC_lexicon_tags_229 },
    { 0x9b64de1, SUC_lexicon_tags_14 },
    { 0x39764de0, SUC_lexicon_tags_11 },
    { 0xf3964de3, SUC_lexicon_tags_17 },
    { 0xf3d64de0, SUC_lexicon_tags_12 },
    { 0x15b64ddf, SUC_lexicon_tags_81 },
    { 0x6ad64de6, SUC_lexicon_tags_37 },
    { 0xcf564de7, SUC_lexicon_tags_37 },
    { 0x59b64de7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfd164dea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcef64dec, SUC_lexicon_tags_43 },
    { 0x72764ded, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d64df0, SUC_lexicon_tags_38 },
    { 0xf3b64df1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53f64df3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6364df7, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0x4a564df9, SUC_lexicon_tags_12 },
    { 0x4f764dfa, SUC_lexicon_tags_17 },
    { 0xbf964dfb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x60164dfe, SUC_lexicon_tags_11 },
    { 0x92b64dfe, SUC_lexicon_tags_15 },
    { 0x69964dfe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f64e02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d764e04, SUC_lexicon_tags_27 },
    { 0xc7764e05, SUC_lexicon_tags_1 },
    { 0xbab64e06, SUC_lexicon_tags_112 },
    { 0x69564e07, SUC_lexicon_tags_60 },
    { 0x2c364e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42f64e0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e364e0e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc1364e10, SUC_lexicon_tags_15 },
    { 0xb8764e11, SUC_lexicon_tags_1 },
    { 0x23764e12, SUC_lexicon_tags_4 },
    { 0x96764e13, SUC_lexicon_tags_39 },
    { 0xee364e14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89564e16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60364e19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66764e1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf364e22, SUC_lexicon_tags_7 },
    { 0x15964e22, SUC_lexicon_tags_1 },
    { 0xd1364e24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89964e26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b364e28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbd64e2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30364e2e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb64e33, SUC_lexicon_tags_25 },
    { 0x4ed64e34, SUC_lexicon_tags_106 },
    { 0xc1f64e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f164e3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7564e3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7564e3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd64e41, SUC_lexicon_tags_87 },
    { 0xb3b64e42, SUC_lexicon_tags_23 },
    { 0x5e764e43, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x63364e46, SUC_lexicon_tags_74 },
    { 0xecf64e47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf64e4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79364e52, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f64e57, SUC_lexicon_tags_17 },
    { 0x4b964e58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf564e5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc2364e5d, SUC_lexicon_tags_33 },
    { 0x89b64e5e, SUC_lexicon_tags_11 },
    { 0x7cf64e5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35164e62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfad64e64, SUC_lexicon_tags_25 },
    { 0xa964e64, SUC_lexicon_tags_1 },
    { 0xb9b64e66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad64e6a, SUC_lexicon_tags_66 },
    { 0x45964e6b, SUC_lexicon_tags_12 },
    { 0xaef64e6c, SUC_lexicon_tags_45 },
    { 0x11f64e6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5764e70, SUC_lexicon_tags_7 },
    { 0xc3364e71, SUC_lexicon_tags_17 },
    { 0x81b64e72, SUC_lexicon_tags_25 },
    { 0x7b964e73, SUC_lexicon_tags_16 },
    { 0xd4964e73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f164e77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa764e7c, SUC_lexicon_tags_7 },
    { 0xcdb64e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48b64e7f, SUC_lexicon_tags_8 },
    { 0x3bd64e80, SUC_lexicon_tags_25 },
    { 0x97f64e81, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15364e8e, SUC_lexicon_tags_15 },
    { 0x5bf64e8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38764e91, SUC_lexicon_tags_1 },
    { 0x33f64e91, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf164e95, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47364e99, SUC_lexicon_tags_97 },
    { 0xf9164e99, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x81764e9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18564ea1, SUC_lexicon_tags_25 },
    { 0x79764ea2, SUC_lexicon_tags_45 },
    { 0x93564ea1, SUC_lexicon_tags_8 },
    { 0x77f64ea4, SUC_lexicon_tags_29 },
    { 0x32964ea5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7564ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3964eaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ab64eac, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x30964eaf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf764eb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6964eb3, SUC_lexicon_tags_12 },
    { 0xc1164eb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95f64eb6, SUC_lexicon_tags_37 },
    { 0x58f64eb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3164eba, SUC_lexicon_tags_11 },
    { 0x58b64ebb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20d64ebd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68964ec4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb64ec8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x79164eca, SUC_lexicon_tags_64 },
    { 0x9db64ecb, SUC_lexicon_tags_37 },
    { 0x8f564ecb, SUC_lexicon_tags_19 },
    { 0xffd64ecd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf64ed4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73964ed6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd564ed9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d64edc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf0764ede, SUC_lexicon_tags_12 },
    { 0x364edf, SUC_lexicon_tags_17 },
    { 0x8fd64ee0, SUC_lexicon_tags_114 },
    { 0x9df64ee1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f64ee5, SUC_lexicon_tags_17 },
    { 0x46764ee6, SUC_lexicon_tags_1318 },
    { 0, NULL },
    { 0, NULL },
    { 0x92164ee9, SUC_lexicon_tags_0 },
    { 0xfc964ee9, SUC_lexicon_tags_47 },
    { 0xd6d64eeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4564eed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d64ef3, SUC_lexicon_tags_23 },
    { 0x72964ef4, SUC_lexicon_tags_27 },
    { 0x28964ef5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f764ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39d64efb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76764eff, SUC_lexicon_tags_43 },
    { 0x8d764f00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7bb64f02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d64f06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d64f09, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x56564f0b, SUC_lexicon_tags_66 },
    { 0xd5d64f0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd64f0f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b364f13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e964f18, SUC_lexicon_tags_17 },
    { 0xd1764f19, SUC_lexicon_tags_12 },
    { 0x34b64f1a, SUC_lexicon_tags_1 },
    { 0xef764f19, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0964f1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc364f24, SUC_lexicon_tags_65 },
    { 0xe2564f25, SUC_lexicon_tags_144 },
    { 0x4c164f26, SUC_lexicon_tags_16 },
    { 0x3ef64f26, SUC_lexicon_tags_7 },
    { 0xc7964f26, SUC_lexicon_tags_4 },
    { 0x3df64f29, SUC_lexicon_tags_25 },
    { 0x56564f24, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xaff64f2c, SUC_lexicon_tags_15 },
    { 0xc0364f2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45764f31, SUC_lexicon_tags_23 },
    { 0xd6964f32, SUC_lexicon_tags_12 },
    { 0x98164f32, SUC_lexicon_tags_17 },
    { 0xc0764f34, SUC_lexicon_tags_103 },
    { 0x10364f34, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1e164f37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e164f3a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb64f3d, SUC_lexicon_tags_23 },
    { 0xd3b64f3d, SUC_lexicon_tags_10 },
    { 0xb5b64f3f, SUC_lexicon_tags_113 },
    { 0x7c964f3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda764f47, SUC_lexicon_tags_17 },
    { 0x39d64f48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x28764f4a, SUC_lexicon_tags_47 },
    { 0x27364f4a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa6764f4d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b64f54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14164f5a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x5e564f5c, SUC_lexicon_tags_15 },
    { 0xffb64f5d, SUC_lexicon_tags_8 },
    { 0x25f64f5d, SUC_lexicon_tags_27 },
    { 0x45164f5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x17564f61, SUC_lexicon_tags_40 },
    { 0x26f64f61, SUC_lexicon_tags_37 },
    { 0x1a764f63, SUC_lexicon_tags_37 },
    { 0x25564f63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d64f69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b164f70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb64f74, SUC_lexicon_tags_29 },
    { 0x3cd64f75, SUC_lexicon_tags_12 },
    { 0x90564f74, SUC_lexicon_tags_1 },
    { 0x11964f77, SUC_lexicon_tags_37 },
    { 0xf9564f78, SUC_lexicon_tags_92 },
    { 0x35564f75, SUC_lexicon_tags_17 },
    { 0x26164f7a, SUC_lexicon_tags_1 },
    { 0xa9964f78, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb64f82, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3364f86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b64f8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4764f96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2164f99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c364f9e, SUC_lexicon_tags_7 },
    { 0xd0164f9e, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4164fa4, SUC_lexicon_tags_8 },
    { 0xe6364fa5, SUC_lexicon_tags_45 },
    { 0x2a964fa6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x49764fa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32364faa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa4964fac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d64fb3, SUC_lexicon_tags_14 },
    { 0x15564fb3, SUC_lexicon_tags_46 },
    { 0x78764fb5, SUC_lexicon_tags_35 },
    { 0x41564fb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6a764fb8, SUC_lexicon_tags_19 },
    { 0x6c364fb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a364fbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99d64fbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a964fc1, SUC_lexicon_tags_25 },
    { 0x59f64fc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x41f64fc4, SUC_lexicon_tags_14 },
    { 0xc1164fc4, SUC_lexicon_tags_1 },
    { 0x1df64fc6, SUC_lexicon_tags_15 },
    { 0x39964fc6, SUC_lexicon_tags_30 },
    { 0x64564fc8, SUC_lexicon_tags_21 },
    { 0x55f64fc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75f64fcb, SUC_lexicon_tags_141 },
    { 0x66f64fcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17564fcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33764fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df64fd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2564fdc, SUC_lexicon_tags_7 },
    { 0xd7b64fdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad364fe9, SUC_lexicon_tags_17 },
    { 0x72364fea, SUC_lexicon_tags_32 },
    { 0x10d64fe9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x12964fee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8dd64ff0, SUC_lexicon_tags_35 },
    { 0x50764ff0, SUC_lexicon_tags_63 },
    { 0x78764ff0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xac164ff5, SUC_lexicon_tags_8 },
    { 0xb9f64ff5, SUC_lexicon_tags_17 },
    { 0x88b64ff6, SUC_lexicon_tags_25 },
    { 0x94164ff8, SUC_lexicon_tags_15 },
    { 0xe8b64ff7, SUC_lexicon_tags_25 },
    { 0x97564ffa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x72164ffd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbf64fff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe165007, SUC_lexicon_tags_45 },
    { 0xa6965008, SUC_lexicon_tags_25 },
    { 0x94565008, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc076500b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f6500e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f65011, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d965016, SUC_lexicon_tags_27 },
    { 0xf3965016, SUC_lexicon_tags_1 },
    { 0x1d65017, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9496501a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1565022, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73d65024, SUC_lexicon_tags_25 },
    { 0xe765025, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad965027, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc836502b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xecf6502d, SUC_lexicon_tags_1 },
    { 0xd3b6502d, SUC_lexicon_tags_106 },
    { 0x456502d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0565033, SUC_lexicon_tags_8 },
    { 0x5cd65034, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f565038, SUC_lexicon_tags_25 },
    { 0x9e365039, SUC_lexicon_tags_66 },
    { 0xab76503a, SUC_lexicon_tags_14 },
    { 0x9636503b, SUC_lexicon_tags_15 },
    { 0x5d36503b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16565045, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34165048, SUC_lexicon_tags_25 },
    { 0xfab65049, SUC_lexicon_tags_2 },
    { 0x5996504a, SUC_lexicon_tags_58 },
    { 0xfcf6504b, SUC_lexicon_tags_4 },
    { 0xfe96504c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d65052, SUC_lexicon_tags_1 },
    { 0x98365053, SUC_lexicon_tags_4 },
    { 0xffd65054, SUC_lexicon_tags_46 },
    { 0x9c565055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff6505d, SUC_lexicon_tags_11 },
    { 0xa576505d, SUC_lexicon_tags_14 },
    { 0x2cf6505f, SUC_lexicon_tags_30 },
    { 0xa0d65060, SUC_lexicon_tags_68 },
    { 0x2816505e, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0xa7765063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2765066, SUC_lexicon_tags_37 },
    { 0xe3365066, SUC_lexicon_tags_7 },
    { 0x1cb65068, SUC_lexicon_tags_37 },
    { 0x10565067, SUC_lexicon_tags_45 },
    { 0xd8b6506a, SUC_lexicon_tags_15 },
    { 0xc4f6506a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe216506f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7b65071, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37165074, SUC_lexicon_tags_100 },
    { 0xc4165075, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d65078, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b6507d, SUC_lexicon_tags_10 },
    { 0x70b6507e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5b65080, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x55765082, SUC_lexicon_tags_1 },
    { 0xfdf65082, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7f65085, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f165088, SUC_lexicon_tags_25 },
    { 0xa9965089, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb65090, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd965093, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd796509a, SUC_lexicon_tags_45 },
    { 0xa976509b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3650a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x239650a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9650a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea5650a9, SUC_lexicon_tags_151 },
    { 0xc7f650aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe05650ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcd1650ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51650b0, SUC_lexicon_tags_15 },
    { 0xd61650b1, SUC_lexicon_tags_27 },
    { 0x467650b1, SUC_lexicon_tags_1029 },
    { 0xb39650b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46f650b5, SUC_lexicon_tags_28 },
    { 0x317650b6, SUC_lexicon_tags_4 },
    { 0xa07650b7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07650bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd650bf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5650c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf650c5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x327650c7, SUC_lexicon_tags_15 },
    { 0x5cf650c8, SUC_lexicon_tags_3 },
    { 0xf2b650c9, SUC_lexicon_tags_25 },
    { 0x9d3650ca, SUC_lexicon_tags_37 },
    { 0x7e1650cb, SUC_lexicon_tags_1 },
    { 0x371650ca, SUC_lexicon_tags_26 },
    { 0x9cb650c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01650d1, SUC_lexicon_tags_1250 },
    { 0x487650d2, SUC_lexicon_tags_15 },
    { 0x3d5650d3, SUC_lexicon_tags_8 },
    { 0xffb650d4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23650d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d650de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27650e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x579650e5, SUC_lexicon_tags_7 },
    { 0x9c5650e5, SUC_lexicon_tags_33 },
    { 0x899650e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb55650e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x96f650eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5650ef, SUC_lexicon_tags_65 },
    { 0x5e3650ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc03650f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb650f7, SUC_lexicon_tags_17 },
    { 0x32d650f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc3650fa, SUC_lexicon_tags_49 },
    { 0xa53650fb, SUC_lexicon_tags_23 },
    { 0x21b650fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5365101, SUC_lexicon_tags_63 },
    { 0x47165102, SUC_lexicon_tags_7 },
    { 0xd0965103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73165109, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f6510d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b65112, SUC_lexicon_tags_12 },
    { 0xcc565113, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c565117, SUC_lexicon_tags_87 },
    { 0x42565117, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd5f6511a, SUC_lexicon_tags_25 },
    { 0x53f6511b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90965121, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9365123, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7676512a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16512d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x69565130, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac965135, SUC_lexicon_tags_60 },
    { 0x65765136, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6bf65138, SUC_lexicon_tags_47 },
    { 0xd7b65139, SUC_lexicon_tags_25 },
    { 0xeb56513a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc896513c, SUC_lexicon_tags_33 },
    { 0x3796513d, SUC_lexicon_tags_31 },
    { 0xfbd6513d, SUC_lexicon_tags_37 },
    { 0x36b6513f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c365142, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x7a965144, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d765148, SUC_lexicon_tags_23 },
    { 0x4a765149, SUC_lexicon_tags_14 },
    { 0x4e96514a, SUC_lexicon_tags_49 },
    { 0xc4b6514b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63f6514d, SUC_lexicon_tags_11 },
    { 0x79f6514e, SUC_lexicon_tags_3 },
    { 0xe436514d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f765154, SUC_lexicon_tags_29 },
    { 0x2d65155, SUC_lexicon_tags_30 },
    { 0x38b65156, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d6515b, SUC_lexicon_tags_19 },
    { 0x6af6515c, SUC_lexicon_tags_37 },
    { 0xcc36515c, SUC_lexicon_tags_23 },
    { 0x2bf6515d, SUC_lexicon_tags_43 },
    { 0x3216515f, SUC_lexicon_tags_31 },
    { 0x1d76515f, SUC_lexicon_tags_30 },
    { 0xee165161, SUC_lexicon_tags_63 },
    { 0xa3765162, SUC_lexicon_tags_25 },
    { 0x5c165162, SUC_lexicon_tags_47 },
    { 0x1f65162, SUC_lexicon_tags_7 },
    { 0x89165165, SUC_lexicon_tags_29 },
    { 0x3ef65163, SUC_lexicon_tags_63 },
    { 0xcf165161, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7965169, SUC_lexicon_tags_38 },
    { 0x56365169, SUC_lexicon_tags_39 },
    { 0x1b76516a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f65173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d6517a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf6517d, SUC_lexicon_tags_11 },
    { 0x4bd6517d, SUC_lexicon_tags_82 },
    { 0x9c56517f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed65183, SUC_lexicon_tags_17 },
    { 0xa9965184, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa165186, SUC_lexicon_tags_65 },
    { 0x97b65187, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f6518a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb896518d, SUC_lexicon_tags_37 },
    { 0x3676518e, SUC_lexicon_tags_15 },
    { 0x2b36518e, SUC_lexicon_tags_124 },
    { 0xbf65190, SUC_lexicon_tags_12 },
    { 0x5f96518f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0965194, SUC_lexicon_tags_37 },
    { 0xc0365194, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79765197, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8165199, SUC_lexicon_tags_30 },
    { 0x4576519a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b76519c, SUC_lexicon_tags_32 },
    { 0x4236519d, SUC_lexicon_tags_12 },
    { 0x556519e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x269651a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf651a4, SUC_lexicon_tags_15 },
    { 0xc6b651a4, SUC_lexicon_tags_63 },
    { 0x15d651a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d651ad, SUC_lexicon_tags_15 },
    { 0xbc7651ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73651ba, SUC_lexicon_tags_1 },
    { 0xa61651bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x18f651bd, SUC_lexicon_tags_49 },
    { 0x74f651be, SUC_lexicon_tags_26 },
    { 0x5fd651bf, SUC_lexicon_tags_11 },
    { 0x58b651bf, SUC_lexicon_tags_25 },
    { 0xbf1651c0, SUC_lexicon_tags_7 },
    { 0x4eb651c2, SUC_lexicon_tags_35 },
    { 0xaa1651c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb651c8, SUC_lexicon_tags_77 },
    { 0xe1b651c9, SUC_lexicon_tags_15 },
    { 0xf41651ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xad5651cc, SUC_lexicon_tags_25 },
    { 0xc57651cd, SUC_lexicon_tags_17 },
    { 0xce7651cd, SUC_lexicon_tags_7 },
    { 0x22f651ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x70d651d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf651d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xff651d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x539651d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201651e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd651e3, SUC_lexicon_tags_11 },
    { 0x8dd651e4, SUC_lexicon_tags_7 },
    { 0x5af651e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d651e9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699651ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61651f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f651f7, SUC_lexicon_tags_38 },
    { 0xae9651f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd79651fa, SUC_lexicon_tags_197 },
    { 0x76b651fa, SUC_lexicon_tags_30 },
    { 0xa57651fa, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9365200, SUC_lexicon_tags_18 },
    { 0x82365200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48965207, SUC_lexicon_tags_61 },
    { 0x34765207, SUC_lexicon_tags_17 },
    { 0x94365209, SUC_lexicon_tags_25 },
    { 0xc6b65208, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6076520f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b65215, SUC_lexicon_tags_23 },
    { 0x54565215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83965219, SUC_lexicon_tags_19 },
    { 0xbf65219, SUC_lexicon_tags_45 },
    { 0x7296521a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7756521e, SUC_lexicon_tags_47 },
    { 0xbd36521f, SUC_lexicon_tags_47 },
    { 0xfd76521e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b965223, SUC_lexicon_tags_0 },
    { 0x75765224, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4eb65226, SUC_lexicon_tags_11 },
    { 0xe4d65227, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf6522d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5996522f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x44365231, SUC_lexicon_tags_11 },
    { 0x16765231, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1165239, SUC_lexicon_tags_6 },
    { 0xed16523a, SUC_lexicon_tags_38 },
    { 0x716523a, SUC_lexicon_tags_26 },
    { 0xf996523c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb16523f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b65243, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5565249, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fb6524b, SUC_lexicon_tags_1 },
    { 0x1bd6524c, SUC_lexicon_tags_7 },
    { 0x21b6524c, SUC_lexicon_tags_7 },
    { 0xcd6524b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb65251, SUC_lexicon_tags_2 },
    { 0x7b565251, SUC_lexicon_tags_2 },
    { 0xb1d65251, SUC_lexicon_tags_39 },
    { 0x5ed65252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x51765257, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d6525a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54b6525c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81765261, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f165268, SUC_lexicon_tags_65 },
    { 0x17b65268, SUC_lexicon_tags_11 },
    { 0x14165269, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba565272, SUC_lexicon_tags_33 },
    { 0xa1965273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2756527e, SUC_lexicon_tags_2 },
    { 0x8616527f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f565282, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e365285, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x92965287, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b16528a, SUC_lexicon_tags_26 },
    { 0x25d6528b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f6528e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9f65290, SUC_lexicon_tags_31 },
    { 0xf5565290, SUC_lexicon_tags_37 },
    { 0x3d565292, SUC_lexicon_tags_25 },
    { 0xdbf65292, SUC_lexicon_tags_7 },
    { 0x3a965290, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9165296, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc2f65298, SUC_lexicon_tags_1 },
    { 0xc8565299, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9816529f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db652a2, SUC_lexicon_tags_17 },
    { 0x81f652a3, SUC_lexicon_tags_28 },
    { 0x443652a4, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1652a7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53652ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe1d652ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d652b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbb9652b2, SUC_lexicon_tags_116 },
    { 0x5b9652b2, SUC_lexicon_tags_12 },
    { 0x92b652b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29652b8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf652bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc3652bd, SUC_lexicon_tags_62 },
    { 0x9dd652be, SUC_lexicon_tags_1 },
    { 0xedd652bf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5652c3, SUC_lexicon_tags_37 },
    { 0x7e3652c4, SUC_lexicon_tags_53 },
    { 0xa5d652c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295652c9, SUC_lexicon_tags_12 },
    { 0x501652ca, SUC_lexicon_tags_30 },
    { 0x63d652cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa97652cd, SUC_lexicon_tags_4 },
    { 0x2bb652ce, SUC_lexicon_tags_14 },
    { 0x319652ce, SUC_lexicon_tags_60 },
    { 0x2cb652d0, SUC_lexicon_tags_7 },
    { 0xba9652d1, SUC_lexicon_tags_25 },
    { 0x101652d2, SUC_lexicon_tags_7 },
    { 0xdcd652d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9652da, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x295652dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb652de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f652e5, SUC_lexicon_tags_37 },
    { 0x961652e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x197652e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad3652eb, SUC_lexicon_tags_17 },
    { 0x679652ec, SUC_lexicon_tags_9 },
    { 0xb47652eb, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3652f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf71652f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x475652f9, SUC_lexicon_tags_1 },
    { 0x8d652fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9652fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce365303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64365306, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6136530b, SUC_lexicon_tags_23 },
    { 0xa196530b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9765310, SUC_lexicon_tags_25 },
    { 0x7765311, SUC_lexicon_tags_11 },
    { 0x6ed65310, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c165314, SUC_lexicon_tags_11 },
    { 0x28f65315, SUC_lexicon_tags_33 },
    { 0x85965315, SUC_lexicon_tags_35 },
    { 0x27565315, SUC_lexicon_tags_17 },
    { 0x5c965318, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x97f6531a, SUC_lexicon_tags_27 },
    { 0x4576531b, SUC_lexicon_tags_25 },
    { 0xae76531a, SUC_lexicon_tags_61 },
    { 0xf916531d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf836531f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f65322, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8376532b, SUC_lexicon_tags_25 },
    { 0xceb6532c, SUC_lexicon_tags_10 },
    { 0x6c16532c, SUC_lexicon_tags_1 },
    { 0x17f6532e, SUC_lexicon_tags_69 },
    { 0xdf56532f, SUC_lexicon_tags_1 },
    { 0x2af65330, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd965333, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff65338, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f6533d, SUC_lexicon_tags_27 },
    { 0x4376533d, SUC_lexicon_tags_33 },
    { 0x8d76533f, SUC_lexicon_tags_60 },
    { 0x33b6533e, SUC_lexicon_tags_14 },
    { 0xd6f65341, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33b65343, SUC_lexicon_tags_1 },
    { 0xe8165344, SUC_lexicon_tags_6 },
    { 0xeb65345, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4765347, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96534a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5df6534c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5876534e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8365351, SUC_lexicon_tags_13 },
    { 0xc0d65352, SUC_lexicon_tags_7 },
    { 0xff165353, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32b65355, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e765357, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296535b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa516535f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b65363, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf565368, SUC_lexicon_tags_49 },
    { 0xf6165369, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c36536d, SUC_lexicon_tags_918 },
    { 0, NULL },
    { 0xf2b6536f, SUC_lexicon_tags_34 },
    { 0xda16536f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96b65372, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83165378, SUC_lexicon_tags_7 },
    { 0x2d65378, SUC_lexicon_tags_37 },
    { 0x39765379, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x1016537c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd236537e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd65381, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f65385, SUC_lexicon_tags_12 },
    { 0xb0b65386, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b6538e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d65393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b653a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7653ac, SUC_lexicon_tags_11 },
    { 0x8c1653ad, SUC_lexicon_tags_2 },
    { 0x177653ad, SUC_lexicon_tags_11 },
    { 0xaf9653ad, SUC_lexicon_tags_14 },
    { 0xaa5653ac, SUC_lexicon_tags_25 },
    { 0x53b653b0, SUC_lexicon_tags_12 },
    { 0x7b9653b2, SUC_lexicon_tags_1 },
    { 0x4b9653b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x535653b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1653bb, SUC_lexicon_tags_1 },
    { 0xe41653bc, SUC_lexicon_tags_37 },
    { 0x7ef653bd, SUC_lexicon_tags_25 },
    { 0xc5b653bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9653c1, SUC_lexicon_tags_14 },
    { 0xd67653c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f653c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1653d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc3653d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219653d7, SUC_lexicon_tags_74 },
    { 0xcb7653d8, SUC_lexicon_tags_39 },
    { 0x505653d9, SUC_lexicon_tags_11 },
    { 0xf8d653da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d653dd, SUC_lexicon_tags_26 },
    { 0x9b653dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa1653e0, SUC_lexicon_tags_18 },
    { 0x1f7653e0, SUC_lexicon_tags_17 },
    { 0x4d1653e1, SUC_lexicon_tags_34 },
    { 0xe67653e3, SUC_lexicon_tags_9 },
    { 0x507653e4, SUC_lexicon_tags_17 },
    { 0xe5f653e5, SUC_lexicon_tags_11 },
    { 0x71b653e6, SUC_lexicon_tags_33 },
    { 0x2ef653e5, SUC_lexicon_tags_6 },
    { 0x351653e8, SUC_lexicon_tags_31 },
    { 0x839653e9, SUC_lexicon_tags_2 },
    { 0xdeb653e6, SUC_lexicon_tags_14 },
    { 0x171653e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601653f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7653f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab653f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88d653fa, SUC_lexicon_tags_27 },
    { 0x137653fb, SUC_lexicon_tags_16 },
    { 0xee5653fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x919653fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0b65400, SUC_lexicon_tags_29 },
    { 0x8fd65401, SUC_lexicon_tags_0 },
    { 0x29765401, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd65405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa565407, SUC_lexicon_tags_15 },
    { 0x9b565407, SUC_lexicon_tags_45 },
    { 0xb5765409, SUC_lexicon_tags_11 },
    { 0xfe56540a, SUC_lexicon_tags_22 },
    { 0xc9565409, SUC_lexicon_tags_13 },
    { 0xee56540a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df65415, SUC_lexicon_tags_1 },
    { 0x59365415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49165418, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd6541b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91365420, SUC_lexicon_tags_25 },
    { 0xa1165421, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e965423, SUC_lexicon_tags_37 },
    { 0x3f565424, SUC_lexicon_tags_66 },
    { 0xf3765425, SUC_lexicon_tags_11 },
    { 0xf0765425, SUC_lexicon_tags_69 },
    { 0x65965427, SUC_lexicon_tags_38 },
    { 0xa3765426, SUC_lexicon_tags_12 },
    { 0x96365428, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb56542b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d16542e, SUC_lexicon_tags_29 },
    { 0x9496542f, SUC_lexicon_tags_7 },
    { 0xc6f65430, SUC_lexicon_tags_43 },
    { 0xa0b65431, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b65438, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa656543a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b6543d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c965440, SUC_lexicon_tags_15 },
    { 0xdfb65441, SUC_lexicon_tags_37 },
    { 0xeed65442, SUC_lexicon_tags_55 },
    { 0xce765442, SUC_lexicon_tags_15 },
    { 0x53f65444, SUC_lexicon_tags_4 },
    { 0xb8765444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ed65447, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x99965449, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80d6544b, SUC_lexicon_tags_26 },
    { 0x3af6544b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d65457, SUC_lexicon_tags_1 },
    { 0x2f365457, SUC_lexicon_tags_33 },
    { 0x4cf65459, SUC_lexicon_tags_11 },
    { 0xba56545a, SUC_lexicon_tags_29 },
    { 0xe3d65457, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x436545d, SUC_lexicon_tags_26 },
    { 0xad6545e, SUC_lexicon_tags_49 },
    { 0x6336545e, SUC_lexicon_tags_0 },
    { 0x5b6545d, SUC_lexicon_tags_0 },
    { 0x6e565461, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb765463, SUC_lexicon_tags_19 },
    { 0xdbd65463, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69b65466, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1476546c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa616546e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f65476, SUC_lexicon_tags_3 },
    { 0xe765477, SUC_lexicon_tags_0 },
    { 0x37b65478, SUC_lexicon_tags_37 },
    { 0xc9f65479, SUC_lexicon_tags_17 },
    { 0x14165477, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e96547c, SUC_lexicon_tags_15 },
    { 0x4a36547d, SUC_lexicon_tags_29 },
    { 0x15d6547d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2765481, SUC_lexicon_tags_17 },
    { 0x40765482, SUC_lexicon_tags_32 },
    { 0x97765482, SUC_lexicon_tags_12 },
    { 0x93365484, SUC_lexicon_tags_37 },
    { 0x20365485, SUC_lexicon_tags_17 },
    { 0x90d65486, SUC_lexicon_tags_15 },
    { 0xcad65487, SUC_lexicon_tags_12 },
    { 0x59965488, SUC_lexicon_tags_13 },
    { 0xf4565486, SUC_lexicon_tags_17 },
    { 0xe356548a, SUC_lexicon_tags_17 },
    { 0xfdb6548a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3316548e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xedf65490, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b65494, SUC_lexicon_tags_44 },
    { 0xc0d65495, SUC_lexicon_tags_2 },
    { 0x2b165496, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7096549a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8e76549c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9654a0, SUC_lexicon_tags_7 },
    { 0x7e5654a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x55f654a3, SUC_lexicon_tags_17 },
    { 0xfe9654a4, SUC_lexicon_tags_1 },
    { 0x293654a5, SUC_lexicon_tags_47 },
    { 0xd3d654a6, SUC_lexicon_tags_35 },
    { 0x15d654a5, SUC_lexicon_tags_11 },
    { 0xdfb654a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309654ae, SUC_lexicon_tags_21 },
    { 0x7e5654af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3b3654b1, SUC_lexicon_tags_25 },
    { 0x4bd654b2, SUC_lexicon_tags_1 },
    { 0x5d5654b2, SUC_lexicon_tags_15 },
    { 0xd9d654b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d654b9, SUC_lexicon_tags_11 },
    { 0x7d7654b9, SUC_lexicon_tags_14 },
    { 0xda7654b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7654c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd65654c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec7654cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff654ce, SUC_lexicon_tags_4 },
    { 0x343654cf, SUC_lexicon_tags_47 },
    { 0xd8d654cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f654d3, SUC_lexicon_tags_47 },
    { 0x917654d3, SUC_lexicon_tags_11 },
    { 0x9af654d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4d654d7, SUC_lexicon_tags_29 },
    { 0x3ab654d8, SUC_lexicon_tags_34 },
    { 0xcff654d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d7654db, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51654e2, SUC_lexicon_tags_60 },
    { 0x6e1654e3, SUC_lexicon_tags_68 },
    { 0x5bb654e3, SUC_lexicon_tags_12 },
    { 0x863654e5, SUC_lexicon_tags_2 },
    { 0xbc5654e6, SUC_lexicon_tags_37 },
    { 0x90f654e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687654eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab654f2, SUC_lexicon_tags_37 },
    { 0xe91654f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed654f5, SUC_lexicon_tags_25 },
    { 0x32f654f5, SUC_lexicon_tags_1 },
    { 0x109654f7, SUC_lexicon_tags_60 },
    { 0x983654f8, SUC_lexicon_tags_15 },
    { 0xcf654f9, SUC_lexicon_tags_9 },
    { 0x3b3654fa, SUC_lexicon_tags_8 },
    { 0x1ad654f8, SUC_lexicon_tags_56 },
    { 0x37d654fc, SUC_lexicon_tags_33 },
    { 0x939654fd, SUC_lexicon_tags_12 },
    { 0xe31654fd, SUC_lexicon_tags_84 },
    { 0x8f9654fc, SUC_lexicon_tags_17 },
    { 0xfd765500, SUC_lexicon_tags_11 },
    { 0xa4b654fe, SUC_lexicon_tags_17 },
    { 0x69f654ff, SUC_lexicon_tags_5 },
    { 0xd6b65503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef365505, SUC_lexicon_tags_48 },
    { 0x10365505, SUC_lexicon_tags_46 },
    { 0xa2f65507, SUC_lexicon_tags_86 },
    { 0xbbf65508, SUC_lexicon_tags_33 },
    { 0xd3565509, SUC_lexicon_tags_33 },
    { 0x1b6550a, SUC_lexicon_tags_25 },
    { 0x1d16550b, SUC_lexicon_tags_60 },
    { 0x6f56550c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd996550e, SUC_lexicon_tags_7 },
    { 0x6e16550f, SUC_lexicon_tags_27 },
    { 0xc165510, SUC_lexicon_tags_11 },
    { 0xaab65511, SUC_lexicon_tags_37 },
    { 0x2656550e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee765514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b65518, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4496551d, SUC_lexicon_tags_16 },
    { 0x2676551e, SUC_lexicon_tags_12 },
    { 0x1976551f, SUC_lexicon_tags_30 },
    { 0x92765520, SUC_lexicon_tags_12 },
    { 0xc5d6551d, SUC_lexicon_tags_38 },
    { 0x68765522, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5965528, SUC_lexicon_tags_15 },
    { 0x6b965529, SUC_lexicon_tags_12 },
    { 0xe365529, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4a96552c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc0b6552e, SUC_lexicon_tags_28 },
    { 0xf836552f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8965537, SUC_lexicon_tags_33 },
    { 0xeb165538, SUC_lexicon_tags_49 },
    { 0x6f365537, SUC_lexicon_tags_3 },
    { 0x2d165539, SUC_lexicon_tags_45 },
    { 0xb5d65537, SUC_lexicon_tags_1 },
    { 0x5e36553c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d76553f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2565546, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x31765548, SUC_lexicon_tags_1 },
    { 0xba965549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d6554c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbe56554e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1565552, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f165558, SUC_lexicon_tags_23 },
    { 0x56965559, SUC_lexicon_tags_37 },
    { 0x8f6555a, SUC_lexicon_tags_30 },
    { 0x1bf6555b, SUC_lexicon_tags_37 },
    { 0x25d6555b, SUC_lexicon_tags_1 },
    { 0xc8d65558, SUC_lexicon_tags_0 },
    { 0xe976555d, SUC_lexicon_tags_49 },
    { 0xe1b6555d, SUC_lexicon_tags_37 },
    { 0xfc165560, SUC_lexicon_tags_7 },
    { 0xb176555e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe0765563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f65566, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f6556a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b16556c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84965570, SUC_lexicon_tags_11 },
    { 0x5365571, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0765578, SUC_lexicon_tags_25 },
    { 0xfc765578, SUC_lexicon_tags_15 },
    { 0x57765578, SUC_lexicon_tags_33 },
    { 0xcc76557b, SUC_lexicon_tags_37 },
    { 0xdb6557c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1936557e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b65585, SUC_lexicon_tags_7 },
    { 0xbb365586, SUC_lexicon_tags_87 },
    { 0xd5b65586, SUC_lexicon_tags_49 },
    { 0x3bf65586, SUC_lexicon_tags_7 },
    { 0x61d65589, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9cf6558b, SUC_lexicon_tags_11 },
    { 0x2c16558c, SUC_lexicon_tags_25 },
    { 0x55d6558b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x32b6558f, SUC_lexicon_tags_15 },
    { 0x65565590, SUC_lexicon_tags_35 },
    { 0x51d65591, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacd65593, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9a365595, SUC_lexicon_tags_34 },
    { 0x31b65596, SUC_lexicon_tags_11 },
    { 0xa2165596, SUC_lexicon_tags_1 },
    { 0x62965598, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8696559c, SUC_lexicon_tags_7 },
    { 0x8296559d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11655a3, SUC_lexicon_tags_60 },
    { 0xe17655a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1655a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53f655ab, SUC_lexicon_tags_37 },
    { 0x53d655ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f655b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87d655b2, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b655b5, SUC_lexicon_tags_37 },
    { 0x759655b6, SUC_lexicon_tags_38 },
    { 0xaad655b5, SUC_lexicon_tags_15 },
    { 0x6d5655b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7d3655ba, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73655bd, SUC_lexicon_tags_37 },
    { 0xde1655be, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73655c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47655cd, SUC_lexicon_tags_8 },
    { 0xd11655cd, SUC_lexicon_tags_27 },
    { 0x5d1655cf, SUC_lexicon_tags_23 },
    { 0xd29655ce, SUC_lexicon_tags_17 },
    { 0x649655d1, SUC_lexicon_tags_25 },
    { 0xe63655cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7655d8, SUC_lexicon_tags_107 },
    { 0xbe3655d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x46f655db, SUC_lexicon_tags_34 },
    { 0xdd7655dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5655df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3655e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1655e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb13655ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18f655ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a7655ee, SUC_lexicon_tags_15 },
    { 0x803655ef, SUC_lexicon_tags_25 },
    { 0x97d655f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b655f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab655f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef655fa, SUC_lexicon_tags_60 },
    { 0xc8f655fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1655ff, SUC_lexicon_tags_66 },
    { 0x8bd65600, SUC_lexicon_tags_66 },
    { 0x88f65601, SUC_lexicon_tags_15 },
    { 0x63365601, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd365608, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b6560b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1dd6560d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c165610, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1165612, SUC_lexicon_tags_87 },
    { 0xa9165613, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a365616, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae36561a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1b6561c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf56561f, SUC_lexicon_tags_6 },
    { 0x9516561f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71565625, SUC_lexicon_tags_17 },
    { 0x5e365625, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f65629, SUC_lexicon_tags_25 },
    { 0x4f65629, SUC_lexicon_tags_25 },
    { 0x3b565629, SUC_lexicon_tags_25 },
    { 0xbd96562c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb36562e, SUC_lexicon_tags_23 },
    { 0xf556562e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x84365632, SUC_lexicon_tags_38 },
    { 0x24165632, SUC_lexicon_tags_23 },
    { 0xcc365633, SUC_lexicon_tags_1 },
    { 0xd2965635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4165637, SUC_lexicon_tags_11 },
    { 0x65165638, SUC_lexicon_tags_43 },
    { 0x84b65639, SUC_lexicon_tags_7 },
    { 0x82f65638, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f65641, SUC_lexicon_tags_65 },
    { 0x59365642, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b6564c, SUC_lexicon_tags_23 },
    { 0xee56564d, SUC_lexicon_tags_28 },
    { 0xdd36564e, SUC_lexicon_tags_23 },
    { 0x2536564f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52d65651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed365653, SUC_lexicon_tags_60 },
    { 0x3d165654, SUC_lexicon_tags_57 },
    { 0x2e365655, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35565657, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ab65659, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56365663, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa6d65665, SUC_lexicon_tags_8 },
    { 0x36365665, SUC_lexicon_tags_0 },
    { 0x44765667, SUC_lexicon_tags_68 },
    { 0x42165668, SUC_lexicon_tags_23 },
    { 0x48565669, SUC_lexicon_tags_16 },
    { 0x10565667, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6996566d, SUC_lexicon_tags_31 },
    { 0x4576566d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9b565670, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f65676, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6765678, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6676567d, SUC_lexicon_tags_45 },
    { 0x7496567e, SUC_lexicon_tags_2 },
    { 0x576567e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b765683, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xae765685, SUC_lexicon_tags_30 },
    { 0xe2365686, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1d765688, SUC_lexicon_tags_17 },
    { 0xa765688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff96568e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbed65690, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf965693, SUC_lexicon_tags_37 },
    { 0x26d65694, SUC_lexicon_tags_11 },
    { 0x4db65694, SUC_lexicon_tags_39 },
    { 0x66765694, SUC_lexicon_tags_27 },
    { 0x6c165694, SUC_lexicon_tags_1 },
    { 0xd2765698, SUC_lexicon_tags_33 },
    { 0xabd65693, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f6569c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd356569f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59d656a1, SUC_lexicon_tags_103 },
    { 0x10d656a1, SUC_lexicon_tags_30 },
    { 0xc3656a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x80b656a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x231656a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f656aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff656b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83656b5, SUC_lexicon_tags_1 },
    { 0xe91656b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe7f656b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5656bf, SUC_lexicon_tags_45 },
    { 0xdcb656c0, SUC_lexicon_tags_47 },
    { 0x59d656c1, SUC_lexicon_tags_45 },
    { 0xef9656c2, SUC_lexicon_tags_17 },
    { 0x501656c2, SUC_lexicon_tags_49 },
    { 0x155656c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af656c8, SUC_lexicon_tags_26 },
    { 0xef1656c9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9656cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6b656ce, SUC_lexicon_tags_0 },
    { 0x725656ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1656d2, SUC_lexicon_tags_38 },
    { 0xfdb656d3, SUC_lexicon_tags_12 },
    { 0xf91656d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc13656d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63656d9, SUC_lexicon_tags_15 },
    { 0x125656d9, SUC_lexicon_tags_37 },
    { 0xb41656db, SUC_lexicon_tags_31 },
    { 0x17656dc, SUC_lexicon_tags_4 },
    { 0x63f656db, SUC_lexicon_tags_1 },
    { 0xe6b656dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb656e1, SUC_lexicon_tags_37 },
    { 0x707656e2, SUC_lexicon_tags_7 },
    { 0x1f656e3, SUC_lexicon_tags_31 },
    { 0x849656e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd21656e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc15656e8, SUC_lexicon_tags_178 },
    { 0xffd656e9, SUC_lexicon_tags_17 },
    { 0xd99656e8, SUC_lexicon_tags_11 },
    { 0x9e5656e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x577656ee, SUC_lexicon_tags_31 },
    { 0xb95656ee, SUC_lexicon_tags_37 },
    { 0xa21656ee, SUC_lexicon_tags_25 },
    { 0x8ab656f1, SUC_lexicon_tags_60 },
    { 0xd33656f0, SUC_lexicon_tags_234 },
    { 0xfe5656f3, SUC_lexicon_tags_60 },
    { 0x60f656f3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xe33656f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cb656f8, SUC_lexicon_tags_26 },
    { 0xa3f656f8, SUC_lexicon_tags_138 },
    { 0xee3656f9, SUC_lexicon_tags_48 },
    { 0xa9b656fb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x7bf656fd, SUC_lexicon_tags_27 },
    { 0xaed656fe, SUC_lexicon_tags_53 },
    { 0xde5656ff, SUC_lexicon_tags_17 },
    { 0x9565700, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52365702, SUC_lexicon_tags_9 },
    { 0xfd165702, SUC_lexicon_tags_25 },
    { 0x1f365704, SUC_lexicon_tags_12 },
    { 0x2c965705, SUC_lexicon_tags_17 },
    { 0xc965705, SUC_lexicon_tags_56 },
    { 0xecd65707, SUC_lexicon_tags_8 },
    { 0xa4765708, SUC_lexicon_tags_25 },
    { 0x6f565709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9396570d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d65710, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff65713, SUC_lexicon_tags_43 },
    { 0x48965713, SUC_lexicon_tags_11 },
    { 0xa6965715, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xae965717, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96571e, SUC_lexicon_tags_29 },
    { 0xf216571f, SUC_lexicon_tags_1 },
    { 0xf1f6571f, SUC_lexicon_tags_1 },
    { 0x72d6571e, SUC_lexicon_tags_47 },
    { 0xdbd65722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40565727, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7936572d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6d6572f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47165731, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6565734, SUC_lexicon_tags_32 },
    { 0x11365735, SUC_lexicon_tags_11 },
    { 0xd0965735, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb6573b, SUC_lexicon_tags_25 },
    { 0x2416573c, SUC_lexicon_tags_15 },
    { 0x8816573d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1565740, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97165742, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8f65744, SUC_lexicon_tags_38 },
    { 0xc565744, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e365747, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50d65749, SUC_lexicon_tags_1 },
    { 0xf6365749, SUC_lexicon_tags_6 },
    { 0x216574b, SUC_lexicon_tags_48 },
    { 0xa4565749, SUC_lexicon_tags_21 },
    { 0x9696574b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15d6574f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12565754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2f65756, SUC_lexicon_tags_87 },
    { 0x5b365757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b6575a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7636575f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f565763, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x30765766, SUC_lexicon_tags_7 },
    { 0x67565766, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c96576d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc165774, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1416577d, SUC_lexicon_tags_17 },
    { 0x5a16577e, SUC_lexicon_tags_2 },
    { 0x44b6577f, SUC_lexicon_tags_46 },
    { 0x33f65780, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8965782, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b65786, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b65789, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f65790, SUC_lexicon_tags_33 },
    { 0xf2765791, SUC_lexicon_tags_66 },
    { 0x86365790, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f65795, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x41365797, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e56579a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83657a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b657a5, SUC_lexicon_tags_47 },
    { 0x8c1657a6, SUC_lexicon_tags_12 },
    { 0x13d657a7, SUC_lexicon_tags_26 },
    { 0x471657a7, SUC_lexicon_tags_17 },
    { 0x7d5657a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d657ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40f657ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f657b1, SUC_lexicon_tags_66 },
    { 0x967657b2, SUC_lexicon_tags_0 },
    { 0xb07657b3, SUC_lexicon_tags_29 },
    { 0xff3657b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51657b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d657ba, SUC_lexicon_tags_2 },
    { 0xdcd657ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f657c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb6f657c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x669657cb, SUC_lexicon_tags_23 },
    { 0x9e3657cc, SUC_lexicon_tags_1 },
    { 0x2d7657cd, SUC_lexicon_tags_63 },
    { 0x7d5657cc, SUC_lexicon_tags_27 },
    { 0x9c9657ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39f657d1, SUC_lexicon_tags_12 },
    { 0x125657d2, SUC_lexicon_tags_14 },
    { 0x431657d3, SUC_lexicon_tags_84 },
    { 0x6e5657d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2ef657d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a5657d8, SUC_lexicon_tags_27 },
    { 0xa1657d9, SUC_lexicon_tags_37 },
    { 0x383657d8, SUC_lexicon_tags_14 },
    { 0x831657db, SUC_lexicon_tags_11 },
    { 0xbe9657dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dd657de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x617657e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x237657e3, SUC_lexicon_tags_2 },
    { 0x431657e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93657e8, SUC_lexicon_tags_37 },
    { 0xde3657e9, SUC_lexicon_tags_3 },
    { 0x55d657e9, SUC_lexicon_tags_15 },
    { 0x743657ea, SUC_lexicon_tags_47 },
    { 0xe3d657ec, SUC_lexicon_tags_25 },
    { 0x57657ec, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851657f1, SUC_lexicon_tags_15 },
    { 0x2b7657f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd63657f4, SUC_lexicon_tags_11 },
    { 0x45f657f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeaf657f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f657fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b657fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc165801, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9f65803, SUC_lexicon_tags_37 },
    { 0x70565803, SUC_lexicon_tags_15 },
    { 0x39165805, SUC_lexicon_tags_13 },
    { 0x5ef65803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996580a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9636580e, SUC_lexicon_tags_3 },
    { 0x6336580f, SUC_lexicon_tags_14 },
    { 0xc576580f, SUC_lexicon_tags_8 },
    { 0xa816580f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b65819, SUC_lexicon_tags_57 },
    { 0xfb36581a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5896582d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb65830, SUC_lexicon_tags_15 },
    { 0x18365831, SUC_lexicon_tags_15 },
    { 0x29f65832, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3165834, SUC_lexicon_tags_282 },
    { 0x52f65835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90365837, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1596583f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b565841, SUC_lexicon_tags_60 },
    { 0xf6565842, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd65848, SUC_lexicon_tags_0 },
    { 0x36165849, SUC_lexicon_tags_19 },
    { 0xde56584a, SUC_lexicon_tags_2 },
    { 0xb7f6584b, SUC_lexicon_tags_30 },
    { 0xb4b6584a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d65854, SUC_lexicon_tags_40 },
    { 0x5f965855, SUC_lexicon_tags_7 },
    { 0xc8b65855, SUC_lexicon_tags_27 },
    { 0x82f65857, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6396585a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2f56585c, SUC_lexicon_tags_16 },
    { 0x8ed6585c, SUC_lexicon_tags_48 },
    { 0xe216585e, SUC_lexicon_tags_49 },
    { 0x6cb6585e, SUC_lexicon_tags_12 },
    { 0x456585e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58765864, SUC_lexicon_tags_15 },
    { 0xd0365864, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd076586a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7036586d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4765873, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e965876, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56587b, SUC_lexicon_tags_25 },
    { 0xdd96587c, SUC_lexicon_tags_1 },
    { 0x7996587b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc556587f, SUC_lexicon_tags_0 },
    { 0x6c565880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f65883, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd65887, SUC_lexicon_tags_11 },
    { 0xfc565888, SUC_lexicon_tags_10 },
    { 0xa0565889, SUC_lexicon_tags_0 },
    { 0xea56588a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74b6588c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5fd6588e, SUC_lexicon_tags_19 },
    { 0xbc96588f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2165892, SUC_lexicon_tags_10 },
    { 0x4fd65893, SUC_lexicon_tags_27 },
    { 0x4a165894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5836589a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb6589d, SUC_lexicon_tags_7 },
    { 0x4d36589d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1658a3, SUC_lexicon_tags_57 },
    { 0x48f658a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb658a7, SUC_lexicon_tags_23 },
    { 0x2f5658a7, SUC_lexicon_tags_1 },
    { 0x5e9658a9, SUC_lexicon_tags_49 },
    { 0x6f5658aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451658ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe69658b0, SUC_lexicon_tags_37 },
    { 0x26f658b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff658b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5658b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345658bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b658c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1658c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d5658c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x325658c9, SUC_lexicon_tags_3 },
    { 0xd0b658ca, SUC_lexicon_tags_37 },
    { 0xc79658cb, SUC_lexicon_tags_2 },
    { 0x35f658ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb99658ce, SUC_lexicon_tags_376 },
    { 0x43b658ce, SUC_lexicon_tags_49 },
    { 0xa07658d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223658d4, SUC_lexicon_tags_23 },
    { 0xc67658d5, SUC_lexicon_tags_2 },
    { 0xa05658d6, SUC_lexicon_tags_11 },
    { 0x2e5658d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb658df, SUC_lexicon_tags_48 },
    { 0x88b658df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc3658e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf37658e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5658e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x499658ea, SUC_lexicon_tags_63 },
    { 0x909658eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d658ef, SUC_lexicon_tags_7 },
    { 0x5f7658f0, SUC_lexicon_tags_6 },
    { 0xdd1658f1, SUC_lexicon_tags_39 },
    { 0xe79658f0, SUC_lexicon_tags_11 },
    { 0x38b658f3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d658f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733658fd, SUC_lexicon_tags_26 },
    { 0x111658fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2365900, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b65905, SUC_lexicon_tags_40 },
    { 0xf9365905, SUC_lexicon_tags_14 },
    { 0x40f65906, SUC_lexicon_tags_38 },
    { 0xba765907, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf56590d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a565916, SUC_lexicon_tags_30 },
    { 0x3f765916, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcd565919, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5716591b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb65921, SUC_lexicon_tags_25 },
    { 0x95b65921, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xdfb65924, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c965929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b6592e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf65932, SUC_lexicon_tags_25 },
    { 0x8b365932, SUC_lexicon_tags_14 },
    { 0xce365933, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16593c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26565941, SUC_lexicon_tags_7 },
    { 0x1f565942, SUC_lexicon_tags_12 },
    { 0xee965943, SUC_lexicon_tags_29 },
    { 0xfe165944, SUC_lexicon_tags_60 },
    { 0x65f65942, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadf65947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a365949, SUC_lexicon_tags_19 },
    { 0xc2165949, SUC_lexicon_tags_33 },
    { 0x5ed6594b, SUC_lexicon_tags_31 },
    { 0xa2d65949, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1bb6594e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec765952, SUC_lexicon_tags_0 },
    { 0x6c765953, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xde765955, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34f65957, SUC_lexicon_tags_11 },
    { 0x1b565958, SUC_lexicon_tags_65 },
    { 0xdb565957, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x136595d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xed96595f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb65962, SUC_lexicon_tags_38 },
    { 0x9a165962, SUC_lexicon_tags_15 },
    { 0xd4b65964, SUC_lexicon_tags_37 },
    { 0xd3965965, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b65969, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b65971, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4565974, SUC_lexicon_tags_27 },
    { 0xedb65974, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x32765977, SUC_lexicon_tags_23 },
    { 0x19b65978, SUC_lexicon_tags_37 },
    { 0x86965978, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6597f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d65984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49f65986, SUC_lexicon_tags_46 },
    { 0x48965986, SUC_lexicon_tags_25 },
    { 0x53765988, SUC_lexicon_tags_47 },
    { 0xba965989, SUC_lexicon_tags_0 },
    { 0x3c965988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef6598d, SUC_lexicon_tags_43 },
    { 0x13f6598d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb565991, SUC_lexicon_tags_91 },
    { 0xb8765992, SUC_lexicon_tags_58 },
    { 0x36965992, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2b65995, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1165999, SUC_lexicon_tags_69 },
    { 0xce365999, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8456599d, SUC_lexicon_tags_37 },
    { 0xe136599d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1659a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1ab659a7, SUC_lexicon_tags_21 },
    { 0xbd1659a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83659ac, SUC_lexicon_tags_12 },
    { 0x667659ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3659b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977659b5, SUC_lexicon_tags_14 },
    { 0xf11659b5, SUC_lexicon_tags_11 },
    { 0x669659b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813659bb, SUC_lexicon_tags_17 },
    { 0xbb3659bb, SUC_lexicon_tags_17 },
    { 0xf51659bd, SUC_lexicon_tags_23 },
    { 0x73659be, SUC_lexicon_tags_37 },
    { 0xb31659bf, SUC_lexicon_tags_1 },
    { 0x7a1659c0, SUC_lexicon_tags_57 },
    { 0xaef659c0, SUC_lexicon_tags_33 },
    { 0xeff659c1, SUC_lexicon_tags_11 },
    { 0x729659bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d659c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69659cc, SUC_lexicon_tags_57 },
    { 0x24d659cd, SUC_lexicon_tags_33 },
    { 0xa33659ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8c5659d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79659d3, SUC_lexicon_tags_15 },
    { 0x83659d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09659d9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235659dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5659e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x68d659e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x229659e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x639659e7, SUC_lexicon_tags_4 },
    { 0xc83659e8, SUC_lexicon_tags_17 },
    { 0xc0f659e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23659ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb659f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x179659f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f659f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1565a01, SUC_lexicon_tags_33 },
    { 0xd5b65a02, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf165a05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfab65a07, SUC_lexicon_tags_85 },
    { 0x8df65a08, SUC_lexicon_tags_47 },
    { 0x12765a09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4bd65a0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38965a0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90d65a11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c365a13, SUC_lexicon_tags_58 },
    { 0xded65a14, SUC_lexicon_tags_17 },
    { 0xac565a15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b165a19, SUC_lexicon_tags_7 },
    { 0x7b565a1a, SUC_lexicon_tags_47 },
    { 0xc0765a1b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef65a1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3965a29, SUC_lexicon_tags_15 },
    { 0x9dd65a2a, SUC_lexicon_tags_2 },
    { 0x3f165a2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56565a2d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6165a31, SUC_lexicon_tags_11 },
    { 0xd7565a31, SUC_lexicon_tags_25 },
    { 0x56d65a33, SUC_lexicon_tags_11 },
    { 0x83765a34, SUC_lexicon_tags_29 },
    { 0x93365a31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x12165a38, SUC_lexicon_tags_29 },
    { 0x1eb65a38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8565a3b, SUC_lexicon_tags_45 },
    { 0xb4b65a3b, SUC_lexicon_tags_7 },
    { 0x6e765a3d, SUC_lexicon_tags_113 },
    { 0x19165a3b, SUC_lexicon_tags_37 },
    { 0x87d65a3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf365a46, SUC_lexicon_tags_28 },
    { 0x83765a47, SUC_lexicon_tags_37 },
    { 0xecb65a48, SUC_lexicon_tags_39 },
    { 0x69165a49, SUC_lexicon_tags_12 },
    { 0x7e965a4a, SUC_lexicon_tags_14 },
    { 0xb0b65a4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77b65a4d, SUC_lexicon_tags_25 },
    { 0xaf365a4e, SUC_lexicon_tags_62 },
    { 0xfb365a4e, SUC_lexicon_tags_39 },
    { 0xd5965a50, SUC_lexicon_tags_11 },
    { 0x2f965a51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f965a53, SUC_lexicon_tags_46 },
    { 0x17365a54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc1f65a56, SUC_lexicon_tags_7 },
    { 0x1cd65a57, SUC_lexicon_tags_141 },
    { 0x28565a58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb65a5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf365a5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f765a64, SUC_lexicon_tags_29 },
    { 0x8c965a65, SUC_lexicon_tags_33 },
    { 0xead65a65, SUC_lexicon_tags_11 },
    { 0x93365a67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x86565a6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab65a6e, SUC_lexicon_tags_17 },
    { 0x1e765a6f, SUC_lexicon_tags_11 },
    { 0x72b65a6f, SUC_lexicon_tags_35 },
    { 0x69765a71, SUC_lexicon_tags_2 },
    { 0x89d65a72, SUC_lexicon_tags_1 },
    { 0x12765a73, SUC_lexicon_tags_70 },
    { 0x58f65a74, SUC_lexicon_tags_47 },
    { 0xf3b65a73, SUC_lexicon_tags_37 },
    { 0x6f65a76, SUC_lexicon_tags_37 },
    { 0x4df65a77, SUC_lexicon_tags_3 },
    { 0xdff65a78, SUC_lexicon_tags_15 },
    { 0x6965a76, SUC_lexicon_tags_38 },
    { 0x6df65a7a, SUC_lexicon_tags_42 },
    { 0xcdd65a7b, SUC_lexicon_tags_25 },
    { 0x92565a78, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff65a7f, SUC_lexicon_tags_459 },
    { 0, NULL },
    { 0xef565a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56b65a83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdad65a85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7565a88, SUC_lexicon_tags_45 },
    { 0xb565a89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x10165a8c, SUC_lexicon_tags_23 },
    { 0x3d565a8d, SUC_lexicon_tags_39 },
    { 0x9e565a8d, SUC_lexicon_tags_60 },
    { 0xc4765a8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7365a92, SUC_lexicon_tags_23 },
    { 0x33965a93, SUC_lexicon_tags_17 },
    { 0xc6f65a93, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3965a96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d65a9b, SUC_lexicon_tags_18 },
    { 0x57165a9c, SUC_lexicon_tags_27 },
    { 0x3165a9b, SUC_lexicon_tags_7 },
    { 0x43765a9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66765aa4, SUC_lexicon_tags_62 },
    { 0xafb65aa5, SUC_lexicon_tags_7 },
    { 0x38565aa4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d65aab, SUC_lexicon_tags_26 },
    { 0xb0365aab, SUC_lexicon_tags_17 },
    { 0xdb365aab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e165ab0, SUC_lexicon_tags_27 },
    { 0x2eb65ab0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d65ab5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2365ac0, SUC_lexicon_tags_35 },
    { 0xb0f65ac1, SUC_lexicon_tags_11 },
    { 0x9fb65ac0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e765ac4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0565ac6, SUC_lexicon_tags_15 },
    { 0x7b565ac6, SUC_lexicon_tags_0 },
    { 0x6365ac7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa765acc, SUC_lexicon_tags_65 },
    { 0xd2165acd, SUC_lexicon_tags_43 },
    { 0x24b65ace, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x25165ad1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb165ad5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37165ada, SUC_lexicon_tags_17 },
    { 0x86165adb, SUC_lexicon_tags_33 },
    { 0xd5b65adc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34365ae2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f65ae5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97365aea, SUC_lexicon_tags_17 },
    { 0x2fd65aeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15565aed, SUC_lexicon_tags_23 },
    { 0xe7165aed, SUC_lexicon_tags_11 },
    { 0xae165aef, SUC_lexicon_tags_15 },
    { 0x10d65aef, SUC_lexicon_tags_37 },
    { 0x88565aef, SUC_lexicon_tags_15 },
    { 0x5cb65af2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21765af6, SUC_lexicon_tags_33 },
    { 0xcab65af6, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd65afa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d65afd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28565b01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67965b09, SUC_lexicon_tags_7 },
    { 0x51765b0a, SUC_lexicon_tags_6 },
    { 0xa9965b0a, SUC_lexicon_tags_38 },
    { 0x33765b0c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x91b65b0e, SUC_lexicon_tags_60 },
    { 0xf9f65b0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12165b14, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1365b18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23965b1d, SUC_lexicon_tags_2 },
    { 0x85165b1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16565b22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf65b26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53365b2e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf65b32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40f65b34, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b365b38, SUC_lexicon_tags_32 },
    { 0xf2165b39, SUC_lexicon_tags_37 },
    { 0xa9b65b39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9765b3c, SUC_lexicon_tags_25 },
    { 0x66965b3c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76365b41, SUC_lexicon_tags_50 },
    { 0xb1565b42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9965b44, SUC_lexicon_tags_0 },
    { 0x6a165b45, SUC_lexicon_tags_12 },
    { 0xaad65b46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d65b49, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd65b53, SUC_lexicon_tags_9 },
    { 0x8dd65b54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93965b58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56365b5c, SUC_lexicon_tags_25 },
    { 0x13165b5d, SUC_lexicon_tags_25 },
    { 0xb565b5e, SUC_lexicon_tags_43 },
    { 0xf9b65b5d, SUC_lexicon_tags_17 },
    { 0xb8965b5c, SUC_lexicon_tags_15 },
    { 0xbc765b5f, SUC_lexicon_tags_8 },
    { 0xc365b5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38765b64, SUC_lexicon_tags_68 },
    { 0xbcb65b64, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7c365b67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x51965b6a, SUC_lexicon_tags_11 },
    { 0x66f65b6b, SUC_lexicon_tags_37 },
    { 0xe6b65b6c, SUC_lexicon_tags_6 },
    { 0xcf65b6a, SUC_lexicon_tags_32 },
    { 0xfeb65b6e, SUC_lexicon_tags_29 },
    { 0xa7565b6c, SUC_lexicon_tags_33 },
    { 0xa2165b70, SUC_lexicon_tags_8 },
    { 0x13f65b70, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x7365b74, SUC_lexicon_tags_2 },
    { 0xa7765b74, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d565b78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f165b7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb65b7d, SUC_lexicon_tags_23 },
    { 0xfc565b7e, SUC_lexicon_tags_12 },
    { 0xb6165b7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d965b81, SUC_lexicon_tags_9 },
    { 0x87f65b81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d765b84, SUC_lexicon_tags_1 },
    { 0x3e165b85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d65b8e, SUC_lexicon_tags_25 },
    { 0xb3965b8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0765b95, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xfff65b97, SUC_lexicon_tags_15 },
    { 0x10b65b98, SUC_lexicon_tags_15 },
    { 0xa9f65b99, SUC_lexicon_tags_26 },
    { 0x1d365b9a, SUC_lexicon_tags_60 },
    { 0x95b65b9b, SUC_lexicon_tags_32 },
    { 0x32365b97, SUC_lexicon_tags_37 },
    { 0x1a165b9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b565ba0, SUC_lexicon_tags_30 },
    { 0x26165ba0, SUC_lexicon_tags_30 },
    { 0x5ed65ba1, SUC_lexicon_tags_24 },
    { 0x23f65ba3, SUC_lexicon_tags_43 },
    { 0xb6d65ba3, SUC_lexicon_tags_2 },
    { 0x1565ba4, SUC_lexicon_tags_17 },
    { 0x2bb65ba3, SUC_lexicon_tags_29 },
    { 0xf4965ba7, SUC_lexicon_tags_11 },
    { 0xddf65ba8, SUC_lexicon_tags_11 },
    { 0x77f65ba6, SUC_lexicon_tags_27 },
    { 0xb565baa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81d65bac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b65baf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d65bb5, SUC_lexicon_tags_15 },
    { 0x2a365bb5, SUC_lexicon_tags_38 },
    { 0x3c365bb5, SUC_lexicon_tags_37 },
    { 0x48565bb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15365bbe, SUC_lexicon_tags_9 },
    { 0x62365bbf, SUC_lexicon_tags_37 },
    { 0xe0f65bbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b65bc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d65bc7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d65bca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd65bcd, SUC_lexicon_tags_1 },
    { 0x67d65bce, SUC_lexicon_tags_61 },
    { 0xec765bcf, SUC_lexicon_tags_15 },
    { 0xff365bd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf365bd4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb65bd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47965bdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27565bdd, SUC_lexicon_tags_46 },
    { 0x3c565bdd, SUC_lexicon_tags_15 },
    { 0xf6365bdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af65be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23965bea, SUC_lexicon_tags_15 },
    { 0x4f765bea, SUC_lexicon_tags_7 },
    { 0x77165bec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b65bf2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34565bf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96365bfc, SUC_lexicon_tags_37 },
    { 0xb1d65bfd, SUC_lexicon_tags_25 },
    { 0x2f365bfc, SUC_lexicon_tags_11 },
    { 0x52565bff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f65c03, SUC_lexicon_tags_14 },
    { 0x5af65c04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a765c07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ef65c09, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd65c0d, SUC_lexicon_tags_25 },
    { 0xf5765c0e, SUC_lexicon_tags_4 },
    { 0x2cb65c0f, SUC_lexicon_tags_1 },
    { 0xdd165c10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4165c12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc765c15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3965c1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd565c23, SUC_lexicon_tags_14 },
    { 0x19d65c24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef65c27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x64d65c29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x66165c2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e965c31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb65c3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13765c42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d65c47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe4f65c49, SUC_lexicon_tags_56 },
    { 0xdab65c4a, SUC_lexicon_tags_69 },
    { 0x3c765c4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d65c52, SUC_lexicon_tags_38 },
    { 0x6165c53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b65c57, SUC_lexicon_tags_26 },
    { 0x71965c57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x79165c5a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x14965c5c, SUC_lexicon_tags_7 },
    { 0x9bd65c5d, SUC_lexicon_tags_25 },
    { 0xb1d65c5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85365c60, SUC_lexicon_tags_108 },
    { 0xbbb65c61, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf65c65, SUC_lexicon_tags_38 },
    { 0x24f65c66, SUC_lexicon_tags_17 },
    { 0xd4b65c67, SUC_lexicon_tags_63 },
    { 0xa9965c66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61165c79, SUC_lexicon_tags_37 },
    { 0x1d765c7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1165c7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45365c80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf965c84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc765c86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85b65c88, SUC_lexicon_tags_13 },
    { 0x68d65c89, SUC_lexicon_tags_47 },
    { 0x4cf65c89, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b65c8e, SUC_lexicon_tags_2 },
    { 0x82965c8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51565c97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4d65c99, SUC_lexicon_tags_88 },
    { 0xfa765c9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f65c9e, SUC_lexicon_tags_60 },
    { 0x22d65c9e, SUC_lexicon_tags_25 },
    { 0x66965ca0, SUC_lexicon_tags_11 },
    { 0xc6765ca1, SUC_lexicon_tags_49 },
    { 0xc3765ca2, SUC_lexicon_tags_49 },
    { 0x55b65c9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63565ca7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x78165ca9, SUC_lexicon_tags_37 },
    { 0x64565caa, SUC_lexicon_tags_0 },
    { 0xe7965cab, SUC_lexicon_tags_17 },
    { 0xd9165cac, SUC_lexicon_tags_1 },
    { 0x50d65cad, SUC_lexicon_tags_15 },
    { 0xffb65cae, SUC_lexicon_tags_37 },
    { 0x39165cae, SUC_lexicon_tags_1 },
    { 0xbd765cae, SUC_lexicon_tags_7 },
    { 0xe3165cae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43365cb5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d65cb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2965cba, SUC_lexicon_tags_2 },
    { 0x5d365cbb, SUC_lexicon_tags_17 },
    { 0xcef65cbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3965cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af65cc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27165cc8, SUC_lexicon_tags_87 },
    { 0xcd365cc9, SUC_lexicon_tags_12 },
    { 0x48765cca, SUC_lexicon_tags_11 },
    { 0x63765ccb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a365cce, SUC_lexicon_tags_17 },
    { 0xaf765ccf, SUC_lexicon_tags_37 },
    { 0xa8365ccf, SUC_lexicon_tags_0 },
    { 0x6c765cd1, SUC_lexicon_tags_7 },
    { 0xc2965cd2, SUC_lexicon_tags_7 },
    { 0xcdf65cd2, SUC_lexicon_tags_17 },
    { 0x14b65cd1, SUC_lexicon_tags_15 },
    { 0x35f65cd5, SUC_lexicon_tags_14 },
    { 0x31165cd6, SUC_lexicon_tags_37 },
    { 0x2bb65cd7, SUC_lexicon_tags_12 },
    { 0xfcd65cd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2965cdb, SUC_lexicon_tags_87 },
    { 0x2d165cdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3365ce3, SUC_lexicon_tags_15 },
    { 0x33d65ce4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55965cea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6965cee, SUC_lexicon_tags_23 },
    { 0xfb565cee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5fd65cf1, SUC_lexicon_tags_49 },
    { 0xb9365cf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b65cf5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a965cf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b65cfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa565cff, SUC_lexicon_tags_12 },
    { 0x1cb65d00, SUC_lexicon_tags_25 },
    { 0x2bd65d01, SUC_lexicon_tags_60 },
    { 0x6b765d02, SUC_lexicon_tags_38 },
    { 0x77765d03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x38965d05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5765d0b, SUC_lexicon_tags_31 },
    { 0x8d965d0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15565d0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf65d12, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f65d16, SUC_lexicon_tags_12 },
    { 0xe3b65d16, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d65d1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f65d21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d565d23, SUC_lexicon_tags_17 },
    { 0x48165d24, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f65d27, SUC_lexicon_tags_38 },
    { 0x8165d28, SUC_lexicon_tags_10 },
    { 0xdc365d29, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d65d30, SUC_lexicon_tags_10 },
    { 0x36f65d31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c165d33, SUC_lexicon_tags_15 },
    { 0x7cf65d33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df65d38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4765d3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b65d3e, SUC_lexicon_tags_25 },
    { 0x50765d3e, SUC_lexicon_tags_68 },
    { 0xfd65d40, SUC_lexicon_tags_17 },
    { 0x30765d3f, SUC_lexicon_tags_62 },
    { 0x5d65d40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8c365d44, SUC_lexicon_tags_43 },
    { 0xb7365d45, SUC_lexicon_tags_1 },
    { 0xf5965d45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71f65d48, SUC_lexicon_tags_11 },
    { 0x60f65d49, SUC_lexicon_tags_13 },
    { 0x69765d4a, SUC_lexicon_tags_26 },
    { 0x67565d48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19565d50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2165d53, SUC_lexicon_tags_12 },
    { 0x5d365d54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda565d57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92365d61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb165d65, SUC_lexicon_tags_14 },
    { 0x79565d66, SUC_lexicon_tags_17 },
    { 0x53b65d67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67365d6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf165d6f, SUC_lexicon_tags_25 },
    { 0xe9565d70, SUC_lexicon_tags_17 },
    { 0x8e765d6f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5a165d73, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a565d78, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8165d85, SUC_lexicon_tags_26 },
    { 0xdfb65d85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b365d89, SUC_lexicon_tags_17 },
    { 0xad65d8a, SUC_lexicon_tags_26 },
    { 0x2b765d8a, SUC_lexicon_tags_38 },
    { 0xb6365d8a, SUC_lexicon_tags_15 },
    { 0x26765d8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27f65d8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44165d93, SUC_lexicon_tags_9 },
    { 0xe5b65d94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x49b65d96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bb65d98, SUC_lexicon_tags_60 },
    { 0xd1565d98, SUC_lexicon_tags_15 },
    { 0xda365d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca965da1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbb65da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb365da5, SUC_lexicon_tags_19 },
    { 0xd9365da6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f65daa, SUC_lexicon_tags_2 },
    { 0x61365dab, SUC_lexicon_tags_1 },
    { 0xb5d65dac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc6165dae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f65db4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5565db6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f65db9, SUC_lexicon_tags_29 },
    { 0x34565dba, SUC_lexicon_tags_37 },
    { 0xbeb65dba, SUC_lexicon_tags_1 },
    { 0x29165dbb, SUC_lexicon_tags_12 },
    { 0x3e965dbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60d65dbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc365dc1, SUC_lexicon_tags_26 },
    { 0x16165dc2, SUC_lexicon_tags_11 },
    { 0x92d65dc3, SUC_lexicon_tags_27 },
    { 0x41365dc3, SUC_lexicon_tags_30 },
    { 0x6f565dc5, SUC_lexicon_tags_8 },
    { 0x5565dc6, SUC_lexicon_tags_17 },
    { 0xd8f65dc7, SUC_lexicon_tags_17 },
    { 0x96b65dc4, SUC_lexicon_tags_21 },
    { 0xe4b65dc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9e565dcb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ab65dcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf65dd1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7bf65dd3, SUC_lexicon_tags_23 },
    { 0x91d65dd4, SUC_lexicon_tags_37 },
    { 0xf4565dd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f65dd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec965ddf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d65de5, SUC_lexicon_tags_19 },
    { 0x60765de6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b65de9, SUC_lexicon_tags_17 },
    { 0xf2565de9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x48b65dec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59965df3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa765df6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2165dfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f765dff, SUC_lexicon_tags_7 },
    { 0x3a965dff, SUC_lexicon_tags_79 },
    { 0x56765e01, SUC_lexicon_tags_2 },
    { 0xa3365e01, SUC_lexicon_tags_37 },
    { 0x9cb65e03, SUC_lexicon_tags_25 },
    { 0x85d65e03, SUC_lexicon_tags_2 },
    { 0xce165e05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed65e07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b765e09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66965e0d, SUC_lexicon_tags_60 },
    { 0x73f65e0e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb65e13, SUC_lexicon_tags_26 },
    { 0x32565e14, SUC_lexicon_tags_25 },
    { 0xab565e15, SUC_lexicon_tags_17 },
    { 0x6b65e15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0965e1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x96365e1d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b65e20, SUC_lexicon_tags_7 },
    { 0x21565e21, SUC_lexicon_tags_10 },
    { 0x97765e22, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7565e27, SUC_lexicon_tags_37 },
    { 0xfdd65e27, SUC_lexicon_tags_38 },
    { 0x42565e29, SUC_lexicon_tags_2 },
    { 0x6e565e2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef65e2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c765e34, SUC_lexicon_tags_45 },
    { 0x1c165e34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf765e38, SUC_lexicon_tags_7 },
    { 0x64565e39, SUC_lexicon_tags_8 },
    { 0xb0965e3a, SUC_lexicon_tags_27 },
    { 0x53365e3b, SUC_lexicon_tags_11 },
    { 0x6c965e3c, SUC_lexicon_tags_9 },
    { 0x40765e3c, SUC_lexicon_tags_9 },
    { 0xe4d65e3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62765e40, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb365e44, SUC_lexicon_tags_162 },
    { 0xe7965e45, SUC_lexicon_tags_14 },
    { 0x39965e46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f65e49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9365e4d, SUC_lexicon_tags_49 },
    { 0x26965e4d, SUC_lexicon_tags_17 },
    { 0xde765e4f, SUC_lexicon_tags_37 },
    { 0x3d765e50, SUC_lexicon_tags_14 },
    { 0xe3165e50, SUC_lexicon_tags_14 },
    { 0x33965e52, SUC_lexicon_tags_19 },
    { 0x22765e51, SUC_lexicon_tags_519 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9765e5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf65e60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa4365e62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xba765e64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8765e6a, SUC_lexicon_tags_11 },
    { 0x7d565e6b, SUC_lexicon_tags_2 },
    { 0x17b65e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b65e70, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d65e73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66165e75, SUC_lexicon_tags_40 },
    { 0x7f965e75, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb365e7a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd65e7d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd65e84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0565e88, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c765e8b, SUC_lexicon_tags_21 },
    { 0x2c765e8c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb3365e8e, SUC_lexicon_tags_11 },
    { 0x2b765e8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55165e92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42565e94, SUC_lexicon_tags_28 },
    { 0x60b65e95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4965e97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b65e9a, SUC_lexicon_tags_25 },
    { 0x39565e9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165ea1, SUC_lexicon_tags_11 },
    { 0x64365ea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74365ea5, SUC_lexicon_tags_43 },
    { 0x52d65ea5, SUC_lexicon_tags_38 },
    { 0x94b65ea5, SUC_lexicon_tags_7 },
    { 0x32365ea7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5df65eaa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8565eae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab65eb1, SUC_lexicon_tags_45 },
    { 0x8165eb1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1a165eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea565ebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a165ec0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d65ec3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db65ec7, SUC_lexicon_tags_25 },
    { 0xdbd65ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc365eca, SUC_lexicon_tags_11 },
    { 0x45165ecb, SUC_lexicon_tags_52 },
    { 0xdbd65ecb, SUC_lexicon_tags_37 },
    { 0x44165eca, SUC_lexicon_tags_17 },
    { 0x28d65ecb, SUC_lexicon_tags_14 },
    { 0x94965eca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee965ed9, SUC_lexicon_tags_84 },
    { 0x48365ed9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc165edd, SUC_lexicon_tags_7 },
    { 0x87f65ede, SUC_lexicon_tags_2 },
    { 0x84d65edf, SUC_lexicon_tags_7 },
    { 0x3c965edf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36365ee3, SUC_lexicon_tags_7 },
    { 0xaa565ee3, SUC_lexicon_tags_2 },
    { 0x93765ee5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd65eea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f65eed, SUC_lexicon_tags_30 },
    { 0x2a965eee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb65ef2, SUC_lexicon_tags_61 },
    { 0x19b65ef3, SUC_lexicon_tags_47 },
    { 0x19b65ef4, SUC_lexicon_tags_7 },
    { 0x5c365ef5, SUC_lexicon_tags_0 },
    { 0x9fd65ef4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb65ef9, SUC_lexicon_tags_2 },
    { 0x92165efa, SUC_lexicon_tags_37 },
    { 0xd8765efa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d365f00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f65f06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d965f0e, SUC_lexicon_tags_43 },
    { 0xd6565f0f, SUC_lexicon_tags_92 },
    { 0x9565f10, SUC_lexicon_tags_1 },
    { 0x87365f10, SUC_lexicon_tags_7 },
    { 0x7c365f12, SUC_lexicon_tags_17 },
    { 0xbe565f13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x365f16, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf1165f18, SUC_lexicon_tags_2 },
    { 0x70765f19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7f65f1b, SUC_lexicon_tags_30 },
    { 0x55f65f1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f65f22, SUC_lexicon_tags_37 },
    { 0xf3765f23, SUC_lexicon_tags_37 },
    { 0xd4165f23, SUC_lexicon_tags_15 },
    { 0x58d65f25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b65f28, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xf8765f2a, SUC_lexicon_tags_48 },
    { 0x5eb65f2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf365f2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b65f35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d65f39, SUC_lexicon_tags_15 },
    { 0x87d65f39, SUC_lexicon_tags_3 },
    { 0x1db65f3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75165f3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87365f3f, SUC_lexicon_tags_33 },
    { 0xab165f40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2365f43, SUC_lexicon_tags_14 },
    { 0x65f65f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd65f4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9d65f4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41d65f4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a965f55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdb65f57, SUC_lexicon_tags_43 },
    { 0x6f965f58, SUC_lexicon_tags_7 },
    { 0x84b65f59, SUC_lexicon_tags_77 },
    { 0xe7d65f5a, SUC_lexicon_tags_38 },
    { 0xaf65f5b, SUC_lexicon_tags_46 },
    { 0xb2d65f5c, SUC_lexicon_tags_39 },
    { 0x59965f58, SUC_lexicon_tags_15 },
    { 0xf1365f5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1fd65f60, SUC_lexicon_tags_47 },
    { 0x4ef65f60, SUC_lexicon_tags_7 },
    { 0x43d65f62, SUC_lexicon_tags_11 },
    { 0x4c365f61, SUC_lexicon_tags_37 },
    { 0x43165f64, SUC_lexicon_tags_72 },
    { 0xb8765f65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f65f6b, SUC_lexicon_tags_12 },
    { 0x7ed65f6c, SUC_lexicon_tags_0 },
    { 0x70365f6d, SUC_lexicon_tags_11 },
    { 0x62165f6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f65f72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a565f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ef65f77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b65f7d, SUC_lexicon_tags_17 },
    { 0x4ab65f7e, SUC_lexicon_tags_17 },
    { 0x11565f7e, SUC_lexicon_tags_17 },
    { 0xe4165f80, SUC_lexicon_tags_19 },
    { 0x2565f81, SUC_lexicon_tags_12 },
    { 0x36765f82, SUC_lexicon_tags_7 },
    { 0x1b165f82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a565f87, SUC_lexicon_tags_17 },
    { 0xbed65f88, SUC_lexicon_tags_23 },
    { 0xd1165f88, SUC_lexicon_tags_11 },
    { 0x78565f8a, SUC_lexicon_tags_29 },
    { 0xa5165f8a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4165f8e, SUC_lexicon_tags_30 },
    { 0xc7d65f8e, SUC_lexicon_tags_8 },
    { 0x81965f8f, SUC_lexicon_tags_0 },
    { 0xc5965f91, SUC_lexicon_tags_15 },
    { 0x99365f92, SUC_lexicon_tags_19 },
    { 0xb4965f93, SUC_lexicon_tags_37 },
    { 0xf1765f90, SUC_lexicon_tags_37 },
    { 0x56b65f95, SUC_lexicon_tags_56 },
    { 0x74565f96, SUC_lexicon_tags_9 },
    { 0xb8965f97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd65f9b, SUC_lexicon_tags_2 },
    { 0xed165f9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21165fa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e365fa7, SUC_lexicon_tags_39 },
    { 0xb9565fa8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b565fad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x96765fb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32565fb2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f65fb7, SUC_lexicon_tags_15 },
    { 0x31965fb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51765fba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b65fc1, SUC_lexicon_tags_2 },
    { 0x5f965fc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d65fc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20565fcc, SUC_lexicon_tags_19 },
    { 0xe965fcc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf565fd0, SUC_lexicon_tags_26 },
    { 0xe0165fd0, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f65fd5, SUC_lexicon_tags_7 },
    { 0x94365fd6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f165fde, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b65fe1, SUC_lexicon_tags_16 },
    { 0xb565fe1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6965fe4, SUC_lexicon_tags_2 },
    { 0x25565fe5, SUC_lexicon_tags_57 },
    { 0x85165fe4, SUC_lexicon_tags_21 },
    { 0x41765fe7, SUC_lexicon_tags_38 },
    { 0x76765fe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37b65fea, SUC_lexicon_tags_19 },
    { 0x98565feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4365fef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x15d65ff1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb165ff4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x11765ff7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8365ffa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10566001, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc966004, SUC_lexicon_tags_37 },
    { 0x5ff66005, SUC_lexicon_tags_17 },
    { 0xc9766006, SUC_lexicon_tags_63 },
    { 0x25566006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea566009, SUC_lexicon_tags_12 },
    { 0xdfb66009, SUC_lexicon_tags_12 },
    { 0x3236600a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9976600f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b66012, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b366017, SUC_lexicon_tags_45 },
    { 0x72b66018, SUC_lexicon_tags_49 },
    { 0xaf366017, SUC_lexicon_tags_7 },
    { 0xb316601a, SUC_lexicon_tags_26 },
    { 0xe756601a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b6601e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9966021, SUC_lexicon_tags_30 },
    { 0xba566021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9966024, SUC_lexicon_tags_37 },
    { 0x45d66024, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7366027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb56602c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8156602e, SUC_lexicon_tags_11 },
    { 0x4736602e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f66033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x36166036, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd076603e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d966044, SUC_lexicon_tags_12 },
    { 0xf1b66044, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x92f66047, SUC_lexicon_tags_0 },
    { 0x82d66048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2436604c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f66050, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f766052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca166054, SUC_lexicon_tags_37 },
    { 0x74566055, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d66058, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb956605a, SUC_lexicon_tags_31 },
    { 0x6f36605a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92766060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9166062, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf66071, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2966078, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xff16607b, SUC_lexicon_tags_39 },
    { 0xaa76607b, SUC_lexicon_tags_17 },
    { 0xaaf6607b, SUC_lexicon_tags_32 },
    { 0xeb6607e, SUC_lexicon_tags_60 },
    { 0x3dd6607e, SUC_lexicon_tags_7 },
    { 0x16b66080, SUC_lexicon_tags_37 },
    { 0xe566081, SUC_lexicon_tags_12 },
    { 0x37766082, SUC_lexicon_tags_25 },
    { 0x57b66083, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa166087, SUC_lexicon_tags_11 },
    { 0x6f566088, SUC_lexicon_tags_38 },
    { 0xea766088, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26f6608b, SUC_lexicon_tags_17 },
    { 0x2816608c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9966090, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20166094, SUC_lexicon_tags_25 },
    { 0x75d66095, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3166099, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36d6609b, SUC_lexicon_tags_144 },
    { 0xb9b6609b, SUC_lexicon_tags_17 },
    { 0x2436609d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa696609f, SUC_lexicon_tags_35 },
    { 0x4596609f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab660a4, SUC_lexicon_tags_43 },
    { 0xecf660a5, SUC_lexicon_tags_72 },
    { 0xa5d660a5, SUC_lexicon_tags_12 },
    { 0x3fb660a6, SUC_lexicon_tags_28 },
    { 0xbf1660a4, SUC_lexicon_tags_4 },
    { 0xc97660a9, SUC_lexicon_tags_8 },
    { 0x18d660aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb660ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9660b2, SUC_lexicon_tags_29 },
    { 0xb7660b3, SUC_lexicon_tags_37 },
    { 0x979660b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb57660b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdb660b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09660bb, SUC_lexicon_tags_17 },
    { 0xd9b660bc, SUC_lexicon_tags_28 },
    { 0x7af660bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13b660bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01660c2, SUC_lexicon_tags_9 },
    { 0xbdb660c3, SUC_lexicon_tags_7 },
    { 0xb47660c4, SUC_lexicon_tags_49 },
    { 0x99b660c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc3d660c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1660cb, SUC_lexicon_tags_30 },
    { 0xf81660cc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937660d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd660d6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9660d9, SUC_lexicon_tags_57 },
    { 0xbc3660da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9660df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb660e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65660e7, SUC_lexicon_tags_7 },
    { 0xe2b660e8, SUC_lexicon_tags_11 },
    { 0x853660e9, SUC_lexicon_tags_27 },
    { 0xd7660ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7660ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d660f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611660f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeff660f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5f660fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70b660fd, SUC_lexicon_tags_87 },
    { 0x639660fd, SUC_lexicon_tags_25 },
    { 0x737660fd, SUC_lexicon_tags_0 },
    { 0x2f966100, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7a966102, SUC_lexicon_tags_0 },
    { 0xce766103, SUC_lexicon_tags_29 },
    { 0xde366103, SUC_lexicon_tags_17 },
    { 0xcfd66102, SUC_lexicon_tags_60 },
    { 0xe9166103, SUC_lexicon_tags_37 },
    { 0xea766105, SUC_lexicon_tags_11 },
    { 0xed366108, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b36610e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8166115, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86966118, SUC_lexicon_tags_3 },
    { 0x70566119, SUC_lexicon_tags_45 },
    { 0xaa16611a, SUC_lexicon_tags_26 },
    { 0x6d76611b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2756611e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x10d66120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a166123, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb66128, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d6612b, SUC_lexicon_tags_55 },
    { 0xeb96612c, SUC_lexicon_tags_11 },
    { 0x7a16612c, SUC_lexicon_tags_11 },
    { 0xdbb6612c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae966134, SUC_lexicon_tags_79 },
    { 0x5f566134, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb66138, SUC_lexicon_tags_69 },
    { 0xc9766139, SUC_lexicon_tags_81 },
    { 0x7eb66138, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd596613c, SUC_lexicon_tags_6 },
    { 0xff16613d, SUC_lexicon_tags_37 },
    { 0x9d76613d, SUC_lexicon_tags_4 },
    { 0xb5f6613e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b66145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16166147, SUC_lexicon_tags_7 },
    { 0x97166148, SUC_lexicon_tags_7 },
    { 0xdb766149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd166154, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8bd66156, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9f66158, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b36615b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7916615e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xed766161, SUC_lexicon_tags_17 },
    { 0xb0b66161, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf256616a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c366171, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84366177, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf6617b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaab6617d, SUC_lexicon_tags_8 },
    { 0xb896617e, SUC_lexicon_tags_11 },
    { 0x856617f, SUC_lexicon_tags_47 },
    { 0x6f166180, SUC_lexicon_tags_37 },
    { 0xbb166181, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af66184, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f966188, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb496618a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb8f6618c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4876618f, SUC_lexicon_tags_0 },
    { 0x4c76618f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9566192, SUC_lexicon_tags_9 },
    { 0x8fd66193, SUC_lexicon_tags_25 },
    { 0xa3766194, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b166196, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f6619b, SUC_lexicon_tags_37 },
    { 0x3316619b, SUC_lexicon_tags_12 },
    { 0xb96619b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf396619f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc29661a1, SUC_lexicon_tags_43 },
    { 0xc63661a2, SUC_lexicon_tags_43 },
    { 0x797661a3, SUC_lexicon_tags_615 },
    { 0xa45661a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45661a8, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x8cf661aa, SUC_lexicon_tags_39 },
    { 0xbc1661ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x50f661ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81661b4, SUC_lexicon_tags_12 },
    { 0x7a7661b5, SUC_lexicon_tags_25 },
    { 0xd79661b6, SUC_lexicon_tags_43 },
    { 0x65d661b7, SUC_lexicon_tags_25 },
    { 0x2dd661b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41b661ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cd661bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5661c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed661c7, SUC_lexicon_tags_30 },
    { 0xa31661c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x437661cb, SUC_lexicon_tags_114 },
    { 0x273661cc, SUC_lexicon_tags_65 },
    { 0x67b661cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb661d2, SUC_lexicon_tags_13 },
    { 0x12b661d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x747661d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4e5661d7, SUC_lexicon_tags_11 },
    { 0xc97661d8, SUC_lexicon_tags_55 },
    { 0x22f661d9, SUC_lexicon_tags_43 },
    { 0xefd661d8, SUC_lexicon_tags_27 },
    { 0xc19661d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed661df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfa3661e1, SUC_lexicon_tags_1 },
    { 0x297661e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955661e6, SUC_lexicon_tags_229 },
    { 0x833661e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf09661e9, SUC_lexicon_tags_37 },
    { 0x193661ea, SUC_lexicon_tags_62 },
    { 0xe2b661ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417661f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3661f6, SUC_lexicon_tags_1 },
    { 0x3c9661f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f661fd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6966202, SUC_lexicon_tags_47 },
    { 0x6c166202, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f966205, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b66208, SUC_lexicon_tags_7 },
    { 0x49766208, SUC_lexicon_tags_15 },
    { 0x8736620a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa96620c, SUC_lexicon_tags_26 },
    { 0xfa16620d, SUC_lexicon_tags_37 },
    { 0xf896620e, SUC_lexicon_tags_37 },
    { 0x2636620f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89366215, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf766219, SUC_lexicon_tags_26 },
    { 0x2ab6621a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4ab6621c, SUC_lexicon_tags_55 },
    { 0x9456621d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d6622a, SUC_lexicon_tags_29 },
    { 0x1956622b, SUC_lexicon_tags_43 },
    { 0x2476622c, SUC_lexicon_tags_12 },
    { 0x3f36622d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4966230, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca166232, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x76966234, SUC_lexicon_tags_17 },
    { 0xbcb66234, SUC_lexicon_tags_39 },
    { 0x2d966236, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db6623e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f566244, SUC_lexicon_tags_6 },
    { 0xdd66245, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f6624d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b76624f, SUC_lexicon_tags_100 },
    { 0x32b66250, SUC_lexicon_tags_27 },
    { 0x63d66250, SUC_lexicon_tags_26 },
    { 0x73166251, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1766254, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f6625a, SUC_lexicon_tags_17 },
    { 0x3556625b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee966261, SUC_lexicon_tags_7 },
    { 0x25766262, SUC_lexicon_tags_25 },
    { 0x45366263, SUC_lexicon_tags_13 },
    { 0x25966262, SUC_lexicon_tags_71 },
    { 0x68f66265, SUC_lexicon_tags_12 },
    { 0x15766266, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa616626e, SUC_lexicon_tags_23 },
    { 0xaed6626f, SUC_lexicon_tags_17 },
    { 0xc896626f, SUC_lexicon_tags_17 },
    { 0x6c966270, SUC_lexicon_tags_7 },
    { 0xb7b66272, SUC_lexicon_tags_15 },
    { 0x8cb66273, SUC_lexicon_tags_194 },
    { 0x5a366272, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78166279, SUC_lexicon_tags_55 },
    { 0x5b96627a, SUC_lexicon_tags_47 },
    { 0x9d36627b, SUC_lexicon_tags_49 },
    { 0x18d6627c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe56627f, SUC_lexicon_tags_7 },
    { 0xf9d6627f, SUC_lexicon_tags_17 },
    { 0x18566281, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xfdf66283, SUC_lexicon_tags_1 },
    { 0xd7966283, SUC_lexicon_tags_29 },
    { 0x4f566284, SUC_lexicon_tags_37 },
    { 0x8c966286, SUC_lexicon_tags_262 },
    { 0x64b66287, SUC_lexicon_tags_856 },
    { 0, NULL },
    { 0, NULL },
    { 0x7916628a, SUC_lexicon_tags_182 },
    { 0x60d6628b, SUC_lexicon_tags_127 },
    { 0x1ab6628b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb66291, SUC_lexicon_tags_12 },
    { 0xf4d66292, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c566297, SUC_lexicon_tags_2 },
    { 0x94f66298, SUC_lexicon_tags_66 },
    { 0xd2166297, SUC_lexicon_tags_19 },
    { 0x8fb66297, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb6629e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa71662a0, SUC_lexicon_tags_2 },
    { 0xddb662a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf662a5, SUC_lexicon_tags_8 },
    { 0x887662a5, SUC_lexicon_tags_45 },
    { 0x8ed662a5, SUC_lexicon_tags_7 },
    { 0x5a1662a7, SUC_lexicon_tags_15 },
    { 0x833662a7, SUC_lexicon_tags_15 },
    { 0x327662aa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x261662ad, SUC_lexicon_tags_7 },
    { 0x93d662ad, SUC_lexicon_tags_15 },
    { 0x123662ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b662b4, SUC_lexicon_tags_37 },
    { 0xbcf662b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79662ba, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b662c2, SUC_lexicon_tags_46 },
    { 0xc1f662c2, SUC_lexicon_tags_15 },
    { 0x533662c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf662c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcc5662c9, SUC_lexicon_tags_30 },
    { 0xb35662ca, SUC_lexicon_tags_2 },
    { 0x31b662cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1662d0, SUC_lexicon_tags_55 },
    { 0x339662d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8cd662d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b662d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc7662d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187662df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3662e9, SUC_lexicon_tags_2 },
    { 0xca1662e9, SUC_lexicon_tags_7 },
    { 0x415662e9, SUC_lexicon_tags_25 },
    { 0xd3662e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9662f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf79662f2, SUC_lexicon_tags_10 },
    { 0xb63662f2, SUC_lexicon_tags_51 },
    { 0x983662f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf55662f6, SUC_lexicon_tags_15 },
    { 0x76d662f7, SUC_lexicon_tags_43 },
    { 0xaa7662f7, SUC_lexicon_tags_65 },
    { 0x61b662f7, SUC_lexicon_tags_7 },
    { 0x1bb662fa, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b662fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f66305, SUC_lexicon_tags_25 },
    { 0xc6966305, SUC_lexicon_tags_107 },
    { 0x3bd66307, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf6630a, SUC_lexicon_tags_2 },
    { 0xacd6630b, SUC_lexicon_tags_11 },
    { 0x41d6630c, SUC_lexicon_tags_44 },
    { 0x3776630b, SUC_lexicon_tags_30 },
    { 0x5cf6630c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2bb66310, SUC_lexicon_tags_45 },
    { 0xa9966311, SUC_lexicon_tags_2 },
    { 0xd0d66311, SUC_lexicon_tags_2 },
    { 0x6e566313, SUC_lexicon_tags_1 },
    { 0xed566314, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61566319, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb56631c, SUC_lexicon_tags_9 },
    { 0xb2f6631c, SUC_lexicon_tags_8 },
    { 0x90f6631e, SUC_lexicon_tags_65 },
    { 0x9436631f, SUC_lexicon_tags_39 },
    { 0xe1d6631d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa166322, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x60566324, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b66327, SUC_lexicon_tags_49 },
    { 0xc9166328, SUC_lexicon_tags_31 },
    { 0x3bb66329, SUC_lexicon_tags_25 },
    { 0x5b36632a, SUC_lexicon_tags_45 },
    { 0xca56632b, SUC_lexicon_tags_12 },
    { 0x7316632c, SUC_lexicon_tags_1 },
    { 0x3316632b, SUC_lexicon_tags_17 },
    { 0xe8f6632c, SUC_lexicon_tags_1 },
    { 0x3ed6632d, SUC_lexicon_tags_31 },
    { 0x66b6632d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x50166333, SUC_lexicon_tags_2 },
    { 0x32566334, SUC_lexicon_tags_47 },
    { 0x3f566334, SUC_lexicon_tags_42 },
    { 0x94f66333, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x22966338, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70f6633a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc436633e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4566342, SUC_lexicon_tags_2 },
    { 0xba966342, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x73166346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40d66348, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd336634b, SUC_lexicon_tags_25 },
    { 0x83f6634b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d6634f, SUC_lexicon_tags_11 },
    { 0xd966350, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62166352, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb566356, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf6635e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d66362, SUC_lexicon_tags_23 },
    { 0x26b66363, SUC_lexicon_tags_33 },
    { 0xabb66362, SUC_lexicon_tags_56 },
    { 0x32d66365, SUC_lexicon_tags_406 },
    { 0x77566363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaaf66368, SUC_lexicon_tags_25 },
    { 0xd5f66369, SUC_lexicon_tags_49 },
    { 0xb366369, SUC_lexicon_tags_25 },
    { 0xaad6636a, SUC_lexicon_tags_47 },
    { 0xe76636c, SUC_lexicon_tags_26 },
    { 0xdcf6636c, SUC_lexicon_tags_17 },
    { 0xb576636d, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b66374, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdaf66376, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3f66378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed6637f, SUC_lexicon_tags_47 },
    { 0x5a566380, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f66383, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x51166385, SUC_lexicon_tags_28 },
    { 0x90166385, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xef366389, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f6638c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe016638e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xbb566390, SUC_lexicon_tags_11 },
    { 0x99966390, SUC_lexicon_tags_11 },
    { 0x66366392, SUC_lexicon_tags_14 },
    { 0x4e366390, SUC_lexicon_tags_224 },
    { 0x55566394, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff766398, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad6639f, SUC_lexicon_tags_17 },
    { 0x7796639f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05663a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93663aa, SUC_lexicon_tags_7 },
    { 0xc2f663aa, SUC_lexicon_tags_16 },
    { 0x6cd663aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74b663ae, SUC_lexicon_tags_23 },
    { 0x513663af, SUC_lexicon_tags_6 },
    { 0x483663af, SUC_lexicon_tags_49 },
    { 0x3cf663b1, SUC_lexicon_tags_7 },
    { 0xd39663b2, SUC_lexicon_tags_47 },
    { 0x8f663b3, SUC_lexicon_tags_17 },
    { 0xd19663b4, SUC_lexicon_tags_7 },
    { 0x861663b5, SUC_lexicon_tags_12 },
    { 0xae5663b5, SUC_lexicon_tags_49 },
    { 0x2a1663b5, SUC_lexicon_tags_27 },
    { 0x2cb663b8, SUC_lexicon_tags_7 },
    { 0x87f663b8, SUC_lexicon_tags_11 },
    { 0x7e7663ba, SUC_lexicon_tags_27 },
    { 0xf5d663b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7663be, SUC_lexicon_tags_4 },
    { 0xc51663bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d663c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21663ca, SUC_lexicon_tags_45 },
    { 0x315663cb, SUC_lexicon_tags_37 },
    { 0xbb9663cc, SUC_lexicon_tags_25 },
    { 0x2f3663cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf45663cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad663d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x693663d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x445663d7, SUC_lexicon_tags_12 },
    { 0x999663d8, SUC_lexicon_tags_28 },
    { 0x24b663d9, SUC_lexicon_tags_7 },
    { 0x443663da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59f663dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05663e0, SUC_lexicon_tags_23 },
    { 0x71d663e0, SUC_lexicon_tags_38 },
    { 0x881663e2, SUC_lexicon_tags_27 },
    { 0x551663e3, SUC_lexicon_tags_63 },
    { 0x2a9663e4, SUC_lexicon_tags_12 },
    { 0x85663e5, SUC_lexicon_tags_49 },
    { 0x571663e6, SUC_lexicon_tags_1 },
    { 0xfcb663e0, SUC_lexicon_tags_37 },
    { 0x705663e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fd663ea, SUC_lexicon_tags_15 },
    { 0xa05663ea, SUC_lexicon_tags_0 },
    { 0xe81663ec, SUC_lexicon_tags_2 },
    { 0xee5663ec, SUC_lexicon_tags_15 },
    { 0x929663ee, SUC_lexicon_tags_33 },
    { 0xeb9663ef, SUC_lexicon_tags_12 },
    { 0x91663ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d663f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9663f6, SUC_lexicon_tags_0 },
    { 0x6af663f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519663fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41566400, SUC_lexicon_tags_12 },
    { 0x9ff66401, SUC_lexicon_tags_12 },
    { 0xa8f66402, SUC_lexicon_tags_2 },
    { 0x31966403, SUC_lexicon_tags_2 },
    { 0xde366404, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db66407, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab6640b, SUC_lexicon_tags_12 },
    { 0xcf36640c, SUC_lexicon_tags_23 },
    { 0x6f6640d, SUC_lexicon_tags_1 },
    { 0xb856640c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd66410, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0366412, SUC_lexicon_tags_12 },
    { 0xdfb66413, SUC_lexicon_tags_43 },
    { 0x75b66413, SUC_lexicon_tags_13 },
    { 0x4a966415, SUC_lexicon_tags_37 },
    { 0x3c766415, SUC_lexicon_tags_7 },
    { 0x7b366412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39566419, SUC_lexicon_tags_12 },
    { 0xbc96641a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd856641d, SUC_lexicon_tags_36 },
    { 0xdd76641d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19766422, SUC_lexicon_tags_37 },
    { 0xf0d66423, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb2566425, SUC_lexicon_tags_39 },
    { 0x4e766426, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6366429, SUC_lexicon_tags_27 },
    { 0x7bb6642a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe16642f, SUC_lexicon_tags_19 },
    { 0x39166430, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c566432, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xff66435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c966438, SUC_lexicon_tags_48 },
    { 0x36d66438, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7316643c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1476643f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb566444, SUC_lexicon_tags_85 },
    { 0x4bb66445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b36644b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb66451, SUC_lexicon_tags_2 },
    { 0xa6366452, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22566457, SUC_lexicon_tags_0 },
    { 0xdc766458, SUC_lexicon_tags_7 },
    { 0x48366459, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x27f6645b, SUC_lexicon_tags_42 },
    { 0xd396645b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50966460, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94166465, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xef166468, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x1256646b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc696646d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76966471, SUC_lexicon_tags_2 },
    { 0xb2966472, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7cd66474, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d66478, SUC_lexicon_tags_96 },
    { 0x7166479, SUC_lexicon_tags_45 },
    { 0x6b566479, SUC_lexicon_tags_7 },
    { 0x1416647a, SUC_lexicon_tags_12 },
    { 0x58566479, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x50d6647e, SUC_lexicon_tags_34 },
    { 0xc2b6647f, SUC_lexicon_tags_10 },
    { 0xe5b6647f, SUC_lexicon_tags_26 },
    { 0xa4d66481, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4566489, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb6648d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6966492, SUC_lexicon_tags_30 },
    { 0xf8566493, SUC_lexicon_tags_37 },
    { 0xe5166493, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58f66496, SUC_lexicon_tags_21 },
    { 0xd2966497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6896649b, SUC_lexicon_tags_25 },
    { 0x9bf6649c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f664a0, SUC_lexicon_tags_485 },
    { 0x7d664a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31664a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f664aa, SUC_lexicon_tags_7 },
    { 0x79f664ab, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd664ae, SUC_lexicon_tags_46 },
    { 0x54d664ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3664b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d664b6, SUC_lexicon_tags_60 },
    { 0x74d664b7, SUC_lexicon_tags_3 },
    { 0xe1d664b8, SUC_lexicon_tags_17 },
    { 0xe1f664b6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b664bd, SUC_lexicon_tags_30 },
    { 0x3664be, SUC_lexicon_tags_37 },
    { 0xe55664bf, SUC_lexicon_tags_7 },
    { 0xb9b664c0, SUC_lexicon_tags_7 },
    { 0xbb7664c0, SUC_lexicon_tags_60 },
    { 0x125664c1, SUC_lexicon_tags_9 },
    { 0xcb5664c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b664c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f664ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x66b664cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9664cf, SUC_lexicon_tags_87 },
    { 0x8a3664cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f664d4, SUC_lexicon_tags_7 },
    { 0x459664d4, SUC_lexicon_tags_39 },
    { 0xaab664d6, SUC_lexicon_tags_29 },
    { 0x755664d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1664da, SUC_lexicon_tags_84 },
    { 0xbb5664db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef664de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d664e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b664e8, SUC_lexicon_tags_116 },
    { 0xf03664e9, SUC_lexicon_tags_11 },
    { 0x579664e8, SUC_lexicon_tags_37 },
    { 0xb8b664e8, SUC_lexicon_tags_29 },
    { 0x6bf664ec, SUC_lexicon_tags_2 },
    { 0x6a9664ed, SUC_lexicon_tags_29 },
    { 0x58b664ed, SUC_lexicon_tags_30 },
    { 0xb93664ec, SUC_lexicon_tags_37 },
    { 0x531664ea, SUC_lexicon_tags_37 },
    { 0xd6b664f0, SUC_lexicon_tags_11 },
    { 0x50d664ef, SUC_lexicon_tags_11 },
    { 0xe27664f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493664f7, SUC_lexicon_tags_37 },
    { 0x70b664f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b664fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa366500, SUC_lexicon_tags_6 },
    { 0x3b366501, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x46566504, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f566506, SUC_lexicon_tags_45 },
    { 0x92766506, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d6650a, SUC_lexicon_tags_27 },
    { 0x5b76650a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbdb6650d, SUC_lexicon_tags_11 },
    { 0x3256650d, SUC_lexicon_tags_26 },
    { 0x3296650f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb66512, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7766516, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b66519, SUC_lexicon_tags_7 },
    { 0xd5366519, SUC_lexicon_tags_2 },
    { 0x2756651b, SUC_lexicon_tags_25 },
    { 0x5896651c, SUC_lexicon_tags_25 },
    { 0xa2566519, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa056651f, SUC_lexicon_tags_25 },
    { 0x3af6651f, SUC_lexicon_tags_15 },
    { 0x72566521, SUC_lexicon_tags_2 },
    { 0x78166522, SUC_lexicon_tags_8 },
    { 0xe5f66523, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6d66525, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b66528, SUC_lexicon_tags_19 },
    { 0xc4366529, SUC_lexicon_tags_37 },
    { 0x2b766528, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d96652f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff6653f, SUC_lexicon_tags_37 },
    { 0x1656653f, SUC_lexicon_tags_15 },
    { 0xa966541, SUC_lexicon_tags_12 },
    { 0x5dd6653f, SUC_lexicon_tags_11 },
    { 0x6966543, SUC_lexicon_tags_12 },
    { 0xe636653f, SUC_lexicon_tags_32 },
    { 0x65566544, SUC_lexicon_tags_37 },
    { 0xf5566541, SUC_lexicon_tags_1 },
    { 0x36966543, SUC_lexicon_tags_37 },
    { 0xc1966544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3636654c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f66550, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78166557, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b6655a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2836655d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc236655f, SUC_lexicon_tags_160 },
    { 0x96b66560, SUC_lexicon_tags_17 },
    { 0xd1f66560, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4f66563, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b966566, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ad66568, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d766572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfbb66574, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9cf66576, SUC_lexicon_tags_9 },
    { 0xfdb66577, SUC_lexicon_tags_2 },
    { 0x29d66578, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd6657b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d66580, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d66584, SUC_lexicon_tags_6 },
    { 0xcb766584, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8166587, SUC_lexicon_tags_11 },
    { 0xeab66588, SUC_lexicon_tags_12 },
    { 0x2a366587, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e76658c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b366590, SUC_lexicon_tags_26 },
    { 0xe3166591, SUC_lexicon_tags_29 },
    { 0xa2766592, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb766599, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1736659c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x736659e, SUC_lexicon_tags_9 },
    { 0xa556659f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x421665a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d1665a3, SUC_lexicon_tags_19 },
    { 0xae7665a3, SUC_lexicon_tags_25 },
    { 0x167665a5, SUC_lexicon_tags_31 },
    { 0xcf5665a6, SUC_lexicon_tags_47 },
    { 0xa43665a7, SUC_lexicon_tags_15 },
    { 0xa35665a4, SUC_lexicon_tags_17 },
    { 0x529665a9, SUC_lexicon_tags_17 },
    { 0x1ef665a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x843665ad, SUC_lexicon_tags_37 },
    { 0xa6b665ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627665bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd665c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8b665c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61665c9, SUC_lexicon_tags_23 },
    { 0x649665ca, SUC_lexicon_tags_37 },
    { 0x299665cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b665cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99665d3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf1665d5, SUC_lexicon_tags_14 },
    { 0x9665d6, SUC_lexicon_tags_48 },
    { 0x1df665d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97665dd, SUC_lexicon_tags_37 },
    { 0xb57665dd, SUC_lexicon_tags_25 },
    { 0xf3665df, SUC_lexicon_tags_2 },
    { 0x3cf665e0, SUC_lexicon_tags_14 },
    { 0x877665e1, SUC_lexicon_tags_37 },
    { 0x5f9665de, SUC_lexicon_tags_25 },
    { 0xbcf665e3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5dd665e5, SUC_lexicon_tags_14 },
    { 0xa35665e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203665ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x493665ef, SUC_lexicon_tags_6 },
    { 0x443665ef, SUC_lexicon_tags_1 },
    { 0x299665f1, SUC_lexicon_tags_31 },
    { 0xea7665f2, SUC_lexicon_tags_15 },
    { 0xd73665f2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37665f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf665f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f665fc, SUC_lexicon_tags_9 },
    { 0xf09665fd, SUC_lexicon_tags_10 },
    { 0x299665fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3166601, SUC_lexicon_tags_17 },
    { 0xe4966601, SUC_lexicon_tags_87 },
    { 0xda566603, SUC_lexicon_tags_30 },
    { 0xa3f66601, SUC_lexicon_tags_37 },
    { 0xaab66605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b6660a, SUC_lexicon_tags_1 },
    { 0x9cb6660b, SUC_lexicon_tags_37 },
    { 0x6316660c, SUC_lexicon_tags_25 },
    { 0x2796660c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d166612, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7dd66614, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7766617, SUC_lexicon_tags_11 },
    { 0xe966617, SUC_lexicon_tags_17 },
    { 0xa1366618, SUC_lexicon_tags_17 },
    { 0x32966618, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b766621, SUC_lexicon_tags_31 },
    { 0x69b66622, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaad66624, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb66627, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xed566629, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb36662b, SUC_lexicon_tags_4 },
    { 0x61b6662b, SUC_lexicon_tags_14 },
    { 0xa656662d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9166634, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x38366636, SUC_lexicon_tags_23 },
    { 0xaa366637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd436663d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27566645, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbdd66647, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70d66649, SUC_lexicon_tags_28 },
    { 0x36b66649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d6664c, SUC_lexicon_tags_4 },
    { 0xa156664d, SUC_lexicon_tags_38 },
    { 0xd456664e, SUC_lexicon_tags_25 },
    { 0xae16664f, SUC_lexicon_tags_11 },
    { 0xcdd6664e, SUC_lexicon_tags_11 },
    { 0x68d6664d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x81166653, SUC_lexicon_tags_12 },
    { 0xfcd66654, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x25166657, SUC_lexicon_tags_17 },
    { 0x6f966657, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2376665a, SUC_lexicon_tags_43 },
    { 0x8e36665b, SUC_lexicon_tags_29 },
    { 0xc3f6665a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfa16665e, SUC_lexicon_tags_30 },
    { 0x1bf6665f, SUC_lexicon_tags_11 },
    { 0xbf766660, SUC_lexicon_tags_15 },
    { 0x8eb66661, SUC_lexicon_tags_636 },
    { 0, NULL },
    { 0x4d566663, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe2566665, SUC_lexicon_tags_28 },
    { 0xcd166666, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c966669, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9d56666b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2966672, SUC_lexicon_tags_43 },
    { 0x95566672, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1fb66675, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x45b66677, SUC_lexicon_tags_11 },
    { 0xa8f66678, SUC_lexicon_tags_18 },
    { 0x8d166677, SUC_lexicon_tags_37 },
    { 0xa516667a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b6667d, SUC_lexicon_tags_47 },
    { 0x4f36667e, SUC_lexicon_tags_27 },
    { 0xb76667f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e766685, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf366687, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb6668a, SUC_lexicon_tags_63 },
    { 0x6416668b, SUC_lexicon_tags_37 },
    { 0x9e56668c, SUC_lexicon_tags_29 },
    { 0xf5b6668b, SUC_lexicon_tags_25 },
    { 0xe196668e, SUC_lexicon_tags_17 },
    { 0xd3b6668f, SUC_lexicon_tags_11 },
    { 0x1296668a, SUC_lexicon_tags_12 },
    { 0x3e566690, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x40766693, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b66698, SUC_lexicon_tags_15 },
    { 0xbc566699, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b6669d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x177666a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc87666a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75666a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x589666a8, SUC_lexicon_tags_11 },
    { 0x669666a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9666ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f666b0, SUC_lexicon_tags_1 },
    { 0x16d666b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05666b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef666b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb25666ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5666bd, SUC_lexicon_tags_166 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f666c2, SUC_lexicon_tags_45 },
    { 0x9ab666c3, SUC_lexicon_tags_21 },
    { 0xdd1666c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x54d666c6, SUC_lexicon_tags_34 },
    { 0xba1666c6, SUC_lexicon_tags_37 },
    { 0x5f1666c8, SUC_lexicon_tags_12 },
    { 0xe61666c8, SUC_lexicon_tags_8 },
    { 0x6ad666ca, SUC_lexicon_tags_25 },
    { 0x56d666c9, SUC_lexicon_tags_29 },
    { 0x4ff666cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5b666ce, SUC_lexicon_tags_28 },
    { 0xd1666ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59666d2, SUC_lexicon_tags_11 },
    { 0xedb666d3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9666d7, SUC_lexicon_tags_60 },
    { 0x5b5666d8, SUC_lexicon_tags_8 },
    { 0xda1666d7, SUC_lexicon_tags_27 },
    { 0xb3d666d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5666e1, SUC_lexicon_tags_47 },
    { 0xd6d666e2, SUC_lexicon_tags_16 },
    { 0x7666e3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb1f666e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89666eb, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xae9666ed, SUC_lexicon_tags_45 },
    { 0xc59666ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa29666f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf21666f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3666f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d666fa, SUC_lexicon_tags_2 },
    { 0xbbf666fa, SUC_lexicon_tags_15 },
    { 0xf0f666fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d66702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf966704, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac366706, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc8b66708, SUC_lexicon_tags_12 },
    { 0x18f66709, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b6670c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22366711, SUC_lexicon_tags_49 },
    { 0xc6166712, SUC_lexicon_tags_38 },
    { 0xdaf66713, SUC_lexicon_tags_15 },
    { 0xc0f66714, SUC_lexicon_tags_7 },
    { 0x89d66715, SUC_lexicon_tags_25 },
    { 0x5c366716, SUC_lexicon_tags_1 },
    { 0x85b66717, SUC_lexicon_tags_37 },
    { 0x9c166711, SUC_lexicon_tags_54 },
    { 0x8166719, SUC_lexicon_tags_23 },
    { 0xb936671a, SUC_lexicon_tags_17 },
    { 0x7d166719, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df6671f, SUC_lexicon_tags_11 },
    { 0x916671f, SUC_lexicon_tags_15 },
    { 0xf1566721, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f66724, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8966727, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2936672a, SUC_lexicon_tags_11 },
    { 0x4bd6672b, SUC_lexicon_tags_60 },
    { 0x9096672c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf776672e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9f966730, SUC_lexicon_tags_17 },
    { 0x5bb66731, SUC_lexicon_tags_22 },
    { 0x2a766732, SUC_lexicon_tags_25 },
    { 0x33b66732, SUC_lexicon_tags_19 },
    { 0x3d366732, SUC_lexicon_tags_23 },
    { 0x90366735, SUC_lexicon_tags_2 },
    { 0xfef66731, SUC_lexicon_tags_29 },
    { 0x28366737, SUC_lexicon_tags_15 },
    { 0x3a366738, SUC_lexicon_tags_13 },
    { 0x75966737, SUC_lexicon_tags_8 },
    { 0x1a96673a, SUC_lexicon_tags_17 },
    { 0xf9f6673b, SUC_lexicon_tags_43 },
    { 0x62b6673c, SUC_lexicon_tags_37 },
    { 0x9b56673c, SUC_lexicon_tags_37 },
    { 0x2096673c, SUC_lexicon_tags_14 },
    { 0x8256673c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9766745, SUC_lexicon_tags_26 },
    { 0x2f66746, SUC_lexicon_tags_25 },
    { 0x3d66747, SUC_lexicon_tags_83 },
    { 0xf7566748, SUC_lexicon_tags_2 },
    { 0x2c566748, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb966750, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e766752, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x73566754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88366756, SUC_lexicon_tags_11 },
    { 0x79966756, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe2166759, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc076675b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff66761, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2b66763, SUC_lexicon_tags_15 },
    { 0xaad66763, SUC_lexicon_tags_23 },
    { 0x6966763, SUC_lexicon_tags_1 },
    { 0x66d66764, SUC_lexicon_tags_25 },
    { 0x82d66766, SUC_lexicon_tags_60 },
    { 0x4fb66768, SUC_lexicon_tags_25 },
    { 0x31b66769, SUC_lexicon_tags_25 },
    { 0x39b6676a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b66771, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e966775, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68d66777, SUC_lexicon_tags_10 },
    { 0xfff66777, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xddd6677a, SUC_lexicon_tags_12 },
    { 0xdbf6677b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65166781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c366783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19166786, SUC_lexicon_tags_45 },
    { 0xae366786, SUC_lexicon_tags_60 },
    { 0xcd966788, SUC_lexicon_tags_2 },
    { 0x63b66789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c16678b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d6678e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d66793, SUC_lexicon_tags_30 },
    { 0x69966794, SUC_lexicon_tags_37 },
    { 0x60b66794, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf166799, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b6679d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27667a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1667a7, SUC_lexicon_tags_11 },
    { 0x557667a7, SUC_lexicon_tags_45 },
    { 0xf4b667a9, SUC_lexicon_tags_140 },
    { 0xb0d667aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263667ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93d667b0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af667b4, SUC_lexicon_tags_7 },
    { 0xa77667b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3667b8, SUC_lexicon_tags_17 },
    { 0xc05667b8, SUC_lexicon_tags_12 },
    { 0x989667ba, SUC_lexicon_tags_15 },
    { 0x763667bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff7667bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64b667bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed667c2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff667c5, SUC_lexicon_tags_25 },
    { 0xa4f667c6, SUC_lexicon_tags_87 },
    { 0xa91667c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf667cb, SUC_lexicon_tags_25 },
    { 0xc69667cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3667d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df667d3, SUC_lexicon_tags_14 },
    { 0xe79667d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x483667d6, SUC_lexicon_tags_12 },
    { 0x4c9667d6, SUC_lexicon_tags_71 },
    { 0x353667d8, SUC_lexicon_tags_63 },
    { 0x44d667d8, SUC_lexicon_tags_11 },
    { 0x199667d6, SUC_lexicon_tags_9 },
    { 0x25f667d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe47667dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x117667df, SUC_lexicon_tags_25 },
    { 0x839667e0, SUC_lexicon_tags_14 },
    { 0x2f1667e0, SUC_lexicon_tags_1 },
    { 0xf7b667e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47d667e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ef667e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xed667e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39667ee, SUC_lexicon_tags_25 },
    { 0x25667ee, SUC_lexicon_tags_65 },
    { 0x6b3667ee, SUC_lexicon_tags_9 },
    { 0x6ab667f1, SUC_lexicon_tags_25 },
    { 0x485667f2, SUC_lexicon_tags_1 },
    { 0xe35667ef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5667f8, SUC_lexicon_tags_15 },
    { 0x2d5667f9, SUC_lexicon_tags_30 },
    { 0x403667f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1667fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85966801, SUC_lexicon_tags_8 },
    { 0x7d366801, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9af66804, SUC_lexicon_tags_43 },
    { 0x35d66805, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf6d66807, SUC_lexicon_tags_11 },
    { 0xa6966808, SUC_lexicon_tags_2 },
    { 0x1af66808, SUC_lexicon_tags_1 },
    { 0xecb66807, SUC_lexicon_tags_84 },
    { 0xab36680a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1766813, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x69966815, SUC_lexicon_tags_54 },
    { 0x9ad66816, SUC_lexicon_tags_37 },
    { 0x4c566817, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3ef66819, SUC_lexicon_tags_2 },
    { 0x5b6681a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f16681d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53766823, SUC_lexicon_tags_27 },
    { 0xd8166824, SUC_lexicon_tags_29 },
    { 0xf1f66825, SUC_lexicon_tags_14 },
    { 0x50b66826, SUC_lexicon_tags_7 },
    { 0x3e566825, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8d66829, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xcdb6682b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8776682e, SUC_lexicon_tags_27 },
    { 0x8556682e, SUC_lexicon_tags_60 },
    { 0xf6966830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6966834, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf6166836, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd566845, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe396684a, SUC_lexicon_tags_17 },
    { 0xf96684b, SUC_lexicon_tags_15 },
    { 0x4fd6684c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x84d6684e, SUC_lexicon_tags_31 },
    { 0x20d6684f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65566851, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58966853, SUC_lexicon_tags_37 },
    { 0xf1d66853, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a366856, SUC_lexicon_tags_11 },
    { 0x58166856, SUC_lexicon_tags_40 },
    { 0x97766858, SUC_lexicon_tags_1 },
    { 0xf366859, SUC_lexicon_tags_11 },
    { 0xcab6685a, SUC_lexicon_tags_6 },
    { 0x8c966859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9f6685d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b66865, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6566867, SUC_lexicon_tags_30 },
    { 0x38966868, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b6686c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd66870, SUC_lexicon_tags_17 },
    { 0xe5966870, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d66875, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1b966877, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4656687b, SUC_lexicon_tags_7 },
    { 0x62b6687c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1356687e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c166882, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f766887, SUC_lexicon_tags_25 },
    { 0x51366888, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f6688b, SUC_lexicon_tags_11 },
    { 0x2a96688c, SUC_lexicon_tags_46 },
    { 0x6d56688c, SUC_lexicon_tags_26 },
    { 0x2956688e, SUC_lexicon_tags_43 },
    { 0x1616688e, SUC_lexicon_tags_37 },
    { 0x62f6688b, SUC_lexicon_tags_28 },
    { 0xdad66891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1166898, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c76689c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4dd6689e, SUC_lexicon_tags_15 },
    { 0x5516689f, SUC_lexicon_tags_37 },
    { 0xed16689e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4c7668a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5668a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433668ac, SUC_lexicon_tags_37 },
    { 0x42d668ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359668b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af668bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x117668be, SUC_lexicon_tags_48 },
    { 0x70f668be, SUC_lexicon_tags_5 },
    { 0x6d668c0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875668c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b3668c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09668d0, SUC_lexicon_tags_29 },
    { 0x8f668d1, SUC_lexicon_tags_17 },
    { 0xd03668d2, SUC_lexicon_tags_10 },
    { 0x70d668d3, SUC_lexicon_tags_25 },
    { 0x931668d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x423668d6, SUC_lexicon_tags_16 },
    { 0x6e3668d7, SUC_lexicon_tags_11 },
    { 0x4f9668d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847668dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5fb668df, SUC_lexicon_tags_0 },
    { 0xf73668e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb57668e2, SUC_lexicon_tags_57 },
    { 0x57f668e3, SUC_lexicon_tags_32 },
    { 0x34f668e4, SUC_lexicon_tags_7 },
    { 0x53668e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205668ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49668f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x799668f6, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0x7a1668f8, SUC_lexicon_tags_38 },
    { 0x5fb668f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa5668fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9d668fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd66905, SUC_lexicon_tags_12 },
    { 0x13b66906, SUC_lexicon_tags_11 },
    { 0xacf66906, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc456690a, SUC_lexicon_tags_16 },
    { 0xd76690b, SUC_lexicon_tags_9 },
    { 0xcf36690c, SUC_lexicon_tags_14 },
    { 0x6676690b, SUC_lexicon_tags_12 },
    { 0x7316690e, SUC_lexicon_tags_9 },
    { 0x2696690f, SUC_lexicon_tags_25 },
    { 0x4756690f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb566918, SUC_lexicon_tags_2 },
    { 0xf6766918, SUC_lexicon_tags_30 },
    { 0xac16691a, SUC_lexicon_tags_12 },
    { 0xb996691a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77566922, SUC_lexicon_tags_30 },
    { 0x7566923, SUC_lexicon_tags_47 },
    { 0xdcf66922, SUC_lexicon_tags_37 },
    { 0xd1766925, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2166929, SUC_lexicon_tags_83 },
    { 0x2496692a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb66931, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56f66933, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb8f66935, SUC_lexicon_tags_31 },
    { 0x9bf66936, SUC_lexicon_tags_11 },
    { 0xe9b66935, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7366939, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b96693b, SUC_lexicon_tags_7 },
    { 0x9d16693b, SUC_lexicon_tags_25 },
    { 0x11d6693d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef966942, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7366945, SUC_lexicon_tags_6 },
    { 0xe0566946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17d66948, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3956694c, SUC_lexicon_tags_60 },
    { 0xea56694c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd66952, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d66954, SUC_lexicon_tags_38 },
    { 0xd3766955, SUC_lexicon_tags_11 },
    { 0x55166956, SUC_lexicon_tags_16 },
    { 0x8a766955, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xa1966959, SUC_lexicon_tags_1 },
    { 0x5cd6695a, SUC_lexicon_tags_11 },
    { 0x3396695a, SUC_lexicon_tags_14 },
    { 0x3a56695c, SUC_lexicon_tags_26 },
    { 0x6916695d, SUC_lexicon_tags_15 },
    { 0xfc56695e, SUC_lexicon_tags_24 },
    { 0xc16695f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xbf766961, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4566965, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9366967, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee16696a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b66972, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa2d66974, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a96697c, SUC_lexicon_tags_19 },
    { 0x1976697d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b66980, SUC_lexicon_tags_27 },
    { 0x75d66980, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x50166984, SUC_lexicon_tags_23 },
    { 0x41d66984, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x656698a, SUC_lexicon_tags_13 },
    { 0xf496698b, SUC_lexicon_tags_53 },
    { 0xbb96698a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6698f, SUC_lexicon_tags_17 },
    { 0x8d76698f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2e166992, SUC_lexicon_tags_33 },
    { 0x65366993, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c966995, SUC_lexicon_tags_14 },
    { 0x1ff66996, SUC_lexicon_tags_15 },
    { 0x25166996, SUC_lexicon_tags_11 },
    { 0x31566998, SUC_lexicon_tags_48 },
    { 0xfc166999, SUC_lexicon_tags_1 },
    { 0xbcf6699a, SUC_lexicon_tags_25 },
    { 0xdc366996, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe856699e, SUC_lexicon_tags_68 },
    { 0x30b6699f, SUC_lexicon_tags_7 },
    { 0xeab669a0, SUC_lexicon_tags_82 },
    { 0x2a3669a1, SUC_lexicon_tags_7 },
    { 0xd176699e, SUC_lexicon_tags_7 },
    { 0x683669a1, SUC_lexicon_tags_15 },
    { 0x793669a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7ad669a6, SUC_lexicon_tags_31 },
    { 0xe7669a7, SUC_lexicon_tags_59 },
    { 0xd1d669a6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x3669ab, SUC_lexicon_tags_0 },
    { 0x20d669ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d669b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9669b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21669bb, SUC_lexicon_tags_55 },
    { 0xf1d669bb, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67669c0, SUC_lexicon_tags_12 },
    { 0xa95669c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d669c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd669cb, SUC_lexicon_tags_27 },
    { 0xac5669cb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1669cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b669d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x673669d5, SUC_lexicon_tags_33 },
    { 0x173669d6, SUC_lexicon_tags_55 },
    { 0xf53669d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81669dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x139669de, SUC_lexicon_tags_226 },
    { 0xe77669df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f669e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7669e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb3669e8, SUC_lexicon_tags_7 },
    { 0x51d669e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55669ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13d669ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73669f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93669f9, SUC_lexicon_tags_23 },
    { 0x7d1669fa, SUC_lexicon_tags_25 },
    { 0xe01669f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb766a01, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d66a05, SUC_lexicon_tags_15 },
    { 0x57366a06, SUC_lexicon_tags_9 },
    { 0xff566a06, SUC_lexicon_tags_120 },
    { 0xdb566a08, SUC_lexicon_tags_28 },
    { 0x3f366a08, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ad66a0b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b66a12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b66a16, SUC_lexicon_tags_29 },
    { 0x9166a16, SUC_lexicon_tags_47 },
    { 0xe2366a18, SUC_lexicon_tags_37 },
    { 0xf1d66a19, SUC_lexicon_tags_25 },
    { 0x5bd66a1a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x76966a1d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77566a21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa9366a23, SUC_lexicon_tags_82 },
    { 0xded66a24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10366a26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8566a2a, SUC_lexicon_tags_43 },
    { 0xac166a2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb766a2f, SUC_lexicon_tags_53 },
    { 0x9b366a30, SUC_lexicon_tags_27 },
    { 0x51f66a30, SUC_lexicon_tags_35 },
    { 0xb9966a32, SUC_lexicon_tags_8 },
    { 0xa3766a33, SUC_lexicon_tags_17 },
    { 0xc7766a34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10166a3a, SUC_lexicon_tags_7 },
    { 0x6d66a3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb966a3e, SUC_lexicon_tags_17 },
    { 0xc2766a3f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45166a46, SUC_lexicon_tags_29 },
    { 0x22766a47, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaf66a49, SUC_lexicon_tags_23 },
    { 0x27366a49, SUC_lexicon_tags_8 },
    { 0x88f66a4b, SUC_lexicon_tags_14 },
    { 0x97566a4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x29166a4f, SUC_lexicon_tags_28 },
    { 0xbcd66a4f, SUC_lexicon_tags_17 },
    { 0xa3f66a4f, SUC_lexicon_tags_37 },
    { 0x11966a51, SUC_lexicon_tags_17 },
    { 0xff166a50, SUC_lexicon_tags_124 },
    { 0x1c166a4f, SUC_lexicon_tags_26 },
    { 0x1ed66a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17766a58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17566a5d, SUC_lexicon_tags_17 },
    { 0x9e966a5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf66a66, SUC_lexicon_tags_12 },
    { 0xc5566a67, SUC_lexicon_tags_17 },
    { 0xf3f66a67, SUC_lexicon_tags_7 },
    { 0x83766a69, SUC_lexicon_tags_30 },
    { 0x16766a67, SUC_lexicon_tags_12 },
    { 0xacb66a69, SUC_lexicon_tags_75 },
    { 0x68f66a68, SUC_lexicon_tags_17 },
    { 0x37f66a6d, SUC_lexicon_tags_11 },
    { 0x2cf66a6c, SUC_lexicon_tags_45 },
    { 0x5fb66a6f, SUC_lexicon_tags_25 },
    { 0xec966a70, SUC_lexicon_tags_12 },
    { 0xee166a6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51166a78, SUC_lexicon_tags_45 },
    { 0xdf766a78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2dd66a7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x18166a7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf766a84, SUC_lexicon_tags_25 },
    { 0xdbf66a85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6766a87, SUC_lexicon_tags_17 },
    { 0xdf766a88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe366a8a, SUC_lexicon_tags_23 },
    { 0x75966a8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19166a90, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9966a93, SUC_lexicon_tags_476 },
    { 0x86966a93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b166a96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9af66a98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd566aa1, SUC_lexicon_tags_16 },
    { 0x9b366aa2, SUC_lexicon_tags_8 },
    { 0x5b366aa2, SUC_lexicon_tags_13 },
    { 0x93366aa4, SUC_lexicon_tags_2 },
    { 0x5df66aa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2f66aa7, SUC_lexicon_tags_23 },
    { 0x7ef66aa8, SUC_lexicon_tags_32 },
    { 0xa1366aa9, SUC_lexicon_tags_31 },
    { 0x8f166aaa, SUC_lexicon_tags_38 },
    { 0xccf66aa9, SUC_lexicon_tags_15 },
    { 0x99b66aac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb766aaf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x33366ab1, SUC_lexicon_tags_28 },
    { 0xda766ab2, SUC_lexicon_tags_63 },
    { 0xb1b66ab2, SUC_lexicon_tags_7 },
    { 0x40766ab4, SUC_lexicon_tags_47 },
    { 0xf966ab3, SUC_lexicon_tags_37 },
    { 0x50966ab3, SUC_lexicon_tags_1 },
    { 0x33166ab7, SUC_lexicon_tags_37 },
    { 0x4ad66ab7, SUC_lexicon_tags_25 },
    { 0x10966ab8, SUC_lexicon_tags_21 },
    { 0x3f366ab2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f66ac0, SUC_lexicon_tags_1 },
    { 0xfef66ac1, SUC_lexicon_tags_21 },
    { 0x6df66ac2, SUC_lexicon_tags_27 },
    { 0xa1b66ac1, SUC_lexicon_tags_12 },
    { 0x11f66ac2, SUC_lexicon_tags_25 },
    { 0x7ad66ac5, SUC_lexicon_tags_37 },
    { 0x6b766ac6, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd566ace, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae366ad2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2b66ad4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2b66ad6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd66ad9, SUC_lexicon_tags_43 },
    { 0xbdd66ada, SUC_lexicon_tags_17 },
    { 0x7e566adb, SUC_lexicon_tags_63 },
    { 0xf9566adb, SUC_lexicon_tags_87 },
    { 0xd0f66add, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52566ae0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcfd66ae2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda566aee, SUC_lexicon_tags_28 },
    { 0xe0366aef, SUC_lexicon_tags_2 },
    { 0x97b66aee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcd66af2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70366af4, SUC_lexicon_tags_33 },
    { 0x5f766af4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd66af8, SUC_lexicon_tags_11 },
    { 0x60366af9, SUC_lexicon_tags_1 },
    { 0x52366afa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2366afc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b66b00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94966b02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xab66b04, SUC_lexicon_tags_11 },
    { 0xc3966b05, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0xfd766b07, SUC_lexicon_tags_1284 },
    { 0, NULL },
    { 0xd9f66b09, SUC_lexicon_tags_15 },
    { 0x37d66b0a, SUC_lexicon_tags_43 },
    { 0x1e566b0b, SUC_lexicon_tags_38 },
    { 0x34366b0c, SUC_lexicon_tags_1 },
    { 0xceb66b0d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd66b10, SUC_lexicon_tags_15 },
    { 0xcf166b10, SUC_lexicon_tags_37 },
    { 0x46f66b12, SUC_lexicon_tags_30 },
    { 0x8366b13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2966b15, SUC_lexicon_tags_47 },
    { 0x2d66b15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d966b1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c566b1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf66b23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d66b26, SUC_lexicon_tags_85 },
    { 0xb5966b27, SUC_lexicon_tags_19 },
    { 0x1e366b26, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf66b2b, SUC_lexicon_tags_88 },
    { 0x3d366b2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24366b31, SUC_lexicon_tags_185 },
    { 0x1d566b32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b66b35, SUC_lexicon_tags_11 },
    { 0xdf166b35, SUC_lexicon_tags_2 },
    { 0x48766b37, SUC_lexicon_tags_34 },
    { 0xb8d66b37, SUC_lexicon_tags_178 },
    { 0x96f66b37, SUC_lexicon_tags_12 },
    { 0x1d566b35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb566b3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb66b42, SUC_lexicon_tags_12 },
    { 0x21966b43, SUC_lexicon_tags_3 },
    { 0x50f66b44, SUC_lexicon_tags_11 },
    { 0xcf66b45, SUC_lexicon_tags_37 },
    { 0x2bd66b43, SUC_lexicon_tags_47 },
    { 0x62566b47, SUC_lexicon_tags_66 },
    { 0x13766b47, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c966b4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f66b51, SUC_lexicon_tags_26 },
    { 0xb2f66b51, SUC_lexicon_tags_1 },
    { 0xf7366b53, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x86966b55, SUC_lexicon_tags_65 },
    { 0x46f66b56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaf966b58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e766b5b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe766b5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe3966b5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc166b62, SUC_lexicon_tags_7 },
    { 0x95566b63, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41366b65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5966b6a, SUC_lexicon_tags_34 },
    { 0xdd966b6a, SUC_lexicon_tags_49 },
    { 0x95d66b6c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa8966b6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5ad66b70, SUC_lexicon_tags_11 },
    { 0x98966b71, SUC_lexicon_tags_12 },
    { 0x48f66b72, SUC_lexicon_tags_82 },
    { 0xd8366b73, SUC_lexicon_tags_47 },
    { 0x25b66b71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd966b76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48166b7a, SUC_lexicon_tags_28 },
    { 0x9a166b7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47f66b7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54166b83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8366b85, SUC_lexicon_tags_33 },
    { 0xa1f66b86, SUC_lexicon_tags_118 },
    { 0xe1366b87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7766b8c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f66b91, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1166b96, SUC_lexicon_tags_49 },
    { 0x9eb66b96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8566b99, SUC_lexicon_tags_19 },
    { 0x33166b9a, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46566b9f, SUC_lexicon_tags_1 },
    { 0xd2f66b9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c566ba2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2366ba5, SUC_lexicon_tags_25 },
    { 0x9966ba6, SUC_lexicon_tags_14 },
    { 0xf1566ba6, SUC_lexicon_tags_2 },
    { 0x79d66ba5, SUC_lexicon_tags_12 },
    { 0x8e566ba9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c966bae, SUC_lexicon_tags_15 },
    { 0x2a366bae, SUC_lexicon_tags_19 },
    { 0xab566bb0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeff66bb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84d66bb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c566bb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5166bc1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd766bc6, SUC_lexicon_tags_48 },
    { 0xbab66bc7, SUC_lexicon_tags_11 },
    { 0x76366bc8, SUC_lexicon_tags_8 },
    { 0x3a566bc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0766bcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59766bcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf66bd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d66bd5, SUC_lexicon_tags_32 },
    { 0x14966bd6, SUC_lexicon_tags_15 },
    { 0x62966bd5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44f66bd9, SUC_lexicon_tags_37 },
    { 0xf766bd9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10566be0, SUC_lexicon_tags_27 },
    { 0xd1366be1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab66be7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad66bec, SUC_lexicon_tags_19 },
    { 0x7cb66bed, SUC_lexicon_tags_25 },
    { 0x39166bee, SUC_lexicon_tags_77 },
    { 0x24166bef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b66bf5, SUC_lexicon_tags_2 },
    { 0xc6566bf6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a766bfb, SUC_lexicon_tags_11 },
    { 0xd4766bfc, SUC_lexicon_tags_14 },
    { 0xe8b66bfc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83d66bff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88366c02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b66c05, SUC_lexicon_tags_1 },
    { 0xa8f66c06, SUC_lexicon_tags_14 },
    { 0x39f66c07, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6eb66c09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5166c0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24966c0f, SUC_lexicon_tags_1 },
    { 0x5cb66c10, SUC_lexicon_tags_63 },
    { 0xcb566c10, SUC_lexicon_tags_0 },
    { 0x44f66c0f, SUC_lexicon_tags_4 },
    { 0xd1d66c11, SUC_lexicon_tags_9 },
    { 0x89966c14, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d66c18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf766c1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16366c22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c366c26, SUC_lexicon_tags_14 },
    { 0x13b66c27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d66c2a, SUC_lexicon_tags_45 },
    { 0x13366c2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ab66c2d, SUC_lexicon_tags_0 },
    { 0x8ab66c2e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd166c32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1566c35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c766c38, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcc166c3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x39966c3d, SUC_lexicon_tags_17 },
    { 0x15166c3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee966c47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e766c49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa6166c4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f66c4e, SUC_lexicon_tags_2 },
    { 0x64366c4f, SUC_lexicon_tags_17 },
    { 0xa9f66c50, SUC_lexicon_tags_108 },
    { 0x25366c50, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x29566c54, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53366c59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7766c5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf66c61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d66c66, SUC_lexicon_tags_38 },
    { 0xa8966c66, SUC_lexicon_tags_33 },
    { 0xccf66c68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96566c6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25766c6d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x87966c6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd7166c71, SUC_lexicon_tags_15 },
    { 0xd5766c72, SUC_lexicon_tags_160 },
    { 0xefb66c73, SUC_lexicon_tags_2 },
    { 0x22766c74, SUC_lexicon_tags_33 },
    { 0xe0166c74, SUC_lexicon_tags_23 },
    { 0xdf566c76, SUC_lexicon_tags_33 },
    { 0x5fb66c76, SUC_lexicon_tags_60 },
    { 0xc7d66c78, SUC_lexicon_tags_0 },
    { 0xdad66c79, SUC_lexicon_tags_29 },
    { 0x82366c79, SUC_lexicon_tags_14 },
    { 0x13566c7b, SUC_lexicon_tags_8 },
    { 0xa1166c7c, SUC_lexicon_tags_11 },
    { 0x4ed66c7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8766c84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3366c88, SUC_lexicon_tags_9 },
    { 0x63566c89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6166c8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb966c8e, SUC_lexicon_tags_11 },
    { 0x15166c8e, SUC_lexicon_tags_23 },
    { 0xba366c90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d66c93, SUC_lexicon_tags_15 },
    { 0x53766c94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34166c99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8bf66c9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4166c9f, SUC_lexicon_tags_1 },
    { 0x15f66ca0, SUC_lexicon_tags_23 },
    { 0x1e766ca1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd766ca5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xec366ca8, SUC_lexicon_tags_262 },
    { 0x73566ca8, SUC_lexicon_tags_23 },
    { 0x15b66caa, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x42766cac, SUC_lexicon_tags_31 },
    { 0x80f66cad, SUC_lexicon_tags_12 },
    { 0x84366cae, SUC_lexicon_tags_7 },
    { 0x45166cad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x61566cb2, SUC_lexicon_tags_107 },
    { 0x89566cb2, SUC_lexicon_tags_23 },
    { 0x49166cb4, SUC_lexicon_tags_19 },
    { 0x6d766cb5, SUC_lexicon_tags_16 },
    { 0x25d66cb6, SUC_lexicon_tags_0 },
    { 0x20b66cb5, SUC_lexicon_tags_17 },
    { 0xea766cb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e366cbd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2f366cbf, SUC_lexicon_tags_37 },
    { 0x88b66cc0, SUC_lexicon_tags_37 },
    { 0x9db66cbf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf966cc6, SUC_lexicon_tags_1 },
    { 0xdc366cc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b166cc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f66cce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5f66cd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d966cd3, SUC_lexicon_tags_29 },
    { 0xd6566cd4, SUC_lexicon_tags_7 },
    { 0x2d766cd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa366cd7, SUC_lexicon_tags_47 },
    { 0xda166cd7, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d566cdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7966cdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78566ce2, SUC_lexicon_tags_2 },
    { 0x3ad66ce3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d66ce7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0566cec, SUC_lexicon_tags_30 },
    { 0xa7366ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8366cf0, SUC_lexicon_tags_47 },
    { 0x73166cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdd66cf3, SUC_lexicon_tags_29 },
    { 0xcd66cf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57366cf7, SUC_lexicon_tags_60 },
    { 0x22366cf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31b66cfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf566cfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87766d01, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf166d10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef66d13, SUC_lexicon_tags_103 },
    { 0x65566d14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d66d19, SUC_lexicon_tags_13 },
    { 0x8af66d1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f66d1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89966d24, SUC_lexicon_tags_60 },
    { 0x18f66d24, SUC_lexicon_tags_11 },
    { 0x9d966d26, SUC_lexicon_tags_33 },
    { 0x9d166d26, SUC_lexicon_tags_33 },
    { 0xad966d28, SUC_lexicon_tags_28 },
    { 0xbe166d29, SUC_lexicon_tags_28 },
    { 0xfd966d25, SUC_lexicon_tags_7 },
    { 0x68566d2b, SUC_lexicon_tags_25 },
    { 0x18b66d2c, SUC_lexicon_tags_32 },
    { 0x86366d2b, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f566d30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f966d32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d66d38, SUC_lexicon_tags_12 },
    { 0x57966d38, SUC_lexicon_tags_28 },
    { 0xc7166d38, SUC_lexicon_tags_1 },
    { 0x8566d3b, SUC_lexicon_tags_26 },
    { 0x4d966d3c, SUC_lexicon_tags_48 },
    { 0x2dd66d3d, SUC_lexicon_tags_66 },
    { 0x2d566d3e, SUC_lexicon_tags_142 },
    { 0, NULL },
    { 0x94166d40, SUC_lexicon_tags_85 },
    { 0x51b66d41, SUC_lexicon_tags_33 },
    { 0x64566d40, SUC_lexicon_tags_25 },
    { 0x44b66d43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10966d45, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f66d49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d66d50, SUC_lexicon_tags_29 },
    { 0xbf766d51, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d66d57, SUC_lexicon_tags_0 },
    { 0xd2166d57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2d66d5a, SUC_lexicon_tags_65 },
    { 0xfb166d5a, SUC_lexicon_tags_48 },
    { 0xb7d66d5c, SUC_lexicon_tags_15 },
    { 0xf7166d5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71366d61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b566d63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x93166d65, SUC_lexicon_tags_17 },
    { 0x7d166d66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0366d6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4966d6e, SUC_lexicon_tags_15 },
    { 0x3966d6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40b66d71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d566d74, SUC_lexicon_tags_7 },
    { 0x5a966d75, SUC_lexicon_tags_37 },
    { 0x7f366d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba566d79, SUC_lexicon_tags_8 },
    { 0xbe366d7a, SUC_lexicon_tags_21 },
    { 0xc9366d7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb66d7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x15166d80, SUC_lexicon_tags_45 },
    { 0x14366d81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb4166d83, SUC_lexicon_tags_7 },
    { 0x27566d84, SUC_lexicon_tags_49 },
    { 0xa2366d85, SUC_lexicon_tags_17 },
    { 0xde766d85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8366d89, SUC_lexicon_tags_23 },
    { 0xe1566d8a, SUC_lexicon_tags_37 },
    { 0x61366d8b, SUC_lexicon_tags_38 },
    { 0xb1966d8a, SUC_lexicon_tags_32 },
    { 0x8bf66d8d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e966d95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc766d99, SUC_lexicon_tags_37 },
    { 0x23166d9a, SUC_lexicon_tags_1 },
    { 0xabf66d9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9566da1, SUC_lexicon_tags_27 },
    { 0xf3b66da2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xf6b66da4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd66da9, SUC_lexicon_tags_222 },
    { 0, NULL },
    { 0xfa166dab, SUC_lexicon_tags_2 },
    { 0xa6166dac, SUC_lexicon_tags_0 },
    { 0x3ff66dac, SUC_lexicon_tags_14 },
    { 0x8d66dae, SUC_lexicon_tags_17 },
    { 0xc7966daf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0766db1, SUC_lexicon_tags_82 },
    { 0xdb366db2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8366db5, SUC_lexicon_tags_17 },
    { 0x68b66db6, SUC_lexicon_tags_33 },
    { 0xbe966db6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9d66db9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2b66dbb, SUC_lexicon_tags_2 },
    { 0xfa366dbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23966dbe, SUC_lexicon_tags_15 },
    { 0x7ed66dbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69566dc1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d766dca, SUC_lexicon_tags_37 },
    { 0x84366dcb, SUC_lexicon_tags_17 },
    { 0xfd166dca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d66dd1, SUC_lexicon_tags_25 },
    { 0xa0766dd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc166dd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d66ddd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x33166ddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f66de3, SUC_lexicon_tags_17 },
    { 0xb5966de3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80766de8, SUC_lexicon_tags_33 },
    { 0x7b66de9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85d66deb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bd66ded, SUC_lexicon_tags_17 },
    { 0xb0566dee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d566df2, SUC_lexicon_tags_29 },
    { 0xbeb66df3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcfb66df5, SUC_lexicon_tags_15 },
    { 0xfd366df6, SUC_lexicon_tags_15 },
    { 0x19966df7, SUC_lexicon_tags_9 },
    { 0xae166df6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9566dfb, SUC_lexicon_tags_11 },
    { 0xbf166dfc, SUC_lexicon_tags_25 },
    { 0xb2f66dfd, SUC_lexicon_tags_12 },
    { 0x56166dfe, SUC_lexicon_tags_11 },
    { 0xcf566dfc, SUC_lexicon_tags_9 },
    { 0x61f66dfe, SUC_lexicon_tags_2 },
    { 0xc9b66e01, SUC_lexicon_tags_63 },
    { 0xba166e01, SUC_lexicon_tags_15 },
    { 0x78566e03, SUC_lexicon_tags_207 },
    { 0xc9366dfd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f66e09, SUC_lexicon_tags_17 },
    { 0x23566e0a, SUC_lexicon_tags_26 },
    { 0x30966e09, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4f766e0d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff66e10, SUC_lexicon_tags_13 },
    { 0xeed66e10, SUC_lexicon_tags_9 },
    { 0x2f566e11, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1d166e14, SUC_lexicon_tags_15 },
    { 0xc1366e15, SUC_lexicon_tags_14 },
    { 0x3df66e15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdfb66e18, SUC_lexicon_tags_37 },
    { 0xdf966e19, SUC_lexicon_tags_47 },
    { 0x14966e1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5766e1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62166e23, SUC_lexicon_tags_1 },
    { 0x6c766e23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa2d66e26, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97366e31, SUC_lexicon_tags_64 },
    { 0x366e31, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d66e3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f966e3d, SUC_lexicon_tags_14 },
    { 0xb0166e3e, SUC_lexicon_tags_113 },
    { 0xb4966e3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8566e43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xadb66e45, SUC_lexicon_tags_7 },
    { 0x6d366e46, SUC_lexicon_tags_8 },
    { 0x24566e46, SUC_lexicon_tags_30 },
    { 0xc2366e48, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x91766e4a, SUC_lexicon_tags_15 },
    { 0xf5b66e4b, SUC_lexicon_tags_38 },
    { 0xaef66e4c, SUC_lexicon_tags_37 },
    { 0x37766e4d, SUC_lexicon_tags_87 },
    { 0x9b66e4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6af66e50, SUC_lexicon_tags_9 },
    { 0xf0d66e51, SUC_lexicon_tags_1 },
    { 0x2366e50, SUC_lexicon_tags_49 },
    { 0xe3166e53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98566e55, SUC_lexicon_tags_1 },
    { 0x14166e55, SUC_lexicon_tags_47 },
    { 0x16966e57, SUC_lexicon_tags_113 },
    { 0xf8f66e58, SUC_lexicon_tags_25 },
    { 0x23d66e56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc966e5b, SUC_lexicon_tags_28 },
    { 0xa1f66e5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f566e5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c166e64, SUC_lexicon_tags_7 },
    { 0xf9766e65, SUC_lexicon_tags_1 },
    { 0x71366e66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d66e6a, SUC_lexicon_tags_14 },
    { 0xf6966e6b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76566e72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f66e79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95566e7d, SUC_lexicon_tags_27 },
    { 0x8b66e7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb66e84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf766e8e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3966e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d766e9f, SUC_lexicon_tags_50 },
    { 0x1d566ea0, SUC_lexicon_tags_55 },
    { 0x8a566e9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19166ea5, SUC_lexicon_tags_0 },
    { 0x8e166ea6, SUC_lexicon_tags_25 },
    { 0x2a366ea6, SUC_lexicon_tags_15 },
    { 0x9566ea8, SUC_lexicon_tags_15 },
    { 0x2f966ea9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f166ead, SUC_lexicon_tags_29 },
    { 0xcdd66eae, SUC_lexicon_tags_7 },
    { 0x8dd66eaf, SUC_lexicon_tags_12 },
    { 0xd1166eb0, SUC_lexicon_tags_7 },
    { 0x72966eb1, SUC_lexicon_tags_15 },
    { 0x97966ead, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44966eb4, SUC_lexicon_tags_841 },
    { 0, NULL },
    { 0, NULL },
    { 0x20366eb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9566eb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6366ebb, SUC_lexicon_tags_17 },
    { 0x6f966ebc, SUC_lexicon_tags_63 },
    { 0x3cd66ebd, SUC_lexicon_tags_15 },
    { 0x2f366ebd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd166ec3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb966ecb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7766ed3, SUC_lexicon_tags_25 },
    { 0x9b166ed3, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xa3366ed6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e966ed8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89366ee1, SUC_lexicon_tags_11 },
    { 0x4f766ee2, SUC_lexicon_tags_8 },
    { 0x15b66ee3, SUC_lexicon_tags_17 },
    { 0x9b766ee2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92966eeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x73766eed, SUC_lexicon_tags_43 },
    { 0x46766eee, SUC_lexicon_tags_29 },
    { 0x9e766eee, SUC_lexicon_tags_26 },
    { 0x80566eed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37166efd, SUC_lexicon_tags_40 },
    { 0x56366efd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbaf66f00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f66f09, SUC_lexicon_tags_7 },
    { 0xb0566f0a, SUC_lexicon_tags_8 },
    { 0x2ed66f0b, SUC_lexicon_tags_7 },
    { 0x1cb66f0c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e766f0f, SUC_lexicon_tags_7 },
    { 0x20966f10, SUC_lexicon_tags_2 },
    { 0x53566f10, SUC_lexicon_tags_1 },
    { 0x2a166f12, SUC_lexicon_tags_60 },
    { 0x8dd66f13, SUC_lexicon_tags_35 },
    { 0x13f66f14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff66f19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde166f1d, SUC_lexicon_tags_43 },
    { 0x3f366f1d, SUC_lexicon_tags_27 },
    { 0x62d66f1d, SUC_lexicon_tags_14 },
    { 0xa7566f20, SUC_lexicon_tags_19 },
    { 0xcf366f21, SUC_lexicon_tags_23 },
    { 0x41366f22, SUC_lexicon_tags_32 },
    { 0x23366f22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d66f2d, SUC_lexicon_tags_11 },
    { 0x7cf66f2e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb6b66f30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4566f33, SUC_lexicon_tags_30 },
    { 0x81b66f33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed66f38, SUC_lexicon_tags_17 },
    { 0x8a166f38, SUC_lexicon_tags_17 },
    { 0x7f166f3a, SUC_lexicon_tags_37 },
    { 0xa6766f38, SUC_lexicon_tags_14 },
    { 0x2a766f3c, SUC_lexicon_tags_30 },
    { 0x28566f3c, SUC_lexicon_tags_28 },
    { 0x76366f3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb66f41, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x66766f44, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd66f4b, SUC_lexicon_tags_7 },
    { 0xf0d66f4c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb66f4f, SUC_lexicon_tags_30 },
    { 0x2cb66f50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x72166f52, SUC_lexicon_tags_17 },
    { 0xcef66f53, SUC_lexicon_tags_63 },
    { 0xb2166f54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fb66f56, SUC_lexicon_tags_2 },
    { 0xee166f57, SUC_lexicon_tags_7 },
    { 0xe0566f58, SUC_lexicon_tags_15 },
    { 0xe8966f58, SUC_lexicon_tags_14 },
    { 0x1eb66f57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0966f5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f66f5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b766f62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5166f68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7566f6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64566f75, SUC_lexicon_tags_981 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4166f7b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c566f7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0766f84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d66f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d66f98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6166f9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf9f66f9d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3766fa1, SUC_lexicon_tags_995 },
    { 0xfcb66fa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20766fad, SUC_lexicon_tags_47 },
    { 0x2ff66fae, SUC_lexicon_tags_60 },
    { 0xf3366fae, SUC_lexicon_tags_37 },
    { 0xf7f66fae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20d66fb2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17766fb7, SUC_lexicon_tags_25 },
    { 0x62166fb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d66fbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b66fc0, SUC_lexicon_tags_27 },
    { 0x1eb66fc1, SUC_lexicon_tags_15 },
    { 0x50366fc2, SUC_lexicon_tags_49 },
    { 0x9bb66fc3, SUC_lexicon_tags_10 },
    { 0x89f66fc4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d66fca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x19566fcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f366fd0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x566fd4, SUC_lexicon_tags_17 },
    { 0x22f66fd5, SUC_lexicon_tags_37 },
    { 0x12d66fd5, SUC_lexicon_tags_17 },
    { 0x76966fd7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1966fdb, SUC_lexicon_tags_7 },
    { 0xd566fdb, SUC_lexicon_tags_46 },
    { 0x5f166fdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d566fdf, SUC_lexicon_tags_15 },
    { 0xa3966fdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3366fe3, SUC_lexicon_tags_15 },
    { 0x43566fe4, SUC_lexicon_tags_30 },
    { 0x7f166fe4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28766fe8, SUC_lexicon_tags_37 },
    { 0xb6f66fe9, SUC_lexicon_tags_15 },
    { 0xff66fea, SUC_lexicon_tags_0 },
    { 0x6db66feb, SUC_lexicon_tags_14 },
    { 0xc7366fec, SUC_lexicon_tags_48 },
    { 0x4bd66fec, SUC_lexicon_tags_37 },
    { 0xb966feb, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f66ff4, SUC_lexicon_tags_17 },
    { 0x41f66ff5, SUC_lexicon_tags_2 },
    { 0xc3366ff4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74366ff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14966ffc, SUC_lexicon_tags_31 },
    { 0x9f166ffd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75166fff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e167001, SUC_lexicon_tags_68 },
    { 0x1ab67001, SUC_lexicon_tags_60 },
    { 0x86167003, SUC_lexicon_tags_17 },
    { 0x10967001, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f167006, SUC_lexicon_tags_17 },
    { 0x3c767007, SUC_lexicon_tags_1 },
    { 0xed367008, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7516700a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa516700d, SUC_lexicon_tags_4 },
    { 0x99d6700e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b767013, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3967016, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca76701b, SUC_lexicon_tags_25 },
    { 0xe156701b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4236701f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53367022, SUC_lexicon_tags_38 },
    { 0x5a167022, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12967026, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6836702b, SUC_lexicon_tags_29 },
    { 0xcad6702c, SUC_lexicon_tags_23 },
    { 0x9f16702d, SUC_lexicon_tags_106 },
    { 0x7a56702e, SUC_lexicon_tags_32 },
    { 0x1036702f, SUC_lexicon_tags_2 },
    { 0x78f67030, SUC_lexicon_tags_49 },
    { 0x6a56702e, SUC_lexicon_tags_19 },
    { 0x8536702b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58f67034, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x91967036, SUC_lexicon_tags_25 },
    { 0x18767037, SUC_lexicon_tags_1 },
    { 0x24167036, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f6703f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f67042, SUC_lexicon_tags_1 },
    { 0x18967043, SUC_lexicon_tags_34 },
    { 0x1a167044, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa56704b, SUC_lexicon_tags_27 },
    { 0x2c16704b, SUC_lexicon_tags_84 },
    { 0x7896704d, SUC_lexicon_tags_15 },
    { 0x5f96704d, SUC_lexicon_tags_30 },
    { 0x17d6704f, SUC_lexicon_tags_37 },
    { 0xd136704f, SUC_lexicon_tags_82 },
    { 0xc8f6704e, SUC_lexicon_tags_160 },
    { 0x74f67052, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf367058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc256705d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a767060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3567063, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7296706a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf976706e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c967070, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a367074, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x44167077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce76707a, SUC_lexicon_tags_1 },
    { 0x3536707b, SUC_lexicon_tags_2 },
    { 0xe756707c, SUC_lexicon_tags_37 },
    { 0x8f56707c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb67081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee967084, SUC_lexicon_tags_1 },
    { 0x8367085, SUC_lexicon_tags_2 },
    { 0xc2967086, SUC_lexicon_tags_45 },
    { 0xf0367086, SUC_lexicon_tags_25 },
    { 0xd7767087, SUC_lexicon_tags_49 },
    { 0xcbf67089, SUC_lexicon_tags_25 },
    { 0x82567087, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8216708f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf67092, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b6709a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8d6709c, SUC_lexicon_tags_17 },
    { 0x4456709c, SUC_lexicon_tags_15 },
    { 0x5ef6709d, SUC_lexicon_tags_27 },
    { 0x6596709f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b670a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x641670a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5670aa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31670af, SUC_lexicon_tags_39 },
    { 0x963670b0, SUC_lexicon_tags_28 },
    { 0x945670b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbf670b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb670bc, SUC_lexicon_tags_8 },
    { 0xecf670bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b670c0, SUC_lexicon_tags_84 },
    { 0x131670c1, SUC_lexicon_tags_79 },
    { 0xa05670c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15670c5, SUC_lexicon_tags_17 },
    { 0xf7f670c6, SUC_lexicon_tags_49 },
    { 0x40f670c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xffb670c9, SUC_lexicon_tags_37 },
    { 0x2c1670ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211670cf, SUC_lexicon_tags_16 },
    { 0x1bb670cf, SUC_lexicon_tags_98 },
    { 0x74f670d0, SUC_lexicon_tags_77 },
    { 0x8e1670d2, SUC_lexicon_tags_6 },
    { 0xba7670d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb7670d5, SUC_lexicon_tags_38 },
    { 0x74b670d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d670dc, SUC_lexicon_tags_30 },
    { 0x521670dd, SUC_lexicon_tags_15 },
    { 0x991670de, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7670e3, SUC_lexicon_tags_77 },
    { 0x123670e4, SUC_lexicon_tags_17 },
    { 0x201670e3, SUC_lexicon_tags_13 },
    { 0x327670e6, SUC_lexicon_tags_49 },
    { 0x743670e6, SUC_lexicon_tags_10 },
    { 0xe39670e6, SUC_lexicon_tags_3 },
    { 0xfa5670e6, SUC_lexicon_tags_7 },
    { 0xb73670e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x199670ed, SUC_lexicon_tags_19 },
    { 0x81670ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d670f1, SUC_lexicon_tags_33 },
    { 0xaef670f2, SUC_lexicon_tags_31 },
    { 0x537670f3, SUC_lexicon_tags_7 },
    { 0xe73670f2, SUC_lexicon_tags_33 },
    { 0xbf3670f2, SUC_lexicon_tags_33 },
    { 0x861670f3, SUC_lexicon_tags_8 },
    { 0x73f670f7, SUC_lexicon_tags_5 },
    { 0xe33670f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3670fb, SUC_lexicon_tags_60 },
    { 0x99b670fb, SUC_lexicon_tags_15 },
    { 0x9e9670fd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb55670ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef67103, SUC_lexicon_tags_38 },
    { 0x41367103, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd8567106, SUC_lexicon_tags_11 },
    { 0xbf367107, SUC_lexicon_tags_37 },
    { 0x29d67108, SUC_lexicon_tags_0 },
    { 0xdbf67107, SUC_lexicon_tags_12 },
    { 0x4767108, SUC_lexicon_tags_15 },
    { 0x5356710b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8916710d, SUC_lexicon_tags_15 },
    { 0x8136710d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb367113, SUC_lexicon_tags_0 },
    { 0xfc767113, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a16711a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1976712a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e56712c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2836712e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47767130, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xecb67132, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56d67134, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e367140, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69567146, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x97f67148, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xff36714b, SUC_lexicon_tags_15 },
    { 0xd396714b, SUC_lexicon_tags_68 },
    { 0x6d76714b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf167150, SUC_lexicon_tags_7 },
    { 0x88567151, SUC_lexicon_tags_6 },
    { 0x4f767151, SUC_lexicon_tags_17 },
    { 0x41d67152, SUC_lexicon_tags_43 },
    { 0xb2f67154, SUC_lexicon_tags_11 },
    { 0x6eb67153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb67158, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f76715c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe8b6715e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1dd67160, SUC_lexicon_tags_28 },
    { 0x2af67160, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecf67163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2767167, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe6167169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d67171, SUC_lexicon_tags_11 },
    { 0xbd67171, SUC_lexicon_tags_37 },
    { 0x70b67173, SUC_lexicon_tags_16 },
    { 0xc3167171, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d67178, SUC_lexicon_tags_14 },
    { 0xbd367178, SUC_lexicon_tags_82 },
    { 0x30167179, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5567180, SUC_lexicon_tags_49 },
    { 0xd0f67181, SUC_lexicon_tags_19 },
    { 0x15767182, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf67186, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0967189, SUC_lexicon_tags_35 },
    { 0xf656718a, SUC_lexicon_tags_15 },
    { 0x22567189, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdf56718d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6496718f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad67199, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f16719e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa81671a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf671a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5671a9, SUC_lexicon_tags_2 },
    { 0x5d671aa, SUC_lexicon_tags_37 },
    { 0xee1671ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee3671ad, SUC_lexicon_tags_26 },
    { 0xe65671ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x83671b0, SUC_lexicon_tags_12 },
    { 0xfb671b1, SUC_lexicon_tags_12 },
    { 0x63671b1, SUC_lexicon_tags_49 },
    { 0xe8d671b3, SUC_lexicon_tags_37 },
    { 0x66b671b4, SUC_lexicon_tags_11 },
    { 0x31b671b4, SUC_lexicon_tags_25 },
    { 0x81671b6, SUC_lexicon_tags_15 },
    { 0x877671b2, SUC_lexicon_tags_9 },
    { 0x823671b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87671bb, SUC_lexicon_tags_68 },
    { 0xc0b671bc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3671c1, SUC_lexicon_tags_7 },
    { 0xba5671c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5671c6, SUC_lexicon_tags_17 },
    { 0xabf671c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70d671c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3671d1, SUC_lexicon_tags_1 },
    { 0x4f5671d2, SUC_lexicon_tags_28 },
    { 0xf6b671d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d671d7, SUC_lexicon_tags_55 },
    { 0x46b671d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f671de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b671e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3671e4, SUC_lexicon_tags_37 },
    { 0xce7671e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55671e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3fd671eb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f671f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5671f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b671fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef7671fd, SUC_lexicon_tags_6 },
    { 0xc71671fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38367200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9367203, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69367205, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x53967208, SUC_lexicon_tags_19 },
    { 0xf7167208, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb76720c, SUC_lexicon_tags_7 },
    { 0xb6b6720d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f67211, SUC_lexicon_tags_29 },
    { 0x36367211, SUC_lexicon_tags_29 },
    { 0xca167213, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f67217, SUC_lexicon_tags_2 },
    { 0xf9f67218, SUC_lexicon_tags_1 },
    { 0xcbb67218, SUC_lexicon_tags_27 },
    { 0xa396721a, SUC_lexicon_tags_15 },
    { 0x24f6721b, SUC_lexicon_tags_11 },
    { 0x3c56721c, SUC_lexicon_tags_6 },
    { 0xe376721a, SUC_lexicon_tags_25 },
    { 0xd3d6721e, SUC_lexicon_tags_55 },
    { 0xc696721a, SUC_lexicon_tags_29 },
    { 0xd6567220, SUC_lexicon_tags_46 },
    { 0x4167220, SUC_lexicon_tags_12 },
    { 0x5bf6721d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44167228, SUC_lexicon_tags_12 },
    { 0xd3d67229, SUC_lexicon_tags_48 },
    { 0x1df67228, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb036722c, SUC_lexicon_tags_47 },
    { 0xba56722d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x55367230, SUC_lexicon_tags_25 },
    { 0x2dd67231, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7367235, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5767238, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7236723c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b6723f, SUC_lexicon_tags_5 },
    { 0xdd76723f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2567243, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f67247, SUC_lexicon_tags_37 },
    { 0x99f67247, SUC_lexicon_tags_23 },
    { 0xe7367249, SUC_lexicon_tags_38 },
    { 0x27f6724a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2456724c, SUC_lexicon_tags_63 },
    { 0x6376724d, SUC_lexicon_tags_27 },
    { 0x776724c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5367252, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e567260, SUC_lexicon_tags_68 },
    { 0xef767261, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23967263, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x26967265, SUC_lexicon_tags_14 },
    { 0x4ed67266, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2eb67268, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab96726b, SUC_lexicon_tags_12 },
    { 0x4a56726c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb67274, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1b67276, SUC_lexicon_tags_37 },
    { 0x71b67277, SUC_lexicon_tags_29 },
    { 0x31367277, SUC_lexicon_tags_83 },
    { 0x37d67279, SUC_lexicon_tags_58 },
    { 0x26d6727a, SUC_lexicon_tags_1 },
    { 0x616727b, SUC_lexicon_tags_25 },
    { 0x6c36727a, SUC_lexicon_tags_26 },
    { 0xbc367279, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb367282, SUC_lexicon_tags_14 },
    { 0x51167283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80767285, SUC_lexicon_tags_13 },
    { 0x68367286, SUC_lexicon_tags_9 },
    { 0xd8567286, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bb67289, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6556728c, SUC_lexicon_tags_17 },
    { 0xc716728d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x43967290, SUC_lexicon_tags_17 },
    { 0x99567291, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f67295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc536729b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5cf6729d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5ed6729f, SUC_lexicon_tags_43 },
    { 0x529672a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb672a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d672ac, SUC_lexicon_tags_38 },
    { 0xdd672ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd672ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719672c2, SUC_lexicon_tags_1 },
    { 0x913672c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27f672c5, SUC_lexicon_tags_43 },
    { 0x9d672c6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b672c9, SUC_lexicon_tags_70 },
    { 0x981672ca, SUC_lexicon_tags_7 },
    { 0xa3d672ca, SUC_lexicon_tags_37 },
    { 0x9fd672ca, SUC_lexicon_tags_1 },
    { 0x97672cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5672d1, SUC_lexicon_tags_11 },
    { 0xa8d672d2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd672d8, SUC_lexicon_tags_23 },
    { 0x82d672d9, SUC_lexicon_tags_2 },
    { 0x955672d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3d672dc, SUC_lexicon_tags_28 },
    { 0x2f3672dd, SUC_lexicon_tags_33 },
    { 0xb89672de, SUC_lexicon_tags_28 },
    { 0x797672de, SUC_lexicon_tags_29 },
    { 0x4af672df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd4f672e2, SUC_lexicon_tags_50 },
    { 0x4f7672e3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9672e6, SUC_lexicon_tags_57 },
    { 0x2db672e7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7672ef, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3e7672f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7672f3, SUC_lexicon_tags_25 },
    { 0xa31672f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2bd672f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xced672f8, SUC_lexicon_tags_61 },
    { 0x179672f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f672ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfa367301, SUC_lexicon_tags_39 },
    { 0x9db67302, SUC_lexicon_tags_37 },
    { 0xca967303, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f67306, SUC_lexicon_tags_37 },
    { 0x2d567307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30767309, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd6730d, SUC_lexicon_tags_15 },
    { 0x1d76730e, SUC_lexicon_tags_12 },
    { 0xfa76730f, SUC_lexicon_tags_25 },
    { 0x69f67310, SUC_lexicon_tags_17 },
    { 0x3b56730f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xd6367313, SUC_lexicon_tags_10 },
    { 0xb6f67314, SUC_lexicon_tags_70 },
    { 0x3367313, SUC_lexicon_tags_47 },
    { 0x85167314, SUC_lexicon_tags_17 },
    { 0x93767317, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b36731c, SUC_lexicon_tags_932 },
    { 0x1376731d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4567321, SUC_lexicon_tags_6 },
    { 0x6e767321, SUC_lexicon_tags_34 },
    { 0xf9567323, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6767327, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c36732e, SUC_lexicon_tags_1 },
    { 0x6576732e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf767336, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f67339, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3776733e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80967340, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f67349, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa736734b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4f6734d, SUC_lexicon_tags_1 },
    { 0x3d96734e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b167354, SUC_lexicon_tags_11 },
    { 0xbb767355, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd6735c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b967360, SUC_lexicon_tags_60 },
    { 0xfed67361, SUC_lexicon_tags_26 },
    { 0xa8f67362, SUC_lexicon_tags_25 },
    { 0x71f67363, SUC_lexicon_tags_0 },
    { 0xe3567364, SUC_lexicon_tags_17 },
    { 0x7e167362, SUC_lexicon_tags_2 },
    { 0x75b67361, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdd67368, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe8b6736a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b6736f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x53767371, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4767374, SUC_lexicon_tags_7 },
    { 0xbe567374, SUC_lexicon_tags_15 },
    { 0x85f67376, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb67379, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f6737c, SUC_lexicon_tags_1 },
    { 0xd696737d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb236737f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d67383, SUC_lexicon_tags_12 },
    { 0xcd767384, SUC_lexicon_tags_37 },
    { 0x20967383, SUC_lexicon_tags_11 },
    { 0x6fd67386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f6738c, SUC_lexicon_tags_37 },
    { 0xba76738d, SUC_lexicon_tags_18 },
    { 0xa596738e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x95b67390, SUC_lexicon_tags_17 },
    { 0xfab67391, SUC_lexicon_tags_4 },
    { 0xd5d67392, SUC_lexicon_tags_61 },
    { 0x55d67393, SUC_lexicon_tags_19 },
    { 0xf4567393, SUC_lexicon_tags_49 },
    { 0x66167395, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab16739c, SUC_lexicon_tags_68 },
    { 0xf776739d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6b6739f, SUC_lexicon_tags_32 },
    { 0xe31673a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x623673a3, SUC_lexicon_tags_1020 },
    { 0, NULL },
    { 0xd7f673a5, SUC_lexicon_tags_17 },
    { 0x327673a6, SUC_lexicon_tags_45 },
    { 0x795673a5, SUC_lexicon_tags_68 },
    { 0x6bd673a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd673ae, SUC_lexicon_tags_17 },
    { 0x9eb673ae, SUC_lexicon_tags_33 },
    { 0xb7673b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35673b3, SUC_lexicon_tags_37 },
    { 0x67d673b4, SUC_lexicon_tags_23 },
    { 0x799673b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69673b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9673bc, SUC_lexicon_tags_60 },
    { 0xd0f673bd, SUC_lexicon_tags_17 },
    { 0x9a3673bd, SUC_lexicon_tags_15 },
    { 0xe53673bd, SUC_lexicon_tags_62 },
    { 0xbc1673be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x823673c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f9673c4, SUC_lexicon_tags_19 },
    { 0xfcd673c4, SUC_lexicon_tags_60 },
    { 0xc41673c6, SUC_lexicon_tags_11 },
    { 0x311673c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x219673c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9673cc, SUC_lexicon_tags_216 },
    { 0xe5f673cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x943673cf, SUC_lexicon_tags_17 },
    { 0x693673d0, SUC_lexicon_tags_250 },
    { 0x899673d1, SUC_lexicon_tags_2 },
    { 0x25d673d1, SUC_lexicon_tags_11 },
    { 0x8a3673d3, SUC_lexicon_tags_37 },
    { 0x731673d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5673db, SUC_lexicon_tags_7 },
    { 0x6af673dc, SUC_lexicon_tags_6 },
    { 0x135673dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871673e1, SUC_lexicon_tags_2 },
    { 0xdbb673e2, SUC_lexicon_tags_15 },
    { 0x96d673e3, SUC_lexicon_tags_17 },
    { 0xab9673e4, SUC_lexicon_tags_46 },
    { 0xecd673e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbab673e7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5673f0, SUC_lexicon_tags_87 },
    { 0xa89673f1, SUC_lexicon_tags_15 },
    { 0x6b1673f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x393673f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11f673f6, SUC_lexicon_tags_2 },
    { 0x139673f7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x445673f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x673673fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef673ff, SUC_lexicon_tags_15 },
    { 0x71367400, SUC_lexicon_tags_17 },
    { 0x28367401, SUC_lexicon_tags_29 },
    { 0xe3167401, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d367407, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x236740a, SUC_lexicon_tags_17 },
    { 0xdb56740a, SUC_lexicon_tags_45 },
    { 0xead6740c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a167410, SUC_lexicon_tags_10 },
    { 0xe7167410, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb967413, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a167417, SUC_lexicon_tags_2 },
    { 0x1c367418, SUC_lexicon_tags_16 },
    { 0x5f67419, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a76741f, SUC_lexicon_tags_17 },
    { 0x71767420, SUC_lexicon_tags_12 },
    { 0x83f67420, SUC_lexicon_tags_45 },
    { 0xd6967421, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78967426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fd67428, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c96742c, SUC_lexicon_tags_38 },
    { 0x3036742d, SUC_lexicon_tags_6 },
    { 0x5556742e, SUC_lexicon_tags_19 },
    { 0x92d6742c, SUC_lexicon_tags_3 },
    { 0xf1d67430, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd767438, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x65f6743a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa316743d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f67446, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x14567449, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956744d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xedb6744f, SUC_lexicon_tags_25 },
    { 0xfe967450, SUC_lexicon_tags_37 },
    { 0x4dd67451, SUC_lexicon_tags_25 },
    { 0x9b567452, SUC_lexicon_tags_45 },
    { 0xc5167452, SUC_lexicon_tags_11 },
    { 0x35567454, SUC_lexicon_tags_2 },
    { 0xbe767452, SUC_lexicon_tags_7 },
    { 0xd8967452, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6967458, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb6745d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb56745f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d67463, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d67466, SUC_lexicon_tags_49 },
    { 0x6c967467, SUC_lexicon_tags_37 },
    { 0xdc167468, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc436746a, SUC_lexicon_tags_4 },
    { 0xc236746b, SUC_lexicon_tags_2 },
    { 0x3d16746a, SUC_lexicon_tags_15 },
    { 0x2576746c, SUC_lexicon_tags_7 },
    { 0xb1f6746e, SUC_lexicon_tags_17 },
    { 0x2e56746f, SUC_lexicon_tags_16 },
    { 0x5567470, SUC_lexicon_tags_38 },
    { 0xa576746f, SUC_lexicon_tags_7 },
    { 0xb3567472, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd367474, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb67478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed6747b, SUC_lexicon_tags_7 },
    { 0x6576747c, SUC_lexicon_tags_63 },
    { 0xb4f6747d, SUC_lexicon_tags_30 },
    { 0xe9f6747e, SUC_lexicon_tags_43 },
    { 0xdeb6747f, SUC_lexicon_tags_15 },
    { 0x4876747b, SUC_lexicon_tags_19 },
    { 0x45367481, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xea167483, SUC_lexicon_tags_17 },
    { 0xd5b67484, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77167486, SUC_lexicon_tags_37 },
    { 0xfb767487, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf316748c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf636748e, SUC_lexicon_tags_26 },
    { 0x7eb6748f, SUC_lexicon_tags_25 },
    { 0x3ab67490, SUC_lexicon_tags_16 },
    { 0x9ab67491, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd67499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd6749e, SUC_lexicon_tags_25 },
    { 0xfe56749f, SUC_lexicon_tags_45 },
    { 0x8b9674a0, SUC_lexicon_tags_47 },
    { 0x16f674a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1674a6, SUC_lexicon_tags_6 },
    { 0xc41674a6, SUC_lexicon_tags_7 },
    { 0x791674a8, SUC_lexicon_tags_45 },
    { 0xe75674a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819674ad, SUC_lexicon_tags_14 },
    { 0x17674ad, SUC_lexicon_tags_57 },
    { 0x7d1674af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x197674b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b674b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd674b9, SUC_lexicon_tags_9 },
    { 0x15b674b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1674bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11674c0, SUC_lexicon_tags_7 },
    { 0x2b5674c1, SUC_lexicon_tags_13 },
    { 0x55674c1, SUC_lexicon_tags_11 },
    { 0x615674c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f674c6, SUC_lexicon_tags_43 },
    { 0x579674c7, SUC_lexicon_tags_10 },
    { 0x9dd674c8, SUC_lexicon_tags_35 },
    { 0x5d674c8, SUC_lexicon_tags_12 },
    { 0x1ed674ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3674ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb674d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53674d4, SUC_lexicon_tags_1 },
    { 0x753674d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x521674d8, SUC_lexicon_tags_65 },
    { 0x17b674d8, SUC_lexicon_tags_4 },
    { 0xe8b674d9, SUC_lexicon_tags_43 },
    { 0xa0b674db, SUC_lexicon_tags_2 },
    { 0x6ff674db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43674e1, SUC_lexicon_tags_0 },
    { 0x39b674e2, SUC_lexicon_tags_12 },
    { 0x51674e3, SUC_lexicon_tags_56 },
    { 0x3db674e4, SUC_lexicon_tags_25 },
    { 0xa3d674e5, SUC_lexicon_tags_60 },
    { 0x8b5674e5, SUC_lexicon_tags_2 },
    { 0x88f674e7, SUC_lexicon_tags_25 },
    { 0xab5674e8, SUC_lexicon_tags_2 },
    { 0x381674e9, SUC_lexicon_tags_25 },
    { 0xb0f674ea, SUC_lexicon_tags_37 },
    { 0xd5b674ea, SUC_lexicon_tags_7 },
    { 0x735674e1, SUC_lexicon_tags_27 },
    { 0x68d674ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f674f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5ed674f3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7674f7, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9674fc, SUC_lexicon_tags_34 },
    { 0xd45674fd, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d367502, SUC_lexicon_tags_7 },
    { 0x7bb67502, SUC_lexicon_tags_25 },
    { 0x14d67503, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7567507, SUC_lexicon_tags_63 },
    { 0xa7f67508, SUC_lexicon_tags_37 },
    { 0x967509, SUC_lexicon_tags_7 },
    { 0x65367509, SUC_lexicon_tags_7 },
    { 0x1696750b, SUC_lexicon_tags_66 },
    { 0xac76750c, SUC_lexicon_tags_44 },
    { 0x7fd6750b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd67510, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb67515, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ed67517, SUC_lexicon_tags_14 },
    { 0x63b67518, SUC_lexicon_tags_13 },
    { 0x3f967518, SUC_lexicon_tags_25 },
    { 0x5dd67519, SUC_lexicon_tags_15 },
    { 0xea76751a, SUC_lexicon_tags_15 },
    { 0xde96751c, SUC_lexicon_tags_91 },
    { 0xa516751d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x82167520, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d567525, SUC_lexicon_tags_63 },
    { 0xaeb67526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4536752b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df67535, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2496753c, SUC_lexicon_tags_56 },
    { 0x37d6753c, SUC_lexicon_tags_1 },
    { 0xe2d6753e, SUC_lexicon_tags_9 },
    { 0x5696753f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad967545, SUC_lexicon_tags_87 },
    { 0xeab67546, SUC_lexicon_tags_9 },
    { 0x3d167546, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd816754b, SUC_lexicon_tags_17 },
    { 0x5016754c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d67550, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4a367552, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1ef67554, SUC_lexicon_tags_17 },
    { 0x31367554, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f67558, SUC_lexicon_tags_16 },
    { 0x98767558, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8136755d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af67560, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49967562, SUC_lexicon_tags_1 },
    { 0x2cf67563, SUC_lexicon_tags_87 },
    { 0x62567564, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca767566, SUC_lexicon_tags_38 },
    { 0x65b67567, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4db67569, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db6756f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf67572, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4167575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff567579, SUC_lexicon_tags_13 },
    { 0xf896757a, SUC_lexicon_tags_11 },
    { 0xe2f6757b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7536757d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3db6757f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b767583, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1167585, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b367589, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65567593, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f6759c, SUC_lexicon_tags_25 },
    { 0xbcf6759d, SUC_lexicon_tags_27 },
    { 0xf276759e, SUC_lexicon_tags_60 },
    { 0xe916759c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0d675a1, SUC_lexicon_tags_37 },
    { 0xb7675a1, SUC_lexicon_tags_16 },
    { 0xebd675a3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x9b3675a5, SUC_lexicon_tags_25 },
    { 0xbb9675a5, SUC_lexicon_tags_19 },
    { 0x573675a6, SUC_lexicon_tags_7 },
    { 0x867675a8, SUC_lexicon_tags_11 },
    { 0xd4d675a6, SUC_lexicon_tags_17 },
    { 0xcdd675aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195675ae, SUC_lexicon_tags_37 },
    { 0x82d675af, SUC_lexicon_tags_61 },
    { 0x725675ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ff675b2, SUC_lexicon_tags_2 },
    { 0x303675b3, SUC_lexicon_tags_48 },
    { 0x381675b2, SUC_lexicon_tags_63 },
    { 0x675675b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x445675b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe27675b9, SUC_lexicon_tags_23 },
    { 0x711675ba, SUC_lexicon_tags_47 },
    { 0x63d675bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5f3675bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x32f675bf, SUC_lexicon_tags_1 },
    { 0x91b675c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd81675c2, SUC_lexicon_tags_31 },
    { 0x457675c2, SUC_lexicon_tags_29 },
    { 0xd49675c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1675c8, SUC_lexicon_tags_12 },
    { 0x297675c9, SUC_lexicon_tags_11 },
    { 0x265675ca, SUC_lexicon_tags_12 },
    { 0x713675cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3675d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20f675d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedd675da, SUC_lexicon_tags_11 },
    { 0xf3d675db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x343675dd, SUC_lexicon_tags_17 },
    { 0xa67675dd, SUC_lexicon_tags_39 },
    { 0xb59675df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a9675e1, SUC_lexicon_tags_2 },
    { 0xbcb675e2, SUC_lexicon_tags_37 },
    { 0x76b675e2, SUC_lexicon_tags_17 },
    { 0xa19675e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7675e8, SUC_lexicon_tags_25 },
    { 0xf63675e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ad675eb, SUC_lexicon_tags_12 },
    { 0x333675ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e1675ee, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa77675f0, SUC_lexicon_tags_11 },
    { 0xa3b675f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x887675f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59675f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab675fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f67601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11767603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc836760b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2676760e, SUC_lexicon_tags_11 },
    { 0x8836760e, SUC_lexicon_tags_32 },
    { 0xa196760f, SUC_lexicon_tags_106 },
    { 0xde767611, SUC_lexicon_tags_47 },
    { 0x7ad6760e, SUC_lexicon_tags_343 },
    { 0x2bd67613, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11167615, SUC_lexicon_tags_31 },
    { 0x1967616, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d6761b, SUC_lexicon_tags_27 },
    { 0x6ef6761b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x86b6761e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xced67621, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe167623, SUC_lexicon_tags_28 },
    { 0x29367624, SUC_lexicon_tags_3 },
    { 0xf4967625, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6762c, SUC_lexicon_tags_48 },
    { 0x2c16762d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6476762f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e367633, SUC_lexicon_tags_8 },
    { 0x95967634, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x66367636, SUC_lexicon_tags_17 },
    { 0x75367636, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf6763e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdc967640, SUC_lexicon_tags_15 },
    { 0x35d67641, SUC_lexicon_tags_14 },
    { 0x19b67640, SUC_lexicon_tags_3 },
    { 0x2c567640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf67649, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8956764f, SUC_lexicon_tags_34 },
    { 0xa8d6764f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7567656, SUC_lexicon_tags_108 },
    { 0x82367657, SUC_lexicon_tags_47 },
    { 0x3367658, SUC_lexicon_tags_15 },
    { 0x5b967659, SUC_lexicon_tags_22 },
    { 0x8f967659, SUC_lexicon_tags_27 },
    { 0x20b6765b, SUC_lexicon_tags_2 },
    { 0x62f6765c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4776765f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d67662, SUC_lexicon_tags_11 },
    { 0xd4367663, SUC_lexicon_tags_15 },
    { 0x8e567664, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c367668, SUC_lexicon_tags_1 },
    { 0x78f67669, SUC_lexicon_tags_29 },
    { 0xff67668, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f167674, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b67678, SUC_lexicon_tags_33 },
    { 0x7967679, SUC_lexicon_tags_22 },
    { 0x296767a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7596767c, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d67680, SUC_lexicon_tags_114 },
    { 0xc9767680, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5567688, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31767693, SUC_lexicon_tags_2 },
    { 0x1f67694, SUC_lexicon_tags_17 },
    { 0x23167695, SUC_lexicon_tags_25 },
    { 0x97f67695, SUC_lexicon_tags_17 },
    { 0xb0d67697, SUC_lexicon_tags_50 },
    { 0x8167697, SUC_lexicon_tags_33 },
    { 0x8167698, SUC_lexicon_tags_29 },
    { 0xc7f6769a, SUC_lexicon_tags_9 },
    { 0x43b6769b, SUC_lexicon_tags_37 },
    { 0xb036769b, SUC_lexicon_tags_12 },
    { 0x796769c, SUC_lexicon_tags_109 },
    { 0xd296769e, SUC_lexicon_tags_38 },
    { 0x8756769e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa79676a1, SUC_lexicon_tags_65 },
    { 0x543676a2, SUC_lexicon_tags_2 },
    { 0x955676a3, SUC_lexicon_tags_17 },
    { 0xdf5676a4, SUC_lexicon_tags_42 },
    { 0x1e7676a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x261676a7, SUC_lexicon_tags_6 },
    { 0x973676a8, SUC_lexicon_tags_68 },
    { 0xc07676a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b676ae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87676b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b676b6, SUC_lexicon_tags_49 },
    { 0x4ad676b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31676ba, SUC_lexicon_tags_15 },
    { 0xf1b676bb, SUC_lexicon_tags_37 },
    { 0xa4f676bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f676bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709676c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc9676c6, SUC_lexicon_tags_27 },
    { 0xd6b676c7, SUC_lexicon_tags_2 },
    { 0x929676c6, SUC_lexicon_tags_17 },
    { 0xd49676c9, SUC_lexicon_tags_427 },
    { 0xd07676c9, SUC_lexicon_tags_1 },
    { 0x6b9676cb, SUC_lexicon_tags_2 },
    { 0xf31676cc, SUC_lexicon_tags_49 },
    { 0xaeb676cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb676d0, SUC_lexicon_tags_6 },
    { 0xddb676d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5676d9, SUC_lexicon_tags_32 },
    { 0xab1676da, SUC_lexicon_tags_25 },
    { 0xd7d676da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55676de, SUC_lexicon_tags_31 },
    { 0x69b676df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb1676e1, SUC_lexicon_tags_7 },
    { 0x649676e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27676e5, SUC_lexicon_tags_21 },
    { 0x349676e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b676eb, SUC_lexicon_tags_27 },
    { 0x523676ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd676f0, SUC_lexicon_tags_7 },
    { 0x83b676f1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb7b676f3, SUC_lexicon_tags_9 },
    { 0x77f676f4, SUC_lexicon_tags_49 },
    { 0xd8b676f5, SUC_lexicon_tags_12 },
    { 0xecf676f5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d676f9, SUC_lexicon_tags_15 },
    { 0xda1676fa, SUC_lexicon_tags_11 },
    { 0x223676fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f5676fd, SUC_lexicon_tags_44 },
    { 0x849676fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb516770d, SUC_lexicon_tags_11 },
    { 0x8336770d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4967713, SUC_lexicon_tags_17 },
    { 0xce567713, SUC_lexicon_tags_38 },
    { 0xf5167713, SUC_lexicon_tags_37 },
    { 0x8db67716, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b67719, SUC_lexicon_tags_37 },
    { 0xd4d6771a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd6771d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x88967720, SUC_lexicon_tags_16 },
    { 0x39567721, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54367728, SUC_lexicon_tags_11 },
    { 0xf6d67728, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d6772d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f67736, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b6773f, SUC_lexicon_tags_27 },
    { 0x61567740, SUC_lexicon_tags_45 },
    { 0x47b67741, SUC_lexicon_tags_17 },
    { 0x5b767741, SUC_lexicon_tags_1 },
    { 0xf9d67743, SUC_lexicon_tags_53 },
    { 0x63167742, SUC_lexicon_tags_17 },
    { 0x5b567744, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcaf67747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd16774a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbf96774c, SUC_lexicon_tags_4 },
    { 0xbd16774d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x39167750, SUC_lexicon_tags_15 },
    { 0xb2567751, SUC_lexicon_tags_15 },
    { 0x3b367750, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe6767754, SUC_lexicon_tags_17 },
    { 0x7f67755, SUC_lexicon_tags_11 },
    { 0x57767756, SUC_lexicon_tags_82 },
    { 0x3167757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f6775d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe167761, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42767766, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56776a, SUC_lexicon_tags_16 },
    { 0x996776a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b16776d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5767773, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10b67775, SUC_lexicon_tags_37 },
    { 0x78d67776, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f67779, SUC_lexicon_tags_21 },
    { 0xa716777a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5176777d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49767786, SUC_lexicon_tags_12 },
    { 0x17b67787, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6eb67789, SUC_lexicon_tags_1 },
    { 0x3d767789, SUC_lexicon_tags_19 },
    { 0xbc96778b, SUC_lexicon_tags_15 },
    { 0xe056778c, SUC_lexicon_tags_13 },
    { 0x1b567789, SUC_lexicon_tags_25 },
    { 0xc016778e, SUC_lexicon_tags_11 },
    { 0x6e16778c, SUC_lexicon_tags_6 },
    { 0x7d967790, SUC_lexicon_tags_53 },
    { 0x9a767791, SUC_lexicon_tags_37 },
    { 0xc7d67791, SUC_lexicon_tags_11 },
    { 0x967793, SUC_lexicon_tags_19 },
    { 0xe9367793, SUC_lexicon_tags_39 },
    { 0x2a967795, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22967799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e96779c, SUC_lexicon_tags_7 },
    { 0xe3f6779d, SUC_lexicon_tags_30 },
    { 0x30f6779c, SUC_lexicon_tags_25 },
    { 0xd336779d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x377677a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc25677a4, SUC_lexicon_tags_14 },
    { 0xae7677a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3677ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159677b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27677b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc9b677ba, SUC_lexicon_tags_21 },
    { 0xcd5677ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb677be, SUC_lexicon_tags_91 },
    { 0xcbb677bf, SUC_lexicon_tags_2 },
    { 0xc61677bf, SUC_lexicon_tags_43 },
    { 0x245677c1, SUC_lexicon_tags_1 },
    { 0x723677c2, SUC_lexicon_tags_2 },
    { 0xfaf677c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93677c6, SUC_lexicon_tags_25 },
    { 0x797677c6, SUC_lexicon_tags_7 },
    { 0x49677c8, SUC_lexicon_tags_17 },
    { 0xe8d677c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f677cc, SUC_lexicon_tags_7 },
    { 0x627677cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6cd677cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf677d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75677d9, SUC_lexicon_tags_26 },
    { 0x7f5677da, SUC_lexicon_tags_31 },
    { 0x675677db, SUC_lexicon_tags_11 },
    { 0x671677dc, SUC_lexicon_tags_2 },
    { 0xf85677dc, SUC_lexicon_tags_17 },
    { 0x65d677db, SUC_lexicon_tags_15 },
    { 0x1b1677de, SUC_lexicon_tags_12 },
    { 0x84d677e0, SUC_lexicon_tags_1 },
    { 0x77d677dc, SUC_lexicon_tags_12 },
    { 0xe3d677e2, SUC_lexicon_tags_37 },
    { 0xdf9677e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff677ea, SUC_lexicon_tags_2 },
    { 0x397677eb, SUC_lexicon_tags_7 },
    { 0x6fb677eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d677ef, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x895677f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553677f5, SUC_lexicon_tags_25 },
    { 0x7bb677f6, SUC_lexicon_tags_12 },
    { 0xc49677f7, SUC_lexicon_tags_79 },
    { 0xc0b677f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf5d677fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5677fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x405677ff, SUC_lexicon_tags_0 },
    { 0x2d767800, SUC_lexicon_tags_7 },
    { 0x6f3677ff, SUC_lexicon_tags_7 },
    { 0xacf67801, SUC_lexicon_tags_14 },
    { 0x50967803, SUC_lexicon_tags_33 },
    { 0xf37677ff, SUC_lexicon_tags_17 },
    { 0xa9f67805, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f6780a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b6780f, SUC_lexicon_tags_28 },
    { 0xfcd6780f, SUC_lexicon_tags_2 },
    { 0x2cf6780f, SUC_lexicon_tags_25 },
    { 0x71367812, SUC_lexicon_tags_1 },
    { 0xb8367812, SUC_lexicon_tags_2 },
    { 0xa9567813, SUC_lexicon_tags_45 },
    { 0xf9d67815, SUC_lexicon_tags_45 },
    { 0x86167816, SUC_lexicon_tags_9 },
    { 0x59f67817, SUC_lexicon_tags_25 },
    { 0xd3367818, SUC_lexicon_tags_49 },
    { 0x1767818, SUC_lexicon_tags_63 },
    { 0x88367818, SUC_lexicon_tags_4 },
    { 0x5156781b, SUC_lexicon_tags_55 },
    { 0xc0f6781b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d6781f, SUC_lexicon_tags_37 },
    { 0x90d6781f, SUC_lexicon_tags_17 },
    { 0xf4f67821, SUC_lexicon_tags_17 },
    { 0x64367822, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83767827, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79167829, SUC_lexicon_tags_2 },
    { 0x43d6782a, SUC_lexicon_tags_1 },
    { 0x396782b, SUC_lexicon_tags_25 },
    { 0xb8f6782b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1836782e, SUC_lexicon_tags_18 },
    { 0x2e16782f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe0367831, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5967833, SUC_lexicon_tags_47 },
    { 0xf2967834, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d67837, SUC_lexicon_tags_60 },
    { 0xecd67837, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4116783b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8716783d, SUC_lexicon_tags_11 },
    { 0x8bd6783d, SUC_lexicon_tags_45 },
    { 0x58b6783f, SUC_lexicon_tags_60 },
    { 0x57f6783d, SUC_lexicon_tags_46 },
    { 0xbb967841, SUC_lexicon_tags_12 },
    { 0x25367840, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd67848, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1a96784a, SUC_lexicon_tags_7 },
    { 0xd036784b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5776784e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94d67850, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b67857, SUC_lexicon_tags_37 },
    { 0x8a367857, SUC_lexicon_tags_5 },
    { 0x9cb67857, SUC_lexicon_tags_13 },
    { 0x4136785a, SUC_lexicon_tags_5 },
    { 0x32b6785b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69f6785d, SUC_lexicon_tags_19 },
    { 0xb256785e, SUC_lexicon_tags_37 },
    { 0x6876785f, SUC_lexicon_tags_23 },
    { 0x60f6785f, SUC_lexicon_tags_17 },
    { 0xe5767860, SUC_lexicon_tags_151 },
    { 0xb4967862, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf67865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6996786c, SUC_lexicon_tags_11 },
    { 0xfe96786d, SUC_lexicon_tags_53 },
    { 0x2756786d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c767870, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d567874, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6dd67876, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x60567879, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3596787b, SUC_lexicon_tags_17 },
    { 0x716787b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5b6787e, SUC_lexicon_tags_13 },
    { 0x3b56787e, SUC_lexicon_tags_17 },
    { 0xa0367880, SUC_lexicon_tags_11 },
    { 0x94367881, SUC_lexicon_tags_19 },
    { 0xfe167882, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78767887, SUC_lexicon_tags_9 },
    { 0x38767887, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7767890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bb67892, SUC_lexicon_tags_60 },
    { 0x3e967893, SUC_lexicon_tags_17 },
    { 0xc9167894, SUC_lexicon_tags_3 },
    { 0x5dd67894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65767898, SUC_lexicon_tags_38 },
    { 0xc567899, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d56789d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2f6789f, SUC_lexicon_tags_88 },
    { 0x643678a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69678a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd19678a6, SUC_lexicon_tags_11 },
    { 0xab1678a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57678ab, SUC_lexicon_tags_10 },
    { 0x2c9678ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d678af, SUC_lexicon_tags_45 },
    { 0x627678b0, SUC_lexicon_tags_7 },
    { 0x2f1678b1, SUC_lexicon_tags_47 },
    { 0x7a1678b2, SUC_lexicon_tags_65 },
    { 0xd33678b0, SUC_lexicon_tags_25 },
    { 0x55678b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d678b7, SUC_lexicon_tags_16 },
    { 0xd17678b8, SUC_lexicon_tags_37 },
    { 0x63f678b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x141678bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31f678bd, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b678c1, SUC_lexicon_tags_38 },
    { 0x459678c1, SUC_lexicon_tags_17 },
    { 0x14d678c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a9678c5, SUC_lexicon_tags_39 },
    { 0xad5678c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537678ca, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39678da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1678dd, SUC_lexicon_tags_17 },
    { 0x96f678dd, SUC_lexicon_tags_2 },
    { 0x8d5678dd, SUC_lexicon_tags_8 },
    { 0x171678de, SUC_lexicon_tags_62 },
    { 0x949678de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175678e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7678ea, SUC_lexicon_tags_68 },
    { 0x851678eb, SUC_lexicon_tags_28 },
    { 0x757678ec, SUC_lexicon_tags_68 },
    { 0x293678ed, SUC_lexicon_tags_7 },
    { 0xa27678ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d678f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff678f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b678fb, SUC_lexicon_tags_2 },
    { 0xfdf678fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38167900, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff167903, SUC_lexicon_tags_14 },
    { 0x56367904, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab6790a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xe176790c, SUC_lexicon_tags_59 },
    { 0x1c36790d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9567911, SUC_lexicon_tags_7 },
    { 0x75367911, SUC_lexicon_tags_749 },
    { 0x3ef67913, SUC_lexicon_tags_37 },
    { 0xebd67914, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b67917, SUC_lexicon_tags_49 },
    { 0xb9d67918, SUC_lexicon_tags_28 },
    { 0x7d767919, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b6791d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f67923, SUC_lexicon_tags_17 },
    { 0x9767924, SUC_lexicon_tags_10 },
    { 0x4dd67923, SUC_lexicon_tags_17 },
    { 0xf7b67926, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd796792a, SUC_lexicon_tags_45 },
    { 0x8ff6792b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f67931, SUC_lexicon_tags_25 },
    { 0xc7967931, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9167937, SUC_lexicon_tags_17 },
    { 0xe1f67938, SUC_lexicon_tags_47 },
    { 0xef167938, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb716793c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f567944, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b67949, SUC_lexicon_tags_29 },
    { 0x7df6794a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd6794e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe5167950, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa367952, SUC_lexicon_tags_0 },
    { 0xc2b67952, SUC_lexicon_tags_0 },
    { 0xd9167954, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4bd67956, SUC_lexicon_tags_2 },
    { 0xfcd67957, SUC_lexicon_tags_11 },
    { 0x77d67956, SUC_lexicon_tags_12 },
    { 0x1367959, SUC_lexicon_tags_10 },
    { 0xcf567956, SUC_lexicon_tags_32 },
    { 0xf5567956, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb176795d, SUC_lexicon_tags_37 },
    { 0x7c76795e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60167963, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb67966, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd376796b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b76796d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb36796f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86567977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf6797a, SUC_lexicon_tags_33 },
    { 0xd356797b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b16797e, SUC_lexicon_tags_8 },
    { 0xb7d6797f, SUC_lexicon_tags_55 },
    { 0xf0f67980, SUC_lexicon_tags_39 },
    { 0xa8567980, SUC_lexicon_tags_14 },
    { 0xa7167982, SUC_lexicon_tags_1 },
    { 0xaaf67983, SUC_lexicon_tags_97 },
    { 0xbf567984, SUC_lexicon_tags_7 },
    { 0xdb967985, SUC_lexicon_tags_11 },
    { 0x4b167985, SUC_lexicon_tags_79 },
    { 0x29567987, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f6798c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd676798e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd67998, SUC_lexicon_tags_12 },
    { 0x3f167999, SUC_lexicon_tags_61 },
    { 0x2f67999, SUC_lexicon_tags_16 },
    { 0x5c167999, SUC_lexicon_tags_27 },
    { 0xc376799a, SUC_lexicon_tags_12 },
    { 0x55b6799d, SUC_lexicon_tags_42 },
    { 0xa836799d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b679a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfd679a3, SUC_lexicon_tags_87 },
    { 0xa25679a3, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb679a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc01679a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33679ac, SUC_lexicon_tags_11 },
    { 0xca7679ad, SUC_lexicon_tags_37 },
    { 0xa3679ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271679b3, SUC_lexicon_tags_15 },
    { 0xbed679b4, SUC_lexicon_tags_32 },
    { 0x9d679b5, SUC_lexicon_tags_47 },
    { 0x27d679b6, SUC_lexicon_tags_25 },
    { 0xbff679b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbd679b9, SUC_lexicon_tags_25 },
    { 0x991679b9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1679be, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761679c4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23679ca, SUC_lexicon_tags_96 },
    { 0xc03679cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d679ce, SUC_lexicon_tags_57 },
    { 0xe83679ce, SUC_lexicon_tags_15 },
    { 0x3d7679d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919679d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94b679d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff679da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x817679dd, SUC_lexicon_tags_17 },
    { 0xf13679dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71f679e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41679e3, SUC_lexicon_tags_8 },
    { 0xdc1679e3, SUC_lexicon_tags_37 },
    { 0x35679e3, SUC_lexicon_tags_7 },
    { 0x7cb679e4, SUC_lexicon_tags_54 },
    { 0x3f9679e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x565679ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe0b679ec, SUC_lexicon_tags_37 },
    { 0xd71679ec, SUC_lexicon_tags_33 },
    { 0x66f679ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7679f1, SUC_lexicon_tags_37 },
    { 0xacb679f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x343679f5, SUC_lexicon_tags_7 },
    { 0x6f7679f6, SUC_lexicon_tags_23 },
    { 0x887679f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff679fa, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced679fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x23d67a00, SUC_lexicon_tags_49 },
    { 0x56567a00, SUC_lexicon_tags_11 },
    { 0xe4b67a02, SUC_lexicon_tags_11 },
    { 0x7fb67a02, SUC_lexicon_tags_39 },
    { 0xd4967a04, SUC_lexicon_tags_1 },
    { 0x3f167a05, SUC_lexicon_tags_7 },
    { 0xca367a06, SUC_lexicon_tags_13 },
    { 0x71367a06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x49367a09, SUC_lexicon_tags_87 },
    { 0x2cf67a0a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d967a0e, SUC_lexicon_tags_7 },
    { 0x45367a0f, SUC_lexicon_tags_17 },
    { 0x62567a0e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7cd67a12, SUC_lexicon_tags_43 },
    { 0x4d167a12, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e367a16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb767a19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d67a1f, SUC_lexicon_tags_14 },
    { 0xe1d67a20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f67a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15967a28, SUC_lexicon_tags_11 },
    { 0xc2167a29, SUC_lexicon_tags_39 },
    { 0x2a767a2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xded67a2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x72b67a2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad767a36, SUC_lexicon_tags_30 },
    { 0xa7167a36, SUC_lexicon_tags_43 },
    { 0x8f67a38, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3967a3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb67a40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4367a42, SUC_lexicon_tags_2 },
    { 0xbe367a43, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xa967a45, SUC_lexicon_tags_17 },
    { 0x1a567a45, SUC_lexicon_tags_7 },
    { 0xf8f67a47, SUC_lexicon_tags_68 },
    { 0xc5367a46, SUC_lexicon_tags_17 },
    { 0x38f67a46, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf367a4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f167a4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd67a52, SUC_lexicon_tags_2 },
    { 0xa8f67a52, SUC_lexicon_tags_2 },
    { 0x79767a52, SUC_lexicon_tags_12 },
    { 0x7f567a55, SUC_lexicon_tags_0 },
    { 0xa3167a54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab67a5b, SUC_lexicon_tags_33 },
    { 0x1b767a5b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3167a5f, SUC_lexicon_tags_6 },
    { 0xad367a60, SUC_lexicon_tags_17 },
    { 0x5a567a61, SUC_lexicon_tags_30 },
    { 0x46b67a61, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd67a65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd967a67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa567a69, SUC_lexicon_tags_11 },
    { 0xa0367a6a, SUC_lexicon_tags_17 },
    { 0x79967a6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70367a6d, SUC_lexicon_tags_15 },
    { 0xd6567a6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38567a70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ed67a72, SUC_lexicon_tags_38 },
    { 0xb2b67a73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62767a78, SUC_lexicon_tags_812 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f67a7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc967a84, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x83d67a86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f67a89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56367a8d, SUC_lexicon_tags_45 },
    { 0x51767a8e, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d67a92, SUC_lexicon_tags_84 },
    { 0xa7b67a93, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4f967a95, SUC_lexicon_tags_37 },
    { 0x20367a96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff567a9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59767a9e, SUC_lexicon_tags_33 },
    { 0x2b367a9f, SUC_lexicon_tags_37 },
    { 0x55567aa0, SUC_lexicon_tags_15 },
    { 0x69f67aa1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8567aa6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c767aa9, SUC_lexicon_tags_34 },
    { 0x6ff67aa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac967aad, SUC_lexicon_tags_7 },
    { 0x1c367aae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b67ab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d367ab6, SUC_lexicon_tags_45 },
    { 0xced67ab7, SUC_lexicon_tags_82 },
    { 0xcf67ab8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b67abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88167abf, SUC_lexicon_tags_21 },
    { 0x67967ac0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d67ac5, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb67aca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b67acd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7367ad4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd67ad9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf1167adb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd67ade, SUC_lexicon_tags_38 },
    { 0xdaf67adf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb67ae2, SUC_lexicon_tags_6 },
    { 0xa7d67ae2, SUC_lexicon_tags_2 },
    { 0x3d67ae4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef67ae9, SUC_lexicon_tags_37 },
    { 0x9bb67aea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81767af2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbd567af4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2567af7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x24967af9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf967aff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8767b01, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd967b06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e367b0a, SUC_lexicon_tags_48 },
    { 0x1b967b0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfad67b0d, SUC_lexicon_tags_17 },
    { 0x24d67b0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f67b14, SUC_lexicon_tags_60 },
    { 0x14167b15, SUC_lexicon_tags_14 },
    { 0x2fd67b15, SUC_lexicon_tags_21 },
    { 0xa0967b17, SUC_lexicon_tags_47 },
    { 0x41d67b18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d67b1c, SUC_lexicon_tags_84 },
    { 0xc1167b1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91567b20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68167b23, SUC_lexicon_tags_12 },
    { 0x56f67b24, SUC_lexicon_tags_37 },
    { 0x11167b23, SUC_lexicon_tags_37 },
    { 0x7167b26, SUC_lexicon_tags_47 },
    { 0xb9567b27, SUC_lexicon_tags_46 },
    { 0x44d67b27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b67b2b, SUC_lexicon_tags_28 },
    { 0x8f567b2b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5567b32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c967b34, SUC_lexicon_tags_37 },
    { 0x11367b35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1d67b37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31b67b39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9fb67b3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69367b40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99567b43, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe967b48, SUC_lexicon_tags_38 },
    { 0x5b167b49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a967b4e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b67b51, SUC_lexicon_tags_11 },
    { 0xa3f67b52, SUC_lexicon_tags_1 },
    { 0xf5367b53, SUC_lexicon_tags_5 },
    { 0x11767b53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cb67b56, SUC_lexicon_tags_17 },
    { 0x8ff67b57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb67b5e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d767b65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68367b6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f567b6f, SUC_lexicon_tags_1 },
    { 0xb4f67b70, SUC_lexicon_tags_11 },
    { 0x8e767b71, SUC_lexicon_tags_26 },
    { 0x37b67b72, SUC_lexicon_tags_7 },
    { 0x38f67b73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69167b77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2767b79, SUC_lexicon_tags_11 },
    { 0x2e567b7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3167b7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb767b80, SUC_lexicon_tags_15 },
    { 0xc1167b81, SUC_lexicon_tags_8 },
    { 0x2f767b82, SUC_lexicon_tags_37 },
    { 0xa3967b83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d67b87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6967b8f, SUC_lexicon_tags_6 },
    { 0x24567b8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x70567b93, SUC_lexicon_tags_7 },
    { 0x81f67b94, SUC_lexicon_tags_37 },
    { 0x7cd67b94, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb67b9a, SUC_lexicon_tags_16 },
    { 0x63967b9b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xced67b9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f967ba4, SUC_lexicon_tags_27 },
    { 0x38b67ba5, SUC_lexicon_tags_37 },
    { 0x35d67ba5, SUC_lexicon_tags_15 },
    { 0x27167ba4, SUC_lexicon_tags_2 },
    { 0xdb167ba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15967baa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x39567bad, SUC_lexicon_tags_37 },
    { 0x3b67bad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85567bb5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f67bb8, SUC_lexicon_tags_45 },
    { 0x81367bb9, SUC_lexicon_tags_30 },
    { 0x7e367bba, SUC_lexicon_tags_11 },
    { 0xdd767bb9, SUC_lexicon_tags_7 },
    { 0x18767bb9, SUC_lexicon_tags_46 },
    { 0xed367bbd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f567bc2, SUC_lexicon_tags_11 },
    { 0xfc367bc3, SUC_lexicon_tags_48 },
    { 0x8f767bc4, SUC_lexicon_tags_2 },
    { 0x6b567bc3, SUC_lexicon_tags_17 },
    { 0xef767bc2, SUC_lexicon_tags_29 },
    { 0x87767bc7, SUC_lexicon_tags_11 },
    { 0xdd567bc8, SUC_lexicon_tags_30 },
    { 0xdfb67bc9, SUC_lexicon_tags_7 },
    { 0xe2567bc9, SUC_lexicon_tags_25 },
    { 0xee567bc8, SUC_lexicon_tags_11 },
    { 0xb4d67bcc, SUC_lexicon_tags_1 },
    { 0x79367bc7, SUC_lexicon_tags_35 },
    { 0x6e767bce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5df67bd0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf8167bd2, SUC_lexicon_tags_39 },
    { 0xc6167bd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30767bd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3dd67bd7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd167bdf, SUC_lexicon_tags_25 },
    { 0xcc167be0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe367be2, SUC_lexicon_tags_31 },
    { 0x34167be3, SUC_lexicon_tags_55 },
    { 0x20d67be4, SUC_lexicon_tags_12 },
    { 0x7d567be2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d67be9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb67bf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5767bf3, SUC_lexicon_tags_47 },
    { 0x6f367bf4, SUC_lexicon_tags_11 },
    { 0xa6767bf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad67bfb, SUC_lexicon_tags_14 },
    { 0x5f567bfb, SUC_lexicon_tags_10 },
    { 0x38967bfb, SUC_lexicon_tags_87 },
    { 0x85d67bfc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25767c02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8967c09, SUC_lexicon_tags_37 },
    { 0xb67c0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d67c0d, SUC_lexicon_tags_7 },
    { 0x8ef67c0e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x15967c11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf767c1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b67c20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9167c22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22367c24, SUC_lexicon_tags_14 },
    { 0x7c967c25, SUC_lexicon_tags_15 },
    { 0x74567c26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd767c2e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa6f67c30, SUC_lexicon_tags_77 },
    { 0xa3f67c30, SUC_lexicon_tags_36 },
    { 0xa1367c32, SUC_lexicon_tags_14 },
    { 0x34967c32, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7cb67c35, SUC_lexicon_tags_19 },
    { 0xc3967c36, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d67c39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf567c3f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c967c44, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0767c4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4367c4f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf67c53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd67c56, SUC_lexicon_tags_49 },
    { 0xb8767c56, SUC_lexicon_tags_33 },
    { 0xd9967c58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58967c5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44967c62, SUC_lexicon_tags_26 },
    { 0xcbd67c63, SUC_lexicon_tags_25 },
    { 0x9f767c64, SUC_lexicon_tags_26 },
    { 0x98f67c63, SUC_lexicon_tags_1 },
    { 0x95767c66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7e767c68, SUC_lexicon_tags_7 },
    { 0x88967c69, SUC_lexicon_tags_12 },
    { 0x27f67c68, SUC_lexicon_tags_47 },
    { 0x2ef67c6b, SUC_lexicon_tags_27 },
    { 0x69b67c6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36d67c6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fd67c70, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf5f67c72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c167c75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8d67c77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab67c7b, SUC_lexicon_tags_398 },
    { 0x2eb67c7c, SUC_lexicon_tags_12 },
    { 0xcf67c7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9b67c7f, SUC_lexicon_tags_28 },
    { 0x3fb67c80, SUC_lexicon_tags_29 },
    { 0xb5f67c81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd767c8e, SUC_lexicon_tags_1 },
    { 0x4b367c8e, SUC_lexicon_tags_21 },
    { 0xbef67c90, SUC_lexicon_tags_77 },
    { 0x1e767c8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42167c93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8dd67c95, SUC_lexicon_tags_27 },
    { 0x97f67c95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f67c9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9767c9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3167c9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e367ca1, SUC_lexicon_tags_11 },
    { 0xde167ca2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9167ca6, SUC_lexicon_tags_35 },
    { 0xe3d67ca7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x66767caa, SUC_lexicon_tags_4 },
    { 0xcfb67cab, SUC_lexicon_tags_30 },
    { 0xfd367cac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x48567caf, SUC_lexicon_tags_7 },
    { 0xdcb67cb0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b167cb5, SUC_lexicon_tags_35 },
    { 0x41967cb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d367cb8, SUC_lexicon_tags_0 },
    { 0x73167cb9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x84f67cbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34167cc0, SUC_lexicon_tags_17 },
    { 0x85967cc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad67ccd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf67cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b67cd8, SUC_lexicon_tags_7 },
    { 0x81b67cd9, SUC_lexicon_tags_1 },
    { 0x9f967cd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f67cdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b967cdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6967ce2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b67ce7, SUC_lexicon_tags_77 },
    { 0x7e767ce8, SUC_lexicon_tags_31 },
    { 0x56167ce7, SUC_lexicon_tags_12 },
    { 0xdbb67cea, SUC_lexicon_tags_2 },
    { 0xc5d67cea, SUC_lexicon_tags_37 },
    { 0x5a567cea, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x37d67cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82967cf0, SUC_lexicon_tags_17 },
    { 0x25b67cf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64767cf3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82567cf7, SUC_lexicon_tags_29 },
    { 0xde367cf8, SUC_lexicon_tags_17 },
    { 0x22b67cf9, SUC_lexicon_tags_1 },
    { 0x2a967cfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28567cfc, SUC_lexicon_tags_7 },
    { 0xca767cfd, SUC_lexicon_tags_61 },
    { 0x43b67cfe, SUC_lexicon_tags_37 },
    { 0x19767cfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15967d01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d67d0b, SUC_lexicon_tags_7 },
    { 0xfd367d0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8167d10, SUC_lexicon_tags_14 },
    { 0xd4967d11, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9567d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f967d17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23767d1a, SUC_lexicon_tags_15 },
    { 0x1a567d1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52d67d1d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97567d26, SUC_lexicon_tags_2 },
    { 0xbdd67d27, SUC_lexicon_tags_37 },
    { 0xb2767d28, SUC_lexicon_tags_12 },
    { 0xb0167d29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d67d2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6567d34, SUC_lexicon_tags_6 },
    { 0x5cd67d35, SUC_lexicon_tags_8 },
    { 0xf0767d36, SUC_lexicon_tags_17 },
    { 0xf8767d37, SUC_lexicon_tags_0 },
    { 0x7bf67d37, SUC_lexicon_tags_29 },
    { 0xb67d38, SUC_lexicon_tags_27 },
    { 0x33767d3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab67d3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d67d45, SUC_lexicon_tags_37 },
    { 0xde367d46, SUC_lexicon_tags_17 },
    { 0xb3367d47, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xf167d49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf67d50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b67d53, SUC_lexicon_tags_7 },
    { 0x99b67d54, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x9ab67d56, SUC_lexicon_tags_0 },
    { 0x70367d57, SUC_lexicon_tags_9 },
    { 0xbc767d57, SUC_lexicon_tags_25 },
    { 0x3367d57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1967d5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x86b67d5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b67d65, SUC_lexicon_tags_37 },
    { 0xf4d67d66, SUC_lexicon_tags_165 },
    { 0x78367d67, SUC_lexicon_tags_30 },
    { 0x70167d68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e167d70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x76567d73, SUC_lexicon_tags_29 },
    { 0xea167d74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb67d79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b67d7c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf6967d7e, SUC_lexicon_tags_25 },
    { 0x89367d7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5967d85, SUC_lexicon_tags_1 },
    { 0x23767d86, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42767d93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73767d95, SUC_lexicon_tags_11 },
    { 0x77b67d96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4567d99, SUC_lexicon_tags_101 },
    { 0xb2f67d9a, SUC_lexicon_tags_37 },
    { 0x48967d9b, SUC_lexicon_tags_17 },
    { 0xd2567d9c, SUC_lexicon_tags_29 },
    { 0x14d67d9d, SUC_lexicon_tags_11 },
    { 0x69967d9c, SUC_lexicon_tags_15 },
    { 0x45167d9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6167da5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd167da9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd67dad, SUC_lexicon_tags_25 },
    { 0x9d567dae, SUC_lexicon_tags_1 },
    { 0x40f67daf, SUC_lexicon_tags_0 },
    { 0x48967daf, SUC_lexicon_tags_33 },
    { 0x10f67db1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4967db3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d67db6, SUC_lexicon_tags_30 },
    { 0x58d67db7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80567dbd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xba967dbf, SUC_lexicon_tags_6 },
    { 0x51767dc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7967dc6, SUC_lexicon_tags_129 },
    { 0x93967dc7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb67dca, SUC_lexicon_tags_25 },
    { 0x3c767dca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6967dd1, SUC_lexicon_tags_11 },
    { 0xee567dd1, SUC_lexicon_tags_2 },
    { 0xb7167dd3, SUC_lexicon_tags_37 },
    { 0x2b167dd1, SUC_lexicon_tags_26 },
    { 0x85967dd3, SUC_lexicon_tags_45 },
    { 0x23767dd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf67dda, SUC_lexicon_tags_17 },
    { 0xa6767ddb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e767de1, SUC_lexicon_tags_28 },
    { 0x7d167de2, SUC_lexicon_tags_15 },
    { 0x5f367de3, SUC_lexicon_tags_1 },
    { 0x31767de4, SUC_lexicon_tags_38 },
    { 0x7d67de5, SUC_lexicon_tags_82 },
    { 0x4a967de6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3367de9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d567deb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af67dee, SUC_lexicon_tags_63 },
    { 0x76b67dee, SUC_lexicon_tags_1 },
    { 0x6bb67df0, SUC_lexicon_tags_37 },
    { 0xf7f67df0, SUC_lexicon_tags_74 },
    { 0x90767df1, SUC_lexicon_tags_15 },
    { 0x46567df2, SUC_lexicon_tags_23 },
    { 0xa9b67df2, SUC_lexicon_tags_60 },
    { 0xea567df5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa7367df7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74b67df9, SUC_lexicon_tags_19 },
    { 0xbe367dfa, SUC_lexicon_tags_11 },
    { 0xde567df9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a367e01, SUC_lexicon_tags_12 },
    { 0x76f67e02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8567e05, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f67e08, SUC_lexicon_tags_37 },
    { 0x76967e09, SUC_lexicon_tags_6 },
    { 0xd8d67e0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6d67e0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xffd67e0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef67e15, SUC_lexicon_tags_19 },
    { 0xc567e16, SUC_lexicon_tags_21 },
    { 0x63967e17, SUC_lexicon_tags_38 },
    { 0xe9d67e18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f67e1b, SUC_lexicon_tags_11 },
    { 0xd6b67e1b, SUC_lexicon_tags_26 },
    { 0xae167e1d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d67e21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4df67e23, SUC_lexicon_tags_12 },
    { 0x3bb67e23, SUC_lexicon_tags_38 },
    { 0xe9d67e25, SUC_lexicon_tags_1 },
    { 0x69967e26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf67e28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeeb67e2a, SUC_lexicon_tags_25 },
    { 0xb3367e2b, SUC_lexicon_tags_12 },
    { 0x3ed67e2c, SUC_lexicon_tags_16 },
    { 0xf2367e2b, SUC_lexicon_tags_37 },
    { 0x64b67e2a, SUC_lexicon_tags_11 },
    { 0x4b767e2f, SUC_lexicon_tags_25 },
    { 0x8a767e2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db67e33, SUC_lexicon_tags_15 },
    { 0xbf767e34, SUC_lexicon_tags_11 },
    { 0xdb67e35, SUC_lexicon_tags_29 },
    { 0xeff67e36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9967e39, SUC_lexicon_tags_25 },
    { 0x59567e3a, SUC_lexicon_tags_39 },
    { 0x10567e39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7f67e3d, SUC_lexicon_tags_26 },
    { 0x71167e3e, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe167e49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2bd67e4b, SUC_lexicon_tags_7 },
    { 0x87567e4c, SUC_lexicon_tags_16 },
    { 0x8d67e4d, SUC_lexicon_tags_65 },
    { 0x9e767e4e, SUC_lexicon_tags_15 },
    { 0x36d67e4f, SUC_lexicon_tags_7 },
    { 0x35b67e50, SUC_lexicon_tags_17 },
    { 0xdfb67e51, SUC_lexicon_tags_11 },
    { 0x8df67e52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x70167e54, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a767e58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5967e5a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x22767e5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d67e62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb967e68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x89967e6a, SUC_lexicon_tags_2 },
    { 0x1d167e6a, SUC_lexicon_tags_47 },
    { 0x2df67e6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7967e70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeff67e72, SUC_lexicon_tags_29 },
    { 0x5d167e73, SUC_lexicon_tags_10 },
    { 0x96f67e74, SUC_lexicon_tags_0 },
    { 0xd0567e74, SUC_lexicon_tags_43 },
    { 0xee967e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b67e82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd167e86, SUC_lexicon_tags_65 },
    { 0x55f67e87, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d67e8e, SUC_lexicon_tags_15 },
    { 0xc0767e8e, SUC_lexicon_tags_12 },
    { 0x75167e90, SUC_lexicon_tags_1031 },
    { 0xcdb67e91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e167e97, SUC_lexicon_tags_4 },
    { 0xb3367e98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd67e9f, SUC_lexicon_tags_70 },
    { 0x4bb67e9f, SUC_lexicon_tags_107 },
    { 0x3cd67ea1, SUC_lexicon_tags_9 },
    { 0xcdb67ea2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbfd67ea4, SUC_lexicon_tags_0 },
    { 0x64f67ea5, SUC_lexicon_tags_31 },
    { 0xadf67ea6, SUC_lexicon_tags_2 },
    { 0xe9d67ea7, SUC_lexicon_tags_29 },
    { 0xae967ea6, SUC_lexicon_tags_23 },
    { 0x34767ea4, SUC_lexicon_tags_12 },
    { 0xb8167ea9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7567eae, SUC_lexicon_tags_7 },
    { 0x95367eaf, SUC_lexicon_tags_12 },
    { 0xf6d67eb0, SUC_lexicon_tags_11 },
    { 0x94967eb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b67eb5, SUC_lexicon_tags_1 },
    { 0x93967eb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd8f67eb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d67ebc, SUC_lexicon_tags_26 },
    { 0x11b67ebc, SUC_lexicon_tags_33 },
    { 0xbd67ebc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5367ec9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfa367ecb, SUC_lexicon_tags_9 },
    { 0xe5b67ecb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe567ece, SUC_lexicon_tags_63 },
    { 0x77967ece, SUC_lexicon_tags_6 },
    { 0xd4167ed0, SUC_lexicon_tags_65 },
    { 0x31f67ed0, SUC_lexicon_tags_11 },
    { 0x78567ece, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xbf167ed4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d67ed6, SUC_lexicon_tags_49 },
    { 0xe0f67ed7, SUC_lexicon_tags_14 },
    { 0x6f567ed7, SUC_lexicon_tags_15 },
    { 0xbf167ed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3567edc, SUC_lexicon_tags_16 },
    { 0xcf767edd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd67ee1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d67ee9, SUC_lexicon_tags_19 },
    { 0xc5b67eea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x18167eed, SUC_lexicon_tags_7 },
    { 0x43767eee, SUC_lexicon_tags_11 },
    { 0xe2b67eef, SUC_lexicon_tags_21 },
    { 0x79567ef0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x38767ef2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d67ef7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1967ef9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf567f02, SUC_lexicon_tags_11 },
    { 0x7567f02, SUC_lexicon_tags_37 },
    { 0xc6d67f04, SUC_lexicon_tags_19 },
    { 0x98967f03, SUC_lexicon_tags_1 },
    { 0x7e567f02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c967f0e, SUC_lexicon_tags_17 },
    { 0x75967f0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a167f15, SUC_lexicon_tags_47 },
    { 0x51d67f16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef567f18, SUC_lexicon_tags_68 },
    { 0x1d367f19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd67f1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6767f20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c967f25, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x83367f27, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0767f2b, SUC_lexicon_tags_10 },
    { 0x31567f2c, SUC_lexicon_tags_3 },
    { 0x11f67f2d, SUC_lexicon_tags_11 },
    { 0x41367f2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8167f33, SUC_lexicon_tags_33 },
    { 0xb9567f34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12167f39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b67f3c, SUC_lexicon_tags_17 },
    { 0xd9f67f3d, SUC_lexicon_tags_29 },
    { 0x1b767f3e, SUC_lexicon_tags_65 },
    { 0xc5767f3f, SUC_lexicon_tags_17 },
    { 0x8b767f40, SUC_lexicon_tags_63 },
    { 0xf0967f3f, SUC_lexicon_tags_69 },
    { 0x10167f3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40767f44, SUC_lexicon_tags_17 },
    { 0xb2367f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96167f4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd67f55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1567f59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41167f5d, SUC_lexicon_tags_27 },
    { 0x8f767f5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x52f67f60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38367f64, SUC_lexicon_tags_55 },
    { 0x4a967f65, SUC_lexicon_tags_2 },
    { 0x61d67f65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab67f72, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f67f76, SUC_lexicon_tags_48 },
    { 0xb5567f77, SUC_lexicon_tags_2 },
    { 0xa2167f78, SUC_lexicon_tags_7 },
    { 0x8b567f79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8967f7d, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0xaf367f7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc8b67f81, SUC_lexicon_tags_37 },
    { 0x6eb67f82, SUC_lexicon_tags_8 },
    { 0x12967f82, SUC_lexicon_tags_27 },
    { 0xeeb67f84, SUC_lexicon_tags_16 },
    { 0x3c367f82, SUC_lexicon_tags_7 },
    { 0x5af67f83, SUC_lexicon_tags_11 },
    { 0xcc367f86, SUC_lexicon_tags_15 },
    { 0x89f67f88, SUC_lexicon_tags_0 },
    { 0x6fb67f85, SUC_lexicon_tags_7 },
    { 0x8c167f8a, SUC_lexicon_tags_11 },
    { 0xafd67f8b, SUC_lexicon_tags_77 },
    { 0x1ad67f85, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x32767f8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9567f91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x97967f93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbdd67f95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf67f9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b67fa0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93967fa5, SUC_lexicon_tags_49 },
    { 0xc5b67fa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9967fa9, SUC_lexicon_tags_30 },
    { 0x43567faa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34767fb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae167fb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d67fb9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1d367fbb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf6767fbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d67fc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd567fc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab167fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d767fc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d67fd3, SUC_lexicon_tags_12 },
    { 0x74167fd4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb67fd7, SUC_lexicon_tags_21 },
    { 0x1db67fd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26367fdf, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5167fe4, SUC_lexicon_tags_190 },
    { 0xf6767fe5, SUC_lexicon_tags_25 },
    { 0x45f67fe6, SUC_lexicon_tags_26 },
    { 0x8167fe7, SUC_lexicon_tags_15 },
    { 0xfc967fe7, SUC_lexicon_tags_12 },
    { 0xfe367fe4, SUC_lexicon_tags_7 },
    { 0xf1967fe7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee367fee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d67ff1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6ab67ff3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b67ff7, SUC_lexicon_tags_39 },
    { 0xb7367ff7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d767ffd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f68000, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7768006, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf96800b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b6800e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8168011, SUC_lexicon_tags_38 },
    { 0x49168011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69368014, SUC_lexicon_tags_8 },
    { 0xa2768015, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d368017, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d68023, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcd168025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8168027, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b6802b, SUC_lexicon_tags_17 },
    { 0xf656802b, SUC_lexicon_tags_10 },
    { 0x6f96802c, SUC_lexicon_tags_23 },
    { 0xf3d6802c, SUC_lexicon_tags_2 },
    { 0x516802e, SUC_lexicon_tags_16 },
    { 0xcfb68030, SUC_lexicon_tags_7 },
    { 0x1636802e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3b68033, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa3768035, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd6803c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2796803e, SUC_lexicon_tags_11 },
    { 0x2ef6803f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32168041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53168044, SUC_lexicon_tags_7 },
    { 0x90368045, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd0768047, SUC_lexicon_tags_9 },
    { 0x82d68048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2956804c, SUC_lexicon_tags_11 },
    { 0xd4d6804d, SUC_lexicon_tags_0 },
    { 0xd596804c, SUC_lexicon_tags_6 },
    { 0x396804f, SUC_lexicon_tags_47 },
    { 0x9e368050, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48768053, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e96805b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4836805d, SUC_lexicon_tags_60 },
    { 0xb436805e, SUC_lexicon_tags_57 },
    { 0xc436805f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb68062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d68065, SUC_lexicon_tags_47 },
    { 0x9b768066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4168068, SUC_lexicon_tags_38 },
    { 0x26d68069, SUC_lexicon_tags_37 },
    { 0x7356806a, SUC_lexicon_tags_17 },
    { 0x30768068, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f6806e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ed68070, SUC_lexicon_tags_68 },
    { 0x5d168071, SUC_lexicon_tags_1 },
    { 0x66168072, SUC_lexicon_tags_37 },
    { 0x4368073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec56807e, SUC_lexicon_tags_11 },
    { 0x6af6807f, SUC_lexicon_tags_37 },
    { 0x87968080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9568083, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd68089, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b6808e, SUC_lexicon_tags_18 },
    { 0xbc36808e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f68092, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x95768095, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc168098, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff36809c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8696809e, SUC_lexicon_tags_7 },
    { 0xb176809e, SUC_lexicon_tags_38 },
    { 0x5556809f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f680a7, SUC_lexicon_tags_2 },
    { 0xe53680a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0d680aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5680ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67680b8, SUC_lexicon_tags_63 },
    { 0xb7680b9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b680bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe29680be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9680c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9680c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7680cc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3680cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x511680d1, SUC_lexicon_tags_87 },
    { 0x387680d2, SUC_lexicon_tags_11 },
    { 0xc53680d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x283680d5, SUC_lexicon_tags_15 },
    { 0x743680d6, SUC_lexicon_tags_65 },
    { 0x1f680d7, SUC_lexicon_tags_25 },
    { 0x217680d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x389680db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29680de, SUC_lexicon_tags_0 },
    { 0x953680de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f680e3, SUC_lexicon_tags_38 },
    { 0x1ed680e3, SUC_lexicon_tags_38 },
    { 0xa4b680e5, SUC_lexicon_tags_35 },
    { 0x883680e4, SUC_lexicon_tags_25 },
    { 0x95b680e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09680ec, SUC_lexicon_tags_65 },
    { 0xad7680ed, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd680f0, SUC_lexicon_tags_1 },
    { 0x319680f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8f680f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xded680f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b680fa, SUC_lexicon_tags_12 },
    { 0x1af680fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb68104, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd568108, SUC_lexicon_tags_38 },
    { 0x7e768109, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf6810e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96b68110, SUC_lexicon_tags_15 },
    { 0x5d968110, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f568113, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc96811a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c16811d, SUC_lexicon_tags_47 },
    { 0x9676811d, SUC_lexicon_tags_1 },
    { 0xbbf6811f, SUC_lexicon_tags_65 },
    { 0xdaf68120, SUC_lexicon_tags_11 },
    { 0x9256811f, SUC_lexicon_tags_12 },
    { 0x18d68121, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d68128, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40d6812a, SUC_lexicon_tags_1 },
    { 0x2276812b, SUC_lexicon_tags_39 },
    { 0xc3d6812a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5576812e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b68138, SUC_lexicon_tags_7 },
    { 0x8568139, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd996813b, SUC_lexicon_tags_65 },
    { 0x3c76813c, SUC_lexicon_tags_2 },
    { 0x92f6813c, SUC_lexicon_tags_1 },
    { 0x2036813c, SUC_lexicon_tags_45 },
    { 0x8036813e, SUC_lexicon_tags_16 },
    { 0x5e368140, SUC_lexicon_tags_17 },
    { 0xde56813c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb68145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d6814a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef6814f, SUC_lexicon_tags_45 },
    { 0x8f56814f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd68153, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2168155, SUC_lexicon_tags_10 },
    { 0x42968155, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9568158, SUC_lexicon_tags_48 },
    { 0x89968159, SUC_lexicon_tags_25 },
    { 0xa6568159, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef56815e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee768161, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61368163, SUC_lexicon_tags_95 },
    { 0xbc368164, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x968166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4d68168, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf6816b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4596816f, SUC_lexicon_tags_7 },
    { 0xdad68170, SUC_lexicon_tags_17 },
    { 0x7676816f, SUC_lexicon_tags_1 },
    { 0x7d68171, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x60968174, SUC_lexicon_tags_37 },
    { 0x36968174, SUC_lexicon_tags_25 },
    { 0x5ed68176, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0768180, SUC_lexicon_tags_15 },
    { 0x58768181, SUC_lexicon_tags_7 },
    { 0x67168180, SUC_lexicon_tags_17 },
    { 0xe2d68183, SUC_lexicon_tags_15 },
    { 0xb3d68184, SUC_lexicon_tags_17 },
    { 0xe8368185, SUC_lexicon_tags_121 },
    { 0xb0d68186, SUC_lexicon_tags_17 },
    { 0xdd68185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57568192, SUC_lexicon_tags_129 },
    { 0x9fb68193, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d68198, SUC_lexicon_tags_11 },
    { 0xadb68199, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a16819b, SUC_lexicon_tags_37 },
    { 0x45f6819c, SUC_lexicon_tags_57 },
    { 0xb876819d, SUC_lexicon_tags_12 },
    { 0xc4b6819e, SUC_lexicon_tags_14 },
    { 0xeb16819e, SUC_lexicon_tags_948 },
    { 0x2176819c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42f681a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95681ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x451681ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687681b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279681b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef681bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9db681bf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x73f681c1, SUC_lexicon_tags_32 },
    { 0x3a3681c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79681c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927681d1, SUC_lexicon_tags_12 },
    { 0x635681d1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3681d6, SUC_lexicon_tags_12 },
    { 0x251681d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601681dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f681df, SUC_lexicon_tags_14 },
    { 0xc4f681df, SUC_lexicon_tags_11 },
    { 0x83b681e1, SUC_lexicon_tags_11 },
    { 0xefb681e2, SUC_lexicon_tags_47 },
    { 0xad681e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d681e6, SUC_lexicon_tags_45 },
    { 0xb29681e7, SUC_lexicon_tags_16 },
    { 0xa3d681e8, SUC_lexicon_tags_55 },
    { 0x9cb681e9, SUC_lexicon_tags_25 },
    { 0x627681ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9681ed, SUC_lexicon_tags_43 },
    { 0xc85681ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x191681f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d681f6, SUC_lexicon_tags_37 },
    { 0x91d681f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7681fa, SUC_lexicon_tags_37 },
    { 0x80b681fb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1a9681fd, SUC_lexicon_tags_43 },
    { 0xd41681fd, SUC_lexicon_tags_0 },
    { 0xea5681fe, SUC_lexicon_tags_25 },
    { 0x55f68200, SUC_lexicon_tags_15 },
    { 0xb8168200, SUC_lexicon_tags_46 },
    { 0x58f681fd, SUC_lexicon_tags_15 },
    { 0xc95681fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbf68205, SUC_lexicon_tags_23 },
    { 0x56b68206, SUC_lexicon_tags_49 },
    { 0xc2168207, SUC_lexicon_tags_17 },
    { 0xebb68207, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db6820d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f68211, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a168216, SUC_lexicon_tags_10 },
    { 0xfc968217, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad68219, SUC_lexicon_tags_1 },
    { 0xc656821a, SUC_lexicon_tags_2 },
    { 0x4d6821b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f6821f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x73f68221, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9316822a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6136822c, SUC_lexicon_tags_47 },
    { 0x6556822d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d68231, SUC_lexicon_tags_37 },
    { 0xadd68232, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db68236, SUC_lexicon_tags_38 },
    { 0x98168236, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d6823d, SUC_lexicon_tags_65 },
    { 0xba36823d, SUC_lexicon_tags_15 },
    { 0x8836823e, SUC_lexicon_tags_17 },
    { 0xe936823f, SUC_lexicon_tags_29 },
    { 0x6cf68241, SUC_lexicon_tags_47 },
    { 0x2b768240, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x63968245, SUC_lexicon_tags_140 },
    { 0xab368245, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e768249, SUC_lexicon_tags_49 },
    { 0xa3368249, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78768252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d168254, SUC_lexicon_tags_29 },
    { 0x4c768255, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x89168257, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc168259, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x6736825b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc76825e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab168261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec968268, SUC_lexicon_tags_11 },
    { 0x43768269, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1036826b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cf6826d, SUC_lexicon_tags_7 },
    { 0xaa96826d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xded68271, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb68274, SUC_lexicon_tags_30 },
    { 0x3a968275, SUC_lexicon_tags_25 },
    { 0x94568276, SUC_lexicon_tags_6 },
    { 0x5cb68277, SUC_lexicon_tags_1 },
    { 0x13f68278, SUC_lexicon_tags_0 },
    { 0x75168277, SUC_lexicon_tags_17 },
    { 0xb8b68277, SUC_lexicon_tags_30 },
    { 0xd2d6827b, SUC_lexicon_tags_2 },
    { 0x2c768277, SUC_lexicon_tags_9 },
    { 0x4276827d, SUC_lexicon_tags_15 },
    { 0xa596827e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b68282, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdc568284, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf6828a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9576828c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bb6828e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0768296, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1296829c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995682a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135682a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cf682a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887682b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3682b9, SUC_lexicon_tags_1 },
    { 0x41f682ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5682bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a3682c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d682c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f682d1, SUC_lexicon_tags_14 },
    { 0x671682d2, SUC_lexicon_tags_13 },
    { 0x77d682d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd682d7, SUC_lexicon_tags_15 },
    { 0xa2b682d8, SUC_lexicon_tags_75 },
    { 0xa8d682d9, SUC_lexicon_tags_7 },
    { 0xb11682da, SUC_lexicon_tags_17 },
    { 0xa2b682db, SUC_lexicon_tags_32 },
    { 0x85682d9, SUC_lexicon_tags_65 },
    { 0x80b682dd, SUC_lexicon_tags_1 },
    { 0x44d682da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897682e6, SUC_lexicon_tags_11 },
    { 0xf3b682e7, SUC_lexicon_tags_49 },
    { 0x96f682e8, SUC_lexicon_tags_14 },
    { 0xd0b682e9, SUC_lexicon_tags_19 },
    { 0x24b682ea, SUC_lexicon_tags_2 },
    { 0xeff682eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b682f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf682fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c968304, SUC_lexicon_tags_1 },
    { 0xab368304, SUC_lexicon_tags_11 },
    { 0x89968306, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec768312, SUC_lexicon_tags_15 },
    { 0xe5968313, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8968318, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8576831f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75368324, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x92968326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4d68328, SUC_lexicon_tags_14 },
    { 0x8cd68329, SUC_lexicon_tags_7 },
    { 0x3b368329, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6832d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1536832f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc968331, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe1168333, SUC_lexicon_tags_12 },
    { 0x35968333, SUC_lexicon_tags_17 },
    { 0x6d68334, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f68338, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3856833c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d168340, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ab68342, SUC_lexicon_tags_17 },
    { 0xbd68343, SUC_lexicon_tags_3 },
    { 0x8cd68344, SUC_lexicon_tags_38 },
    { 0x27368344, SUC_lexicon_tags_49 },
    { 0x5e968346, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26168348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81d6834a, SUC_lexicon_tags_43 },
    { 0x4bb6834b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9b56834d, SUC_lexicon_tags_26 },
    { 0x99b6834e, SUC_lexicon_tags_0 },
    { 0xe016834f, SUC_lexicon_tags_68 },
    { 0xaef6834e, SUC_lexicon_tags_2 },
    { 0x26d68351, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c368358, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1556835b, SUC_lexicon_tags_27 },
    { 0x92f6835b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57968360, SUC_lexicon_tags_11 },
    { 0x95568361, SUC_lexicon_tags_37 },
    { 0xa7768361, SUC_lexicon_tags_2 },
    { 0x8b368363, SUC_lexicon_tags_57 },
    { 0x2ab68364, SUC_lexicon_tags_43 },
    { 0xbb68365, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7376836d, SUC_lexicon_tags_101 },
    { 0x8296836d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x35568371, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0368379, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6837f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7368381, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57b68383, SUC_lexicon_tags_0 },
    { 0x41f68384, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb6838b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd68390, SUC_lexicon_tags_39 },
    { 0x63568391, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f568393, SUC_lexicon_tags_37 },
    { 0xf0b68393, SUC_lexicon_tags_57 },
    { 0x85d68395, SUC_lexicon_tags_14 },
    { 0xfff68394, SUC_lexicon_tags_23 },
    { 0xe7b68397, SUC_lexicon_tags_34 },
    { 0x4ed68394, SUC_lexicon_tags_14 },
    { 0xf9f68396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595683a4, SUC_lexicon_tags_37 },
    { 0x3c5683a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x815683a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509683ad, SUC_lexicon_tags_68 },
    { 0xeff683ae, SUC_lexicon_tags_25 },
    { 0xaf683af, SUC_lexicon_tags_25 },
    { 0x5bd683ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7683b7, SUC_lexicon_tags_40 },
    { 0x489683b8, SUC_lexicon_tags_11 },
    { 0x69683b8, SUC_lexicon_tags_15 },
    { 0xdf683ba, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59683bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389683c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91683cb, SUC_lexicon_tags_37 },
    { 0x5f683cc, SUC_lexicon_tags_7 },
    { 0x1e3683cb, SUC_lexicon_tags_70 },
    { 0xe8d683cb, SUC_lexicon_tags_25 },
    { 0x89d683cf, SUC_lexicon_tags_44 },
    { 0x543683cd, SUC_lexicon_tags_37 },
    { 0x6e1683d1, SUC_lexicon_tags_7 },
    { 0x96d683d2, SUC_lexicon_tags_11 },
    { 0xd0b683d2, SUC_lexicon_tags_24 },
    { 0xf83683d4, SUC_lexicon_tags_1 },
    { 0xfa3683d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95683df, SUC_lexicon_tags_23 },
    { 0x161683df, SUC_lexicon_tags_15 },
    { 0xe3b683df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481683e5, SUC_lexicon_tags_43 },
    { 0x4a5683e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfd683e8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe97683ea, SUC_lexicon_tags_87 },
    { 0x295683eb, SUC_lexicon_tags_11 },
    { 0x497683ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x781683ef, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9683f2, SUC_lexicon_tags_37 },
    { 0xead683f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x567683f5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1683f8, SUC_lexicon_tags_12 },
    { 0x733683f8, SUC_lexicon_tags_25 },
    { 0xcb7683f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x381683fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf68403, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2ef68405, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x15168408, SUC_lexicon_tags_3 },
    { 0x5ad68409, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9bd6840b, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b16840e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f68413, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a368416, SUC_lexicon_tags_25 },
    { 0xb7f68417, SUC_lexicon_tags_7 },
    { 0x18968416, SUC_lexicon_tags_1075 },
    { 0, NULL },
    { 0x8d96841a, SUC_lexicon_tags_12 },
    { 0xb5d6841b, SUC_lexicon_tags_57 },
    { 0x8df6841c, SUC_lexicon_tags_1 },
    { 0x3d56841d, SUC_lexicon_tags_15 },
    { 0x9af6841d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x26d68420, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36368425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf68428, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e96842a, SUC_lexicon_tags_37 },
    { 0x82d6842a, SUC_lexicon_tags_9 },
    { 0x5896842c, SUC_lexicon_tags_39 },
    { 0x92f6842a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f68433, SUC_lexicon_tags_55 },
    { 0x8168434, SUC_lexicon_tags_37 },
    { 0x51768435, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85968439, SUC_lexicon_tags_46 },
    { 0xa9d6843a, SUC_lexicon_tags_32 },
    { 0x2e56843a, SUC_lexicon_tags_43 },
    { 0x17568439, SUC_lexicon_tags_17 },
    { 0x7c16843d, SUC_lexicon_tags_65 },
    { 0x1536843d, SUC_lexicon_tags_45 },
    { 0x676843b, SUC_lexicon_tags_12 },
    { 0xca168440, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21d68442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd68445, SUC_lexicon_tags_11 },
    { 0xbdf68446, SUC_lexicon_tags_68 },
    { 0x15368447, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b368449, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x3436844b, SUC_lexicon_tags_28 },
    { 0x5236844c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a368450, SUC_lexicon_tags_37 },
    { 0xf368450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b68454, SUC_lexicon_tags_19 },
    { 0x87768454, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf768457, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa16845a, SUC_lexicon_tags_1265 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd6845f, SUC_lexicon_tags_37 },
    { 0xa216845f, SUC_lexicon_tags_15 },
    { 0xf9b6845f, SUC_lexicon_tags_63 },
    { 0xae768462, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x64768464, SUC_lexicon_tags_17 },
    { 0xd2568465, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30368469, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5916846b, SUC_lexicon_tags_47 },
    { 0x33f6846b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d6846f, SUC_lexicon_tags_27 },
    { 0x6f968470, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd68474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50568479, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f6847d, SUC_lexicon_tags_68 },
    { 0xe1f6847e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6f68480, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c368482, SUC_lexicon_tags_35 },
    { 0xff968483, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xee768485, SUC_lexicon_tags_19 },
    { 0xb9f68486, SUC_lexicon_tags_30 },
    { 0x56568486, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b96848c, SUC_lexicon_tags_87 },
    { 0x8eb6848d, SUC_lexicon_tags_31 },
    { 0xc956848c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb368491, SUC_lexicon_tags_29 },
    { 0xd568492, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xab368495, SUC_lexicon_tags_7 },
    { 0xab768496, SUC_lexicon_tags_30 },
    { 0x7f68496, SUC_lexicon_tags_61 },
    { 0x17168498, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcb6849a, SUC_lexicon_tags_26 },
    { 0xe1d6849b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f684a1, SUC_lexicon_tags_10 },
    { 0xe7b684a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d684a5, SUC_lexicon_tags_47 },
    { 0x37684a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279684ae, SUC_lexicon_tags_38 },
    { 0x65b684ae, SUC_lexicon_tags_37 },
    { 0x4db684ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3d684b2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3d684b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13684b7, SUC_lexicon_tags_14 },
    { 0x4cb684b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1684bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d684bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x625684c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49684ce, SUC_lexicon_tags_12 },
    { 0x2c9684cf, SUC_lexicon_tags_55 },
    { 0x733684d0, SUC_lexicon_tags_37 },
    { 0xbc7684d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d684d4, SUC_lexicon_tags_49 },
    { 0xd03684d5, SUC_lexicon_tags_15 },
    { 0x1a3684d5, SUC_lexicon_tags_49 },
    { 0x975684d6, SUC_lexicon_tags_23 },
    { 0x4c1684d8, SUC_lexicon_tags_37 },
    { 0x441684d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b684dd, SUC_lexicon_tags_43 },
    { 0x749684dd, SUC_lexicon_tags_16 },
    { 0x6b9684de, SUC_lexicon_tags_7 },
    { 0x3ed684dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f684e4, SUC_lexicon_tags_11 },
    { 0x6f9684e4, SUC_lexicon_tags_17 },
    { 0x61684e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6db684e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ff684ea, SUC_lexicon_tags_27 },
    { 0x1f3684eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71d684ed, SUC_lexicon_tags_14 },
    { 0xd3b684ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf91684f0, SUC_lexicon_tags_43 },
    { 0x1a3684f0, SUC_lexicon_tags_62 },
    { 0xf97684f1, SUC_lexicon_tags_37 },
    { 0x963684f3, SUC_lexicon_tags_26 },
    { 0x681684f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc61684f6, SUC_lexicon_tags_12 },
    { 0x509684f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9bd684f9, SUC_lexicon_tags_19 },
    { 0xf27684f9, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09684fd, SUC_lexicon_tags_0 },
    { 0x7e9684fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb2d68500, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39b68502, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7568506, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25f68508, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf496850c, SUC_lexicon_tags_4 },
    { 0xb36850c, SUC_lexicon_tags_17 },
    { 0xed16850d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b68511, SUC_lexicon_tags_23 },
    { 0xaaf68512, SUC_lexicon_tags_15 },
    { 0x91b68513, SUC_lexicon_tags_37 },
    { 0xa0568514, SUC_lexicon_tags_17 },
    { 0x29368515, SUC_lexicon_tags_19 },
    { 0xe3b68516, SUC_lexicon_tags_16 },
    { 0xd5b68516, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d6851b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f6851f, SUC_lexicon_tags_38 },
    { 0x34d68520, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b568523, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2968528, SUC_lexicon_tags_6 },
    { 0x93168529, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8456852c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4a56852e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe0168530, SUC_lexicon_tags_60 },
    { 0x6bf68531, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3568539, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d6853c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34968542, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5368545, SUC_lexicon_tags_29 },
    { 0x4fb68545, SUC_lexicon_tags_0 },
    { 0xbf768547, SUC_lexicon_tags_2 },
    { 0x30968546, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31368557, SUC_lexicon_tags_12 },
    { 0x51768558, SUC_lexicon_tags_14 },
    { 0x4b968557, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d6855e, SUC_lexicon_tags_11 },
    { 0x8fb6855e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1df68561, SUC_lexicon_tags_14 },
    { 0x64d68562, SUC_lexicon_tags_12 },
    { 0x4a968563, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3568567, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0x336856a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcc56856c, SUC_lexicon_tags_31 },
    { 0x21d6856d, SUC_lexicon_tags_25 },
    { 0xfc96856e, SUC_lexicon_tags_19 },
    { 0x6a36856e, SUC_lexicon_tags_25 },
    { 0xb836856d, SUC_lexicon_tags_1 },
    { 0x82768571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25f68573, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9568576, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6d68578, SUC_lexicon_tags_14 },
    { 0x45b68578, SUC_lexicon_tags_38 },
    { 0x7616857a, SUC_lexicon_tags_17 },
    { 0xa136857b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f76857f, SUC_lexicon_tags_27 },
    { 0x3ed68580, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xba768582, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xde968584, SUC_lexicon_tags_11 },
    { 0xbff68585, SUC_lexicon_tags_14 },
    { 0x6d768585, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24968588, SUC_lexicon_tags_30 },
    { 0xcdd68589, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf776858f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb68592, SUC_lexicon_tags_47 },
    { 0x18568593, SUC_lexicon_tags_9 },
    { 0x36b68592, SUC_lexicon_tags_47 },
    { 0x60768595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddf68597, SUC_lexicon_tags_22 },
    { 0xda168598, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976859c, SUC_lexicon_tags_7 },
    { 0xa316859d, SUC_lexicon_tags_29 },
    { 0x1b56859e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83685a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf685a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07685ac, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667685b0, SUC_lexicon_tags_111 },
    { 0xa57685b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1685b5, SUC_lexicon_tags_11 },
    { 0xec5685b6, SUC_lexicon_tags_37 },
    { 0x1b3685b6, SUC_lexicon_tags_12 },
    { 0x45d685b8, SUC_lexicon_tags_11 },
    { 0xa6b685b9, SUC_lexicon_tags_30 },
    { 0xf2b685ba, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7685c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b685c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d685cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7685d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7685d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdaf685d5, SUC_lexicon_tags_39 },
    { 0xec3685d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b685d9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d685df, SUC_lexicon_tags_12 },
    { 0xdf7685e0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3b9685e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x527685e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf53685e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x285685e9, SUC_lexicon_tags_17 },
    { 0x78b685e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7685ee, SUC_lexicon_tags_25 },
    { 0xec5685ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b685f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x495685f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87685fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c5685fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc368602, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad68607, SUC_lexicon_tags_34 },
    { 0xa3168607, SUC_lexicon_tags_37 },
    { 0xda568608, SUC_lexicon_tags_11 },
    { 0x5d96860a, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab6860f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d68612, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb56861c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b368621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42b68623, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0368628, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d6862b, SUC_lexicon_tags_11 },
    { 0xe4d6862b, SUC_lexicon_tags_1 },
    { 0x3956862b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2b6862f, SUC_lexicon_tags_41 },
    { 0x51d68630, SUC_lexicon_tags_27 },
    { 0xd5f68630, SUC_lexicon_tags_4 },
    { 0xdcb68632, SUC_lexicon_tags_65 },
    { 0x7f368631, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x96b68635, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x20b68637, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa236863a, SUC_lexicon_tags_15 },
    { 0x1736863b, SUC_lexicon_tags_2 },
    { 0x3856863c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa696863e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f68643, SUC_lexicon_tags_84 },
    { 0xe1968644, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2368646, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c768649, SUC_lexicon_tags_37 },
    { 0xe7b6864a, SUC_lexicon_tags_26 },
    { 0x5d16864b, SUC_lexicon_tags_27 },
    { 0xb16864b, SUC_lexicon_tags_33 },
    { 0xe356864c, SUC_lexicon_tags_60 },
    { 0x716864a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e96865b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d68660, SUC_lexicon_tags_25 },
    { 0x28768661, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd9968663, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91168665, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a768668, SUC_lexicon_tags_55 },
    { 0x6a768669, SUC_lexicon_tags_11 },
    { 0xed36866a, SUC_lexicon_tags_56 },
    { 0x46f68669, SUC_lexicon_tags_23 },
    { 0x4e76866c, SUC_lexicon_tags_46 },
    { 0xaf96866d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b68670, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc368672, SUC_lexicon_tags_37 },
    { 0x15168672, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62d68675, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91368677, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7056867b, SUC_lexicon_tags_25 },
    { 0xe96867c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa56867e, SUC_lexicon_tags_12 },
    { 0x4176867f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33168681, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8368683, SUC_lexicon_tags_11 },
    { 0xacd68684, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf968686, SUC_lexicon_tags_58 },
    { 0xa5b68687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5136868b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e56868e, SUC_lexicon_tags_2 },
    { 0xb196868f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51368691, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2168694, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f6869d, SUC_lexicon_tags_15 },
    { 0x7996869e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x597686a0, SUC_lexicon_tags_65 },
    { 0xd9686a1, SUC_lexicon_tags_27 },
    { 0x615686a2, SUC_lexicon_tags_28 },
    { 0x4b7686a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b686a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x91686a8, SUC_lexicon_tags_47 },
    { 0x9d1686a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x699686ab, SUC_lexicon_tags_28 },
    { 0x13d686ac, SUC_lexicon_tags_11 },
    { 0xc5d686ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x767686b0, SUC_lexicon_tags_33 },
    { 0x92b686b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401686ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd01686bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b686bf, SUC_lexicon_tags_38 },
    { 0x809686c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f686c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97686cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdb7686cd, SUC_lexicon_tags_69 },
    { 0xe4d686ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x261686d0, SUC_lexicon_tags_45 },
    { 0xd1f686d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x961686d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7686d8, SUC_lexicon_tags_16 },
    { 0x19f686d8, SUC_lexicon_tags_65 },
    { 0xeab686d9, SUC_lexicon_tags_15 },
    { 0x5cd686db, SUC_lexicon_tags_32 },
    { 0xff1686dc, SUC_lexicon_tags_2 },
    { 0x6d5686da, SUC_lexicon_tags_25 },
    { 0xd41686de, SUC_lexicon_tags_10 },
    { 0x22d686df, SUC_lexicon_tags_11 },
    { 0x4e5686e0, SUC_lexicon_tags_2 },
    { 0x227686e0, SUC_lexicon_tags_37 },
    { 0xcf7686e2, SUC_lexicon_tags_25 },
    { 0x71f686e3, SUC_lexicon_tags_48 },
    { 0x7af686de, SUC_lexicon_tags_17 },
    { 0xc9686e3, SUC_lexicon_tags_87 },
    { 0x551686e6, SUC_lexicon_tags_19 },
    { 0x1a5686e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x415686ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x747686ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x147686ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519686f7, SUC_lexicon_tags_151 },
    { 0x5ef686f7, SUC_lexicon_tags_12 },
    { 0x9ad686f9, SUC_lexicon_tags_21 },
    { 0x63d686fa, SUC_lexicon_tags_82 },
    { 0xa9d686fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4a7686fd, SUC_lexicon_tags_34 },
    { 0x773686fe, SUC_lexicon_tags_1 },
    { 0x3b686ff, SUC_lexicon_tags_45 },
    { 0x8a9686ff, SUC_lexicon_tags_11 },
    { 0x7b968701, SUC_lexicon_tags_25 },
    { 0x56168700, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a96870b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d68712, SUC_lexicon_tags_38 },
    { 0xb1368713, SUC_lexicon_tags_2 },
    { 0x33568714, SUC_lexicon_tags_14 },
    { 0xefd68714, SUC_lexicon_tags_1 },
    { 0x2e968715, SUC_lexicon_tags_15 },
    { 0x7a968714, SUC_lexicon_tags_7 },
    { 0x89768716, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb976871a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa76871c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf156871f, SUC_lexicon_tags_45 },
    { 0x93368720, SUC_lexicon_tags_32 },
    { 0xbfd68721, SUC_lexicon_tags_8 },
    { 0x57f68722, SUC_lexicon_tags_37 },
    { 0x8e168722, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b68727, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef6872a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccf6872c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d96872e, SUC_lexicon_tags_14 },
    { 0xb776872f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b68732, SUC_lexicon_tags_17 },
    { 0x91168732, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe6768735, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xcc168737, SUC_lexicon_tags_1 },
    { 0x4bf68738, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe776873d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x8c36873f, SUC_lexicon_tags_235 },
    { 0x35168740, SUC_lexicon_tags_0 },
    { 0xb376873f, SUC_lexicon_tags_47 },
    { 0x46368742, SUC_lexicon_tags_30 },
    { 0x60168743, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46368745, SUC_lexicon_tags_7 },
    { 0x62968746, SUC_lexicon_tags_23 },
    { 0xf7568745, SUC_lexicon_tags_37 },
    { 0x11f68748, SUC_lexicon_tags_46 },
    { 0x50168749, SUC_lexicon_tags_0 },
    { 0xf3f68748, SUC_lexicon_tags_11 },
    { 0x77b6874b, SUC_lexicon_tags_25 },
    { 0x76b6874c, SUC_lexicon_tags_17 },
    { 0x78d68749, SUC_lexicon_tags_25 },
    { 0x1d6874e, SUC_lexicon_tags_15 },
    { 0x7f368749, SUC_lexicon_tags_12 },
    { 0xbe768750, SUC_lexicon_tags_15 },
    { 0x18f68749, SUC_lexicon_tags_63 },
    { 0x56168752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2968755, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9236875a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb96875d, SUC_lexicon_tags_7 },
    { 0xcff6875d, SUC_lexicon_tags_104 },
    { 0x45f6875f, SUC_lexicon_tags_15 },
    { 0x46f68760, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f68765, SUC_lexicon_tags_17 },
    { 0x66968766, SUC_lexicon_tags_37 },
    { 0xd1168765, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8476876a, SUC_lexicon_tags_80 },
    { 0x1476876b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b6876f, SUC_lexicon_tags_0 },
    { 0xb7f6876f, SUC_lexicon_tags_11 },
    { 0xe6568770, SUC_lexicon_tags_17 },
    { 0x33b68772, SUC_lexicon_tags_2 },
    { 0xf3368772, SUC_lexicon_tags_42 },
    { 0xea768773, SUC_lexicon_tags_2 },
    { 0x32768772, SUC_lexicon_tags_2 },
    { 0x64168776, SUC_lexicon_tags_2 },
    { 0x8e968776, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d96877d, SUC_lexicon_tags_79 },
    { 0xad6877e, SUC_lexicon_tags_7 },
    { 0xd1d6877f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6568784, SUC_lexicon_tags_17 },
    { 0x16b68784, SUC_lexicon_tags_0 },
    { 0x3d68786, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c968788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4d6878a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e36878e, SUC_lexicon_tags_49 },
    { 0x90d6878e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7568792, SUC_lexicon_tags_66 },
    { 0x568793, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14368798, SUC_lexicon_tags_10 },
    { 0x5b768799, SUC_lexicon_tags_37 },
    { 0x1356879a, SUC_lexicon_tags_1 },
    { 0xc2b68799, SUC_lexicon_tags_33 },
    { 0x77568799, SUC_lexicon_tags_37 },
    { 0xe1d6879c, SUC_lexicon_tags_32 },
    { 0x3d76879c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x959687a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47687a3, SUC_lexicon_tags_15 },
    { 0x603687a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d687a9, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b687ac, SUC_lexicon_tags_17 },
    { 0x59f687ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d687b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187687b4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x243687b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b687b9, SUC_lexicon_tags_85 },
    { 0xf5d687ba, SUC_lexicon_tags_37 },
    { 0xad3687bb, SUC_lexicon_tags_60 },
    { 0x6c5687bc, SUC_lexicon_tags_85 },
    { 0x293687bd, SUC_lexicon_tags_26 },
    { 0x7687bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d9687c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9687c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95b687c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985687cd, SUC_lexicon_tags_28 },
    { 0x6dd687ce, SUC_lexicon_tags_14 },
    { 0xa39687cf, SUC_lexicon_tags_17 },
    { 0xd99687ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b687d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf11687d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b687da, SUC_lexicon_tags_63 },
    { 0x35d687da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f687df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x647687e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc9687e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b687e7, SUC_lexicon_tags_29 },
    { 0xa9b687e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7687ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4eb687ef, SUC_lexicon_tags_11 },
    { 0xbc1687f0, SUC_lexicon_tags_19 },
    { 0x169687ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed687f5, SUC_lexicon_tags_46 },
    { 0xcc5687f6, SUC_lexicon_tags_11 },
    { 0x533687f6, SUC_lexicon_tags_23 },
    { 0x66b687f6, SUC_lexicon_tags_3 },
    { 0x23f687f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbab687fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833687ff, SUC_lexicon_tags_38 },
    { 0xf7568800, SUC_lexicon_tags_37 },
    { 0xb5968801, SUC_lexicon_tags_57 },
    { 0xa768800, SUC_lexicon_tags_23 },
    { 0x20568801, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86368805, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xcff68807, SUC_lexicon_tags_11 },
    { 0x94768807, SUC_lexicon_tags_9 },
    { 0x5b368809, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b6880d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfc36880f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19168811, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xec968813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27d68815, SUC_lexicon_tags_37 },
    { 0x2ab68815, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe936881a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xeb36881c, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0x5216881e, SUC_lexicon_tags_37 },
    { 0xef36881f, SUC_lexicon_tags_32 },
    { 0xe0f6881e, SUC_lexicon_tags_35 },
    { 0x3796881f, SUC_lexicon_tags_15 },
    { 0xe4568822, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x11b68824, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe3f68826, SUC_lexicon_tags_7 },
    { 0x3cb68826, SUC_lexicon_tags_11 },
    { 0x4df68828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbb6882a, SUC_lexicon_tags_29 },
    { 0xfb96882b, SUC_lexicon_tags_17 },
    { 0x7a96882c, SUC_lexicon_tags_1 },
    { 0x1b36882d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6176882f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8168831, SUC_lexicon_tags_1 },
    { 0x23768832, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6968834, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d6883a, SUC_lexicon_tags_30 },
    { 0x6b16883a, SUC_lexicon_tags_65 },
    { 0xce36883b, SUC_lexicon_tags_25 },
    { 0xe756883b, SUC_lexicon_tags_1 },
    { 0xb856883e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x14168840, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda568845, SUC_lexicon_tags_37 },
    { 0xfd368846, SUC_lexicon_tags_2 },
    { 0xfa168846, SUC_lexicon_tags_26 },
    { 0x84168848, SUC_lexicon_tags_12 },
    { 0xb7d68848, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b6884e, SUC_lexicon_tags_25 },
    { 0x72f6884f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86368851, SUC_lexicon_tags_11 },
    { 0xaef68852, SUC_lexicon_tags_3 },
    { 0x6f568853, SUC_lexicon_tags_39 },
    { 0xb1968852, SUC_lexicon_tags_11 },
    { 0xfa168852, SUC_lexicon_tags_138 },
    { 0x40768856, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52968859, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c76885e, SUC_lexicon_tags_12 },
    { 0xd636885f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6ef68861, SUC_lexicon_tags_7 },
    { 0x7f368862, SUC_lexicon_tags_37 },
    { 0xb4b68862, SUC_lexicon_tags_45 },
    { 0xa1768862, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x16568866, SUC_lexicon_tags_4 },
    { 0x73968867, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf6886d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x14768870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0568879, SUC_lexicon_tags_63 },
    { 0xc6768879, SUC_lexicon_tags_25 },
    { 0x22d6887b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3176887d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb68881, SUC_lexicon_tags_68 },
    { 0x48b68882, SUC_lexicon_tags_14 },
    { 0xc1f68883, SUC_lexicon_tags_17 },
    { 0x9f768884, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6768887, SUC_lexicon_tags_57 },
    { 0x5f168887, SUC_lexicon_tags_17 },
    { 0x8df68889, SUC_lexicon_tags_27 },
    { 0x45f6888a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3396888e, SUC_lexicon_tags_66 },
    { 0xe396888f, SUC_lexicon_tags_39 },
    { 0x6b36888f, SUC_lexicon_tags_7 },
    { 0x7c768891, SUC_lexicon_tags_0 },
    { 0xbe76888f, SUC_lexicon_tags_19 },
    { 0x30b68892, SUC_lexicon_tags_43 },
    { 0x32568891, SUC_lexicon_tags_31 },
    { 0x5596888f, SUC_lexicon_tags_27 },
    { 0xfbb68896, SUC_lexicon_tags_11 },
    { 0xe1168897, SUC_lexicon_tags_2 },
    { 0x78d68898, SUC_lexicon_tags_64 },
    { 0xd5568896, SUC_lexicon_tags_27 },
    { 0xabf68893, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b16889c, SUC_lexicon_tags_25 },
    { 0xb156889d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d688a0, SUC_lexicon_tags_2 },
    { 0xcaf688a0, SUC_lexicon_tags_47 },
    { 0x321688a2, SUC_lexicon_tags_7 },
    { 0x521688a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x765688a6, SUC_lexicon_tags_17 },
    { 0xc15688a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x467688a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd688ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x547688af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x427688b2, SUC_lexicon_tags_11 },
    { 0xf25688b3, SUC_lexicon_tags_17 },
    { 0x131688b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44b688b6, SUC_lexicon_tags_63 },
    { 0x491688b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed688bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x55f688bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xee5688bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x903688c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f3688c4, SUC_lexicon_tags_25 },
    { 0x523688c4, SUC_lexicon_tags_37 },
    { 0x4a1688c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4ab688c8, SUC_lexicon_tags_17 },
    { 0x829688c8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7cd688cb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x365688cd, SUC_lexicon_tags_19 },
    { 0x885688cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49688d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7688d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6f9688d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9688df, SUC_lexicon_tags_0 },
    { 0x13f688e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8bf688e2, SUC_lexicon_tags_31 },
    { 0x19d688e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb41688e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f688e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe77688eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab688f0, SUC_lexicon_tags_1199 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77688f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x507688f6, SUC_lexicon_tags_11 },
    { 0x13f688f7, SUC_lexicon_tags_37 },
    { 0x6c1688f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8368902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff68905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1716890a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b68912, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf368916, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc76891e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b968927, SUC_lexicon_tags_60 },
    { 0xbbd68927, SUC_lexicon_tags_0 },
    { 0xd9368929, SUC_lexicon_tags_17 },
    { 0xdab68928, SUC_lexicon_tags_17 },
    { 0xafd68928, SUC_lexicon_tags_17 },
    { 0x6596892c, SUC_lexicon_tags_66 },
    { 0x8ab68927, SUC_lexicon_tags_56 },
    { 0x3db6892a, SUC_lexicon_tags_47 },
    { 0x9dd6892f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f68935, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf8968937, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb16893b, SUC_lexicon_tags_47 },
    { 0x45f6893c, SUC_lexicon_tags_37 },
    { 0xdf36893d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a36893f, SUC_lexicon_tags_25 },
    { 0x74d6893f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf68946, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab56894a, SUC_lexicon_tags_1 },
    { 0xd256894b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5696894d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc6b6894f, SUC_lexicon_tags_17 },
    { 0x6a568950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3168953, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b68956, SUC_lexicon_tags_30 },
    { 0xbdd68957, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xee968959, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad6895d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa0f6895f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13168961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5168963, SUC_lexicon_tags_57 },
    { 0xa1568964, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e768967, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456896b, SUC_lexicon_tags_11 },
    { 0xd056896c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b68970, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b68975, SUC_lexicon_tags_37 },
    { 0xc2568976, SUC_lexicon_tags_29 },
    { 0x44968976, SUC_lexicon_tags_35 },
    { 0x30168978, SUC_lexicon_tags_25 },
    { 0x7e168976, SUC_lexicon_tags_1 },
    { 0x19168978, SUC_lexicon_tags_47 },
    { 0x2b6897b, SUC_lexicon_tags_87 },
    { 0x6696897c, SUC_lexicon_tags_12 },
    { 0xded68976, SUC_lexicon_tags_15 },
    { 0x656897c, SUC_lexicon_tags_68 },
    { 0x24b6897f, SUC_lexicon_tags_17 },
    { 0xa776897c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2fd68982, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d68986, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d6898d, SUC_lexicon_tags_30 },
    { 0xe056898e, SUC_lexicon_tags_37 },
    { 0x6e36898d, SUC_lexicon_tags_16 },
    { 0x8456898d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5768992, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65168998, SUC_lexicon_tags_14 },
    { 0xf4b68998, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1996899b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb836899d, SUC_lexicon_tags_25 },
    { 0x8836899e, SUC_lexicon_tags_35 },
    { 0xe356899f, SUC_lexicon_tags_7 },
    { 0xf7f6899f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97689a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb13689a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x135689a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1689aa, SUC_lexicon_tags_63 },
    { 0x36b689ab, SUC_lexicon_tags_39 },
    { 0xf09689ac, SUC_lexicon_tags_25 },
    { 0xb09689ad, SUC_lexicon_tags_1 },
    { 0x109689ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df689b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909689b9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb689be, SUC_lexicon_tags_17 },
    { 0xfa5689bf, SUC_lexicon_tags_37 },
    { 0xa7d689c0, SUC_lexicon_tags_17 },
    { 0xf95689c0, SUC_lexicon_tags_35 },
    { 0x7d3689c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b689c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239689d3, SUC_lexicon_tags_15 },
    { 0xc1b689d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e5689d6, SUC_lexicon_tags_9 },
    { 0x1d7689d7, SUC_lexicon_tags_11 },
    { 0xd5d689d8, SUC_lexicon_tags_47 },
    { 0x619689d9, SUC_lexicon_tags_25 },
    { 0xde3689da, SUC_lexicon_tags_19 },
    { 0x733689db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34b689dd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9689e0, SUC_lexicon_tags_21 },
    { 0x20d689e1, SUC_lexicon_tags_7 },
    { 0xc43689e2, SUC_lexicon_tags_60 },
    { 0x349689e1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3689e7, SUC_lexicon_tags_38 },
    { 0x18f689e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03689ef, SUC_lexicon_tags_21 },
    { 0xaa3689ef, SUC_lexicon_tags_25 },
    { 0x39b689ef, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x635689f3, SUC_lexicon_tags_47 },
    { 0xb97689f4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2c1689f6, SUC_lexicon_tags_16 },
    { 0x999689f6, SUC_lexicon_tags_82 },
    { 0x5a9689f8, SUC_lexicon_tags_1 },
    { 0xd75689f9, SUC_lexicon_tags_17 },
    { 0x52b689f8, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x567689fd, SUC_lexicon_tags_28 },
    { 0xfcd689fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11968a06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6768a09, SUC_lexicon_tags_11 },
    { 0x52168a0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b68a0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6368a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3b68a13, SUC_lexicon_tags_37 },
    { 0x8e968a14, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda768a18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99768a20, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96368a26, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbed68a28, SUC_lexicon_tags_32 },
    { 0x29768a28, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x24768a2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4968a2e, SUC_lexicon_tags_1 },
    { 0x36568a2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdf68a31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb2b68a33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xac168a35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d68a38, SUC_lexicon_tags_3 },
    { 0xf4f68a39, SUC_lexicon_tags_27 },
    { 0xc0168a38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7b368a3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x77368a3e, SUC_lexicon_tags_37 },
    { 0xb8568a3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d68a44, SUC_lexicon_tags_15 },
    { 0x92b68a44, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97968a49, SUC_lexicon_tags_8 },
    { 0x1f168a49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd68a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd968a54, SUC_lexicon_tags_17 },
    { 0x75968a54, SUC_lexicon_tags_34 },
    { 0x5b568a56, SUC_lexicon_tags_68 },
    { 0x8e768a56, SUC_lexicon_tags_15 },
    { 0x68968a58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12968a5b, SUC_lexicon_tags_14 },
    { 0xc2d68a5c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf2168a5e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4d168a60, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7b68a62, SUC_lexicon_tags_63 },
    { 0x88b68a63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe5168a65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfab68a67, SUC_lexicon_tags_21 },
    { 0x12968a68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa168a6f, SUC_lexicon_tags_19 },
    { 0xcd968a70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8368a74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda368a7b, SUC_lexicon_tags_15 },
    { 0xfbd68a7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34968a82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb68a86, SUC_lexicon_tags_12 },
    { 0x15f68a87, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbdb68a89, SUC_lexicon_tags_87 },
    { 0x71b68a8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80768a8d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8768a91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cd68a93, SUC_lexicon_tags_2 },
    { 0xca968a93, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x84b68a96, SUC_lexicon_tags_37 },
    { 0x3b68a97, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc2168a99, SUC_lexicon_tags_17 },
    { 0x7f68a99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38768a9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf1b68a9e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa2d68aa0, SUC_lexicon_tags_25 },
    { 0x5ff68aa1, SUC_lexicon_tags_2 },
    { 0xa0968aa2, SUC_lexicon_tags_60 },
    { 0xfad68aa3, SUC_lexicon_tags_11 },
    { 0xc0168aa2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x58168aa6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b68aae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb68ab3, SUC_lexicon_tags_45 },
    { 0x71b68ab4, SUC_lexicon_tags_33 },
    { 0xfa368ab5, SUC_lexicon_tags_37 },
    { 0x27568ab5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99768abc, SUC_lexicon_tags_32 },
    { 0xd6568abc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb168ac8, SUC_lexicon_tags_29 },
    { 0xee568ac9, SUC_lexicon_tags_25 },
    { 0x1b568aca, SUC_lexicon_tags_498 },
    { 0x85968acb, SUC_lexicon_tags_26 },
    { 0x6c768aca, SUC_lexicon_tags_7 },
    { 0xbfd68acc, SUC_lexicon_tags_32 },
    { 0x52568ace, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d968ad2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d68ad6, SUC_lexicon_tags_26 },
    { 0x168ad6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64568adb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc3768add, SUC_lexicon_tags_25 },
    { 0xa1968add, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d68ae3, SUC_lexicon_tags_15 },
    { 0xab768ae4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d368aeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d68af4, SUC_lexicon_tags_12 },
    { 0x3e368af5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac968afb, SUC_lexicon_tags_13 },
    { 0xc3b68afc, SUC_lexicon_tags_4 },
    { 0xa1768afd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d68b02, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x98b68b04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e168b07, SUC_lexicon_tags_45 },
    { 0x1f568b07, SUC_lexicon_tags_25 },
    { 0x89168b09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28768b0d, SUC_lexicon_tags_37 },
    { 0xdcd68b0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6368b15, SUC_lexicon_tags_31 },
    { 0x7e968b16, SUC_lexicon_tags_27 },
    { 0xd0168b16, SUC_lexicon_tags_11 },
    { 0xd1968b18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca968b1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f68b22, SUC_lexicon_tags_2 },
    { 0x2bb68b23, SUC_lexicon_tags_25 },
    { 0xfad68b24, SUC_lexicon_tags_170 },
    { 0x11d68b25, SUC_lexicon_tags_17 },
    { 0x4d68b26, SUC_lexicon_tags_7 },
    { 0xccf68b24, SUC_lexicon_tags_25 },
    { 0x41568b28, SUC_lexicon_tags_19 },
    { 0x1f368b28, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x46168b2b, SUC_lexicon_tags_17 },
    { 0x29968b2c, SUC_lexicon_tags_25 },
    { 0x5168b2c, SUC_lexicon_tags_25 },
    { 0x85d68b2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33b68b30, SUC_lexicon_tags_15 },
    { 0x97168b31, SUC_lexicon_tags_45 },
    { 0xe5f68b31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcc768b34, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16168b3b, SUC_lexicon_tags_87 },
    { 0xf6168b3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf68b44, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1dd68b46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb68b49, SUC_lexicon_tags_7 },
    { 0x4cf68b4a, SUC_lexicon_tags_7 },
    { 0xbb968b4b, SUC_lexicon_tags_42 },
    { 0x2968b4c, SUC_lexicon_tags_27 },
    { 0xca768b4c, SUC_lexicon_tags_29 },
    { 0x53568b4e, SUC_lexicon_tags_12 },
    { 0xaef68b4f, SUC_lexicon_tags_25 },
    { 0x7eb68b50, SUC_lexicon_tags_57 },
    { 0x6fd68b51, SUC_lexicon_tags_37 },
    { 0x6968b4e, SUC_lexicon_tags_46 },
    { 0x21968b53, SUC_lexicon_tags_12 },
    { 0x56b68b4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3168b57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83168b5c, SUC_lexicon_tags_15 },
    { 0xb368b5d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb68b61, SUC_lexicon_tags_66 },
    { 0xbe768b62, SUC_lexicon_tags_68 },
    { 0xac968b62, SUC_lexicon_tags_11 },
    { 0xd0d68b63, SUC_lexicon_tags_7 },
    { 0x98568b61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0968b67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb4b68b69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18768b6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f68b6e, SUC_lexicon_tags_37 },
    { 0x69168b6f, SUC_lexicon_tags_30 },
    { 0xe1168b70, SUC_lexicon_tags_63 },
    { 0xa5968b70, SUC_lexicon_tags_31 },
    { 0x7e168b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd768b76, SUC_lexicon_tags_26 },
    { 0xbef68b76, SUC_lexicon_tags_43 },
    { 0x3dd68b78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85768b7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xecb68b7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f68b7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9e568b81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeb768b83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb968b89, SUC_lexicon_tags_1254 },
    { 0x41568b8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd68b8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77568b8f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x68b68b91, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb68b97, SUC_lexicon_tags_2 },
    { 0x25968b98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79168b9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9d68b9c, SUC_lexicon_tags_33 },
    { 0xcaf68b9d, SUC_lexicon_tags_7 },
    { 0xdd368b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99d68ba0, SUC_lexicon_tags_0 },
    { 0x2d768ba1, SUC_lexicon_tags_11 },
    { 0x74568ba2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d568ba5, SUC_lexicon_tags_26 },
    { 0x90768ba6, SUC_lexicon_tags_69 },
    { 0xdaf68ba6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc3568ba9, SUC_lexicon_tags_112 },
    { 0x61768baa, SUC_lexicon_tags_29 },
    { 0x37768ba9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x92f68bad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d68bb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b768bb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97b68bb5, SUC_lexicon_tags_48 },
    { 0x68768bb5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9368bbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d68bc5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd368bd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1568bd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9b68bd6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15f68bd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x20568bda, SUC_lexicon_tags_25 },
    { 0xc8d68bdb, SUC_lexicon_tags_47 },
    { 0xc6f68bdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5968bde, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc768be3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x49f68be5, SUC_lexicon_tags_38 },
    { 0xb3d68be6, SUC_lexicon_tags_10 },
    { 0x8f768be7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3168bec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c968bef, SUC_lexicon_tags_30 },
    { 0x4d568bf0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4968bf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb768bf5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x79768bf7, SUC_lexicon_tags_33 },
    { 0x16568bf7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d68bfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab68bff, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d568c04, SUC_lexicon_tags_15 },
    { 0xe2b68c05, SUC_lexicon_tags_49 },
    { 0xdcd68c06, SUC_lexicon_tags_35 },
    { 0x3ad68c07, SUC_lexicon_tags_27 },
    { 0xe168c07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x44b68c0a, SUC_lexicon_tags_15 },
    { 0xfa168c0b, SUC_lexicon_tags_1 },
    { 0xb8168c0a, SUC_lexicon_tags_32 },
    { 0xe6368c0b, SUC_lexicon_tags_37 },
    { 0x46b68c0b, SUC_lexicon_tags_8 },
    { 0xe4168c0a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x7c968c11, SUC_lexicon_tags_25 },
    { 0x36768c12, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x8fd68c14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b768c16, SUC_lexicon_tags_25 },
    { 0x25b68c17, SUC_lexicon_tags_11 },
    { 0xe968c18, SUC_lexicon_tags_37 },
    { 0x9db68c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d68c1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x21968c1f, SUC_lexicon_tags_34 },
    { 0xa0968c20, SUC_lexicon_tags_39 },
    { 0x68168c1f, SUC_lexicon_tags_2 },
    { 0x3f568c1f, SUC_lexicon_tags_6 },
    { 0x6ef68c23, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb1168c25, SUC_lexicon_tags_1 },
    { 0xdad68c26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2968c2b, SUC_lexicon_tags_25 },
    { 0x57f68c2c, SUC_lexicon_tags_27 },
    { 0x13b68c2b, SUC_lexicon_tags_28 },
    { 0xebf68c2e, SUC_lexicon_tags_17 },
    { 0xcbd68c2f, SUC_lexicon_tags_66 },
    { 0xc9968c30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d68c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20d68c36, SUC_lexicon_tags_17 },
    { 0xaad68c37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f368c3e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc0968c40, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b168c46, SUC_lexicon_tags_129 },
    { 0x67768c46, SUC_lexicon_tags_12 },
    { 0xdbb68c46, SUC_lexicon_tags_46 },
    { 0x74568c48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3768c4c, SUC_lexicon_tags_84 },
    { 0xfc368c4d, SUC_lexicon_tags_47 },
    { 0xc4968c4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x40d68c50, SUC_lexicon_tags_15 },
    { 0x8fd68c51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x68368c54, SUC_lexicon_tags_37 },
    { 0x5cb68c55, SUC_lexicon_tags_57 },
    { 0xa3168c56, SUC_lexicon_tags_1 },
    { 0x15968c57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89368c59, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d568c60, SUC_lexicon_tags_7 },
    { 0xeaf68c60, SUC_lexicon_tags_59 },
    { 0x4ff68c62, SUC_lexicon_tags_17 },
    { 0x15368c63, SUC_lexicon_tags_66 },
    { 0x71768c62, SUC_lexicon_tags_2 },
    { 0xa9768c65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6968c67, SUC_lexicon_tags_63 },
    { 0xca968c68, SUC_lexicon_tags_26 },
    { 0x75568c69, SUC_lexicon_tags_11 },
    { 0xff968c6a, SUC_lexicon_tags_37 },
    { 0x1c168c6b, SUC_lexicon_tags_84 },
    { 0xb1568c6b, SUC_lexicon_tags_19 },
    { 0x3f168c6d, SUC_lexicon_tags_15 },
    { 0xb1968c68, SUC_lexicon_tags_37 },
    { 0x5d68c6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54368c72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b68c79, SUC_lexicon_tags_28 },
    { 0x99768c7a, SUC_lexicon_tags_14 },
    { 0x37568c7a, SUC_lexicon_tags_7 },
    { 0x9cd68c7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d568c7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b968c81, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc968c84, SUC_lexicon_tags_17 },
    { 0xdb968c85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x76968c87, SUC_lexicon_tags_65 },
    { 0xd168c88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f68c91, SUC_lexicon_tags_12 },
    { 0x98168c92, SUC_lexicon_tags_15 },
    { 0xd6768c93, SUC_lexicon_tags_7 },
    { 0xf8568c94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65568c9c, SUC_lexicon_tags_47 },
    { 0xdb968c9d, SUC_lexicon_tags_49 },
    { 0x9f68c9c, SUC_lexicon_tags_9 },
    { 0xa8d68c9f, SUC_lexicon_tags_47 },
    { 0xba368ca0, SUC_lexicon_tags_15 },
    { 0x5b968ca1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b68ca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf968cac, SUC_lexicon_tags_9 },
    { 0x54368cac, SUC_lexicon_tags_25 },
    { 0x85b68cae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xad768cb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9568cb3, SUC_lexicon_tags_47 },
    { 0x21768cb4, SUC_lexicon_tags_25 },
    { 0x66768cb3, SUC_lexicon_tags_25 },
    { 0x37368cb4, SUC_lexicon_tags_8 },
    { 0x8a968cb7, SUC_lexicon_tags_8 },
    { 0x8eb68cb8, SUC_lexicon_tags_25 },
    { 0x17168cb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd368cbc, SUC_lexicon_tags_30 },
    { 0xe1168cbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xae368cc0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8168cca, SUC_lexicon_tags_10 },
    { 0xa5968ccb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93768cd0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62568cd5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59568cda, SUC_lexicon_tags_65 },
    { 0x81368cda, SUC_lexicon_tags_37 },
    { 0x48b68cdc, SUC_lexicon_tags_27 },
    { 0x60d68cdd, SUC_lexicon_tags_45 },
    { 0x46d68cde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d68ce1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d68ce5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4dd68ce7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db68ceb, SUC_lexicon_tags_66 },
    { 0xdbd68ceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d68cf0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50568d01, SUC_lexicon_tags_17 },
    { 0xe9768d01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c968d0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34368d12, SUC_lexicon_tags_21 },
    { 0x3bf68d12, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80968d19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x31568d1c, SUC_lexicon_tags_49 },
    { 0xb3368d1d, SUC_lexicon_tags_53 },
    { 0x45f68d1d, SUC_lexicon_tags_30 },
    { 0x7ed68d1f, SUC_lexicon_tags_17 },
    { 0xa4d68d1f, SUC_lexicon_tags_15 },
    { 0x42368d1f, SUC_lexicon_tags_2 },
    { 0xe7168d22, SUC_lexicon_tags_47 },
    { 0x92f68d23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38b68d25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f68d28, SUC_lexicon_tags_66 },
    { 0x23368d29, SUC_lexicon_tags_7 },
    { 0xf0d68d2a, SUC_lexicon_tags_2 },
    { 0xf9768d28, SUC_lexicon_tags_14 },
    { 0x25768d2c, SUC_lexicon_tags_1 },
    { 0xecd68d2b, SUC_lexicon_tags_12 },
    { 0x48b68d2e, SUC_lexicon_tags_37 },
    { 0xbed68d2f, SUC_lexicon_tags_60 },
    { 0xa1568d30, SUC_lexicon_tags_25 },
    { 0xecb68d30, SUC_lexicon_tags_37 },
    { 0x37568d2c, SUC_lexicon_tags_0 },
    { 0x3ef68d2f, SUC_lexicon_tags_30 },
    { 0x92b68d34, SUC_lexicon_tags_33 },
    { 0x6f568d35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12768d37, SUC_lexicon_tags_19 },
    { 0xbef68d37, SUC_lexicon_tags_66 },
    { 0xd2168d37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b568d3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f768d41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f68d44, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x6b168d46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x21d68d48, SUC_lexicon_tags_226 },
    { 0xc9368d48, SUC_lexicon_tags_107 },
    { 0xd2568d48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5168d4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb68d51, SUC_lexicon_tags_77 },
    { 0x6db68d51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3768d56, SUC_lexicon_tags_43 },
    { 0x38d68d57, SUC_lexicon_tags_79 },
    { 0x98368d58, SUC_lexicon_tags_4 },
    { 0xdc768d56, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab68d60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16568d64, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60568d68, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xad168d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5768d6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af68d7a, SUC_lexicon_tags_2 },
    { 0x3c568d7b, SUC_lexicon_tags_17 },
    { 0xac968d7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2168d7e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64568d84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51368d86, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf368d89, SUC_lexicon_tags_79 },
    { 0x6db68d89, SUC_lexicon_tags_38 },
    { 0x3f968d8b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1568d8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d68d91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1968d93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b68d99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38568d9d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x23168da0, SUC_lexicon_tags_15 },
    { 0x3f768da0, SUC_lexicon_tags_17 },
    { 0xe6d68da0, SUC_lexicon_tags_17 },
    { 0x86568da3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30568daa, SUC_lexicon_tags_27 },
    { 0x8bf68dab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95968dad, SUC_lexicon_tags_37 },
    { 0x2af68dae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0168db6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8568db9, SUC_lexicon_tags_65 },
    { 0xca968dba, SUC_lexicon_tags_9 },
    { 0x69f68dbb, SUC_lexicon_tags_1 },
    { 0xed368db9, SUC_lexicon_tags_44 },
    { 0x82b68dbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f68dc0, SUC_lexicon_tags_47 },
    { 0x12168dc1, SUC_lexicon_tags_7 },
    { 0x80d68dc1, SUC_lexicon_tags_43 },
    { 0xabf68dc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0968dc7, SUC_lexicon_tags_17 },
    { 0x7dd68dc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d68dce, SUC_lexicon_tags_47 },
    { 0x568dce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44968dd1, SUC_lexicon_tags_17 },
    { 0xa8168dd1, SUC_lexicon_tags_2 },
    { 0x70968dd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87968dd9, SUC_lexicon_tags_63 },
    { 0xc8568dda, SUC_lexicon_tags_7 },
    { 0xf6f68ddb, SUC_lexicon_tags_28 },
    { 0x13568ddb, SUC_lexicon_tags_7 },
    { 0xd3b68ddd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda368de0, SUC_lexicon_tags_11 },
    { 0x91b68de0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4968de3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96568de6, SUC_lexicon_tags_7 },
    { 0x6ab68de6, SUC_lexicon_tags_49 },
    { 0x33968de8, SUC_lexicon_tags_49 },
    { 0xad968de7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a568dec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d68df1, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff568df5, SUC_lexicon_tags_28 },
    { 0x67168df6, SUC_lexicon_tags_37 },
    { 0x72768df7, SUC_lexicon_tags_46 },
    { 0x7d68df7, SUC_lexicon_tags_11 },
    { 0xff768df7, SUC_lexicon_tags_37 },
    { 0x65f68dfa, SUC_lexicon_tags_15 },
    { 0xaf968dfb, SUC_lexicon_tags_7 },
    { 0xc6368dfc, SUC_lexicon_tags_43 },
    { 0xee968dfb, SUC_lexicon_tags_1 },
    { 0x56968dfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x168e05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa8168e07, SUC_lexicon_tags_37 },
    { 0xb2d68e08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15368e0d, SUC_lexicon_tags_15 },
    { 0xf3168e0e, SUC_lexicon_tags_0 },
    { 0x7c968e0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f68e14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81768e18, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd68e23, SUC_lexicon_tags_25 },
    { 0x6ff68e24, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a768e29, SUC_lexicon_tags_22 },
    { 0x58968e2a, SUC_lexicon_tags_61 },
    { 0x88f68e2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14968e2e, SUC_lexicon_tags_11 },
    { 0x43b68e2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x77968e31, SUC_lexicon_tags_25 },
    { 0x95f68e32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb68e35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59168e39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7f168e3b, SUC_lexicon_tags_11 },
    { 0xbb568e3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b68e40, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd68e44, SUC_lexicon_tags_167 },
    { 0x7368e44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6568e47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db68e4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e968e4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f68e54, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff368e58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d68e5e, SUC_lexicon_tags_30 },
    { 0xfdf68e5f, SUC_lexicon_tags_1 },
    { 0x71b68e5f, SUC_lexicon_tags_15 },
    { 0x4bb68e61, SUC_lexicon_tags_87 },
    { 0xe8568e61, SUC_lexicon_tags_29 },
    { 0x5d768e63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15968e6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb168e73, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc2d68e75, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70368e7a, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xd9368e7c, SUC_lexicon_tags_56 },
    { 0xeb68e7d, SUC_lexicon_tags_9 },
    { 0x1a568e7e, SUC_lexicon_tags_37 },
    { 0xc5368e7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8d68e81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1768e85, SUC_lexicon_tags_1 },
    { 0x5cb68e85, SUC_lexicon_tags_9 },
    { 0x3c968e85, SUC_lexicon_tags_49 },
    { 0x24768e87, SUC_lexicon_tags_25 },
    { 0xc3768e88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb68e8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a368e92, SUC_lexicon_tags_63 },
    { 0x57d68e93, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf68e96, SUC_lexicon_tags_43 },
    { 0x3e768e97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b68e9a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93768e9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3768ea3, SUC_lexicon_tags_12 },
    { 0xbcf68ea4, SUC_lexicon_tags_19 },
    { 0xccb68ea5, SUC_lexicon_tags_16 },
    { 0x8d768ea4, SUC_lexicon_tags_12 },
    { 0xafd68ea7, SUC_lexicon_tags_14 },
    { 0x3ab68ea3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef68eaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97568eb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60368eb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d368eb8, SUC_lexicon_tags_2 },
    { 0xd0568eb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18568ebd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72168ec1, SUC_lexicon_tags_45 },
    { 0xce768ec2, SUC_lexicon_tags_25 },
    { 0x26768ec3, SUC_lexicon_tags_11 },
    { 0x12f68ec4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f68ec6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b68ecc, SUC_lexicon_tags_6 },
    { 0xb8368ecd, SUC_lexicon_tags_15 },
    { 0xfbb68ecd, SUC_lexicon_tags_25 },
    { 0xe5568ecf, SUC_lexicon_tags_1 },
    { 0xdf568ecd, SUC_lexicon_tags_49 },
    { 0x1d568ed1, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x52768ed3, SUC_lexicon_tags_11 },
    { 0x6168ed4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xabd68ed6, SUC_lexicon_tags_37 },
    { 0xb968ed7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc968edd, SUC_lexicon_tags_43 },
    { 0xf8768ede, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7968ee4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf68eed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52768ef2, SUC_lexicon_tags_33 },
    { 0x95368ef3, SUC_lexicon_tags_27 },
    { 0x56968ef4, SUC_lexicon_tags_27 },
    { 0xf7f68ef2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb68ef8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36768efa, SUC_lexicon_tags_37 },
    { 0xe9f68efb, SUC_lexicon_tags_4 },
    { 0x3fd68efc, SUC_lexicon_tags_46 },
    { 0x28d68efd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf68f04, SUC_lexicon_tags_60 },
    { 0x94968f04, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde368f09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8368f0c, SUC_lexicon_tags_40 },
    { 0x1b168f0d, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0x79568f0f, SUC_lexicon_tags_15 },
    { 0x1a168f10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b68f17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94568f1c, SUC_lexicon_tags_17 },
    { 0x65b68f1c, SUC_lexicon_tags_39 },
    { 0x33b68f1c, SUC_lexicon_tags_15 },
    { 0x91168f1f, SUC_lexicon_tags_15 },
    { 0xc4768f20, SUC_lexicon_tags_1 },
    { 0xed368f21, SUC_lexicon_tags_60 },
    { 0x3db68f22, SUC_lexicon_tags_63 },
    { 0x58b68f22, SUC_lexicon_tags_32 },
    { 0x40168f21, SUC_lexicon_tags_19 },
    { 0x38368f25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x91168f28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe168f2c, SUC_lexicon_tags_7 },
    { 0xdc768f2d, SUC_lexicon_tags_7 },
    { 0xe9568f2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd68f33, SUC_lexicon_tags_37 },
    { 0xa168f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cb68f36, SUC_lexicon_tags_6 },
    { 0xd7368f36, SUC_lexicon_tags_91 },
    { 0x31d68f37, SUC_lexicon_tags_26 },
    { 0x5e968f39, SUC_lexicon_tags_26 },
    { 0x1d568f3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f68f3e, SUC_lexicon_tags_55 },
    { 0x9a768f3f, SUC_lexicon_tags_172 },
    { 0x6df68f3f, SUC_lexicon_tags_14 },
    { 0xc768f3f, SUC_lexicon_tags_12 },
    { 0x14568f41, SUC_lexicon_tags_47 },
    { 0x8568f43, SUC_lexicon_tags_45 },
    { 0x50d68f43, SUC_lexicon_tags_15 },
    { 0x6d368f45, SUC_lexicon_tags_9 },
    { 0x11b68f44, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e368f4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c568f5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11968f5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0768f63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df68f66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe1368f68, SUC_lexicon_tags_28 },
    { 0x7d968f69, SUC_lexicon_tags_1 },
    { 0x3768f68, SUC_lexicon_tags_10 },
    { 0xd7768f69, SUC_lexicon_tags_15 },
    { 0xfc768f6c, SUC_lexicon_tags_11 },
    { 0x6bb68f69, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7568f71, SUC_lexicon_tags_12 },
    { 0xa7f68f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbd68f74, SUC_lexicon_tags_12 },
    { 0x6b968f75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f768f7a, SUC_lexicon_tags_82 },
    { 0x44368f7b, SUC_lexicon_tags_6 },
    { 0x19d68f7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed68f88, SUC_lexicon_tags_17 },
    { 0xf4168f89, SUC_lexicon_tags_37 },
    { 0xf0568f8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3768f8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd68f91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40168f94, SUC_lexicon_tags_33 },
    { 0xea568f95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd368f97, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9ef68f99, SUC_lexicon_tags_40 },
    { 0x8ef68f99, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8bd68f9c, SUC_lexicon_tags_11 },
    { 0xc168f9d, SUC_lexicon_tags_26 },
    { 0x6b168f9e, SUC_lexicon_tags_17 },
    { 0xdcb68f9c, SUC_lexicon_tags_17 },
    { 0x38368fa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87168fa2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4968fa4, SUC_lexicon_tags_37 },
    { 0x9568fa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4f68fa7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x23168fa9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e768fb0, SUC_lexicon_tags_14 },
    { 0xa9f68fb0, SUC_lexicon_tags_3 },
    { 0x86368fb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5968fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb68fb9, SUC_lexicon_tags_14 },
    { 0x1b968fb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b68fbf, SUC_lexicon_tags_6 },
    { 0xaa768fc0, SUC_lexicon_tags_37 },
    { 0x3a368fc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd968fc3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb68fc7, SUC_lexicon_tags_58 },
    { 0xdb768fc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ad68fca, SUC_lexicon_tags_48 },
    { 0xb9168fcb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d568fce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f68fd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89368fd3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f568fd6, SUC_lexicon_tags_13 },
    { 0x72168fd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35968fdb, SUC_lexicon_tags_25 },
    { 0x59168fdb, SUC_lexicon_tags_23 },
    { 0x9a168fdd, SUC_lexicon_tags_2 },
    { 0xd4768fde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52d68fe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87968fe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac168fee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f68ff5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xce368ff8, SUC_lexicon_tags_37 },
    { 0xd5d68ff8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60368ffd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c569000, SUC_lexicon_tags_45 },
    { 0xd4b69000, SUC_lexicon_tags_26 },
    { 0xa4369002, SUC_lexicon_tags_17 },
    { 0x40969003, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d69007, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d6900a, SUC_lexicon_tags_23 },
    { 0xd896900b, SUC_lexicon_tags_11 },
    { 0x8896900c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b6900f, SUC_lexicon_tags_2 },
    { 0x3a16900f, SUC_lexicon_tags_17 },
    { 0x296900f, SUC_lexicon_tags_39 },
    { 0x2f569012, SUC_lexicon_tags_63 },
    { 0x8c769011, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1f69015, SUC_lexicon_tags_8 },
    { 0xe0369016, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d69019, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x176901c, SUC_lexicon_tags_47 },
    { 0x8d36901c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x816901f, SUC_lexicon_tags_4 },
    { 0x46b6901f, SUC_lexicon_tags_23 },
    { 0x54969021, SUC_lexicon_tags_1 },
    { 0xff569021, SUC_lexicon_tags_160 },
    { 0x4f769023, SUC_lexicon_tags_37 },
    { 0xc7f69023, SUC_lexicon_tags_37 },
    { 0x81d69025, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd169027, SUC_lexicon_tags_9 },
    { 0xbbf69027, SUC_lexicon_tags_23 },
    { 0x84d69028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2eb6902b, SUC_lexicon_tags_2 },
    { 0x596902c, SUC_lexicon_tags_31 },
    { 0xc9f6902c, SUC_lexicon_tags_12 },
    { 0x34f6902c, SUC_lexicon_tags_17 },
    { 0x6476902c, SUC_lexicon_tags_54 },
    { 0xae569030, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa8569032, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46169036, SUC_lexicon_tags_47 },
    { 0xbc169037, SUC_lexicon_tags_60 },
    { 0xf7569038, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f6903b, SUC_lexicon_tags_49 },
    { 0xb576903b, SUC_lexicon_tags_12 },
    { 0x7036903b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9796903f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d69042, SUC_lexicon_tags_18 },
    { 0x5369043, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9a969045, SUC_lexicon_tags_19 },
    { 0x94169046, SUC_lexicon_tags_47 },
    { 0x5a769047, SUC_lexicon_tags_43 },
    { 0xe5d69048, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6336904c, SUC_lexicon_tags_2 },
    { 0x24b6904c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82169053, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f69056, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xde569059, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5836905d, SUC_lexicon_tags_2 },
    { 0xcf76905e, SUC_lexicon_tags_45 },
    { 0x9a56905f, SUC_lexicon_tags_11 },
    { 0xbe369060, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae769062, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x1f769064, SUC_lexicon_tags_17 },
    { 0xd0b69065, SUC_lexicon_tags_45 },
    { 0x79d69065, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee16906a, SUC_lexicon_tags_35 },
    { 0xeab6906b, SUC_lexicon_tags_14 },
    { 0x4db6906b, SUC_lexicon_tags_15 },
    { 0x6f16906d, SUC_lexicon_tags_43 },
    { 0xcbd6906d, SUC_lexicon_tags_17 },
    { 0x6d96906f, SUC_lexicon_tags_29 },
    { 0x6d769070, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff69077, SUC_lexicon_tags_25 },
    { 0xfa969078, SUC_lexicon_tags_29 },
    { 0x8e769077, SUC_lexicon_tags_7 },
    { 0x28569079, SUC_lexicon_tags_31 },
    { 0xf6b6907b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb6907e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xea169080, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e369082, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2969084, SUC_lexicon_tags_12 },
    { 0x3d69084, SUC_lexicon_tags_25 },
    { 0x3af69086, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5ed69088, SUC_lexicon_tags_12 },
    { 0xf1769088, SUC_lexicon_tags_11 },
    { 0x4776908a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf6908e, SUC_lexicon_tags_1 },
    { 0x9b6908f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c969093, SUC_lexicon_tags_17 },
    { 0x8a969094, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0769097, SUC_lexicon_tags_11 },
    { 0xff769098, SUC_lexicon_tags_38 },
    { 0x5c769098, SUC_lexicon_tags_61 },
    { 0x1a56909a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb6909f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53690a3, SUC_lexicon_tags_33 },
    { 0xee1690a4, SUC_lexicon_tags_25 },
    { 0x46d690a5, SUC_lexicon_tags_15 },
    { 0x13f690a6, SUC_lexicon_tags_19 },
    { 0x45690a5, SUC_lexicon_tags_31 },
    { 0x7ab690a8, SUC_lexicon_tags_27 },
    { 0x121690a9, SUC_lexicon_tags_35 },
    { 0xb09690aa, SUC_lexicon_tags_46 },
    { 0x6e9690a4, SUC_lexicon_tags_11 },
    { 0x531690ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb690af, SUC_lexicon_tags_12 },
    { 0x7bf690af, SUC_lexicon_tags_31 },
    { 0xcb9690b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc77690b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505690b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253690bc, SUC_lexicon_tags_37 },
    { 0x9e3690bd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b690c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85690c3, SUC_lexicon_tags_17 },
    { 0x57b690c3, SUC_lexicon_tags_15 },
    { 0xe95690c5, SUC_lexicon_tags_60 },
    { 0x949690c6, SUC_lexicon_tags_11 },
    { 0x767690c6, SUC_lexicon_tags_47 },
    { 0xff1690c7, SUC_lexicon_tags_17 },
    { 0x25690c9, SUC_lexicon_tags_17 },
    { 0xd690c7, SUC_lexicon_tags_15 },
    { 0xd1d690cb, SUC_lexicon_tags_12 },
    { 0xac7690ca, SUC_lexicon_tags_34 },
    { 0xee7690cd, SUC_lexicon_tags_6 },
    { 0x265690ce, SUC_lexicon_tags_1 },
    { 0x451690cf, SUC_lexicon_tags_25 },
    { 0x889690cf, SUC_lexicon_tags_21 },
    { 0x7f1690c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3690d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d690d8, SUC_lexicon_tags_42 },
    { 0xbab690d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7690de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d690e5, SUC_lexicon_tags_28 },
    { 0x175690e6, SUC_lexicon_tags_10 },
    { 0xbcf690e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x427690e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85690f2, SUC_lexicon_tags_11 },
    { 0xe47690f3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab690fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563690ff, SUC_lexicon_tags_39 },
    { 0xa6169100, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d169103, SUC_lexicon_tags_25 },
    { 0x63f69103, SUC_lexicon_tags_31 },
    { 0x4eb69104, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd16910c, SUC_lexicon_tags_27 },
    { 0x1096910d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca76910f, SUC_lexicon_tags_31 },
    { 0x4c36910f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad969114, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6a769116, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xec169118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf69120, SUC_lexicon_tags_201 },
    { 0x6d369120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1369123, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x16369125, SUC_lexicon_tags_17 },
    { 0xa7369126, SUC_lexicon_tags_7 },
    { 0x59369127, SUC_lexicon_tags_59 },
    { 0x12969126, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6c56912a, SUC_lexicon_tags_17 },
    { 0xa496912b, SUC_lexicon_tags_15 },
    { 0xd6f6912b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6912f, SUC_lexicon_tags_26 },
    { 0x33d6912f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57569133, SUC_lexicon_tags_45 },
    { 0x13769134, SUC_lexicon_tags_25 },
    { 0xbed69135, SUC_lexicon_tags_63 },
    { 0xe8569136, SUC_lexicon_tags_14 },
    { 0xcf969135, SUC_lexicon_tags_27 },
    { 0xed369136, SUC_lexicon_tags_68 },
    { 0x69769139, SUC_lexicon_tags_49 },
    { 0x6a369139, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf016913d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x96369140, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d69142, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x9c169144, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36d69146, SUC_lexicon_tags_49 },
    { 0x76f69146, SUC_lexicon_tags_45 },
    { 0xc0169148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed6914c, SUC_lexicon_tags_1 },
    { 0xcdb6914d, SUC_lexicon_tags_49 },
    { 0x41f6914e, SUC_lexicon_tags_4 },
    { 0x7356914d, SUC_lexicon_tags_37 },
    { 0x38f69150, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb3f69152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcf69154, SUC_lexicon_tags_37 },
    { 0x18569155, SUC_lexicon_tags_15 },
    { 0x2569155, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f69159, SUC_lexicon_tags_0 },
    { 0x92b69159, SUC_lexicon_tags_7 },
    { 0xc916915b, SUC_lexicon_tags_17 },
    { 0xff6915c, SUC_lexicon_tags_15 },
    { 0x7d96915c, SUC_lexicon_tags_28 },
    { 0xc6d6915e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90169160, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1c169162, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c969168, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd6916b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5736916e, SUC_lexicon_tags_3 },
    { 0xc456916f, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd69174, SUC_lexicon_tags_49 },
    { 0xa7369175, SUC_lexicon_tags_25 },
    { 0x72169174, SUC_lexicon_tags_12 },
    { 0x70169177, SUC_lexicon_tags_38 },
    { 0xf8d69178, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9056917b, SUC_lexicon_tags_2 },
    { 0x2756917c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b6917f, SUC_lexicon_tags_42 },
    { 0x43569180, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf369182, SUC_lexicon_tags_17 },
    { 0xe6b69182, SUC_lexicon_tags_1 },
    { 0xa9f69182, SUC_lexicon_tags_64 },
    { 0x38369185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f6918c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3cf6918e, SUC_lexicon_tags_23 },
    { 0x8bd6918f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7969191, SUC_lexicon_tags_7 },
    { 0xd5d69192, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xccd69194, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7769197, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd776919a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb691a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x437691a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3691a8, SUC_lexicon_tags_2 },
    { 0xd25691a9, SUC_lexicon_tags_1 },
    { 0xbfb691aa, SUC_lexicon_tags_15 },
    { 0xa15691ab, SUC_lexicon_tags_1 },
    { 0xf11691ac, SUC_lexicon_tags_33 },
    { 0xc75691a9, SUC_lexicon_tags_11 },
    { 0x745691ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3691b3, SUC_lexicon_tags_45 },
    { 0xa55691b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37691b7, SUC_lexicon_tags_14 },
    { 0x2f5691b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad691bd, SUC_lexicon_tags_37 },
    { 0x77d691be, SUC_lexicon_tags_14 },
    { 0xd05691bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1691c6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f691c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x221691cc, SUC_lexicon_tags_19 },
    { 0x849691cc, SUC_lexicon_tags_14 },
    { 0x2d5691ce, SUC_lexicon_tags_13 },
    { 0xffb691cf, SUC_lexicon_tags_23 },
    { 0xf23691d0, SUC_lexicon_tags_77 },
    { 0xe15691d1, SUC_lexicon_tags_60 },
    { 0x389691d2, SUC_lexicon_tags_9 },
    { 0xaef691d3, SUC_lexicon_tags_17 },
    { 0x2e1691d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01691da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x113691dc, SUC_lexicon_tags_17 },
    { 0xe23691dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d691e2, SUC_lexicon_tags_7 },
    { 0x681691e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309691ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x903691ec, SUC_lexicon_tags_25 },
    { 0xa0d691ed, SUC_lexicon_tags_12 },
    { 0xcb7691ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17691f1, SUC_lexicon_tags_234 },
    { 0x7ed691f1, SUC_lexicon_tags_37 },
    { 0xf47691f2, SUC_lexicon_tags_2 },
    { 0xd691f3, SUC_lexicon_tags_23 },
    { 0x7f5691f5, SUC_lexicon_tags_37 },
    { 0x733691f6, SUC_lexicon_tags_77 },
    { 0x137691f6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd691fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4b3691fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x211691ff, SUC_lexicon_tags_27 },
    { 0x5b569200, SUC_lexicon_tags_25 },
    { 0xaf569201, SUC_lexicon_tags_14 },
    { 0x2569202, SUC_lexicon_tags_15 },
    { 0xe0d69203, SUC_lexicon_tags_0 },
    { 0x80969203, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa2169206, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3516920b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d69210, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdc569212, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f69215, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf69219, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77969221, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2569223, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6569229, SUC_lexicon_tags_12 },
    { 0xc4b6922a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f6922f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28969232, SUC_lexicon_tags_38 },
    { 0xd0d69233, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d6923a, SUC_lexicon_tags_26 },
    { 0x84d6923b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf6923f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcaf69241, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98769246, SUC_lexicon_tags_60 },
    { 0xce169246, SUC_lexicon_tags_17 },
    { 0x83169248, SUC_lexicon_tags_43 },
    { 0xf6569249, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9256924d, SUC_lexicon_tags_13 },
    { 0x2796924e, SUC_lexicon_tags_1 },
    { 0x4f56924e, SUC_lexicon_tags_29 },
    { 0xbb969250, SUC_lexicon_tags_11 },
    { 0x9b969251, SUC_lexicon_tags_47 },
    { 0x5ad6924e, SUC_lexicon_tags_57 },
    { 0x83969250, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a569259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9316925b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9056925e, SUC_lexicon_tags_34 },
    { 0xf716925f, SUC_lexicon_tags_7 },
    { 0xa3d6925f, SUC_lexicon_tags_31 },
    { 0x3196925e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85769266, SUC_lexicon_tags_27 },
    { 0x6cd69266, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaed69269, SUC_lexicon_tags_25 },
    { 0xb876926a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1836926d, SUC_lexicon_tags_48 },
    { 0xbb6926e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df69271, SUC_lexicon_tags_80 },
    { 0x3c569272, SUC_lexicon_tags_20 },
    { 0x80169271, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f6927d, SUC_lexicon_tags_6 },
    { 0xc2f6927e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb69281, SUC_lexicon_tags_8 },
    { 0xe4169282, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0569284, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd4d69286, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0x62969289, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2d6928b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xaa76928d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c56928f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdeb69291, SUC_lexicon_tags_11 },
    { 0xe0769292, SUC_lexicon_tags_17 },
    { 0x63169293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd69299, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf6929c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc336929f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3692a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff692a5, SUC_lexicon_tags_2 },
    { 0x683692a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937692ad, SUC_lexicon_tags_14 },
    { 0xd35692ae, SUC_lexicon_tags_23 },
    { 0xe19692af, SUC_lexicon_tags_747 },
    { 0x177692b0, SUC_lexicon_tags_23 },
    { 0x9f1692ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7692b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x225692b6, SUC_lexicon_tags_85 },
    { 0x1a7692b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x557692b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x18b692bb, SUC_lexicon_tags_37 },
    { 0xddb692bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd4f692be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d7692c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b692c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23692c8, SUC_lexicon_tags_12 },
    { 0x2c7692c8, SUC_lexicon_tags_49 },
    { 0xaef692c8, SUC_lexicon_tags_30 },
    { 0x247692cb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x615692ce, SUC_lexicon_tags_4 },
    { 0xb15692ce, SUC_lexicon_tags_43 },
    { 0xf67692cf, SUC_lexicon_tags_17 },
    { 0x375692d1, SUC_lexicon_tags_1 },
    { 0xb1f692d2, SUC_lexicon_tags_46 },
    { 0xaf1692d2, SUC_lexicon_tags_17 },
    { 0xbc1692d4, SUC_lexicon_tags_26 },
    { 0x449692d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f692de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95692e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccd692e6, SUC_lexicon_tags_15 },
    { 0xa81692e7, SUC_lexicon_tags_57 },
    { 0x1d1692e8, SUC_lexicon_tags_37 },
    { 0x31b692e9, SUC_lexicon_tags_12 },
    { 0x199692ea, SUC_lexicon_tags_43 },
    { 0x273692ea, SUC_lexicon_tags_39 },
    { 0xa75692ec, SUC_lexicon_tags_33 },
    { 0xa15692e8, SUC_lexicon_tags_14 },
    { 0x5ff692e6, SUC_lexicon_tags_2 },
    { 0x34d692ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b1692f1, SUC_lexicon_tags_45 },
    { 0x44b692f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d692f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x937692f9, SUC_lexicon_tags_2 },
    { 0x37692fa, SUC_lexicon_tags_77 },
    { 0xcff692fb, SUC_lexicon_tags_37 },
    { 0xf23692f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae169302, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d69305, SUC_lexicon_tags_0 },
    { 0x85369305, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xec969308, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6e76930a, SUC_lexicon_tags_49 },
    { 0xb676930a, SUC_lexicon_tags_25 },
    { 0xf2b6930c, SUC_lexicon_tags_1577 },
    { 0xff6930d, SUC_lexicon_tags_37 },
    { 0x3116930e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b69314, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62969319, SUC_lexicon_tags_33 },
    { 0x6f76931a, SUC_lexicon_tags_11 },
    { 0xaeb6931a, SUC_lexicon_tags_46 },
    { 0x4316931c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed69322, SUC_lexicon_tags_2 },
    { 0x56d69322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66b69325, SUC_lexicon_tags_25 },
    { 0xc4769326, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d6932b, SUC_lexicon_tags_29 },
    { 0x71b6932b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc569332, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a969339, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8f16933b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5996933d, SUC_lexicon_tags_28 },
    { 0x1996933d, SUC_lexicon_tags_25 },
    { 0x1db6933d, SUC_lexicon_tags_11 },
    { 0xca76933f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9d969342, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x94769344, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78769349, SUC_lexicon_tags_17 },
    { 0x3f6934a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea56934e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db69351, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58569354, SUC_lexicon_tags_2 },
    { 0x5f769355, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f16935d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef69363, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb69368, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd6936f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x59569372, SUC_lexicon_tags_37 },
    { 0x62169373, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4169378, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa56937c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e16937e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b69381, SUC_lexicon_tags_61 },
    { 0xdff69382, SUC_lexicon_tags_37 },
    { 0x36169381, SUC_lexicon_tags_1 },
    { 0x22569381, SUC_lexicon_tags_7 },
    { 0x40169384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f69389, SUC_lexicon_tags_17 },
    { 0xff169389, SUC_lexicon_tags_710 },
    { 0, NULL },
    { 0x17b6938c, SUC_lexicon_tags_37 },
    { 0x8436938d, SUC_lexicon_tags_27 },
    { 0x5af6938c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd69391, SUC_lexicon_tags_60 },
    { 0x8d169392, SUC_lexicon_tags_15 },
    { 0x5b569393, SUC_lexicon_tags_45 },
    { 0x7f169393, SUC_lexicon_tags_9 },
    { 0x28d69395, SUC_lexicon_tags_17 },
    { 0x79169395, SUC_lexicon_tags_37 },
    { 0xf569396, SUC_lexicon_tags_66 },
    { 0x33969398, SUC_lexicon_tags_37 },
    { 0x84569398, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xf256939b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f693a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1693a9, SUC_lexicon_tags_87 },
    { 0x989693a9, SUC_lexicon_tags_38 },
    { 0x6a9693ab, SUC_lexicon_tags_6 },
    { 0xf9d693ac, SUC_lexicon_tags_7 },
    { 0xfc7693ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1693b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323693b5, SUC_lexicon_tags_60 },
    { 0xadd693b6, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0x959693b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d693bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x917693bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x167693c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1b693c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7693c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2ef693c8, SUC_lexicon_tags_15 },
    { 0x899693c8, SUC_lexicon_tags_8 },
    { 0x801693ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13b693cc, SUC_lexicon_tags_33 },
    { 0x6c3693cc, SUC_lexicon_tags_79 },
    { 0x39693ce, SUC_lexicon_tags_0 },
    { 0x783693cd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9693d3, SUC_lexicon_tags_37 },
    { 0x57b693d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5693d8, SUC_lexicon_tags_21 },
    { 0xe6f693d8, SUC_lexicon_tags_11 },
    { 0x5df693da, SUC_lexicon_tags_126 },
    { 0xb93693d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b693de, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611693e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd693e4, SUC_lexicon_tags_37 },
    { 0x40f693e4, SUC_lexicon_tags_25 },
    { 0xc1693e6, SUC_lexicon_tags_16 },
    { 0x7f7693e7, SUC_lexicon_tags_28 },
    { 0xc8f693e8, SUC_lexicon_tags_33 },
    { 0x865693e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d693f1, SUC_lexicon_tags_17 },
    { 0x3bd693f2, SUC_lexicon_tags_37 },
    { 0xcf9693f3, SUC_lexicon_tags_61 },
    { 0x613693f4, SUC_lexicon_tags_2 },
    { 0xfa7693f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a7693f7, SUC_lexicon_tags_43 },
    { 0xee7693f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d693fb, SUC_lexicon_tags_30 },
    { 0xd47693fb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x751693fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13769402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d69405, SUC_lexicon_tags_12 },
    { 0xea969406, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa6169408, SUC_lexicon_tags_28 },
    { 0xd0369409, SUC_lexicon_tags_1 },
    { 0xb36940a, SUC_lexicon_tags_9 },
    { 0x5fb69409, SUC_lexicon_tags_11 },
    { 0x23d6940c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xea36940e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda169415, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x2a969417, SUC_lexicon_tags_29 },
    { 0xf6169417, SUC_lexicon_tags_1 },
    { 0x32769418, SUC_lexicon_tags_23 },
    { 0x14769417, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13f6941c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd69420, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccf69422, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f969426, SUC_lexicon_tags_12 },
    { 0x58169427, SUC_lexicon_tags_25 },
    { 0x97569428, SUC_lexicon_tags_17 },
    { 0xfcf69428, SUC_lexicon_tags_25 },
    { 0x42b69428, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6942d, SUC_lexicon_tags_30 },
    { 0xec56942d, SUC_lexicon_tags_47 },
    { 0x9bf6942d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7169432, SUC_lexicon_tags_27 },
    { 0xf6f69432, SUC_lexicon_tags_64 },
    { 0x40d69434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b69438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb196943d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb36944b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f6944e, SUC_lexicon_tags_4 },
    { 0x9eb6944f, SUC_lexicon_tags_37 },
    { 0x83969450, SUC_lexicon_tags_1 },
    { 0x3c969451, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x62169453, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b69456, SUC_lexicon_tags_63 },
    { 0x71369456, SUC_lexicon_tags_14 },
    { 0xeb969458, SUC_lexicon_tags_11 },
    { 0xc1169457, SUC_lexicon_tags_39 },
    { 0xda16945a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f69467, SUC_lexicon_tags_2 },
    { 0x10169468, SUC_lexicon_tags_11 },
    { 0x3f369468, SUC_lexicon_tags_56 },
    { 0xe7b69467, SUC_lexicon_tags_15 },
    { 0xa3f6946b, SUC_lexicon_tags_27 },
    { 0xcf369468, SUC_lexicon_tags_49 },
    { 0x6b96946d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23569471, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf69474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba569478, SUC_lexicon_tags_9 },
    { 0xb8f69479, SUC_lexicon_tags_45 },
    { 0x3bd6947a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e96947f, SUC_lexicon_tags_7 },
    { 0x60369480, SUC_lexicon_tags_2 },
    { 0x4d569480, SUC_lexicon_tags_23 },
    { 0x71769482, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf6948c, SUC_lexicon_tags_15 },
    { 0x5ef6948d, SUC_lexicon_tags_25 },
    { 0xda16948e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb69496, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9b69498, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5376949a, SUC_lexicon_tags_12 },
    { 0x4af6949b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1676949d, SUC_lexicon_tags_50 },
    { 0x1996949e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3b694a0, SUC_lexicon_tags_49 },
    { 0xbaf694a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1694a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5eb694a6, SUC_lexicon_tags_11 },
    { 0x181694a7, SUC_lexicon_tags_15 },
    { 0x6eb694a7, SUC_lexicon_tags_77 },
    { 0xec7694a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1f694ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2f7694ad, SUC_lexicon_tags_15 },
    { 0x22f694ad, SUC_lexicon_tags_10 },
    { 0xbf1694af, SUC_lexicon_tags_8 },
    { 0x847694ae, SUC_lexicon_tags_28 },
    { 0xbdd694b1, SUC_lexicon_tags_6 },
    { 0x3b7694b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417694b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x361694ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79694c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b694c4, SUC_lexicon_tags_26 },
    { 0xba7694c5, SUC_lexicon_tags_23 },
    { 0x771694c6, SUC_lexicon_tags_15 },
    { 0x2b3694c6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d694cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37694cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcb9694d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x969694d3, SUC_lexicon_tags_49 },
    { 0xe23694d3, SUC_lexicon_tags_11 },
    { 0x8b3694d4, SUC_lexicon_tags_25 },
    { 0x2d7694d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf05694d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcd5694da, SUC_lexicon_tags_37 },
    { 0x1d3694db, SUC_lexicon_tags_7 },
    { 0x223694dc, SUC_lexicon_tags_14 },
    { 0x6f9694dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7694e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb694e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa89694e7, SUC_lexicon_tags_6 },
    { 0xe57694e8, SUC_lexicon_tags_11 },
    { 0x731694e9, SUC_lexicon_tags_38 },
    { 0xa87694ea, SUC_lexicon_tags_15 },
    { 0xf69694eb, SUC_lexicon_tags_28 },
    { 0xbeb694ea, SUC_lexicon_tags_17 },
    { 0xe55694e9, SUC_lexicon_tags_37 },
    { 0x96b694ee, SUC_lexicon_tags_238 },
    { 0xd43694ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3694f2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa57694f4, SUC_lexicon_tags_37 },
    { 0x395694f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3c5694f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301694ff, SUC_lexicon_tags_5 },
    { 0xa8169500, SUC_lexicon_tags_17 },
    { 0xd7f69501, SUC_lexicon_tags_33 },
    { 0x5df69502, SUC_lexicon_tags_2 },
    { 0x48569503, SUC_lexicon_tags_1 },
    { 0x60969504, SUC_lexicon_tags_14 },
    { 0x88f69504, SUC_lexicon_tags_54 },
    { 0xdd69505, SUC_lexicon_tags_14 },
    { 0x19b69502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e169509, SUC_lexicon_tags_15 },
    { 0x8216950a, SUC_lexicon_tags_9 },
    { 0x5256950b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff6950f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d69514, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb16951e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4569522, SUC_lexicon_tags_32 },
    { 0x41969523, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7296952b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2756952d, SUC_lexicon_tags_29 },
    { 0x4176952d, SUC_lexicon_tags_48 },
    { 0xd9f6952e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d69538, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d69542, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x10369544, SUC_lexicon_tags_37 },
    { 0x94569545, SUC_lexicon_tags_28 },
    { 0xf4369544, SUC_lexicon_tags_30 },
    { 0xb5d69546, SUC_lexicon_tags_36 },
    { 0xbd569545, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb6954c, SUC_lexicon_tags_12 },
    { 0xff56954d, SUC_lexicon_tags_60 },
    { 0xc7f6954c, SUC_lexicon_tags_12 },
    { 0x7a16954f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd69556, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f6955a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3696955c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c76955e, SUC_lexicon_tags_27 },
    { 0xcb16955e, SUC_lexicon_tags_37 },
    { 0x3569560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90169566, SUC_lexicon_tags_85 },
    { 0x1f569567, SUC_lexicon_tags_6 },
    { 0x5ad69566, SUC_lexicon_tags_61 },
    { 0x15f69566, SUC_lexicon_tags_39 },
    { 0x5636956a, SUC_lexicon_tags_14 },
    { 0xa916956b, SUC_lexicon_tags_3 },
    { 0x5076956b, SUC_lexicon_tags_9 },
    { 0xc6d6956d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f69570, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68769577, SUC_lexicon_tags_17 },
    { 0xba369578, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb796957d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94969581, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b69587, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e16958e, SUC_lexicon_tags_28 },
    { 0x556958f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21569595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d69598, SUC_lexicon_tags_29 },
    { 0x51969599, SUC_lexicon_tags_30 },
    { 0xbd569599, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6396959c, SUC_lexicon_tags_33 },
    { 0xb436959d, SUC_lexicon_tags_33 },
    { 0x52b6959c, SUC_lexicon_tags_15 },
    { 0x4a76959e, SUC_lexicon_tags_462 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541695a8, SUC_lexicon_tags_4 },
    { 0x44f695a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bf695ab, SUC_lexicon_tags_15 },
    { 0x351695ac, SUC_lexicon_tags_31 },
    { 0x233695ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x511695af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b695b4, SUC_lexicon_tags_15 },
    { 0xc43695b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53695b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd7695bb, SUC_lexicon_tags_23 },
    { 0x82b695bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87695c1, SUC_lexicon_tags_17 },
    { 0x69d695c1, SUC_lexicon_tags_37 },
    { 0xac7695c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b3695c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x927695c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f695cb, SUC_lexicon_tags_46 },
    { 0xb0b695cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe09695ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967695d6, SUC_lexicon_tags_45 },
    { 0x53b695d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x389695da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d695e8, SUC_lexicon_tags_85 },
    { 0x38d695e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1695f5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7695f8, SUC_lexicon_tags_165 },
    { 0x2f3695f9, SUC_lexicon_tags_11 },
    { 0x641695fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15695fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f69601, SUC_lexicon_tags_37 },
    { 0x9af69602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd69606, SUC_lexicon_tags_26 },
    { 0xef769607, SUC_lexicon_tags_29 },
    { 0x9c569608, SUC_lexicon_tags_2 },
    { 0xe2d69609, SUC_lexicon_tags_25 },
    { 0x596960a, SUC_lexicon_tags_25 },
    { 0x7096960b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23569611, SUC_lexicon_tags_35 },
    { 0x82969612, SUC_lexicon_tags_12 },
    { 0x37569612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94169616, SUC_lexicon_tags_12 },
    { 0xdb569616, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98369619, SUC_lexicon_tags_14 },
    { 0xf656961a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xbd16961c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb536961e, SUC_lexicon_tags_7 },
    { 0x5756961f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9969622, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x21169624, SUC_lexicon_tags_44 },
    { 0x3b169625, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x26969628, SUC_lexicon_tags_37 },
    { 0x8e569629, SUC_lexicon_tags_7 },
    { 0xbcf69628, SUC_lexicon_tags_43 },
    { 0x36d6962b, SUC_lexicon_tags_16 },
    { 0x86f6962c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf916962e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31169633, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfe169635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30569639, SUC_lexicon_tags_47 },
    { 0x92f69639, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3916963c, SUC_lexicon_tags_7 },
    { 0x2b96963c, SUC_lexicon_tags_63 },
    { 0x5356963c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf69640, SUC_lexicon_tags_19 },
    { 0xb4969641, SUC_lexicon_tags_12 },
    { 0xb0769641, SUC_lexicon_tags_38 },
    { 0x22969643, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70969646, SUC_lexicon_tags_15 },
    { 0xffd69646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df6964c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2f36964e, SUC_lexicon_tags_26 },
    { 0xa616964f, SUC_lexicon_tags_12 },
    { 0x7cd69650, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde969653, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64369658, SUC_lexicon_tags_43 },
    { 0xceb69658, SUC_lexicon_tags_16 },
    { 0x1d56965a, SUC_lexicon_tags_32 },
    { 0x2516965b, SUC_lexicon_tags_93 },
    { 0x39569658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72969662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1369666, SUC_lexicon_tags_29 },
    { 0x56969666, SUC_lexicon_tags_11 },
    { 0xbbf69667, SUC_lexicon_tags_47 },
    { 0x18d69669, SUC_lexicon_tags_6 },
    { 0xe7969667, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6a36966c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd16966f, SUC_lexicon_tags_1 },
    { 0xc0b69670, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c569678, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f6967e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35369682, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d69686, SUC_lexicon_tags_37 },
    { 0xe5369687, SUC_lexicon_tags_47 },
    { 0xc7369686, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf6968e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb169690, SUC_lexicon_tags_17 },
    { 0x8cd69690, SUC_lexicon_tags_15 },
    { 0xf5f69692, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed69695, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e969698, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc56969b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7696a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf696a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd696a8, SUC_lexicon_tags_37 },
    { 0x369696a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261696ad, SUC_lexicon_tags_37 },
    { 0x19f696ae, SUC_lexicon_tags_37 },
    { 0xf51696ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25696b3, SUC_lexicon_tags_0 },
    { 0x20d696b3, SUC_lexicon_tags_9 },
    { 0x297696b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393696b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcb1696bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85696bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b696c2, SUC_lexicon_tags_129 },
    { 0x7c7696c3, SUC_lexicon_tags_30 },
    { 0xec7696c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf7696c6, SUC_lexicon_tags_25 },
    { 0x61f696c7, SUC_lexicon_tags_21 },
    { 0x227696c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93696d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7696d5, SUC_lexicon_tags_23 },
    { 0x98f696d6, SUC_lexicon_tags_66 },
    { 0xdd7696d7, SUC_lexicon_tags_47 },
    { 0x973696d8, SUC_lexicon_tags_12 },
    { 0x715696d8, SUC_lexicon_tags_47 },
    { 0x663696da, SUC_lexicon_tags_21 },
    { 0x94b696d8, SUC_lexicon_tags_61 },
    { 0xdd9696da, SUC_lexicon_tags_2 },
    { 0xfd7696da, SUC_lexicon_tags_11 },
    { 0x5bb696dc, SUC_lexicon_tags_2 },
    { 0xe93696da, SUC_lexicon_tags_11 },
    { 0xe39696d8, SUC_lexicon_tags_31 },
    { 0xadb696e1, SUC_lexicon_tags_37 },
    { 0xf81696e1, SUC_lexicon_tags_37 },
    { 0x539696e1, SUC_lexicon_tags_17 },
    { 0xd5d696d5, SUC_lexicon_tags_12 },
    { 0x883696e5, SUC_lexicon_tags_11 },
    { 0x697696df, SUC_lexicon_tags_17 },
    { 0x48d696d6, SUC_lexicon_tags_9 },
    { 0xb7b696e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d696ed, SUC_lexicon_tags_17 },
    { 0xa23696ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb1696f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549696f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9696fc, SUC_lexicon_tags_23 },
    { 0x8b9696fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b69704, SUC_lexicon_tags_25 },
    { 0x20f69705, SUC_lexicon_tags_37 },
    { 0x73f69706, SUC_lexicon_tags_38 },
    { 0x29d69707, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x31169709, SUC_lexicon_tags_21 },
    { 0x37f6970a, SUC_lexicon_tags_34 },
    { 0x92f6970b, SUC_lexicon_tags_1 },
    { 0xcf6970c, SUC_lexicon_tags_27 },
    { 0x51969709, SUC_lexicon_tags_43 },
    { 0xa256970e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3969716, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb896971b, SUC_lexicon_tags_56 },
    { 0xa796971b, SUC_lexicon_tags_26 },
    { 0xa5f6971d, SUC_lexicon_tags_45 },
    { 0x7df6971e, SUC_lexicon_tags_7 },
    { 0xbb6971c, SUC_lexicon_tags_6 },
    { 0x2556971d, SUC_lexicon_tags_2 },
    { 0x27d69721, SUC_lexicon_tags_25 },
    { 0x7369722, SUC_lexicon_tags_84 },
    { 0x8bb69722, SUC_lexicon_tags_2 },
    { 0xfb69722, SUC_lexicon_tags_65 },
    { 0x9b76971e, SUC_lexicon_tags_15 },
    { 0x1cd69724, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8056972b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab169730, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb0969732, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b69736, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f76973b, SUC_lexicon_tags_2 },
    { 0xded6973c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b69743, SUC_lexicon_tags_15 },
    { 0x18569744, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f569746, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2076974b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9d6974d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf756974f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b369757, SUC_lexicon_tags_37 },
    { 0x13d69758, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6976975c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95569760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1969763, SUC_lexicon_tags_10 },
    { 0x5d769763, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c169766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e56976a, SUC_lexicon_tags_14 },
    { 0x46d6976b, SUC_lexicon_tags_12 },
    { 0x6d76976a, SUC_lexicon_tags_33 },
    { 0x63f6976d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40769771, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6569775, SUC_lexicon_tags_21 },
    { 0x74f69775, SUC_lexicon_tags_28 },
    { 0xeab69777, SUC_lexicon_tags_2 },
    { 0xd2b69777, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab6977b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x89b6977d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1db6977f, SUC_lexicon_tags_25 },
    { 0xd3f6977f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84b69782, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf69786, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb176978b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6978e, SUC_lexicon_tags_84 },
    { 0x6836978f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4969792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f69795, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x10b69797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3969799, SUC_lexicon_tags_8 },
    { 0xff16979a, SUC_lexicon_tags_17 },
    { 0x9256979a, SUC_lexicon_tags_23 },
    { 0x62f6979c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1697a2, SUC_lexicon_tags_124 },
    { 0xa33697a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d697a6, SUC_lexicon_tags_21 },
    { 0xb5697a7, SUC_lexicon_tags_1 },
    { 0x7a5697a6, SUC_lexicon_tags_25 },
    { 0x96d697a9, SUC_lexicon_tags_15 },
    { 0x3ad697aa, SUC_lexicon_tags_6 },
    { 0xb99697ab, SUC_lexicon_tags_29 },
    { 0x2c9697ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x821697ae, SUC_lexicon_tags_11 },
    { 0xb21697af, SUC_lexicon_tags_7 },
    { 0x945697af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f697b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5697b8, SUC_lexicon_tags_25 },
    { 0x8f3697b8, SUC_lexicon_tags_2 },
    { 0x14b697ba, SUC_lexicon_tags_27 },
    { 0x6e1697b9, SUC_lexicon_tags_11 },
    { 0x19b697bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27697c4, SUC_lexicon_tags_17 },
    { 0x7c5697c5, SUC_lexicon_tags_17 },
    { 0xde3697c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34f697c8, SUC_lexicon_tags_25 },
    { 0xa67697c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503697cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3697d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b697d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6d697d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x127697d9, SUC_lexicon_tags_37 },
    { 0x19f697da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393697de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b697e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55697e6, SUC_lexicon_tags_49 },
    { 0xf85697e6, SUC_lexicon_tags_63 },
    { 0x4e7697e8, SUC_lexicon_tags_17 },
    { 0xb15697e9, SUC_lexicon_tags_84 },
    { 0xb79697ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x567697ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x879697ef, SUC_lexicon_tags_2 },
    { 0x801697f0, SUC_lexicon_tags_12 },
    { 0xa9697f1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33697f8, SUC_lexicon_tags_30 },
    { 0xe05697f9, SUC_lexicon_tags_43 },
    { 0x7db697f9, SUC_lexicon_tags_25 },
    { 0x609697fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee9697fd, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x99b697ff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc569803, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b769806, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0569808, SUC_lexicon_tags_26 },
    { 0xf1369809, SUC_lexicon_tags_17 },
    { 0x49569808, SUC_lexicon_tags_14 },
    { 0x73d6980b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23d6980d, SUC_lexicon_tags_31 },
    { 0x4496980e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a769814, SUC_lexicon_tags_14 },
    { 0x96b69815, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xac169818, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d6981b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2169823, SUC_lexicon_tags_11 },
    { 0x73169824, SUC_lexicon_tags_62 },
    { 0xa8969823, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67569829, SUC_lexicon_tags_63 },
    { 0x31f69829, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b56982d, SUC_lexicon_tags_12 },
    { 0x9af6982e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ed69830, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9169834, SUC_lexicon_tags_25 },
    { 0xf7b69835, SUC_lexicon_tags_47 },
    { 0x32569836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d6983e, SUC_lexicon_tags_18 },
    { 0x9996983f, SUC_lexicon_tags_15 },
    { 0x8756983f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b69844, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x2e369846, SUC_lexicon_tags_7 },
    { 0x58769846, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9c769849, SUC_lexicon_tags_15 },
    { 0x5656984a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8716984f, SUC_lexicon_tags_138 },
    { 0x7c569850, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79f69852, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf69855, SUC_lexicon_tags_107 },
    { 0xf169856, SUC_lexicon_tags_21 },
    { 0x16169857, SUC_lexicon_tags_17 },
    { 0xf2b69858, SUC_lexicon_tags_16 },
    { 0x68169857, SUC_lexicon_tags_7 },
    { 0xaed69857, SUC_lexicon_tags_4 },
    { 0x49d6985b, SUC_lexicon_tags_11 },
    { 0x856985c, SUC_lexicon_tags_97 },
    { 0x34f6985b, SUC_lexicon_tags_60 },
    { 0x47b6985e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd69861, SUC_lexicon_tags_64 },
    { 0xb69861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ff69864, SUC_lexicon_tags_1 },
    { 0xb5169865, SUC_lexicon_tags_11 },
    { 0xd3b69865, SUC_lexicon_tags_27 },
    { 0x42b69867, SUC_lexicon_tags_43 },
    { 0x8e169868, SUC_lexicon_tags_11 },
    { 0x28369869, SUC_lexicon_tags_37 },
    { 0x9d969869, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x636986c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc769871, SUC_lexicon_tags_23 },
    { 0x1dd69871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6d69874, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf969876, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc076987e, SUC_lexicon_tags_49 },
    { 0xfdb6987f, SUC_lexicon_tags_9 },
    { 0x5b16987f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31569886, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc36988a, SUC_lexicon_tags_11 },
    { 0x8af6988b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb6988f, SUC_lexicon_tags_49 },
    { 0x23369890, SUC_lexicon_tags_6 },
    { 0x1ed69891, SUC_lexicon_tags_56 },
    { 0x8d969892, SUC_lexicon_tags_7 },
    { 0x7016988f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b6989a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef6989d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2a96989f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5698ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9698af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79698b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5698b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07698c0, SUC_lexicon_tags_2 },
    { 0xb8b698c1, SUC_lexicon_tags_13 },
    { 0xbbd698c1, SUC_lexicon_tags_17 },
    { 0x707698c3, SUC_lexicon_tags_11 },
    { 0x9db698c3, SUC_lexicon_tags_2 },
    { 0xbf5698c3, SUC_lexicon_tags_37 },
    { 0x7d9698c6, SUC_lexicon_tags_15 },
    { 0x5f698c5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1e9698c9, SUC_lexicon_tags_30 },
    { 0xdb9698c9, SUC_lexicon_tags_25 },
    { 0x5cd698ca, SUC_lexicon_tags_7 },
    { 0xf03698cc, SUC_lexicon_tags_32 },
    { 0x6c1698cd, SUC_lexicon_tags_12 },
    { 0x5cf698cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b698d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5698da, SUC_lexicon_tags_7 },
    { 0x9b9698db, SUC_lexicon_tags_17 },
    { 0x3c5698db, SUC_lexicon_tags_9 },
    { 0x6c9698db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9698e2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc19698e4, SUC_lexicon_tags_12 },
    { 0xef5698e5, SUC_lexicon_tags_43 },
    { 0xad7698e6, SUC_lexicon_tags_69 },
    { 0x34b698e4, SUC_lexicon_tags_60 },
    { 0xfad698e8, SUC_lexicon_tags_11 },
    { 0xd2b698e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d1698eb, SUC_lexicon_tags_11 },
    { 0x7c3698ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b698ef, SUC_lexicon_tags_49 },
    { 0x4fb698f0, SUC_lexicon_tags_12 },
    { 0xe43698f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27698f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6db698f8, SUC_lexicon_tags_79 },
    { 0xbb7698f9, SUC_lexicon_tags_11 },
    { 0xba7698fa, SUC_lexicon_tags_47 },
    { 0x6e9698fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3698ff, SUC_lexicon_tags_62 },
    { 0x1a5698ff, SUC_lexicon_tags_12 },
    { 0x52b69901, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c969903, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2a969905, SUC_lexicon_tags_45 },
    { 0xd8d69906, SUC_lexicon_tags_11 },
    { 0x6369907, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa969910, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x90d69912, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42769916, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cf69918, SUC_lexicon_tags_38 },
    { 0xb1d69919, SUC_lexicon_tags_19 },
    { 0x49b6991a, SUC_lexicon_tags_33 },
    { 0xef56991b, SUC_lexicon_tags_9 },
    { 0x5ad6991b, SUC_lexicon_tags_87 },
    { 0xd8d69918, SUC_lexicon_tags_27 },
    { 0x2116991e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1969923, SUC_lexicon_tags_37 },
    { 0xdc169924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7169926, SUC_lexicon_tags_10 },
    { 0xbad69926, SUC_lexicon_tags_17 },
    { 0xb7369926, SUC_lexicon_tags_17 },
    { 0x25f69929, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b6992c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2af6992e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x87769931, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d69935, SUC_lexicon_tags_62 },
    { 0x3dd69936, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3636993f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xd769941, SUC_lexicon_tags_33 },
    { 0x45b69942, SUC_lexicon_tags_60 },
    { 0xaa169943, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x44569946, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x27369948, SUC_lexicon_tags_49 },
    { 0xd1969949, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x99f6994b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf6994e, SUC_lexicon_tags_2 },
    { 0x2e96994f, SUC_lexicon_tags_28 },
    { 0xeb169950, SUC_lexicon_tags_11 },
    { 0x46369951, SUC_lexicon_tags_33 },
    { 0x8996994f, SUC_lexicon_tags_39 },
    { 0x36569950, SUC_lexicon_tags_28 },
    { 0x86569954, SUC_lexicon_tags_2 },
    { 0x72569952, SUC_lexicon_tags_27 },
    { 0x6ad69956, SUC_lexicon_tags_11 },
    { 0xa1369957, SUC_lexicon_tags_11 },
    { 0xdff69957, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x18b6995a, SUC_lexicon_tags_27 },
    { 0xbe76995a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e36995e, SUC_lexicon_tags_19 },
    { 0xffb6995f, SUC_lexicon_tags_17 },
    { 0xa8969960, SUC_lexicon_tags_28 },
    { 0x1236995e, SUC_lexicon_tags_14 },
    { 0xe1b69960, SUC_lexicon_tags_63 },
    { 0xa9169963, SUC_lexicon_tags_15 },
    { 0xc569960, SUC_lexicon_tags_15 },
    { 0x73969965, SUC_lexicon_tags_1 },
    { 0xcff69966, SUC_lexicon_tags_14 },
    { 0x75569962, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d6996a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd6996d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc336996f, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d69973, SUC_lexicon_tags_38 },
    { 0x2cd69973, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff769978, SUC_lexicon_tags_68 },
    { 0xb2b69979, SUC_lexicon_tags_8 },
    { 0x82d69979, SUC_lexicon_tags_45 },
    { 0xa0d6997b, SUC_lexicon_tags_11 },
    { 0x6776997c, SUC_lexicon_tags_31 },
    { 0xc336997d, SUC_lexicon_tags_17 },
    { 0xe896997a, SUC_lexicon_tags_1 },
    { 0xd56997a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30769984, SUC_lexicon_tags_14 },
    { 0xc5369985, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x14569988, SUC_lexicon_tags_29 },
    { 0x54769988, SUC_lexicon_tags_12 },
    { 0xd9f69989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce769996, SUC_lexicon_tags_33 },
    { 0x90369997, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf6999e, SUC_lexicon_tags_1 },
    { 0x51f6999f, SUC_lexicon_tags_14 },
    { 0x4a5699a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64f699a2, SUC_lexicon_tags_30 },
    { 0x85699a3, SUC_lexicon_tags_31 },
    { 0xf9699a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x461699a6, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xdbd699a8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x88b699aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcbb699ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5699b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cd699b8, SUC_lexicon_tags_11 },
    { 0xb6f699b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa69699bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf699bf, SUC_lexicon_tags_19 },
    { 0x8d3699c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7699c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f5699c8, SUC_lexicon_tags_56 },
    { 0x4cd699c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9699ce, SUC_lexicon_tags_15 },
    { 0x155699cf, SUC_lexicon_tags_17 },
    { 0xd85699ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbb5699d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbd699d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x243699d6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa55699d8, SUC_lexicon_tags_29 },
    { 0x7c7699d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x605699db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b699de, SUC_lexicon_tags_9 },
    { 0x665699de, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x127699e2, SUC_lexicon_tags_17 },
    { 0x2ad699e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x293699e5, SUC_lexicon_tags_1 },
    { 0xa57699e6, SUC_lexicon_tags_25 },
    { 0x6e1699e5, SUC_lexicon_tags_21 },
    { 0x29f699e8, SUC_lexicon_tags_2 },
    { 0x9c3699e8, SUC_lexicon_tags_1 },
    { 0x35b699ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9b699ec, SUC_lexicon_tags_29 },
    { 0xbf1699ed, SUC_lexicon_tags_17 },
    { 0x9df699ec, SUC_lexicon_tags_57 },
    { 0xb65699ef, SUC_lexicon_tags_38 },
    { 0x6ed699f0, SUC_lexicon_tags_37 },
    { 0xc0b699f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb1699f3, SUC_lexicon_tags_4 },
    { 0xff9699f3, SUC_lexicon_tags_1 },
    { 0xca7699f5, SUC_lexicon_tags_17 },
    { 0x8cd699f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d369a01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x81769a03, SUC_lexicon_tags_25 },
    { 0xb8d69a03, SUC_lexicon_tags_231 },
    { 0x9d969a05, SUC_lexicon_tags_68 },
    { 0xb3b69a05, SUC_lexicon_tags_59 },
    { 0xe9169a04, SUC_lexicon_tags_38 },
    { 0x85969a05, SUC_lexicon_tags_45 },
    { 0xeb769a04, SUC_lexicon_tags_37 },
    { 0xe9b69a0a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x28d69a0c, SUC_lexicon_tags_11 },
    { 0x7e969a0c, SUC_lexicon_tags_17 },
    { 0xab169a0e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90169a15, SUC_lexicon_tags_15 },
    { 0x11b69a15, SUC_lexicon_tags_48 },
    { 0x96769a15, SUC_lexicon_tags_11 },
    { 0xd4169a18, SUC_lexicon_tags_27 },
    { 0x76b69a15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b69a1c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d69a1f, SUC_lexicon_tags_8 },
    { 0xabf69a1f, SUC_lexicon_tags_55 },
    { 0x6fd69a21, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xe969a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e769a26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39369a2b, SUC_lexicon_tags_28 },
    { 0xa4d69a2c, SUC_lexicon_tags_47 },
    { 0x62969a2d, SUC_lexicon_tags_37 },
    { 0x5169a2e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd69a32, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1369a36, SUC_lexicon_tags_95 },
    { 0x45969a37, SUC_lexicon_tags_11 },
    { 0xd8969a37, SUC_lexicon_tags_17 },
    { 0x98369a39, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15569a3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf69a41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf69a45, SUC_lexicon_tags_1210 },
    { 0xab969a46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4d369a48, SUC_lexicon_tags_7 },
    { 0x4569a49, SUC_lexicon_tags_27 },
    { 0x94169a4a, SUC_lexicon_tags_23 },
    { 0x4169a4a, SUC_lexicon_tags_151 },
    { 0x20969a4c, SUC_lexicon_tags_12 },
    { 0x96769a4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f69a52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b69a56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e969a59, SUC_lexicon_tags_3 },
    { 0x9d569a5a, SUC_lexicon_tags_12 },
    { 0xaa369a59, SUC_lexicon_tags_37 },
    { 0xa2b69a5a, SUC_lexicon_tags_27 },
    { 0x4b369a5d, SUC_lexicon_tags_37 },
    { 0xcf69a5e, SUC_lexicon_tags_19 },
    { 0xfd169a5e, SUC_lexicon_tags_11 },
    { 0x91b69a5a, SUC_lexicon_tags_1 },
    { 0x9cf69a61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd769a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72769a69, SUC_lexicon_tags_68 },
    { 0xf9369a69, SUC_lexicon_tags_45 },
    { 0x46569a6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76369a6d, SUC_lexicon_tags_15 },
    { 0x75769a6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8369a78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x57769a7b, SUC_lexicon_tags_60 },
    { 0x18169a7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x50769a7f, SUC_lexicon_tags_14 },
    { 0x30969a80, SUC_lexicon_tags_37 },
    { 0xbe369a81, SUC_lexicon_tags_37 },
    { 0x5d569a82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33769a85, SUC_lexicon_tags_17 },
    { 0x8c969a86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6569a88, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xd3769a8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c169a8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11169a91, SUC_lexicon_tags_45 },
    { 0xa0369a91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9969a94, SUC_lexicon_tags_14 },
    { 0xde369a95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef69a98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x86569a9b, SUC_lexicon_tags_11 },
    { 0x97369a9c, SUC_lexicon_tags_25 },
    { 0x2e769a9d, SUC_lexicon_tags_2 },
    { 0x6f969a9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b69aa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60769aad, SUC_lexicon_tags_30 },
    { 0x52569aae, SUC_lexicon_tags_32 },
    { 0xa9f69aaf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d69ab4, SUC_lexicon_tags_26 },
    { 0x5e569ab5, SUC_lexicon_tags_26 },
    { 0x8bf69ab6, SUC_lexicon_tags_53 },
    { 0x4a169ab6, SUC_lexicon_tags_26 },
    { 0xfa569ab8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d69abd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c569ac0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb69ac3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a969ac7, SUC_lexicon_tags_6 },
    { 0x25569ac8, SUC_lexicon_tags_116 },
    { 0x68f69ac9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b69acc, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x19969ace, SUC_lexicon_tags_15 },
    { 0x5cd69ace, SUC_lexicon_tags_7 },
    { 0xd4b69ad0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd769ad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfd69ada, SUC_lexicon_tags_17 },
    { 0x5769ada, SUC_lexicon_tags_29 },
    { 0xd4b69adc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f69ae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d369af8, SUC_lexicon_tags_25 },
    { 0xe0f69af8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c769afd, SUC_lexicon_tags_34 },
    { 0x84769afe, SUC_lexicon_tags_9 },
    { 0x94d69afe, SUC_lexicon_tags_37 },
    { 0x42f69b00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3969b03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17969b06, SUC_lexicon_tags_169 },
    { 0xfd969b06, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x85f69b09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62d69b0b, SUC_lexicon_tags_43 },
    { 0x75969b0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df69b0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82569b12, SUC_lexicon_tags_27 },
    { 0x62769b13, SUC_lexicon_tags_7 },
    { 0x38769b12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83569b16, SUC_lexicon_tags_12 },
    { 0xa0d69b17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8969b1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xebd69b1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab69b1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b69b22, SUC_lexicon_tags_54 },
    { 0xa3969b23, SUC_lexicon_tags_45 },
    { 0x3c769b22, SUC_lexicon_tags_32 },
    { 0x4b569b22, SUC_lexicon_tags_15 },
    { 0x13f69b23, SUC_lexicon_tags_68 },
    { 0x2c569b27, SUC_lexicon_tags_9 },
    { 0x95169b27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d369b2c, SUC_lexicon_tags_45 },
    { 0xf9f69b2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6369b33, SUC_lexicon_tags_47 },
    { 0x84769b34, SUC_lexicon_tags_47 },
    { 0x79769b34, SUC_lexicon_tags_47 },
    { 0x9cf69b34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52969b3b, SUC_lexicon_tags_66 },
    { 0x8ab69b3c, SUC_lexicon_tags_11 },
    { 0xff69b3c, SUC_lexicon_tags_47 },
    { 0x3db69b3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef69b43, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb0769b45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdd769b47, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x95f69b49, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3c769b4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5769b4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x72769b50, SUC_lexicon_tags_0 },
    { 0x2b569b51, SUC_lexicon_tags_11 },
    { 0xed369b52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5d69b54, SUC_lexicon_tags_17 },
    { 0xd8969b54, SUC_lexicon_tags_19 },
    { 0x9dd69b56, SUC_lexicon_tags_49 },
    { 0x16969b55, SUC_lexicon_tags_15 },
    { 0xab769b56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1d969b5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d69b5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b569b62, SUC_lexicon_tags_63 },
    { 0x9f369b63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd69b68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89769b6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ed69b70, SUC_lexicon_tags_65 },
    { 0x73569b70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26969b78, SUC_lexicon_tags_2 },
    { 0x9af69b79, SUC_lexicon_tags_63 },
    { 0x49769b7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b69b7d, SUC_lexicon_tags_17 },
    { 0x8f369b7e, SUC_lexicon_tags_821 },
    { 0x8569b7f, SUC_lexicon_tags_54 },
    { 0x5f969b80, SUC_lexicon_tags_4 },
    { 0x4bd69b81, SUC_lexicon_tags_14 },
    { 0xcf369b7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67369b88, SUC_lexicon_tags_1 },
    { 0xc4d69b88, SUC_lexicon_tags_33 },
    { 0xf0769b89, SUC_lexicon_tags_11 },
    { 0x1f969b8b, SUC_lexicon_tags_16 },
    { 0x81569b8b, SUC_lexicon_tags_12 },
    { 0xdf369b8b, SUC_lexicon_tags_47 },
    { 0xe1169b8c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb69b93, SUC_lexicon_tags_26 },
    { 0xe5b69b94, SUC_lexicon_tags_31 },
    { 0x65f69b94, SUC_lexicon_tags_37 },
    { 0xf369b96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab769b99, SUC_lexicon_tags_6 },
    { 0xd6b69b9a, SUC_lexicon_tags_19 },
    { 0x2bf69b9b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x24969b9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11369ba2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60769ba6, SUC_lexicon_tags_351 },
    { 0x5369ba7, SUC_lexicon_tags_33 },
    { 0xeb969ba7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad69bae, SUC_lexicon_tags_48 },
    { 0x2f569bae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f369bb5, SUC_lexicon_tags_25 },
    { 0x43f69bb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15569bbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x31f69bbd, SUC_lexicon_tags_8 },
    { 0x18b69bbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2569bc1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb969bc6, SUC_lexicon_tags_27 },
    { 0x1e369bc6, SUC_lexicon_tags_65 },
    { 0xe1b69bc6, SUC_lexicon_tags_38 },
    { 0xba369bc9, SUC_lexicon_tags_7 },
    { 0xf3169bc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4569bd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7b969bd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d69bd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0769bdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18169be8, SUC_lexicon_tags_60 },
    { 0x24969be9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb969beb, SUC_lexicon_tags_2 },
    { 0x55169beb, SUC_lexicon_tags_25 },
    { 0x99369bed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8169bf2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15369bf7, SUC_lexicon_tags_12 },
    { 0x92369bf8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc569bfa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4369c01, SUC_lexicon_tags_1 },
    { 0x46969c01, SUC_lexicon_tags_12 },
    { 0xf0369c01, SUC_lexicon_tags_15 },
    { 0xfe769c04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71d69c06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x85369c08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48969c0d, SUC_lexicon_tags_17 },
    { 0x79b69c0e, SUC_lexicon_tags_45 },
    { 0xd2d69c0e, SUC_lexicon_tags_11 },
    { 0x55969c10, SUC_lexicon_tags_198 },
    { 0x19b69c11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1769c17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f69c19, SUC_lexicon_tags_11 },
    { 0x87969c1a, SUC_lexicon_tags_66 },
    { 0xcfd69c1a, SUC_lexicon_tags_11 },
    { 0x8eb69c1b, SUC_lexicon_tags_0 },
    { 0xe3b69c1c, SUC_lexicon_tags_66 },
    { 0x94369c1e, SUC_lexicon_tags_19 },
    { 0x32d69c1f, SUC_lexicon_tags_10 },
    { 0x4b569c1f, SUC_lexicon_tags_4 },
    { 0xf9969c21, SUC_lexicon_tags_47 },
    { 0x4969c1f, SUC_lexicon_tags_17 },
    { 0xb2569c1d, SUC_lexicon_tags_11 },
    { 0xe9b69c20, SUC_lexicon_tags_2 },
    { 0x39169c25, SUC_lexicon_tags_0 },
    { 0x42369c1a, SUC_lexicon_tags_35 },
    { 0x4fb69c27, SUC_lexicon_tags_1 },
    { 0x6b69c28, SUC_lexicon_tags_12 },
    { 0x82f69c1a, SUC_lexicon_tags_1 },
    { 0xab769c29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b569c2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13369c31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10169c33, SUC_lexicon_tags_39 },
    { 0xf3369c33, SUC_lexicon_tags_21 },
    { 0xa0f69c35, SUC_lexicon_tags_96 },
    { 0x1a569c36, SUC_lexicon_tags_25 },
    { 0x6e569c37, SUC_lexicon_tags_8 },
    { 0xa7569c38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d69c3b, SUC_lexicon_tags_0 },
    { 0x5b769c3c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3f969c3e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a969c41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb69c44, SUC_lexicon_tags_7 },
    { 0xcc569c45, SUC_lexicon_tags_2 },
    { 0xadf69c46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78769c49, SUC_lexicon_tags_17 },
    { 0x3769c4a, SUC_lexicon_tags_33 },
    { 0xc169c4a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x6d569c4d, SUC_lexicon_tags_60 },
    { 0xdfb69c4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xed169c51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb69c5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2169c67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef369c69, SUC_lexicon_tags_2 },
    { 0xb3769c6a, SUC_lexicon_tags_2 },
    { 0xc2169c6b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ad69c6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f169c72, SUC_lexicon_tags_10 },
    { 0x3a969c73, SUC_lexicon_tags_45 },
    { 0x80969c72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd69c77, SUC_lexicon_tags_38 },
    { 0xd9369c78, SUC_lexicon_tags_15 },
    { 0x3f69c77, SUC_lexicon_tags_49 },
    { 0x3c569c79, SUC_lexicon_tags_30 },
    { 0x99f69c7b, SUC_lexicon_tags_14 },
    { 0x37369c7c, SUC_lexicon_tags_17 },
    { 0xc1f69c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1bd69c7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb69c81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f169c84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff369c86, SUC_lexicon_tags_66 },
    { 0x57969c87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda569c8d, SUC_lexicon_tags_27 },
    { 0x26769c8d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd69c93, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xab369c96, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed69c99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3d69c9b, SUC_lexicon_tags_25 },
    { 0x19f69c9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce169c9e, SUC_lexicon_tags_65 },
    { 0x9b969c9f, SUC_lexicon_tags_12 },
    { 0x20569ca0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad69ca3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x34769ca5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21169ca9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x19769cab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7169cb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ad69cb2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b69cb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52769cb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6569cba, SUC_lexicon_tags_25 },
    { 0xf3769cbb, SUC_lexicon_tags_27 },
    { 0x18d69cba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe4f69cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44369cc2, SUC_lexicon_tags_30 },
    { 0x4c569cc3, SUC_lexicon_tags_786 },
    { 0x92369cc3, SUC_lexicon_tags_8 },
    { 0x44969cc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5769cc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3769cc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ed69ccb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1369ccd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3769cd1, SUC_lexicon_tags_23 },
    { 0x9d969cd2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4369cd8, SUC_lexicon_tags_25 },
    { 0xe0769cd8, SUC_lexicon_tags_81 },
    { 0xa3969cda, SUC_lexicon_tags_37 },
    { 0x8c169cda, SUC_lexicon_tags_1 },
    { 0x8d69cdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad69ce1, SUC_lexicon_tags_17 },
    { 0xe8369ce1, SUC_lexicon_tags_12 },
    { 0x6c769ce2, SUC_lexicon_tags_62 },
    { 0x32769ce4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73569ced, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1bb69cef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x93769cf1, SUC_lexicon_tags_33 },
    { 0x86f69cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ad69cf4, SUC_lexicon_tags_12 },
    { 0x3e169cf5, SUC_lexicon_tags_37 },
    { 0xbff69cf6, SUC_lexicon_tags_79 },
    { 0x3d369cf5, SUC_lexicon_tags_14 },
    { 0xc3169cf6, SUC_lexicon_tags_34 },
    { 0xff69cf9, SUC_lexicon_tags_43 },
    { 0xb4169cfa, SUC_lexicon_tags_1 },
    { 0x8769cf9, SUC_lexicon_tags_23 },
    { 0x82569cfc, SUC_lexicon_tags_1 },
    { 0xbe969cfd, SUC_lexicon_tags_38 },
    { 0x26f69cfc, SUC_lexicon_tags_11 },
    { 0xa0d69cff, SUC_lexicon_tags_37 },
    { 0xa7d69cfd, SUC_lexicon_tags_9 },
    { 0x45f69cfd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x34169d03, SUC_lexicon_tags_16 },
    { 0x7369d04, SUC_lexicon_tags_25 },
    { 0xff169d05, SUC_lexicon_tags_8 },
    { 0x35369d06, SUC_lexicon_tags_11 },
    { 0x3169d06, SUC_lexicon_tags_4 },
    { 0x54969d08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4d69d0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f569d0e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f69d14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf169d16, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x41169d18, SUC_lexicon_tags_6 },
    { 0xc3569d19, SUC_lexicon_tags_8 },
    { 0x16d69d1a, SUC_lexicon_tags_15 },
    { 0xd8569d1a, SUC_lexicon_tags_12 },
    { 0x6c369d1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb69d1f, SUC_lexicon_tags_31 },
    { 0xe5f69d1f, SUC_lexicon_tags_25 },
    { 0x9d169d20, SUC_lexicon_tags_25 },
    { 0xc8b69d22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b69d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad69d2c, SUC_lexicon_tags_11 },
    { 0x29b69d2c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf769d36, SUC_lexicon_tags_11 },
    { 0x3d169d37, SUC_lexicon_tags_2 },
    { 0x4ef69d38, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x38569d3a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b69d3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf0b69d3f, SUC_lexicon_tags_11 },
    { 0x23569d3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54369d42, SUC_lexicon_tags_11 },
    { 0xd69d43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f569d47, SUC_lexicon_tags_38 },
    { 0x70369d48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df69d4b, SUC_lexicon_tags_101 },
    { 0x62969d4b, SUC_lexicon_tags_19 },
    { 0xa0d69d4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x51f69d4f, SUC_lexicon_tags_46 },
    { 0x8df69d50, SUC_lexicon_tags_33 },
    { 0xc4b69d51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x969d53, SUC_lexicon_tags_15 },
    { 0xcd769d54, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x15d69d56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d69d5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6969d5f, SUC_lexicon_tags_6 },
    { 0x8b169d60, SUC_lexicon_tags_64 },
    { 0x9a969d61, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf4f69d63, SUC_lexicon_tags_12 },
    { 0x6f69d63, SUC_lexicon_tags_14 },
    { 0x13569d65, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b69d69, SUC_lexicon_tags_35 },
    { 0xccb69d6a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe4b69d6c, SUC_lexicon_tags_2 },
    { 0x35b69d6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74169d6f, SUC_lexicon_tags_38 },
    { 0x71369d6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d69d73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf69d76, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x84769d79, SUC_lexicon_tags_91 },
    { 0x42f69d7a, SUC_lexicon_tags_33 },
    { 0xb769d7b, SUC_lexicon_tags_11 },
    { 0x40f69d7c, SUC_lexicon_tags_29 },
    { 0x9dd69d7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd69d80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71369d84, SUC_lexicon_tags_7 },
    { 0x95169d85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26169d8a, SUC_lexicon_tags_12 },
    { 0x93769d8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23769d8f, SUC_lexicon_tags_33 },
    { 0xeb69d90, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x64769d93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x52569d95, SUC_lexicon_tags_11 },
    { 0xccf69d96, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f69d99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf69d9c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29969da5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f69dab, SUC_lexicon_tags_38 },
    { 0x9a169dac, SUC_lexicon_tags_31 },
    { 0xf0569dac, SUC_lexicon_tags_37 },
    { 0xceb69dae, SUC_lexicon_tags_25 },
    { 0x22169dae, SUC_lexicon_tags_27 },
    { 0x4b969db0, SUC_lexicon_tags_19 },
    { 0x21369db1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f69db4, SUC_lexicon_tags_38 },
    { 0x9f569db4, SUC_lexicon_tags_47 },
    { 0x9bf69db4, SUC_lexicon_tags_37 },
    { 0x19f69db7, SUC_lexicon_tags_37 },
    { 0x5bf69db8, SUC_lexicon_tags_43 },
    { 0x96169db5, SUC_lexicon_tags_19 },
    { 0x6e769dba, SUC_lexicon_tags_17 },
    { 0x3b769dba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6569dbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x26769dbf, SUC_lexicon_tags_17 },
    { 0x9b769dbf, SUC_lexicon_tags_43 },
    { 0xb3d69dc1, SUC_lexicon_tags_2 },
    { 0xcf769dc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdab69dc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87169dc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3369dcc, SUC_lexicon_tags_33 },
    { 0xc8769dcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9df69dcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89369dd2, SUC_lexicon_tags_84 },
    { 0xc1369dd3, SUC_lexicon_tags_0 },
    { 0x8169dd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d69dd9, SUC_lexicon_tags_38 },
    { 0x9f69dd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53369de0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7369dee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73d69df0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40369df5, SUC_lexicon_tags_37 },
    { 0xef369df6, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b969dfa, SUC_lexicon_tags_52 },
    { 0x38b69dfb, SUC_lexicon_tags_7 },
    { 0xf2f69dfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6969dff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60169e0a, SUC_lexicon_tags_16 },
    { 0xa8f69e0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4369e0d, SUC_lexicon_tags_2 },
    { 0x5f769e0e, SUC_lexicon_tags_19 },
    { 0xce569e0f, SUC_lexicon_tags_15 },
    { 0xd9969e0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69e15, SUC_lexicon_tags_14 },
    { 0x85b69e15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b69e19, SUC_lexicon_tags_26 },
    { 0x77369e19, SUC_lexicon_tags_60 },
    { 0xa8f69e19, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b69e21, SUC_lexicon_tags_2 },
    { 0x6c369e22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65d69e24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x74f69e26, SUC_lexicon_tags_6 },
    { 0xe9369e26, SUC_lexicon_tags_208 },
    { 0x2d369e28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb569e2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72b69e2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29569e32, SUC_lexicon_tags_2 },
    { 0x15769e33, SUC_lexicon_tags_31 },
    { 0xa4f69e34, SUC_lexicon_tags_37 },
    { 0x76f69e35, SUC_lexicon_tags_0 },
    { 0x9b769e36, SUC_lexicon_tags_103 },
    { 0x98b69e35, SUC_lexicon_tags_33 },
    { 0x68369e38, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0x17369e3a, SUC_lexicon_tags_15 },
    { 0xc7169e3a, SUC_lexicon_tags_23 },
    { 0x48169e3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1169e42, SUC_lexicon_tags_38 },
    { 0xec369e43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6769e45, SUC_lexicon_tags_30 },
    { 0x75369e45, SUC_lexicon_tags_15 },
    { 0xe5369e47, SUC_lexicon_tags_28 },
    { 0x5e169e48, SUC_lexicon_tags_17 },
    { 0x5d569e49, SUC_lexicon_tags_63 },
    { 0xb7f69e49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaad69e4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x56169e4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a369e53, SUC_lexicon_tags_45 },
    { 0xc9569e53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f969e58, SUC_lexicon_tags_1 },
    { 0x81969e58, SUC_lexicon_tags_41 },
    { 0x8369e5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45969e60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66569e66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b769e68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd969e6a, SUC_lexicon_tags_11 },
    { 0xd9169e6b, SUC_lexicon_tags_16 },
    { 0x97969e6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b969e6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2b69e70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d69e73, SUC_lexicon_tags_23 },
    { 0x5e169e73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd69e78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5569e7d, SUC_lexicon_tags_55 },
    { 0x29369e7d, SUC_lexicon_tags_4 },
    { 0x4169e7f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6ad69e81, SUC_lexicon_tags_1 },
    { 0x8b69e81, SUC_lexicon_tags_4 },
    { 0xdf69e83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65969e87, SUC_lexicon_tags_15 },
    { 0x66569e87, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb69e8d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a369e93, SUC_lexicon_tags_23 },
    { 0xbab69e94, SUC_lexicon_tags_2 },
    { 0x9dd69e94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f69e9a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f69e9d, SUC_lexicon_tags_26 },
    { 0xf1d69e9e, SUC_lexicon_tags_45 },
    { 0x36369e9d, SUC_lexicon_tags_65 },
    { 0xc6169ea0, SUC_lexicon_tags_2 },
    { 0x8ed69ea1, SUC_lexicon_tags_54 },
    { 0xd0169e9e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed69ea8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78369eac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb969eb5, SUC_lexicon_tags_15 },
    { 0xd0569eb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f69eb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52969ebc, SUC_lexicon_tags_134 },
    { 0xa569ebc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47369ebf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27d69ec1, SUC_lexicon_tags_49 },
    { 0xdcb69ec2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4969ec4, SUC_lexicon_tags_2 },
    { 0x30769ec5, SUC_lexicon_tags_39 },
    { 0x6b569ec6, SUC_lexicon_tags_27 },
    { 0xa5f69ec6, SUC_lexicon_tags_11 },
    { 0xaaf69ec8, SUC_lexicon_tags_43 },
    { 0xb7569ec7, SUC_lexicon_tags_15 },
    { 0xee369ec9, SUC_lexicon_tags_15 },
    { 0xfa969ecb, SUC_lexicon_tags_12 },
    { 0x87769ecb, SUC_lexicon_tags_63 },
    { 0xc9769ecb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb69ed2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd69ed5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b769ed8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd69edb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d69ee3, SUC_lexicon_tags_38 },
    { 0xed169ee3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaad69ee6, SUC_lexicon_tags_43 },
    { 0xf5169ee7, SUC_lexicon_tags_55 },
    { 0xc3769ee8, SUC_lexicon_tags_27 },
    { 0x83b69ee7, SUC_lexicon_tags_49 },
    { 0xc7369ee9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17369eec, SUC_lexicon_tags_7 },
    { 0xb3369eed, SUC_lexicon_tags_92 },
    { 0x84569eec, SUC_lexicon_tags_29 },
    { 0x94769eee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84369ef8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75369efa, SUC_lexicon_tags_47 },
    { 0x18369efb, SUC_lexicon_tags_49 },
    { 0xdd769efc, SUC_lexicon_tags_15 },
    { 0x44b69efc, SUC_lexicon_tags_37 },
    { 0x41969efc, SUC_lexicon_tags_12 },
    { 0x5bb69eff, SUC_lexicon_tags_28 },
    { 0x8df69efe, SUC_lexicon_tags_11 },
    { 0x28f69f01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3769f05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef69f0a, SUC_lexicon_tags_14 },
    { 0x6a969f0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5db69f0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed369f0f, SUC_lexicon_tags_43 },
    { 0xca769f0f, SUC_lexicon_tags_15 },
    { 0x9f569f10, SUC_lexicon_tags_11 },
    { 0x96969f12, SUC_lexicon_tags_30 },
    { 0x1d369f13, SUC_lexicon_tags_10 },
    { 0x63b69f14, SUC_lexicon_tags_12 },
    { 0x41769f10, SUC_lexicon_tags_2 },
    { 0xc769f11, SUC_lexicon_tags_64 },
    { 0x21369f11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d69f1a, SUC_lexicon_tags_14 },
    { 0x3c769f1b, SUC_lexicon_tags_15 },
    { 0x90d69f1c, SUC_lexicon_tags_11 },
    { 0x23569f1d, SUC_lexicon_tags_68 },
    { 0x91f69f1e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x20769f21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5d69f23, SUC_lexicon_tags_0 },
    { 0x44569f23, SUC_lexicon_tags_63 },
    { 0x40f69f25, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d969f2e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf6169f30, SUC_lexicon_tags_37 },
    { 0xf8569f31, SUC_lexicon_tags_6 },
    { 0xdaf69f32, SUC_lexicon_tags_23 },
    { 0x44f69f33, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x80f69f35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b69f38, SUC_lexicon_tags_7 },
    { 0x6ff69f39, SUC_lexicon_tags_31 },
    { 0xe2369f3a, SUC_lexicon_tags_82 },
    { 0x3d969f3b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x44d69f3d, SUC_lexicon_tags_65 },
    { 0x11969f3e, SUC_lexicon_tags_7 },
    { 0x9e769f3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75169f41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef69f4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d69f4f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda169f56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed69f5a, SUC_lexicon_tags_14 },
    { 0x90f69f5b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb7969f5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd5969f5f, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d69f65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e969f68, SUC_lexicon_tags_37 },
    { 0xa2f69f69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9169f78, SUC_lexicon_tags_11 },
    { 0x76b69f79, SUC_lexicon_tags_0 },
    { 0x50969f7a, SUC_lexicon_tags_19 },
    { 0xa8b69f7a, SUC_lexicon_tags_43 },
    { 0xe2d69f7c, SUC_lexicon_tags_3 },
    { 0x1169f7d, SUC_lexicon_tags_11 },
    { 0xf9769f7d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3569f82, SUC_lexicon_tags_14 },
    { 0xdfb69f82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x10569f86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b69f89, SUC_lexicon_tags_4 },
    { 0xc8b69f8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd369f91, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d769f94, SUC_lexicon_tags_21 },
    { 0xfcd69f95, SUC_lexicon_tags_25 },
    { 0x13169f96, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad69f99, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69f9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af69fa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca969fa4, SUC_lexicon_tags_17 },
    { 0x18169fa5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b69fac, SUC_lexicon_tags_14 },
    { 0x4e769fac, SUC_lexicon_tags_28 },
    { 0x8ed69fac, SUC_lexicon_tags_42 },
    { 0xddb69faf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2bf69fb1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0369fb5, SUC_lexicon_tags_27 },
    { 0xa1969fb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e569fb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b69fbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77b69fc1, SUC_lexicon_tags_15 },
    { 0xe8369fc1, SUC_lexicon_tags_25 },
    { 0xa969fc1, SUC_lexicon_tags_20 },
    { 0x79369fc4, SUC_lexicon_tags_7 },
    { 0x55969fc2, SUC_lexicon_tags_25 },
    { 0xf8169fc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d69fc9, SUC_lexicon_tags_2 },
    { 0x78f69fca, SUC_lexicon_tags_65 },
    { 0x3f69fca, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x11769fcd, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0xd8f69fcf, SUC_lexicon_tags_37 },
    { 0x80169fd0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df69fd5, SUC_lexicon_tags_14 },
    { 0x9f69fd5, SUC_lexicon_tags_39 },
    { 0xdd69fd7, SUC_lexicon_tags_12 },
    { 0x86369fd8, SUC_lexicon_tags_32 },
    { 0xd7769fd8, SUC_lexicon_tags_25 },
    { 0xff169fda, SUC_lexicon_tags_65 },
    { 0x3d969fda, SUC_lexicon_tags_19 },
    { 0xc6f69fd5, SUC_lexicon_tags_16 },
    { 0x91169fdb, SUC_lexicon_tags_47 },
    { 0xe3769fde, SUC_lexicon_tags_2 },
    { 0xa0969fdf, SUC_lexicon_tags_87 },
    { 0xad169fe0, SUC_lexicon_tags_25 },
    { 0xb6169fe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17169fe5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7a769fe7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e969fec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x37969fee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x10769ff1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb0369ff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d969ffb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2969ffe, SUC_lexicon_tags_7 },
    { 0xd3d69fff, SUC_lexicon_tags_14 },
    { 0x63d69fff, SUC_lexicon_tags_13 },
    { 0x2356a001, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb76a007, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd6a00e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d96a012, SUC_lexicon_tags_45 },
    { 0x3a16a013, SUC_lexicon_tags_8 },
    { 0x2676a014, SUC_lexicon_tags_23 },
    { 0x94f6a015, SUC_lexicon_tags_60 },
    { 0x8856a014, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4536a01e, SUC_lexicon_tags_8 },
    { 0xa8f6a01e, SUC_lexicon_tags_25 },
    { 0x6236a020, SUC_lexicon_tags_15 },
    { 0x88f6a021, SUC_lexicon_tags_25 },
    { 0x2016a022, SUC_lexicon_tags_30 },
    { 0xcdd6a023, SUC_lexicon_tags_12 },
    { 0xc536a022, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d96a027, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff6a02a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea56a02e, SUC_lexicon_tags_2 },
    { 0xf1f6a02f, SUC_lexicon_tags_17 },
    { 0xe996a02e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbad6a032, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2316a037, SUC_lexicon_tags_1 },
    { 0xd796a038, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78f6a03a, SUC_lexicon_tags_60 },
    { 0xed76a03b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d36a03e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf16a040, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x756a043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2dd6a045, SUC_lexicon_tags_161 },
    { 0x36b6a046, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x48b6a048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2536a051, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f6a054, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d76a057, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b6a05b, SUC_lexicon_tags_25 },
    { 0xebf6a05c, SUC_lexicon_tags_38 },
    { 0x3236a05b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x42b6a05f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f6a062, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6316a064, SUC_lexicon_tags_14 },
    { 0x92b6a065, SUC_lexicon_tags_25 },
    { 0x616a064, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf6a06e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7d6a070, SUC_lexicon_tags_66 },
    { 0xec36a070, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2b6a073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1f6a075, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d6a078, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa756a07e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5f6a080, SUC_lexicon_tags_44 },
    { 0x5e36a081, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d6a084, SUC_lexicon_tags_7 },
    { 0xe776a085, SUC_lexicon_tags_37 },
    { 0x4976a085, SUC_lexicon_tags_1 },
    { 0x6bd6a087, SUC_lexicon_tags_43 },
    { 0x8cf6a087, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbaf6a08a, SUC_lexicon_tags_27 },
    { 0xc6d6a08b, SUC_lexicon_tags_11 },
    { 0x9436a08c, SUC_lexicon_tags_60 },
    { 0x6936a08a, SUC_lexicon_tags_19 },
    { 0x93d6a08e, SUC_lexicon_tags_14 },
    { 0x2596a08f, SUC_lexicon_tags_55 },
    { 0xc656a08d, SUC_lexicon_tags_11 },
    { 0x9036a091, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf36a099, SUC_lexicon_tags_48 },
    { 0x4f56a09a, SUC_lexicon_tags_17 },
    { 0x9bd6a09b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7896a09d, SUC_lexicon_tags_45 },
    { 0x7656a09e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e56a0a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d56a0a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ef6a0a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2d6a0aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e96a0ac, SUC_lexicon_tags_25 },
    { 0x4716a0ad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1856a0af, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x436a0b1, SUC_lexicon_tags_23 },
    { 0xeb96a0b1, SUC_lexicon_tags_29 },
    { 0x4a56a0b1, SUC_lexicon_tags_33 },
    { 0x3716a0b4, SUC_lexicon_tags_11 },
    { 0xfbf6a0b4, SUC_lexicon_tags_37 },
    { 0x4a36a0b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6a0b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5e36a0bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2576a0bd, SUC_lexicon_tags_17 },
    { 0x6356a0be, SUC_lexicon_tags_6 },
    { 0xdc16a0bf, SUC_lexicon_tags_26 },
    { 0x2c36a0c0, SUC_lexicon_tags_11 },
    { 0xe6b6a0c1, SUC_lexicon_tags_17 },
    { 0xb0f6a0c1, SUC_lexicon_tags_17 },
    { 0x1036a0c3, SUC_lexicon_tags_25 },
    { 0xe916a0c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa316a0c7, SUC_lexicon_tags_17 },
    { 0x8ab6a0c8, SUC_lexicon_tags_48 },
    { 0x28d6a0c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa616a0cd, SUC_lexicon_tags_2 },
    { 0xe756a0ce, SUC_lexicon_tags_37 },
    { 0x74b6a0cf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb56a0d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf6a0d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4096a0d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0f6a0d9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b56a0dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb6a0e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3596a0e2, SUC_lexicon_tags_31 },
    { 0xfab6a0e3, SUC_lexicon_tags_2 },
    { 0x3016a0e4, SUC_lexicon_tags_16 },
    { 0x196a0e5, SUC_lexicon_tags_37 },
    { 0x5916a0e6, SUC_lexicon_tags_11 },
    { 0x37f6a0e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b6a0ed, SUC_lexicon_tags_19 },
    { 0x7b36a0ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc936a0f0, SUC_lexicon_tags_25 },
    { 0x5dd6a0f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74d6a0f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4576a0f5, SUC_lexicon_tags_7 },
    { 0xbdd6a0f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58b6a0f8, SUC_lexicon_tags_37 },
    { 0x1f56a0f9, SUC_lexicon_tags_2 },
    { 0x3756a0fa, SUC_lexicon_tags_11 },
    { 0x6bf6a0f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2676a100, SUC_lexicon_tags_15 },
    { 0x6156a101, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7856a103, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46b6a105, SUC_lexicon_tags_12 },
    { 0x236a106, SUC_lexicon_tags_0 },
    { 0x16f6a107, SUC_lexicon_tags_14 },
    { 0x2c36a107, SUC_lexicon_tags_17 },
    { 0x3956a109, SUC_lexicon_tags_2 },
    { 0x11b6a10a, SUC_lexicon_tags_37 },
    { 0xd216a10b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5776a110, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c96a117, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2916a11d, SUC_lexicon_tags_30 },
    { 0xed76a11e, SUC_lexicon_tags_60 },
    { 0x4996a11f, SUC_lexicon_tags_0 },
    { 0x3a96a11f, SUC_lexicon_tags_30 },
    { 0xc8d6a120, SUC_lexicon_tags_19 },
    { 0xc356a120, SUC_lexicon_tags_25 },
    { 0x3836a11e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8196a127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e16a129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8796a12c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18d6a12e, SUC_lexicon_tags_25 },
    { 0x9556a12f, SUC_lexicon_tags_27 },
    { 0xb76a12f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d6a134, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x16d6a136, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b6a13a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4d6a13c, SUC_lexicon_tags_15 },
    { 0xb6a13d, SUC_lexicon_tags_37 },
    { 0x76b6a13e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7156a142, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ef6a144, SUC_lexicon_tags_31 },
    { 0x9bd6a145, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d6a148, SUC_lexicon_tags_2 },
    { 0xafd6a148, SUC_lexicon_tags_43 },
    { 0xb676a149, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e76a152, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bb6a154, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5276a156, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc56a15f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff6a162, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe36a165, SUC_lexicon_tags_7 },
    { 0xc676a166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf096a169, SUC_lexicon_tags_2 },
    { 0x9bd6a16a, SUC_lexicon_tags_19 },
    { 0xfcb6a16b, SUC_lexicon_tags_1 },
    { 0x1df6a16c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8496a16f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d6a173, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f6a177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf236a179, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc796a17c, SUC_lexicon_tags_17 },
    { 0x78f6a17d, SUC_lexicon_tags_33 },
    { 0x9016a17e, SUC_lexicon_tags_11 },
    { 0x6356a17e, SUC_lexicon_tags_35 },
    { 0xaf56a17e, SUC_lexicon_tags_14 },
    { 0xccf6a181, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4676a184, SUC_lexicon_tags_12 },
    { 0x7596a185, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x396a187, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d6a18a, SUC_lexicon_tags_37 },
    { 0xfa96a18a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db6a18f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f76a194, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf56a199, SUC_lexicon_tags_98 },
    { 0x4eb6a19a, SUC_lexicon_tags_23 },
    { 0xcdf6a19b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3936a1a3, SUC_lexicon_tags_15 },
    { 0xa0b6a1a3, SUC_lexicon_tags_7 },
    { 0xb9b6a1a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f6a1a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e76a1ab, SUC_lexicon_tags_37 },
    { 0xf1d6a1ac, SUC_lexicon_tags_14 },
    { 0xafd6a1ad, SUC_lexicon_tags_34 },
    { 0x4356a1ab, SUC_lexicon_tags_37 },
    { 0xbad6a1ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe756a1b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d6a1b9, SUC_lexicon_tags_11 },
    { 0xbdb6a1ba, SUC_lexicon_tags_12 },
    { 0xc8f6a1bb, SUC_lexicon_tags_46 },
    { 0xb936a1bc, SUC_lexicon_tags_17 },
    { 0x32d6a1bb, SUC_lexicon_tags_11 },
    { 0xb216a1bc, SUC_lexicon_tags_17 },
    { 0xbe96a1bc, SUC_lexicon_tags_17 },
    { 0xa236a1c0, SUC_lexicon_tags_19 },
    { 0xf4b6a1c1, SUC_lexicon_tags_25 },
    { 0x5316a1c2, SUC_lexicon_tags_2 },
    { 0x65b6a1c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x45d6a1c5, SUC_lexicon_tags_2 },
    { 0x8516a1c5, SUC_lexicon_tags_37 },
    { 0xd8b6a1c5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf776a1c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b6a1cc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfc76a1ce, SUC_lexicon_tags_15 },
    { 0x64d6a1ce, SUC_lexicon_tags_11 },
    { 0x6f56a1d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1696a1d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6a1d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9096a1db, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xeef6a1dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd196a1e0, SUC_lexicon_tags_345 },
    { 0, NULL },
    { 0x15d6a1e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6056a1e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb6a1e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb976a1ea, SUC_lexicon_tags_28 },
    { 0x476a1eb, SUC_lexicon_tags_25 },
    { 0xce36a1eb, SUC_lexicon_tags_48 },
    { 0x1656a1ec, SUC_lexicon_tags_29 },
    { 0xc416a1ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e36a1f0, SUC_lexicon_tags_15 },
    { 0x9f16a1f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf976a1f4, SUC_lexicon_tags_7 },
    { 0xeed6a1f5, SUC_lexicon_tags_21 },
    { 0x6f36a1f6, SUC_lexicon_tags_0 },
    { 0x1556a1f7, SUC_lexicon_tags_47 },
    { 0xf496a1f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2ad6a1fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd56a200, SUC_lexicon_tags_45 },
    { 0xed76a201, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa876a204, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba96a20a, SUC_lexicon_tags_19 },
    { 0x9696a20b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7136a20d, SUC_lexicon_tags_8 },
    { 0x7c56a20e, SUC_lexicon_tags_27 },
    { 0x9ad6a20e, SUC_lexicon_tags_25 },
    { 0x1d96a210, SUC_lexicon_tags_2 },
    { 0xc616a20d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e36a213, SUC_lexicon_tags_29 },
    { 0xe9b6a214, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d6a218, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b6a21b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2776a21e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e56a224, SUC_lexicon_tags_38 },
    { 0x7f96a224, SUC_lexicon_tags_12 },
    { 0x47f6a226, SUC_lexicon_tags_7 },
    { 0xe116a226, SUC_lexicon_tags_31 },
    { 0x3396a224, SUC_lexicon_tags_79 },
    { 0xe296a225, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1776a22b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6416a22e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x68d6a230, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7f6a232, SUC_lexicon_tags_17 },
    { 0x85b6a232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87b6a235, SUC_lexicon_tags_2 },
    { 0xa956a236, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7fb6a238, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdf76a23a, SUC_lexicon_tags_17 },
    { 0x67d6a23a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef6a241, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa996a24b, SUC_lexicon_tags_37 },
    { 0xb056a24b, SUC_lexicon_tags_0 },
    { 0x99b6a24d, SUC_lexicon_tags_37 },
    { 0x3e56a24d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd96a251, SUC_lexicon_tags_7 },
    { 0x7b96a252, SUC_lexicon_tags_12 },
    { 0xbbf6a251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc536a257, SUC_lexicon_tags_17 },
    { 0x9e16a258, SUC_lexicon_tags_11 },
    { 0x81f6a257, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b6a25e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37d6a260, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf496a263, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8476a267, SUC_lexicon_tags_12 },
    { 0x7f96a267, SUC_lexicon_tags_11 },
    { 0x3416a267, SUC_lexicon_tags_37 },
    { 0xca96a268, SUC_lexicon_tags_25 },
    { 0x4e36a26b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1736a26f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b6a273, SUC_lexicon_tags_38 },
    { 0xd716a274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b56a276, SUC_lexicon_tags_43 },
    { 0x5ed6a277, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x456a27e, SUC_lexicon_tags_17 },
    { 0xabb6a27e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab6a286, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced6a28a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d96a28c, SUC_lexicon_tags_26 },
    { 0x93b6a28d, SUC_lexicon_tags_17 },
    { 0xd0b6a28e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2456a292, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7156a298, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7276a29c, SUC_lexicon_tags_7 },
    { 0x4d56a29d, SUC_lexicon_tags_390 },
    { 0x8396a29e, SUC_lexicon_tags_11 },
    { 0xd716a29f, SUC_lexicon_tags_2 },
    { 0xd316a29f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6a2a5, SUC_lexicon_tags_17 },
    { 0x5276a2a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1256a2ab, SUC_lexicon_tags_30 },
    { 0xb176a2ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d6a2af, SUC_lexicon_tags_7 },
    { 0x7c36a2b0, SUC_lexicon_tags_11 },
    { 0x4dd6a2b0, SUC_lexicon_tags_14 },
    { 0x69b6a2b2, SUC_lexicon_tags_27 },
    { 0xc416a2b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a96a2b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf16a2be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe76a2c4, SUC_lexicon_tags_25 },
    { 0x7556a2c5, SUC_lexicon_tags_55 },
    { 0xe536a2c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x696a2c8, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b6a2cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc216a2cd, SUC_lexicon_tags_74 },
    { 0x1a56a2ce, SUC_lexicon_tags_7 },
    { 0x40d6a2cf, SUC_lexicon_tags_14 },
    { 0xbd76a2d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8376a2d6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1196a2dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb076a2df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa956a2e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b16a2e6, SUC_lexicon_tags_47 },
    { 0xefb6a2e7, SUC_lexicon_tags_23 },
    { 0xec36a2e8, SUC_lexicon_tags_37 },
    { 0x13d6a2e9, SUC_lexicon_tags_82 },
    { 0x4316a2e8, SUC_lexicon_tags_37 },
    { 0xf536a2eb, SUC_lexicon_tags_7 },
    { 0xf296a2ec, SUC_lexicon_tags_4 },
    { 0x3156a2ed, SUC_lexicon_tags_49 },
    { 0xb5f6a2ec, SUC_lexicon_tags_1 },
    { 0x4216a2ef, SUC_lexicon_tags_19 },
    { 0x1fb6a2f0, SUC_lexicon_tags_30 },
    { 0xf436a2e8, SUC_lexicon_tags_7 },
    { 0x3f16a2f2, SUC_lexicon_tags_33 },
    { 0xa616a2f3, SUC_lexicon_tags_97 },
    { 0x3916a2ed, SUC_lexicon_tags_104 },
    { 0x2a96a2f1, SUC_lexicon_tags_23 },
    { 0xb6f6a2f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8776a2f9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e76a2fd, SUC_lexicon_tags_33 },
    { 0x8b76a2fe, SUC_lexicon_tags_29 },
    { 0x236a2ff, SUC_lexicon_tags_4 },
    { 0x2516a300, SUC_lexicon_tags_11 },
    { 0xc316a301, SUC_lexicon_tags_68 },
    { 0x55d6a302, SUC_lexicon_tags_15 },
    { 0x2ad6a300, SUC_lexicon_tags_11 },
    { 0x2e16a304, SUC_lexicon_tags_2 },
    { 0xc6b6a305, SUC_lexicon_tags_37 },
    { 0xf7b6a306, SUC_lexicon_tags_7 },
    { 0xcfd6a307, SUC_lexicon_tags_9 },
    { 0x7016a305, SUC_lexicon_tags_68 },
    { 0x4a36a2fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d6a30c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2a16a30e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6176a313, SUC_lexicon_tags_25 },
    { 0x8516a313, SUC_lexicon_tags_1 },
    { 0xbeb6a315, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf116a317, SUC_lexicon_tags_14 },
    { 0x8c56a318, SUC_lexicon_tags_11 },
    { 0x99d6a319, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5176a31e, SUC_lexicon_tags_63 },
    { 0x2876a31e, SUC_lexicon_tags_61 },
    { 0xb476a31e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd6a324, SUC_lexicon_tags_23 },
    { 0x6816a325, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea76a327, SUC_lexicon_tags_11 },
    { 0x1cb6a328, SUC_lexicon_tags_45 },
    { 0xd876a329, SUC_lexicon_tags_26 },
    { 0x43d6a32a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe96a32c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2496a330, SUC_lexicon_tags_37 },
    { 0x1c36a331, SUC_lexicon_tags_43 },
    { 0x2a56a332, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9496a334, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b56a336, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e76a33d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb456a342, SUC_lexicon_tags_17 },
    { 0x3e76a343, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16a348, SUC_lexicon_tags_31 },
    { 0x1b6a349, SUC_lexicon_tags_14 },
    { 0xe616a34a, SUC_lexicon_tags_7 },
    { 0x5f16a348, SUC_lexicon_tags_0 },
    { 0x9c96a34c, SUC_lexicon_tags_38 },
    { 0xdd36a34c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb6a350, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf376a353, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7816a358, SUC_lexicon_tags_17 },
    { 0xef76a359, SUC_lexicon_tags_17 },
    { 0x7916a35a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b6a35f, SUC_lexicon_tags_9 },
    { 0xf6a360, SUC_lexicon_tags_26 },
    { 0xe696a360, SUC_lexicon_tags_11 },
    { 0x8bb6a362, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x456a364, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd6a369, SUC_lexicon_tags_10 },
    { 0xe216a36a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f16a36d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd6a371, SUC_lexicon_tags_19 },
    { 0x52b6a372, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6a376, SUC_lexicon_tags_44 },
    { 0xd2d6a376, SUC_lexicon_tags_87 },
    { 0x1196a378, SUC_lexicon_tags_2 },
    { 0xc4d6a378, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4696a381, SUC_lexicon_tags_35 },
    { 0x9556a382, SUC_lexicon_tags_45 },
    { 0x7776a383, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d6a38a, SUC_lexicon_tags_17 },
    { 0xaab6a38b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3756a38d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf476a390, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4076a392, SUC_lexicon_tags_15 },
    { 0xdd16a393, SUC_lexicon_tags_23 },
    { 0x34f6a393, SUC_lexicon_tags_14 },
    { 0x396a394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b6a399, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8916a39d, SUC_lexicon_tags_7 },
    { 0x6616a39e, SUC_lexicon_tags_44 },
    { 0xb0f6a39f, SUC_lexicon_tags_11 },
    { 0x66f6a39f, SUC_lexicon_tags_60 },
    { 0xef36a39f, SUC_lexicon_tags_38 },
    { 0xd0d6a3a2, SUC_lexicon_tags_47 },
    { 0x5e76a39f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa056a3a9, SUC_lexicon_tags_26 },
    { 0x1276a3aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x136a3ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc016a3b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2176a3b4, SUC_lexicon_tags_17 },
    { 0x1cd6a3b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd956a3b7, SUC_lexicon_tags_0 },
    { 0xef36a3b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e96a3bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed6a3c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6836a3c6, SUC_lexicon_tags_12 },
    { 0x7b76a3c6, SUC_lexicon_tags_40 },
    { 0x6576a3c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd796a3cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb956a3ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5596a3d1, SUC_lexicon_tags_38 },
    { 0x8416a3d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0b6a3d4, SUC_lexicon_tags_37 },
    { 0x7376a3d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8576a3d7, SUC_lexicon_tags_336 },
    { 0x9b96a3d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdff6a3da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x18b6a3dc, SUC_lexicon_tags_29 },
    { 0x5336a3dd, SUC_lexicon_tags_12 },
    { 0xced6a3de, SUC_lexicon_tags_7 },
    { 0x5d76a3de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa36a3e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63b6a3e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4236a3ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52d6a3ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4856a3f1, SUC_lexicon_tags_34 },
    { 0x77d6a3f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3216a3f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8516a3fa, SUC_lexicon_tags_49 },
    { 0x2b76a3fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b36a3fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef6a401, SUC_lexicon_tags_7 },
    { 0x7c96a402, SUC_lexicon_tags_14 },
    { 0xc8d6a402, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x23f6a405, SUC_lexicon_tags_37 },
    { 0xf0f6a406, SUC_lexicon_tags_1 },
    { 0x9996a406, SUC_lexicon_tags_8 },
    { 0xabd6a406, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb6a40b, SUC_lexicon_tags_57 },
    { 0x45b6a40c, SUC_lexicon_tags_7 },
    { 0x7936a40d, SUC_lexicon_tags_26 },
    { 0x63f6a40e, SUC_lexicon_tags_21 },
    { 0x7616a40f, SUC_lexicon_tags_9 },
    { 0xd8f6a410, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9616a41b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c56a420, SUC_lexicon_tags_48 },
    { 0x9c16a421, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c76a426, SUC_lexicon_tags_25 },
    { 0x49b6a427, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd6a42c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb976a430, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd396a433, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5116a436, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x716a438, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5296a43a, SUC_lexicon_tags_2 },
    { 0xfad6a43b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb036a43e, SUC_lexicon_tags_3 },
    { 0x1916a43f, SUC_lexicon_tags_37 },
    { 0xc9f6a440, SUC_lexicon_tags_45 },
    { 0x67f6a441, SUC_lexicon_tags_30 },
    { 0x60f6a442, SUC_lexicon_tags_26 },
    { 0x8176a441, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2136a445, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8056a448, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd556a44b, SUC_lexicon_tags_2 },
    { 0x4a36a44b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f16a450, SUC_lexicon_tags_70 },
    { 0x6a76a451, SUC_lexicon_tags_39 },
    { 0xe176a452, SUC_lexicon_tags_25 },
    { 0x9c16a453, SUC_lexicon_tags_12 },
    { 0xe416a450, SUC_lexicon_tags_37 },
    { 0x6ef6a455, SUC_lexicon_tags_33 },
    { 0x2436a455, SUC_lexicon_tags_17 },
    { 0xff36a455, SUC_lexicon_tags_63 },
    { 0x3476a457, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9036a45c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf356a45e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f36a462, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc36a466, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b6a469, SUC_lexicon_tags_37 },
    { 0xfb96a46a, SUC_lexicon_tags_37 },
    { 0xa916a46a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd516a470, SUC_lexicon_tags_17 },
    { 0x5a96a471, SUC_lexicon_tags_68 },
    { 0x9396a470, SUC_lexicon_tags_38 },
    { 0xb2d6a470, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d16a475, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d6a47f, SUC_lexicon_tags_33 },
    { 0x8d16a480, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd536a483, SUC_lexicon_tags_37 },
    { 0x1276a484, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x676a489, SUC_lexicon_tags_15 },
    { 0x4b16a48a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdb16a48c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8996a492, SUC_lexicon_tags_25 },
    { 0x6216a493, SUC_lexicon_tags_29 },
    { 0x7bd6a493, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7156a496, SUC_lexicon_tags_26 },
    { 0xac56a496, SUC_lexicon_tags_82 },
    { 0xcf76a496, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b6a49c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3636a49e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a56a4a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9616a4a4, SUC_lexicon_tags_47 },
    { 0x3fd6a4a4, SUC_lexicon_tags_2 },
    { 0x6bb6a4a6, SUC_lexicon_tags_37 },
    { 0xc976a4a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2396a4ac, SUC_lexicon_tags_11 },
    { 0x61d6a4ad, SUC_lexicon_tags_37 },
    { 0x6096a4ac, SUC_lexicon_tags_37 },
    { 0x9c56a4af, SUC_lexicon_tags_8 },
    { 0x90b6a4af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c56a4b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f16a4bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2896a4c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa876a4c3, SUC_lexicon_tags_60 },
    { 0x48b6a4c4, SUC_lexicon_tags_11 },
    { 0xa9f6a4c3, SUC_lexicon_tags_367 },
    { 0xaaf6a4c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb6a4cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d96a4d0, SUC_lexicon_tags_11 },
    { 0x9f36a4d0, SUC_lexicon_tags_37 },
    { 0x3d6a4d1, SUC_lexicon_tags_15 },
    { 0x2296a4d0, SUC_lexicon_tags_47 },
    { 0x25f6a4d4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x40d6a4d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3036a4d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4096a4db, SUC_lexicon_tags_15 },
    { 0xc756a4dc, SUC_lexicon_tags_29 },
    { 0xba56a4dd, SUC_lexicon_tags_43 },
    { 0x13d6a4db, SUC_lexicon_tags_29 },
    { 0x5ef6a4df, SUC_lexicon_tags_39 },
    { 0x6dd6a4df, SUC_lexicon_tags_37 },
    { 0xadd6a4e1, SUC_lexicon_tags_7 },
    { 0xa836a4e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a16a4e4, SUC_lexicon_tags_7 },
    { 0x8bd6a4e5, SUC_lexicon_tags_31 },
    { 0x2376a4e6, SUC_lexicon_tags_1 },
    { 0x1d76a4e6, SUC_lexicon_tags_11 },
    { 0x9536a4e8, SUC_lexicon_tags_17 },
    { 0x1396a4e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8856a4ef, SUC_lexicon_tags_17 },
    { 0xf8f6a4f0, SUC_lexicon_tags_47 },
    { 0x23f6a4f1, SUC_lexicon_tags_45 },
    { 0x2416a4f2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5616a4f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x536a4fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b6a500, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd2b6a502, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5e76a504, SUC_lexicon_tags_6 },
    { 0x9ff6a505, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d6a508, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c56a514, SUC_lexicon_tags_92 },
    { 0xbe96a515, SUC_lexicon_tags_12 },
    { 0x5cf6a516, SUC_lexicon_tags_62 },
    { 0xfa56a516, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d6a51a, SUC_lexicon_tags_38 },
    { 0x75d6a51b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd496a521, SUC_lexicon_tags_3 },
    { 0x4356a522, SUC_lexicon_tags_7 },
    { 0xea56a523, SUC_lexicon_tags_48 },
    { 0x9356a523, SUC_lexicon_tags_11 },
    { 0x7ab6a522, SUC_lexicon_tags_17 },
    { 0xff36a526, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdc76a528, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996a530, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f6a533, SUC_lexicon_tags_7 },
    { 0x9b56a534, SUC_lexicon_tags_11 },
    { 0xb896a534, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1fd6a537, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6a53a, SUC_lexicon_tags_91 },
    { 0x52b6a53a, SUC_lexicon_tags_390 },
    { 0x92f6a53c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a76a53f, SUC_lexicon_tags_28 },
    { 0x1476a540, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f6a545, SUC_lexicon_tags_48 },
    { 0xd7b6a546, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0xa76a549, SUC_lexicon_tags_47 },
    { 0xb6f6a54a, SUC_lexicon_tags_7 },
    { 0x80f6a54a, SUC_lexicon_tags_11 },
    { 0x1196a54c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd736a54e, SUC_lexicon_tags_37 },
    { 0x43b6a54f, SUC_lexicon_tags_47 },
    { 0x3516a550, SUC_lexicon_tags_12 },
    { 0xd7d6a550, SUC_lexicon_tags_0 },
    { 0x516a54f, SUC_lexicon_tags_11 },
    { 0x4c16a552, SUC_lexicon_tags_11 },
    { 0x1e16a553, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd216a557, SUC_lexicon_tags_207 },
    { 0xa2f6a557, SUC_lexicon_tags_17 },
    { 0x5d76a559, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4af6a55b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5196a55d, SUC_lexicon_tags_56 },
    { 0x86b6a55d, SUC_lexicon_tags_17 },
    { 0x5c36a55d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb996a561, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2896a564, SUC_lexicon_tags_11 },
    { 0x74b6a564, SUC_lexicon_tags_11 },
    { 0x61d6a566, SUC_lexicon_tags_88 },
    { 0x7b96a566, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f6a56a, SUC_lexicon_tags_6 },
    { 0x496a56a, SUC_lexicon_tags_1 },
    { 0xd616a56c, SUC_lexicon_tags_15 },
    { 0x7fd6a56a, SUC_lexicon_tags_9 },
    { 0x4ef6a56e, SUC_lexicon_tags_25 },
    { 0x6056a56e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb6a573, SUC_lexicon_tags_275 },
    { 0xadb6a574, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4d6a576, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df6a579, SUC_lexicon_tags_60 },
    { 0x7fb6a57a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced6a57f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd116a581, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7136a585, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d6a58a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f6a591, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b6a594, SUC_lexicon_tags_63 },
    { 0x8f36a594, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2816a59a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa56a59d, SUC_lexicon_tags_0 },
    { 0x7e36a59d, SUC_lexicon_tags_11 },
    { 0x7d36a59f, SUC_lexicon_tags_43 },
    { 0x84b6a59f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b6a5ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f96a5b3, SUC_lexicon_tags_17 },
    { 0xd8d6a5b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40b6a5b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc676a5bb, SUC_lexicon_tags_1 },
    { 0xc556a5bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd6a5bf, SUC_lexicon_tags_33 },
    { 0x88f6a5c0, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b16a5c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe536a5c8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf6a5cb, SUC_lexicon_tags_37 },
    { 0xff36a5cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb576a5cf, SUC_lexicon_tags_14 },
    { 0xc256a5d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b16a5d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x37d6a5d9, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf6a5df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b16a5e2, SUC_lexicon_tags_25 },
    { 0xf016a5e2, SUC_lexicon_tags_31 },
    { 0xac36a5e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b56a5e7, SUC_lexicon_tags_48 },
    { 0x4c76a5e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x37b6a5ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd276a5f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad6a5f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd6a5fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6696a5fd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf036a5ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e36a602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa736a607, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3016a60d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4716a611, SUC_lexicon_tags_39 },
    { 0xc776a612, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f6a615, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc56a617, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d96a619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f96a61b, SUC_lexicon_tags_117 },
    { 0xdff6a61c, SUC_lexicon_tags_9 },
    { 0xb756a61c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5a36a61f, SUC_lexicon_tags_25 },
    { 0xfd36a620, SUC_lexicon_tags_49 },
    { 0xf916a621, SUC_lexicon_tags_25 },
    { 0xa5f6a620, SUC_lexicon_tags_60 },
    { 0xe516a61f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f6a626, SUC_lexicon_tags_49 },
    { 0xbd6a627, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd336a62b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3016a62d, SUC_lexicon_tags_43 },
    { 0xbcf6a62e, SUC_lexicon_tags_79 },
    { 0x5916a62f, SUC_lexicon_tags_33 },
    { 0x70d6a630, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf696a632, SUC_lexicon_tags_9 },
    { 0xae16a633, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xba96a636, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f6a639, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b6a63c, SUC_lexicon_tags_23 },
    { 0xce36a63d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf36a648, SUC_lexicon_tags_17 },
    { 0x55b6a649, SUC_lexicon_tags_2 },
    { 0x29f6a649, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae76a64e, SUC_lexicon_tags_47 },
    { 0x2ad6a64f, SUC_lexicon_tags_10 },
    { 0x85b6a64e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc056a656, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3696a65a, SUC_lexicon_tags_11 },
    { 0xb416a65b, SUC_lexicon_tags_11 },
    { 0xbb96a65c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f76a660, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5656a662, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5b6a664, SUC_lexicon_tags_138 },
    { 0x1856a665, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d6a668, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f56a66f, SUC_lexicon_tags_27 },
    { 0xd4d6a670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb776a673, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab76a67b, SUC_lexicon_tags_1 },
    { 0x78d6a67b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9d6a67e, SUC_lexicon_tags_13 },
    { 0x9536a67f, SUC_lexicon_tags_57 },
    { 0xe9b6a67e, SUC_lexicon_tags_37 },
    { 0xebb6a681, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa796a689, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xea56a68b, SUC_lexicon_tags_60 },
    { 0xb796a68c, SUC_lexicon_tags_1 },
    { 0x6d16a68d, SUC_lexicon_tags_18 },
    { 0x54d6a68b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f6a693, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb96a695, SUC_lexicon_tags_55 },
    { 0x27f6a695, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476a69a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbcf6a69c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f76a6a3, SUC_lexicon_tags_15 },
    { 0xe536a6a4, SUC_lexicon_tags_19 },
    { 0x40d6a6a5, SUC_lexicon_tags_26 },
    { 0xe896a6a5, SUC_lexicon_tags_35 },
    { 0x6216a6a4, SUC_lexicon_tags_84 },
    { 0xdb36a6a8, SUC_lexicon_tags_43 },
    { 0x6096a6a9, SUC_lexicon_tags_9 },
    { 0x5616a6aa, SUC_lexicon_tags_37 },
    { 0x1676a6a9, SUC_lexicon_tags_97 },
    { 0xe136a6aa, SUC_lexicon_tags_61 },
    { 0x1676a6ad, SUC_lexicon_tags_17 },
    { 0x3196a6ae, SUC_lexicon_tags_15 },
    { 0xc276a6af, SUC_lexicon_tags_2 },
    { 0xdb56a6b0, SUC_lexicon_tags_70 },
    { 0x42b6a6ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b6a6b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b76a6bc, SUC_lexicon_tags_11 },
    { 0xcf76a6bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48b6a6bf, SUC_lexicon_tags_12 },
    { 0xd056a6c0, SUC_lexicon_tags_38 },
    { 0xfef6a6c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb156a6c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6336a6c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb6a6c8, SUC_lexicon_tags_37 },
    { 0x2036a6c9, SUC_lexicon_tags_37 },
    { 0xafb6a6c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37b6a6cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa2b6a6ce, SUC_lexicon_tags_47 },
    { 0xe656a6ce, SUC_lexicon_tags_44 },
    { 0x9ad6a6d0, SUC_lexicon_tags_2 },
    { 0x8ff6a6d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b16a6d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc836a6d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb116a6dc, SUC_lexicon_tags_25 },
    { 0x5c56a6dd, SUC_lexicon_tags_38 },
    { 0x7156a6de, SUC_lexicon_tags_21 },
    { 0x9716a6de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e36a6e3, SUC_lexicon_tags_49 },
    { 0xf36a6e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36a6ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4496a6ee, SUC_lexicon_tags_37 },
    { 0xfdf6a6ef, SUC_lexicon_tags_21 },
    { 0xcc36a6ee, SUC_lexicon_tags_23 },
    { 0x2a76a6f1, SUC_lexicon_tags_45 },
    { 0xe8b6a6f2, SUC_lexicon_tags_7 },
    { 0xf676a6f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3116a6f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d6a6f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe56a6fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70f6a6ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaff6a701, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3296a704, SUC_lexicon_tags_2 },
    { 0x6576a704, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb56a70d, SUC_lexicon_tags_7 },
    { 0xaaf6a70e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5796a711, SUC_lexicon_tags_30 },
    { 0x9416a711, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9896a714, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b6a72a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd6a730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xad76a733, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7976a738, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b6a73e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf6a743, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2356a745, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef6a74a, SUC_lexicon_tags_37 },
    { 0xaaf6a74b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf6a74f, SUC_lexicon_tags_17 },
    { 0x8a56a750, SUC_lexicon_tags_31 },
    { 0x5cf6a751, SUC_lexicon_tags_38 },
    { 0xa836a751, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb6a758, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b6a75b, SUC_lexicon_tags_49 },
    { 0x5296a75b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b56a75f, SUC_lexicon_tags_37 },
    { 0x9eb6a75f, SUC_lexicon_tags_108 },
    { 0xff76a761, SUC_lexicon_tags_64 },
    { 0xbab6a760, SUC_lexicon_tags_1 },
    { 0x61d6a763, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a76a76a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f36a76d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd096a76f, SUC_lexicon_tags_25 },
    { 0x9136a770, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8696a774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a56a778, SUC_lexicon_tags_63 },
    { 0xa796a779, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe636a77b, SUC_lexicon_tags_12 },
    { 0xd656a77b, SUC_lexicon_tags_336 },
    { 0xaf36a77d, SUC_lexicon_tags_1 },
    { 0x9d96a77e, SUC_lexicon_tags_25 },
    { 0x7396a77d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7096a782, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x696a784, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d6a787, SUC_lexicon_tags_1 },
    { 0x61d6a788, SUC_lexicon_tags_1 },
    { 0xe436a789, SUC_lexicon_tags_21 },
    { 0x7396a78a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd036a78d, SUC_lexicon_tags_15 },
    { 0x1d56a78e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f6a794, SUC_lexicon_tags_62 },
    { 0xdc56a794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5df6a797, SUC_lexicon_tags_12 },
    { 0x20f6a798, SUC_lexicon_tags_25 },
    { 0xe516a798, SUC_lexicon_tags_37 },
    { 0xc7d6a79a, SUC_lexicon_tags_80 },
    { 0x3936a79b, SUC_lexicon_tags_15 },
    { 0xeeb6a798, SUC_lexicon_tags_46 },
    { 0xdbf6a79d, SUC_lexicon_tags_37 },
    { 0x8b76a79d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8896a7a0, SUC_lexicon_tags_7 },
    { 0x5a76a7a0, SUC_lexicon_tags_43 },
    { 0xddb6a7a2, SUC_lexicon_tags_25 },
    { 0x676a7a3, SUC_lexicon_tags_27 },
    { 0x28b6a7a4, SUC_lexicon_tags_17 },
    { 0xfb76a7a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2256a7a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb6a7aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd6a7ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed76a7b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd436a7b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x356a7b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad96a7ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe416a7bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5136a7be, SUC_lexicon_tags_37 },
    { 0xa156a7bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b6a7c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1196a7c8, SUC_lexicon_tags_47 },
    { 0x80d6a7c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd6a7cc, SUC_lexicon_tags_82 },
    { 0x2316a7cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d36a7d0, SUC_lexicon_tags_17 },
    { 0x9d36a7d1, SUC_lexicon_tags_30 },
    { 0x5056a7d1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb6a7d4, SUC_lexicon_tags_15 },
    { 0x3cd6a7d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9196a7d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f56a7dd, SUC_lexicon_tags_30 },
    { 0x3e96a7dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8096a7e0, SUC_lexicon_tags_43 },
    { 0x5416a7e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffb6a7e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f6a7e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e36a7e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd6a7ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef6a7f2, SUC_lexicon_tags_27 },
    { 0xfeb6a7f3, SUC_lexicon_tags_17 },
    { 0x5b36a7f3, SUC_lexicon_tags_29 },
    { 0xe0d6a7f2, SUC_lexicon_tags_43 },
    { 0x7dd6a7f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc36a7ff, SUC_lexicon_tags_63 },
    { 0x8a16a7ff, SUC_lexicon_tags_77 },
    { 0xd056a801, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4016a806, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6336a80b, SUC_lexicon_tags_87 },
    { 0x5d96a80c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd1d6a80e, SUC_lexicon_tags_26 },
    { 0x7e36a80e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4eb6a811, SUC_lexicon_tags_17 },
    { 0xd016a811, SUC_lexicon_tags_33 },
    { 0x9d16a813, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f6a81c, SUC_lexicon_tags_12 },
    { 0xe4b6a81c, SUC_lexicon_tags_3 },
    { 0x7856a81e, SUC_lexicon_tags_1 },
    { 0x4d56a81c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad6a824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d16a829, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b6a831, SUC_lexicon_tags_7 },
    { 0xa9f6a832, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916a838, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf56a83f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1576a841, SUC_lexicon_tags_47 },
    { 0x4256a842, SUC_lexicon_tags_37 },
    { 0xfdf6a841, SUC_lexicon_tags_11 },
    { 0x8296a841, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4496a84b, SUC_lexicon_tags_274 },
    { 0x5e36a84b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x26b6a84e, SUC_lexicon_tags_11 },
    { 0x9996a84f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d6a856, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5996a85b, SUC_lexicon_tags_23 },
    { 0x8616a85c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f6a85f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bf6a861, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff6a865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef36a867, SUC_lexicon_tags_47 },
    { 0x436a868, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf176a86d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1976a86f, SUC_lexicon_tags_37 },
    { 0xfff6a870, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf156a874, SUC_lexicon_tags_124 },
    { 0xd276a875, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33f6a877, SUC_lexicon_tags_37 },
    { 0x15b6a878, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a56a883, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2676a888, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5cf6a88a, SUC_lexicon_tags_7 },
    { 0xd916a88b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba16a88f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f6a899, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b6a89d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b6a8a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9816a8a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf456a8aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6156a8ac, SUC_lexicon_tags_17 },
    { 0xd356a8ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf316a8af, SUC_lexicon_tags_1626 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d6a8b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6296a8ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc416a8bd, SUC_lexicon_tags_49 },
    { 0x57b6a8bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c56a8c1, SUC_lexicon_tags_9 },
    { 0x5c56a8c2, SUC_lexicon_tags_11 },
    { 0xc7b6a8c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7096a8c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3bb6a8c8, SUC_lexicon_tags_14 },
    { 0x4b76a8c9, SUC_lexicon_tags_23 },
    { 0xc576a8ca, SUC_lexicon_tags_37 },
    { 0x4476a8cb, SUC_lexicon_tags_38 },
    { 0xdf16a8c9, SUC_lexicon_tags_8 },
    { 0xedd6a8cd, SUC_lexicon_tags_17 },
    { 0x18b6a8ce, SUC_lexicon_tags_3 },
    { 0xc616a8cf, SUC_lexicon_tags_4 },
    { 0x6196a8cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8976a8d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb6a8d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc436a8d8, SUC_lexicon_tags_11 },
    { 0x7f56a8d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3696a8dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf056a8de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa896a8e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b16a8ec, SUC_lexicon_tags_45 },
    { 0xe116a8ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7616a8ef, SUC_lexicon_tags_65 },
    { 0x8836a8ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb6a8f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d96a8f6, SUC_lexicon_tags_7 },
    { 0x3816a8f6, SUC_lexicon_tags_14 },
    { 0xc7d6a8f6, SUC_lexicon_tags_7 },
    { 0xf516a8f6, SUC_lexicon_tags_45 },
    { 0x74f6a8f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x696a8fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6a900, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f6a905, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6016a907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd56a90c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c96a90e, SUC_lexicon_tags_92 },
    { 0xdd56a90f, SUC_lexicon_tags_2 },
    { 0x61f6a910, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d36a914, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d6a917, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b6a91e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x996a923, SUC_lexicon_tags_104 },
    { 0xb096a923, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbb6a926, SUC_lexicon_tags_14 },
    { 0x4136a927, SUC_lexicon_tags_25 },
    { 0x37f6a927, SUC_lexicon_tags_29 },
    { 0x3296a929, SUC_lexicon_tags_46 },
    { 0xd876a926, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df6a92d, SUC_lexicon_tags_27 },
    { 0xc556a92e, SUC_lexicon_tags_11 },
    { 0x3a56a92e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc176a932, SUC_lexicon_tags_25 },
    { 0xea36a933, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe516a935, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb6a939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a96a93b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f6a944, SUC_lexicon_tags_1 },
    { 0x12f6a945, SUC_lexicon_tags_11 },
    { 0xe876a946, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x93f6a948, SUC_lexicon_tags_25 },
    { 0x3ed6a948, SUC_lexicon_tags_12 },
    { 0xc816a94a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d6a952, SUC_lexicon_tags_30 },
    { 0x2bf6a953, SUC_lexicon_tags_17 },
    { 0x1556a954, SUC_lexicon_tags_2 },
    { 0x3a56a952, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4036a957, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f6a95b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x78b6a95d, SUC_lexicon_tags_11 },
    { 0xabb6a95e, SUC_lexicon_tags_14 },
    { 0x2436a95f, SUC_lexicon_tags_11 },
    { 0x8ad6a95f, SUC_lexicon_tags_8 },
    { 0x1fb6a95f, SUC_lexicon_tags_6 },
    { 0xa256a962, SUC_lexicon_tags_14 },
    { 0x7396a962, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd6a966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2296a96d, SUC_lexicon_tags_47 },
    { 0x2356a96e, SUC_lexicon_tags_11 },
    { 0x5636a96d, SUC_lexicon_tags_15 },
    { 0x7d76a96d, SUC_lexicon_tags_25 },
    { 0x8f6a96d, SUC_lexicon_tags_2 },
    { 0x296a970, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb16a976, SUC_lexicon_tags_40 },
    { 0x4916a977, SUC_lexicon_tags_37 },
    { 0x8a56a978, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd696a97a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf636a981, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf6a985, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9036a987, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fd6a989, SUC_lexicon_tags_60 },
    { 0xb556a98a, SUC_lexicon_tags_17 },
    { 0x20f6a989, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b56a991, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf36a993, SUC_lexicon_tags_25 },
    { 0x6bd6a993, SUC_lexicon_tags_61 },
    { 0x9636a995, SUC_lexicon_tags_29 },
    { 0x6c56a995, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb396a99b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f6a9a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6a9a4, SUC_lexicon_tags_11 },
    { 0x356a9a5, SUC_lexicon_tags_15 },
    { 0xc96a9a6, SUC_lexicon_tags_37 },
    { 0x1b36a9a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb6a9b0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d16a9b5, SUC_lexicon_tags_6 },
    { 0x3236a9b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36b6a9b8, SUC_lexicon_tags_49 },
    { 0xdcb6a9b9, SUC_lexicon_tags_15 },
    { 0x4f16a9b9, SUC_lexicon_tags_25 },
    { 0xdbd6a9bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e96a9be, SUC_lexicon_tags_8 },
    { 0x3a76a9bf, SUC_lexicon_tags_12 },
    { 0x29d6a9c0, SUC_lexicon_tags_60 },
    { 0xfab6a9c1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d6a9c4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f6a9c7, SUC_lexicon_tags_88 },
    { 0xfc76a9c8, SUC_lexicon_tags_28 },
    { 0x4f36a9c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5136a9cd, SUC_lexicon_tags_11 },
    { 0x3336a9ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f6a9d1, SUC_lexicon_tags_25 },
    { 0x176a9d2, SUC_lexicon_tags_26 },
    { 0x5d96a9d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e76a9d6, SUC_lexicon_tags_92 },
    { 0x1b36a9d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa976a9d9, SUC_lexicon_tags_12 },
    { 0xe256a9d9, SUC_lexicon_tags_19 },
    { 0x5196a9db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9756a9de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7096a9e0, SUC_lexicon_tags_48 },
    { 0xca96a9e0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x6456a9e3, SUC_lexicon_tags_25 },
    { 0x3d36a9e4, SUC_lexicon_tags_15 },
    { 0x7f16a9e5, SUC_lexicon_tags_109 },
    { 0xcff6a9e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad16a9ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b96a9ef, SUC_lexicon_tags_7 },
    { 0x15d6a9f0, SUC_lexicon_tags_31 },
    { 0x2436a9ef, SUC_lexicon_tags_14 },
    { 0x7e76a9ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8296a9fa, SUC_lexicon_tags_49 },
    { 0x1916a9fb, SUC_lexicon_tags_33 },
    { 0xb3b6a9fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9176aa03, SUC_lexicon_tags_37 },
    { 0xb6d6aa03, SUC_lexicon_tags_11 },
    { 0xb176aa03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae56aa0a, SUC_lexicon_tags_14 },
    { 0xbdb6aa0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76aa12, SUC_lexicon_tags_15 },
    { 0x6ad6aa13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4df6aa15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd56aa18, SUC_lexicon_tags_0 },
    { 0x6216aa18, SUC_lexicon_tags_47 },
    { 0xed36aa18, SUC_lexicon_tags_65 },
    { 0x156aa1b, SUC_lexicon_tags_11 },
    { 0x7496aa1b, SUC_lexicon_tags_10 },
    { 0x4c76aa19, SUC_lexicon_tags_11 },
    { 0xf1f6aa1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed56aa20, SUC_lexicon_tags_17 },
    { 0x2c16aa20, SUC_lexicon_tags_27 },
    { 0x26b6aa22, SUC_lexicon_tags_15 },
    { 0xcbb6aa23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b6aa28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af6aa2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c36aa30, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xab76aa33, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d6aa36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb76aa3a, SUC_lexicon_tags_12 },
    { 0x3336aa3b, SUC_lexicon_tags_9 },
    { 0x8f96aa3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b6aa40, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8376aa4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d76aa4f, SUC_lexicon_tags_14 },
    { 0xcb16aa4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6176aa52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f6aa57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a96aa59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6836aa5d, SUC_lexicon_tags_25 },
    { 0x2256aa5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb856aa60, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5056aa62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad56aa66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd096aa68, SUC_lexicon_tags_45 },
    { 0x9fb6aa69, SUC_lexicon_tags_17 },
    { 0x3c96aa69, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa376aa6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa816aa75, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf656aa7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd6aa7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2016aa82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2916aa84, SUC_lexicon_tags_11 },
    { 0xfd36aa85, SUC_lexicon_tags_2 },
    { 0x7c16aa85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa156aa8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6536aa8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32b6aa8f, SUC_lexicon_tags_25 },
    { 0x88d6aa90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb256aa94, SUC_lexicon_tags_14 },
    { 0xbcf6aa95, SUC_lexicon_tags_37 },
    { 0x80b6aa96, SUC_lexicon_tags_37 },
    { 0x9836aa96, SUC_lexicon_tags_34 },
    { 0x28f6aa96, SUC_lexicon_tags_2 },
    { 0xe0f6aa94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d36aa9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9b6aa9d, SUC_lexicon_tags_26 },
    { 0x3636aa9e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x616aaa0, SUC_lexicon_tags_1 },
    { 0xef36aaa0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b6aaa4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x45b6aaa6, SUC_lexicon_tags_13 },
    { 0x2956aaa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac56aaa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4b6aaab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf6aaad, SUC_lexicon_tags_47 },
    { 0xf596aaad, SUC_lexicon_tags_63 },
    { 0x4296aaae, SUC_lexicon_tags_33 },
    { 0xbc96aab0, SUC_lexicon_tags_12 },
    { 0x8e96aab0, SUC_lexicon_tags_151 },
    { 0xb736aab2, SUC_lexicon_tags_37 },
    { 0xf696aab1, SUC_lexicon_tags_25 },
    { 0x6376aab4, SUC_lexicon_tags_11 },
    { 0x9d36aab5, SUC_lexicon_tags_82 },
    { 0x836aab1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6856aab9, SUC_lexicon_tags_0 },
    { 0xf5b6aaba, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7276aac0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7356aac2, SUC_lexicon_tags_1 },
    { 0xd9b6aac2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f96aac5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7356aaca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f6aad1, SUC_lexicon_tags_7 },
    { 0x1f36aad2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d6aad7, SUC_lexicon_tags_9 },
    { 0xea96aad8, SUC_lexicon_tags_230 },
    { 0x9c36aad7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfed6aadb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe2b6aadd, SUC_lexicon_tags_11 },
    { 0xc4f6aade, SUC_lexicon_tags_7 },
    { 0xc616aadf, SUC_lexicon_tags_25 },
    { 0xe596aae0, SUC_lexicon_tags_44 },
    { 0xef96aae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa276aae3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8976aae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e16aaeb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9df6aaed, SUC_lexicon_tags_32 },
    { 0x1316aaee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x776aaf0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x7816aaf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf76aafa, SUC_lexicon_tags_40 },
    { 0xa1f6aafb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9176aafd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2096ab00, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3676ab04, SUC_lexicon_tags_28 },
    { 0x8456ab05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b6ab09, SUC_lexicon_tags_17 },
    { 0x7376ab0a, SUC_lexicon_tags_1 },
    { 0x7556ab0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3796ab0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8816ab0f, SUC_lexicon_tags_1 },
    { 0xc596ab10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c96ab13, SUC_lexicon_tags_104 },
    { 0x8ef6ab14, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a76ab17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916ab1b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb76ab1f, SUC_lexicon_tags_1 },
    { 0x7d36ab20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d6ab25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9756ab28, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b6ab2b, SUC_lexicon_tags_33 },
    { 0x1316ab2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9836ab30, SUC_lexicon_tags_5 },
    { 0xb6d6ab30, SUC_lexicon_tags_11 },
    { 0x92d6ab32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4996ab34, SUC_lexicon_tags_69 },
    { 0x8516ab34, SUC_lexicon_tags_12 },
    { 0xabb6ab36, SUC_lexicon_tags_17 },
    { 0xae56ab34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb296ab3a, SUC_lexicon_tags_37 },
    { 0xb636ab3a, SUC_lexicon_tags_15 },
    { 0x28f6ab3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36ab41, SUC_lexicon_tags_31 },
    { 0x70f6ab41, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef36ab48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf876ab4b, SUC_lexicon_tags_55 },
    { 0x42b6ab4b, SUC_lexicon_tags_15 },
    { 0x3196ab4b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b6ab53, SUC_lexicon_tags_83 },
    { 0x96f6ab53, SUC_lexicon_tags_68 },
    { 0x3196ab55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef6ab57, SUC_lexicon_tags_15 },
    { 0x7cb6ab57, SUC_lexicon_tags_7 },
    { 0xd456ab57, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7e96ab5b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad76ab5f, SUC_lexicon_tags_214 },
    { 0x3236ab60, SUC_lexicon_tags_29 },
    { 0xd56ab61, SUC_lexicon_tags_30 },
    { 0xc616ab62, SUC_lexicon_tags_11 },
    { 0x2ad6ab63, SUC_lexicon_tags_92 },
    { 0x4a56ab5f, SUC_lexicon_tags_19 },
    { 0x4d56ab65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf976ab67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb6ab6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a76ab6e, SUC_lexicon_tags_12 },
    { 0xc016ab6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb196ab71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff6ab78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d76ab7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b6ab80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6416ab82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e96ab84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6396ab86, SUC_lexicon_tags_47 },
    { 0xe9f6ab87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4136ab8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3136ab8f, SUC_lexicon_tags_15 },
    { 0xb436ab90, SUC_lexicon_tags_11 },
    { 0x1df6ab91, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b6ab95, SUC_lexicon_tags_1 },
    { 0x43f6ab96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb16ab98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa796aba1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df6aba7, SUC_lexicon_tags_34 },
    { 0x75f6aba8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c36abae, SUC_lexicon_tags_2 },
    { 0x4096abaf, SUC_lexicon_tags_7 },
    { 0x2336abb0, SUC_lexicon_tags_29 },
    { 0x7976abb1, SUC_lexicon_tags_7 },
    { 0x896abb2, SUC_lexicon_tags_1 },
    { 0x9956abb3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xb076abb6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc56abba, SUC_lexicon_tags_87 },
    { 0xca36abba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8876abbf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa476abc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa816abcc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f16abcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6396abd2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x576abd4, SUC_lexicon_tags_37 },
    { 0xeb56abd5, SUC_lexicon_tags_12 },
    { 0x3e16abd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5036abd8, SUC_lexicon_tags_47 },
    { 0x43b6abd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d6abdc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2976abdf, SUC_lexicon_tags_14 },
    { 0xa576abdf, SUC_lexicon_tags_28 },
    { 0x44d6abdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6b6abe3, SUC_lexicon_tags_28 },
    { 0xca36abe4, SUC_lexicon_tags_1 },
    { 0x6c56abe5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x89d6abe7, SUC_lexicon_tags_7 },
    { 0x47b6abe7, SUC_lexicon_tags_47 },
    { 0x4236abe7, SUC_lexicon_tags_23 },
    { 0xe0f6abe8, SUC_lexicon_tags_1 },
    { 0x2696abeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaad6abed, SUC_lexicon_tags_25 },
    { 0xaa36abee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x136abf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8d6abf3, SUC_lexicon_tags_10 },
    { 0x8836abf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2436abf6, SUC_lexicon_tags_23 },
    { 0xe976abf7, SUC_lexicon_tags_0 },
    { 0xcd56abf6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8076abfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac76abff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xce96ac02, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b6ac08, SUC_lexicon_tags_55 },
    { 0x28d6ac09, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96ac0d, SUC_lexicon_tags_56 },
    { 0xe836ac0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc16ac12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9916ac1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5976ac1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b6ac20, SUC_lexicon_tags_30 },
    { 0x9736ac21, SUC_lexicon_tags_2 },
    { 0xf136ac21, SUC_lexicon_tags_45 },
    { 0x96b6ac20, SUC_lexicon_tags_1 },
    { 0x6356ac24, SUC_lexicon_tags_25 },
    { 0x4ed6ac22, SUC_lexicon_tags_87 },
    { 0x98d6ac22, SUC_lexicon_tags_29 },
    { 0x7696ac26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe396ac29, SUC_lexicon_tags_26 },
    { 0x5fb6ac29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f6ac38, SUC_lexicon_tags_26 },
    { 0xfc16ac39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d6ac3d, SUC_lexicon_tags_14 },
    { 0x4636ac3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d6ac49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2376ac4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd456ac50, SUC_lexicon_tags_46 },
    { 0x31d6ac51, SUC_lexicon_tags_23 },
    { 0xac76ac52, SUC_lexicon_tags_933 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc896ac56, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf9f6ac58, SUC_lexicon_tags_19 },
    { 0x2016ac58, SUC_lexicon_tags_28 },
    { 0x9156ac5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8136ac65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd76ac68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf6ac6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1296ac6d, SUC_lexicon_tags_17 },
    { 0xd496ac6e, SUC_lexicon_tags_47 },
    { 0xed6ac6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6096ac72, SUC_lexicon_tags_43 },
    { 0x45b6ac73, SUC_lexicon_tags_23 },
    { 0x41f6ac72, SUC_lexicon_tags_60 },
    { 0x43b6ac75, SUC_lexicon_tags_63 },
    { 0x8016ac76, SUC_lexicon_tags_44 },
    { 0xb2b6ac75, SUC_lexicon_tags_25 },
    { 0x9716ac77, SUC_lexicon_tags_15 },
    { 0x2176ac77, SUC_lexicon_tags_0 },
    { 0x5d36ac77, SUC_lexicon_tags_26 },
    { 0xfe96ac7b, SUC_lexicon_tags_25 },
    { 0x396ac7b, SUC_lexicon_tags_15 },
    { 0x9856ac7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee56ac80, SUC_lexicon_tags_17 },
    { 0x7536ac81, SUC_lexicon_tags_17 },
    { 0x5c56ac81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b96ac84, SUC_lexicon_tags_25 },
    { 0x4bf6ac85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56ac89, SUC_lexicon_tags_15 },
    { 0x9ab6ac8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda96ac92, SUC_lexicon_tags_10 },
    { 0xb656ac93, SUC_lexicon_tags_15 },
    { 0x11d6ac93, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f6ac97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd996ac99, SUC_lexicon_tags_6 },
    { 0xc196ac9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d6ac9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b36aca6, SUC_lexicon_tags_7 },
    { 0x22b6aca6, SUC_lexicon_tags_17 },
    { 0xf1b6aca7, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d6acac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8736acb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3596acb4, SUC_lexicon_tags_37 },
    { 0xbab6acb5, SUC_lexicon_tags_25 },
    { 0xd676acb6, SUC_lexicon_tags_9 },
    { 0xe4f6acb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f6acbc, SUC_lexicon_tags_87 },
    { 0xa76acbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2d6acbf, SUC_lexicon_tags_42 },
    { 0xabf6acc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc776acc6, SUC_lexicon_tags_7 },
    { 0x5a16acc7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x23b6acc9, SUC_lexicon_tags_37 },
    { 0xd496acca, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d56acce, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f6acd3, SUC_lexicon_tags_68 },
    { 0xe836acd4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b6acd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa56acda, SUC_lexicon_tags_2 },
    { 0xe656acdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd856acde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf096ace0, SUC_lexicon_tags_19 },
    { 0x3716ace0, SUC_lexicon_tags_23 },
    { 0x6a36ace2, SUC_lexicon_tags_0 },
    { 0x1d96ace3, SUC_lexicon_tags_63 },
    { 0xd1b6ace2, SUC_lexicon_tags_28 },
    { 0x8576ace3, SUC_lexicon_tags_38 },
    { 0xa3d6ace2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d6aceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb316acf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1216acf3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd16acf8, SUC_lexicon_tags_33 },
    { 0xfa76acf8, SUC_lexicon_tags_59 },
    { 0xf8d6acfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb6acfd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe656acff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7176ad05, SUC_lexicon_tags_37 },
    { 0x6356ad06, SUC_lexicon_tags_15 },
    { 0x79b6ad05, SUC_lexicon_tags_1 },
    { 0x6e36ad08, SUC_lexicon_tags_22 },
    { 0x2ab6ad08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd136ad10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e56ad13, SUC_lexicon_tags_11 },
    { 0x4ab6ad13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f6ad19, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d6ad1c, SUC_lexicon_tags_7 },
    { 0x8696ad1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d6ad20, SUC_lexicon_tags_1 },
    { 0x4ff6ad20, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8876ad25, SUC_lexicon_tags_1 },
    { 0x67d6ad26, SUC_lexicon_tags_1 },
    { 0x7c96ad25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab6ad2f, SUC_lexicon_tags_1 },
    { 0xb236ad30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef76ad34, SUC_lexicon_tags_25 },
    { 0x4fb6ad35, SUC_lexicon_tags_14 },
    { 0x16b6ad34, SUC_lexicon_tags_8 },
    { 0x9616ad37, SUC_lexicon_tags_29 },
    { 0xd916ad34, SUC_lexicon_tags_25 },
    { 0xbed6ad38, SUC_lexicon_tags_63 },
    { 0x57b6ad3a, SUC_lexicon_tags_15 },
    { 0x78f6ad3b, SUC_lexicon_tags_37 },
    { 0x7f36ad36, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa136ad3f, SUC_lexicon_tags_60 },
    { 0x3e76ad3f, SUC_lexicon_tags_8 },
    { 0x17d6ad41, SUC_lexicon_tags_7 },
    { 0x28b6ad3f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d16ad46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b6ad49, SUC_lexicon_tags_11 },
    { 0xe236ad4a, SUC_lexicon_tags_23 },
    { 0xd7d6ad4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d6ad4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31d6ad51, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d6ad5b, SUC_lexicon_tags_133 },
    { 0xf4b6ad5b, SUC_lexicon_tags_11 },
    { 0x7a16ad5b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x9b36ad5f, SUC_lexicon_tags_27 },
    { 0x1af6ad60, SUC_lexicon_tags_60 },
    { 0xd2b6ad61, SUC_lexicon_tags_15 },
    { 0xfcd6ad62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d6ad68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ed6ad6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f6ad6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2256ad70, SUC_lexicon_tags_15 },
    { 0xf056ad71, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e76ad77, SUC_lexicon_tags_23 },
    { 0x2b56ad77, SUC_lexicon_tags_12 },
    { 0x15b6ad79, SUC_lexicon_tags_40 },
    { 0x2c36ad79, SUC_lexicon_tags_27 },
    { 0x8b76ad7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf16ad7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd6ad81, SUC_lexicon_tags_33 },
    { 0x1fd6ad82, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f16ad85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6ad88, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f6ad8b, SUC_lexicon_tags_2 },
    { 0x1d6ad8b, SUC_lexicon_tags_37 },
    { 0xa7f6ad8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc676ad8f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b6ad94, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f6ad98, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf6ad9b, SUC_lexicon_tags_12 },
    { 0xba96ad9c, SUC_lexicon_tags_37 },
    { 0x3656ad9b, SUC_lexicon_tags_14 },
    { 0x4f16ad9e, SUC_lexicon_tags_47 },
    { 0x6196ad9c, SUC_lexicon_tags_17 },
    { 0x696ad9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe516ada2, SUC_lexicon_tags_25 },
    { 0x96b6ada3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb5d6ada5, SUC_lexicon_tags_45 },
    { 0xd736ada6, SUC_lexicon_tags_7 },
    { 0xa596ada7, SUC_lexicon_tags_17 },
    { 0x3f16ada8, SUC_lexicon_tags_9 },
    { 0x7836ada9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6376adab, SUC_lexicon_tags_45 },
    { 0xf996adac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf776adae, SUC_lexicon_tags_47 },
    { 0x38d6adaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f36adb1, SUC_lexicon_tags_9 },
    { 0xdcb6adb1, SUC_lexicon_tags_31 },
    { 0xa436adb2, SUC_lexicon_tags_84 },
    { 0xe0b6adb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31d6adb6, SUC_lexicon_tags_12 },
    { 0x8056adb6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57b6adb9, SUC_lexicon_tags_37 },
    { 0xa936adb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8796adbe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6fd6adc0, SUC_lexicon_tags_17 },
    { 0x78b6adc0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd76adc3, SUC_lexicon_tags_15 },
    { 0x356adc4, SUC_lexicon_tags_7 },
    { 0xa5f6adc4, SUC_lexicon_tags_44 },
    { 0xb3b6adc6, SUC_lexicon_tags_7 },
    { 0x4b56adc5, SUC_lexicon_tags_25 },
    { 0xbb96adc8, SUC_lexicon_tags_9 },
    { 0xd736adc8, SUC_lexicon_tags_10 },
    { 0xa136adc5, SUC_lexicon_tags_39 },
    { 0xa196adcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c76adcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b96adcf, SUC_lexicon_tags_0 },
    { 0x2316add0, SUC_lexicon_tags_17 },
    { 0xa4f6adcf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7196add3, SUC_lexicon_tags_57 },
    { 0xea36add4, SUC_lexicon_tags_29 },
    { 0xd936add4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd6addb, SUC_lexicon_tags_31 },
    { 0x6cd6addc, SUC_lexicon_tags_17 },
    { 0x76b6addd, SUC_lexicon_tags_138 },
    { 0x9656addb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf816ade0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2816ade2, SUC_lexicon_tags_38 },
    { 0xf556ade3, SUC_lexicon_tags_17 },
    { 0x1a56ade3, SUC_lexicon_tags_33 },
    { 0xf5d6ade3, SUC_lexicon_tags_60 },
    { 0x4116ade6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b6ade9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f16aded, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b6adef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf776adf1, SUC_lexicon_tags_1 },
    { 0x1c96adf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4756adfa, SUC_lexicon_tags_920 },
    { 0xa636adfa, SUC_lexicon_tags_37 },
    { 0x8a16adfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d76ae04, SUC_lexicon_tags_15 },
    { 0x6a76ae04, SUC_lexicon_tags_1 },
    { 0x3c36ae06, SUC_lexicon_tags_27 },
    { 0x31b6ae06, SUC_lexicon_tags_11 },
    { 0xb8f6ae08, SUC_lexicon_tags_25 },
    { 0x4fb6ae09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e96ae0b, SUC_lexicon_tags_7 },
    { 0xf076ae0c, SUC_lexicon_tags_42 },
    { 0xe3b6ae0c, SUC_lexicon_tags_68 },
    { 0x31b6ae0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40f6ae10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b6ae1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2456ae22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9736ae25, SUC_lexicon_tags_25 },
    { 0xccf6ae26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4276ae28, SUC_lexicon_tags_48 },
    { 0x4696ae29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6ae2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6b6ae31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8156ae34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb6ae3c, SUC_lexicon_tags_23 },
    { 0x4196ae3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf856ae42, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f6ae45, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3296ae48, SUC_lexicon_tags_25 },
    { 0x4076ae49, SUC_lexicon_tags_45 },
    { 0xc3d6ae4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc496ae4d, SUC_lexicon_tags_47 },
    { 0x1e16ae4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb6ae56, SUC_lexicon_tags_15 },
    { 0xa136ae56, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xce76ae59, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x75f6ae5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe436ae5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af6ae64, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf156ae66, SUC_lexicon_tags_63 },
    { 0x8f16ae67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf16ae69, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xefb6ae6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f36ae6d, SUC_lexicon_tags_33 },
    { 0x5bb6ae6d, SUC_lexicon_tags_63 },
    { 0x5916ae6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad6ae72, SUC_lexicon_tags_23 },
    { 0x1296ae72, SUC_lexicon_tags_4 },
    { 0x9eb6ae74, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4776ae77, SUC_lexicon_tags_25 },
    { 0xa116ae77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe96ae81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a76ae89, SUC_lexicon_tags_10 },
    { 0xbc16ae8a, SUC_lexicon_tags_41 },
    { 0x3af6ae89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f6ae8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad6ae98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d96ae9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d96aea4, SUC_lexicon_tags_31 },
    { 0x676aea5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa36aeaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9f6aeac, SUC_lexicon_tags_15 },
    { 0x7e16aeac, SUC_lexicon_tags_33 },
    { 0xfb76aeac, SUC_lexicon_tags_1 },
    { 0xc4b6aeaf, SUC_lexicon_tags_37 },
    { 0xbab6aeaf, SUC_lexicon_tags_112 },
    { 0x7fb6aead, SUC_lexicon_tags_9 },
    { 0x13b6aeb2, SUC_lexicon_tags_37 },
    { 0xe96aeb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b36aeb7, SUC_lexicon_tags_27 },
    { 0x11f6aeb8, SUC_lexicon_tags_39 },
    { 0x75f6aeb8, SUC_lexicon_tags_15 },
    { 0x6436aeb7, SUC_lexicon_tags_12 },
    { 0xef16aebb, SUC_lexicon_tags_673 },
    { 0, NULL },
    { 0x3876aebd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc636aec1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccb6aec3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa956aec6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4df6aec8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1476aecd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3236aecf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6aed2, SUC_lexicon_tags_1 },
    { 0x1b76aed3, SUC_lexicon_tags_17 },
    { 0x5536aed3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4916aed6, SUC_lexicon_tags_4 },
    { 0xbeb6aed7, SUC_lexicon_tags_31 },
    { 0x8d16aed8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3136aedc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7336aee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2416aee5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f6aeec, SUC_lexicon_tags_43 },
    { 0x1636aeec, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd6aef4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf356aef6, SUC_lexicon_tags_10 },
    { 0x5956aef7, SUC_lexicon_tags_66 },
    { 0x2e76aef7, SUC_lexicon_tags_237 },
    { 0x58d6aef9, SUC_lexicon_tags_45 },
    { 0x4216aef8, SUC_lexicon_tags_47 },
    { 0xced6aef8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd956af05, SUC_lexicon_tags_101 },
    { 0xabd6af06, SUC_lexicon_tags_37 },
    { 0xe976af06, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xef36af0a, SUC_lexicon_tags_6 },
    { 0x55d6af0a, SUC_lexicon_tags_104 },
    { 0xf2d6af0c, SUC_lexicon_tags_12 },
    { 0x3e96af0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9116af0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9656af12, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac96af14, SUC_lexicon_tags_2 },
    { 0xd816af15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f6af18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bb6af1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d6af1d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f6af20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6236af22, SUC_lexicon_tags_23 },
    { 0x7dd6af22, SUC_lexicon_tags_8 },
    { 0xb076af24, SUC_lexicon_tags_18 },
    { 0x3ff6af22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b6af2c, SUC_lexicon_tags_79 },
    { 0x6396af2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4776af2f, SUC_lexicon_tags_25 },
    { 0x14d6af2f, SUC_lexicon_tags_33 },
    { 0x50f6af31, SUC_lexicon_tags_21 },
    { 0xf5f6af32, SUC_lexicon_tags_1 },
    { 0x5fb6af33, SUC_lexicon_tags_47 },
    { 0x9116af33, SUC_lexicon_tags_17 },
    { 0x1936af31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd256af3b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xfb96af3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e16af41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff6af47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6d6af49, SUC_lexicon_tags_31 },
    { 0x37f6af4a, SUC_lexicon_tags_45 },
    { 0xa336af4a, SUC_lexicon_tags_15 },
    { 0x2fb6af4c, SUC_lexicon_tags_23 },
    { 0x7296af4b, SUC_lexicon_tags_64 },
    { 0x1316af4e, SUC_lexicon_tags_12 },
    { 0xe016af4e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa776af56, SUC_lexicon_tags_47 },
    { 0xf416af57, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf776af59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1396af5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3116af5d, SUC_lexicon_tags_34 },
    { 0x4376af5e, SUC_lexicon_tags_34 },
    { 0x1e36af5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9076af64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4396af67, SUC_lexicon_tags_37 },
    { 0xdb16af68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf16af6c, SUC_lexicon_tags_30 },
    { 0x1c76af6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1556af71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab96af74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b6af7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb836af7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb236af86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f6af8d, SUC_lexicon_tags_25 },
    { 0x1176af8e, SUC_lexicon_tags_25 },
    { 0xbaf6af8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa76af91, SUC_lexicon_tags_17 },
    { 0x9eb6af92, SUC_lexicon_tags_4 },
    { 0x5656af91, SUC_lexicon_tags_27 },
    { 0x73f6af94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8636af98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d6af9a, SUC_lexicon_tags_23 },
    { 0x17d6af9b, SUC_lexicon_tags_37 },
    { 0xc0f6af9b, SUC_lexicon_tags_30 },
    { 0xcb76af9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8f6af9f, SUC_lexicon_tags_8 },
    { 0x5b56afa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6afa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa196afa6, SUC_lexicon_tags_0 },
    { 0x3c56afa6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf876afa9, SUC_lexicon_tags_12 },
    { 0x5136afa9, SUC_lexicon_tags_26 },
    { 0xc356afab, SUC_lexicon_tags_29 },
    { 0xb8f6afac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b6afb1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdd76afb3, SUC_lexicon_tags_2 },
    { 0x6776afb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77b6afb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbef6afb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55b6afba, SUC_lexicon_tags_9 },
    { 0x4cd6afbb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6316afbe, SUC_lexicon_tags_68 },
    { 0x49f6afbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb6afc9, SUC_lexicon_tags_1 },
    { 0xae76afc9, SUC_lexicon_tags_55 },
    { 0x78f6afca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9356afcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f6afd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a56afd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96afdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1416afde, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc996afe3, SUC_lexicon_tags_37 },
    { 0xcd76afe4, SUC_lexicon_tags_23 },
    { 0x1016afe4, SUC_lexicon_tags_23 },
    { 0xb6f6afe5, SUC_lexicon_tags_26 },
    { 0xce96afe6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b6afeb, SUC_lexicon_tags_25 },
    { 0x3136afec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d6aff3, SUC_lexicon_tags_30 },
    { 0xeed6aff3, SUC_lexicon_tags_38 },
    { 0x62d6aff5, SUC_lexicon_tags_31 },
    { 0xad6aff6, SUC_lexicon_tags_17 },
    { 0xd56aff7, SUC_lexicon_tags_61 },
    { 0x4216aff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e56b001, SUC_lexicon_tags_27 },
    { 0xd796b002, SUC_lexicon_tags_104 },
    { 0x3d76b003, SUC_lexicon_tags_27 },
    { 0xb096b004, SUC_lexicon_tags_12 },
    { 0x8ef6b003, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa436b011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1536b016, SUC_lexicon_tags_9 },
    { 0x7016b017, SUC_lexicon_tags_10 },
    { 0x2b6b018, SUC_lexicon_tags_59 },
    { 0xdb36b019, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9916b01e, SUC_lexicon_tags_113 },
    { 0x7936b01f, SUC_lexicon_tags_16 },
    { 0x1256b020, SUC_lexicon_tags_2 },
    { 0xc6d6b01f, SUC_lexicon_tags_17 },
    { 0xec36b01e, SUC_lexicon_tags_2 },
    { 0xd056b023, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x496b025, SUC_lexicon_tags_0 },
    { 0x6a56b025, SUC_lexicon_tags_920 },
    { 0x5096b026, SUC_lexicon_tags_28 },
    { 0x7076b028, SUC_lexicon_tags_21 },
    { 0xb6b028, SUC_lexicon_tags_19 },
    { 0x2d36b02a, SUC_lexicon_tags_6 },
    { 0x3776b029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6196b02d, SUC_lexicon_tags_11 },
    { 0x5996b02d, SUC_lexicon_tags_37 },
    { 0x1936b02e, SUC_lexicon_tags_11 },
    { 0xa2f6b02e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd6b034, SUC_lexicon_tags_49 },
    { 0xe556b035, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2696b037, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd3f6b039, SUC_lexicon_tags_25 },
    { 0x3836b039, SUC_lexicon_tags_25 },
    { 0xaf96b03b, SUC_lexicon_tags_19 },
    { 0x2cb6b03b, SUC_lexicon_tags_37 },
    { 0xd3f6b03d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf36b040, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3776b045, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc156b049, SUC_lexicon_tags_37 },
    { 0xc1b6b049, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216b04e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4b56b050, SUC_lexicon_tags_1 },
    { 0x5336b051, SUC_lexicon_tags_25 },
    { 0x8d6b051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3036b055, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f6b05b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d6b05f, SUC_lexicon_tags_45 },
    { 0x6156b060, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9796b062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cb6b064, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf56b066, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbd6b068, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd56b06d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e76b06f, SUC_lexicon_tags_12 },
    { 0x2636b070, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6316b075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd136b079, SUC_lexicon_tags_27 },
    { 0xcab6b07a, SUC_lexicon_tags_49 },
    { 0x24d6b079, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56b07f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9256b082, SUC_lexicon_tags_25 },
    { 0xe76b083, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2716b085, SUC_lexicon_tags_2 },
    { 0x416b086, SUC_lexicon_tags_1 },
    { 0xb2b6b087, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d6b08a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe16b08c, SUC_lexicon_tags_8 },
    { 0x4c96b08d, SUC_lexicon_tags_33 },
    { 0x3856b08d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b6b097, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b76b09b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe176b09e, SUC_lexicon_tags_12 },
    { 0xd9d6b09f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d6b0a2, SUC_lexicon_tags_17 },
    { 0xc916b0a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d6b0a5, SUC_lexicon_tags_15 },
    { 0x17f6b0a6, SUC_lexicon_tags_15 },
    { 0x47d6b0a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3116b0aa, SUC_lexicon_tags_31 },
    { 0xcf36b0ab, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7b56b0ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff56b0b2, SUC_lexicon_tags_11 },
    { 0x7cd6b0b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x30f6b0b5, SUC_lexicon_tags_63 },
    { 0xf716b0b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f6b0b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfd6b0bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff96b0be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7636b0c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b6b0c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f6b0c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa176b0ca, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e96b0cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f16b0d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4696b0d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac56b0d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8796b0da, SUC_lexicon_tags_32 },
    { 0x496b0da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5516b0e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d6b0e9, SUC_lexicon_tags_80 },
    { 0xf676b0ea, SUC_lexicon_tags_16 },
    { 0xc076b0eb, SUC_lexicon_tags_7 },
    { 0x6ad6b0eb, SUC_lexicon_tags_0 },
    { 0xd2f6b0ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836b0f2, SUC_lexicon_tags_12 },
    { 0xf076b0f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf356b0f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb6b0fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5136b10c, SUC_lexicon_tags_23 },
    { 0xfb96b10d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa436b111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9776b11a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5276b11d, SUC_lexicon_tags_2 },
    { 0xff6b11e, SUC_lexicon_tags_63 },
    { 0xf356b11f, SUC_lexicon_tags_17 },
    { 0x3236b120, SUC_lexicon_tags_3 },
    { 0xf76b120, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf6b127, SUC_lexicon_tags_2 },
    { 0xd656b128, SUC_lexicon_tags_48 },
    { 0x4f6b128, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb256b12b, SUC_lexicon_tags_49 },
    { 0xf376b12b, SUC_lexicon_tags_43 },
    { 0xd236b12b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9996b132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d36b135, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7636b140, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe156b143, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d6b147, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f6b14b, SUC_lexicon_tags_27 },
    { 0x2136b14c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2ff6b14e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1896b157, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f6b15d, SUC_lexicon_tags_1 },
    { 0xadf6b15e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc16b165, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb76b167, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab6b16d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f56b170, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd696b174, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa3d6b176, SUC_lexicon_tags_7 },
    { 0xcdb6b177, SUC_lexicon_tags_63 },
    { 0x2c16b178, SUC_lexicon_tags_46 },
    { 0x2e36b178, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1116b17f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b186, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xea36b189, SUC_lexicon_tags_25 },
    { 0xae96b189, SUC_lexicon_tags_37 },
    { 0xaeb6b18b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa816b192, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad6b195, SUC_lexicon_tags_25 },
    { 0x60f6b196, SUC_lexicon_tags_17 },
    { 0x5556b196, SUC_lexicon_tags_2 },
    { 0xcfb6b196, SUC_lexicon_tags_5 },
    { 0x57d6b195, SUC_lexicon_tags_37 },
    { 0xfa76b19a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe996b19c, SUC_lexicon_tags_140 },
    { 0x6236b19d, SUC_lexicon_tags_25 },
    { 0x1576b19e, SUC_lexicon_tags_26 },
    { 0xadd6b19f, SUC_lexicon_tags_31 },
    { 0x3b36b19f, SUC_lexicon_tags_19 },
    { 0xad6b1a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf36b1a8, SUC_lexicon_tags_17 },
    { 0x3b36b1a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f16b1ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe236b1b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce76b1b7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xcb96b1b9, SUC_lexicon_tags_14 },
    { 0x4756b1b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3936b1bd, SUC_lexicon_tags_66 },
    { 0x7d96b1be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4eb6b1c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ef6b1c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4076b1c5, SUC_lexicon_tags_25 },
    { 0xf56b1c6, SUC_lexicon_tags_11 },
    { 0x8b96b1c7, SUC_lexicon_tags_46 },
    { 0x1336b1c8, SUC_lexicon_tags_17 },
    { 0xfa96b1c9, SUC_lexicon_tags_25 },
    { 0xaa76b1c8, SUC_lexicon_tags_28 },
    { 0x3c76b1c5, SUC_lexicon_tags_15 },
    { 0xb7b6b1cc, SUC_lexicon_tags_63 },
    { 0x17f6b1cc, SUC_lexicon_tags_17 },
    { 0x28b6b1cc, SUC_lexicon_tags_15 },
    { 0x4b36b1c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d6b1d2, SUC_lexicon_tags_37 },
    { 0x3276b1d2, SUC_lexicon_tags_25 },
    { 0x1476b1d4, SUC_lexicon_tags_0 },
    { 0xf476b1d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd6b1db, SUC_lexicon_tags_7 },
    { 0xa936b1db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2856b1e1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf6b1e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2096b1e7, SUC_lexicon_tags_17 },
    { 0x5fd6b1e7, SUC_lexicon_tags_33 },
    { 0xbd36b1e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf4f6b1eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d6b1f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8816b1fb, SUC_lexicon_tags_17 },
    { 0xa096b1fc, SUC_lexicon_tags_17 },
    { 0x6716b1fb, SUC_lexicon_tags_15 },
    { 0x18b6b1fb, SUC_lexicon_tags_525 },
    { 0xba76b1fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3716b202, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff96b207, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb596b20a, SUC_lexicon_tags_37 },
    { 0x44f6b20b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f6b20e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x8556b211, SUC_lexicon_tags_1 },
    { 0x1c76b211, SUC_lexicon_tags_29 },
    { 0x8c96b211, SUC_lexicon_tags_11 },
    { 0x67b6b214, SUC_lexicon_tags_4 },
    { 0x4716b213, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d6b219, SUC_lexicon_tags_15 },
    { 0x1716b21a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb616b21c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4856b21e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d6b222, SUC_lexicon_tags_26 },
    { 0xa7f6b222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6256b226, SUC_lexicon_tags_15 },
    { 0xafd6b227, SUC_lexicon_tags_39 },
    { 0xf896b228, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe916b22a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7456b22d, SUC_lexicon_tags_27 },
    { 0xa636b22e, SUC_lexicon_tags_2 },
    { 0x3a16b22f, SUC_lexicon_tags_37 },
    { 0x1df6b22f, SUC_lexicon_tags_47 },
    { 0xeb6b231, SUC_lexicon_tags_38 },
    { 0xa4d6b231, SUC_lexicon_tags_17 },
    { 0xaad6b22e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8536b236, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed6b239, SUC_lexicon_tags_146 },
    { 0xf616b23a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x436b23c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa776b23e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b6b242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb6b246, SUC_lexicon_tags_21 },
    { 0x6cf6b246, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc816b24e, SUC_lexicon_tags_15 },
    { 0xafd6b24e, SUC_lexicon_tags_12 },
    { 0x4cf6b250, SUC_lexicon_tags_49 },
    { 0x4d76b24e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf796b253, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf236b255, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6236b258, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf736b25f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe5f6b261, SUC_lexicon_tags_0 },
    { 0x2f6b262, SUC_lexicon_tags_37 },
    { 0x35d6b263, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9756b265, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5336b268, SUC_lexicon_tags_17 },
    { 0x6e56b269, SUC_lexicon_tags_38 },
    { 0x7576b26a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3276b26e, SUC_lexicon_tags_1 },
    { 0x71b6b26e, SUC_lexicon_tags_49 },
    { 0x29d6b26f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd556b272, SUC_lexicon_tags_60 },
    { 0xfdb6b272, SUC_lexicon_tags_35 },
    { 0xe376b274, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2356b27a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7836b27c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf456b27f, SUC_lexicon_tags_15 },
    { 0xd5d6b280, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x9336b282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b6b28a, SUC_lexicon_tags_14 },
    { 0x94b6b28b, SUC_lexicon_tags_1 },
    { 0x2bd6b28a, SUC_lexicon_tags_17 },
    { 0x49b6b28c, SUC_lexicon_tags_7 },
    { 0xe076b28b, SUC_lexicon_tags_0 },
    { 0xa2f6b28e, SUC_lexicon_tags_28 },
    { 0xc2f6b28f, SUC_lexicon_tags_11 },
    { 0xa6d6b28c, SUC_lexicon_tags_127 },
    { 0x9716b292, SUC_lexicon_tags_15 },
    { 0x3416b293, SUC_lexicon_tags_12 },
    { 0x5fb6b294, SUC_lexicon_tags_57 },
    { 0xb36b295, SUC_lexicon_tags_15 },
    { 0x91f6b296, SUC_lexicon_tags_17 },
    { 0x476b295, SUC_lexicon_tags_47 },
    { 0xea36b298, SUC_lexicon_tags_25 },
    { 0xb676b299, SUC_lexicon_tags_1 },
    { 0xce56b290, SUC_lexicon_tags_30 },
    { 0x5396b29b, SUC_lexicon_tags_7 },
    { 0x4616b29c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe296b29e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7d6b2a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1b6b2a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f6b2a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef6b2a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb6b2ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5116b2af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa5f6b2b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79b6b2b3, SUC_lexicon_tags_7 },
    { 0x16b2b4, SUC_lexicon_tags_25 },
    { 0x4836b2b5, SUC_lexicon_tags_29 },
    { 0x5db6b2b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e36b2ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8456b2bf, SUC_lexicon_tags_2 },
    { 0xa556b2bf, SUC_lexicon_tags_12 },
    { 0x42b6b2c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f56b2c5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xed16b2c7, SUC_lexicon_tags_17 },
    { 0x9a96b2c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6936b2cc, SUC_lexicon_tags_56 },
    { 0x3456b2cd, SUC_lexicon_tags_33 },
    { 0x7cb6b2ce, SUC_lexicon_tags_27 },
    { 0xf1f6b2ce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa236b2d3, SUC_lexicon_tags_33 },
    { 0x7a16b2d3, SUC_lexicon_tags_28 },
    { 0xe36b2d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf6b2dc, SUC_lexicon_tags_17 },
    { 0x3636b2dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f6b2e1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x29d6b2e3, SUC_lexicon_tags_37 },
    { 0x2a96b2e4, SUC_lexicon_tags_31 },
    { 0xbbb6b2e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x34d6b2e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa336b2e9, SUC_lexicon_tags_0 },
    { 0xcf36b2ea, SUC_lexicon_tags_38 },
    { 0xf2d6b2eb, SUC_lexicon_tags_2 },
    { 0xc9d6b2ec, SUC_lexicon_tags_37 },
    { 0x4216b2ec, SUC_lexicon_tags_9 },
    { 0x1756b2ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f6b2f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b56b2f4, SUC_lexicon_tags_27 },
    { 0xc656b2f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5556b2f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6b304, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b6b30b, SUC_lexicon_tags_26 },
    { 0x10f6b30c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16b310, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa296b312, SUC_lexicon_tags_25 },
    { 0x4bd6b313, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce16b31c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f16b321, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95d6b323, SUC_lexicon_tags_13 },
    { 0x9376b323, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b6b327, SUC_lexicon_tags_23 },
    { 0xbdb6b328, SUC_lexicon_tags_7 },
    { 0x60b6b329, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6b33d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d6b343, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb3f6b345, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xaa36b347, SUC_lexicon_tags_37 },
    { 0x6d16b348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa536b34a, SUC_lexicon_tags_66 },
    { 0x8476b34b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b6b34f, SUC_lexicon_tags_37 },
    { 0xadd6b34f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9296b352, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9816b359, SUC_lexicon_tags_0 },
    { 0x5676b35a, SUC_lexicon_tags_12 },
    { 0x3bf6b35b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d16b35e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb736b364, SUC_lexicon_tags_27 },
    { 0x9776b365, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ab6b367, SUC_lexicon_tags_2 },
    { 0x4896b367, SUC_lexicon_tags_65 },
    { 0xca36b369, SUC_lexicon_tags_11 },
    { 0xedb6b367, SUC_lexicon_tags_37 },
    { 0x2e36b368, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6376b370, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xf796b372, SUC_lexicon_tags_45 },
    { 0x6296b373, SUC_lexicon_tags_14 },
    { 0xf936b373, SUC_lexicon_tags_27 },
    { 0x7656b374, SUC_lexicon_tags_49 },
    { 0xab96b376, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xabf6b378, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1396b383, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16b386, SUC_lexicon_tags_60 },
    { 0x41b6b387, SUC_lexicon_tags_25 },
    { 0x6f36b388, SUC_lexicon_tags_15 },
    { 0x3b16b388, SUC_lexicon_tags_42 },
    { 0x56b6b38a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f56b38e, SUC_lexicon_tags_21 },
    { 0x5ff6b38e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d96b396, SUC_lexicon_tags_31 },
    { 0x2fb6b397, SUC_lexicon_tags_15 },
    { 0x4316b398, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb156b39a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc36b39d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b6b3a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5176b3a4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f6b3af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4f6b3b1, SUC_lexicon_tags_15 },
    { 0x4b76b3b1, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x716b3b5, SUC_lexicon_tags_33 },
    { 0x84b6b3b6, SUC_lexicon_tags_37 },
    { 0xde96b3b7, SUC_lexicon_tags_27 },
    { 0x5216b3b5, SUC_lexicon_tags_15 },
    { 0x4d76b3b8, SUC_lexicon_tags_37 },
    { 0x8796b3ba, SUC_lexicon_tags_60 },
    { 0xcbd6b3bb, SUC_lexicon_tags_79 },
    { 0x7cd6b3bc, SUC_lexicon_tags_82 },
    { 0x81f6b3b8, SUC_lexicon_tags_82 },
    { 0x60d6b3b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4396b3c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa736b3c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d6b3cb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f6b3ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5916b3d0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1336b3d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe376b3da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6296b3dc, SUC_lexicon_tags_2 },
    { 0xca76b3dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb776b3e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d6b3ec, SUC_lexicon_tags_1 },
    { 0x79f6b3ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a36b3f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f76b3fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8f36b400, SUC_lexicon_tags_49 },
    { 0x2836b401, SUC_lexicon_tags_37 },
    { 0xd56b400, SUC_lexicon_tags_17 },
    { 0xe2d6b402, SUC_lexicon_tags_2 },
    { 0x3396b404, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xadd6b406, SUC_lexicon_tags_23 },
    { 0x9636b407, SUC_lexicon_tags_70 },
    { 0xc816b407, SUC_lexicon_tags_9 },
    { 0xd696b409, SUC_lexicon_tags_26 },
    { 0xf336b40a, SUC_lexicon_tags_31 },
    { 0x6356b40a, SUC_lexicon_tags_27 },
    { 0x7276b408, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e36b410, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f6b414, SUC_lexicon_tags_25 },
    { 0xc116b415, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xf436b417, SUC_lexicon_tags_15 },
    { 0xab76b418, SUC_lexicon_tags_7 },
    { 0xed76b419, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf6b41d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd716b420, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8796b423, SUC_lexicon_tags_45 },
    { 0xad6b423, SUC_lexicon_tags_37 },
    { 0x2336b425, SUC_lexicon_tags_47 },
    { 0xe5f6b426, SUC_lexicon_tags_19 },
    { 0xa076b426, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d96b42d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc56b430, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f6b433, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x79f6b435, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4036b437, SUC_lexicon_tags_31 },
    { 0xa376b438, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb16b43b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x93b6b43d, SUC_lexicon_tags_87 },
    { 0xed76b43e, SUC_lexicon_tags_29 },
    { 0x5176b43e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf6b442, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1696b445, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc236b448, SUC_lexicon_tags_75 },
    { 0x24d6b449, SUC_lexicon_tags_49 },
    { 0xf796b44a, SUC_lexicon_tags_33 },
    { 0x1816b44b, SUC_lexicon_tags_35 },
    { 0x33b6b44c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x796b450, SUC_lexicon_tags_25 },
    { 0xe236b451, SUC_lexicon_tags_33 },
    { 0xc716b450, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab76b454, SUC_lexicon_tags_11 },
    { 0x6956b454, SUC_lexicon_tags_47 },
    { 0x28d6b456, SUC_lexicon_tags_10 },
    { 0xdbf6b457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefd6b459, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3016b45b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a56b45e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d76b460, SUC_lexicon_tags_15 },
    { 0x5896b461, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe976b463, SUC_lexicon_tags_29 },
    { 0x4356b463, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5816b466, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c96b46a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37f6b46c, SUC_lexicon_tags_11 },
    { 0x42b6b46d, SUC_lexicon_tags_33 },
    { 0x9af6b46e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ff6b470, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4596b475, SUC_lexicon_tags_15 },
    { 0x5076b476, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c36b47b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7656b47d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4296b47f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf616b481, SUC_lexicon_tags_37 },
    { 0x4f36b482, SUC_lexicon_tags_33 },
    { 0x8676b483, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd56b486, SUC_lexicon_tags_25 },
    { 0x2776b486, SUC_lexicon_tags_2 },
    { 0x84f6b488, SUC_lexicon_tags_26 },
    { 0x5616b489, SUC_lexicon_tags_2 },
    { 0x10b6b489, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b6b492, SUC_lexicon_tags_43 },
    { 0x66b6b493, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b6b496, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x4456b498, SUC_lexicon_tags_25 },
    { 0xe736b499, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e56b4a0, SUC_lexicon_tags_354 },
    { 0x5f56b4a1, SUC_lexicon_tags_60 },
    { 0x6596b4a2, SUC_lexicon_tags_11 },
    { 0xc216b4a3, SUC_lexicon_tags_33 },
    { 0xb876b4a3, SUC_lexicon_tags_17 },
    { 0x75d6b4a5, SUC_lexicon_tags_45 },
    { 0xeb56b4a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5516b4a9, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb356b4b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd936b4b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b96b4ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df6b4bd, SUC_lexicon_tags_29 },
    { 0xa916b4be, SUC_lexicon_tags_12 },
    { 0x21f6b4bf, SUC_lexicon_tags_11 },
    { 0x7276b4c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfad6b4c2, SUC_lexicon_tags_12 },
    { 0xf5d6b4c3, SUC_lexicon_tags_12 },
    { 0x71d6b4c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b16b4cb, SUC_lexicon_tags_2 },
    { 0x3296b4cc, SUC_lexicon_tags_12 },
    { 0xd836b4cd, SUC_lexicon_tags_84 },
    { 0x47b6b4ce, SUC_lexicon_tags_37 },
    { 0x47f6b4cf, SUC_lexicon_tags_1343 },
    { 0xcd36b4d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e56b4d9, SUC_lexicon_tags_17 },
    { 0x5ef6b4d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2cd6b4dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xef36b4de, SUC_lexicon_tags_28 },
    { 0x3cf6b4de, SUC_lexicon_tags_38 },
    { 0xfd76b4df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe696b4e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc716b4e6, SUC_lexicon_tags_32 },
    { 0xec36b4e7, SUC_lexicon_tags_17 },
    { 0x6ff6b4e8, SUC_lexicon_tags_11 },
    { 0x57d6b4e6, SUC_lexicon_tags_15 },
    { 0x5996b4ea, SUC_lexicon_tags_35 },
    { 0x4af6b4eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfd6b4ed, SUC_lexicon_tags_11 },
    { 0xd336b4ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d6b4f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2fb6b4f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc896b4f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e76b4f8, SUC_lexicon_tags_12 },
    { 0x7816b4f8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf036b4fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b6b508, SUC_lexicon_tags_2 },
    { 0xb076b508, SUC_lexicon_tags_12 },
    { 0xfad6b50a, SUC_lexicon_tags_3 },
    { 0xb996b50b, SUC_lexicon_tags_15 },
    { 0x69f6b509, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x49d6b50e, SUC_lexicon_tags_17 },
    { 0x7276b50e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf236b511, SUC_lexicon_tags_38 },
    { 0x2056b512, SUC_lexicon_tags_47 },
    { 0x3956b512, SUC_lexicon_tags_25 },
    { 0xcb16b513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8356b519, SUC_lexicon_tags_2 },
    { 0x5cb6b51a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf6b51e, SUC_lexicon_tags_60 },
    { 0xdb36b51e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc536b521, SUC_lexicon_tags_2 },
    { 0xd196b521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb6b525, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2356b529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f6b531, SUC_lexicon_tags_38 },
    { 0xe736b532, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e56b535, SUC_lexicon_tags_87 },
    { 0x8e36b535, SUC_lexicon_tags_17 },
    { 0x6056b535, SUC_lexicon_tags_7 },
    { 0x78f6b538, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6b53b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d6b53e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd316b548, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2256b54a, SUC_lexicon_tags_0 },
    { 0x1816b54b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e76b54f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5996b551, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d6b555, SUC_lexicon_tags_1 },
    { 0xbe56b556, SUC_lexicon_tags_4 },
    { 0x6a76b557, SUC_lexicon_tags_47 },
    { 0x9f6b557, SUC_lexicon_tags_1 },
    { 0x1fd6b559, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4376b55d, SUC_lexicon_tags_12 },
    { 0xa996b55e, SUC_lexicon_tags_0 },
    { 0x15d6b55f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc956b561, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa616b565, SUC_lexicon_tags_2 },
    { 0x5836b566, SUC_lexicon_tags_25 },
    { 0x3af6b566, SUC_lexicon_tags_38 },
    { 0xc356b568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a36b56d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe496b56f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1c96b571, SUC_lexicon_tags_17 },
    { 0xab56b572, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f6b576, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9776b57d, SUC_lexicon_tags_1 },
    { 0xb5b6b57e, SUC_lexicon_tags_60 },
    { 0x36d6b57f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8676b581, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf6b584, SUC_lexicon_tags_37 },
    { 0x7f6b584, SUC_lexicon_tags_2 },
    { 0xec36b586, SUC_lexicon_tags_28 },
    { 0xe3f6b586, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8d6b589, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc0b6b58b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4df6b58d, SUC_lexicon_tags_26 },
    { 0x5b56b58e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1af6b590, SUC_lexicon_tags_60 },
    { 0x3556b590, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd16b599, SUC_lexicon_tags_10 },
    { 0x55b6b59a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ad6b59c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f6b5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb676b5a8, SUC_lexicon_tags_70 },
    { 0xf3b6b5a8, SUC_lexicon_tags_69 },
    { 0xf6f6b5aa, SUC_lexicon_tags_6 },
    { 0x5a96b5a8, SUC_lexicon_tags_7 },
    { 0xa4b6b5ac, SUC_lexicon_tags_2 },
    { 0x5496b5ad, SUC_lexicon_tags_1 },
    { 0x4476b5aa, SUC_lexicon_tags_17 },
    { 0x5376b5af, SUC_lexicon_tags_21 },
    { 0xe056b5b0, SUC_lexicon_tags_26 },
    { 0x6bb6b5b1, SUC_lexicon_tags_1 },
    { 0x7a36b5a8, SUC_lexicon_tags_11 },
    { 0x9cf6b5b3, SUC_lexicon_tags_22 },
    { 0x5a76b5b4, SUC_lexicon_tags_6 },
    { 0x78f6b5b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf636b5b8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5316b5bc, SUC_lexicon_tags_37 },
    { 0xd536b5bd, SUC_lexicon_tags_16 },
    { 0xcdf6b5be, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4556b5c6, SUC_lexicon_tags_478 },
    { 0x3736b5c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc56b5ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3816b5cd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x6416b5cf, SUC_lexicon_tags_26 },
    { 0xdad6b5d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee16b5d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b6b5d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d6b5d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdeb6b5db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe96b5e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc56b5e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x276b5eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacd6b5ed, SUC_lexicon_tags_4 },
    { 0x16f6b5ee, SUC_lexicon_tags_10 },
    { 0x3036b5ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3f6b5f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a16b5f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b6b5fd, SUC_lexicon_tags_9 },
    { 0xfc36b5fe, SUC_lexicon_tags_179 },
    { 0x75b6b5fe, SUC_lexicon_tags_19 },
    { 0xd016b5ff, SUC_lexicon_tags_15 },
    { 0x7a16b600, SUC_lexicon_tags_1 },
    { 0xe8f6b5ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9216b605, SUC_lexicon_tags_43 },
    { 0x3a76b606, SUC_lexicon_tags_1 },
    { 0x1b76b607, SUC_lexicon_tags_43 },
    { 0xd496b608, SUC_lexicon_tags_17 },
    { 0x9b96b607, SUC_lexicon_tags_29 },
    { 0xbdf6b605, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40d6b60c, SUC_lexicon_tags_14 },
    { 0x5236b60c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cd6b60f, SUC_lexicon_tags_37 },
    { 0x48d6b610, SUC_lexicon_tags_27 },
    { 0x4f56b611, SUC_lexicon_tags_17 },
    { 0x5c96b611, SUC_lexicon_tags_37 },
    { 0x56b6b611, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d6b617, SUC_lexicon_tags_15 },
    { 0x4bf6b618, SUC_lexicon_tags_22 },
    { 0x2516b617, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c56b61c, SUC_lexicon_tags_65 },
    { 0x2556b61d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfad6b61f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb6b627, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6376b629, SUC_lexicon_tags_2 },
    { 0xfad6b62a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x24f6b62c, SUC_lexicon_tags_11 },
    { 0xa636b62d, SUC_lexicon_tags_68 },
    { 0xa1b6b62e, SUC_lexicon_tags_15 },
    { 0xd736b62d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa256b632, SUC_lexicon_tags_17 },
    { 0x7156b632, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e96b636, SUC_lexicon_tags_25 },
    { 0x376b637, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed6b63f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8d6b641, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad6b644, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f6b647, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8776b64b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac16b656, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee16b65a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56b65e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5d6b660, SUC_lexicon_tags_17 },
    { 0x1fb6b661, SUC_lexicon_tags_178 },
    { 0x3836b662, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b76b666, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x35d6b668, SUC_lexicon_tags_15 },
    { 0x81b6b668, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf36b66c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe596b66f, SUC_lexicon_tags_7 },
    { 0x90f6b670, SUC_lexicon_tags_17 },
    { 0x13d6b670, SUC_lexicon_tags_25 },
    { 0x796b672, SUC_lexicon_tags_2 },
    { 0xe7b6b673, SUC_lexicon_tags_42 },
    { 0x2a76b673, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6696b677, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe856b67b, SUC_lexicon_tags_27 },
    { 0x8df6b67c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x17f6b67e, SUC_lexicon_tags_11 },
    { 0xdcb6b67f, SUC_lexicon_tags_175 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9116b683, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6b687, SUC_lexicon_tags_15 },
    { 0x2a36b688, SUC_lexicon_tags_11 },
    { 0xe9f6b688, SUC_lexicon_tags_13 },
    { 0x8156b68a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e56b68e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db6b693, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb996b698, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc636b69a, SUC_lexicon_tags_2 },
    { 0xb516b69b, SUC_lexicon_tags_9 },
    { 0xabd6b69a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f6b6a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6836b6a5, SUC_lexicon_tags_9 },
    { 0xcd96b6a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb36b6ab, SUC_lexicon_tags_14 },
    { 0xcf96b6ac, SUC_lexicon_tags_7 },
    { 0x58f6b6ad, SUC_lexicon_tags_8 },
    { 0x85d6b6ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b6b6b5, SUC_lexicon_tags_39 },
    { 0x6e56b6b6, SUC_lexicon_tags_29 },
    { 0xb6f6b6b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f6b6ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5956b6bd, SUC_lexicon_tags_36 },
    { 0xb856b6be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2316b6c5, SUC_lexicon_tags_45 },
    { 0xff36b6c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb756b6d3, SUC_lexicon_tags_15 },
    { 0xa156b6d3, SUC_lexicon_tags_2 },
    { 0x9d56b6d3, SUC_lexicon_tags_37 },
    { 0xf96b6d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16b6d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1276b6db, SUC_lexicon_tags_17 },
    { 0x2c56b6dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40b6b6de, SUC_lexicon_tags_2 },
    { 0xabd6b6df, SUC_lexicon_tags_71 },
    { 0x5916b6e0, SUC_lexicon_tags_48 },
    { 0xf1d6b6e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda96b6e3, SUC_lexicon_tags_9 },
    { 0x64f6b6e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896b6f0, SUC_lexicon_tags_3 },
    { 0x6f36b6f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d36b6f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c36b6f6, SUC_lexicon_tags_19 },
    { 0x2b56b6f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3216b6fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b76b6fc, SUC_lexicon_tags_29 },
    { 0x6356b6fd, SUC_lexicon_tags_49 },
    { 0x2fb6b6fc, SUC_lexicon_tags_10 },
    { 0x45f6b6ff, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x8956b701, SUC_lexicon_tags_11 },
    { 0xc116b702, SUC_lexicon_tags_31 },
    { 0xbc76b703, SUC_lexicon_tags_23 },
    { 0xacd6b704, SUC_lexicon_tags_39 },
    { 0x4216b705, SUC_lexicon_tags_15 },
    { 0x9ab6b703, SUC_lexicon_tags_23 },
    { 0x7cb6b703, SUC_lexicon_tags_37 },
    { 0x9d96b708, SUC_lexicon_tags_17 },
    { 0xa656b709, SUC_lexicon_tags_37 },
    { 0x44f6b70a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd256b70e, SUC_lexicon_tags_11 },
    { 0x25b6b70f, SUC_lexicon_tags_43 },
    { 0x7356b710, SUC_lexicon_tags_173 },
    { 0x6af6b70e, SUC_lexicon_tags_91 },
    { 0x836b711, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9036b714, SUC_lexicon_tags_38 },
    { 0xa316b715, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d6b71c, SUC_lexicon_tags_32 },
    { 0x9016b71c, SUC_lexicon_tags_37 },
    { 0x80d6b71c, SUC_lexicon_tags_0 },
    { 0xcb16b71f, SUC_lexicon_tags_25 },
    { 0xf616b720, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d16b724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa16b726, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1716b728, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f6b72e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2956b730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82d6b732, SUC_lexicon_tags_17 },
    { 0x4b96b732, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1156b742, SUC_lexicon_tags_8 },
    { 0xc76b743, SUC_lexicon_tags_15 },
    { 0xecd6b742, SUC_lexicon_tags_38 },
    { 0xb96b744, SUC_lexicon_tags_12 },
    { 0x9b76b742, SUC_lexicon_tags_15 },
    { 0x7cd6b747, SUC_lexicon_tags_25 },
    { 0x8896b748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe556b74a, SUC_lexicon_tags_37 },
    { 0x8836b74b, SUC_lexicon_tags_2 },
    { 0x65b6b74c, SUC_lexicon_tags_205 },
    { 0x2a56b74c, SUC_lexicon_tags_1373 },
    { 0x5a96b74d, SUC_lexicon_tags_47 },
    { 0xb1f6b74f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd136b753, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb6b757, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1076b759, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5d6b75b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d6b75e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd196b760, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf16b763, SUC_lexicon_tags_37 },
    { 0x3116b763, SUC_lexicon_tags_2 },
    { 0x8936b765, SUC_lexicon_tags_37 },
    { 0xc9d6b765, SUC_lexicon_tags_7 },
    { 0x3556b767, SUC_lexicon_tags_33 },
    { 0x8ef6b768, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3616b76b, SUC_lexicon_tags_29 },
    { 0x6616b76c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd136b771, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x4796b774, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b56b777, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa496b77c, SUC_lexicon_tags_47 },
    { 0xa3f6b77c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d16b780, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3936b785, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f6b78d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa256b798, SUC_lexicon_tags_39 },
    { 0x6e76b798, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f56b79e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd6b7a6, SUC_lexicon_tags_17 },
    { 0x8fd6b7a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f16b7ae, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f6b7b2, SUC_lexicon_tags_15 },
    { 0x7176b7b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf356b7b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8216b7bc, SUC_lexicon_tags_874 },
    { 0, NULL },
    { 0x8436b7be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec76b7c2, SUC_lexicon_tags_26 },
    { 0x88d6b7c3, SUC_lexicon_tags_1 },
    { 0xed6b7c2, SUC_lexicon_tags_25 },
    { 0xed76b7c5, SUC_lexicon_tags_2 },
    { 0xd876b7c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6176b7c9, SUC_lexicon_tags_28 },
    { 0xed96b7c9, SUC_lexicon_tags_15 },
    { 0x4b96b7cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3056b7cf, SUC_lexicon_tags_11 },
    { 0xeeb6b7cf, SUC_lexicon_tags_17 },
    { 0xb896b7d1, SUC_lexicon_tags_21 },
    { 0xabd6b7d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b6b7d5, SUC_lexicon_tags_7 },
    { 0x5fb6b7d5, SUC_lexicon_tags_17 },
    { 0xa076b7d7, SUC_lexicon_tags_17 },
    { 0x5f6b7d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3fb6b7da, SUC_lexicon_tags_15 },
    { 0x8b56b7db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xadb6b7dd, SUC_lexicon_tags_8 },
    { 0xd776b7dd, SUC_lexicon_tags_20 },
    { 0xf236b7dd, SUC_lexicon_tags_20 },
    { 0x9016b7e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf6b7ea, SUC_lexicon_tags_25 },
    { 0x8276b7eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x29b6b7ed, SUC_lexicon_tags_36 },
    { 0x20f6b7ee, SUC_lexicon_tags_12 },
    { 0xe0d6b7ed, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xf296b7f1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6ed6b7f3, SUC_lexicon_tags_7 },
    { 0x90b6b7f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef6b7f7, SUC_lexicon_tags_37 },
    { 0xea56b7f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1996b7fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f36b7fe, SUC_lexicon_tags_27 },
    { 0x9a76b7ff, SUC_lexicon_tags_37 },
    { 0xf476b800, SUC_lexicon_tags_23 },
    { 0x2bf6b801, SUC_lexicon_tags_82 },
    { 0xa3b6b7ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5776b805, SUC_lexicon_tags_25 },
    { 0x2f6b806, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x5c96b808, SUC_lexicon_tags_23 },
    { 0x5d76b809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf56b811, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3756b813, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa1b6b815, SUC_lexicon_tags_37 },
    { 0xf416b815, SUC_lexicon_tags_15 },
    { 0x7556b815, SUC_lexicon_tags_19 },
    { 0xf4f6b818, SUC_lexicon_tags_37 },
    { 0x40b6b818, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3256b81b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8116b81f, SUC_lexicon_tags_27 },
    { 0x87b6b81f, SUC_lexicon_tags_12 },
    { 0x15d6b81f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d6b824, SUC_lexicon_tags_8 },
    { 0xd7d6b825, SUC_lexicon_tags_10 },
    { 0xe76b824, SUC_lexicon_tags_29 },
    { 0x336b824, SUC_lexicon_tags_2 },
    { 0x4656b828, SUC_lexicon_tags_14 },
    { 0xedb6b829, SUC_lexicon_tags_84 },
    { 0x7536b827, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ad6b82c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f56b830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac76b833, SUC_lexicon_tags_37 },
    { 0xb1d6b834, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2916b836, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa3f6b838, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9616b83a, SUC_lexicon_tags_79 },
    { 0x9636b83b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b6b83d, SUC_lexicon_tags_19 },
    { 0x64f6b83d, SUC_lexicon_tags_37 },
    { 0x6fb6b83e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf76b844, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6576b846, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e56b84e, SUC_lexicon_tags_30 },
    { 0x9ed6b84f, SUC_lexicon_tags_29 },
    { 0xcef6b850, SUC_lexicon_tags_29 },
    { 0xb556b84e, SUC_lexicon_tags_8 },
    { 0x7436b852, SUC_lexicon_tags_141 },
    { 0xb5b6b853, SUC_lexicon_tags_68 },
    { 0x6b6b854, SUC_lexicon_tags_11 },
    { 0x196b850, SUC_lexicon_tags_2 },
    { 0xf336b856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74d6b858, SUC_lexicon_tags_0 },
    { 0xd676b859, SUC_lexicon_tags_1 },
    { 0xb8d6b85a, SUC_lexicon_tags_12 },
    { 0x2456b85a, SUC_lexicon_tags_25 },
    { 0xce16b85c, SUC_lexicon_tags_30 },
    { 0x3676b858, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176b861, SUC_lexicon_tags_17 },
    { 0x98f6b862, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc816b867, SUC_lexicon_tags_6 },
    { 0x6c56b867, SUC_lexicon_tags_1 },
    { 0xef6b869, SUC_lexicon_tags_19 },
    { 0xdcf6b86a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4976b871, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d6b874, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1d36b876, SUC_lexicon_tags_57 },
    { 0xb976b877, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b6b87f, SUC_lexicon_tags_1 },
    { 0xd496b87f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2fb6b882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb96b885, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd76b88a, SUC_lexicon_tags_3 },
    { 0xdb96b88a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6b894, SUC_lexicon_tags_30 },
    { 0x5eb6b894, SUC_lexicon_tags_16 },
    { 0x7296b896, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d6b89d, SUC_lexicon_tags_7 },
    { 0xb596b89d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf6b8a3, SUC_lexicon_tags_17 },
    { 0xa9b6b8a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7416b8a7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb6b8b5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf076b8b9, SUC_lexicon_tags_19 },
    { 0x3456b8ba, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d6b8be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd76b8c0, SUC_lexicon_tags_11 },
    { 0x1776b8c0, SUC_lexicon_tags_45 },
    { 0x1696b8c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4436b8c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d6b8ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa756b8d1, SUC_lexicon_tags_25 },
    { 0xf736b8d2, SUC_lexicon_tags_37 },
    { 0x3996b8d3, SUC_lexicon_tags_1 },
    { 0xd296b8d2, SUC_lexicon_tags_84 },
    { 0x9036b8d5, SUC_lexicon_tags_60 },
    { 0xe8d6b8d6, SUC_lexicon_tags_37 },
    { 0xe056b8d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1596b8d9, SUC_lexicon_tags_14 },
    { 0x6056b8da, SUC_lexicon_tags_47 },
    { 0xaf16b8da, SUC_lexicon_tags_1 },
    { 0xf7b6b8dc, SUC_lexicon_tags_16 },
    { 0xf9d6b8dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2396b8df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1876b8e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7576b8e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9176b8e6, SUC_lexicon_tags_12 },
    { 0xa5d6b8e6, SUC_lexicon_tags_60 },
    { 0xa116b8e8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xbd76b8ea, SUC_lexicon_tags_7 },
    { 0x8d36b8ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b6b8ef, SUC_lexicon_tags_14 },
    { 0x59b6b8f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb656b8f3, SUC_lexicon_tags_231 },
    { 0xe76b8f4, SUC_lexicon_tags_17 },
    { 0x3576b8f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8576b8f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6b8fb, SUC_lexicon_tags_65 },
    { 0xc016b8fc, SUC_lexicon_tags_37 },
    { 0xf676b8fc, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xfd56b8ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf56b902, SUC_lexicon_tags_49 },
    { 0xad56b903, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa536b905, SUC_lexicon_tags_19 },
    { 0x3bd6b905, SUC_lexicon_tags_12 },
    { 0xd8f6b907, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d6b90a, SUC_lexicon_tags_37 },
    { 0xd096b90b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb516b90f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7816b912, SUC_lexicon_tags_17 },
    { 0x7f96b913, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7376b91a, SUC_lexicon_tags_55 },
    { 0x8e56b91b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe776b91d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8496b921, SUC_lexicon_tags_2 },
    { 0x6536b922, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3076b927, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d56b929, SUC_lexicon_tags_11 },
    { 0x1d96b929, SUC_lexicon_tags_16 },
    { 0x9356b92b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2096b930, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b6b935, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21b6b937, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c16b93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d16b943, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc916b945, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b6b948, SUC_lexicon_tags_11 },
    { 0x7576b948, SUC_lexicon_tags_127 },
    { 0xe0d6b94a, SUC_lexicon_tags_85 },
    { 0x2a96b94b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2796b94e, SUC_lexicon_tags_2 },
    { 0x5a36b94f, SUC_lexicon_tags_79 },
    { 0xa2d6b94e, SUC_lexicon_tags_11 },
    { 0x2876b951, SUC_lexicon_tags_47 },
    { 0xb6d6b951, SUC_lexicon_tags_17 },
    { 0xb8f6b952, SUC_lexicon_tags_23 },
    { 0x9f96b952, SUC_lexicon_tags_32 },
    { 0x5fb6b955, SUC_lexicon_tags_12 },
    { 0x3776b956, SUC_lexicon_tags_78 },
    { 0x46d6b956, SUC_lexicon_tags_37 },
    { 0x3b16b957, SUC_lexicon_tags_15 },
    { 0xd9f6b956, SUC_lexicon_tags_0 },
    { 0x31b6b958, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3456b95c, SUC_lexicon_tags_25 },
    { 0xb6f6b95d, SUC_lexicon_tags_25 },
    { 0x9ed6b95e, SUC_lexicon_tags_26 },
    { 0x8d6b95c, SUC_lexicon_tags_17 },
    { 0x9916b95d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf6b967, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b6b96a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7816b96d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f6b970, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b96b974, SUC_lexicon_tags_24 },
    { 0xc936b975, SUC_lexicon_tags_27 },
    { 0x8076b976, SUC_lexicon_tags_17 },
    { 0xa6d6b977, SUC_lexicon_tags_2 },
    { 0xadd6b978, SUC_lexicon_tags_37 },
    { 0xb3b6b979, SUC_lexicon_tags_15 },
    { 0x70f6b979, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8996b982, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f6b985, SUC_lexicon_tags_25 },
    { 0xbc56b985, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad6b989, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3f16b98b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db6b992, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5736b994, SUC_lexicon_tags_15 },
    { 0x6176b995, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb296b997, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb6f6b999, SUC_lexicon_tags_33 },
    { 0x8b76b99a, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f6b99e, SUC_lexicon_tags_7 },
    { 0x1956b99f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e56b9a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79b6b9a3, SUC_lexicon_tags_33 },
    { 0xbd36b9a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356b9ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe56b9b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdb6b9b3, SUC_lexicon_tags_43 },
    { 0x2f96b9b3, SUC_lexicon_tags_37 },
    { 0xb156b9b3, SUC_lexicon_tags_6 },
    { 0x9ff6b9b6, SUC_lexicon_tags_49 },
    { 0xf36b9b6, SUC_lexicon_tags_7 },
    { 0xfa96b9b8, SUC_lexicon_tags_11 },
    { 0xf876b9b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10d6b9bb, SUC_lexicon_tags_37 },
    { 0x3476b9bc, SUC_lexicon_tags_38 },
    { 0x36b9bd, SUC_lexicon_tags_65 },
    { 0x64f6b9be, SUC_lexicon_tags_37 },
    { 0xd176b9bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe776b9c1, SUC_lexicon_tags_15 },
    { 0x9b76b9c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c16b9c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b6b9ca, SUC_lexicon_tags_32 },
    { 0xd796b9cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf96b9cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc36b9cf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d6b9d7, SUC_lexicon_tags_21 },
    { 0xf4f6b9d8, SUC_lexicon_tags_25 },
    { 0xe216b9d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5856b9db, SUC_lexicon_tags_11 },
    { 0x9ed6b9dc, SUC_lexicon_tags_1 },
    { 0xbeb6b9dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1976b9e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe116b9e3, SUC_lexicon_tags_23 },
    { 0x5d16b9e4, SUC_lexicon_tags_15 },
    { 0xd256b9e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2576b9f1, SUC_lexicon_tags_17 },
    { 0x1f96b9f2, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xa1f6b9f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c76b9f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb76b9fa, SUC_lexicon_tags_17 },
    { 0x86d6b9fb, SUC_lexicon_tags_37 },
    { 0xad16b9fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c36ba00, SUC_lexicon_tags_37 },
    { 0x64b6ba01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5296ba04, SUC_lexicon_tags_424 },
    { 0x9276ba05, SUC_lexicon_tags_14 },
    { 0xf636ba06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcf96ba08, SUC_lexicon_tags_1 },
    { 0xc7f6ba08, SUC_lexicon_tags_11 },
    { 0x5556ba0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69b6ba0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f6ba11, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5d76ba13, SUC_lexicon_tags_69 },
    { 0x4ff6ba14, SUC_lexicon_tags_16 },
    { 0x9996ba15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b6ba1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f6ba20, SUC_lexicon_tags_0 },
    { 0xf1b6ba21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x776ba23, SUC_lexicon_tags_43 },
    { 0xef96ba23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f36ba27, SUC_lexicon_tags_17 },
    { 0xaf6ba28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6ba2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb6ba30, SUC_lexicon_tags_15 },
    { 0x5736ba31, SUC_lexicon_tags_25 },
    { 0xf216ba30, SUC_lexicon_tags_8 },
    { 0x7a76ba32, SUC_lexicon_tags_57 },
    { 0x57b6ba32, SUC_lexicon_tags_22 },
    { 0x5b36ba30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b6ba38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb796ba3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde76ba3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc76ba41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68b6ba43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd16ba46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed6ba4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23d6ba4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac36ba51, SUC_lexicon_tags_11 },
    { 0xba96ba52, SUC_lexicon_tags_17 },
    { 0xd916ba52, SUC_lexicon_tags_11 },
    { 0xed56ba54, SUC_lexicon_tags_11 },
    { 0xe876ba55, SUC_lexicon_tags_18 },
    { 0x2476ba51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d76ba5b, SUC_lexicon_tags_37 },
    { 0x4c96ba5b, SUC_lexicon_tags_32 },
    { 0x15b6ba5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56ba60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b6ba63, SUC_lexicon_tags_12 },
    { 0x1276ba64, SUC_lexicon_tags_32 },
    { 0x9f6ba65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb6ba68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5136ba6c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b6ba72, SUC_lexicon_tags_14 },
    { 0x4eb6ba72, SUC_lexicon_tags_63 },
    { 0x61f6ba74, SUC_lexicon_tags_59 },
    { 0x7f96ba74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa456ba77, SUC_lexicon_tags_1 },
    { 0x6376ba78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb6ba80, SUC_lexicon_tags_49 },
    { 0x2b6ba81, SUC_lexicon_tags_7 },
    { 0x29f6ba82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2896ba85, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b6ba89, SUC_lexicon_tags_43 },
    { 0xced6ba8a, SUC_lexicon_tags_14 },
    { 0x42d6ba8b, SUC_lexicon_tags_33 },
    { 0x1f56ba8b, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x18f6ba8e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7476ba91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e16ba93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9356ba99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd936ba9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa76ba9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd096baa1, SUC_lexicon_tags_30 },
    { 0xf636baa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6736baa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff6baa9, SUC_lexicon_tags_25 },
    { 0x4056baaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd6baae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5776bab3, SUC_lexicon_tags_65 },
    { 0x8556bab3, SUC_lexicon_tags_27 },
    { 0xb796bab5, SUC_lexicon_tags_15 },
    { 0xd656bab6, SUC_lexicon_tags_17 },
    { 0x75d6bab3, SUC_lexicon_tags_28 },
    { 0xda76bab8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd6babb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8156babe, SUC_lexicon_tags_4 },
    { 0x6796babe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7816bac2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa756bac4, SUC_lexicon_tags_17 },
    { 0x8fb6bac5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c56bac8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b6bacc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec76bace, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8856bad0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc016bad4, SUC_lexicon_tags_19 },
    { 0xff56bad5, SUC_lexicon_tags_259 },
    { 0xabd6bad5, SUC_lexicon_tags_17 },
    { 0xc416bad6, SUC_lexicon_tags_8 },
    { 0xfdd6bad6, SUC_lexicon_tags_15 },
    { 0x55d6bad9, SUC_lexicon_tags_11 },
    { 0xdff6bada, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd876badd, SUC_lexicon_tags_1 },
    { 0xebb6bade, SUC_lexicon_tags_65 },
    { 0x4cd6bade, SUC_lexicon_tags_17 },
    { 0xfdb6bae0, SUC_lexicon_tags_7 },
    { 0x9616bae0, SUC_lexicon_tags_8 },
    { 0xa576bae2, SUC_lexicon_tags_19 },
    { 0x8bd6badf, SUC_lexicon_tags_37 },
    { 0x9296bae1, SUC_lexicon_tags_19 },
    { 0x40f6bade, SUC_lexicon_tags_27 },
    { 0xa296bae6, SUC_lexicon_tags_87 },
    { 0x4176bae7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa896baea, SUC_lexicon_tags_23 },
    { 0x7a76baea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a76baed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7156baf2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b6baf8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d76bafe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xff96bb01, SUC_lexicon_tags_198 },
    { 0xc6b6bb02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x33d6bb04, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b6bb0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xea16bb0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8736bb13, SUC_lexicon_tags_14 },
    { 0xb5f6bb13, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6bb17, SUC_lexicon_tags_46 },
    { 0x49b6bb17, SUC_lexicon_tags_55 },
    { 0xfc56bb19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3176bb1d, SUC_lexicon_tags_7 },
    { 0xc016bb1e, SUC_lexicon_tags_1 },
    { 0x29b6bb1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d6bb23, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb56bb28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76bb2c, SUC_lexicon_tags_165 },
    { 0x82d6bb2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc536bb37, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb6bb3b, SUC_lexicon_tags_23 },
    { 0x63f6bb3c, SUC_lexicon_tags_7 },
    { 0x8636bb3c, SUC_lexicon_tags_10 },
    { 0x3556bb3e, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x556bb40, SUC_lexicon_tags_11 },
    { 0x9d6bb41, SUC_lexicon_tags_38 },
    { 0x4876bb42, SUC_lexicon_tags_27 },
    { 0x4996bb41, SUC_lexicon_tags_0 },
    { 0x67b6bb41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6bb49, SUC_lexicon_tags_27 },
    { 0x2176bb4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b6bb53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2196bb58, SUC_lexicon_tags_1 },
    { 0x1f16bb58, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d6bb65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5936bb69, SUC_lexicon_tags_17 },
    { 0x5636bb6a, SUC_lexicon_tags_11 },
    { 0xb3f6bb6b, SUC_lexicon_tags_201 },
    { 0x2236bb6c, SUC_lexicon_tags_1 },
    { 0x7876bb6d, SUC_lexicon_tags_23 },
    { 0xd416bb6e, SUC_lexicon_tags_25 },
    { 0x70d6bb6e, SUC_lexicon_tags_45 },
    { 0x8c36bb6f, SUC_lexicon_tags_9 },
    { 0xfeb6bb71, SUC_lexicon_tags_55 },
    { 0x7756bb72, SUC_lexicon_tags_4 },
    { 0x76f6bb6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f6bb76, SUC_lexicon_tags_37 },
    { 0x10b6bb77, SUC_lexicon_tags_70 },
    { 0xe736bb78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad6bb7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd36bb81, SUC_lexicon_tags_11 },
    { 0x3756bb82, SUC_lexicon_tags_32 },
    { 0xb056bb83, SUC_lexicon_tags_47 },
    { 0xebd6bb82, SUC_lexicon_tags_25 },
    { 0x956bb84, SUC_lexicon_tags_92 },
    { 0xc9b6bb83, SUC_lexicon_tags_25 },
    { 0x11d6bb86, SUC_lexicon_tags_10 },
    { 0x8176bb83, SUC_lexicon_tags_8 },
    { 0xc7f6bb89, SUC_lexicon_tags_15 },
    { 0xc0b6bb87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a96bb93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa216bb98, SUC_lexicon_tags_1 },
    { 0xd696bb98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd76bb9c, SUC_lexicon_tags_49 },
    { 0x7ef6bb9d, SUC_lexicon_tags_46 },
    { 0x3e56bb9e, SUC_lexicon_tags_54 },
    { 0x21f6bb9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc836bba1, SUC_lexicon_tags_11 },
    { 0x1cb6bba2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe36bba6, SUC_lexicon_tags_55 },
    { 0x2456bba7, SUC_lexicon_tags_38 },
    { 0xd6bba6, SUC_lexicon_tags_17 },
    { 0xc176bba9, SUC_lexicon_tags_7 },
    { 0x3236bba8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa076bbb2, SUC_lexicon_tags_2 },
    { 0xc5d6bbb3, SUC_lexicon_tags_11 },
    { 0x64b6bbb3, SUC_lexicon_tags_37 },
    { 0x6056bbb3, SUC_lexicon_tags_33 },
    { 0x2c16bbb5, SUC_lexicon_tags_35 },
    { 0x7416bbb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb6bbbb, SUC_lexicon_tags_15 },
    { 0xb7d6bbbc, SUC_lexicon_tags_7 },
    { 0x4bf6bbbd, SUC_lexicon_tags_10 },
    { 0x77d6bbbe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b6bbc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfb96bbc5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdbb6bbc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5976bbcf, SUC_lexicon_tags_25 },
    { 0xe6d6bbcf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfd76bbd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b6bbd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8676bbda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8236bbdc, SUC_lexicon_tags_66 },
    { 0x4656bbdd, SUC_lexicon_tags_37 },
    { 0x2396bbde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d6bbe1, SUC_lexicon_tags_57 },
    { 0x88d6bbe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef6bbe6, SUC_lexicon_tags_2 },
    { 0xd16bbe7, SUC_lexicon_tags_7 },
    { 0x12f6bbe8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d6bbf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1016bbf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb816bbfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f6bbff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba76bc03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae96bc07, SUC_lexicon_tags_7 },
    { 0x76b6bc08, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab96bc0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb216bc0f, SUC_lexicon_tags_25 },
    { 0x2296bc0f, SUC_lexicon_tags_37 },
    { 0xd236bc11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2656bc16, SUC_lexicon_tags_37 },
    { 0xd736bc16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10d6bc19, SUC_lexicon_tags_43 },
    { 0x73d6bc1a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf316bc1d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4716bc23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7476bc2a, SUC_lexicon_tags_12 },
    { 0xcc16bc2b, SUC_lexicon_tags_15 },
    { 0xda96bc2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4b6bc2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb476bc30, SUC_lexicon_tags_47 },
    { 0xa276bc31, SUC_lexicon_tags_17 },
    { 0x3f6bc32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd796bc38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3516bc3a, SUC_lexicon_tags_1 },
    { 0xbaf6bc3b, SUC_lexicon_tags_31 },
    { 0x4a56bc3b, SUC_lexicon_tags_29 },
    { 0xe016bc3a, SUC_lexicon_tags_0 },
    { 0x14b6bc3e, SUC_lexicon_tags_71 },
    { 0x5df6bc3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b6bc44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa76bc49, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf6bc4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f6bc56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbd56bc58, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7e76bc5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3716bc5c, SUC_lexicon_tags_15 },
    { 0x5796bc5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x59f6bc5f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3836bc66, SUC_lexicon_tags_25 },
    { 0xf8b6bc67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x736bc6a, SUC_lexicon_tags_23 },
    { 0x136bc6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3656bc70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd356bc73, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b6bc76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed96bc7a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f6bc80, SUC_lexicon_tags_13 },
    { 0xd0f6bc81, SUC_lexicon_tags_2 },
    { 0x4376bc82, SUC_lexicon_tags_7 },
    { 0x216bc83, SUC_lexicon_tags_25 },
    { 0x3f76bc81, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8e36bc86, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d6bc8a, SUC_lexicon_tags_17 },
    { 0x5336bc8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d6bc8e, SUC_lexicon_tags_29 },
    { 0x4f76bc8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c96bc97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb6bc9c, SUC_lexicon_tags_17 },
    { 0x2e36bc9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b6bca8, SUC_lexicon_tags_87 },
    { 0xf076bca9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9116bcab, SUC_lexicon_tags_46 },
    { 0x2196bcab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x87d6bcae, SUC_lexicon_tags_17 },
    { 0x3b16bcaf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb596bcb5, SUC_lexicon_tags_17 },
    { 0x6c36bcb6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3316bcbc, SUC_lexicon_tags_10 },
    { 0x14f6bcbd, SUC_lexicon_tags_7 },
    { 0x8676bcbd, SUC_lexicon_tags_14 },
    { 0x98b6bcbf, SUC_lexicon_tags_12 },
    { 0x1176bcc0, SUC_lexicon_tags_25 },
    { 0x97d6bcc1, SUC_lexicon_tags_55 },
    { 0xb436bcc0, SUC_lexicon_tags_61 },
    { 0xa836bcc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef96bcc6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde16bcce, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916bcd2, SUC_lexicon_tags_15 },
    { 0xf156bcd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9956bcd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa536bcd7, SUC_lexicon_tags_47 },
    { 0x9436bcd8, SUC_lexicon_tags_2 },
    { 0x6796bcd9, SUC_lexicon_tags_7 },
    { 0x5f56bcd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1216bcde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc036bce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8996bce3, SUC_lexicon_tags_33 },
    { 0x3f76bce4, SUC_lexicon_tags_402 },
    { 0x7c96bce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fd6bce7, SUC_lexicon_tags_23 },
    { 0x4176bce8, SUC_lexicon_tags_7 },
    { 0x6bb6bce9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad6bcec, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf316bcf1, SUC_lexicon_tags_13 },
    { 0x3776bcf2, SUC_lexicon_tags_60 },
    { 0xa316bcf3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa576bcf5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f6bcf9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e56bcfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe276bcfe, SUC_lexicon_tags_40 },
    { 0x66f6bcff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7216bd02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c36bd0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f6bd17, SUC_lexicon_tags_27 },
    { 0xd8b6bd17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b6bd1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6c76bd21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4496bd25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6bd28, SUC_lexicon_tags_17 },
    { 0x2976bd28, SUC_lexicon_tags_7 },
    { 0x7b76bd2a, SUC_lexicon_tags_71 },
    { 0xd736bd29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed6bd2e, SUC_lexicon_tags_27 },
    { 0x9e16bd2e, SUC_lexicon_tags_17 },
    { 0xb016bd30, SUC_lexicon_tags_15 },
    { 0x656bd31, SUC_lexicon_tags_35 },
    { 0x1956bd32, SUC_lexicon_tags_17 },
    { 0xfad6bd33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3756bd35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd036bd39, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a56bd3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf936bd3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b6bd43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa096bd47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3036bd49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x916bd4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36bd55, SUC_lexicon_tags_17 },
    { 0xa276bd55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bd5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2496bd5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b56bd61, SUC_lexicon_tags_7 },
    { 0x6a76bd62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ed6bd64, SUC_lexicon_tags_28 },
    { 0x4a96bd65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf6bd69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd6bd6d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1636bd75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b36bd77, SUC_lexicon_tags_10 },
    { 0xb96bd77, SUC_lexicon_tags_14 },
    { 0x8dd6bd79, SUC_lexicon_tags_63 },
    { 0x4736bd78, SUC_lexicon_tags_79 },
    { 0xbc76bd78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6816bd7d, SUC_lexicon_tags_25 },
    { 0x4f76bd7e, SUC_lexicon_tags_85 },
    { 0xb5f6bd7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5976bd81, SUC_lexicon_tags_15 },
    { 0x5fd6bd81, SUC_lexicon_tags_10 },
    { 0xc516bd82, SUC_lexicon_tags_37 },
    { 0xea36bd84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2096bd88, SUC_lexicon_tags_33 },
    { 0x7556bd89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbff6bd8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a16bd90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f6bd93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a16bd98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe36bd9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8316bd9f, SUC_lexicon_tags_1 },
    { 0xde36bda0, SUC_lexicon_tags_23 },
    { 0xd136bda1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b6bda9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d56bdab, SUC_lexicon_tags_7 },
    { 0x4136bdac, SUC_lexicon_tags_25 },
    { 0xd8b6bdac, SUC_lexicon_tags_8 },
    { 0x53d6bdae, SUC_lexicon_tags_97 },
    { 0xaa76bdac, SUC_lexicon_tags_27 },
    { 0xc16bdb0, SUC_lexicon_tags_79 },
    { 0x3d6bdb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b6bdb5, SUC_lexicon_tags_2 },
    { 0x36bdb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad6bdbb, SUC_lexicon_tags_25 },
    { 0x9a76bdbb, SUC_lexicon_tags_7 },
    { 0x856bdbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b36bdc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9376bdc2, SUC_lexicon_tags_25 },
    { 0xb136bdc2, SUC_lexicon_tags_25 },
    { 0x8fb6bdc3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x79f6bdc6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f6bdc9, SUC_lexicon_tags_1 },
    { 0x6356bdca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a16bdd2, SUC_lexicon_tags_17 },
    { 0xa956bdd2, SUC_lexicon_tags_35 },
    { 0xb456bdd4, SUC_lexicon_tags_7 },
    { 0xf856bdd4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b6bdd9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a56bddc, SUC_lexicon_tags_37 },
    { 0xf596bddd, SUC_lexicon_tags_7 },
    { 0xea56bdde, SUC_lexicon_tags_37 },
    { 0x4656bdde, SUC_lexicon_tags_60 },
    { 0xf776bddf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc36bde4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c36bde6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8916bdec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d6bdf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab6bdf8, SUC_lexicon_tags_10 },
    { 0xa2b6bdf9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f76bdfd, SUC_lexicon_tags_26 },
    { 0xed36bdfe, SUC_lexicon_tags_19 },
    { 0x6cf6bdff, SUC_lexicon_tags_37 },
    { 0x7cf6be00, SUC_lexicon_tags_85 },
    { 0x7e76bdfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b6be04, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d6be0f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9616be11, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8356be15, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x41f6be17, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xa296be19, SUC_lexicon_tags_2 },
    { 0x9f36be1a, SUC_lexicon_tags_47 },
    { 0x3b76be1a, SUC_lexicon_tags_53 },
    { 0x1876be1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec96be20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef6be28, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xe576be2a, SUC_lexicon_tags_37 },
    { 0x5f76be2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc4d6be2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3496be30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8796be32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d6be36, SUC_lexicon_tags_69 },
    { 0x3436be37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb196be3c, SUC_lexicon_tags_133 },
    { 0x7816be3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3156be40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfcf6be42, SUC_lexicon_tags_8 },
    { 0x87f6be43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb6be4a, SUC_lexicon_tags_1 },
    { 0x5f6be4b, SUC_lexicon_tags_32 },
    { 0x47d6be4c, SUC_lexicon_tags_11 },
    { 0xb256be4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed6be50, SUC_lexicon_tags_4 },
    { 0xb0f6be51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f6be54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f6be57, SUC_lexicon_tags_49 },
    { 0xfcd6be57, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe5f6be5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b6be5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb776be5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d6be62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x856be64, SUC_lexicon_tags_45 },
    { 0xce76be64, SUC_lexicon_tags_2 },
    { 0x6796be65, SUC_lexicon_tags_28 },
    { 0xf996be67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6f6be69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d6be6c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb6be6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9a76be71, SUC_lexicon_tags_4 },
    { 0x8596be72, SUC_lexicon_tags_37 },
    { 0x4b16be73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c36be75, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa196be7b, SUC_lexicon_tags_27 },
    { 0xcdf6be7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb6be83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976be8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2156be8e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5fd6be90, SUC_lexicon_tags_13 },
    { 0x2f36be91, SUC_lexicon_tags_2 },
    { 0x656be92, SUC_lexicon_tags_25 },
    { 0x2c96be92, SUC_lexicon_tags_17 },
    { 0xf6d6be94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d6be98, SUC_lexicon_tags_37 },
    { 0xe916be98, SUC_lexicon_tags_11 },
    { 0x1756be9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd416be9d, SUC_lexicon_tags_13 },
    { 0x1976be9e, SUC_lexicon_tags_31 },
    { 0xe436be9d, SUC_lexicon_tags_17 },
    { 0xc776bea0, SUC_lexicon_tags_38 },
    { 0x876bea0, SUC_lexicon_tags_63 },
    { 0x8876bea2, SUC_lexicon_tags_37 },
    { 0x43b6bea3, SUC_lexicon_tags_0 },
    { 0xff96bea2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5536bea6, SUC_lexicon_tags_37 },
    { 0x2b96bea6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1456beaa, SUC_lexicon_tags_2 },
    { 0xe76beab, SUC_lexicon_tags_17 },
    { 0x3536beac, SUC_lexicon_tags_23 },
    { 0x14f6beac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x48f6beaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf696beb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b6beb9, SUC_lexicon_tags_16 },
    { 0x5636beb9, SUC_lexicon_tags_17 },
    { 0x2856bebb, SUC_lexicon_tags_17 },
    { 0xb6bebb, SUC_lexicon_tags_47 },
    { 0x3fb6bebb, SUC_lexicon_tags_318 },
    { 0x9556bebc, SUC_lexicon_tags_26 },
    { 0x2a96bebb, SUC_lexicon_tags_1 },
    { 0x5bb6beb9, SUC_lexicon_tags_31 },
    { 0xa516bec1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe816bec3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7d6bec5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f6bec8, SUC_lexicon_tags_21 },
    { 0xf256bec9, SUC_lexicon_tags_43 },
    { 0xb716beca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1816becd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3936bed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f6bed3, SUC_lexicon_tags_23 },
    { 0x26b6bed4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd3b6bed6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc956beda, SUC_lexicon_tags_37 },
    { 0x7196beda, SUC_lexicon_tags_39 },
    { 0x6d56bedc, SUC_lexicon_tags_25 },
    { 0x7776bedc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6bee1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d6bee4, SUC_lexicon_tags_37 },
    { 0xd336bee4, SUC_lexicon_tags_29 },
    { 0xd856bee6, SUC_lexicon_tags_8 },
    { 0x4cd6bee6, SUC_lexicon_tags_60 },
    { 0x3df6bee8, SUC_lexicon_tags_33 },
    { 0xd2f6bee7, SUC_lexicon_tags_12 },
    { 0xe856beea, SUC_lexicon_tags_1 },
    { 0x17b6beea, SUC_lexicon_tags_45 },
    { 0x9c16beeb, SUC_lexicon_tags_7 },
    { 0xa9f6beed, SUC_lexicon_tags_38 },
    { 0x13d6beed, SUC_lexicon_tags_15 },
    { 0x1216beef, SUC_lexicon_tags_0 },
    { 0xc016bef0, SUC_lexicon_tags_11 },
    { 0x6876bef1, SUC_lexicon_tags_59 },
    { 0xb236beef, SUC_lexicon_tags_1481 },
    { 0xd596bef3, SUC_lexicon_tags_28 },
    { 0xd76bef4, SUC_lexicon_tags_7 },
    { 0xa136bef5, SUC_lexicon_tags_39 },
    { 0xfbf6bef6, SUC_lexicon_tags_65 },
    { 0xe36bef7, SUC_lexicon_tags_6 },
    { 0x5b76bef8, SUC_lexicon_tags_17 },
    { 0x2856bef7, SUC_lexicon_tags_34 },
    { 0xc456befa, SUC_lexicon_tags_63 },
    { 0x2316beef, SUC_lexicon_tags_17 },
    { 0x6b16beed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df6bf00, SUC_lexicon_tags_11 },
    { 0x3f56bf01, SUC_lexicon_tags_30 },
    { 0x9b6bf02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5336bf09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2156bf0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f16bf0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b6bf11, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa516bf15, SUC_lexicon_tags_7 },
    { 0xedb6bf16, SUC_lexicon_tags_32 },
    { 0xce56bf16, SUC_lexicon_tags_25 },
    { 0xc496bf15, SUC_lexicon_tags_47 },
    { 0x4f76bf19, SUC_lexicon_tags_11 },
    { 0x9416bf1a, SUC_lexicon_tags_15 },
    { 0x3676bf15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd6bf1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf976bf21, SUC_lexicon_tags_0 },
    { 0x6bd6bf22, SUC_lexicon_tags_37 },
    { 0x1656bf22, SUC_lexicon_tags_26 },
    { 0x2fb6bf24, SUC_lexicon_tags_47 },
    { 0xb636bf24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f6bf2b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4616bf31, SUC_lexicon_tags_43 },
    { 0x6f6bf32, SUC_lexicon_tags_11 },
    { 0x5456bf33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7616bf36, SUC_lexicon_tags_17 },
    { 0x396bf37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b6bf3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff6bf4d, SUC_lexicon_tags_37 },
    { 0x3156bf4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d56bf50, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf56bf54, SUC_lexicon_tags_11 },
    { 0xdbd6bf54, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d96bf5b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8836bf5f, SUC_lexicon_tags_38 },
    { 0xd7d6bf60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xded6bf63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5936bf66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4696bf6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa136bf6d, SUC_lexicon_tags_11 },
    { 0x8f96bf6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3996bf74, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf536bf79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x24b6bf7b, SUC_lexicon_tags_60 },
    { 0xa5d6bf7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b6bf82, SUC_lexicon_tags_11 },
    { 0x9ad6bf82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1416bf86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31f6bf88, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b96bf8b, SUC_lexicon_tags_12 },
    { 0xdf36bf8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x15b6bf8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x296bf90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf716bf94, SUC_lexicon_tags_7 },
    { 0x1a76bf95, SUC_lexicon_tags_25 },
    { 0x7e36bf94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b6bf9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd536bf9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd596bf9f, SUC_lexicon_tags_7 },
    { 0xb776bfa0, SUC_lexicon_tags_12 },
    { 0x1196bfa0, SUC_lexicon_tags_26 },
    { 0xc6f6bfa0, SUC_lexicon_tags_32 },
    { 0xf396bf9f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe156bfa6, SUC_lexicon_tags_301 },
    { 0x9f36bfa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf96bfaa, SUC_lexicon_tags_69 },
    { 0x35d6bfaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaa16bfad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac36bfb3, SUC_lexicon_tags_17 },
    { 0x5896bfb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d6bfb7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa356bfb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ef6bfbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b6bfbe, SUC_lexicon_tags_27 },
    { 0x2f76bfbe, SUC_lexicon_tags_10 },
    { 0x5e36bfbf, SUC_lexicon_tags_4 },
    { 0x47b6bfc1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56bfc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d6bfc7, SUC_lexicon_tags_1 },
    { 0x9056bfc8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d6bfcd, SUC_lexicon_tags_37 },
    { 0x5db6bfce, SUC_lexicon_tags_7 },
    { 0x85d6bfcf, SUC_lexicon_tags_108 },
    { 0x16f6bfd0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf6bfd4, SUC_lexicon_tags_134 },
    { 0x62d6bfd4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5936bfdc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1336bfdf, SUC_lexicon_tags_28 },
    { 0x4d96bfe0, SUC_lexicon_tags_15 },
    { 0xdb56bfe0, SUC_lexicon_tags_118 },
    { 0x91d6bfe2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf836bfe4, SUC_lexicon_tags_2 },
    { 0x6256bfe5, SUC_lexicon_tags_25 },
    { 0xf6b6bfe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8816bfed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b6bff1, SUC_lexicon_tags_38 },
    { 0x396bff2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e56bff6, SUC_lexicon_tags_25 },
    { 0xcdf6bff7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb6bffb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5796c002, SUC_lexicon_tags_11 },
    { 0xc396c003, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6c00f, SUC_lexicon_tags_7 },
    { 0xeb16c010, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9096c012, SUC_lexicon_tags_25 },
    { 0x7996c013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5396c015, SUC_lexicon_tags_38 },
    { 0x5956c015, SUC_lexicon_tags_32 },
    { 0xd5d6c017, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e36c01a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d6c01d, SUC_lexicon_tags_11 },
    { 0xafb6c01e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x936c020, SUC_lexicon_tags_37 },
    { 0xed56c021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6556c024, SUC_lexicon_tags_17 },
    { 0x6e16c025, SUC_lexicon_tags_68 },
    { 0x3596c025, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb6c02c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb696c02e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c56c031, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb976c035, SUC_lexicon_tags_37 },
    { 0x38f6c036, SUC_lexicon_tags_23 },
    { 0x9176c037, SUC_lexicon_tags_1 },
    { 0x2456c038, SUC_lexicon_tags_48 },
    { 0x5a36c039, SUC_lexicon_tags_1 },
    { 0x6bb6c038, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f6c03d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176c045, SUC_lexicon_tags_68 },
    { 0x7e56c046, SUC_lexicon_tags_0 },
    { 0x8b56c046, SUC_lexicon_tags_47 },
    { 0x5c76c048, SUC_lexicon_tags_768 },
    { 0x576c047, SUC_lexicon_tags_23 },
    { 0xf5f6c04a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce56c051, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ab6c053, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a36c055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7496c05e, SUC_lexicon_tags_30 },
    { 0x17d6c05f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd6c062, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7716c065, SUC_lexicon_tags_45 },
    { 0xf4b6c066, SUC_lexicon_tags_7 },
    { 0x1196c067, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff6c071, SUC_lexicon_tags_9 },
    { 0xea16c071, SUC_lexicon_tags_11 },
    { 0xc396c072, SUC_lexicon_tags_48 },
    { 0xd4f6c074, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4956c07a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d6c07d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b36c080, SUC_lexicon_tags_39 },
    { 0x90f6c080, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f16c085, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4f6c087, SUC_lexicon_tags_12 },
    { 0xc4b6c088, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf6c08c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab6c090, SUC_lexicon_tags_27 },
    { 0x9716c090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f16c099, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b76c09c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9056c0a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed6c0a7, SUC_lexicon_tags_1 },
    { 0xed76c0a8, SUC_lexicon_tags_26 },
    { 0x6276c0a8, SUC_lexicon_tags_6 },
    { 0x9ff6c0a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb496c0ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f6c0af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbb16c0b1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c36c0b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f6c0b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef6c0b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe96c0be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f6c0c7, SUC_lexicon_tags_77 },
    { 0x7eb6c0c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x196c0cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf6c0d0, SUC_lexicon_tags_25 },
    { 0x12f6c0d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64f6c0d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8156c0e2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb096c0e4, SUC_lexicon_tags_96 },
    { 0x5f96c0e5, SUC_lexicon_tags_7 },
    { 0xfe36c0e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfdf6c0e8, SUC_lexicon_tags_17 },
    { 0x5376c0e9, SUC_lexicon_tags_38 },
    { 0xca96c0ea, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x2b96c0ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70b6c0ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3916c0f1, SUC_lexicon_tags_68 },
    { 0xe6b6c0f1, SUC_lexicon_tags_30 },
    { 0xc1b6c0f2, SUC_lexicon_tags_11 },
    { 0x516c0f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff96c0f6, SUC_lexicon_tags_577 },
    { 0xa7b6c0f7, SUC_lexicon_tags_2 },
    { 0xfd56c0f7, SUC_lexicon_tags_15 },
    { 0x5356c0f9, SUC_lexicon_tags_26 },
    { 0x5876c0f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d16c0fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x55f6c100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd96c102, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9656c104, SUC_lexicon_tags_2 },
    { 0x5d76c104, SUC_lexicon_tags_57 },
    { 0x5ef6c106, SUC_lexicon_tags_11 },
    { 0x7216c107, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b6c10b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7996c10e, SUC_lexicon_tags_30 },
    { 0x7256c10f, SUC_lexicon_tags_15 },
    { 0x90f6c10f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db6c114, SUC_lexicon_tags_62 },
    { 0x32f6c114, SUC_lexicon_tags_172 },
    { 0x36c116, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe076c11a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9016c120, SUC_lexicon_tags_37 },
    { 0x3e16c121, SUC_lexicon_tags_37 },
    { 0x49f6c122, SUC_lexicon_tags_14 },
    { 0x3456c121, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6c126, SUC_lexicon_tags_7 },
    { 0xd7b6c126, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb076c129, SUC_lexicon_tags_40 },
    { 0xd016c12a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b6c12f, SUC_lexicon_tags_15 },
    { 0x8116c130, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e76c133, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d36c13c, SUC_lexicon_tags_63 },
    { 0x4c76c13d, SUC_lexicon_tags_1 },
    { 0x2116c13e, SUC_lexicon_tags_19 },
    { 0x91b6c13c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92f6c141, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f6c145, SUC_lexicon_tags_17 },
    { 0x1cd6c146, SUC_lexicon_tags_43 },
    { 0x4d6c147, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a96c14a, SUC_lexicon_tags_55 },
    { 0x67f6c14b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1b56c14d, SUC_lexicon_tags_17 },
    { 0x5f36c14d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x39b6c150, SUC_lexicon_tags_2 },
    { 0x9176c150, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55f6c153, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xfc96c155, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc716c158, SUC_lexicon_tags_23 },
    { 0xbbb6c159, SUC_lexicon_tags_39 },
    { 0xcb6c15a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd176c161, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6416c163, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f6c16c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e76c172, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa776c175, SUC_lexicon_tags_12 },
    { 0x9216c176, SUC_lexicon_tags_201 },
    { 0x17d6c177, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3c96c179, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d6c180, SUC_lexicon_tags_33 },
    { 0x2f6c181, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b6c184, SUC_lexicon_tags_87 },
    { 0xb876c185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b6c189, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8216c18b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a96c18d, SUC_lexicon_tags_1 },
    { 0xcc56c18e, SUC_lexicon_tags_7 },
    { 0x89f6c18f, SUC_lexicon_tags_40 },
    { 0x56d6c190, SUC_lexicon_tags_3 },
    { 0x7676c18f, SUC_lexicon_tags_136 },
    { 0x3856c192, SUC_lexicon_tags_1 },
    { 0x5456c193, SUC_lexicon_tags_29 },
    { 0x8d76c194, SUC_lexicon_tags_11 },
    { 0x8cd6c195, SUC_lexicon_tags_32 },
    { 0xcf36c193, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6996c19e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa36c1a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa716c1a4, SUC_lexicon_tags_17 },
    { 0xd8d6c1a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb16c1a8, SUC_lexicon_tags_15 },
    { 0xc16c1a9, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d6c1ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d6c1b1, SUC_lexicon_tags_49 },
    { 0xdd36c1b1, SUC_lexicon_tags_116 },
    { 0xd6b6c1b2, SUC_lexicon_tags_49 },
    { 0xdc36c1b3, SUC_lexicon_tags_23 },
    { 0xddd6c1b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e56c1b8, SUC_lexicon_tags_6 },
    { 0x9776c1b9, SUC_lexicon_tags_35 },
    { 0x6916c1b9, SUC_lexicon_tags_33 },
    { 0x62f6c1b9, SUC_lexicon_tags_23 },
    { 0x17f6c1bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2776c1c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc736c1c6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1536c1cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d6c1d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4696c1dd, SUC_lexicon_tags_25 },
    { 0xe96c1de, SUC_lexicon_tags_25 },
    { 0xe516c1df, SUC_lexicon_tags_38 },
    { 0xbf76c1dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9436c1e3, SUC_lexicon_tags_49 },
    { 0xe076c1e4, SUC_lexicon_tags_14 },
    { 0x39b6c1e4, SUC_lexicon_tags_30 },
    { 0x7396c1e4, SUC_lexicon_tags_11 },
    { 0x41d6c1e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xced6c1e9, SUC_lexicon_tags_42 },
    { 0x5e36c1ea, SUC_lexicon_tags_12 },
    { 0xb96c1eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe716c1ee, SUC_lexicon_tags_68 },
    { 0xb336c1ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x636c1f2, SUC_lexicon_tags_33 },
    { 0x1996c1f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa096c1f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee96c1f9, SUC_lexicon_tags_30 },
    { 0xb8b6c1f9, SUC_lexicon_tags_7 },
    { 0xeb76c1fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f36c1fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6116c1ff, SUC_lexicon_tags_37 },
    { 0xecd6c200, SUC_lexicon_tags_9 },
    { 0x5716c200, SUC_lexicon_tags_11 },
    { 0xce96c202, SUC_lexicon_tags_21 },
    { 0x9e96c1ff, SUC_lexicon_tags_1 },
    { 0x58d6c202, SUC_lexicon_tags_32 },
    { 0x17f6c205, SUC_lexicon_tags_45 },
    { 0x5f16c205, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d36c209, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d6c20c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1d6c20e, SUC_lexicon_tags_7 },
    { 0xb356c20f, SUC_lexicon_tags_17 },
    { 0xcc76c210, SUC_lexicon_tags_6 },
    { 0xc016c211, SUC_lexicon_tags_63 },
    { 0x9a96c212, SUC_lexicon_tags_29 },
    { 0x3016c20f, SUC_lexicon_tags_17 },
    { 0x1016c213, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c219, SUC_lexicon_tags_34 },
    { 0xa976c219, SUC_lexicon_tags_47 },
    { 0xc336c21a, SUC_lexicon_tags_7 },
    { 0x1576c21c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e56c21e, SUC_lexicon_tags_9 },
    { 0xf556c21e, SUC_lexicon_tags_38 },
    { 0x64d6c21e, SUC_lexicon_tags_37 },
    { 0x2596c221, SUC_lexicon_tags_0 },
    { 0x9cb6c222, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d36c227, SUC_lexicon_tags_563 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a76c22c, SUC_lexicon_tags_23 },
    { 0xd96c22d, SUC_lexicon_tags_35 },
    { 0x86d6c22e, SUC_lexicon_tags_79 },
    { 0xaa36c22f, SUC_lexicon_tags_26 },
    { 0xac56c230, SUC_lexicon_tags_15 },
    { 0xeed6c22f, SUC_lexicon_tags_12 },
    { 0x2876c230, SUC_lexicon_tags_33 },
    { 0x2e36c233, SUC_lexicon_tags_38 },
    { 0x8536c22e, SUC_lexicon_tags_39 },
    { 0x7c36c235, SUC_lexicon_tags_25 },
    { 0xd316c232, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6856c23b, SUC_lexicon_tags_54 },
    { 0xa116c23c, SUC_lexicon_tags_17 },
    { 0x5df6c23d, SUC_lexicon_tags_38 },
    { 0x1b36c23c, SUC_lexicon_tags_17 },
    { 0x2bb6c23f, SUC_lexicon_tags_2 },
    { 0x1db6c240, SUC_lexicon_tags_1 },
    { 0x58f6c23e, SUC_lexicon_tags_10 },
    { 0xd96c23b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6c245, SUC_lexicon_tags_15 },
    { 0xf0f6c245, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c16c24b, SUC_lexicon_tags_77 },
    { 0x396c24b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc096c251, SUC_lexicon_tags_12 },
    { 0x416c252, SUC_lexicon_tags_1 },
    { 0x80b6c251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c56c255, SUC_lexicon_tags_19 },
    { 0xdf16c256, SUC_lexicon_tags_1 },
    { 0x2a36c257, SUC_lexicon_tags_0 },
    { 0xbb6c257, SUC_lexicon_tags_23 },
    { 0x1176c259, SUC_lexicon_tags_37 },
    { 0x7776c259, SUC_lexicon_tags_21 },
    { 0xdc56c25b, SUC_lexicon_tags_15 },
    { 0x1a36c25a, SUC_lexicon_tags_25 },
    { 0xa456c25c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb6c260, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d6c265, SUC_lexicon_tags_25 },
    { 0xbdd6c266, SUC_lexicon_tags_38 },
    { 0xac96c265, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe656c26a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d76c26d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7276c274, SUC_lexicon_tags_87 },
    { 0x4bb6c274, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8176c27a, SUC_lexicon_tags_20 },
    { 0xfab6c27a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3516c27f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4776c282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5076c284, SUC_lexicon_tags_49 },
    { 0x9476c285, SUC_lexicon_tags_4 },
    { 0xef56c286, SUC_lexicon_tags_47 },
    { 0xa9d6c285, SUC_lexicon_tags_129 },
    { 0xfd6c284, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6156c28a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d6c28f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf6c293, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6376c298, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5896c29b, SUC_lexicon_tags_30 },
    { 0x4196c29c, SUC_lexicon_tags_7 },
    { 0xd1b6c29d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc476c2a3, SUC_lexicon_tags_14 },
    { 0xf936c2a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab6c2a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8116c2aa, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xe136c2ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef6c2bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b16c2c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86b6c2c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b6c2c9, SUC_lexicon_tags_25 },
    { 0xc3d6c2ca, SUC_lexicon_tags_38 },
    { 0xf136c2cb, SUC_lexicon_tags_10 },
    { 0xcdb6c2ca, SUC_lexicon_tags_27 },
    { 0x56c2cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed6c2d2, SUC_lexicon_tags_15 },
    { 0x2df6c2d3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfa16c2d5, SUC_lexicon_tags_0 },
    { 0xaf76c2d6, SUC_lexicon_tags_12 },
    { 0xfff6c2d5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef6c2da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f6c2e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb6c2e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6c2ec, SUC_lexicon_tags_61 },
    { 0x89d6c2ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf616c2f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e76c2f5, SUC_lexicon_tags_17 },
    { 0x4556c2f6, SUC_lexicon_tags_15 },
    { 0x2e96c2f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3416c2fd, SUC_lexicon_tags_0 },
    { 0xadf6c2fd, SUC_lexicon_tags_15 },
    { 0xb676c2ff, SUC_lexicon_tags_47 },
    { 0x8416c300, SUC_lexicon_tags_7 },
    { 0x6b56c2fd, SUC_lexicon_tags_35 },
    { 0xdbb6c302, SUC_lexicon_tags_37 },
    { 0x84f6c300, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe776c307, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb096c309, SUC_lexicon_tags_14 },
    { 0xabb6c30a, SUC_lexicon_tags_17 },
    { 0xdb96c30b, SUC_lexicon_tags_84 },
    { 0x8a96c30c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7896c30f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60b6c311, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x47b6c313, SUC_lexicon_tags_12 },
    { 0x6976c314, SUC_lexicon_tags_11 },
    { 0xed16c315, SUC_lexicon_tags_554 },
    { 0x3a76c313, SUC_lexicon_tags_32 },
    { 0x4cf6c317, SUC_lexicon_tags_7 },
    { 0x93b6c317, SUC_lexicon_tags_30 },
    { 0xf36c315, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x736c31e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5816c320, SUC_lexicon_tags_0 },
    { 0x9cb6c320, SUC_lexicon_tags_12 },
    { 0xfd76c322, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb6c326, SUC_lexicon_tags_1 },
    { 0xd0b6c327, SUC_lexicon_tags_25 },
    { 0x376c328, SUC_lexicon_tags_44 },
    { 0x6476c328, SUC_lexicon_tags_17 },
    { 0xf36c32a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe956c32d, SUC_lexicon_tags_12 },
    { 0xd9d6c32e, SUC_lexicon_tags_15 },
    { 0x98f6c32f, SUC_lexicon_tags_1 },
    { 0xf276c330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb6c335, SUC_lexicon_tags_12 },
    { 0xb956c336, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa536c338, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f6c33c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x11f6c33e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc496c340, SUC_lexicon_tags_17 },
    { 0x34b6c341, SUC_lexicon_tags_38 },
    { 0xcbd6c341, SUC_lexicon_tags_7 },
    { 0x3556c341, SUC_lexicon_tags_31 },
    { 0x2f6c342, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd6c34a, SUC_lexicon_tags_11 },
    { 0x2016c34b, SUC_lexicon_tags_2 },
    { 0xd96c34b, SUC_lexicon_tags_37 },
    { 0xbed6c34c, SUC_lexicon_tags_1613 },
    { 0xaa16c34e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9d36c350, SUC_lexicon_tags_37 },
    { 0x7d76c351, SUC_lexicon_tags_7 },
    { 0x9236c352, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0x6916c354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3916c357, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df6c35b, SUC_lexicon_tags_31 },
    { 0x8c36c35c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc56c367, SUC_lexicon_tags_1 },
    { 0x4d16c367, SUC_lexicon_tags_0 },
    { 0x26b6c369, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d6c36c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3256c36e, SUC_lexicon_tags_34 },
    { 0xb4f6c36f, SUC_lexicon_tags_49 },
    { 0xc756c370, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa16c373, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c56c376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2036c378, SUC_lexicon_tags_8 },
    { 0x4236c378, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc436c37d, SUC_lexicon_tags_21 },
    { 0x7f6c37e, SUC_lexicon_tags_12 },
    { 0xd736c37f, SUC_lexicon_tags_11 },
    { 0x6b76c37d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf6c383, SUC_lexicon_tags_31 },
    { 0x6c96c383, SUC_lexicon_tags_21 },
    { 0x8ab6c385, SUC_lexicon_tags_33 },
    { 0x3d16c384, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x616c389, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a36c395, SUC_lexicon_tags_45 },
    { 0x7c76c396, SUC_lexicon_tags_12 },
    { 0xb096c395, SUC_lexicon_tags_42 },
    { 0x76b6c398, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fb6c39a, SUC_lexicon_tags_32 },
    { 0xcc76c39b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3f96c39d, SUC_lexicon_tags_6 },
    { 0x42d6c39d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf856c3a1, SUC_lexicon_tags_2 },
    { 0x7236c3a2, SUC_lexicon_tags_19 },
    { 0x61f6c3a3, SUC_lexicon_tags_38 },
    { 0xe016c3a4, SUC_lexicon_tags_17 },
    { 0x7376c3a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d6c3a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4916c3ab, SUC_lexicon_tags_49 },
    { 0xba56c3ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd6c3b0, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xd936c3b2, SUC_lexicon_tags_45 },
    { 0xf4b6c3b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf6c3b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb6c3bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc96c3c0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x69f6c3c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8416c3c5, SUC_lexicon_tags_11 },
    { 0xe796c3c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa576c3c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6c3ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab6c3cf, SUC_lexicon_tags_15 },
    { 0x1776c3cf, SUC_lexicon_tags_29 },
    { 0xbb76c3d1, SUC_lexicon_tags_11 },
    { 0x14d6c3d1, SUC_lexicon_tags_14 },
    { 0xf416c3d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa076c3d8, SUC_lexicon_tags_0 },
    { 0x6ff6c3d9, SUC_lexicon_tags_15 },
    { 0xedf6c3da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd36c3e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1d56c3e4, SUC_lexicon_tags_1 },
    { 0x45b6c3e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde16c3e9, SUC_lexicon_tags_31 },
    { 0x8e36c3ea, SUC_lexicon_tags_11 },
    { 0x4a56c3ea, SUC_lexicon_tags_1 },
    { 0xf136c3ec, SUC_lexicon_tags_37 },
    { 0x42b6c3ed, SUC_lexicon_tags_14 },
    { 0xc236c3ee, SUC_lexicon_tags_46 },
    { 0x99f6c3ef, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d6c3f2, SUC_lexicon_tags_12 },
    { 0x5436c3f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d6c3f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f36c3f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5dd6c3fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b6c3fe, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1ff6c400, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1596c409, SUC_lexicon_tags_7 },
    { 0xf2d6c409, SUC_lexicon_tags_1 },
    { 0x4c76c409, SUC_lexicon_tags_11 },
    { 0xa0f6c409, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf6c40f, SUC_lexicon_tags_11 },
    { 0x21b6c40f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc816c417, SUC_lexicon_tags_19 },
    { 0xdf56c418, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b56c41f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x60d6c421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27d6c423, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc556c439, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77d6c43b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f36c43d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf316c443, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f6c446, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf6c44d, SUC_lexicon_tags_21 },
    { 0x9dd6c44e, SUC_lexicon_tags_10 },
    { 0x41b6c44e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3776c456, SUC_lexicon_tags_12 },
    { 0x9916c457, SUC_lexicon_tags_37 },
    { 0x5a76c458, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe056c45e, SUC_lexicon_tags_47 },
    { 0x5a76c45e, SUC_lexicon_tags_30 },
    { 0x8616c460, SUC_lexicon_tags_63 },
    { 0x1976c45e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2416c463, SUC_lexicon_tags_56 },
    { 0x7b56c464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d6c467, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd6c469, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9fb6c46b, SUC_lexicon_tags_43 },
    { 0x4cd6c46c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65b6c46e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4976c472, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x23b6c474, SUC_lexicon_tags_2 },
    { 0xe4f6c475, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d6c478, SUC_lexicon_tags_57 },
    { 0xcb6c479, SUC_lexicon_tags_25 },
    { 0x4196c479, SUC_lexicon_tags_11 },
    { 0xdd16c47a, SUC_lexicon_tags_7 },
    { 0x8f36c47a, SUC_lexicon_tags_11 },
    { 0xddf6c47c, SUC_lexicon_tags_47 },
    { 0xc676c47e, SUC_lexicon_tags_72 },
    { 0x1b96c47e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5756c482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9496c485, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf036c48b, SUC_lexicon_tags_49 },
    { 0xbbf6c48c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57b6c48e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f6c494, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b6c497, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2ab6c499, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d6c49f, SUC_lexicon_tags_0 },
    { 0x7056c49f, SUC_lexicon_tags_21 },
    { 0x696c4a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4796c4a3, SUC_lexicon_tags_43 },
    { 0x2636c4a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2256c4a7, SUC_lexicon_tags_151 },
    { 0x37f6c4a8, SUC_lexicon_tags_17 },
    { 0x2536c4a7, SUC_lexicon_tags_17 },
    { 0x5616c4aa, SUC_lexicon_tags_68 },
    { 0x60d6c4a7, SUC_lexicon_tags_43 },
    { 0xb796c4ac, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d6c4b1, SUC_lexicon_tags_17 },
    { 0xe616c4b2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df6c4b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd036c4ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb56c4bd, SUC_lexicon_tags_45 },
    { 0x6536c4be, SUC_lexicon_tags_25 },
    { 0xee76c4bf, SUC_lexicon_tags_7 },
    { 0x1d96c4c0, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf576c4c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x636c4c6, SUC_lexicon_tags_43 },
    { 0x7d6c4c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf056c4cf, SUC_lexicon_tags_10 },
    { 0x2196c4d0, SUC_lexicon_tags_7 },
    { 0xcc16c4d1, SUC_lexicon_tags_7 },
    { 0xb016c4d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2236c4d4, SUC_lexicon_tags_23 },
    { 0x1ff6c4d5, SUC_lexicon_tags_25 },
    { 0x68f6c4d6, SUC_lexicon_tags_12 },
    { 0xb56c4d6, SUC_lexicon_tags_14 },
    { 0x3af6c4d8, SUC_lexicon_tags_26 },
    { 0xe56c4d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfbb6c4db, SUC_lexicon_tags_2 },
    { 0x36d6c4dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff6c4e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa056c4ee, SUC_lexicon_tags_15 },
    { 0xb216c4ef, SUC_lexicon_tags_27 },
    { 0x15d6c4f0, SUC_lexicon_tags_12 },
    { 0x4d56c4f1, SUC_lexicon_tags_63 },
    { 0x29b6c4f2, SUC_lexicon_tags_30 },
    { 0x3e76c4f2, SUC_lexicon_tags_37 },
    { 0xdbf6c4ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd6c4f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb676c4fa, SUC_lexicon_tags_7 },
    { 0x38d6c4fb, SUC_lexicon_tags_11 },
    { 0x29b6c4fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcaf6c4fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa276c504, SUC_lexicon_tags_0 },
    { 0xeb16c505, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc736c507, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5db6c509, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb496c50b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e76c511, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8cf6c513, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad36c515, SUC_lexicon_tags_37 },
    { 0xf796c515, SUC_lexicon_tags_15 },
    { 0x30f6c516, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d6c51c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xafd6c51e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ef6c520, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3676c529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc76c52b, SUC_lexicon_tags_43 },
    { 0x636c52b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d6c52f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14f6c531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb16c534, SUC_lexicon_tags_1 },
    { 0x6e96c535, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1cd6c537, SUC_lexicon_tags_7 },
    { 0x4af6c538, SUC_lexicon_tags_21 },
    { 0x9b96c539, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdbf6c53b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa16c53e, SUC_lexicon_tags_23 },
    { 0x11b6c53f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x87b6c541, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f6c544, SUC_lexicon_tags_17 },
    { 0x7696c545, SUC_lexicon_tags_25 },
    { 0xa596c546, SUC_lexicon_tags_37 },
    { 0x8636c544, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3116c54a, SUC_lexicon_tags_109 },
    { 0x2a76c54b, SUC_lexicon_tags_37 },
    { 0x1d56c54c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b16c54f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe076c551, SUC_lexicon_tags_7 },
    { 0x8036c552, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c36c557, SUC_lexicon_tags_19 },
    { 0x4976c558, SUC_lexicon_tags_29 },
    { 0x3196c558, SUC_lexicon_tags_49 },
    { 0xcc16c55a, SUC_lexicon_tags_37 },
    { 0x1656c55a, SUC_lexicon_tags_26 },
    { 0x3016c55c, SUC_lexicon_tags_87 },
    { 0x1c36c55d, SUC_lexicon_tags_27 },
    { 0xbab6c55e, SUC_lexicon_tags_30 },
    { 0x8776c55d, SUC_lexicon_tags_17 },
    { 0xb796c55e, SUC_lexicon_tags_60 },
    { 0x9216c55b, SUC_lexicon_tags_23 },
    { 0x2596c562, SUC_lexicon_tags_19 },
    { 0x6636c563, SUC_lexicon_tags_11 },
    { 0x4256c55f, SUC_lexicon_tags_429 },
    { 0xadd6c55c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f76c567, SUC_lexicon_tags_55 },
    { 0x4276c568, SUC_lexicon_tags_1319 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c16c56d, SUC_lexicon_tags_482 },
    { 0xd16c56d, SUC_lexicon_tags_0 },
    { 0x7816c56f, SUC_lexicon_tags_45 },
    { 0x9356c570, SUC_lexicon_tags_226 },
    { 0x5256c56e, SUC_lexicon_tags_9 },
    { 0x66b6c56f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b16c576, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff6c57c, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5796c583, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd256c585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea76c588, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7df6c58a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef6c58f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa476c592, SUC_lexicon_tags_82 },
    { 0xa5d6c593, SUC_lexicon_tags_65 },
    { 0xc736c592, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe456c596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f6c59a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef76c5a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8076c5a7, SUC_lexicon_tags_23 },
    { 0xcf36c5a8, SUC_lexicon_tags_43 },
    { 0xaf76c5a9, SUC_lexicon_tags_47 },
    { 0x31f6c5a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5d6c5ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab6c5b0, SUC_lexicon_tags_23 },
    { 0x8936c5b1, SUC_lexicon_tags_33 },
    { 0xe1f6c5b2, SUC_lexicon_tags_19 },
    { 0xedf6c5b3, SUC_lexicon_tags_33 },
    { 0xb096c5b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6c5b7, SUC_lexicon_tags_37 },
    { 0x5b36c5b7, SUC_lexicon_tags_38 },
    { 0x7256c5b7, SUC_lexicon_tags_23 },
    { 0x5e36c5b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2656c5bd, SUC_lexicon_tags_15 },
    { 0xe7d6c5be, SUC_lexicon_tags_63 },
    { 0xea96c5bd, SUC_lexicon_tags_60 },
    { 0x2b56c5bf, SUC_lexicon_tags_15 },
    { 0x2576c5c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76c5c5, SUC_lexicon_tags_1 },
    { 0xb8f6c5c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5256c5ca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc736c5cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16c5d1, SUC_lexicon_tags_11 },
    { 0x79f6c5d2, SUC_lexicon_tags_17 },
    { 0xbb36c5d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc596c5d5, SUC_lexicon_tags_45 },
    { 0xf76c5d6, SUC_lexicon_tags_25 },
    { 0xbed6c5d7, SUC_lexicon_tags_15 },
    { 0xa656c5d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf3b6c5da, SUC_lexicon_tags_13 },
    { 0x2e16c5db, SUC_lexicon_tags_17 },
    { 0x8976c5db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe876c5de, SUC_lexicon_tags_11 },
    { 0x5ed6c5df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9996c5e4, SUC_lexicon_tags_17 },
    { 0xa676c5e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x216c5e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd496c5ed, SUC_lexicon_tags_23 },
    { 0xacf6c5ee, SUC_lexicon_tags_55 },
    { 0x796c5ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b6c5f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f6c601, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9516c604, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b96c60a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8636c60e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfa16c610, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3196c613, SUC_lexicon_tags_32 },
    { 0x5736c614, SUC_lexicon_tags_55 },
    { 0xaa36c613, SUC_lexicon_tags_39 },
    { 0x936c616, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b6c61a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd16c621, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6836c623, SUC_lexicon_tags_11 },
    { 0xe3b6c623, SUC_lexicon_tags_4 },
    { 0x3b16c625, SUC_lexicon_tags_25 },
    { 0xef96c625, SUC_lexicon_tags_37 },
    { 0xdd96c626, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc756c62d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d6c633, SUC_lexicon_tags_47 },
    { 0x916c633, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe096c637, SUC_lexicon_tags_6 },
    { 0x7a56c637, SUC_lexicon_tags_2 },
    { 0x2276c639, SUC_lexicon_tags_8 },
    { 0x66d6c63a, SUC_lexicon_tags_19 },
    { 0x3996c63b, SUC_lexicon_tags_8 },
    { 0x3016c638, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f6c640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc76c642, SUC_lexicon_tags_66 },
    { 0x3d36c642, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf276c647, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa716c64c, SUC_lexicon_tags_1 },
    { 0x7596c64d, SUC_lexicon_tags_3 },
    { 0x6416c64e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d96c653, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c76c655, SUC_lexicon_tags_8 },
    { 0xfff6c656, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f6c65d, SUC_lexicon_tags_87 },
    { 0xc696c65e, SUC_lexicon_tags_6 },
    { 0xf6c65d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd56c663, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7756c66a, SUC_lexicon_tags_27 },
    { 0x5cb6c66b, SUC_lexicon_tags_12 },
    { 0xf596c66b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2f6c66e, SUC_lexicon_tags_2 },
    { 0x5f96c66e, SUC_lexicon_tags_26 },
    { 0x3c36c670, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd6c675, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d6c678, SUC_lexicon_tags_15 },
    { 0xc416c679, SUC_lexicon_tags_7 },
    { 0x5376c67a, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5016c67e, SUC_lexicon_tags_15 },
    { 0x8716c67e, SUC_lexicon_tags_19 },
    { 0x3ef6c680, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1876c682, SUC_lexicon_tags_12 },
    { 0xa296c683, SUC_lexicon_tags_1 },
    { 0xcf96c684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb6c689, SUC_lexicon_tags_25 },
    { 0x1ed6c68a, SUC_lexicon_tags_43 },
    { 0x4a16c68a, SUC_lexicon_tags_2 },
    { 0x80f6c689, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a76c692, SUC_lexicon_tags_11 },
    { 0xa4d6c692, SUC_lexicon_tags_1 },
    { 0x4996c694, SUC_lexicon_tags_26 },
    { 0x6d16c695, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdc96c697, SUC_lexicon_tags_6 },
    { 0xcc56c698, SUC_lexicon_tags_117 },
    { 0xd8b6c699, SUC_lexicon_tags_37 },
    { 0x7536c69a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa876c69f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c36c6a7, SUC_lexicon_tags_16 },
    { 0x4176c6a8, SUC_lexicon_tags_7 },
    { 0xdcb6c6a9, SUC_lexicon_tags_7 },
    { 0xddb6c6aa, SUC_lexicon_tags_70 },
    { 0x2256c6ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf6c6b0, SUC_lexicon_tags_11 },
    { 0x3016c6b0, SUC_lexicon_tags_17 },
    { 0x6736c6b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xae56c6b5, SUC_lexicon_tags_11 },
    { 0x416c6b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f6c6ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7516c6bd, SUC_lexicon_tags_17 },
    { 0xd6c6be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d6c6c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b6c6c5, SUC_lexicon_tags_2 },
    { 0xe136c6c6, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5a36c6c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc36c6cc, SUC_lexicon_tags_27 },
    { 0xa256c6cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed6c6d3, SUC_lexicon_tags_32 },
    { 0xad96c6d4, SUC_lexicon_tags_17 },
    { 0x8e56c6d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe696c6da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb816c6dd, SUC_lexicon_tags_27 },
    { 0x7b76c6de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b6c6e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x176c6e3, SUC_lexicon_tags_21 },
    { 0xdc36c6e4, SUC_lexicon_tags_37 },
    { 0x5d56c6e5, SUC_lexicon_tags_55 },
    { 0x5a96c6e3, SUC_lexicon_tags_38 },
    { 0x5456c6e7, SUC_lexicon_tags_11 },
    { 0xf0b6c6e8, SUC_lexicon_tags_11 },
    { 0x2556c6e8, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0x556c6eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae76c6ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6716c6f2, SUC_lexicon_tags_11 },
    { 0x5996c6f3, SUC_lexicon_tags_17 },
    { 0x2b6c6f2, SUC_lexicon_tags_12 },
    { 0x1d56c6f5, SUC_lexicon_tags_1016 },
    { 0xf1d6c6f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd6c6fa, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6976c6fc, SUC_lexicon_tags_25 },
    { 0x2196c6fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf6c702, SUC_lexicon_tags_11 },
    { 0xa2f6c703, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x73f6c705, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5a96c707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb416c709, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb7d6c70b, SUC_lexicon_tags_25 },
    { 0x5b6c70b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f6c711, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad6c714, SUC_lexicon_tags_63 },
    { 0xea76c715, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf6c719, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9736c71c, SUC_lexicon_tags_6 },
    { 0x9bf6c71c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e56c720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7156c726, SUC_lexicon_tags_15 },
    { 0xb616c727, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f96c729, SUC_lexicon_tags_11 },
    { 0x1c16c729, SUC_lexicon_tags_12 },
    { 0x3fb6c72a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd276c72e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3236c733, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d16c737, SUC_lexicon_tags_47 },
    { 0x2756c738, SUC_lexicon_tags_2 },
    { 0x6b36c739, SUC_lexicon_tags_28 },
    { 0x2c96c73a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b6c740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2236c747, SUC_lexicon_tags_45 },
    { 0xd0d6c748, SUC_lexicon_tags_37 },
    { 0x4cf6c749, SUC_lexicon_tags_24 },
    { 0xe0b6c74a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d36c74c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9856c74f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb16c75d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e76c75f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b6c766, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76d6c768, SUC_lexicon_tags_17 },
    { 0x76d6c769, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3b6c76b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58f6c76d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8b6c76f, SUC_lexicon_tags_34 },
    { 0xce76c76f, SUC_lexicon_tags_48 },
    { 0x40f6c771, SUC_lexicon_tags_37 },
    { 0x8d36c772, SUC_lexicon_tags_63 },
    { 0x20d6c76f, SUC_lexicon_tags_11 },
    { 0x1e56c774, SUC_lexicon_tags_63 },
    { 0xaa36c774, SUC_lexicon_tags_11 },
    { 0x2756c775, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a76c778, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd16c77c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8996c786, SUC_lexicon_tags_25 },
    { 0x5b56c786, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb6c78c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb6c792, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f6c799, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3656c79b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x64d6c79d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb36c7a1, SUC_lexicon_tags_2 },
    { 0xc7d6c7a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x176c7a6, SUC_lexicon_tags_1 },
    { 0x776c7a7, SUC_lexicon_tags_33 },
    { 0xacf6c7a7, SUC_lexicon_tags_10 },
    { 0xdc76c7a9, SUC_lexicon_tags_2 },
    { 0x3676c7a7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f96c7af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc336c7b2, SUC_lexicon_tags_19 },
    { 0xd7d6c7b3, SUC_lexicon_tags_15 },
    { 0x8dd6c7b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7f6c7b6, SUC_lexicon_tags_27 },
    { 0x37d6c7b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b6c7bb, SUC_lexicon_tags_17 },
    { 0xb316c7bc, SUC_lexicon_tags_1 },
    { 0x1a56c7bd, SUC_lexicon_tags_0 },
    { 0xfcf6c7be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba76c7c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f96c7c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f6c7ca, SUC_lexicon_tags_63 },
    { 0xcc76c7ca, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf6c7ce, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e16c7d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6516c7d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7e96c7db, SUC_lexicon_tags_10 },
    { 0x9d96c7dc, SUC_lexicon_tags_1 },
    { 0x7316c7dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b6c7e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5216c7e3, SUC_lexicon_tags_37 },
    { 0x8196c7e4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6576c7e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5676c7e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4596c7ec, SUC_lexicon_tags_25 },
    { 0x88f6c7ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b6c7f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62f6c7f9, SUC_lexicon_tags_28 },
    { 0xf976c7fa, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6116c7ff, SUC_lexicon_tags_2 },
    { 0x95b6c800, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab6c804, SUC_lexicon_tags_11 },
    { 0x6816c805, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xce56c808, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62b6c80a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3016c80d, SUC_lexicon_tags_12 },
    { 0xd2b6c80d, SUC_lexicon_tags_1692 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f6c811, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa656c819, SUC_lexicon_tags_10 },
    { 0x38b6c81a, SUC_lexicon_tags_16 },
    { 0x56b6c81a, SUC_lexicon_tags_0 },
    { 0x7356c81c, SUC_lexicon_tags_37 },
    { 0x8d56c819, SUC_lexicon_tags_39 },
    { 0xb856c81e, SUC_lexicon_tags_55 },
    { 0x4c56c81c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1076c822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b6c825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3596c82a, SUC_lexicon_tags_27 },
    { 0x5776c82a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6616c838, SUC_lexicon_tags_14 },
    { 0x5756c839, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfdb6c83b, SUC_lexicon_tags_13 },
    { 0x22b6c83c, SUC_lexicon_tags_0 },
    { 0xf416c83d, SUC_lexicon_tags_15 },
    { 0x4856c83e, SUC_lexicon_tags_23 },
    { 0x2b56c83f, SUC_lexicon_tags_12 },
    { 0xf876c83e, SUC_lexicon_tags_9 },
    { 0xae96c83d, SUC_lexicon_tags_49 },
    { 0xbd6c83d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af6c845, SUC_lexicon_tags_17 },
    { 0x4dd6c846, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d6c84b, SUC_lexicon_tags_40 },
    { 0x31b6c84c, SUC_lexicon_tags_1 },
    { 0xe4f6c84d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6c850, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa6d6c852, SUC_lexicon_tags_1 },
    { 0x37d6c853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7d6c855, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db6c858, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b16c85c, SUC_lexicon_tags_10 },
    { 0x9356c85d, SUC_lexicon_tags_43 },
    { 0xe136c85e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96c863, SUC_lexicon_tags_7 },
    { 0x58f6c864, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd256c866, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd6c869, SUC_lexicon_tags_44 },
    { 0xb4d6c86a, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc256c870, SUC_lexicon_tags_7 },
    { 0xc216c870, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2856c873, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa416c875, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d6c878, SUC_lexicon_tags_29 },
    { 0xae76c878, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8816c87c, SUC_lexicon_tags_2 },
    { 0xc796c87d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff6c884, SUC_lexicon_tags_9 },
    { 0xd536c885, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x596c889, SUC_lexicon_tags_46 },
    { 0x41d6c889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b6c890, SUC_lexicon_tags_25 },
    { 0xa1d6c890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa456c893, SUC_lexicon_tags_16 },
    { 0x1cb6c894, SUC_lexicon_tags_33 },
    { 0x5b76c894, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3b6c897, SUC_lexicon_tags_29 },
    { 0x13f6c898, SUC_lexicon_tags_12 },
    { 0x5416c899, SUC_lexicon_tags_82 },
    { 0xa496c899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d6c89d, SUC_lexicon_tags_6 },
    { 0xb9f6c89d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xacf6c8a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a36c8a7, SUC_lexicon_tags_1 },
    { 0x6996c8a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a16c8aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a16c8ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d6c8af, SUC_lexicon_tags_15 },
    { 0xd956c8af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f76c8b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd6c8b6, SUC_lexicon_tags_0 },
    { 0x56b6c8b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce36c8ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6c8c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3716c8c4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc56c8c7, SUC_lexicon_tags_17 },
    { 0x4376c8c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd6c8ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a76c8d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5416c8dd, SUC_lexicon_tags_11 },
    { 0x7796c8dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb16c8e1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4216c8e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3d6c8e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf116c8e7, SUC_lexicon_tags_14 },
    { 0x9896c8e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff6c8eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcdb6c8ed, SUC_lexicon_tags_1 },
    { 0x7776c8ee, SUC_lexicon_tags_25 },
    { 0xafb6c8ef, SUC_lexicon_tags_11 },
    { 0x3496c8ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc36c8f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff36c8f7, SUC_lexicon_tags_11 },
    { 0x9016c8f8, SUC_lexicon_tags_14 },
    { 0xf316c8f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df6c8ff, SUC_lexicon_tags_25 },
    { 0x79b6c8ff, SUC_lexicon_tags_31 },
    { 0x7e56c900, SUC_lexicon_tags_15 },
    { 0xf0d6c901, SUC_lexicon_tags_19 },
    { 0xb8f6c900, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf316c909, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9b6c90b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3016c90d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f76c90f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7156c911, SUC_lexicon_tags_7 },
    { 0xf076c912, SUC_lexicon_tags_47 },
    { 0x3cf6c912, SUC_lexicon_tags_45 },
    { 0x996c911, SUC_lexicon_tags_47 },
    { 0xa216c911, SUC_lexicon_tags_49 },
    { 0x1c16c912, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35f6c918, SUC_lexicon_tags_1 },
    { 0x7296c919, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa696c91c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x156c91e, SUC_lexicon_tags_70 },
    { 0x7716c91e, SUC_lexicon_tags_21 },
    { 0x1416c920, SUC_lexicon_tags_11 },
    { 0x59d6c921, SUC_lexicon_tags_30 },
    { 0x8b96c922, SUC_lexicon_tags_7 },
    { 0x956c91f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52b6c925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7656c929, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb6c930, SUC_lexicon_tags_8 },
    { 0x2db6c931, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d6c935, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b96c939, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d6c93f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9316c944, SUC_lexicon_tags_165 },
    { 0x8756c945, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f76c94d, SUC_lexicon_tags_24 },
    { 0xd416c94e, SUC_lexicon_tags_11 },
    { 0x84d6c94d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc3d6c951, SUC_lexicon_tags_49 },
    { 0xbe56c951, SUC_lexicon_tags_46 },
    { 0x5a36c953, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4896c956, SUC_lexicon_tags_3 },
    { 0x28f6c957, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb6c95a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1336c95d, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f36c962, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb4d6c964, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d96c967, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x916c969, SUC_lexicon_tags_14 },
    { 0xd556c969, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9636c96c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb436c96e, SUC_lexicon_tags_27 },
    { 0x24b6c96f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x83f6c971, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5256c973, SUC_lexicon_tags_47 },
    { 0x6056c973, SUC_lexicon_tags_47 },
    { 0xfa76c975, SUC_lexicon_tags_38 },
    { 0x3396c975, SUC_lexicon_tags_37 },
    { 0x54d6c977, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae36c97b, SUC_lexicon_tags_21 },
    { 0xbfb6c97b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa656c981, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f6c986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec76c98a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b6c991, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b6c996, SUC_lexicon_tags_19 },
    { 0xc3f6c997, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7016c99f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9056c9a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8036c9a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd116c9a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c96c9a9, SUC_lexicon_tags_1 },
    { 0xea76c9aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1db6c9ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbb36c9ae, SUC_lexicon_tags_1 },
    { 0xd396c9af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb996c9bf, SUC_lexicon_tags_68 },
    { 0xa256c9c0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1076c9c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2116c9ca, SUC_lexicon_tags_16 },
    { 0xaa56c9cb, SUC_lexicon_tags_2 },
    { 0xc536c9ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x75f6c9ce, SUC_lexicon_tags_37 },
    { 0x4c76c9cf, SUC_lexicon_tags_77 },
    { 0xbf76c9cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1336c9d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4656c9d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2156c9d8, SUC_lexicon_tags_25 },
    { 0xaa56c9d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x80b6c9db, SUC_lexicon_tags_28 },
    { 0x4216c9db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ed6c9de, SUC_lexicon_tags_9 },
    { 0xb536c9de, SUC_lexicon_tags_138 },
    { 0x6716c9df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb6c9e5, SUC_lexicon_tags_2 },
    { 0x7bb6c9e6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7876c9e9, SUC_lexicon_tags_37 },
    { 0x3b96c9ea, SUC_lexicon_tags_17 },
    { 0xb996c9e9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd6c9ef, SUC_lexicon_tags_43 },
    { 0x41f6c9f0, SUC_lexicon_tags_27 },
    { 0xc216c9f1, SUC_lexicon_tags_297 },
    { 0, NULL },
    { 0, NULL },
    { 0xec96c9f4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1ab6c9f6, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2076c9fa, SUC_lexicon_tags_19 },
    { 0xb1d6c9fa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6fd6c9fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f6ca09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8e56ca0b, SUC_lexicon_tags_11 },
    { 0x7d36ca0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7316ca10, SUC_lexicon_tags_53 },
    { 0xbdb6ca11, SUC_lexicon_tags_62 },
    { 0x27d6ca12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6156ca1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3196ca1f, SUC_lexicon_tags_30 },
    { 0xf1f6ca1f, SUC_lexicon_tags_31 },
    { 0xaa16ca20, SUC_lexicon_tags_15 },
    { 0x8db6ca20, SUC_lexicon_tags_15 },
    { 0x6a16ca23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b56ca2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b6ca31, SUC_lexicon_tags_7 },
    { 0x5b76ca31, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed16ca3b, SUC_lexicon_tags_4 },
    { 0x5656ca3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf456ca41, SUC_lexicon_tags_37 },
    { 0x8f76ca41, SUC_lexicon_tags_37 },
    { 0xa376ca42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6116ca45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff6ca4b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x916ca4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5716ca4f, SUC_lexicon_tags_9 },
    { 0x3876ca4f, SUC_lexicon_tags_66 },
    { 0xd576ca51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xff56ca54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92f6ca56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b6ca5b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b6ca5f, SUC_lexicon_tags_25 },
    { 0x7276ca60, SUC_lexicon_tags_30 },
    { 0xf556ca60, SUC_lexicon_tags_12 },
    { 0x2eb6ca5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c36ca6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f6ca76, SUC_lexicon_tags_62 },
    { 0x78f6ca77, SUC_lexicon_tags_47 },
    { 0xb3f6ca78, SUC_lexicon_tags_11 },
    { 0xf616ca79, SUC_lexicon_tags_63 },
    { 0x6bb6ca7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb596ca7c, SUC_lexicon_tags_11 },
    { 0xbed6ca7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4a36ca7f, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe596ca84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b6ca8c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc96ca92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9116ca99, SUC_lexicon_tags_6 },
    { 0xab56ca99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd6ca9d, SUC_lexicon_tags_15 },
    { 0xf316ca9e, SUC_lexicon_tags_37 },
    { 0x596ca9e, SUC_lexicon_tags_14 },
    { 0x2f76caa0, SUC_lexicon_tags_11 },
    { 0xe36caa0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c96caa4, SUC_lexicon_tags_60 },
    { 0x8b76caa5, SUC_lexicon_tags_37 },
    { 0xacd6caa6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd476caa8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb6caac, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0xf216caae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb716cab2, SUC_lexicon_tags_17 },
    { 0x3716cab3, SUC_lexicon_tags_11 },
    { 0x4a96cab4, SUC_lexicon_tags_9 },
    { 0x3fd6cab4, SUC_lexicon_tags_19 },
    { 0xcf56cab6, SUC_lexicon_tags_25 },
    { 0xd4b6cab7, SUC_lexicon_tags_45 },
    { 0x2c56cab2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe36cabe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b6cac1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d6cac4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d6cacb, SUC_lexicon_tags_37 },
    { 0xd936cacb, SUC_lexicon_tags_15 },
    { 0x3616cacd, SUC_lexicon_tags_57 },
    { 0x73b6cace, SUC_lexicon_tags_37 },
    { 0x37d6cacd, SUC_lexicon_tags_37 },
    { 0x22b6cacd, SUC_lexicon_tags_37 },
    { 0xc236cad1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db6cad5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b6cad8, SUC_lexicon_tags_2 },
    { 0x3b76cad9, SUC_lexicon_tags_2 },
    { 0x57b6cada, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6016cade, SUC_lexicon_tags_1 },
    { 0x2d36cadf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d6cae3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8016caed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d6caf0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa76caf6, SUC_lexicon_tags_37 },
    { 0x6256caf7, SUC_lexicon_tags_49 },
    { 0x6bd6caf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d6cafb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1916caff, SUC_lexicon_tags_2 },
    { 0x2596cb00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d16cb02, SUC_lexicon_tags_25 },
    { 0x2bb6cb03, SUC_lexicon_tags_47 },
    { 0x80f6cb04, SUC_lexicon_tags_43 },
    { 0x1876cb05, SUC_lexicon_tags_17 },
    { 0x1876cb03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b16cb0a, SUC_lexicon_tags_14 },
    { 0x5036cb0a, SUC_lexicon_tags_30 },
    { 0xd656cb0c, SUC_lexicon_tags_74 },
    { 0xc96cb0c, SUC_lexicon_tags_38 },
    { 0x1e56cb0e, SUC_lexicon_tags_1 },
    { 0xf1f6cb0e, SUC_lexicon_tags_45 },
    { 0xef6cb0a, SUC_lexicon_tags_12 },
    { 0x57d6cb0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x86b6cb13, SUC_lexicon_tags_66 },
    { 0xc396cb13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d96cb18, SUC_lexicon_tags_19 },
    { 0x1856cb18, SUC_lexicon_tags_2 },
    { 0xa6f6cb1a, SUC_lexicon_tags_17 },
    { 0x3376cb1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8276cb1f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x796cb21, SUC_lexicon_tags_16 },
    { 0x3d76cb21, SUC_lexicon_tags_7 },
    { 0x2516cb23, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd336cb26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f6cb2d, SUC_lexicon_tags_17 },
    { 0xac56cb2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16cb32, SUC_lexicon_tags_53 },
    { 0x23f6cb33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2576cb38, SUC_lexicon_tags_11 },
    { 0xa216cb39, SUC_lexicon_tags_27 },
    { 0xf896cb3a, SUC_lexicon_tags_38 },
    { 0x59f6cb38, SUC_lexicon_tags_47 },
    { 0x47f6cb3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9cf6cb3e, SUC_lexicon_tags_25 },
    { 0xa256cb3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf516cb48, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfc16cb4a, SUC_lexicon_tags_17 },
    { 0x2af6cb4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa156cb4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b36cb52, SUC_lexicon_tags_7 },
    { 0x5416cb52, SUC_lexicon_tags_120 },
    { 0xea96cb54, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc176cb57, SUC_lexicon_tags_77 },
    { 0xafb6cb57, SUC_lexicon_tags_25 },
    { 0xddf6cb59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6076cb62, SUC_lexicon_tags_6 },
    { 0x8a56cb62, SUC_lexicon_tags_37 },
    { 0xd776cb63, SUC_lexicon_tags_37 },
    { 0x2ab6cb65, SUC_lexicon_tags_12 },
    { 0x9956cb66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbf96cb68, SUC_lexicon_tags_12 },
    { 0xf6d6cb69, SUC_lexicon_tags_25 },
    { 0x6716cb68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6cb73, SUC_lexicon_tags_8 },
    { 0x2796cb74, SUC_lexicon_tags_37 },
    { 0x6416cb75, SUC_lexicon_tags_17 },
    { 0xf836cb75, SUC_lexicon_tags_37 },
    { 0xbeb6cb77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa756cb79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8096cb86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2356cb8b, SUC_lexicon_tags_66 },
    { 0x11d6cb8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d56cb8f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x89b6cb91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba96cb95, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf396cb99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d6cb9d, SUC_lexicon_tags_61 },
    { 0x9976cb9e, SUC_lexicon_tags_33 },
    { 0xf256cb9f, SUC_lexicon_tags_33 },
    { 0x2e16cba0, SUC_lexicon_tags_11 },
    { 0x2ad6cb9d, SUC_lexicon_tags_15 },
    { 0x7616cba2, SUC_lexicon_tags_14 },
    { 0x7136cba3, SUC_lexicon_tags_17 },
    { 0x9176cba4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe416cbaa, SUC_lexicon_tags_33 },
    { 0xa2b6cbab, SUC_lexicon_tags_63 },
    { 0x4756cbac, SUC_lexicon_tags_6 },
    { 0x9016cbad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d16cbb5, SUC_lexicon_tags_66 },
    { 0xf9b6cbb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f6cbbb, SUC_lexicon_tags_27 },
    { 0xc9b6cbbc, SUC_lexicon_tags_9 },
    { 0x8616cbbc, SUC_lexicon_tags_25 },
    { 0x80d6cbbd, SUC_lexicon_tags_7 },
    { 0xe516cbbf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2576cbc3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x836cbc7, SUC_lexicon_tags_26 },
    { 0x8d36cbc8, SUC_lexicon_tags_15 },
    { 0x28f6cbc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1076cbce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a16cbd8, SUC_lexicon_tags_7 },
    { 0x7276cbd9, SUC_lexicon_tags_7 },
    { 0xe396cbda, SUC_lexicon_tags_7 },
    { 0x5476cbda, SUC_lexicon_tags_62 },
    { 0x99d6cbdc, SUC_lexicon_tags_7 },
    { 0x9db6cbdd, SUC_lexicon_tags_32 },
    { 0x92f6cbde, SUC_lexicon_tags_12 },
    { 0x9476cbdd, SUC_lexicon_tags_1 },
    { 0x88d6cbde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb56cbe7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8316cbf0, SUC_lexicon_tags_43 },
    { 0x4b36cbf1, SUC_lexicon_tags_1 },
    { 0x1db6cbf2, SUC_lexicon_tags_17 },
    { 0xd36cbf3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf1f6cbf5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe676cbf7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3296cbfa, SUC_lexicon_tags_63 },
    { 0xd5d6cbfb, SUC_lexicon_tags_17 },
    { 0xf976cbfc, SUC_lexicon_tags_0 },
    { 0x53b6cbfd, SUC_lexicon_tags_262 },
    { 0x25b6cbfd, SUC_lexicon_tags_7 },
    { 0xb8f6cbff, SUC_lexicon_tags_64 },
    { 0xfcb6cc00, SUC_lexicon_tags_1 },
    { 0x6856cc01, SUC_lexicon_tags_43 },
    { 0xd716cc02, SUC_lexicon_tags_46 },
    { 0x7336cc03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f6cc06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6cc0a, SUC_lexicon_tags_21 },
    { 0xd576cc0b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd036cc0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff6cc11, SUC_lexicon_tags_1 },
    { 0x28d6cc11, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9d6cc14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d6cc17, SUC_lexicon_tags_289 },
    { 0x3b96cc18, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b6cc1b, SUC_lexicon_tags_68 },
    { 0x9db6cc1c, SUC_lexicon_tags_27 },
    { 0xd176cc1d, SUC_lexicon_tags_21 },
    { 0xa3d6cc1e, SUC_lexicon_tags_8 },
    { 0xfc96cc1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb876cc29, SUC_lexicon_tags_25 },
    { 0xa716cc29, SUC_lexicon_tags_12 },
    { 0x9936cc29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3836cc2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe396cc30, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa956cc36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7b6cc38, SUC_lexicon_tags_7 },
    { 0xaff6cc38, SUC_lexicon_tags_37 },
    { 0x31f6cc3a, SUC_lexicon_tags_43 },
    { 0x8cf6cc39, SUC_lexicon_tags_32 },
    { 0x9116cc3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6cc41, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe116cc44, SUC_lexicon_tags_39 },
    { 0x5dd6cc45, SUC_lexicon_tags_29 },
    { 0x7d96cc45, SUC_lexicon_tags_17 },
    { 0x5856cc47, SUC_lexicon_tags_10 },
    { 0x5056cc48, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x2736cc4b, SUC_lexicon_tags_41 },
    { 0xacb6cc4b, SUC_lexicon_tags_1271 },
    { 0, NULL },
    { 0, NULL },
    { 0x1916cc4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x456cc51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccf6cc53, SUC_lexicon_tags_33 },
    { 0x9e96cc53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ab6cc56, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3336cc59, SUC_lexicon_tags_33 },
    { 0xd656cc59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f96cc5d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x68b6cc5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f6cc62, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f6cc65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8136cc67, SUC_lexicon_tags_23 },
    { 0xc616cc68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1716cc6a, SUC_lexicon_tags_37 },
    { 0xe4d6cc6b, SUC_lexicon_tags_14 },
    { 0x176cc6c, SUC_lexicon_tags_17 },
    { 0x3836cc6a, SUC_lexicon_tags_25 },
    { 0x3af6cc6c, SUC_lexicon_tags_17 },
    { 0xde36cc6e, SUC_lexicon_tags_42 },
    { 0xc296cc70, SUC_lexicon_tags_7 },
    { 0xa476cc6d, SUC_lexicon_tags_201 },
    { 0xc6f6cc72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9916cc74, SUC_lexicon_tags_47 },
    { 0x2cf6cc75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb6cc78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda16cc7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8436cc7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb16cc80, SUC_lexicon_tags_17 },
    { 0x99d6cc80, SUC_lexicon_tags_25 },
    { 0x4f6cc81, SUC_lexicon_tags_7 },
    { 0xc36cc80, SUC_lexicon_tags_68 },
    { 0x7fd6cc81, SUC_lexicon_tags_17 },
    { 0x2836cc82, SUC_lexicon_tags_91 },
    { 0xfbf6cc83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf6cc8b, SUC_lexicon_tags_38 },
    { 0xd456cc8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe556cc92, SUC_lexicon_tags_11 },
    { 0x5156cc93, SUC_lexicon_tags_11 },
    { 0x3356cc92, SUC_lexicon_tags_2 },
    { 0xbb96cc95, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b6cc98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f36cc9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed6cca6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xdbf6cca8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7636ccaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b6ccb3, SUC_lexicon_tags_47 },
    { 0x3d6ccb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe776ccb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5436ccb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8f6ccba, SUC_lexicon_tags_11 },
    { 0xad6ccbb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1776ccbf, SUC_lexicon_tags_11 },
    { 0x12b6ccc0, SUC_lexicon_tags_11 },
    { 0x1456ccc0, SUC_lexicon_tags_15 },
    { 0xda36ccc2, SUC_lexicon_tags_15 },
    { 0x496ccc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f6ccc6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b6ccca, SUC_lexicon_tags_17 },
    { 0x56ccca, SUC_lexicon_tags_26 },
    { 0xca36cccb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5236ccce, SUC_lexicon_tags_49 },
    { 0x6276cccf, SUC_lexicon_tags_14 },
    { 0x3d6ccd0, SUC_lexicon_tags_6 },
    { 0x1b76ccd1, SUC_lexicon_tags_15 },
    { 0xa876ccd2, SUC_lexicon_tags_0 },
    { 0xbd6ccd3, SUC_lexicon_tags_25 },
    { 0xbfb6ccd3, SUC_lexicon_tags_37 },
    { 0xccf6ccd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d36ccd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe56ccdc, SUC_lexicon_tags_37 },
    { 0x1f16ccdd, SUC_lexicon_tags_26 },
    { 0xe696ccdd, SUC_lexicon_tags_25 },
    { 0x6196ccde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4396cce1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce96cce5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f6cceb, SUC_lexicon_tags_25 },
    { 0xb6d6ccec, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd6ccf0, SUC_lexicon_tags_65 },
    { 0x5996ccf1, SUC_lexicon_tags_26 },
    { 0x3fb6ccf0, SUC_lexicon_tags_11 },
    { 0xeaf6ccf3, SUC_lexicon_tags_1 },
    { 0x1756ccf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc576ccf6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba36ccf8, SUC_lexicon_tags_49 },
    { 0x46f6ccf8, SUC_lexicon_tags_31 },
    { 0x4576ccfa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe576ccfd, SUC_lexicon_tags_29 },
    { 0x6016ccfd, SUC_lexicon_tags_30 },
    { 0x6656ccff, SUC_lexicon_tags_11 },
    { 0xff56cd00, SUC_lexicon_tags_25 },
    { 0xc596cd01, SUC_lexicon_tags_45 },
    { 0xe496ccff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d6cd05, SUC_lexicon_tags_15 },
    { 0x6b16cd06, SUC_lexicon_tags_68 },
    { 0xcbf6cd06, SUC_lexicon_tags_29 },
    { 0x2af6cd08, SUC_lexicon_tags_15 },
    { 0xfb36cd09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad56cd0e, SUC_lexicon_tags_62 },
    { 0x1ed6cd0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8376cd12, SUC_lexicon_tags_11 },
    { 0x3dd6cd12, SUC_lexicon_tags_2 },
    { 0xd676cd14, SUC_lexicon_tags_14 },
    { 0x3796cd15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff6cd21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4676cd25, SUC_lexicon_tags_10 },
    { 0x2756cd26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db6cd2c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x976cd31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b6cd34, SUC_lexicon_tags_7 },
    { 0x9a56cd35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16cd3a, SUC_lexicon_tags_7 },
    { 0xc376cd3b, SUC_lexicon_tags_25 },
    { 0xa396cd3b, SUC_lexicon_tags_37 },
    { 0xcef6cd3d, SUC_lexicon_tags_2 },
    { 0x3976cd3d, SUC_lexicon_tags_48 },
    { 0x1976cd3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f36cd46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f6cd49, SUC_lexicon_tags_9 },
    { 0x2b76cd4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b96cd4e, SUC_lexicon_tags_23 },
    { 0xc856cd4f, SUC_lexicon_tags_25 },
    { 0x26f6cd4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c56cd53, SUC_lexicon_tags_2 },
    { 0x9cd6cd53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2d6cd56, SUC_lexicon_tags_16 },
    { 0x3016cd57, SUC_lexicon_tags_27 },
    { 0xc796cd58, SUC_lexicon_tags_11 },
    { 0xb56cd59, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5516cd5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d6cd68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad6cd6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b6cd71, SUC_lexicon_tags_37 },
    { 0x3536cd72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe716cd7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16cd7f, SUC_lexicon_tags_17 },
    { 0x1976cd7f, SUC_lexicon_tags_17 },
    { 0xc776cd81, SUC_lexicon_tags_12 },
    { 0x1916cd82, SUC_lexicon_tags_15 },
    { 0xcc96cd82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8d6cd85, SUC_lexicon_tags_27 },
    { 0xddf6cd85, SUC_lexicon_tags_17 },
    { 0x6d76cd87, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x82d6cd89, SUC_lexicon_tags_19 },
    { 0xf276cd8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc76cd90, SUC_lexicon_tags_43 },
    { 0x1f96cd91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d6cd94, SUC_lexicon_tags_1 },
    { 0x8ff6cd95, SUC_lexicon_tags_6 },
    { 0x75b6cd96, SUC_lexicon_tags_18 },
    { 0x3c96cd97, SUC_lexicon_tags_17 },
    { 0x2636cd98, SUC_lexicon_tags_9 },
    { 0xc3d6cd99, SUC_lexicon_tags_45 },
    { 0x17d6cd9a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd076cd9c, SUC_lexicon_tags_138 },
    { 0x6996cd9c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f76cda0, SUC_lexicon_tags_12 },
    { 0xc096cda1, SUC_lexicon_tags_25 },
    { 0xab6cda0, SUC_lexicon_tags_25 },
    { 0xce96cda3, SUC_lexicon_tags_16 },
    { 0xfb56cda3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f6cda9, SUC_lexicon_tags_7 },
    { 0x7636cdaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f6cdae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d56cdb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2d6cdb7, SUC_lexicon_tags_1 },
    { 0xb656cdb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77b6cdba, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x8e16cdbc, SUC_lexicon_tags_8 },
    { 0x3ab6cdbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5956cdbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2376cdc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f36cdcd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf6cdd3, SUC_lexicon_tags_7 },
    { 0xc536cdd4, SUC_lexicon_tags_47 },
    { 0xa7d6cdd5, SUC_lexicon_tags_9 },
    { 0x8c36cdd6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd276cdd8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f6cde1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7616cde3, SUC_lexicon_tags_0 },
    { 0x6a16cde4, SUC_lexicon_tags_15 },
    { 0xae96cde5, SUC_lexicon_tags_17 },
    { 0x60f6cde5, SUC_lexicon_tags_11 },
    { 0x7396cde6, SUC_lexicon_tags_25 },
    { 0x9d56cde8, SUC_lexicon_tags_1 },
    { 0x3216cde9, SUC_lexicon_tags_16 },
    { 0x57f6cdea, SUC_lexicon_tags_12 },
    { 0xb736cdea, SUC_lexicon_tags_83 },
    { 0x22b6cde5, SUC_lexicon_tags_17 },
    { 0x7516cdea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6396cdf3, SUC_lexicon_tags_7 },
    { 0x2896cdf4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3556cdf7, SUC_lexicon_tags_37 },
    { 0xc0f6cdf8, SUC_lexicon_tags_1 },
    { 0xb116cdf8, SUC_lexicon_tags_10 },
    { 0xd1b6cdf8, SUC_lexicon_tags_25 },
    { 0x49f6cdfb, SUC_lexicon_tags_37 },
    { 0xfb96cdfc, SUC_lexicon_tags_12 },
    { 0xdc56cdfd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d6ce00, SUC_lexicon_tags_26 },
    { 0x7d36ce00, SUC_lexicon_tags_26 },
    { 0x72b6ce02, SUC_lexicon_tags_17 },
    { 0xa9d6ce00, SUC_lexicon_tags_7 },
    { 0x12b6ce00, SUC_lexicon_tags_49 },
    { 0x5856ce03, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b6ce09, SUC_lexicon_tags_37 },
    { 0x8576ce09, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3176ce0d, SUC_lexicon_tags_0 },
    { 0x6ed6ce0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c36ce10, SUC_lexicon_tags_14 },
    { 0xf1f6ce11, SUC_lexicon_tags_34 },
    { 0x2a96ce12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36ce15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x61b6ce17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd6ce1a, SUC_lexicon_tags_15 },
    { 0xd776ce1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b6ce1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf936ce24, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a36ce2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b6ce32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9d6ce34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5856ce37, SUC_lexicon_tags_10 },
    { 0xbfb6ce38, SUC_lexicon_tags_60 },
    { 0x6a76ce39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d6ce3c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b6ce40, SUC_lexicon_tags_62 },
    { 0xa396ce41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d6ce46, SUC_lexicon_tags_1 },
    { 0x9796ce46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39b6ce49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4956ce4b, SUC_lexicon_tags_56 },
    { 0xf556ce4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe36ce4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d6ce51, SUC_lexicon_tags_17 },
    { 0xcfd6ce51, SUC_lexicon_tags_1048 },
    { 0, NULL },
    { 0, NULL },
    { 0xe776ce55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff6ce5e, SUC_lexicon_tags_0 },
    { 0x35f6ce5f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8296ce61, SUC_lexicon_tags_31 },
    { 0x35b6ce62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2916ce64, SUC_lexicon_tags_14 },
    { 0xa496ce65, SUC_lexicon_tags_15 },
    { 0xffb6ce66, SUC_lexicon_tags_10 },
    { 0xf936ce66, SUC_lexicon_tags_15 },
    { 0xfc56ce65, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c76ce6b, SUC_lexicon_tags_25 },
    { 0x9d6ce6c, SUC_lexicon_tags_25 },
    { 0x4ad6ce6d, SUC_lexicon_tags_14 },
    { 0xf576ce6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf6ce72, SUC_lexicon_tags_15 },
    { 0xe196ce72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x936ce75, SUC_lexicon_tags_37 },
    { 0x356ce75, SUC_lexicon_tags_60 },
    { 0x5036ce77, SUC_lexicon_tags_14 },
    { 0x2ab6ce78, SUC_lexicon_tags_11 },
    { 0xdd36ce79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d36ce7c, SUC_lexicon_tags_60 },
    { 0xc036ce7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f56ce80, SUC_lexicon_tags_37 },
    { 0x1556ce81, SUC_lexicon_tags_11 },
    { 0x1376ce81, SUC_lexicon_tags_17 },
    { 0xf1b6ce83, SUC_lexicon_tags_39 },
    { 0x86f6ce84, SUC_lexicon_tags_17 },
    { 0xcf96ce85, SUC_lexicon_tags_2 },
    { 0xd3b6ce83, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xbab6ce88, SUC_lexicon_tags_1 },
    { 0x7b96ce88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d6ce8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2416ce90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf216ce92, SUC_lexicon_tags_893 },
    { 0x8416ce92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cd6ce95, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b76ce98, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af6ce9e, SUC_lexicon_tags_25 },
    { 0xf96ce9f, SUC_lexicon_tags_1 },
    { 0xc016ce9f, SUC_lexicon_tags_46 },
    { 0x7e36ce9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa36cea6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb196cea8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16b6ceaa, SUC_lexicon_tags_17 },
    { 0xe736ceab, SUC_lexicon_tags_1 },
    { 0xd756ceac, SUC_lexicon_tags_37 },
    { 0xb0f6ceac, SUC_lexicon_tags_11 },
    { 0x676ceab, SUC_lexicon_tags_1 },
    { 0xc116ceae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a36ceb3, SUC_lexicon_tags_60 },
    { 0x81b6ceb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8176ceb7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9496ceb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadd6cebb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f6cec5, SUC_lexicon_tags_1 },
    { 0x8b16cec5, SUC_lexicon_tags_11 },
    { 0x7cd6cec7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc736cecc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb456cecf, SUC_lexicon_tags_45 },
    { 0x1716cecf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f96ced8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b6cedc, SUC_lexicon_tags_37 },
    { 0x85d6cedd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d6cee3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1996cee9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58d6ceeb, SUC_lexicon_tags_15 },
    { 0x8916ceec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6ceef, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db6cef2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde76cef4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43f6cef6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd696cf02, SUC_lexicon_tags_28 },
    { 0xfcf6cf03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9436cf08, SUC_lexicon_tags_17 },
    { 0xb236cf09, SUC_lexicon_tags_17 },
    { 0x56b6cf09, SUC_lexicon_tags_37 },
    { 0xe576cf0a, SUC_lexicon_tags_12 },
    { 0x1b56cf0b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3b16cf0e, SUC_lexicon_tags_37 },
    { 0xda16cf0f, SUC_lexicon_tags_18 },
    { 0xa036cf0f, SUC_lexicon_tags_15 },
    { 0x77f6cf11, SUC_lexicon_tags_12 },
    { 0x89d6cf12, SUC_lexicon_tags_1 },
    { 0x9936cf13, SUC_lexicon_tags_38 },
    { 0x6516cf13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3856cf17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e16cf1c, SUC_lexicon_tags_23 },
    { 0xa7b6cf1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7836cf21, SUC_lexicon_tags_27 },
    { 0xe976cf22, SUC_lexicon_tags_15 },
    { 0x4196cf23, SUC_lexicon_tags_17 },
    { 0x6396cf21, SUC_lexicon_tags_25 },
    { 0xdd16cf25, SUC_lexicon_tags_31 },
    { 0xf436cf26, SUC_lexicon_tags_14 },
    { 0x2536cf26, SUC_lexicon_tags_15 },
    { 0xcb16cf28, SUC_lexicon_tags_3 },
    { 0xacd6cf28, SUC_lexicon_tags_85 },
    { 0xac56cf2a, SUC_lexicon_tags_47 },
    { 0x8fb6cf2a, SUC_lexicon_tags_23 },
    { 0xd556cf2c, SUC_lexicon_tags_14 },
    { 0x3cf6cf2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf636cf2f, SUC_lexicon_tags_11 },
    { 0x49f6cf30, SUC_lexicon_tags_1 },
    { 0xdfd6cf31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3976cf39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf76cf3e, SUC_lexicon_tags_0 },
    { 0x2d96cf3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9336cf41, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4af6cf43, SUC_lexicon_tags_84 },
    { 0xcd16cf43, SUC_lexicon_tags_38 },
    { 0xcab6cf45, SUC_lexicon_tags_45 },
    { 0xe8f6cf46, SUC_lexicon_tags_7 },
    { 0x9f36cf47, SUC_lexicon_tags_8 },
    { 0x1736cf47, SUC_lexicon_tags_87 },
    { 0xd6d6cf49, SUC_lexicon_tags_17 },
    { 0xb616cf4a, SUC_lexicon_tags_7 },
    { 0x14d6cf4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8276cf51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d56cf53, SUC_lexicon_tags_12 },
    { 0xad6cf54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4876cf56, SUC_lexicon_tags_37 },
    { 0xfa76cf57, SUC_lexicon_tags_17 },
    { 0x3316cf58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe116cf5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5b6cf5e, SUC_lexicon_tags_37 },
    { 0x3056cf5e, SUC_lexicon_tags_12 },
    { 0x1dd6cf5e, SUC_lexicon_tags_40 },
    { 0xee16cf61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6cf6b, SUC_lexicon_tags_63 },
    { 0x40f6cf6c, SUC_lexicon_tags_1 },
    { 0x1b6cf6d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a16cf70, SUC_lexicon_tags_22 },
    { 0xd656cf71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd6cf76, SUC_lexicon_tags_17 },
    { 0x53b6cf77, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x3916cf79, SUC_lexicon_tags_19 },
    { 0xae16cf7a, SUC_lexicon_tags_45 },
    { 0x1596cf7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3b6cf7d, SUC_lexicon_tags_26 },
    { 0x48f6cf7e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc3b6cf80, SUC_lexicon_tags_33 },
    { 0x6156cf80, SUC_lexicon_tags_14 },
    { 0xd2f6cf81, SUC_lexicon_tags_45 },
    { 0xb976cf81, SUC_lexicon_tags_37 },
    { 0xce56cf81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b6cf8b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x30b6cf8d, SUC_lexicon_tags_44 },
    { 0x79b6cf8d, SUC_lexicon_tags_37 },
    { 0xb4f6cf8f, SUC_lexicon_tags_15 },
    { 0xce16cf90, SUC_lexicon_tags_21 },
    { 0x9796cf91, SUC_lexicon_tags_11 },
    { 0xa256cf90, SUC_lexicon_tags_15 },
    { 0x6f96cf93, SUC_lexicon_tags_4 },
    { 0x7536cf93, SUC_lexicon_tags_71 },
    { 0x5596cf91, SUC_lexicon_tags_27 },
    { 0xa4b6cf96, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xac76cf99, SUC_lexicon_tags_38 },
    { 0xa016cf99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d56cf9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad6cfa5, SUC_lexicon_tags_2 },
    { 0xa836cfa5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1176cfaa, SUC_lexicon_tags_15 },
    { 0xd96cfab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4236cfb0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdd96cfb2, SUC_lexicon_tags_57 },
    { 0x5496cfb3, SUC_lexicon_tags_11 },
    { 0x79b6cfb2, SUC_lexicon_tags_46 },
    { 0x2a96cfb5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb536cfb7, SUC_lexicon_tags_7 },
    { 0xd4b6cfb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3216cfbb, SUC_lexicon_tags_5 },
    { 0x8b16cfbb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xad6cfbf, SUC_lexicon_tags_84 },
    { 0x1776cfc0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcc56cfc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1316cfc4, SUC_lexicon_tags_31 },
    { 0x5376cfc5, SUC_lexicon_tags_15 },
    { 0xa736cfc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb376cfc9, SUC_lexicon_tags_14 },
    { 0xe96cfca, SUC_lexicon_tags_66 },
    { 0xb416cfca, SUC_lexicon_tags_19 },
    { 0x3376cfcb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d6cfd0, SUC_lexicon_tags_37 },
    { 0x7b16cfd1, SUC_lexicon_tags_37 },
    { 0x8cf6cfd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd596cfd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa996cfd7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d6cfdd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc76cfe3, SUC_lexicon_tags_28 },
    { 0xe0f6cfe4, SUC_lexicon_tags_1115 },
    { 0x2736cfe5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6356cfe7, SUC_lexicon_tags_27 },
    { 0xdc36cfe8, SUC_lexicon_tags_48 },
    { 0xbd96cfe9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x716cfec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b76cfef, SUC_lexicon_tags_25 },
    { 0xa976cff0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e76cff6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc276cff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5156cffb, SUC_lexicon_tags_66 },
    { 0xbb16cffb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf516cfff, SUC_lexicon_tags_29 },
    { 0x7ad6cfff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa16d004, SUC_lexicon_tags_40 },
    { 0x4576d005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc376d008, SUC_lexicon_tags_65 },
    { 0x9076d009, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c96d00d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a36d01f, SUC_lexicon_tags_0 },
    { 0x95f6d020, SUC_lexicon_tags_12 },
    { 0x5416d021, SUC_lexicon_tags_87 },
    { 0x3016d022, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd616d024, SUC_lexicon_tags_1 },
    { 0xb7d6d025, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9036d02a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb6d02e, SUC_lexicon_tags_14 },
    { 0xe596d02f, SUC_lexicon_tags_33 },
    { 0x7676d030, SUC_lexicon_tags_38 },
    { 0xcbd6d02e, SUC_lexicon_tags_37 },
    { 0x6b16d031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc816d035, SUC_lexicon_tags_53 },
    { 0x7116d036, SUC_lexicon_tags_2 },
    { 0xbb6d037, SUC_lexicon_tags_0 },
    { 0x3ff6d036, SUC_lexicon_tags_7 },
    { 0x1936d036, SUC_lexicon_tags_30 },
    { 0x79d6d03a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xeb76d03c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab6d040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a76d045, SUC_lexicon_tags_7 },
    { 0x1a76d046, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1f6d048, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f6d04c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb6d050, SUC_lexicon_tags_9 },
    { 0x9a56d051, SUC_lexicon_tags_35 },
    { 0x7c56d050, SUC_lexicon_tags_46 },
    { 0x2d56d050, SUC_lexicon_tags_15 },
    { 0xe996d052, SUC_lexicon_tags_2 },
    { 0x13f6d055, SUC_lexicon_tags_42 },
    { 0xfdf6d056, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8736d05f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d6d062, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1136d064, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x7f36d066, SUC_lexicon_tags_25 },
    { 0x24b6d067, SUC_lexicon_tags_26 },
    { 0x1216d068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2416d071, SUC_lexicon_tags_15 },
    { 0x4d76d072, SUC_lexicon_tags_0 },
    { 0xb276d073, SUC_lexicon_tags_17 },
    { 0x6436d074, SUC_lexicon_tags_37 },
    { 0x56d071, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d6d079, SUC_lexicon_tags_551 },
    { 0, NULL },
    { 0x3496d07b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf156d080, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2316d086, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb6d089, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb6d08d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab6d090, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d6d097, SUC_lexicon_tags_60 },
    { 0x5b76d098, SUC_lexicon_tags_11 },
    { 0xee76d098, SUC_lexicon_tags_61 },
    { 0xd016d097, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe116d09c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb6d0a9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x136d0ac, SUC_lexicon_tags_18 },
    { 0xeeb6d0ad, SUC_lexicon_tags_87 },
    { 0x90b6d0ac, SUC_lexicon_tags_30 },
    { 0x6f56d0af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f6d0b3, SUC_lexicon_tags_33 },
    { 0x4c96d0b4, SUC_lexicon_tags_37 },
    { 0x36f6d0b3, SUC_lexicon_tags_33 },
    { 0xb136d0b3, SUC_lexicon_tags_23 },
    { 0x7b36d0b7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f6d0ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa676d0be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87d6d0c0, SUC_lexicon_tags_17 },
    { 0xd796d0c0, SUC_lexicon_tags_152 },
    { 0xaf16d0c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcdb6d0c4, SUC_lexicon_tags_7 },
    { 0x9bf6d0c5, SUC_lexicon_tags_33 },
    { 0x66f6d0c6, SUC_lexicon_tags_37 },
    { 0x4396d0c7, SUC_lexicon_tags_60 },
    { 0x6176d0c5, SUC_lexicon_tags_17 },
    { 0x1a36d0c7, SUC_lexicon_tags_2 },
    { 0xff76d0c9, SUC_lexicon_tags_84 },
    { 0x6a36d0c5, SUC_lexicon_tags_60 },
    { 0xd616d0cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8696d0d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa656d0d4, SUC_lexicon_tags_31 },
    { 0x3136d0d4, SUC_lexicon_tags_8 },
    { 0x3976d0d6, SUC_lexicon_tags_37 },
    { 0x8f6d0d7, SUC_lexicon_tags_23 },
    { 0xd616d0d8, SUC_lexicon_tags_15 },
    { 0xc4f6d0d8, SUC_lexicon_tags_63 },
    { 0x6216d0d6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb596d0dc, SUC_lexicon_tags_11 },
    { 0x1636d0dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4496d0df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad6d0e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d36d0ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd936d0ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa96d0f0, SUC_lexicon_tags_23 },
    { 0x4d56d0f1, SUC_lexicon_tags_14 },
    { 0x1836d0f0, SUC_lexicon_tags_63 },
    { 0x7056d0f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8596d0f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6456d0f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe756d0fb, SUC_lexicon_tags_11 },
    { 0xfc96d0fc, SUC_lexicon_tags_39 },
    { 0x82d6d0fb, SUC_lexicon_tags_64 },
    { 0x1696d0fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f6d102, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x12d6d104, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x716d106, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2a36d108, SUC_lexicon_tags_51 },
    { 0x7c36d108, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f6d10c, SUC_lexicon_tags_12 },
    { 0xc6d6d10d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b6d110, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d96d112, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc56d116, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xe556d118, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe396d11c, SUC_lexicon_tags_15 },
    { 0xe1b6d11d, SUC_lexicon_tags_49 },
    { 0x42d6d11e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x616d125, SUC_lexicon_tags_37 },
    { 0x8f6d126, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd76d12a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4416d12d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3116d12f, SUC_lexicon_tags_60 },
    { 0xab96d130, SUC_lexicon_tags_16 },
    { 0xf596d12f, SUC_lexicon_tags_12 },
    { 0xb996d130, SUC_lexicon_tags_16 },
    { 0x4d56d133, SUC_lexicon_tags_25 },
    { 0xfd16d134, SUC_lexicon_tags_7 },
    { 0x5d16d135, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x8156d138, SUC_lexicon_tags_1 },
    { 0x3356d139, SUC_lexicon_tags_15 },
    { 0x7416d139, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b6d13d, SUC_lexicon_tags_25 },
    { 0x4156d13e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5d6d140, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7276d148, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf376d14c, SUC_lexicon_tags_12 },
    { 0xae76d14d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1556d150, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d6d156, SUC_lexicon_tags_2 },
    { 0x4356d157, SUC_lexicon_tags_39 },
    { 0x6576d158, SUC_lexicon_tags_25 },
    { 0x1dd6d158, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x9f96d15b, SUC_lexicon_tags_7 },
    { 0x2ad6d15c, SUC_lexicon_tags_1 },
    { 0x21f6d15c, SUC_lexicon_tags_19 },
    { 0x99b6d15e, SUC_lexicon_tags_33 },
    { 0x99d6d15f, SUC_lexicon_tags_37 },
    { 0xaf96d15c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b56d165, SUC_lexicon_tags_7 },
    { 0x8176d166, SUC_lexicon_tags_167 },
    { 0xf056d167, SUC_lexicon_tags_26 },
    { 0xa596d166, SUC_lexicon_tags_23 },
    { 0xa196d169, SUC_lexicon_tags_14 },
    { 0xf036d16a, SUC_lexicon_tags_17 },
    { 0xb556d169, SUC_lexicon_tags_12 },
    { 0xf956d16c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e16d174, SUC_lexicon_tags_1 },
    { 0xf796d175, SUC_lexicon_tags_33 },
    { 0x5896d176, SUC_lexicon_tags_25 },
    { 0xa4f6d175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b6d183, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe716d185, SUC_lexicon_tags_60 },
    { 0x9856d186, SUC_lexicon_tags_11 },
    { 0x5556d185, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f96d189, SUC_lexicon_tags_47 },
    { 0xac76d18a, SUC_lexicon_tags_38 },
    { 0xc3f6d18b, SUC_lexicon_tags_55 },
    { 0xa096d18a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8376d194, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x1396d196, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2316d198, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d6d19c, SUC_lexicon_tags_7 },
    { 0x3076d19d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4696d19f, SUC_lexicon_tags_37 },
    { 0xcd96d19f, SUC_lexicon_tags_87 },
    { 0x4196d1a1, SUC_lexicon_tags_7 },
    { 0x4536d1a2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b6d1a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b6d1ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4336d1b6, SUC_lexicon_tags_66 },
    { 0x3f36d1b7, SUC_lexicon_tags_1 },
    { 0x19f6d1b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b6d1be, SUC_lexicon_tags_15 },
    { 0x58b6d1be, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7116d1c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9816d1c7, SUC_lexicon_tags_60 },
    { 0x8936d1c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db6d1ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d6d1d1, SUC_lexicon_tags_60 },
    { 0xc776d1d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9516d1d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af6d1da, SUC_lexicon_tags_63 },
    { 0x7bd6d1db, SUC_lexicon_tags_8 },
    { 0x5ff6d1dc, SUC_lexicon_tags_1 },
    { 0xcfd6d1dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d96d1df, SUC_lexicon_tags_7 },
    { 0x8116d1e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d36d1e3, SUC_lexicon_tags_7 },
    { 0xc676d1e4, SUC_lexicon_tags_15 },
    { 0x1d6d1e5, SUC_lexicon_tags_23 },
    { 0x7856d1e4, SUC_lexicon_tags_10 },
    { 0x5cf6d1e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c56d1ea, SUC_lexicon_tags_46 },
    { 0xd356d1eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f6d1ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d16d1f1, SUC_lexicon_tags_10 },
    { 0x9c16d1f1, SUC_lexicon_tags_61 },
    { 0x7776d1f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbef6d1f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe516d1fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x996d200, SUC_lexicon_tags_15 },
    { 0xb2f6d201, SUC_lexicon_tags_38 },
    { 0xfeb6d202, SUC_lexicon_tags_34 },
    { 0xd596d202, SUC_lexicon_tags_25 },
    { 0x99d6d204, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2376d206, SUC_lexicon_tags_15 },
    { 0x4156d206, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83b6d209, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d20e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d56d210, SUC_lexicon_tags_37 },
    { 0xb216d210, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9296d217, SUC_lexicon_tags_12 },
    { 0x956d218, SUC_lexicon_tags_33 },
    { 0x5eb6d219, SUC_lexicon_tags_19 },
    { 0xa96d219, SUC_lexicon_tags_35 },
    { 0x9016d21a, SUC_lexicon_tags_25 },
    { 0x2c56d21c, SUC_lexicon_tags_4 },
    { 0x3ed6d21c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af6d224, SUC_lexicon_tags_10 },
    { 0x29f6d225, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b36d22a, SUC_lexicon_tags_5 },
    { 0xc2f6d22b, SUC_lexicon_tags_49 },
    { 0x8496d22c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8476d233, SUC_lexicon_tags_14 },
    { 0x2376d234, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36f6d236, SUC_lexicon_tags_7 },
    { 0x70b6d236, SUC_lexicon_tags_31 },
    { 0x1c56d236, SUC_lexicon_tags_84 },
    { 0x8c36d236, SUC_lexicon_tags_37 },
    { 0xf3b6d23a, SUC_lexicon_tags_37 },
    { 0x6a96d23b, SUC_lexicon_tags_37 },
    { 0xc236d23c, SUC_lexicon_tags_7 },
    { 0x5f76d23d, SUC_lexicon_tags_37 },
    { 0x9836d23e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f96d244, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4e96d246, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4396d248, SUC_lexicon_tags_2 },
    { 0xfd36d248, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33b6d24b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd6d250, SUC_lexicon_tags_17 },
    { 0xb496d251, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f6d254, SUC_lexicon_tags_7 },
    { 0xdf56d255, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b76d25e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc456d263, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5536d265, SUC_lexicon_tags_25 },
    { 0x3cb6d265, SUC_lexicon_tags_4 },
    { 0x2756d267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e76d26a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7196d26c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5f56d26e, SUC_lexicon_tags_17 },
    { 0xc6f6d26e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb76d271, SUC_lexicon_tags_1 },
    { 0x39d6d272, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe636d277, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5276d279, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e76d27c, SUC_lexicon_tags_15 },
    { 0xdaf6d27d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6d282, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d96d286, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd76d28c, SUC_lexicon_tags_11 },
    { 0x6216d28c, SUC_lexicon_tags_27 },
    { 0x4db6d28e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d76d291, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5876d294, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34f6d296, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22d6d298, SUC_lexicon_tags_15 },
    { 0x1d6d299, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc2f6d29b, SUC_lexicon_tags_7 },
    { 0x9636d29c, SUC_lexicon_tags_2 },
    { 0xc056d29b, SUC_lexicon_tags_11 },
    { 0xa936d29c, SUC_lexicon_tags_49 },
    { 0xac96d29e, SUC_lexicon_tags_39 },
    { 0x3336d2a0, SUC_lexicon_tags_0 },
    { 0xda76d29b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa56d2a6, SUC_lexicon_tags_25 },
    { 0x9bd6d2a6, SUC_lexicon_tags_57 },
    { 0xcf36d2a8, SUC_lexicon_tags_15 },
    { 0x5656d2a6, SUC_lexicon_tags_25 },
    { 0xb736d2aa, SUC_lexicon_tags_2 },
    { 0xa836d2ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb6d2ae, SUC_lexicon_tags_30 },
    { 0x7836d2af, SUC_lexicon_tags_10 },
    { 0x8956d2b0, SUC_lexicon_tags_77 },
    { 0x6fb6d2af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f6d2b5, SUC_lexicon_tags_47 },
    { 0xda76d2b5, SUC_lexicon_tags_19 },
    { 0x1576d2b7, SUC_lexicon_tags_12 },
    { 0xd7f6d2b8, SUC_lexicon_tags_51 },
    { 0xe456d2b9, SUC_lexicon_tags_1 },
    { 0x20d6d2b6, SUC_lexicon_tags_28 },
    { 0xc836d2bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b6d2c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaeb6d2c5, SUC_lexicon_tags_56 },
    { 0x4d6d2c6, SUC_lexicon_tags_17 },
    { 0xd236d2c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e16d2c9, SUC_lexicon_tags_45 },
    { 0x41b6d2c9, SUC_lexicon_tags_10 },
    { 0xeb16d2cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6556d2d0, SUC_lexicon_tags_7 },
    { 0x7e76d2d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3df6d2d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b96d2d6, SUC_lexicon_tags_13 },
    { 0x2b56d2d7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xb816d2da, SUC_lexicon_tags_30 },
    { 0xc116d2db, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe836d2de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f6d2e2, SUC_lexicon_tags_6 },
    { 0xbfb6d2e3, SUC_lexicon_tags_40 },
    { 0xea76d2e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa376d2e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a16d2ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2d6d2ec, SUC_lexicon_tags_45 },
    { 0x60b6d2ec, SUC_lexicon_tags_4 },
    { 0x7bf6d2ec, SUC_lexicon_tags_7 },
    { 0xedf6d2ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc56d2f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd6d2fd, SUC_lexicon_tags_33 },
    { 0xaeb6d2fd, SUC_lexicon_tags_25 },
    { 0x1436d2ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e16d302, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16d307, SUC_lexicon_tags_37 },
    { 0xccb6d307, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b56d30c, SUC_lexicon_tags_46 },
    { 0xa716d30d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9156d310, SUC_lexicon_tags_12 },
    { 0x7076d310, SUC_lexicon_tags_17 },
    { 0x4a96d311, SUC_lexicon_tags_39 },
    { 0x4f56d313, SUC_lexicon_tags_7 },
    { 0xbf76d314, SUC_lexicon_tags_26 },
    { 0xa616d313, SUC_lexicon_tags_11 },
    { 0x8e56d316, SUC_lexicon_tags_26 },
    { 0xec76d310, SUC_lexicon_tags_26 },
    { 0x3e36d318, SUC_lexicon_tags_27 },
    { 0x1916d310, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb196d31f, SUC_lexicon_tags_37 },
    { 0xbb6d31f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x956d322, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d6d325, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b6d32e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd6d331, SUC_lexicon_tags_854 },
    { 0x2f56d332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc756d335, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26f6d337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df6d343, SUC_lexicon_tags_2 },
    { 0x2d56d344, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5616d347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2556d34b, SUC_lexicon_tags_28 },
    { 0x5056d34b, SUC_lexicon_tags_31 },
    { 0xe696d34b, SUC_lexicon_tags_2 },
    { 0xb916d34e, SUC_lexicon_tags_24 },
    { 0x3116d34f, SUC_lexicon_tags_25 },
    { 0xddd6d34e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d6d355, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b6d35a, SUC_lexicon_tags_60 },
    { 0xc3d6d35a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff6d368, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f6d36b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2656d36d, SUC_lexicon_tags_15 },
    { 0xfb56d36e, SUC_lexicon_tags_23 },
    { 0xf996d36f, SUC_lexicon_tags_25 },
    { 0x83d6d36f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55f6d372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85b6d374, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4c76d376, SUC_lexicon_tags_37 },
    { 0x1636d377, SUC_lexicon_tags_12 },
    { 0x316d378, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd76d37b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xff56d37e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7396d382, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8d6d384, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e16d386, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe676d392, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc176d397, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3076d39a, SUC_lexicon_tags_37 },
    { 0xa016d39b, SUC_lexicon_tags_27 },
    { 0xb816d39c, SUC_lexicon_tags_34 },
    { 0x4ed6d39d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b6d3a0, SUC_lexicon_tags_11 },
    { 0x2f76d3a0, SUC_lexicon_tags_127 },
    { 0xaf56d3a2, SUC_lexicon_tags_33 },
    { 0xc436d3a2, SUC_lexicon_tags_4 },
    { 0x49d6d3a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe96d3a6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb416d3a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbf6d3aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1776d3ad, SUC_lexicon_tags_14 },
    { 0xc896d3ae, SUC_lexicon_tags_12 },
    { 0x36d3ae, SUC_lexicon_tags_14 },
    { 0x2216d3b0, SUC_lexicon_tags_47 },
    { 0x13f6d3b1, SUC_lexicon_tags_47 },
    { 0x4376d3af, SUC_lexicon_tags_7 },
    { 0xc1d6d3b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb856d3b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a96d3ba, SUC_lexicon_tags_23 },
    { 0x4c96d3bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe656d3c1, SUC_lexicon_tags_1 },
    { 0x2456d3c2, SUC_lexicon_tags_66 },
    { 0x91b6d3c1, SUC_lexicon_tags_12 },
    { 0x4eb6d3c2, SUC_lexicon_tags_25 },
    { 0xb116d3c5, SUC_lexicon_tags_6 },
    { 0xf3d6d3c6, SUC_lexicon_tags_11 },
    { 0xd6f6d3c5, SUC_lexicon_tags_1 },
    { 0xdcd6d3c8, SUC_lexicon_tags_11 },
    { 0x24b6d3c9, SUC_lexicon_tags_16 },
    { 0xd2f6d3ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9776d3d0, SUC_lexicon_tags_33 },
    { 0xf036d3d1, SUC_lexicon_tags_12 },
    { 0xeab6d3d1, SUC_lexicon_tags_12 },
    { 0x2296d3d3, SUC_lexicon_tags_29 },
    { 0x5ff6d3d4, SUC_lexicon_tags_45 },
    { 0x5f56d3d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f6d3dc, SUC_lexicon_tags_49 },
    { 0x7056d3dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xef56d3df, SUC_lexicon_tags_17 },
    { 0xdb16d3e0, SUC_lexicon_tags_65 },
    { 0x88d6d3e0, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad6d3e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd36d3e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa876d3e8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d16d3eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2876d3f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd6d3f8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf536d3fa, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9316d402, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd096d405, SUC_lexicon_tags_12 },
    { 0x95f6d405, SUC_lexicon_tags_57 },
    { 0x7736d407, SUC_lexicon_tags_23 },
    { 0xdbd6d407, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7ab6d40a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc96d40c, SUC_lexicon_tags_39 },
    { 0xa2f6d40d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc616d410, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc36d412, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d6d418, SUC_lexicon_tags_16 },
    { 0x6d76d418, SUC_lexicon_tags_19 },
    { 0x5016d419, SUC_lexicon_tags_31 },
    { 0x74d6d41a, SUC_lexicon_tags_17 },
    { 0x2096d41c, SUC_lexicon_tags_16 },
    { 0xad16d41a, SUC_lexicon_tags_17 },
    { 0xd1f6d41e, SUC_lexicon_tags_55 },
    { 0xf496d41b, SUC_lexicon_tags_25 },
    { 0x9956d420, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf676d42d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf336d430, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9096d433, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7876d435, SUC_lexicon_tags_2 },
    { 0x9616d435, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1556d43d, SUC_lexicon_tags_11 },
    { 0x5856d43e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbf6d440, SUC_lexicon_tags_45 },
    { 0xb6d6d441, SUC_lexicon_tags_25 },
    { 0xb276d442, SUC_lexicon_tags_12 },
    { 0x3ff6d441, SUC_lexicon_tags_2 },
    { 0xe316d441, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b6d447, SUC_lexicon_tags_65 },
    { 0xf996d448, SUC_lexicon_tags_2 },
    { 0x5c56d448, SUC_lexicon_tags_43 },
    { 0xae76d44a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc636d44c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc796d451, SUC_lexicon_tags_1 },
    { 0x6ef6d452, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516d457, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1b6d459, SUC_lexicon_tags_25 },
    { 0x4196d45a, SUC_lexicon_tags_15 },
    { 0xf876d45b, SUC_lexicon_tags_27 },
    { 0x8596d459, SUC_lexicon_tags_49 },
    { 0x8c16d45d, SUC_lexicon_tags_25 },
    { 0xf2b6d45e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5896d462, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd16d467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7176d469, SUC_lexicon_tags_17 },
    { 0x9596d46a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9396d46c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e16d472, SUC_lexicon_tags_0 },
    { 0xfb6d473, SUC_lexicon_tags_2 },
    { 0x5fb6d474, SUC_lexicon_tags_17 },
    { 0x3316d475, SUC_lexicon_tags_35 },
    { 0xd5f6d476, SUC_lexicon_tags_68 },
    { 0x4cb6d477, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2576d479, SUC_lexicon_tags_4 },
    { 0xd536d47a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c96d48a, SUC_lexicon_tags_85 },
    { 0xfeb6d48b, SUC_lexicon_tags_25 },
    { 0x5d56d48c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd016d491, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed6d498, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb96d49f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ff6d4a1, SUC_lexicon_tags_37 },
    { 0x99b6d4a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15b6d4a4, SUC_lexicon_tags_120 },
    { 0xcad6d4a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9f96d4a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8576d4a9, SUC_lexicon_tags_16 },
    { 0xc236d4aa, SUC_lexicon_tags_15 },
    { 0x41f6d4ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd6d4b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2616d4b6, SUC_lexicon_tags_11 },
    { 0x7bf6d4b7, SUC_lexicon_tags_8 },
    { 0x3376d4b8, SUC_lexicon_tags_29 },
    { 0x9f76d4b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d16d4bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7136d4c0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4856d4c8, SUC_lexicon_tags_10 },
    { 0x33d6d4c9, SUC_lexicon_tags_38 },
    { 0xe836d4ca, SUC_lexicon_tags_49 },
    { 0xff96d4ca, SUC_lexicon_tags_14 },
    { 0xbab6d4c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f6d4d1, SUC_lexicon_tags_15 },
    { 0x3276d4d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5156d4d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd676d4d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d96d4d8, SUC_lexicon_tags_15 },
    { 0x1456d4d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4756d4df, SUC_lexicon_tags_26 },
    { 0x18b6d4df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8576d4e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe496d4e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df6d4e7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b6d4ed, SUC_lexicon_tags_7 },
    { 0x8c16d4ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6536d4f5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1996d4fb, SUC_lexicon_tags_8 },
    { 0x35f6d4fc, SUC_lexicon_tags_29 },
    { 0x6a56d4fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf56d501, SUC_lexicon_tags_39 },
    { 0x8bd6d502, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b6d505, SUC_lexicon_tags_15 },
    { 0xe3f6d506, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f56d508, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa936d50d, SUC_lexicon_tags_19 },
    { 0x3136d50e, SUC_lexicon_tags_7 },
    { 0x9c16d50f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d36d511, SUC_lexicon_tags_2 },
    { 0x7b96d511, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f6d516, SUC_lexicon_tags_15 },
    { 0x58b6d517, SUC_lexicon_tags_15 },
    { 0x7456d518, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d6d51b, SUC_lexicon_tags_17 },
    { 0x98d6d51c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2716d520, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa036d525, SUC_lexicon_tags_3 },
    { 0x1d16d526, SUC_lexicon_tags_17 },
    { 0x3956d527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7156d529, SUC_lexicon_tags_12 },
    { 0x74d6d52a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46f6d52c, SUC_lexicon_tags_23 },
    { 0xa936d52c, SUC_lexicon_tags_38 },
    { 0x8176d52e, SUC_lexicon_tags_25 },
    { 0x6516d52f, SUC_lexicon_tags_46 },
    { 0x3e36d52f, SUC_lexicon_tags_14 },
    { 0xf736d52d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa556d533, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x77f6d535, SUC_lexicon_tags_12 },
    { 0x5f6d535, SUC_lexicon_tags_8 },
    { 0x8936d535, SUC_lexicon_tags_8 },
    { 0x8336d538, SUC_lexicon_tags_39 },
    { 0x696d535, SUC_lexicon_tags_37 },
    { 0x2236d538, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x81d6d53c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6176d540, SUC_lexicon_tags_7 },
    { 0x6d76d541, SUC_lexicon_tags_23 },
    { 0x3556d542, SUC_lexicon_tags_17 },
    { 0xf236d543, SUC_lexicon_tags_37 },
    { 0x9ab6d544, SUC_lexicon_tags_45 },
    { 0x88b6d545, SUC_lexicon_tags_15 },
    { 0x94d6d543, SUC_lexicon_tags_6 },
    { 0x6ad6d541, SUC_lexicon_tags_53 },
    { 0xdd16d548, SUC_lexicon_tags_37 },
    { 0xce36d549, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba76d54b, SUC_lexicon_tags_37 },
    { 0x8056d54c, SUC_lexicon_tags_60 },
    { 0xabf6d54c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4316d553, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc076d559, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3136d561, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x55d6d563, SUC_lexicon_tags_19 },
    { 0x8656d564, SUC_lexicon_tags_17 },
    { 0xf696d563, SUC_lexicon_tags_55 },
    { 0x8116d565, SUC_lexicon_tags_27 },
    { 0xd7b6d564, SUC_lexicon_tags_15 },
    { 0x9216d568, SUC_lexicon_tags_17 },
    { 0x7c76d567, SUC_lexicon_tags_7 },
    { 0xde36d565, SUC_lexicon_tags_15 },
    { 0xd876d565, SUC_lexicon_tags_17 },
    { 0x476d566, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c76d570, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b56d574, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x2ad6d576, SUC_lexicon_tags_11 },
    { 0x1e16d576, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4276d57d, SUC_lexicon_tags_29 },
    { 0xb7f6d57e, SUC_lexicon_tags_91 },
    { 0x17f6d57d, SUC_lexicon_tags_12 },
    { 0xe5f6d57d, SUC_lexicon_tags_11 },
    { 0xfbb6d581, SUC_lexicon_tags_47 },
    { 0x1556d581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b6d585, SUC_lexicon_tags_1 },
    { 0x7196d586, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d6d58b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea56d59a, SUC_lexicon_tags_25 },
    { 0xac96d59b, SUC_lexicon_tags_9 },
    { 0x9956d59b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b76d59e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4536d5a0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8876d5a2, SUC_lexicon_tags_16 },
    { 0xc6d6d5a3, SUC_lexicon_tags_23 },
    { 0x9bb6d5a4, SUC_lexicon_tags_1 },
    { 0x1676d5a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28b6d5a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9396d5a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3996d5ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe816d5b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe176d5b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef6d5bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd56d5bf, SUC_lexicon_tags_17 },
    { 0xad76d5c0, SUC_lexicon_tags_17 },
    { 0x7856d5c0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5f16d5c3, SUC_lexicon_tags_49 },
    { 0x50d6d5c4, SUC_lexicon_tags_43 },
    { 0xdcb6d5c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d96d5c8, SUC_lexicon_tags_16 },
    { 0xa936d5c9, SUC_lexicon_tags_17 },
    { 0x816d5c8, SUC_lexicon_tags_17 },
    { 0xddd6d5c8, SUC_lexicon_tags_25 },
    { 0x38b6d5c8, SUC_lexicon_tags_0 },
    { 0xdf96d5cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb56d5d1, SUC_lexicon_tags_17 },
    { 0x56b6d5d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5eb6d5d4, SUC_lexicon_tags_12 },
    { 0xb276d5d5, SUC_lexicon_tags_47 },
    { 0x54d6d5d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc616d5d8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x1f96d5da, SUC_lexicon_tags_30 },
    { 0xae76d5db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5556d5dd, SUC_lexicon_tags_1 },
    { 0x9876d5de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a76d5e2, SUC_lexicon_tags_1586 },
    { 0x9b76d5e3, SUC_lexicon_tags_16 },
    { 0xba56d5e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e96d5e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf7b6d5ea, SUC_lexicon_tags_15 },
    { 0xefd6d5eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3076d5ee, SUC_lexicon_tags_25 },
    { 0x1b56d5ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f6d5f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d6d606, SUC_lexicon_tags_25 },
    { 0xfad6d606, SUC_lexicon_tags_30 },
    { 0xc36d608, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f6d60b, SUC_lexicon_tags_38 },
    { 0x1056d60c, SUC_lexicon_tags_37 },
    { 0x12f6d60d, SUC_lexicon_tags_47 },
    { 0xb2f6d60c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d6d612, SUC_lexicon_tags_55 },
    { 0x3ef6d613, SUC_lexicon_tags_25 },
    { 0xcc56d614, SUC_lexicon_tags_33 },
    { 0xc2d6d613, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6876d617, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xdc16d619, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7696d61b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96d621, SUC_lexicon_tags_17 },
    { 0xedb6d621, SUC_lexicon_tags_37 },
    { 0xc4d6d623, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d6d629, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5996d62c, SUC_lexicon_tags_23 },
    { 0xa296d62d, SUC_lexicon_tags_19 },
    { 0x2c16d62c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7ff6d630, SUC_lexicon_tags_224 },
    { 0x87d6d630, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xefd6d633, SUC_lexicon_tags_23 },
    { 0x39f6d634, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7996d63e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b6d64b, SUC_lexicon_tags_68 },
    { 0xd396d64c, SUC_lexicon_tags_17 },
    { 0x61b6d64d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1736d651, SUC_lexicon_tags_38 },
    { 0xca56d652, SUC_lexicon_tags_10 },
    { 0x5c56d653, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d656, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f6d65c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd6d663, SUC_lexicon_tags_37 },
    { 0x6f96d664, SUC_lexicon_tags_56 },
    { 0xa856d665, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a56d668, SUC_lexicon_tags_37 },
    { 0x5436d669, SUC_lexicon_tags_14 },
    { 0xd816d669, SUC_lexicon_tags_25 },
    { 0x74f6d669, SUC_lexicon_tags_14 },
    { 0xc136d66b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb796d671, SUC_lexicon_tags_11 },
    { 0xd4b6d672, SUC_lexicon_tags_2 },
    { 0x9876d671, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b96d676, SUC_lexicon_tags_12 },
    { 0xe96d677, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd96d67e, SUC_lexicon_tags_9 },
    { 0x3e56d67e, SUC_lexicon_tags_11 },
    { 0xf736d67f, SUC_lexicon_tags_7 },
    { 0x376d681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb076d683, SUC_lexicon_tags_40 },
    { 0x97d6d683, SUC_lexicon_tags_32 },
    { 0x3936d685, SUC_lexicon_tags_63 },
    { 0xdd36d686, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1156d689, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b6d68d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf936d697, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa356d69b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf3f6d69d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d6d6a8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2176d6ad, SUC_lexicon_tags_9 },
    { 0xef56d6ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9e16d6b0, SUC_lexicon_tags_37 },
    { 0xc936d6b0, SUC_lexicon_tags_5 },
    { 0x7296d6b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b16d6b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5236d6b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db6d6bc, SUC_lexicon_tags_65 },
    { 0xe856d6bd, SUC_lexicon_tags_37 },
    { 0x3b16d6be, SUC_lexicon_tags_38 },
    { 0xf5d6d6bc, SUC_lexicon_tags_359 },
    { 0xa716d6c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd756d6c3, SUC_lexicon_tags_23 },
    { 0x5776d6c3, SUC_lexicon_tags_8 },
    { 0x25f6d6c3, SUC_lexicon_tags_14 },
    { 0x8556d6c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6036d6ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3016d6cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf6d6d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6d6e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa876d6e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af6d6ef, SUC_lexicon_tags_17 },
    { 0xef6d6f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfd56d6f2, SUC_lexicon_tags_11 },
    { 0xb6b6d6f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb136d6f6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d56d6fd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbe76d6ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b16d702, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46d6d704, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a96d707, SUC_lexicon_tags_4 },
    { 0x37f6d708, SUC_lexicon_tags_14 },
    { 0x6296d707, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa256d70c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe256d710, SUC_lexicon_tags_29 },
    { 0xfb16d710, SUC_lexicon_tags_6 },
    { 0x4076d712, SUC_lexicon_tags_118 },
    { 0xc5b6d713, SUC_lexicon_tags_31 },
    { 0xd636d714, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8056d716, SUC_lexicon_tags_11 },
    { 0x4cb6d717, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe816d71b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfcd6d71d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x24f6d71f, SUC_lexicon_tags_37 },
    { 0x6c16d71f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1636d722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf176d725, SUC_lexicon_tags_2 },
    { 0x33f6d725, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb6d72b, SUC_lexicon_tags_2 },
    { 0xe676d72b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd096d732, SUC_lexicon_tags_43 },
    { 0x41f6d733, SUC_lexicon_tags_29 },
    { 0xdc76d734, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b6d737, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e16d73e, SUC_lexicon_tags_23 },
    { 0x4996d73f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7616d741, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaeb6d743, SUC_lexicon_tags_33 },
    { 0x4276d743, SUC_lexicon_tags_9 },
    { 0xf8f6d745, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6d748, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x396d74a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6036d74c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe316d74f, SUC_lexicon_tags_27 },
    { 0x2656d750, SUC_lexicon_tags_37 },
    { 0xd256d751, SUC_lexicon_tags_27 },
    { 0x7976d752, SUC_lexicon_tags_14 },
    { 0xb1b6d751, SUC_lexicon_tags_60 },
    { 0xe0b6d754, SUC_lexicon_tags_61 },
    { 0x7ef6d750, SUC_lexicon_tags_1 },
    { 0xf916d751, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f6d759, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef6d75e, SUC_lexicon_tags_37 },
    { 0x5636d75f, SUC_lexicon_tags_4 },
    { 0x2856d760, SUC_lexicon_tags_39 },
    { 0x6876d761, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7d16d763, SUC_lexicon_tags_27 },
    { 0x1eb6d764, SUC_lexicon_tags_8 },
    { 0x55b6d765, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6116d76a, SUC_lexicon_tags_17 },
    { 0xc356d76b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa216d76e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f6d771, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac76d773, SUC_lexicon_tags_1 },
    { 0xe716d773, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4276d776, SUC_lexicon_tags_25 },
    { 0xa876d777, SUC_lexicon_tags_35 },
    { 0x5896d777, SUC_lexicon_tags_4 },
    { 0x7756d779, SUC_lexicon_tags_17 },
    { 0xfe16d777, SUC_lexicon_tags_17 },
    { 0xd1d6d77b, SUC_lexicon_tags_7 },
    { 0x4796d777, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8c96d77e, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x9596d781, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62f6d783, SUC_lexicon_tags_15 },
    { 0x4116d783, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f96d787, SUC_lexicon_tags_0 },
    { 0x34b6d788, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84d6d78a, SUC_lexicon_tags_7 },
    { 0xeb96d78a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bb6d78d, SUC_lexicon_tags_11 },
    { 0x1cf6d78e, SUC_lexicon_tags_2 },
    { 0xee56d78d, SUC_lexicon_tags_42 },
    { 0x2596d78d, SUC_lexicon_tags_28 },
    { 0x1316d78d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa336d796, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e76d799, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab16d79f, SUC_lexicon_tags_56 },
    { 0xf296d7a0, SUC_lexicon_tags_25 },
    { 0x21f6d7a1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5016d7a3, SUC_lexicon_tags_11 },
    { 0xde16d7a4, SUC_lexicon_tags_7 },
    { 0x4c16d7a5, SUC_lexicon_tags_109 },
    { 0x96b6d7a4, SUC_lexicon_tags_28 },
    { 0xc56d7a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef6d7aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7276d7ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff6d7ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc696d7b0, SUC_lexicon_tags_11 },
    { 0xb376d7b1, SUC_lexicon_tags_74 },
    { 0x3cb6d7b1, SUC_lexicon_tags_43 },
    { 0xcef6d7b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad6d7b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f6d7b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc156d7bf, SUC_lexicon_tags_12 },
    { 0xdc56d7bf, SUC_lexicon_tags_8 },
    { 0x8cb6d7c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c76d7cb, SUC_lexicon_tags_359 },
    { 0x52b6d7cc, SUC_lexicon_tags_11 },
    { 0xff36d7cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db6d7d0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed6d7dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb6d7e1, SUC_lexicon_tags_1 },
    { 0x6096d7e2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa716d7e5, SUC_lexicon_tags_2 },
    { 0x7f56d7e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9116d7eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb316d7ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd56d7f3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7356d7f5, SUC_lexicon_tags_15 },
    { 0x52f6d7f6, SUC_lexicon_tags_47 },
    { 0x6f6d7f6, SUC_lexicon_tags_0 },
    { 0x6d56d7f8, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0xa696d7fa, SUC_lexicon_tags_25 },
    { 0x7636d7fa, SUC_lexicon_tags_37 },
    { 0xc36d7fb, SUC_lexicon_tags_33 },
    { 0x4816d7fb, SUC_lexicon_tags_62 },
    { 0x6db6d7fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4436d802, SUC_lexicon_tags_37 },
    { 0x1f96d803, SUC_lexicon_tags_12 },
    { 0xd9d6d804, SUC_lexicon_tags_39 },
    { 0x6f76d803, SUC_lexicon_tags_15 },
    { 0xd476d803, SUC_lexicon_tags_46 },
    { 0x8a76d807, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5916d809, SUC_lexicon_tags_0 },
    { 0xf076d80a, SUC_lexicon_tags_25 },
    { 0xb356d80a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6d80e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e36d810, SUC_lexicon_tags_15 },
    { 0xb936d811, SUC_lexicon_tags_15 },
    { 0x2016d811, SUC_lexicon_tags_45 },
    { 0x5336d813, SUC_lexicon_tags_33 },
    { 0xf276d814, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47f6d816, SUC_lexicon_tags_49 },
    { 0xbe36d817, SUC_lexicon_tags_45 },
    { 0x5256d818, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6516d81a, SUC_lexicon_tags_1 },
    { 0xf576d81a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb6d81e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df6d824, SUC_lexicon_tags_68 },
    { 0xbf16d825, SUC_lexicon_tags_35 },
    { 0x2836d826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f6d829, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7796d82d, SUC_lexicon_tags_15 },
    { 0x2d6d82d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0f6d830, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf6d83a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e56d83c, SUC_lexicon_tags_12 },
    { 0x3dd6d83c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xae96d840, SUC_lexicon_tags_28 },
    { 0xe636d841, SUC_lexicon_tags_66 },
    { 0xbd56d842, SUC_lexicon_tags_7 },
    { 0x6496d843, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6156d845, SUC_lexicon_tags_32 },
    { 0x2916d845, SUC_lexicon_tags_14 },
    { 0xab96d847, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f6d84a, SUC_lexicon_tags_10 },
    { 0x9976d84a, SUC_lexicon_tags_63 },
    { 0x5f16d84c, SUC_lexicon_tags_4 },
    { 0xe656d84c, SUC_lexicon_tags_12 },
    { 0xa9d6d84d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb6d852, SUC_lexicon_tags_54 },
    { 0xffd6d853, SUC_lexicon_tags_42 },
    { 0xf7d6d853, SUC_lexicon_tags_40 },
    { 0x9af6d855, SUC_lexicon_tags_17 },
    { 0xaad6d853, SUC_lexicon_tags_32 },
    { 0x4896d857, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25b6d859, SUC_lexicon_tags_65 },
    { 0xc796d85a, SUC_lexicon_tags_47 },
    { 0x54f6d85b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a56d85f, SUC_lexicon_tags_0 },
    { 0xc0f6d860, SUC_lexicon_tags_17 },
    { 0x5a76d861, SUC_lexicon_tags_1 },
    { 0xd476d860, SUC_lexicon_tags_83 },
    { 0x1a36d861, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9716d865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc116d871, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f96d876, SUC_lexicon_tags_37 },
    { 0xa896d877, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9196d879, SUC_lexicon_tags_9 },
    { 0xf096d87a, SUC_lexicon_tags_11 },
    { 0x6196d87b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4656d87e, SUC_lexicon_tags_0 },
    { 0x4df6d87f, SUC_lexicon_tags_1 },
    { 0x5f6d87f, SUC_lexicon_tags_27 },
    { 0xe156d881, SUC_lexicon_tags_12 },
    { 0x88f6d882, SUC_lexicon_tags_1 },
    { 0x14b6d881, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf456d886, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbf6d888, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ad6d88a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e96d899, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67f6d89b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x96d89d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b6d8a8, SUC_lexicon_tags_7 },
    { 0xfff6d8a9, SUC_lexicon_tags_25 },
    { 0x2456d8aa, SUC_lexicon_tags_27 },
    { 0x2616d8a9, SUC_lexicon_tags_47 },
    { 0x69d6d8ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf6d8af, SUC_lexicon_tags_12 },
    { 0xfa56d8b0, SUC_lexicon_tags_68 },
    { 0x2356d8af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d6d8b7, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf6d8ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x54b6d8bc, SUC_lexicon_tags_47 },
    { 0x5f96d8bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956d8c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8256d8c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9496d8c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d6d8ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d6d8cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6e36d8cf, SUC_lexicon_tags_47 },
    { 0xced6d8d0, SUC_lexicon_tags_10 },
    { 0x6b36d8d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1076d8d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d6d8d8, SUC_lexicon_tags_33 },
    { 0xdd16d8d8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e56d8eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16d8f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8356d8f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8b6d8f6, SUC_lexicon_tags_56 },
    { 0x9cf6d8f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf716d8f9, SUC_lexicon_tags_11 },
    { 0x16f6d8f9, SUC_lexicon_tags_1 },
    { 0x4716d8fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1456d8ff, SUC_lexicon_tags_12 },
    { 0x3db6d8ff, SUC_lexicon_tags_109 },
    { 0x8d16d900, SUC_lexicon_tags_49 },
    { 0xc496d902, SUC_lexicon_tags_7 },
    { 0x9db6d903, SUC_lexicon_tags_60 },
    { 0x9156d904, SUC_lexicon_tags_2 },
    { 0x1356d904, SUC_lexicon_tags_2 },
    { 0xdb36d906, SUC_lexicon_tags_3 },
    { 0x73b6d904, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xfbd6d909, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f6d90c, SUC_lexicon_tags_45 },
    { 0x7896d90c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db6d910, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3396d912, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b6d915, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6016d918, SUC_lexicon_tags_18 },
    { 0x64f6d918, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7756d91c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d6d91f, SUC_lexicon_tags_27 },
    { 0xe8d6d920, SUC_lexicon_tags_33 },
    { 0x2636d920, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd6d927, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1136d929, SUC_lexicon_tags_607 },
    { 0x9596d929, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xabd6d92c, SUC_lexicon_tags_15 },
    { 0x7f96d92d, SUC_lexicon_tags_6 },
    { 0x99b6d92d, SUC_lexicon_tags_22 },
    { 0x30f6d92f, SUC_lexicon_tags_1 },
    { 0x28b6d930, SUC_lexicon_tags_11 },
    { 0x99d6d930, SUC_lexicon_tags_8 },
    { 0x49d6d931, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x66b6d934, SUC_lexicon_tags_11 },
    { 0xcdd6d935, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9b36d937, SUC_lexicon_tags_17 },
    { 0x6b76d938, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd436d93d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9336d940, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb96d944, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3116d946, SUC_lexicon_tags_17 },
    { 0x2016d946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3876d94b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe216d94d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7696d94f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1b6d951, SUC_lexicon_tags_23 },
    { 0x8a56d952, SUC_lexicon_tags_37 },
    { 0x6656d952, SUC_lexicon_tags_7 },
    { 0x7ef6d954, SUC_lexicon_tags_96 },
    { 0x5476d955, SUC_lexicon_tags_7 },
    { 0xf956d951, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6756d95d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4156d966, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276d96c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3456d96f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d96d973, SUC_lexicon_tags_47 },
    { 0x7c96d974, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1316d977, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd6d97c, SUC_lexicon_tags_11 },
    { 0x1cb6d97d, SUC_lexicon_tags_410 },
    { 0xf476d97e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d6d982, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb616d984, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4636d98a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5976d98c, SUC_lexicon_tags_49 },
    { 0x3496d98c, SUC_lexicon_tags_32 },
    { 0x2656d98e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b6d994, SUC_lexicon_tags_11 },
    { 0x6536d995, SUC_lexicon_tags_27 },
    { 0x1996d996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf956d998, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f6d99c, SUC_lexicon_tags_47 },
    { 0x9116d99c, SUC_lexicon_tags_17 },
    { 0x15f6d99e, SUC_lexicon_tags_11 },
    { 0xed16d99f, SUC_lexicon_tags_7 },
    { 0x2d36d99f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x356d9a4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd3d6d9a6, SUC_lexicon_tags_37 },
    { 0x9516d9a6, SUC_lexicon_tags_37 },
    { 0xfef6d9a8, SUC_lexicon_tags_0 },
    { 0x90f6d9a9, SUC_lexicon_tags_25 },
    { 0x4076d9aa, SUC_lexicon_tags_88 },
    { 0xa056d9ab, SUC_lexicon_tags_37 },
    { 0xf56d9ac, SUC_lexicon_tags_8 },
    { 0xfef6d9a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c96d9af, SUC_lexicon_tags_26 },
    { 0x1a16d9b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x16b6d9b2, SUC_lexicon_tags_37 },
    { 0x8e96d9b3, SUC_lexicon_tags_14 },
    { 0xf316d9b2, SUC_lexicon_tags_17 },
    { 0xbbf6d9b5, SUC_lexicon_tags_7 },
    { 0x8056d9b6, SUC_lexicon_tags_25 },
    { 0x3b56d9b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba36d9be, SUC_lexicon_tags_23 },
    { 0x9756d9bf, SUC_lexicon_tags_37 },
    { 0xae16d9c0, SUC_lexicon_tags_30 },
    { 0xc7f6d9c1, SUC_lexicon_tags_25 },
    { 0x98f6d9c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b6d9ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1cb6d9cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1376d9cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb6d9d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6456d9d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9676d9db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b6d9de, SUC_lexicon_tags_33 },
    { 0xe336d9df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe416d9e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5776d9e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed96d9ea, SUC_lexicon_tags_25 },
    { 0x3716d9ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb436d9ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d6d9f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa196d9f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5936d9f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfb76d9fb, SUC_lexicon_tags_1 },
    { 0x4b16d9fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef6d9ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f36da01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd6da05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa896da08, SUC_lexicon_tags_25 },
    { 0xbfd6da09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87f6da0b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe836da0f, SUC_lexicon_tags_0 },
    { 0xfbd6da10, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x276da14, SUC_lexicon_tags_5 },
    { 0x8a56da15, SUC_lexicon_tags_65 },
    { 0xd3f6da14, SUC_lexicon_tags_25 },
    { 0xe1d6da17, SUC_lexicon_tags_17 },
    { 0xd8f6da18, SUC_lexicon_tags_42 },
    { 0xe996da19, SUC_lexicon_tags_35 },
    { 0xa76da19, SUC_lexicon_tags_12 },
    { 0xb116da19, SUC_lexicon_tags_23 },
    { 0x57f6da1c, SUC_lexicon_tags_25 },
    { 0x95f6da1d, SUC_lexicon_tags_15 },
    { 0xd236da1b, SUC_lexicon_tags_34 },
    { 0xdff6da1c, SUC_lexicon_tags_30 },
    { 0x5076da14, SUC_lexicon_tags_30 },
    { 0xb596da21, SUC_lexicon_tags_32 },
    { 0x4b96da22, SUC_lexicon_tags_21 },
    { 0x6816da23, SUC_lexicon_tags_2 },
    { 0xa4d6da15, SUC_lexicon_tags_45 },
    { 0x8bd6da25, SUC_lexicon_tags_28 },
    { 0x3016da25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf6da34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc76da37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf6da39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b6da3f, SUC_lexicon_tags_11 },
    { 0x1b36da40, SUC_lexicon_tags_65 },
    { 0x2d16da41, SUC_lexicon_tags_2 },
    { 0xacd6da40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd56da45, SUC_lexicon_tags_8 },
    { 0xb236da45, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x76da49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9976da4d, SUC_lexicon_tags_63 },
    { 0xd456da4e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe096da50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x96f6da52, SUC_lexicon_tags_38 },
    { 0x956da53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5e36da55, SUC_lexicon_tags_25 },
    { 0x2916da56, SUC_lexicon_tags_23 },
    { 0x32d6da56, SUC_lexicon_tags_1 },
    { 0x536da57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8136da5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd096da5f, SUC_lexicon_tags_29 },
    { 0x7c96da60, SUC_lexicon_tags_17 },
    { 0x7216da60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb636da68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b6da6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed6da70, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b6da74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76da7c, SUC_lexicon_tags_9 },
    { 0x58d6da7c, SUC_lexicon_tags_2 },
    { 0xd6da7c, SUC_lexicon_tags_37 },
    { 0x2316da7e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc56da82, SUC_lexicon_tags_17 },
    { 0x2676da83, SUC_lexicon_tags_129 },
    { 0x4c56da83, SUC_lexicon_tags_7 },
    { 0xb8d6da85, SUC_lexicon_tags_27 },
    { 0x6356da86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed6da8b, SUC_lexicon_tags_12 },
    { 0x34b6da8c, SUC_lexicon_tags_11 },
    { 0xbaf6da8c, SUC_lexicon_tags_31 },
    { 0x2536da8e, SUC_lexicon_tags_12 },
    { 0xe2f6da8f, SUC_lexicon_tags_7 },
    { 0x6b96da90, SUC_lexicon_tags_17 },
    { 0xf936da91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef6da96, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5416da9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b6daa3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2216daa8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6d16daaa, SUC_lexicon_tags_14 },
    { 0x8b96daab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2796daaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4776dab1, SUC_lexicon_tags_33 },
    { 0x5e16dab1, SUC_lexicon_tags_37 },
    { 0x616dab2, SUC_lexicon_tags_11 },
    { 0x3af6dab1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1896dab8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad16dabc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1096dabf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab96dac5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296dac9, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f6dacc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb036dad0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb6dad8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b36dae3, SUC_lexicon_tags_25 },
    { 0x97f6dae3, SUC_lexicon_tags_15 },
    { 0x63d6dae5, SUC_lexicon_tags_25 },
    { 0xad76dae6, SUC_lexicon_tags_12 },
    { 0xef96dae7, SUC_lexicon_tags_30 },
    { 0x5cb6dae8, SUC_lexicon_tags_14 },
    { 0x6736dae9, SUC_lexicon_tags_37 },
    { 0x6096daea, SUC_lexicon_tags_9 },
    { 0x6eb6daeb, SUC_lexicon_tags_7 },
    { 0xb916daec, SUC_lexicon_tags_11 },
    { 0xaa36daec, SUC_lexicon_tags_43 },
    { 0x1236dae9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f36daf2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9116daf9, SUC_lexicon_tags_46 },
    { 0xb236daf9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa1f6dafc, SUC_lexicon_tags_65 },
    { 0x9ef6dafd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x796db00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc396db03, SUC_lexicon_tags_77 },
    { 0x9256db04, SUC_lexicon_tags_2 },
    { 0xfdf6db05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f6db0e, SUC_lexicon_tags_40 },
    { 0xe956db0e, SUC_lexicon_tags_8 },
    { 0x6516db0e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf6db13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9896db15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68d6db17, SUC_lexicon_tags_14 },
    { 0x3bf6db18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7576db1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee96db1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6db28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed6db2e, SUC_lexicon_tags_79 },
    { 0xd956db2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4696db31, SUC_lexicon_tags_70 },
    { 0xb036db32, SUC_lexicon_tags_38 },
    { 0xda16db32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b36db36, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2436db39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57d6db3b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d6db3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b6db47, SUC_lexicon_tags_30 },
    { 0xcd96db48, SUC_lexicon_tags_105 },
    { 0x41d6db49, SUC_lexicon_tags_14 },
    { 0x95f6db49, SUC_lexicon_tags_17 },
    { 0xa7f6db49, SUC_lexicon_tags_63 },
    { 0x3156db49, SUC_lexicon_tags_11 },
    { 0x2416db4c, SUC_lexicon_tags_25 },
    { 0xe0b6db4e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3696db53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf356db5a, SUC_lexicon_tags_31 },
    { 0x13b6db5b, SUC_lexicon_tags_47 },
    { 0x7136db5c, SUC_lexicon_tags_38 },
    { 0x9a16db5a, SUC_lexicon_tags_62 },
    { 0x8256db5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dd6db60, SUC_lexicon_tags_47 },
    { 0xaa76db61, SUC_lexicon_tags_25 },
    { 0xe1f6db62, SUC_lexicon_tags_25 },
    { 0x7c76db63, SUC_lexicon_tags_10 },
    { 0x7116db64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4216db67, SUC_lexicon_tags_25 },
    { 0x80b6db67, SUC_lexicon_tags_15 },
    { 0x77d6db69, SUC_lexicon_tags_2 },
    { 0xc976db67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f6db6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf116db72, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9056db78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd616db7e, SUC_lexicon_tags_19 },
    { 0x6cd6db7e, SUC_lexicon_tags_37 },
    { 0x5ef6db7e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d6db83, SUC_lexicon_tags_365 },
    { 0, NULL },
    { 0xc7b6db85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd676db87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x696db8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab16db96, SUC_lexicon_tags_23 },
    { 0x1616db96, SUC_lexicon_tags_8 },
    { 0xafd6db98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a76db9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa296db9e, SUC_lexicon_tags_35 },
    { 0xfa76db9f, SUC_lexicon_tags_16 },
    { 0x7616db9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f6dba5, SUC_lexicon_tags_37 },
    { 0xa136dba5, SUC_lexicon_tags_9 },
    { 0x44d6dba6, SUC_lexicon_tags_37 },
    { 0xff36dba6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b6dbaf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d6dbb3, SUC_lexicon_tags_11 },
    { 0x5536dbb4, SUC_lexicon_tags_7 },
    { 0xd616dbb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4736dbb8, SUC_lexicon_tags_1 },
    { 0xfa96dbb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6056dbbc, SUC_lexicon_tags_12 },
    { 0xe8b6dbbd, SUC_lexicon_tags_6 },
    { 0xd5b6dbbe, SUC_lexicon_tags_45 },
    { 0x1236dbbf, SUC_lexicon_tags_39 },
    { 0x7816dbc0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x25f6dbc2, SUC_lexicon_tags_39 },
    { 0x5076dbc3, SUC_lexicon_tags_27 },
    { 0x9a76dbc4, SUC_lexicon_tags_15 },
    { 0x9e16dbc5, SUC_lexicon_tags_80 },
    { 0x8116dbc6, SUC_lexicon_tags_8 },
    { 0xa7f6dbc3, SUC_lexicon_tags_2 },
    { 0x74f6dbc5, SUC_lexicon_tags_27 },
    { 0x936dbc9, SUC_lexicon_tags_38 },
    { 0x3816dbc9, SUC_lexicon_tags_17 },
    { 0xa196dbcb, SUC_lexicon_tags_31 },
    { 0x2096dbc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc936dbd2, SUC_lexicon_tags_12 },
    { 0xe916dbd3, SUC_lexicon_tags_16 },
    { 0x91d6dbd2, SUC_lexicon_tags_1 },
    { 0x2356dbd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d56dbda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x11f6dbdc, SUC_lexicon_tags_57 },
    { 0x55f6dbdc, SUC_lexicon_tags_25 },
    { 0x9ad6dbdd, SUC_lexicon_tags_1 },
    { 0x896dbdf, SUC_lexicon_tags_25 },
    { 0x4416dbdf, SUC_lexicon_tags_0 },
    { 0xadb6dbdf, SUC_lexicon_tags_0 },
    { 0x6c56dbe2, SUC_lexicon_tags_32 },
    { 0x4396dbe2, SUC_lexicon_tags_11 },
    { 0xf256dbe4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4716dbe6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa376dbeb, SUC_lexicon_tags_39 },
    { 0x4996dbec, SUC_lexicon_tags_14 },
    { 0xe096dbec, SUC_lexicon_tags_38 },
    { 0x73b6dbee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b6dbf3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6b6dbf5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd976dbf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2056dbfd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a56dc00, SUC_lexicon_tags_25 },
    { 0xf936dc00, SUC_lexicon_tags_3 },
    { 0x4556dc01, SUC_lexicon_tags_17 },
    { 0xcd76dc00, SUC_lexicon_tags_38 },
    { 0x6fb6dc02, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d6dc09, SUC_lexicon_tags_30 },
    { 0xb0d6dc0a, SUC_lexicon_tags_17 },
    { 0xa196dc09, SUC_lexicon_tags_17 },
    { 0x5a76dc0c, SUC_lexicon_tags_43 },
    { 0x5636dc0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1576dc0f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b36dc12, SUC_lexicon_tags_7 },
    { 0x96b6dc12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5f6dc15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8bb6dc17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5d16dc19, SUC_lexicon_tags_47 },
    { 0xbfb6dc19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c96dc1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe476dc21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5356dc27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x24d6dc29, SUC_lexicon_tags_2 },
    { 0x2336dc29, SUC_lexicon_tags_68 },
    { 0xf796dc2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef76dc2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e96dc30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2956dc35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd6dc38, SUC_lexicon_tags_37 },
    { 0x40f6dc39, SUC_lexicon_tags_26 },
    { 0x30d6dc38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b16dc3c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x73b6dc3e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb2f6dc40, SUC_lexicon_tags_14 },
    { 0xb396dc41, SUC_lexicon_tags_37 },
    { 0xee36dc42, SUC_lexicon_tags_11 },
    { 0xff56dc43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a96dc46, SUC_lexicon_tags_45 },
    { 0x9096dc46, SUC_lexicon_tags_11 },
    { 0xf9b6dc48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4716dc4e, SUC_lexicon_tags_87 },
    { 0x6b76dc4e, SUC_lexicon_tags_15 },
    { 0x9e76dc4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b6dc54, SUC_lexicon_tags_37 },
    { 0x4c76dc55, SUC_lexicon_tags_14 },
    { 0xbe76dc56, SUC_lexicon_tags_39 },
    { 0x7c36dc57, SUC_lexicon_tags_17 },
    { 0x9d56dc58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dc62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f6dc67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5f6dc69, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1236dc6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfa56dc6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb056dc6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x296dc72, SUC_lexicon_tags_13 },
    { 0x3b56dc72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc876dc75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb456dc77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecf6dc79, SUC_lexicon_tags_17 },
    { 0xc576dc79, SUC_lexicon_tags_15 },
    { 0x8a56dc7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6b6dc7d, SUC_lexicon_tags_14 },
    { 0x88b6dc7e, SUC_lexicon_tags_27 },
    { 0x5c16dc7f, SUC_lexicon_tags_0 },
    { 0xce56dc7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a96dc82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xde56dc84, SUC_lexicon_tags_173 },
    { 0x7876dc85, SUC_lexicon_tags_17 },
    { 0xbb16dc85, SUC_lexicon_tags_25 },
    { 0xfe16dc84, SUC_lexicon_tags_37 },
    { 0xafb6dc84, SUC_lexicon_tags_45 },
    { 0x3bb6dc86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c36dc8b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1036dc8e, SUC_lexicon_tags_25 },
    { 0x496dc8f, SUC_lexicon_tags_15 },
    { 0x6d96dc90, SUC_lexicon_tags_38 },
    { 0x1c56dc91, SUC_lexicon_tags_84 },
    { 0xaf96dc91, SUC_lexicon_tags_7 },
    { 0xa8d6dc90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc7f6dc95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf916dc98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce96dc9c, SUC_lexicon_tags_7 },
    { 0xebb6dc9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1236dca0, SUC_lexicon_tags_27 },
    { 0x8e96dca1, SUC_lexicon_tags_55 },
    { 0x8036dca0, SUC_lexicon_tags_12 },
    { 0xa556dca3, SUC_lexicon_tags_10 },
    { 0xb216dca4, SUC_lexicon_tags_15 },
    { 0x8356dca4, SUC_lexicon_tags_15 },
    { 0xb5f6dca5, SUC_lexicon_tags_37 },
    { 0x4876dca5, SUC_lexicon_tags_1 },
    { 0xea16dca8, SUC_lexicon_tags_15 },
    { 0x1df6dca9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe096dcae, SUC_lexicon_tags_38 },
    { 0xe676dcaf, SUC_lexicon_tags_30 },
    { 0x316dcb0, SUC_lexicon_tags_85 },
    { 0x77f6dcb1, SUC_lexicon_tags_27 },
    { 0xa416dcb2, SUC_lexicon_tags_45 },
    { 0xe3f6dcb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f6dcb9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd6dcbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b6dcc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9056dcc8, SUC_lexicon_tags_37 },
    { 0x47b6dcc8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcd56dccb, SUC_lexicon_tags_35 },
    { 0xc936dccc, SUC_lexicon_tags_37 },
    { 0xac76dccc, SUC_lexicon_tags_49 },
    { 0xd716dccc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7736dcd9, SUC_lexicon_tags_15 },
    { 0x1556dcda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4496dcdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9296dce1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef6dce4, SUC_lexicon_tags_38 },
    { 0x5b6dce5, SUC_lexicon_tags_19 },
    { 0x3f56dce6, SUC_lexicon_tags_7 },
    { 0x7476dce6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc36dceb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x716dcee, SUC_lexicon_tags_45 },
    { 0x6ad6dcef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf6dcf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f6dd06, SUC_lexicon_tags_15 },
    { 0xf176dd07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df6dd12, SUC_lexicon_tags_23 },
    { 0x5076dd13, SUC_lexicon_tags_1 },
    { 0x1d36dd13, SUC_lexicon_tags_7 },
    { 0xec56dd14, SUC_lexicon_tags_0 },
    { 0x1996dd16, SUC_lexicon_tags_53 },
    { 0x7bf6dd17, SUC_lexicon_tags_21 },
    { 0x7876dd13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b6dd1d, SUC_lexicon_tags_12 },
    { 0x6836dd1e, SUC_lexicon_tags_12 },
    { 0x6836dd1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d6dd22, SUC_lexicon_tags_160 },
    { 0xceb6dd22, SUC_lexicon_tags_31 },
    { 0xacd6dd24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b6dd2b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e76dd2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4fd6dd30, SUC_lexicon_tags_21 },
    { 0x4736dd31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x73f6dd33, SUC_lexicon_tags_21 },
    { 0xee16dd34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb676dd36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b6dd39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x27d6dd3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d6dd40, SUC_lexicon_tags_25 },
    { 0xe8b6dd41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3236dd45, SUC_lexicon_tags_49 },
    { 0x9b56dd46, SUC_lexicon_tags_7 },
    { 0xf4f6dd46, SUC_lexicon_tags_25 },
    { 0x83b6dd48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5456dd4a, SUC_lexicon_tags_77 },
    { 0x98d6dd4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b6dd4e, SUC_lexicon_tags_2 },
    { 0xe4f6dd4f, SUC_lexicon_tags_9 },
    { 0x4716dd50, SUC_lexicon_tags_30 },
    { 0x2f96dd51, SUC_lexicon_tags_18 },
    { 0x1296dd4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6fd6dd54, SUC_lexicon_tags_11 },
    { 0x6f6dd55, SUC_lexicon_tags_15 },
    { 0xf2b6dd55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dd59, SUC_lexicon_tags_5 },
    { 0x3036dd5a, SUC_lexicon_tags_37 },
    { 0x6c96dd5a, SUC_lexicon_tags_47 },
    { 0xb716dd5c, SUC_lexicon_tags_33 },
    { 0x8396dd5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b36dd5f, SUC_lexicon_tags_7 },
    { 0x1d6dd60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a56dd64, SUC_lexicon_tags_85 },
    { 0xa216dd65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35d6dd67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d6dd6a, SUC_lexicon_tags_25 },
    { 0xa7d6dd6b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf356dd78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b16dd7c, SUC_lexicon_tags_63 },
    { 0x3d96dd7d, SUC_lexicon_tags_25 },
    { 0x8636dd7c, SUC_lexicon_tags_31 },
    { 0x81b6dd7f, SUC_lexicon_tags_11 },
    { 0x1916dd7c, SUC_lexicon_tags_23 },
    { 0x6816dd81, SUC_lexicon_tags_29 },
    { 0xa6b6dd7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc036dd87, SUC_lexicon_tags_25 },
    { 0x5e56dd87, SUC_lexicon_tags_7 },
    { 0x3cd6dd89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d76dd8b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4156dd8d, SUC_lexicon_tags_25 },
    { 0xa5b6dd8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d6dd91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8216dd97, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8616dd99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d16dd9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d6dda2, SUC_lexicon_tags_29 },
    { 0xca16dda3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa676ddaa, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4856ddae, SUC_lexicon_tags_19 },
    { 0x16b6ddaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f56ddb8, SUC_lexicon_tags_25 },
    { 0xf116ddb9, SUC_lexicon_tags_1 },
    { 0x1816ddba, SUC_lexicon_tags_37 },
    { 0x7c76ddbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef76ddbd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5716ddc2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe16ddc5, SUC_lexicon_tags_47 },
    { 0xdc36ddc6, SUC_lexicon_tags_2 },
    { 0x7296ddc6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6ddca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb976ddcc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5816ddce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a36ddd1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6336ddd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2856ddda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d96dddd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa456dddf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2796dde1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97b6dde3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf856dde5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x476dde7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdab6dde9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d6ddec, SUC_lexicon_tags_27 },
    { 0x5e76ddec, SUC_lexicon_tags_10 },
    { 0xf5d6ddee, SUC_lexicon_tags_33 },
    { 0x1616ddec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcad6ddf1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4836ddf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d56ddf8, SUC_lexicon_tags_22 },
    { 0x5d96ddf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9576ddfc, SUC_lexicon_tags_25 },
    { 0x68d6ddfd, SUC_lexicon_tags_19 },
    { 0xb4d6ddfc, SUC_lexicon_tags_4 },
    { 0xd7d6ddff, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xf616de02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd56de04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7f36de06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc76de08, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6036de0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd296de0c, SUC_lexicon_tags_61 },
    { 0x8776de0d, SUC_lexicon_tags_47 },
    { 0x636de0c, SUC_lexicon_tags_17 },
    { 0xe956de0d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc76de18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416de1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2876de22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf256de2f, SUC_lexicon_tags_25 },
    { 0xdd6de30, SUC_lexicon_tags_2 },
    { 0xfb76de30, SUC_lexicon_tags_45 },
    { 0xa56de32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b6de35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd6de3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f6de3e, SUC_lexicon_tags_15 },
    { 0xd2b6de3f, SUC_lexicon_tags_57 },
    { 0xcab6de3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28f6de42, SUC_lexicon_tags_84 },
    { 0x79b6de43, SUC_lexicon_tags_33 },
    { 0xd2f6de43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7316de4a, SUC_lexicon_tags_0 },
    { 0xdf16de4b, SUC_lexicon_tags_336 },
    { 0x616de4c, SUC_lexicon_tags_30 },
    { 0x5dd6de4d, SUC_lexicon_tags_37 },
    { 0x6a36de4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3916de50, SUC_lexicon_tags_15 },
    { 0xe236de51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1856de54, SUC_lexicon_tags_17 },
    { 0xe376de55, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d6de5f, SUC_lexicon_tags_29 },
    { 0x7556de60, SUC_lexicon_tags_37 },
    { 0x9116de61, SUC_lexicon_tags_39 },
    { 0xae96de61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f16de68, SUC_lexicon_tags_33 },
    { 0x5b56de69, SUC_lexicon_tags_37 },
    { 0x7836de69, SUC_lexicon_tags_1 },
    { 0x76de6b, SUC_lexicon_tags_31 },
    { 0xf6de6a, SUC_lexicon_tags_16 },
    { 0x40f6de69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8056de6f, SUC_lexicon_tags_0 },
    { 0xe816de6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8776de75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b96de77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6036de79, SUC_lexicon_tags_17 },
    { 0x2476de7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e56de81, SUC_lexicon_tags_19 },
    { 0x34b6de82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeab6de84, SUC_lexicon_tags_19 },
    { 0xb036de85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb6de89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6de93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd996de95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0d6de97, SUC_lexicon_tags_38 },
    { 0xc876de97, SUC_lexicon_tags_15 },
    { 0x3116de97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8856de9f, SUC_lexicon_tags_6 },
    { 0x12d6de9f, SUC_lexicon_tags_37 },
    { 0x196dea0, SUC_lexicon_tags_14 },
    { 0xd2b6dea0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df6dea6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f6deab, SUC_lexicon_tags_16 },
    { 0x44f6deac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcbf6deae, SUC_lexicon_tags_6 },
    { 0x6396deaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf96deb2, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x8076deb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e56deb7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb6debb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2276debe, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb6dec6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b6deca, SUC_lexicon_tags_84 },
    { 0xd16decb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x476ded0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7096ded2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9516ded5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda96deda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d6dede, SUC_lexicon_tags_65 },
    { 0x5a76dede, SUC_lexicon_tags_45 },
    { 0x9d6dee0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f6dee6, SUC_lexicon_tags_15 },
    { 0x68f6dee7, SUC_lexicon_tags_17 },
    { 0x54f6dee8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6096deeb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb8f6deed, SUC_lexicon_tags_11 },
    { 0x6bf6deee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f36def4, SUC_lexicon_tags_17 },
    { 0xc636def5, SUC_lexicon_tags_11 },
    { 0x1596def5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb36defa, SUC_lexicon_tags_17 },
    { 0x3696defb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb6df08, SUC_lexicon_tags_39 },
    { 0x2bd6df09, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0xa556df0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc36df0e, SUC_lexicon_tags_6 },
    { 0x4196df0f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x86f6df11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa236df20, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb516df22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc236df26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6a56df28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc056df2a, SUC_lexicon_tags_30 },
    { 0x47f6df2b, SUC_lexicon_tags_17 },
    { 0x4396df2c, SUC_lexicon_tags_10 },
    { 0x1d36df2a, SUC_lexicon_tags_43 },
    { 0x16b6df2e, SUC_lexicon_tags_11 },
    { 0x6076df2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf416df31, SUC_lexicon_tags_47 },
    { 0x7976df31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ed6df34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcb76df36, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f6df3a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x6516df3d, SUC_lexicon_tags_7 },
    { 0xa996df3d, SUC_lexicon_tags_25 },
    { 0xc3b6df3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfa96df41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f6df45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bf6df47, SUC_lexicon_tags_19 },
    { 0x62b6df47, SUC_lexicon_tags_19 },
    { 0x9b36df49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1616df4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b6df51, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb856df53, SUC_lexicon_tags_43 },
    { 0x4d16df53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabd6df56, SUC_lexicon_tags_30 },
    { 0xd816df57, SUC_lexicon_tags_7 },
    { 0x7996df56, SUC_lexicon_tags_60 },
    { 0x17b6df58, SUC_lexicon_tags_15 },
    { 0x4f36df5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb6df62, SUC_lexicon_tags_37 },
    { 0x9fd6df62, SUC_lexicon_tags_39 },
    { 0x9656df63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd6df69, SUC_lexicon_tags_100 },
    { 0xa1b6df69, SUC_lexicon_tags_215 },
    { 0x80d6df6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d76df6e, SUC_lexicon_tags_11 },
    { 0x5296df6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xce36df71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a76df74, SUC_lexicon_tags_23 },
    { 0x2a16df74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d6df7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1016df82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x396df84, SUC_lexicon_tags_11 },
    { 0x3df6df85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d6df88, SUC_lexicon_tags_9 },
    { 0xb576df88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6ed6df8b, SUC_lexicon_tags_11 },
    { 0xabb6df8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9796df8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf136df90, SUC_lexicon_tags_63 },
    { 0xc7f6df91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd6df9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb756df9f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e16dfa3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6456dfa5, SUC_lexicon_tags_31 },
    { 0x3b76dfa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7996dfaa, SUC_lexicon_tags_12 },
    { 0x7b6dfab, SUC_lexicon_tags_7 },
    { 0x7476dfaa, SUC_lexicon_tags_87 },
    { 0xc056dfad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb36dfb0, SUC_lexicon_tags_0 },
    { 0xc2d6dfb1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x336dfb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x236dfb6, SUC_lexicon_tags_17 },
    { 0xb5b6dfb7, SUC_lexicon_tags_29 },
    { 0x34d6dfb7, SUC_lexicon_tags_15 },
    { 0x6df6dfb9, SUC_lexicon_tags_3 },
    { 0x67b6dfb6, SUC_lexicon_tags_37 },
    { 0x7136dfb8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab6dfbe, SUC_lexicon_tags_92 },
    { 0x81b6dfbe, SUC_lexicon_tags_15 },
    { 0x5a36dfc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf6dfc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb796dfc9, SUC_lexicon_tags_27 },
    { 0xe5f6dfca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d6dfcd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe776dfd1, SUC_lexicon_tags_17 },
    { 0x9936dfd2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9696dfd4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e56dfda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7276dfdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd6dfe0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f6dfe2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdeb6dfe4, SUC_lexicon_tags_12 },
    { 0xed76dfe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f6dfed, SUC_lexicon_tags_17 },
    { 0x4c76dfed, SUC_lexicon_tags_2 },
    { 0x9956dfee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b16dff2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd6dff7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd36dffa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f6dffd, SUC_lexicon_tags_1 },
    { 0x1956dffd, SUC_lexicon_tags_14 },
    { 0xbc56dfff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1696e006, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b6e009, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b6e00f, SUC_lexicon_tags_27 },
    { 0xd016e00f, SUC_lexicon_tags_2 },
    { 0x7056e011, SUC_lexicon_tags_1550 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd6e015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb6e017, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf816e019, SUC_lexicon_tags_40 },
    { 0xa596e01a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd836e01c, SUC_lexicon_tags_47 },
    { 0x5896e01d, SUC_lexicon_tags_33 },
    { 0xa4d6e01c, SUC_lexicon_tags_7 },
    { 0xb4b6e01c, SUC_lexicon_tags_65 },
    { 0x36e01d, SUC_lexicon_tags_27 },
    { 0xb0b6e020, SUC_lexicon_tags_37 },
    { 0x2896e01c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc076e029, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7a36e02b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d6e02f, SUC_lexicon_tags_37 },
    { 0xf5f6e02f, SUC_lexicon_tags_390 },
    { 0x1f16e031, SUC_lexicon_tags_84 },
    { 0x1696e032, SUC_lexicon_tags_15 },
    { 0x9916e033, SUC_lexicon_tags_25 },
    { 0x57b6e031, SUC_lexicon_tags_26 },
    { 0xbf36e031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb36e03a, SUC_lexicon_tags_37 },
    { 0x7c76e03a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6e040, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf436e042, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe96e044, SUC_lexicon_tags_46 },
    { 0x21b6e044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d6e04a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5cf6e04c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc036e04e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xca76e051, SUC_lexicon_tags_15 },
    { 0x87f6e052, SUC_lexicon_tags_15 },
    { 0xb236e053, SUC_lexicon_tags_33 },
    { 0x1a96e054, SUC_lexicon_tags_11 },
    { 0x9156e053, SUC_lexicon_tags_15 },
    { 0xa156e056, SUC_lexicon_tags_17 },
    { 0x6076e057, SUC_lexicon_tags_17 },
    { 0xc776e056, SUC_lexicon_tags_2 },
    { 0xb756e055, SUC_lexicon_tags_2 },
    { 0x8756e059, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced6e05f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5836e064, SUC_lexicon_tags_11 },
    { 0x956e065, SUC_lexicon_tags_27 },
    { 0xb496e066, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe996e068, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7db6e06a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b16e06d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xddb6e06f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36b6e071, SUC_lexicon_tags_33 },
    { 0xd5b6e071, SUC_lexicon_tags_17 },
    { 0xe376e072, SUC_lexicon_tags_17 },
    { 0xd496e074, SUC_lexicon_tags_4 },
    { 0xe556e075, SUC_lexicon_tags_33 },
    { 0xa716e074, SUC_lexicon_tags_11 },
    { 0x9736e077, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x5b6e079, SUC_lexicon_tags_38 },
    { 0x76d6e07a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a96e07c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef6e080, SUC_lexicon_tags_28 },
    { 0x7e96e080, SUC_lexicon_tags_49 },
    { 0x2d96e082, SUC_lexicon_tags_16 },
    { 0x7f96e082, SUC_lexicon_tags_37 },
    { 0x5f36e084, SUC_lexicon_tags_2 },
    { 0x3f6e083, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa36e096, SUC_lexicon_tags_25 },
    { 0x8b6e097, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad6e09a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf096e09d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf796e0a0, SUC_lexicon_tags_19 },
    { 0x7156e0a1, SUC_lexicon_tags_7 },
    { 0xb1b6e0a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b76e0a4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4116e0a6, SUC_lexicon_tags_43 },
    { 0x4836e0a7, SUC_lexicon_tags_29 },
    { 0x96b6e0a7, SUC_lexicon_tags_37 },
    { 0x696e0a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca96e0ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe976e0af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb136e0b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7596e0b3, SUC_lexicon_tags_33 },
    { 0xad96e0b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb0d6e0b6, SUC_lexicon_tags_7 },
    { 0xd176e0b7, SUC_lexicon_tags_1 },
    { 0xd916e0b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b6e0bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb236e0be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc816e0c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16e0c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b36e0c7, SUC_lexicon_tags_4 },
    { 0xcd56e0c8, SUC_lexicon_tags_11 },
    { 0xf1d6e0c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad6e0cd, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad76e0d1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6116e0d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3276e0d5, SUC_lexicon_tags_64 },
    { 0x2616e0d6, SUC_lexicon_tags_36 },
    { 0xa056e0d7, SUC_lexicon_tags_11 },
    { 0xa376e0d8, SUC_lexicon_tags_25 },
    { 0x2696e0d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16e0df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47d6e0e1, SUC_lexicon_tags_11 },
    { 0xc7b6e0e2, SUC_lexicon_tags_12 },
    { 0xa5b6e0e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf6e0e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8df6e0e9, SUC_lexicon_tags_37 },
    { 0xfa76e0ea, SUC_lexicon_tags_64 },
    { 0x5a76e0eb, SUC_lexicon_tags_15 },
    { 0x37b6e0ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d16e0ee, SUC_lexicon_tags_14 },
    { 0x8336e0ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f16e0f3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9176e0f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b6e0fa, SUC_lexicon_tags_1 },
    { 0x6a96e0fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf636e0ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x436e104, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf6e109, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3476e10d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1d76e10f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb596e113, SUC_lexicon_tags_8 },
    { 0xd2f6e114, SUC_lexicon_tags_55 },
    { 0x156e115, SUC_lexicon_tags_37 },
    { 0x99b6e116, SUC_lexicon_tags_27 },
    { 0x356e116, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb6e11b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd516e11d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46d6e11f, SUC_lexicon_tags_32 },
    { 0x8d16e120, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9596e12f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b16e134, SUC_lexicon_tags_18 },
    { 0xf56e134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f96e138, SUC_lexicon_tags_72 },
    { 0x6856e138, SUC_lexicon_tags_26 },
    { 0xe7f6e138, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc36e13c, SUC_lexicon_tags_37 },
    { 0xe976e13c, SUC_lexicon_tags_17 },
    { 0xa9b6e13e, SUC_lexicon_tags_37 },
    { 0x98b6e13c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde36e143, SUC_lexicon_tags_63 },
    { 0xefd6e143, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd6e148, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe736e14e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7276e153, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd216e155, SUC_lexicon_tags_12 },
    { 0x9796e156, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6e159, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6e15e, SUC_lexicon_tags_49 },
    { 0xdbf6e15f, SUC_lexicon_tags_14 },
    { 0x39b6e160, SUC_lexicon_tags_29 },
    { 0x5c76e15f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb6e165, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2956e16a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3296e16d, SUC_lexicon_tags_12 },
    { 0x5c76e16e, SUC_lexicon_tags_14 },
    { 0x4256e16f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f6e172, SUC_lexicon_tags_21 },
    { 0xd076e172, SUC_lexicon_tags_38 },
    { 0x7616e174, SUC_lexicon_tags_9 },
    { 0xcc36e175, SUC_lexicon_tags_171 },
    { 0x6a96e176, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df6e17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f6e17f, SUC_lexicon_tags_15 },
    { 0xfc16e180, SUC_lexicon_tags_17 },
    { 0xb976e181, SUC_lexicon_tags_50 },
    { 0x5fd6e181, SUC_lexicon_tags_37 },
    { 0x6c56e180, SUC_lexicon_tags_15 },
    { 0xa96e184, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba76e188, SUC_lexicon_tags_17 },
    { 0x6a36e189, SUC_lexicon_tags_25 },
    { 0xf056e189, SUC_lexicon_tags_26 },
    { 0x5676e18a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4936e18d, SUC_lexicon_tags_7 },
    { 0xa196e18e, SUC_lexicon_tags_14 },
    { 0xd2b6e18f, SUC_lexicon_tags_31 },
    { 0xbe36e18f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xded6e192, SUC_lexicon_tags_39 },
    { 0x4ef6e193, SUC_lexicon_tags_29 },
    { 0xe2f6e194, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xac16e197, SUC_lexicon_tags_66 },
    { 0x72b6e198, SUC_lexicon_tags_14 },
    { 0xdb76e199, SUC_lexicon_tags_3 },
    { 0xcb96e19a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8bb6e19c, SUC_lexicon_tags_37 },
    { 0x9a76e19d, SUC_lexicon_tags_11 },
    { 0x1876e19e, SUC_lexicon_tags_7 },
    { 0x6376e19f, SUC_lexicon_tags_25 },
    { 0x8076e19e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf36e1a4, SUC_lexicon_tags_8 },
    { 0xffb6e1a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9896e1a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b6e1ab, SUC_lexicon_tags_11 },
    { 0x2f36e1ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a36e1af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b6e1b4, SUC_lexicon_tags_31 },
    { 0x8916e1b5, SUC_lexicon_tags_41 },
    { 0x76d6e1b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd36e1bf, SUC_lexicon_tags_37 },
    { 0xe436e1bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e16e1c4, SUC_lexicon_tags_7 },
    { 0x4456e1c5, SUC_lexicon_tags_55 },
    { 0xe316e1c6, SUC_lexicon_tags_47 },
    { 0x4a16e1c4, SUC_lexicon_tags_34 },
    { 0x3076e1c4, SUC_lexicon_tags_27 },
    { 0x2cb6e1c9, SUC_lexicon_tags_14 },
    { 0xf56e1ca, SUC_lexicon_tags_12 },
    { 0x8376e1cb, SUC_lexicon_tags_29 },
    { 0xe4b6e1cb, SUC_lexicon_tags_63 },
    { 0x856e1c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d6e1d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5436e1d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f6e1d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a16e1da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1856e1de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x336e1e2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe76e1e4, SUC_lexicon_tags_9 },
    { 0x9a36e1e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfe96e1e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4576e1ed, SUC_lexicon_tags_43 },
    { 0xa656e1ee, SUC_lexicon_tags_1 },
    { 0x1df6e1ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb4b6e1f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f6e1f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f16e1f7, SUC_lexicon_tags_17 },
    { 0xd56e1f7, SUC_lexicon_tags_1 },
    { 0x4996e1f9, SUC_lexicon_tags_238 },
    { 0x176e1fa, SUC_lexicon_tags_15 },
    { 0x3636e1fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe176e1fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad6e200, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee56e202, SUC_lexicon_tags_37 },
    { 0x5cf6e203, SUC_lexicon_tags_1 },
    { 0x8fd6e204, SUC_lexicon_tags_7 },
    { 0x64b6e205, SUC_lexicon_tags_12 },
    { 0xbf16e206, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b6e20b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb6e20e, SUC_lexicon_tags_32 },
    { 0xbef6e20e, SUC_lexicon_tags_6 },
    { 0xce96e20e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5d6e212, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa96e216, SUC_lexicon_tags_27 },
    { 0x4536e216, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb696e219, SUC_lexicon_tags_38 },
    { 0x5b56e219, SUC_lexicon_tags_2 },
    { 0x5756e219, SUC_lexicon_tags_250 },
    { 0x7856e21c, SUC_lexicon_tags_37 },
    { 0xa776e21d, SUC_lexicon_tags_21 },
    { 0x3a96e21d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd96e221, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d6e225, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbf6e227, SUC_lexicon_tags_12 },
    { 0xfb96e227, SUC_lexicon_tags_51 },
    { 0x936e227, SUC_lexicon_tags_25 },
    { 0xb2f6e22a, SUC_lexicon_tags_37 },
    { 0xfa76e22b, SUC_lexicon_tags_2 },
    { 0xcc56e22b, SUC_lexicon_tags_25 },
    { 0xcc96e22d, SUC_lexicon_tags_45 },
    { 0xeab6e22e, SUC_lexicon_tags_19 },
    { 0xa016e22f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef56e231, SUC_lexicon_tags_45 },
    { 0xb336e232, SUC_lexicon_tags_61 },
    { 0xc356e233, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf616e23a, SUC_lexicon_tags_14 },
    { 0x74f6e23a, SUC_lexicon_tags_37 },
    { 0x10d6e23a, SUC_lexicon_tags_1 },
    { 0x2716e23a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c76e240, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6276e242, SUC_lexicon_tags_37 },
    { 0xc6f6e242, SUC_lexicon_tags_25 },
    { 0xde16e244, SUC_lexicon_tags_19 },
    { 0xd7f6e245, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef76e24c, SUC_lexicon_tags_17 },
    { 0x5b56e24d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2af6e24f, SUC_lexicon_tags_17 },
    { 0x4d16e24f, SUC_lexicon_tags_17 },
    { 0x6b96e251, SUC_lexicon_tags_25 },
    { 0x6196e252, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x896e258, SUC_lexicon_tags_7 },
    { 0x2956e259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9056e25d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d6e262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce96e265, SUC_lexicon_tags_47 },
    { 0x8bd6e266, SUC_lexicon_tags_28 },
    { 0x8cd6e267, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd296e269, SUC_lexicon_tags_92 },
    { 0xaf96e269, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb6e26d, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xf936e26f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec76e272, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d6e277, SUC_lexicon_tags_10 },
    { 0xa296e278, SUC_lexicon_tags_1 },
    { 0x9bf6e277, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe716e27b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc516e27d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x656e27f, SUC_lexicon_tags_26 },
    { 0x4d96e27f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b6e283, SUC_lexicon_tags_27 },
    { 0x7436e283, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f6e287, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd576e28a, SUC_lexicon_tags_25 },
    { 0xf2f6e28b, SUC_lexicon_tags_11 },
    { 0xefb6e28b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5216e28f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf996e291, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c56e293, SUC_lexicon_tags_47 },
    { 0xc8f6e293, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f6e29a, SUC_lexicon_tags_17 },
    { 0x6776e29b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5176e29f, SUC_lexicon_tags_34 },
    { 0xd1f6e2a0, SUC_lexicon_tags_2 },
    { 0xc896e2a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1756e2a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d6e2a8, SUC_lexicon_tags_8 },
    { 0x21d6e2a8, SUC_lexicon_tags_38 },
    { 0xf296e2aa, SUC_lexicon_tags_10 },
    { 0x2576e2ab, SUC_lexicon_tags_15 },
    { 0x40b6e2ac, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d6e2b0, SUC_lexicon_tags_1 },
    { 0x2176e2b1, SUC_lexicon_tags_2 },
    { 0x1536e2b0, SUC_lexicon_tags_17 },
    { 0x6036e2b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc936e2b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2676e2bb, SUC_lexicon_tags_28 },
    { 0x2f96e2bc, SUC_lexicon_tags_37 },
    { 0x1c16e2bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd76e2c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e96e2d0, SUC_lexicon_tags_23 },
    { 0xeed6e2d1, SUC_lexicon_tags_12 },
    { 0x5116e2d2, SUC_lexicon_tags_11 },
    { 0xc96e2d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed96e2d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f6e2dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4416e2e2, SUC_lexicon_tags_9 },
    { 0xe956e2e3, SUC_lexicon_tags_49 },
    { 0xc156e2e3, SUC_lexicon_tags_11 },
    { 0x6276e2e4, SUC_lexicon_tags_11 },
    { 0xba56e2e3, SUC_lexicon_tags_13 },
    { 0x5bf6e2e7, SUC_lexicon_tags_77 },
    { 0x1436e2e8, SUC_lexicon_tags_42 },
    { 0xc636e2e8, SUC_lexicon_tags_37 },
    { 0xb9f6e2ea, SUC_lexicon_tags_56 },
    { 0xb7b6e2e4, SUC_lexicon_tags_32 },
    { 0x3576e2e2, SUC_lexicon_tags_33 },
    { 0x336e2ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x90f6e2ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2216e2f3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfeb6e2f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a16e2f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c76e2f9, SUC_lexicon_tags_33 },
    { 0xfff6e2fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb6e2fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90f6e2ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb6e302, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9236e307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b6e30a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54f6e30c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef76e30e, SUC_lexicon_tags_7 },
    { 0x5d96e30e, SUC_lexicon_tags_47 },
    { 0x7096e310, SUC_lexicon_tags_25 },
    { 0xee16e311, SUC_lexicon_tags_15 },
    { 0x59d6e312, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b6e317, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb6e31a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f6e31e, SUC_lexicon_tags_13 },
    { 0x456e31e, SUC_lexicon_tags_1551 },
    { 0x47b6e320, SUC_lexicon_tags_30 },
    { 0x59d6e321, SUC_lexicon_tags_17 },
    { 0xf916e321, SUC_lexicon_tags_32 },
    { 0xc336e321, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe636e327, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf736e32b, SUC_lexicon_tags_37 },
    { 0xadf6e32c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2096e32f, SUC_lexicon_tags_17 },
    { 0x9d56e32f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5bd6e332, SUC_lexicon_tags_37 },
    { 0xe356e333, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef6e338, SUC_lexicon_tags_25 },
    { 0x84f6e339, SUC_lexicon_tags_80 },
    { 0x47d6e339, SUC_lexicon_tags_14 },
    { 0xae16e33b, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xef96e33e, SUC_lexicon_tags_11 },
    { 0xed16e33f, SUC_lexicon_tags_3 },
    { 0x9236e33e, SUC_lexicon_tags_47 },
    { 0x9d36e340, SUC_lexicon_tags_2 },
    { 0xa5b6e342, SUC_lexicon_tags_25 },
    { 0xea96e343, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d6e347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af6e34a, SUC_lexicon_tags_37 },
    { 0x7b96e34a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcdf6e34d, SUC_lexicon_tags_8 },
    { 0x976e34e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdad6e350, SUC_lexicon_tags_2 },
    { 0xa116e351, SUC_lexicon_tags_55 },
    { 0x4c96e352, SUC_lexicon_tags_7 },
    { 0x8f16e352, SUC_lexicon_tags_14 },
    { 0x2796e353, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb216e356, SUC_lexicon_tags_12 },
    { 0x9596e357, SUC_lexicon_tags_17 },
    { 0xb36e358, SUC_lexicon_tags_55 },
    { 0x6bf6e357, SUC_lexicon_tags_17 },
    { 0xdd76e35a, SUC_lexicon_tags_148 },
    { 0x576e35b, SUC_lexicon_tags_37 },
    { 0x5776e35c, SUC_lexicon_tags_27 },
    { 0x5ef6e35d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x22b6e35f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f6e362, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8816e364, SUC_lexicon_tags_663 },
    { 0, NULL },
    { 0x6136e366, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2516e368, SUC_lexicon_tags_16 },
    { 0xb736e368, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4416e36d, SUC_lexicon_tags_15 },
    { 0xe696e36d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xba16e371, SUC_lexicon_tags_70 },
    { 0xa796e372, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa76e375, SUC_lexicon_tags_68 },
    { 0xba16e376, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb136e37a, SUC_lexicon_tags_46 },
    { 0xb496e37a, SUC_lexicon_tags_43 },
    { 0x42d6e37c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416e380, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3016e383, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x376e387, SUC_lexicon_tags_16 },
    { 0x2736e388, SUC_lexicon_tags_11 },
    { 0x656e387, SUC_lexicon_tags_25 },
    { 0x3756e38a, SUC_lexicon_tags_27 },
    { 0x5a96e38b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cf6e38d, SUC_lexicon_tags_20 },
    { 0x8e36e38e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e76e3a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1896e3a4, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x5ad6e3a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f6e3a9, SUC_lexicon_tags_33 },
    { 0x43d6e3aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9116e3ae, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb6e3b6, SUC_lexicon_tags_569 },
    { 0x95f6e3b6, SUC_lexicon_tags_11 },
    { 0x1fd6e3b8, SUC_lexicon_tags_2 },
    { 0x5876e3b7, SUC_lexicon_tags_14 },
    { 0xd296e3b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x956e3bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d6e3c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af6e3c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3636e3c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x75b6e3cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe236e3cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf8f6e3cf, SUC_lexicon_tags_29 },
    { 0xe976e3d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee76e3d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db6e3d8, SUC_lexicon_tags_12 },
    { 0x3936e3d8, SUC_lexicon_tags_218 },
    { 0x9d96e3d9, SUC_lexicon_tags_63 },
    { 0x4236e3d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b76e3de, SUC_lexicon_tags_2 },
    { 0xc756e3de, SUC_lexicon_tags_1 },
    { 0xd7b6e3df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af6e3e9, SUC_lexicon_tags_7 },
    { 0x2f76e3ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb6e3ed, SUC_lexicon_tags_0 },
    { 0x21d6e3ee, SUC_lexicon_tags_82 },
    { 0x83b6e3ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd536e3f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d6e3f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3936e3f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed56e3fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf6e401, SUC_lexicon_tags_14 },
    { 0xc1d6e402, SUC_lexicon_tags_12 },
    { 0x6b96e401, SUC_lexicon_tags_33 },
    { 0xecd6e404, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6656e406, SUC_lexicon_tags_4 },
    { 0x3916e406, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f6e40a, SUC_lexicon_tags_77 },
    { 0x50d6e40a, SUC_lexicon_tags_25 },
    { 0x8ad6e40c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcc16e40e, SUC_lexicon_tags_25 },
    { 0x4476e40e, SUC_lexicon_tags_26 },
    { 0x71b6e410, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3196e412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3696e418, SUC_lexicon_tags_60 },
    { 0x5956e418, SUC_lexicon_tags_9 },
    { 0x2876e41a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f6e41d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d96e424, SUC_lexicon_tags_27 },
    { 0x9b76e424, SUC_lexicon_tags_47 },
    { 0xd136e426, SUC_lexicon_tags_47 },
    { 0xbc56e427, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae36e429, SUC_lexicon_tags_25 },
    { 0x4b16e42a, SUC_lexicon_tags_651 },
    { 0x3fd6e429, SUC_lexicon_tags_37 },
    { 0xd676e42a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd136e430, SUC_lexicon_tags_37 },
    { 0x2f16e431, SUC_lexicon_tags_91 },
    { 0xf8d6e432, SUC_lexicon_tags_25 },
    { 0xb796e432, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd96e436, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84f6e438, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb6e43b, SUC_lexicon_tags_17 },
    { 0x9756e43c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x636e43f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbe76e441, SUC_lexicon_tags_25 },
    { 0x25f6e442, SUC_lexicon_tags_11 },
    { 0x61f6e442, SUC_lexicon_tags_13 },
    { 0x3876e444, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd6e447, SUC_lexicon_tags_11 },
    { 0x7a16e447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf036e44f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b6e453, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6e456, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f6e465, SUC_lexicon_tags_14 },
    { 0x236e466, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd756e46b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a36e46f, SUC_lexicon_tags_30 },
    { 0x1d56e470, SUC_lexicon_tags_55 },
    { 0x5cb6e471, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf296e473, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef96e476, SUC_lexicon_tags_43 },
    { 0x3976e476, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd6e47a, SUC_lexicon_tags_87 },
    { 0x3fd6e47a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c56e47e, SUC_lexicon_tags_26 },
    { 0x7316e47f, SUC_lexicon_tags_28 },
    { 0xacb6e47f, SUC_lexicon_tags_37 },
    { 0x3a76e481, SUC_lexicon_tags_1 },
    { 0x42d6e47e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f6e488, SUC_lexicon_tags_2 },
    { 0x98f6e489, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31b6e48b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7176e48d, SUC_lexicon_tags_11 },
    { 0x3716e48d, SUC_lexicon_tags_11 },
    { 0xf656e48f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6916e492, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa816e496, SUC_lexicon_tags_25 },
    { 0xc436e497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2796e49b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59b6e49d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb6e4a0, SUC_lexicon_tags_17 },
    { 0x22b6e4a0, SUC_lexicon_tags_42 },
    { 0x1656e4a2, SUC_lexicon_tags_25 },
    { 0x58f6e4a2, SUC_lexicon_tags_17 },
    { 0x5bd6e4a1, SUC_lexicon_tags_17 },
    { 0x5d36e4a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3b6e4a7, SUC_lexicon_tags_15 },
    { 0xde16e4a8, SUC_lexicon_tags_65 },
    { 0x2116e4a8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f6e4ae, SUC_lexicon_tags_6 },
    { 0x83f6e4ae, SUC_lexicon_tags_65 },
    { 0xff96e4af, SUC_lexicon_tags_66 },
    { 0x7bb6e4b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96d6e4b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x276e4b5, SUC_lexicon_tags_63 },
    { 0xfcf6e4b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc596e4b9, SUC_lexicon_tags_6 },
    { 0x2bb6e4b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f6e4c1, SUC_lexicon_tags_201 },
    { 0xb436e4c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f36e4c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd036e4c8, SUC_lexicon_tags_9 },
    { 0x1ef6e4c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xedf6e4cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4b6e4cd, SUC_lexicon_tags_18 },
    { 0x82f6e4ce, SUC_lexicon_tags_25 },
    { 0xb2f6e4cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db6e4d3, SUC_lexicon_tags_12 },
    { 0xbad6e4d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa596e4d6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe396e4dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f6e4df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb296e4e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b6e4e5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b6e4e9, SUC_lexicon_tags_11 },
    { 0xbfd6e4ea, SUC_lexicon_tags_15 },
    { 0x4196e4eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef6e4f4, SUC_lexicon_tags_37 },
    { 0xcc96e4f4, SUC_lexicon_tags_26 },
    { 0x5956e4f6, SUC_lexicon_tags_1 },
    { 0x98b6e4f6, SUC_lexicon_tags_15 },
    { 0xe056e4f7, SUC_lexicon_tags_25 },
    { 0x596e4f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbf6e4fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f6e4fe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe576e502, SUC_lexicon_tags_45 },
    { 0xe676e502, SUC_lexicon_tags_1 },
    { 0xcf36e503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x316e50d, SUC_lexicon_tags_2 },
    { 0xbc16e50e, SUC_lexicon_tags_0 },
    { 0x8376e50d, SUC_lexicon_tags_49 },
    { 0xc0f6e50f, SUC_lexicon_tags_38 },
    { 0xe196e511, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf6e51f, SUC_lexicon_tags_28 },
    { 0x5896e520, SUC_lexicon_tags_30 },
    { 0xd776e521, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d56e523, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9436e528, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66f6e52a, SUC_lexicon_tags_11 },
    { 0x6a56e52b, SUC_lexicon_tags_1 },
    { 0x7076e52b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2276e533, SUC_lexicon_tags_65 },
    { 0xcb16e533, SUC_lexicon_tags_0 },
    { 0x1876e535, SUC_lexicon_tags_33 },
    { 0x9396e534, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3736e538, SUC_lexicon_tags_16 },
    { 0x50d6e538, SUC_lexicon_tags_57 },
    { 0xcf56e539, SUC_lexicon_tags_1 },
    { 0xef96e539, SUC_lexicon_tags_60 },
    { 0x60f6e539, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a96e53e, SUC_lexicon_tags_17 },
    { 0x5616e53f, SUC_lexicon_tags_25 },
    { 0x9196e53f, SUC_lexicon_tags_53 },
    { 0x6d36e540, SUC_lexicon_tags_56 },
    { 0x856e540, SUC_lexicon_tags_37 },
    { 0x91b6e53e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d6e54a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1256e54d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf436e552, SUC_lexicon_tags_25 },
    { 0x89f6e553, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x25d6e555, SUC_lexicon_tags_14 },
    { 0xfe96e556, SUC_lexicon_tags_7 },
    { 0xf396e557, SUC_lexicon_tags_172 },
    { 0x3296e558, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b6e55c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7296e55f, SUC_lexicon_tags_7 },
    { 0x3c56e560, SUC_lexicon_tags_33 },
    { 0xce56e561, SUC_lexicon_tags_49 },
    { 0xf8d6e562, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc416e56a, SUC_lexicon_tags_43 },
    { 0xbd16e56a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1996e56d, SUC_lexicon_tags_35 },
    { 0x1536e56d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f36e573, SUC_lexicon_tags_34 },
    { 0xa7d6e574, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6076e578, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d6e57d, SUC_lexicon_tags_16 },
    { 0x4976e57e, SUC_lexicon_tags_11 },
    { 0x3c76e57e, SUC_lexicon_tags_29 },
    { 0x6e16e57f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9bf6e582, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6e588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1016e58a, SUC_lexicon_tags_37 },
    { 0x9496e58a, SUC_lexicon_tags_21 },
    { 0x70d6e58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61d6e58e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b6e591, SUC_lexicon_tags_19 },
    { 0x7ef6e592, SUC_lexicon_tags_87 },
    { 0xeed6e591, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe76e597, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8656e599, SUC_lexicon_tags_23 },
    { 0xe336e59a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb996e59c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xba36e59f, SUC_lexicon_tags_11 },
    { 0x91d6e5a0, SUC_lexicon_tags_14 },
    { 0xd1f6e59f, SUC_lexicon_tags_11 },
    { 0x4cb6e5a2, SUC_lexicon_tags_49 },
    { 0x9616e5a0, SUC_lexicon_tags_269 },
    { 0xd16e5a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe036e5a6, SUC_lexicon_tags_25 },
    { 0x4336e5a7, SUC_lexicon_tags_1 },
    { 0xcb96e5a8, SUC_lexicon_tags_11 },
    { 0xde96e5a9, SUC_lexicon_tags_25 },
    { 0x26f6e5a8, SUC_lexicon_tags_60 },
    { 0x9f16e5aa, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b56e5af, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf6e5b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7236e5b7, SUC_lexicon_tags_7 },
    { 0xbc76e5b7, SUC_lexicon_tags_27 },
    { 0x3e76e5b8, SUC_lexicon_tags_27 },
    { 0xc416e5b9, SUC_lexicon_tags_32 },
    { 0xdf96e5ba, SUC_lexicon_tags_47 },
    { 0xc7f6e5bc, SUC_lexicon_tags_25 },
    { 0x1656e5bc, SUC_lexicon_tags_49 },
    { 0x4cf6e5be, SUC_lexicon_tags_45 },
    { 0xff6e5bf, SUC_lexicon_tags_63 },
    { 0xf896e5bf, SUC_lexicon_tags_37 },
    { 0x3296e5bf, SUC_lexicon_tags_25 },
    { 0xf6b6e5c0, SUC_lexicon_tags_2 },
    { 0x98f6e5c3, SUC_lexicon_tags_19 },
    { 0x596e5c4, SUC_lexicon_tags_28 },
    { 0xff36e5be, SUC_lexicon_tags_37 },
    { 0x49b6e5c0, SUC_lexicon_tags_43 },
    { 0xe8d6e5c7, SUC_lexicon_tags_17 },
    { 0x7236e5c6, SUC_lexicon_tags_7 },
    { 0x596e5c9, SUC_lexicon_tags_45 },
    { 0x2bb6e5ca, SUC_lexicon_tags_7 },
    { 0xa276e5cb, SUC_lexicon_tags_29 },
    { 0x5d6e5cc, SUC_lexicon_tags_84 },
    { 0x4716e5bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d6e5d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd6e5d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b6e5da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb6e5e1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x55b6e5e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b6e5e6, SUC_lexicon_tags_12 },
    { 0x6616e5e6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f6e5ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3636e5f1, SUC_lexicon_tags_17 },
    { 0x1336e5f1, SUC_lexicon_tags_11 },
    { 0x9296e5f1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b6e5f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2296e5fb, SUC_lexicon_tags_11 },
    { 0xa8d6e5fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1916e5fe, SUC_lexicon_tags_37 },
    { 0xef6e5fe, SUC_lexicon_tags_21 },
    { 0xb036e5ff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcd76e602, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad36e604, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e56e608, SUC_lexicon_tags_60 },
    { 0x2ab6e608, SUC_lexicon_tags_11 },
    { 0x83d6e609, SUC_lexicon_tags_9 },
    { 0x13d6e60b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d6e60e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4496e612, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d76e616, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e36e619, SUC_lexicon_tags_12 },
    { 0xf6d6e619, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x81f6e61c, SUC_lexicon_tags_7 },
    { 0x14b6e61c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1476e620, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x52d6e622, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87f6e624, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6f76e626, SUC_lexicon_tags_33 },
    { 0x9e16e627, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xecf6e629, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab16e62b, SUC_lexicon_tags_16 },
    { 0x5f36e62c, SUC_lexicon_tags_46 },
    { 0x76d6e62c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae76e63c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3c76e63e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x996e641, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5356e643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8036e646, SUC_lexicon_tags_1 },
    { 0xd156e647, SUC_lexicon_tags_26 },
    { 0x74d6e648, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff6e64c, SUC_lexicon_tags_17 },
    { 0x3896e64d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8396e650, SUC_lexicon_tags_23 },
    { 0x96b6e650, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa16e656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d6e658, SUC_lexicon_tags_8 },
    { 0x2dd6e659, SUC_lexicon_tags_15 },
    { 0x19f6e659, SUC_lexicon_tags_1 },
    { 0x6d6e65b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb616e65d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6376e660, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea96e662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf356e665, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe856e668, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5536e66d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb996e671, SUC_lexicon_tags_37 },
    { 0xc496e672, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x60b6e674, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9496e676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3296e679, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9036e67c, SUC_lexicon_tags_26 },
    { 0xb8b6e67c, SUC_lexicon_tags_15 },
    { 0xa96e67e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd6e683, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8ed6e685, SUC_lexicon_tags_20 },
    { 0xe716e685, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c56e68e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5156e695, SUC_lexicon_tags_25 },
    { 0x8e96e695, SUC_lexicon_tags_11 },
    { 0x8596e696, SUC_lexicon_tags_30 },
    { 0xe236e698, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdeb6e69a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd36e69d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b6e6a0, SUC_lexicon_tags_26 },
    { 0x3016e6a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa596e6a3, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x9af6e6a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x91b6e6a7, SUC_lexicon_tags_47 },
    { 0xff36e6a7, SUC_lexicon_tags_7 },
    { 0x76d6e6a7, SUC_lexicon_tags_38 },
    { 0xde36e6aa, SUC_lexicon_tags_37 },
    { 0x336e6aa, SUC_lexicon_tags_87 },
    { 0xc16e6ab, SUC_lexicon_tags_2 },
    { 0x6c76e6ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c16e6af, SUC_lexicon_tags_48 },
    { 0x2b6e6b0, SUC_lexicon_tags_34 },
    { 0x59f6e6b0, SUC_lexicon_tags_11 },
    { 0x4676e6b2, SUC_lexicon_tags_19 },
    { 0x13f6e6b3, SUC_lexicon_tags_45 },
    { 0xb6d6e6b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3b56e6b6, SUC_lexicon_tags_1 },
    { 0x1196e6b7, SUC_lexicon_tags_7 },
    { 0xd156e6b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e56e6bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84f6e6bd, SUC_lexicon_tags_32 },
    { 0xcf6e6be, SUC_lexicon_tags_30 },
    { 0xa36e6bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99b6e6c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2636e6c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a96e6c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1556e6cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa976e6ce, SUC_lexicon_tags_14 },
    { 0x9f6e6ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b6e6db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78b6e6dd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d6e6e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f6e6e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b6e6e9, SUC_lexicon_tags_16 },
    { 0x1616e6ea, SUC_lexicon_tags_37 },
    { 0x4836e6ea, SUC_lexicon_tags_37 },
    { 0x5e96e6ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9016e6f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb076e6f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e16e6f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b96e6fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa456e6ff, SUC_lexicon_tags_46 },
    { 0xbef6e700, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8d6e702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd56e707, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e96e70a, SUC_lexicon_tags_1 },
    { 0xf076e70b, SUC_lexicon_tags_15 },
    { 0x44f6e70c, SUC_lexicon_tags_46 },
    { 0x40b6e70d, SUC_lexicon_tags_17 },
    { 0x2c16e70e, SUC_lexicon_tags_25 },
    { 0x8416e70f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb156e715, SUC_lexicon_tags_7 },
    { 0x3bf6e715, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f6e718, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf636e71c, SUC_lexicon_tags_15 },
    { 0xef96e71d, SUC_lexicon_tags_23 },
    { 0x4f36e71e, SUC_lexicon_tags_15 },
    { 0xde36e71f, SUC_lexicon_tags_0 },
    { 0x5d96e71e, SUC_lexicon_tags_14 },
    { 0xe796e720, SUC_lexicon_tags_12 },
    { 0x7ab6e722, SUC_lexicon_tags_47 },
    { 0x83f6e723, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1776e725, SUC_lexicon_tags_1 },
    { 0xba56e725, SUC_lexicon_tags_15 },
    { 0x8596e727, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd356e729, SUC_lexicon_tags_1 },
    { 0xf996e729, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3eb6e72c, SUC_lexicon_tags_11 },
    { 0xb876e72d, SUC_lexicon_tags_37 },
    { 0xb8d6e72e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5816e732, SUC_lexicon_tags_17 },
    { 0xa736e732, SUC_lexicon_tags_7 },
    { 0x536e732, SUC_lexicon_tags_46 },
    { 0x4936e733, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd6e737, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd656e73a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x676e73c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c16e73f, SUC_lexicon_tags_2 },
    { 0x636e740, SUC_lexicon_tags_37 },
    { 0xb696e741, SUC_lexicon_tags_37 },
    { 0xae56e73f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc156e746, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf76e748, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1456e74c, SUC_lexicon_tags_21 },
    { 0xed56e74d, SUC_lexicon_tags_12 },
    { 0x3f76e74e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6e753, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3496e755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6096e75b, SUC_lexicon_tags_27 },
    { 0x6b96e75c, SUC_lexicon_tags_38 },
    { 0xd356e75b, SUC_lexicon_tags_19 },
    { 0x816e75e, SUC_lexicon_tags_38 },
    { 0xc416e75f, SUC_lexicon_tags_55 },
    { 0x3dd6e760, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd56e763, SUC_lexicon_tags_17 },
    { 0x836e763, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf6e76c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5096e76f, SUC_lexicon_tags_47 },
    { 0xdc36e770, SUC_lexicon_tags_43 },
    { 0x616e76f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9276e778, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa96e77c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca16e77e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe316e787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b6e78a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd536e78e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x1756e790, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5256e792, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7536e795, SUC_lexicon_tags_2 },
    { 0xd4b6e796, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb6e79e, SUC_lexicon_tags_39 },
    { 0xe056e79f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a96e7a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda56e7a4, SUC_lexicon_tags_11 },
    { 0xad6e7a5, SUC_lexicon_tags_25 },
    { 0xf996e7a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f6e7af, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4e36e7b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc436e7b8, SUC_lexicon_tags_2 },
    { 0x4996e7b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe636e7bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb116e7c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b6e7c5, SUC_lexicon_tags_15 },
    { 0x9b96e7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9516e7ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d96e7cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7cd6e7ce, SUC_lexicon_tags_23 },
    { 0x7736e7cf, SUC_lexicon_tags_19 },
    { 0xe936e7d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa436e7d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8776e7da, SUC_lexicon_tags_35 },
    { 0x2d36e7db, SUC_lexicon_tags_47 },
    { 0x3156e7db, SUC_lexicon_tags_3 },
    { 0xb256e7dd, SUC_lexicon_tags_2 },
    { 0xbc36e7dc, SUC_lexicon_tags_65 },
    { 0x46f6e7de, SUC_lexicon_tags_14 },
    { 0x7b36e7e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6656e7e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf16e7e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe56e7ee, SUC_lexicon_tags_33 },
    { 0xd876e7ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd6e7f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1496e7f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61d6e7fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb696e7fd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f76e800, SUC_lexicon_tags_1 },
    { 0x3e96e801, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2d36e803, SUC_lexicon_tags_83 },
    { 0x27d6e803, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6676e808, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad56e80b, SUC_lexicon_tags_21 },
    { 0xa4d6e80b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x816e80f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b6e812, SUC_lexicon_tags_79 },
    { 0x596e812, SUC_lexicon_tags_29 },
    { 0x2cb6e812, SUC_lexicon_tags_82 },
    { 0x24f6e815, SUC_lexicon_tags_4 },
    { 0x64b6e813, SUC_lexicon_tags_26 },
    { 0xb216e816, SUC_lexicon_tags_46 },
    { 0xe3d6e818, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc536e81a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3136e81d, SUC_lexicon_tags_7 },
    { 0xcef6e81e, SUC_lexicon_tags_27 },
    { 0xd416e81f, SUC_lexicon_tags_2 },
    { 0xf776e820, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1a56e822, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd176e829, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b6e833, SUC_lexicon_tags_11 },
    { 0x8796e834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8736e837, SUC_lexicon_tags_12 },
    { 0x5416e837, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xddd6e83a, SUC_lexicon_tags_8 },
    { 0x1f76e83b, SUC_lexicon_tags_82 },
    { 0xeb96e83c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5216e83e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2576e842, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb16e845, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe76e849, SUC_lexicon_tags_29 },
    { 0x9336e849, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9536e84c, SUC_lexicon_tags_11 },
    { 0xd16e84c, SUC_lexicon_tags_27 },
    { 0xf8f6e84d, SUC_lexicon_tags_8 },
    { 0x9976e84d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b6e854, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6916e856, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d6e85b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72d6e85d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b6e864, SUC_lexicon_tags_17 },
    { 0xd1b6e865, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdfd6e867, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa76e86a, SUC_lexicon_tags_19 },
    { 0x37b6e86a, SUC_lexicon_tags_8 },
    { 0x7376e86c, SUC_lexicon_tags_7 },
    { 0xa996e86d, SUC_lexicon_tags_37 },
    { 0x4dd6e86e, SUC_lexicon_tags_35 },
    { 0xeb76e86f, SUC_lexicon_tags_49 },
    { 0x7b36e86c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd6e877, SUC_lexicon_tags_38 },
    { 0x8616e878, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b36e87b, SUC_lexicon_tags_14 },
    { 0xd6e87c, SUC_lexicon_tags_2 },
    { 0x2e16e87b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b6e882, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab6e88b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c56e88d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d6e891, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d6e895, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc256e898, SUC_lexicon_tags_28 },
    { 0xd5b6e899, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde56e8a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff6e8a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf6e8ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23d6e8b0, SUC_lexicon_tags_518 },
    { 0x6d6e8b0, SUC_lexicon_tags_37 },
    { 0xcf16e8b0, SUC_lexicon_tags_25 },
    { 0x3176e8b0, SUC_lexicon_tags_38 },
    { 0x98b6e8b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b16e8ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcef6e8bc, SUC_lexicon_tags_26 },
    { 0x5ef6e8bd, SUC_lexicon_tags_47 },
    { 0xd2b6e8bd, SUC_lexicon_tags_17 },
    { 0x49b6e8bc, SUC_lexicon_tags_8 },
    { 0xbe36e8c0, SUC_lexicon_tags_1 },
    { 0x3c76e8c1, SUC_lexicon_tags_18 },
    { 0xb636e8c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4936e8c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb436e8c8, SUC_lexicon_tags_12 },
    { 0x87d6e8c9, SUC_lexicon_tags_17 },
    { 0x6436e8ca, SUC_lexicon_tags_25 },
    { 0xaa36e8ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f36e8d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2836e8d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf6e8d9, SUC_lexicon_tags_17 },
    { 0x1dd6e8da, SUC_lexicon_tags_12 },
    { 0x5cf6e8db, SUC_lexicon_tags_15 },
    { 0x9396e8dc, SUC_lexicon_tags_1 },
    { 0x69f6e8dd, SUC_lexicon_tags_47 },
    { 0xa196e8de, SUC_lexicon_tags_0 },
    { 0x3d6e8de, SUC_lexicon_tags_2 },
    { 0xcc96e8dd, SUC_lexicon_tags_11 },
    { 0xc5b6e8e1, SUC_lexicon_tags_11 },
    { 0x4d96e8e2, SUC_lexicon_tags_11 },
    { 0x49d6e8de, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xcd76e8e5, SUC_lexicon_tags_29 },
    { 0xa5f6e8e6, SUC_lexicon_tags_29 },
    { 0x9e36e8e7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b96e8eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdf96e8ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4836e8ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd756e8f1, SUC_lexicon_tags_11 },
    { 0xa996e8f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5456e8f4, SUC_lexicon_tags_49 },
    { 0x636e8f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b6e8f8, SUC_lexicon_tags_37 },
    { 0x2216e8f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b6e8fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2f6e8fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f6e904, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30b6e906, SUC_lexicon_tags_11 },
    { 0x7fd6e906, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63b6e909, SUC_lexicon_tags_0 },
    { 0x1576e90a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf256e90e, SUC_lexicon_tags_26 },
    { 0x2a16e90e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d6e912, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7756e91a, SUC_lexicon_tags_47 },
    { 0x7d36e91b, SUC_lexicon_tags_37 },
    { 0xa736e91c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6976e922, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3356e925, SUC_lexicon_tags_2 },
    { 0x9b36e925, SUC_lexicon_tags_17 },
    { 0x4996e926, SUC_lexicon_tags_6 },
    { 0xd5b6e928, SUC_lexicon_tags_6 },
    { 0x5a76e929, SUC_lexicon_tags_37 },
    { 0x3476e92a, SUC_lexicon_tags_31 },
    { 0x6e56e928, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa336e92f, SUC_lexicon_tags_1 },
    { 0x2e56e92f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d6e933, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b6e938, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4716e93a, SUC_lexicon_tags_1 },
    { 0x836e93b, SUC_lexicon_tags_2 },
    { 0x13b6e93c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3f6e93e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa6f6e940, SUC_lexicon_tags_37 },
    { 0x3c76e940, SUC_lexicon_tags_17 },
    { 0x48d6e942, SUC_lexicon_tags_45 },
    { 0xf2f6e943, SUC_lexicon_tags_1 },
    { 0xbb16e944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9516e947, SUC_lexicon_tags_23 },
    { 0x796e947, SUC_lexicon_tags_17 },
    { 0xe516e949, SUC_lexicon_tags_7 },
    { 0x9236e948, SUC_lexicon_tags_11 },
    { 0x92b6e94b, SUC_lexicon_tags_63 },
    { 0x7096e94c, SUC_lexicon_tags_12 },
    { 0x5976e94d, SUC_lexicon_tags_11 },
    { 0xbdd6e94c, SUC_lexicon_tags_8 },
    { 0x1316e947, SUC_lexicon_tags_15 },
    { 0x3896e950, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f6e953, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd6e956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7896e959, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c96e95b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f6e96d, SUC_lexicon_tags_42 },
    { 0x9b6e96d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x76e970, SUC_lexicon_tags_28 },
    { 0xe16e970, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf6e975, SUC_lexicon_tags_14 },
    { 0x4d16e975, SUC_lexicon_tags_1 },
    { 0xd236e975, SUC_lexicon_tags_20 },
    { 0xcb76e977, SUC_lexicon_tags_0 },
    { 0xcfd6e975, SUC_lexicon_tags_37 },
    { 0xc2f6e977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7036e97c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd6e97f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6436e982, SUC_lexicon_tags_17 },
    { 0x5e76e982, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f36e985, SUC_lexicon_tags_21 },
    { 0xb96e985, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x71d6e988, SUC_lexicon_tags_19 },
    { 0xa476e989, SUC_lexicon_tags_19 },
    { 0x1996e98a, SUC_lexicon_tags_14 },
    { 0x6d16e98a, SUC_lexicon_tags_27 },
    { 0xcf76e98a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1436e98e, SUC_lexicon_tags_25 },
    { 0x36d6e98f, SUC_lexicon_tags_25 },
    { 0xde76e990, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b76e999, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb696e99b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5816e9a6, SUC_lexicon_tags_45 },
    { 0x9b96e9a7, SUC_lexicon_tags_12 },
    { 0x6df6e9a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2456e9aa, SUC_lexicon_tags_1 },
    { 0x90f6e9ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63b6e9ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd5b6e9af, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd736e9b1, SUC_lexicon_tags_15 },
    { 0x3096e9b1, SUC_lexicon_tags_28 },
    { 0x29f6e9b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b6e9b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a36e9bc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b6e9c3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3976e9cc, SUC_lexicon_tags_1 },
    { 0x8d56e9cd, SUC_lexicon_tags_12 },
    { 0x7776e9ce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7616e9d0, SUC_lexicon_tags_29 },
    { 0x5c36e9d1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8c96e9d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4476e9d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb76e9de, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb176e9e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccb6e9e4, SUC_lexicon_tags_7 },
    { 0x736e9e4, SUC_lexicon_tags_61 },
    { 0x7d56e9e5, SUC_lexicon_tags_14 },
    { 0x9af6e9e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb416e9e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f76e9ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7676e9f2, SUC_lexicon_tags_47 },
    { 0xb276e9f3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4f96e9f5, SUC_lexicon_tags_23 },
    { 0x17f6e9f6, SUC_lexicon_tags_47 },
    { 0x9016e9f5, SUC_lexicon_tags_37 },
    { 0xc8b6e9f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8836e9fb, SUC_lexicon_tags_15 },
    { 0x3b6e9fc, SUC_lexicon_tags_48 },
    { 0x2796e9fc, SUC_lexicon_tags_15 },
    { 0x4376e9fe, SUC_lexicon_tags_16 },
    { 0xe56e9ff, SUC_lexicon_tags_14 },
    { 0x636ea00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc1f6ea02, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df6ea06, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6516ea08, SUC_lexicon_tags_1 },
    { 0xe836ea08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b6ea0e, SUC_lexicon_tags_11 },
    { 0xf976ea0f, SUC_lexicon_tags_38 },
    { 0x1bf6ea10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d16ea16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d76ea1c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b6ea20, SUC_lexicon_tags_14 },
    { 0xaad6ea21, SUC_lexicon_tags_224 },
    { 0xf496ea22, SUC_lexicon_tags_15 },
    { 0x4d16ea21, SUC_lexicon_tags_15 },
    { 0x5d56ea24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c56ea27, SUC_lexicon_tags_39 },
    { 0xc796ea28, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa176ea2c, SUC_lexicon_tags_35 },
    { 0x5056ea2c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca36ea32, SUC_lexicon_tags_14 },
    { 0x9bb6ea33, SUC_lexicon_tags_11 },
    { 0xa956ea32, SUC_lexicon_tags_12 },
    { 0x6b76ea35, SUC_lexicon_tags_47 },
    { 0x2d76ea32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa176ea3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9156ea3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b76ea3f, SUC_lexicon_tags_15 },
    { 0x99b6ea40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3216ea43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4756ea45, SUC_lexicon_tags_37 },
    { 0x3e96ea45, SUC_lexicon_tags_35 },
    { 0xddd6ea46, SUC_lexicon_tags_8 },
    { 0xf076ea48, SUC_lexicon_tags_12 },
    { 0x7636ea49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x71b6ea4b, SUC_lexicon_tags_47 },
    { 0x5836ea4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf6ea4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb6ea53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf616ea59, SUC_lexicon_tags_65 },
    { 0x6636ea5a, SUC_lexicon_tags_17 },
    { 0x6556ea5a, SUC_lexicon_tags_11 },
    { 0x7896ea5a, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xc7d6ea5e, SUC_lexicon_tags_7 },
    { 0xcc96ea5f, SUC_lexicon_tags_47 },
    { 0x8816ea5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7516ea6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd756ea6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cb6ea6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c36ea71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a76ea73, SUC_lexicon_tags_25 },
    { 0xc236ea74, SUC_lexicon_tags_2 },
    { 0x2f56ea73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3596ea79, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc296ea7e, SUC_lexicon_tags_38 },
    { 0x4816ea7f, SUC_lexicon_tags_25 },
    { 0x5b6ea7f, SUC_lexicon_tags_17 },
    { 0x4016ea81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f6ea84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd616ea87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7116ea89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab6ea90, SUC_lexicon_tags_17 },
    { 0x3dd6ea90, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6cb6ea93, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab6ea96, SUC_lexicon_tags_36 },
    { 0xa7d6ea96, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96ea9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1b6ea9d, SUC_lexicon_tags_28 },
    { 0xe576ea9e, SUC_lexicon_tags_46 },
    { 0xc0d6ea9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b6eaa4, SUC_lexicon_tags_5 },
    { 0x816eaa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf616eaaa, SUC_lexicon_tags_23 },
    { 0x6616eaab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4576eaad, SUC_lexicon_tags_14 },
    { 0x116eaae, SUC_lexicon_tags_17 },
    { 0xab6eaae, SUC_lexicon_tags_63 },
    { 0xef76eab0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdff6eab2, SUC_lexicon_tags_27 },
    { 0x716eab3, SUC_lexicon_tags_11 },
    { 0xdf56eab4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9196eab6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf6eab9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2d76eabb, SUC_lexicon_tags_11 },
    { 0x24f6eabc, SUC_lexicon_tags_4 },
    { 0x53b6eabd, SUC_lexicon_tags_17 },
    { 0xcf16eabe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6456eac2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa016eac4, SUC_lexicon_tags_37 },
    { 0xed76eac5, SUC_lexicon_tags_2 },
    { 0x4456eac6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc816eac8, SUC_lexicon_tags_16 },
    { 0xbab6eac8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d6eace, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d6ead9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3676eadb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f6eade, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea96eae1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d6eaea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x496eaee, SUC_lexicon_tags_27 },
    { 0xa4b6eaef, SUC_lexicon_tags_28 },
    { 0x5f16eaef, SUC_lexicon_tags_49 },
    { 0xf6eaf1, SUC_lexicon_tags_65 },
    { 0x94d6eaf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb696eaf7, SUC_lexicon_tags_2 },
    { 0xa096eaf7, SUC_lexicon_tags_37 },
    { 0xfe56eaf9, SUC_lexicon_tags_29 },
    { 0x23b6eafa, SUC_lexicon_tags_15 },
    { 0xe796eaf9, SUC_lexicon_tags_88 },
    { 0x4156eafb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe116eb03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf056eb05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db6eb0e, SUC_lexicon_tags_25 },
    { 0xd596eb0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b6eb12, SUC_lexicon_tags_17 },
    { 0x5f56eb13, SUC_lexicon_tags_61 },
    { 0x1316eb14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d6eb1a, SUC_lexicon_tags_55 },
    { 0x14d6eb1b, SUC_lexicon_tags_15 },
    { 0xb1f6eb1a, SUC_lexicon_tags_38 },
    { 0xc36eb1a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e96eb24, SUC_lexicon_tags_27 },
    { 0xf16eb25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f36eb28, SUC_lexicon_tags_43 },
    { 0xd936eb29, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbed6eb2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51f6eb2d, SUC_lexicon_tags_15 },
    { 0xf836eb2e, SUC_lexicon_tags_43 },
    { 0x9cf6eb2f, SUC_lexicon_tags_25 },
    { 0x3b56eb2f, SUC_lexicon_tags_84 },
    { 0x2a56eb31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5296eb36, SUC_lexicon_tags_25 },
    { 0x8876eb36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c56eb3a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e56eb40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d6eb44, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc576eb49, SUC_lexicon_tags_82 },
    { 0xa056eb4a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x80d6eb4c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfd36eb4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9936eb51, SUC_lexicon_tags_4 },
    { 0x1a76eb52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbdd6eb54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb136eb5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8236eb63, SUC_lexicon_tags_45 },
    { 0xb356eb64, SUC_lexicon_tags_1 },
    { 0x71f6eb63, SUC_lexicon_tags_25 },
    { 0xfdf6eb66, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8b6eb68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b76eb6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b6eb6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df6eb77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc076eb7d, SUC_lexicon_tags_17 },
    { 0x8576eb7e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf216eb81, SUC_lexicon_tags_68 },
    { 0xaaf6eb82, SUC_lexicon_tags_1 },
    { 0x1756eb81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88b6eb85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd76eb8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b6eb93, SUC_lexicon_tags_238 },
    { 0xd676eb94, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f16eb9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d6eba3, SUC_lexicon_tags_7 },
    { 0x7e16eba3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d6eba8, SUC_lexicon_tags_17 },
    { 0x8856eba9, SUC_lexicon_tags_60 },
    { 0xf16ebaa, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf6ebaf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a16ebb4, SUC_lexicon_tags_38 },
    { 0x4b6ebb5, SUC_lexicon_tags_43 },
    { 0x9536ebb4, SUC_lexicon_tags_17 },
    { 0xdf96ebb4, SUC_lexicon_tags_39 },
    { 0x9dd6ebb7, SUC_lexicon_tags_55 },
    { 0xd636ebb9, SUC_lexicon_tags_37 },
    { 0xd76ebb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7396ebbc, SUC_lexicon_tags_37 },
    { 0x9856ebbd, SUC_lexicon_tags_25 },
    { 0x27d6ebbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xce56ebc0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7956ebc7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b6ebca, SUC_lexicon_tags_14 },
    { 0xeb16ebcb, SUC_lexicon_tags_26 },
    { 0xe456ebcc, SUC_lexicon_tags_61 },
    { 0x57f6ebcc, SUC_lexicon_tags_118 },
    { 0x3416ebcd, SUC_lexicon_tags_37 },
    { 0x80f6ebce, SUC_lexicon_tags_29 },
    { 0x6e56ebcc, SUC_lexicon_tags_17 },
    { 0x9ff6ebd1, SUC_lexicon_tags_17 },
    { 0x7a96ebd1, SUC_lexicon_tags_1 },
    { 0x6c56ebd3, SUC_lexicon_tags_1 },
    { 0xc8b6ebca, SUC_lexicon_tags_38 },
    { 0x9376ebd0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7556ebd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6316ebdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d6ebdf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96ebe8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdab6ebeb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe436ebed, SUC_lexicon_tags_12 },
    { 0xc536ebee, SUC_lexicon_tags_37 },
    { 0xeb16ebee, SUC_lexicon_tags_38 },
    { 0x7db6ebf0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e56ebf4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeed6ebf6, SUC_lexicon_tags_2 },
    { 0x6376ebf6, SUC_lexicon_tags_11 },
    { 0xffd6ebf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcff6ebfa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b6ec03, SUC_lexicon_tags_15 },
    { 0x4a36ec03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5976ec07, SUC_lexicon_tags_28 },
    { 0xea16ec07, SUC_lexicon_tags_12 },
    { 0xdd56ec08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9956ec0b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc056ec0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6236ec0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd36ec13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab6ec1b, SUC_lexicon_tags_2 },
    { 0x8936ec1b, SUC_lexicon_tags_37 },
    { 0x7656ec1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd456ec20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4276ec2c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f6ec30, SUC_lexicon_tags_25 },
    { 0xb076ec31, SUC_lexicon_tags_16 },
    { 0x4036ec32, SUC_lexicon_tags_1 },
    { 0xc6b6ec32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd416ec37, SUC_lexicon_tags_12 },
    { 0x3e76ec37, SUC_lexicon_tags_56 },
    { 0x2656ec37, SUC_lexicon_tags_30 },
    { 0xf936ec3a, SUC_lexicon_tags_1 },
    { 0xfa16ec3b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3856ec41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7456ec47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x696ec49, SUC_lexicon_tags_106 },
    { 0x15d6ec4a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf076ec4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc156ec53, SUC_lexicon_tags_82 },
    { 0x6f16ec53, SUC_lexicon_tags_91 },
    { 0x5fd6ec55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda16ec57, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6996ec59, SUC_lexicon_tags_70 },
    { 0x3e96ec5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2ef6ec5c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f6ec61, SUC_lexicon_tags_13 },
    { 0x4776ec62, SUC_lexicon_tags_7 },
    { 0x3236ec63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d6ec66, SUC_lexicon_tags_37 },
    { 0xecf6ec67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5dd6ec69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a56ec6b, SUC_lexicon_tags_7 },
    { 0x74b6ec6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b96ec6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25f6ec71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1476ec73, SUC_lexicon_tags_65 },
    { 0x76d6ec73, SUC_lexicon_tags_2 },
    { 0x96b6ec75, SUC_lexicon_tags_1 },
    { 0xa776ec76, SUC_lexicon_tags_23 },
    { 0xbab6ec77, SUC_lexicon_tags_30 },
    { 0x1256ec76, SUC_lexicon_tags_7 },
    { 0xa336ec77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea36ec7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa76ec7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfcb6ec80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1836ec83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b6ec87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d96ec8d, SUC_lexicon_tags_33 },
    { 0x7356ec8e, SUC_lexicon_tags_27 },
    { 0xe1f6ec8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc616ec91, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d6ec94, SUC_lexicon_tags_62 },
    { 0x2fd6ec95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f36ec9a, SUC_lexicon_tags_25 },
    { 0xa056ec9a, SUC_lexicon_tags_0 },
    { 0xe9b6ec9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba56eca2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b6eca5, SUC_lexicon_tags_3 },
    { 0xcf36eca6, SUC_lexicon_tags_2 },
    { 0x27f6eca7, SUC_lexicon_tags_25 },
    { 0x5d76eca8, SUC_lexicon_tags_47 },
    { 0x4a96eca7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d6ecac, SUC_lexicon_tags_43 },
    { 0x21d6ecad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8136ecb3, SUC_lexicon_tags_37 },
    { 0x19d6ecb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cf6ecb6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4756ecb8, SUC_lexicon_tags_31 },
    { 0x44d6ecb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e16ecbc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd6ecbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4896ecc3, SUC_lexicon_tags_37 },
    { 0x95d6ecc4, SUC_lexicon_tags_1 },
    { 0x7d6ecc5, SUC_lexicon_tags_15 },
    { 0xcdb6ecc5, SUC_lexicon_tags_66 },
    { 0xe7b6ecc3, SUC_lexicon_tags_28 },
    { 0xc276ecc8, SUC_lexicon_tags_46 },
    { 0x1396ecc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf16eccb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xde56eccd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x416ecd7, SUC_lexicon_tags_31 },
    { 0xcfb6ecd8, SUC_lexicon_tags_60 },
    { 0xe716ecd9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd96ecdc, SUC_lexicon_tags_25 },
    { 0xecf6ecdd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f6ece0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4396ece3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e56ece5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f6ecec, SUC_lexicon_tags_12 },
    { 0x2d56eced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9376ecef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2896ecf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1136ecf7, SUC_lexicon_tags_15 },
    { 0xd4b6ecf8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xde76ecfb, SUC_lexicon_tags_23 },
    { 0x5796ecfc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d6ed02, SUC_lexicon_tags_15 },
    { 0x9b6ed02, SUC_lexicon_tags_25 },
    { 0x136ed04, SUC_lexicon_tags_14 },
    { 0x21d6ed04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc36ed0b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xca36ed0d, SUC_lexicon_tags_37 },
    { 0x9f36ed0d, SUC_lexicon_tags_62 },
    { 0xe2f6ed0e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f6ed17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d6ed1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b96ed22, SUC_lexicon_tags_33 },
    { 0xc896ed23, SUC_lexicon_tags_2 },
    { 0x9a36ed24, SUC_lexicon_tags_37 },
    { 0x2256ed25, SUC_lexicon_tags_1 },
    { 0x1056ed22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ed2a, SUC_lexicon_tags_46 },
    { 0x5cd6ed2b, SUC_lexicon_tags_37 },
    { 0x1b16ed2b, SUC_lexicon_tags_25 },
    { 0x84d6ed2a, SUC_lexicon_tags_27 },
    { 0x8e36ed2a, SUC_lexicon_tags_100 },
    { 0xcf76ed2f, SUC_lexicon_tags_17 },
    { 0x6756ed2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf456ed34, SUC_lexicon_tags_4 },
    { 0xa196ed34, SUC_lexicon_tags_17 },
    { 0x116ed36, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe376ed3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d6ed45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa496ed47, SUC_lexicon_tags_2 },
    { 0x4e36ed48, SUC_lexicon_tags_37 },
    { 0x9176ed48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa016ed4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9816ed4f, SUC_lexicon_tags_9 },
    { 0x5e36ed4f, SUC_lexicon_tags_29 },
    { 0xe9d6ed51, SUC_lexicon_tags_29 },
    { 0x4876ed52, SUC_lexicon_tags_31 },
    { 0xee76ed52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5d76ed55, SUC_lexicon_tags_2 },
    { 0xbd16ed56, SUC_lexicon_tags_252 },
    { 0x4316ed57, SUC_lexicon_tags_87 },
    { 0x2296ed56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff6ed5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6256ed5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x98d6ed60, SUC_lexicon_tags_38 },
    { 0xa3b6ed61, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2476ed6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3776ed73, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0xd5f6ed75, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d6ed78, SUC_lexicon_tags_46 },
    { 0x31d6ed79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6036ed7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4976ed7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb6ed85, SUC_lexicon_tags_47 },
    { 0xdb16ed86, SUC_lexicon_tags_28 },
    { 0xf3f6ed86, SUC_lexicon_tags_35 },
    { 0xc76ed86, SUC_lexicon_tags_29 },
    { 0x97b6ed89, SUC_lexicon_tags_27 },
    { 0x716ed8a, SUC_lexicon_tags_27 },
    { 0x2656ed89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b76ed8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b6ed92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf736ed95, SUC_lexicon_tags_728 },
    { 0xd916ed96, SUC_lexicon_tags_33 },
    { 0xc636ed95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1b6ed99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f76ed9c, SUC_lexicon_tags_43 },
    { 0x6a76ed9d, SUC_lexicon_tags_26 },
    { 0x8176ed9e, SUC_lexicon_tags_2 },
    { 0x5e56ed9f, SUC_lexicon_tags_9 },
    { 0xcf6ed9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd6eda3, SUC_lexicon_tags_25 },
    { 0x2556eda3, SUC_lexicon_tags_55 },
    { 0x1976eda3, SUC_lexicon_tags_25 },
    { 0x676eda6, SUC_lexicon_tags_16 },
    { 0x48b6eda3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d6edad, SUC_lexicon_tags_30 },
    { 0x336edae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd56edb4, SUC_lexicon_tags_40 },
    { 0x7d76edb5, SUC_lexicon_tags_1 },
    { 0xe2d6edb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b6edbb, SUC_lexicon_tags_30 },
    { 0x856edbc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4196edbe, SUC_lexicon_tags_33 },
    { 0x4376edbf, SUC_lexicon_tags_46 },
    { 0x54d6edc0, SUC_lexicon_tags_49 },
    { 0x2f76edc1, SUC_lexicon_tags_23 },
    { 0xb076edc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f6edc6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2436edca, SUC_lexicon_tags_2 },
    { 0xa856edcb, SUC_lexicon_tags_7 },
    { 0xc536edcc, SUC_lexicon_tags_7 },
    { 0xb9d6edcd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b6edd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab6edd3, SUC_lexicon_tags_38 },
    { 0x7fd6edd4, SUC_lexicon_tags_15 },
    { 0xbaf6edd5, SUC_lexicon_tags_17 },
    { 0xe956edd6, SUC_lexicon_tags_17 },
    { 0x38b6edd7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb856edd9, SUC_lexicon_tags_19 },
    { 0x3c16edd9, SUC_lexicon_tags_12 },
    { 0x1896eddb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8356eddd, SUC_lexicon_tags_2 },
    { 0x5636edde, SUC_lexicon_tags_15 },
    { 0x4a76edde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82f6ede1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b6ede5, SUC_lexicon_tags_46 },
    { 0x8056ede6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb16ede9, SUC_lexicon_tags_9 },
    { 0xba96edea, SUC_lexicon_tags_7 },
    { 0x15f6edeb, SUC_lexicon_tags_28 },
    { 0xcab6edec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc96edee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b6edf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f6edf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d6edfa, SUC_lexicon_tags_25 },
    { 0x7076edfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad6edff, SUC_lexicon_tags_26 },
    { 0x49b6edff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb6ee03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b6ee06, SUC_lexicon_tags_91 },
    { 0x7836ee07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb6ee0d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x81b6ee0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7336ee14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa116ee18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f36ee1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1096ee1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97b6ee1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a56ee27, SUC_lexicon_tags_25 },
    { 0x4616ee27, SUC_lexicon_tags_32 },
    { 0xac76ee28, SUC_lexicon_tags_121 },
    { 0x7cf6ee2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9716ee2c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12b6ee2e, SUC_lexicon_tags_11 },
    { 0x9f36ee2f, SUC_lexicon_tags_138 },
    { 0x8e36ee30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f56ee33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b6ee37, SUC_lexicon_tags_74 },
    { 0x4a56ee38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6d6ee3a, SUC_lexicon_tags_11 },
    { 0xd3d6ee3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc736ee3e, SUC_lexicon_tags_33 },
    { 0x7816ee3e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2736ee42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd816ee45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3136ee48, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb156ee4a, SUC_lexicon_tags_46 },
    { 0xd576ee4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd976ee4e, SUC_lexicon_tags_7 },
    { 0xfd56ee4f, SUC_lexicon_tags_26 },
    { 0x6196ee4e, SUC_lexicon_tags_25 },
    { 0xc796ee50, SUC_lexicon_tags_27 },
    { 0xf9f6ee4e, SUC_lexicon_tags_21 },
    { 0xaa16ee50, SUC_lexicon_tags_1 },
    { 0x7456ee54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xddf6ee56, SUC_lexicon_tags_15 },
    { 0x5f36ee57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacf6ee59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f36ee5d, SUC_lexicon_tags_19 },
    { 0xbcd6ee5e, SUC_lexicon_tags_45 },
    { 0x1216ee5e, SUC_lexicon_tags_27 },
    { 0x2e76ee5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc956ee6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b6ee74, SUC_lexicon_tags_30 },
    { 0xaa36ee75, SUC_lexicon_tags_11 },
    { 0xe096ee76, SUC_lexicon_tags_17 },
    { 0xc456ee75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf6ee7b, SUC_lexicon_tags_15 },
    { 0x6796ee7b, SUC_lexicon_tags_38 },
    { 0x5d76ee7d, SUC_lexicon_tags_25 },
    { 0x2036ee7e, SUC_lexicon_tags_30 },
    { 0xedb6ee7f, SUC_lexicon_tags_7 },
    { 0x9e76ee7e, SUC_lexicon_tags_23 },
    { 0xfe16ee7f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f6ee85, SUC_lexicon_tags_28 },
    { 0xf796ee86, SUC_lexicon_tags_25 },
    { 0x2b16ee87, SUC_lexicon_tags_45 },
    { 0x48f6ee87, SUC_lexicon_tags_40 },
    { 0x8ef6ee89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b6ee8c, SUC_lexicon_tags_15 },
    { 0x7a36ee8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9476ee8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4636ee93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4816ee95, SUC_lexicon_tags_10 },
    { 0x3856ee95, SUC_lexicon_tags_12 },
    { 0x3af6ee97, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d56ee9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93b6ee9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b36eea5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd7f6eea7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a16eeaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4016eead, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9d56eeaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d76eeb1, SUC_lexicon_tags_27 },
    { 0xb3d6eeb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1236eeb6, SUC_lexicon_tags_46 },
    { 0x3d16eeb6, SUC_lexicon_tags_13 },
    { 0x7856eeb8, SUC_lexicon_tags_87 },
    { 0x4dd6eeb7, SUC_lexicon_tags_12 },
    { 0x16eeb8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c56eebf, SUC_lexicon_tags_1 },
    { 0x4c16eebf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd0f6eec2, SUC_lexicon_tags_31 },
    { 0x4236eec2, SUC_lexicon_tags_25 },
    { 0xd116eec2, SUC_lexicon_tags_14 },
    { 0xf236eec4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b6eeca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b6eece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d16eed3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8536eed6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe116eedb, SUC_lexicon_tags_11 },
    { 0x7436eedc, SUC_lexicon_tags_13 },
    { 0xd836eedc, SUC_lexicon_tags_87 },
    { 0x4c96eede, SUC_lexicon_tags_37 },
    { 0x7096eedf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d6eee3, SUC_lexicon_tags_14 },
    { 0xe6f6eee4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e16eee8, SUC_lexicon_tags_9 },
    { 0xc576eee9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd6eeed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6216eef0, SUC_lexicon_tags_25 },
    { 0xbe36eef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2816eef4, SUC_lexicon_tags_31 },
    { 0x1eb6eef4, SUC_lexicon_tags_12 },
    { 0xb196eef6, SUC_lexicon_tags_27 },
    { 0xc9f6eef4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6536ef00, SUC_lexicon_tags_12 },
    { 0xf796ef00, SUC_lexicon_tags_38 },
    { 0x63b6ef00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb016ef04, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d16ef06, SUC_lexicon_tags_13 },
    { 0x3f36ef07, SUC_lexicon_tags_47 },
    { 0x6316ef08, SUC_lexicon_tags_45 },
    { 0x44f6ef09, SUC_lexicon_tags_1 },
    { 0x6d36ef0a, SUC_lexicon_tags_30 },
    { 0x1596ef0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb76ef0e, SUC_lexicon_tags_11 },
    { 0xd3d6ef0f, SUC_lexicon_tags_23 },
    { 0x4696ef10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2ad6ef12, SUC_lexicon_tags_12 },
    { 0x62d6ef13, SUC_lexicon_tags_17 },
    { 0xe576ef13, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6c16ef16, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d6ef1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5016ef1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb016ef22, SUC_lexicon_tags_26 },
    { 0xf516ef23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf96ef25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab6ef28, SUC_lexicon_tags_38 },
    { 0x9b96ef29, SUC_lexicon_tags_33 },
    { 0x82d6ef2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf456ef30, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd6ef33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf476ef35, SUC_lexicon_tags_43 },
    { 0xd596ef36, SUC_lexicon_tags_37 },
    { 0x2836ef37, SUC_lexicon_tags_26 },
    { 0x8476ef38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7116ef3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18d6ef3d, SUC_lexicon_tags_17 },
    { 0x97f6ef3e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6bd6ef40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x23f6ef42, SUC_lexicon_tags_11 },
    { 0x2cf6ef42, SUC_lexicon_tags_29 },
    { 0xe8f6ef43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b6ef47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f6ef4a, SUC_lexicon_tags_31 },
    { 0xd56ef4a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3cb6ef4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4716ef4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2376ef54, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd6ef57, SUC_lexicon_tags_1 },
    { 0x476ef57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe36ef5a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x68d6ef5c, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8496ef61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4216ef63, SUC_lexicon_tags_63 },
    { 0xd5d6ef64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b6ef67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5bd6ef69, SUC_lexicon_tags_82 },
    { 0x4816ef69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb636ef79, SUC_lexicon_tags_17 },
    { 0xfe56ef7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b6ef7f, SUC_lexicon_tags_43 },
    { 0x4eb6ef80, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f6ef83, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa596ef85, SUC_lexicon_tags_19 },
    { 0xd576ef86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3196ef89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f56ef8c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1076ef90, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc16ef93, SUC_lexicon_tags_6 },
    { 0xfb16ef94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd6ef98, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xfd6ef9a, SUC_lexicon_tags_65 },
    { 0x2d16ef9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b6ef9e, SUC_lexicon_tags_21 },
    { 0x5176ef9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c36efa5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcad6efa7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f6efae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff76efb1, SUC_lexicon_tags_0 },
    { 0xdf16efb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7696efb4, SUC_lexicon_tags_2 },
    { 0x5e16efb5, SUC_lexicon_tags_25 },
    { 0xfad6efb6, SUC_lexicon_tags_14 },
    { 0x2e76efb7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xda36efb9, SUC_lexicon_tags_14 },
    { 0x6316efba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf316efbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab6efc5, SUC_lexicon_tags_53 },
    { 0x49d6efc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b6efca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2236efcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc416efd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f36efd3, SUC_lexicon_tags_2 },
    { 0xceb6efd3, SUC_lexicon_tags_34 },
    { 0x7616efd5, SUC_lexicon_tags_19 },
    { 0x5db6efd5, SUC_lexicon_tags_165 },
    { 0x6e96efd6, SUC_lexicon_tags_12 },
    { 0x90b6efd8, SUC_lexicon_tags_9 },
    { 0x21b6efd9, SUC_lexicon_tags_46 },
    { 0x2a76efd9, SUC_lexicon_tags_25 },
    { 0x5656efd9, SUC_lexicon_tags_19 },
    { 0xc96efd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7656efe0, SUC_lexicon_tags_17 },
    { 0x99b6efe0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6876efe6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d6efea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b6eff0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d6eff3, SUC_lexicon_tags_25 },
    { 0x6ed6eff4, SUC_lexicon_tags_8 },
    { 0xe156eff4, SUC_lexicon_tags_47 },
    { 0x1ad6eff6, SUC_lexicon_tags_66 },
    { 0x2a56eff7, SUC_lexicon_tags_47 },
    { 0x2eb6eff5, SUC_lexicon_tags_17 },
    { 0x7476eff9, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xc936effb, SUC_lexicon_tags_160 },
    { 0xdbb6effc, SUC_lexicon_tags_15 },
    { 0xf16effd, SUC_lexicon_tags_6 },
    { 0x6a56effe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8796f003, SUC_lexicon_tags_38 },
    { 0x1776f004, SUC_lexicon_tags_49 },
    { 0xd2f6f005, SUC_lexicon_tags_46 },
    { 0xa816f003, SUC_lexicon_tags_28 },
    { 0x8936f004, SUC_lexicon_tags_31 },
    { 0x13b6f006, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1e16f00a, SUC_lexicon_tags_26 },
    { 0x7776f00a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e36f00d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a56f010, SUC_lexicon_tags_2 },
    { 0x2f6f010, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd16f018, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b6f01b, SUC_lexicon_tags_61 },
    { 0xb256f01c, SUC_lexicon_tags_2 },
    { 0x676f01b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa96f01f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b6f023, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab6f026, SUC_lexicon_tags_11 },
    { 0x9d56f026, SUC_lexicon_tags_15 },
    { 0x1e56f026, SUC_lexicon_tags_63 },
    { 0xd7d6f026, SUC_lexicon_tags_17 },
    { 0xab56f02a, SUC_lexicon_tags_47 },
    { 0x3836f02b, SUC_lexicon_tags_19 },
    { 0xb956f02c, SUC_lexicon_tags_8 },
    { 0x9df6f02b, SUC_lexicon_tags_17 },
    { 0xbaf6f02e, SUC_lexicon_tags_19 },
    { 0x71d6f02e, SUC_lexicon_tags_126 },
    { 0xc916f028, SUC_lexicon_tags_14 },
    { 0x4076f02b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd256f033, SUC_lexicon_tags_14 },
    { 0xbb16f033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6896f037, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3056f03a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd6f041, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9576f044, SUC_lexicon_tags_11 },
    { 0xd656f044, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3596f047, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e76f04c, SUC_lexicon_tags_29 },
    { 0x4976f04c, SUC_lexicon_tags_38 },
    { 0x5bd6f04e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6736f050, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f6f053, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b76f058, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf6f05d, SUC_lexicon_tags_1 },
    { 0x2936f05e, SUC_lexicon_tags_28 },
    { 0xef16f05f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed6f062, SUC_lexicon_tags_23 },
    { 0xd356f063, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x12d6f065, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd6f06a, SUC_lexicon_tags_12 },
    { 0x316f06b, SUC_lexicon_tags_60 },
    { 0xeeb6f06c, SUC_lexicon_tags_17 },
    { 0xf9f6f06d, SUC_lexicon_tags_29 },
    { 0xff36f06e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9016f074, SUC_lexicon_tags_87 },
    { 0xcf56f075, SUC_lexicon_tags_60 },
    { 0x8e76f075, SUC_lexicon_tags_536 },
    { 0xc7b6f077, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef6f07c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab6f080, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafb6f082, SUC_lexicon_tags_2 },
    { 0xc736f082, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9176f087, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd396f092, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe216f099, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf96f09b, SUC_lexicon_tags_7 },
    { 0xa976f09c, SUC_lexicon_tags_1 },
    { 0x64f6f09c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf696f0a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5896f0ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1696f0b0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f6f0b4, SUC_lexicon_tags_57 },
    { 0x9c16f0b4, SUC_lexicon_tags_0 },
    { 0xb6f0b6, SUC_lexicon_tags_25 },
    { 0x99b6f0b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18f6f0b9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5116f0bb, SUC_lexicon_tags_79 },
    { 0xfa96f0bc, SUC_lexicon_tags_43 },
    { 0xb5b6f0bc, SUC_lexicon_tags_6 },
    { 0x4676f0be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c96f0c3, SUC_lexicon_tags_25 },
    { 0xb416f0c3, SUC_lexicon_tags_25 },
    { 0xb616f0c5, SUC_lexicon_tags_60 },
    { 0xb8d6f0c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdab6f0c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8056f0cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9696f0ce, SUC_lexicon_tags_14 },
    { 0x7d96f0cf, SUC_lexicon_tags_19 },
    { 0x4936f0d0, SUC_lexicon_tags_15 },
    { 0xb2f6f0ce, SUC_lexicon_tags_15 },
    { 0x8c56f0d1, SUC_lexicon_tags_15 },
    { 0x7656f0d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee36f0d5, SUC_lexicon_tags_7 },
    { 0x7696f0d6, SUC_lexicon_tags_16 },
    { 0x156f0d6, SUC_lexicon_tags_26 },
    { 0x5356f0d8, SUC_lexicon_tags_60 },
    { 0x57b6f0d9, SUC_lexicon_tags_10 },
    { 0x1216f0d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8256f0dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa516f0de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6536f0e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b6f0e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5356f0e6, SUC_lexicon_tags_30 },
    { 0xdeb6f0e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3816f0ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb5b6f0ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a96f0ee, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf6f0f5, SUC_lexicon_tags_39 },
    { 0x1d76f0f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1356f0f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d6f0fd, SUC_lexicon_tags_63 },
    { 0xa556f0fe, SUC_lexicon_tags_1 },
    { 0x1196f0ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3756f115, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c56f11a, SUC_lexicon_tags_17 },
    { 0x2b96f11a, SUC_lexicon_tags_60 },
    { 0x3e96f11b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4376f11e, SUC_lexicon_tags_1 },
    { 0x2a76f11f, SUC_lexicon_tags_11 },
    { 0x5cd6f120, SUC_lexicon_tags_33 },
    { 0x2456f11f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b6f124, SUC_lexicon_tags_57 },
    { 0xcbd6f125, SUC_lexicon_tags_47 },
    { 0x6e56f126, SUC_lexicon_tags_11 },
    { 0x196f127, SUC_lexicon_tags_37 },
    { 0x6616f128, SUC_lexicon_tags_17 },
    { 0x1b56f128, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef56f12d, SUC_lexicon_tags_25 },
    { 0x9476f12e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16f135, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a16f137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b16f139, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c16f13c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d6f143, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8476f145, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b6f149, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3396f14b, SUC_lexicon_tags_25 },
    { 0x9796f14b, SUC_lexicon_tags_43 },
    { 0x1636f14d, SUC_lexicon_tags_32 },
    { 0x8a36f14e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1236f153, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2976f156, SUC_lexicon_tags_87 },
    { 0xed6f157, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda76f159, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3eb6f15b, SUC_lexicon_tags_17 },
    { 0xb856f15c, SUC_lexicon_tags_26 },
    { 0x5956f15d, SUC_lexicon_tags_23 },
    { 0xb636f15c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x23d6f160, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4456f163, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3d6f165, SUC_lexicon_tags_46 },
    { 0x34b6f166, SUC_lexicon_tags_62 },
    { 0x4976f166, SUC_lexicon_tags_10 },
    { 0x7bf6f165, SUC_lexicon_tags_15 },
    { 0xc76f169, SUC_lexicon_tags_32 },
    { 0xe2f6f16a, SUC_lexicon_tags_27 },
    { 0xc56f16b, SUC_lexicon_tags_39 },
    { 0x95f6f16c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8cd6f16e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff6f171, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d96f174, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8176f178, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3cd6f17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4976f17e, SUC_lexicon_tags_15 },
    { 0x1d96f17f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b6f18b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfed6f18d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9b6f18f, SUC_lexicon_tags_1 },
    { 0x7fb6f190, SUC_lexicon_tags_7 },
    { 0x5cb6f191, SUC_lexicon_tags_26 },
    { 0xe396f190, SUC_lexicon_tags_35 },
    { 0x30b6f193, SUC_lexicon_tags_38 },
    { 0xd556f194, SUC_lexicon_tags_37 },
    { 0x8dd6f191, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f6f19a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b76f1a3, SUC_lexicon_tags_60 },
    { 0x8316f1a4, SUC_lexicon_tags_37 },
    { 0xcf76f1a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af6f1ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1836f1af, SUC_lexicon_tags_8 },
    { 0xaa36f1b0, SUC_lexicon_tags_15 },
    { 0xe376f1b1, SUC_lexicon_tags_47 },
    { 0xdc56f1af, SUC_lexicon_tags_11 },
    { 0x4e36f1b3, SUC_lexicon_tags_194 },
    { 0xb956f1b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc556f1b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd96f1bd, SUC_lexicon_tags_7 },
    { 0xcfd6f1be, SUC_lexicon_tags_39 },
    { 0xab56f1bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5776f1c6, SUC_lexicon_tags_38 },
    { 0x5d36f1c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b6f1cc, SUC_lexicon_tags_35 },
    { 0xa156f1cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa7b6f1cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff36f1d4, SUC_lexicon_tags_40 },
    { 0xad56f1d5, SUC_lexicon_tags_7 },
    { 0x3056f1d5, SUC_lexicon_tags_32 },
    { 0xbf36f1d7, SUC_lexicon_tags_32 },
    { 0xcb6f1d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1996f1db, SUC_lexicon_tags_19 },
    { 0x3436f1dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1596f1df, SUC_lexicon_tags_30 },
    { 0xcff6f1df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x57d6f1e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb136f1ed, SUC_lexicon_tags_11 },
    { 0x3b96f1ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f6f1f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe36f1f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3d6f1f9, SUC_lexicon_tags_27 },
    { 0x9e36f1fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf5d6f1fc, SUC_lexicon_tags_45 },
    { 0x4a96f1fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4816f200, SUC_lexicon_tags_92 },
    { 0x3bb6f200, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf096f207, SUC_lexicon_tags_38 },
    { 0xacd6f207, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c76f211, SUC_lexicon_tags_3 },
    { 0x5ad6f211, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d6f215, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df6f218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b6f220, SUC_lexicon_tags_17 },
    { 0xe96f221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3916f224, SUC_lexicon_tags_61 },
    { 0x8ef6f225, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b6f228, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff36f22e, SUC_lexicon_tags_1 },
    { 0xe896f22e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b6f232, SUC_lexicon_tags_11 },
    { 0xebd6f232, SUC_lexicon_tags_11 },
    { 0x7b96f234, SUC_lexicon_tags_15 },
    { 0x2236f234, SUC_lexicon_tags_25 },
    { 0x5036f235, SUC_lexicon_tags_2 },
    { 0xced6f233, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb36f239, SUC_lexicon_tags_30 },
    { 0x8516f23a, SUC_lexicon_tags_112 },
    { 0x6416f23b, SUC_lexicon_tags_1 },
    { 0xb376f23c, SUC_lexicon_tags_1 },
    { 0x5936f23c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd6f245, SUC_lexicon_tags_0 },
    { 0x3496f246, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf256f24a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3956f24c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf136f257, SUC_lexicon_tags_25 },
    { 0xb436f257, SUC_lexicon_tags_12 },
    { 0x1436f258, SUC_lexicon_tags_12 },
    { 0x6e76f25a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f6f25f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5076f266, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5276f269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8856f26b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x296f26f, SUC_lexicon_tags_31 },
    { 0x8536f270, SUC_lexicon_tags_19 },
    { 0x5896f271, SUC_lexicon_tags_11 },
    { 0xf6d6f271, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b6f27b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb6f27f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed16f282, SUC_lexicon_tags_23 },
    { 0x3616f282, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1536f288, SUC_lexicon_tags_14 },
    { 0x85f6f289, SUC_lexicon_tags_42 },
    { 0xed56f289, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b76f28e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3036f291, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d6f293, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df6f296, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5876f299, SUC_lexicon_tags_29 },
    { 0x22d6f299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63f6f29c, SUC_lexicon_tags_7 },
    { 0x8c16f29d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f6f2a0, SUC_lexicon_tags_39 },
    { 0xbf76f2a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xad96f2a3, SUC_lexicon_tags_29 },
    { 0x30d6f2a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3756f2a6, SUC_lexicon_tags_31 },
    { 0x3fb6f2a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6f2aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4036f2ad, SUC_lexicon_tags_66 },
    { 0x2736f2ae, SUC_lexicon_tags_171 },
    { 0xec16f2ad, SUC_lexicon_tags_38 },
    { 0xd36f2b0, SUC_lexicon_tags_17 },
    { 0x9dd6f2b0, SUC_lexicon_tags_11 },
    { 0x1676f2b0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f6f2b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc96f2b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae76f2bb, SUC_lexicon_tags_34 },
    { 0xf96f2bc, SUC_lexicon_tags_27 },
    { 0x6276f2bb, SUC_lexicon_tags_27 },
    { 0x27d6f2be, SUC_lexicon_tags_2 },
    { 0x80b6f2bc, SUC_lexicon_tags_33 },
    { 0xb5f6f2be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa8b6f2c2, SUC_lexicon_tags_65 },
    { 0xa036f2c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8956f2c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc856f2c8, SUC_lexicon_tags_37 },
    { 0xe716f2c9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9df6f2cb, SUC_lexicon_tags_11 },
    { 0xdd96f2cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd4d6f2ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe296f2d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96f2d7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x76f6f2d9, SUC_lexicon_tags_2 },
    { 0xf296f2da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58f6f2dc, SUC_lexicon_tags_16 },
    { 0x9616f2dd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3b6f2df, SUC_lexicon_tags_25 },
    { 0x1156f2e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc756f2e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b6f2eb, SUC_lexicon_tags_2 },
    { 0xc776f2eb, SUC_lexicon_tags_60 },
    { 0x6b6f2ed, SUC_lexicon_tags_23 },
    { 0x51d6f2ec, SUC_lexicon_tags_14 },
    { 0x8316f2ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3d6f2f1, SUC_lexicon_tags_33 },
    { 0x27f6f2f2, SUC_lexicon_tags_31 },
    { 0x3576f2f2, SUC_lexicon_tags_25 },
    { 0x96f2f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f6f2f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c56f2fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f36f305, SUC_lexicon_tags_16 },
    { 0x3336f306, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b6f309, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21f6f30b, SUC_lexicon_tags_17 },
    { 0x4956f30b, SUC_lexicon_tags_49 },
    { 0x5bd6f30d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc656f312, SUC_lexicon_tags_15 },
    { 0x69f6f313, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x22b6f315, SUC_lexicon_tags_53 },
    { 0xfdb6f316, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1076f319, SUC_lexicon_tags_17 },
    { 0x81f6f319, SUC_lexicon_tags_45 },
    { 0xe536f319, SUC_lexicon_tags_17 },
    { 0xf216f31c, SUC_lexicon_tags_25 },
    { 0xda96f31c, SUC_lexicon_tags_6 },
    { 0xbb96f31e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd316f322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1b6f324, SUC_lexicon_tags_26 },
    { 0xbb56f324, SUC_lexicon_tags_49 },
    { 0x7d36f326, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9536f328, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f6f32b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x496f32d, SUC_lexicon_tags_2 },
    { 0x3916f32e, SUC_lexicon_tags_7 },
    { 0xd56f32f, SUC_lexicon_tags_65 },
    { 0x50f6f330, SUC_lexicon_tags_27 },
    { 0x7036f331, SUC_lexicon_tags_49 },
    { 0xa496f330, SUC_lexicon_tags_45 },
    { 0x6f56f331, SUC_lexicon_tags_7 },
    { 0x1956f331, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf16f337, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d6f33a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4656f33e, SUC_lexicon_tags_45 },
    { 0x2e16f33f, SUC_lexicon_tags_15 },
    { 0x92d6f340, SUC_lexicon_tags_25 },
    { 0x7e16f341, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xaa36f343, SUC_lexicon_tags_26 },
    { 0xde56f344, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a16f346, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbfd6f348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6756f34e, SUC_lexicon_tags_7 },
    { 0x7636f34e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c76f353, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b6f35c, SUC_lexicon_tags_25 },
    { 0xa8b6f35d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe896f35f, SUC_lexicon_tags_43 },
    { 0x64b6f35f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb6f363, SUC_lexicon_tags_17 },
    { 0xa956f363, SUC_lexicon_tags_1679 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e76f368, SUC_lexicon_tags_62 },
    { 0x66f6f369, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4016f36c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4556f36f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb336f37b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x88b6f37d, SUC_lexicon_tags_49 },
    { 0x2596f37e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d6f382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf16f384, SUC_lexicon_tags_14 },
    { 0x9a96f385, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb6f389, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb6f38c, SUC_lexicon_tags_45 },
    { 0xc796f38d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb776f390, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa96f395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7576f399, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf876f39f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b6f3a2, SUC_lexicon_tags_14 },
    { 0x3576f3a2, SUC_lexicon_tags_23 },
    { 0xa076f3a4, SUC_lexicon_tags_25 },
    { 0x1276f3a3, SUC_lexicon_tags_17 },
    { 0x6616f3a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b76f3a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xafd6f3aa, SUC_lexicon_tags_40 },
    { 0xb2d6f3ab, SUC_lexicon_tags_12 },
    { 0xcf36f3ab, SUC_lexicon_tags_2 },
    { 0x1656f3ad, SUC_lexicon_tags_15 },
    { 0x7b56f3ac, SUC_lexicon_tags_19 },
    { 0xc236f3af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff76f3b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x936f3ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f6f3bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xccb6f3bf, SUC_lexicon_tags_19 },
    { 0xa756f3c0, SUC_lexicon_tags_25 },
    { 0xd0d6f3c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd6f3c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f36f3ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f6f3cf, SUC_lexicon_tags_14 },
    { 0xbcb6f3cf, SUC_lexicon_tags_11 },
    { 0x6936f3cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea36f3d5, SUC_lexicon_tags_15 },
    { 0x8fb6f3d6, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe56f3dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c56f3df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a96f3e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5416f3e6, SUC_lexicon_tags_26 },
    { 0x6bf6f3e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd336f3ea, SUC_lexicon_tags_12 },
    { 0xa956f3ea, SUC_lexicon_tags_46 },
    { 0xeb96f3ec, SUC_lexicon_tags_61 },
    { 0xc6d6f3ed, SUC_lexicon_tags_11 },
    { 0x27d6f3ee, SUC_lexicon_tags_43 },
    { 0x6876f3ef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7956f3f1, SUC_lexicon_tags_55 },
    { 0x4836f3f2, SUC_lexicon_tags_8 },
    { 0x11b6f3f3, SUC_lexicon_tags_12 },
    { 0x8ff6f3f1, SUC_lexicon_tags_45 },
    { 0x2236f3f3, SUC_lexicon_tags_17 },
    { 0x7736f3f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f96f3f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8016f3fc, SUC_lexicon_tags_14 },
    { 0xcb6f3fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb656f406, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f6f412, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d6f416, SUC_lexicon_tags_38 },
    { 0x1df6f417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb6f41a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5556f41c, SUC_lexicon_tags_62 },
    { 0x2bf6f41d, SUC_lexicon_tags_56 },
    { 0x7a96f41e, SUC_lexicon_tags_17 },
    { 0xcff6f41d, SUC_lexicon_tags_37 },
    { 0xf6d6f41e, SUC_lexicon_tags_33 },
    { 0x5c56f41f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf6f427, SUC_lexicon_tags_38 },
    { 0x4316f428, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xedd6f42a, SUC_lexicon_tags_60 },
    { 0xa656f42b, SUC_lexicon_tags_17 },
    { 0xedb6f42b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5756f430, SUC_lexicon_tags_37 },
    { 0x6cf6f431, SUC_lexicon_tags_17 },
    { 0x7436f432, SUC_lexicon_tags_30 },
    { 0x1f36f431, SUC_lexicon_tags_2 },
    { 0x9516f432, SUC_lexicon_tags_0 },
    { 0xddb6f435, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf956f439, SUC_lexicon_tags_7 },
    { 0x39d6f439, SUC_lexicon_tags_11 },
    { 0x8bd6f43b, SUC_lexicon_tags_17 },
    { 0x2af6f43b, SUC_lexicon_tags_33 },
    { 0x8bf6f43d, SUC_lexicon_tags_27 },
    { 0x6096f43e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb436f443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd616f445, SUC_lexicon_tags_10 },
    { 0x6a56f446, SUC_lexicon_tags_9 },
    { 0xd896f446, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4976f452, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d6f455, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbd56f457, SUC_lexicon_tags_31 },
    { 0x7d56f457, SUC_lexicon_tags_37 },
    { 0xeef6f459, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xba16f45b, SUC_lexicon_tags_19 },
    { 0x9fd6f45b, SUC_lexicon_tags_17 },
    { 0xa296f45d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1776f461, SUC_lexicon_tags_31 },
    { 0x5ef6f462, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb2b6f464, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d6f468, SUC_lexicon_tags_11 },
    { 0x896f469, SUC_lexicon_tags_11 },
    { 0x9916f46a, SUC_lexicon_tags_13 },
    { 0x1d36f46b, SUC_lexicon_tags_37 },
    { 0xc896f46a, SUC_lexicon_tags_1 },
    { 0xc0d6f46d, SUC_lexicon_tags_12 },
    { 0x9756f46e, SUC_lexicon_tags_17 },
    { 0xd736f468, SUC_lexicon_tags_11 },
    { 0xde56f470, SUC_lexicon_tags_27 },
    { 0xecf6f470, SUC_lexicon_tags_25 },
    { 0x3696f472, SUC_lexicon_tags_28 },
    { 0xe296f473, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb416f479, SUC_lexicon_tags_45 },
    { 0x4336f479, SUC_lexicon_tags_198 },
    { 0x5176f47b, SUC_lexicon_tags_83 },
    { 0xa6d6f47c, SUC_lexicon_tags_37 },
    { 0x4f6f47c, SUC_lexicon_tags_25 },
    { 0xc616f47c, SUC_lexicon_tags_1 },
    { 0xcfd6f479, SUC_lexicon_tags_45 },
    { 0x2736f47e, SUC_lexicon_tags_14 },
    { 0xacb6f47f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc76f485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4196f489, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc236f48c, SUC_lexicon_tags_10 },
    { 0x5056f48d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2136f491, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93f6f493, SUC_lexicon_tags_16 },
    { 0x1156f494, SUC_lexicon_tags_130 },
    { 0xbf6f495, SUC_lexicon_tags_28 },
    { 0x43d6f496, SUC_lexicon_tags_12 },
    { 0x8b36f497, SUC_lexicon_tags_2 },
    { 0xe536f498, SUC_lexicon_tags_4 },
    { 0xc636f498, SUC_lexicon_tags_25 },
    { 0x55d6f49a, SUC_lexicon_tags_37 },
    { 0xec76f497, SUC_lexicon_tags_49 },
    { 0xc276f49c, SUC_lexicon_tags_55 },
    { 0x5916f493, SUC_lexicon_tags_33 },
    { 0x2ff6f49e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c96f4a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2496f4a7, SUC_lexicon_tags_15 },
    { 0x97f6f4a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x62d6f4aa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7356f4ae, SUC_lexicon_tags_28 },
    { 0x8c56f4af, SUC_lexicon_tags_37 },
    { 0x7956f4b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8356f4b2, SUC_lexicon_tags_2 },
    { 0x8ad6f4b2, SUC_lexicon_tags_28 },
    { 0x21d6f4b4, SUC_lexicon_tags_49 },
    { 0x7fd6f4b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a76f4b9, SUC_lexicon_tags_31 },
    { 0x9f36f4b9, SUC_lexicon_tags_29 },
    { 0x4dd6f4bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f4c1, SUC_lexicon_tags_7 },
    { 0x6b56f4c2, SUC_lexicon_tags_2 },
    { 0x3656f4c3, SUC_lexicon_tags_1 },
    { 0x3756f4c1, SUC_lexicon_tags_144 },
    { 0xe016f4c1, SUC_lexicon_tags_104 },
    { 0xfdb6f4c6, SUC_lexicon_tags_26 },
    { 0xf8b6f4c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b56f4cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9596f4cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe416f4cf, SUC_lexicon_tags_92 },
    { 0x6c16f4cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8136f4d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa16f4da, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xed96f4dd, SUC_lexicon_tags_77 },
    { 0x17f6f4de, SUC_lexicon_tags_15 },
    { 0x1036f4dd, SUC_lexicon_tags_17 },
    { 0x9376f4de, SUC_lexicon_tags_49 },
    { 0x9456f4e1, SUC_lexicon_tags_11 },
    { 0xe816f4de, SUC_lexicon_tags_11 },
    { 0xf8b6f4e2, SUC_lexicon_tags_11 },
    { 0x98b6f4e4, SUC_lexicon_tags_11 },
    { 0x5d96f4e5, SUC_lexicon_tags_1 },
    { 0x4616f4e4, SUC_lexicon_tags_29 },
    { 0x4036f4e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b6f4ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3516f4ec, SUC_lexicon_tags_17 },
    { 0xa6f6f4ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70b6f4ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4156f4f3, SUC_lexicon_tags_39 },
    { 0xb796f4f3, SUC_lexicon_tags_60 },
    { 0xfc56f4f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66d6f4f7, SUC_lexicon_tags_1 },
    { 0x7256f4f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8f6f4fa, SUC_lexicon_tags_7 },
    { 0xaef6f4fb, SUC_lexicon_tags_39 },
    { 0x6b36f4fa, SUC_lexicon_tags_113 },
    { 0x69d6f4fd, SUC_lexicon_tags_60 },
    { 0x7216f4fe, SUC_lexicon_tags_82 },
    { 0x3f96f4ff, SUC_lexicon_tags_178 },
    { 0x2636f4ff, SUC_lexicon_tags_12 },
    { 0x2ad6f501, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f6f504, SUC_lexicon_tags_47 },
    { 0xb616f505, SUC_lexicon_tags_8 },
    { 0x28d6f504, SUC_lexicon_tags_12 },
    { 0xdd76f507, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b6f50b, SUC_lexicon_tags_4 },
    { 0xc976f50c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d16f50f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf876f514, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d6f521, SUC_lexicon_tags_10 },
    { 0x5d16f522, SUC_lexicon_tags_49 },
    { 0xee56f523, SUC_lexicon_tags_60 },
    { 0x3eb6f524, SUC_lexicon_tags_23 },
    { 0x7576f521, SUC_lexicon_tags_11 },
    { 0x31f6f526, SUC_lexicon_tags_33 },
    { 0xf016f527, SUC_lexicon_tags_47 },
    { 0xe256f526, SUC_lexicon_tags_12 },
    { 0xae76f523, SUC_lexicon_tags_37 },
    { 0x3db6f522, SUC_lexicon_tags_33 },
    { 0xfd6f52b, SUC_lexicon_tags_37 },
    { 0x42d6f52b, SUC_lexicon_tags_19 },
    { 0x356f52c, SUC_lexicon_tags_10 },
    { 0x3af6f52a, SUC_lexicon_tags_10 },
    { 0x6936f52f, SUC_lexicon_tags_12 },
    { 0x60f6f530, SUC_lexicon_tags_14 },
    { 0x8eb6f531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd336f533, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d6f53b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa36f540, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc556f542, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdb16f544, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0xed6f546, SUC_lexicon_tags_1 },
    { 0xd836f546, SUC_lexicon_tags_2 },
    { 0x2df6f547, SUC_lexicon_tags_149 },
    { 0x2356f549, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1976f54c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf576f54e, SUC_lexicon_tags_29 },
    { 0xb4d6f54f, SUC_lexicon_tags_7 },
    { 0xb4d6f550, SUC_lexicon_tags_69 },
    { 0xdcf6f551, SUC_lexicon_tags_2 },
    { 0xd196f552, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c56f554, SUC_lexicon_tags_8 },
    { 0xdf96f555, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc6f6f557, SUC_lexicon_tags_7 },
    { 0xae96f558, SUC_lexicon_tags_12 },
    { 0x1d96f559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8816f55d, SUC_lexicon_tags_14 },
    { 0xee76f55e, SUC_lexicon_tags_27 },
    { 0x5176f55f, SUC_lexicon_tags_17 },
    { 0xdb56f560, SUC_lexicon_tags_25 },
    { 0xe336f560, SUC_lexicon_tags_15 },
    { 0x3156f562, SUC_lexicon_tags_2 },
    { 0xeab6f561, SUC_lexicon_tags_2 },
    { 0x96b6f564, SUC_lexicon_tags_28 },
    { 0x6ab6f565, SUC_lexicon_tags_15 },
    { 0xf896f560, SUC_lexicon_tags_11 },
    { 0x1676f565, SUC_lexicon_tags_37 },
    { 0x6b56f568, SUC_lexicon_tags_22 },
    { 0x736f568, SUC_lexicon_tags_17 },
    { 0x4a56f568, SUC_lexicon_tags_16 },
    { 0xeb96f569, SUC_lexicon_tags_29 },
    { 0x8996f563, SUC_lexicon_tags_6 },
    { 0x52b6f56d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7276f56f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b16f574, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x176f576, SUC_lexicon_tags_31 },
    { 0x9cd6f577, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x516f57b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb6f57f, SUC_lexicon_tags_11 },
    { 0xb196f580, SUC_lexicon_tags_82 },
    { 0x7196f580, SUC_lexicon_tags_55 },
    { 0x7636f581, SUC_lexicon_tags_1 },
    { 0x9c76f583, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf936f586, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8336f58a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e76f58c, SUC_lexicon_tags_23 },
    { 0xa8b6f58d, SUC_lexicon_tags_25 },
    { 0x2556f58d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2f6f590, SUC_lexicon_tags_11 },
    { 0xb3b6f591, SUC_lexicon_tags_27 },
    { 0x9556f592, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7216f596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c76f599, SUC_lexicon_tags_37 },
    { 0xc156f59a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b36f5a0, SUC_lexicon_tags_12 },
    { 0xb7f6f5a1, SUC_lexicon_tags_30 },
    { 0xc736f5a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e96f5aa, SUC_lexicon_tags_15 },
    { 0xb976f5ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f6f5af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f76f5b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5216f5b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2116f5bc, SUC_lexicon_tags_25 },
    { 0x5276f5bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf96f5c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x856f5c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b6f5c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa556f5cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f6f5d7, SUC_lexicon_tags_25 },
    { 0x3cb6f5d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdb36f5da, SUC_lexicon_tags_55 },
    { 0x4b56f5db, SUC_lexicon_tags_49 },
    { 0xa7b6f5dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4256f5e3, SUC_lexicon_tags_63 },
    { 0x23d6f5e3, SUC_lexicon_tags_8 },
    { 0x4756f5e4, SUC_lexicon_tags_19 },
    { 0x736f5e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb6f5f0, SUC_lexicon_tags_17 },
    { 0xb7b6f5f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7476f5f4, SUC_lexicon_tags_14 },
    { 0x2096f5f5, SUC_lexicon_tags_21 },
    { 0x4696f5f6, SUC_lexicon_tags_17 },
    { 0x9036f5f7, SUC_lexicon_tags_37 },
    { 0x6076f5f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8736f5fe, SUC_lexicon_tags_34 },
    { 0xb2b6f5ff, SUC_lexicon_tags_12 },
    { 0xed96f5ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d6f604, SUC_lexicon_tags_32 },
    { 0x1cb6f605, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd496f609, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc676f60d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b6f610, SUC_lexicon_tags_1 },
    { 0xbc76f611, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca16f613, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac16f618, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb6f61b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc416f61e, SUC_lexicon_tags_10 },
    { 0x2556f61e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8276f624, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b6f628, SUC_lexicon_tags_25 },
    { 0x3db6f628, SUC_lexicon_tags_124 },
    { 0x2d76f62a, SUC_lexicon_tags_1 },
    { 0x58f6f629, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf0b6f62d, SUC_lexicon_tags_45 },
    { 0x4a56f62d, SUC_lexicon_tags_33 },
    { 0x9896f62f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3976f634, SUC_lexicon_tags_15 },
    { 0x3af6f635, SUC_lexicon_tags_38 },
    { 0x3b36f636, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef6f639, SUC_lexicon_tags_25 },
    { 0xf8d6f63a, SUC_lexicon_tags_37 },
    { 0xdcb6f63b, SUC_lexicon_tags_38 },
    { 0xdb16f63c, SUC_lexicon_tags_60 },
    { 0xdff6f63d, SUC_lexicon_tags_25 },
    { 0x8236f63d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e96f640, SUC_lexicon_tags_17 },
    { 0x8f76f641, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde36f645, SUC_lexicon_tags_28 },
    { 0xdbd6f645, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2396f648, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df6f64c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f6f653, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb6f658, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d6f65e, SUC_lexicon_tags_2 },
    { 0x56b6f65e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5fd6f661, SUC_lexicon_tags_35 },
    { 0xf436f661, SUC_lexicon_tags_82 },
    { 0xf1d6f663, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80b6f665, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd476f667, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3f6f669, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d96f66d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b6f674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a96f678, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46d6f67a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb6f67e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb76f682, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f36f686, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6276f688, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf6f68b, SUC_lexicon_tags_0 },
    { 0x7cf6f68c, SUC_lexicon_tags_16 },
    { 0xdf76f68d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x5b6f68f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb56f691, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5dd6f693, SUC_lexicon_tags_33 },
    { 0x4d56f694, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc716f698, SUC_lexicon_tags_6 },
    { 0x2256f699, SUC_lexicon_tags_55 },
    { 0x8c16f69a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd236f69d, SUC_lexicon_tags_1 },
    { 0x9b56f69d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf56f6a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86b6f6a9, SUC_lexicon_tags_37 },
    { 0xbe56f6aa, SUC_lexicon_tags_45 },
    { 0xc36f6aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d6f6af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8156f6bc, SUC_lexicon_tags_2 },
    { 0x5ab6f6bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d6f6c1, SUC_lexicon_tags_2 },
    { 0x976f6c2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b6f6c7, SUC_lexicon_tags_45 },
    { 0xa256f6c7, SUC_lexicon_tags_85 },
    { 0x36f6c7, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd6f6cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb6f6d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d36f6d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1596f6de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b6f6e1, SUC_lexicon_tags_11 },
    { 0x9ad6f6e2, SUC_lexicon_tags_55 },
    { 0xfe76f6e3, SUC_lexicon_tags_15 },
    { 0x15f6f6e4, SUC_lexicon_tags_29 },
    { 0xc3b6f6e3, SUC_lexicon_tags_1 },
    { 0x5296f6e2, SUC_lexicon_tags_2 },
    { 0x11f6f6e7, SUC_lexicon_tags_47 },
    { 0x5ff6f6e7, SUC_lexicon_tags_116 },
    { 0xf936f6e9, SUC_lexicon_tags_1 },
    { 0xaed6f6e7, SUC_lexicon_tags_25 },
    { 0x2cf6f6ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8856f6ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b6f6f0, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0xc4d6f6f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc396f6f5, SUC_lexicon_tags_38 },
    { 0xbf6f6f6, SUC_lexicon_tags_8 },
    { 0xacf6f6f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9896f709, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xca16f70c, SUC_lexicon_tags_25 },
    { 0xb8b6f70d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b6f714, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd396f720, SUC_lexicon_tags_12 },
    { 0xc7b6f721, SUC_lexicon_tags_14 },
    { 0x1216f722, SUC_lexicon_tags_82 },
    { 0xd896f723, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd6f729, SUC_lexicon_tags_19 },
    { 0x8936f72a, SUC_lexicon_tags_0 },
    { 0x3b76f72b, SUC_lexicon_tags_18 },
    { 0xcbf6f72a, SUC_lexicon_tags_0 },
    { 0x3956f72a, SUC_lexicon_tags_1 },
    { 0xac36f72e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e16f734, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd6f738, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf76f73c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2396f73f, SUC_lexicon_tags_1 },
    { 0xf836f740, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b96f742, SUC_lexicon_tags_2 },
    { 0x7616f743, SUC_lexicon_tags_11 },
    { 0xf3d6f744, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f6f747, SUC_lexicon_tags_32 },
    { 0x4ef6f747, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5876f74a, SUC_lexicon_tags_25 },
    { 0x5076f74a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd416f74e, SUC_lexicon_tags_68 },
    { 0x436f74f, SUC_lexicon_tags_33 },
    { 0x19b6f74f, SUC_lexicon_tags_0 },
    { 0x1896f751, SUC_lexicon_tags_26 },
    { 0x2356f752, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd6f755, SUC_lexicon_tags_1 },
    { 0x8736f755, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa016f758, SUC_lexicon_tags_31 },
    { 0x93b6f759, SUC_lexicon_tags_12 },
    { 0x89d6f759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbb6f75c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb896f75e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5816f760, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x656f763, SUC_lexicon_tags_0 },
    { 0x87d6f764, SUC_lexicon_tags_17 },
    { 0xf8d6f765, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x256f767, SUC_lexicon_tags_17 },
    { 0x34d6f767, SUC_lexicon_tags_11 },
    { 0x30f6f769, SUC_lexicon_tags_44 },
    { 0xd956f76a, SUC_lexicon_tags_60 },
    { 0xd856f76b, SUC_lexicon_tags_15 },
    { 0x7116f767, SUC_lexicon_tags_23 },
    { 0x6176f76d, SUC_lexicon_tags_15 },
    { 0x26b6f76d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x59b6f770, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6836f772, SUC_lexicon_tags_23 },
    { 0x5a96f773, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9316f777, SUC_lexicon_tags_26 },
    { 0x5bb6f778, SUC_lexicon_tags_6 },
    { 0x1ff6f779, SUC_lexicon_tags_43 },
    { 0x1f56f779, SUC_lexicon_tags_12 },
    { 0xc96f779, SUC_lexicon_tags_10 },
    { 0x54f6f77c, SUC_lexicon_tags_11 },
    { 0xa536f779, SUC_lexicon_tags_4 },
    { 0xed96f77e, SUC_lexicon_tags_25 },
    { 0x6356f77f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4976f781, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a36f784, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3856f786, SUC_lexicon_tags_17 },
    { 0x3e16f787, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4cb6f789, SUC_lexicon_tags_7 },
    { 0x3c56f789, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb096f78c, SUC_lexicon_tags_37 },
    { 0xffb6f78d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe116f790, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9516f792, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d6f796, SUC_lexicon_tags_26 },
    { 0xb9f6f796, SUC_lexicon_tags_49 },
    { 0x2b6f798, SUC_lexicon_tags_2 },
    { 0x7bd6f799, SUC_lexicon_tags_45 },
    { 0x196f799, SUC_lexicon_tags_28 },
    { 0x5cd6f79b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b6f7a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b6f7a5, SUC_lexicon_tags_12 },
    { 0xb3f6f7a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc256f7a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc16f7ab, SUC_lexicon_tags_140 },
    { 0x156f7ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf56f7ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5356f7b0, SUC_lexicon_tags_60 },
    { 0xc36f7b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa816f7b4, SUC_lexicon_tags_8 },
    { 0x9696f7b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe356f7b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe776f7ba, SUC_lexicon_tags_17 },
    { 0x28d6f7bb, SUC_lexicon_tags_38 },
    { 0x8396f7bc, SUC_lexicon_tags_15 },
    { 0x1336f7bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc896f7bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f6f7c3, SUC_lexicon_tags_20 },
    { 0xc976f7c4, SUC_lexicon_tags_17 },
    { 0x4356f7c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db6f7cc, SUC_lexicon_tags_47 },
    { 0xb436f7cd, SUC_lexicon_tags_43 },
    { 0xc6d6f7cc, SUC_lexicon_tags_37 },
    { 0x34b6f7ce, SUC_lexicon_tags_37 },
    { 0xcbb6f7ce, SUC_lexicon_tags_2 },
    { 0x5636f7d1, SUC_lexicon_tags_9 },
    { 0xd3b6f7ce, SUC_lexicon_tags_10 },
    { 0xe7b6f7d3, SUC_lexicon_tags_83 },
    { 0xeb76f7d4, SUC_lexicon_tags_3 },
    { 0xfe96f7d5, SUC_lexicon_tags_7 },
    { 0x5336f7d6, SUC_lexicon_tags_13 },
    { 0xb036f7d6, SUC_lexicon_tags_59 },
    { 0xe36f7d6, SUC_lexicon_tags_7 },
    { 0x9d96f7d9, SUC_lexicon_tags_45 },
    { 0xb2b6f7d1, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa616f7dd, SUC_lexicon_tags_16 },
    { 0x34d6f7de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8a96f7e0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x16f7e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6f6f7e4, SUC_lexicon_tags_32 },
    { 0x4cf6f7e5, SUC_lexicon_tags_42 },
    { 0x8ef6f7e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f6f7e9, SUC_lexicon_tags_63 },
    { 0x20f6f7ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63d6f7ec, SUC_lexicon_tags_30 },
    { 0x5276f7ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d6f7f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f6f7f9, SUC_lexicon_tags_79 },
    { 0x2cb6f7f9, SUC_lexicon_tags_15 },
    { 0x80d6f7fb, SUC_lexicon_tags_17 },
    { 0xc8d6f7fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb6f7ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe056f805, SUC_lexicon_tags_77 },
    { 0x5e56f806, SUC_lexicon_tags_23 },
    { 0xf996f806, SUC_lexicon_tags_7 },
    { 0x956f808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f6f80d, SUC_lexicon_tags_17 },
    { 0x48b6f80e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b6f811, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3d6f813, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b6f81d, SUC_lexicon_tags_0 },
    { 0xd096f81e, SUC_lexicon_tags_30 },
    { 0xc2b6f81d, SUC_lexicon_tags_39 },
    { 0x8b56f820, SUC_lexicon_tags_15 },
    { 0x3f16f821, SUC_lexicon_tags_28 },
    { 0x3056f820, SUC_lexicon_tags_3 },
    { 0xdc56f81f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d6f82a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe376f82e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe76f835, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcf36f837, SUC_lexicon_tags_15 },
    { 0xea96f837, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9076f83c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61d6f83e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f6f842, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b6f845, SUC_lexicon_tags_25 },
    { 0x3e36f846, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6956f84c, SUC_lexicon_tags_47 },
    { 0x7c36f84d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34d6f84f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0b6f851, SUC_lexicon_tags_17 },
    { 0x2cd6f852, SUC_lexicon_tags_61 },
    { 0x90b6f852, SUC_lexicon_tags_38 },
    { 0x47d6f854, SUC_lexicon_tags_7 },
    { 0x9416f855, SUC_lexicon_tags_23 },
    { 0x3056f856, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b6f85a, SUC_lexicon_tags_33 },
    { 0x2b76f85b, SUC_lexicon_tags_38 },
    { 0x1356f85c, SUC_lexicon_tags_15 },
    { 0xf9b6f85d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9576f860, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34f6f862, SUC_lexicon_tags_30 },
    { 0xff16f863, SUC_lexicon_tags_25 },
    { 0x9ed6f863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd96f869, SUC_lexicon_tags_43 },
    { 0x3156f869, SUC_lexicon_tags_23 },
    { 0x6cb6f86b, SUC_lexicon_tags_1 },
    { 0x4d56f86a, SUC_lexicon_tags_17 },
    { 0xf056f86d, SUC_lexicon_tags_6 },
    { 0x5d56f86e, SUC_lexicon_tags_47 },
    { 0x1df6f86f, SUC_lexicon_tags_17 },
    { 0x8856f870, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b6f875, SUC_lexicon_tags_17 },
    { 0x4a16f875, SUC_lexicon_tags_1 },
    { 0x6056f877, SUC_lexicon_tags_9 },
    { 0xdeb6f878, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bb6f87a, SUC_lexicon_tags_1 },
    { 0xa76f87b, SUC_lexicon_tags_7 },
    { 0x6516f87a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df6f87f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe496f882, SUC_lexicon_tags_29 },
    { 0xc996f882, SUC_lexicon_tags_13 },
    { 0xf476f883, SUC_lexicon_tags_11 },
    { 0xab56f885, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae96f889, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8496f88d, SUC_lexicon_tags_12 },
    { 0x2656f88d, SUC_lexicon_tags_60 },
    { 0x8316f88f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d6f892, SUC_lexicon_tags_107 },
    { 0x17d6f892, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb676f895, SUC_lexicon_tags_90 },
    { 0xd556f896, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd6f89b, SUC_lexicon_tags_29 },
    { 0x92b6f89c, SUC_lexicon_tags_8 },
    { 0xe0d6f89d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf6f8a1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf6f8a4, SUC_lexicon_tags_15 },
    { 0x6c76f8a5, SUC_lexicon_tags_38 },
    { 0xe7b6f8a6, SUC_lexicon_tags_12 },
    { 0x3176f8a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9396f8a9, SUC_lexicon_tags_8 },
    { 0xe3f6f8aa, SUC_lexicon_tags_43 },
    { 0x4b76f8ab, SUC_lexicon_tags_60 },
    { 0x496f8aa, SUC_lexicon_tags_6 },
    { 0x2776f8ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd516f8b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf56f8b5, SUC_lexicon_tags_39 },
    { 0xdc16f8b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8956f8ba, SUC_lexicon_tags_15 },
    { 0xb0b6f8ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3676f8bf, SUC_lexicon_tags_11 },
    { 0x8616f8c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b6f8c4, SUC_lexicon_tags_1 },
    { 0x5556f8c4, SUC_lexicon_tags_63 },
    { 0x9436f8c6, SUC_lexicon_tags_1 },
    { 0x7956f8c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa316f8cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbbf6f8cf, SUC_lexicon_tags_29 },
    { 0x1c96f8cf, SUC_lexicon_tags_25 },
    { 0xb56f8d1, SUC_lexicon_tags_43 },
    { 0xef16f8cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2f6f8d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8ad6f8d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd16f8d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe296f8dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3176f8e0, SUC_lexicon_tags_65 },
    { 0x5e16f8e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd6f8e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9676f8e7, SUC_lexicon_tags_12 },
    { 0xf136f8e7, SUC_lexicon_tags_57 },
    { 0x6af6f8e9, SUC_lexicon_tags_34 },
    { 0xa8b6f8e8, SUC_lexicon_tags_2 },
    { 0xdc76f8e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d6f8f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xad16f8f3, SUC_lexicon_tags_1 },
    { 0x8cf6f8f4, SUC_lexicon_tags_37 },
    { 0x31b6f8f5, SUC_lexicon_tags_2 },
    { 0xdef6f8f6, SUC_lexicon_tags_9 },
    { 0x6d96f8f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa276f901, SUC_lexicon_tags_47 },
    { 0xbe56f902, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d6f907, SUC_lexicon_tags_19 },
    { 0xcfd6f908, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc36f90b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc876f90e, SUC_lexicon_tags_49 },
    { 0xd616f90f, SUC_lexicon_tags_30 },
    { 0x8436f90e, SUC_lexicon_tags_43 },
    { 0xaf96f911, SUC_lexicon_tags_85 },
    { 0x7356f911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e56f914, SUC_lexicon_tags_7 },
    { 0x8736f914, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff6f918, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa636f91b, SUC_lexicon_tags_60 },
    { 0xc7b6f91b, SUC_lexicon_tags_6 },
    { 0x5bd6f91c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b6f920, SUC_lexicon_tags_11 },
    { 0x2996f921, SUC_lexicon_tags_37 },
    { 0x6756f922, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a56f929, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9b6f92b, SUC_lexicon_tags_33 },
    { 0xa8f6f92c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f6f92f, SUC_lexicon_tags_16 },
    { 0xcf76f92f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf6f93d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f6f940, SUC_lexicon_tags_32 },
    { 0x46b6f941, SUC_lexicon_tags_12 },
    { 0x1ed6f942, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb476f944, SUC_lexicon_tags_38 },
    { 0xb956f944, SUC_lexicon_tags_15 },
    { 0xfd16f946, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe596f94f, SUC_lexicon_tags_235 },
    { 0x7416f950, SUC_lexicon_tags_17 },
    { 0x4676f950, SUC_lexicon_tags_1 },
    { 0x54f6f950, SUC_lexicon_tags_1 },
    { 0x97d6f950, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd36f956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d36f958, SUC_lexicon_tags_43 },
    { 0xd0b6f959, SUC_lexicon_tags_33 },
    { 0x7e96f95a, SUC_lexicon_tags_66 },
    { 0x3876f95a, SUC_lexicon_tags_53 },
    { 0xfb36f95a, SUC_lexicon_tags_30 },
    { 0x3c76f958, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbb96f95f, SUC_lexicon_tags_47 },
    { 0x11f6f960, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db6f967, SUC_lexicon_tags_43 },
    { 0x5436f968, SUC_lexicon_tags_21 },
    { 0x3cd6f969, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd356f96b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8876f96f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x836f972, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b96f974, SUC_lexicon_tags_14 },
    { 0x49d6f974, SUC_lexicon_tags_60 },
    { 0x7156f976, SUC_lexicon_tags_25 },
    { 0x8b96f977, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68b6f979, SUC_lexicon_tags_15 },
    { 0x8bd6f979, SUC_lexicon_tags_27 },
    { 0xbc56f97b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b6f97e, SUC_lexicon_tags_7 },
    { 0x1b16f97f, SUC_lexicon_tags_47 },
    { 0x7776f980, SUC_lexicon_tags_12 },
    { 0xfd76f981, SUC_lexicon_tags_12 },
    { 0xfdb6f982, SUC_lexicon_tags_25 },
    { 0x3c36f980, SUC_lexicon_tags_1 },
    { 0xccf6f984, SUC_lexicon_tags_17 },
    { 0x5376f985, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f6f988, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8736f98c, SUC_lexicon_tags_6 },
    { 0xc076f98c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa636f98f, SUC_lexicon_tags_7 },
    { 0x9db6f990, SUC_lexicon_tags_7 },
    { 0x5256f991, SUC_lexicon_tags_47 },
    { 0x66d6f990, SUC_lexicon_tags_27 },
    { 0x65d6f993, SUC_lexicon_tags_15 },
    { 0x8a76f994, SUC_lexicon_tags_87 },
    { 0x55b6f993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f36f997, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe36f99d, SUC_lexicon_tags_299 },
    { 0x83d6f99e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d6f9a3, SUC_lexicon_tags_1 },
    { 0xe736f9a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396f9a9, SUC_lexicon_tags_47 },
    { 0x98f6f9aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b6f9ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d76f9b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddd6f9b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb96f9b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3176f9b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc16f9bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f96f9bf, SUC_lexicon_tags_12 },
    { 0xdcb6f9c0, SUC_lexicon_tags_25 },
    { 0xefd6f9c0, SUC_lexicon_tags_60 },
    { 0x6c96f9bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6936f9c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf6f9c7, SUC_lexicon_tags_47 },
    { 0x3696f9c8, SUC_lexicon_tags_29 },
    { 0xac56f9c8, SUC_lexicon_tags_16 },
    { 0x2a96f9ca, SUC_lexicon_tags_33 },
    { 0x4b56f9c9, SUC_lexicon_tags_0 },
    { 0x3636f9c8, SUC_lexicon_tags_12 },
    { 0x8eb6f9cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27b6f9cf, SUC_lexicon_tags_37 },
    { 0xb2b6f9d0, SUC_lexicon_tags_56 },
    { 0x33f6f9d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa676f9d6, SUC_lexicon_tags_25 },
    { 0xb376f9d7, SUC_lexicon_tags_11 },
    { 0xee56f9d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6536f9da, SUC_lexicon_tags_4 },
    { 0x6196f9db, SUC_lexicon_tags_17 },
    { 0x32d6f9db, SUC_lexicon_tags_37 },
    { 0x3976f9dd, SUC_lexicon_tags_25 },
    { 0x8b56f9de, SUC_lexicon_tags_40 },
    { 0x20d6f9dd, SUC_lexicon_tags_2 },
    { 0x42f6f9e0, SUC_lexicon_tags_13 },
    { 0xe56f9e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9756f9e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf96f9e7, SUC_lexicon_tags_1 },
    { 0x9636f9e7, SUC_lexicon_tags_37 },
    { 0x24b6f9e8, SUC_lexicon_tags_34 },
    { 0x9536f9ea, SUC_lexicon_tags_25 },
    { 0x4216f9e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f6f9ee, SUC_lexicon_tags_2 },
    { 0x1f96f9ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8676f9f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4896f9f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf816f9f8, SUC_lexicon_tags_7 },
    { 0xa396f9f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf6f9fe, SUC_lexicon_tags_21 },
    { 0xcbf6f9ff, SUC_lexicon_tags_33 },
    { 0xb7d6f9fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d36fa02, SUC_lexicon_tags_45 },
    { 0x20d6fa03, SUC_lexicon_tags_2 },
    { 0xa2f6fa02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d6fa08, SUC_lexicon_tags_28 },
    { 0x1f16fa09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb56fa0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d6fa11, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d6fa15, SUC_lexicon_tags_45 },
    { 0xc0f6fa16, SUC_lexicon_tags_11 },
    { 0x4ed6fa15, SUC_lexicon_tags_25 },
    { 0xbe76fa18, SUC_lexicon_tags_55 },
    { 0x52f6fa18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf36fa1f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf6fa22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb236fa24, SUC_lexicon_tags_3 },
    { 0xcf6fa24, SUC_lexicon_tags_29 },
    { 0xfb76fa25, SUC_lexicon_tags_12 },
    { 0xc776fa24, SUC_lexicon_tags_15 },
    { 0xbaf6fa28, SUC_lexicon_tags_45 },
    { 0xf756fa29, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8dd6fa2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50d6fa2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5456fa32, SUC_lexicon_tags_29 },
    { 0x596fa33, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x7b76fa35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7416fa38, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4856fa43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff16fa47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d6fa4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a36fa4e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d6fa52, SUC_lexicon_tags_37 },
    { 0x3af6fa52, SUC_lexicon_tags_4 },
    { 0x4216fa53, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa096fa5a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4876fa5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10d6fa5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f6fa69, SUC_lexicon_tags_1 },
    { 0x17b6fa6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc56fa6d, SUC_lexicon_tags_25 },
    { 0x9a76fa6e, SUC_lexicon_tags_17 },
    { 0xa036fa6f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9756fa71, SUC_lexicon_tags_37 },
    { 0x4b36fa71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4f6fa74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf56fa77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f6fa7c, SUC_lexicon_tags_15 },
    { 0xe956fa7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef6fa82, SUC_lexicon_tags_23 },
    { 0xd136fa83, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xde96fa85, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xde76fa87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa156fa8a, SUC_lexicon_tags_63 },
    { 0x4776fa8a, SUC_lexicon_tags_15 },
    { 0x9bf6fa8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb16fa92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c16fa97, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7e36fa99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f6fa9c, SUC_lexicon_tags_15 },
    { 0xf1f6fa9c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c76faa1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3576faa4, SUC_lexicon_tags_15 },
    { 0xecb6faa4, SUC_lexicon_tags_38 },
    { 0x8ef6faa5, SUC_lexicon_tags_0 },
    { 0xc196faa7, SUC_lexicon_tags_33 },
    { 0xb736faa5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e56faaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d36faaf, SUC_lexicon_tags_25 },
    { 0xdf36faaf, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xa2f6fab2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf436fab4, SUC_lexicon_tags_70 },
    { 0x1436fab5, SUC_lexicon_tags_17 },
    { 0xd5b6fab5, SUC_lexicon_tags_22 },
    { 0xfad6fab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2416fabd, SUC_lexicon_tags_1 },
    { 0x56f6fabe, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9056fac7, SUC_lexicon_tags_23 },
    { 0xb456fac8, SUC_lexicon_tags_83 },
    { 0x576fac9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6696face, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb636fad3, SUC_lexicon_tags_27 },
    { 0x9a36fad4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e16fada, SUC_lexicon_tags_12 },
    { 0xbbf6fada, SUC_lexicon_tags_1 },
    { 0x7216fadc, SUC_lexicon_tags_212 },
    { 0x1196fadc, SUC_lexicon_tags_45 },
    { 0xeef6fadd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb6fae3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b6faee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8656faf0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd4d6faf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd36faf4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x136faf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8156faf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd736fafd, SUC_lexicon_tags_31 },
    { 0xb656fafe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4136fb01, SUC_lexicon_tags_37 },
    { 0x976fb01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1896fb07, SUC_lexicon_tags_38 },
    { 0x41b6fb08, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd736fb0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f6fb11, SUC_lexicon_tags_14 },
    { 0x4f76fb11, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf6fb16, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fb19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd6fb1e, SUC_lexicon_tags_27 },
    { 0xaf76fb1f, SUC_lexicon_tags_11 },
    { 0x78f6fb20, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x5076fb22, SUC_lexicon_tags_45 },
    { 0x9f76fb22, SUC_lexicon_tags_60 },
    { 0x77f6fb22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3b6fb26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb6fb2b, SUC_lexicon_tags_17 },
    { 0x6936fb2c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d6fb32, SUC_lexicon_tags_15 },
    { 0xbfd6fb33, SUC_lexicon_tags_2 },
    { 0xc4f6fb33, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f6fb3d, SUC_lexicon_tags_25 },
    { 0x7036fb3e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8656fb42, SUC_lexicon_tags_37 },
    { 0xe416fb43, SUC_lexicon_tags_25 },
    { 0xb536fb44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd6fb49, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6a36fb4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea56fb4d, SUC_lexicon_tags_11 },
    { 0xecb6fb4e, SUC_lexicon_tags_7 },
    { 0xa556fb4d, SUC_lexicon_tags_29 },
    { 0xc376fb4f, SUC_lexicon_tags_77 },
    { 0x69b6fb51, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d6fb56, SUC_lexicon_tags_25 },
    { 0x84f6fb56, SUC_lexicon_tags_39 },
    { 0x5eb6fb56, SUC_lexicon_tags_87 },
    { 0x29f6fb57, SUC_lexicon_tags_25 },
    { 0xe1d6fb57, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f6fb60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd956fb63, SUC_lexicon_tags_17 },
    { 0x8996fb64, SUC_lexicon_tags_29 },
    { 0xe2f6fb65, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f36fb68, SUC_lexicon_tags_6 },
    { 0x6436fb69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1216fb6c, SUC_lexicon_tags_49 },
    { 0x89b6fb6c, SUC_lexicon_tags_23 },
    { 0x2396fb6e, SUC_lexicon_tags_6 },
    { 0x65b6fb6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35f6fb71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63d6fb73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6fb78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x18b6fb7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4716fb80, SUC_lexicon_tags_12 },
    { 0x136fb81, SUC_lexicon_tags_1 },
    { 0x1ad6fb82, SUC_lexicon_tags_33 },
    { 0xd7f6fb81, SUC_lexicon_tags_17 },
    { 0xa4f6fb83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb6fb87, SUC_lexicon_tags_367 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db6fb8b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1896fb8e, SUC_lexicon_tags_30 },
    { 0x3a76fb8f, SUC_lexicon_tags_19 },
    { 0xedb6fb90, SUC_lexicon_tags_38 },
    { 0x9b6fb91, SUC_lexicon_tags_49 },
    { 0x696fb8f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f6fb99, SUC_lexicon_tags_45 },
    { 0x74b6fb99, SUC_lexicon_tags_34 },
    { 0x1636fb99, SUC_lexicon_tags_60 },
    { 0x55d6fb9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5956fba1, SUC_lexicon_tags_33 },
    { 0x3e16fba1, SUC_lexicon_tags_26 },
    { 0xc996fba3, SUC_lexicon_tags_17 },
    { 0x3b56fba4, SUC_lexicon_tags_37 },
    { 0x9e36fba5, SUC_lexicon_tags_15 },
    { 0x60f6fba2, SUC_lexicon_tags_1 },
    { 0xc336fba1, SUC_lexicon_tags_1 },
    { 0x2396fba4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d6fbab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb056fbad, SUC_lexicon_tags_56 },
    { 0x30b6fbad, SUC_lexicon_tags_67 },
    { 0xf716fbaf, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x5736fbb1, SUC_lexicon_tags_1 },
    { 0x78b6fbb1, SUC_lexicon_tags_26 },
    { 0x916fbb3, SUC_lexicon_tags_30 },
    { 0x5f96fbb3, SUC_lexicon_tags_66 },
    { 0xbf6fbb5, SUC_lexicon_tags_7 },
    { 0x9b96fbb6, SUC_lexicon_tags_37 },
    { 0xb996fbb2, SUC_lexicon_tags_45 },
    { 0x50d6fbb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd936fbba, SUC_lexicon_tags_12 },
    { 0xe6d6fbbb, SUC_lexicon_tags_15 },
    { 0xd376fbbc, SUC_lexicon_tags_47 },
    { 0x2f16fbbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1016fbbf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd316fbc5, SUC_lexicon_tags_2 },
    { 0x9eb6fbc6, SUC_lexicon_tags_50 },
    { 0x82f6fbc5, SUC_lexicon_tags_31 },
    { 0x1e36fbc6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6556fbcb, SUC_lexicon_tags_12 },
    { 0x6df6fbcc, SUC_lexicon_tags_24 },
    { 0x7596fbcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6656fbd1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b6fbd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf6fbd8, SUC_lexicon_tags_0 },
    { 0xf9d6fbd9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf6fbde, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf6fbe5, SUC_lexicon_tags_12 },
    { 0xedd6fbe6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d6fbea, SUC_lexicon_tags_2 },
    { 0xf696fbeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b6fbef, SUC_lexicon_tags_6 },
    { 0x2e56fbef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa616fbf2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x67b6fbf4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9ab6fbf6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f6fbf9, SUC_lexicon_tags_77 },
    { 0x5f6fbf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b6fbfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e96fc00, SUC_lexicon_tags_7 },
    { 0xbd56fc01, SUC_lexicon_tags_3 },
    { 0x85d6fc00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad6fc05, SUC_lexicon_tags_38 },
    { 0x68d6fc06, SUC_lexicon_tags_31 },
    { 0xef36fc07, SUC_lexicon_tags_6 },
    { 0x91f6fc06, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f6fc0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4896fc0f, SUC_lexicon_tags_87 },
    { 0x44d6fc0f, SUC_lexicon_tags_28 },
    { 0xa916fc11, SUC_lexicon_tags_37 },
    { 0x4cb6fc12, SUC_lexicon_tags_74 },
    { 0xa176fc11, SUC_lexicon_tags_6 },
    { 0x236fc13, SUC_lexicon_tags_23 },
    { 0x4156fc15, SUC_lexicon_tags_7 },
    { 0x2516fc16, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x87f6fc18, SUC_lexicon_tags_11 },
    { 0x7976fc18, SUC_lexicon_tags_0 },
    { 0xa836fc1a, SUC_lexicon_tags_33 },
    { 0xf36fc1b, SUC_lexicon_tags_2 },
    { 0xecf6fc1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1916fc21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5af6fc23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2716fc25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d36fc27, SUC_lexicon_tags_25 },
    { 0x5f76fc27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x33f6fc2a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f16fc30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f6fc33, SUC_lexicon_tags_26 },
    { 0x3216fc34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62d6fc36, SUC_lexicon_tags_31 },
    { 0xdb16fc36, SUC_lexicon_tags_11 },
    { 0x9116fc37, SUC_lexicon_tags_43 },
    { 0x92b6fc36, SUC_lexicon_tags_41 },
    { 0x20d6fc39, SUC_lexicon_tags_9 },
    { 0xe396fc3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d6fc45, SUC_lexicon_tags_2 },
    { 0x5096fc46, SUC_lexicon_tags_60 },
    { 0xdb16fc47, SUC_lexicon_tags_60 },
    { 0xb116fc45, SUC_lexicon_tags_87 },
    { 0x4e76fc49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3556fc4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab6fc4f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b6fc53, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7b56fc55, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e16fc5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd6fc68, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6fc6c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9536fc6f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb036fc7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed6fc81, SUC_lexicon_tags_32 },
    { 0x8cf6fc81, SUC_lexicon_tags_1 },
    { 0x7236fc83, SUC_lexicon_tags_37 },
    { 0x68b6fc84, SUC_lexicon_tags_39 },
    { 0x10b6fc82, SUC_lexicon_tags_49 },
    { 0x2696fc86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c56fc89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb096fc8b, SUC_lexicon_tags_19 },
    { 0x6b36fc8c, SUC_lexicon_tags_17 },
    { 0x2c76fc8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7596fc8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd96fc92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b6fc95, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xec36fc97, SUC_lexicon_tags_12 },
    { 0x4b16fc98, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbdd6fc9a, SUC_lexicon_tags_45 },
    { 0x3016fc9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1d6fc9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf276fca1, SUC_lexicon_tags_15 },
    { 0xbcb6fca1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x976fca4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb876fca9, SUC_lexicon_tags_33 },
    { 0xa016fcaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd6fcae, SUC_lexicon_tags_16 },
    { 0xe9f6fcae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec36fcb4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f6fcb7, SUC_lexicon_tags_2 },
    { 0x8276fcb8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc76fcbb, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x1156fcbe, SUC_lexicon_tags_14 },
    { 0x8cb6fcbf, SUC_lexicon_tags_2 },
    { 0x9bd6fcc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d36fcc3, SUC_lexicon_tags_17 },
    { 0x2ed6fcc4, SUC_lexicon_tags_37 },
    { 0xdd36fcc4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f6fcc8, SUC_lexicon_tags_12 },
    { 0xf2d6fcc8, SUC_lexicon_tags_7 },
    { 0x74f6fcc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d36fcd1, SUC_lexicon_tags_12 },
    { 0xb976fcd2, SUC_lexicon_tags_38 },
    { 0x7696fcd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefd6fcd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc36fcda, SUC_lexicon_tags_55 },
    { 0xa796fcda, SUC_lexicon_tags_37 },
    { 0x9f56fcdc, SUC_lexicon_tags_32 },
    { 0xb276fcdd, SUC_lexicon_tags_23 },
    { 0x91b6fcdd, SUC_lexicon_tags_1 },
    { 0x8116fcdf, SUC_lexicon_tags_7 },
    { 0xd0b6fcde, SUC_lexicon_tags_11 },
    { 0xf676fce0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5856fce3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb556fce6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad76fce8, SUC_lexicon_tags_14 },
    { 0xd276fce9, SUC_lexicon_tags_11 },
    { 0x7716fce9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd776fcec, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5076fcef, SUC_lexicon_tags_6 },
    { 0x1eb6fcf0, SUC_lexicon_tags_16 },
    { 0x95f6fcef, SUC_lexicon_tags_60 },
    { 0x8fd6fcf2, SUC_lexicon_tags_49 },
    { 0xea16fcef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5916fcf5, SUC_lexicon_tags_12 },
    { 0x1a76fcf5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d6fcfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79f6fcfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce76fd04, SUC_lexicon_tags_15 },
    { 0x6076fd05, SUC_lexicon_tags_0 },
    { 0x2496fd06, SUC_lexicon_tags_60 },
    { 0x7a36fd07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5136fd0b, SUC_lexicon_tags_47 },
    { 0xd616fd0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b6fd11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe476fd16, SUC_lexicon_tags_28 },
    { 0x956fd17, SUC_lexicon_tags_45 },
    { 0x10d6fd17, SUC_lexicon_tags_11 },
    { 0x41b6fd18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b6fd1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfcb6fd1e, SUC_lexicon_tags_26 },
    { 0xb076fd1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf836fd21, SUC_lexicon_tags_3 },
    { 0xe9d6fd22, SUC_lexicon_tags_15 },
    { 0x8dd6fd23, SUC_lexicon_tags_60 },
    { 0xc636fd24, SUC_lexicon_tags_11 },
    { 0x7fd6fd23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f76fd27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b36fd29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e56fd2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4716fd33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f96fd35, SUC_lexicon_tags_17 },
    { 0xacb6fd36, SUC_lexicon_tags_2 },
    { 0x1536fd35, SUC_lexicon_tags_14 },
    { 0xf836fd35, SUC_lexicon_tags_9 },
    { 0x2e76fd39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4436fd3b, SUC_lexicon_tags_146 },
    { 0x4516fd3b, SUC_lexicon_tags_11 },
    { 0x8b56fd3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe696fd41, SUC_lexicon_tags_25 },
    { 0x7976fd42, SUC_lexicon_tags_231 },
    { 0x7a96fd42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe636fd45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3176fd47, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd76fd4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b76fd5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd156fd5e, SUC_lexicon_tags_37 },
    { 0xb916fd5f, SUC_lexicon_tags_2 },
    { 0x4476fd5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7876fd62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d6fd66, SUC_lexicon_tags_19 },
    { 0x9d36fd67, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8416fd69, SUC_lexicon_tags_37 },
    { 0xd876fd69, SUC_lexicon_tags_43 },
    { 0x52f6fd6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3696fd70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac16fd74, SUC_lexicon_tags_38 },
    { 0xfdd6fd75, SUC_lexicon_tags_7 },
    { 0xb696fd76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa336fd7e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xde56fd81, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6d76fd83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb96fd85, SUC_lexicon_tags_254 },
    { 0x3d76fd86, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3696fd88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7db6fd8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c76fd8d, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe976fd97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef6fd9a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf156fd9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x256fda5, SUC_lexicon_tags_4 },
    { 0x6796fda5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8f6fda8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1076fdaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc736fdaf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb216fdb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d6fdba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d96fdbf, SUC_lexicon_tags_37 },
    { 0x7956fdbf, SUC_lexicon_tags_15 },
    { 0xc0b6fdc1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x3b16fdc3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6036fdcb, SUC_lexicon_tags_32 },
    { 0x27b6fdcb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x196fdce, SUC_lexicon_tags_33 },
    { 0xc2f6fdcf, SUC_lexicon_tags_17 },
    { 0x9d16fdd0, SUC_lexicon_tags_30 },
    { 0xac36fdcf, SUC_lexicon_tags_27 },
    { 0xeb56fdd2, SUC_lexicon_tags_26 },
    { 0xecd6fdd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5516fdda, SUC_lexicon_tags_15 },
    { 0x17f6fddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f6fde5, SUC_lexicon_tags_37 },
    { 0x4a96fde6, SUC_lexicon_tags_7 },
    { 0x4a16fde7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30f6fde9, SUC_lexicon_tags_11 },
    { 0xda16fde9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d6fded, SUC_lexicon_tags_47 },
    { 0x7c96fdee, SUC_lexicon_tags_43 },
    { 0x8c56fdef, SUC_lexicon_tags_9 },
    { 0xfad6fdf0, SUC_lexicon_tags_33 },
    { 0x2fd6fdf0, SUC_lexicon_tags_8 },
    { 0x6536fdf0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8056fdf7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b6fdfc, SUC_lexicon_tags_49 },
    { 0xfbd6fdfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f16fe02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d6fe05, SUC_lexicon_tags_37 },
    { 0x1156fe06, SUC_lexicon_tags_7 },
    { 0x4696fe07, SUC_lexicon_tags_28 },
    { 0x6e56fe06, SUC_lexicon_tags_12 },
    { 0xb176fe07, SUC_lexicon_tags_275 },
    { 0xdf36fe09, SUC_lexicon_tags_33 },
    { 0xc276fe0a, SUC_lexicon_tags_23 },
    { 0x9bf6fe05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea56fe11, SUC_lexicon_tags_2 },
    { 0xf696fe12, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5376fe16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb116fe19, SUC_lexicon_tags_37 },
    { 0x5c76fe1a, SUC_lexicon_tags_27 },
    { 0x9f56fe1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb856fe1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d6fe21, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f6fe27, SUC_lexicon_tags_46 },
    { 0xfdb6fe28, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe036fe2a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6576fe2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a76fe31, SUC_lexicon_tags_25 },
    { 0x8096fe32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1236fe36, SUC_lexicon_tags_65 },
    { 0x3036fe37, SUC_lexicon_tags_17 },
    { 0x4d6fe37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf6fe3b, SUC_lexicon_tags_1 },
    { 0x4016fe3c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8eb6fe3e, SUC_lexicon_tags_2 },
    { 0x756fe3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6836fe41, SUC_lexicon_tags_38 },
    { 0x9f16fe42, SUC_lexicon_tags_2 },
    { 0xb916fe42, SUC_lexicon_tags_65 },
    { 0xd716fe44, SUC_lexicon_tags_15 },
    { 0xb5b6fe42, SUC_lexicon_tags_14 },
    { 0x46f6fe46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x26f6fe48, SUC_lexicon_tags_15 },
    { 0x7756fe49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa056fe4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf196fe4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf476fe50, SUC_lexicon_tags_6 },
    { 0x78d6fe51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb16fe54, SUC_lexicon_tags_17 },
    { 0x3c96fe55, SUC_lexicon_tags_11 },
    { 0x4936fe55, SUC_lexicon_tags_39 },
    { 0x1e16fe55, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e16fe5a, SUC_lexicon_tags_11 },
    { 0xf7f6fe5a, SUC_lexicon_tags_48 },
    { 0x64d6fe5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1796fe5e, SUC_lexicon_tags_25 },
    { 0x9e76fe5e, SUC_lexicon_tags_11 },
    { 0xc496fe5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1076fe63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef6fe69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf36fe6c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9136fe72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec76fe74, SUC_lexicon_tags_6 },
    { 0x9856fe75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6d6fe77, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3256fe7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe36fe7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5676fe82, SUC_lexicon_tags_17 },
    { 0xd456fe82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c56fe85, SUC_lexicon_tags_2 },
    { 0xa9f6fe86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3516fe89, SUC_lexicon_tags_15 },
    { 0x4b76fe8a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf596fe8c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb6fe8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7836fe91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1f6fe93, SUC_lexicon_tags_47 },
    { 0xfd16fe93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8276fe97, SUC_lexicon_tags_13 },
    { 0x3a56fe97, SUC_lexicon_tags_17 },
    { 0x896fe97, SUC_lexicon_tags_15 },
    { 0x7cb6fe99, SUC_lexicon_tags_31 },
    { 0x22d6fe98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd36fe9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b6fea2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3e36fea4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5456fea6, SUC_lexicon_tags_29 },
    { 0x4436fea6, SUC_lexicon_tags_16 },
    { 0x2a76fea7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad6feac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7276feb0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f6feb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3596feb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a56feba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d16fec0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbf16fec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd16fec8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1616fed3, SUC_lexicon_tags_29 },
    { 0x85d6fed4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8eb6fed6, SUC_lexicon_tags_11 },
    { 0x2336fed7, SUC_lexicon_tags_0 },
    { 0xd036fed6, SUC_lexicon_tags_63 },
    { 0x3fd6fed8, SUC_lexicon_tags_26 },
    { 0x9d16fed6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9816fedc, SUC_lexicon_tags_14 },
    { 0x7096fedc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f6fee1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a56fee3, SUC_lexicon_tags_45 },
    { 0xc476fee4, SUC_lexicon_tags_12 },
    { 0xdbb6fee5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd456fee7, SUC_lexicon_tags_17 },
    { 0x3e16fee8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e56feea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6516feee, SUC_lexicon_tags_49 },
    { 0xb376feef, SUC_lexicon_tags_34 },
    { 0xdb16fef0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4f6fef2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d6fef8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae16fefa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba16fefd, SUC_lexicon_tags_12 },
    { 0xb916fefe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f6ff01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa756ff05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1536ff0d, SUC_lexicon_tags_44 },
    { 0xe696ff0e, SUC_lexicon_tags_12 },
    { 0xed76ff0d, SUC_lexicon_tags_68 },
    { 0x6976ff0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f6ff16, SUC_lexicon_tags_25 },
    { 0x74b6ff17, SUC_lexicon_tags_48 },
    { 0xa696ff17, SUC_lexicon_tags_27 },
    { 0x3936ff16, SUC_lexicon_tags_26 },
    { 0x3696ff1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b6ff1f, SUC_lexicon_tags_12 },
    { 0xf7f6ff1f, SUC_lexicon_tags_25 },
    { 0xdc56ff1f, SUC_lexicon_tags_2 },
    { 0x4056ff21, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd036ff2d, SUC_lexicon_tags_38 },
    { 0xf9d6ff2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc016ff34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4416ff36, SUC_lexicon_tags_69 },
    { 0xcf36ff37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb456ff3b, SUC_lexicon_tags_2 },
    { 0xab36ff3b, SUC_lexicon_tags_47 },
    { 0x9416ff3d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7716ff43, SUC_lexicon_tags_37 },
    { 0xc3b6ff44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7156ff46, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b56ff4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab6ff4e, SUC_lexicon_tags_37 },
    { 0x5596ff4f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c16ff52, SUC_lexicon_tags_8 },
    { 0x9f6ff53, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396ff58, SUC_lexicon_tags_29 },
    { 0xd116ff58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7276ff5d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf296ff5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8736ff63, SUC_lexicon_tags_24 },
    { 0xc216ff64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b6ff68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1af6ff6a, SUC_lexicon_tags_87 },
    { 0xd716ff6a, SUC_lexicon_tags_49 },
    { 0xa736ff6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd76ff6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2cd6ff71, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8196ff74, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6f16ff76, SUC_lexicon_tags_37 },
    { 0x1496ff77, SUC_lexicon_tags_11 },
    { 0x3356ff77, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xced6ff7a, SUC_lexicon_tags_31 },
    { 0x6776ff7b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee16ff7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a16ff82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d6ff85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf656ff90, SUC_lexicon_tags_12 },
    { 0x4816ff90, SUC_lexicon_tags_37 },
    { 0xf816ff92, SUC_lexicon_tags_6 },
    { 0xee76ff92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a56ff97, SUC_lexicon_tags_30 },
    { 0x12f6ff98, SUC_lexicon_tags_12 },
    { 0x8656ff97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa516ffa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df6ffa6, SUC_lexicon_tags_7 },
    { 0xe2f6ffa7, SUC_lexicon_tags_7 },
    { 0xc2d6ffa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf856ffae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4336ffb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad6ffb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe76ffbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f6ffc0, SUC_lexicon_tags_11 },
    { 0x8a16ffc1, SUC_lexicon_tags_3 },
    { 0xa716ffc2, SUC_lexicon_tags_0 },
    { 0x85f6ffc3, SUC_lexicon_tags_49 },
    { 0x6016ffc2, SUC_lexicon_tags_33 },
    { 0xe3b6ffc0, SUC_lexicon_tags_7 },
    { 0x52b6ffc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af6ffc9, SUC_lexicon_tags_49 },
    { 0x4276ffca, SUC_lexicon_tags_25 },
    { 0x3096ffc9, SUC_lexicon_tags_39 },
    { 0x5556ffc9, SUC_lexicon_tags_1 },
    { 0xdad6ffc9, SUC_lexicon_tags_29 },
    { 0x2996ffce, SUC_lexicon_tags_17 },
    { 0xcb36ffce, SUC_lexicon_tags_15 },
    { 0x9bf6ffd0, SUC_lexicon_tags_37 },
    { 0x3076ffcc, SUC_lexicon_tags_23 },
    { 0x6496ffd2, SUC_lexicon_tags_17 },
    { 0x4676ffd3, SUC_lexicon_tags_0 },
    { 0xc976ffd0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d6ffd9, SUC_lexicon_tags_87 },
    { 0xa396ffda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8176ffe2, SUC_lexicon_tags_31 },
    { 0x3956ffe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc36ffe8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb6ffec, SUC_lexicon_tags_15 },
    { 0x31f6ffed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1816fff2, SUC_lexicon_tags_11 },
    { 0x5c56fff3, SUC_lexicon_tags_37 },
    { 0x356fff4, SUC_lexicon_tags_1 },
    { 0xc56fff3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18b6fff7, SUC_lexicon_tags_12 },
    { 0xe7b6fff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fb6fffa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd70000, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd970005, SUC_lexicon_tags_23 },
    { 0xdcd70006, SUC_lexicon_tags_12 },
    { 0x28970006, SUC_lexicon_tags_15 },
    { 0x2d370008, SUC_lexicon_tags_27 },
    { 0xd3570008, SUC_lexicon_tags_16 },
    { 0xe0370008, SUC_lexicon_tags_25 },
    { 0xc8b70009, SUC_lexicon_tags_12 },
    { 0xe6d70008, SUC_lexicon_tags_31 },
    { 0x15370009, SUC_lexicon_tags_27 },
    { 0xa697000e, SUC_lexicon_tags_25 },
    { 0xdbb70007, SUC_lexicon_tags_2 },
    { 0x43b70010, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd570014, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35370018, SUC_lexicon_tags_3 },
    { 0x3af70019, SUC_lexicon_tags_28 },
    { 0x8197001a, SUC_lexicon_tags_47 },
    { 0x4957001a, SUC_lexicon_tags_82 },
    { 0x1937001a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73770020, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfc970022, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db70028, SUC_lexicon_tags_45 },
    { 0x1fb70028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87770035, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe1f70037, SUC_lexicon_tags_27 },
    { 0xa8770038, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f7003b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb70041, SUC_lexicon_tags_19 },
    { 0xdf170041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4970046, SUC_lexicon_tags_16 },
    { 0xd4770047, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42770049, SUC_lexicon_tags_37 },
    { 0x2317004a, SUC_lexicon_tags_47 },
    { 0x9d77004b, SUC_lexicon_tags_53 },
    { 0x2cf7004a, SUC_lexicon_tags_39 },
    { 0x2f37004b, SUC_lexicon_tags_60 },
    { 0x2697004c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca770053, SUC_lexicon_tags_38 },
    { 0xcad70053, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x51d70056, SUC_lexicon_tags_15 },
    { 0x8cb70057, SUC_lexicon_tags_26 },
    { 0x18b70058, SUC_lexicon_tags_25 },
    { 0x67170059, SUC_lexicon_tags_11 },
    { 0x8e97005a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98370067, SUC_lexicon_tags_92 },
    { 0x8f970068, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2117006a, SUC_lexicon_tags_14 },
    { 0xcef7006b, SUC_lexicon_tags_9 },
    { 0xf837006b, SUC_lexicon_tags_33 },
    { 0xdad7006c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2277006f, SUC_lexicon_tags_37 },
    { 0x31b70070, SUC_lexicon_tags_0 },
    { 0x89370071, SUC_lexicon_tags_1 },
    { 0xee770071, SUC_lexicon_tags_33 },
    { 0xdb70073, SUC_lexicon_tags_27 },
    { 0x82570071, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b7007c, SUC_lexicon_tags_6 },
    { 0xda77007d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd70087, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x53370089, SUC_lexicon_tags_2 },
    { 0xe0f7008a, SUC_lexicon_tags_47 },
    { 0x6157008a, SUC_lexicon_tags_27 },
    { 0xd0f7008b, SUC_lexicon_tags_2 },
    { 0xa0f7008d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c57008f, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62970093, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb170097, SUC_lexicon_tags_16 },
    { 0x87370098, SUC_lexicon_tags_17 },
    { 0xfb970099, SUC_lexicon_tags_11 },
    { 0xf0d7009a, SUC_lexicon_tags_12 },
    { 0xdb7009a, SUC_lexicon_tags_48 },
    { 0x2737009c, SUC_lexicon_tags_1 },
    { 0x9ab7009d, SUC_lexicon_tags_37 },
    { 0x9657009e, SUC_lexicon_tags_38 },
    { 0xb6f7009e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad700a6, SUC_lexicon_tags_79 },
    { 0x85d700a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f700ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1700af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x253700b2, SUC_lexicon_tags_10 },
    { 0xd99700b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95700c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d700cc, SUC_lexicon_tags_65 },
    { 0x93d700cd, SUC_lexicon_tags_37 },
    { 0x8cf700cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e7700d0, SUC_lexicon_tags_37 },
    { 0x6f1700d0, SUC_lexicon_tags_12 },
    { 0x951700d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73700d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99700d9, SUC_lexicon_tags_11 },
    { 0xdf1700da, SUC_lexicon_tags_77 },
    { 0x773700db, SUC_lexicon_tags_37 },
    { 0x3a1700d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d700e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed700e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16b700e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733700ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe69700ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd53700f1, SUC_lexicon_tags_21 },
    { 0xd95700f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7700f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x693700fa, SUC_lexicon_tags_25 },
    { 0x811700fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf6d700fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x719700ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18770108, SUC_lexicon_tags_87 },
    { 0xca370108, SUC_lexicon_tags_15 },
    { 0x24970108, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b17010c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa37010e, SUC_lexicon_tags_1 },
    { 0x88f7010e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d70112, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb3970114, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd70118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d7011b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd617011e, SUC_lexicon_tags_25 },
    { 0x97d7011f, SUC_lexicon_tags_23 },
    { 0xb9d70120, SUC_lexicon_tags_1699 },
    { 0x2f170121, SUC_lexicon_tags_33 },
    { 0x9d770122, SUC_lexicon_tags_10 },
    { 0x84170121, SUC_lexicon_tags_17 },
    { 0x66f70122, SUC_lexicon_tags_14 },
    { 0x7dd70123, SUC_lexicon_tags_17 },
    { 0x7ab70122, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3b970128, SUC_lexicon_tags_47 },
    { 0x6ad70129, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6970130, SUC_lexicon_tags_5 },
    { 0xf9170131, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xde570133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b17013c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b17013f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e770141, SUC_lexicon_tags_1 },
    { 0x64b70142, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x54970144, SUC_lexicon_tags_54 },
    { 0xa4b70145, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d70148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d7014c, SUC_lexicon_tags_48 },
    { 0x3a17014d, SUC_lexicon_tags_43 },
    { 0x8657014d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b70153, SUC_lexicon_tags_37 },
    { 0x8ed70154, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f7015c, SUC_lexicon_tags_75 },
    { 0xc237015d, SUC_lexicon_tags_39 },
    { 0x2bb7015e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30170160, SUC_lexicon_tags_15 },
    { 0x2df70161, SUC_lexicon_tags_25 },
    { 0x1b70161, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d70166, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ad70168, SUC_lexicon_tags_23 },
    { 0x41770169, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd7016c, SUC_lexicon_tags_19 },
    { 0x38b7016d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3117016f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43970175, SUC_lexicon_tags_19 },
    { 0x6c970176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab70178, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xa957017a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3197017c, SUC_lexicon_tags_12 },
    { 0xebb7017d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45370181, SUC_lexicon_tags_10 },
    { 0x47d70181, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x22570184, SUC_lexicon_tags_7 },
    { 0xd1170185, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba970187, SUC_lexicon_tags_37 },
    { 0x68970188, SUC_lexicon_tags_37 },
    { 0xb7370189, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe977018c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb7018f, SUC_lexicon_tags_2 },
    { 0x6a170190, SUC_lexicon_tags_49 },
    { 0xf6570190, SUC_lexicon_tags_15 },
    { 0xce570190, SUC_lexicon_tags_12 },
    { 0x17018f, SUC_lexicon_tags_1 },
    { 0x64970194, SUC_lexicon_tags_7 },
    { 0x8e170195, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7970197, SUC_lexicon_tags_47 },
    { 0xd0170197, SUC_lexicon_tags_15 },
    { 0x90b70199, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb7019e, SUC_lexicon_tags_43 },
    { 0x42f7019e, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0x501701a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9701a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x943701a9, SUC_lexicon_tags_32 },
    { 0x7ab701aa, SUC_lexicon_tags_2 },
    { 0xf8b701ab, SUC_lexicon_tags_39 },
    { 0x9bf701ac, SUC_lexicon_tags_25 },
    { 0xb43701ad, SUC_lexicon_tags_25 },
    { 0xe81701ae, SUC_lexicon_tags_7 },
    { 0xb1701ae, SUC_lexicon_tags_15 },
    { 0x867701af, SUC_lexicon_tags_85 },
    { 0xc65701ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x767701b3, SUC_lexicon_tags_7 },
    { 0xaad701b4, SUC_lexicon_tags_15 },
    { 0x741701b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfbf701b7, SUC_lexicon_tags_7 },
    { 0x87701b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d701be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05701c1, SUC_lexicon_tags_27 },
    { 0x313701c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc61701c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e5701c6, SUC_lexicon_tags_15 },
    { 0x5bb701c7, SUC_lexicon_tags_1 },
    { 0x85b701c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1701cb, SUC_lexicon_tags_12 },
    { 0xd15701cc, SUC_lexicon_tags_38 },
    { 0xb4d701cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b701d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9701d9, SUC_lexicon_tags_27 },
    { 0xba9701da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f701e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41701e5, SUC_lexicon_tags_47 },
    { 0xe67701e5, SUC_lexicon_tags_25 },
    { 0x989701e7, SUC_lexicon_tags_12 },
    { 0x3701e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d701eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a9701ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f701ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d3701f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3701f9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x379701fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf970202, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d70207, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x30f70209, SUC_lexicon_tags_27 },
    { 0x9617020a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d77020e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b70215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53970218, SUC_lexicon_tags_17 },
    { 0xebb70218, SUC_lexicon_tags_7 },
    { 0x1e57021a, SUC_lexicon_tags_0 },
    { 0x1bd7021b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb70222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf170225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a370229, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd677022e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2170230, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f70237, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b57023c, SUC_lexicon_tags_37 },
    { 0x64d7023d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f770242, SUC_lexicon_tags_1 },
    { 0x5970243, SUC_lexicon_tags_60 },
    { 0x49370243, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f70249, SUC_lexicon_tags_19 },
    { 0xf5b70249, SUC_lexicon_tags_37 },
    { 0xfb7024b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1570250, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b70256, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1417025c, SUC_lexicon_tags_65 },
    { 0x3577025d, SUC_lexicon_tags_106 },
    { 0x3e37025e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0570261, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f70264, SUC_lexicon_tags_47 },
    { 0x34370264, SUC_lexicon_tags_18 },
    { 0xb5970264, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f7026a, SUC_lexicon_tags_45 },
    { 0x19f7026b, SUC_lexicon_tags_11 },
    { 0x4af7026c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63b7026e, SUC_lexicon_tags_65 },
    { 0x7577026f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x83f70271, SUC_lexicon_tags_15 },
    { 0x23570272, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa637027c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d70280, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd8b70282, SUC_lexicon_tags_37 },
    { 0xa4b70283, SUC_lexicon_tags_2 },
    { 0xee370284, SUC_lexicon_tags_60 },
    { 0xa1970284, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d70288, SUC_lexicon_tags_4 },
    { 0xa8370289, SUC_lexicon_tags_15 },
    { 0x15d70288, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae57028c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c57028e, SUC_lexicon_tags_15 },
    { 0x7db7028f, SUC_lexicon_tags_15 },
    { 0x87f70290, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b70295, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b7029b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975702a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x487702a5, SUC_lexicon_tags_12 },
    { 0xd69702a6, SUC_lexicon_tags_11 },
    { 0xf91702a7, SUC_lexicon_tags_1 },
    { 0x4f3702a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x97b702aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x587702ad, SUC_lexicon_tags_35 },
    { 0xcbb702ad, SUC_lexicon_tags_14 },
    { 0x5e9702ad, SUC_lexicon_tags_25 },
    { 0xb3b702af, SUC_lexicon_tags_43 },
    { 0x871702b1, SUC_lexicon_tags_193 },
    { 0x845702b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5702b6, SUC_lexicon_tags_116 },
    { 0x65f702b6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5702bc, SUC_lexicon_tags_29 },
    { 0x63d702bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927702c1, SUC_lexicon_tags_0 },
    { 0xc23702c2, SUC_lexicon_tags_25 },
    { 0x615702c3, SUC_lexicon_tags_29 },
    { 0xc61702c3, SUC_lexicon_tags_38 },
    { 0x221702c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b702ca, SUC_lexicon_tags_16 },
    { 0x69f702cb, SUC_lexicon_tags_6 },
    { 0xb8b702ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f702cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x483702d1, SUC_lexicon_tags_1 },
    { 0x831702d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3702d6, SUC_lexicon_tags_1 },
    { 0xc3d702d7, SUC_lexicon_tags_12 },
    { 0xc7b702d8, SUC_lexicon_tags_24 },
    { 0xf47702d7, SUC_lexicon_tags_37 },
    { 0xd93702d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b702dd, SUC_lexicon_tags_31 },
    { 0x889702de, SUC_lexicon_tags_12 },
    { 0xcc1702df, SUC_lexicon_tags_12 },
    { 0xf47702e0, SUC_lexicon_tags_7 },
    { 0x2e9702e1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb3702e3, SUC_lexicon_tags_28 },
    { 0x249702e4, SUC_lexicon_tags_37 },
    { 0x1f7702e3, SUC_lexicon_tags_0 },
    { 0xcff702e6, SUC_lexicon_tags_17 },
    { 0xc17702e6, SUC_lexicon_tags_584 },
    { 0x415702e8, SUC_lexicon_tags_1 },
    { 0xd59702e9, SUC_lexicon_tags_63 },
    { 0xe85702e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f970301, SUC_lexicon_tags_47 },
    { 0x36370301, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f7030d, SUC_lexicon_tags_17 },
    { 0x4c57030d, SUC_lexicon_tags_37 },
    { 0xcb97030d, SUC_lexicon_tags_7 },
    { 0x65370310, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f70313, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x77370315, SUC_lexicon_tags_33 },
    { 0x3ff70316, SUC_lexicon_tags_7 },
    { 0xa8570316, SUC_lexicon_tags_37 },
    { 0xe5f70317, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f57031e, SUC_lexicon_tags_49 },
    { 0x7757031f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xee970322, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97770325, SUC_lexicon_tags_12 },
    { 0x6f370325, SUC_lexicon_tags_11 },
    { 0x94170327, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2d70329, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1297032b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd70331, SUC_lexicon_tags_17 },
    { 0x38b70331, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69970339, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc537033c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x4157033f, SUC_lexicon_tags_42 },
    { 0x72770340, SUC_lexicon_tags_19 },
    { 0x92770340, SUC_lexicon_tags_11 },
    { 0xfa570342, SUC_lexicon_tags_27 },
    { 0xb0970342, SUC_lexicon_tags_53 },
    { 0x86f70344, SUC_lexicon_tags_2 },
    { 0xd5970344, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6337034a, SUC_lexicon_tags_13 },
    { 0x577034b, SUC_lexicon_tags_44 },
    { 0x85d7034b, SUC_lexicon_tags_23 },
    { 0xe2f7034d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4770350, SUC_lexicon_tags_31 },
    { 0xa7570351, SUC_lexicon_tags_26 },
    { 0xac970352, SUC_lexicon_tags_60 },
    { 0xfe570351, SUC_lexicon_tags_63 },
    { 0x80b70354, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x2c970356, SUC_lexicon_tags_11 },
    { 0x24f70357, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f37035b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd497035f, SUC_lexicon_tags_7 },
    { 0xdeb7035f, SUC_lexicon_tags_22 },
    { 0x37d70360, SUC_lexicon_tags_19 },
    { 0xa8f70362, SUC_lexicon_tags_7 },
    { 0x2a370363, SUC_lexicon_tags_10 },
    { 0xbd570361, SUC_lexicon_tags_0 },
    { 0x6ab70360, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b7036c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5317036e, SUC_lexicon_tags_60 },
    { 0x3297036e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66b70371, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b170374, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xead70377, SUC_lexicon_tags_25 },
    { 0xfbd70378, SUC_lexicon_tags_12 },
    { 0xd4f70379, SUC_lexicon_tags_1 },
    { 0xb4f7037a, SUC_lexicon_tags_15 },
    { 0xd657037a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x76d7037d, SUC_lexicon_tags_39 },
    { 0x5ab7037d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2170383, SUC_lexicon_tags_11 },
    { 0xc7770383, SUC_lexicon_tags_1 },
    { 0xba570385, SUC_lexicon_tags_11 },
    { 0x3cf70385, SUC_lexicon_tags_17 },
    { 0xf4570385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b7038b, SUC_lexicon_tags_1 },
    { 0xa5f7038c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf077038f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1170392, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5970396, SUC_lexicon_tags_82 },
    { 0x2e770397, SUC_lexicon_tags_14 },
    { 0x41170398, SUC_lexicon_tags_87 },
    { 0x5e770399, SUC_lexicon_tags_11 },
    { 0x6f570398, SUC_lexicon_tags_28 },
    { 0xac37039b, SUC_lexicon_tags_47 },
    { 0x3bd7039c, SUC_lexicon_tags_60 },
    { 0x8e97039d, SUC_lexicon_tags_87 },
    { 0x9e170399, SUC_lexicon_tags_11 },
    { 0xbe77039f, SUC_lexicon_tags_21 },
    { 0x283703a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3703a4, SUC_lexicon_tags_1 },
    { 0x2e3703a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd4d703a7, SUC_lexicon_tags_65 },
    { 0xcf703a8, SUC_lexicon_tags_14 },
    { 0x7c3703a9, SUC_lexicon_tags_12 },
    { 0xf49703a9, SUC_lexicon_tags_14 },
    { 0x50b703ab, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1703b4, SUC_lexicon_tags_2 },
    { 0xa5b703b5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdbf703b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369703bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x643703be, SUC_lexicon_tags_17 },
    { 0xacf703bf, SUC_lexicon_tags_158 },
    { 0x955703be, SUC_lexicon_tags_7 },
    { 0x5bb703bf, SUC_lexicon_tags_12 },
    { 0x775703c0, SUC_lexicon_tags_9 },
    { 0x6f3703c3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x235703c5, SUC_lexicon_tags_107 },
    { 0x671703c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc47703c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3703cd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75703d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f703da, SUC_lexicon_tags_301 },
    { 0x465703da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37703e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55703e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6a3703e5, SUC_lexicon_tags_17 },
    { 0x777703e6, SUC_lexicon_tags_29 },
    { 0x3b1703e6, SUC_lexicon_tags_83 },
    { 0x89b703e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501703f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x485703f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1703fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x635703fe, SUC_lexicon_tags_17 },
    { 0xc19703ff, SUC_lexicon_tags_116 },
    { 0xdbf703ff, SUC_lexicon_tags_1 },
    { 0xe49703fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xab570403, SUC_lexicon_tags_31 },
    { 0xae370403, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x25170406, SUC_lexicon_tags_25 },
    { 0x43970407, SUC_lexicon_tags_17 },
    { 0x4c570407, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4197040b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f770411, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16770419, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7c37041b, SUC_lexicon_tags_17 },
    { 0xa857041c, SUC_lexicon_tags_37 },
    { 0x7977041d, SUC_lexicon_tags_68 },
    { 0xdef7041d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a970421, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2970424, SUC_lexicon_tags_1 },
    { 0xeed70425, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7042a, SUC_lexicon_tags_16 },
    { 0x3517042a, SUC_lexicon_tags_25 },
    { 0xbbd7042c, SUC_lexicon_tags_11 },
    { 0xe6d7042d, SUC_lexicon_tags_34 },
    { 0xf177042e, SUC_lexicon_tags_24 },
    { 0xfe97042c, SUC_lexicon_tags_37 },
    { 0x4257042c, SUC_lexicon_tags_49 },
    { 0xda370431, SUC_lexicon_tags_37 },
    { 0xe1f7042a, SUC_lexicon_tags_9 },
    { 0xb9170433, SUC_lexicon_tags_32 },
    { 0xab170434, SUC_lexicon_tags_0 },
    { 0x88d70430, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa377043f, SUC_lexicon_tags_11 },
    { 0x76970440, SUC_lexicon_tags_13 },
    { 0xda770440, SUC_lexicon_tags_37 },
    { 0xa2570442, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd70446, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb7044b, SUC_lexicon_tags_37 },
    { 0xca97044c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49370451, SUC_lexicon_tags_9 },
    { 0x70970451, SUC_lexicon_tags_17 },
    { 0xb7f70453, SUC_lexicon_tags_37 },
    { 0xaf70453, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2637045a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57045f, SUC_lexicon_tags_68 },
    { 0xdd970460, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab70464, SUC_lexicon_tags_9 },
    { 0xac170464, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f17046e, SUC_lexicon_tags_84 },
    { 0x4777046e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26b70471, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x82570473, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8970478, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f7047f, SUC_lexicon_tags_1 },
    { 0xb6b7047f, SUC_lexicon_tags_3 },
    { 0xe3d7047f, SUC_lexicon_tags_38 },
    { 0x7ab70482, SUC_lexicon_tags_47 },
    { 0xa8170483, SUC_lexicon_tags_37 },
    { 0x5d570484, SUC_lexicon_tags_7 },
    { 0xd7b70484, SUC_lexicon_tags_46 },
    { 0xa2b70480, SUC_lexicon_tags_14 },
    { 0x48970481, SUC_lexicon_tags_23 },
    { 0x2c970484, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa77048b, SUC_lexicon_tags_60 },
    { 0x4bb7048c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9770497, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f7049a, SUC_lexicon_tags_0 },
    { 0x6f77049b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d7049e, SUC_lexicon_tags_17 },
    { 0xa517049e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0f704a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423704a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x843704a8, SUC_lexicon_tags_19 },
    { 0x637704a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b704ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f704b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1704bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf704c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x765704c2, SUC_lexicon_tags_1 },
    { 0x897704c2, SUC_lexicon_tags_1 },
    { 0xf49704c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27f704c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb704cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5704cf, SUC_lexicon_tags_1 },
    { 0x2c1704d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x111704d3, SUC_lexicon_tags_25 },
    { 0x3e9704d4, SUC_lexicon_tags_6 },
    { 0xa7f704d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3704dc, SUC_lexicon_tags_24 },
    { 0x7e5704dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbcd704df, SUC_lexicon_tags_32 },
    { 0xde7704e0, SUC_lexicon_tags_11 },
    { 0x10b704e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33b704e3, SUC_lexicon_tags_65 },
    { 0xab3704e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb704e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617704ed, SUC_lexicon_tags_1 },
    { 0x679704ee, SUC_lexicon_tags_27 },
    { 0x74b704ef, SUC_lexicon_tags_46 },
    { 0x97b704f0, SUC_lexicon_tags_45 },
    { 0x429704ee, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x451704f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d704fe, SUC_lexicon_tags_16 },
    { 0x46d704ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8f70501, SUC_lexicon_tags_23 },
    { 0x5eb70502, SUC_lexicon_tags_14 },
    { 0x7ef70503, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb3170505, SUC_lexicon_tags_13 },
    { 0x10770506, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcd70508, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd697050b, SUC_lexicon_tags_30 },
    { 0xa077050b, SUC_lexicon_tags_12 },
    { 0xb1d7050d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xaf37050f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56f70511, SUC_lexicon_tags_47 },
    { 0xbf370511, SUC_lexicon_tags_49 },
    { 0xf4570511, SUC_lexicon_tags_62 },
    { 0xc8570511, SUC_lexicon_tags_2 },
    { 0xe3f70511, SUC_lexicon_tags_12 },
    { 0x32170514, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b70519, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa117051b, SUC_lexicon_tags_11 },
    { 0x4c17051c, SUC_lexicon_tags_15 },
    { 0xa597051c, SUC_lexicon_tags_9 },
    { 0x7b57051e, SUC_lexicon_tags_11 },
    { 0xd6f7051f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f70523, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557052a, SUC_lexicon_tags_43 },
    { 0x3997052b, SUC_lexicon_tags_17 },
    { 0xe7b7052b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6197052e, SUC_lexicon_tags_15 },
    { 0x5117052e, SUC_lexicon_tags_1 },
    { 0xfe770530, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd8d70532, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6eb70534, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe570539, SUC_lexicon_tags_26 },
    { 0x2a97053a, SUC_lexicon_tags_1 },
    { 0x36f7053a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1057053e, SUC_lexicon_tags_9 },
    { 0xa3b7053e, SUC_lexicon_tags_37 },
    { 0x6cd7053f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x19b70542, SUC_lexicon_tags_1 },
    { 0xc8f70543, SUC_lexicon_tags_2 },
    { 0x56570543, SUC_lexicon_tags_15 },
    { 0x2c370545, SUC_lexicon_tags_7 },
    { 0x1bd70546, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf970549, SUC_lexicon_tags_39 },
    { 0x92370549, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7370551, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b70556, SUC_lexicon_tags_47 },
    { 0x5cf70557, SUC_lexicon_tags_19 },
    { 0xcfb70558, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f70567, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf7056d, SUC_lexicon_tags_37 },
    { 0x297056d, SUC_lexicon_tags_28 },
    { 0x2437056d, SUC_lexicon_tags_11 },
    { 0xd2b7056f, SUC_lexicon_tags_0 },
    { 0x6f570571, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b170573, SUC_lexicon_tags_26 },
    { 0x42d70573, SUC_lexicon_tags_12 },
    { 0xb7370573, SUC_lexicon_tags_7 },
    { 0x5f70576, SUC_lexicon_tags_37 },
    { 0x2c170577, SUC_lexicon_tags_37 },
    { 0x5e770573, SUC_lexicon_tags_1 },
    { 0x21170577, SUC_lexicon_tags_30 },
    { 0x74370577, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3937057c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x85d7057e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xae970580, SUC_lexicon_tags_2 },
    { 0x7ef70581, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd7058a, SUC_lexicon_tags_30 },
    { 0x1717058a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd077058d, SUC_lexicon_tags_235 },
    { 0xf697058d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84b70590, SUC_lexicon_tags_32 },
    { 0xb5370591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46170593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96f70595, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c170598, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b97059b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b705a1, SUC_lexicon_tags_49 },
    { 0xcd3705a2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff705ab, SUC_lexicon_tags_1 },
    { 0x263705ac, SUC_lexicon_tags_11 },
    { 0x5e3705ad, SUC_lexicon_tags_11 },
    { 0x97b705ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x529705b1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1c7705b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735705b9, SUC_lexicon_tags_13 },
    { 0x7d9705b9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x853705bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65705be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9705c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9b705c4, SUC_lexicon_tags_37 },
    { 0xaa5705c5, SUC_lexicon_tags_37 },
    { 0x165705c6, SUC_lexicon_tags_25 },
    { 0x8bb705c7, SUC_lexicon_tags_1 },
    { 0x899705c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6af705ca, SUC_lexicon_tags_2 },
    { 0x30f705cb, SUC_lexicon_tags_23 },
    { 0xfc3705cc, SUC_lexicon_tags_38 },
    { 0xa45705cc, SUC_lexicon_tags_0 },
    { 0xdf705ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1b1705d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9705d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x411705d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361705e1, SUC_lexicon_tags_31 },
    { 0xdeb705e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67705e6, SUC_lexicon_tags_25 },
    { 0x871705e7, SUC_lexicon_tags_37 },
    { 0x447705e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9705ec, SUC_lexicon_tags_1706 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb705f1, SUC_lexicon_tags_25 },
    { 0x439705f2, SUC_lexicon_tags_33 },
    { 0xa0b705f3, SUC_lexicon_tags_44 },
    { 0xa05705f4, SUC_lexicon_tags_23 },
    { 0x981705f4, SUC_lexicon_tags_25 },
    { 0x2b1705f6, SUC_lexicon_tags_11 },
    { 0xc91705f2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907705fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x977705fd, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x4b3705ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x85970602, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x46170604, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d70608, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bf7060a, SUC_lexicon_tags_60 },
    { 0x4817060b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4337060e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab70612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1370614, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b970618, SUC_lexicon_tags_27 },
    { 0xc1d70618, SUC_lexicon_tags_37 },
    { 0x1457061a, SUC_lexicon_tags_30 },
    { 0x1cb7061a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b7061f, SUC_lexicon_tags_19 },
    { 0xdb770620, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec970624, SUC_lexicon_tags_39 },
    { 0x74570624, SUC_lexicon_tags_23 },
    { 0xb1170624, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdf770628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f97062c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc417062e, SUC_lexicon_tags_37 },
    { 0x4d7062f, SUC_lexicon_tags_17 },
    { 0xab97062f, SUC_lexicon_tags_17 },
    { 0x2037062e, SUC_lexicon_tags_63 },
    { 0x3e170632, SUC_lexicon_tags_9 },
    { 0x8f7062f, SUC_lexicon_tags_33 },
    { 0x2bf70634, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b770637, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9370639, SUC_lexicon_tags_9 },
    { 0x1c57063a, SUC_lexicon_tags_31 },
    { 0xbd37063b, SUC_lexicon_tags_4 },
    { 0x6517063b, SUC_lexicon_tags_25 },
    { 0xa4d7063b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f70642, SUC_lexicon_tags_13 },
    { 0xe8970642, SUC_lexicon_tags_7 },
    { 0x48d70644, SUC_lexicon_tags_25 },
    { 0x22170645, SUC_lexicon_tags_43 },
    { 0xef970646, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f37064c, SUC_lexicon_tags_12 },
    { 0x4957064d, SUC_lexicon_tags_1 },
    { 0x4bb7064e, SUC_lexicon_tags_37 },
    { 0xb877064f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5170652, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x58370654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77370659, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd17065d, SUC_lexicon_tags_12 },
    { 0xf0f7065e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47970663, SUC_lexicon_tags_82 },
    { 0x87f70664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df70667, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9770669, SUC_lexicon_tags_9 },
    { 0x3bf7066a, SUC_lexicon_tags_41 },
    { 0x79d7066b, SUC_lexicon_tags_13 },
    { 0xeb17066c, SUC_lexicon_tags_37 },
    { 0x2db7066b, SUC_lexicon_tags_37 },
    { 0x1277066b, SUC_lexicon_tags_11 },
    { 0xb137066f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24d70671, SUC_lexicon_tags_7 },
    { 0x5fb70672, SUC_lexicon_tags_12 },
    { 0xcb70673, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7137067f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f170681, SUC_lexicon_tags_3 },
    { 0x69b70682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3170687, SUC_lexicon_tags_1 },
    { 0x52370688, SUC_lexicon_tags_19 },
    { 0x84b70689, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d7068b, SUC_lexicon_tags_63 },
    { 0xda97068b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd877068e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x13570690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6cd70692, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c770694, SUC_lexicon_tags_27 },
    { 0x3e170695, SUC_lexicon_tags_6 },
    { 0xc2370695, SUC_lexicon_tags_23 },
    { 0x89d70695, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b706a6, SUC_lexicon_tags_63 },
    { 0x96b706a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x603706a9, SUC_lexicon_tags_17 },
    { 0x91b706a9, SUC_lexicon_tags_68 },
    { 0x82f706a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad706b0, SUC_lexicon_tags_43 },
    { 0xe23706b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x937706b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93706b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x767706b8, SUC_lexicon_tags_15 },
    { 0x805706b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f706bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x331706be, SUC_lexicon_tags_7 },
    { 0x883706bf, SUC_lexicon_tags_37 },
    { 0xe97706be, SUC_lexicon_tags_62 },
    { 0xdd706c0, SUC_lexicon_tags_31 },
    { 0x695706c2, SUC_lexicon_tags_7 },
    { 0xadf706c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d706c6, SUC_lexicon_tags_37 },
    { 0x939706c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b706ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30b706cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5706cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15706d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1706d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe85706da, SUC_lexicon_tags_138 },
    { 0x61d706db, SUC_lexicon_tags_0 },
    { 0xb89706db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fd706de, SUC_lexicon_tags_57 },
    { 0xc01706de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5706e3, SUC_lexicon_tags_37 },
    { 0xa15706e4, SUC_lexicon_tags_33 },
    { 0x31706e5, SUC_lexicon_tags_47 },
    { 0x369706e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x343706e8, SUC_lexicon_tags_32 },
    { 0xdf5706e8, SUC_lexicon_tags_39 },
    { 0x913706ea, SUC_lexicon_tags_45 },
    { 0xb6d706eb, SUC_lexicon_tags_26 },
    { 0xba7706ec, SUC_lexicon_tags_39 },
    { 0x49706ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9b706ef, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1706f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x755706f6, SUC_lexicon_tags_62 },
    { 0xfef706f6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x999706f9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9370704, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81370709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7417070e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0370710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d70716, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d37071a, SUC_lexicon_tags_17 },
    { 0x7c77071a, SUC_lexicon_tags_46 },
    { 0x28f7071a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff7071f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5370725, SUC_lexicon_tags_60 },
    { 0xab70725, SUC_lexicon_tags_33 },
    { 0x46770726, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a57072c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7537072e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f70735, SUC_lexicon_tags_57 },
    { 0xeb370735, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b7073a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf970741, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3e170743, SUC_lexicon_tags_7 },
    { 0x7d770743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e570748, SUC_lexicon_tags_8 },
    { 0x5d370749, SUC_lexicon_tags_25 },
    { 0x95d70749, SUC_lexicon_tags_7 },
    { 0x73b7074b, SUC_lexicon_tags_7 },
    { 0xcab7074b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a970753, SUC_lexicon_tags_65 },
    { 0x4b170754, SUC_lexicon_tags_40 },
    { 0x2e970754, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b7075a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0570762, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xbb370764, SUC_lexicon_tags_2 },
    { 0xbf70764, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x81170768, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb7076b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef7076e, SUC_lexicon_tags_25 },
    { 0xf3d7076e, SUC_lexicon_tags_23 },
    { 0x4d170770, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e170774, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e770777, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x70d70779, SUC_lexicon_tags_31 },
    { 0x5477077a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21d7077c, SUC_lexicon_tags_25 },
    { 0xe337077d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e77077f, SUC_lexicon_tags_2 },
    { 0x6bb70780, SUC_lexicon_tags_7 },
    { 0x36770781, SUC_lexicon_tags_61 },
    { 0x57970781, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80170788, SUC_lexicon_tags_63 },
    { 0xd9370789, SUC_lexicon_tags_37 },
    { 0x27b7078a, SUC_lexicon_tags_12 },
    { 0xf0b7078b, SUC_lexicon_tags_33 },
    { 0x7bf7078a, SUC_lexicon_tags_43 },
    { 0x7e770789, SUC_lexicon_tags_15 },
    { 0x3477078e, SUC_lexicon_tags_1 },
    { 0xef370788, SUC_lexicon_tags_1708 },
    { 0x1c170790, SUC_lexicon_tags_7 },
    { 0x76570791, SUC_lexicon_tags_37 },
    { 0x1c970790, SUC_lexicon_tags_105 },
    { 0x69570793, SUC_lexicon_tags_7 },
    { 0x8af70794, SUC_lexicon_tags_57 },
    { 0x5f70795, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f70798, SUC_lexicon_tags_30 },
    { 0xb7570798, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab7079c, SUC_lexicon_tags_37 },
    { 0x2657079c, SUC_lexicon_tags_133 },
    { 0xb737079e, SUC_lexicon_tags_43 },
    { 0x28b7079e, SUC_lexicon_tags_11 },
    { 0xc17707a0, SUC_lexicon_tags_11 },
    { 0xb657079f, SUC_lexicon_tags_9 },
    { 0x8857079c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x845707a4, SUC_lexicon_tags_29 },
    { 0x7f1707a5, SUC_lexicon_tags_15 },
    { 0x893707a5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xfdb707a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x585707ab, SUC_lexicon_tags_14 },
    { 0x2d7707ac, SUC_lexicon_tags_34 },
    { 0xf2d707ad, SUC_lexicon_tags_74 },
    { 0xb29707ae, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f707b1, SUC_lexicon_tags_8 },
    { 0x8bd707b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7707b9, SUC_lexicon_tags_17 },
    { 0xb9707b9, SUC_lexicon_tags_25 },
    { 0xcbd707bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b707be, SUC_lexicon_tags_37 },
    { 0xdd3707be, SUC_lexicon_tags_23 },
    { 0xde7707c0, SUC_lexicon_tags_7 },
    { 0x61707c1, SUC_lexicon_tags_28 },
    { 0xf27707c0, SUC_lexicon_tags_48 },
    { 0x289707c2, SUC_lexicon_tags_31 },
    { 0x9b7707c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a3707c6, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795707ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3cb707cc, SUC_lexicon_tags_46 },
    { 0xc8d707cd, SUC_lexicon_tags_17 },
    { 0xe37707ce, SUC_lexicon_tags_19 },
    { 0xdf707cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb87707d1, SUC_lexicon_tags_37 },
    { 0x9d9707d2, SUC_lexicon_tags_66 },
    { 0xa1d707d2, SUC_lexicon_tags_14 },
    { 0x68d707d3, SUC_lexicon_tags_30 },
    { 0x8c5707d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9707da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15b707dc, SUC_lexicon_tags_2 },
    { 0x53d707dd, SUC_lexicon_tags_7 },
    { 0xf77707de, SUC_lexicon_tags_45 },
    { 0x599707dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb707e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905707eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7707f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3707f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7707fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe7707fc, SUC_lexicon_tags_12 },
    { 0xa7b707fd, SUC_lexicon_tags_15 },
    { 0xfbd707fd, SUC_lexicon_tags_15 },
    { 0x91b707fc, SUC_lexicon_tags_18 },
    { 0x78370800, SUC_lexicon_tags_63 },
    { 0x313707ff, SUC_lexicon_tags_15 },
    { 0x23570802, SUC_lexicon_tags_19 },
    { 0x4b770803, SUC_lexicon_tags_23 },
    { 0x64b70802, SUC_lexicon_tags_17 },
    { 0xf2570805, SUC_lexicon_tags_17 },
    { 0x9ab70806, SUC_lexicon_tags_27 },
    { 0xf3b70806, SUC_lexicon_tags_15 },
    { 0x25b70808, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5b77080a, SUC_lexicon_tags_46 },
    { 0x3337080a, SUC_lexicon_tags_17 },
    { 0x5257080c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb897080e, SUC_lexicon_tags_15 },
    { 0x2557080e, SUC_lexicon_tags_60 },
    { 0x3fd7080e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76b70812, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90570815, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa1370817, SUC_lexicon_tags_31 },
    { 0xdcf70818, SUC_lexicon_tags_61 },
    { 0x2fb70818, SUC_lexicon_tags_26 },
    { 0x23170819, SUC_lexicon_tags_30 },
    { 0xa637081b, SUC_lexicon_tags_17 },
    { 0x3257081b, SUC_lexicon_tags_25 },
    { 0x24f7081d, SUC_lexicon_tags_15 },
    { 0x9417081e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x56d70820, SUC_lexicon_tags_27 },
    { 0xd7170821, SUC_lexicon_tags_2 },
    { 0x75570822, SUC_lexicon_tags_45 },
    { 0xf3170823, SUC_lexicon_tags_126 },
    { 0x96f70823, SUC_lexicon_tags_15 },
    { 0xe2d70824, SUC_lexicon_tags_11 },
    { 0x22970826, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5857082b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8257082d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x44170830, SUC_lexicon_tags_43 },
    { 0x54d70831, SUC_lexicon_tags_30 },
    { 0x81d70830, SUC_lexicon_tags_647 },
    { 0, NULL },
    { 0x15770834, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2570836, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb370838, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8057083b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a57083e, SUC_lexicon_tags_30 },
    { 0xe8f7083e, SUC_lexicon_tags_33 },
    { 0xc1770840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a970845, SUC_lexicon_tags_17 },
    { 0x17170846, SUC_lexicon_tags_25 },
    { 0xac770847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb297084e, SUC_lexicon_tags_4 },
    { 0xdcf7084e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6a370851, SUC_lexicon_tags_23 },
    { 0xe3b70852, SUC_lexicon_tags_2 },
    { 0xba770853, SUC_lexicon_tags_38 },
    { 0x5b570854, SUC_lexicon_tags_61 },
    { 0x3a370854, SUC_lexicon_tags_65 },
    { 0x1b370856, SUC_lexicon_tags_12 },
    { 0xe3f70854, SUC_lexicon_tags_17 },
    { 0x36370858, SUC_lexicon_tags_32 },
    { 0x79370858, SUC_lexicon_tags_6 },
    { 0xfa370851, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe57085c, SUC_lexicon_tags_7 },
    { 0x3977085d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17170861, SUC_lexicon_tags_12 },
    { 0xc570862, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35570864, SUC_lexicon_tags_18 },
    { 0x8ad70864, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e37086b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af7086f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5170873, SUC_lexicon_tags_13 },
    { 0x3fd70873, SUC_lexicon_tags_45 },
    { 0x1b170873, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb77087a, SUC_lexicon_tags_17 },
    { 0x5d37087b, SUC_lexicon_tags_7 },
    { 0x65d7087c, SUC_lexicon_tags_15 },
    { 0x4897087c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee170883, SUC_lexicon_tags_12 },
    { 0xa7170884, SUC_lexicon_tags_11 },
    { 0xe0970884, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7c370887, SUC_lexicon_tags_13 },
    { 0xe8770888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f70895, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d70898, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b708a0, SUC_lexicon_tags_9 },
    { 0x48f708a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x737708a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8f708a5, SUC_lexicon_tags_0 },
    { 0xd59708a6, SUC_lexicon_tags_7 },
    { 0xead708a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf6f708a9, SUC_lexicon_tags_25 },
    { 0x593708a9, SUC_lexicon_tags_103 },
    { 0x1bd708ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f708af, SUC_lexicon_tags_1 },
    { 0x6f3708b0, SUC_lexicon_tags_1 },
    { 0xe7f708b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b708b4, SUC_lexicon_tags_17 },
    { 0xccb708b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd0d708b7, SUC_lexicon_tags_27 },
    { 0xacd708b7, SUC_lexicon_tags_27 },
    { 0xc71708b9, SUC_lexicon_tags_37 },
    { 0x761708b9, SUC_lexicon_tags_11 },
    { 0x19b708b9, SUC_lexicon_tags_23 },
    { 0x70b708bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43708c0, SUC_lexicon_tags_0 },
    { 0x9cd708c0, SUC_lexicon_tags_21 },
    { 0xbbf708c2, SUC_lexicon_tags_27 },
    { 0x511708c2, SUC_lexicon_tags_15 },
    { 0x8cf708c4, SUC_lexicon_tags_11 },
    { 0xcd3708c1, SUC_lexicon_tags_27 },
    { 0x4a9708c2, SUC_lexicon_tags_27 },
    { 0xff3708c7, SUC_lexicon_tags_1 },
    { 0x8b9708c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3708d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd3708d4, SUC_lexicon_tags_14 },
    { 0x457708d4, SUC_lexicon_tags_37 },
    { 0x3d3708d4, SUC_lexicon_tags_17 },
    { 0xa85708d7, SUC_lexicon_tags_33 },
    { 0xb3708d8, SUC_lexicon_tags_30 },
    { 0xafd708d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b708dc, SUC_lexicon_tags_37 },
    { 0xbeb708dd, SUC_lexicon_tags_35 },
    { 0x28f708dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d708e1, SUC_lexicon_tags_2 },
    { 0x3ed708e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbe9708e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375708e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbb7708ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x351708ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db708ef, SUC_lexicon_tags_17 },
    { 0x6a9708f0, SUC_lexicon_tags_48 },
    { 0x2cb708f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf708f3, SUC_lexicon_tags_15 },
    { 0xf95708f4, SUC_lexicon_tags_17 },
    { 0x405708f4, SUC_lexicon_tags_151 },
    { 0x79708f6, SUC_lexicon_tags_57 },
    { 0x6b1708f7, SUC_lexicon_tags_28 },
    { 0x40d708f7, SUC_lexicon_tags_75 },
    { 0x637708f8, SUC_lexicon_tags_30 },
    { 0xe5708f6, SUC_lexicon_tags_38 },
    { 0x9f3708fb, SUC_lexicon_tags_27 },
    { 0xa27708fc, SUC_lexicon_tags_11 },
    { 0x413708fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19370902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba770905, SUC_lexicon_tags_25 },
    { 0x16570905, SUC_lexicon_tags_230 },
    { 0x20570905, SUC_lexicon_tags_1 },
    { 0x8770908, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8c97090a, SUC_lexicon_tags_56 },
    { 0xf0f7090b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcc77090d, SUC_lexicon_tags_75 },
    { 0xf957090d, SUC_lexicon_tags_37 },
    { 0x55b7090f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6d70911, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x43970913, SUC_lexicon_tags_11 },
    { 0xb970914, SUC_lexicon_tags_2 },
    { 0x95170914, SUC_lexicon_tags_1 },
    { 0xbf170914, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf317091a, SUC_lexicon_tags_7 },
    { 0x53d7091a, SUC_lexicon_tags_17 },
    { 0xc6b7091c, SUC_lexicon_tags_45 },
    { 0xba57091d, SUC_lexicon_tags_7 },
    { 0x41f7091d, SUC_lexicon_tags_25 },
    { 0x59b7091f, SUC_lexicon_tags_37 },
    { 0xe597091c, SUC_lexicon_tags_1 },
    { 0xfe370921, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac370929, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa317092b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b70930, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc570933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf857093a, SUC_lexicon_tags_33 },
    { 0xbcb7093b, SUC_lexicon_tags_0 },
    { 0xc897093c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f70940, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x18d70942, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1770945, SUC_lexicon_tags_68 },
    { 0x6a170946, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdcb70948, SUC_lexicon_tags_47 },
    { 0x9f770949, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3557094b, SUC_lexicon_tags_11 },
    { 0x4317094b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd77094e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf70953, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7d70955, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f70958, SUC_lexicon_tags_60 },
    { 0x1c770959, SUC_lexicon_tags_3 },
    { 0x8397095a, SUC_lexicon_tags_2 },
    { 0x8477095a, SUC_lexicon_tags_4 },
    { 0x9d17095c, SUC_lexicon_tags_6 },
    { 0x3a170958, SUC_lexicon_tags_6 },
    { 0x9237095e, SUC_lexicon_tags_87 },
    { 0xab17095d, SUC_lexicon_tags_15 },
    { 0x16b7095f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9b70962, SUC_lexicon_tags_0 },
    { 0x67570963, SUC_lexicon_tags_12 },
    { 0x38170964, SUC_lexicon_tags_31 },
    { 0x32770965, SUC_lexicon_tags_26 },
    { 0xa2570963, SUC_lexicon_tags_11 },
    { 0xd970967, SUC_lexicon_tags_1 },
    { 0xeb170968, SUC_lexicon_tags_38 },
    { 0x5ff70969, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d70970, SUC_lexicon_tags_63 },
    { 0x7a770971, SUC_lexicon_tags_37 },
    { 0x16b70972, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf70976, SUC_lexicon_tags_61 },
    { 0xdaf70976, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7097a, SUC_lexicon_tags_11 },
    { 0x3a37097b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1877097d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab57097f, SUC_lexicon_tags_63 },
    { 0x10570980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e570983, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d70987, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc397098a, SUC_lexicon_tags_17 },
    { 0xe557098b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb70991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45370994, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35d70996, SUC_lexicon_tags_2 },
    { 0x46170997, SUC_lexicon_tags_37 },
    { 0x3bf70998, SUC_lexicon_tags_89 },
    { 0x77d70999, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1857099b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb57099d, SUC_lexicon_tags_17 },
    { 0x657099e, SUC_lexicon_tags_25 },
    { 0x3337099f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf65709a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7709a5, SUC_lexicon_tags_11 },
    { 0xaab709a6, SUC_lexicon_tags_39 },
    { 0xc1709a7, SUC_lexicon_tags_136 },
    { 0x349709a8, SUC_lexicon_tags_79 },
    { 0xb25709a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b709ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7c3709af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41709b2, SUC_lexicon_tags_25 },
    { 0x133709b3, SUC_lexicon_tags_17 },
    { 0x411709b4, SUC_lexicon_tags_26 },
    { 0x4a3709b5, SUC_lexicon_tags_9 },
    { 0xcd9709b6, SUC_lexicon_tags_37 },
    { 0x35d709b7, SUC_lexicon_tags_37 },
    { 0x255709b6, SUC_lexicon_tags_2 },
    { 0xa69709b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109709be, SUC_lexicon_tags_49 },
    { 0x71f709bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x731709c2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xd53709c4, SUC_lexicon_tags_25 },
    { 0x2b709c5, SUC_lexicon_tags_15 },
    { 0x887709c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11709cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5709cf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b709d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db709d8, SUC_lexicon_tags_7 },
    { 0x853709d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc99709db, SUC_lexicon_tags_37 },
    { 0x4db709db, SUC_lexicon_tags_3 },
    { 0xb3b709dd, SUC_lexicon_tags_31 },
    { 0x599709dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd709e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb709ec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x81d709ee, SUC_lexicon_tags_17 },
    { 0x2cd709ee, SUC_lexicon_tags_10 },
    { 0x4b3709ee, SUC_lexicon_tags_14 },
    { 0x555709f1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x525709f3, SUC_lexicon_tags_19 },
    { 0x349709f4, SUC_lexicon_tags_27 },
    { 0x4cd709f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415709f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x215709fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a770a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22970a05, SUC_lexicon_tags_29 },
    { 0xdf970a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e170a08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3170a0f, SUC_lexicon_tags_9 },
    { 0xa8570a10, SUC_lexicon_tags_7 },
    { 0x70b70a11, SUC_lexicon_tags_37 },
    { 0xcc370a11, SUC_lexicon_tags_37 },
    { 0xcd370a11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb170a15, SUC_lexicon_tags_75 },
    { 0xa8570a15, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad70a1d, SUC_lexicon_tags_37 },
    { 0x49570a1e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d370a22, SUC_lexicon_tags_7 },
    { 0x27f70a23, SUC_lexicon_tags_12 },
    { 0x39770a24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f970a26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1170a29, SUC_lexicon_tags_11 },
    { 0x48570a2a, SUC_lexicon_tags_12 },
    { 0x8a970a2b, SUC_lexicon_tags_10 },
    { 0x6bf70a2c, SUC_lexicon_tags_2 },
    { 0xd3570a2a, SUC_lexicon_tags_37 },
    { 0xf8370a29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9770a30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8170a39, SUC_lexicon_tags_25 },
    { 0x2e770a3a, SUC_lexicon_tags_0 },
    { 0x45970a3b, SUC_lexicon_tags_2 },
    { 0x10570a3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73170a3e, SUC_lexicon_tags_21 },
    { 0x9c170a3e, SUC_lexicon_tags_68 },
    { 0x42570a40, SUC_lexicon_tags_11 },
    { 0x8bf70a40, SUC_lexicon_tags_66 },
    { 0xb2b70a3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x86170a45, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e770a4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83f70a4c, SUC_lexicon_tags_26 },
    { 0x9b370a4d, SUC_lexicon_tags_49 },
    { 0x50970a4e, SUC_lexicon_tags_48 },
    { 0x7b70a4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b70a53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3b70a55, SUC_lexicon_tags_45 },
    { 0xb0d70a56, SUC_lexicon_tags_37 },
    { 0xa3370a57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95370a5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b170a5d, SUC_lexicon_tags_37 },
    { 0x48170a5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf770a61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0370a65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58770a67, SUC_lexicon_tags_19 },
    { 0x50770a68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb770a6b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f70a6e, SUC_lexicon_tags_37 },
    { 0xc9970a6e, SUC_lexicon_tags_2 },
    { 0x49b70a70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd70a76, SUC_lexicon_tags_37 },
    { 0x8b170a77, SUC_lexicon_tags_129 },
    { 0x26170a77, SUC_lexicon_tags_14 },
    { 0x3c770a79, SUC_lexicon_tags_17 },
    { 0x19f70a7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed770a7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c370a7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f70a82, SUC_lexicon_tags_7 },
    { 0xfeb70a83, SUC_lexicon_tags_49 },
    { 0x8ef70a82, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81770a88, SUC_lexicon_tags_25 },
    { 0x87970a89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6770a8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0370a93, SUC_lexicon_tags_8 },
    { 0xeb770a94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf970a96, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdff70a98, SUC_lexicon_tags_57 },
    { 0x15170a99, SUC_lexicon_tags_47 },
    { 0xd770a9a, SUC_lexicon_tags_32 },
    { 0x8b570a99, SUC_lexicon_tags_7 },
    { 0x74570a9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31570aa3, SUC_lexicon_tags_24 },
    { 0x60970aa3, SUC_lexicon_tags_49 },
    { 0x63370aa3, SUC_lexicon_tags_11 },
    { 0xf8f70aa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8970ab3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4370ab7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f170ab9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4970abb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33770ac0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3970ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df70aca, SUC_lexicon_tags_47 },
    { 0xaf770acb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e770ace, SUC_lexicon_tags_63 },
    { 0x72f70acf, SUC_lexicon_tags_62 },
    { 0xafb70ad0, SUC_lexicon_tags_66 },
    { 0x6c970ad1, SUC_lexicon_tags_7 },
    { 0x78770ad2, SUC_lexicon_tags_1 },
    { 0x7d170ad0, SUC_lexicon_tags_29 },
    { 0xef170ad3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a970ad6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd8b70ad8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9570adb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb70adf, SUC_lexicon_tags_8 },
    { 0x50970adf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd0f70ae2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b70ae7, SUC_lexicon_tags_46 },
    { 0xe0b70ae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96b70aea, SUC_lexicon_tags_38 },
    { 0xef570aea, SUC_lexicon_tags_17 },
    { 0x3c370aeb, SUC_lexicon_tags_37 },
    { 0xd0570aed, SUC_lexicon_tags_26 },
    { 0x87b70aee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f370af1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb970af4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x15970af6, SUC_lexicon_tags_62 },
    { 0xcb170af6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67570afa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41570b01, SUC_lexicon_tags_27 },
    { 0x2cd70b02, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa370b06, SUC_lexicon_tags_151 },
    { 0x2170b07, SUC_lexicon_tags_35 },
    { 0x57d70b07, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb1b70b0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c570b0e, SUC_lexicon_tags_0 },
    { 0xc770b0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d70b13, SUC_lexicon_tags_17 },
    { 0x39d70b14, SUC_lexicon_tags_0 },
    { 0xe8570b14, SUC_lexicon_tags_38 },
    { 0xef170b16, SUC_lexicon_tags_19 },
    { 0xaeb70b17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x81970b19, SUC_lexicon_tags_18 },
    { 0x2570b1a, SUC_lexicon_tags_37 },
    { 0x3370b1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25370b1e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xad370b21, SUC_lexicon_tags_32 },
    { 0x6b570b21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9770b26, SUC_lexicon_tags_42 },
    { 0x90370b26, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d70b2f, SUC_lexicon_tags_38 },
    { 0x78770b30, SUC_lexicon_tags_11 },
    { 0xaa770b2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14570b37, SUC_lexicon_tags_37 },
    { 0x45f70b37, SUC_lexicon_tags_10 },
    { 0x76b70b38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55370b43, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x16970b46, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x93970b48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d70b4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18d70b4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69570b51, SUC_lexicon_tags_30 },
    { 0x12570b51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd7770b54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac970b5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60570b5e, SUC_lexicon_tags_7 },
    { 0x3c570b5f, SUC_lexicon_tags_27 },
    { 0xc2b70b5e, SUC_lexicon_tags_37 },
    { 0x71b70b5e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f70b65, SUC_lexicon_tags_12 },
    { 0xb5570b65, SUC_lexicon_tags_15 },
    { 0xd7370b67, SUC_lexicon_tags_15 },
    { 0x57770b68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1570b75, SUC_lexicon_tags_1 },
    { 0xfdb70b76, SUC_lexicon_tags_7 },
    { 0xbab70b77, SUC_lexicon_tags_15 },
    { 0x9fd70b77, SUC_lexicon_tags_15 },
    { 0x7e170b75, SUC_lexicon_tags_33 },
    { 0x85b70b7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7970b7e, SUC_lexicon_tags_47 },
    { 0x1db70b7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9970b81, SUC_lexicon_tags_10 },
    { 0xb2970b82, SUC_lexicon_tags_12 },
    { 0xfa170b83, SUC_lexicon_tags_1 },
    { 0x53970b84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c970b86, SUC_lexicon_tags_11 },
    { 0x36d70b87, SUC_lexicon_tags_68 },
    { 0xa5770b88, SUC_lexicon_tags_28 },
    { 0xa1b70b89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd70b8c, SUC_lexicon_tags_23 },
    { 0x1e770b8d, SUC_lexicon_tags_190 },
    { 0x5a170b8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8770b93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5770b97, SUC_lexicon_tags_12 },
    { 0x92970b98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b770b9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7770b9d, SUC_lexicon_tags_14 },
    { 0xdfd70b9e, SUC_lexicon_tags_25 },
    { 0xeab70b9e, SUC_lexicon_tags_4 },
    { 0x1f170b9e, SUC_lexicon_tags_11 },
    { 0x2cf70b9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5f70ba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5770ba7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd70bae, SUC_lexicon_tags_27 },
    { 0x14d70bae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf570bb3, SUC_lexicon_tags_37 },
    { 0xca770bb4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x31370bb6, SUC_lexicon_tags_7 },
    { 0xb8b70bb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb970bbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39570bc1, SUC_lexicon_tags_11 },
    { 0x54d70bc2, SUC_lexicon_tags_66 },
    { 0x80170bc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b70bc9, SUC_lexicon_tags_38 },
    { 0x2d170bca, SUC_lexicon_tags_26 },
    { 0x72170bcb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38370bd0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf70bda, SUC_lexicon_tags_160 },
    { 0xaeb70bdb, SUC_lexicon_tags_11 },
    { 0x31170bda, SUC_lexicon_tags_14 },
    { 0xc4970bdc, SUC_lexicon_tags_26 },
    { 0xefd70bde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b170be1, SUC_lexicon_tags_17 },
    { 0x27170be2, SUC_lexicon_tags_31 },
    { 0x2370be1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd370be8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b70bed, SUC_lexicon_tags_43 },
    { 0x37570bed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd70bf1, SUC_lexicon_tags_60 },
    { 0xd70bf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff70bfc, SUC_lexicon_tags_17 },
    { 0xa0970bfc, SUC_lexicon_tags_37 },
    { 0x65770bfd, SUC_lexicon_tags_2 },
    { 0xddf70bfe, SUC_lexicon_tags_25 },
    { 0xc8570c00, SUC_lexicon_tags_2 },
    { 0x95f70c00, SUC_lexicon_tags_30 },
    { 0xa3970c02, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1c570c04, SUC_lexicon_tags_17 },
    { 0x7cd70c04, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x97b70c07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d170c09, SUC_lexicon_tags_26 },
    { 0xeef70c0a, SUC_lexicon_tags_31 },
    { 0x54370c0a, SUC_lexicon_tags_11 },
    { 0xc9770c0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b70c0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f70c12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7370c15, SUC_lexicon_tags_37 },
    { 0xadb70c16, SUC_lexicon_tags_37 },
    { 0x30770c17, SUC_lexicon_tags_17 },
    { 0x4f770c17, SUC_lexicon_tags_17 },
    { 0x97770c18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f70c1c, SUC_lexicon_tags_25 },
    { 0xcad70c1c, SUC_lexicon_tags_18 },
    { 0x26d70c1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3570c22, SUC_lexicon_tags_38 },
    { 0x85570c23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1970c26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f70c29, SUC_lexicon_tags_32 },
    { 0xb6570c2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b70c2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff70c31, SUC_lexicon_tags_179 },
    { 0xc9d70c32, SUC_lexicon_tags_48 },
    { 0xced70c33, SUC_lexicon_tags_61 },
    { 0xc7f70c34, SUC_lexicon_tags_37 },
    { 0xd8770c35, SUC_lexicon_tags_37 },
    { 0x8b170c35, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f70c39, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e770c41, SUC_lexicon_tags_65 },
    { 0x6e170c42, SUC_lexicon_tags_6 },
    { 0xa5370c42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb970c45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a770c4a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x79b70c4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb70c4f, SUC_lexicon_tags_27 },
    { 0xaef70c50, SUC_lexicon_tags_66 },
    { 0x68b70c51, SUC_lexicon_tags_26 },
    { 0x33f70c51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35970c57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39970c62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94b70c64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d70c67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd6370c69, SUC_lexicon_tags_1 },
    { 0xd9d70c6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77370c70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe770c79, SUC_lexicon_tags_11 },
    { 0x1f370c7a, SUC_lexicon_tags_10 },
    { 0x43570c7b, SUC_lexicon_tags_15 },
    { 0x88370c7c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd70c7f, SUC_lexicon_tags_26 },
    { 0xcb570c7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b70c86, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78570c8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e770c8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b70c91, SUC_lexicon_tags_27 },
    { 0x81970c91, SUC_lexicon_tags_15 },
    { 0x3770c91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58170c97, SUC_lexicon_tags_11 },
    { 0x9770c98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd770ca3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b70ca7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6770caa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e970cae, SUC_lexicon_tags_16 },
    { 0xe6b70cae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d370cb9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af70cbf, SUC_lexicon_tags_37 },
    { 0x58370cc0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x370cc2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1770cc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x31370cc9, SUC_lexicon_tags_25 },
    { 0xf1b70cc9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b70ccd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9170cd0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74770cd4, SUC_lexicon_tags_33 },
    { 0x11b70cd5, SUC_lexicon_tags_32 },
    { 0x6cd70cd6, SUC_lexicon_tags_38 },
    { 0x38970cd7, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa170ce0, SUC_lexicon_tags_6 },
    { 0xb6f70ce0, SUC_lexicon_tags_584 },
    { 0xb9370ce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b70ce7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4370ce9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3b970ceb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2f370ced, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a970cf0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b770cf3, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d70cf6, SUC_lexicon_tags_63 },
    { 0xdb770cf7, SUC_lexicon_tags_56 },
    { 0xbff70cf8, SUC_lexicon_tags_21 },
    { 0x24770cf7, SUC_lexicon_tags_19 },
    { 0x5b570cfa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f70d00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63570d06, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1770d08, SUC_lexicon_tags_28 },
    { 0xeb170d08, SUC_lexicon_tags_15 },
    { 0x75370d08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88170d0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf370d16, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b70d1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30b70d20, SUC_lexicon_tags_21 },
    { 0x5a170d21, SUC_lexicon_tags_11 },
    { 0x71970d22, SUC_lexicon_tags_23 },
    { 0xc7b70d23, SUC_lexicon_tags_32 },
    { 0x8fd70d24, SUC_lexicon_tags_55 },
    { 0x53b70d21, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d70d28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cb70d2a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x79770d2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef70d32, SUC_lexicon_tags_15 },
    { 0x88570d32, SUC_lexicon_tags_11 },
    { 0x5ab70d32, SUC_lexicon_tags_15 },
    { 0xe7170d35, SUC_lexicon_tags_37 },
    { 0x79570d33, SUC_lexicon_tags_26 },
    { 0x52f70d35, SUC_lexicon_tags_65 },
    { 0xa6b70d34, SUC_lexicon_tags_27 },
    { 0x3b770d34, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x23170d3b, SUC_lexicon_tags_46 },
    { 0xb6770d3c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2170d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76570d41, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d70d44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d70d48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x84770d4b, SUC_lexicon_tags_26 },
    { 0x94570d4c, SUC_lexicon_tags_25 },
    { 0xaff70d4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1f70d4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8b70d51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5170d55, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x10170d57, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xa1970d59, SUC_lexicon_tags_352 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd70d5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b70d62, SUC_lexicon_tags_37 },
    { 0x77d70d63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56970d65, SUC_lexicon_tags_80 },
    { 0x86570d66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15970d6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19170d6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47170d71, SUC_lexicon_tags_2 },
    { 0x2db70d72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58170d74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df70d7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f70d7d, SUC_lexicon_tags_26 },
    { 0x81d70d7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9f70d80, SUC_lexicon_tags_22 },
    { 0xf5b70d81, SUC_lexicon_tags_27 },
    { 0x64d70d82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d370d84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d170d88, SUC_lexicon_tags_31 },
    { 0xf370d89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f70d8e, SUC_lexicon_tags_45 },
    { 0xd0f70d8f, SUC_lexicon_tags_9 },
    { 0x41b70d90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4970d94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b70d99, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc3f70d9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85170da2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x51770da5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40970da9, SUC_lexicon_tags_26 },
    { 0xfc570da9, SUC_lexicon_tags_2 },
    { 0xc2f70dab, SUC_lexicon_tags_27 },
    { 0xd8970dac, SUC_lexicon_tags_14 },
    { 0x43f70dad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29370db2, SUC_lexicon_tags_37 },
    { 0x1bf70db2, SUC_lexicon_tags_14 },
    { 0xebd70db2, SUC_lexicon_tags_6 },
    { 0x52d70db5, SUC_lexicon_tags_25 },
    { 0xf0570db5, SUC_lexicon_tags_43 },
    { 0xc8170db3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xedf70db9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce770dbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6df70dbe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f70dc4, SUC_lexicon_tags_7 },
    { 0x34b70dc5, SUC_lexicon_tags_9 },
    { 0x89d70dc6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe0970dc8, SUC_lexicon_tags_81 },
    { 0x1b170dc9, SUC_lexicon_tags_0 },
    { 0x44570dca, SUC_lexicon_tags_28 },
    { 0xe8b70dc8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a370dd5, SUC_lexicon_tags_7 },
    { 0x54d70dd6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16570dda, SUC_lexicon_tags_17 },
    { 0xd5f70dda, SUC_lexicon_tags_17 },
    { 0x17970ddc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66170dde, SUC_lexicon_tags_35 },
    { 0xb8f70ddf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdb570de1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa6570de3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb170de6, SUC_lexicon_tags_12 },
    { 0x37b70de7, SUC_lexicon_tags_25 },
    { 0xe7970de7, SUC_lexicon_tags_55 },
    { 0x12970de9, SUC_lexicon_tags_2 },
    { 0x1d970dea, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe2570dec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67b70dee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28970df0, SUC_lexicon_tags_49 },
    { 0xe9370df1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x48170df3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x62570df5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7370df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5370dff, SUC_lexicon_tags_34 },
    { 0xa9f70e00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x59f70e02, SUC_lexicon_tags_25 },
    { 0x20b70e03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33f70e05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28770e07, SUC_lexicon_tags_17 },
    { 0x34570e08, SUC_lexicon_tags_0 },
    { 0xd2170e09, SUC_lexicon_tags_15 },
    { 0xd1970e07, SUC_lexicon_tags_17 },
    { 0x6ef70e0a, SUC_lexicon_tags_0 },
    { 0x2f570e08, SUC_lexicon_tags_11 },
    { 0xd6d70e0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b70e15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62570e19, SUC_lexicon_tags_60 },
    { 0xc6b70e1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb70e1f, SUC_lexicon_tags_66 },
    { 0x4b970e20, SUC_lexicon_tags_33 },
    { 0x41970e21, SUC_lexicon_tags_7 },
    { 0x5e970e22, SUC_lexicon_tags_8 },
    { 0x82170e22, SUC_lexicon_tags_678 },
    { 0x6ad70e24, SUC_lexicon_tags_33 },
    { 0x38d70e25, SUC_lexicon_tags_15 },
    { 0xae970e26, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1370e2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xca170e2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1970e31, SUC_lexicon_tags_3 },
    { 0xa7770e32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb570e34, SUC_lexicon_tags_29 },
    { 0x1ab70e35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1f70e37, SUC_lexicon_tags_1 },
    { 0x3f370e37, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9c570e3a, SUC_lexicon_tags_11 },
    { 0xdc970e3b, SUC_lexicon_tags_17 },
    { 0x1bf70e3c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4170e40, SUC_lexicon_tags_17 },
    { 0x67d70e41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6970e46, SUC_lexicon_tags_15 },
    { 0xe7b70e46, SUC_lexicon_tags_19 },
    { 0xaf570e47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe170e54, SUC_lexicon_tags_141 },
    { 0x66770e55, SUC_lexicon_tags_989 },
    { 0xe370e54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d70e5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb70e5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b70e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9b70e64, SUC_lexicon_tags_32 },
    { 0x7570e65, SUC_lexicon_tags_25 },
    { 0xa1d70e65, SUC_lexicon_tags_43 },
    { 0xca970e67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d170e6a, SUC_lexicon_tags_1 },
    { 0x61770e6b, SUC_lexicon_tags_3 },
    { 0x50b70e6c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b70e70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f70e74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d70e77, SUC_lexicon_tags_66 },
    { 0x82570e77, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x86370e7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b70e7d, SUC_lexicon_tags_10 },
    { 0x7570e7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a370e86, SUC_lexicon_tags_37 },
    { 0x72970e86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1570e89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db70e8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45370e91, SUC_lexicon_tags_55 },
    { 0xefb70e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb70e95, SUC_lexicon_tags_15 },
    { 0x28d70e96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfa170e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61770e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2d70e9e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4770ea5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9970ea8, SUC_lexicon_tags_1 },
    { 0x5db70ea8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d970eac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe7d70eae, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d70eb2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5770eba, SUC_lexicon_tags_37 },
    { 0x1eb70ebb, SUC_lexicon_tags_11 },
    { 0x2b70ebc, SUC_lexicon_tags_1 },
    { 0xd6b70ebd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2770ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b70ec7, SUC_lexicon_tags_61 },
    { 0xf7970ec8, SUC_lexicon_tags_29 },
    { 0x57170ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b70ed0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x66170ed3, SUC_lexicon_tags_46 },
    { 0xf2b70ed3, SUC_lexicon_tags_3 },
    { 0x13370ed3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb570ed8, SUC_lexicon_tags_12 },
    { 0x85570ed8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd0370edb, SUC_lexicon_tags_17 },
    { 0x13570edc, SUC_lexicon_tags_37 },
    { 0x2ed70edc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef170ee0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f370ee2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1770ee5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25770ee9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe8170eeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63970eee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94970ef5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3370ef8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf9970efa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3170efe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6570f02, SUC_lexicon_tags_9 },
    { 0xf0d70f03, SUC_lexicon_tags_1 },
    { 0x4ed70f04, SUC_lexicon_tags_8 },
    { 0xddf70f05, SUC_lexicon_tags_27 },
    { 0x4fb70f05, SUC_lexicon_tags_25 },
    { 0xdd70f07, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb7b70f09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c970f11, SUC_lexicon_tags_11 },
    { 0x9cf70f12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9770f16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x91370f19, SUC_lexicon_tags_21 },
    { 0x86d70f1a, SUC_lexicon_tags_17 },
    { 0xfb970f1a, SUC_lexicon_tags_14 },
    { 0x49570f1c, SUC_lexicon_tags_13 },
    { 0x94b70f1d, SUC_lexicon_tags_15 },
    { 0xcfb70f1d, SUC_lexicon_tags_25 },
    { 0x6bb70f1e, SUC_lexicon_tags_60 },
    { 0xbeb70f1c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed70f29, SUC_lexicon_tags_82 },
    { 0x7c970f29, SUC_lexicon_tags_30 },
    { 0x27770f2b, SUC_lexicon_tags_27 },
    { 0x11f70f2a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f970f34, SUC_lexicon_tags_8 },
    { 0x6d770f35, SUC_lexicon_tags_7 },
    { 0x80b70f36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97970f39, SUC_lexicon_tags_22 },
    { 0xb6370f3a, SUC_lexicon_tags_4 },
    { 0x72b70f3b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x81370f3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb70f3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42170f43, SUC_lexicon_tags_57 },
    { 0x70370f44, SUC_lexicon_tags_66 },
    { 0xcb570f45, SUC_lexicon_tags_38 },
    { 0xcc970f46, SUC_lexicon_tags_47 },
    { 0x45f70f46, SUC_lexicon_tags_25 },
    { 0x11970f48, SUC_lexicon_tags_12 },
    { 0x40d70f48, SUC_lexicon_tags_87 },
    { 0x15370f4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d70f51, SUC_lexicon_tags_27 },
    { 0x98570f51, SUC_lexicon_tags_32 },
    { 0x2a970f53, SUC_lexicon_tags_12 },
    { 0xcdd70f54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74b70f56, SUC_lexicon_tags_26 },
    { 0xdb970f57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3970f5b, SUC_lexicon_tags_7 },
    { 0xa1970f5c, SUC_lexicon_tags_17 },
    { 0xfe970f5d, SUC_lexicon_tags_36 },
    { 0x12970f5e, SUC_lexicon_tags_1 },
    { 0x9b770f5c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23570f65, SUC_lexicon_tags_7 },
    { 0x5d770f66, SUC_lexicon_tags_23 },
    { 0xf7d70f66, SUC_lexicon_tags_25 },
    { 0xbd570f68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3770f6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b70f70, SUC_lexicon_tags_14 },
    { 0x7b370f71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43f70f73, SUC_lexicon_tags_17 },
    { 0xf1170f74, SUC_lexicon_tags_16 },
    { 0x1cf70f75, SUC_lexicon_tags_25 },
    { 0xb7d70f73, SUC_lexicon_tags_23 },
    { 0xa3b70f73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6570f7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62970f7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce770f7e, SUC_lexicon_tags_30 },
    { 0x8ff70f7f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbc770f81, SUC_lexicon_tags_14 },
    { 0xb2f70f82, SUC_lexicon_tags_1 },
    { 0x5570f81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bb70f85, SUC_lexicon_tags_14 },
    { 0xb8170f85, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2970f8b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97970f8f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9970f94, SUC_lexicon_tags_13 },
    { 0xf7b70f94, SUC_lexicon_tags_26 },
    { 0x4fd70f96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9db70f98, SUC_lexicon_tags_0 },
    { 0x66970f99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80b70f9b, SUC_lexicon_tags_8 },
    { 0xde770f9c, SUC_lexicon_tags_134 },
    { 0x35770f9d, SUC_lexicon_tags_25 },
    { 0x84570f9e, SUC_lexicon_tags_7 },
    { 0x79170f9f, SUC_lexicon_tags_13 },
    { 0x72970f9b, SUC_lexicon_tags_63 },
    { 0xf5d70f9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d70fa6, SUC_lexicon_tags_66 },
    { 0x49d70fa7, SUC_lexicon_tags_37 },
    { 0x4a970fa8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98770faf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a970fb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b370fb5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x78570fb7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf970fbd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b370fc0, SUC_lexicon_tags_63 },
    { 0x39f70fc1, SUC_lexicon_tags_15 },
    { 0x1f370fc0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x770fc5, SUC_lexicon_tags_65 },
    { 0x90f70fc6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3770fc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0570fcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90d70fcf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3ff70fd1, SUC_lexicon_tags_9 },
    { 0xd3b70fd2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f570fd5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x51370fd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d70fdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26770fe3, SUC_lexicon_tags_30 },
    { 0x87170fe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab70fe8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78770fea, SUC_lexicon_tags_14 },
    { 0x3c370fea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb370ff4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5770ffb, SUC_lexicon_tags_17 },
    { 0xe8370ffc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5bf70ffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59571005, SUC_lexicon_tags_31 },
    { 0xa4371006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117100f, SUC_lexicon_tags_15 },
    { 0xd217100f, SUC_lexicon_tags_2 },
    { 0xd371011, SUC_lexicon_tags_9 },
    { 0x2371012, SUC_lexicon_tags_37 },
    { 0x67371013, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d71019, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c97101c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b71020, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa9971022, SUC_lexicon_tags_15 },
    { 0x93971022, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c571027, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7737102b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a771035, SUC_lexicon_tags_7 },
    { 0x4cf71036, SUC_lexicon_tags_11 },
    { 0xe7971036, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377103b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x3457103d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e171041, SUC_lexicon_tags_49 },
    { 0xbe971042, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78171046, SUC_lexicon_tags_49 },
    { 0xa1d71047, SUC_lexicon_tags_33 },
    { 0xf9771047, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd7104c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b71052, SUC_lexicon_tags_15 },
    { 0x6f71053, SUC_lexicon_tags_37 },
    { 0x50d71054, SUC_lexicon_tags_12 },
    { 0x4ed71055, SUC_lexicon_tags_33 },
    { 0x6af71056, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29971058, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe297105a, SUC_lexicon_tags_9 },
    { 0x37b7105a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3171060, SUC_lexicon_tags_39 },
    { 0x58371061, SUC_lexicon_tags_1 },
    { 0xf371061, SUC_lexicon_tags_90 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab71065, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5171067, SUC_lexicon_tags_33 },
    { 0xec771068, SUC_lexicon_tags_19 },
    { 0x8ad71069, SUC_lexicon_tags_49 },
    { 0x43b7106a, SUC_lexicon_tags_21 },
    { 0xb137106a, SUC_lexicon_tags_7 },
    { 0x99d7106a, SUC_lexicon_tags_45 },
    { 0xd9371067, SUC_lexicon_tags_1 },
    { 0x6437106d, SUC_lexicon_tags_14 },
    { 0xb397106d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x95d71071, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f71074, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70771079, SUC_lexicon_tags_17 },
    { 0xc1771079, SUC_lexicon_tags_17 },
    { 0x99f7107b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2577107d, SUC_lexicon_tags_14 },
    { 0xa537107d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb71081, SUC_lexicon_tags_17 },
    { 0x44f71082, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x33571084, SUC_lexicon_tags_329 },
    { 0x54b71085, SUC_lexicon_tags_21 },
    { 0xdf971084, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a77108a, SUC_lexicon_tags_35 },
    { 0xad57108a, SUC_lexicon_tags_17 },
    { 0x6477108c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf097108e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43371092, SUC_lexicon_tags_14 },
    { 0x74f71092, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4357109b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7b7109d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefb7109f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8b3710a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65710a8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd710ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf710ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1b710b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b710b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d710b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29710c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105710c6, SUC_lexicon_tags_62 },
    { 0xaeb710c7, SUC_lexicon_tags_12 },
    { 0xf79710c8, SUC_lexicon_tags_35 },
    { 0xef9710c8, SUC_lexicon_tags_37 },
    { 0xec5710c8, SUC_lexicon_tags_37 },
    { 0xc2d710cb, SUC_lexicon_tags_39 },
    { 0x9e7710cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53710d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e5710d3, SUC_lexicon_tags_43 },
    { 0xa5f710d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219710d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113710de, SUC_lexicon_tags_29 },
    { 0x551710df, SUC_lexicon_tags_63 },
    { 0x81b710e0, SUC_lexicon_tags_188 },
    { 0xb6b710de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7710e9, SUC_lexicon_tags_7 },
    { 0x4fd710ea, SUC_lexicon_tags_38 },
    { 0x239710eb, SUC_lexicon_tags_25 },
    { 0x3c1710eb, SUC_lexicon_tags_56 },
    { 0x6e5710ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x815710ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571710f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db710fd, SUC_lexicon_tags_84 },
    { 0xb3710fe, SUC_lexicon_tags_8 },
    { 0x4d710ff, SUC_lexicon_tags_5 },
    { 0x86f71100, SUC_lexicon_tags_9 },
    { 0x1df71101, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d71104, SUC_lexicon_tags_23 },
    { 0xb0171105, SUC_lexicon_tags_15 },
    { 0x67971104, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23171108, SUC_lexicon_tags_17 },
    { 0xbed71109, SUC_lexicon_tags_11 },
    { 0xf7371109, SUC_lexicon_tags_7 },
    { 0xf717110a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcbd7110d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c371111, SUC_lexicon_tags_11 },
    { 0x67371112, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb71117, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc777111a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb437111e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb71121, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x50171124, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeef71126, SUC_lexicon_tags_15 },
    { 0x99371127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b17112b, SUC_lexicon_tags_37 },
    { 0x2cb7112c, SUC_lexicon_tags_14 },
    { 0xc577112d, SUC_lexicon_tags_32 },
    { 0x397112e, SUC_lexicon_tags_11 },
    { 0x3977112f, SUC_lexicon_tags_17 },
    { 0x6b77112e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6971134, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e171141, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc171149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa337114b, SUC_lexicon_tags_15 },
    { 0x3d77114b, SUC_lexicon_tags_34 },
    { 0xbb97114d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26571155, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd8571157, SUC_lexicon_tags_37 },
    { 0x51971158, SUC_lexicon_tags_14 },
    { 0xd8971157, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37115d, SUC_lexicon_tags_28 },
    { 0xdeb7115e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33371163, SUC_lexicon_tags_12 },
    { 0x2a571164, SUC_lexicon_tags_1 },
    { 0xb3371163, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77571167, SUC_lexicon_tags_2 },
    { 0xe3f71167, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f7116e, SUC_lexicon_tags_1 },
    { 0x7617116e, SUC_lexicon_tags_37 },
    { 0xea771170, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db71177, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf171179, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b7117c, SUC_lexicon_tags_60 },
    { 0x58f7117c, SUC_lexicon_tags_60 },
    { 0xbb57117e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e771182, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df71187, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a97118a, SUC_lexicon_tags_31 },
    { 0x4a17118a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9737118d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80971194, SUC_lexicon_tags_17 },
    { 0xf5571195, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8bd71197, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x66371199, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d7119c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f711a0, SUC_lexicon_tags_4 },
    { 0x2d5711a0, SUC_lexicon_tags_29 },
    { 0x245711a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843711ac, SUC_lexicon_tags_25 },
    { 0x19f711ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1711b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc95711b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761711be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761711c2, SUC_lexicon_tags_12 },
    { 0xb29711c3, SUC_lexicon_tags_17 },
    { 0x127711c4, SUC_lexicon_tags_7 },
    { 0x15b711c3, SUC_lexicon_tags_8 },
    { 0x811711c6, SUC_lexicon_tags_37 },
    { 0xb53711c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb711cd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49711d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61711d3, SUC_lexicon_tags_37 },
    { 0x543711d4, SUC_lexicon_tags_1 },
    { 0xc5f711d3, SUC_lexicon_tags_6 },
    { 0x9c1711d6, SUC_lexicon_tags_25 },
    { 0xf31711d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb77711d9, SUC_lexicon_tags_20 },
    { 0xf47711da, SUC_lexicon_tags_25 },
    { 0x323711db, SUC_lexicon_tags_11 },
    { 0x13711dc, SUC_lexicon_tags_6 },
    { 0x9c3711dc, SUC_lexicon_tags_30 },
    { 0x391711de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43b711e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f711e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5711e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x131711e9, SUC_lexicon_tags_25 },
    { 0x585711ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3711ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77711f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41711fe, SUC_lexicon_tags_74 },
    { 0x55f711ff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4db71201, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x73d71203, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x29571205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d77120a, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0xf577120c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf777120f, SUC_lexicon_tags_1 },
    { 0xa2371210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25771214, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd571218, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd157121d, SUC_lexicon_tags_25 },
    { 0xbaf7121e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d371220, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b71225, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa897122c, SUC_lexicon_tags_7 },
    { 0x7f7122d, SUC_lexicon_tags_31 },
    { 0xcc17122e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4971231, SUC_lexicon_tags_25 },
    { 0xf3971232, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb71236, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4571240, SUC_lexicon_tags_8 },
    { 0x3e771241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50571245, SUC_lexicon_tags_25 },
    { 0x98b71246, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe817124d, SUC_lexicon_tags_1 },
    { 0xf9f7124e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad71252, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x83371255, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb771259, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8997125c, SUC_lexicon_tags_37 },
    { 0x1537125d, SUC_lexicon_tags_2 },
    { 0x45d7125e, SUC_lexicon_tags_17 },
    { 0x5b17125f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35f71261, SUC_lexicon_tags_48 },
    { 0x4b571262, SUC_lexicon_tags_15 },
    { 0x19d71262, SUC_lexicon_tags_12 },
    { 0xd8b71264, SUC_lexicon_tags_101 },
    { 0xce571265, SUC_lexicon_tags_37 },
    { 0xe4571261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a77126a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x81b7126c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb717126e, SUC_lexicon_tags_39 },
    { 0x637126e, SUC_lexicon_tags_17 },
    { 0xaf171270, SUC_lexicon_tags_170 },
    { 0x4717126e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73971274, SUC_lexicon_tags_9 },
    { 0xfb71274, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcc171277, SUC_lexicon_tags_6 },
    { 0x1d171277, SUC_lexicon_tags_45 },
    { 0x36d71278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa37127b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd7127d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db71282, SUC_lexicon_tags_4 },
    { 0xf3d71283, SUC_lexicon_tags_8 },
    { 0x76d71284, SUC_lexicon_tags_49 },
    { 0x15f71285, SUC_lexicon_tags_23 },
    { 0x70571284, SUC_lexicon_tags_31 },
    { 0xc2f71282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7128c, SUC_lexicon_tags_17 },
    { 0x6ad7128d, SUC_lexicon_tags_1 },
    { 0x7597128e, SUC_lexicon_tags_21 },
    { 0xe3b7128f, SUC_lexicon_tags_0 },
    { 0x59b7128f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42771298, SUC_lexicon_tags_31 },
    { 0x7cb71299, SUC_lexicon_tags_25 },
    { 0xa9f71298, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3037129d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3712a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49712ac, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0xd59712ae, SUC_lexicon_tags_15 },
    { 0xc75712af, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x917712b1, SUC_lexicon_tags_19 },
    { 0xfbb712b2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb01712b4, SUC_lexicon_tags_11 },
    { 0xb6b712b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa65712b7, SUC_lexicon_tags_32 },
    { 0x535712b8, SUC_lexicon_tags_37 },
    { 0xa83712b9, SUC_lexicon_tags_17 },
    { 0x4e7712b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf01712bc, SUC_lexicon_tags_2 },
    { 0x443712bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdef712bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571712c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xce7712c5, SUC_lexicon_tags_11 },
    { 0x3e1712c6, SUC_lexicon_tags_39 },
    { 0x24b712c6, SUC_lexicon_tags_56 },
    { 0xcc9712c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa05712ca, SUC_lexicon_tags_2 },
    { 0x745712cb, SUC_lexicon_tags_12 },
    { 0xc7b712cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x417712ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1b712d0, SUC_lexicon_tags_47 },
    { 0x477712d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3712d5, SUC_lexicon_tags_39 },
    { 0xe89712d6, SUC_lexicon_tags_56 },
    { 0xc3d712d5, SUC_lexicon_tags_11 },
    { 0x937712d8, SUC_lexicon_tags_47 },
    { 0x275712d9, SUC_lexicon_tags_8 },
    { 0x26d712da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7712dd, SUC_lexicon_tags_12 },
    { 0xb35712dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x339712e1, SUC_lexicon_tags_25 },
    { 0xbd5712e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95712e6, SUC_lexicon_tags_60 },
    { 0xd97712e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2b1712e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77712ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975712f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5712fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac171300, SUC_lexicon_tags_1 },
    { 0x66571301, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b7130a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3057130c, SUC_lexicon_tags_17 },
    { 0x4cb7130d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f71315, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa3171317, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae97131c, SUC_lexicon_tags_20 },
    { 0x5437131d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3357131f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e371322, SUC_lexicon_tags_2 },
    { 0x98971323, SUC_lexicon_tags_37 },
    { 0xd8971324, SUC_lexicon_tags_25 },
    { 0xc1571325, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x16371328, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfab7132a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f7132e, SUC_lexicon_tags_1 },
    { 0x4ff7132e, SUC_lexicon_tags_45 },
    { 0xf4f71330, SUC_lexicon_tags_37 },
    { 0xb1771331, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91171335, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94571339, SUC_lexicon_tags_29 },
    { 0xeb37133a, SUC_lexicon_tags_14 },
    { 0xf0f7133a, SUC_lexicon_tags_0 },
    { 0xef7133a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96171342, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11d71344, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d37134a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8eb7134c, SUC_lexicon_tags_49 },
    { 0x5b57134d, SUC_lexicon_tags_23 },
    { 0x5d7134c, SUC_lexicon_tags_104 },
    { 0xa637134c, SUC_lexicon_tags_65 },
    { 0x26d71350, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac171354, SUC_lexicon_tags_45 },
    { 0x49971355, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x86771358, SUC_lexicon_tags_15 },
    { 0x8f171358, SUC_lexicon_tags_2 },
    { 0x5d971358, SUC_lexicon_tags_27 },
    { 0x32b7135b, SUC_lexicon_tags_48 },
    { 0xcd7135c, SUC_lexicon_tags_33 },
    { 0x4477135b, SUC_lexicon_tags_307 },
    { 0x6597135e, SUC_lexicon_tags_7 },
    { 0x8517135f, SUC_lexicon_tags_21 },
    { 0x5cb71360, SUC_lexicon_tags_45 },
    { 0xe1371359, SUC_lexicon_tags_27 },
    { 0x6b971362, SUC_lexicon_tags_38 },
    { 0x6bb71363, SUC_lexicon_tags_79 },
    { 0xf7f7135b, SUC_lexicon_tags_1 },
    { 0x577135b, SUC_lexicon_tags_2 },
    { 0xbcb71366, SUC_lexicon_tags_23 },
    { 0xeb71358, SUC_lexicon_tags_47 },
    { 0xd1b7135c, SUC_lexicon_tags_25 },
    { 0x6db71369, SUC_lexicon_tags_31 },
    { 0x8a97136a, SUC_lexicon_tags_11 },
    { 0xe857136b, SUC_lexicon_tags_12 },
    { 0x2f971369, SUC_lexicon_tags_23 },
    { 0x62f7136d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11771372, SUC_lexicon_tags_2 },
    { 0xab171372, SUC_lexicon_tags_48 },
    { 0x19371374, SUC_lexicon_tags_27 },
    { 0x11771372, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xadb71377, SUC_lexicon_tags_28 },
    { 0x36371378, SUC_lexicon_tags_47 },
    { 0xd9171379, SUC_lexicon_tags_45 },
    { 0x95b71379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ff7137c, SUC_lexicon_tags_23 },
    { 0x3777137d, SUC_lexicon_tags_47 },
    { 0x5717137e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61771380, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96971382, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9f71384, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d71389, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x4a97138b, SUC_lexicon_tags_38 },
    { 0x5c37138c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd657138e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb71392, SUC_lexicon_tags_8 },
    { 0x17971392, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b171395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d71398, SUC_lexicon_tags_2 },
    { 0x5171399, SUC_lexicon_tags_2 },
    { 0x81b7139a, SUC_lexicon_tags_7 },
    { 0xd317139b, SUC_lexicon_tags_45 },
    { 0xe0f7139c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129713a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x96b713a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x427713a6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275713aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f713b1, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b713be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5713c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5713c8, SUC_lexicon_tags_31 },
    { 0xa7d713c8, SUC_lexicon_tags_17 },
    { 0x31f713ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9713cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b713d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x795713d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d713da, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571713de, SUC_lexicon_tags_1 },
    { 0xbd9713de, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef713e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563713e9, SUC_lexicon_tags_37 },
    { 0x6cb713e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3713ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1ff713ef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeaf713f1, SUC_lexicon_tags_17 },
    { 0x565713f1, SUC_lexicon_tags_11 },
    { 0xf6d713f2, SUC_lexicon_tags_70 },
    { 0xebd713f4, SUC_lexicon_tags_32 },
    { 0x301713f1, SUC_lexicon_tags_0 },
    { 0x951713f6, SUC_lexicon_tags_37 },
    { 0x7e9713f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79713fb, SUC_lexicon_tags_25 },
    { 0xb81713fc, SUC_lexicon_tags_11 },
    { 0xa75713fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc3d713ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf171402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbaf71404, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x87d71406, SUC_lexicon_tags_7 },
    { 0x29371407, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad77140d, SUC_lexicon_tags_1 },
    { 0x5477140d, SUC_lexicon_tags_252 },
    { 0xf457140e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2171416, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75f71418, SUC_lexicon_tags_29 },
    { 0xd3971418, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7237141d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6971421, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d71426, SUC_lexicon_tags_37 },
    { 0xac171427, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff7142c, SUC_lexicon_tags_1 },
    { 0x2417142d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda371430, SUC_lexicon_tags_1 },
    { 0xf9371431, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa371439, SUC_lexicon_tags_7 },
    { 0x5017143a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1257143d, SUC_lexicon_tags_47 },
    { 0x3657143e, SUC_lexicon_tags_11 },
    { 0xa6d7143d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xff171441, SUC_lexicon_tags_15 },
    { 0x36171442, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b71448, SUC_lexicon_tags_43 },
    { 0x90b71448, SUC_lexicon_tags_70 },
    { 0x4657144a, SUC_lexicon_tags_17 },
    { 0x83f7144b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa3b7144d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3771451, SUC_lexicon_tags_17 },
    { 0xb5f71452, SUC_lexicon_tags_72 },
    { 0x81971453, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f71456, SUC_lexicon_tags_15 },
    { 0xb9f71457, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7145a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd797145c, SUC_lexicon_tags_43 },
    { 0x2dd7145d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5c17145f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde771461, SUC_lexicon_tags_15 },
    { 0xe0b71462, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3771466, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b7146d, SUC_lexicon_tags_29 },
    { 0x5e97146e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x98f71470, SUC_lexicon_tags_27 },
    { 0xefd71471, SUC_lexicon_tags_1 },
    { 0x5df71472, SUC_lexicon_tags_79 },
    { 0x4c371473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5371475, SUC_lexicon_tags_43 },
    { 0x6f771476, SUC_lexicon_tags_15 },
    { 0x58b71477, SUC_lexicon_tags_12 },
    { 0x1fb71476, SUC_lexicon_tags_7 },
    { 0x25371479, SUC_lexicon_tags_17 },
    { 0x16b7147a, SUC_lexicon_tags_23 },
    { 0xbbb71479, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd297147f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2371481, SUC_lexicon_tags_38 },
    { 0xed571482, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39d71484, SUC_lexicon_tags_12 },
    { 0x3f571485, SUC_lexicon_tags_47 },
    { 0x31b71486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4657148c, SUC_lexicon_tags_25 },
    { 0xdb7148d, SUC_lexicon_tags_37 },
    { 0x46d7148d, SUC_lexicon_tags_37 },
    { 0xe517148e, SUC_lexicon_tags_33 },
    { 0xeed71490, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81771497, SUC_lexicon_tags_17 },
    { 0xbf71498, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9c17149a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7714a2, SUC_lexicon_tags_37 },
    { 0x883714a3, SUC_lexicon_tags_32 },
    { 0xfc1714a3, SUC_lexicon_tags_25 },
    { 0xcff714a5, SUC_lexicon_tags_7 },
    { 0xdf1714a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7714ac, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555714b0, SUC_lexicon_tags_17 },
    { 0xc67714b0, SUC_lexicon_tags_12 },
    { 0xa11714b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b714b5, SUC_lexicon_tags_11 },
    { 0x479714b6, SUC_lexicon_tags_37 },
    { 0x547714b7, SUC_lexicon_tags_31 },
    { 0xc2d714b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb65714ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629714c0, SUC_lexicon_tags_14 },
    { 0x403714c1, SUC_lexicon_tags_37 },
    { 0xe85714c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9714c5, SUC_lexicon_tags_47 },
    { 0x5fd714c6, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb714c9, SUC_lexicon_tags_2 },
    { 0x98f714ca, SUC_lexicon_tags_38 },
    { 0x771714ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x767714cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7d1714cf, SUC_lexicon_tags_45 },
    { 0xddf714d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x155714d3, SUC_lexicon_tags_9 },
    { 0x7a1714d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b714d7, SUC_lexicon_tags_7 },
    { 0xdb9714d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x767714db, SUC_lexicon_tags_63 },
    { 0xd9f714dc, SUC_lexicon_tags_37 },
    { 0x565714dd, SUC_lexicon_tags_12 },
    { 0x6fb714dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9714e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b714e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d714e9, SUC_lexicon_tags_2 },
    { 0xd9b714e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec9714ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89714f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x217714f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9714f7, SUC_lexicon_tags_36 },
    { 0x83714f8, SUC_lexicon_tags_2 },
    { 0xba7714f9, SUC_lexicon_tags_463 },
    { 0x9fd714fa, SUC_lexicon_tags_2 },
    { 0x693714f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d71506, SUC_lexicon_tags_37 },
    { 0xb4571506, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57150b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb557150d, SUC_lexicon_tags_17 },
    { 0x9d17150e, SUC_lexicon_tags_37 },
    { 0x43b7150f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x22371511, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca371514, SUC_lexicon_tags_37 },
    { 0x3ed71515, SUC_lexicon_tags_12 },
    { 0xd5f71514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0371518, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf837151e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a171523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26d71525, SUC_lexicon_tags_0 },
    { 0xda571526, SUC_lexicon_tags_37 },
    { 0x93d71527, SUC_lexicon_tags_8 },
    { 0x39371527, SUC_lexicon_tags_6 },
    { 0xe9f71529, SUC_lexicon_tags_11 },
    { 0x2f971528, SUC_lexicon_tags_47 },
    { 0xc8d71527, SUC_lexicon_tags_15 },
    { 0x8e971528, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90971531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87771535, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x98b71537, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa9171539, SUC_lexicon_tags_38 },
    { 0x7b7153a, SUC_lexicon_tags_35 },
    { 0xedd7153b, SUC_lexicon_tags_0 },
    { 0xaf77153c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9477153e, SUC_lexicon_tags_10 },
    { 0x54d7153e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62171542, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d971546, SUC_lexicon_tags_198 },
    { 0x89771546, SUC_lexicon_tags_39 },
    { 0xa5371546, SUC_lexicon_tags_17 },
    { 0xff171549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedd7154b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda37154d, SUC_lexicon_tags_47 },
    { 0xe977154e, SUC_lexicon_tags_191 },
    { 0x8597154f, SUC_lexicon_tags_19 },
    { 0xeab71550, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2171554, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80d71556, SUC_lexicon_tags_28 },
    { 0xaf71556, SUC_lexicon_tags_0 },
    { 0x6a771558, SUC_lexicon_tags_68 },
    { 0x96f71557, SUC_lexicon_tags_0 },
    { 0xb837155a, SUC_lexicon_tags_17 },
    { 0x6e57155b, SUC_lexicon_tags_19 },
    { 0x8177155a, SUC_lexicon_tags_17 },
    { 0x26d7155d, SUC_lexicon_tags_27 },
    { 0xbab7155a, SUC_lexicon_tags_35 },
    { 0x9097155f, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd71562, SUC_lexicon_tags_12 },
    { 0x5ef71562, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6d771565, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b371567, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49971569, SUC_lexicon_tags_6 },
    { 0xc37156a, SUC_lexicon_tags_25 },
    { 0x47971569, SUC_lexicon_tags_55 },
    { 0xb5971569, SUC_lexicon_tags_44 },
    { 0x4117156d, SUC_lexicon_tags_37 },
    { 0x6757156e, SUC_lexicon_tags_11 },
    { 0x4917156a, SUC_lexicon_tags_26 },
    { 0x32b7156d, SUC_lexicon_tags_0 },
    { 0xc371571, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5571574, SUC_lexicon_tags_0 },
    { 0xd0b71574, SUC_lexicon_tags_45 },
    { 0x37371576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe3d71578, SUC_lexicon_tags_17 },
    { 0x43771578, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4917157b, SUC_lexicon_tags_4 },
    { 0x5d97157c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8257157e, SUC_lexicon_tags_1 },
    { 0x3637157f, SUC_lexicon_tags_14 },
    { 0x7ef7157e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2771587, SUC_lexicon_tags_11 },
    { 0x9f971587, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f7158c, SUC_lexicon_tags_14 },
    { 0x10f7158d, SUC_lexicon_tags_26 },
    { 0xf217158d, SUC_lexicon_tags_11 },
    { 0x8cf7158f, SUC_lexicon_tags_68 },
    { 0x7657158e, SUC_lexicon_tags_37 },
    { 0x38571591, SUC_lexicon_tags_17 },
    { 0xc971592, SUC_lexicon_tags_16 },
    { 0x4a971592, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d71596, SUC_lexicon_tags_3 },
    { 0x9b971597, SUC_lexicon_tags_30 },
    { 0x24d71597, SUC_lexicon_tags_12 },
    { 0xb7371599, SUC_lexicon_tags_2 },
    { 0x2597159a, SUC_lexicon_tags_30 },
    { 0x2771597, SUC_lexicon_tags_47 },
    { 0xeff7159a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7fd7159e, SUC_lexicon_tags_14 },
    { 0x8997159e, SUC_lexicon_tags_1 },
    { 0xb3f715a0, SUC_lexicon_tags_25 },
    { 0xb37715a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607715a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe79715aa, SUC_lexicon_tags_38 },
    { 0x1a5715aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf715ae, SUC_lexicon_tags_21 },
    { 0x511715af, SUC_lexicon_tags_77 },
    { 0xc83715af, SUC_lexicon_tags_65 },
    { 0x86f715af, SUC_lexicon_tags_60 },
    { 0x263715b2, SUC_lexicon_tags_56 },
    { 0xcb5715af, SUC_lexicon_tags_1 },
    { 0x7b9715b4, SUC_lexicon_tags_25 },
    { 0x799715b3, SUC_lexicon_tags_15 },
    { 0x3b9715b5, SUC_lexicon_tags_23 },
    { 0x911715b7, SUC_lexicon_tags_38 },
    { 0x90b715b8, SUC_lexicon_tags_2 },
    { 0x4b3715b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x71f715bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa4f715bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x409715bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b715c3, SUC_lexicon_tags_16 },
    { 0x37d715c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1715cb, SUC_lexicon_tags_2 },
    { 0x4b7715cc, SUC_lexicon_tags_2 },
    { 0x2ad715cd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed715d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f715d6, SUC_lexicon_tags_2 },
    { 0xa0b715d7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3ab715d9, SUC_lexicon_tags_37 },
    { 0x8db715da, SUC_lexicon_tags_63 },
    { 0xf2f715da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x947715de, SUC_lexicon_tags_11 },
    { 0xf29715de, SUC_lexicon_tags_7 },
    { 0x789715e0, SUC_lexicon_tags_2 },
    { 0x635715e0, SUC_lexicon_tags_79 },
    { 0xa81715e0, SUC_lexicon_tags_33 },
    { 0x779715e1, SUC_lexicon_tags_11 },
    { 0x1ed715e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817715e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3715eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82b715ed, SUC_lexicon_tags_19 },
    { 0x177715ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf715f6, SUC_lexicon_tags_108 },
    { 0x189715f7, SUC_lexicon_tags_7 },
    { 0x2ab715f6, SUC_lexicon_tags_38 },
    { 0xfed715f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1715fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c371608, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc717160a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7837160c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd71613, SUC_lexicon_tags_49 },
    { 0x6e371614, SUC_lexicon_tags_37 },
    { 0x45171613, SUC_lexicon_tags_16 },
    { 0x75b71616, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17161d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b7162a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28f7162c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb937162e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d71632, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8d171634, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x57571636, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e371638, SUC_lexicon_tags_63 },
    { 0x27171639, SUC_lexicon_tags_96 },
    { 0x86571639, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17163d, SUC_lexicon_tags_37 },
    { 0x9ab7163e, SUC_lexicon_tags_1 },
    { 0x8c97163f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd71642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f71645, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7571648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8197164a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8971650, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf71653, SUC_lexicon_tags_8 },
    { 0xeb971653, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc8f71656, SUC_lexicon_tags_23 },
    { 0x6fd71657, SUC_lexicon_tags_7 },
    { 0x14f71658, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x29d7165a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd377165c, SUC_lexicon_tags_37 },
    { 0x5b57165d, SUC_lexicon_tags_31 },
    { 0xf8f7165c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b71661, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f71664, SUC_lexicon_tags_11 },
    { 0x70d71665, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47b71667, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7166a, SUC_lexicon_tags_38 },
    { 0x4957166a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3237166f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xef71671, SUC_lexicon_tags_12 },
    { 0xa571672, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f71679, SUC_lexicon_tags_17 },
    { 0xc9f7167a, SUC_lexicon_tags_16 },
    { 0x6d37167b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8957167f, SUC_lexicon_tags_8 },
    { 0xeef7167f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f71684, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec771689, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec37168b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2477168d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf571690, SUC_lexicon_tags_29 },
    { 0x1c571690, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39571693, SUC_lexicon_tags_0 },
    { 0x2571693, SUC_lexicon_tags_38 },
    { 0x43971695, SUC_lexicon_tags_15 },
    { 0x97171696, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf9d71698, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ff7169a, SUC_lexicon_tags_170 },
    { 0x6ef7169b, SUC_lexicon_tags_6 },
    { 0x9a37169c, SUC_lexicon_tags_94 },
    { 0xcf17169d, SUC_lexicon_tags_65 },
    { 0x2497169d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5716a1, SUC_lexicon_tags_26 },
    { 0x6b5716a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7716a5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x759716a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b716aa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc9f716ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b716b0, SUC_lexicon_tags_39 },
    { 0x355716b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1716b6, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f716bc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf2f716be, SUC_lexicon_tags_17 },
    { 0xa7716bf, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xd17716c1, SUC_lexicon_tags_23 },
    { 0x64f716c1, SUC_lexicon_tags_33 },
    { 0xd8d716c3, SUC_lexicon_tags_48 },
    { 0x1d716c4, SUC_lexicon_tags_39 },
    { 0xd59716c1, SUC_lexicon_tags_15 },
    { 0x321716c6, SUC_lexicon_tags_33 },
    { 0x561716c7, SUC_lexicon_tags_7 },
    { 0x365716c7, SUC_lexicon_tags_14 },
    { 0xb83716c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7716cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad716d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef716d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117716d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01716dd, SUC_lexicon_tags_26 },
    { 0x5d5716de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923716e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03716ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd69716ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff716f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5716f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3716f7, SUC_lexicon_tags_54 },
    { 0x72b716f7, SUC_lexicon_tags_39 },
    { 0xa01716f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf716fe, SUC_lexicon_tags_11 },
    { 0x4d1716ff, SUC_lexicon_tags_2 },
    { 0x7ab71700, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb571707, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd7170b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf71712, SUC_lexicon_tags_10 },
    { 0xf8d71713, SUC_lexicon_tags_25 },
    { 0xee171714, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a371718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeed7171a, SUC_lexicon_tags_11 },
    { 0x9537171b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa37171d, SUC_lexicon_tags_7 },
    { 0x75f7171e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e171722, SUC_lexicon_tags_17 },
    { 0x9f171722, SUC_lexicon_tags_28 },
    { 0x82971723, SUC_lexicon_tags_37 },
    { 0xd1971725, SUC_lexicon_tags_37 },
    { 0x32b71726, SUC_lexicon_tags_26 },
    { 0x9b171726, SUC_lexicon_tags_10 },
    { 0xc5971725, SUC_lexicon_tags_27 },
    { 0xe0f71724, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8557172b, SUC_lexicon_tags_8 },
    { 0x9677172c, SUC_lexicon_tags_63 },
    { 0x8977172c, SUC_lexicon_tags_35 },
    { 0xefb7172d, SUC_lexicon_tags_37 },
    { 0x6897172d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f71732, SUC_lexicon_tags_25 },
    { 0xfc771733, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcc571735, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2417173b, SUC_lexicon_tags_4 },
    { 0x6077173c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d71742, SUC_lexicon_tags_0 },
    { 0x78f71742, SUC_lexicon_tags_37 },
    { 0x9df71744, SUC_lexicon_tags_11 },
    { 0x29171744, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d7174b, SUC_lexicon_tags_15 },
    { 0x20f7174c, SUC_lexicon_tags_7 },
    { 0xdf97174d, SUC_lexicon_tags_2 },
    { 0x557174e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x371750, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a771755, SUC_lexicon_tags_29 },
    { 0xcb371756, SUC_lexicon_tags_29 },
    { 0xd0171757, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7597175e, SUC_lexicon_tags_31 },
    { 0x9417175e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7771762, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x23571764, SUC_lexicon_tags_38 },
    { 0xc5971765, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x56971767, SUC_lexicon_tags_572 },
    { 0xb5171768, SUC_lexicon_tags_25 },
    { 0xcff71769, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f7176e, SUC_lexicon_tags_7 },
    { 0x1a37176f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62971774, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd71777, SUC_lexicon_tags_15 },
    { 0x26571778, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb7177b, SUC_lexicon_tags_37 },
    { 0x9a37177b, SUC_lexicon_tags_12 },
    { 0xc517177d, SUC_lexicon_tags_25 },
    { 0x5797177d, SUC_lexicon_tags_11 },
    { 0x37b7177f, SUC_lexicon_tags_11 },
    { 0x7a57177d, SUC_lexicon_tags_15 },
    { 0x85b71781, SUC_lexicon_tags_92 },
    { 0xd5b7177d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b71789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb037178c, SUC_lexicon_tags_2 },
    { 0xa8b7178d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5937178f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27971796, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f7179d, SUC_lexicon_tags_25 },
    { 0xfb17179e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9717a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1717a3, SUC_lexicon_tags_29 },
    { 0xd75717a3, SUC_lexicon_tags_39 },
    { 0xac7717a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91717a9, SUC_lexicon_tags_61 },
    { 0x395717a9, SUC_lexicon_tags_47 },
    { 0xeef717ab, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed717af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e3717b1, SUC_lexicon_tags_1 },
    { 0x613717b2, SUC_lexicon_tags_21 },
    { 0xad7717b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31717bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed717bf, SUC_lexicon_tags_31 },
    { 0x5b717c0, SUC_lexicon_tags_9 },
    { 0x813717c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x801717c3, SUC_lexicon_tags_56 },
    { 0xa19717c4, SUC_lexicon_tags_12 },
    { 0x92f717c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7717ca, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b717ce, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf83717d0, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x29717d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3717d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x359717d7, SUC_lexicon_tags_33 },
    { 0x621717d7, SUC_lexicon_tags_43 },
    { 0xd89717d9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab717dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd717e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83717ed, SUC_lexicon_tags_17 },
    { 0xfff717ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23717f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d717f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b71801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f71809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6d7180b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd897180d, SUC_lexicon_tags_198 },
    { 0x33f7180e, SUC_lexicon_tags_63 },
    { 0x2e77180f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20371814, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x76371817, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9977181b, SUC_lexicon_tags_1 },
    { 0x9617181c, SUC_lexicon_tags_60 },
    { 0x54d7181d, SUC_lexicon_tags_11 },
    { 0x9177181e, SUC_lexicon_tags_37 },
    { 0x4737181f, SUC_lexicon_tags_28 },
    { 0xd5571820, SUC_lexicon_tags_22 },
    { 0xd4571820, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad71823, SUC_lexicon_tags_29 },
    { 0x18f71824, SUC_lexicon_tags_17 },
    { 0xe4371825, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e57182a, SUC_lexicon_tags_25 },
    { 0xc8f7182b, SUC_lexicon_tags_45 },
    { 0xe017182c, SUC_lexicon_tags_17 },
    { 0xc5f7182d, SUC_lexicon_tags_43 },
    { 0xfb37182e, SUC_lexicon_tags_5 },
    { 0xa397182f, SUC_lexicon_tags_2 },
    { 0x457182e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77171833, SUC_lexicon_tags_19 },
    { 0xa7371834, SUC_lexicon_tags_37 },
    { 0x1dd71835, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8c171837, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff7183c, SUC_lexicon_tags_19 },
    { 0xd977183d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x877183f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8cd71841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c971845, SUC_lexicon_tags_60 },
    { 0xd3d71846, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd7184c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bd7184e, SUC_lexicon_tags_45 },
    { 0x60b7184f, SUC_lexicon_tags_14 },
    { 0xf8b7184e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab71853, SUC_lexicon_tags_17 },
    { 0xdb71854, SUC_lexicon_tags_204 },
    { 0x63f71853, SUC_lexicon_tags_7 },
    { 0x76d71856, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa371858, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2357185a, SUC_lexicon_tags_7 },
    { 0x2857185b, SUC_lexicon_tags_38 },
    { 0x2397185a, SUC_lexicon_tags_23 },
    { 0x8f7185d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd371860, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x73b71862, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9571866, SUC_lexicon_tags_7 },
    { 0xf3371867, SUC_lexicon_tags_27 },
    { 0x3ff71868, SUC_lexicon_tags_307 },
    { 0, NULL },
    { 0, NULL },
    { 0xa777186b, SUC_lexicon_tags_6 },
    { 0xd9b7186c, SUC_lexicon_tags_19 },
    { 0xc157186b, SUC_lexicon_tags_0 },
    { 0xa8d7186c, SUC_lexicon_tags_34 },
    { 0x6eb7186c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x58771871, SUC_lexicon_tags_1 },
    { 0xb1171871, SUC_lexicon_tags_44 },
    { 0xc8f71873, SUC_lexicon_tags_66 },
    { 0xeeb71873, SUC_lexicon_tags_25 },
    { 0xf4171874, SUC_lexicon_tags_23 },
    { 0x1d771876, SUC_lexicon_tags_25 },
    { 0x23b71877, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x11171879, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x4c97187b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb97187d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83571881, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee771886, SUC_lexicon_tags_33 },
    { 0xa5d71887, SUC_lexicon_tags_25 },
    { 0xc0b71888, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe3f7188a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7017188d, SUC_lexicon_tags_9 },
    { 0x2737188e, SUC_lexicon_tags_7 },
    { 0x6c17188f, SUC_lexicon_tags_33 },
    { 0xef37188e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b71892, SUC_lexicon_tags_61 },
    { 0x77171893, SUC_lexicon_tags_23 },
    { 0x771894, SUC_lexicon_tags_21 },
    { 0xd2771895, SUC_lexicon_tags_36 },
    { 0x50571896, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad7189c, SUC_lexicon_tags_2 },
    { 0x7177189c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x573718a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf1718a2, SUC_lexicon_tags_15 },
    { 0x7a1718a3, SUC_lexicon_tags_17 },
    { 0xd4f718a4, SUC_lexicon_tags_12 },
    { 0xe1718a2, SUC_lexicon_tags_7 },
    { 0x8bd718a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55718aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31718ad, SUC_lexicon_tags_3 },
    { 0xd57718ae, SUC_lexicon_tags_16 },
    { 0x645718ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7718b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd6d718b8, SUC_lexicon_tags_37 },
    { 0xcbf718b8, SUC_lexicon_tags_43 },
    { 0x32d718b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x979718bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf99718c0, SUC_lexicon_tags_1 },
    { 0xe63718c1, SUC_lexicon_tags_27 },
    { 0x583718c2, SUC_lexicon_tags_194 },
    { 0x1dd718c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd718c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b718ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x449718cd, SUC_lexicon_tags_30 },
    { 0x31718ce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5718d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1718d6, SUC_lexicon_tags_34 },
    { 0x3ad718d7, SUC_lexicon_tags_11 },
    { 0xc4f718d8, SUC_lexicon_tags_9 },
    { 0xd09718d9, SUC_lexicon_tags_45 },
    { 0x5e1718d9, SUC_lexicon_tags_12 },
    { 0xcb5718d8, SUC_lexicon_tags_45 },
    { 0x545718d8, SUC_lexicon_tags_33 },
    { 0xc47718da, SUC_lexicon_tags_6 },
    { 0xb6b718d8, SUC_lexicon_tags_36 },
    { 0x41b718dd, SUC_lexicon_tags_6 },
    { 0x693718e0, SUC_lexicon_tags_27 },
    { 0x299718e1, SUC_lexicon_tags_6 },
    { 0xa85718de, SUC_lexicon_tags_47 },
    { 0xb5f718e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f718ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x939718ef, SUC_lexicon_tags_19 },
    { 0xc9718f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40d718f2, SUC_lexicon_tags_38 },
    { 0xa31718f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x705718f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x885718f7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71718fb, SUC_lexicon_tags_1 },
    { 0x7b718fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f71900, SUC_lexicon_tags_14 },
    { 0x38571901, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc171907, SUC_lexicon_tags_7 },
    { 0xe1d71908, SUC_lexicon_tags_43 },
    { 0x79571908, SUC_lexicon_tags_45 },
    { 0x96771909, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8971913, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd71918, SUC_lexicon_tags_11 },
    { 0xc4371918, SUC_lexicon_tags_47 },
    { 0x58971918, SUC_lexicon_tags_15 },
    { 0xb6f7191b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd7191f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b71928, SUC_lexicon_tags_13 },
    { 0xa2171929, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb577192d, SUC_lexicon_tags_47 },
    { 0x1a37192d, SUC_lexicon_tags_14 },
    { 0x3377192f, SUC_lexicon_tags_1 },
    { 0xc997192f, SUC_lexicon_tags_17 },
    { 0x3757192e, SUC_lexicon_tags_11 },
    { 0xcdf7192e, SUC_lexicon_tags_43 },
    { 0x1b571933, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6357193b, SUC_lexicon_tags_1 },
    { 0x8297193c, SUC_lexicon_tags_0 },
    { 0x8537193b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed971940, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1771943, SUC_lexicon_tags_2 },
    { 0x8c771943, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d77195a, SUC_lexicon_tags_37 },
    { 0x5bb7195b, SUC_lexicon_tags_11 },
    { 0x7937195c, SUC_lexicon_tags_28 },
    { 0xa9d7195c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d71962, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb71967, SUC_lexicon_tags_2 },
    { 0x3bd71968, SUC_lexicon_tags_37 },
    { 0x40b71969, SUC_lexicon_tags_7 },
    { 0x2077196a, SUC_lexicon_tags_47 },
    { 0x371968, SUC_lexicon_tags_66 },
    { 0xb1d7196a, SUC_lexicon_tags_25 },
    { 0xbb77196d, SUC_lexicon_tags_19 },
    { 0x6f17196d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4df71970, SUC_lexicon_tags_11 },
    { 0x78371970, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53971973, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d71976, SUC_lexicon_tags_15 },
    { 0x7c571976, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa597197e, SUC_lexicon_tags_37 },
    { 0x5eb7197f, SUC_lexicon_tags_43 },
    { 0xc2571980, SUC_lexicon_tags_17 },
    { 0x76b71980, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff71987, SUC_lexicon_tags_8 },
    { 0x44771988, SUC_lexicon_tags_63 },
    { 0x26771989, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcfd7198b, SUC_lexicon_tags_37 },
    { 0xcc77198c, SUC_lexicon_tags_15 },
    { 0x2e57198d, SUC_lexicon_tags_25 },
    { 0xd337198e, SUC_lexicon_tags_68 },
    { 0x73f7198e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdaf71991, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabf71993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb7199a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb157199f, SUC_lexicon_tags_17 },
    { 0x2a1719a0, SUC_lexicon_tags_37 },
    { 0xd11719a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab719a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7d719a7, SUC_lexicon_tags_45 },
    { 0x9d3719a8, SUC_lexicon_tags_8 },
    { 0x73f719a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09719ac, SUC_lexicon_tags_17 },
    { 0x4a1719ad, SUC_lexicon_tags_30 },
    { 0xde1719ae, SUC_lexicon_tags_19 },
    { 0xdf1719af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8bb719b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf719b7, SUC_lexicon_tags_49 },
    { 0x64b719b7, SUC_lexicon_tags_65 },
    { 0x7f5719b8, SUC_lexicon_tags_30 },
    { 0x223719ba, SUC_lexicon_tags_2 },
    { 0x20f719bb, SUC_lexicon_tags_2 },
    { 0xf47719bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16d719be, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93719c2, SUC_lexicon_tags_25 },
    { 0xc63719c3, SUC_lexicon_tags_11 },
    { 0x633719c4, SUC_lexicon_tags_45 },
    { 0x29719c5, SUC_lexicon_tags_28 },
    { 0xc29719c5, SUC_lexicon_tags_17 },
    { 0x743719c5, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575719cd, SUC_lexicon_tags_7 },
    { 0x63719ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb719d9, SUC_lexicon_tags_2 },
    { 0xc09719d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537719df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d719e2, SUC_lexicon_tags_48 },
    { 0x777719e3, SUC_lexicon_tags_34 },
    { 0x113719e2, SUC_lexicon_tags_7 },
    { 0xbcf719e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b719ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73719f1, SUC_lexicon_tags_10 },
    { 0xc69719f2, SUC_lexicon_tags_30 },
    { 0x913719f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b719f6, SUC_lexicon_tags_37 },
    { 0x195719f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e5719f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde3719fb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe21719fd, SUC_lexicon_tags_63 },
    { 0x1b7719fe, SUC_lexicon_tags_69 },
    { 0x4a3719fe, SUC_lexicon_tags_14 },
    { 0x70f71a00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd971a02, SUC_lexicon_tags_2 },
    { 0xd8971a03, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb971a06, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f971a0b, SUC_lexicon_tags_9 },
    { 0xdfd71a0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf71a10, SUC_lexicon_tags_14 },
    { 0x78571a10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f71a19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7cd71a1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2a571a1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1571a24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa171a27, SUC_lexicon_tags_91 },
    { 0x15771a28, SUC_lexicon_tags_2 },
    { 0x46771a29, SUC_lexicon_tags_42 },
    { 0x66571a27, SUC_lexicon_tags_7 },
    { 0x17771a2b, SUC_lexicon_tags_11 },
    { 0x99771a29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbf71a2e, SUC_lexicon_tags_12 },
    { 0xca171a2f, SUC_lexicon_tags_17 },
    { 0xf5b71a30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b371a35, SUC_lexicon_tags_14 },
    { 0xf8571a36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea971a39, SUC_lexicon_tags_30 },
    { 0x8ad71a3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b71a3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb71a42, SUC_lexicon_tags_14 },
    { 0xe5571a43, SUC_lexicon_tags_38 },
    { 0x60d71a43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f71a47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf71a4b, SUC_lexicon_tags_87 },
    { 0xc2571a4c, SUC_lexicon_tags_0 },
    { 0xbd571a4b, SUC_lexicon_tags_27 },
    { 0x6cb71a4e, SUC_lexicon_tags_17 },
    { 0x12371a4b, SUC_lexicon_tags_2 },
    { 0xfc971a50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16571a54, SUC_lexicon_tags_27 },
    { 0xb771a55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a571a5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d71a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc971a65, SUC_lexicon_tags_27 },
    { 0x9b171a66, SUC_lexicon_tags_12 },
    { 0xf4b71a67, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d71a6e, SUC_lexicon_tags_46 },
    { 0x67d71a6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d71a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe171a79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x68771a7c, SUC_lexicon_tags_68 },
    { 0x73571a7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd971a84, SUC_lexicon_tags_1 },
    { 0xcfd71a84, SUC_lexicon_tags_0 },
    { 0x29771a84, SUC_lexicon_tags_2 },
    { 0x5c171a87, SUC_lexicon_tags_17 },
    { 0xc7371a86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b71a8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf571a95, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b571a98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b771a9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c971a9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d971aa3, SUC_lexicon_tags_15 },
    { 0x71171aa4, SUC_lexicon_tags_11 },
    { 0xfc771aa5, SUC_lexicon_tags_2 },
    { 0x39571aa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa971aa8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1571aab, SUC_lexicon_tags_12 },
    { 0x6a971aab, SUC_lexicon_tags_2 },
    { 0x20171aad, SUC_lexicon_tags_17 },
    { 0x79971aac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3171ab5, SUC_lexicon_tags_17 },
    { 0x27b71ab6, SUC_lexicon_tags_27 },
    { 0x8c971ab7, SUC_lexicon_tags_2 },
    { 0xe8f71ab8, SUC_lexicon_tags_47 },
    { 0xfbb71ab5, SUC_lexicon_tags_38 },
    { 0x39771ab5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa9571abc, SUC_lexicon_tags_63 },
    { 0x5af71abc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63d71abf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb71ac4, SUC_lexicon_tags_37 },
    { 0xa7171ac5, SUC_lexicon_tags_7 },
    { 0x95b71ac6, SUC_lexicon_tags_7 },
    { 0x6ef71ac6, SUC_lexicon_tags_61 },
    { 0x19971ac8, SUC_lexicon_tags_11 },
    { 0xf171ac9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde971acc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x28f71ace, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44371ad3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x43771ad6, SUC_lexicon_tags_1 },
    { 0xb0971ad6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5371adb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x17371ade, SUC_lexicon_tags_17 },
    { 0x56971ade, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d71ae1, SUC_lexicon_tags_25 },
    { 0xab171ae2, SUC_lexicon_tags_2 },
    { 0xecd71ae2, SUC_lexicon_tags_47 },
    { 0x20d71ae4, SUC_lexicon_tags_33 },
    { 0xc2f71ae5, SUC_lexicon_tags_11 },
    { 0x7fb71ae6, SUC_lexicon_tags_8 },
    { 0x17971ae4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd71aed, SUC_lexicon_tags_0 },
    { 0xccd71aee, SUC_lexicon_tags_38 },
    { 0x81771aef, SUC_lexicon_tags_60 },
    { 0x55d71af0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad71af4, SUC_lexicon_tags_31 },
    { 0xe7971af5, SUC_lexicon_tags_47 },
    { 0x22171af4, SUC_lexicon_tags_2 },
    { 0xe771af7, SUC_lexicon_tags_37 },
    { 0x33d71af7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28971afc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91171afe, SUC_lexicon_tags_2 },
    { 0x40171aff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdaf71b01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc971b03, SUC_lexicon_tags_48 },
    { 0xecd71b03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8171b06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26371b08, SUC_lexicon_tags_70 },
    { 0x6f371b09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4771b0b, SUC_lexicon_tags_60 },
    { 0xe2171b0c, SUC_lexicon_tags_55 },
    { 0x53571b0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb71b10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6171b13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40371b17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc5b71b19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f771b1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e771b20, SUC_lexicon_tags_19 },
    { 0xc3771b21, SUC_lexicon_tags_69 },
    { 0x67971b21, SUC_lexicon_tags_25 },
    { 0xb3571b23, SUC_lexicon_tags_23 },
    { 0xda371b23, SUC_lexicon_tags_37 },
    { 0xc0f71b25, SUC_lexicon_tags_47 },
    { 0xfdb71b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49771b29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6971b2b, SUC_lexicon_tags_65 },
    { 0xbc171b2b, SUC_lexicon_tags_27 },
    { 0x8c971b2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f71b31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74171b34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b71b39, SUC_lexicon_tags_12 },
    { 0x18d71b3a, SUC_lexicon_tags_30 },
    { 0xedd71b39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b71b3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d71b42, SUC_lexicon_tags_32 },
    { 0xc8371b42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd71b46, SUC_lexicon_tags_2 },
    { 0x1eb71b47, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1af71b49, SUC_lexicon_tags_22 },
    { 0x77771b49, SUC_lexicon_tags_60 },
    { 0xcf71b49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc571b4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x82b71b50, SUC_lexicon_tags_9 },
    { 0x34371b51, SUC_lexicon_tags_29 },
    { 0x3f71b50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8bd71b54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf5371b56, SUC_lexicon_tags_17 },
    { 0x3eb71b57, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf8f71b59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f71b60, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c371b63, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4f71b65, SUC_lexicon_tags_31 },
    { 0xdb971b65, SUC_lexicon_tags_11 },
    { 0x85f71b66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89571b6f, SUC_lexicon_tags_12 },
    { 0x83b71b6f, SUC_lexicon_tags_37 },
    { 0x4b171b71, SUC_lexicon_tags_1 },
    { 0x90b71b6f, SUC_lexicon_tags_25 },
    { 0x19371b73, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4371b76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1571b78, SUC_lexicon_tags_29 },
    { 0xfb571b78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d71b7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6571b81, SUC_lexicon_tags_40 },
    { 0x92771b81, SUC_lexicon_tags_7 },
    { 0x3a371b83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad71b88, SUC_lexicon_tags_7 },
    { 0xc4371b88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d171b94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe771b96, SUC_lexicon_tags_37 },
    { 0x13171b96, SUC_lexicon_tags_17 },
    { 0x4cb71b98, SUC_lexicon_tags_28 },
    { 0xb4b71b99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cd71b9b, SUC_lexicon_tags_25 },
    { 0xb5171b9b, SUC_lexicon_tags_29 },
    { 0x28771b9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ef71b9f, SUC_lexicon_tags_15 },
    { 0xa6b71b9f, SUC_lexicon_tags_7 },
    { 0xf1b71ba1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xceb71ba3, SUC_lexicon_tags_10 },
    { 0xfa771ba4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2571bad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a371bb0, SUC_lexicon_tags_6 },
    { 0xbe771bb1, SUC_lexicon_tags_66 },
    { 0x1a571bb2, SUC_lexicon_tags_11 },
    { 0x35d71bb3, SUC_lexicon_tags_15 },
    { 0x1cd71bb4, SUC_lexicon_tags_28 },
    { 0xf7d71bb5, SUC_lexicon_tags_23 },
    { 0xa6971bb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde971bbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd71bc3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xee971bc5, SUC_lexicon_tags_8 },
    { 0x68f71bc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fb71bc8, SUC_lexicon_tags_12 },
    { 0x2dd71bc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4971bcc, SUC_lexicon_tags_15 },
    { 0xcbf71bcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51571bcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66f71bd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23171bd6, SUC_lexicon_tags_11 },
    { 0x6f371bd7, SUC_lexicon_tags_30 },
    { 0xc5771bd7, SUC_lexicon_tags_37 },
    { 0x3d571bd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3f71bdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e371bdd, SUC_lexicon_tags_60 },
    { 0x9d171bdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f71be1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95771be4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d71be7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd571bec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5eb71bee, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd7771bf0, SUC_lexicon_tags_25 },
    { 0xe0d71bf1, SUC_lexicon_tags_1 },
    { 0x87371bf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b171bf5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc971bf7, SUC_lexicon_tags_28 },
    { 0x53771bf7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x60971bfa, SUC_lexicon_tags_43 },
    { 0xdf571bfb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x92f71bfd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1371c02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb71c05, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb9771c07, SUC_lexicon_tags_26 },
    { 0xba171c08, SUC_lexicon_tags_79 },
    { 0x33b71c09, SUC_lexicon_tags_37 },
    { 0xc2371c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40371c0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db71c12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55571c16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde771c1a, SUC_lexicon_tags_21 },
    { 0x1db71c1b, SUC_lexicon_tags_33 },
    { 0x29771c1b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf71c1f, SUC_lexicon_tags_33 },
    { 0x54571c20, SUC_lexicon_tags_27 },
    { 0xef71c1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9571c25, SUC_lexicon_tags_17 },
    { 0xeeb71c26, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b71c2c, SUC_lexicon_tags_11 },
    { 0x96171c2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdb971c2f, SUC_lexicon_tags_2 },
    { 0xd2f71c2f, SUC_lexicon_tags_34 },
    { 0xb4571c30, SUC_lexicon_tags_23 },
    { 0x4dd71c32, SUC_lexicon_tags_5 },
    { 0x10371c33, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8171c37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad71c3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4171c42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d71c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6371c4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11971c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98d71c54, SUC_lexicon_tags_17 },
    { 0x8b371c54, SUC_lexicon_tags_37 },
    { 0x7fb71c56, SUC_lexicon_tags_59 },
    { 0x45f71c57, SUC_lexicon_tags_0 },
    { 0xee971c54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d71c5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71971c5f, SUC_lexicon_tags_6 },
    { 0xd0571c5f, SUC_lexicon_tags_19 },
    { 0x62171c5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea171c66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a971c69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2371c6c, SUC_lexicon_tags_8 },
    { 0x4d171c6d, SUC_lexicon_tags_0 },
    { 0x8f371c6c, SUC_lexicon_tags_68 },
    { 0xcd71c6d, SUC_lexicon_tags_40 },
    { 0x62171c70, SUC_lexicon_tags_10 },
    { 0x1c571c71, SUC_lexicon_tags_17 },
    { 0xdbd71c70, SUC_lexicon_tags_33 },
    { 0x62371c6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f71c7c, SUC_lexicon_tags_30 },
    { 0xc5771c7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b71c80, SUC_lexicon_tags_32 },
    { 0x71571c81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26571c84, SUC_lexicon_tags_11 },
    { 0x24f71c85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b71c8d, SUC_lexicon_tags_60 },
    { 0xefb71c8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e971c91, SUC_lexicon_tags_14 },
    { 0xad71c92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48971c94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f71c9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f71c9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d71ca2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c571ca7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb371ca9, SUC_lexicon_tags_17 },
    { 0xcbf71caa, SUC_lexicon_tags_68 },
    { 0xb2371cab, SUC_lexicon_tags_19 },
    { 0x27f71caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ef71cae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f971cb3, SUC_lexicon_tags_1 },
    { 0x87b71cb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd571cb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab971cb8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1eb71cba, SUC_lexicon_tags_43 },
    { 0x91971cbb, SUC_lexicon_tags_60 },
    { 0x15371cbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec971cbe, SUC_lexicon_tags_11 },
    { 0xc7b71cbe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf971cc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b71cc8, SUC_lexicon_tags_7 },
    { 0x56971cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f71ccc, SUC_lexicon_tags_84 },
    { 0x34171ccd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69771ccf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6771cd3, SUC_lexicon_tags_85 },
    { 0x6ff71cd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94171cd6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb4f71cd8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51171cda, SUC_lexicon_tags_1 },
    { 0xa6f71cda, SUC_lexicon_tags_11 },
    { 0xb2971cdc, SUC_lexicon_tags_7 },
    { 0x7bf71cdd, SUC_lexicon_tags_37 },
    { 0x4e571cde, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x61971ce0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9171ce6, SUC_lexicon_tags_60 },
    { 0x8f171ce7, SUC_lexicon_tags_43 },
    { 0xc4371ce8, SUC_lexicon_tags_25 },
    { 0xdb571ce8, SUC_lexicon_tags_27 },
    { 0x9f771ce9, SUC_lexicon_tags_97 },
    { 0x8fb71ceb, SUC_lexicon_tags_25 },
    { 0x86d71cec, SUC_lexicon_tags_49 },
    { 0x43b71ce7, SUC_lexicon_tags_43 },
    { 0x68d71cee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e171cf0, SUC_lexicon_tags_7 },
    { 0xedb71cf1, SUC_lexicon_tags_25 },
    { 0x55b71cf2, SUC_lexicon_tags_15 },
    { 0xc7571cf2, SUC_lexicon_tags_23 },
    { 0x9b971cf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf771cf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58171cfd, SUC_lexicon_tags_37 },
    { 0x47571cfd, SUC_lexicon_tags_11 },
    { 0x79f71cff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x49371d02, SUC_lexicon_tags_15 },
    { 0xa6371d02, SUC_lexicon_tags_11 },
    { 0xb7d71d04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4b71d06, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57371d0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb371d12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff771d1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x90171d1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff71d1f, SUC_lexicon_tags_17 },
    { 0xaa971d20, SUC_lexicon_tags_10 },
    { 0x11371d21, SUC_lexicon_tags_15 },
    { 0x6b71d22, SUC_lexicon_tags_2 },
    { 0x88b71d22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d71d27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23171d2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f71d2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b71d30, SUC_lexicon_tags_11 },
    { 0xf0171d31, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcef71d33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d71d38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8771d3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23171d41, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5771d44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbb71d46, SUC_lexicon_tags_66 },
    { 0xa8b71d47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac171d4e, SUC_lexicon_tags_8 },
    { 0xd7b71d4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b571d57, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xa1971d59, SUC_lexicon_tags_17 },
    { 0xba771d5a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf5371d5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0771d64, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b71d67, SUC_lexicon_tags_17 },
    { 0x2db71d67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a371d6d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f71d72, SUC_lexicon_tags_43 },
    { 0xcb371d72, SUC_lexicon_tags_25 },
    { 0xdf171d74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf71d79, SUC_lexicon_tags_3 },
    { 0x99171d7a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x94171d7c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62371d80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f71d83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x27571d86, SUC_lexicon_tags_31 },
    { 0x34171d87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75171d8c, SUC_lexicon_tags_47 },
    { 0x3cf71d8d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6171d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60971d93, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84371d97, SUC_lexicon_tags_47 },
    { 0x43d71d98, SUC_lexicon_tags_87 },
    { 0xba171d99, SUC_lexicon_tags_35 },
    { 0x80571d9a, SUC_lexicon_tags_31 },
    { 0x38f71d9a, SUC_lexicon_tags_87 },
    { 0xd5f71d9c, SUC_lexicon_tags_48 },
    { 0xb2771d9b, SUC_lexicon_tags_25 },
    { 0x78571d9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38571da2, SUC_lexicon_tags_12 },
    { 0xc0171da2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2771dac, SUC_lexicon_tags_36 },
    { 0xb8771dac, SUC_lexicon_tags_1 },
    { 0xbe971dae, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f71db1, SUC_lexicon_tags_47 },
    { 0xc8571db2, SUC_lexicon_tags_60 },
    { 0x96771db2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1571dbc, SUC_lexicon_tags_2 },
    { 0xc1b71dbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22771dc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb571dc6, SUC_lexicon_tags_1 },
    { 0x97371dc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b771dcb, SUC_lexicon_tags_33 },
    { 0x3ed71dcc, SUC_lexicon_tags_11 },
    { 0x82171dcd, SUC_lexicon_tags_7 },
    { 0x68971dce, SUC_lexicon_tags_14 },
    { 0x96971dcd, SUC_lexicon_tags_151 },
    { 0xf0f71dcc, SUC_lexicon_tags_27 },
    { 0xe8371dd1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba371dd3, SUC_lexicon_tags_1 },
    { 0x96371dd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f71dd8, SUC_lexicon_tags_25 },
    { 0x40771dd9, SUC_lexicon_tags_25 },
    { 0x47971dd9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8f171ddc, SUC_lexicon_tags_29 },
    { 0x5b171ddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21171ddf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb71de3, SUC_lexicon_tags_28 },
    { 0x9f771de4, SUC_lexicon_tags_30 },
    { 0x67771de4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed71deb, SUC_lexicon_tags_25 },
    { 0x62b71dec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x76971dee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f71df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f71df8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f371dfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80f71dff, SUC_lexicon_tags_37 },
    { 0x72771e00, SUC_lexicon_tags_1 },
    { 0x6f571e01, SUC_lexicon_tags_9 },
    { 0xcc171e00, SUC_lexicon_tags_16 },
    { 0x5571e00, SUC_lexicon_tags_28 },
    { 0xaeb71e03, SUC_lexicon_tags_79 },
    { 0x5b771e01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1771e07, SUC_lexicon_tags_2 },
    { 0xd571e07, SUC_lexicon_tags_56 },
    { 0xc5971e09, SUC_lexicon_tags_11 },
    { 0x12d71e0a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf0371e0c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48971e13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed371e19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7af71e1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e171e22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb71e26, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab71e31, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb371e3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f71e3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b171e41, SUC_lexicon_tags_1 },
    { 0xbfd71e42, SUC_lexicon_tags_12 },
    { 0x86f71e41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2371e47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa7571e49, SUC_lexicon_tags_10 },
    { 0x16f71e4a, SUC_lexicon_tags_63 },
    { 0xf6f71e4b, SUC_lexicon_tags_17 },
    { 0x30971e49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d71e50, SUC_lexicon_tags_7 },
    { 0xf5b71e51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a971e55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70171e57, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x27171e5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68571e5c, SUC_lexicon_tags_92 },
    { 0xa8171e5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80571e5f, SUC_lexicon_tags_48 },
    { 0x60171e5f, SUC_lexicon_tags_9 },
    { 0x58d71e61, SUC_lexicon_tags_33 },
    { 0xfc171e62, SUC_lexicon_tags_11 },
    { 0xc2d71e62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b371e68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f71e6c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f71e73, SUC_lexicon_tags_17 },
    { 0x43171e74, SUC_lexicon_tags_12 },
    { 0xb0771e75, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e971e7b, SUC_lexicon_tags_15 },
    { 0xe5b71e7c, SUC_lexicon_tags_28 },
    { 0xac571e7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1971e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99771e88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2971e8a, SUC_lexicon_tags_47 },
    { 0x5d71e8b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8771e8e, SUC_lexicon_tags_15 },
    { 0x95b71e8e, SUC_lexicon_tags_10 },
    { 0xd6771e90, SUC_lexicon_tags_3 },
    { 0x69371e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4dd71e93, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x89571e96, SUC_lexicon_tags_3 },
    { 0xa3171e97, SUC_lexicon_tags_0 },
    { 0xd9b71e98, SUC_lexicon_tags_61 },
    { 0xacf71e99, SUC_lexicon_tags_25 },
    { 0xe771e9a, SUC_lexicon_tags_25 },
    { 0xed71e9a, SUC_lexicon_tags_15 },
    { 0xe3171e9b, SUC_lexicon_tags_6 },
    { 0x7c371e9d, SUC_lexicon_tags_2 },
    { 0x4d571e9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb71ea1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd71ea6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xcff71ea8, SUC_lexicon_tags_17 },
    { 0x21f71ea9, SUC_lexicon_tags_0 },
    { 0xe6d71ea9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8f71eac, SUC_lexicon_tags_47 },
    { 0x9e371eac, SUC_lexicon_tags_1 },
    { 0xff971ead, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4971eb1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38771eb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23371eb8, SUC_lexicon_tags_1 },
    { 0x95971eb9, SUC_lexicon_tags_31 },
    { 0xf971eba, SUC_lexicon_tags_83 },
    { 0x56571ebb, SUC_lexicon_tags_49 },
    { 0x3d71eba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf971ec3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a371ec5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56171ecf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95b71ed1, SUC_lexicon_tags_7 },
    { 0xb9971ed2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22f71ed4, SUC_lexicon_tags_43 },
    { 0xf5b71ed5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd71ed8, SUC_lexicon_tags_11 },
    { 0xb1f71ed9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a571edb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7cd71edd, SUC_lexicon_tags_25 },
    { 0x2f571ede, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b71ee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fb71eea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f71eed, SUC_lexicon_tags_55 },
    { 0xd8f71eee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71d71ef0, SUC_lexicon_tags_15 },
    { 0x55571ef1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fb71ef3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15771ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8971f04, SUC_lexicon_tags_0 },
    { 0x1d571f05, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc971f0c, SUC_lexicon_tags_103 },
    { 0x13171f0d, SUC_lexicon_tags_6 },
    { 0x57f71f0c, SUC_lexicon_tags_1 },
    { 0xe3571f0d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24971f13, SUC_lexicon_tags_65 },
    { 0xdad71f14, SUC_lexicon_tags_8 },
    { 0xbd71f15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19571f18, SUC_lexicon_tags_65 },
    { 0xe0371f19, SUC_lexicon_tags_476 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf371f1c, SUC_lexicon_tags_17 },
    { 0x87971f1d, SUC_lexicon_tags_11 },
    { 0x55971f1e, SUC_lexicon_tags_7 },
    { 0xa3971f1e, SUC_lexicon_tags_17 },
    { 0x43d71f1d, SUC_lexicon_tags_148 },
    { 0xe0771f1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92d71f23, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b71f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5771f2b, SUC_lexicon_tags_47 },
    { 0x77971f2c, SUC_lexicon_tags_19 },
    { 0x8ed71f2d, SUC_lexicon_tags_17 },
    { 0x46b71f2c, SUC_lexicon_tags_11 },
    { 0xd9371f2c, SUC_lexicon_tags_141 },
    { 0xc5b71f2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x171f32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5571f40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf71f44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x39171f47, SUC_lexicon_tags_27 },
    { 0x45171f48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc371f4d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7771f52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28b71f54, SUC_lexicon_tags_1 },
    { 0x45f71f54, SUC_lexicon_tags_12 },
    { 0xcad71f56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91371f5a, SUC_lexicon_tags_11 },
    { 0x33771f5a, SUC_lexicon_tags_6 },
    { 0x83371f5c, SUC_lexicon_tags_26 },
    { 0x2f971f5d, SUC_lexicon_tags_47 },
    { 0xd9171f5d, SUC_lexicon_tags_11 },
    { 0x39971f5c, SUC_lexicon_tags_38 },
    { 0xc3b71f5f, SUC_lexicon_tags_63 },
    { 0xf5371f5e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f71f65, SUC_lexicon_tags_25 },
    { 0x2c371f66, SUC_lexicon_tags_8 },
    { 0x60971f67, SUC_lexicon_tags_57 },
    { 0xca971f66, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21971f71, SUC_lexicon_tags_0 },
    { 0x13571f71, SUC_lexicon_tags_2 },
    { 0x91571f73, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x28971f75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f71f79, SUC_lexicon_tags_7 },
    { 0xd2571f7a, SUC_lexicon_tags_11 },
    { 0x8fd71f7b, SUC_lexicon_tags_229 },
    { 0x82f71f7a, SUC_lexicon_tags_15 },
    { 0x771f7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb971f81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef71f84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f71f87, SUC_lexicon_tags_25 },
    { 0x52171f88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1571f8a, SUC_lexicon_tags_15 },
    { 0xf1f71f8b, SUC_lexicon_tags_4 },
    { 0x8fb71f8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb71f8f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b71f92, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1ad71f94, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f71f97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d171f9a, SUC_lexicon_tags_38 },
    { 0x7c971f9b, SUC_lexicon_tags_14 },
    { 0x19371f9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2571fa0, SUC_lexicon_tags_60 },
    { 0xa1b71fa1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x71571fa4, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xcfd71fa6, SUC_lexicon_tags_87 },
    { 0x63b71fa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d71faf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab171fb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf71fb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b71fbf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a571fc2, SUC_lexicon_tags_25 },
    { 0xe7b71fc3, SUC_lexicon_tags_29 },
    { 0x12771fc3, SUC_lexicon_tags_47 },
    { 0x3e571fc5, SUC_lexicon_tags_17 },
    { 0x1ad71fc5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d171fc9, SUC_lexicon_tags_63 },
    { 0x3b971fca, SUC_lexicon_tags_1 },
    { 0x14171fcb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96571fd0, SUC_lexicon_tags_28 },
    { 0xc4b71fd1, SUC_lexicon_tags_14 },
    { 0x5f771fd2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed371fd6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d71fda, SUC_lexicon_tags_25 },
    { 0xca171fda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26171fe1, SUC_lexicon_tags_31 },
    { 0xce771fe2, SUC_lexicon_tags_19 },
    { 0x15571fe3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23771fe9, SUC_lexicon_tags_64 },
    { 0x29971fea, SUC_lexicon_tags_17 },
    { 0x8c371feb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2971fef, SUC_lexicon_tags_27 },
    { 0x93d71fef, SUC_lexicon_tags_17 },
    { 0xf2571ff0, SUC_lexicon_tags_15 },
    { 0x40171ff1, SUC_lexicon_tags_8 },
    { 0xadd71ff3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb71ff7, SUC_lexicon_tags_25 },
    { 0x61571ff8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2371ffc, SUC_lexicon_tags_39 },
    { 0xa9971ffd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x68772000, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b72003, SUC_lexicon_tags_60 },
    { 0x2d972003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5372006, SUC_lexicon_tags_38 },
    { 0x69572006, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd417200e, SUC_lexicon_tags_7 },
    { 0xe7d7200f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b972014, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x46b72016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94572019, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb7201d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x53172020, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b572023, SUC_lexicon_tags_11 },
    { 0x89172024, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7af72026, SUC_lexicon_tags_25 },
    { 0x6ab72027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40972029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8857202b, SUC_lexicon_tags_17 },
    { 0x55b7202b, SUC_lexicon_tags_27 },
    { 0x2997202c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2772030, SUC_lexicon_tags_16 },
    { 0x3c772031, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfa572033, SUC_lexicon_tags_26 },
    { 0xcc572033, SUC_lexicon_tags_0 },
    { 0x22d72035, SUC_lexicon_tags_11 },
    { 0xc6b72036, SUC_lexicon_tags_37 },
    { 0x1bf72033, SUC_lexicon_tags_37 },
    { 0xfa772038, SUC_lexicon_tags_6 },
    { 0xbc972037, SUC_lexicon_tags_15 },
    { 0xe437203a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb257203c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df72047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b7204d, SUC_lexicon_tags_2 },
    { 0xef7204e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44372052, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b72058, SUC_lexicon_tags_37 },
    { 0x38172059, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb77205c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba372062, SUC_lexicon_tags_16 },
    { 0x1f772062, SUC_lexicon_tags_11 },
    { 0x1cd72064, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfe772066, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc572069, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8c37206b, SUC_lexicon_tags_23 },
    { 0x3a37206b, SUC_lexicon_tags_11 },
    { 0xb257206b, SUC_lexicon_tags_2 },
    { 0xf257206d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18972076, SUC_lexicon_tags_10 },
    { 0x6a372077, SUC_lexicon_tags_37 },
    { 0xca772078, SUC_lexicon_tags_65 },
    { 0xe972078, SUC_lexicon_tags_8 },
    { 0x3172078, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7fd7207c, SUC_lexicon_tags_11 },
    { 0x8877207d, SUC_lexicon_tags_9 },
    { 0x12b7207e, SUC_lexicon_tags_9 },
    { 0x9597207d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b72085, SUC_lexicon_tags_17 },
    { 0x48572086, SUC_lexicon_tags_4 },
    { 0xf0f72087, SUC_lexicon_tags_39 },
    { 0x8b772088, SUC_lexicon_tags_29 },
    { 0x7972088, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6137208b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b37208f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44372092, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f7209d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99720a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657720a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757720b9, SUC_lexicon_tags_1 },
    { 0x3e9720ba, SUC_lexicon_tags_17 },
    { 0x7af720bb, SUC_lexicon_tags_14 },
    { 0x5d5720bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7720c1, SUC_lexicon_tags_2 },
    { 0x83f720c2, SUC_lexicon_tags_19 },
    { 0x2ef720c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1f720c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13720c9, SUC_lexicon_tags_30 },
    { 0x779720ca, SUC_lexicon_tags_15 },
    { 0x505720cb, SUC_lexicon_tags_17 },
    { 0x4c5720cc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3720d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45720d9, SUC_lexicon_tags_47 },
    { 0x1a9720d9, SUC_lexicon_tags_6 },
    { 0x77f720da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d1720dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d720e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x205720e6, SUC_lexicon_tags_61 },
    { 0xccb720e6, SUC_lexicon_tags_25 },
    { 0x4c9720e8, SUC_lexicon_tags_0 },
    { 0x18b720e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xadb720eb, SUC_lexicon_tags_12 },
    { 0xdd7720ec, SUC_lexicon_tags_2 },
    { 0xaa7720eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83f720ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd720f5, SUC_lexicon_tags_11 },
    { 0xe59720f5, SUC_lexicon_tags_87 },
    { 0x517720f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31720fa, SUC_lexicon_tags_7 },
    { 0x4f1720fa, SUC_lexicon_tags_5 },
    { 0xb95720fb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f72102, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce772104, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed7210b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x577210e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d72112, SUC_lexicon_tags_17 },
    { 0xe7d72112, SUC_lexicon_tags_65 },
    { 0x3f772114, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35772118, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12b7211a, SUC_lexicon_tags_31 },
    { 0x4037211b, SUC_lexicon_tags_36 },
    { 0x1457211b, SUC_lexicon_tags_60 },
    { 0x84d7211b, SUC_lexicon_tags_15 },
    { 0xa737211e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef72121, SUC_lexicon_tags_454 },
    { 0x63772122, SUC_lexicon_tags_6 },
    { 0x81d72121, SUC_lexicon_tags_43 },
    { 0x93972123, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ef72126, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4772130, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb772132, SUC_lexicon_tags_3 },
    { 0xd0d72132, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd6772135, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1172137, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe357213c, SUC_lexicon_tags_25 },
    { 0xcdf7213c, SUC_lexicon_tags_37 },
    { 0x34d7213e, SUC_lexicon_tags_7 },
    { 0xa437213f, SUC_lexicon_tags_33 },
    { 0xcf972140, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef72143, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1b572145, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb5372147, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb7214a, SUC_lexicon_tags_91 },
    { 0x3ff7214b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b37214d, SUC_lexicon_tags_17 },
    { 0xb657214e, SUC_lexicon_tags_2 },
    { 0x4af7214d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed72155, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb372158, SUC_lexicon_tags_47 },
    { 0x57772158, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5697215d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4797215f, SUC_lexicon_tags_2 },
    { 0x28172160, SUC_lexicon_tags_7 },
    { 0xb0b72161, SUC_lexicon_tags_27 },
    { 0x23d72161, SUC_lexicon_tags_1 },
    { 0xd3d72161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e17216f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7972178, SUC_lexicon_tags_15 },
    { 0x43372179, SUC_lexicon_tags_2 },
    { 0x6777217a, SUC_lexicon_tags_28 },
    { 0x1577217a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ed7217d, SUC_lexicon_tags_11 },
    { 0xe737217e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28172182, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28172187, SUC_lexicon_tags_106 },
    { 0xcc972188, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc057218d, SUC_lexicon_tags_19 },
    { 0x4f77218d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf72194, SUC_lexicon_tags_9 },
    { 0xa0d72195, SUC_lexicon_tags_30 },
    { 0xb6372196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd357219b, SUC_lexicon_tags_45 },
    { 0x6017219b, SUC_lexicon_tags_7 },
    { 0x9a37219d, SUC_lexicon_tags_14 },
    { 0x5e97219e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47721a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21721ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b9721ae, SUC_lexicon_tags_23 },
    { 0x4cb721af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe17721b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd5d721b3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xe43721b5, SUC_lexicon_tags_13 },
    { 0xc0f721b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45721bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19721c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345721c9, SUC_lexicon_tags_38 },
    { 0x9c9721c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65721ce, SUC_lexicon_tags_2 },
    { 0xa89721ce, SUC_lexicon_tags_63 },
    { 0x3d1721d0, SUC_lexicon_tags_92 },
    { 0xfd721d1, SUC_lexicon_tags_25 },
    { 0xf05721d1, SUC_lexicon_tags_2 },
    { 0x883721d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39721d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57721da, SUC_lexicon_tags_2 },
    { 0xc37721db, SUC_lexicon_tags_7 },
    { 0xa29721dc, SUC_lexicon_tags_37 },
    { 0x56b721dd, SUC_lexicon_tags_15 },
    { 0x671721de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1721e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9721e5, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49721ea, SUC_lexicon_tags_1 },
    { 0xe17721eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b721ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af721f4, SUC_lexicon_tags_37 },
    { 0x78d721f5, SUC_lexicon_tags_16 },
    { 0xa9d721f5, SUC_lexicon_tags_60 },
    { 0xb35721f7, SUC_lexicon_tags_38 },
    { 0x7b3721f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x459721fa, SUC_lexicon_tags_103 },
    { 0x701721fb, SUC_lexicon_tags_7 },
    { 0x29f721fc, SUC_lexicon_tags_8 },
    { 0xae5721fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xab72200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f72206, SUC_lexicon_tags_1 },
    { 0x6e972206, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f57220b, SUC_lexicon_tags_37 },
    { 0x2ed7220c, SUC_lexicon_tags_29 },
    { 0x4697220d, SUC_lexicon_tags_14 },
    { 0xcd37220e, SUC_lexicon_tags_38 },
    { 0x87d7220e, SUC_lexicon_tags_25 },
    { 0x66f72210, SUC_lexicon_tags_11 },
    { 0x93b72211, SUC_lexicon_tags_3 },
    { 0xf5b72212, SUC_lexicon_tags_57 },
    { 0xf1372213, SUC_lexicon_tags_7 },
    { 0x52372213, SUC_lexicon_tags_55 },
    { 0x13772215, SUC_lexicon_tags_61 },
    { 0x1ff72216, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b7221d, SUC_lexicon_tags_12 },
    { 0x5b97221d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45372225, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70f72227, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737222c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b7222f, SUC_lexicon_tags_37 },
    { 0xa5372230, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd772233, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6ad72235, SUC_lexicon_tags_17 },
    { 0xedb72236, SUC_lexicon_tags_17 },
    { 0xe4f72237, SUC_lexicon_tags_27 },
    { 0x21b72237, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7017223a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40b7223c, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x3337223e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc772241, SUC_lexicon_tags_17 },
    { 0xcd972242, SUC_lexicon_tags_37 },
    { 0x78572242, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3c972245, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a572249, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f17224e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xac572250, SUC_lexicon_tags_11 },
    { 0xf7172250, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1af72253, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a772258, SUC_lexicon_tags_30 },
    { 0xf9372258, SUC_lexicon_tags_38 },
    { 0x3d17225a, SUC_lexicon_tags_6 },
    { 0x1017225b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf7225e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x13b72260, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7972269, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd277226b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed97226e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b72271, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d972276, SUC_lexicon_tags_33 },
    { 0xca972277, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb77227b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17972285, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a372287, SUC_lexicon_tags_28 },
    { 0x11172288, SUC_lexicon_tags_31 },
    { 0x6c972289, SUC_lexicon_tags_7 },
    { 0x3c972289, SUC_lexicon_tags_57 },
    { 0x3077228b, SUC_lexicon_tags_12 },
    { 0x7d97228c, SUC_lexicon_tags_48 },
    { 0x33d7228c, SUC_lexicon_tags_14 },
    { 0x5837228e, SUC_lexicon_tags_14 },
    { 0x17d7228f, SUC_lexicon_tags_65 },
    { 0x7117228d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd0972292, SUC_lexicon_tags_8 },
    { 0x86372292, SUC_lexicon_tags_29 },
    { 0x28172294, SUC_lexicon_tags_6 },
    { 0xffd72295, SUC_lexicon_tags_25 },
    { 0xadf72296, SUC_lexicon_tags_31 },
    { 0x1b572295, SUC_lexicon_tags_37 },
    { 0x56372298, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f7229b, SUC_lexicon_tags_38 },
    { 0x6d7229b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1f77229e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31722a2, SUC_lexicon_tags_33 },
    { 0x7f1722a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab722ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1722ae, SUC_lexicon_tags_7 },
    { 0x57b722af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbd722b1, SUC_lexicon_tags_25 },
    { 0x397722b1, SUC_lexicon_tags_23 },
    { 0xa2f722b3, SUC_lexicon_tags_6 },
    { 0x43b722b4, SUC_lexicon_tags_17 },
    { 0x3a9722b5, SUC_lexicon_tags_33 },
    { 0x92f722b6, SUC_lexicon_tags_12 },
    { 0xbc9722b3, SUC_lexicon_tags_138 },
    { 0x34d722b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81d722ba, SUC_lexicon_tags_60 },
    { 0x141722bb, SUC_lexicon_tags_15 },
    { 0x79b722bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63722c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d722c3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2b722c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x131722c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bd722ca, SUC_lexicon_tags_7 },
    { 0x55b722ca, SUC_lexicon_tags_7 },
    { 0xccf722cc, SUC_lexicon_tags_17 },
    { 0x3cd722cc, SUC_lexicon_tags_42 },
    { 0xff9722cb, SUC_lexicon_tags_47 },
    { 0x7f9722cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b722d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37722d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633722dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33722e3, SUC_lexicon_tags_65 },
    { 0xa7d722e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb722ed, SUC_lexicon_tags_39 },
    { 0x517722ee, SUC_lexicon_tags_11 },
    { 0x7c9722ef, SUC_lexicon_tags_37 },
    { 0x9a3722f0, SUC_lexicon_tags_26 },
    { 0x34b722f0, SUC_lexicon_tags_60 },
    { 0x1b1722f1, SUC_lexicon_tags_21 },
    { 0xdcb722f2, SUC_lexicon_tags_4 },
    { 0xf7722ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09722f7, SUC_lexicon_tags_17 },
    { 0xfc3722f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3722fe, SUC_lexicon_tags_2 },
    { 0x5bb722ff, SUC_lexicon_tags_14 },
    { 0xcef72300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e77230a, SUC_lexicon_tags_2 },
    { 0x6177230b, SUC_lexicon_tags_12 },
    { 0xb917230c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x92b7230e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4d72310, SUC_lexicon_tags_261 },
    { 0x20d72310, SUC_lexicon_tags_60 },
    { 0x3ad72312, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x93b72314, SUC_lexicon_tags_37 },
    { 0xf3572315, SUC_lexicon_tags_11 },
    { 0x4e972316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46372319, SUC_lexicon_tags_92 },
    { 0x75f7231a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb7231d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f772322, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2972326, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57772329, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9717232b, SUC_lexicon_tags_11 },
    { 0x3d37232c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4377232f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d72332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0772336, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37b72338, SUC_lexicon_tags_55 },
    { 0x56172339, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb72346, SUC_lexicon_tags_23 },
    { 0x90572346, SUC_lexicon_tags_29 },
    { 0x2e172348, SUC_lexicon_tags_1 },
    { 0x6f772348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b7234d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d72357, SUC_lexicon_tags_45 },
    { 0xd1f72358, SUC_lexicon_tags_2 },
    { 0x1b72359, SUC_lexicon_tags_37 },
    { 0x35b7235a, SUC_lexicon_tags_7 },
    { 0x1c57235b, SUC_lexicon_tags_65 },
    { 0xaf372358, SUC_lexicon_tags_49 },
    { 0x4df7235d, SUC_lexicon_tags_25 },
    { 0xdb17235d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc8b72360, SUC_lexicon_tags_2 },
    { 0x6bb72361, SUC_lexicon_tags_49 },
    { 0xa4f72362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb72365, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84972369, SUC_lexicon_tags_38 },
    { 0x59f72369, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d37236d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd772374, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b72378, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e37237b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4f7237d, SUC_lexicon_tags_11 },
    { 0x4937237e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7972380, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b72388, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c77238c, SUC_lexicon_tags_0 },
    { 0x9617238d, SUC_lexicon_tags_12 },
    { 0x9997238e, SUC_lexicon_tags_29 },
    { 0xa1d7238f, SUC_lexicon_tags_37 },
    { 0xd772390, SUC_lexicon_tags_12 },
    { 0x6bf72390, SUC_lexicon_tags_9 },
    { 0xb0172392, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9fd72394, SUC_lexicon_tags_117 },
    { 0xfcb72395, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x31b72397, SUC_lexicon_tags_17 },
    { 0xdc572398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7239d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f723a0, SUC_lexicon_tags_25 },
    { 0x575723a1, SUC_lexicon_tags_60 },
    { 0x55d723a2, SUC_lexicon_tags_23 },
    { 0xac7723a0, SUC_lexicon_tags_23 },
    { 0x333723a4, SUC_lexicon_tags_118 },
    { 0x363723a0, SUC_lexicon_tags_11 },
    { 0x74d723a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x619723a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661723b0, SUC_lexicon_tags_17 },
    { 0x1d3723b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3723b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa63723b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd723bf, SUC_lexicon_tags_17 },
    { 0xaf5723c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43723c3, SUC_lexicon_tags_65 },
    { 0x93b723c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09723c8, SUC_lexicon_tags_19 },
    { 0xf2d723c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf4d723cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7723ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e1723d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df723d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49723d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f723dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b723df, SUC_lexicon_tags_27 },
    { 0xaa7723df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32d723e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29723e6, SUC_lexicon_tags_11 },
    { 0xbdd723e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9f723e9, SUC_lexicon_tags_4 },
    { 0x2e9723e9, SUC_lexicon_tags_28 },
    { 0x9a7723eb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65723ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629723f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f723f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x715723f9, SUC_lexicon_tags_25 },
    { 0x911723fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3723fd, SUC_lexicon_tags_4 },
    { 0xe11723fe, SUC_lexicon_tags_104 },
    { 0xb53723ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x91372401, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f72405, SUC_lexicon_tags_29 },
    { 0x5ab72406, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0f72408, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7577240a, SUC_lexicon_tags_45 },
    { 0x7ff7240b, SUC_lexicon_tags_14 },
    { 0x1857240c, SUC_lexicon_tags_46 },
    { 0x9bf7240c, SUC_lexicon_tags_17 },
    { 0xc5d7240d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3172413, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23972417, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed172419, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6737241e, SUC_lexicon_tags_12 },
    { 0x5df7241f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60172424, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d7242a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d37242c, SUC_lexicon_tags_38 },
    { 0x49b7242d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b72432, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d72437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb7243b, SUC_lexicon_tags_15 },
    { 0x4d7243b, SUC_lexicon_tags_59 },
    { 0xdfb7243b, SUC_lexicon_tags_23 },
    { 0x1c17243c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d7244c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef72451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f372455, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b72459, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b7245b, SUC_lexicon_tags_11 },
    { 0x20f7245b, SUC_lexicon_tags_49 },
    { 0xd6d7245c, SUC_lexicon_tags_25 },
    { 0x1337245c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb972460, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9572463, SUC_lexicon_tags_30 },
    { 0x77972463, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1277246c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d72472, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb72477, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd7247a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4d7247c, SUC_lexicon_tags_2 },
    { 0x7b77247d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe277247f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4ad72481, SUC_lexicon_tags_17 },
    { 0x2d972481, SUC_lexicon_tags_0 },
    { 0xb0972483, SUC_lexicon_tags_9 },
    { 0x4c772482, SUC_lexicon_tags_65 },
    { 0xcd572485, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8372488, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d7248c, SUC_lexicon_tags_1 },
    { 0xb037248c, SUC_lexicon_tags_7 },
    { 0xf3f7248e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db72494, SUC_lexicon_tags_37 },
    { 0x5a572494, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0172498, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1eb7249a, SUC_lexicon_tags_15 },
    { 0x8377249b, SUC_lexicon_tags_2 },
    { 0x43b7249b, SUC_lexicon_tags_25 },
    { 0x3317249b, SUC_lexicon_tags_7 },
    { 0x9457249e, SUC_lexicon_tags_37 },
    { 0x4957249e, SUC_lexicon_tags_2 },
    { 0xc017249f, SUC_lexicon_tags_47 },
    { 0x3c1724a1, SUC_lexicon_tags_31 },
    { 0xee3724a2, SUC_lexicon_tags_33 },
    { 0x4b3724a1, SUC_lexicon_tags_46 },
    { 0x5cf7249e, SUC_lexicon_tags_47 },
    { 0x95d724a5, SUC_lexicon_tags_19 },
    { 0x67724a6, SUC_lexicon_tags_47 },
    { 0x567724a7, SUC_lexicon_tags_14 },
    { 0x679724a8, SUC_lexicon_tags_23 },
    { 0xf37724a1, SUC_lexicon_tags_33 },
    { 0x415724aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1724b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647724b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5724ba, SUC_lexicon_tags_37 },
    { 0xb7d724bb, SUC_lexicon_tags_49 },
    { 0xad724bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x40d724be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b3724c0, SUC_lexicon_tags_11 },
    { 0x4d3724c0, SUC_lexicon_tags_31 },
    { 0xc4d724c1, SUC_lexicon_tags_45 },
    { 0xf93724c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32f724c5, SUC_lexicon_tags_29 },
    { 0xe6d724c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1724ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b724cd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed724d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d724d8, SUC_lexicon_tags_63 },
    { 0x517724d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3724de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef724e1, SUC_lexicon_tags_37 },
    { 0x285724e1, SUC_lexicon_tags_2 },
    { 0x69f724e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x259724e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d724e9, SUC_lexicon_tags_35 },
    { 0x40f724ea, SUC_lexicon_tags_9 },
    { 0xc11724eb, SUC_lexicon_tags_17 },
    { 0x8f3724ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81b724ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6c1724f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5724f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8cd724f6, SUC_lexicon_tags_25 },
    { 0x50f724f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965724fb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73724fe, SUC_lexicon_tags_37 },
    { 0xd6f724ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5372502, SUC_lexicon_tags_12 },
    { 0xf6d72502, SUC_lexicon_tags_156 },
    { 0xd1f72504, SUC_lexicon_tags_12 },
    { 0x77f72503, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7250c, SUC_lexicon_tags_19 },
    { 0xef97250d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6017250f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd72514, SUC_lexicon_tags_17 },
    { 0x5f772515, SUC_lexicon_tags_42 },
    { 0xa772516, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x60572518, SUC_lexicon_tags_11 },
    { 0x1bf72518, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf7251d, SUC_lexicon_tags_1 },
    { 0xb057251d, SUC_lexicon_tags_66 },
    { 0xdc17251f, SUC_lexicon_tags_6 },
    { 0x9a772520, SUC_lexicon_tags_12 },
    { 0xb5172520, SUC_lexicon_tags_12 },
    { 0xdcd7251e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d72525, SUC_lexicon_tags_65 },
    { 0xd972525, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4e572528, SUC_lexicon_tags_106 },
    { 0x57b72529, SUC_lexicon_tags_14 },
    { 0xda572528, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d7252f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54372533, SUC_lexicon_tags_11 },
    { 0x41372533, SUC_lexicon_tags_33 },
    { 0x3e572535, SUC_lexicon_tags_49 },
    { 0xe9172534, SUC_lexicon_tags_129 },
    { 0x46372537, SUC_lexicon_tags_39 },
    { 0x64b72533, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5357253a, SUC_lexicon_tags_17 },
    { 0x89b7253a, SUC_lexicon_tags_49 },
    { 0xe7d7253a, SUC_lexicon_tags_0 },
    { 0x8f77253d, SUC_lexicon_tags_31 },
    { 0x4a37253c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4772545, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f972548, SUC_lexicon_tags_33 },
    { 0xc572549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd7254c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d57254e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b72551, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5ab72553, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x71f72555, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa772559, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f72561, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbb772563, SUC_lexicon_tags_2 },
    { 0xcc572564, SUC_lexicon_tags_15 },
    { 0xd4f72565, SUC_lexicon_tags_38 },
    { 0x4a972565, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f7256c, SUC_lexicon_tags_2 },
    { 0x1f97256d, SUC_lexicon_tags_1 },
    { 0x3eb7256e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a572572, SUC_lexicon_tags_37 },
    { 0x86f72573, SUC_lexicon_tags_26 },
    { 0xab772574, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32972579, SUC_lexicon_tags_31 },
    { 0xac57257a, SUC_lexicon_tags_69 },
    { 0x4c57257b, SUC_lexicon_tags_12 },
    { 0x3697257c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1657257f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87972581, SUC_lexicon_tags_23 },
    { 0xfd572581, SUC_lexicon_tags_19 },
    { 0x67f72583, SUC_lexicon_tags_17 },
    { 0x4c772584, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x85372587, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11172589, SUC_lexicon_tags_10 },
    { 0xef772589, SUC_lexicon_tags_27 },
    { 0x9f17258b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff72591, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb72598, SUC_lexicon_tags_23 },
    { 0x62972599, SUC_lexicon_tags_37 },
    { 0x72d7259a, SUC_lexicon_tags_12 },
    { 0x7e37259b, SUC_lexicon_tags_25 },
    { 0x1617259c, SUC_lexicon_tags_0 },
    { 0xfe57259d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e97259f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf7725a1, SUC_lexicon_tags_8 },
    { 0x843725a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5725a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x441725aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd725ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5f1725af, SUC_lexicon_tags_1 },
    { 0x573725b0, SUC_lexicon_tags_23 },
    { 0xbe5725af, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x1fb725b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x435725b6, SUC_lexicon_tags_15 },
    { 0x6b3725b6, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x623725b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3725bf, SUC_lexicon_tags_57 },
    { 0x3a1725c0, SUC_lexicon_tags_37 },
    { 0x849725c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9725d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde9725d3, SUC_lexicon_tags_9 },
    { 0xf87725d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b725d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb725dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f725de, SUC_lexicon_tags_11 },
    { 0x9d725df, SUC_lexicon_tags_23 },
    { 0x679725e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d725e3, SUC_lexicon_tags_17 },
    { 0x503725e4, SUC_lexicon_tags_47 },
    { 0x945725e5, SUC_lexicon_tags_151 },
    { 0x8b725e6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f725ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d725ef, SUC_lexicon_tags_17 },
    { 0x79725f0, SUC_lexicon_tags_299 },
    { 0xdb3725f1, SUC_lexicon_tags_14 },
    { 0x1b7725f1, SUC_lexicon_tags_42 },
    { 0xc7d725f1, SUC_lexicon_tags_33 },
    { 0xe4d725f4, SUC_lexicon_tags_11 },
    { 0xad9725f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f725f8, SUC_lexicon_tags_17 },
    { 0x8ed725f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3725fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f72604, SUC_lexicon_tags_21 },
    { 0x5372605, SUC_lexicon_tags_26 },
    { 0x62f72606, SUC_lexicon_tags_1 },
    { 0x2e172607, SUC_lexicon_tags_1 },
    { 0x6b972606, SUC_lexicon_tags_11 },
    { 0x3572607, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb437260c, SUC_lexicon_tags_49 },
    { 0xe077260d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdff7260f, SUC_lexicon_tags_23 },
    { 0x15372610, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b72613, SUC_lexicon_tags_198 },
    { 0x3f72614, SUC_lexicon_tags_17 },
    { 0x60d72613, SUC_lexicon_tags_17 },
    { 0x21d72615, SUC_lexicon_tags_27 },
    { 0xf3572617, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b57261d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92b7261f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf72622, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78972624, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x99972626, SUC_lexicon_tags_29 },
    { 0x3e972627, SUC_lexicon_tags_17 },
    { 0xde572628, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c57262c, SUC_lexicon_tags_9 },
    { 0x6d17262d, SUC_lexicon_tags_25 },
    { 0x6437262d, SUC_lexicon_tags_37 },
    { 0x8af7262f, SUC_lexicon_tags_29 },
    { 0xfff7262f, SUC_lexicon_tags_47 },
    { 0x5f72631, SUC_lexicon_tags_25 },
    { 0xe8572632, SUC_lexicon_tags_4 },
    { 0x5ef72633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5172635, SUC_lexicon_tags_7 },
    { 0xec972636, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e972638, SUC_lexicon_tags_25 },
    { 0xd6172639, SUC_lexicon_tags_11 },
    { 0xf8d72638, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d7263d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b72641, SUC_lexicon_tags_60 },
    { 0x172642, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf572645, SUC_lexicon_tags_11 },
    { 0x62972645, SUC_lexicon_tags_37 },
    { 0xe772646, SUC_lexicon_tags_1 },
    { 0xf0372648, SUC_lexicon_tags_12 },
    { 0x7d172645, SUC_lexicon_tags_37 },
    { 0x77b7264a, SUC_lexicon_tags_10 },
    { 0xd617264a, SUC_lexicon_tags_33 },
    { 0x60f7264b, SUC_lexicon_tags_2 },
    { 0x1f972646, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd72657, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x86572659, SUC_lexicon_tags_37 },
    { 0x7917265a, SUC_lexicon_tags_14 },
    { 0xcb17265a, SUC_lexicon_tags_19 },
    { 0x20d7265a, SUC_lexicon_tags_12 },
    { 0x74b7265d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e572669, SUC_lexicon_tags_63 },
    { 0xf2f7266a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597266e, SUC_lexicon_tags_15 },
    { 0x5357266e, SUC_lexicon_tags_29 },
    { 0x9777266f, SUC_lexicon_tags_10 },
    { 0x6c572671, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17172673, SUC_lexicon_tags_6 },
    { 0x98372674, SUC_lexicon_tags_1 },
    { 0x99f72674, SUC_lexicon_tags_49 },
    { 0xba772675, SUC_lexicon_tags_1 },
    { 0x8d372677, SUC_lexicon_tags_14 },
    { 0xfcb72678, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d72682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43172689, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe5f7268b, SUC_lexicon_tags_15 },
    { 0x6d57268c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17268f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed72692, SUC_lexicon_tags_17 },
    { 0x5eb72692, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92972699, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f7269c, SUC_lexicon_tags_49 },
    { 0xaf97269c, SUC_lexicon_tags_49 },
    { 0x7b57269c, SUC_lexicon_tags_65 },
    { 0x7cd7269d, SUC_lexicon_tags_46 },
    { 0xd137269d, SUC_lexicon_tags_25 },
    { 0x299726a1, SUC_lexicon_tags_13 },
    { 0x185726a1, SUC_lexicon_tags_17 },
    { 0x94b726a3, SUC_lexicon_tags_32 },
    { 0x389726a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d726a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf51726aa, SUC_lexicon_tags_14 },
    { 0xf05726aa, SUC_lexicon_tags_60 },
    { 0xfed726ac, SUC_lexicon_tags_46 },
    { 0x961726ad, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5726b2, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xb63726b4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9726b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe41726b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9726bf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4bf726c1, SUC_lexicon_tags_33 },
    { 0x8a5726c1, SUC_lexicon_tags_45 },
    { 0x919726c3, SUC_lexicon_tags_13 },
    { 0xddd726c2, SUC_lexicon_tags_82 },
    { 0xc2d726c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1726cd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6bb726cf, SUC_lexicon_tags_0 },
    { 0xb9f726d0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f726d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e3726d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f726e3, SUC_lexicon_tags_70 },
    { 0x6bd726e4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d726e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa33726ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae9726ec, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509726f1, SUC_lexicon_tags_8 },
    { 0xe53726f2, SUC_lexicon_tags_23 },
    { 0x207726f3, SUC_lexicon_tags_49 },
    { 0xbf5726f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f7726f6, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f726fb, SUC_lexicon_tags_591 },
    { 0x43726fb, SUC_lexicon_tags_11 },
    { 0x821726fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b72701, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf72705, SUC_lexicon_tags_15 },
    { 0x23772706, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8317270a, SUC_lexicon_tags_34 },
    { 0x7cd7270a, SUC_lexicon_tags_14 },
    { 0x6f57270b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60372712, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3817271e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2ed72720, SUC_lexicon_tags_31 },
    { 0xd6172721, SUC_lexicon_tags_29 },
    { 0x78d72720, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x52b72724, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b7273a, SUC_lexicon_tags_63 },
    { 0x7ed7273b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5d7273d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5972741, SUC_lexicon_tags_7 },
    { 0xd2772742, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2937274b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf572751, SUC_lexicon_tags_7 },
    { 0xadd72751, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4972759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f7275d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d72762, SUC_lexicon_tags_108 },
    { 0x7ab72763, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d72765, SUC_lexicon_tags_12 },
    { 0x21972766, SUC_lexicon_tags_25 },
    { 0xc972767, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6917276a, SUC_lexicon_tags_29 },
    { 0x8497276b, SUC_lexicon_tags_17 },
    { 0x3bb7276b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2317276e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd3572770, SUC_lexicon_tags_27 },
    { 0x9572770, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36372777, SUC_lexicon_tags_1 },
    { 0xf9772778, SUC_lexicon_tags_46 },
    { 0x4eb72778, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab172780, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57772785, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x89772788, SUC_lexicon_tags_8 },
    { 0x4b972789, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x197278c, SUC_lexicon_tags_151 },
    { 0x75d7278c, SUC_lexicon_tags_8 },
    { 0x1fb7278c, SUC_lexicon_tags_15 },
    { 0xf917278f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1772791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6972797, SUC_lexicon_tags_12 },
    { 0xb5972798, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa417279c, SUC_lexicon_tags_47 },
    { 0xe537279d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd727a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x229727a5, SUC_lexicon_tags_0 },
    { 0x2eb727a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b727ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf39727ae, SUC_lexicon_tags_10 },
    { 0x5cf727af, SUC_lexicon_tags_14 },
    { 0x5c3727af, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc27727b2, SUC_lexicon_tags_25 },
    { 0x51f727b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe29727b5, SUC_lexicon_tags_7 },
    { 0x24f727b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf95727b8, SUC_lexicon_tags_9 },
    { 0xaa9727b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b727bc, SUC_lexicon_tags_46 },
    { 0x741727bd, SUC_lexicon_tags_17 },
    { 0x3f9727be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd727c2, SUC_lexicon_tags_1 },
    { 0xd2b727c3, SUC_lexicon_tags_14 },
    { 0xdc3727c4, SUC_lexicon_tags_25 },
    { 0x947727c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1727cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab727d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd97727d2, SUC_lexicon_tags_26 },
    { 0xb77727d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f727d7, SUC_lexicon_tags_1 },
    { 0x36b727d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213727dd, SUC_lexicon_tags_32 },
    { 0xf4d727dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1db727e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199727e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf727e9, SUC_lexicon_tags_7 },
    { 0x283727ea, SUC_lexicon_tags_16 },
    { 0x505727eb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbc5727ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x913727f0, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb727f4, SUC_lexicon_tags_7 },
    { 0xf47727f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb99727f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebf727f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b727fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b72809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd377280c, SUC_lexicon_tags_230 },
    { 0x5ab7280d, SUC_lexicon_tags_12 },
    { 0x62d7280e, SUC_lexicon_tags_12 },
    { 0x2c97280f, SUC_lexicon_tags_37 },
    { 0xd737280f, SUC_lexicon_tags_23 },
    { 0xc457280f, SUC_lexicon_tags_35 },
    { 0xbdd72812, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x65f72814, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb3f72816, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf72819, SUC_lexicon_tags_37 },
    { 0x617281a, SUC_lexicon_tags_25 },
    { 0x3fd7281b, SUC_lexicon_tags_25 },
    { 0xa5b7281c, SUC_lexicon_tags_25 },
    { 0xedd7281d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x16d7281f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0d72821, SUC_lexicon_tags_19 },
    { 0x62f72821, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d72825, SUC_lexicon_tags_7 },
    { 0x2bd72825, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f72829, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8777282b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe397282e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdf72830, SUC_lexicon_tags_37 },
    { 0x52972830, SUC_lexicon_tags_56 },
    { 0xbdd72831, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c972834, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45572838, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2497283a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f7283e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc9772840, SUC_lexicon_tags_91 },
    { 0xd3172841, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d72845, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23172847, SUC_lexicon_tags_0 },
    { 0x8c572847, SUC_lexicon_tags_21 },
    { 0x1dd72849, SUC_lexicon_tags_17 },
    { 0x917284a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe837284f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6d972851, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677285b, SUC_lexicon_tags_25 },
    { 0x19d7285b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb7285f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfd572861, SUC_lexicon_tags_7 },
    { 0xa1f72862, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x14772864, SUC_lexicon_tags_37 },
    { 0x93772865, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5572867, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c572869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8b7286b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf7286f, SUC_lexicon_tags_63 },
    { 0x2a77286f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22972872, SUC_lexicon_tags_17 },
    { 0x47b72873, SUC_lexicon_tags_45 },
    { 0x2cd72873, SUC_lexicon_tags_20 },
    { 0xbed72875, SUC_lexicon_tags_37 },
    { 0x40172875, SUC_lexicon_tags_7 },
    { 0x91772877, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca17287c, SUC_lexicon_tags_39 },
    { 0xab17287c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb72880, SUC_lexicon_tags_87 },
    { 0x8a772881, SUC_lexicon_tags_2 },
    { 0xe8f72881, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91772887, SUC_lexicon_tags_12 },
    { 0x32b72888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac37288a, SUC_lexicon_tags_2 },
    { 0x4937288a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd772892, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0372894, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaaf72896, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d7289a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3257289f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9728a3, SUC_lexicon_tags_37 },
    { 0xa3728a3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x299728a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b1728a9, SUC_lexicon_tags_37 },
    { 0x279728aa, SUC_lexicon_tags_1 },
    { 0x731728ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b728ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669728b2, SUC_lexicon_tags_14 },
    { 0xcdf728b3, SUC_lexicon_tags_30 },
    { 0xa8d728b4, SUC_lexicon_tags_13 },
    { 0xba1728b5, SUC_lexicon_tags_21 },
    { 0x211728b6, SUC_lexicon_tags_11 },
    { 0xea3728b7, SUC_lexicon_tags_17 },
    { 0x973728b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9728bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157728c5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17728ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5728ce, SUC_lexicon_tags_14 },
    { 0x2f728ce, SUC_lexicon_tags_13 },
    { 0x291728cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21728d4, SUC_lexicon_tags_21 },
    { 0xf05728d4, SUC_lexicon_tags_4 },
    { 0x24d728d6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535728da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad728dd, SUC_lexicon_tags_17 },
    { 0xd79728dd, SUC_lexicon_tags_12 },
    { 0xa19728df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f728e4, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xf3d728e6, SUC_lexicon_tags_14 },
    { 0xdb728e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd5d728e9, SUC_lexicon_tags_49 },
    { 0x8b5728e9, SUC_lexicon_tags_10 },
    { 0xa37728eb, SUC_lexicon_tags_15 },
    { 0x84f728ec, SUC_lexicon_tags_12 },
    { 0x679728eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3728f0, SUC_lexicon_tags_15 },
    { 0x15728f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f728f5, SUC_lexicon_tags_19 },
    { 0xfe5728f6, SUC_lexicon_tags_14 },
    { 0x869728f7, SUC_lexicon_tags_7 },
    { 0xeab728f8, SUC_lexicon_tags_37 },
    { 0xd3b728f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28f728fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79728fd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1772902, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f72907, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa97290a, SUC_lexicon_tags_7 },
    { 0x9937290a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad57290e, SUC_lexicon_tags_15 },
    { 0xaa57290f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59172915, SUC_lexicon_tags_0 },
    { 0xed372915, SUC_lexicon_tags_15 },
    { 0x15572916, SUC_lexicon_tags_12 },
    { 0x7c772918, SUC_lexicon_tags_47 },
    { 0xbbd72919, SUC_lexicon_tags_11 },
    { 0x37172919, SUC_lexicon_tags_1 },
    { 0xd4d72919, SUC_lexicon_tags_6 },
    { 0x42f7291a, SUC_lexicon_tags_23 },
    { 0x8757291c, SUC_lexicon_tags_37 },
    { 0x62772916, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8972923, SUC_lexicon_tags_2 },
    { 0x1cb72924, SUC_lexicon_tags_43 },
    { 0x42772924, SUC_lexicon_tags_15 },
    { 0xb1172925, SUC_lexicon_tags_4 },
    { 0xccb72926, SUC_lexicon_tags_11 },
    { 0x40d72928, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f7292f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf72936, SUC_lexicon_tags_12 },
    { 0x81372937, SUC_lexicon_tags_4 },
    { 0x1cb72938, SUC_lexicon_tags_12 },
    { 0x87972938, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2897293e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53372945, SUC_lexicon_tags_47 },
    { 0xf5772946, SUC_lexicon_tags_68 },
    { 0x6d372946, SUC_lexicon_tags_15 },
    { 0x7f972946, SUC_lexicon_tags_47 },
    { 0xadb72949, SUC_lexicon_tags_6 },
    { 0x90172949, SUC_lexicon_tags_15 },
    { 0xf7294b, SUC_lexicon_tags_37 },
    { 0xad7294a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd172951, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9172956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88d72958, SUC_lexicon_tags_26 },
    { 0xd9372959, SUC_lexicon_tags_26 },
    { 0x4117295a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7737295d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2372960, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d72963, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d7296a, SUC_lexicon_tags_12 },
    { 0x5037296b, SUC_lexicon_tags_28 },
    { 0xac17296a, SUC_lexicon_tags_2 },
    { 0x6737296d, SUC_lexicon_tags_8 },
    { 0xd9f7296b, SUC_lexicon_tags_48 },
    { 0x9697296f, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0x4ef72971, SUC_lexicon_tags_79 },
    { 0x38f72972, SUC_lexicon_tags_28 },
    { 0xc9572972, SUC_lexicon_tags_2 },
    { 0xfc372974, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f72977, SUC_lexicon_tags_12 },
    { 0xf1572978, SUC_lexicon_tags_29 },
    { 0x9a572979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6917297c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0b7297e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d72981, SUC_lexicon_tags_1 },
    { 0xa0b72982, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2972985, SUC_lexicon_tags_12 },
    { 0x53d72986, SUC_lexicon_tags_43 },
    { 0xc2572985, SUC_lexicon_tags_534 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd7298a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4797298d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb72993, SUC_lexicon_tags_15 },
    { 0x45b72994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98b72996, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef7299e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb729a2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7d5729a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d3729a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319729aa, SUC_lexicon_tags_23 },
    { 0xf71729aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab729ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81729b1, SUC_lexicon_tags_37 },
    { 0xa33729b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf729b6, SUC_lexicon_tags_25 },
    { 0xcb5729b7, SUC_lexicon_tags_68 },
    { 0x40d729b8, SUC_lexicon_tags_47 },
    { 0x81b729b8, SUC_lexicon_tags_2 },
    { 0xacd729b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1729c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319729c6, SUC_lexicon_tags_49 },
    { 0xedf729c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd729cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4dd729cd, SUC_lexicon_tags_2 },
    { 0xde9729ce, SUC_lexicon_tags_15 },
    { 0xc1729cf, SUC_lexicon_tags_93 },
    { 0x5f7729cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdf729d2, SUC_lexicon_tags_11 },
    { 0xf4d729d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fb729d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29b729d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33729db, SUC_lexicon_tags_9 },
    { 0x4e9729db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd729df, SUC_lexicon_tags_6 },
    { 0xe19729e0, SUC_lexicon_tags_27 },
    { 0x7b7729e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1729e7, SUC_lexicon_tags_8 },
    { 0xb5b729e7, SUC_lexicon_tags_25 },
    { 0x991729e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ab729eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73729ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x475729f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd51729f4, SUC_lexicon_tags_10 },
    { 0x1d9729f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff729f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x21b729fb, SUC_lexicon_tags_6 },
    { 0x601729fb, SUC_lexicon_tags_65 },
    { 0xb57729fb, SUC_lexicon_tags_34 },
    { 0x7cb729fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe5572a00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68772a02, SUC_lexicon_tags_26 },
    { 0x6e372a02, SUC_lexicon_tags_29 },
    { 0x32372a04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76372a0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd372a14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d72a17, SUC_lexicon_tags_60 },
    { 0xdcd72a17, SUC_lexicon_tags_2 },
    { 0xfef72a19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b72a1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed572a25, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe0172a27, SUC_lexicon_tags_1 },
    { 0xdc972a27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d72a2b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xae572a2d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1572a30, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb2972a32, SUC_lexicon_tags_29 },
    { 0xd7d72a32, SUC_lexicon_tags_2 },
    { 0x30172a34, SUC_lexicon_tags_15 },
    { 0x89972a35, SUC_lexicon_tags_4 },
    { 0x5cd72a36, SUC_lexicon_tags_55 },
    { 0xc572a34, SUC_lexicon_tags_18 },
    { 0x6c572a34, SUC_lexicon_tags_17 },
    { 0xe972a38, SUC_lexicon_tags_4 },
    { 0xb7d72a3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcdf72a3c, SUC_lexicon_tags_63 },
    { 0xe6972a3d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x78372a3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a372a41, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x71772a44, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4372a4e, SUC_lexicon_tags_12 },
    { 0xc9772a4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a372a52, SUC_lexicon_tags_11 },
    { 0x1b572a53, SUC_lexicon_tags_7 },
    { 0xbb72a52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce372a5f, SUC_lexicon_tags_63 },
    { 0x4ad72a5f, SUC_lexicon_tags_12 },
    { 0x68372a61, SUC_lexicon_tags_9 },
    { 0xf8572a62, SUC_lexicon_tags_38 },
    { 0xe8b72a62, SUC_lexicon_tags_1 },
    { 0x1db72a63, SUC_lexicon_tags_17 },
    { 0xc7b72a64, SUC_lexicon_tags_7 },
    { 0xeb172a63, SUC_lexicon_tags_14 },
    { 0x49b72a67, SUC_lexicon_tags_11 },
    { 0x86d72a68, SUC_lexicon_tags_14 },
    { 0x94f72a69, SUC_lexicon_tags_63 },
    { 0x50972a6a, SUC_lexicon_tags_33 },
    { 0x2c972a61, SUC_lexicon_tags_37 },
    { 0x62572a6c, SUC_lexicon_tags_6 },
    { 0x2ed72a65, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2772a70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14172a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d72a7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7772a80, SUC_lexicon_tags_15 },
    { 0x62d72a80, SUC_lexicon_tags_0 },
    { 0xa1972a82, SUC_lexicon_tags_17 },
    { 0x74f72a83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1372a85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74972a89, SUC_lexicon_tags_49 },
    { 0x9c772a89, SUC_lexicon_tags_12 },
    { 0xf3772a8b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc6172a8d, SUC_lexicon_tags_15 },
    { 0x15372a8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49972a90, SUC_lexicon_tags_47 },
    { 0xee372a90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d72a95, SUC_lexicon_tags_25 },
    { 0x30172a96, SUC_lexicon_tags_87 },
    { 0xc9d72a95, SUC_lexicon_tags_27 },
    { 0xadd72a98, SUC_lexicon_tags_1 },
    { 0x1c172a98, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x19572a9b, SUC_lexicon_tags_17 },
    { 0x6d372a9c, SUC_lexicon_tags_124 },
    { 0xa4772a9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5b72a9f, SUC_lexicon_tags_63 },
    { 0xf5372a9f, SUC_lexicon_tags_84 },
    { 0xc372aa1, SUC_lexicon_tags_24 },
    { 0x27f72aa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8572aa7, SUC_lexicon_tags_13 },
    { 0x62b72aa8, SUC_lexicon_tags_27 },
    { 0x74b72aa9, SUC_lexicon_tags_66 },
    { 0x47372aa9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe5b72aac, SUC_lexicon_tags_17 },
    { 0xe9372aad, SUC_lexicon_tags_1 },
    { 0xe5772aad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd72ab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef72aba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89f72abc, SUC_lexicon_tags_40 },
    { 0x62572abc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e372ac0, SUC_lexicon_tags_26 },
    { 0x2fd72ac0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f172ac5, SUC_lexicon_tags_37 },
    { 0x18f72ac5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad172ac8, SUC_lexicon_tags_12 },
    { 0xce572ac8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4fd72acb, SUC_lexicon_tags_25 },
    { 0x5f172acc, SUC_lexicon_tags_91 },
    { 0xd5b72acd, SUC_lexicon_tags_11 },
    { 0x1f372ace, SUC_lexicon_tags_37 },
    { 0x46372ace, SUC_lexicon_tags_12 },
    { 0xa2172ad0, SUC_lexicon_tags_15 },
    { 0x13372acd, SUC_lexicon_tags_2 },
    { 0xee372ad2, SUC_lexicon_tags_11 },
    { 0x74172ad0, SUC_lexicon_tags_12 },
    { 0xbcb72ad4, SUC_lexicon_tags_25 },
    { 0xb0372ad5, SUC_lexicon_tags_15 },
    { 0x19f72acc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af72adb, SUC_lexicon_tags_28 },
    { 0x5b372adc, SUC_lexicon_tags_8 },
    { 0xd1172adc, SUC_lexicon_tags_12 },
    { 0xa9972add, SUC_lexicon_tags_2 },
    { 0x59172adf, SUC_lexicon_tags_15 },
    { 0xead72ae0, SUC_lexicon_tags_38 },
    { 0x22f72ae0, SUC_lexicon_tags_37 },
    { 0xef572ae0, SUC_lexicon_tags_51 },
    { 0x77b72ae3, SUC_lexicon_tags_25 },
    { 0x91972ae3, SUC_lexicon_tags_1 },
    { 0xcdd72ae3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb72ae7, SUC_lexicon_tags_47 },
    { 0x69d72ae7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8372aeb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b372af2, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1372af5, SUC_lexicon_tags_68 },
    { 0x3572af5, SUC_lexicon_tags_13 },
    { 0x62972af5, SUC_lexicon_tags_23 },
    { 0x4e372af6, SUC_lexicon_tags_28 },
    { 0xd2372af7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f72b01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30772b03, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xbd572b05, SUC_lexicon_tags_33 },
    { 0xe4f72b06, SUC_lexicon_tags_311 },
    { 0x76372b07, SUC_lexicon_tags_49 },
    { 0xea572b08, SUC_lexicon_tags_38 },
    { 0x74772b09, SUC_lexicon_tags_34 },
    { 0xaed72b08, SUC_lexicon_tags_23 },
    { 0x78d72b07, SUC_lexicon_tags_2 },
    { 0x19f72b0c, SUC_lexicon_tags_12 },
    { 0x81372b0d, SUC_lexicon_tags_0 },
    { 0xe4f72b0d, SUC_lexicon_tags_12 },
    { 0x9bb72b09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a572b16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12572b1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b772b1f, SUC_lexicon_tags_48 },
    { 0x8e572b20, SUC_lexicon_tags_11 },
    { 0x93b72b20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90372b26, SUC_lexicon_tags_7 },
    { 0x5b772b26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4c772b29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f172b2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43372b2d, SUC_lexicon_tags_29 },
    { 0xff572b2e, SUC_lexicon_tags_1 },
    { 0xdab72b2f, SUC_lexicon_tags_7 },
    { 0x8a372b30, SUC_lexicon_tags_107 },
    { 0xbaf72b31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d72b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c572b39, SUC_lexicon_tags_27 },
    { 0xd5172b39, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x22572b3d, SUC_lexicon_tags_1 },
    { 0x572b3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2572b42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31372b47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47572b4c, SUC_lexicon_tags_87 },
    { 0xd4d72b4d, SUC_lexicon_tags_2 },
    { 0x4a372b4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21372b53, SUC_lexicon_tags_9 },
    { 0x58972b54, SUC_lexicon_tags_25 },
    { 0x53572b55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x59772b57, SUC_lexicon_tags_21 },
    { 0x3ef72b57, SUC_lexicon_tags_43 },
    { 0xb5572b57, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1e372b5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22d72b5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e972b5f, SUC_lexicon_tags_43 },
    { 0xfef72b60, SUC_lexicon_tags_49 },
    { 0xc1372b61, SUC_lexicon_tags_1 },
    { 0x4dd72b5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1372b65, SUC_lexicon_tags_87 },
    { 0x39f72b66, SUC_lexicon_tags_37 },
    { 0x8b772b66, SUC_lexicon_tags_17 },
    { 0xea572b68, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89972b6f, SUC_lexicon_tags_11 },
    { 0xf372b70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58972b73, SUC_lexicon_tags_12 },
    { 0x5c372b73, SUC_lexicon_tags_11 },
    { 0x87972b75, SUC_lexicon_tags_65 },
    { 0x74372b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46572b78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7172b7c, SUC_lexicon_tags_45 },
    { 0xbef72b7c, SUC_lexicon_tags_37 },
    { 0xb1b72b7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f572b80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50572b82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0972b84, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f972b89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f72b8e, SUC_lexicon_tags_47 },
    { 0x3a972b8f, SUC_lexicon_tags_17 },
    { 0x97f72b8f, SUC_lexicon_tags_19 },
    { 0x5f572b90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f372b95, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2dd72b97, SUC_lexicon_tags_11 },
    { 0x9dd72b98, SUC_lexicon_tags_39 },
    { 0x4db72b99, SUC_lexicon_tags_37 },
    { 0xc6b72b9a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb572b9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x80772b9e, SUC_lexicon_tags_14 },
    { 0x4d172b9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xea372ba1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd72ba6, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x4b72ba8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x84372baa, SUC_lexicon_tags_37 },
    { 0x16372bab, SUC_lexicon_tags_11 },
    { 0x6df72baa, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xb72bae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x88972bb1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e772bb6, SUC_lexicon_tags_17 },
    { 0xb9772bb6, SUC_lexicon_tags_56 },
    { 0x4e372bb8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x47172bba, SUC_lexicon_tags_26 },
    { 0x90572bbb, SUC_lexicon_tags_15 },
    { 0xc9772bbb, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x87d72bbe, SUC_lexicon_tags_7 },
    { 0x24f72bbf, SUC_lexicon_tags_0 },
    { 0xa2b72bbe, SUC_lexicon_tags_11 },
    { 0x71572bc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc772bc3, SUC_lexicon_tags_45 },
    { 0x8b972bc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb72bc7, SUC_lexicon_tags_15 },
    { 0x83172bc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8372bcb, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xdcb72bcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2372bd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5b72bd3, SUC_lexicon_tags_2 },
    { 0xdfd72bd4, SUC_lexicon_tags_12 },
    { 0x86b72bd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d72bd9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d72bde, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47972be0, SUC_lexicon_tags_2 },
    { 0xdc972be1, SUC_lexicon_tags_29 },
    { 0x41b72be2, SUC_lexicon_tags_45 },
    { 0xd3372be0, SUC_lexicon_tags_39 },
    { 0xe8972be3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5972be7, SUC_lexicon_tags_6 },
    { 0x75172be8, SUC_lexicon_tags_47 },
    { 0x7cd72be7, SUC_lexicon_tags_10 },
    { 0x87b72bea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3b72bec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f72bef, SUC_lexicon_tags_21 },
    { 0x49772bf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d72bf4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f72bf8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf72bfc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8d172bfe, SUC_lexicon_tags_17 },
    { 0x29d72bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29172c02, SUC_lexicon_tags_38 },
    { 0x1ed72c02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xed572c05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac172c07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad72c09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x24572c0c, SUC_lexicon_tags_7 },
    { 0x2eb72c0d, SUC_lexicon_tags_113 },
    { 0x39d72c0d, SUC_lexicon_tags_11 },
    { 0x6972c0f, SUC_lexicon_tags_47 },
    { 0x3ad72c10, SUC_lexicon_tags_25 },
    { 0xa0772c10, SUC_lexicon_tags_37 },
    { 0xaa772c11, SUC_lexicon_tags_14 },
    { 0xd6b72c13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0972c16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb5b72c18, SUC_lexicon_tags_17 },
    { 0xde972c19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4d72c1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x62372c1d, SUC_lexicon_tags_45 },
    { 0x7172c1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0372c23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d172c29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89972c2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed72c30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b72c38, SUC_lexicon_tags_17 },
    { 0x60f72c39, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcdf72c3b, SUC_lexicon_tags_30 },
    { 0x75f72c3c, SUC_lexicon_tags_37 },
    { 0xb772c3d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd72c40, SUC_lexicon_tags_11 },
    { 0x54972c41, SUC_lexicon_tags_38 },
    { 0x2fb72c41, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee572c46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff172c4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d72c4e, SUC_lexicon_tags_17 },
    { 0xc9372c4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8172c51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef72c58, SUC_lexicon_tags_77 },
    { 0x84b72c59, SUC_lexicon_tags_19 },
    { 0x3772c5a, SUC_lexicon_tags_37 },
    { 0x1cd72c5b, SUC_lexicon_tags_0 },
    { 0xc5372c5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd72c62, SUC_lexicon_tags_7 },
    { 0xa6d72c62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52772c65, SUC_lexicon_tags_9 },
    { 0xb8972c66, SUC_lexicon_tags_56 },
    { 0x972c66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc972c6f, SUC_lexicon_tags_40 },
    { 0x20572c70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86772c72, SUC_lexicon_tags_26 },
    { 0x95f72c72, SUC_lexicon_tags_481 },
    { 0x4af72c74, SUC_lexicon_tags_12 },
    { 0x27172c75, SUC_lexicon_tags_37 },
    { 0x4b972c72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b72c79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38772c7b, SUC_lexicon_tags_17 },
    { 0x9cd72c7c, SUC_lexicon_tags_60 },
    { 0x46372c7d, SUC_lexicon_tags_6 },
    { 0xa8f72c7e, SUC_lexicon_tags_2 },
    { 0x36b72c7f, SUC_lexicon_tags_38 },
    { 0xaf72c80, SUC_lexicon_tags_12 },
    { 0x6e572c81, SUC_lexicon_tags_49 },
    { 0xa7972c81, SUC_lexicon_tags_25 },
    { 0x26d72c80, SUC_lexicon_tags_17 },
    { 0x9f72c82, SUC_lexicon_tags_9 },
    { 0xcfd72c85, SUC_lexicon_tags_7 },
    { 0x72772c7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b72c8a, SUC_lexicon_tags_31 },
    { 0x24d72c8b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f72c90, SUC_lexicon_tags_2 },
    { 0xf3372c91, SUC_lexicon_tags_7 },
    { 0x5f372c92, SUC_lexicon_tags_25 },
    { 0x77b72c92, SUC_lexicon_tags_11 },
    { 0xbcd72c94, SUC_lexicon_tags_25 },
    { 0xebf72c95, SUC_lexicon_tags_11 },
    { 0x16b72c90, SUC_lexicon_tags_65 },
    { 0xe4b72c97, SUC_lexicon_tags_23 },
    { 0xacd72c92, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f72c9c, SUC_lexicon_tags_25 },
    { 0x15b72c9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16172ca2, SUC_lexicon_tags_13 },
    { 0x69372ca3, SUC_lexicon_tags_30 },
    { 0x99172ca4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3b772ca6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b72cb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2a572cb8, SUC_lexicon_tags_17 },
    { 0x78572cb9, SUC_lexicon_tags_1 },
    { 0xb5772cb8, SUC_lexicon_tags_141 },
    { 0xd5172cba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf372cbe, SUC_lexicon_tags_0 },
    { 0x3f572cbf, SUC_lexicon_tags_2 },
    { 0x42572cc0, SUC_lexicon_tags_82 },
    { 0x38372cc1, SUC_lexicon_tags_15 },
    { 0x33772cc1, SUC_lexicon_tags_65 },
    { 0xb1172cc3, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0, NULL },
    { 0x59772cc6, SUC_lexicon_tags_12 },
    { 0x3a172cc6, SUC_lexicon_tags_28 },
    { 0x4db72cc8, SUC_lexicon_tags_29 },
    { 0xf4372cc9, SUC_lexicon_tags_37 },
    { 0xc372cca, SUC_lexicon_tags_17 },
    { 0x1e172ccb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x59172ccd, SUC_lexicon_tags_60 },
    { 0xd72cce, SUC_lexicon_tags_31 },
    { 0xb372ccd, SUC_lexicon_tags_19 },
    { 0x79172cd0, SUC_lexicon_tags_12 },
    { 0x7b972cd1, SUC_lexicon_tags_45 },
    { 0xf7172cd2, SUC_lexicon_tags_1 },
    { 0x6e372cd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76172cd6, SUC_lexicon_tags_4 },
    { 0x54772cd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0572cd9, SUC_lexicon_tags_17 },
    { 0x18972cd9, SUC_lexicon_tags_46 },
    { 0xcf972cd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0372cdd, SUC_lexicon_tags_16 },
    { 0x3bf72cde, SUC_lexicon_tags_30 },
    { 0xa8972cdf, SUC_lexicon_tags_7 },
    { 0x28772cde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62b72ce2, SUC_lexicon_tags_27 },
    { 0xf1b72ce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d572ce7, SUC_lexicon_tags_11 },
    { 0x50d72ce8, SUC_lexicon_tags_25 },
    { 0x6372ce9, SUC_lexicon_tags_25 },
    { 0x11972cea, SUC_lexicon_tags_38 },
    { 0x7972ceb, SUC_lexicon_tags_62 },
    { 0x12372cec, SUC_lexicon_tags_45 },
    { 0x66f72ced, SUC_lexicon_tags_37 },
    { 0xc372ced, SUC_lexicon_tags_37 },
    { 0x6a172cea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd72cf2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f72cf8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xa6972cfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d572d06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e772d08, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c772d0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d372d0e, SUC_lexicon_tags_7 },
    { 0x15b72d0f, SUC_lexicon_tags_75 },
    { 0x11972d0e, SUC_lexicon_tags_63 },
    { 0x3df72d11, SUC_lexicon_tags_25 },
    { 0xbbb72d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd72d16, SUC_lexicon_tags_1 },
    { 0x3f72d16, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a972d1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f72d1f, SUC_lexicon_tags_17 },
    { 0x5b972d20, SUC_lexicon_tags_19 },
    { 0xab972d20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e372d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1572d26, SUC_lexicon_tags_2 },
    { 0x89372d26, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f72d2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbed72d2c, SUC_lexicon_tags_87 },
    { 0xdab72d2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6972d30, SUC_lexicon_tags_47 },
    { 0xafb72d30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x82372d33, SUC_lexicon_tags_38 },
    { 0x71172d34, SUC_lexicon_tags_27 },
    { 0x33172d34, SUC_lexicon_tags_43 },
    { 0x92372d33, SUC_lexicon_tags_1 },
    { 0x5b972d37, SUC_lexicon_tags_1 },
    { 0x572d34, SUC_lexicon_tags_25 },
    { 0xcbf72d36, SUC_lexicon_tags_47 },
    { 0xbcf72d3a, SUC_lexicon_tags_15 },
    { 0xd372d3b, SUC_lexicon_tags_37 },
    { 0x6d772d39, SUC_lexicon_tags_12 },
    { 0xce372d3d, SUC_lexicon_tags_69 },
    { 0xb8f72d3e, SUC_lexicon_tags_15 },
    { 0xe4b72d34, SUC_lexicon_tags_7 },
    { 0xbd372d3f, SUC_lexicon_tags_43 },
    { 0xa6f72d41, SUC_lexicon_tags_28 },
    { 0xe8372d42, SUC_lexicon_tags_17 },
    { 0x85572d41, SUC_lexicon_tags_60 },
    { 0xc572d44, SUC_lexicon_tags_47 },
    { 0x65972d44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde572d47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a172d49, SUC_lexicon_tags_87 },
    { 0xa0372d4a, SUC_lexicon_tags_60 },
    { 0x71972d4a, SUC_lexicon_tags_23 },
    { 0x11772d4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b772d57, SUC_lexicon_tags_6 },
    { 0xb572d57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55d72d5a, SUC_lexicon_tags_10 },
    { 0x1b172d5b, SUC_lexicon_tags_63 },
    { 0xa4372d5c, SUC_lexicon_tags_26 },
    { 0xb6972d5c, SUC_lexicon_tags_38 },
    { 0x76172d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef72d61, SUC_lexicon_tags_11 },
    { 0xd3772d61, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d72d65, SUC_lexicon_tags_15 },
    { 0x11572d66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b72d69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab72d6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd72d70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7572d77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3f72d79, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16172d7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f972d80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d72d83, SUC_lexicon_tags_25 },
    { 0xd3772d83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b572d87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5772d8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f772d8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30172d92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa972d9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b72d9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88372da6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x68f72da8, SUC_lexicon_tags_26 },
    { 0xcdd72da9, SUC_lexicon_tags_15 },
    { 0xd4f72daa, SUC_lexicon_tags_26 },
    { 0xe2372daa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31772db2, SUC_lexicon_tags_82 },
    { 0x7e372db3, SUC_lexicon_tags_66 },
    { 0x74172db4, SUC_lexicon_tags_3 },
    { 0x4d772db5, SUC_lexicon_tags_12 },
    { 0x6172db5, SUC_lexicon_tags_47 },
    { 0x7572db7, SUC_lexicon_tags_199 },
    { 0, NULL },
    { 0xfed72db9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b72dbf, SUC_lexicon_tags_17 },
    { 0x22972dc0, SUC_lexicon_tags_40 },
    { 0xbd172dc0, SUC_lexicon_tags_21 },
    { 0x88b72dc0, SUC_lexicon_tags_7 },
    { 0xa6b72dc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1372dc8, SUC_lexicon_tags_11 },
    { 0xcb372dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7772dcb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7172dce, SUC_lexicon_tags_2 },
    { 0x79572dcf, SUC_lexicon_tags_7 },
    { 0x88972dd0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e772dd8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc9b72dda, SUC_lexicon_tags_48 },
    { 0xb7172dda, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced72ddf, SUC_lexicon_tags_47 },
    { 0x88b72ddf, SUC_lexicon_tags_9 },
    { 0xc1772de0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15572de7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb72dea, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d372ded, SUC_lexicon_tags_43 },
    { 0xa8772dee, SUC_lexicon_tags_12 },
    { 0x37172def, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb72df2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf7972df4, SUC_lexicon_tags_1 },
    { 0xeb172df5, SUC_lexicon_tags_60 },
    { 0x4772df5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd172df8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8372dfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc372dfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43f72e00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0972e02, SUC_lexicon_tags_35 },
    { 0x91d72e03, SUC_lexicon_tags_15 },
    { 0x5d172e03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8172e0b, SUC_lexicon_tags_33 },
    { 0x4c372e0c, SUC_lexicon_tags_26 },
    { 0x1b72e0b, SUC_lexicon_tags_11 },
    { 0xe2d72e0c, SUC_lexicon_tags_38 },
    { 0x88b72e0b, SUC_lexicon_tags_7 },
    { 0xc6f72e10, SUC_lexicon_tags_37 },
    { 0x2b372e11, SUC_lexicon_tags_0 },
    { 0xde172e12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b72e21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63572e26, SUC_lexicon_tags_47 },
    { 0x23372e27, SUC_lexicon_tags_48 },
    { 0x3cb72e26, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6772e2c, SUC_lexicon_tags_4 },
    { 0x3a172e2c, SUC_lexicon_tags_38 },
    { 0x75572e2e, SUC_lexicon_tags_92 },
    { 0x37f72e2d, SUC_lexicon_tags_1 },
    { 0x5d772e30, SUC_lexicon_tags_10 },
    { 0x8df72e31, SUC_lexicon_tags_1 },
    { 0x7b172e32, SUC_lexicon_tags_7 },
    { 0x4cd72e33, SUC_lexicon_tags_22 },
    { 0x75772e33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f172e36, SUC_lexicon_tags_11 },
    { 0x5b772e36, SUC_lexicon_tags_17 },
    { 0x28572e38, SUC_lexicon_tags_23 },
    { 0xc1d72e39, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1bf72e3b, SUC_lexicon_tags_42 },
    { 0xdd172e3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3372e3f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab72e44, SUC_lexicon_tags_17 },
    { 0xc4f72e45, SUC_lexicon_tags_25 },
    { 0x55972e46, SUC_lexicon_tags_12 },
    { 0x3ff72e45, SUC_lexicon_tags_60 },
    { 0x15572e48, SUC_lexicon_tags_7 },
    { 0x7b372e48, SUC_lexicon_tags_32 },
    { 0x7772e4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e772e54, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x89f72e56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55372e5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46772e5e, SUC_lexicon_tags_23 },
    { 0xcf972e5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb72e64, SUC_lexicon_tags_1 },
    { 0x11572e64, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e572e69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x16172e6b, SUC_lexicon_tags_30 },
    { 0x65772e6c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xeb372e6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40772e74, SUC_lexicon_tags_25 },
    { 0x54172e75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f72e78, SUC_lexicon_tags_43 },
    { 0xbc372e79, SUC_lexicon_tags_224 },
    { 0x83d72e7a, SUC_lexicon_tags_0 },
    { 0x19d72e7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c972e7f, SUC_lexicon_tags_14 },
    { 0x7e572e80, SUC_lexicon_tags_15 },
    { 0xcbd72e7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ad72e83, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b72e8c, SUC_lexicon_tags_8 },
    { 0x82172e8c, SUC_lexicon_tags_28 },
    { 0xe7572e8e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf8372e90, SUC_lexicon_tags_13 },
    { 0xb6372e90, SUC_lexicon_tags_17 },
    { 0x78372e91, SUC_lexicon_tags_9 },
    { 0x25572e93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb772e99, SUC_lexicon_tags_15 },
    { 0xf4572e99, SUC_lexicon_tags_17 },
    { 0xf9972e9b, SUC_lexicon_tags_15 },
    { 0x7db72e9c, SUC_lexicon_tags_8 },
    { 0xed972e9d, SUC_lexicon_tags_11 },
    { 0x7d972e9e, SUC_lexicon_tags_15 },
    { 0x52b72e9e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6cb72ea1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37972ea5, SUC_lexicon_tags_30 },
    { 0x8f572ea6, SUC_lexicon_tags_8 },
    { 0x19d72ea7, SUC_lexicon_tags_31 },
    { 0x49d72ea8, SUC_lexicon_tags_92 },
    { 0xd4d72ea6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44972ead, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d72eb1, SUC_lexicon_tags_240 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd72eb4, SUC_lexicon_tags_7 },
    { 0x38572eb5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc0772eb7, SUC_lexicon_tags_55 },
    { 0xd8372eb8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed72ebd, SUC_lexicon_tags_25 },
    { 0x43572ebd, SUC_lexicon_tags_37 },
    { 0x42572ebe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x46572ec2, SUC_lexicon_tags_8 },
    { 0x9b372ec2, SUC_lexicon_tags_28 },
    { 0x87572ec4, SUC_lexicon_tags_60 },
    { 0x1bd72ec5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84572ecd, SUC_lexicon_tags_17 },
    { 0xc7d72ecd, SUC_lexicon_tags_12 },
    { 0xec772ecf, SUC_lexicon_tags_12 },
    { 0x61372ecd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f372ed2, SUC_lexicon_tags_37 },
    { 0x6b772ed2, SUC_lexicon_tags_66 },
    { 0xbb972ed4, SUC_lexicon_tags_12 },
    { 0x89772ed4, SUC_lexicon_tags_7 },
    { 0x64572ed6, SUC_lexicon_tags_17 },
    { 0x6ff72ed7, SUC_lexicon_tags_43 },
    { 0xec772ed4, SUC_lexicon_tags_25 },
    { 0x8e972ed9, SUC_lexicon_tags_37 },
    { 0xfad72ed8, SUC_lexicon_tags_32 },
    { 0x23572edb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8172edd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98b72edf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25b72ee1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3572ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96972ee8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb772eea, SUC_lexicon_tags_10 },
    { 0x79d72eea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d172ef0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b72ef9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb72eff, SUC_lexicon_tags_140 },
    { 0x15572f00, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf1172f02, SUC_lexicon_tags_15 },
    { 0xb2172f03, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d72f07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39572f10, SUC_lexicon_tags_46 },
    { 0x3f772f10, SUC_lexicon_tags_4 },
    { 0xe3f72f12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4372f17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d72f1d, SUC_lexicon_tags_3 },
    { 0x1cd72f1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8372f21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ab72f23, SUC_lexicon_tags_82 },
    { 0x64572f24, SUC_lexicon_tags_25 },
    { 0xda372f24, SUC_lexicon_tags_14 },
    { 0xbb372f26, SUC_lexicon_tags_11 },
    { 0x74972f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7572f2f, SUC_lexicon_tags_37 },
    { 0x97972f30, SUC_lexicon_tags_27 },
    { 0x94372f31, SUC_lexicon_tags_29 },
    { 0x1ef72f31, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3972f36, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f172f3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6b72f3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd572f41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cf72f43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3f72f45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b372f48, SUC_lexicon_tags_7 },
    { 0x4d72f49, SUC_lexicon_tags_0 },
    { 0x3372f4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40572f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6d72f4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f72f56, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b972f59, SUC_lexicon_tags_11 },
    { 0xb0972f5a, SUC_lexicon_tags_11 },
    { 0xe3b72f5b, SUC_lexicon_tags_15 },
    { 0xd7572f5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b372f5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5b72f61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d72f64, SUC_lexicon_tags_4 },
    { 0x3c572f64, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3972f68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef372f6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb8f72f6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd72f70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a172f72, SUC_lexicon_tags_2 },
    { 0xfdf72f73, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x47572f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d72f78, SUC_lexicon_tags_60 },
    { 0x3e972f79, SUC_lexicon_tags_26 },
    { 0x6e372f7a, SUC_lexicon_tags_33 },
    { 0x6f72f7a, SUC_lexicon_tags_27 },
    { 0x1772f7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f172f84, SUC_lexicon_tags_15 },
    { 0xe3572f84, SUC_lexicon_tags_1 },
    { 0x34972f84, SUC_lexicon_tags_6 },
    { 0x67372f87, SUC_lexicon_tags_17 },
    { 0x25372f88, SUC_lexicon_tags_6 },
    { 0x3ab72f87, SUC_lexicon_tags_11 },
    { 0xe5f72f88, SUC_lexicon_tags_102 },
    { 0, NULL },
    { 0xee372f8c, SUC_lexicon_tags_23 },
    { 0xeb372f8d, SUC_lexicon_tags_0 },
    { 0x35572f8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd372f92, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda172f96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f72f99, SUC_lexicon_tags_27 },
    { 0xaed72f9a, SUC_lexicon_tags_43 },
    { 0x1b972f9a, SUC_lexicon_tags_1 },
    { 0x7db72f9c, SUC_lexicon_tags_27 },
    { 0xc5d72f9a, SUC_lexicon_tags_25 },
    { 0x6eb72f9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2172fa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8972fa5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f72faa, SUC_lexicon_tags_4 },
    { 0xe372faa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa9d72fad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1972fb1, SUC_lexicon_tags_17 },
    { 0x2e972fb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb772fb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d72fbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f572fbe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee572fc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3372fd3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df72fd7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe372fda, SUC_lexicon_tags_2 },
    { 0x89172fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7372fdf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa2b72fe1, SUC_lexicon_tags_7 },
    { 0x46172fe2, SUC_lexicon_tags_30 },
    { 0x4ab72fe3, SUC_lexicon_tags_25 },
    { 0x8b372fe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d72fea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f72fef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e572ff3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73772ff7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f72ffc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe3d72ffe, SUC_lexicon_tags_47 },
    { 0xc172ffe, SUC_lexicon_tags_25 },
    { 0x89973000, SUC_lexicon_tags_37 },
    { 0xdd772ffe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9173008, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa677300e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf73012, SUC_lexicon_tags_82 },
    { 0x54573013, SUC_lexicon_tags_19 },
    { 0x2f973014, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x80973017, SUC_lexicon_tags_37 },
    { 0xb9373018, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6637301a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2973020, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af73024, SUC_lexicon_tags_0 },
    { 0x2f573024, SUC_lexicon_tags_60 },
    { 0x97973025, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x92173029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5117302e, SUC_lexicon_tags_12 },
    { 0xb697302f, SUC_lexicon_tags_15 },
    { 0xc017302f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d573033, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b773036, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22373039, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e97303e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b73046, SUC_lexicon_tags_17 },
    { 0xffd73047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2777304c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54773052, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x37973055, SUC_lexicon_tags_25 },
    { 0x69373056, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b73059, SUC_lexicon_tags_25 },
    { 0xad7305a, SUC_lexicon_tags_7 },
    { 0x6737305b, SUC_lexicon_tags_37 },
    { 0x69d7305c, SUC_lexicon_tags_34 },
    { 0xef37305d, SUC_lexicon_tags_9 },
    { 0x4737305a, SUC_lexicon_tags_33 },
    { 0x36b7305f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfb73061, SUC_lexicon_tags_56 },
    { 0xf8573061, SUC_lexicon_tags_19 },
    { 0x98b73061, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0973065, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81573069, SUC_lexicon_tags_2 },
    { 0xe6f7306a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47573070, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87b73072, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x47f73074, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb957307b, SUC_lexicon_tags_7 },
    { 0x2e57307c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8657307e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7d73080, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9773082, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a173085, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb7308a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d73090, SUC_lexicon_tags_0 },
    { 0xa0b73091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb173094, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe917309a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637730a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7730a2, SUC_lexicon_tags_7 },
    { 0xd5b730a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b730a6, SUC_lexicon_tags_37 },
    { 0x7db730a6, SUC_lexicon_tags_33 },
    { 0x305730a8, SUC_lexicon_tags_65 },
    { 0x26f730a8, SUC_lexicon_tags_5 },
    { 0x5d5730a9, SUC_lexicon_tags_25 },
    { 0xc17730aa, SUC_lexicon_tags_7 },
    { 0x59f730ab, SUC_lexicon_tags_15 },
    { 0x5d9730a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51730b0, SUC_lexicon_tags_27 },
    { 0xb09730b0, SUC_lexicon_tags_8 },
    { 0xadd730b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb730b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545730bc, SUC_lexicon_tags_11 },
    { 0x13b730bd, SUC_lexicon_tags_17 },
    { 0xaeb730bd, SUC_lexicon_tags_0 },
    { 0xbbf730bf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9d730c1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc4b730c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65730c9, SUC_lexicon_tags_62 },
    { 0x5b5730ca, SUC_lexicon_tags_10 },
    { 0x12f730c9, SUC_lexicon_tags_43 },
    { 0x507730cc, SUC_lexicon_tags_15 },
    { 0xad3730cb, SUC_lexicon_tags_47 },
    { 0x8f5730cb, SUC_lexicon_tags_23 },
    { 0x355730cd, SUC_lexicon_tags_34 },
    { 0xc37730d0, SUC_lexicon_tags_13 },
    { 0x695730ce, SUC_lexicon_tags_33 },
    { 0x5e5730d2, SUC_lexicon_tags_45 },
    { 0x317730cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5730d6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa21730d8, SUC_lexicon_tags_39 },
    { 0x59d730d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe1d730db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f730e0, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d730e6, SUC_lexicon_tags_7 },
    { 0x935730e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc51730e9, SUC_lexicon_tags_49 },
    { 0xa0f730e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3730f0, SUC_lexicon_tags_1 },
    { 0x917730f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb730f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb59730f6, SUC_lexicon_tags_37 },
    { 0xfc9730f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541730ff, SUC_lexicon_tags_23 },
    { 0xc6973100, SUC_lexicon_tags_47 },
    { 0x7e7730ff, SUC_lexicon_tags_60 },
    { 0x555730ff, SUC_lexicon_tags_37 },
    { 0x9f973103, SUC_lexicon_tags_27 },
    { 0xf4373104, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af73107, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea17310d, SUC_lexicon_tags_1 },
    { 0x2c17310e, SUC_lexicon_tags_2 },
    { 0xeb7310d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f173111, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db73117, SUC_lexicon_tags_19 },
    { 0x86773117, SUC_lexicon_tags_1 },
    { 0x14373119, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f7311c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e573120, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18573123, SUC_lexicon_tags_25 },
    { 0x29373123, SUC_lexicon_tags_60 },
    { 0x50f73125, SUC_lexicon_tags_7 },
    { 0xc2373126, SUC_lexicon_tags_12 },
    { 0x3c373124, SUC_lexicon_tags_88 },
    { 0x10373128, SUC_lexicon_tags_15 },
    { 0xef73129, SUC_lexicon_tags_17 },
    { 0xa173124, SUC_lexicon_tags_87 },
    { 0x5db7312a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a37312e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56973131, SUC_lexicon_tags_12 },
    { 0xdbd73132, SUC_lexicon_tags_37 },
    { 0x3c573133, SUC_lexicon_tags_45 },
    { 0xde973132, SUC_lexicon_tags_23 },
    { 0x5d773132, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d7313b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7177313d, SUC_lexicon_tags_27 },
    { 0xaad7313e, SUC_lexicon_tags_2 },
    { 0x2797313f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60573142, SUC_lexicon_tags_25 },
    { 0xd173142, SUC_lexicon_tags_66 },
    { 0x62d73143, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xad973146, SUC_lexicon_tags_37 },
    { 0xd7373146, SUC_lexicon_tags_60 },
    { 0xb3373148, SUC_lexicon_tags_17 },
    { 0x72373148, SUC_lexicon_tags_83 },
    { 0x2ed73147, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf7314f, SUC_lexicon_tags_7 },
    { 0xa397314f, SUC_lexicon_tags_1 },
    { 0xcf973151, SUC_lexicon_tags_10 },
    { 0x5973151, SUC_lexicon_tags_25 },
    { 0x84f73152, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7773157, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8577315b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe437315d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d57315f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1973163, SUC_lexicon_tags_15 },
    { 0x90573164, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67973167, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f7316c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f73171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb373174, SUC_lexicon_tags_30 },
    { 0x2db73174, SUC_lexicon_tags_21 },
    { 0xff173176, SUC_lexicon_tags_7 },
    { 0xd1373177, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f7317b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa117317d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39773181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf173188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9997318e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb373190, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0973199, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d7319c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1117319e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b731a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe23731a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbf731a6, SUC_lexicon_tags_31 },
    { 0xdc7731a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf15731a9, SUC_lexicon_tags_30 },
    { 0x529731aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53731ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe83731af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb731b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71b731b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90b731b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1731bd, SUC_lexicon_tags_7 },
    { 0x91d731be, SUC_lexicon_tags_38 },
    { 0x10d731bd, SUC_lexicon_tags_0 },
    { 0xbbf731be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x889731c2, SUC_lexicon_tags_43 },
    { 0x755731c3, SUC_lexicon_tags_15 },
    { 0xefb731c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73731ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db731ce, SUC_lexicon_tags_47 },
    { 0xff5731cf, SUC_lexicon_tags_27 },
    { 0xc4f731d0, SUC_lexicon_tags_57 },
    { 0xe7d731d0, SUC_lexicon_tags_15 },
    { 0x8ab731d1, SUC_lexicon_tags_10 },
    { 0x495731d1, SUC_lexicon_tags_23 },
    { 0xc7f731d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x761731d7, SUC_lexicon_tags_17 },
    { 0xa55731d7, SUC_lexicon_tags_32 },
    { 0xff5731d9, SUC_lexicon_tags_27 },
    { 0x4e5731da, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d731de, SUC_lexicon_tags_12 },
    { 0xe85731df, SUC_lexicon_tags_113 },
    { 0x29731e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879731e5, SUC_lexicon_tags_11 },
    { 0x2cb731e5, SUC_lexicon_tags_7 },
    { 0x21b731e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80b731e9, SUC_lexicon_tags_7 },
    { 0x46b731ea, SUC_lexicon_tags_63 },
    { 0x3b1731eb, SUC_lexicon_tags_13 },
    { 0x6bd731ea, SUC_lexicon_tags_10 },
    { 0xa23731ed, SUC_lexicon_tags_1 },
    { 0x3f5731e9, SUC_lexicon_tags_47 },
    { 0xd3b731ef, SUC_lexicon_tags_0 },
    { 0x851731f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9731f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x757731f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f3731f8, SUC_lexicon_tags_12 },
    { 0xd89731f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x773731fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf773205, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4773209, SUC_lexicon_tags_15 },
    { 0x837320a, SUC_lexicon_tags_17 },
    { 0xcf57320b, SUC_lexicon_tags_46 },
    { 0x1117320a, SUC_lexicon_tags_14 },
    { 0x7577320d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d73216, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7ed73218, SUC_lexicon_tags_676 },
    { 0, NULL },
    { 0x8c17321a, SUC_lexicon_tags_5 },
    { 0x8657321a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a77321f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x31773222, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff73228, SUC_lexicon_tags_14 },
    { 0x1df73229, SUC_lexicon_tags_32 },
    { 0xd4b7322a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd2b7322c, SUC_lexicon_tags_14 },
    { 0x9157322d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c773231, SUC_lexicon_tags_29 },
    { 0xb1f73232, SUC_lexicon_tags_9 },
    { 0xbf173233, SUC_lexicon_tags_37 },
    { 0x5e173231, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33373240, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b73247, SUC_lexicon_tags_11 },
    { 0xe4973248, SUC_lexicon_tags_2 },
    { 0x8d973249, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba17324b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x317324d, SUC_lexicon_tags_25 },
    { 0x70f7324d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81773253, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc773255, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7325c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5217325f, SUC_lexicon_tags_21 },
    { 0x78b7325f, SUC_lexicon_tags_19 },
    { 0x83773261, SUC_lexicon_tags_17 },
    { 0x21b73262, SUC_lexicon_tags_25 },
    { 0xd337325f, SUC_lexicon_tags_25 },
    { 0x3b573260, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x61973266, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1297326a, SUC_lexicon_tags_12 },
    { 0xebb7326a, SUC_lexicon_tags_9 },
    { 0x4697326a, SUC_lexicon_tags_66 },
    { 0x11b7326d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea57326f, SUC_lexicon_tags_1 },
    { 0x73d73270, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x38f73272, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab73277, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6037327b, SUC_lexicon_tags_19 },
    { 0xd677327c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x23d7327e, SUC_lexicon_tags_23 },
    { 0x1a57327f, SUC_lexicon_tags_2 },
    { 0x96d73280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3f73282, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55173288, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f73290, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc973293, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a573297, SUC_lexicon_tags_0 },
    { 0x52b73298, SUC_lexicon_tags_17 },
    { 0x35b73299, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5457329d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb732a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59732a4, SUC_lexicon_tags_29 },
    { 0xef3732a5, SUC_lexicon_tags_66 },
    { 0x50b732a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33732ab, SUC_lexicon_tags_29 },
    { 0x6bf732ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5732b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe4b732b2, SUC_lexicon_tags_17 },
    { 0xe09732b3, SUC_lexicon_tags_25 },
    { 0x8ab732b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1d732b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x603732b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f732bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d732bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635732c3, SUC_lexicon_tags_37 },
    { 0xd83732c4, SUC_lexicon_tags_11 },
    { 0x923732c3, SUC_lexicon_tags_275 },
    { 0xdb732c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2bb732c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1732cc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x85f732ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f732d1, SUC_lexicon_tags_26 },
    { 0xd81732d1, SUC_lexicon_tags_17 },
    { 0xd39732d3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1fb732d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13732d8, SUC_lexicon_tags_4 },
    { 0x5e7732d9, SUC_lexicon_tags_17 },
    { 0x969732d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaef732dc, SUC_lexicon_tags_7 },
    { 0x39b732dd, SUC_lexicon_tags_27 },
    { 0x87f732dd, SUC_lexicon_tags_37 },
    { 0xb47732de, SUC_lexicon_tags_8 },
    { 0xa31732e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb732e3, SUC_lexicon_tags_63 },
    { 0x691732e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b732e7, SUC_lexicon_tags_37 },
    { 0x93732e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565732ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xead732ef, SUC_lexicon_tags_2 },
    { 0xff9732ef, SUC_lexicon_tags_56 },
    { 0x7df732f1, SUC_lexicon_tags_121 },
    { 0xab732ef, SUC_lexicon_tags_37 },
    { 0xda9732f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x877732f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb732fa, SUC_lexicon_tags_37 },
    { 0x54b732fb, SUC_lexicon_tags_65 },
    { 0xe1b732fc, SUC_lexicon_tags_27 },
    { 0x867732fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28d732ff, SUC_lexicon_tags_1 },
    { 0x9af73300, SUC_lexicon_tags_33 },
    { 0x79d73301, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfeb73303, SUC_lexicon_tags_30 },
    { 0x3b573303, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d7330d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8773311, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2173314, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d73318, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1557331e, SUC_lexicon_tags_11 },
    { 0xe4d7331f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46573324, SUC_lexicon_tags_68 },
    { 0x23f73325, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf573329, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe617332c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xedd7332e, SUC_lexicon_tags_33 },
    { 0xa1f7332f, SUC_lexicon_tags_12 },
    { 0x9ed7332f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4973333, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x42d73335, SUC_lexicon_tags_17 },
    { 0xaa173336, SUC_lexicon_tags_33 },
    { 0x1e573336, SUC_lexicon_tags_9 },
    { 0xc9373338, SUC_lexicon_tags_12 },
    { 0x22773337, SUC_lexicon_tags_17 },
    { 0xabf7333a, SUC_lexicon_tags_19 },
    { 0x7db73338, SUC_lexicon_tags_37 },
    { 0xed7333c, SUC_lexicon_tags_2 },
    { 0x3b37333d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd573340, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x21173343, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x79f73345, SUC_lexicon_tags_1 },
    { 0xd4d73345, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf477334a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b7334e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f573353, SUC_lexicon_tags_13 },
    { 0xdc373353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9497335a, SUC_lexicon_tags_60 },
    { 0x1057335b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb7b7335d, SUC_lexicon_tags_0 },
    { 0x9517335e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f73361, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c773363, SUC_lexicon_tags_12 },
    { 0x86973363, SUC_lexicon_tags_11 },
    { 0x49373363, SUC_lexicon_tags_11 },
    { 0xfc573363, SUC_lexicon_tags_16 },
    { 0xb173363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9df73369, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6597336c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f17336f, SUC_lexicon_tags_19 },
    { 0x50f73370, SUC_lexicon_tags_11 },
    { 0x2b573370, SUC_lexicon_tags_48 },
    { 0xfed73371, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x13373374, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff77337e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf37338c, SUC_lexicon_tags_194 },
    { 0x2797338d, SUC_lexicon_tags_45 },
    { 0xb4f7338e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd73391, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45573395, SUC_lexicon_tags_17 },
    { 0xc5d73395, SUC_lexicon_tags_25 },
    { 0x35773397, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449733a3, SUC_lexicon_tags_48 },
    { 0x1cd733a4, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71733aa, SUC_lexicon_tags_2 },
    { 0x5733aa, SUC_lexicon_tags_11 },
    { 0x353733ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x295733ae, SUC_lexicon_tags_17 },
    { 0xb9f733af, SUC_lexicon_tags_12 },
    { 0xafb733b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f733b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1733bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231733c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x54b733c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3733c5, SUC_lexicon_tags_45 },
    { 0xc5b733c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3733c9, SUC_lexicon_tags_45 },
    { 0x18f733ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddb733cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x151733ce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbe3733d0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x883733d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7733d5, SUC_lexicon_tags_11 },
    { 0xbb7733d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab733db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3733e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4ed733e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x93b733e7, SUC_lexicon_tags_11 },
    { 0xcd9733e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b733ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x297733f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9733f6, SUC_lexicon_tags_234 },
    { 0x489733f7, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5733fe, SUC_lexicon_tags_25 },
    { 0x1ab733ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59573401, SUC_lexicon_tags_2 },
    { 0x69b73401, SUC_lexicon_tags_25 },
    { 0x85b73401, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b73409, SUC_lexicon_tags_15 },
    { 0x56b73409, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57340e, SUC_lexicon_tags_50 },
    { 0xa957340f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2773418, SUC_lexicon_tags_0 },
    { 0xf3373419, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7297341f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2773425, SUC_lexicon_tags_7 },
    { 0x65173426, SUC_lexicon_tags_7 },
    { 0xf9b73427, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d17342b, SUC_lexicon_tags_55 },
    { 0xaf57342c, SUC_lexicon_tags_2 },
    { 0x4e57342d, SUC_lexicon_tags_25 },
    { 0x3017342d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8773430, SUC_lexicon_tags_15 },
    { 0xe3173431, SUC_lexicon_tags_15 },
    { 0x36b73431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e573434, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f373436, SUC_lexicon_tags_15 },
    { 0x83173437, SUC_lexicon_tags_7 },
    { 0x26773436, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa317343d, SUC_lexicon_tags_7 },
    { 0xb7b7343e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96573442, SUC_lexicon_tags_19 },
    { 0xc4973442, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab973445, SUC_lexicon_tags_14 },
    { 0xc3773446, SUC_lexicon_tags_12 },
    { 0x88573447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad7344a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d17344d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f773453, SUC_lexicon_tags_17 },
    { 0x74f73453, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5173459, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7477345c, SUC_lexicon_tags_1 },
    { 0x1597345c, SUC_lexicon_tags_60 },
    { 0x9c77345e, SUC_lexicon_tags_37 },
    { 0xb297345e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8f173461, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e773465, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32573469, SUC_lexicon_tags_1 },
    { 0xcb17346a, SUC_lexicon_tags_624 },
    { 0xf417346b, SUC_lexicon_tags_1 },
    { 0x2d7346b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1737346e, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xb373470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d73474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63973477, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb997347b, SUC_lexicon_tags_47 },
    { 0xfa57347c, SUC_lexicon_tags_11 },
    { 0xccb7347d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5557347f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29b73481, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc773483, SUC_lexicon_tags_27 },
    { 0x4c573484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8477348a, SUC_lexicon_tags_17 },
    { 0x62b7348b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95173491, SUC_lexicon_tags_26 },
    { 0x21b73492, SUC_lexicon_tags_37 },
    { 0xd4973493, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ff73495, SUC_lexicon_tags_7 },
    { 0x53373496, SUC_lexicon_tags_25 },
    { 0x47973496, SUC_lexicon_tags_1 },
    { 0x1a973495, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f77349a, SUC_lexicon_tags_2 },
    { 0xe197349b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f7349f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f734a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1734a8, SUC_lexicon_tags_60 },
    { 0xb21734a9, SUC_lexicon_tags_1 },
    { 0x49d734aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f734ae, SUC_lexicon_tags_37 },
    { 0xd1f734ae, SUC_lexicon_tags_27 },
    { 0x771734b0, SUC_lexicon_tags_28 },
    { 0xeff734b0, SUC_lexicon_tags_29 },
    { 0xd67734af, SUC_lexicon_tags_25 },
    { 0x6cd734b3, SUC_lexicon_tags_19 },
    { 0x793734b1, SUC_lexicon_tags_15 },
    { 0x18f734ae, SUC_lexicon_tags_7 },
    { 0x65f734b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9734c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x295734c3, SUC_lexicon_tags_23 },
    { 0x295734c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69734c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe99734c9, SUC_lexicon_tags_23 },
    { 0xa4d734c9, SUC_lexicon_tags_60 },
    { 0xa9734cb, SUC_lexicon_tags_27 },
    { 0xee5734cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe27734ce, SUC_lexicon_tags_7 },
    { 0xc21734ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131734d3, SUC_lexicon_tags_16 },
    { 0x315734d4, SUC_lexicon_tags_11 },
    { 0x19b734d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5734df, SUC_lexicon_tags_11 },
    { 0x56b734df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab734e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b1734e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee1734e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x837734ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x81734ec, SUC_lexicon_tags_61 },
    { 0xd4d734ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad734f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x971734f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f734fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f734fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x78173501, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e973504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b73507, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb3b73509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9bf7350b, SUC_lexicon_tags_37 },
    { 0xd297350b, SUC_lexicon_tags_80 },
    { 0x1a37350d, SUC_lexicon_tags_12 },
    { 0x6617350e, SUC_lexicon_tags_77 },
    { 0xf4f7350f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x72373512, SUC_lexicon_tags_1 },
    { 0x4973513, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xad373516, SUC_lexicon_tags_17 },
    { 0x6a173516, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb7351b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xca17351e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc973520, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef73523, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc2b73525, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef7352a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf457352d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c973534, SUC_lexicon_tags_17 },
    { 0x8f573534, SUC_lexicon_tags_2 },
    { 0x32173536, SUC_lexicon_tags_9 },
    { 0x71d73537, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1c573539, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f7353c, SUC_lexicon_tags_319 },
    { 0, NULL },
    { 0x1e77353e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73541, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48373546, SUC_lexicon_tags_25 },
    { 0x42d73547, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xced7354a, SUC_lexicon_tags_10 },
    { 0xdcf7354b, SUC_lexicon_tags_37 },
    { 0x9c17354c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44d7354e, SUC_lexicon_tags_12 },
    { 0x15b7354f, SUC_lexicon_tags_25 },
    { 0x46173550, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x44973552, SUC_lexicon_tags_4 },
    { 0x3b773553, SUC_lexicon_tags_43 },
    { 0x65973552, SUC_lexicon_tags_33 },
    { 0x81773555, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x36573557, SUC_lexicon_tags_47 },
    { 0xf4373558, SUC_lexicon_tags_19 },
    { 0x27773558, SUC_lexicon_tags_16 },
    { 0xb597355a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb77355c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9597355f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff7356a, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb73574, SUC_lexicon_tags_19 },
    { 0x3fd73574, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7357b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d7357e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8173587, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b7358e, SUC_lexicon_tags_37 },
    { 0xf317358e, SUC_lexicon_tags_6 },
    { 0x74973590, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb73593, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3f73595, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38773597, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8373599, SUC_lexicon_tags_21 },
    { 0xe597359a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x68f7359c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0f7359e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581735a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x82b735a6, SUC_lexicon_tags_15 },
    { 0x663735a7, SUC_lexicon_tags_26 },
    { 0xd1f735a8, SUC_lexicon_tags_1 },
    { 0xe79735a7, SUC_lexicon_tags_25 },
    { 0x2ad735a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d735ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe33735b0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9f7735b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75b735b4, SUC_lexicon_tags_11 },
    { 0x9bf735b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d735c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1735c3, SUC_lexicon_tags_37 },
    { 0xdb735c4, SUC_lexicon_tags_23 },
    { 0x64b735c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59735c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x859735cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97735cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6f735d1, SUC_lexicon_tags_23 },
    { 0xed3735d2, SUC_lexicon_tags_8 },
    { 0xf83735d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x815735d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadf735d7, SUC_lexicon_tags_33 },
    { 0x93f735d8, SUC_lexicon_tags_37 },
    { 0x7b5735d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbad735db, SUC_lexicon_tags_148 },
    { 0xcf9735dc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd5d735de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7735e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79735e8, SUC_lexicon_tags_11 },
    { 0x109735e9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5735f0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7735f3, SUC_lexicon_tags_12 },
    { 0x28d735f4, SUC_lexicon_tags_11 },
    { 0x7a9735f3, SUC_lexicon_tags_246 },
    { 0xb61735f6, SUC_lexicon_tags_3 },
    { 0x7e1735f7, SUC_lexicon_tags_16 },
    { 0xe2d735f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1735fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x679735fe, SUC_lexicon_tags_27 },
    { 0xbad735ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b73605, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x2b573607, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1837360a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd7360d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea97360f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x53b73611, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0573614, SUC_lexicon_tags_28 },
    { 0x17773615, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f773619, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe217361c, SUC_lexicon_tags_19 },
    { 0xa577361d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad73626, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x62173628, SUC_lexicon_tags_1 },
    { 0xfdf73629, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef773632, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd5f73634, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f973636, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89573639, SUC_lexicon_tags_16 },
    { 0xd137363a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa817363e, SUC_lexicon_tags_12 },
    { 0x777363f, SUC_lexicon_tags_12 },
    { 0xfe17363f, SUC_lexicon_tags_11 },
    { 0x7d73641, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26973647, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9457364a, SUC_lexicon_tags_9 },
    { 0x9b37364a, SUC_lexicon_tags_17 },
    { 0x9017364b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf137364e, SUC_lexicon_tags_37 },
    { 0x47f7364f, SUC_lexicon_tags_37 },
    { 0x2037364f, SUC_lexicon_tags_37 },
    { 0x17d73651, SUC_lexicon_tags_37 },
    { 0x96373652, SUC_lexicon_tags_37 },
    { 0xa2b73652, SUC_lexicon_tags_2 },
    { 0x5797364e, SUC_lexicon_tags_12 },
    { 0xc1b73655, SUC_lexicon_tags_19 },
    { 0x45773656, SUC_lexicon_tags_25 },
    { 0xaf373656, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c17365a, SUC_lexicon_tags_23 },
    { 0x8617365b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a573667, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ad73669, SUC_lexicon_tags_12 },
    { 0xa5173669, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a57366c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32973671, SUC_lexicon_tags_167 },
    { 0xfab73672, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4373677, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa3373679, SUC_lexicon_tags_38 },
    { 0xba97367a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6357368e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xda573690, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b73696, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35173698, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd337369c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd736a3, SUC_lexicon_tags_15 },
    { 0x387736a4, SUC_lexicon_tags_45 },
    { 0x6b7736a5, SUC_lexicon_tags_27 },
    { 0xaa9736a6, SUC_lexicon_tags_31 },
    { 0xbf9736a6, SUC_lexicon_tags_10 },
    { 0xfd1736a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x903736ab, SUC_lexicon_tags_7 },
    { 0xc19736ac, SUC_lexicon_tags_17 },
    { 0x8fd736ad, SUC_lexicon_tags_29 },
    { 0xdab736ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x109736b0, SUC_lexicon_tags_2 },
    { 0xcdf736b1, SUC_lexicon_tags_1 },
    { 0xb75736b1, SUC_lexicon_tags_30 },
    { 0x3c9736b1, SUC_lexicon_tags_53 },
    { 0x6dd736b4, SUC_lexicon_tags_17 },
    { 0xdd9736b5, SUC_lexicon_tags_4 },
    { 0x6a3736b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb45736b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b5736ba, SUC_lexicon_tags_37 },
    { 0x5ff736bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b736c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x433736cb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x349736cd, SUC_lexicon_tags_29 },
    { 0x2b736ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505736d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad736da, SUC_lexicon_tags_27 },
    { 0x8f736db, SUC_lexicon_tags_2 },
    { 0xe03736db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537736e0, SUC_lexicon_tags_25 },
    { 0x85f736e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5736e4, SUC_lexicon_tags_25 },
    { 0x293736e5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d736e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x983736ea, SUC_lexicon_tags_26 },
    { 0xc5736ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc3736ed, SUC_lexicon_tags_2 },
    { 0xd53736ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307736f6, SUC_lexicon_tags_26 },
    { 0xe07736f7, SUC_lexicon_tags_14 },
    { 0x3f1736f7, SUC_lexicon_tags_77 },
    { 0x8c7736f9, SUC_lexicon_tags_7 },
    { 0xf3b736f9, SUC_lexicon_tags_10 },
    { 0xd2b736f8, SUC_lexicon_tags_28 },
    { 0x49736fc, SUC_lexicon_tags_47 },
    { 0x42b736fa, SUC_lexicon_tags_49 },
    { 0xce3736fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe9773700, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1df73702, SUC_lexicon_tags_1 },
    { 0x80f73703, SUC_lexicon_tags_7 },
    { 0x70b73704, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d73706, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30f73708, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a17370c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3373710, SUC_lexicon_tags_61 },
    { 0x7ed73710, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f73714, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x173716, SUC_lexicon_tags_47 },
    { 0xbbd73716, SUC_lexicon_tags_15 },
    { 0xc2f73718, SUC_lexicon_tags_16 },
    { 0xc1173718, SUC_lexicon_tags_47 },
    { 0x7ef73717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5eb7371c, SUC_lexicon_tags_7 },
    { 0xbf97371d, SUC_lexicon_tags_21 },
    { 0xb837371e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9d73720, SUC_lexicon_tags_37 },
    { 0x57b73721, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66773726, SUC_lexicon_tags_29 },
    { 0x94173727, SUC_lexicon_tags_17 },
    { 0x61173728, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6277372c, SUC_lexicon_tags_39 },
    { 0x8417372c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xca37372f, SUC_lexicon_tags_37 },
    { 0x1b97372f, SUC_lexicon_tags_0 },
    { 0xab773731, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x99973733, SUC_lexicon_tags_19 },
    { 0x2c173733, SUC_lexicon_tags_60 },
    { 0xe8973735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3773737, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1577373e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96773740, SUC_lexicon_tags_9 },
    { 0xf4d73740, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x4e373743, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf373748, SUC_lexicon_tags_37 },
    { 0xb0173749, SUC_lexicon_tags_0 },
    { 0xad17374a, SUC_lexicon_tags_39 },
    { 0x6757374a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa237374d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb73753, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b973756, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5277375a, SUC_lexicon_tags_25 },
    { 0x9dd7375a, SUC_lexicon_tags_38 },
    { 0xb717375b, SUC_lexicon_tags_25 },
    { 0xbb77375d, SUC_lexicon_tags_27 },
    { 0x54b7375b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x24973760, SUC_lexicon_tags_31 },
    { 0xaf773760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f773764, SUC_lexicon_tags_2 },
    { 0x6d773765, SUC_lexicon_tags_6 },
    { 0xa0d73765, SUC_lexicon_tags_82 },
    { 0x50b73765, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc573769, SUC_lexicon_tags_64 },
    { 0xce17376a, SUC_lexicon_tags_2 },
    { 0xbcd7376a, SUC_lexicon_tags_25 },
    { 0x9b17376a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xee97376f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7773771, SUC_lexicon_tags_6 },
    { 0x55b73771, SUC_lexicon_tags_7 },
    { 0x1e973773, SUC_lexicon_tags_8 },
    { 0x5b173771, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d7377a, SUC_lexicon_tags_55 },
    { 0xfc17377a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd7377f, SUC_lexicon_tags_37 },
    { 0xca373780, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac573782, SUC_lexicon_tags_14 },
    { 0x20b73783, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42573788, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d7378e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b573792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c373798, SUC_lexicon_tags_33 },
    { 0xc8f73798, SUC_lexicon_tags_2 },
    { 0xbc77379a, SUC_lexicon_tags_39 },
    { 0x8a573798, SUC_lexicon_tags_23 },
    { 0xa057379c, SUC_lexicon_tags_7 },
    { 0xae773798, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5737a3, SUC_lexicon_tags_47 },
    { 0x2b3737a4, SUC_lexicon_tags_0 },
    { 0x8d737a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d737a9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb737ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc03737ae, SUC_lexicon_tags_618 },
    { 0, NULL },
    { 0, NULL },
    { 0xf79737b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97737b4, SUC_lexicon_tags_13 },
    { 0xd4b737b5, SUC_lexicon_tags_2 },
    { 0x801737b6, SUC_lexicon_tags_91 },
    { 0x44d737b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5737ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7737c6, SUC_lexicon_tags_37 },
    { 0x4f1737c7, SUC_lexicon_tags_43 },
    { 0x4db737c8, SUC_lexicon_tags_29 },
    { 0x239737c9, SUC_lexicon_tags_11 },
    { 0x6b1737ca, SUC_lexicon_tags_23 },
    { 0x4eb737cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917737cf, SUC_lexicon_tags_1 },
    { 0x281737d0, SUC_lexicon_tags_12 },
    { 0x3f1737d0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xba1737d3, SUC_lexicon_tags_8 },
    { 0xebf737d4, SUC_lexicon_tags_7 },
    { 0x989737d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d3737d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed737dc, SUC_lexicon_tags_16 },
    { 0x813737dc, SUC_lexicon_tags_3 },
    { 0x6cd737dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xddb737e0, SUC_lexicon_tags_174 },
    { 0xfe5737e1, SUC_lexicon_tags_2 },
    { 0x935737e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x753737e5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265737ea, SUC_lexicon_tags_1 },
    { 0x155737eb, SUC_lexicon_tags_17 },
    { 0x95737ec, SUC_lexicon_tags_25 },
    { 0xe5b737eb, SUC_lexicon_tags_11 },
    { 0xdf5737ee, SUC_lexicon_tags_1 },
    { 0xa35737ef, SUC_lexicon_tags_37 },
    { 0x599737ef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeb1737f2, SUC_lexicon_tags_27 },
    { 0x483737f3, SUC_lexicon_tags_11 },
    { 0xaab737f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc9737f6, SUC_lexicon_tags_7 },
    { 0xf43737f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9737fa, SUC_lexicon_tags_37 },
    { 0xd53737fb, SUC_lexicon_tags_14 },
    { 0x615737fc, SUC_lexicon_tags_63 },
    { 0xe09737fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff73802, SUC_lexicon_tags_8 },
    { 0xe4d73803, SUC_lexicon_tags_56 },
    { 0x11773803, SUC_lexicon_tags_31 },
    { 0xfb373805, SUC_lexicon_tags_15 },
    { 0xb3d73803, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b7380d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d573810, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b73813, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1573815, SUC_lexicon_tags_30 },
    { 0x5fb73816, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85173819, SUC_lexicon_tags_23 },
    { 0x7e573819, SUC_lexicon_tags_11 },
    { 0xcc17381a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6877381d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91373825, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x84773827, SUC_lexicon_tags_63 },
    { 0x8ed73828, SUC_lexicon_tags_37 },
    { 0x72d73829, SUC_lexicon_tags_1 },
    { 0x24973827, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e37382e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d73834, SUC_lexicon_tags_9 },
    { 0xd9573835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf7383d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8773840, SUC_lexicon_tags_201 },
    { 0xb1d73841, SUC_lexicon_tags_43 },
    { 0x8cd73842, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7c573844, SUC_lexicon_tags_14 },
    { 0x3d773845, SUC_lexicon_tags_65 },
    { 0x67b73846, SUC_lexicon_tags_29 },
    { 0xa4173846, SUC_lexicon_tags_23 },
    { 0xd5573844, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa37384b, SUC_lexicon_tags_61 },
    { 0xf3b7384b, SUC_lexicon_tags_1 },
    { 0x9ff7384d, SUC_lexicon_tags_55 },
    { 0x1c37384e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d73854, SUC_lexicon_tags_8 },
    { 0x70573854, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc097385b, SUC_lexicon_tags_25 },
    { 0xcb17385b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf637385e, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa973861, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7b73863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9cf73865, SUC_lexicon_tags_31 },
    { 0xd6d73866, SUC_lexicon_tags_15 },
    { 0x83f73866, SUC_lexicon_tags_11 },
    { 0xe5973866, SUC_lexicon_tags_55 },
    { 0x5d973869, SUC_lexicon_tags_156 },
    { 0x1b17386a, SUC_lexicon_tags_39 },
    { 0x1137386a, SUC_lexicon_tags_47 },
    { 0x2577386c, SUC_lexicon_tags_0 },
    { 0x8817386c, SUC_lexicon_tags_47 },
    { 0x4a17386e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfd773870, SUC_lexicon_tags_21 },
    { 0x5573870, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24773877, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd37387a, SUC_lexicon_tags_25 },
    { 0x6017387a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa77387d, SUC_lexicon_tags_37 },
    { 0xc677387e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b73881, SUC_lexicon_tags_38 },
    { 0xedd73882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8b73884, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x5ad73886, SUC_lexicon_tags_17 },
    { 0xd8373887, SUC_lexicon_tags_15 },
    { 0x62373888, SUC_lexicon_tags_9 },
    { 0x1f173888, SUC_lexicon_tags_27 },
    { 0x5217388a, SUC_lexicon_tags_19 },
    { 0x64f7388b, SUC_lexicon_tags_3 },
    { 0x7937388c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7df7388e, SUC_lexicon_tags_23 },
    { 0x7637388f, SUC_lexicon_tags_23 },
    { 0x8eb73890, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdef73892, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed373896, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca973899, SUC_lexicon_tags_48 },
    { 0x36d73899, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b77389f, SUC_lexicon_tags_29 },
    { 0x7777389f, SUC_lexicon_tags_10 },
    { 0xfb7389f, SUC_lexicon_tags_14 },
    { 0x8af738a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93d738a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1738aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31738ad, SUC_lexicon_tags_12 },
    { 0xa33738ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e1738b0, SUC_lexicon_tags_424 },
    { 0x6f738b0, SUC_lexicon_tags_12 },
    { 0x46f738b2, SUC_lexicon_tags_23 },
    { 0xd31738b3, SUC_lexicon_tags_14 },
    { 0xd37738b2, SUC_lexicon_tags_6 },
    { 0xe43738b5, SUC_lexicon_tags_25 },
    { 0xc25738b6, SUC_lexicon_tags_15 },
    { 0xe91738b2, SUC_lexicon_tags_7 },
    { 0x911738b8, SUC_lexicon_tags_43 },
    { 0x897738b9, SUC_lexicon_tags_38 },
    { 0xf1b738b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79738be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09738c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9738c7, SUC_lexicon_tags_26 },
    { 0x693738c8, SUC_lexicon_tags_1 },
    { 0x465738c7, SUC_lexicon_tags_7 },
    { 0x659738ca, SUC_lexicon_tags_17 },
    { 0x6ad738cb, SUC_lexicon_tags_53 },
    { 0xf0b738ca, SUC_lexicon_tags_10 },
    { 0x9bf738ca, SUC_lexicon_tags_6 },
    { 0x9bb738ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x60f738d0, SUC_lexicon_tags_1 },
    { 0x771738d1, SUC_lexicon_tags_69 },
    { 0x501738d1, SUC_lexicon_tags_307 },
    { 0xdf1738d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xec9738d5, SUC_lexicon_tags_28 },
    { 0xd51738d6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xccd738d8, SUC_lexicon_tags_25 },
    { 0xe0b738d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13738de, SUC_lexicon_tags_84 },
    { 0x81d738df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347738e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb738ed, SUC_lexicon_tags_17 },
    { 0x7738ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03738f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71738fe, SUC_lexicon_tags_43 },
    { 0xc6f738ff, SUC_lexicon_tags_9 },
    { 0x1e573900, SUC_lexicon_tags_10 },
    { 0x335738fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbb73903, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb73908, SUC_lexicon_tags_33 },
    { 0xb9973909, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c57390b, SUC_lexicon_tags_1 },
    { 0xa7b7390b, SUC_lexicon_tags_38 },
    { 0x8497390d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93373910, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f73915, SUC_lexicon_tags_25 },
    { 0x1ed73916, SUC_lexicon_tags_15 },
    { 0x56573915, SUC_lexicon_tags_25 },
    { 0x1d373918, SUC_lexicon_tags_28 },
    { 0xaf373916, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96573920, SUC_lexicon_tags_25 },
    { 0xef373920, SUC_lexicon_tags_136 },
    { 0x8fb73922, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fb73924, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1857392a, SUC_lexicon_tags_11 },
    { 0x9477392b, SUC_lexicon_tags_38 },
    { 0xda17392c, SUC_lexicon_tags_80 },
    { 0xe417392d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe9f7392f, SUC_lexicon_tags_25 },
    { 0x84b73930, SUC_lexicon_tags_12 },
    { 0x51d73930, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4173938, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x73b7393a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b37393c, SUC_lexicon_tags_31 },
    { 0xd417393d, SUC_lexicon_tags_8 },
    { 0x3577393d, SUC_lexicon_tags_1 },
    { 0xbbf7393c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f173941, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12173945, SUC_lexicon_tags_28 },
    { 0x98573946, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdff73948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c17394c, SUC_lexicon_tags_11 },
    { 0x8037394d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd373950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c973953, SUC_lexicon_tags_47 },
    { 0xcf173953, SUC_lexicon_tags_14 },
    { 0x6af73955, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x31573958, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f7395c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b77395f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd8373961, SUC_lexicon_tags_11 },
    { 0x91f73961, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6173966, SUC_lexicon_tags_37 },
    { 0x2b73967, SUC_lexicon_tags_17 },
    { 0xa0d73966, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2597396a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f7396e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b973971, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7973975, SUC_lexicon_tags_54 },
    { 0xb3973976, SUC_lexicon_tags_63 },
    { 0xe1773977, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7397d, SUC_lexicon_tags_17 },
    { 0x68d7397e, SUC_lexicon_tags_1 },
    { 0xd257397f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b573982, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedb73984, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x92373986, SUC_lexicon_tags_37 },
    { 0xcdd73986, SUC_lexicon_tags_0 },
    { 0x20373988, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xecd7398a, SUC_lexicon_tags_12 },
    { 0xfe97398a, SUC_lexicon_tags_46 },
    { 0x3eb7398b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xadb7398e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e173990, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x4173992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26373995, SUC_lexicon_tags_7 },
    { 0x7b573995, SUC_lexicon_tags_34 },
    { 0xde973996, SUC_lexicon_tags_12 },
    { 0xf9573997, SUC_lexicon_tags_25 },
    { 0xbd73996, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f7399d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3739a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d739a4, SUC_lexicon_tags_35 },
    { 0xb03739a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f739ad, SUC_lexicon_tags_15 },
    { 0x6d9739ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea9739b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95739b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19739b7, SUC_lexicon_tags_17 },
    { 0xc9f739b7, SUC_lexicon_tags_37 },
    { 0x5c7739b9, SUC_lexicon_tags_12 },
    { 0x66f739ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9739bd, SUC_lexicon_tags_2 },
    { 0x9f5739bd, SUC_lexicon_tags_9 },
    { 0xfb739bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x169739c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x589739c4, SUC_lexicon_tags_38 },
    { 0xe1d739c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09739ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2e5739cc, SUC_lexicon_tags_33 },
    { 0xd77739cd, SUC_lexicon_tags_11 },
    { 0x45d739ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35739d3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1739d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x287739d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e3739da, SUC_lexicon_tags_17 },
    { 0x44b739db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3b739dd, SUC_lexicon_tags_43 },
    { 0x743739dd, SUC_lexicon_tags_17 },
    { 0xaf7739df, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97739e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3739ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd739ed, SUC_lexicon_tags_43 },
    { 0xc0f739ee, SUC_lexicon_tags_2 },
    { 0x687739ef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75739f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39739f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f739fe, SUC_lexicon_tags_11 },
    { 0xa59739fe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5b973a01, SUC_lexicon_tags_25 },
    { 0xa2b73a01, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7c773a04, SUC_lexicon_tags_37 },
    { 0x5173a04, SUC_lexicon_tags_27 },
    { 0x56973a06, SUC_lexicon_tags_48 },
    { 0xbcf73a07, SUC_lexicon_tags_47 },
    { 0xa3d73a08, SUC_lexicon_tags_4 },
    { 0xe8773a09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d73a10, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef73a14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7373a16, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd73a1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f73a1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7173a25, SUC_lexicon_tags_4 },
    { 0x75373a25, SUC_lexicon_tags_1 },
    { 0x5b173a27, SUC_lexicon_tags_8 },
    { 0xc7373a28, SUC_lexicon_tags_1 },
    { 0xd0d73a25, SUC_lexicon_tags_1 },
    { 0x3c373a2a, SUC_lexicon_tags_25 },
    { 0xdeb73a2a, SUC_lexicon_tags_1 },
    { 0x2d173a25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f73a32, SUC_lexicon_tags_6 },
    { 0x7a373a33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xda373a36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26573a41, SUC_lexicon_tags_8 },
    { 0x16173a41, SUC_lexicon_tags_23 },
    { 0x28f73a43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f73a47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6773a49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb373a50, SUC_lexicon_tags_38 },
    { 0xc8373a51, SUC_lexicon_tags_13 },
    { 0x6d173a52, SUC_lexicon_tags_37 },
    { 0xd8973a52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b73a56, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc9373a58, SUC_lexicon_tags_14 },
    { 0x70973a59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b73a5c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdf973a5e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x25b73a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb73a63, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a573a6b, SUC_lexicon_tags_11 },
    { 0x7af73a6c, SUC_lexicon_tags_27 },
    { 0x80973a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba373a75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b73a78, SUC_lexicon_tags_87 },
    { 0x6cf73a78, SUC_lexicon_tags_68 },
    { 0x13973a7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f73a7d, SUC_lexicon_tags_49 },
    { 0x17173a7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b73a82, SUC_lexicon_tags_37 },
    { 0x68773a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb573a86, SUC_lexicon_tags_4 },
    { 0x6db73a87, SUC_lexicon_tags_2 },
    { 0x41373a88, SUC_lexicon_tags_7 },
    { 0x18573a88, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa2973a8b, SUC_lexicon_tags_38 },
    { 0x9d73a8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa773a91, SUC_lexicon_tags_17 },
    { 0x58573a92, SUC_lexicon_tags_12 },
    { 0x7ff73a92, SUC_lexicon_tags_0 },
    { 0x91f73a93, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22573a98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd73a9e, SUC_lexicon_tags_80 },
    { 0x71b73a9e, SUC_lexicon_tags_17 },
    { 0xa1b73a9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb73aa5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f73aa8, SUC_lexicon_tags_27 },
    { 0x23d73aa9, SUC_lexicon_tags_7 },
    { 0xe9573aa8, SUC_lexicon_tags_13 },
    { 0x1bf73aab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab73ab1, SUC_lexicon_tags_23 },
    { 0x26b73ab1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52173abb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41973abd, SUC_lexicon_tags_48 },
    { 0x9c173abe, SUC_lexicon_tags_37 },
    { 0x7e973abd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8973ac2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf8173ac4, SUC_lexicon_tags_38 },
    { 0xa5173ac5, SUC_lexicon_tags_29 },
    { 0x96373ac4, SUC_lexicon_tags_47 },
    { 0xff73ac6, SUC_lexicon_tags_12 },
    { 0xa6b73ac8, SUC_lexicon_tags_1 },
    { 0xef773ac8, SUC_lexicon_tags_17 },
    { 0xf4d73ac9, SUC_lexicon_tags_21 },
    { 0x91373acb, SUC_lexicon_tags_14 },
    { 0xfd173acc, SUC_lexicon_tags_25 },
    { 0x67373acd, SUC_lexicon_tags_25 },
    { 0xe6d73ac8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed73ad3, SUC_lexicon_tags_33 },
    { 0x27d73ad4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb373ad6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x67973ad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72373ae3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0b73ae5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55373ae7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f573ae9, SUC_lexicon_tags_31 },
    { 0x8a173aea, SUC_lexicon_tags_11 },
    { 0xd5573ae9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b73af1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7b73af3, SUC_lexicon_tags_69 },
    { 0x63b73af3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd73af7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f73afc, SUC_lexicon_tags_10 },
    { 0xea973afd, SUC_lexicon_tags_1 },
    { 0x4cf73afc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b773b00, SUC_lexicon_tags_2 },
    { 0x42373b01, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x64173b03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d973b07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd73b0d, SUC_lexicon_tags_76 },
    { 0x82173b0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82b73b10, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x54173b13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2373b16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x19573b18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70f73b1a, SUC_lexicon_tags_61 },
    { 0x5c373b1b, SUC_lexicon_tags_38 },
    { 0xba773b1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9f73b1e, SUC_lexicon_tags_37 },
    { 0x42d73b1e, SUC_lexicon_tags_14 },
    { 0xefb73b1f, SUC_lexicon_tags_27 },
    { 0x3c373b20, SUC_lexicon_tags_8 },
    { 0xd0b73b22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7173b28, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xde773b2a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc573b2e, SUC_lexicon_tags_15 },
    { 0x82773b2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f73b3b, SUC_lexicon_tags_1 },
    { 0xdc773b3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d73b3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77b73b40, SUC_lexicon_tags_2 },
    { 0xf4573b41, SUC_lexicon_tags_13 },
    { 0x89573b41, SUC_lexicon_tags_13 },
    { 0x1d173b43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68d73b45, SUC_lexicon_tags_25 },
    { 0x48773b45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d73b4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29773b51, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd73b54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94773b59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0773b5b, SUC_lexicon_tags_7 },
    { 0x77373b5b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3373b60, SUC_lexicon_tags_33 },
    { 0x4b573b61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5d73b63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f373b65, SUC_lexicon_tags_17 },
    { 0x40973b65, SUC_lexicon_tags_17 },
    { 0x51773b67, SUC_lexicon_tags_4 },
    { 0xed73b68, SUC_lexicon_tags_0 },
    { 0xfa173b65, SUC_lexicon_tags_27 },
    { 0x3a773b67, SUC_lexicon_tags_2 },
    { 0x2d773b6b, SUC_lexicon_tags_12 },
    { 0xc3b73b69, SUC_lexicon_tags_2 },
    { 0xa2973b6d, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x8a973b6f, SUC_lexicon_tags_15 },
    { 0xeab73b70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d773b72, SUC_lexicon_tags_27 },
    { 0xc0173b73, SUC_lexicon_tags_80 },
    { 0x2bb73b74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab73b78, SUC_lexicon_tags_37 },
    { 0xf3773b79, SUC_lexicon_tags_201 },
    { 0xe2973b78, SUC_lexicon_tags_1 },
    { 0xf2973b7a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d73b84, SUC_lexicon_tags_6 },
    { 0x25b73b85, SUC_lexicon_tags_25 },
    { 0x6c173b86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3973b8a, SUC_lexicon_tags_2 },
    { 0x7f373b8b, SUC_lexicon_tags_17 },
    { 0xec173b8b, SUC_lexicon_tags_57 },
    { 0x17b73b8b, SUC_lexicon_tags_37 },
    { 0xbf73b8e, SUC_lexicon_tags_25 },
    { 0x3e173b8e, SUC_lexicon_tags_19 },
    { 0xe6573b8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1973b94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80d73b96, SUC_lexicon_tags_138 },
    { 0xd5573b96, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5373ba0, SUC_lexicon_tags_3 },
    { 0x46d73ba1, SUC_lexicon_tags_54 },
    { 0x49773ba2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad73bac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f73bb3, SUC_lexicon_tags_11 },
    { 0xc8973bb4, SUC_lexicon_tags_27 },
    { 0x65d73bb4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3373bbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9d73bbd, SUC_lexicon_tags_29 },
    { 0x8d973bbd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9cd73bc0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xab973bc2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf73bc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b73bca, SUC_lexicon_tags_16 },
    { 0x2dd73bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf73bcd, SUC_lexicon_tags_11 },
    { 0xadf73bcd, SUC_lexicon_tags_5 },
    { 0xcbd73bcd, SUC_lexicon_tags_15 },
    { 0x3f973bcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f773bdb, SUC_lexicon_tags_1 },
    { 0xdc373bdc, SUC_lexicon_tags_28 },
    { 0x1a173bdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f73bdf, SUC_lexicon_tags_9 },
    { 0x12d73be0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x74b73be2, SUC_lexicon_tags_1 },
    { 0x3ed73be3, SUC_lexicon_tags_11 },
    { 0x48f73be4, SUC_lexicon_tags_11 },
    { 0xccf73be4, SUC_lexicon_tags_47 },
    { 0xf1773be6, SUC_lexicon_tags_2 },
    { 0x9973be6, SUC_lexicon_tags_45 },
    { 0x99173be8, SUC_lexicon_tags_61 },
    { 0x29573be9, SUC_lexicon_tags_15 },
    { 0x59773be8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef73bee, SUC_lexicon_tags_1 },
    { 0x7b773bee, SUC_lexicon_tags_30 },
    { 0x81d73bf0, SUC_lexicon_tags_2 },
    { 0xd1573bee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42773bf6, SUC_lexicon_tags_17 },
    { 0x5c373bf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x40573bf9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d73bfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b973c05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd73c08, SUC_lexicon_tags_25 },
    { 0x67b73c08, SUC_lexicon_tags_28 },
    { 0xb4173c0a, SUC_lexicon_tags_37 },
    { 0xabf73c0b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfc773c0d, SUC_lexicon_tags_26 },
    { 0x39373c0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51573c11, SUC_lexicon_tags_19 },
    { 0x92b73c12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4173c14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75973c16, SUC_lexicon_tags_7 },
    { 0xbc373c17, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9eb73c19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46173c1d, SUC_lexicon_tags_27 },
    { 0x6df73c1e, SUC_lexicon_tags_37 },
    { 0x10f73c1e, SUC_lexicon_tags_10 },
    { 0xb1373c1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d73c23, SUC_lexicon_tags_37 },
    { 0x8a373c24, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x43f73c26, SUC_lexicon_tags_9 },
    { 0x5a573c26, SUC_lexicon_tags_17 },
    { 0x9c973c28, SUC_lexicon_tags_49 },
    { 0x5bd73c29, SUC_lexicon_tags_48 },
    { 0x38373c2a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x7bb73c2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2973c2f, SUC_lexicon_tags_11 },
    { 0x81773c2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1773c34, SUC_lexicon_tags_45 },
    { 0x92373c35, SUC_lexicon_tags_65 },
    { 0x1af73c36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49973c38, SUC_lexicon_tags_2 },
    { 0xafd73c39, SUC_lexicon_tags_8 },
    { 0x61573c3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x58173c3d, SUC_lexicon_tags_25 },
    { 0x18f73c3d, SUC_lexicon_tags_12 },
    { 0xb0373c3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64573c41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4173c43, SUC_lexicon_tags_25 },
    { 0x84d73c44, SUC_lexicon_tags_37 },
    { 0x1f73c45, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe8173c47, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55373c4c, SUC_lexicon_tags_7 },
    { 0x64f73c4d, SUC_lexicon_tags_26 },
    { 0xc9f73c4d, SUC_lexicon_tags_19 },
    { 0x6bf73c4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3173c52, SUC_lexicon_tags_2 },
    { 0x46973c52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x56573c56, SUC_lexicon_tags_379 },
    { 0x98773c57, SUC_lexicon_tags_6 },
    { 0xa2b73c56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d73c5b, SUC_lexicon_tags_17 },
    { 0x3c573c5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9173c60, SUC_lexicon_tags_1 },
    { 0x17573c61, SUC_lexicon_tags_25 },
    { 0xc4b73c62, SUC_lexicon_tags_16 },
    { 0x9573c63, SUC_lexicon_tags_54 },
    { 0x8b573c64, SUC_lexicon_tags_37 },
    { 0x38573c65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d73c69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd6373c6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3173c70, SUC_lexicon_tags_46 },
    { 0xb3973c70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf973c74, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f73c7d, SUC_lexicon_tags_38 },
    { 0x94d73c7e, SUC_lexicon_tags_45 },
    { 0xf0373c7e, SUC_lexicon_tags_4 },
    { 0x68773c80, SUC_lexicon_tags_11 },
    { 0xbb73c81, SUC_lexicon_tags_25 },
    { 0x12573c82, SUC_lexicon_tags_32 },
    { 0xe3973c7d, SUC_lexicon_tags_68 },
    { 0x10373c84, SUC_lexicon_tags_2 },
    { 0x1a573c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b73c89, SUC_lexicon_tags_9 },
    { 0x1af73c8a, SUC_lexicon_tags_37 },
    { 0xc3173c8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb773c8e, SUC_lexicon_tags_37 },
    { 0xe4573c8e, SUC_lexicon_tags_11 },
    { 0x87773c90, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xeb973c92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d73c95, SUC_lexicon_tags_16 },
    { 0x8f173c96, SUC_lexicon_tags_14 },
    { 0x67d73c97, SUC_lexicon_tags_2 },
    { 0x67373c98, SUC_lexicon_tags_7 },
    { 0xc3573c98, SUC_lexicon_tags_38 },
    { 0x6f173c95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf73c9d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xffb73c9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd173ca2, SUC_lexicon_tags_33 },
    { 0xc5573ca3, SUC_lexicon_tags_1 },
    { 0x84f73ca4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f73ca8, SUC_lexicon_tags_74 },
    { 0x9b773ca9, SUC_lexicon_tags_29 },
    { 0x76973ca9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8573cac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ab73cae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b73cb1, SUC_lexicon_tags_37 },
    { 0x4c573cb2, SUC_lexicon_tags_17 },
    { 0xecb73cb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43173cbe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0373cc0, SUC_lexicon_tags_47 },
    { 0x47573cc1, SUC_lexicon_tags_38 },
    { 0x9e773cc1, SUC_lexicon_tags_12 },
    { 0x87b73cc3, SUC_lexicon_tags_15 },
    { 0xd6d73cc4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x50b73cc6, SUC_lexicon_tags_37 },
    { 0xd1973cc7, SUC_lexicon_tags_2 },
    { 0xd3573cc8, SUC_lexicon_tags_77 },
    { 0x5db73cc9, SUC_lexicon_tags_47 },
    { 0x71373cc9, SUC_lexicon_tags_252 },
    { 0xb7b73cca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x22b73ccd, SUC_lexicon_tags_16 },
    { 0xae973cce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b73cd3, SUC_lexicon_tags_8 },
    { 0x7773cd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d73cd9, SUC_lexicon_tags_17 },
    { 0xe6d73cda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d73ce3, SUC_lexicon_tags_17 },
    { 0xf7b73ce4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d73ce8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaef73cea, SUC_lexicon_tags_63 },
    { 0x4d773ceb, SUC_lexicon_tags_35 },
    { 0xdb173cec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66373cf0, SUC_lexicon_tags_55 },
    { 0xe0b73cf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb8773cf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb73cf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d73cfb, SUC_lexicon_tags_37 },
    { 0xd0173cfc, SUC_lexicon_tags_39 },
    { 0xc4d73cfc, SUC_lexicon_tags_25 },
    { 0x63d73cfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb973d00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b73d03, SUC_lexicon_tags_45 },
    { 0x31973d04, SUC_lexicon_tags_2 },
    { 0xef973d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b773d09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99773d0f, SUC_lexicon_tags_7 },
    { 0xb5573d10, SUC_lexicon_tags_38 },
    { 0x53373d10, SUC_lexicon_tags_1 },
    { 0x9d173d12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e373d14, SUC_lexicon_tags_9 },
    { 0xae173d15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd73d1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a173d20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f73d2b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e973d2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb73d31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b73d3a, SUC_lexicon_tags_25 },
    { 0x47773d3b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef73d40, SUC_lexicon_tags_37 },
    { 0xc5b73d40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x55b73d43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfd573d45, SUC_lexicon_tags_17 },
    { 0x59f73d46, SUC_lexicon_tags_23 },
    { 0x23173d47, SUC_lexicon_tags_12 },
    { 0x53b73d46, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xcf173d4a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b73d4d, SUC_lexicon_tags_17 },
    { 0x5b173d4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c373d51, SUC_lexicon_tags_11 },
    { 0xac573d52, SUC_lexicon_tags_28 },
    { 0x8a773d53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d73d58, SUC_lexicon_tags_37 },
    { 0xd5d73d58, SUC_lexicon_tags_38 },
    { 0x8a973d59, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x573d5c, SUC_lexicon_tags_1 },
    { 0x94573d5c, SUC_lexicon_tags_23 },
    { 0x3173d5d, SUC_lexicon_tags_65 },
    { 0x93973d5f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbbf73d61, SUC_lexicon_tags_7 },
    { 0x9973d61, SUC_lexicon_tags_25 },
    { 0x45d73d61, SUC_lexicon_tags_28 },
    { 0x9bf73d64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47373d66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd573d6c, SUC_lexicon_tags_1 },
    { 0xfc173d6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82d73d6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b73d75, SUC_lexicon_tags_17 },
    { 0x1c973d75, SUC_lexicon_tags_34 },
    { 0x87f73d75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef73d7c, SUC_lexicon_tags_33 },
    { 0xafd73d7d, SUC_lexicon_tags_2 },
    { 0xd0973d7e, SUC_lexicon_tags_17 },
    { 0xf4573d7e, SUC_lexicon_tags_65 },
    { 0xdcd73d7f, SUC_lexicon_tags_25 },
    { 0x19f73d7c, SUC_lexicon_tags_37 },
    { 0xb6173d81, SUC_lexicon_tags_45 },
    { 0x14373d83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d73d85, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a573d8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1a373d8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39773d91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a773d95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7773d9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d573da1, SUC_lexicon_tags_23 },
    { 0x60f73da2, SUC_lexicon_tags_9 },
    { 0xa2173da2, SUC_lexicon_tags_17 },
    { 0xc2573da3, SUC_lexicon_tags_87 },
    { 0x98973da5, SUC_lexicon_tags_38 },
    { 0xc5573da6, SUC_lexicon_tags_14 },
    { 0x93f73da7, SUC_lexicon_tags_1 },
    { 0x9773da8, SUC_lexicon_tags_75 },
    { 0xb7d73da7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf73dad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45773db5, SUC_lexicon_tags_11 },
    { 0xdcf73db6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8773db8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af73dbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9f973dbe, SUC_lexicon_tags_9 },
    { 0xf4773dbe, SUC_lexicon_tags_33 },
    { 0xf4173dc0, SUC_lexicon_tags_33 },
    { 0x60b73dbe, SUC_lexicon_tags_27 },
    { 0x56173dc1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef73dc6, SUC_lexicon_tags_26 },
    { 0x78d73dc7, SUC_lexicon_tags_26 },
    { 0x51d73dc8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1573dcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f73dce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20373dd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac973dd5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25573dd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb8973ddb, SUC_lexicon_tags_49 },
    { 0xb4773ddc, SUC_lexicon_tags_14 },
    { 0x1ed73ddb, SUC_lexicon_tags_25 },
    { 0x68773dde, SUC_lexicon_tags_15 },
    { 0xa6b73dde, SUC_lexicon_tags_37 },
    { 0xa8f73dde, SUC_lexicon_tags_7 },
    { 0xe4373dde, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79373de3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x53773de5, SUC_lexicon_tags_108 },
    { 0x47773de6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4dd73de8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36773dea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d73df1, SUC_lexicon_tags_17 },
    { 0xed973df2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x49373df4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x81173df7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd73dfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26173dfe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x95173e00, SUC_lexicon_tags_7 },
    { 0xe4f73e00, SUC_lexicon_tags_11 },
    { 0x42173e01, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6573e08, SUC_lexicon_tags_25 },
    { 0x63f73e09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14373e0b, SUC_lexicon_tags_10 },
    { 0x9fb73e0c, SUC_lexicon_tags_12 },
    { 0xdd773e0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b73e15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd2173e17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82373e1a, SUC_lexicon_tags_23 },
    { 0x42173e1b, SUC_lexicon_tags_30 },
    { 0xe4573e1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc1173e1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d73e21, SUC_lexicon_tags_37 },
    { 0xc5573e21, SUC_lexicon_tags_17 },
    { 0xd973e23, SUC_lexicon_tags_37 },
    { 0x27373e24, SUC_lexicon_tags_12 },
    { 0xbf973e25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab73e28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44b73e2a, SUC_lexicon_tags_37 },
    { 0x6d373e2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48173e2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7573e30, SUC_lexicon_tags_66 },
    { 0x86b73e30, SUC_lexicon_tags_17 },
    { 0xd3173e31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8773e3b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x68573e3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb973e41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x15173e44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51973e4d, SUC_lexicon_tags_7 },
    { 0x15b73e4e, SUC_lexicon_tags_11 },
    { 0xe4d73e4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58773e55, SUC_lexicon_tags_15 },
    { 0x96573e56, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3373e58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3573e5b, SUC_lexicon_tags_2 },
    { 0xb0b73e5c, SUC_lexicon_tags_12 },
    { 0xb5973e5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b373e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2773e65, SUC_lexicon_tags_15 },
    { 0x1f773e65, SUC_lexicon_tags_11 },
    { 0x4ad73e67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0173e69, SUC_lexicon_tags_38 },
    { 0x28f73e6a, SUC_lexicon_tags_43 },
    { 0xe7373e6b, SUC_lexicon_tags_25 },
    { 0x30d73e69, SUC_lexicon_tags_2 },
    { 0xb0173e6b, SUC_lexicon_tags_65 },
    { 0x93573e6a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3773e71, SUC_lexicon_tags_27 },
    { 0x6c173e71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29573e76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4173e7c, SUC_lexicon_tags_0 },
    { 0x5e973e7c, SUC_lexicon_tags_0 },
    { 0x88d73e7d, SUC_lexicon_tags_16 },
    { 0xe8373e7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x76973e81, SUC_lexicon_tags_2 },
    { 0x4b573e81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c373e84, SUC_lexicon_tags_11 },
    { 0x96f73e84, SUC_lexicon_tags_32 },
    { 0x63373e84, SUC_lexicon_tags_49 },
    { 0x53373e87, SUC_lexicon_tags_108 },
    { 0xf1373e88, SUC_lexicon_tags_47 },
    { 0xee973e89, SUC_lexicon_tags_1 },
    { 0x32573e89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc373e8c, SUC_lexicon_tags_27 },
    { 0x96173e8d, SUC_lexicon_tags_26 },
    { 0x29573e8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b73e92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24773e97, SUC_lexicon_tags_2 },
    { 0x48173e98, SUC_lexicon_tags_12 },
    { 0x76773e98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea573e9e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff73ea1, SUC_lexicon_tags_33 },
    { 0xb373ea2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7573ea5, SUC_lexicon_tags_63 },
    { 0x83373ea6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21373eab, SUC_lexicon_tags_12 },
    { 0x2cd73eab, SUC_lexicon_tags_9 },
    { 0x3ef73ead, SUC_lexicon_tags_37 },
    { 0x6f573eae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbc973eb0, SUC_lexicon_tags_31 },
    { 0x7a973eb1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e773eb4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5973eb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af73ec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d73ec5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4573ec8, SUC_lexicon_tags_10 },
    { 0x6b773ec9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73b73ecb, SUC_lexicon_tags_35 },
    { 0x68573ecc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8373ece, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa573ed2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd73ed5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x75973ed7, SUC_lexicon_tags_11 },
    { 0x8e573ed8, SUC_lexicon_tags_16 },
    { 0xe0b73ed9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad73edd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd73ee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96973ee6, SUC_lexicon_tags_37 },
    { 0xab73ee7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xad973eea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd373eec, SUC_lexicon_tags_133 },
    { 0x14b73eed, SUC_lexicon_tags_84 },
    { 0xe0373eed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb73ef1, SUC_lexicon_tags_15 },
    { 0x60773ef2, SUC_lexicon_tags_15 },
    { 0xd6373ef3, SUC_lexicon_tags_47 },
    { 0x56b73ef4, SUC_lexicon_tags_84 },
    { 0x8b973ef3, SUC_lexicon_tags_7 },
    { 0x54f73ef6, SUC_lexicon_tags_38 },
    { 0x61d73ef7, SUC_lexicon_tags_33 },
    { 0xdfd73ef7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8f173efa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d173f00, SUC_lexicon_tags_15 },
    { 0x9f773f01, SUC_lexicon_tags_14 },
    { 0xc8373f01, SUC_lexicon_tags_30 },
    { 0x69373f03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38773f09, SUC_lexicon_tags_21 },
    { 0x54573f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30d73f0c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd9573f0e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f73f13, SUC_lexicon_tags_27 },
    { 0xea173f14, SUC_lexicon_tags_2 },
    { 0xbcd73f15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x41173f17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d73f1a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd7f73f1c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb73f22, SUC_lexicon_tags_19 },
    { 0x45373f23, SUC_lexicon_tags_43 },
    { 0xd8b73f23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef973f29, SUC_lexicon_tags_14 },
    { 0x30973f29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef773f30, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x3373f32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc173f36, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9fb73f38, SUC_lexicon_tags_17 },
    { 0xf173f39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead73f3e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b73f42, SUC_lexicon_tags_62 },
    { 0x51373f43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99973f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1973f4a, SUC_lexicon_tags_33 },
    { 0x12f73f4b, SUC_lexicon_tags_35 },
    { 0x60973f4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd973f4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0573f51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c773f53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84573f5c, SUC_lexicon_tags_11 },
    { 0x6f73f5d, SUC_lexicon_tags_37 },
    { 0xd4973f5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b73f65, SUC_lexicon_tags_11 },
    { 0xc173f66, SUC_lexicon_tags_45 },
    { 0x83373f67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54973f6a, SUC_lexicon_tags_14 },
    { 0xd573f6b, SUC_lexicon_tags_33 },
    { 0x2d173f6c, SUC_lexicon_tags_0 },
    { 0xe9373f6c, SUC_lexicon_tags_17 },
    { 0x8f173f6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xded73f70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6173f7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d73f7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1af73f80, SUC_lexicon_tags_0 },
    { 0x35773f81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44173f85, SUC_lexicon_tags_11 },
    { 0x63373f86, SUC_lexicon_tags_25 },
    { 0x5df73f86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6373f8c, SUC_lexicon_tags_38 },
    { 0xc7d73f8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1573f97, SUC_lexicon_tags_1 },
    { 0x69373f98, SUC_lexicon_tags_37 },
    { 0x9cf73f97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5973f9d, SUC_lexicon_tags_506 },
    { 0xebb73f9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xca573fa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91173fa3, SUC_lexicon_tags_25 },
    { 0x6973fa4, SUC_lexicon_tags_8 },
    { 0x63973fa5, SUC_lexicon_tags_14 },
    { 0x25f73fa3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d73faa, SUC_lexicon_tags_26 },
    { 0x55373fab, SUC_lexicon_tags_11 },
    { 0x63573fac, SUC_lexicon_tags_8 },
    { 0x7d73fad, SUC_lexicon_tags_49 },
    { 0xa8373fad, SUC_lexicon_tags_12 },
    { 0xb773fae, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f73fb7, SUC_lexicon_tags_12 },
    { 0x18373fb7, SUC_lexicon_tags_19 },
    { 0x66373fb9, SUC_lexicon_tags_25 },
    { 0x59f73fb9, SUC_lexicon_tags_33 },
    { 0x8f773fba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3b73fbd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef73fc1, SUC_lexicon_tags_49 },
    { 0xb9373fc2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d73fc5, SUC_lexicon_tags_8 },
    { 0x6b373fc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f73fcc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x11973fcf, SUC_lexicon_tags_39 },
    { 0xe8f73fd0, SUC_lexicon_tags_0 },
    { 0xba573fd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3973fd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c973fd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43373fde, SUC_lexicon_tags_17 },
    { 0x72173fdf, SUC_lexicon_tags_42 },
    { 0x43373fdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8573fe4, SUC_lexicon_tags_25 },
    { 0xc9173fe5, SUC_lexicon_tags_65 },
    { 0xaf973fe5, SUC_lexicon_tags_12 },
    { 0x4bf73fe7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdf573fe9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1e773feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12373ff0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d73ff3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31773ff7, SUC_lexicon_tags_25 },
    { 0xca173ff7, SUC_lexicon_tags_9 },
    { 0xc6373ff9, SUC_lexicon_tags_1 },
    { 0x33973ffa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf2173ffc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3574000, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d74003, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x21374005, SUC_lexicon_tags_19 },
    { 0xd4174006, SUC_lexicon_tags_19 },
    { 0xa1b74007, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb17400d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbbb7400f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4eb74011, SUC_lexicon_tags_25 },
    { 0xe0174012, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef74015, SUC_lexicon_tags_37 },
    { 0x47d74016, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30774018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff77401d, SUC_lexicon_tags_34 },
    { 0xbb17401e, SUC_lexicon_tags_1 },
    { 0xa997401f, SUC_lexicon_tags_25 },
    { 0x3ab74020, SUC_lexicon_tags_37 },
    { 0xb7f74021, SUC_lexicon_tags_1 },
    { 0x7f77401d, SUC_lexicon_tags_25 },
    { 0x6ad74023, SUC_lexicon_tags_85 },
    { 0xc2974021, SUC_lexicon_tags_17 },
    { 0x8a974021, SUC_lexicon_tags_38 },
    { 0xa3374026, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfef74028, SUC_lexicon_tags_47 },
    { 0x97f74028, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa737402d, SUC_lexicon_tags_23 },
    { 0x14f7402e, SUC_lexicon_tags_9 },
    { 0x6e57402f, SUC_lexicon_tags_1 },
    { 0xbf374030, SUC_lexicon_tags_37 },
    { 0xc9774031, SUC_lexicon_tags_1 },
    { 0x21d74030, SUC_lexicon_tags_11 },
    { 0x84174033, SUC_lexicon_tags_55 },
    { 0x48774030, SUC_lexicon_tags_12 },
    { 0x36d7402d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f7403c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15b7403e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd74041, SUC_lexicon_tags_63 },
    { 0xf7774041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d74044, SUC_lexicon_tags_11 },
    { 0x38d74045, SUC_lexicon_tags_0 },
    { 0x39574046, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3ff74048, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f97404d, SUC_lexicon_tags_11 },
    { 0xa057404d, SUC_lexicon_tags_17 },
    { 0xba37404d, SUC_lexicon_tags_0 },
    { 0xb2b7404d, SUC_lexicon_tags_37 },
    { 0xcc57404f, SUC_lexicon_tags_38 },
    { 0x2db7404e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3774059, SUC_lexicon_tags_46 },
    { 0xe337405a, SUC_lexicon_tags_35 },
    { 0xa1f7405b, SUC_lexicon_tags_38 },
    { 0x8397405b, SUC_lexicon_tags_29 },
    { 0x3017405d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d574065, SUC_lexicon_tags_87 },
    { 0x9d74066, SUC_lexicon_tags_47 },
    { 0x8ff74067, SUC_lexicon_tags_15 },
    { 0x10574067, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f7406b, SUC_lexicon_tags_8 },
    { 0x90b7406c, SUC_lexicon_tags_17 },
    { 0xf177406d, SUC_lexicon_tags_4 },
    { 0x8817406d, SUC_lexicon_tags_10 },
    { 0xc6f7406b, SUC_lexicon_tags_29 },
    { 0xeb774070, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x774073, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x73774075, SUC_lexicon_tags_23 },
    { 0x95974076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd037407b, SUC_lexicon_tags_12 },
    { 0x4057407b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcbd7407e, SUC_lexicon_tags_15 },
    { 0xa117407f, SUC_lexicon_tags_197 },
    { 0x5af74080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d74083, SUC_lexicon_tags_33 },
    { 0xe6574084, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfc974086, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c97408c, SUC_lexicon_tags_19 },
    { 0x6657408d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb317408f, SUC_lexicon_tags_17 },
    { 0xe3d74090, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22374094, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df74097, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759740a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33740a5, SUC_lexicon_tags_15 },
    { 0x3ed740a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5740a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x967740ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x899740ae, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39740b2, SUC_lexicon_tags_7 },
    { 0xccf740b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d740b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6a9740b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93740bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x225740be, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91740c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb740c6, SUC_lexicon_tags_13 },
    { 0xd47740c7, SUC_lexicon_tags_14 },
    { 0x3e3740c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec3740ca, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb740cd, SUC_lexicon_tags_8 },
    { 0x787740cd, SUC_lexicon_tags_17 },
    { 0x19740cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75d740d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131740d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9740dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x623740df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b740e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd53740e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f740ea, SUC_lexicon_tags_37 },
    { 0x69740eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b740f1, SUC_lexicon_tags_79 },
    { 0xa9f740f2, SUC_lexicon_tags_1 },
    { 0x3e7740f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf740fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdb740fc, SUC_lexicon_tags_25 },
    { 0x75f740fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5f740ff, SUC_lexicon_tags_11 },
    { 0x435740ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db74103, SUC_lexicon_tags_15 },
    { 0xa0f74103, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xed174107, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf7410a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf6f7410c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3774111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d74117, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb6774119, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa857411b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8117411f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9174125, SUC_lexicon_tags_11 },
    { 0x5f974125, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2897412f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb74132, SUC_lexicon_tags_59 },
    { 0xf174132, SUC_lexicon_tags_15 },
    { 0x85774134, SUC_lexicon_tags_11 },
    { 0x30974135, SUC_lexicon_tags_37 },
    { 0x2cf74136, SUC_lexicon_tags_17 },
    { 0xa0174137, SUC_lexicon_tags_14 },
    { 0x18d74138, SUC_lexicon_tags_27 },
    { 0x30f74139, SUC_lexicon_tags_12 },
    { 0x85d7413a, SUC_lexicon_tags_16 },
    { 0x51774139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28774140, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f74144, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f74147, SUC_lexicon_tags_23 },
    { 0x12574148, SUC_lexicon_tags_11 },
    { 0xeff74149, SUC_lexicon_tags_34 },
    { 0xcf374149, SUC_lexicon_tags_27 },
    { 0xc4174148, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf74153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5974156, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x87174159, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1657415d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62974163, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf74168, SUC_lexicon_tags_13 },
    { 0x8e374169, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5737416c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9974171, SUC_lexicon_tags_7 },
    { 0x8f174172, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed7417c, SUC_lexicon_tags_27 },
    { 0xad17417d, SUC_lexicon_tags_19 },
    { 0x4e17417c, SUC_lexicon_tags_6 },
    { 0x8a57417f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x75574181, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50974186, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf77418a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57418d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f74190, SUC_lexicon_tags_11 },
    { 0x25574190, SUC_lexicon_tags_7 },
    { 0x43174192, SUC_lexicon_tags_7 },
    { 0x87d74191, SUC_lexicon_tags_11 },
    { 0x49174193, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda37419b, SUC_lexicon_tags_0 },
    { 0x94f7419b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd677419e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73b741a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x467741a2, SUC_lexicon_tags_17 },
    { 0xb55741a3, SUC_lexicon_tags_33 },
    { 0xea5741a4, SUC_lexicon_tags_23 },
    { 0xda5741a2, SUC_lexicon_tags_28 },
    { 0xa89741a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2db741a8, SUC_lexicon_tags_2 },
    { 0x9d1741a9, SUC_lexicon_tags_1 },
    { 0x6c7741a9, SUC_lexicon_tags_37 },
    { 0xa69741aa, SUC_lexicon_tags_0 },
    { 0x163741ac, SUC_lexicon_tags_32 },
    { 0x75d741ad, SUC_lexicon_tags_21 },
    { 0xa7741a9, SUC_lexicon_tags_25 },
    { 0x8f7741ae, SUC_lexicon_tags_27 },
    { 0x3f3741b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x57d741b2, SUC_lexicon_tags_15 },
    { 0xa97741b3, SUC_lexicon_tags_12 },
    { 0x859741b4, SUC_lexicon_tags_3 },
    { 0xd05741b5, SUC_lexicon_tags_7 },
    { 0x183741b3, SUC_lexicon_tags_30 },
    { 0x891741b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667741bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b741c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b741c5, SUC_lexicon_tags_0 },
    { 0x70b741c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7741c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85741cc, SUC_lexicon_tags_63 },
    { 0xb7f741cd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7741d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6fb741d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143741d6, SUC_lexicon_tags_1 },
    { 0xcf5741d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e7741d9, SUC_lexicon_tags_7 },
    { 0xaab741d9, SUC_lexicon_tags_1604 },
    { 0xce1741db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35741df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x293741e2, SUC_lexicon_tags_37 },
    { 0xd37741e3, SUC_lexicon_tags_15 },
    { 0xac9741e4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdd5741e6, SUC_lexicon_tags_2 },
    { 0x933741e6, SUC_lexicon_tags_22 },
    { 0x17f741e8, SUC_lexicon_tags_30 },
    { 0x333741e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x66f741eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7741ef, SUC_lexicon_tags_5 },
    { 0xf7b741f0, SUC_lexicon_tags_37 },
    { 0x8d7741f1, SUC_lexicon_tags_13 },
    { 0x19d741f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x419741f4, SUC_lexicon_tags_60 },
    { 0xf57741f5, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df741fa, SUC_lexicon_tags_25 },
    { 0xa3f741fb, SUC_lexicon_tags_60 },
    { 0xe99741fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ef741fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93974201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88974203, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x76774206, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8097420a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab7420d, SUC_lexicon_tags_14 },
    { 0x2877420e, SUC_lexicon_tags_11 },
    { 0xb037420d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb0374211, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb74214, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3e374216, SUC_lexicon_tags_29 },
    { 0x24174217, SUC_lexicon_tags_79 },
    { 0xc8774218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd77421c, SUC_lexicon_tags_2 },
    { 0x4e77421c, SUC_lexicon_tags_31 },
    { 0x44d7421e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd74221, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x59974224, SUC_lexicon_tags_37 },
    { 0x36574225, SUC_lexicon_tags_12 },
    { 0xa5574226, SUC_lexicon_tags_7 },
    { 0x19974227, SUC_lexicon_tags_55 },
    { 0x98374228, SUC_lexicon_tags_26 },
    { 0xc7774229, SUC_lexicon_tags_2 },
    { 0xc517422a, SUC_lexicon_tags_1 },
    { 0xceb7422a, SUC_lexicon_tags_19 },
    { 0x30b74228, SUC_lexicon_tags_14 },
    { 0xdd374226, SUC_lexicon_tags_2 },
    { 0xdd37422e, SUC_lexicon_tags_38 },
    { 0x9bd74224, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42374231, SUC_lexicon_tags_45 },
    { 0xedb74232, SUC_lexicon_tags_17 },
    { 0xaa974233, SUC_lexicon_tags_17 },
    { 0x86174234, SUC_lexicon_tags_35 },
    { 0xfed74235, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e57423a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ad7423c, SUC_lexicon_tags_1 },
    { 0x2fb7423c, SUC_lexicon_tags_25 },
    { 0x2a97423e, SUC_lexicon_tags_25 },
    { 0xcab7423f, SUC_lexicon_tags_11 },
    { 0xf6b74240, SUC_lexicon_tags_18 },
    { 0x46774240, SUC_lexicon_tags_1 },
    { 0x82174242, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f974247, SUC_lexicon_tags_15 },
    { 0xd4374247, SUC_lexicon_tags_91 },
    { 0xa4d74249, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad74252, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0d74254, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x81f74256, SUC_lexicon_tags_15 },
    { 0xa7f74257, SUC_lexicon_tags_15 },
    { 0xcb974258, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d7425d, SUC_lexicon_tags_28 },
    { 0xe97425e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x46174261, SUC_lexicon_tags_28 },
    { 0xa8f74262, SUC_lexicon_tags_19 },
    { 0xa9974263, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50574267, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc57426a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc597426c, SUC_lexicon_tags_17 },
    { 0x4617426d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89374270, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b74275, SUC_lexicon_tags_23 },
    { 0x7f74276, SUC_lexicon_tags_2 },
    { 0x81f74277, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd7427e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf74282, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa9d74284, SUC_lexicon_tags_7 },
    { 0x7bd74285, SUC_lexicon_tags_14 },
    { 0xacd74286, SUC_lexicon_tags_38 },
    { 0xe0974286, SUC_lexicon_tags_37 },
    { 0x1d574287, SUC_lexicon_tags_7 },
    { 0xa9374289, SUC_lexicon_tags_34 },
    { 0x8477428a, SUC_lexicon_tags_23 },
    { 0x2337428b, SUC_lexicon_tags_2 },
    { 0xfc77428c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf57428e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb74291, SUC_lexicon_tags_65 },
    { 0x30d74291, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7bb74294, SUC_lexicon_tags_10 },
    { 0xa3174295, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26374297, SUC_lexicon_tags_30 },
    { 0xd5f74298, SUC_lexicon_tags_14 },
    { 0x55374299, SUC_lexicon_tags_17 },
    { 0xc7b7429a, SUC_lexicon_tags_70 },
    { 0x76b74298, SUC_lexicon_tags_12 },
    { 0x9a37429b, SUC_lexicon_tags_33 },
    { 0xe597429d, SUC_lexicon_tags_27 },
    { 0x30f74297, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b742a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9742aa, SUC_lexicon_tags_79 },
    { 0x6f1742ab, SUC_lexicon_tags_11 },
    { 0x85b742ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x575742ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f742b2, SUC_lexicon_tags_7 },
    { 0xcd9742b3, SUC_lexicon_tags_37 },
    { 0xee7742b4, SUC_lexicon_tags_45 },
    { 0xc41742b5, SUC_lexicon_tags_57 },
    { 0x8a7742b6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed742b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3742bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f9742be, SUC_lexicon_tags_17 },
    { 0xff9742bf, SUC_lexicon_tags_15 },
    { 0xc0f742bf, SUC_lexicon_tags_12 },
    { 0xfff742c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce9742c3, SUC_lexicon_tags_17 },
    { 0x455742c4, SUC_lexicon_tags_17 },
    { 0x293742c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9742ca, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xadf742cc, SUC_lexicon_tags_37 },
    { 0xc7742cc, SUC_lexicon_tags_27 },
    { 0xd9742ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f742d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495742d9, SUC_lexicon_tags_11 },
    { 0x429742da, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d742dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf742eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3742ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe1742f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x673742f2, SUC_lexicon_tags_16 },
    { 0x363742f3, SUC_lexicon_tags_5 },
    { 0x5cb742f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59742fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe73742fd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b74300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b74305, SUC_lexicon_tags_66 },
    { 0xc5374305, SUC_lexicon_tags_39 },
    { 0x5cd74307, SUC_lexicon_tags_15 },
    { 0x2eb74308, SUC_lexicon_tags_0 },
    { 0xf6d74309, SUC_lexicon_tags_100 },
    { 0x90b74307, SUC_lexicon_tags_8 },
    { 0xc4774305, SUC_lexicon_tags_60 },
    { 0x2af7430c, SUC_lexicon_tags_37 },
    { 0xa8f7430c, SUC_lexicon_tags_37 },
    { 0x8697430e, SUC_lexicon_tags_37 },
    { 0xaab7430f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16b74311, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6774315, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d7431a, SUC_lexicon_tags_7 },
    { 0xc837431b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeab7431d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f74320, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xccf74322, SUC_lexicon_tags_70 },
    { 0x2d174323, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda974325, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f574328, SUC_lexicon_tags_9 },
    { 0x1bd74328, SUC_lexicon_tags_12 },
    { 0x3774328, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b974330, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x86374333, SUC_lexicon_tags_8 },
    { 0xdf574334, SUC_lexicon_tags_12 },
    { 0xb2f74334, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc777433b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5977433d, SUC_lexicon_tags_2 },
    { 0x1257433e, SUC_lexicon_tags_19 },
    { 0x26d7433f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea774341, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d74344, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x40574346, SUC_lexicon_tags_45 },
    { 0x10f74347, SUC_lexicon_tags_11 },
    { 0xf0b74348, SUC_lexicon_tags_0 },
    { 0x87574347, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf77434d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1937435c, SUC_lexicon_tags_2 },
    { 0xceb7435d, SUC_lexicon_tags_47 },
    { 0xaf77435e, SUC_lexicon_tags_66 },
    { 0x917435d, SUC_lexicon_tags_37 },
    { 0x3c374360, SUC_lexicon_tags_15 },
    { 0x4e374360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81374363, SUC_lexicon_tags_1 },
    { 0x37774364, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb37436c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b7436f, SUC_lexicon_tags_11 },
    { 0xca574370, SUC_lexicon_tags_37 },
    { 0x44974371, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x80974373, SUC_lexicon_tags_12 },
    { 0xc6574374, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b774376, SUC_lexicon_tags_34 },
    { 0x97974377, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd137437b, SUC_lexicon_tags_30 },
    { 0x7ad7437c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8174380, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4374383, SUC_lexicon_tags_37 },
    { 0x97374384, SUC_lexicon_tags_11 },
    { 0x95b74384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa174387, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b7438a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f74391, SUC_lexicon_tags_14 },
    { 0x7ab74392, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x21374395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda17439c, SUC_lexicon_tags_10 },
    { 0x3df7439d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47f7439f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5743a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3743a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9743b0, SUC_lexicon_tags_1 },
    { 0x43743b0, SUC_lexicon_tags_29 },
    { 0x197743b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1743ba, SUC_lexicon_tags_17 },
    { 0x3e9743ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d743be, SUC_lexicon_tags_11 },
    { 0x145743be, SUC_lexicon_tags_10 },
    { 0x109743c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893743c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x539743c9, SUC_lexicon_tags_146 },
    { 0xb1743ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1743cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21743e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd743e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d743e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9743ef, SUC_lexicon_tags_14 },
    { 0x6f7743ef, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3743f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe31743f7, SUC_lexicon_tags_10 },
    { 0x829743f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d743fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7f5743fe, SUC_lexicon_tags_37 },
    { 0xe8b743fe, SUC_lexicon_tags_87 },
    { 0x55d743fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d74404, SUC_lexicon_tags_12 },
    { 0x7c774405, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd74408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x737440b, SUC_lexicon_tags_63 },
    { 0xb897440c, SUC_lexicon_tags_17 },
    { 0x1257440b, SUC_lexicon_tags_105 },
    { 0x7cb7440e, SUC_lexicon_tags_1 },
    { 0xa297440f, SUC_lexicon_tags_7 },
    { 0x61574410, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xadd74412, SUC_lexicon_tags_7 },
    { 0x38174412, SUC_lexicon_tags_63 },
    { 0xb374413, SUC_lexicon_tags_9 },
    { 0xee574415, SUC_lexicon_tags_37 },
    { 0x2c174415, SUC_lexicon_tags_38 },
    { 0xbe774417, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x397441a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb7d7441c, SUC_lexicon_tags_14 },
    { 0xfa57441d, SUC_lexicon_tags_1 },
    { 0x2a97441e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b174421, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc7974423, SUC_lexicon_tags_14 },
    { 0x53974424, SUC_lexicon_tags_37 },
    { 0xf8d74424, SUC_lexicon_tags_33 },
    { 0xa9974426, SUC_lexicon_tags_27 },
    { 0xd4d74423, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d7442c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa977442e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54374432, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97374435, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54d74437, SUC_lexicon_tags_65 },
    { 0x79b74437, SUC_lexicon_tags_15 },
    { 0x54b74439, SUC_lexicon_tags_68 },
    { 0x7f77443a, SUC_lexicon_tags_2 },
    { 0xcc37443b, SUC_lexicon_tags_25 },
    { 0xb1b7443b, SUC_lexicon_tags_17 },
    { 0x82b74437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e574445, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e374448, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c57444f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b74452, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71d74454, SUC_lexicon_tags_1 },
    { 0x7c174454, SUC_lexicon_tags_17 },
    { 0xbb774456, SUC_lexicon_tags_25 },
    { 0xb4374457, SUC_lexicon_tags_15 },
    { 0x50974458, SUC_lexicon_tags_29 },
    { 0xdd374459, SUC_lexicon_tags_61 },
    { 0x9937445a, SUC_lexicon_tags_1 },
    { 0xcc37445a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd257445d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde17445f, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0x7b174461, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f774464, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2574467, SUC_lexicon_tags_3 },
    { 0x98974468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde97446d, SUC_lexicon_tags_63 },
    { 0x6a57446e, SUC_lexicon_tags_23 },
    { 0xfa97446f, SUC_lexicon_tags_11 },
    { 0x33574470, SUC_lexicon_tags_1 },
    { 0x6017446d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff974477, SUC_lexicon_tags_254 },
    { 0xecf74478, SUC_lexicon_tags_0 },
    { 0xd1f74479, SUC_lexicon_tags_15 },
    { 0xe917447a, SUC_lexicon_tags_21 },
    { 0x23974479, SUC_lexicon_tags_25 },
    { 0xeb77447c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef7447f, SUC_lexicon_tags_0 },
    { 0x2e374480, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d774484, SUC_lexicon_tags_34 },
    { 0xc6574484, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x43574487, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc2574489, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd74491, SUC_lexicon_tags_37 },
    { 0x84d74492, SUC_lexicon_tags_8 },
    { 0x4f74492, SUC_lexicon_tags_25 },
    { 0x4b574491, SUC_lexicon_tags_47 },
    { 0xfad74495, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6374497, SUC_lexicon_tags_0 },
    { 0x81174497, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b7449b, SUC_lexicon_tags_23 },
    { 0x8617449b, SUC_lexicon_tags_60 },
    { 0x26d7449d, SUC_lexicon_tags_17 },
    { 0xb3d7449d, SUC_lexicon_tags_15 },
    { 0x297449f, SUC_lexicon_tags_31 },
    { 0xaab7449c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05744a3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7744a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f744ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1d744b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f744b5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7744b8, SUC_lexicon_tags_27 },
    { 0xc83744b8, SUC_lexicon_tags_12 },
    { 0xc0f744ba, SUC_lexicon_tags_38 },
    { 0x92b744bb, SUC_lexicon_tags_49 },
    { 0x1a1744bb, SUC_lexicon_tags_25 },
    { 0x687744bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf1744bf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5744c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3744c9, SUC_lexicon_tags_33 },
    { 0x1b5744c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d744cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5744d3, SUC_lexicon_tags_47 },
    { 0x931744d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x541744d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb744e0, SUC_lexicon_tags_7 },
    { 0xe31744e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1744e5, SUC_lexicon_tags_2 },
    { 0x50d744e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59744ec, SUC_lexicon_tags_0 },
    { 0xc5f744ed, SUC_lexicon_tags_12 },
    { 0xd13744ec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11744f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad9744f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9744fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f74502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af7450a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd7450d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad74510, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb6774512, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36974514, SUC_lexicon_tags_11 },
    { 0xfb774515, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93574517, SUC_lexicon_tags_11 },
    { 0x1a774518, SUC_lexicon_tags_1 },
    { 0x3a574517, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4277451d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8774521, SUC_lexicon_tags_27 },
    { 0x10774522, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe857452b, SUC_lexicon_tags_2 },
    { 0x70f7452c, SUC_lexicon_tags_37 },
    { 0x2d97452c, SUC_lexicon_tags_63 },
    { 0x6497452e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x78774531, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x14174534, SUC_lexicon_tags_11 },
    { 0xf74535, SUC_lexicon_tags_19 },
    { 0x7a374536, SUC_lexicon_tags_2 },
    { 0xbe574537, SUC_lexicon_tags_27 },
    { 0x1cf74537, SUC_lexicon_tags_17 },
    { 0x31974539, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eb7453b, SUC_lexicon_tags_45 },
    { 0x3ff7453c, SUC_lexicon_tags_15 },
    { 0x1db7453d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x82374540, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5574543, SUC_lexicon_tags_21 },
    { 0x6f174544, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d374549, SUC_lexicon_tags_37 },
    { 0x3517454a, SUC_lexicon_tags_34 },
    { 0xc97454b, SUC_lexicon_tags_63 },
    { 0x7cd7454c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd774553, SUC_lexicon_tags_16 },
    { 0x9574554, SUC_lexicon_tags_15 },
    { 0xc3974554, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa177455a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e77455f, SUC_lexicon_tags_33 },
    { 0x11f7455f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x99f74562, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd877456a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa97456e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b74573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19d74575, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb7457a, SUC_lexicon_tags_26 },
    { 0xebd7457b, SUC_lexicon_tags_32 },
    { 0xbe57457b, SUC_lexicon_tags_19 },
    { 0x3ed7457b, SUC_lexicon_tags_12 },
    { 0x1677457e, SUC_lexicon_tags_38 },
    { 0x117457f, SUC_lexicon_tags_9 },
    { 0xb374580, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad74583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x47174586, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x73f74588, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0x7b97458a, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xa5b7458c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b74591, SUC_lexicon_tags_1 },
    { 0xaf974592, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe174595, SUC_lexicon_tags_1 },
    { 0x82974596, SUC_lexicon_tags_1 },
    { 0xaf574595, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xac57459a, SUC_lexicon_tags_17 },
    { 0x26f7459b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7745a1, SUC_lexicon_tags_54 },
    { 0xf85745a2, SUC_lexicon_tags_28 },
    { 0xccd745a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299745a8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f745ab, SUC_lexicon_tags_1 },
    { 0xc57745ab, SUC_lexicon_tags_2 },
    { 0x331745ad, SUC_lexicon_tags_11 },
    { 0x4ed745ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad5745b0, SUC_lexicon_tags_31 },
    { 0xb3d745b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e5745b3, SUC_lexicon_tags_12 },
    { 0x9eb745b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d745c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb47745c2, SUC_lexicon_tags_77 },
    { 0xe69745c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88b745c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b745c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb745d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb745da, SUC_lexicon_tags_23 },
    { 0xdbf745db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ad745dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x177745e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39f745e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15745ea, SUC_lexicon_tags_12 },
    { 0x83b745ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1745ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69f745f0, SUC_lexicon_tags_15 },
    { 0x741745f1, SUC_lexicon_tags_17 },
    { 0xab3745f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8a5745f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x341745f7, SUC_lexicon_tags_38 },
    { 0x781745f8, SUC_lexicon_tags_49 },
    { 0x6a5745f8, SUC_lexicon_tags_45 },
    { 0x5c7745fa, SUC_lexicon_tags_31 },
    { 0xb6f745f8, SUC_lexicon_tags_8 },
    { 0xa69745fb, SUC_lexicon_tags_25 },
    { 0xd17745f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff74601, SUC_lexicon_tags_72 },
    { 0xe1174602, SUC_lexicon_tags_38 },
    { 0x6b574603, SUC_lexicon_tags_37 },
    { 0x1a374604, SUC_lexicon_tags_17 },
    { 0x55d74602, SUC_lexicon_tags_23 },
    { 0xe8d74606, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf477460a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xc757460d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f74610, SUC_lexicon_tags_25 },
    { 0x2e774611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2974620, SUC_lexicon_tags_0 },
    { 0x26574621, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab74625, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdbb74627, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7817462c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd7462f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xfcf74631, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24574634, SUC_lexicon_tags_11 },
    { 0x9eb74635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41d74637, SUC_lexicon_tags_14 },
    { 0xc2574638, SUC_lexicon_tags_14 },
    { 0x29774637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f7463f, SUC_lexicon_tags_69 },
    { 0xd2174640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef74646, SUC_lexicon_tags_1 },
    { 0xae174647, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7174649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b7464c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6237464f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0374651, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x56774654, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xae974656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7465a, SUC_lexicon_tags_37 },
    { 0x1a17465a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xded7465d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24374662, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d374665, SUC_lexicon_tags_2 },
    { 0x4e374666, SUC_lexicon_tags_27 },
    { 0x1f174667, SUC_lexicon_tags_31 },
    { 0x58974668, SUC_lexicon_tags_33 },
    { 0xdab74669, SUC_lexicon_tags_23 },
    { 0x21374669, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd74670, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe174677, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8df74679, SUC_lexicon_tags_48 },
    { 0xc717467a, SUC_lexicon_tags_8 },
    { 0x25d7467a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97467e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f574683, SUC_lexicon_tags_15 },
    { 0x3a974684, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2cf74686, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc77468d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1774696, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x72f74698, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c77469a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf57469c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881746a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477746a8, SUC_lexicon_tags_19 },
    { 0xb23746a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9746ac, SUC_lexicon_tags_590 },
    { 0, NULL },
    { 0x313746ae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61746b1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x42d746b3, SUC_lexicon_tags_45 },
    { 0x5a3746b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc47746b6, SUC_lexicon_tags_4 },
    { 0xb19746b6, SUC_lexicon_tags_37 },
    { 0x237746b6, SUC_lexicon_tags_61 },
    { 0x69f746b9, SUC_lexicon_tags_14 },
    { 0x109746ba, SUC_lexicon_tags_21 },
    { 0xa7f746b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923746c5, SUC_lexicon_tags_61 },
    { 0x99b746c5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b746cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1746d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3746dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x279746de, SUC_lexicon_tags_14 },
    { 0x49f746df, SUC_lexicon_tags_15 },
    { 0xfdd746e0, SUC_lexicon_tags_252 },
    { 0x3ff746de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x26b746e3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29746ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d5746ef, SUC_lexicon_tags_7 },
    { 0x173746f0, SUC_lexicon_tags_29 },
    { 0xd9b746f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3b1746f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd6d746f5, SUC_lexicon_tags_1 },
    { 0x383746f5, SUC_lexicon_tags_61 },
    { 0xfc1746f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x415746f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22f746fb, SUC_lexicon_tags_12 },
    { 0x443746fc, SUC_lexicon_tags_17 },
    { 0x461746fd, SUC_lexicon_tags_1 },
    { 0x1d5746fe, SUC_lexicon_tags_1 },
    { 0x38d746ff, SUC_lexicon_tags_12 },
    { 0x8fb746fc, SUC_lexicon_tags_60 },
    { 0x1bd746fd, SUC_lexicon_tags_11 },
    { 0x4a374702, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6174704, SUC_lexicon_tags_47 },
    { 0x46574705, SUC_lexicon_tags_26 },
    { 0x54b74704, SUC_lexicon_tags_49 },
    { 0x24574707, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b74710, SUC_lexicon_tags_25 },
    { 0xa7574711, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86174713, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2574715, SUC_lexicon_tags_60 },
    { 0xca974716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d7471a, SUC_lexicon_tags_23 },
    { 0x6f77471a, SUC_lexicon_tags_160 },
    { 0xd877471c, SUC_lexicon_tags_4 },
    { 0x7337471c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43b7471f, SUC_lexicon_tags_31 },
    { 0x4e574720, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d74722, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0974725, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb174727, SUC_lexicon_tags_47 },
    { 0xea774728, SUC_lexicon_tags_32 },
    { 0xcc774728, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b7472d, SUC_lexicon_tags_1 },
    { 0xbdf7472d, SUC_lexicon_tags_11 },
    { 0x5277472f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf74733, SUC_lexicon_tags_5 },
    { 0xad774733, SUC_lexicon_tags_14 },
    { 0x13374735, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf774738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe617473a, SUC_lexicon_tags_2 },
    { 0xf5d7473b, SUC_lexicon_tags_8 },
    { 0xd9f7473b, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x757473e, SUC_lexicon_tags_31 },
    { 0x7c17473f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7374741, SUC_lexicon_tags_15 },
    { 0xe1574741, SUC_lexicon_tags_2 },
    { 0xcc574743, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54974749, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f7474c, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x5317474e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe7374750, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d74753, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7374755, SUC_lexicon_tags_62 },
    { 0xc3774756, SUC_lexicon_tags_56 },
    { 0x99974756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d7475b, SUC_lexicon_tags_17 },
    { 0xf7f7475c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d7475e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59974765, SUC_lexicon_tags_31 },
    { 0xd0b74765, SUC_lexicon_tags_37 },
    { 0x27574767, SUC_lexicon_tags_68 },
    { 0x27d74765, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d97476c, SUC_lexicon_tags_37 },
    { 0x457476c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb574770, SUC_lexicon_tags_3 },
    { 0x26d74771, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db74774, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd4574776, SUC_lexicon_tags_136 },
    { 0xe3974777, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed7477a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5117477c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d97477f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c574783, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6174786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f7478b, SUC_lexicon_tags_2 },
    { 0x737478c, SUC_lexicon_tags_61 },
    { 0x5a97478d, SUC_lexicon_tags_14 },
    { 0x93f7478b, SUC_lexicon_tags_17 },
    { 0xc337478d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d74795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ef74797, SUC_lexicon_tags_17 },
    { 0x74574798, SUC_lexicon_tags_17 },
    { 0x45b74799, SUC_lexicon_tags_17 },
    { 0x6f574799, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10f7479c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d7479f, SUC_lexicon_tags_15 },
    { 0xb597479f, SUC_lexicon_tags_899 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f747a5, SUC_lexicon_tags_0 },
    { 0xf2f747a6, SUC_lexicon_tags_23 },
    { 0x81f747a7, SUC_lexicon_tags_7 },
    { 0xc77747a7, SUC_lexicon_tags_23 },
    { 0x4d3747a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd747ae, SUC_lexicon_tags_7 },
    { 0x125747ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf97747b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb747b6, SUC_lexicon_tags_33 },
    { 0x855747b7, SUC_lexicon_tags_47 },
    { 0x1a3747b8, SUC_lexicon_tags_15 },
    { 0x817747b9, SUC_lexicon_tags_37 },
    { 0x847747b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893747c0, SUC_lexicon_tags_37 },
    { 0x9e1747c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x511747c3, SUC_lexicon_tags_35 },
    { 0x8d3747c3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x513747c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b747cb, SUC_lexicon_tags_10 },
    { 0x55747cc, SUC_lexicon_tags_7 },
    { 0xd0d747cd, SUC_lexicon_tags_15 },
    { 0xb11747cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc95747d0, SUC_lexicon_tags_16 },
    { 0xd5f747d0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1fd747d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5747d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79747db, SUC_lexicon_tags_9 },
    { 0xf03747dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91747e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47747e7, SUC_lexicon_tags_9 },
    { 0xe9b747e8, SUC_lexicon_tags_27 },
    { 0xfe7747e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3747ed, SUC_lexicon_tags_46 },
    { 0xbc5747ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b747f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31747fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75d747fc, SUC_lexicon_tags_61 },
    { 0x4b1747fd, SUC_lexicon_tags_42 },
    { 0x3f1747fd, SUC_lexicon_tags_26 },
    { 0x153747ff, SUC_lexicon_tags_0 },
    { 0x95774800, SUC_lexicon_tags_27 },
    { 0x3774800, SUC_lexicon_tags_16 },
    { 0xe3974802, SUC_lexicon_tags_55 },
    { 0xe4f747fc, SUC_lexicon_tags_17 },
    { 0xdef747ff, SUC_lexicon_tags_34 },
    { 0xf7774805, SUC_lexicon_tags_7 },
    { 0x93574805, SUC_lexicon_tags_12 },
    { 0x2f774800, SUC_lexicon_tags_7 },
    { 0x935747fc, SUC_lexicon_tags_0 },
    { 0x66574809, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7480d, SUC_lexicon_tags_25 },
    { 0x5ad7480e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d74811, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d374813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6974817, SUC_lexicon_tags_7 },
    { 0x28774817, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3577481e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16574820, SUC_lexicon_tags_45 },
    { 0xf6d74821, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa574825, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d7482c, SUC_lexicon_tags_0 },
    { 0xed17482d, SUC_lexicon_tags_1 },
    { 0xda77482c, SUC_lexicon_tags_25 },
    { 0x1217482f, SUC_lexicon_tags_8 },
    { 0xcf57482c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54574832, SUC_lexicon_tags_49 },
    { 0x5ed74833, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9174838, SUC_lexicon_tags_87 },
    { 0x3af74839, SUC_lexicon_tags_15 },
    { 0xc337483a, SUC_lexicon_tags_25 },
    { 0x92f7483a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x63f7483d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb657483f, SUC_lexicon_tags_7 },
    { 0xd1574840, SUC_lexicon_tags_12 },
    { 0xb0f74840, SUC_lexicon_tags_37 },
    { 0x7574842, SUC_lexicon_tags_7 },
    { 0xd3374843, SUC_lexicon_tags_25 },
    { 0xda574844, SUC_lexicon_tags_14 },
    { 0x4c374845, SUC_lexicon_tags_38 },
    { 0x38d74846, SUC_lexicon_tags_4 },
    { 0x57174847, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc17484b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97484d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe037484f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb374855, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf97485c, SUC_lexicon_tags_1536 },
    { 0, NULL },
    { 0x21d7485e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea174862, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a574866, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d74869, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d37486b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b74873, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb74879, SUC_lexicon_tags_12 },
    { 0xc17487a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2157487c, SUC_lexicon_tags_1 },
    { 0x4d7487d, SUC_lexicon_tags_14 },
    { 0x4197487d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e574881, SUC_lexicon_tags_17 },
    { 0x4e574882, SUC_lexicon_tags_33 },
    { 0x4bb74883, SUC_lexicon_tags_0 },
    { 0x82b74882, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3ad74886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd74889, SUC_lexicon_tags_17 },
    { 0xbb7488a, SUC_lexicon_tags_68 },
    { 0xab37488b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf7488e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e774890, SUC_lexicon_tags_7 },
    { 0x7cf74891, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d7489b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb7489f, SUC_lexicon_tags_57 },
    { 0x4cb7489f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb748a2, SUC_lexicon_tags_6 },
    { 0xc59748a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719748ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff748af, SUC_lexicon_tags_43 },
    { 0x3a3748b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcb748b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9d748b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb748ba, SUC_lexicon_tags_12 },
    { 0x4ab748ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x371748bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3748c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb47748c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x115748c4, SUC_lexicon_tags_7 },
    { 0xe11748c4, SUC_lexicon_tags_45 },
    { 0xd53748c6, SUC_lexicon_tags_15 },
    { 0x917748c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd21748c9, SUC_lexicon_tags_10 },
    { 0xe15748ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5748ce, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xfe3748d0, SUC_lexicon_tags_8 },
    { 0xb77748d1, SUC_lexicon_tags_7 },
    { 0x9af748d2, SUC_lexicon_tags_55 },
    { 0x323748d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4cd748d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef5748d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5748db, SUC_lexicon_tags_10 },
    { 0x51f748dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729748e0, SUC_lexicon_tags_111 },
    { 0xaf9748e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd748e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x301748e9, SUC_lexicon_tags_17 },
    { 0x41b748e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b748f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1748f4, SUC_lexicon_tags_17 },
    { 0x499748f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75748f9, SUC_lexicon_tags_23 },
    { 0x193748fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97574906, SUC_lexicon_tags_11 },
    { 0x9d574907, SUC_lexicon_tags_17 },
    { 0x4c774908, SUC_lexicon_tags_9 },
    { 0xe2b74909, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf97490f, SUC_lexicon_tags_43 },
    { 0x8d574910, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c174914, SUC_lexicon_tags_27 },
    { 0xa7d74915, SUC_lexicon_tags_56 },
    { 0x5c174916, SUC_lexicon_tags_7 },
    { 0x46174916, SUC_lexicon_tags_17 },
    { 0x36f74918, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2377491f, SUC_lexicon_tags_11 },
    { 0x84b7491f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76774924, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe5174926, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d774928, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8257492d, SUC_lexicon_tags_38 },
    { 0xb97492e, SUC_lexicon_tags_12 },
    { 0x6097492f, SUC_lexicon_tags_15 },
    { 0x1c97492e, SUC_lexicon_tags_25 },
    { 0x1ab74930, SUC_lexicon_tags_7 },
    { 0xa2174932, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd374935, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd4374937, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe2374939, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f7493e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdf174940, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af74945, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2774949, SUC_lexicon_tags_57 },
    { 0x3d574949, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7494e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f174952, SUC_lexicon_tags_27 },
    { 0x25b74953, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe9f74955, SUC_lexicon_tags_11 },
    { 0x3ff74956, SUC_lexicon_tags_1 },
    { 0x25d74956, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b7495c, SUC_lexicon_tags_9 },
    { 0xcef7495d, SUC_lexicon_tags_77 },
    { 0x8f7495d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8974960, SUC_lexicon_tags_63 },
    { 0xbfb74961, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e374964, SUC_lexicon_tags_23 },
    { 0xd8974965, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7496a, SUC_lexicon_tags_1 },
    { 0x9217496b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf877496d, SUC_lexicon_tags_27 },
    { 0x7bd7496d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f74972, SUC_lexicon_tags_7 },
    { 0x7a774972, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1574979, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4317497d, SUC_lexicon_tags_14 },
    { 0xe7f7497e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5774980, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe0774982, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9974988, SUC_lexicon_tags_27 },
    { 0x3c974989, SUC_lexicon_tags_25 },
    { 0x29174989, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb17498c, SUC_lexicon_tags_38 },
    { 0xf737498c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2597498f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd74995, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c974998, SUC_lexicon_tags_47 },
    { 0xe5974999, SUC_lexicon_tags_2 },
    { 0x9b574999, SUC_lexicon_tags_21 },
    { 0x6197499a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2357499e, SUC_lexicon_tags_25 },
    { 0xde97499f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbef749a1, SUC_lexicon_tags_45 },
    { 0xa0d749a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45749a4, SUC_lexicon_tags_65 },
    { 0x46b749a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d749a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f749ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af749b3, SUC_lexicon_tags_6 },
    { 0xe61749b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279749c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x581749ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d749cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1749d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcad749d5, SUC_lexicon_tags_26 },
    { 0xa9749d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f749db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f749de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d749e7, SUC_lexicon_tags_7 },
    { 0x19d749e7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7749ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x68d749ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8b5749f0, SUC_lexicon_tags_15 },
    { 0xc1d749f0, SUC_lexicon_tags_35 },
    { 0xfcb749f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed749f6, SUC_lexicon_tags_204 },
    { 0x5cd749f7, SUC_lexicon_tags_37 },
    { 0xc99749f6, SUC_lexicon_tags_37 },
    { 0xb9749f9, SUC_lexicon_tags_23 },
    { 0xab3749fa, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0x359749fc, SUC_lexicon_tags_1 },
    { 0x8df749fd, SUC_lexicon_tags_11 },
    { 0x159749fe, SUC_lexicon_tags_15 },
    { 0xf51749fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d574a04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1774a06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x41974a09, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f74a0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf974a13, SUC_lexicon_tags_6 },
    { 0xadd74a14, SUC_lexicon_tags_11 },
    { 0xdf574a14, SUC_lexicon_tags_19 },
    { 0x95d74a16, SUC_lexicon_tags_15 },
    { 0x82974a13, SUC_lexicon_tags_46 },
    { 0x57174a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed374a1e, SUC_lexicon_tags_13 },
    { 0x4bb74a1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad74a23, SUC_lexicon_tags_7 },
    { 0xf9174a24, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4774a29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f374a2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x85f74a31, SUC_lexicon_tags_70 },
    { 0xb6374a32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x34174a34, SUC_lexicon_tags_11 },
    { 0x9a174a35, SUC_lexicon_tags_1 },
    { 0xb2174a35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22574a3a, SUC_lexicon_tags_25 },
    { 0x9e174a3b, SUC_lexicon_tags_27 },
    { 0x3e974a3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f74a40, SUC_lexicon_tags_39 },
    { 0xc3d74a40, SUC_lexicon_tags_37 },
    { 0x48174a42, SUC_lexicon_tags_287 },
    { 0xcdb74a42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d774a48, SUC_lexicon_tags_8 },
    { 0x97774a49, SUC_lexicon_tags_60 },
    { 0xddd74a4a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc5374a4c, SUC_lexicon_tags_14 },
    { 0xbfb74a4d, SUC_lexicon_tags_49 },
    { 0x24574a4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c974a53, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xad74a56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf74a59, SUC_lexicon_tags_19 },
    { 0xa0974a5a, SUC_lexicon_tags_22 },
    { 0xac374a5b, SUC_lexicon_tags_11 },
    { 0x76974a5c, SUC_lexicon_tags_11 },
    { 0xac174a5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8774a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb774a63, SUC_lexicon_tags_7 },
    { 0xbe574a63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xee774a66, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1574a6a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x55974a6d, SUC_lexicon_tags_17 },
    { 0x18974a6d, SUC_lexicon_tags_30 },
    { 0x7e974a6e, SUC_lexicon_tags_47 },
    { 0xdc974a70, SUC_lexicon_tags_0 },
    { 0xb1774a6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b74a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bd74a76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce974a7d, SUC_lexicon_tags_4 },
    { 0xbdd74a7e, SUC_lexicon_tags_160 },
    { 0x9dd74a7f, SUC_lexicon_tags_38 },
    { 0x2b774a7e, SUC_lexicon_tags_28 },
    { 0x35574a81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd774a88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5974a8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56374a8c, SUC_lexicon_tags_12 },
    { 0x86b74a8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a974a90, SUC_lexicon_tags_31 },
    { 0xced74a90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3fd74a93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87574a9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d74a9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xce974aa0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90374aa3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2974aa9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc5974aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43574aae, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46974ab3, SUC_lexicon_tags_15 },
    { 0x5cb74ab4, SUC_lexicon_tags_7 },
    { 0xfdb74ab5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1574abc, SUC_lexicon_tags_16 },
    { 0x94f74abc, SUC_lexicon_tags_17 },
    { 0x3d174abe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4974ac8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1b74aca, SUC_lexicon_tags_37 },
    { 0xc7174acb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44774acd, SUC_lexicon_tags_29 },
    { 0xb1174ace, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2b974ad0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc1774ad2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaf174ad4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3ef74ad6, SUC_lexicon_tags_4 },
    { 0x77574ad7, SUC_lexicon_tags_7 },
    { 0x4fd74ad8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9974ada, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a774adf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac174ae1, SUC_lexicon_tags_17 },
    { 0x9d74ae1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f74ae5, SUC_lexicon_tags_14 },
    { 0xfad74ae5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db74aee, SUC_lexicon_tags_11 },
    { 0x48574aef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf74af6, SUC_lexicon_tags_11 },
    { 0x4f774af7, SUC_lexicon_tags_37 },
    { 0xb9574af8, SUC_lexicon_tags_46 },
    { 0xd9b74af7, SUC_lexicon_tags_3 },
    { 0x9e974afa, SUC_lexicon_tags_47 },
    { 0x7f574afb, SUC_lexicon_tags_2 },
    { 0x7e574afb, SUC_lexicon_tags_47 },
    { 0x2b374afb, SUC_lexicon_tags_15 },
    { 0x40b74af8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad74b01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c974b04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x40174b07, SUC_lexicon_tags_17 },
    { 0xf8f74b08, SUC_lexicon_tags_17 },
    { 0xf1774b08, SUC_lexicon_tags_26 },
    { 0x32f74b0a, SUC_lexicon_tags_19 },
    { 0x9c574b09, SUC_lexicon_tags_33 },
    { 0xf2374b07, SUC_lexicon_tags_12 },
    { 0x2bb74b0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f974b13, SUC_lexicon_tags_0 },
    { 0x54b74b13, SUC_lexicon_tags_26 },
    { 0x9df74b13, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f774b1c, SUC_lexicon_tags_17 },
    { 0x98174b1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7374b1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46574b21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8374b23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a774b27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7374b2b, SUC_lexicon_tags_45 },
    { 0x6574b2b, SUC_lexicon_tags_65 },
    { 0x4df74b2c, SUC_lexicon_tags_138 },
    { 0x61f74b2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79974b32, SUC_lexicon_tags_31 },
    { 0xc774b32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c174b36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13774b3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7974b41, SUC_lexicon_tags_10 },
    { 0x75b74b42, SUC_lexicon_tags_25 },
    { 0x1cf74b41, SUC_lexicon_tags_11 },
    { 0x80174b44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb574b46, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2774b49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x90174b4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x14f74b4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52974b54, SUC_lexicon_tags_62 },
    { 0xf574b55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a374b5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x37f74b5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f74b63, SUC_lexicon_tags_11 },
    { 0x11f74b63, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x66574b66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb0974b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38f74b6a, SUC_lexicon_tags_11 },
    { 0xf3574b6b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf74b70, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6b174b72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd74b77, SUC_lexicon_tags_7 },
    { 0x2e574b77, SUC_lexicon_tags_12 },
    { 0xbaf74b77, SUC_lexicon_tags_25 },
    { 0xd5774b78, SUC_lexicon_tags_14 },
    { 0x9ab74b7b, SUC_lexicon_tags_33 },
    { 0x98774b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df74b84, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39174b89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd74b90, SUC_lexicon_tags_3 },
    { 0xc3774b91, SUC_lexicon_tags_25 },
    { 0x35374b92, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39774b97, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd74ba1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb974ba4, SUC_lexicon_tags_7 },
    { 0x9cf74ba5, SUC_lexicon_tags_11 },
    { 0x82d74ba6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23174bab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd74bb1, SUC_lexicon_tags_2 },
    { 0x32b74bb2, SUC_lexicon_tags_11 },
    { 0x42374bb3, SUC_lexicon_tags_7 },
    { 0x28574bb4, SUC_lexicon_tags_1 },
    { 0x22174bb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc574bb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51774bbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb74bc4, SUC_lexicon_tags_77 },
    { 0x3bf74bc4, SUC_lexicon_tags_23 },
    { 0x19974bc4, SUC_lexicon_tags_37 },
    { 0xc6b74bc6, SUC_lexicon_tags_1 },
    { 0xa5374bc8, SUC_lexicon_tags_15 },
    { 0x33974bc8, SUC_lexicon_tags_4 },
    { 0x80f74bc9, SUC_lexicon_tags_65 },
    { 0xce174bcb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c574bd0, SUC_lexicon_tags_23 },
    { 0x22574bd1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfa174bd3, SUC_lexicon_tags_2 },
    { 0xa5574bd4, SUC_lexicon_tags_27 },
    { 0x66974bd5, SUC_lexicon_tags_70 },
    { 0xfc974bd6, SUC_lexicon_tags_11 },
    { 0x17b74bd4, SUC_lexicon_tags_11 },
    { 0xc5f74bd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd74bdb, SUC_lexicon_tags_15 },
    { 0xdfb74bdc, SUC_lexicon_tags_17 },
    { 0xe0f74bdd, SUC_lexicon_tags_3 },
    { 0xd6b74bdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x45574be1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x54774be4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6774be9, SUC_lexicon_tags_140 },
    { 0x6d974bea, SUC_lexicon_tags_11 },
    { 0x22974beb, SUC_lexicon_tags_17 },
    { 0x8e574bea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d74bf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b774bf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb2d74bf6, SUC_lexicon_tags_2 },
    { 0x21974bf7, SUC_lexicon_tags_15 },
    { 0x2b774bf6, SUC_lexicon_tags_14 },
    { 0x1a574bf9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb374bfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f974bfd, SUC_lexicon_tags_14 },
    { 0x67f74bfd, SUC_lexicon_tags_129 },
    { 0x6c574bff, SUC_lexicon_tags_49 },
    { 0x7dd74bfe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf0774c02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x13b74c04, SUC_lexicon_tags_11 },
    { 0xecf74c05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8374c09, SUC_lexicon_tags_19 },
    { 0x36774c0a, SUC_lexicon_tags_2 },
    { 0xafd74c0a, SUC_lexicon_tags_9 },
    { 0xebb74c0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79174c10, SUC_lexicon_tags_6 },
    { 0x40374c10, SUC_lexicon_tags_38 },
    { 0x57774c12, SUC_lexicon_tags_37 },
    { 0x62574c13, SUC_lexicon_tags_25 },
    { 0xdd74c13, SUC_lexicon_tags_56 },
    { 0x6a974c15, SUC_lexicon_tags_2 },
    { 0xf5374c12, SUC_lexicon_tags_37 },
    { 0xe9b74c17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d74c1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71774c1d, SUC_lexicon_tags_15 },
    { 0x10d74c1e, SUC_lexicon_tags_21 },
    { 0x27974c1d, SUC_lexicon_tags_37 },
    { 0x55f74c20, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2974c24, SUC_lexicon_tags_30 },
    { 0xbbb74c25, SUC_lexicon_tags_26 },
    { 0xad174c24, SUC_lexicon_tags_1 },
    { 0x26174c27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1174c2a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8774c2c, SUC_lexicon_tags_17 },
    { 0x7e974c2d, SUC_lexicon_tags_38 },
    { 0xfdf74c2d, SUC_lexicon_tags_17 },
    { 0x81974c2e, SUC_lexicon_tags_27 },
    { 0x1a374c2f, SUC_lexicon_tags_45 },
    { 0x48b74c31, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x25974c33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80574c38, SUC_lexicon_tags_7 },
    { 0x36974c38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6574c41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d74c47, SUC_lexicon_tags_12 },
    { 0x7774c48, SUC_lexicon_tags_11 },
    { 0xb174c49, SUC_lexicon_tags_25 },
    { 0x79374c4a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x68f74c4d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf74c55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b74c59, SUC_lexicon_tags_16 },
    { 0x25974c59, SUC_lexicon_tags_15 },
    { 0x47374c5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb74c66, SUC_lexicon_tags_10 },
    { 0x2e374c66, SUC_lexicon_tags_26 },
    { 0x22f74c68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4f74c6a, SUC_lexicon_tags_68 },
    { 0x66974c6b, SUC_lexicon_tags_12 },
    { 0x5a374c6c, SUC_lexicon_tags_49 },
    { 0x24374c6c, SUC_lexicon_tags_25 },
    { 0xfbd74c6c, SUC_lexicon_tags_13 },
    { 0xe2f74c6f, SUC_lexicon_tags_49 },
    { 0xb0774c6f, SUC_lexicon_tags_29 },
    { 0x75f74c6e, SUC_lexicon_tags_12 },
    { 0x43374c70, SUC_lexicon_tags_37 },
    { 0x10f74c70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f74c79, SUC_lexicon_tags_27 },
    { 0xc3574c7a, SUC_lexicon_tags_34 },
    { 0xaf374c7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8774c7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9c774c81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9374c85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5174c8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb74c97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7574c9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5374c9d, SUC_lexicon_tags_17 },
    { 0x3a374c9e, SUC_lexicon_tags_38 },
    { 0xc1f74c9f, SUC_lexicon_tags_15 },
    { 0xdd774c9f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7774ca4, SUC_lexicon_tags_33 },
    { 0x20f74ca4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16974caa, SUC_lexicon_tags_31 },
    { 0x97d74cab, SUC_lexicon_tags_28 },
    { 0xd2374caa, SUC_lexicon_tags_276 },
    { 0x4574cad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x79774cb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3db74cb2, SUC_lexicon_tags_9 },
    { 0x9d774cb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa4974cb5, SUC_lexicon_tags_57 },
    { 0x9fd74cb6, SUC_lexicon_tags_11 },
    { 0xf5774cb6, SUC_lexicon_tags_7 },
    { 0x7174cb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd74cbd, SUC_lexicon_tags_15 },
    { 0x64174cbe, SUC_lexicon_tags_37 },
    { 0x3e574cbd, SUC_lexicon_tags_27 },
    { 0x6e374cc0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb974cc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60974cc6, SUC_lexicon_tags_9 },
    { 0xa4d74cc7, SUC_lexicon_tags_17 },
    { 0xb0374cc6, SUC_lexicon_tags_672 },
    { 0x3db74cc6, SUC_lexicon_tags_43 },
    { 0xb974cc7, SUC_lexicon_tags_1 },
    { 0x75574ccb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d374cd3, SUC_lexicon_tags_12 },
    { 0xe6f74cd4, SUC_lexicon_tags_1 },
    { 0x26f74cd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b74cd8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c374cde, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b74ce2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85774ce6, SUC_lexicon_tags_15 },
    { 0xe9374ce7, SUC_lexicon_tags_11 },
    { 0xa0b74ce8, SUC_lexicon_tags_37 },
    { 0x67b74ce7, SUC_lexicon_tags_25 },
    { 0x71374cea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c574cef, SUC_lexicon_tags_45 },
    { 0xc7374cef, SUC_lexicon_tags_11 },
    { 0x48374cf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee374cf3, SUC_lexicon_tags_17 },
    { 0x6b974cf3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd974cf7, SUC_lexicon_tags_37 },
    { 0x2fd74cf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd774cfd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d74d05, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66174d0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f74d12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f74d15, SUC_lexicon_tags_6 },
    { 0x4e174d15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d74d19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd74d1c, SUC_lexicon_tags_47 },
    { 0xa0d74d1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x25974d1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d74d23, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6d574d25, SUC_lexicon_tags_41 },
    { 0x78d74d26, SUC_lexicon_tags_28 },
    { 0x4cf74d27, SUC_lexicon_tags_38 },
    { 0xd7d74d25, SUC_lexicon_tags_65 },
    { 0xc7f74d26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f74d2d, SUC_lexicon_tags_38 },
    { 0x3ff74d2e, SUC_lexicon_tags_77 },
    { 0xbc774d2f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x86d74d31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xde174d34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6974d37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b174d39, SUC_lexicon_tags_47 },
    { 0x6db74d3a, SUC_lexicon_tags_37 },
    { 0x3dd74d3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf74d41, SUC_lexicon_tags_5 },
    { 0x50774d42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe774d45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb574d47, SUC_lexicon_tags_2 },
    { 0xc5774d47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41774d4f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4774d59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f74d60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5ad74d62, SUC_lexicon_tags_11 },
    { 0xf9974d63, SUC_lexicon_tags_4 },
    { 0x90374d63, SUC_lexicon_tags_12 },
    { 0x95b74d65, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb74d6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98574d6f, SUC_lexicon_tags_46 },
    { 0x83374d70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8174d72, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca574d7c, SUC_lexicon_tags_60 },
    { 0x23174d7d, SUC_lexicon_tags_48 },
    { 0x80974d7e, SUC_lexicon_tags_7 },
    { 0x5db74d7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8574d82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f574d85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56574d89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd74d90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b74d96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40d74d98, SUC_lexicon_tags_38 },
    { 0x29174d98, SUC_lexicon_tags_11 },
    { 0xe4d74d9a, SUC_lexicon_tags_46 },
    { 0xd2374d98, SUC_lexicon_tags_1628 },
    { 0, NULL },
    { 0xf8174d9d, SUC_lexicon_tags_23 },
    { 0x5f174d9d, SUC_lexicon_tags_17 },
    { 0xe7974d9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6d74da1, SUC_lexicon_tags_55 },
    { 0xa5f74da2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f74da6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd9b74da8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb74dad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdff74daf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd74db3, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20974db8, SUC_lexicon_tags_12 },
    { 0x65974db8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3b74dbb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd4974dbd, SUC_lexicon_tags_33 },
    { 0xf4774dbe, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x46374dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc574dc5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfb174dc7, SUC_lexicon_tags_1 },
    { 0x5ed74dc8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb7374dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0b74dcc, SUC_lexicon_tags_12 },
    { 0x2db74dcc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbbd74dcf, SUC_lexicon_tags_57 },
    { 0xc174dd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15d74dd2, SUC_lexicon_tags_7 },
    { 0xa6374dd3, SUC_lexicon_tags_33 },
    { 0xaff74dd3, SUC_lexicon_tags_11 },
    { 0xfd774dd4, SUC_lexicon_tags_27 },
    { 0xe8374dd6, SUC_lexicon_tags_37 },
    { 0x76174dd7, SUC_lexicon_tags_33 },
    { 0x9c574dd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1374ddc, SUC_lexicon_tags_2 },
    { 0x82f74ddd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf8f74ddf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb974de2, SUC_lexicon_tags_27 },
    { 0x17d74de2, SUC_lexicon_tags_15 },
    { 0x86d74de4, SUC_lexicon_tags_29 },
    { 0x3b174de2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9774de9, SUC_lexicon_tags_47 },
    { 0xbcb74de9, SUC_lexicon_tags_82 },
    { 0xbf774dea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc8f74ded, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b74df5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6574dfd, SUC_lexicon_tags_66 },
    { 0xcc174dfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42774e00, SUC_lexicon_tags_31 },
    { 0x95b74e01, SUC_lexicon_tags_15 },
    { 0xd0574e02, SUC_lexicon_tags_1 },
    { 0x60774e03, SUC_lexicon_tags_87 },
    { 0xd7974e03, SUC_lexicon_tags_47 },
    { 0x73774e05, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaa174e07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35174e09, SUC_lexicon_tags_38 },
    { 0x3df74e0a, SUC_lexicon_tags_17 },
    { 0xdc974e0b, SUC_lexicon_tags_19 },
    { 0xded74e0c, SUC_lexicon_tags_27 },
    { 0xf3f74e09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66174e0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28f74e11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c974e1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75974e22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2374e25, SUC_lexicon_tags_37 },
    { 0x40974e26, SUC_lexicon_tags_7 },
    { 0xc5974e27, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4774e2a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f374e2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34174e31, SUC_lexicon_tags_14 },
    { 0x36974e32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f974e34, SUC_lexicon_tags_21 },
    { 0xc0374e35, SUC_lexicon_tags_34 },
    { 0x3a174e35, SUC_lexicon_tags_17 },
    { 0x2b174e37, SUC_lexicon_tags_201 },
    { 0xcb74e38, SUC_lexicon_tags_1 },
    { 0xd8374e39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b74e3c, SUC_lexicon_tags_1 },
    { 0x95974e3c, SUC_lexicon_tags_38 },
    { 0xf4d74e3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc6974e40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b74e43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71b74e45, SUC_lexicon_tags_16 },
    { 0xa7d74e46, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x85374e48, SUC_lexicon_tags_15 },
    { 0x8774e48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd2974e4b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28374e54, SUC_lexicon_tags_165 },
    { 0x5b74e55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2df74e57, SUC_lexicon_tags_49 },
    { 0x84974e57, SUC_lexicon_tags_20 },
    { 0x7a374e58, SUC_lexicon_tags_15 },
    { 0xeb774e58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90374e61, SUC_lexicon_tags_25 },
    { 0xa2f74e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b374e69, SUC_lexicon_tags_21 },
    { 0x48374e69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x51774e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e174e6e, SUC_lexicon_tags_13 },
    { 0xca174e6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8174e74, SUC_lexicon_tags_30 },
    { 0x8cb74e75, SUC_lexicon_tags_45 },
    { 0x6d374e76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9a974e78, SUC_lexicon_tags_13 },
    { 0x2df74e79, SUC_lexicon_tags_2 },
    { 0x5df74e7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed374e7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d74e7f, SUC_lexicon_tags_11 },
    { 0xfcd74e80, SUC_lexicon_tags_6 },
    { 0x23974e81, SUC_lexicon_tags_62 },
    { 0xc5d74e82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c174e89, SUC_lexicon_tags_9 },
    { 0xb74e89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15974e8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5574e98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb74e9e, SUC_lexicon_tags_2 },
    { 0x84374e9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8974ead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca974eb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9174eb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f374ebb, SUC_lexicon_tags_0 },
    { 0x46b74ebb, SUC_lexicon_tags_12 },
    { 0x96174ebd, SUC_lexicon_tags_15 },
    { 0x9e374ebe, SUC_lexicon_tags_37 },
    { 0xe5774ebe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f74ec3, SUC_lexicon_tags_40 },
    { 0xb2b74ec3, SUC_lexicon_tags_56 },
    { 0x35f74ec5, SUC_lexicon_tags_11 },
    { 0x68174ec5, SUC_lexicon_tags_3 },
    { 0x41174ec3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2db74ec9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f74ecc, SUC_lexicon_tags_2 },
    { 0xf174ecc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36174ed2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x174ed4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x43574ed6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f74ed9, SUC_lexicon_tags_14 },
    { 0x2bd74eda, SUC_lexicon_tags_17 },
    { 0xa0d74edb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a774edf, SUC_lexicon_tags_37 },
    { 0xea574ee0, SUC_lexicon_tags_27 },
    { 0xa0174ee1, SUC_lexicon_tags_2 },
    { 0xb7974ee2, SUC_lexicon_tags_43 },
    { 0xa3374ee3, SUC_lexicon_tags_47 },
    { 0x8cd74ee3, SUC_lexicon_tags_30 },
    { 0x23b74ee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd974ee9, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xd3f74eeb, SUC_lexicon_tags_33 },
    { 0x44374eec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95974ef0, SUC_lexicon_tags_28 },
    { 0x4ab74ef1, SUC_lexicon_tags_46 },
    { 0xda774ef2, SUC_lexicon_tags_11 },
    { 0x89574ef3, SUC_lexicon_tags_26 },
    { 0xec174ef4, SUC_lexicon_tags_37 },
    { 0x91374ef5, SUC_lexicon_tags_1 },
    { 0x68774ef5, SUC_lexicon_tags_1 },
    { 0x90d74ef7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa0d74ef9, SUC_lexicon_tags_37 },
    { 0x50774efa, SUC_lexicon_tags_17 },
    { 0x2c574efa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd9b74efd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3374eff, SUC_lexicon_tags_1 },
    { 0xc9774f00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f74f0b, SUC_lexicon_tags_37 },
    { 0xe574f0c, SUC_lexicon_tags_164 },
    { 0xd1b74f0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a574f14, SUC_lexicon_tags_17 },
    { 0x50d74f15, SUC_lexicon_tags_14 },
    { 0x26774f14, SUC_lexicon_tags_49 },
    { 0xe0374f17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf374f1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x20374f1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd74f20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf74f27, SUC_lexicon_tags_25 },
    { 0xd2974f28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30f74f2a, SUC_lexicon_tags_63 },
    { 0x7c574f2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5374f30, SUC_lexicon_tags_6 },
    { 0x64974f30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc374f34, SUC_lexicon_tags_31 },
    { 0x52174f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0174f3b, SUC_lexicon_tags_3 },
    { 0xc4f74f3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf74f42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70774f49, SUC_lexicon_tags_131 },
    { 0x2ff74f4a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x75774f4d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6574f55, SUC_lexicon_tags_25 },
    { 0xb6d74f55, SUC_lexicon_tags_40 },
    { 0xb9f74f57, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f74f5a, SUC_lexicon_tags_17 },
    { 0xfa774f5b, SUC_lexicon_tags_15 },
    { 0xf4d74f5c, SUC_lexicon_tags_11 },
    { 0xd4b74f5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd774f60, SUC_lexicon_tags_37 },
    { 0x6fd74f61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a174f67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55774f6a, SUC_lexicon_tags_0 },
    { 0x24d74f6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8574f6d, SUC_lexicon_tags_17 },
    { 0xb4d74f6e, SUC_lexicon_tags_2 },
    { 0xa3974f6e, SUC_lexicon_tags_17 },
    { 0xdd174f70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1774f75, SUC_lexicon_tags_18 },
    { 0xacb74f76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8774f79, SUC_lexicon_tags_31 },
    { 0x85d74f79, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f774f7d, SUC_lexicon_tags_730 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8574f82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b974f85, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0774f8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x61374f8f, SUC_lexicon_tags_172 },
    { 0x5a974f90, SUC_lexicon_tags_26 },
    { 0x8bd74f90, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4774f93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d74f98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8974f9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd774fa2, SUC_lexicon_tags_37 },
    { 0x66d74fa2, SUC_lexicon_tags_2 },
    { 0x68174fa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21774fa7, SUC_lexicon_tags_14 },
    { 0xaff74fa8, SUC_lexicon_tags_27 },
    { 0x8e374fa9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd774fb4, SUC_lexicon_tags_30 },
    { 0x2eb74fb5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x33574fb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a774fb9, SUC_lexicon_tags_17 },
    { 0xdd774fba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7974fbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df74fc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d74fc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75774fd3, SUC_lexicon_tags_1174 },
    { 0xf6774fd4, SUC_lexicon_tags_9 },
    { 0x2f74fd5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b74fda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5374fe4, SUC_lexicon_tags_0 },
    { 0x74174fe4, SUC_lexicon_tags_29 },
    { 0xe8174fe6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b574fea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af74fed, SUC_lexicon_tags_79 },
    { 0xc9174fee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7974ff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb74ff4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e774ffb, SUC_lexicon_tags_65 },
    { 0x76974ffc, SUC_lexicon_tags_15 },
    { 0x66b74ffd, SUC_lexicon_tags_124 },
    { 0x86574ffe, SUC_lexicon_tags_7 },
    { 0x46174ffb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf975003, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f75007, SUC_lexicon_tags_32 },
    { 0x64375008, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5537500b, SUC_lexicon_tags_7 },
    { 0x4b77500c, SUC_lexicon_tags_37 },
    { 0xadf7500d, SUC_lexicon_tags_1 },
    { 0x7c77500d, SUC_lexicon_tags_1 },
    { 0xd437500f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75012, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde775014, SUC_lexicon_tags_12 },
    { 0x68175014, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe317501c, SUC_lexicon_tags_11 },
    { 0x3bd7501c, SUC_lexicon_tags_68 },
    { 0x2217501d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf775022, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf75026, SUC_lexicon_tags_11 },
    { 0xd5375027, SUC_lexicon_tags_37 },
    { 0xeb975028, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef7502d, SUC_lexicon_tags_3 },
    { 0x4517502e, SUC_lexicon_tags_27 },
    { 0xc757502f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fb75031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ad75033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf75040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72375044, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf575048, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8137504b, SUC_lexicon_tags_12 },
    { 0x29b7504c, SUC_lexicon_tags_25 },
    { 0xaf17504c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe737504f, SUC_lexicon_tags_60 },
    { 0xc6975050, SUC_lexicon_tags_17 },
    { 0x64775051, SUC_lexicon_tags_64 },
    { 0x78f75052, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc1175054, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd75058, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef77505d, SUC_lexicon_tags_7 },
    { 0x8f77505e, SUC_lexicon_tags_1 },
    { 0x36d7505e, SUC_lexicon_tags_29 },
    { 0xcd17505e, SUC_lexicon_tags_26 },
    { 0x7237505f, SUC_lexicon_tags_25 },
    { 0x60175062, SUC_lexicon_tags_49 },
    { 0xc7575063, SUC_lexicon_tags_28 },
    { 0xe4b75062, SUC_lexicon_tags_26 },
    { 0x4b375061, SUC_lexicon_tags_27 },
    { 0x55d75066, SUC_lexicon_tags_25 },
    { 0x61f75067, SUC_lexicon_tags_31 },
    { 0xe0f75066, SUC_lexicon_tags_32 },
    { 0xffb75061, SUC_lexicon_tags_148 },
    { 0xf4175068, SUC_lexicon_tags_11 },
    { 0x9b175067, SUC_lexicon_tags_15 },
    { 0x77b75062, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f75076, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab7507a, SUC_lexicon_tags_43 },
    { 0x73b7507b, SUC_lexicon_tags_0 },
    { 0xd4d7507c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c375080, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb75084, SUC_lexicon_tags_37 },
    { 0xa8375085, SUC_lexicon_tags_27 },
    { 0x30975085, SUC_lexicon_tags_9 },
    { 0xd3b75087, SUC_lexicon_tags_77 },
    { 0xda775088, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2437508d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b75091, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f375095, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75098, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x397509a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d7509d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5317509f, SUC_lexicon_tags_23 },
    { 0x42f7509f, SUC_lexicon_tags_49 },
    { 0x583750a1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7f750a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d750a8, SUC_lexicon_tags_7 },
    { 0xca9750a9, SUC_lexicon_tags_17 },
    { 0xf8f750aa, SUC_lexicon_tags_11 },
    { 0x555750a8, SUC_lexicon_tags_15 },
    { 0x703750ac, SUC_lexicon_tags_2 },
    { 0xfc3750ad, SUC_lexicon_tags_6 },
    { 0x501750ae, SUC_lexicon_tags_7 },
    { 0x885750ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbed750b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1750b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7750b7, SUC_lexicon_tags_12 },
    { 0xdeb750b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11750bc, SUC_lexicon_tags_27 },
    { 0xfe1750bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb750c1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9af750c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf750c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x321750c9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3750cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xff750d0, SUC_lexicon_tags_41 },
    { 0x31b750d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xac5750d3, SUC_lexicon_tags_2 },
    { 0x99b750d4, SUC_lexicon_tags_14 },
    { 0xf43750d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1750dc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297750e0, SUC_lexicon_tags_69 },
    { 0x619750e1, SUC_lexicon_tags_11 },
    { 0x2c9750e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fb750e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1750ef, SUC_lexicon_tags_37 },
    { 0xb79750f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63b750f2, SUC_lexicon_tags_15 },
    { 0xdd9750f3, SUC_lexicon_tags_2 },
    { 0x2d9750f3, SUC_lexicon_tags_11 },
    { 0x24f750f5, SUC_lexicon_tags_17 },
    { 0xe27750f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf750fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe0b750fc, SUC_lexicon_tags_49 },
    { 0xd8d750fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11975101, SUC_lexicon_tags_10 },
    { 0x75775102, SUC_lexicon_tags_1 },
    { 0xb0d75103, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55d75105, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b7510b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f17510e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d75112, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f75115, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b775118, SUC_lexicon_tags_11 },
    { 0x98d75119, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8a37511b, SUC_lexicon_tags_12 },
    { 0x9df7511b, SUC_lexicon_tags_31 },
    { 0xdd57511c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa975123, SUC_lexicon_tags_65 },
    { 0xb7175123, SUC_lexicon_tags_25 },
    { 0x9f975123, SUC_lexicon_tags_17 },
    { 0xd3375126, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577512a, SUC_lexicon_tags_12 },
    { 0x3a17512b, SUC_lexicon_tags_10 },
    { 0x5d57512c, SUC_lexicon_tags_5 },
    { 0x3a57512c, SUC_lexicon_tags_8 },
    { 0xd17512e, SUC_lexicon_tags_15 },
    { 0x8717512f, SUC_lexicon_tags_84 },
    { 0xdbb75130, SUC_lexicon_tags_45 },
    { 0xb9575131, SUC_lexicon_tags_27 },
    { 0x90975131, SUC_lexicon_tags_37 },
    { 0x62b7512e, SUC_lexicon_tags_55 },
    { 0x6517512e, SUC_lexicon_tags_29 },
    { 0xb537512c, SUC_lexicon_tags_12 },
    { 0xc697512c, SUC_lexicon_tags_37 },
    { 0x82b75137, SUC_lexicon_tags_25 },
    { 0x2eb7512a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7513b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d7513e, SUC_lexicon_tags_38 },
    { 0xecf7513f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3f75141, SUC_lexicon_tags_15 },
    { 0x71575142, SUC_lexicon_tags_42 },
    { 0xa0575142, SUC_lexicon_tags_27 },
    { 0x12775141, SUC_lexicon_tags_12 },
    { 0xb2f75145, SUC_lexicon_tags_38 },
    { 0xf9375141, SUC_lexicon_tags_11 },
    { 0x9d775147, SUC_lexicon_tags_1702 },
    { 0, NULL },
    { 0, NULL },
    { 0xed57514a, SUC_lexicon_tags_28 },
    { 0x69b7514b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7775153, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3775156, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x95975159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d97515e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe375161, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97516b, SUC_lexicon_tags_55 },
    { 0xb837516c, SUC_lexicon_tags_10 },
    { 0xdb37516c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72f7516f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22f75171, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba775175, SUC_lexicon_tags_25 },
    { 0x24f75176, SUC_lexicon_tags_7 },
    { 0xf6f75176, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f7517b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2897517d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef75184, SUC_lexicon_tags_29 },
    { 0x7f175185, SUC_lexicon_tags_31 },
    { 0x6f575186, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c97518a, SUC_lexicon_tags_29 },
    { 0xfed7518b, SUC_lexicon_tags_11 },
    { 0xc357518c, SUC_lexicon_tags_7 },
    { 0xd8b7518c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa797518f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6ef75191, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9975194, SUC_lexicon_tags_75 },
    { 0x56775194, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d75198, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31b7519a, SUC_lexicon_tags_38 },
    { 0x9357519b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x8557519d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d751a8, SUC_lexicon_tags_19 },
    { 0x113751a8, SUC_lexicon_tags_29 },
    { 0x133751a8, SUC_lexicon_tags_7 },
    { 0xc55751ab, SUC_lexicon_tags_12 },
    { 0x809751ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x101751ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9751b1, SUC_lexicon_tags_39 },
    { 0xfa1751b1, SUC_lexicon_tags_70 },
    { 0x50b751b3, SUC_lexicon_tags_60 },
    { 0x4af751b4, SUC_lexicon_tags_2 },
    { 0xa97751b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9751b8, SUC_lexicon_tags_77 },
    { 0x35b751b8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77751be, SUC_lexicon_tags_10 },
    { 0x329751bf, SUC_lexicon_tags_14 },
    { 0xa37751c0, SUC_lexicon_tags_19 },
    { 0xadd751bf, SUC_lexicon_tags_48 },
    { 0xb0d751bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x237751c5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1751cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b751d3, SUC_lexicon_tags_2 },
    { 0x90b751d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7751d7, SUC_lexicon_tags_37 },
    { 0x55d751d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97751dc, SUC_lexicon_tags_33 },
    { 0x571751dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x717751e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdab751e2, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xc5b751e4, SUC_lexicon_tags_25 },
    { 0x7f9751e4, SUC_lexicon_tags_37 },
    { 0xde7751e6, SUC_lexicon_tags_46 },
    { 0xb8b751e4, SUC_lexicon_tags_8 },
    { 0x6ef751e8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d751ee, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5751f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab751f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95751fe, SUC_lexicon_tags_19 },
    { 0x759751ff, SUC_lexicon_tags_25 },
    { 0xa375200, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32175203, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d75206, SUC_lexicon_tags_27 },
    { 0x8fb75207, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xf4b75209, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1175210, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd75215, SUC_lexicon_tags_42 },
    { 0x72975215, SUC_lexicon_tags_14 },
    { 0xe2175217, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60b75219, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f37521c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49375220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b75223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34b75225, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf75228, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa337522c, SUC_lexicon_tags_11 },
    { 0xc517522c, SUC_lexicon_tags_37 },
    { 0x5877522d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd75231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4975235, SUC_lexicon_tags_26 },
    { 0xadd75235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f75239, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d7523f, SUC_lexicon_tags_6 },
    { 0xb3575240, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f75243, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b175247, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7524b, SUC_lexicon_tags_2 },
    { 0xf997524c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedd7524e, SUC_lexicon_tags_31 },
    { 0xc7d7524f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4175257, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23f75259, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d7525e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b775263, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xba975265, SUC_lexicon_tags_1058 },
    { 0, NULL },
    { 0x77b75267, SUC_lexicon_tags_55 },
    { 0x81d75268, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa357526a, SUC_lexicon_tags_12 },
    { 0x96d7526b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f7526e, SUC_lexicon_tags_14 },
    { 0x5217526f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2175272, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd97527f, SUC_lexicon_tags_6 },
    { 0xa4975280, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f575282, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7375285, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4175289, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf517528b, SUC_lexicon_tags_37 },
    { 0x8757528b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6317528e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21575292, SUC_lexicon_tags_17 },
    { 0xea975292, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf297529b, SUC_lexicon_tags_31 },
    { 0xa357529b, SUC_lexicon_tags_14 },
    { 0x8cf7529d, SUC_lexicon_tags_38 },
    { 0xb177529e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xedb752a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3752ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1752b1, SUC_lexicon_tags_12 },
    { 0x66f752b2, SUC_lexicon_tags_2 },
    { 0x827752b3, SUC_lexicon_tags_38 },
    { 0x1f752b2, SUC_lexicon_tags_1 },
    { 0xed9752b5, SUC_lexicon_tags_129 },
    { 0x885752b5, SUC_lexicon_tags_6 },
    { 0xf05752b7, SUC_lexicon_tags_6 },
    { 0x1a1752b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb73752ba, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x305752bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d752bf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511752c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff752c8, SUC_lexicon_tags_34 },
    { 0x13f752c9, SUC_lexicon_tags_2 },
    { 0xdd7752c8, SUC_lexicon_tags_22 },
    { 0xd81752cb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf59752cd, SUC_lexicon_tags_54 },
    { 0x185752ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47f752d0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69752d4, SUC_lexicon_tags_63 },
    { 0x1bd752d5, SUC_lexicon_tags_10 },
    { 0xb5d752d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d752e3, SUC_lexicon_tags_17 },
    { 0x751752e3, SUC_lexicon_tags_2 },
    { 0xb37752e4, SUC_lexicon_tags_55 },
    { 0x47b752e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5ab752e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x917752eb, SUC_lexicon_tags_25 },
    { 0xec7752ec, SUC_lexicon_tags_27 },
    { 0x8bb752ec, SUC_lexicon_tags_11 },
    { 0x319752ed, SUC_lexicon_tags_25 },
    { 0x78d752ef, SUC_lexicon_tags_15 },
    { 0xa35752ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7752f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc89752f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x989752fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3752fd, SUC_lexicon_tags_82 },
    { 0x677752fe, SUC_lexicon_tags_25 },
    { 0xa37752ff, SUC_lexicon_tags_37 },
    { 0x2bf752fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7b75302, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45575305, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3775308, SUC_lexicon_tags_9 },
    { 0x71b75309, SUC_lexicon_tags_37 },
    { 0x1f77530a, SUC_lexicon_tags_11 },
    { 0x277530b, SUC_lexicon_tags_29 },
    { 0x9c97530b, SUC_lexicon_tags_7 },
    { 0x8db75309, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b75310, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b75313, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c775316, SUC_lexicon_tags_15 },
    { 0xb3575317, SUC_lexicon_tags_23 },
    { 0xd8175316, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a77531b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b7531e, SUC_lexicon_tags_15 },
    { 0xdef7531f, SUC_lexicon_tags_17 },
    { 0xd0b75320, SUC_lexicon_tags_59 },
    { 0x79d75320, SUC_lexicon_tags_7 },
    { 0x7e375320, SUC_lexicon_tags_2 },
    { 0x5d175323, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x17575325, SUC_lexicon_tags_14 },
    { 0x83775326, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c37532b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb77532d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6175330, SUC_lexicon_tags_1 },
    { 0x4ef75330, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9775333, SUC_lexicon_tags_59 },
    { 0x56b75334, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff75337, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697533b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a37533e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16375345, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x775348, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a77534b, SUC_lexicon_tags_2 },
    { 0xee17534c, SUC_lexicon_tags_38 },
    { 0x7937534d, SUC_lexicon_tags_7 },
    { 0x9577534d, SUC_lexicon_tags_29 },
    { 0x7ed7534e, SUC_lexicon_tags_39 },
    { 0x75b7534d, SUC_lexicon_tags_17 },
    { 0xd037534b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ed75353, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b75358, SUC_lexicon_tags_61 },
    { 0x1dd75358, SUC_lexicon_tags_37 },
    { 0xccf7535a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c97535e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa775362, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2775367, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79775378, SUC_lexicon_tags_14 },
    { 0x2cb75379, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b7537e, SUC_lexicon_tags_12 },
    { 0x1977537e, SUC_lexicon_tags_27 },
    { 0xb1d7537e, SUC_lexicon_tags_47 },
    { 0xe0175381, SUC_lexicon_tags_25 },
    { 0x61775381, SUC_lexicon_tags_29 },
    { 0x85775382, SUC_lexicon_tags_21 },
    { 0xc5f75384, SUC_lexicon_tags_19 },
    { 0xf2375385, SUC_lexicon_tags_43 },
    { 0x80375382, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcbf75388, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe337538c, SUC_lexicon_tags_49 },
    { 0xb9b7538c, SUC_lexicon_tags_19 },
    { 0x8857538e, SUC_lexicon_tags_33 },
    { 0xbd37538c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda775392, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b75396, SUC_lexicon_tags_2 },
    { 0xf2375397, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0375399, SUC_lexicon_tags_12 },
    { 0xc2b7539a, SUC_lexicon_tags_7 },
    { 0xd0b7539b, SUC_lexicon_tags_25 },
    { 0x57575399, SUC_lexicon_tags_0 },
    { 0xbbb75399, SUC_lexicon_tags_12 },
    { 0xedf7539a, SUC_lexicon_tags_15 },
    { 0x9577539f, SUC_lexicon_tags_17 },
    { 0xf7b753a0, SUC_lexicon_tags_14 },
    { 0xb19753a0, SUC_lexicon_tags_61 },
    { 0xfdd753a2, SUC_lexicon_tags_27 },
    { 0x2db753a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab753a6, SUC_lexicon_tags_69 },
    { 0xf75753a6, SUC_lexicon_tags_49 },
    { 0xaf3753a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x225753aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09753b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b9753b3, SUC_lexicon_tags_37 },
    { 0x5c5753b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf753b7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f753ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x381753bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4bd753be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd753c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9753c5, SUC_lexicon_tags_43 },
    { 0x2b5753c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7753c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x647753cb, SUC_lexicon_tags_7 },
    { 0x87d753cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xad753cf, SUC_lexicon_tags_13 },
    { 0x95753d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85753d4, SUC_lexicon_tags_23 },
    { 0x9cf753d4, SUC_lexicon_tags_37 },
    { 0x21f753d6, SUC_lexicon_tags_8 },
    { 0xad7753d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x667753da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57753dd, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1753e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9753e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7753e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf89753ec, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9ff753ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad753f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x915753f3, SUC_lexicon_tags_15 },
    { 0x39753f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb89753f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x113753f8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b753fc, SUC_lexicon_tags_47 },
    { 0x6fd753fd, SUC_lexicon_tags_84 },
    { 0x633753fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c375400, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96575402, SUC_lexicon_tags_27 },
    { 0xd0175402, SUC_lexicon_tags_47 },
    { 0x4d375403, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9775407, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee375411, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca975417, SUC_lexicon_tags_11 },
    { 0x5ab75418, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf077541b, SUC_lexicon_tags_37 },
    { 0x5377541c, SUC_lexicon_tags_32 },
    { 0xc317541d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0575422, SUC_lexicon_tags_60 },
    { 0x70175423, SUC_lexicon_tags_15 },
    { 0x17b75423, SUC_lexicon_tags_25 },
    { 0x1f575423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d75428, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a57542b, SUC_lexicon_tags_31 },
    { 0x26b7542c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb677542f, SUC_lexicon_tags_45 },
    { 0x12775430, SUC_lexicon_tags_7 },
    { 0xd1b7542f, SUC_lexicon_tags_36 },
    { 0xd2b7542f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d75438, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9af7543a, SUC_lexicon_tags_396 },
    { 0x9177543b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1175443, SUC_lexicon_tags_2 },
    { 0x89375443, SUC_lexicon_tags_48 },
    { 0x8b75444, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x975447, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6175450, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8575453, SUC_lexicon_tags_27 },
    { 0x45375454, SUC_lexicon_tags_27 },
    { 0x2b575454, SUC_lexicon_tags_2 },
    { 0x71575456, SUC_lexicon_tags_26 },
    { 0x7bb75453, SUC_lexicon_tags_2 },
    { 0x20775453, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd17545a, SUC_lexicon_tags_12 },
    { 0x877545b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b7545f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53775464, SUC_lexicon_tags_19 },
    { 0xf1575464, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1775469, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf575470, SUC_lexicon_tags_16 },
    { 0x32f75471, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd75476, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d75479, SUC_lexicon_tags_14 },
    { 0x6717547a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43175482, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6dd75484, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ab75486, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c17548c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d75492, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e175494, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa775497, SUC_lexicon_tags_11 },
    { 0x6bd75498, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17549d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3754a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x231754a6, SUC_lexicon_tags_43 },
    { 0x1d754a7, SUC_lexicon_tags_11 },
    { 0x573754a8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447754ac, SUC_lexicon_tags_107 },
    { 0x909754ad, SUC_lexicon_tags_2 },
    { 0x4ab754ad, SUC_lexicon_tags_17 },
    { 0x957754ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf754b4, SUC_lexicon_tags_0 },
    { 0x323754b5, SUC_lexicon_tags_38 },
    { 0x88d754b5, SUC_lexicon_tags_29 },
    { 0x975754b6, SUC_lexicon_tags_11 },
    { 0xbd1754b8, SUC_lexicon_tags_32 },
    { 0xccb754b7, SUC_lexicon_tags_30 },
    { 0x36d754b9, SUC_lexicon_tags_37 },
    { 0x9f7754b6, SUC_lexicon_tags_0 },
    { 0xaa1754bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb5b754be, SUC_lexicon_tags_60 },
    { 0x57754bf, SUC_lexicon_tags_25 },
    { 0x809754c0, SUC_lexicon_tags_1 },
    { 0x215754c1, SUC_lexicon_tags_25 },
    { 0x543754bf, SUC_lexicon_tags_49 },
    { 0x98f754c3, SUC_lexicon_tags_11 },
    { 0xc65754c4, SUC_lexicon_tags_34 },
    { 0x893754c5, SUC_lexicon_tags_37 },
    { 0x637754c6, SUC_lexicon_tags_37 },
    { 0xe45754c7, SUC_lexicon_tags_11 },
    { 0x709754c7, SUC_lexicon_tags_25 },
    { 0x3a7754c9, SUC_lexicon_tags_63 },
    { 0xcc1754ca, SUC_lexicon_tags_1053 },
    { 0x3f754ca, SUC_lexicon_tags_40 },
    { 0x37b754cc, SUC_lexicon_tags_45 },
    { 0xe1b754cd, SUC_lexicon_tags_39 },
    { 0xf71754cd, SUC_lexicon_tags_31 },
    { 0x36f754cf, SUC_lexicon_tags_55 },
    { 0x69b754c5, SUC_lexicon_tags_25 },
    { 0xec3754cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x953754d3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x29754d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d754d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x973754da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807754e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7bb754e2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf754e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x371754e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7754ee, SUC_lexicon_tags_69 },
    { 0x9d1754ef, SUC_lexicon_tags_3 },
    { 0x29f754ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae1754f2, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x919754f4, SUC_lexicon_tags_47 },
    { 0x347754f5, SUC_lexicon_tags_37 },
    { 0xf2f754f6, SUC_lexicon_tags_14 },
    { 0xc45754f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d975500, SUC_lexicon_tags_29 },
    { 0xbd775501, SUC_lexicon_tags_15 },
    { 0x4e975501, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xae575504, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd7550c, SUC_lexicon_tags_34 },
    { 0xe757550c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b75512, SUC_lexicon_tags_70 },
    { 0x4d575513, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce37551f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f75523, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b775527, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f7552a, SUC_lexicon_tags_0 },
    { 0x9ef7552a, SUC_lexicon_tags_25 },
    { 0xbe37552a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56375530, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67d75532, SUC_lexicon_tags_19 },
    { 0x8e575532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef7553b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab7553e, SUC_lexicon_tags_10 },
    { 0x9697553f, SUC_lexicon_tags_15 },
    { 0xc937553f, SUC_lexicon_tags_66 },
    { 0xc5775541, SUC_lexicon_tags_21 },
    { 0x7fb75542, SUC_lexicon_tags_17 },
    { 0xd437553e, SUC_lexicon_tags_2 },
    { 0xb175544, SUC_lexicon_tags_27 },
    { 0x81d75544, SUC_lexicon_tags_60 },
    { 0x8d575546, SUC_lexicon_tags_17 },
    { 0x5cb75546, SUC_lexicon_tags_11 },
    { 0xe1b75548, SUC_lexicon_tags_17 },
    { 0x47375548, SUC_lexicon_tags_23 },
    { 0x30b75540, SUC_lexicon_tags_2 },
    { 0xe217553f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd175551, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a975559, SUC_lexicon_tags_158 },
    { 0x1f975559, SUC_lexicon_tags_12 },
    { 0x8bd7555a, SUC_lexicon_tags_11 },
    { 0x7af7555b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38375562, SUC_lexicon_tags_23 },
    { 0xaf75563, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6575567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b17556a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f75573, SUC_lexicon_tags_37 },
    { 0xe5975574, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce575576, SUC_lexicon_tags_8 },
    { 0x1e375577, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb7557a, SUC_lexicon_tags_12 },
    { 0xd057557b, SUC_lexicon_tags_37 },
    { 0xdbd7557c, SUC_lexicon_tags_12 },
    { 0x3fb7557b, SUC_lexicon_tags_37 },
    { 0xf137557e, SUC_lexicon_tags_17 },
    { 0x3677557f, SUC_lexicon_tags_1 },
    { 0x77d7557d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1975583, SUC_lexicon_tags_1 },
    { 0x47175584, SUC_lexicon_tags_37 },
    { 0x19b75585, SUC_lexicon_tags_66 },
    { 0x6bb75586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b7558f, SUC_lexicon_tags_27 },
    { 0xf3f75590, SUC_lexicon_tags_2 },
    { 0x7cb7558f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xde975594, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5b75596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cf75598, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23b7559a, SUC_lexicon_tags_1391 },
    { 0xe637559a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a57559d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2817559f, SUC_lexicon_tags_15 },
    { 0xcf9755a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501755a9, SUC_lexicon_tags_9 },
    { 0x9f3755aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6e3755ac, SUC_lexicon_tags_37 },
    { 0x895755ac, SUC_lexicon_tags_25 },
    { 0x3a9755ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48d755b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc53755b3, SUC_lexicon_tags_12 },
    { 0x4bd755b4, SUC_lexicon_tags_599 },
    { 0xeed755b4, SUC_lexicon_tags_23 },
    { 0xfd1755b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825755bc, SUC_lexicon_tags_6 },
    { 0x1b755bc, SUC_lexicon_tags_87 },
    { 0x54b755be, SUC_lexicon_tags_17 },
    { 0x69f755bf, SUC_lexicon_tags_37 },
    { 0x2e3755c0, SUC_lexicon_tags_31 },
    { 0x25d755c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd755c4, SUC_lexicon_tags_201 },
    { 0xc77755c4, SUC_lexicon_tags_9 },
    { 0xd89755c6, SUC_lexicon_tags_12 },
    { 0x639755c7, SUC_lexicon_tags_1 },
    { 0xea1755c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e3755ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77755cf, SUC_lexicon_tags_7 },
    { 0x59755d0, SUC_lexicon_tags_2 },
    { 0x207755d1, SUC_lexicon_tags_27 },
    { 0xfef755d1, SUC_lexicon_tags_2 },
    { 0xb7b755d3, SUC_lexicon_tags_47 },
    { 0x6eb755d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x781755d6, SUC_lexicon_tags_10 },
    { 0x21755d6, SUC_lexicon_tags_5 },
    { 0xaf755d8, SUC_lexicon_tags_3 },
    { 0x19755d6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe95755db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693755e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93755e8, SUC_lexicon_tags_7 },
    { 0x1f755e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1755ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75755f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3755f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20575601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27975605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1775607, SUC_lexicon_tags_6 },
    { 0xc0375608, SUC_lexicon_tags_14 },
    { 0x8a575607, SUC_lexicon_tags_3 },
    { 0x2ad7560a, SUC_lexicon_tags_27 },
    { 0x42b7560a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd375611, SUC_lexicon_tags_37 },
    { 0xc1b75612, SUC_lexicon_tags_33 },
    { 0x77375613, SUC_lexicon_tags_25 },
    { 0x3a775614, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf75618, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd7561d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1df7561f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff75625, SUC_lexicon_tags_17 },
    { 0x40375626, SUC_lexicon_tags_0 },
    { 0xeff75627, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe17562c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb7562f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf975631, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c375637, SUC_lexicon_tags_1 },
    { 0xbe375638, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0xa797563b, SUC_lexicon_tags_8 },
    { 0x8877563c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa337563f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x51375641, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f75646, SUC_lexicon_tags_2 },
    { 0x61d75647, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35575649, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3817564e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d775651, SUC_lexicon_tags_82 },
    { 0xc6575652, SUC_lexicon_tags_46 },
    { 0x3ff75653, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81375655, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b75657, SUC_lexicon_tags_14 },
    { 0x68975657, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4537565a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6597565f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bf75661, SUC_lexicon_tags_11 },
    { 0xf1975662, SUC_lexicon_tags_13 },
    { 0x31f75661, SUC_lexicon_tags_23 },
    { 0x1b775664, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x85f75666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2775668, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b7566e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b75671, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26775673, SUC_lexicon_tags_211 },
    { 0x25575674, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50975678, SUC_lexicon_tags_37 },
    { 0xe975679, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f77567e, SUC_lexicon_tags_0 },
    { 0x30f7567f, SUC_lexicon_tags_10 },
    { 0x3175680, SUC_lexicon_tags_11 },
    { 0x26175680, SUC_lexicon_tags_17 },
    { 0xdf75681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9375687, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db7568c, SUC_lexicon_tags_14 },
    { 0xde37568c, SUC_lexicon_tags_131 },
    { 0xec57568c, SUC_lexicon_tags_25 },
    { 0xbc37568f, SUC_lexicon_tags_0 },
    { 0xd197568f, SUC_lexicon_tags_60 },
    { 0x9a775691, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9375695, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8975698, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e17569b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb756a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x979756a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf756a7, SUC_lexicon_tags_47 },
    { 0x63d756a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5756ac, SUC_lexicon_tags_57 },
    { 0x72b756ad, SUC_lexicon_tags_2 },
    { 0x55b756ac, SUC_lexicon_tags_1 },
    { 0xa51756ad, SUC_lexicon_tags_27 },
    { 0xe9f756ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf756b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb756b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x67756b9, SUC_lexicon_tags_37 },
    { 0xf2b756b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df756bd, SUC_lexicon_tags_7 },
    { 0x485756be, SUC_lexicon_tags_23 },
    { 0xa7d756bf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509756c3, SUC_lexicon_tags_17 },
    { 0x703756c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x883756c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701756cd, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981756d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb95756d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7b756d8, SUC_lexicon_tags_0 },
    { 0x959756d9, SUC_lexicon_tags_17 },
    { 0xdd3756da, SUC_lexicon_tags_47 },
    { 0x865756db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd756e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xced756e2, SUC_lexicon_tags_336 },
    { 0x5a1756e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x903756e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x63d756e8, SUC_lexicon_tags_11 },
    { 0x357756e8, SUC_lexicon_tags_17 },
    { 0xce7756ea, SUC_lexicon_tags_27 },
    { 0xe2b756ea, SUC_lexicon_tags_37 },
    { 0x403756e9, SUC_lexicon_tags_35 },
    { 0xde5756ed, SUC_lexicon_tags_14 },
    { 0xb9d756ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71756f2, SUC_lexicon_tags_2 },
    { 0xc03756f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x56b756f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x859756f8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73756fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f75701, SUC_lexicon_tags_11 },
    { 0xc2d75702, SUC_lexicon_tags_12 },
    { 0x21b75703, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2775707, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f97570c, SUC_lexicon_tags_1 },
    { 0x9977570d, SUC_lexicon_tags_23 },
    { 0xe7d7570c, SUC_lexicon_tags_22 },
    { 0x3a17570f, SUC_lexicon_tags_17 },
    { 0x58f7570f, SUC_lexicon_tags_8 },
    { 0xf3575710, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b75717, SUC_lexicon_tags_15 },
    { 0x5d575717, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f7571c, SUC_lexicon_tags_2 },
    { 0x7c17571d, SUC_lexicon_tags_14 },
    { 0x2df7571d, SUC_lexicon_tags_26 },
    { 0xf837571f, SUC_lexicon_tags_262 },
    { 0xcc375720, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd75722, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7575725, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdb175727, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a575729, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa617572b, SUC_lexicon_tags_37 },
    { 0xc9d7572c, SUC_lexicon_tags_11 },
    { 0xe6f7572d, SUC_lexicon_tags_39 },
    { 0x2737572d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe175733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4d75735, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c175737, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657573b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d7573e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b975744, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff17574b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa737574e, SUC_lexicon_tags_7 },
    { 0x1977574f, SUC_lexicon_tags_11 },
    { 0x8a37574f, SUC_lexicon_tags_0 },
    { 0x7d97574f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5c975753, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c375755, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xec975757, SUC_lexicon_tags_25 },
    { 0xee175758, SUC_lexicon_tags_25 },
    { 0x5df75759, SUC_lexicon_tags_37 },
    { 0xdb75759, SUC_lexicon_tags_103 },
    { 0x62375759, SUC_lexicon_tags_47 },
    { 0x8077575c, SUC_lexicon_tags_47 },
    { 0x9af7575d, SUC_lexicon_tags_11 },
    { 0x3bb7575e, SUC_lexicon_tags_28 },
    { 0x59f7575b, SUC_lexicon_tags_27 },
    { 0x8f575760, SUC_lexicon_tags_37 },
    { 0x22375761, SUC_lexicon_tags_6 },
    { 0x6fb75762, SUC_lexicon_tags_25 },
    { 0x3b975760, SUC_lexicon_tags_37 },
    { 0xdf975760, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a57576f, SUC_lexicon_tags_12 },
    { 0xc0f75770, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23375774, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9375776, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db7577d, SUC_lexicon_tags_23 },
    { 0x6357577d, SUC_lexicon_tags_55 },
    { 0x1cd7577d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf75782, SUC_lexicon_tags_15 },
    { 0x31575783, SUC_lexicon_tags_37 },
    { 0x1e575783, SUC_lexicon_tags_8 },
    { 0x84975784, SUC_lexicon_tags_66 },
    { 0x3d975786, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10f75788, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc377578a, SUC_lexicon_tags_80 },
    { 0xed17578b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f75791, SUC_lexicon_tags_17 },
    { 0x58975792, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8375796, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7757a1, SUC_lexicon_tags_7 },
    { 0x36d757a2, SUC_lexicon_tags_32 },
    { 0x1d7757a3, SUC_lexicon_tags_95 },
    { 0xd09757a4, SUC_lexicon_tags_966 },
    { 0x36f757a5, SUC_lexicon_tags_15 },
    { 0xc8f757a3, SUC_lexicon_tags_56 },
    { 0x865757a4, SUC_lexicon_tags_79 },
    { 0xa6b757a8, SUC_lexicon_tags_55 },
    { 0x5b1757a8, SUC_lexicon_tags_15 },
    { 0x81d757a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb87757ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc19757ae, SUC_lexicon_tags_47 },
    { 0x415757ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x719757b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb757b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd37757b9, SUC_lexicon_tags_27 },
    { 0x58d757b9, SUC_lexicon_tags_7 },
    { 0x2bd757bb, SUC_lexicon_tags_17 },
    { 0x4ef757bc, SUC_lexicon_tags_11 },
    { 0x563757bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5757c2, SUC_lexicon_tags_38 },
    { 0xc81757c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453757c7, SUC_lexicon_tags_23 },
    { 0xe93757c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67757d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d757d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df757da, SUC_lexicon_tags_37 },
    { 0xc4f757db, SUC_lexicon_tags_7 },
    { 0xf7757da, SUC_lexicon_tags_63 },
    { 0xc29757da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03757e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9757e3, SUC_lexicon_tags_7 },
    { 0x99757e4, SUC_lexicon_tags_12 },
    { 0x281757e5, SUC_lexicon_tags_11 },
    { 0x93b757e6, SUC_lexicon_tags_1 },
    { 0xf99757e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd757ea, SUC_lexicon_tags_704 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683757ee, SUC_lexicon_tags_12 },
    { 0x901757ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35757f3, SUC_lexicon_tags_37 },
    { 0x1eb757f4, SUC_lexicon_tags_1 },
    { 0x1db757f5, SUC_lexicon_tags_30 },
    { 0x5f757f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f757f9, SUC_lexicon_tags_25 },
    { 0x2e1757fa, SUC_lexicon_tags_26 },
    { 0xd59757fb, SUC_lexicon_tags_2 },
    { 0xdb757fc, SUC_lexicon_tags_6 },
    { 0x7ed757fd, SUC_lexicon_tags_34 },
    { 0x747757fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0775800, SUC_lexicon_tags_3 },
    { 0x89975801, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3197580d, SUC_lexicon_tags_1 },
    { 0xdb97580e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f75811, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x47375814, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7975816, SUC_lexicon_tags_24 },
    { 0xd4175817, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7317581c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f7581f, SUC_lexicon_tags_1 },
    { 0xa0575820, SUC_lexicon_tags_25 },
    { 0x40575820, SUC_lexicon_tags_12 },
    { 0xf437581f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80775828, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc77582a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc397582c, SUC_lexicon_tags_17 },
    { 0x3637582c, SUC_lexicon_tags_47 },
    { 0x6597582e, SUC_lexicon_tags_39 },
    { 0x9137582e, SUC_lexicon_tags_7 },
    { 0x96d7582e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c775832, SUC_lexicon_tags_77 },
    { 0x66175833, SUC_lexicon_tags_25 },
    { 0x4cd75832, SUC_lexicon_tags_65 },
    { 0x50975834, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3bb75837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7775839, SUC_lexicon_tags_12 },
    { 0xdaf7583a, SUC_lexicon_tags_45 },
    { 0xb0b7583b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f77583d, SUC_lexicon_tags_15 },
    { 0xf57583e, SUC_lexicon_tags_9 },
    { 0x6337583f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68375841, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4d175843, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db75847, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2b75849, SUC_lexicon_tags_21 },
    { 0x2477584a, SUC_lexicon_tags_30 },
    { 0x88b7584b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc37584f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9975851, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb75856, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d17585e, SUC_lexicon_tags_2 },
    { 0xc837585f, SUC_lexicon_tags_7 },
    { 0xf797585e, SUC_lexicon_tags_23 },
    { 0xd975861, SUC_lexicon_tags_26 },
    { 0x69d75862, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6975868, SUC_lexicon_tags_11 },
    { 0x2af75869, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb257586b, SUC_lexicon_tags_25 },
    { 0x1b17586c, SUC_lexicon_tags_127 },
    { 0x1d17586b, SUC_lexicon_tags_38 },
    { 0xce57586e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d75871, SUC_lexicon_tags_437 },
    { 0xd4175872, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67575874, SUC_lexicon_tags_2 },
    { 0xfaf75874, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1597587b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b7587e, SUC_lexicon_tags_57 },
    { 0x17b7587e, SUC_lexicon_tags_28 },
    { 0x60775880, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd75883, SUC_lexicon_tags_49 },
    { 0x7bf75884, SUC_lexicon_tags_7 },
    { 0x16975883, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9975888, SUC_lexicon_tags_37 },
    { 0x8f975888, SUC_lexicon_tags_29 },
    { 0xf0f75889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d7588d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf75891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4175894, SUC_lexicon_tags_25 },
    { 0x95375895, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3617589b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd77589f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x747758a1, SUC_lexicon_tags_11 },
    { 0x3ab758a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7758a5, SUC_lexicon_tags_14 },
    { 0xa1f758a6, SUC_lexicon_tags_25 },
    { 0x303758a7, SUC_lexicon_tags_7 },
    { 0xcd7758a8, SUC_lexicon_tags_19 },
    { 0x68f758a8, SUC_lexicon_tags_15 },
    { 0x931758a9, SUC_lexicon_tags_1 },
    { 0x175758ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19758b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9758b6, SUC_lexicon_tags_1 },
    { 0x949758b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c9758b9, SUC_lexicon_tags_17 },
    { 0xd1758ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17758be, SUC_lexicon_tags_38 },
    { 0xd4f758bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7758c1, SUC_lexicon_tags_27 },
    { 0x9c3758c1, SUC_lexicon_tags_7 },
    { 0xa47758c3, SUC_lexicon_tags_19 },
    { 0xc81758c4, SUC_lexicon_tags_37 },
    { 0x4e9758c5, SUC_lexicon_tags_10 },
    { 0xdb1758c6, SUC_lexicon_tags_2 },
    { 0x1d7758c7, SUC_lexicon_tags_85 },
    { 0x13d758c8, SUC_lexicon_tags_12 },
    { 0x7e3758c5, SUC_lexicon_tags_26 },
    { 0x2e3758c1, SUC_lexicon_tags_7 },
    { 0xb31758c7, SUC_lexicon_tags_9 },
    { 0xc37758cc, SUC_lexicon_tags_12 },
    { 0x15b758cd, SUC_lexicon_tags_38 },
    { 0x3f9758ce, SUC_lexicon_tags_28 },
    { 0x38f758cc, SUC_lexicon_tags_47 },
    { 0xff1758ce, SUC_lexicon_tags_43 },
    { 0xfb7758c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead758d8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11758df, SUC_lexicon_tags_43 },
    { 0x945758e0, SUC_lexicon_tags_12 },
    { 0xfb5758e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9758e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf71758e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3758e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe3758eb, SUC_lexicon_tags_95 },
    { 0x8bb758eb, SUC_lexicon_tags_27 },
    { 0x21d758ec, SUC_lexicon_tags_14 },
    { 0x73f758ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1758f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab758f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57758fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f758fd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb175900, SUC_lexicon_tags_30 },
    { 0x26f75901, SUC_lexicon_tags_11 },
    { 0x62175901, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b375908, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd297590a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b7590d, SUC_lexicon_tags_11 },
    { 0x7f97590e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83d75910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x30775913, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b7591e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x46575920, SUC_lexicon_tags_19 },
    { 0xb9775921, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x65375923, SUC_lexicon_tags_1 },
    { 0xf4975924, SUC_lexicon_tags_47 },
    { 0xcc775925, SUC_lexicon_tags_11 },
    { 0x9975926, SUC_lexicon_tags_25 },
    { 0xaf75925, SUC_lexicon_tags_3 },
    { 0xbf775928, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2fb7592a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ed7592c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2997592f, SUC_lexicon_tags_15 },
    { 0xc1f7592f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x56775933, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85b75935, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77175939, SUC_lexicon_tags_19 },
    { 0x8f37593a, SUC_lexicon_tags_82 },
    { 0xe0f75939, SUC_lexicon_tags_15 },
    { 0x59f7593a, SUC_lexicon_tags_63 },
    { 0xe797593b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x19375940, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x24175942, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e375944, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6775947, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf3b75949, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c97594c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6557594e, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0xa9d75950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd75956, SUC_lexicon_tags_11 },
    { 0x50775956, SUC_lexicon_tags_29 },
    { 0x73175958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73175960, SUC_lexicon_tags_33 },
    { 0x85d75960, SUC_lexicon_tags_16 },
    { 0x31175961, SUC_lexicon_tags_66 },
    { 0x1175963, SUC_lexicon_tags_26 },
    { 0xbf75961, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xecd75966, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3cf75968, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddf7596a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6b7596c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb597596f, SUC_lexicon_tags_60 },
    { 0xf4775970, SUC_lexicon_tags_11 },
    { 0x98175970, SUC_lexicon_tags_0 },
    { 0x29175972, SUC_lexicon_tags_45 },
    { 0x86d75972, SUC_lexicon_tags_81 },
    { 0xc5175972, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3375977, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f7597a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa597597c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe257597e, SUC_lexicon_tags_33 },
    { 0xb17597f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82175981, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b775990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19775992, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99975997, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58975999, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6fb7599b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e17599d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa637599f, SUC_lexicon_tags_85 },
    { 0xc9d7599f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2bb759a2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd51759a4, SUC_lexicon_tags_75 },
    { 0x3bd759a5, SUC_lexicon_tags_1 },
    { 0xcf7759a5, SUC_lexicon_tags_45 },
    { 0x985759a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d759ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7759b3, SUC_lexicon_tags_12 },
    { 0x1a3759b4, SUC_lexicon_tags_17 },
    { 0x42f759b4, SUC_lexicon_tags_37 },
    { 0x42b759b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f7759b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x299759bb, SUC_lexicon_tags_49 },
    { 0xe5d759bc, SUC_lexicon_tags_25 },
    { 0xa39759bd, SUC_lexicon_tags_2 },
    { 0xa89759be, SUC_lexicon_tags_23 },
    { 0x31f759bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f759c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47759c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71759cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5d759ce, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xe99759d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xac5759d2, SUC_lexicon_tags_25 },
    { 0x6ad759d2, SUC_lexicon_tags_19 },
    { 0xffb759d4, SUC_lexicon_tags_37 },
    { 0xf1f759d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe93759d7, SUC_lexicon_tags_32 },
    { 0x6ed759d8, SUC_lexicon_tags_7 },
    { 0x557759d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7759dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9df759de, SUC_lexicon_tags_25 },
    { 0xf4b759df, SUC_lexicon_tags_31 },
    { 0xcb5759e0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd15759e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569759e6, SUC_lexicon_tags_25 },
    { 0x2b3759e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91759eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d759ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3759f2, SUC_lexicon_tags_10 },
    { 0x501759f3, SUC_lexicon_tags_37 },
    { 0x395759f3, SUC_lexicon_tags_25 },
    { 0xcfd759f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b3759f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x417759f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed759fd, SUC_lexicon_tags_43 },
    { 0xfbd759fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af75a04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57175a09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x27d75a0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52775a0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb175a10, SUC_lexicon_tags_55 },
    { 0x45f75a11, SUC_lexicon_tags_31 },
    { 0x10b75a11, SUC_lexicon_tags_17 },
    { 0x45b75a13, SUC_lexicon_tags_47 },
    { 0x92b75a11, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5575a18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d75a22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f575a26, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d975a29, SUC_lexicon_tags_37 },
    { 0x14f75a2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x49975a2c, SUC_lexicon_tags_7 },
    { 0x85775a2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6d75a2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd75a32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9b75a34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad75a38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b75a3f, SUC_lexicon_tags_124 },
    { 0x84375a40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44175a42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa375a45, SUC_lexicon_tags_37 },
    { 0x5d375a46, SUC_lexicon_tags_56 },
    { 0xcf775a47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x70d75a49, SUC_lexicon_tags_15 },
    { 0x34375a4a, SUC_lexicon_tags_37 },
    { 0x57375a4b, SUC_lexicon_tags_47 },
    { 0x63375a4b, SUC_lexicon_tags_15 },
    { 0x20b75a4d, SUC_lexicon_tags_11 },
    { 0x3a575a4e, SUC_lexicon_tags_14 },
    { 0x7f575a4b, SUC_lexicon_tags_11 },
    { 0xa0d75a50, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x175a52, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20175a54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20175a56, SUC_lexicon_tags_11 },
    { 0x85175a56, SUC_lexicon_tags_12 },
    { 0xabd75a56, SUC_lexicon_tags_136 },
    { 0x49975a59, SUC_lexicon_tags_45 },
    { 0x4b975a56, SUC_lexicon_tags_15 },
    { 0x29f75a5a, SUC_lexicon_tags_17 },
    { 0xd7775a5c, SUC_lexicon_tags_60 },
    { 0x46b75a58, SUC_lexicon_tags_21 },
    { 0xa1d75a5e, SUC_lexicon_tags_2 },
    { 0xd4d75a5f, SUC_lexicon_tags_37 },
    { 0x93b75a5f, SUC_lexicon_tags_14 },
    { 0x86775a61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b75a64, SUC_lexicon_tags_25 },
    { 0x32575a65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f775a67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65575a69, SUC_lexicon_tags_11 },
    { 0x92b75a6a, SUC_lexicon_tags_14 },
    { 0xb8b75a6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc775a71, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d75a79, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6d375a7b, SUC_lexicon_tags_25 },
    { 0x70d75a7c, SUC_lexicon_tags_25 },
    { 0x7b575a7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d75a82, SUC_lexicon_tags_25 },
    { 0x92775a82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xab975a85, SUC_lexicon_tags_6 },
    { 0xb6f75a86, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f175a8b, SUC_lexicon_tags_7 },
    { 0x69b75a8c, SUC_lexicon_tags_63 },
    { 0x3d175a8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b75a90, SUC_lexicon_tags_7 },
    { 0x38175a90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75975a96, SUC_lexicon_tags_33 },
    { 0x11575a97, SUC_lexicon_tags_43 },
    { 0x1cb75a98, SUC_lexicon_tags_6 },
    { 0x6ff75a99, SUC_lexicon_tags_797 },
    { 0x12b75a9a, SUC_lexicon_tags_68 },
    { 0x4b175a9b, SUC_lexicon_tags_59 },
    { 0x29175a9c, SUC_lexicon_tags_1 },
    { 0x10975a9b, SUC_lexicon_tags_11 },
    { 0x9cf75a9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8175aa1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64975aad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1775ab3, SUC_lexicon_tags_33 },
    { 0x77975ab4, SUC_lexicon_tags_25 },
    { 0x56175ab5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8175abc, SUC_lexicon_tags_26 },
    { 0x71375abc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xebd75abf, SUC_lexicon_tags_38 },
    { 0x78f75ac0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57975acb, SUC_lexicon_tags_25 },
    { 0x77d75acc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5eb75ace, SUC_lexicon_tags_37 },
    { 0x9e775acf, SUC_lexicon_tags_2 },
    { 0x14375acf, SUC_lexicon_tags_7 },
    { 0x75775ace, SUC_lexicon_tags_17 },
    { 0xf7d75ad1, SUC_lexicon_tags_19 },
    { 0xc7975ad1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7975ad6, SUC_lexicon_tags_56 },
    { 0xc7975ad7, SUC_lexicon_tags_9 },
    { 0x35175ad8, SUC_lexicon_tags_2 },
    { 0xbb75ad7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54575ade, SUC_lexicon_tags_38 },
    { 0xbe775adf, SUC_lexicon_tags_39 },
    { 0xcd375adf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde575ae6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x33375ae9, SUC_lexicon_tags_11 },
    { 0x79775aea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d75aef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa775af1, SUC_lexicon_tags_25 },
    { 0x99775af1, SUC_lexicon_tags_37 },
    { 0xca175af1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb375af7, SUC_lexicon_tags_14 },
    { 0x69b75af7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf3f75afa, SUC_lexicon_tags_11 },
    { 0xf4d75afb, SUC_lexicon_tags_45 },
    { 0xaaf75afb, SUC_lexicon_tags_7 },
    { 0x15175afa, SUC_lexicon_tags_8 },
    { 0x5ad75afe, SUC_lexicon_tags_7 },
    { 0xb9575aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae375b02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa175b04, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x27b75b06, SUC_lexicon_tags_8 },
    { 0x24975b06, SUC_lexicon_tags_39 },
    { 0x35b75b08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd975b0c, SUC_lexicon_tags_14 },
    { 0x58975b0d, SUC_lexicon_tags_3 },
    { 0xde375b0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d75b14, SUC_lexicon_tags_17 },
    { 0x5a775b14, SUC_lexicon_tags_49 },
    { 0x82575b14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d75b19, SUC_lexicon_tags_39 },
    { 0x67d75b19, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59575b21, SUC_lexicon_tags_11 },
    { 0xab775b22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40375b26, SUC_lexicon_tags_49 },
    { 0xee975b27, SUC_lexicon_tags_45 },
    { 0xa9575b28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad375b2e, SUC_lexicon_tags_56 },
    { 0x5d975b2f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x47175b31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d75b35, SUC_lexicon_tags_15 },
    { 0xd0b75b36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf975b3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x72375b3c, SUC_lexicon_tags_29 },
    { 0x36b75b3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84175b46, SUC_lexicon_tags_55 },
    { 0x73375b47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b75b4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1575b4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x76375b4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d75b53, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff75b58, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x25375b5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75175b60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x66375b62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc775b69, SUC_lexicon_tags_37 },
    { 0xb8b75b6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x52575b6c, SUC_lexicon_tags_38 },
    { 0xced75b6d, SUC_lexicon_tags_12 },
    { 0x19f75b6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12175b70, SUC_lexicon_tags_66 },
    { 0x44d75b70, SUC_lexicon_tags_66 },
    { 0xa3375b72, SUC_lexicon_tags_49 },
    { 0x9e775b71, SUC_lexicon_tags_17 },
    { 0x91975b74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2175b77, SUC_lexicon_tags_40 },
    { 0xc4175b78, SUC_lexicon_tags_14 },
    { 0xe9975b77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x24d75b7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba775b7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d75b84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ff75b86, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed75b90, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb175b98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc7575b9a, SUC_lexicon_tags_37 },
    { 0x56775b9a, SUC_lexicon_tags_45 },
    { 0x92975b9c, SUC_lexicon_tags_15 },
    { 0xb7f75b9c, SUC_lexicon_tags_23 },
    { 0x83b75b9a, SUC_lexicon_tags_39 },
    { 0x25775b9f, SUC_lexicon_tags_1 },
    { 0x3cd75ba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3375ba7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f75baa, SUC_lexicon_tags_166 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f75bae, SUC_lexicon_tags_64 },
    { 0xd8375baf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xee375bb1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd75bb7, SUC_lexicon_tags_17 },
    { 0xbc175bb8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x58575bba, SUC_lexicon_tags_25 },
    { 0x52375bba, SUC_lexicon_tags_27 },
    { 0x6bf75bbc, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24575bc3, SUC_lexicon_tags_7 },
    { 0xec775bc4, SUC_lexicon_tags_14 },
    { 0x59575bc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76375bc7, SUC_lexicon_tags_49 },
    { 0xf5775bc8, SUC_lexicon_tags_54 },
    { 0x89b75bc8, SUC_lexicon_tags_16 },
    { 0x90f75bca, SUC_lexicon_tags_60 },
    { 0xd0575bca, SUC_lexicon_tags_23 },
    { 0x91d75bc8, SUC_lexicon_tags_7 },
    { 0x90d75bcb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e775bd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b175bd6, SUC_lexicon_tags_25 },
    { 0xe4375bd7, SUC_lexicon_tags_38 },
    { 0x72f75bd8, SUC_lexicon_tags_37 },
    { 0xa7975bd7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc8575bdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5fb75bdd, SUC_lexicon_tags_63 },
    { 0xcd375bde, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13b75be0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1975be4, SUC_lexicon_tags_17 },
    { 0xf1975be4, SUC_lexicon_tags_15 },
    { 0xef975be4, SUC_lexicon_tags_25 },
    { 0x84175be4, SUC_lexicon_tags_19 },
    { 0xb5975be4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b775bed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf75bf1, SUC_lexicon_tags_17 },
    { 0xd1575bf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8575bf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44175bf6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b75bfc, SUC_lexicon_tags_14 },
    { 0x1c375bfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c975bff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76975c03, SUC_lexicon_tags_12 },
    { 0xc4975c04, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xad375c07, SUC_lexicon_tags_104 },
    { 0x9c375c08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xded75c0b, SUC_lexicon_tags_68 },
    { 0x2b175c0b, SUC_lexicon_tags_17 },
    { 0x50975c0d, SUC_lexicon_tags_26 },
    { 0x33975c0e, SUC_lexicon_tags_2 },
    { 0xc0775c0f, SUC_lexicon_tags_8 },
    { 0xf8175c0e, SUC_lexicon_tags_7 },
    { 0xe7d75c11, SUC_lexicon_tags_25 },
    { 0x25375c11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7375c15, SUC_lexicon_tags_33 },
    { 0xd2f75c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57175c1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25d75c1e, SUC_lexicon_tags_12 },
    { 0x9c175c1e, SUC_lexicon_tags_11 },
    { 0x9d375c20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaa375c22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa175c25, SUC_lexicon_tags_57 },
    { 0xab75c26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54375c29, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0375c2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff75c31, SUC_lexicon_tags_70 },
    { 0xdcf75c31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0175c35, SUC_lexicon_tags_12 },
    { 0x7f775c36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf75c3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97975c3f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d75c43, SUC_lexicon_tags_63 },
    { 0x1a775c44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6af75c46, SUC_lexicon_tags_7 },
    { 0xcad75c46, SUC_lexicon_tags_2 },
    { 0xcc775c48, SUC_lexicon_tags_28 },
    { 0xb1975c48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2d75c4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc575c4f, SUC_lexicon_tags_11 },
    { 0xc8975c50, SUC_lexicon_tags_35 },
    { 0x1375c51, SUC_lexicon_tags_2 },
    { 0x52f75c50, SUC_lexicon_tags_25 },
    { 0x83f75c51, SUC_lexicon_tags_12 },
    { 0x78975c4f, SUC_lexicon_tags_11 },
    { 0xff375c55, SUC_lexicon_tags_1 },
    { 0x55375c56, SUC_lexicon_tags_14 },
    { 0x38375c57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe175c5c, SUC_lexicon_tags_65 },
    { 0xb2d75c5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75d75c5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3575c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e375c68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a775c6e, SUC_lexicon_tags_1 },
    { 0x72175c6f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x21375c71, SUC_lexicon_tags_11 },
    { 0xa1975c72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d75c75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee975c78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a575c7c, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x5a975c7e, SUC_lexicon_tags_11 },
    { 0x37375c7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2575c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d75c88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b775c8b, SUC_lexicon_tags_27 },
    { 0x54775c8c, SUC_lexicon_tags_33 },
    { 0x5e375c8d, SUC_lexicon_tags_11 },
    { 0xc1375c8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc375c91, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d75c94, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6b175c96, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa775c99, SUC_lexicon_tags_12 },
    { 0x9f575c9a, SUC_lexicon_tags_28 },
    { 0x17575c9b, SUC_lexicon_tags_46 },
    { 0x94d75c9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9575c9e, SUC_lexicon_tags_33 },
    { 0xe7575c9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40975ca1, SUC_lexicon_tags_0 },
    { 0x84d75ca2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x65b75ca4, SUC_lexicon_tags_25 },
    { 0x1d775ca4, SUC_lexicon_tags_32 },
    { 0xf8b75ca6, SUC_lexicon_tags_47 },
    { 0x2c775ca6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x70775caa, SUC_lexicon_tags_11 },
    { 0xcf575caa, SUC_lexicon_tags_17 },
    { 0x2d575cac, SUC_lexicon_tags_207 },
    { 0xf7775cad, SUC_lexicon_tags_14 },
    { 0xcfd75cad, SUC_lexicon_tags_17 },
    { 0xf8375caa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b75cb2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5775cb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f75cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1f75cbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4775cbe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d75cc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30775cc4, SUC_lexicon_tags_60 },
    { 0xdb975cc5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6dd75cc7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x15775cc9, SUC_lexicon_tags_25 },
    { 0x2c775cc9, SUC_lexicon_tags_19 },
    { 0x63f75ccb, SUC_lexicon_tags_17 },
    { 0x34375ccc, SUC_lexicon_tags_60 },
    { 0x7b975ccb, SUC_lexicon_tags_27 },
    { 0x34575cce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a175cd5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b75cd8, SUC_lexicon_tags_9 },
    { 0xc6175cd9, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a775ce3, SUC_lexicon_tags_14 },
    { 0x11d75ce4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6375cea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d75cee, SUC_lexicon_tags_25 },
    { 0x73375cee, SUC_lexicon_tags_12 },
    { 0xcd75cf0, SUC_lexicon_tags_1 },
    { 0x70575cf0, SUC_lexicon_tags_38 },
    { 0x9bb75cf2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f75cf5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xad75cf8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f75cfc, SUC_lexicon_tags_11 },
    { 0x39f75cfd, SUC_lexicon_tags_37 },
    { 0x9d775cfe, SUC_lexicon_tags_37 },
    { 0xb1375cff, SUC_lexicon_tags_46 },
    { 0x87575d00, SUC_lexicon_tags_11 },
    { 0x81b75d01, SUC_lexicon_tags_45 },
    { 0xc3575d01, SUC_lexicon_tags_18 },
    { 0xf0b75d03, SUC_lexicon_tags_12 },
    { 0xa8975d04, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xda175d06, SUC_lexicon_tags_14 },
    { 0xa2b75d07, SUC_lexicon_tags_8 },
    { 0xafd75d08, SUC_lexicon_tags_28 },
    { 0x78375d08, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed75d0f, SUC_lexicon_tags_15 },
    { 0x86b75d10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe8775d12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb75d19, SUC_lexicon_tags_7 },
    { 0xe2375d19, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9bd75d1c, SUC_lexicon_tags_15 },
    { 0x53f75d1d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe575d26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x41975d28, SUC_lexicon_tags_49 },
    { 0x3e575d29, SUC_lexicon_tags_26 },
    { 0xf2375d28, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x16575d2c, SUC_lexicon_tags_37 },
    { 0xaa775d2d, SUC_lexicon_tags_46 },
    { 0x72975d2e, SUC_lexicon_tags_46 },
    { 0x45775d2e, SUC_lexicon_tags_37 },
    { 0x51f75d2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x78175d32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8575d35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37575d3a, SUC_lexicon_tags_60 },
    { 0x1fb75d3b, SUC_lexicon_tags_65 },
    { 0xd4375d3c, SUC_lexicon_tags_38 },
    { 0x6c975d3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbab75d3f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb75d43, SUC_lexicon_tags_46 },
    { 0x97175d43, SUC_lexicon_tags_60 },
    { 0x45f75d43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79775d4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd75d4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb75d50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5375d52, SUC_lexicon_tags_12 },
    { 0x71d75d53, SUC_lexicon_tags_1 },
    { 0xee975d54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38175d5a, SUC_lexicon_tags_38 },
    { 0xa8175d5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d75d5e, SUC_lexicon_tags_113 },
    { 0xd3d75d5f, SUC_lexicon_tags_26 },
    { 0x4b175d60, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5375d63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3375d65, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0775d6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf75d72, SUC_lexicon_tags_65 },
    { 0x2ff75d73, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53775d76, SUC_lexicon_tags_25 },
    { 0x53975d77, SUC_lexicon_tags_2 },
    { 0x4f575d78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6175d7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f75d7d, SUC_lexicon_tags_37 },
    { 0x23f75d7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29375d82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a175d86, SUC_lexicon_tags_17 },
    { 0xb5575d87, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbbd75d89, SUC_lexicon_tags_2 },
    { 0xdbb75d8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d75d90, SUC_lexicon_tags_37 },
    { 0x22f75d90, SUC_lexicon_tags_1 },
    { 0x19b75d90, SUC_lexicon_tags_12 },
    { 0xc4975d91, SUC_lexicon_tags_13 },
    { 0xb9f75d90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4975d98, SUC_lexicon_tags_37 },
    { 0x8bd75d99, SUC_lexicon_tags_4 },
    { 0xb8b75d9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6db75d9c, SUC_lexicon_tags_19 },
    { 0x29975d9d, SUC_lexicon_tags_48 },
    { 0xd5175d9d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x36d75da0, SUC_lexicon_tags_15 },
    { 0x57375da0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d75da4, SUC_lexicon_tags_45 },
    { 0x35d75da5, SUC_lexicon_tags_8 },
    { 0x3af75da6, SUC_lexicon_tags_31 },
    { 0xc2975da6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x29975daa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb375dae, SUC_lexicon_tags_7 },
    { 0xec575daf, SUC_lexicon_tags_19 },
    { 0xd2d75db0, SUC_lexicon_tags_11 },
    { 0xe4b75db0, SUC_lexicon_tags_45 },
    { 0xaa175dae, SUC_lexicon_tags_15 },
    { 0xfcf75db2, SUC_lexicon_tags_33 },
    { 0xb775dae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b75dc2, SUC_lexicon_tags_25 },
    { 0xe5575dc3, SUC_lexicon_tags_2 },
    { 0x8ad75dc3, SUC_lexicon_tags_16 },
    { 0xa4775dc2, SUC_lexicon_tags_45 },
    { 0x4a175dc5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86575dcb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf9f75dcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec575dd1, SUC_lexicon_tags_14 },
    { 0x38b75dd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab75ddb, SUC_lexicon_tags_45 },
    { 0xaab75ddc, SUC_lexicon_tags_60 },
    { 0x89375ddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10375de1, SUC_lexicon_tags_29 },
    { 0x69b75de1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7575de5, SUC_lexicon_tags_19 },
    { 0xb7975de6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d75de9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e975deb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f775ded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa6175def, SUC_lexicon_tags_25 },
    { 0xf975def, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb75df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d175df5, SUC_lexicon_tags_2 },
    { 0xc7175df6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef75e01, SUC_lexicon_tags_63 },
    { 0x3c975e02, SUC_lexicon_tags_62 },
    { 0xd6975e03, SUC_lexicon_tags_12 },
    { 0xe6f75e04, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61975e10, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab975e14, SUC_lexicon_tags_16 },
    { 0xd7975e14, SUC_lexicon_tags_14 },
    { 0xeab75e16, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f75e1a, SUC_lexicon_tags_47 },
    { 0xaf75e1b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xae775e1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8175e1f, SUC_lexicon_tags_11 },
    { 0xb75e1f, SUC_lexicon_tags_43 },
    { 0x44975e21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f75e25, SUC_lexicon_tags_1 },
    { 0xd8375e26, SUC_lexicon_tags_19 },
    { 0x775e25, SUC_lexicon_tags_38 },
    { 0xc5975e25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81575e2d, SUC_lexicon_tags_25 },
    { 0xea175e2e, SUC_lexicon_tags_10 },
    { 0x80975e2e, SUC_lexicon_tags_11 },
    { 0x1b975e30, SUC_lexicon_tags_14 },
    { 0xbc975e31, SUC_lexicon_tags_25 },
    { 0xb775e2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b75e37, SUC_lexicon_tags_32 },
    { 0xd75e37, SUC_lexicon_tags_31 },
    { 0x6a575e39, SUC_lexicon_tags_17 },
    { 0xbe975e39, SUC_lexicon_tags_6 },
    { 0x75975e38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d75e3e, SUC_lexicon_tags_39 },
    { 0xdd75e3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20175e42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb75e46, SUC_lexicon_tags_25 },
    { 0xfcb75e47, SUC_lexicon_tags_6 },
    { 0xb1d75e48, SUC_lexicon_tags_19 },
    { 0x8c775e48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b975e50, SUC_lexicon_tags_7 },
    { 0x5cd75e51, SUC_lexicon_tags_17 },
    { 0xaef75e50, SUC_lexicon_tags_11 },
    { 0x9e575e53, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd5f75e55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b75e5c, SUC_lexicon_tags_17 },
    { 0x94f75e5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80775e61, SUC_lexicon_tags_28 },
    { 0xfd175e62, SUC_lexicon_tags_11 },
    { 0x9c775e63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60775e66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b75e6a, SUC_lexicon_tags_37 },
    { 0x12d75e6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff775e6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3375e71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5775e76, SUC_lexicon_tags_27 },
    { 0x40575e76, SUC_lexicon_tags_27 },
    { 0xa9375e78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1175e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30375e81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf175e83, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2a175e85, SUC_lexicon_tags_17 },
    { 0x9af75e86, SUC_lexicon_tags_63 },
    { 0x76975e86, SUC_lexicon_tags_37 },
    { 0xd1f75e88, SUC_lexicon_tags_30 },
    { 0xdb175e89, SUC_lexicon_tags_7 },
    { 0x2bb75e85, SUC_lexicon_tags_11 },
    { 0xfbd75e88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2375e8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd0575e90, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a975e94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55975e98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58375e9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74975ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5575ea7, SUC_lexicon_tags_7 },
    { 0x36b75ea8, SUC_lexicon_tags_23 },
    { 0x15575ea7, SUC_lexicon_tags_8 },
    { 0xc3b75eaa, SUC_lexicon_tags_19 },
    { 0x33b75eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54775ead, SUC_lexicon_tags_49 },
    { 0xa4975eae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d75eb1, SUC_lexicon_tags_11 },
    { 0x1d175eb2, SUC_lexicon_tags_27 },
    { 0xcbf75eb3, SUC_lexicon_tags_3 },
    { 0x13775eb4, SUC_lexicon_tags_14 },
    { 0x4d775eb5, SUC_lexicon_tags_25 },
    { 0xdb775eb6, SUC_lexicon_tags_572 },
    { 0x8bb75eb6, SUC_lexicon_tags_26 },
    { 0xc1175eb8, SUC_lexicon_tags_7 },
    { 0xa1175eb8, SUC_lexicon_tags_30 },
    { 0xf7d75eb4, SUC_lexicon_tags_19 },
    { 0x2975ebb, SUC_lexicon_tags_26 },
    { 0x4a175eba, SUC_lexicon_tags_8 },
    { 0x92175ebd, SUC_lexicon_tags_49 },
    { 0x4b375ebe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b75ec2, SUC_lexicon_tags_27 },
    { 0x4f75ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf75ec8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f75ecd, SUC_lexicon_tags_88 },
    { 0x1f375ece, SUC_lexicon_tags_10 },
    { 0xe9175ecf, SUC_lexicon_tags_27 },
    { 0xdd75ed0, SUC_lexicon_tags_60 },
    { 0xc8f75ed1, SUC_lexicon_tags_46 },
    { 0xb8b75ed0, SUC_lexicon_tags_47 },
    { 0xcd575ecd, SUC_lexicon_tags_12 },
    { 0x3ed75ed4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c375ed6, SUC_lexicon_tags_38 },
    { 0x72d75ed7, SUC_lexicon_tags_2 },
    { 0x11975ed6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d75edd, SUC_lexicon_tags_15 },
    { 0x96175edd, SUC_lexicon_tags_12 },
    { 0xe4d75edf, SUC_lexicon_tags_17 },
    { 0xb7d75edf, SUC_lexicon_tags_11 },
    { 0xf7575ee1, SUC_lexicon_tags_1 },
    { 0x65775ee2, SUC_lexicon_tags_28 },
    { 0x6e975ee2, SUC_lexicon_tags_160 },
    { 0x82175ee3, SUC_lexicon_tags_39 },
    { 0x94d75ee3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xfdf75ee7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f75eec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf775eee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab75efd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf75f04, SUC_lexicon_tags_0 },
    { 0x41175f05, SUC_lexicon_tags_0 },
    { 0x4f775f06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb75f09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a775f0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1d75f0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f175f10, SUC_lexicon_tags_37 },
    { 0xd8775f10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc775f13, SUC_lexicon_tags_49 },
    { 0x82f75f14, SUC_lexicon_tags_0 },
    { 0x1bf75f15, SUC_lexicon_tags_7 },
    { 0x7b75f16, SUC_lexicon_tags_7 },
    { 0x79775f17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9b75f19, SUC_lexicon_tags_49 },
    { 0x65975f1a, SUC_lexicon_tags_7 },
    { 0xfff75f1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c175f1d, SUC_lexicon_tags_633 },
    { 0, NULL },
    { 0, NULL },
    { 0xca175f20, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80575f24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab575f2c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65975f33, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8975f38, SUC_lexicon_tags_25 },
    { 0x2df75f39, SUC_lexicon_tags_37 },
    { 0xf1175f3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83775f3c, SUC_lexicon_tags_0 },
    { 0x7c575f3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f75f43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x68b75f45, SUC_lexicon_tags_2 },
    { 0xc5b75f46, SUC_lexicon_tags_28 },
    { 0x40375f45, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6375f4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c975f4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15775f52, SUC_lexicon_tags_38 },
    { 0xc3175f53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed575f58, SUC_lexicon_tags_204 },
    { 0xbe975f58, SUC_lexicon_tags_15 },
    { 0x2b575f5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78775f5e, SUC_lexicon_tags_32 },
    { 0x83b75f5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa775f62, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x54f75f64, SUC_lexicon_tags_4 },
    { 0xf0175f64, SUC_lexicon_tags_18 },
    { 0x26175f66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f375f6b, SUC_lexicon_tags_11 },
    { 0x67175f6c, SUC_lexicon_tags_43 },
    { 0xc6575f6c, SUC_lexicon_tags_2 },
    { 0x1ef75f6b, SUC_lexicon_tags_23 },
    { 0xe0575f6c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe2175f71, SUC_lexicon_tags_77 },
    { 0x10775f72, SUC_lexicon_tags_4 },
    { 0xb1975f72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6575f78, SUC_lexicon_tags_63 },
    { 0xd7975f78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4f75f7b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5775f81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfbf75f83, SUC_lexicon_tags_68 },
    { 0x75975f84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45175f89, SUC_lexicon_tags_26 },
    { 0xc1d75f8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79975f8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92175f90, SUC_lexicon_tags_25 },
    { 0x30375f91, SUC_lexicon_tags_33 },
    { 0x7375f91, SUC_lexicon_tags_37 },
    { 0x30b75f90, SUC_lexicon_tags_6 },
    { 0x1f175f94, SUC_lexicon_tags_37 },
    { 0x3b975f95, SUC_lexicon_tags_1703 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf75f9a, SUC_lexicon_tags_47 },
    { 0x5e375f9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc575f9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f75fa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d75fa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f75fab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5975fae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef75fb3, SUC_lexicon_tags_11 },
    { 0x74775fb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x97575fb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52575fb9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d75fbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7175fc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2375fc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7575fc9, SUC_lexicon_tags_12 },
    { 0x9a375fc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6575fcc, SUC_lexicon_tags_25 },
    { 0x79d75fcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4775fd0, SUC_lexicon_tags_10 },
    { 0xf5375fd1, SUC_lexicon_tags_47 },
    { 0x4ab75fd2, SUC_lexicon_tags_28 },
    { 0x65775fd3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x19575fd6, SUC_lexicon_tags_8 },
    { 0xccf75fd7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1375fda, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x92575fdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b75fe0, SUC_lexicon_tags_15 },
    { 0x15175fe1, SUC_lexicon_tags_11 },
    { 0xa9175fe0, SUC_lexicon_tags_201 },
    { 0x7b175fe0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b175fea, SUC_lexicon_tags_37 },
    { 0x8f375feb, SUC_lexicon_tags_17 },
    { 0x71775fec, SUC_lexicon_tags_11 },
    { 0xc1375fed, SUC_lexicon_tags_16 },
    { 0x95d75fec, SUC_lexicon_tags_17 },
    { 0x31b75fef, SUC_lexicon_tags_282 },
    { 0xb0d75ff0, SUC_lexicon_tags_7 },
    { 0x37d75ff1, SUC_lexicon_tags_47 },
    { 0x26375ff2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x34375ff5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd75ffb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad175ffd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad76000, SUC_lexicon_tags_47 },
    { 0x8a376001, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9376003, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x56776006, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d576009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d7600c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc237600f, SUC_lexicon_tags_62 },
    { 0xac376010, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca176015, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf717601e, SUC_lexicon_tags_11 },
    { 0x6677601e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb3376021, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a176024, SUC_lexicon_tags_11 },
    { 0x65d76025, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67576027, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0xb3576029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c97602f, SUC_lexicon_tags_17 },
    { 0xa517602f, SUC_lexicon_tags_65 },
    { 0xc6976031, SUC_lexicon_tags_23 },
    { 0xc1176032, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6b76034, SUC_lexicon_tags_25 },
    { 0xb3f76035, SUC_lexicon_tags_49 },
    { 0x18176036, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b76039, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5d7603b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9877603d, SUC_lexicon_tags_2 },
    { 0xbf17603e, SUC_lexicon_tags_25 },
    { 0x8297603f, SUC_lexicon_tags_47 },
    { 0xf0b76040, SUC_lexicon_tags_1 },
    { 0xd7f76040, SUC_lexicon_tags_38 },
    { 0xfa976040, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b76045, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d7604a, SUC_lexicon_tags_26 },
    { 0x397604b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc77604f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc176052, SUC_lexicon_tags_17 },
    { 0x1ef76053, SUC_lexicon_tags_1 },
    { 0x4ad76054, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec376059, SUC_lexicon_tags_25 },
    { 0x35b7605a, SUC_lexicon_tags_47 },
    { 0x34f7605b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa277605e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13576062, SUC_lexicon_tags_33 },
    { 0xf2576062, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24776068, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb77606d, SUC_lexicon_tags_641 },
    { 0xe157606d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2576071, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeb376073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f76076, SUC_lexicon_tags_17 },
    { 0x40576077, SUC_lexicon_tags_33 },
    { 0x21176077, SUC_lexicon_tags_63 },
    { 0xa9d76078, SUC_lexicon_tags_47 },
    { 0x90d76077, SUC_lexicon_tags_376 },
    { 0x2077607b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b7607f, SUC_lexicon_tags_25 },
    { 0xd2176080, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0d76082, SUC_lexicon_tags_27 },
    { 0xaf776083, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d76088, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f7608c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd77608e, SUC_lexicon_tags_11 },
    { 0x3817608f, SUC_lexicon_tags_17 },
    { 0x2897608e, SUC_lexicon_tags_17 },
    { 0x29776090, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad76097, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4376099, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1760a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33760a9, SUC_lexicon_tags_21 },
    { 0x5d760aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d760ad, SUC_lexicon_tags_1 },
    { 0x3f9760ae, SUC_lexicon_tags_25 },
    { 0x13760af, SUC_lexicon_tags_17 },
    { 0x865760b0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9760b8, SUC_lexicon_tags_14 },
    { 0x7ed760b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x963760bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5760bf, SUC_lexicon_tags_12 },
    { 0x877760c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x599760c3, SUC_lexicon_tags_33 },
    { 0xab5760c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4df760c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23760cb, SUC_lexicon_tags_70 },
    { 0x803760cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaab760ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff760d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d760d8, SUC_lexicon_tags_39 },
    { 0xc33760d9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5760e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf760eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29760ee, SUC_lexicon_tags_11 },
    { 0x3760ef, SUC_lexicon_tags_1 },
    { 0x59d760ee, SUC_lexicon_tags_8 },
    { 0x6ad760f1, SUC_lexicon_tags_25 },
    { 0x6fd760f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc69760f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd760f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba5760f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b760fc, SUC_lexicon_tags_4 },
    { 0x635760fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd1760ff, SUC_lexicon_tags_11 },
    { 0x17f76100, SUC_lexicon_tags_1 },
    { 0x71d760ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f76105, SUC_lexicon_tags_11 },
    { 0x1db76106, SUC_lexicon_tags_56 },
    { 0x61f76106, SUC_lexicon_tags_16 },
    { 0xf1f76106, SUC_lexicon_tags_25 },
    { 0x7e376109, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f76110, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf976113, SUC_lexicon_tags_2 },
    { 0x4576114, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68176116, SUC_lexicon_tags_4 },
    { 0x38f76116, SUC_lexicon_tags_29 },
    { 0x5eb76118, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7077611a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb76120, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4576124, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8eb76126, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x88376128, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7612b, SUC_lexicon_tags_10 },
    { 0xd5f7612c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33176130, SUC_lexicon_tags_37 },
    { 0x18d76130, SUC_lexicon_tags_82 },
    { 0x87176132, SUC_lexicon_tags_7 },
    { 0xf0776133, SUC_lexicon_tags_17 },
    { 0xa2776133, SUC_lexicon_tags_60 },
    { 0x51d76134, SUC_lexicon_tags_27 },
    { 0x39f76133, SUC_lexicon_tags_38 },
    { 0x1a376137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe037613e, SUC_lexicon_tags_12 },
    { 0x8577613f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc176141, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f76144, SUC_lexicon_tags_47 },
    { 0x3a176145, SUC_lexicon_tags_17 },
    { 0xe3d76144, SUC_lexicon_tags_2 },
    { 0xcfd76145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7977614d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96b7614f, SUC_lexicon_tags_55 },
    { 0x58976150, SUC_lexicon_tags_45 },
    { 0x6c376150, SUC_lexicon_tags_17 },
    { 0xe7b76150, SUC_lexicon_tags_13 },
    { 0x20b76153, SUC_lexicon_tags_63 },
    { 0xa8b76154, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7dd76156, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a976159, SUC_lexicon_tags_1 },
    { 0xc377615a, SUC_lexicon_tags_14 },
    { 0x51f7615a, SUC_lexicon_tags_17 },
    { 0xfef7615a, SUC_lexicon_tags_37 },
    { 0x5ab7615d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c176162, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c176164, SUC_lexicon_tags_15 },
    { 0x88576164, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11776169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b77616b, SUC_lexicon_tags_151 },
    { 0x2317616c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7616f, SUC_lexicon_tags_37 },
    { 0x3ab76170, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9db76172, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7176175, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7b76177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1797617a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf176184, SUC_lexicon_tags_37 },
    { 0x76f76185, SUC_lexicon_tags_4 },
    { 0xd1b76186, SUC_lexicon_tags_92 },
    { 0x5d976184, SUC_lexicon_tags_104 },
    { 0xbd376186, SUC_lexicon_tags_49 },
    { 0x69776185, SUC_lexicon_tags_12 },
    { 0x2ab7618a, SUC_lexicon_tags_29 },
    { 0xdb97618b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf437618d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b176190, SUC_lexicon_tags_37 },
    { 0xc2976191, SUC_lexicon_tags_26 },
    { 0x52176192, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74176198, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a17619d, SUC_lexicon_tags_15 },
    { 0x13f7619e, SUC_lexicon_tags_12 },
    { 0x7bd7619f, SUC_lexicon_tags_6 },
    { 0xaf761a0, SUC_lexicon_tags_8 },
    { 0xfa5761a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x483761a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5761aa, SUC_lexicon_tags_15 },
    { 0xdd761ab, SUC_lexicon_tags_12 },
    { 0x27761aa, SUC_lexicon_tags_7 },
    { 0xa97761ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x983761b0, SUC_lexicon_tags_60 },
    { 0xbf761b0, SUC_lexicon_tags_43 },
    { 0xa79761b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef761b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13761be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b761c4, SUC_lexicon_tags_5 },
    { 0x6af761c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87d761c7, SUC_lexicon_tags_28 },
    { 0x379761c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105761d2, SUC_lexicon_tags_17 },
    { 0x953761d3, SUC_lexicon_tags_15 },
    { 0x323761d4, SUC_lexicon_tags_19 },
    { 0x15761d4, SUC_lexicon_tags_26 },
    { 0xf8b761d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14d761d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7761db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91761e0, SUC_lexicon_tags_16 },
    { 0xd6f761e0, SUC_lexicon_tags_68 },
    { 0xd8b761e2, SUC_lexicon_tags_17 },
    { 0x45761e3, SUC_lexicon_tags_7 },
    { 0x3a7761e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f761e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3761eb, SUC_lexicon_tags_31 },
    { 0xa17761ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7761ef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4a3761f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92f761f3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65761f7, SUC_lexicon_tags_46 },
    { 0xd29761f8, SUC_lexicon_tags_50 },
    { 0x689761f8, SUC_lexicon_tags_49 },
    { 0x671761fa, SUC_lexicon_tags_35 },
    { 0x89d761fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x923761fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25976200, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2376208, SUC_lexicon_tags_11 },
    { 0x1d176208, SUC_lexicon_tags_17 },
    { 0x117620a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8357620d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e976212, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef376217, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f776219, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc97621d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b76224, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d76228, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x55f7622a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69376234, SUC_lexicon_tags_0 },
    { 0x22b76235, SUC_lexicon_tags_151 },
    { 0x91376235, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5597623d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cb7623f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4576241, SUC_lexicon_tags_35 },
    { 0xb9976242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f76250, SUC_lexicon_tags_12 },
    { 0x65976251, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11376255, SUC_lexicon_tags_32 },
    { 0x55176256, SUC_lexicon_tags_17 },
    { 0x6a176257, SUC_lexicon_tags_2 },
    { 0xe6f76258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef17625a, SUC_lexicon_tags_25 },
    { 0xe017625b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78d7625d, SUC_lexicon_tags_45 },
    { 0x8f7625e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60376263, SUC_lexicon_tags_37 },
    { 0x87176264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad76267, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf7626b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb357626d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25576270, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4576276, SUC_lexicon_tags_1 },
    { 0x9f176276, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cb76279, SUC_lexicon_tags_31 },
    { 0xbbb76279, SUC_lexicon_tags_12 },
    { 0x58376279, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb7627f, SUC_lexicon_tags_2 },
    { 0xeab76280, SUC_lexicon_tags_33 },
    { 0xd7627f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e176283, SUC_lexicon_tags_1 },
    { 0x24176284, SUC_lexicon_tags_45 },
    { 0xbad76285, SUC_lexicon_tags_6 },
    { 0xd9376283, SUC_lexicon_tags_26 },
    { 0x9c376283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9576290, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b976297, SUC_lexicon_tags_3 },
    { 0xe7d76297, SUC_lexicon_tags_25 },
    { 0xfb376298, SUC_lexicon_tags_37 },
    { 0x7f176297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d7629f, SUC_lexicon_tags_17 },
    { 0xa45762a0, SUC_lexicon_tags_17 },
    { 0x6b7762a0, SUC_lexicon_tags_1 },
    { 0x23f762a2, SUC_lexicon_tags_29 },
    { 0xfe5762a0, SUC_lexicon_tags_45 },
    { 0x5177629f, SUC_lexicon_tags_33 },
    { 0xaa9762a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9762a9, SUC_lexicon_tags_0 },
    { 0x24b762a9, SUC_lexicon_tags_6 },
    { 0x5c5762aa, SUC_lexicon_tags_55 },
    { 0x6dd762aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x439762ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b762b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c7762b4, SUC_lexicon_tags_26 },
    { 0x38f762b4, SUC_lexicon_tags_17 },
    { 0x5d5762b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd762ba, SUC_lexicon_tags_27 },
    { 0x4c1762bb, SUC_lexicon_tags_13 },
    { 0x84f762ba, SUC_lexicon_tags_11 },
    { 0xa2f762bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd75762bf, SUC_lexicon_tags_17 },
    { 0x951762c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f762c3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x76f762c5, SUC_lexicon_tags_8 },
    { 0xf71762c6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d762cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa21762cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed7762d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9762d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5762da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7762e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x631762e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f762e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x70f762e8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443762ec, SUC_lexicon_tags_37 },
    { 0x71d762ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f762f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337762f5, SUC_lexicon_tags_7 },
    { 0xc3f762f5, SUC_lexicon_tags_26 },
    { 0xac3762f7, SUC_lexicon_tags_17 },
    { 0x395762f5, SUC_lexicon_tags_37 },
    { 0xb762f9, SUC_lexicon_tags_12 },
    { 0xf91762fa, SUC_lexicon_tags_37 },
    { 0x601762f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1762fe, SUC_lexicon_tags_17 },
    { 0x33d762ff, SUC_lexicon_tags_14 },
    { 0x2f976300, SUC_lexicon_tags_1 },
    { 0xfb576300, SUC_lexicon_tags_45 },
    { 0x33576302, SUC_lexicon_tags_0 },
    { 0x6b576303, SUC_lexicon_tags_27 },
    { 0xd93762ff, SUC_lexicon_tags_26 },
    { 0x7f976305, SUC_lexicon_tags_62 },
    { 0xcb176305, SUC_lexicon_tags_5 },
    { 0x80776305, SUC_lexicon_tags_46 },
    { 0x87376304, SUC_lexicon_tags_45 },
    { 0x56176309, SUC_lexicon_tags_1 },
    { 0xff17630a, SUC_lexicon_tags_2 },
    { 0xacb76306, SUC_lexicon_tags_56 },
    { 0x4f776309, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4fd7630e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb76312, SUC_lexicon_tags_61 },
    { 0x54376313, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f76319, SUC_lexicon_tags_84 },
    { 0x50176319, SUC_lexicon_tags_29 },
    { 0x60576319, SUC_lexicon_tags_2 },
    { 0xa3b7631c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd76320, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5176323, SUC_lexicon_tags_37 },
    { 0x39376324, SUC_lexicon_tags_32 },
    { 0xd1376325, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f76328, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8f7632a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d176334, SUC_lexicon_tags_37 },
    { 0xe4776334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad76339, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8257633c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbef7633e, SUC_lexicon_tags_47 },
    { 0x9057633f, SUC_lexicon_tags_14 },
    { 0x1d376340, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70976344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97976347, SUC_lexicon_tags_37 },
    { 0xa9176348, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x18b7634a, SUC_lexicon_tags_55 },
    { 0x41b7634b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x397634d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96776358, SUC_lexicon_tags_30 },
    { 0x26576359, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f7635c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f76364, SUC_lexicon_tags_48 },
    { 0xf9176364, SUC_lexicon_tags_39 },
    { 0x8bf76365, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa576369, SUC_lexicon_tags_31 },
    { 0x897636a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc976373, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xae176376, SUC_lexicon_tags_0 },
    { 0x59d76377, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb7637c, SUC_lexicon_tags_60 },
    { 0x37637d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6776382, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b76385, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56b76387, SUC_lexicon_tags_2 },
    { 0x44b76387, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5717638e, SUC_lexicon_tags_8 },
    { 0xcc37638f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d76394, SUC_lexicon_tags_12 },
    { 0x7f576395, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b7639a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6837639f, SUC_lexicon_tags_10 },
    { 0x281763a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9763a9, SUC_lexicon_tags_47 },
    { 0x147763aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x195763ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7763b1, SUC_lexicon_tags_17 },
    { 0x247763b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83763b8, SUC_lexicon_tags_0 },
    { 0x43763b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bb763bb, SUC_lexicon_tags_29 },
    { 0xffd763bc, SUC_lexicon_tags_21 },
    { 0x719763bc, SUC_lexicon_tags_46 },
    { 0x415763be, SUC_lexicon_tags_27 },
    { 0x931763be, SUC_lexicon_tags_68 },
    { 0x2b3763bf, SUC_lexicon_tags_33 },
    { 0xa23763c1, SUC_lexicon_tags_4 },
    { 0x4ab763c2, SUC_lexicon_tags_23 },
    { 0x5a3763c3, SUC_lexicon_tags_50 },
    { 0xf01763c3, SUC_lexicon_tags_25 },
    { 0x13d763c5, SUC_lexicon_tags_30 },
    { 0x383763c6, SUC_lexicon_tags_47 },
    { 0x13b763bb, SUC_lexicon_tags_66 },
    { 0x47763c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x177763ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x629763cd, SUC_lexicon_tags_12 },
    { 0x9d9763ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3763d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2bb763d8, SUC_lexicon_tags_2 },
    { 0x227763d9, SUC_lexicon_tags_27 },
    { 0x7e1763d9, SUC_lexicon_tags_11 },
    { 0x8e3763db, SUC_lexicon_tags_39 },
    { 0x3c3763dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55763df, SUC_lexicon_tags_23 },
    { 0xa99763e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94f763e2, SUC_lexicon_tags_30 },
    { 0xfdd763e3, SUC_lexicon_tags_37 },
    { 0x751763e3, SUC_lexicon_tags_9 },
    { 0xef9763e4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27763ea, SUC_lexicon_tags_17 },
    { 0xec3763eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd763ef, SUC_lexicon_tags_10 },
    { 0x77763f0, SUC_lexicon_tags_75 },
    { 0x417763f1, SUC_lexicon_tags_23 },
    { 0x4d763f2, SUC_lexicon_tags_1 },
    { 0x43f763f3, SUC_lexicon_tags_25 },
    { 0xc41763f1, SUC_lexicon_tags_12 },
    { 0x217763f5, SUC_lexicon_tags_15 },
    { 0x3fb763f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xedf763f8, SUC_lexicon_tags_60 },
    { 0xfd1763f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x371763fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49d763fe, SUC_lexicon_tags_11 },
    { 0x847763fe, SUC_lexicon_tags_17 },
    { 0xc5b76400, SUC_lexicon_tags_25 },
    { 0x21976401, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf76404, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74776409, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b76410, SUC_lexicon_tags_25 },
    { 0xbe976411, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db76415, SUC_lexicon_tags_11 },
    { 0xeb576416, SUC_lexicon_tags_2 },
    { 0x90776416, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9037641a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5637641c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5157641f, SUC_lexicon_tags_25 },
    { 0xddb76420, SUC_lexicon_tags_2 },
    { 0x5b7641f, SUC_lexicon_tags_60 },
    { 0x39376422, SUC_lexicon_tags_534 },
    { 0, NULL },
    { 0x33b76424, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3376428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf77642a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d77642c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1837642f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66376433, SUC_lexicon_tags_35 },
    { 0x23b76434, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1976438, SUC_lexicon_tags_2 },
    { 0x54d76439, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa617643b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66576444, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d76449, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b7644c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb417644f, SUC_lexicon_tags_37 },
    { 0x6bf76450, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44176458, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbef7645a, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3237645e, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d76461, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba976466, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b97646e, SUC_lexicon_tags_16 },
    { 0x4cb7646f, SUC_lexicon_tags_2 },
    { 0x9a176470, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaaf76472, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebd76474, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df7647a, SUC_lexicon_tags_7 },
    { 0x9df7647b, SUC_lexicon_tags_17 },
    { 0x7d97647b, SUC_lexicon_tags_14 },
    { 0x7ad7647c, SUC_lexicon_tags_28 },
    { 0xf1b7647b, SUC_lexicon_tags_10 },
    { 0x53d7647f, SUC_lexicon_tags_15 },
    { 0x6576480, SUC_lexicon_tags_25 },
    { 0xd4f76480, SUC_lexicon_tags_15 },
    { 0x6a97647e, SUC_lexicon_tags_65 },
    { 0x91776483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51576485, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xef576487, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f7648b, SUC_lexicon_tags_37 },
    { 0x89b7648b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x14f7648e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa576492, SUC_lexicon_tags_19 },
    { 0xa6376493, SUC_lexicon_tags_66 },
    { 0x9af76494, SUC_lexicon_tags_14 },
    { 0xbb76494, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42976499, SUC_lexicon_tags_32 },
    { 0x29b7649a, SUC_lexicon_tags_15 },
    { 0x35b7649b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d7649f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669764a4, SUC_lexicon_tags_29 },
    { 0x2ef764a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f764a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53764b5, SUC_lexicon_tags_25 },
    { 0x7d9764b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321764bc, SUC_lexicon_tags_60 },
    { 0xb4b764bd, SUC_lexicon_tags_19 },
    { 0xfed764be, SUC_lexicon_tags_0 },
    { 0x5a7764bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f9764c1, SUC_lexicon_tags_37 },
    { 0x547764c2, SUC_lexicon_tags_48 },
    { 0xf8d764c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305764cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af764cf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f764d7, SUC_lexicon_tags_7 },
    { 0x683764d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf764dd, SUC_lexicon_tags_0 },
    { 0x77f764de, SUC_lexicon_tags_45 },
    { 0x1c5764de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x335764e1, SUC_lexicon_tags_17 },
    { 0x8e7764e2, SUC_lexicon_tags_37 },
    { 0xfb5764e1, SUC_lexicon_tags_2 },
    { 0xa83764e1, SUC_lexicon_tags_1 },
    { 0x6e5764e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x78f764e7, SUC_lexicon_tags_39 },
    { 0xdd7764e8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b764eb, SUC_lexicon_tags_395 },
    { 0xe87764eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe3f764ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a3764f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x255764f3, SUC_lexicon_tags_37 },
    { 0xc9f764f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92176500, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72776503, SUC_lexicon_tags_47 },
    { 0x24576504, SUC_lexicon_tags_45 },
    { 0x8cb76504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b7650a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc817650d, SUC_lexicon_tags_17 },
    { 0x1217650e, SUC_lexicon_tags_68 },
    { 0x9c57650f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf576513, SUC_lexicon_tags_25 },
    { 0xfb976513, SUC_lexicon_tags_26 },
    { 0x2cf76515, SUC_lexicon_tags_9 },
    { 0x69976516, SUC_lexicon_tags_38 },
    { 0x55576517, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe617651d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x797651f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf976522, SUC_lexicon_tags_54 },
    { 0x9cf76522, SUC_lexicon_tags_19 },
    { 0xe3d76523, SUC_lexicon_tags_17 },
    { 0x55976524, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4d76527, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce37652c, SUC_lexicon_tags_485 },
    { 0x69d7652d, SUC_lexicon_tags_15 },
    { 0x1537652c, SUC_lexicon_tags_12 },
    { 0x52d7652d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff576533, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f76537, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x33376539, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd217653d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1077653f, SUC_lexicon_tags_1 },
    { 0x8af76540, SUC_lexicon_tags_6 },
    { 0xa6376541, SUC_lexicon_tags_2 },
    { 0x83176541, SUC_lexicon_tags_25 },
    { 0x6e376543, SUC_lexicon_tags_33 },
    { 0xd8976540, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd1776546, SUC_lexicon_tags_5 },
    { 0xc5976547, SUC_lexicon_tags_1 },
    { 0xc776547, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x557654a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5176551, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x57776554, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71d76556, SUC_lexicon_tags_33 },
    { 0x7a376557, SUC_lexicon_tags_15 },
    { 0xab76558, SUC_lexicon_tags_5 },
    { 0xad776557, SUC_lexicon_tags_47 },
    { 0xa576557, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xc537655d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb76560, SUC_lexicon_tags_27 },
    { 0x67976561, SUC_lexicon_tags_2 },
    { 0x3d576562, SUC_lexicon_tags_28 },
    { 0xb9176562, SUC_lexicon_tags_25 },
    { 0x80b76564, SUC_lexicon_tags_27 },
    { 0x3db76565, SUC_lexicon_tags_87 },
    { 0xde376566, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf937656b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaaf7656d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff77656f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7776571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7976576, SUC_lexicon_tags_47 },
    { 0x9d76576, SUC_lexicon_tags_25 },
    { 0xf6176578, SUC_lexicon_tags_43 },
    { 0x78376577, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xaef7657b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf77657e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb6576580, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16776584, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x10576586, SUC_lexicon_tags_49 },
    { 0x2176587, SUC_lexicon_tags_26 },
    { 0xd1f76587, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b7658c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc137658f, SUC_lexicon_tags_88 },
    { 0x20176590, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbad76592, SUC_lexicon_tags_17 },
    { 0xa3976593, SUC_lexicon_tags_27 },
    { 0x12976593, SUC_lexicon_tags_61 },
    { 0xfe176594, SUC_lexicon_tags_12 },
    { 0xe0f76596, SUC_lexicon_tags_25 },
    { 0x21d76594, SUC_lexicon_tags_15 },
    { 0xad376598, SUC_lexicon_tags_13 },
    { 0x53d76595, SUC_lexicon_tags_28 },
    { 0xeed7659a, SUC_lexicon_tags_16 },
    { 0x2cb7659b, SUC_lexicon_tags_25 },
    { 0x32176592, SUC_lexicon_tags_17 },
    { 0x72b76599, SUC_lexicon_tags_8 },
    { 0x1817659e, SUC_lexicon_tags_1 },
    { 0xea976599, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d765a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3765a9, SUC_lexicon_tags_30 },
    { 0x3ed765aa, SUC_lexicon_tags_14 },
    { 0xa3d765aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1765af, SUC_lexicon_tags_37 },
    { 0xfff765af, SUC_lexicon_tags_75 },
    { 0xebb765b1, SUC_lexicon_tags_21 },
    { 0x1ff765b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ad765b4, SUC_lexicon_tags_37 },
    { 0xc4f765b5, SUC_lexicon_tags_11 },
    { 0xc69765b4, SUC_lexicon_tags_7 },
    { 0xae1765b5, SUC_lexicon_tags_1 },
    { 0x2c3765b8, SUC_lexicon_tags_37 },
    { 0xdb3765b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb765bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173765c1, SUC_lexicon_tags_23 },
    { 0x789765c2, SUC_lexicon_tags_25 },
    { 0xae9765c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659765ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae7765d0, SUC_lexicon_tags_17 },
    { 0x287765d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41765d7, SUC_lexicon_tags_7 },
    { 0xd81765d8, SUC_lexicon_tags_30 },
    { 0xe61765d9, SUC_lexicon_tags_0 },
    { 0x9bb765da, SUC_lexicon_tags_49 },
    { 0xfbf765db, SUC_lexicon_tags_15 },
    { 0xec1765da, SUC_lexicon_tags_23 },
    { 0x8b9765db, SUC_lexicon_tags_37 },
    { 0x6b765de, SUC_lexicon_tags_47 },
    { 0x679765df, SUC_lexicon_tags_87 },
    { 0x1b9765e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x285765e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86f765e5, SUC_lexicon_tags_33 },
    { 0x561765e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x999765e8, SUC_lexicon_tags_38 },
    { 0xe3765e9, SUC_lexicon_tags_43 },
    { 0x965765ea, SUC_lexicon_tags_19 },
    { 0xcdd765e9, SUC_lexicon_tags_23 },
    { 0xe53765ea, SUC_lexicon_tags_27 },
    { 0xb8f765ed, SUC_lexicon_tags_8 },
    { 0x765765ee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5765f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691765f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151765fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x373765fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f76601, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63576605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b76608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b7660c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9377660e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb76611, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9776617, SUC_lexicon_tags_11 },
    { 0x16b76618, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1697661b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83b7661d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x80576620, SUC_lexicon_tags_60 },
    { 0x7b976620, SUC_lexicon_tags_49 },
    { 0xc0f76621, SUC_lexicon_tags_25 },
    { 0xb0376623, SUC_lexicon_tags_79 },
    { 0xf4376624, SUC_lexicon_tags_0 },
    { 0x8e776624, SUC_lexicon_tags_30 },
    { 0xd0376620, SUC_lexicon_tags_34 },
    { 0x44f76627, SUC_lexicon_tags_7 },
    { 0xf4776627, SUC_lexicon_tags_11 },
    { 0x1b576627, SUC_lexicon_tags_7 },
    { 0x56b76627, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5576632, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7576639, SUC_lexicon_tags_4 },
    { 0x18576639, SUC_lexicon_tags_32 },
    { 0xa3576639, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf7663e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e376643, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b376648, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c37664d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99376650, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9d76652, SUC_lexicon_tags_17 },
    { 0xf1376652, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db76656, SUC_lexicon_tags_37 },
    { 0xcfb76656, SUC_lexicon_tags_7 },
    { 0x93176658, SUC_lexicon_tags_23 },
    { 0xbbd76658, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7757665f, SUC_lexicon_tags_25 },
    { 0xfd576660, SUC_lexicon_tags_33 },
    { 0x7576661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7176664, SUC_lexicon_tags_15 },
    { 0x5fb76665, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b376667, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf157666b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf77666e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b376674, SUC_lexicon_tags_47 },
    { 0x91f76674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee77667d, SUC_lexicon_tags_65 },
    { 0x2ab7667e, SUC_lexicon_tags_0 },
    { 0x9757667f, SUC_lexicon_tags_17 },
    { 0x6157667d, SUC_lexicon_tags_25 },
    { 0x7a376681, SUC_lexicon_tags_30 },
    { 0xb9b76682, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97776688, SUC_lexicon_tags_37 },
    { 0xb8b76689, SUC_lexicon_tags_2 },
    { 0x7597668a, SUC_lexicon_tags_37 },
    { 0x5737668b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b7668f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f76693, SUC_lexicon_tags_13 },
    { 0x58f76694, SUC_lexicon_tags_33 },
    { 0x8c376695, SUC_lexicon_tags_45 },
    { 0xf9976695, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d7669b, SUC_lexicon_tags_7 },
    { 0xb1d7669c, SUC_lexicon_tags_0 },
    { 0xb8d7669d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71766a1, SUC_lexicon_tags_45 },
    { 0xbff766a2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x9a5766a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1766a9, SUC_lexicon_tags_19 },
    { 0xbeb766aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x541766ac, SUC_lexicon_tags_47 },
    { 0xd55766ad, SUC_lexicon_tags_7 },
    { 0xeab766ac, SUC_lexicon_tags_69 },
    { 0x67f766ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x221766b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71766b6, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0x367766b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939766bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f766c0, SUC_lexicon_tags_10 },
    { 0x5db766c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781766c5, SUC_lexicon_tags_7 },
    { 0xb43766c6, SUC_lexicon_tags_3 },
    { 0xbaf766c7, SUC_lexicon_tags_23 },
    { 0xbdf766c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3a9766ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8fd766cc, SUC_lexicon_tags_37 },
    { 0x4f3766cd, SUC_lexicon_tags_1 },
    { 0x2f5766ce, SUC_lexicon_tags_2 },
    { 0x29f766cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f766d5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2cd766d7, SUC_lexicon_tags_15 },
    { 0x4d3766d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413766de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74d766e0, SUC_lexicon_tags_37 },
    { 0xdf7766e1, SUC_lexicon_tags_17 },
    { 0x2cb766e2, SUC_lexicon_tags_9 },
    { 0xc4b766e1, SUC_lexicon_tags_8 },
    { 0x2f3766e4, SUC_lexicon_tags_70 },
    { 0x487766e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c1766e7, SUC_lexicon_tags_7 },
    { 0x517766e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7766ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdcf766ec, SUC_lexicon_tags_38 },
    { 0xa21766ed, SUC_lexicon_tags_11 },
    { 0x25b766ee, SUC_lexicon_tags_11 },
    { 0x87b766ef, SUC_lexicon_tags_46 },
    { 0xc15766f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6ab766f2, SUC_lexicon_tags_25 },
    { 0xd39766f3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xecd766f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7766fb, SUC_lexicon_tags_25 },
    { 0xf77766fb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b76706, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46d76708, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db7670b, SUC_lexicon_tags_1 },
    { 0x6ed7670c, SUC_lexicon_tags_7 },
    { 0x8eb7670b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58776712, SUC_lexicon_tags_17 },
    { 0x3d376713, SUC_lexicon_tags_15 },
    { 0xcfb76712, SUC_lexicon_tags_16 },
    { 0xcf576715, SUC_lexicon_tags_37 },
    { 0xaa176716, SUC_lexicon_tags_15 },
    { 0x59976717, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad17671a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f57671d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94f7671f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b76724, SUC_lexicon_tags_11 },
    { 0x6ad76725, SUC_lexicon_tags_4 },
    { 0x79376726, SUC_lexicon_tags_0 },
    { 0xc0376724, SUC_lexicon_tags_36 },
    { 0x9f976724, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec97672e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bb76730, SUC_lexicon_tags_26 },
    { 0x8376731, SUC_lexicon_tags_56 },
    { 0x65176732, SUC_lexicon_tags_17 },
    { 0xa4176733, SUC_lexicon_tags_37 },
    { 0x1a776733, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19576737, SUC_lexicon_tags_9 },
    { 0x54b76737, SUC_lexicon_tags_45 },
    { 0x48576738, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1097673e, SUC_lexicon_tags_1 },
    { 0x7477673f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d576744, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc576748, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9177674b, SUC_lexicon_tags_17 },
    { 0x87f7674c, SUC_lexicon_tags_55 },
    { 0xbf37674b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf76752, SUC_lexicon_tags_55 },
    { 0x6ff76753, SUC_lexicon_tags_37 },
    { 0xc2976754, SUC_lexicon_tags_63 },
    { 0xbd576755, SUC_lexicon_tags_47 },
    { 0x3fb76756, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9776758, SUC_lexicon_tags_7 },
    { 0x77176759, SUC_lexicon_tags_14 },
    { 0xceb7675a, SUC_lexicon_tags_25 },
    { 0xa6d7675b, SUC_lexicon_tags_47 },
    { 0xb637675c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa7f7675e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb76761, SUC_lexicon_tags_25 },
    { 0x9f776762, SUC_lexicon_tags_2 },
    { 0x70376763, SUC_lexicon_tags_71 },
    { 0xe2b76761, SUC_lexicon_tags_46 },
    { 0xc5976761, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d76768, SUC_lexicon_tags_25 },
    { 0xf0776768, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2776772, SUC_lexicon_tags_42 },
    { 0xd9576772, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3e776775, SUC_lexicon_tags_25 },
    { 0x9976775, SUC_lexicon_tags_15 },
    { 0xc7976777, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d7677a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b97677c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf757677f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86f76781, SUC_lexicon_tags_65 },
    { 0x2f776782, SUC_lexicon_tags_248 },
    { 0, NULL },
    { 0, NULL },
    { 0xea176785, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc376787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f97678d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d76794, SUC_lexicon_tags_8 },
    { 0x75f76795, SUC_lexicon_tags_46 },
    { 0x9d576795, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6976798, SUC_lexicon_tags_43 },
    { 0x20376799, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb957679c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb767a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b767a5, SUC_lexicon_tags_2 },
    { 0xd4b767a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b1767a8, SUC_lexicon_tags_31 },
    { 0x345767a9, SUC_lexicon_tags_23 },
    { 0xcc3767aa, SUC_lexicon_tags_15 },
    { 0x491767ab, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5767ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561767bf, SUC_lexicon_tags_68 },
    { 0xcd7767c0, SUC_lexicon_tags_23 },
    { 0x1767c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369767c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57767cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x335767cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25767d1, SUC_lexicon_tags_27 },
    { 0xc5b767d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3767d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d767db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x119767dd, SUC_lexicon_tags_49 },
    { 0xb1767de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf767e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13767e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f767eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f767ee, SUC_lexicon_tags_11 },
    { 0x1e3767ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f767f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73f767f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5767fc, SUC_lexicon_tags_21 },
    { 0xff3767fd, SUC_lexicon_tags_38 },
    { 0x5ed767fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82976801, SUC_lexicon_tags_37 },
    { 0xe6f76802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53376804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35176807, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4477680f, SUC_lexicon_tags_49 },
    { 0xba176810, SUC_lexicon_tags_2 },
    { 0x6a576810, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a976813, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8576815, SUC_lexicon_tags_0 },
    { 0x31776816, SUC_lexicon_tags_25 },
    { 0x45376817, SUC_lexicon_tags_25 },
    { 0x2576816, SUC_lexicon_tags_31 },
    { 0x6b776819, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b7681d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x3517681f, SUC_lexicon_tags_37 },
    { 0x3577681f, SUC_lexicon_tags_37 },
    { 0x42b76821, SUC_lexicon_tags_48 },
    { 0xe176822, SUC_lexicon_tags_38 },
    { 0x50576822, SUC_lexicon_tags_27 },
    { 0xf8576821, SUC_lexicon_tags_11 },
    { 0xd8d76823, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb976827, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd757682c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d76830, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d7683a, SUC_lexicon_tags_37 },
    { 0xab7683b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde77683d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc176840, SUC_lexicon_tags_0 },
    { 0xc4976841, SUC_lexicon_tags_18 },
    { 0x5ef76842, SUC_lexicon_tags_37 },
    { 0xddb76843, SUC_lexicon_tags_0 },
    { 0xd5376844, SUC_lexicon_tags_49 },
    { 0x73776845, SUC_lexicon_tags_47 },
    { 0x3cd76844, SUC_lexicon_tags_11 },
    { 0x9ab76846, SUC_lexicon_tags_25 },
    { 0x3c976845, SUC_lexicon_tags_30 },
    { 0xecf76847, SUC_lexicon_tags_15 },
    { 0x3797684a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67976852, SUC_lexicon_tags_230 },
    { 0xf3b76853, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x72776855, SUC_lexicon_tags_27 },
    { 0xf7b76855, SUC_lexicon_tags_63 },
    { 0x69176857, SUC_lexicon_tags_14 },
    { 0xadf76858, SUC_lexicon_tags_30 },
    { 0xef976856, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d76862, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a176864, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x37d76866, SUC_lexicon_tags_18 },
    { 0x4c776867, SUC_lexicon_tags_68 },
    { 0x19576866, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b7686c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73776872, SUC_lexicon_tags_15 },
    { 0x5e776873, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8b76875, SUC_lexicon_tags_12 },
    { 0x60176875, SUC_lexicon_tags_33 },
    { 0xe4b76875, SUC_lexicon_tags_37 },
    { 0x75f76875, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21f7687a, SUC_lexicon_tags_47 },
    { 0x8417687a, SUC_lexicon_tags_37 },
    { 0x7a37687c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6f7687e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f76882, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6817688b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6357688e, SUC_lexicon_tags_62 },
    { 0xd97688f, SUC_lexicon_tags_11 },
    { 0xc7f76890, SUC_lexicon_tags_17 },
    { 0x33976891, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb76895, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17576897, SUC_lexicon_tags_25 },
    { 0x34776898, SUC_lexicon_tags_63 },
    { 0x76576899, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a57689c, SUC_lexicon_tags_15 },
    { 0xad97689d, SUC_lexicon_tags_63 },
    { 0xfeb7689e, SUC_lexicon_tags_17 },
    { 0xa717689c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdef768a1, SUC_lexicon_tags_0 },
    { 0x485768a2, SUC_lexicon_tags_31 },
    { 0x393768a2, SUC_lexicon_tags_17 },
    { 0xc17768a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9768aa, SUC_lexicon_tags_49 },
    { 0x145768ab, SUC_lexicon_tags_2 },
    { 0x2d7768ac, SUC_lexicon_tags_7 },
    { 0x603768ab, SUC_lexicon_tags_17 },
    { 0xec1768ae, SUC_lexicon_tags_27 },
    { 0x355768ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5768b2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7768b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931768c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99768c4, SUC_lexicon_tags_66 },
    { 0x2f1768c5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b768ca, SUC_lexicon_tags_17 },
    { 0xb45768cb, SUC_lexicon_tags_103 },
    { 0xf9b768ca, SUC_lexicon_tags_30 },
    { 0x63768cd, SUC_lexicon_tags_61 },
    { 0x20d768cc, SUC_lexicon_tags_8 },
    { 0xe81768cf, SUC_lexicon_tags_12 },
    { 0xac9768cc, SUC_lexicon_tags_32 },
    { 0x79f768d1, SUC_lexicon_tags_29 },
    { 0x737768ce, SUC_lexicon_tags_66 },
    { 0xdcd768ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7cd768d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x651768d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad768da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f768dd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x391768df, SUC_lexicon_tags_12 },
    { 0x19d768e0, SUC_lexicon_tags_11 },
    { 0xe41768e1, SUC_lexicon_tags_11 },
    { 0x585768df, SUC_lexicon_tags_49 },
    { 0xed9768df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1768e6, SUC_lexicon_tags_11 },
    { 0x2eb768e7, SUC_lexicon_tags_17 },
    { 0x96f768e8, SUC_lexicon_tags_37 },
    { 0x7f1768e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe21768eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x213768ee, SUC_lexicon_tags_60 },
    { 0xa2d768ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d768f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565768f8, SUC_lexicon_tags_16 },
    { 0xdc3768f9, SUC_lexicon_tags_23 },
    { 0xe51768fa, SUC_lexicon_tags_12 },
    { 0x67d768f9, SUC_lexicon_tags_27 },
    { 0xea9768f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb768ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd576901, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c976904, SUC_lexicon_tags_37 },
    { 0x84376904, SUC_lexicon_tags_63 },
    { 0x3fb76906, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f76909, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb497690b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe077690d, SUC_lexicon_tags_12 },
    { 0x7077690e, SUC_lexicon_tags_32 },
    { 0x697690f, SUC_lexicon_tags_33 },
    { 0x5e37690f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4d76912, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x95576914, SUC_lexicon_tags_25 },
    { 0xfc576915, SUC_lexicon_tags_40 },
    { 0x82776915, SUC_lexicon_tags_2 },
    { 0x1b976917, SUC_lexicon_tags_463 },
    { 0x55776918, SUC_lexicon_tags_5 },
    { 0x83b76915, SUC_lexicon_tags_17 },
    { 0x31d7691a, SUC_lexicon_tags_11 },
    { 0x2b97691a, SUC_lexicon_tags_12 },
    { 0xf57691c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77776921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e176924, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6377692b, SUC_lexicon_tags_37 },
    { 0x3c77692b, SUC_lexicon_tags_25 },
    { 0x1237692d, SUC_lexicon_tags_66 },
    { 0x7b57692e, SUC_lexicon_tags_68 },
    { 0x9177692f, SUC_lexicon_tags_14 },
    { 0x42b76930, SUC_lexicon_tags_38 },
    { 0x8577692d, SUC_lexicon_tags_37 },
    { 0x6737692b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29d76934, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58176939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3d7693b, SUC_lexicon_tags_48 },
    { 0xd157693c, SUC_lexicon_tags_43 },
    { 0x13f7693c, SUC_lexicon_tags_12 },
    { 0xb277693b, SUC_lexicon_tags_53 },
    { 0x1377693f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23376941, SUC_lexicon_tags_37 },
    { 0xa1576941, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a976945, SUC_lexicon_tags_12 },
    { 0xadf76946, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6f76948, SUC_lexicon_tags_3 },
    { 0x88576949, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a37694c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f77694e, SUC_lexicon_tags_234 },
    { 0xef37694f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5576956, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c17695b, SUC_lexicon_tags_28 },
    { 0x3557695c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6917695e, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a976961, SUC_lexicon_tags_22 },
    { 0x21b76962, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b76967, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8577696b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66176978, SUC_lexicon_tags_28 },
    { 0x3976979, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba376980, SUC_lexicon_tags_10 },
    { 0x4b776981, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3e576983, SUC_lexicon_tags_26 },
    { 0x59376984, SUC_lexicon_tags_1 },
    { 0xbfd76985, SUC_lexicon_tags_11 },
    { 0x78d76986, SUC_lexicon_tags_47 },
    { 0x3d976986, SUC_lexicon_tags_11 },
    { 0xd2b76987, SUC_lexicon_tags_74 },
    { 0xf6976989, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b76993, SUC_lexicon_tags_11 },
    { 0xe4776994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397699c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23769a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe53769a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa87769a4, SUC_lexicon_tags_47 },
    { 0x78d769a5, SUC_lexicon_tags_25 },
    { 0xa4b769a6, SUC_lexicon_tags_43 },
    { 0xac5769a6, SUC_lexicon_tags_19 },
    { 0x465769a7, SUC_lexicon_tags_12 },
    { 0x243769a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb1b769ab, SUC_lexicon_tags_48 },
    { 0xf4d769ac, SUC_lexicon_tags_37 },
    { 0x10f769ad, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xc71769af, SUC_lexicon_tags_37 },
    { 0x9bd769b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d9769b2, SUC_lexicon_tags_25 },
    { 0x6a1769b2, SUC_lexicon_tags_25 },
    { 0x82f769b2, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f769b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b3769b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39769bd, SUC_lexicon_tags_48 },
    { 0x775769bd, SUC_lexicon_tags_30 },
    { 0xc5f769bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9769c5, SUC_lexicon_tags_30 },
    { 0x755769c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc91769c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x289769cb, SUC_lexicon_tags_47 },
    { 0x1f7769cb, SUC_lexicon_tags_2 },
    { 0xf31769cd, SUC_lexicon_tags_25 },
    { 0x38b769ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15769d2, SUC_lexicon_tags_87 },
    { 0xec9769d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfbf769d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b769d8, SUC_lexicon_tags_6 },
    { 0x8c7769d9, SUC_lexicon_tags_1 },
    { 0x831769da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b1769dc, SUC_lexicon_tags_0 },
    { 0x73d769dc, SUC_lexicon_tags_60 },
    { 0x861769dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e9769e0, SUC_lexicon_tags_26 },
    { 0x8bf769e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x249769e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4b769e5, SUC_lexicon_tags_12 },
    { 0x871769e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x301769e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467769ee, SUC_lexicon_tags_43 },
    { 0x54b769ef, SUC_lexicon_tags_57 },
    { 0xb73769ef, SUC_lexicon_tags_4 },
    { 0x8f7769f1, SUC_lexicon_tags_2 },
    { 0x909769f1, SUC_lexicon_tags_25 },
    { 0xa17769f3, SUC_lexicon_tags_25 },
    { 0x265769f4, SUC_lexicon_tags_4 },
    { 0x743769f4, SUC_lexicon_tags_15 },
    { 0xd6b769f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1769fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50376a00, SUC_lexicon_tags_23 },
    { 0xead76a01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f776a03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59376a0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87776a0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8d976a10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x69976a12, SUC_lexicon_tags_17 },
    { 0x77776a13, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5d776a15, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78176a1a, SUC_lexicon_tags_27 },
    { 0xf6b76a1a, SUC_lexicon_tags_11 },
    { 0xb3976a1c, SUC_lexicon_tags_49 },
    { 0x16776a1d, SUC_lexicon_tags_3 },
    { 0xdb976a1e, SUC_lexicon_tags_82 },
    { 0x4bf76a1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xac376a22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5dd76a24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2b76a26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7df76a28, SUC_lexicon_tags_37 },
    { 0xe1d76a28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa776a2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda376a30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d976a40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd776a44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75176a46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6576a51, SUC_lexicon_tags_15 },
    { 0xdb976a52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8b76a54, SUC_lexicon_tags_17 },
    { 0x33d76a55, SUC_lexicon_tags_4 },
    { 0x80176a55, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d76a5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd376a5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd76a62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb76a68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b776a6f, SUC_lexicon_tags_1 },
    { 0xcf76a6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a576a7a, SUC_lexicon_tags_0 },
    { 0x34f76a7b, SUC_lexicon_tags_43 },
    { 0x5eb76a7b, SUC_lexicon_tags_130 },
    { 0x8a176a7d, SUC_lexicon_tags_138 },
    { 0x75d76a7e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b176a80, SUC_lexicon_tags_11 },
    { 0xc776a81, SUC_lexicon_tags_12 },
    { 0x72376a82, SUC_lexicon_tags_2 },
    { 0xb4d76a83, SUC_lexicon_tags_46 },
    { 0xb9776a82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b176a8a, SUC_lexicon_tags_14 },
    { 0x7c576a8b, SUC_lexicon_tags_19 },
    { 0x89576a8c, SUC_lexicon_tags_68 },
    { 0x19576a8d, SUC_lexicon_tags_47 },
    { 0x6e976a8b, SUC_lexicon_tags_17 },
    { 0x8d376a8f, SUC_lexicon_tags_6 },
    { 0x1f176a90, SUC_lexicon_tags_11 },
    { 0xd4d76a8f, SUC_lexicon_tags_48 },
    { 0x63576a90, SUC_lexicon_tags_11 },
    { 0x93376a93, SUC_lexicon_tags_19 },
    { 0xbbb76a93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5376a97, SUC_lexicon_tags_34 },
    { 0xcf776a98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e576a9f, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0xae576aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad76aad, SUC_lexicon_tags_43 },
    { 0x17d76aad, SUC_lexicon_tags_14 },
    { 0x41b76aaf, SUC_lexicon_tags_1 },
    { 0x17b76ab0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x57376ab2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4376ab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8376ab8, SUC_lexicon_tags_28 },
    { 0xa2176ab9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb76abe, SUC_lexicon_tags_15 },
    { 0x33f76abe, SUC_lexicon_tags_17 },
    { 0xaed76ac0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d76ac6, SUC_lexicon_tags_37 },
    { 0x28d76ac6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d76aca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac776acc, SUC_lexicon_tags_31 },
    { 0xc3976acc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x10f76acf, SUC_lexicon_tags_7 },
    { 0x29376ad0, SUC_lexicon_tags_25 },
    { 0x8c576ad0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d376ad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f176adc, SUC_lexicon_tags_37 },
    { 0xc0776adc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e776ae2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b576ae6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2576ae9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x96176aec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48176afb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0376b02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e976b05, SUC_lexicon_tags_25 },
    { 0xad576b06, SUC_lexicon_tags_15 },
    { 0x87376b07, SUC_lexicon_tags_25 },
    { 0x96176b05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f76b0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10976b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63976b11, SUC_lexicon_tags_2 },
    { 0x25976b12, SUC_lexicon_tags_33 },
    { 0xf9176b13, SUC_lexicon_tags_7 },
    { 0xc4976b14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3176b17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f76b1c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1376b1f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7ad76b21, SUC_lexicon_tags_2 },
    { 0x27576b21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60976b24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad76b27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f76b2e, SUC_lexicon_tags_37 },
    { 0x5ff76b2f, SUC_lexicon_tags_25 },
    { 0xcc776b2e, SUC_lexicon_tags_28 },
    { 0xd5576b2f, SUC_lexicon_tags_2 },
    { 0x70376b32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea376b35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f776b39, SUC_lexicon_tags_15 },
    { 0x1776b39, SUC_lexicon_tags_1 },
    { 0xbc576b3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6976b3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c776b42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d76b49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe3576b4b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3776b4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b76b51, SUC_lexicon_tags_27 },
    { 0xc0576b51, SUC_lexicon_tags_87 },
    { 0x36776b52, SUC_lexicon_tags_60 },
    { 0x58776b54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb776b56, SUC_lexicon_tags_37 },
    { 0x26376b57, SUC_lexicon_tags_2 },
    { 0x32176b57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5976b5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a976b5f, SUC_lexicon_tags_32 },
    { 0x27b76b60, SUC_lexicon_tags_2 },
    { 0xddf76b61, SUC_lexicon_tags_14 },
    { 0xbd576b62, SUC_lexicon_tags_55 },
    { 0x39976b62, SUC_lexicon_tags_23 },
    { 0xa8b76b63, SUC_lexicon_tags_0 },
    { 0x3576b65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc376b68, SUC_lexicon_tags_37 },
    { 0x91f76b68, SUC_lexicon_tags_7 },
    { 0xbeb76b69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e576b6c, SUC_lexicon_tags_64 },
    { 0xf0d76b6d, SUC_lexicon_tags_85 },
    { 0x89f76b6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8f76b70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca776b75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95f76b77, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8376b7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d976b7f, SUC_lexicon_tags_1249 },
    { 0x96b76b80, SUC_lexicon_tags_17 },
    { 0x73376b81, SUC_lexicon_tags_15 },
    { 0xe1176b82, SUC_lexicon_tags_25 },
    { 0xf4176b83, SUC_lexicon_tags_17 },
    { 0x4a376b84, SUC_lexicon_tags_12 },
    { 0xc6d76b82, SUC_lexicon_tags_45 },
    { 0xa3376b86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x37776b88, SUC_lexicon_tags_2 },
    { 0x70576b89, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xef576b8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0776b8d, SUC_lexicon_tags_11 },
    { 0x27376b8e, SUC_lexicon_tags_32 },
    { 0xbe576b8d, SUC_lexicon_tags_6 },
    { 0x8db76b8e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x93176b93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f76b99, SUC_lexicon_tags_26 },
    { 0x26b76b9a, SUC_lexicon_tags_25 },
    { 0x84f76b9a, SUC_lexicon_tags_15 },
    { 0xc8776b9c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xce176b9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43776ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6576ba3, SUC_lexicon_tags_14 },
    { 0x2b976ba3, SUC_lexicon_tags_49 },
    { 0xb8176ba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d76ba9, SUC_lexicon_tags_37 },
    { 0x9af76baa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0376bad, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d76bb0, SUC_lexicon_tags_29 },
    { 0x22976bb1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5176bb4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8176bba, SUC_lexicon_tags_63 },
    { 0xd4376bbb, SUC_lexicon_tags_17 },
    { 0x95f76bbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d976bbf, SUC_lexicon_tags_1 },
    { 0x80576bc0, SUC_lexicon_tags_17 },
    { 0xa5176bc1, SUC_lexicon_tags_17 },
    { 0xec776bc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab976bc6, SUC_lexicon_tags_6 },
    { 0x9e776bc7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5576bc9, SUC_lexicon_tags_27 },
    { 0xeef76bca, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xab976bcd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b76bd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27b76bd2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfdb76bd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f76bda, SUC_lexicon_tags_26 },
    { 0x71b76bdb, SUC_lexicon_tags_47 },
    { 0x35b76bdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92976bde, SUC_lexicon_tags_38 },
    { 0xfa176bdf, SUC_lexicon_tags_8 },
    { 0x51176be0, SUC_lexicon_tags_19 },
    { 0xdc576bdf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6376be6, SUC_lexicon_tags_32 },
    { 0x5f976be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x576bea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b76bed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6cb76bef, SUC_lexicon_tags_33 },
    { 0x8b776bf0, SUC_lexicon_tags_2 },
    { 0xe6176bf1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1e176bf3, SUC_lexicon_tags_17 },
    { 0xe1176bf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6376bf7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x25576bf9, SUC_lexicon_tags_17 },
    { 0x73f76bf9, SUC_lexicon_tags_68 },
    { 0xe1576bfa, SUC_lexicon_tags_11 },
    { 0xc3f76bf9, SUC_lexicon_tags_17 },
    { 0x15976bfd, SUC_lexicon_tags_1 },
    { 0x44d76bfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51176c04, SUC_lexicon_tags_89 },
    { 0x21176c05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95376c0e, SUC_lexicon_tags_14 },
    { 0xd0176c0f, SUC_lexicon_tags_16 },
    { 0x8c976c10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b776c14, SUC_lexicon_tags_37 },
    { 0x83976c15, SUC_lexicon_tags_33 },
    { 0x84d76c16, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc4376c18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55776c1a, SUC_lexicon_tags_15 },
    { 0x4b376c1b, SUC_lexicon_tags_37 },
    { 0xf2576c1a, SUC_lexicon_tags_42 },
    { 0xf8176c1d, SUC_lexicon_tags_136 },
    { 0xef576c1d, SUC_lexicon_tags_25 },
    { 0x55576c1a, SUC_lexicon_tags_25 },
    { 0xeb176c20, SUC_lexicon_tags_15 },
    { 0x1ff76c21, SUC_lexicon_tags_11 },
    { 0x3ff76c21, SUC_lexicon_tags_82 },
    { 0x67d76c21, SUC_lexicon_tags_63 },
    { 0xc0776c22, SUC_lexicon_tags_1 },
    { 0xb5176c22, SUC_lexicon_tags_17 },
    { 0x5eb76c23, SUC_lexicon_tags_7 },
    { 0x4b776c24, SUC_lexicon_tags_1 },
    { 0x4fd76c28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe7976c2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55176c2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42776c2f, SUC_lexicon_tags_1 },
    { 0x65976c30, SUC_lexicon_tags_37 },
    { 0x3ad76c31, SUC_lexicon_tags_15 },
    { 0xc0d76c2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65376c38, SUC_lexicon_tags_84 },
    { 0x3f776c39, SUC_lexicon_tags_27 },
    { 0x10f76c3a, SUC_lexicon_tags_37 },
    { 0x18f76c3b, SUC_lexicon_tags_6 },
    { 0x5f376c3b, SUC_lexicon_tags_7 },
    { 0x18d76c3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4176c41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce576c43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2cd76c45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae576c49, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16976c4d, SUC_lexicon_tags_25 },
    { 0xd5776c4d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x99976c50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37976c54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d76c57, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x42d76c59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcad76c5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98776c5f, SUC_lexicon_tags_43 },
    { 0xceb76c60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64776c62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b76c69, SUC_lexicon_tags_46 },
    { 0x9cb76c69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5976c6d, SUC_lexicon_tags_37 },
    { 0xcc176c6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x23776c71, SUC_lexicon_tags_2 },
    { 0xe9f76c72, SUC_lexicon_tags_37 },
    { 0x4776c73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db76c78, SUC_lexicon_tags_11 },
    { 0x39b76c78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f576c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde376c83, SUC_lexicon_tags_61 },
    { 0x80776c84, SUC_lexicon_tags_15 },
    { 0x21176c85, SUC_lexicon_tags_30 },
    { 0x8ab76c84, SUC_lexicon_tags_11 },
    { 0x176c84, SUC_lexicon_tags_37 },
    { 0x22d76c85, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5b376c8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffd76c8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0776c8e, SUC_lexicon_tags_15 },
    { 0x2c176c8f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x17376c91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85976c93, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5c176c95, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85176c99, SUC_lexicon_tags_37 },
    { 0x5d76c9a, SUC_lexicon_tags_17 },
    { 0x4376c9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xded76c9e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3976ca4, SUC_lexicon_tags_0 },
    { 0xacb76ca5, SUC_lexicon_tags_17 },
    { 0x59176ca5, SUC_lexicon_tags_1 },
    { 0x74176ca7, SUC_lexicon_tags_37 },
    { 0x30b76ca5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x46376caa, SUC_lexicon_tags_12 },
    { 0xb1576cab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ad76cad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f76cb5, SUC_lexicon_tags_3 },
    { 0x97176cb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1776cbc, SUC_lexicon_tags_26 },
    { 0x50176cbd, SUC_lexicon_tags_47 },
    { 0x8dd76cbe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8976cc4, SUC_lexicon_tags_2 },
    { 0x31176cc5, SUC_lexicon_tags_1 },
    { 0xdeb76cc6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55176ccd, SUC_lexicon_tags_28 },
    { 0x6a976cce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b76cd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8376cd5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e576cde, SUC_lexicon_tags_14 },
    { 0xe9176cdf, SUC_lexicon_tags_60 },
    { 0xf5376cdf, SUC_lexicon_tags_11 },
    { 0x6b76ce1, SUC_lexicon_tags_11 },
    { 0x78976ce2, SUC_lexicon_tags_8 },
    { 0x44976ce0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34976ce7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24f76ce9, SUC_lexicon_tags_2 },
    { 0xd4d76cea, SUC_lexicon_tags_65 },
    { 0x46376ce9, SUC_lexicon_tags_11 },
    { 0x80376cec, SUC_lexicon_tags_13 },
    { 0xd2f76ced, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0176cf1, SUC_lexicon_tags_43 },
    { 0x6b776cf1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d76cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0376cf8, SUC_lexicon_tags_25 },
    { 0xb8776cf9, SUC_lexicon_tags_33 },
    { 0xca576cfa, SUC_lexicon_tags_14 },
    { 0xba176cfb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb76d02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f76d05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c776d0a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b76d0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe8376d0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf76d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdd76d14, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f76d19, SUC_lexicon_tags_15 },
    { 0x56f76d19, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4376d1e, SUC_lexicon_tags_37 },
    { 0x82776d1e, SUC_lexicon_tags_11 },
    { 0x5976d20, SUC_lexicon_tags_6 },
    { 0xf6176d21, SUC_lexicon_tags_4 },
    { 0x64f76d21, SUC_lexicon_tags_7 },
    { 0x42d76d23, SUC_lexicon_tags_11 },
    { 0x9fb76d1f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x92576d26, SUC_lexicon_tags_15 },
    { 0x96376d27, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47776d2e, SUC_lexicon_tags_15 },
    { 0x93576d2e, SUC_lexicon_tags_1 },
    { 0x1a576d2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af76d34, SUC_lexicon_tags_11 },
    { 0x16576d34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a576d3b, SUC_lexicon_tags_19 },
    { 0x47776d3b, SUC_lexicon_tags_15 },
    { 0x6d976d3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xca976d3f, SUC_lexicon_tags_3 },
    { 0x1fb76d3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4776d44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b376d4d, SUC_lexicon_tags_1 },
    { 0x50376d4e, SUC_lexicon_tags_33 },
    { 0xe2f76d4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d76d53, SUC_lexicon_tags_2 },
    { 0xea576d53, SUC_lexicon_tags_33 },
    { 0xbdd76d55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d76d59, SUC_lexicon_tags_11 },
    { 0xab76d59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e376d63, SUC_lexicon_tags_2 },
    { 0x35d76d64, SUC_lexicon_tags_37 },
    { 0xfb176d65, SUC_lexicon_tags_11 },
    { 0x29576d65, SUC_lexicon_tags_37 },
    { 0x99976d65, SUC_lexicon_tags_16 },
    { 0xd0976d68, SUC_lexicon_tags_1 },
    { 0x77376d69, SUC_lexicon_tags_12 },
    { 0xa4976d6a, SUC_lexicon_tags_0 },
    { 0xfb976d68, SUC_lexicon_tags_28 },
    { 0x3c176d65, SUC_lexicon_tags_28 },
    { 0x3bd76d66, SUC_lexicon_tags_14 },
    { 0x61376d66, SUC_lexicon_tags_15 },
    { 0x10776d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b76d75, SUC_lexicon_tags_15 },
    { 0xff576d75, SUC_lexicon_tags_27 },
    { 0x1af76d77, SUC_lexicon_tags_1 },
    { 0xf6f76d78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77776d7a, SUC_lexicon_tags_11 },
    { 0x54f76d7b, SUC_lexicon_tags_33 },
    { 0x2f76d7c, SUC_lexicon_tags_2 },
    { 0xe7976d7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b376d7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x33d76d81, SUC_lexicon_tags_29 },
    { 0xb3f76d81, SUC_lexicon_tags_47 },
    { 0x8a376d83, SUC_lexicon_tags_37 },
    { 0xe176d84, SUC_lexicon_tags_55 },
    { 0x19776d85, SUC_lexicon_tags_37 },
    { 0xd9b76d86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2776d8c, SUC_lexicon_tags_46 },
    { 0xf5d76d8d, SUC_lexicon_tags_7 },
    { 0xca976d8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x18176d91, SUC_lexicon_tags_33 },
    { 0x87f76d92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe176d9e, SUC_lexicon_tags_25 },
    { 0x80b76d9f, SUC_lexicon_tags_25 },
    { 0x77176d9f, SUC_lexicon_tags_79 },
    { 0x1cf76da1, SUC_lexicon_tags_17 },
    { 0x1f376da2, SUC_lexicon_tags_17 },
    { 0xdc776da0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d76da6, SUC_lexicon_tags_11 },
    { 0x47b76da7, SUC_lexicon_tags_15 },
    { 0x50776da7, SUC_lexicon_tags_14 },
    { 0xc5776da9, SUC_lexicon_tags_1 },
    { 0x8bb76daa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b576dac, SUC_lexicon_tags_37 },
    { 0xb376dad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5776db3, SUC_lexicon_tags_31 },
    { 0xc0176db4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x64376db7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f76dbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd76dc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30376dc2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d76dc5, SUC_lexicon_tags_64 },
    { 0x6fd76dc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18776dc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7576dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca176dcd, SUC_lexicon_tags_9 },
    { 0x29776dcd, SUC_lexicon_tags_44 },
    { 0xd3b76dcf, SUC_lexicon_tags_14 },
    { 0xe1776dcf, SUC_lexicon_tags_11 },
    { 0x3bd76dd1, SUC_lexicon_tags_2 },
    { 0xbf376dd2, SUC_lexicon_tags_47 },
    { 0x6bd76dd3, SUC_lexicon_tags_17 },
    { 0x18b76dd0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0376dd8, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b76de0, SUC_lexicon_tags_15 },
    { 0x75376de1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3576de3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x20576de6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43976de8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5976ded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad76df2, SUC_lexicon_tags_15 },
    { 0xd7576df3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xec576df6, SUC_lexicon_tags_49 },
    { 0x36f76df6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfb76df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9176dfb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d176dfe, SUC_lexicon_tags_33 },
    { 0x8eb76dff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2376e01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b576e06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed76e10, SUC_lexicon_tags_61 },
    { 0x36576e11, SUC_lexicon_tags_26 },
    { 0xbdb76e12, SUC_lexicon_tags_19 },
    { 0x5b576e11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f976e16, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e776e19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f76e23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd76e26, SUC_lexicon_tags_12 },
    { 0xf4376e27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b976e29, SUC_lexicon_tags_12 },
    { 0xdf976e2a, SUC_lexicon_tags_46 },
    { 0x2a976e29, SUC_lexicon_tags_15 },
    { 0x9cf76e2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f176e30, SUC_lexicon_tags_14 },
    { 0x63b76e31, SUC_lexicon_tags_39 },
    { 0xabd76e32, SUC_lexicon_tags_2 },
    { 0x1a376e33, SUC_lexicon_tags_14 },
    { 0x8cb76e30, SUC_lexicon_tags_40 },
    { 0xa8376e35, SUC_lexicon_tags_30 },
    { 0xb5176e35, SUC_lexicon_tags_1 },
    { 0xf3f76e33, SUC_lexicon_tags_37 },
    { 0x1f76e38, SUC_lexicon_tags_25 },
    { 0x25976e39, SUC_lexicon_tags_47 },
    { 0x3f76e36, SUC_lexicon_tags_17 },
    { 0xf7976e37, SUC_lexicon_tags_47 },
    { 0x39376e35, SUC_lexicon_tags_29 },
    { 0x94576e3d, SUC_lexicon_tags_17 },
    { 0x7ed76e3e, SUC_lexicon_tags_1 },
    { 0x9f76e3d, SUC_lexicon_tags_1 },
    { 0xf3976e32, SUC_lexicon_tags_16 },
    { 0xf4d76e3e, SUC_lexicon_tags_25 },
    { 0x64d76e36, SUC_lexicon_tags_19 },
    { 0x77176e43, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1df76e45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f76e47, SUC_lexicon_tags_7 },
    { 0x9f576e48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10576e4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f76e50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25776e57, SUC_lexicon_tags_45 },
    { 0x7b376e58, SUC_lexicon_tags_2 },
    { 0xd1776e59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x62976e5c, SUC_lexicon_tags_19 },
    { 0x53d76e5d, SUC_lexicon_tags_25 },
    { 0xeb976e5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f576e60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51776e62, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80976e6f, SUC_lexicon_tags_27 },
    { 0x8d176e6f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3c376e72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed76e77, SUC_lexicon_tags_1 },
    { 0x42376e77, SUC_lexicon_tags_6 },
    { 0xd3f76e79, SUC_lexicon_tags_11 },
    { 0xc1d76e79, SUC_lexicon_tags_47 },
    { 0x5c176e77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b76e7f, SUC_lexicon_tags_7 },
    { 0x28b76e7f, SUC_lexicon_tags_172 },
    { 0xa3976e81, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x39776e84, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92576e8a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c576e8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2c176e90, SUC_lexicon_tags_33 },
    { 0xc0576e91, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x27376e93, SUC_lexicon_tags_15 },
    { 0xfeb76e94, SUC_lexicon_tags_37 },
    { 0x23976e94, SUC_lexicon_tags_25 },
    { 0xbfb76e94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x176e98, SUC_lexicon_tags_2 },
    { 0x1f376e99, SUC_lexicon_tags_31 },
    { 0x52f76e9a, SUC_lexicon_tags_11 },
    { 0x37d76e98, SUC_lexicon_tags_37 },
    { 0xf7376e9b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf3b76e9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7576ea3, SUC_lexicon_tags_45 },
    { 0xc2f76ea3, SUC_lexicon_tags_11 },
    { 0x68976ea3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x74576ea8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad776eab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3576eb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x74976eb7, SUC_lexicon_tags_15 },
    { 0xe3d76eb8, SUC_lexicon_tags_95 },
    { 0xa8b76eb9, SUC_lexicon_tags_25 },
    { 0x54d76eba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8376ec1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9776ec7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36576ecb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb76ecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78f76ed1, SUC_lexicon_tags_27 },
    { 0x5f976ed1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6976ed4, SUC_lexicon_tags_45 },
    { 0xc4f76ed4, SUC_lexicon_tags_0 },
    { 0x44576ed5, SUC_lexicon_tags_11 },
    { 0x8f176ed7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72576edb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8376edd, SUC_lexicon_tags_6 },
    { 0xd9d76ede, SUC_lexicon_tags_37 },
    { 0x9376edf, SUC_lexicon_tags_65 },
    { 0x23576ee0, SUC_lexicon_tags_15 },
    { 0xdc176ee0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb76ee5, SUC_lexicon_tags_8 },
    { 0xc2776ee6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d76eea, SUC_lexicon_tags_18 },
    { 0xe3b76eea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e776eed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87176ef1, SUC_lexicon_tags_28 },
    { 0xa7576ef1, SUC_lexicon_tags_12 },
    { 0x65176ef3, SUC_lexicon_tags_0 },
    { 0x37576ef4, SUC_lexicon_tags_7 },
    { 0x5576ef5, SUC_lexicon_tags_29 },
    { 0x1d376ef6, SUC_lexicon_tags_38 },
    { 0xeff76ef4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19b76ef9, SUC_lexicon_tags_11 },
    { 0x1d776efa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c176efd, SUC_lexicon_tags_7 },
    { 0xf2176efe, SUC_lexicon_tags_33 },
    { 0x92976eff, SUC_lexicon_tags_25 },
    { 0x2c376f00, SUC_lexicon_tags_49 },
    { 0x2c576efe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8776f04, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5fd76f06, SUC_lexicon_tags_41 },
    { 0x8376f07, SUC_lexicon_tags_17 },
    { 0xa3176f06, SUC_lexicon_tags_25 },
    { 0xa8376f09, SUC_lexicon_tags_1 },
    { 0x74976f0a, SUC_lexicon_tags_11 },
    { 0x15776f0b, SUC_lexicon_tags_1 },
    { 0x47d76f09, SUC_lexicon_tags_19 },
    { 0x64176f0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff76f14, SUC_lexicon_tags_11 },
    { 0xef176f15, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0576f18, SUC_lexicon_tags_63 },
    { 0x5f176f19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x92576f1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21776f1d, SUC_lexicon_tags_15 },
    { 0x89b76f1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61976f20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f76f23, SUC_lexicon_tags_7 },
    { 0xe9f76f24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42576f26, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87776f32, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74976f37, SUC_lexicon_tags_70 },
    { 0x1cb76f38, SUC_lexicon_tags_7 },
    { 0x85976f38, SUC_lexicon_tags_17 },
    { 0x8c176f39, SUC_lexicon_tags_25 },
    { 0xc3976f37, SUC_lexicon_tags_1 },
    { 0xdf76f3c, SUC_lexicon_tags_38 },
    { 0x13176f3d, SUC_lexicon_tags_0 },
    { 0x3bd76f3e, SUC_lexicon_tags_38 },
    { 0xd6976f3f, SUC_lexicon_tags_17 },
    { 0x15d76f3f, SUC_lexicon_tags_2 },
    { 0xf9d76f41, SUC_lexicon_tags_179 },
    { 0xc8776f41, SUC_lexicon_tags_61 },
    { 0xe9f76f43, SUC_lexicon_tags_1 },
    { 0xf6576f44, SUC_lexicon_tags_32 },
    { 0x74176f44, SUC_lexicon_tags_15 },
    { 0x8f976f41, SUC_lexicon_tags_49 },
    { 0xf8176f37, SUC_lexicon_tags_27 },
    { 0xc2976f48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74576f4a, SUC_lexicon_tags_12 },
    { 0xb6d76f4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d376f4e, SUC_lexicon_tags_0 },
    { 0x70376f4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ab76f51, SUC_lexicon_tags_37 },
    { 0xcbd76f52, SUC_lexicon_tags_68 },
    { 0xf5776f53, SUC_lexicon_tags_1 },
    { 0xbe176f53, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad76f59, SUC_lexicon_tags_63 },
    { 0xd1576f59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86976f5e, SUC_lexicon_tags_17 },
    { 0xe2376f5f, SUC_lexicon_tags_613 },
    { 0x2e176f5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46976f62, SUC_lexicon_tags_25 },
    { 0xc4176f62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b76f69, SUC_lexicon_tags_15 },
    { 0x8c576f6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d176f6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9576f70, SUC_lexicon_tags_61 },
    { 0x96b76f71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50976f74, SUC_lexicon_tags_107 },
    { 0x8ff76f75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabb76f77, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf76f7a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f76f7f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9176f82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97776f90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45376f94, SUC_lexicon_tags_17 },
    { 0x58d76f95, SUC_lexicon_tags_30 },
    { 0xb1776f95, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x58b76f98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af76f9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d76fa4, SUC_lexicon_tags_4 },
    { 0x1976fa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37976fa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94b76faa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30976fac, SUC_lexicon_tags_37 },
    { 0xb6b76fad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb376fb1, SUC_lexicon_tags_25 },
    { 0x2df76fb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf76fb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff76fba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36b76fbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x80b76fbe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb76fc1, SUC_lexicon_tags_8 },
    { 0x5cf76fc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad376fc5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87576fc9, SUC_lexicon_tags_37 },
    { 0x6776fca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x43576fcd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x62f76fcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4976fd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f76fd7, SUC_lexicon_tags_47 },
    { 0x40176fd8, SUC_lexicon_tags_7 },
    { 0xb9776fd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d76fde, SUC_lexicon_tags_11 },
    { 0x5f776fdf, SUC_lexicon_tags_60 },
    { 0x4576fe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4176fe6, SUC_lexicon_tags_60 },
    { 0xfdb76fe6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x78776fe9, SUC_lexicon_tags_11 },
    { 0xb76fe9, SUC_lexicon_tags_43 },
    { 0xd1f76feb, SUC_lexicon_tags_12 },
    { 0xe0176fec, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d776fef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c176ff1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9f76ff3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf76ff7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee576ffc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb9776ffe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8577000, SUC_lexicon_tags_11 },
    { 0xaf977001, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b77006, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7977011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed377016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4777018, SUC_lexicon_tags_12 },
    { 0xc2777018, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbf7701b, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0x3bf7701d, SUC_lexicon_tags_2 },
    { 0xcfb7701e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x69977021, SUC_lexicon_tags_14 },
    { 0x88977022, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99577026, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1677702d, SUC_lexicon_tags_47 },
    { 0x1d7702d, SUC_lexicon_tags_12 },
    { 0x7757702d, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d77033, SUC_lexicon_tags_1 },
    { 0x71377033, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe917703a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c97703e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe8377040, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b77043, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25777045, SUC_lexicon_tags_88 },
    { 0x75b77045, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c577048, SUC_lexicon_tags_29 },
    { 0x1cb77049, SUC_lexicon_tags_25 },
    { 0xe5d7704a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa5b7704c, SUC_lexicon_tags_37 },
    { 0x58f7704d, SUC_lexicon_tags_7 },
    { 0xb597704c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe977050, SUC_lexicon_tags_25 },
    { 0x17777051, SUC_lexicon_tags_23 },
    { 0xfad77052, SUC_lexicon_tags_37 },
    { 0x9a377052, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88377055, SUC_lexicon_tags_16 },
    { 0x2ad77056, SUC_lexicon_tags_15 },
    { 0x46377057, SUC_lexicon_tags_39 },
    { 0xda977058, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd817705c, SUC_lexicon_tags_48 },
    { 0x22f7705d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe377065, SUC_lexicon_tags_31 },
    { 0x68b77066, SUC_lexicon_tags_45 },
    { 0x8b977067, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1977706b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5397706f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef977078, SUC_lexicon_tags_19 },
    { 0x84177079, SUC_lexicon_tags_28 },
    { 0x5417707a, SUC_lexicon_tags_8 },
    { 0xa817707a, SUC_lexicon_tags_49 },
    { 0xbbf7707a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x937707e, SUC_lexicon_tags_37 },
    { 0xcfd7707f, SUC_lexicon_tags_49 },
    { 0x2537707e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b77083, SUC_lexicon_tags_25 },
    { 0x7f377083, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88d77086, SUC_lexicon_tags_14 },
    { 0x41b77087, SUC_lexicon_tags_65 },
    { 0x76f77088, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c37708c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab77092, SUC_lexicon_tags_68 },
    { 0xe7377093, SUC_lexicon_tags_88 },
    { 0xf7177093, SUC_lexicon_tags_11 },
    { 0xb6d77095, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f77099, SUC_lexicon_tags_66 },
    { 0x6bd77099, SUC_lexicon_tags_14 },
    { 0x1657709b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f57709d, SUC_lexicon_tags_15 },
    { 0x4cd7709e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73770a1, SUC_lexicon_tags_16 },
    { 0x21770a2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x699770a5, SUC_lexicon_tags_30 },
    { 0x50d770a5, SUC_lexicon_tags_2 },
    { 0xe21770a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f770ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47770b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde7770b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf770b7, SUC_lexicon_tags_1414 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5770ba, SUC_lexicon_tags_1 },
    { 0x921770ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd7770bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d770c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb770c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b770c9, SUC_lexicon_tags_11 },
    { 0x969770ca, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb770d5, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f770d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x69770dc, SUC_lexicon_tags_15 },
    { 0x6f7770dd, SUC_lexicon_tags_37 },
    { 0xcff770de, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d770e3, SUC_lexicon_tags_21 },
    { 0x3bb770e4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d770ed, SUC_lexicon_tags_37 },
    { 0x377770ed, SUC_lexicon_tags_0 },
    { 0x907770ef, SUC_lexicon_tags_15 },
    { 0xcbf770ee, SUC_lexicon_tags_45 },
    { 0x349770f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8bb770f3, SUC_lexicon_tags_11 },
    { 0x275770f4, SUC_lexicon_tags_37 },
    { 0xb29770f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09770fa, SUC_lexicon_tags_49 },
    { 0xbe3770fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc33770fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ad770ff, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22977103, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff577105, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35377109, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f7710e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20d77110, SUC_lexicon_tags_47 },
    { 0x23d77111, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15977113, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a377115, SUC_lexicon_tags_47 },
    { 0x80377116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3177118, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb37711b, SUC_lexicon_tags_61 },
    { 0x5b7711b, SUC_lexicon_tags_17 },
    { 0x6717711c, SUC_lexicon_tags_14 },
    { 0xc297711e, SUC_lexicon_tags_11 },
    { 0xb7b7711e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbdf77121, SUC_lexicon_tags_25 },
    { 0x8d377122, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd77127, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f7712f, SUC_lexicon_tags_57 },
    { 0x67577130, SUC_lexicon_tags_11 },
    { 0x80f77130, SUC_lexicon_tags_11 },
    { 0x22377131, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f77137, SUC_lexicon_tags_7 },
    { 0x81b77137, SUC_lexicon_tags_82 },
    { 0xd3d77139, SUC_lexicon_tags_37 },
    { 0xb1d7713a, SUC_lexicon_tags_62 },
    { 0x20177137, SUC_lexicon_tags_37 },
    { 0x34b7713c, SUC_lexicon_tags_84 },
    { 0x7377139, SUC_lexicon_tags_1 },
    { 0xabb7713c, SUC_lexicon_tags_87 },
    { 0x6e57713c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9777141, SUC_lexicon_tags_2 },
    { 0xd977142, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b77147, SUC_lexicon_tags_31 },
    { 0xccb77147, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xad37714a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd157714f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1777151, SUC_lexicon_tags_65 },
    { 0x65777152, SUC_lexicon_tags_37 },
    { 0x9f577151, SUC_lexicon_tags_15 },
    { 0x95977154, SUC_lexicon_tags_15 },
    { 0x1d377155, SUC_lexicon_tags_29 },
    { 0x53177156, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe397715f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b77162, SUC_lexicon_tags_213 },
    { 0xa2777163, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb77167, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9797716a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15d7716c, SUC_lexicon_tags_7 },
    { 0xf497716c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac777176, SUC_lexicon_tags_47 },
    { 0xe0577176, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31577179, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f97717e, SUC_lexicon_tags_29 },
    { 0x6817717f, SUC_lexicon_tags_12 },
    { 0x4f377180, SUC_lexicon_tags_4 },
    { 0x2f17717e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75377183, SUC_lexicon_tags_12 },
    { 0xde577184, SUC_lexicon_tags_28 },
    { 0x13f77183, SUC_lexicon_tags_21 },
    { 0x65977186, SUC_lexicon_tags_38 },
    { 0xe6377185, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b7718a, SUC_lexicon_tags_23 },
    { 0xe77718a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdaf7718d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f77190, SUC_lexicon_tags_17 },
    { 0x96b77191, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d77194, SUC_lexicon_tags_1 },
    { 0x6c577195, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61377199, SUC_lexicon_tags_49 },
    { 0x77577199, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2057719d, SUC_lexicon_tags_29 },
    { 0xd017719e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b771a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcdf771a3, SUC_lexicon_tags_37 },
    { 0x13d771a4, SUC_lexicon_tags_23 },
    { 0xb9d771a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x559771a7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153771af, SUC_lexicon_tags_19 },
    { 0xf35771b0, SUC_lexicon_tags_17 },
    { 0xfe5771b0, SUC_lexicon_tags_47 },
    { 0xe91771b0, SUC_lexicon_tags_15 },
    { 0x7cd771b3, SUC_lexicon_tags_45 },
    { 0x47f771b0, SUC_lexicon_tags_33 },
    { 0xae9771b5, SUC_lexicon_tags_11 },
    { 0x8c3771b6, SUC_lexicon_tags_3 },
    { 0x5cf771b7, SUC_lexicon_tags_7 },
    { 0x21d771b7, SUC_lexicon_tags_15 },
    { 0xee9771b7, SUC_lexicon_tags_61 },
    { 0xf69771b3, SUC_lexicon_tags_38 },
    { 0x4a5771b7, SUC_lexicon_tags_1 },
    { 0x577771b9, SUC_lexicon_tags_25 },
    { 0x7f7771bd, SUC_lexicon_tags_1 },
    { 0x603771bd, SUC_lexicon_tags_66 },
    { 0x86d771bf, SUC_lexicon_tags_17 },
    { 0x24b771af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3771c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x557771c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7771cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9d771d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x13f771d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb65771d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05771de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebf771e0, SUC_lexicon_tags_25 },
    { 0xc09771e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5d5771e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x549771e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895771ee, SUC_lexicon_tags_38 },
    { 0x6ff771ef, SUC_lexicon_tags_15 },
    { 0x123771f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51771f4, SUC_lexicon_tags_26 },
    { 0x411771f4, SUC_lexicon_tags_27 },
    { 0xb13771f5, SUC_lexicon_tags_23 },
    { 0x53b771f7, SUC_lexicon_tags_39 },
    { 0x3bb771f7, SUC_lexicon_tags_201 },
    { 0xf91771f8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbfd771fb, SUC_lexicon_tags_28 },
    { 0x8c3771fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac3771fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cf77200, SUC_lexicon_tags_7 },
    { 0x33177200, SUC_lexicon_tags_2 },
    { 0xaa977202, SUC_lexicon_tags_17 },
    { 0x12b77203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde577208, SUC_lexicon_tags_69 },
    { 0xd0977209, SUC_lexicon_tags_15 },
    { 0x5a17720a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89577211, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee377213, SUC_lexicon_tags_17 },
    { 0x62b77213, SUC_lexicon_tags_31 },
    { 0x6eb77215, SUC_lexicon_tags_17 },
    { 0xc7177216, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5917721c, SUC_lexicon_tags_17 },
    { 0x8237721c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f77223, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd77226, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea977228, SUC_lexicon_tags_43 },
    { 0x80977229, SUC_lexicon_tags_151 },
    { 0xf57722a, SUC_lexicon_tags_1 },
    { 0xbc97722a, SUC_lexicon_tags_17 },
    { 0x60f7722a, SUC_lexicon_tags_7 },
    { 0xf3377229, SUC_lexicon_tags_12 },
    { 0x2277722c, SUC_lexicon_tags_25 },
    { 0x61d7722d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0777235, SUC_lexicon_tags_15 },
    { 0xb8577235, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9817723b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc357723d, SUC_lexicon_tags_45 },
    { 0xfd37723e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e177249, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7457724e, SUC_lexicon_tags_0 },
    { 0xb037724f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e177251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbab77253, SUC_lexicon_tags_7 },
    { 0x56b77254, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9577257, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c57725a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xcff7725c, SUC_lexicon_tags_70 },
    { 0xd4d7725c, SUC_lexicon_tags_63 },
    { 0x7c77725c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x15777261, SUC_lexicon_tags_7 },
    { 0xedd77262, SUC_lexicon_tags_47 },
    { 0xfcd77262, SUC_lexicon_tags_25 },
    { 0x14577264, SUC_lexicon_tags_19 },
    { 0xc5977264, SUC_lexicon_tags_23 },
    { 0x9b777264, SUC_lexicon_tags_37 },
    { 0x20f77267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc917726a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf57726c, SUC_lexicon_tags_34 },
    { 0x66d7726d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf057726f, SUC_lexicon_tags_33 },
    { 0x12977270, SUC_lexicon_tags_25 },
    { 0x5b777270, SUC_lexicon_tags_15 },
    { 0x9b177272, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75b77274, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cf77276, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28f77278, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a97727a, SUC_lexicon_tags_1 },
    { 0xc1b7727b, SUC_lexicon_tags_11 },
    { 0x817727c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd597727e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b977284, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fd77286, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x81777289, SUC_lexicon_tags_11 },
    { 0x54577289, SUC_lexicon_tags_43 },
    { 0xd6b7728b, SUC_lexicon_tags_25 },
    { 0x5277728b, SUC_lexicon_tags_6 },
    { 0x60b7728b, SUC_lexicon_tags_37 },
    { 0x8257728b, SUC_lexicon_tags_2 },
    { 0x2337728b, SUC_lexicon_tags_57 },
    { 0xaa77728c, SUC_lexicon_tags_25 },
    { 0xf577291, SUC_lexicon_tags_28 },
    { 0xf3d77292, SUC_lexicon_tags_11 },
    { 0xb737728f, SUC_lexicon_tags_2 },
    { 0x52b77292, SUC_lexicon_tags_15 },
    { 0xb0977290, SUC_lexicon_tags_1 },
    { 0x57f77293, SUC_lexicon_tags_60 },
    { 0x3b777297, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x69177299, SUC_lexicon_tags_33 },
    { 0xb257729a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8a17729c, SUC_lexicon_tags_9 },
    { 0xa997729d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7772a2, SUC_lexicon_tags_17 },
    { 0x605772a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805772ac, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b772b1, SUC_lexicon_tags_1 },
    { 0x99f772b2, SUC_lexicon_tags_13 },
    { 0x895772b1, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb772b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad772be, SUC_lexicon_tags_63 },
    { 0x6ad772be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x977772c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29772c5, SUC_lexicon_tags_17 },
    { 0x5c7772c5, SUC_lexicon_tags_37 },
    { 0x607772c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d772cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8ff772cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf772d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd43772d3, SUC_lexicon_tags_15 },
    { 0xdf7772d4, SUC_lexicon_tags_11 },
    { 0x295772d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc99772d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe1b772d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75772dc, SUC_lexicon_tags_45 },
    { 0xed5772dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e1772df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf772e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad772eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7772ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81772f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31772fe, SUC_lexicon_tags_14 },
    { 0x80d772fe, SUC_lexicon_tags_26 },
    { 0x6ad77300, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d377303, SUC_lexicon_tags_17 },
    { 0x39777304, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df77309, SUC_lexicon_tags_15 },
    { 0x8e37730a, SUC_lexicon_tags_27 },
    { 0x54d7730b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf337730e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdfd77310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa977313, SUC_lexicon_tags_11 },
    { 0xe5577314, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7cb77316, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b177318, SUC_lexicon_tags_37 },
    { 0xe9d77319, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f7731f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x80977321, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3577329, SUC_lexicon_tags_33 },
    { 0x92177329, SUC_lexicon_tags_37 },
    { 0x8fd7732b, SUC_lexicon_tags_7 },
    { 0xf577732c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b77732e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a377333, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x7e977335, SUC_lexicon_tags_9 },
    { 0x23f77335, SUC_lexicon_tags_82 },
    { 0x9fb77337, SUC_lexicon_tags_37 },
    { 0xef77338, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f17733a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd677733c, SUC_lexicon_tags_27 },
    { 0x5f57733d, SUC_lexicon_tags_25 },
    { 0x8f17733d, SUC_lexicon_tags_9 },
    { 0x7ef7733f, SUC_lexicon_tags_38 },
    { 0x70f7733e, SUC_lexicon_tags_11 },
    { 0x5a57733f, SUC_lexicon_tags_12 },
    { 0x5c177342, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x37577344, SUC_lexicon_tags_9 },
    { 0x96d77345, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e377347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7734c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6017734e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38577352, SUC_lexicon_tags_29 },
    { 0x66d77353, SUC_lexicon_tags_14 },
    { 0xba777354, SUC_lexicon_tags_47 },
    { 0x43d77354, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb7735b, SUC_lexicon_tags_25 },
    { 0x1b57735c, SUC_lexicon_tags_7 },
    { 0xb17735d, SUC_lexicon_tags_45 },
    { 0x1617735b, SUC_lexicon_tags_27 },
    { 0x6717735c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71577363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a977365, SUC_lexicon_tags_14 },
    { 0xbd177365, SUC_lexicon_tags_69 },
    { 0x9577366, SUC_lexicon_tags_65 },
    { 0xc7977365, SUC_lexicon_tags_45 },
    { 0x5eb77369, SUC_lexicon_tags_28 },
    { 0xbc377366, SUC_lexicon_tags_11 },
    { 0x1017736b, SUC_lexicon_tags_31 },
    { 0xeff7736c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b77373, SUC_lexicon_tags_7 },
    { 0xd77373, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82577377, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6197737c, SUC_lexicon_tags_12 },
    { 0xf7f7737c, SUC_lexicon_tags_66 },
    { 0x6df7737e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b77381, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb5b77383, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d77386, SUC_lexicon_tags_27 },
    { 0x24d77387, SUC_lexicon_tags_42 },
    { 0xcdb77388, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1537738a, SUC_lexicon_tags_32 },
    { 0x7157738b, SUC_lexicon_tags_25 },
    { 0xc677738a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b77390, SUC_lexicon_tags_15 },
    { 0xff577391, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff77395, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x76b77398, SUC_lexicon_tags_31 },
    { 0x3df77399, SUC_lexicon_tags_27 },
    { 0x9557739a, SUC_lexicon_tags_45 },
    { 0x8897739b, SUC_lexicon_tags_23 },
    { 0x9a97739c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d773a0, SUC_lexicon_tags_12 },
    { 0x82d773a1, SUC_lexicon_tags_65 },
    { 0xd57773a2, SUC_lexicon_tags_17 },
    { 0x789773a2, SUC_lexicon_tags_48 },
    { 0xfcb773a4, SUC_lexicon_tags_13 },
    { 0xd09773a5, SUC_lexicon_tags_39 },
    { 0x3bd773a5, SUC_lexicon_tags_11 },
    { 0xd4d773a7, SUC_lexicon_tags_3 },
    { 0x50d773a5, SUC_lexicon_tags_23 },
    { 0x8c7773a5, SUC_lexicon_tags_38 },
    { 0xe3d773a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d773ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe19773af, SUC_lexicon_tags_33 },
    { 0x9bd773af, SUC_lexicon_tags_25 },
    { 0x37f773b0, SUC_lexicon_tags_30 },
    { 0x545773b1, SUC_lexicon_tags_19 },
    { 0xda9773b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1773b6, SUC_lexicon_tags_11 },
    { 0xb6d773b7, SUC_lexicon_tags_37 },
    { 0x5b3773b8, SUC_lexicon_tags_11 },
    { 0x94d773b9, SUC_lexicon_tags_37 },
    { 0xab7773ba, SUC_lexicon_tags_15 },
    { 0x1af773bb, SUC_lexicon_tags_47 },
    { 0x21f773bc, SUC_lexicon_tags_15 },
    { 0xb67773bc, SUC_lexicon_tags_11 },
    { 0x4d9773bc, SUC_lexicon_tags_47 },
    { 0xa25773bf, SUC_lexicon_tags_25 },
    { 0xb31773bc, SUC_lexicon_tags_12 },
    { 0x76f773c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b773c4, SUC_lexicon_tags_19 },
    { 0x56d773c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe5773c7, SUC_lexicon_tags_7 },
    { 0xe7b773c7, SUC_lexicon_tags_19 },
    { 0x1fb773c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d773cd, SUC_lexicon_tags_11 },
    { 0x90d773cd, SUC_lexicon_tags_31 },
    { 0xe39773cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af773d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x84d773d4, SUC_lexicon_tags_43 },
    { 0x19773d5, SUC_lexicon_tags_62 },
    { 0x9e1773d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cf773d8, SUC_lexicon_tags_33 },
    { 0x113773d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17773e0, SUC_lexicon_tags_37 },
    { 0xe23773e1, SUC_lexicon_tags_1 },
    { 0x28d773e1, SUC_lexicon_tags_81 },
    { 0xa5773e3, SUC_lexicon_tags_19 },
    { 0x173773e4, SUC_lexicon_tags_33 },
    { 0x52b773e5, SUC_lexicon_tags_25 },
    { 0xa3773e6, SUC_lexicon_tags_37 },
    { 0x5bf773e5, SUC_lexicon_tags_33 },
    { 0x2a1773e4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x401773ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x357773ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbf773ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609773f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f773f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9773fd, SUC_lexicon_tags_17 },
    { 0xd09773fd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d377401, SUC_lexicon_tags_39 },
    { 0x34977401, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12577405, SUC_lexicon_tags_6 },
    { 0x84577406, SUC_lexicon_tags_7 },
    { 0xb1977407, SUC_lexicon_tags_31 },
    { 0x10b77408, SUC_lexicon_tags_12 },
    { 0x8f377405, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc597740e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xbd377410, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x21577413, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f77418, SUC_lexicon_tags_60 },
    { 0x10177419, SUC_lexicon_tags_1 },
    { 0xd8f7741a, SUC_lexicon_tags_29 },
    { 0xf557741b, SUC_lexicon_tags_37 },
    { 0x4377741a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd77420, SUC_lexicon_tags_48 },
    { 0x13377421, SUC_lexicon_tags_17 },
    { 0x96577422, SUC_lexicon_tags_56 },
    { 0xfcd77423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71577425, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b77428, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997742e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6577430, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd77434, SUC_lexicon_tags_15 },
    { 0x44577435, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x77d77437, SUC_lexicon_tags_37 },
    { 0x36f77438, SUC_lexicon_tags_7 },
    { 0xe3177439, SUC_lexicon_tags_25 },
    { 0x5517743a, SUC_lexicon_tags_25 },
    { 0x7fd77437, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d7743e, SUC_lexicon_tags_25 },
    { 0x5177743f, SUC_lexicon_tags_35 },
    { 0x99b77440, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd77443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d377446, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd37744d, SUC_lexicon_tags_60 },
    { 0x94d7744d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b377455, SUC_lexicon_tags_11 },
    { 0x40177456, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd77459, SUC_lexicon_tags_15 },
    { 0x3217745a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68177460, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf77465, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b7746c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3777475, SUC_lexicon_tags_19 },
    { 0x6cd77475, SUC_lexicon_tags_40 },
    { 0x3177476, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd9377479, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e37747f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xba777481, SUC_lexicon_tags_25 },
    { 0x90177482, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x33577484, SUC_lexicon_tags_19 },
    { 0x73177485, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d77489, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa557748b, SUC_lexicon_tags_46 },
    { 0x5e97748c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f37748f, SUC_lexicon_tags_47 },
    { 0x79b77490, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b377493, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f77497, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x92177499, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf7749d, SUC_lexicon_tags_37 },
    { 0x2c17749e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17774a1, SUC_lexicon_tags_16 },
    { 0x93774a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcfb774a4, SUC_lexicon_tags_4 },
    { 0x609774a5, SUC_lexicon_tags_12 },
    { 0x3cd774a5, SUC_lexicon_tags_19 },
    { 0xa13774a5, SUC_lexicon_tags_0 },
    { 0xfb5774a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57774b3, SUC_lexicon_tags_7 },
    { 0x4dd774b4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d774b7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b774bb, SUC_lexicon_tags_43 },
    { 0xc53774bc, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb774c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc3b774c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25774cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f774d0, SUC_lexicon_tags_224 },
    { 0xa15774d1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x551774d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x937774d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27774da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf774dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789774e6, SUC_lexicon_tags_23 },
    { 0x815774e7, SUC_lexicon_tags_7 },
    { 0xa9b774e7, SUC_lexicon_tags_30 },
    { 0xd5d774e9, SUC_lexicon_tags_33 },
    { 0x3d7774ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7774ed, SUC_lexicon_tags_37 },
    { 0x38f774ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x231774f0, SUC_lexicon_tags_17 },
    { 0x1c3774f0, SUC_lexicon_tags_12 },
    { 0x56f774f1, SUC_lexicon_tags_1 },
    { 0x2cf774f3, SUC_lexicon_tags_37 },
    { 0x18f774f3, SUC_lexicon_tags_7 },
    { 0xa35774f3, SUC_lexicon_tags_43 },
    { 0x5d9774f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d774fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575774ff, SUC_lexicon_tags_63 },
    { 0x19777500, SUC_lexicon_tags_17 },
    { 0x85177500, SUC_lexicon_tags_60 },
    { 0xa9b77502, SUC_lexicon_tags_14 },
    { 0x59977503, SUC_lexicon_tags_1 },
    { 0xe8b77503, SUC_lexicon_tags_65 },
    { 0xbfb77502, SUC_lexicon_tags_33 },
    { 0x65b77501, SUC_lexicon_tags_4 },
    { 0x8cd77507, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab77510, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa477751a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb217751c, SUC_lexicon_tags_49 },
    { 0x6c37751d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5577523, SUC_lexicon_tags_11 },
    { 0x96977524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8237752a, SUC_lexicon_tags_23 },
    { 0x7717752b, SUC_lexicon_tags_37 },
    { 0x7c77752a, SUC_lexicon_tags_37 },
    { 0xb957752d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x57752f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea977533, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd77538, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb277753b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1977540, SUC_lexicon_tags_35 },
    { 0x98577541, SUC_lexicon_tags_2 },
    { 0x7af77541, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd77545, SUC_lexicon_tags_12 },
    { 0xae977546, SUC_lexicon_tags_8 },
    { 0x7ef77546, SUC_lexicon_tags_7 },
    { 0x19f77546, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d77754c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8337754e, SUC_lexicon_tags_92 },
    { 0x5577754e, SUC_lexicon_tags_43 },
    { 0xef977550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcb77552, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80977554, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe197755a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f17755f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f577565, SUC_lexicon_tags_7 },
    { 0xdc377565, SUC_lexicon_tags_49 },
    { 0x85377567, SUC_lexicon_tags_17 },
    { 0xd7d77565, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa637756f, SUC_lexicon_tags_43 },
    { 0xb1977570, SUC_lexicon_tags_25 },
    { 0x53577570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7977573, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab77577, SUC_lexicon_tags_60 },
    { 0x26977578, SUC_lexicon_tags_47 },
    { 0x39377577, SUC_lexicon_tags_66 },
    { 0x5377579, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f77757c, SUC_lexicon_tags_12 },
    { 0xafd7757d, SUC_lexicon_tags_1 },
    { 0x5a97757e, SUC_lexicon_tags_31 },
    { 0x3c17757e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5777582, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5777586, SUC_lexicon_tags_26 },
    { 0x4d77586, SUC_lexicon_tags_103 },
    { 0x3ed77586, SUC_lexicon_tags_2 },
    { 0x8c777589, SUC_lexicon_tags_37 },
    { 0x7417758a, SUC_lexicon_tags_12 },
    { 0x8d777587, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd757758f, SUC_lexicon_tags_37 },
    { 0x977758f, SUC_lexicon_tags_1 },
    { 0x9577590, SUC_lexicon_tags_7 },
    { 0x2577592, SUC_lexicon_tags_17 },
    { 0x4b977591, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83d77595, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af7759c, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x9a37759e, SUC_lexicon_tags_48 },
    { 0x70b7759f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5775a2, SUC_lexicon_tags_11 },
    { 0x939775a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43775a6, SUC_lexicon_tags_14 },
    { 0xfa5775a7, SUC_lexicon_tags_12 },
    { 0x183775a8, SUC_lexicon_tags_39 },
    { 0x71f775a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x861775ab, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87775b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb775b9, SUC_lexicon_tags_17 },
    { 0xba7775b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7775bd, SUC_lexicon_tags_19 },
    { 0x883775be, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1775c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1775c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7775cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f775cf, SUC_lexicon_tags_33 },
    { 0x50b775d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b775d3, SUC_lexicon_tags_60 },
    { 0xf5d775d3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed775db, SUC_lexicon_tags_27 },
    { 0xf37775dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89775e2, SUC_lexicon_tags_57 },
    { 0x791775e2, SUC_lexicon_tags_65 },
    { 0x277775e4, SUC_lexicon_tags_37 },
    { 0xcc7775e2, SUC_lexicon_tags_11 },
    { 0xce5775e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f775ea, SUC_lexicon_tags_11 },
    { 0xa7f775eb, SUC_lexicon_tags_47 },
    { 0x47b775eb, SUC_lexicon_tags_38 },
    { 0xaeb775ed, SUC_lexicon_tags_65 },
    { 0x289775ee, SUC_lexicon_tags_0 },
    { 0xc8f775ef, SUC_lexicon_tags_7 },
    { 0x2db775f0, SUC_lexicon_tags_61 },
    { 0x87f775f1, SUC_lexicon_tags_12 },
    { 0x105775ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x779775f4, SUC_lexicon_tags_7 },
    { 0xbd775f5, SUC_lexicon_tags_87 },
    { 0x18b775f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef775f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x555775fb, SUC_lexicon_tags_37 },
    { 0xb0f775fb, SUC_lexicon_tags_17 },
    { 0xfdd775fd, SUC_lexicon_tags_3 },
    { 0x249775fe, SUC_lexicon_tags_11 },
    { 0x405775fc, SUC_lexicon_tags_26 },
    { 0x74177600, SUC_lexicon_tags_15 },
    { 0x5fb775fb, SUC_lexicon_tags_1 },
    { 0x4dd77602, SUC_lexicon_tags_13 },
    { 0xb0f77603, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf7760c, SUC_lexicon_tags_45 },
    { 0x2b37760c, SUC_lexicon_tags_37 },
    { 0xeb37760e, SUC_lexicon_tags_15 },
    { 0x5217760f, SUC_lexicon_tags_7 },
    { 0x20d7760f, SUC_lexicon_tags_2 },
    { 0x3ad77611, SUC_lexicon_tags_15 },
    { 0x87777612, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a377615, SUC_lexicon_tags_25 },
    { 0xb1b77616, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad77619, SUC_lexicon_tags_17 },
    { 0x8b577619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf97761c, SUC_lexicon_tags_82 },
    { 0x1497761d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff77620, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74377628, SUC_lexicon_tags_33 },
    { 0x74977629, SUC_lexicon_tags_2 },
    { 0x7e77762a, SUC_lexicon_tags_57 },
    { 0xb5777628, SUC_lexicon_tags_37 },
    { 0x4af7762a, SUC_lexicon_tags_11 },
    { 0x2737762d, SUC_lexicon_tags_15 },
    { 0x73f7762d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe777632, SUC_lexicon_tags_2 },
    { 0xd577632, SUC_lexicon_tags_34 },
    { 0xd6977634, SUC_lexicon_tags_34 },
    { 0x82f77634, SUC_lexicon_tags_60 },
    { 0x977632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb7763a, SUC_lexicon_tags_17 },
    { 0x5317763b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f7763f, SUC_lexicon_tags_30 },
    { 0xb497763f, SUC_lexicon_tags_37 },
    { 0x26177641, SUC_lexicon_tags_42 },
    { 0x83177642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb77646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ef77648, SUC_lexicon_tags_2 },
    { 0xee777649, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b17764c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8977650, SUC_lexicon_tags_77 },
    { 0x38177651, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c57765e, SUC_lexicon_tags_2 },
    { 0xad17765f, SUC_lexicon_tags_63 },
    { 0xdb7765f, SUC_lexicon_tags_7 },
    { 0x55f7765f, SUC_lexicon_tags_25 },
    { 0x58d7765f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3777664, SUC_lexicon_tags_33 },
    { 0x4fd77665, SUC_lexicon_tags_17 },
    { 0x2c377665, SUC_lexicon_tags_12 },
    { 0xf3b77664, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e97766f, SUC_lexicon_tags_11 },
    { 0xb5d77670, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12f77672, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b377677, SUC_lexicon_tags_37 },
    { 0xcc777678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x90b7767a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7767d, SUC_lexicon_tags_2 },
    { 0xb577767e, SUC_lexicon_tags_37 },
    { 0xb77767f, SUC_lexicon_tags_39 },
    { 0x66377680, SUC_lexicon_tags_1 },
    { 0x5fb77681, SUC_lexicon_tags_26 },
    { 0xbef77682, SUC_lexicon_tags_25 },
    { 0xad777681, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87177688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e17768b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56d7768e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23377690, SUC_lexicon_tags_23 },
    { 0xf0377690, SUC_lexicon_tags_33 },
    { 0x20b77690, SUC_lexicon_tags_37 },
    { 0xbb377693, SUC_lexicon_tags_37 },
    { 0xcc777690, SUC_lexicon_tags_48 },
    { 0x26b77695, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b77698, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb77769e, SUC_lexicon_tags_37 },
    { 0x57f7769f, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x7c7776a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5f776a3, SUC_lexicon_tags_30 },
    { 0x74f776a3, SUC_lexicon_tags_19 },
    { 0xa17776a5, SUC_lexicon_tags_28 },
    { 0xa8f776a6, SUC_lexicon_tags_1 },
    { 0x835776a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x761776a9, SUC_lexicon_tags_14 },
    { 0x8fd776aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3776b3, SUC_lexicon_tags_37 },
    { 0x7d7776b3, SUC_lexicon_tags_29 },
    { 0x7d7776b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc5776b7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf776ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb85776bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca1776be, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4df776c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2af776c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbf776c4, SUC_lexicon_tags_2 },
    { 0x9d7776c4, SUC_lexicon_tags_13 },
    { 0x2b3776c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23776ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5776cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81776d1, SUC_lexicon_tags_17 },
    { 0x69776d2, SUC_lexicon_tags_27 },
    { 0xf9f776d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91776d6, SUC_lexicon_tags_2 },
    { 0x86b776d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1776db, SUC_lexicon_tags_56 },
    { 0x5ef776db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39776e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb776e6, SUC_lexicon_tags_2 },
    { 0x33d776e7, SUC_lexicon_tags_28 },
    { 0xac9776e8, SUC_lexicon_tags_7 },
    { 0xe61776e9, SUC_lexicon_tags_7 },
    { 0x1d9776e8, SUC_lexicon_tags_25 },
    { 0x3ad776e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1776f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb776ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b177707, SUC_lexicon_tags_63 },
    { 0x8b177708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9117770b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d7770f, SUC_lexicon_tags_17 },
    { 0x55377710, SUC_lexicon_tags_7 },
    { 0x58577711, SUC_lexicon_tags_60 },
    { 0x4a177711, SUC_lexicon_tags_10 },
    { 0xedb77713, SUC_lexicon_tags_2 },
    { 0x8d777711, SUC_lexicon_tags_42 },
    { 0x1bb77715, SUC_lexicon_tags_49 },
    { 0xb5b77716, SUC_lexicon_tags_66 },
    { 0xcd177717, SUC_lexicon_tags_11 },
    { 0xdb977715, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f7771e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f77721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a777726, SUC_lexicon_tags_16 },
    { 0xec977727, SUC_lexicon_tags_7 },
    { 0xab777728, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1497772e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x90377730, SUC_lexicon_tags_28 },
    { 0x8977731, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f77735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x98977738, SUC_lexicon_tags_33 },
    { 0x18177738, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf877773c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c977743, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b577746, SUC_lexicon_tags_49 },
    { 0x12777747, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd6d77749, SUC_lexicon_tags_25 },
    { 0xe577774a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb5d7774c, SUC_lexicon_tags_2 },
    { 0x1837774c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e77774f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4577755, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5117775d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x22377760, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x46577763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f77766, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1777769, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef7776f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfd77771, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa377773, SUC_lexicon_tags_49 },
    { 0xe8977773, SUC_lexicon_tags_179 },
    { 0x1ef77775, SUC_lexicon_tags_15 },
    { 0x52977775, SUC_lexicon_tags_11 },
    { 0x5cd77777, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6297777a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd237777d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed77780, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf7377782, SUC_lexicon_tags_11 },
    { 0xa177783, SUC_lexicon_tags_17 },
    { 0xbd77784, SUC_lexicon_tags_21 },
    { 0xa5377784, SUC_lexicon_tags_19 },
    { 0xf3d77784, SUC_lexicon_tags_11 },
    { 0xa1777783, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaff77789, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe97778c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4177792, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0577796, SUC_lexicon_tags_26 },
    { 0xb3177797, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ff77799, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f7779d, SUC_lexicon_tags_17 },
    { 0xf1d7779e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x251777a0, SUC_lexicon_tags_10 },
    { 0x8a1777a1, SUC_lexicon_tags_7 },
    { 0xa4d777a2, SUC_lexicon_tags_0 },
    { 0xc5d777a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5777a7, SUC_lexicon_tags_17 },
    { 0x4c7777a8, SUC_lexicon_tags_29 },
    { 0xe4b777a7, SUC_lexicon_tags_17 },
    { 0x60f777a7, SUC_lexicon_tags_38 },
    { 0xa03777ab, SUC_lexicon_tags_9 },
    { 0xadf777ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655777b5, SUC_lexicon_tags_10 },
    { 0x5d7777b6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xea3777b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1777bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d777be, SUC_lexicon_tags_17 },
    { 0x57777bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37777c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x17f777c7, SUC_lexicon_tags_42 },
    { 0x82f777c8, SUC_lexicon_tags_12 },
    { 0x20b777c9, SUC_lexicon_tags_46 },
    { 0x97777ca, SUC_lexicon_tags_26 },
    { 0xb1777ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3a7777cd, SUC_lexicon_tags_50 },
    { 0x983777ce, SUC_lexicon_tags_26 },
    { 0x7e9777ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3777d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x865777d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d777d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x73777db, SUC_lexicon_tags_14 },
    { 0xda7777dc, SUC_lexicon_tags_25 },
    { 0x663777dd, SUC_lexicon_tags_25 },
    { 0xaf777de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3777e9, SUC_lexicon_tags_29 },
    { 0xe05777e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59777f2, SUC_lexicon_tags_55 },
    { 0x87b777f2, SUC_lexicon_tags_37 },
    { 0x685777f2, SUC_lexicon_tags_37 },
    { 0x6bd777f5, SUC_lexicon_tags_12 },
    { 0x2c9777f6, SUC_lexicon_tags_43 },
    { 0x919777f6, SUC_lexicon_tags_15 },
    { 0xd2f777f5, SUC_lexicon_tags_14 },
    { 0x9d777f9, SUC_lexicon_tags_13 },
    { 0xa47777fa, SUC_lexicon_tags_15 },
    { 0xfd9777fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61777ff, SUC_lexicon_tags_12 },
    { 0xb0177800, SUC_lexicon_tags_45 },
    { 0xccf77801, SUC_lexicon_tags_37 },
    { 0xbd777ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf77805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19577808, SUC_lexicon_tags_1 },
    { 0xdb177808, SUC_lexicon_tags_15 },
    { 0xad17780a, SUC_lexicon_tags_47 },
    { 0x22b7780b, SUC_lexicon_tags_37 },
    { 0x4577780c, SUC_lexicon_tags_45 },
    { 0x55f7780d, SUC_lexicon_tags_12 },
    { 0x6357780b, SUC_lexicon_tags_2 },
    { 0x3f377808, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e577814, SUC_lexicon_tags_12 },
    { 0xac777815, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2777817, SUC_lexicon_tags_77 },
    { 0x36577818, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30b7781a, SUC_lexicon_tags_17 },
    { 0x35b7781b, SUC_lexicon_tags_0 },
    { 0x1e97781c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb337781e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21777823, SUC_lexicon_tags_0 },
    { 0xca577823, SUC_lexicon_tags_7 },
    { 0x6d977824, SUC_lexicon_tags_38 },
    { 0xda777826, SUC_lexicon_tags_15 },
    { 0xfd577826, SUC_lexicon_tags_2 },
    { 0x8177828, SUC_lexicon_tags_87 },
    { 0x377829, SUC_lexicon_tags_37 },
    { 0x2817782a, SUC_lexicon_tags_64 },
    { 0xc457782b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b7782e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab77832, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82377834, SUC_lexicon_tags_87 },
    { 0x30977835, SUC_lexicon_tags_11 },
    { 0xb0177836, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1497783d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46777841, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x67377843, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xff377846, SUC_lexicon_tags_7 },
    { 0x94d77846, SUC_lexicon_tags_7 },
    { 0x62177848, SUC_lexicon_tags_25 },
    { 0x70177849, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab7784f, SUC_lexicon_tags_15 },
    { 0x3b977850, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f77854, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30d77856, SUC_lexicon_tags_15 },
    { 0x8d977857, SUC_lexicon_tags_39 },
    { 0xad77858, SUC_lexicon_tags_15 },
    { 0xc2377858, SUC_lexicon_tags_7 },
    { 0x96577859, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d17785e, SUC_lexicon_tags_34 },
    { 0x21f7785f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f77862, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68977864, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e37786a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4cd7786c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d7786f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b77873, SUC_lexicon_tags_8 },
    { 0xbbb77873, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab7787c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd77787e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b77882, SUC_lexicon_tags_12 },
    { 0x69b77882, SUC_lexicon_tags_69 },
    { 0x44177882, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad77887, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b7788d, SUC_lexicon_tags_47 },
    { 0xc1f7788d, SUC_lexicon_tags_11 },
    { 0x8fb7788d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc1777891, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8cf77893, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xabd77895, SUC_lexicon_tags_21 },
    { 0x90b77896, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f77899, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d17789b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3778a3, SUC_lexicon_tags_17 },
    { 0x131778a4, SUC_lexicon_tags_37 },
    { 0x665778a4, SUC_lexicon_tags_9 },
    { 0xee7778a3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3778a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f778ac, SUC_lexicon_tags_2 },
    { 0x483778ad, SUC_lexicon_tags_1 },
    { 0xff5778ad, SUC_lexicon_tags_12 },
    { 0x8f778af, SUC_lexicon_tags_11 },
    { 0x513778ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe4d778b2, SUC_lexicon_tags_25 },
    { 0x751778b3, SUC_lexicon_tags_54 },
    { 0xef9778b4, SUC_lexicon_tags_27 },
    { 0x6c9778b5, SUC_lexicon_tags_25 },
    { 0xeeb778b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747778bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681778c2, SUC_lexicon_tags_17 },
    { 0xa4d778c3, SUC_lexicon_tags_43 },
    { 0xf41778c4, SUC_lexicon_tags_54 },
    { 0xdcb778c4, SUC_lexicon_tags_26 },
    { 0x61f778c6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x42b778c8, SUC_lexicon_tags_23 },
    { 0xdf9778c9, SUC_lexicon_tags_2 },
    { 0xf6d778ca, SUC_lexicon_tags_104 },
    { 0xe99778cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe83778cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c9778cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d778d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3778df, SUC_lexicon_tags_33 },
    { 0x2b778e0, SUC_lexicon_tags_17 },
    { 0xb6d778df, SUC_lexicon_tags_39 },
    { 0xd1f778df, SUC_lexicon_tags_28 },
    { 0x183778e3, SUC_lexicon_tags_25 },
    { 0x8af778e4, SUC_lexicon_tags_87 },
    { 0x3fb778e4, SUC_lexicon_tags_61 },
    { 0x787778e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x401778e9, SUC_lexicon_tags_15 },
    { 0xd09778e9, SUC_lexicon_tags_11 },
    { 0x14f778eb, SUC_lexicon_tags_47 },
    { 0x891778ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f778f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x673778f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3dd778f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb778f7, SUC_lexicon_tags_57 },
    { 0xbc7778f7, SUC_lexicon_tags_37 },
    { 0xabf778f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x537778fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d778fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd77902, SUC_lexicon_tags_4 },
    { 0xcb377903, SUC_lexicon_tags_56 },
    { 0x94f77904, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d77907, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7ed77909, SUC_lexicon_tags_25 },
    { 0xc0f77909, SUC_lexicon_tags_30 },
    { 0x5117790a, SUC_lexicon_tags_12 },
    { 0xb077790c, SUC_lexicon_tags_81 },
    { 0x36b77909, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6717790f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16577916, SUC_lexicon_tags_8 },
    { 0x45777917, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x16377919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d57791e, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x9ab77920, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f7792a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18d7792c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45177935, SUC_lexicon_tags_48 },
    { 0xa3177936, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3617793a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5977793c, SUC_lexicon_tags_25 },
    { 0x9277793d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7857793f, SUC_lexicon_tags_17 },
    { 0x2ad7793f, SUC_lexicon_tags_17 },
    { 0x6bd7793f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8177943, SUC_lexicon_tags_7 },
    { 0x23777944, SUC_lexicon_tags_1 },
    { 0xeb577944, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e77794b, SUC_lexicon_tags_49 },
    { 0x21f7794c, SUC_lexicon_tags_38 },
    { 0x9677794d, SUC_lexicon_tags_7 },
    { 0x5517794e, SUC_lexicon_tags_19 },
    { 0xf457794e, SUC_lexicon_tags_7 },
    { 0x3257794f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd777959, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xec57795c, SUC_lexicon_tags_26 },
    { 0xed7795d, SUC_lexicon_tags_253 },
    { 0x7177795e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76577963, SUC_lexicon_tags_57 },
    { 0x23577964, SUC_lexicon_tags_25 },
    { 0xe0777964, SUC_lexicon_tags_34 },
    { 0x23d77965, SUC_lexicon_tags_55 },
    { 0x23977967, SUC_lexicon_tags_11 },
    { 0xa1777966, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf457796d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b77971, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde977973, SUC_lexicon_tags_2 },
    { 0xded77974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef77978, SUC_lexicon_tags_48 },
    { 0xec577979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c77797e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c17798b, SUC_lexicon_tags_12 },
    { 0xee77798c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56777991, SUC_lexicon_tags_23 },
    { 0x78377992, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94d77994, SUC_lexicon_tags_60 },
    { 0x82977994, SUC_lexicon_tags_11 },
    { 0x1d177996, SUC_lexicon_tags_57 },
    { 0xc7577994, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb7799a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7817799c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd7799f, SUC_lexicon_tags_11 },
    { 0x737779a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b779a4, SUC_lexicon_tags_0 },
    { 0x421779a5, SUC_lexicon_tags_47 },
    { 0xf3b779a6, SUC_lexicon_tags_16 },
    { 0xe3f779a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593779ac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27779af, SUC_lexicon_tags_25 },
    { 0x715779b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5779b4, SUC_lexicon_tags_37 },
    { 0xd93779b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab779bf, SUC_lexicon_tags_45 },
    { 0x51d779c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637779c5, SUC_lexicon_tags_12 },
    { 0x191779c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1779ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d779d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfd779d2, SUC_lexicon_tags_27 },
    { 0x199779d2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3f7779d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x28f779d7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d779dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x981779df, SUC_lexicon_tags_11 },
    { 0x6b5779e0, SUC_lexicon_tags_46 },
    { 0xeb779e1, SUC_lexicon_tags_47 },
    { 0x36f779e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd779e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173779eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd39779ed, SUC_lexicon_tags_7 },
    { 0xe8b779ee, SUC_lexicon_tags_58 },
    { 0xd21779ef, SUC_lexicon_tags_34 },
    { 0x91d779f0, SUC_lexicon_tags_3 },
    { 0x1cd779f1, SUC_lexicon_tags_37 },
    { 0x6e5779f2, SUC_lexicon_tags_7 },
    { 0x59f779f3, SUC_lexicon_tags_11 },
    { 0x69b779ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb779f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x129779fb, SUC_lexicon_tags_65 },
    { 0x55d779fc, SUC_lexicon_tags_2 },
    { 0x1f7779fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d77a00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55377a03, SUC_lexicon_tags_6 },
    { 0xdcb77a04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb1177a06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44977a08, SUC_lexicon_tags_33 },
    { 0x3bb77a08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d977a0b, SUC_lexicon_tags_25 },
    { 0x58377a0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90577a10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33977a19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6b77a1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77777a1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a777a21, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed77a24, SUC_lexicon_tags_49 },
    { 0x5bd77a25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df77a29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a577a2d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b177a34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d77a37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71b77a39, SUC_lexicon_tags_2 },
    { 0xc6777a3a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7577a3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54577a3f, SUC_lexicon_tags_5 },
    { 0xf9d77a40, SUC_lexicon_tags_1 },
    { 0x30377a41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10977a48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4777a4c, SUC_lexicon_tags_40 },
    { 0x47b77a4c, SUC_lexicon_tags_15 },
    { 0x6e977a4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b77a57, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x4ef77a59, SUC_lexicon_tags_48 },
    { 0x96577a59, SUC_lexicon_tags_33 },
    { 0x59577a59, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2ed77a5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85377a61, SUC_lexicon_tags_9 },
    { 0x95177a62, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f577a6b, SUC_lexicon_tags_15 },
    { 0x31777a6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa177a71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39777a73, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf8d77a75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b77a7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb177a7d, SUC_lexicon_tags_2 },
    { 0x62d77a7e, SUC_lexicon_tags_19 },
    { 0x97b77a7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42977a87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89777a8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9777a8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9377a91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38577a96, SUC_lexicon_tags_42 },
    { 0x6bf77a97, SUC_lexicon_tags_8 },
    { 0xfa777a98, SUC_lexicon_tags_70 },
    { 0x1d977a99, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e977a9c, SUC_lexicon_tags_19 },
    { 0xd6d77a9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9977aa0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3577aa2, SUC_lexicon_tags_0 },
    { 0x48377aa2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f77aa6, SUC_lexicon_tags_12 },
    { 0x1e577aa6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85377aae, SUC_lexicon_tags_11 },
    { 0x8a177aaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35977ab4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe1177ab6, SUC_lexicon_tags_60 },
    { 0x94177ab7, SUC_lexicon_tags_4 },
    { 0xc2177ab7, SUC_lexicon_tags_65 },
    { 0xbdb77ab9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b77abd, SUC_lexicon_tags_37 },
    { 0xac177abe, SUC_lexicon_tags_12 },
    { 0xed77abd, SUC_lexicon_tags_26 },
    { 0x76d77ac0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df77ac4, SUC_lexicon_tags_2 },
    { 0x9f777ac5, SUC_lexicon_tags_17 },
    { 0x6d777ac5, SUC_lexicon_tags_60 },
    { 0xd0d77ac7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xee777ac9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b377acb, SUC_lexicon_tags_27 },
    { 0xd3f77acb, SUC_lexicon_tags_2 },
    { 0x28977acc, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b77ad0, SUC_lexicon_tags_12 },
    { 0xb9177ad1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5177ada, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x19777adc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4777ade, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd77ae5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x37b77ae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f77aec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d577af1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff77af6, SUC_lexicon_tags_31 },
    { 0xf1f77af7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x50177af9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d77aff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8977b01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xddb77b03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef77b08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f77b0c, SUC_lexicon_tags_27 },
    { 0x6bd77b0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34977b13, SUC_lexicon_tags_45 },
    { 0x61b77b13, SUC_lexicon_tags_8 },
    { 0x64577b15, SUC_lexicon_tags_27 },
    { 0xecd77b16, SUC_lexicon_tags_2 },
    { 0xfe377b15, SUC_lexicon_tags_7 },
    { 0x6cf77b18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52377b1d, SUC_lexicon_tags_33 },
    { 0xf577b1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b777b21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7777b24, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x11577b26, SUC_lexicon_tags_107 },
    { 0x43977b27, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab77b2b, SUC_lexicon_tags_11 },
    { 0x11d77b2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce577b30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51577b33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x77b77b35, SUC_lexicon_tags_11 },
    { 0xe1f77b35, SUC_lexicon_tags_0 },
    { 0x81377b37, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af77b3b, SUC_lexicon_tags_12 },
    { 0x33377b3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x46377b3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x89777b41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b77b44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x85777b46, SUC_lexicon_tags_2 },
    { 0x4c777b47, SUC_lexicon_tags_27 },
    { 0xb7b77b46, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3977b4b, SUC_lexicon_tags_136 },
    { 0xd9177b4c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x13377b4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67377b53, SUC_lexicon_tags_31 },
    { 0x7b577b53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xca377b56, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b377b5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4377b5c, SUC_lexicon_tags_15 },
    { 0xe6777b5d, SUC_lexicon_tags_17 },
    { 0x14f77b5e, SUC_lexicon_tags_15 },
    { 0x44d77b5d, SUC_lexicon_tags_34 },
    { 0x6f177b5c, SUC_lexicon_tags_38 },
    { 0xcd377b61, SUC_lexicon_tags_23 },
    { 0x79777b62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d77b67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c377b69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6d177b6b, SUC_lexicon_tags_17 },
    { 0x99b77b6c, SUC_lexicon_tags_7 },
    { 0x99f77b6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1777b71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f77b75, SUC_lexicon_tags_0 },
    { 0xd1377b75, SUC_lexicon_tags_10 },
    { 0xc6b77b77, SUC_lexicon_tags_17 },
    { 0x43377b78, SUC_lexicon_tags_30 },
    { 0xf7f77b79, SUC_lexicon_tags_1 },
    { 0x3b377b79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd77b7d, SUC_lexicon_tags_27 },
    { 0xa5777b7d, SUC_lexicon_tags_29 },
    { 0x94f77b7f, SUC_lexicon_tags_15 },
    { 0x99977b7d, SUC_lexicon_tags_30 },
    { 0xd0b77b81, SUC_lexicon_tags_37 },
    { 0x9ad77b82, SUC_lexicon_tags_17 },
    { 0x58777b7f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xf7177b85, SUC_lexicon_tags_17 },
    { 0x9f377b86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ab77b88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4177b8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b77b8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a777b90, SUC_lexicon_tags_25 },
    { 0xae377b91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6177b94, SUC_lexicon_tags_45 },
    { 0x84577b95, SUC_lexicon_tags_30 },
    { 0x38b77b95, SUC_lexicon_tags_15 },
    { 0x40d77b97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0377b9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf977b9c, SUC_lexicon_tags_27 },
    { 0x53b77b9d, SUC_lexicon_tags_37 },
    { 0x46577b9e, SUC_lexicon_tags_0 },
    { 0xa5377b9c, SUC_lexicon_tags_26 },
    { 0xdf377ba0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x14977ba2, SUC_lexicon_tags_2 },
    { 0xa7377ba3, SUC_lexicon_tags_27 },
    { 0x50d77ba3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x71d77ba6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd977ba9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0377bb0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e777bb3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6377bb8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd177bc5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36377bc7, SUC_lexicon_tags_29 },
    { 0x4f977bc7, SUC_lexicon_tags_37 },
    { 0xc3b77bc8, SUC_lexicon_tags_84 },
    { 0x36177bca, SUC_lexicon_tags_55 },
    { 0x7e577bcb, SUC_lexicon_tags_15 },
    { 0xf2977bc8, SUC_lexicon_tags_38 },
    { 0xde377bcd, SUC_lexicon_tags_1 },
    { 0xbe777bcd, SUC_lexicon_tags_113 },
    { 0x60977bcf, SUC_lexicon_tags_0 },
    { 0xa777bd0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f77bd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb77bd9, SUC_lexicon_tags_1 },
    { 0x56177bda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdf777bdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59777be1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfd777be3, SUC_lexicon_tags_12 },
    { 0x91577be4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7577be8, SUC_lexicon_tags_7 },
    { 0x9d777be8, SUC_lexicon_tags_45 },
    { 0x45b77bea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f77bf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb77bf4, SUC_lexicon_tags_43 },
    { 0x6cb77bf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f77bfe, SUC_lexicon_tags_1046 },
    { 0x4df77bff, SUC_lexicon_tags_68 },
    { 0x93b77bff, SUC_lexicon_tags_33 },
    { 0xc5577c01, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b77c04, SUC_lexicon_tags_17 },
    { 0xa3577c05, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x27777c08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4177c0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd77c12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbcf77c14, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x47177c16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77777c18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9377c20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f77c24, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xbd977c26, SUC_lexicon_tags_25 },
    { 0x23377c27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d77c2b, SUC_lexicon_tags_11 },
    { 0x8eb77c2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35777c31, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xead77c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f77c38, SUC_lexicon_tags_31 },
    { 0xec177c39, SUC_lexicon_tags_68 },
    { 0x43377c39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc977c3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50977c3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f77c42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d77c44, SUC_lexicon_tags_60 },
    { 0xa4b77c45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0577c4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d177c4c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xec177c4e, SUC_lexicon_tags_31 },
    { 0x58b77c4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9377c58, SUC_lexicon_tags_7 },
    { 0xed777c59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e377c5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d977c5d, SUC_lexicon_tags_1 },
    { 0x90f77c5e, SUC_lexicon_tags_57 },
    { 0xdf377c5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b77c66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27977c68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57b77c6a, SUC_lexicon_tags_14 },
    { 0xe6577c6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48977c6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64977c6f, SUC_lexicon_tags_16 },
    { 0x24577c70, SUC_lexicon_tags_25 },
    { 0xbe977c71, SUC_lexicon_tags_10 },
    { 0x3e977c71, SUC_lexicon_tags_11 },
    { 0xb6577c73, SUC_lexicon_tags_19 },
    { 0xfef77c73, SUC_lexicon_tags_1 },
    { 0xad977c73, SUC_lexicon_tags_2 },
    { 0x11577c72, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e377c7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f77c7e, SUC_lexicon_tags_43 },
    { 0x8e177c7f, SUC_lexicon_tags_14 },
    { 0x6f377c7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87177c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb77c85, SUC_lexicon_tags_23 },
    { 0x46b77c86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x97777c89, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb6177c8b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xa9b77c8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df77c91, SUC_lexicon_tags_7 },
    { 0xc4777c92, SUC_lexicon_tags_38 },
    { 0x6c377c93, SUC_lexicon_tags_12 },
    { 0xdc777c94, SUC_lexicon_tags_2 },
    { 0x24577c94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c777c98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf77c9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68577ca9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf77caf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x577cb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfb77cb3, SUC_lexicon_tags_11 },
    { 0xc8f77cb4, SUC_lexicon_tags_4 },
    { 0xc6377cb5, SUC_lexicon_tags_37 },
    { 0xc8d77cb4, SUC_lexicon_tags_174 },
    { 0xc5377cb7, SUC_lexicon_tags_20 },
    { 0xa7177cb6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb77cbc, SUC_lexicon_tags_82 },
    { 0x36377cbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1177cc3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4177cc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d77ccc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87377cd0, SUC_lexicon_tags_7 },
    { 0xcbf77cd1, SUC_lexicon_tags_10 },
    { 0x63b77cd2, SUC_lexicon_tags_11 },
    { 0x48b77cd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83977cda, SUC_lexicon_tags_19 },
    { 0xbcb77cdb, SUC_lexicon_tags_108 },
    { 0x50377cdb, SUC_lexicon_tags_10 },
    { 0xd4777cdd, SUC_lexicon_tags_82 },
    { 0x2bd77cdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73f77ce0, SUC_lexicon_tags_167 },
    { 0x50777ce1, SUC_lexicon_tags_1 },
    { 0x4e177ce2, SUC_lexicon_tags_3 },
    { 0x23d77ce3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba977ce9, SUC_lexicon_tags_37 },
    { 0xa1d77cea, SUC_lexicon_tags_2 },
    { 0xd5577ceb, SUC_lexicon_tags_49 },
    { 0xd8d77cec, SUC_lexicon_tags_17 },
    { 0xd0777ce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b77cf2, SUC_lexicon_tags_17 },
    { 0x8b377cf2, SUC_lexicon_tags_47 },
    { 0x86977cf4, SUC_lexicon_tags_0 },
    { 0x7b577cf5, SUC_lexicon_tags_37 },
    { 0xc9777cf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbf77cf8, SUC_lexicon_tags_11 },
    { 0x89177cf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64977cfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72577d02, SUC_lexicon_tags_17 },
    { 0xee977d03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d77d06, SUC_lexicon_tags_47 },
    { 0x8a577d07, SUC_lexicon_tags_28 },
    { 0xe0777d08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16977d0e, SUC_lexicon_tags_9 },
    { 0x23177d0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e977d12, SUC_lexicon_tags_14 },
    { 0x7f977d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71377d15, SUC_lexicon_tags_25 },
    { 0x7af77d16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d77d1d, SUC_lexicon_tags_5 },
    { 0x3dd77d1e, SUC_lexicon_tags_2 },
    { 0x41177d1f, SUC_lexicon_tags_25 },
    { 0x88f77d1f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6d377d22, SUC_lexicon_tags_27 },
    { 0x99f77d22, SUC_lexicon_tags_2 },
    { 0xb0377d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2177d28, SUC_lexicon_tags_25 },
    { 0x74977d29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1977d2b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x68f77d2d, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0x3377d30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78177d32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x98177d35, SUC_lexicon_tags_37 },
    { 0x10577d36, SUC_lexicon_tags_62 },
    { 0x9f77d37, SUC_lexicon_tags_11 },
    { 0xcd777d36, SUC_lexicon_tags_7 },
    { 0x9d577d39, SUC_lexicon_tags_45 },
    { 0x24177d3a, SUC_lexicon_tags_34 },
    { 0x6cd77d3a, SUC_lexicon_tags_25 },
    { 0x2f77d3c, SUC_lexicon_tags_49 },
    { 0x2d377d3d, SUC_lexicon_tags_10 },
    { 0x32f77d3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b77d42, SUC_lexicon_tags_17 },
    { 0xbb777d43, SUC_lexicon_tags_27 },
    { 0x14577d44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9b977d46, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31777d51, SUC_lexicon_tags_37 },
    { 0xbab77d52, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xe8177d54, SUC_lexicon_tags_45 },
    { 0x38977d55, SUC_lexicon_tags_14 },
    { 0xd0b77d56, SUC_lexicon_tags_82 },
    { 0xa9377d57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b577d5b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x66177d5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x20177d61, SUC_lexicon_tags_2 },
    { 0xedd77d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67177d65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc377d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f977d6d, SUC_lexicon_tags_15 },
    { 0xd4177d6e, SUC_lexicon_tags_4 },
    { 0x9fb77d6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa577d71, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0977d75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4377d7c, SUC_lexicon_tags_1 },
    { 0x6d177d7d, SUC_lexicon_tags_9 },
    { 0x51d77d7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25177d81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb77d84, SUC_lexicon_tags_61 },
    { 0xf777d84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddf77d87, SUC_lexicon_tags_27 },
    { 0x27377d88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf577d8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1177d94, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f77d9b, SUC_lexicon_tags_1 },
    { 0x83377d9b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed77da3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcd77da5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b77dad, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f77db0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d977db5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87d77db7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2777dbc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1977dc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b77dc7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27d77dc9, SUC_lexicon_tags_15 },
    { 0xb0f77dc9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92977dcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2777dd1, SUC_lexicon_tags_56 },
    { 0xfd977dd2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d77dd7, SUC_lexicon_tags_21 },
    { 0xeaf77dd7, SUC_lexicon_tags_37 },
    { 0x7e977dd8, SUC_lexicon_tags_11 },
    { 0x24377dda, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc7b77ddc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15f77dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d77de6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57977dea, SUC_lexicon_tags_39 },
    { 0xa1177dea, SUC_lexicon_tags_17 },
    { 0x6c577dec, SUC_lexicon_tags_47 },
    { 0x15577dec, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd77df0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d577df3, SUC_lexicon_tags_37 },
    { 0xd5b77df4, SUC_lexicon_tags_28 },
    { 0x81177df5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8177dfb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a977e01, SUC_lexicon_tags_25 },
    { 0xb3f77e02, SUC_lexicon_tags_31 },
    { 0x38577e03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x38377e06, SUC_lexicon_tags_10 },
    { 0xcf177e06, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xce377e0a, SUC_lexicon_tags_31 },
    { 0xb9f77e0a, SUC_lexicon_tags_9 },
    { 0xa8b77e0b, SUC_lexicon_tags_49 },
    { 0x4f77e0d, SUC_lexicon_tags_0 },
    { 0xdad77e0d, SUC_lexicon_tags_33 },
    { 0x17777e0e, SUC_lexicon_tags_47 },
    { 0xfc177e0b, SUC_lexicon_tags_66 },
    { 0xc3977e0d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3977e15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23377e18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd77e1b, SUC_lexicon_tags_14 },
    { 0x53977e1c, SUC_lexicon_tags_15 },
    { 0xf7577e1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b77e24, SUC_lexicon_tags_7 },
    { 0xbf977e25, SUC_lexicon_tags_7 },
    { 0x99377e26, SUC_lexicon_tags_63 },
    { 0x5b377e24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x14977e2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c977e2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e177e32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ad77e34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8af77e36, SUC_lexicon_tags_11 },
    { 0x33f77e37, SUC_lexicon_tags_26 },
    { 0x10977e38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60377e3c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f177e41, SUC_lexicon_tags_4 },
    { 0x8e977e41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33177e44, SUC_lexicon_tags_8 },
    { 0xa2b77e44, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9777e4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3f77e4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5977e51, SUC_lexicon_tags_7 },
    { 0x14577e52, SUC_lexicon_tags_0 },
    { 0x33377e53, SUC_lexicon_tags_23 },
    { 0xe3977e53, SUC_lexicon_tags_2 },
    { 0xac577e54, SUC_lexicon_tags_19 },
    { 0x3cd77e52, SUC_lexicon_tags_63 },
    { 0x17f77e57, SUC_lexicon_tags_0 },
    { 0xe8d77e54, SUC_lexicon_tags_12 },
    { 0xf2d77e59, SUC_lexicon_tags_17 },
    { 0x19b77e52, SUC_lexicon_tags_45 },
    { 0x1577e53, SUC_lexicon_tags_8 },
    { 0x1dd77e52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7b77e5e, SUC_lexicon_tags_16 },
    { 0x9d77e5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf77e62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f77e65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb977e67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f977e69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f577e6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc577e73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77577e77, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe3377e79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9777e7c, SUC_lexicon_tags_33 },
    { 0xfa177e7d, SUC_lexicon_tags_17 },
    { 0xd8f77e7d, SUC_lexicon_tags_25 },
    { 0xbd377e7f, SUC_lexicon_tags_55 },
    { 0x9a777e80, SUC_lexicon_tags_8 },
    { 0xbe177e7e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c177e85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64977e87, SUC_lexicon_tags_25 },
    { 0x2d377e87, SUC_lexicon_tags_70 },
    { 0x53377e89, SUC_lexicon_tags_47 },
    { 0xc5177e87, SUC_lexicon_tags_17 },
    { 0x3c777e89, SUC_lexicon_tags_6 },
    { 0xd7977e8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd3d77e8e, SUC_lexicon_tags_23 },
    { 0xf5977e8e, SUC_lexicon_tags_55 },
    { 0xb1977e90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f77e93, SUC_lexicon_tags_35 },
    { 0x3d377e94, SUC_lexicon_tags_31 },
    { 0x17f77e94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf77e98, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf77e9c, SUC_lexicon_tags_46 },
    { 0x71d77e9d, SUC_lexicon_tags_2 },
    { 0x31777e9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecf77ea0, SUC_lexicon_tags_14 },
    { 0x44977ea0, SUC_lexicon_tags_104 },
    { 0x57177ea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd777ea4, SUC_lexicon_tags_47 },
    { 0x10977ea4, SUC_lexicon_tags_39 },
    { 0x7eb77ea6, SUC_lexicon_tags_8 },
    { 0xe5777ea7, SUC_lexicon_tags_0 },
    { 0xd8f77ea8, SUC_lexicon_tags_1 },
    { 0x7b77ea7, SUC_lexicon_tags_7 },
    { 0xc3577eaa, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4777eac, SUC_lexicon_tags_25 },
    { 0x7b177eac, SUC_lexicon_tags_46 },
    { 0x70377ead, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8577eb1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6977eb6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b77ebb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa9d77ebd, SUC_lexicon_tags_17 },
    { 0x71f77ebe, SUC_lexicon_tags_48 },
    { 0xc2777ebf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x81d77ec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf77ec7, SUC_lexicon_tags_0 },
    { 0x35977ec7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3577ecc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x54777ece, SUC_lexicon_tags_17 },
    { 0xf8f77ece, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x89177ed1, SUC_lexicon_tags_47 },
    { 0x4cf77ed2, SUC_lexicon_tags_14 },
    { 0x6db77ed3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f77ed8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b77edc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b77ee2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fb77ee4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a377ee8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7fb77eea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9177eee, SUC_lexicon_tags_84 },
    { 0x80b77eef, SUC_lexicon_tags_17 },
    { 0xa4377eef, SUC_lexicon_tags_396 },
    { 0xe6b77ef0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2577ef3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7177ef5, SUC_lexicon_tags_11 },
    { 0x5b377ef6, SUC_lexicon_tags_2 },
    { 0x50377ef7, SUC_lexicon_tags_1 },
    { 0x1ab77ef8, SUC_lexicon_tags_17 },
    { 0xa3777ef8, SUC_lexicon_tags_38 },
    { 0xa4977efa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90577f00, SUC_lexicon_tags_25 },
    { 0xc3977f01, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb77f08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6577f0d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8977f12, SUC_lexicon_tags_61 },
    { 0x18177f13, SUC_lexicon_tags_12 },
    { 0xb5b77f14, SUC_lexicon_tags_1 },
    { 0x3e177f15, SUC_lexicon_tags_48 },
    { 0x50f77f16, SUC_lexicon_tags_7 },
    { 0x4777f13, SUC_lexicon_tags_14 },
    { 0xd0577f18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79977f1b, SUC_lexicon_tags_17 },
    { 0xa8377f1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6577f1e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d177f21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3177f23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87577f26, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8a977f28, SUC_lexicon_tags_14 },
    { 0x77b77f29, SUC_lexicon_tags_11 },
    { 0x25977f2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x77d77f2c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a377f30, SUC_lexicon_tags_11 },
    { 0x1af77f31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x24b77f33, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7e977f35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b377f38, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea177f3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f777f41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62577f43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f77f46, SUC_lexicon_tags_3 },
    { 0x12377f46, SUC_lexicon_tags_4 },
    { 0xc0d77f48, SUC_lexicon_tags_4 },
    { 0xe6377f49, SUC_lexicon_tags_63 },
    { 0x91377f4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9f77f4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31777f4e, SUC_lexicon_tags_19 },
    { 0x9e377f4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83177f51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb777f58, SUC_lexicon_tags_3 },
    { 0x6ef77f59, SUC_lexicon_tags_7 },
    { 0xaf177f5a, SUC_lexicon_tags_0 },
    { 0x6bd77f5b, SUC_lexicon_tags_26 },
    { 0x6db77f5a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6d577f5e, SUC_lexicon_tags_43 },
    { 0x8177f5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf77f63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37777f68, SUC_lexicon_tags_10 },
    { 0xf9b77f69, SUC_lexicon_tags_0 },
    { 0x1e977f6a, SUC_lexicon_tags_46 },
    { 0x56377f6b, SUC_lexicon_tags_80 },
    { 0xcf777f6b, SUC_lexicon_tags_2 },
    { 0x70377f6d, SUC_lexicon_tags_9 },
    { 0x19b77f6d, SUC_lexicon_tags_6 },
    { 0xb4777f6d, SUC_lexicon_tags_25 },
    { 0x2c777f6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d77f73, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d77f7b, SUC_lexicon_tags_31 },
    { 0x8e777f7c, SUC_lexicon_tags_14 },
    { 0xffb77f7d, SUC_lexicon_tags_3 },
    { 0x3a777f7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70777f80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c577f82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f77f86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d577f89, SUC_lexicon_tags_1 },
    { 0x9a177f8a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63577f8f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2777f93, SUC_lexicon_tags_36 },
    { 0x86777f94, SUC_lexicon_tags_47 },
    { 0x5af77f93, SUC_lexicon_tags_46 },
    { 0x25377f95, SUC_lexicon_tags_14 },
    { 0xd0977f97, SUC_lexicon_tags_79 },
    { 0x64d77f98, SUC_lexicon_tags_7 },
    { 0xf1977f99, SUC_lexicon_tags_60 },
    { 0xcd177f9a, SUC_lexicon_tags_19 },
    { 0x75d77f97, SUC_lexicon_tags_37 },
    { 0x59977f9c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcef77f9e, SUC_lexicon_tags_39 },
    { 0xdfd77f9f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x26177fa1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97577fa9, SUC_lexicon_tags_32 },
    { 0x24b77faa, SUC_lexicon_tags_1 },
    { 0x5cf77fab, SUC_lexicon_tags_17 },
    { 0xcab77faa, SUC_lexicon_tags_17 },
    { 0x42d77fac, SUC_lexicon_tags_29 },
    { 0xe4f77fab, SUC_lexicon_tags_0 },
    { 0x5e577faf, SUC_lexicon_tags_37 },
    { 0xc7f77faf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8df77fb2, SUC_lexicon_tags_17 },
    { 0xe5977fb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89777fb5, SUC_lexicon_tags_17 },
    { 0x45177fb5, SUC_lexicon_tags_15 },
    { 0x74d77fb7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x24777fb9, SUC_lexicon_tags_43 },
    { 0x5d377fba, SUC_lexicon_tags_46 },
    { 0x7fb77fbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e777fbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42377fc0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9777fc4, SUC_lexicon_tags_17 },
    { 0x4c177fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45177fc7, SUC_lexicon_tags_16 },
    { 0x34b77fc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b577fca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88d77fcc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6f377fce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b77fd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91577fd6, SUC_lexicon_tags_45 },
    { 0xb5777fd7, SUC_lexicon_tags_45 },
    { 0xadb77fd7, SUC_lexicon_tags_11 },
    { 0x67377fd9, SUC_lexicon_tags_12 },
    { 0xead77fda, SUC_lexicon_tags_1 },
    { 0x6a377fd7, SUC_lexicon_tags_63 },
    { 0x9af77fdc, SUC_lexicon_tags_2 },
    { 0x977fdd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5377fe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f77fe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d777fe6, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad77fe9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdc377feb, SUC_lexicon_tags_41 },
    { 0xa2577fec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d977ff5, SUC_lexicon_tags_45 },
    { 0xd7d77ff5, SUC_lexicon_tags_43 },
    { 0x99777ff7, SUC_lexicon_tags_43 },
    { 0xcbd77ff8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f77ffc, SUC_lexicon_tags_12 },
    { 0x1c977ffd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb978000, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c978002, SUC_lexicon_tags_29 },
    { 0xb778002, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6778009, SUC_lexicon_tags_19 },
    { 0xca57800a, SUC_lexicon_tags_11 },
    { 0x6917800b, SUC_lexicon_tags_33 },
    { 0x2317800c, SUC_lexicon_tags_5 },
    { 0x64d7800a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb78011, SUC_lexicon_tags_26 },
    { 0x18178011, SUC_lexicon_tags_17 },
    { 0xc2f78012, SUC_lexicon_tags_14 },
    { 0x24378014, SUC_lexicon_tags_11 },
    { 0xc0d78014, SUC_lexicon_tags_37 },
    { 0x56778016, SUC_lexicon_tags_25 },
    { 0x60178017, SUC_lexicon_tags_11 },
    { 0x9ab78013, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d7801b, SUC_lexicon_tags_113 },
    { 0xc117801b, SUC_lexicon_tags_113 },
    { 0x7197801d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b378020, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb78025, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6978027, SUC_lexicon_tags_213 },
    { 0x66b78028, SUC_lexicon_tags_1 },
    { 0x7d378029, SUC_lexicon_tags_27 },
    { 0x8fb7802a, SUC_lexicon_tags_70 },
    { 0x637802b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2077802e, SUC_lexicon_tags_0 },
    { 0x80d7802f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1af78031, SUC_lexicon_tags_12 },
    { 0xf7578031, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x43978035, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3477803b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe157803e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d78041, SUC_lexicon_tags_14 },
    { 0x71378042, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5778047, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e578049, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd97804d, SUC_lexicon_tags_7 },
    { 0x90b7804d, SUC_lexicon_tags_1 },
    { 0x6db7804d, SUC_lexicon_tags_32 },
    { 0xd877804e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f78058, SUC_lexicon_tags_2 },
    { 0x1e778059, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2d17805b, SUC_lexicon_tags_37 },
    { 0xb637805b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3eb7805e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5b778060, SUC_lexicon_tags_37 },
    { 0xa7178060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65178065, SUC_lexicon_tags_15 },
    { 0x99378066, SUC_lexicon_tags_27 },
    { 0x3978066, SUC_lexicon_tags_37 },
    { 0x1d78068, SUC_lexicon_tags_2 },
    { 0x58178066, SUC_lexicon_tags_11 },
    { 0xaf778067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5578072, SUC_lexicon_tags_46 },
    { 0x83978072, SUC_lexicon_tags_17 },
    { 0x81378074, SUC_lexicon_tags_38 },
    { 0xd7d78075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67178077, SUC_lexicon_tags_23 },
    { 0x5dd78078, SUC_lexicon_tags_25 },
    { 0x52d78077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d978085, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab7808a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1d7808c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8178090, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb78097, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd1d78099, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb7809c, SUC_lexicon_tags_25 },
    { 0x4f57809d, SUC_lexicon_tags_38 },
    { 0xb517809e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d780a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b780a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d780a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ad780aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x41780ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x963780ae, SUC_lexicon_tags_25 },
    { 0xb19780af, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3780b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b780b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x571780ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb780bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5780c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x665780c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1780cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3df780ce, SUC_lexicon_tags_7 },
    { 0x935780cf, SUC_lexicon_tags_63 },
    { 0xd9b780d0, SUC_lexicon_tags_26 },
    { 0xfc3780d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531780d5, SUC_lexicon_tags_15 },
    { 0x537780d6, SUC_lexicon_tags_63 },
    { 0xb99780d7, SUC_lexicon_tags_47 },
    { 0x741780d5, SUC_lexicon_tags_14 },
    { 0x267780d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f5780db, SUC_lexicon_tags_1 },
    { 0x745780dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x475780df, SUC_lexicon_tags_17 },
    { 0x4d780df, SUC_lexicon_tags_11 },
    { 0xef780e1, SUC_lexicon_tags_30 },
    { 0x5eb780df, SUC_lexicon_tags_29 },
    { 0xa81780e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5780eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925780f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1780f3, SUC_lexicon_tags_11 },
    { 0x4a3780f3, SUC_lexicon_tags_16 },
    { 0x551780f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd78102, SUC_lexicon_tags_23 },
    { 0xdcb78103, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf5b78105, SUC_lexicon_tags_63 },
    { 0x6ff78105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31578108, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80d7810a, SUC_lexicon_tags_8 },
    { 0x4c57810b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf037810e, SUC_lexicon_tags_39 },
    { 0xf0d7810f, SUC_lexicon_tags_15 },
    { 0xeb778110, SUC_lexicon_tags_38 },
    { 0xbf778111, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d78114, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99578116, SUC_lexicon_tags_27 },
    { 0x7fb78117, SUC_lexicon_tags_11 },
    { 0x17d78117, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb637811f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb78123, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb378126, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc337812a, SUC_lexicon_tags_85 },
    { 0xa577812b, SUC_lexicon_tags_8 },
    { 0x2a37812c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17778135, SUC_lexicon_tags_37 },
    { 0x32178135, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db7813c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e17813f, SUC_lexicon_tags_11 },
    { 0x51578140, SUC_lexicon_tags_0 },
    { 0x79b78141, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc178147, SUC_lexicon_tags_0 },
    { 0x91778148, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb597814c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad78150, SUC_lexicon_tags_15 },
    { 0x9c578151, SUC_lexicon_tags_7 },
    { 0xff178152, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc578155, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a578158, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabd7815a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2bf7815c, SUC_lexicon_tags_25 },
    { 0x4c97815d, SUC_lexicon_tags_45 },
    { 0x7937815e, SUC_lexicon_tags_92 },
    { 0x12b7815f, SUC_lexicon_tags_32 },
    { 0x3e578160, SUC_lexicon_tags_31 },
    { 0xd317815d, SUC_lexicon_tags_47 },
    { 0x9b78162, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1ed78164, SUC_lexicon_tags_25 },
    { 0x4f378165, SUC_lexicon_tags_45 },
    { 0x2c578166, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8378169, SUC_lexicon_tags_113 },
    { 0xd8178169, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd7816e, SUC_lexicon_tags_10 },
    { 0x2bb7816e, SUC_lexicon_tags_27 },
    { 0x38178170, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7a778172, SUC_lexicon_tags_8 },
    { 0x3e978172, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7778175, SUC_lexicon_tags_2 },
    { 0xd9b78176, SUC_lexicon_tags_31 },
    { 0xdd378177, SUC_lexicon_tags_17 },
    { 0x44578178, SUC_lexicon_tags_44 },
    { 0x91378176, SUC_lexicon_tags_37 },
    { 0xf817817a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae97817c, SUC_lexicon_tags_25 },
    { 0xcfb7817d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d78190, SUC_lexicon_tags_37 },
    { 0xcbf78191, SUC_lexicon_tags_38 },
    { 0x57778191, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x4a578194, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1cd78196, SUC_lexicon_tags_1 },
    { 0x2bd78197, SUC_lexicon_tags_15 },
    { 0x11178196, SUC_lexicon_tags_15 },
    { 0xd1378199, SUC_lexicon_tags_48 },
    { 0x6537819a, SUC_lexicon_tags_33 },
    { 0x6457819a, SUC_lexicon_tags_55 },
    { 0xa177819c, SUC_lexicon_tags_30 },
    { 0xfd77819d, SUC_lexicon_tags_17 },
    { 0x5537819d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae9781a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x919781a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269781a7, SUC_lexicon_tags_57 },
    { 0xdaf781a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x961781aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd71781ac, SUC_lexicon_tags_37 },
    { 0x185781ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1781b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151781bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263781c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7781c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef781c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef781ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96f781cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f781cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875781d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5781d8, SUC_lexicon_tags_37 },
    { 0xcb7781d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc45781db, SUC_lexicon_tags_26 },
    { 0xeb7781dc, SUC_lexicon_tags_43 },
    { 0x24f781dc, SUC_lexicon_tags_37 },
    { 0x9f3781dc, SUC_lexicon_tags_2 },
    { 0xadf781df, SUC_lexicon_tags_7 },
    { 0x571781df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e5781e2, SUC_lexicon_tags_11 },
    { 0xdf781e3, SUC_lexicon_tags_47 },
    { 0x551781e3, SUC_lexicon_tags_25 },
    { 0xa8d781e5, SUC_lexicon_tags_11 },
    { 0x8bb781e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07781eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599781f3, SUC_lexicon_tags_12 },
    { 0xc0f781f4, SUC_lexicon_tags_15 },
    { 0x9a9781f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767781ff, SUC_lexicon_tags_107 },
    { 0x9c578200, SUC_lexicon_tags_65 },
    { 0x9b78201, SUC_lexicon_tags_55 },
    { 0xc3378201, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4df78204, SUC_lexicon_tags_8 },
    { 0x4f578204, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa997820c, SUC_lexicon_tags_15 },
    { 0xef17820c, SUC_lexicon_tags_16 },
    { 0x4e97820e, SUC_lexicon_tags_3 },
    { 0x36d7820d, SUC_lexicon_tags_25 },
    { 0xdad78210, SUC_lexicon_tags_46 },
    { 0x53778211, SUC_lexicon_tags_37 },
    { 0xba778210, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2ff78214, SUC_lexicon_tags_25 },
    { 0x6df78214, SUC_lexicon_tags_25 },
    { 0x57d78216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97578219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1b7821b, SUC_lexicon_tags_47 },
    { 0x3757821c, SUC_lexicon_tags_11 },
    { 0x6af7821b, SUC_lexicon_tags_1 },
    { 0x85b7821e, SUC_lexicon_tags_60 },
    { 0x797821e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x30378221, SUC_lexicon_tags_11 },
    { 0xd7778222, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76378226, SUC_lexicon_tags_12 },
    { 0xc6178226, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa5578229, SUC_lexicon_tags_27 },
    { 0x3ed7822a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18378235, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2978239, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b7823e, SUC_lexicon_tags_26 },
    { 0xd57823f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18778241, SUC_lexicon_tags_15 },
    { 0x88f78241, SUC_lexicon_tags_7 },
    { 0xe7778243, SUC_lexicon_tags_31 },
    { 0x5bb78244, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5378249, SUC_lexicon_tags_7 },
    { 0xd357824a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68978250, SUC_lexicon_tags_30 },
    { 0x33f78251, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b78255, SUC_lexicon_tags_6 },
    { 0xa0178255, SUC_lexicon_tags_26 },
    { 0x4ef78257, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25178259, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc57825b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb78261, SUC_lexicon_tags_37 },
    { 0xa9978262, SUC_lexicon_tags_17 },
    { 0x17f78262, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d37826b, SUC_lexicon_tags_7 },
    { 0xa137826c, SUC_lexicon_tags_14 },
    { 0x9d37826c, SUC_lexicon_tags_25 },
    { 0xd77826e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa578270, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb578274, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44978278, SUC_lexicon_tags_60 },
    { 0xc7778279, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa917827b, SUC_lexicon_tags_10 },
    { 0xd017827b, SUC_lexicon_tags_11 },
    { 0xf297827c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f78281, SUC_lexicon_tags_98 },
    { 0x2e578281, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xac778284, SUC_lexicon_tags_8 },
    { 0x55578285, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0578287, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc957828c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b778291, SUC_lexicon_tags_51 },
    { 0x9178292, SUC_lexicon_tags_19 },
    { 0x68978293, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9578298, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab97829f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d782a3, SUC_lexicon_tags_46 },
    { 0xa8b782a4, SUC_lexicon_tags_18 },
    { 0x499782a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa61782a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b782ab, SUC_lexicon_tags_11 },
    { 0x3cf782ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23782b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf782b3, SUC_lexicon_tags_69 },
    { 0xe39782b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595782b8, SUC_lexicon_tags_15 },
    { 0xc7f782b9, SUC_lexicon_tags_1 },
    { 0x329782b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbd7782bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b782c6, SUC_lexicon_tags_25 },
    { 0x9df782c6, SUC_lexicon_tags_33 },
    { 0x1f7782c8, SUC_lexicon_tags_1 },
    { 0x365782c6, SUC_lexicon_tags_17 },
    { 0xb75782ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x283782cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8a9782cf, SUC_lexicon_tags_37 },
    { 0xfcd782d0, SUC_lexicon_tags_66 },
    { 0xc71782d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101782d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x181782da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77d782dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d782e6, SUC_lexicon_tags_12 },
    { 0xac9782e7, SUC_lexicon_tags_6 },
    { 0x2b782e6, SUC_lexicon_tags_26 },
    { 0xf4d782e9, SUC_lexicon_tags_37 },
    { 0x935782ea, SUC_lexicon_tags_3 },
    { 0x55782eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9782f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb79782f2, SUC_lexicon_tags_2 },
    { 0xef9782f2, SUC_lexicon_tags_7 },
    { 0x8ab782f4, SUC_lexicon_tags_12 },
    { 0xdd9782f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd782fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b78300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a178304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d78307, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x297830a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f7830e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26578313, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc0778315, SUC_lexicon_tags_1 },
    { 0x2c778315, SUC_lexicon_tags_29 },
    { 0xd8978315, SUC_lexicon_tags_66 },
    { 0xc6b78318, SUC_lexicon_tags_38 },
    { 0x70178319, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f77831c, SUC_lexicon_tags_12 },
    { 0xa257831d, SUC_lexicon_tags_39 },
    { 0xed37831e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59f78320, SUC_lexicon_tags_7 },
    { 0x5ed78321, SUC_lexicon_tags_57 },
    { 0xa3778320, SUC_lexicon_tags_30 },
    { 0xea978323, SUC_lexicon_tags_2 },
    { 0x7d778324, SUC_lexicon_tags_16 },
    { 0x9cf78325, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa7978327, SUC_lexicon_tags_37 },
    { 0x2d778328, SUC_lexicon_tags_11 },
    { 0x7978329, SUC_lexicon_tags_65 },
    { 0x7e97832a, SUC_lexicon_tags_9 },
    { 0x3257832b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d78330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95178332, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8378334, SUC_lexicon_tags_25 },
    { 0x3c178334, SUC_lexicon_tags_48 },
    { 0xc7778336, SUC_lexicon_tags_77 },
    { 0x15978337, SUC_lexicon_tags_11 },
    { 0x3d378338, SUC_lexicon_tags_30 },
    { 0x9c378339, SUC_lexicon_tags_2 },
    { 0x13378338, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b7833d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd757833f, SUC_lexicon_tags_28 },
    { 0xc0d78340, SUC_lexicon_tags_37 },
    { 0xf2178340, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14778343, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23178345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52d78347, SUC_lexicon_tags_48 },
    { 0x96178348, SUC_lexicon_tags_25 },
    { 0xec578349, SUC_lexicon_tags_33 },
    { 0x4357834a, SUC_lexicon_tags_17 },
    { 0x26f7834a, SUC_lexicon_tags_43 },
    { 0xb717834c, SUC_lexicon_tags_29 },
    { 0xccd7834d, SUC_lexicon_tags_11 },
    { 0xe0d7834c, SUC_lexicon_tags_2 },
    { 0x5cd7834d, SUC_lexicon_tags_11 },
    { 0xbfb7834e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77578356, SUC_lexicon_tags_1 },
    { 0x64978357, SUC_lexicon_tags_26 },
    { 0x6a578357, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d57835c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66778360, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b78365, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea17836a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f7836d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59778370, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a178373, SUC_lexicon_tags_21 },
    { 0xf2f78374, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7578379, SUC_lexicon_tags_25 },
    { 0xc657837a, SUC_lexicon_tags_21 },
    { 0xb777837b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xccb7837d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e578380, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3af78382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f78385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2037838b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1257838d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb78390, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed178395, SUC_lexicon_tags_1 },
    { 0x33f78396, SUC_lexicon_tags_49 },
    { 0x38178396, SUC_lexicon_tags_8 },
    { 0xecb78398, SUC_lexicon_tags_113 },
    { 0x46178396, SUC_lexicon_tags_14 },
    { 0x67b7839a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d97839f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2a7783a1, SUC_lexicon_tags_17 },
    { 0xcab783a1, SUC_lexicon_tags_0 },
    { 0x6a5783a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x543783a5, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1783a8, SUC_lexicon_tags_17 },
    { 0x65783a9, SUC_lexicon_tags_29 },
    { 0x5b7783aa, SUC_lexicon_tags_46 },
    { 0xfef783a8, SUC_lexicon_tags_7 },
    { 0xa93783ac, SUC_lexicon_tags_17 },
    { 0x8d7783ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705783b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd93783b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f783b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f783ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x281783bc, SUC_lexicon_tags_17 },
    { 0x459783bd, SUC_lexicon_tags_12 },
    { 0xbc5783bd, SUC_lexicon_tags_33 },
    { 0x4a7783bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd07783c1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9783c6, SUC_lexicon_tags_1 },
    { 0xe8b783c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7783cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1783d2, SUC_lexicon_tags_15 },
    { 0x8df783d2, SUC_lexicon_tags_1 },
    { 0x325783d4, SUC_lexicon_tags_19 },
    { 0xe97783d4, SUC_lexicon_tags_21 },
    { 0xcd9783d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1783e1, SUC_lexicon_tags_37 },
    { 0xc37783e1, SUC_lexicon_tags_12 },
    { 0xf83783e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ef783e5, SUC_lexicon_tags_25 },
    { 0x3e9783e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1783e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x785783eb, SUC_lexicon_tags_0 },
    { 0xe85783eb, SUC_lexicon_tags_0 },
    { 0xc05783ed, SUC_lexicon_tags_1 },
    { 0xed7783ee, SUC_lexicon_tags_38 },
    { 0x59d783ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5783f3, SUC_lexicon_tags_19 },
    { 0xafd783f3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823783f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a1783fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef783ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfd78401, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b978405, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3817840c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f778413, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af78417, SUC_lexicon_tags_25 },
    { 0xf7778418, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b7841e, SUC_lexicon_tags_19 },
    { 0x1517841f, SUC_lexicon_tags_39 },
    { 0xf8d78420, SUC_lexicon_tags_27 },
    { 0x8d78421, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x52578423, SUC_lexicon_tags_77 },
    { 0x10b78424, SUC_lexicon_tags_55 },
    { 0xba378424, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b78429, SUC_lexicon_tags_55 },
    { 0x4477842a, SUC_lexicon_tags_30 },
    { 0xcd778429, SUC_lexicon_tags_25 },
    { 0x54d7842a, SUC_lexicon_tags_31 },
    { 0x46f7842d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x51378430, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb78433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb78436, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2e978438, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f97843a, SUC_lexicon_tags_7 },
    { 0x1057843b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35178440, SUC_lexicon_tags_14 },
    { 0xe8d78441, SUC_lexicon_tags_25 },
    { 0xe8f78442, SUC_lexicon_tags_45 },
    { 0xbeb78443, SUC_lexicon_tags_39 },
    { 0x82778444, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f7844b, SUC_lexicon_tags_15 },
    { 0x3037844c, SUC_lexicon_tags_37 },
    { 0xb6d7844d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0578451, SUC_lexicon_tags_84 },
    { 0x6ed78452, SUC_lexicon_tags_63 },
    { 0x91378453, SUC_lexicon_tags_21 },
    { 0x10578454, SUC_lexicon_tags_2 },
    { 0x9cf78452, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4978459, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e77845c, SUC_lexicon_tags_37 },
    { 0xb97845d, SUC_lexicon_tags_42 },
    { 0xe277845e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68378460, SUC_lexicon_tags_63 },
    { 0xe5b78461, SUC_lexicon_tags_12 },
    { 0x72578460, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4b778464, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x61378467, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3178469, SUC_lexicon_tags_30 },
    { 0x1477846a, SUC_lexicon_tags_6 },
    { 0x9dd7846b, SUC_lexicon_tags_5 },
    { 0x4537846b, SUC_lexicon_tags_65 },
    { 0xb3f7846c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a178471, SUC_lexicon_tags_33 },
    { 0xa9978472, SUC_lexicon_tags_91 },
    { 0x49978472, SUC_lexicon_tags_4 },
    { 0x8dd78474, SUC_lexicon_tags_19 },
    { 0x78b78475, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xde378477, SUC_lexicon_tags_49 },
    { 0x8cd78477, SUC_lexicon_tags_63 },
    { 0x84778479, SUC_lexicon_tags_103 },
    { 0x2517847a, SUC_lexicon_tags_25 },
    { 0xab17847b, SUC_lexicon_tags_17 },
    { 0x5a57847a, SUC_lexicon_tags_33 },
    { 0xa2d7847b, SUC_lexicon_tags_48 },
    { 0x8b578477, SUC_lexicon_tags_1 },
    { 0x6e77847f, SUC_lexicon_tags_8 },
    { 0xf4978480, SUC_lexicon_tags_37 },
    { 0xf9178477, SUC_lexicon_tags_25 },
    { 0xf0978482, SUC_lexicon_tags_27 },
    { 0x17f78483, SUC_lexicon_tags_37 },
    { 0x8cb78483, SUC_lexicon_tags_37 },
    { 0xc9f78478, SUC_lexicon_tags_2 },
    { 0x92f7847a, SUC_lexicon_tags_31 },
    { 0x8ab78487, SUC_lexicon_tags_1 },
    { 0x60b78487, SUC_lexicon_tags_12 },
    { 0xea778489, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea17848d, SUC_lexicon_tags_12 },
    { 0xa057848d, SUC_lexicon_tags_37 },
    { 0xdbb7848d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4f78491, SUC_lexicon_tags_0 },
    { 0x97578492, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7978498, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f7849c, SUC_lexicon_tags_0 },
    { 0x8537849d, SUC_lexicon_tags_31 },
    { 0xf957849e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9784a2, SUC_lexicon_tags_30 },
    { 0x4c3784a3, SUC_lexicon_tags_37 },
    { 0xfbf784a4, SUC_lexicon_tags_9 },
    { 0x843784a2, SUC_lexicon_tags_19 },
    { 0x489784a6, SUC_lexicon_tags_12 },
    { 0x5a1784a7, SUC_lexicon_tags_25 },
    { 0x711784a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd0b784aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6fd784ac, SUC_lexicon_tags_60 },
    { 0xa53784ad, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d784b1, SUC_lexicon_tags_82 },
    { 0xadd784b1, SUC_lexicon_tags_25 },
    { 0x3fd784b3, SUC_lexicon_tags_26 },
    { 0xb95784b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x835784b6, SUC_lexicon_tags_68 },
    { 0xccd784b7, SUC_lexicon_tags_31 },
    { 0x9b9784b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcf3784ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9784be, SUC_lexicon_tags_7 },
    { 0x4f1784bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15784c5, SUC_lexicon_tags_17 },
    { 0xb11784c5, SUC_lexicon_tags_15 },
    { 0xedb784c5, SUC_lexicon_tags_1 },
    { 0x46f784c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83b784ca, SUC_lexicon_tags_37 },
    { 0xc3f784cb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d784d4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d784d8, SUC_lexicon_tags_36 },
    { 0x561784d9, SUC_lexicon_tags_15 },
    { 0x2f5784d9, SUC_lexicon_tags_8 },
    { 0x55f784db, SUC_lexicon_tags_23 },
    { 0x51b784db, SUC_lexicon_tags_47 },
    { 0x9d5784dd, SUC_lexicon_tags_17 },
    { 0x145784dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431784e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773784ea, SUC_lexicon_tags_38 },
    { 0x7c3784eb, SUC_lexicon_tags_23 },
    { 0x60f784eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63784f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d784f9, SUC_lexicon_tags_30 },
    { 0xb8b784fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f784fd, SUC_lexicon_tags_238 },
    { 0x66b784fe, SUC_lexicon_tags_46 },
    { 0x94b784ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0b78501, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd78505, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b78509, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b78514, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa57851a, SUC_lexicon_tags_37 },
    { 0x8eb7851a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d7851f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f578521, SUC_lexicon_tags_29 },
    { 0x25f78521, SUC_lexicon_tags_2 },
    { 0xa8d78523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52378527, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xedd78529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f7852c, SUC_lexicon_tags_10 },
    { 0x5557852c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xacb7852f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b78532, SUC_lexicon_tags_26 },
    { 0x5d778533, SUC_lexicon_tags_8 },
    { 0xe2378533, SUC_lexicon_tags_62 },
    { 0x4578534, SUC_lexicon_tags_66 },
    { 0xd7978536, SUC_lexicon_tags_37 },
    { 0x6cf78537, SUC_lexicon_tags_8 },
    { 0xf8778538, SUC_lexicon_tags_17 },
    { 0xd178533, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b7853f, SUC_lexicon_tags_37 },
    { 0xd1f78540, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x33b78542, SUC_lexicon_tags_60 },
    { 0x50778543, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x19178545, SUC_lexicon_tags_23 },
    { 0x83178546, SUC_lexicon_tags_65 },
    { 0x73778546, SUC_lexicon_tags_7 },
    { 0x7df78545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8637854e, SUC_lexicon_tags_46 },
    { 0x7c77854f, SUC_lexicon_tags_23 },
    { 0x65f7854e, SUC_lexicon_tags_17 },
    { 0xb557854f, SUC_lexicon_tags_1 },
    { 0xe3778552, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d78555, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad78557, SUC_lexicon_tags_27 },
    { 0x3978558, SUC_lexicon_tags_11 },
    { 0x25778559, SUC_lexicon_tags_25 },
    { 0x68f7855a, SUC_lexicon_tags_33 },
    { 0x64b7855b, SUC_lexicon_tags_28 },
    { 0x3717855b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b57855f, SUC_lexicon_tags_49 },
    { 0x3777855f, SUC_lexicon_tags_27 },
    { 0xe5b78561, SUC_lexicon_tags_39 },
    { 0x74978562, SUC_lexicon_tags_37 },
    { 0x28b78563, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa4978565, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x42d78567, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bb78569, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed7856c, SUC_lexicon_tags_47 },
    { 0x1157856d, SUC_lexicon_tags_8 },
    { 0x5ed7856e, SUC_lexicon_tags_11 },
    { 0x9e37856c, SUC_lexicon_tags_38 },
    { 0x77b78570, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe9978572, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b78575, SUC_lexicon_tags_37 },
    { 0x68b78576, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2978579, SUC_lexicon_tags_16 },
    { 0xaa97857a, SUC_lexicon_tags_1 },
    { 0xf877857a, SUC_lexicon_tags_34 },
    { 0x60f7857a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96178584, SUC_lexicon_tags_10 },
    { 0x23778585, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317858a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x21b7858c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda178590, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11778594, SUC_lexicon_tags_38 },
    { 0x35378594, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8b78597, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfff78599, SUC_lexicon_tags_9 },
    { 0x237859a, SUC_lexicon_tags_45 },
    { 0xcbf78599, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc97859e, SUC_lexicon_tags_15 },
    { 0xa157859f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x407785a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x819785a3, SUC_lexicon_tags_30 },
    { 0x16f785a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a7785a6, SUC_lexicon_tags_7 },
    { 0x299785a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xced785aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe33785ac, SUC_lexicon_tags_29 },
    { 0xd03785ad, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527785b2, SUC_lexicon_tags_97 },
    { 0x839785b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39785b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1f785b9, SUC_lexicon_tags_12 },
    { 0x81d785ba, SUC_lexicon_tags_87 },
    { 0x3e3785b9, SUC_lexicon_tags_30 },
    { 0xc11785b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b785c1, SUC_lexicon_tags_505 },
    { 0xe05785c1, SUC_lexicon_tags_7 },
    { 0x18b785c1, SUC_lexicon_tags_37 },
    { 0xd5d785c4, SUC_lexicon_tags_47 },
    { 0xdf7785c5, SUC_lexicon_tags_14 },
    { 0x32d785c3, SUC_lexicon_tags_8 },
    { 0xfad785c7, SUC_lexicon_tags_17 },
    { 0x905785c4, SUC_lexicon_tags_31 },
    { 0xc45785c9, SUC_lexicon_tags_12 },
    { 0xbb3785ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfd785cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc27785ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb4b785d0, SUC_lexicon_tags_17 },
    { 0xaef785d1, SUC_lexicon_tags_0 },
    { 0xb33785d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeb9785d4, SUC_lexicon_tags_17 },
    { 0xbe7785d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x691785d7, SUC_lexicon_tags_37 },
    { 0xe51785d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217785dc, SUC_lexicon_tags_14 },
    { 0xe8f785dc, SUC_lexicon_tags_207 },
    { 0x8e1785de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43785e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x875785e4, SUC_lexicon_tags_29 },
    { 0xb79785e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9785e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x323785eb, SUC_lexicon_tags_34 },
    { 0x97785ec, SUC_lexicon_tags_27 },
    { 0x1f785ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f785f0, SUC_lexicon_tags_26 },
    { 0x645785f1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x849785f3, SUC_lexicon_tags_23 },
    { 0x659785f3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdab785f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f785fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3578602, SUC_lexicon_tags_29 },
    { 0x79f78603, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c778606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12978608, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf317860d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1bb7860f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4e178611, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d78614, SUC_lexicon_tags_37 },
    { 0x22d78615, SUC_lexicon_tags_11 },
    { 0xe7f78615, SUC_lexicon_tags_1 },
    { 0x67178617, SUC_lexicon_tags_34 },
    { 0xcdd78616, SUC_lexicon_tags_20 },
    { 0x84378617, SUC_lexicon_tags_230 },
    { 0x84978619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf37861f, SUC_lexicon_tags_16 },
    { 0xbeb78620, SUC_lexicon_tags_14 },
    { 0x84378621, SUC_lexicon_tags_19 },
    { 0xa4378622, SUC_lexicon_tags_37 },
    { 0x49778623, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88d78625, SUC_lexicon_tags_6 },
    { 0xb8b78626, SUC_lexicon_tags_37 },
    { 0x9f178627, SUC_lexicon_tags_49 },
    { 0x86978628, SUC_lexicon_tags_1 },
    { 0xefd78629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe717862c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c37862f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f78632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44178634, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x4178636, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f78639, SUC_lexicon_tags_3 },
    { 0x38378639, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38978640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d78645, SUC_lexicon_tags_46 },
    { 0xfd978645, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72378649, SUC_lexicon_tags_17 },
    { 0xc4b7864a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1137864d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x46578650, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19b78652, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6778654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64b78656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bd78658, SUC_lexicon_tags_37 },
    { 0x4cf78659, SUC_lexicon_tags_1 },
    { 0x99578658, SUC_lexicon_tags_84 },
    { 0xf817865b, SUC_lexicon_tags_46 },
    { 0xfa77865b, SUC_lexicon_tags_7 },
    { 0xa557865d, SUC_lexicon_tags_82 },
    { 0xc1b7865a, SUC_lexicon_tags_38 },
    { 0x2b57865f, SUC_lexicon_tags_45 },
    { 0x2c578660, SUC_lexicon_tags_30 },
    { 0xbd378660, SUC_lexicon_tags_11 },
    { 0xe8178662, SUC_lexicon_tags_11 },
    { 0x75d78662, SUC_lexicon_tags_17 },
    { 0x7b778664, SUC_lexicon_tags_38 },
    { 0xc1f78665, SUC_lexicon_tags_33 },
    { 0xccb78666, SUC_lexicon_tags_124 },
    { 0x9ed78667, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2337866b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef7866f, SUC_lexicon_tags_25 },
    { 0x9e378670, SUC_lexicon_tags_55 },
    { 0x5d578671, SUC_lexicon_tags_37 },
    { 0xeeb7866f, SUC_lexicon_tags_14 },
    { 0xf6778672, SUC_lexicon_tags_8 },
    { 0xc2d78674, SUC_lexicon_tags_31 },
    { 0xb3f78675, SUC_lexicon_tags_37 },
    { 0xf3978670, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37867a, SUC_lexicon_tags_15 },
    { 0x1a37867a, SUC_lexicon_tags_48 },
    { 0x7957867a, SUC_lexicon_tags_84 },
    { 0xfeb7867d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab78683, SUC_lexicon_tags_995 },
    { 0, NULL },
    { 0xe0f78685, SUC_lexicon_tags_29 },
    { 0x12b78685, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba57868d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0378692, SUC_lexicon_tags_33 },
    { 0x38d78693, SUC_lexicon_tags_25 },
    { 0x91578692, SUC_lexicon_tags_7 },
    { 0x1e178695, SUC_lexicon_tags_7 },
    { 0xecf78696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cd78698, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af7869d, SUC_lexicon_tags_14 },
    { 0xb257869e, SUC_lexicon_tags_25 },
    { 0xdad7869e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbbf786a1, SUC_lexicon_tags_65 },
    { 0x313786a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb57786a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7f1786a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb1d786a8, SUC_lexicon_tags_15 },
    { 0xc6f786a9, SUC_lexicon_tags_82 },
    { 0xc89786a8, SUC_lexicon_tags_7 },
    { 0x69786ab, SUC_lexicon_tags_26 },
    { 0xa85786ac, SUC_lexicon_tags_28 },
    { 0xd3d786ad, SUC_lexicon_tags_30 },
    { 0xd2f786a8, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7786ba, SUC_lexicon_tags_21 },
    { 0x983786ba, SUC_lexicon_tags_47 },
    { 0x53b786ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff7786be, SUC_lexicon_tags_25 },
    { 0xdd786be, SUC_lexicon_tags_61 },
    { 0xea5786be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275786c8, SUC_lexicon_tags_27 },
    { 0x4c3786c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213786cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89786d2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1786d6, SUC_lexicon_tags_11 },
    { 0x1bd786d7, SUC_lexicon_tags_11 },
    { 0xe9f786d7, SUC_lexicon_tags_1 },
    { 0x741786d9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49786dd, SUC_lexicon_tags_30 },
    { 0x953786de, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3786e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45786e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5786ea, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x255786ed, SUC_lexicon_tags_19 },
    { 0x1b5786ee, SUC_lexicon_tags_30 },
    { 0xb85786ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x423786f2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3786fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72578705, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4178708, SUC_lexicon_tags_2 },
    { 0xb8978709, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b7870d, SUC_lexicon_tags_12 },
    { 0xc677870d, SUC_lexicon_tags_19 },
    { 0x5377870f, SUC_lexicon_tags_13 },
    { 0x2377870f, SUC_lexicon_tags_1 },
    { 0xd7b7870f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7871f, SUC_lexicon_tags_37 },
    { 0xd97871f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d778725, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80f78727, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8f78729, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xfa17872b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d7872e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x20578731, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd7778733, SUC_lexicon_tags_7 },
    { 0xf3778734, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb0d78736, SUC_lexicon_tags_11 },
    { 0x6eb78737, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc757873a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7dd7873c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e97873e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d978744, SUC_lexicon_tags_65 },
    { 0xf5f78745, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4e778747, SUC_lexicon_tags_38 },
    { 0x1db78747, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac97874d, SUC_lexicon_tags_30 },
    { 0x4717874e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab78758, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab7875d, SUC_lexicon_tags_25 },
    { 0x7e37875d, SUC_lexicon_tags_11 },
    { 0x5ab7875f, SUC_lexicon_tags_65 },
    { 0x1b17875f, SUC_lexicon_tags_95 },
    { 0x56f78761, SUC_lexicon_tags_16 },
    { 0xb5578761, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef78770, SUC_lexicon_tags_0 },
    { 0xfb778770, SUC_lexicon_tags_7 },
    { 0x5bb78772, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e978775, SUC_lexicon_tags_11 },
    { 0x2778776, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b578778, SUC_lexicon_tags_1 },
    { 0xff78779, SUC_lexicon_tags_13 },
    { 0xcb378778, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b7877e, SUC_lexicon_tags_0 },
    { 0xd4f7877f, SUC_lexicon_tags_37 },
    { 0xdab7877e, SUC_lexicon_tags_66 },
    { 0x7c97877f, SUC_lexicon_tags_31 },
    { 0x4978781, SUC_lexicon_tags_71 },
    { 0x26d78783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bb78785, SUC_lexicon_tags_33 },
    { 0x49978786, SUC_lexicon_tags_141 },
    { 0xcb78785, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x85d78789, SUC_lexicon_tags_77 },
    { 0x36778789, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e37878c, SUC_lexicon_tags_27 },
    { 0x7eb7878d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf157878f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1578795, SUC_lexicon_tags_37 },
    { 0xe8978796, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94778798, SUC_lexicon_tags_17 },
    { 0xdb178798, SUC_lexicon_tags_45 },
    { 0xca97879a, SUC_lexicon_tags_48 },
    { 0x99f7879a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb437879f, SUC_lexicon_tags_14 },
    { 0x49d7879f, SUC_lexicon_tags_19 },
    { 0x4097879f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319787a7, SUC_lexicon_tags_15 },
    { 0xfff787a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x353787aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95787ae, SUC_lexicon_tags_25 },
    { 0xab1787ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57787b2, SUC_lexicon_tags_17 },
    { 0x221787b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x677787b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x703787b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd787bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf787c2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9d1787c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df787c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb97787c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1787cf, SUC_lexicon_tags_39 },
    { 0xf01787d0, SUC_lexicon_tags_410 },
    { 0x15b787d1, SUC_lexicon_tags_13 },
    { 0xbed787d2, SUC_lexicon_tags_0 },
    { 0xea1787d3, SUC_lexicon_tags_25 },
    { 0x21d787d4, SUC_lexicon_tags_15 },
    { 0xc57787d5, SUC_lexicon_tags_8 },
    { 0x649787d1, SUC_lexicon_tags_9 },
    { 0x79f787d1, SUC_lexicon_tags_27 },
    { 0xd47787d5, SUC_lexicon_tags_9 },
    { 0x837787d9, SUC_lexicon_tags_38 },
    { 0xb89787da, SUC_lexicon_tags_12 },
    { 0x543787cf, SUC_lexicon_tags_25 },
    { 0x465787d8, SUC_lexicon_tags_1 },
    { 0xdf9787dd, SUC_lexicon_tags_0 },
    { 0x763787cf, SUC_lexicon_tags_12 },
    { 0x7fd787dd, SUC_lexicon_tags_25 },
    { 0xf1b787e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe85787e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6b787e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d787e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc9787e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf787ec, SUC_lexicon_tags_13 },
    { 0xee9787ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45787f1, SUC_lexicon_tags_17 },
    { 0xb1b787f1, SUC_lexicon_tags_47 },
    { 0x213787f1, SUC_lexicon_tags_0 },
    { 0x4f7787f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79787f7, SUC_lexicon_tags_113 },
    { 0x8b9787f8, SUC_lexicon_tags_14 },
    { 0x3d7787f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee9787fb, SUC_lexicon_tags_17 },
    { 0x8cb787fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa49787fe, SUC_lexicon_tags_1 },
    { 0x5d5787ff, SUC_lexicon_tags_4 },
    { 0x1378800, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22b78802, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f78806, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf78809, SUC_lexicon_tags_43 },
    { 0xdd7880a, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5417880f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9978813, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f7881b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83f7881d, SUC_lexicon_tags_172 },
    { 0x95d7881d, SUC_lexicon_tags_0 },
    { 0x6e57881d, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3178824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49d78826, SUC_lexicon_tags_11 },
    { 0x3e778827, SUC_lexicon_tags_15 },
    { 0x90578828, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7837882a, SUC_lexicon_tags_1 },
    { 0xfe77882a, SUC_lexicon_tags_25 },
    { 0x77d7882a, SUC_lexicon_tags_17 },
    { 0xf677882a, SUC_lexicon_tags_1 },
    { 0x1957882e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d178832, SUC_lexicon_tags_45 },
    { 0x69578833, SUC_lexicon_tags_2 },
    { 0xc8978834, SUC_lexicon_tags_26 },
    { 0x1a578832, SUC_lexicon_tags_60 },
    { 0xbfd78834, SUC_lexicon_tags_17 },
    { 0x69b78835, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f97883a, SUC_lexicon_tags_11 },
    { 0x95f7883b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9777883f, SUC_lexicon_tags_32 },
    { 0x91178840, SUC_lexicon_tags_17 },
    { 0x71778841, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x21b78843, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b78847, SUC_lexicon_tags_43 },
    { 0xcad78848, SUC_lexicon_tags_4 },
    { 0x71f78849, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb7884d, SUC_lexicon_tags_85 },
    { 0x317884e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf0f78850, SUC_lexicon_tags_11 },
    { 0xf1778851, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f78855, SUC_lexicon_tags_15 },
    { 0x72578856, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x7f178858, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5b7885a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb7885d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d78864, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83578869, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e77886b, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0978872, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49578876, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7757887a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca57887f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8178881, SUC_lexicon_tags_0 },
    { 0x88f78881, SUC_lexicon_tags_28 },
    { 0xdf578883, SUC_lexicon_tags_38 },
    { 0x8bf78883, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1a178886, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ed78888, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f7888b, SUC_lexicon_tags_37 },
    { 0xb857888b, SUC_lexicon_tags_10 },
    { 0xb337888d, SUC_lexicon_tags_6 },
    { 0x4517888e, SUC_lexicon_tags_37 },
    { 0x1497888d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b578894, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8578899, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f7889e, SUC_lexicon_tags_33 },
    { 0x54d7889f, SUC_lexicon_tags_37 },
    { 0x649788a0, SUC_lexicon_tags_0 },
    { 0x7bd788a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x491788a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x247788a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9788aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1788af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d788b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe71788b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff788b7, SUC_lexicon_tags_37 },
    { 0x59788b8, SUC_lexicon_tags_48 },
    { 0xf25788b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1f788bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887788c3, SUC_lexicon_tags_27 },
    { 0x4db788c3, SUC_lexicon_tags_38 },
    { 0x7c7788c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b788c8, SUC_lexicon_tags_37 },
    { 0x7cb788c9, SUC_lexicon_tags_82 },
    { 0xc3788ca, SUC_lexicon_tags_27 },
    { 0x997788cb, SUC_lexicon_tags_25 },
    { 0x9a7788cb, SUC_lexicon_tags_23 },
    { 0x425788cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x74d788cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f788d2, SUC_lexicon_tags_39 },
    { 0x7e3788d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d788d7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f788da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d1788dc, SUC_lexicon_tags_8 },
    { 0x501788dd, SUC_lexicon_tags_30 },
    { 0x315788de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf788e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7788e5, SUC_lexicon_tags_1 },
    { 0x89d788e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f788ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4b7788ec, SUC_lexicon_tags_15 },
    { 0x995788ed, SUC_lexicon_tags_7 },
    { 0x1d5788ee, SUC_lexicon_tags_1 },
    { 0x3f9788ee, SUC_lexicon_tags_60 },
    { 0x865788ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1788f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3788fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b788fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa89788ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26578901, SUC_lexicon_tags_55 },
    { 0x26f78902, SUC_lexicon_tags_15 },
    { 0x94578903, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9978908, SUC_lexicon_tags_70 },
    { 0xacd78908, SUC_lexicon_tags_11 },
    { 0xa217890a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf157890f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xad378911, SUC_lexicon_tags_11 },
    { 0x47778912, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8a978914, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43378916, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x58b78919, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff78920, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca378925, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f78928, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd97892a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe277892c, SUC_lexicon_tags_23 },
    { 0x3a37892d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6cd7892f, SUC_lexicon_tags_2 },
    { 0x5b78930, SUC_lexicon_tags_91 },
    { 0x7c578931, SUC_lexicon_tags_19 },
    { 0xd0778932, SUC_lexicon_tags_43 },
    { 0x2a378932, SUC_lexicon_tags_21 },
    { 0x14f78934, SUC_lexicon_tags_1 },
    { 0x3e378935, SUC_lexicon_tags_43 },
    { 0x1af78936, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6378939, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6377893b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf17893e, SUC_lexicon_tags_1 },
    { 0xdf37893e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c578941, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7578946, SUC_lexicon_tags_11 },
    { 0x65378947, SUC_lexicon_tags_8 },
    { 0x22b78948, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b78950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63378952, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74178954, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4378958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa317895a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9817895c, SUC_lexicon_tags_25 },
    { 0xdad7895c, SUC_lexicon_tags_29 },
    { 0x6417895e, SUC_lexicon_tags_25 },
    { 0x97895f, SUC_lexicon_tags_21 },
    { 0x3317895e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d578962, SUC_lexicon_tags_12 },
    { 0xddf78962, SUC_lexicon_tags_12 },
    { 0x81178964, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x38778967, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af7896d, SUC_lexicon_tags_17 },
    { 0x57d7896e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x61778970, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd78974, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2677897b, SUC_lexicon_tags_64 },
    { 0x61b7897c, SUC_lexicon_tags_19 },
    { 0x9597897d, SUC_lexicon_tags_220 },
    { 0xfaf7897d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d78986, SUC_lexicon_tags_16 },
    { 0xe0d78986, SUC_lexicon_tags_49 },
    { 0x21978987, SUC_lexicon_tags_15 },
    { 0x1c778987, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff57898d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f78992, SUC_lexicon_tags_0 },
    { 0xc8f78993, SUC_lexicon_tags_65 },
    { 0x17578993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f78998, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xec17899b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x277899e, SUC_lexicon_tags_12 },
    { 0xfd97899e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x889789a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x419789a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61789a6, SUC_lexicon_tags_2 },
    { 0x49b789a7, SUC_lexicon_tags_56 },
    { 0x563789a6, SUC_lexicon_tags_17 },
    { 0x7f789a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb11789ab, SUC_lexicon_tags_55 },
    { 0xe5d789ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511789b0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5789b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d789bf, SUC_lexicon_tags_63 },
    { 0xb6d789c0, SUC_lexicon_tags_15 },
    { 0xa43789c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb789c4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8ed789c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9789c9, SUC_lexicon_tags_16 },
    { 0xc9f789c9, SUC_lexicon_tags_27 },
    { 0x87789ca, SUC_lexicon_tags_25 },
    { 0x6f1789cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39789cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb11789d1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87789d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded789da, SUC_lexicon_tags_68 },
    { 0x555789db, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7789e1, SUC_lexicon_tags_56 },
    { 0x877789e2, SUC_lexicon_tags_17 },
    { 0xa1f789e2, SUC_lexicon_tags_14 },
    { 0x27d789e4, SUC_lexicon_tags_15 },
    { 0x511789e5, SUC_lexicon_tags_26 },
    { 0xcd9789e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121789ea, SUC_lexicon_tags_7 },
    { 0xb1789ea, SUC_lexicon_tags_2 },
    { 0x85789ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x58b789ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x107789f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb789f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39789fd, SUC_lexicon_tags_30 },
    { 0x47b789fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df78a02, SUC_lexicon_tags_33 },
    { 0x19378a02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b78a05, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f78a09, SUC_lexicon_tags_30 },
    { 0x8f978a0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd8f78a0c, SUC_lexicon_tags_39 },
    { 0x38978a0c, SUC_lexicon_tags_14 },
    { 0x45978a0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe2578a10, SUC_lexicon_tags_37 },
    { 0x4e778a10, SUC_lexicon_tags_15 },
    { 0x8a978a12, SUC_lexicon_tags_47 },
    { 0x97d78a13, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd78a18, SUC_lexicon_tags_6 },
    { 0xd6f78a18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x61378a1b, SUC_lexicon_tags_1 },
    { 0x78778a1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb78a21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced78a28, SUC_lexicon_tags_65 },
    { 0x11d78a29, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x29778a2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f78a2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab78a31, SUC_lexicon_tags_8 },
    { 0x9cb78a31, SUC_lexicon_tags_47 },
    { 0xe4178a33, SUC_lexicon_tags_61 },
    { 0x57578a33, SUC_lexicon_tags_7 },
    { 0xb1778a33, SUC_lexicon_tags_17 },
    { 0x3ad78a36, SUC_lexicon_tags_29 },
    { 0xb978a37, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xeed78a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d78a3f, SUC_lexicon_tags_38 },
    { 0x7d578a40, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc8578a42, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9cd78a44, SUC_lexicon_tags_4 },
    { 0xcc178a45, SUC_lexicon_tags_32 },
    { 0x7e378a44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x90178a49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1778a4b, SUC_lexicon_tags_11 },
    { 0x69f78a4b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce778a59, SUC_lexicon_tags_47 },
    { 0xf7778a59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d78a5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f78a60, SUC_lexicon_tags_8 },
    { 0x978a61, SUC_lexicon_tags_37 },
    { 0x6b778a61, SUC_lexicon_tags_47 },
    { 0x3a978a63, SUC_lexicon_tags_31 },
    { 0x978a64, SUC_lexicon_tags_1 },
    { 0x1f778a64, SUC_lexicon_tags_11 },
    { 0xdbd78a64, SUC_lexicon_tags_1 },
    { 0x85b78a65, SUC_lexicon_tags_60 },
    { 0x67778a63, SUC_lexicon_tags_14 },
    { 0x13d78a66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f978a6b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x55f78a6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e578a6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb78a72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5778a7b, SUC_lexicon_tags_30 },
    { 0x4e578a7c, SUC_lexicon_tags_25 },
    { 0xc0b78a7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e578a83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b78a86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcf78a88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d378a8a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdd778a8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a378a8e, SUC_lexicon_tags_11 },
    { 0xea578a8e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb78a98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccd78a9a, SUC_lexicon_tags_11 },
    { 0xf0d78a9a, SUC_lexicon_tags_23 },
    { 0x8578a9b, SUC_lexicon_tags_12 },
    { 0x1778a9d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf78aa0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d78aa6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c178aa8, SUC_lexicon_tags_14 },
    { 0xef778aa9, SUC_lexicon_tags_1 },
    { 0xb7778aaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1f78aac, SUC_lexicon_tags_17 },
    { 0xfc778aac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74b78aaf, SUC_lexicon_tags_12 },
    { 0x93978ab0, SUC_lexicon_tags_15 },
    { 0xe4578ab1, SUC_lexicon_tags_28 },
    { 0xec178ab2, SUC_lexicon_tags_2 },
    { 0x9f378ab3, SUC_lexicon_tags_64 },
    { 0xee178ab4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d778ab6, SUC_lexicon_tags_0 },
    { 0xd5178ab7, SUC_lexicon_tags_37 },
    { 0xe6b78ab8, SUC_lexicon_tags_28 },
    { 0x4d778ab6, SUC_lexicon_tags_48 },
    { 0x57378ab6, SUC_lexicon_tags_1 },
    { 0xa1f78ab9, SUC_lexicon_tags_60 },
    { 0x6f378abc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd78abf, SUC_lexicon_tags_17 },
    { 0x27378ac0, SUC_lexicon_tags_21 },
    { 0x54f78abf, SUC_lexicon_tags_37 },
    { 0xedf78ac2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff78aca, SUC_lexicon_tags_43 },
    { 0xa9378acb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf978ace, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f378ad0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf178ad7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b178ada, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d978adc, SUC_lexicon_tags_7 },
    { 0xb2978add, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f78ae2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb78ae5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x34978ae8, SUC_lexicon_tags_45 },
    { 0xf2178ae9, SUC_lexicon_tags_22 },
    { 0x6cf78aea, SUC_lexicon_tags_53 },
    { 0x1a378ae8, SUC_lexicon_tags_60 },
    { 0xebb78ae8, SUC_lexicon_tags_8 },
    { 0x2ff78ae8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3178aef, SUC_lexicon_tags_45 },
    { 0x62b78af0, SUC_lexicon_tags_17 },
    { 0x5ab78af1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61178af3, SUC_lexicon_tags_37 },
    { 0xe3f78af3, SUC_lexicon_tags_11 },
    { 0x5d778af3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f78afc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf978afe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b178b02, SUC_lexicon_tags_21 },
    { 0xa6f78b03, SUC_lexicon_tags_12 },
    { 0x7d178b04, SUC_lexicon_tags_47 },
    { 0x69978b04, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f78b0b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b78b10, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db78b16, SUC_lexicon_tags_12 },
    { 0xccb78b17, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e378b1b, SUC_lexicon_tags_25 },
    { 0xd0978b1b, SUC_lexicon_tags_33 },
    { 0xd1178b1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe578b1f, SUC_lexicon_tags_33 },
    { 0xd1178b20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8978b23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb78b26, SUC_lexicon_tags_0 },
    { 0x5f578b27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd78b2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bb78b31, SUC_lexicon_tags_45 },
    { 0xb8578b31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82378b36, SUC_lexicon_tags_12 },
    { 0x8eb78b36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36778b3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4178b42, SUC_lexicon_tags_14 },
    { 0xb7578b43, SUC_lexicon_tags_3 },
    { 0xac178b43, SUC_lexicon_tags_43 },
    { 0xfa378b45, SUC_lexicon_tags_63 },
    { 0x78578b46, SUC_lexicon_tags_25 },
    { 0xbd578b47, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x578b49, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x78578b4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27378b4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af78b51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3f78b53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd78b58, SUC_lexicon_tags_37 },
    { 0x9b378b59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d78b5c, SUC_lexicon_tags_82 },
    { 0x78178b5c, SUC_lexicon_tags_8 },
    { 0xa7978b5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1b78b60, SUC_lexicon_tags_7 },
    { 0x38b78b61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd78b64, SUC_lexicon_tags_45 },
    { 0x37d78b65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28778b67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7f78b69, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf778b6c, SUC_lexicon_tags_7 },
    { 0xb2778b6d, SUC_lexicon_tags_15 },
    { 0x51d78b6d, SUC_lexicon_tags_11 },
    { 0x1f978b6f, SUC_lexicon_tags_66 },
    { 0xf0b78b6f, SUC_lexicon_tags_15 },
    { 0xbd178b6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd778b73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44178b84, SUC_lexicon_tags_12 },
    { 0x42978b84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x31378b87, SUC_lexicon_tags_19 },
    { 0x5b978b88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb78b8c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5178b8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x81578b90, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd78b96, SUC_lexicon_tags_17 },
    { 0xcb778b96, SUC_lexicon_tags_37 },
    { 0xe0d78b98, SUC_lexicon_tags_25 },
    { 0x81178b98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a378b9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb78ba3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b578ba6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3a378ba8, SUC_lexicon_tags_7 },
    { 0x59578ba8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe1d78bab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x36978bad, SUC_lexicon_tags_25 },
    { 0xd5978bae, SUC_lexicon_tags_15 },
    { 0x50978baf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb78bb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0978bb4, SUC_lexicon_tags_8 },
    { 0x1778bb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda978bb7, SUC_lexicon_tags_24 },
    { 0xb578bb8, SUC_lexicon_tags_11 },
    { 0x5f578bb9, SUC_lexicon_tags_3 },
    { 0x7b978bb7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6778bbd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x17778bc0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d978bcd, SUC_lexicon_tags_17 },
    { 0x17b78bce, SUC_lexicon_tags_31 },
    { 0x4e178bcf, SUC_lexicon_tags_106 },
    { 0x9b78bcd, SUC_lexicon_tags_14 },
    { 0xc7578bce, SUC_lexicon_tags_8 },
    { 0x1178bd0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x43778bd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82578bd9, SUC_lexicon_tags_7 },
    { 0x59b78bda, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d378bdf, SUC_lexicon_tags_1 },
    { 0xea778bdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86d78be2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f78be5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f78be8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f978bea, SUC_lexicon_tags_47 },
    { 0xf0d78bea, SUC_lexicon_tags_0 },
    { 0xf8f78bec, SUC_lexicon_tags_1 },
    { 0xdd378bed, SUC_lexicon_tags_35 },
    { 0x75978bee, SUC_lexicon_tags_15 },
    { 0xbd178bef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef78bf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a778bf8, SUC_lexicon_tags_14 },
    { 0x3e978bf8, SUC_lexicon_tags_4 },
    { 0xe5578bfa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1578bfd, SUC_lexicon_tags_39 },
    { 0x34778bfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6378c07, SUC_lexicon_tags_29 },
    { 0xaa778c08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4378c0b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc3d78c0d, SUC_lexicon_tags_33 },
    { 0x6ab78c0e, SUC_lexicon_tags_43 },
    { 0xbd578c0f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54f78c11, SUC_lexicon_tags_7 },
    { 0xd5d78c11, SUC_lexicon_tags_15 },
    { 0x7f78c13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37b78c15, SUC_lexicon_tags_66 },
    { 0x21578c16, SUC_lexicon_tags_12 },
    { 0x6f778c17, SUC_lexicon_tags_25 },
    { 0x54d78c17, SUC_lexicon_tags_6 },
    { 0x7b578c19, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xed978c1c, SUC_lexicon_tags_8 },
    { 0xce378c1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe378c1f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x25978c22, SUC_lexicon_tags_11 },
    { 0xf0d78c23, SUC_lexicon_tags_9 },
    { 0x9c378c24, SUC_lexicon_tags_38 },
    { 0xc8b78c25, SUC_lexicon_tags_19 },
    { 0x22178c26, SUC_lexicon_tags_47 },
    { 0x3f778c27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb178c2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b978c31, SUC_lexicon_tags_35 },
    { 0x1bf78c31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xae378c35, SUC_lexicon_tags_45 },
    { 0xd3778c35, SUC_lexicon_tags_11 },
    { 0x8d378c37, SUC_lexicon_tags_37 },
    { 0xd5978c38, SUC_lexicon_tags_38 },
    { 0x92778c39, SUC_lexicon_tags_12 },
    { 0x78778c3a, SUC_lexicon_tags_25 },
    { 0x5ad78c39, SUC_lexicon_tags_55 },
    { 0xc0f78c3c, SUC_lexicon_tags_16 },
    { 0xa2378c3d, SUC_lexicon_tags_43 },
    { 0x54d78c35, SUC_lexicon_tags_15 },
    { 0x3d178c3e, SUC_lexicon_tags_37 },
    { 0x2f78c40, SUC_lexicon_tags_17 },
    { 0xc8578c41, SUC_lexicon_tags_37 },
    { 0xb2178c3d, SUC_lexicon_tags_15 },
    { 0x9eb78c43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5cf78c45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2778c49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2178c4b, SUC_lexicon_tags_7 },
    { 0x57778c4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75178c5b, SUC_lexicon_tags_11 },
    { 0x9e178c5c, SUC_lexicon_tags_31 },
    { 0x85978c5b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3378c61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf378c64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0d78c66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff78c6b, SUC_lexicon_tags_7 },
    { 0x81f78c6c, SUC_lexicon_tags_14 },
    { 0x55378c6d, SUC_lexicon_tags_12 },
    { 0x50378c6c, SUC_lexicon_tags_11 },
    { 0x30578c6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e778c74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b78c7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb78c7f, SUC_lexicon_tags_0 },
    { 0xecd78c7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d578c82, SUC_lexicon_tags_38 },
    { 0x35f78c83, SUC_lexicon_tags_0 },
    { 0xb4578c84, SUC_lexicon_tags_2 },
    { 0x61778c85, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x97178c87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8b78c89, SUC_lexicon_tags_63 },
    { 0xb5d78c8a, SUC_lexicon_tags_17 },
    { 0xbad78c8b, SUC_lexicon_tags_7 },
    { 0xf4178c8c, SUC_lexicon_tags_26 },
    { 0x89f78c8c, SUC_lexicon_tags_11 },
    { 0xfdd78c8d, SUC_lexicon_tags_17 },
    { 0xeb578c8e, SUC_lexicon_tags_12 },
    { 0x5ad78c90, SUC_lexicon_tags_16 },
    { 0x21b78c8e, SUC_lexicon_tags_23 },
    { 0xa0d78c92, SUC_lexicon_tags_7 },
    { 0x3bb78c8f, SUC_lexicon_tags_15 },
    { 0xf2b78c94, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d78c97, SUC_lexicon_tags_26 },
    { 0x66778c98, SUC_lexicon_tags_15 },
    { 0xa2978c99, SUC_lexicon_tags_11 },
    { 0xc3b78c9a, SUC_lexicon_tags_26 },
    { 0xcf378c97, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc2778c9d, SUC_lexicon_tags_63 },
    { 0xc9b78c9e, SUC_lexicon_tags_12 },
    { 0x1d578c9f, SUC_lexicon_tags_38 },
    { 0x4ed78ca0, SUC_lexicon_tags_15 },
    { 0x7c578ca0, SUC_lexicon_tags_27 },
    { 0x92778ca2, SUC_lexicon_tags_1067 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e378ca5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6578ca8, SUC_lexicon_tags_45 },
    { 0xcf378ca8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20778cb5, SUC_lexicon_tags_15 },
    { 0x39778cb6, SUC_lexicon_tags_14 },
    { 0x90778cb6, SUC_lexicon_tags_37 },
    { 0x71d78cb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99778cba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df78cc0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4178cc5, SUC_lexicon_tags_17 },
    { 0xc4b78cc6, SUC_lexicon_tags_9 },
    { 0xc8578cc5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5b78cc9, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x2178ccb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92178ccf, SUC_lexicon_tags_124 },
    { 0x9ab78ccf, SUC_lexicon_tags_23 },
    { 0xed178cd0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x20778cd4, SUC_lexicon_tags_1 },
    { 0x18978cd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f78cdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb78ce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0b78ce4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e378ce8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b778cee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49378cf2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b78cf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c978cf8, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b78cfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3a978cfe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6978d01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd978d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fd78d07, SUC_lexicon_tags_12 },
    { 0xe0378d08, SUC_lexicon_tags_48 },
    { 0x34978d09, SUC_lexicon_tags_47 },
    { 0x42378d0a, SUC_lexicon_tags_4 },
    { 0x17d78d08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb178d0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48578d13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2378d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c978d1b, SUC_lexicon_tags_45 },
    { 0xf0178d1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94578d1e, SUC_lexicon_tags_0 },
    { 0xa2b78d1e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd0978d21, SUC_lexicon_tags_1 },
    { 0xbfd78d22, SUC_lexicon_tags_1 },
    { 0x8d178d23, SUC_lexicon_tags_26 },
    { 0x87378d24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0778d26, SUC_lexicon_tags_19 },
    { 0xa0578d26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4978d2b, SUC_lexicon_tags_25 },
    { 0x85178d2c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x83178d2e, SUC_lexicon_tags_12 },
    { 0xcbf78d2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91f78d31, SUC_lexicon_tags_49 },
    { 0x9e178d32, SUC_lexicon_tags_42 },
    { 0xd7378d33, SUC_lexicon_tags_1 },
    { 0x8e578d31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x22178d36, SUC_lexicon_tags_48 },
    { 0xd3378d37, SUC_lexicon_tags_79 },
    { 0x83b78d37, SUC_lexicon_tags_14 },
    { 0x8eb78d38, SUC_lexicon_tags_37 },
    { 0xbe178d3a, SUC_lexicon_tags_87 },
    { 0xc7778d3b, SUC_lexicon_tags_43 },
    { 0xc778d3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddb78d3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x51578d41, SUC_lexicon_tags_68 },
    { 0x57178d42, SUC_lexicon_tags_31 },
    { 0xd7178d43, SUC_lexicon_tags_25 },
    { 0xe0178d41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f78d49, SUC_lexicon_tags_17 },
    { 0x71778d4a, SUC_lexicon_tags_11 },
    { 0xb1d78d4b, SUC_lexicon_tags_37 },
    { 0x3b978d4a, SUC_lexicon_tags_82 },
    { 0xd1978d4d, SUC_lexicon_tags_29 },
    { 0xf6378d4b, SUC_lexicon_tags_1 },
    { 0xcc778d4f, SUC_lexicon_tags_293 },
    { 0x32178d4f, SUC_lexicon_tags_17 },
    { 0x4cd78d4a, SUC_lexicon_tags_25 },
    { 0x5db78d52, SUC_lexicon_tags_12 },
    { 0x45f78d4c, SUC_lexicon_tags_11 },
    { 0xcc778d54, SUC_lexicon_tags_7 },
    { 0xccd78d55, SUC_lexicon_tags_1 },
    { 0x45178d56, SUC_lexicon_tags_32 },
    { 0xc4978d57, SUC_lexicon_tags_31 },
    { 0xfed78d58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd78d60, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf578d65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb778d6a, SUC_lexicon_tags_37 },
    { 0xc1378d6b, SUC_lexicon_tags_47 },
    { 0x7f78d6a, SUC_lexicon_tags_19 },
    { 0x66f78d6d, SUC_lexicon_tags_47 },
    { 0xbb78d6e, SUC_lexicon_tags_2 },
    { 0x46d78d6f, SUC_lexicon_tags_26 },
    { 0x27d78d6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11978d75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25778d77, SUC_lexicon_tags_19 },
    { 0x7f378d78, SUC_lexicon_tags_87 },
    { 0xda378d79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfed78d7b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x31378d7d, SUC_lexicon_tags_207 },
    { 0x5778d7e, SUC_lexicon_tags_17 },
    { 0xc7578d7f, SUC_lexicon_tags_37 },
    { 0x3c178d80, SUC_lexicon_tags_10 },
    { 0xadb78d81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d78d87, SUC_lexicon_tags_2 },
    { 0xebd78d88, SUC_lexicon_tags_66 },
    { 0xf4f78d89, SUC_lexicon_tags_21 },
    { 0xbd978d8a, SUC_lexicon_tags_49 },
    { 0x3bb78d8b, SUC_lexicon_tags_3 },
    { 0xbf378d8c, SUC_lexicon_tags_37 },
    { 0x15178d8a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9378d92, SUC_lexicon_tags_10 },
    { 0x7db78d92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd78d96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f978d9a, SUC_lexicon_tags_2 },
    { 0xd5178d9b, SUC_lexicon_tags_160 },
    { 0xfd978d9c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89578da1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98f78da3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e178da5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f78da9, SUC_lexicon_tags_17 },
    { 0x79b78da9, SUC_lexicon_tags_13 },
    { 0xac978dab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b78daf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa178db2, SUC_lexicon_tags_63 },
    { 0x9778db2, SUC_lexicon_tags_12 },
    { 0x9378db4, SUC_lexicon_tags_19 },
    { 0x7c978db3, SUC_lexicon_tags_37 },
    { 0x8978db6, SUC_lexicon_tags_0 },
    { 0xfa178db7, SUC_lexicon_tags_1 },
    { 0xce378db6, SUC_lexicon_tags_11 },
    { 0x64778db3, SUC_lexicon_tags_8 },
    { 0x7cd78db8, SUC_lexicon_tags_0 },
    { 0xba378db5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b78dc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d78dc5, SUC_lexicon_tags_25 },
    { 0x7e978dc5, SUC_lexicon_tags_41 },
    { 0x9ed78dc7, SUC_lexicon_tags_25 },
    { 0x2a578dc8, SUC_lexicon_tags_25 },
    { 0x45378dc9, SUC_lexicon_tags_9 },
    { 0xe0d78dc8, SUC_lexicon_tags_0 },
    { 0x5778dcb, SUC_lexicon_tags_19 },
    { 0x41578dcc, SUC_lexicon_tags_27 },
    { 0xf178dca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd978dd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9d78dd6, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c378dd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3178ddc, SUC_lexicon_tags_32 },
    { 0xdd978ddc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f78de4, SUC_lexicon_tags_17 },
    { 0x7a378de5, SUC_lexicon_tags_144 },
    { 0xa6578de5, SUC_lexicon_tags_2 },
    { 0x65f78de5, SUC_lexicon_tags_25 },
    { 0x9e378de5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27178dec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53778df0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7cf78df2, SUC_lexicon_tags_15 },
    { 0x9b178df3, SUC_lexicon_tags_13 },
    { 0xdb78df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32b78df6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f378dfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd378e05, SUC_lexicon_tags_1144 },
    { 0x35f78e05, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b78e0d, SUC_lexicon_tags_37 },
    { 0x8c978e0e, SUC_lexicon_tags_15 },
    { 0xd8378e0e, SUC_lexicon_tags_87 },
    { 0xe9d78e0d, SUC_lexicon_tags_61 },
    { 0x6578e11, SUC_lexicon_tags_7 },
    { 0x3ab78e12, SUC_lexicon_tags_37 },
    { 0xc0778e12, SUC_lexicon_tags_46 },
    { 0x3c178e12, SUC_lexicon_tags_49 },
    { 0x73778e15, SUC_lexicon_tags_0 },
    { 0x43378e15, SUC_lexicon_tags_2 },
    { 0x43178e13, SUC_lexicon_tags_6 },
    { 0xf6d78e18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x53978e1b, SUC_lexicon_tags_9 },
    { 0xaaf78e1b, SUC_lexicon_tags_7 },
    { 0x87b78e1b, SUC_lexicon_tags_63 },
    { 0x42b78e1e, SUC_lexicon_tags_41 },
    { 0x43378e1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb78e26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc978e28, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe3178e2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30378e2d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db78e32, SUC_lexicon_tags_17 },
    { 0x4ad78e32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c178e36, SUC_lexicon_tags_23 },
    { 0xb4d78e37, SUC_lexicon_tags_1 },
    { 0x1a978e37, SUC_lexicon_tags_216 },
    { 0x75578e38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3178e3b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf6b78e3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14d78e3f, SUC_lexicon_tags_17 },
    { 0xae978e40, SUC_lexicon_tags_7 },
    { 0x2c778e41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9d78e43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b78e46, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b78e4b, SUC_lexicon_tags_0 },
    { 0xc1d78e4c, SUC_lexicon_tags_23 },
    { 0x7c378e4d, SUC_lexicon_tags_1 },
    { 0x68978e4c, SUC_lexicon_tags_65 },
    { 0xed778e4f, SUC_lexicon_tags_33 },
    { 0x5f178e50, SUC_lexicon_tags_37 },
    { 0xb3978e4b, SUC_lexicon_tags_1 },
    { 0x2fd78e4b, SUC_lexicon_tags_15 },
    { 0x44378e4f, SUC_lexicon_tags_91 },
    { 0xf5978e50, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2578e5a, SUC_lexicon_tags_43 },
    { 0xb3178e5a, SUC_lexicon_tags_57 },
    { 0xd6578e5c, SUC_lexicon_tags_63 },
    { 0x6c178e5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a178e5f, SUC_lexicon_tags_29 },
    { 0x3cb78e60, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4f78e62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1978e68, SUC_lexicon_tags_9 },
    { 0x2e578e69, SUC_lexicon_tags_14 },
    { 0x2a178e6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9d78e6c, SUC_lexicon_tags_7 },
    { 0x47178e6d, SUC_lexicon_tags_4 },
    { 0xf8778e6d, SUC_lexicon_tags_113 },
    { 0x87178e6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d78e72, SUC_lexicon_tags_37 },
    { 0x93978e73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7978e76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7178e7c, SUC_lexicon_tags_11 },
    { 0x5b178e7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc178e7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x80978e82, SUC_lexicon_tags_56 },
    { 0x11f78e83, SUC_lexicon_tags_63 },
    { 0x5778e82, SUC_lexicon_tags_25 },
    { 0x61d78e83, SUC_lexicon_tags_59 },
    { 0xed578e82, SUC_lexicon_tags_11 },
    { 0x94b78e87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff78e8d, SUC_lexicon_tags_8 },
    { 0x68b78e8e, SUC_lexicon_tags_25 },
    { 0x3eb78e8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x94378e91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70378e9a, SUC_lexicon_tags_14 },
    { 0xc5f78e9b, SUC_lexicon_tags_10 },
    { 0xe8378e9b, SUC_lexicon_tags_25 },
    { 0xdff78e9d, SUC_lexicon_tags_23 },
    { 0x6eb78e9d, SUC_lexicon_tags_37 },
    { 0x65f78e9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf78ea2, SUC_lexicon_tags_23 },
    { 0x4cd78ea3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd78ea5, SUC_lexicon_tags_25 },
    { 0x3f78ea6, SUC_lexicon_tags_12 },
    { 0x30178ea6, SUC_lexicon_tags_7 },
    { 0xcf78ea8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa578ead, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9178eaf, SUC_lexicon_tags_12 },
    { 0xfdb78eb0, SUC_lexicon_tags_12 },
    { 0x5d178eb1, SUC_lexicon_tags_23 },
    { 0x11378eb1, SUC_lexicon_tags_37 },
    { 0x8d78eb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29778ebc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b178ec0, SUC_lexicon_tags_27 },
    { 0x58b78ec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0578ec6, SUC_lexicon_tags_19 },
    { 0xabf78ec7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb78ecb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x49b78ecd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f78ed2, SUC_lexicon_tags_0 },
    { 0x77578ed3, SUC_lexicon_tags_418 },
    { 0x39978ed4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad78ed7, SUC_lexicon_tags_61 },
    { 0x4378ed8, SUC_lexicon_tags_33 },
    { 0xec978ed8, SUC_lexicon_tags_37 },
    { 0xcc378ed7, SUC_lexicon_tags_1 },
    { 0x83378ed8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4fb78edd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9178ee1, SUC_lexicon_tags_37 },
    { 0xe7d78ee2, SUC_lexicon_tags_0 },
    { 0xc1578ee3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83578ee9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb978eed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9178ef0, SUC_lexicon_tags_23 },
    { 0xe6d78ef1, SUC_lexicon_tags_11 },
    { 0x6f978ef2, SUC_lexicon_tags_17 },
    { 0x5c778ef3, SUC_lexicon_tags_37 },
    { 0x9d378ef4, SUC_lexicon_tags_43 },
    { 0x9cb78ef4, SUC_lexicon_tags_48 },
    { 0x64178ef3, SUC_lexicon_tags_0 },
    { 0x92778ef5, SUC_lexicon_tags_48 },
    { 0x84778ef8, SUC_lexicon_tags_7 },
    { 0x26d78ef9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d378efb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf78efe, SUC_lexicon_tags_15 },
    { 0xf5978eff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xafb78f01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d78f04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd78f06, SUC_lexicon_tags_3 },
    { 0x89d78f07, SUC_lexicon_tags_30 },
    { 0x8ad78f08, SUC_lexicon_tags_12 },
    { 0x94578f06, SUC_lexicon_tags_26 },
    { 0x26378f0a, SUC_lexicon_tags_37 },
    { 0xdc778f0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd778f0f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8978f17, SUC_lexicon_tags_2 },
    { 0xc0978f18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b78f1e, SUC_lexicon_tags_32 },
    { 0x84d78f1f, SUC_lexicon_tags_60 },
    { 0x6a378f20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4978f22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafd78f24, SUC_lexicon_tags_26 },
    { 0x23178f25, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x49f78f27, SUC_lexicon_tags_37 },
    { 0x8e778f27, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab78f30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33978f33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb578f39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90978f3f, SUC_lexicon_tags_14 },
    { 0xd4578f40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31578f47, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b78f50, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4778f54, SUC_lexicon_tags_47 },
    { 0xa1778f55, SUC_lexicon_tags_15 },
    { 0x7f978f56, SUC_lexicon_tags_87 },
    { 0x3ef78f57, SUC_lexicon_tags_23 },
    { 0x61978f55, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f78f5c, SUC_lexicon_tags_1 },
    { 0x9a778f5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x69578f5f, SUC_lexicon_tags_27 },
    { 0xa1778f60, SUC_lexicon_tags_29 },
    { 0x31378f61, SUC_lexicon_tags_29 },
    { 0x39d78f62, SUC_lexicon_tags_14 },
    { 0x87978f62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b78f67, SUC_lexicon_tags_37 },
    { 0x30578f68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b78f6d, SUC_lexicon_tags_34 },
    { 0x2b178f6e, SUC_lexicon_tags_38 },
    { 0x50d78f6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5978f71, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16578f76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8978f81, SUC_lexicon_tags_17 },
    { 0x5df78f82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8e978f84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92f78f86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb178f88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef78f8d, SUC_lexicon_tags_1 },
    { 0x4f578f8d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf378f95, SUC_lexicon_tags_2 },
    { 0xcb178f96, SUC_lexicon_tags_7 },
    { 0x4db78f97, SUC_lexicon_tags_30 },
    { 0x37d78f97, SUC_lexicon_tags_0 },
    { 0x58d78f99, SUC_lexicon_tags_47 },
    { 0xeb578f99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13178f9d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2378fa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf78fa5, SUC_lexicon_tags_49 },
    { 0x42578fa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f78faa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6378fae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f78fb5, SUC_lexicon_tags_6 },
    { 0xfe778fb6, SUC_lexicon_tags_37 },
    { 0x23f78fb7, SUC_lexicon_tags_49 },
    { 0xa4b78fb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7b978fba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcc378fbc, SUC_lexicon_tags_34 },
    { 0x40778fbd, SUC_lexicon_tags_51 },
    { 0x74d78fbc, SUC_lexicon_tags_63 },
    { 0xc2778fbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c578fc3, SUC_lexicon_tags_37 },
    { 0xe4778fc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c378fc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa178fcb, SUC_lexicon_tags_19 },
    { 0xd7978fcc, SUC_lexicon_tags_14 },
    { 0x2b378fcb, SUC_lexicon_tags_6 },
    { 0x76d78fce, SUC_lexicon_tags_17 },
    { 0x8d378fcf, SUC_lexicon_tags_40 },
    { 0x50178fd0, SUC_lexicon_tags_51 },
    { 0xf6578fcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdd78fd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c378fd7, SUC_lexicon_tags_1 },
    { 0xf7778fd8, SUC_lexicon_tags_33 },
    { 0xbb378fd9, SUC_lexicon_tags_16 },
    { 0x1e978fda, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed78fe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7778fe9, SUC_lexicon_tags_17 },
    { 0x99b78fe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b78ff0, SUC_lexicon_tags_14 },
    { 0xa4178ff1, SUC_lexicon_tags_63 },
    { 0xfe378ff0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f78ff6, SUC_lexicon_tags_35 },
    { 0x3b578ff6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd78fff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xba579002, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8eb79004, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb9b79006, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f79009, SUC_lexicon_tags_49 },
    { 0x2af7900a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d7900e, SUC_lexicon_tags_25 },
    { 0xf457900f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92379013, SUC_lexicon_tags_12 },
    { 0x30979014, SUC_lexicon_tags_2 },
    { 0x6579013, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d79018, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb17901a, SUC_lexicon_tags_6 },
    { 0x1a77901a, SUC_lexicon_tags_13 },
    { 0x9c97901c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb637901e, SUC_lexicon_tags_19 },
    { 0x3637901f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf79022, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f179025, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x2b79027, SUC_lexicon_tags_45 },
    { 0xf1179028, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa677902e, SUC_lexicon_tags_201 },
    { 0x1117902f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9b79031, SUC_lexicon_tags_17 },
    { 0xba379032, SUC_lexicon_tags_56 },
    { 0x8a179032, SUC_lexicon_tags_25 },
    { 0xc8b79032, SUC_lexicon_tags_7 },
    { 0xf3379035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14179037, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b7903b, SUC_lexicon_tags_45 },
    { 0x9d37903c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc779041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9597904b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed7904e, SUC_lexicon_tags_37 },
    { 0x3a57904f, SUC_lexicon_tags_43 },
    { 0x2179050, SUC_lexicon_tags_7 },
    { 0x36979051, SUC_lexicon_tags_11 },
    { 0xf557904f, SUC_lexicon_tags_8 },
    { 0x25779050, SUC_lexicon_tags_46 },
    { 0x24579052, SUC_lexicon_tags_7 },
    { 0x7db7904e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7379057, SUC_lexicon_tags_14 },
    { 0xa0379058, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda37905f, SUC_lexicon_tags_40 },
    { 0x7bf79060, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x30b79062, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b79065, SUC_lexicon_tags_6 },
    { 0x7f779065, SUC_lexicon_tags_33 },
    { 0x6ab79065, SUC_lexicon_tags_43 },
    { 0xe2f79066, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b79070, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86979074, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb79077, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xe637907a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d7907c, SUC_lexicon_tags_12 },
    { 0xd7907d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b79085, SUC_lexicon_tags_12 },
    { 0x92d79085, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b79089, SUC_lexicon_tags_4 },
    { 0xb857908a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d79090, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8179095, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d79097, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0179099, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3f7909b, SUC_lexicon_tags_2 },
    { 0x5397909b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc57909f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b790a2, SUC_lexicon_tags_17 },
    { 0x977790a3, SUC_lexicon_tags_2 },
    { 0x901790a4, SUC_lexicon_tags_77 },
    { 0x387790a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf09790a7, SUC_lexicon_tags_60 },
    { 0xc03790a8, SUC_lexicon_tags_43 },
    { 0x4d1790a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f3790ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7790b3, SUC_lexicon_tags_8 },
    { 0xafd790b4, SUC_lexicon_tags_0 },
    { 0x8cb790b5, SUC_lexicon_tags_37 },
    { 0xb29790b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3790b9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7790bc, SUC_lexicon_tags_174 },
    { 0x5af790bc, SUC_lexicon_tags_12 },
    { 0x8d3790bd, SUC_lexicon_tags_17 },
    { 0x10f790bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07790c4, SUC_lexicon_tags_0 },
    { 0x35b790c4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3790c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x599790ca, SUC_lexicon_tags_7 },
    { 0xc7b790cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9790d0, SUC_lexicon_tags_17 },
    { 0x153790d1, SUC_lexicon_tags_11 },
    { 0xf07790d0, SUC_lexicon_tags_26 },
    { 0x8bb790d3, SUC_lexicon_tags_17 },
    { 0xbb790d4, SUC_lexicon_tags_47 },
    { 0xccd790d5, SUC_lexicon_tags_12 },
    { 0x977790d0, SUC_lexicon_tags_2 },
    { 0x1b790d7, SUC_lexicon_tags_17 },
    { 0xf5d790d8, SUC_lexicon_tags_17 },
    { 0x21b790d9, SUC_lexicon_tags_4 },
    { 0x27790da, SUC_lexicon_tags_15 },
    { 0x8ff790da, SUC_lexicon_tags_62 },
    { 0x95d790dc, SUC_lexicon_tags_27 },
    { 0x23d790dd, SUC_lexicon_tags_61 },
    { 0x3fd790dd, SUC_lexicon_tags_2 },
    { 0xf19790d9, SUC_lexicon_tags_25 },
    { 0xf1f790e0, SUC_lexicon_tags_37 },
    { 0x293790e1, SUC_lexicon_tags_38 },
    { 0xa8d790db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b790e5, SUC_lexicon_tags_1 },
    { 0x76d790e6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x749790e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39790ec, SUC_lexicon_tags_12 },
    { 0xdf5790ec, SUC_lexicon_tags_2 },
    { 0x1cd790ee, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff790fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6779102, SUC_lexicon_tags_12 },
    { 0x9a579102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7d79105, SUC_lexicon_tags_26 },
    { 0x89779106, SUC_lexicon_tags_45 },
    { 0xdf79107, SUC_lexicon_tags_4 },
    { 0xaa979108, SUC_lexicon_tags_9 },
    { 0xbf79107, SUC_lexicon_tags_12 },
    { 0x7477910a, SUC_lexicon_tags_63 },
    { 0xbb57910b, SUC_lexicon_tags_91 },
    { 0x60b79105, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7379110, SUC_lexicon_tags_6 },
    { 0xf2d79111, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb779115, SUC_lexicon_tags_33 },
    { 0x9cd79116, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8b79118, SUC_lexicon_tags_61 },
    { 0xe2d79118, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe17911b, SUC_lexicon_tags_7 },
    { 0xdef7911c, SUC_lexicon_tags_28 },
    { 0x8e37911d, SUC_lexicon_tags_178 },
    { 0x57911d, SUC_lexicon_tags_2 },
    { 0x177911f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb6d79121, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7f379123, SUC_lexicon_tags_9 },
    { 0xc5979124, SUC_lexicon_tags_23 },
    { 0x20579125, SUC_lexicon_tags_1 },
    { 0xc6179125, SUC_lexicon_tags_2 },
    { 0xa4979127, SUC_lexicon_tags_1 },
    { 0x6ed79127, SUC_lexicon_tags_1 },
    { 0xbef79129, SUC_lexicon_tags_55 },
    { 0x76579129, SUC_lexicon_tags_17 },
    { 0xf117912a, SUC_lexicon_tags_38 },
    { 0xb5d7912c, SUC_lexicon_tags_7 },
    { 0xf9979129, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d79130, SUC_lexicon_tags_0 },
    { 0x81379131, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2979135, SUC_lexicon_tags_37 },
    { 0x4bf79136, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ab79138, SUC_lexicon_tags_14 },
    { 0xc7b79138, SUC_lexicon_tags_2 },
    { 0x6d179139, SUC_lexicon_tags_61 },
    { 0xe2f7913a, SUC_lexicon_tags_28 },
    { 0xa0f7913a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28379141, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e579143, SUC_lexicon_tags_37 },
    { 0x9b579143, SUC_lexicon_tags_25 },
    { 0xc6b79143, SUC_lexicon_tags_15 },
    { 0x5e779146, SUC_lexicon_tags_3 },
    { 0x44b79145, SUC_lexicon_tags_65 },
    { 0xe579148, SUC_lexicon_tags_1 },
    { 0xa8f79149, SUC_lexicon_tags_26 },
    { 0x76b7914a, SUC_lexicon_tags_45 },
    { 0x23779143, SUC_lexicon_tags_7 },
    { 0x2297914c, SUC_lexicon_tags_13 },
    { 0x5c979143, SUC_lexicon_tags_30 },
    { 0x94f7914e, SUC_lexicon_tags_18 },
    { 0x3c379146, SUC_lexicon_tags_7 },
    { 0x3ad79150, SUC_lexicon_tags_3 },
    { 0x8db79151, SUC_lexicon_tags_37 },
    { 0xc4d79152, SUC_lexicon_tags_12 },
    { 0xfeb79153, SUC_lexicon_tags_27 },
    { 0xcfb79154, SUC_lexicon_tags_56 },
    { 0x72179153, SUC_lexicon_tags_39 },
    { 0x9ed79156, SUC_lexicon_tags_6 },
    { 0x92379157, SUC_lexicon_tags_17 },
    { 0x77379157, SUC_lexicon_tags_47 },
    { 0xb0b79155, SUC_lexicon_tags_10 },
    { 0x72979154, SUC_lexicon_tags_60 },
    { 0x54d7915b, SUC_lexicon_tags_23 },
    { 0xebd7915b, SUC_lexicon_tags_53 },
    { 0xfcd79152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f79160, SUC_lexicon_tags_14 },
    { 0xb5b79161, SUC_lexicon_tags_49 },
    { 0xe9b79162, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8979164, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xca379166, SUC_lexicon_tags_23 },
    { 0x28379167, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f7916d, SUC_lexicon_tags_16 },
    { 0x9e77916e, SUC_lexicon_tags_30 },
    { 0xee17916e, SUC_lexicon_tags_12 },
    { 0xf6379170, SUC_lexicon_tags_7 },
    { 0x24979171, SUC_lexicon_tags_37 },
    { 0x6dd79171, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf579175, SUC_lexicon_tags_17 },
    { 0xffd79176, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed79179, SUC_lexicon_tags_8 },
    { 0x4cd7917a, SUC_lexicon_tags_26 },
    { 0xd1f7917a, SUC_lexicon_tags_27 },
    { 0xa37917a, SUC_lexicon_tags_27 },
    { 0xd17917c, SUC_lexicon_tags_38 },
    { 0x3937917e, SUC_lexicon_tags_23 },
    { 0x63f7917d, SUC_lexicon_tags_29 },
    { 0x12d79180, SUC_lexicon_tags_37 },
    { 0x31979181, SUC_lexicon_tags_37 },
    { 0x88379182, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf1b79184, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b7918f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe179191, SUC_lexicon_tags_4 },
    { 0xab379192, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5237919a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x75d7919c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5791a3, SUC_lexicon_tags_2 },
    { 0xbed791a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4f791a6, SUC_lexicon_tags_17 },
    { 0xd17791a7, SUC_lexicon_tags_32 },
    { 0x90d791a8, SUC_lexicon_tags_9 },
    { 0x35d791a9, SUC_lexicon_tags_11 },
    { 0x275791a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc83791ac, SUC_lexicon_tags_113 },
    { 0x259791ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x645791b0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef791b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1791b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x155791ba, SUC_lexicon_tags_33 },
    { 0x155791ba, SUC_lexicon_tags_15 },
    { 0xe75791bc, SUC_lexicon_tags_63 },
    { 0x5a9791bd, SUC_lexicon_tags_28 },
    { 0x1a3791bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89f791c0, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x949791c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9791c6, SUC_lexicon_tags_11 },
    { 0xe1d791c6, SUC_lexicon_tags_33 },
    { 0x465791c8, SUC_lexicon_tags_47 },
    { 0xd19791c7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5791ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32d791d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b9791d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39791db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd11791dd, SUC_lexicon_tags_1 },
    { 0xf5f791de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x563791e0, SUC_lexicon_tags_6 },
    { 0x69d791e1, SUC_lexicon_tags_7 },
    { 0x2a9791e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9791e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe01791e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9791eb, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x465791ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d791f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1791f7, SUC_lexicon_tags_87 },
    { 0x8b1791f8, SUC_lexicon_tags_9 },
    { 0xb11791f9, SUC_lexicon_tags_7 },
    { 0x33b791fa, SUC_lexicon_tags_1 },
    { 0xe47791fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa73791fd, SUC_lexicon_tags_17 },
    { 0xf33791fe, SUC_lexicon_tags_7 },
    { 0x93d791fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22579201, SUC_lexicon_tags_107 },
    { 0x3f979202, SUC_lexicon_tags_10 },
    { 0x23779202, SUC_lexicon_tags_11 },
    { 0xebb79202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5979210, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64779215, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd79225, SUC_lexicon_tags_65 },
    { 0x31d79226, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48d79228, SUC_lexicon_tags_1 },
    { 0x57379228, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf137922f, SUC_lexicon_tags_37 },
    { 0xe6d79230, SUC_lexicon_tags_38 },
    { 0x4d97922f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ab79233, SUC_lexicon_tags_4 },
    { 0x1d79234, SUC_lexicon_tags_27 },
    { 0xd9779234, SUC_lexicon_tags_19 },
    { 0x97179236, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb7923d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8017923f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb3579241, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x94979244, SUC_lexicon_tags_25 },
    { 0xa8579245, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b79248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf217924a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5937924d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b79250, SUC_lexicon_tags_11 },
    { 0x5e779251, SUC_lexicon_tags_47 },
    { 0xf2579252, SUC_lexicon_tags_13 },
    { 0x6e179253, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3179259, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d7925d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef77925f, SUC_lexicon_tags_9 },
    { 0xa3179260, SUC_lexicon_tags_141 },
    { 0xfcf79261, SUC_lexicon_tags_1 },
    { 0x6b79260, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc3d79264, SUC_lexicon_tags_34 },
    { 0x75f79265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd979272, SUC_lexicon_tags_38 },
    { 0x4379273, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe177927b, SUC_lexicon_tags_17 },
    { 0x7fd7927c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4257927e, SUC_lexicon_tags_7 },
    { 0x44f7927f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f79282, SUC_lexicon_tags_21 },
    { 0x61979283, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb579285, SUC_lexicon_tags_63 },
    { 0x98b79286, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x53979289, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd7928d, SUC_lexicon_tags_12 },
    { 0xa3d7928d, SUC_lexicon_tags_7 },
    { 0x7357928d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42979297, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba57929c, SUC_lexicon_tags_11 },
    { 0x157929d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb792a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x771792a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb792ac, SUC_lexicon_tags_15 },
    { 0x8bd792ad, SUC_lexicon_tags_28 },
    { 0x8fb792ae, SUC_lexicon_tags_14 },
    { 0x4f3792ae, SUC_lexicon_tags_45 },
    { 0x5cf792b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7792b4, SUC_lexicon_tags_6 },
    { 0xebd792b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xedd792b7, SUC_lexicon_tags_46 },
    { 0xe73792b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b792bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x389792c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313792c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615792d2, SUC_lexicon_tags_77 },
    { 0x607792d3, SUC_lexicon_tags_29 },
    { 0xa07792d4, SUC_lexicon_tags_43 },
    { 0xcf7792d5, SUC_lexicon_tags_28 },
    { 0x113792d5, SUC_lexicon_tags_46 },
    { 0xa63792d4, SUC_lexicon_tags_29 },
    { 0x8ff792d8, SUC_lexicon_tags_15 },
    { 0x8cf792d9, SUC_lexicon_tags_47 },
    { 0x6bd792da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb81792dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeeb792de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69792e5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x827792e7, SUC_lexicon_tags_85 },
    { 0x9f9792e7, SUC_lexicon_tags_37 },
    { 0x3cf792e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5792ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f5792f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f792f4, SUC_lexicon_tags_39 },
    { 0xe17792f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3792f8, SUC_lexicon_tags_28 },
    { 0xf07792f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x99f792fb, SUC_lexicon_tags_49 },
    { 0xfa9792fc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b979301, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead79307, SUC_lexicon_tags_1 },
    { 0x7ff79308, SUC_lexicon_tags_42 },
    { 0x48f79307, SUC_lexicon_tags_21 },
    { 0x79d7930a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad77930c, SUC_lexicon_tags_33 },
    { 0xffd7930c, SUC_lexicon_tags_19 },
    { 0x2db7930e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f79311, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf6379313, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86d79315, SUC_lexicon_tags_12 },
    { 0xc4d79316, SUC_lexicon_tags_7 },
    { 0x22979315, SUC_lexicon_tags_463 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f79320, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617932d, SUC_lexicon_tags_158 },
    { 0x4b97932e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x18379330, SUC_lexicon_tags_47 },
    { 0xfaf79331, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f979333, SUC_lexicon_tags_37 },
    { 0x56979334, SUC_lexicon_tags_88 },
    { 0xb1779335, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b79339, SUC_lexicon_tags_26 },
    { 0x2ff7933a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2379340, SUC_lexicon_tags_27 },
    { 0x99b79340, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x10779344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8db79346, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c379349, SUC_lexicon_tags_17 },
    { 0xa37934a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6517934f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30979351, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5979356, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a77935e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d79361, SUC_lexicon_tags_151 },
    { 0x57379361, SUC_lexicon_tags_6 },
    { 0xf6f79363, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb37936b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ed7936d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8897936f, SUC_lexicon_tags_17 },
    { 0x2a779370, SUC_lexicon_tags_25 },
    { 0x22d79371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f79375, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70b79377, SUC_lexicon_tags_2 },
    { 0xab579378, SUC_lexicon_tags_23 },
    { 0x1f579378, SUC_lexicon_tags_17 },
    { 0xb837937a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc617937d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9f7937f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ab79381, SUC_lexicon_tags_8 },
    { 0x19379382, SUC_lexicon_tags_37 },
    { 0xbd79382, SUC_lexicon_tags_2 },
    { 0xb3d79382, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3397938b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f79391, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x99379393, SUC_lexicon_tags_11 },
    { 0xbdd79393, SUC_lexicon_tags_5 },
    { 0x5eb79395, SUC_lexicon_tags_7 },
    { 0xa1979396, SUC_lexicon_tags_1 },
    { 0xa2979397, SUC_lexicon_tags_31 },
    { 0x64979398, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8577939a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x24b7939d, SUC_lexicon_tags_77 },
    { 0x5077939d, SUC_lexicon_tags_30 },
    { 0x6897939f, SUC_lexicon_tags_17 },
    { 0x11d7939f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e5793a2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc41793a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x765793a6, SUC_lexicon_tags_19 },
    { 0x8f793a7, SUC_lexicon_tags_2 },
    { 0xfad793a7, SUC_lexicon_tags_25 },
    { 0x95b793a8, SUC_lexicon_tags_25 },
    { 0x225793aa, SUC_lexicon_tags_0 },
    { 0xc61793a8, SUC_lexicon_tags_2 },
    { 0xa0b793a8, SUC_lexicon_tags_63 },
    { 0x785793ab, SUC_lexicon_tags_12 },
    { 0x737793ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2d793b0, SUC_lexicon_tags_29 },
    { 0x76d793b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5793b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf793ba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865793bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39793c3, SUC_lexicon_tags_77 },
    { 0xc99793c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d793ca, SUC_lexicon_tags_0 },
    { 0x1bd793cb, SUC_lexicon_tags_0 },
    { 0x5d5793cc, SUC_lexicon_tags_29 },
    { 0x6d3793ca, SUC_lexicon_tags_0 },
    { 0xc6f793cb, SUC_lexicon_tags_1273 },
    { 0xb3f793ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71793d1, SUC_lexicon_tags_37 },
    { 0x61b793d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44f793d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe03793d6, SUC_lexicon_tags_39 },
    { 0x9e1793d7, SUC_lexicon_tags_15 },
    { 0x43793d8, SUC_lexicon_tags_28 },
    { 0x7b9793d9, SUC_lexicon_tags_19 },
    { 0x669793da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269793e2, SUC_lexicon_tags_37 },
    { 0x3a7793e3, SUC_lexicon_tags_2 },
    { 0x879793e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139793eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84b793ed, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37793f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81793f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x899793f8, SUC_lexicon_tags_19 },
    { 0x3c5793f9, SUC_lexicon_tags_56 },
    { 0x627793fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb39793fc, SUC_lexicon_tags_19 },
    { 0x95793fd, SUC_lexicon_tags_37 },
    { 0xc35793fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a179400, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef79404, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd6b79406, SUC_lexicon_tags_16 },
    { 0x48779407, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b7940a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b79411, SUC_lexicon_tags_62 },
    { 0xb0179412, SUC_lexicon_tags_2 },
    { 0xaa579413, SUC_lexicon_tags_17 },
    { 0x4db79413, SUC_lexicon_tags_7 },
    { 0x37179415, SUC_lexicon_tags_25 },
    { 0xd6d79414, SUC_lexicon_tags_46 },
    { 0xb1379417, SUC_lexicon_tags_7 },
    { 0xd8b79417, SUC_lexicon_tags_33 },
    { 0x4a379419, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f77941b, SUC_lexicon_tags_25 },
    { 0xe6d7941b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x897941f, SUC_lexicon_tags_22 },
    { 0x17d7941f, SUC_lexicon_tags_38 },
    { 0x5b979421, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62379423, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff79428, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60979431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5779437, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2197943b, SUC_lexicon_tags_33 },
    { 0xd3f7943c, SUC_lexicon_tags_37 },
    { 0xce17943d, SUC_lexicon_tags_12 },
    { 0x2bb7943e, SUC_lexicon_tags_54 },
    { 0x2377943b, SUC_lexicon_tags_37 },
    { 0xcf77943d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0d79442, SUC_lexicon_tags_17 },
    { 0x7a179443, SUC_lexicon_tags_12 },
    { 0x9b379444, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6579450, SUC_lexicon_tags_0 },
    { 0x78579451, SUC_lexicon_tags_27 },
    { 0x16779452, SUC_lexicon_tags_15 },
    { 0xddb79452, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f79459, SUC_lexicon_tags_26 },
    { 0x26d7945a, SUC_lexicon_tags_27 },
    { 0x1f17945b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8b7945d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db79466, SUC_lexicon_tags_19 },
    { 0xac579466, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab7946a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf757946c, SUC_lexicon_tags_15 },
    { 0x7bd7946d, SUC_lexicon_tags_66 },
    { 0x8917946e, SUC_lexicon_tags_14 },
    { 0xfd77946e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8379474, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad179477, SUC_lexicon_tags_35 },
    { 0xdb579478, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc177947a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b7947e, SUC_lexicon_tags_17 },
    { 0x1b57947f, SUC_lexicon_tags_43 },
    { 0x4597947e, SUC_lexicon_tags_17 },
    { 0x4d37947e, SUC_lexicon_tags_48 },
    { 0xd017947f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad17948f, SUC_lexicon_tags_27 },
    { 0xf5d7948f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c579492, SUC_lexicon_tags_43 },
    { 0x7a379492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b79499, SUC_lexicon_tags_10 },
    { 0xf477949a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37794a2, SUC_lexicon_tags_38 },
    { 0xfeb794a2, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf794a7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f794ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe61794ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x257794b0, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf2d794b2, SUC_lexicon_tags_37 },
    { 0x3fd794b3, SUC_lexicon_tags_4 },
    { 0x8db794b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93794b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc35794b9, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0xb7f794bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899794bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e7794c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc2f794c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51794c7, SUC_lexicon_tags_11 },
    { 0x7eb794c7, SUC_lexicon_tags_37 },
    { 0x45d794c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5794cf, SUC_lexicon_tags_1 },
    { 0xaa5794d0, SUC_lexicon_tags_14 },
    { 0x3e7794d1, SUC_lexicon_tags_15 },
    { 0x7fd794d2, SUC_lexicon_tags_54 },
    { 0x41d794d3, SUC_lexicon_tags_9 },
    { 0x451794d3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x421794d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x251794d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16d794db, SUC_lexicon_tags_3 },
    { 0xa5794dc, SUC_lexicon_tags_11 },
    { 0x987794dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53794e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7c3794e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d794e9, SUC_lexicon_tags_37 },
    { 0x6ed794ea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x909794ec, SUC_lexicon_tags_46 },
    { 0x915794ec, SUC_lexicon_tags_12 },
    { 0xa3b794ee, SUC_lexicon_tags_11 },
    { 0xefd794ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x939794f1, SUC_lexicon_tags_39 },
    { 0x7f1794f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9794fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b794fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb579501, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a179506, SUC_lexicon_tags_37 },
    { 0xb3b79507, SUC_lexicon_tags_11 },
    { 0xe0979506, SUC_lexicon_tags_16 },
    { 0xf8d79509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3077950c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd79512, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x9fb79514, SUC_lexicon_tags_4 },
    { 0x20579515, SUC_lexicon_tags_12 },
    { 0xbf979516, SUC_lexicon_tags_61 },
    { 0x73379517, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d17951d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3717951f, SUC_lexicon_tags_7 },
    { 0x17d79520, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41979523, SUC_lexicon_tags_19 },
    { 0xe4979524, SUC_lexicon_tags_85 },
    { 0xbcf79525, SUC_lexicon_tags_63 },
    { 0xef579524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2097952c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25d7952e, SUC_lexicon_tags_37 },
    { 0x5dd7952f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc0579531, SUC_lexicon_tags_54 },
    { 0x42779532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc379538, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9597953e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x21179540, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb79543, SUC_lexicon_tags_0 },
    { 0x1c779543, SUC_lexicon_tags_0 },
    { 0x1779545, SUC_lexicon_tags_12 },
    { 0xb4779545, SUC_lexicon_tags_37 },
    { 0xc2179547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1197954b, SUC_lexicon_tags_82 },
    { 0xfcf7954c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9179551, SUC_lexicon_tags_87 },
    { 0x81d79551, SUC_lexicon_tags_53 },
    { 0xd8179553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5379555, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeef79557, SUC_lexicon_tags_39 },
    { 0x47579557, SUC_lexicon_tags_35 },
    { 0x11b79559, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b79560, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf79566, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a77956a, SUC_lexicon_tags_17 },
    { 0x7037956a, SUC_lexicon_tags_31 },
    { 0xba77956c, SUC_lexicon_tags_9 },
    { 0xdbf7956d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd757956f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f79572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10779576, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd377957b, SUC_lexicon_tags_33 },
    { 0xc117957b, SUC_lexicon_tags_37 },
    { 0x9877957d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7817957f, SUC_lexicon_tags_14 },
    { 0x30f7957f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c179584, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977958a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b57958c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb79590, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6979593, SUC_lexicon_tags_9 },
    { 0xe9579594, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e179597, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e57959b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a97959f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21795a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30b795a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x117795a9, SUC_lexicon_tags_0 },
    { 0x489795aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbd9795ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7795b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x503795b4, SUC_lexicon_tags_20 },
    { 0xab795b5, SUC_lexicon_tags_43 },
    { 0xc0d795b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x947795b8, SUC_lexicon_tags_47 },
    { 0xe1795b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d795bc, SUC_lexicon_tags_49 },
    { 0x4bb795bd, SUC_lexicon_tags_15 },
    { 0xa3795bc, SUC_lexicon_tags_6 },
    { 0xddf795bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa3795c1, SUC_lexicon_tags_25 },
    { 0xfdb795c1, SUC_lexicon_tags_30 },
    { 0x56b795c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83b795c5, SUC_lexicon_tags_12 },
    { 0x911795c5, SUC_lexicon_tags_30 },
    { 0x31b795c7, SUC_lexicon_tags_123 },
    { 0xedf795c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb795ca, SUC_lexicon_tags_17 },
    { 0xbeb795cb, SUC_lexicon_tags_6 },
    { 0x28b795ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fd795ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b7795d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11795d3, SUC_lexicon_tags_38 },
    { 0xc77795d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd795d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f795db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03795de, SUC_lexicon_tags_15 },
    { 0xd97795de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe19795e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9795e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577795e8, SUC_lexicon_tags_37 },
    { 0x89b795e9, SUC_lexicon_tags_79 },
    { 0xfff795ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15795ed, SUC_lexicon_tags_45 },
    { 0x70b795ee, SUC_lexicon_tags_27 },
    { 0xdd9795ef, SUC_lexicon_tags_2 },
    { 0x8e9795ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5795f6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387795fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81179601, SUC_lexicon_tags_91 },
    { 0xd4379602, SUC_lexicon_tags_37 },
    { 0xadd79603, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4579605, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab79608, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x38f7960a, SUC_lexicon_tags_38 },
    { 0xee77960a, SUC_lexicon_tags_43 },
    { 0x76f7960a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef7960f, SUC_lexicon_tags_37 },
    { 0x11779610, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb779613, SUC_lexicon_tags_1 },
    { 0x48779614, SUC_lexicon_tags_9 },
    { 0x12179615, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2579617, SUC_lexicon_tags_25 },
    { 0x91979618, SUC_lexicon_tags_14 },
    { 0xa6179619, SUC_lexicon_tags_81 },
    { 0xad57961a, SUC_lexicon_tags_17 },
    { 0xd577961a, SUC_lexicon_tags_1 },
    { 0xb117961b, SUC_lexicon_tags_12 },
    { 0xfc57961d, SUC_lexicon_tags_47 },
    { 0x4a37961e, SUC_lexicon_tags_66 },
    { 0x82b7961d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xee179621, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d79625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7b79627, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f77962b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b7962f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9b79631, SUC_lexicon_tags_10 },
    { 0x4bd79632, SUC_lexicon_tags_2 },
    { 0x5f579632, SUC_lexicon_tags_8 },
    { 0x95379634, SUC_lexicon_tags_6 },
    { 0x18579634, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf979638, SUC_lexicon_tags_25 },
    { 0xfdf79639, SUC_lexicon_tags_82 },
    { 0x43d7963a, SUC_lexicon_tags_1 },
    { 0xa697963b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f79643, SUC_lexicon_tags_2 },
    { 0xd1579644, SUC_lexicon_tags_7 },
    { 0x93d79644, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd979648, SUC_lexicon_tags_7 },
    { 0xd6579648, SUC_lexicon_tags_9 },
    { 0xeef7964a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x2777964c, SUC_lexicon_tags_1 },
    { 0x4637964d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d17964f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x16d79651, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb4379653, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef979658, SUC_lexicon_tags_2 },
    { 0xebd79659, SUC_lexicon_tags_11 },
    { 0x18d79658, SUC_lexicon_tags_1 },
    { 0x4f57965b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f57965f, SUC_lexicon_tags_38 },
    { 0x59979660, SUC_lexicon_tags_25 },
    { 0xf7d79660, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21379666, SUC_lexicon_tags_46 },
    { 0xa2d79666, SUC_lexicon_tags_97 },
    { 0x5c179668, SUC_lexicon_tags_13 },
    { 0x8979669, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf717966d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9b7966f, SUC_lexicon_tags_12 },
    { 0x1579670, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b79674, SUC_lexicon_tags_25 },
    { 0xbf79674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52779679, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b7967d, SUC_lexicon_tags_37 },
    { 0x9197967d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb1779680, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd0779682, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b79686, SUC_lexicon_tags_23 },
    { 0x45f79686, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d97968d, SUC_lexicon_tags_0 },
    { 0x9737968d, SUC_lexicon_tags_26 },
    { 0xd337968f, SUC_lexicon_tags_2 },
    { 0x9457968e, SUC_lexicon_tags_53 },
    { 0x5b779691, SUC_lexicon_tags_15 },
    { 0x62179691, SUC_lexicon_tags_0 },
    { 0xf097968f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x81579696, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf17969b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5737969d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd796a0, SUC_lexicon_tags_47 },
    { 0xcf796a0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5796a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b796ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6fd796ae, SUC_lexicon_tags_25 },
    { 0x9a5796af, SUC_lexicon_tags_7 },
    { 0xead796b0, SUC_lexicon_tags_45 },
    { 0xb07796ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ed796b3, SUC_lexicon_tags_25 },
    { 0xc25796b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313796b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55b796ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09796bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd796c2, SUC_lexicon_tags_37 },
    { 0x87d796c2, SUC_lexicon_tags_14 },
    { 0x4d3796c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x58d796c6, SUC_lexicon_tags_15 },
    { 0xd95796c6, SUC_lexicon_tags_28 },
    { 0x5bf796c8, SUC_lexicon_tags_2 },
    { 0x961796c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d796cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b796d0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb5796d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91796e0, SUC_lexicon_tags_17 },
    { 0x333796e1, SUC_lexicon_tags_16 },
    { 0xaf3796e2, SUC_lexicon_tags_2 },
    { 0xdab796e1, SUC_lexicon_tags_19 },
    { 0xfd796e4, SUC_lexicon_tags_37 },
    { 0x4cd796e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1796e8, SUC_lexicon_tags_11 },
    { 0x8e5796e9, SUC_lexicon_tags_68 },
    { 0x7a5796ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc67796ec, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f796f0, SUC_lexicon_tags_1 },
    { 0xd23796f1, SUC_lexicon_tags_23 },
    { 0xce5796f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9796f5, SUC_lexicon_tags_8 },
    { 0xe5796f6, SUC_lexicon_tags_15 },
    { 0xde5796f7, SUC_lexicon_tags_25 },
    { 0x887796f8, SUC_lexicon_tags_37 },
    { 0x907796f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d5796fb, SUC_lexicon_tags_4 },
    { 0xa45796fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11379706, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x20979709, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb7970c, SUC_lexicon_tags_25 },
    { 0x9897970d, SUC_lexicon_tags_77 },
    { 0x8d57970e, SUC_lexicon_tags_37 },
    { 0x6cd7970c, SUC_lexicon_tags_1 },
    { 0xde979710, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd79713, SUC_lexicon_tags_80 },
    { 0x9b979714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4579716, SUC_lexicon_tags_37 },
    { 0x6cb79717, SUC_lexicon_tags_17 },
    { 0x4dd79718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc077971f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x3f379721, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f779725, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d79729, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c37972f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad79733, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d179735, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba979739, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc379741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3379746, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed7974b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b7974e, SUC_lexicon_tags_11 },
    { 0x3e17974f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f779751, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd6979753, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84979755, SUC_lexicon_tags_25 },
    { 0x97f79755, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d7975d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdc97975f, SUC_lexicon_tags_29 },
    { 0x47779760, SUC_lexicon_tags_30 },
    { 0x9f979761, SUC_lexicon_tags_32 },
    { 0x6c379761, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11579769, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d37976e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a779775, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f979779, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b779783, SUC_lexicon_tags_8 },
    { 0x9a379784, SUC_lexicon_tags_37 },
    { 0x69779785, SUC_lexicon_tags_15 },
    { 0xd5b79785, SUC_lexicon_tags_49 },
    { 0x4b179786, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7978e, SUC_lexicon_tags_15 },
    { 0x5b37978f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a779791, SUC_lexicon_tags_56 },
    { 0xce579792, SUC_lexicon_tags_54 },
    { 0x12b79791, SUC_lexicon_tags_17 },
    { 0x88b79794, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b7979d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9797a0, SUC_lexicon_tags_47 },
    { 0x9e5797a1, SUC_lexicon_tags_0 },
    { 0xa5797a2, SUC_lexicon_tags_32 },
    { 0x90b797a3, SUC_lexicon_tags_11 },
    { 0x803797a3, SUC_lexicon_tags_17 },
    { 0x9a5797a5, SUC_lexicon_tags_12 },
    { 0xd27797a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff797a9, SUC_lexicon_tags_5 },
    { 0xcad797aa, SUC_lexicon_tags_47 },
    { 0x77797aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f797af, SUC_lexicon_tags_22 },
    { 0xe3797b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d3797b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f797b5, SUC_lexicon_tags_15 },
    { 0x27797b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1797b9, SUC_lexicon_tags_25 },
    { 0xca9797b9, SUC_lexicon_tags_11 },
    { 0x5fb797ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd797be, SUC_lexicon_tags_17 },
    { 0x399797be, SUC_lexicon_tags_2 },
    { 0x3eb797c0, SUC_lexicon_tags_27 },
    { 0x2ad797be, SUC_lexicon_tags_21 },
    { 0xa9f797c0, SUC_lexicon_tags_0 },
    { 0xbe3797c3, SUC_lexicon_tags_7 },
    { 0xbdb797c3, SUC_lexicon_tags_8 },
    { 0x795797c4, SUC_lexicon_tags_14 },
    { 0x2ad797c6, SUC_lexicon_tags_646 },
    { 0x771797c7, SUC_lexicon_tags_19 },
    { 0xf33797c6, SUC_lexicon_tags_25 },
    { 0x533797c1, SUC_lexicon_tags_37 },
    { 0xeaf797c1, SUC_lexicon_tags_23 },
    { 0xe33797c5, SUC_lexicon_tags_57 },
    { 0x189797c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d797d3, SUC_lexicon_tags_33 },
    { 0x683797d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf797dc, SUC_lexicon_tags_2 },
    { 0xcdf797dd, SUC_lexicon_tags_49 },
    { 0x46d797de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x805797e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x175797e3, SUC_lexicon_tags_23 },
    { 0x8d1797e3, SUC_lexicon_tags_7 },
    { 0xd17797e5, SUC_lexicon_tags_19 },
    { 0x84b797e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x681797e9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467797ef, SUC_lexicon_tags_15 },
    { 0xa09797ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb17797f2, SUC_lexicon_tags_16 },
    { 0x293797f3, SUC_lexicon_tags_6 },
    { 0xf95797f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d797fb, SUC_lexicon_tags_28 },
    { 0x9c1797fc, SUC_lexicon_tags_1 },
    { 0xfcf797fd, SUC_lexicon_tags_27 },
    { 0x3c1797fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c779807, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a97980a, SUC_lexicon_tags_3 },
    { 0x437980a, SUC_lexicon_tags_11 },
    { 0xd117980c, SUC_lexicon_tags_15 },
    { 0xbc37980c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6797980f, SUC_lexicon_tags_60 },
    { 0x3eb79810, SUC_lexicon_tags_37 },
    { 0x9e379811, SUC_lexicon_tags_23 },
    { 0x60d7980f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x29379814, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19779818, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b7981b, SUC_lexicon_tags_45 },
    { 0x42f7981c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d37981f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a779828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc277982b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc717982d, SUC_lexicon_tags_23 },
    { 0x7b77982d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2979836, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c97983b, SUC_lexicon_tags_43 },
    { 0x7097983c, SUC_lexicon_tags_37 },
    { 0x6737983d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x11f7983f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea579841, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1a179843, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6979849, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3417984c, SUC_lexicon_tags_30 },
    { 0x8637984d, SUC_lexicon_tags_31 },
    { 0xa0d7984e, SUC_lexicon_tags_37 },
    { 0x87f7984f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18179852, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x39379855, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b79859, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f97985d, SUC_lexicon_tags_66 },
    { 0x9d17985e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79779860, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bd79862, SUC_lexicon_tags_39 },
    { 0x5e779863, SUC_lexicon_tags_580 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b7986a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe717986d, SUC_lexicon_tags_44 },
    { 0x2197986e, SUC_lexicon_tags_46 },
    { 0x36b7986e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67579875, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb779877, SUC_lexicon_tags_17 },
    { 0x46f79878, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f379883, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1637988a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b7988f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a179892, SUC_lexicon_tags_14 },
    { 0xfc379893, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b7989b, SUC_lexicon_tags_174 },
    { 0xb797989b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb7989f, SUC_lexicon_tags_15 },
    { 0x6ad798a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b798a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7798a5, SUC_lexicon_tags_25 },
    { 0x16d798a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb47798a8, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0xdeb798aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f798ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x299798b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d798b3, SUC_lexicon_tags_15 },
    { 0xd3f798b4, SUC_lexicon_tags_11 },
    { 0x6fb798b4, SUC_lexicon_tags_14 },
    { 0x851798b6, SUC_lexicon_tags_63 },
    { 0xe3798b5, SUC_lexicon_tags_62 },
    { 0xb4f798b8, SUC_lexicon_tags_65 },
    { 0xb3f798b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b798bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x831798be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f798c1, SUC_lexicon_tags_30 },
    { 0x7f798c2, SUC_lexicon_tags_25 },
    { 0x6ff798c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db798c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7798cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9798cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b5798d1, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0x293798d3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b798d6, SUC_lexicon_tags_37 },
    { 0x973798d7, SUC_lexicon_tags_2 },
    { 0x521798d6, SUC_lexicon_tags_77 },
    { 0x4d7798d7, SUC_lexicon_tags_9 },
    { 0xe89798d9, SUC_lexicon_tags_60 },
    { 0x1b3798d8, SUC_lexicon_tags_14 },
    { 0xa21798d7, SUC_lexicon_tags_7 },
    { 0xa47798d8, SUC_lexicon_tags_0 },
    { 0x99798de, SUC_lexicon_tags_12 },
    { 0x5df798d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b5798e1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x28d798e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78d798e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaad798e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x421798e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d798ec, SUC_lexicon_tags_68 },
    { 0xffd798ec, SUC_lexicon_tags_8 },
    { 0x709798ee, SUC_lexicon_tags_37 },
    { 0xa17798ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d798f2, SUC_lexicon_tags_2 },
    { 0xb13798f3, SUC_lexicon_tags_63 },
    { 0xdb1798f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4e9798f6, SUC_lexicon_tags_39 },
    { 0xc6d798f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeed798f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf798fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x809798ff, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0x9fb79901, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d79905, SUC_lexicon_tags_66 },
    { 0xf4179906, SUC_lexicon_tags_12 },
    { 0xfdf79907, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2897990c, SUC_lexicon_tags_21 },
    { 0x3b7990d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c37990f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x74179912, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c579919, SUC_lexicon_tags_37 },
    { 0x23f7991a, SUC_lexicon_tags_16 },
    { 0x3fd7991b, SUC_lexicon_tags_8 },
    { 0x82b79919, SUC_lexicon_tags_8 },
    { 0xa0d7991c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8037991f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b179921, SUC_lexicon_tags_17 },
    { 0xe6179921, SUC_lexicon_tags_27 },
    { 0xbd979923, SUC_lexicon_tags_60 },
    { 0xabd79922, SUC_lexicon_tags_39 },
    { 0xa6b79924, SUC_lexicon_tags_26 },
    { 0x1a579926, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1437992a, SUC_lexicon_tags_60 },
    { 0xc7f7992b, SUC_lexicon_tags_8 },
    { 0x8737992c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3379930, SUC_lexicon_tags_7 },
    { 0x27b79930, SUC_lexicon_tags_74 },
    { 0x52579932, SUC_lexicon_tags_47 },
    { 0x21579933, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26179939, SUC_lexicon_tags_56 },
    { 0x3bb7993a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xff97993c, SUC_lexicon_tags_17 },
    { 0x5777993d, SUC_lexicon_tags_17 },
    { 0x73d7993d, SUC_lexicon_tags_12 },
    { 0xe8f7993f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a579943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2979945, SUC_lexicon_tags_37 },
    { 0xebb79946, SUC_lexicon_tags_17 },
    { 0x44379947, SUC_lexicon_tags_26 },
    { 0x1cb79947, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c17994d, SUC_lexicon_tags_27 },
    { 0x3fb7994d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f79951, SUC_lexicon_tags_161 },
    { 0xb7579952, SUC_lexicon_tags_49 },
    { 0xf5979953, SUC_lexicon_tags_2 },
    { 0x55b79954, SUC_lexicon_tags_17 },
    { 0xeed79953, SUC_lexicon_tags_22 },
    { 0x83f79956, SUC_lexicon_tags_13 },
    { 0x5c779952, SUC_lexicon_tags_15 },
    { 0x63379956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b7995c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf79960, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc379962, SUC_lexicon_tags_49 },
    { 0x82379963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34d79965, SUC_lexicon_tags_21 },
    { 0xbb579966, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85979969, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d7996d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9579978, SUC_lexicon_tags_87 },
    { 0x61b79979, SUC_lexicon_tags_11 },
    { 0xd777997a, SUC_lexicon_tags_11 },
    { 0x6e57997b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b7997e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x60779980, SUC_lexicon_tags_1 },
    { 0xbe779981, SUC_lexicon_tags_21 },
    { 0x73d79982, SUC_lexicon_tags_44 },
    { 0x76579980, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe497998a, SUC_lexicon_tags_2 },
    { 0x1397998b, SUC_lexicon_tags_37 },
    { 0x317998a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff79991, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50179996, SUC_lexicon_tags_27 },
    { 0x86379997, SUC_lexicon_tags_15 },
    { 0x60379996, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135799a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795799a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f799ab, SUC_lexicon_tags_39 },
    { 0xeef799ac, SUC_lexicon_tags_33 },
    { 0x995799ac, SUC_lexicon_tags_6 },
    { 0x3df799ad, SUC_lexicon_tags_7 },
    { 0x10b799ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x389799b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f799bc, SUC_lexicon_tags_25 },
    { 0x31799bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x939799bf, SUC_lexicon_tags_1 },
    { 0x555799c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe41799c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf2b799c4, SUC_lexicon_tags_17 },
    { 0x6b799c4, SUC_lexicon_tags_19 },
    { 0xad3799c6, SUC_lexicon_tags_17 },
    { 0x3f7799c4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x89b799c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1799cc, SUC_lexicon_tags_65 },
    { 0x237799cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2b799cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cf799d1, SUC_lexicon_tags_0 },
    { 0xf47799d2, SUC_lexicon_tags_2 },
    { 0x953799d3, SUC_lexicon_tags_11 },
    { 0x3db799d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5d799d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff799da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1799e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf53799e5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x599799e7, SUC_lexicon_tags_17 },
    { 0x1bb799e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd799eb, SUC_lexicon_tags_10 },
    { 0x7a5799ec, SUC_lexicon_tags_11 },
    { 0x543799ed, SUC_lexicon_tags_42 },
    { 0x82d799ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1799f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf7799f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x559799fb, SUC_lexicon_tags_25 },
    { 0xfd3799fc, SUC_lexicon_tags_12 },
    { 0x679799fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff79a00, SUC_lexicon_tags_33 },
    { 0x69579a00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf79a04, SUC_lexicon_tags_15 },
    { 0x779a05, SUC_lexicon_tags_14 },
    { 0xf0f79a06, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc2579a08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b79a0c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x61979a0e, SUC_lexicon_tags_25 },
    { 0xea179a0f, SUC_lexicon_tags_2 },
    { 0xca579a10, SUC_lexicon_tags_27 },
    { 0x44779a11, SUC_lexicon_tags_1 },
    { 0x77179a11, SUC_lexicon_tags_49 },
    { 0x70579a12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79a17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68b79a1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd179a1d, SUC_lexicon_tags_56 },
    { 0xd7779a1e, SUC_lexicon_tags_15 },
    { 0xf2179a1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab79a27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x29d79a29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3779a2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x33979a2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc579a31, SUC_lexicon_tags_0 },
    { 0x4b179a32, SUC_lexicon_tags_15 },
    { 0x70f79a33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d379a38, SUC_lexicon_tags_17 },
    { 0x11379a39, SUC_lexicon_tags_15 },
    { 0xe3979a3a, SUC_lexicon_tags_12 },
    { 0xb0179a3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30779a42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd79a47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x10379a4a, SUC_lexicon_tags_27 },
    { 0x92b79a4b, SUC_lexicon_tags_28 },
    { 0xb79a4c, SUC_lexicon_tags_0 },
    { 0x51b79a4a, SUC_lexicon_tags_32 },
    { 0xe5979a4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20b79a50, SUC_lexicon_tags_25 },
    { 0x5eb79a51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2379a53, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f79a57, SUC_lexicon_tags_23 },
    { 0xc7979a58, SUC_lexicon_tags_37 },
    { 0x8c379a59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97979a5b, SUC_lexicon_tags_53 },
    { 0x86379a5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1bf79a5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6cf79a60, SUC_lexicon_tags_1 },
    { 0xa8179a60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d79a65, SUC_lexicon_tags_46 },
    { 0x9a179a66, SUC_lexicon_tags_17 },
    { 0x85579a67, SUC_lexicon_tags_34 },
    { 0x9a179a68, SUC_lexicon_tags_48 },
    { 0x50d79a67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85579a6e, SUC_lexicon_tags_37 },
    { 0x9c979a6f, SUC_lexicon_tags_24 },
    { 0xb3779a6f, SUC_lexicon_tags_26 },
    { 0x98379a6f, SUC_lexicon_tags_11 },
    { 0xbc779a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80b79a74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56979a78, SUC_lexicon_tags_60 },
    { 0x1179a78, SUC_lexicon_tags_23 },
    { 0xb9779a7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb79a7e, SUC_lexicon_tags_63 },
    { 0x98d79a7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed79a84, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x21379a87, SUC_lexicon_tags_37 },
    { 0x5bf79a88, SUC_lexicon_tags_96 },
    { 0xeb179a89, SUC_lexicon_tags_16 },
    { 0xf0579a89, SUC_lexicon_tags_22 },
    { 0xe2179a89, SUC_lexicon_tags_7 },
    { 0x2b379a8c, SUC_lexicon_tags_25 },
    { 0x14b79a8d, SUC_lexicon_tags_17 },
    { 0x68f79a8e, SUC_lexicon_tags_7 },
    { 0xf0d79a88, SUC_lexicon_tags_49 },
    { 0xcb179a8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x66779a92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f79a95, SUC_lexicon_tags_33 },
    { 0xf6579a96, SUC_lexicon_tags_62 },
    { 0x8e579a95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a579a9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b79aa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc979aac, SUC_lexicon_tags_42 },
    { 0x74579aad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb379aaf, SUC_lexicon_tags_1 },
    { 0xe7579aaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55579ab4, SUC_lexicon_tags_25 },
    { 0xc8f79ab5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x22179ab8, SUC_lexicon_tags_17 },
    { 0xc7979ab8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c779ac0, SUC_lexicon_tags_27 },
    { 0x94d79ac0, SUC_lexicon_tags_14 },
    { 0xe2179ac2, SUC_lexicon_tags_55 },
    { 0x4cb79ac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8379ac6, SUC_lexicon_tags_47 },
    { 0xbad79ac7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x14979ac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f179acb, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf79ad2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40779ad6, SUC_lexicon_tags_65 },
    { 0x10b79ad6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b79ae0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45779ae3, SUC_lexicon_tags_2 },
    { 0x72979ae4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x42779ae6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f979aed, SUC_lexicon_tags_13 },
    { 0x35d79aed, SUC_lexicon_tags_160 },
    { 0x8b379aef, SUC_lexicon_tags_4 },
    { 0x3f179af0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f379af4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb579af7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30379af9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d379b06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda779b09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf3b79b0b, SUC_lexicon_tags_11 },
    { 0xf7379b0b, SUC_lexicon_tags_25 },
    { 0x4c379b0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9779b10, SUC_lexicon_tags_11 },
    { 0x9f79b10, SUC_lexicon_tags_2 },
    { 0xd5179b12, SUC_lexicon_tags_12 },
    { 0x31979b13, SUC_lexicon_tags_11 },
    { 0xf779b14, SUC_lexicon_tags_37 },
    { 0xaa179b14, SUC_lexicon_tags_26 },
    { 0xb3f79b16, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19979b1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f79b22, SUC_lexicon_tags_11 },
    { 0x64d79b22, SUC_lexicon_tags_13 },
    { 0xdeb79b23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7179b26, SUC_lexicon_tags_9 },
    { 0xaab79b27, SUC_lexicon_tags_17 },
    { 0xa7379b28, SUC_lexicon_tags_37 },
    { 0xf4379b29, SUC_lexicon_tags_33 },
    { 0x8d779b29, SUC_lexicon_tags_74 },
    { 0x61b79b29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf79b2e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x57d79b30, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f79b36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda579b43, SUC_lexicon_tags_37 },
    { 0xfe379b44, SUC_lexicon_tags_25 },
    { 0x40379b45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1379b49, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58579b51, SUC_lexicon_tags_9 },
    { 0x30579b52, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1579b58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f79b5e, SUC_lexicon_tags_6 },
    { 0x22779b5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f79b62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6979b67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b79b6b, SUC_lexicon_tags_47 },
    { 0xbc379b6c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x38d79b6e, SUC_lexicon_tags_66 },
    { 0xa7379b6f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f579b72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9579b80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0379b82, SUC_lexicon_tags_14 },
    { 0x1e579b82, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9779b87, SUC_lexicon_tags_60 },
    { 0x56579b88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7179b8a, SUC_lexicon_tags_29 },
    { 0x4c379b8a, SUC_lexicon_tags_7 },
    { 0x51779b8c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x58779b8f, SUC_lexicon_tags_16 },
    { 0xc5979b90, SUC_lexicon_tags_33 },
    { 0x2f979b90, SUC_lexicon_tags_158 },
    { 0xd1d79b91, SUC_lexicon_tags_46 },
    { 0x77979b8f, SUC_lexicon_tags_37 },
    { 0xaa379b94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25779b99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21579b9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f79ba2, SUC_lexicon_tags_45 },
    { 0x29d79ba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81379ba7, SUC_lexicon_tags_17 },
    { 0x7c779ba8, SUC_lexicon_tags_49 },
    { 0xd5d79ba9, SUC_lexicon_tags_17 },
    { 0x37179ba7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7b779bac, SUC_lexicon_tags_39 },
    { 0xb3f79bad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bd79baf, SUC_lexicon_tags_2 },
    { 0x8f79bb0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7779bb4, SUC_lexicon_tags_47 },
    { 0x92b79bb5, SUC_lexicon_tags_17 },
    { 0x56b79bb5, SUC_lexicon_tags_25 },
    { 0x7a579bb7, SUC_lexicon_tags_87 },
    { 0xe2179bb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79bc1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef79bca, SUC_lexicon_tags_7 },
    { 0x7a379bcb, SUC_lexicon_tags_23 },
    { 0xb6579bcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d179bd3, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa579bd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x58579bda, SUC_lexicon_tags_10 },
    { 0x83379bda, SUC_lexicon_tags_30 },
    { 0xd5579bda, SUC_lexicon_tags_53 },
    { 0x67979bdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb979bdf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89979bec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46379bf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c579bf3, SUC_lexicon_tags_23 },
    { 0xe1979bf4, SUC_lexicon_tags_12 },
    { 0x1f79bf4, SUC_lexicon_tags_2 },
    { 0xbd79bf6, SUC_lexicon_tags_39 },
    { 0x3c779bf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb979bfa, SUC_lexicon_tags_55 },
    { 0xf2579bfa, SUC_lexicon_tags_27 },
    { 0xfc779bfc, SUC_lexicon_tags_15 },
    { 0xb8379bfb, SUC_lexicon_tags_20 },
    { 0xedf79bfd, SUC_lexicon_tags_37 },
    { 0x9cf79bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd979c01, SUC_lexicon_tags_17 },
    { 0x82d79c02, SUC_lexicon_tags_37 },
    { 0x5f779c02, SUC_lexicon_tags_27 },
    { 0xf8979c04, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7979c07, SUC_lexicon_tags_11 },
    { 0xd4379c08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ad79c0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf79c0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2b79c0f, SUC_lexicon_tags_23 },
    { 0x85779c0f, SUC_lexicon_tags_15 },
    { 0xc579c11, SUC_lexicon_tags_10 },
    { 0xf6f79c12, SUC_lexicon_tags_12 },
    { 0xb0379c10, SUC_lexicon_tags_55 },
    { 0x26d79c13, SUC_lexicon_tags_10 },
    { 0xe4579c15, SUC_lexicon_tags_11 },
    { 0x30379c16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8779c19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18179c20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8a179c22, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6979c26, SUC_lexicon_tags_28 },
    { 0xa7b79c27, SUC_lexicon_tags_37 },
    { 0x35579c28, SUC_lexicon_tags_8 },
    { 0x7579c28, SUC_lexicon_tags_25 },
    { 0xb5b79c2a, SUC_lexicon_tags_31 },
    { 0x94d79c2b, SUC_lexicon_tags_23 },
    { 0xb1979c2b, SUC_lexicon_tags_36 },
    { 0xdef79c28, SUC_lexicon_tags_63 },
    { 0x68579c28, SUC_lexicon_tags_11 },
    { 0x45d79c2c, SUC_lexicon_tags_45 },
    { 0xc6579c2a, SUC_lexicon_tags_2 },
    { 0x99179c31, SUC_lexicon_tags_85 },
    { 0x76979c31, SUC_lexicon_tags_146 },
    { 0x16b79c31, SUC_lexicon_tags_33 },
    { 0xe6979c34, SUC_lexicon_tags_32 },
    { 0xec779c35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3379c39, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179c49, SUC_lexicon_tags_66 },
    { 0x6bd79c4a, SUC_lexicon_tags_25 },
    { 0x9d79c4b, SUC_lexicon_tags_25 },
    { 0x24b79c4c, SUC_lexicon_tags_11 },
    { 0xfdb79c4d, SUC_lexicon_tags_45 },
    { 0x15d79c4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3779c52, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3179c57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e179c59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b79c5d, SUC_lexicon_tags_25 },
    { 0x5b979c5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xebf79c60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f79c68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xae579c6b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xed779c6d, SUC_lexicon_tags_45 },
    { 0x92f79c6d, SUC_lexicon_tags_31 },
    { 0xfd179c6f, SUC_lexicon_tags_37 },
    { 0x89579c70, SUC_lexicon_tags_26 },
    { 0x86179c6f, SUC_lexicon_tags_31 },
    { 0x7b979c6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xae579c75, SUC_lexicon_tags_7 },
    { 0xd8579c76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29179c78, SUC_lexicon_tags_7 },
    { 0x96979c78, SUC_lexicon_tags_63 },
    { 0x2c579c7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d79c7f, SUC_lexicon_tags_31 },
    { 0xa1b79c80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c379c8a, SUC_lexicon_tags_2 },
    { 0x1d179c8b, SUC_lexicon_tags_25 },
    { 0x4579c8b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x70f79c8e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbb579c90, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x3e779c92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1d79c94, SUC_lexicon_tags_38 },
    { 0x5c779c95, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3a579c97, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3579c9b, SUC_lexicon_tags_30 },
    { 0xdf379c9c, SUC_lexicon_tags_8 },
    { 0xcad79c9d, SUC_lexicon_tags_37 },
    { 0x44b79c9e, SUC_lexicon_tags_49 },
    { 0x83d79c9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb79ca7, SUC_lexicon_tags_37 },
    { 0xe0779ca8, SUC_lexicon_tags_74 },
    { 0xc979ca7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b79cac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x45d79cae, SUC_lexicon_tags_37 },
    { 0x69579cae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58379cb1, SUC_lexicon_tags_17 },
    { 0x59d79cb2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc0779cb4, SUC_lexicon_tags_25 },
    { 0xc8179cb4, SUC_lexicon_tags_47 },
    { 0xf5f79cb6, SUC_lexicon_tags_7 },
    { 0x17579cb7, SUC_lexicon_tags_17 },
    { 0x79379cb7, SUC_lexicon_tags_11 },
    { 0x9f79cb9, SUC_lexicon_tags_17 },
    { 0xc0979cba, SUC_lexicon_tags_16 },
    { 0xc6f79cb9, SUC_lexicon_tags_15 },
    { 0x66179cb4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd79cc1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f79cc9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x65379ccc, SUC_lexicon_tags_63 },
    { 0x49979ccd, SUC_lexicon_tags_37 },
    { 0xa1b79cce, SUC_lexicon_tags_25 },
    { 0x96379ccc, SUC_lexicon_tags_2 },
    { 0x11179cd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb79cd4, SUC_lexicon_tags_1 },
    { 0x68779cd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x38379cd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb79cdb, SUC_lexicon_tags_12 },
    { 0x93d79cdb, SUC_lexicon_tags_43 },
    { 0x87979cdb, SUC_lexicon_tags_21 },
    { 0x41579cde, SUC_lexicon_tags_12 },
    { 0xa9579cde, SUC_lexicon_tags_1 },
    { 0xbb379cdd, SUC_lexicon_tags_6 },
    { 0x2d379ce1, SUC_lexicon_tags_45 },
    { 0x71f79ce0, SUC_lexicon_tags_23 },
    { 0x50379ce2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d179ce9, SUC_lexicon_tags_66 },
    { 0xa979cea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed579cec, SUC_lexicon_tags_11 },
    { 0xb5979ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88979cf0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b79cf6, SUC_lexicon_tags_63 },
    { 0x54b79cf6, SUC_lexicon_tags_70 },
    { 0xd5779cf6, SUC_lexicon_tags_63 },
    { 0x98579cf6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12779cfe, SUC_lexicon_tags_11 },
    { 0x1b379cff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f79d03, SUC_lexicon_tags_37 },
    { 0xc4d79d03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f179d08, SUC_lexicon_tags_11 },
    { 0x4a779d09, SUC_lexicon_tags_25 },
    { 0x65979d0a, SUC_lexicon_tags_15 },
    { 0x85179d0b, SUC_lexicon_tags_45 },
    { 0x98379d0c, SUC_lexicon_tags_25 },
    { 0xeff79d0d, SUC_lexicon_tags_6 },
    { 0x53779d0e, SUC_lexicon_tags_27 },
    { 0xe2379d0f, SUC_lexicon_tags_15 },
    { 0x54579d10, SUC_lexicon_tags_30 },
    { 0xa7f79d11, SUC_lexicon_tags_1 },
    { 0x59179d12, SUC_lexicon_tags_37 },
    { 0xc6579d13, SUC_lexicon_tags_21 },
    { 0x89579d11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7179d16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce779d1a, SUC_lexicon_tags_25 },
    { 0x22179d1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67579d20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb79d23, SUC_lexicon_tags_9 },
    { 0xdc179d24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b79d2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6d79d2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf0d79d2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdff79d30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3779d33, SUC_lexicon_tags_120 },
    { 0x66f79d34, SUC_lexicon_tags_68 },
    { 0x7df79d35, SUC_lexicon_tags_18 },
    { 0xdbd79d35, SUC_lexicon_tags_343 },
    { 0x4979d37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d79d3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b79d43, SUC_lexicon_tags_17 },
    { 0xccd79d44, SUC_lexicon_tags_31 },
    { 0xadf79d44, SUC_lexicon_tags_25 },
    { 0xd7f79d46, SUC_lexicon_tags_44 },
    { 0xd1179d47, SUC_lexicon_tags_14 },
    { 0xc5f79d48, SUC_lexicon_tags_17 },
    { 0x85379d43, SUC_lexicon_tags_23 },
    { 0x179d4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b79d4d, SUC_lexicon_tags_38 },
    { 0x45579d4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd0779d50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ff79d52, SUC_lexicon_tags_11 },
    { 0xc5979d53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed79d57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78179d5a, SUC_lexicon_tags_11 },
    { 0xd9179d5b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x23779d5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3379d60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82779d62, SUC_lexicon_tags_85 },
    { 0xb4379d63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28979d68, SUC_lexicon_tags_47 },
    { 0x6b379d68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14d79d6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22579d6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88579d71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6179d79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d79d7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef579d80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13579d84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e579d8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b579d8c, SUC_lexicon_tags_28 },
    { 0x3e779d8d, SUC_lexicon_tags_17 },
    { 0x7c379d8e, SUC_lexicon_tags_12 },
    { 0xd3f79d8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1179d94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d379d97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b79d9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0f79d9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc8179da0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e379da7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b79daa, SUC_lexicon_tags_37 },
    { 0x29b79daa, SUC_lexicon_tags_11 },
    { 0x9f779dac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1979db0, SUC_lexicon_tags_108 },
    { 0x17f79db1, SUC_lexicon_tags_15 },
    { 0x75d79db1, SUC_lexicon_tags_39 },
    { 0xea779db3, SUC_lexicon_tags_25 },
    { 0x1c779db4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc579db8, SUC_lexicon_tags_25 },
    { 0xd6179db8, SUC_lexicon_tags_2 },
    { 0x5af79dba, SUC_lexicon_tags_11 },
    { 0xe5179db8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48979dbd, SUC_lexicon_tags_64 },
    { 0x67779dbe, SUC_lexicon_tags_37 },
    { 0x69d79dbf, SUC_lexicon_tags_37 },
    { 0xa2b79dbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86579dc6, SUC_lexicon_tags_27 },
    { 0x12579dc6, SUC_lexicon_tags_172 },
    { 0x3b179dc8, SUC_lexicon_tags_14 },
    { 0x86f79dc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe6b79dcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4379dcd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb79dd1, SUC_lexicon_tags_9 },
    { 0x58779dd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf979dd6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1979ddc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af79de3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd79de8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb79df5, SUC_lexicon_tags_28 },
    { 0x17d79df6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb979df9, SUC_lexicon_tags_0 },
    { 0xb5979dfa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e779e04, SUC_lexicon_tags_12 },
    { 0xc0779e05, SUC_lexicon_tags_0 },
    { 0xc4979e05, SUC_lexicon_tags_37 },
    { 0x67f79e05, SUC_lexicon_tags_31 },
    { 0xabb79e05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66d79e0a, SUC_lexicon_tags_1 },
    { 0x19179e0b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63779e0f, SUC_lexicon_tags_8 },
    { 0x64f79e0f, SUC_lexicon_tags_3 },
    { 0x80179e0f, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf79e14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92179e1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7d79e1f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x51f79e21, SUC_lexicon_tags_32 },
    { 0xa9d79e22, SUC_lexicon_tags_84 },
    { 0x49b79e22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b79e27, SUC_lexicon_tags_124 },
    { 0x86579e28, SUC_lexicon_tags_65 },
    { 0x65179e27, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21179e2d, SUC_lexicon_tags_30 },
    { 0x38979e2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb79e31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4979e34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1b79e36, SUC_lexicon_tags_37 },
    { 0x1d779e37, SUC_lexicon_tags_5 },
    { 0x19d79e37, SUC_lexicon_tags_95 },
    { 0x30379e38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x43579e3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb79e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa779e44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49579e47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71d79e49, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b79e4e, SUC_lexicon_tags_37 },
    { 0x90379e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f779e55, SUC_lexicon_tags_37 },
    { 0x90b79e56, SUC_lexicon_tags_33 },
    { 0x79b79e56, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x45579e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bf79e5b, SUC_lexicon_tags_15 },
    { 0xca579e5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x56f79e5f, SUC_lexicon_tags_27 },
    { 0x7ed79e5f, SUC_lexicon_tags_1 },
    { 0xe4179e61, SUC_lexicon_tags_17 },
    { 0x99779e62, SUC_lexicon_tags_2 },
    { 0x2fd79e62, SUC_lexicon_tags_12 },
    { 0xe8579e60, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x17d79e66, SUC_lexicon_tags_37 },
    { 0x7b79e67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d79e6a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d79e6d, SUC_lexicon_tags_55 },
    { 0xb1d79e6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61379e77, SUC_lexicon_tags_25 },
    { 0x82f79e77, SUC_lexicon_tags_12 },
    { 0x95b79e79, SUC_lexicon_tags_33 },
    { 0xab79e79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd79e85, SUC_lexicon_tags_37 },
    { 0xba979e86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e579e88, SUC_lexicon_tags_45 },
    { 0x8f79e89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e979e8d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42379e91, SUC_lexicon_tags_37 },
    { 0xc9779e91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b79e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b779e9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6579e9f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f79ea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c979ea8, SUC_lexicon_tags_60 },
    { 0x1fd79ea8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdd979eab, SUC_lexicon_tags_29 },
    { 0x2e979eac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f579eb0, SUC_lexicon_tags_13 },
    { 0x87b79eb1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe579eb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80579eba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1979ebd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c179ec3, SUC_lexicon_tags_35 },
    { 0xabb79ec4, SUC_lexicon_tags_15 },
    { 0x86379ec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf79ec7, SUC_lexicon_tags_55 },
    { 0x3e179ec8, SUC_lexicon_tags_12 },
    { 0xf8d79ec7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce779ecb, SUC_lexicon_tags_27 },
    { 0xd0579ecb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31379ed5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab79edd, SUC_lexicon_tags_15 },
    { 0xc3f79ede, SUC_lexicon_tags_25 },
    { 0xeb379edf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa179ee2, SUC_lexicon_tags_27 },
    { 0xd5979ee3, SUC_lexicon_tags_7 },
    { 0x45779ee4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x94579ee7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xae779eea, SUC_lexicon_tags_14 },
    { 0xa4f79eeb, SUC_lexicon_tags_14 },
    { 0x52579eec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35579ef2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe379efa, SUC_lexicon_tags_31 },
    { 0x62379efb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1579f03, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x15779f05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab79f08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73979f0c, SUC_lexicon_tags_25 },
    { 0xfad79f0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8eb79f0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15979f18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f779f1b, SUC_lexicon_tags_37 },
    { 0xd7179f1c, SUC_lexicon_tags_27 },
    { 0x7cd79f1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x87179f20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93179f23, SUC_lexicon_tags_27 },
    { 0xfb379f24, SUC_lexicon_tags_49 },
    { 0xb8179f24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa379f29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2179f2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96579f30, SUC_lexicon_tags_56 },
    { 0xbed79f31, SUC_lexicon_tags_15 },
    { 0xc3579f32, SUC_lexicon_tags_47 },
    { 0xd3379f31, SUC_lexicon_tags_10 },
    { 0xc2979f34, SUC_lexicon_tags_8 },
    { 0xa4179f32, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa9579f37, SUC_lexicon_tags_70 },
    { 0x5179f38, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x93379f3b, SUC_lexicon_tags_11 },
    { 0x58d79f3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f379f3e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71979f47, SUC_lexicon_tags_37 },
    { 0xc4d79f48, SUC_lexicon_tags_38 },
    { 0x39179f47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb79f53, SUC_lexicon_tags_17 },
    { 0xb5d79f54, SUC_lexicon_tags_2 },
    { 0xaa579f55, SUC_lexicon_tags_17 },
    { 0x94d79f56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49379f58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b79f60, SUC_lexicon_tags_10 },
    { 0x76179f61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df79f68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d79f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd79f76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8979f79, SUC_lexicon_tags_14 },
    { 0xb7b79f7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9179f80, SUC_lexicon_tags_8 },
    { 0x12579f81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ad79f83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27979f8a, SUC_lexicon_tags_14 },
    { 0xec579f8b, SUC_lexicon_tags_30 },
    { 0xfeb79f8a, SUC_lexicon_tags_120 },
    { 0x7b179f8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82d79f8f, SUC_lexicon_tags_12 },
    { 0xd7579f90, SUC_lexicon_tags_18 },
    { 0x73179f90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b79f99, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb5579f9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e179f9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9d379fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff79fa5, SUC_lexicon_tags_35 },
    { 0x3d779fa6, SUC_lexicon_tags_39 },
    { 0xa4b79fa7, SUC_lexicon_tags_17 },
    { 0xecf79fa8, SUC_lexicon_tags_15 },
    { 0xecb79fa7, SUC_lexicon_tags_47 },
    { 0x19579faa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab79fae, SUC_lexicon_tags_30 },
    { 0x7a779fae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce579fb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe0179fb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd79fb6, SUC_lexicon_tags_25 },
    { 0x7a979fb7, SUC_lexicon_tags_2 },
    { 0x15979fb7, SUC_lexicon_tags_12 },
    { 0x85d79fb9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x89379fbb, SUC_lexicon_tags_10 },
    { 0x22179fbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87779fbf, SUC_lexicon_tags_70 },
    { 0xb9779fc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e179fc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b79fca, SUC_lexicon_tags_2 },
    { 0xbe179fcb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x59979fcd, SUC_lexicon_tags_12 },
    { 0x74b79fce, SUC_lexicon_tags_47 },
    { 0x41579fce, SUC_lexicon_tags_50 },
    { 0x22f79fd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a779fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea179fd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e379fda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c179fdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed79fdf, SUC_lexicon_tags_136 },
    { 0xccb79fe0, SUC_lexicon_tags_43 },
    { 0xc4379fdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd779fe6, SUC_lexicon_tags_12 },
    { 0x8d79fe7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51979feb, SUC_lexicon_tags_25 },
    { 0x94179fec, SUC_lexicon_tags_2 },
    { 0xa1179feb, SUC_lexicon_tags_1 },
    { 0x88979fee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a179ff0, SUC_lexicon_tags_26 },
    { 0xb4979ff0, SUC_lexicon_tags_7 },
    { 0x15d79ff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2379ff8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d7a001, SUC_lexicon_tags_17 },
    { 0x12d7a002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d7a005, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d77a008, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3357a00d, SUC_lexicon_tags_2 },
    { 0x39d7a00e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d7a011, SUC_lexicon_tags_17 },
    { 0xf857a012, SUC_lexicon_tags_1 },
    { 0x1df7a013, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e57a015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af7a018, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d7a01e, SUC_lexicon_tags_30 },
    { 0xe9d7a01f, SUC_lexicon_tags_10 },
    { 0xefb7a020, SUC_lexicon_tags_79 },
    { 0x32b7a01f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc37a027, SUC_lexicon_tags_25 },
    { 0x2957a028, SUC_lexicon_tags_37 },
    { 0x6757a029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb57a02c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a77a030, SUC_lexicon_tags_8 },
    { 0xa457a030, SUC_lexicon_tags_33 },
    { 0xeeb7a032, SUC_lexicon_tags_1 },
    { 0x3697a033, SUC_lexicon_tags_2 },
    { 0x1157a034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa317a036, SUC_lexicon_tags_1 },
    { 0x1f17a036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b7a03a, SUC_lexicon_tags_63 },
    { 0xb397a03b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a77a040, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5957a045, SUC_lexicon_tags_17 },
    { 0x9917a046, SUC_lexicon_tags_45 },
    { 0xe5f7a045, SUC_lexicon_tags_15 },
    { 0x1937a048, SUC_lexicon_tags_151 },
    { 0x4f17a049, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6f7a04b, SUC_lexicon_tags_14 },
    { 0x8b77a04c, SUC_lexicon_tags_15 },
    { 0x5437a04d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcd37a04f, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf7a056, SUC_lexicon_tags_44 },
    { 0x2c97a057, SUC_lexicon_tags_15 },
    { 0x18f7a058, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd9b7a05a, SUC_lexicon_tags_37 },
    { 0xbc57a05b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2417a05e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6017a060, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d7a063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ed7a065, SUC_lexicon_tags_38 },
    { 0x17b7a065, SUC_lexicon_tags_4 },
    { 0x7ff7a065, SUC_lexicon_tags_2 },
    { 0x2e97a068, SUC_lexicon_tags_25 },
    { 0xe9b7a069, SUC_lexicon_tags_11 },
    { 0xedf7a066, SUC_lexicon_tags_1 },
    { 0x2b17a06b, SUC_lexicon_tags_23 },
    { 0xc0d7a06c, SUC_lexicon_tags_14 },
    { 0x957a06d, SUC_lexicon_tags_87 },
    { 0xc777a06b, SUC_lexicon_tags_68 },
    { 0x89b7a06a, SUC_lexicon_tags_17 },
    { 0x1837a070, SUC_lexicon_tags_33 },
    { 0xb0f7a071, SUC_lexicon_tags_38 },
    { 0xfe57a071, SUC_lexicon_tags_37 },
    { 0xb7d7a073, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x2d7a075, SUC_lexicon_tags_38 },
    { 0x2cf7a075, SUC_lexicon_tags_31 },
    { 0xe4b7a077, SUC_lexicon_tags_63 },
    { 0x6517a078, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d7a07d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4897a082, SUC_lexicon_tags_37 },
    { 0x9b17a082, SUC_lexicon_tags_2 },
    { 0xefd7a084, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2037a08b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x417a08d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x917a08f, SUC_lexicon_tags_25 },
    { 0x6cf7a090, SUC_lexicon_tags_11 },
    { 0xa7d7a091, SUC_lexicon_tags_14 },
    { 0xa417a092, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0xe757a095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35f7a097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd677a099, SUC_lexicon_tags_10 },
    { 0x5377a09a, SUC_lexicon_tags_12 },
    { 0x2597a09b, SUC_lexicon_tags_28 },
    { 0x75d7a09a, SUC_lexicon_tags_25 },
    { 0x5ed7a09d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xda17a09f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b7a0a2, SUC_lexicon_tags_33 },
    { 0xaff7a0a3, SUC_lexicon_tags_17 },
    { 0x6c17a0a2, SUC_lexicon_tags_49 },
    { 0xbbd7a0a5, SUC_lexicon_tags_33 },
    { 0xabd7a0a6, SUC_lexicon_tags_7 },
    { 0x62b7a0a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92d7a0a9, SUC_lexicon_tags_17 },
    { 0xe097a0aa, SUC_lexicon_tags_4 },
    { 0x3e97a0aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb77a0ad, SUC_lexicon_tags_37 },
    { 0x4977a0ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2a17a0b0, SUC_lexicon_tags_14 },
    { 0x4d77a0b1, SUC_lexicon_tags_41 },
    { 0x11f7a0b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe097a0b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87f7a0b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757a0bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8fb7a0bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec97a0c9, SUC_lexicon_tags_15 },
    { 0x857a0ca, SUC_lexicon_tags_218 },
    { 0x3c77a0cb, SUC_lexicon_tags_17 },
    { 0xf337a0cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3837a0cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f7a0d3, SUC_lexicon_tags_45 },
    { 0x4257a0d3, SUC_lexicon_tags_37 },
    { 0xd0f7a0d3, SUC_lexicon_tags_63 },
    { 0x6bf7a0d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf17a0d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b37a0db, SUC_lexicon_tags_1 },
    { 0xfd17a0dc, SUC_lexicon_tags_79 },
    { 0x8e17a0dc, SUC_lexicon_tags_31 },
    { 0xed7a0de, SUC_lexicon_tags_17 },
    { 0x2f37a0de, SUC_lexicon_tags_0 },
    { 0xcf7a0e0, SUC_lexicon_tags_49 },
    { 0x9d17a0e0, SUC_lexicon_tags_11 },
    { 0x6357a0df, SUC_lexicon_tags_17 },
    { 0x4ab7a0e3, SUC_lexicon_tags_63 },
    { 0xd057a0e4, SUC_lexicon_tags_15 },
    { 0xa57a0e2, SUC_lexicon_tags_19 },
    { 0xbed7a0e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f7a0e9, SUC_lexicon_tags_37 },
    { 0xfb77a0ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5577a0ed, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f7a0f0, SUC_lexicon_tags_15 },
    { 0xdf7a0f1, SUC_lexicon_tags_11 },
    { 0xd5f7a0f2, SUC_lexicon_tags_17 },
    { 0xc577a0f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2397a0f6, SUC_lexicon_tags_45 },
    { 0xe037a0f7, SUC_lexicon_tags_118 },
    { 0x7ef7a0f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997a0fc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b7a101, SUC_lexicon_tags_12 },
    { 0x8597a102, SUC_lexicon_tags_26 },
    { 0xccf7a102, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd77a107, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f7a111, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4f77a113, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1757a117, SUC_lexicon_tags_1 },
    { 0xd977a118, SUC_lexicon_tags_13 },
    { 0xb097a119, SUC_lexicon_tags_11 },
    { 0xc537a119, SUC_lexicon_tags_33 },
    { 0x5997a11b, SUC_lexicon_tags_17 },
    { 0xad37a119, SUC_lexicon_tags_1 },
    { 0x62b7a11d, SUC_lexicon_tags_2 },
    { 0xddb7a119, SUC_lexicon_tags_28 },
    { 0x2b7a119, SUC_lexicon_tags_7 },
    { 0x7e17a120, SUC_lexicon_tags_25 },
    { 0xe557a121, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f7a124, SUC_lexicon_tags_8 },
    { 0xf557a125, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc957a12a, SUC_lexicon_tags_12 },
    { 0x9a97a12b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2517a12d, SUC_lexicon_tags_11 },
    { 0x6e37a12d, SUC_lexicon_tags_4 },
    { 0x1ff7a12d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab7a134, SUC_lexicon_tags_12 },
    { 0x5d7a135, SUC_lexicon_tags_57 },
    { 0xc1b7a136, SUC_lexicon_tags_46 },
    { 0x6517a137, SUC_lexicon_tags_45 },
    { 0x39f7a134, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc397a140, SUC_lexicon_tags_46 },
    { 0x2f97a141, SUC_lexicon_tags_10 },
    { 0xa337a142, SUC_lexicon_tags_60 },
    { 0x8eb7a140, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba77a14b, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x5217a14d, SUC_lexicon_tags_15 },
    { 0xd6f7a14e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7a153, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0xba97a155, SUC_lexicon_tags_17 },
    { 0x1cd7a156, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f7a15b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa757a15d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697a163, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf057a165, SUC_lexicon_tags_30 },
    { 0xf2d7a166, SUC_lexicon_tags_17 },
    { 0x8857a167, SUC_lexicon_tags_14 },
    { 0xa8f7a165, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1017a16a, SUC_lexicon_tags_26 },
    { 0xebf7a16b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d7a174, SUC_lexicon_tags_11 },
    { 0x8cf7a174, SUC_lexicon_tags_29 },
    { 0x7f77a175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf7a17d, SUC_lexicon_tags_26 },
    { 0xbdf7a17e, SUC_lexicon_tags_14 },
    { 0x7897a17e, SUC_lexicon_tags_85 },
    { 0xde97a180, SUC_lexicon_tags_7 },
    { 0x9397a180, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc97a185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b7a188, SUC_lexicon_tags_43 },
    { 0x6457a189, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91b7a18b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67f7a18d, SUC_lexicon_tags_26 },
    { 0x7137a18d, SUC_lexicon_tags_60 },
    { 0x9f97a18d, SUC_lexicon_tags_7 },
    { 0xacd7a18f, SUC_lexicon_tags_25 },
    { 0xf937a191, SUC_lexicon_tags_12 },
    { 0x5a77a191, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9637a197, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b7a19d, SUC_lexicon_tags_17 },
    { 0x65d7a19e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef7a1a1, SUC_lexicon_tags_27 },
    { 0x137a1a2, SUC_lexicon_tags_60 },
    { 0xd5f7a1a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f7a1a9, SUC_lexicon_tags_63 },
    { 0x2ff7a1a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b7a1b1, SUC_lexicon_tags_23 },
    { 0x2577a1b1, SUC_lexicon_tags_26 },
    { 0xac17a1b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2497a1b6, SUC_lexicon_tags_29 },
    { 0x9937a1b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d97a1ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8457a1c0, SUC_lexicon_tags_37 },
    { 0xffd7a1c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b57a1c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6737a1c7, SUC_lexicon_tags_2 },
    { 0x817a1c8, SUC_lexicon_tags_17 },
    { 0xd2d7a1c9, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd7a1cd, SUC_lexicon_tags_15 },
    { 0x5ad7a1ce, SUC_lexicon_tags_11 },
    { 0xf0d7a1ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd537a1d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1257a1d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf7a1d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20b7a1da, SUC_lexicon_tags_15 },
    { 0x5c57a1db, SUC_lexicon_tags_17 },
    { 0xdcf7a1dc, SUC_lexicon_tags_12 },
    { 0x7bb7a1dc, SUC_lexicon_tags_6 },
    { 0x58f7a1de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5197a1e0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3297a1e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe6f7a1e4, SUC_lexicon_tags_12 },
    { 0x9e37a1e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d7a1ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb777a1ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3b7a1f0, SUC_lexicon_tags_47 },
    { 0x6cb7a1f0, SUC_lexicon_tags_39 },
    { 0x46b7a1f2, SUC_lexicon_tags_33 },
    { 0xf077a1f2, SUC_lexicon_tags_46 },
    { 0x3f37a1f3, SUC_lexicon_tags_11 },
    { 0x3837a1f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e97a1fa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5497a1fc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d7a1ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd897a201, SUC_lexicon_tags_1 },
    { 0x48f7a202, SUC_lexicon_tags_11 },
    { 0x93f7a202, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3397a206, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f37a209, SUC_lexicon_tags_38 },
    { 0xd137a20a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea57a20c, SUC_lexicon_tags_15 },
    { 0xbf37a20d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f7a211, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9a17a213, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b7a216, SUC_lexicon_tags_38 },
    { 0x59d7a216, SUC_lexicon_tags_17 },
    { 0xdcd7a218, SUC_lexicon_tags_0 },
    { 0x8af7a219, SUC_lexicon_tags_12 },
    { 0xb8f7a216, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2257a21c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b7a21f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a222, SUC_lexicon_tags_13 },
    { 0xa917a223, SUC_lexicon_tags_17 },
    { 0xac57a224, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d7a227, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x41b7a229, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdc77a22b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a57a22f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3077a231, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b7a236, SUC_lexicon_tags_114 },
    { 0xf197a237, SUC_lexicon_tags_17 },
    { 0x2337a238, SUC_lexicon_tags_19 },
    { 0xaa57a239, SUC_lexicon_tags_37 },
    { 0xc837a23a, SUC_lexicon_tags_37 },
    { 0xcb37a23b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xab77a23d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2077a23f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a57a248, SUC_lexicon_tags_19 },
    { 0x1737a248, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d97a24b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f7a251, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9277a25c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c37a25f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5717a262, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd17a268, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf17a26c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b7a273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe477a278, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55f7a27a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8237a27c, SUC_lexicon_tags_454 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e97a285, SUC_lexicon_tags_40 },
    { 0x6177a286, SUC_lexicon_tags_0 },
    { 0xba77a287, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a37a28b, SUC_lexicon_tags_87 },
    { 0x6837a28b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3d7a28e, SUC_lexicon_tags_16 },
    { 0x377a28f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x717a292, SUC_lexicon_tags_21 },
    { 0xedb7a293, SUC_lexicon_tags_14 },
    { 0x7057a293, SUC_lexicon_tags_61 },
    { 0xb4d7a295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7b7a297, SUC_lexicon_tags_45 },
    { 0x8ef7a298, SUC_lexicon_tags_17 },
    { 0xb337a298, SUC_lexicon_tags_17 },
    { 0xc5f7a29a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba17a2a1, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x55b7a2a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8517a2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41b7a2ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7057a2af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf917a2b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8957a2b6, SUC_lexicon_tags_114 },
    { 0x1bd7a2b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3237a2bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b97a2c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a97a2c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd7a2d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1837a2d9, SUC_lexicon_tags_0 },
    { 0x41f7a2da, SUC_lexicon_tags_10 },
    { 0xe617a2db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4437a2df, SUC_lexicon_tags_63 },
    { 0xf757a2df, SUC_lexicon_tags_17 },
    { 0x6417a2df, SUC_lexicon_tags_63 },
    { 0xf537a2e2, SUC_lexicon_tags_2 },
    { 0xeb97a2df, SUC_lexicon_tags_11 },
    { 0xe8f7a2e4, SUC_lexicon_tags_14 },
    { 0xfc17a2e2, SUC_lexicon_tags_46 },
    { 0xc037a2e3, SUC_lexicon_tags_21 },
    { 0xd737a2e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xac57a2e9, SUC_lexicon_tags_0 },
    { 0xcc17a2ea, SUC_lexicon_tags_17 },
    { 0xff57a2eb, SUC_lexicon_tags_49 },
    { 0x5537a2eb, SUC_lexicon_tags_23 },
    { 0x2137a2ed, SUC_lexicon_tags_32 },
    { 0x637a2ee, SUC_lexicon_tags_30 },
    { 0xe877a2eb, SUC_lexicon_tags_7 },
    { 0x2977a2f0, SUC_lexicon_tags_29 },
    { 0xa957a2eb, SUC_lexicon_tags_12 },
    { 0x28d7a2f2, SUC_lexicon_tags_44 },
    { 0xd337a2f3, SUC_lexicon_tags_25 },
    { 0xaf57a2f4, SUC_lexicon_tags_123 },
    { 0xe277a2f4, SUC_lexicon_tags_46 },
    { 0x4277a2ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f7a2fc, SUC_lexicon_tags_10 },
    { 0x1337a2fd, SUC_lexicon_tags_12 },
    { 0x6097a2fc, SUC_lexicon_tags_0 },
    { 0x8417a2ff, SUC_lexicon_tags_7 },
    { 0xa2f7a2fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81b7a302, SUC_lexicon_tags_38 },
    { 0x6e37a302, SUC_lexicon_tags_11 },
    { 0x4897a304, SUC_lexicon_tags_88 },
    { 0x1bb7a305, SUC_lexicon_tags_46 },
    { 0xa37a306, SUC_lexicon_tags_92 },
    { 0xc237a307, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b7a30a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1837a30f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b7a311, SUC_lexicon_tags_2 },
    { 0x3537a312, SUC_lexicon_tags_1 },
    { 0xd537a313, SUC_lexicon_tags_12 },
    { 0xc757a311, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef7a319, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf7a324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf7a329, SUC_lexicon_tags_2 },
    { 0xc0f7a32a, SUC_lexicon_tags_23 },
    { 0xa3d7a32b, SUC_lexicon_tags_19 },
    { 0xbb37a32b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc857a32e, SUC_lexicon_tags_7 },
    { 0xbab7a32f, SUC_lexicon_tags_16 },
    { 0xead7a330, SUC_lexicon_tags_12 },
    { 0x6457a331, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x68f7a333, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb7a337, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b7a33c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd7a341, SUC_lexicon_tags_12 },
    { 0x5b77a342, SUC_lexicon_tags_14 },
    { 0x2b17a342, SUC_lexicon_tags_7 },
    { 0x57a344, SUC_lexicon_tags_25 },
    { 0xddf7a345, SUC_lexicon_tags_7 },
    { 0xbb17a346, SUC_lexicon_tags_26 },
    { 0x71f7a344, SUC_lexicon_tags_23 },
    { 0xc477a348, SUC_lexicon_tags_8 },
    { 0xf6b7a349, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd137a34d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c77a350, SUC_lexicon_tags_12 },
    { 0x29d7a350, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe4b7a353, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd237a35c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7a364, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x41d7a366, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf77a36f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb497a371, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba17a375, SUC_lexicon_tags_25 },
    { 0xbf7a376, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b7a384, SUC_lexicon_tags_23 },
    { 0x7ad7a385, SUC_lexicon_tags_151 },
    { 0xe057a385, SUC_lexicon_tags_1 },
    { 0x5657a385, SUC_lexicon_tags_2 },
    { 0xcc17a388, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6777a38d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b37a395, SUC_lexicon_tags_33 },
    { 0x7ad7a396, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x94d7a398, SUC_lexicon_tags_14 },
    { 0x1877a398, SUC_lexicon_tags_11 },
    { 0x4bf7a39a, SUC_lexicon_tags_12 },
    { 0xa797a39b, SUC_lexicon_tags_35 },
    { 0x5457a39b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2677a39e, SUC_lexicon_tags_17 },
    { 0xbcd7a39e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb97a3a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f57a3a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d7a3af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9d37a3b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b7a3b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6617a3c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b17a3c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1997a3ca, SUC_lexicon_tags_17 },
    { 0x95b7a3ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb7a3cf, SUC_lexicon_tags_2 },
    { 0xb8f7a3cf, SUC_lexicon_tags_7 },
    { 0xd6b7a3cf, SUC_lexicon_tags_17 },
    { 0xf617a3d2, SUC_lexicon_tags_103 },
    { 0x1877a3d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdf37a3d5, SUC_lexicon_tags_28 },
    { 0x48d7a3d6, SUC_lexicon_tags_29 },
    { 0x84d7a3d7, SUC_lexicon_tags_37 },
    { 0xbe37a3d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7637a3dd, SUC_lexicon_tags_15 },
    { 0xab57a3de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe957a3e0, SUC_lexicon_tags_15 },
    { 0xf5b7a3e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f7a3e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f7a3ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf277a3f1, SUC_lexicon_tags_37 },
    { 0xe477a3f2, SUC_lexicon_tags_15 },
    { 0x737a3f2, SUC_lexicon_tags_12 },
    { 0x1c17a3f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb297a3f7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb7a3fa, SUC_lexicon_tags_37 },
    { 0xaeb7a3fb, SUC_lexicon_tags_17 },
    { 0x4a37a3fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4137a400, SUC_lexicon_tags_2 },
    { 0xe997a401, SUC_lexicon_tags_17 },
    { 0x3937a402, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b7a405, SUC_lexicon_tags_43 },
    { 0xa5d7a406, SUC_lexicon_tags_11 },
    { 0x1677a406, SUC_lexicon_tags_68 },
    { 0xf9f7a408, SUC_lexicon_tags_84 },
    { 0xc177a409, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x7537a40c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf1f7a40e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb7a411, SUC_lexicon_tags_68 },
    { 0x2837a411, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfb97a414, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4b77a416, SUC_lexicon_tags_26 },
    { 0x9297a416, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6477a41b, SUC_lexicon_tags_33 },
    { 0x3c77a41c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b7a420, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c77a426, SUC_lexicon_tags_14 },
    { 0xd4b7a427, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97a42e, SUC_lexicon_tags_63 },
    { 0x9257a42e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x357a432, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x937a434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f7a437, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd817a43b, SUC_lexicon_tags_37 },
    { 0x6937a43c, SUC_lexicon_tags_13 },
    { 0x85d7a43b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f77a43f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa757a444, SUC_lexicon_tags_27 },
    { 0xf0d7a445, SUC_lexicon_tags_14 },
    { 0x15f7a446, SUC_lexicon_tags_1 },
    { 0x2757a447, SUC_lexicon_tags_14 },
    { 0xef37a445, SUC_lexicon_tags_2 },
    { 0x5f97a449, SUC_lexicon_tags_11 },
    { 0x8977a44a, SUC_lexicon_tags_2 },
    { 0x7797a44b, SUC_lexicon_tags_17 },
    { 0x94f7a44a, SUC_lexicon_tags_43 },
    { 0xb7b7a44d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8df7a44f, SUC_lexicon_tags_79 },
    { 0xf8d7a450, SUC_lexicon_tags_27 },
    { 0xe637a44f, SUC_lexicon_tags_37 },
    { 0x1357a450, SUC_lexicon_tags_29 },
    { 0x27d7a451, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf757a457, SUC_lexicon_tags_6 },
    { 0x57f7a457, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe917a45a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3197a45d, SUC_lexicon_tags_23 },
    { 0xc617a45e, SUC_lexicon_tags_63 },
    { 0x6ab7a45f, SUC_lexicon_tags_38 },
    { 0x4397a45e, SUC_lexicon_tags_45 },
    { 0xdf77a45f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3797a464, SUC_lexicon_tags_15 },
    { 0xe2f7a465, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d17a46c, SUC_lexicon_tags_19 },
    { 0x2f37a46c, SUC_lexicon_tags_31 },
    { 0xf497a46c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4717a470, SUC_lexicon_tags_84 },
    { 0xd97a471, SUC_lexicon_tags_2 },
    { 0x5ff7a471, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf2f7a474, SUC_lexicon_tags_84 },
    { 0xceb7a475, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb7a47a, SUC_lexicon_tags_7 },
    { 0xcf57a47a, SUC_lexicon_tags_47 },
    { 0x9d97a47a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db7a482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab7a487, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb537a48b, SUC_lexicon_tags_6 },
    { 0x8a57a48b, SUC_lexicon_tags_11 },
    { 0x7177a48c, SUC_lexicon_tags_46 },
    { 0x2f37a48e, SUC_lexicon_tags_35 },
    { 0xa797a48f, SUC_lexicon_tags_10 },
    { 0x9737a490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f7a494, SUC_lexicon_tags_25 },
    { 0x65f7a495, SUC_lexicon_tags_19 },
    { 0xf0f7a494, SUC_lexicon_tags_19 },
    { 0x1897a496, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa57a49a, SUC_lexicon_tags_14 },
    { 0xb7a49b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d7a4a1, SUC_lexicon_tags_45 },
    { 0x3897a4a1, SUC_lexicon_tags_39 },
    { 0x8577a4a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d7a4a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8057a4a9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x5577a4ab, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x7317a4ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x617a4b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b77a4b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e97a4bd, SUC_lexicon_tags_48 },
    { 0x6ef7a4be, SUC_lexicon_tags_10 },
    { 0x3df7a4bf, SUC_lexicon_tags_39 },
    { 0x597a4c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b7a4c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2337a4c8, SUC_lexicon_tags_831 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d7a4cb, SUC_lexicon_tags_17 },
    { 0x71f7a4cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f7a4cf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a37a4d2, SUC_lexicon_tags_60 },
    { 0x7c77a4d2, SUC_lexicon_tags_47 },
    { 0x1337a4d3, SUC_lexicon_tags_39 },
    { 0xb2f7a4d4, SUC_lexicon_tags_11 },
    { 0xb657a4d6, SUC_lexicon_tags_28 },
    { 0xa837a4d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe637a4d9, SUC_lexicon_tags_75 },
    { 0xe597a4da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe37a4dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb7a4e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4917a4e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d7a4ec, SUC_lexicon_tags_37 },
    { 0x9377a4ed, SUC_lexicon_tags_12 },
    { 0x5937a4ee, SUC_lexicon_tags_2 },
    { 0x2fb7a4ef, SUC_lexicon_tags_87 },
    { 0x2ab7a4f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xde17a4f2, SUC_lexicon_tags_21 },
    { 0xbc17a4f3, SUC_lexicon_tags_25 },
    { 0xe337a4f4, SUC_lexicon_tags_37 },
    { 0xac97a4f5, SUC_lexicon_tags_4 },
    { 0x2a97a4f6, SUC_lexicon_tags_37 },
    { 0x9c37a4f3, SUC_lexicon_tags_17 },
    { 0x42f7a4f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b7a4fc, SUC_lexicon_tags_11 },
    { 0xc977a4fc, SUC_lexicon_tags_35 },
    { 0xa577a4fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f17a501, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe617a504, SUC_lexicon_tags_11 },
    { 0x85d7a505, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe757a508, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb7a50b, SUC_lexicon_tags_17 },
    { 0x27d7a50b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a57a510, SUC_lexicon_tags_44 },
    { 0x3e17a511, SUC_lexicon_tags_65 },
    { 0x3997a511, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc997a514, SUC_lexicon_tags_37 },
    { 0xe437a515, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a51a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4bd7a51c, SUC_lexicon_tags_12 },
    { 0xbab7a51c, SUC_lexicon_tags_38 },
    { 0x2077a51c, SUC_lexicon_tags_63 },
    { 0xd137a51f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7397a522, SUC_lexicon_tags_28 },
    { 0xdb97a523, SUC_lexicon_tags_37 },
    { 0x7db7a524, SUC_lexicon_tags_77 },
    { 0xffb7a522, SUC_lexicon_tags_104 },
    { 0x2c97a524, SUC_lexicon_tags_2 },
    { 0x7bd7a525, SUC_lexicon_tags_29 },
    { 0xf677a528, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec37a52c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b7a531, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e77a533, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8517a536, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e77a538, SUC_lexicon_tags_15 },
    { 0x6d7a539, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd17a53c, SUC_lexicon_tags_57 },
    { 0x4c97a53c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6557a540, SUC_lexicon_tags_49 },
    { 0x6fb7a541, SUC_lexicon_tags_7 },
    { 0xfed7a542, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x637a544, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab7a547, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed7a54b, SUC_lexicon_tags_15 },
    { 0xeb97a54c, SUC_lexicon_tags_12 },
    { 0x1f17a54b, SUC_lexicon_tags_36 },
    { 0x1257a54c, SUC_lexicon_tags_19 },
    { 0x6d97a54c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d7a555, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb57a55d, SUC_lexicon_tags_1484 },
    { 0, NULL },
    { 0, NULL },
    { 0x77a560, SUC_lexicon_tags_47 },
    { 0x4c97a561, SUC_lexicon_tags_1 },
    { 0xed37a562, SUC_lexicon_tags_12 },
    { 0x7e57a563, SUC_lexicon_tags_60 },
    { 0x5c17a560, SUC_lexicon_tags_37 },
    { 0xa677a565, SUC_lexicon_tags_29 },
    { 0xf637a566, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d7a569, SUC_lexicon_tags_12 },
    { 0x4317a56a, SUC_lexicon_tags_37 },
    { 0xc817a56b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7e57a56d, SUC_lexicon_tags_17 },
    { 0x66b7a56e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45f7a570, SUC_lexicon_tags_16 },
    { 0x3777a571, SUC_lexicon_tags_12 },
    { 0xb9f7a572, SUC_lexicon_tags_29 },
    { 0xd437a571, SUC_lexicon_tags_11 },
    { 0x6c37a573, SUC_lexicon_tags_35 },
    { 0x19d7a575, SUC_lexicon_tags_2 },
    { 0x57a576, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8097a57c, SUC_lexicon_tags_11 },
    { 0xd717a57c, SUC_lexicon_tags_32 },
    { 0xae77a57d, SUC_lexicon_tags_17 },
    { 0x6597a57f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf37a582, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcef7a584, SUC_lexicon_tags_17 },
    { 0x9f97a585, SUC_lexicon_tags_32 },
    { 0x86b7a584, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf597a588, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x27b7a58a, SUC_lexicon_tags_0 },
    { 0xbb77a58b, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x7477a58e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xa337a591, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b97a593, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b7a596, SUC_lexicon_tags_160 },
    { 0xa517a596, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4997a599, SUC_lexicon_tags_15 },
    { 0x8177a59a, SUC_lexicon_tags_94 },
    { 0xa977a59b, SUC_lexicon_tags_17 },
    { 0x4f57a59c, SUC_lexicon_tags_7 },
    { 0xfef7a59c, SUC_lexicon_tags_17 },
    { 0x4617a59e, SUC_lexicon_tags_35 },
    { 0xe957a59e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3317a5a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4817a5aa, SUC_lexicon_tags_19 },
    { 0x3b57a5ab, SUC_lexicon_tags_14 },
    { 0xa6f7a5ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xefd7a5ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb837a5b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31d7a5b4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xbdf7a5b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf7a5bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb537a5bd, SUC_lexicon_tags_37 },
    { 0x1277a5be, SUC_lexicon_tags_74 },
    { 0x7dd7a5bd, SUC_lexicon_tags_48 },
    { 0xe257a5c0, SUC_lexicon_tags_14 },
    { 0x4d97a5c1, SUC_lexicon_tags_25 },
    { 0x12f7a5c2, SUC_lexicon_tags_92 },
    { 0x56f7a5c3, SUC_lexicon_tags_7 },
    { 0xb437a5c3, SUC_lexicon_tags_7 },
    { 0xf237a5c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f7a5c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8797a5cb, SUC_lexicon_tags_0 },
    { 0x9997a5cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd857a5d0, SUC_lexicon_tags_25 },
    { 0x9257a5d1, SUC_lexicon_tags_49 },
    { 0x50b7a5d2, SUC_lexicon_tags_21 },
    { 0xccf7a5d1, SUC_lexicon_tags_32 },
    { 0x9117a5d4, SUC_lexicon_tags_0 },
    { 0x2d77a5d3, SUC_lexicon_tags_14 },
    { 0x1297a5d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff7a5da, SUC_lexicon_tags_37 },
    { 0x1437a5da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d7a5df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8617a5e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa657a5e7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2457a5e9, SUC_lexicon_tags_17 },
    { 0xf7b7a5e9, SUC_lexicon_tags_12 },
    { 0xef97a5eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb17a5ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xff77a5f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7a5f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd457a5f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc037a5f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa0b7a5fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2077a604, SUC_lexicon_tags_43 },
    { 0x3a77a604, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee17a607, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d7a60e, SUC_lexicon_tags_87 },
    { 0x3817a60f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb837a61d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2777a620, SUC_lexicon_tags_7 },
    { 0xfd97a621, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2017a624, SUC_lexicon_tags_151 },
    { 0xc957a625, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1277a627, SUC_lexicon_tags_14 },
    { 0xd9f7a628, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf317a62b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff77a62d, SUC_lexicon_tags_30 },
    { 0x7b37a62e, SUC_lexicon_tags_0 },
    { 0x7d17a62e, SUC_lexicon_tags_12 },
    { 0xa317a62f, SUC_lexicon_tags_496 },
    { 0x9cd7a62e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d77a635, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9257a63d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa237a642, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2117a644, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0x94f7a646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7b7a648, SUC_lexicon_tags_11 },
    { 0x21d7a648, SUC_lexicon_tags_63 },
    { 0xc6f7a64a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4117a64c, SUC_lexicon_tags_37 },
    { 0xf297a64d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae77a64f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb7a659, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb7a65d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2877a666, SUC_lexicon_tags_15 },
    { 0x21b7a667, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b97a66a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b7a66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6957a674, SUC_lexicon_tags_25 },
    { 0x4e57a675, SUC_lexicon_tags_63 },
    { 0xccb7a675, SUC_lexicon_tags_14 },
    { 0xf6d7a675, SUC_lexicon_tags_14 },
    { 0x4277a677, SUC_lexicon_tags_47 },
    { 0x22b7a679, SUC_lexicon_tags_17 },
    { 0xd9f7a676, SUC_lexicon_tags_37 },
    { 0x8657a67b, SUC_lexicon_tags_66 },
    { 0x61b7a675, SUC_lexicon_tags_12 },
    { 0xb877a67c, SUC_lexicon_tags_14 },
    { 0x2917a67d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8597a681, SUC_lexicon_tags_25 },
    { 0xa4b7a682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44f7a684, SUC_lexicon_tags_49 },
    { 0x7277a685, SUC_lexicon_tags_37 },
    { 0xfc77a686, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7f97a688, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb7a691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a17a693, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47d7a695, SUC_lexicon_tags_116 },
    { 0x9c97a696, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f7a699, SUC_lexicon_tags_16 },
    { 0xf7f7a69a, SUC_lexicon_tags_11 },
    { 0x1177a69b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6877a69f, SUC_lexicon_tags_9 },
    { 0xbff7a6a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f7a6a3, SUC_lexicon_tags_155 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f7a6a8, SUC_lexicon_tags_45 },
    { 0x4dd7a6a8, SUC_lexicon_tags_1 },
    { 0x9c97a6aa, SUC_lexicon_tags_37 },
    { 0xac17a6ab, SUC_lexicon_tags_0 },
    { 0x9517a6a9, SUC_lexicon_tags_15 },
    { 0x6c97a6ab, SUC_lexicon_tags_17 },
    { 0x5b77a6ad, SUC_lexicon_tags_1681 },
    { 0, NULL },
    { 0xc4b7a6b0, SUC_lexicon_tags_15 },
    { 0x3437a6b1, SUC_lexicon_tags_19 },
    { 0x1cd7a6b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47b7a6b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6717a6b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4f7a6bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c57a6bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2737a6c1, SUC_lexicon_tags_12 },
    { 0xc197a6c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf277a6cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1517a6d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89d7a6d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x92d7a6d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd917a6d6, SUC_lexicon_tags_63 },
    { 0xdb77a6d7, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xf3d7a6d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b7a6dd, SUC_lexicon_tags_1357 },
    { 0xb8d7a6de, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0x7877a6e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x86d7a6e2, SUC_lexicon_tags_64 },
    { 0xa037a6e2, SUC_lexicon_tags_38 },
    { 0x5d37a6e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf057a6e7, SUC_lexicon_tags_9 },
    { 0x94d7a6e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x94f7a6ea, SUC_lexicon_tags_19 },
    { 0x3957a6eb, SUC_lexicon_tags_35 },
    { 0x9a57a6ec, SUC_lexicon_tags_19 },
    { 0x64f7a6ec, SUC_lexicon_tags_10 },
    { 0x86b7a6eb, SUC_lexicon_tags_15 },
    { 0xb697a6ef, SUC_lexicon_tags_49 },
    { 0x9417a6ed, SUC_lexicon_tags_19 },
    { 0x8e97a6f1, SUC_lexicon_tags_31 },
    { 0x97b7a6f2, SUC_lexicon_tags_11 },
    { 0xc297a6f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e17a6f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f7a6f8, SUC_lexicon_tags_25 },
    { 0x9f57a6f9, SUC_lexicon_tags_1 },
    { 0xf877a6f9, SUC_lexicon_tags_60 },
    { 0x54b7a6fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf97a702, SUC_lexicon_tags_12 },
    { 0x3537a703, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa477a705, SUC_lexicon_tags_48 },
    { 0xcc37a706, SUC_lexicon_tags_43 },
    { 0x3917a707, SUC_lexicon_tags_7 },
    { 0x6877a707, SUC_lexicon_tags_17 },
    { 0xcd77a709, SUC_lexicon_tags_17 },
    { 0x6977a706, SUC_lexicon_tags_1 },
    { 0xb557a70b, SUC_lexicon_tags_28 },
    { 0x6a77a70b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff37a712, SUC_lexicon_tags_2 },
    { 0xec57a713, SUC_lexicon_tags_1 },
    { 0xe837a713, SUC_lexicon_tags_11 },
    { 0x417a713, SUC_lexicon_tags_0 },
    { 0xc397a714, SUC_lexicon_tags_27 },
    { 0x31f7a716, SUC_lexicon_tags_63 },
    { 0x6937a714, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f7a71b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa857a71f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb377a724, SUC_lexicon_tags_45 },
    { 0xfed7a724, SUC_lexicon_tags_11 },
    { 0xc37a726, SUC_lexicon_tags_25 },
    { 0x4a57a727, SUC_lexicon_tags_11 },
    { 0xcfb7a728, SUC_lexicon_tags_32 },
    { 0xf457a726, SUC_lexicon_tags_7 },
    { 0x4357a72a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b7a72e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7e97a730, SUC_lexicon_tags_17 },
    { 0x3cf7a730, SUC_lexicon_tags_17 },
    { 0x1e17a732, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3697a738, SUC_lexicon_tags_17 },
    { 0x9c57a739, SUC_lexicon_tags_38 },
    { 0x4cb7a73a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde97a744, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9657a747, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb437a74a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef77a74f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf117a752, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f97a755, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb1b7a757, SUC_lexicon_tags_116 },
    { 0x49f7a758, SUC_lexicon_tags_47 },
    { 0x4b37a759, SUC_lexicon_tags_17 },
    { 0x67d7a75a, SUC_lexicon_tags_37 },
    { 0x84b7a758, SUC_lexicon_tags_28 },
    { 0xc837a75c, SUC_lexicon_tags_43 },
    { 0x8357a758, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb7a766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f7a768, SUC_lexicon_tags_2 },
    { 0x9817a768, SUC_lexicon_tags_213 },
    { 0x6297a76a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8777a76e, SUC_lexicon_tags_106 },
    { 0xe917a76f, SUC_lexicon_tags_7 },
    { 0xfab7a76e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657a77c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc17a77f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad7a781, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x98b7a783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6837a790, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f7a792, SUC_lexicon_tags_23 },
    { 0x9477a793, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6df7a795, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977a79c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd977a7a1, SUC_lexicon_tags_0 },
    { 0xfb97a7a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97a7a5, SUC_lexicon_tags_45 },
    { 0xef77a7a5, SUC_lexicon_tags_17 },
    { 0xc677a7a7, SUC_lexicon_tags_12 },
    { 0x93f7a7a8, SUC_lexicon_tags_6 },
    { 0x6a17a7a9, SUC_lexicon_tags_104 },
    { 0x8577a7a8, SUC_lexicon_tags_1 },
    { 0x1717a7ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa217a7ae, SUC_lexicon_tags_17 },
    { 0x9957a7ae, SUC_lexicon_tags_14 },
    { 0xfbd7a7ae, SUC_lexicon_tags_12 },
    { 0xda57a7b1, SUC_lexicon_tags_54 },
    { 0xe7f7a7ae, SUC_lexicon_tags_11 },
    { 0xe217a7b1, SUC_lexicon_tags_23 },
    { 0xfcb7a7af, SUC_lexicon_tags_7 },
    { 0xa5f7a7b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9917a7bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc77a7c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c57a7c2, SUC_lexicon_tags_17 },
    { 0x797a7c2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e37a7c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9f7a7c8, SUC_lexicon_tags_15 },
    { 0xb7f7a7c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2477a7cb, SUC_lexicon_tags_37 },
    { 0x1ab7a7cc, SUC_lexicon_tags_2 },
    { 0x6dd7a7cd, SUC_lexicon_tags_10 },
    { 0xfd97a7ce, SUC_lexicon_tags_10 },
    { 0x6177a7cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xae17a7d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x397a7d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f7a7d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a97a7d9, SUC_lexicon_tags_15 },
    { 0x5137a7da, SUC_lexicon_tags_151 },
    { 0x4357a7db, SUC_lexicon_tags_1 },
    { 0x5757a7db, SUC_lexicon_tags_49 },
    { 0xb1d7a7dd, SUC_lexicon_tags_47 },
    { 0x1477a7de, SUC_lexicon_tags_8 },
    { 0xbcd7a7df, SUC_lexicon_tags_17 },
    { 0x6b7a7e0, SUC_lexicon_tags_37 },
    { 0xaff7a7e1, SUC_lexicon_tags_37 },
    { 0x6c97a7da, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7757a7e4, SUC_lexicon_tags_1 },
    { 0x59b7a7e5, SUC_lexicon_tags_3 },
    { 0xd057a7e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2897a7e8, SUC_lexicon_tags_15 },
    { 0x74f7a7e9, SUC_lexicon_tags_15 },
    { 0xb3b7a7e9, SUC_lexicon_tags_84 },
    { 0x8e77a7eb, SUC_lexicon_tags_30 },
    { 0x25b7a7ec, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d7a7ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f17a7f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f7a7f8, SUC_lexicon_tags_11 },
    { 0xa5b7a7f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd7a7fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b7a803, SUC_lexicon_tags_2 },
    { 0xa2f7a803, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab7a808, SUC_lexicon_tags_7 },
    { 0x56f7a808, SUC_lexicon_tags_45 },
    { 0xc377a80a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8d7a80c, SUC_lexicon_tags_1 },
    { 0x4cf7a80d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7a811, SUC_lexicon_tags_8 },
    { 0x3117a811, SUC_lexicon_tags_0 },
    { 0x3697a811, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7a81a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb57a821, SUC_lexicon_tags_14 },
    { 0x71b7a822, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b7a826, SUC_lexicon_tags_45 },
    { 0xd917a827, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf7a82b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d17a82f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f7a839, SUC_lexicon_tags_3 },
    { 0x8137a83a, SUC_lexicon_tags_65 },
    { 0x9437a839, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc397a83d, SUC_lexicon_tags_17 },
    { 0x7c17a83e, SUC_lexicon_tags_37 },
    { 0xa777a83d, SUC_lexicon_tags_25 },
    { 0xe437a840, SUC_lexicon_tags_38 },
    { 0x2677a841, SUC_lexicon_tags_27 },
    { 0xcf97a842, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5137a847, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd797a84f, SUC_lexicon_tags_11 },
    { 0xcd97a850, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56f7a852, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d37a855, SUC_lexicon_tags_17 },
    { 0x2a97a856, SUC_lexicon_tags_7 },
    { 0xd237a855, SUC_lexicon_tags_23 },
    { 0xf557a858, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d7a85e, SUC_lexicon_tags_37 },
    { 0xd337a85e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f77a861, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x657a863, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cf7a865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8f7a867, SUC_lexicon_tags_55 },
    { 0x3757a868, SUC_lexicon_tags_17 },
    { 0x7dd7a869, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4b37a86b, SUC_lexicon_tags_0 },
    { 0xd37a86b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa97a86e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x7337a870, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd837a873, SUC_lexicon_tags_69 },
    { 0xa557a874, SUC_lexicon_tags_1 },
    { 0x6ab7a874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd7a878, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf7a87e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb917a883, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a57a88a, SUC_lexicon_tags_9 },
    { 0x9ad7a88b, SUC_lexicon_tags_15 },
    { 0x9ef7a88c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d7a88f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb97a893, SUC_lexicon_tags_37 },
    { 0x51b7a894, SUC_lexicon_tags_28 },
    { 0x9117a895, SUC_lexicon_tags_19 },
    { 0xe977a896, SUC_lexicon_tags_27 },
    { 0xe897a895, SUC_lexicon_tags_45 },
    { 0x8f17a893, SUC_lexicon_tags_11 },
    { 0x7ed7a895, SUC_lexicon_tags_27 },
    { 0x7577a893, SUC_lexicon_tags_68 },
    { 0x8457a899, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd477a89d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7797a8a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2d7a8a3, SUC_lexicon_tags_60 },
    { 0x8917a8a4, SUC_lexicon_tags_1 },
    { 0x42d7a8a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4077a8aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc577a8b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd517a8b9, SUC_lexicon_tags_47 },
    { 0x8ff7a8ba, SUC_lexicon_tags_12 },
    { 0xcc77a8b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d7a8bf, SUC_lexicon_tags_1 },
    { 0xd777a8bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd7a8c4, SUC_lexicon_tags_9 },
    { 0xa817a8c5, SUC_lexicon_tags_9 },
    { 0x63f7a8c6, SUC_lexicon_tags_30 },
    { 0x5217a8c6, SUC_lexicon_tags_6 },
    { 0xa7f7a8c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc57a8ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b7a8d0, SUC_lexicon_tags_45 },
    { 0x7197a8d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4d7a8d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd77a8d6, SUC_lexicon_tags_4 },
    { 0x5597a8d6, SUC_lexicon_tags_17 },
    { 0x6cd7a8d7, SUC_lexicon_tags_29 },
    { 0x4917a8d7, SUC_lexicon_tags_11 },
    { 0xfd37a8da, SUC_lexicon_tags_37 },
    { 0x6337a8d8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9277a8de, SUC_lexicon_tags_15 },
    { 0xea17a8df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5537a8e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c17a8e8, SUC_lexicon_tags_26 },
    { 0xa8b7a8e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb777a8ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaff7a8f1, SUC_lexicon_tags_66 },
    { 0xd537a8f2, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x1ed7a8f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f17a8f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa897a8fc, SUC_lexicon_tags_7 },
    { 0xd957a8fd, SUC_lexicon_tags_72 },
    { 0x43f7a8fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97a905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b7a908, SUC_lexicon_tags_7 },
    { 0xa5d7a909, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xde57a90b, SUC_lexicon_tags_613 },
    { 0x2a37a90b, SUC_lexicon_tags_39 },
    { 0xc177a90c, SUC_lexicon_tags_37 },
    { 0x1a57a90e, SUC_lexicon_tags_7 },
    { 0xe217a90f, SUC_lexicon_tags_11 },
    { 0x52d7a910, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8917a913, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8db7a915, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7217a918, SUC_lexicon_tags_25 },
    { 0xcb7a918, SUC_lexicon_tags_37 },
    { 0x8f57a91a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd537a91e, SUC_lexicon_tags_7 },
    { 0x3b37a91e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e17a921, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd7a924, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b57a927, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e17a92d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7497a92f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb7a934, SUC_lexicon_tags_60 },
    { 0xf997a935, SUC_lexicon_tags_14 },
    { 0x8ed7a935, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac57a938, SUC_lexicon_tags_17 },
    { 0xb657a939, SUC_lexicon_tags_6 },
    { 0x4b17a939, SUC_lexicon_tags_47 },
    { 0x1757a93b, SUC_lexicon_tags_43 },
    { 0x6bb7a93a, SUC_lexicon_tags_17 },
    { 0x8857a93c, SUC_lexicon_tags_1 },
    { 0x1137a93e, SUC_lexicon_tags_63 },
    { 0x2d57a93f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb997a941, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0x99f7a943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa77a94a, SUC_lexicon_tags_17 },
    { 0x54f7a94a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc797a94d, SUC_lexicon_tags_11 },
    { 0xcf97a94e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8f57a950, SUC_lexicon_tags_17 },
    { 0xbeb7a951, SUC_lexicon_tags_25 },
    { 0x7c77a951, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d17a956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd297a958, SUC_lexicon_tags_1 },
    { 0x7cb7a959, SUC_lexicon_tags_25 },
    { 0xbb7a95a, SUC_lexicon_tags_25 },
    { 0x8517a95b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf797a95f, SUC_lexicon_tags_25 },
    { 0x1497a95f, SUC_lexicon_tags_35 },
    { 0x1277a95f, SUC_lexicon_tags_8 },
    { 0xc977a962, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d7a965, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf97a968, SUC_lexicon_tags_26 },
    { 0x1bb7a968, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd17a96d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f7a972, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f97a982, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d7a989, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe37a98e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2977a992, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x57b7a994, SUC_lexicon_tags_15 },
    { 0x54d7a995, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc17a997, SUC_lexicon_tags_158 },
    { 0x6217a997, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f7a99c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6877a99f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5b77a9a1, SUC_lexicon_tags_14 },
    { 0xaab7a9a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b7a9a6, SUC_lexicon_tags_14 },
    { 0x90b7a9a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2297a9ae, SUC_lexicon_tags_11 },
    { 0x2d7a9af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd277a9b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf637a9b5, SUC_lexicon_tags_25 },
    { 0x4eb7a9b5, SUC_lexicon_tags_17 },
    { 0x2277a9b7, SUC_lexicon_tags_63 },
    { 0x9a17a9b8, SUC_lexicon_tags_70 },
    { 0xf2f7a9b9, SUC_lexicon_tags_14 },
    { 0xdeb7a9ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a37a9bd, SUC_lexicon_tags_33 },
    { 0xf277a9be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf137a9c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef7a9c7, SUC_lexicon_tags_7 },
    { 0x57d7a9c7, SUC_lexicon_tags_29 },
    { 0xf97a9c7, SUC_lexicon_tags_66 },
    { 0x14b7a9ca, SUC_lexicon_tags_25 },
    { 0xab17a9c7, SUC_lexicon_tags_4 },
    { 0x2097a9ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf97a9d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe097a9d8, SUC_lexicon_tags_29 },
    { 0x78d7a9d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d7a9dc, SUC_lexicon_tags_2 },
    { 0x5037a9dd, SUC_lexicon_tags_47 },
    { 0x2277a9dd, SUC_lexicon_tags_39 },
    { 0x9e57a9df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7337a9e9, SUC_lexicon_tags_5 },
    { 0xc537a9e9, SUC_lexicon_tags_12 },
    { 0xf377a9eb, SUC_lexicon_tags_38 },
    { 0x83d7a9ea, SUC_lexicon_tags_77 },
    { 0x4657a9ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b77a9f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1437a9f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d7a9f6, SUC_lexicon_tags_7 },
    { 0x5577a9f6, SUC_lexicon_tags_43 },
    { 0x32b7a9f6, SUC_lexicon_tags_17 },
    { 0xcff7a9f9, SUC_lexicon_tags_12 },
    { 0xe997a9f8, SUC_lexicon_tags_37 },
    { 0x8197a9fb, SUC_lexicon_tags_11 },
    { 0xe417a9f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab7a9ff, SUC_lexicon_tags_8 },
    { 0x88f7aa00, SUC_lexicon_tags_69 },
    { 0x3857a9ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7aa05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc57aa0e, SUC_lexicon_tags_15 },
    { 0xbc97aa0f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d7aa17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2937aa19, SUC_lexicon_tags_25 },
    { 0xb777aa19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf17aa1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccd7aa1f, SUC_lexicon_tags_11 },
    { 0xf657aa20, SUC_lexicon_tags_37 },
    { 0xb117aa21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cd7aa23, SUC_lexicon_tags_15 },
    { 0xd1f7aa23, SUC_lexicon_tags_1277 },
    { 0x51d7aa25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2057aa28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7577aa2b, SUC_lexicon_tags_12 },
    { 0xee97aa2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf457aa2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d7aa34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd7aa39, SUC_lexicon_tags_47 },
    { 0x5b97aa39, SUC_lexicon_tags_17 },
    { 0x5857aa3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c17aa47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb997aa49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76d7aa4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b7aa4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50b7aa50, SUC_lexicon_tags_11 },
    { 0x9657aa51, SUC_lexicon_tags_43 },
    { 0x457aa50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x72b7aa54, SUC_lexicon_tags_75 },
    { 0xbf7aa55, SUC_lexicon_tags_17 },
    { 0x3877aa56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff7aa5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17aa5f, SUC_lexicon_tags_6 },
    { 0xb9b7aa60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4217aa62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f7aa66, SUC_lexicon_tags_0 },
    { 0x4d57aa66, SUC_lexicon_tags_56 },
    { 0x54b7aa67, SUC_lexicon_tags_79 },
    { 0x5db7aa68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe77aa6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb397aa71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b37aa73, SUC_lexicon_tags_14 },
    { 0x9ab7aa74, SUC_lexicon_tags_10 },
    { 0x1c17aa74, SUC_lexicon_tags_27 },
    { 0xde17aa74, SUC_lexicon_tags_15 },
    { 0xaef7aa77, SUC_lexicon_tags_17 },
    { 0xfab7aa75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd7aa7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197aa83, SUC_lexicon_tags_55 },
    { 0xe257aa83, SUC_lexicon_tags_2 },
    { 0xb0f7aa85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ff7aa87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f7aa91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ef7aa93, SUC_lexicon_tags_11 },
    { 0x24f7aa94, SUC_lexicon_tags_17 },
    { 0xae37aa93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe537aa97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac77aa9a, SUC_lexicon_tags_4 },
    { 0xb637aa9a, SUC_lexicon_tags_27 },
    { 0xd3d7aa9c, SUC_lexicon_tags_9 },
    { 0x7fb7aa9d, SUC_lexicon_tags_1 },
    { 0xb2d7aa9e, SUC_lexicon_tags_12 },
    { 0x3577aa9f, SUC_lexicon_tags_60 },
    { 0x2397aaa0, SUC_lexicon_tags_1 },
    { 0x7a97aaa1, SUC_lexicon_tags_2 },
    { 0x8f57aa9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf17aaa4, SUC_lexicon_tags_28 },
    { 0x6d7aaa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7057aaaa, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5537aaaf, SUC_lexicon_tags_37 },
    { 0xae17aab0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4cb7aab2, SUC_lexicon_tags_47 },
    { 0x5fb7aab3, SUC_lexicon_tags_92 },
    { 0xa837aab3, SUC_lexicon_tags_47 },
    { 0xb257aab5, SUC_lexicon_tags_37 },
    { 0x2157aab2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6197aab8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2517aabc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e57aac3, SUC_lexicon_tags_65 },
    { 0x6bd7aac4, SUC_lexicon_tags_60 },
    { 0x6d7aac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3437aacb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcb17aacd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd7aad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2377aad4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1357aad9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2277aadb, SUC_lexicon_tags_25 },
    { 0x6777aadc, SUC_lexicon_tags_25 },
    { 0x88d7aadc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86b7aadf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa9f7aae1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d7aae4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb957aaeb, SUC_lexicon_tags_62 },
    { 0x6fb7aaec, SUC_lexicon_tags_27 },
    { 0xd77aaec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd77aaf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2097ab02, SUC_lexicon_tags_16 },
    { 0x3557ab03, SUC_lexicon_tags_47 },
    { 0x9057ab03, SUC_lexicon_tags_15 },
    { 0xb317ab03, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7fb7ab07, SUC_lexicon_tags_25 },
    { 0x6f37ab07, SUC_lexicon_tags_57 },
    { 0xcb7ab07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8bb7ab0b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c37ab0f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xedd7ab11, SUC_lexicon_tags_25 },
    { 0x4f17ab12, SUC_lexicon_tags_17 },
    { 0x3717ab13, SUC_lexicon_tags_15 },
    { 0xc717ab11, SUC_lexicon_tags_45 },
    { 0xc0f7ab15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b97ab1c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5797ab22, SUC_lexicon_tags_15 },
    { 0xc0d7ab22, SUC_lexicon_tags_11 },
    { 0x1497ab24, SUC_lexicon_tags_612 },
    { 0x5df7ab25, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8417ab28, SUC_lexicon_tags_47 },
    { 0x76b7ab29, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb317ab2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f7ab30, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf8d7ab32, SUC_lexicon_tags_6 },
    { 0x4857ab33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b7ab37, SUC_lexicon_tags_37 },
    { 0xc7b7ab37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337ab3d, SUC_lexicon_tags_7 },
    { 0xcd97ab3d, SUC_lexicon_tags_15 },
    { 0xeb7ab3f, SUC_lexicon_tags_91 },
    { 0xc4f7ab3e, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xb7b7ab42, SUC_lexicon_tags_37 },
    { 0x7717ab42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96b7ab45, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4457ab47, SUC_lexicon_tags_54 },
    { 0xfb7ab48, SUC_lexicon_tags_8 },
    { 0x1797ab48, SUC_lexicon_tags_49 },
    { 0x3b17ab48, SUC_lexicon_tags_47 },
    { 0x1557ab4a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd537ab4e, SUC_lexicon_tags_12 },
    { 0x1337ab4f, SUC_lexicon_tags_37 },
    { 0x77f7ab50, SUC_lexicon_tags_30 },
    { 0xd357ab51, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd5f7ab53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f7ab57, SUC_lexicon_tags_26 },
    { 0xd437ab58, SUC_lexicon_tags_17 },
    { 0xf997ab58, SUC_lexicon_tags_11 },
    { 0x7617ab58, SUC_lexicon_tags_8 },
    { 0x3057ab5a, SUC_lexicon_tags_25 },
    { 0x8ad7ab5b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1677ab5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba77ab60, SUC_lexicon_tags_4 },
    { 0x9157ab61, SUC_lexicon_tags_25 },
    { 0xdbf7ab61, SUC_lexicon_tags_28 },
    { 0x5b77ab63, SUC_lexicon_tags_14 },
    { 0xd717ab61, SUC_lexicon_tags_11 },
    { 0x33d7ab64, SUC_lexicon_tags_80 },
    { 0xf177ab62, SUC_lexicon_tags_12 },
    { 0x5697ab61, SUC_lexicon_tags_28 },
    { 0x5d7ab68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b7ab6f, SUC_lexicon_tags_28 },
    { 0x5177ab6f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc597ab72, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f7ab76, SUC_lexicon_tags_12 },
    { 0xff97ab77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7ab7e, SUC_lexicon_tags_11 },
    { 0xc517ab7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7757ab85, SUC_lexicon_tags_38 },
    { 0xff7ab86, SUC_lexicon_tags_15 },
    { 0xd9d7ab85, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1817ab8f, SUC_lexicon_tags_11 },
    { 0xfed7ab90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe97ab92, SUC_lexicon_tags_23 },
    { 0x1b57ab93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd437ab95, SUC_lexicon_tags_136 },
    { 0x1937ab96, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf057ab9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe337ab9c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa517aba2, SUC_lexicon_tags_11 },
    { 0xa897aba3, SUC_lexicon_tags_285 },
    { 0xa337aba4, SUC_lexicon_tags_25 },
    { 0xed7aba5, SUC_lexicon_tags_25 },
    { 0x137aba3, SUC_lexicon_tags_11 },
    { 0x5057aba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d7abae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd7abb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d7abb8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa777abbb, SUC_lexicon_tags_7 },
    { 0x7c37abbc, SUC_lexicon_tags_27 },
    { 0x9137abbd, SUC_lexicon_tags_19 },
    { 0x8af7abbc, SUC_lexicon_tags_45 },
    { 0x4457abbf, SUC_lexicon_tags_1 },
    { 0x63d7abc0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1277abc5, SUC_lexicon_tags_14 },
    { 0x6d57abc6, SUC_lexicon_tags_37 },
    { 0x4197abc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96b7abc9, SUC_lexicon_tags_19 },
    { 0x3eb7abca, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7abcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1637abd0, SUC_lexicon_tags_14 },
    { 0x3a97abd0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbb7abd3, SUC_lexicon_tags_11 },
    { 0x44d7abd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f7abd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9377abd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b7abdc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7abe0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b97abe3, SUC_lexicon_tags_0 },
    { 0x8a17abe3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7abe7, SUC_lexicon_tags_61 },
    { 0x5db7abe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b7abf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe217abf7, SUC_lexicon_tags_27 },
    { 0x8d97abf8, SUC_lexicon_tags_9 },
    { 0x1f37abf8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f7abfc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46f7abfe, SUC_lexicon_tags_1060 },
    { 0x8997abfe, SUC_lexicon_tags_2 },
    { 0xded7ac00, SUC_lexicon_tags_11 },
    { 0xca57abff, SUC_lexicon_tags_11 },
    { 0x81d7ac02, SUC_lexicon_tags_11 },
    { 0xc257ac00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f7ac07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd637ac0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4797ac0c, SUC_lexicon_tags_34 },
    { 0x637ac0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5f7ac0f, SUC_lexicon_tags_25 },
    { 0xff97ac10, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb7ac14, SUC_lexicon_tags_37 },
    { 0x40d7ac15, SUC_lexicon_tags_25 },
    { 0x13d7ac16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9017ac18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb4b7ac1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe897ac1c, SUC_lexicon_tags_21 },
    { 0x2bb7ac1c, SUC_lexicon_tags_2 },
    { 0xf857ac1e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44f7ac20, SUC_lexicon_tags_10 },
    { 0x4ef7ac20, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6297ac24, SUC_lexicon_tags_7 },
    { 0xac37ac24, SUC_lexicon_tags_56 },
    { 0x8997ac24, SUC_lexicon_tags_7 },
    { 0xaa37ac25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2f17ac29, SUC_lexicon_tags_31 },
    { 0xe737ac2a, SUC_lexicon_tags_17 },
    { 0xe317ac2a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe6d7ac2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6317ac33, SUC_lexicon_tags_32 },
    { 0xe5b7ac34, SUC_lexicon_tags_11 },
    { 0x3c37ac35, SUC_lexicon_tags_16 },
    { 0x1ed7ac33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8337ac3a, SUC_lexicon_tags_14 },
    { 0x5f97ac3b, SUC_lexicon_tags_37 },
    { 0x63d7ac3c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab7ac40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5637ac42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6377ac49, SUC_lexicon_tags_12 },
    { 0x36d7ac4a, SUC_lexicon_tags_37 },
    { 0x77b7ac4b, SUC_lexicon_tags_17 },
    { 0xc757ac4c, SUC_lexicon_tags_29 },
    { 0x2e97ac49, SUC_lexicon_tags_2 },
    { 0x89f7ac4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe37ac50, SUC_lexicon_tags_11 },
    { 0x90b7ac51, SUC_lexicon_tags_7 },
    { 0x2777ac52, SUC_lexicon_tags_17 },
    { 0x577ac53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb7ac56, SUC_lexicon_tags_17 },
    { 0x2b97ac57, SUC_lexicon_tags_12 },
    { 0x7b17ac58, SUC_lexicon_tags_17 },
    { 0xbf37ac58, SUC_lexicon_tags_7 },
    { 0x1737ac58, SUC_lexicon_tags_11 },
    { 0x6117ac59, SUC_lexicon_tags_87 },
    { 0x8397ac56, SUC_lexicon_tags_38 },
    { 0x5137ac5a, SUC_lexicon_tags_79 },
    { 0x7cf7ac5e, SUC_lexicon_tags_77 },
    { 0x7d37ac5a, SUC_lexicon_tags_37 },
    { 0x7fd7ac5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb477ac62, SUC_lexicon_tags_2 },
    { 0xa6b7ac62, SUC_lexicon_tags_27 },
    { 0xb357ac64, SUC_lexicon_tags_11 },
    { 0xb777ac65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe577ac68, SUC_lexicon_tags_13 },
    { 0xb2f7ac68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd3d7ac6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc117ac74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd777ac7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x757ac7f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9997ac81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f7ac84, SUC_lexicon_tags_1 },
    { 0x3dd7ac84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9357ac87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1437ac8a, SUC_lexicon_tags_38 },
    { 0x6037ac8b, SUC_lexicon_tags_37 },
    { 0xfaf7ac8b, SUC_lexicon_tags_26 },
    { 0xc437ac8c, SUC_lexicon_tags_28 },
    { 0x1c97ac8d, SUC_lexicon_tags_12 },
    { 0xf917ac8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b7ac92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81f7ac94, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3e77ac96, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7497ac9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd277ac9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79d7aca0, SUC_lexicon_tags_61 },
    { 0x82f7aca1, SUC_lexicon_tags_7 },
    { 0x11b7aca2, SUC_lexicon_tags_11 },
    { 0xb9b7aca3, SUC_lexicon_tags_43 },
    { 0x7f17aca1, SUC_lexicon_tags_2 },
    { 0x3577aca5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b7acaa, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb017acae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed7acb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd917acb6, SUC_lexicon_tags_37 },
    { 0x1bb7acb6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f7acbb, SUC_lexicon_tags_45 },
    { 0xf677acbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd957acbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc217acc2, SUC_lexicon_tags_23 },
    { 0x6537acc2, SUC_lexicon_tags_17 },
    { 0xdeb7acc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc797acc7, SUC_lexicon_tags_14 },
    { 0x5e57acc7, SUC_lexicon_tags_7 },
    { 0x8037acc8, SUC_lexicon_tags_12 },
    { 0xdab7acca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b7acce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f7acd9, SUC_lexicon_tags_92 },
    { 0xbe17acd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e37acdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4897acdf, SUC_lexicon_tags_151 },
    { 0x1477ace0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed7ace4, SUC_lexicon_tags_60 },
    { 0x6197ace4, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7197ace9, SUC_lexicon_tags_11 },
    { 0xb237acea, SUC_lexicon_tags_37 },
    { 0xec17aceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce17aced, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b7acf1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77acfb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x54d7acfd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb7ad05, SUC_lexicon_tags_15 },
    { 0x4817ad05, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x83f7ad08, SUC_lexicon_tags_37 },
    { 0x80d7ad09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7ad0c, SUC_lexicon_tags_12 },
    { 0x4657ad0d, SUC_lexicon_tags_37 },
    { 0x14b7ad0e, SUC_lexicon_tags_27 },
    { 0x8377ad0e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf37ad15, SUC_lexicon_tags_37 },
    { 0xd957ad16, SUC_lexicon_tags_30 },
    { 0x5f17ad17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7a97ad19, SUC_lexicon_tags_15 },
    { 0x2cb7ad1a, SUC_lexicon_tags_51 },
    { 0x9a57ad1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b7ad20, SUC_lexicon_tags_6 },
    { 0x20d7ad21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957ad26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e77ad2b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe917ad2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d97ad30, SUC_lexicon_tags_74 },
    { 0x3757ad30, SUC_lexicon_tags_0 },
    { 0xa6d7ad32, SUC_lexicon_tags_12 },
    { 0x85d7ad31, SUC_lexicon_tags_25 },
    { 0x2c57ad34, SUC_lexicon_tags_38 },
    { 0x24b7ad33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9137ad38, SUC_lexicon_tags_11 },
    { 0x7937ad39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e17ad41, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b17ad44, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc757ad46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db7ad4a, SUC_lexicon_tags_37 },
    { 0xd37ad4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91d7ad4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x497ad4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b7ad55, SUC_lexicon_tags_25 },
    { 0xe897ad56, SUC_lexicon_tags_37 },
    { 0x7b77ad57, SUC_lexicon_tags_11 },
    { 0x2957ad57, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1997ad5e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x2b37ad60, SUC_lexicon_tags_33 },
    { 0x9c97ad61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e37ad65, SUC_lexicon_tags_1 },
    { 0xdfd7ad65, SUC_lexicon_tags_37 },
    { 0x7757ad67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb117ad6d, SUC_lexicon_tags_1 },
    { 0x6617ad6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f77ad71, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf7ad76, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xee17ad79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1cb7ad7b, SUC_lexicon_tags_17 },
    { 0xaa97ad7b, SUC_lexicon_tags_37 },
    { 0xa237ad7b, SUC_lexicon_tags_37 },
    { 0x8a57ad7e, SUC_lexicon_tags_2 },
    { 0x7737ad7f, SUC_lexicon_tags_26 },
    { 0xfbf7ad80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2eb7ad82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6db7ad84, SUC_lexicon_tags_11 },
    { 0x15f7ad85, SUC_lexicon_tags_2 },
    { 0xeb17ad86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f7ad8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf737ad8d, SUC_lexicon_tags_84 },
    { 0x1557ad8e, SUC_lexicon_tags_43 },
    { 0xd297ad8d, SUC_lexicon_tags_79 },
    { 0x6817ad90, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xd337ad93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2537ad96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4717ad9c, SUC_lexicon_tags_25 },
    { 0x8e37ad9d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe797ada2, SUC_lexicon_tags_31 },
    { 0x78d7ada2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f7ada8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa977adb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f7adb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec97adbd, SUC_lexicon_tags_60 },
    { 0x1fb7adbe, SUC_lexicon_tags_9 },
    { 0x9117adbe, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf177adc4, SUC_lexicon_tags_0 },
    { 0x897adc5, SUC_lexicon_tags_21 },
    { 0x49d7adc5, SUC_lexicon_tags_28 },
    { 0x12d7adc7, SUC_lexicon_tags_11 },
    { 0x2297adc8, SUC_lexicon_tags_27 },
    { 0xe217adc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b7adcc, SUC_lexicon_tags_301 },
    { 0x1a17adcc, SUC_lexicon_tags_12 },
    { 0xc817adcd, SUC_lexicon_tags_49 },
    { 0xcbb7adcf, SUC_lexicon_tags_11 },
    { 0x8e17add0, SUC_lexicon_tags_12 },
    { 0x96b7add1, SUC_lexicon_tags_23 },
    { 0xee77adcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d7add5, SUC_lexicon_tags_193 },
    { 0x59f7add6, SUC_lexicon_tags_12 },
    { 0x23b7add7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c37adda, SUC_lexicon_tags_34 },
    { 0xd5f7addb, SUC_lexicon_tags_49 },
    { 0x477addb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f7ade1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d7ade7, SUC_lexicon_tags_17 },
    { 0xad57ade7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97d7adea, SUC_lexicon_tags_65 },
    { 0x3c17adeb, SUC_lexicon_tags_61 },
    { 0x8bb7adeb, SUC_lexicon_tags_17 },
    { 0xd4f7aded, SUC_lexicon_tags_40 },
    { 0x8357aded, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb7adf3, SUC_lexicon_tags_11 },
    { 0xba57adf3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc717adf6, SUC_lexicon_tags_17 },
    { 0x64b7adf6, SUC_lexicon_tags_104 },
    { 0x9d37adf6, SUC_lexicon_tags_11 },
    { 0x2cf7adf9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4677adfc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a57adff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23b7ae01, SUC_lexicon_tags_7 },
    { 0x6ad7ae02, SUC_lexicon_tags_65 },
    { 0xbdb7ae03, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f7ae07, SUC_lexicon_tags_87 },
    { 0x697ae08, SUC_lexicon_tags_38 },
    { 0x74f7ae09, SUC_lexicon_tags_17 },
    { 0xa537ae09, SUC_lexicon_tags_11 },
    { 0x8d97ae09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f7ae11, SUC_lexicon_tags_31 },
    { 0x697ae12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd57ae14, SUC_lexicon_tags_37 },
    { 0x5097ae14, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b37ae19, SUC_lexicon_tags_2 },
    { 0xb4d7ae19, SUC_lexicon_tags_28 },
    { 0xa5f7ae1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8857ae1f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7257ae23, SUC_lexicon_tags_29 },
    { 0xd17ae24, SUC_lexicon_tags_26 },
    { 0xac17ae25, SUC_lexicon_tags_9 },
    { 0xc4d7ae26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3857ae28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6077ae33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4237ae3c, SUC_lexicon_tags_39 },
    { 0xe037ae3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff7ae40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdd7ae42, SUC_lexicon_tags_12 },
    { 0xf9b7ae43, SUC_lexicon_tags_17 },
    { 0xe4f7ae44, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf7ae4c, SUC_lexicon_tags_12 },
    { 0x2f57ae4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc457ae4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf7ae59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa817ae5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d7ae61, SUC_lexicon_tags_87 },
    { 0x1fd7ae62, SUC_lexicon_tags_14 },
    { 0x1817ae62, SUC_lexicon_tags_25 },
    { 0x6517ae61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a17ae6b, SUC_lexicon_tags_14 },
    { 0xe1d7ae6c, SUC_lexicon_tags_63 },
    { 0x1937ae6d, SUC_lexicon_tags_27 },
    { 0x8217ae6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13d7ae70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf7ae75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b7ae7b, SUC_lexicon_tags_2 },
    { 0x66d7ae7c, SUC_lexicon_tags_14 },
    { 0xf3d7ae7c, SUC_lexicon_tags_31 },
    { 0x60f7ae7c, SUC_lexicon_tags_27 },
    { 0xceb7ae7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d7ae83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1497ae85, SUC_lexicon_tags_27 },
    { 0xe377ae86, SUC_lexicon_tags_812 },
    { 0, NULL },
    { 0x4c17ae88, SUC_lexicon_tags_27 },
    { 0xc097ae89, SUC_lexicon_tags_17 },
    { 0xbef7ae89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd857ae90, SUC_lexicon_tags_34 },
    { 0x4277ae91, SUC_lexicon_tags_1 },
    { 0x4b37ae92, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x26f7ae94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b17ae96, SUC_lexicon_tags_1 },
    { 0xe957ae97, SUC_lexicon_tags_25 },
    { 0xd777ae98, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6777ae9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc297ae9f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3797aea1, SUC_lexicon_tags_11 },
    { 0x5bf7aea2, SUC_lexicon_tags_1 },
    { 0x89f7aea2, SUC_lexicon_tags_16 },
    { 0x5957aea4, SUC_lexicon_tags_65 },
    { 0x58b7aea5, SUC_lexicon_tags_1 },
    { 0x9797aea3, SUC_lexicon_tags_15 },
    { 0x59f7aea5, SUC_lexicon_tags_7 },
    { 0x21f7aea2, SUC_lexicon_tags_11 },
    { 0x6757aea8, SUC_lexicon_tags_37 },
    { 0x3ab7aeaa, SUC_lexicon_tags_11 },
    { 0xc857aea2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6697aead, SUC_lexicon_tags_57 },
    { 0x1997aeae, SUC_lexicon_tags_47 },
    { 0x43b7aeaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf137aeb4, SUC_lexicon_tags_1 },
    { 0x7617aeb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfd7aeb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc157aebb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b7aebf, SUC_lexicon_tags_27 },
    { 0xe897aebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d7aec3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e77aec6, SUC_lexicon_tags_37 },
    { 0xb857aec6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30f7aec9, SUC_lexicon_tags_15 },
    { 0xac97aec9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f7aece, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96b7aed0, SUC_lexicon_tags_61 },
    { 0x6c77aed0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd617aed3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9717aede, SUC_lexicon_tags_17 },
    { 0xbf57aedf, SUC_lexicon_tags_38 },
    { 0xaf7aee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf77aeea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb7aeee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb777aef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e77aef4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe877aef9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x63b7aefb, SUC_lexicon_tags_17 },
    { 0xc8d7aefc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f77af01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a37af03, SUC_lexicon_tags_16 },
    { 0x6937af04, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d17af07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf7af0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d7af0f, SUC_lexicon_tags_8 },
    { 0xe5b7af10, SUC_lexicon_tags_15 },
    { 0xa77af11, SUC_lexicon_tags_7 },
    { 0x1db7af12, SUC_lexicon_tags_45 },
    { 0xde97af13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b17af20, SUC_lexicon_tags_60 },
    { 0xecd7af20, SUC_lexicon_tags_47 },
    { 0x72b7af22, SUC_lexicon_tags_1 },
    { 0x2357af22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9017af27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xffd7af29, SUC_lexicon_tags_17 },
    { 0x75f7af2a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2a37af2c, SUC_lexicon_tags_28 },
    { 0x6297af2d, SUC_lexicon_tags_25 },
    { 0x15b7af2d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b7af33, SUC_lexicon_tags_10 },
    { 0xa737af33, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4e37af36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d57af39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf197af3b, SUC_lexicon_tags_12 },
    { 0xee37af3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f7af41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb557af44, SUC_lexicon_tags_34 },
    { 0xdb77af45, SUC_lexicon_tags_17 },
    { 0x5637af46, SUC_lexicon_tags_41 },
    { 0x2cd7af46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa297af4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d7af4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea37af51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b7af57, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x59b7af59, SUC_lexicon_tags_1 },
    { 0x2db7af59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2077af5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4857af5e, SUC_lexicon_tags_37 },
    { 0x9017af5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6537af62, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe117af65, SUC_lexicon_tags_11 },
    { 0x8fd7af66, SUC_lexicon_tags_25 },
    { 0xd117af67, SUC_lexicon_tags_8 },
    { 0xdf37af68, SUC_lexicon_tags_15 },
    { 0x85b7af66, SUC_lexicon_tags_12 },
    { 0xcad7af6a, SUC_lexicon_tags_17 },
    { 0x8e77af6a, SUC_lexicon_tags_17 },
    { 0x1fd7af65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5957af6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2937af73, SUC_lexicon_tags_2 },
    { 0xa717af73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b7af7a, SUC_lexicon_tags_19 },
    { 0xe277af7b, SUC_lexicon_tags_51 },
    { 0x8137af7c, SUC_lexicon_tags_60 },
    { 0x7d97af7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f17af86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84d7af88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76d7af8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa677af8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd7af93, SUC_lexicon_tags_27 },
    { 0x10d7af94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6197af96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f77af9c, SUC_lexicon_tags_17 },
    { 0xa177af9d, SUC_lexicon_tags_8 },
    { 0x3457af9e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x76b7afa0, SUC_lexicon_tags_30 },
    { 0x2b17afa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197afa8, SUC_lexicon_tags_11 },
    { 0xd497afa8, SUC_lexicon_tags_23 },
    { 0x2857afaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e37afac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2537afae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7657afb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2517afb3, SUC_lexicon_tags_7 },
    { 0xbdb7afb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17afb7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc037afbb, SUC_lexicon_tags_0 },
    { 0x3277afbc, SUC_lexicon_tags_7 },
    { 0x4797afbb, SUC_lexicon_tags_2 },
    { 0xf237afbb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4497afc0, SUC_lexicon_tags_15 },
    { 0x2437afc1, SUC_lexicon_tags_23 },
    { 0x9937afc2, SUC_lexicon_tags_65 },
    { 0x68f7afc0, SUC_lexicon_tags_65 },
    { 0x2f17afc4, SUC_lexicon_tags_2 },
    { 0x3b57afc5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7afca, SUC_lexicon_tags_37 },
    { 0xf2b7afca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5597afcd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa117afcf, SUC_lexicon_tags_32 },
    { 0xfcb7afcf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa497afd5, SUC_lexicon_tags_37 },
    { 0xc1b7afd5, SUC_lexicon_tags_48 },
    { 0x2e17afd7, SUC_lexicon_tags_28 },
    { 0xd957afd7, SUC_lexicon_tags_63 },
    { 0xce37afd9, SUC_lexicon_tags_1 },
    { 0x10b7afda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb917afdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe157afe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e17afe7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b7aff0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5bf7aff2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb7aff5, SUC_lexicon_tags_25 },
    { 0xbc37aff6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b7affa, SUC_lexicon_tags_29 },
    { 0x7db7affb, SUC_lexicon_tags_15 },
    { 0xa517affc, SUC_lexicon_tags_23 },
    { 0xe517affb, SUC_lexicon_tags_14 },
    { 0x6357affb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d7b003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd617b005, SUC_lexicon_tags_6 },
    { 0x7b7b005, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f7b00e, SUC_lexicon_tags_11 },
    { 0x38f7b00e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e77b012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf797b01d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3917b01f, SUC_lexicon_tags_34 },
    { 0x14b7b020, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d7b024, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b57b02b, SUC_lexicon_tags_7 },
    { 0x8ef7b02b, SUC_lexicon_tags_1 },
    { 0x28b7b02d, SUC_lexicon_tags_27 },
    { 0x20f7b02d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa77b033, SUC_lexicon_tags_47 },
    { 0xd097b034, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f7b03c, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b7b041, SUC_lexicon_tags_17 },
    { 0x5b17b042, SUC_lexicon_tags_12 },
    { 0x6417b042, SUC_lexicon_tags_11 },
    { 0xb317b044, SUC_lexicon_tags_33 },
    { 0x7bf7b043, SUC_lexicon_tags_25 },
    { 0xa297b046, SUC_lexicon_tags_47 },
    { 0x1a97b045, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f7b04a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bd7b04c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa657b051, SUC_lexicon_tags_6 },
    { 0x4837b051, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8577b055, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf7b058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b7b05d, SUC_lexicon_tags_47 },
    { 0x56b7b05e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f7b061, SUC_lexicon_tags_12 },
    { 0x7697b061, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c17b067, SUC_lexicon_tags_38 },
    { 0xd417b068, SUC_lexicon_tags_6 },
    { 0x11b7b068, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb7b06f, SUC_lexicon_tags_1 },
    { 0xe57b06f, SUC_lexicon_tags_1 },
    { 0x2577b071, SUC_lexicon_tags_1 },
    { 0xb097b072, SUC_lexicon_tags_30 },
    { 0xae17b073, SUC_lexicon_tags_378 },
    { 0x6517b074, SUC_lexicon_tags_37 },
    { 0x1d7b074, SUC_lexicon_tags_14 },
    { 0x1557b076, SUC_lexicon_tags_26 },
    { 0x8bb7b074, SUC_lexicon_tags_55 },
    { 0x5117b078, SUC_lexicon_tags_17 },
    { 0x8137b079, SUC_lexicon_tags_26 },
    { 0xe157b079, SUC_lexicon_tags_16 },
    { 0xaab7b077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8737b07e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4337b080, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1c57b082, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6757b084, SUC_lexicon_tags_2 },
    { 0xd3d7b084, SUC_lexicon_tags_6 },
    { 0xc7d7b086, SUC_lexicon_tags_19 },
    { 0x2737b087, SUC_lexicon_tags_25 },
    { 0x3db7b087, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8577b08b, SUC_lexicon_tags_14 },
    { 0xaaf7b08c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb077b08f, SUC_lexicon_tags_6 },
    { 0xb837b090, SUC_lexicon_tags_7 },
    { 0xe017b091, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2937b094, SUC_lexicon_tags_118 },
    { 0x7097b095, SUC_lexicon_tags_95 },
    { 0x6cf7b095, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf97b09e, SUC_lexicon_tags_35 },
    { 0x80f7b09f, SUC_lexicon_tags_15 },
    { 0x2537b09f, SUC_lexicon_tags_23 },
    { 0xdcb7b0a1, SUC_lexicon_tags_30 },
    { 0x8fd7b0a0, SUC_lexicon_tags_29 },
    { 0x7517b0a3, SUC_lexicon_tags_39 },
    { 0x75b7b09f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b77b0a8, SUC_lexicon_tags_12 },
    { 0x28b7b0a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73f7b0ab, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9397b0ad, SUC_lexicon_tags_33 },
    { 0x8c97b0ae, SUC_lexicon_tags_26 },
    { 0x8077b0ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c57b0b2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce77b0b7, SUC_lexicon_tags_12 },
    { 0xd6b7b0b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3297b0bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd7b0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa897b0cb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4617b0ce, SUC_lexicon_tags_1 },
    { 0x997b0ce, SUC_lexicon_tags_14 },
    { 0xaaf7b0d0, SUC_lexicon_tags_15 },
    { 0xd177b0d1, SUC_lexicon_tags_47 },
    { 0x44b7b0d2, SUC_lexicon_tags_62 },
    { 0xb817b0d0, SUC_lexicon_tags_28 },
    { 0x51b7b0d4, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f7b0d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcaf7b0db, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc497b0e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f7b0e7, SUC_lexicon_tags_28 },
    { 0xedb7b0e8, SUC_lexicon_tags_37 },
    { 0xb5f7b0e9, SUC_lexicon_tags_15 },
    { 0x7777b0e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5057b0f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b7b0f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f57b0f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4d17b0fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f7b0ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef37b106, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b7b109, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f7b111, SUC_lexicon_tags_25 },
    { 0xceb7b112, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf817b114, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e37b116, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc117b118, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7b11c, SUC_lexicon_tags_37 },
    { 0x5ad7b11d, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa737b121, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2997b124, SUC_lexicon_tags_64 },
    { 0xca97b125, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d17b128, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d7b12c, SUC_lexicon_tags_25 },
    { 0x7177b12c, SUC_lexicon_tags_9 },
    { 0xc657b12d, SUC_lexicon_tags_11 },
    { 0xc877b12f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8877b131, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe377b133, SUC_lexicon_tags_23 },
    { 0x24f7b133, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9097b137, SUC_lexicon_tags_29 },
    { 0xc677b138, SUC_lexicon_tags_19 },
    { 0xf477b139, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5157b13c, SUC_lexicon_tags_17 },
    { 0xcfb7b13d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab7b143, SUC_lexicon_tags_7 },
    { 0x2f17b144, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf7b149, SUC_lexicon_tags_25 },
    { 0xf8f7b14a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe097b14d, SUC_lexicon_tags_30 },
    { 0x1117b14e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d7b153, SUC_lexicon_tags_15 },
    { 0x7417b154, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff97b15f, SUC_lexicon_tags_37 },
    { 0x3977b160, SUC_lexicon_tags_27 },
    { 0x6077b160, SUC_lexicon_tags_37 },
    { 0xaed7b15f, SUC_lexicon_tags_37 },
    { 0x3557b15f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b17b169, SUC_lexicon_tags_7 },
    { 0xa1b7b16a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d7b16d, SUC_lexicon_tags_12 },
    { 0x2817b16e, SUC_lexicon_tags_29 },
    { 0xbb97b16d, SUC_lexicon_tags_68 },
    { 0x38f7b16e, SUC_lexicon_tags_2 },
    { 0xbd7b171, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f97b174, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79f7b176, SUC_lexicon_tags_7 },
    { 0x77d7b177, SUC_lexicon_tags_65 },
    { 0x7ef7b178, SUC_lexicon_tags_14 },
    { 0xee57b179, SUC_lexicon_tags_70 },
    { 0xa037b17a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2f7b17c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9437b17f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc857b182, SUC_lexicon_tags_60 },
    { 0x5ef7b183, SUC_lexicon_tags_15 },
    { 0x1157b183, SUC_lexicon_tags_17 },
    { 0x50d7b184, SUC_lexicon_tags_65 },
    { 0x6337b186, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe077b18f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe437b191, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d7b197, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb7b19a, SUC_lexicon_tags_25 },
    { 0x837b19b, SUC_lexicon_tags_49 },
    { 0xb7b7b19c, SUC_lexicon_tags_15 },
    { 0x9ad7b19b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9777b1a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x237b1a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b7b1a7, SUC_lexicon_tags_1270 },
    { 0x11d7b1a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe897b1ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f97b1ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5357b1b5, SUC_lexicon_tags_46 },
    { 0x4957b1b6, SUC_lexicon_tags_33 },
    { 0x3637b1b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7b1ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd57b1be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27f7b1c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd7b1c3, SUC_lexicon_tags_2 },
    { 0xb557b1c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f97b1c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8997b1cc, SUC_lexicon_tags_25 },
    { 0x49d7b1cd, SUC_lexicon_tags_27 },
    { 0xdbf7b1ce, SUC_lexicon_tags_6 },
    { 0x5c17b1cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb7b1d5, SUC_lexicon_tags_60 },
    { 0xc477b1d6, SUC_lexicon_tags_19 },
    { 0xc8b7b1d7, SUC_lexicon_tags_2 },
    { 0x17f7b1d6, SUC_lexicon_tags_47 },
    { 0x737b1d6, SUC_lexicon_tags_7 },
    { 0x8ab7b1da, SUC_lexicon_tags_15 },
    { 0xbbf7b1d9, SUC_lexicon_tags_37 },
    { 0xe2b7b1dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4817b1de, SUC_lexicon_tags_16 },
    { 0xf7b1de, SUC_lexicon_tags_48 },
    { 0x2077b1de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437b1e4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f7b1e8, SUC_lexicon_tags_74 },
    { 0xa9d7b1e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa17b1eb, SUC_lexicon_tags_1 },
    { 0x3277b1eb, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f7b1ef, SUC_lexicon_tags_23 },
    { 0xe9b7b1f0, SUC_lexicon_tags_14 },
    { 0x2e77b1f1, SUC_lexicon_tags_1 },
    { 0x2797b1f2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4cb7b1f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x997b1f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7337b1fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a97b205, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f37b209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b37b20c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3037b212, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef7b216, SUC_lexicon_tags_17 },
    { 0xd837b217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77b219, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff7b220, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe4f7b222, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddb7b224, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d57b228, SUC_lexicon_tags_8 },
    { 0x2af7b229, SUC_lexicon_tags_60 },
    { 0xacb7b229, SUC_lexicon_tags_11 },
    { 0x47b7b229, SUC_lexicon_tags_104 },
    { 0x78f7b22c, SUC_lexicon_tags_7 },
    { 0x6657b228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f7b232, SUC_lexicon_tags_45 },
    { 0xed7b233, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7b238, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3857b23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6677b23e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5137b241, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d7b244, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7457b24a, SUC_lexicon_tags_17 },
    { 0x80f7b24b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f7b251, SUC_lexicon_tags_9 },
    { 0xb877b252, SUC_lexicon_tags_14 },
    { 0xe97b253, SUC_lexicon_tags_34 },
    { 0x6f17b254, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b7b258, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe597b25a, SUC_lexicon_tags_23 },
    { 0x6977b25b, SUC_lexicon_tags_63 },
    { 0x7717b25b, SUC_lexicon_tags_12 },
    { 0xfef7b25d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f7b260, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f97b262, SUC_lexicon_tags_29 },
    { 0x3957b263, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4337b265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b97b273, SUC_lexicon_tags_15 },
    { 0xb4f7b273, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45d7b276, SUC_lexicon_tags_1 },
    { 0xaad7b277, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd6d7b279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64b7b27b, SUC_lexicon_tags_141 },
    { 0x2c37b27b, SUC_lexicon_tags_0 },
    { 0xde77b27d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b7b281, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd1b7b283, SUC_lexicon_tags_11 },
    { 0xcfd7b284, SUC_lexicon_tags_11 },
    { 0xb317b285, SUC_lexicon_tags_30 },
    { 0x17b7b285, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b77b288, SUC_lexicon_tags_17 },
    { 0x60f7b289, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f7b28c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5617b28e, SUC_lexicon_tags_68 },
    { 0x957b28e, SUC_lexicon_tags_1 },
    { 0x9cd7b28e, SUC_lexicon_tags_61 },
    { 0x1117b291, SUC_lexicon_tags_1 },
    { 0x69b7b290, SUC_lexicon_tags_11 },
    { 0xe237b293, SUC_lexicon_tags_17 },
    { 0x9e37b294, SUC_lexicon_tags_33 },
    { 0x95d7b295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x917b297, SUC_lexicon_tags_56 },
    { 0x517b298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4977b29b, SUC_lexicon_tags_32 },
    { 0x5097b29c, SUC_lexicon_tags_29 },
    { 0xaf97b29b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6037b2a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb7b2a5, SUC_lexicon_tags_33 },
    { 0xb5b7b2a5, SUC_lexicon_tags_26 },
    { 0xe097b2a5, SUC_lexicon_tags_49 },
    { 0x5ef7b2a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe877b2ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b7b2b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e97b2b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb7b2bd, SUC_lexicon_tags_17 },
    { 0x27f7b2be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd37b2c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d57b2c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa617b2c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7077b2c8, SUC_lexicon_tags_37 },
    { 0x3ff7b2c9, SUC_lexicon_tags_1 },
    { 0x28b7b2ca, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a77b2cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b7b2d0, SUC_lexicon_tags_45 },
    { 0xba57b2d0, SUC_lexicon_tags_14 },
    { 0xb1f7b2d2, SUC_lexicon_tags_25 },
    { 0xbfd7b2d3, SUC_lexicon_tags_33 },
    { 0xfff7b2d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd57b2d6, SUC_lexicon_tags_27 },
    { 0x53d7b2d6, SUC_lexicon_tags_66 },
    { 0x6197b2d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f97b2db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8ad7b2dd, SUC_lexicon_tags_23 },
    { 0xe677b2de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5617b2e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3b7b2e4, SUC_lexicon_tags_1 },
    { 0xfa77b2e4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf3b7b2e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37b7b2e9, SUC_lexicon_tags_39 },
    { 0xac17b2ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab77b2f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x80b7b2f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f7b2f9, SUC_lexicon_tags_37 },
    { 0x4957b2f9, SUC_lexicon_tags_53 },
    { 0xd197b2fb, SUC_lexicon_tags_27 },
    { 0xcfb7b2fc, SUC_lexicon_tags_39 },
    { 0xa977b2fc, SUC_lexicon_tags_15 },
    { 0x8457b2fe, SUC_lexicon_tags_32 },
    { 0xd17b2ff, SUC_lexicon_tags_9 },
    { 0x3917b300, SUC_lexicon_tags_1 },
    { 0x3617b2fb, SUC_lexicon_tags_28 },
    { 0xba97b2ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4d97b304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd77b306, SUC_lexicon_tags_8 },
    { 0x96b7b307, SUC_lexicon_tags_8 },
    { 0x5f7b307, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3677b30a, SUC_lexicon_tags_19 },
    { 0xa357b30b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe5d7b30d, SUC_lexicon_tags_55 },
    { 0x8bb7b30d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe057b314, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb117b317, SUC_lexicon_tags_2 },
    { 0x86d7b318, SUC_lexicon_tags_17 },
    { 0xb857b319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3337b31c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf277b321, SUC_lexicon_tags_25 },
    { 0xed17b321, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d17b327, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d17b32d, SUC_lexicon_tags_7 },
    { 0x3cf7b32d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f17b332, SUC_lexicon_tags_63 },
    { 0xc097b333, SUC_lexicon_tags_21 },
    { 0x25f7b334, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb7b33b, SUC_lexicon_tags_1 },
    { 0x6a57b33c, SUC_lexicon_tags_30 },
    { 0x4c17b33b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb597b341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7b346, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8737b348, SUC_lexicon_tags_2 },
    { 0xd697b349, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1f77b34b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b7b34f, SUC_lexicon_tags_11 },
    { 0xd137b350, SUC_lexicon_tags_34 },
    { 0xb5b7b351, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17b354, SUC_lexicon_tags_37 },
    { 0xb057b355, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d7b358, SUC_lexicon_tags_17 },
    { 0x2577b359, SUC_lexicon_tags_16 },
    { 0xadb7b35a, SUC_lexicon_tags_0 },
    { 0xa4b7b35a, SUC_lexicon_tags_9 },
    { 0xc937b358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa677b365, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b7b36f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7b373, SUC_lexicon_tags_27 },
    { 0xfe57b373, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb7b379, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a17b37c, SUC_lexicon_tags_42 },
    { 0x1677b37d, SUC_lexicon_tags_17 },
    { 0x1897b37e, SUC_lexicon_tags_11 },
    { 0x1637b37f, SUC_lexicon_tags_20 },
    { 0x9557b37c, SUC_lexicon_tags_45 },
    { 0xfc57b381, SUC_lexicon_tags_116 },
    { 0xc357b382, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7357b385, SUC_lexicon_tags_48 },
    { 0xfcf7b385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8f7b388, SUC_lexicon_tags_47 },
    { 0x3477b389, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb7b38e, SUC_lexicon_tags_7 },
    { 0x7bd7b38f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f7b395, SUC_lexicon_tags_14 },
    { 0x7bf7b395, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7b399, SUC_lexicon_tags_25 },
    { 0xb9b7b39a, SUC_lexicon_tags_0 },
    { 0x4977b39b, SUC_lexicon_tags_61 },
    { 0xc717b39c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f7b3a1, SUC_lexicon_tags_8 },
    { 0x9eb7b3a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd757b3a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1997b3aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa877b3ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7257b3ae, SUC_lexicon_tags_25 },
    { 0x37b3af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2977b3b7, SUC_lexicon_tags_29 },
    { 0x9e37b3b8, SUC_lexicon_tags_19 },
    { 0x4a57b3b8, SUC_lexicon_tags_4 },
    { 0x1617b3b8, SUC_lexicon_tags_15 },
    { 0x2477b3bb, SUC_lexicon_tags_12 },
    { 0xca57b3b8, SUC_lexicon_tags_31 },
    { 0xde97b3bd, SUC_lexicon_tags_1 },
    { 0xbef7b3bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b7b3c1, SUC_lexicon_tags_63 },
    { 0xaa77b3c2, SUC_lexicon_tags_1 },
    { 0x4bf7b3c3, SUC_lexicon_tags_402 },
    { 0x6b7b3c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb97b3c6, SUC_lexicon_tags_47 },
    { 0x2df7b3c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8157b3cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d7b3d2, SUC_lexicon_tags_1 },
    { 0x6377b3d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x13b7b3d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa837b3dc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x817b3df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3417b3e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bf7b3e3, SUC_lexicon_tags_37 },
    { 0x3497b3e3, SUC_lexicon_tags_38 },
    { 0xe057b3e5, SUC_lexicon_tags_28 },
    { 0xae57b3e5, SUC_lexicon_tags_1 },
    { 0x4b17b3e7, SUC_lexicon_tags_7 },
    { 0x26f7b3e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c97b3f3, SUC_lexicon_tags_7 },
    { 0x5217b3f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xad7b3f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb7b3fa, SUC_lexicon_tags_17 },
    { 0xc3d7b3fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb597b3ff, SUC_lexicon_tags_37 },
    { 0x50b7b400, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f7b406, SUC_lexicon_tags_45 },
    { 0xaff7b407, SUC_lexicon_tags_17 },
    { 0x4d7b408, SUC_lexicon_tags_39 },
    { 0x6997b408, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd97b40e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d7b414, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd457b416, SUC_lexicon_tags_26 },
    { 0xfd57b417, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe37b419, SUC_lexicon_tags_30 },
    { 0x1517b41a, SUC_lexicon_tags_61 },
    { 0xd817b41a, SUC_lexicon_tags_61 },
    { 0x6257b41c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf297b41f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d17b422, SUC_lexicon_tags_10 },
    { 0xbbd7b423, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4997b427, SUC_lexicon_tags_34 },
    { 0x1477b428, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8017b42d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe837b430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2657b435, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6637b43a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd057b43d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2017b440, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad7b44a, SUC_lexicon_tags_27 },
    { 0xce97b44a, SUC_lexicon_tags_2 },
    { 0x9737b44a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3477b452, SUC_lexicon_tags_17 },
    { 0xa0d7b453, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85d7b455, SUC_lexicon_tags_31 },
    { 0xa237b456, SUC_lexicon_tags_30 },
    { 0x9957b457, SUC_lexicon_tags_31 },
    { 0xae57b457, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9697b45c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd7b460, SUC_lexicon_tags_4 },
    { 0x7957b461, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a37b463, SUC_lexicon_tags_29 },
    { 0xba37b463, SUC_lexicon_tags_33 },
    { 0xc9f7b465, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a57b46c, SUC_lexicon_tags_17 },
    { 0xe77b46c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd7b476, SUC_lexicon_tags_12 },
    { 0x8fb7b477, SUC_lexicon_tags_17 },
    { 0x96f7b478, SUC_lexicon_tags_1110 },
    { 0x5817b479, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7c37b47b, SUC_lexicon_tags_33 },
    { 0x1ad7b47b, SUC_lexicon_tags_37 },
    { 0x99d7b47c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf7b47f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f7b489, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1757b48d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a77b494, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b7b498, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a57b4a0, SUC_lexicon_tags_5 },
    { 0x6477b4a1, SUC_lexicon_tags_30 },
    { 0xe37b4a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf277b4ab, SUC_lexicon_tags_17 },
    { 0x9417b4ac, SUC_lexicon_tags_60 },
    { 0xecd7b4ac, SUC_lexicon_tags_15 },
    { 0x17b4ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb8f7b4b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb7b4b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb997b4b8, SUC_lexicon_tags_2 },
    { 0x2177b4b8, SUC_lexicon_tags_13 },
    { 0x1c37b4ba, SUC_lexicon_tags_87 },
    { 0x9c57b4bb, SUC_lexicon_tags_43 },
    { 0x2eb7b4bc, SUC_lexicon_tags_19 },
    { 0x20d7b4bc, SUC_lexicon_tags_8 },
    { 0x6fb7b4be, SUC_lexicon_tags_25 },
    { 0x3717b4bf, SUC_lexicon_tags_6 },
    { 0x5757b4c0, SUC_lexicon_tags_39 },
    { 0x9f97b4c0, SUC_lexicon_tags_8 },
    { 0x4957b4bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe257b4c4, SUC_lexicon_tags_8 },
    { 0x7ff7b4c5, SUC_lexicon_tags_48 },
    { 0xa6d7b4c4, SUC_lexicon_tags_23 },
    { 0xb1f7b4c7, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f97b4cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e57b4cd, SUC_lexicon_tags_56 },
    { 0x3b57b4ce, SUC_lexicon_tags_11 },
    { 0x1457b4ce, SUC_lexicon_tags_11 },
    { 0x6d37b4cf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x55f7b4d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3657b4d6, SUC_lexicon_tags_27 },
    { 0x977b4d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8af7b4d9, SUC_lexicon_tags_47 },
    { 0xc8f7b4d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad7b4e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f7b4ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41d7b4ec, SUC_lexicon_tags_17 },
    { 0x9337b4ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d7b4f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7497b4f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8597b4fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b7b4fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa797b500, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d7b504, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd7b509, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7797b50f, SUC_lexicon_tags_38 },
    { 0x4f7b510, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7317b513, SUC_lexicon_tags_63 },
    { 0x9f7b514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28b7b516, SUC_lexicon_tags_20 },
    { 0x5337b517, SUC_lexicon_tags_11 },
    { 0x6b57b518, SUC_lexicon_tags_48 },
    { 0xce17b516, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42d7b51b, SUC_lexicon_tags_14 },
    { 0x31d7b51b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f7b521, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9357b52b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7b7b52d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ed7b52f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6737b532, SUC_lexicon_tags_48 },
    { 0x57b533, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d7b537, SUC_lexicon_tags_1 },
    { 0xbf17b537, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3877b53b, SUC_lexicon_tags_7 },
    { 0xadb7b53c, SUC_lexicon_tags_7 },
    { 0x90d7b53d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d7b540, SUC_lexicon_tags_9 },
    { 0x1c97b541, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b37b545, SUC_lexicon_tags_38 },
    { 0x35f7b546, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b7b549, SUC_lexicon_tags_37 },
    { 0x9e57b549, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa1d7b54c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f57b54e, SUC_lexicon_tags_8 },
    { 0x9bf7b54f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f77b551, SUC_lexicon_tags_65 },
    { 0x8057b552, SUC_lexicon_tags_48 },
    { 0x4db7b552, SUC_lexicon_tags_57 },
    { 0x70d7b554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2957b557, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8437b55b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d7b55f, SUC_lexicon_tags_15 },
    { 0xa677b55f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5637b564, SUC_lexicon_tags_2 },
    { 0x15f7b565, SUC_lexicon_tags_15 },
    { 0xd8b7b565, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d7b569, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb217b56c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6557b56f, SUC_lexicon_tags_30 },
    { 0xeed7b570, SUC_lexicon_tags_25 },
    { 0x13b7b571, SUC_lexicon_tags_45 },
    { 0xcc77b571, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f17b579, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8957b57b, SUC_lexicon_tags_14 },
    { 0x2317b57c, SUC_lexicon_tags_37 },
    { 0x1397b57c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4037b57f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad7b586, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b37b589, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77b58c, SUC_lexicon_tags_37 },
    { 0x9c97b58d, SUC_lexicon_tags_172 },
    { 0xa9d7b58d, SUC_lexicon_tags_27 },
    { 0x1597b58c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xef97b592, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf7b599, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65b7b59b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f57b5a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5457b5a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b7b5ab, SUC_lexicon_tags_113 },
    { 0xa4b7b5ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb7b5b1, SUC_lexicon_tags_47 },
    { 0xf437b5b1, SUC_lexicon_tags_33 },
    { 0xf277b5b3, SUC_lexicon_tags_57 },
    { 0xa157b5b2, SUC_lexicon_tags_77 },
    { 0x8d97b5b2, SUC_lexicon_tags_1 },
    { 0x1017b5b1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6817b5b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d7b5bb, SUC_lexicon_tags_47 },
    { 0x4e17b5bc, SUC_lexicon_tags_38 },
    { 0x87b7b5bb, SUC_lexicon_tags_13 },
    { 0x26d7b5bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0f7b5c0, SUC_lexicon_tags_12 },
    { 0x4d57b5c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9ab7b5c3, SUC_lexicon_tags_14 },
    { 0xd4b7b5c4, SUC_lexicon_tags_14 },
    { 0x5c77b5c3, SUC_lexicon_tags_9 },
    { 0x98f7b5c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9557b5c8, SUC_lexicon_tags_7 },
    { 0x7877b5c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc697b5cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf7b5d0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c97b5d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2597b5db, SUC_lexicon_tags_26 },
    { 0x6f17b5dc, SUC_lexicon_tags_1 },
    { 0xc337b5dd, SUC_lexicon_tags_12 },
    { 0x1cb7b5dd, SUC_lexicon_tags_17 },
    { 0x32d7b5df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f97b5e2, SUC_lexicon_tags_48 },
    { 0xc457b5e2, SUC_lexicon_tags_37 },
    { 0xf9b7b5e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3797b5e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd477b5e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7197b5eb, SUC_lexicon_tags_15 },
    { 0xea97b5ec, SUC_lexicon_tags_0 },
    { 0x3ab7b5ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d7b5f1, SUC_lexicon_tags_87 },
    { 0x3757b5f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af7b5fa, SUC_lexicon_tags_17 },
    { 0xf7b7b5fb, SUC_lexicon_tags_30 },
    { 0x8897b5fb, SUC_lexicon_tags_25 },
    { 0x9cd7b5fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1837b600, SUC_lexicon_tags_65 },
    { 0xfcd7b600, SUC_lexicon_tags_113 },
    { 0xa637b601, SUC_lexicon_tags_79 },
    { 0x8c77b603, SUC_lexicon_tags_10 },
    { 0x76b7b603, SUC_lexicon_tags_17 },
    { 0x9af7b605, SUC_lexicon_tags_43 },
    { 0xa017b605, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d7b609, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaed7b60b, SUC_lexicon_tags_15 },
    { 0xddd7b60c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7297b615, SUC_lexicon_tags_12 },
    { 0x4757b615, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7b61c, SUC_lexicon_tags_14 },
    { 0xc57b61c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc517b61f, SUC_lexicon_tags_64 },
    { 0x2417b620, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9577b624, SUC_lexicon_tags_64 },
    { 0x8f77b624, SUC_lexicon_tags_60 },
    { 0x8e57b625, SUC_lexicon_tags_14 },
    { 0xaf17b627, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b7b62c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7097b62e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x76d7b630, SUC_lexicon_tags_49 },
    { 0x6937b630, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe137b639, SUC_lexicon_tags_140 },
    { 0x25f7b63a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee17b63d, SUC_lexicon_tags_7 },
    { 0x2f77b63e, SUC_lexicon_tags_30 },
    { 0xb8d7b63f, SUC_lexicon_tags_23 },
    { 0x89b7b63d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb697b643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd37b64d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf157b650, SUC_lexicon_tags_27 },
    { 0xa3b7b651, SUC_lexicon_tags_44 },
    { 0x2577b652, SUC_lexicon_tags_25 },
    { 0x88b7b650, SUC_lexicon_tags_25 },
    { 0x48d7b654, SUC_lexicon_tags_26 },
    { 0x7af7b655, SUC_lexicon_tags_25 },
    { 0x2ab7b655, SUC_lexicon_tags_25 },
    { 0x35b7b657, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b7b65a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c77b65c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf7b65f, SUC_lexicon_tags_37 },
    { 0xc337b65f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d7b665, SUC_lexicon_tags_11 },
    { 0xf917b666, SUC_lexicon_tags_17 },
    { 0xc817b667, SUC_lexicon_tags_7 },
    { 0x3cf7b666, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9577b66c, SUC_lexicon_tags_15 },
    { 0x6497b66c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c97b671, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d7b675, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597b67b, SUC_lexicon_tags_38 },
    { 0xa077b67c, SUC_lexicon_tags_27 },
    { 0x97f7b67d, SUC_lexicon_tags_2 },
    { 0xf557b67e, SUC_lexicon_tags_1 },
    { 0x5ad7b67e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb357b681, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f7b685, SUC_lexicon_tags_1 },
    { 0xfb97b686, SUC_lexicon_tags_25 },
    { 0xb97b687, SUC_lexicon_tags_11 },
    { 0xdb97b687, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8817b68f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7317b696, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb0f7b698, SUC_lexicon_tags_1 },
    { 0xcef7b698, SUC_lexicon_tags_17 },
    { 0xde57b699, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa317b69f, SUC_lexicon_tags_23 },
    { 0xf5b7b6a0, SUC_lexicon_tags_18 },
    { 0xf377b6a0, SUC_lexicon_tags_38 },
    { 0x4497b6a2, SUC_lexicon_tags_17 },
    { 0x3bd7b6a2, SUC_lexicon_tags_65 },
    { 0xf837b6a4, SUC_lexicon_tags_14 },
    { 0x79f7b6a5, SUC_lexicon_tags_30 },
    { 0x5bb7b6a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c77b6ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f7b6b3, SUC_lexicon_tags_21 },
    { 0x5617b6b4, SUC_lexicon_tags_1 },
    { 0xa737b6b5, SUC_lexicon_tags_15 },
    { 0xd617b6b5, SUC_lexicon_tags_6 },
    { 0xdd77b6b7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d77b6bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa357b6c5, SUC_lexicon_tags_87 },
    { 0x7a17b6c5, SUC_lexicon_tags_12 },
    { 0x1517b6c6, SUC_lexicon_tags_2 },
    { 0x4357b6c8, SUC_lexicon_tags_68 },
    { 0xfc37b6c9, SUC_lexicon_tags_2 },
    { 0xe997b6ca, SUC_lexicon_tags_43 },
    { 0x4bd7b6c8, SUC_lexicon_tags_66 },
    { 0x9017b6c8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd797b6ce, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc8b7b6d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4277b6d9, SUC_lexicon_tags_28 },
    { 0x7757b6d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a77b6dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa557b6df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b7b6e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde77b6e9, SUC_lexicon_tags_14 },
    { 0x64d7b6ea, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec77b6ee, SUC_lexicon_tags_19 },
    { 0x37f7b6ee, SUC_lexicon_tags_60 },
    { 0xdd7b6ef, SUC_lexicon_tags_87 },
    { 0x2337b6f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaad7b6f3, SUC_lexicon_tags_1 },
    { 0x1917b6f4, SUC_lexicon_tags_1 },
    { 0xba77b6f5, SUC_lexicon_tags_3 },
    { 0x14b7b6f4, SUC_lexicon_tags_46 },
    { 0x9057b6f7, SUC_lexicon_tags_60 },
    { 0xd5d7b6f8, SUC_lexicon_tags_17 },
    { 0xa417b6f9, SUC_lexicon_tags_10 },
    { 0x60d7b6f9, SUC_lexicon_tags_29 },
    { 0x6b37b6fb, SUC_lexicon_tags_37 },
    { 0xadf7b6f9, SUC_lexicon_tags_37 },
    { 0x2557b6f8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc757b707, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2637b709, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d7b70b, SUC_lexicon_tags_8 },
    { 0xd617b70c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e77b713, SUC_lexicon_tags_43 },
    { 0x63f7b713, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7897b722, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb6b7b724, SUC_lexicon_tags_6 },
    { 0x55f7b724, SUC_lexicon_tags_38 },
    { 0x277b726, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd7b728, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x96f7b72a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd7b72e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4af7b730, SUC_lexicon_tags_15 },
    { 0x37f7b731, SUC_lexicon_tags_27 },
    { 0x4ad7b732, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb7b73a, SUC_lexicon_tags_11 },
    { 0x9997b73b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1357b73d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x9bd7b73f, SUC_lexicon_tags_77 },
    { 0xacb7b740, SUC_lexicon_tags_33 },
    { 0x13d7b740, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5277b743, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x32b7b745, SUC_lexicon_tags_1 },
    { 0x99f7b746, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd7b749, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4cf7b74b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd7b74f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb537b753, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d7b75d, SUC_lexicon_tags_60 },
    { 0xf277b75e, SUC_lexicon_tags_7 },
    { 0x4937b75f, SUC_lexicon_tags_6 },
    { 0x4337b75f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3097b762, SUC_lexicon_tags_852 },
    { 0xed57b762, SUC_lexicon_tags_21 },
    { 0x6857b762, SUC_lexicon_tags_45 },
    { 0x6cd7b765, SUC_lexicon_tags_37 },
    { 0xbb37b765, SUC_lexicon_tags_28 },
    { 0x3b77b767, SUC_lexicon_tags_17 },
    { 0xec37b762, SUC_lexicon_tags_63 },
    { 0xdab7b769, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1737b76c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3117b76e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb7b772, SUC_lexicon_tags_47 },
    { 0x75f7b773, SUC_lexicon_tags_15 },
    { 0xfb37b773, SUC_lexicon_tags_2 },
    { 0x2fd7b775, SUC_lexicon_tags_46 },
    { 0x2eb7b775, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b77b77a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d57b77c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7037b77e, SUC_lexicon_tags_12 },
    { 0x3f37b77e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf817b781, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xda97b784, SUC_lexicon_tags_15 },
    { 0x3517b785, SUC_lexicon_tags_16 },
    { 0x2eb7b786, SUC_lexicon_tags_60 },
    { 0xc4d7b786, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0xb177b78a, SUC_lexicon_tags_11 },
    { 0x4f7b78b, SUC_lexicon_tags_43 },
    { 0x2737b78c, SUC_lexicon_tags_38 },
    { 0x2f57b78c, SUC_lexicon_tags_2 },
    { 0x2df7b78b, SUC_lexicon_tags_6 },
    { 0xeb17b78a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a77b793, SUC_lexicon_tags_15 },
    { 0x15b7b794, SUC_lexicon_tags_60 },
    { 0xa5d7b795, SUC_lexicon_tags_12 },
    { 0xed17b794, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f37b79a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe97b79d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31b7b79f, SUC_lexicon_tags_25 },
    { 0x7cb7b7a0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3217b7a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x997b7a8, SUC_lexicon_tags_38 },
    { 0x90b7b7a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac97b7ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f7b7b1, SUC_lexicon_tags_15 },
    { 0x7cb7b7b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb997b7b5, SUC_lexicon_tags_37 },
    { 0xac17b7b6, SUC_lexicon_tags_45 },
    { 0x9ff7b7b6, SUC_lexicon_tags_25 },
    { 0xcd97b7b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c17b7bb, SUC_lexicon_tags_2 },
    { 0x9477b7bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f37b7be, SUC_lexicon_tags_39 },
    { 0xbd17b7be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9857b7c1, SUC_lexicon_tags_17 },
    { 0x6037b7c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f57b7c6, SUC_lexicon_tags_12 },
    { 0x63d7b7c6, SUC_lexicon_tags_14 },
    { 0xa957b7c8, SUC_lexicon_tags_17 },
    { 0xef17b7c6, SUC_lexicon_tags_37 },
    { 0xb657b7ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97b7d1, SUC_lexicon_tags_12 },
    { 0x1b37b7d2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b77b7da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd737b7e9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9597b7eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc637b7f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb7b7f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1137b7fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad7b800, SUC_lexicon_tags_89 },
    { 0xfb37b801, SUC_lexicon_tags_1 },
    { 0xbf17b802, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x8f97b804, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1217b807, SUC_lexicon_tags_19 },
    { 0xe7d7b808, SUC_lexicon_tags_47 },
    { 0x4057b808, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac17b80f, SUC_lexicon_tags_2 },
    { 0x757b80f, SUC_lexicon_tags_35 },
    { 0x14d7b811, SUC_lexicon_tags_38 },
    { 0x2357b812, SUC_lexicon_tags_17 },
    { 0x4597b813, SUC_lexicon_tags_80 },
    { 0x577b812, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae37b819, SUC_lexicon_tags_61 },
    { 0x50b7b819, SUC_lexicon_tags_37 },
    { 0x8837b81b, SUC_lexicon_tags_6 },
    { 0x7f77b819, SUC_lexicon_tags_38 },
    { 0xe6b7b81d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f77b825, SUC_lexicon_tags_12 },
    { 0x6617b826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6297b829, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd7b82c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f7b832, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb337b835, SUC_lexicon_tags_37 },
    { 0x9037b836, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x76f7b838, SUC_lexicon_tags_37 },
    { 0xbad7b838, SUC_lexicon_tags_82 },
    { 0x3317b839, SUC_lexicon_tags_65 },
    { 0x8397b838, SUC_lexicon_tags_2 },
    { 0x9777b83c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf477b83f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe097b842, SUC_lexicon_tags_0 },
    { 0x8a17b843, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d57b845, SUC_lexicon_tags_47 },
    { 0x2317b846, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa377b848, SUC_lexicon_tags_23 },
    { 0x9f17b848, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d97b84d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf997b854, SUC_lexicon_tags_17 },
    { 0xb457b854, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfd7b857, SUC_lexicon_tags_11 },
    { 0x7cb7b857, SUC_lexicon_tags_17 },
    { 0xff97b859, SUC_lexicon_tags_7 },
    { 0x9d57b85a, SUC_lexicon_tags_17 },
    { 0x34b7b859, SUC_lexicon_tags_1 },
    { 0xe397b85b, SUC_lexicon_tags_84 },
    { 0x6457b85d, SUC_lexicon_tags_7 },
    { 0x7217b85d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f7b861, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb7b865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb77b867, SUC_lexicon_tags_25 },
    { 0x2ad7b868, SUC_lexicon_tags_2 },
    { 0xa6b7b868, SUC_lexicon_tags_44 },
    { 0xe217b86a, SUC_lexicon_tags_30 },
    { 0x1c37b86a, SUC_lexicon_tags_12 },
    { 0xe177b86b, SUC_lexicon_tags_11 },
    { 0xc077b868, SUC_lexicon_tags_27 },
    { 0xb8d7b86b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc317b870, SUC_lexicon_tags_32 },
    { 0xb457b871, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe897b875, SUC_lexicon_tags_47 },
    { 0x5a57b875, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x3af7b878, SUC_lexicon_tags_113 },
    { 0xfad7b879, SUC_lexicon_tags_160 },
    { 0x3d37b87a, SUC_lexicon_tags_6 },
    { 0x8637b87a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5757b87d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b7b881, SUC_lexicon_tags_15 },
    { 0xa8b7b882, SUC_lexicon_tags_17 },
    { 0x8517b883, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdad7b885, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7257b887, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8237b88a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86d7b88c, SUC_lexicon_tags_45 },
    { 0x5a97b88d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5457b891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b7b894, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2937b897, SUC_lexicon_tags_19 },
    { 0x4b7b898, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x1057b89a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe4f7b89c, SUC_lexicon_tags_11 },
    { 0x9d97b89d, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d7b8a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2317b8a9, SUC_lexicon_tags_12 },
    { 0xa7b7b8aa, SUC_lexicon_tags_37 },
    { 0x4797b8ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd7b8af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a17b8b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7057b8ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72d7b8bc, SUC_lexicon_tags_12 },
    { 0xc117b8bc, SUC_lexicon_tags_1 },
    { 0x1097b8be, SUC_lexicon_tags_14 },
    { 0xf697b8bf, SUC_lexicon_tags_2 },
    { 0x12b7b8c0, SUC_lexicon_tags_7 },
    { 0xd697b8bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df7b8c5, SUC_lexicon_tags_2 },
    { 0xf337b8c5, SUC_lexicon_tags_14 },
    { 0x5a37b8c7, SUC_lexicon_tags_0 },
    { 0x6b97b8c7, SUC_lexicon_tags_34 },
    { 0x5177b8c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe737b8cc, SUC_lexicon_tags_30 },
    { 0xb157b8cc, SUC_lexicon_tags_12 },
    { 0x13d7b8cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb57b8d0, SUC_lexicon_tags_37 },
    { 0xb057b8d0, SUC_lexicon_tags_43 },
    { 0xd657b8d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b8d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe57b8d9, SUC_lexicon_tags_37 },
    { 0x4c97b8da, SUC_lexicon_tags_49 },
    { 0x36f7b8d9, SUC_lexicon_tags_27 },
    { 0xbd77b8db, SUC_lexicon_tags_68 },
    { 0xc777b8dd, SUC_lexicon_tags_2 },
    { 0x3bd7b8dd, SUC_lexicon_tags_0 },
    { 0x5597b8df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a57b8e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef7b8e8, SUC_lexicon_tags_0 },
    { 0x9577b8e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd937b8eb, SUC_lexicon_tags_2 },
    { 0xa0b7b8ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xea17b8ef, SUC_lexicon_tags_12 },
    { 0xe97b8f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x31f7b8f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b7b8f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57b8f7, SUC_lexicon_tags_1 },
    { 0x2ed7b8f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5b37b8fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4377b903, SUC_lexicon_tags_148 },
    { 0xd5d7b903, SUC_lexicon_tags_85 },
    { 0xc777b905, SUC_lexicon_tags_68 },
    { 0x2d97b905, SUC_lexicon_tags_63 },
    { 0x33b7b904, SUC_lexicon_tags_1 },
    { 0x7897b904, SUC_lexicon_tags_19 },
    { 0x2b37b906, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7b910, SUC_lexicon_tags_28 },
    { 0xc117b911, SUC_lexicon_tags_4 },
    { 0x14f7b911, SUC_lexicon_tags_27 },
    { 0xed37b910, SUC_lexicon_tags_17 },
    { 0xd4f7b910, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe937b916, SUC_lexicon_tags_224 },
    { 0x4197b916, SUC_lexicon_tags_37 },
    { 0x83d7b916, SUC_lexicon_tags_46 },
    { 0xd4f7b918, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2737b91b, SUC_lexicon_tags_37 },
    { 0xb137b91c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8897b921, SUC_lexicon_tags_16 },
    { 0x9417b921, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb137b924, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4897b928, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c97b92b, SUC_lexicon_tags_9 },
    { 0xfa17b92b, SUC_lexicon_tags_7 },
    { 0x2e17b92d, SUC_lexicon_tags_15 },
    { 0x3397b92e, SUC_lexicon_tags_65 },
    { 0x2ed7b92f, SUC_lexicon_tags_10 },
    { 0x60f7b930, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x497b932, SUC_lexicon_tags_4 },
    { 0x37b7b933, SUC_lexicon_tags_16 },
    { 0x9ad7b934, SUC_lexicon_tags_64 },
    { 0xb177b935, SUC_lexicon_tags_1 },
    { 0xefd7b934, SUC_lexicon_tags_17 },
    { 0x5fb7b933, SUC_lexicon_tags_0 },
    { 0x5d17b937, SUC_lexicon_tags_45 },
    { 0x8b7b932, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9937b93d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf117b940, SUC_lexicon_tags_15 },
    { 0xab57b941, SUC_lexicon_tags_28 },
    { 0x74d7b941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf7b946, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f7b949, SUC_lexicon_tags_4 },
    { 0x6137b949, SUC_lexicon_tags_25 },
    { 0x3717b94b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb0d7b94d, SUC_lexicon_tags_77 },
    { 0xc057b94e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb017b950, SUC_lexicon_tags_64 },
    { 0xfad7b950, SUC_lexicon_tags_39 },
    { 0xe997b952, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4677b957, SUC_lexicon_tags_2 },
    { 0xbd57b957, SUC_lexicon_tags_0 },
    { 0xde37b959, SUC_lexicon_tags_17 },
    { 0x2e77b95a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6337b964, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5377b968, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd7b975, SUC_lexicon_tags_41 },
    { 0xd977b976, SUC_lexicon_tags_15 },
    { 0x5197b976, SUC_lexicon_tags_15 },
    { 0xbdb7b976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6997b97b, SUC_lexicon_tags_17 },
    { 0x7ed7b97c, SUC_lexicon_tags_182 },
    { 0xe297b97d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd737b97f, SUC_lexicon_tags_25 },
    { 0x6337b97f, SUC_lexicon_tags_84 },
    { 0x4117b981, SUC_lexicon_tags_17 },
    { 0xdd77b982, SUC_lexicon_tags_17 },
    { 0xce17b983, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7597b98e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf097b996, SUC_lexicon_tags_1131 },
    { 0, NULL },
    { 0x4197b998, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf77b9a0, SUC_lexicon_tags_45 },
    { 0xcd37b9a1, SUC_lexicon_tags_2 },
    { 0x7337b9a2, SUC_lexicon_tags_2 },
    { 0xed17b9a3, SUC_lexicon_tags_2 },
    { 0xc57b9a1, SUC_lexicon_tags_21 },
    { 0xcf57b9a5, SUC_lexicon_tags_75 },
    { 0x1b97b9a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa57b9a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf37b9ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7477b9ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5137b9b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe37b9b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b7b9be, SUC_lexicon_tags_2 },
    { 0xe8b7b9bf, SUC_lexicon_tags_47 },
    { 0xc777b9c0, SUC_lexicon_tags_396 },
    { 0x8817b9c1, SUC_lexicon_tags_2 },
    { 0xbb77b9c1, SUC_lexicon_tags_17 },
    { 0xd597b9c2, SUC_lexicon_tags_10 },
    { 0xc117b9c4, SUC_lexicon_tags_2 },
    { 0xa297b9c4, SUC_lexicon_tags_27 },
    { 0x86f7b9c6, SUC_lexicon_tags_27 },
    { 0x78f7b9c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb977b9c9, SUC_lexicon_tags_43 },
    { 0x60d7b9c9, SUC_lexicon_tags_47 },
    { 0x4737b9cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f7b9ce, SUC_lexicon_tags_68 },
    { 0xad97b9cf, SUC_lexicon_tags_15 },
    { 0x3137b9d0, SUC_lexicon_tags_45 },
    { 0x9557b9d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2417b9da, SUC_lexicon_tags_0 },
    { 0x2417b9db, SUC_lexicon_tags_25 },
    { 0x68d7b9db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe037b9df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6117b9e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b97b9e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b7b9e7, SUC_lexicon_tags_37 },
    { 0xc4f7b9e8, SUC_lexicon_tags_20 },
    { 0xc2b7b9e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4537b9ed, SUC_lexicon_tags_47 },
    { 0x1d17b9ee, SUC_lexicon_tags_151 },
    { 0x95f7b9ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d7b9f2, SUC_lexicon_tags_82 },
    { 0xcd97b9f3, SUC_lexicon_tags_1 },
    { 0xbef7b9f3, SUC_lexicon_tags_17 },
    { 0xf817b9f5, SUC_lexicon_tags_26 },
    { 0x9b97b9f6, SUC_lexicon_tags_14 },
    { 0xf417b9f7, SUC_lexicon_tags_11 },
    { 0x7bf7b9f8, SUC_lexicon_tags_6 },
    { 0xef97b9f9, SUC_lexicon_tags_55 },
    { 0x477b9fa, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3077ba00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d7ba0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b7ba10, SUC_lexicon_tags_55 },
    { 0xead7ba11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9057ba13, SUC_lexicon_tags_34 },
    { 0x6757ba14, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1017ba18, SUC_lexicon_tags_14 },
    { 0x4877ba19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d57ba1b, SUC_lexicon_tags_14 },
    { 0x9a57ba1b, SUC_lexicon_tags_17 },
    { 0x9637ba1d, SUC_lexicon_tags_1 },
    { 0x7677ba1e, SUC_lexicon_tags_26 },
    { 0xdc57ba1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff7ba22, SUC_lexicon_tags_19 },
    { 0x3bb7ba23, SUC_lexicon_tags_12 },
    { 0x7177ba22, SUC_lexicon_tags_28 },
    { 0x6157ba25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8137ba28, SUC_lexicon_tags_12 },
    { 0x8937ba29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf737ba2d, SUC_lexicon_tags_15 },
    { 0x1a17ba2e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a57ba32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe377ba38, SUC_lexicon_tags_25 },
    { 0xb537ba39, SUC_lexicon_tags_11 },
    { 0xa97ba39, SUC_lexicon_tags_240 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f17ba3e, SUC_lexicon_tags_37 },
    { 0x8cd7ba3f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7897ba42, SUC_lexicon_tags_17 },
    { 0x54b7ba43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d17ba50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50f7ba52, SUC_lexicon_tags_16 },
    { 0x6137ba53, SUC_lexicon_tags_26 },
    { 0xd3b7ba54, SUC_lexicon_tags_66 },
    { 0x43d7ba55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2917ba59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f7ba5d, SUC_lexicon_tags_27 },
    { 0x52f7ba5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb4f7ba60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf7ba63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2157ba66, SUC_lexicon_tags_1 },
    { 0x5a57ba67, SUC_lexicon_tags_63 },
    { 0x4337ba68, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57ba71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd057ba74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x537ba76, SUC_lexicon_tags_12 },
    { 0xc757ba77, SUC_lexicon_tags_14 },
    { 0x3a17ba78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3457ba7d, SUC_lexicon_tags_25 },
    { 0xcaf7ba7e, SUC_lexicon_tags_48 },
    { 0x58b7ba7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f7ba82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd7ba85, SUC_lexicon_tags_7 },
    { 0x9997ba86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55b7ba88, SUC_lexicon_tags_17 },
    { 0xac57ba89, SUC_lexicon_tags_38 },
    { 0x9357ba8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb357ba8d, SUC_lexicon_tags_12 },
    { 0x277ba8e, SUC_lexicon_tags_27 },
    { 0xe217ba8f, SUC_lexicon_tags_29 },
    { 0xfe17ba90, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4197ba92, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f7ba95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadf7ba97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb7ba9a, SUC_lexicon_tags_15 },
    { 0x4bd7ba9b, SUC_lexicon_tags_65 },
    { 0xf7d7ba9a, SUC_lexicon_tags_65 },
    { 0x5df7ba9d, SUC_lexicon_tags_17 },
    { 0xe2d7ba9e, SUC_lexicon_tags_1 },
    { 0xa537ba9d, SUC_lexicon_tags_37 },
    { 0x4a77baa0, SUC_lexicon_tags_0 },
    { 0x9357ba9d, SUC_lexicon_tags_47 },
    { 0xc57baa2, SUC_lexicon_tags_4 },
    { 0x38d7ba9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7657baa6, SUC_lexicon_tags_46 },
    { 0xdb17baa6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3997baac, SUC_lexicon_tags_37 },
    { 0x5b77baad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2417bab1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd7bab4, SUC_lexicon_tags_39 },
    { 0x1d7bab5, SUC_lexicon_tags_63 },
    { 0x5097bab6, SUC_lexicon_tags_0 },
    { 0x1037bab6, SUC_lexicon_tags_15 },
    { 0x7ed7bab5, SUC_lexicon_tags_60 },
    { 0xc5b7bab7, SUC_lexicon_tags_12 },
    { 0x8577baba, SUC_lexicon_tags_56 },
    { 0x1c77babb, SUC_lexicon_tags_14 },
    { 0x9ef7baba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb7babf, SUC_lexicon_tags_15 },
    { 0xc017bac0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc0d7bac2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x417bac4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7557baca, SUC_lexicon_tags_42 },
    { 0xd6f7baca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc817bacd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8977bad3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d77bad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf297badb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f7bade, SUC_lexicon_tags_15 },
    { 0xbf57badf, SUC_lexicon_tags_47 },
    { 0xcc37bade, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe77bae2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4b7bae4, SUC_lexicon_tags_37 },
    { 0x9697bae4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded7bae9, SUC_lexicon_tags_7 },
    { 0xaa17baea, SUC_lexicon_tags_11 },
    { 0x81b7baeb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcd97baed, SUC_lexicon_tags_26 },
    { 0x9b17baee, SUC_lexicon_tags_26 },
    { 0x9cd7baef, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe677baf2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7837baf8, SUC_lexicon_tags_60 },
    { 0x74f7baf9, SUC_lexicon_tags_15 },
    { 0xedf7bafa, SUC_lexicon_tags_14 },
    { 0xe577bafa, SUC_lexicon_tags_45 },
    { 0x43d7baf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2437bafe, SUC_lexicon_tags_25 },
    { 0xfd57bafe, SUC_lexicon_tags_25 },
    { 0xa257bb00, SUC_lexicon_tags_15 },
    { 0xa497bb01, SUC_lexicon_tags_37 },
    { 0x2b17bafe, SUC_lexicon_tags_11 },
    { 0x2497bb03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ff7bb05, SUC_lexicon_tags_15 },
    { 0x7297bb05, SUC_lexicon_tags_15 },
    { 0x5217bb06, SUC_lexicon_tags_38 },
    { 0x8717bb08, SUC_lexicon_tags_10 },
    { 0x71b7bb09, SUC_lexicon_tags_11 },
    { 0x5e57bb09, SUC_lexicon_tags_28 },
    { 0xfe57bb0b, SUC_lexicon_tags_25 },
    { 0x8777bb0c, SUC_lexicon_tags_30 },
    { 0xeb17bb0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f7bb11, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f7bb16, SUC_lexicon_tags_12 },
    { 0xe0d7bb17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf377bb1a, SUC_lexicon_tags_14 },
    { 0x6cd7bb1b, SUC_lexicon_tags_37 },
    { 0x1397bb1c, SUC_lexicon_tags_6 },
    { 0xbe37bb1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7bb20, SUC_lexicon_tags_31 },
    { 0xd3d7bb20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f97bb24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1057bb26, SUC_lexicon_tags_1 },
    { 0x8977bb27, SUC_lexicon_tags_8 },
    { 0x1597bb26, SUC_lexicon_tags_16 },
    { 0xd577bb29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd757bb2f, SUC_lexicon_tags_29 },
    { 0x95d7bb30, SUC_lexicon_tags_37 },
    { 0x5677bb31, SUC_lexicon_tags_22 },
    { 0xa3f7bb30, SUC_lexicon_tags_47 },
    { 0xd0b7bb32, SUC_lexicon_tags_17 },
    { 0x9597bb34, SUC_lexicon_tags_8 },
    { 0x5bb7bb34, SUC_lexicon_tags_5 },
    { 0x3af7bb35, SUC_lexicon_tags_19 },
    { 0xf17bb34, SUC_lexicon_tags_11 },
    { 0x9d77bb38, SUC_lexicon_tags_30 },
    { 0xba97bb39, SUC_lexicon_tags_6 },
    { 0xcef7bb39, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb7bb3d, SUC_lexicon_tags_39 },
    { 0xa4f7bb3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d7bb41, SUC_lexicon_tags_49 },
    { 0xf057bb42, SUC_lexicon_tags_1 },
    { 0x3757bb41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38b7bb45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6597bb4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc37bb4c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d7bb50, SUC_lexicon_tags_38 },
    { 0x84d7bb51, SUC_lexicon_tags_31 },
    { 0xc757bb51, SUC_lexicon_tags_10 },
    { 0x2ef7bb51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd7bb5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2057bb61, SUC_lexicon_tags_1 },
    { 0x9b77bb62, SUC_lexicon_tags_218 },
    { 0x7ad7bb61, SUC_lexicon_tags_29 },
    { 0x737bb64, SUC_lexicon_tags_7 },
    { 0x8477bb65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6117bb68, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdb17bb6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b7bb6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf757bb74, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7097bb7b, SUC_lexicon_tags_14 },
    { 0xc5d7bb7b, SUC_lexicon_tags_1 },
    { 0xf437bb7b, SUC_lexicon_tags_38 },
    { 0xdf17bb7e, SUC_lexicon_tags_11 },
    { 0xc0f7bb7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff7bb84, SUC_lexicon_tags_2 },
    { 0x7997bb85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb7bb88, SUC_lexicon_tags_45 },
    { 0xc457bb89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9637bb8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd7bb90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af7bb93, SUC_lexicon_tags_27 },
    { 0x59d7bb94, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997bb98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x45f7bb9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1477bb9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d97bb9f, SUC_lexicon_tags_25 },
    { 0x1257bb9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb7bba4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3437bba8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf537bbad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d7bbb0, SUC_lexicon_tags_82 },
    { 0xe217bbb1, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x1317bbb3, SUC_lexicon_tags_6 },
    { 0x9d17bbb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc897bbb6, SUC_lexicon_tags_11 },
    { 0x877bbb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b97bbc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad7bbc5, SUC_lexicon_tags_118 },
    { 0xcf57bbc6, SUC_lexicon_tags_5 },
    { 0x2ab7bbc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa817bbca, SUC_lexicon_tags_2 },
    { 0xfb97bbcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f7bbd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc997bbd2, SUC_lexicon_tags_25 },
    { 0x3c57bbd2, SUC_lexicon_tags_43 },
    { 0x19f7bbd4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9cf7bbd6, SUC_lexicon_tags_0 },
    { 0x5bd7bbd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcd7bbd9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfa57bbdb, SUC_lexicon_tags_8 },
    { 0xe617bbdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297bbe2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb7bbe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef7bbe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a97bbf1, SUC_lexicon_tags_19 },
    { 0x1817bbf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9497bbf4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf6f7bbf6, SUC_lexicon_tags_17 },
    { 0x7d37bbf6, SUC_lexicon_tags_11 },
    { 0xc377bbf8, SUC_lexicon_tags_15 },
    { 0x5717bbf8, SUC_lexicon_tags_11 },
    { 0x2777bbfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1637bbfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd317bbff, SUC_lexicon_tags_30 },
    { 0xac77bc00, SUC_lexicon_tags_38 },
    { 0xf357bc01, SUC_lexicon_tags_43 },
    { 0xf757bc02, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff7bc06, SUC_lexicon_tags_37 },
    { 0x3f97bc07, SUC_lexicon_tags_7 },
    { 0xbb7bc08, SUC_lexicon_tags_15 },
    { 0x16b7bc09, SUC_lexicon_tags_25 },
    { 0x3377bc08, SUC_lexicon_tags_11 },
    { 0x32f7bc08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf977bc0e, SUC_lexicon_tags_19 },
    { 0xcb17bc0e, SUC_lexicon_tags_46 },
    { 0xb017bc10, SUC_lexicon_tags_1 },
    { 0x9cf7bc10, SUC_lexicon_tags_77 },
    { 0x4977bc12, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7bc1a, SUC_lexicon_tags_15 },
    { 0x7bd7bc1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xad77bc1e, SUC_lexicon_tags_5 },
    { 0xb037bc1f, SUC_lexicon_tags_39 },
    { 0xecf7bc1f, SUC_lexicon_tags_3 },
    { 0x4cf7bc20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f37bc23, SUC_lexicon_tags_37 },
    { 0xec17bc24, SUC_lexicon_tags_35 },
    { 0xbf7bc25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b17bc27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa117bc2f, SUC_lexicon_tags_37 },
    { 0xcdd7bc30, SUC_lexicon_tags_56 },
    { 0xde57bc31, SUC_lexicon_tags_19 },
    { 0x2e77bc32, SUC_lexicon_tags_63 },
    { 0x9bd7bc33, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe9b7bc35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9137bc3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6877bc41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7957bc43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb7bc48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b7bc4c, SUC_lexicon_tags_2 },
    { 0xc8d7bc4d, SUC_lexicon_tags_4 },
    { 0xae77bc4e, SUC_lexicon_tags_49 },
    { 0x5ed7bc4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8237bc52, SUC_lexicon_tags_14 },
    { 0xb877bc52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d7bc5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec57bc61, SUC_lexicon_tags_7 },
    { 0xa4f7bc61, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5557bc6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7237bc6e, SUC_lexicon_tags_37 },
    { 0x1bd7bc6e, SUC_lexicon_tags_33 },
    { 0xc1d7bc70, SUC_lexicon_tags_115 },
    { 0xc937bc70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc917bc77, SUC_lexicon_tags_15 },
    { 0x59b7bc77, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5657bc7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f7bc7f, SUC_lexicon_tags_0 },
    { 0xd977bc7f, SUC_lexicon_tags_30 },
    { 0x9777bc80, SUC_lexicon_tags_11 },
    { 0x6997bc82, SUC_lexicon_tags_15 },
    { 0x7ed7bc82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8417bc86, SUC_lexicon_tags_45 },
    { 0x4897bc86, SUC_lexicon_tags_15 },
    { 0x9157bc88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3817bc8a, SUC_lexicon_tags_11 },
    { 0xeb77bc8a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b7bc97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d7bc9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d7bca3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457bca8, SUC_lexicon_tags_87 },
    { 0x8117bca8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce37bcac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f7bcaf, SUC_lexicon_tags_25 },
    { 0x5317bcaf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c37bcb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d7bcb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf7bcbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe157bcc1, SUC_lexicon_tags_14 },
    { 0x6197bcc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb757bcc5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b7bccc, SUC_lexicon_tags_9 },
    { 0xe277bccd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbbb7bccf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6437bcd1, SUC_lexicon_tags_56 },
    { 0xf17bcd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d7bcd5, SUC_lexicon_tags_799 },
    { 0x25f7bcd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc737bcd9, SUC_lexicon_tags_7 },
    { 0x1637bcd9, SUC_lexicon_tags_30 },
    { 0x6ed7bcdb, SUC_lexicon_tags_11 },
    { 0x9617bcda, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd7bce1, SUC_lexicon_tags_63 },
    { 0xaf17bce2, SUC_lexicon_tags_15 },
    { 0xfd97bce3, SUC_lexicon_tags_11 },
    { 0x9517bce1, SUC_lexicon_tags_25 },
    { 0x76d7bce1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84b7bce7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97bcec, SUC_lexicon_tags_15 },
    { 0x5ab7bced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91b7bcef, SUC_lexicon_tags_25 },
    { 0x6737bcef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb937bcf5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f7bcfb, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f7bd00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed7bd05, SUC_lexicon_tags_15 },
    { 0x46b7bd06, SUC_lexicon_tags_33 },
    { 0x1637bd07, SUC_lexicon_tags_37 },
    { 0x12b7bd08, SUC_lexicon_tags_1123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa37bd0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb7bd0f, SUC_lexicon_tags_38 },
    { 0xded7bd10, SUC_lexicon_tags_17 },
    { 0xccb7bd11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9077bd14, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f7bd17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb7bd1c, SUC_lexicon_tags_47 },
    { 0x6057bd1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8477bd24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8337bd29, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe157bd2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77bd2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa757bd31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce57bd33, SUC_lexicon_tags_60 },
    { 0xae37bd34, SUC_lexicon_tags_23 },
    { 0x6f7bd34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b7bd39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f7bd3c, SUC_lexicon_tags_2 },
    { 0x4bd7bd3d, SUC_lexicon_tags_48 },
    { 0xc5f7bd3e, SUC_lexicon_tags_57 },
    { 0xfab7bd3d, SUC_lexicon_tags_60 },
    { 0xcff7bd40, SUC_lexicon_tags_11 },
    { 0xb017bd41, SUC_lexicon_tags_2 },
    { 0x537bd41, SUC_lexicon_tags_68 },
    { 0xc637bd42, SUC_lexicon_tags_35 },
    { 0x66d7bd3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d7bd48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9537bd4b, SUC_lexicon_tags_11 },
    { 0x7617bd4c, SUC_lexicon_tags_11 },
    { 0x2037bd4d, SUC_lexicon_tags_11 },
    { 0x7417bd4c, SUC_lexicon_tags_15 },
    { 0x2997bd4f, SUC_lexicon_tags_7 },
    { 0x8977bd4f, SUC_lexicon_tags_82 },
    { 0x2c57bd4d, SUC_lexicon_tags_1 },
    { 0x7f57bd50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61b7bd54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd1f7bd56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b57bd58, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1557bd5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b97bd5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf497bd62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf7bd66, SUC_lexicon_tags_11 },
    { 0x5f97bd66, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x16f7bd69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe497bd6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb7bd70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5f37bd72, SUC_lexicon_tags_27 },
    { 0x1cf7bd73, SUC_lexicon_tags_12 },
    { 0xfcf7bd72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d7bd76, SUC_lexicon_tags_7 },
    { 0x9637bd77, SUC_lexicon_tags_2 },
    { 0x5077bd77, SUC_lexicon_tags_248 },
    { 0x41d7bd79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b7bd7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a57bd83, SUC_lexicon_tags_4 },
    { 0x32d7bd84, SUC_lexicon_tags_3 },
    { 0xa737bd85, SUC_lexicon_tags_2 },
    { 0xccf7bd86, SUC_lexicon_tags_37 },
    { 0x8d17bd87, SUC_lexicon_tags_29 },
    { 0x6177bd84, SUC_lexicon_tags_56 },
    { 0xf257bd89, SUC_lexicon_tags_47 },
    { 0x5617bd85, SUC_lexicon_tags_37 },
    { 0x5397bd8a, SUC_lexicon_tags_2 },
    { 0xa9d7bd8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d97bd8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd097bd91, SUC_lexicon_tags_65 },
    { 0x2a37bd92, SUC_lexicon_tags_33 },
    { 0x1457bd92, SUC_lexicon_tags_2 },
    { 0x95b7bd94, SUC_lexicon_tags_17 },
    { 0xa097bd91, SUC_lexicon_tags_1 },
    { 0xd557bd96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b37bd9a, SUC_lexicon_tags_1 },
    { 0x6717bd9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa037bd9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f57bda2, SUC_lexicon_tags_27 },
    { 0x43b7bda3, SUC_lexicon_tags_17 },
    { 0xbd37bda4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b7bda7, SUC_lexicon_tags_37 },
    { 0x2eb7bda8, SUC_lexicon_tags_27 },
    { 0xc757bda9, SUC_lexicon_tags_8 },
    { 0x6b57bdaa, SUC_lexicon_tags_4 },
    { 0x86b7bdaa, SUC_lexicon_tags_43 },
    { 0xae57bdaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf337bdaf, SUC_lexicon_tags_25 },
    { 0xc697bdb0, SUC_lexicon_tags_11 },
    { 0x2a17bdb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc637bdb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9357bdb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf57bdb8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f7bdbd, SUC_lexicon_tags_10 },
    { 0xf97bdbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a17bdc1, SUC_lexicon_tags_33 },
    { 0x8d57bdc2, SUC_lexicon_tags_7 },
    { 0x3bb7bdc3, SUC_lexicon_tags_23 },
    { 0x1917bdc4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff7bdc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf2b7bdc9, SUC_lexicon_tags_12 },
    { 0x18d7bdc9, SUC_lexicon_tags_32 },
    { 0xa9b7bdcb, SUC_lexicon_tags_33 },
    { 0x1d77bdcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f7bdd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3097bdd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f7bdd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf77bdda, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf457bddd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f57bde1, SUC_lexicon_tags_11 },
    { 0xf697bde2, SUC_lexicon_tags_2 },
    { 0x51f7bde1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe797bde6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77bde9, SUC_lexicon_tags_4 },
    { 0xd357bde9, SUC_lexicon_tags_17 },
    { 0xd8b7bdeb, SUC_lexicon_tags_7 },
    { 0x2777bdeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4957bdf1, SUC_lexicon_tags_6 },
    { 0x7d17bdf1, SUC_lexicon_tags_1 },
    { 0x5917bdf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d7bdf7, SUC_lexicon_tags_12 },
    { 0x1d97bdf8, SUC_lexicon_tags_38 },
    { 0xabf7bdf7, SUC_lexicon_tags_33 },
    { 0xd637bdf7, SUC_lexicon_tags_68 },
    { 0x47d7bdfb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4497bdfe, SUC_lexicon_tags_37 },
    { 0xaed7bdff, SUC_lexicon_tags_1 },
    { 0x6ad7bdfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb77be04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf837be06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb777be09, SUC_lexicon_tags_12 },
    { 0x8797be0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec57be0c, SUC_lexicon_tags_27 },
    { 0x7b17be0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3237be15, SUC_lexicon_tags_27 },
    { 0x3577be15, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xebf7be18, SUC_lexicon_tags_17 },
    { 0x6f97be19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1b7be1b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d7be1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20f7be20, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd317be22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x437be24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa717be28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d7be2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf157be32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe857be34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f97be3a, SUC_lexicon_tags_15 },
    { 0xe6f7be3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe417be3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6757be41, SUC_lexicon_tags_29 },
    { 0xa6d7be42, SUC_lexicon_tags_1 },
    { 0x1477be43, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb97be49, SUC_lexicon_tags_14 },
    { 0xecf7be49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3957be4f, SUC_lexicon_tags_65 },
    { 0x5f77be50, SUC_lexicon_tags_21 },
    { 0xb297be51, SUC_lexicon_tags_17 },
    { 0xd317be52, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed7be58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b7be5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7417be5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf217be60, SUC_lexicon_tags_15 },
    { 0xf217be60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba57be63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe17be65, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf17be6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc6b7be6d, SUC_lexicon_tags_1 },
    { 0x2cb7be6d, SUC_lexicon_tags_37 },
    { 0x7177be6f, SUC_lexicon_tags_63 },
    { 0x7cf7be70, SUC_lexicon_tags_1 },
    { 0x8c97be71, SUC_lexicon_tags_32 },
    { 0x3cb7be72, SUC_lexicon_tags_15 },
    { 0xb9f7be70, SUC_lexicon_tags_11 },
    { 0x7e37be72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6217be76, SUC_lexicon_tags_25 },
    { 0xf7be76, SUC_lexicon_tags_224 },
    { 0xcf37be78, SUC_lexicon_tags_25 },
    { 0xb957be76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b7be7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b7be83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb7be86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51d7be88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb7be8e, SUC_lexicon_tags_60 },
    { 0x8917be8f, SUC_lexicon_tags_2 },
    { 0x4557be8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1eb7be92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1817be95, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa437be99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa957be9b, SUC_lexicon_tags_23 },
    { 0x1e77be9c, SUC_lexicon_tags_37 },
    { 0x2b7be9d, SUC_lexicon_tags_17 },
    { 0xa6d7be9d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdc77bea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc137bea2, SUC_lexicon_tags_15 },
    { 0xdb77bea3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23b7bea5, SUC_lexicon_tags_21 },
    { 0x1557bea5, SUC_lexicon_tags_37 },
    { 0x4297bea6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa7d7bea9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf57beab, SUC_lexicon_tags_30 },
    { 0x7377beac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf57beae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8257beb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8d7beb2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5937beb4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74f7beb6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x1017beb8, SUC_lexicon_tags_23 },
    { 0x6ef7beb8, SUC_lexicon_tags_6 },
    { 0x3677beba, SUC_lexicon_tags_27 },
    { 0x5917beb8, SUC_lexicon_tags_55 },
    { 0x5cd7beba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7017bebe, SUC_lexicon_tags_37 },
    { 0x9137bebe, SUC_lexicon_tags_10 },
    { 0x1397bec0, SUC_lexicon_tags_14 },
    { 0x8c17bebf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a17bec3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1597bec5, SUC_lexicon_tags_6 },
    { 0x4677bec5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b7bec9, SUC_lexicon_tags_45 },
    { 0xe37beca, SUC_lexicon_tags_11 },
    { 0xc5f7becb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1b7becd, SUC_lexicon_tags_12 },
    { 0x72b7bece, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5917bed1, SUC_lexicon_tags_6 },
    { 0xd097bed2, SUC_lexicon_tags_27 },
    { 0xc17bed3, SUC_lexicon_tags_23 },
    { 0xef7bed2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfc97bed6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde37bed9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b7bedc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x84b7bede, SUC_lexicon_tags_27 },
    { 0xde57bedf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x477bee1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d7bee4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb877bee7, SUC_lexicon_tags_25 },
    { 0x2f97bee7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3237beea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8557beec, SUC_lexicon_tags_29 },
    { 0x797beed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba17bef1, SUC_lexicon_tags_63 },
    { 0x3ad7bef2, SUC_lexicon_tags_37 },
    { 0x657bef3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba97befc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa837bf01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed7bf04, SUC_lexicon_tags_25 },
    { 0x1af7bf04, SUC_lexicon_tags_65 },
    { 0x1b97bf04, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97bf09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1477bf0b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d7bf0e, SUC_lexicon_tags_17 },
    { 0x2c57bf0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80b7bf11, SUC_lexicon_tags_6 },
    { 0xa397bf12, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d7bf19, SUC_lexicon_tags_17 },
    { 0xfd17bf19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9a77bf1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f7bf1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1317bf23, SUC_lexicon_tags_11 },
    { 0xdb37bf23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd37bf26, SUC_lexicon_tags_23 },
    { 0xa417bf26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3057bf2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8117bf2f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x23d7bf31, SUC_lexicon_tags_49 },
    { 0x5f97bf31, SUC_lexicon_tags_17 },
    { 0xcbf7bf33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c37bf38, SUC_lexicon_tags_19 },
    { 0x78d7bf39, SUC_lexicon_tags_17 },
    { 0x10d7bf3a, SUC_lexicon_tags_6 },
    { 0x54d7bf3b, SUC_lexicon_tags_44 },
    { 0xbbd7bf3c, SUC_lexicon_tags_37 },
    { 0x90b7bf39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81d7bf3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c37bf42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x597bf45, SUC_lexicon_tags_1 },
    { 0x64d7bf46, SUC_lexicon_tags_29 },
    { 0x4f57bf47, SUC_lexicon_tags_11 },
    { 0xab17bf47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2897bf4c, SUC_lexicon_tags_12 },
    { 0xb517bf4c, SUC_lexicon_tags_25 },
    { 0xa157bf4c, SUC_lexicon_tags_11 },
    { 0x7057bf4c, SUC_lexicon_tags_47 },
    { 0x8997bf50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b97bf55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b7bf5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7797bf60, SUC_lexicon_tags_158 },
    { 0xf0d7bf61, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa097bf63, SUC_lexicon_tags_7 },
    { 0x1017bf63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x717bf66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd337bf6c, SUC_lexicon_tags_15 },
    { 0x137bf6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d7bf72, SUC_lexicon_tags_23 },
    { 0x1ad7bf73, SUC_lexicon_tags_12 },
    { 0x1017bf74, SUC_lexicon_tags_25 },
    { 0x9d37bf75, SUC_lexicon_tags_33 },
    { 0x6317bf75, SUC_lexicon_tags_35 },
    { 0xa037bf77, SUC_lexicon_tags_31 },
    { 0x10f7bf77, SUC_lexicon_tags_10 },
    { 0x18d7bf73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f77bf7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf737bf82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1737bf85, SUC_lexicon_tags_16 },
    { 0xa337bf86, SUC_lexicon_tags_7 },
    { 0xd757bf87, SUC_lexicon_tags_45 },
    { 0x37f7bf86, SUC_lexicon_tags_46 },
    { 0xe0d7bf89, SUC_lexicon_tags_6 },
    { 0x32b7bf8a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6577bf8c, SUC_lexicon_tags_30 },
    { 0xfb17bf8d, SUC_lexicon_tags_31 },
    { 0x5d17bf8c, SUC_lexicon_tags_87 },
    { 0xe637bf8e, SUC_lexicon_tags_46 },
    { 0x86d7bf90, SUC_lexicon_tags_12 },
    { 0xdd7bf90, SUC_lexicon_tags_84 },
    { 0x537bf90, SUC_lexicon_tags_17 },
    { 0x4517bf91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f7bf97, SUC_lexicon_tags_2 },
    { 0xe677bf98, SUC_lexicon_tags_46 },
    { 0xe9d7bf99, SUC_lexicon_tags_26 },
    { 0xc817bf9a, SUC_lexicon_tags_14 },
    { 0x5257bf9b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db7bf9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6937bfa7, SUC_lexicon_tags_84 },
    { 0x2457bfa7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc397bfab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf757bfae, SUC_lexicon_tags_37 },
    { 0x1517bfaf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x697bfb2, SUC_lexicon_tags_17 },
    { 0xbcd7bfb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb7bfb7, SUC_lexicon_tags_14 },
    { 0xaf77bfb8, SUC_lexicon_tags_17 },
    { 0x86d7bfb8, SUC_lexicon_tags_2 },
    { 0xb497bfba, SUC_lexicon_tags_48 },
    { 0xa957bfb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7097bfbe, SUC_lexicon_tags_14 },
    { 0x9157bfbf, SUC_lexicon_tags_37 },
    { 0x1657bfbe, SUC_lexicon_tags_21 },
    { 0xc957bfc1, SUC_lexicon_tags_29 },
    { 0x56b7bfbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f37bfc9, SUC_lexicon_tags_29 },
    { 0x2257bfc9, SUC_lexicon_tags_101 },
    { 0xb877bfca, SUC_lexicon_tags_31 },
    { 0xff77bfcc, SUC_lexicon_tags_39 },
    { 0xe657bfcc, SUC_lexicon_tags_17 },
    { 0x6177bfce, SUC_lexicon_tags_37 },
    { 0x1f57bfcf, SUC_lexicon_tags_12 },
    { 0xde17bfd0, SUC_lexicon_tags_60 },
    { 0x1297bfcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8497bfd3, SUC_lexicon_tags_6 },
    { 0xfd7bfd4, SUC_lexicon_tags_37 },
    { 0x50d7bfd5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e97bfd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x37bfdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa737bfe0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc017bfe3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d7bfe7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5637bfe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed77bfed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1417bff2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9677bff4, SUC_lexicon_tags_17 },
    { 0xd257bff5, SUC_lexicon_tags_7 },
    { 0xefd7bff6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f57bff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb7bffe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb7c001, SUC_lexicon_tags_47 },
    { 0xab57c002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb57c007, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb7c00a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2637c00c, SUC_lexicon_tags_14 },
    { 0x2437c00c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xde57c010, SUC_lexicon_tags_56 },
    { 0x6377c010, SUC_lexicon_tags_34 },
    { 0x3677c012, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3037c014, SUC_lexicon_tags_84 },
    { 0x8c97c015, SUC_lexicon_tags_7 },
    { 0x64b7c015, SUC_lexicon_tags_63 },
    { 0xb7b7c015, SUC_lexicon_tags_17 },
    { 0x45d7c018, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6817c01c, SUC_lexicon_tags_1 },
    { 0xec77c01d, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x8537c020, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x557c022, SUC_lexicon_tags_9 },
    { 0x4457c022, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7c029, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8937c02b, SUC_lexicon_tags_9 },
    { 0x9757c02b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd97c02e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3197c031, SUC_lexicon_tags_12 },
    { 0x8357c031, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3cd7c034, SUC_lexicon_tags_1 },
    { 0x3fd7c034, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd7c03b, SUC_lexicon_tags_207 },
    { 0x3f7c03c, SUC_lexicon_tags_12 },
    { 0xac37c03d, SUC_lexicon_tags_0 },
    { 0x4db7c03e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf7c04b, SUC_lexicon_tags_11 },
    { 0xa97c04b, SUC_lexicon_tags_15 },
    { 0xc5f7c04d, SUC_lexicon_tags_63 },
    { 0x3697c04e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x16d7c050, SUC_lexicon_tags_15 },
    { 0x2d97c051, SUC_lexicon_tags_17 },
    { 0x1957c051, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x16d7c054, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4117c05f, SUC_lexicon_tags_26 },
    { 0x5837c060, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb757c062, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d7c06a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfff7c06c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9217c06f, SUC_lexicon_tags_7 },
    { 0x9d97c070, SUC_lexicon_tags_4 },
    { 0xd637c070, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf7c074, SUC_lexicon_tags_2 },
    { 0xd4f7c075, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4157c077, SUC_lexicon_tags_9 },
    { 0x2ed7c077, SUC_lexicon_tags_28 },
    { 0x39b7c077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf7c07c, SUC_lexicon_tags_19 },
    { 0x6db7c07c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cf7c07f, SUC_lexicon_tags_69 },
    { 0x58b7c080, SUC_lexicon_tags_30 },
    { 0xcef7c080, SUC_lexicon_tags_12 },
    { 0x3d57c082, SUC_lexicon_tags_9 },
    { 0xe117c081, SUC_lexicon_tags_28 },
    { 0xa577c084, SUC_lexicon_tags_39 },
    { 0xdf97c080, SUC_lexicon_tags_65 },
    { 0xeb17c081, SUC_lexicon_tags_37 },
    { 0x6457c087, SUC_lexicon_tags_17 },
    { 0x4ff7c088, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd7c08f, SUC_lexicon_tags_30 },
    { 0x3137c090, SUC_lexicon_tags_29 },
    { 0x7017c090, SUC_lexicon_tags_11 },
    { 0x2597c08f, SUC_lexicon_tags_17 },
    { 0x4bf7c093, SUC_lexicon_tags_48 },
    { 0x7bf7c090, SUC_lexicon_tags_26 },
    { 0xa5f7c095, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8a17c097, SUC_lexicon_tags_65 },
    { 0x9ed7c097, SUC_lexicon_tags_47 },
    { 0x1157c097, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b7c09c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9457c0a1, SUC_lexicon_tags_23 },
    { 0xa497c0a2, SUC_lexicon_tags_11 },
    { 0x4477c0a1, SUC_lexicon_tags_23 },
    { 0xbfb7c0a1, SUC_lexicon_tags_34 },
    { 0x78f7c0a1, SUC_lexicon_tags_13 },
    { 0xfed7c0a6, SUC_lexicon_tags_45 },
    { 0x48b7c0a5, SUC_lexicon_tags_15 },
    { 0xd657c0a8, SUC_lexicon_tags_12 },
    { 0x1c77c0a9, SUC_lexicon_tags_56 },
    { 0xe577c0aa, SUC_lexicon_tags_14 },
    { 0xe217c0ab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad7c0b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d97c0b7, SUC_lexicon_tags_13 },
    { 0xbdd7c0b8, SUC_lexicon_tags_23 },
    { 0x8bd7c0b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa37c0bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xead7c0c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4717c0c6, SUC_lexicon_tags_21 },
    { 0xd9f7c0c6, SUC_lexicon_tags_29 },
    { 0x84f7c0c8, SUC_lexicon_tags_9 },
    { 0xabf7c0c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8117c0cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc17c0d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f17c0d3, SUC_lexicon_tags_17 },
    { 0xeeb7c0d4, SUC_lexicon_tags_25 },
    { 0x6cb7c0d5, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x637c0d7, SUC_lexicon_tags_17 },
    { 0x82b7c0d8, SUC_lexicon_tags_33 },
    { 0x93d7c0d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8757c0df, SUC_lexicon_tags_6 },
    { 0x70b7c0df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4437c0e2, SUC_lexicon_tags_26 },
    { 0x5857c0e2, SUC_lexicon_tags_14 },
    { 0xa437c0e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d17c0e8, SUC_lexicon_tags_45 },
    { 0x8557c0e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f7c0ed, SUC_lexicon_tags_37 },
    { 0xb417c0ed, SUC_lexicon_tags_15 },
    { 0xb0f7c0ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20f7c0f1, SUC_lexicon_tags_9 },
    { 0x7077c0f1, SUC_lexicon_tags_9 },
    { 0x5df7c0f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8c77c0f5, SUC_lexicon_tags_43 },
    { 0x6017c0f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf7c0f9, SUC_lexicon_tags_17 },
    { 0xf937c0fa, SUC_lexicon_tags_37 },
    { 0xa217c0fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe37c0fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4657c101, SUC_lexicon_tags_4 },
    { 0xb737c102, SUC_lexicon_tags_1 },
    { 0xbab7c103, SUC_lexicon_tags_38 },
    { 0x8df7c104, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x37b7c106, SUC_lexicon_tags_2 },
    { 0x84f7c107, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd2d7c109, SUC_lexicon_tags_38 },
    { 0xbfb7c10a, SUC_lexicon_tags_45 },
    { 0x51b7c109, SUC_lexicon_tags_37 },
    { 0x3a17c10b, SUC_lexicon_tags_38 },
    { 0x7977c10d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d7c110, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdb37c112, SUC_lexicon_tags_43 },
    { 0xe4f7c113, SUC_lexicon_tags_2 },
    { 0x2317c114, SUC_lexicon_tags_12 },
    { 0x5fb7c115, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4c17c117, SUC_lexicon_tags_31 },
    { 0x61d7c118, SUC_lexicon_tags_23 },
    { 0xbc57c117, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f97c11c, SUC_lexicon_tags_43 },
    { 0xf77c11c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d7c122, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a77c125, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe097c127, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb857c130, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5457c13c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd257c13f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d7c146, SUC_lexicon_tags_37 },
    { 0x4937c146, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3317c149, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9217c14d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8757c154, SUC_lexicon_tags_11 },
    { 0x5cb7c154, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38f7c157, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff97c15e, SUC_lexicon_tags_37 },
    { 0x9897c15f, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0xaf7c161, SUC_lexicon_tags_25 },
    { 0xa017c162, SUC_lexicon_tags_63 },
    { 0x68b7c163, SUC_lexicon_tags_45 },
    { 0x3db7c164, SUC_lexicon_tags_37 },
    { 0xec37c164, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd377c167, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d7c16b, SUC_lexicon_tags_17 },
    { 0x8af7c16b, SUC_lexicon_tags_15 },
    { 0xbd37c16d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb437c16f, SUC_lexicon_tags_2 },
    { 0xe6b7c16f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6737c17a, SUC_lexicon_tags_37 },
    { 0x3137c17b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7c17f, SUC_lexicon_tags_12 },
    { 0x3657c180, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf97c186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d7c18b, SUC_lexicon_tags_14 },
    { 0xacd7c18c, SUC_lexicon_tags_5 },
    { 0xedf7c18d, SUC_lexicon_tags_26 },
    { 0x5d17c18d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5197c191, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f7c196, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x59d7c198, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37c19c, SUC_lexicon_tags_20 },
    { 0x1f57c19d, SUC_lexicon_tags_2 },
    { 0x45b7c19c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5517c1a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d7c1a7, SUC_lexicon_tags_10 },
    { 0xef37c1a8, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xe137c1aa, SUC_lexicon_tags_63 },
    { 0xf4b7c1ab, SUC_lexicon_tags_2 },
    { 0xb317c1ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa17c1ae, SUC_lexicon_tags_25 },
    { 0xdfb7c1af, SUC_lexicon_tags_2 },
    { 0xe477c1af, SUC_lexicon_tags_2 },
    { 0x5b37c1b1, SUC_lexicon_tags_2 },
    { 0xf4f7c1ae, SUC_lexicon_tags_16 },
    { 0x2377c1b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af7c1b6, SUC_lexicon_tags_2 },
    { 0xc937c1b6, SUC_lexicon_tags_47 },
    { 0xe97c1b7, SUC_lexicon_tags_37 },
    { 0x78f7c1b9, SUC_lexicon_tags_37 },
    { 0xf4d7c1b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed7c1bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d7c1bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7857c1c1, SUC_lexicon_tags_47 },
    { 0x3c97c1c1, SUC_lexicon_tags_25 },
    { 0xf3d7c1c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f57c1c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb917c1cb, SUC_lexicon_tags_37 },
    { 0xdef7c1cc, SUC_lexicon_tags_48 },
    { 0x5d7c1cb, SUC_lexicon_tags_13 },
    { 0xaf37c1cb, SUC_lexicon_tags_17 },
    { 0xde57c1cc, SUC_lexicon_tags_62 },
    { 0x4237c1d0, SUC_lexicon_tags_37 },
    { 0x1917c1d1, SUC_lexicon_tags_15 },
    { 0x3137c1d0, SUC_lexicon_tags_26 },
    { 0x7af7c1d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f37c1d6, SUC_lexicon_tags_17 },
    { 0x1ef7c1d6, SUC_lexicon_tags_45 },
    { 0xf217c1d7, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0x6037c1da, SUC_lexicon_tags_47 },
    { 0xb457c1db, SUC_lexicon_tags_9 },
    { 0xb717c1dc, SUC_lexicon_tags_7 },
    { 0x7697c1dc, SUC_lexicon_tags_17 },
    { 0xf6d7c1de, SUC_lexicon_tags_25 },
    { 0x2937c1df, SUC_lexicon_tags_62 },
    { 0x5f37c1de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8d7c1e2, SUC_lexicon_tags_14 },
    { 0x8a57c1e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4337c1e8, SUC_lexicon_tags_11 },
    { 0x8297c1e8, SUC_lexicon_tags_37 },
    { 0x45f7c1e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5d7c1ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd7c1f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92d7c1f4, SUC_lexicon_tags_0 },
    { 0x6117c1f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf437c1fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a57c200, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b17c202, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f57c20a, SUC_lexicon_tags_33 },
    { 0x2157c20a, SUC_lexicon_tags_92 },
    { 0x3817c20c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7717c20e, SUC_lexicon_tags_39 },
    { 0x3077c20f, SUC_lexicon_tags_17 },
    { 0x317c210, SUC_lexicon_tags_33 },
    { 0x2d97c211, SUC_lexicon_tags_61 },
    { 0xebf7c212, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1337c215, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xad37c218, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28d7c21a, SUC_lexicon_tags_16 },
    { 0x9c37c21b, SUC_lexicon_tags_27 },
    { 0xa9d7c21a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x5ef7c21e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe357c222, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4bd7c224, SUC_lexicon_tags_14 },
    { 0x93b7c225, SUC_lexicon_tags_12 },
    { 0xc3d7c226, SUC_lexicon_tags_91 },
    { 0xec57c225, SUC_lexicon_tags_65 },
    { 0xc5d7c228, SUC_lexicon_tags_43 },
    { 0xdd77c229, SUC_lexicon_tags_37 },
    { 0xe2b7c229, SUC_lexicon_tags_15 },
    { 0xe197c22a, SUC_lexicon_tags_1 },
    { 0xb6d7c229, SUC_lexicon_tags_12 },
    { 0x9e77c22a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc157c230, SUC_lexicon_tags_11 },
    { 0x23f7c230, SUC_lexicon_tags_25 },
    { 0x3937c231, SUC_lexicon_tags_39 },
    { 0xf4d7c233, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb17c238, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9577c23a, SUC_lexicon_tags_35 },
    { 0x50b7c23a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c37c241, SUC_lexicon_tags_2 },
    { 0xa957c242, SUC_lexicon_tags_66 },
    { 0xca17c243, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4577c24a, SUC_lexicon_tags_70 },
    { 0x8d37c24a, SUC_lexicon_tags_33 },
    { 0xf297c24a, SUC_lexicon_tags_37 },
    { 0x5e77c24d, SUC_lexicon_tags_12 },
    { 0xf557c24e, SUC_lexicon_tags_7 },
    { 0x3ad7c24f, SUC_lexicon_tags_12 },
    { 0xe197c24e, SUC_lexicon_tags_55 },
    { 0x9097c24e, SUC_lexicon_tags_68 },
    { 0xf177c24e, SUC_lexicon_tags_15 },
    { 0xe917c24f, SUC_lexicon_tags_25 },
    { 0x3577c251, SUC_lexicon_tags_37 },
    { 0x86f7c255, SUC_lexicon_tags_17 },
    { 0xfb77c24b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa977c258, SUC_lexicon_tags_4 },
    { 0xed17c259, SUC_lexicon_tags_17 },
    { 0x3417c25a, SUC_lexicon_tags_27 },
    { 0x2877c25b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef7c260, SUC_lexicon_tags_68 },
    { 0x3c77c261, SUC_lexicon_tags_10 },
    { 0xa17c260, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x76b7c264, SUC_lexicon_tags_45 },
    { 0x2fb7c265, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce17c267, SUC_lexicon_tags_9 },
    { 0x67f7c268, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc897c271, SUC_lexicon_tags_13 },
    { 0x3637c272, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x99b7c274, SUC_lexicon_tags_14 },
    { 0x14b7c274, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbf7c277, SUC_lexicon_tags_17 },
    { 0x9637c278, SUC_lexicon_tags_33 },
    { 0xec57c279, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced7c281, SUC_lexicon_tags_15 },
    { 0x9f37c282, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f7c28e, SUC_lexicon_tags_25 },
    { 0x7357c28f, SUC_lexicon_tags_48 },
    { 0x9977c28e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce97c293, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd97c29d, SUC_lexicon_tags_1 },
    { 0x39d7c29d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c2a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb97c2a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b7c2b4, SUC_lexicon_tags_49 },
    { 0xc4f7c2b5, SUC_lexicon_tags_4 },
    { 0x4ff7c2b6, SUC_lexicon_tags_7 },
    { 0x6b97c2b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb7c2bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae77c2c3, SUC_lexicon_tags_49 },
    { 0x2917c2c4, SUC_lexicon_tags_26 },
    { 0x12d7c2c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x31f7c2c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d7c2cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7d7c2ce, SUC_lexicon_tags_37 },
    { 0x4cf7c2ce, SUC_lexicon_tags_14 },
    { 0xa517c2d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd457c2d4, SUC_lexicon_tags_77 },
    { 0x29b7c2d5, SUC_lexicon_tags_79 },
    { 0xf937c2d6, SUC_lexicon_tags_27 },
    { 0x4437c2d7, SUC_lexicon_tags_38 },
    { 0x2417c2d5, SUC_lexicon_tags_63 },
    { 0x7ff7c2d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80d7c2db, SUC_lexicon_tags_17 },
    { 0x19f7c2db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8357c2df, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0x20b7c2e1, SUC_lexicon_tags_1143 },
    { 0xff57c2e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5eb7c2e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd977c2e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc297c2e8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5317c2ea, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb177c2ee, SUC_lexicon_tags_30 },
    { 0xf717c2ef, SUC_lexicon_tags_12 },
    { 0xfe17c2f0, SUC_lexicon_tags_14 },
    { 0xa137c2f0, SUC_lexicon_tags_12 },
    { 0xa357c2ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d7c2fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d97c2fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8717c300, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a77c303, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5537c306, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba17c308, SUC_lexicon_tags_37 },
    { 0x9af7c308, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3597c30e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb097c310, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8757c316, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d17c31f, SUC_lexicon_tags_43 },
    { 0x89b7c320, SUC_lexicon_tags_45 },
    { 0xeff7c320, SUC_lexicon_tags_37 },
    { 0xe637c322, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17c327, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d7c32a, SUC_lexicon_tags_87 },
    { 0x4617c32b, SUC_lexicon_tags_2 },
    { 0xd637c32c, SUC_lexicon_tags_61 },
    { 0x7657c32c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd7c330, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd7c334, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xee97c336, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab7c33c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4797c343, SUC_lexicon_tags_17 },
    { 0x2177c344, SUC_lexicon_tags_10 },
    { 0x8897c345, SUC_lexicon_tags_19 },
    { 0x7a57c346, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1377c348, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57c34b, SUC_lexicon_tags_0 },
    { 0x7bd7c34b, SUC_lexicon_tags_65 },
    { 0x9ab7c34d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x39d7c34f, SUC_lexicon_tags_37 },
    { 0x2d57c350, SUC_lexicon_tags_9 },
    { 0xda17c350, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f7c358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3177c35d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b7c364, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe857c36f, SUC_lexicon_tags_19 },
    { 0xb9d7c370, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f7c375, SUC_lexicon_tags_7 },
    { 0x75f7c376, SUC_lexicon_tags_17 },
    { 0x59d7c377, SUC_lexicon_tags_43 },
    { 0x1637c378, SUC_lexicon_tags_4 },
    { 0xad77c379, SUC_lexicon_tags_29 },
    { 0x4ed7c376, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1277c37e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1457c380, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5717c384, SUC_lexicon_tags_3 },
    { 0x54d7c385, SUC_lexicon_tags_11 },
    { 0x68f7c385, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5837c388, SUC_lexicon_tags_19 },
    { 0x6097c388, SUC_lexicon_tags_17 },
    { 0xe97c38a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd7c392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb357c39a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8757c39d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1717c3a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4617c3a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b7c3a9, SUC_lexicon_tags_1 },
    { 0xf8f7c3aa, SUC_lexicon_tags_17 },
    { 0xae77c3aa, SUC_lexicon_tags_65 },
    { 0x5d37c3ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab57c3ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6977c3b1, SUC_lexicon_tags_25 },
    { 0xc4f7c3b2, SUC_lexicon_tags_38 },
    { 0x5277c3b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe77c3b5, SUC_lexicon_tags_45 },
    { 0x2c77c3b6, SUC_lexicon_tags_23 },
    { 0xaf97c3b6, SUC_lexicon_tags_1 },
    { 0x55b7c3b7, SUC_lexicon_tags_25 },
    { 0x5e37c3b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b7c3bf, SUC_lexicon_tags_11 },
    { 0xf477c3c0, SUC_lexicon_tags_9 },
    { 0x3a17c3c1, SUC_lexicon_tags_12 },
    { 0xdcd7c3c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b7c3c6, SUC_lexicon_tags_60 },
    { 0xf877c3c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b17c3cc, SUC_lexicon_tags_3 },
    { 0x8b7c3cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf7c3d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7357c3d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad7c3dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c37c3e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4777c3e7, SUC_lexicon_tags_25 },
    { 0x2517c3e8, SUC_lexicon_tags_37 },
    { 0xeb37c3e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e37c3ed, SUC_lexicon_tags_17 },
    { 0x9317c3ee, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x4a97c3f0, SUC_lexicon_tags_3 },
    { 0x1ff7c3f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f7c3f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a77c3f8, SUC_lexicon_tags_10 },
    { 0x99b7c3f9, SUC_lexicon_tags_2 },
    { 0x9597c3fa, SUC_lexicon_tags_10 },
    { 0x3377c3fb, SUC_lexicon_tags_38 },
    { 0xc457c3fb, SUC_lexicon_tags_10 },
    { 0x91f7c3fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3697c3ff, SUC_lexicon_tags_69 },
    { 0x4577c400, SUC_lexicon_tags_27 },
    { 0x3157c401, SUC_lexicon_tags_11 },
    { 0x6017c402, SUC_lexicon_tags_12 },
    { 0x1a97c401, SUC_lexicon_tags_9 },
    { 0xc997c401, SUC_lexicon_tags_33 },
    { 0x6317c405, SUC_lexicon_tags_25 },
    { 0x5e97c403, SUC_lexicon_tags_1 },
    { 0xf9b7c404, SUC_lexicon_tags_25 },
    { 0x7617c403, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2557c40e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa17c411, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd7c416, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xff97c419, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf157c424, SUC_lexicon_tags_11 },
    { 0xf997c425, SUC_lexicon_tags_11 },
    { 0x3af7c426, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x69b7c428, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a77c42c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d77c430, SUC_lexicon_tags_29 },
    { 0xfd37c430, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3157c43c, SUC_lexicon_tags_37 },
    { 0x5d57c43d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b7c442, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b7c44b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc937c44d, SUC_lexicon_tags_53 },
    { 0xcaf7c44d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab7c451, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcb57c453, SUC_lexicon_tags_12 },
    { 0xb617c454, SUC_lexicon_tags_25 },
    { 0x6057c455, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d17c45a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e17c45c, SUC_lexicon_tags_25 },
    { 0xb677c45d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3b7c45f, SUC_lexicon_tags_47 },
    { 0xa4f7c460, SUC_lexicon_tags_26 },
    { 0x7d17c45f, SUC_lexicon_tags_2 },
    { 0xa677c461, SUC_lexicon_tags_13 },
    { 0x71b7c462, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f97c465, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3917c46e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa2b7c470, SUC_lexicon_tags_47 },
    { 0xc17c471, SUC_lexicon_tags_10 },
    { 0xad77c472, SUC_lexicon_tags_21 },
    { 0x83f7c473, SUC_lexicon_tags_28 },
    { 0x56b7c474, SUC_lexicon_tags_14 },
    { 0xca77c475, SUC_lexicon_tags_10 },
    { 0xc237c471, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b7c47f, SUC_lexicon_tags_15 },
    { 0x8df7c480, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b7c48a, SUC_lexicon_tags_15 },
    { 0xf337c48a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe457c492, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea97c497, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb277c49d, SUC_lexicon_tags_43 },
    { 0xe2b7c49e, SUC_lexicon_tags_37 },
    { 0x2577c49e, SUC_lexicon_tags_47 },
    { 0x3b77c49f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f7c4aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f7c4ad, SUC_lexicon_tags_15 },
    { 0x7b97c4ad, SUC_lexicon_tags_17 },
    { 0x22b7c4ad, SUC_lexicon_tags_15 },
    { 0x417c4b0, SUC_lexicon_tags_17 },
    { 0x6a37c4ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a97c4b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda57c4b7, SUC_lexicon_tags_16 },
    { 0x3dd7c4b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6317c4ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd497c4bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf097c4c2, SUC_lexicon_tags_25 },
    { 0x29f7c4c3, SUC_lexicon_tags_37 },
    { 0x9157c4c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1517c4c6, SUC_lexicon_tags_21 },
    { 0xe9b7c4c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a77c4cb, SUC_lexicon_tags_38 },
    { 0xa4f7c4cc, SUC_lexicon_tags_35 },
    { 0xc5b7c4cd, SUC_lexicon_tags_14 },
    { 0xe9b7c4cc, SUC_lexicon_tags_14 },
    { 0x5997c4cf, SUC_lexicon_tags_33 },
    { 0xac37c4d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77c4d4, SUC_lexicon_tags_9 },
    { 0x8717c4d4, SUC_lexicon_tags_34 },
    { 0xf2f7c4d6, SUC_lexicon_tags_1 },
    { 0xdfd7c4d7, SUC_lexicon_tags_2 },
    { 0x3e97c4d8, SUC_lexicon_tags_14 },
    { 0x6d7c4d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x54f7c4db, SUC_lexicon_tags_10 },
    { 0xaeb7c4dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d7c4df, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe337c4e5, SUC_lexicon_tags_17 },
    { 0xa597c4e6, SUC_lexicon_tags_15 },
    { 0xe297c4e7, SUC_lexicon_tags_121 },
    { 0x4237c4e7, SUC_lexicon_tags_17 },
    { 0xe897c4e9, SUC_lexicon_tags_23 },
    { 0x2097c4ea, SUC_lexicon_tags_12 },
    { 0xe737c4eb, SUC_lexicon_tags_60 },
    { 0xc9f7c4eb, SUC_lexicon_tags_14 },
    { 0x5b37c4e9, SUC_lexicon_tags_37 },
    { 0x6517c4ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9637c4f1, SUC_lexicon_tags_25 },
    { 0x5197c4f1, SUC_lexicon_tags_29 },
    { 0x5c57c4f3, SUC_lexicon_tags_11 },
    { 0xc7d7c4f3, SUC_lexicon_tags_7 },
    { 0x8b77c4f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa997c4fb, SUC_lexicon_tags_66 },
    { 0xe877c4fb, SUC_lexicon_tags_10 },
    { 0xb617c4fd, SUC_lexicon_tags_68 },
    { 0x3f77c4fb, SUC_lexicon_tags_21 },
    { 0x9837c4fb, SUC_lexicon_tags_13 },
    { 0xc677c500, SUC_lexicon_tags_19 },
    { 0xacb7c501, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd197c504, SUC_lexicon_tags_11 },
    { 0xefd7c505, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38d7c507, SUC_lexicon_tags_12 },
    { 0xbe17c507, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf7c50d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b7c510, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4817c51a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37c521, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ab7c523, SUC_lexicon_tags_11 },
    { 0x1037c523, SUC_lexicon_tags_11 },
    { 0x79f7c525, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7eb7c527, SUC_lexicon_tags_25 },
    { 0x5757c527, SUC_lexicon_tags_63 },
    { 0xc717c529, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35f7c52b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad57c536, SUC_lexicon_tags_30 },
    { 0xce77c536, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7bf7c539, SUC_lexicon_tags_27 },
    { 0x91b7c53a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f7c53f, SUC_lexicon_tags_13 },
    { 0x1df7c53f, SUC_lexicon_tags_60 },
    { 0x8a37c541, SUC_lexicon_tags_17 },
    { 0xded7c542, SUC_lexicon_tags_12 },
    { 0xda57c540, SUC_lexicon_tags_60 },
    { 0xb0f7c544, SUC_lexicon_tags_7 },
    { 0xc737c545, SUC_lexicon_tags_44 },
    { 0x73f7c546, SUC_lexicon_tags_25 },
    { 0xdc57c540, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcad7c549, SUC_lexicon_tags_2 },
    { 0xdd7c54a, SUC_lexicon_tags_17 },
    { 0x2857c549, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a77c550, SUC_lexicon_tags_43 },
    { 0xa997c551, SUC_lexicon_tags_47 },
    { 0x8737c552, SUC_lexicon_tags_2 },
    { 0x2557c553, SUC_lexicon_tags_17 },
    { 0x297c552, SUC_lexicon_tags_10 },
    { 0x18b7c555, SUC_lexicon_tags_0 },
    { 0xfd77c556, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8357c558, SUC_lexicon_tags_60 },
    { 0x36b7c559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1897c563, SUC_lexicon_tags_0 },
    { 0x2e37c563, SUC_lexicon_tags_7 },
    { 0x5cf7c565, SUC_lexicon_tags_15 },
    { 0x8457c566, SUC_lexicon_tags_23 },
    { 0x3977c567, SUC_lexicon_tags_31 },
    { 0x1b57c567, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe417c56a, SUC_lexicon_tags_18 },
    { 0x5737c56a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a77c56e, SUC_lexicon_tags_33 },
    { 0x6ed7c56f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c573, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2157c575, SUC_lexicon_tags_39 },
    { 0x4af7c576, SUC_lexicon_tags_19 },
    { 0x4417c577, SUC_lexicon_tags_28 },
    { 0x3e77c576, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d7c57c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d7c582, SUC_lexicon_tags_17 },
    { 0x6b17c583, SUC_lexicon_tags_63 },
    { 0xf497c584, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a17c593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd17c596, SUC_lexicon_tags_54 },
    { 0xa017c597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb7c59b, SUC_lexicon_tags_68 },
    { 0x9e77c59c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b7c5a1, SUC_lexicon_tags_2 },
    { 0xef57c5a2, SUC_lexicon_tags_2 },
    { 0xcd7c5a3, SUC_lexicon_tags_27 },
    { 0x2417c5a4, SUC_lexicon_tags_14 },
    { 0x9b97c5a5, SUC_lexicon_tags_19 },
    { 0x2517c5a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e97c5ab, SUC_lexicon_tags_47 },
    { 0x5e97c5ab, SUC_lexicon_tags_14 },
    { 0x6bb7c5ad, SUC_lexicon_tags_17 },
    { 0x3637c5ae, SUC_lexicon_tags_31 },
    { 0x2df7c5af, SUC_lexicon_tags_37 },
    { 0xe597c5b0, SUC_lexicon_tags_1 },
    { 0xf897c5af, SUC_lexicon_tags_9 },
    { 0x9517c5b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a17c5b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf477c5b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2117c5ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea97c5bc, SUC_lexicon_tags_63 },
    { 0x3cf7c5bd, SUC_lexicon_tags_37 },
    { 0xad7c5be, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6517c5c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7137c5c4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xffb7c5c6, SUC_lexicon_tags_11 },
    { 0x1697c5c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8937c5c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f17c5cc, SUC_lexicon_tags_7 },
    { 0xf0b7c5cd, SUC_lexicon_tags_37 },
    { 0xf317c5ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb557c5d1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5857c5d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f37c5da, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8957c5dd, SUC_lexicon_tags_45 },
    { 0xf577c5de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df7c5e2, SUC_lexicon_tags_60 },
    { 0xe117c5e3, SUC_lexicon_tags_43 },
    { 0xef17c5e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7c5ea, SUC_lexicon_tags_27 },
    { 0xdff7c5eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd7c5ee, SUC_lexicon_tags_37 },
    { 0x2f77c5ef, SUC_lexicon_tags_11 },
    { 0x1df7c5f0, SUC_lexicon_tags_82 },
    { 0xfc57c5f0, SUC_lexicon_tags_30 },
    { 0x96d7c5f1, SUC_lexicon_tags_37 },
    { 0x50d7c5f0, SUC_lexicon_tags_15 },
    { 0x29f7c5f2, SUC_lexicon_tags_49 },
    { 0xe3f7c5f5, SUC_lexicon_tags_7 },
    { 0x9dd7c5f4, SUC_lexicon_tags_27 },
    { 0x6b77c5f7, SUC_lexicon_tags_5 },
    { 0xac77c5f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf7c5fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73b7c5fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c37c601, SUC_lexicon_tags_70 },
    { 0xc6b7c602, SUC_lexicon_tags_11 },
    { 0x5a77c603, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f57c608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb377c60c, SUC_lexicon_tags_25 },
    { 0x32d7c60c, SUC_lexicon_tags_8 },
    { 0xedd7c60d, SUC_lexicon_tags_35 },
    { 0x67b7c60f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7f7c611, SUC_lexicon_tags_37 },
    { 0xb477c612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3017c614, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4497c61a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df7c61d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8197c625, SUC_lexicon_tags_6 },
    { 0x4db7c625, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2897c628, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff7c633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf7c639, SUC_lexicon_tags_31 },
    { 0xfc17c63a, SUC_lexicon_tags_17 },
    { 0xabd7c63b, SUC_lexicon_tags_15 },
    { 0xe7f7c63c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5657c63e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f7c641, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xab17c643, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f7c647, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917c64d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4797c650, SUC_lexicon_tags_33 },
    { 0xe277c651, SUC_lexicon_tags_53 },
    { 0xf057c652, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd677c656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7897c65a, SUC_lexicon_tags_9 },
    { 0xd5d7c65b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9037c65e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb7c663, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9a57c665, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc857c66b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc17c66d, SUC_lexicon_tags_2 },
    { 0x5837c66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6117c671, SUC_lexicon_tags_11 },
    { 0x86d7c671, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f97c674, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb617c676, SUC_lexicon_tags_1 },
    { 0x4217c677, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa57c679, SUC_lexicon_tags_15 },
    { 0xb8b7c67a, SUC_lexicon_tags_23 },
    { 0xe0f7c67b, SUC_lexicon_tags_2 },
    { 0x1337c67b, SUC_lexicon_tags_16 },
    { 0x4717c67a, SUC_lexicon_tags_25 },
    { 0x9cb7c67d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e97c682, SUC_lexicon_tags_61 },
    { 0x83d7c682, SUC_lexicon_tags_12 },
    { 0xc3b7c684, SUC_lexicon_tags_25 },
    { 0xe897c684, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb657c689, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d7c68b, SUC_lexicon_tags_42 },
    { 0x9c97c68c, SUC_lexicon_tags_11 },
    { 0x3897c68d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b7c698, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5257c69c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6937c69f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f7c6a3, SUC_lexicon_tags_12 },
    { 0x7117c6a4, SUC_lexicon_tags_26 },
    { 0x84b7c6a5, SUC_lexicon_tags_63 },
    { 0xcdf7c6a6, SUC_lexicon_tags_27 },
    { 0xa897c6a7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d7c6ae, SUC_lexicon_tags_37 },
    { 0xd6b7c6ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5677c6b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7c6b5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xef57c6b7, SUC_lexicon_tags_39 },
    { 0x1d77c6b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6397c6ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe757c6bc, SUC_lexicon_tags_87 },
    { 0xaa17c6bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f97c6bf, SUC_lexicon_tags_28 },
    { 0xfd57c6c0, SUC_lexicon_tags_11 },
    { 0x2037c6c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d7c6c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb7c6cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd517c6cf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6277c6d3, SUC_lexicon_tags_39 },
    { 0x87b7c6d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1617c6d6, SUC_lexicon_tags_11 },
    { 0x9b77c6d7, SUC_lexicon_tags_31 },
    { 0x7657c6d7, SUC_lexicon_tags_19 },
    { 0xb277c6d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2837c6dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc037c6e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9557c6e6, SUC_lexicon_tags_7 },
    { 0xb37c6e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf337c6eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d7c6ef, SUC_lexicon_tags_25 },
    { 0xf17c6ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f7c6f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd37c6f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc317c6f7, SUC_lexicon_tags_39 },
    { 0x9af7c6f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c6fc, SUC_lexicon_tags_15 },
    { 0x1eb7c6fd, SUC_lexicon_tags_45 },
    { 0x3b17c6fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2f7c700, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff77c705, SUC_lexicon_tags_1 },
    { 0x8b57c706, SUC_lexicon_tags_1 },
    { 0xc5d7c707, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8b7c709, SUC_lexicon_tags_26 },
    { 0xa8f7c70a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8937c70c, SUC_lexicon_tags_38 },
    { 0xfab7c70d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f7c711, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1b57c713, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6c17c715, SUC_lexicon_tags_29 },
    { 0x93f7c716, SUC_lexicon_tags_69 },
    { 0x7a77c717, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb7c71a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f7c721, SUC_lexicon_tags_34 },
    { 0xb917c722, SUC_lexicon_tags_15 },
    { 0xf017c722, SUC_lexicon_tags_7 },
    { 0xb3f7c724, SUC_lexicon_tags_27 },
    { 0x2617c725, SUC_lexicon_tags_17 },
    { 0x1f37c724, SUC_lexicon_tags_25 },
    { 0x2757c724, SUC_lexicon_tags_2 },
    { 0x7b97c728, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe697c72a, SUC_lexicon_tags_45 },
    { 0x1997c72b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb437c72f, SUC_lexicon_tags_45 },
    { 0x8fb7c72f, SUC_lexicon_tags_37 },
    { 0xef97c731, SUC_lexicon_tags_1 },
    { 0x61b7c731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa097c737, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f7c73c, SUC_lexicon_tags_26 },
    { 0xe7b7c73c, SUC_lexicon_tags_49 },
    { 0xe77c73e, SUC_lexicon_tags_17 },
    { 0x6fb7c73f, SUC_lexicon_tags_63 },
    { 0x6117c73d, SUC_lexicon_tags_47 },
    { 0xdb97c741, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb77c743, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf77c746, SUC_lexicon_tags_38 },
    { 0x1537c747, SUC_lexicon_tags_15 },
    { 0x19b7c748, SUC_lexicon_tags_15 },
    { 0x5777c746, SUC_lexicon_tags_7 },
    { 0x44f7c74a, SUC_lexicon_tags_27 },
    { 0xb5d7c74b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b7c74e, SUC_lexicon_tags_16 },
    { 0x1977c74f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e37c753, SUC_lexicon_tags_19 },
    { 0x9557c754, SUC_lexicon_tags_25 },
    { 0x7ad7c755, SUC_lexicon_tags_32 },
    { 0x71b7c756, SUC_lexicon_tags_11 },
    { 0x6d17c754, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb77c75f, SUC_lexicon_tags_17 },
    { 0x77f7c760, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae17c763, SUC_lexicon_tags_18 },
    { 0x46b7c764, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36b7c766, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d7c76d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa0f7c76f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x7ab7c771, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x49b7c773, SUC_lexicon_tags_27 },
    { 0xf437c773, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd677c778, SUC_lexicon_tags_19 },
    { 0x5917c779, SUC_lexicon_tags_510 },
    { 0, NULL },
    { 0, NULL },
    { 0xc097c77c, SUC_lexicon_tags_7 },
    { 0x95f7c77c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f7c781, SUC_lexicon_tags_107 },
    { 0x5677c782, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f7c785, SUC_lexicon_tags_7 },
    { 0xb437c786, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2417c78a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37c78e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa317c791, SUC_lexicon_tags_1 },
    { 0xf917c792, SUC_lexicon_tags_17 },
    { 0xc917c793, SUC_lexicon_tags_11 },
    { 0xb597c794, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ed7c796, SUC_lexicon_tags_17 },
    { 0x10d7c797, SUC_lexicon_tags_0 },
    { 0xecb7c798, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x757c79b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7937c79d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe8f7c79f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af7c7aa, SUC_lexicon_tags_32 },
    { 0x86b7c7ab, SUC_lexicon_tags_82 },
    { 0xba37c7ac, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x1277c7ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d7c7b4, SUC_lexicon_tags_37 },
    { 0x8417c7b5, SUC_lexicon_tags_27 },
    { 0x2757c7b6, SUC_lexicon_tags_23 },
    { 0xab77c7b4, SUC_lexicon_tags_25 },
    { 0x4b7c7b8, SUC_lexicon_tags_12 },
    { 0x2e17c7b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9f7c7bb, SUC_lexicon_tags_21 },
    { 0x1c77c7bb, SUC_lexicon_tags_29 },
    { 0x517c7bd, SUC_lexicon_tags_37 },
    { 0x3b57c7bd, SUC_lexicon_tags_0 },
    { 0xc0f7c7bb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc337c7c1, SUC_lexicon_tags_2 },
    { 0xff97c7c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d7c7c5, SUC_lexicon_tags_47 },
    { 0x4117c7c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff7c7c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35f7c7cb, SUC_lexicon_tags_37 },
    { 0x8997c7cc, SUC_lexicon_tags_28 },
    { 0xcdf7c7cc, SUC_lexicon_tags_33 },
    { 0x1f7c7ce, SUC_lexicon_tags_25 },
    { 0x6577c7cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde17c7d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1837c7d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd7c7d9, SUC_lexicon_tags_31 },
    { 0xf657c7da, SUC_lexicon_tags_37 },
    { 0xc5b7c7da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd17c7dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7617c7e0, SUC_lexicon_tags_8 },
    { 0x2f77c7e0, SUC_lexicon_tags_27 },
    { 0x6fb7c7e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f7c7e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817c7f1, SUC_lexicon_tags_2 },
    { 0x4e77c7f1, SUC_lexicon_tags_4 },
    { 0x6db7c7f3, SUC_lexicon_tags_17 },
    { 0xf6d7c7f3, SUC_lexicon_tags_37 },
    { 0x25f7c7f3, SUC_lexicon_tags_17 },
    { 0x1e17c7f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a77c7f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd17c7fb, SUC_lexicon_tags_7 },
    { 0x6c37c7fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5997c800, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa477c805, SUC_lexicon_tags_74 },
    { 0x2d37c806, SUC_lexicon_tags_19 },
    { 0xa257c807, SUC_lexicon_tags_11 },
    { 0x9417c808, SUC_lexicon_tags_12 },
    { 0x3ab7c808, SUC_lexicon_tags_27 },
    { 0xad17c80a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x52f7c80c, SUC_lexicon_tags_113 },
    { 0x8197c80d, SUC_lexicon_tags_19 },
    { 0x1037c80c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4357c810, SUC_lexicon_tags_1 },
    { 0x40b7c810, SUC_lexicon_tags_12 },
    { 0xe337c812, SUC_lexicon_tags_46 },
    { 0xebf7c813, SUC_lexicon_tags_68 },
    { 0x3477c814, SUC_lexicon_tags_18 },
    { 0x7137c815, SUC_lexicon_tags_12 },
    { 0xc5f7c815, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc257c81a, SUC_lexicon_tags_65 },
    { 0x4b97c81b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f7c82b, SUC_lexicon_tags_25 },
    { 0x8bd7c82c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d7c833, SUC_lexicon_tags_15 },
    { 0x6a57c834, SUC_lexicon_tags_29 },
    { 0xd077c835, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee37c839, SUC_lexicon_tags_11 },
    { 0xc317c839, SUC_lexicon_tags_23 },
    { 0x3397c83b, SUC_lexicon_tags_1 },
    { 0x1797c83c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x36d7c83e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d7c840, SUC_lexicon_tags_12 },
    { 0xe637c841, SUC_lexicon_tags_14 },
    { 0x2d37c841, SUC_lexicon_tags_7 },
    { 0x2537c843, SUC_lexicon_tags_26 },
    { 0xd737c844, SUC_lexicon_tags_1 },
    { 0x6737c841, SUC_lexicon_tags_9 },
    { 0x3977c846, SUC_lexicon_tags_0 },
    { 0x56f7c840, SUC_lexicon_tags_85 },
    { 0xbd57c845, SUC_lexicon_tags_25 },
    { 0x8dd7c845, SUC_lexicon_tags_17 },
    { 0x3477c84a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4637c84c, SUC_lexicon_tags_2 },
    { 0x5ed7c84c, SUC_lexicon_tags_10 },
    { 0x48b7c84c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6077c853, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x937c855, SUC_lexicon_tags_18 },
    { 0x9477c856, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5df7c858, SUC_lexicon_tags_38 },
    { 0x95d7c859, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d7c85d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb77c861, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf257c863, SUC_lexicon_tags_11 },
    { 0xb797c864, SUC_lexicon_tags_389 },
    { 0, NULL },
    { 0x9217c866, SUC_lexicon_tags_37 },
    { 0x8837c867, SUC_lexicon_tags_29 },
    { 0x4217c867, SUC_lexicon_tags_47 },
    { 0xa077c868, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2017c86b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc37c86d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7df7c86f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8757c875, SUC_lexicon_tags_11 },
    { 0xb697c876, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3fd7c878, SUC_lexicon_tags_15 },
    { 0xb197c879, SUC_lexicon_tags_8 },
    { 0xb9d7c879, SUC_lexicon_tags_84 },
    { 0x7237c879, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837c881, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3317c886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d7c88c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b37c891, SUC_lexicon_tags_7 },
    { 0x4097c891, SUC_lexicon_tags_14 },
    { 0xd6b7c893, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7f37c895, SUC_lexicon_tags_0 },
    { 0x49d7c895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58d7c898, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f7c89b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4b37c89d, SUC_lexicon_tags_25 },
    { 0x8777c89d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ff7c8a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa637c8a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8677c8a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b7c8ac, SUC_lexicon_tags_17 },
    { 0x4e37c8ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7997c8b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f7c8b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af7c8b9, SUC_lexicon_tags_23 },
    { 0xf697c8b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5f7c8bc, SUC_lexicon_tags_61 },
    { 0x51b7c8bd, SUC_lexicon_tags_12 },
    { 0x12d7c8bc, SUC_lexicon_tags_33 },
    { 0xe557c8be, SUC_lexicon_tags_33 },
    { 0x7497c8c0, SUC_lexicon_tags_17 },
    { 0xc6f7c8c1, SUC_lexicon_tags_31 },
    { 0xcaf7c8c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc237c8c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7997c8c7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xf757c8c9, SUC_lexicon_tags_21 },
    { 0x9217c8ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2777c8d1, SUC_lexicon_tags_10 },
    { 0x7a57c8d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x777c8d4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3057c8d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb77c8db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b57c8e8, SUC_lexicon_tags_65 },
    { 0x1377c8e9, SUC_lexicon_tags_12 },
    { 0xee57c8ea, SUC_lexicon_tags_11 },
    { 0xeb97c8e9, SUC_lexicon_tags_14 },
    { 0xe9b7c8ec, SUC_lexicon_tags_25 },
    { 0xff37c8ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4297c8f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf637c8f3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x7317c8f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2597c8f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1fd7c8f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x94d7c8fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5477c8fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xba17c901, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31d7c903, SUC_lexicon_tags_67 },
    { 0x317c904, SUC_lexicon_tags_1 },
    { 0xae37c905, SUC_lexicon_tags_60 },
    { 0xb3f7c906, SUC_lexicon_tags_17 },
    { 0x4f37c905, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb057c90b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbeb7c90d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9ad7c90f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd717c911, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6bd7c913, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca57c917, SUC_lexicon_tags_37 },
    { 0x5357c917, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b37c925, SUC_lexicon_tags_37 },
    { 0xeb7c926, SUC_lexicon_tags_53 },
    { 0xd757c925, SUC_lexicon_tags_26 },
    { 0x13d7c926, SUC_lexicon_tags_32 },
    { 0xe937c929, SUC_lexicon_tags_49 },
    { 0x5f77c927, SUC_lexicon_tags_12 },
    { 0xe7d7c929, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d7c932, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4577c936, SUC_lexicon_tags_25 },
    { 0xc337c936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd037c939, SUC_lexicon_tags_17 },
    { 0x8157c93a, SUC_lexicon_tags_37 },
    { 0x8077c93b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3b7c93d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf277c940, SUC_lexicon_tags_25 },
    { 0xed57c941, SUC_lexicon_tags_37 },
    { 0xb1f7c942, SUC_lexicon_tags_39 },
    { 0x4857c943, SUC_lexicon_tags_17 },
    { 0x98b7c944, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b17c948, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3d7c94a, SUC_lexicon_tags_19 },
    { 0x7d57c94b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d7c951, SUC_lexicon_tags_19 },
    { 0xf037c952, SUC_lexicon_tags_17 },
    { 0x7517c953, SUC_lexicon_tags_17 },
    { 0xdb7c952, SUC_lexicon_tags_9 },
    { 0xe997c953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb437c958, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7db7c95a, SUC_lexicon_tags_17 },
    { 0x4257c95a, SUC_lexicon_tags_4 },
    { 0xbb17c95a, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9677c963, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b7c96c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d7c96f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e77c973, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a57c975, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb7c97a, SUC_lexicon_tags_4 },
    { 0xc117c97a, SUC_lexicon_tags_2 },
    { 0x657c97c, SUC_lexicon_tags_46 },
    { 0x74b7c97d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c77c982, SUC_lexicon_tags_12 },
    { 0xd817c983, SUC_lexicon_tags_113 },
    { 0x8ab7c984, SUC_lexicon_tags_11 },
    { 0x6d7c984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb7c98a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c17c994, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4b7c996, SUC_lexicon_tags_15 },
    { 0x3f17c996, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d7c99d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x92b7c99f, SUC_lexicon_tags_14 },
    { 0xb497c9a0, SUC_lexicon_tags_47 },
    { 0x1d17c9a1, SUC_lexicon_tags_32 },
    { 0xa417c9a1, SUC_lexicon_tags_23 },
    { 0x7e97c9a1, SUC_lexicon_tags_12 },
    { 0xff7c9a4, SUC_lexicon_tags_23 },
    { 0xf917c99f, SUC_lexicon_tags_11 },
    { 0xb697c9a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb7c9a9, SUC_lexicon_tags_35 },
    { 0xd9f7c9aa, SUC_lexicon_tags_37 },
    { 0x3c17c9aa, SUC_lexicon_tags_1 },
    { 0xff17c9aa, SUC_lexicon_tags_6 },
    { 0x8897c9ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b57c9b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b7c9b6, SUC_lexicon_tags_8 },
    { 0x5b97c9b6, SUC_lexicon_tags_17 },
    { 0x70b7c9b6, SUC_lexicon_tags_68 },
    { 0x7d37c9b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8337c9c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9fb7c9c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a37c9c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa717c9c9, SUC_lexicon_tags_740 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c9cc, SUC_lexicon_tags_28 },
    { 0x9957c9cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5717c9d1, SUC_lexicon_tags_8 },
    { 0xb57c9d1, SUC_lexicon_tags_11 },
    { 0xcfd7c9d1, SUC_lexicon_tags_45 },
    { 0x3a77c9d1, SUC_lexicon_tags_141 },
    { 0xfd37c9d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f7c9d8, SUC_lexicon_tags_35 },
    { 0xb6f7c9d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2637c9df, SUC_lexicon_tags_15 },
    { 0x197c9e0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6197c9e4, SUC_lexicon_tags_9 },
    { 0x6137c9e4, SUC_lexicon_tags_15 },
    { 0xa937c9e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6af7c9e8, SUC_lexicon_tags_60 },
    { 0x88d7c9e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb877c9ed, SUC_lexicon_tags_63 },
    { 0x78f7c9ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bf7c9f0, SUC_lexicon_tags_25 },
    { 0x17c9f1, SUC_lexicon_tags_19 },
    { 0xfb17c9f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd97c9f5, SUC_lexicon_tags_23 },
    { 0x5717c9f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b7c9fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5937ca04, SUC_lexicon_tags_29 },
    { 0xd817ca04, SUC_lexicon_tags_17 },
    { 0xdf7ca06, SUC_lexicon_tags_5 },
    { 0x3637ca06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b17ca0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe717ca11, SUC_lexicon_tags_7 },
    { 0x497ca11, SUC_lexicon_tags_17 },
    { 0x2397ca13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ff7ca15, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xd937ca17, SUC_lexicon_tags_7 },
    { 0x9e77ca17, SUC_lexicon_tags_31 },
    { 0x1037ca17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ab7ca1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8897ca1d, SUC_lexicon_tags_47 },
    { 0x3597ca1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff7ca24, SUC_lexicon_tags_60 },
    { 0xf877ca25, SUC_lexicon_tags_15 },
    { 0xb8d7ca24, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f7ca2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8277ca30, SUC_lexicon_tags_11 },
    { 0xe6b7ca31, SUC_lexicon_tags_19 },
    { 0x9937ca31, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8657ca35, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a57ca39, SUC_lexicon_tags_1 },
    { 0x2bd7ca3a, SUC_lexicon_tags_27 },
    { 0xc9f7ca3b, SUC_lexicon_tags_23 },
    { 0x71f7ca3a, SUC_lexicon_tags_37 },
    { 0xe077ca3b, SUC_lexicon_tags_17 },
    { 0xebb7ca3d, SUC_lexicon_tags_82 },
    { 0xab77ca3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4337ca42, SUC_lexicon_tags_29 },
    { 0x2a57ca42, SUC_lexicon_tags_49 },
    { 0xa857ca44, SUC_lexicon_tags_9 },
    { 0x2bd7ca45, SUC_lexicon_tags_17 },
    { 0x80b7ca45, SUC_lexicon_tags_31 },
    { 0x8497ca46, SUC_lexicon_tags_61 },
    { 0x15b7ca46, SUC_lexicon_tags_15 },
    { 0x5677ca49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e97ca4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5f7ca4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30f7ca4f, SUC_lexicon_tags_1 },
    { 0xe2d7ca50, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2c17ca52, SUC_lexicon_tags_46 },
    { 0xaab7ca53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e37ca57, SUC_lexicon_tags_43 },
    { 0xbcd7ca58, SUC_lexicon_tags_2 },
    { 0x5d17ca59, SUC_lexicon_tags_37 },
    { 0x8617ca5a, SUC_lexicon_tags_45 },
    { 0x3ef7ca5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1617ca5d, SUC_lexicon_tags_79 },
    { 0xf897ca5e, SUC_lexicon_tags_82 },
    { 0xe037ca5e, SUC_lexicon_tags_7 },
    { 0x4217ca5d, SUC_lexicon_tags_28 },
    { 0xdfd7ca61, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd897ca63, SUC_lexicon_tags_46 },
    { 0x8597ca64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c37ca69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b57ca6e, SUC_lexicon_tags_30 },
    { 0x6657ca6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b57ca73, SUC_lexicon_tags_45 },
    { 0x6eb7ca74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe237ca81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ca84, SUC_lexicon_tags_37 },
    { 0x2e77ca85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f17ca87, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc057ca8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe017ca90, SUC_lexicon_tags_23 },
    { 0x55f7ca91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b17ca93, SUC_lexicon_tags_15 },
    { 0xb997ca94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ad7ca96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df7ca9a, SUC_lexicon_tags_19 },
    { 0x5b77ca9b, SUC_lexicon_tags_7 },
    { 0xc8d7ca9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x277ca9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb657caa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3757caa4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd817caa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d7caae, SUC_lexicon_tags_11 },
    { 0x3057caaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd7cab3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf17cab6, SUC_lexicon_tags_1 },
    { 0x437cab7, SUC_lexicon_tags_37 },
    { 0xba77cab8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8a57caba, SUC_lexicon_tags_11 },
    { 0x1f37caba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17cac0, SUC_lexicon_tags_23 },
    { 0x8fd7cac0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdeb7cac3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d7cac6, SUC_lexicon_tags_23 },
    { 0xc77cac7, SUC_lexicon_tags_26 },
    { 0x4e97cac6, SUC_lexicon_tags_2 },
    { 0xa877cac7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf937cad0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f7cad4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf17cad6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb637cad9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b7cadf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3457cae3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb7cae7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd337caec, SUC_lexicon_tags_19 },
    { 0xfa77caed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d7caf2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6e37caf4, SUC_lexicon_tags_43 },
    { 0x7597caf5, SUC_lexicon_tags_11 },
    { 0x1697caf6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb77cafd, SUC_lexicon_tags_25 },
    { 0xf837cafe, SUC_lexicon_tags_26 },
    { 0xa837caff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4a97cb01, SUC_lexicon_tags_30 },
    { 0x30f7cb01, SUC_lexicon_tags_38 },
    { 0x91b7cb03, SUC_lexicon_tags_37 },
    { 0xf2f7cb04, SUC_lexicon_tags_60 },
    { 0x5eb7cb03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef7cb09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357cb0f, SUC_lexicon_tags_26 },
    { 0x2097cb10, SUC_lexicon_tags_4 },
    { 0xe137cb11, SUC_lexicon_tags_32 },
    { 0x5817cb0f, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c57cb18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe9b7cb1a, SUC_lexicon_tags_2 },
    { 0x17f7cb1b, SUC_lexicon_tags_84 },
    { 0xfb17cb1c, SUC_lexicon_tags_45 },
    { 0x80b7cb1d, SUC_lexicon_tags_63 },
    { 0x5cd7cb1d, SUC_lexicon_tags_7 },
    { 0xc9b7cb1d, SUC_lexicon_tags_66 },
    { 0xb617cb1e, SUC_lexicon_tags_15 },
    { 0xcb57cb1d, SUC_lexicon_tags_23 },
    { 0x3b77cb22, SUC_lexicon_tags_1 },
    { 0xbe17cb23, SUC_lexicon_tags_37 },
    { 0xdc37cb24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc277cb26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9417cb2a, SUC_lexicon_tags_15 },
    { 0x5337cb2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdeb7cb2d, SUC_lexicon_tags_14 },
    { 0x5337cb2e, SUC_lexicon_tags_25 },
    { 0x3b37cb2f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7cb33, SUC_lexicon_tags_57 },
    { 0x86b7cb34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f7cb36, SUC_lexicon_tags_15 },
    { 0xf37cb37, SUC_lexicon_tags_65 },
    { 0x7697cb37, SUC_lexicon_tags_7 },
    { 0x48d7cb39, SUC_lexicon_tags_37 },
    { 0xd8b7cb38, SUC_lexicon_tags_31 },
    { 0x9a17cb3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77cb40, SUC_lexicon_tags_15 },
    { 0xcc77cb41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb7cb46, SUC_lexicon_tags_7 },
    { 0x3257cb47, SUC_lexicon_tags_1 },
    { 0xe577cb46, SUC_lexicon_tags_26 },
    { 0xc2d7cb48, SUC_lexicon_tags_0 },
    { 0xc637cb47, SUC_lexicon_tags_37 },
    { 0xf197cb46, SUC_lexicon_tags_37 },
    { 0x9b57cb4b, SUC_lexicon_tags_68 },
    { 0x49d7cb4d, SUC_lexicon_tags_10 },
    { 0x98b7cb4e, SUC_lexicon_tags_37 },
    { 0x1e77cb4d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6457cb5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d7cb5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6e37cb61, SUC_lexicon_tags_68 },
    { 0x4d77cb62, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5617cb66, SUC_lexicon_tags_65 },
    { 0x95b7cb66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9d37cb69, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf557cb6b, SUC_lexicon_tags_19 },
    { 0x81f7cb6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdbb7cb6e, SUC_lexicon_tags_1 },
    { 0xe757cb6f, SUC_lexicon_tags_15 },
    { 0x4eb7cb6f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5b7cb72, SUC_lexicon_tags_34 },
    { 0x2e17cb73, SUC_lexicon_tags_92 },
    { 0xe757cb73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d7cb78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8577cb7b, SUC_lexicon_tags_15 },
    { 0xdb77cb7c, SUC_lexicon_tags_16 },
    { 0xf357cb7d, SUC_lexicon_tags_19 },
    { 0x44d7cb7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17cb85, SUC_lexicon_tags_45 },
    { 0x30f7cb86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf717cb8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f7cb8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4797cb90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98b7cb92, SUC_lexicon_tags_27 },
    { 0x6577cb93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd7cb99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb7cba6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeeb7cba8, SUC_lexicon_tags_11 },
    { 0x4f97cba9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e77cbae, SUC_lexicon_tags_26 },
    { 0xa437cbaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce17cbb1, SUC_lexicon_tags_19 },
    { 0xb717cbb2, SUC_lexicon_tags_21 },
    { 0x9d17cbb3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b7cbba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f77cbbc, SUC_lexicon_tags_11 },
    { 0x5857cbbc, SUC_lexicon_tags_17 },
    { 0x36f7cbbc, SUC_lexicon_tags_35 },
    { 0x87b7cbbf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7437cbc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x71f7cbc5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6677cbc7, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0x7377cbc9, SUC_lexicon_tags_37 },
    { 0xefd7cbca, SUC_lexicon_tags_25 },
    { 0xe657cbcb, SUC_lexicon_tags_25 },
    { 0x2257cbcc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf0f7cbce, SUC_lexicon_tags_60 },
    { 0x73f7cbce, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f7cbd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7857cbd6, SUC_lexicon_tags_1 },
    { 0xe5f7cbd7, SUC_lexicon_tags_17 },
    { 0x31f7cbd8, SUC_lexicon_tags_2 },
    { 0xdd77cbd8, SUC_lexicon_tags_77 },
    { 0x4b37cbd9, SUC_lexicon_tags_64 },
    { 0xcef7cbd7, SUC_lexicon_tags_1 },
    { 0x1a17cbd8, SUC_lexicon_tags_1 },
    { 0xc137cbdd, SUC_lexicon_tags_8 },
    { 0x5d17cbde, SUC_lexicon_tags_45 },
    { 0xec57cbdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8277cbe4, SUC_lexicon_tags_23 },
    { 0x74f7cbe4, SUC_lexicon_tags_17 },
    { 0xc837cbe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2397cbe8, SUC_lexicon_tags_8 },
    { 0x60b7cbe9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1037cbeb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2377cbed, SUC_lexicon_tags_7 },
    { 0x1897cbee, SUC_lexicon_tags_25 },
    { 0x2cf7cbef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb397cbf5, SUC_lexicon_tags_2 },
    { 0x28f7cbf6, SUC_lexicon_tags_17 },
    { 0x74d7cbf7, SUC_lexicon_tags_25 },
    { 0x5417cbf7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3757cbfd, SUC_lexicon_tags_66 },
    { 0xfa97cbfd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7577cc03, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c77cc0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4357cc0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5797cc11, SUC_lexicon_tags_9 },
    { 0x4e57cc11, SUC_lexicon_tags_54 },
    { 0x4c57cc11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab7cc16, SUC_lexicon_tags_38 },
    { 0x8c37cc17, SUC_lexicon_tags_11 },
    { 0xbcb7cc18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f7cc1e, SUC_lexicon_tags_37 },
    { 0xc2f7cc1e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf7cc22, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c97cc2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c37cc30, SUC_lexicon_tags_13 },
    { 0x5717cc31, SUC_lexicon_tags_17 },
    { 0x5217cc30, SUC_lexicon_tags_82 },
    { 0x9f97cc32, SUC_lexicon_tags_47 },
    { 0xb5b7cc34, SUC_lexicon_tags_37 },
    { 0x4b7cc32, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x2d57cc37, SUC_lexicon_tags_17 },
    { 0xf017cc38, SUC_lexicon_tags_27 },
    { 0x2e37cc39, SUC_lexicon_tags_27 },
    { 0x68d7cc37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7b7cc3c, SUC_lexicon_tags_7 },
    { 0xe2b7cc3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c17cc44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2517cc46, SUC_lexicon_tags_23 },
    { 0xb1d7cc47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4097cc53, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef7cc59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb017cc5d, SUC_lexicon_tags_49 },
    { 0x21f7cc5e, SUC_lexicon_tags_20 },
    { 0x5ef7cc5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca37cc63, SUC_lexicon_tags_45 },
    { 0xc4f7cc64, SUC_lexicon_tags_18 },
    { 0xc77cc64, SUC_lexicon_tags_37 },
    { 0x2037cc66, SUC_lexicon_tags_7 },
    { 0xb497cc64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b7cc6a, SUC_lexicon_tags_32 },
    { 0xb797cc6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf97cc6d, SUC_lexicon_tags_21 },
    { 0x1e97cc6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x38f7cc70, SUC_lexicon_tags_64 },
    { 0x6777cc70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5f37cc73, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b7cc7b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3c57cc7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d7cc82, SUC_lexicon_tags_1 },
    { 0x9077cc82, SUC_lexicon_tags_75 },
    { 0xd117cc84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df7cc8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba17cc93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8037cc95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f7cc99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f7cc9e, SUC_lexicon_tags_11 },
    { 0x63b7cc9f, SUC_lexicon_tags_10 },
    { 0x8657cc9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d7cca3, SUC_lexicon_tags_15 },
    { 0xa197cca4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad7cca9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5657ccab, SUC_lexicon_tags_11 },
    { 0x1fd7ccac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8677ccae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0d7ccb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5157ccb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1537ccba, SUC_lexicon_tags_11 },
    { 0xd2b7ccbb, SUC_lexicon_tags_1 },
    { 0x78b7ccbc, SUC_lexicon_tags_47 },
    { 0x2937ccbd, SUC_lexicon_tags_2 },
    { 0x2ef7ccbe, SUC_lexicon_tags_21 },
    { 0x2b7ccbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d7ccc6, SUC_lexicon_tags_15 },
    { 0x8937ccc7, SUC_lexicon_tags_12 },
    { 0x95f7ccc7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9997cccd, SUC_lexicon_tags_14 },
    { 0x4a77cccd, SUC_lexicon_tags_37 },
    { 0x66f7cccf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed7ccd4, SUC_lexicon_tags_26 },
    { 0x7eb7ccd5, SUC_lexicon_tags_28 },
    { 0x6857ccd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce17ccd9, SUC_lexicon_tags_25 },
    { 0xb0d7ccda, SUC_lexicon_tags_1 },
    { 0xa717ccda, SUC_lexicon_tags_19 },
    { 0x9037ccdc, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb617cce2, SUC_lexicon_tags_35 },
    { 0xd3b7cce3, SUC_lexicon_tags_4 },
    { 0x937cce4, SUC_lexicon_tags_62 },
    { 0x717cce3, SUC_lexicon_tags_37 },
    { 0xdc17cce6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x95f7cce8, SUC_lexicon_tags_25 },
    { 0x3237cce9, SUC_lexicon_tags_6 },
    { 0x35d7ccea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e77ccec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b37ccee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe37ccf0, SUC_lexicon_tags_17 },
    { 0x1897ccf1, SUC_lexicon_tags_38 },
    { 0xd037ccf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc837ccf4, SUC_lexicon_tags_108 },
    { 0x1257ccf5, SUC_lexicon_tags_7 },
    { 0xa077ccf6, SUC_lexicon_tags_17 },
    { 0xfb57ccf7, SUC_lexicon_tags_79 },
    { 0x9517ccf8, SUC_lexicon_tags_17 },
    { 0x7cb7ccf9, SUC_lexicon_tags_37 },
    { 0x30d7ccf8, SUC_lexicon_tags_282 },
    { 0x4277ccfb, SUC_lexicon_tags_1 },
    { 0xd437ccf5, SUC_lexicon_tags_60 },
    { 0x4457ccfd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf997cd05, SUC_lexicon_tags_65 },
    { 0xa237cd05, SUC_lexicon_tags_15 },
    { 0x3797cd07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7897cd0b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf237cd0d, SUC_lexicon_tags_30 },
    { 0x8f97cd0d, SUC_lexicon_tags_46 },
    { 0x3437cd0d, SUC_lexicon_tags_63 },
    { 0xe957cd10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58d7cd12, SUC_lexicon_tags_6 },
    { 0x3317cd13, SUC_lexicon_tags_11 },
    { 0x5757cd13, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb7cd21, SUC_lexicon_tags_0 },
    { 0xb5f7cd22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b97cd25, SUC_lexicon_tags_1 },
    { 0xebb7cd26, SUC_lexicon_tags_45 },
    { 0x8cb7cd25, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b7cd2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x737cd2e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x64b7cd30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc397cd36, SUC_lexicon_tags_8 },
    { 0x37cd37, SUC_lexicon_tags_2 },
    { 0xa7f7cd37, SUC_lexicon_tags_95 },
    { 0xb8d7cd38, SUC_lexicon_tags_11 },
    { 0xd257cd37, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4d57cd3c, SUC_lexicon_tags_38 },
    { 0x71b7cd3d, SUC_lexicon_tags_31 },
    { 0xaa37cd3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce37cd43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa557cd45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1737cd47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8297cd49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3237cd4b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5f7cd4d, SUC_lexicon_tags_1 },
    { 0x5d77cd4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa77cd54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b7cd58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97cd5d, SUC_lexicon_tags_14 },
    { 0xebb7cd5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb677cd60, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4357cd68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8377cd6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf377cd6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed7cd76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbcf7cd78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48f7cd7a, SUC_lexicon_tags_26 },
    { 0x7677cd7a, SUC_lexicon_tags_47 },
    { 0x26f7cd7a, SUC_lexicon_tags_12 },
    { 0xa4f7cd7d, SUC_lexicon_tags_49 },
    { 0x84b7cd7e, SUC_lexicon_tags_8 },
    { 0x6157cd7f, SUC_lexicon_tags_33 },
    { 0x56f7cd7b, SUC_lexicon_tags_25 },
    { 0x2077cd7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c77cd83, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x617cd86, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1ed7cd88, SUC_lexicon_tags_47 },
    { 0xed37cd88, SUC_lexicon_tags_30 },
    { 0xed37cd8a, SUC_lexicon_tags_27 },
    { 0x41b7cd88, SUC_lexicon_tags_37 },
    { 0x17f7cd89, SUC_lexicon_tags_11 },
    { 0xeeb7cd8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5577cd8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb457cd92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc257cd94, SUC_lexicon_tags_37 },
    { 0xebb7cd95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb97cd9c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b7cda4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb997cdac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5737cdb0, SUC_lexicon_tags_14 },
    { 0x2937cdb1, SUC_lexicon_tags_85 },
    { 0xc737cdb2, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b7cdb8, SUC_lexicon_tags_31 },
    { 0x7957cdb9, SUC_lexicon_tags_0 },
    { 0x71b7cdb9, SUC_lexicon_tags_12 },
    { 0x9b17cdb8, SUC_lexicon_tags_14 },
    { 0x5e97cdb8, SUC_lexicon_tags_55 },
    { 0xb6f7cdbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2137cdbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d7cdc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd0f7cdc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48d7cdc6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8d7cdc8, SUC_lexicon_tags_17 },
    { 0x2d17cdc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3957cdcb, SUC_lexicon_tags_60 },
    { 0xa257cdcb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa9b7cdce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa697cdd0, SUC_lexicon_tags_2 },
    { 0x6857cdd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5477cdd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5557cdd6, SUC_lexicon_tags_12 },
    { 0x4e57cdd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe437cdda, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b7cdde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3137cdea, SUC_lexicon_tags_2 },
    { 0x7b7cdeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d97cded, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d37cdf0, SUC_lexicon_tags_12 },
    { 0x24d7cdf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19f7cdf3, SUC_lexicon_tags_12 },
    { 0xb097cdf4, SUC_lexicon_tags_30 },
    { 0xa177cdf5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef7cdf8, SUC_lexicon_tags_39 },
    { 0x997cdf9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3097cdfc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb7ce00, SUC_lexicon_tags_25 },
    { 0xdf57ce00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd497ce05, SUC_lexicon_tags_15 },
    { 0x2057ce06, SUC_lexicon_tags_77 },
    { 0x97d7ce07, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe617ce0b, SUC_lexicon_tags_37 },
    { 0x6db7ce0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d7ce11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd7ce15, SUC_lexicon_tags_33 },
    { 0xb417ce16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53f7ce18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117ce1c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xff7ce1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b7ce24, SUC_lexicon_tags_43 },
    { 0x5217ce24, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d17ce28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1737ce2f, SUC_lexicon_tags_45 },
    { 0x4937ce30, SUC_lexicon_tags_1 },
    { 0x62f7ce30, SUC_lexicon_tags_4 },
    { 0x4777ce30, SUC_lexicon_tags_30 },
    { 0xe6f7ce32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af7ce38, SUC_lexicon_tags_7 },
    { 0x5cd7ce38, SUC_lexicon_tags_37 },
    { 0xd4d7ce3a, SUC_lexicon_tags_3 },
    { 0x65b7ce3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe577ce3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a37ce41, SUC_lexicon_tags_1 },
    { 0x1d17ce42, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8c77ce44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b97ce49, SUC_lexicon_tags_12 },
    { 0x117ce4a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6f57ce4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3f17ce4e, SUC_lexicon_tags_9 },
    { 0x4757ce4f, SUC_lexicon_tags_25 },
    { 0xe5d7ce4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdcb7ce52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597ce57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f7ce5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b7ce60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf097ce62, SUC_lexicon_tags_11 },
    { 0xf717ce62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7537ce67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b7ce6b, SUC_lexicon_tags_57 },
    { 0x8897ce6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd7ce71, SUC_lexicon_tags_14 },
    { 0x9317ce72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f7ce76, SUC_lexicon_tags_11 },
    { 0x3577ce77, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4b97ce79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x617ce7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbc37ce7e, SUC_lexicon_tags_43 },
    { 0xf817ce7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9757ce81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5337ce87, SUC_lexicon_tags_29 },
    { 0xa957ce87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4077ce8a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2937ce8f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf857ce91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef7ce94, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe6f7ce96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61d7ce98, SUC_lexicon_tags_15 },
    { 0xdc77ce99, SUC_lexicon_tags_47 },
    { 0x80f7ce99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7cea0, SUC_lexicon_tags_25 },
    { 0x2c77cea1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ab7cea3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe197cea6, SUC_lexicon_tags_25 },
    { 0xa497cea7, SUC_lexicon_tags_38 },
    { 0xf0f7cea7, SUC_lexicon_tags_11 },
    { 0xccb7cea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c77ceaf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe757ceb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5157ceb7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd7cebd, SUC_lexicon_tags_1 },
    { 0x1df7cebe, SUC_lexicon_tags_56 },
    { 0x9357cebf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6b7cec1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb7cec5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b7cec9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b7cecf, SUC_lexicon_tags_109 },
    { 0x6917ced0, SUC_lexicon_tags_7 },
    { 0x1677ced1, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x9897ced4, SUC_lexicon_tags_16 },
    { 0xb4d7ced5, SUC_lexicon_tags_87 },
    { 0xbb57ced5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc677cedb, SUC_lexicon_tags_35 },
    { 0x1857cedc, SUC_lexicon_tags_8 },
    { 0x6ed7cedd, SUC_lexicon_tags_27 },
    { 0xf2b7cedc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d7cee3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x54d7cee5, SUC_lexicon_tags_47 },
    { 0x5c77cee6, SUC_lexicon_tags_9 },
    { 0x7257cee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e57cee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b7ceed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f7cef4, SUC_lexicon_tags_45 },
    { 0x9017cef5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5757cef7, SUC_lexicon_tags_68 },
    { 0x88d7cef8, SUC_lexicon_tags_60 },
    { 0x7ff7cef8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37cefd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb337cf00, SUC_lexicon_tags_33 },
    { 0xa117cf01, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb57cf0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a77cf10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c57cf1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd257cf22, SUC_lexicon_tags_37 },
    { 0x1837cf23, SUC_lexicon_tags_83 },
    { 0x5637cf22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7737cf2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7cf32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4b7cf34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7057cf36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8b7cf38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f7cf3b, SUC_lexicon_tags_23 },
    { 0x237cf3c, SUC_lexicon_tags_11 },
    { 0xbe77cf3d, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0xf3d7cf3f, SUC_lexicon_tags_23 },
    { 0x1a57cf40, SUC_lexicon_tags_11 },
    { 0xd1f7cf41, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x70b7cf43, SUC_lexicon_tags_63 },
    { 0xbbd7cf44, SUC_lexicon_tags_11 },
    { 0xcdd7cf45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b7cf48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x797cf4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a57cf53, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb797cf57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f97cf59, SUC_lexicon_tags_28 },
    { 0x81b7cf5a, SUC_lexicon_tags_38 },
    { 0x7877cf5b, SUC_lexicon_tags_45 },
    { 0x6f57cf5c, SUC_lexicon_tags_17 },
    { 0xeef7cf5b, SUC_lexicon_tags_11 },
    { 0x6c97cf5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe77cf60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f7cf63, SUC_lexicon_tags_11 },
    { 0x1637cf64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xac37cf67, SUC_lexicon_tags_12 },
    { 0xb7b7cf68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9d7cf6a, SUC_lexicon_tags_30 },
    { 0x6b97cf6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f57cf70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3897cf75, SUC_lexicon_tags_19 },
    { 0x62b7cf76, SUC_lexicon_tags_37 },
    { 0x5737cf76, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4097cf7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b7cf82, SUC_lexicon_tags_60 },
    { 0x4217cf83, SUC_lexicon_tags_13 },
    { 0xc037cf84, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc697cf87, SUC_lexicon_tags_6 },
    { 0x9717cf88, SUC_lexicon_tags_27 },
    { 0xdb57cf89, SUC_lexicon_tags_23 },
    { 0x43d7cf8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa7b7cf8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d7cf92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2977cf95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f7cfa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b7cfa3, SUC_lexicon_tags_26 },
    { 0xce17cfa4, SUC_lexicon_tags_11 },
    { 0xd277cfa4, SUC_lexicon_tags_11 },
    { 0x5c77cfa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f7cfab, SUC_lexicon_tags_60 },
    { 0xbf57cfac, SUC_lexicon_tags_5 },
    { 0x1dd7cfad, SUC_lexicon_tags_87 },
    { 0x72d7cfae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb77cfb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9277cfb5, SUC_lexicon_tags_33 },
    { 0x2d37cfb6, SUC_lexicon_tags_56 },
    { 0x5b77cfb7, SUC_lexicon_tags_37 },
    { 0xad17cfb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af7cfbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x89b7cfbd, SUC_lexicon_tags_11 },
    { 0x4a77cfbe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf897cfc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9df7cfc2, SUC_lexicon_tags_16 },
    { 0xfc77cfc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bb7cfc5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc2b7cfc7, SUC_lexicon_tags_26 },
    { 0x6a57cfc7, SUC_lexicon_tags_65 },
    { 0x637cfc9, SUC_lexicon_tags_47 },
    { 0xb837cfc9, SUC_lexicon_tags_11 },
    { 0x4b17cfcb, SUC_lexicon_tags_7 },
    { 0xe3d7cfcc, SUC_lexicon_tags_29 },
    { 0xa897cfca, SUC_lexicon_tags_7 },
    { 0xc4f7cfce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417cfd2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x297cfd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3477cfd7, SUC_lexicon_tags_3 },
    { 0x297cfd7, SUC_lexicon_tags_37 },
    { 0x84b7cfd7, SUC_lexicon_tags_17 },
    { 0x6537cfda, SUC_lexicon_tags_15 },
    { 0x8e97cfdb, SUC_lexicon_tags_8 },
    { 0xe8b7cfdc, SUC_lexicon_tags_39 },
    { 0x2897cfdb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x72d7cfdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d7cfe4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc97cfe6, SUC_lexicon_tags_25 },
    { 0x88b7cfe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb17cfe9, SUC_lexicon_tags_21 },
    { 0x1f17cfea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9b7cfec, SUC_lexicon_tags_9 },
    { 0x8a17cfec, SUC_lexicon_tags_25 },
    { 0x3037cfee, SUC_lexicon_tags_9 },
    { 0x39d7cfef, SUC_lexicon_tags_37 },
    { 0x8d77cfef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed7cff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e97cff7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fd7cff9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82b7cffb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d37cffd, SUC_lexicon_tags_25 },
    { 0xe817cffd, SUC_lexicon_tags_15 },
    { 0x30b7cffd, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a17d005, SUC_lexicon_tags_47 },
    { 0xbcb7d006, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8777d009, SUC_lexicon_tags_7 },
    { 0xd8b7d009, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f7d00d, SUC_lexicon_tags_25 },
    { 0x3777d00e, SUC_lexicon_tags_37 },
    { 0x4317d00e, SUC_lexicon_tags_46 },
    { 0x37d00d, SUC_lexicon_tags_25 },
    { 0xd197d011, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f17d01a, SUC_lexicon_tags_17 },
    { 0x3537d01b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe297d024, SUC_lexicon_tags_8 },
    { 0x98f7d025, SUC_lexicon_tags_12 },
    { 0xe437d026, SUC_lexicon_tags_23 },
    { 0x2537d027, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c97d02a, SUC_lexicon_tags_43 },
    { 0x4637d02b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xbb37d02d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a37d031, SUC_lexicon_tags_37 },
    { 0x1777d032, SUC_lexicon_tags_7 },
    { 0xde17d033, SUC_lexicon_tags_19 },
    { 0x4857d034, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4257d039, SUC_lexicon_tags_16 },
    { 0x1b97d03a, SUC_lexicon_tags_17 },
    { 0xed97d039, SUC_lexicon_tags_8 },
    { 0x24b7d03b, SUC_lexicon_tags_23 },
    { 0x1637d03d, SUC_lexicon_tags_14 },
    { 0x2997d03e, SUC_lexicon_tags_2 },
    { 0x9877d03f, SUC_lexicon_tags_30 },
    { 0xcf57d040, SUC_lexicon_tags_15 },
    { 0xf57d03d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa057d045, SUC_lexicon_tags_7 },
    { 0xfff7d046, SUC_lexicon_tags_14 },
    { 0x8b77d046, SUC_lexicon_tags_6 },
    { 0xe077d048, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c17d04e, SUC_lexicon_tags_17 },
    { 0x6277d04e, SUC_lexicon_tags_17 },
    { 0xd237d04f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce17d058, SUC_lexicon_tags_82 },
    { 0xe117d058, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6017d05b, SUC_lexicon_tags_14 },
    { 0x2737d05b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc857d05e, SUC_lexicon_tags_43 },
    { 0xd697d05f, SUC_lexicon_tags_8 },
    { 0xc437d060, SUC_lexicon_tags_7 },
    { 0x24b7d060, SUC_lexicon_tags_26 },
    { 0x9cd7d061, SUC_lexicon_tags_79 },
    { 0xffd7d063, SUC_lexicon_tags_8 },
    { 0x2f7d064, SUC_lexicon_tags_46 },
    { 0x3c17d064, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf17d067, SUC_lexicon_tags_7 },
    { 0x5e37d068, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9677d06a, SUC_lexicon_tags_21 },
    { 0xd2b7d06b, SUC_lexicon_tags_85 },
    { 0x3677d06c, SUC_lexicon_tags_26 },
    { 0xf697d06b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf7d071, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d7d074, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x57d076, SUC_lexicon_tags_49 },
    { 0x197d077, SUC_lexicon_tags_15 },
    { 0x6997d078, SUC_lexicon_tags_15 },
    { 0x42b7d076, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb737d07b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db7d080, SUC_lexicon_tags_19 },
    { 0x1f77d081, SUC_lexicon_tags_11 },
    { 0x2997d081, SUC_lexicon_tags_26 },
    { 0x74b7d083, SUC_lexicon_tags_23 },
    { 0xb5f7d083, SUC_lexicon_tags_47 },
    { 0x7037d082, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xae77d087, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d7d08b, SUC_lexicon_tags_27 },
    { 0xd57d08c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5537d08e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe877d090, SUC_lexicon_tags_9 },
    { 0xcbd7d091, SUC_lexicon_tags_37 },
    { 0x5c37d092, SUC_lexicon_tags_17 },
    { 0xf1f7d093, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcd7d095, SUC_lexicon_tags_8 },
    { 0x35f7d095, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4bf7d098, SUC_lexicon_tags_11 },
    { 0x5137d098, SUC_lexicon_tags_33 },
    { 0x35f7d098, SUC_lexicon_tags_43 },
    { 0x6377d09b, SUC_lexicon_tags_2 },
    { 0x9597d09b, SUC_lexicon_tags_3 },
    { 0x7817d09d, SUC_lexicon_tags_11 },
    { 0xccf7d09d, SUC_lexicon_tags_8 },
    { 0x357d09f, SUC_lexicon_tags_37 },
    { 0xa817d09d, SUC_lexicon_tags_17 },
    { 0x9b17d0a1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee17d0a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd77d0aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7e97d0ac, SUC_lexicon_tags_26 },
    { 0x5277d0ad, SUC_lexicon_tags_24 },
    { 0xcad7d0ae, SUC_lexicon_tags_25 },
    { 0x9997d0ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7657d0b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b37d0b7, SUC_lexicon_tags_37 },
    { 0x5bf7d0b7, SUC_lexicon_tags_11 },
    { 0x54d7d0b7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2697d0bc, SUC_lexicon_tags_12 },
    { 0xae17d0bd, SUC_lexicon_tags_10 },
    { 0x9bd7d0bd, SUC_lexicon_tags_61 },
    { 0xe117d0bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xedd7d0c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc417d0c5, SUC_lexicon_tags_39 },
    { 0x3a57d0c6, SUC_lexicon_tags_38 },
    { 0x9d97d0c7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b7d0cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x117d0ce, SUC_lexicon_tags_53 },
    { 0xbd97d0cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6237d0d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16b7d0d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37d0dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa57d0e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce97d0eb, SUC_lexicon_tags_17 },
    { 0x20d7d0ec, SUC_lexicon_tags_15 },
    { 0x4377d0ec, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b77d0f4, SUC_lexicon_tags_7 },
    { 0xf2f7d0f5, SUC_lexicon_tags_0 },
    { 0x2257d0f4, SUC_lexicon_tags_32 },
    { 0xfab7d0f7, SUC_lexicon_tags_47 },
    { 0x1f77d0f4, SUC_lexicon_tags_11 },
    { 0x10f7d0f9, SUC_lexicon_tags_39 },
    { 0x5717d0fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1117d106, SUC_lexicon_tags_27 },
    { 0x5b77d106, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb217d10f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3397d112, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b7d117, SUC_lexicon_tags_15 },
    { 0x9737d118, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b7d11f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d7d126, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cd7d128, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xacf7d12a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad97d131, SUC_lexicon_tags_26 },
    { 0x7a17d132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3897d138, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd7d13a, SUC_lexicon_tags_55 },
    { 0xfc17d13b, SUC_lexicon_tags_14 },
    { 0x5777d13c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae97d141, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x94f7d143, SUC_lexicon_tags_11 },
    { 0x8917d144, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5d7d146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb7d14c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd7d151, SUC_lexicon_tags_47 },
    { 0xd257d151, SUC_lexicon_tags_33 },
    { 0x9c57d152, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1bf7d155, SUC_lexicon_tags_2 },
    { 0x5497d156, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2b37d158, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f7d15e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe97d161, SUC_lexicon_tags_12 },
    { 0x7ab7d161, SUC_lexicon_tags_17 },
    { 0xb017d163, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab7d166, SUC_lexicon_tags_33 },
    { 0x9f77d166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb7d16a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd7d176, SUC_lexicon_tags_0 },
    { 0x7af7d177, SUC_lexicon_tags_0 },
    { 0x1c17d178, SUC_lexicon_tags_27 },
    { 0xd8f7d178, SUC_lexicon_tags_37 },
    { 0x7d17d17a, SUC_lexicon_tags_27 },
    { 0xce97d17b, SUC_lexicon_tags_49 },
    { 0x7cb7d17c, SUC_lexicon_tags_12 },
    { 0x8fd7d17d, SUC_lexicon_tags_23 },
    { 0xe777d17e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f7d182, SUC_lexicon_tags_25 },
    { 0x2137d183, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6597d185, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bb7d187, SUC_lexicon_tags_15 },
    { 0x9757d188, SUC_lexicon_tags_11 },
    { 0x2e97d187, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29d7d18b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f7d18e, SUC_lexicon_tags_15 },
    { 0xc357d18f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a77d191, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde97d193, SUC_lexicon_tags_38 },
    { 0x7d97d194, SUC_lexicon_tags_14 },
    { 0xfe57d195, SUC_lexicon_tags_17 },
    { 0x6a97d196, SUC_lexicon_tags_33 },
    { 0x39b7d196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8257d199, SUC_lexicon_tags_43 },
    { 0x7297d199, SUC_lexicon_tags_12 },
    { 0xa97d19b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf7d19e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa557d1a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d7d1af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f7d1b4, SUC_lexicon_tags_37 },
    { 0x4a77d1b5, SUC_lexicon_tags_39 },
    { 0x76f7d1b5, SUC_lexicon_tags_19 },
    { 0xda77d1b5, SUC_lexicon_tags_47 },
    { 0xab57d1b8, SUC_lexicon_tags_218 },
    { 0x9f57d1b9, SUC_lexicon_tags_32 },
    { 0xa377d1b7, SUC_lexicon_tags_12 },
    { 0xa277d1bb, SUC_lexicon_tags_37 },
    { 0x6217d1b5, SUC_lexicon_tags_0 },
    { 0xf697d1bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1197d1c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x87d7d1c2, SUC_lexicon_tags_37 },
    { 0xe8f7d1c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1097d1c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf7d1c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe397d1ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd17d1d1, SUC_lexicon_tags_11 },
    { 0xf4b7d1d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8477d1d5, SUC_lexicon_tags_11 },
    { 0xcb77d1d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8557d1da, SUC_lexicon_tags_32 },
    { 0xd5f7d1da, SUC_lexicon_tags_44 },
    { 0xfd7d1dc, SUC_lexicon_tags_11 },
    { 0x6437d1dd, SUC_lexicon_tags_77 },
    { 0x14d7d1de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd7d1e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0b7d1e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa457d1e7, SUC_lexicon_tags_15 },
    { 0xfab7d1e8, SUC_lexicon_tags_56 },
    { 0x2d37d1e7, SUC_lexicon_tags_23 },
    { 0xd697d1e7, SUC_lexicon_tags_27 },
    { 0x81d7d1eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17d1ef, SUC_lexicon_tags_12 },
    { 0x3cb7d1f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c77d1f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf397d1f7, SUC_lexicon_tags_37 },
    { 0xda37d1f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6957d1fa, SUC_lexicon_tags_7 },
    { 0xd037d1fa, SUC_lexicon_tags_28 },
    { 0xee57d1fb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b7d200, SUC_lexicon_tags_69 },
    { 0x7057d201, SUC_lexicon_tags_82 },
    { 0xb177d200, SUC_lexicon_tags_1 },
    { 0xbd17d203, SUC_lexicon_tags_6 },
    { 0xef17d204, SUC_lexicon_tags_46 },
    { 0x897d205, SUC_lexicon_tags_63 },
    { 0x66b7d206, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d37d208, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4557d20c, SUC_lexicon_tags_14 },
    { 0x1c37d20d, SUC_lexicon_tags_25 },
    { 0x4dd7d20c, SUC_lexicon_tags_15 },
    { 0x1197d20f, SUC_lexicon_tags_17 },
    { 0xf357d20f, SUC_lexicon_tags_25 },
    { 0x4397d20d, SUC_lexicon_tags_7 },
    { 0x1217d20f, SUC_lexicon_tags_2 },
    { 0x2337d213, SUC_lexicon_tags_21 },
    { 0x44d7d213, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c37d216, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4317d218, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d7d21d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b17d220, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4d97d222, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b7d228, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa6b7d22a, SUC_lexicon_tags_831 },
    { 0x3217d22a, SUC_lexicon_tags_48 },
    { 0x76d7d22c, SUC_lexicon_tags_29 },
    { 0xc397d22b, SUC_lexicon_tags_25 },
    { 0x7357d22d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e57d231, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x95b7d233, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x8d37d235, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbf7d237, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9777d23d, SUC_lexicon_tags_2 },
    { 0xd8d7d23e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3217d240, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d7d244, SUC_lexicon_tags_7 },
    { 0x677d244, SUC_lexicon_tags_27 },
    { 0x72b7d246, SUC_lexicon_tags_11 },
    { 0xb3b7d246, SUC_lexicon_tags_103 },
    { 0x8c37d245, SUC_lexicon_tags_0 },
    { 0x1737d248, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdb7d24b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d77d250, SUC_lexicon_tags_2 },
    { 0x48d7d250, SUC_lexicon_tags_25 },
    { 0x75b7d252, SUC_lexicon_tags_25 },
    { 0xebd7d250, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd7d256, SUC_lexicon_tags_0 },
    { 0x6377d257, SUC_lexicon_tags_49 },
    { 0x8bb7d256, SUC_lexicon_tags_7 },
    { 0xc137d259, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4c97d25b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe117d25f, SUC_lexicon_tags_80 },
    { 0xdc37d260, SUC_lexicon_tags_28 },
    { 0x1757d261, SUC_lexicon_tags_11 },
    { 0xf537d262, SUC_lexicon_tags_18 },
    { 0x1b57d262, SUC_lexicon_tags_0 },
    { 0x5097d264, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9817d26c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a97d26e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4177d272, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f7d27b, SUC_lexicon_tags_1491 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed7d27e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f17d281, SUC_lexicon_tags_2 },
    { 0xc5b7d282, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe857d284, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x717d287, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5d7d289, SUC_lexicon_tags_170 },
    { 0x7897d28a, SUC_lexicon_tags_12 },
    { 0x3f17d289, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce17d28e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f7d294, SUC_lexicon_tags_46 },
    { 0xfe97d295, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d7d29e, SUC_lexicon_tags_55 },
    { 0xc9d7d29e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d7d2ac, SUC_lexicon_tags_46 },
    { 0xae97d2ac, SUC_lexicon_tags_69 },
    { 0x5657d2ae, SUC_lexicon_tags_23 },
    { 0xd057d2af, SUC_lexicon_tags_12 },
    { 0xa617d2b0, SUC_lexicon_tags_43 },
    { 0x9337d2b0, SUC_lexicon_tags_31 },
    { 0xb937d2b0, SUC_lexicon_tags_7 },
    { 0x44d7d2b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad7d2b6, SUC_lexicon_tags_18 },
    { 0xe3d7d2b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d57d2c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd7d2c6, SUC_lexicon_tags_16 },
    { 0x1037d2c7, SUC_lexicon_tags_19 },
    { 0xa1f7d2c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4497d2ca, SUC_lexicon_tags_65 },
    { 0x6817d2ca, SUC_lexicon_tags_32 },
    { 0x6d77d2cc, SUC_lexicon_tags_6 },
    { 0x7297d2cd, SUC_lexicon_tags_11 },
    { 0x2757d2ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f7d2d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c17d2d9, SUC_lexicon_tags_37 },
    { 0xa9f7d2da, SUC_lexicon_tags_17 },
    { 0xf757d2db, SUC_lexicon_tags_17 },
    { 0x8997d2db, SUC_lexicon_tags_7 },
    { 0x6537d2d9, SUC_lexicon_tags_11 },
    { 0x8517d2de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x117d2e0, SUC_lexicon_tags_32 },
    { 0x2d37d2e0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x3737d2e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77b7d2e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b7d2ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db7d2ee, SUC_lexicon_tags_49 },
    { 0xd517d2ee, SUC_lexicon_tags_19 },
    { 0x3797d2ee, SUC_lexicon_tags_23 },
    { 0x3797d2f1, SUC_lexicon_tags_15 },
    { 0xce97d2f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c17d2f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8697d2f7, SUC_lexicon_tags_17 },
    { 0xb737d2f8, SUC_lexicon_tags_79 },
    { 0x8457d2f7, SUC_lexicon_tags_17 },
    { 0xfd37d2fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x577d2fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d7d302, SUC_lexicon_tags_34 },
    { 0x6e17d303, SUC_lexicon_tags_17 },
    { 0xd497d304, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37d30a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5057d30d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7177d311, SUC_lexicon_tags_14 },
    { 0x3597d312, SUC_lexicon_tags_60 },
    { 0x94f7d313, SUC_lexicon_tags_17 },
    { 0xa177d313, SUC_lexicon_tags_23 },
    { 0xb897d315, SUC_lexicon_tags_68 },
    { 0xb717d316, SUC_lexicon_tags_107 },
    { 0x5657d317, SUC_lexicon_tags_69 },
    { 0x9517d318, SUC_lexicon_tags_15 },
    { 0x86b7d319, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3137d31c, SUC_lexicon_tags_12 },
    { 0x2797d31d, SUC_lexicon_tags_17 },
    { 0x8597d31e, SUC_lexicon_tags_5 },
    { 0xf4f7d31f, SUC_lexicon_tags_37 },
    { 0x1ad7d320, SUC_lexicon_tags_12 },
    { 0xdbf7d31f, SUC_lexicon_tags_37 },
    { 0x737d322, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6957d327, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe277d329, SUC_lexicon_tags_25 },
    { 0x81f7d32a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x51d7d32c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x12d7d32e, SUC_lexicon_tags_37 },
    { 0x9df7d32f, SUC_lexicon_tags_66 },
    { 0xe877d32f, SUC_lexicon_tags_8 },
    { 0x3137d32f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2097d33d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af7d340, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6577d34d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277d351, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb37d353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f7d35b, SUC_lexicon_tags_37 },
    { 0x4e17d35c, SUC_lexicon_tags_49 },
    { 0x32d7d35b, SUC_lexicon_tags_37 },
    { 0x75d7d35b, SUC_lexicon_tags_1 },
    { 0xf77d35f, SUC_lexicon_tags_19 },
    { 0x4a97d360, SUC_lexicon_tags_68 },
    { 0x6557d360, SUC_lexicon_tags_33 },
    { 0x1c57d361, SUC_lexicon_tags_65 },
    { 0xae77d363, SUC_lexicon_tags_47 },
    { 0xfcd7d363, SUC_lexicon_tags_37 },
    { 0xe837d365, SUC_lexicon_tags_30 },
    { 0xa6b7d366, SUC_lexicon_tags_19 },
    { 0x557d367, SUC_lexicon_tags_11 },
    { 0x2917d368, SUC_lexicon_tags_17 },
    { 0xdbb7d369, SUC_lexicon_tags_25 },
    { 0xd0b7d365, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x5497d36d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb37d370, SUC_lexicon_tags_29 },
    { 0x5f37d370, SUC_lexicon_tags_17 },
    { 0xe0b7d370, SUC_lexicon_tags_25 },
    { 0xaeb7d373, SUC_lexicon_tags_37 },
    { 0x277d374, SUC_lexicon_tags_2 },
    { 0x1817d375, SUC_lexicon_tags_48 },
    { 0x36d7d375, SUC_lexicon_tags_8 },
    { 0xd1b7d377, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d7d37b, SUC_lexicon_tags_2 },
    { 0xdc97d37c, SUC_lexicon_tags_1 },
    { 0x1c57d37b, SUC_lexicon_tags_66 },
    { 0x537d37e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x4137d380, SUC_lexicon_tags_12 },
    { 0xb317d381, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b57d383, SUC_lexicon_tags_25 },
    { 0x25f7d384, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc117d38a, SUC_lexicon_tags_79 },
    { 0x3b57d38b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1bb7d38d, SUC_lexicon_tags_31 },
    { 0xca97d38d, SUC_lexicon_tags_13 },
    { 0x29b7d38f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1dd7d391, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd437d393, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa37d39b, SUC_lexicon_tags_28 },
    { 0x51d7d39c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b7d3a4, SUC_lexicon_tags_1 },
    { 0x9a17d3a4, SUC_lexicon_tags_28 },
    { 0xbef7d3a6, SUC_lexicon_tags_10 },
    { 0xea17d3a6, SUC_lexicon_tags_6 },
    { 0x3417d3a4, SUC_lexicon_tags_48 },
    { 0x9617d3a8, SUC_lexicon_tags_1 },
    { 0xfa37d3aa, SUC_lexicon_tags_2 },
    { 0x43f7d3aa, SUC_lexicon_tags_12 },
    { 0xe537d3ac, SUC_lexicon_tags_7 },
    { 0xdf77d3ad, SUC_lexicon_tags_60 },
    { 0x8357d3a5, SUC_lexicon_tags_37 },
    { 0x9ab7d3a9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1437d3b8, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb7d3be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f7d3c9, SUC_lexicon_tags_30 },
    { 0x7397d3ca, SUC_lexicon_tags_25 },
    { 0x20b7d3cb, SUC_lexicon_tags_39 },
    { 0x9c37d3c9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb817d3d1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c57d3d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfcb7d3d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef7d3de, SUC_lexicon_tags_11 },
    { 0x517d3df, SUC_lexicon_tags_29 },
    { 0x62f7d3e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1077d3e4, SUC_lexicon_tags_25 },
    { 0x87b7d3e4, SUC_lexicon_tags_16 },
    { 0x9417d3e6, SUC_lexicon_tags_7 },
    { 0xae37d3e5, SUC_lexicon_tags_2 },
    { 0x6af7d3e4, SUC_lexicon_tags_11 },
    { 0xd717d3e9, SUC_lexicon_tags_44 },
    { 0x3897d3ea, SUC_lexicon_tags_31 },
    { 0x7cd7d3e9, SUC_lexicon_tags_38 },
    { 0x1697d3ec, SUC_lexicon_tags_1 },
    { 0x8e37d3ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7d3f2, SUC_lexicon_tags_11 },
    { 0xb817d3f3, SUC_lexicon_tags_7 },
    { 0x63f7d3f3, SUC_lexicon_tags_1 },
    { 0x3437d3f3, SUC_lexicon_tags_11 },
    { 0x8ab7d3f6, SUC_lexicon_tags_21 },
    { 0x5a57d3f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f7d3fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8057d3fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7d401, SUC_lexicon_tags_84 },
    { 0x50b7d401, SUC_lexicon_tags_32 },
    { 0x2057d403, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe297d408, SUC_lexicon_tags_33 },
    { 0xe3f7d409, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf7d411, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4057d416, SUC_lexicon_tags_37 },
    { 0x2df7d417, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f37d41d, SUC_lexicon_tags_17 },
    { 0x6cd7d41d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4f97d420, SUC_lexicon_tags_30 },
    { 0x2b7d420, SUC_lexicon_tags_22 },
    { 0xbdb7d422, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd897d427, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d7d42a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf5d7d42c, SUC_lexicon_tags_12 },
    { 0x9d97d42d, SUC_lexicon_tags_19 },
    { 0xb2b7d42e, SUC_lexicon_tags_37 },
    { 0x1e97d42e, SUC_lexicon_tags_6 },
    { 0xf997d42f, SUC_lexicon_tags_46 },
    { 0x457d431, SUC_lexicon_tags_1 },
    { 0x33b7d431, SUC_lexicon_tags_2 },
    { 0xf657d42e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7d437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfaf7d439, SUC_lexicon_tags_7 },
    { 0x3037d43a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b17d43c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7197d443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c17d446, SUC_lexicon_tags_12 },
    { 0x5677d447, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7357d44a, SUC_lexicon_tags_14 },
    { 0xa837d44a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b7d44e, SUC_lexicon_tags_37 },
    { 0x6637d44f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe257d451, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb97d453, SUC_lexicon_tags_30 },
    { 0xa5f7d454, SUC_lexicon_tags_11 },
    { 0x8957d454, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbb97d457, SUC_lexicon_tags_25 },
    { 0xdc57d458, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8577d45d, SUC_lexicon_tags_14 },
    { 0x1fd7d45d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6017d460, SUC_lexicon_tags_37 },
    { 0x6fb7d460, SUC_lexicon_tags_46 },
    { 0x5257d462, SUC_lexicon_tags_2 },
    { 0x9f7d461, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb57d466, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8977d46b, SUC_lexicon_tags_17 },
    { 0xaeb7d46b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b7d46f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e17d471, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437d479, SUC_lexicon_tags_2 },
    { 0xffd7d47a, SUC_lexicon_tags_16 },
    { 0x4d7d47a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b7d47e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f7d482, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b7d486, SUC_lexicon_tags_29 },
    { 0xe4f7d487, SUC_lexicon_tags_31 },
    { 0xe637d487, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b77d48f, SUC_lexicon_tags_11 },
    { 0xa477d490, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f97d49d, SUC_lexicon_tags_6 },
    { 0x7d97d49d, SUC_lexicon_tags_25 },
    { 0xf6d7d49f, SUC_lexicon_tags_1 },
    { 0x1137d4a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7757d4a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f7d4a8, SUC_lexicon_tags_17 },
    { 0x1037d4a9, SUC_lexicon_tags_2 },
    { 0xf937d4a9, SUC_lexicon_tags_8 },
    { 0xdb97d4a9, SUC_lexicon_tags_14 },
    { 0x8257d4ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6237d4af, SUC_lexicon_tags_61 },
    { 0x3c77d4b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b7d4b5, SUC_lexicon_tags_8 },
    { 0xacd7d4b5, SUC_lexicon_tags_27 },
    { 0x7cd7d4b6, SUC_lexicon_tags_17 },
    { 0x40f7d4b8, SUC_lexicon_tags_17 },
    { 0x17b7d4b8, SUC_lexicon_tags_9 },
    { 0x2357d4ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1257d4bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1417d4c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1637d4c9, SUC_lexicon_tags_49 },
    { 0xe257d4c9, SUC_lexicon_tags_7 },
    { 0xf977d4cb, SUC_lexicon_tags_45 },
    { 0x4397d4cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317d4d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xac77d4d3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd197d4d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa597d4db, SUC_lexicon_tags_60 },
    { 0x517d4dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4d7d4de, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad7d4e2, SUC_lexicon_tags_30 },
    { 0x1d77d4e2, SUC_lexicon_tags_4 },
    { 0xa977d4e2, SUC_lexicon_tags_1 },
    { 0xf757d4e5, SUC_lexicon_tags_25 },
    { 0x6557d4e6, SUC_lexicon_tags_33 },
    { 0xec57d4e6, SUC_lexicon_tags_37 },
    { 0x9dd7d4e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8957d4f8, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc857d502, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5997d508, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb77d50a, SUC_lexicon_tags_12 },
    { 0xfef7d50b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda77d50d, SUC_lexicon_tags_15 },
    { 0xd857d50e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef7d512, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2637d515, SUC_lexicon_tags_37 },
    { 0x3b7d515, SUC_lexicon_tags_11 },
    { 0x4077d515, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af7d51d, SUC_lexicon_tags_45 },
    { 0x9057d51d, SUC_lexicon_tags_11 },
    { 0xb1d7d51f, SUC_lexicon_tags_15 },
    { 0x1237d51d, SUC_lexicon_tags_49 },
    { 0x8337d521, SUC_lexicon_tags_11 },
    { 0xd6d7d522, SUC_lexicon_tags_15 },
    { 0x5277d523, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d7d527, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1877d529, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d52c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x62f7d52e, SUC_lexicon_tags_87 },
    { 0x5757d52e, SUC_lexicon_tags_32 },
    { 0xd877d530, SUC_lexicon_tags_15 },
    { 0x5b37d530, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe457d534, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d7d53c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db7d542, SUC_lexicon_tags_25 },
    { 0xe357d543, SUC_lexicon_tags_44 },
    { 0x6f57d544, SUC_lexicon_tags_11 },
    { 0xa9b7d545, SUC_lexicon_tags_11 },
    { 0x5a57d546, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb77d548, SUC_lexicon_tags_47 },
    { 0x7997d549, SUC_lexicon_tags_66 },
    { 0x8b77d549, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f17d551, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4277d556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b7d55d, SUC_lexicon_tags_11 },
    { 0x8277d55e, SUC_lexicon_tags_25 },
    { 0x5d7d55f, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x3d77d561, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b7d564, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b17d567, SUC_lexicon_tags_95 },
    { 0x4577d567, SUC_lexicon_tags_77 },
    { 0xb977d568, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d7d56f, SUC_lexicon_tags_33 },
    { 0x9d7d570, SUC_lexicon_tags_10 },
    { 0xacd7d570, SUC_lexicon_tags_28 },
    { 0x7cd7d572, SUC_lexicon_tags_7 },
    { 0xf57d573, SUC_lexicon_tags_39 },
    { 0x32f7d572, SUC_lexicon_tags_30 },
    { 0xf617d575, SUC_lexicon_tags_17 },
    { 0x5f97d575, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc477d57a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9697d580, SUC_lexicon_tags_45 },
    { 0xbb77d581, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd357d583, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3737d587, SUC_lexicon_tags_44 },
    { 0x3057d588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7317d58b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88b7d58d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb7d593, SUC_lexicon_tags_68 },
    { 0x8537d594, SUC_lexicon_tags_27 },
    { 0xe957d594, SUC_lexicon_tags_10 },
    { 0xc737d596, SUC_lexicon_tags_43 },
    { 0x1817d597, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8277d59b, SUC_lexicon_tags_26 },
    { 0xb017d59c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf317d59e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb5b7d5a0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd7d5a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3777d5a7, SUC_lexicon_tags_29 },
    { 0xc737d5a8, SUC_lexicon_tags_34 },
    { 0xbbb7d5a9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd917d5ac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9d77d5ae, SUC_lexicon_tags_30 },
    { 0x1df7d5af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd7d5b6, SUC_lexicon_tags_29 },
    { 0xe6d7d5b6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x8ab7d5b9, SUC_lexicon_tags_1 },
    { 0xd3b7d5b9, SUC_lexicon_tags_61 },
    { 0x5f97d5ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6417d5bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b7d5c0, SUC_lexicon_tags_1109 },
    { 0xb3d7d5c1, SUC_lexicon_tags_17 },
    { 0x75d7d5c0, SUC_lexicon_tags_33 },
    { 0x7af7d5c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777d5c7, SUC_lexicon_tags_8 },
    { 0xc457d5c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f37d5cc, SUC_lexicon_tags_25 },
    { 0xf897d5cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc757d5d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a17d5dd, SUC_lexicon_tags_47 },
    { 0xa5f7d5dd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7db7d5e0, SUC_lexicon_tags_30 },
    { 0x79f7d5e1, SUC_lexicon_tags_2 },
    { 0x5337d5e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb7d5e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x257d5e7, SUC_lexicon_tags_2 },
    { 0x5277d5e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f7d5eb, SUC_lexicon_tags_37 },
    { 0x63b7d5ec, SUC_lexicon_tags_1 },
    { 0xd3d7d5ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed7d5f2, SUC_lexicon_tags_87 },
    { 0x4df7d5f2, SUC_lexicon_tags_14 },
    { 0x7457d5f4, SUC_lexicon_tags_25 },
    { 0xeaf7d5f5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x277d5f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4257d5f9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2657d5fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd7d5ff, SUC_lexicon_tags_56 },
    { 0x1b57d600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x677d603, SUC_lexicon_tags_11 },
    { 0x8cd7d603, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4d7d606, SUC_lexicon_tags_43 },
    { 0x43b7d606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6437d60a, SUC_lexicon_tags_17 },
    { 0x91f7d60b, SUC_lexicon_tags_55 },
    { 0x3737d60b, SUC_lexicon_tags_49 },
    { 0x9ab7d60d, SUC_lexicon_tags_31 },
    { 0xeb97d60b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df7d612, SUC_lexicon_tags_252 },
    { 0xe297d612, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757d61f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x17d621, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe717d623, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff7d62a, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd7d62d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5d37d62f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf957d631, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38d7d633, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf57d635, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb7d63a, SUC_lexicon_tags_2 },
    { 0x60f7d63b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4b97d63d, SUC_lexicon_tags_17 },
    { 0xdf37d63d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e97d640, SUC_lexicon_tags_11 },
    { 0xbcd7d640, SUC_lexicon_tags_31 },
    { 0x4197d641, SUC_lexicon_tags_18 },
    { 0x3f37d640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57d7d645, SUC_lexicon_tags_1 },
    { 0x77b7d646, SUC_lexicon_tags_4 },
    { 0x5617d647, SUC_lexicon_tags_0 },
    { 0xa97d648, SUC_lexicon_tags_0 },
    { 0x9ef7d649, SUC_lexicon_tags_11 },
    { 0xc957d649, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb7d64e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf897d651, SUC_lexicon_tags_158 },
    { 0xb657d652, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4977d65a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3997d65d, SUC_lexicon_tags_15 },
    { 0xec17d65e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837d663, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d57d667, SUC_lexicon_tags_46 },
    { 0xa3f7d667, SUC_lexicon_tags_17 },
    { 0x6917d669, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8017d66c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6857d671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5337d673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bf7d675, SUC_lexicon_tags_7 },
    { 0x9557d676, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf7d67e, SUC_lexicon_tags_27 },
    { 0xfd77d67f, SUC_lexicon_tags_45 },
    { 0x857d67f, SUC_lexicon_tags_34 },
    { 0xa4f7d67e, SUC_lexicon_tags_8 },
    { 0xde37d682, SUC_lexicon_tags_11 },
    { 0x5617d67e, SUC_lexicon_tags_11 },
    { 0x7997d684, SUC_lexicon_tags_4 },
    { 0xa437d680, SUC_lexicon_tags_113 },
    { 0xe257d680, SUC_lexicon_tags_1 },
    { 0x67d7d687, SUC_lexicon_tags_46 },
    { 0xd97d683, SUC_lexicon_tags_16 },
    { 0x3ad7d689, SUC_lexicon_tags_2 },
    { 0xc837d689, SUC_lexicon_tags_43 },
    { 0x3157d68b, SUC_lexicon_tags_37 },
    { 0x777d680, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62b7d68e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4877d690, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee97d694, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd7d69a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43f7d69c, SUC_lexicon_tags_82 },
    { 0x7557d69d, SUC_lexicon_tags_38 },
    { 0x9037d69e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc097d6a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf937d6a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed77d6a4, SUC_lexicon_tags_14 },
    { 0x9957d6a5, SUC_lexicon_tags_37 },
    { 0x9557d6a6, SUC_lexicon_tags_45 },
    { 0xe1f7d6a7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xe237d6a9, SUC_lexicon_tags_29 },
    { 0x8957d6aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b97d6ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f7d6b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xee57d6b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2757d6bc, SUC_lexicon_tags_14 },
    { 0x8d77d6bd, SUC_lexicon_tags_28 },
    { 0xc677d6be, SUC_lexicon_tags_60 },
    { 0x3b17d6bf, SUC_lexicon_tags_15 },
    { 0x7757d6bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2477d6c4, SUC_lexicon_tags_2 },
    { 0xac77d6c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d77d6c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb8d7d6ca, SUC_lexicon_tags_46 },
    { 0xf3b7d6cb, SUC_lexicon_tags_39 },
    { 0x1f57d6cc, SUC_lexicon_tags_33 },
    { 0x8bd7d6cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d37d6d1, SUC_lexicon_tags_27 },
    { 0xe9b7d6d2, SUC_lexicon_tags_82 },
    { 0x67d7d6d3, SUC_lexicon_tags_2 },
    { 0xdd7d6d4, SUC_lexicon_tags_37 },
    { 0xab17d6d4, SUC_lexicon_tags_7 },
    { 0x17d7d6d6, SUC_lexicon_tags_29 },
    { 0xd557d6d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf7d6e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf677d6e9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa917d6f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc237d6f3, SUC_lexicon_tags_27 },
    { 0x4177d6f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9b7d6f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb337d6f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1457d6fb, SUC_lexicon_tags_37 },
    { 0x22b7d6fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa97d6fe, SUC_lexicon_tags_61 },
    { 0x5877d6ff, SUC_lexicon_tags_23 },
    { 0x9d37d6ff, SUC_lexicon_tags_14 },
    { 0x6df7d701, SUC_lexicon_tags_25 },
    { 0x42f7d702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5637d70a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b97d70e, SUC_lexicon_tags_2 },
    { 0x8397d70f, SUC_lexicon_tags_29 },
    { 0xa897d710, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d57d715, SUC_lexicon_tags_60 },
    { 0xc1d7d716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b7d718, SUC_lexicon_tags_34 },
    { 0xbff7d719, SUC_lexicon_tags_43 },
    { 0x6497d718, SUC_lexicon_tags_26 },
    { 0x8e77d71b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8217d720, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb957d72c, SUC_lexicon_tags_43 },
    { 0x5997d72d, SUC_lexicon_tags_2 },
    { 0x8737d72e, SUC_lexicon_tags_11 },
    { 0xa8f7d72f, SUC_lexicon_tags_17 },
    { 0x1bd7d730, SUC_lexicon_tags_25 },
    { 0x22d7d72f, SUC_lexicon_tags_7 },
    { 0x7d97d732, SUC_lexicon_tags_17 },
    { 0x897d733, SUC_lexicon_tags_15 },
    { 0xc917d734, SUC_lexicon_tags_25 },
    { 0x70f7d733, SUC_lexicon_tags_1 },
    { 0xc097d735, SUC_lexicon_tags_8 },
    { 0x7af7d731, SUC_lexicon_tags_79 },
    { 0xc417d72d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad17d73a, SUC_lexicon_tags_30 },
    { 0x8877d73b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x617d73d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xef77d73f, SUC_lexicon_tags_0 },
    { 0xbd17d740, SUC_lexicon_tags_26 },
    { 0x2817d73f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d77d743, SUC_lexicon_tags_11 },
    { 0xc257d744, SUC_lexicon_tags_11 },
    { 0x85f7d745, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b7d74a, SUC_lexicon_tags_53 },
    { 0x637d74b, SUC_lexicon_tags_47 },
    { 0x7137d74a, SUC_lexicon_tags_15 },
    { 0x9737d74b, SUC_lexicon_tags_8 },
    { 0xf437d74e, SUC_lexicon_tags_26 },
    { 0xaa57d74e, SUC_lexicon_tags_11 },
    { 0xb277d750, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b57d753, SUC_lexicon_tags_55 },
    { 0xd017d754, SUC_lexicon_tags_43 },
    { 0xccb7d754, SUC_lexicon_tags_23 },
    { 0x9097d754, SUC_lexicon_tags_15 },
    { 0x6577d754, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c37d75d, SUC_lexicon_tags_31 },
    { 0xb2d7d75e, SUC_lexicon_tags_17 },
    { 0xbbb7d75f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb517d761, SUC_lexicon_tags_14 },
    { 0x5177d761, SUC_lexicon_tags_9 },
    { 0x3a17d763, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb977d765, SUC_lexicon_tags_30 },
    { 0x6d57d766, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e77d769, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f37d76b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d7d76e, SUC_lexicon_tags_1 },
    { 0xa17d76f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d7d772, SUC_lexicon_tags_12 },
    { 0x1a17d773, SUC_lexicon_tags_47 },
    { 0xcd37d774, SUC_lexicon_tags_43 },
    { 0x2677d775, SUC_lexicon_tags_15 },
    { 0xf617d772, SUC_lexicon_tags_1 },
    { 0x41b7d777, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b7d77a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f7d77d, SUC_lexicon_tags_27 },
    { 0xcdd7d77e, SUC_lexicon_tags_21 },
    { 0x4197d77f, SUC_lexicon_tags_17 },
    { 0x1b37d77d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5277d786, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637d78a, SUC_lexicon_tags_12 },
    { 0xc357d78b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5997d78e, SUC_lexicon_tags_15 },
    { 0x1cd7d78f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9d77d791, SUC_lexicon_tags_8 },
    { 0x4a17d792, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd817d795, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb7d79d, SUC_lexicon_tags_17 },
    { 0x5197d79e, SUC_lexicon_tags_12 },
    { 0x8837d79d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837d7a3, SUC_lexicon_tags_45 },
    { 0x3337d7a3, SUC_lexicon_tags_11 },
    { 0x6ff7d7a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82f7d7a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337d7ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb017d7b4, SUC_lexicon_tags_138 },
    { 0xdc97d7b5, SUC_lexicon_tags_15 },
    { 0x1ab7d7b5, SUC_lexicon_tags_25 },
    { 0x8d7d7b7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97d7bd, SUC_lexicon_tags_37 },
    { 0x30b7d7be, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f7d7c5, SUC_lexicon_tags_1 },
    { 0xbb37d7c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9937d7ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b7d7d1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b7d7d8, SUC_lexicon_tags_33 },
    { 0xb6b7d7d8, SUC_lexicon_tags_15 },
    { 0x21f7d7da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4497d7df, SUC_lexicon_tags_17 },
    { 0xd537d7df, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2337d7e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb7d7ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1817d7ef, SUC_lexicon_tags_29 },
    { 0x24d7d7f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4d7d7f2, SUC_lexicon_tags_28 },
    { 0xfef7d7f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77d7fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xef17d7fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af7d800, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaf7d802, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba17d806, SUC_lexicon_tags_3 },
    { 0x9ef7d807, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd517d809, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9af7d80b, SUC_lexicon_tags_95 },
    { 0xbb77d80c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe517d80e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1877d811, SUC_lexicon_tags_15 },
    { 0x9f57d812, SUC_lexicon_tags_1 },
    { 0x3677d813, SUC_lexicon_tags_11 },
    { 0x5917d814, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27b7d816, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77d81a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f7d81d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5ad7d81f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5db7d821, SUC_lexicon_tags_2 },
    { 0xfcf7d822, SUC_lexicon_tags_15 },
    { 0x2a57d822, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b7d826, SUC_lexicon_tags_16 },
    { 0xa617d827, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1577d82a, SUC_lexicon_tags_0 },
    { 0x7c37d82b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc17d82e, SUC_lexicon_tags_37 },
    { 0x7d17d82f, SUC_lexicon_tags_37 },
    { 0xac37d830, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f17d836, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8b7d838, SUC_lexicon_tags_25 },
    { 0x55d7d839, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63b7d83b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2617d83d, SUC_lexicon_tags_0 },
    { 0x777d83d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b57d841, SUC_lexicon_tags_21 },
    { 0x9d57d842, SUC_lexicon_tags_6 },
    { 0x2937d842, SUC_lexicon_tags_1 },
    { 0xccf7d841, SUC_lexicon_tags_23 },
    { 0xcf37d845, SUC_lexicon_tags_68 },
    { 0x3197d846, SUC_lexicon_tags_33 },
    { 0x9737d847, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f7d84c, SUC_lexicon_tags_25 },
    { 0xf177d84c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa5f7d84f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb537d853, SUC_lexicon_tags_3 },
    { 0x417d854, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf577d858, SUC_lexicon_tags_12 },
    { 0xc057d859, SUC_lexicon_tags_8 },
    { 0x5a97d85a, SUC_lexicon_tags_106 },
    { 0x16f7d85a, SUC_lexicon_tags_25 },
    { 0x97d85c, SUC_lexicon_tags_15 },
    { 0x4297d85d, SUC_lexicon_tags_47 },
    { 0x80d7d85e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd7d863, SUC_lexicon_tags_45 },
    { 0x9f97d863, SUC_lexicon_tags_33 },
    { 0x9b97d864, SUC_lexicon_tags_12 },
    { 0xc877d866, SUC_lexicon_tags_15 },
    { 0x397d867, SUC_lexicon_tags_127 },
    { 0x7dd7d868, SUC_lexicon_tags_7 },
    { 0x78d7d867, SUC_lexicon_tags_12 },
    { 0x4637d86a, SUC_lexicon_tags_17 },
    { 0x86f7d86a, SUC_lexicon_tags_11 },
    { 0xed37d86a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8ad7d86e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2597d870, SUC_lexicon_tags_35 },
    { 0x84f7d870, SUC_lexicon_tags_0 },
    { 0xb377d872, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f37d874, SUC_lexicon_tags_197 },
    { 0x6137d875, SUC_lexicon_tags_7 },
    { 0x97d7d876, SUC_lexicon_tags_153 },
    { 0, NULL },
    { 0x3c57d878, SUC_lexicon_tags_37 },
    { 0xe137d878, SUC_lexicon_tags_32 },
    { 0x7617d87a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a77d87e, SUC_lexicon_tags_27 },
    { 0x8797d87e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe657d883, SUC_lexicon_tags_3 },
    { 0x9117d883, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a97d887, SUC_lexicon_tags_0 },
    { 0x3417d888, SUC_lexicon_tags_27 },
    { 0x38f7d889, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3857d88c, SUC_lexicon_tags_7 },
    { 0x9a97d88d, SUC_lexicon_tags_7 },
    { 0x2297d88e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x497d891, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7717d896, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb277d89b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f7d89e, SUC_lexicon_tags_1 },
    { 0xac17d89e, SUC_lexicon_tags_33 },
    { 0xc957d8a0, SUC_lexicon_tags_17 },
    { 0xeb97d8a1, SUC_lexicon_tags_11 },
    { 0x6bf7d8a2, SUC_lexicon_tags_11 },
    { 0x8f7d8a2, SUC_lexicon_tags_27 },
    { 0xa3d7d8a4, SUC_lexicon_tags_11 },
    { 0xe317d8a2, SUC_lexicon_tags_6 },
    { 0x4297d8a0, SUC_lexicon_tags_25 },
    { 0x1837d8a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa117d8a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b7d8ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b7d8ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x777d8bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc97d8c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1757d8cb, SUC_lexicon_tags_1 },
    { 0xea77d8cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c77d8cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5657d8d5, SUC_lexicon_tags_28 },
    { 0x18f7d8d6, SUC_lexicon_tags_47 },
    { 0xdaf7d8d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc157d8d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee97d8dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a37d8e1, SUC_lexicon_tags_56 },
    { 0x11f7d8e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf737d8e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3df7d8e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6b7d8e9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b7d8ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d7d8f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd7d8f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f7d8f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe177d8fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb7d8fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb1f7d8ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d7d903, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17d906, SUC_lexicon_tags_45 },
    { 0xe057d907, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f7d90a, SUC_lexicon_tags_37 },
    { 0xa357d90b, SUC_lexicon_tags_48 },
    { 0xaa17d90b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf7d90e, SUC_lexicon_tags_14 },
    { 0x6b7d90f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7197d911, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d7d917, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf057d91d, SUC_lexicon_tags_43 },
    { 0x38f7d91d, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5077d925, SUC_lexicon_tags_23 },
    { 0x4817d925, SUC_lexicon_tags_7 },
    { 0x82b7d927, SUC_lexicon_tags_25 },
    { 0x7b57d928, SUC_lexicon_tags_2 },
    { 0xeaf7d929, SUC_lexicon_tags_2 },
    { 0xd7f7d92a, SUC_lexicon_tags_20 },
    { 0xc217d926, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84b7d92d, SUC_lexicon_tags_11 },
    { 0xc177d92d, SUC_lexicon_tags_33 },
    { 0x3897d92f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4997d933, SUC_lexicon_tags_31 },
    { 0x70f7d933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad7d937, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc57d93a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7297d93e, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f7d943, SUC_lexicon_tags_25 },
    { 0x9c17d944, SUC_lexicon_tags_6 },
    { 0x2df7d943, SUC_lexicon_tags_2 },
    { 0xc517d946, SUC_lexicon_tags_45 },
    { 0xc4f7d944, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5797d94a, SUC_lexicon_tags_1 },
    { 0x2457d94a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a77d951, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2497d957, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1297d959, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe37d95f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde37d961, SUC_lexicon_tags_59 },
    { 0xc57d962, SUC_lexicon_tags_14 },
    { 0x4d37d963, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d17d966, SUC_lexicon_tags_12 },
    { 0x71b7d967, SUC_lexicon_tags_23 },
    { 0x377d966, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf9f7d96a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef7d96d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe697d970, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d7d973, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3757d975, SUC_lexicon_tags_7 },
    { 0x297d976, SUC_lexicon_tags_11 },
    { 0xa417d977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7037d97a, SUC_lexicon_tags_14 },
    { 0x12f7d97b, SUC_lexicon_tags_60 },
    { 0xa537d97c, SUC_lexicon_tags_11 },
    { 0x29f7d97b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f17d982, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x59f7d984, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ff7d986, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b7d989, SUC_lexicon_tags_27 },
    { 0x6f7d989, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8817d98d, SUC_lexicon_tags_2 },
    { 0xb917d98d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d7d993, SUC_lexicon_tags_55 },
    { 0x1057d994, SUC_lexicon_tags_1 },
    { 0x917d994, SUC_lexicon_tags_23 },
    { 0xb757d996, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6577d99a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd7d9a2, SUC_lexicon_tags_25 },
    { 0xd3d7d9a3, SUC_lexicon_tags_38 },
    { 0x3517d9a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe197d9a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe297d9ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd957d9b0, SUC_lexicon_tags_28 },
    { 0x5657d9b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a77d9b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd837d9ba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe117d9bf, SUC_lexicon_tags_11 },
    { 0xc377d9c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b7d9c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a97d9d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f7d9d3, SUC_lexicon_tags_32 },
    { 0x3bf7d9d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7397d9d6, SUC_lexicon_tags_11 },
    { 0xa097d9d7, SUC_lexicon_tags_8 },
    { 0xce17d9d6, SUC_lexicon_tags_37 },
    { 0x9b17d9d9, SUC_lexicon_tags_8 },
    { 0xbb57d9da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d97d9de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd7d9e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x85b7d9e3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c37d9e9, SUC_lexicon_tags_7 },
    { 0x7a77d9ea, SUC_lexicon_tags_15 },
    { 0x5437d9eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6e17d9ed, SUC_lexicon_tags_48 },
    { 0x7bb7d9ee, SUC_lexicon_tags_26 },
    { 0x3057d9ed, SUC_lexicon_tags_2 },
    { 0xe537d9f0, SUC_lexicon_tags_12 },
    { 0x7537d9f0, SUC_lexicon_tags_43 },
    { 0x1837d9f2, SUC_lexicon_tags_39 },
    { 0x3b77d9f3, SUC_lexicon_tags_29 },
    { 0xfd77d9f2, SUC_lexicon_tags_37 },
    { 0xb537d9f5, SUC_lexicon_tags_25 },
    { 0xe1f7d9f4, SUC_lexicon_tags_1 },
    { 0x3037d9f2, SUC_lexicon_tags_15 },
    { 0xa157d9f2, SUC_lexicon_tags_82 },
    { 0x4037d9f0, SUC_lexicon_tags_10 },
    { 0x4057d9fa, SUC_lexicon_tags_47 },
    { 0x8917d9fb, SUC_lexicon_tags_61 },
    { 0xaab7d9fa, SUC_lexicon_tags_11 },
    { 0x28f7d9f9, SUC_lexicon_tags_22 },
    { 0x6bf7d9f6, SUC_lexicon_tags_4 },
    { 0xcd7d9ff, SUC_lexicon_tags_9 },
    { 0x42b7da00, SUC_lexicon_tags_11 },
    { 0xb417da01, SUC_lexicon_tags_17 },
    { 0x80d7da00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab7da06, SUC_lexicon_tags_55 },
    { 0x3db7da06, SUC_lexicon_tags_2 },
    { 0x9eb7da07, SUC_lexicon_tags_17 },
    { 0x1c17da09, SUC_lexicon_tags_2 },
    { 0x8137da0a, SUC_lexicon_tags_37 },
    { 0x6f77da08, SUC_lexicon_tags_27 },
    { 0x37da0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf17da0e, SUC_lexicon_tags_19 },
    { 0x2017da0f, SUC_lexicon_tags_37 },
    { 0xec57da0e, SUC_lexicon_tags_37 },
    { 0x49d7da11, SUC_lexicon_tags_1 },
    { 0xe397da12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb677da16, SUC_lexicon_tags_15 },
    { 0x32f7da17, SUC_lexicon_tags_37 },
    { 0xa1d7da18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea57da1e, SUC_lexicon_tags_25 },
    { 0x69b7da1e, SUC_lexicon_tags_195 },
    { 0x1a77da20, SUC_lexicon_tags_1 },
    { 0x8377da1e, SUC_lexicon_tags_37 },
    { 0x3c97da1f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6277da24, SUC_lexicon_tags_11 },
    { 0xbcb7da25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b37da27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf7da2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b7da2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8fd7da30, SUC_lexicon_tags_45 },
    { 0x7a17da30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4317da34, SUC_lexicon_tags_25 },
    { 0x32f7da35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad7da37, SUC_lexicon_tags_17 },
    { 0x7df7da38, SUC_lexicon_tags_43 },
    { 0x1db7da38, SUC_lexicon_tags_11 },
    { 0x8837da3a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7897da3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe797da43, SUC_lexicon_tags_34 },
    { 0xfed7da44, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x15d7da46, SUC_lexicon_tags_37 },
    { 0x2d17da47, SUC_lexicon_tags_12 },
    { 0xe497da46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb7da4d, SUC_lexicon_tags_36 },
    { 0x2a37da4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa017da50, SUC_lexicon_tags_38 },
    { 0xd077da50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d7da54, SUC_lexicon_tags_2 },
    { 0x7917da54, SUC_lexicon_tags_14 },
    { 0x47f7da56, SUC_lexicon_tags_1 },
    { 0x1637da57, SUC_lexicon_tags_8 },
    { 0xc4f7da56, SUC_lexicon_tags_14 },
    { 0x7277da59, SUC_lexicon_tags_1 },
    { 0x9797da5a, SUC_lexicon_tags_13 },
    { 0xa157da5a, SUC_lexicon_tags_69 },
    { 0xca57da5c, SUC_lexicon_tags_23 },
    { 0x6357da5a, SUC_lexicon_tags_27 },
    { 0x9177da5e, SUC_lexicon_tags_60 },
    { 0x4d57da5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6317da61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb397da63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61b7da65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf37da69, SUC_lexicon_tags_15 },
    { 0x757da6a, SUC_lexicon_tags_15 },
    { 0x9797da69, SUC_lexicon_tags_10 },
    { 0xa0b7da6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6317da70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64b7da72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf757da75, SUC_lexicon_tags_7 },
    { 0x24b7da75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7da7a, SUC_lexicon_tags_7 },
    { 0xdab7da7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f7da7d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe17da80, SUC_lexicon_tags_25 },
    { 0x2d7da80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8db7da83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81b7da85, SUC_lexicon_tags_26 },
    { 0x9277da85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf617da88, SUC_lexicon_tags_47 },
    { 0x1f7da88, SUC_lexicon_tags_12 },
    { 0xc0f7da89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef57da8e, SUC_lexicon_tags_25 },
    { 0x4b7da8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9397da93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc477da95, SUC_lexicon_tags_45 },
    { 0xc137da96, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb417da9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x47f7da9e, SUC_lexicon_tags_47 },
    { 0x4d97da9f, SUC_lexicon_tags_63 },
    { 0x3af7daa0, SUC_lexicon_tags_25 },
    { 0x9ef7daa1, SUC_lexicon_tags_12 },
    { 0x6c77daa2, SUC_lexicon_tags_17 },
    { 0xa8f7daa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17daa6, SUC_lexicon_tags_158 },
    { 0x5df7daa7, SUC_lexicon_tags_25 },
    { 0x8777daa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f7daac, SUC_lexicon_tags_47 },
    { 0x74f7daad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4077daaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeed7dab1, SUC_lexicon_tags_12 },
    { 0xc8f7dab1, SUC_lexicon_tags_14 },
    { 0x3b97dab3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41b7dab5, SUC_lexicon_tags_39 },
    { 0xd937dab5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c57daba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3377dabd, SUC_lexicon_tags_87 },
    { 0xeb7dabe, SUC_lexicon_tags_47 },
    { 0x1c57dabf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb37dac1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2657dac6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa17dac9, SUC_lexicon_tags_28 },
    { 0xe2d7dac9, SUC_lexicon_tags_46 },
    { 0xb297dac9, SUC_lexicon_tags_1 },
    { 0x1997daca, SUC_lexicon_tags_55 },
    { 0xe1b7dacd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc17dad0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6777dad2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xda97dad5, SUC_lexicon_tags_53 },
    { 0x3237dad6, SUC_lexicon_tags_29 },
    { 0xb077dad5, SUC_lexicon_tags_64 },
    { 0x357dad7, SUC_lexicon_tags_33 },
    { 0xc497dad9, SUC_lexicon_tags_46 },
    { 0x2f7dada, SUC_lexicon_tags_82 },
    { 0x75f7dada, SUC_lexicon_tags_111 },
    { 0x7317dadc, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd7dae3, SUC_lexicon_tags_1 },
    { 0xbc97dae3, SUC_lexicon_tags_56 },
    { 0xb337dae4, SUC_lexicon_tags_63 },
    { 0xedb7dae6, SUC_lexicon_tags_11 },
    { 0x5397dae4, SUC_lexicon_tags_14 },
    { 0x8137dae8, SUC_lexicon_tags_2 },
    { 0x817dae6, SUC_lexicon_tags_40 },
    { 0x6297daea, SUC_lexicon_tags_2 },
    { 0xbf57dae3, SUC_lexicon_tags_32 },
    { 0x66b7dae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fb7daee, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5497daf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e97daf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f7dafc, SUC_lexicon_tags_32 },
    { 0x4637dafc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b57db04, SUC_lexicon_tags_162 },
    { 0xc357db05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8037db0d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf897db0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b37db11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6d7db13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e77db16, SUC_lexicon_tags_6 },
    { 0x4577db17, SUC_lexicon_tags_30 },
    { 0x92b7db18, SUC_lexicon_tags_25 },
    { 0x3a97db18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb37db1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d7db20, SUC_lexicon_tags_27 },
    { 0xd2f7db21, SUC_lexicon_tags_17 },
    { 0x8837db22, SUC_lexicon_tags_54 },
    { 0xd857db23, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df7db2b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf1b7db2d, SUC_lexicon_tags_11 },
    { 0xcf97db2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6517db30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37db36, SUC_lexicon_tags_1 },
    { 0x9457db37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5df7db39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdaf7db3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc6f7db3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7277db3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf157db43, SUC_lexicon_tags_47 },
    { 0x8737db43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db7db4c, SUC_lexicon_tags_172 },
    { 0xc777db4d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd357db51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d97db55, SUC_lexicon_tags_47 },
    { 0xe5b7db56, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x5917db58, SUC_lexicon_tags_33 },
    { 0x6fb7db59, SUC_lexicon_tags_15 },
    { 0xecd7db59, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8457db60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1637db63, SUC_lexicon_tags_25 },
    { 0x33b7db64, SUC_lexicon_tags_31 },
    { 0x9a57db64, SUC_lexicon_tags_1 },
    { 0xf6b7db66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c97db6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x317db70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7957db72, SUC_lexicon_tags_25 },
    { 0xa657db72, SUC_lexicon_tags_45 },
    { 0x72b7db74, SUC_lexicon_tags_45 },
    { 0xf537db72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5557db79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b7db7c, SUC_lexicon_tags_17 },
    { 0x59b7db7d, SUC_lexicon_tags_11 },
    { 0xb17db7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff37db80, SUC_lexicon_tags_17 },
    { 0x29f7db80, SUC_lexicon_tags_7 },
    { 0x9a37db82, SUC_lexicon_tags_46 },
    { 0xe4b7db83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8137db87, SUC_lexicon_tags_13 },
    { 0x4297db87, SUC_lexicon_tags_21 },
    { 0x32f7db88, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc37db8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce17db8f, SUC_lexicon_tags_59 },
    { 0xdd57db90, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397db94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9457db9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19f7db9f, SUC_lexicon_tags_46 },
    { 0xf317dba0, SUC_lexicon_tags_69 },
    { 0xe0b7db9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd57dba4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf57dba7, SUC_lexicon_tags_87 },
    { 0xc1f7dba8, SUC_lexicon_tags_17 },
    { 0xe5d7dba9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb7dbae, SUC_lexicon_tags_1 },
    { 0x78d7dbae, SUC_lexicon_tags_45 },
    { 0xf6d7dbb0, SUC_lexicon_tags_37 },
    { 0xbd57dbb1, SUC_lexicon_tags_7 },
    { 0x1bf7dbb0, SUC_lexicon_tags_19 },
    { 0x7d7dbb1, SUC_lexicon_tags_69 },
    { 0xf3b7dbb1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f57dbb8, SUC_lexicon_tags_37 },
    { 0xd917dbb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f97dbbd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd297dbc3, SUC_lexicon_tags_32 },
    { 0xfad7dbc4, SUC_lexicon_tags_37 },
    { 0xe9f7dbc3, SUC_lexicon_tags_37 },
    { 0xa7f7dbc6, SUC_lexicon_tags_37 },
    { 0x6797dbc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3937dbc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b57dbd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7917dbd6, SUC_lexicon_tags_33 },
    { 0x3957dbd7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x497dbd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9357dbdc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd497dbdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa217dbe1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe817dbe4, SUC_lexicon_tags_25 },
    { 0xd977dbe5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc97dbe9, SUC_lexicon_tags_13 },
    { 0x8797dbea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d7dbef, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab7dbf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b7dbfc, SUC_lexicon_tags_2 },
    { 0x8997dbfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1837dbff, SUC_lexicon_tags_17 },
    { 0xe937dc00, SUC_lexicon_tags_26 },
    { 0x3bf7dc01, SUC_lexicon_tags_37 },
    { 0xe5d7dc02, SUC_lexicon_tags_14 },
    { 0x2ed7dc00, SUC_lexicon_tags_46 },
    { 0x1c17dc02, SUC_lexicon_tags_7 },
    { 0x4097dc05, SUC_lexicon_tags_33 },
    { 0x8377dc06, SUC_lexicon_tags_15 },
    { 0x8f7dc03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d7dc0a, SUC_lexicon_tags_13 },
    { 0x5f37dc0b, SUC_lexicon_tags_1 },
    { 0x6317dc0c, SUC_lexicon_tags_28 },
    { 0x7357dc0b, SUC_lexicon_tags_6 },
    { 0x1ad7dc0e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d7dc16, SUC_lexicon_tags_11 },
    { 0x52d7dc17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f7dc1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9257dc1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe957dc21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d7dc24, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d57dc27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57dc2e, SUC_lexicon_tags_2 },
    { 0xc57dc2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8037dc39, SUC_lexicon_tags_11 },
    { 0x35b7dc39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9d7dc3c, SUC_lexicon_tags_7 },
    { 0xe5f7dc3d, SUC_lexicon_tags_38 },
    { 0x1f7dc3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d37dc41, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd657dc46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f7dc53, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd5b7dc55, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xf717dc58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b97dc5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2977dc60, SUC_lexicon_tags_13 },
    { 0x81d7dc61, SUC_lexicon_tags_2 },
    { 0x70f7dc61, SUC_lexicon_tags_1 },
    { 0x6817dc62, SUC_lexicon_tags_17 },
    { 0x6cf7dc61, SUC_lexicon_tags_10 },
    { 0x4657dc65, SUC_lexicon_tags_43 },
    { 0x6617dc66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b7dc69, SUC_lexicon_tags_37 },
    { 0x70f7dc6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b7dc72, SUC_lexicon_tags_37 },
    { 0xfdf7dc72, SUC_lexicon_tags_14 },
    { 0x7e37dc74, SUC_lexicon_tags_13 },
    { 0x7657dc75, SUC_lexicon_tags_114 },
    { 0xd117dc75, SUC_lexicon_tags_47 },
    { 0x16b7dc74, SUC_lexicon_tags_25 },
    { 0x4b7dc78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f37dc7a, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f7dc7d, SUC_lexicon_tags_12 },
    { 0x89b7dc7e, SUC_lexicon_tags_316 },
    { 0xd1d7dc7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5697dc81, SUC_lexicon_tags_84 },
    { 0x7df7dc82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbf7dc84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd37dc87, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbcb7dc89, SUC_lexicon_tags_35 },
    { 0x34f7dc8a, SUC_lexicon_tags_38 },
    { 0xd6b7dc8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf657dc8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbad7dc8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d7dc92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf7dc95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea97dc98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab7dc9e, SUC_lexicon_tags_14 },
    { 0x47b7dc9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde37dca3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x24b7dca5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d57dca7, SUC_lexicon_tags_37 },
    { 0x2b7dca8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4337dcac, SUC_lexicon_tags_25 },
    { 0xfdd7dcac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb297dcb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd617dcb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf437dcba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8977dcbf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557dcc5, SUC_lexicon_tags_11 },
    { 0xfe17dcc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2277dcc8, SUC_lexicon_tags_38 },
    { 0xda97dcc9, SUC_lexicon_tags_11 },
    { 0x6b57dcca, SUC_lexicon_tags_15 },
    { 0x30d7dccb, SUC_lexicon_tags_25 },
    { 0x5cf7dcca, SUC_lexicon_tags_37 },
    { 0x88f7dcca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb97dcd4, SUC_lexicon_tags_14 },
    { 0xb637dcd5, SUC_lexicon_tags_11 },
    { 0xfe77dcd5, SUC_lexicon_tags_15 },
    { 0xf9b7dcd7, SUC_lexicon_tags_2 },
    { 0xf957dcd6, SUC_lexicon_tags_17 },
    { 0x4277dcd7, SUC_lexicon_tags_252 },
    { 0x80d7dcda, SUC_lexicon_tags_1 },
    { 0x42b7dcda, SUC_lexicon_tags_37 },
    { 0x7437dcdc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7dce1, SUC_lexicon_tags_2 },
    { 0x28d7dce2, SUC_lexicon_tags_23 },
    { 0x4657dce3, SUC_lexicon_tags_5 },
    { 0x72b7dce3, SUC_lexicon_tags_92 },
    { 0x3f97dce5, SUC_lexicon_tags_21 },
    { 0xd7f7dce2, SUC_lexicon_tags_17 },
    { 0x5c57dce1, SUC_lexicon_tags_15 },
    { 0x9397dce8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e57dced, SUC_lexicon_tags_11 },
    { 0x5197dcee, SUC_lexicon_tags_23 },
    { 0xa3b7dcee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6497dcf1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3497dcf7, SUC_lexicon_tags_18 },
    { 0x2cf7dcf8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2ff7dcfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3377dcfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5457dcfe, SUC_lexicon_tags_37 },
    { 0x6bd7dcfe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef77dd04, SUC_lexicon_tags_7 },
    { 0x79b7dd05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6677dd0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d7dd13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7dd1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb7dd1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef7dd22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc117dd24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad7dd27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1277dd29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x237dd2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf677dd2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b7dd33, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb257dd38, SUC_lexicon_tags_11 },
    { 0xd097dd38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b37dd3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a17dd3d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x24b7dd3f, SUC_lexicon_tags_33 },
    { 0x2f77dd3f, SUC_lexicon_tags_14 },
    { 0xa037dd41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5197dd47, SUC_lexicon_tags_1 },
    { 0xb7b7dd48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x517dd4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8cf7dd4d, SUC_lexicon_tags_17 },
    { 0x1637dd4e, SUC_lexicon_tags_11 },
    { 0x1e97dd4e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d7dd54, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d7dd57, SUC_lexicon_tags_33 },
    { 0x757dd58, SUC_lexicon_tags_26 },
    { 0xf8d7dd59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b17dd63, SUC_lexicon_tags_17 },
    { 0x4bf7dd64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc37dd66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bb7dd68, SUC_lexicon_tags_0 },
    { 0x2937dd69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa797dd6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1617dd6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb37dd71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb057dd75, SUC_lexicon_tags_48 },
    { 0x8297dd75, SUC_lexicon_tags_14 },
    { 0x6b37dd77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f97dd7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6557dd7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9877dd83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb7dd86, SUC_lexicon_tags_17 },
    { 0x65f7dd87, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d7dd8f, SUC_lexicon_tags_63 },
    { 0x9317dd90, SUC_lexicon_tags_7 },
    { 0xe7f7dd91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xac97dd94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a97dd98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xde97dd9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee77dd9f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd237dda1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4677dda3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57dda6, SUC_lexicon_tags_26 },
    { 0xce77dda6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b57dda9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb537ddad, SUC_lexicon_tags_65 },
    { 0x4cf7ddae, SUC_lexicon_tags_37 },
    { 0x1cd7ddaf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9bb7ddb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb177ddb4, SUC_lexicon_tags_1 },
    { 0x5197ddb5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177ddbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad7ddbf, SUC_lexicon_tags_37 },
    { 0x9a97ddbf, SUC_lexicon_tags_0 },
    { 0x72f7ddc1, SUC_lexicon_tags_8 },
    { 0x83b7ddc2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf177ddc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa857ddc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb7ddce, SUC_lexicon_tags_6 },
    { 0x3b77ddce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef17ddd5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff7ddda, SUC_lexicon_tags_45 },
    { 0x3977dddb, SUC_lexicon_tags_25 },
    { 0xc937dddc, SUC_lexicon_tags_25 },
    { 0xc0d7dddd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f7dde1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c97ddea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa777ddec, SUC_lexicon_tags_66 },
    { 0x6d37dded, SUC_lexicon_tags_12 },
    { 0x9957ddee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa117ddf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98d7ddf2, SUC_lexicon_tags_12 },
    { 0xf517ddf2, SUC_lexicon_tags_5 },
    { 0x4eb7ddf4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1bd7ddf6, SUC_lexicon_tags_0 },
    { 0x1c57ddf6, SUC_lexicon_tags_15 },
    { 0xd297ddf7, SUC_lexicon_tags_28 },
    { 0xfe77ddf9, SUC_lexicon_tags_37 },
    { 0xd797ddfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d7ddfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d7de00, SUC_lexicon_tags_7 },
    { 0x5ab7de01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc2b7de03, SUC_lexicon_tags_11 },
    { 0x85f7de04, SUC_lexicon_tags_37 },
    { 0x66b7de04, SUC_lexicon_tags_19 },
    { 0xa357de05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b7de0b, SUC_lexicon_tags_66 },
    { 0xbc77de0c, SUC_lexicon_tags_32 },
    { 0x7617de0c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb7de10, SUC_lexicon_tags_37 },
    { 0xe837de10, SUC_lexicon_tags_59 },
    { 0xd237de12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa97de14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f7de19, SUC_lexicon_tags_60 },
    { 0xde37de1a, SUC_lexicon_tags_7 },
    { 0xdbf7de19, SUC_lexicon_tags_47 },
    { 0xcd77de1c, SUC_lexicon_tags_33 },
    { 0x79d7de1a, SUC_lexicon_tags_2 },
    { 0x8417de1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32b7de20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f7de24, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x83f7de26, SUC_lexicon_tags_60 },
    { 0xf7d7de27, SUC_lexicon_tags_65 },
    { 0x3557de26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4f7de2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1877de30, SUC_lexicon_tags_33 },
    { 0xf837de30, SUC_lexicon_tags_32 },
    { 0xeff7de32, SUC_lexicon_tags_38 },
    { 0xe397de33, SUC_lexicon_tags_16 },
    { 0x5d7de30, SUC_lexicon_tags_7 },
    { 0x2bb7de32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa477de37, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f7de3c, SUC_lexicon_tags_2 },
    { 0xbf77de3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a97de41, SUC_lexicon_tags_14 },
    { 0xedd7de42, SUC_lexicon_tags_23 },
    { 0x1257de42, SUC_lexicon_tags_28 },
    { 0x1697de42, SUC_lexicon_tags_25 },
    { 0xcd97de41, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0x99b7de47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe957de4a, SUC_lexicon_tags_45 },
    { 0x5617de4b, SUC_lexicon_tags_1 },
    { 0xeb57de4b, SUC_lexicon_tags_1 },
    { 0x4417de4d, SUC_lexicon_tags_0 },
    { 0xed57de4e, SUC_lexicon_tags_8 },
    { 0x53b7de4d, SUC_lexicon_tags_11 },
    { 0x1f7de4c, SUC_lexicon_tags_31 },
    { 0xe957de51, SUC_lexicon_tags_77 },
    { 0x5297de51, SUC_lexicon_tags_12 },
    { 0x9dd7de51, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd7de66, SUC_lexicon_tags_27 },
    { 0xc497de67, SUC_lexicon_tags_13 },
    { 0x4d97de67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6d7de6a, SUC_lexicon_tags_7 },
    { 0x3877de6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d7de6e, SUC_lexicon_tags_15 },
    { 0x3ff7de6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87b7de71, SUC_lexicon_tags_47 },
    { 0xcb57de71, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x697de75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3717de77, SUC_lexicon_tags_45 },
    { 0x6017de77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b7de7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cb7de7d, SUC_lexicon_tags_23 },
    { 0xc937de7e, SUC_lexicon_tags_47 },
    { 0xc6b7de7f, SUC_lexicon_tags_20 },
    { 0x117de7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80b7de82, SUC_lexicon_tags_9 },
    { 0xb8d7de83, SUC_lexicon_tags_48 },
    { 0x4777de83, SUC_lexicon_tags_17 },
    { 0x7ed7de83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d77de8a, SUC_lexicon_tags_66 },
    { 0xb4d7de8b, SUC_lexicon_tags_15 },
    { 0x3c57de8b, SUC_lexicon_tags_27 },
    { 0xb8d7de8b, SUC_lexicon_tags_9 },
    { 0x69d7de8e, SUC_lexicon_tags_12 },
    { 0xfed7de8f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x3b17de91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6737de94, SUC_lexicon_tags_47 },
    { 0xa5b7de95, SUC_lexicon_tags_23 },
    { 0x357de96, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b7de9a, SUC_lexicon_tags_25 },
    { 0x797de9b, SUC_lexicon_tags_14 },
    { 0x3d97de9b, SUC_lexicon_tags_12 },
    { 0x3157de9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6797dea0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12b7dea2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc537dea6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8397dea8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5f57deaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c77deaf, SUC_lexicon_tags_14 },
    { 0x8417deb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2277deb2, SUC_lexicon_tags_12 },
    { 0x2c97deb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf37deb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96b7deb9, SUC_lexicon_tags_11 },
    { 0xca17deb9, SUC_lexicon_tags_12 },
    { 0xa017deb9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf0f7debd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2497dec1, SUC_lexicon_tags_32 },
    { 0x6037dec2, SUC_lexicon_tags_63 },
    { 0x3d77dec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d17dec5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1017dec8, SUC_lexicon_tags_11 },
    { 0xc9d7dec8, SUC_lexicon_tags_37 },
    { 0x3797dec9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb477decd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff7ded2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8297ded6, SUC_lexicon_tags_33 },
    { 0xd957ded7, SUC_lexicon_tags_37 },
    { 0x64b7ded7, SUC_lexicon_tags_14 },
    { 0xde17ded8, SUC_lexicon_tags_187 },
    { 0x1c37deda, SUC_lexicon_tags_25 },
    { 0x2677deda, SUC_lexicon_tags_15 },
    { 0xf77dedc, SUC_lexicon_tags_84 },
    { 0x4d57dedd, SUC_lexicon_tags_31 },
    { 0xbe97dedc, SUC_lexicon_tags_14 },
    { 0x9177dedf, SUC_lexicon_tags_25 },
    { 0x6077dedd, SUC_lexicon_tags_19 },
    { 0xf9d7dee1, SUC_lexicon_tags_11 },
    { 0xb9f7dee2, SUC_lexicon_tags_11 },
    { 0x81b7dedf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaad7dee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb77deea, SUC_lexicon_tags_46 },
    { 0x38b7deeb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x57deed, SUC_lexicon_tags_17 },
    { 0x3b7deee, SUC_lexicon_tags_37 },
    { 0x7037deee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe397def2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62b7def4, SUC_lexicon_tags_38 },
    { 0xdc77def5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3337def8, SUC_lexicon_tags_25 },
    { 0xcf7def9, SUC_lexicon_tags_17 },
    { 0x5c17def9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e97deff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b7df05, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f97df0f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7917df11, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb777df15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf477df18, SUC_lexicon_tags_30 },
    { 0xeb77df18, SUC_lexicon_tags_17 },
    { 0xe5f7df1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab7df1d, SUC_lexicon_tags_61 },
    { 0xaf77df1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b7df20, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b7df24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x417df26, SUC_lexicon_tags_0 },
    { 0x8697df26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b7df2d, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b7df32, SUC_lexicon_tags_87 },
    { 0xb77df33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb97df39, SUC_lexicon_tags_37 },
    { 0xc557df3a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1857df3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef7df41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf457df47, SUC_lexicon_tags_45 },
    { 0xebb7df48, SUC_lexicon_tags_2 },
    { 0xcfd7df47, SUC_lexicon_tags_11 },
    { 0x28b7df4a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa697df4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd7df52, SUC_lexicon_tags_37 },
    { 0x7c57df53, SUC_lexicon_tags_8 },
    { 0xf957df54, SUC_lexicon_tags_17 },
    { 0x7a37df55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d7df5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe797df62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4797df64, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5017df66, SUC_lexicon_tags_12 },
    { 0x8297df66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d7df6a, SUC_lexicon_tags_37 },
    { 0xb517df6b, SUC_lexicon_tags_25 },
    { 0xa3f7df6c, SUC_lexicon_tags_11 },
    { 0x90d7df6d, SUC_lexicon_tags_37 },
    { 0xbc57df6e, SUC_lexicon_tags_15 },
    { 0xd537df6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa197df7b, SUC_lexicon_tags_7 },
    { 0x7d37df7c, SUC_lexicon_tags_1 },
    { 0x2157df7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75d7df7f, SUC_lexicon_tags_26 },
    { 0x8517df80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b37df87, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb7df8b, SUC_lexicon_tags_33 },
    { 0x1697df8c, SUC_lexicon_tags_60 },
    { 0xc2d7df8d, SUC_lexicon_tags_33 },
    { 0x5b7df8c, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977df92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f7df97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4677df9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6297dfa6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7117dfa9, SUC_lexicon_tags_25 },
    { 0x3157dfaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4ef7dfac, SUC_lexicon_tags_18 },
    { 0x74d7dfac, SUC_lexicon_tags_29 },
    { 0xcef7dfac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7bf7dfb0, SUC_lexicon_tags_0 },
    { 0x8df7dfb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d7dfb4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4957dfb7, SUC_lexicon_tags_17 },
    { 0xa8b7dfb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb7dfbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d77dfbe, SUC_lexicon_tags_14 },
    { 0x6237dfbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7a77dfc1, SUC_lexicon_tags_15 },
    { 0x8d77dfc1, SUC_lexicon_tags_16 },
    { 0xf5f7dfc1, SUC_lexicon_tags_0 },
    { 0xcff7dfc3, SUC_lexicon_tags_11 },
    { 0xeb17dfc5, SUC_lexicon_tags_70 },
    { 0x4657dfc6, SUC_lexicon_tags_37 },
    { 0x117dfc3, SUC_lexicon_tags_506 },
    { 0x3bd7dfc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd97dfca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b7dfd0, SUC_lexicon_tags_37 },
    { 0x8557dfd0, SUC_lexicon_tags_2 },
    { 0xe3f7dfd2, SUC_lexicon_tags_230 },
    { 0x5177dfd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc437dfde, SUC_lexicon_tags_120 },
    { 0x17dfdf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7dfe7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc197dfea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfc97dfec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd937dfee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9277dff1, SUC_lexicon_tags_1 },
    { 0xc6b7dff2, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x5617dff4, SUC_lexicon_tags_0 },
    { 0xf397dff5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe297dff7, SUC_lexicon_tags_75 },
    { 0x5557dff8, SUC_lexicon_tags_17 },
    { 0x67b7dff9, SUC_lexicon_tags_61 },
    { 0x617dff9, SUC_lexicon_tags_46 },
    { 0x90d7dff8, SUC_lexicon_tags_11 },
    { 0x5697dff8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4577dffe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9057e002, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d7e005, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf417e007, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc337e00a, SUC_lexicon_tags_15 },
    { 0xfd57e00b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f7e012, SUC_lexicon_tags_91 },
    { 0x50d7e013, SUC_lexicon_tags_39 },
    { 0x9d97e013, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77e01b, SUC_lexicon_tags_19 },
    { 0x43d7e01c, SUC_lexicon_tags_11 },
    { 0x4417e01b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa857e01f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9ef7e021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec97e023, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc297e029, SUC_lexicon_tags_7 },
    { 0xff57e02a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f7e030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7477e032, SUC_lexicon_tags_43 },
    { 0xe9b7e033, SUC_lexicon_tags_37 },
    { 0xbd37e034, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd057e038, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b77e040, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9577e042, SUC_lexicon_tags_17 },
    { 0xffd7e043, SUC_lexicon_tags_46 },
    { 0xe137e043, SUC_lexicon_tags_2 },
    { 0x6457e045, SUC_lexicon_tags_12 },
    { 0xfd17e046, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d7e04a, SUC_lexicon_tags_27 },
    { 0xe217e04a, SUC_lexicon_tags_45 },
    { 0x6097e04b, SUC_lexicon_tags_15 },
    { 0x32f7e04a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef7e051, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe457e053, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab7e057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ad7e059, SUC_lexicon_tags_0 },
    { 0x8b97e05a, SUC_lexicon_tags_17 },
    { 0x1cd7e059, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b37e062, SUC_lexicon_tags_48 },
    { 0x84f7e063, SUC_lexicon_tags_140 },
    { 0xd317e063, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf7e069, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x56f7e06b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x157e06d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d7e070, SUC_lexicon_tags_44 },
    { 0x4e77e070, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd37e073, SUC_lexicon_tags_32 },
    { 0xa1f7e074, SUC_lexicon_tags_151 },
    { 0x3657e075, SUC_lexicon_tags_23 },
    { 0x2477e076, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea57e078, SUC_lexicon_tags_63 },
    { 0x9a37e079, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2837e07c, SUC_lexicon_tags_29 },
    { 0x8677e07d, SUC_lexicon_tags_9 },
    { 0xc997e07d, SUC_lexicon_tags_7 },
    { 0x68f7e07f, SUC_lexicon_tags_7 },
    { 0x217e07c, SUC_lexicon_tags_25 },
    { 0x6897e081, SUC_lexicon_tags_82 },
    { 0x5ff7e082, SUC_lexicon_tags_57 },
    { 0xddb7e083, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5357e087, SUC_lexicon_tags_15 },
    { 0xa377e088, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f7e095, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f7e098, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77e09a, SUC_lexicon_tags_23 },
    { 0x8557e09b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb7e09e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf57e0a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1797e0ac, SUC_lexicon_tags_1 },
    { 0x3cf7e0ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xab77e0af, SUC_lexicon_tags_14 },
    { 0x2a77e0b0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937e0b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa257e0b6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2517e0b8, SUC_lexicon_tags_2 },
    { 0xf1d7e0b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c17e0bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f97e0c3, SUC_lexicon_tags_11 },
    { 0x1a57e0c3, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xbd37e0c6, SUC_lexicon_tags_23 },
    { 0x1cd7e0c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6757e0cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9237e0d0, SUC_lexicon_tags_5 },
    { 0xf477e0d0, SUC_lexicon_tags_12 },
    { 0xfd37e0d2, SUC_lexicon_tags_12 },
    { 0xc6f7e0d3, SUC_lexicon_tags_9 },
    { 0x2d97e0d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477e0d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b57e0da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5697e0dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa457e0df, SUC_lexicon_tags_37 },
    { 0xcd7e0e0, SUC_lexicon_tags_11 },
    { 0x99d7e0df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7357e0e3, SUC_lexicon_tags_7 },
    { 0x7d97e0e3, SUC_lexicon_tags_25 },
    { 0x1497e0e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e77e0e8, SUC_lexicon_tags_1 },
    { 0x1d97e0e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5357e0ed, SUC_lexicon_tags_11 },
    { 0xe97e0ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b17e0f4, SUC_lexicon_tags_27 },
    { 0x1b17e0f5, SUC_lexicon_tags_15 },
    { 0xbf7e0f6, SUC_lexicon_tags_39 },
    { 0x20f7e0f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a97e0fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b7e0fe, SUC_lexicon_tags_11 },
    { 0x9b17e0fe, SUC_lexicon_tags_74 },
    { 0xf7e100, SUC_lexicon_tags_2 },
    { 0xc97e100, SUC_lexicon_tags_34 },
    { 0xeb57e100, SUC_lexicon_tags_2 },
    { 0xf097e103, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c77e106, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x117e108, SUC_lexicon_tags_49 },
    { 0x7437e108, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x57e10b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7e10e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9617e110, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917e115, SUC_lexicon_tags_33 },
    { 0xbd77e116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78f7e118, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe5f7e11a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa0b7e11c, SUC_lexicon_tags_41 },
    { 0xb497e11c, SUC_lexicon_tags_12 },
    { 0xca17e11e, SUC_lexicon_tags_12 },
    { 0xc7b7e11d, SUC_lexicon_tags_17 },
    { 0x2bd7e120, SUC_lexicon_tags_68 },
    { 0xe417e11d, SUC_lexicon_tags_11 },
    { 0xdf57e122, SUC_lexicon_tags_11 },
    { 0x3e17e123, SUC_lexicon_tags_31 },
    { 0x63d7e123, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1a57e126, SUC_lexicon_tags_13 },
    { 0x5f37e127, SUC_lexicon_tags_38 },
    { 0x5837e127, SUC_lexicon_tags_25 },
    { 0xe517e129, SUC_lexicon_tags_30 },
    { 0xb3b7e12a, SUC_lexicon_tags_9 },
    { 0xb0b7e12b, SUC_lexicon_tags_12 },
    { 0xf2b7e12a, SUC_lexicon_tags_15 },
    { 0xce57e127, SUC_lexicon_tags_33 },
    { 0x3e17e12a, SUC_lexicon_tags_1 },
    { 0x8a57e12f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8137e131, SUC_lexicon_tags_11 },
    { 0x617e131, SUC_lexicon_tags_23 },
    { 0xd417e133, SUC_lexicon_tags_65 },
    { 0xdff7e134, SUC_lexicon_tags_19 },
    { 0xef97e135, SUC_lexicon_tags_26 },
    { 0x9e97e135, SUC_lexicon_tags_22 },
    { 0x2277e136, SUC_lexicon_tags_12 },
    { 0x9db7e134, SUC_lexicon_tags_8 },
    { 0xffd7e139, SUC_lexicon_tags_26 },
    { 0x43d7e136, SUC_lexicon_tags_4 },
    { 0x2a57e13b, SUC_lexicon_tags_60 },
    { 0x33b7e13c, SUC_lexicon_tags_45 },
    { 0x8337e13c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f7e143, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a97e145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d37e148, SUC_lexicon_tags_8 },
    { 0xe057e149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52b7e14b, SUC_lexicon_tags_12 },
    { 0xe157e14b, SUC_lexicon_tags_7 },
    { 0x64d7e14b, SUC_lexicon_tags_55 },
    { 0x8b57e14c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef7e157, SUC_lexicon_tags_37 },
    { 0x7b17e158, SUC_lexicon_tags_25 },
    { 0x1617e159, SUC_lexicon_tags_49 },
    { 0x4537e15a, SUC_lexicon_tags_28 },
    { 0x15b7e15b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f7e15e, SUC_lexicon_tags_23 },
    { 0xbad7e15f, SUC_lexicon_tags_53 },
    { 0x6d7e15e, SUC_lexicon_tags_33 },
    { 0x2897e161, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2477e163, SUC_lexicon_tags_15 },
    { 0x5b17e163, SUC_lexicon_tags_15 },
    { 0x997e165, SUC_lexicon_tags_7 },
    { 0x3297e165, SUC_lexicon_tags_26 },
    { 0x5c17e167, SUC_lexicon_tags_25 },
    { 0xf497e168, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1417e16b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7537e16f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6377e173, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd7e17c, SUC_lexicon_tags_19 },
    { 0xa8f7e17d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4077e180, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2517e184, SUC_lexicon_tags_9 },
    { 0x9177e185, SUC_lexicon_tags_17 },
    { 0x5a17e185, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe2d7e188, SUC_lexicon_tags_1 },
    { 0x4a57e188, SUC_lexicon_tags_59 },
    { 0xe157e189, SUC_lexicon_tags_10 },
    { 0x4417e188, SUC_lexicon_tags_25 },
    { 0x8297e188, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff7e190, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e37e193, SUC_lexicon_tags_1 },
    { 0xa3d7e194, SUC_lexicon_tags_7 },
    { 0x71b7e195, SUC_lexicon_tags_2 },
    { 0x4cb7e193, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1dd7e198, SUC_lexicon_tags_4 },
    { 0x34b7e199, SUC_lexicon_tags_7 },
    { 0xf17e198, SUC_lexicon_tags_27 },
    { 0x4557e199, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28d7e19d, SUC_lexicon_tags_29 },
    { 0x3dd7e19d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf97e1a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57e1a3, SUC_lexicon_tags_37 },
    { 0x6d77e1a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137e1a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x30f7e1aa, SUC_lexicon_tags_14 },
    { 0x3317e1ab, SUC_lexicon_tags_6 },
    { 0x7597e1ac, SUC_lexicon_tags_87 },
    { 0xf517e1aa, SUC_lexicon_tags_43 },
    { 0xe057e1ac, SUC_lexicon_tags_33 },
    { 0xa2d7e1af, SUC_lexicon_tags_11 },
    { 0x8677e1b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd6f7e1b2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6877e1b5, SUC_lexicon_tags_38 },
    { 0x5c37e1b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9557e1ba, SUC_lexicon_tags_37 },
    { 0x2b7e1bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f97e1bd, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb7e1c0, SUC_lexicon_tags_17 },
    { 0x6f17e1c0, SUC_lexicon_tags_7 },
    { 0xb977e1c0, SUC_lexicon_tags_7 },
    { 0xd3b7e1c1, SUC_lexicon_tags_39 },
    { 0xbd57e1c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b7e1c9, SUC_lexicon_tags_15 },
    { 0x13b7e1ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc97e1cf, SUC_lexicon_tags_35 },
    { 0x9497e1cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x657e1d3, SUC_lexicon_tags_27 },
    { 0x6757e1d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b7e1db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86d7e1dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f7e1e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc817e1e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1517e1ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1077e1f2, SUC_lexicon_tags_14 },
    { 0x6c37e1f2, SUC_lexicon_tags_11 },
    { 0x2197e1f4, SUC_lexicon_tags_37 },
    { 0x70d7e1f5, SUC_lexicon_tags_7 },
    { 0x27d7e1f5, SUC_lexicon_tags_1 },
    { 0xb57e1f7, SUC_lexicon_tags_10 },
    { 0x7bb7e1f3, SUC_lexicon_tags_16 },
    { 0x6b7e1f9, SUC_lexicon_tags_15 },
    { 0x4297e1fa, SUC_lexicon_tags_12 },
    { 0x18f7e1fb, SUC_lexicon_tags_2 },
    { 0xe897e1fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x817e1fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a17e204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec97e207, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5337e20c, SUC_lexicon_tags_8 },
    { 0x4277e20d, SUC_lexicon_tags_1117 },
    { 0, NULL },
    { 0x717e20f, SUC_lexicon_tags_3 },
    { 0xdeb7e210, SUC_lexicon_tags_6 },
    { 0x90b7e210, SUC_lexicon_tags_7 },
    { 0xced7e212, SUC_lexicon_tags_15 },
    { 0xeff7e212, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb657e216, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb7e21c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa617e222, SUC_lexicon_tags_7 },
    { 0xceb7e222, SUC_lexicon_tags_21 },
    { 0xcbb7e223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d7e228, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8ef7e22a, SUC_lexicon_tags_39 },
    { 0xe6f7e22a, SUC_lexicon_tags_66 },
    { 0x83b7e22c, SUC_lexicon_tags_1 },
    { 0xb2b7e22a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a17e230, SUC_lexicon_tags_42 },
    { 0x3cf7e231, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xde97e234, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3997e236, SUC_lexicon_tags_8 },
    { 0x9637e237, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a37e23b, SUC_lexicon_tags_17 },
    { 0x2437e23c, SUC_lexicon_tags_12 },
    { 0x9477e23c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b7e245, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd97e24c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe617e24e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd717e250, SUC_lexicon_tags_17 },
    { 0x8077e251, SUC_lexicon_tags_27 },
    { 0x5997e252, SUC_lexicon_tags_7 },
    { 0xbd77e252, SUC_lexicon_tags_28 },
    { 0xe5f7e254, SUC_lexicon_tags_17 },
    { 0x3b77e255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd77e257, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf17e259, SUC_lexicon_tags_12 },
    { 0x177e25a, SUC_lexicon_tags_50 },
    { 0xfaf7e259, SUC_lexicon_tags_7 },
    { 0xab77e25c, SUC_lexicon_tags_19 },
    { 0x5db7e25b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d7e262, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29d7e264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b7e268, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa957e273, SUC_lexicon_tags_37 },
    { 0x91b7e273, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f7e277, SUC_lexicon_tags_66 },
    { 0x4037e278, SUC_lexicon_tags_2 },
    { 0xa877e277, SUC_lexicon_tags_15 },
    { 0xdb97e27a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2837e27c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db7e285, SUC_lexicon_tags_17 },
    { 0xac37e286, SUC_lexicon_tags_0 },
    { 0x1837e287, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb057e28c, SUC_lexicon_tags_68 },
    { 0x2b17e28c, SUC_lexicon_tags_5 },
    { 0x76d7e28c, SUC_lexicon_tags_2 },
    { 0x11f7e28d, SUC_lexicon_tags_2 },
    { 0x9417e290, SUC_lexicon_tags_27 },
    { 0x7ad7e28f, SUC_lexicon_tags_16 },
    { 0xd297e292, SUC_lexicon_tags_15 },
    { 0x3937e28c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df7e297, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa337e29b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d57e29d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb77e2a0, SUC_lexicon_tags_17 },
    { 0xab37e2a0, SUC_lexicon_tags_7 },
    { 0xc3b7e2a2, SUC_lexicon_tags_43 },
    { 0x7517e2a3, SUC_lexicon_tags_17 },
    { 0xd617e2a1, SUC_lexicon_tags_25 },
    { 0x3eb7e2a5, SUC_lexicon_tags_45 },
    { 0xc1d7e2a6, SUC_lexicon_tags_79 },
    { 0x8817e2a5, SUC_lexicon_tags_38 },
    { 0xbc77e2a8, SUC_lexicon_tags_11 },
    { 0x7937e2a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1257e2ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b7e2b1, SUC_lexicon_tags_43 },
    { 0x5ed7e2b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9557e2b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf17e2bb, SUC_lexicon_tags_7 },
    { 0xc837e2bb, SUC_lexicon_tags_60 },
    { 0xc457e2bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb497e2c0, SUC_lexicon_tags_70 },
    { 0xe477e2c1, SUC_lexicon_tags_38 },
    { 0x8137e2c1, SUC_lexicon_tags_2 },
    { 0xbd17e2c3, SUC_lexicon_tags_25 },
    { 0x7e17e2c4, SUC_lexicon_tags_37 },
    { 0x31b7e2c4, SUC_lexicon_tags_1 },
    { 0xe417e2c4, SUC_lexicon_tags_7 },
    { 0x677e2c2, SUC_lexicon_tags_87 },
    { 0x99d7e2c8, SUC_lexicon_tags_2 },
    { 0x5057e2c2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x49b7e2cb, SUC_lexicon_tags_66 },
    { 0x1477e2cc, SUC_lexicon_tags_44 },
    { 0x71f7e2cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16f7e2cf, SUC_lexicon_tags_39 },
    { 0xa517e2cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d7e2d6, SUC_lexicon_tags_25 },
    { 0x75b7e2d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a97e2d9, SUC_lexicon_tags_25 },
    { 0xe357e2da, SUC_lexicon_tags_149 },
    { 0x5c37e2db, SUC_lexicon_tags_15 },
    { 0xb037e2da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4f77e2de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c57e2e4, SUC_lexicon_tags_11 },
    { 0x2df7e2e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf437e2ef, SUC_lexicon_tags_7 },
    { 0xd397e2f0, SUC_lexicon_tags_9 },
    { 0xcd37e2f1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f57e2f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e97e2f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8377e2fb, SUC_lexicon_tags_9 },
    { 0x6cd7e2fc, SUC_lexicon_tags_12 },
    { 0x53b7e2fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed37e2ff, SUC_lexicon_tags_15 },
    { 0x95d7e2ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61f7e302, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe77e305, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x2db7e307, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd477e309, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3457e30e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x47b7e310, SUC_lexicon_tags_25 },
    { 0x3e57e311, SUC_lexicon_tags_15 },
    { 0x8017e312, SUC_lexicon_tags_45 },
    { 0xbdb7e311, SUC_lexicon_tags_31 },
    { 0x8cf7e314, SUC_lexicon_tags_37 },
    { 0xff17e310, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f7e318, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a57e31b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b7e31e, SUC_lexicon_tags_17 },
    { 0xc177e31e, SUC_lexicon_tags_32 },
    { 0xdb17e31f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d37e325, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf7e328, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f77e32a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f57e32d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d7e331, SUC_lexicon_tags_11 },
    { 0xffd7e331, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e37e33a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29f7e33c, SUC_lexicon_tags_16 },
    { 0x6517e33d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b7e340, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xed37e342, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac77e34b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ef7e34d, SUC_lexicon_tags_25 },
    { 0x2297e34e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a77e354, SUC_lexicon_tags_12 },
    { 0x64f7e355, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7977e357, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d7e35c, SUC_lexicon_tags_47 },
    { 0xf437e35c, SUC_lexicon_tags_607 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf7e360, SUC_lexicon_tags_49 },
    { 0x5597e360, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1737e365, SUC_lexicon_tags_7 },
    { 0x43d7e366, SUC_lexicon_tags_12 },
    { 0xbfb7e367, SUC_lexicon_tags_141 },
    { 0x34b7e368, SUC_lexicon_tags_2 },
    { 0x8f17e365, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb197e36d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc757e370, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x39b7e372, SUC_lexicon_tags_40 },
    { 0xeb57e373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6777e378, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8957e37e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177e385, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd7e388, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5657e38b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7057e392, SUC_lexicon_tags_37 },
    { 0x4ad7e393, SUC_lexicon_tags_11 },
    { 0x53b7e394, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4137e396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbb7e398, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8497e39b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d37e3a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4237e3a6, SUC_lexicon_tags_60 },
    { 0x2737e3a7, SUC_lexicon_tags_43 },
    { 0x7157e3a8, SUC_lexicon_tags_3 },
    { 0x9737e3a9, SUC_lexicon_tags_7 },
    { 0x24b7e3a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a77e3b5, SUC_lexicon_tags_47 },
    { 0x42f7e3b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f77e3bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297e3c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d7e3cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2217e3d0, SUC_lexicon_tags_28 },
    { 0x3b37e3d1, SUC_lexicon_tags_79 },
    { 0x7577e3d1, SUC_lexicon_tags_48 },
    { 0x3d7e3d3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3e77e3d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b37e3da, SUC_lexicon_tags_11 },
    { 0xfcf7e3db, SUC_lexicon_tags_27 },
    { 0xadd7e3db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff7e3e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f7e3e3, SUC_lexicon_tags_2 },
    { 0x8137e3e3, SUC_lexicon_tags_28 },
    { 0x8c17e3e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5157e3e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb7e3ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b77e3ee, SUC_lexicon_tags_43 },
    { 0x4997e3ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d7e3fa, SUC_lexicon_tags_31 },
    { 0x7897e3fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb7e401, SUC_lexicon_tags_1 },
    { 0x8757e402, SUC_lexicon_tags_11 },
    { 0xab57e401, SUC_lexicon_tags_19 },
    { 0xb377e403, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb877e406, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd7e40b, SUC_lexicon_tags_16 },
    { 0x1b37e40c, SUC_lexicon_tags_27 },
    { 0x2d37e40d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb7e412, SUC_lexicon_tags_33 },
    { 0xaa77e412, SUC_lexicon_tags_0 },
    { 0xf317e414, SUC_lexicon_tags_11 },
    { 0x5e17e413, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc057e418, SUC_lexicon_tags_2 },
    { 0x3b7e419, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7117e41f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10b7e421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47f7e423, SUC_lexicon_tags_11 },
    { 0x4517e424, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x377e426, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x40b7e428, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35d7e42a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdd7e42c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc97e432, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe977e436, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d7e43c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7757e440, SUC_lexicon_tags_28 },
    { 0x657e441, SUC_lexicon_tags_2 },
    { 0x5677e442, SUC_lexicon_tags_29 },
    { 0x5857e443, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e37e445, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5117e448, SUC_lexicon_tags_35 },
    { 0x27f7e449, SUC_lexicon_tags_7 },
    { 0xf737e449, SUC_lexicon_tags_11 },
    { 0x4d97e449, SUC_lexicon_tags_63 },
    { 0xcf17e44a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab37e453, SUC_lexicon_tags_7 },
    { 0xff57e454, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed7e458, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9057e45f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb217e461, SUC_lexicon_tags_9 },
    { 0x4e17e462, SUC_lexicon_tags_30 },
    { 0xf517e463, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f7e467, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4837e46b, SUC_lexicon_tags_63 },
    { 0xd617e46c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7917e46f, SUC_lexicon_tags_7 },
    { 0xe697e470, SUC_lexicon_tags_2 },
    { 0x6237e471, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e17e475, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f7e478, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b7e47b, SUC_lexicon_tags_11 },
    { 0x9617e47c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4737e47f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b57e482, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe377e488, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee37e48e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf97e490, SUC_lexicon_tags_25 },
    { 0x1157e491, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2857e495, SUC_lexicon_tags_2 },
    { 0xf057e496, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe257e49a, SUC_lexicon_tags_63 },
    { 0xc2b7e49b, SUC_lexicon_tags_7 },
    { 0x8c37e49b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x957e49e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb937e4a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2a57e4a2, SUC_lexicon_tags_60 },
    { 0xb77e4a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5977e4a8, SUC_lexicon_tags_64 },
    { 0xe657e4a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3337e4ad, SUC_lexicon_tags_65 },
    { 0xcab7e4ad, SUC_lexicon_tags_14 },
    { 0x1277e4af, SUC_lexicon_tags_6 },
    { 0x2697e4ad, SUC_lexicon_tags_37 },
    { 0x837e4b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8557e4b3, SUC_lexicon_tags_1 },
    { 0xf877e4b4, SUC_lexicon_tags_37 },
    { 0xdff7e4b3, SUC_lexicon_tags_54 },
    { 0xab97e4b6, SUC_lexicon_tags_7 },
    { 0x32f7e4b5, SUC_lexicon_tags_43 },
    { 0x5517e4b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1497e4ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf7e4bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6117e4bf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f7e4c4, SUC_lexicon_tags_45 },
    { 0xffd7e4c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b7e4ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b7e4cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4cd7e4d1, SUC_lexicon_tags_33 },
    { 0x4357e4d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f97e4d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa797e4d7, SUC_lexicon_tags_37 },
    { 0x2957e4d8, SUC_lexicon_tags_25 },
    { 0xf9b7e4d7, SUC_lexicon_tags_25 },
    { 0x6877e4da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe517e4dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c37e4e0, SUC_lexicon_tags_12 },
    { 0x9817e4e1, SUC_lexicon_tags_4 },
    { 0x6017e4e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb7e4e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d7e4e8, SUC_lexicon_tags_7 },
    { 0x8f97e4e9, SUC_lexicon_tags_37 },
    { 0x1db7e4ea, SUC_lexicon_tags_29 },
    { 0x3a97e4eb, SUC_lexicon_tags_18 },
    { 0x9157e4eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4357e4ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8977e4f0, SUC_lexicon_tags_34 },
    { 0xa0b7e4f1, SUC_lexicon_tags_19 },
    { 0xcbd7e4f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad7e4f5, SUC_lexicon_tags_32 },
    { 0xa797e4f6, SUC_lexicon_tags_27 },
    { 0xa77e4f7, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57e4fb, SUC_lexicon_tags_17 },
    { 0x7f57e4fc, SUC_lexicon_tags_17 },
    { 0xe9d7e4fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe17e503, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae77e505, SUC_lexicon_tags_0 },
    { 0xd4b7e506, SUC_lexicon_tags_39 },
    { 0x3877e505, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2057e509, SUC_lexicon_tags_7 },
    { 0x2f7e50a, SUC_lexicon_tags_15 },
    { 0x3957e50b, SUC_lexicon_tags_25 },
    { 0x8337e50c, SUC_lexicon_tags_37 },
    { 0x6337e50d, SUC_lexicon_tags_33 },
    { 0x7e77e509, SUC_lexicon_tags_19 },
    { 0xe5b7e50a, SUC_lexicon_tags_23 },
    { 0xa8b7e50a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab7e518, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1f7e51a, SUC_lexicon_tags_1 },
    { 0x1717e51a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b97e521, SUC_lexicon_tags_43 },
    { 0x9bd7e522, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x257e524, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb7e52c, SUC_lexicon_tags_28 },
    { 0xb1b7e52d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5557e530, SUC_lexicon_tags_25 },
    { 0xe557e530, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8817e533, SUC_lexicon_tags_14 },
    { 0xf937e533, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b7e537, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1097e539, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x317e53c, SUC_lexicon_tags_437 },
    { 0xa0b7e53d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba17e546, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7e54a, SUC_lexicon_tags_15 },
    { 0x5fb7e54a, SUC_lexicon_tags_19 },
    { 0x1e77e54b, SUC_lexicon_tags_7 },
    { 0x6257e54d, SUC_lexicon_tags_32 },
    { 0x14b7e54c, SUC_lexicon_tags_28 },
    { 0x1a77e54f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f7e552, SUC_lexicon_tags_25 },
    { 0xa317e553, SUC_lexicon_tags_38 },
    { 0x5837e554, SUC_lexicon_tags_84 },
    { 0xb977e555, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d7e55a, SUC_lexicon_tags_28 },
    { 0x5d97e55b, SUC_lexicon_tags_23 },
    { 0x5c77e55c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f7e561, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f7e565, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d7e56a, SUC_lexicon_tags_37 },
    { 0x1697e56b, SUC_lexicon_tags_25 },
    { 0xf9d7e56b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf7e572, SUC_lexicon_tags_15 },
    { 0xabb7e573, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5317e577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfef7e579, SUC_lexicon_tags_17 },
    { 0x1337e57a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa317e57c, SUC_lexicon_tags_28 },
    { 0xac97e57c, SUC_lexicon_tags_9 },
    { 0xba97e57e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d7e583, SUC_lexicon_tags_82 },
    { 0xdaf7e583, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb7e587, SUC_lexicon_tags_63 },
    { 0xf7f7e587, SUC_lexicon_tags_66 },
    { 0xb917e588, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb957e58c, SUC_lexicon_tags_25 },
    { 0x19f7e58d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7e596, SUC_lexicon_tags_12 },
    { 0xdbf7e597, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3857e599, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3717e59c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x54d7e59e, SUC_lexicon_tags_23 },
    { 0x9db7e59e, SUC_lexicon_tags_8 },
    { 0x5f17e5a0, SUC_lexicon_tags_25 },
    { 0x71f7e5a1, SUC_lexicon_tags_1 },
    { 0xad37e5a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a37e5a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3857e5a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8737e5b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdb97e5b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5bf7e5b5, SUC_lexicon_tags_94 },
    { 0x53b7e5b6, SUC_lexicon_tags_2 },
    { 0xa4f7e5b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c77e5b9, SUC_lexicon_tags_14 },
    { 0xbff7e5b9, SUC_lexicon_tags_1 },
    { 0xe3f7e5bb, SUC_lexicon_tags_17 },
    { 0x5037e5bc, SUC_lexicon_tags_11 },
    { 0xa977e5bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb77e5bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4717e5c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e57e5c4, SUC_lexicon_tags_38 },
    { 0x6ab7e5c4, SUC_lexicon_tags_87 },
    { 0x8397e5c5, SUC_lexicon_tags_27 },
    { 0x72d7e5c7, SUC_lexicon_tags_15 },
    { 0x7637e5c8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5657e5ca, SUC_lexicon_tags_38 },
    { 0xc2f7e5cb, SUC_lexicon_tags_25 },
    { 0xa897e5ca, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x6a17e5ce, SUC_lexicon_tags_12 },
    { 0x3017e5ce, SUC_lexicon_tags_0 },
    { 0xfcf7e5d0, SUC_lexicon_tags_8 },
    { 0xfcb7e5ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b77e5d7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1a17e5d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4257e5dc, SUC_lexicon_tags_37 },
    { 0xc797e5dd, SUC_lexicon_tags_39 },
    { 0xbd97e5de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd237e5e3, SUC_lexicon_tags_19 },
    { 0x65f7e5e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d7e5e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x917e5ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2657e5f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1397e5f7, SUC_lexicon_tags_7 },
    { 0xecd7e5f7, SUC_lexicon_tags_6 },
    { 0x1c57e5f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb357e5fd, SUC_lexicon_tags_17 },
    { 0xf617e5fe, SUC_lexicon_tags_45 },
    { 0x15f7e5ff, SUC_lexicon_tags_25 },
    { 0x277e5fe, SUC_lexicon_tags_37 },
    { 0xc597e601, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x417e603, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2237e606, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd7e60a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a57e60d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6d7e60f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b37e61b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6517e61e, SUC_lexicon_tags_2 },
    { 0x62b7e61f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6157e622, SUC_lexicon_tags_19 },
    { 0x4ed7e623, SUC_lexicon_tags_43 },
    { 0xeeb7e624, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3797e627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3d7e629, SUC_lexicon_tags_25 },
    { 0x2277e62a, SUC_lexicon_tags_37 },
    { 0x16f7e629, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9d97e62d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1397e631, SUC_lexicon_tags_29 },
    { 0xdf37e632, SUC_lexicon_tags_0 },
    { 0x2297e633, SUC_lexicon_tags_2 },
    { 0x7b7e633, SUC_lexicon_tags_2 },
    { 0xf137e635, SUC_lexicon_tags_28 },
    { 0x7617e635, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf17e63c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa857e642, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57e646, SUC_lexicon_tags_59 },
    { 0xd057e647, SUC_lexicon_tags_37 },
    { 0xb057e647, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf177e64c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9797e64f, SUC_lexicon_tags_1 },
    { 0xfef7e64f, SUC_lexicon_tags_10 },
    { 0xc857e651, SUC_lexicon_tags_275 },
    { 0x10d7e652, SUC_lexicon_tags_37 },
    { 0xe217e653, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd757e656, SUC_lexicon_tags_17 },
    { 0x5f97e656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b7e65b, SUC_lexicon_tags_0 },
    { 0xb5d7e65c, SUC_lexicon_tags_25 },
    { 0x39b7e65d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c37e65f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe97e662, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1957e668, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2617e66c, SUC_lexicon_tags_72 },
    { 0xffb7e66d, SUC_lexicon_tags_30 },
    { 0xa4b7e66d, SUC_lexicon_tags_40 },
    { 0x5897e66c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xceb7e671, SUC_lexicon_tags_118 },
    { 0xd1d7e671, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d7e678, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4417e67a, SUC_lexicon_tags_17 },
    { 0x2417e67a, SUC_lexicon_tags_30 },
    { 0xa997e67c, SUC_lexicon_tags_38 },
    { 0x3ab7e67c, SUC_lexicon_tags_7 },
    { 0x317e67e, SUC_lexicon_tags_11 },
    { 0xa797e67b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2877e682, SUC_lexicon_tags_25 },
    { 0x5ad7e683, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c97e685, SUC_lexicon_tags_56 },
    { 0x1057e685, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f7e68e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56d7e690, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfd7e692, SUC_lexicon_tags_45 },
    { 0xf477e693, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7377e695, SUC_lexicon_tags_6 },
    { 0x9017e695, SUC_lexicon_tags_15 },
    { 0x9817e695, SUC_lexicon_tags_37 },
    { 0x3317e698, SUC_lexicon_tags_17 },
    { 0x277e699, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b7e69c, SUC_lexicon_tags_17 },
    { 0x5ff7e69d, SUC_lexicon_tags_9 },
    { 0x9377e69c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5017e6a5, SUC_lexicon_tags_23 },
    { 0x1037e6a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a57e6ad, SUC_lexicon_tags_70 },
    { 0xa97e6ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9197e6b3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f7e6b7, SUC_lexicon_tags_47 },
    { 0xc57e6b8, SUC_lexicon_tags_47 },
    { 0xe6f7e6b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5117e6bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9997e6c1, SUC_lexicon_tags_27 },
    { 0x9bf7e6c1, SUC_lexicon_tags_1 },
    { 0x8c97e6c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f7e6c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb7e6cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf877e6d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd177e6d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd0f7e6d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f7e6df, SUC_lexicon_tags_15 },
    { 0x74d7e6e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4377e6e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c97e6e8, SUC_lexicon_tags_113 },
    { 0xbc77e6e9, SUC_lexicon_tags_43 },
    { 0x5e97e6ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf77e6ec, SUC_lexicon_tags_7 },
    { 0x20b7e6ed, SUC_lexicon_tags_39 },
    { 0x9b97e6ee, SUC_lexicon_tags_37 },
    { 0x8797e6ed, SUC_lexicon_tags_25 },
    { 0xc657e6ec, SUC_lexicon_tags_11 },
    { 0x53d7e6f1, SUC_lexicon_tags_11 },
    { 0xc197e6f2, SUC_lexicon_tags_27 },
    { 0xb857e6f3, SUC_lexicon_tags_38 },
    { 0xa097e6f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6477e704, SUC_lexicon_tags_66 },
    { 0xb9f7e705, SUC_lexicon_tags_63 },
    { 0x6497e706, SUC_lexicon_tags_1 },
    { 0xd737e706, SUC_lexicon_tags_103 },
    { 0x3d7e708, SUC_lexicon_tags_8 },
    { 0x70f7e704, SUC_lexicon_tags_15 },
    { 0xe917e70a, SUC_lexicon_tags_12 },
    { 0x6637e70a, SUC_lexicon_tags_17 },
    { 0x4c57e70c, SUC_lexicon_tags_33 },
    { 0xeed7e70d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b7e710, SUC_lexicon_tags_11 },
    { 0x8657e710, SUC_lexicon_tags_43 },
    { 0x7317e712, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe4d7e714, SUC_lexicon_tags_77 },
    { 0x21f7e715, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c37e71a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b97e71d, SUC_lexicon_tags_0 },
    { 0x3fd7e71d, SUC_lexicon_tags_19 },
    { 0x8217e71f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb7e722, SUC_lexicon_tags_56 },
    { 0x5bf7e723, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64d7e725, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a77e729, SUC_lexicon_tags_87 },
    { 0x76d7e72a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbd7e72c, SUC_lexicon_tags_51 },
    { 0x5037e72d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8977e72f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b7e733, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x70f7e735, SUC_lexicon_tags_11 },
    { 0x5d97e736, SUC_lexicon_tags_19 },
    { 0xfbf7e737, SUC_lexicon_tags_7 },
    { 0x8ab7e738, SUC_lexicon_tags_11 },
    { 0x3737e739, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5937e73c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6c97e73e, SUC_lexicon_tags_45 },
    { 0xae37e73f, SUC_lexicon_tags_7 },
    { 0x94b7e73f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7337e743, SUC_lexicon_tags_7 },
    { 0xd657e744, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb577e746, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc0f7e748, SUC_lexicon_tags_37 },
    { 0xaf77e748, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5657e74b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f7e74f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8297e751, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd17e753, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d97e757, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8037e75b, SUC_lexicon_tags_7 },
    { 0xd2d7e75c, SUC_lexicon_tags_17 },
    { 0xc8d7e75d, SUC_lexicon_tags_37 },
    { 0x8f57e75c, SUC_lexicon_tags_29 },
    { 0xce97e75f, SUC_lexicon_tags_15 },
    { 0x1177e760, SUC_lexicon_tags_51 },
    { 0xc5d7e761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2717e763, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfe17e765, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe737e76d, SUC_lexicon_tags_3 },
    { 0x1cb7e76d, SUC_lexicon_tags_60 },
    { 0xc6d7e76e, SUC_lexicon_tags_37 },
    { 0xad17e770, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b7e775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc537e778, SUC_lexicon_tags_55 },
    { 0x777e779, SUC_lexicon_tags_27 },
    { 0xa97e77a, SUC_lexicon_tags_2 },
    { 0xf137e778, SUC_lexicon_tags_47 },
    { 0xb0b7e77c, SUC_lexicon_tags_108 },
    { 0xad57e77c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4137e782, SUC_lexicon_tags_30 },
    { 0x7b7e782, SUC_lexicon_tags_7 },
    { 0x7457e784, SUC_lexicon_tags_14 },
    { 0x3057e784, SUC_lexicon_tags_84 },
    { 0xd637e785, SUC_lexicon_tags_12 },
    { 0x91d7e787, SUC_lexicon_tags_37 },
    { 0x4d7e788, SUC_lexicon_tags_1 },
    { 0x8277e789, SUC_lexicon_tags_62 },
    { 0xb377e78a, SUC_lexicon_tags_25 },
    { 0x7d37e78a, SUC_lexicon_tags_17 },
    { 0xb137e78c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd557e795, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7457e79b, SUC_lexicon_tags_15 },
    { 0x15b7e79b, SUC_lexicon_tags_15 },
    { 0xf2d7e79d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee97e79f, SUC_lexicon_tags_25 },
    { 0xc0d7e7a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3637e7a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef7e7a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b7e7ae, SUC_lexicon_tags_2 },
    { 0x3ed7e7af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f7e7b4, SUC_lexicon_tags_25 },
    { 0x1d37e7b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db7e7b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7577e7c0, SUC_lexicon_tags_63 },
    { 0x6c37e7c1, SUC_lexicon_tags_23 },
    { 0x18d7e7c0, SUC_lexicon_tags_30 },
    { 0x2b17e7c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f7e7ca, SUC_lexicon_tags_14 },
    { 0x6677e7cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f7e7cf, SUC_lexicon_tags_12 },
    { 0x7ab7e7cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5737e7d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa537e7d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x657e7d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1997e7dc, SUC_lexicon_tags_17 },
    { 0x2e77e7dc, SUC_lexicon_tags_15 },
    { 0x46d7e7de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3617e7e1, SUC_lexicon_tags_2 },
    { 0x2f37e7e2, SUC_lexicon_tags_19 },
    { 0x5ad7e7e2, SUC_lexicon_tags_25 },
    { 0x3677e7e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5157e7e8, SUC_lexicon_tags_14 },
    { 0x7597e7e9, SUC_lexicon_tags_33 },
    { 0xdbf7e7e8, SUC_lexicon_tags_68 },
    { 0x1817e7ea, SUC_lexicon_tags_138 },
    { 0xd7e7ec, SUC_lexicon_tags_61 },
    { 0x3777e7ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7e7f4, SUC_lexicon_tags_25 },
    { 0xfc97e7f5, SUC_lexicon_tags_45 },
    { 0x67b7e7f6, SUC_lexicon_tags_2 },
    { 0xfc77e7f7, SUC_lexicon_tags_1 },
    { 0xd117e7f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d7e7fb, SUC_lexicon_tags_36 },
    { 0xa837e7fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd97e7fe, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1917e801, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d7e806, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b57e80d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf297e813, SUC_lexicon_tags_40 },
    { 0xfe97e813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d37e816, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb9f7e818, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7e81c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f37e820, SUC_lexicon_tags_30 },
    { 0x7e37e820, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1d97e823, SUC_lexicon_tags_30 },
    { 0x1577e823, SUC_lexicon_tags_11 },
    { 0xc397e825, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f57e827, SUC_lexicon_tags_11 },
    { 0x15d7e828, SUC_lexicon_tags_0 },
    { 0xa6d7e829, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e17e830, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6777e836, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcff7e838, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8117e83a, SUC_lexicon_tags_24 },
    { 0xf6d7e83b, SUC_lexicon_tags_1 },
    { 0xddd7e83b, SUC_lexicon_tags_63 },
    { 0x8817e83d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d7e841, SUC_lexicon_tags_31 },
    { 0x1bf7e841, SUC_lexicon_tags_23 },
    { 0xa7d7e843, SUC_lexicon_tags_12 },
    { 0x13b7e844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfd7e846, SUC_lexicon_tags_7 },
    { 0x6b57e847, SUC_lexicon_tags_547 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d7e84b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6217e851, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7777e856, SUC_lexicon_tags_0 },
    { 0x6197e857, SUC_lexicon_tags_21 },
    { 0x91d7e858, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed7e85c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd0b7e85e, SUC_lexicon_tags_49 },
    { 0xa177e85f, SUC_lexicon_tags_9 },
    { 0xeed7e85f, SUC_lexicon_tags_66 },
    { 0xae57e860, SUC_lexicon_tags_12 },
    { 0x45f7e862, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1497e864, SUC_lexicon_tags_1 },
    { 0xc697e865, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7e86b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d7e86f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f7e878, SUC_lexicon_tags_5 },
    { 0xd057e879, SUC_lexicon_tags_15 },
    { 0xd337e87a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c17e87c, SUC_lexicon_tags_11 },
    { 0xf177e87d, SUC_lexicon_tags_49 },
    { 0xc9b7e87c, SUC_lexicon_tags_38 },
    { 0x1357e87d, SUC_lexicon_tags_47 },
    { 0xc4d7e880, SUC_lexicon_tags_30 },
    { 0x7477e881, SUC_lexicon_tags_61 },
    { 0x1317e87d, SUC_lexicon_tags_11 },
    { 0xed7e883, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba77e887, SUC_lexicon_tags_15 },
    { 0x2a77e887, SUC_lexicon_tags_2 },
    { 0xc897e889, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb477e88b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d7e891, SUC_lexicon_tags_84 },
    { 0x8317e891, SUC_lexicon_tags_28 },
    { 0xc377e893, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d7e896, SUC_lexicon_tags_45 },
    { 0xef7e896, SUC_lexicon_tags_15 },
    { 0x6757e896, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x837e89b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfcb7e89d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef7e8a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf757e8a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe717e8aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1637e8ad, SUC_lexicon_tags_19 },
    { 0x4477e8ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a37e8b0, SUC_lexicon_tags_17 },
    { 0xbd97e8b1, SUC_lexicon_tags_14 },
    { 0x5557e8b2, SUC_lexicon_tags_12 },
    { 0x8f77e8b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e77e8b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9337e8b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b97e8ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa3d7e8bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf897e8c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xceb7e8c2, SUC_lexicon_tags_3 },
    { 0x4a77e8c2, SUC_lexicon_tags_15 },
    { 0xae57e8c4, SUC_lexicon_tags_47 },
    { 0x7497e8c5, SUC_lexicon_tags_6 },
    { 0x6537e8c2, SUC_lexicon_tags_25 },
    { 0xbd37e8c5, SUC_lexicon_tags_46 },
    { 0x17f7e8c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd7e8cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x377e8ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf797e8d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39d7e8d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c97e8d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7e8e0, SUC_lexicon_tags_30 },
    { 0x1197e8e0, SUC_lexicon_tags_65 },
    { 0x8677e8e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f17e8e6, SUC_lexicon_tags_307 },
    { 0x3b57e8e6, SUC_lexicon_tags_15 },
    { 0xe517e8e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7517e8ed, SUC_lexicon_tags_0 },
    { 0xa57e8ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb477e8f0, SUC_lexicon_tags_19 },
    { 0xb957e8f1, SUC_lexicon_tags_2 },
    { 0x3a37e8f2, SUC_lexicon_tags_0 },
    { 0xe9b7e8f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3917e8f5, SUC_lexicon_tags_8 },
    { 0x88f7e8f6, SUC_lexicon_tags_17 },
    { 0xf797e8f5, SUC_lexicon_tags_32 },
    { 0xb437e8f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6997e8fd, SUC_lexicon_tags_29 },
    { 0x6437e8fe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc157e900, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa837e903, SUC_lexicon_tags_23 },
    { 0xe057e903, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc757e90e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b77e911, SUC_lexicon_tags_17 },
    { 0xb897e912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd597e914, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1257e916, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b7e920, SUC_lexicon_tags_65 },
    { 0x8f97e921, SUC_lexicon_tags_2 },
    { 0xb4f7e922, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7717e925, SUC_lexicon_tags_11 },
    { 0xdf77e926, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfed7e928, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabb7e92a, SUC_lexicon_tags_14 },
    { 0x62d7e92a, SUC_lexicon_tags_38 },
    { 0xf497e92a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5437e930, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbfb7e932, SUC_lexicon_tags_45 },
    { 0x8937e933, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd7e938, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe297e93b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa037e93d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d7e941, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfff7e943, SUC_lexicon_tags_1 },
    { 0xd357e944, SUC_lexicon_tags_12 },
    { 0x5ad7e945, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf677e948, SUC_lexicon_tags_63 },
    { 0x4537e949, SUC_lexicon_tags_15 },
    { 0x6677e948, SUC_lexicon_tags_57 },
    { 0xa917e94b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d7e957, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8777e959, SUC_lexicon_tags_17 },
    { 0xc637e959, SUC_lexicon_tags_13 },
    { 0x4077e95b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc77e95e, SUC_lexicon_tags_7 },
    { 0x7777e95f, SUC_lexicon_tags_37 },
    { 0xdc77e960, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd617e967, SUC_lexicon_tags_19 },
    { 0x4877e968, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f7e96b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6377e96d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1357e970, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa237e972, SUC_lexicon_tags_33 },
    { 0x4ed7e973, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xef97e975, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3477e97c, SUC_lexicon_tags_77 },
    { 0xa557e97d, SUC_lexicon_tags_25 },
    { 0xcd77e97e, SUC_lexicon_tags_19 },
    { 0x9657e97e, SUC_lexicon_tags_29 },
    { 0x2897e980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab77e98a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f57e98d, SUC_lexicon_tags_1 },
    { 0xdf77e98e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f7e992, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa3f7e994, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95d7e996, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b17e99b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf77e9a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d7e9a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe837e9a8, SUC_lexicon_tags_39 },
    { 0xf2f7e9a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd77e9ab, SUC_lexicon_tags_55 },
    { 0x9c97e9ac, SUC_lexicon_tags_68 },
    { 0x4e77e9ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18b7e9af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b17e9b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc717e9b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa037e9b9, SUC_lexicon_tags_17 },
    { 0xb197e9ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f37e9bd, SUC_lexicon_tags_8 },
    { 0xaf37e9be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d17e9c3, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x9a37e9c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb7e9ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3197e9ce, SUC_lexicon_tags_1 },
    { 0x8277e9cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac37e9d5, SUC_lexicon_tags_183 },
    { 0x9bd7e9d6, SUC_lexicon_tags_64 },
    { 0xcdb7e9d7, SUC_lexicon_tags_37 },
    { 0x51b7e9d8, SUC_lexicon_tags_27 },
    { 0xe37e9d5, SUC_lexicon_tags_17 },
    { 0xe537e9da, SUC_lexicon_tags_1 },
    { 0xd97e9da, SUC_lexicon_tags_17 },
    { 0x8417e9da, SUC_lexicon_tags_25 },
    { 0x9437e9dd, SUC_lexicon_tags_49 },
    { 0xb6b7e9de, SUC_lexicon_tags_45 },
    { 0xae37e9df, SUC_lexicon_tags_9 },
    { 0xa997e9dd, SUC_lexicon_tags_12 },
    { 0xbed7e9e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52b7e9e3, SUC_lexicon_tags_46 },
    { 0x53b7e9e3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x457e9e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36d7e9e9, SUC_lexicon_tags_66 },
    { 0x577e9ea, SUC_lexicon_tags_9 },
    { 0x4317e9ea, SUC_lexicon_tags_63 },
    { 0xac97e9ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef97e9ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89b7e9f0, SUC_lexicon_tags_33 },
    { 0x1157e9f1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb0d7e9f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7377e9f6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6637e9fa, SUC_lexicon_tags_84 },
    { 0x4197e9fa, SUC_lexicon_tags_37 },
    { 0xa8d7e9fc, SUC_lexicon_tags_301 },
    { 0xfc77e9fd, SUC_lexicon_tags_37 },
    { 0x8a77e9fc, SUC_lexicon_tags_11 },
    { 0x6317e9fc, SUC_lexicon_tags_25 },
    { 0x89d7ea00, SUC_lexicon_tags_60 },
    { 0xb5d7ea01, SUC_lexicon_tags_19 },
    { 0x60b7ea02, SUC_lexicon_tags_17 },
    { 0xd397ea03, SUC_lexicon_tags_42 },
    { 0xaab7ea04, SUC_lexicon_tags_15 },
    { 0x12b7ea05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe517ea07, SUC_lexicon_tags_17 },
    { 0x8317ea08, SUC_lexicon_tags_23 },
    { 0x63b7ea09, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x11b7ea0b, SUC_lexicon_tags_25 },
    { 0x8bf7ea0b, SUC_lexicon_tags_27 },
    { 0x957ea0d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d7ea18, SUC_lexicon_tags_30 },
    { 0x4d77ea19, SUC_lexicon_tags_47 },
    { 0x1ad7ea19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x697ea1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb37ea22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97ea24, SUC_lexicon_tags_47 },
    { 0xc2d7ea25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc977ea2d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3d77ea2f, SUC_lexicon_tags_54 },
    { 0x1157ea30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b17ea34, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d77ea38, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf7ea3b, SUC_lexicon_tags_27 },
    { 0x68b7ea3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f57ea42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xabf7ea44, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757ea48, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9757ea4c, SUC_lexicon_tags_27 },
    { 0xf2b7ea4d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b77ea50, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7897ea53, SUC_lexicon_tags_10 },
    { 0xc517ea54, SUC_lexicon_tags_71 },
    { 0x8137ea54, SUC_lexicon_tags_25 },
    { 0x22d7ea56, SUC_lexicon_tags_14 },
    { 0xa117ea56, SUC_lexicon_tags_31 },
    { 0xea57ea57, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5077ea6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af7ea70, SUC_lexicon_tags_23 },
    { 0xbc17ea71, SUC_lexicon_tags_43 },
    { 0x5857ea72, SUC_lexicon_tags_2 },
    { 0x2f57ea73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa577ea75, SUC_lexicon_tags_38 },
    { 0xfb7ea75, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x62b7ea78, SUC_lexicon_tags_25 },
    { 0xfc37ea78, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc177ea7c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f7ea84, SUC_lexicon_tags_33 },
    { 0xf977ea85, SUC_lexicon_tags_12 },
    { 0xe5d7ea86, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf577ea89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ed7ea8b, SUC_lexicon_tags_30 },
    { 0x4337ea8c, SUC_lexicon_tags_0 },
    { 0x53f7ea8c, SUC_lexicon_tags_23 },
    { 0x5c97ea8d, SUC_lexicon_tags_10 },
    { 0xa37ea8f, SUC_lexicon_tags_10 },
    { 0x34f7ea8b, SUC_lexicon_tags_27 },
    { 0xa457ea8d, SUC_lexicon_tags_68 },
    { 0x81f7ea92, SUC_lexicon_tags_0 },
    { 0xf897ea93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60d7ea95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd117ea99, SUC_lexicon_tags_11 },
    { 0xf617ea9a, SUC_lexicon_tags_39 },
    { 0xc297ea9b, SUC_lexicon_tags_49 },
    { 0x5777ea9b, SUC_lexicon_tags_38 },
    { 0x7e57ea9c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f7eaa3, SUC_lexicon_tags_11 },
    { 0x25d7eaa4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1317eaa7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd757eaaa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa037eaaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xed37eab1, SUC_lexicon_tags_12 },
    { 0xb337eab2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f7eab5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d97eab8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xab77eabb, SUC_lexicon_tags_12 },
    { 0x48f7eabb, SUC_lexicon_tags_87 },
    { 0xaf7eabd, SUC_lexicon_tags_7 },
    { 0x60b7eabd, SUC_lexicon_tags_37 },
    { 0xfa57eabf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaaf7eac1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c37eac4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b7eac9, SUC_lexicon_tags_47 },
    { 0xb997eac9, SUC_lexicon_tags_17 },
    { 0x1097eacb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd037eacd, SUC_lexicon_tags_42 },
    { 0xaa17eace, SUC_lexicon_tags_29 },
    { 0x2f97eacf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8097ead2, SUC_lexicon_tags_47 },
    { 0x3b17ead2, SUC_lexicon_tags_3 },
    { 0x1437ead3, SUC_lexicon_tags_0 },
    { 0x8bf7ead5, SUC_lexicon_tags_2 },
    { 0x897ead6, SUC_lexicon_tags_37 },
    { 0xa397ead7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb337ead9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d57eae4, SUC_lexicon_tags_62 },
    { 0x29b7eae4, SUC_lexicon_tags_45 },
    { 0xddb7eae5, SUC_lexicon_tags_11 },
    { 0x1b77eae7, SUC_lexicon_tags_61 },
    { 0x4897eae6, SUC_lexicon_tags_15 },
    { 0xe017eae8, SUC_lexicon_tags_43 },
    { 0x29d7eaea, SUC_lexicon_tags_63 },
    { 0x897eae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f7eaf2, SUC_lexicon_tags_1 },
    { 0x7cf7eaf3, SUC_lexicon_tags_49 },
    { 0xc037eaf2, SUC_lexicon_tags_2 },
    { 0xe2f7eaf5, SUC_lexicon_tags_23 },
    { 0x8617eaf2, SUC_lexicon_tags_103 },
    { 0x817eaf6, SUC_lexicon_tags_39 },
    { 0xe817eaf8, SUC_lexicon_tags_18 },
    { 0x9717eaf9, SUC_lexicon_tags_7 },
    { 0x52b7eafa, SUC_lexicon_tags_11 },
    { 0x9e37eaf5, SUC_lexicon_tags_65 },
    { 0x3537eafc, SUC_lexicon_tags_12 },
    { 0x4077eafd, SUC_lexicon_tags_15 },
    { 0xaf17eafe, SUC_lexicon_tags_37 },
    { 0x5eb7eafe, SUC_lexicon_tags_9 },
    { 0xb957eb00, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b7eb04, SUC_lexicon_tags_17 },
    { 0xb8d7eb05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c97eb08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe77eb0b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3517eb0f, SUC_lexicon_tags_41 },
    { 0xb617eb10, SUC_lexicon_tags_37 },
    { 0xd797eb10, SUC_lexicon_tags_30 },
    { 0x7877eb12, SUC_lexicon_tags_7 },
    { 0xd537eb13, SUC_lexicon_tags_27 },
    { 0xbdf7eb10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7917eb18, SUC_lexicon_tags_7 },
    { 0xb5d7eb18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b7eb1f, SUC_lexicon_tags_60 },
    { 0x3897eb20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5977eb22, SUC_lexicon_tags_7 },
    { 0xcbf7eb23, SUC_lexicon_tags_17 },
    { 0x3c17eb24, SUC_lexicon_tags_33 },
    { 0x5d7eb25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577eb2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4cd7eb2e, SUC_lexicon_tags_39 },
    { 0xeab7eb2f, SUC_lexicon_tags_12 },
    { 0xae37eb30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd7eb34, SUC_lexicon_tags_66 },
    { 0x18f7eb35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87d7eb37, SUC_lexicon_tags_42 },
    { 0x6d97eb38, SUC_lexicon_tags_15 },
    { 0xcff7eb39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x657eb3b, SUC_lexicon_tags_46 },
    { 0x69f7eb3b, SUC_lexicon_tags_17 },
    { 0x837eb3d, SUC_lexicon_tags_57 },
    { 0xd9b7eb3e, SUC_lexicon_tags_15 },
    { 0x3fd7eb3f, SUC_lexicon_tags_11 },
    { 0x8f37eb3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d7eb45, SUC_lexicon_tags_25 },
    { 0xd1d7eb45, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf1b7eb48, SUC_lexicon_tags_26 },
    { 0xa3f7eb49, SUC_lexicon_tags_85 },
    { 0x5117eb49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8057eb4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f57eb4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6957eb52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab7eb55, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb837eb5b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3757eb5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf77eb5f, SUC_lexicon_tags_15 },
    { 0x78f7eb60, SUC_lexicon_tags_17 },
    { 0xfa17eb61, SUC_lexicon_tags_17 },
    { 0x6e77eb62, SUC_lexicon_tags_17 },
    { 0xff17eb60, SUC_lexicon_tags_12 },
    { 0x7b77eb60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d57eb6a, SUC_lexicon_tags_37 },
    { 0xf177eb6a, SUC_lexicon_tags_37 },
    { 0x9cd7eb6a, SUC_lexicon_tags_30 },
    { 0xda97eb6a, SUC_lexicon_tags_17 },
    { 0xff37eb6e, SUC_lexicon_tags_48 },
    { 0xc6d7eb6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e77eb79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77eb7b, SUC_lexicon_tags_15 },
    { 0xd17eb7c, SUC_lexicon_tags_39 },
    { 0x6257eb7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f7eb80, SUC_lexicon_tags_16 },
    { 0x9677eb81, SUC_lexicon_tags_19 },
    { 0xcd17eb80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f7eb85, SUC_lexicon_tags_6 },
    { 0x3937eb86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe77eb89, SUC_lexicon_tags_7 },
    { 0x2457eb89, SUC_lexicon_tags_12 },
    { 0xd7b7eb8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x517eb8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e37eb90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1237eb94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3ad7eb96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b7eb99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b97eb9d, SUC_lexicon_tags_47 },
    { 0x2d57eb9d, SUC_lexicon_tags_21 },
    { 0x3ed7eb9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb577eba3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf317ebad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db7ebb0, SUC_lexicon_tags_30 },
    { 0x2e77ebb1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8897ebb3, SUC_lexicon_tags_63 },
    { 0xdd97ebb4, SUC_lexicon_tags_39 },
    { 0x8eb7ebb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4817ebb9, SUC_lexicon_tags_43 },
    { 0xb677ebba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f77ebbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f37ebbf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b7ebca, SUC_lexicon_tags_2 },
    { 0x78f7ebcb, SUC_lexicon_tags_11 },
    { 0x2017ebca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2937ebcf, SUC_lexicon_tags_37 },
    { 0xc537ebd0, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xceb7ebd2, SUC_lexicon_tags_4 },
    { 0xa2b7ebd3, SUC_lexicon_tags_14 },
    { 0xd437ebd4, SUC_lexicon_tags_9 },
    { 0xa057ebd2, SUC_lexicon_tags_33 },
    { 0x3f77ebd6, SUC_lexicon_tags_17 },
    { 0x4917ebd7, SUC_lexicon_tags_26 },
    { 0x7dd7ebd8, SUC_lexicon_tags_0 },
    { 0xc617ebd8, SUC_lexicon_tags_2 },
    { 0x797ebda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3477ebe7, SUC_lexicon_tags_38 },
    { 0x42b7ebe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4377ebea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8b7ebec, SUC_lexicon_tags_17 },
    { 0x2a17ebed, SUC_lexicon_tags_11 },
    { 0xc6d7ebee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57ebf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517ebf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2237ebfe, SUC_lexicon_tags_46 },
    { 0xc897ebff, SUC_lexicon_tags_11 },
    { 0x51f7ec00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xed57ec02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa1f7ec04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b7ec09, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc17ec10, SUC_lexicon_tags_47 },
    { 0xd597ec11, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b7ec14, SUC_lexicon_tags_29 },
    { 0x5f77ec15, SUC_lexicon_tags_16 },
    { 0xb377ec14, SUC_lexicon_tags_26 },
    { 0x4dd7ec14, SUC_lexicon_tags_15 },
    { 0xfdd7ec18, SUC_lexicon_tags_25 },
    { 0xe737ec16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa737ec1b, SUC_lexicon_tags_57 },
    { 0x5a97ec1b, SUC_lexicon_tags_1 },
    { 0xf617ec1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d7ec20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b97ec27, SUC_lexicon_tags_32 },
    { 0x137ec27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f77ec2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x16d7ec30, SUC_lexicon_tags_17 },
    { 0x7417ec31, SUC_lexicon_tags_17 },
    { 0x3d57ec30, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6777ec35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c97ec40, SUC_lexicon_tags_32 },
    { 0x377ec41, SUC_lexicon_tags_1 },
    { 0x8097ec42, SUC_lexicon_tags_9 },
    { 0x9837ec43, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x44d7ec45, SUC_lexicon_tags_32 },
    { 0x62d7ec46, SUC_lexicon_tags_39 },
    { 0x2e97ec46, SUC_lexicon_tags_23 },
    { 0xaad7ec48, SUC_lexicon_tags_23 },
    { 0x7cb7ec49, SUC_lexicon_tags_17 },
    { 0x5bb7ec4a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4737ec4c, SUC_lexicon_tags_2 },
    { 0xcdf7ec4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a77ec4f, SUC_lexicon_tags_37 },
    { 0x7fb7ec50, SUC_lexicon_tags_112 },
    { 0xfef7ec51, SUC_lexicon_tags_1 },
    { 0x1317ec51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec57ec55, SUC_lexicon_tags_25 },
    { 0xa537ec56, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd7ec59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf457ec5b, SUC_lexicon_tags_37 },
    { 0x8ab7ec5c, SUC_lexicon_tags_81 },
    { 0xe5f7ec5d, SUC_lexicon_tags_17 },
    { 0x21d7ec5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7ec65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd7ec6a, SUC_lexicon_tags_14 },
    { 0x817ec6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46b7ec6d, SUC_lexicon_tags_2 },
    { 0x37d7ec6d, SUC_lexicon_tags_17 },
    { 0x93d7ec6f, SUC_lexicon_tags_17 },
    { 0x16d7ec6f, SUC_lexicon_tags_25 },
    { 0x8cf7ec71, SUC_lexicon_tags_47 },
    { 0xd297ec71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57ec75, SUC_lexicon_tags_23 },
    { 0xe897ec76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf7ec7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa717ec80, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc77ec89, SUC_lexicon_tags_39 },
    { 0x557ec8a, SUC_lexicon_tags_3 },
    { 0xffd7ec8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33b7ec8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9257ec90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a97ec93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f37ec96, SUC_lexicon_tags_77 },
    { 0x32d7ec97, SUC_lexicon_tags_28 },
    { 0x6eb7ec98, SUC_lexicon_tags_11 },
    { 0x37f7ec96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f7ec9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc217ec9e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5c17eca0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e57eca7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1cf7eca9, SUC_lexicon_tags_26 },
    { 0xf017ecaa, SUC_lexicon_tags_25 },
    { 0xcf17ecab, SUC_lexicon_tags_27 },
    { 0xe677ecac, SUC_lexicon_tags_3 },
    { 0x6f77ecad, SUC_lexicon_tags_26 },
    { 0x1cb7ecaa, SUC_lexicon_tags_49 },
    { 0x1fb7ecaf, SUC_lexicon_tags_68 },
    { 0x2d77ecab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe577ecb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4217ecb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde37ecb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7757ecbb, SUC_lexicon_tags_25 },
    { 0x317ecbc, SUC_lexicon_tags_9 },
    { 0x7b97ecbb, SUC_lexicon_tags_30 },
    { 0xebd7ecbe, SUC_lexicon_tags_19 },
    { 0x98d7ecbc, SUC_lexicon_tags_37 },
    { 0x2097ecbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed7ecc4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf37ecc7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9497ecca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5637eccc, SUC_lexicon_tags_11 },
    { 0x9d77eccd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf537eccf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8937ecd1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1597ecd3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b17ecdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92f7ecdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5157ece0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff77eceb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd7ecef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72b7ecf1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d57ecf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff7ecfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe557ecfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c17ed01, SUC_lexicon_tags_12 },
    { 0x21b7ed01, SUC_lexicon_tags_63 },
    { 0xfad7ed03, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab7ed06, SUC_lexicon_tags_38 },
    { 0xf2d7ed06, SUC_lexicon_tags_60 },
    { 0x2ad7ed08, SUC_lexicon_tags_0 },
    { 0xa517ed09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xced7ed0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f7ed0f, SUC_lexicon_tags_17 },
    { 0xc657ed10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x22b7ed12, SUC_lexicon_tags_32 },
    { 0x51b7ed13, SUC_lexicon_tags_37 },
    { 0x5777ed13, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb7ed18, SUC_lexicon_tags_19 },
    { 0xd497ed18, SUC_lexicon_tags_64 },
    { 0x7e97ed1a, SUC_lexicon_tags_129 },
    { 0x4bf7ed19, SUC_lexicon_tags_63 },
    { 0x2a97ed1c, SUC_lexicon_tags_25 },
    { 0x7697ed1c, SUC_lexicon_tags_27 },
    { 0x3337ed1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5617ed22, SUC_lexicon_tags_35 },
    { 0x2757ed23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4217ed25, SUC_lexicon_tags_56 },
    { 0xb197ed26, SUC_lexicon_tags_8 },
    { 0x6077ed27, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf817ed2d, SUC_lexicon_tags_68 },
    { 0x90f7ed2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b77ed31, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b7ed37, SUC_lexicon_tags_17 },
    { 0xf637ed37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe677ed3c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6337ed40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d17ed43, SUC_lexicon_tags_23 },
    { 0xfdb7ed44, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7757ed49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3717ed4b, SUC_lexicon_tags_63 },
    { 0x6117ed4b, SUC_lexicon_tags_15 },
    { 0x2077ed4c, SUC_lexicon_tags_2 },
    { 0x3f37ed4b, SUC_lexicon_tags_37 },
    { 0xedf7ed4c, SUC_lexicon_tags_37 },
    { 0xe77ed50, SUC_lexicon_tags_88 },
    { 0x7657ed4f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1097ed54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9777ed56, SUC_lexicon_tags_7 },
    { 0x9b17ed57, SUC_lexicon_tags_61 },
    { 0xb577ed57, SUC_lexicon_tags_10 },
    { 0xe357ed59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xea77ed5b, SUC_lexicon_tags_15 },
    { 0x6af7ed5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa0b7ed5e, SUC_lexicon_tags_19 },
    { 0x1797ed5f, SUC_lexicon_tags_39 },
    { 0x89d7ed5f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6137ed64, SUC_lexicon_tags_23 },
    { 0xad77ed64, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f37ed68, SUC_lexicon_tags_34 },
    { 0x7497ed68, SUC_lexicon_tags_17 },
    { 0x4537ed69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57ed6e, SUC_lexicon_tags_63 },
    { 0x7fb7ed6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b7ed72, SUC_lexicon_tags_17 },
    { 0xe97ed72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d7ed77, SUC_lexicon_tags_15 },
    { 0xd597ed78, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6037ed88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1497ed94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f7ed98, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb7ed9b, SUC_lexicon_tags_37 },
    { 0xc6b7ed9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc777ed9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f7eda5, SUC_lexicon_tags_1 },
    { 0x5ef7eda6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa437eda9, SUC_lexicon_tags_9 },
    { 0x9957edaa, SUC_lexicon_tags_27 },
    { 0xa137edab, SUC_lexicon_tags_45 },
    { 0xaab7eda9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d7edb1, SUC_lexicon_tags_15 },
    { 0x4e97edb2, SUC_lexicon_tags_7 },
    { 0x3117edb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb7edb6, SUC_lexicon_tags_40 },
    { 0xa837edb7, SUC_lexicon_tags_8 },
    { 0x9357edb6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbf7edba, SUC_lexicon_tags_15 },
    { 0x5717edba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a17edbd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xba17edbf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9257edc1, SUC_lexicon_tags_92 },
    { 0x9657edc2, SUC_lexicon_tags_24 },
    { 0x5457edc3, SUC_lexicon_tags_2 },
    { 0x66b7edc4, SUC_lexicon_tags_17 },
    { 0x85f7edc5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557edcb, SUC_lexicon_tags_2 },
    { 0x9197edcc, SUC_lexicon_tags_28 },
    { 0x5d97edcb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d7edd0, SUC_lexicon_tags_161 },
    { 0x45b7edd0, SUC_lexicon_tags_7 },
    { 0x4c77edd0, SUC_lexicon_tags_43 },
    { 0x4677edd1, SUC_lexicon_tags_31 },
    { 0xa717edd3, SUC_lexicon_tags_37 },
    { 0x2c57edd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca57eddf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb37ede1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c77ede6, SUC_lexicon_tags_46 },
    { 0x30d7ede7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5677edeb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf957edee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9f7edf0, SUC_lexicon_tags_17 },
    { 0xb937edf1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb877edf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f7edf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d97edfb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbf97edfd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc077ee07, SUC_lexicon_tags_17 },
    { 0xc0d7ee07, SUC_lexicon_tags_11 },
    { 0x72f7ee09, SUC_lexicon_tags_37 },
    { 0xfbb7ee09, SUC_lexicon_tags_25 },
    { 0x5b97ee09, SUC_lexicon_tags_7 },
    { 0x54d7ee0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9d7ee0e, SUC_lexicon_tags_19 },
    { 0xe5d7ee0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d7ee15, SUC_lexicon_tags_11 },
    { 0x8d37ee16, SUC_lexicon_tags_7 },
    { 0x5b37ee17, SUC_lexicon_tags_15 },
    { 0xc337ee16, SUC_lexicon_tags_45 },
    { 0xd617ee19, SUC_lexicon_tags_25 },
    { 0x3e37ee1a, SUC_lexicon_tags_84 },
    { 0xa17ee19, SUC_lexicon_tags_15 },
    { 0x2997ee1b, SUC_lexicon_tags_12 },
    { 0x8297ee15, SUC_lexicon_tags_17 },
    { 0x8277ee15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5517ee22, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb657ee24, SUC_lexicon_tags_12 },
    { 0x6237ee24, SUC_lexicon_tags_30 },
    { 0x3037ee26, SUC_lexicon_tags_15 },
    { 0xd0d7ee27, SUC_lexicon_tags_7 },
    { 0x6b37ee28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f7ee2c, SUC_lexicon_tags_15 },
    { 0xa277ee2d, SUC_lexicon_tags_30 },
    { 0x97d7ee2c, SUC_lexicon_tags_33 },
    { 0xee97ee2f, SUC_lexicon_tags_37 },
    { 0xb357ee2f, SUC_lexicon_tags_30 },
    { 0x357ee31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d7ee33, SUC_lexicon_tags_17 },
    { 0xf137ee34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf17ee37, SUC_lexicon_tags_114 },
    { 0x3317ee38, SUC_lexicon_tags_29 },
    { 0xf9b7ee37, SUC_lexicon_tags_66 },
    { 0x9437ee37, SUC_lexicon_tags_27 },
    { 0x2d77ee3b, SUC_lexicon_tags_39 },
    { 0x1c77ee3b, SUC_lexicon_tags_7 },
    { 0x85d7ee3d, SUC_lexicon_tags_39 },
    { 0x7ab7ee3e, SUC_lexicon_tags_8 },
    { 0x7c17ee3d, SUC_lexicon_tags_17 },
    { 0x7d77ee40, SUC_lexicon_tags_17 },
    { 0xce77ee37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd17ee43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2617ee47, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc37ee4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf957ee57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8517ee5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d7ee61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f7ee68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a77ee6e, SUC_lexicon_tags_25 },
    { 0x2037ee6f, SUC_lexicon_tags_15 },
    { 0x8af7ee70, SUC_lexicon_tags_1 },
    { 0x8f37ee71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xebd7ee73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b37ee78, SUC_lexicon_tags_17 },
    { 0x4b37ee79, SUC_lexicon_tags_11 },
    { 0xcaf7ee79, SUC_lexicon_tags_7 },
    { 0x6937ee7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a17ee7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f57ee85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc917ee87, SUC_lexicon_tags_12 },
    { 0x74b7ee88, SUC_lexicon_tags_233 },
    { 0x6e57ee89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7277ee8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xadb7ee8d, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b7ee90, SUC_lexicon_tags_45 },
    { 0x5a37ee91, SUC_lexicon_tags_414 },
    { 0x2b77ee92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc697ee98, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb7ee9f, SUC_lexicon_tags_9 },
    { 0x10f7eea0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317eea6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xecb7eea8, SUC_lexicon_tags_35 },
    { 0xe3b7eea9, SUC_lexicon_tags_3 },
    { 0x197eea9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf7eeae, SUC_lexicon_tags_23 },
    { 0x5297eeaf, SUC_lexicon_tags_63 },
    { 0xc197eeb0, SUC_lexicon_tags_17 },
    { 0xc2b7eeb1, SUC_lexicon_tags_38 },
    { 0x85b7eeb1, SUC_lexicon_tags_39 },
    { 0x49b7eeb0, SUC_lexicon_tags_7 },
    { 0x817eeb3, SUC_lexicon_tags_19 },
    { 0x1db7eeb0, SUC_lexicon_tags_19 },
    { 0x3537eeb0, SUC_lexicon_tags_295 },
    { 0x3917eeb7, SUC_lexicon_tags_29 },
    { 0x3e37eeb8, SUC_lexicon_tags_1 },
    { 0x3b17eeb9, SUC_lexicon_tags_37 },
    { 0xd697eeba, SUC_lexicon_tags_37 },
    { 0x337eebb, SUC_lexicon_tags_27 },
    { 0x5b77eebc, SUC_lexicon_tags_2 },
    { 0x11b7eebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9477eebf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dd7eec1, SUC_lexicon_tags_2 },
    { 0x87b7eec1, SUC_lexicon_tags_12 },
    { 0xd9d7eec3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd37eec6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd737eec9, SUC_lexicon_tags_30 },
    { 0x75b7eeca, SUC_lexicon_tags_250 },
    { 0xfb57eecb, SUC_lexicon_tags_37 },
    { 0x3657eecc, SUC_lexicon_tags_31 },
    { 0x25b7eecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d7eed7, SUC_lexicon_tags_15 },
    { 0x99f7eed7, SUC_lexicon_tags_63 },
    { 0x9337eed7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a97eedd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817eee2, SUC_lexicon_tags_12 },
    { 0x7bd7eee2, SUC_lexicon_tags_37 },
    { 0xa9b7eee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb77eee6, SUC_lexicon_tags_2 },
    { 0x2e77eee7, SUC_lexicon_tags_26 },
    { 0x9737eee8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9837eeeb, SUC_lexicon_tags_37 },
    { 0x91d7eeec, SUC_lexicon_tags_47 },
    { 0x3f97eeed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6557eeef, SUC_lexicon_tags_37 },
    { 0xe577eef0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfcd7eef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1837eef4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d57eef8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cb7eefa, SUC_lexicon_tags_17 },
    { 0x54d7eefa, SUC_lexicon_tags_12 },
    { 0x8e57eefc, SUC_lexicon_tags_2 },
    { 0xd8f7eefd, SUC_lexicon_tags_6 },
    { 0xce37eefd, SUC_lexicon_tags_12 },
    { 0xb997eefb, SUC_lexicon_tags_23 },
    { 0x7977ef00, SUC_lexicon_tags_17 },
    { 0x3017ef01, SUC_lexicon_tags_2 },
    { 0xeb17eefe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3297ef05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc7f7ef07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b7ef0a, SUC_lexicon_tags_25 },
    { 0x1197ef0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb877ef0d, SUC_lexicon_tags_61 },
    { 0x7457ef0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6337ef10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa197ef12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed7ef16, SUC_lexicon_tags_65 },
    { 0x7277ef17, SUC_lexicon_tags_17 },
    { 0xa1f7ef18, SUC_lexicon_tags_45 },
    { 0x5f77ef19, SUC_lexicon_tags_37 },
    { 0xc677ef1a, SUC_lexicon_tags_8 },
    { 0xd297ef18, SUC_lexicon_tags_1 },
    { 0xac37ef1a, SUC_lexicon_tags_1584 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef7ef24, SUC_lexicon_tags_48 },
    { 0xaaf7ef24, SUC_lexicon_tags_31 },
    { 0xec17ef26, SUC_lexicon_tags_148 },
    { 0x61b7ef27, SUC_lexicon_tags_7 },
    { 0x61d7ef28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4197ef34, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b7ef38, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d7ef3d, SUC_lexicon_tags_30 },
    { 0xfb17ef3d, SUC_lexicon_tags_7 },
    { 0xc077ef3d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xa6f7ef41, SUC_lexicon_tags_17 },
    { 0xbe57ef42, SUC_lexicon_tags_1 },
    { 0xe037ef43, SUC_lexicon_tags_15 },
    { 0x9c37ef44, SUC_lexicon_tags_1 },
    { 0x66f7ef43, SUC_lexicon_tags_31 },
    { 0x66f7ef46, SUC_lexicon_tags_170 },
    { 0x9697ef46, SUC_lexicon_tags_66 },
    { 0x85f7ef48, SUC_lexicon_tags_12 },
    { 0x237ef48, SUC_lexicon_tags_2 },
    { 0xf6b7ef43, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd997ef50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f7ef53, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xce77ef55, SUC_lexicon_tags_7 },
    { 0xe197ef56, SUC_lexicon_tags_2 },
    { 0xbf7ef56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84d7ef59, SUC_lexicon_tags_66 },
    { 0x9297ef5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad7ef5d, SUC_lexicon_tags_26 },
    { 0xe937ef5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3777ef60, SUC_lexicon_tags_14 },
    { 0x9f97ef60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9f7ef63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e57ef67, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b7ef6c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2357ef73, SUC_lexicon_tags_15 },
    { 0x31b7ef73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7717ef76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd197ef7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d7ef82, SUC_lexicon_tags_0 },
    { 0x5d17ef83, SUC_lexicon_tags_37 },
    { 0xd8d7ef83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2b97ef86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6637ef88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb37ef8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde17ef8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72d7ef90, SUC_lexicon_tags_160 },
    { 0xb797ef91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf817ef97, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9117ef9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d7ef9e, SUC_lexicon_tags_55 },
    { 0x75b7ef9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd37efa2, SUC_lexicon_tags_31 },
    { 0xd0f7efa3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2bd7efa5, SUC_lexicon_tags_68 },
    { 0x2b37efa6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x797efa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8897efac, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d37efaf, SUC_lexicon_tags_15 },
    { 0xedd7efb0, SUC_lexicon_tags_228 },
    { 0x3597efaf, SUC_lexicon_tags_12 },
    { 0xb517efb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfb7efb4, SUC_lexicon_tags_28 },
    { 0xe357efb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde77efb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde57efbd, SUC_lexicon_tags_30 },
    { 0xfad7efbe, SUC_lexicon_tags_26 },
    { 0x67b7efbe, SUC_lexicon_tags_25 },
    { 0xd177efc0, SUC_lexicon_tags_55 },
    { 0x5937efbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8497efc3, SUC_lexicon_tags_14 },
    { 0x8057efc4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3757efc8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f7efcc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xea77efcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd517efd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5017efd4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x94d7efd6, SUC_lexicon_tags_7 },
    { 0x6977efd7, SUC_lexicon_tags_2 },
    { 0xa417efd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8377efda, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe197efdd, SUC_lexicon_tags_28 },
    { 0x7f97efdd, SUC_lexicon_tags_15 },
    { 0xd9d7efdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3997efe3, SUC_lexicon_tags_26 },
    { 0xb1d7efe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb7efe6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a97efea, SUC_lexicon_tags_14 },
    { 0x217efeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe017eff2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d7eff8, SUC_lexicon_tags_60 },
    { 0xfcf7eff9, SUC_lexicon_tags_66 },
    { 0x9377effa, SUC_lexicon_tags_2 },
    { 0xb457effb, SUC_lexicon_tags_29 },
    { 0x14d7effc, SUC_lexicon_tags_17 },
    { 0xd3d7effb, SUC_lexicon_tags_5 },
    { 0x25f7effe, SUC_lexicon_tags_6 },
    { 0xfa77effb, SUC_lexicon_tags_25 },
    { 0xfe77effd, SUC_lexicon_tags_17 },
    { 0xd0b7efff, SUC_lexicon_tags_2 },
    { 0xcf77f002, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1117f005, SUC_lexicon_tags_27 },
    { 0x1b57f006, SUC_lexicon_tags_17 },
    { 0x83b7f005, SUC_lexicon_tags_33 },
    { 0x4897f007, SUC_lexicon_tags_16 },
    { 0x2d17f009, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7417f00e, SUC_lexicon_tags_12 },
    { 0x8857f00f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac97f011, SUC_lexicon_tags_31 },
    { 0x9837f011, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d77f015, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1577f01c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7f020, SUC_lexicon_tags_17 },
    { 0x37d7f021, SUC_lexicon_tags_39 },
    { 0x86d7f022, SUC_lexicon_tags_12 },
    { 0xf437f023, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f7f026, SUC_lexicon_tags_12 },
    { 0x3dd7f027, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d7f02c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb57f033, SUC_lexicon_tags_29 },
    { 0xb2d7f033, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb7f037, SUC_lexicon_tags_1 },
    { 0xbdd7f037, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c97f03c, SUC_lexicon_tags_37 },
    { 0x5e57f03d, SUC_lexicon_tags_39 },
    { 0x7457f03e, SUC_lexicon_tags_1 },
    { 0x7b37f03f, SUC_lexicon_tags_31 },
    { 0x6597f040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8677f042, SUC_lexicon_tags_1 },
    { 0xd157f043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cb7f045, SUC_lexicon_tags_2 },
    { 0x3637f045, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b7f049, SUC_lexicon_tags_17 },
    { 0xa337f04a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa777f04c, SUC_lexicon_tags_25 },
    { 0xd377f04c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f7f050, SUC_lexicon_tags_65 },
    { 0xb897f051, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3ff7f053, SUC_lexicon_tags_63 },
    { 0x5957f054, SUC_lexicon_tags_12 },
    { 0x32b7f053, SUC_lexicon_tags_11 },
    { 0xf097f056, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d17f05a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3837f05c, SUC_lexicon_tags_7 },
    { 0x7cd7f05d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f7f062, SUC_lexicon_tags_43 },
    { 0x2937f063, SUC_lexicon_tags_1 },
    { 0x8757f064, SUC_lexicon_tags_12 },
    { 0x6db7f065, SUC_lexicon_tags_17 },
    { 0xbeb7f065, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa897f06f, SUC_lexicon_tags_15 },
    { 0x9f57f070, SUC_lexicon_tags_47 },
    { 0x7497f071, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3b7f073, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69f7f075, SUC_lexicon_tags_47 },
    { 0xc177f076, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1f7f078, SUC_lexicon_tags_26 },
    { 0x7437f078, SUC_lexicon_tags_11 },
    { 0x5457f079, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa357f07c, SUC_lexicon_tags_12 },
    { 0xc797f07d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9697f082, SUC_lexicon_tags_0 },
    { 0xd897f083, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5577f085, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7977f088, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc837f08e, SUC_lexicon_tags_25 },
    { 0x4777f08f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4d7f091, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd177f093, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7cd7f095, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff7f098, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3fb7f09a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1297f09c, SUC_lexicon_tags_21 },
    { 0x6437f09d, SUC_lexicon_tags_15 },
    { 0x5cb7f09e, SUC_lexicon_tags_25 },
    { 0x9457f09e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x517f0a2, SUC_lexicon_tags_47 },
    { 0x5277f0a2, SUC_lexicon_tags_37 },
    { 0x9657f0a2, SUC_lexicon_tags_17 },
    { 0xecd7f0a4, SUC_lexicon_tags_61 },
    { 0xebd7f0a6, SUC_lexicon_tags_11 },
    { 0xd037f0a7, SUC_lexicon_tags_17 },
    { 0xb857f0a3, SUC_lexicon_tags_30 },
    { 0x1db7f0a2, SUC_lexicon_tags_12 },
    { 0xccf7f0aa, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x34b7f0ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28f7f0ae, SUC_lexicon_tags_7 },
    { 0xe5b7f0ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37f0b2, SUC_lexicon_tags_45 },
    { 0xfc17f0b2, SUC_lexicon_tags_0 },
    { 0xf917f0b4, SUC_lexicon_tags_70 },
    { 0xf297f0b5, SUC_lexicon_tags_37 },
    { 0xfa57f0b4, SUC_lexicon_tags_14 },
    { 0xef17f0b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea37f0b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf657f0bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4fd7f0bd, SUC_lexicon_tags_16 },
    { 0x75f7f0bd, SUC_lexicon_tags_6 },
    { 0x7df7f0bf, SUC_lexicon_tags_4 },
    { 0x7417f0c0, SUC_lexicon_tags_106 },
    { 0x65b7f0c1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b7f0c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6957f0c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef17f0cb, SUC_lexicon_tags_7 },
    { 0xfb17f0cc, SUC_lexicon_tags_49 },
    { 0x3d17f0cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7577f0d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca77f0d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed7f0dc, SUC_lexicon_tags_12 },
    { 0x9137f0dc, SUC_lexicon_tags_60 },
    { 0x9b97f0de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1cd7f0e0, SUC_lexicon_tags_15 },
    { 0xa977f0e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c77f0e5, SUC_lexicon_tags_30 },
    { 0xdb7f0e6, SUC_lexicon_tags_112 },
    { 0xcf77f0e5, SUC_lexicon_tags_37 },
    { 0x9697f0e8, SUC_lexicon_tags_25 },
    { 0x63b7f0e9, SUC_lexicon_tags_18 },
    { 0xdc77f0e5, SUC_lexicon_tags_17 },
    { 0x777f0e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4197f0f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee77f0fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded7f101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x177f103, SUC_lexicon_tags_7 },
    { 0xaa97f104, SUC_lexicon_tags_47 },
    { 0x4c37f105, SUC_lexicon_tags_17 },
    { 0x6fb7f105, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf7f7f108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1437f10a, SUC_lexicon_tags_29 },
    { 0xa317f10b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed57f10d, SUC_lexicon_tags_14 },
    { 0xe737f10e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c17f113, SUC_lexicon_tags_40 },
    { 0x957f114, SUC_lexicon_tags_8 },
    { 0xe217f113, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41d7f117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5737f119, SUC_lexicon_tags_49 },
    { 0x21d7f11a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7957f11c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f7f124, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d37f12a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6957f12d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd917f131, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb477f133, SUC_lexicon_tags_27 },
    { 0x6417f134, SUC_lexicon_tags_2 },
    { 0x66f7f135, SUC_lexicon_tags_15 },
    { 0x4517f135, SUC_lexicon_tags_11 },
    { 0x37f137, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb897f13c, SUC_lexicon_tags_30 },
    { 0x6af7f13d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7637f140, SUC_lexicon_tags_25 },
    { 0xdff7f141, SUC_lexicon_tags_351 },
    { 0xd077f140, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7f7f144, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab77f146, SUC_lexicon_tags_48 },
    { 0x9937f147, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af7f14a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa97f14f, SUC_lexicon_tags_33 },
    { 0x5277f14f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda97f158, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7f15b, SUC_lexicon_tags_10 },
    { 0xf0d7f15c, SUC_lexicon_tags_37 },
    { 0x8a37f15c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd37f160, SUC_lexicon_tags_38 },
    { 0xe637f161, SUC_lexicon_tags_38 },
    { 0x75f7f162, SUC_lexicon_tags_69 },
    { 0x8937f162, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3f57f165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77f169, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7797f16c, SUC_lexicon_tags_37 },
    { 0x7017f16d, SUC_lexicon_tags_47 },
    { 0xb357f16e, SUC_lexicon_tags_17 },
    { 0x6797f16d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5a77f171, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa17f173, SUC_lexicon_tags_11 },
    { 0xf157f174, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1a57f176, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6ed7f178, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d37f17e, SUC_lexicon_tags_37 },
    { 0xe0b7f17f, SUC_lexicon_tags_37 },
    { 0x6c97f17f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf697f182, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f97f184, SUC_lexicon_tags_25 },
    { 0xd117f185, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f7f189, SUC_lexicon_tags_14 },
    { 0x2917f18a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c77f18c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad7f192, SUC_lexicon_tags_17 },
    { 0xa577f193, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7f196, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff7f19f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7557f1a2, SUC_lexicon_tags_91 },
    { 0x67f7f1a3, SUC_lexicon_tags_37 },
    { 0x8e17f1a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b7f1aa, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x98b7f1ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd7f1af, SUC_lexicon_tags_1 },
    { 0xf2f7f1b0, SUC_lexicon_tags_198 },
    { 0x1a37f1b0, SUC_lexicon_tags_27 },
    { 0x8bb7f1b2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc77f1b7, SUC_lexicon_tags_25 },
    { 0xd337f1b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f77f1bb, SUC_lexicon_tags_66 },
    { 0x2397f1bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf7f1c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8537f1c6, SUC_lexicon_tags_7 },
    { 0xdf7f1c7, SUC_lexicon_tags_38 },
    { 0xb6f7f1c8, SUC_lexicon_tags_8 },
    { 0x66d7f1c9, SUC_lexicon_tags_55 },
    { 0x9497f1ca, SUC_lexicon_tags_92 },
    { 0xf7f1cb, SUC_lexicon_tags_11 },
    { 0x5bb7f1c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb7f1cf, SUC_lexicon_tags_25 },
    { 0xda57f1cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb77f1d4, SUC_lexicon_tags_47 },
    { 0xe097f1d5, SUC_lexicon_tags_14 },
    { 0xf057f1d6, SUC_lexicon_tags_25 },
    { 0x1d7f1d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6197f1d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b97f1dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x40b7f1df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f1e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2497f1ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3037f1ef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdcd7f1f1, SUC_lexicon_tags_16 },
    { 0x4f57f1f2, SUC_lexicon_tags_37 },
    { 0xd877f1f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9f7f1f5, SUC_lexicon_tags_11 },
    { 0xcff7f1f5, SUC_lexicon_tags_37 },
    { 0xc677f1f7, SUC_lexicon_tags_39 },
    { 0x2797f1f8, SUC_lexicon_tags_49 },
    { 0x7bd7f1f9, SUC_lexicon_tags_0 },
    { 0x69d7f1f6, SUC_lexicon_tags_7 },
    { 0x11d7f1fb, SUC_lexicon_tags_38 },
    { 0x7a37f1fc, SUC_lexicon_tags_29 },
    { 0xa5d7f1fd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc777f208, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea57f20d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4157f210, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5057f213, SUC_lexicon_tags_25 },
    { 0xbf57f214, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb7f217, SUC_lexicon_tags_15 },
    { 0x16b7f218, SUC_lexicon_tags_9 },
    { 0x7cb7f219, SUC_lexicon_tags_17 },
    { 0x2277f21a, SUC_lexicon_tags_26 },
    { 0xe337f218, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x957f21e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa697f225, SUC_lexicon_tags_28 },
    { 0xe5f7f226, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd57f230, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc037f233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f57f235, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d7f239, SUC_lexicon_tags_33 },
    { 0xfe57f239, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe877f23d, SUC_lexicon_tags_37 },
    { 0x88b7f23d, SUC_lexicon_tags_7 },
    { 0x3197f23f, SUC_lexicon_tags_25 },
    { 0x477f240, SUC_lexicon_tags_17 },
    { 0x3d77f23d, SUC_lexicon_tags_33 },
    { 0x5577f242, SUC_lexicon_tags_10 },
    { 0xbe37f23f, SUC_lexicon_tags_26 },
    { 0x3977f243, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7f247, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a77f24b, SUC_lexicon_tags_5 },
    { 0xea57f24c, SUC_lexicon_tags_2 },
    { 0x77f7f24c, SUC_lexicon_tags_1 },
    { 0x8ab7f24e, SUC_lexicon_tags_25 },
    { 0xa5d7f24f, SUC_lexicon_tags_7 },
    { 0x66b7f250, SUC_lexicon_tags_32 },
    { 0x9817f251, SUC_lexicon_tags_27 },
    { 0xda37f252, SUC_lexicon_tags_17 },
    { 0x59d7f253, SUC_lexicon_tags_33 },
    { 0x6df7f253, SUC_lexicon_tags_37 },
    { 0x10b7f250, SUC_lexicon_tags_7 },
    { 0xe237f253, SUC_lexicon_tags_46 },
    { 0x937f257, SUC_lexicon_tags_25 },
    { 0x4837f258, SUC_lexicon_tags_7 },
    { 0xc3b7f259, SUC_lexicon_tags_8 },
    { 0x4357f25a, SUC_lexicon_tags_42 },
    { 0xc4f7f25b, SUC_lexicon_tags_27 },
    { 0x1a37f257, SUC_lexicon_tags_7 },
    { 0xd377f25d, SUC_lexicon_tags_7 },
    { 0xffd7f25c, SUC_lexicon_tags_58 },
    { 0xf0d7f25f, SUC_lexicon_tags_37 },
    { 0xbeb7f260, SUC_lexicon_tags_12 },
    { 0x57f261, SUC_lexicon_tags_30 },
    { 0xbf17f25f, SUC_lexicon_tags_70 },
    { 0xbb17f263, SUC_lexicon_tags_26 },
    { 0xccb7f25d, SUC_lexicon_tags_15 },
    { 0xf077f265, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b17f268, SUC_lexicon_tags_25 },
    { 0xc1b7f269, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d7f26e, SUC_lexicon_tags_468 },
    { 0x4397f26f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5337f273, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfe17f275, SUC_lexicon_tags_7 },
    { 0x4937f276, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x8357f278, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf7f27c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7937f280, SUC_lexicon_tags_49 },
    { 0x88f7f280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b57f287, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc37f289, SUC_lexicon_tags_65 },
    { 0x1a57f289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef57f28e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc17f293, SUC_lexicon_tags_8 },
    { 0x1057f294, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3157f299, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6097f29b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa957f29d, SUC_lexicon_tags_14 },
    { 0xfb57f29e, SUC_lexicon_tags_31 },
    { 0x3157f29f, SUC_lexicon_tags_15 },
    { 0xacb7f2a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22b7f2a2, SUC_lexicon_tags_14 },
    { 0xf257f2a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9437f2a6, SUC_lexicon_tags_26 },
    { 0x3d7f2a6, SUC_lexicon_tags_37 },
    { 0x4857f2a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x70d7f2aa, SUC_lexicon_tags_14 },
    { 0x2f17f2ab, SUC_lexicon_tags_5 },
    { 0xc4b7f2aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2217f2ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3157f2b2, SUC_lexicon_tags_47 },
    { 0x18d7f2b3, SUC_lexicon_tags_6 },
    { 0x85d7f2b4, SUC_lexicon_tags_17 },
    { 0x6bd7f2b3, SUC_lexicon_tags_12 },
    { 0xbb37f2b6, SUC_lexicon_tags_12 },
    { 0xb117f2b7, SUC_lexicon_tags_29 },
    { 0xb217f2b6, SUC_lexicon_tags_21 },
    { 0x137f2b9, SUC_lexicon_tags_39 },
    { 0xc17f2b9, SUC_lexicon_tags_25 },
    { 0x7077f2b9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df7f2be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2df7f2c0, SUC_lexicon_tags_21 },
    { 0x33d7f2c1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a17f2c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6db7f2ca, SUC_lexicon_tags_33 },
    { 0x9457f2cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7f2cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4257f2d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b7f2d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbeb7f2d7, SUC_lexicon_tags_30 },
    { 0x9cf7f2d8, SUC_lexicon_tags_15 },
    { 0x8f7f2d9, SUC_lexicon_tags_8 },
    { 0x5ef7f2d8, SUC_lexicon_tags_23 },
    { 0xb757f2db, SUC_lexicon_tags_37 },
    { 0x4d37f2db, SUC_lexicon_tags_46 },
    { 0x6997f2db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad17f2e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb077f2e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2617f2ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb7f2ed, SUC_lexicon_tags_2 },
    { 0xce97f2ed, SUC_lexicon_tags_11 },
    { 0xe3b7f2ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b7f2f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f37f301, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x24b7f303, SUC_lexicon_tags_17 },
    { 0x9197f304, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe637f307, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc397f30f, SUC_lexicon_tags_12 },
    { 0x93b7f30f, SUC_lexicon_tags_121 },
    { 0x2857f310, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed7f314, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3697f316, SUC_lexicon_tags_15 },
    { 0xd857f316, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb7f31b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d7f325, SUC_lexicon_tags_12 },
    { 0x13b7f326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac17f32c, SUC_lexicon_tags_387 },
    { 0x71b7f32d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23d7f32f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe337f334, SUC_lexicon_tags_9 },
    { 0x6337f335, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe97f337, SUC_lexicon_tags_25 },
    { 0x5bf7f338, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7f33f, SUC_lexicon_tags_91 },
    { 0x4eb7f340, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb77f345, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4c37f347, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d7f34a, SUC_lexicon_tags_33 },
    { 0x6657f34b, SUC_lexicon_tags_28 },
    { 0xd557f34a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5eb7f34e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7f357, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd557f359, SUC_lexicon_tags_2 },
    { 0x2a77f35a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x417f35d, SUC_lexicon_tags_1 },
    { 0x4377f35e, SUC_lexicon_tags_37 },
    { 0xa17f35f, SUC_lexicon_tags_7 },
    { 0x89f7f35f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeeb7f362, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f7f366, SUC_lexicon_tags_0 },
    { 0x3b37f367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4437f369, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b7f36c, SUC_lexicon_tags_45 },
    { 0xd877f36d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9037f378, SUC_lexicon_tags_2 },
    { 0x9957f379, SUC_lexicon_tags_17 },
    { 0xd477f37a, SUC_lexicon_tags_1010 },
    { 0x4937f379, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2937f37e, SUC_lexicon_tags_31 },
    { 0xe117f37f, SUC_lexicon_tags_1 },
    { 0xd357f37e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7977f382, SUC_lexicon_tags_15 },
    { 0x4cd7f383, SUC_lexicon_tags_38 },
    { 0x77d7f382, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd7f387, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x41f7f389, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f7f38e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfab7f390, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcaf7f392, SUC_lexicon_tags_33 },
    { 0xa557f393, SUC_lexicon_tags_47 },
    { 0xd977f393, SUC_lexicon_tags_34 },
    { 0x8557f395, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad7f3a2, SUC_lexicon_tags_2 },
    { 0x3db7f3a3, SUC_lexicon_tags_30 },
    { 0xc8b7f3a3, SUC_lexicon_tags_17 },
    { 0xb157f3a5, SUC_lexicon_tags_27 },
    { 0x1597f3a6, SUC_lexicon_tags_46 },
    { 0xb037f3a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f7f3ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff7f3b2, SUC_lexicon_tags_2 },
    { 0xeb97f3b3, SUC_lexicon_tags_66 },
    { 0x917f3b3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf57f3b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa477f3ba, SUC_lexicon_tags_47 },
    { 0x1f77f3ba, SUC_lexicon_tags_38 },
    { 0x6857f3ba, SUC_lexicon_tags_26 },
    { 0xe1d7f3ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb17f3c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb7f3c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b17f3cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2077f3cf, SUC_lexicon_tags_25 },
    { 0x8af7f3d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef7f3d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cd7f3d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd817f3df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1b7f3e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6197f3e3, SUC_lexicon_tags_12 },
    { 0x7337f3e3, SUC_lexicon_tags_29 },
    { 0x7297f3e5, SUC_lexicon_tags_1 },
    { 0x7d97f3e6, SUC_lexicon_tags_19 },
    { 0x3697f3e7, SUC_lexicon_tags_17 },
    { 0xb17f3e7, SUC_lexicon_tags_25 },
    { 0x9197f3e9, SUC_lexicon_tags_4 },
    { 0x8197f3e7, SUC_lexicon_tags_0 },
    { 0x17d7f3e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3217f3ef, SUC_lexicon_tags_27 },
    { 0xc37f3ef, SUC_lexicon_tags_15 },
    { 0xfcb7f3ef, SUC_lexicon_tags_29 },
    { 0x5497f3f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d7f3f6, SUC_lexicon_tags_33 },
    { 0xb4d7f3f7, SUC_lexicon_tags_179 },
    { 0xe537f3f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd957f3fc, SUC_lexicon_tags_7 },
    { 0xdb7f3fd, SUC_lexicon_tags_11 },
    { 0xaa97f3fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5217f403, SUC_lexicon_tags_60 },
    { 0xa117f404, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf7f40c, SUC_lexicon_tags_71 },
    { 0x8cb7f40d, SUC_lexicon_tags_118 },
    { 0xe217f40e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3477f413, SUC_lexicon_tags_37 },
    { 0xb297f414, SUC_lexicon_tags_2 },
    { 0xc817f414, SUC_lexicon_tags_88 },
    { 0xfbd7f415, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad7f419, SUC_lexicon_tags_38 },
    { 0xcd97f41a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd7f424, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b57f426, SUC_lexicon_tags_37 },
    { 0x8d7f427, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7f42b, SUC_lexicon_tags_11 },
    { 0x1df7f42c, SUC_lexicon_tags_37 },
    { 0xd97f42d, SUC_lexicon_tags_70 },
    { 0x9637f42e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf7f433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a97f436, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4197f438, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e77f43b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f17f43e, SUC_lexicon_tags_11 },
    { 0x5457f43f, SUC_lexicon_tags_48 },
    { 0x3677f440, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3b97f442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b7f447, SUC_lexicon_tags_238 },
    { 0x9797f448, SUC_lexicon_tags_25 },
    { 0xf5b7f447, SUC_lexicon_tags_27 },
    { 0x5dd7f449, SUC_lexicon_tags_15 },
    { 0xea37f448, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb7f450, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7f458, SUC_lexicon_tags_39 },
    { 0xdb7f459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9117f45b, SUC_lexicon_tags_15 },
    { 0xff77f45b, SUC_lexicon_tags_23 },
    { 0x6f77f45d, SUC_lexicon_tags_32 },
    { 0x9597f45e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2397f461, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3077f465, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9337f467, SUC_lexicon_tags_1 },
    { 0x2657f468, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb7f46c, SUC_lexicon_tags_8 },
    { 0x1f17f46d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d7f475, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6637f478, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3437f47a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6417f47d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd17f47f, SUC_lexicon_tags_13 },
    { 0xca57f47f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf697f486, SUC_lexicon_tags_57 },
    { 0xa3f7f487, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8477f489, SUC_lexicon_tags_7 },
    { 0x8817f489, SUC_lexicon_tags_11 },
    { 0x54d7f48b, SUC_lexicon_tags_15 },
    { 0xd657f48a, SUC_lexicon_tags_17 },
    { 0x3237f48a, SUC_lexicon_tags_12 },
    { 0x9517f48e, SUC_lexicon_tags_15 },
    { 0x2f77f48e, SUC_lexicon_tags_19 },
    { 0xa397f48f, SUC_lexicon_tags_11 },
    { 0xa817f48f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f37f493, SUC_lexicon_tags_25 },
    { 0xe0b7f494, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef7f498, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f7f49f, SUC_lexicon_tags_21 },
    { 0x54d7f4a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4d57f4a2, SUC_lexicon_tags_1 },
    { 0xa197f4a2, SUC_lexicon_tags_65 },
    { 0x5b77f4a2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f7f4ab, SUC_lexicon_tags_30 },
    { 0x117f4ac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xea57f4ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3097f4b1, SUC_lexicon_tags_17 },
    { 0xb5d7f4b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd117f4b5, SUC_lexicon_tags_37 },
    { 0xa357f4b6, SUC_lexicon_tags_77 },
    { 0x1437f4b7, SUC_lexicon_tags_11 },
    { 0x1e77f4b8, SUC_lexicon_tags_37 },
    { 0xa2f7f4b9, SUC_lexicon_tags_45 },
    { 0xcbb7f4ba, SUC_lexicon_tags_10 },
    { 0x49b7f4ba, SUC_lexicon_tags_11 },
    { 0x2457f4b9, SUC_lexicon_tags_17 },
    { 0xc97f4b5, SUC_lexicon_tags_37 },
    { 0xb7d7f4be, SUC_lexicon_tags_15 },
    { 0x6877f4be, SUC_lexicon_tags_43 },
    { 0xb2d7f4be, SUC_lexicon_tags_6 },
    { 0x1117f4c1, SUC_lexicon_tags_30 },
    { 0xf5b7f4b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d7f4c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa077f4d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dd7f4d3, SUC_lexicon_tags_55 },
    { 0xce37f4d4, SUC_lexicon_tags_13 },
    { 0x11b7f4d5, SUC_lexicon_tags_29 },
    { 0x60f7f4d5, SUC_lexicon_tags_15 },
    { 0x96f7f4d7, SUC_lexicon_tags_37 },
    { 0xdeb7f4d8, SUC_lexicon_tags_14 },
    { 0x9f77f4d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1157f4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5357f4df, SUC_lexicon_tags_32 },
    { 0x4977f4e0, SUC_lexicon_tags_37 },
    { 0xa277f4e1, SUC_lexicon_tags_37 },
    { 0x1617f4e0, SUC_lexicon_tags_15 },
    { 0xcb57f4e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7f4e6, SUC_lexicon_tags_12 },
    { 0x15f7f4e7, SUC_lexicon_tags_15 },
    { 0x6637f4e8, SUC_lexicon_tags_17 },
    { 0xa6f7f4e9, SUC_lexicon_tags_15 },
    { 0xab77f4e7, SUC_lexicon_tags_65 },
    { 0xaeb7f4ea, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3597f4ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x737f4ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c37f4f3, SUC_lexicon_tags_2 },
    { 0x2c57f4f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2117f4f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f57f4f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefd7f4fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd617f4fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9257f501, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb7f506, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe777f508, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1717f50b, SUC_lexicon_tags_37 },
    { 0xe977f50c, SUC_lexicon_tags_7 },
    { 0x1f7f50d, SUC_lexicon_tags_17 },
    { 0x2cd7f50b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d7f514, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbeb7f516, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d7f51b, SUC_lexicon_tags_46 },
    { 0x4b7f51c, SUC_lexicon_tags_32 },
    { 0x8517f51b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd377f522, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed7f526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7117f52c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc97f53d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b37f541, SUC_lexicon_tags_43 },
    { 0x2f57f542, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8417f546, SUC_lexicon_tags_25 },
    { 0x4cf7f547, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb417f54c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b7f554, SUC_lexicon_tags_65 },
    { 0xdb97f555, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d77f55d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8717f560, SUC_lexicon_tags_25 },
    { 0xdf77f561, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa897f564, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a97f569, SUC_lexicon_tags_1 },
    { 0x22f7f56a, SUC_lexicon_tags_39 },
    { 0xc217f569, SUC_lexicon_tags_11 },
    { 0xa1f7f56c, SUC_lexicon_tags_26 },
    { 0xaa77f56d, SUC_lexicon_tags_50 },
    { 0x12f7f56e, SUC_lexicon_tags_34 },
    { 0x6f57f56f, SUC_lexicon_tags_25 },
    { 0x9cd7f570, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a17f575, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x97d7f577, SUC_lexicon_tags_26 },
    { 0xf8d7f577, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4317f57b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4237f582, SUC_lexicon_tags_7 },
    { 0x32d7f583, SUC_lexicon_tags_47 },
    { 0x2cb7f584, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8657f586, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf37f58d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5357f590, SUC_lexicon_tags_8 },
    { 0x7fb7f591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6117f593, SUC_lexicon_tags_49 },
    { 0x8cf7f594, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d7f598, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab7f59b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc957f59e, SUC_lexicon_tags_1 },
    { 0x75d7f59f, SUC_lexicon_tags_11 },
    { 0x4fd7f5a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4057f5a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc057f5a6, SUC_lexicon_tags_9 },
    { 0xb977f5a7, SUC_lexicon_tags_37 },
    { 0x34f7f5a8, SUC_lexicon_tags_37 },
    { 0xf57f5a9, SUC_lexicon_tags_84 },
    { 0x5077f5a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf717f5ae, SUC_lexicon_tags_2 },
    { 0xc217f5ae, SUC_lexicon_tags_37 },
    { 0x8cf7f5b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd637f5b3, SUC_lexicon_tags_4 },
    { 0xa997f5b4, SUC_lexicon_tags_2 },
    { 0x8117f5b5, SUC_lexicon_tags_15 },
    { 0xd117f5b6, SUC_lexicon_tags_13 },
    { 0x7917f5b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47b7f5b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3597f5bc, SUC_lexicon_tags_11 },
    { 0x9f77f5bd, SUC_lexicon_tags_17 },
    { 0x5f7f5bc, SUC_lexicon_tags_29 },
    { 0x6677f5bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd97f5c6, SUC_lexicon_tags_84 },
    { 0xeb37f5c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe057f5cb, SUC_lexicon_tags_15 },
    { 0x1af7f5cc, SUC_lexicon_tags_78 },
    { 0xfb7f5cd, SUC_lexicon_tags_23 },
    { 0xf1f7f5ce, SUC_lexicon_tags_17 },
    { 0xc6d7f5cf, SUC_lexicon_tags_29 },
    { 0x7597f5cf, SUC_lexicon_tags_26 },
    { 0x6117f5cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8697f5d3, SUC_lexicon_tags_13 },
    { 0xb737f5d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb697f5d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa7d7f5d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef7f5db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x777f5dd, SUC_lexicon_tags_14 },
    { 0x597f5de, SUC_lexicon_tags_26 },
    { 0xc5f7f5df, SUC_lexicon_tags_49 },
    { 0x2c97f5df, SUC_lexicon_tags_37 },
    { 0x6f17f5e1, SUC_lexicon_tags_39 },
    { 0xe757f5e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f7f5e5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3337f5ef, SUC_lexicon_tags_17 },
    { 0x8017f5ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7497f5f3, SUC_lexicon_tags_91 },
    { 0x6977f5f3, SUC_lexicon_tags_44 },
    { 0xc517f5f4, SUC_lexicon_tags_13 },
    { 0xc7f7f5f4, SUC_lexicon_tags_47 },
    { 0xce97f5f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f7f5fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb357f5fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d7f605, SUC_lexicon_tags_10 },
    { 0xdf17f605, SUC_lexicon_tags_37 },
    { 0x67f7f605, SUC_lexicon_tags_3 },
    { 0x6817f607, SUC_lexicon_tags_30 },
    { 0x4657f609, SUC_lexicon_tags_21 },
    { 0xd417f60a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9657f60c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb7f60f, SUC_lexicon_tags_15 },
    { 0xc2b7f610, SUC_lexicon_tags_30 },
    { 0x3637f610, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b7f614, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d17f619, SUC_lexicon_tags_30 },
    { 0x70b7f619, SUC_lexicon_tags_63 },
    { 0x7ff7f61a, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0x5f7f61d, SUC_lexicon_tags_32 },
    { 0xc3f7f61e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff17f622, SUC_lexicon_tags_15 },
    { 0x2457f623, SUC_lexicon_tags_57 },
    { 0x7957f624, SUC_lexicon_tags_36 },
    { 0x9537f625, SUC_lexicon_tags_13 },
    { 0xfcd7f626, SUC_lexicon_tags_56 },
    { 0x9ab7f626, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb7f62d, SUC_lexicon_tags_37 },
    { 0x5f97f62e, SUC_lexicon_tags_11 },
    { 0x30f7f62e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5657f633, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df7f637, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b7f63b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a57f63e, SUC_lexicon_tags_79 },
    { 0xb3f7f63f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc077f641, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc517f643, SUC_lexicon_tags_15 },
    { 0xe597f643, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6637f646, SUC_lexicon_tags_45 },
    { 0xc4d7f647, SUC_lexicon_tags_32 },
    { 0x6657f648, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced7f64d, SUC_lexicon_tags_17 },
    { 0x2b77f64d, SUC_lexicon_tags_17 },
    { 0x2a57f64f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9f7f651, SUC_lexicon_tags_47 },
    { 0xeed7f651, SUC_lexicon_tags_15 },
    { 0x9df7f651, SUC_lexicon_tags_11 },
    { 0xee97f654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f7f657, SUC_lexicon_tags_14 },
    { 0x1157f657, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d17f65a, SUC_lexicon_tags_28 },
    { 0x597f65a, SUC_lexicon_tags_68 },
    { 0x4ad7f65c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd97f65f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9197f663, SUC_lexicon_tags_43 },
    { 0x76b7f663, SUC_lexicon_tags_1 },
    { 0x8897f663, SUC_lexicon_tags_8 },
    { 0xcc57f666, SUC_lexicon_tags_63 },
    { 0x1b77f666, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf697f66d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d7f670, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9737f672, SUC_lexicon_tags_17 },
    { 0x5777f673, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f67a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a37f67d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d57f686, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad7f689, SUC_lexicon_tags_32 },
    { 0xf297f68a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d7f691, SUC_lexicon_tags_11 },
    { 0x3117f692, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2757f694, SUC_lexicon_tags_5 },
    { 0x94f7f695, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5677f697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4397f699, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9797f6a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17f6a4, SUC_lexicon_tags_2 },
    { 0xa1d7f6a5, SUC_lexicon_tags_47 },
    { 0x4af7f6a6, SUC_lexicon_tags_37 },
    { 0x59b7f6a7, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0xd077f6a9, SUC_lexicon_tags_7 },
    { 0xb677f6aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1577f6ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9937f6af, SUC_lexicon_tags_56 },
    { 0x7357f6b0, SUC_lexicon_tags_4 },
    { 0xec57f6b1, SUC_lexicon_tags_27 },
    { 0x79f7f6b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x957f6b5, SUC_lexicon_tags_10 },
    { 0x5897f6b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8957f6b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6177f6bc, SUC_lexicon_tags_37 },
    { 0xa1b7f6bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d97f6c0, SUC_lexicon_tags_25 },
    { 0xa917f6c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f7f6ca, SUC_lexicon_tags_46 },
    { 0xa3f7f6ca, SUC_lexicon_tags_17 },
    { 0xc057f6cb, SUC_lexicon_tags_45 },
    { 0xea77f6cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9357f6cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d7f6d2, SUC_lexicon_tags_9 },
    { 0xd497f6d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1b7f6d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb437f6d8, SUC_lexicon_tags_12 },
    { 0x2d37f6d9, SUC_lexicon_tags_68 },
    { 0x1ab7f6d9, SUC_lexicon_tags_28 },
    { 0x93f7f6d9, SUC_lexicon_tags_15 },
    { 0xcaf7f6db, SUC_lexicon_tags_18 },
    { 0x57d7f6dd, SUC_lexicon_tags_72 },
    { 0x3cb7f6dd, SUC_lexicon_tags_37 },
    { 0x72f7f6da, SUC_lexicon_tags_17 },
    { 0x6fd7f6db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb957f6ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d7f6f1, SUC_lexicon_tags_19 },
    { 0x3b77f6f2, SUC_lexicon_tags_49 },
    { 0x90b7f6f1, SUC_lexicon_tags_17 },
    { 0xbd97f6f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7f6f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4717f6fb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd437f6fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1437f700, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b7f704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9537f708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7337f70d, SUC_lexicon_tags_12 },
    { 0x3c77f70e, SUC_lexicon_tags_17 },
    { 0x56f7f70e, SUC_lexicon_tags_43 },
    { 0x15d7f710, SUC_lexicon_tags_45 },
    { 0xbc17f711, SUC_lexicon_tags_27 },
    { 0xad17f710, SUC_lexicon_tags_47 },
    { 0xb137f713, SUC_lexicon_tags_37 },
    { 0x2a37f714, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd7f717, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x25b7f719, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b7f720, SUC_lexicon_tags_38 },
    { 0x7177f721, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe957f726, SUC_lexicon_tags_15 },
    { 0x7e77f727, SUC_lexicon_tags_81 },
    { 0x1e37f728, SUC_lexicon_tags_42 },
    { 0xe357f726, SUC_lexicon_tags_37 },
    { 0xe817f72a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa477f72c, SUC_lexicon_tags_25 },
    { 0x1197f72c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8957f736, SUC_lexicon_tags_48 },
    { 0x9a17f737, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2cb7f739, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb057f73b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf937f73e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7537f742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb17f744, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8097f74a, SUC_lexicon_tags_43 },
    { 0x70f7f74b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a97f74f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa497f752, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1517f754, SUC_lexicon_tags_112 },
    { 0xe337f754, SUC_lexicon_tags_47 },
    { 0x5457f756, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb017f758, SUC_lexicon_tags_37 },
    { 0xf4d7f758, SUC_lexicon_tags_12 },
    { 0x1577f759, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8797f763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d7f76a, SUC_lexicon_tags_4 },
    { 0xb57f76b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96b7f76f, SUC_lexicon_tags_43 },
    { 0xc597f770, SUC_lexicon_tags_11 },
    { 0xf877f771, SUC_lexicon_tags_37 },
    { 0x74d7f772, SUC_lexicon_tags_69 },
    { 0x76f7f773, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2217f775, SUC_lexicon_tags_21 },
    { 0xec57f775, SUC_lexicon_tags_7 },
    { 0xfbf7f777, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4897f77a, SUC_lexicon_tags_29 },
    { 0x9417f77a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb97f77d, SUC_lexicon_tags_25 },
    { 0x2e17f77e, SUC_lexicon_tags_25 },
    { 0x4797f77d, SUC_lexicon_tags_8 },
    { 0xed37f780, SUC_lexicon_tags_0 },
    { 0x8077f77f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff17f785, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4c37f787, SUC_lexicon_tags_12 },
    { 0xb2f7f788, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9437f793, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe037f79b, SUC_lexicon_tags_46 },
    { 0xc797f79b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5cd7f79e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a57f7a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd97f7a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf97f7a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe717f7ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49f7f7ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7337f7b3, SUC_lexicon_tags_38 },
    { 0x2c37f7b4, SUC_lexicon_tags_25 },
    { 0x6337f7b5, SUC_lexicon_tags_12 },
    { 0x8357f7b4, SUC_lexicon_tags_15 },
    { 0x1617f7b7, SUC_lexicon_tags_20 },
    { 0x57f7f7b6, SUC_lexicon_tags_7 },
    { 0xd117f7b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc377f7be, SUC_lexicon_tags_6 },
    { 0x9ed7f7be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x917f7c1, SUC_lexicon_tags_7 },
    { 0xdbd7f7c1, SUC_lexicon_tags_45 },
    { 0x6b77f7c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf37f7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2057f7c8, SUC_lexicon_tags_61 },
    { 0x78d7f7c9, SUC_lexicon_tags_17 },
    { 0x9b7f7c8, SUC_lexicon_tags_17 },
    { 0x5f97f7cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x90f7f7cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e77f7d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa137f7d4, SUC_lexicon_tags_32 },
    { 0x81f7f7d4, SUC_lexicon_tags_12 },
    { 0x68b7f7d5, SUC_lexicon_tags_37 },
    { 0x8cd7f7d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8297f7d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2997f7dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd057f7e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7b7f7e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f7f7eb, SUC_lexicon_tags_14 },
    { 0x257f7ec, SUC_lexicon_tags_7 },
    { 0x8817f7ec, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d7f7f1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf97f7f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdf7f7fa, SUC_lexicon_tags_63 },
    { 0x1f37f7fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd7f7fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0b7f800, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f7f804, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8957f807, SUC_lexicon_tags_46 },
    { 0xee97f808, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b7f80e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4657f810, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b7f814, SUC_lexicon_tags_7 },
    { 0xc6b7f815, SUC_lexicon_tags_25 },
    { 0x8ad7f816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdbb7f818, SUC_lexicon_tags_37 },
    { 0x2197f819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda57f81d, SUC_lexicon_tags_17 },
    { 0xe7d7f81e, SUC_lexicon_tags_27 },
    { 0xd7d7f81f, SUC_lexicon_tags_7 },
    { 0xf717f820, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3197f826, SUC_lexicon_tags_7 },
    { 0xfb7f827, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba37f82a, SUC_lexicon_tags_65 },
    { 0xe3d7f82b, SUC_lexicon_tags_26 },
    { 0x2897f82c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8957f82e, SUC_lexicon_tags_17 },
    { 0x1d77f82f, SUC_lexicon_tags_44 },
    { 0x26d7f82e, SUC_lexicon_tags_61 },
    { 0x5ff7f831, SUC_lexicon_tags_27 },
    { 0x7f57f832, SUC_lexicon_tags_23 },
    { 0xa4d7f831, SUC_lexicon_tags_12 },
    { 0xd2b7f834, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf7f838, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc17f83a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5677f83c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f7f83f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98f7f841, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe117f843, SUC_lexicon_tags_2 },
    { 0x8df7f844, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9457f848, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70b7f84a, SUC_lexicon_tags_49 },
    { 0xaf97f84b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af7f851, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb97f855, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d77f859, SUC_lexicon_tags_12 },
    { 0x2c97f859, SUC_lexicon_tags_17 },
    { 0x3277f85a, SUC_lexicon_tags_1 },
    { 0xf697f85c, SUC_lexicon_tags_15 },
    { 0x4877f85c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1f7f85f, SUC_lexicon_tags_3 },
    { 0x1cd7f860, SUC_lexicon_tags_1 },
    { 0xaa17f861, SUC_lexicon_tags_25 },
    { 0xf9f7f862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd7f864, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d7f86a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7577f86d, SUC_lexicon_tags_37 },
    { 0x1ad7f86e, SUC_lexicon_tags_17 },
    { 0xd8f7f86d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9177f872, SUC_lexicon_tags_0 },
    { 0x3237f873, SUC_lexicon_tags_14 },
    { 0xe077f872, SUC_lexicon_tags_25 },
    { 0x7f37f873, SUC_lexicon_tags_43 },
    { 0x77b7f876, SUC_lexicon_tags_40 },
    { 0x50f7f872, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d57f882, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c97f886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6877f88a, SUC_lexicon_tags_17 },
    { 0x7b17f88a, SUC_lexicon_tags_15 },
    { 0xc417f88c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c37f88f, SUC_lexicon_tags_31 },
    { 0x7b77f88f, SUC_lexicon_tags_6 },
    { 0xa097f890, SUC_lexicon_tags_17 },
    { 0x7217f890, SUC_lexicon_tags_11 },
    { 0xf597f893, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7837f895, SUC_lexicon_tags_23 },
    { 0xfb57f895, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6897f898, SUC_lexicon_tags_17 },
    { 0xe2b7f899, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde17f89f, SUC_lexicon_tags_82 },
    { 0xcc77f89f, SUC_lexicon_tags_19 },
    { 0x7f7f8a1, SUC_lexicon_tags_17 },
    { 0xa7f7f8a2, SUC_lexicon_tags_25 },
    { 0x3397f8a3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef7f8a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaeb7f8a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e37f8ac, SUC_lexicon_tags_6 },
    { 0xbd57f8ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f37f8b0, SUC_lexicon_tags_5 },
    { 0xdb37f8b1, SUC_lexicon_tags_25 },
    { 0x6bf7f8b2, SUC_lexicon_tags_49 },
    { 0xb517f8b3, SUC_lexicon_tags_31 },
    { 0x9cf7f8b1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f7f8b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b7f8bd, SUC_lexicon_tags_38 },
    { 0x66b7f8be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e77f8c7, SUC_lexicon_tags_46 },
    { 0x73d7f8c8, SUC_lexicon_tags_19 },
    { 0x3f77f8c8, SUC_lexicon_tags_17 },
    { 0x6a37f8c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff7f8cd, SUC_lexicon_tags_23 },
    { 0xea97f8ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb7f8d2, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f37f8d5, SUC_lexicon_tags_31 },
    { 0x2f57f8d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda37f8d8, SUC_lexicon_tags_39 },
    { 0xbdd7f8d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4f7f8db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e57f8e3, SUC_lexicon_tags_27 },
    { 0xc37f8e4, SUC_lexicon_tags_11 },
    { 0x39f7f8e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd77f8e7, SUC_lexicon_tags_2 },
    { 0xa57f8e8, SUC_lexicon_tags_37 },
    { 0x32b7f8e9, SUC_lexicon_tags_23 },
    { 0x3f97f8e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7f8ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f7f8f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2ff7f8f6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d7f8ff, SUC_lexicon_tags_1 },
    { 0x5d37f900, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f7f90b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc697f90d, SUC_lexicon_tags_33 },
    { 0xaad7f90e, SUC_lexicon_tags_25 },
    { 0xe637f90f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8257f913, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf7f916, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d7f91a, SUC_lexicon_tags_8 },
    { 0x86d7f91a, SUC_lexicon_tags_9 },
    { 0xebf7f91c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2537f91e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f57f921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d7f924, SUC_lexicon_tags_9 },
    { 0x99d7f925, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f77f929, SUC_lexicon_tags_60 },
    { 0x66d7f929, SUC_lexicon_tags_27 },
    { 0x52f7f92b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b7f92f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d7f932, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f37f939, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bf7f93b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe257f945, SUC_lexicon_tags_27 },
    { 0xd777f946, SUC_lexicon_tags_17 },
    { 0xb2d7f947, SUC_lexicon_tags_37 },
    { 0xd6f7f946, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e17f94b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed97f94d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb7f953, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b7f956, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f7f959, SUC_lexicon_tags_14 },
    { 0x5bd7f95a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x717f95c, SUC_lexicon_tags_84 },
    { 0xd37f95d, SUC_lexicon_tags_9 },
    { 0x2db7f95c, SUC_lexicon_tags_23 },
    { 0xb537f95d, SUC_lexicon_tags_39 },
    { 0x4e57f95c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b17f969, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b7f96d, SUC_lexicon_tags_10 },
    { 0x6777f96e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f7f972, SUC_lexicon_tags_22 },
    { 0xabb7f973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7f977, SUC_lexicon_tags_82 },
    { 0xf4f7f978, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9157f980, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5c17f982, SUC_lexicon_tags_12 },
    { 0xcbb7f982, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf777f986, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35b7f988, SUC_lexicon_tags_17 },
    { 0xad17f989, SUC_lexicon_tags_29 },
    { 0x84f7f98a, SUC_lexicon_tags_15 },
    { 0x65f7f989, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x58f7f98d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a77f98f, SUC_lexicon_tags_2 },
    { 0xd4b7f990, SUC_lexicon_tags_7 },
    { 0x897f98f, SUC_lexicon_tags_12 },
    { 0x9957f990, SUC_lexicon_tags_77 },
    { 0xd637f993, SUC_lexicon_tags_6 },
    { 0x34f7f994, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c57f996, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa9b7f998, SUC_lexicon_tags_16 },
    { 0xadf7f999, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2d7f99b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb7f99f, SUC_lexicon_tags_29 },
    { 0x3ff7f9a0, SUC_lexicon_tags_0 },
    { 0x2d17f9a1, SUC_lexicon_tags_0 },
    { 0x4d17f9a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef7f9a5, SUC_lexicon_tags_11 },
    { 0xd7f9a5, SUC_lexicon_tags_70 },
    { 0x6597f9a7, SUC_lexicon_tags_1 },
    { 0x1917f9a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9797f9af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9137f9b2, SUC_lexicon_tags_11 },
    { 0x53d7f9b3, SUC_lexicon_tags_7 },
    { 0xfe77f9b4, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xaff7f9b6, SUC_lexicon_tags_15 },
    { 0xb4f7f9b6, SUC_lexicon_tags_406 },
    { 0x21f7f9b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d7f9bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64f7f9be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe457f9c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa037f9c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7f9cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff7f9d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b97f9d2, SUC_lexicon_tags_19 },
    { 0x6237f9d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91f7f9d5, SUC_lexicon_tags_61 },
    { 0x11b7f9d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9057f9d9, SUC_lexicon_tags_47 },
    { 0x59d7f9da, SUC_lexicon_tags_15 },
    { 0x2297f9db, SUC_lexicon_tags_15 },
    { 0x9fb7f9db, SUC_lexicon_tags_57 },
    { 0xbfd7f9dd, SUC_lexicon_tags_69 },
    { 0x1f17f9db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1057f9e2, SUC_lexicon_tags_0 },
    { 0x8bb7f9e3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb177f9e6, SUC_lexicon_tags_14 },
    { 0x9957f9e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9057f9ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7f9ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d7f9f1, SUC_lexicon_tags_15 },
    { 0xe217f9f2, SUC_lexicon_tags_23 },
    { 0xcab7f9f2, SUC_lexicon_tags_27 },
    { 0xca37f9f4, SUC_lexicon_tags_37 },
    { 0xa197f9f4, SUC_lexicon_tags_37 },
    { 0x7397f9f4, SUC_lexicon_tags_140 },
    { 0x50d7f9f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1817f9ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa657fa07, SUC_lexicon_tags_17 },
    { 0xf417fa08, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a37fa0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf57fa11, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e17fa14, SUC_lexicon_tags_37 },
    { 0x5ab7fa14, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d7fa18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7fa1c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7437fa1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa637fa21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf7fa25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad7fa29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc37fa2e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef7fa32, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xea57fa34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf797fa36, SUC_lexicon_tags_29 },
    { 0x8b37fa37, SUC_lexicon_tags_14 },
    { 0x117fa37, SUC_lexicon_tags_34 },
    { 0xcd97fa36, SUC_lexicon_tags_87 },
    { 0x9ff7fa3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd7fa3d, SUC_lexicon_tags_5 },
    { 0xd3f7fa3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6257fa41, SUC_lexicon_tags_33 },
    { 0xa077fa42, SUC_lexicon_tags_7 },
    { 0x1937fa43, SUC_lexicon_tags_25 },
    { 0x4c97fa44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b7fa49, SUC_lexicon_tags_9 },
    { 0xc737fa4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2e37fa4c, SUC_lexicon_tags_19 },
    { 0x5337fa4c, SUC_lexicon_tags_17 },
    { 0xd777fa4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x49b7fa50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d7fa54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed7fa57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d7fa59, SUC_lexicon_tags_17 },
    { 0x777fa5a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6217fa66, SUC_lexicon_tags_15 },
    { 0x1f77fa67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7fa6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa097fa6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6617fa76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1f7fa78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3a77fa7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e97fa7c, SUC_lexicon_tags_7 },
    { 0x8257fa7c, SUC_lexicon_tags_7 },
    { 0xb597fa7e, SUC_lexicon_tags_9 },
    { 0x94d7fa7f, SUC_lexicon_tags_11 },
    { 0x1cd7fa80, SUC_lexicon_tags_43 },
    { 0x6277fa81, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb37fa85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc97fa8c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b7fa8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b7fa93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb7fa96, SUC_lexicon_tags_26 },
    { 0x32d7fa97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf17fa9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa0f7fa9c, SUC_lexicon_tags_38 },
    { 0x94b7fa9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf077fa9f, SUC_lexicon_tags_47 },
    { 0x2057faa0, SUC_lexicon_tags_4 },
    { 0xf637fa9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf357faa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4257faa6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f37faa9, SUC_lexicon_tags_1 },
    { 0xcb37faaa, SUC_lexicon_tags_28 },
    { 0x7c37faab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf57fab0, SUC_lexicon_tags_14 },
    { 0x4557fab1, SUC_lexicon_tags_25 },
    { 0x2e37fab2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd797fab4, SUC_lexicon_tags_243 },
    { 0xbe17fab5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab77fab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3037fabf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9557fac2, SUC_lexicon_tags_19 },
    { 0x5277fac2, SUC_lexicon_tags_60 },
    { 0x7cb7fac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x797fac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb017faca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf7fad1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf7fad6, SUC_lexicon_tags_47 },
    { 0x3137fad6, SUC_lexicon_tags_62 },
    { 0xce37fad7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6237fadf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8777fae7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb7faed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1457faef, SUC_lexicon_tags_27 },
    { 0xca97faef, SUC_lexicon_tags_15 },
    { 0xa957faf1, SUC_lexicon_tags_29 },
    { 0xaf7faf2, SUC_lexicon_tags_29 },
    { 0x8e37faf3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7faf9, SUC_lexicon_tags_29 },
    { 0xeb97faf9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf357faff, SUC_lexicon_tags_15 },
    { 0x9a97fb00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2777fb04, SUC_lexicon_tags_14 },
    { 0xc997fb05, SUC_lexicon_tags_26 },
    { 0x1277fb06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa557fb08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd37fb0a, SUC_lexicon_tags_55 },
    { 0x1417fb0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe57fb11, SUC_lexicon_tags_4 },
    { 0x8d37fb12, SUC_lexicon_tags_10 },
    { 0x14b7fb13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa77fb18, SUC_lexicon_tags_26 },
    { 0x2a57fb18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a17fb1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc077fb26, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad7fb2f, SUC_lexicon_tags_21 },
    { 0x2b17fb2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcd7fb32, SUC_lexicon_tags_89 },
    { 0xe7f7fb32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a97fb36, SUC_lexicon_tags_25 },
    { 0xe5d7fb37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f7fb3b, SUC_lexicon_tags_43 },
    { 0x6157fb3b, SUC_lexicon_tags_7 },
    { 0xc9f7fb3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b7fb41, SUC_lexicon_tags_33 },
    { 0xb537fb42, SUC_lexicon_tags_28 },
    { 0x8417fb43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc237fb46, SUC_lexicon_tags_6 },
    { 0x7dd7fb47, SUC_lexicon_tags_16 },
    { 0xe3f7fb47, SUC_lexicon_tags_15 },
    { 0x85f7fb49, SUC_lexicon_tags_1 },
    { 0x62b7fb46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d7fb55, SUC_lexicon_tags_37 },
    { 0xa317fb55, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9b77fb58, SUC_lexicon_tags_23 },
    { 0x2377fb59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf17fb5d, SUC_lexicon_tags_33 },
    { 0x97f7fb5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe177fb6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4417fb6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d7fb70, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7fb75, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4397fb78, SUC_lexicon_tags_6 },
    { 0xecb7fb79, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9997fb7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3f7fb7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d7fb84, SUC_lexicon_tags_17 },
    { 0xb3f7fb85, SUC_lexicon_tags_47 },
    { 0x917fb86, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed7fb8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x377fb8c, SUC_lexicon_tags_11 },
    { 0x67d7fb8d, SUC_lexicon_tags_37 },
    { 0x8277fb8e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xbc57fb90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb7fb93, SUC_lexicon_tags_38 },
    { 0x60f7fb93, SUC_lexicon_tags_0 },
    { 0x8677fb95, SUC_lexicon_tags_2 },
    { 0x7bd7fb93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd7fb9b, SUC_lexicon_tags_7 },
    { 0x6577fb9b, SUC_lexicon_tags_33 },
    { 0x4b57fb9d, SUC_lexicon_tags_17 },
    { 0x6e57fb9c, SUC_lexicon_tags_37 },
    { 0xceb7fb9f, SUC_lexicon_tags_11 },
    { 0xc6d7fba0, SUC_lexicon_tags_28 },
    { 0x6017fba0, SUC_lexicon_tags_37 },
    { 0xf5b7fb9f, SUC_lexicon_tags_12 },
    { 0x9fd7fb9f, SUC_lexicon_tags_17 },
    { 0x9c37fba4, SUC_lexicon_tags_17 },
    { 0xe097fb9d, SUC_lexicon_tags_23 },
    { 0xe2d7fba1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d17fba8, SUC_lexicon_tags_38 },
    { 0xfb77fba8, SUC_lexicon_tags_25 },
    { 0x58d7fbaa, SUC_lexicon_tags_15 },
    { 0xc277fbab, SUC_lexicon_tags_25 },
    { 0xafd7fbac, SUC_lexicon_tags_7 },
    { 0x8317fbad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed7fbb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9f7fbb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2337fbb4, SUC_lexicon_tags_4 },
    { 0x30d7fbb5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd7fbb8, SUC_lexicon_tags_47 },
    { 0xadb7fbb9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb7fbbd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7fbc0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53f7fbc2, SUC_lexicon_tags_17 },
    { 0xf5b7fbc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97fbc6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd57fbcc, SUC_lexicon_tags_15 },
    { 0x3697fbcc, SUC_lexicon_tags_30 },
    { 0x33d7fbcc, SUC_lexicon_tags_1 },
    { 0x9517fbcf, SUC_lexicon_tags_15 },
    { 0x63d7fbd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x137fbd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1eb7fbd4, SUC_lexicon_tags_28 },
    { 0x88f7fbd5, SUC_lexicon_tags_1 },
    { 0xdc97fbd5, SUC_lexicon_tags_7 },
    { 0xaf57fbd5, SUC_lexicon_tags_11 },
    { 0xa4d7fbd8, SUC_lexicon_tags_34 },
    { 0xab97fbd9, SUC_lexicon_tags_26 },
    { 0x1037fbda, SUC_lexicon_tags_1 },
    { 0x93f7fbd9, SUC_lexicon_tags_2 },
    { 0x37fbdc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde77fbe1, SUC_lexicon_tags_7 },
    { 0xeb77fbe1, SUC_lexicon_tags_91 },
    { 0xa517fbe3, SUC_lexicon_tags_25 },
    { 0x6c17fbe4, SUC_lexicon_tags_15 },
    { 0x2397fbe5, SUC_lexicon_tags_25 },
    { 0x2657fbe5, SUC_lexicon_tags_23 },
    { 0xbfd7fbe2, SUC_lexicon_tags_11 },
    { 0x5a57fbe8, SUC_lexicon_tags_26 },
    { 0x5477fbe2, SUC_lexicon_tags_43 },
    { 0x5c97fbe9, SUC_lexicon_tags_29 },
    { 0x1af7fbe5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfb77fbed, SUC_lexicon_tags_46 },
    { 0x9c97fbee, SUC_lexicon_tags_38 },
    { 0xe677fbed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9917fbf2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e37fbf5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b7fbff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d7fc02, SUC_lexicon_tags_37 },
    { 0xf9d7fc02, SUC_lexicon_tags_31 },
    { 0xf3b7fc02, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xead7fc06, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a97fc0b, SUC_lexicon_tags_87 },
    { 0x3dd7fc0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f77fc0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d57fc11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7477fc15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8497fc17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcb17fc19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d7fc1d, SUC_lexicon_tags_27 },
    { 0xa357fc1e, SUC_lexicon_tags_30 },
    { 0x76d7fc1d, SUC_lexicon_tags_1 },
    { 0x9557fc1f, SUC_lexicon_tags_17 },
    { 0xa4d7fc21, SUC_lexicon_tags_15 },
    { 0x9537fc22, SUC_lexicon_tags_37 },
    { 0x337fc23, SUC_lexicon_tags_27 },
    { 0xf677fc21, SUC_lexicon_tags_27 },
    { 0x76f7fc25, SUC_lexicon_tags_12 },
    { 0xc297fc23, SUC_lexicon_tags_3 },
    { 0x6cb7fc27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6897fc29, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd17fc2d, SUC_lexicon_tags_25 },
    { 0xe577fc2d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x7217fc30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc077fc33, SUC_lexicon_tags_20 },
    { 0xdd37fc33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c17fc3e, SUC_lexicon_tags_49 },
    { 0x4dd7fc3f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf7fc42, SUC_lexicon_tags_27 },
    { 0x20d7fc43, SUC_lexicon_tags_29 },
    { 0x44f7fc42, SUC_lexicon_tags_17 },
    { 0x2617fc43, SUC_lexicon_tags_29 },
    { 0x7557fc44, SUC_lexicon_tags_80 },
    { 0x317fc43, SUC_lexicon_tags_8 },
    { 0x5bf7fc48, SUC_lexicon_tags_11 },
    { 0xa797fc46, SUC_lexicon_tags_27 },
    { 0x1ed7fc48, SUC_lexicon_tags_31 },
    { 0xd397fc4b, SUC_lexicon_tags_16 },
    { 0x4b77fc47, SUC_lexicon_tags_17 },
    { 0x6917fc4d, SUC_lexicon_tags_74 },
    { 0xc437fc4e, SUC_lexicon_tags_1 },
    { 0x6fd7fc4f, SUC_lexicon_tags_30 },
    { 0x7cb7fc4f, SUC_lexicon_tags_26 },
    { 0x6277fc51, SUC_lexicon_tags_8 },
    { 0x5277fc4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd7fc55, SUC_lexicon_tags_11 },
    { 0x6137fc56, SUC_lexicon_tags_19 },
    { 0x15b7fc57, SUC_lexicon_tags_17 },
    { 0x4257fc55, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8957fc5b, SUC_lexicon_tags_60 },
    { 0x2577fc5c, SUC_lexicon_tags_1 },
    { 0xdb7fc5d, SUC_lexicon_tags_7 },
    { 0x6b37fc5e, SUC_lexicon_tags_19 },
    { 0xe0d7fc5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2d7fc61, SUC_lexicon_tags_39 },
    { 0x2797fc62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9057fc64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f7fc68, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xb757fc6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3997fc6e, SUC_lexicon_tags_29 },
    { 0xe057fc6f, SUC_lexicon_tags_87 },
    { 0x9d17fc70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c37fc72, SUC_lexicon_tags_47 },
    { 0x2657fc73, SUC_lexicon_tags_0 },
    { 0x57b7fc72, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8df7fc76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa537fc78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2437fc7c, SUC_lexicon_tags_7 },
    { 0x57b7fc7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x37fc7f, SUC_lexicon_tags_2 },
    { 0xbe17fc80, SUC_lexicon_tags_7 },
    { 0x2a17fc81, SUC_lexicon_tags_37 },
    { 0x9017fc82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed7fc8b, SUC_lexicon_tags_11 },
    { 0xe597fc8c, SUC_lexicon_tags_43 },
    { 0xefb7fc8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d7fc90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7337fc94, SUC_lexicon_tags_0 },
    { 0x4f57fc95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3517fc98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c97fc9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3157fc9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf517fca0, SUC_lexicon_tags_11 },
    { 0x3857fca1, SUC_lexicon_tags_65 },
    { 0x2a17fca1, SUC_lexicon_tags_17 },
    { 0x74f7fca0, SUC_lexicon_tags_15 },
    { 0x25d7fca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa517fca7, SUC_lexicon_tags_47 },
    { 0xa0b7fca8, SUC_lexicon_tags_14 },
    { 0x40d7fca7, SUC_lexicon_tags_43 },
    { 0xbbf7fca8, SUC_lexicon_tags_7 },
    { 0xb9f7fcab, SUC_lexicon_tags_47 },
    { 0xb557fcac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f7fcb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd017fcb2, SUC_lexicon_tags_14 },
    { 0x3817fcb3, SUC_lexicon_tags_37 },
    { 0x2617fcb4, SUC_lexicon_tags_12 },
    { 0xbd37fcb3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7937fcb7, SUC_lexicon_tags_14 },
    { 0x9bb7fcb8, SUC_lexicon_tags_45 },
    { 0x2cf7fcb9, SUC_lexicon_tags_25 },
    { 0xb237fcb8, SUC_lexicon_tags_12 },
    { 0xc37fcba, SUC_lexicon_tags_23 },
    { 0xd797fcbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9017fcbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa497fcc1, SUC_lexicon_tags_63 },
    { 0x6197fcc2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd7fcc6, SUC_lexicon_tags_29 },
    { 0xb6b7fcc6, SUC_lexicon_tags_15 },
    { 0x58f7fcc8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57fccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcad7fcce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f7fcd4, SUC_lexicon_tags_11 },
    { 0x9577fcd5, SUC_lexicon_tags_28 },
    { 0x42b7fcd6, SUC_lexicon_tags_6 },
    { 0x6cf7fcd6, SUC_lexicon_tags_7 },
    { 0x3d57fcd8, SUC_lexicon_tags_7 },
    { 0x7af7fcd8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf97fcdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b7fce1, SUC_lexicon_tags_35 },
    { 0x7b17fce2, SUC_lexicon_tags_47 },
    { 0x9ef7fce3, SUC_lexicon_tags_55 },
    { 0x717fce4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x35f7fce6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f7fceb, SUC_lexicon_tags_12 },
    { 0x4577fceb, SUC_lexicon_tags_32 },
    { 0xdbb7fcec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb077fcf4, SUC_lexicon_tags_12 },
    { 0x7717fcf5, SUC_lexicon_tags_37 },
    { 0xc237fcf6, SUC_lexicon_tags_0 },
    { 0x317fcf6, SUC_lexicon_tags_15 },
    { 0xf397fcf8, SUC_lexicon_tags_11 },
    { 0x91f7fcf8, SUC_lexicon_tags_33 },
    { 0x5e17fcfa, SUC_lexicon_tags_28 },
    { 0x7557fcf4, SUC_lexicon_tags_25 },
    { 0x8577fcf6, SUC_lexicon_tags_11 },
    { 0xdb97fcfd, SUC_lexicon_tags_9 },
    { 0x6257fcfe, SUC_lexicon_tags_17 },
    { 0xf9d7fcf6, SUC_lexicon_tags_37 },
    { 0xde97fd00, SUC_lexicon_tags_25 },
    { 0xb0b7fd01, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc737fd06, SUC_lexicon_tags_13 },
    { 0x32d7fd07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd37fd09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f7fd0d, SUC_lexicon_tags_11 },
    { 0x8e57fd0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d7fd12, SUC_lexicon_tags_60 },
    { 0xb177fd12, SUC_lexicon_tags_14 },
    { 0xb797fd12, SUC_lexicon_tags_45 },
    { 0x88f7fd12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8477fd18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xab97fd1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4297fd21, SUC_lexicon_tags_12 },
    { 0xa157fd22, SUC_lexicon_tags_26 },
    { 0x7657fd23, SUC_lexicon_tags_11 },
    { 0xd3d7fd24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8417fd26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbc77fd28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5d7fd2a, SUC_lexicon_tags_12 },
    { 0xac37fd2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf7fd30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f7fd35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e77fd37, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d37fd41, SUC_lexicon_tags_6 },
    { 0x76b7fd41, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f7fd49, SUC_lexicon_tags_17 },
    { 0x3eb7fd49, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1857fd50, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x62d7fd52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1277fd54, SUC_lexicon_tags_15 },
    { 0x9ef7fd55, SUC_lexicon_tags_37 },
    { 0x2437fd56, SUC_lexicon_tags_1 },
    { 0x6a17fd54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb57fd59, SUC_lexicon_tags_7 },
    { 0x5ef7fd59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb397fd5f, SUC_lexicon_tags_6 },
    { 0x9fd7fd60, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8d37fd62, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b7fd66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe117fd68, SUC_lexicon_tags_66 },
    { 0x9d17fd69, SUC_lexicon_tags_7 },
    { 0xf2d7fd6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f7fd6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb6f7fd6f, SUC_lexicon_tags_22 },
    { 0xe8f7fd70, SUC_lexicon_tags_12 },
    { 0xf7b7fd70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x28b7fd73, SUC_lexicon_tags_7 },
    { 0x5197fd74, SUC_lexicon_tags_9 },
    { 0xe617fd75, SUC_lexicon_tags_60 },
    { 0xef37fd75, SUC_lexicon_tags_26 },
    { 0xf757fd74, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x31b7fd79, SUC_lexicon_tags_2 },
    { 0xc8d7fd79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb917fd7c, SUC_lexicon_tags_7 },
    { 0xc7b7fd7d, SUC_lexicon_tags_37 },
    { 0x2597fd7e, SUC_lexicon_tags_15 },
    { 0x8c57fd7e, SUC_lexicon_tags_65 },
    { 0xa77fd7c, SUC_lexicon_tags_37 },
    { 0xebd7fd81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d7fd89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe817fd8b, SUC_lexicon_tags_37 },
    { 0x1917fd8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfff7fd8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd57fd92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97d7fd94, SUC_lexicon_tags_27 },
    { 0x8e97fd95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb17fd97, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e57fd9f, SUC_lexicon_tags_11 },
    { 0x5017fda0, SUC_lexicon_tags_7 },
    { 0xd917fd9f, SUC_lexicon_tags_66 },
    { 0x30d7fda2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb017fdaa, SUC_lexicon_tags_27 },
    { 0x857fdaa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd7fdae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x41b7fdb0, SUC_lexicon_tags_11 },
    { 0xbd7fdb1, SUC_lexicon_tags_25 },
    { 0x3ef7fdb2, SUC_lexicon_tags_17 },
    { 0x3d57fdb2, SUC_lexicon_tags_15 },
    { 0xeed7fdb4, SUC_lexicon_tags_1 },
    { 0xbb77fdb5, SUC_lexicon_tags_49 },
    { 0xae97fdb6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e57fdb9, SUC_lexicon_tags_31 },
    { 0x7e57fdb9, SUC_lexicon_tags_26 },
    { 0xf877fdbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd717fdbf, SUC_lexicon_tags_14 },
    { 0xdd37fdc0, SUC_lexicon_tags_2 },
    { 0xd497fdbf, SUC_lexicon_tags_16 },
    { 0xe757fdc0, SUC_lexicon_tags_151 },
    { 0x1e97fdc3, SUC_lexicon_tags_2 },
    { 0x6737fdc3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d7fdca, SUC_lexicon_tags_17 },
    { 0x12b7fdca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1d77fdcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf997fdd1, SUC_lexicon_tags_11 },
    { 0x5c57fdd1, SUC_lexicon_tags_46 },
    { 0xe877fdd3, SUC_lexicon_tags_37 },
    { 0x3f7fdd4, SUC_lexicon_tags_17 },
    { 0x3517fdd5, SUC_lexicon_tags_45 },
    { 0x1d77fdd5, SUC_lexicon_tags_19 },
    { 0xab57fdd7, SUC_lexicon_tags_19 },
    { 0xca97fdd8, SUC_lexicon_tags_27 },
    { 0x6b7fdd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7957fddc, SUC_lexicon_tags_49 },
    { 0x9737fddc, SUC_lexicon_tags_12 },
    { 0x5557fdde, SUC_lexicon_tags_37 },
    { 0xf4b7fdde, SUC_lexicon_tags_25 },
    { 0x277fde0, SUC_lexicon_tags_0 },
    { 0x9d57fde1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7017fde6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3977fde9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fb7fdeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b7fdee, SUC_lexicon_tags_68 },
    { 0xbdb7fdef, SUC_lexicon_tags_47 },
    { 0xf597fdf0, SUC_lexicon_tags_12 },
    { 0xb517fdf0, SUC_lexicon_tags_38 },
    { 0xe2f7fdf1, SUC_lexicon_tags_49 },
    { 0x3cb7fdf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d7fdf6, SUC_lexicon_tags_12 },
    { 0x6937fdf7, SUC_lexicon_tags_17 },
    { 0xb1d7fdf6, SUC_lexicon_tags_4 },
    { 0xe977fdf9, SUC_lexicon_tags_60 },
    { 0xdab7fdf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa97fe01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3277fe05, SUC_lexicon_tags_13 },
    { 0x7c97fe06, SUC_lexicon_tags_22 },
    { 0x3537fe07, SUC_lexicon_tags_2 },
    { 0x8677fe08, SUC_lexicon_tags_63 },
    { 0xc257fe09, SUC_lexicon_tags_25 },
    { 0xe2f7fe0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa077fe0c, SUC_lexicon_tags_47 },
    { 0xb6f7fe0c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2a77fe0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f17fe15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd937fe17, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb017fe1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7857fe21, SUC_lexicon_tags_17 },
    { 0xf457fe22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc177fe24, SUC_lexicon_tags_1 },
    { 0xa977fe24, SUC_lexicon_tags_84 },
    { 0x3c77fe26, SUC_lexicon_tags_46 },
    { 0x4417fe27, SUC_lexicon_tags_1032 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c97fe2a, SUC_lexicon_tags_33 },
    { 0x7b7fe2b, SUC_lexicon_tags_37 },
    { 0x1f57fe2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d7fe2f, SUC_lexicon_tags_10 },
    { 0xa777fe30, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9957fe35, SUC_lexicon_tags_23 },
    { 0xd997fe36, SUC_lexicon_tags_11 },
    { 0x1077fe36, SUC_lexicon_tags_77 },
    { 0x7117fe38, SUC_lexicon_tags_75 },
    { 0xe937fe35, SUC_lexicon_tags_42 },
    { 0xa557fe3a, SUC_lexicon_tags_23 },
    { 0xf3b7fe3b, SUC_lexicon_tags_37 },
    { 0x3257fe3a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4897fe3e, SUC_lexicon_tags_29 },
    { 0x16f7fe3f, SUC_lexicon_tags_27 },
    { 0x957fe3e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9497fe44, SUC_lexicon_tags_15 },
    { 0x8397fe45, SUC_lexicon_tags_42 },
    { 0x9837fe46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cb7fe48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6677fe4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb97fe50, SUC_lexicon_tags_49 },
    { 0x9237fe51, SUC_lexicon_tags_7 },
    { 0x7f37fe52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f7fe59, SUC_lexicon_tags_2 },
    { 0x20d7fe59, SUC_lexicon_tags_4 },
    { 0xa697fe59, SUC_lexicon_tags_17 },
    { 0x8d57fe5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b77fe62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b7fe67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb37fe69, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbd17fe6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f77fe6e, SUC_lexicon_tags_6 },
    { 0x8ff7fe6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d17fe73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3dd7fe75, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e97fe77, SUC_lexicon_tags_25 },
    { 0x3777fe78, SUC_lexicon_tags_17 },
    { 0x4db7fe79, SUC_lexicon_tags_7 },
    { 0xa317fe78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf917fe7c, SUC_lexicon_tags_25 },
    { 0x4497fe7d, SUC_lexicon_tags_37 },
    { 0x39d7fe7d, SUC_lexicon_tags_1 },
    { 0x4797fe7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21b7fe81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee97fe83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c57fe88, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5097fe8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3277fe8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb7fe92, SUC_lexicon_tags_1 },
    { 0x9517fe93, SUC_lexicon_tags_0 },
    { 0x5437fe92, SUC_lexicon_tags_33 },
    { 0x52d7fe95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5d7fe97, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4037fe9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad37fe9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1277fea2, SUC_lexicon_tags_25 },
    { 0xa917fea3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d7feac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e17feaf, SUC_lexicon_tags_63 },
    { 0xb457feb0, SUC_lexicon_tags_15 },
    { 0x9497feb1, SUC_lexicon_tags_17 },
    { 0x42d7feb2, SUC_lexicon_tags_0 },
    { 0x8797feaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b7feb6, SUC_lexicon_tags_15 },
    { 0x5e77feb7, SUC_lexicon_tags_37 },
    { 0xc5b7feb6, SUC_lexicon_tags_9 },
    { 0xf6b7feb9, SUC_lexicon_tags_7 },
    { 0x1d7feb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb7febf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f7fec3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4037fec7, SUC_lexicon_tags_23 },
    { 0x7b77fec8, SUC_lexicon_tags_15 },
    { 0xcbd7fec9, SUC_lexicon_tags_1 },
    { 0x8857fec9, SUC_lexicon_tags_7 },
    { 0x8fb7fecb, SUC_lexicon_tags_7 },
    { 0xeeb7fecc, SUC_lexicon_tags_7 },
    { 0xd6d7fecd, SUC_lexicon_tags_11 },
    { 0x4b97fecc, SUC_lexicon_tags_4 },
    { 0xd8d7fecf, SUC_lexicon_tags_25 },
    { 0xa7b7fecc, SUC_lexicon_tags_63 },
    { 0x5197fed1, SUC_lexicon_tags_2 },
    { 0xd3f7fed2, SUC_lexicon_tags_11 },
    { 0x7bf7feca, SUC_lexicon_tags_37 },
    { 0x7837feca, SUC_lexicon_tags_37 },
    { 0xe217fecc, SUC_lexicon_tags_25 },
    { 0x8ab7fed1, SUC_lexicon_tags_40 },
    { 0xd4d7fed7, SUC_lexicon_tags_25 },
    { 0xec77fed8, SUC_lexicon_tags_12 },
    { 0x9b77fec7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf357fee4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337feea, SUC_lexicon_tags_27 },
    { 0x257feeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa097fef6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb7fef9, SUC_lexicon_tags_11 },
    { 0xa897fefa, SUC_lexicon_tags_1 },
    { 0x6637fefb, SUC_lexicon_tags_43 },
    { 0xbd57fefa, SUC_lexicon_tags_25 },
    { 0x86d7fefd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8137ff01, SUC_lexicon_tags_68 },
    { 0x71f7ff01, SUC_lexicon_tags_12 },
    { 0xba17ff03, SUC_lexicon_tags_151 },
    { 0xcb37ff03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5797ff0b, SUC_lexicon_tags_1 },
    { 0x7d17ff0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed7ff0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d17ff11, SUC_lexicon_tags_80 },
    { 0x84f7ff12, SUC_lexicon_tags_25 },
    { 0xb57ff11, SUC_lexicon_tags_12 },
    { 0x3577ff14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1257ff17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d37ff1a, SUC_lexicon_tags_38 },
    { 0x19f7ff1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5637ff1e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb7ff21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef7ff24, SUC_lexicon_tags_9 },
    { 0x6bf7ff25, SUC_lexicon_tags_11 },
    { 0x2f37ff26, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9537ff2a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe537ff2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4857ff34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x397ff36, SUC_lexicon_tags_60 },
    { 0xc957ff37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef37ff39, SUC_lexicon_tags_17 },
    { 0xf017ff3a, SUC_lexicon_tags_15 },
    { 0x6297ff39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d7ff3f, SUC_lexicon_tags_23 },
    { 0xe517ff40, SUC_lexicon_tags_7 },
    { 0x6e77ff41, SUC_lexicon_tags_45 },
    { 0xabd7ff41, SUC_lexicon_tags_26 },
    { 0x7a17ff43, SUC_lexicon_tags_63 },
    { 0x1c97ff43, SUC_lexicon_tags_29 },
    { 0x7877ff3f, SUC_lexicon_tags_0 },
    { 0xb9d7ff46, SUC_lexicon_tags_25 },
    { 0x8b77ff47, SUC_lexicon_tags_23 },
    { 0xc7b7ff47, SUC_lexicon_tags_12 },
    { 0xc2d7ff49, SUC_lexicon_tags_12 },
    { 0x5497ff46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf977ff4e, SUC_lexicon_tags_19 },
    { 0x7177ff4f, SUC_lexicon_tags_15 },
    { 0x6217ff50, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7057ff52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7b7ff54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd7ff5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf897ff5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d7ff62, SUC_lexicon_tags_23 },
    { 0x2ef7ff62, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe317ff6d, SUC_lexicon_tags_6 },
    { 0x4637ff6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f7ff72, SUC_lexicon_tags_82 },
    { 0x3477ff72, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1997ff75, SUC_lexicon_tags_37 },
    { 0x8bd7ff75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a17ff7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad97ff80, SUC_lexicon_tags_37 },
    { 0x7a17ff80, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8777ff84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2fb7ff86, SUC_lexicon_tags_14 },
    { 0x22b7ff87, SUC_lexicon_tags_37 },
    { 0x3ad7ff88, SUC_lexicon_tags_17 },
    { 0xc437ff89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8177ff8c, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37ff90, SUC_lexicon_tags_21 },
    { 0x8177ff90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd7ff94, SUC_lexicon_tags_37 },
    { 0x2477ff95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5797ff99, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8c37ff9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa597ff9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b7ffa5, SUC_lexicon_tags_37 },
    { 0x44f7ffa6, SUC_lexicon_tags_28 },
    { 0x6c37ffa7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe917ffae, SUC_lexicon_tags_7 },
    { 0xe3b7ffaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d7ffb3, SUC_lexicon_tags_37 },
    { 0xef37ffb4, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b7ffbf, SUC_lexicon_tags_1 },
    { 0xd2d7ffc0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd17ffc4, SUC_lexicon_tags_15 },
    { 0x19b7ffc5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef7ffc9, SUC_lexicon_tags_37 },
    { 0x79b7ffc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d37ffcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb7ffd1, SUC_lexicon_tags_25 },
    { 0x6bb7ffd2, SUC_lexicon_tags_37 },
    { 0x4d57ffd2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x11b7ffd5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b7ffd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2977ffe2, SUC_lexicon_tags_49 },
    { 0x3717ffe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d7ffe6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9677ffea, SUC_lexicon_tags_30 },
    { 0x8457ffeb, SUC_lexicon_tags_43 },
    { 0x1417ffec, SUC_lexicon_tags_0 },
    { 0xae57ffec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2f7ffef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa157fff2, SUC_lexicon_tags_65 },
    { 0xbf97fff2, SUC_lexicon_tags_37 },
    { 0x6cd7fff4, SUC_lexicon_tags_38 },
    { 0x6137fff5, SUC_lexicon_tags_17 },
    { 0xd017fff6, SUC_lexicon_tags_1 },
    { 0x9997fff3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42f7fff9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2597fffd, SUC_lexicon_tags_68 },
    { 0x77f7fffe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28380005, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed80008, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f80015, SUC_lexicon_tags_2 },
    { 0x23780016, SUC_lexicon_tags_79 },
    { 0x20f80017, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x22380019, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ad8001b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8638001e, SUC_lexicon_tags_113 },
    { 0xb0f8001e, SUC_lexicon_tags_11 },
    { 0xa1380020, SUC_lexicon_tags_62 },
    { 0xe1780020, SUC_lexicon_tags_7 },
    { 0x55b80022, SUC_lexicon_tags_19 },
    { 0x7ad80022, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed80028, SUC_lexicon_tags_38 },
    { 0xba980028, SUC_lexicon_tags_25 },
    { 0xa4b8002a, SUC_lexicon_tags_104 },
    { 0xda38002b, SUC_lexicon_tags_8 },
    { 0x40d80029, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed8002f, SUC_lexicon_tags_12 },
    { 0x4f18002f, SUC_lexicon_tags_32 },
    { 0xe2380030, SUC_lexicon_tags_14 },
    { 0xc0380032, SUC_lexicon_tags_12 },
    { 0x70d8002f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e180038, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16580041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39b80043, SUC_lexicon_tags_17 },
    { 0x81b80043, SUC_lexicon_tags_33 },
    { 0x3c380045, SUC_lexicon_tags_1 },
    { 0x15f80045, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c780049, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f78004b, SUC_lexicon_tags_235 },
    { 0x1958004c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd580053, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93380056, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef8005a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7398005d, SUC_lexicon_tags_46 },
    { 0x2298005d, SUC_lexicon_tags_25 },
    { 0x1cb8005f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd80062, SUC_lexicon_tags_31 },
    { 0x61580063, SUC_lexicon_tags_11 },
    { 0xc1180064, SUC_lexicon_tags_46 },
    { 0x97b80065, SUC_lexicon_tags_28 },
    { 0x9ad80064, SUC_lexicon_tags_37 },
    { 0x45780064, SUC_lexicon_tags_37 },
    { 0xe8b80064, SUC_lexicon_tags_22 },
    { 0x28380069, SUC_lexicon_tags_25 },
    { 0x55780065, SUC_lexicon_tags_47 },
    { 0x20f80068, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xf358006d, SUC_lexicon_tags_11 },
    { 0x3d38006d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16980071, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11380075, SUC_lexicon_tags_2 },
    { 0x2d580075, SUC_lexicon_tags_37 },
    { 0x6ed80076, SUC_lexicon_tags_17 },
    { 0x6db80078, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5678007c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf8007f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f80083, SUC_lexicon_tags_99 },
    { 0x94980083, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf9180086, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55b80088, SUC_lexicon_tags_2 },
    { 0x3c580088, SUC_lexicon_tags_87 },
    { 0x40f8008a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4780091, SUC_lexicon_tags_23 },
    { 0xf4380091, SUC_lexicon_tags_11 },
    { 0x6d580093, SUC_lexicon_tags_37 },
    { 0xb4380093, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x98980097, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f78009d, SUC_lexicon_tags_38 },
    { 0x2258009e, SUC_lexicon_tags_1366 },
    { 0xfbf8009f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c5800a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f800a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43800aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1800b0, SUC_lexicon_tags_14 },
    { 0x895800b1, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f800b4, SUC_lexicon_tags_11 },
    { 0x2d7800b4, SUC_lexicon_tags_25 },
    { 0xe5800b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x761800b9, SUC_lexicon_tags_37 },
    { 0x2f3800b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3d9800bc, SUC_lexicon_tags_8 },
    { 0x15800bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d800c1, SUC_lexicon_tags_37 },
    { 0x87800c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x845800c4, SUC_lexicon_tags_104 },
    { 0x377800c5, SUC_lexicon_tags_7 },
    { 0xa01800c6, SUC_lexicon_tags_14 },
    { 0xb7800c5, SUC_lexicon_tags_37 },
    { 0x129800c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x929800ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09800ce, SUC_lexicon_tags_2 },
    { 0xb05800ce, SUC_lexicon_tags_63 },
    { 0x333800ce, SUC_lexicon_tags_61 },
    { 0x935800d1, SUC_lexicon_tags_11 },
    { 0x99f800d2, SUC_lexicon_tags_14 },
    { 0xdb800ce, SUC_lexicon_tags_46 },
    { 0x8bb800d4, SUC_lexicon_tags_37 },
    { 0x26d800d4, SUC_lexicon_tags_10 },
    { 0xa4f800d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9800dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x831800de, SUC_lexicon_tags_34 },
    { 0x767800df, SUC_lexicon_tags_56 },
    { 0xc87800e0, SUC_lexicon_tags_7 },
    { 0xf6f800e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1800e4, SUC_lexicon_tags_87 },
    { 0x48b800e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5f800e7, SUC_lexicon_tags_37 },
    { 0x69b800e8, SUC_lexicon_tags_15 },
    { 0x903800e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85800ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9800ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x533800f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23800f5, SUC_lexicon_tags_11 },
    { 0x531800f6, SUC_lexicon_tags_0 },
    { 0xbbd800f5, SUC_lexicon_tags_29 },
    { 0x4f800f8, SUC_lexicon_tags_13 },
    { 0x149800f9, SUC_lexicon_tags_11 },
    { 0x9ab800f8, SUC_lexicon_tags_9 },
    { 0x60d800fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9dd800fd, SUC_lexicon_tags_60 },
    { 0x75800fd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b980102, SUC_lexicon_tags_45 },
    { 0xe0980103, SUC_lexicon_tags_1 },
    { 0x75b80104, SUC_lexicon_tags_31 },
    { 0x93780104, SUC_lexicon_tags_15 },
    { 0x72180102, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc1580108, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3878010d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f80111, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97580114, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff80119, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cf8011b, SUC_lexicon_tags_46 },
    { 0x52d8011c, SUC_lexicon_tags_33 },
    { 0x6f8011b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b780122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f80125, SUC_lexicon_tags_11 },
    { 0xfd780126, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63780128, SUC_lexicon_tags_7 },
    { 0x28580129, SUC_lexicon_tags_37 },
    { 0xb338012a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9158012c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf518012e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd6d80130, SUC_lexicon_tags_37 },
    { 0x49f80131, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50780133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5980136, SUC_lexicon_tags_7 },
    { 0x5fb80137, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b8013f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f580141, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x15780144, SUC_lexicon_tags_68 },
    { 0xbb180145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88580147, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8580149, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18014e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb80152, SUC_lexicon_tags_15 },
    { 0x8cd80153, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x92180156, SUC_lexicon_tags_43 },
    { 0x9bb80157, SUC_lexicon_tags_30 },
    { 0x92980157, SUC_lexicon_tags_84 },
    { 0x85f80156, SUC_lexicon_tags_11 },
    { 0x7678015a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f98015d, SUC_lexicon_tags_38 },
    { 0xb698015d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x55780160, SUC_lexicon_tags_2 },
    { 0xea980161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c580163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed18016a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8978016c, SUC_lexicon_tags_14 },
    { 0xde98016d, SUC_lexicon_tags_1527 },
    { 0, NULL },
    { 0x21b8016f, SUC_lexicon_tags_15 },
    { 0xb180170, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d80173, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f80177, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0x96380179, SUC_lexicon_tags_83 },
    { 0x6f58017a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd018017c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4180180, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e780182, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xddf80184, SUC_lexicon_tags_68 },
    { 0xe780184, SUC_lexicon_tags_51 },
    { 0x2fd80185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b8018a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf8018d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeb38018f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3ef80191, SUC_lexicon_tags_7 },
    { 0xaff80192, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x59380195, SUC_lexicon_tags_11 },
    { 0xd7580196, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x52380198, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbd8019a, SUC_lexicon_tags_11 },
    { 0xbf78019b, SUC_lexicon_tags_79 },
    { 0xcf8019a, SUC_lexicon_tags_17 },
    { 0x46f8019c, SUC_lexicon_tags_37 },
    { 0xfcd8019e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5801a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x373801a5, SUC_lexicon_tags_25 },
    { 0xb79801a6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x521801a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb85801ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5801ae, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3801b4, SUC_lexicon_tags_735 },
    { 0x407801b5, SUC_lexicon_tags_46 },
    { 0x3df801b5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3801bb, SUC_lexicon_tags_1 },
    { 0x181801bb, SUC_lexicon_tags_11 },
    { 0xcb9801bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb801c0, SUC_lexicon_tags_234 },
    { 0x3f801c0, SUC_lexicon_tags_7 },
    { 0xa77801c2, SUC_lexicon_tags_25 },
    { 0xca1801c2, SUC_lexicon_tags_30 },
    { 0x545801c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963801c8, SUC_lexicon_tags_12 },
    { 0xa4b801c9, SUC_lexicon_tags_15 },
    { 0xec1801c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf29801cc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad801d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241801e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15801e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83801ed, SUC_lexicon_tags_3 },
    { 0xb23801ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d801f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x455801f4, SUC_lexicon_tags_27 },
    { 0xf11801f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x173801f7, SUC_lexicon_tags_17 },
    { 0x7a9801f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb15801fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55801fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25b80200, SUC_lexicon_tags_45 },
    { 0xc3980201, SUC_lexicon_tags_96 },
    { 0x62d80202, SUC_lexicon_tags_2 },
    { 0xce180201, SUC_lexicon_tags_30 },
    { 0x18b80204, SUC_lexicon_tags_38 },
    { 0x4d580205, SUC_lexicon_tags_25 },
    { 0x97780206, SUC_lexicon_tags_25 },
    { 0x4b780207, SUC_lexicon_tags_7 },
    { 0xdb580208, SUC_lexicon_tags_37 },
    { 0xb9d80209, SUC_lexicon_tags_27 },
    { 0xdb58020a, SUC_lexicon_tags_29 },
    { 0xef78020a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9798020f, SUC_lexicon_tags_82 },
    { 0x9980210, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d980213, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd80219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5758021d, SUC_lexicon_tags_11 },
    { 0x3bd8021e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4380223, SUC_lexicon_tags_11 },
    { 0x25580223, SUC_lexicon_tags_43 },
    { 0x13d80225, SUC_lexicon_tags_27 },
    { 0xdf380226, SUC_lexicon_tags_12 },
    { 0xb980225, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf8022d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f80231, SUC_lexicon_tags_17 },
    { 0xccd80231, SUC_lexicon_tags_179 },
    { 0xbb980231, SUC_lexicon_tags_17 },
    { 0x52780234, SUC_lexicon_tags_1 },
    { 0xcfd80234, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfff80237, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6a780239, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2898023f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f80244, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xab580246, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbeb80248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ad8024a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xff58024c, SUC_lexicon_tags_27 },
    { 0xe378024d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96580254, SUC_lexicon_tags_197 },
    { 0x13380255, SUC_lexicon_tags_11 },
    { 0xe4d80256, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f8025c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf80263, SUC_lexicon_tags_27 },
    { 0x5fd80263, SUC_lexicon_tags_28 },
    { 0x7a380265, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50780267, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d8026a, SUC_lexicon_tags_9 },
    { 0x698026b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34780270, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88980275, SUC_lexicon_tags_38 },
    { 0x36180275, SUC_lexicon_tags_15 },
    { 0x18380277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3738027a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f8027e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x77380280, SUC_lexicon_tags_29 },
    { 0xaab80281, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd80285, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x81780287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea58028f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb80295, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10380298, SUC_lexicon_tags_12 },
    { 0x9ab80299, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd8029c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad802a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b802a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe8f802a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xeb5802a7, SUC_lexicon_tags_0 },
    { 0xf89802a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x935802aa, SUC_lexicon_tags_2 },
    { 0xfad802ab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1802af, SUC_lexicon_tags_23 },
    { 0x8d9802b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96b802b2, SUC_lexicon_tags_17 },
    { 0x5f9802b3, SUC_lexicon_tags_25 },
    { 0x649802b4, SUC_lexicon_tags_7 },
    { 0xd8d802b4, SUC_lexicon_tags_25 },
    { 0x9b9802b2, SUC_lexicon_tags_60 },
    { 0x45802b7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d802ba, SUC_lexicon_tags_0 },
    { 0x1d7802bb, SUC_lexicon_tags_46 },
    { 0xaff802bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c3802be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389802c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59802ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8c5802cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df802cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe89802d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01802df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x553802e2, SUC_lexicon_tags_49 },
    { 0xbb802e2, SUC_lexicon_tags_17 },
    { 0x36d802e4, SUC_lexicon_tags_14 },
    { 0x887802e5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad802e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7802ec, SUC_lexicon_tags_12 },
    { 0xd05802ec, SUC_lexicon_tags_15 },
    { 0x12b802ee, SUC_lexicon_tags_11 },
    { 0x1e3802ef, SUC_lexicon_tags_37 },
    { 0x43802f0, SUC_lexicon_tags_14 },
    { 0xf61802f0, SUC_lexicon_tags_37 },
    { 0x395802f1, SUC_lexicon_tags_66 },
    { 0xc69802ed, SUC_lexicon_tags_12 },
    { 0xb5802f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b802f8, SUC_lexicon_tags_11 },
    { 0xae1802f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x337802fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1802fe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca380306, SUC_lexicon_tags_17 },
    { 0xe7980307, SUC_lexicon_tags_32 },
    { 0x10780307, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6198030a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5098030c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xedd8030e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df80311, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x8780314, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b8031b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbc18031d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3580320, SUC_lexicon_tags_1 },
    { 0x60980321, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9df80323, SUC_lexicon_tags_7 },
    { 0xfaf80324, SUC_lexicon_tags_14 },
    { 0x2eb80325, SUC_lexicon_tags_77 },
    { 0xd7180326, SUC_lexicon_tags_17 },
    { 0x9cb80327, SUC_lexicon_tags_25 },
    { 0x6db80328, SUC_lexicon_tags_47 },
    { 0x23180326, SUC_lexicon_tags_25 },
    { 0xbcf80327, SUC_lexicon_tags_1 },
    { 0x54d8032b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x118032e, SUC_lexicon_tags_1 },
    { 0xeb78032f, SUC_lexicon_tags_15 },
    { 0xc618032f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9380337, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7980339, SUC_lexicon_tags_26 },
    { 0xfa78033a, SUC_lexicon_tags_17 },
    { 0x8a98033b, SUC_lexicon_tags_12 },
    { 0x44f8033b, SUC_lexicon_tags_9 },
    { 0x12f8033d, SUC_lexicon_tags_60 },
    { 0xa5d8033e, SUC_lexicon_tags_49 },
    { 0x2998033e, SUC_lexicon_tags_27 },
    { 0xfc180340, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd38034d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4958034f, SUC_lexicon_tags_30 },
    { 0xde38034f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4380352, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb6f80354, SUC_lexicon_tags_69 },
    { 0xdf780355, SUC_lexicon_tags_48 },
    { 0xcb980355, SUC_lexicon_tags_17 },
    { 0xda180357, SUC_lexicon_tags_25 },
    { 0x2b180354, SUC_lexicon_tags_11 },
    { 0x26380355, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x30b8035b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d80360, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81980366, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb1980368, SUC_lexicon_tags_37 },
    { 0x14980369, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdb8036b, SUC_lexicon_tags_0 },
    { 0xb6b8036c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb8036f, SUC_lexicon_tags_47 },
    { 0xd0780370, SUC_lexicon_tags_69 },
    { 0xfaf80371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a980378, SUC_lexicon_tags_19 },
    { 0x88380378, SUC_lexicon_tags_25 },
    { 0xd598037a, SUC_lexicon_tags_49 },
    { 0xcf98037b, SUC_lexicon_tags_17 },
    { 0xcd380378, SUC_lexicon_tags_23 },
    { 0x1c78037a, SUC_lexicon_tags_0 },
    { 0x9c58037d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0380382, SUC_lexicon_tags_14 },
    { 0xb7b80383, SUC_lexicon_tags_7 },
    { 0xf0980384, SUC_lexicon_tags_39 },
    { 0x7a380385, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb80387, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36580389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2d8038b, SUC_lexicon_tags_17 },
    { 0x4198038c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f80391, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x33380393, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48780395, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4780398, SUC_lexicon_tags_1 },
    { 0xe5980398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3eb8039b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1958039f, SUC_lexicon_tags_7 },
    { 0x249803a0, SUC_lexicon_tags_7 },
    { 0xf938039f, SUC_lexicon_tags_42 },
    { 0x253803a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d803a9, SUC_lexicon_tags_1 },
    { 0x4e3803a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c3803ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4df803ae, SUC_lexicon_tags_25 },
    { 0x9803af, SUC_lexicon_tags_49 },
    { 0x5e3803b0, SUC_lexicon_tags_43 },
    { 0x54d803b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b7803b3, SUC_lexicon_tags_2 },
    { 0xbdf803b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1b3803b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9803be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f803c2, SUC_lexicon_tags_7 },
    { 0x4cd803c2, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37803c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1803cc, SUC_lexicon_tags_65 },
    { 0x123803cd, SUC_lexicon_tags_12 },
    { 0x5d3803cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d803d2, SUC_lexicon_tags_32 },
    { 0x905803d3, SUC_lexicon_tags_87 },
    { 0xdef803d3, SUC_lexicon_tags_49 },
    { 0x307803d5, SUC_lexicon_tags_27 },
    { 0x6e3803d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9803d9, SUC_lexicon_tags_14 },
    { 0xb17803da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77803df, SUC_lexicon_tags_14 },
    { 0xf7f803df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad803e5, SUC_lexicon_tags_11 },
    { 0x66b803e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1803e9, SUC_lexicon_tags_71 },
    { 0x599803ea, SUC_lexicon_tags_11 },
    { 0x2a7803e9, SUC_lexicon_tags_1 },
    { 0x8cf803e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93803ef, SUC_lexicon_tags_7 },
    { 0xe7d803ef, SUC_lexicon_tags_27 },
    { 0x783803f1, SUC_lexicon_tags_19 },
    { 0x505803f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645803f6, SUC_lexicon_tags_37 },
    { 0xea7803f6, SUC_lexicon_tags_1 },
    { 0x143803f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7803fe, SUC_lexicon_tags_63 },
    { 0x8c7803ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d380402, SUC_lexicon_tags_12 },
    { 0xb3180403, SUC_lexicon_tags_17 },
    { 0x6f980404, SUC_lexicon_tags_129 },
    { 0x73380404, SUC_lexicon_tags_37 },
    { 0x22b80406, SUC_lexicon_tags_12 },
    { 0x70380406, SUC_lexicon_tags_39 },
    { 0xb3780402, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb58040a, SUC_lexicon_tags_0 },
    { 0x2818040b, SUC_lexicon_tags_167 },
    { 0x3118040c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b80412, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8780414, SUC_lexicon_tags_21 },
    { 0xe2b80415, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f80419, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f8041c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2478041e, SUC_lexicon_tags_7 },
    { 0x4b8041f, SUC_lexicon_tags_2 },
    { 0xffd80420, SUC_lexicon_tags_33 },
    { 0xb978041e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9780425, SUC_lexicon_tags_8 },
    { 0x14580425, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac98042b, SUC_lexicon_tags_2 },
    { 0x9e58042b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe58042f, SUC_lexicon_tags_9 },
    { 0xf4f8042f, SUC_lexicon_tags_45 },
    { 0x10780431, SUC_lexicon_tags_27 },
    { 0x6bb8042f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e980434, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf980436, SUC_lexicon_tags_3 },
    { 0x78b80437, SUC_lexicon_tags_9 },
    { 0x32580436, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f8043b, SUC_lexicon_tags_26 },
    { 0xcdf8043c, SUC_lexicon_tags_1 },
    { 0x4c98043c, SUC_lexicon_tags_31 },
    { 0x5578043d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55380442, SUC_lexicon_tags_1 },
    { 0xf1980443, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcf80445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf180447, SUC_lexicon_tags_12 },
    { 0xf7380448, SUC_lexicon_tags_10 },
    { 0x7b980449, SUC_lexicon_tags_11 },
    { 0x538044a, SUC_lexicon_tags_37 },
    { 0xffb80449, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x95b8044d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c380450, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x87380453, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9478045d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d80460, SUC_lexicon_tags_33 },
    { 0xf8980461, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2b80463, SUC_lexicon_tags_7 },
    { 0x28980464, SUC_lexicon_tags_60 },
    { 0x9fd80465, SUC_lexicon_tags_11 },
    { 0x22f80466, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9980468, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd58046b, SUC_lexicon_tags_10 },
    { 0x73b8046c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe558046e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x60180471, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x23380474, SUC_lexicon_tags_41 },
    { 0x2fd80475, SUC_lexicon_tags_63 },
    { 0xebf80476, SUC_lexicon_tags_38 },
    { 0xe2b80475, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef8047b, SUC_lexicon_tags_35 },
    { 0x7598047b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e980481, SUC_lexicon_tags_47 },
    { 0xe2d80482, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b80486, SUC_lexicon_tags_9 },
    { 0x39780487, SUC_lexicon_tags_28 },
    { 0x9d380487, SUC_lexicon_tags_11 },
    { 0x9e180489, SUC_lexicon_tags_65 },
    { 0xa7380488, SUC_lexicon_tags_26 },
    { 0x1e78048b, SUC_lexicon_tags_26 },
    { 0x6c38048c, SUC_lexicon_tags_25 },
    { 0x43b8048b, SUC_lexicon_tags_25 },
    { 0xc338048c, SUC_lexicon_tags_48 },
    { 0xa2b8048f, SUC_lexicon_tags_12 },
    { 0xbc180490, SUC_lexicon_tags_57 },
    { 0xbeb80490, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6580493, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f780496, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d8049b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b8049f, SUC_lexicon_tags_43 },
    { 0x4a1804a0, SUC_lexicon_tags_25 },
    { 0x38d804a1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x869804a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9804aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xed5804ac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5804b0, SUC_lexicon_tags_30 },
    { 0x8b1804b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65804b5, SUC_lexicon_tags_40 },
    { 0xbe9804b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29804bc, SUC_lexicon_tags_10 },
    { 0x1b1804bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b804c0, SUC_lexicon_tags_8 },
    { 0x91d804c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89804c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8d804c7, SUC_lexicon_tags_1 },
    { 0x9e3804c7, SUC_lexicon_tags_38 },
    { 0xb21804c9, SUC_lexicon_tags_17 },
    { 0xef804ca, SUC_lexicon_tags_37 },
    { 0x27d804ca, SUC_lexicon_tags_37 },
    { 0x4a1804cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28b804ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f804d2, SUC_lexicon_tags_2 },
    { 0xd9b804d2, SUC_lexicon_tags_43 },
    { 0x5b3804d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5804da, SUC_lexicon_tags_19 },
    { 0x28f804db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7d804dd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681804e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef804e4, SUC_lexicon_tags_3 },
    { 0x3eb804e4, SUC_lexicon_tags_30 },
    { 0x851804e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f804e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37804f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a7804f4, SUC_lexicon_tags_7 },
    { 0xf15804f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f804fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d80508, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8580510, SUC_lexicon_tags_0 },
    { 0xec580510, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91580514, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d8051d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3780521, SUC_lexicon_tags_49 },
    { 0x3bf80522, SUC_lexicon_tags_4 },
    { 0x2a980523, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3dd80525, SUC_lexicon_tags_33 },
    { 0x53180526, SUC_lexicon_tags_15 },
    { 0xdb780527, SUC_lexicon_tags_7 },
    { 0x10580527, SUC_lexicon_tags_13 },
    { 0x14780529, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad8052c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ef8052e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b80534, SUC_lexicon_tags_65 },
    { 0x59b80535, SUC_lexicon_tags_354 },
    { 0x8fd80536, SUC_lexicon_tags_5 },
    { 0xee780537, SUC_lexicon_tags_25 },
    { 0x74580538, SUC_lexicon_tags_36 },
    { 0x98d80538, SUC_lexicon_tags_1 },
    { 0x20780537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8558053c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47580540, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3fd80542, SUC_lexicon_tags_2 },
    { 0x62b80543, SUC_lexicon_tags_14 },
    { 0xc0f80543, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcf980546, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1380549, SUC_lexicon_tags_6 },
    { 0x3c38054a, SUC_lexicon_tags_7 },
    { 0x5838054b, SUC_lexicon_tags_17 },
    { 0x3738054b, SUC_lexicon_tags_29 },
    { 0x9b58054c, SUC_lexicon_tags_7 },
    { 0xfe78054e, SUC_lexicon_tags_563 },
    { 0x2738054e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f180551, SUC_lexicon_tags_15 },
    { 0x89b80552, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf80556, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf758055b, SUC_lexicon_tags_25 },
    { 0xae38055b, SUC_lexicon_tags_12 },
    { 0x58f8055b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7358055f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc2b80561, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x52180563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91380566, SUC_lexicon_tags_66 },
    { 0x74580567, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b78056f, SUC_lexicon_tags_12 },
    { 0xd3380570, SUC_lexicon_tags_84 },
    { 0xa2580570, SUC_lexicon_tags_1 },
    { 0x66780572, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x76380575, SUC_lexicon_tags_7 },
    { 0xb3380576, SUC_lexicon_tags_33 },
    { 0x89980577, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6580579, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d8057d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb618057f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x32d80581, SUC_lexicon_tags_15 },
    { 0xd4980582, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4db80584, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x44f80586, SUC_lexicon_tags_32 },
    { 0x65180586, SUC_lexicon_tags_1 },
    { 0xb1180587, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e980598, SUC_lexicon_tags_14 },
    { 0xeff80599, SUC_lexicon_tags_60 },
    { 0xf58059a, SUC_lexicon_tags_46 },
    { 0x87580599, SUC_lexicon_tags_23 },
    { 0x8b98059c, SUC_lexicon_tags_25 },
    { 0x4038059d, SUC_lexicon_tags_47 },
    { 0xdbf8059c, SUC_lexicon_tags_37 },
    { 0xb218059e, SUC_lexicon_tags_25 },
    { 0x6bf8059c, SUC_lexicon_tags_34 },
    { 0x339805a1, SUC_lexicon_tags_25 },
    { 0x987805a2, SUC_lexicon_tags_23 },
    { 0x71d805a3, SUC_lexicon_tags_25 },
    { 0x6dd805a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd805ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0b805ad, SUC_lexicon_tags_60 },
    { 0x507805ad, SUC_lexicon_tags_7 },
    { 0xdcd805af, SUC_lexicon_tags_15 },
    { 0x199805af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7805b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f805bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f805c2, SUC_lexicon_tags_15 },
    { 0xd77805c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef805c6, SUC_lexicon_tags_25 },
    { 0x36b805c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdfb805c9, SUC_lexicon_tags_14 },
    { 0x419805c9, SUC_lexicon_tags_22 },
    { 0xbd3805cb, SUC_lexicon_tags_31 },
    { 0x4c5805cc, SUC_lexicon_tags_16 },
    { 0x749805cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417805d1, SUC_lexicon_tags_77 },
    { 0xef805d1, SUC_lexicon_tags_25 },
    { 0xfa7805d1, SUC_lexicon_tags_65 },
    { 0x3d7805d1, SUC_lexicon_tags_17 },
    { 0x147805d5, SUC_lexicon_tags_7 },
    { 0x795805d3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb805dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33805e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b805e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5805ec, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x293805ef, SUC_lexicon_tags_6 },
    { 0x4bf805f0, SUC_lexicon_tags_19 },
    { 0x3b805f1, SUC_lexicon_tags_17 },
    { 0xe9b805ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x83805f5, SUC_lexicon_tags_4 },
    { 0x851805f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43805f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d805fd, SUC_lexicon_tags_1 },
    { 0x459805fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e180603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8180608, SUC_lexicon_tags_37 },
    { 0x8380609, SUC_lexicon_tags_4 },
    { 0xc1780609, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4338060c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a780612, SUC_lexicon_tags_15 },
    { 0x47380612, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf80619, SUC_lexicon_tags_43 },
    { 0xabb8061a, SUC_lexicon_tags_49 },
    { 0x3f58061b, SUC_lexicon_tags_48 },
    { 0x6058061c, SUC_lexicon_tags_47 },
    { 0x82d8061c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff80625, SUC_lexicon_tags_26 },
    { 0xe2180626, SUC_lexicon_tags_37 },
    { 0xa8380627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f980629, SUC_lexicon_tags_37 },
    { 0x8ef8062a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88580633, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6cd80635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c980638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5d8063a, SUC_lexicon_tags_12 },
    { 0xe458063b, SUC_lexicon_tags_43 },
    { 0xd3b8063a, SUC_lexicon_tags_19 },
    { 0x3ab8063d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf380642, SUC_lexicon_tags_14 },
    { 0x27380643, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb80645, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21580648, SUC_lexicon_tags_38 },
    { 0x82380648, SUC_lexicon_tags_25 },
    { 0x1038064a, SUC_lexicon_tags_7 },
    { 0x3838064b, SUC_lexicon_tags_55 },
    { 0x8e980648, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcdb8064e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48180652, SUC_lexicon_tags_11 },
    { 0xadb80652, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b80656, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xea780658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f80660, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f380662, SUC_lexicon_tags_11 },
    { 0x780663, SUC_lexicon_tags_63 },
    { 0x32580662, SUC_lexicon_tags_15 },
    { 0xc0b80665, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeaf80667, SUC_lexicon_tags_7 },
    { 0xd9780667, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd758066a, SUC_lexicon_tags_25 },
    { 0x21f8066a, SUC_lexicon_tags_27 },
    { 0x28b8066b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3c78066e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72780670, SUC_lexicon_tags_17 },
    { 0x83180670, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed80678, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5058067b, SUC_lexicon_tags_45 },
    { 0x5538067c, SUC_lexicon_tags_11 },
    { 0x5278067b, SUC_lexicon_tags_14 },
    { 0xe0f8067e, SUC_lexicon_tags_28 },
    { 0x5dd8067c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f80687, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x30580689, SUC_lexicon_tags_0 },
    { 0xa698068a, SUC_lexicon_tags_23 },
    { 0xbf580689, SUC_lexicon_tags_26 },
    { 0xaed80689, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe498068f, SUC_lexicon_tags_7 },
    { 0x24180690, SUC_lexicon_tags_61 },
    { 0xe7980690, SUC_lexicon_tags_17 },
    { 0xe9b80692, SUC_lexicon_tags_11 },
    { 0xf0980693, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b80697, SUC_lexicon_tags_11 },
    { 0x96980698, SUC_lexicon_tags_17 },
    { 0xdb380698, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6278069d, SUC_lexicon_tags_605 },
    { 0x6c38069d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x83f806a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37806a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d806a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x803806a9, SUC_lexicon_tags_17 },
    { 0x209806aa, SUC_lexicon_tags_14 },
    { 0x8f9806aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1806ae, SUC_lexicon_tags_17 },
    { 0x37d806af, SUC_lexicon_tags_28 },
    { 0x1a3806b0, SUC_lexicon_tags_17 },
    { 0xaf9806af, SUC_lexicon_tags_37 },
    { 0x6b5806b0, SUC_lexicon_tags_15 },
    { 0x2bb806b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439806b8, SUC_lexicon_tags_129 },
    { 0x951806b9, SUC_lexicon_tags_17 },
    { 0x1e3806b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc2f806bc, SUC_lexicon_tags_63 },
    { 0x201806bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x71806bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd806c2, SUC_lexicon_tags_30 },
    { 0xfb1806c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525806cb, SUC_lexicon_tags_39 },
    { 0x6bf806cc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x829806ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1806d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f806d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaef806d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x315806db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b806de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b806e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb806e6, SUC_lexicon_tags_33 },
    { 0x81b806e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbbd806e9, SUC_lexicon_tags_28 },
    { 0xe5d806ea, SUC_lexicon_tags_12 },
    { 0xd39806eb, SUC_lexicon_tags_26 },
    { 0x639806ea, SUC_lexicon_tags_9 },
    { 0x107806eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff806f0, SUC_lexicon_tags_5 },
    { 0xe65806f1, SUC_lexicon_tags_11 },
    { 0xb79806f1, SUC_lexicon_tags_63 },
    { 0x157806f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57b806f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7180700, SUC_lexicon_tags_7 },
    { 0x90380701, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc180704, SUC_lexicon_tags_71 },
    { 0xa0580705, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d80709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7398070c, SUC_lexicon_tags_69 },
    { 0xbcd8070d, SUC_lexicon_tags_17 },
    { 0xf9b8070e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5580711, SUC_lexicon_tags_47 },
    { 0x26980712, SUC_lexicon_tags_46 },
    { 0x46b80713, SUC_lexicon_tags_10 },
    { 0xf1d80712, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c580717, SUC_lexicon_tags_1 },
    { 0x17f80717, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d8071f, SUC_lexicon_tags_17 },
    { 0xd1d80720, SUC_lexicon_tags_61 },
    { 0x71180720, SUC_lexicon_tags_14 },
    { 0x380721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d80726, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf5580728, SUC_lexicon_tags_37 },
    { 0x4c180729, SUC_lexicon_tags_7 },
    { 0xde38072a, SUC_lexicon_tags_0 },
    { 0xef18072b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f8072f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5180731, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b80734, SUC_lexicon_tags_19 },
    { 0xba980734, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf8073c, SUC_lexicon_tags_7 },
    { 0x9358073c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7180740, SUC_lexicon_tags_7 },
    { 0xd2b80740, SUC_lexicon_tags_7 },
    { 0x61d80740, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd80746, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x24d80748, SUC_lexicon_tags_25 },
    { 0x4d780749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8074e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f80751, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6580757, SUC_lexicon_tags_38 },
    { 0xb6580758, SUC_lexicon_tags_26 },
    { 0x37b80758, SUC_lexicon_tags_17 },
    { 0x4c18075a, SUC_lexicon_tags_12 },
    { 0x82180757, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5780760, SUC_lexicon_tags_1642 },
    { 0, NULL },
    { 0x25f80762, SUC_lexicon_tags_6 },
    { 0x3c980763, SUC_lexicon_tags_7 },
    { 0xc5f80763, SUC_lexicon_tags_25 },
    { 0x2c380762, SUC_lexicon_tags_38 },
    { 0x18980763, SUC_lexicon_tags_1 },
    { 0xbd80766, SUC_lexicon_tags_49 },
    { 0xb8780768, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb180770, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d80774, SUC_lexicon_tags_11 },
    { 0xe0180775, SUC_lexicon_tags_60 },
    { 0x42180776, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e58077b, SUC_lexicon_tags_12 },
    { 0xcf58077b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7778077e, SUC_lexicon_tags_30 },
    { 0xfe58077e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60580782, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce580787, SUC_lexicon_tags_0 },
    { 0x39b80788, SUC_lexicon_tags_38 },
    { 0xe0b80788, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d8078d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd758078f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30980791, SUC_lexicon_tags_30 },
    { 0x11580792, SUC_lexicon_tags_3 },
    { 0xee980793, SUC_lexicon_tags_11 },
    { 0x26380792, SUC_lexicon_tags_12 },
    { 0x20980792, SUC_lexicon_tags_121 },
    { 0xf0f80793, SUC_lexicon_tags_37 },
    { 0xc3780796, SUC_lexicon_tags_0 },
    { 0x3d980798, SUC_lexicon_tags_33 },
    { 0xb3780798, SUC_lexicon_tags_12 },
    { 0x64d80791, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd18079d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7807a0, SUC_lexicon_tags_1 },
    { 0xcbf807a1, SUC_lexicon_tags_38 },
    { 0x89b807a2, SUC_lexicon_tags_1 },
    { 0xe91807a2, SUC_lexicon_tags_29 },
    { 0x239807a3, SUC_lexicon_tags_47 },
    { 0xe6b807a5, SUC_lexicon_tags_60 },
    { 0xa97807a4, SUC_lexicon_tags_11 },
    { 0xf19807a7, SUC_lexicon_tags_0 },
    { 0x7cb807a7, SUC_lexicon_tags_12 },
    { 0x7b9807a1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49807ac, SUC_lexicon_tags_43 },
    { 0x903807ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x353807af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1807b4, SUC_lexicon_tags_45 },
    { 0xa05807b5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x14b807b7, SUC_lexicon_tags_30 },
    { 0x62b807b8, SUC_lexicon_tags_17 },
    { 0x129807b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19807bf, SUC_lexicon_tags_43 },
    { 0x60d807c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x17d807c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d807c5, SUC_lexicon_tags_57 },
    { 0x67807c6, SUC_lexicon_tags_37 },
    { 0x3e9807c6, SUC_lexicon_tags_27 },
    { 0xb1b807c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f807cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcf807cd, SUC_lexicon_tags_17 },
    { 0xaa9807ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x661807d0, SUC_lexicon_tags_8 },
    { 0xe4f807d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d807d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20f807d6, SUC_lexicon_tags_23 },
    { 0xd79807d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b807dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed807e3, SUC_lexicon_tags_27 },
    { 0xc57807e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5ad807e6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129807ea, SUC_lexicon_tags_28 },
    { 0x38f807eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f807f3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb807f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6dd807fa, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505807fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b80800, SUC_lexicon_tags_36 },
    { 0x44180800, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24380805, SUC_lexicon_tags_8 },
    { 0x21780806, SUC_lexicon_tags_7 },
    { 0x6a780806, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd18080b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90d8080d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf18080f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef80813, SUC_lexicon_tags_33 },
    { 0xb0180813, SUC_lexicon_tags_2 },
    { 0x24980815, SUC_lexicon_tags_17 },
    { 0x89380816, SUC_lexicon_tags_63 },
    { 0xff180817, SUC_lexicon_tags_65 },
    { 0xea980818, SUC_lexicon_tags_2 },
    { 0xdeb80817, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb8081e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc780820, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32980824, SUC_lexicon_tags_37 },
    { 0xd1780825, SUC_lexicon_tags_38 },
    { 0xf4180826, SUC_lexicon_tags_63 },
    { 0xc5d80827, SUC_lexicon_tags_47 },
    { 0xe7b80827, SUC_lexicon_tags_37 },
    { 0xa4180827, SUC_lexicon_tags_35 },
    { 0x1e380827, SUC_lexicon_tags_51 },
    { 0xcbd8082b, SUC_lexicon_tags_6 },
    { 0xe6780828, SUC_lexicon_tags_12 },
    { 0x6d58082a, SUC_lexicon_tags_17 },
    { 0xb778082e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9780830, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26780834, SUC_lexicon_tags_14 },
    { 0xcab80835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6b80837, SUC_lexicon_tags_11 },
    { 0x61b80838, SUC_lexicon_tags_81 },
    { 0xa0780838, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37180842, SUC_lexicon_tags_1 },
    { 0xcb380843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15d80845, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1980847, SUC_lexicon_tags_11 },
    { 0xa6380848, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b58084a, SUC_lexicon_tags_2 },
    { 0x3f58084b, SUC_lexicon_tags_14 },
    { 0x41d8084b, SUC_lexicon_tags_2 },
    { 0xaad8084d, SUC_lexicon_tags_597 },
    { 0x48d8084e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d80852, SUC_lexicon_tags_33 },
    { 0x7f80853, SUC_lexicon_tags_170 },
    { 0x2d80854, SUC_lexicon_tags_46 },
    { 0xaab80853, SUC_lexicon_tags_13 },
    { 0x3ad80855, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0, NULL },
    { 0x2580859, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d78085d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4180863, SUC_lexicon_tags_66 },
    { 0x1f380863, SUC_lexicon_tags_11 },
    { 0x7a180863, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed80868, SUC_lexicon_tags_12 },
    { 0xfa980869, SUC_lexicon_tags_6 },
    { 0x21380868, SUC_lexicon_tags_37 },
    { 0xf2f80869, SUC_lexicon_tags_25 },
    { 0x9b98086c, SUC_lexicon_tags_37 },
    { 0x2938086d, SUC_lexicon_tags_26 },
    { 0x8858086e, SUC_lexicon_tags_65 },
    { 0x8458086f, SUC_lexicon_tags_11 },
    { 0x2a58086d, SUC_lexicon_tags_45 },
    { 0x22980871, SUC_lexicon_tags_25 },
    { 0xf2f80871, SUC_lexicon_tags_38 },
    { 0x8780872, SUC_lexicon_tags_25 },
    { 0xb9f80874, SUC_lexicon_tags_1 },
    { 0x2d38086c, SUC_lexicon_tags_60 },
    { 0x45380872, SUC_lexicon_tags_0 },
    { 0x50d8086c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae780879, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6018087b, SUC_lexicon_tags_26 },
    { 0x6d38087c, SUC_lexicon_tags_17 },
    { 0xe0f8087d, SUC_lexicon_tags_23 },
    { 0xedb8087e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b80885, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49980889, SUC_lexicon_tags_3 },
    { 0x3780889, SUC_lexicon_tags_49 },
    { 0x3858088b, SUC_lexicon_tags_19 },
    { 0x11b8088b, SUC_lexicon_tags_10 },
    { 0x8038088d, SUC_lexicon_tags_10 },
    { 0x56f8088e, SUC_lexicon_tags_11 },
    { 0xb5d8088f, SUC_lexicon_tags_664 },
    { 0x9a38088e, SUC_lexicon_tags_31 },
    { 0x19380891, SUC_lexicon_tags_11 },
    { 0x4a780891, SUC_lexicon_tags_21 },
    { 0x95f80893, SUC_lexicon_tags_17 },
    { 0x2078088d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x75f80896, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc3180898, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18089f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf7b808a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51808a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07808a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab1808aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5808b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x581808b6, SUC_lexicon_tags_534 },
    { 0x179808b6, SUC_lexicon_tags_11 },
    { 0x191808b8, SUC_lexicon_tags_37 },
    { 0xb81808b7, SUC_lexicon_tags_7 },
    { 0x221808b7, SUC_lexicon_tags_37 },
    { 0xa19808b8, SUC_lexicon_tags_24 },
    { 0x45f808bc, SUC_lexicon_tags_30 },
    { 0x58f808bd, SUC_lexicon_tags_17 },
    { 0x9f808b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67808c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813808ce, SUC_lexicon_tags_45 },
    { 0x80b808ce, SUC_lexicon_tags_15 },
    { 0x993808d0, SUC_lexicon_tags_17 },
    { 0x84b808d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7808db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x499808de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f808e9, SUC_lexicon_tags_32 },
    { 0xe21808ea, SUC_lexicon_tags_82 },
    { 0x315808e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247808f0, SUC_lexicon_tags_7 },
    { 0xaa9808f1, SUC_lexicon_tags_51 },
    { 0x6cd808f1, SUC_lexicon_tags_68 },
    { 0x5d7808f3, SUC_lexicon_tags_38 },
    { 0x207808f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45808f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x771808f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fb808fb, SUC_lexicon_tags_27 },
    { 0x565808fc, SUC_lexicon_tags_1 },
    { 0x615808fd, SUC_lexicon_tags_7 },
    { 0xbe7808fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a180905, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca78090a, SUC_lexicon_tags_1 },
    { 0x3e18090b, SUC_lexicon_tags_7 },
    { 0x7f8090c, SUC_lexicon_tags_25 },
    { 0x9bd8090c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9780913, SUC_lexicon_tags_108 },
    { 0xbcf80914, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d8091b, SUC_lexicon_tags_0 },
    { 0x2b98091c, SUC_lexicon_tags_25 },
    { 0x7818091c, SUC_lexicon_tags_18 },
    { 0x9f8091e, SUC_lexicon_tags_11 },
    { 0x5e78091d, SUC_lexicon_tags_25 },
    { 0xa78091c, SUC_lexicon_tags_28 },
    { 0x6f580921, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72380924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb980926, SUC_lexicon_tags_17 },
    { 0xe3380927, SUC_lexicon_tags_11 },
    { 0xe4180928, SUC_lexicon_tags_37 },
    { 0x1b980929, SUC_lexicon_tags_40 },
    { 0x39d80929, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d80932, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f80936, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7780938, SUC_lexicon_tags_60 },
    { 0xa3780939, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe258093d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17980943, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8580949, SUC_lexicon_tags_22 },
    { 0x5b580949, SUC_lexicon_tags_7 },
    { 0xae980949, SUC_lexicon_tags_14 },
    { 0xaa18094a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f80955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43d80957, SUC_lexicon_tags_38 },
    { 0x38b80958, SUC_lexicon_tags_12 },
    { 0xaf80957, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfc98095b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b78095e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd580964, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb238096c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af8096f, SUC_lexicon_tags_25 },
    { 0x21b8096f, SUC_lexicon_tags_29 },
    { 0x70d80971, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b80976, SUC_lexicon_tags_31 },
    { 0x2bf80977, SUC_lexicon_tags_7 },
    { 0x44580978, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9658097b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa1d8097d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c380981, SUC_lexicon_tags_8 },
    { 0x63f80982, SUC_lexicon_tags_11 },
    { 0x55980981, SUC_lexicon_tags_1 },
    { 0x39980981, SUC_lexicon_tags_38 },
    { 0x92f80985, SUC_lexicon_tags_22 },
    { 0x3fb80985, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a18098c, SUC_lexicon_tags_12 },
    { 0xef78098c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d80992, SUC_lexicon_tags_10 },
    { 0x4980993, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e980995, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb80998, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf38099b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xded8099d, SUC_lexicon_tags_25 },
    { 0x1218099e, SUC_lexicon_tags_23 },
    { 0xde18099e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc27809a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe5d809a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b809aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d809b1, SUC_lexicon_tags_11 },
    { 0x15b809b2, SUC_lexicon_tags_46 },
    { 0x7eb809b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x63f809b5, SUC_lexicon_tags_15 },
    { 0x695809b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x109809b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x681809bb, SUC_lexicon_tags_25 },
    { 0x753809bc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc85809be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7809c1, SUC_lexicon_tags_17 },
    { 0x4bb809c2, SUC_lexicon_tags_25 },
    { 0x941809c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1809c7, SUC_lexicon_tags_11 },
    { 0x135809c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289809cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44d809cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1809dc, SUC_lexicon_tags_9 },
    { 0x9cd809dc, SUC_lexicon_tags_27 },
    { 0xbbd809dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xced809e0, SUC_lexicon_tags_17 },
    { 0x2d3809e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf809ee, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff809f2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499809f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b980a00, SUC_lexicon_tags_23 },
    { 0xff980a00, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a980a04, SUC_lexicon_tags_12 },
    { 0xa1f80a05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xba580a08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2af80a0a, SUC_lexicon_tags_27 },
    { 0x65f80a0b, SUC_lexicon_tags_63 },
    { 0xc3180a0c, SUC_lexicon_tags_7 },
    { 0x12780a0c, SUC_lexicon_tags_11 },
    { 0xc6180a0c, SUC_lexicon_tags_11 },
    { 0xec380a0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26980a11, SUC_lexicon_tags_39 },
    { 0xfa780a12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf80a17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2780a1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x80580a1f, SUC_lexicon_tags_37 },
    { 0x14780a20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa780a23, SUC_lexicon_tags_12 },
    { 0x56780a23, SUC_lexicon_tags_25 },
    { 0x7f780a24, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b80a2b, SUC_lexicon_tags_2 },
    { 0xa5780a2c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x89380a2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1380a31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c780a33, SUC_lexicon_tags_17 },
    { 0x1a980a34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2180a3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83180a3c, SUC_lexicon_tags_16 },
    { 0xf7b80a3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa780a40, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xba580a42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xad80a44, SUC_lexicon_tags_8 },
    { 0xaff80a44, SUC_lexicon_tags_37 },
    { 0x69580a46, SUC_lexicon_tags_37 },
    { 0xed980a47, SUC_lexicon_tags_12 },
    { 0x8b580a47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ad80a4a, SUC_lexicon_tags_2 },
    { 0x2d980a4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9380a53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d80a57, SUC_lexicon_tags_29 },
    { 0xf9380a58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe380a5b, SUC_lexicon_tags_0 },
    { 0xdb580a5b, SUC_lexicon_tags_11 },
    { 0xa9580a5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1980a60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3f80a62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b80a6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x72580a6d, SUC_lexicon_tags_66 },
    { 0x27180a6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8380a70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e380a73, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49580a77, SUC_lexicon_tags_46 },
    { 0xfc580a77, SUC_lexicon_tags_14 },
    { 0xee780a77, SUC_lexicon_tags_33 },
    { 0x7b80a78, SUC_lexicon_tags_37 },
    { 0xff780a78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17580a7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x46980a81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7580a89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbd780a8b, SUC_lexicon_tags_37 },
    { 0xd3b80a8b, SUC_lexicon_tags_2 },
    { 0x380a8d, SUC_lexicon_tags_27 },
    { 0x90180a8e, SUC_lexicon_tags_15 },
    { 0xe0780a8e, SUC_lexicon_tags_61 },
    { 0x3db80a8e, SUC_lexicon_tags_38 },
    { 0xef580a8f, SUC_lexicon_tags_26 },
    { 0x66f80a92, SUC_lexicon_tags_1 },
    { 0xd1580a93, SUC_lexicon_tags_25 },
    { 0x56580a8b, SUC_lexicon_tags_12 },
    { 0x62f80a90, SUC_lexicon_tags_12 },
    { 0x28180a90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28780a98, SUC_lexicon_tags_17 },
    { 0x56580a99, SUC_lexicon_tags_1 },
    { 0xf7980a9a, SUC_lexicon_tags_2 },
    { 0xd2b80a9b, SUC_lexicon_tags_31 },
    { 0x56b80a9c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f80a9f, SUC_lexicon_tags_151 },
    { 0xb0d80a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1780aa2, SUC_lexicon_tags_62 },
    { 0x77780aa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3780aa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f80aa9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5180aac, SUC_lexicon_tags_1 },
    { 0x33d80aad, SUC_lexicon_tags_38 },
    { 0x72b80aae, SUC_lexicon_tags_37 },
    { 0xfd780aaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb80ab3, SUC_lexicon_tags_11 },
    { 0x1fd80ab4, SUC_lexicon_tags_15 },
    { 0x41380ab3, SUC_lexicon_tags_1 },
    { 0x45d80ab6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f80ab9, SUC_lexicon_tags_25 },
    { 0x91780ab9, SUC_lexicon_tags_82 },
    { 0xea580abb, SUC_lexicon_tags_25 },
    { 0xb1980ab9, SUC_lexicon_tags_7 },
    { 0x31f80abc, SUC_lexicon_tags_27 },
    { 0x16f80abe, SUC_lexicon_tags_61 },
    { 0x6cb80abf, SUC_lexicon_tags_37 },
    { 0xd1b80ac0, SUC_lexicon_tags_7 },
    { 0xb0d80abe, SUC_lexicon_tags_1 },
    { 0xcdf80ac2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x30980ac5, SUC_lexicon_tags_2 },
    { 0xc9980ac6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b80ac9, SUC_lexicon_tags_38 },
    { 0x3cd80aca, SUC_lexicon_tags_19 },
    { 0xe2f80acb, SUC_lexicon_tags_1 },
    { 0x59780acc, SUC_lexicon_tags_17 },
    { 0x59b80acd, SUC_lexicon_tags_34 },
    { 0xd9b80ace, SUC_lexicon_tags_37 },
    { 0x83180acd, SUC_lexicon_tags_12 },
    { 0xeef80acb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ed80ad2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65580ad8, SUC_lexicon_tags_16 },
    { 0x99380ad8, SUC_lexicon_tags_11 },
    { 0xbff80ada, SUC_lexicon_tags_28 },
    { 0x1b980adb, SUC_lexicon_tags_15 },
    { 0xc8180adc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40580ae1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4180ae8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cb80aea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xea180aed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0580af1, SUC_lexicon_tags_17 },
    { 0xaef80af2, SUC_lexicon_tags_11 },
    { 0x24980af3, SUC_lexicon_tags_33 },
    { 0x45380af2, SUC_lexicon_tags_116 },
    { 0x41180af5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0380af8, SUC_lexicon_tags_43 },
    { 0x1ef80af9, SUC_lexicon_tags_1 },
    { 0xf8d80afa, SUC_lexicon_tags_28 },
    { 0x5ad80afb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b80afe, SUC_lexicon_tags_37 },
    { 0x7c780afe, SUC_lexicon_tags_7 },
    { 0xd1f80afe, SUC_lexicon_tags_1 },
    { 0xef80b01, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x68580b03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7980b08, SUC_lexicon_tags_2 },
    { 0x9d780b09, SUC_lexicon_tags_2 },
    { 0x6c980b09, SUC_lexicon_tags_19 },
    { 0xbd180b0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f80b17, SUC_lexicon_tags_33 },
    { 0x60180b18, SUC_lexicon_tags_836 },
    { 0x1cd80b19, SUC_lexicon_tags_2 },
    { 0x50f80b1a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3ef80b1c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af80b20, SUC_lexicon_tags_26 },
    { 0xa3380b21, SUC_lexicon_tags_13 },
    { 0xf9d80b21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53780b29, SUC_lexicon_tags_11 },
    { 0x3ab80b2a, SUC_lexicon_tags_30 },
    { 0x17180b2b, SUC_lexicon_tags_27 },
    { 0xf0f80b2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b80b30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d380b32, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7780b36, SUC_lexicon_tags_16 },
    { 0x45d80b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d80b3c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb80b3f, SUC_lexicon_tags_1 },
    { 0xbc580b40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x53b80b42, SUC_lexicon_tags_15 },
    { 0x92780b42, SUC_lexicon_tags_2 },
    { 0x53780b42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6580b48, SUC_lexicon_tags_37 },
    { 0x82180b49, SUC_lexicon_tags_15 },
    { 0xe580b49, SUC_lexicon_tags_1 },
    { 0xaa580b4a, SUC_lexicon_tags_6 },
    { 0x53980b4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26580b57, SUC_lexicon_tags_15 },
    { 0x43180b58, SUC_lexicon_tags_45 },
    { 0x36d80b58, SUC_lexicon_tags_15 },
    { 0xd3780b5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f80b5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb80b63, SUC_lexicon_tags_17 },
    { 0x65b80b63, SUC_lexicon_tags_49 },
    { 0x3e980b64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90d80b67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a380b6d, SUC_lexicon_tags_11 },
    { 0x3df80b6e, SUC_lexicon_tags_47 },
    { 0x26380b6f, SUC_lexicon_tags_37 },
    { 0xf0580b70, SUC_lexicon_tags_12 },
    { 0x1c180b70, SUC_lexicon_tags_27 },
    { 0x26380b72, SUC_lexicon_tags_53 },
    { 0xdff80b73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0580b75, SUC_lexicon_tags_17 },
    { 0x60d80b76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb8b80b78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68180b7c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe2780b7e, SUC_lexicon_tags_17 },
    { 0x47780b7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba980b85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af80b89, SUC_lexicon_tags_37 },
    { 0x8e380b89, SUC_lexicon_tags_49 },
    { 0xc2580b8a, SUC_lexicon_tags_12 },
    { 0x47b80b8c, SUC_lexicon_tags_49 },
    { 0xc8d80b8b, SUC_lexicon_tags_37 },
    { 0x46380b8c, SUC_lexicon_tags_38 },
    { 0xf0180b8f, SUC_lexicon_tags_17 },
    { 0xc9980b8c, SUC_lexicon_tags_23 },
    { 0xb0b80b91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x79580b94, SUC_lexicon_tags_25 },
    { 0xd7380b94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d380b98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ad80b9a, SUC_lexicon_tags_0 },
    { 0xc4580b9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89380ba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5580ba5, SUC_lexicon_tags_28 },
    { 0xa6380ba6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0580bab, SUC_lexicon_tags_68 },
    { 0x58780bac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6f80bae, SUC_lexicon_tags_47 },
    { 0xc9380baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6980bb1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x57180bb3, SUC_lexicon_tags_18 },
    { 0x87580bb4, SUC_lexicon_tags_37 },
    { 0x89980bb5, SUC_lexicon_tags_12 },
    { 0xc0180bb6, SUC_lexicon_tags_17 },
    { 0x8a380bb7, SUC_lexicon_tags_19 },
    { 0x17980bb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9380bbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f80bc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62180bcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b380bd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdab80bd2, SUC_lexicon_tags_4 },
    { 0x27580bd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0980bd9, SUC_lexicon_tags_4 },
    { 0xecd80bda, SUC_lexicon_tags_33 },
    { 0x45d80bdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13180bde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67980be2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x34580be4, SUC_lexicon_tags_30 },
    { 0x16d80be5, SUC_lexicon_tags_29 },
    { 0x2380be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af80bea, SUC_lexicon_tags_26 },
    { 0x10380beb, SUC_lexicon_tags_7 },
    { 0x25d80bec, SUC_lexicon_tags_15 },
    { 0xf8780beb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86380bef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcc180bf1, SUC_lexicon_tags_17 },
    { 0xf2b80bf2, SUC_lexicon_tags_26 },
    { 0xffd80bf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14980bf8, SUC_lexicon_tags_68 },
    { 0xbf980bf9, SUC_lexicon_tags_2 },
    { 0xa1780bfa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a980bfc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1580c00, SUC_lexicon_tags_25 },
    { 0xda380c00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10180c06, SUC_lexicon_tags_28 },
    { 0xb8780c07, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57980c0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef980c10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ad80c12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d780c1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb80c1d, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xa5180c1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd380c21, SUC_lexicon_tags_25 },
    { 0xb4580c22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1180c24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66580c28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x84180c2b, SUC_lexicon_tags_1 },
    { 0x71380c2b, SUC_lexicon_tags_1 },
    { 0xabb80c2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4ff80c2f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6c780c31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f80c39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8f80c3b, SUC_lexicon_tags_12 },
    { 0x1380c3c, SUC_lexicon_tags_11 },
    { 0xf8d80c3d, SUC_lexicon_tags_19 },
    { 0x3f180c3c, SUC_lexicon_tags_6 },
    { 0xb6b80c3d, SUC_lexicon_tags_1 },
    { 0xa180c40, SUC_lexicon_tags_37 },
    { 0xa7980c3d, SUC_lexicon_tags_26 },
    { 0xa9d80c42, SUC_lexicon_tags_14 },
    { 0x8eb80c43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28780c46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c380c49, SUC_lexicon_tags_17 },
    { 0xd2180c4a, SUC_lexicon_tags_6 },
    { 0xef980c4b, SUC_lexicon_tags_7 },
    { 0x45980c4c, SUC_lexicon_tags_11 },
    { 0x35980c4c, SUC_lexicon_tags_11 },
    { 0x3f380c4c, SUC_lexicon_tags_27 },
    { 0xc6180c4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff80c55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8380c5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab80c5e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f80c63, SUC_lexicon_tags_57 },
    { 0x96980c64, SUC_lexicon_tags_23 },
    { 0x50380c65, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94f80c67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24f80c69, SUC_lexicon_tags_43 },
    { 0x25580c6a, SUC_lexicon_tags_28 },
    { 0x7df80c6a, SUC_lexicon_tags_0 },
    { 0xf6b80c6c, SUC_lexicon_tags_11 },
    { 0xa8180c6d, SUC_lexicon_tags_31 },
    { 0x48580c6e, SUC_lexicon_tags_17 },
    { 0x10780c6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad80c76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d80c7b, SUC_lexicon_tags_37 },
    { 0xb5b80c7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b80c8e, SUC_lexicon_tags_37 },
    { 0x46980c8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73d80c91, SUC_lexicon_tags_25 },
    { 0x32580c92, SUC_lexicon_tags_77 },
    { 0xba980c93, SUC_lexicon_tags_25 },
    { 0x6b80c91, SUC_lexicon_tags_45 },
    { 0xd580c94, SUC_lexicon_tags_39 },
    { 0x1f780c93, SUC_lexicon_tags_17 },
    { 0x43b80c97, SUC_lexicon_tags_25 },
    { 0x43780c97, SUC_lexicon_tags_25 },
    { 0x2b980c93, SUC_lexicon_tags_4 },
    { 0x98f80c92, SUC_lexicon_tags_12 },
    { 0xf5180c97, SUC_lexicon_tags_17 },
    { 0xe1b80c9c, SUC_lexicon_tags_17 },
    { 0x21b80c9d, SUC_lexicon_tags_17 },
    { 0xa0780c9b, SUC_lexicon_tags_15 },
    { 0x67780c91, SUC_lexicon_tags_56 },
    { 0x35f80ca0, SUC_lexicon_tags_21 },
    { 0xcbd80ca0, SUC_lexicon_tags_37 },
    { 0x31580c98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x16180ca4, SUC_lexicon_tags_63 },
    { 0x60f80ca5, SUC_lexicon_tags_14 },
    { 0x83f80ca5, SUC_lexicon_tags_63 },
    { 0x64180ca7, SUC_lexicon_tags_9 },
    { 0x7f780ca8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb580cac, SUC_lexicon_tags_45 },
    { 0xaa780cac, SUC_lexicon_tags_27 },
    { 0x69180cac, SUC_lexicon_tags_25 },
    { 0xd2380cac, SUC_lexicon_tags_0 },
    { 0x180cad, SUC_lexicon_tags_45 },
    { 0xa7b80cac, SUC_lexicon_tags_65 },
    { 0x78380cb2, SUC_lexicon_tags_12 },
    { 0x85d80cb3, SUC_lexicon_tags_15 },
    { 0x35580cb0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5df80cb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd380cb8, SUC_lexicon_tags_14 },
    { 0xf9d80cb8, SUC_lexicon_tags_25 },
    { 0x8dd80cb9, SUC_lexicon_tags_160 },
    { 0xef780cbb, SUC_lexicon_tags_19 },
    { 0x9f80cbb, SUC_lexicon_tags_5 },
    { 0x39f80cbb, SUC_lexicon_tags_30 },
    { 0xfe580cbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fb80cc0, SUC_lexicon_tags_298 },
    { 0x2d80cc1, SUC_lexicon_tags_234 },
    { 0x76f80cc0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd80cc5, SUC_lexicon_tags_2 },
    { 0xda580cc5, SUC_lexicon_tags_7 },
    { 0x60180cc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c780cc9, SUC_lexicon_tags_49 },
    { 0x9ad80cca, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x47780ccd, SUC_lexicon_tags_68 },
    { 0x24d80ccd, SUC_lexicon_tags_15 },
    { 0xa3b80cce, SUC_lexicon_tags_6 },
    { 0xb6380cd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fd80cd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b80cdb, SUC_lexicon_tags_43 },
    { 0xd5980cdb, SUC_lexicon_tags_29 },
    { 0x54f80cdb, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63380ce6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42180cea, SUC_lexicon_tags_7 },
    { 0x4580ceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f580cf1, SUC_lexicon_tags_9 },
    { 0x9db80cf2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x73580cf4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x15980cf7, SUC_lexicon_tags_16 },
    { 0xa6780cf7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f80cfc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f80d00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e380d04, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc8b80d06, SUC_lexicon_tags_15 },
    { 0x3a980d07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb80d0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57580d10, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a380d18, SUC_lexicon_tags_23 },
    { 0x6e180d19, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x79580d1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4180d20, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17380d27, SUC_lexicon_tags_23 },
    { 0xbab80d28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19380d2e, SUC_lexicon_tags_29 },
    { 0x2b980d2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d380d34, SUC_lexicon_tags_1 },
    { 0xfb980d35, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x16580d37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x70180d39, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27780d3d, SUC_lexicon_tags_7 },
    { 0x2ef80d3e, SUC_lexicon_tags_37 },
    { 0x54180d3e, SUC_lexicon_tags_1 },
    { 0x2ed80d40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6580d43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f380d45, SUC_lexicon_tags_7 },
    { 0xb0180d45, SUC_lexicon_tags_1 },
    { 0xddd80d47, SUC_lexicon_tags_11 },
    { 0x6b180d47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x58780d4b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb8380d4d, SUC_lexicon_tags_7 },
    { 0xa6780d4d, SUC_lexicon_tags_8 },
    { 0x62d80d4d, SUC_lexicon_tags_13 },
    { 0xf0580d50, SUC_lexicon_tags_25 },
    { 0x9eb80d51, SUC_lexicon_tags_11 },
    { 0x35380d52, SUC_lexicon_tags_15 },
    { 0x64b80d52, SUC_lexicon_tags_47 },
    { 0x81f80d54, SUC_lexicon_tags_7 },
    { 0x4c980d52, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b780d59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37b80d5b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a580d5f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d80d67, SUC_lexicon_tags_7 },
    { 0x71580d68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf7180d6a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d580d6e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d180d71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0b80d73, SUC_lexicon_tags_27 },
    { 0x27580d74, SUC_lexicon_tags_30 },
    { 0x66b80d74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xad380d78, SUC_lexicon_tags_17 },
    { 0x5180d79, SUC_lexicon_tags_11 },
    { 0x7a380d79, SUC_lexicon_tags_10 },
    { 0xc1380d7b, SUC_lexicon_tags_21 },
    { 0x2780d7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28180d7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87b80d80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7580d83, SUC_lexicon_tags_17 },
    { 0x36380d84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71980d86, SUC_lexicon_tags_19 },
    { 0x89580d86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x25b80d89, SUC_lexicon_tags_17 },
    { 0x6980d8a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x87580d8c, SUC_lexicon_tags_27 },
    { 0xff580d8d, SUC_lexicon_tags_25 },
    { 0x7dd80d8e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x60b80d90, SUC_lexicon_tags_34 },
    { 0x2380d91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4c380d93, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12980d98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x74b80d9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x87780d9c, SUC_lexicon_tags_10 },
    { 0x2e780d9d, SUC_lexicon_tags_60 },
    { 0x43d80d9d, SUC_lexicon_tags_26 },
    { 0x5d80d9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd980da2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43380da7, SUC_lexicon_tags_472 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab80dac, SUC_lexicon_tags_14 },
    { 0x2a580dad, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f780db1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfdb80db3, SUC_lexicon_tags_84 },
    { 0x2c180db4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb80dba, SUC_lexicon_tags_16 },
    { 0x20f80dbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6380dbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f980dc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0980dc9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b80dcf, SUC_lexicon_tags_30 },
    { 0x89780dd0, SUC_lexicon_tags_65 },
    { 0xc7980dd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd1980dd3, SUC_lexicon_tags_49 },
    { 0x16180dd4, SUC_lexicon_tags_60 },
    { 0x1fd80dd4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9580dd8, SUC_lexicon_tags_2 },
    { 0xac580dd9, SUC_lexicon_tags_1 },
    { 0x7af80dd8, SUC_lexicon_tags_33 },
    { 0xc3980ddb, SUC_lexicon_tags_1 },
    { 0x4bd80ddc, SUC_lexicon_tags_9 },
    { 0x5bf80dd8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b180de1, SUC_lexicon_tags_17 },
    { 0xad980de1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f380de6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b780dea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27180dec, SUC_lexicon_tags_25 },
    { 0xdb180dec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b80dfd, SUC_lexicon_tags_38 },
    { 0x2bb80dfe, SUC_lexicon_tags_10 },
    { 0x69380dfd, SUC_lexicon_tags_1226 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a180e02, SUC_lexicon_tags_17 },
    { 0x8d980e02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9580e07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb80e0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa980e0f, SUC_lexicon_tags_1 },
    { 0x12780e10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87180e15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17f80e17, SUC_lexicon_tags_43 },
    { 0xc7f80e18, SUC_lexicon_tags_17 },
    { 0xe4580e17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f80e20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd80e26, SUC_lexicon_tags_12 },
    { 0x77b80e27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb80e2d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x27180e30, SUC_lexicon_tags_12 },
    { 0xa4f80e30, SUC_lexicon_tags_8 },
    { 0x9f980e31, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe1d80e34, SUC_lexicon_tags_25 },
    { 0xcdf80e35, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1180e3a, SUC_lexicon_tags_32 },
    { 0xaf180e3b, SUC_lexicon_tags_39 },
    { 0x79d80e3c, SUC_lexicon_tags_14 },
    { 0xbd780e3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2780e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6580e4f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x89780e51, SUC_lexicon_tags_38 },
    { 0x9db80e52, SUC_lexicon_tags_63 },
    { 0x5d780e53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d80e59, SUC_lexicon_tags_57 },
    { 0xe3f80e5a, SUC_lexicon_tags_17 },
    { 0x2cd80e5b, SUC_lexicon_tags_2 },
    { 0x6d180e5c, SUC_lexicon_tags_49 },
    { 0x8fd80e5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61580e64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x23780e66, SUC_lexicon_tags_11 },
    { 0x91780e66, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc3f80e69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d780e72, SUC_lexicon_tags_37 },
    { 0x24980e73, SUC_lexicon_tags_19 },
    { 0x9980e74, SUC_lexicon_tags_37 },
    { 0xaf80e74, SUC_lexicon_tags_1 },
    { 0x9e380e74, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4580e7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98380e7e, SUC_lexicon_tags_47 },
    { 0xfb80e7e, SUC_lexicon_tags_25 },
    { 0x29180e7e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56580e85, SUC_lexicon_tags_0 },
    { 0x16780e86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb980e89, SUC_lexicon_tags_33 },
    { 0x6f780e89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f80e8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70980e95, SUC_lexicon_tags_65 },
    { 0x11780e96, SUC_lexicon_tags_7 },
    { 0x89d80e97, SUC_lexicon_tags_49 },
    { 0xd6180e96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82980e9e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc9380ea0, SUC_lexicon_tags_30 },
    { 0xcf580ea1, SUC_lexicon_tags_15 },
    { 0x7c380ea0, SUC_lexicon_tags_8 },
    { 0xa6180ea1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a180ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7380ea8, SUC_lexicon_tags_34 },
    { 0xf2380ea8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x62780eab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c380ead, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde380eb2, SUC_lexicon_tags_47 },
    { 0xc3b80eb2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4580eb6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb3180eb8, SUC_lexicon_tags_11 },
    { 0x22580eb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30380ec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe8780ecb, SUC_lexicon_tags_25 },
    { 0xe1180ecc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d980ed1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62980ed9, SUC_lexicon_tags_11 },
    { 0x2a980eda, SUC_lexicon_tags_11 },
    { 0xb6180edb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x20d80edd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe180ee0, SUC_lexicon_tags_25 },
    { 0xf9780ee1, SUC_lexicon_tags_49 },
    { 0x73580ee0, SUC_lexicon_tags_17 },
    { 0xe1580ee3, SUC_lexicon_tags_17 },
    { 0x62580ee4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a980ee8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2b80eea, SUC_lexicon_tags_25 },
    { 0xa6f80eeb, SUC_lexicon_tags_37 },
    { 0xcd580eec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f980eef, SUC_lexicon_tags_14 },
    { 0x20b80eef, SUC_lexicon_tags_59 },
    { 0x8af80ef1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38580ef3, SUC_lexicon_tags_16 },
    { 0xb0980ef3, SUC_lexicon_tags_21 },
    { 0x2eb80ef4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a380efd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fb80eff, SUC_lexicon_tags_12 },
    { 0x63f80f00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd580f06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3db80f08, SUC_lexicon_tags_12 },
    { 0xfdd80f09, SUC_lexicon_tags_11 },
    { 0xc2d80f08, SUC_lexicon_tags_6 },
    { 0xc5d80f0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbbf80f0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b380f0f, SUC_lexicon_tags_25 },
    { 0xcd80f10, SUC_lexicon_tags_30 },
    { 0x97b80f0f, SUC_lexicon_tags_2 },
    { 0x20580f12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x54780f15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf0580f17, SUC_lexicon_tags_12 },
    { 0x83780f17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd980f1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c580f1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x69f80f20, SUC_lexicon_tags_27 },
    { 0xb5380f21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c180f24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf80f27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d80f2e, SUC_lexicon_tags_31 },
    { 0x9ef80f2f, SUC_lexicon_tags_2 },
    { 0xcd380f30, SUC_lexicon_tags_1 },
    { 0xc980f31, SUC_lexicon_tags_37 },
    { 0x45d80f31, SUC_lexicon_tags_8 },
    { 0x3a980f32, SUC_lexicon_tags_11 },
    { 0xef180f34, SUC_lexicon_tags_26 },
    { 0x3c980f33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb80f38, SUC_lexicon_tags_12 },
    { 0xd2980f39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a780f3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc180f43, SUC_lexicon_tags_25 },
    { 0xe0980f44, SUC_lexicon_tags_11 },
    { 0xd7980f44, SUC_lexicon_tags_25 },
    { 0xb2580f46, SUC_lexicon_tags_9 },
    { 0x38d80f47, SUC_lexicon_tags_2 },
    { 0x94180f48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2580f4b, SUC_lexicon_tags_47 },
    { 0x55780f4b, SUC_lexicon_tags_14 },
    { 0x49780f4d, SUC_lexicon_tags_15 },
    { 0xad980f4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d80f52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x11f80f54, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60980f5a, SUC_lexicon_tags_43 },
    { 0xf1980f5b, SUC_lexicon_tags_37 },
    { 0x78b80f5c, SUC_lexicon_tags_11 },
    { 0xded80f5c, SUC_lexicon_tags_49 },
    { 0x6d980f5e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f380f60, SUC_lexicon_tags_37 },
    { 0xf5f80f61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83180f65, SUC_lexicon_tags_25 },
    { 0x2fd80f66, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf7f80f68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b180f6a, SUC_lexicon_tags_15 },
    { 0xfbf80f6b, SUC_lexicon_tags_27 },
    { 0xbf980f6c, SUC_lexicon_tags_60 },
    { 0x1d580f6d, SUC_lexicon_tags_27 },
    { 0x93580f6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64380f70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49180f73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8bf80f75, SUC_lexicon_tags_11 },
    { 0x69f80f76, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e380f7e, SUC_lexicon_tags_1 },
    { 0x36780f7f, SUC_lexicon_tags_7 },
    { 0x9cf80f80, SUC_lexicon_tags_7 },
    { 0x49b80f7f, SUC_lexicon_tags_6 },
    { 0xcc180f81, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4dd80f84, SUC_lexicon_tags_17 },
    { 0x7d580f85, SUC_lexicon_tags_12 },
    { 0x39380f86, SUC_lexicon_tags_15 },
    { 0x81180f84, SUC_lexicon_tags_26 },
    { 0xf5b80f88, SUC_lexicon_tags_47 },
    { 0x3980f89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f80f8e, SUC_lexicon_tags_37 },
    { 0x89180f8f, SUC_lexicon_tags_15 },
    { 0x4d980f8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf80f96, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4e380f98, SUC_lexicon_tags_47 },
    { 0xf580f98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d380f9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34780fa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2f80fa6, SUC_lexicon_tags_15 },
    { 0x5ab80fa6, SUC_lexicon_tags_37 },
    { 0x19980fa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f80fad, SUC_lexicon_tags_11 },
    { 0x5bf80fae, SUC_lexicon_tags_0 },
    { 0x62580faf, SUC_lexicon_tags_61 },
    { 0x75180fae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47780fb4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x21580fb6, SUC_lexicon_tags_11 },
    { 0xea780fb7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3380fba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x36180fbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29180fbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c380fc4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e380fc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ef80fc9, SUC_lexicon_tags_28 },
    { 0x98f80fca, SUC_lexicon_tags_55 },
    { 0xd3380fc9, SUC_lexicon_tags_44 },
    { 0x6c780fcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d80fd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f380fd5, SUC_lexicon_tags_1 },
    { 0x22780fd6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf180fd9, SUC_lexicon_tags_15 },
    { 0x28b80fda, SUC_lexicon_tags_4 },
    { 0xabb80fdb, SUC_lexicon_tags_111 },
    { 0x39b80fd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4980fde, SUC_lexicon_tags_45 },
    { 0xa580fdf, SUC_lexicon_tags_14 },
    { 0x26780fdf, SUC_lexicon_tags_14 },
    { 0xcaf80fdf, SUC_lexicon_tags_19 },
    { 0x7ad80fdf, SUC_lexicon_tags_37 },
    { 0x8cf80fe3, SUC_lexicon_tags_11 },
    { 0x6e380fe3, SUC_lexicon_tags_47 },
    { 0x29f80fe3, SUC_lexicon_tags_6 },
    { 0x1dd80fe4, SUC_lexicon_tags_43 },
    { 0x79b80fde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab80feb, SUC_lexicon_tags_26 },
    { 0x83380fec, SUC_lexicon_tags_31 },
    { 0x43d80fed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b80ff0, SUC_lexicon_tags_18 },
    { 0xcf80ff1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdd80ff3, SUC_lexicon_tags_25 },
    { 0xa7d80ff4, SUC_lexicon_tags_32 },
    { 0x67380ff5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3380ffd, SUC_lexicon_tags_17 },
    { 0xb7780ffe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb981000, SUC_lexicon_tags_49 },
    { 0xd4981001, SUC_lexicon_tags_43 },
    { 0xc1f81002, SUC_lexicon_tags_25 },
    { 0xf5581003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfff81005, SUC_lexicon_tags_66 },
    { 0x36181006, SUC_lexicon_tags_1 },
    { 0xc8781007, SUC_lexicon_tags_442 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b8100a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe38100f, SUC_lexicon_tags_29 },
    { 0xd18100f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x71181012, SUC_lexicon_tags_12 },
    { 0x7ef81012, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3981015, SUC_lexicon_tags_37 },
    { 0x3cf81016, SUC_lexicon_tags_17 },
    { 0xc0981016, SUC_lexicon_tags_15 },
    { 0x52781016, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1558101b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf81021, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d81025, SUC_lexicon_tags_13 },
    { 0x20981025, SUC_lexicon_tags_34 },
    { 0xd8181027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a381029, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed8102e, SUC_lexicon_tags_17 },
    { 0xfff8102f, SUC_lexicon_tags_19 },
    { 0xe4f81030, SUC_lexicon_tags_25 },
    { 0x31f8102f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd81036, SUC_lexicon_tags_19 },
    { 0x7f381036, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f98103b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda381041, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81781043, SUC_lexicon_tags_15 },
    { 0xa3d81043, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf3981046, SUC_lexicon_tags_47 },
    { 0x4cd81046, SUC_lexicon_tags_15 },
    { 0xe1381046, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3338104e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73581052, SUC_lexicon_tags_27 },
    { 0xbb781052, SUC_lexicon_tags_11 },
    { 0x6e981053, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd81057, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc8f81059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c78105c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb8105f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd381061, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb181065, SUC_lexicon_tags_2 },
    { 0x7ab81066, SUC_lexicon_tags_7 },
    { 0x44381066, SUC_lexicon_tags_87 },
    { 0xecf81065, SUC_lexicon_tags_17 },
    { 0xc3181069, SUC_lexicon_tags_12 },
    { 0x9c981069, SUC_lexicon_tags_6 },
    { 0x61d8106b, SUC_lexicon_tags_38 },
    { 0x4838106b, SUC_lexicon_tags_8 },
    { 0x1ed8106d, SUC_lexicon_tags_17 },
    { 0xceb8106e, SUC_lexicon_tags_46 },
    { 0xb0d8106b, SUC_lexicon_tags_33 },
    { 0x1798106f, SUC_lexicon_tags_2 },
    { 0x96f81071, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc981075, SUC_lexicon_tags_14 },
    { 0x36781075, SUC_lexicon_tags_45 },
    { 0xdc581077, SUC_lexicon_tags_38 },
    { 0x69381078, SUC_lexicon_tags_43 },
    { 0xb1981078, SUC_lexicon_tags_1 },
    { 0x1358107a, SUC_lexicon_tags_17 },
    { 0xdc78107a, SUC_lexicon_tags_17 },
    { 0x1578107c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0181080, SUC_lexicon_tags_204 },
    { 0x52d81081, SUC_lexicon_tags_15 },
    { 0x6fd81081, SUC_lexicon_tags_8 },
    { 0xcf181082, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd581086, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff8108a, SUC_lexicon_tags_14 },
    { 0x698108a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda78108e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f181090, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5181093, SUC_lexicon_tags_17 },
    { 0x8381094, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b781097, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b181099, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58109c, SUC_lexicon_tags_15 },
    { 0x8f98109d, SUC_lexicon_tags_11 },
    { 0xcb18109d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1810a1, SUC_lexicon_tags_118 },
    { 0x509810a1, SUC_lexicon_tags_38 },
    { 0x14b810a3, SUC_lexicon_tags_116 },
    { 0xc8f810a1, SUC_lexicon_tags_31 },
    { 0xe8d810a5, SUC_lexicon_tags_64 },
    { 0x463810a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x871810a9, SUC_lexicon_tags_43 },
    { 0x5b810aa, SUC_lexicon_tags_12 },
    { 0x10d810ab, SUC_lexicon_tags_27 },
    { 0x343810aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5810b0, SUC_lexicon_tags_8 },
    { 0xc3f810b1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x355810b3, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0x693810b6, SUC_lexicon_tags_2 },
    { 0x911810b7, SUC_lexicon_tags_11 },
    { 0x6a5810b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25810c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9f810c2, SUC_lexicon_tags_53 },
    { 0x65d810c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db810ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c5810cc, SUC_lexicon_tags_37 },
    { 0x119810cd, SUC_lexicon_tags_21 },
    { 0xf83810ce, SUC_lexicon_tags_11 },
    { 0x941810cf, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b810d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499810d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xacf810db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f810de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf59810e0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x77810e2, SUC_lexicon_tags_25 },
    { 0x3dd810e3, SUC_lexicon_tags_17 },
    { 0x153810e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9dd810e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89d810e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d5810ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d5810ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x97810ee, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x723810f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5810f3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x895810f6, SUC_lexicon_tags_11 },
    { 0x3f1810f7, SUC_lexicon_tags_7 },
    { 0xa0b810f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x86f810fa, SUC_lexicon_tags_16 },
    { 0x319810fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557810ff, SUC_lexicon_tags_80 },
    { 0x75981100, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f81103, SUC_lexicon_tags_8 },
    { 0xb2b81104, SUC_lexicon_tags_45 },
    { 0x9ed81105, SUC_lexicon_tags_14 },
    { 0xbf381105, SUC_lexicon_tags_27 },
    { 0x50f81107, SUC_lexicon_tags_9 },
    { 0xdf781106, SUC_lexicon_tags_0 },
    { 0x56581109, SUC_lexicon_tags_37 },
    { 0x5eb8110a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6d8110c, SUC_lexicon_tags_25 },
    { 0x2c98110d, SUC_lexicon_tags_30 },
    { 0x5b78110d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e181110, SUC_lexicon_tags_60 },
    { 0x60381111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c981115, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b581119, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x58111b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db81120, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26581125, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xda581127, SUC_lexicon_tags_1 },
    { 0xf8981128, SUC_lexicon_tags_1 },
    { 0xaa381128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdbd8112b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5298112e, SUC_lexicon_tags_1 },
    { 0xaa18112f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b981133, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb381137, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e781139, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b8113e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9581140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d81146, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc8181148, SUC_lexicon_tags_12 },
    { 0xb1b81149, SUC_lexicon_tags_33 },
    { 0x4e581149, SUC_lexicon_tags_8 },
    { 0x3bb81148, SUC_lexicon_tags_29 },
    { 0x5478114b, SUC_lexicon_tags_45 },
    { 0x4578114b, SUC_lexicon_tags_19 },
    { 0xb618114e, SUC_lexicon_tags_37 },
    { 0x42f8114e, SUC_lexicon_tags_63 },
    { 0xaef8114b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84981154, SUC_lexicon_tags_29 },
    { 0x4ff81155, SUC_lexicon_tags_79 },
    { 0x2c981156, SUC_lexicon_tags_7 },
    { 0xf5581155, SUC_lexicon_tags_37 },
    { 0xc5f81157, SUC_lexicon_tags_80 },
    { 0x90581154, SUC_lexicon_tags_1 },
    { 0xecf8115a, SUC_lexicon_tags_11 },
    { 0x7798115b, SUC_lexicon_tags_25 },
    { 0x7f18115c, SUC_lexicon_tags_17 },
    { 0x6cf8115d, SUC_lexicon_tags_38 },
    { 0x64d8115e, SUC_lexicon_tags_2 },
    { 0x9318115f, SUC_lexicon_tags_30 },
    { 0x998115e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc581165, SUC_lexicon_tags_17 },
    { 0xc4d81166, SUC_lexicon_tags_45 },
    { 0x92581167, SUC_lexicon_tags_10 },
    { 0x6cf81168, SUC_lexicon_tags_9 },
    { 0xe9d81169, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d98116c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89f8116e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe5181170, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d981172, SUC_lexicon_tags_2 },
    { 0xdf581172, SUC_lexicon_tags_70 },
    { 0x2cf81174, SUC_lexicon_tags_37 },
    { 0xf8981172, SUC_lexicon_tags_2 },
    { 0x1d581174, SUC_lexicon_tags_19 },
    { 0x6ff81172, SUC_lexicon_tags_25 },
    { 0x6f81175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3298117a, SUC_lexicon_tags_33 },
    { 0xef38117b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a981181, SUC_lexicon_tags_14 },
    { 0xbbf81181, SUC_lexicon_tags_37 },
    { 0x62d81182, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7981185, SUC_lexicon_tags_45 },
    { 0x7c381186, SUC_lexicon_tags_15 },
    { 0xcb381187, SUC_lexicon_tags_61 },
    { 0x41581187, SUC_lexicon_tags_0 },
    { 0x58781188, SUC_lexicon_tags_15 },
    { 0x77d8118a, SUC_lexicon_tags_2 },
    { 0x4e38118b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4518118d, SUC_lexicon_tags_43 },
    { 0x8bb8118d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b81191, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc381197, SUC_lexicon_tags_26 },
    { 0x8f781198, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2818119e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d811a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21811a5, SUC_lexicon_tags_47 },
    { 0xcf811a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x349811a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427811ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed811b0, SUC_lexicon_tags_37 },
    { 0x637811b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37811b4, SUC_lexicon_tags_37 },
    { 0x59d811b5, SUC_lexicon_tags_26 },
    { 0x59811b6, SUC_lexicon_tags_2 },
    { 0x9e1811b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff811bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd811c5, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9811cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f811d2, SUC_lexicon_tags_25 },
    { 0x717811d2, SUC_lexicon_tags_6 },
    { 0xcf811d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b811d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9811df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415811ec, SUC_lexicon_tags_66 },
    { 0x531811ed, SUC_lexicon_tags_1 },
    { 0x7dd811ed, SUC_lexicon_tags_17 },
    { 0x745811ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd811f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd811f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x781811f8, SUC_lexicon_tags_49 },
    { 0x993811f9, SUC_lexicon_tags_25 },
    { 0xbb811fa, SUC_lexicon_tags_329 },
    { 0x37f811fb, SUC_lexicon_tags_17 },
    { 0x7d5811fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf81201, SUC_lexicon_tags_10 },
    { 0x5af81202, SUC_lexicon_tags_25 },
    { 0x4bb81203, SUC_lexicon_tags_47 },
    { 0x59b81202, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78381209, SUC_lexicon_tags_37 },
    { 0x1ff81209, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8318120c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5758120e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3781212, SUC_lexicon_tags_66 },
    { 0x8cb81213, SUC_lexicon_tags_37 },
    { 0xcfb81213, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa781218, SUC_lexicon_tags_47 },
    { 0xf0981218, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x178121b, SUC_lexicon_tags_4 },
    { 0x738121c, SUC_lexicon_tags_7 },
    { 0xfb98121d, SUC_lexicon_tags_37 },
    { 0x5a38121e, SUC_lexicon_tags_3 },
    { 0x5cb8121f, SUC_lexicon_tags_10 },
    { 0x1a981220, SUC_lexicon_tags_11 },
    { 0x3838121e, SUC_lexicon_tags_77 },
    { 0xc818121d, SUC_lexicon_tags_37 },
    { 0x27781223, SUC_lexicon_tags_11 },
    { 0xd3b81224, SUC_lexicon_tags_6 },
    { 0x84181220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda581229, SUC_lexicon_tags_19 },
    { 0xf2d81229, SUC_lexicon_tags_63 },
    { 0x718122b, SUC_lexicon_tags_2 },
    { 0x3938122b, SUC_lexicon_tags_107 },
    { 0x5eb8122d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50381236, SUC_lexicon_tags_2 },
    { 0xfb981236, SUC_lexicon_tags_27 },
    { 0x5381238, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3558123c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9178123e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8581248, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e581250, SUC_lexicon_tags_22 },
    { 0x80981250, SUC_lexicon_tags_37 },
    { 0x35381252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5f81254, SUC_lexicon_tags_31 },
    { 0x4bd81254, SUC_lexicon_tags_0 },
    { 0x59181254, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb8125a, SUC_lexicon_tags_55 },
    { 0x4d98125b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3238125f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62981264, SUC_lexicon_tags_17 },
    { 0xfe781265, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde781273, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb81277, SUC_lexicon_tags_88 },
    { 0xbd781277, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2178127a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d181283, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8d81285, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e981287, SUC_lexicon_tags_59 },
    { 0xdb581288, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a38128b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb8128e, SUC_lexicon_tags_1 },
    { 0x3c98128e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4381293, SUC_lexicon_tags_45 },
    { 0x41581294, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff8129b, SUC_lexicon_tags_11 },
    { 0x41b8129c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf118129e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x58d812a0, SUC_lexicon_tags_17 },
    { 0x69f812a0, SUC_lexicon_tags_38 },
    { 0xfc5812a2, SUC_lexicon_tags_12 },
    { 0x663812a2, SUC_lexicon_tags_4 },
    { 0x1f812a3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53812ab, SUC_lexicon_tags_11 },
    { 0x537812ac, SUC_lexicon_tags_23 },
    { 0x363812ad, SUC_lexicon_tags_15 },
    { 0x35f812ad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x241812b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d812b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9db812b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d812b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45f812ba, SUC_lexicon_tags_7 },
    { 0xcaf812bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x675812be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1d812c0, SUC_lexicon_tags_25 },
    { 0x49812c0, SUC_lexicon_tags_63 },
    { 0x559812c0, SUC_lexicon_tags_0 },
    { 0x37b812c2, SUC_lexicon_tags_43 },
    { 0x5d5812c3, SUC_lexicon_tags_14 },
    { 0x1d5812c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef812c9, SUC_lexicon_tags_63 },
    { 0x95f812ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff812ce, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x893812d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5812d5, SUC_lexicon_tags_68 },
    { 0xe21812d6, SUC_lexicon_tags_37 },
    { 0x42f812d5, SUC_lexicon_tags_179 },
    { 0xba7812d8, SUC_lexicon_tags_27 },
    { 0x777812d9, SUC_lexicon_tags_31 },
    { 0x479812da, SUC_lexicon_tags_7 },
    { 0xae3812d9, SUC_lexicon_tags_68 },
    { 0xc7d812da, SUC_lexicon_tags_12 },
    { 0xa5812d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33812e1, SUC_lexicon_tags_39 },
    { 0x165812e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493812e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193812f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa51812f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf812f8, SUC_lexicon_tags_15 },
    { 0x1cb812f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x293812fc, SUC_lexicon_tags_114 },
    { 0x29b812fc, SUC_lexicon_tags_54 },
    { 0x803812fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde181306, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff81309, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe798130d, SUC_lexicon_tags_15 },
    { 0x82b8130d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8db81310, SUC_lexicon_tags_17 },
    { 0x95581311, SUC_lexicon_tags_62 },
    { 0x74781312, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b781314, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9f81316, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x34b81318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d98131c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb238131e, SUC_lexicon_tags_32 },
    { 0x3258131f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb81323, SUC_lexicon_tags_17 },
    { 0xb5981324, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b81328, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4398132e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e181334, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9981339, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf858133d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4381340, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5eb81342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17381345, SUC_lexicon_tags_7 },
    { 0x66f81346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef8134a, SUC_lexicon_tags_87 },
    { 0x3358134a, SUC_lexicon_tags_17 },
    { 0xb018134c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc381350, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f81354, SUC_lexicon_tags_33 },
    { 0x45d81354, SUC_lexicon_tags_17 },
    { 0x2581355, SUC_lexicon_tags_17 },
    { 0xec781354, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc981362, SUC_lexicon_tags_37 },
    { 0x95581363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b81368, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d18136d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd8137c, SUC_lexicon_tags_23 },
    { 0xaa98137d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c581381, SUC_lexicon_tags_47 },
    { 0x26981382, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65581386, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f981388, SUC_lexicon_tags_55 },
    { 0xb6181389, SUC_lexicon_tags_57 },
    { 0x7e18138a, SUC_lexicon_tags_12 },
    { 0x9f38138b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea98138d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d581392, SUC_lexicon_tags_14 },
    { 0x4ef81393, SUC_lexicon_tags_27 },
    { 0x27981393, SUC_lexicon_tags_43 },
    { 0xf1d81393, SUC_lexicon_tags_52 },
    { 0x11981392, SUC_lexicon_tags_2 },
    { 0x6d581397, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131813a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f813a5, SUC_lexicon_tags_37 },
    { 0x2cd813a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x76f813a8, SUC_lexicon_tags_46 },
    { 0x36f813a9, SUC_lexicon_tags_15 },
    { 0x1d1813aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedd813ac, SUC_lexicon_tags_14 },
    { 0xbe9813ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5813b4, SUC_lexicon_tags_17 },
    { 0xbb1813b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07813b8, SUC_lexicon_tags_7 },
    { 0x737813b8, SUC_lexicon_tags_27 },
    { 0xf25813b9, SUC_lexicon_tags_28 },
    { 0xde9813bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751813c5, SUC_lexicon_tags_12 },
    { 0xf5d813c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467813ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb813d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871813d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421813db, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xdab813dd, SUC_lexicon_tags_9 },
    { 0xb11813dd, SUC_lexicon_tags_47 },
    { 0x487813df, SUC_lexicon_tags_13 },
    { 0xb0f813e0, SUC_lexicon_tags_27 },
    { 0xe11813e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5813e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1813e7, SUC_lexicon_tags_17 },
    { 0x565813e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fb813ea, SUC_lexicon_tags_14 },
    { 0x991813eb, SUC_lexicon_tags_45 },
    { 0x7b5813ec, SUC_lexicon_tags_92 },
    { 0x317813ed, SUC_lexicon_tags_47 },
    { 0x725813ed, SUC_lexicon_tags_17 },
    { 0xf9b813ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b813f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b813f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x733813f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb813fc, SUC_lexicon_tags_11 },
    { 0xdf813fd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a781400, SUC_lexicon_tags_60 },
    { 0x8a981400, SUC_lexicon_tags_2 },
    { 0xdcd81400, SUC_lexicon_tags_1 },
    { 0x19781403, SUC_lexicon_tags_32 },
    { 0xb2f81402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbaf81406, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bf81408, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a18140c, SUC_lexicon_tags_0 },
    { 0x4c18140c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed381410, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab81413, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2f581415, SUC_lexicon_tags_11 },
    { 0xb9181416, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe718141d, SUC_lexicon_tags_17 },
    { 0xf418141e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8381421, SUC_lexicon_tags_82 },
    { 0x94581422, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2cb81424, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae181426, SUC_lexicon_tags_29 },
    { 0x30781427, SUC_lexicon_tags_35 },
    { 0xc3781427, SUC_lexicon_tags_25 },
    { 0xd9181429, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b81431, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb781433, SUC_lexicon_tags_28 },
    { 0xaf981434, SUC_lexicon_tags_12 },
    { 0xc8981435, SUC_lexicon_tags_1 },
    { 0x22181433, SUC_lexicon_tags_11 },
    { 0x8e981437, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xad981439, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f8143c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9c98143e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9181442, SUC_lexicon_tags_53 },
    { 0x3d781443, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84381447, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52581449, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f81452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8318145c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x998145f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76381462, SUC_lexicon_tags_13 },
    { 0x82d81463, SUC_lexicon_tags_32 },
    { 0xdd581464, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd2781466, SUC_lexicon_tags_0 },
    { 0x1af81466, SUC_lexicon_tags_10 },
    { 0xe8181468, SUC_lexicon_tags_49 },
    { 0x61981468, SUC_lexicon_tags_7 },
    { 0xc581467, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xb6d8146c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc58146e, SUC_lexicon_tags_8 },
    { 0xd098146f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9d981471, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3f581473, SUC_lexicon_tags_15 },
    { 0x3dd81474, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90781478, SUC_lexicon_tags_60 },
    { 0x90381479, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1381482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3cb81484, SUC_lexicon_tags_15 },
    { 0x4ab81484, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f781487, SUC_lexicon_tags_38 },
    { 0xfef81488, SUC_lexicon_tags_33 },
    { 0xce581488, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb8148d, SUC_lexicon_tags_47 },
    { 0x12b8148d, SUC_lexicon_tags_104 },
    { 0xcd38148e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f81492, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb998149a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe4f8149c, SUC_lexicon_tags_12 },
    { 0x6178149d, SUC_lexicon_tags_26 },
    { 0xb078149e, SUC_lexicon_tags_11 },
    { 0x2f58149e, SUC_lexicon_tags_8 },
    { 0x199814a0, SUC_lexicon_tags_37 },
    { 0xdf814a1, SUC_lexicon_tags_77 },
    { 0xa5b814a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd814a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9814ac, SUC_lexicon_tags_34 },
    { 0x949814ad, SUC_lexicon_tags_35 },
    { 0xb43814ad, SUC_lexicon_tags_12 },
    { 0xc1b814af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f814bb, SUC_lexicon_tags_60 },
    { 0xb2f814bc, SUC_lexicon_tags_32 },
    { 0x89b814bd, SUC_lexicon_tags_15 },
    { 0x9db814bd, SUC_lexicon_tags_26 },
    { 0x9cf814bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7814c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b1814c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87814c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d814cb, SUC_lexicon_tags_27 },
    { 0x635814cb, SUC_lexicon_tags_11 },
    { 0xc87814cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423814d2, SUC_lexicon_tags_25 },
    { 0xb41814d3, SUC_lexicon_tags_11 },
    { 0x895814d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877814da, SUC_lexicon_tags_28 },
    { 0xb69814db, SUC_lexicon_tags_0 },
    { 0x155814dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x753814df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83814e7, SUC_lexicon_tags_47 },
    { 0xced814e8, SUC_lexicon_tags_199 },
    { 0x4ff814e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5814ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfb5814ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3814f1, SUC_lexicon_tags_46 },
    { 0xb87814f1, SUC_lexicon_tags_37 },
    { 0x7b1814f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb5814f5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xda1814f7, SUC_lexicon_tags_12 },
    { 0x653814f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe87814fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x403814fd, SUC_lexicon_tags_27 },
    { 0xf51814fd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a781501, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd8150a, SUC_lexicon_tags_29 },
    { 0x3258150b, SUC_lexicon_tags_25 },
    { 0xfb38150b, SUC_lexicon_tags_1 },
    { 0x7a38150d, SUC_lexicon_tags_65 },
    { 0x3438150b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d81511, SUC_lexicon_tags_5 },
    { 0x19b81512, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x30d81514, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa381517, SUC_lexicon_tags_7 },
    { 0x6ff81518, SUC_lexicon_tags_60 },
    { 0x34d81519, SUC_lexicon_tags_60 },
    { 0x8978151a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38151e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5038152b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc258152e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80781533, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd81537, SUC_lexicon_tags_25 },
    { 0x7a981537, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb8153b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b81543, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2ff81545, SUC_lexicon_tags_33 },
    { 0x58781545, SUC_lexicon_tags_7 },
    { 0x20781546, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cf81549, SUC_lexicon_tags_4 },
    { 0x6538154a, SUC_lexicon_tags_0 },
    { 0x5818154a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8558154d, SUC_lexicon_tags_43 },
    { 0xaad8154e, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb81554, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3381557, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x7718155a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f81561, SUC_lexicon_tags_45 },
    { 0x19181562, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56781566, SUC_lexicon_tags_37 },
    { 0xcab81567, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d8156d, SUC_lexicon_tags_9 },
    { 0x6c58156e, SUC_lexicon_tags_7 },
    { 0x5478156f, SUC_lexicon_tags_43 },
    { 0x99381570, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6e181572, SUC_lexicon_tags_28 },
    { 0x50981572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e381575, SUC_lexicon_tags_14 },
    { 0x13f81576, SUC_lexicon_tags_17 },
    { 0xc3581576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6798157b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdff8157d, SUC_lexicon_tags_11 },
    { 0x2b8157d, SUC_lexicon_tags_25 },
    { 0xe8f8157d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2158158b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb981594, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c981597, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8159e, SUC_lexicon_tags_25 },
    { 0x3858159e, SUC_lexicon_tags_15 },
    { 0x503815a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb815a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d3815a7, SUC_lexicon_tags_25 },
    { 0x2f5815a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37815ab, SUC_lexicon_tags_7 },
    { 0xf05815ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1815b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743815ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf815bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973815c7, SUC_lexicon_tags_37 },
    { 0x849815c8, SUC_lexicon_tags_15 },
    { 0x965815c7, SUC_lexicon_tags_17 },
    { 0x753815c7, SUC_lexicon_tags_34 },
    { 0x1cd815ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3815d1, SUC_lexicon_tags_9 },
    { 0x88d815d2, SUC_lexicon_tags_2 },
    { 0xe33815d3, SUC_lexicon_tags_39 },
    { 0x8ed815d4, SUC_lexicon_tags_17 },
    { 0x3eb815d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35815d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x571815da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1815de, SUC_lexicon_tags_34 },
    { 0x5f1815de, SUC_lexicon_tags_2 },
    { 0x4ed815e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xafb815e2, SUC_lexicon_tags_70 },
    { 0x2cd815e3, SUC_lexicon_tags_0 },
    { 0xe6d815e2, SUC_lexicon_tags_40 },
    { 0xfbb815e5, SUC_lexicon_tags_61 },
    { 0x41b815e6, SUC_lexicon_tags_17 },
    { 0xfc5815e5, SUC_lexicon_tags_26 },
    { 0xe01815e6, SUC_lexicon_tags_32 },
    { 0xb13815e6, SUC_lexicon_tags_14 },
    { 0xc5815e3, SUC_lexicon_tags_1 },
    { 0x647815eb, SUC_lexicon_tags_23 },
    { 0x769815ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87815ef, SUC_lexicon_tags_396 },
    { 0xac7815ef, SUC_lexicon_tags_3 },
    { 0x2ad815ef, SUC_lexicon_tags_9 },
    { 0xcd9815f2, SUC_lexicon_tags_27 },
    { 0xd1f815f2, SUC_lexicon_tags_2 },
    { 0x123815f0, SUC_lexicon_tags_25 },
    { 0x803815f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f815f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb815fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a1815fd, SUC_lexicon_tags_0 },
    { 0xb2f815fe, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc781601, SUC_lexicon_tags_124 },
    { 0xc6381602, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd8160a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d81612, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x74781615, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d8161c, SUC_lexicon_tags_43 },
    { 0xd598161d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8278161f, SUC_lexicon_tags_11 },
    { 0x2db81620, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d81624, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17d81626, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd8162b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6338162d, SUC_lexicon_tags_37 },
    { 0x8758162e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d81631, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f781634, SUC_lexicon_tags_4 },
    { 0x71181635, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac581639, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b8163e, SUC_lexicon_tags_23 },
    { 0x7ff8163f, SUC_lexicon_tags_34 },
    { 0xeef81640, SUC_lexicon_tags_2 },
    { 0xc8381641, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2138164e, SUC_lexicon_tags_30 },
    { 0xf658164f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b81652, SUC_lexicon_tags_14 },
    { 0xfe581653, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb81659, SUC_lexicon_tags_56 },
    { 0x4dd8165a, SUC_lexicon_tags_26 },
    { 0x11781659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3578165d, SUC_lexicon_tags_24 },
    { 0x3ab8165e, SUC_lexicon_tags_14 },
    { 0x22d8165f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e181667, SUC_lexicon_tags_6 },
    { 0x37981668, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7b8166a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2898166e, SUC_lexicon_tags_57 },
    { 0x2778166f, SUC_lexicon_tags_17 },
    { 0x8e58166f, SUC_lexicon_tags_45 },
    { 0x4b181671, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3858167b, SUC_lexicon_tags_17 },
    { 0xc818167c, SUC_lexicon_tags_10 },
    { 0x7b38167c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x13181680, SUC_lexicon_tags_7 },
    { 0x2c581681, SUC_lexicon_tags_37 },
    { 0xc981682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a981684, SUC_lexicon_tags_23 },
    { 0xd6981685, SUC_lexicon_tags_37 },
    { 0x4fd81686, SUC_lexicon_tags_25 },
    { 0x79181685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd8168a, SUC_lexicon_tags_2 },
    { 0x5c18168b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe598168d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d981690, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a581693, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd981695, SUC_lexicon_tags_19 },
    { 0xd8d81696, SUC_lexicon_tags_25 },
    { 0x8381696, SUC_lexicon_tags_45 },
    { 0xd1781695, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6678169c, SUC_lexicon_tags_54 },
    { 0x4b38169d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d816a0, SUC_lexicon_tags_87 },
    { 0xc55816a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d816a4, SUC_lexicon_tags_7 },
    { 0x5b816a5, SUC_lexicon_tags_23 },
    { 0x20b816a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d816ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5cf816ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x18b816b0, SUC_lexicon_tags_11 },
    { 0xd65816b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325816b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7816b8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d816bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8c3816bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe41816bf, SUC_lexicon_tags_31 },
    { 0x197816bf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x163816c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37816c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x24b816c7, SUC_lexicon_tags_36 },
    { 0x74b816c8, SUC_lexicon_tags_7 },
    { 0x2d9816c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3816cc, SUC_lexicon_tags_17 },
    { 0xa63816cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5816cf, SUC_lexicon_tags_17 },
    { 0x68b816d0, SUC_lexicon_tags_2 },
    { 0x7c9816d1, SUC_lexicon_tags_17 },
    { 0xec9816d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x155816d4, SUC_lexicon_tags_37 },
    { 0x933816d5, SUC_lexicon_tags_48 },
    { 0x19816d6, SUC_lexicon_tags_17 },
    { 0x231816d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1816db, SUC_lexicon_tags_7 },
    { 0x8f3816dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5816e6, SUC_lexicon_tags_42 },
    { 0x931816e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7816ea, SUC_lexicon_tags_46 },
    { 0x88b816eb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853816ef, SUC_lexicon_tags_15 },
    { 0x45816f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc73816f2, SUC_lexicon_tags_26 },
    { 0x477816f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1816f6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb816f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x661816fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f81700, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe6b81702, SUC_lexicon_tags_37 },
    { 0xd7d81703, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9381706, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b81709, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2338170c, SUC_lexicon_tags_17 },
    { 0x8a18170c, SUC_lexicon_tags_15 },
    { 0x4518170c, SUC_lexicon_tags_12 },
    { 0xecd8170f, SUC_lexicon_tags_17 },
    { 0x47d8170c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ad81712, SUC_lexicon_tags_11 },
    { 0x4e181713, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf81718, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5778171d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xad8171f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xce781721, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf81725, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb8172b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf818172d, SUC_lexicon_tags_63 },
    { 0x22f8172e, SUC_lexicon_tags_12 },
    { 0x3d98172d, SUC_lexicon_tags_30 },
    { 0x9e181730, SUC_lexicon_tags_17 },
    { 0xa3781731, SUC_lexicon_tags_160 },
    { 0xf7981732, SUC_lexicon_tags_24 },
    { 0x4bb81730, SUC_lexicon_tags_32 },
    { 0xb6b81734, SUC_lexicon_tags_0 },
    { 0xe5981733, SUC_lexicon_tags_37 },
    { 0x6d381736, SUC_lexicon_tags_35 },
    { 0x3cf81736, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d8173a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5838173e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f581740, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b781743, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe178174c, SUC_lexicon_tags_2 },
    { 0x41b8174d, SUC_lexicon_tags_7 },
    { 0xf498174e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad81754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f8175b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb358175d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd3d8175f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1781763, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdd81765, SUC_lexicon_tags_1119 },
    { 0, NULL },
    { 0x22181767, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d8176f, SUC_lexicon_tags_113 },
    { 0x83181770, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72d81772, SUC_lexicon_tags_0 },
    { 0xc0b81773, SUC_lexicon_tags_11 },
    { 0xdc181774, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f81778, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4938177c, SUC_lexicon_tags_87 },
    { 0x5398177d, SUC_lexicon_tags_23 },
    { 0xb4b8177e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b81782, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6181784, SUC_lexicon_tags_11 },
    { 0x8ef81784, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x68381787, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b98178b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4438178f, SUC_lexicon_tags_55 },
    { 0x3781790, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa3f81792, SUC_lexicon_tags_25 },
    { 0x1af81792, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5981799, SUC_lexicon_tags_43 },
    { 0xad381799, SUC_lexicon_tags_1 },
    { 0x2298179b, SUC_lexicon_tags_47 },
    { 0xc618179c, SUC_lexicon_tags_11 },
    { 0x91f8179d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d817a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e5817a2, SUC_lexicon_tags_12 },
    { 0x3c9817a3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f817a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219817ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b817b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f817be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf7817c0, SUC_lexicon_tags_13 },
    { 0x973817c1, SUC_lexicon_tags_23 },
    { 0xff5817c0, SUC_lexicon_tags_2 },
    { 0x821817c3, SUC_lexicon_tags_14 },
    { 0x8cd817c4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfa1817c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d817c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d817cc, SUC_lexicon_tags_17 },
    { 0x561817cd, SUC_lexicon_tags_37 },
    { 0x5f7817ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d817d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f817df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283817e6, SUC_lexicon_tags_19 },
    { 0xee9817e6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3817ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe67817ec, SUC_lexicon_tags_15 },
    { 0x16b817ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49817f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771817f8, SUC_lexicon_tags_25 },
    { 0x171817f8, SUC_lexicon_tags_14 },
    { 0xb1817f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf817fd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x775817ff, SUC_lexicon_tags_37 },
    { 0x9e581800, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72381806, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f81809, SUC_lexicon_tags_6 },
    { 0x6b8180a, SUC_lexicon_tags_28 },
    { 0x1b98180a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb81812, SUC_lexicon_tags_7 },
    { 0x62981813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb81815, SUC_lexicon_tags_56 },
    { 0xabb81816, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d8181c, SUC_lexicon_tags_12 },
    { 0x1958181d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4f8181f, SUC_lexicon_tags_298 },
    { 0x44d81820, SUC_lexicon_tags_46 },
    { 0xe1f81821, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43581824, SUC_lexicon_tags_47 },
    { 0xb3181824, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde781827, SUC_lexicon_tags_60 },
    { 0xb6581828, SUC_lexicon_tags_15 },
    { 0xe781827, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b8182c, SUC_lexicon_tags_11 },
    { 0x5ed8182c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff81830, SUC_lexicon_tags_28 },
    { 0x2a781831, SUC_lexicon_tags_8 },
    { 0x15981830, SUC_lexicon_tags_25 },
    { 0xdd381833, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f98183d, SUC_lexicon_tags_15 },
    { 0x5a38183e, SUC_lexicon_tags_1 },
    { 0xa2b8183d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36381843, SUC_lexicon_tags_34 },
    { 0xf181844, SUC_lexicon_tags_2 },
    { 0x5f81844, SUC_lexicon_tags_335 },
    { 0x8e181846, SUC_lexicon_tags_19 },
    { 0x2a981846, SUC_lexicon_tags_10 },
    { 0x85581847, SUC_lexicon_tags_12 },
    { 0x27b81848, SUC_lexicon_tags_19 },
    { 0x1dd81849, SUC_lexicon_tags_25 },
    { 0x60181845, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f181851, SUC_lexicon_tags_0 },
    { 0xf5b81852, SUC_lexicon_tags_45 },
    { 0x4bd81853, SUC_lexicon_tags_45 },
    { 0x7e581853, SUC_lexicon_tags_15 },
    { 0xb5f81851, SUC_lexicon_tags_25 },
    { 0x53581856, SUC_lexicon_tags_39 },
    { 0x4b781857, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc998185a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78781863, SUC_lexicon_tags_14 },
    { 0x3581864, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa981869, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8186f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b81872, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd81876, SUC_lexicon_tags_9 },
    { 0xbb81876, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d8187d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6ab8187f, SUC_lexicon_tags_39 },
    { 0x8ab81880, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2581882, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf81885, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9af81887, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3918188f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f81899, SUC_lexicon_tags_27 },
    { 0xcf181899, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6818189c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc478189e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b818a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15818a4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa17818a6, SUC_lexicon_tags_17 },
    { 0x8ab818a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f818ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x753818ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdbf818b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x47d818b2, SUC_lexicon_tags_7 },
    { 0xc23818b2, SUC_lexicon_tags_25 },
    { 0xcf818b3, SUC_lexicon_tags_1 },
    { 0x275818b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b818c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea3818c3, SUC_lexicon_tags_6 },
    { 0xbbf818c4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7818d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d818d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b818dc, SUC_lexicon_tags_12 },
    { 0xf79818dd, SUC_lexicon_tags_79 },
    { 0xe9818dc, SUC_lexicon_tags_2 },
    { 0x3a9818df, SUC_lexicon_tags_48 },
    { 0xe6f818e0, SUC_lexicon_tags_17 },
    { 0x7b7818dc, SUC_lexicon_tags_11 },
    { 0xab818e0, SUC_lexicon_tags_47 },
    { 0xe8d818e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x213818e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x749818e8, SUC_lexicon_tags_37 },
    { 0x491818e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a9818eb, SUC_lexicon_tags_49 },
    { 0xd4f818ec, SUC_lexicon_tags_107 },
    { 0x39f818ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcd7818ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f818f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x291818fa, SUC_lexicon_tags_15 },
    { 0x8f5818fa, SUC_lexicon_tags_30 },
    { 0x27f818fc, SUC_lexicon_tags_38 },
    { 0xcc9818fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db81900, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe381904, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf381908, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd8190d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3118190f, SUC_lexicon_tags_46 },
    { 0x9ef81910, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x11d81912, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff381914, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3b781916, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15d81918, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d8191b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71b8191d, SUC_lexicon_tags_11 },
    { 0x36f8191d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce581920, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xacf81922, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54981926, SUC_lexicon_tags_12 },
    { 0x18581927, SUC_lexicon_tags_7 },
    { 0x20781928, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d8192e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5181931, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f81934, SUC_lexicon_tags_17 },
    { 0x7ed81935, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaf381937, SUC_lexicon_tags_1 },
    { 0xea981938, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66381947, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df8194d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d381951, SUC_lexicon_tags_17 },
    { 0x29781952, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc6d81954, SUC_lexicon_tags_49 },
    { 0xc181955, SUC_lexicon_tags_15 },
    { 0xfbd81954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee381958, SUC_lexicon_tags_2 },
    { 0xb4d81958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd8195c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x838195f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3781961, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b81964, SUC_lexicon_tags_17 },
    { 0x28981965, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d8196b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc781970, SUC_lexicon_tags_25 },
    { 0xb2981971, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d81978, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff8197e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3bf81980, SUC_lexicon_tags_60 },
    { 0x8ff81981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab81986, SUC_lexicon_tags_25 },
    { 0x51581987, SUC_lexicon_tags_15 },
    { 0x56581988, SUC_lexicon_tags_27 },
    { 0xed981989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb798198f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f81996, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2178199c, SUC_lexicon_tags_14 },
    { 0x8698199d, SUC_lexicon_tags_25 },
    { 0x7c18199d, SUC_lexicon_tags_11 },
    { 0x53f8199e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1df819a1, SUC_lexicon_tags_49 },
    { 0x86d819a2, SUC_lexicon_tags_18 },
    { 0xd37819a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf7819a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x933819a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x72d819aa, SUC_lexicon_tags_15 },
    { 0xe07819aa, SUC_lexicon_tags_1 },
    { 0x1d819ac, SUC_lexicon_tags_33 },
    { 0xdb1819ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f819b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdbb819b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59819b7, SUC_lexicon_tags_12 },
    { 0x123819b7, SUC_lexicon_tags_23 },
    { 0xc85819b7, SUC_lexicon_tags_25 },
    { 0x9f819ba, SUC_lexicon_tags_1 },
    { 0xe1819b9, SUC_lexicon_tags_37 },
    { 0xbb1819ba, SUC_lexicon_tags_29 },
    { 0x1b3819b8, SUC_lexicon_tags_33 },
    { 0x791819be, SUC_lexicon_tags_37 },
    { 0x471819bb, SUC_lexicon_tags_11 },
    { 0xd49819ba, SUC_lexicon_tags_9 },
    { 0xc83819be, SUC_lexicon_tags_59 },
    { 0xe27819ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d819ca, SUC_lexicon_tags_69 },
    { 0xdb9819cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55819d6, SUC_lexicon_tags_12 },
    { 0xf3b819d6, SUC_lexicon_tags_15 },
    { 0x5f5819d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7819db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357819e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51819e2, SUC_lexicon_tags_60 },
    { 0x3b9819e3, SUC_lexicon_tags_63 },
    { 0x409819e2, SUC_lexicon_tags_0 },
    { 0x80d819e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x145819e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcd819ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77f819ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01819ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xef3819f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03819f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71819fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x395819fd, SUC_lexicon_tags_11 },
    { 0xf73819fe, SUC_lexicon_tags_113 },
    { 0x899819fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d81a02, SUC_lexicon_tags_1 },
    { 0xd3981a03, SUC_lexicon_tags_30 },
    { 0x86b81a04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96581a08, SUC_lexicon_tags_2 },
    { 0xec581a08, SUC_lexicon_tags_1 },
    { 0xf4781a0a, SUC_lexicon_tags_1 },
    { 0xf7181a0b, SUC_lexicon_tags_37 },
    { 0x90581a09, SUC_lexicon_tags_39 },
    { 0xf4581a0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x99981a0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f981a16, SUC_lexicon_tags_26 },
    { 0xd9781a17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7981a1c, SUC_lexicon_tags_47 },
    { 0xcdb81a1d, SUC_lexicon_tags_47 },
    { 0x6fb81a1d, SUC_lexicon_tags_33 },
    { 0x8f581a1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde581a28, SUC_lexicon_tags_31 },
    { 0x2ff81a28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b81a2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f81a35, SUC_lexicon_tags_25 },
    { 0x19581a36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4381a38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80781a3e, SUC_lexicon_tags_30 },
    { 0x81f81a3f, SUC_lexicon_tags_32 },
    { 0x47b81a40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69581a42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c181a45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x41981a48, SUC_lexicon_tags_37 },
    { 0xc7181a48, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4bf81a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74781a4d, SUC_lexicon_tags_25 },
    { 0x1d81a4e, SUC_lexicon_tags_37 },
    { 0xbe581a4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d81a54, SUC_lexicon_tags_10 },
    { 0x47f81a55, SUC_lexicon_tags_19 },
    { 0xddb81a55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f81a5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4381a5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde981a5e, SUC_lexicon_tags_29 },
    { 0xf3d81a5e, SUC_lexicon_tags_84 },
    { 0x6f581a60, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3181a63, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf81a70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3581a73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73781a77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b81a7d, SUC_lexicon_tags_45 },
    { 0x1d381a7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5181a84, SUC_lexicon_tags_19 },
    { 0x9b781a85, SUC_lexicon_tags_9 },
    { 0x7cd81a86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80781a88, SUC_lexicon_tags_11 },
    { 0xfdd81a89, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f81a8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81181a90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb81a94, SUC_lexicon_tags_12 },
    { 0x6cf81a95, SUC_lexicon_tags_7 },
    { 0x8b81a94, SUC_lexicon_tags_17 },
    { 0x7d181a97, SUC_lexicon_tags_9 },
    { 0x28d81a97, SUC_lexicon_tags_11 },
    { 0x7e381a96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55981a9b, SUC_lexicon_tags_2 },
    { 0xa4f81a9c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4e981a9e, SUC_lexicon_tags_10 },
    { 0xe8981a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f81aa2, SUC_lexicon_tags_17 },
    { 0xbfb81aa3, SUC_lexicon_tags_49 },
    { 0x57581aa3, SUC_lexicon_tags_0 },
    { 0xe0581aa3, SUC_lexicon_tags_37 },
    { 0x4d181aa4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xecd81aa8, SUC_lexicon_tags_19 },
    { 0x22981aa9, SUC_lexicon_tags_55 },
    { 0xc7981aa9, SUC_lexicon_tags_15 },
    { 0xac981aab, SUC_lexicon_tags_17 },
    { 0xc1781aac, SUC_lexicon_tags_37 },
    { 0x13781aac, SUC_lexicon_tags_17 },
    { 0x45f81aab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b81ab1, SUC_lexicon_tags_15 },
    { 0x1ff81ab1, SUC_lexicon_tags_23 },
    { 0xe0b81ab3, SUC_lexicon_tags_31 },
    { 0xcfd81ab2, SUC_lexicon_tags_63 },
    { 0x1c381ab5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf381ab7, SUC_lexicon_tags_1 },
    { 0xa981ab8, SUC_lexicon_tags_15 },
    { 0xa3b81ab7, SUC_lexicon_tags_8 },
    { 0x51f81ab7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x56981abc, SUC_lexicon_tags_77 },
    { 0xfd981abd, SUC_lexicon_tags_12 },
    { 0x5fb81abe, SUC_lexicon_tags_37 },
    { 0x1ad81abf, SUC_lexicon_tags_37 },
    { 0xe4581abc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5d81ac2, SUC_lexicon_tags_33 },
    { 0x54181ac3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b381ac6, SUC_lexicon_tags_28 },
    { 0x2c981ac7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d81aca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6781ace, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b81ad2, SUC_lexicon_tags_0 },
    { 0xc4981ad2, SUC_lexicon_tags_30 },
    { 0xee181ad4, SUC_lexicon_tags_7 },
    { 0x5b381ad5, SUC_lexicon_tags_60 },
    { 0xfeb81ad5, SUC_lexicon_tags_23 },
    { 0x73581ad5, SUC_lexicon_tags_38 },
    { 0xdbd81ad8, SUC_lexicon_tags_25 },
    { 0x6ef81ad9, SUC_lexicon_tags_29 },
    { 0x34f81ad7, SUC_lexicon_tags_37 },
    { 0xfdb81adb, SUC_lexicon_tags_25 },
    { 0x76d81adc, SUC_lexicon_tags_27 },
    { 0x36981adb, SUC_lexicon_tags_9 },
    { 0xce581ad4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d81ae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75381ae6, SUC_lexicon_tags_14 },
    { 0x30981ae6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab81aea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2581aed, SUC_lexicon_tags_11 },
    { 0xbd81aee, SUC_lexicon_tags_47 },
    { 0x3ab81aef, SUC_lexicon_tags_246 },
    { 0x2a981af0, SUC_lexicon_tags_2 },
    { 0x30181af1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c981af3, SUC_lexicon_tags_25 },
    { 0x7a181af4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x60981af6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d981aff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xde181b01, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6181b06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5981b08, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11781b0e, SUC_lexicon_tags_9 },
    { 0xf2f81b0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27d81b11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6fd81b13, SUC_lexicon_tags_11 },
    { 0x96781b13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f381b1a, SUC_lexicon_tags_15 },
    { 0xa6b81b1a, SUC_lexicon_tags_17 },
    { 0xee781b1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1581b1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1581b24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f781b29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18b81b2b, SUC_lexicon_tags_2 },
    { 0x33381b2c, SUC_lexicon_tags_2 },
    { 0xbb381b2d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a181b34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59781b41, SUC_lexicon_tags_25 },
    { 0x8e181b42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7781b44, SUC_lexicon_tags_19 },
    { 0x90181b45, SUC_lexicon_tags_7 },
    { 0x72781b44, SUC_lexicon_tags_11 },
    { 0x46b81b44, SUC_lexicon_tags_38 },
    { 0x81d81b48, SUC_lexicon_tags_63 },
    { 0xb1381b46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x76381b4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d81b4f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa581b52, SUC_lexicon_tags_1 },
    { 0x13b81b53, SUC_lexicon_tags_15 },
    { 0x53781b53, SUC_lexicon_tags_7 },
    { 0x5d381b55, SUC_lexicon_tags_11 },
    { 0x1d181b55, SUC_lexicon_tags_7 },
    { 0xa4381b54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b81b5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb81b5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99181b62, SUC_lexicon_tags_23 },
    { 0x2bb81b63, SUC_lexicon_tags_7 },
    { 0xb4781b64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1ab81b66, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x71781b69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5181b6e, SUC_lexicon_tags_23 },
    { 0x3df81b6f, SUC_lexicon_tags_47 },
    { 0x76781b6f, SUC_lexicon_tags_49 },
    { 0x63b81b71, SUC_lexicon_tags_17 },
    { 0xbf781b72, SUC_lexicon_tags_60 },
    { 0x4f181b72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd981b77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b81b7e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13981b85, SUC_lexicon_tags_0 },
    { 0xc781b85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f81b90, SUC_lexicon_tags_17 },
    { 0x44381b90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53b81b93, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f81b97, SUC_lexicon_tags_264 },
    { 0xf181b97, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae181b9d, SUC_lexicon_tags_17 },
    { 0x74381b9d, SUC_lexicon_tags_31 },
    { 0xca981b9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffb81ba1, SUC_lexicon_tags_2 },
    { 0xa3781ba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee981ba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c381ba8, SUC_lexicon_tags_1 },
    { 0xd5d81ba9, SUC_lexicon_tags_14 },
    { 0x60181baa, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b781baf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x86d81bb1, SUC_lexicon_tags_70 },
    { 0x65781bb1, SUC_lexicon_tags_7 },
    { 0x18981bb3, SUC_lexicon_tags_9 },
    { 0xd6381bb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbfb81bb6, SUC_lexicon_tags_6 },
    { 0xf981bb7, SUC_lexicon_tags_49 },
    { 0x33181bb8, SUC_lexicon_tags_37 },
    { 0x5a181bb8, SUC_lexicon_tags_1 },
    { 0x94b81bba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea181bbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad81bbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f81bc5, SUC_lexicon_tags_68 },
    { 0x6e981bc5, SUC_lexicon_tags_1 },
    { 0xe981bc7, SUC_lexicon_tags_32 },
    { 0xbdd81bc8, SUC_lexicon_tags_77 },
    { 0x44b81bc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42381bcf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x73f81bd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54181bd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35981bd5, SUC_lexicon_tags_15 },
    { 0x17181bd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9581bd8, SUC_lexicon_tags_15 },
    { 0xae781bd8, SUC_lexicon_tags_61 },
    { 0x31d81bd8, SUC_lexicon_tags_43 },
    { 0x3981bdb, SUC_lexicon_tags_23 },
    { 0x75181bdc, SUC_lexicon_tags_13 },
    { 0xcf381bdc, SUC_lexicon_tags_12 },
    { 0x58381bda, SUC_lexicon_tags_28 },
    { 0x40181bde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b81be7, SUC_lexicon_tags_4 },
    { 0xb4781be7, SUC_lexicon_tags_37 },
    { 0x8bf81be8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc381beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92381bee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x90381bf1, SUC_lexicon_tags_15 },
    { 0x9e581bf2, SUC_lexicon_tags_17 },
    { 0x47781bf3, SUC_lexicon_tags_2 },
    { 0x21f81bf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9181bf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15581bfb, SUC_lexicon_tags_30 },
    { 0x581bfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b81c00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab381c03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11f81c05, SUC_lexicon_tags_8 },
    { 0x39581c06, SUC_lexicon_tags_28 },
    { 0x7d981c07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d381c0a, SUC_lexicon_tags_37 },
    { 0x75781c0b, SUC_lexicon_tags_11 },
    { 0xd8381c0a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x30981c0e, SUC_lexicon_tags_37 },
    { 0xf6581c0f, SUC_lexicon_tags_15 },
    { 0x44d81c10, SUC_lexicon_tags_70 },
    { 0xac581c10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b81c15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f81c1a, SUC_lexicon_tags_21 },
    { 0x78d81c1b, SUC_lexicon_tags_4 },
    { 0x1a981c1c, SUC_lexicon_tags_15 },
    { 0xc2581c1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9181c21, SUC_lexicon_tags_31 },
    { 0x1b981c21, SUC_lexicon_tags_17 },
    { 0x76181c23, SUC_lexicon_tags_21 },
    { 0xe1d81c24, SUC_lexicon_tags_127 },
    { 0xae181c23, SUC_lexicon_tags_13 },
    { 0x8f781c26, SUC_lexicon_tags_45 },
    { 0xff581c27, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe581c2c, SUC_lexicon_tags_28 },
    { 0xb1581c2d, SUC_lexicon_tags_32 },
    { 0x12b81c2e, SUC_lexicon_tags_47 },
    { 0x55f81c2c, SUC_lexicon_tags_31 },
    { 0x67181c30, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7181c33, SUC_lexicon_tags_37 },
    { 0x3ad81c34, SUC_lexicon_tags_17 },
    { 0x70d81c34, SUC_lexicon_tags_27 },
    { 0xf7d81c36, SUC_lexicon_tags_77 },
    { 0x31f81c37, SUC_lexicon_tags_21 },
    { 0x8bb81c38, SUC_lexicon_tags_25 },
    { 0x4cf81c35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77381c3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e181c3f, SUC_lexicon_tags_45 },
    { 0x54181c3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13981c44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4181c46, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d81c4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d81c4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b81c50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf381c52, SUC_lexicon_tags_37 },
    { 0xe1581c53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe181c58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d81c5b, SUC_lexicon_tags_9 },
    { 0x9e581c5c, SUC_lexicon_tags_37 },
    { 0xa3381c5d, SUC_lexicon_tags_1 },
    { 0x11581c5b, SUC_lexicon_tags_2 },
    { 0x2bd81c5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc981c61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x53f81c63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2581c65, SUC_lexicon_tags_33 },
    { 0x38d81c65, SUC_lexicon_tags_10 },
    { 0x60981c67, SUC_lexicon_tags_17 },
    { 0x6ab81c68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45181c6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x78981c6e, SUC_lexicon_tags_77 },
    { 0x61d81c6f, SUC_lexicon_tags_16 },
    { 0x5db81c70, SUC_lexicon_tags_43 },
    { 0x58181c6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xead81c73, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c781c78, SUC_lexicon_tags_10 },
    { 0x2fd81c79, SUC_lexicon_tags_14 },
    { 0x87381c7a, SUC_lexicon_tags_12 },
    { 0xb8181c7a, SUC_lexicon_tags_37 },
    { 0xdbf81c7c, SUC_lexicon_tags_11 },
    { 0xda781c7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd181c87, SUC_lexicon_tags_45 },
    { 0xfc381c87, SUC_lexicon_tags_2 },
    { 0x51781c89, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x76381c8c, SUC_lexicon_tags_39 },
    { 0x26f81c8d, SUC_lexicon_tags_19 },
    { 0x3b181c8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe581c91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f81c95, SUC_lexicon_tags_12 },
    { 0x85381c96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a781c99, SUC_lexicon_tags_14 },
    { 0xf4181c9a, SUC_lexicon_tags_13 },
    { 0x35581c9b, SUC_lexicon_tags_37 },
    { 0xc3581c9c, SUC_lexicon_tags_15 },
    { 0x17b81c9c, SUC_lexicon_tags_7 },
    { 0x26181c9d, SUC_lexicon_tags_1 },
    { 0xef781c9f, SUC_lexicon_tags_11 },
    { 0xb5981c9f, SUC_lexicon_tags_21 },
    { 0xf3781c9e, SUC_lexicon_tags_7 },
    { 0xcc781c9b, SUC_lexicon_tags_34 },
    { 0xfff81c9b, SUC_lexicon_tags_14 },
    { 0x2f581ca4, SUC_lexicon_tags_28 },
    { 0xa2781ca4, SUC_lexicon_tags_37 },
    { 0xc7781ca6, SUC_lexicon_tags_61 },
    { 0x66f81ca7, SUC_lexicon_tags_68 },
    { 0x6c781ca4, SUC_lexicon_tags_37 },
    { 0xe6b81ca4, SUC_lexicon_tags_61 },
    { 0xe7981ca5, SUC_lexicon_tags_7 },
    { 0xb9b81cab, SUC_lexicon_tags_34 },
    { 0x10181ca2, SUC_lexicon_tags_6 },
    { 0xf5b81cad, SUC_lexicon_tags_12 },
    { 0x16781cad, SUC_lexicon_tags_30 },
    { 0xef981c9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54381cb1, SUC_lexicon_tags_57 },
    { 0x30b81cb2, SUC_lexicon_tags_14 },
    { 0xebb81cb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55181cba, SUC_lexicon_tags_26 },
    { 0x3f781cbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31781cbd, SUC_lexicon_tags_27 },
    { 0x29b81cbe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x12781cc0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbeb81cc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c181cc8, SUC_lexicon_tags_60 },
    { 0x63581cc8, SUC_lexicon_tags_61 },
    { 0x35181cca, SUC_lexicon_tags_2 },
    { 0x81381ccb, SUC_lexicon_tags_12 },
    { 0x55d81ccc, SUC_lexicon_tags_7 },
    { 0x51781ccd, SUC_lexicon_tags_0 },
    { 0xc3d81ccc, SUC_lexicon_tags_11 },
    { 0x5bf81ccf, SUC_lexicon_tags_7 },
    { 0x1c781cd0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4781cd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc181cd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79381ce1, SUC_lexicon_tags_7 },
    { 0x3a381ce1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d81ce6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61181ce8, SUC_lexicon_tags_15 },
    { 0x2d581ce9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7781ced, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d981cf0, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xb9381cf2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8181cf4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xab981cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6af81cf8, SUC_lexicon_tags_2 },
    { 0xf2181cf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9781cfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82581d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38781d02, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7181d06, SUC_lexicon_tags_15 },
    { 0xbe181d06, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x3e381d09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80f81d0b, SUC_lexicon_tags_7 },
    { 0xccd81d0b, SUC_lexicon_tags_15 },
    { 0x48d81d0d, SUC_lexicon_tags_17 },
    { 0x14d81d0c, SUC_lexicon_tags_17 },
    { 0xdf781d0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d81d15, SUC_lexicon_tags_7 },
    { 0xd5b81d16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d81d1a, SUC_lexicon_tags_7 },
    { 0x6dd81d1b, SUC_lexicon_tags_1 },
    { 0x37b81d1a, SUC_lexicon_tags_0 },
    { 0x28181d1d, SUC_lexicon_tags_37 },
    { 0x71d81d1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3781d21, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff81d26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8181d2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a781d2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66981d37, SUC_lexicon_tags_5 },
    { 0xc581d37, SUC_lexicon_tags_12 },
    { 0x4db81d39, SUC_lexicon_tags_37 },
    { 0xfcf81d3a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5e181d3c, SUC_lexicon_tags_27 },
    { 0xb8381d3c, SUC_lexicon_tags_11 },
    { 0xd9f81d3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x25581d40, SUC_lexicon_tags_1 },
    { 0x80f81d41, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x33f81d43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c781d49, SUC_lexicon_tags_0 },
    { 0xd7981d4a, SUC_lexicon_tags_19 },
    { 0xa4f81d49, SUC_lexicon_tags_11 },
    { 0x8b181d4c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9181d51, SUC_lexicon_tags_31 },
    { 0xd9b81d52, SUC_lexicon_tags_697 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f781d56, SUC_lexicon_tags_7 },
    { 0x99b81d57, SUC_lexicon_tags_29 },
    { 0xfab81d57, SUC_lexicon_tags_11 },
    { 0x5d81d59, SUC_lexicon_tags_49 },
    { 0xd5781d57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35781d5d, SUC_lexicon_tags_2 },
    { 0x52781d5d, SUC_lexicon_tags_17 },
    { 0x8a381d5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0381d63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51981d65, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x12781d68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b81d6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e581d77, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1581d7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2381d82, SUC_lexicon_tags_926 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52181d86, SUC_lexicon_tags_11 },
    { 0x28981d87, SUC_lexicon_tags_25 },
    { 0x1ed81d88, SUC_lexicon_tags_12 },
    { 0xd5d81d89, SUC_lexicon_tags_12 },
    { 0x41581d8a, SUC_lexicon_tags_37 },
    { 0x4ad81d8b, SUC_lexicon_tags_33 },
    { 0x5c781d8b, SUC_lexicon_tags_1 },
    { 0x1581d8d, SUC_lexicon_tags_1 },
    { 0xaf781d8e, SUC_lexicon_tags_12 },
    { 0xacf81d8b, SUC_lexicon_tags_11 },
    { 0x5bf81d90, SUC_lexicon_tags_30 },
    { 0xfa381d8c, SUC_lexicon_tags_47 },
    { 0x60f81d8b, SUC_lexicon_tags_2 },
    { 0x5f381d87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6581d98, SUC_lexicon_tags_1 },
    { 0xc3581d99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d81d9d, SUC_lexicon_tags_47 },
    { 0x7a181d9e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff81da1, SUC_lexicon_tags_12 },
    { 0x89781da2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x71981da4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e781da8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f581daf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb81dba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5781dc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6dd81dc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4381dc7, SUC_lexicon_tags_17 },
    { 0x72f81dc7, SUC_lexicon_tags_60 },
    { 0x8b981dc8, SUC_lexicon_tags_2 },
    { 0xaa181dc7, SUC_lexicon_tags_2 },
    { 0x25f81dcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2981dd3, SUC_lexicon_tags_105 },
    { 0x2d781dd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cd81dd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x46781dd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18581ddd, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a781de0, SUC_lexicon_tags_38 },
    { 0x3c181de1, SUC_lexicon_tags_47 },
    { 0xb4d81de2, SUC_lexicon_tags_37 },
    { 0x1af81de3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9381de7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2c181de9, SUC_lexicon_tags_31 },
    { 0x45b81dea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b81df6, SUC_lexicon_tags_25 },
    { 0x84d81df6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafb81df9, SUC_lexicon_tags_28 },
    { 0xbd181dfa, SUC_lexicon_tags_38 },
    { 0xab81dfa, SUC_lexicon_tags_56 },
    { 0x71f81dfa, SUC_lexicon_tags_43 },
    { 0x1f781dfd, SUC_lexicon_tags_53 },
    { 0xbc381dfe, SUC_lexicon_tags_15 },
    { 0xee381dfd, SUC_lexicon_tags_37 },
    { 0xa0981dfd, SUC_lexicon_tags_46 },
    { 0x50d81e00, SUC_lexicon_tags_37 },
    { 0x39581dfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc981e05, SUC_lexicon_tags_17 },
    { 0xa981e05, SUC_lexicon_tags_25 },
    { 0x26781e07, SUC_lexicon_tags_11 },
    { 0x98f81e06, SUC_lexicon_tags_17 },
    { 0x33f81e09, SUC_lexicon_tags_10 },
    { 0xd5381e09, SUC_lexicon_tags_29 },
    { 0xc4181e09, SUC_lexicon_tags_29 },
    { 0x14b81e0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d81e10, SUC_lexicon_tags_1 },
    { 0x34f81e11, SUC_lexicon_tags_47 },
    { 0xbf981e10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b81e18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x12781e1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d81e21, SUC_lexicon_tags_43 },
    { 0xbe581e22, SUC_lexicon_tags_2 },
    { 0x34d81e22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27781e25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xad81e28, SUC_lexicon_tags_2 },
    { 0xc6981e28, SUC_lexicon_tags_15 },
    { 0x91381e2a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc6f81e2c, SUC_lexicon_tags_0 },
    { 0x72d81e2c, SUC_lexicon_tags_12 },
    { 0x76d81e2d, SUC_lexicon_tags_66 },
    { 0xad181e2f, SUC_lexicon_tags_15 },
    { 0xcdf81e2f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7981e34, SUC_lexicon_tags_11 },
    { 0xb3781e35, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f81e39, SUC_lexicon_tags_697 },
    { 0x8ad81e3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf81e3e, SUC_lexicon_tags_14 },
    { 0xb8981e3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b81e43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f81e47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d181e49, SUC_lexicon_tags_47 },
    { 0xcd181e4a, SUC_lexicon_tags_25 },
    { 0xdf81e4a, SUC_lexicon_tags_17 },
    { 0x81181e4c, SUC_lexicon_tags_53 },
    { 0xb5981e4d, SUC_lexicon_tags_7 },
    { 0x75d81e49, SUC_lexicon_tags_48 },
    { 0xb3981e4a, SUC_lexicon_tags_23 },
    { 0xe3d81e4b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc981e55, SUC_lexicon_tags_25 },
    { 0xc6381e56, SUC_lexicon_tags_17 },
    { 0x73381e57, SUC_lexicon_tags_25 },
    { 0x86b81e58, SUC_lexicon_tags_57 },
    { 0xccd81e57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bb81e5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b81e62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x74381e65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaf581e67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f81e6a, SUC_lexicon_tags_47 },
    { 0x22b81e6b, SUC_lexicon_tags_0 },
    { 0xac381e6c, SUC_lexicon_tags_11 },
    { 0xc3d81e6b, SUC_lexicon_tags_10 },
    { 0x4e181e6e, SUC_lexicon_tags_11 },
    { 0x79781e6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe2b81e71, SUC_lexicon_tags_25 },
    { 0xf9981e72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b81e75, SUC_lexicon_tags_31 },
    { 0x53b81e76, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x98181e78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c581e7e, SUC_lexicon_tags_40 },
    { 0x7781e7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2f81e81, SUC_lexicon_tags_3 },
    { 0x70d81e81, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3f81e84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4981e86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0981e8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa0581e8e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4381e91, SUC_lexicon_tags_47 },
    { 0xa3981e92, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x81781e94, SUC_lexicon_tags_34 },
    { 0xcd81e94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd81e99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e181e9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d81ea4, SUC_lexicon_tags_38 },
    { 0xbd781ea5, SUC_lexicon_tags_38 },
    { 0x34d81ea6, SUC_lexicon_tags_43 },
    { 0x63581ea6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81d81ea9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87981eaf, SUC_lexicon_tags_68 },
    { 0x32981eaf, SUC_lexicon_tags_23 },
    { 0xdb81eb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8d81eb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d81eb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68181eb9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x10781ebb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x61181ebd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d781ec0, SUC_lexicon_tags_17 },
    { 0x6ed81ec1, SUC_lexicon_tags_1 },
    { 0x1c381ec2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb381ec4, SUC_lexicon_tags_9 },
    { 0x34781ec5, SUC_lexicon_tags_16 },
    { 0x94581ec4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd81ec9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d781ece, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x13381ed0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85981ed5, SUC_lexicon_tags_11 },
    { 0xf0b81ed6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f81edc, SUC_lexicon_tags_10 },
    { 0xd4981edd, SUC_lexicon_tags_21 },
    { 0xdf981ede, SUC_lexicon_tags_23 },
    { 0xd3b81edf, SUC_lexicon_tags_15 },
    { 0x7a781ee0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1181ee5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1c381ee7, SUC_lexicon_tags_11 },
    { 0x26981ee8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb181eea, SUC_lexicon_tags_17 },
    { 0x42f81eeb, SUC_lexicon_tags_1 },
    { 0x19781eec, SUC_lexicon_tags_25 },
    { 0x47581eed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e181eef, SUC_lexicon_tags_43 },
    { 0x97f81ef0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdf81ef2, SUC_lexicon_tags_5 },
    { 0xb6781ef2, SUC_lexicon_tags_15 },
    { 0xb0f81ef3, SUC_lexicon_tags_1 },
    { 0x20f81ef5, SUC_lexicon_tags_11 },
    { 0x49381ef3, SUC_lexicon_tags_17 },
    { 0x9fd81ef4, SUC_lexicon_tags_12 },
    { 0x71781ef5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3eb81efa, SUC_lexicon_tags_12 },
    { 0x55d81efa, SUC_lexicon_tags_9 },
    { 0x43981efc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa181efe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c181f02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94381f04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90181f0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c981f0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd0d81f11, SUC_lexicon_tags_0 },
    { 0x5d81f11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e581f15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d81f18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x40781f1a, SUC_lexicon_tags_108 },
    { 0xc2f81f1b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b81f1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b81f24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae781f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff81f29, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab181f2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf181f30, SUC_lexicon_tags_37 },
    { 0x86d81f31, SUC_lexicon_tags_1 },
    { 0x5bf81f32, SUC_lexicon_tags_27 },
    { 0x37f81f33, SUC_lexicon_tags_49 },
    { 0x3af81f31, SUC_lexicon_tags_26 },
    { 0xfdb81f35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d81f3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25581f41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b381f43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0181f4d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xb1381f4f, SUC_lexicon_tags_45 },
    { 0x3c781f50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f81f54, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf981f56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1b81f58, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4981f5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d81f60, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d81f63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2181f66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16781f68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d81f6c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x49981f6f, SUC_lexicon_tags_27 },
    { 0x8e581f6f, SUC_lexicon_tags_4 },
    { 0xdb981f71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76f81f73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1381f77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81581f82, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2b981f84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9781f86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb81f8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4981f93, SUC_lexicon_tags_45 },
    { 0xaed81f94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a781f96, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e181f9e, SUC_lexicon_tags_46 },
    { 0xbef81f9e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4181fa3, SUC_lexicon_tags_47 },
    { 0x4a781fa4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x36181fa6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f81fab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0581faf, SUC_lexicon_tags_37 },
    { 0x3f981fb0, SUC_lexicon_tags_2 },
    { 0x6ff81fb1, SUC_lexicon_tags_45 },
    { 0x22181fb0, SUC_lexicon_tags_37 },
    { 0xef581fb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48381fb7, SUC_lexicon_tags_87 },
    { 0x7e781fb8, SUC_lexicon_tags_14 },
    { 0x72781fb8, SUC_lexicon_tags_2 },
    { 0x6e381fb8, SUC_lexicon_tags_39 },
    { 0xca581fbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d181fbf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9e381fc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74981fca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb781fcd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x72d81fcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72181fd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x87381fdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b381fe5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x50581fe7, SUC_lexicon_tags_31 },
    { 0xc981fe8, SUC_lexicon_tags_33 },
    { 0x6781fe7, SUC_lexicon_tags_11 },
    { 0x6df81fea, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0x73581fec, SUC_lexicon_tags_15 },
    { 0xa4181fed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df81ff4, SUC_lexicon_tags_68 },
    { 0xc5b81ff5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x98381ff7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef981ffb, SUC_lexicon_tags_47 },
    { 0x27781ffc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe781ffe, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86782004, SUC_lexicon_tags_66 },
    { 0x12b82005, SUC_lexicon_tags_45 },
    { 0x78582006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e982008, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad8200d, SUC_lexicon_tags_27 },
    { 0x1df8200e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8582010, SUC_lexicon_tags_10 },
    { 0x40982011, SUC_lexicon_tags_1 },
    { 0xf2f82012, SUC_lexicon_tags_2 },
    { 0xe5f82012, SUC_lexicon_tags_11 },
    { 0xd5382014, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f82019, SUC_lexicon_tags_11 },
    { 0xbb782019, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75f8201c, SUC_lexicon_tags_8 },
    { 0x6fd8201c, SUC_lexicon_tags_17 },
    { 0x38201e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf82027, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6458202c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd8202f, SUC_lexicon_tags_37 },
    { 0x1ef82030, SUC_lexicon_tags_2 },
    { 0xf8d82031, SUC_lexicon_tags_47 },
    { 0x2982030, SUC_lexicon_tags_32 },
    { 0x44582033, SUC_lexicon_tags_45 },
    { 0xe1782033, SUC_lexicon_tags_47 },
    { 0xd8782035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37382037, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd8203b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed82040, SUC_lexicon_tags_33 },
    { 0x4bf82041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34582043, SUC_lexicon_tags_60 },
    { 0x73582044, SUC_lexicon_tags_47 },
    { 0x37f82045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffd82047, SUC_lexicon_tags_103 },
    { 0xdcd82048, SUC_lexicon_tags_1 },
    { 0xf3782049, SUC_lexicon_tags_37 },
    { 0x23182049, SUC_lexicon_tags_1 },
    { 0x28b8204b, SUC_lexicon_tags_8 },
    { 0x2ef8204b, SUC_lexicon_tags_11 },
    { 0xdb582047, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d82050, SUC_lexicon_tags_25 },
    { 0x2af82051, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18182055, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e582057, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5818205d, SUC_lexicon_tags_16 },
    { 0xe478205e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43582060, SUC_lexicon_tags_38 },
    { 0xa7b82061, SUC_lexicon_tags_26 },
    { 0x6c182062, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xac182064, SUC_lexicon_tags_53 },
    { 0x45982064, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd8206a, SUC_lexicon_tags_27 },
    { 0x2db8206a, SUC_lexicon_tags_1 },
    { 0xae38206b, SUC_lexicon_tags_16 },
    { 0x9f58206d, SUC_lexicon_tags_7 },
    { 0xf8206e, SUC_lexicon_tags_11 },
    { 0x9dd8206f, SUC_lexicon_tags_25 },
    { 0x3a18206f, SUC_lexicon_tags_7 },
    { 0x3078206e, SUC_lexicon_tags_7 },
    { 0xac382072, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f82075, SUC_lexicon_tags_12 },
    { 0xfb182076, SUC_lexicon_tags_11 },
    { 0x78582076, SUC_lexicon_tags_15 },
    { 0xb7382078, SUC_lexicon_tags_2 },
    { 0x47382079, SUC_lexicon_tags_1 },
    { 0x4d8207a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b8207d, SUC_lexicon_tags_7 },
    { 0xd4d8207e, SUC_lexicon_tags_29 },
    { 0x1af8207f, SUC_lexicon_tags_43 },
    { 0xa5d82080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24382084, SUC_lexicon_tags_34 },
    { 0x43782085, SUC_lexicon_tags_37 },
    { 0x81f82085, SUC_lexicon_tags_2 },
    { 0xd2782087, SUC_lexicon_tags_18 },
    { 0xc6582088, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb8208e, SUC_lexicon_tags_17 },
    { 0x9e58208f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f82097, SUC_lexicon_tags_11 },
    { 0x1f782098, SUC_lexicon_tags_23 },
    { 0xcc782098, SUC_lexicon_tags_63 },
    { 0xe658209a, SUC_lexicon_tags_2 },
    { 0xb7d8209b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf8209e, SUC_lexicon_tags_35 },
    { 0xfb78209e, SUC_lexicon_tags_65 },
    { 0x43820a0, SUC_lexicon_tags_30 },
    { 0xf97820a1, SUC_lexicon_tags_47 },
    { 0x1eb820a2, SUC_lexicon_tags_14 },
    { 0x88f820a2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb820a8, SUC_lexicon_tags_6 },
    { 0xc15820a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd39820ab, SUC_lexicon_tags_27 },
    { 0x3d5820ac, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73820b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30b820b2, SUC_lexicon_tags_37 },
    { 0x6bb820b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9820b7, SUC_lexicon_tags_1 },
    { 0xdf7820b8, SUC_lexicon_tags_11 },
    { 0x385820b9, SUC_lexicon_tags_7 },
    { 0x8d5820ba, SUC_lexicon_tags_11 },
    { 0xd79820bb, SUC_lexicon_tags_43 },
    { 0x7fd820bc, SUC_lexicon_tags_0 },
    { 0xdd1820bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcc7820bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f820c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75820ce, SUC_lexicon_tags_14 },
    { 0xa5b820cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b820da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x131820dc, SUC_lexicon_tags_10 },
    { 0x6bb820dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b820e0, SUC_lexicon_tags_26 },
    { 0x6f1820e1, SUC_lexicon_tags_12 },
    { 0x6cb820e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb820e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaef820e7, SUC_lexicon_tags_23 },
    { 0xf89820e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa67820ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f820f0, SUC_lexicon_tags_6 },
    { 0x23820f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51820f5, SUC_lexicon_tags_14 },
    { 0x5a9820f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf75820f8, SUC_lexicon_tags_84 },
    { 0x86b820f9, SUC_lexicon_tags_6 },
    { 0xb1b820f8, SUC_lexicon_tags_151 },
    { 0x13f820f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9820fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x29782100, SUC_lexicon_tags_27 },
    { 0xf3182101, SUC_lexicon_tags_17 },
    { 0x7eb82102, SUC_lexicon_tags_14 },
    { 0x9e182103, SUC_lexicon_tags_12 },
    { 0xe3982103, SUC_lexicon_tags_25 },
    { 0x1d82103, SUC_lexicon_tags_2 },
    { 0x55382102, SUC_lexicon_tags_84 },
    { 0xf2382107, SUC_lexicon_tags_7 },
    { 0x38182107, SUC_lexicon_tags_17 },
    { 0x1d782102, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3578210f, SUC_lexicon_tags_6 },
    { 0x98782110, SUC_lexicon_tags_10 },
    { 0x5e382110, SUC_lexicon_tags_25 },
    { 0x7b782112, SUC_lexicon_tags_23 },
    { 0x45f8210f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f982116, SUC_lexicon_tags_56 },
    { 0xe9782117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a58211a, SUC_lexicon_tags_37 },
    { 0xd8d8211a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2558211f, SUC_lexicon_tags_45 },
    { 0x3478211f, SUC_lexicon_tags_47 },
    { 0xff782121, SUC_lexicon_tags_6 },
    { 0xd4182120, SUC_lexicon_tags_47 },
    { 0xff582123, SUC_lexicon_tags_25 },
    { 0xa0182120, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x27982126, SUC_lexicon_tags_57 },
    { 0xf6f82127, SUC_lexicon_tags_26 },
    { 0xef382127, SUC_lexicon_tags_49 },
    { 0x85f82128, SUC_lexicon_tags_27 },
    { 0x8478212a, SUC_lexicon_tags_14 },
    { 0x54f8212b, SUC_lexicon_tags_8 },
    { 0x4b382129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd598212e, SUC_lexicon_tags_730 },
    { 0xf918212f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f82133, SUC_lexicon_tags_47 },
    { 0x20382134, SUC_lexicon_tags_11 },
    { 0xa5b82133, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5f82137, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5818213b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbf8213d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb82140, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f82146, SUC_lexicon_tags_7 },
    { 0x3ff82147, SUC_lexicon_tags_17 },
    { 0x27182148, SUC_lexicon_tags_37 },
    { 0x23d82149, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b38214c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98182153, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b58215c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x718215f, SUC_lexicon_tags_17 },
    { 0xa4b82160, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d82163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d182165, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x278216b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19b8216d, SUC_lexicon_tags_151 },
    { 0x35d8216e, SUC_lexicon_tags_4 },
    { 0x66f8216d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16782173, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef82177, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf82179, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x958217b, SUC_lexicon_tags_47 },
    { 0xf898217c, SUC_lexicon_tags_49 },
    { 0xb358217d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd82181, SUC_lexicon_tags_11 },
    { 0x31182181, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e982184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb582186, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7918218a, SUC_lexicon_tags_7 },
    { 0xebf8218b, SUC_lexicon_tags_2 },
    { 0x4998218c, SUC_lexicon_tags_17 },
    { 0xc038218a, SUC_lexicon_tags_23 },
    { 0xc78218c, SUC_lexicon_tags_87 },
    { 0x8d58218b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x62d82191, SUC_lexicon_tags_49 },
    { 0x43582191, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb182195, SUC_lexicon_tags_1 },
    { 0xe3782196, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x17382198, SUC_lexicon_tags_23 },
    { 0x24382198, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe238219c, SUC_lexicon_tags_14 },
    { 0x4c98219d, SUC_lexicon_tags_2 },
    { 0xb2f8219d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533821a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x603821a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb821a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9821ac, SUC_lexicon_tags_25 },
    { 0x367821ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef821b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed7821b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x343821ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85821bf, SUC_lexicon_tags_37 },
    { 0xa8d821c0, SUC_lexicon_tags_35 },
    { 0xae1821c0, SUC_lexicon_tags_14 },
    { 0xdfd821c0, SUC_lexicon_tags_0 },
    { 0xdd821c3, SUC_lexicon_tags_1 },
    { 0x135821c3, SUC_lexicon_tags_45 },
    { 0x329821c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35821cb, SUC_lexicon_tags_1 },
    { 0xd4d821cb, SUC_lexicon_tags_29 },
    { 0x647821cd, SUC_lexicon_tags_30 },
    { 0xf0b821ce, SUC_lexicon_tags_29 },
    { 0x769821cb, SUC_lexicon_tags_43 },
    { 0x475821d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb99821d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f821d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac3821d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03821db, SUC_lexicon_tags_0 },
    { 0x529821db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x14b821de, SUC_lexicon_tags_30 },
    { 0x7b5821df, SUC_lexicon_tags_11 },
    { 0xa83821e0, SUC_lexicon_tags_182 },
    { 0x989821e0, SUC_lexicon_tags_37 },
    { 0x6b9821e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd821e6, SUC_lexicon_tags_63 },
    { 0x721821e7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbc7821e9, SUC_lexicon_tags_7 },
    { 0x351821ea, SUC_lexicon_tags_45 },
    { 0x465821eb, SUC_lexicon_tags_14 },
    { 0x7f9821ec, SUC_lexicon_tags_59 },
    { 0x7ab821e9, SUC_lexicon_tags_87 },
    { 0x75d821ee, SUC_lexicon_tags_15 },
    { 0x7df821ef, SUC_lexicon_tags_23 },
    { 0x4dd821f0, SUC_lexicon_tags_47 },
    { 0xdc1821f0, SUC_lexicon_tags_23 },
    { 0x331821f0, SUC_lexicon_tags_15 },
    { 0xf6d821f3, SUC_lexicon_tags_0 },
    { 0xc6f821e9, SUC_lexicon_tags_17 },
    { 0x311821f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7821f8, SUC_lexicon_tags_4 },
    { 0xb91821f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x167821fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x985821fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed82202, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42182207, SUC_lexicon_tags_51 },
    { 0x73582207, SUC_lexicon_tags_15 },
    { 0x5e982208, SUC_lexicon_tags_7 },
    { 0xe0f8220a, SUC_lexicon_tags_12 },
    { 0x38b8220b, SUC_lexicon_tags_82 },
    { 0xdcb8220c, SUC_lexicon_tags_15 },
    { 0x1a782209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab82212, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f98221c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e782220, SUC_lexicon_tags_25 },
    { 0x91782220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e982226, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d8222a, SUC_lexicon_tags_26 },
    { 0xa538222b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64382232, SUC_lexicon_tags_47 },
    { 0xed582233, SUC_lexicon_tags_17 },
    { 0x45f82234, SUC_lexicon_tags_15 },
    { 0x32d82234, SUC_lexicon_tags_14 },
    { 0x4ed82233, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d82239, SUC_lexicon_tags_15 },
    { 0x83f82239, SUC_lexicon_tags_37 },
    { 0x4498223b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1cb8223d, SUC_lexicon_tags_2 },
    { 0xbff8223e, SUC_lexicon_tags_38 },
    { 0xf6f8223f, SUC_lexicon_tags_37 },
    { 0x34782240, SUC_lexicon_tags_37 },
    { 0xa18223d, SUC_lexicon_tags_12 },
    { 0x67382242, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1382245, SUC_lexicon_tags_47 },
    { 0xc382245, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f8224f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb82253, SUC_lexicon_tags_37 },
    { 0xd7f82254, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6d82256, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb978225a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb258225f, SUC_lexicon_tags_27 },
    { 0x23182260, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b982264, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x79982266, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x33b82268, SUC_lexicon_tags_37 },
    { 0xc8382269, SUC_lexicon_tags_11 },
    { 0x60382268, SUC_lexicon_tags_47 },
    { 0x79f8226b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe518226d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe382270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92982276, SUC_lexicon_tags_46 },
    { 0xb9382277, SUC_lexicon_tags_9 },
    { 0xbb82276, SUC_lexicon_tags_25 },
    { 0x6ab82276, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f8227e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea382280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b582285, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0382288, SUC_lexicon_tags_17 },
    { 0xb5382289, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d98228d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x50182290, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db82296, SUC_lexicon_tags_25 },
    { 0x79f82297, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xca38229a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f8229d, SUC_lexicon_tags_2 },
    { 0x6fd8229e, SUC_lexicon_tags_12 },
    { 0xd398229f, SUC_lexicon_tags_6 },
    { 0x7358229f, SUC_lexicon_tags_3 },
    { 0xbfb8229f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17822a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3b1822ab, SUC_lexicon_tags_42 },
    { 0x565822ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd822b2, SUC_lexicon_tags_11 },
    { 0x339822b3, SUC_lexicon_tags_9 },
    { 0xba1822b3, SUC_lexicon_tags_12 },
    { 0x9bd822b5, SUC_lexicon_tags_32 },
    { 0xb1822b2, SUC_lexicon_tags_4 },
    { 0x905822b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7822bc, SUC_lexicon_tags_14 },
    { 0xa75822bd, SUC_lexicon_tags_7 },
    { 0x1ef822bd, SUC_lexicon_tags_21 },
    { 0x18f822bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfdd822c1, SUC_lexicon_tags_37 },
    { 0xe67822c2, SUC_lexicon_tags_1 },
    { 0x745822c3, SUC_lexicon_tags_9 },
    { 0x319822c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5822d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485822d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45822dc, SUC_lexicon_tags_7 },
    { 0xcf822dc, SUC_lexicon_tags_3 },
    { 0x21b822dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df822e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a3822e9, SUC_lexicon_tags_14 },
    { 0x281822ea, SUC_lexicon_tags_5 },
    { 0xf0b822eb, SUC_lexicon_tags_23 },
    { 0x91b822ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d822ef, SUC_lexicon_tags_2 },
    { 0x43d822f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1822f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbef822f4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9822fb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8d7822fd, SUC_lexicon_tags_37 },
    { 0x1ff822fe, SUC_lexicon_tags_17 },
    { 0x7a1822ff, SUC_lexicon_tags_37 },
    { 0x7c182300, SUC_lexicon_tags_11 },
    { 0x7d822ff, SUC_lexicon_tags_14 },
    { 0xfe182302, SUC_lexicon_tags_17 },
    { 0xef182301, SUC_lexicon_tags_1 },
    { 0xad382304, SUC_lexicon_tags_12 },
    { 0x25d82301, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd8230b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7b8230d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f82314, SUC_lexicon_tags_37 },
    { 0x56d82314, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa182318, SUC_lexicon_tags_46 },
    { 0x38182318, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb82323, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x83582325, SUC_lexicon_tags_17 },
    { 0x6cd82326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f82329, SUC_lexicon_tags_2 },
    { 0x86582329, SUC_lexicon_tags_37 },
    { 0x378232a, SUC_lexicon_tags_66 },
    { 0xec98232c, SUC_lexicon_tags_67 },
    { 0xb1f8232d, SUC_lexicon_tags_15 },
    { 0xa878232c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x57d82331, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd5982333, SUC_lexicon_tags_23 },
    { 0x12b82334, SUC_lexicon_tags_17 },
    { 0xddd82334, SUC_lexicon_tags_15 },
    { 0x63d82336, SUC_lexicon_tags_37 },
    { 0x6df82337, SUC_lexicon_tags_33 },
    { 0xf2f82338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b8233d, SUC_lexicon_tags_2 },
    { 0xb218233e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c582341, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf582343, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f82347, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b58234f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14982351, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xab382354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9582357, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc078235a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4f8235c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3cb8235e, SUC_lexicon_tags_37 },
    { 0x7d98235f, SUC_lexicon_tags_14 },
    { 0xeb82360, SUC_lexicon_tags_31 },
    { 0xde182360, SUC_lexicon_tags_27 },
    { 0x2818235e, SUC_lexicon_tags_15 },
    { 0xdd382363, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f8236b, SUC_lexicon_tags_1 },
    { 0xb0f8236c, SUC_lexicon_tags_156 },
    { 0x3518236c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf038237b, SUC_lexicon_tags_23 },
    { 0xac38237c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d82381, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb82387, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd418238a, SUC_lexicon_tags_7 },
    { 0x69f8238a, SUC_lexicon_tags_26 },
    { 0x42d8238b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9518238f, SUC_lexicon_tags_25 },
    { 0x7b982390, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f82396, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2978239b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa3f8239d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8958239f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d823a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x193823a5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d823ab, SUC_lexicon_tags_4 },
    { 0x73823ac, SUC_lexicon_tags_37 },
    { 0x7a5823ad, SUC_lexicon_tags_4 },
    { 0x3e3823ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1823b4, SUC_lexicon_tags_19 },
    { 0x743823b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7823b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3f823ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d7823bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca3823be, SUC_lexicon_tags_27 },
    { 0xa61823be, SUC_lexicon_tags_23 },
    { 0xd89823c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3eb823c2, SUC_lexicon_tags_33 },
    { 0x565823c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd5823c5, SUC_lexicon_tags_30 },
    { 0xddb823c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb823c9, SUC_lexicon_tags_7 },
    { 0x17b823ca, SUC_lexicon_tags_15 },
    { 0x34d823cb, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x853823ce, SUC_lexicon_tags_14 },
    { 0x323823cf, SUC_lexicon_tags_37 },
    { 0xc3f823ce, SUC_lexicon_tags_84 },
    { 0x87b823d0, SUC_lexicon_tags_17 },
    { 0x8e7823d0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd823dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbef823df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213823e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x209823ea, SUC_lexicon_tags_15 },
    { 0xe4b823ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409823f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x219823f4, SUC_lexicon_tags_38 },
    { 0xc65823f5, SUC_lexicon_tags_61 },
    { 0xd5f823f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5823fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69582401, SUC_lexicon_tags_2 },
    { 0x3bf82401, SUC_lexicon_tags_2 },
    { 0x37d82403, SUC_lexicon_tags_15 },
    { 0x2bf82404, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3d82406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d82409, SUC_lexicon_tags_65 },
    { 0xca98240a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8240d, SUC_lexicon_tags_92 },
    { 0x3558240d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b82417, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f8241b, SUC_lexicon_tags_8 },
    { 0x5e98241c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db82421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1182423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cb82425, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b82428, SUC_lexicon_tags_47 },
    { 0xa1f82429, SUC_lexicon_tags_63 },
    { 0x6f18242a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b8242d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d82430, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b82434, SUC_lexicon_tags_25 },
    { 0x27582435, SUC_lexicon_tags_25 },
    { 0xe9982435, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16f82438, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d182441, SUC_lexicon_tags_47 },
    { 0xf2d82442, SUC_lexicon_tags_46 },
    { 0xd5982443, SUC_lexicon_tags_23 },
    { 0x54f82444, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3e982446, SUC_lexicon_tags_11 },
    { 0x7bb82447, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb6d82449, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b8244c, SUC_lexicon_tags_37 },
    { 0x47f8244d, SUC_lexicon_tags_38 },
    { 0x55b8244c, SUC_lexicon_tags_38 },
    { 0x4c18244c, SUC_lexicon_tags_10 },
    { 0x5558244e, SUC_lexicon_tags_25 },
    { 0xa582451, SUC_lexicon_tags_38 },
    { 0x33782452, SUC_lexicon_tags_11 },
    { 0x6d582452, SUC_lexicon_tags_7 },
    { 0x7fb82454, SUC_lexicon_tags_17 },
    { 0x95982455, SUC_lexicon_tags_16 },
    { 0xed982452, SUC_lexicon_tags_25 },
    { 0xb2582457, SUC_lexicon_tags_25 },
    { 0x25f82457, SUC_lexicon_tags_25 },
    { 0x46382455, SUC_lexicon_tags_17 },
    { 0x6b58245a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c38245d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x38582460, SUC_lexicon_tags_2 },
    { 0x9d582461, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62582466, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d8246c, SUC_lexicon_tags_6 },
    { 0xbe38246d, SUC_lexicon_tags_62 },
    { 0x1c78246e, SUC_lexicon_tags_1 },
    { 0x2158246f, SUC_lexicon_tags_45 },
    { 0xaeb82470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f82473, SUC_lexicon_tags_11 },
    { 0x5b982473, SUC_lexicon_tags_21 },
    { 0x5f982475, SUC_lexicon_tags_11 },
    { 0x88d82476, SUC_lexicon_tags_43 },
    { 0x8b782475, SUC_lexicon_tags_23 },
    { 0xd8382475, SUC_lexicon_tags_8 },
    { 0x61f82479, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87182482, SUC_lexicon_tags_19 },
    { 0x35182483, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3e582485, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5982488, SUC_lexicon_tags_30 },
    { 0xfbf82489, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb278248c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4582491, SUC_lexicon_tags_65 },
    { 0xf5582492, SUC_lexicon_tags_38 },
    { 0xdd982492, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x87382496, SUC_lexicon_tags_4 },
    { 0x1e982496, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f8249a, SUC_lexicon_tags_12 },
    { 0xdf58249b, SUC_lexicon_tags_8 },
    { 0x8078249b, SUC_lexicon_tags_11 },
    { 0x1e98249c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b824a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b824a8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x91824aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb824ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd87824af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275824b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d824b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x681824ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db824c1, SUC_lexicon_tags_25 },
    { 0x577824c2, SUC_lexicon_tags_47 },
    { 0x37824c3, SUC_lexicon_tags_38 },
    { 0xd39824c3, SUC_lexicon_tags_38 },
    { 0x99b824c5, SUC_lexicon_tags_63 },
    { 0xb3824c4, SUC_lexicon_tags_47 },
    { 0x5b9824c6, SUC_lexicon_tags_7 },
    { 0xa5824c8, SUC_lexicon_tags_37 },
    { 0xf7f824c2, SUC_lexicon_tags_47 },
    { 0xd63824ca, SUC_lexicon_tags_38 },
    { 0x91b824c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d824ce, SUC_lexicon_tags_127 },
    { 0x37824cf, SUC_lexicon_tags_6 },
    { 0x5bb824d0, SUC_lexicon_tags_14 },
    { 0x2eb824d1, SUC_lexicon_tags_21 },
    { 0x3e5824d0, SUC_lexicon_tags_11 },
    { 0xf6b824d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12f824d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51824d9, SUC_lexicon_tags_61 },
    { 0x1ab824d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5824de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91824e3, SUC_lexicon_tags_38 },
    { 0x99f824e4, SUC_lexicon_tags_12 },
    { 0x611824e5, SUC_lexicon_tags_37 },
    { 0xea5824e6, SUC_lexicon_tags_55 },
    { 0xe7f824e4, SUC_lexicon_tags_11 },
    { 0xc27824e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf824eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd91824ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261824f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9824f4, SUC_lexicon_tags_34 },
    { 0xac5824f5, SUC_lexicon_tags_37 },
    { 0xa7b824f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7824f9, SUC_lexicon_tags_1 },
    { 0xc79824fa, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xf47824fc, SUC_lexicon_tags_17 },
    { 0x5e3824fd, SUC_lexicon_tags_17 },
    { 0x523824fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b582501, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd82504, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf418250a, SUC_lexicon_tags_2 },
    { 0xbdd8250b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4f38250d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd798250f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd82512, SUC_lexicon_tags_11 },
    { 0xf9182513, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b82518, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d8251b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d8251f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b82525, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x90182527, SUC_lexicon_tags_11 },
    { 0x18b82528, SUC_lexicon_tags_15 },
    { 0x8ad82529, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5698252d, SUC_lexicon_tags_25 },
    { 0x7818252e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x31982530, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d82533, SUC_lexicon_tags_17 },
    { 0x65182533, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe5782536, SUC_lexicon_tags_12 },
    { 0x5c782537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd8253c, SUC_lexicon_tags_38 },
    { 0x3fd8253c, SUC_lexicon_tags_37 },
    { 0x89d8253d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a982541, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf82545, SUC_lexicon_tags_25 },
    { 0xd5382546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d982549, SUC_lexicon_tags_7 },
    { 0x48d8254a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa238254c, SUC_lexicon_tags_11 },
    { 0x13d8254d, SUC_lexicon_tags_11 },
    { 0xb38254c, SUC_lexicon_tags_29 },
    { 0x66b8254f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65382553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b182555, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb4982557, SUC_lexicon_tags_7 },
    { 0xa6582557, SUC_lexicon_tags_17 },
    { 0xc5782557, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb8255d, SUC_lexicon_tags_14 },
    { 0x54d8255e, SUC_lexicon_tags_37 },
    { 0xb598255d, SUC_lexicon_tags_21 },
    { 0x2ad8255d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e582565, SUC_lexicon_tags_15 },
    { 0x8c782565, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff8256c, SUC_lexicon_tags_26 },
    { 0xb258256c, SUC_lexicon_tags_32 },
    { 0xeb38256e, SUC_lexicon_tags_38 },
    { 0x4dd8256d, SUC_lexicon_tags_7 },
    { 0x21b8256e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4d82572, SUC_lexicon_tags_1 },
    { 0x27b82573, SUC_lexicon_tags_38 },
    { 0xc8d82574, SUC_lexicon_tags_25 },
    { 0x9382574, SUC_lexicon_tags_8 },
    { 0xdc382576, SUC_lexicon_tags_11 },
    { 0xf3382575, SUC_lexicon_tags_6 },
    { 0x9b382575, SUC_lexicon_tags_84 },
    { 0x9a982578, SUC_lexicon_tags_27 },
    { 0x7a182574, SUC_lexicon_tags_37 },
    { 0xc9d82574, SUC_lexicon_tags_7 },
    { 0xfbf82575, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15782581, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56182584, SUC_lexicon_tags_84 },
    { 0x45d82584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878258c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab8258f, SUC_lexicon_tags_37 },
    { 0xca98258f, SUC_lexicon_tags_1 },
    { 0xeb782590, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x90782593, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64982596, SUC_lexicon_tags_17 },
    { 0x1582597, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49382599, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad78259b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xde78259e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab825a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x343825a3, SUC_lexicon_tags_11 },
    { 0xfb5825a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad825aa, SUC_lexicon_tags_17 },
    { 0x815825aa, SUC_lexicon_tags_43 },
    { 0x8af825ac, SUC_lexicon_tags_31 },
    { 0xca7825ac, SUC_lexicon_tags_43 },
    { 0x96b825ab, SUC_lexicon_tags_1 },
    { 0xf3f825af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f825b4, SUC_lexicon_tags_12 },
    { 0x897825b5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf73825b7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed825c4, SUC_lexicon_tags_16 },
    { 0x335825c4, SUC_lexicon_tags_34 },
    { 0x6ad825c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x897825c9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1ff825cb, SUC_lexicon_tags_25 },
    { 0x605825cb, SUC_lexicon_tags_15 },
    { 0x21f825cc, SUC_lexicon_tags_7 },
    { 0x83d825cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7825d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab825d8, SUC_lexicon_tags_17 },
    { 0x2ef825d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17825dd, SUC_lexicon_tags_43 },
    { 0xcdb825de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b825e4, SUC_lexicon_tags_17 },
    { 0xe0f825e4, SUC_lexicon_tags_9 },
    { 0xc77825e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9825eb, SUC_lexicon_tags_25 },
    { 0xcb825ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31825f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdb825f3, SUC_lexicon_tags_0 },
    { 0x4af825f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d825f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91825fd, SUC_lexicon_tags_28 },
    { 0x4e5825fd, SUC_lexicon_tags_1 },
    { 0xc21825fd, SUC_lexicon_tags_25 },
    { 0x89f82600, SUC_lexicon_tags_39 },
    { 0xc6f82600, SUC_lexicon_tags_45 },
    { 0x757825ff, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a782605, SUC_lexicon_tags_9 },
    { 0x14182606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe8382608, SUC_lexicon_tags_15 },
    { 0xcd982608, SUC_lexicon_tags_0 },
    { 0x7938260a, SUC_lexicon_tags_2 },
    { 0x6958260b, SUC_lexicon_tags_34 },
    { 0x7978260c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea182610, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84782612, SUC_lexicon_tags_2 },
    { 0xcc382613, SUC_lexicon_tags_8 },
    { 0x1b782614, SUC_lexicon_tags_87 },
    { 0x61582612, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x45d82617, SUC_lexicon_tags_91 },
    { 0xebd82618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf538261a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7582622, SUC_lexicon_tags_27 },
    { 0x96182623, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90182625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f82629, SUC_lexicon_tags_25 },
    { 0x1878262a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf598262c, SUC_lexicon_tags_37 },
    { 0xc598262d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa638262f, SUC_lexicon_tags_33 },
    { 0x32f82630, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf82633, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x49982635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd8263b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7478263e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x31182641, SUC_lexicon_tags_9 },
    { 0x31982642, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b82645, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d82650, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5982653, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b82658, SUC_lexicon_tags_19 },
    { 0x2c182659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf8265d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1178265f, SUC_lexicon_tags_0 },
    { 0x83982660, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbc582662, SUC_lexicon_tags_15 },
    { 0x6e782662, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfcf82665, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f8266f, SUC_lexicon_tags_37 },
    { 0x5ad82670, SUC_lexicon_tags_57 },
    { 0x51d82671, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x75b82673, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f58267a, SUC_lexicon_tags_4 },
    { 0x15b8267a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb98267d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90982684, SUC_lexicon_tags_0 },
    { 0x65782684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7182688, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3558268b, SUC_lexicon_tags_30 },
    { 0x74f8268c, SUC_lexicon_tags_95 },
    { 0x4018268d, SUC_lexicon_tags_39 },
    { 0x58268e, SUC_lexicon_tags_1 },
    { 0x1178268c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f82692, SUC_lexicon_tags_1 },
    { 0x66982692, SUC_lexicon_tags_11 },
    { 0xb5382694, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f582699, SUC_lexicon_tags_37 },
    { 0x12f8269a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1826a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5826a8, SUC_lexicon_tags_17 },
    { 0xc29826a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3826ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f826b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ab826b2, SUC_lexicon_tags_1 },
    { 0x565826b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7826b7, SUC_lexicon_tags_19 },
    { 0xb59826b8, SUC_lexicon_tags_37 },
    { 0x709826b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee5826bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd75826bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1826c3, SUC_lexicon_tags_28 },
    { 0x6c9826c3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xedd826c6, SUC_lexicon_tags_11 },
    { 0x56d826c7, SUC_lexicon_tags_45 },
    { 0xb0d826c8, SUC_lexicon_tags_33 },
    { 0x459826c6, SUC_lexicon_tags_0 },
    { 0xd05826ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf07826cc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5a5826ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9826d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3826dc, SUC_lexicon_tags_38 },
    { 0xe95826dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7826eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87826ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd826f3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7826fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d826ff, SUC_lexicon_tags_39 },
    { 0xab826ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33782704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7198270a, SUC_lexicon_tags_47 },
    { 0xeaf8270a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa638270e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x64582711, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b82716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2382719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b8271e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5182726, SUC_lexicon_tags_77 },
    { 0xbb982727, SUC_lexicon_tags_37 },
    { 0xeaf82728, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce98272a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3382735, SUC_lexicon_tags_9 },
    { 0x22b82735, SUC_lexicon_tags_25 },
    { 0x1bb82736, SUC_lexicon_tags_12 },
    { 0xf9182738, SUC_lexicon_tags_37 },
    { 0x9eb82739, SUC_lexicon_tags_12 },
    { 0xb278273a, SUC_lexicon_tags_43 },
    { 0x5d8273a, SUC_lexicon_tags_6 },
    { 0x8818273c, SUC_lexicon_tags_19 },
    { 0x57b8273b, SUC_lexicon_tags_47 },
    { 0xe418273e, SUC_lexicon_tags_9 },
    { 0xa0d8273e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a182743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a582747, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd138274a, SUC_lexicon_tags_38 },
    { 0x3098274b, SUC_lexicon_tags_6 },
    { 0x5d18274c, SUC_lexicon_tags_1 },
    { 0x9d18274c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4158274f, SUC_lexicon_tags_12 },
    { 0x29b8274f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2db82752, SUC_lexicon_tags_63 },
    { 0x5e582752, SUC_lexicon_tags_4 },
    { 0x3e582754, SUC_lexicon_tags_11 },
    { 0xd5782755, SUC_lexicon_tags_234 },
    { 0x22b82754, SUC_lexicon_tags_36 },
    { 0x59d82755, SUC_lexicon_tags_32 },
    { 0x51d82758, SUC_lexicon_tags_33 },
    { 0x85f82759, SUC_lexicon_tags_12 },
    { 0x4e382759, SUC_lexicon_tags_32 },
    { 0x3638275b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d8275e, SUC_lexicon_tags_12 },
    { 0xda78275e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f82762, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62782766, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3618276b, SUC_lexicon_tags_25 },
    { 0xd6b8276b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55b8276e, SUC_lexicon_tags_61 },
    { 0x3f78276e, SUC_lexicon_tags_17 },
    { 0x24d8276e, SUC_lexicon_tags_37 },
    { 0xec38276f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66182773, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b982776, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa2582778, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf738277c, SUC_lexicon_tags_66 },
    { 0xc5d8277c, SUC_lexicon_tags_7 },
    { 0x2c18277e, SUC_lexicon_tags_63 },
    { 0x3ab8277e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b82785, SUC_lexicon_tags_2 },
    { 0xfa982785, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb138278c, SUC_lexicon_tags_11 },
    { 0x3ed8278d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f82790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d82793, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d582799, SUC_lexicon_tags_25 },
    { 0x1e58279a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a78279d, SUC_lexicon_tags_15 },
    { 0x1398279e, SUC_lexicon_tags_11 },
    { 0xda78279f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7dd827a1, SUC_lexicon_tags_17 },
    { 0xde1827a2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b827a8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xad5827aa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x78d827ac, SUC_lexicon_tags_2 },
    { 0x2dd827ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697827b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45827b6, SUC_lexicon_tags_8 },
    { 0x56d827b7, SUC_lexicon_tags_25 },
    { 0x6cb827b6, SUC_lexicon_tags_21 },
    { 0x635827b8, SUC_lexicon_tags_82 },
    { 0xc27827b7, SUC_lexicon_tags_47 },
    { 0x5e3827b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3827be, SUC_lexicon_tags_23 },
    { 0xda9827bf, SUC_lexicon_tags_43 },
    { 0x739827c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853827c6, SUC_lexicon_tags_35 },
    { 0x285827c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13b827c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x41b827cb, SUC_lexicon_tags_11 },
    { 0x449827cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d827cf, SUC_lexicon_tags_17 },
    { 0x99827cf, SUC_lexicon_tags_160 },
    { 0xefb827d0, SUC_lexicon_tags_19 },
    { 0x3b3827d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3827d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69827da, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673827e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57827e6, SUC_lexicon_tags_25 },
    { 0x961827e6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x2e9827e9, SUC_lexicon_tags_31 },
    { 0x34b827ea, SUC_lexicon_tags_15 },
    { 0x5a3827e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x805827ed, SUC_lexicon_tags_46 },
    { 0xd29827ed, SUC_lexicon_tags_0 },
    { 0x557827ef, SUC_lexicon_tags_11 },
    { 0x259827ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3827f5, SUC_lexicon_tags_26 },
    { 0x96f827f6, SUC_lexicon_tags_26 },
    { 0xea7827f5, SUC_lexicon_tags_15 },
    { 0x343827f8, SUC_lexicon_tags_31 },
    { 0x3c1827f9, SUC_lexicon_tags_33 },
    { 0x727827fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6182803, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b8280c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd638280f, SUC_lexicon_tags_87 },
    { 0x6a382810, SUC_lexicon_tags_30 },
    { 0x4d182810, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f782813, SUC_lexicon_tags_15 },
    { 0x2e382813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e982819, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc358281d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50782821, SUC_lexicon_tags_12 },
    { 0xe6182822, SUC_lexicon_tags_47 },
    { 0x61182823, SUC_lexicon_tags_26 },
    { 0x5582824, SUC_lexicon_tags_7 },
    { 0xc2d82821, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6582828, SUC_lexicon_tags_17 },
    { 0xeb82829, SUC_lexicon_tags_35 },
    { 0xb338282a, SUC_lexicon_tags_37 },
    { 0xf9b8282b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca58282d, SUC_lexicon_tags_33 },
    { 0x3258282e, SUC_lexicon_tags_17 },
    { 0x7d8282f, SUC_lexicon_tags_6 },
    { 0x86782830, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3582837, SUC_lexicon_tags_46 },
    { 0x8d182838, SUC_lexicon_tags_7 },
    { 0xd7182838, SUC_lexicon_tags_47 },
    { 0xcf182837, SUC_lexicon_tags_42 },
    { 0x1cd8283b, SUC_lexicon_tags_56 },
    { 0x53b8283c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14582842, SUC_lexicon_tags_6 },
    { 0xf7b82842, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x73982845, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf82847, SUC_lexicon_tags_33 },
    { 0x19b82848, SUC_lexicon_tags_15 },
    { 0xab82849, SUC_lexicon_tags_1 },
    { 0xb782848, SUC_lexicon_tags_12 },
    { 0x2418284b, SUC_lexicon_tags_29 },
    { 0x66f82849, SUC_lexicon_tags_17 },
    { 0xd4f82847, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f82851, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdef82853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b982857, SUC_lexicon_tags_15 },
    { 0x4c582858, SUC_lexicon_tags_17 },
    { 0x5b582859, SUC_lexicon_tags_1 },
    { 0xb138285a, SUC_lexicon_tags_25 },
    { 0x3d58285b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7638285d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7298285f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6382862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81782866, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb8286e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb6382870, SUC_lexicon_tags_19 },
    { 0x25982871, SUC_lexicon_tags_11 },
    { 0xd6b82872, SUC_lexicon_tags_55 },
    { 0xeb982872, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12582875, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xded82878, SUC_lexicon_tags_42 },
    { 0x4b982879, SUC_lexicon_tags_2 },
    { 0xeb182878, SUC_lexicon_tags_2 },
    { 0x8f78287b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25d8287d, SUC_lexicon_tags_63 },
    { 0xaa98287e, SUC_lexicon_tags_11 },
    { 0xd78287d, SUC_lexicon_tags_33 },
    { 0x56b8287f, SUC_lexicon_tags_11 },
    { 0xc8182881, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f782887, SUC_lexicon_tags_7 },
    { 0xcd82888, SUC_lexicon_tags_33 },
    { 0xecd82889, SUC_lexicon_tags_19 },
    { 0x82d8288a, SUC_lexicon_tags_42 },
    { 0xf78288b, SUC_lexicon_tags_16 },
    { 0x6df8288a, SUC_lexicon_tags_37 },
    { 0x5f98288b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb82890, SUC_lexicon_tags_17 },
    { 0x62182890, SUC_lexicon_tags_2 },
    { 0xec582890, SUC_lexicon_tags_37 },
    { 0xc8982893, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad782895, SUC_lexicon_tags_14 },
    { 0xe2782896, SUC_lexicon_tags_1 },
    { 0x5c382897, SUC_lexicon_tags_27 },
    { 0x3e982895, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45828aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c3828ac, SUC_lexicon_tags_11 },
    { 0xdc7828ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x957828b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67828b7, SUC_lexicon_tags_1 },
    { 0x967828b7, SUC_lexicon_tags_11 },
    { 0x813828b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x219828bb, SUC_lexicon_tags_11 },
    { 0x7c9828bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3828c1, SUC_lexicon_tags_60 },
    { 0x571828c2, SUC_lexicon_tags_37 },
    { 0x739828c2, SUC_lexicon_tags_26 },
    { 0x33f828c3, SUC_lexicon_tags_33 },
    { 0xa21828c5, SUC_lexicon_tags_11 },
    { 0xd75828c6, SUC_lexicon_tags_0 },
    { 0x37d828c7, SUC_lexicon_tags_23 },
    { 0x37b828c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d828cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7828d0, SUC_lexicon_tags_25 },
    { 0x491828d1, SUC_lexicon_tags_8 },
    { 0x48d828d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7828d5, SUC_lexicon_tags_12 },
    { 0x8eb828d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d828d9, SUC_lexicon_tags_85 },
    { 0xc29828da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84b828dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xddb828de, SUC_lexicon_tags_14 },
    { 0x83828de, SUC_lexicon_tags_27 },
    { 0x265828e0, SUC_lexicon_tags_37 },
    { 0xb01828de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea5828e3, SUC_lexicon_tags_39 },
    { 0xc1f828e4, SUC_lexicon_tags_12 },
    { 0xb77828e3, SUC_lexicon_tags_11 },
    { 0x769828e6, SUC_lexicon_tags_2 },
    { 0x47b828e7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdc7828e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93828eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x709828ed, SUC_lexicon_tags_66 },
    { 0xd828ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3cd828f0, SUC_lexicon_tags_28 },
    { 0x59b828f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855828f7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef828fb, SUC_lexicon_tags_37 },
    { 0x8ef828fc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe2d828fe, SUC_lexicon_tags_0 },
    { 0x5b5828ff, SUC_lexicon_tags_1 },
    { 0xc09828ff, SUC_lexicon_tags_17 },
    { 0xc2d82901, SUC_lexicon_tags_17 },
    { 0xfe982902, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x52382905, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x11182908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ad8290a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4278290e, SUC_lexicon_tags_8 },
    { 0xb738290f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f382912, SUC_lexicon_tags_26 },
    { 0x79782913, SUC_lexicon_tags_12 },
    { 0xff782914, SUC_lexicon_tags_1 },
    { 0xa2582914, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x78382917, SUC_lexicon_tags_25 },
    { 0x59b82918, SUC_lexicon_tags_13 },
    { 0xc4b82919, SUC_lexicon_tags_14 },
    { 0x75b82917, SUC_lexicon_tags_43 },
    { 0xbeb8291b, SUC_lexicon_tags_6 },
    { 0x4438291c, SUC_lexicon_tags_14 },
    { 0xf3b8291d, SUC_lexicon_tags_91 },
    { 0x36b82917, SUC_lexicon_tags_37 },
    { 0xb1b8291f, SUC_lexicon_tags_7 },
    { 0x92982920, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b8292a, SUC_lexicon_tags_12 },
    { 0x1cd8292a, SUC_lexicon_tags_57 },
    { 0xd378292a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b82932, SUC_lexicon_tags_37 },
    { 0xda382933, SUC_lexicon_tags_25 },
    { 0x46782934, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3fd82936, SUC_lexicon_tags_25 },
    { 0x49582937, SUC_lexicon_tags_23 },
    { 0x7d82937, SUC_lexicon_tags_21 },
    { 0x54582937, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba18293b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6eb8293d, SUC_lexicon_tags_15 },
    { 0x9558293e, SUC_lexicon_tags_68 },
    { 0x7e58293d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe182941, SUC_lexicon_tags_15 },
    { 0xf3382942, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f82946, SUC_lexicon_tags_60 },
    { 0x49582947, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e382949, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed8294e, SUC_lexicon_tags_25 },
    { 0x6538294f, SUC_lexicon_tags_14 },
    { 0x4b982950, SUC_lexicon_tags_5 },
    { 0x1f8294f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c182957, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1982959, SUC_lexicon_tags_7 },
    { 0x5db8295a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f82962, SUC_lexicon_tags_38 },
    { 0x8b782962, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4782965, SUC_lexicon_tags_0 },
    { 0xcdf82966, SUC_lexicon_tags_0 },
    { 0x75582966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1838296d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53b8296f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd058297d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97982986, SUC_lexicon_tags_15 },
    { 0x97382987, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd598298c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72382991, SUC_lexicon_tags_26 },
    { 0x34582991, SUC_lexicon_tags_7 },
    { 0xc9582991, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4982997, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee582999, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7829a1, SUC_lexicon_tags_17 },
    { 0x425829a2, SUC_lexicon_tags_37 },
    { 0xd81829a2, SUC_lexicon_tags_26 },
    { 0x25d829a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9829a8, SUC_lexicon_tags_7 },
    { 0x28b829a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe21829ab, SUC_lexicon_tags_55 },
    { 0xd7b829ac, SUC_lexicon_tags_25 },
    { 0xd71829ad, SUC_lexicon_tags_11 },
    { 0x87d829ad, SUC_lexicon_tags_19 },
    { 0x473829ab, SUC_lexicon_tags_61 },
    { 0x3f7829af, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe0b829b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf829b6, SUC_lexicon_tags_1 },
    { 0x6f9829b7, SUC_lexicon_tags_1 },
    { 0xbc9829b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b829bf, SUC_lexicon_tags_30 },
    { 0x80d829c0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x199829c3, SUC_lexicon_tags_19 },
    { 0x329829c3, SUC_lexicon_tags_11 },
    { 0x507829c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd829c8, SUC_lexicon_tags_28 },
    { 0xabd829c9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x277829cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5829ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f829d3, SUC_lexicon_tags_17 },
    { 0x965829d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f829d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815829db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x169829de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x35829e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293829e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961829e8, SUC_lexicon_tags_43 },
    { 0xe43829e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7af829eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x43829ed, SUC_lexicon_tags_7 },
    { 0xa43829ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39829f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b829f9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x27d829fb, SUC_lexicon_tags_23 },
    { 0x20b829fc, SUC_lexicon_tags_45 },
    { 0x7ef829fd, SUC_lexicon_tags_14 },
    { 0xa4b829fe, SUC_lexicon_tags_24 },
    { 0xcb7829fe, SUC_lexicon_tags_37 },
    { 0x3e982a00, SUC_lexicon_tags_9 },
    { 0xefb82a01, SUC_lexicon_tags_15 },
    { 0xa9d82a01, SUC_lexicon_tags_27 },
    { 0xb5182a03, SUC_lexicon_tags_107 },
    { 0x2e182a04, SUC_lexicon_tags_68 },
    { 0xefb829ff, SUC_lexicon_tags_14 },
    { 0x47b82a00, SUC_lexicon_tags_11 },
    { 0xfad82a05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa582a09, SUC_lexicon_tags_38 },
    { 0x9bb82a09, SUC_lexicon_tags_55 },
    { 0xec382a0a, SUC_lexicon_tags_17 },
    { 0xee182a0c, SUC_lexicon_tags_56 },
    { 0x10582a0a, SUC_lexicon_tags_1 },
    { 0xa6f82a0e, SUC_lexicon_tags_49 },
    { 0x82782a0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a782a11, SUC_lexicon_tags_43 },
    { 0x98f82a12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6982a17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb0982a19, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeb82a1b, SUC_lexicon_tags_11 },
    { 0x2582a1c, SUC_lexicon_tags_91 },
    { 0xa8382a1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42d82a1f, SUC_lexicon_tags_7 },
    { 0x1e182a20, SUC_lexicon_tags_37 },
    { 0x8f582a21, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x11f82a23, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27982a28, SUC_lexicon_tags_7 },
    { 0x74d82a28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d382a2b, SUC_lexicon_tags_13 },
    { 0x23d82a2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75582a2e, SUC_lexicon_tags_27 },
    { 0x58b82a2f, SUC_lexicon_tags_16 },
    { 0x34382a2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43382a33, SUC_lexicon_tags_38 },
    { 0x7a582a34, SUC_lexicon_tags_8 },
    { 0xcdd82a34, SUC_lexicon_tags_47 },
    { 0x7e582a36, SUC_lexicon_tags_19 },
    { 0x55b82a37, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xed982a39, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90582a3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7582a3f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdd782a41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6182a43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd82a46, SUC_lexicon_tags_30 },
    { 0x31182a47, SUC_lexicon_tags_29 },
    { 0x2e782a48, SUC_lexicon_tags_31 },
    { 0x3b382a49, SUC_lexicon_tags_26 },
    { 0x5b82a48, SUC_lexicon_tags_8 },
    { 0xd8582a48, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x74b82a4d, SUC_lexicon_tags_26 },
    { 0x182a4e, SUC_lexicon_tags_415 },
    { 0, NULL },
    { 0, NULL },
    { 0x58982a51, SUC_lexicon_tags_9 },
    { 0xe5382a52, SUC_lexicon_tags_25 },
    { 0xfa982a53, SUC_lexicon_tags_15 },
    { 0x16582a54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x86b82a56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b82a59, SUC_lexicon_tags_15 },
    { 0x48b82a5a, SUC_lexicon_tags_11 },
    { 0x23982a59, SUC_lexicon_tags_26 },
    { 0x24582a5c, SUC_lexicon_tags_11 },
    { 0x7e982a5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23982a60, SUC_lexicon_tags_9 },
    { 0x7dd82a61, SUC_lexicon_tags_26 },
    { 0x5c982a62, SUC_lexicon_tags_11 },
    { 0x40182a63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87d82a65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d182a67, SUC_lexicon_tags_23 },
    { 0x8382a67, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xe3d82a6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c582a6d, SUC_lexicon_tags_25 },
    { 0xffb82a6e, SUC_lexicon_tags_28 },
    { 0x10f82a6f, SUC_lexicon_tags_8 },
    { 0xcd582a70, SUC_lexicon_tags_27 },
    { 0xc7982a70, SUC_lexicon_tags_9 },
    { 0xb6b82a72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c782a75, SUC_lexicon_tags_17 },
    { 0xd1582a76, SUC_lexicon_tags_47 },
    { 0x61582a76, SUC_lexicon_tags_19 },
    { 0x8bf82a78, SUC_lexicon_tags_7 },
    { 0x57182a79, SUC_lexicon_tags_7 },
    { 0x81382a79, SUC_lexicon_tags_17 },
    { 0x7af82a78, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d82a7f, SUC_lexicon_tags_85 },
    { 0xf8382a80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b82a83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6bd82a85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16382a88, SUC_lexicon_tags_1215 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f82a8b, SUC_lexicon_tags_11 },
    { 0x3ad82a8c, SUC_lexicon_tags_2 },
    { 0xfcb82a8b, SUC_lexicon_tags_8 },
    { 0x982a8e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb82a91, SUC_lexicon_tags_12 },
    { 0x15382a91, SUC_lexicon_tags_15 },
    { 0x2a182a93, SUC_lexicon_tags_37 },
    { 0xf5582a91, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x28382a96, SUC_lexicon_tags_64 },
    { 0x97d82a97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf982a99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24382a9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb9b82a9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b382aa6, SUC_lexicon_tags_11 },
    { 0x4382aa6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30182aab, SUC_lexicon_tags_65 },
    { 0x45f82aab, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a982ab5, SUC_lexicon_tags_12 },
    { 0xcb782ab6, SUC_lexicon_tags_10 },
    { 0xb9b82ab7, SUC_lexicon_tags_37 },
    { 0x43182ab6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b82abc, SUC_lexicon_tags_1043 },
    { 0, NULL },
    { 0xf9982abe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73182ad0, SUC_lexicon_tags_37 },
    { 0x40b82ad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59782ad6, SUC_lexicon_tags_6 },
    { 0x67982ad7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d382ad9, SUC_lexicon_tags_17 },
    { 0x73182ada, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x44982adc, SUC_lexicon_tags_47 },
    { 0x3f982add, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7182ae1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d182ae9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e782aeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12582af1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b82af4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5982af7, SUC_lexicon_tags_17 },
    { 0xe8182af7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc182afe, SUC_lexicon_tags_45 },
    { 0x19982aff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xba382b01, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd5382b03, SUC_lexicon_tags_11 },
    { 0xe9582b04, SUC_lexicon_tags_25 },
    { 0x66382b05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2a582b07, SUC_lexicon_tags_37 },
    { 0xf2182b08, SUC_lexicon_tags_17 },
    { 0xfbd82b09, SUC_lexicon_tags_6 },
    { 0x9c982b07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d382b10, SUC_lexicon_tags_156 },
    { 0xffb82b10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49782b18, SUC_lexicon_tags_57 },
    { 0x13f82b19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd82b1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e382b20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d982b2a, SUC_lexicon_tags_129 },
    { 0x42982b2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacf82b2d, SUC_lexicon_tags_43 },
    { 0x60782b2e, SUC_lexicon_tags_17 },
    { 0xb3782b2e, SUC_lexicon_tags_42 },
    { 0xf9782b2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8782b33, SUC_lexicon_tags_38 },
    { 0x24f82b34, SUC_lexicon_tags_28 },
    { 0xbd382b35, SUC_lexicon_tags_8 },
    { 0x6a982b36, SUC_lexicon_tags_7 },
    { 0x4e582b37, SUC_lexicon_tags_178 },
    { 0x23182b38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7182b3c, SUC_lexicon_tags_37 },
    { 0x6eb82b3c, SUC_lexicon_tags_23 },
    { 0x3df82b3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77582b41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d82b45, SUC_lexicon_tags_3 },
    { 0x47f82b46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb82b49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf182b53, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5582b55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b82b58, SUC_lexicon_tags_224 },
    { 0x34582b59, SUC_lexicon_tags_124 },
    { 0x60382b59, SUC_lexicon_tags_11 },
    { 0x70f82b58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb82b5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca982b60, SUC_lexicon_tags_9 },
    { 0xee982b61, SUC_lexicon_tags_7 },
    { 0xd9782b62, SUC_lexicon_tags_1 },
    { 0xa8382b60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b382b67, SUC_lexicon_tags_19 },
    { 0x32982b67, SUC_lexicon_tags_68 },
    { 0x56982b69, SUC_lexicon_tags_37 },
    { 0x5d782b6a, SUC_lexicon_tags_106 },
    { 0xb5b82b6b, SUC_lexicon_tags_16 },
    { 0x28b82b6b, SUC_lexicon_tags_13 },
    { 0xb3582b6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x86b82b6f, SUC_lexicon_tags_37 },
    { 0x58f82b70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f182b77, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x382b79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e582b7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0382b7e, SUC_lexicon_tags_8 },
    { 0xf9582b7f, SUC_lexicon_tags_77 },
    { 0x9c582b80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8382b83, SUC_lexicon_tags_2 },
    { 0xe1b82b84, SUC_lexicon_tags_37 },
    { 0xf1582b84, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3782b89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca782b8b, SUC_lexicon_tags_37 },
    { 0x87f82b8c, SUC_lexicon_tags_118 },
    { 0x64f82b8d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7f382b8f, SUC_lexicon_tags_34 },
    { 0xbb582b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53782b92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f82b97, SUC_lexicon_tags_384 },
    { 0, NULL },
    { 0x3ff82b99, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e382b9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61782b9f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x58d82ba1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0382ba5, SUC_lexicon_tags_47 },
    { 0x4b182ba5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b82ba9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d82bac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d82bb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b82bb6, SUC_lexicon_tags_19 },
    { 0x3bd82bb7, SUC_lexicon_tags_17 },
    { 0xe7982bb8, SUC_lexicon_tags_37 },
    { 0x51f82bb8, SUC_lexicon_tags_17 },
    { 0x26f82bba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7582bbc, SUC_lexicon_tags_1 },
    { 0x5b382bbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f82bc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cf82bc2, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x29182bc4, SUC_lexicon_tags_57 },
    { 0x82782bc5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b82bd0, SUC_lexicon_tags_17 },
    { 0x8ab82bd1, SUC_lexicon_tags_25 },
    { 0x1ff82bd1, SUC_lexicon_tags_11 },
    { 0x2af82bd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7982bd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c982bdb, SUC_lexicon_tags_17 },
    { 0x3e182bdc, SUC_lexicon_tags_11 },
    { 0x37f82bdc, SUC_lexicon_tags_22 },
    { 0x17f82bde, SUC_lexicon_tags_25 },
    { 0x36182bdf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb382be4, SUC_lexicon_tags_15 },
    { 0x88d82be5, SUC_lexicon_tags_11 },
    { 0x7a782be5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7182bef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x63b82bf1, SUC_lexicon_tags_65 },
    { 0xf8182bf1, SUC_lexicon_tags_2 },
    { 0x1d982bf3, SUC_lexicon_tags_26 },
    { 0xf6b82bf4, SUC_lexicon_tags_29 },
    { 0x89b82bf2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5382bf7, SUC_lexicon_tags_27 },
    { 0x66582bf8, SUC_lexicon_tags_46 },
    { 0x69382bf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0182bff, SUC_lexicon_tags_64 },
    { 0x90382bff, SUC_lexicon_tags_37 },
    { 0xa7f82c01, SUC_lexicon_tags_105 },
    { 0xb8182c02, SUC_lexicon_tags_31 },
    { 0x49782c03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xce982c05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf8982c07, SUC_lexicon_tags_1 },
    { 0x93782c08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26b82c0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b82c0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94582c10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeb382c12, SUC_lexicon_tags_1 },
    { 0x5d182c13, SUC_lexicon_tags_19 },
    { 0x59582c14, SUC_lexicon_tags_390 },
    { 0x38182c12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1d82c17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66782c1b, SUC_lexicon_tags_37 },
    { 0xaef82c1c, SUC_lexicon_tags_27 },
    { 0xea182c1d, SUC_lexicon_tags_12 },
    { 0x91d82c1d, SUC_lexicon_tags_7 },
    { 0x8d782c1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b182c24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9782c28, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0x64782c2a, SUC_lexicon_tags_26 },
    { 0x4c782c2b, SUC_lexicon_tags_37 },
    { 0xf6182c2c, SUC_lexicon_tags_7 },
    { 0x56982c2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44382c38, SUC_lexicon_tags_26 },
    { 0x4582c39, SUC_lexicon_tags_11 },
    { 0xe5182c3a, SUC_lexicon_tags_65 },
    { 0xe6782c3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x13382c3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb382c45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f82c4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7382c55, SUC_lexicon_tags_17 },
    { 0x77d82c56, SUC_lexicon_tags_30 },
    { 0x63782c55, SUC_lexicon_tags_7 },
    { 0x4e382c55, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x12382c5b, SUC_lexicon_tags_43 },
    { 0x56b82c5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a182c60, SUC_lexicon_tags_37 },
    { 0xea982c60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf782c65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f82c68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd382c6f, SUC_lexicon_tags_2 },
    { 0x32182c6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd4782c72, SUC_lexicon_tags_7 },
    { 0xf1182c73, SUC_lexicon_tags_17 },
    { 0x9b982c73, SUC_lexicon_tags_64 },
    { 0x74382c73, SUC_lexicon_tags_37 },
    { 0x8e982c72, SUC_lexicon_tags_9 },
    { 0xa782c77, SUC_lexicon_tags_8 },
    { 0xe3b82c77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x26d82c7a, SUC_lexicon_tags_71 },
    { 0x2782c7b, SUC_lexicon_tags_60 },
    { 0xe7782c7b, SUC_lexicon_tags_3 },
    { 0xa5b82c7a, SUC_lexicon_tags_15 },
    { 0x69382c7e, SUC_lexicon_tags_43 },
    { 0xf1782c7f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa982c81, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x72182c84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52b82c86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4982c8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96582c8f, SUC_lexicon_tags_12 },
    { 0xdb382c90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85782c92, SUC_lexicon_tags_38 },
    { 0x4982c93, SUC_lexicon_tags_2 },
    { 0xb3d82c94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4582c96, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5f382c98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9bf82c9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad82ca0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad82ca5, SUC_lexicon_tags_37 },
    { 0x73b82ca6, SUC_lexicon_tags_7 },
    { 0xdf382ca6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf182ca9, SUC_lexicon_tags_30 },
    { 0xc9d82caa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c182cac, SUC_lexicon_tags_66 },
    { 0xef182cac, SUC_lexicon_tags_6 },
    { 0x82782cac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45d82cb0, SUC_lexicon_tags_2 },
    { 0xe5582cb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf2f82cb3, SUC_lexicon_tags_12 },
    { 0xcd382cb4, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf82cb9, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd82cbd, SUC_lexicon_tags_12 },
    { 0xd4d82cbe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x982cc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42182cc2, SUC_lexicon_tags_14 },
    { 0x3782cc2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x25182cc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0182ccc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16582cd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f82cd5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2582cd9, SUC_lexicon_tags_25 },
    { 0x55982cd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f82cdf, SUC_lexicon_tags_37 },
    { 0xa5982cdf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48782ce8, SUC_lexicon_tags_17 },
    { 0x59182ce9, SUC_lexicon_tags_28 },
    { 0x21782cea, SUC_lexicon_tags_8 },
    { 0xbf982cea, SUC_lexicon_tags_25 },
    { 0xb9d82cec, SUC_lexicon_tags_2 },
    { 0xd4382cec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93382cf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b782cf4, SUC_lexicon_tags_11 },
    { 0xc382cf4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3cd82cf7, SUC_lexicon_tags_7 },
    { 0xdd382cf8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32182cfa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7982cfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d82d04, SUC_lexicon_tags_6 },
    { 0x67b82d05, SUC_lexicon_tags_32 },
    { 0x7f82d06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4382d0a, SUC_lexicon_tags_9 },
    { 0x59182d0b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21382d13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88382d16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b82d1a, SUC_lexicon_tags_7 },
    { 0xaef82d1b, SUC_lexicon_tags_14 },
    { 0x38382d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3782d22, SUC_lexicon_tags_1 },
    { 0x7f782d23, SUC_lexicon_tags_37 },
    { 0x8fb82d24, SUC_lexicon_tags_15 },
    { 0x1ad82d25, SUC_lexicon_tags_39 },
    { 0x1fb82d25, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8782d29, SUC_lexicon_tags_49 },
    { 0x6d582d2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d82d2d, SUC_lexicon_tags_42 },
    { 0x79782d2e, SUC_lexicon_tags_65 },
    { 0xfa182d2f, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba982d34, SUC_lexicon_tags_12 },
    { 0xc2182d34, SUC_lexicon_tags_11 },
    { 0xfe382d36, SUC_lexicon_tags_1 },
    { 0x60982d37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b82d39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15f82d3b, SUC_lexicon_tags_17 },
    { 0xf8182d3b, SUC_lexicon_tags_33 },
    { 0xc7f82d3b, SUC_lexicon_tags_0 },
    { 0xd9b82d3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e782d44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d582d46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa382d48, SUC_lexicon_tags_1 },
    { 0xfab82d48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93582d4b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe1982d4d, SUC_lexicon_tags_37 },
    { 0xf4582d4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5182d54, SUC_lexicon_tags_1 },
    { 0xf4f82d55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66582d58, SUC_lexicon_tags_6 },
    { 0x9db82d59, SUC_lexicon_tags_15 },
    { 0x83982d5a, SUC_lexicon_tags_33 },
    { 0xf9b82d58, SUC_lexicon_tags_14 },
    { 0xd2f82d59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77582d5e, SUC_lexicon_tags_12 },
    { 0xd3382d5f, SUC_lexicon_tags_19 },
    { 0xae982d5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0982d65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x11782d67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef582d6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff82d6d, SUC_lexicon_tags_15 },
    { 0xed982d6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d82d73, SUC_lexicon_tags_27 },
    { 0x56d82d74, SUC_lexicon_tags_7 },
    { 0xb6182d74, SUC_lexicon_tags_37 },
    { 0xf6982d76, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3b782d78, SUC_lexicon_tags_63 },
    { 0x78982d78, SUC_lexicon_tags_9 },
    { 0x8f582d78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d82d7d, SUC_lexicon_tags_7 },
    { 0x4f582d7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x89582d80, SUC_lexicon_tags_127 },
    { 0x7df82d81, SUC_lexicon_tags_18 },
    { 0x6f982d82, SUC_lexicon_tags_83 },
    { 0x2a382d83, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0x58382d86, SUC_lexicon_tags_33 },
    { 0xeb582d87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe982d8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d82d8f, SUC_lexicon_tags_79 },
    { 0xed982d90, SUC_lexicon_tags_1 },
    { 0x3182d8f, SUC_lexicon_tags_15 },
    { 0xc2b82d92, SUC_lexicon_tags_45 },
    { 0xa0f82d91, SUC_lexicon_tags_198 },
    { 0xb1b82d94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25182d96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a982d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43382da5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4cd82da7, SUC_lexicon_tags_25 },
    { 0x3f382da8, SUC_lexicon_tags_10 },
    { 0x2b382da8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48382dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79982dae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a582db1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b82db6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef782db8, SUC_lexicon_tags_37 },
    { 0x70b82db9, SUC_lexicon_tags_31 },
    { 0x51f82db9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a982dbc, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f82dbf, SUC_lexicon_tags_15 },
    { 0x33b82dc0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x44182dc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47382dc5, SUC_lexicon_tags_27 },
    { 0xf182dc5, SUC_lexicon_tags_995 },
    { 0x8cb82dc6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94182dcb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b82dd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b82dd7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d82dda, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x75982ddd, SUC_lexicon_tags_14 },
    { 0x93382ddd, SUC_lexicon_tags_23 },
    { 0x2b982ddf, SUC_lexicon_tags_11 },
    { 0xb0982de0, SUC_lexicon_tags_12 },
    { 0x33d82de0, SUC_lexicon_tags_17 },
    { 0x9a782de2, SUC_lexicon_tags_65 },
    { 0xf4982de1, SUC_lexicon_tags_6 },
    { 0x56d82de4, SUC_lexicon_tags_23 },
    { 0xc0582ddf, SUC_lexicon_tags_17 },
    { 0x24b82de5, SUC_lexicon_tags_12 },
    { 0x6f382de4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdf82de9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf82def, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8182df5, SUC_lexicon_tags_25 },
    { 0xc1382df5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f82dfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60582e01, SUC_lexicon_tags_12 },
    { 0x8d82e01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b82e07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b82e0f, SUC_lexicon_tags_25 },
    { 0x6bf82e10, SUC_lexicon_tags_2 },
    { 0xe0182e10, SUC_lexicon_tags_17 },
    { 0x96d82e0f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x46b82e14, SUC_lexicon_tags_9 },
    { 0x75d82e15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1582e18, SUC_lexicon_tags_84 },
    { 0x8f182e18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a582e1c, SUC_lexicon_tags_7 },
    { 0x52982e1d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb82e20, SUC_lexicon_tags_11 },
    { 0xa0582e21, SUC_lexicon_tags_25 },
    { 0xfd82e22, SUC_lexicon_tags_12 },
    { 0x3c182e20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc6b82e25, SUC_lexicon_tags_68 },
    { 0x6a382e25, SUC_lexicon_tags_17 },
    { 0x3a382e27, SUC_lexicon_tags_7 },
    { 0xe582e26, SUC_lexicon_tags_2 },
    { 0xba182e26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4db82e2b, SUC_lexicon_tags_45 },
    { 0x3182e2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b82e30, SUC_lexicon_tags_49 },
    { 0xc7182e30, SUC_lexicon_tags_23 },
    { 0xd1182e32, SUC_lexicon_tags_21 },
    { 0x42b82e32, SUC_lexicon_tags_23 },
    { 0xc8d82e34, SUC_lexicon_tags_22 },
    { 0x66d82e35, SUC_lexicon_tags_9 },
    { 0x69582e33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f982e3c, SUC_lexicon_tags_26 },
    { 0xa2782e3d, SUC_lexicon_tags_37 },
    { 0x23782e3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b82e41, SUC_lexicon_tags_12 },
    { 0x2c182e42, SUC_lexicon_tags_47 },
    { 0xa5982e43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7382e47, SUC_lexicon_tags_14 },
    { 0x86f82e47, SUC_lexicon_tags_45 },
    { 0x7e782e48, SUC_lexicon_tags_11 },
    { 0x13b82e4a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d82e4d, SUC_lexicon_tags_47 },
    { 0x1ed82e4d, SUC_lexicon_tags_17 },
    { 0xfb982e4f, SUC_lexicon_tags_12 },
    { 0x1a582e50, SUC_lexicon_tags_17 },
    { 0xdff82e4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35b82e53, SUC_lexicon_tags_25 },
    { 0x6db82e53, SUC_lexicon_tags_280 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23982e58, SUC_lexicon_tags_315 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b82e5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1582e63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f782e6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6982e6d, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x99782e6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1cb82e71, SUC_lexicon_tags_79 },
    { 0xa0d82e72, SUC_lexicon_tags_25 },
    { 0xa9382e73, SUC_lexicon_tags_13 },
    { 0x29d82e73, SUC_lexicon_tags_2 },
    { 0x5f582e74, SUC_lexicon_tags_7 },
    { 0xc6382e74, SUC_lexicon_tags_32 },
    { 0x50582e73, SUC_lexicon_tags_31 },
    { 0xb5d82e78, SUC_lexicon_tags_19 },
    { 0x67f82e78, SUC_lexicon_tags_9 },
    { 0x72982e79, SUC_lexicon_tags_11 },
    { 0xcd982e7b, SUC_lexicon_tags_63 },
    { 0x5eb82e7b, SUC_lexicon_tags_37 },
    { 0x7182e73, SUC_lexicon_tags_0 },
    { 0xed382e76, SUC_lexicon_tags_9 },
    { 0x71b82e71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21782e87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42382e8b, SUC_lexicon_tags_11 },
    { 0xb3782e8c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd782e95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d82e9b, SUC_lexicon_tags_7 },
    { 0xbcd82e9c, SUC_lexicon_tags_114 },
    { 0x93582e9b, SUC_lexicon_tags_26 },
    { 0xc7182e9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9db82ea0, SUC_lexicon_tags_9 },
    { 0xf2b82ea1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36582ea4, SUC_lexicon_tags_8 },
    { 0x3df82ea4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8dd82ea7, SUC_lexicon_tags_6 },
    { 0xe4982ea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a782ead, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32982eb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58582eb3, SUC_lexicon_tags_28 },
    { 0xbd982eb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1382ebf, SUC_lexicon_tags_37 },
    { 0xb9f82ec0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3582ec3, SUC_lexicon_tags_8 },
    { 0x40382ec4, SUC_lexicon_tags_37 },
    { 0xf5d82ec5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61582ec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca982ecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51182ecf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d982ed3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x14582ed5, SUC_lexicon_tags_45 },
    { 0xb8782ed5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x55982ed9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5782edb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0182ee0, SUC_lexicon_tags_33 },
    { 0xab582ee1, SUC_lexicon_tags_63 },
    { 0x74382ee2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf82ee6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa1782ee8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a382eea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a382eed, SUC_lexicon_tags_25 },
    { 0x17982eee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad82ef2, SUC_lexicon_tags_11 },
    { 0xa8582ef2, SUC_lexicon_tags_8 },
    { 0x3f982ef4, SUC_lexicon_tags_14 },
    { 0x96782ef4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8c982ef7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4182efa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85982efc, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1582f01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x65782f04, SUC_lexicon_tags_87 },
    { 0x8b982f05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1182f07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7182f0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3582f12, SUC_lexicon_tags_37 },
    { 0xf7982f12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf82f17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc4f82f19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef982f1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3b382f1d, SUC_lexicon_tags_60 },
    { 0x65b82f1d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3982f21, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb2182f23, SUC_lexicon_tags_23 },
    { 0x2fd82f24, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f82f2f, SUC_lexicon_tags_27 },
    { 0xaaf82f30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b382f32, SUC_lexicon_tags_15 },
    { 0xf2f82f33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad82f3a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x91d82f3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b82f40, SUC_lexicon_tags_151 },
    { 0xdf182f41, SUC_lexicon_tags_17 },
    { 0xd5782f42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c382f45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5d82f47, SUC_lexicon_tags_12 },
    { 0x8c182f47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97782f54, SUC_lexicon_tags_17 },
    { 0xe5f82f55, SUC_lexicon_tags_41 },
    { 0xb2182f56, SUC_lexicon_tags_755 },
    { 0xf2982f55, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xea782f5a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96582f5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1b82f60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7582f62, SUC_lexicon_tags_25 },
    { 0x40382f63, SUC_lexicon_tags_17 },
    { 0x93582f64, SUC_lexicon_tags_11 },
    { 0x67582f64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa9382f67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d82f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea382f7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7982f7d, SUC_lexicon_tags_33 },
    { 0x93782f7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69782f80, SUC_lexicon_tags_11 },
    { 0x5a382f80, SUC_lexicon_tags_37 },
    { 0x5b782f81, SUC_lexicon_tags_17 },
    { 0x78182f83, SUC_lexicon_tags_7 },
    { 0x70382f84, SUC_lexicon_tags_105 },
    { 0xc6b82f82, SUC_lexicon_tags_30 },
    { 0x50382f86, SUC_lexicon_tags_32 },
    { 0x7eb82f87, SUC_lexicon_tags_19 },
    { 0xb9982f88, SUC_lexicon_tags_2 },
    { 0x6dd82f82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5182f8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa982f93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c382f96, SUC_lexicon_tags_10 },
    { 0xc6182f97, SUC_lexicon_tags_14 },
    { 0xf8b82f97, SUC_lexicon_tags_25 },
    { 0x5382f99, SUC_lexicon_tags_12 },
    { 0x3db82f9a, SUC_lexicon_tags_37 },
    { 0x49d82f99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64982f9e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9182fa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2eb82fa6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4582faa, SUC_lexicon_tags_43 },
    { 0xf4582faa, SUC_lexicon_tags_15 },
    { 0x33982fab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d82fb4, SUC_lexicon_tags_23 },
    { 0xb7f82fb4, SUC_lexicon_tags_11 },
    { 0x9a382fb6, SUC_lexicon_tags_58 },
    { 0x5a782fb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b82fb9, SUC_lexicon_tags_140 },
    { 0x65382fb9, SUC_lexicon_tags_25 },
    { 0x35d82fbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f982fbf, SUC_lexicon_tags_37 },
    { 0xd2582fc0, SUC_lexicon_tags_79 },
    { 0x2ef82fc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe782fc6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f82fc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x34982fca, SUC_lexicon_tags_8 },
    { 0x6e382fca, SUC_lexicon_tags_14 },
    { 0xe7182fcc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x73582fcf, SUC_lexicon_tags_37 },
    { 0xb0782fd0, SUC_lexicon_tags_23 },
    { 0x20382fd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b82fd4, SUC_lexicon_tags_11 },
    { 0xeb82fd4, SUC_lexicon_tags_85 },
    { 0x1bb82fd5, SUC_lexicon_tags_2 },
    { 0xe5782fd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d182fda, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f782fdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b82fe3, SUC_lexicon_tags_19 },
    { 0xe9f82fe4, SUC_lexicon_tags_44 },
    { 0xe7b82fe3, SUC_lexicon_tags_2 },
    { 0x35382fe6, SUC_lexicon_tags_60 },
    { 0x62f82fe4, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x18d82fe9, SUC_lexicon_tags_7 },
    { 0x38982fe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11382fec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27382fef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f82ff8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x13582ffa, SUC_lexicon_tags_30 },
    { 0xc9382ffa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54182ffd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d83002, SUC_lexicon_tags_17 },
    { 0x41783003, SUC_lexicon_tags_106 },
    { 0x46183004, SUC_lexicon_tags_88 },
    { 0x19983002, SUC_lexicon_tags_45 },
    { 0x49f83003, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c18300b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf8300e, SUC_lexicon_tags_9 },
    { 0x7f78300f, SUC_lexicon_tags_47 },
    { 0xc8b83010, SUC_lexicon_tags_17 },
    { 0x40f83011, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f383013, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb83016, SUC_lexicon_tags_7 },
    { 0xfe583017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6458301a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b8301d, SUC_lexicon_tags_47 },
    { 0x1438301e, SUC_lexicon_tags_12 },
    { 0x82d8301f, SUC_lexicon_tags_7 },
    { 0x5bf83020, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6af83022, SUC_lexicon_tags_31 },
    { 0xd2383023, SUC_lexicon_tags_13 },
    { 0x82d83024, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x78583027, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xeb383029, SUC_lexicon_tags_27 },
    { 0x3a98302a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f58302c, SUC_lexicon_tags_4 },
    { 0x318302d, SUC_lexicon_tags_25 },
    { 0x2998302d, SUC_lexicon_tags_2 },
    { 0x9c98302f, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x37583031, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d83034, SUC_lexicon_tags_34 },
    { 0xb1983035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd383037, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d8303a, SUC_lexicon_tags_47 },
    { 0x4278303a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7238303e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52983041, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x88983043, SUC_lexicon_tags_37 },
    { 0x7ab83044, SUC_lexicon_tags_27 },
    { 0x51583045, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f783047, SUC_lexicon_tags_31 },
    { 0xc2783048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5358304a, SUC_lexicon_tags_47 },
    { 0x5d98304a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d83055, SUC_lexicon_tags_1 },
    { 0xb7f83055, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3b83058, SUC_lexicon_tags_2 },
    { 0x61983059, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b8305c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81583060, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5383068, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d8306c, SUC_lexicon_tags_39 },
    { 0x5e58306c, SUC_lexicon_tags_60 },
    { 0xf8306e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb83071, SUC_lexicon_tags_106 },
    { 0xbf983071, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7b83074, SUC_lexicon_tags_43 },
    { 0x93d83075, SUC_lexicon_tags_903 },
    { 0x40b83074, SUC_lexicon_tags_3 },
    { 0x1cd83077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb878307c, SUC_lexicon_tags_11 },
    { 0xafb8307c, SUC_lexicon_tags_1 },
    { 0xb558307e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaed83080, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc983086, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x2fb83088, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc298308c, SUC_lexicon_tags_32 },
    { 0x878308c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf83090, SUC_lexicon_tags_7 },
    { 0xeb83091, SUC_lexicon_tags_21 },
    { 0x7e383090, SUC_lexicon_tags_28 },
    { 0x96783093, SUC_lexicon_tags_17 },
    { 0x88f83090, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd783096, SUC_lexicon_tags_15 },
    { 0x9b383096, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af830a0, SUC_lexicon_tags_56 },
    { 0x10b830a0, SUC_lexicon_tags_45 },
    { 0x58f830a1, SUC_lexicon_tags_37 },
    { 0x4eb830a3, SUC_lexicon_tags_6 },
    { 0xf9830a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x265830a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x90d830a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x479830ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x393830ad, SUC_lexicon_tags_16 },
    { 0x9c1830ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x505830b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1830b6, SUC_lexicon_tags_12 },
    { 0x2b3830b7, SUC_lexicon_tags_15 },
    { 0xb15830b8, SUC_lexicon_tags_11 },
    { 0x9d3830b8, SUC_lexicon_tags_2 },
    { 0x37d830ba, SUC_lexicon_tags_56 },
    { 0x7f7830bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x665830bd, SUC_lexicon_tags_11 },
    { 0xe0d830be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f830c1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f830c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f830cd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5830d0, SUC_lexicon_tags_1 },
    { 0xe7b830d1, SUC_lexicon_tags_2 },
    { 0xa25830d0, SUC_lexicon_tags_25 },
    { 0xf71830d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcfb830d5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc39830d7, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xe99830d9, SUC_lexicon_tags_25 },
    { 0x709830da, SUC_lexicon_tags_77 },
    { 0xbf9830db, SUC_lexicon_tags_32 },
    { 0x5ab830da, SUC_lexicon_tags_2 },
    { 0x8c5830dd, SUC_lexicon_tags_2 },
    { 0x6cf830de, SUC_lexicon_tags_15 },
    { 0x1830d9, SUC_lexicon_tags_37 },
    { 0x747830de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc03830e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xecb830e4, SUC_lexicon_tags_37 },
    { 0xf1830e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bb830e7, SUC_lexicon_tags_6 },
    { 0x6fd830e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7830ec, SUC_lexicon_tags_23 },
    { 0xb37830ed, SUC_lexicon_tags_25 },
    { 0xbb1830ed, SUC_lexicon_tags_11 },
    { 0xf65830ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333830f4, SUC_lexicon_tags_17 },
    { 0x44f830f5, SUC_lexicon_tags_17 },
    { 0x7d7830f6, SUC_lexicon_tags_7 },
    { 0xcf5830f7, SUC_lexicon_tags_10 },
    { 0x583830f7, SUC_lexicon_tags_47 },
    { 0x791830f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3fd830fb, SUC_lexicon_tags_80 },
    { 0x78b830fb, SUC_lexicon_tags_2 },
    { 0x6e7830fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd83102, SUC_lexicon_tags_63 },
    { 0x40583103, SUC_lexicon_tags_17 },
    { 0x60583104, SUC_lexicon_tags_25 },
    { 0x9ad83105, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb183109, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7b8310b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa38310e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6783111, SUC_lexicon_tags_65 },
    { 0xa4f83112, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6b383114, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b383118, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf598311e, SUC_lexicon_tags_46 },
    { 0xde38311e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b583121, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd8312a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb83132, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14583137, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd8313a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b83141, SUC_lexicon_tags_17 },
    { 0xd2383142, SUC_lexicon_tags_15 },
    { 0x77383143, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee783146, SUC_lexicon_tags_37 },
    { 0x20183147, SUC_lexicon_tags_26 },
    { 0x80183147, SUC_lexicon_tags_12 },
    { 0xae183146, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd8314e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef83157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb2183159, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f8315c, SUC_lexicon_tags_15 },
    { 0x3798315d, SUC_lexicon_tags_2 },
    { 0xef8315e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf83168, SUC_lexicon_tags_87 },
    { 0x37783169, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xff98316c, SUC_lexicon_tags_138 },
    { 0x8a38316d, SUC_lexicon_tags_60 },
    { 0xa778316e, SUC_lexicon_tags_56 },
    { 0x5eb8316f, SUC_lexicon_tags_10 },
    { 0x4183170, SUC_lexicon_tags_1 },
    { 0x16d83170, SUC_lexicon_tags_48 },
    { 0xdf383170, SUC_lexicon_tags_17 },
    { 0x92f83173, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8583175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb98317b, SUC_lexicon_tags_38 },
    { 0xa218317b, SUC_lexicon_tags_23 },
    { 0xfd38317d, SUC_lexicon_tags_29 },
    { 0x8838317c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x61383181, SUC_lexicon_tags_17 },
    { 0x20183182, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x25d83184, SUC_lexicon_tags_49 },
    { 0x31183185, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x26983187, SUC_lexicon_tags_21 },
    { 0x1eb83187, SUC_lexicon_tags_37 },
    { 0x9fb83189, SUC_lexicon_tags_11 },
    { 0xe2783188, SUC_lexicon_tags_15 },
    { 0x56b83189, SUC_lexicon_tags_23 },
    { 0xcab8318c, SUC_lexicon_tags_63 },
    { 0xd078318a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa1f8318f, SUC_lexicon_tags_0 },
    { 0x67583190, SUC_lexicon_tags_11 },
    { 0x49383190, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f831a0, SUC_lexicon_tags_16 },
    { 0x241831a1, SUC_lexicon_tags_74 },
    { 0x9f7831a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd831a8, SUC_lexicon_tags_15 },
    { 0x915831a9, SUC_lexicon_tags_13 },
    { 0x743831aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd03831ac, SUC_lexicon_tags_38 },
    { 0x5db831ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3831b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82d831b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb21831b4, SUC_lexicon_tags_9 },
    { 0x529831b4, SUC_lexicon_tags_2 },
    { 0x583831b6, SUC_lexicon_tags_1 },
    { 0x415831b6, SUC_lexicon_tags_8 },
    { 0xd7831b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3831c1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859831c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x293831c8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5831cb, SUC_lexicon_tags_60 },
    { 0x65f831cc, SUC_lexicon_tags_46 },
    { 0xb0b831cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x96f831cf, SUC_lexicon_tags_47 },
    { 0x45d831cf, SUC_lexicon_tags_29 },
    { 0x6b7831d1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341831d6, SUC_lexicon_tags_17 },
    { 0x919831d6, SUC_lexicon_tags_17 },
    { 0x5cf831d7, SUC_lexicon_tags_12 },
    { 0x119831d7, SUC_lexicon_tags_37 },
    { 0x579831d6, SUC_lexicon_tags_17 },
    { 0x337831db, SUC_lexicon_tags_25 },
    { 0x295831db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7831df, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f831e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67831e5, SUC_lexicon_tags_1 },
    { 0xd3f831e5, SUC_lexicon_tags_1101 },
    { 0xb65831e6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7831ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf831f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f831f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693831fd, SUC_lexicon_tags_17 },
    { 0xb27831fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x78183201, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x18f83203, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39583205, SUC_lexicon_tags_23 },
    { 0x22583206, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x49d83208, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc898320e, SUC_lexicon_tags_37 },
    { 0x5778320f, SUC_lexicon_tags_24 },
    { 0xa7583210, SUC_lexicon_tags_0 },
    { 0x57983211, SUC_lexicon_tags_7 },
    { 0x72f83212, SUC_lexicon_tags_37 },
    { 0xf383213, SUC_lexicon_tags_11 },
    { 0xd6b83214, SUC_lexicon_tags_12 },
    { 0xfbd83215, SUC_lexicon_tags_12 },
    { 0x67d83216, SUC_lexicon_tags_27 },
    { 0x8583217, SUC_lexicon_tags_17 },
    { 0x95183211, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed8321d, SUC_lexicon_tags_81 },
    { 0x2a38321d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91983223, SUC_lexicon_tags_9 },
    { 0x3b583224, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15583226, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8358322b, SUC_lexicon_tags_29 },
    { 0x60d8322c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xac38322e, SUC_lexicon_tags_46 },
    { 0x8b38322f, SUC_lexicon_tags_21 },
    { 0x81b83230, SUC_lexicon_tags_39 },
    { 0x17583231, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d83234, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b83237, SUC_lexicon_tags_12 },
    { 0x54383238, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf8323a, SUC_lexicon_tags_2 },
    { 0xe378323a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7d18323d, SUC_lexicon_tags_17 },
    { 0x99d8323e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xa7d83240, SUC_lexicon_tags_2 },
    { 0x72983241, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6983244, SUC_lexicon_tags_47 },
    { 0xf7f83244, SUC_lexicon_tags_26 },
    { 0x5c183244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb983249, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6218324b, SUC_lexicon_tags_11 },
    { 0x6af8324c, SUC_lexicon_tags_7 },
    { 0xa9f8324d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf783250, SUC_lexicon_tags_17 },
    { 0x68583250, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8983253, SUC_lexicon_tags_6 },
    { 0x64983254, SUC_lexicon_tags_0 },
    { 0x9ef83253, SUC_lexicon_tags_38 },
    { 0x2df83254, SUC_lexicon_tags_47 },
    { 0x49383257, SUC_lexicon_tags_37 },
    { 0x1af83258, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d8325b, SUC_lexicon_tags_19 },
    { 0xf198325c, SUC_lexicon_tags_25 },
    { 0xbd58325d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1f8325f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f83262, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55383266, SUC_lexicon_tags_87 },
    { 0x91b83267, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe438326c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d8326f, SUC_lexicon_tags_14 },
    { 0xb0f83270, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4c383272, SUC_lexicon_tags_12 },
    { 0x2583272, SUC_lexicon_tags_17 },
    { 0x49583273, SUC_lexicon_tags_66 },
    { 0xd5d83272, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe9783277, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2783279, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae783280, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa183283, SUC_lexicon_tags_9 },
    { 0x3b83283, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xee983287, SUC_lexicon_tags_27 },
    { 0xf2f83287, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc838328c, SUC_lexicon_tags_25 },
    { 0x5238328c, SUC_lexicon_tags_33 },
    { 0x4e98328e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6f83290, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb83293, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd983296, SUC_lexicon_tags_25 },
    { 0x30f83297, SUC_lexicon_tags_37 },
    { 0xc9383297, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9658329a, SUC_lexicon_tags_7 },
    { 0x3198329a, SUC_lexicon_tags_37 },
    { 0x7f38329b, SUC_lexicon_tags_25 },
    { 0xfa98329d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3618329f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47f832a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x409832a3, SUC_lexicon_tags_2 },
    { 0xe5832a4, SUC_lexicon_tags_15 },
    { 0x5fb832a5, SUC_lexicon_tags_12 },
    { 0x9a9832a4, SUC_lexicon_tags_23 },
    { 0xf85832a7, SUC_lexicon_tags_9 },
    { 0x441832a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61832ae, SUC_lexicon_tags_45 },
    { 0x74b832af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27832b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff832b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1832bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d832c2, SUC_lexicon_tags_2 },
    { 0x4cb832c3, SUC_lexicon_tags_23 },
    { 0x247832c4, SUC_lexicon_tags_0 },
    { 0x69f832c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f832cc, SUC_lexicon_tags_7 },
    { 0xee9832cd, SUC_lexicon_tags_87 },
    { 0x73d832cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f9832d0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77832d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f832d8, SUC_lexicon_tags_38 },
    { 0xd9832d9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933832de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99832e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25832e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7832ea, SUC_lexicon_tags_2 },
    { 0x709832ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69d832ed, SUC_lexicon_tags_8 },
    { 0xa9f832ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdd9832f0, SUC_lexicon_tags_74 },
    { 0xa6f832f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59832f6, SUC_lexicon_tags_9 },
    { 0x6c3832f6, SUC_lexicon_tags_29 },
    { 0x85f832f7, SUC_lexicon_tags_47 },
    { 0x435832f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d832fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49783301, SUC_lexicon_tags_12 },
    { 0x76583301, SUC_lexicon_tags_33 },
    { 0x23b83303, SUC_lexicon_tags_15 },
    { 0xc1d83301, SUC_lexicon_tags_1 },
    { 0xf783305, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5118330a, SUC_lexicon_tags_25 },
    { 0x1a98330b, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59983311, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa0583313, SUC_lexicon_tags_0 },
    { 0xbeb83314, SUC_lexicon_tags_31 },
    { 0x79783314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67383318, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdd8331a, SUC_lexicon_tags_49 },
    { 0x3978331b, SUC_lexicon_tags_47 },
    { 0x21d8331c, SUC_lexicon_tags_0 },
    { 0xdd18331d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f83326, SUC_lexicon_tags_42 },
    { 0x47b83327, SUC_lexicon_tags_47 },
    { 0x1ed83328, SUC_lexicon_tags_15 },
    { 0x44183329, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9418332b, SUC_lexicon_tags_1 },
    { 0xae98332c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8332f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbed83331, SUC_lexicon_tags_33 },
    { 0x7f983332, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29983334, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23983336, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4058333a, SUC_lexicon_tags_27 },
    { 0xf418333b, SUC_lexicon_tags_8 },
    { 0x3d98333a, SUC_lexicon_tags_29 },
    { 0xd618333d, SUC_lexicon_tags_48 },
    { 0x62f8333e, SUC_lexicon_tags_3 },
    { 0x5d78333e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad83343, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc783345, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xad83347, SUC_lexicon_tags_17 },
    { 0xe2583348, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd778334a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8e58334c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4438334e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11983350, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xde583352, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10783356, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x63383358, SUC_lexicon_tags_28 },
    { 0xb6183359, SUC_lexicon_tags_15 },
    { 0xc2b8335a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8018335d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8538335f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db83365, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d78336f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75383372, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb83375, SUC_lexicon_tags_17 },
    { 0x51183375, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8098337c, SUC_lexicon_tags_27 },
    { 0x58f8337d, SUC_lexicon_tags_12 },
    { 0xd2d8337e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d83380, SUC_lexicon_tags_17 },
    { 0x1e583380, SUC_lexicon_tags_45 },
    { 0xd1b83382, SUC_lexicon_tags_1 },
    { 0x36b83383, SUC_lexicon_tags_13 },
    { 0x5f183381, SUC_lexicon_tags_37 },
    { 0xc3383385, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1eb83387, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f8338a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed98338f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62183397, SUC_lexicon_tags_37 },
    { 0x51983398, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5d58339a, SUC_lexicon_tags_6 },
    { 0x99d8339b, SUC_lexicon_tags_11 },
    { 0x5dd8339c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f8339f, SUC_lexicon_tags_35 },
    { 0x935833a0, SUC_lexicon_tags_36 },
    { 0xdb5833a0, SUC_lexicon_tags_28 },
    { 0x9278339f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85833a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9833aa, SUC_lexicon_tags_47 },
    { 0x18d833aa, SUC_lexicon_tags_28 },
    { 0x1f833aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1833af, SUC_lexicon_tags_14 },
    { 0x665833af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x813833b3, SUC_lexicon_tags_11 },
    { 0xa93833b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263833b8, SUC_lexicon_tags_6 },
    { 0x733833b9, SUC_lexicon_tags_37 },
    { 0x5bf833ba, SUC_lexicon_tags_7 },
    { 0xb87833bb, SUC_lexicon_tags_25 },
    { 0xaa3833bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d9833be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc2b833c0, SUC_lexicon_tags_16 },
    { 0x785833c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7833c4, SUC_lexicon_tags_47 },
    { 0x7b833c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d833ce, SUC_lexicon_tags_38 },
    { 0x17d833cf, SUC_lexicon_tags_31 },
    { 0xc6f833d0, SUC_lexicon_tags_23 },
    { 0x15b833d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd833d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73833db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x365833dd, SUC_lexicon_tags_2 },
    { 0x669833dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14b833e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9833e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8d7833e6, SUC_lexicon_tags_17 },
    { 0xdcd833e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f833ea, SUC_lexicon_tags_25 },
    { 0x8e5833eb, SUC_lexicon_tags_13 },
    { 0xa99833ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79833f0, SUC_lexicon_tags_65 },
    { 0x343833f1, SUC_lexicon_tags_12 },
    { 0xabb833f2, SUC_lexicon_tags_15 },
    { 0x1b3833f3, SUC_lexicon_tags_31 },
    { 0xad7833f3, SUC_lexicon_tags_17 },
    { 0x9b1833f5, SUC_lexicon_tags_6 },
    { 0xfc7833f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab833ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd83402, SUC_lexicon_tags_117 },
    { 0x93f83403, SUC_lexicon_tags_8 },
    { 0x67b83404, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f83408, SUC_lexicon_tags_30 },
    { 0xdd583409, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9258340b, SUC_lexicon_tags_37 },
    { 0xfe98340c, SUC_lexicon_tags_11 },
    { 0x54b8340b, SUC_lexicon_tags_37 },
    { 0x1d98340c, SUC_lexicon_tags_0 },
    { 0x7398340f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x49783412, SUC_lexicon_tags_2 },
    { 0xca583413, SUC_lexicon_tags_2 },
    { 0xab383412, SUC_lexicon_tags_11 },
    { 0x9cd83412, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10983419, SUC_lexicon_tags_11 },
    { 0x6db8341a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d8341e, SUC_lexicon_tags_87 },
    { 0x22d8341e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b183423, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x66183426, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d983428, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2338342a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3478342c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xefd8342e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b83431, SUC_lexicon_tags_25 },
    { 0xbe383431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6183435, SUC_lexicon_tags_14 },
    { 0xd4783436, SUC_lexicon_tags_2 },
    { 0xd5d83437, SUC_lexicon_tags_1 },
    { 0xa8b83438, SUC_lexicon_tags_7 },
    { 0xb5f83439, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc678343b, SUC_lexicon_tags_0 },
    { 0x4158343c, SUC_lexicon_tags_47 },
    { 0x4a58343b, SUC_lexicon_tags_49 },
    { 0x8038343b, SUC_lexicon_tags_2 },
    { 0x4438343f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7583441, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1383448, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8344c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4598344f, SUC_lexicon_tags_1 },
    { 0x74f83450, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4383454, SUC_lexicon_tags_37 },
    { 0xc0383455, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33783459, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce98345d, SUC_lexicon_tags_14 },
    { 0x2df8345d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0783463, SUC_lexicon_tags_15 },
    { 0xd7183463, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed8346a, SUC_lexicon_tags_323 },
    { 0xcd38346a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c983472, SUC_lexicon_tags_37 },
    { 0x73d83473, SUC_lexicon_tags_16 },
    { 0x76383474, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d58347b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab83482, SUC_lexicon_tags_26 },
    { 0x19f83483, SUC_lexicon_tags_11 },
    { 0xd5783484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e583486, SUC_lexicon_tags_32 },
    { 0x56383487, SUC_lexicon_tags_7 },
    { 0xd5983488, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed8348d, SUC_lexicon_tags_37 },
    { 0xf9d8348d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd983490, SUC_lexicon_tags_77 },
    { 0x7a183491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf83494, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb583498, SUC_lexicon_tags_30 },
    { 0x65f83499, SUC_lexicon_tags_29 },
    { 0xd618349a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a38349c, SUC_lexicon_tags_25 },
    { 0xee58349d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f834a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63834ac, SUC_lexicon_tags_10 },
    { 0x43d834ad, SUC_lexicon_tags_11 },
    { 0x827834ae, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x293834b1, SUC_lexicon_tags_12 },
    { 0x9f834b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1834b5, SUC_lexicon_tags_48 },
    { 0xbe5834b5, SUC_lexicon_tags_2 },
    { 0xe9f834b7, SUC_lexicon_tags_11 },
    { 0x2cb834b8, SUC_lexicon_tags_12 },
    { 0xd31834b8, SUC_lexicon_tags_37 },
    { 0x5f9834b8, SUC_lexicon_tags_21 },
    { 0xfad834b9, SUC_lexicon_tags_183 },
    { 0xc15834bc, SUC_lexicon_tags_87 },
    { 0xe7834ba, SUC_lexicon_tags_4 },
    { 0x709834be, SUC_lexicon_tags_39 },
    { 0x6ad834bb, SUC_lexicon_tags_17 },
    { 0xef5834c0, SUC_lexicon_tags_8 },
    { 0xdf5834be, SUC_lexicon_tags_57 },
    { 0x843834c2, SUC_lexicon_tags_23 },
    { 0x1d9834c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1eb834c5, SUC_lexicon_tags_30 },
    { 0x61834c6, SUC_lexicon_tags_1 },
    { 0x139834c7, SUC_lexicon_tags_17 },
    { 0xf8f834c8, SUC_lexicon_tags_17 },
    { 0xab9834c9, SUC_lexicon_tags_63 },
    { 0xad834ca, SUC_lexicon_tags_28 },
    { 0x635834cb, SUC_lexicon_tags_141 },
    { 0x15b834cc, SUC_lexicon_tags_37 },
    { 0xc15834ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1834d3, SUC_lexicon_tags_151 },
    { 0xe37834d4, SUC_lexicon_tags_1 },
    { 0xb57834d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfef834d7, SUC_lexicon_tags_27 },
    { 0xba5834d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991834de, SUC_lexicon_tags_7 },
    { 0xe7d834df, SUC_lexicon_tags_25 },
    { 0x33d834e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3834e9, SUC_lexicon_tags_82 },
    { 0x2f3834e9, SUC_lexicon_tags_23 },
    { 0xccf834e9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d834f4, SUC_lexicon_tags_25 },
    { 0x3bd834f5, SUC_lexicon_tags_65 },
    { 0x33d834f6, SUC_lexicon_tags_47 },
    { 0x319834f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22f834f9, SUC_lexicon_tags_40 },
    { 0x431834fa, SUC_lexicon_tags_2 },
    { 0x7db834fb, SUC_lexicon_tags_14 },
    { 0xab834fc, SUC_lexicon_tags_37 },
    { 0x3eb834fc, SUC_lexicon_tags_63 },
    { 0x51f834fe, SUC_lexicon_tags_49 },
    { 0x8f3834fa, SUC_lexicon_tags_11 },
    { 0x7b9834ff, SUC_lexicon_tags_26 },
    { 0xec783501, SUC_lexicon_tags_14 },
    { 0x39583502, SUC_lexicon_tags_25 },
    { 0x4c783502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b83515, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e783518, SUC_lexicon_tags_29 },
    { 0x18f83518, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6918351c, SUC_lexicon_tags_4 },
    { 0x4818351c, SUC_lexicon_tags_45 },
    { 0x5778351d, SUC_lexicon_tags_47 },
    { 0x3498351f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcd83521, SUC_lexicon_tags_37 },
    { 0x4f783522, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb783525, SUC_lexicon_tags_29 },
    { 0x3b183525, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4983529, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f8352c, SUC_lexicon_tags_9 },
    { 0x9d78352c, SUC_lexicon_tags_29 },
    { 0x9738352e, SUC_lexicon_tags_1 },
    { 0x68b8352f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b83533, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa8583535, SUC_lexicon_tags_43 },
    { 0xfa383535, SUC_lexicon_tags_12 },
    { 0xc7383537, SUC_lexicon_tags_43 },
    { 0xbb383538, SUC_lexicon_tags_2 },
    { 0x79983536, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af8353d, SUC_lexicon_tags_60 },
    { 0xdb58353d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf83543, SUC_lexicon_tags_15 },
    { 0x1cb83544, SUC_lexicon_tags_16 },
    { 0xa2983545, SUC_lexicon_tags_25 },
    { 0x75f83546, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b8354f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f83555, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd783558, SUC_lexicon_tags_0 },
    { 0x3a983559, SUC_lexicon_tags_8 },
    { 0xf4b8355a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd183564, SUC_lexicon_tags_37 },
    { 0xfed83565, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6583569, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcb8356b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9cb8356d, SUC_lexicon_tags_12 },
    { 0xa238356e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61d83570, SUC_lexicon_tags_47 },
    { 0xaf783571, SUC_lexicon_tags_37 },
    { 0x9f183570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f83578, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b38357a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5978357c, SUC_lexicon_tags_27 },
    { 0x1238357c, SUC_lexicon_tags_6 },
    { 0xe158357c, SUC_lexicon_tags_9 },
    { 0xb4f8357f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x69583582, SUC_lexicon_tags_63 },
    { 0x6d983583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d183586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f8358b, SUC_lexicon_tags_16 },
    { 0xa458358c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f83594, SUC_lexicon_tags_37 },
    { 0x4a783595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa798359d, SUC_lexicon_tags_40 },
    { 0x7278359d, SUC_lexicon_tags_13 },
    { 0xc1b8359f, SUC_lexicon_tags_1 },
    { 0x279835a0, SUC_lexicon_tags_1 },
    { 0x117835a1, SUC_lexicon_tags_25 },
    { 0xa7b8359f, SUC_lexicon_tags_15 },
    { 0x63835a3, SUC_lexicon_tags_15 },
    { 0x1b835a4, SUC_lexicon_tags_47 },
    { 0xfd1835a3, SUC_lexicon_tags_15 },
    { 0xfcd835a4, SUC_lexicon_tags_14 },
    { 0x445835a7, SUC_lexicon_tags_20 },
    { 0x257835a8, SUC_lexicon_tags_45 },
    { 0xe95835a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609835af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa25835b1, SUC_lexicon_tags_47 },
    { 0xf8f835b1, SUC_lexicon_tags_2 },
    { 0xfeb835b2, SUC_lexicon_tags_12 },
    { 0x64d835b4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x815835b6, SUC_lexicon_tags_43 },
    { 0xe7835b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae5835b9, SUC_lexicon_tags_25 },
    { 0x37d835ba, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87835bf, SUC_lexicon_tags_336 },
    { 0x1af835c0, SUC_lexicon_tags_15 },
    { 0x13b835c1, SUC_lexicon_tags_44 },
    { 0x7a9835c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x813835c4, SUC_lexicon_tags_0 },
    { 0x8cf835c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831835cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f835cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97835d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d835de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed835e1, SUC_lexicon_tags_11 },
    { 0xfdd835e2, SUC_lexicon_tags_38 },
    { 0x257835e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89835e6, SUC_lexicon_tags_92 },
    { 0x38f835e7, SUC_lexicon_tags_92 },
    { 0x547835e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07835eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa45835ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf835f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91835f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0983602, SUC_lexicon_tags_14 },
    { 0x3a983603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bf83605, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19d83607, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f8360a, SUC_lexicon_tags_4 },
    { 0x1d78360a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f83612, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d83615, SUC_lexicon_tags_23 },
    { 0xa4183616, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d83619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec983620, SUC_lexicon_tags_11 },
    { 0xf383621, SUC_lexicon_tags_47 },
    { 0xb2983622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78383624, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0183627, SUC_lexicon_tags_31 },
    { 0x33f83628, SUC_lexicon_tags_36 },
    { 0xe1f83628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8b8362b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f18362f, SUC_lexicon_tags_2 },
    { 0x50983630, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d383633, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2983636, SUC_lexicon_tags_11 },
    { 0xf8383636, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5218363a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6958363c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd6f8363e, SUC_lexicon_tags_34 },
    { 0x7118363f, SUC_lexicon_tags_16 },
    { 0x48383640, SUC_lexicon_tags_29 },
    { 0xacb83641, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b83646, SUC_lexicon_tags_55 },
    { 0x9a983646, SUC_lexicon_tags_172 },
    { 0xd8b83647, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91783654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcab83656, SUC_lexicon_tags_19 },
    { 0x45783657, SUC_lexicon_tags_11 },
    { 0xba183658, SUC_lexicon_tags_60 },
    { 0x7bb83659, SUC_lexicon_tags_17 },
    { 0x91b8365a, SUC_lexicon_tags_17 },
    { 0x5a38365b, SUC_lexicon_tags_1 },
    { 0x35d8365c, SUC_lexicon_tags_68 },
    { 0xa5f8365b, SUC_lexicon_tags_65 },
    { 0x6e98365d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e583660, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca383664, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d83668, SUC_lexicon_tags_37 },
    { 0xb8783669, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd998366e, SUC_lexicon_tags_37 },
    { 0x8c18366f, SUC_lexicon_tags_103 },
    { 0x71d8366f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8367a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0383680, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b8368b, SUC_lexicon_tags_66 },
    { 0x86f8368c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f83690, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e383694, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x80183697, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5cb83699, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x4298369b, SUC_lexicon_tags_33 },
    { 0xd2b8369c, SUC_lexicon_tags_600 },
    { 0xf838369d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x433836a0, SUC_lexicon_tags_12 },
    { 0x451836a1, SUC_lexicon_tags_37 },
    { 0x409836a2, SUC_lexicon_tags_23 },
    { 0x979836a3, SUC_lexicon_tags_1 },
    { 0x1a7836a3, SUC_lexicon_tags_10 },
    { 0xeb5836a1, SUC_lexicon_tags_17 },
    { 0xa61836a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x571836a9, SUC_lexicon_tags_25 },
    { 0xb4f836aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9eb836ac, SUC_lexicon_tags_84 },
    { 0x9c5836ad, SUC_lexicon_tags_8 },
    { 0x97836ac, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x221836b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47836b3, SUC_lexicon_tags_25 },
    { 0x641836b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x49b836b6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85836ba, SUC_lexicon_tags_25 },
    { 0xaa1836ba, SUC_lexicon_tags_85 },
    { 0xffb836bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x469836be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x827836c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x97b836c2, SUC_lexicon_tags_8 },
    { 0x7f7836c3, SUC_lexicon_tags_47 },
    { 0x9a1836c4, SUC_lexicon_tags_29 },
    { 0x459836c5, SUC_lexicon_tags_29 },
    { 0xef3836c5, SUC_lexicon_tags_2 },
    { 0x69d836c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd5836c9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf25836cb, SUC_lexicon_tags_11 },
    { 0xc6b836cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9836cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31b836d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf836d5, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3836d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x173836dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d836e2, SUC_lexicon_tags_7 },
    { 0x855836e3, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x423836e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad836ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb836f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383836fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13836fe, SUC_lexicon_tags_4 },
    { 0xc0f836fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x59583702, SUC_lexicon_tags_47 },
    { 0x2b83703, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf83706, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ab83708, SUC_lexicon_tags_37 },
    { 0x41b83709, SUC_lexicon_tags_47 },
    { 0xef98370a, SUC_lexicon_tags_1 },
    { 0xb298370a, SUC_lexicon_tags_7 },
    { 0xc018370c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b8370f, SUC_lexicon_tags_60 },
    { 0x1a38370f, SUC_lexicon_tags_37 },
    { 0x50f83711, SUC_lexicon_tags_8 },
    { 0xe4583712, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c583714, SUC_lexicon_tags_60 },
    { 0x3bb83715, SUC_lexicon_tags_43 },
    { 0x58b83716, SUC_lexicon_tags_8 },
    { 0xd4b83717, SUC_lexicon_tags_71 },
    { 0xfb83718, SUC_lexicon_tags_14 },
    { 0x73b83719, SUC_lexicon_tags_1 },
    { 0xa9d83716, SUC_lexicon_tags_9 },
    { 0xf4383714, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d8371f, SUC_lexicon_tags_71 },
    { 0x50183720, SUC_lexicon_tags_39 },
    { 0x33383721, SUC_lexicon_tags_12 },
    { 0x4f183722, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2983724, SUC_lexicon_tags_33 },
    { 0x5f383725, SUC_lexicon_tags_23 },
    { 0xf6583726, SUC_lexicon_tags_25 },
    { 0x9e983726, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead8372b, SUC_lexicon_tags_9 },
    { 0x3438372c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf8372f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf583733, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b383736, SUC_lexicon_tags_19 },
    { 0x31f83737, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9d83739, SUC_lexicon_tags_17 },
    { 0xa6b8373a, SUC_lexicon_tags_6 },
    { 0x94583739, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd98373f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c983744, SUC_lexicon_tags_25 },
    { 0x7bb83745, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32983749, SUC_lexicon_tags_43 },
    { 0xf0b8374a, SUC_lexicon_tags_29 },
    { 0x2918374a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad83750, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41183755, SUC_lexicon_tags_17 },
    { 0xee983756, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3758375d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c58375f, SUC_lexicon_tags_0 },
    { 0x6f183760, SUC_lexicon_tags_27 },
    { 0xeaf83761, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14d83763, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18583767, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8438376b, SUC_lexicon_tags_27 },
    { 0x4298376c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe78376e, SUC_lexicon_tags_25 },
    { 0x41f8376e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c783774, SUC_lexicon_tags_61 },
    { 0x50d83775, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6783779, SUC_lexicon_tags_1 },
    { 0xfe18377a, SUC_lexicon_tags_1 },
    { 0x11d8377a, SUC_lexicon_tags_12 },
    { 0xdcf8377a, SUC_lexicon_tags_151 },
    { 0x6578377d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3458377f, SUC_lexicon_tags_29 },
    { 0xfc383780, SUC_lexicon_tags_11 },
    { 0xea783781, SUC_lexicon_tags_32 },
    { 0xdb983781, SUC_lexicon_tags_37 },
    { 0xe4f83781, SUC_lexicon_tags_45 },
    { 0x11583784, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d8378a, SUC_lexicon_tags_17 },
    { 0xb578378b, SUC_lexicon_tags_0 },
    { 0x1278378c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f38378e, SUC_lexicon_tags_0 },
    { 0x47b8378f, SUC_lexicon_tags_30 },
    { 0x84f8378f, SUC_lexicon_tags_1 },
    { 0xb8783790, SUC_lexicon_tags_27 },
    { 0xe6d83791, SUC_lexicon_tags_662 },
    { 0x68d8378e, SUC_lexicon_tags_5 },
    { 0x2e783791, SUC_lexicon_tags_0 },
    { 0x8bd83795, SUC_lexicon_tags_32 },
    { 0xd6783796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b837a1, SUC_lexicon_tags_70 },
    { 0x993837a2, SUC_lexicon_tags_3 },
    { 0xfa9837a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14b837a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x659837a7, SUC_lexicon_tags_47 },
    { 0x7f3837a7, SUC_lexicon_tags_2 },
    { 0x71d837a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbb837ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1837b1, SUC_lexicon_tags_43 },
    { 0x2cb837b2, SUC_lexicon_tags_26 },
    { 0x4d837b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5837b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49837be, SUC_lexicon_tags_37 },
    { 0x8d3837be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d837c2, SUC_lexicon_tags_34 },
    { 0x6d1837c3, SUC_lexicon_tags_336 },
    { 0x167837c4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b837c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd837cd, SUC_lexicon_tags_82 },
    { 0x4cf837cd, SUC_lexicon_tags_7 },
    { 0x22b837ce, SUC_lexicon_tags_17 },
    { 0x24f837cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1837d7, SUC_lexicon_tags_23 },
    { 0x94f837d7, SUC_lexicon_tags_65 },
    { 0x83d837d7, SUC_lexicon_tags_15 },
    { 0x7eb837d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21837de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b837e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f837e8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed837eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7837f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x859837f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x183837f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d837fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x855837fd, SUC_lexicon_tags_26 },
    { 0x135837fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5183801, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xca983804, SUC_lexicon_tags_37 },
    { 0x23583804, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb8380e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7983811, SUC_lexicon_tags_7 },
    { 0x33b83812, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcdd83814, SUC_lexicon_tags_0 },
    { 0x64183815, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff38381a, SUC_lexicon_tags_15 },
    { 0xdb58381b, SUC_lexicon_tags_44 },
    { 0x6d98381c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc038381e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75983823, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f83828, SUC_lexicon_tags_15 },
    { 0xe5d83828, SUC_lexicon_tags_8 },
    { 0x68f83829, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a58382c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc078382e, SUC_lexicon_tags_43 },
    { 0x2c78382f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4d383831, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced83835, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x52f83837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b983839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe478383c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xddd8383e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0583840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79783842, SUC_lexicon_tags_12 },
    { 0x9d783842, SUC_lexicon_tags_47 },
    { 0xec183844, SUC_lexicon_tags_7 },
    { 0x2df83845, SUC_lexicon_tags_14 },
    { 0xc5b83846, SUC_lexicon_tags_11 },
    { 0x36383847, SUC_lexicon_tags_15 },
    { 0xa3983848, SUC_lexicon_tags_1 },
    { 0x49d83846, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6118384b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27183853, SUC_lexicon_tags_15 },
    { 0xf9d83853, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9583856, SUC_lexicon_tags_37 },
    { 0x5bf83857, SUC_lexicon_tags_29 },
    { 0x5b583858, SUC_lexicon_tags_3 },
    { 0xe2983859, SUC_lexicon_tags_38 },
    { 0x47f8385a, SUC_lexicon_tags_10 },
    { 0xe638385b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d83862, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99383866, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d8386b, SUC_lexicon_tags_25 },
    { 0xc4d8386c, SUC_lexicon_tags_60 },
    { 0x4338386d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f83872, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e783876, SUC_lexicon_tags_25 },
    { 0x76f83877, SUC_lexicon_tags_57 },
    { 0x13f83878, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x2fd8387a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4578387f, SUC_lexicon_tags_11 },
    { 0x33b8387f, SUC_lexicon_tags_49 },
    { 0x99983880, SUC_lexicon_tags_16 },
    { 0x84b83880, SUC_lexicon_tags_16 },
    { 0xa183883, SUC_lexicon_tags_1 },
    { 0x3383884, SUC_lexicon_tags_7 },
    { 0x15583880, SUC_lexicon_tags_2 },
    { 0xad83886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df8388b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2818388d, SUC_lexicon_tags_7 },
    { 0x71b8388d, SUC_lexicon_tags_23 },
    { 0x3038388d, SUC_lexicon_tags_27 },
    { 0x6d83890, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe183894, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf4f83896, SUC_lexicon_tags_33 },
    { 0x7d183897, SUC_lexicon_tags_25 },
    { 0x45b83897, SUC_lexicon_tags_21 },
    { 0xeeb83898, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb8389d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13838a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1838a7, SUC_lexicon_tags_45 },
    { 0xf7f838a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x57f838aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x76d838ac, SUC_lexicon_tags_12 },
    { 0x745838ac, SUC_lexicon_tags_48 },
    { 0x2b9838ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x281838b0, SUC_lexicon_tags_28 },
    { 0xa41838b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x489838b3, SUC_lexicon_tags_37 },
    { 0xd73838b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85f838b6, SUC_lexicon_tags_60 },
    { 0xe83838b6, SUC_lexicon_tags_36 },
    { 0xc87838b8, SUC_lexicon_tags_2 },
    { 0xa37838b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d838bc, SUC_lexicon_tags_46 },
    { 0xdcb838bc, SUC_lexicon_tags_194 },
    { 0x277838be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf7838c0, SUC_lexicon_tags_11 },
    { 0xe3f838c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97f838c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x313838c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13838cc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1838cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb838d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdaf838d4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d838da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f838e3, SUC_lexicon_tags_29 },
    { 0xb2b838e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01838ed, SUC_lexicon_tags_26 },
    { 0xcd9838ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x403838f1, SUC_lexicon_tags_37 },
    { 0xa87838f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9838f6, SUC_lexicon_tags_28 },
    { 0x169838f6, SUC_lexicon_tags_11 },
    { 0xbff838f8, SUC_lexicon_tags_1 },
    { 0x7e9838f9, SUC_lexicon_tags_60 },
    { 0x977838fa, SUC_lexicon_tags_8 },
    { 0x287838fb, SUC_lexicon_tags_1 },
    { 0x993838fc, SUC_lexicon_tags_3 },
    { 0xc57838fa, SUC_lexicon_tags_1 },
    { 0x3a1838fe, SUC_lexicon_tags_63 },
    { 0x9f3838ff, SUC_lexicon_tags_2 },
    { 0x15d838ff, SUC_lexicon_tags_0 },
    { 0xd73838ff, SUC_lexicon_tags_1 },
    { 0xab5838f8, SUC_lexicon_tags_60 },
    { 0x231838fb, SUC_lexicon_tags_1 },
    { 0xa0983904, SUC_lexicon_tags_32 },
    { 0xcbb83904, SUC_lexicon_tags_60 },
    { 0xaf83906, SUC_lexicon_tags_33 },
    { 0x37383904, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32183911, SUC_lexicon_tags_49 },
    { 0xce783911, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9a583914, SUC_lexicon_tags_38 },
    { 0xd0b83915, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a783918, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe858391a, SUC_lexicon_tags_7 },
    { 0x3a18391b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b8391d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a78391f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1983922, SUC_lexicon_tags_43 },
    { 0x8f783922, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a183929, SUC_lexicon_tags_25 },
    { 0x3c58392a, SUC_lexicon_tags_11 },
    { 0xe9f8392a, SUC_lexicon_tags_14 },
    { 0x7a78392c, SUC_lexicon_tags_11 },
    { 0x1b18392d, SUC_lexicon_tags_1 },
    { 0x66d83929, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b83931, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed83934, SUC_lexicon_tags_11 },
    { 0x11d83935, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96783937, SUC_lexicon_tags_25 },
    { 0x2d383938, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd58393e, SUC_lexicon_tags_21 },
    { 0xe638393f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8f83941, SUC_lexicon_tags_2 },
    { 0x85383941, SUC_lexicon_tags_11 },
    { 0x35b83943, SUC_lexicon_tags_8 },
    { 0x2df83941, SUC_lexicon_tags_33 },
    { 0xbdd83943, SUC_lexicon_tags_25 },
    { 0x75b83941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7583948, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78394c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b83950, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8af83952, SUC_lexicon_tags_6 },
    { 0x4a383953, SUC_lexicon_tags_26 },
    { 0xc5383954, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f383959, SUC_lexicon_tags_46 },
    { 0x97d8395a, SUC_lexicon_tags_12 },
    { 0x6eb8395b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9eb8395d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc218395f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17583961, SUC_lexicon_tags_33 },
    { 0xfa983962, SUC_lexicon_tags_79 },
    { 0xe0f83961, SUC_lexicon_tags_0 },
    { 0xb4d83964, SUC_lexicon_tags_25 },
    { 0xa5d83961, SUC_lexicon_tags_15 },
    { 0xc9b83963, SUC_lexicon_tags_2 },
    { 0xf8583967, SUC_lexicon_tags_25 },
    { 0xcdd83968, SUC_lexicon_tags_12 },
    { 0xd5383969, SUC_lexicon_tags_1 },
    { 0xb4d8396a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86d8396c, SUC_lexicon_tags_15 },
    { 0x42d8396d, SUC_lexicon_tags_25 },
    { 0x5c78396d, SUC_lexicon_tags_51 },
    { 0x7f78396f, SUC_lexicon_tags_2 },
    { 0x1438396f, SUC_lexicon_tags_45 },
    { 0xef18396f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6383977, SUC_lexicon_tags_7 },
    { 0x3d83978, SUC_lexicon_tags_8 },
    { 0xabd83979, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe558397c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b38397f, SUC_lexicon_tags_1 },
    { 0xa98397f, SUC_lexicon_tags_37 },
    { 0xb4183981, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfb83983, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xc3f83985, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2183989, SUC_lexicon_tags_11 },
    { 0xd0983989, SUC_lexicon_tags_17 },
    { 0xb2583989, SUC_lexicon_tags_0 },
    { 0x9cf83989, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd878398e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda583991, SUC_lexicon_tags_15 },
    { 0xd2583992, SUC_lexicon_tags_23 },
    { 0x6f983992, SUC_lexicon_tags_33 },
    { 0xcc783994, SUC_lexicon_tags_1 },
    { 0x46383995, SUC_lexicon_tags_12 },
    { 0xeb983996, SUC_lexicon_tags_14 },
    { 0xfc183997, SUC_lexicon_tags_61 },
    { 0x37b83994, SUC_lexicon_tags_7 },
    { 0x62583993, SUC_lexicon_tags_1 },
    { 0xfe38399a, SUC_lexicon_tags_25 },
    { 0x63d8399b, SUC_lexicon_tags_19 },
    { 0x75b8399c, SUC_lexicon_tags_11 },
    { 0x4758399d, SUC_lexicon_tags_17 },
    { 0x4cd8399e, SUC_lexicon_tags_8 },
    { 0xae98399d, SUC_lexicon_tags_43 },
    { 0x78f8399e, SUC_lexicon_tags_77 },
    { 0x618399d, SUC_lexicon_tags_12 },
    { 0x85583993, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f839a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23839a8, SUC_lexicon_tags_45 },
    { 0x869839a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7fd839ab, SUC_lexicon_tags_19 },
    { 0x427839ac, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5839af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f839b2, SUC_lexicon_tags_45 },
    { 0xa53839b2, SUC_lexicon_tags_15 },
    { 0xc87839b4, SUC_lexicon_tags_9 },
    { 0xd7f839b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x335839b7, SUC_lexicon_tags_31 },
    { 0x80d839b8, SUC_lexicon_tags_79 },
    { 0xc8b839b9, SUC_lexicon_tags_14 },
    { 0x303839ba, SUC_lexicon_tags_19 },
    { 0x55f839bb, SUC_lexicon_tags_37 },
    { 0x3f839bc, SUC_lexicon_tags_117 },
    { 0x499839bc, SUC_lexicon_tags_25 },
    { 0x6c7839bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f839c4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf13839c6, SUC_lexicon_tags_17 },
    { 0x2a3839c7, SUC_lexicon_tags_15 },
    { 0xa39839c8, SUC_lexicon_tags_17 },
    { 0xb7839c6, SUC_lexicon_tags_12 },
    { 0x219839c6, SUC_lexicon_tags_3 },
    { 0x70b839c9, SUC_lexicon_tags_104 },
    { 0x7c1839c7, SUC_lexicon_tags_15 },
    { 0x8d839cd, SUC_lexicon_tags_17 },
    { 0x4c5839ce, SUC_lexicon_tags_37 },
    { 0x921839ca, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x347839d2, SUC_lexicon_tags_25 },
    { 0xcef839d3, SUC_lexicon_tags_37 },
    { 0x58f839d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b839d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaf7839db, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41839de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfd7839e0, SUC_lexicon_tags_49 },
    { 0x27b839e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe11839e3, SUC_lexicon_tags_45 },
    { 0x595839e4, SUC_lexicon_tags_15 },
    { 0x565839e3, SUC_lexicon_tags_46 },
    { 0xaeb839e6, SUC_lexicon_tags_79 },
    { 0x39d839e5, SUC_lexicon_tags_33 },
    { 0xf4b839e8, SUC_lexicon_tags_60 },
    { 0x343839e9, SUC_lexicon_tags_45 },
    { 0xf83839ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x755839ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f7839ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199839f2, SUC_lexicon_tags_17 },
    { 0x3f1839f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x565839f5, SUC_lexicon_tags_12 },
    { 0x6a9839f6, SUC_lexicon_tags_37 },
    { 0xaa9839f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1839fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8983a00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14f83a0c, SUC_lexicon_tags_25 },
    { 0xecf83a0c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e583a16, SUC_lexicon_tags_46 },
    { 0x8383a16, SUC_lexicon_tags_25 },
    { 0x42d83a18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3983a1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf83a20, SUC_lexicon_tags_7 },
    { 0x64583a21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x80583a23, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12183a2a, SUC_lexicon_tags_39 },
    { 0xce783a2a, SUC_lexicon_tags_44 },
    { 0x4ed83a2c, SUC_lexicon_tags_17 },
    { 0xffb83a2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1183a30, SUC_lexicon_tags_49 },
    { 0x40383a30, SUC_lexicon_tags_7 },
    { 0xadb83a32, SUC_lexicon_tags_14 },
    { 0xd4583a33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d83a36, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb83a3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ed83a3e, SUC_lexicon_tags_11 },
    { 0xc583a3f, SUC_lexicon_tags_17 },
    { 0xc1183a40, SUC_lexicon_tags_29 },
    { 0xa5b83a41, SUC_lexicon_tags_8 },
    { 0xfb83a3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89783a46, SUC_lexicon_tags_14 },
    { 0x57383a47, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0383a51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d83a57, SUC_lexicon_tags_25 },
    { 0x38583a57, SUC_lexicon_tags_17 },
    { 0x97583a59, SUC_lexicon_tags_12 },
    { 0x6ff83a5a, SUC_lexicon_tags_37 },
    { 0xc1983a5b, SUC_lexicon_tags_12 },
    { 0xdc183a5b, SUC_lexicon_tags_2 },
    { 0xf7783a5d, SUC_lexicon_tags_28 },
    { 0xe4f83a5e, SUC_lexicon_tags_37 },
    { 0x15583a5f, SUC_lexicon_tags_12 },
    { 0xcd183a5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29583a64, SUC_lexicon_tags_37 },
    { 0x9ed83a64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f383a69, SUC_lexicon_tags_7 },
    { 0x6c383a6a, SUC_lexicon_tags_10 },
    { 0x37383a6a, SUC_lexicon_tags_13 },
    { 0xcb783a6a, SUC_lexicon_tags_3 },
    { 0x59d83a6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d83a70, SUC_lexicon_tags_8 },
    { 0x9e183a71, SUC_lexicon_tags_37 },
    { 0xba383a70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b983a74, SUC_lexicon_tags_23 },
    { 0x8e583a75, SUC_lexicon_tags_17 },
    { 0x56983a75, SUC_lexicon_tags_0 },
    { 0xf8583a77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93783a79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93983a7c, SUC_lexicon_tags_37 },
    { 0x3b183a7c, SUC_lexicon_tags_60 },
    { 0xced83a7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26783a84, SUC_lexicon_tags_10 },
    { 0xfbd83a84, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x84b83a87, SUC_lexicon_tags_1 },
    { 0x50783a87, SUC_lexicon_tags_7 },
    { 0xc6983a87, SUC_lexicon_tags_45 },
    { 0x2ff83a8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60d83a8c, SUC_lexicon_tags_37 },
    { 0x50f83a8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b83a91, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x40183a94, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x5783a97, SUC_lexicon_tags_1 },
    { 0x9383a98, SUC_lexicon_tags_46 },
    { 0x7f183a99, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d383a9c, SUC_lexicon_tags_12 },
    { 0x58383a9d, SUC_lexicon_tags_11 },
    { 0x5f783a9d, SUC_lexicon_tags_11 },
    { 0x1c383a9f, SUC_lexicon_tags_91 },
    { 0xecb83aa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa983aa5, SUC_lexicon_tags_11 },
    { 0xc0183aa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44383aa8, SUC_lexicon_tags_17 },
    { 0xb9583aa9, SUC_lexicon_tags_10 },
    { 0xbe183aaa, SUC_lexicon_tags_43 },
    { 0xc5583aaa, SUC_lexicon_tags_17 },
    { 0x51d83aab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34583aae, SUC_lexicon_tags_33 },
    { 0x34183aaf, SUC_lexicon_tags_87 },
    { 0xb9783ab0, SUC_lexicon_tags_37 },
    { 0x86383ab0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x63583ab3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f983ab5, SUC_lexicon_tags_46 },
    { 0x7a383ab5, SUC_lexicon_tags_0 },
    { 0x6af83ab7, SUC_lexicon_tags_12 },
    { 0xf6b83ab6, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d83abe, SUC_lexicon_tags_1 },
    { 0x7c983abe, SUC_lexicon_tags_17 },
    { 0x13783abe, SUC_lexicon_tags_26 },
    { 0x7d383ac1, SUC_lexicon_tags_37 },
    { 0xd0783ac0, SUC_lexicon_tags_26 },
    { 0x3d983ac3, SUC_lexicon_tags_6 },
    { 0x28983ac3, SUC_lexicon_tags_17 },
    { 0x74183ac5, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66983ac9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b183ace, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f183ad2, SUC_lexicon_tags_10 },
    { 0xceb83ad2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a183ad6, SUC_lexicon_tags_65 },
    { 0x13783ad6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29983ad9, SUC_lexicon_tags_32 },
    { 0x7dd83ada, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f83ae0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81383ae6, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b83ae9, SUC_lexicon_tags_37 },
    { 0x8df83aea, SUC_lexicon_tags_11 },
    { 0xacf83aea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49983aee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c983af0, SUC_lexicon_tags_27 },
    { 0xf8383af1, SUC_lexicon_tags_37 },
    { 0xa3183af2, SUC_lexicon_tags_7 },
    { 0x61583af3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f83af6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34583af8, SUC_lexicon_tags_45 },
    { 0x58783af8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac183afc, SUC_lexicon_tags_17 },
    { 0xcd583afc, SUC_lexicon_tags_26 },
    { 0x65383afd, SUC_lexicon_tags_174 },
    { 0x63383afe, SUC_lexicon_tags_27 },
    { 0xdab83afc, SUC_lexicon_tags_8 },
    { 0x53183b01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd383b0c, SUC_lexicon_tags_1 },
    { 0xbdb83b0d, SUC_lexicon_tags_29 },
    { 0x80783b0d, SUC_lexicon_tags_2 },
    { 0xc3f83b0e, SUC_lexicon_tags_151 },
    { 0x8b583b10, SUC_lexicon_tags_1 },
    { 0xa5183b0d, SUC_lexicon_tags_29 },
    { 0xb7783b12, SUC_lexicon_tags_3 },
    { 0xdef83b13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b183b15, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b83b18, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f583b1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9383b21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1783b24, SUC_lexicon_tags_16 },
    { 0x4a183b25, SUC_lexicon_tags_47 },
    { 0xe4d83b26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4583b28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1e583b2a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e583b2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74583b30, SUC_lexicon_tags_39 },
    { 0x55983b30, SUC_lexicon_tags_4 },
    { 0x90183b31, SUC_lexicon_tags_15 },
    { 0x7df83b33, SUC_lexicon_tags_11 },
    { 0xbf83b34, SUC_lexicon_tags_9 },
    { 0xef583b34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf983b38, SUC_lexicon_tags_15 },
    { 0x13183b39, SUC_lexicon_tags_2 },
    { 0xf4783b38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40383b3e, SUC_lexicon_tags_17 },
    { 0xb7b83b3f, SUC_lexicon_tags_28 },
    { 0xecf83b40, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb83b43, SUC_lexicon_tags_8 },
    { 0x7983b43, SUC_lexicon_tags_14 },
    { 0xa1183b45, SUC_lexicon_tags_275 },
    { 0x7fb83b46, SUC_lexicon_tags_29 },
    { 0x4c783b46, SUC_lexicon_tags_17 },
    { 0x94b83b48, SUC_lexicon_tags_11 },
    { 0x1cf83b43, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92983b4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b983b52, SUC_lexicon_tags_47 },
    { 0x34783b52, SUC_lexicon_tags_35 },
    { 0xf4f83b52, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf83b57, SUC_lexicon_tags_37 },
    { 0x5a983b57, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef583b5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41183b65, SUC_lexicon_tags_12 },
    { 0x99d83b66, SUC_lexicon_tags_47 },
    { 0xef383b67, SUC_lexicon_tags_36 },
    { 0xd1583b68, SUC_lexicon_tags_12 },
    { 0xce783b68, SUC_lexicon_tags_1 },
    { 0x1bd83b6a, SUC_lexicon_tags_38 },
    { 0x91383b69, SUC_lexicon_tags_30 },
    { 0xc9383b67, SUC_lexicon_tags_2 },
    { 0xb1b83b68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95183b74, SUC_lexicon_tags_63 },
    { 0xce583b74, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xce383b77, SUC_lexicon_tags_63 },
    { 0x28b83b77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22d83b7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec183b7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7183b82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9983b84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0183b86, SUC_lexicon_tags_21 },
    { 0xc4183b87, SUC_lexicon_tags_6 },
    { 0x88783b88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78583b8c, SUC_lexicon_tags_109 },
    { 0x4ff83b8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f83b91, SUC_lexicon_tags_55 },
    { 0x32d83b91, SUC_lexicon_tags_46 },
    { 0xa2183b91, SUC_lexicon_tags_23 },
    { 0xacf83b94, SUC_lexicon_tags_31 },
    { 0x82183b95, SUC_lexicon_tags_29 },
    { 0x6c583b95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f83b9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d83ba2, SUC_lexicon_tags_7 },
    { 0xeef83ba3, SUC_lexicon_tags_29 },
    { 0x6f783ba4, SUC_lexicon_tags_28 },
    { 0x10783ba4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd983baa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2bf83bac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c583bae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe5f83bb0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef83bb4, SUC_lexicon_tags_14 },
    { 0x7b783bb5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28583bbb, SUC_lexicon_tags_33 },
    { 0x2a783bbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa183bc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a783bc3, SUC_lexicon_tags_11 },
    { 0x3cf83bc3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf783bc7, SUC_lexicon_tags_34 },
    { 0x59583bc7, SUC_lexicon_tags_2 },
    { 0x93b83bc9, SUC_lexicon_tags_55 },
    { 0xad983bca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20983bcd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5783bd0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf1783bd2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x91583bd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb983bdb, SUC_lexicon_tags_1 },
    { 0x35783bdc, SUC_lexicon_tags_39 },
    { 0xbf783bdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d83be0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f983be5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce983bea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10583bee, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x98d83bf0, SUC_lexicon_tags_63 },
    { 0x54b83bf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59783bf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4983bf9, SUC_lexicon_tags_6 },
    { 0xb1b83bfa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28783c00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbe183c02, SUC_lexicon_tags_61 },
    { 0x73183c02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb83c06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4183c0b, SUC_lexicon_tags_19 },
    { 0x1af83c0b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab83c10, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xceb83c12, SUC_lexicon_tags_33 },
    { 0x28183c13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55783c1a, SUC_lexicon_tags_0 },
    { 0x3e783c1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd83c1f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b83c22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b783c27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51183c2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2983c33, SUC_lexicon_tags_28 },
    { 0xe2f83c33, SUC_lexicon_tags_25 },
    { 0xebd83c34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f83c3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb0183c3f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94983c43, SUC_lexicon_tags_23 },
    { 0x92d83c43, SUC_lexicon_tags_37 },
    { 0xcd183c45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57783c4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9783c4c, SUC_lexicon_tags_63 },
    { 0x54f83c4d, SUC_lexicon_tags_7 },
    { 0xa8783c4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc1783c50, SUC_lexicon_tags_11 },
    { 0x91983c50, SUC_lexicon_tags_789 },
    { 0x76783c51, SUC_lexicon_tags_29 },
    { 0xe7183c53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f83c57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79183c5a, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a383c5d, SUC_lexicon_tags_60 },
    { 0x5ab83c5e, SUC_lexicon_tags_27 },
    { 0xfbb83c5e, SUC_lexicon_tags_47 },
    { 0x62383c60, SUC_lexicon_tags_22 },
    { 0x7ef83c60, SUC_lexicon_tags_7 },
    { 0x65383c5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69b83c64, SUC_lexicon_tags_37 },
    { 0x16f83c64, SUC_lexicon_tags_79 },
    { 0x88783c65, SUC_lexicon_tags_7 },
    { 0x61983c66, SUC_lexicon_tags_17 },
    { 0x2383c68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1983c6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b83c70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f83c73, SUC_lexicon_tags_27 },
    { 0xfbd83c74, SUC_lexicon_tags_2 },
    { 0x37b83c75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd83c79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51b83c7b, SUC_lexicon_tags_30 },
    { 0x46383c7c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2583c88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b83c93, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2583c99, SUC_lexicon_tags_65 },
    { 0x9ef83c9a, SUC_lexicon_tags_27 },
    { 0xe9183c9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb83c9e, SUC_lexicon_tags_19 },
    { 0xac783c9e, SUC_lexicon_tags_43 },
    { 0xf2d83ca0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x95183ca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27f83ca4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5783ca7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63383caa, SUC_lexicon_tags_1041 },
    { 0, NULL },
    { 0xa5583cac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2983cb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26983cb2, SUC_lexicon_tags_39 },
    { 0x10383cb3, SUC_lexicon_tags_27 },
    { 0x2eb83cb4, SUC_lexicon_tags_2 },
    { 0xde983cb5, SUC_lexicon_tags_17 },
    { 0x983cb4, SUC_lexicon_tags_0 },
    { 0x51383cb2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd83cba, SUC_lexicon_tags_12 },
    { 0x41b83cbb, SUC_lexicon_tags_1 },
    { 0x72383cba, SUC_lexicon_tags_60 },
    { 0x44f83cbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5183cc0, SUC_lexicon_tags_7 },
    { 0xc5b83cc1, SUC_lexicon_tags_37 },
    { 0x1bf83cc1, SUC_lexicon_tags_57 },
    { 0x2783cc0, SUC_lexicon_tags_2 },
    { 0x8383cc0, SUC_lexicon_tags_47 },
    { 0x81783cc5, SUC_lexicon_tags_11 },
    { 0x6583cc6, SUC_lexicon_tags_4 },
    { 0xb6d83cc5, SUC_lexicon_tags_23 },
    { 0xc7183cc8, SUC_lexicon_tags_6 },
    { 0x32783cc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89583cd0, SUC_lexicon_tags_25 },
    { 0x2a983cd1, SUC_lexicon_tags_27 },
    { 0x61d83cd1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f83cd5, SUC_lexicon_tags_17 },
    { 0x47383cd5, SUC_lexicon_tags_15 },
    { 0xe3f83cd6, SUC_lexicon_tags_2 },
    { 0x1bb83cd7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xad583cda, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1183ce0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8583ce7, SUC_lexicon_tags_43 },
    { 0x24183ce8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d83cf3, SUC_lexicon_tags_15 },
    { 0x1783cf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13783cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebd83cf8, SUC_lexicon_tags_37 },
    { 0x74383cf8, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xebd83cfb, SUC_lexicon_tags_32 },
    { 0x8e983cfb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61583d00, SUC_lexicon_tags_2 },
    { 0x60d83d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d83d06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x58583d08, SUC_lexicon_tags_33 },
    { 0x5ff83d09, SUC_lexicon_tags_1 },
    { 0x8e983d0a, SUC_lexicon_tags_17 },
    { 0x5ed83d0a, SUC_lexicon_tags_1 },
    { 0x26383d0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1f83d0e, SUC_lexicon_tags_1 },
    { 0xedf83d0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7583d19, SUC_lexicon_tags_23 },
    { 0xe9783d1a, SUC_lexicon_tags_9 },
    { 0x90183d1b, SUC_lexicon_tags_31 },
    { 0x96b83d1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c583d1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f83d23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73183d30, SUC_lexicon_tags_0 },
    { 0x16983d30, SUC_lexicon_tags_38 },
    { 0x58783d32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdb583d34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a583d38, SUC_lexicon_tags_31 },
    { 0xf2183d38, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c383d3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bd83d3e, SUC_lexicon_tags_30 },
    { 0xde383d3f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x87783d41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x48983d44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23383d46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3983d4b, SUC_lexicon_tags_75 },
    { 0x9d583d4c, SUC_lexicon_tags_45 },
    { 0x83783d4d, SUC_lexicon_tags_47 },
    { 0x5af83d4b, SUC_lexicon_tags_23 },
    { 0x16f83d4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf983d51, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d83d57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75d83d59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd83d5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c583d5f, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23583d63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12783d65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca983d67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b83d6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7783d71, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8383d74, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f83d7a, SUC_lexicon_tags_33 },
    { 0x35d83d7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb783d80, SUC_lexicon_tags_12 },
    { 0x92d83d81, SUC_lexicon_tags_1 },
    { 0x3b783d82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d83d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab83d8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96983d8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92b83d90, SUC_lexicon_tags_8 },
    { 0x51b83d90, SUC_lexicon_tags_33 },
    { 0x1583d91, SUC_lexicon_tags_8 },
    { 0x73d83d93, SUC_lexicon_tags_16 },
    { 0x39f83d94, SUC_lexicon_tags_12 },
    { 0xedf83d92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68783d99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x79783d9b, SUC_lexicon_tags_11 },
    { 0x90f83d9c, SUC_lexicon_tags_47 },
    { 0x6583d9d, SUC_lexicon_tags_55 },
    { 0xd0d83d9e, SUC_lexicon_tags_48 },
    { 0x98983d9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb383da1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x90183da3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb983da8, SUC_lexicon_tags_28 },
    { 0x2d583da9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2583dac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e583db5, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29383dba, SUC_lexicon_tags_2 },
    { 0xcdd83dba, SUC_lexicon_tags_2 },
    { 0xf8b83dbc, SUC_lexicon_tags_1 },
    { 0xe6383dbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xac583dbf, SUC_lexicon_tags_7 },
    { 0xa0d83dbf, SUC_lexicon_tags_8 },
    { 0x53583dc1, SUC_lexicon_tags_31 },
    { 0x64183dbf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f83dc5, SUC_lexicon_tags_35 },
    { 0xbe183dc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff83dc9, SUC_lexicon_tags_37 },
    { 0xfeb83dca, SUC_lexicon_tags_14 },
    { 0xe4783dca, SUC_lexicon_tags_43 },
    { 0x6ad83dcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb83dd0, SUC_lexicon_tags_37 },
    { 0xf6f83dd1, SUC_lexicon_tags_17 },
    { 0x98983dd0, SUC_lexicon_tags_14 },
    { 0xd0383dd3, SUC_lexicon_tags_48 },
    { 0x4c183dd4, SUC_lexicon_tags_1417 },
    { 0, NULL },
    { 0x6c783dd6, SUC_lexicon_tags_158 },
    { 0x83b83dd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f83ddb, SUC_lexicon_tags_25 },
    { 0x1dd83ddc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4383dde, SUC_lexicon_tags_45 },
    { 0x20783ddf, SUC_lexicon_tags_11 },
    { 0x46d83de0, SUC_lexicon_tags_11 },
    { 0x34383dde, SUC_lexicon_tags_27 },
    { 0xadd83de1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f83de5, SUC_lexicon_tags_37 },
    { 0x7f783de6, SUC_lexicon_tags_2 },
    { 0x6a583de7, SUC_lexicon_tags_13 },
    { 0xf0183de8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99783df1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2383df5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f183dfb, SUC_lexicon_tags_29 },
    { 0x90983dfc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f83e01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e383e07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ad83e09, SUC_lexicon_tags_37 },
    { 0x22183e09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9383e0e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf83e12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d383e15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd983e18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52183e1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1983e20, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x75983e22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5bb83e24, SUC_lexicon_tags_27 },
    { 0x8d983e24, SUC_lexicon_tags_69 },
    { 0x95383e24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6983e2b, SUC_lexicon_tags_48 },
    { 0x6bb83e2c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaeb83e2e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66383e34, SUC_lexicon_tags_37 },
    { 0x3c583e35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd83e39, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb1183e3b, SUC_lexicon_tags_27 },
    { 0x3583e3b, SUC_lexicon_tags_17 },
    { 0xb2383e3b, SUC_lexicon_tags_2 },
    { 0xc6b83e3e, SUC_lexicon_tags_10 },
    { 0x74f83e3e, SUC_lexicon_tags_25 },
    { 0xd8583e3e, SUC_lexicon_tags_5 },
    { 0xa4783e3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d83e44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa0783e46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf83e4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9383e4c, SUC_lexicon_tags_63 },
    { 0x32983e4d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xaff83e4f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f83e57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0383e5b, SUC_lexicon_tags_2 },
    { 0x89183e5b, SUC_lexicon_tags_60 },
    { 0x4f83e5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b83e63, SUC_lexicon_tags_34 },
    { 0xaef83e63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x96b83e66, SUC_lexicon_tags_35 },
    { 0x9a983e66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d83e6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58583e6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67383e71, SUC_lexicon_tags_15 },
    { 0x6bb83e71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x96383e75, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x86583e78, SUC_lexicon_tags_15 },
    { 0x64f83e79, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b83e7e, SUC_lexicon_tags_17 },
    { 0x51983e7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac583e83, SUC_lexicon_tags_6 },
    { 0x8df83e84, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b83e89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc383e8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27383e94, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf83e98, SUC_lexicon_tags_341 },
    { 0x46f83e99, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0x24383e9b, SUC_lexicon_tags_17 },
    { 0xfdd83e9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcef83e9e, SUC_lexicon_tags_47 },
    { 0x6d583e9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a383ea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a183ea3, SUC_lexicon_tags_2 },
    { 0xeed83ea3, SUC_lexicon_tags_47 },
    { 0x4d783ea5, SUC_lexicon_tags_10 },
    { 0x66783ea6, SUC_lexicon_tags_43 },
    { 0xc0b83ea5, SUC_lexicon_tags_98 },
    { 0x6b83ea3, SUC_lexicon_tags_46 },
    { 0xbbd83ea3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d183eae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab83eb1, SUC_lexicon_tags_11 },
    { 0x20583eb2, SUC_lexicon_tags_25 },
    { 0x39783eb3, SUC_lexicon_tags_32 },
    { 0xd6183eb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7b583eb6, SUC_lexicon_tags_47 },
    { 0x66583eb7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x54183eb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6b83ebb, SUC_lexicon_tags_12 },
    { 0xb0d83ebc, SUC_lexicon_tags_48 },
    { 0xe1183ebc, SUC_lexicon_tags_16 },
    { 0xaa383ebe, SUC_lexicon_tags_31 },
    { 0xe5383ebf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0983ec5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7f83ec7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf83ecd, SUC_lexicon_tags_17 },
    { 0xbe783ecd, SUC_lexicon_tags_39 },
    { 0x6b83ecf, SUC_lexicon_tags_33 },
    { 0xe9183ecf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab83ed3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66583ed7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67b83ed9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb83ede, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9183ee3, SUC_lexicon_tags_23 },
    { 0xca183ee4, SUC_lexicon_tags_60 },
    { 0xb2383ee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99783ef3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d983efd, SUC_lexicon_tags_37 },
    { 0x60d83efe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x96183f00, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x583f02, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc4783f04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83183f09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf83f12, SUC_lexicon_tags_12 },
    { 0xc4d83f12, SUC_lexicon_tags_23 },
    { 0xe5183f14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59583f17, SUC_lexicon_tags_11 },
    { 0x8df83f17, SUC_lexicon_tags_7 },
    { 0xc3783f19, SUC_lexicon_tags_12 },
    { 0x96b83f1a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b83f21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b83f24, SUC_lexicon_tags_25 },
    { 0xbe783f24, SUC_lexicon_tags_10 },
    { 0x4bd83f24, SUC_lexicon_tags_87 },
    { 0xc5983f27, SUC_lexicon_tags_0 },
    { 0x58183f27, SUC_lexicon_tags_38 },
    { 0x2fd83f25, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc4183f2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20983f32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62d83f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95b83f36, SUC_lexicon_tags_518 },
    { 0x3383f37, SUC_lexicon_tags_60 },
    { 0xca583f38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f983f3c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d83f40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6383f43, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af83f4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6383f52, SUC_lexicon_tags_32 },
    { 0x74383f53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6983f56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8f83f58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13183f5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2183f64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b83f68, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x88d83f6a, SUC_lexicon_tags_29 },
    { 0x2cb83f6a, SUC_lexicon_tags_15 },
    { 0x17583f6c, SUC_lexicon_tags_25 },
    { 0xe3d83f6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd83f75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd83f78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80d83f7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13f83f7c, SUC_lexicon_tags_7 },
    { 0x26583f7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93783f7f, SUC_lexicon_tags_7 },
    { 0x70983f80, SUC_lexicon_tags_14 },
    { 0x76583f81, SUC_lexicon_tags_5 },
    { 0xa4f83f82, SUC_lexicon_tags_43 },
    { 0xba383f82, SUC_lexicon_tags_87 },
    { 0x76983f82, SUC_lexicon_tags_16 },
    { 0x55d83f85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd83f8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4383f97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xba183f9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd83f9e, SUC_lexicon_tags_37 },
    { 0xc5183f9e, SUC_lexicon_tags_12 },
    { 0xc0983f9f, SUC_lexicon_tags_17 },
    { 0xd4383fa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d383fa3, SUC_lexicon_tags_14 },
    { 0x9e183fa4, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b83fa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac583fab, SUC_lexicon_tags_12 },
    { 0xfb783fac, SUC_lexicon_tags_107 },
    { 0x3f583fac, SUC_lexicon_tags_11 },
    { 0xef983fae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa183fb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d83fb3, SUC_lexicon_tags_11 },
    { 0xea183fb4, SUC_lexicon_tags_11 },
    { 0xec183fb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51183fb8, SUC_lexicon_tags_25 },
    { 0xd6383fb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b83fbf, SUC_lexicon_tags_14 },
    { 0xf9b83fc0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe2183fc2, SUC_lexicon_tags_38 },
    { 0xacf83fc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f83fc7, SUC_lexicon_tags_11 },
    { 0x40b83fc8, SUC_lexicon_tags_17 },
    { 0x8983fc9, SUC_lexicon_tags_7 },
    { 0xbdf83fc7, SUC_lexicon_tags_68 },
    { 0xa1783fcb, SUC_lexicon_tags_29 },
    { 0x83583fcb, SUC_lexicon_tags_32 },
    { 0x38d83fca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbcd83fcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfed83fd1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd383fd6, SUC_lexicon_tags_25 },
    { 0x63583fd7, SUC_lexicon_tags_7 },
    { 0xe1b83fd7, SUC_lexicon_tags_2 },
    { 0xefb83fd9, SUC_lexicon_tags_29 },
    { 0x5a983fda, SUC_lexicon_tags_27 },
    { 0x6b83fdb, SUC_lexicon_tags_26 },
    { 0xe6983fdc, SUC_lexicon_tags_28 },
    { 0xb1783fdd, SUC_lexicon_tags_43 },
    { 0xb5b83fd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed83fe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe983fea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4983fee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7983ff2, SUC_lexicon_tags_6 },
    { 0x4fb83ff2, SUC_lexicon_tags_49 },
    { 0x6383ff4, SUC_lexicon_tags_17 },
    { 0xca583ff5, SUC_lexicon_tags_3 },
    { 0xfb383ff5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb983ff8, SUC_lexicon_tags_2 },
    { 0xc7b83ff8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xacb83ffb, SUC_lexicon_tags_1 },
    { 0xb8d83ffc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x25183ffe, SUC_lexicon_tags_25 },
    { 0x41d83fff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x80384002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1584009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9778400b, SUC_lexicon_tags_38 },
    { 0xd698400b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b18400e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee184013, SUC_lexicon_tags_17 },
    { 0xd4984014, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf84018, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8401c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcbf8401e, SUC_lexicon_tags_2 },
    { 0xd958401f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb84022, SUC_lexicon_tags_22 },
    { 0x45384022, SUC_lexicon_tags_29 },
    { 0xd2584023, SUC_lexicon_tags_1 },
    { 0x10d84025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd8402b, SUC_lexicon_tags_79 },
    { 0xa1b8402b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3738402e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f84033, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x58784036, SUC_lexicon_tags_11 },
    { 0xb1384037, SUC_lexicon_tags_26 },
    { 0xcab84036, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f84041, SUC_lexicon_tags_11 },
    { 0x20d84042, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b84045, SUC_lexicon_tags_29 },
    { 0x29984045, SUC_lexicon_tags_2 },
    { 0x44184047, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd158404f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48584055, SUC_lexicon_tags_17 },
    { 0x28f84056, SUC_lexicon_tags_12 },
    { 0xeb784056, SUC_lexicon_tags_17 },
    { 0xfe984057, SUC_lexicon_tags_34 },
    { 0xee584058, SUC_lexicon_tags_30 },
    { 0xcdb8405a, SUC_lexicon_tags_27 },
    { 0x54184057, SUC_lexicon_tags_31 },
    { 0x59d84058, SUC_lexicon_tags_69 },
    { 0xf9584058, SUC_lexicon_tags_7 },
    { 0x50384058, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f984061, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe5784063, SUC_lexicon_tags_82 },
    { 0x62f84064, SUC_lexicon_tags_12 },
    { 0xfe784065, SUC_lexicon_tags_92 },
    { 0x2c384063, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f8406a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc658406f, SUC_lexicon_tags_1 },
    { 0xebf8406f, SUC_lexicon_tags_37 },
    { 0xa1584070, SUC_lexicon_tags_0 },
    { 0x34784072, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8407b, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd84081, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe6b84083, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x14384085, SUC_lexicon_tags_37 },
    { 0x76f84086, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67984088, SUC_lexicon_tags_2 },
    { 0x24784089, SUC_lexicon_tags_47 },
    { 0xc218408a, SUC_lexicon_tags_27 },
    { 0x88b84089, SUC_lexicon_tags_65 },
    { 0xc0f84089, SUC_lexicon_tags_34 },
    { 0x2578408d, SUC_lexicon_tags_14 },
    { 0x5818408e, SUC_lexicon_tags_37 },
    { 0x4e78408f, SUC_lexicon_tags_37 },
    { 0x39784090, SUC_lexicon_tags_38 },
    { 0x38d8408b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x69384093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb84099, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8798409d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x92d8409f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f840a6, SUC_lexicon_tags_1 },
    { 0xe43840a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d840ab, SUC_lexicon_tags_12 },
    { 0xccd840ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d840b0, SUC_lexicon_tags_82 },
    { 0x31f840b0, SUC_lexicon_tags_35 },
    { 0xd6b840b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc53840b4, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67840b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323840bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x775840bf, SUC_lexicon_tags_9 },
    { 0x593840c0, SUC_lexicon_tags_17 },
    { 0x66b840c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab840c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25d840c9, SUC_lexicon_tags_17 },
    { 0xc37840ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xef1840cc, SUC_lexicon_tags_11 },
    { 0x6fd840cd, SUC_lexicon_tags_11 },
    { 0xf45840cd, SUC_lexicon_tags_8 },
    { 0x381840cd, SUC_lexicon_tags_1 },
    { 0x4b840d0, SUC_lexicon_tags_12 },
    { 0x5b9840cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x975840d3, SUC_lexicon_tags_7 },
    { 0x4ff840d3, SUC_lexicon_tags_37 },
    { 0x6b7840d4, SUC_lexicon_tags_37 },
    { 0x685840d6, SUC_lexicon_tags_82 },
    { 0x695840d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef1840d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7840dd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xa81840df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfe9840e1, SUC_lexicon_tags_25 },
    { 0xa49840e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd840e6, SUC_lexicon_tags_37 },
    { 0x845840e7, SUC_lexicon_tags_39 },
    { 0x4b7840e8, SUC_lexicon_tags_17 },
    { 0xb79840e9, SUC_lexicon_tags_47 },
    { 0x8bf840ea, SUC_lexicon_tags_1 },
    { 0x643840eb, SUC_lexicon_tags_35 },
    { 0x82d840ec, SUC_lexicon_tags_25 },
    { 0x46b840ed, SUC_lexicon_tags_17 },
    { 0x1f840ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x611840f1, SUC_lexicon_tags_26 },
    { 0x93d840f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b840f7, SUC_lexicon_tags_60 },
    { 0xa9840f8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7840fb, SUC_lexicon_tags_49 },
    { 0x7f9840fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x595840fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d584100, SUC_lexicon_tags_82 },
    { 0x5bd84101, SUC_lexicon_tags_47 },
    { 0x95384102, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5e184104, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f84106, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b8410a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42584112, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41184116, SUC_lexicon_tags_25 },
    { 0x96384117, SUC_lexicon_tags_25 },
    { 0x4cb84118, SUC_lexicon_tags_38 },
    { 0x85984119, SUC_lexicon_tags_7 },
    { 0x1a984117, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62784120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68784122, SUC_lexicon_tags_14 },
    { 0x8bf84123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a784126, SUC_lexicon_tags_35 },
    { 0x98584126, SUC_lexicon_tags_53 },
    { 0xad184126, SUC_lexicon_tags_1 },
    { 0x54984127, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b18412c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9918412f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91984132, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a384134, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b84137, SUC_lexicon_tags_11 },
    { 0x13184138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58413e, SUC_lexicon_tags_35 },
    { 0x2a58413e, SUC_lexicon_tags_0 },
    { 0xdb384140, SUC_lexicon_tags_19 },
    { 0xa9584140, SUC_lexicon_tags_47 },
    { 0xb4984140, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c184144, SUC_lexicon_tags_37 },
    { 0xd4584144, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab8414a, SUC_lexicon_tags_7 },
    { 0x53f8414b, SUC_lexicon_tags_15 },
    { 0x2658414c, SUC_lexicon_tags_47 },
    { 0x1058414c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7218414f, SUC_lexicon_tags_11 },
    { 0xddd84150, SUC_lexicon_tags_23 },
    { 0x5eb84151, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x31584154, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb84158, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9178415a, SUC_lexicon_tags_11 },
    { 0x72b8415b, SUC_lexicon_tags_11 },
    { 0x1398415b, SUC_lexicon_tags_47 },
    { 0xd78415d, SUC_lexicon_tags_7 },
    { 0x2f8415e, SUC_lexicon_tags_77 },
    { 0xb5b8415f, SUC_lexicon_tags_13 },
    { 0xc958415f, SUC_lexicon_tags_27 },
    { 0xeaf84161, SUC_lexicon_tags_27 },
    { 0xe9b84162, SUC_lexicon_tags_20 },
    { 0x77184162, SUC_lexicon_tags_15 },
    { 0x2c58415d, SUC_lexicon_tags_37 },
    { 0xd6b84161, SUC_lexicon_tags_37 },
    { 0xb638415e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73f84168, SUC_lexicon_tags_42 },
    { 0x5c984169, SUC_lexicon_tags_45 },
    { 0x5a38416a, SUC_lexicon_tags_127 },
    { 0x87384168, SUC_lexicon_tags_83 },
    { 0xa9d8416c, SUC_lexicon_tags_0 },
    { 0x638416b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa878416f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4984172, SUC_lexicon_tags_2 },
    { 0x12184173, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d8417a, SUC_lexicon_tags_17 },
    { 0xc758417b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa98417e, SUC_lexicon_tags_15 },
    { 0xb78417f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x99184182, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x12384185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69b84187, SUC_lexicon_tags_11 },
    { 0xe0184188, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe498418b, SUC_lexicon_tags_10 },
    { 0x6eb8418c, SUC_lexicon_tags_26 },
    { 0x9978418b, SUC_lexicon_tags_17 },
    { 0xa3f8418e, SUC_lexicon_tags_37 },
    { 0x9638418c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d984195, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db84199, SUC_lexicon_tags_17 },
    { 0xb858419a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff841a6, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xb79841a8, SUC_lexicon_tags_63 },
    { 0xc95841a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf841b1, SUC_lexicon_tags_45 },
    { 0x4ef841b2, SUC_lexicon_tags_7 },
    { 0xc0f841b3, SUC_lexicon_tags_2 },
    { 0xfc1841b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3841b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c7841ba, SUC_lexicon_tags_66 },
    { 0x383841ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f841be, SUC_lexicon_tags_2 },
    { 0xd65841bf, SUC_lexicon_tags_25 },
    { 0x44d841c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211841c5, SUC_lexicon_tags_6 },
    { 0xf1f841c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b841ce, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x483841d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf9841d2, SUC_lexicon_tags_3 },
    { 0x98b841d2, SUC_lexicon_tags_25 },
    { 0x20d841d4, SUC_lexicon_tags_25 },
    { 0x8a3841d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf1841d7, SUC_lexicon_tags_37 },
    { 0x1f9841d7, SUC_lexicon_tags_15 },
    { 0xce3841d9, SUC_lexicon_tags_21 },
    { 0x761841d9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f841dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729841e6, SUC_lexicon_tags_26 },
    { 0x867841e7, SUC_lexicon_tags_3 },
    { 0x649841e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x813841ea, SUC_lexicon_tags_1 },
    { 0x497841eb, SUC_lexicon_tags_15 },
    { 0xb11841ec, SUC_lexicon_tags_1 },
    { 0x629841ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa49841ef, SUC_lexicon_tags_31 },
    { 0x8eb841ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb841f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x579841f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3841fb, SUC_lexicon_tags_19 },
    { 0xcb841fc, SUC_lexicon_tags_10 },
    { 0x43f841fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x549841ff, SUC_lexicon_tags_25 },
    { 0x20f84200, SUC_lexicon_tags_1 },
    { 0xe0d84201, SUC_lexicon_tags_47 },
    { 0xe8f84202, SUC_lexicon_tags_25 },
    { 0x89184203, SUC_lexicon_tags_25 },
    { 0xa2384203, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x97f84206, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94384209, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe778420d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1078420f, SUC_lexicon_tags_29 },
    { 0x2bd8420f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd84213, SUC_lexicon_tags_63 },
    { 0xb1f84214, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdab84216, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x75184218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x458421a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a98421c, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f8421f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf338422c, SUC_lexicon_tags_17 },
    { 0xa238422c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b84231, SUC_lexicon_tags_14 },
    { 0xbc184231, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2384234, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19584237, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8318423a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabf8423c, SUC_lexicon_tags_17 },
    { 0x4e98423c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ef8423f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2184242, SUC_lexicon_tags_15 },
    { 0x61d84243, SUC_lexicon_tags_1 },
    { 0x2a784242, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f184246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72984248, SUC_lexicon_tags_198 },
    { 0x2d84248, SUC_lexicon_tags_1 },
    { 0xb798424a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1558424d, SUC_lexicon_tags_39 },
    { 0x9518424e, SUC_lexicon_tags_23 },
    { 0x3eb8424d, SUC_lexicon_tags_40 },
    { 0x8d184250, SUC_lexicon_tags_2 },
    { 0xd1384251, SUC_lexicon_tags_12 },
    { 0xad984252, SUC_lexicon_tags_17 },
    { 0x57384250, SUC_lexicon_tags_37 },
    { 0xb8b84253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9038425a, SUC_lexicon_tags_15 },
    { 0x3c98425b, SUC_lexicon_tags_14 },
    { 0xfcb8425c, SUC_lexicon_tags_17 },
    { 0x4ab8425c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x8ef8425f, SUC_lexicon_tags_25 },
    { 0x3598425f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40784262, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb584265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b8426b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab8426f, SUC_lexicon_tags_17 },
    { 0x66584270, SUC_lexicon_tags_21 },
    { 0x92184271, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f18427a, SUC_lexicon_tags_15 },
    { 0x38d8427b, SUC_lexicon_tags_11 },
    { 0xb518427a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c984281, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x48f84283, SUC_lexicon_tags_12 },
    { 0x33384284, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d8428c, SUC_lexicon_tags_30 },
    { 0x5a58428d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43184292, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d84299, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb5b8429b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d8429e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d842a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x489842a3, SUC_lexicon_tags_46 },
    { 0x257842a3, SUC_lexicon_tags_56 },
    { 0xfb3842a4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d842b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f842b5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b842b8, SUC_lexicon_tags_65 },
    { 0xe6b842b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d842bf, SUC_lexicon_tags_60 },
    { 0x567842bf, SUC_lexicon_tags_47 },
    { 0xe1842c1, SUC_lexicon_tags_48 },
    { 0x6fd842c2, SUC_lexicon_tags_15 },
    { 0x197842c1, SUC_lexicon_tags_15 },
    { 0x195842c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f7842c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xeab842c9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7b3842cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39d842cd, SUC_lexicon_tags_37 },
    { 0xcf842cd, SUC_lexicon_tags_1 },
    { 0xf21842cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ed842d1, SUC_lexicon_tags_38 },
    { 0xc5d842d1, SUC_lexicon_tags_8 },
    { 0x503842d2, SUC_lexicon_tags_68 },
    { 0x8fb842d3, SUC_lexicon_tags_11 },
    { 0xa77842d5, SUC_lexicon_tags_11 },
    { 0x605842d6, SUC_lexicon_tags_17 },
    { 0xb19842d6, SUC_lexicon_tags_15 },
    { 0xb95842d8, SUC_lexicon_tags_1 },
    { 0xe0b842d9, SUC_lexicon_tags_11 },
    { 0xff9842da, SUC_lexicon_tags_1 },
    { 0xd79842d8, SUC_lexicon_tags_9 },
    { 0x3af842dc, SUC_lexicon_tags_12 },
    { 0x28d842d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f842e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x231842e3, SUC_lexicon_tags_11 },
    { 0x729842e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1ff842e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843842f0, SUC_lexicon_tags_44 },
    { 0xb15842f0, SUC_lexicon_tags_18 },
    { 0xf97842f2, SUC_lexicon_tags_16 },
    { 0x731842f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x753842f5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8384300, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b84303, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37584305, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf3d84307, SUC_lexicon_tags_25 },
    { 0xdf984308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd18430d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2184310, SUC_lexicon_tags_12 },
    { 0xfa184310, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e784314, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70984316, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d18431a, SUC_lexicon_tags_1 },
    { 0x2e78431b, SUC_lexicon_tags_17 },
    { 0x3b38431c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd84321, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d184325, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x17784328, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba58432e, SUC_lexicon_tags_68 },
    { 0x2b38432f, SUC_lexicon_tags_37 },
    { 0xe5384330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99784333, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf784336, SUC_lexicon_tags_0 },
    { 0xc3b84336, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcc784339, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb8433e, SUC_lexicon_tags_87 },
    { 0x64d8433f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28d84341, SUC_lexicon_tags_15 },
    { 0xfd384342, SUC_lexicon_tags_61 },
    { 0x58f84343, SUC_lexicon_tags_11 },
    { 0x1c184344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaab84346, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb8784348, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af8434c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd84350, SUC_lexicon_tags_34 },
    { 0xcc984351, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0384357, SUC_lexicon_tags_13 },
    { 0x99184357, SUC_lexicon_tags_4 },
    { 0x6bd84357, SUC_lexicon_tags_37 },
    { 0x8d584357, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd8435f, SUC_lexicon_tags_16 },
    { 0xa0784360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf184365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87384367, SUC_lexicon_tags_37 },
    { 0x3f984368, SUC_lexicon_tags_29 },
    { 0xa2384369, SUC_lexicon_tags_282 },
    { 0x8418436a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78436d, SUC_lexicon_tags_2 },
    { 0xfd58436e, SUC_lexicon_tags_7 },
    { 0x9cb8436d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x90584372, SUC_lexicon_tags_27 },
    { 0xb0184372, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb784375, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30784377, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa7584379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e78437b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6918437d, SUC_lexicon_tags_37 },
    { 0x5518437e, SUC_lexicon_tags_45 },
    { 0xea18437e, SUC_lexicon_tags_62 },
    { 0xd0b8437f, SUC_lexicon_tags_49 },
    { 0x9d84381, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac184383, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x73d84385, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0384387, SUC_lexicon_tags_0 },
    { 0x12f84388, SUC_lexicon_tags_17 },
    { 0xc3984388, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38438d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b84391, SUC_lexicon_tags_39 },
    { 0xd9b84392, SUC_lexicon_tags_7 },
    { 0xe9984393, SUC_lexicon_tags_77 },
    { 0x27184393, SUC_lexicon_tags_45 },
    { 0xc6184395, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1384398, SUC_lexicon_tags_37 },
    { 0x9db84398, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd198439d, SUC_lexicon_tags_2 },
    { 0x8a98439e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d843a3, SUC_lexicon_tags_45 },
    { 0x22d843a4, SUC_lexicon_tags_121 },
    { 0xb7d843a5, SUC_lexicon_tags_43 },
    { 0xdf843a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5843ad, SUC_lexicon_tags_47 },
    { 0xbbd843ae, SUC_lexicon_tags_37 },
    { 0xca9843af, SUC_lexicon_tags_14 },
    { 0xdb843b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59843b8, SUC_lexicon_tags_15 },
    { 0xb77843b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc77843bb, SUC_lexicon_tags_11 },
    { 0x15f843bc, SUC_lexicon_tags_17 },
    { 0x5f7843bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x513843c0, SUC_lexicon_tags_0 },
    { 0xed9843c1, SUC_lexicon_tags_15 },
    { 0xbd9843c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf843cc, SUC_lexicon_tags_15 },
    { 0x135843cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb843cf, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5843da, SUC_lexicon_tags_17 },
    { 0xd15843db, SUC_lexicon_tags_15 },
    { 0xa19843dc, SUC_lexicon_tags_3 },
    { 0xdf9843dc, SUC_lexicon_tags_0 },
    { 0x86b843dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9843e6, SUC_lexicon_tags_2 },
    { 0x1843e6, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b843ee, SUC_lexicon_tags_1 },
    { 0x555843ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0b843f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b7843f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c1843f5, SUC_lexicon_tags_14 },
    { 0xe31843f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b843fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd5843fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6184402, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f84406, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2384408, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8918440b, SUC_lexicon_tags_32 },
    { 0x9b38440c, SUC_lexicon_tags_33 },
    { 0xc1f8440b, SUC_lexicon_tags_40 },
    { 0x7218440c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d84413, SUC_lexicon_tags_23 },
    { 0xf3184413, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd84417, SUC_lexicon_tags_55 },
    { 0x26f84418, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed8441d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea984421, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e184423, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x3cd84425, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ff84427, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x59184429, SUC_lexicon_tags_17 },
    { 0xa858442a, SUC_lexicon_tags_17 },
    { 0x6ad8442b, SUC_lexicon_tags_7 },
    { 0x82f8442a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c18442f, SUC_lexicon_tags_33 },
    { 0x91184430, SUC_lexicon_tags_46 },
    { 0xf8984431, SUC_lexicon_tags_19 },
    { 0xd198442f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f984434, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x78984437, SUC_lexicon_tags_60 },
    { 0xe4184437, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d8443d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2184441, SUC_lexicon_tags_17 },
    { 0xd5784442, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa1384444, SUC_lexicon_tags_49 },
    { 0x9c184445, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c38444f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb38445d, SUC_lexicon_tags_1 },
    { 0x3058445e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d84465, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3f84467, SUC_lexicon_tags_60 },
    { 0x37584468, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf84470, SUC_lexicon_tags_17 },
    { 0x66784471, SUC_lexicon_tags_42 },
    { 0x1f984472, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d84476, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d84479, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf478447b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3838447e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb84481, SUC_lexicon_tags_25 },
    { 0x8ef84482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b84486, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b58448b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b384490, SUC_lexicon_tags_77 },
    { 0x74184491, SUC_lexicon_tags_6 },
    { 0x91184492, SUC_lexicon_tags_39 },
    { 0x98384490, SUC_lexicon_tags_136 },
    { 0x4a584494, SUC_lexicon_tags_39 },
    { 0x54384495, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d8449e, SUC_lexicon_tags_10 },
    { 0x2538449e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781844a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f844a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb844ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9844af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac9844b1, SUC_lexicon_tags_17 },
    { 0x7a3844b2, SUC_lexicon_tags_37 },
    { 0x48b844b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ab844b5, SUC_lexicon_tags_33 },
    { 0xdef844b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff844c0, SUC_lexicon_tags_43 },
    { 0xf65844c1, SUC_lexicon_tags_14 },
    { 0x27b844c2, SUC_lexicon_tags_23 },
    { 0x4c7844c0, SUC_lexicon_tags_11 },
    { 0x705844c2, SUC_lexicon_tags_47 },
    { 0x235844c5, SUC_lexicon_tags_49 },
    { 0xb83844c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa5844c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84f844ca, SUC_lexicon_tags_70 },
    { 0xd15844cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d844d3, SUC_lexicon_tags_65 },
    { 0x71844d4, SUC_lexicon_tags_6 },
    { 0x76b844d3, SUC_lexicon_tags_47 },
    { 0xf57844d6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x689844d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf844dc, SUC_lexicon_tags_1 },
    { 0xcf1844dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9844e1, SUC_lexicon_tags_25 },
    { 0x869844e2, SUC_lexicon_tags_47 },
    { 0x2ed844e3, SUC_lexicon_tags_2 },
    { 0xd49844e4, SUC_lexicon_tags_11 },
    { 0xfe1844e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537844e9, SUC_lexicon_tags_25 },
    { 0x71b844ea, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x26b844ec, SUC_lexicon_tags_17 },
    { 0x50b844ec, SUC_lexicon_tags_82 },
    { 0x8c5844ee, SUC_lexicon_tags_25 },
    { 0x29844ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469844f4, SUC_lexicon_tags_11 },
    { 0xe7f844f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5af844f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x275844fa, SUC_lexicon_tags_7 },
    { 0xf35844fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3844ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9e384501, SUC_lexicon_tags_17 },
    { 0xe584502, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f84505, SUC_lexicon_tags_7 },
    { 0x99984506, SUC_lexicon_tags_19 },
    { 0x82f84507, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1cf84509, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d8450c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78384510, SUC_lexicon_tags_7 },
    { 0xb2984510, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85784513, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf84516, SUC_lexicon_tags_7 },
    { 0x6a384516, SUC_lexicon_tags_60 },
    { 0x3ff84518, SUC_lexicon_tags_15 },
    { 0xdb184518, SUC_lexicon_tags_30 },
    { 0x6258451a, SUC_lexicon_tags_17 },
    { 0xb8b8451a, SUC_lexicon_tags_43 },
    { 0x84d8451c, SUC_lexicon_tags_30 },
    { 0x77b84519, SUC_lexicon_tags_45 },
    { 0x20984518, SUC_lexicon_tags_7 },
    { 0xcdb8451d, SUC_lexicon_tags_10 },
    { 0xae984520, SUC_lexicon_tags_2 },
    { 0xf6784519, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18384524, SUC_lexicon_tags_12 },
    { 0x4a384524, SUC_lexicon_tags_66 },
    { 0x22f84524, SUC_lexicon_tags_84 },
    { 0xc8784527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb8452b, SUC_lexicon_tags_26 },
    { 0x1078452c, SUC_lexicon_tags_47 },
    { 0x1198452d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf84533, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb18453e, SUC_lexicon_tags_65 },
    { 0xd698453f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38984549, SUC_lexicon_tags_37 },
    { 0xea184549, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98d8454c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d84550, SUC_lexicon_tags_55 },
    { 0xbb584551, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6784554, SUC_lexicon_tags_63 },
    { 0x19984554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2818455b, SUC_lexicon_tags_23 },
    { 0x4c38455b, SUC_lexicon_tags_49 },
    { 0xb158455d, SUC_lexicon_tags_1 },
    { 0x36f8455d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83584560, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a784566, SUC_lexicon_tags_42 },
    { 0x2d584566, SUC_lexicon_tags_29 },
    { 0x2d184568, SUC_lexicon_tags_66 },
    { 0xb9b84569, SUC_lexicon_tags_11 },
    { 0x6418456a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87584570, SUC_lexicon_tags_37 },
    { 0x8f984570, SUC_lexicon_tags_37 },
    { 0xfed84570, SUC_lexicon_tags_2 },
    { 0x39b84573, SUC_lexicon_tags_6 },
    { 0x5e784574, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb84577, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb8457a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2984588, SUC_lexicon_tags_49 },
    { 0xf9784589, SUC_lexicon_tags_39 },
    { 0x5ad84588, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a584596, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b8459a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3018459c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ff8459e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b845a1, SUC_lexicon_tags_28 },
    { 0xfe7845a2, SUC_lexicon_tags_234 },
    { 0x8e3845a2, SUC_lexicon_tags_25 },
    { 0x2a3845a2, SUC_lexicon_tags_30 },
    { 0x439845a5, SUC_lexicon_tags_19 },
    { 0x2ff845a6, SUC_lexicon_tags_6 },
    { 0x38b845a7, SUC_lexicon_tags_27 },
    { 0xaed845a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865845b1, SUC_lexicon_tags_60 },
    { 0x30b845b1, SUC_lexicon_tags_6 },
    { 0x57845b3, SUC_lexicon_tags_25 },
    { 0xa49845b4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x205845b6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x777845b9, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x891845bb, SUC_lexicon_tags_33 },
    { 0x18b845bb, SUC_lexicon_tags_0 },
    { 0x4d845bd, SUC_lexicon_tags_17 },
    { 0xf9f845bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x857845c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4e3845c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5845c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd845c9, SUC_lexicon_tags_39 },
    { 0xdcd845ca, SUC_lexicon_tags_37 },
    { 0xca1845cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x179845cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691845d1, SUC_lexicon_tags_12 },
    { 0xf9b845d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc67845d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5845d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdcd845d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b845dc, SUC_lexicon_tags_11 },
    { 0xc2d845dd, SUC_lexicon_tags_65 },
    { 0xdcb845dd, SUC_lexicon_tags_11 },
    { 0x8f1845de, SUC_lexicon_tags_7 },
    { 0x2b5845e0, SUC_lexicon_tags_2 },
    { 0x395845e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x359845e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1845eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69845ef, SUC_lexicon_tags_6 },
    { 0x58b845f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd845f4, SUC_lexicon_tags_12 },
    { 0x90d845f5, SUC_lexicon_tags_25 },
    { 0xd8f845f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x829845f9, SUC_lexicon_tags_14 },
    { 0x4af845fa, SUC_lexicon_tags_8 },
    { 0x50f845fb, SUC_lexicon_tags_11 },
    { 0x3b3845fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5845ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d84606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af8460b, SUC_lexicon_tags_2 },
    { 0x6d98460c, SUC_lexicon_tags_2 },
    { 0x8898460d, SUC_lexicon_tags_47 },
    { 0xd698460d, SUC_lexicon_tags_11 },
    { 0xd5d8460b, SUC_lexicon_tags_84 },
    { 0xeb8460e, SUC_lexicon_tags_47 },
    { 0x8ef84611, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63584616, SUC_lexicon_tags_17 },
    { 0xaa584617, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f84620, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc584622, SUC_lexicon_tags_1 },
    { 0x6e584623, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b84626, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe4184628, SUC_lexicon_tags_4 },
    { 0x3b984629, SUC_lexicon_tags_6 },
    { 0xd558462a, SUC_lexicon_tags_47 },
    { 0xb378462a, SUC_lexicon_tags_37 },
    { 0x2578462c, SUC_lexicon_tags_7 },
    { 0x3fb8462d, SUC_lexicon_tags_28 },
    { 0xcb18462e, SUC_lexicon_tags_35 },
    { 0x8d8462f, SUC_lexicon_tags_25 },
    { 0x59384630, SUC_lexicon_tags_919 },
    { 0xf9584630, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0x10d84633, SUC_lexicon_tags_25 },
    { 0xd4b84633, SUC_lexicon_tags_37 },
    { 0x51184635, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x984637, SUC_lexicon_tags_37 },
    { 0x48984637, SUC_lexicon_tags_25 },
    { 0x7384637, SUC_lexicon_tags_12 },
    { 0xbcd8463a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x38d8463c, SUC_lexicon_tags_60 },
    { 0x7e18463d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31984641, SUC_lexicon_tags_14 },
    { 0x7db84642, SUC_lexicon_tags_63 },
    { 0x3eb84642, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f84646, SUC_lexicon_tags_13 },
    { 0x93184647, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa298464b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca38464f, SUC_lexicon_tags_15 },
    { 0x4dd84650, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7f84652, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15384656, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd8465a, SUC_lexicon_tags_17 },
    { 0xcc98465b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1984660, SUC_lexicon_tags_0 },
    { 0x29784661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6784663, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0384666, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8078466b, SUC_lexicon_tags_69 },
    { 0xf638466c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc358466e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64f84670, SUC_lexicon_tags_14 },
    { 0x61784671, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34984676, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef8467d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4f8467f, SUC_lexicon_tags_47 },
    { 0xc8b84680, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27184684, SUC_lexicon_tags_4 },
    { 0xebb84685, SUC_lexicon_tags_33 },
    { 0xd784686, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f8468e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5984690, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16584695, SUC_lexicon_tags_40 },
    { 0x5b184696, SUC_lexicon_tags_25 },
    { 0x70584697, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d8469a, SUC_lexicon_tags_31 },
    { 0x52b8469a, SUC_lexicon_tags_1 },
    { 0xe0d8469c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4318469e, SUC_lexicon_tags_63 },
    { 0x74f8469e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x383846a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5846a5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x46b846a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d846ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4b846ae, SUC_lexicon_tags_23 },
    { 0x473846af, SUC_lexicon_tags_25 },
    { 0x3a1846ae, SUC_lexicon_tags_11 },
    { 0xb61846b1, SUC_lexicon_tags_43 },
    { 0x5dd846b1, SUC_lexicon_tags_55 },
    { 0xe5b846b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81846b9, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0xc3b846bb, SUC_lexicon_tags_25 },
    { 0x40d846bb, SUC_lexicon_tags_14 },
    { 0xab9846bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7846c0, SUC_lexicon_tags_7 },
    { 0xd3846c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f846c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf47846ca, SUC_lexicon_tags_2 },
    { 0xbdb846ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807846d3, SUC_lexicon_tags_87 },
    { 0x771846d4, SUC_lexicon_tags_12 },
    { 0x6b7846d4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811846da, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db846e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f846e7, SUC_lexicon_tags_0 },
    { 0x9b5846e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x901846ea, SUC_lexicon_tags_4 },
    { 0xdb7846eb, SUC_lexicon_tags_11 },
    { 0xac9846eb, SUC_lexicon_tags_17 },
    { 0x7dd846ed, SUC_lexicon_tags_0 },
    { 0x9e7846ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf846f1, SUC_lexicon_tags_113 },
    { 0x3d9846f1, SUC_lexicon_tags_4 },
    { 0x2bf846f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d846f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x16f846f8, SUC_lexicon_tags_38 },
    { 0xc5d846f9, SUC_lexicon_tags_0 },
    { 0x85846f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1846fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb15846ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad84701, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa4984703, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b84706, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2384708, SUC_lexicon_tags_19 },
    { 0x1e384709, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xeb8470b, SUC_lexicon_tags_38 },
    { 0xec38470b, SUC_lexicon_tags_29 },
    { 0x3258470d, SUC_lexicon_tags_32 },
    { 0x9078470d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb84711, SUC_lexicon_tags_84 },
    { 0x8eb84711, SUC_lexicon_tags_15 },
    { 0x5fd84711, SUC_lexicon_tags_12 },
    { 0x81184714, SUC_lexicon_tags_14 },
    { 0x5f84715, SUC_lexicon_tags_11 },
    { 0xa9584716, SUC_lexicon_tags_11 },
    { 0x37584717, SUC_lexicon_tags_37 },
    { 0x20184718, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3918471a, SUC_lexicon_tags_4 },
    { 0x5798471b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f8471f, SUC_lexicon_tags_9 },
    { 0x5e584720, SUC_lexicon_tags_11 },
    { 0x37184721, SUC_lexicon_tags_91 },
    { 0xe9f84722, SUC_lexicon_tags_2 },
    { 0xfcd84723, SUC_lexicon_tags_17 },
    { 0xe3f84724, SUC_lexicon_tags_7 },
    { 0xa5584724, SUC_lexicon_tags_47 },
    { 0xda984720, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef8472c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d8472f, SUC_lexicon_tags_62 },
    { 0xb9784730, SUC_lexicon_tags_63 },
    { 0x94784730, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb384737, SUC_lexicon_tags_37 },
    { 0x22f84738, SUC_lexicon_tags_39 },
    { 0x21984737, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c78473c, SUC_lexicon_tags_27 },
    { 0x9b98473d, SUC_lexicon_tags_25 },
    { 0x6cb8473e, SUC_lexicon_tags_17 },
    { 0xf118473f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7984741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d984743, SUC_lexicon_tags_2 },
    { 0x9f84744, SUC_lexicon_tags_48 },
    { 0xf1b84745, SUC_lexicon_tags_27 },
    { 0xeab84744, SUC_lexicon_tags_1 },
    { 0xe0984747, SUC_lexicon_tags_37 },
    { 0x41184747, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9578474e, SUC_lexicon_tags_25 },
    { 0x9738474f, SUC_lexicon_tags_2 },
    { 0xddd84750, SUC_lexicon_tags_8 },
    { 0xfaf8474e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3384754, SUC_lexicon_tags_15 },
    { 0x2af84755, SUC_lexicon_tags_301 },
    { 0x1fd84756, SUC_lexicon_tags_10 },
    { 0x13b84757, SUC_lexicon_tags_48 },
    { 0xf7984758, SUC_lexicon_tags_15 },
    { 0xfa384759, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d38475e, SUC_lexicon_tags_11 },
    { 0xacb8475f, SUC_lexicon_tags_14 },
    { 0xfab84760, SUC_lexicon_tags_28 },
    { 0xe938475f, SUC_lexicon_tags_37 },
    { 0x2c784761, SUC_lexicon_tags_15 },
    { 0xd1984763, SUC_lexicon_tags_54 },
    { 0x57984764, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57384769, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b8476c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd8476f, SUC_lexicon_tags_14 },
    { 0xdfb84770, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d84773, SUC_lexicon_tags_6 },
    { 0x81d84774, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85184777, SUC_lexicon_tags_37 },
    { 0xe0184777, SUC_lexicon_tags_50 },
    { 0xdd984777, SUC_lexicon_tags_49 },
    { 0x52b8477a, SUC_lexicon_tags_37 },
    { 0x78d84778, SUC_lexicon_tags_37 },
    { 0x44f8477c, SUC_lexicon_tags_40 },
    { 0x6ef8477c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8984784, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xded84786, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd918478b, SUC_lexicon_tags_12 },
    { 0xe2f8478c, SUC_lexicon_tags_38 },
    { 0x20b8478c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d84793, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1f84795, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77984798, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x778479e, SUC_lexicon_tags_23 },
    { 0x3658479f, SUC_lexicon_tags_2 },
    { 0xde3847a0, SUC_lexicon_tags_15 },
    { 0x715847a0, SUC_lexicon_tags_55 },
    { 0x661847a2, SUC_lexicon_tags_61 },
    { 0x285847a2, SUC_lexicon_tags_40 },
    { 0xe73847a2, SUC_lexicon_tags_0 },
    { 0x273847a3, SUC_lexicon_tags_23 },
    { 0x52d8479e, SUC_lexicon_tags_46 },
    { 0x225847a5, SUC_lexicon_tags_7 },
    { 0xf1f847a8, SUC_lexicon_tags_37 },
    { 0xaa9847a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9847ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d847b1, SUC_lexicon_tags_25 },
    { 0x4ad847b2, SUC_lexicon_tags_12 },
    { 0x83d847b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a9847b5, SUC_lexicon_tags_13 },
    { 0xd47847b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797847bc, SUC_lexicon_tags_25 },
    { 0x2d3847bd, SUC_lexicon_tags_25 },
    { 0x66f847be, SUC_lexicon_tags_30 },
    { 0xbaf847bd, SUC_lexicon_tags_49 },
    { 0x81d847c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499847c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x109847cb, SUC_lexicon_tags_43 },
    { 0x40f847cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1847d6, SUC_lexicon_tags_11 },
    { 0xaa7847d6, SUC_lexicon_tags_2 },
    { 0xbb3847d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f847db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475847e0, SUC_lexicon_tags_7 },
    { 0xd37847e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d847e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87847e8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6bb847ea, SUC_lexicon_tags_47 },
    { 0xc4b847ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d847f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30b847f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99847f9, SUC_lexicon_tags_19 },
    { 0x677847f9, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5847fd, SUC_lexicon_tags_14 },
    { 0xa99847fe, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3584802, SUC_lexicon_tags_49 },
    { 0xfed84803, SUC_lexicon_tags_37 },
    { 0x88384803, SUC_lexicon_tags_19 },
    { 0x1df84805, SUC_lexicon_tags_6 },
    { 0x16384802, SUC_lexicon_tags_56 },
    { 0x1ab84802, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e58480b, SUC_lexicon_tags_29 },
    { 0xc518480c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6c58480e, SUC_lexicon_tags_414 },
    { 0x6658480e, SUC_lexicon_tags_1 },
    { 0xdb784810, SUC_lexicon_tags_43 },
    { 0x64f84810, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb84814, SUC_lexicon_tags_43 },
    { 0x35384814, SUC_lexicon_tags_62 },
    { 0xbc384814, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f84824, SUC_lexicon_tags_37 },
    { 0x56984824, SUC_lexicon_tags_6 },
    { 0xbef84826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3784828, SUC_lexicon_tags_61 },
    { 0x64984829, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4818482c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad84831, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93584833, SUC_lexicon_tags_12 },
    { 0x86384834, SUC_lexicon_tags_25 },
    { 0x85784833, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8184837, SUC_lexicon_tags_11 },
    { 0x14384837, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8483c, SUC_lexicon_tags_43 },
    { 0xea78483d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b84841, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x55384844, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f84847, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5078484a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb1b8484c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab18485b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6384862, SUC_lexicon_tags_26 },
    { 0xcd384863, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3b184865, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x89584867, SUC_lexicon_tags_1 },
    { 0x61184868, SUC_lexicon_tags_1 },
    { 0x7b184869, SUC_lexicon_tags_47 },
    { 0x9398486a, SUC_lexicon_tags_37 },
    { 0x47b8486a, SUC_lexicon_tags_19 },
    { 0x2638486b, SUC_lexicon_tags_12 },
    { 0x4838486b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2584876, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f184878, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb378487a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c18487c, SUC_lexicon_tags_23 },
    { 0x2978487d, SUC_lexicon_tags_7 },
    { 0xf8d8487e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xae384880, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbcd84882, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f84888, SUC_lexicon_tags_15 },
    { 0x87984888, SUC_lexicon_tags_17 },
    { 0x83584888, SUC_lexicon_tags_46 },
    { 0x78184889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x258488d, SUC_lexicon_tags_37 },
    { 0x7038488d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d84893, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3384898, SUC_lexicon_tags_38 },
    { 0xfc584899, SUC_lexicon_tags_63 },
    { 0xa2584898, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e98489c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1848a1, SUC_lexicon_tags_12 },
    { 0xc9848a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27848a4, SUC_lexicon_tags_1 },
    { 0xe89848a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3848ad, SUC_lexicon_tags_33 },
    { 0x44b848ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb67848b0, SUC_lexicon_tags_25 },
    { 0x32b848b1, SUC_lexicon_tags_2 },
    { 0xd8d848b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7848b5, SUC_lexicon_tags_17 },
    { 0x281848b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc1848b8, SUC_lexicon_tags_172 },
    { 0x737848b9, SUC_lexicon_tags_8 },
    { 0xceb848ba, SUC_lexicon_tags_12 },
    { 0x5b3848bb, SUC_lexicon_tags_45 },
    { 0xf61848bb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3bb848be, SUC_lexicon_tags_12 },
    { 0x1e9848bf, SUC_lexicon_tags_15 },
    { 0x417848c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3848c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99848ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5848cd, SUC_lexicon_tags_17 },
    { 0x1e3848cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b848d1, SUC_lexicon_tags_107 },
    { 0x24f848d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x477848d4, SUC_lexicon_tags_37 },
    { 0x545848d5, SUC_lexicon_tags_15 },
    { 0x63b848d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601848da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf79848dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb9848df, SUC_lexicon_tags_31 },
    { 0xc09848df, SUC_lexicon_tags_31 },
    { 0x979848df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595848eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5848ee, SUC_lexicon_tags_37 },
    { 0x979848ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x17b848f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x91848f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x81d848f6, SUC_lexicon_tags_27 },
    { 0x86b848f7, SUC_lexicon_tags_17 },
    { 0x7e3848f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d848fb, SUC_lexicon_tags_6 },
    { 0x481848fc, SUC_lexicon_tags_262 },
    { 0x749848fc, SUC_lexicon_tags_8 },
    { 0x9d7848fe, SUC_lexicon_tags_15 },
    { 0x2b7848ff, SUC_lexicon_tags_12 },
    { 0xb8f848fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf184902, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd184906, SUC_lexicon_tags_34 },
    { 0xcff84906, SUC_lexicon_tags_15 },
    { 0x3d384906, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b84916, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe584919, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x93f8491b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b84922, SUC_lexicon_tags_36 },
    { 0xdb584923, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x74184926, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d8492b, SUC_lexicon_tags_2 },
    { 0x4698492c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1218492e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcad84930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63184934, SUC_lexicon_tags_6 },
    { 0xf8784934, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf8493a, SUC_lexicon_tags_8 },
    { 0xf4d8493a, SUC_lexicon_tags_60 },
    { 0x3618493c, SUC_lexicon_tags_25 },
    { 0x8878493d, SUC_lexicon_tags_63 },
    { 0x3758493d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66384944, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e984946, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1598494d, SUC_lexicon_tags_48 },
    { 0x798494d, SUC_lexicon_tags_14 },
    { 0x2ff8494d, SUC_lexicon_tags_33 },
    { 0xf298494d, SUC_lexicon_tags_59 },
    { 0x30b8494d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8184955, SUC_lexicon_tags_62 },
    { 0xd4d84956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3718495c, SUC_lexicon_tags_6 },
    { 0x79b8495c, SUC_lexicon_tags_46 },
    { 0x3d98495e, SUC_lexicon_tags_32 },
    { 0x3a78495e, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b184963, SUC_lexicon_tags_33 },
    { 0xdf984964, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2bd84966, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b784968, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0384970, SUC_lexicon_tags_12 },
    { 0x3584971, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf784973, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5584976, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7658497e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xd5384980, SUC_lexicon_tags_15 },
    { 0xc9184981, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53584986, SUC_lexicon_tags_6 },
    { 0x91384987, SUC_lexicon_tags_91 },
    { 0x94984988, SUC_lexicon_tags_27 },
    { 0xdf784989, SUC_lexicon_tags_37 },
    { 0xdbf8498a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8178498e, SUC_lexicon_tags_2 },
    { 0xf6f8498e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda584992, SUC_lexicon_tags_26 },
    { 0xdc984993, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f384999, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9678499d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x378499f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb849a3, SUC_lexicon_tags_2 },
    { 0xb43849a4, SUC_lexicon_tags_2 },
    { 0x7a1849a5, SUC_lexicon_tags_11 },
    { 0xb77849a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdbb849a8, SUC_lexicon_tags_31 },
    { 0x751849a9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8ef849ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d849af, SUC_lexicon_tags_47 },
    { 0x945849b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x451849b2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b849b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x325849ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15849be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x115849c0, SUC_lexicon_tags_19 },
    { 0x47d849c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4d849c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4db849c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x219849c9, SUC_lexicon_tags_17 },
    { 0x96f849ca, SUC_lexicon_tags_49 },
    { 0x32f849cb, SUC_lexicon_tags_6 },
    { 0x373849cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd81849ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81849d5, SUC_lexicon_tags_17 },
    { 0xda7849d6, SUC_lexicon_tags_7 },
    { 0xc87849d6, SUC_lexicon_tags_25 },
    { 0xf65849d7, SUC_lexicon_tags_37 },
    { 0xc6d849d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901849dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f849e4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfbf849e6, SUC_lexicon_tags_16 },
    { 0xe15849e6, SUC_lexicon_tags_0 },
    { 0xe45849e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaf3849ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1849f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f849f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f849f9, SUC_lexicon_tags_17 },
    { 0xa45849fa, SUC_lexicon_tags_37 },
    { 0x1c5849fa, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8184a09, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70984a0d, SUC_lexicon_tags_12 },
    { 0xf9d84a0e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9184a14, SUC_lexicon_tags_37 },
    { 0x7a184a14, SUC_lexicon_tags_14 },
    { 0x84b84a16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a584a1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f84a1d, SUC_lexicon_tags_17 },
    { 0xae384a1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66784a21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f84a25, SUC_lexicon_tags_62 },
    { 0x27584a26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21184a2c, SUC_lexicon_tags_28 },
    { 0x93f84a2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d584a31, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xb2784a33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d784a36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb384a3d, SUC_lexicon_tags_1 },
    { 0x62b84a3e, SUC_lexicon_tags_4 },
    { 0x4fd84a3f, SUC_lexicon_tags_37 },
    { 0x69d84a40, SUC_lexicon_tags_12 },
    { 0x73984a40, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4c184a43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70984a46, SUC_lexicon_tags_33 },
    { 0x66f84a46, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x11d84a49, SUC_lexicon_tags_61 },
    { 0x7b584a49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94584a4c, SUC_lexicon_tags_12 },
    { 0x83784a4d, SUC_lexicon_tags_0 },
    { 0xd1584a4e, SUC_lexicon_tags_27 },
    { 0x8af84a4d, SUC_lexicon_tags_12 },
    { 0xae984a50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3f184a52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe184a57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6984a5e, SUC_lexicon_tags_14 },
    { 0xfdd84a5f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x13d84a61, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f84a66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb384a68, SUC_lexicon_tags_15 },
    { 0xf8f84a68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea584a71, SUC_lexicon_tags_28 },
    { 0xe0984a72, SUC_lexicon_tags_103 },
    { 0x8dd84a73, SUC_lexicon_tags_15 },
    { 0x88f84a74, SUC_lexicon_tags_7 },
    { 0x25d84a75, SUC_lexicon_tags_26 },
    { 0x2ad84a74, SUC_lexicon_tags_6 },
    { 0x25d84a77, SUC_lexicon_tags_65 },
    { 0x36784a78, SUC_lexicon_tags_65 },
    { 0x62984a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84184a7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ab84a7f, SUC_lexicon_tags_28 },
    { 0xeab84a7f, SUC_lexicon_tags_7 },
    { 0x29784a81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f84a84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b84a88, SUC_lexicon_tags_37 },
    { 0x17d84a89, SUC_lexicon_tags_7 },
    { 0x4ad84a8a, SUC_lexicon_tags_19 },
    { 0xd7984a89, SUC_lexicon_tags_11 },
    { 0xbaf84a89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e384a94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8584a96, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec984a9a, SUC_lexicon_tags_9 },
    { 0x89f84a9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f84aa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d84aa8, SUC_lexicon_tags_25 },
    { 0xae784aa8, SUC_lexicon_tags_24 },
    { 0x9984aa9, SUC_lexicon_tags_33 },
    { 0xa8584aab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1984aaf, SUC_lexicon_tags_37 },
    { 0x83f84ab0, SUC_lexicon_tags_25 },
    { 0xab184aaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f584ab3, SUC_lexicon_tags_98 },
    { 0x3b184ab4, SUC_lexicon_tags_31 },
    { 0xfdf84ab5, SUC_lexicon_tags_85 },
    { 0x784ab5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb84ab9, SUC_lexicon_tags_15 },
    { 0xd584ab9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf584abc, SUC_lexicon_tags_7 },
    { 0x33984abd, SUC_lexicon_tags_29 },
    { 0x59184abc, SUC_lexicon_tags_12 },
    { 0x63984abd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18584ac1, SUC_lexicon_tags_37 },
    { 0x80984ac2, SUC_lexicon_tags_25 },
    { 0xfd984ac3, SUC_lexicon_tags_138 },
    { 0x53f84ac4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d784ac6, SUC_lexicon_tags_12 },
    { 0xebf84ac7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b84aca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd84ace, SUC_lexicon_tags_17 },
    { 0x8df84acf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8dd84ad1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2b84ad3, SUC_lexicon_tags_35 },
    { 0x15d84ad3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb84ad7, SUC_lexicon_tags_30 },
    { 0xc2784ad7, SUC_lexicon_tags_4 },
    { 0x42b84ad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b84add, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x66384adf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b84ae2, SUC_lexicon_tags_0 },
    { 0xd6384ae3, SUC_lexicon_tags_31 },
    { 0xdd584ae4, SUC_lexicon_tags_31 },
    { 0xdb784ae4, SUC_lexicon_tags_66 },
    { 0xd5f84ae6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6984aec, SUC_lexicon_tags_49 },
    { 0xec584aed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b84af0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf984af3, SUC_lexicon_tags_1 },
    { 0xbdb84af4, SUC_lexicon_tags_11 },
    { 0x8b584af5, SUC_lexicon_tags_17 },
    { 0xb2b84af5, SUC_lexicon_tags_577 },
    { 0x58384af7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db84afa, SUC_lexicon_tags_15 },
    { 0x46d84afb, SUC_lexicon_tags_17 },
    { 0x65f84afa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d84b00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5184b06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x70d84b08, SUC_lexicon_tags_2 },
    { 0x35d84b09, SUC_lexicon_tags_68 },
    { 0xde784b0a, SUC_lexicon_tags_43 },
    { 0x8184b0b, SUC_lexicon_tags_12 },
    { 0x26384b0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b84b11, SUC_lexicon_tags_29 },
    { 0xef384b11, SUC_lexicon_tags_47 },
    { 0x33f84b13, SUC_lexicon_tags_23 },
    { 0x72b84b11, SUC_lexicon_tags_2 },
    { 0x4cd84b15, SUC_lexicon_tags_25 },
    { 0x8d584b16, SUC_lexicon_tags_63 },
    { 0xc5b84b17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bd84b19, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f84b1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb184b1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b384b21, SUC_lexicon_tags_15 },
    { 0xbf184b21, SUC_lexicon_tags_35 },
    { 0x3c384b22, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfb184b25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6184b29, SUC_lexicon_tags_27 },
    { 0xf8984b2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b84b2e, SUC_lexicon_tags_17 },
    { 0x5b184b2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b84b32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d84b37, SUC_lexicon_tags_15 },
    { 0xc2584b37, SUC_lexicon_tags_1 },
    { 0x7c584b37, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x68b84b3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc784b40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda784b43, SUC_lexicon_tags_77 },
    { 0x6ab84b43, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4bf84b46, SUC_lexicon_tags_37 },
    { 0xb84b47, SUC_lexicon_tags_45 },
    { 0x33984b48, SUC_lexicon_tags_28 },
    { 0x5d584b49, SUC_lexicon_tags_23 },
    { 0xe9384b49, SUC_lexicon_tags_12 },
    { 0x7d984b49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad84b4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d84b54, SUC_lexicon_tags_15 },
    { 0xa584b55, SUC_lexicon_tags_55 },
    { 0xb1384b56, SUC_lexicon_tags_2 },
    { 0x7c784b57, SUC_lexicon_tags_27 },
    { 0x74b84b56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4b84b5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1584b64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53f84b66, SUC_lexicon_tags_25 },
    { 0x9384b67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a184b6a, SUC_lexicon_tags_47 },
    { 0xdc184b6b, SUC_lexicon_tags_25 },
    { 0xd9f84b6c, SUC_lexicon_tags_33 },
    { 0x97384b6b, SUC_lexicon_tags_6 },
    { 0x72d84b6e, SUC_lexicon_tags_87 },
    { 0xf2384b6f, SUC_lexicon_tags_15 },
    { 0xb7384b6c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6184b72, SUC_lexicon_tags_34 },
    { 0x1a184b72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70784b76, SUC_lexicon_tags_17 },
    { 0x76b84b76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28784b7b, SUC_lexicon_tags_12 },
    { 0x94f84b7b, SUC_lexicon_tags_17 },
    { 0xa1f84b7d, SUC_lexicon_tags_17 },
    { 0x60984b7e, SUC_lexicon_tags_38 },
    { 0xbdd84b7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b84b84, SUC_lexicon_tags_9 },
    { 0x7fd84b85, SUC_lexicon_tags_17 },
    { 0x2d384b85, SUC_lexicon_tags_179 },
    { 0xe6184b87, SUC_lexicon_tags_1 },
    { 0xf9b84b84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f84b8d, SUC_lexicon_tags_15 },
    { 0xaed84b8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x46584b91, SUC_lexicon_tags_2 },
    { 0xe5b84b92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad84b9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8b84b9c, SUC_lexicon_tags_3 },
    { 0xee784b9d, SUC_lexicon_tags_8 },
    { 0x67784b9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8784ba1, SUC_lexicon_tags_37 },
    { 0x6df84ba1, SUC_lexicon_tags_37 },
    { 0xbf784ba3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63984ba5, SUC_lexicon_tags_27 },
    { 0xcb84ba6, SUC_lexicon_tags_25 },
    { 0x80384ba6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9784ba9, SUC_lexicon_tags_1 },
    { 0xabd84baa, SUC_lexicon_tags_60 },
    { 0x8eb84bab, SUC_lexicon_tags_12 },
    { 0xac384bac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df84baf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b84bb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d84bb7, SUC_lexicon_tags_0 },
    { 0x7cf84bb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6784bbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8184bbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb84bc4, SUC_lexicon_tags_17 },
    { 0xaf384bc5, SUC_lexicon_tags_15 },
    { 0x8a784bc5, SUC_lexicon_tags_104 },
    { 0xf0984bc7, SUC_lexicon_tags_17 },
    { 0x99384bc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90184bcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d84bd1, SUC_lexicon_tags_15 },
    { 0x68f84bd2, SUC_lexicon_tags_7 },
    { 0x39f84bd3, SUC_lexicon_tags_25 },
    { 0x15d84bd1, SUC_lexicon_tags_87 },
    { 0xd8984bd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f84bdb, SUC_lexicon_tags_10 },
    { 0x52984bdb, SUC_lexicon_tags_37 },
    { 0xfd784bdd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x30384be0, SUC_lexicon_tags_38 },
    { 0x86784be1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7fb84be3, SUC_lexicon_tags_33 },
    { 0x32f84be4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34984be8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf84bec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d84bef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x15f84bf2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef184bf8, SUC_lexicon_tags_15 },
    { 0x5584bf9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54984bfd, SUC_lexicon_tags_45 },
    { 0x4a784bfe, SUC_lexicon_tags_31 },
    { 0xfcb84bfe, SUC_lexicon_tags_2 },
    { 0x50d84c00, SUC_lexicon_tags_11 },
    { 0x1b384c01, SUC_lexicon_tags_10 },
    { 0x91184bfe, SUC_lexicon_tags_45 },
    { 0xf3184bff, SUC_lexicon_tags_2 },
    { 0x40f84c04, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12784c09, SUC_lexicon_tags_28 },
    { 0x96984c0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a384c0d, SUC_lexicon_tags_15 },
    { 0x22b84c0e, SUC_lexicon_tags_33 },
    { 0x72184c0f, SUC_lexicon_tags_38 },
    { 0x48b84c10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1384c15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x33584c17, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb84c1a, SUC_lexicon_tags_65 },
    { 0x36f84c1a, SUC_lexicon_tags_43 },
    { 0x68984c1b, SUC_lexicon_tags_0 },
    { 0x58384c1d, SUC_lexicon_tags_30 },
    { 0xa6984c1e, SUC_lexicon_tags_82 },
    { 0xb7384c1e, SUC_lexicon_tags_37 },
    { 0x5b184c1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7b84c22, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xccd84c24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7184c27, SUC_lexicon_tags_47 },
    { 0xf6b84c28, SUC_lexicon_tags_12 },
    { 0x7984c27, SUC_lexicon_tags_15 },
    { 0x18984c2a, SUC_lexicon_tags_11 },
    { 0x6fb84c2b, SUC_lexicon_tags_30 },
    { 0x5b784c2c, SUC_lexicon_tags_17 },
    { 0x67984c2c, SUC_lexicon_tags_45 },
    { 0x30b84c2c, SUC_lexicon_tags_12 },
    { 0xc5584c2e, SUC_lexicon_tags_17 },
    { 0x3af84c2d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d784c35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3384c37, SUC_lexicon_tags_14 },
    { 0x4f384c38, SUC_lexicon_tags_37 },
    { 0xaa584c39, SUC_lexicon_tags_37 },
    { 0xd8b84c3a, SUC_lexicon_tags_9 },
    { 0xf4784c3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b84c40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1384c42, SUC_lexicon_tags_44 },
    { 0x3a984c42, SUC_lexicon_tags_45 },
    { 0x74b84c43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15d84c46, SUC_lexicon_tags_25 },
    { 0x5e984c47, SUC_lexicon_tags_35 },
    { 0x8b84c48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb84c4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d84c4e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe4b84c50, SUC_lexicon_tags_17 },
    { 0xef384c51, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b84c54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f84c57, SUC_lexicon_tags_49 },
    { 0x93584c58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38b84c5a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x92184c5c, SUC_lexicon_tags_48 },
    { 0x19f84c5d, SUC_lexicon_tags_15 },
    { 0xf3d84c5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x96184c61, SUC_lexicon_tags_17 },
    { 0xd6b84c62, SUC_lexicon_tags_19 },
    { 0x3b84c62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6c184c65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd84c6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70784c6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20b84c6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18384c72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd84c79, SUC_lexicon_tags_45 },
    { 0xd6384c7a, SUC_lexicon_tags_30 },
    { 0x81184c7b, SUC_lexicon_tags_25 },
    { 0xd0384c7c, SUC_lexicon_tags_17 },
    { 0x61384c7b, SUC_lexicon_tags_0 },
    { 0x1c184c7a, SUC_lexicon_tags_49 },
    { 0x92f84c7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda984c81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f784c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed84c89, SUC_lexicon_tags_11 },
    { 0x36f84c8a, SUC_lexicon_tags_17 },
    { 0x66b84c8b, SUC_lexicon_tags_17 },
    { 0x95184c8c, SUC_lexicon_tags_12 },
    { 0xa7384c89, SUC_lexicon_tags_31 },
    { 0xfdb84c8b, SUC_lexicon_tags_29 },
    { 0xff584c8f, SUC_lexicon_tags_55 },
    { 0x4f384c8f, SUC_lexicon_tags_47 },
    { 0x70b84c8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53984c95, SUC_lexicon_tags_15 },
    { 0x87b84c95, SUC_lexicon_tags_27 },
    { 0xb384c97, SUC_lexicon_tags_7 },
    { 0x30d84c98, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc7f84c9a, SUC_lexicon_tags_33 },
    { 0x40584c9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d84c9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d84ca2, SUC_lexicon_tags_15 },
    { 0xd0d84ca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3384ca6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb184ca8, SUC_lexicon_tags_15 },
    { 0x10984ca9, SUC_lexicon_tags_10 },
    { 0x29d84ca9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcf184cac, SUC_lexicon_tags_10 },
    { 0x72b84cad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb784caf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68584cb3, SUC_lexicon_tags_75 },
    { 0x7c584cb4, SUC_lexicon_tags_64 },
    { 0x47784cb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f784cb9, SUC_lexicon_tags_19 },
    { 0x11984cb9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74584cbe, SUC_lexicon_tags_55 },
    { 0xd5184cbf, SUC_lexicon_tags_33 },
    { 0x7784cc0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db84cc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75984cc9, SUC_lexicon_tags_48 },
    { 0x2b784cc9, SUC_lexicon_tags_28 },
    { 0x9d984ccb, SUC_lexicon_tags_18 },
    { 0x8e584ccc, SUC_lexicon_tags_47 },
    { 0x46784ccd, SUC_lexicon_tags_25 },
    { 0xac784cce, SUC_lexicon_tags_37 },
    { 0x79584ccf, SUC_lexicon_tags_45 },
    { 0xcf84cc9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f84cd7, SUC_lexicon_tags_12 },
    { 0xfad84cd8, SUC_lexicon_tags_15 },
    { 0x7f784cd7, SUC_lexicon_tags_12 },
    { 0x5a384cda, SUC_lexicon_tags_26 },
    { 0xd1384cdb, SUC_lexicon_tags_106 },
    { 0x83f84cdb, SUC_lexicon_tags_47 },
    { 0x5f184cd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6184ce3, SUC_lexicon_tags_10 },
    { 0x57984ce3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a784ce7, SUC_lexicon_tags_28 },
    { 0x3fb84ce7, SUC_lexicon_tags_49 },
    { 0x3584ce9, SUC_lexicon_tags_25 },
    { 0x1c984ce7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfd584cec, SUC_lexicon_tags_65 },
    { 0x5df84cec, SUC_lexicon_tags_17 },
    { 0xc6d84cee, SUC_lexicon_tags_17 },
    { 0xd1784cef, SUC_lexicon_tags_46 },
    { 0xb9984cf0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f84cf3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b84cfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34584d02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35184d09, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcdf84d0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb184d0d, SUC_lexicon_tags_15 },
    { 0x60584d0e, SUC_lexicon_tags_235 },
    { 0xc0184d0f, SUC_lexicon_tags_16 },
    { 0x27584d10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe0b84d12, SUC_lexicon_tags_44 },
    { 0xe5b84d13, SUC_lexicon_tags_2 },
    { 0xc9584d14, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7984d19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c384d1e, SUC_lexicon_tags_11 },
    { 0x1d784d1f, SUC_lexicon_tags_15 },
    { 0xa8584d20, SUC_lexicon_tags_19 },
    { 0x4d784d21, SUC_lexicon_tags_57 },
    { 0x59184d22, SUC_lexicon_tags_12 },
    { 0x10b84d23, SUC_lexicon_tags_45 },
    { 0x74584d22, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b84d29, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1f84d2b, SUC_lexicon_tags_6 },
    { 0xa4184d2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cf84d2e, SUC_lexicon_tags_99 },
    { 0x8b584d2e, SUC_lexicon_tags_0 },
    { 0xf5584d2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd84d32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c184d35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f84d38, SUC_lexicon_tags_11 },
    { 0x80984d39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xde784d3b, SUC_lexicon_tags_28 },
    { 0xc6384d3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x62584d3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1184d41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b84d47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa4b84d49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf784d4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9784d4e, SUC_lexicon_tags_0 },
    { 0xd7d84d4f, SUC_lexicon_tags_14 },
    { 0x3784d4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a984d5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b384d63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88184d67, SUC_lexicon_tags_31 },
    { 0x3b84d67, SUC_lexicon_tags_17 },
    { 0xe0b84d67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6f84d6b, SUC_lexicon_tags_14 },
    { 0x3a184d6c, SUC_lexicon_tags_12 },
    { 0x77784d6d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64984d79, SUC_lexicon_tags_15 },
    { 0xd6584d79, SUC_lexicon_tags_57 },
    { 0x6c384d7a, SUC_lexicon_tags_60 },
    { 0xda984d7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1584d82, SUC_lexicon_tags_55 },
    { 0xaf984d83, SUC_lexicon_tags_124 },
    { 0x5bf84d84, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b184d8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xac784d8e, SUC_lexicon_tags_45 },
    { 0xf9584d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51584d93, SUC_lexicon_tags_46 },
    { 0xa5984d94, SUC_lexicon_tags_4 },
    { 0x2c184d95, SUC_lexicon_tags_15 },
    { 0x81584d96, SUC_lexicon_tags_37 },
    { 0x3ff84d95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x45984d9a, SUC_lexicon_tags_38 },
    { 0xd784d9b, SUC_lexicon_tags_11 },
    { 0xd84d9c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6bf84d9e, SUC_lexicon_tags_49 },
    { 0xc4d84d9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x46784da2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95984da6, SUC_lexicon_tags_43 },
    { 0x83d84da6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69784dad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f84db0, SUC_lexicon_tags_37 },
    { 0x89f84db1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde384db6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb7784db8, SUC_lexicon_tags_14 },
    { 0xf2384db8, SUC_lexicon_tags_872 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf84dbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6784dc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53984dc3, SUC_lexicon_tags_49 },
    { 0x2d784dc3, SUC_lexicon_tags_21 },
    { 0x10584dc3, SUC_lexicon_tags_84 },
    { 0x33f84dc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62d84dc8, SUC_lexicon_tags_11 },
    { 0x98984dc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c384dce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fd84dd0, SUC_lexicon_tags_25 },
    { 0x2db84dd1, SUC_lexicon_tags_7 },
    { 0x25784dd2, SUC_lexicon_tags_23 },
    { 0x87784dd3, SUC_lexicon_tags_633 },
    { 0xdf984dd3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf784dd7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f384dda, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41184de0, SUC_lexicon_tags_7 },
    { 0x42b84de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f84de5, SUC_lexicon_tags_7 },
    { 0xaf984de6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74384df1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6384df4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd584e06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd4384e08, SUC_lexicon_tags_37 },
    { 0x24f84e09, SUC_lexicon_tags_17 },
    { 0xdf84e0a, SUC_lexicon_tags_12 },
    { 0x72984e0a, SUC_lexicon_tags_55 },
    { 0x92d84e0c, SUC_lexicon_tags_28 },
    { 0x80384e0d, SUC_lexicon_tags_37 },
    { 0x8184e0e, SUC_lexicon_tags_15 },
    { 0x47184e0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c784e15, SUC_lexicon_tags_17 },
    { 0xbf984e16, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53184e1c, SUC_lexicon_tags_15 },
    { 0x7e584e1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38584e1f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c184e24, SUC_lexicon_tags_26 },
    { 0x76784e25, SUC_lexicon_tags_17 },
    { 0xaa584e26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3184e2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78584e32, SUC_lexicon_tags_2 },
    { 0xe9b84e33, SUC_lexicon_tags_27 },
    { 0x72384e32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b84e3a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b84e3e, SUC_lexicon_tags_27 },
    { 0x25b84e3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac784e41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45784e44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcfb84e46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa984e51, SUC_lexicon_tags_17 },
    { 0x3b84e52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78784e54, SUC_lexicon_tags_7 },
    { 0xe7984e55, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80384e5d, SUC_lexicon_tags_4 },
    { 0xca384e5e, SUC_lexicon_tags_19 },
    { 0xefb84e5e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d584e64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdbb84e66, SUC_lexicon_tags_28 },
    { 0xb3b84e66, SUC_lexicon_tags_43 },
    { 0xaa384e68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f784e6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec384e6c, SUC_lexicon_tags_25 },
    { 0xf5584e6c, SUC_lexicon_tags_11 },
    { 0x76d84e6c, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce384e77, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3184e79, SUC_lexicon_tags_7 },
    { 0x71b84e7a, SUC_lexicon_tags_0 },
    { 0x12784e7b, SUC_lexicon_tags_77 },
    { 0x5d384e7b, SUC_lexicon_tags_17 },
    { 0x4784e7b, SUC_lexicon_tags_25 },
    { 0xd7b84e7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x83384e80, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f84e86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff84e89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a984e8c, SUC_lexicon_tags_49 },
    { 0xf0784e8d, SUC_lexicon_tags_23 },
    { 0xda184e8e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9584e91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf784e97, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7a984e99, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6984e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc984e9f, SUC_lexicon_tags_27 },
    { 0x44584ea0, SUC_lexicon_tags_3 },
    { 0xb4184ea0, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x4f584ea3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18184ea7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3984eab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7584eae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b84eb8, SUC_lexicon_tags_934 },
    { 0x8d984eb9, SUC_lexicon_tags_1 },
    { 0x11584eba, SUC_lexicon_tags_2 },
    { 0x41d84eb9, SUC_lexicon_tags_27 },
    { 0x2b784ebc, SUC_lexicon_tags_37 },
    { 0x6584ebd, SUC_lexicon_tags_11 },
    { 0x7984eb9, SUC_lexicon_tags_4 },
    { 0x44784ebf, SUC_lexicon_tags_28 },
    { 0x6bb84eb9, SUC_lexicon_tags_19 },
    { 0x59584ec1, SUC_lexicon_tags_14 },
    { 0x7fd84ec2, SUC_lexicon_tags_11 },
    { 0xfa384ec3, SUC_lexicon_tags_33 },
    { 0x8e384ec3, SUC_lexicon_tags_25 },
    { 0xd1f84ec0, SUC_lexicon_tags_43 },
    { 0xa3184ec6, SUC_lexicon_tags_1 },
    { 0x71984ec7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e384ec9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7784ecc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdb84ece, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c584ed2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2384ed9, SUC_lexicon_tags_17 },
    { 0xf9b84eda, SUC_lexicon_tags_7 },
    { 0xfd84edb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf84ee9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b84eee, SUC_lexicon_tags_4 },
    { 0x8ab84eef, SUC_lexicon_tags_2 },
    { 0x1584ef0, SUC_lexicon_tags_0 },
    { 0xea184eef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x43b84ef3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10784efc, SUC_lexicon_tags_25 },
    { 0x3bd84efc, SUC_lexicon_tags_113 },
    { 0x66b84efc, SUC_lexicon_tags_7 },
    { 0x4d184eff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf8b84f01, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4984f07, SUC_lexicon_tags_37 },
    { 0x3f384f07, SUC_lexicon_tags_101 },
    { 0xc3d84f08, SUC_lexicon_tags_2 },
    { 0x6cd84f0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65384f0e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff84f11, SUC_lexicon_tags_10 },
    { 0x51784f12, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6ef84f14, SUC_lexicon_tags_17 },
    { 0xec984f14, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x24384f18, SUC_lexicon_tags_12 },
    { 0xb8184f19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df84f1c, SUC_lexicon_tags_2 },
    { 0xb4f84f1d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x26384f20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8784f26, SUC_lexicon_tags_33 },
    { 0xd0f84f26, SUC_lexicon_tags_12 },
    { 0x50f84f28, SUC_lexicon_tags_11 },
    { 0xfb84f29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1d84f2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a984f2d, SUC_lexicon_tags_1 },
    { 0xd3384f2d, SUC_lexicon_tags_11 },
    { 0xfdf84f2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9184f33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e184f36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f84f3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb5d84f3c, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad84f3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe584f47, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x5ed84f49, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d984f4c, SUC_lexicon_tags_2 },
    { 0x36184f4d, SUC_lexicon_tags_25 },
    { 0x9784f4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9784f53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f84f56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0b84f58, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db84f5c, SUC_lexicon_tags_25 },
    { 0xeb384f5d, SUC_lexicon_tags_23 },
    { 0xbd784f5d, SUC_lexicon_tags_13 },
    { 0xd7b84f5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30984f62, SUC_lexicon_tags_28 },
    { 0xdcf84f62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ad84f65, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xae184f67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b84f69, SUC_lexicon_tags_0 },
    { 0xe3184f69, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed84f6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d84f6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b84f72, SUC_lexicon_tags_38 },
    { 0x74384f73, SUC_lexicon_tags_2 },
    { 0x14f84f72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a784f76, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x68f84f78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f84f7c, SUC_lexicon_tags_11 },
    { 0xfcf84f7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed84f80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x47384f82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd584f84, SUC_lexicon_tags_11 },
    { 0x34f84f85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae184f87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e184f89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85184f8b, SUC_lexicon_tags_9 },
    { 0x47f84f8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b84f91, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d84f94, SUC_lexicon_tags_15 },
    { 0xc9b84f94, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd84fa0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1584fa3, SUC_lexicon_tags_11 },
    { 0x55584fa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2984fa9, SUC_lexicon_tags_77 },
    { 0x8b384fa9, SUC_lexicon_tags_16 },
    { 0x42384fab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xed984fae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66984fb0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6784fb7, SUC_lexicon_tags_45 },
    { 0x16184fb7, SUC_lexicon_tags_8 },
    { 0x5a384fb9, SUC_lexicon_tags_14 },
    { 0xc9984fba, SUC_lexicon_tags_0 },
    { 0x38b84fbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f584fbd, SUC_lexicon_tags_12 },
    { 0x87f84fbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9d84fc0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff784fc4, SUC_lexicon_tags_8 },
    { 0xa9784fc5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x61384fc8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf184fcf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf84fd4, SUC_lexicon_tags_25 },
    { 0xedb84fd5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2af84fd7, SUC_lexicon_tags_1 },
    { 0x26584fd8, SUC_lexicon_tags_11 },
    { 0xe4384fd9, SUC_lexicon_tags_11 },
    { 0x9df84fd7, SUC_lexicon_tags_49 },
    { 0xf0b84fd9, SUC_lexicon_tags_3 },
    { 0x91784fdc, SUC_lexicon_tags_15 },
    { 0x3a384fdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe984fe1, SUC_lexicon_tags_7 },
    { 0xf1f84fe2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb8984fe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf84fea, SUC_lexicon_tags_25 },
    { 0x39584feb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28584ff0, SUC_lexicon_tags_14 },
    { 0xc7384ff0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f84ff5, SUC_lexicon_tags_14 },
    { 0x66d84ff6, SUC_lexicon_tags_7 },
    { 0x83b84ff6, SUC_lexicon_tags_37 },
    { 0xd5384ff7, SUC_lexicon_tags_43 },
    { 0x50d84ff9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2984fff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f85002, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd9385004, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf8500f, SUC_lexicon_tags_31 },
    { 0x3c985010, SUC_lexicon_tags_32 },
    { 0x3e185011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50b85013, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ed85015, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf85019, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef8501d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b85028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd8502b, SUC_lexicon_tags_27 },
    { 0xbdb8502b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17785030, SUC_lexicon_tags_65 },
    { 0x4b85031, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad85039, SUC_lexicon_tags_12 },
    { 0xf7b8503a, SUC_lexicon_tags_25 },
    { 0xa978503b, SUC_lexicon_tags_26 },
    { 0xd958503c, SUC_lexicon_tags_23 },
    { 0x19f85039, SUC_lexicon_tags_29 },
    { 0x7e78503e, SUC_lexicon_tags_12 },
    { 0xe2d8503f, SUC_lexicon_tags_11 },
    { 0x7c585039, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f85045, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb238504b, SUC_lexicon_tags_15 },
    { 0x1d8504c, SUC_lexicon_tags_37 },
    { 0x4938504c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87585050, SUC_lexicon_tags_27 },
    { 0x58385050, SUC_lexicon_tags_11 },
    { 0xb7f85051, SUC_lexicon_tags_60 },
    { 0xc9985053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92385055, SUC_lexicon_tags_27 },
    { 0xcc585056, SUC_lexicon_tags_25 },
    { 0x28d85057, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd0985059, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f585060, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x19585062, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71d85064, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21385068, SUC_lexicon_tags_129 },
    { 0x93d85068, SUC_lexicon_tags_32 },
    { 0x28385069, SUC_lexicon_tags_35 },
    { 0xb4f8506a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x23d8506d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad85072, SUC_lexicon_tags_87 },
    { 0xd8f85073, SUC_lexicon_tags_8 },
    { 0xe1b85074, SUC_lexicon_tags_7 },
    { 0xea185075, SUC_lexicon_tags_12 },
    { 0x3a585073, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x87185078, SUC_lexicon_tags_23 },
    { 0x12b85079, SUC_lexicon_tags_35 },
    { 0xebf8507a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5958507e, SUC_lexicon_tags_38 },
    { 0xcd58507f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26985081, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f785084, SUC_lexicon_tags_37 },
    { 0xc3385084, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7985087, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f8508a, SUC_lexicon_tags_37 },
    { 0x25f8508a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd85090, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f85093, SUC_lexicon_tags_56 },
    { 0x2dd85094, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d85099, SUC_lexicon_tags_11 },
    { 0x2a18509a, SUC_lexicon_tags_14 },
    { 0x5b18509a, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0xcef8509e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x137850a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad850a4, SUC_lexicon_tags_60 },
    { 0xecb850a4, SUC_lexicon_tags_33 },
    { 0xfa9850a6, SUC_lexicon_tags_9 },
    { 0xcd7850a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f850ae, SUC_lexicon_tags_37 },
    { 0x89f850af, SUC_lexicon_tags_5 },
    { 0x539850af, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85850b7, SUC_lexicon_tags_17 },
    { 0xb87850b8, SUC_lexicon_tags_25 },
    { 0xef850b9, SUC_lexicon_tags_39 },
    { 0xcfb850ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec1850bc, SUC_lexicon_tags_18 },
    { 0xbef850bc, SUC_lexicon_tags_37 },
    { 0x4ab850bc, SUC_lexicon_tags_80 },
    { 0xa91850bf, SUC_lexicon_tags_27 },
    { 0xd1850bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf850c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x499850c5, SUC_lexicon_tags_19 },
    { 0x615850c6, SUC_lexicon_tags_25 },
    { 0x2e3850c7, SUC_lexicon_tags_25 },
    { 0x4a3850c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x811850ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8c9850cc, SUC_lexicon_tags_15 },
    { 0xed5850cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b850d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaad850d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467850db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb63850dd, SUC_lexicon_tags_17 },
    { 0x70b850de, SUC_lexicon_tags_11 },
    { 0xdcb850df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91850e3, SUC_lexicon_tags_11 },
    { 0x85850e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef850e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc7850eb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27850ee, SUC_lexicon_tags_17 },
    { 0xe1b850ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627850f3, SUC_lexicon_tags_19 },
    { 0x1f5850f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x355850f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b850f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x361850fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7e5850fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd985103, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4185106, SUC_lexicon_tags_25 },
    { 0xc5d85106, SUC_lexicon_tags_25 },
    { 0xc5785106, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd158510d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96d8510f, SUC_lexicon_tags_23 },
    { 0xbc185110, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd185112, SUC_lexicon_tags_17 },
    { 0x18b85113, SUC_lexicon_tags_11 },
    { 0x70585114, SUC_lexicon_tags_141 },
    { 0x1b85114, SUC_lexicon_tags_37 },
    { 0x9e985114, SUC_lexicon_tags_25 },
    { 0xccf85117, SUC_lexicon_tags_14 },
    { 0x5df85118, SUC_lexicon_tags_48 },
    { 0xdf185119, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9558511b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb38511e, SUC_lexicon_tags_37 },
    { 0x4378511f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a985121, SUC_lexicon_tags_389 },
    { 0xadf85121, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92d85124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d85127, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34385129, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c38512c, SUC_lexicon_tags_37 },
    { 0x958512c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a185131, SUC_lexicon_tags_14 },
    { 0x7c585132, SUC_lexicon_tags_43 },
    { 0x5cd85132, SUC_lexicon_tags_25 },
    { 0x5d185134, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd8513b, SUC_lexicon_tags_43 },
    { 0x92d8513b, SUC_lexicon_tags_16 },
    { 0xa2b8513c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b185140, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0985145, SUC_lexicon_tags_23 },
    { 0xedf85146, SUC_lexicon_tags_37 },
    { 0x62185145, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x44185149, SUC_lexicon_tags_37 },
    { 0x8858514a, SUC_lexicon_tags_29 },
    { 0x2258514a, SUC_lexicon_tags_37 },
    { 0xfcb8514b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9385150, SUC_lexicon_tags_25 },
    { 0x51585150, SUC_lexicon_tags_23 },
    { 0xb3985152, SUC_lexicon_tags_63 },
    { 0x92b85153, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59985155, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd098515a, SUC_lexicon_tags_12 },
    { 0x5ed8515b, SUC_lexicon_tags_0 },
    { 0x4498515c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x5fb8515e, SUC_lexicon_tags_35 },
    { 0x13f8515f, SUC_lexicon_tags_17 },
    { 0x5f585160, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a185163, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad85167, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6385169, SUC_lexicon_tags_2 },
    { 0xc2f8516a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb8516f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d85178, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93d8517a, SUC_lexicon_tags_16 },
    { 0x13d8517a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1298517e, SUC_lexicon_tags_25 },
    { 0x5518517e, SUC_lexicon_tags_17 },
    { 0x82385180, SUC_lexicon_tags_2 },
    { 0x64985180, SUC_lexicon_tags_37 },
    { 0x88f85180, SUC_lexicon_tags_10 },
    { 0xe7b85183, SUC_lexicon_tags_25 },
    { 0xd3385183, SUC_lexicon_tags_207 },
    { 0x69585185, SUC_lexicon_tags_2 },
    { 0xc6b85182, SUC_lexicon_tags_11 },
    { 0x5bd85187, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d8518b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65185190, SUC_lexicon_tags_27 },
    { 0xc0d85191, SUC_lexicon_tags_34 },
    { 0x84f85191, SUC_lexicon_tags_1 },
    { 0xe6985193, SUC_lexicon_tags_2 },
    { 0x69585193, SUC_lexicon_tags_108 },
    { 0x98f85193, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xe6585197, SUC_lexicon_tags_27 },
    { 0x66385197, SUC_lexicon_tags_107 },
    { 0x36385197, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1851a0, SUC_lexicon_tags_14 },
    { 0x981851a0, SUC_lexicon_tags_33 },
    { 0x48d851a2, SUC_lexicon_tags_25 },
    { 0x123851a2, SUC_lexicon_tags_15 },
    { 0x901851a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5851a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd851aa, SUC_lexicon_tags_7 },
    { 0x2b851aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5851ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f851b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53851ba, SUC_lexicon_tags_6 },
    { 0x153851bb, SUC_lexicon_tags_30 },
    { 0x96b851bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753851c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387851c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x321851c8, SUC_lexicon_tags_60 },
    { 0x849851c9, SUC_lexicon_tags_2 },
    { 0x105851ca, SUC_lexicon_tags_63 },
    { 0x1ab851cb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f851d0, SUC_lexicon_tags_58 },
    { 0x8ad851d1, SUC_lexicon_tags_12 },
    { 0xccd851d2, SUC_lexicon_tags_7 },
    { 0xe6d851d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af851d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501851db, SUC_lexicon_tags_13 },
    { 0xbc5851db, SUC_lexicon_tags_12 },
    { 0x5ed851db, SUC_lexicon_tags_37 },
    { 0xd8f851de, SUC_lexicon_tags_17 },
    { 0x1a7851df, SUC_lexicon_tags_37 },
    { 0x6fd851e0, SUC_lexicon_tags_12 },
    { 0x221851e0, SUC_lexicon_tags_11 },
    { 0x533851e1, SUC_lexicon_tags_46 },
    { 0x1b5851e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7851e8, SUC_lexicon_tags_33 },
    { 0xa43851e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151851ee, SUC_lexicon_tags_37 },
    { 0xa11851ee, SUC_lexicon_tags_11 },
    { 0x97b851f0, SUC_lexicon_tags_66 },
    { 0x983851ee, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x647851f4, SUC_lexicon_tags_25 },
    { 0x849851f4, SUC_lexicon_tags_1 },
    { 0x339851f6, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765851fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab85201, SUC_lexicon_tags_17 },
    { 0xf0d85202, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b185207, SUC_lexicon_tags_27 },
    { 0x72785208, SUC_lexicon_tags_67 },
    { 0x15f85209, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf8d8520b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17985211, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6585213, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd985218, SUC_lexicon_tags_17 },
    { 0x84185218, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff85222, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a385224, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2785226, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d985230, SUC_lexicon_tags_11 },
    { 0x39385231, SUC_lexicon_tags_37 },
    { 0x4185232, SUC_lexicon_tags_37 },
    { 0x78585233, SUC_lexicon_tags_12 },
    { 0xc6985234, SUC_lexicon_tags_37 },
    { 0x78f85235, SUC_lexicon_tags_17 },
    { 0x9e185236, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4378523f, SUC_lexicon_tags_7 },
    { 0xc9785240, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b98524e, SUC_lexicon_tags_47 },
    { 0xf3d8524f, SUC_lexicon_tags_1 },
    { 0x91185250, SUC_lexicon_tags_61 },
    { 0xf4385250, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f85254, SUC_lexicon_tags_45 },
    { 0x70b85255, SUC_lexicon_tags_17 },
    { 0x69d85256, SUC_lexicon_tags_1 },
    { 0xa6785257, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8785259, SUC_lexicon_tags_25 },
    { 0xf8785259, SUC_lexicon_tags_27 },
    { 0x76f85259, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2458525d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa578525f, SUC_lexicon_tags_34 },
    { 0x7ef85260, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67785264, SUC_lexicon_tags_28 },
    { 0xd1785264, SUC_lexicon_tags_23 },
    { 0x46985266, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d985268, SUC_lexicon_tags_15 },
    { 0x61985268, SUC_lexicon_tags_10 },
    { 0x2d785269, SUC_lexicon_tags_9 },
    { 0x4b85268, SUC_lexicon_tags_12 },
    { 0x47b8526a, SUC_lexicon_tags_7 },
    { 0xc578526d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d85273, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x55785275, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33d85277, SUC_lexicon_tags_8 },
    { 0x15985278, SUC_lexicon_tags_3 },
    { 0xf1d85278, SUC_lexicon_tags_15 },
    { 0x9eb85277, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b85280, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185285, SUC_lexicon_tags_15 },
    { 0xf5b85286, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x41385288, SUC_lexicon_tags_1 },
    { 0xda385289, SUC_lexicon_tags_25 },
    { 0xae38528a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x10d8528c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b85290, SUC_lexicon_tags_45 },
    { 0x52985290, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d85293, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7585295, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60d85297, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b852a1, SUC_lexicon_tags_25 },
    { 0x39852a2, SUC_lexicon_tags_35 },
    { 0xfc5852a2, SUC_lexicon_tags_7 },
    { 0xc6d852a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15852a9, SUC_lexicon_tags_9 },
    { 0x6a1852aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3852b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5852b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85852b9, SUC_lexicon_tags_37 },
    { 0xe55852ba, SUC_lexicon_tags_11 },
    { 0x79852bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983852bf, SUC_lexicon_tags_7 },
    { 0x8d852bf, SUC_lexicon_tags_46 },
    { 0x5e7852bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359852c5, SUC_lexicon_tags_16 },
    { 0x113852c6, SUC_lexicon_tags_37 },
    { 0xf61852c6, SUC_lexicon_tags_25 },
    { 0x809852c7, SUC_lexicon_tags_19 },
    { 0xd9b852c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33852cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe0f852cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x773852d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e1852d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3852e1, SUC_lexicon_tags_12 },
    { 0xe87852e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3852e5, SUC_lexicon_tags_11 },
    { 0xdad852e6, SUC_lexicon_tags_48 },
    { 0x573852e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x129852e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcab852eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d85300, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1785302, SUC_lexicon_tags_45 },
    { 0x9c185303, SUC_lexicon_tags_37 },
    { 0x38f85302, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4785306, SUC_lexicon_tags_37 },
    { 0xfd785307, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa478530b, SUC_lexicon_tags_1 },
    { 0xc4f8530b, SUC_lexicon_tags_5 },
    { 0xe758530d, SUC_lexicon_tags_2 },
    { 0x2818530d, SUC_lexicon_tags_8 },
    { 0x19b8530f, SUC_lexicon_tags_2 },
    { 0x85f8530f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5db85312, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27785315, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57585319, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d85324, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x88b85326, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83585328, SUC_lexicon_tags_2 },
    { 0xed785329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb18532d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f18532f, SUC_lexicon_tags_14 },
    { 0x72185330, SUC_lexicon_tags_37 },
    { 0xe5385331, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded85338, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f38533a, SUC_lexicon_tags_7 },
    { 0x6758533b, SUC_lexicon_tags_1 },
    { 0x4918533c, SUC_lexicon_tags_15 },
    { 0x8018533d, SUC_lexicon_tags_55 },
    { 0x5598533b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd185341, SUC_lexicon_tags_28 },
    { 0x96585342, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8db85345, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x77785347, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d85349, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6018534b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db8534e, SUC_lexicon_tags_11 },
    { 0xee98534e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe185352, SUC_lexicon_tags_63 },
    { 0x1ab85353, SUC_lexicon_tags_2 },
    { 0xd9385354, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3585359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf638535c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e185361, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6985363, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8185366, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x64185369, SUC_lexicon_tags_49 },
    { 0xbb58536a, SUC_lexicon_tags_23 },
    { 0xbdb85369, SUC_lexicon_tags_7 },
    { 0xc4d8536c, SUC_lexicon_tags_63 },
    { 0x30b8536b, SUC_lexicon_tags_25 },
    { 0x8c58536a, SUC_lexicon_tags_12 },
    { 0x7ad8536a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1bb85371, SUC_lexicon_tags_7 },
    { 0x70d85372, SUC_lexicon_tags_76 },
    { 0x8db85373, SUC_lexicon_tags_31 },
    { 0xca385374, SUC_lexicon_tags_23 },
    { 0xdff85374, SUC_lexicon_tags_9 },
    { 0x70f85375, SUC_lexicon_tags_23 },
    { 0x60785372, SUC_lexicon_tags_30 },
    { 0x1bf85378, SUC_lexicon_tags_11 },
    { 0xa5585376, SUC_lexicon_tags_0 },
    { 0x99d85376, SUC_lexicon_tags_12 },
    { 0x7385371, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba785381, SUC_lexicon_tags_14 },
    { 0x9e785382, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef985386, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x35185388, SUC_lexicon_tags_11 },
    { 0x20185389, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff8538c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b85390, SUC_lexicon_tags_12 },
    { 0x11785390, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d85394, SUC_lexicon_tags_17 },
    { 0x70185395, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf1b85397, SUC_lexicon_tags_11 },
    { 0x88385398, SUC_lexicon_tags_14 },
    { 0xd85398, SUC_lexicon_tags_2 },
    { 0xbed8539a, SUC_lexicon_tags_2 },
    { 0x3cb8539b, SUC_lexicon_tags_4 },
    { 0xc4d8539a, SUC_lexicon_tags_2 },
    { 0x1cd8539c, SUC_lexicon_tags_21 },
    { 0xb9d8539e, SUC_lexicon_tags_23 },
    { 0x5a18539c, SUC_lexicon_tags_29 },
    { 0x8e9853a0, SUC_lexicon_tags_0 },
    { 0xdfd8539f, SUC_lexicon_tags_37 },
    { 0x7a58539c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3853a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7853ab, SUC_lexicon_tags_0 },
    { 0xcf853ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f853af, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449853bb, SUC_lexicon_tags_37 },
    { 0xb9b853bb, SUC_lexicon_tags_12 },
    { 0x565853bd, SUC_lexicon_tags_11 },
    { 0x13853be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27853c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83853c6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb31853c8, SUC_lexicon_tags_25 },
    { 0xa4d853c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a9853cb, SUC_lexicon_tags_11 },
    { 0xb57853cb, SUC_lexicon_tags_25 },
    { 0xd1d853cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203853d1, SUC_lexicon_tags_83 },
    { 0x259853d1, SUC_lexicon_tags_14 },
    { 0xa3d853d3, SUC_lexicon_tags_7 },
    { 0xe3f853d1, SUC_lexicon_tags_174 },
    { 0x1ed853d3, SUC_lexicon_tags_136 },
    { 0xda7853d6, SUC_lexicon_tags_11 },
    { 0x233853d7, SUC_lexicon_tags_0 },
    { 0xb19853d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa07853da, SUC_lexicon_tags_17 },
    { 0xb2f853db, SUC_lexicon_tags_38 },
    { 0xf5f853db, SUC_lexicon_tags_11 },
    { 0xb73853da, SUC_lexicon_tags_14 },
    { 0x69853de, SUC_lexicon_tags_35 },
    { 0x327853df, SUC_lexicon_tags_25 },
    { 0xf73853e0, SUC_lexicon_tags_17 },
    { 0xd11853e1, SUC_lexicon_tags_26 },
    { 0x391853df, SUC_lexicon_tags_15 },
    { 0xd71853dc, SUC_lexicon_tags_17 },
    { 0xa23853e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207853ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7853ed, SUC_lexicon_tags_28 },
    { 0x37b853ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713853f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d853f8, SUC_lexicon_tags_112 },
    { 0x3eb853f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd853fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98985401, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x99585403, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7985406, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdfd85408, SUC_lexicon_tags_7 },
    { 0x74d85409, SUC_lexicon_tags_25 },
    { 0xf978540a, SUC_lexicon_tags_38 },
    { 0x1b785408, SUC_lexicon_tags_15 },
    { 0x58d8540c, SUC_lexicon_tags_49 },
    { 0xba98540c, SUC_lexicon_tags_39 },
    { 0xff38540e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2385413, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x21385415, SUC_lexicon_tags_9 },
    { 0xdcb85416, SUC_lexicon_tags_28 },
    { 0xf0785415, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34585419, SUC_lexicon_tags_12 },
    { 0x8658541a, SUC_lexicon_tags_33 },
    { 0xa58541b, SUC_lexicon_tags_1 },
    { 0x9098541c, SUC_lexicon_tags_39 },
    { 0x7e58541c, SUC_lexicon_tags_25 },
    { 0x7018541e, SUC_lexicon_tags_10 },
    { 0xcd18541e, SUC_lexicon_tags_27 },
    { 0x41f85420, SUC_lexicon_tags_33 },
    { 0x92b85421, SUC_lexicon_tags_37 },
    { 0x25785420, SUC_lexicon_tags_0 },
    { 0xc6d8541c, SUC_lexicon_tags_17 },
    { 0x5d85423, SUC_lexicon_tags_15 },
    { 0x7b585425, SUC_lexicon_tags_12 },
    { 0x7bf85426, SUC_lexicon_tags_11 },
    { 0xb0185420, SUC_lexicon_tags_15 },
    { 0x35b85427, SUC_lexicon_tags_12 },
    { 0x65785429, SUC_lexicon_tags_14 },
    { 0xb738542a, SUC_lexicon_tags_82 },
    { 0xa2385424, SUC_lexicon_tags_94 },
    { 0xaaf85428, SUC_lexicon_tags_37 },
    { 0x3cb85420, SUC_lexicon_tags_15 },
    { 0xcf98542e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48f85430, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x12585432, SUC_lexicon_tags_9 },
    { 0xd8385432, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaf985435, SUC_lexicon_tags_13 },
    { 0xb7785435, SUC_lexicon_tags_10 },
    { 0xd0585437, SUC_lexicon_tags_25 },
    { 0x6f785438, SUC_lexicon_tags_17 },
    { 0xd6b85437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2898543e, SUC_lexicon_tags_108 },
    { 0x4df8543f, SUC_lexicon_tags_12 },
    { 0xf4185440, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5785445, SUC_lexicon_tags_11 },
    { 0x6c185446, SUC_lexicon_tags_16 },
    { 0x13f85447, SUC_lexicon_tags_66 },
    { 0x8f585448, SUC_lexicon_tags_27 },
    { 0xa3185447, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b8544e, SUC_lexicon_tags_6 },
    { 0xd9b8544e, SUC_lexicon_tags_9 },
    { 0x26d85450, SUC_lexicon_tags_17 },
    { 0x88d8544f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d585453, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c785458, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe358545a, SUC_lexicon_tags_38 },
    { 0x5f38545b, SUC_lexicon_tags_7 },
    { 0x1f38545b, SUC_lexicon_tags_47 },
    { 0x4018545d, SUC_lexicon_tags_45 },
    { 0xbcd8545a, SUC_lexicon_tags_17 },
    { 0x8518545f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d85463, SUC_lexicon_tags_7 },
    { 0xb9f85464, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x83185466, SUC_lexicon_tags_1 },
    { 0xebb85467, SUC_lexicon_tags_68 },
    { 0x56b85468, SUC_lexicon_tags_16 },
    { 0xcd185468, SUC_lexicon_tags_8 },
    { 0xda98546a, SUC_lexicon_tags_12 },
    { 0xe898546b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x20d8546d, SUC_lexicon_tags_1 },
    { 0xcf58546e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x65185470, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xac985473, SUC_lexicon_tags_2 },
    { 0x81585474, SUC_lexicon_tags_15 },
    { 0x41585473, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebf85477, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x24185479, SUC_lexicon_tags_19 },
    { 0x26d8547a, SUC_lexicon_tags_82 },
    { 0x9ed8547b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d8547f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x28385481, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd185484, SUC_lexicon_tags_4 },
    { 0xdad85484, SUC_lexicon_tags_63 },
    { 0x7bf85486, SUC_lexicon_tags_37 },
    { 0x67385487, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd8548a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xadf8548c, SUC_lexicon_tags_29 },
    { 0x2d38548d, SUC_lexicon_tags_17 },
    { 0x98b8548e, SUC_lexicon_tags_1 },
    { 0x3258548c, SUC_lexicon_tags_31 },
    { 0x45585490, SUC_lexicon_tags_38 },
    { 0x3cb85490, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9785494, SUC_lexicon_tags_14 },
    { 0xbb785495, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c185499, SUC_lexicon_tags_7 },
    { 0x9118549a, SUC_lexicon_tags_23 },
    { 0x9d585499, SUC_lexicon_tags_47 },
    { 0x4378549c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df854a0, SUC_lexicon_tags_2 },
    { 0x8f9854a0, SUC_lexicon_tags_6 },
    { 0x2df854a1, SUC_lexicon_tags_2 },
    { 0xaf854a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d854a7, SUC_lexicon_tags_6 },
    { 0xd55854a8, SUC_lexicon_tags_1 },
    { 0x47b854a8, SUC_lexicon_tags_49 },
    { 0x3c1854aa, SUC_lexicon_tags_27 },
    { 0x235854aa, SUC_lexicon_tags_14 },
    { 0x245854ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b854b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627854b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x549854ba, SUC_lexicon_tags_34 },
    { 0xe07854bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3854be, SUC_lexicon_tags_15 },
    { 0x93d854bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd854c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545854c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc6f854c8, SUC_lexicon_tags_26 },
    { 0x8cf854c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba9854cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab854d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1854d4, SUC_lexicon_tags_43 },
    { 0x5e7854d4, SUC_lexicon_tags_15 },
    { 0xf9d854d5, SUC_lexicon_tags_68 },
    { 0x4bf854d6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x56b854d9, SUC_lexicon_tags_46 },
    { 0x52f854da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67854e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead854ed, SUC_lexicon_tags_15 },
    { 0x89d854ed, SUC_lexicon_tags_45 },
    { 0x663854ef, SUC_lexicon_tags_45 },
    { 0x2db854f0, SUC_lexicon_tags_4 },
    { 0x799854f0, SUC_lexicon_tags_2 },
    { 0x969854ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfd7854f4, SUC_lexicon_tags_17 },
    { 0x3ad854f4, SUC_lexicon_tags_3 },
    { 0x291854f6, SUC_lexicon_tags_63 },
    { 0x803854f7, SUC_lexicon_tags_10 },
    { 0x7a1854f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bd854fa, SUC_lexicon_tags_38 },
    { 0x7d7854fa, SUC_lexicon_tags_30 },
    { 0x98d854fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f185501, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99385508, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b8550b, SUC_lexicon_tags_18 },
    { 0x898550c, SUC_lexicon_tags_47 },
    { 0xe258550d, SUC_lexicon_tags_65 },
    { 0xe458550e, SUC_lexicon_tags_11 },
    { 0xe38550f, SUC_lexicon_tags_25 },
    { 0x2c98550e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2985512, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad8551a, SUC_lexicon_tags_0 },
    { 0xdc98551a, SUC_lexicon_tags_10 },
    { 0x9c38551b, SUC_lexicon_tags_25 },
    { 0x3bf8551d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d85523, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11985530, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f85533, SUC_lexicon_tags_35 },
    { 0xe2385534, SUC_lexicon_tags_29 },
    { 0xac385534, SUC_lexicon_tags_25 },
    { 0xc3985536, SUC_lexicon_tags_47 },
    { 0xe185537, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc518553a, SUC_lexicon_tags_38 },
    { 0x8438553b, SUC_lexicon_tags_17 },
    { 0xcc38553b, SUC_lexicon_tags_23 },
    { 0x1e18553d, SUC_lexicon_tags_53 },
    { 0xd1d8553d, SUC_lexicon_tags_12 },
    { 0xc58553b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9785541, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f85544, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f8554a, SUC_lexicon_tags_12 },
    { 0x7678554a, SUC_lexicon_tags_14 },
    { 0xfb78554a, SUC_lexicon_tags_17 },
    { 0x518554d, SUC_lexicon_tags_28 },
    { 0x1678554b, SUC_lexicon_tags_15 },
    { 0xcb98554a, SUC_lexicon_tags_23 },
    { 0x73f85550, SUC_lexicon_tags_7 },
    { 0xa7d85551, SUC_lexicon_tags_7 },
    { 0x59185551, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f8555d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c785561, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3538556a, SUC_lexicon_tags_38 },
    { 0x9138556b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9898556d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb158556f, SUC_lexicon_tags_19 },
    { 0x9418556f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66785572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f85576, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26985578, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf858557c, SUC_lexicon_tags_15 },
    { 0xabf8557c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe178557f, SUC_lexicon_tags_26 },
    { 0x43585580, SUC_lexicon_tags_31 },
    { 0x65985580, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5db85583, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf185586, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x65985588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9778558d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a185592, SUC_lexicon_tags_7 },
    { 0xbd585593, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x30985595, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc785599, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d8559f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x531855a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7f1855a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89855a7, SUC_lexicon_tags_49 },
    { 0xdab855a8, SUC_lexicon_tags_7 },
    { 0x795855a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x911855ab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9855ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f855b3, SUC_lexicon_tags_47 },
    { 0x9eb855b3, SUC_lexicon_tags_37 },
    { 0xbf3855b5, SUC_lexicon_tags_25 },
    { 0x20b855b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d855b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x15855bc, SUC_lexicon_tags_25 },
    { 0x5df855bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x353855c0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x255855c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba9855c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b855c8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5855cc, SUC_lexicon_tags_64 },
    { 0x693855cd, SUC_lexicon_tags_17 },
    { 0x867855cc, SUC_lexicon_tags_299 },
    { 0x1fb855cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x69f855d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x85f855d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfcd855d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe01855d7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f855da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x331855dc, SUC_lexicon_tags_7 },
    { 0x36f855dc, SUC_lexicon_tags_2 },
    { 0xc5855dc, SUC_lexicon_tags_0 },
    { 0x8a1855df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff855e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f855e5, SUC_lexicon_tags_37 },
    { 0x1d1855e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07855ea, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583855ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x101855f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6f855f3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x3cd855f5, SUC_lexicon_tags_25 },
    { 0x7f9855f6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659855fb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x241855fe, SUC_lexicon_tags_14 },
    { 0x36f855ff, SUC_lexicon_tags_27 },
    { 0x137855fe, SUC_lexicon_tags_17 },
    { 0x2af855ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd85604, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96385606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37585609, SUC_lexicon_tags_12 },
    { 0x4a185609, SUC_lexicon_tags_11 },
    { 0x41b8560a, SUC_lexicon_tags_38 },
    { 0xd278560c, SUC_lexicon_tags_45 },
    { 0xa258560b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6585619, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b98561c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6278561e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8785621, SUC_lexicon_tags_6 },
    { 0x94b85622, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d85625, SUC_lexicon_tags_37 },
    { 0xc0d85626, SUC_lexicon_tags_31 },
    { 0x76585626, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57f85629, SUC_lexicon_tags_12 },
    { 0x3ad8562a, SUC_lexicon_tags_28 },
    { 0xb98562b, SUC_lexicon_tags_37 },
    { 0xa258562c, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f85631, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d85634, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db8563c, SUC_lexicon_tags_6 },
    { 0x8d8563c, SUC_lexicon_tags_15 },
    { 0xf4d8563d, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985647, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b8564a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80785650, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d785653, SUC_lexicon_tags_63 },
    { 0xc585654, SUC_lexicon_tags_17 },
    { 0x49585654, SUC_lexicon_tags_49 },
    { 0xd4385655, SUC_lexicon_tags_9 },
    { 0xb9d85656, SUC_lexicon_tags_45 },
    { 0x76b85658, SUC_lexicon_tags_43 },
    { 0x45785659, SUC_lexicon_tags_46 },
    { 0x85d85659, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6785661, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13385668, SUC_lexicon_tags_92 },
    { 0xf8f85669, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6ff8566b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f785674, SUC_lexicon_tags_0 },
    { 0x9dd85674, SUC_lexicon_tags_18 },
    { 0xf4385674, SUC_lexicon_tags_2 },
    { 0xf8585677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab8567b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b785680, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x47985682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbf85684, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f85687, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f8568b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x6738568d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d85691, SUC_lexicon_tags_11 },
    { 0x3b185692, SUC_lexicon_tags_47 },
    { 0x87785693, SUC_lexicon_tags_7 },
    { 0x6af85694, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48185696, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f85699, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b8569c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937856a2, SUC_lexicon_tags_11 },
    { 0x289856a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79856a8, SUC_lexicon_tags_13 },
    { 0x647856a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa15856ab, SUC_lexicon_tags_33 },
    { 0x5ab856ac, SUC_lexicon_tags_12 },
    { 0x8e5856ab, SUC_lexicon_tags_7 },
    { 0x617856ae, SUC_lexicon_tags_17 },
    { 0xac5856ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09856b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237856b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9b9856b9, SUC_lexicon_tags_37 },
    { 0x18f856b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d856bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f856c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d856c3, SUC_lexicon_tags_17 },
    { 0xd07856c4, SUC_lexicon_tags_47 },
    { 0xf89856c4, SUC_lexicon_tags_77 },
    { 0x607856c3, SUC_lexicon_tags_14 },
    { 0xd45856c7, SUC_lexicon_tags_43 },
    { 0x4c3856c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97856cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f5856cd, SUC_lexicon_tags_25 },
    { 0xd87856cd, SUC_lexicon_tags_79 },
    { 0xb99856cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b856d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x909856d5, SUC_lexicon_tags_14 },
    { 0x4a1856d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x245856d8, SUC_lexicon_tags_4 },
    { 0x66f856d9, SUC_lexicon_tags_45 },
    { 0x6b856da, SUC_lexicon_tags_37 },
    { 0x5fb856db, SUC_lexicon_tags_37 },
    { 0x137856dc, SUC_lexicon_tags_3 },
    { 0xb0d856d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x149856df, SUC_lexicon_tags_30 },
    { 0xa01856e0, SUC_lexicon_tags_17 },
    { 0x805856e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd856e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21856e9, SUC_lexicon_tags_2 },
    { 0xfdd856ea, SUC_lexicon_tags_32 },
    { 0xfff856e9, SUC_lexicon_tags_11 },
    { 0xa0f856eb, SUC_lexicon_tags_14 },
    { 0x575856ed, SUC_lexicon_tags_12 },
    { 0x23f856e9, SUC_lexicon_tags_74 },
    { 0xacb856ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b856f3, SUC_lexicon_tags_23 },
    { 0x9d3856f3, SUC_lexicon_tags_85 },
    { 0x377856f5, SUC_lexicon_tags_2 },
    { 0x4e7856f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xacf856f8, SUC_lexicon_tags_31 },
    { 0x779856f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1856fd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a385700, SUC_lexicon_tags_15 },
    { 0x2f385700, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f85704, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd85708, SUC_lexicon_tags_15 },
    { 0x60b85709, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5785712, SUC_lexicon_tags_7 },
    { 0xced85713, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec385715, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb2985717, SUC_lexicon_tags_11 },
    { 0x93b85718, SUC_lexicon_tags_77 },
    { 0xfc185719, SUC_lexicon_tags_4 },
    { 0x8c785719, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad8571d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b98571f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78985731, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94385736, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5998573f, SUC_lexicon_tags_79 },
    { 0x48d85740, SUC_lexicon_tags_46 },
    { 0xcc385741, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x76385744, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x20f85746, SUC_lexicon_tags_9 },
    { 0x42185747, SUC_lexicon_tags_25 },
    { 0x2ef85747, SUC_lexicon_tags_10 },
    { 0xa7585749, SUC_lexicon_tags_45 },
    { 0x79985747, SUC_lexicon_tags_49 },
    { 0xbe18574b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8b8574e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82b85750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b385752, SUC_lexicon_tags_37 },
    { 0x8db85753, SUC_lexicon_tags_37 },
    { 0x19785752, SUC_lexicon_tags_25 },
    { 0xfab85755, SUC_lexicon_tags_23 },
    { 0x30385752, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf9b85758, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6058575a, SUC_lexicon_tags_19 },
    { 0x4ef8575b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaa58575d, SUC_lexicon_tags_2 },
    { 0xf218575d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f85761, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57985763, SUC_lexicon_tags_23 },
    { 0xc7985764, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b385768, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa618576a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d8576f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d85779, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc718577b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff85781, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6185785, SUC_lexicon_tags_11 },
    { 0x5985785, SUC_lexicon_tags_7 },
    { 0x10785787, SUC_lexicon_tags_8 },
    { 0x1dd85787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e58578c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d8578f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd8b85791, SUC_lexicon_tags_6 },
    { 0xe6185791, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf85796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f8579a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d58579d, SUC_lexicon_tags_10 },
    { 0xf98579e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f3857a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e9857a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d857a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x771857a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07857ac, SUC_lexicon_tags_25 },
    { 0x2f857ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f857b0, SUC_lexicon_tags_27 },
    { 0x877857b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67857b8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f857bd, SUC_lexicon_tags_56 },
    { 0x9db857bd, SUC_lexicon_tags_12 },
    { 0xa1b857bf, SUC_lexicon_tags_8 },
    { 0xffb857bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x971857c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x53d857c4, SUC_lexicon_tags_61 },
    { 0x47f857c5, SUC_lexicon_tags_37 },
    { 0x1f5857c6, SUC_lexicon_tags_72 },
    { 0xde9857c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x645857c9, SUC_lexicon_tags_27 },
    { 0xaef857c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7857d2, SUC_lexicon_tags_19 },
    { 0xb13857d3, SUC_lexicon_tags_49 },
    { 0x43d857d4, SUC_lexicon_tags_60 },
    { 0x5857d5, SUC_lexicon_tags_7 },
    { 0xb59857d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbbb857d8, SUC_lexicon_tags_2 },
    { 0xd61857d9, SUC_lexicon_tags_37 },
    { 0x6f9857da, SUC_lexicon_tags_15 },
    { 0xb1b857da, SUC_lexicon_tags_17 },
    { 0xeaf857dc, SUC_lexicon_tags_42 },
    { 0x8e1857dd, SUC_lexicon_tags_17 },
    { 0x887857dd, SUC_lexicon_tags_14 },
    { 0x79d857dd, SUC_lexicon_tags_11 },
    { 0xfe1857df, SUC_lexicon_tags_82 },
    { 0x519857de, SUC_lexicon_tags_45 },
    { 0x66d857e2, SUC_lexicon_tags_23 },
    { 0xead857e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763857e8, SUC_lexicon_tags_1 },
    { 0xfc3857e8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897857ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd3857ef, SUC_lexicon_tags_27 },
    { 0x2e7857f0, SUC_lexicon_tags_19 },
    { 0x123857f0, SUC_lexicon_tags_37 },
    { 0x4c3857ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79857f6, SUC_lexicon_tags_25 },
    { 0x97b857f7, SUC_lexicon_tags_38 },
    { 0xe9857f7, SUC_lexicon_tags_61 },
    { 0x437857f7, SUC_lexicon_tags_1 },
    { 0x4fd857f6, SUC_lexicon_tags_29 },
    { 0x967857f6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc13857fd, SUC_lexicon_tags_17 },
    { 0x1b3857fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9385801, SUC_lexicon_tags_48 },
    { 0x35b85801, SUC_lexicon_tags_12 },
    { 0x15985801, SUC_lexicon_tags_13 },
    { 0x2cb85804, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc385806, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ed85808, SUC_lexicon_tags_17 },
    { 0x93185809, SUC_lexicon_tags_40 },
    { 0x94385809, SUC_lexicon_tags_16 },
    { 0xc7d8580a, SUC_lexicon_tags_1 },
    { 0x1298580c, SUC_lexicon_tags_49 },
    { 0x84b8580d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f85811, SUC_lexicon_tags_19 },
    { 0xafb85811, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d85815, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb758581b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f785827, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e785829, SUC_lexicon_tags_12 },
    { 0xd218582a, SUC_lexicon_tags_25 },
    { 0xe4785829, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb8582d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x318582f, SUC_lexicon_tags_65 },
    { 0xc1385830, SUC_lexicon_tags_1 },
    { 0x8a38582f, SUC_lexicon_tags_11 },
    { 0x5d385832, SUC_lexicon_tags_33 },
    { 0xad985833, SUC_lexicon_tags_7 },
    { 0x71b85834, SUC_lexicon_tags_23 },
    { 0x8cd85832, SUC_lexicon_tags_1 },
    { 0xf3585835, SUC_lexicon_tags_42 },
    { 0x40b85837, SUC_lexicon_tags_207 },
    { 0xc0b85836, SUC_lexicon_tags_7 },
    { 0xda385831, SUC_lexicon_tags_17 },
    { 0x5e785832, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe785842, SUC_lexicon_tags_7 },
    { 0x7e185842, SUC_lexicon_tags_37 },
    { 0x13f85842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96b85846, SUC_lexicon_tags_14 },
    { 0x55f85847, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1378584c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb85851, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70985853, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ed85855, SUC_lexicon_tags_37 },
    { 0x44585855, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8b85858, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1818585a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81d8585c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4985860, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6585863, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1185867, SUC_lexicon_tags_45 },
    { 0xd8985867, SUC_lexicon_tags_0 },
    { 0x50d85869, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x8bb8586b, SUC_lexicon_tags_39 },
    { 0xdeb8586c, SUC_lexicon_tags_1 },
    { 0x64d8586c, SUC_lexicon_tags_32 },
    { 0xd258586e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1e585870, SUC_lexicon_tags_23 },
    { 0x8785870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18385873, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7985877, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d8587c, SUC_lexicon_tags_17 },
    { 0x1718587d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f85881, SUC_lexicon_tags_33 },
    { 0xec185881, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd785886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7785888, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a38588d, SUC_lexicon_tags_17 },
    { 0x2e38588d, SUC_lexicon_tags_33 },
    { 0x52f8588f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85785891, SUC_lexicon_tags_31 },
    { 0xec385892, SUC_lexicon_tags_15 },
    { 0x83185893, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68585895, SUC_lexicon_tags_9 },
    { 0xfa185896, SUC_lexicon_tags_25 },
    { 0xe7b85897, SUC_lexicon_tags_15 },
    { 0x9bd85898, SUC_lexicon_tags_26 },
    { 0x6ef85898, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813858aa, SUC_lexicon_tags_11 },
    { 0xd9f858ab, SUC_lexicon_tags_7 },
    { 0xc51858aa, SUC_lexicon_tags_11 },
    { 0xfb1858ad, SUC_lexicon_tags_7 },
    { 0xe7f858ae, SUC_lexicon_tags_33 },
    { 0xe5b858ab, SUC_lexicon_tags_15 },
    { 0xf49858b0, SUC_lexicon_tags_224 },
    { 0x239858b0, SUC_lexicon_tags_17 },
    { 0x887858ae, SUC_lexicon_tags_44 },
    { 0xab7858b3, SUC_lexicon_tags_35 },
    { 0x37b858b4, SUC_lexicon_tags_17 },
    { 0x321858b3, SUC_lexicon_tags_1 },
    { 0xa35858b2, SUC_lexicon_tags_0 },
    { 0x463858b7, SUC_lexicon_tags_47 },
    { 0xc67858b8, SUC_lexicon_tags_39 },
    { 0x6d3858b7, SUC_lexicon_tags_11 },
    { 0x503858b8, SUC_lexicon_tags_75 },
    { 0xde5858bb, SUC_lexicon_tags_17 },
    { 0x965858bc, SUC_lexicon_tags_29 },
    { 0xe0d858bd, SUC_lexicon_tags_2 },
    { 0xbaf858aa, SUC_lexicon_tags_47 },
    { 0xd27858bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14f858c1, SUC_lexicon_tags_14 },
    { 0x6bb858c2, SUC_lexicon_tags_37 },
    { 0x43858c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x711858c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9858cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153858d0, SUC_lexicon_tags_37 },
    { 0x19f858d1, SUC_lexicon_tags_25 },
    { 0x95d858d1, SUC_lexicon_tags_37 },
    { 0x7c7858d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed858d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7858db, SUC_lexicon_tags_65 },
    { 0x775858dc, SUC_lexicon_tags_38 },
    { 0x5ad858dd, SUC_lexicon_tags_25 },
    { 0xc67858dd, SUC_lexicon_tags_19 },
    { 0xca9858de, SUC_lexicon_tags_12 },
    { 0xcb3858dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f858e4, SUC_lexicon_tags_7 },
    { 0xc65858e5, SUC_lexicon_tags_51 },
    { 0x73b858e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf858e9, SUC_lexicon_tags_15 },
    { 0x59f858e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b858ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3858f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49858f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed1858f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f858f9, SUC_lexicon_tags_31 },
    { 0xa13858f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d85904, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb585907, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f8590a, SUC_lexicon_tags_1 },
    { 0x4e38590a, SUC_lexicon_tags_11 },
    { 0xec38590a, SUC_lexicon_tags_25 },
    { 0x8758590a, SUC_lexicon_tags_11 },
    { 0x9b58590e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76985910, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x26585912, SUC_lexicon_tags_70 },
    { 0xf7f85912, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed85916, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa9585918, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe918591e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x73d85920, SUC_lexicon_tags_19 },
    { 0x1c385921, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15f85923, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x37f85925, SUC_lexicon_tags_7 },
    { 0x8db85925, SUC_lexicon_tags_1 },
    { 0x2a985927, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfe585929, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f8592c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59185931, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e785934, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb85937, SUC_lexicon_tags_2 },
    { 0xba785938, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8eb8593a, SUC_lexicon_tags_7 },
    { 0xd658593b, SUC_lexicon_tags_82 },
    { 0x76f8593c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8278593e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50185940, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe7585942, SUC_lexicon_tags_60 },
    { 0xab585943, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c38594a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b8594d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f585952, SUC_lexicon_tags_2 },
    { 0x71385953, SUC_lexicon_tags_37 },
    { 0x87385953, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4658595a, SUC_lexicon_tags_0 },
    { 0xcf98595b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab8595e, SUC_lexicon_tags_43 },
    { 0xa8b8595e, SUC_lexicon_tags_43 },
    { 0x99f85960, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd185966, SUC_lexicon_tags_1 },
    { 0xa1785967, SUC_lexicon_tags_5 },
    { 0xb0785966, SUC_lexicon_tags_3 },
    { 0xb3d85969, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x998596d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b985975, SUC_lexicon_tags_47 },
    { 0x9ff85976, SUC_lexicon_tags_141 },
    { 0xc0185977, SUC_lexicon_tags_547 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe798597b, SUC_lexicon_tags_7 },
    { 0x638597c, SUC_lexicon_tags_25 },
    { 0xd698597d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc658597f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x86f85981, SUC_lexicon_tags_10 },
    { 0x4fd85982, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f8598a, SUC_lexicon_tags_3 },
    { 0xf238598b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf18598d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8185990, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f85996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x34f85998, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb8599b, SUC_lexicon_tags_0 },
    { 0xed8599c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f859a2, SUC_lexicon_tags_11 },
    { 0x3af859a3, SUC_lexicon_tags_30 },
    { 0xa7859a4, SUC_lexicon_tags_26 },
    { 0x977859a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x553859a7, SUC_lexicon_tags_37 },
    { 0x71f859a8, SUC_lexicon_tags_26 },
    { 0xe25859a8, SUC_lexicon_tags_11 },
    { 0xdf7859aa, SUC_lexicon_tags_87 },
    { 0x949859a9, SUC_lexicon_tags_21 },
    { 0x4fb859ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7859b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3859b5, SUC_lexicon_tags_12 },
    { 0x96b859b5, SUC_lexicon_tags_17 },
    { 0x16f859b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb859ba, SUC_lexicon_tags_16 },
    { 0x52d859bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf15859bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d859c3, SUC_lexicon_tags_37 },
    { 0x2b5859c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd03859c6, SUC_lexicon_tags_37 },
    { 0xf73859c7, SUC_lexicon_tags_91 },
    { 0x883859c7, SUC_lexicon_tags_17 },
    { 0xcd7859c9, SUC_lexicon_tags_1 },
    { 0xd0d859ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd2b859cc, SUC_lexicon_tags_45 },
    { 0x19b859cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7859d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf79859d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x705859d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69859d8, SUC_lexicon_tags_23 },
    { 0x295859d9, SUC_lexicon_tags_11 },
    { 0x26f859da, SUC_lexicon_tags_25 },
    { 0x9cb859db, SUC_lexicon_tags_63 },
    { 0xc99859d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa2f859de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601859e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669859e7, SUC_lexicon_tags_17 },
    { 0x1c7859e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7859ea, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5859ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db859f1, SUC_lexicon_tags_17 },
    { 0x3a5859f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf53859f4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277859f8, SUC_lexicon_tags_1 },
    { 0xd0f859f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x679859fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd385a00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9785a03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e585a05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70385a07, SUC_lexicon_tags_6 },
    { 0xf1585a07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad85a0b, SUC_lexicon_tags_30 },
    { 0x52785a0b, SUC_lexicon_tags_10 },
    { 0x81185a0d, SUC_lexicon_tags_12 },
    { 0xe5185a0e, SUC_lexicon_tags_39 },
    { 0x57185a0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf1385a11, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1785a13, SUC_lexicon_tags_15 },
    { 0xe785a14, SUC_lexicon_tags_45 },
    { 0x1bf85a15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49985a1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d85a23, SUC_lexicon_tags_38 },
    { 0xb2d85a24, SUC_lexicon_tags_7 },
    { 0xded85a24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb85a28, SUC_lexicon_tags_2 },
    { 0x61f85a29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94785a2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4385a31, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30785a35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85785a3d, SUC_lexicon_tags_113 },
    { 0xc6385a3d, SUC_lexicon_tags_1 },
    { 0x3a985a3f, SUC_lexicon_tags_46 },
    { 0xa3385a3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f85a47, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f585a4a, SUC_lexicon_tags_25 },
    { 0x81f85a4a, SUC_lexicon_tags_0 },
    { 0x4f585a4c, SUC_lexicon_tags_19 },
    { 0x6c185a4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc585a52, SUC_lexicon_tags_22 },
    { 0x79b85a52, SUC_lexicon_tags_17 },
    { 0x40b85a54, SUC_lexicon_tags_61 },
    { 0x97f85a54, SUC_lexicon_tags_7 },
    { 0x7785a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d85a5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b85a63, SUC_lexicon_tags_37 },
    { 0x1c985a64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d85a68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73785a6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad85a74, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xef585a76, SUC_lexicon_tags_2 },
    { 0xc1d85a77, SUC_lexicon_tags_11 },
    { 0x28785a78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc585a7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8185a82, SUC_lexicon_tags_63 },
    { 0x28385a83, SUC_lexicon_tags_0 },
    { 0xab385a83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28585a8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f85a90, SUC_lexicon_tags_138 },
    { 0xed185a91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa9385a93, SUC_lexicon_tags_60 },
    { 0xf385a94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77185a98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f85aa3, SUC_lexicon_tags_37 },
    { 0x46f85aa4, SUC_lexicon_tags_26 },
    { 0x82b85aa4, SUC_lexicon_tags_84 },
    { 0xa785aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44185aaa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf85ab7, SUC_lexicon_tags_25 },
    { 0xbc585ab8, SUC_lexicon_tags_17 },
    { 0xc0d85ab8, SUC_lexicon_tags_25 },
    { 0xd7385ab9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb585abc, SUC_lexicon_tags_81 },
    { 0xce785abd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd85ac0, SUC_lexicon_tags_18 },
    { 0xfa785ac0, SUC_lexicon_tags_68 },
    { 0x44f85ac0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1785acb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46385ad6, SUC_lexicon_tags_26 },
    { 0xde385ad6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b85adc, SUC_lexicon_tags_0 },
    { 0x6d785add, SUC_lexicon_tags_23 },
    { 0x16b85add, SUC_lexicon_tags_12 },
    { 0x18985add, SUC_lexicon_tags_14 },
    { 0x60b85add, SUC_lexicon_tags_7 },
    { 0x51185adc, SUC_lexicon_tags_4 },
    { 0x22b85ae0, SUC_lexicon_tags_47 },
    { 0xf0b85ae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3585ae7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f85aec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b85af0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c985af2, SUC_lexicon_tags_1 },
    { 0x1d785af3, SUC_lexicon_tags_60 },
    { 0xcc985af3, SUC_lexicon_tags_2 },
    { 0x32f85af5, SUC_lexicon_tags_2 },
    { 0xafd85af6, SUC_lexicon_tags_37 },
    { 0x53185af7, SUC_lexicon_tags_95 },
    { 0x16f85af8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b85afe, SUC_lexicon_tags_38 },
    { 0x66985afe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x26785b02, SUC_lexicon_tags_17 },
    { 0xe0d85b03, SUC_lexicon_tags_12 },
    { 0x8d985b04, SUC_lexicon_tags_11 },
    { 0x22f85b05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ef85b07, SUC_lexicon_tags_28 },
    { 0x94185b08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x61f85b0a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x29785b0d, SUC_lexicon_tags_79 },
    { 0x72f85b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2785b11, SUC_lexicon_tags_11 },
    { 0xe3785b12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf85b1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b85b22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22785b27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23585b29, SUC_lexicon_tags_17 },
    { 0xe9785b2a, SUC_lexicon_tags_3 },
    { 0x5eb85b2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xda185b2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc985b31, SUC_lexicon_tags_17 },
    { 0x34585b31, SUC_lexicon_tags_55 },
    { 0x5f785b33, SUC_lexicon_tags_25 },
    { 0x59f85b32, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6ed85b36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20785b3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6785b3e, SUC_lexicon_tags_11 },
    { 0xdbf85b3f, SUC_lexicon_tags_11 },
    { 0xe5185b40, SUC_lexicon_tags_23 },
    { 0xd7185b41, SUC_lexicon_tags_25 },
    { 0x9c585b41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf85b45, SUC_lexicon_tags_19 },
    { 0x14385b46, SUC_lexicon_tags_1 },
    { 0x5b85b46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37585b4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff85b4e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af85b52, SUC_lexicon_tags_33 },
    { 0x70785b52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d85b56, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2f185b58, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19785b5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f985b62, SUC_lexicon_tags_9 },
    { 0xff385b62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48785b68, SUC_lexicon_tags_11 },
    { 0x3cf85b69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d85b6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef585b71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0185b78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50b85b7c, SUC_lexicon_tags_324 },
    { 0xb8b85b7c, SUC_lexicon_tags_46 },
    { 0x67385b7e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x40985b80, SUC_lexicon_tags_45 },
    { 0xa9185b81, SUC_lexicon_tags_69 },
    { 0x51985b81, SUC_lexicon_tags_38 },
    { 0xde385b81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc785b86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f85b89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9985b8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1f85b8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x15785b90, SUC_lexicon_tags_42 },
    { 0x2e785b90, SUC_lexicon_tags_11 },
    { 0xca185b91, SUC_lexicon_tags_87 },
    { 0x8d785b93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d85b9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13585b9f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x86785ba2, SUC_lexicon_tags_7 },
    { 0xf0185ba2, SUC_lexicon_tags_1 },
    { 0xc8385ba4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b985bac, SUC_lexicon_tags_33 },
    { 0xe9785bad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f185bb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d585bb9, SUC_lexicon_tags_30 },
    { 0xc2985bb9, SUC_lexicon_tags_1 },
    { 0xfd85bb9, SUC_lexicon_tags_37 },
    { 0xcdf85bbc, SUC_lexicon_tags_11 },
    { 0x1d785bbd, SUC_lexicon_tags_9 },
    { 0x57185bbc, SUC_lexicon_tags_47 },
    { 0x1fd85bbf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60785bc9, SUC_lexicon_tags_7 },
    { 0xbeb85bca, SUC_lexicon_tags_2 },
    { 0xd85bcb, SUC_lexicon_tags_35 },
    { 0x8a585bcc, SUC_lexicon_tags_15 },
    { 0xbb785bca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d85bd0, SUC_lexicon_tags_11 },
    { 0xaed85bd1, SUC_lexicon_tags_11 },
    { 0x3d185bd1, SUC_lexicon_tags_66 },
    { 0x8bf85bd3, SUC_lexicon_tags_12 },
    { 0x79b85bd4, SUC_lexicon_tags_11 },
    { 0xef85bd4, SUC_lexicon_tags_2 },
    { 0x1d85bd5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5d785bd8, SUC_lexicon_tags_25 },
    { 0xe4f85bd9, SUC_lexicon_tags_80 },
    { 0x87785bd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af85be5, SUC_lexicon_tags_37 },
    { 0x9cf85be6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f85be9, SUC_lexicon_tags_55 },
    { 0x50185bea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f85bef, SUC_lexicon_tags_12 },
    { 0xc5985bf0, SUC_lexicon_tags_81 },
    { 0xed185bf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4985bf3, SUC_lexicon_tags_0 },
    { 0x9bb85bf4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb785bf7, SUC_lexicon_tags_37 },
    { 0x2cf85bf7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa4185bfa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd4385bfc, SUC_lexicon_tags_1 },
    { 0xe6985bfc, SUC_lexicon_tags_83 },
    { 0x81d85bfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5985c01, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb85c04, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb4b85c06, SUC_lexicon_tags_8 },
    { 0x38185c07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2585c0a, SUC_lexicon_tags_63 },
    { 0xef185c0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b85c10, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f85c14, SUC_lexicon_tags_4 },
    { 0x20785c15, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f85c19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1f85c1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66d85c1d, SUC_lexicon_tags_47 },
    { 0xb1585c1e, SUC_lexicon_tags_40 },
    { 0x63d85c1e, SUC_lexicon_tags_1 },
    { 0x92985c20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d385c22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75985c28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd8f85c2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe785c2c, SUC_lexicon_tags_12 },
    { 0x7b385c2c, SUC_lexicon_tags_45 },
    { 0x3ef85c2e, SUC_lexicon_tags_11 },
    { 0xf3f85c2f, SUC_lexicon_tags_12 },
    { 0xcb985c2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfb85c32, SUC_lexicon_tags_19 },
    { 0x85585c32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13385c37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe985c39, SUC_lexicon_tags_259 },
    { 0x23785c3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9185c3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1185c43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35f85c45, SUC_lexicon_tags_3 },
    { 0x35d85c46, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdf85c48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3585c4a, SUC_lexicon_tags_37 },
    { 0x7af85c4b, SUC_lexicon_tags_25 },
    { 0x9df85c4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b85c50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b85c53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce785c58, SUC_lexicon_tags_6 },
    { 0xe2185c59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb85c5c, SUC_lexicon_tags_45 },
    { 0x4e785c5d, SUC_lexicon_tags_23 },
    { 0x87b85c5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d85c68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20385c6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d85c6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce385c6f, SUC_lexicon_tags_56 },
    { 0xb6b85c6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd85c74, SUC_lexicon_tags_17 },
    { 0x7a385c75, SUC_lexicon_tags_47 },
    { 0x60b85c74, SUC_lexicon_tags_19 },
    { 0x7fd85c74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa785c7c, SUC_lexicon_tags_25 },
    { 0xbab85c7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32985c81, SUC_lexicon_tags_23 },
    { 0x15985c82, SUC_lexicon_tags_11 },
    { 0x3c785c81, SUC_lexicon_tags_7 },
    { 0x8a785c81, SUC_lexicon_tags_26 },
    { 0x7fd85c85, SUC_lexicon_tags_1 },
    { 0x90f85c86, SUC_lexicon_tags_29 },
    { 0xaa185c86, SUC_lexicon_tags_9 },
    { 0xec785c85, SUC_lexicon_tags_14 },
    { 0xdb385c84, SUC_lexicon_tags_11 },
    { 0x46185c89, SUC_lexicon_tags_25 },
    { 0xc3185c8b, SUC_lexicon_tags_17 },
    { 0xe3d85c8c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d85c8f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf9b85c91, SUC_lexicon_tags_15 },
    { 0xdc985c92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe585c99, SUC_lexicon_tags_33 },
    { 0x17385c9a, SUC_lexicon_tags_14 },
    { 0x62f85c9a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30585ca0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4985ca3, SUC_lexicon_tags_25 },
    { 0xc2785ca4, SUC_lexicon_tags_6 },
    { 0xc0385ca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa585ca7, SUC_lexicon_tags_47 },
    { 0x5e185ca8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1bf85caa, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d85cad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88585cb1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc5585cb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f785cba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd85cbe, SUC_lexicon_tags_27 },
    { 0x3ab85cbf, SUC_lexicon_tags_12 },
    { 0xd3985cc0, SUC_lexicon_tags_15 },
    { 0xfd85cc0, SUC_lexicon_tags_5 },
    { 0x93b85cc0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x11585cc4, SUC_lexicon_tags_2 },
    { 0x24b85cc4, SUC_lexicon_tags_33 },
    { 0x23185cc5, SUC_lexicon_tags_25 },
    { 0x2bf85cc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b385cc9, SUC_lexicon_tags_37 },
    { 0x6e985cca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d85cd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d85cd8, SUC_lexicon_tags_19 },
    { 0xd7585cd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd85cdc, SUC_lexicon_tags_2 },
    { 0xfb985cdc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeef85cdf, SUC_lexicon_tags_167 },
    { 0x4d185ce0, SUC_lexicon_tags_17 },
    { 0x46d85ce1, SUC_lexicon_tags_14 },
    { 0x78585ce2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xff385ce5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x28d85ce7, SUC_lexicon_tags_25 },
    { 0xf6185ce8, SUC_lexicon_tags_47 },
    { 0x50b85ce9, SUC_lexicon_tags_66 },
    { 0x71b85ce9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f85cf0, SUC_lexicon_tags_19 },
    { 0x8d985cf0, SUC_lexicon_tags_7 },
    { 0x7d85cf2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f85cf6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5a185cf8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdd185cfa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xefd85cfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f785cff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdef85d01, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3b85d03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91785d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d85d0e, SUC_lexicon_tags_118 },
    { 0xc3385d0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c985d12, SUC_lexicon_tags_1 },
    { 0xa7d85d12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf85d17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x89785d19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26385d1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef585d25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd585d27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8985d2a, SUC_lexicon_tags_7 },
    { 0x2ad85d2a, SUC_lexicon_tags_55 },
    { 0xc8f85d2b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d85d2f, SUC_lexicon_tags_28 },
    { 0xd5d85d2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbd385d32, SUC_lexicon_tags_38 },
    { 0x15f85d33, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd185d38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed85d3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe985d3e, SUC_lexicon_tags_28 },
    { 0x3cb85d3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x45985d41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe585d43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f85d4c, SUC_lexicon_tags_15 },
    { 0x4d985d4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeef85d4f, SUC_lexicon_tags_33 },
    { 0xf9785d50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x77185d53, SUC_lexicon_tags_7 },
    { 0xc4785d54, SUC_lexicon_tags_15 },
    { 0x93185d55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b85d59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af85d5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2385d6e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd85d71, SUC_lexicon_tags_37 },
    { 0xf3585d71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b85d7b, SUC_lexicon_tags_7 },
    { 0x34b85d7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca185d7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe585d83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed85d86, SUC_lexicon_tags_1 },
    { 0x89d85d86, SUC_lexicon_tags_1 },
    { 0x18b85d88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5d985d8a, SUC_lexicon_tags_3 },
    { 0xd9b85d8b, SUC_lexicon_tags_38 },
    { 0x81185d8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb185d91, SUC_lexicon_tags_27 },
    { 0xf785d91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68185d94, SUC_lexicon_tags_7 },
    { 0x73f85d95, SUC_lexicon_tags_37 },
    { 0x8e385d95, SUC_lexicon_tags_12 },
    { 0xd3d85d95, SUC_lexicon_tags_19 },
    { 0x84185d94, SUC_lexicon_tags_25 },
    { 0xeef85d98, SUC_lexicon_tags_45 },
    { 0x36985d9a, SUC_lexicon_tags_0 },
    { 0xef385d9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d85d9e, SUC_lexicon_tags_17 },
    { 0x38985d9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f85da3, SUC_lexicon_tags_252 },
    { 0x53985da3, SUC_lexicon_tags_37 },
    { 0x51385da5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9585dac, SUC_lexicon_tags_60 },
    { 0x9ef85dac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c385daf, SUC_lexicon_tags_27 },
    { 0x24d85db0, SUC_lexicon_tags_17 },
    { 0x2eb85db1, SUC_lexicon_tags_43 },
    { 0xe3785db1, SUC_lexicon_tags_6 },
    { 0x98385db2, SUC_lexicon_tags_47 },
    { 0x8b985db4, SUC_lexicon_tags_63 },
    { 0xc2d85db5, SUC_lexicon_tags_7 },
    { 0x89d85db1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b985dbb, SUC_lexicon_tags_2 },
    { 0x28985dbc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4c785dbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d85dc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e985dc5, SUC_lexicon_tags_45 },
    { 0x48f85dc5, SUC_lexicon_tags_17 },
    { 0x13985dc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f85dca, SUC_lexicon_tags_11 },
    { 0xcf985dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f85dcd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef85dd0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d85dd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18785dd6, SUC_lexicon_tags_37 },
    { 0xe2985dd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f385ddb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x63785ddd, SUC_lexicon_tags_25 },
    { 0x385dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72f85de0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56985de4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2785de7, SUC_lexicon_tags_15 },
    { 0x90185de8, SUC_lexicon_tags_8 },
    { 0xee385de8, SUC_lexicon_tags_87 },
    { 0xa4f85dea, SUC_lexicon_tags_37 },
    { 0xd0185de7, SUC_lexicon_tags_25 },
    { 0xb5185de8, SUC_lexicon_tags_11 },
    { 0xb0785ded, SUC_lexicon_tags_18 },
    { 0xa7f85de7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3385df1, SUC_lexicon_tags_7 },
    { 0x40585df2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58585df7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb85dfc, SUC_lexicon_tags_31 },
    { 0x7ab85dfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb85e00, SUC_lexicon_tags_49 },
    { 0x73d85e00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4785e08, SUC_lexicon_tags_17 },
    { 0xbaf85e08, SUC_lexicon_tags_66 },
    { 0x1185e08, SUC_lexicon_tags_37 },
    { 0xa3585e0b, SUC_lexicon_tags_33 },
    { 0xe6b85e0b, SUC_lexicon_tags_45 },
    { 0x8d985e0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34185e10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b85e16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f85e1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd0b85e1d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9785e1f, SUC_lexicon_tags_1 },
    { 0x92b85e20, SUC_lexicon_tags_23 },
    { 0x87985e21, SUC_lexicon_tags_5 },
    { 0xf3b85e22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69d85e24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d85e27, SUC_lexicon_tags_7 },
    { 0x24585e27, SUC_lexicon_tags_25 },
    { 0x65185e28, SUC_lexicon_tags_38 },
    { 0x9f85e2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4985e2c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4785e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd85e35, SUC_lexicon_tags_38 },
    { 0x28985e36, SUC_lexicon_tags_10 },
    { 0x25985e37, SUC_lexicon_tags_30 },
    { 0xfa985e37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed785e42, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c785e44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f85e49, SUC_lexicon_tags_19 },
    { 0xa1f85e49, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdcb85e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45385e4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa6985e50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d785e53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7385e5a, SUC_lexicon_tags_45 },
    { 0x3bd85e5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc185e5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70d85e60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7185e62, SUC_lexicon_tags_31 },
    { 0x5a785e62, SUC_lexicon_tags_45 },
    { 0x5bf85e64, SUC_lexicon_tags_12 },
    { 0x36385e65, SUC_lexicon_tags_7 },
    { 0xabd85e64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b85e69, SUC_lexicon_tags_17 },
    { 0xd9385e69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38f85e6c, SUC_lexicon_tags_25 },
    { 0xf5585e6d, SUC_lexicon_tags_2 },
    { 0x38f85e6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb385e71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x10f85e74, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a585e79, SUC_lexicon_tags_11 },
    { 0x6ab85e7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaf85e7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2f85e7e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d85e81, SUC_lexicon_tags_28 },
    { 0x64585e82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8185e84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab185e86, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1385e88, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa7b85e8a, SUC_lexicon_tags_15 },
    { 0xd2385e8b, SUC_lexicon_tags_47 },
    { 0xb9785e8a, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x3bd85e8e, SUC_lexicon_tags_25 },
    { 0x92f85e8f, SUC_lexicon_tags_25 },
    { 0x56385e90, SUC_lexicon_tags_7 },
    { 0xa0f85e90, SUC_lexicon_tags_11 },
    { 0x17b85e92, SUC_lexicon_tags_0 },
    { 0x10985e93, SUC_lexicon_tags_38 },
    { 0x21985e94, SUC_lexicon_tags_2 },
    { 0x92185e95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2385e9c, SUC_lexicon_tags_11 },
    { 0xe4185e9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d85ea1, SUC_lexicon_tags_17 },
    { 0xf4d85ea2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4185ea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b85eac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6f85eae, SUC_lexicon_tags_9 },
    { 0x26585eae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe0185eb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab85eb4, SUC_lexicon_tags_62 },
    { 0x73d85eb4, SUC_lexicon_tags_27 },
    { 0xaf185eb4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1585ec0, SUC_lexicon_tags_11 },
    { 0xe8f85ec1, SUC_lexicon_tags_25 },
    { 0xec985ec2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b785ec4, SUC_lexicon_tags_2 },
    { 0xdf385ec4, SUC_lexicon_tags_15 },
    { 0xbc985ec4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc785ec9, SUC_lexicon_tags_87 },
    { 0x4e785eca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2385ed1, SUC_lexicon_tags_1 },
    { 0x2dd85ed1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d85ed7, SUC_lexicon_tags_91 },
    { 0x9c185ed8, SUC_lexicon_tags_2 },
    { 0x56785ed9, SUC_lexicon_tags_49 },
    { 0x3585eda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e785edc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e185ee8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9985eec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1985ef2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd85ef7, SUC_lexicon_tags_37 },
    { 0x76b85ef7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3185efd, SUC_lexicon_tags_28 },
    { 0x93785efe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb85f04, SUC_lexicon_tags_64 },
    { 0xf2585f05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb585f07, SUC_lexicon_tags_6 },
    { 0x5f385f07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2785f0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb585f0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b985f1b, SUC_lexicon_tags_15 },
    { 0x8dd85f1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94d85f1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x94b85f20, SUC_lexicon_tags_97 },
    { 0x57f85f21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4585f24, SUC_lexicon_tags_37 },
    { 0xef785f25, SUC_lexicon_tags_0 },
    { 0x80985f26, SUC_lexicon_tags_11 },
    { 0xa7185f25, SUC_lexicon_tags_3 },
    { 0xc9b85f26, SUC_lexicon_tags_6 },
    { 0xd1185f24, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18385f2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c385f32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b85f38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1785f3a, SUC_lexicon_tags_16 },
    { 0xa5585f3a, SUC_lexicon_tags_49 },
    { 0xf0b85f3c, SUC_lexicon_tags_1 },
    { 0x5b785f3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89d85f3f, SUC_lexicon_tags_1 },
    { 0xea385f40, SUC_lexicon_tags_12 },
    { 0x6e585f41, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x61785f43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x87785f45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f385f49, SUC_lexicon_tags_11 },
    { 0xeb385f4a, SUC_lexicon_tags_1690 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1785f4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f85f52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b185f54, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f85f58, SUC_lexicon_tags_33 },
    { 0x72985f58, SUC_lexicon_tags_7 },
    { 0xc2585f58, SUC_lexicon_tags_25 },
    { 0x46785f5b, SUC_lexicon_tags_124 },
    { 0x5585f5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d85f5f, SUC_lexicon_tags_15 },
    { 0x51985f60, SUC_lexicon_tags_17 },
    { 0x49185f61, SUC_lexicon_tags_77 },
    { 0x6bb85f60, SUC_lexicon_tags_17 },
    { 0xd0185f63, SUC_lexicon_tags_11 },
    { 0x22385f64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e585f66, SUC_lexicon_tags_12 },
    { 0xf1985f66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa585f6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d85f6e, SUC_lexicon_tags_33 },
    { 0x61585f6f, SUC_lexicon_tags_37 },
    { 0xb3f85f6f, SUC_lexicon_tags_25 },
    { 0xfc785f71, SUC_lexicon_tags_25 },
    { 0xfdd85f72, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4f985f74, SUC_lexicon_tags_49 },
    { 0x76d85f75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb85f79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f785f7c, SUC_lexicon_tags_30 },
    { 0x78985f7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7f85f7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b85f82, SUC_lexicon_tags_66 },
    { 0xfef85f83, SUC_lexicon_tags_7 },
    { 0xf1385f83, SUC_lexicon_tags_49 },
    { 0x68785f85, SUC_lexicon_tags_37 },
    { 0xb6f85f82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdbf85f88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed85f8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b585f8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1985f8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d85f92, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5d385f94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1785f96, SUC_lexicon_tags_28 },
    { 0x4bb85f97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f85f9a, SUC_lexicon_tags_6 },
    { 0xfa785f9b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6785f9d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x91385f9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f85fa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4785fa9, SUC_lexicon_tags_12 },
    { 0x5d585faa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3ed85fac, SUC_lexicon_tags_11 },
    { 0xd2185fad, SUC_lexicon_tags_12 },
    { 0xfb185fad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac585fb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0385fb5, SUC_lexicon_tags_14 },
    { 0xebf85fb6, SUC_lexicon_tags_37 },
    { 0x35d85fb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf185fba, SUC_lexicon_tags_1 },
    { 0x37785fbb, SUC_lexicon_tags_43 },
    { 0x6985fbc, SUC_lexicon_tags_11 },
    { 0x76985fbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf85fc3, SUC_lexicon_tags_1 },
    { 0xd5185fc4, SUC_lexicon_tags_19 },
    { 0xb9585fc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7185fcb, SUC_lexicon_tags_47 },
    { 0x6b785fcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d85fd1, SUC_lexicon_tags_7 },
    { 0xdd585fd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91585fd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36985fd7, SUC_lexicon_tags_0 },
    { 0x59785fd7, SUC_lexicon_tags_14 },
    { 0xca785fd9, SUC_lexicon_tags_89 },
    { 0x9fd85fda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e785fdf, SUC_lexicon_tags_0 },
    { 0x7d385fe0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f85fe4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf85fe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d585fee, SUC_lexicon_tags_11 },
    { 0xf4785fef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f85ff2, SUC_lexicon_tags_46 },
    { 0xdc585ff2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b85ffa, SUC_lexicon_tags_43 },
    { 0x18785ffb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x24585ffd, SUC_lexicon_tags_49 },
    { 0xa0585ffe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb386002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa986004, SUC_lexicon_tags_17 },
    { 0x91786005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf586007, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe378600a, SUC_lexicon_tags_673 },
    { 0x1e78600a, SUC_lexicon_tags_11 },
    { 0x8718600c, SUC_lexicon_tags_14 },
    { 0x9978600d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a986010, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17386014, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32d86016, SUC_lexicon_tags_37 },
    { 0xbdf86017, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f8601a, SUC_lexicon_tags_14 },
    { 0x3238601b, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e18601e, SUC_lexicon_tags_53 },
    { 0x6918601f, SUC_lexicon_tags_12 },
    { 0x5318601f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d86022, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43786028, SUC_lexicon_tags_7 },
    { 0xa8786029, SUC_lexicon_tags_21 },
    { 0xb798602a, SUC_lexicon_tags_11 },
    { 0xe5d86028, SUC_lexicon_tags_0 },
    { 0xdcd8602c, SUC_lexicon_tags_4 },
    { 0x5778602c, SUC_lexicon_tags_37 },
    { 0xf318602b, SUC_lexicon_tags_63 },
    { 0xa18602f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcef86031, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b86034, SUC_lexicon_tags_25 },
    { 0x79986035, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9186038, SUC_lexicon_tags_37 },
    { 0xc6386039, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99b8603b, SUC_lexicon_tags_1 },
    { 0x358603c, SUC_lexicon_tags_4 },
    { 0x5f8603b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b86040, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b86046, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d86049, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x80d8604b, SUC_lexicon_tags_45 },
    { 0x97f8604c, SUC_lexicon_tags_29 },
    { 0x83b8604d, SUC_lexicon_tags_11 },
    { 0xb4f8604d, SUC_lexicon_tags_15 },
    { 0xb5d8604d, SUC_lexicon_tags_12 },
    { 0x39786050, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f86056, SUC_lexicon_tags_7 },
    { 0xe5386057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55786059, SUC_lexicon_tags_15 },
    { 0x2418605a, SUC_lexicon_tags_37 },
    { 0x8fb86059, SUC_lexicon_tags_38 },
    { 0x1e38605c, SUC_lexicon_tags_25 },
    { 0x50b8605b, SUC_lexicon_tags_7 },
    { 0x4ed8605e, SUC_lexicon_tags_2 },
    { 0xd4f8605f, SUC_lexicon_tags_7 },
    { 0x4978605d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x55f86062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b8606e, SUC_lexicon_tags_1 },
    { 0x5f38606f, SUC_lexicon_tags_43 },
    { 0x27d86070, SUC_lexicon_tags_118 },
    { 0x5418606f, SUC_lexicon_tags_7 },
    { 0x2786072, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc786079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bb8607b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73186085, SUC_lexicon_tags_1 },
    { 0x84f86086, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1eb86088, SUC_lexicon_tags_14 },
    { 0x9ff86088, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xda58608b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd758608d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6518608f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x85986092, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb8609b, SUC_lexicon_tags_151 },
    { 0x7938609c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8c18609e, SUC_lexicon_tags_60 },
    { 0x1b8609e, SUC_lexicon_tags_33 },
    { 0x481860a0, SUC_lexicon_tags_11 },
    { 0xe2b860a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe9860a3, SUC_lexicon_tags_25 },
    { 0x1d9860a4, SUC_lexicon_tags_43 },
    { 0xbab860a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd860a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff860af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ab860b1, SUC_lexicon_tags_11 },
    { 0x237860b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ef860b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33860bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315860bf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xcf860c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441860c5, SUC_lexicon_tags_23 },
    { 0x1c7860c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5860ca, SUC_lexicon_tags_12 },
    { 0x151860cb, SUC_lexicon_tags_63 },
    { 0xa17860ca, SUC_lexicon_tags_25 },
    { 0x801860cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663860d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2d3860d4, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0x233860d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7860db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xef9860dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75860df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb860e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x107860e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x397860e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x19860e8, SUC_lexicon_tags_27 },
    { 0xc1d860e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2cf860eb, SUC_lexicon_tags_45 },
    { 0x35b860ec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437860f3, SUC_lexicon_tags_1 },
    { 0x1a7860f3, SUC_lexicon_tags_1 },
    { 0xd8d860f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93f860f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ed860f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2b1860fb, SUC_lexicon_tags_85 },
    { 0xa2b860fc, SUC_lexicon_tags_43 },
    { 0xa93860fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x386101, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2386103, SUC_lexicon_tags_15 },
    { 0xfab86104, SUC_lexicon_tags_37 },
    { 0x48f86104, SUC_lexicon_tags_16 },
    { 0x98d86106, SUC_lexicon_tags_23 },
    { 0x88f86106, SUC_lexicon_tags_31 },
    { 0x3c386106, SUC_lexicon_tags_10 },
    { 0xdd586109, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f8610c, SUC_lexicon_tags_23 },
    { 0x1778610c, SUC_lexicon_tags_25 },
    { 0xab38610e, SUC_lexicon_tags_38 },
    { 0x5838610f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69386113, SUC_lexicon_tags_30 },
    { 0x1bf86114, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4986117, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x54986119, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xacb8611b, SUC_lexicon_tags_2 },
    { 0x4f98611b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28586120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27386127, SUC_lexicon_tags_25 },
    { 0x73b86127, SUC_lexicon_tags_46 },
    { 0x2a586127, SUC_lexicon_tags_2 },
    { 0xc778612a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b8612d, SUC_lexicon_tags_57 },
    { 0xe338612d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f786130, SUC_lexicon_tags_12 },
    { 0x43386130, SUC_lexicon_tags_30 },
    { 0x59386132, SUC_lexicon_tags_13 },
    { 0x9f86133, SUC_lexicon_tags_14 },
    { 0xc1386134, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1186138, SUC_lexicon_tags_8 },
    { 0xf386138, SUC_lexicon_tags_10 },
    { 0x1eb8613a, SUC_lexicon_tags_1 },
    { 0x9298613b, SUC_lexicon_tags_15 },
    { 0x218613c, SUC_lexicon_tags_17 },
    { 0xe238613d, SUC_lexicon_tags_37 },
    { 0x3338613e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x66586140, SUC_lexicon_tags_74 },
    { 0xd9186140, SUC_lexicon_tags_28 },
    { 0xa6586141, SUC_lexicon_tags_1 },
    { 0x51f86143, SUC_lexicon_tags_26 },
    { 0x5ff86144, SUC_lexicon_tags_69 },
    { 0x7c986145, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4186149, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8d18614b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41d8614d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x238614f, SUC_lexicon_tags_17 },
    { 0xf3b86150, SUC_lexicon_tags_10 },
    { 0x50186151, SUC_lexicon_tags_151 },
    { 0xbe586152, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf86155, SUC_lexicon_tags_37 },
    { 0x74586155, SUC_lexicon_tags_1013 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a38615b, SUC_lexicon_tags_12 },
    { 0xe098615c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e58615f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f86162, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f86165, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x43386168, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa18616d, SUC_lexicon_tags_25 },
    { 0x4818616e, SUC_lexicon_tags_42 },
    { 0xeef8616f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8786173, SUC_lexicon_tags_27 },
    { 0x52d86174, SUC_lexicon_tags_11 },
    { 0xcf986175, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2986178, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6186181, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a586188, SUC_lexicon_tags_43 },
    { 0xed386189, SUC_lexicon_tags_2 },
    { 0x6d98618a, SUC_lexicon_tags_11 },
    { 0x8fb8618b, SUC_lexicon_tags_1 },
    { 0xe0f8618b, SUC_lexicon_tags_60 },
    { 0xa818618d, SUC_lexicon_tags_26 },
    { 0x9dd8618d, SUC_lexicon_tags_63 },
    { 0xa2d8618a, SUC_lexicon_tags_17 },
    { 0x1578618e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xebd86192, SUC_lexicon_tags_0 },
    { 0xf3186193, SUC_lexicon_tags_42 },
    { 0xef586192, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e786197, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x13f86199, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb538619d, SUC_lexicon_tags_37 },
    { 0x698619e, SUC_lexicon_tags_29 },
    { 0xbf98619d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d861a3, SUC_lexicon_tags_37 },
    { 0x6af861a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c7861a6, SUC_lexicon_tags_49 },
    { 0x34d861a6, SUC_lexicon_tags_1 },
    { 0xb3d861a6, SUC_lexicon_tags_38 },
    { 0x1e5861a9, SUC_lexicon_tags_27 },
    { 0xdc5861a9, SUC_lexicon_tags_17 },
    { 0xb0f861aa, SUC_lexicon_tags_27 },
    { 0x7d3861a7, SUC_lexicon_tags_12 },
    { 0xa7f861a7, SUC_lexicon_tags_11 },
    { 0xc93861ab, SUC_lexicon_tags_30 },
    { 0x441861af, SUC_lexicon_tags_25 },
    { 0x735861a9, SUC_lexicon_tags_46 },
    { 0xdbd861ad, SUC_lexicon_tags_11 },
    { 0xb59861b2, SUC_lexicon_tags_7 },
    { 0x2fb861b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b861ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33861c1, SUC_lexicon_tags_55 },
    { 0x149861c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3861c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51861c8, SUC_lexicon_tags_7 },
    { 0x56d861c9, SUC_lexicon_tags_28 },
    { 0x589861c8, SUC_lexicon_tags_12 },
    { 0x7b9861cb, SUC_lexicon_tags_11 },
    { 0x6f7861cb, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab861d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f861d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7861d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61b861db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5861de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x717861e1, SUC_lexicon_tags_27 },
    { 0x439861e2, SUC_lexicon_tags_8 },
    { 0x5c5861e2, SUC_lexicon_tags_37 },
    { 0x4bd861e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19861e6, SUC_lexicon_tags_8 },
    { 0xa2f861e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52d861e9, SUC_lexicon_tags_1 },
    { 0x2c5861e9, SUC_lexicon_tags_10 },
    { 0x8f861eb, SUC_lexicon_tags_61 },
    { 0x831861ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3861f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x373861f3, SUC_lexicon_tags_19 },
    { 0xf73861f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11861f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xedf861f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x631861fb, SUC_lexicon_tags_37 },
    { 0x8f5861fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae5861fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb986200, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd8620c, SUC_lexicon_tags_4 },
    { 0x74d8620c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf386211, SUC_lexicon_tags_15 },
    { 0x1fb86212, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xee186214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f86220, SUC_lexicon_tags_23 },
    { 0x13f86220, SUC_lexicon_tags_1 },
    { 0x4f86221, SUC_lexicon_tags_55 },
    { 0x20d86223, SUC_lexicon_tags_15 },
    { 0xb2186224, SUC_lexicon_tags_9 },
    { 0xd1786224, SUC_lexicon_tags_9 },
    { 0x85d86220, SUC_lexicon_tags_12 },
    { 0x37786224, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x15386229, SUC_lexicon_tags_9 },
    { 0xc058622a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8622d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf86230, SUC_lexicon_tags_49 },
    { 0x48786231, SUC_lexicon_tags_68 },
    { 0x86386230, SUC_lexicon_tags_11 },
    { 0x2d586232, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af86239, SUC_lexicon_tags_2 },
    { 0xadf86239, SUC_lexicon_tags_12 },
    { 0x578623b, SUC_lexicon_tags_11 },
    { 0x69b8623c, SUC_lexicon_tags_17 },
    { 0xc818623d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd86242, SUC_lexicon_tags_7 },
    { 0x88186243, SUC_lexicon_tags_37 },
    { 0xaf386242, SUC_lexicon_tags_44 },
    { 0x2bb86242, SUC_lexicon_tags_30 },
    { 0x75f86242, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d38624f, SUC_lexicon_tags_12 },
    { 0xd8d86250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf986253, SUC_lexicon_tags_68 },
    { 0xd8786254, SUC_lexicon_tags_42 },
    { 0x14986255, SUC_lexicon_tags_103 },
    { 0xca386256, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1798625b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d86268, SUC_lexicon_tags_29 },
    { 0xe2586268, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3678626c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda78626f, SUC_lexicon_tags_1 },
    { 0x33d86270, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67786273, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf386277, SUC_lexicon_tags_84 },
    { 0xcdb86278, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3698627a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6cf8627c, SUC_lexicon_tags_7 },
    { 0x55d8627d, SUC_lexicon_tags_15 },
    { 0x3638627e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d386280, SUC_lexicon_tags_14 },
    { 0x51f86280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d186284, SUC_lexicon_tags_16 },
    { 0x36f86284, SUC_lexicon_tags_93 },
    { 0xca386285, SUC_lexicon_tags_25 },
    { 0xee586287, SUC_lexicon_tags_45 },
    { 0x51386284, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbfb8628a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc158628e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6786294, SUC_lexicon_tags_46 },
    { 0xd0386295, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7518629b, SUC_lexicon_tags_1 },
    { 0x2ff8629c, SUC_lexicon_tags_26 },
    { 0xda98629d, SUC_lexicon_tags_23 },
    { 0xe678629e, SUC_lexicon_tags_60 },
    { 0x198629e, SUC_lexicon_tags_1 },
    { 0x6258629b, SUC_lexicon_tags_6 },
    { 0x4b862a1, SUC_lexicon_tags_456 },
    { 0xc9f862a2, SUC_lexicon_tags_12 },
    { 0x169862a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3862a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f862a9, SUC_lexicon_tags_65 },
    { 0x30b862aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d862af, SUC_lexicon_tags_14 },
    { 0xac9862b0, SUC_lexicon_tags_2 },
    { 0x91b862af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x273862b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9862b7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493862bd, SUC_lexicon_tags_55 },
    { 0x259862be, SUC_lexicon_tags_11 },
    { 0x21f862be, SUC_lexicon_tags_60 },
    { 0x2a5862bd, SUC_lexicon_tags_25 },
    { 0x4f9862c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb4b862c3, SUC_lexicon_tags_25 },
    { 0xd67862c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6f862c6, SUC_lexicon_tags_27 },
    { 0xb01862c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f862cc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x399862ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5862d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1862d9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb862dc, SUC_lexicon_tags_10 },
    { 0xe3b862dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29f862df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5862e4, SUC_lexicon_tags_1 },
    { 0xb862e4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x227862e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74d862e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803862f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x691862f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b862f8, SUC_lexicon_tags_60 },
    { 0x835862f9, SUC_lexicon_tags_23 },
    { 0x23b862fa, SUC_lexicon_tags_70 },
    { 0x225862fb, SUC_lexicon_tags_68 },
    { 0x151862fc, SUC_lexicon_tags_60 },
    { 0x9ab862fd, SUC_lexicon_tags_1 },
    { 0xb862fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99186304, SUC_lexicon_tags_2 },
    { 0xb9b86304, SUC_lexicon_tags_7 },
    { 0x74786305, SUC_lexicon_tags_11 },
    { 0x3d186305, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3398630e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d86312, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d86315, SUC_lexicon_tags_15 },
    { 0x6b986315, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4586319, SUC_lexicon_tags_45 },
    { 0xbc586319, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a586320, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c586324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50d86326, SUC_lexicon_tags_108 },
    { 0x2d786327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e38632a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8986331, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeb586333, SUC_lexicon_tags_12 },
    { 0x4d986334, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaff86336, SUC_lexicon_tags_25 },
    { 0x2f986337, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5ad86339, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c78633f, SUC_lexicon_tags_15 },
    { 0xda986340, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a58634a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78634e, SUC_lexicon_tags_47 },
    { 0xd958634e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x38b86351, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb386355, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43386357, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f8635a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcf78635c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6d18635e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5b386360, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db86366, SUC_lexicon_tags_31 },
    { 0x39d86367, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18636d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75786373, SUC_lexicon_tags_46 },
    { 0x9d86373, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4386376, SUC_lexicon_tags_1 },
    { 0x56386377, SUC_lexicon_tags_60 },
    { 0xed986377, SUC_lexicon_tags_8 },
    { 0x16f86378, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1658637b, SUC_lexicon_tags_25 },
    { 0x3378637b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b86380, SUC_lexicon_tags_11 },
    { 0xbeb86380, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f86384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f86386, SUC_lexicon_tags_4 },
    { 0x4db86387, SUC_lexicon_tags_45 },
    { 0x8e586388, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad8638b, SUC_lexicon_tags_6 },
    { 0x5718638c, SUC_lexicon_tags_2 },
    { 0x3d78638d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1586395, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x27586397, SUC_lexicon_tags_82 },
    { 0x4d586398, SUC_lexicon_tags_11 },
    { 0xd0186399, SUC_lexicon_tags_28 },
    { 0x6378639a, SUC_lexicon_tags_37 },
    { 0x4c58639b, SUC_lexicon_tags_15 },
    { 0xb118639b, SUC_lexicon_tags_15 },
    { 0x54f86398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9863a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b5863a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6d863a5, SUC_lexicon_tags_17 },
    { 0x16b863a6, SUC_lexicon_tags_2 },
    { 0xc43863a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd65863a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdad863ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd863ae, SUC_lexicon_tags_14 },
    { 0x105863ae, SUC_lexicon_tags_37 },
    { 0x1fb863b0, SUC_lexicon_tags_19 },
    { 0x39863b0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xfa7863b3, SUC_lexicon_tags_11 },
    { 0xf0d863b3, SUC_lexicon_tags_14 },
    { 0x617863b5, SUC_lexicon_tags_10 },
    { 0x3bb863b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141863c1, SUC_lexicon_tags_12 },
    { 0xf65863c1, SUC_lexicon_tags_7 },
    { 0x29863c3, SUC_lexicon_tags_42 },
    { 0x39863c4, SUC_lexicon_tags_11 },
    { 0xcb863c5, SUC_lexicon_tags_16 },
    { 0x1f863c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf863cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b5863cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x29d863cf, SUC_lexicon_tags_79 },
    { 0x9c7863d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d863d4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfb7863d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbed863d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x753863da, SUC_lexicon_tags_22 },
    { 0x669863db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd07863dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3f863df, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x209863e1, SUC_lexicon_tags_45 },
    { 0xd47863e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5863e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e1863e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f863eb, SUC_lexicon_tags_3 },
    { 0xf27863eb, SUC_lexicon_tags_27 },
    { 0x751863eb, SUC_lexicon_tags_11 },
    { 0xdd1863ed, SUC_lexicon_tags_34 },
    { 0x2dd863ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905863f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x657863f7, SUC_lexicon_tags_9 },
    { 0xebd863f8, SUC_lexicon_tags_1 },
    { 0x25d863f9, SUC_lexicon_tags_32 },
    { 0x961863f9, SUC_lexicon_tags_26 },
    { 0x689863fb, SUC_lexicon_tags_37 },
    { 0x5d5863fc, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x903863fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xeb386400, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80386406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b18640a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20d8640c, SUC_lexicon_tags_37 },
    { 0xec78640d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69786410, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc9386412, SUC_lexicon_tags_14 },
    { 0x14386412, SUC_lexicon_tags_12 },
    { 0xc8786414, SUC_lexicon_tags_74 },
    { 0x86d86412, SUC_lexicon_tags_66 },
    { 0x20586413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b86429, SUC_lexicon_tags_31 },
    { 0x83b8642a, SUC_lexicon_tags_13 },
    { 0x9858642a, SUC_lexicon_tags_14 },
    { 0x8fb8642b, SUC_lexicon_tags_16 },
    { 0x9d8642b, SUC_lexicon_tags_8 },
    { 0x81f8642e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22986430, SUC_lexicon_tags_16 },
    { 0x26986431, SUC_lexicon_tags_1 },
    { 0x44186432, SUC_lexicon_tags_14 },
    { 0x98986433, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5186435, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41186437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde186439, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c18643b, SUC_lexicon_tags_10 },
    { 0xa438643b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3538643f, SUC_lexicon_tags_15 },
    { 0xcab8643f, SUC_lexicon_tags_10 },
    { 0x27d86441, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda586444, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd86446, SUC_lexicon_tags_38 },
    { 0x71786447, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f8644b, SUC_lexicon_tags_47 },
    { 0xb98644c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d86451, SUC_lexicon_tags_37 },
    { 0x54b86452, SUC_lexicon_tags_14 },
    { 0x85d86453, SUC_lexicon_tags_31 },
    { 0x25386454, SUC_lexicon_tags_27 },
    { 0x7f586452, SUC_lexicon_tags_3 },
    { 0x9a786452, SUC_lexicon_tags_2 },
    { 0x3b586457, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b38645b, SUC_lexicon_tags_55 },
    { 0x36d8645b, SUC_lexicon_tags_37 },
    { 0x3bb8645d, SUC_lexicon_tags_1 },
    { 0xc3f8645d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b86461, SUC_lexicon_tags_15 },
    { 0xffd86461, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29986465, SUC_lexicon_tags_15 },
    { 0xc2586465, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x96386469, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2f58646b, SUC_lexicon_tags_15 },
    { 0xb678646c, SUC_lexicon_tags_28 },
    { 0x91d8646d, SUC_lexicon_tags_0 },
    { 0x5318646e, SUC_lexicon_tags_65 },
    { 0xc58646d, SUC_lexicon_tags_31 },
    { 0x3c78646c, SUC_lexicon_tags_55 },
    { 0xe2586471, SUC_lexicon_tags_0 },
    { 0xe3186471, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf86477, SUC_lexicon_tags_11 },
    { 0x3d986477, SUC_lexicon_tags_17 },
    { 0x186479, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdb38647b, SUC_lexicon_tags_37 },
    { 0x218647c, SUC_lexicon_tags_38 },
    { 0xe898647d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b586482, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x39186485, SUC_lexicon_tags_1 },
    { 0x76586486, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda98648b, SUC_lexicon_tags_11 },
    { 0x18d8648b, SUC_lexicon_tags_11 },
    { 0x4d98648b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1986490, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x78386493, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d86497, SUC_lexicon_tags_11 },
    { 0x7c786498, SUC_lexicon_tags_8 },
    { 0x34986498, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd758649b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3864a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xedf864a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709864a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x777864ab, SUC_lexicon_tags_12 },
    { 0x51f864ac, SUC_lexicon_tags_2 },
    { 0xb9d864ad, SUC_lexicon_tags_23 },
    { 0xe8f864ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3864b4, SUC_lexicon_tags_47 },
    { 0xd3d864b5, SUC_lexicon_tags_49 },
    { 0x819864b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x247864b8, SUC_lexicon_tags_15 },
    { 0x8a5864b9, SUC_lexicon_tags_1 },
    { 0x471864ba, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x93b864bc, SUC_lexicon_tags_8 },
    { 0x7f864bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271864c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x35864ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229864d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc3b864d2, SUC_lexicon_tags_39 },
    { 0xd4d864d3, SUC_lexicon_tags_44 },
    { 0x1ed864d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43b864d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x535864d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d864dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b864e2, SUC_lexicon_tags_45 },
    { 0xe19864e3, SUC_lexicon_tags_38 },
    { 0xfe1864e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0b864e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d864eb, SUC_lexicon_tags_25 },
    { 0x1c9864ec, SUC_lexicon_tags_55 },
    { 0xcf7864ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3864f0, SUC_lexicon_tags_6 },
    { 0xbd7864f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787864f5, SUC_lexicon_tags_17 },
    { 0xa1f864f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x315864f9, SUC_lexicon_tags_37 },
    { 0x4eb864fa, SUC_lexicon_tags_76 },
    { 0x2f864fb, SUC_lexicon_tags_14 },
    { 0x8e7864fa, SUC_lexicon_tags_37 },
    { 0xb13864fc, SUC_lexicon_tags_12 },
    { 0xe67864fe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59586503, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb86509, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2786510, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4f86512, SUC_lexicon_tags_26 },
    { 0xa9b86513, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x68986516, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x32586518, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe18651c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c986520, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff86523, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99b86525, SUC_lexicon_tags_25 },
    { 0x21386526, SUC_lexicon_tags_37 },
    { 0x43986525, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb78652a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40586530, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x15786533, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb78653b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe458653d, SUC_lexicon_tags_49 },
    { 0xf158653e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x39586541, SUC_lexicon_tags_10 },
    { 0x9c186542, SUC_lexicon_tags_50 },
    { 0xe0586543, SUC_lexicon_tags_23 },
    { 0x1c786543, SUC_lexicon_tags_17 },
    { 0xc4b86545, SUC_lexicon_tags_82 },
    { 0x56986542, SUC_lexicon_tags_8 },
    { 0xbf186542, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d8654b, SUC_lexicon_tags_25 },
    { 0xdc18654c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd8654f, SUC_lexicon_tags_14 },
    { 0xf8d8654f, SUC_lexicon_tags_27 },
    { 0x40b86551, SUC_lexicon_tags_182 },
    { 0x87b86551, SUC_lexicon_tags_17 },
    { 0xa7d86553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b86558, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f58655d, SUC_lexicon_tags_27 },
    { 0x7a78655d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f86561, SUC_lexicon_tags_2 },
    { 0x76986562, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6786564, SUC_lexicon_tags_63 },
    { 0x2c386564, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d8656d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed386571, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b86576, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28186579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d86581, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x91f86583, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x63386586, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb86589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d8658e, SUC_lexicon_tags_16 },
    { 0x90d8658e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef86594, SUC_lexicon_tags_27 },
    { 0xfa786595, SUC_lexicon_tags_25 },
    { 0xfb386596, SUC_lexicon_tags_7 },
    { 0xabd86596, SUC_lexicon_tags_33 },
    { 0x84586597, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4058659d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc218659f, SUC_lexicon_tags_134 },
    { 0x58b865a0, SUC_lexicon_tags_27 },
    { 0xc938659f, SUC_lexicon_tags_47 },
    { 0x539865a2, SUC_lexicon_tags_11 },
    { 0xc65865a1, SUC_lexicon_tags_15 },
    { 0xb99865a2, SUC_lexicon_tags_47 },
    { 0xfe3865a5, SUC_lexicon_tags_26 },
    { 0x4c9865a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d865ae, SUC_lexicon_tags_26 },
    { 0x5df865af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x457865b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f865b8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf865bd, SUC_lexicon_tags_37 },
    { 0xda3865be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77865c5, SUC_lexicon_tags_37 },
    { 0x679865c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee7865c8, SUC_lexicon_tags_33 },
    { 0x789865c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01865cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1865d4, SUC_lexicon_tags_23 },
    { 0xda5865d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d865e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc13865e4, SUC_lexicon_tags_21 },
    { 0x55d865e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x177865e7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6d9865e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3865ec, SUC_lexicon_tags_1 },
    { 0xe95865ed, SUC_lexicon_tags_2 },
    { 0xf81865ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9865f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663865f7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xced865f9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad865fd, SUC_lexicon_tags_25 },
    { 0x489865fd, SUC_lexicon_tags_9 },
    { 0xf39865fe, SUC_lexicon_tags_60 },
    { 0xee586600, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9986604, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b18660a, SUC_lexicon_tags_11 },
    { 0xa498660a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd238660d, SUC_lexicon_tags_37 },
    { 0x2998660d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf86611, SUC_lexicon_tags_7 },
    { 0x84986612, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad86618, SUC_lexicon_tags_12 },
    { 0x4b786619, SUC_lexicon_tags_25 },
    { 0x7b98661a, SUC_lexicon_tags_37 },
    { 0xb2b8661b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb838661d, SUC_lexicon_tags_23 },
    { 0x9238661e, SUC_lexicon_tags_17 },
    { 0xdd78661d, SUC_lexicon_tags_2 },
    { 0x99186620, SUC_lexicon_tags_17 },
    { 0xd0d86621, SUC_lexicon_tags_25 },
    { 0x82f86621, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef986626, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf238662b, SUC_lexicon_tags_10 },
    { 0x23f8662c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4186633, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63986638, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6278663a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xafb8663c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e58663e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d86641, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab86647, SUC_lexicon_tags_7 },
    { 0x5c386648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6238664a, SUC_lexicon_tags_6 },
    { 0xd2b8664b, SUC_lexicon_tags_11 },
    { 0x9ef8664b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xceb8664e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe7786650, SUC_lexicon_tags_36 },
    { 0x9b186651, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66186656, SUC_lexicon_tags_11 },
    { 0xe5d86657, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb0586659, SUC_lexicon_tags_60 },
    { 0x8ef86659, SUC_lexicon_tags_2 },
    { 0xfb78665b, SUC_lexicon_tags_25 },
    { 0xd4b8665c, SUC_lexicon_tags_4 },
    { 0x5558665d, SUC_lexicon_tags_8 },
    { 0x7758665b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77386660, SUC_lexicon_tags_7 },
    { 0xeeb86661, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af86664, SUC_lexicon_tags_7 },
    { 0x93d86665, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26d86667, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50186669, SUC_lexicon_tags_33 },
    { 0x6838666a, SUC_lexicon_tags_9 },
    { 0xf938666b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x4218666e, SUC_lexicon_tags_21 },
    { 0x2ff8666f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d86673, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacb86675, SUC_lexicon_tags_14 },
    { 0x3186676, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb8667a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86d8667c, SUC_lexicon_tags_2 },
    { 0x2f98667d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa678667f, SUC_lexicon_tags_11 },
    { 0xdb18667f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d86683, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9586688, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9838668e, SUC_lexicon_tags_46 },
    { 0x80f8668f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x21386692, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x99d86694, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaa786696, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f8669e, SUC_lexicon_tags_17 },
    { 0x51b8669f, SUC_lexicon_tags_66 },
    { 0xd138669f, SUC_lexicon_tags_29 },
    { 0x5bb866a1, SUC_lexicon_tags_2 },
    { 0xd0f866a2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133866a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa37866aa, SUC_lexicon_tags_7 },
    { 0xaa7866ab, SUC_lexicon_tags_31 },
    { 0x7c7866ac, SUC_lexicon_tags_25 },
    { 0x219866ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4a5866af, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xef3866b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1866b3, SUC_lexicon_tags_9 },
    { 0x40d866b3, SUC_lexicon_tags_33 },
    { 0xe3866b4, SUC_lexicon_tags_37 },
    { 0x1db866b6, SUC_lexicon_tags_17 },
    { 0x399866b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7866bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829866c6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d866cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51866cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x481866d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f9866d4, SUC_lexicon_tags_25 },
    { 0x10d866d5, SUC_lexicon_tags_17 },
    { 0xb83866d6, SUC_lexicon_tags_8 },
    { 0xfc7866d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f866da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21866e4, SUC_lexicon_tags_37 },
    { 0x9db866e5, SUC_lexicon_tags_4 },
    { 0x677866e6, SUC_lexicon_tags_45 },
    { 0x1d866e7, SUC_lexicon_tags_11 },
    { 0xaef866e5, SUC_lexicon_tags_14 },
    { 0xf9f866e9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x32d866eb, SUC_lexicon_tags_276 },
    { 0, NULL },
    { 0xc07866ed, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f866f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x187866f3, SUC_lexicon_tags_14 },
    { 0x9d3866f4, SUC_lexicon_tags_15 },
    { 0x9b7866f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291866f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f586700, SUC_lexicon_tags_33 },
    { 0xaf386701, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64186704, SUC_lexicon_tags_7 },
    { 0xcf86705, SUC_lexicon_tags_483 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3586709, SUC_lexicon_tags_17 },
    { 0x9378670a, SUC_lexicon_tags_27 },
    { 0x6bd8670b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd8670f, SUC_lexicon_tags_7 },
    { 0x7438670f, SUC_lexicon_tags_17 },
    { 0x91b86711, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db86718, SUC_lexicon_tags_116 },
    { 0xf7986718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1838671e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f386721, SUC_lexicon_tags_17 },
    { 0x23f86721, SUC_lexicon_tags_7 },
    { 0xfc786721, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x99b86725, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff786729, SUC_lexicon_tags_17 },
    { 0x9198672a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7b8672c, SUC_lexicon_tags_37 },
    { 0x5418672d, SUC_lexicon_tags_25 },
    { 0x9c58672e, SUC_lexicon_tags_60 },
    { 0xc058672e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0986736, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd8673a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdfb8673c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7898673e, SUC_lexicon_tags_9 },
    { 0x5e78673f, SUC_lexicon_tags_14 },
    { 0x48386740, SUC_lexicon_tags_2 },
    { 0x29986740, SUC_lexicon_tags_9 },
    { 0x1f586741, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b86747, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f58674a, SUC_lexicon_tags_2 },
    { 0x9998674b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd8674e, SUC_lexicon_tags_17 },
    { 0xf858674f, SUC_lexicon_tags_68 },
    { 0x6ff8674f, SUC_lexicon_tags_37 },
    { 0x31b86751, SUC_lexicon_tags_37 },
    { 0x2b586752, SUC_lexicon_tags_11 },
    { 0x18786752, SUC_lexicon_tags_33 },
    { 0xaaf86753, SUC_lexicon_tags_27 },
    { 0x43d86755, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36386759, SUC_lexicon_tags_25 },
    { 0x5d58675a, SUC_lexicon_tags_25 },
    { 0xa6386759, SUC_lexicon_tags_11 },
    { 0x3ed8675c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd98675e, SUC_lexicon_tags_15 },
    { 0x25d8675f, SUC_lexicon_tags_43 },
    { 0xae786760, SUC_lexicon_tags_43 },
    { 0xddf8675f, SUC_lexicon_tags_14 },
    { 0x4fb8675f, SUC_lexicon_tags_14 },
    { 0x52f86763, SUC_lexicon_tags_25 },
    { 0x29186764, SUC_lexicon_tags_27 },
    { 0x73386760, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21586769, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2386770, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b386774, SUC_lexicon_tags_15 },
    { 0xc8386774, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8c786777, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb318677a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74d8677c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa7d8677e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b386782, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65386788, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x1958678b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f86794, SUC_lexicon_tags_12 },
    { 0x7a386795, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df86798, SUC_lexicon_tags_82 },
    { 0x70d86799, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497867a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7867aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e9867ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb867af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfc1867b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd867b4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x85f867b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36d867b8, SUC_lexicon_tags_37 },
    { 0x619867b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257867bd, SUC_lexicon_tags_15 },
    { 0xaaf867bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x441867c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x607867c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x185867c7, SUC_lexicon_tags_11 },
    { 0x445867c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49867cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f867d3, SUC_lexicon_tags_37 },
    { 0x4db867d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff867d8, SUC_lexicon_tags_2 },
    { 0x54b867d8, SUC_lexicon_tags_63 },
    { 0x23d867d9, SUC_lexicon_tags_27 },
    { 0x285867db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3b867dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x393867df, SUC_lexicon_tags_0 },
    { 0x8f3867e0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb867ea, SUC_lexicon_tags_0 },
    { 0x501867eb, SUC_lexicon_tags_23 },
    { 0x6a3867ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109867f1, SUC_lexicon_tags_17 },
    { 0x5c1867f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x377867f4, SUC_lexicon_tags_15 },
    { 0x919867f5, SUC_lexicon_tags_14 },
    { 0x197867f5, SUC_lexicon_tags_33 },
    { 0x1b3867f7, SUC_lexicon_tags_44 },
    { 0x45d867f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3867fe, SUC_lexicon_tags_31 },
    { 0x9a1867ff, SUC_lexicon_tags_17 },
    { 0x107867ff, SUC_lexicon_tags_37 },
    { 0x42186801, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18586807, SUC_lexicon_tags_46 },
    { 0x73386808, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc978680a, SUC_lexicon_tags_32 },
    { 0x17d8680b, SUC_lexicon_tags_27 },
    { 0xb018680c, SUC_lexicon_tags_10 },
    { 0xbb98680b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7278680f, SUC_lexicon_tags_15 },
    { 0x2186810, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb86819, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa538681d, SUC_lexicon_tags_38 },
    { 0xb7d8681d, SUC_lexicon_tags_30 },
    { 0x4458681f, SUC_lexicon_tags_11 },
    { 0x39986820, SUC_lexicon_tags_12 },
    { 0x59d8681d, SUC_lexicon_tags_8 },
    { 0x3c586820, SUC_lexicon_tags_12 },
    { 0x16186823, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf86825, SUC_lexicon_tags_82 },
    { 0x49f86825, SUC_lexicon_tags_11 },
    { 0xbb586827, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d8682a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6738682d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5478683a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf8683f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52186844, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe1986846, SUC_lexicon_tags_38 },
    { 0x40186847, SUC_lexicon_tags_27 },
    { 0xe7786848, SUC_lexicon_tags_18 },
    { 0xf4786849, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd018684c, SUC_lexicon_tags_20 },
    { 0xeef8684d, SUC_lexicon_tags_25 },
    { 0xb3f8684d, SUC_lexicon_tags_25 },
    { 0xf6d8684c, SUC_lexicon_tags_2 },
    { 0x7b586850, SUC_lexicon_tags_23 },
    { 0xcef86850, SUC_lexicon_tags_45 },
    { 0x18d8684e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88786854, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc38685b, SUC_lexicon_tags_23 },
    { 0x5a38685c, SUC_lexicon_tags_2 },
    { 0x78b8685d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4658685f, SUC_lexicon_tags_25 },
    { 0xd3186860, SUC_lexicon_tags_38 },
    { 0x7fd8685f, SUC_lexicon_tags_37 },
    { 0xa7d86862, SUC_lexicon_tags_0 },
    { 0x3ad86861, SUC_lexicon_tags_191 },
    { 0xf4386864, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8c386866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe98686c, SUC_lexicon_tags_37 },
    { 0x5f8686d, SUC_lexicon_tags_1 },
    { 0xcf8686c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe186871, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc386873, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1af86875, SUC_lexicon_tags_26 },
    { 0x70386876, SUC_lexicon_tags_7 },
    { 0xc8d86876, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6558687c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcdd8687e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35986888, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15f8688a, SUC_lexicon_tags_63 },
    { 0x33d8688b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2478688d, SUC_lexicon_tags_65 },
    { 0xeb98688e, SUC_lexicon_tags_17 },
    { 0x1b98688d, SUC_lexicon_tags_82 },
    { 0xdd58688e, SUC_lexicon_tags_53 },
    { 0xdf86890, SUC_lexicon_tags_61 },
    { 0xf6d8688e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c186897, SUC_lexicon_tags_8 },
    { 0x4a386898, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5858689c, SUC_lexicon_tags_0 },
    { 0x5c78689c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x599868a0, SUC_lexicon_tags_15 },
    { 0x177868a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeaf868a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95868aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3868ad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57868b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5b868b2, SUC_lexicon_tags_1 },
    { 0x417868b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f868b7, SUC_lexicon_tags_70 },
    { 0x365868b8, SUC_lexicon_tags_0 },
    { 0x99b868b8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83868bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xed5868be, SUC_lexicon_tags_12 },
    { 0x1eb868be, SUC_lexicon_tags_30 },
    { 0x169868c0, SUC_lexicon_tags_15 },
    { 0x9c7868c1, SUC_lexicon_tags_45 },
    { 0xfc7868c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x233868c4, SUC_lexicon_tags_23 },
    { 0x313868c4, SUC_lexicon_tags_27 },
    { 0x3db868c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc1868c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9868cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5f868ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d868d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x177868d8, SUC_lexicon_tags_38 },
    { 0x1c9868d9, SUC_lexicon_tags_31 },
    { 0x3ad868da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x853868dc, SUC_lexicon_tags_55 },
    { 0x7e5868dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7868e4, SUC_lexicon_tags_87 },
    { 0x545868e5, SUC_lexicon_tags_63 },
    { 0xca9868e5, SUC_lexicon_tags_27 },
    { 0xaf9868e7, SUC_lexicon_tags_25 },
    { 0x327868e7, SUC_lexicon_tags_25 },
    { 0x63f868e9, SUC_lexicon_tags_27 },
    { 0xeef868ea, SUC_lexicon_tags_12 },
    { 0x19b868eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf99868ed, SUC_lexicon_tags_68 },
    { 0x9fb868ee, SUC_lexicon_tags_26 },
    { 0xc93868ef, SUC_lexicon_tags_7 },
    { 0x6a3868f0, SUC_lexicon_tags_2 },
    { 0xbed868ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe89868f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1868f9, SUC_lexicon_tags_37 },
    { 0x73b868fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173868fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfd86900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf86903, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a986905, SUC_lexicon_tags_14 },
    { 0x3b386906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f8690c, SUC_lexicon_tags_65 },
    { 0x1cb8690d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc38690f, SUC_lexicon_tags_27 },
    { 0xe4f86910, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28186916, SUC_lexicon_tags_17 },
    { 0x54186917, SUC_lexicon_tags_26 },
    { 0xcf586916, SUC_lexicon_tags_49 },
    { 0x9ff86918, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb9f8691b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd678691f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b986924, SUC_lexicon_tags_4 },
    { 0x40586925, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad86929, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4018692c, SUC_lexicon_tags_1 },
    { 0x76d8692c, SUC_lexicon_tags_0 },
    { 0xd7d8692d, SUC_lexicon_tags_23 },
    { 0x5d58692f, SUC_lexicon_tags_37 },
    { 0x36f86930, SUC_lexicon_tags_49 },
    { 0x8378692f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b186935, SUC_lexicon_tags_49 },
    { 0x5ff86936, SUC_lexicon_tags_1 },
    { 0xd9786937, SUC_lexicon_tags_31 },
    { 0x45786935, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb18693a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72786943, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d86947, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f8694c, SUC_lexicon_tags_47 },
    { 0x8d38694d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5786951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96586953, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26186956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82b86958, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8118695c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2586961, SUC_lexicon_tags_7 },
    { 0xf0b86962, SUC_lexicon_tags_25 },
    { 0xb0f86961, SUC_lexicon_tags_37 },
    { 0x38d86964, SUC_lexicon_tags_15 },
    { 0xca986965, SUC_lexicon_tags_40 },
    { 0x4cb86965, SUC_lexicon_tags_37 },
    { 0xfa786964, SUC_lexicon_tags_7 },
    { 0x26b86968, SUC_lexicon_tags_11 },
    { 0x1b386969, SUC_lexicon_tags_65 },
    { 0xc1986968, SUC_lexicon_tags_25 },
    { 0xdcf8696b, SUC_lexicon_tags_11 },
    { 0x3f8696c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22186970, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x69786972, SUC_lexicon_tags_7 },
    { 0x1d186973, SUC_lexicon_tags_63 },
    { 0x59b86974, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf86978, SUC_lexicon_tags_6 },
    { 0xb3986979, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8818697c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70786982, SUC_lexicon_tags_25 },
    { 0xd186983, SUC_lexicon_tags_77 },
    { 0x22986983, SUC_lexicon_tags_37 },
    { 0xde386984, SUC_lexicon_tags_28 },
    { 0x58f86984, SUC_lexicon_tags_15 },
    { 0x2b386987, SUC_lexicon_tags_11 },
    { 0x1d386987, SUC_lexicon_tags_7 },
    { 0xa9b86988, SUC_lexicon_tags_17 },
    { 0x986984, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3578698c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4058698e, SUC_lexicon_tags_10 },
    { 0x11f8698f, SUC_lexicon_tags_25 },
    { 0x9158698f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f586997, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe786999, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd18699c, SUC_lexicon_tags_28 },
    { 0x5398699d, SUC_lexicon_tags_14 },
    { 0xd638699e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9869a1, SUC_lexicon_tags_26 },
    { 0xbb3869a1, SUC_lexicon_tags_15 },
    { 0xb99869a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b869a9, SUC_lexicon_tags_11 },
    { 0xb93869aa, SUC_lexicon_tags_11 },
    { 0xb2b869ab, SUC_lexicon_tags_42 },
    { 0xf53869aa, SUC_lexicon_tags_38 },
    { 0x105869ac, SUC_lexicon_tags_7 },
    { 0xa21869ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x21d869b0, SUC_lexicon_tags_15 },
    { 0x2d7869b0, SUC_lexicon_tags_37 },
    { 0x18f869b1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1869b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b869bc, SUC_lexicon_tags_15 },
    { 0x19f869bd, SUC_lexicon_tags_7 },
    { 0xce3869bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x37b869c0, SUC_lexicon_tags_65 },
    { 0xae3869c1, SUC_lexicon_tags_25 },
    { 0x393869c1, SUC_lexicon_tags_47 },
    { 0xc8f869c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6f7869c5, SUC_lexicon_tags_25 },
    { 0x71b869c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9b869c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb869cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f869d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45869d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x215869dc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x951869de, SUC_lexicon_tags_37 },
    { 0x31869de, SUC_lexicon_tags_11 },
    { 0x599869de, SUC_lexicon_tags_45 },
    { 0x609869de, SUC_lexicon_tags_8 },
    { 0xd1869df, SUC_lexicon_tags_740 },
    { 0x853869e3, SUC_lexicon_tags_23 },
    { 0x78b869e4, SUC_lexicon_tags_4 },
    { 0x9bf869e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11b869e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d869eb, SUC_lexicon_tags_11 },
    { 0x78b869ec, SUC_lexicon_tags_63 },
    { 0x32b869ec, SUC_lexicon_tags_26 },
    { 0xff9869ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1d869f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x173869f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259869f7, SUC_lexicon_tags_4 },
    { 0x291869f8, SUC_lexicon_tags_27 },
    { 0xe47869f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x42b869fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67386a02, SUC_lexicon_tags_10 },
    { 0xb9986a03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5586a08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd86a0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e186a11, SUC_lexicon_tags_57 },
    { 0x4a986a12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56f86a15, SUC_lexicon_tags_17 },
    { 0x1ff86a16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83786a18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb86a20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51786a23, SUC_lexicon_tags_24 },
    { 0x6f86a23, SUC_lexicon_tags_55 },
    { 0xa786a24, SUC_lexicon_tags_229 },
    { 0xd3d86a26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25986a2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51786a36, SUC_lexicon_tags_45 },
    { 0xf0386a36, SUC_lexicon_tags_31 },
    { 0x31586a38, SUC_lexicon_tags_39 },
    { 0x44986a39, SUC_lexicon_tags_63 },
    { 0xf1d86a3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7186a3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e786a40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f786a46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdeb86a48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf86a4b, SUC_lexicon_tags_14 },
    { 0x4cd86a4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x93186a4e, SUC_lexicon_tags_7 },
    { 0x15b86a4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb86a56, SUC_lexicon_tags_25 },
    { 0xbc786a57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65586a5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8586a61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b86a65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8386a67, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe7586a69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x33186a6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed86a6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d86a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf86a75, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x67d86a77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xee186a79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70386a7b, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b86a7f, SUC_lexicon_tags_12 },
    { 0xd5386a80, SUC_lexicon_tags_49 },
    { 0x85586a81, SUC_lexicon_tags_11 },
    { 0x1d186a82, SUC_lexicon_tags_63 },
    { 0x8e786a83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3386a85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x19386a88, SUC_lexicon_tags_47 },
    { 0xd7586a89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6786a8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4386a8e, SUC_lexicon_tags_17 },
    { 0xdcd86a8e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85586a97, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb386a9b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d86aa0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb86aa8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d86aae, SUC_lexicon_tags_14 },
    { 0x29f86aaf, SUC_lexicon_tags_60 },
    { 0x1bb86ab0, SUC_lexicon_tags_47 },
    { 0xde786ab1, SUC_lexicon_tags_46 },
    { 0x41d86ab2, SUC_lexicon_tags_25 },
    { 0x28786ab3, SUC_lexicon_tags_23 },
    { 0xf4786ab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55786ab7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x78186ab9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f386abf, SUC_lexicon_tags_60 },
    { 0x11986ac0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd186ac2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f86ac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5586ace, SUC_lexicon_tags_14 },
    { 0xc3586ace, SUC_lexicon_tags_49 },
    { 0xca186acf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27786ad2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab86ad7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4986add, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x55f86adf, SUC_lexicon_tags_61 },
    { 0xaab86adf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d86ae4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5186aee, SUC_lexicon_tags_47 },
    { 0x4cf86aef, SUC_lexicon_tags_26 },
    { 0x8ab86aef, SUC_lexicon_tags_31 },
    { 0x7ff86af0, SUC_lexicon_tags_17 },
    { 0x22f86af2, SUC_lexicon_tags_2 },
    { 0xa3986af3, SUC_lexicon_tags_37 },
    { 0xfc386af4, SUC_lexicon_tags_252 },
    { 0xce786af5, SUC_lexicon_tags_8 },
    { 0x23b86af5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4786afb, SUC_lexicon_tags_68 },
    { 0x40b86afc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5186aff, SUC_lexicon_tags_23 },
    { 0x81586b00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xddf86b02, SUC_lexicon_tags_37 },
    { 0xb3186b03, SUC_lexicon_tags_12 },
    { 0xe8386b04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d86b07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1186b0c, SUC_lexicon_tags_87 },
    { 0x22586b0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2b86b0f, SUC_lexicon_tags_37 },
    { 0x45786b10, SUC_lexicon_tags_12 },
    { 0xa1386b11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df86b14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b386b16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81786b21, SUC_lexicon_tags_48 },
    { 0xc6986b21, SUC_lexicon_tags_0 },
    { 0x50586b23, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f86b26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84b86b28, SUC_lexicon_tags_15 },
    { 0x3f986b28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f86b2d, SUC_lexicon_tags_39 },
    { 0x97786b2e, SUC_lexicon_tags_12 },
    { 0x7d986b2f, SUC_lexicon_tags_2 },
    { 0x83f86b2e, SUC_lexicon_tags_113 },
    { 0x5786b31, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc7586b33, SUC_lexicon_tags_8 },
    { 0x78b86b34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb86b37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c586b3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1af86b3c, SUC_lexicon_tags_68 },
    { 0x10786b3d, SUC_lexicon_tags_8 },
    { 0x6cf86b3e, SUC_lexicon_tags_23 },
    { 0x9db86b3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda786b43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc986b45, SUC_lexicon_tags_12 },
    { 0x22386b45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb586b48, SUC_lexicon_tags_60 },
    { 0xcd86b49, SUC_lexicon_tags_47 },
    { 0xc9186b4a, SUC_lexicon_tags_7 },
    { 0x42986b48, SUC_lexicon_tags_25 },
    { 0xc1986b4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6586b53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1386b58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f86b5d, SUC_lexicon_tags_53 },
    { 0x2e186b5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad986b60, SUC_lexicon_tags_19 },
    { 0x5dd86b60, SUC_lexicon_tags_47 },
    { 0xbe386b61, SUC_lexicon_tags_19 },
    { 0xa8186b63, SUC_lexicon_tags_1 },
    { 0x92386b60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4786b6a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f786b6e, SUC_lexicon_tags_15 },
    { 0xb7386b6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f86b75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb86b7a, SUC_lexicon_tags_7 },
    { 0xd0d86b7a, SUC_lexicon_tags_17 },
    { 0x55f86b7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d86b7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8b86b81, SUC_lexicon_tags_19 },
    { 0xc5d86b81, SUC_lexicon_tags_15 },
    { 0x39186b83, SUC_lexicon_tags_12 },
    { 0xf1d86b84, SUC_lexicon_tags_47 },
    { 0x60f86b85, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x1af86b87, SUC_lexicon_tags_7 },
    { 0xf7586b88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b86b8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5586b8e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8186b95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d86b9b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa7f86b9d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55186ba4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97b86ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a386bae, SUC_lexicon_tags_25 },
    { 0xd4b86baf, SUC_lexicon_tags_47 },
    { 0xe3586bb0, SUC_lexicon_tags_37 },
    { 0x8cb86bb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14586bb3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f86bb6, SUC_lexicon_tags_12 },
    { 0xf3786bb7, SUC_lexicon_tags_45 },
    { 0x50186bb8, SUC_lexicon_tags_23 },
    { 0xff386bb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb186bbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd86bbf, SUC_lexicon_tags_49 },
    { 0xaf186bbf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f586bc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a986bc7, SUC_lexicon_tags_2 },
    { 0xd7186bc7, SUC_lexicon_tags_11 },
    { 0xbcd86bc9, SUC_lexicon_tags_12 },
    { 0x6fd86bca, SUC_lexicon_tags_68 },
    { 0x53d86bcb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a786bce, SUC_lexicon_tags_9 },
    { 0x73586bce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd186bdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0786bdf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaf86be1, SUC_lexicon_tags_21 },
    { 0x52986be1, SUC_lexicon_tags_63 },
    { 0xe186be2, SUC_lexicon_tags_6 },
    { 0x49b86be3, SUC_lexicon_tags_4 },
    { 0xf8b86be5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d86be8, SUC_lexicon_tags_2 },
    { 0xe2b86be8, SUC_lexicon_tags_1 },
    { 0x11d86bea, SUC_lexicon_tags_25 },
    { 0xbb786beb, SUC_lexicon_tags_61 },
    { 0x90786bec, SUC_lexicon_tags_64 },
    { 0x93586be8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x14986bef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49386bf2, SUC_lexicon_tags_12 },
    { 0x56386bf3, SUC_lexicon_tags_37 },
    { 0x87b86bf3, SUC_lexicon_tags_42 },
    { 0xcc186bf5, SUC_lexicon_tags_11 },
    { 0x6d586bf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3386bf9, SUC_lexicon_tags_25 },
    { 0xced86bfa, SUC_lexicon_tags_11 },
    { 0xbdf86bfb, SUC_lexicon_tags_49 },
    { 0x79b86bfa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6f86bfe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5af86c00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f86c0b, SUC_lexicon_tags_26 },
    { 0x1b586c0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa786c10, SUC_lexicon_tags_11 },
    { 0x40986c11, SUC_lexicon_tags_15 },
    { 0x13386c11, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3dd86c14, SUC_lexicon_tags_1 },
    { 0xd5b86c15, SUC_lexicon_tags_12 },
    { 0x3cb86c14, SUC_lexicon_tags_29 },
    { 0x9cd86c17, SUC_lexicon_tags_55 },
    { 0xb2d86c14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5186c1c, SUC_lexicon_tags_25 },
    { 0x25786c1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf86c23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba186c25, SUC_lexicon_tags_11 },
    { 0x35386c25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9b86c28, SUC_lexicon_tags_46 },
    { 0x3cf86c28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb186c2b, SUC_lexicon_tags_82 },
    { 0x56b86c2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x76386c2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b86c32, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdbf86c34, SUC_lexicon_tags_17 },
    { 0x7df86c34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f986c37, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13786c42, SUC_lexicon_tags_2 },
    { 0x85186c42, SUC_lexicon_tags_35 },
    { 0x66586c44, SUC_lexicon_tags_17 },
    { 0x87f86c42, SUC_lexicon_tags_54 },
    { 0xf6b86c46, SUC_lexicon_tags_26 },
    { 0xfa186c47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5786c4a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x40986c4d, SUC_lexicon_tags_45 },
    { 0xe1f86c4d, SUC_lexicon_tags_37 },
    { 0x71786c4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x95386c52, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff86c57, SUC_lexicon_tags_11 },
    { 0x97b86c57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97186c5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf786c60, SUC_lexicon_tags_12 },
    { 0x6b986c61, SUC_lexicon_tags_63 },
    { 0x2e386c61, SUC_lexicon_tags_271 },
    { 0xfa186c62, SUC_lexicon_tags_5 },
    { 0x8ad86c62, SUC_lexicon_tags_2 },
    { 0xc386c65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4986c6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f86c6d, SUC_lexicon_tags_30 },
    { 0xaab86c6e, SUC_lexicon_tags_17 },
    { 0xa6786c6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e586c72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75186c7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f86c83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x55d86c85, SUC_lexicon_tags_12 },
    { 0x41f86c86, SUC_lexicon_tags_8 },
    { 0xacf86c87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb86c8f, SUC_lexicon_tags_7 },
    { 0x44b86c8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeff86c92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8986c94, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f786c99, SUC_lexicon_tags_25 },
    { 0x29d86c9a, SUC_lexicon_tags_30 },
    { 0x29586c9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7586c9f, SUC_lexicon_tags_7 },
    { 0xcc586c9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b386ca3, SUC_lexicon_tags_37 },
    { 0x95786ca3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af86caa, SUC_lexicon_tags_7 },
    { 0xb7b86cab, SUC_lexicon_tags_53 },
    { 0xa4986cac, SUC_lexicon_tags_179 },
    { 0x4ab86cad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x68d86caf, SUC_lexicon_tags_11 },
    { 0xfe586cb0, SUC_lexicon_tags_7 },
    { 0x94f86caf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd86cb5, SUC_lexicon_tags_33 },
    { 0xd3986cb6, SUC_lexicon_tags_60 },
    { 0x73d86cb7, SUC_lexicon_tags_1 },
    { 0x12986cb7, SUC_lexicon_tags_37 },
    { 0xac186cb9, SUC_lexicon_tags_12 },
    { 0xb3986cb6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff86cc0, SUC_lexicon_tags_7 },
    { 0x70f86cc1, SUC_lexicon_tags_17 },
    { 0x3e786cc2, SUC_lexicon_tags_23 },
    { 0xad986cc3, SUC_lexicon_tags_1 },
    { 0xf6b86cc4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa2b86cc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabb86cc8, SUC_lexicon_tags_25 },
    { 0x2fd86cc9, SUC_lexicon_tags_60 },
    { 0x9586cca, SUC_lexicon_tags_583 },
    { 0xc986ccb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46386ccf, SUC_lexicon_tags_2 },
    { 0xf7f86ccf, SUC_lexicon_tags_2 },
    { 0x58186cd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9986cda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10986cde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26586ce5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x27986ce8, SUC_lexicon_tags_37 },
    { 0xbb586ce9, SUC_lexicon_tags_19 },
    { 0xa786cea, SUC_lexicon_tags_1 },
    { 0x6ed86ceb, SUC_lexicon_tags_27 },
    { 0xc6186cec, SUC_lexicon_tags_3 },
    { 0xbe586ced, SUC_lexicon_tags_23 },
    { 0xe0d86cee, SUC_lexicon_tags_17 },
    { 0x24786cef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a786cf1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbcf86cf3, SUC_lexicon_tags_11 },
    { 0x9d986cf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2f86cf6, SUC_lexicon_tags_8 },
    { 0xc8d86cf7, SUC_lexicon_tags_60 },
    { 0xd8f86cf8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb1786cfa, SUC_lexicon_tags_19 },
    { 0x1ad86cfa, SUC_lexicon_tags_37 },
    { 0x14d86cfc, SUC_lexicon_tags_27 },
    { 0x52186cfa, SUC_lexicon_tags_11 },
    { 0xe9b86cfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d186d01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f86d04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4786d06, SUC_lexicon_tags_25 },
    { 0x60b86d06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27786d09, SUC_lexicon_tags_17 },
    { 0xdef86d0a, SUC_lexicon_tags_15 },
    { 0x93786d09, SUC_lexicon_tags_19 },
    { 0x9c186d0c, SUC_lexicon_tags_27 },
    { 0x18d86d0b, SUC_lexicon_tags_17 },
    { 0xc2f86d0e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b86d14, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4bb86d16, SUC_lexicon_tags_79 },
    { 0xfef86d16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5dd86d19, SUC_lexicon_tags_2 },
    { 0x53b86d1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b86d1e, SUC_lexicon_tags_26 },
    { 0xac986d1f, SUC_lexicon_tags_27 },
    { 0x34986d1e, SUC_lexicon_tags_0 },
    { 0x29386d21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f86d28, SUC_lexicon_tags_15 },
    { 0x2e186d29, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa586d2c, SUC_lexicon_tags_11 },
    { 0xa9786d2c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5186d30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0586d35, SUC_lexicon_tags_1 },
    { 0x8f786d36, SUC_lexicon_tags_63 },
    { 0x10f86d37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x23386d39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88786d3e, SUC_lexicon_tags_45 },
    { 0x4b786d3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b86d43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ef86d45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b86d4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0186d50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70f86d52, SUC_lexicon_tags_23 },
    { 0xe2186d53, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe786d58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2386d5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b586d5e, SUC_lexicon_tags_3 },
    { 0x29586d5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b86d63, SUC_lexicon_tags_16 },
    { 0x80786d63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f86d68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd0986d6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x44386d6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7786d6f, SUC_lexicon_tags_37 },
    { 0x4186d70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92d86d73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7786d77, SUC_lexicon_tags_7 },
    { 0x97386d77, SUC_lexicon_tags_12 },
    { 0x13386d77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b86d7c, SUC_lexicon_tags_47 },
    { 0x39186d7d, SUC_lexicon_tags_0 },
    { 0x70f86d7e, SUC_lexicon_tags_38 },
    { 0x9c986d7e, SUC_lexicon_tags_47 },
    { 0x2f986d7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f986d83, SUC_lexicon_tags_133 },
    { 0x4c186d84, SUC_lexicon_tags_11 },
    { 0x76786d85, SUC_lexicon_tags_7 },
    { 0x2e586d84, SUC_lexicon_tags_16 },
    { 0x9ad86d87, SUC_lexicon_tags_30 },
    { 0x3f186d86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d86d8d, SUC_lexicon_tags_44 },
    { 0x3df86d8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b86d91, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d86d97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef86d9a, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xcbf86d9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf86d9f, SUC_lexicon_tags_25 },
    { 0xa7f86da0, SUC_lexicon_tags_87 },
    { 0xaff86da1, SUC_lexicon_tags_48 },
    { 0xd4386da1, SUC_lexicon_tags_29 },
    { 0x9f786da2, SUC_lexicon_tags_14 },
    { 0x54f86da4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb86da8, SUC_lexicon_tags_37 },
    { 0xe7d86da9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xde786dac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f86daf, SUC_lexicon_tags_77 },
    { 0x89786db0, SUC_lexicon_tags_23 },
    { 0x4bb86db1, SUC_lexicon_tags_27 },
    { 0x5a786db0, SUC_lexicon_tags_12 },
    { 0xa9786db2, SUC_lexicon_tags_87 },
    { 0x5b86db0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83186db6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd86db9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x46b86dbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcad86dbd, SUC_lexicon_tags_1 },
    { 0x41f86dbe, SUC_lexicon_tags_37 },
    { 0xc5786dbf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd186dc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x65786dc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8986dcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f86dda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9786ddc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d86de0, SUC_lexicon_tags_68 },
    { 0x56986de1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb1786de3, SUC_lexicon_tags_63 },
    { 0x8b786de3, SUC_lexicon_tags_20 },
    { 0xad986de5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcd86de7, SUC_lexicon_tags_17 },
    { 0x1d186de8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8986ded, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85586df2, SUC_lexicon_tags_63 },
    { 0xc9186df3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcab86df5, SUC_lexicon_tags_38 },
    { 0xe0986df6, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb86dfb, SUC_lexicon_tags_1 },
    { 0xb5386dfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x13786dff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d86e05, SUC_lexicon_tags_7 },
    { 0xbf386e06, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4bb86e08, SUC_lexicon_tags_23 },
    { 0xe2d86e08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5986e0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0386e12, SUC_lexicon_tags_12 },
    { 0x4fb86e13, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x81186e15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7f86e17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b86e1f, SUC_lexicon_tags_38 },
    { 0x87386e20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd86e23, SUC_lexicon_tags_37 },
    { 0x74586e24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb186e27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b86e2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b386e2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24586e31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f586e34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2586e36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b86e3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe586e3e, SUC_lexicon_tags_17 },
    { 0xf3786e3f, SUC_lexicon_tags_30 },
    { 0x1c986e40, SUC_lexicon_tags_11 },
    { 0xd86e40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a986e43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19b86e45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf3586e47, SUC_lexicon_tags_11 },
    { 0xb1786e48, SUC_lexicon_tags_26 },
    { 0xd2786e47, SUC_lexicon_tags_17 },
    { 0x82386e4a, SUC_lexicon_tags_21 },
    { 0xe2586e48, SUC_lexicon_tags_15 },
    { 0x56786e4c, SUC_lexicon_tags_19 },
    { 0x98b86e4c, SUC_lexicon_tags_25 },
    { 0xf2586e4e, SUC_lexicon_tags_87 },
    { 0xe0586e4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f86e54, SUC_lexicon_tags_7 },
    { 0x58b86e55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f186e5c, SUC_lexicon_tags_38 },
    { 0xf8186e5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93986e65, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0986e68, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81386e6c, SUC_lexicon_tags_25 },
    { 0x41786e6c, SUC_lexicon_tags_17 },
    { 0x55b86e6d, SUC_lexicon_tags_25 },
    { 0xc7186e6c, SUC_lexicon_tags_25 },
    { 0xc7986e70, SUC_lexicon_tags_30 },
    { 0x50d86e6c, SUC_lexicon_tags_2 },
    { 0xe4f86e72, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x36186e74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5386e7e, SUC_lexicon_tags_60 },
    { 0xb0f86e7e, SUC_lexicon_tags_37 },
    { 0x3e186e7f, SUC_lexicon_tags_959 },
    { 0x22586e81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b586e84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b86e88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76d86e8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8e186e8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f86e91, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b86e95, SUC_lexicon_tags_17 },
    { 0xcb186e96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e186e99, SUC_lexicon_tags_25 },
    { 0x68f86e9a, SUC_lexicon_tags_37 },
    { 0xff86e9a, SUC_lexicon_tags_17 },
    { 0xd7b86e9c, SUC_lexicon_tags_0 },
    { 0xea586e9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d86ea1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee186ea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4186ea7, SUC_lexicon_tags_118 },
    { 0xa0f86ea7, SUC_lexicon_tags_14 },
    { 0xb86ea9, SUC_lexicon_tags_2 },
    { 0x83d86ea7, SUC_lexicon_tags_19 },
    { 0xefd86eab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf86eb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x66986eb2, SUC_lexicon_tags_9 },
    { 0xbf186eb2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf986eb6, SUC_lexicon_tags_37 },
    { 0xad86eb6, SUC_lexicon_tags_48 },
    { 0x72986eb8, SUC_lexicon_tags_49 },
    { 0x9d86eb8, SUC_lexicon_tags_14 },
    { 0x9bb86eb9, SUC_lexicon_tags_60 },
    { 0xc9586ebb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8eb86ebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28186ebf, SUC_lexicon_tags_37 },
    { 0x6a186ebf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f86ec7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xce586ec9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95786ecd, SUC_lexicon_tags_1 },
    { 0xd1986ece, SUC_lexicon_tags_42 },
    { 0xcab86ecf, SUC_lexicon_tags_1 },
    { 0x4b186ed0, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4386ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2df86ed7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f386edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c386ede, SUC_lexicon_tags_25 },
    { 0x8c786edf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76386ee1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20386eed, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc0386eef, SUC_lexicon_tags_25 },
    { 0x1af86ef0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3386ef3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xff586ef5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d86efb, SUC_lexicon_tags_25 },
    { 0x3c386efc, SUC_lexicon_tags_29 },
    { 0xef386efc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2386eff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x41b86f01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90786f03, SUC_lexicon_tags_33 },
    { 0xa1d86f03, SUC_lexicon_tags_25 },
    { 0xf9786f03, SUC_lexicon_tags_2 },
    { 0xb6986f06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2786f0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8786f0e, SUC_lexicon_tags_0 },
    { 0x15d86f0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98586f11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3586f13, SUC_lexicon_tags_0 },
    { 0x89d86f14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd4186f16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd86f1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d86f21, SUC_lexicon_tags_17 },
    { 0x1eb86f22, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f86f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6af86f28, SUC_lexicon_tags_6 },
    { 0xbd186f29, SUC_lexicon_tags_34 },
    { 0x5d386f2a, SUC_lexicon_tags_27 },
    { 0xd0786f2b, SUC_lexicon_tags_26 },
    { 0x2fd86f2c, SUC_lexicon_tags_15 },
    { 0xcbb86f28, SUC_lexicon_tags_136 },
    { 0xed986f2e, SUC_lexicon_tags_43 },
    { 0xc6786f2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b86f33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c586f35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3986f38, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8386f3a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8ad86f3c, SUC_lexicon_tags_17 },
    { 0x30d86f3d, SUC_lexicon_tags_38 },
    { 0x71986f3c, SUC_lexicon_tags_45 },
    { 0xc6d86f3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x38386f41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf86f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d86f4b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x94786f4d, SUC_lexicon_tags_55 },
    { 0x7a186f4d, SUC_lexicon_tags_60 },
    { 0x3d986f4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e786f59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e386f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8386f61, SUC_lexicon_tags_60 },
    { 0x5d86f62, SUC_lexicon_tags_47 },
    { 0x2f786f62, SUC_lexicon_tags_37 },
    { 0xa6786f63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1186f68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f586f6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe8186f6c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f86f70, SUC_lexicon_tags_17 },
    { 0x6bf86f70, SUC_lexicon_tags_2 },
    { 0xd0786f72, SUC_lexicon_tags_1 },
    { 0x8db86f70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc586f79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff386f7b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc786f80, SUC_lexicon_tags_15 },
    { 0xf4586f81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52186f85, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x91386f88, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad386f8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2786f8c, SUC_lexicon_tags_12 },
    { 0x8fb86f8c, SUC_lexicon_tags_68 },
    { 0xa4b86f8e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0986f92, SUC_lexicon_tags_17 },
    { 0xcb986f93, SUC_lexicon_tags_37 },
    { 0x8e986f94, SUC_lexicon_tags_11 },
    { 0x10986f95, SUC_lexicon_tags_37 },
    { 0xd9186f94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3986f9c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9786f9e, SUC_lexicon_tags_12 },
    { 0x45186f9e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x61586fa1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d86fa5, SUC_lexicon_tags_37 },
    { 0xaab86fa6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8d786fa8, SUC_lexicon_tags_6 },
    { 0xaf786fa9, SUC_lexicon_tags_47 },
    { 0xdcf86fa8, SUC_lexicon_tags_15 },
    { 0x68d86fa8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb586fae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb786fb1, SUC_lexicon_tags_62 },
    { 0x1cd86fb2, SUC_lexicon_tags_23 },
    { 0xfbd86fb1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x87186fb5, SUC_lexicon_tags_33 },
    { 0x3e986fb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed86fb9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b86fbf, SUC_lexicon_tags_4 },
    { 0x6e186fbf, SUC_lexicon_tags_46 },
    { 0xfbb86fc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb586fc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb786fc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed86fcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa186fd1, SUC_lexicon_tags_2 },
    { 0x9bb86fd1, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb86fd8, SUC_lexicon_tags_37 },
    { 0xd8386fd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcfb86fdb, SUC_lexicon_tags_35 },
    { 0x52986fdb, SUC_lexicon_tags_47 },
    { 0x72b86fdb, SUC_lexicon_tags_79 },
    { 0x7b586fdd, SUC_lexicon_tags_0 },
    { 0x2bf86fdb, SUC_lexicon_tags_25 },
    { 0x6d986fe0, SUC_lexicon_tags_14 },
    { 0x2b86fe0, SUC_lexicon_tags_32 },
    { 0x7a586fe1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2586fe7, SUC_lexicon_tags_15 },
    { 0xbd186fe7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b86feb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d86ff4, SUC_lexicon_tags_71 },
    { 0x5bd86ff4, SUC_lexicon_tags_31 },
    { 0x3f86ff5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc186ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f86fff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b87006, SUC_lexicon_tags_17 },
    { 0xa9587007, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e58700b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d8700e, SUC_lexicon_tags_29 },
    { 0xbd8700f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb5987011, SUC_lexicon_tags_11 },
    { 0xac987012, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79587018, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3398701a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa918701c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57d8701e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87987022, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f87026, SUC_lexicon_tags_32 },
    { 0xae587026, SUC_lexicon_tags_12 },
    { 0xa3187028, SUC_lexicon_tags_2 },
    { 0x81b87029, SUC_lexicon_tags_2 },
    { 0x57f87026, SUC_lexicon_tags_27 },
    { 0xe218702b, SUC_lexicon_tags_65 },
    { 0x1298702c, SUC_lexicon_tags_63 },
    { 0xbbd8702d, SUC_lexicon_tags_2 },
    { 0xdb18702b, SUC_lexicon_tags_12 },
    { 0x2a98702f, SUC_lexicon_tags_2 },
    { 0x8e587030, SUC_lexicon_tags_10 },
    { 0x2df87030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd87038, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb58703b, SUC_lexicon_tags_30 },
    { 0xc6f8703b, SUC_lexicon_tags_37 },
    { 0x3878703b, SUC_lexicon_tags_2 },
    { 0x8c78703e, SUC_lexicon_tags_11 },
    { 0x5c58703e, SUC_lexicon_tags_7 },
    { 0x28987040, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2387042, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3987044, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e587046, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x92187049, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d8704c, SUC_lexicon_tags_25 },
    { 0x6738704c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f87050, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f587053, SUC_lexicon_tags_12 },
    { 0x10f87053, SUC_lexicon_tags_21 },
    { 0x9a387053, SUC_lexicon_tags_27 },
    { 0x5ff87056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9787058, SUC_lexicon_tags_37 },
    { 0xe6f87058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9978705f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb87063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b187065, SUC_lexicon_tags_45 },
    { 0x8d787066, SUC_lexicon_tags_25 },
    { 0x61787067, SUC_lexicon_tags_33 },
    { 0x9c987066, SUC_lexicon_tags_11 },
    { 0xe2987068, SUC_lexicon_tags_2 },
    { 0x618706a, SUC_lexicon_tags_43 },
    { 0xa387066, SUC_lexicon_tags_47 },
    { 0xb8d8706c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa187070, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfb187072, SUC_lexicon_tags_38 },
    { 0x9d987073, SUC_lexicon_tags_11 },
    { 0x4c987072, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e787078, SUC_lexicon_tags_15 },
    { 0xabb87079, SUC_lexicon_tags_7 },
    { 0xea98707a, SUC_lexicon_tags_6 },
    { 0x54d8707b, SUC_lexicon_tags_39 },
    { 0xa4d8707c, SUC_lexicon_tags_10 },
    { 0x2d87079, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4698707f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x58b87081, SUC_lexicon_tags_60 },
    { 0xb5787081, SUC_lexicon_tags_109 },
    { 0x7df87081, SUC_lexicon_tags_68 },
    { 0xd8587083, SUC_lexicon_tags_26 },
    { 0x2b87085, SUC_lexicon_tags_30 },
    { 0xa3d87086, SUC_lexicon_tags_17 },
    { 0x9a387085, SUC_lexicon_tags_17 },
    { 0x2c387088, SUC_lexicon_tags_33 },
    { 0xb4387088, SUC_lexicon_tags_1 },
    { 0xf3d8708a, SUC_lexicon_tags_60 },
    { 0x11787083, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ff8708d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f87090, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf9387092, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb87095, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7187097, SUC_lexicon_tags_31 },
    { 0x1eb87098, SUC_lexicon_tags_37 },
    { 0x79587099, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf018709e, SUC_lexicon_tags_14 },
    { 0xd5f8709f, SUC_lexicon_tags_27 },
    { 0x4b3870a0, SUC_lexicon_tags_14 },
    { 0xca5870a1, SUC_lexicon_tags_47 },
    { 0x987870a0, SUC_lexicon_tags_25 },
    { 0x67d870a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593870a7, SUC_lexicon_tags_15 },
    { 0x84d870a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89870af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9870b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x221870b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb870b9, SUC_lexicon_tags_5 },
    { 0xb7d870ba, SUC_lexicon_tags_25 },
    { 0x4a9870b9, SUC_lexicon_tags_2 },
    { 0xc79870bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125870c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf55870c3, SUC_lexicon_tags_9 },
    { 0x457870c3, SUC_lexicon_tags_37 },
    { 0x2e5870c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d870cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39870d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad870d3, SUC_lexicon_tags_17 },
    { 0x201870d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393870d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x517870db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7870e1, SUC_lexicon_tags_12 },
    { 0x5af870e2, SUC_lexicon_tags_17 },
    { 0x5e9870e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc71870e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2db870e7, SUC_lexicon_tags_8 },
    { 0xa17870e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d870eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dd870ed, SUC_lexicon_tags_23 },
    { 0xf07870ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1870f3, SUC_lexicon_tags_11 },
    { 0x86f870f3, SUC_lexicon_tags_18 },
    { 0x6af870f5, SUC_lexicon_tags_15 },
    { 0x29870f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7870fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb870fe, SUC_lexicon_tags_25 },
    { 0xd85870ff, SUC_lexicon_tags_44 },
    { 0x66787100, SUC_lexicon_tags_7 },
    { 0xced87100, SUC_lexicon_tags_7 },
    { 0xa4187102, SUC_lexicon_tags_65 },
    { 0x131870fe, SUC_lexicon_tags_12 },
    { 0x49b87104, SUC_lexicon_tags_65 },
    { 0xc3f87105, SUC_lexicon_tags_25 },
    { 0xb4f87103, SUC_lexicon_tags_11 },
    { 0x3c787107, SUC_lexicon_tags_4 },
    { 0xf5987108, SUC_lexicon_tags_7 },
    { 0x7f987109, SUC_lexicon_tags_108 },
    { 0x77187108, SUC_lexicon_tags_25 },
    { 0xaff8710b, SUC_lexicon_tags_23 },
    { 0x27587107, SUC_lexicon_tags_14 },
    { 0x2b8710d, SUC_lexicon_tags_57 },
    { 0x45d8710d, SUC_lexicon_tags_25 },
    { 0xd718710f, SUC_lexicon_tags_32 },
    { 0xc798710f, SUC_lexicon_tags_37 },
    { 0x91d87111, SUC_lexicon_tags_29 },
    { 0x53d8710b, SUC_lexicon_tags_15 },
    { 0xd698710b, SUC_lexicon_tags_15 },
    { 0xa1b87112, SUC_lexicon_tags_13 },
    { 0xd5f87115, SUC_lexicon_tags_11 },
    { 0x3bd87116, SUC_lexicon_tags_9 },
    { 0xdb987115, SUC_lexicon_tags_7 },
    { 0x88d87114, SUC_lexicon_tags_17 },
    { 0x78b87109, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb098711c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3258711f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d987123, SUC_lexicon_tags_2 },
    { 0x95b87124, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2378712a, SUC_lexicon_tags_37 },
    { 0x9898712a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d58712f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1f87131, SUC_lexicon_tags_32 },
    { 0xc8787132, SUC_lexicon_tags_2 },
    { 0xe0787132, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a987137, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xacd87139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5298713c, SUC_lexicon_tags_12 },
    { 0xc2d8713c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfff8713f, SUC_lexicon_tags_70 },
    { 0xf118713f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf4f87142, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f87147, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b8714a, SUC_lexicon_tags_37 },
    { 0x9978714b, SUC_lexicon_tags_60 },
    { 0xe318714c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf187152, SUC_lexicon_tags_21 },
    { 0x7c587153, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d87157, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85187159, SUC_lexicon_tags_12 },
    { 0xc938715a, SUC_lexicon_tags_49 },
    { 0x8578715b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d87160, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1eb87162, SUC_lexicon_tags_2 },
    { 0x40987163, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f587165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14787167, SUC_lexicon_tags_17 },
    { 0x1c987168, SUC_lexicon_tags_15 },
    { 0xe0787169, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6958716c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b87171, SUC_lexicon_tags_11 },
    { 0xfaf87172, SUC_lexicon_tags_18 },
    { 0xadb87173, SUC_lexicon_tags_37 },
    { 0xaef87174, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x71b87176, SUC_lexicon_tags_0 },
    { 0x36187176, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89787179, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d38717e, SUC_lexicon_tags_26 },
    { 0x3198717f, SUC_lexicon_tags_32 },
    { 0xcb587180, SUC_lexicon_tags_26 },
    { 0x3dd87180, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe5f87183, SUC_lexicon_tags_6 },
    { 0x23187183, SUC_lexicon_tags_14 },
    { 0xd6387185, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3987188, SUC_lexicon_tags_23 },
    { 0x27787189, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a98718f, SUC_lexicon_tags_12 },
    { 0x7f87190, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c387193, SUC_lexicon_tags_1 },
    { 0xf3787193, SUC_lexicon_tags_3 },
    { 0xafd87195, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3438719d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3871a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d871a3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1871aa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x84b871ac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185871b0, SUC_lexicon_tags_82 },
    { 0xb0f871b1, SUC_lexicon_tags_31 },
    { 0xf3d871b1, SUC_lexicon_tags_43 },
    { 0x74f871b2, SUC_lexicon_tags_25 },
    { 0xcc5871b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef871b7, SUC_lexicon_tags_9 },
    { 0xee5871b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x881871ba, SUC_lexicon_tags_23 },
    { 0xb9b871bb, SUC_lexicon_tags_7 },
    { 0xcc5871bc, SUC_lexicon_tags_7 },
    { 0x57d871bc, SUC_lexicon_tags_9 },
    { 0x189871ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd871c2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x251871c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63871cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf6f871cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727871da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x893871dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9871df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x627871e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1871e5, SUC_lexicon_tags_6 },
    { 0x4c1871e6, SUC_lexicon_tags_26 },
    { 0x4b7871e7, SUC_lexicon_tags_39 },
    { 0x1f5871e7, SUC_lexicon_tags_23 },
    { 0x97b871e8, SUC_lexicon_tags_30 },
    { 0x6ab871e9, SUC_lexicon_tags_23 },
    { 0xbf7871e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x827871ed, SUC_lexicon_tags_37 },
    { 0x6eb871ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861871f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625871f7, SUC_lexicon_tags_11 },
    { 0xedb871f7, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xa4f871fa, SUC_lexicon_tags_45 },
    { 0xb43871fb, SUC_lexicon_tags_30 },
    { 0x84b871fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91d871fe, SUC_lexicon_tags_12 },
    { 0x373871ff, SUC_lexicon_tags_12 },
    { 0x44587200, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9387204, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8187208, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf38720b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b78720e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5f187210, SUC_lexicon_tags_31 },
    { 0x41f87210, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12387219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3758721d, SUC_lexicon_tags_37 },
    { 0x5df8721e, SUC_lexicon_tags_25 },
    { 0xea58721f, SUC_lexicon_tags_15 },
    { 0x8a78721d, SUC_lexicon_tags_30 },
    { 0x2c18721d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af87224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b87229, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c58722f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b987236, SUC_lexicon_tags_37 },
    { 0xb3387236, SUC_lexicon_tags_8 },
    { 0xc5787237, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ed8723a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x298723d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f87242, SUC_lexicon_tags_37 },
    { 0xf87243, SUC_lexicon_tags_2 },
    { 0xb4187244, SUC_lexicon_tags_79 },
    { 0x24b87243, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33787249, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47b8724c, SUC_lexicon_tags_64 },
    { 0xa338724c, SUC_lexicon_tags_63 },
    { 0xa618724d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf87252, SUC_lexicon_tags_7 },
    { 0x5bb87253, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b987256, SUC_lexicon_tags_37 },
    { 0x8e587257, SUC_lexicon_tags_2 },
    { 0xb3f87258, SUC_lexicon_tags_17 },
    { 0xa8987259, SUC_lexicon_tags_2 },
    { 0x1a787258, SUC_lexicon_tags_12 },
    { 0x1c38725b, SUC_lexicon_tags_12 },
    { 0xe5f8725c, SUC_lexicon_tags_17 },
    { 0xc1987259, SUC_lexicon_tags_8 },
    { 0xb058725c, SUC_lexicon_tags_17 },
    { 0x26b8725d, SUC_lexicon_tags_17 },
    { 0x90787260, SUC_lexicon_tags_14 },
    { 0xb387256, SUC_lexicon_tags_21 },
    { 0x3787257, SUC_lexicon_tags_37 },
    { 0x9a78725a, SUC_lexicon_tags_11 },
    { 0xecd87259, SUC_lexicon_tags_31 },
    { 0x50187261, SUC_lexicon_tags_1 },
    { 0x2b387266, SUC_lexicon_tags_15 },
    { 0x9f387267, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb8726a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d8726f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38387274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84d87276, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d8727c, SUC_lexicon_tags_7 },
    { 0xc4f8727d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59187281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a587283, SUC_lexicon_tags_14 },
    { 0xca187284, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59587288, SUC_lexicon_tags_2 },
    { 0x95987289, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf4b8728b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5878728d, SUC_lexicon_tags_23 },
    { 0x9bb8728e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf87295, SUC_lexicon_tags_12 },
    { 0x7187296, SUC_lexicon_tags_19 },
    { 0xe2f87297, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7b87299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3158729f, SUC_lexicon_tags_25 },
    { 0x5a58729f, SUC_lexicon_tags_6 },
    { 0x6a9872a0, SUC_lexicon_tags_38 },
    { 0xf1d872a1, SUC_lexicon_tags_30 },
    { 0xca5872a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33872a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17872a9, SUC_lexicon_tags_17 },
    { 0x113872a9, SUC_lexicon_tags_6 },
    { 0xa51872a9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f872af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x141872b1, SUC_lexicon_tags_25 },
    { 0x79872b1, SUC_lexicon_tags_37 },
    { 0x65b872b3, SUC_lexicon_tags_551 },
    { 0, NULL },
    { 0xd1b872b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1872b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xffb872ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17872c1, SUC_lexicon_tags_15 },
    { 0x453872c1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x30d872c4, SUC_lexicon_tags_2 },
    { 0x46d872c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37872c9, SUC_lexicon_tags_11 },
    { 0x493872ca, SUC_lexicon_tags_38 },
    { 0x3bf872cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c3872cd, SUC_lexicon_tags_32 },
    { 0xd0b872ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf59872d0, SUC_lexicon_tags_25 },
    { 0xe75872d0, SUC_lexicon_tags_28 },
    { 0x739872d0, SUC_lexicon_tags_27 },
    { 0x2e7872d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01872d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa21872d8, SUC_lexicon_tags_21 },
    { 0xbe9872d9, SUC_lexicon_tags_13 },
    { 0x6a7872d9, SUC_lexicon_tags_7 },
    { 0x27b872db, SUC_lexicon_tags_15 },
    { 0x833872db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1872df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x837872e2, SUC_lexicon_tags_522 },
    { 0xa39872e3, SUC_lexicon_tags_12 },
    { 0xb9d872e3, SUC_lexicon_tags_30 },
    { 0x109872e4, SUC_lexicon_tags_0 },
    { 0x3c7872e6, SUC_lexicon_tags_15 },
    { 0x9a1872e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe9872e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93872ec, SUC_lexicon_tags_11 },
    { 0x547872ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7872f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f872f4, SUC_lexicon_tags_47 },
    { 0x9ab872f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf872f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x965872fa, SUC_lexicon_tags_27 },
    { 0xd2d872fb, SUC_lexicon_tags_11 },
    { 0xe05872fb, SUC_lexicon_tags_6 },
    { 0x4eb872fd, SUC_lexicon_tags_27 },
    { 0x543872fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4787304, SUC_lexicon_tags_15 },
    { 0x66d87305, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb87311, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd987315, SUC_lexicon_tags_61 },
    { 0xc4187316, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d8731b, SUC_lexicon_tags_14 },
    { 0x8178731b, SUC_lexicon_tags_2 },
    { 0x658731b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xec98731f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b87323, SUC_lexicon_tags_19 },
    { 0xd2d87324, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5587327, SUC_lexicon_tags_253 },
    { 0xedb87328, SUC_lexicon_tags_8 },
    { 0x67187328, SUC_lexicon_tags_15 },
    { 0x33b8732a, SUC_lexicon_tags_3 },
    { 0x5478732a, SUC_lexicon_tags_43 },
    { 0xef78732a, SUC_lexicon_tags_0 },
    { 0x9cb87327, SUC_lexicon_tags_37 },
    { 0x81b8732c, SUC_lexicon_tags_28 },
    { 0xe7b8732f, SUC_lexicon_tags_0 },
    { 0x2978732e, SUC_lexicon_tags_15 },
    { 0xa478732f, SUC_lexicon_tags_7 },
    { 0x1587332, SUC_lexicon_tags_37 },
    { 0xd7b8732b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f87336, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x27987339, SUC_lexicon_tags_37 },
    { 0x8118733a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f8733e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85987341, SUC_lexicon_tags_37 },
    { 0x5bb87342, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48d87344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb87347, SUC_lexicon_tags_63 },
    { 0x4587347, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a38734b, SUC_lexicon_tags_9 },
    { 0x1978734c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68387350, SUC_lexicon_tags_37 },
    { 0x5eb87351, SUC_lexicon_tags_60 },
    { 0x47187352, SUC_lexicon_tags_17 },
    { 0x23387352, SUC_lexicon_tags_44 },
    { 0x57787354, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e387357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf8735b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc978735f, SUC_lexicon_tags_17 },
    { 0xddb87360, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd3387362, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6587365, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc3187367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3f87369, SUC_lexicon_tags_17 },
    { 0xc138736a, SUC_lexicon_tags_35 },
    { 0x8d58736b, SUC_lexicon_tags_7 },
    { 0xef18736c, SUC_lexicon_tags_23 },
    { 0xd818736c, SUC_lexicon_tags_62 },
    { 0x2178736e, SUC_lexicon_tags_27 },
    { 0x27f8736f, SUC_lexicon_tags_9 },
    { 0x6a78736f, SUC_lexicon_tags_11 },
    { 0xf8d87371, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b87373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb438737a, SUC_lexicon_tags_28 },
    { 0x1b98737b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11987383, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0187387, SUC_lexicon_tags_49 },
    { 0xe8587388, SUC_lexicon_tags_15 },
    { 0x5987389, SUC_lexicon_tags_25 },
    { 0xd2b8738a, SUC_lexicon_tags_49 },
    { 0xc2b8738b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb8738f, SUC_lexicon_tags_7 },
    { 0xce787390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff787392, SUC_lexicon_tags_2 },
    { 0xbc187392, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80387398, SUC_lexicon_tags_11 },
    { 0x53d87399, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d8739c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13873a1, SUC_lexicon_tags_65 },
    { 0x51b873a2, SUC_lexicon_tags_37 },
    { 0x4c9873a3, SUC_lexicon_tags_1 },
    { 0x33d873a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79873ad, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xf9b873af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439873b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57873bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad873c9, SUC_lexicon_tags_37 },
    { 0x957873ca, SUC_lexicon_tags_11 },
    { 0xd5d873cb, SUC_lexicon_tags_60 },
    { 0x5f1873cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65873cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x775873d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x957873d4, SUC_lexicon_tags_165 },
    { 0x6b873d5, SUC_lexicon_tags_17 },
    { 0x7fd873d6, SUC_lexicon_tags_4 },
    { 0x5f3873d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3873de, SUC_lexicon_tags_19 },
    { 0x857873df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe91873e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f873e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf873eb, SUC_lexicon_tags_0 },
    { 0x2e3873ec, SUC_lexicon_tags_87 },
    { 0xbdb873ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf873f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b873f7, SUC_lexicon_tags_17 },
    { 0xde1873f8, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65587403, SUC_lexicon_tags_2 },
    { 0x98d87403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2787407, SUC_lexicon_tags_2 },
    { 0xc3187408, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4838740e, SUC_lexicon_tags_49 },
    { 0x6e58740e, SUC_lexicon_tags_25 },
    { 0xd6987410, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d87415, SUC_lexicon_tags_47 },
    { 0xa5787415, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d87419, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xff98741b, SUC_lexicon_tags_28 },
    { 0xfa18741c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5d8741e, SUC_lexicon_tags_7 },
    { 0xce38741f, SUC_lexicon_tags_28 },
    { 0x4fb8741f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e987425, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb558742a, SUC_lexicon_tags_15 },
    { 0x6c78742a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5838742d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f87434, SUC_lexicon_tags_87 },
    { 0xca587435, SUC_lexicon_tags_45 },
    { 0xf3b87436, SUC_lexicon_tags_27 },
    { 0x68f87437, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd938743d, SUC_lexicon_tags_38 },
    { 0x3598743d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f87441, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6387446, SUC_lexicon_tags_11 },
    { 0x1f587447, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd6987449, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66787450, SUC_lexicon_tags_15 },
    { 0x39d87450, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b387459, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaab8745b, SUC_lexicon_tags_61 },
    { 0x48b8745b, SUC_lexicon_tags_47 },
    { 0x2ef8745b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f87460, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d87464, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf87468, SUC_lexicon_tags_37 },
    { 0x87587469, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f8746c, SUC_lexicon_tags_53 },
    { 0x2778746d, SUC_lexicon_tags_15 },
    { 0x1398746e, SUC_lexicon_tags_112 },
    { 0x67d8746e, SUC_lexicon_tags_25 },
    { 0xe9987470, SUC_lexicon_tags_25 },
    { 0x8ff87470, SUC_lexicon_tags_30 },
    { 0xe7987471, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1987474, SUC_lexicon_tags_33 },
    { 0xdb987475, SUC_lexicon_tags_10 },
    { 0xe8387476, SUC_lexicon_tags_6 },
    { 0xbeb87477, SUC_lexicon_tags_37 },
    { 0xc7587478, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x3298747a, SUC_lexicon_tags_0 },
    { 0xef38747b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa518747d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab787482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18748c, SUC_lexicon_tags_11 },
    { 0x1598748d, SUC_lexicon_tags_2 },
    { 0x64d8748e, SUC_lexicon_tags_113 },
    { 0x7ed8748d, SUC_lexicon_tags_15 },
    { 0xf4f87490, SUC_lexicon_tags_7 },
    { 0x3098748d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9587493, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x11387495, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6d187497, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xfb787499, SUC_lexicon_tags_522 },
    { 0x5d187499, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f8749d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x858749f, SUC_lexicon_tags_29 },
    { 0xb9b874a0, SUC_lexicon_tags_3 },
    { 0x6958749f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63874a6, SUC_lexicon_tags_0 },
    { 0xf9874a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x127874aa, SUC_lexicon_tags_25 },
    { 0xdc1874ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05874ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x813874b0, SUC_lexicon_tags_0 },
    { 0x871874b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d874b4, SUC_lexicon_tags_60 },
    { 0x73f874b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc0b874b7, SUC_lexicon_tags_43 },
    { 0x367874b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9874ba, SUC_lexicon_tags_12 },
    { 0x98d874bb, SUC_lexicon_tags_15 },
    { 0x911874bb, SUC_lexicon_tags_87 },
    { 0xe75874bd, SUC_lexicon_tags_17 },
    { 0x813874bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89d874c0, SUC_lexicon_tags_2 },
    { 0xf21874c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d874c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x721874c7, SUC_lexicon_tags_8 },
    { 0x9a1874c7, SUC_lexicon_tags_61 },
    { 0x62f874c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x37f874cb, SUC_lexicon_tags_11 },
    { 0xc5f874cb, SUC_lexicon_tags_31 },
    { 0x73874cd, SUC_lexicon_tags_25 },
    { 0x9af874cd, SUC_lexicon_tags_31 },
    { 0x927874cc, SUC_lexicon_tags_33 },
    { 0xad5874d0, SUC_lexicon_tags_15 },
    { 0xf95874d1, SUC_lexicon_tags_19 },
    { 0x5a3874d0, SUC_lexicon_tags_23 },
    { 0x21f874d3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd874db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9874e0, SUC_lexicon_tags_37 },
    { 0x303874e0, SUC_lexicon_tags_37 },
    { 0xca9874e2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x749874e4, SUC_lexicon_tags_11 },
    { 0x83f874e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x959874e8, SUC_lexicon_tags_55 },
    { 0xb23874e9, SUC_lexicon_tags_49 },
    { 0xdc3874ea, SUC_lexicon_tags_47 },
    { 0xbab874eb, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xbbf874ed, SUC_lexicon_tags_6 },
    { 0x317874ee, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x781874f0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x515874f2, SUC_lexicon_tags_17 },
    { 0xf9f874f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x505874f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa15874f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07874fc, SUC_lexicon_tags_4 },
    { 0xd23874fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d87501, SUC_lexicon_tags_60 },
    { 0x76787502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d87504, SUC_lexicon_tags_26 },
    { 0x4e387505, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x44d87507, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d8750b, SUC_lexicon_tags_49 },
    { 0x39b8750c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c38750e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b987513, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd87517, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4fd87519, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd58751b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfe38751d, SUC_lexicon_tags_19 },
    { 0x9998751e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4587521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cd87523, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef87528, SUC_lexicon_tags_851 },
    { 0xb7b87529, SUC_lexicon_tags_56 },
    { 0xc0787528, SUC_lexicon_tags_14 },
    { 0xe1d87528, SUC_lexicon_tags_8 },
    { 0x43d8752c, SUC_lexicon_tags_33 },
    { 0x6fb8752d, SUC_lexicon_tags_12 },
    { 0xfdf87528, SUC_lexicon_tags_17 },
    { 0xf8752f, SUC_lexicon_tags_25 },
    { 0xcc387530, SUC_lexicon_tags_82 },
    { 0x5858752e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x34387533, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd987537, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc278753d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec587540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3f87542, SUC_lexicon_tags_12 },
    { 0x6d187542, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15187546, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d8754c, SUC_lexicon_tags_27 },
    { 0x1398754d, SUC_lexicon_tags_43 },
    { 0x1b98754e, SUC_lexicon_tags_37 },
    { 0xf118754f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92587555, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5187557, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf8755a, SUC_lexicon_tags_1 },
    { 0xc178755b, SUC_lexicon_tags_34 },
    { 0xe5b8755a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d87560, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8987566, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af87569, SUC_lexicon_tags_23 },
    { 0xe138756a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf87572, SUC_lexicon_tags_23 },
    { 0x39787573, SUC_lexicon_tags_37 },
    { 0x81587574, SUC_lexicon_tags_6 },
    { 0x17787575, SUC_lexicon_tags_14 },
    { 0xb8387575, SUC_lexicon_tags_23 },
    { 0x55587577, SUC_lexicon_tags_50 },
    { 0xa4d87578, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33187584, SUC_lexicon_tags_17 },
    { 0x66387585, SUC_lexicon_tags_60 },
    { 0xe0187585, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9587588, SUC_lexicon_tags_45 },
    { 0xef187589, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb8758d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2587593, SUC_lexicon_tags_47 },
    { 0xbe187594, SUC_lexicon_tags_1 },
    { 0xabf87594, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xecf87597, SUC_lexicon_tags_25 },
    { 0x5c387598, SUC_lexicon_tags_19 },
    { 0xa1d87599, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48d8759f, SUC_lexicon_tags_17 },
    { 0xadd875a0, SUC_lexicon_tags_160 },
    { 0xf89875a1, SUC_lexicon_tags_826 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243875a6, SUC_lexicon_tags_12 },
    { 0xad9875a7, SUC_lexicon_tags_7 },
    { 0xea9875a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa27875ab, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x79f875ad, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f875b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d875b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f875bb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x33d875bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b875c5, SUC_lexicon_tags_31 },
    { 0x3b5875c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1875c9, SUC_lexicon_tags_53 },
    { 0x1df875ca, SUC_lexicon_tags_37 },
    { 0xe39875cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcf875cd, SUC_lexicon_tags_7 },
    { 0x4bf875ce, SUC_lexicon_tags_43 },
    { 0x6dd875ce, SUC_lexicon_tags_38 },
    { 0xbe5875ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597875d7, SUC_lexicon_tags_17 },
    { 0x10f875d8, SUC_lexicon_tags_14 },
    { 0xe85875d9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb875dc, SUC_lexicon_tags_28 },
    { 0xe47875dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3875e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03875e5, SUC_lexicon_tags_15 },
    { 0xdd5875e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f875ea, SUC_lexicon_tags_15 },
    { 0x20b875eb, SUC_lexicon_tags_1 },
    { 0x65f875ec, SUC_lexicon_tags_25 },
    { 0xc6f875ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7875f8, SUC_lexicon_tags_57 },
    { 0x9d7875f9, SUC_lexicon_tags_10 },
    { 0xf4f875f9, SUC_lexicon_tags_17 },
    { 0x15f875fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x13d875fd, SUC_lexicon_tags_5 },
    { 0xb99875fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2187600, SUC_lexicon_tags_11 },
    { 0xc6f87601, SUC_lexicon_tags_12 },
    { 0xe8b87602, SUC_lexicon_tags_17 },
    { 0x5a787602, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8187605, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c38760b, SUC_lexicon_tags_15 },
    { 0xb0d8760c, SUC_lexicon_tags_1 },
    { 0x8698760b, SUC_lexicon_tags_7 },
    { 0xcb18760e, SUC_lexicon_tags_19 },
    { 0xedd8760f, SUC_lexicon_tags_7 },
    { 0xd658760c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98987616, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34987619, SUC_lexicon_tags_45 },
    { 0x9b787619, SUC_lexicon_tags_42 },
    { 0xe1387619, SUC_lexicon_tags_11 },
    { 0x8e38761b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd8761f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60587624, SUC_lexicon_tags_12 },
    { 0x3fd87625, SUC_lexicon_tags_65 },
    { 0x13987625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8787628, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73f8762b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb818762d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c78762f, SUC_lexicon_tags_8 },
    { 0x9418762f, SUC_lexicon_tags_1 },
    { 0x8f187631, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb87634, SUC_lexicon_tags_46 },
    { 0xa6b87635, SUC_lexicon_tags_37 },
    { 0xbf787636, SUC_lexicon_tags_29 },
    { 0x93187636, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddd87639, SUC_lexicon_tags_15 },
    { 0x82f8763a, SUC_lexicon_tags_29 },
    { 0x21b87639, SUC_lexicon_tags_1 },
    { 0x4c78763c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1198763e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b87641, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fd87643, SUC_lexicon_tags_60 },
    { 0x22987644, SUC_lexicon_tags_28 },
    { 0xccf87644, SUC_lexicon_tags_30 },
    { 0x20387646, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d87649, SUC_lexicon_tags_12 },
    { 0x3e187649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb87651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22587654, SUC_lexicon_tags_11 },
    { 0x36f87654, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb87658, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9438765e, SUC_lexicon_tags_16 },
    { 0x60f8765f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa787665, SUC_lexicon_tags_11 },
    { 0xb2d87666, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x44787668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97b8766a, SUC_lexicon_tags_17 },
    { 0x7b8766a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f8766e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f87671, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d87679, SUC_lexicon_tags_25 },
    { 0x6018767a, SUC_lexicon_tags_7 },
    { 0x1a98767b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8b18767d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabf8767f, SUC_lexicon_tags_7 },
    { 0xe5187680, SUC_lexicon_tags_17 },
    { 0x65787681, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36f87683, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba58768c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb387691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d87697, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f8769a, SUC_lexicon_tags_38 },
    { 0x9e78769b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb8769e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69876a0, SUC_lexicon_tags_2 },
    { 0x3c3876a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x21d876a3, SUC_lexicon_tags_15 },
    { 0x4cf876a4, SUC_lexicon_tags_47 },
    { 0x2d1876a4, SUC_lexicon_tags_31 },
    { 0xced876a6, SUC_lexicon_tags_2 },
    { 0xb07876a6, SUC_lexicon_tags_7 },
    { 0xd3876a8, SUC_lexicon_tags_37 },
    { 0x159876a6, SUC_lexicon_tags_11 },
    { 0xeb5876a4, SUC_lexicon_tags_10 },
    { 0xdcb876ab, SUC_lexicon_tags_25 },
    { 0x107876ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f876af, SUC_lexicon_tags_27 },
    { 0x4e5876b0, SUC_lexicon_tags_45 },
    { 0x29b876b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2d9876b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80b876b5, SUC_lexicon_tags_11 },
    { 0x4df876b6, SUC_lexicon_tags_29 },
    { 0xc49876b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609876bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd3876bf, SUC_lexicon_tags_1 },
    { 0x42b876c0, SUC_lexicon_tags_6 },
    { 0x575876c1, SUC_lexicon_tags_11 },
    { 0x109876c2, SUC_lexicon_tags_17 },
    { 0xf4b876c3, SUC_lexicon_tags_2 },
    { 0xed7876c2, SUC_lexicon_tags_2 },
    { 0xd51876c5, SUC_lexicon_tags_17 },
    { 0x38f876c5, SUC_lexicon_tags_27 },
    { 0x98b876c7, SUC_lexicon_tags_7 },
    { 0x47f876c8, SUC_lexicon_tags_38 },
    { 0x9a3876c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa2f876cb, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87876ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab876d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3876d5, SUC_lexicon_tags_37 },
    { 0xa61876d6, SUC_lexicon_tags_12 },
    { 0xb05876d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf1b876d9, SUC_lexicon_tags_12 },
    { 0xc39876d9, SUC_lexicon_tags_11 },
    { 0xaa3876da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddf876dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7876e3, SUC_lexicon_tags_82 },
    { 0xc71876e3, SUC_lexicon_tags_138 },
    { 0x69876e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x795876e8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x31876eb, SUC_lexicon_tags_37 },
    { 0x197876ec, SUC_lexicon_tags_4 },
    { 0xe55876ed, SUC_lexicon_tags_25 },
    { 0x6f876ee, SUC_lexicon_tags_11 },
    { 0xecf876ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x41d876f1, SUC_lexicon_tags_250 },
    { 0xcb876f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35876f7, SUC_lexicon_tags_56 },
    { 0xd31876f8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x445876fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43f876fc, SUC_lexicon_tags_19 },
    { 0xe5876fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x201876ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67787702, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab87706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3518770a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe258770e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb4187710, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57387716, SUC_lexicon_tags_27 },
    { 0xef787717, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed8771a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x438771c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8438771e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff987722, SUC_lexicon_tags_31 },
    { 0xd9787722, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8f587725, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd87729, SUC_lexicon_tags_7 },
    { 0x8f87729, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdeb8772c, SUC_lexicon_tags_25 },
    { 0xb598772c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b87734, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd387736, SUC_lexicon_tags_15 },
    { 0xc4f87737, SUC_lexicon_tags_38 },
    { 0x8d387736, SUC_lexicon_tags_40 },
    { 0xc4b87737, SUC_lexicon_tags_23 },
    { 0x8cf87736, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b87741, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75387745, SUC_lexicon_tags_10 },
    { 0x74d87746, SUC_lexicon_tags_22 },
    { 0x1187747, SUC_lexicon_tags_0 },
    { 0xcad87747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b187752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab87755, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4f87757, SUC_lexicon_tags_15 },
    { 0xe2b87758, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa898775a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb18775d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd87762, SUC_lexicon_tags_25 },
    { 0x35b87762, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x65187766, SUC_lexicon_tags_17 },
    { 0x87387766, SUC_lexicon_tags_1 },
    { 0x9787768, SUC_lexicon_tags_12 },
    { 0x75587767, SUC_lexicon_tags_42 },
    { 0xbfd8776a, SUC_lexicon_tags_25 },
    { 0xcd18776b, SUC_lexicon_tags_33 },
    { 0x9298776c, SUC_lexicon_tags_38 },
    { 0x1e78776d, SUC_lexicon_tags_25 },
    { 0x25b87767, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b387771, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c387774, SUC_lexicon_tags_37 },
    { 0x49d87774, SUC_lexicon_tags_29 },
    { 0x6c187775, SUC_lexicon_tags_37 },
    { 0x2eb87777, SUC_lexicon_tags_12 },
    { 0x3f87778, SUC_lexicon_tags_26 },
    { 0xe7787779, SUC_lexicon_tags_23 },
    { 0x3978777a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94787782, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe5d87784, SUC_lexicon_tags_33 },
    { 0xf7f87785, SUC_lexicon_tags_38 },
    { 0x43787786, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f8778d, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed87794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8779f, SUC_lexicon_tags_15 },
    { 0xdb5877a0, SUC_lexicon_tags_60 },
    { 0x1dd877a1, SUC_lexicon_tags_1 },
    { 0xdc7877a2, SUC_lexicon_tags_25 },
    { 0x977877a2, SUC_lexicon_tags_27 },
    { 0xbc3877a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd89877a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbed877a8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x34b877aa, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xd7b877ac, SUC_lexicon_tags_43 },
    { 0x269877ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccb877af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b1877b1, SUC_lexicon_tags_25 },
    { 0xae9877b2, SUC_lexicon_tags_25 },
    { 0x1f1877b3, SUC_lexicon_tags_30 },
    { 0x5b3877b2, SUC_lexicon_tags_25 },
    { 0x43877b5, SUC_lexicon_tags_56 },
    { 0xfeb877b6, SUC_lexicon_tags_8 },
    { 0x7f3877b3, SUC_lexicon_tags_23 },
    { 0xf63877b5, SUC_lexicon_tags_208 },
    { 0xeb1877b9, SUC_lexicon_tags_7 },
    { 0x5a9877ba, SUC_lexicon_tags_8 },
    { 0x335877ba, SUC_lexicon_tags_26 },
    { 0x925877bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa9f877be, SUC_lexicon_tags_17 },
    { 0xef1877be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5877c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b877cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b877cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x617877d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1877d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe75877d7, SUC_lexicon_tags_32 },
    { 0x267877d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171877dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5877df, SUC_lexicon_tags_12 },
    { 0xf33877e0, SUC_lexicon_tags_87 },
    { 0x7d1877e1, SUC_lexicon_tags_65 },
    { 0xe3b877e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa11877e4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x309877e7, SUC_lexicon_tags_65 },
    { 0xbb1877e8, SUC_lexicon_tags_8 },
    { 0xc63877e9, SUC_lexicon_tags_11 },
    { 0xd0d877ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b877ed, SUC_lexicon_tags_43 },
    { 0x6a1877ee, SUC_lexicon_tags_39 },
    { 0xdab877ef, SUC_lexicon_tags_37 },
    { 0xb3d877f0, SUC_lexicon_tags_77 },
    { 0xf69877f0, SUC_lexicon_tags_30 },
    { 0x46d877ed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x383877f4, SUC_lexicon_tags_9 },
    { 0xa3f877f5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1877f9, SUC_lexicon_tags_160 },
    { 0xff5877fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cb877fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68387800, SUC_lexicon_tags_15 },
    { 0xa6b87800, SUC_lexicon_tags_0 },
    { 0xae187800, SUC_lexicon_tags_31 },
    { 0x6eb87803, SUC_lexicon_tags_47 },
    { 0x4cf87804, SUC_lexicon_tags_25 },
    { 0x6d787805, SUC_lexicon_tags_26 },
    { 0xdfd87803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2db87808, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a98780c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb87810, SUC_lexicon_tags_23 },
    { 0x64987810, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbdf87813, SUC_lexicon_tags_37 },
    { 0x69f87813, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xe0987816, SUC_lexicon_tags_12 },
    { 0x1cd87816, SUC_lexicon_tags_1 },
    { 0xa1587817, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9d8781a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3fd8781c, SUC_lexicon_tags_37 },
    { 0x66b8781c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3d18781f, SUC_lexicon_tags_0 },
    { 0x9bf87820, SUC_lexicon_tags_1 },
    { 0x46387821, SUC_lexicon_tags_32 },
    { 0x6eb87822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae987828, SUC_lexicon_tags_17 },
    { 0x27f87829, SUC_lexicon_tags_17 },
    { 0x2cf8782a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe498782c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5318782e, SUC_lexicon_tags_33 },
    { 0xea18782e, SUC_lexicon_tags_33 },
    { 0xa3b87830, SUC_lexicon_tags_31 },
    { 0xa987831, SUC_lexicon_tags_23 },
    { 0xfff87831, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9b87834, SUC_lexicon_tags_8 },
    { 0x47787835, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d387837, SUC_lexicon_tags_8 },
    { 0xd8f87838, SUC_lexicon_tags_27 },
    { 0xb3387839, SUC_lexicon_tags_38 },
    { 0x51587839, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x558783c, SUC_lexicon_tags_4 },
    { 0x48b8783d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3bf8783f, SUC_lexicon_tags_47 },
    { 0xb0b87840, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35387844, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45d87846, SUC_lexicon_tags_63 },
    { 0xf3587847, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1898784a, SUC_lexicon_tags_15 },
    { 0x4d8784b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e98784d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26987850, SUC_lexicon_tags_65 },
    { 0x4d587851, SUC_lexicon_tags_11 },
    { 0x67d87852, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c387855, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd87858, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb8785b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7cb8785d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b87861, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71787864, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4f87866, SUC_lexicon_tags_37 },
    { 0xfa987867, SUC_lexicon_tags_7 },
    { 0x34187866, SUC_lexicon_tags_79 },
    { 0x3bd87867, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2158786f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db87875, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb338787a, SUC_lexicon_tags_16 },
    { 0xa7b8787b, SUC_lexicon_tags_37 },
    { 0x438787c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4038787e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x2f87880, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb9f87882, SUC_lexicon_tags_38 },
    { 0x7cd87883, SUC_lexicon_tags_25 },
    { 0xb587883, SUC_lexicon_tags_43 },
    { 0x2dd87885, SUC_lexicon_tags_17 },
    { 0xbc587885, SUC_lexicon_tags_33 },
    { 0xce187885, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe787889, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0f8788b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4738788d, SUC_lexicon_tags_33 },
    { 0x45f8788d, SUC_lexicon_tags_6 },
    { 0x6218788f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd87893, SUC_lexicon_tags_15 },
    { 0xaa587894, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87898, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b78789e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41878ac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ad878ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x531878b0, SUC_lexicon_tags_49 },
    { 0xe19878b1, SUC_lexicon_tags_25 },
    { 0x10d878b1, SUC_lexicon_tags_25 },
    { 0xe5878b3, SUC_lexicon_tags_17 },
    { 0xce3878b4, SUC_lexicon_tags_64 },
    { 0xb05878b2, SUC_lexicon_tags_25 },
    { 0x149878b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x193878b9, SUC_lexicon_tags_25 },
    { 0x1c1878ba, SUC_lexicon_tags_63 },
    { 0xe1d878bb, SUC_lexicon_tags_11 },
    { 0x9878ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x507878be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e7878c0, SUC_lexicon_tags_37 },
    { 0xad1878c0, SUC_lexicon_tags_31 },
    { 0xac1878c2, SUC_lexicon_tags_0 },
    { 0x963878c2, SUC_lexicon_tags_38 },
    { 0x739878c4, SUC_lexicon_tags_2 },
    { 0x42b878c5, SUC_lexicon_tags_7 },
    { 0xd9f878c6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819878cb, SUC_lexicon_tags_2 },
    { 0x527878cb, SUC_lexicon_tags_11 },
    { 0x9fb878cb, SUC_lexicon_tags_37 },
    { 0x2a7878cd, SUC_lexicon_tags_56 },
    { 0x84d878cf, SUC_lexicon_tags_47 },
    { 0x22b878cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbad878d2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b878d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1878d8, SUC_lexicon_tags_15 },
    { 0x8e3878d8, SUC_lexicon_tags_38 },
    { 0xa5d878da, SUC_lexicon_tags_17 },
    { 0x55f878da, SUC_lexicon_tags_7 },
    { 0xb91878dc, SUC_lexicon_tags_55 },
    { 0x48b878da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x103878df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5878e2, SUC_lexicon_tags_1 },
    { 0xee3878e3, SUC_lexicon_tags_17 },
    { 0x4df878e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b878e7, SUC_lexicon_tags_3 },
    { 0xd37878e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53878ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x961878ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x403878f0, SUC_lexicon_tags_60 },
    { 0xc29878f0, SUC_lexicon_tags_14 },
    { 0x3b7878f0, SUC_lexicon_tags_70 },
    { 0x42d878f3, SUC_lexicon_tags_19 },
    { 0xf79878f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa79878f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d878fa, SUC_lexicon_tags_12 },
    { 0x273878fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99d878fd, SUC_lexicon_tags_1242 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b987900, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b387906, SUC_lexicon_tags_37 },
    { 0x37d87907, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6118790a, SUC_lexicon_tags_1 },
    { 0xbaf8790a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd7d8790d, SUC_lexicon_tags_25 },
    { 0x7898790e, SUC_lexicon_tags_7 },
    { 0x3318790f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73387913, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25f87915, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60187919, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1798791c, SUC_lexicon_tags_26 },
    { 0x9e38791d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b87920, SUC_lexicon_tags_43 },
    { 0x79987920, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f787927, SUC_lexicon_tags_17 },
    { 0x65987928, SUC_lexicon_tags_25 },
    { 0xd0387929, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b8792e, SUC_lexicon_tags_17 },
    { 0x3c38792f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d87937, SUC_lexicon_tags_0 },
    { 0x88987937, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xde18793a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1d8793c, SUC_lexicon_tags_27 },
    { 0xa618793d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d87940, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0b87942, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6b87944, SUC_lexicon_tags_11 },
    { 0xbf187944, SUC_lexicon_tags_2 },
    { 0xc5987946, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a987949, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeef8794b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa318794d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x52787950, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b87954, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f587959, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0x3678795b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8795f, SUC_lexicon_tags_10 },
    { 0x96187960, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f787968, SUC_lexicon_tags_10 },
    { 0xcdf87969, SUC_lexicon_tags_30 },
    { 0x3a587969, SUC_lexicon_tags_3 },
    { 0xd138796b, SUC_lexicon_tags_15 },
    { 0x6bd87969, SUC_lexicon_tags_38 },
    { 0xb4b8796d, SUC_lexicon_tags_7 },
    { 0x4818796b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b87971, SUC_lexicon_tags_11 },
    { 0xd0787971, SUC_lexicon_tags_11 },
    { 0xd4d87973, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd8b87975, SUC_lexicon_tags_43 },
    { 0x48987976, SUC_lexicon_tags_2 },
    { 0x40d87975, SUC_lexicon_tags_85 },
    { 0x59387978, SUC_lexicon_tags_11 },
    { 0x49387979, SUC_lexicon_tags_0 },
    { 0x7df87976, SUC_lexicon_tags_27 },
    { 0xbcb8797b, SUC_lexicon_tags_1 },
    { 0x7378797c, SUC_lexicon_tags_60 },
    { 0x49d8797d, SUC_lexicon_tags_26 },
    { 0x6018797e, SUC_lexicon_tags_37 },
    { 0xdbd8797e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b87982, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a58798d, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d87992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xded87995, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf87998, SUC_lexicon_tags_62 },
    { 0x4c787999, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4738799e, SUC_lexicon_tags_12 },
    { 0x3b18799f, SUC_lexicon_tags_27 },
    { 0xf9f879a0, SUC_lexicon_tags_0 },
    { 0xbdd8799f, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f879a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81879ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe1b879ad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3879b0, SUC_lexicon_tags_37 },
    { 0xd7d879b1, SUC_lexicon_tags_49 },
    { 0x3bf879b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x101879b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1879bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3879c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe27879c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b879c8, SUC_lexicon_tags_32 },
    { 0x3a1879c9, SUC_lexicon_tags_14 },
    { 0x84d879ca, SUC_lexicon_tags_63 },
    { 0xf3b879cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa13879cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79879d0, SUC_lexicon_tags_17 },
    { 0x6ef879d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x359879d3, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0x84d879d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b879d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc7879da, SUC_lexicon_tags_27 },
    { 0xb01879db, SUC_lexicon_tags_37 },
    { 0x571879da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4f879de, SUC_lexicon_tags_27 },
    { 0xfc5879df, SUC_lexicon_tags_49 },
    { 0x271879e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xec7879e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601879e8, SUC_lexicon_tags_21 },
    { 0x1d9879e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcd879eb, SUC_lexicon_tags_29 },
    { 0xb65879ec, SUC_lexicon_tags_25 },
    { 0xf9879ed, SUC_lexicon_tags_1 },
    { 0x101879ec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9879f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435879f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7a5879f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed879fc, SUC_lexicon_tags_37 },
    { 0x54f879fd, SUC_lexicon_tags_160 },
    { 0x5d879fd, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb787a01, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b587a0d, SUC_lexicon_tags_27 },
    { 0xe7d87a0d, SUC_lexicon_tags_27 },
    { 0x4387a0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20387a14, SUC_lexicon_tags_12 },
    { 0xccb87a14, SUC_lexicon_tags_21 },
    { 0xc1787a15, SUC_lexicon_tags_11 },
    { 0x24187a17, SUC_lexicon_tags_4 },
    { 0x15d87a17, SUC_lexicon_tags_66 },
    { 0x46387a19, SUC_lexicon_tags_15 },
    { 0x7f587a1a, SUC_lexicon_tags_136 },
    { 0x28187a15, SUC_lexicon_tags_11 },
    { 0x3f987a1c, SUC_lexicon_tags_46 },
    { 0x75387a1c, SUC_lexicon_tags_13 },
    { 0xbf787a1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98f87a20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b87a23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6587a32, SUC_lexicon_tags_2 },
    { 0xba787a33, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80987a3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d87a3e, SUC_lexicon_tags_25 },
    { 0x38387a3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66587a41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb87a45, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x82387a48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2bd87a4a, SUC_lexicon_tags_37 },
    { 0x84f87a4b, SUC_lexicon_tags_63 },
    { 0x2e787a4b, SUC_lexicon_tags_17 },
    { 0xf7b87a4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5787a51, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad87a5c, SUC_lexicon_tags_14 },
    { 0xf7987a5d, SUC_lexicon_tags_87 },
    { 0x14d87a5d, SUC_lexicon_tags_23 },
    { 0x6787a5e, SUC_lexicon_tags_17 },
    { 0x7eb87a60, SUC_lexicon_tags_7 },
    { 0x41d87a61, SUC_lexicon_tags_17 },
    { 0xec387a5e, SUC_lexicon_tags_10 },
    { 0xee587a63, SUC_lexicon_tags_29 },
    { 0x62d87a64, SUC_lexicon_tags_282 },
    { 0x2b587a64, SUC_lexicon_tags_46 },
    { 0x3bd87a64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c387a68, SUC_lexicon_tags_30 },
    { 0x97787a68, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25787a6d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b87a75, SUC_lexicon_tags_207 },
    { 0xe4f87a76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9587a78, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82187a7c, SUC_lexicon_tags_17 },
    { 0x80f87a7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbf87a7f, SUC_lexicon_tags_25 },
    { 0x5b987a80, SUC_lexicon_tags_11 },
    { 0x81387a80, SUC_lexicon_tags_68 },
    { 0xcfb87a81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x31d87a84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd87a89, SUC_lexicon_tags_68 },
    { 0x57b87a8a, SUC_lexicon_tags_29 },
    { 0xc6b87a8a, SUC_lexicon_tags_11 },
    { 0x77b87a89, SUC_lexicon_tags_17 },
    { 0xf5f87a89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d87a95, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18587a9a, SUC_lexicon_tags_45 },
    { 0xf9787a9b, SUC_lexicon_tags_45 },
    { 0xbe187a9c, SUC_lexicon_tags_47 },
    { 0xe0d87a9b, SUC_lexicon_tags_37 },
    { 0x5b587a9e, SUC_lexicon_tags_31 },
    { 0x80987a9f, SUC_lexicon_tags_11 },
    { 0x3e387a9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb787aa4, SUC_lexicon_tags_30 },
    { 0x43f87aa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ad87aa7, SUC_lexicon_tags_11 },
    { 0x7c587aa7, SUC_lexicon_tags_27 },
    { 0x9e187aa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1f87aab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8387aae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7387ab1, SUC_lexicon_tags_25 },
    { 0x95d87ab2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d87ab5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3187abd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3c987abf, SUC_lexicon_tags_2 },
    { 0x1ed87ac0, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x80d87ac2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x48787ac5, SUC_lexicon_tags_38 },
    { 0x7987ac5, SUC_lexicon_tags_12 },
    { 0x68b87ac7, SUC_lexicon_tags_15 },
    { 0xc0187ac8, SUC_lexicon_tags_15 },
    { 0x8e987ac9, SUC_lexicon_tags_30 },
    { 0x7a187aca, SUC_lexicon_tags_33 },
    { 0x68b87ac8, SUC_lexicon_tags_17 },
    { 0xeaf87ac9, SUC_lexicon_tags_15 },
    { 0x61187acb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a987ad3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb87ad6, SUC_lexicon_tags_46 },
    { 0x13d87ad6, SUC_lexicon_tags_15 },
    { 0xa6d87ad7, SUC_lexicon_tags_32 },
    { 0xf187ad9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3af87adb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd3787add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21b87adf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f87ae6, SUC_lexicon_tags_43 },
    { 0x6f987ae6, SUC_lexicon_tags_37 },
    { 0x8d387ae8, SUC_lexicon_tags_15 },
    { 0xa1187ae9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x28787aeb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3e587aed, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f87af1, SUC_lexicon_tags_12 },
    { 0x17587af2, SUC_lexicon_tags_33 },
    { 0xed587af2, SUC_lexicon_tags_41 },
    { 0xbab87af3, SUC_lexicon_tags_63 },
    { 0x35387af3, SUC_lexicon_tags_17 },
    { 0xb8587af4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73d87af8, SUC_lexicon_tags_15 },
    { 0xc9f87af9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6187afb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f387afe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x96f87b00, SUC_lexicon_tags_7 },
    { 0xb0d87b01, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b587b03, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x31187b05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d787b07, SUC_lexicon_tags_66 },
    { 0x6f987b08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ab87b0a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8387b0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8f387b10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb387b16, SUC_lexicon_tags_47 },
    { 0x20f87b17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd87b1a, SUC_lexicon_tags_2 },
    { 0xa7d87b1a, SUC_lexicon_tags_12 },
    { 0x39787b1c, SUC_lexicon_tags_540 },
    { 0xff787b1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd87b20, SUC_lexicon_tags_34 },
    { 0x54787b21, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9b187b23, SUC_lexicon_tags_2 },
    { 0x5fb87b24, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x4eb87b26, SUC_lexicon_tags_25 },
    { 0x32b87b27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b87b2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d187b2f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b87b36, SUC_lexicon_tags_12 },
    { 0x46587b37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ed87b39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69587b3b, SUC_lexicon_tags_30 },
    { 0x8e187b3b, SUC_lexicon_tags_15 },
    { 0x1cb87b3d, SUC_lexicon_tags_14 },
    { 0xe1987b3e, SUC_lexicon_tags_160 },
    { 0x97d87b3e, SUC_lexicon_tags_45 },
    { 0xac987b3b, SUC_lexicon_tags_15 },
    { 0xfa387b41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x44d87b44, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xedf87b46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f187b4b, SUC_lexicon_tags_38 },
    { 0xe6f87b4c, SUC_lexicon_tags_12 },
    { 0xf4387b4d, SUC_lexicon_tags_28 },
    { 0x6b87b4c, SUC_lexicon_tags_1 },
    { 0x7c387b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98f87b51, SUC_lexicon_tags_49 },
    { 0xded87b52, SUC_lexicon_tags_17 },
    { 0x6c587b53, SUC_lexicon_tags_62 },
    { 0x74787b54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f587b57, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3d987b59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3987b5c, SUC_lexicon_tags_9 },
    { 0x49b87b5c, SUC_lexicon_tags_33 },
    { 0xddd87b5e, SUC_lexicon_tags_65 },
    { 0x1db87b5f, SUC_lexicon_tags_37 },
    { 0xa6787b60, SUC_lexicon_tags_108 },
    { 0x16b87b61, SUC_lexicon_tags_0 },
    { 0xa187b62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8587b66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6387b6a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34987b73, SUC_lexicon_tags_151 },
    { 0xbdb87b74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcd587b76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff87b7e, SUC_lexicon_tags_17 },
    { 0x4a987b7f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21987b84, SUC_lexicon_tags_14 },
    { 0xfcf87b84, SUC_lexicon_tags_56 },
    { 0x93b87b86, SUC_lexicon_tags_12 },
    { 0xa9187b87, SUC_lexicon_tags_47 },
    { 0xccf87b88, SUC_lexicon_tags_65 },
    { 0x97b87b89, SUC_lexicon_tags_37 },
    { 0xdb987b88, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a787b8d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4787b92, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7bb87b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63187b98, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf987b9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26587b9f, SUC_lexicon_tags_42 },
    { 0xde787b9f, SUC_lexicon_tags_19 },
    { 0x4a987ba1, SUC_lexicon_tags_21 },
    { 0x76d87ba1, SUC_lexicon_tags_14 },
    { 0x41187ba2, SUC_lexicon_tags_17 },
    { 0xc5d87ba3, SUC_lexicon_tags_37 },
    { 0xcc387ba5, SUC_lexicon_tags_63 },
    { 0x27987ba5, SUC_lexicon_tags_19 },
    { 0xb6d87ba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f87baa, SUC_lexicon_tags_25 },
    { 0xdb387baa, SUC_lexicon_tags_48 },
    { 0xe7587bac, SUC_lexicon_tags_48 },
    { 0x12187bab, SUC_lexicon_tags_0 },
    { 0xb0787bae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77387bb7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd4d87bb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c387bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66987bc1, SUC_lexicon_tags_25 },
    { 0x38787bc1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcfd87bc4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x36b87bc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9187bc9, SUC_lexicon_tags_2 },
    { 0x34387bc9, SUC_lexicon_tags_11 },
    { 0x7f387bcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d87bd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61b87bd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x10b87bd4, SUC_lexicon_tags_23 },
    { 0x5387bd5, SUC_lexicon_tags_33 },
    { 0x38587bd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9787bda, SUC_lexicon_tags_2 },
    { 0xec787bdb, SUC_lexicon_tags_11 },
    { 0x4e587bdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f87bea, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd87bed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef387bf0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f87bf8, SUC_lexicon_tags_56 },
    { 0x92b87bf9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7a187bfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6bb87bfd, SUC_lexicon_tags_2 },
    { 0x9d387bfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6587c00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39587c02, SUC_lexicon_tags_55 },
    { 0x88787c03, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b87c06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30787c0a, SUC_lexicon_tags_31 },
    { 0x67187c0a, SUC_lexicon_tags_8 },
    { 0xc9f87c0b, SUC_lexicon_tags_37 },
    { 0xd4787c0d, SUC_lexicon_tags_1 },
    { 0xf8f87c0c, SUC_lexicon_tags_17 },
    { 0x6d387c0b, SUC_lexicon_tags_15 },
    { 0xab587c0d, SUC_lexicon_tags_9 },
    { 0x91b87c0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0387c15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x43187c18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd87c1c, SUC_lexicon_tags_27 },
    { 0x8d787c1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2387c25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2387c2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1387c2c, SUC_lexicon_tags_47 },
    { 0xa7d87c2c, SUC_lexicon_tags_14 },
    { 0x94d87c2e, SUC_lexicon_tags_11 },
    { 0x99b87c2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xde987c32, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x65d87c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f187c36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef87c3b, SUC_lexicon_tags_12 },
    { 0x6f87c3b, SUC_lexicon_tags_1 },
    { 0x57f87c3d, SUC_lexicon_tags_11 },
    { 0x71987c3e, SUC_lexicon_tags_33 },
    { 0xa6987c3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b87c44, SUC_lexicon_tags_40 },
    { 0x95787c45, SUC_lexicon_tags_7 },
    { 0x18587c46, SUC_lexicon_tags_37 },
    { 0x86f87c45, SUC_lexicon_tags_38 },
    { 0x89987c44, SUC_lexicon_tags_37 },
    { 0x42f87c49, SUC_lexicon_tags_8 },
    { 0x87d87c49, SUC_lexicon_tags_37 },
    { 0x34187c45, SUC_lexicon_tags_6 },
    { 0x61587c47, SUC_lexicon_tags_15 },
    { 0x67187c49, SUC_lexicon_tags_48 },
    { 0x42f87c4e, SUC_lexicon_tags_12 },
    { 0xad87c4f, SUC_lexicon_tags_23 },
    { 0x6f787c4b, SUC_lexicon_tags_25 },
    { 0xad187c51, SUC_lexicon_tags_31 },
    { 0x2b387c52, SUC_lexicon_tags_19 },
    { 0x2e787c4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12187c5b, SUC_lexicon_tags_19 },
    { 0xa3787c5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f187c5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xce387c61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x10387c64, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x32b87c66, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb5d87c68, SUC_lexicon_tags_37 },
    { 0x2b987c69, SUC_lexicon_tags_117 },
    { 0x3f787c69, SUC_lexicon_tags_0 },
    { 0x45187c68, SUC_lexicon_tags_23 },
    { 0xf9387c6c, SUC_lexicon_tags_31 },
    { 0xc5b87c6c, SUC_lexicon_tags_87 },
    { 0x7f387c6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf87c71, SUC_lexicon_tags_14 },
    { 0xec987c72, SUC_lexicon_tags_2 },
    { 0xf5387c72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23187c77, SUC_lexicon_tags_27 },
    { 0xdad87c78, SUC_lexicon_tags_15 },
    { 0x8787c78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb87c7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc987c7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d87c81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa787c84, SUC_lexicon_tags_2 },
    { 0x3f187c85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b87c8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d587c8f, SUC_lexicon_tags_3 },
    { 0xfeb87c90, SUC_lexicon_tags_60 },
    { 0x89587c91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90987c95, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d87c9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x69387c9e, SUC_lexicon_tags_15 },
    { 0x74f87c9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf87ca2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c787ca8, SUC_lexicon_tags_11 },
    { 0x84b87ca9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbcb87cab, SUC_lexicon_tags_25 },
    { 0xe1d87cac, SUC_lexicon_tags_8 },
    { 0x71d87cab, SUC_lexicon_tags_17 },
    { 0xffd87cad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b87cb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb387cb3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd187cb5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed87cb8, SUC_lexicon_tags_7 },
    { 0x787cb8, SUC_lexicon_tags_34 },
    { 0xf2f87cb8, SUC_lexicon_tags_25 },
    { 0xacd87cb8, SUC_lexicon_tags_7 },
    { 0xb7587cba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72b87cbe, SUC_lexicon_tags_33 },
    { 0xd4d87cbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x93787cc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a187cc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d87cc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1587cca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10187ccc, SUC_lexicon_tags_2 },
    { 0x39587ccd, SUC_lexicon_tags_17 },
    { 0xf8787ccd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86987cd3, SUC_lexicon_tags_32 },
    { 0xd7d87cd4, SUC_lexicon_tags_47 },
    { 0x34987cd4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9f387cd7, SUC_lexicon_tags_15 },
    { 0xe9d87cd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d87cdd, SUC_lexicon_tags_48 },
    { 0x7db87cde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb87ce2, SUC_lexicon_tags_1 },
    { 0x91387ce2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac787ce6, SUC_lexicon_tags_4 },
    { 0x93187ce7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc187cea, SUC_lexicon_tags_9 },
    { 0x79d87ceb, SUC_lexicon_tags_11 },
    { 0xa3787ceb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd87cf1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xced87cf4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec387d00, SUC_lexicon_tags_25 },
    { 0xbf987d01, SUC_lexicon_tags_82 },
    { 0xbf87d01, SUC_lexicon_tags_12 },
    { 0x3b787d01, SUC_lexicon_tags_42 },
    { 0x7d787d03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c187d06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b87d0c, SUC_lexicon_tags_11 },
    { 0xea787d0d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x31d87d0f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e787d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb587d14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe187d1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69b87d1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x987d21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f787d27, SUC_lexicon_tags_4 },
    { 0x3ef87d28, SUC_lexicon_tags_1 },
    { 0x3d387d29, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87987d2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d87d30, SUC_lexicon_tags_1 },
    { 0x59187d31, SUC_lexicon_tags_65 },
    { 0x1cd87d32, SUC_lexicon_tags_43 },
    { 0x5b987d32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f387d38, SUC_lexicon_tags_74 },
    { 0x89787d39, SUC_lexicon_tags_15 },
    { 0x10387d3a, SUC_lexicon_tags_11 },
    { 0x17987d38, SUC_lexicon_tags_0 },
    { 0x5a387d39, SUC_lexicon_tags_43 },
    { 0x56587d3c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b87d41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x64587d43, SUC_lexicon_tags_43 },
    { 0xb587d43, SUC_lexicon_tags_43 },
    { 0xd4587d44, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5787d49, SUC_lexicon_tags_7 },
    { 0xfb987d49, SUC_lexicon_tags_15 },
    { 0xe7f87d4a, SUC_lexicon_tags_82 },
    { 0x5f387d4b, SUC_lexicon_tags_7 },
    { 0x74d87d4d, SUC_lexicon_tags_25 },
    { 0x32587d4e, SUC_lexicon_tags_28 },
    { 0x20d87d4e, SUC_lexicon_tags_37 },
    { 0x36187d50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed87d53, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf8987d55, SUC_lexicon_tags_87 },
    { 0x17b87d56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e587d58, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x57f87d5a, SUC_lexicon_tags_11 },
    { 0x73987d5b, SUC_lexicon_tags_14 },
    { 0xe2787d5c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0587d5f, SUC_lexicon_tags_17 },
    { 0x62987d5f, SUC_lexicon_tags_1 },
    { 0x73f87d61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb0987d63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9587d65, SUC_lexicon_tags_9 },
    { 0x4b587d65, SUC_lexicon_tags_12 },
    { 0x70387d67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc587d6c, SUC_lexicon_tags_37 },
    { 0x7db87d6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5387d74, SUC_lexicon_tags_33 },
    { 0x92987d75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b987d7b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b87d84, SUC_lexicon_tags_55 },
    { 0x76587d85, SUC_lexicon_tags_45 },
    { 0x35987d84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6787d8c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc3987d8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5c187d90, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4787d96, SUC_lexicon_tags_30 },
    { 0xe6f87d97, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d87d9b, SUC_lexicon_tags_37 },
    { 0xd4987d9c, SUC_lexicon_tags_1 },
    { 0xc2987d9b, SUC_lexicon_tags_25 },
    { 0xbd387d9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd87da2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4787da4, SUC_lexicon_tags_37 },
    { 0xbc787da4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec187da7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c787da9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f987dab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75187dad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b87db1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x21d87db3, SUC_lexicon_tags_63 },
    { 0xe5d87db4, SUC_lexicon_tags_46 },
    { 0xfe987db4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x65787db7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd87dbe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8187dc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18587dd3, SUC_lexicon_tags_112 },
    { 0x8c987dd3, SUC_lexicon_tags_11 },
    { 0x5fd87dd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7db87dd7, SUC_lexicon_tags_14 },
    { 0xb4b87dd7, SUC_lexicon_tags_37 },
    { 0xcd787dd8, SUC_lexicon_tags_25 },
    { 0x5e787dda, SUC_lexicon_tags_38 },
    { 0x90787dd7, SUC_lexicon_tags_25 },
    { 0x2c187dd8, SUC_lexicon_tags_2 },
    { 0x4e187ddd, SUC_lexicon_tags_28 },
    { 0xd5987dde, SUC_lexicon_tags_37 },
    { 0xf3587dde, SUC_lexicon_tags_29 },
    { 0x3bb87ddf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7d87de2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a387de5, SUC_lexicon_tags_38 },
    { 0xf0187de5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77987dea, SUC_lexicon_tags_16 },
    { 0xdc387deb, SUC_lexicon_tags_661 },
    { 0x58987dea, SUC_lexicon_tags_8 },
    { 0x8a387dec, SUC_lexicon_tags_47 },
    { 0xa4787dec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4cf87df0, SUC_lexicon_tags_11 },
    { 0x1db87df1, SUC_lexicon_tags_17 },
    { 0x7ef87df0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3987df6, SUC_lexicon_tags_2 },
    { 0xc9f87df6, SUC_lexicon_tags_65 },
    { 0x76987df6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdbb87dfa, SUC_lexicon_tags_82 },
    { 0x4e387dfa, SUC_lexicon_tags_1 },
    { 0xd5387dfc, SUC_lexicon_tags_25 },
    { 0x30187dfd, SUC_lexicon_tags_17 },
    { 0xa1f87dfd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd187e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53987e07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e587e0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c387e0f, SUC_lexicon_tags_12 },
    { 0x4ef87e10, SUC_lexicon_tags_29 },
    { 0x3b87e11, SUC_lexicon_tags_17 },
    { 0x9e587e10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6187e16, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91787e1a, SUC_lexicon_tags_37 },
    { 0x16387e1a, SUC_lexicon_tags_41 },
    { 0xb7b87e1c, SUC_lexicon_tags_17 },
    { 0xdff87e1d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f87e21, SUC_lexicon_tags_26 },
    { 0xa6387e21, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x13787e24, SUC_lexicon_tags_45 },
    { 0x1a587e24, SUC_lexicon_tags_37 },
    { 0x3d87e26, SUC_lexicon_tags_17 },
    { 0x1a987e27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77187e2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9387e2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b87e33, SUC_lexicon_tags_37 },
    { 0x88f87e34, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6987e37, SUC_lexicon_tags_27 },
    { 0xa0d87e38, SUC_lexicon_tags_16 },
    { 0xb9987e39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0587e3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeab87e3d, SUC_lexicon_tags_45 },
    { 0xc5387e3d, SUC_lexicon_tags_37 },
    { 0x84387e3f, SUC_lexicon_tags_21 },
    { 0xab87e3e, SUC_lexicon_tags_1 },
    { 0xdb587e40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9f87e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc787e4d, SUC_lexicon_tags_11 },
    { 0x8f187e4e, SUC_lexicon_tags_1 },
    { 0xa7787e4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c787e51, SUC_lexicon_tags_11 },
    { 0xa3587e52, SUC_lexicon_tags_23 },
    { 0x5bb87e51, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9787e57, SUC_lexicon_tags_63 },
    { 0xe7b87e57, SUC_lexicon_tags_39 },
    { 0x2ad87e59, SUC_lexicon_tags_11 },
    { 0xf8787e57, SUC_lexicon_tags_12 },
    { 0xaed87e5b, SUC_lexicon_tags_17 },
    { 0x4f87e59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d87e5f, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd187e62, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f87e65, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b87e68, SUC_lexicon_tags_12 },
    { 0x70b87e69, SUC_lexicon_tags_66 },
    { 0xc7f87e6a, SUC_lexicon_tags_33 },
    { 0x81f87e69, SUC_lexicon_tags_7 },
    { 0xb0787e6c, SUC_lexicon_tags_11 },
    { 0xeab87e6d, SUC_lexicon_tags_28 },
    { 0xf3387e6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef87e75, SUC_lexicon_tags_14 },
    { 0x31787e76, SUC_lexicon_tags_49 },
    { 0x13987e76, SUC_lexicon_tags_87 },
    { 0x7a587e78, SUC_lexicon_tags_8 },
    { 0x21b87e76, SUC_lexicon_tags_8 },
    { 0x74787e77, SUC_lexicon_tags_17 },
    { 0xa5387e7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdbd87e7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f87e81, SUC_lexicon_tags_17 },
    { 0xbff87e81, SUC_lexicon_tags_23 },
    { 0xbe387e81, SUC_lexicon_tags_15 },
    { 0xf8f87e83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8587e86, SUC_lexicon_tags_8 },
    { 0xdeb87e87, SUC_lexicon_tags_7 },
    { 0xadf87e86, SUC_lexicon_tags_17 },
    { 0x63b87e89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd87e8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b187e90, SUC_lexicon_tags_37 },
    { 0x87387e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf587e95, SUC_lexicon_tags_30 },
    { 0xccf87e96, SUC_lexicon_tags_23 },
    { 0x57d87e95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af87e9b, SUC_lexicon_tags_26 },
    { 0x2d987e9b, SUC_lexicon_tags_43 },
    { 0xb0d87e9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95987ea9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67b87eab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x39387ead, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x28587eaf, SUC_lexicon_tags_18 },
    { 0x8bf87eb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67987eb2, SUC_lexicon_tags_30 },
    { 0xc6187eb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d87eb8, SUC_lexicon_tags_198 },
    { 0x8d987eb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f87ebd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcc987ebf, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4787ec2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56787ec5, SUC_lexicon_tags_11 },
    { 0xb5187ec5, SUC_lexicon_tags_37 },
    { 0xea587ec7, SUC_lexicon_tags_37 },
    { 0xa1587ec6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd587eca, SUC_lexicon_tags_43 },
    { 0x97f87ecb, SUC_lexicon_tags_31 },
    { 0x56387ecc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18987ed4, SUC_lexicon_tags_19 },
    { 0xf1787ed5, SUC_lexicon_tags_10 },
    { 0x30f87ed4, SUC_lexicon_tags_37 },
    { 0x9c387ed7, SUC_lexicon_tags_43 },
    { 0xebd87ed4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd2d87eda, SUC_lexicon_tags_7 },
    { 0x90f87eda, SUC_lexicon_tags_37 },
    { 0x45587edc, SUC_lexicon_tags_5 },
    { 0x38787edc, SUC_lexicon_tags_17 },
    { 0xac587ede, SUC_lexicon_tags_37 },
    { 0x5a987edf, SUC_lexicon_tags_25 },
    { 0x4dd87edd, SUC_lexicon_tags_45 },
    { 0x97f87edb, SUC_lexicon_tags_32 },
    { 0x7d987ee2, SUC_lexicon_tags_17 },
    { 0x4f387ee1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92587ee7, SUC_lexicon_tags_11 },
    { 0xb4787ee8, SUC_lexicon_tags_46 },
    { 0x82387ee9, SUC_lexicon_tags_7 },
    { 0x3cf87ee8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab87ef2, SUC_lexicon_tags_39 },
    { 0xe5d87ef3, SUC_lexicon_tags_2 },
    { 0x9bb87ef3, SUC_lexicon_tags_7 },
    { 0x42f87ef4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcf187ef7, SUC_lexicon_tags_23 },
    { 0xd5b87ef7, SUC_lexicon_tags_63 },
    { 0x2af87ef9, SUC_lexicon_tags_12 },
    { 0x66787efa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f787f00, SUC_lexicon_tags_28 },
    { 0xbc387f00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87987f03, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b87f0a, SUC_lexicon_tags_15 },
    { 0x7c987f0b, SUC_lexicon_tags_82 },
    { 0xf6787f0b, SUC_lexicon_tags_37 },
    { 0x31d87f0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42587f0f, SUC_lexicon_tags_25 },
    { 0xb6387f10, SUC_lexicon_tags_17 },
    { 0xb6987f11, SUC_lexicon_tags_15 },
    { 0x3ab87f12, SUC_lexicon_tags_11 },
    { 0x90587f12, SUC_lexicon_tags_37 },
    { 0xffb87f12, SUC_lexicon_tags_13 },
    { 0x5787f10, SUC_lexicon_tags_39 },
    { 0x67f87f12, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b87f1b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92587f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187f24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d87f2a, SUC_lexicon_tags_1 },
    { 0x94787f2b, SUC_lexicon_tags_43 },
    { 0x5b187f2c, SUC_lexicon_tags_37 },
    { 0x42b87f2c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad87f30, SUC_lexicon_tags_25 },
    { 0xd3d87f31, SUC_lexicon_tags_47 },
    { 0xe0587f32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5987f39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b387f3d, SUC_lexicon_tags_23 },
    { 0x69d87f3d, SUC_lexicon_tags_49 },
    { 0xdd187f3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98787f42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7187f44, SUC_lexicon_tags_9 },
    { 0xadd87f45, SUC_lexicon_tags_6 },
    { 0xb5b87f46, SUC_lexicon_tags_35 },
    { 0xb7387f47, SUC_lexicon_tags_45 },
    { 0x1bd87f48, SUC_lexicon_tags_15 },
    { 0x98387f49, SUC_lexicon_tags_55 },
    { 0x44587f4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf7b87f4c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa0187f4e, SUC_lexicon_tags_45 },
    { 0x8d387f4e, SUC_lexicon_tags_26 },
    { 0x8e387f4e, SUC_lexicon_tags_37 },
    { 0x40987f50, SUC_lexicon_tags_47 },
    { 0x7df87f52, SUC_lexicon_tags_27 },
    { 0x98b87f53, SUC_lexicon_tags_1 },
    { 0x25b87f54, SUC_lexicon_tags_0 },
    { 0x70f87f50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a387f5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad587f62, SUC_lexicon_tags_16 },
    { 0xba387f63, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d87f67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52387f6b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b87f6e, SUC_lexicon_tags_15 },
    { 0xd587f6f, SUC_lexicon_tags_1 },
    { 0x6f387f70, SUC_lexicon_tags_38 },
    { 0x6987f70, SUC_lexicon_tags_26 },
    { 0x84b87f72, SUC_lexicon_tags_23 },
    { 0xbdf87f73, SUC_lexicon_tags_10 },
    { 0xbbf87f74, SUC_lexicon_tags_4 },
    { 0xf5987f6f, SUC_lexicon_tags_2 },
    { 0x74f87f70, SUC_lexicon_tags_0 },
    { 0xeb787f77, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6e787f79, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x65f87f7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f87f7e, SUC_lexicon_tags_11 },
    { 0xbd587f7f, SUC_lexicon_tags_6 },
    { 0x28f87f7f, SUC_lexicon_tags_37 },
    { 0xc5987f7f, SUC_lexicon_tags_19 },
    { 0x9b987f82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db87f85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf587f87, SUC_lexicon_tags_10 },
    { 0x9db87f87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b87f8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b87f91, SUC_lexicon_tags_25 },
    { 0x29d87f92, SUC_lexicon_tags_28 },
    { 0x8ed87f93, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x85187f96, SUC_lexicon_tags_47 },
    { 0x66b87f97, SUC_lexicon_tags_8 },
    { 0x4ff87f98, SUC_lexicon_tags_37 },
    { 0x9e387f97, SUC_lexicon_tags_25 },
    { 0x87d87f96, SUC_lexicon_tags_29 },
    { 0x68187f9b, SUC_lexicon_tags_30 },
    { 0xf8587f9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b87fa3, SUC_lexicon_tags_57 },
    { 0x85987fa4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f87fa7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x76387fa9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f87fac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b87fb0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x42f87fb2, SUC_lexicon_tags_82 },
    { 0xc8387fb3, SUC_lexicon_tags_49 },
    { 0x9e787fb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43187fba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf187fbd, SUC_lexicon_tags_25 },
    { 0x1dd87fbd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x73987fc0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff87fc3, SUC_lexicon_tags_1 },
    { 0x73587fc4, SUC_lexicon_tags_28 },
    { 0x1f187fc4, SUC_lexicon_tags_29 },
    { 0xba187fc6, SUC_lexicon_tags_12 },
    { 0xfd387fc7, SUC_lexicon_tags_1 },
    { 0xd3d87fc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7187fca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2387fcf, SUC_lexicon_tags_19 },
    { 0xf0d87fd0, SUC_lexicon_tags_45 },
    { 0x54587fcf, SUC_lexicon_tags_19 },
    { 0x5a787fd1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf587fd8, SUC_lexicon_tags_11 },
    { 0x47787fd9, SUC_lexicon_tags_49 },
    { 0xaf187fda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xead87fdc, SUC_lexicon_tags_198 },
    { 0x17787fdc, SUC_lexicon_tags_7 },
    { 0xff987fdd, SUC_lexicon_tags_47 },
    { 0x6bf87fdf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69787fe5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaeb87fe7, SUC_lexicon_tags_9 },
    { 0x66987fe8, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed87fec, SUC_lexicon_tags_45 },
    { 0x8b987fec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1387ff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5587ffa, SUC_lexicon_tags_23 },
    { 0x2187ffa, SUC_lexicon_tags_15 },
    { 0x67f87ffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f187fff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb88001, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61188003, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa588007, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3d388009, SUC_lexicon_tags_28 },
    { 0x7298800a, SUC_lexicon_tags_64 },
    { 0x6bf8800a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b988010, SUC_lexicon_tags_25 },
    { 0xf2988011, SUC_lexicon_tags_11 },
    { 0x6d588011, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed88016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a588019, SUC_lexicon_tags_23 },
    { 0x7858801a, SUC_lexicon_tags_7 },
    { 0x76d8801b, SUC_lexicon_tags_9 },
    { 0x2318801c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c58801e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b88022, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8bb88024, SUC_lexicon_tags_23 },
    { 0x6cd88025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed8802b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaef8802d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe078802f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa8388031, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x98188033, SUC_lexicon_tags_19 },
    { 0x40b88033, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f88038, SUC_lexicon_tags_19 },
    { 0xa7388038, SUC_lexicon_tags_7 },
    { 0xc178803a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3178803c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa078803e, SUC_lexicon_tags_33 },
    { 0x1bf8803f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df88044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d88047, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d8804d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2cd8804f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbaf88051, SUC_lexicon_tags_42 },
    { 0xcb88052, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c388055, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf388058, SUC_lexicon_tags_45 },
    { 0x10d88059, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd8805c, SUC_lexicon_tags_45 },
    { 0xbaf8805d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbcb8805f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e588061, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe7d88063, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd388067, SUC_lexicon_tags_87 },
    { 0xf5f88067, SUC_lexicon_tags_46 },
    { 0x59388069, SUC_lexicon_tags_529 },
    { 0xd238806a, SUC_lexicon_tags_45 },
    { 0xe018806a, SUC_lexicon_tags_25 },
    { 0x73588069, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6638806e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcdf88070, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af88074, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd498807a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3578807c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xeb78807e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x91f88080, SUC_lexicon_tags_37 },
    { 0xc7988081, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e988086, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa978808c, SUC_lexicon_tags_31 },
    { 0xcd98808d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6388091, SUC_lexicon_tags_21 },
    { 0x588091, SUC_lexicon_tags_97 },
    { 0x99d88091, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed88096, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x81788098, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6f58809a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x60b8809c, SUC_lexicon_tags_11 },
    { 0x9d98809c, SUC_lexicon_tags_28 },
    { 0xdb98809c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d880a4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343880a9, SUC_lexicon_tags_47 },
    { 0xc25880aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x415880ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf880af, SUC_lexicon_tags_26 },
    { 0x9b5880b0, SUC_lexicon_tags_47 },
    { 0x5b880b0, SUC_lexicon_tags_26 },
    { 0x81d880b2, SUC_lexicon_tags_17 },
    { 0x449880b3, SUC_lexicon_tags_23 },
    { 0x193880b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b880b6, SUC_lexicon_tags_9 },
    { 0x58f880b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd39880b9, SUC_lexicon_tags_84 },
    { 0xf5d880ba, SUC_lexicon_tags_25 },
    { 0x3b5880ba, SUC_lexicon_tags_7 },
    { 0x96b880bc, SUC_lexicon_tags_37 },
    { 0xd2f880bd, SUC_lexicon_tags_33 },
    { 0x81880bd, SUC_lexicon_tags_37 },
    { 0xaab880ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab880c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b880c5, SUC_lexicon_tags_33 },
    { 0xeff880c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275880ca, SUC_lexicon_tags_46 },
    { 0xe39880ca, SUC_lexicon_tags_14 },
    { 0x1ab880cc, SUC_lexicon_tags_45 },
    { 0x84d880cd, SUC_lexicon_tags_25 },
    { 0xd4d880ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd5880d0, SUC_lexicon_tags_37 },
    { 0x693880d0, SUC_lexicon_tags_8 },
    { 0x597880d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285880d6, SUC_lexicon_tags_14 },
    { 0x1d880d6, SUC_lexicon_tags_29 },
    { 0x415880d8, SUC_lexicon_tags_37 },
    { 0xd6f880d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867880df, SUC_lexicon_tags_49 },
    { 0x51f880e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x755880e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d880e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd880e8, SUC_lexicon_tags_8 },
    { 0x927880e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x589880ec, SUC_lexicon_tags_37 },
    { 0x835880ed, SUC_lexicon_tags_12 },
    { 0x929880ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327880f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127880f8, SUC_lexicon_tags_0 },
    { 0xd95880f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x363880fc, SUC_lexicon_tags_15 },
    { 0x881880fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec188105, SUC_lexicon_tags_34 },
    { 0x3d388106, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79388108, SUC_lexicon_tags_31 },
    { 0x2a988109, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8318810c, SUC_lexicon_tags_15 },
    { 0xa538810d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf88112, SUC_lexicon_tags_37 },
    { 0xb7388113, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe898811c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42988121, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67588123, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2058812a, SUC_lexicon_tags_62 },
    { 0xb98812b, SUC_lexicon_tags_8 },
    { 0xfb38812b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1198812e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80b88130, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f88135, SUC_lexicon_tags_60 },
    { 0xe4d88136, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d38813a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb38813e, SUC_lexicon_tags_293 },
    { 0xb878813e, SUC_lexicon_tags_25 },
    { 0x7f58813f, SUC_lexicon_tags_15 },
    { 0xa158813f, SUC_lexicon_tags_17 },
    { 0x59d88142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d88148, SUC_lexicon_tags_7 },
    { 0x6ad88148, SUC_lexicon_tags_87 },
    { 0xa3d8814a, SUC_lexicon_tags_17 },
    { 0x29188148, SUC_lexicon_tags_49 },
    { 0x1d18814a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x79d8814e, SUC_lexicon_tags_15 },
    { 0xc0d8814f, SUC_lexicon_tags_32 },
    { 0xeed88150, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d88153, SUC_lexicon_tags_1 },
    { 0x22988154, SUC_lexicon_tags_23 },
    { 0x2c988153, SUC_lexicon_tags_6 },
    { 0xbc588154, SUC_lexicon_tags_45 },
    { 0xb788154, SUC_lexicon_tags_12 },
    { 0x61188153, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b8815d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b88161, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c788165, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x70788167, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8098816a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1758816c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb58816f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x68588171, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a988173, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7388176, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b588179, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b8817c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf3d8817e, SUC_lexicon_tags_63 },
    { 0x5e38817f, SUC_lexicon_tags_15 },
    { 0xa4b88180, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d188188, SUC_lexicon_tags_2 },
    { 0x82388189, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3378818e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a988192, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83b88194, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x91588196, SUC_lexicon_tags_15 },
    { 0x21b88197, SUC_lexicon_tags_12 },
    { 0x87788198, SUC_lexicon_tags_5 },
    { 0x26f88199, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc178819f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaed881a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b881a5, SUC_lexicon_tags_49 },
    { 0x9a3881a6, SUC_lexicon_tags_15 },
    { 0x417881a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fb881a9, SUC_lexicon_tags_25 },
    { 0xb5b881aa, SUC_lexicon_tags_27 },
    { 0x6f9881a9, SUC_lexicon_tags_8 },
    { 0xf57881aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x873881ae, SUC_lexicon_tags_8 },
    { 0x3e1881ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1b7881b1, SUC_lexicon_tags_63 },
    { 0x837881b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59881ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x491881bc, SUC_lexicon_tags_37 },
    { 0x3eb881bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1881c8, SUC_lexicon_tags_79 },
    { 0x439881c9, SUC_lexicon_tags_15 },
    { 0xdcb881c9, SUC_lexicon_tags_25 },
    { 0xfbf881cb, SUC_lexicon_tags_15 },
    { 0x645881cb, SUC_lexicon_tags_21 },
    { 0x1b1881c9, SUC_lexicon_tags_12 },
    { 0x61881cb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b881d3, SUC_lexicon_tags_33 },
    { 0xfad881d4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d881d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad1881da, SUC_lexicon_tags_16 },
    { 0x4ed881db, SUC_lexicon_tags_179 },
    { 0xfeb881db, SUC_lexicon_tags_25 },
    { 0xf1b881dc, SUC_lexicon_tags_7 },
    { 0x25b881de, SUC_lexicon_tags_11 },
    { 0x81881df, SUC_lexicon_tags_10 },
    { 0x27b881e0, SUC_lexicon_tags_1 },
    { 0x9e5881e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65881e7, SUC_lexicon_tags_38 },
    { 0xf5881e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779881ed, SUC_lexicon_tags_17 },
    { 0x487881ee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f881f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0188200, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71588204, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0588208, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef58820c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5988210, SUC_lexicon_tags_44 },
    { 0x93388210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba388215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5588218, SUC_lexicon_tags_15 },
    { 0x88188218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc978821b, SUC_lexicon_tags_43 },
    { 0x84d8821c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6978821f, SUC_lexicon_tags_38 },
    { 0xc9b88220, SUC_lexicon_tags_65 },
    { 0xb3b88221, SUC_lexicon_tags_49 },
    { 0x48388222, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe5188224, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4988228, SUC_lexicon_tags_34 },
    { 0x61788229, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x65f8822b, SUC_lexicon_tags_11 },
    { 0x77f8822b, SUC_lexicon_tags_13 },
    { 0xed18822d, SUC_lexicon_tags_24 },
    { 0x21b8822e, SUC_lexicon_tags_37 },
    { 0x718822f, SUC_lexicon_tags_61 },
    { 0x8b18822e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff88233, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65188237, SUC_lexicon_tags_10 },
    { 0xee588237, SUC_lexicon_tags_17 },
    { 0xb3588238, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1818823b, SUC_lexicon_tags_7 },
    { 0x3bf8823c, SUC_lexicon_tags_15 },
    { 0xd318823d, SUC_lexicon_tags_108 },
    { 0x30d8823d, SUC_lexicon_tags_14 },
    { 0x2b98823c, SUC_lexicon_tags_27 },
    { 0x96788240, SUC_lexicon_tags_4 },
    { 0xd5188241, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11188244, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b88248, SUC_lexicon_tags_25 },
    { 0x3c188249, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f78824c, SUC_lexicon_tags_4 },
    { 0x4858824d, SUC_lexicon_tags_7 },
    { 0xb718824e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x52788250, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5cb88252, SUC_lexicon_tags_79 },
    { 0x3fd88253, SUC_lexicon_tags_27 },
    { 0x988253, SUC_lexicon_tags_12 },
    { 0x14588255, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa588257, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d8825b, SUC_lexicon_tags_32 },
    { 0x1158825b, SUC_lexicon_tags_20 },
    { 0x94b8825b, SUC_lexicon_tags_7 },
    { 0x9538825e, SUC_lexicon_tags_1 },
    { 0x8ad8825d, SUC_lexicon_tags_15 },
    { 0x7d988260, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2c388262, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12788266, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad88269, SUC_lexicon_tags_17 },
    { 0xc018826a, SUC_lexicon_tags_136 },
    { 0xf8f8826b, SUC_lexicon_tags_80 },
    { 0x5bb88269, SUC_lexicon_tags_14 },
    { 0x5558826d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd238826f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b88275, SUC_lexicon_tags_1 },
    { 0xd5b88276, SUC_lexicon_tags_6 },
    { 0x18788277, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe278827b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f38827d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27588281, SUC_lexicon_tags_21 },
    { 0x2b188281, SUC_lexicon_tags_11 },
    { 0x9b88283, SUC_lexicon_tags_25 },
    { 0x53188284, SUC_lexicon_tags_129 },
    { 0x1788285, SUC_lexicon_tags_84 },
    { 0x9cd88285, SUC_lexicon_tags_15 },
    { 0xcd988287, SUC_lexicon_tags_2 },
    { 0x4b188288, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3498828a, SUC_lexicon_tags_12 },
    { 0xe2f8828a, SUC_lexicon_tags_1 },
    { 0x2c98828a, SUC_lexicon_tags_15 },
    { 0x8898828c, SUC_lexicon_tags_11 },
    { 0x93d8828e, SUC_lexicon_tags_40 },
    { 0xc4d8828f, SUC_lexicon_tags_27 },
    { 0x9fb8828f, SUC_lexicon_tags_7 },
    { 0xf4788291, SUC_lexicon_tags_38 },
    { 0xf4988292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x24788295, SUC_lexicon_tags_34 },
    { 0x1bf88296, SUC_lexicon_tags_47 },
    { 0xf6788296, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b58829c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb882a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835882af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137882b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x729882b9, SUC_lexicon_tags_10 },
    { 0x35d882b9, SUC_lexicon_tags_27 },
    { 0x495882bb, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x215882be, SUC_lexicon_tags_107 },
    { 0xef3882bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xded882c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f882c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b882cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb882d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb882d4, SUC_lexicon_tags_17 },
    { 0x8fb882d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5882d9, SUC_lexicon_tags_0 },
    { 0x129882da, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7882dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9882e2, SUC_lexicon_tags_63 },
    { 0xd0b882e2, SUC_lexicon_tags_1 },
    { 0xe6b882e4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd19882e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed882f0, SUC_lexicon_tags_49 },
    { 0xec1882f1, SUC_lexicon_tags_8 },
    { 0x42b882f2, SUC_lexicon_tags_12 },
    { 0xcef882f2, SUC_lexicon_tags_16 },
    { 0x4a1882f4, SUC_lexicon_tags_48 },
    { 0xcbd882f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f882f8, SUC_lexicon_tags_57 },
    { 0x405882f9, SUC_lexicon_tags_11 },
    { 0xe93882fa, SUC_lexicon_tags_112 },
    { 0x39b882fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd35882fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x765882ff, SUC_lexicon_tags_411 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b388303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7588309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b8830c, SUC_lexicon_tags_87 },
    { 0x7918830d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f88311, SUC_lexicon_tags_25 },
    { 0x7ad88312, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3388314, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x90588317, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2588321, SUC_lexicon_tags_12 },
    { 0xd7588321, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7788324, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d588327, SUC_lexicon_tags_2 },
    { 0x7588327, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc178832d, SUC_lexicon_tags_12 },
    { 0x2bf8832e, SUC_lexicon_tags_60 },
    { 0x9ff8832d, SUC_lexicon_tags_11 },
    { 0x42388330, SUC_lexicon_tags_84 },
    { 0xff188331, SUC_lexicon_tags_22 },
    { 0x25788331, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x88b88334, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1388336, SUC_lexicon_tags_11 },
    { 0xf9988336, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d38833c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2458833e, SUC_lexicon_tags_63 },
    { 0xe458833f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87588345, SUC_lexicon_tags_35 },
    { 0x97d88346, SUC_lexicon_tags_60 },
    { 0x5e788347, SUC_lexicon_tags_12 },
    { 0x4188346, SUC_lexicon_tags_27 },
    { 0xa6788349, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7698834b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd118834d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0d8834f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf88354, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb988359, SUC_lexicon_tags_49 },
    { 0x11b8835a, SUC_lexicon_tags_1 },
    { 0x4bd8835b, SUC_lexicon_tags_10 },
    { 0x1f38835c, SUC_lexicon_tags_12 },
    { 0xeb58835d, SUC_lexicon_tags_27 },
    { 0x3ed88359, SUC_lexicon_tags_1 },
    { 0x51b8835f, SUC_lexicon_tags_82 },
    { 0xd2188359, SUC_lexicon_tags_46 },
    { 0x1f388361, SUC_lexicon_tags_107 },
    { 0xa1b88362, SUC_lexicon_tags_40 },
    { 0x9dd88362, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb988366, SUC_lexicon_tags_32 },
    { 0x20f88367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a588369, SUC_lexicon_tags_12 },
    { 0xcfb88369, SUC_lexicon_tags_33 },
    { 0x2038836a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64388370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff388375, SUC_lexicon_tags_7 },
    { 0xd1f88376, SUC_lexicon_tags_17 },
    { 0x40588375, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb58837b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c58837d, SUC_lexicon_tags_91 },
    { 0x6bb8837e, SUC_lexicon_tags_38 },
    { 0x80b8837e, SUC_lexicon_tags_48 },
    { 0xf998837e, SUC_lexicon_tags_7 },
    { 0xe18837f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x70788384, SUC_lexicon_tags_49 },
    { 0x45f88385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1058838c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a988392, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x58388394, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa0188396, SUC_lexicon_tags_2 },
    { 0xf0788397, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb98839c, SUC_lexicon_tags_17 },
    { 0xa98839d, SUC_lexicon_tags_11 },
    { 0x3dd8839e, SUC_lexicon_tags_79 },
    { 0x1bb8839f, SUC_lexicon_tags_17 },
    { 0x135883a0, SUC_lexicon_tags_25 },
    { 0x857883a1, SUC_lexicon_tags_25 },
    { 0x9bd883a2, SUC_lexicon_tags_11 },
    { 0x485883a3, SUC_lexicon_tags_7 },
    { 0x55d883a4, SUC_lexicon_tags_237 },
    { 0xf67883a5, SUC_lexicon_tags_163 },
    { 0x853883a3, SUC_lexicon_tags_12 },
    { 0xfc5883a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x275883aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f883ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x973883b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d883b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43883b6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b883b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7883bd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d883c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69883c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce7883c5, SUC_lexicon_tags_11 },
    { 0xc99883c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad883c9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x273883cb, SUC_lexicon_tags_26 },
    { 0x5f9883cc, SUC_lexicon_tags_27 },
    { 0x24d883cd, SUC_lexicon_tags_23 },
    { 0x941883ce, SUC_lexicon_tags_2 },
    { 0x4d5883cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc5883d1, SUC_lexicon_tags_17 },
    { 0x60f883d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7883d6, SUC_lexicon_tags_163 },
    { 0x4b883d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d883dd, SUC_lexicon_tags_37 },
    { 0xd85883dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89883e3, SUC_lexicon_tags_0 },
    { 0xed7883e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09883e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381883ec, SUC_lexicon_tags_15 },
    { 0xdf9883ec, SUC_lexicon_tags_25 },
    { 0x57d883ec, SUC_lexicon_tags_43 },
    { 0x4a9883ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x851883f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x845883f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x807883f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9e1883f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db88402, SUC_lexicon_tags_38 },
    { 0x28988403, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd8840f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b88414, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd88417, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9198841b, SUC_lexicon_tags_4 },
    { 0x6558841b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58588428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d8842d, SUC_lexicon_tags_14 },
    { 0xac78842d, SUC_lexicon_tags_14 },
    { 0xf38842f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea988431, SUC_lexicon_tags_49 },
    { 0x55588431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79988437, SUC_lexicon_tags_3 },
    { 0x3f388438, SUC_lexicon_tags_1 },
    { 0x3a588437, SUC_lexicon_tags_38 },
    { 0xb378843a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9458843c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58588441, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f88445, SUC_lexicon_tags_10 },
    { 0xecb88446, SUC_lexicon_tags_14 },
    { 0x7c988447, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe178844a, SUC_lexicon_tags_2 },
    { 0x5f8844b, SUC_lexicon_tags_2 },
    { 0x7838844a, SUC_lexicon_tags_59 },
    { 0xd678844b, SUC_lexicon_tags_140 },
    { 0x72b8844a, SUC_lexicon_tags_38 },
    { 0xf978844f, SUC_lexicon_tags_39 },
    { 0x90388450, SUC_lexicon_tags_19 },
    { 0xa918844e, SUC_lexicon_tags_0 },
    { 0xe758844c, SUC_lexicon_tags_160 },
    { 0x89f8844e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86388455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd8845b, SUC_lexicon_tags_108 },
    { 0xc418845b, SUC_lexicon_tags_14 },
    { 0xca58845d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52388462, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2638846c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f88472, SUC_lexicon_tags_38 },
    { 0x85188472, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad88478, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24988480, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d88488, SUC_lexicon_tags_43 },
    { 0x8d588489, SUC_lexicon_tags_49 },
    { 0xb538848a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d8848d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa378848f, SUC_lexicon_tags_45 },
    { 0x40d88490, SUC_lexicon_tags_66 },
    { 0xc4988491, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a188493, SUC_lexicon_tags_2 },
    { 0x6c588494, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b88497, SUC_lexicon_tags_2 },
    { 0x1f988498, SUC_lexicon_tags_17 },
    { 0x3d188499, SUC_lexicon_tags_11 },
    { 0xdb8849a, SUC_lexicon_tags_33 },
    { 0xe3588497, SUC_lexicon_tags_37 },
    { 0x68b8849b, SUC_lexicon_tags_63 },
    { 0xc9788498, SUC_lexicon_tags_1 },
    { 0x2858849e, SUC_lexicon_tags_30 },
    { 0x16b8849c, SUC_lexicon_tags_11 },
    { 0x63d884a0, SUC_lexicon_tags_31 },
    { 0xead884a1, SUC_lexicon_tags_46 },
    { 0xead884a1, SUC_lexicon_tags_37 },
    { 0x863884a1, SUC_lexicon_tags_16 },
    { 0x709884a4, SUC_lexicon_tags_7 },
    { 0x307884a4, SUC_lexicon_tags_61 },
    { 0xe81884a6, SUC_lexicon_tags_11 },
    { 0xf23884a7, SUC_lexicon_tags_15 },
    { 0xa41884a7, SUC_lexicon_tags_79 },
    { 0xb03884a9, SUC_lexicon_tags_28 },
    { 0xb75884a9, SUC_lexicon_tags_45 },
    { 0xc2f884a5, SUC_lexicon_tags_22 },
    { 0xb7884aa, SUC_lexicon_tags_30 },
    { 0xbf1884ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349884b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5af884b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47884ba, SUC_lexicon_tags_31 },
    { 0x78d884bb, SUC_lexicon_tags_15 },
    { 0x8ef884bc, SUC_lexicon_tags_19 },
    { 0x3f1884bc, SUC_lexicon_tags_0 },
    { 0x24b884bd, SUC_lexicon_tags_49 },
    { 0x985884bc, SUC_lexicon_tags_1 },
    { 0x973884c0, SUC_lexicon_tags_2 },
    { 0xea3884be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc11884c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff884c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d884cc, SUC_lexicon_tags_26 },
    { 0x499884cd, SUC_lexicon_tags_14 },
    { 0x6b1884cd, SUC_lexicon_tags_47 },
    { 0x1ef884cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcd884d1, SUC_lexicon_tags_29 },
    { 0x9e5884d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3884d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb19884d9, SUC_lexicon_tags_2 },
    { 0xa75884da, SUC_lexicon_tags_11 },
    { 0xbd1884d9, SUC_lexicon_tags_25 },
    { 0x1cf884dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa05884de, SUC_lexicon_tags_65 },
    { 0x5ed884df, SUC_lexicon_tags_2 },
    { 0x33b884de, SUC_lexicon_tags_45 },
    { 0x8f1884e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1884e5, SUC_lexicon_tags_27 },
    { 0xc31884e5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xedb884e8, SUC_lexicon_tags_16 },
    { 0x435884e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1df884eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7884f6, SUC_lexicon_tags_8 },
    { 0x529884f7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb884fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d88500, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61588502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65b88504, SUC_lexicon_tags_28 },
    { 0xf988505, SUC_lexicon_tags_38 },
    { 0x29988506, SUC_lexicon_tags_14 },
    { 0xebb88507, SUC_lexicon_tags_1 },
    { 0xe0f88508, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x818850a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x14f8850d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2918850f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75788511, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e188515, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd738851b, SUC_lexicon_tags_1 },
    { 0xa5f8851b, SUC_lexicon_tags_7 },
    { 0x3878851b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2188524, SUC_lexicon_tags_2 },
    { 0xadd88524, SUC_lexicon_tags_17 },
    { 0xb3388524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd788529, SUC_lexicon_tags_27 },
    { 0x98d88529, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad588530, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59188535, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43188539, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf158853d, SUC_lexicon_tags_38 },
    { 0xc718853e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b188541, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xae388544, SUC_lexicon_tags_21 },
    { 0x7c188545, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f188549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb318854e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d388552, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc588555, SUC_lexicon_tags_26 },
    { 0x5f788556, SUC_lexicon_tags_28 },
    { 0xb5b88557, SUC_lexicon_tags_15 },
    { 0xb1d88558, SUC_lexicon_tags_18 },
    { 0x74388559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5338855b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f88561, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b88563, SUC_lexicon_tags_1 },
    { 0x82f88563, SUC_lexicon_tags_7 },
    { 0x72988563, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef88569, SUC_lexicon_tags_17 },
    { 0x28b8856a, SUC_lexicon_tags_66 },
    { 0x1558856b, SUC_lexicon_tags_1 },
    { 0x36b8856b, SUC_lexicon_tags_2 },
    { 0xbb58856d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc388570, SUC_lexicon_tags_63 },
    { 0x80188571, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2988575, SUC_lexicon_tags_2 },
    { 0x5e188576, SUC_lexicon_tags_38 },
    { 0x28188575, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa078857f, SUC_lexicon_tags_17 },
    { 0xaf788580, SUC_lexicon_tags_63 },
    { 0xb788581, SUC_lexicon_tags_0 },
    { 0xc8788582, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x90f88584, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef988586, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe38858b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb78858e, SUC_lexicon_tags_37 },
    { 0xb318858f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98388592, SUC_lexicon_tags_2 },
    { 0xa2788593, SUC_lexicon_tags_12 },
    { 0xe9388594, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef18859a, SUC_lexicon_tags_38 },
    { 0x30d8859b, SUC_lexicon_tags_9 },
    { 0x5c98859b, SUC_lexicon_tags_8 },
    { 0x1978859a, SUC_lexicon_tags_49 },
    { 0x15b8859c, SUC_lexicon_tags_2 },
    { 0xc238859f, SUC_lexicon_tags_57 },
    { 0x11d8859f, SUC_lexicon_tags_9 },
    { 0xfb9885a1, SUC_lexicon_tags_60 },
    { 0x31d885a2, SUC_lexicon_tags_28 },
    { 0xe3885a1, SUC_lexicon_tags_19 },
    { 0x8f9885a2, SUC_lexicon_tags_7 },
    { 0xcfb885a5, SUC_lexicon_tags_2 },
    { 0xeb885a6, SUC_lexicon_tags_23 },
    { 0xb85885a7, SUC_lexicon_tags_1 },
    { 0x541885a8, SUC_lexicon_tags_84 },
    { 0x429885a9, SUC_lexicon_tags_5 },
    { 0x81885a9, SUC_lexicon_tags_42 },
    { 0x9d3885ab, SUC_lexicon_tags_63 },
    { 0xe41885ab, SUC_lexicon_tags_19 },
    { 0xae1885ad, SUC_lexicon_tags_1 },
    { 0xdd7885ae, SUC_lexicon_tags_25 },
    { 0xd57885af, SUC_lexicon_tags_47 },
    { 0xcc3885ac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987885b5, SUC_lexicon_tags_17 },
    { 0x53d885b5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d885b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x797885bb, SUC_lexicon_tags_43 },
    { 0xacf885bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35885c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd885c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5885cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b885d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69885d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5885dc, SUC_lexicon_tags_11 },
    { 0xe1f885dd, SUC_lexicon_tags_26 },
    { 0xcad885de, SUC_lexicon_tags_16 },
    { 0xd43885df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe7f885e1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13885e4, SUC_lexicon_tags_1705 },
    { 0xbad885e5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe7f885e7, SUC_lexicon_tags_11 },
    { 0xa1b885e8, SUC_lexicon_tags_17 },
    { 0x909885e9, SUC_lexicon_tags_34 },
    { 0xfdf885ea, SUC_lexicon_tags_87 },
    { 0xa13885e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9885fa, SUC_lexicon_tags_1 },
    { 0x811885fb, SUC_lexicon_tags_38 },
    { 0x421885fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1788601, SUC_lexicon_tags_17 },
    { 0x3b788602, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4388607, SUC_lexicon_tags_34 },
    { 0xd5388607, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f38860c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54588610, SUC_lexicon_tags_14 },
    { 0xde588611, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0b88613, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab88618, SUC_lexicon_tags_15 },
    { 0x83788618, SUC_lexicon_tags_32 },
    { 0x27b8861a, SUC_lexicon_tags_5 },
    { 0xeb788619, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa938861d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed88621, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b88624, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b88627, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8b88629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36f8862b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f388631, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6988635, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3788637, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98f88639, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa638863c, SUC_lexicon_tags_2 },
    { 0x87d8863c, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf88645, SUC_lexicon_tags_25 },
    { 0x4a388646, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f8864c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2398864e, SUC_lexicon_tags_7 },
    { 0xc958864f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b588653, SUC_lexicon_tags_29 },
    { 0x14188654, SUC_lexicon_tags_39 },
    { 0x84788655, SUC_lexicon_tags_17 },
    { 0x62388656, SUC_lexicon_tags_46 },
    { 0xc0d88657, SUC_lexicon_tags_17 },
    { 0x87588658, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678865f, SUC_lexicon_tags_7 },
    { 0x1a988660, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d88663, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87f88665, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b588668, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8af8866a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82d8866c, SUC_lexicon_tags_7 },
    { 0x33b8866d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5158866f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc78867f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f388682, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x41788685, SUC_lexicon_tags_37 },
    { 0xa788686, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b88689, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d8868c, SUC_lexicon_tags_2 },
    { 0x1df8868c, SUC_lexicon_tags_2 },
    { 0x2fb8868e, SUC_lexicon_tags_45 },
    { 0xd798868f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb88692, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfc588694, SUC_lexicon_tags_1 },
    { 0x4a988694, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7138869f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d3886a1, SUC_lexicon_tags_107 },
    { 0x20f886a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b886a8, SUC_lexicon_tags_87 },
    { 0x515886a8, SUC_lexicon_tags_17 },
    { 0x1b1886aa, SUC_lexicon_tags_43 },
    { 0x74d886aa, SUC_lexicon_tags_37 },
    { 0xfad886aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b886b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x227886b8, SUC_lexicon_tags_17 },
    { 0x86f886b8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe2d886bb, SUC_lexicon_tags_277 },
    { 0xf2b886bc, SUC_lexicon_tags_35 },
    { 0x691886bd, SUC_lexicon_tags_10 },
    { 0x9eb886be, SUC_lexicon_tags_15 },
    { 0x3a3886bf, SUC_lexicon_tags_70 },
    { 0xaf9886be, SUC_lexicon_tags_46 },
    { 0x623886bd, SUC_lexicon_tags_27 },
    { 0x96b886c2, SUC_lexicon_tags_37 },
    { 0x8dd886c3, SUC_lexicon_tags_55 },
    { 0xd6d886c4, SUC_lexicon_tags_28 },
    { 0xbb886c5, SUC_lexicon_tags_63 },
    { 0x64f886c6, SUC_lexicon_tags_26 },
    { 0x75f886c7, SUC_lexicon_tags_12 },
    { 0x367886c2, SUC_lexicon_tags_17 },
    { 0xf41886c9, SUC_lexicon_tags_1 },
    { 0x965886c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3886cd, SUC_lexicon_tags_36 },
    { 0x6d1886ce, SUC_lexicon_tags_1 },
    { 0xf57886cd, SUC_lexicon_tags_29 },
    { 0x35d886d0, SUC_lexicon_tags_15 },
    { 0x919886d0, SUC_lexicon_tags_35 },
    { 0x4d886d2, SUC_lexicon_tags_16 },
    { 0x3cf886d3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759886d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x179886dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f886e0, SUC_lexicon_tags_27 },
    { 0x531886e1, SUC_lexicon_tags_37 },
    { 0x96d886e1, SUC_lexicon_tags_15 },
    { 0xb65886e3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf886e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb886f0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6e7886f2, SUC_lexicon_tags_23 },
    { 0x243886f2, SUC_lexicon_tags_116 },
    { 0xfa7886f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b886fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33886fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f88702, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeff88704, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc3b88706, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xbe988708, SUC_lexicon_tags_17 },
    { 0x90588708, SUC_lexicon_tags_28 },
    { 0x22b8870a, SUC_lexicon_tags_17 },
    { 0x8b98870b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xabf8870d, SUC_lexicon_tags_63 },
    { 0xa9d8870d, SUC_lexicon_tags_15 },
    { 0xd358870e, SUC_lexicon_tags_23 },
    { 0xb9f88710, SUC_lexicon_tags_17 },
    { 0xb2588711, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x46388714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c388717, SUC_lexicon_tags_11 },
    { 0x1d588717, SUC_lexicon_tags_1 },
    { 0x86d88719, SUC_lexicon_tags_11 },
    { 0xcfd8871a, SUC_lexicon_tags_25 },
    { 0x31188717, SUC_lexicon_tags_8 },
    { 0xfc58871c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f8871f, SUC_lexicon_tags_47 },
    { 0xf9f88720, SUC_lexicon_tags_63 },
    { 0xa5188721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a588723, SUC_lexicon_tags_11 },
    { 0x8c988724, SUC_lexicon_tags_15 },
    { 0x6f588724, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f988729, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3938872c, SUC_lexicon_tags_37 },
    { 0x40f8872d, SUC_lexicon_tags_28 },
    { 0xbbb8872e, SUC_lexicon_tags_22 },
    { 0xdc38872f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ff88731, SUC_lexicon_tags_15 },
    { 0x49f88732, SUC_lexicon_tags_11 },
    { 0x8b988731, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90d88735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29988737, SUC_lexicon_tags_32 },
    { 0x1cf88738, SUC_lexicon_tags_19 },
    { 0x35988739, SUC_lexicon_tags_28 },
    { 0xa278873a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf38873c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f88740, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2188743, SUC_lexicon_tags_19 },
    { 0x37988743, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd1788746, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf198874c, SUC_lexicon_tags_15 },
    { 0x4518874d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4988750, SUC_lexicon_tags_11 },
    { 0x1e388751, SUC_lexicon_tags_38 },
    { 0x15d88750, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea388756, SUC_lexicon_tags_25 },
    { 0x77b88756, SUC_lexicon_tags_17 },
    { 0x9f188756, SUC_lexicon_tags_49 },
    { 0xf6b88757, SUC_lexicon_tags_37 },
    { 0x1d58875a, SUC_lexicon_tags_37 },
    { 0xf4988756, SUC_lexicon_tags_17 },
    { 0x6b8875c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf88760, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec988763, SUC_lexicon_tags_47 },
    { 0x4c388763, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d88768, SUC_lexicon_tags_31 },
    { 0x7d388768, SUC_lexicon_tags_45 },
    { 0xa8188768, SUC_lexicon_tags_83 },
    { 0xe278876a, SUC_lexicon_tags_1 },
    { 0x3f788769, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xae18876e, SUC_lexicon_tags_13 },
    { 0x24f8876e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f88772, SUC_lexicon_tags_63 },
    { 0x90788773, SUC_lexicon_tags_37 },
    { 0x588772, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b88779, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6358877d, SUC_lexicon_tags_43 },
    { 0x4798877e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8788780, SUC_lexicon_tags_37 },
    { 0xb8788780, SUC_lexicon_tags_2 },
    { 0x6bd88782, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f588784, SUC_lexicon_tags_37 },
    { 0xb0388784, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d988788, SUC_lexicon_tags_2 },
    { 0x85988789, SUC_lexicon_tags_2 },
    { 0x4e988788, SUC_lexicon_tags_17 },
    { 0xda18878a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7758878f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b88795, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x60b88797, SUC_lexicon_tags_17 },
    { 0xb6188798, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe238879c, SUC_lexicon_tags_33 },
    { 0x5e78879c, SUC_lexicon_tags_11 },
    { 0x94b8879e, SUC_lexicon_tags_505 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29887a1, SUC_lexicon_tags_2 },
    { 0xb91887a1, SUC_lexicon_tags_11 },
    { 0xa09887a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151887a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe09887a9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x819887ac, SUC_lexicon_tags_0 },
    { 0x5f887ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x413887af, SUC_lexicon_tags_38 },
    { 0xf73887b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb7887b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63887b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35887bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad887c3, SUC_lexicon_tags_12 },
    { 0x5887c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437887ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87887d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3887d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdfb887d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07887da, SUC_lexicon_tags_56 },
    { 0x209887db, SUC_lexicon_tags_25 },
    { 0x63d887dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9887e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627887e5, SUC_lexicon_tags_14 },
    { 0xdb3887e6, SUC_lexicon_tags_1 },
    { 0x2ab887e7, SUC_lexicon_tags_15 },
    { 0x3e3887e6, SUC_lexicon_tags_15 },
    { 0x2cf887e9, SUC_lexicon_tags_1 },
    { 0x811887ea, SUC_lexicon_tags_9 },
    { 0x8ed887ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf887ef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf887f3, SUC_lexicon_tags_38 },
    { 0x1ff887f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3887f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01887fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x893887fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39b88800, SUC_lexicon_tags_7 },
    { 0xe6988800, SUC_lexicon_tags_17 },
    { 0x72788802, SUC_lexicon_tags_17 },
    { 0x55388803, SUC_lexicon_tags_7 },
    { 0x99f88802, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2538880b, SUC_lexicon_tags_47 },
    { 0xbe98880c, SUC_lexicon_tags_6 },
    { 0xa538880d, SUC_lexicon_tags_19 },
    { 0x8458880d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x24388810, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75788816, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaad88818, SUC_lexicon_tags_8 },
    { 0xac588819, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe038881b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb88820, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xad88823, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11588828, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5958882b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b8882e, SUC_lexicon_tags_2 },
    { 0xc78882e, SUC_lexicon_tags_15 },
    { 0x338882f, SUC_lexicon_tags_37 },
    { 0x4a588831, SUC_lexicon_tags_1 },
    { 0x1c588832, SUC_lexicon_tags_37 },
    { 0x4bf8882e, SUC_lexicon_tags_17 },
    { 0xdd188834, SUC_lexicon_tags_15 },
    { 0xf8d88835, SUC_lexicon_tags_8 },
    { 0x89d88836, SUC_lexicon_tags_173 },
    { 0xa5188837, SUC_lexicon_tags_0 },
    { 0x10388834, SUC_lexicon_tags_46 },
    { 0x4b988838, SUC_lexicon_tags_49 },
    { 0x8358883a, SUC_lexicon_tags_47 },
    { 0x5738883b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34d8883d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f388841, SUC_lexicon_tags_17 },
    { 0xf388842, SUC_lexicon_tags_17 },
    { 0x32788842, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7988848, SUC_lexicon_tags_30 },
    { 0xb5f88848, SUC_lexicon_tags_68 },
    { 0xef188848, SUC_lexicon_tags_2 },
    { 0x6bf8884b, SUC_lexicon_tags_37 },
    { 0x21b8884c, SUC_lexicon_tags_14 },
    { 0x7d18884d, SUC_lexicon_tags_19 },
    { 0x4698884b, SUC_lexicon_tags_4 },
    { 0xbe98884f, SUC_lexicon_tags_23 },
    { 0x89d8884f, SUC_lexicon_tags_1 },
    { 0x57188851, SUC_lexicon_tags_1 },
    { 0x84988852, SUC_lexicon_tags_27 },
    { 0xad388853, SUC_lexicon_tags_11 },
    { 0xcab88851, SUC_lexicon_tags_25 },
    { 0xb0388855, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe0f88857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb8885c, SUC_lexicon_tags_3 },
    { 0x9858885c, SUC_lexicon_tags_87 },
    { 0xa18885e, SUC_lexicon_tags_1 },
    { 0x9178885f, SUC_lexicon_tags_14 },
    { 0x25f88860, SUC_lexicon_tags_16 },
    { 0x51588861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af88866, SUC_lexicon_tags_7 },
    { 0x7f988867, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8538886a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7c78886c, SUC_lexicon_tags_23 },
    { 0xe18886d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c188870, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb88879, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56b8887b, SUC_lexicon_tags_25 },
    { 0xbf78887c, SUC_lexicon_tags_37 },
    { 0x7bd8887d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b588881, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1198888a, SUC_lexicon_tags_1 },
    { 0x4df8888b, SUC_lexicon_tags_35 },
    { 0xd218888c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15f8888e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f88891, SUC_lexicon_tags_6 },
    { 0x8d88891, SUC_lexicon_tags_9 },
    { 0x63388893, SUC_lexicon_tags_0 },
    { 0xa4588893, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1388896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc138889a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2478889d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f888a5, SUC_lexicon_tags_11 },
    { 0xc43888a5, SUC_lexicon_tags_17 },
    { 0x9d1888a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511888af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f888b8, SUC_lexicon_tags_47 },
    { 0x265888b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f888be, SUC_lexicon_tags_11 },
    { 0xa09888bf, SUC_lexicon_tags_32 },
    { 0xeb1888c0, SUC_lexicon_tags_38 },
    { 0x2d9888bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x645888c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x633888c6, SUC_lexicon_tags_38 },
    { 0xc7888c7, SUC_lexicon_tags_2 },
    { 0x185888c8, SUC_lexicon_tags_14 },
    { 0x705888c7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf3b888cb, SUC_lexicon_tags_15 },
    { 0x20d888cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5888d4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x773888d6, SUC_lexicon_tags_30 },
    { 0xf23888d7, SUC_lexicon_tags_0 },
    { 0x2fb888d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x903888da, SUC_lexicon_tags_11 },
    { 0xdef888db, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x627888dd, SUC_lexicon_tags_17 },
    { 0xbf3888de, SUC_lexicon_tags_12 },
    { 0xd85888df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x409888e1, SUC_lexicon_tags_2 },
    { 0x811888e2, SUC_lexicon_tags_7 },
    { 0x62b888e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb888e6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f888e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc7888eb, SUC_lexicon_tags_17 },
    { 0x3f888eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381888f3, SUC_lexicon_tags_17 },
    { 0xe25888f4, SUC_lexicon_tags_63 },
    { 0x6f1888f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x641888f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537888fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f88900, SUC_lexicon_tags_12 },
    { 0xb9988901, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa388905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5b88907, SUC_lexicon_tags_45 },
    { 0xed788908, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf378890b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e78890d, SUC_lexicon_tags_48 },
    { 0x18d8890d, SUC_lexicon_tags_8 },
    { 0x8a38890f, SUC_lexicon_tags_11 },
    { 0x7f588910, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4388919, SUC_lexicon_tags_150 },
    { 0xc558891a, SUC_lexicon_tags_11 },
    { 0xc698891b, SUC_lexicon_tags_12 },
    { 0x5078891a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6f8891e, SUC_lexicon_tags_37 },
    { 0xad58891e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6b788921, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3188924, SUC_lexicon_tags_0 },
    { 0x66188925, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x85988928, SUC_lexicon_tags_92 },
    { 0x32388929, SUC_lexicon_tags_11 },
    { 0x87588929, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5118892e, SUC_lexicon_tags_49 },
    { 0xe638892f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2b88931, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f88937, SUC_lexicon_tags_60 },
    { 0xfd988938, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5818893c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe538893f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x21588941, SUC_lexicon_tags_7 },
    { 0x50b88942, SUC_lexicon_tags_11 },
    { 0x71788943, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6bf88945, SUC_lexicon_tags_7 },
    { 0xc2188946, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4f88948, SUC_lexicon_tags_31 },
    { 0xe2788948, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x30d8894b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb98894d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe498894f, SUC_lexicon_tags_255 },
    { 0x8c38894f, SUC_lexicon_tags_21 },
    { 0x6e788951, SUC_lexicon_tags_15 },
    { 0xf8d88951, SUC_lexicon_tags_17 },
    { 0xa3f88953, SUC_lexicon_tags_11 },
    { 0xd988953, SUC_lexicon_tags_14 },
    { 0x2b188953, SUC_lexicon_tags_16 },
    { 0x47f88956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d88959, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa738895d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd158895f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdab88961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a988965, SUC_lexicon_tags_38 },
    { 0xd9d88965, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d8896a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3678896c, SUC_lexicon_tags_23 },
    { 0x1778896d, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a588972, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfcf88974, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf788976, SUC_lexicon_tags_37 },
    { 0x47988976, SUC_lexicon_tags_63 },
    { 0xc3588978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d18897a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcf8897c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71788980, SUC_lexicon_tags_37 },
    { 0x11788980, SUC_lexicon_tags_33 },
    { 0xdab88981, SUC_lexicon_tags_19 },
    { 0x44188983, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d88986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb158898b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d8898e, SUC_lexicon_tags_1 },
    { 0xac38898e, SUC_lexicon_tags_63 },
    { 0x51388990, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xffd88992, SUC_lexicon_tags_45 },
    { 0x63b88993, SUC_lexicon_tags_12 },
    { 0x91d88994, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbdb88996, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc158899b, SUC_lexicon_tags_0 },
    { 0xdbb8899c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfdb8899e, SUC_lexicon_tags_0 },
    { 0x6ad8899f, SUC_lexicon_tags_2 },
    { 0x9938899e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac1889a2, SUC_lexicon_tags_16 },
    { 0xeab889a3, SUC_lexicon_tags_29 },
    { 0x75b889a3, SUC_lexicon_tags_105 },
    { 0x4c7889a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23f889a7, SUC_lexicon_tags_17 },
    { 0xbeb889a8, SUC_lexicon_tags_15 },
    { 0x91889a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x821889ac, SUC_lexicon_tags_31 },
    { 0xc3889ad, SUC_lexicon_tags_25 },
    { 0xe61889ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f889b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21889b6, SUC_lexicon_tags_2 },
    { 0xab7889b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609889bc, SUC_lexicon_tags_37 },
    { 0x179889bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5889c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9f889c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7889c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa3b889cb, SUC_lexicon_tags_12 },
    { 0xacd889cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3889d1, SUC_lexicon_tags_7 },
    { 0x781889d2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x383889d5, SUC_lexicon_tags_17 },
    { 0xc3889d5, SUC_lexicon_tags_34 },
    { 0x26b889d7, SUC_lexicon_tags_37 },
    { 0xd47889d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xee3889da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7889e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x925889e8, SUC_lexicon_tags_15 },
    { 0x81d889e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f889ec, SUC_lexicon_tags_47 },
    { 0xb07889ed, SUC_lexicon_tags_63 },
    { 0x3d1889ee, SUC_lexicon_tags_28 },
    { 0xcdf889ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad889f8, SUC_lexicon_tags_14 },
    { 0x3b3889f9, SUC_lexicon_tags_6 },
    { 0x6b7889fa, SUC_lexicon_tags_8 },
    { 0xc97889fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49889fe, SUC_lexicon_tags_17 },
    { 0x3889fe, SUC_lexicon_tags_19 },
    { 0x64588a00, SUC_lexicon_tags_37 },
    { 0xd1d88a01, SUC_lexicon_tags_17 },
    { 0xa6788a02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86788a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d88a0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51188a10, SUC_lexicon_tags_23 },
    { 0x70f88a11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f388a13, SUC_lexicon_tags_11 },
    { 0xe0988a13, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb88a19, SUC_lexicon_tags_31 },
    { 0xea188a1a, SUC_lexicon_tags_12 },
    { 0x1c388a1a, SUC_lexicon_tags_47 },
    { 0xc8988a1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d88a25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f88a29, SUC_lexicon_tags_19 },
    { 0xc1588a2a, SUC_lexicon_tags_7 },
    { 0x28d88a2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9188a2d, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea188a31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c388a39, SUC_lexicon_tags_37 },
    { 0xc1588a3a, SUC_lexicon_tags_49 },
    { 0x48388a3b, SUC_lexicon_tags_12 },
    { 0xc4b88a3c, SUC_lexicon_tags_35 },
    { 0xa3788a3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95b88a3f, SUC_lexicon_tags_11 },
    { 0x39788a3f, SUC_lexicon_tags_37 },
    { 0xcf788a3f, SUC_lexicon_tags_55 },
    { 0x45788a40, SUC_lexicon_tags_37 },
    { 0xa1588a42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79f88a45, SUC_lexicon_tags_68 },
    { 0x91f88a46, SUC_lexicon_tags_314 },
    { 0x3ed88a46, SUC_lexicon_tags_38 },
    { 0xe1188a46, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff88a4d, SUC_lexicon_tags_0 },
    { 0x54f88a4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf88a53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52b88a55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea988a59, SUC_lexicon_tags_15 },
    { 0x46388a5a, SUC_lexicon_tags_37 },
    { 0xcab88a5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e588a5d, SUC_lexicon_tags_437 },
    { 0, NULL },
    { 0xd388a5f, SUC_lexicon_tags_6 },
    { 0xe4b88a60, SUC_lexicon_tags_5 },
    { 0x20d88a61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79988a66, SUC_lexicon_tags_1424 },
    { 0x4ab88a67, SUC_lexicon_tags_7 },
    { 0x70588a68, SUC_lexicon_tags_19 },
    { 0x8ad88a69, SUC_lexicon_tags_3 },
    { 0x32b88a6a, SUC_lexicon_tags_12 },
    { 0x1388a6b, SUC_lexicon_tags_301 },
    { 0xb5d88a6a, SUC_lexicon_tags_0 },
    { 0xfdd88a6d, SUC_lexicon_tags_0 },
    { 0x44188a6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63f88a70, SUC_lexicon_tags_0 },
    { 0x47d88a70, SUC_lexicon_tags_64 },
    { 0x93588a72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc788a75, SUC_lexicon_tags_19 },
    { 0x2a988a76, SUC_lexicon_tags_146 },
    { 0x1a188a75, SUC_lexicon_tags_27 },
    { 0xda788a78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22f88a7a, SUC_lexicon_tags_26 },
    { 0x94988a7b, SUC_lexicon_tags_2 },
    { 0x37788a7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c388a7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2188a87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x35788a89, SUC_lexicon_tags_25 },
    { 0x64388a89, SUC_lexicon_tags_53 },
    { 0x6a788a89, SUC_lexicon_tags_46 },
    { 0x80388a8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76788a92, SUC_lexicon_tags_47 },
    { 0x29188a93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65988a95, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b88a99, SUC_lexicon_tags_37 },
    { 0x28988a99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf88a9f, SUC_lexicon_tags_37 },
    { 0x5fd88a9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94788aa5, SUC_lexicon_tags_12 },
    { 0xb6b88aa6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6bb88aa8, SUC_lexicon_tags_27 },
    { 0x91988aa8, SUC_lexicon_tags_37 },
    { 0xa3f88aa8, SUC_lexicon_tags_66 },
    { 0xf8388aab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5388aae, SUC_lexicon_tags_60 },
    { 0xa7788aae, SUC_lexicon_tags_47 },
    { 0xc8588aaf, SUC_lexicon_tags_56 },
    { 0x92788ab1, SUC_lexicon_tags_7 },
    { 0x35f88ab2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x92988ab5, SUC_lexicon_tags_1 },
    { 0xf6388ab6, SUC_lexicon_tags_26 },
    { 0xd2f88ab7, SUC_lexicon_tags_27 },
    { 0x32588ab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d988aba, SUC_lexicon_tags_28 },
    { 0x74b88abb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c388abe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b88ac2, SUC_lexicon_tags_2 },
    { 0x49188ac3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d388ac8, SUC_lexicon_tags_1 },
    { 0xc3f88ac9, SUC_lexicon_tags_21 },
    { 0xbf388aca, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xc4f88acc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90388acf, SUC_lexicon_tags_11 },
    { 0x57788acf, SUC_lexicon_tags_43 },
    { 0xbeb88acf, SUC_lexicon_tags_15 },
    { 0xffd88ad2, SUC_lexicon_tags_0 },
    { 0x34388ad2, SUC_lexicon_tags_17 },
    { 0x43588ad4, SUC_lexicon_tags_65 },
    { 0xdcb88ad4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7a788ad7, SUC_lexicon_tags_48 },
    { 0x97988ad7, SUC_lexicon_tags_23 },
    { 0x9bb88ad9, SUC_lexicon_tags_7 },
    { 0xc5b88ad9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a388adf, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d88ae6, SUC_lexicon_tags_32 },
    { 0x86f88ae7, SUC_lexicon_tags_11 },
    { 0x85588ae7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14f88aea, SUC_lexicon_tags_31 },
    { 0x44788aeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9388aee, SUC_lexicon_tags_2 },
    { 0x23d88aee, SUC_lexicon_tags_48 },
    { 0x4b388af0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d88af7, SUC_lexicon_tags_43 },
    { 0xed788af7, SUC_lexicon_tags_17 },
    { 0xd6388af9, SUC_lexicon_tags_16 },
    { 0x9bd88afa, SUC_lexicon_tags_25 },
    { 0xe4988af8, SUC_lexicon_tags_11 },
    { 0x93188afc, SUC_lexicon_tags_30 },
    { 0xdc988af9, SUC_lexicon_tags_61 },
    { 0xd2988afe, SUC_lexicon_tags_47 },
    { 0xaf388aff, SUC_lexicon_tags_11 },
    { 0x78588afb, SUC_lexicon_tags_3 },
    { 0xa4588af9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10b88b03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f88b0b, SUC_lexicon_tags_87 },
    { 0x12f88b0b, SUC_lexicon_tags_17 },
    { 0x37588b0b, SUC_lexicon_tags_23 },
    { 0x55188b0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff188b12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x59988b14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af88b17, SUC_lexicon_tags_26 },
    { 0xc3588b17, SUC_lexicon_tags_60 },
    { 0x96d88b19, SUC_lexicon_tags_38 },
    { 0x4fd88b1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10388b1c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d188b21, SUC_lexicon_tags_7 },
    { 0x30788b21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2188b27, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca988b2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c188b30, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb88b38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d88b3b, SUC_lexicon_tags_88 },
    { 0x21f88b3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a388b42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf88b45, SUC_lexicon_tags_1 },
    { 0x5188b46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x43b88b48, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x2388b4b, SUC_lexicon_tags_9 },
    { 0x42388b4c, SUC_lexicon_tags_77 },
    { 0x63588b4d, SUC_lexicon_tags_66 },
    { 0xc8988b4c, SUC_lexicon_tags_70 },
    { 0xc9d88b4f, SUC_lexicon_tags_13 },
    { 0xea788b4d, SUC_lexicon_tags_16 },
    { 0xb0588b51, SUC_lexicon_tags_74 },
    { 0x93988b4e, SUC_lexicon_tags_6 },
    { 0x56988b50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4588b5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3188b60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e988b63, SUC_lexicon_tags_4 },
    { 0xa2788b64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6b88b66, SUC_lexicon_tags_15 },
    { 0x2af88b66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3c788b69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3988b6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9188b71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd88b7c, SUC_lexicon_tags_1 },
    { 0x9f388b7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42788b85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d188b88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8588b8b, SUC_lexicon_tags_12 },
    { 0x6f388b8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb3b88b8e, SUC_lexicon_tags_6 },
    { 0x21d88b8f, SUC_lexicon_tags_27 },
    { 0x92d88b8f, SUC_lexicon_tags_17 },
    { 0xa9388b91, SUC_lexicon_tags_1 },
    { 0x8ab88b92, SUC_lexicon_tags_18 },
    { 0x89388b93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f188b97, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd5f88b99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7588b9c, SUC_lexicon_tags_68 },
    { 0x97588b9d, SUC_lexicon_tags_11 },
    { 0x8c188b9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b88ba2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9188ba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55f88ba7, SUC_lexicon_tags_8 },
    { 0xfe588ba8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b88bab, SUC_lexicon_tags_129 },
    { 0x71988bac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4588baf, SUC_lexicon_tags_25 },
    { 0xd6588bb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde788bb3, SUC_lexicon_tags_11 },
    { 0x5db88bb3, SUC_lexicon_tags_16 },
    { 0x26188bb4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15388bbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x16988bbe, SUC_lexicon_tags_25 },
    { 0xec588bbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b88bc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f88bc8, SUC_lexicon_tags_0 },
    { 0xb3788bc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a988bce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x80f88bd0, SUC_lexicon_tags_29 },
    { 0xc8588bd1, SUC_lexicon_tags_37 },
    { 0xb788bd2, SUC_lexicon_tags_14 },
    { 0x59588bd3, SUC_lexicon_tags_7 },
    { 0xa9788bd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad88bd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28788bda, SUC_lexicon_tags_9 },
    { 0xb7588bda, SUC_lexicon_tags_25 },
    { 0x31b88bdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31388be0, SUC_lexicon_tags_15 },
    { 0x44388be1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b88be6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f88be9, SUC_lexicon_tags_12 },
    { 0x2e588be9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9588bfa, SUC_lexicon_tags_91 },
    { 0xa6988bfa, SUC_lexicon_tags_47 },
    { 0x90f88bfc, SUC_lexicon_tags_33 },
    { 0x14f88bfa, SUC_lexicon_tags_1 },
    { 0x89f88bfc, SUC_lexicon_tags_47 },
    { 0xbef88bff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83588c01, SUC_lexicon_tags_6 },
    { 0xf0588c02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7188c05, SUC_lexicon_tags_15 },
    { 0x6588c06, SUC_lexicon_tags_19 },
    { 0xc2188c07, SUC_lexicon_tags_48 },
    { 0xc0188c07, SUC_lexicon_tags_14 },
    { 0x60b88c08, SUC_lexicon_tags_40 },
    { 0x65f88c0a, SUC_lexicon_tags_1 },
    { 0x8c588c0b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b88c0f, SUC_lexicon_tags_17 },
    { 0xdb988c10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9788c12, SUC_lexicon_tags_2 },
    { 0x4b188c13, SUC_lexicon_tags_11 },
    { 0x30188c14, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb88c1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53388c21, SUC_lexicon_tags_31 },
    { 0x75988c22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb88c25, SUC_lexicon_tags_35 },
    { 0x24388c26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df88c29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b88c30, SUC_lexicon_tags_37 },
    { 0x63988c30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22988c33, SUC_lexicon_tags_7 },
    { 0x97388c34, SUC_lexicon_tags_47 },
    { 0x37388c35, SUC_lexicon_tags_37 },
    { 0xdcb88c36, SUC_lexicon_tags_12 },
    { 0xf6388c37, SUC_lexicon_tags_21 },
    { 0xc2f88c37, SUC_lexicon_tags_15 },
    { 0x4f388c39, SUC_lexicon_tags_13 },
    { 0xacd88c39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28788c3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc788c41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdaf88c43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3588c48, SUC_lexicon_tags_17 },
    { 0xa9b88c48, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6788c4c, SUC_lexicon_tags_11 },
    { 0xba788c4d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x59b88c4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xec788c52, SUC_lexicon_tags_27 },
    { 0xb9388c53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9588c58, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8788c5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd588c62, SUC_lexicon_tags_67 },
    { 0xad988c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90588c65, SUC_lexicon_tags_11 },
    { 0x5bf88c65, SUC_lexicon_tags_27 },
    { 0x55188c67, SUC_lexicon_tags_23 },
    { 0x64388c68, SUC_lexicon_tags_23 },
    { 0x2b88c68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9e388c6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e188c6f, SUC_lexicon_tags_7 },
    { 0x9e988c70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6988c74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1788c79, SUC_lexicon_tags_2 },
    { 0x98788c79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d88c82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b88c86, SUC_lexicon_tags_37 },
    { 0x21b88c86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f588c89, SUC_lexicon_tags_43 },
    { 0xb7988c8a, SUC_lexicon_tags_25 },
    { 0x72d88c89, SUC_lexicon_tags_37 },
    { 0xb3d88c8c, SUC_lexicon_tags_13 },
    { 0xcad88c8d, SUC_lexicon_tags_11 },
    { 0x4e388c8e, SUC_lexicon_tags_9 },
    { 0x1e188c8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x93188c92, SUC_lexicon_tags_35 },
    { 0xdf188c93, SUC_lexicon_tags_12 },
    { 0xa4788c93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa5d88c96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58b88c98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xacb88c9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f88c9e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76988ca0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d88ca3, SUC_lexicon_tags_17 },
    { 0x8e188ca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0588ca7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53788ca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51b88cab, SUC_lexicon_tags_12 },
    { 0xbcd88cac, SUC_lexicon_tags_40 },
    { 0xc5588cad, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18388cb2, SUC_lexicon_tags_11 },
    { 0x52588cb2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd4d88cb5, SUC_lexicon_tags_33 },
    { 0x35788cb5, SUC_lexicon_tags_9 },
    { 0xf7588cb7, SUC_lexicon_tags_28 },
    { 0xfd788cb8, SUC_lexicon_tags_16 },
    { 0x73b88cb9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe2788cbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf4988cbd, SUC_lexicon_tags_37 },
    { 0x66588cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15388cc5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1988cc7, SUC_lexicon_tags_10 },
    { 0xa6d88cc8, SUC_lexicon_tags_15 },
    { 0xa7588cc8, SUC_lexicon_tags_11 },
    { 0x2db88cca, SUC_lexicon_tags_37 },
    { 0x8c188cc8, SUC_lexicon_tags_37 },
    { 0x9df88ccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61588cce, SUC_lexicon_tags_14 },
    { 0x34d88cce, SUC_lexicon_tags_64 },
    { 0x1a788ccf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb88cd5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4388cd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14588cdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc988cdf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb7388ce1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3a588ce3, SUC_lexicon_tags_23 },
    { 0xa5f88ce3, SUC_lexicon_tags_14 },
    { 0x78988ce5, SUC_lexicon_tags_11 },
    { 0x6cf88ce3, SUC_lexicon_tags_79 },
    { 0x50d88ce7, SUC_lexicon_tags_11 },
    { 0x2cf88ce8, SUC_lexicon_tags_33 },
    { 0x22788ce6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45388ceb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x40388cee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b88cf5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32188cf9, SUC_lexicon_tags_1 },
    { 0x6b988cfa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd88d02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8388d04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77188d06, SUC_lexicon_tags_45 },
    { 0xf2588d06, SUC_lexicon_tags_37 },
    { 0x59b88d08, SUC_lexicon_tags_37 },
    { 0xf3f88d07, SUC_lexicon_tags_1 },
    { 0x1db88d0a, SUC_lexicon_tags_45 },
    { 0xd0188d08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25388d16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0788d18, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2988d1b, SUC_lexicon_tags_37 },
    { 0x2d188d1c, SUC_lexicon_tags_5 },
    { 0xacf88d1d, SUC_lexicon_tags_11 },
    { 0xa6f88d1c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f88d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30788d29, SUC_lexicon_tags_38 },
    { 0x7c588d29, SUC_lexicon_tags_12 },
    { 0x28f88d2b, SUC_lexicon_tags_21 },
    { 0xc2188d2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4388d36, SUC_lexicon_tags_28 },
    { 0xb7388d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4588d39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd788d3d, SUC_lexicon_tags_47 },
    { 0xaa188d3d, SUC_lexicon_tags_7 },
    { 0xd388d3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb88d44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7788d49, SUC_lexicon_tags_12 },
    { 0x64988d4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf88d52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d88d55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1588d57, SUC_lexicon_tags_25 },
    { 0xa9788d57, SUC_lexicon_tags_1 },
    { 0xad188d57, SUC_lexicon_tags_47 },
    { 0x4bb88d5a, SUC_lexicon_tags_33 },
    { 0x59b88d5a, SUC_lexicon_tags_37 },
    { 0xf788d5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b88d60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf88d64, SUC_lexicon_tags_11 },
    { 0xac988d64, SUC_lexicon_tags_27 },
    { 0x9f588d66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26b88d68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f88d6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53988d6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x26788d70, SUC_lexicon_tags_15 },
    { 0x8a188d70, SUC_lexicon_tags_26 },
    { 0x79588d72, SUC_lexicon_tags_25 },
    { 0x34588d72, SUC_lexicon_tags_80 },
    { 0x76988d73, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5988d78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b188d7c, SUC_lexicon_tags_12 },
    { 0xf7388d7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9988d81, SUC_lexicon_tags_44 },
    { 0x7c188d81, SUC_lexicon_tags_14 },
    { 0x21f88d82, SUC_lexicon_tags_1 },
    { 0x74788d84, SUC_lexicon_tags_46 },
    { 0x99d88d85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbcd88d87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x50988d8a, SUC_lexicon_tags_45 },
    { 0xd4788d8b, SUC_lexicon_tags_95 },
    { 0x5c788d8c, SUC_lexicon_tags_25 },
    { 0xedb88d8c, SUC_lexicon_tags_37 },
    { 0xc6788d8d, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd88d97, SUC_lexicon_tags_66 },
    { 0x92f88d98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda588d9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f88d9f, SUC_lexicon_tags_3 },
    { 0xfe388da0, SUC_lexicon_tags_29 },
    { 0x80b88da1, SUC_lexicon_tags_47 },
    { 0x66388da2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93388da4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cb88da6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55588daa, SUC_lexicon_tags_23 },
    { 0xa1988dab, SUC_lexicon_tags_7 },
    { 0xc8d88dac, SUC_lexicon_tags_19 },
    { 0xb0f88dab, SUC_lexicon_tags_35 },
    { 0x3d988dac, SUC_lexicon_tags_29 },
    { 0x20788daf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6788db2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e988dba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a788dbf, SUC_lexicon_tags_11 },
    { 0xd8988dc0, SUC_lexicon_tags_15 },
    { 0xccd88dc1, SUC_lexicon_tags_34 },
    { 0xfc188dc2, SUC_lexicon_tags_151 },
    { 0x91788dc3, SUC_lexicon_tags_47 },
    { 0x13588dc4, SUC_lexicon_tags_1 },
    { 0x61788dc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed88dc8, SUC_lexicon_tags_27 },
    { 0x7b188dc9, SUC_lexicon_tags_7 },
    { 0xfe788dca, SUC_lexicon_tags_25 },
    { 0x54788dc9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f88dce, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x26788dd1, SUC_lexicon_tags_23 },
    { 0xb988dd1, SUC_lexicon_tags_25 },
    { 0xc1988dd3, SUC_lexicon_tags_12 },
    { 0x7b588dd3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db88dd7, SUC_lexicon_tags_5 },
    { 0xaff88dd8, SUC_lexicon_tags_25 },
    { 0xadd88dd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e788ddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e788de2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8388de7, SUC_lexicon_tags_37 },
    { 0x2d588de7, SUC_lexicon_tags_11 },
    { 0x71d88de7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x788dee, SUC_lexicon_tags_49 },
    { 0xd7588def, SUC_lexicon_tags_47 },
    { 0x75f88dee, SUC_lexicon_tags_12 },
    { 0x4f388df1, SUC_lexicon_tags_23 },
    { 0x3e188df2, SUC_lexicon_tags_1 },
    { 0x62d88df3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e788df5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b88df9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9788dfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25988dfd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x55588dff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24988e01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e988e04, SUC_lexicon_tags_26 },
    { 0x62188e05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7588e0b, SUC_lexicon_tags_43 },
    { 0x1bd88e0c, SUC_lexicon_tags_12 },
    { 0x94588e0d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x1cb88e0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xccd88e12, SUC_lexicon_tags_38 },
    { 0x73d88e12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2788e19, SUC_lexicon_tags_38 },
    { 0x1eb88e19, SUC_lexicon_tags_31 },
    { 0x17d88e19, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf88e1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1b88e20, SUC_lexicon_tags_19 },
    { 0x8a588e20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82988e25, SUC_lexicon_tags_12 },
    { 0x2e388e26, SUC_lexicon_tags_37 },
    { 0x26b88e27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5188e29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b88e2d, SUC_lexicon_tags_17 },
    { 0xf5188e2d, SUC_lexicon_tags_11 },
    { 0x86388e2f, SUC_lexicon_tags_25 },
    { 0xc2b88e2f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x60f88e32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf88e37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfc988e39, SUC_lexicon_tags_2 },
    { 0x5e788e3a, SUC_lexicon_tags_25 },
    { 0x2b88e3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cd88e3d, SUC_lexicon_tags_17 },
    { 0x75f88e3d, SUC_lexicon_tags_127 },
    { 0xed788e3f, SUC_lexicon_tags_0 },
    { 0x7c788e40, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8f588e42, SUC_lexicon_tags_7 },
    { 0xa988e42, SUC_lexicon_tags_11 },
    { 0xdb88e44, SUC_lexicon_tags_37 },
    { 0xb8588e44, SUC_lexicon_tags_34 },
    { 0x1b988e46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6588e48, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xd2f88e4a, SUC_lexicon_tags_23 },
    { 0xf188e4a, SUC_lexicon_tags_15 },
    { 0xf0788e4c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b988e56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1cb88e58, SUC_lexicon_tags_4 },
    { 0xa2988e59, SUC_lexicon_tags_37 },
    { 0x50988e5a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x39188e5d, SUC_lexicon_tags_77 },
    { 0x1e988e5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7988e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1f88e62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed388e65, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xce988e68, SUC_lexicon_tags_49 },
    { 0x52388e69, SUC_lexicon_tags_16 },
    { 0xcfb88e6a, SUC_lexicon_tags_17 },
    { 0xbc988e6b, SUC_lexicon_tags_7 },
    { 0x8d188e6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd88e6f, SUC_lexicon_tags_12 },
    { 0xbc588e70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b88e76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f588e78, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x77188e7b, SUC_lexicon_tags_27 },
    { 0xd8988e7c, SUC_lexicon_tags_63 },
    { 0xea788e7b, SUC_lexicon_tags_43 },
    { 0x72d88e7e, SUC_lexicon_tags_29 },
    { 0x7d388e7b, SUC_lexicon_tags_0 },
    { 0x5bd88e80, SUC_lexicon_tags_1 },
    { 0x6e588e81, SUC_lexicon_tags_27 },
    { 0x79988e81, SUC_lexicon_tags_37 },
    { 0x8c988e7d, SUC_lexicon_tags_37 },
    { 0x51188e83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x40388e87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f388e89, SUC_lexicon_tags_29 },
    { 0xf7b88e8a, SUC_lexicon_tags_47 },
    { 0x7f388e8b, SUC_lexicon_tags_7 },
    { 0x44588e8c, SUC_lexicon_tags_17 },
    { 0x22d88e8d, SUC_lexicon_tags_15 },
    { 0x88f88e8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d988e93, SUC_lexicon_tags_68 },
    { 0xe4b88e94, SUC_lexicon_tags_38 },
    { 0xa2988e95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d788e9a, SUC_lexicon_tags_16 },
    { 0xca988e9b, SUC_lexicon_tags_37 },
    { 0x1fb88e9c, SUC_lexicon_tags_87 },
    { 0x64588e9d, SUC_lexicon_tags_37 },
    { 0x4bd88e9d, SUC_lexicon_tags_37 },
    { 0x9dd88e9f, SUC_lexicon_tags_27 },
    { 0xf5b88ea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74788eab, SUC_lexicon_tags_53 },
    { 0x77788eac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7188eb0, SUC_lexicon_tags_2 },
    { 0xa588eb0, SUC_lexicon_tags_12 },
    { 0xf5388eb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x74b88eb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8988ebe, SUC_lexicon_tags_25 },
    { 0xb6b88ebf, SUC_lexicon_tags_33 },
    { 0x99988ebf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2588ec4, SUC_lexicon_tags_25 },
    { 0xbb988ec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a388ec7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f88ecb, SUC_lexicon_tags_77 },
    { 0x3af88ecc, SUC_lexicon_tags_18 },
    { 0x66788ecd, SUC_lexicon_tags_1 },
    { 0x86188ecd, SUC_lexicon_tags_37 },
    { 0x61b88ece, SUC_lexicon_tags_63 },
    { 0xa6d88ecd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee788ed2, SUC_lexicon_tags_38 },
    { 0xfdd88ed3, SUC_lexicon_tags_37 },
    { 0xdd588ed3, SUC_lexicon_tags_11 },
    { 0x7f388ed2, SUC_lexicon_tags_27 },
    { 0x65988ed6, SUC_lexicon_tags_0 },
    { 0x59388ed7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e388eda, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d88ede, SUC_lexicon_tags_34 },
    { 0x3b188edf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20f88ee1, SUC_lexicon_tags_11 },
    { 0xded88ee2, SUC_lexicon_tags_10 },
    { 0x6a788ee3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d88ee6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a588ef0, SUC_lexicon_tags_37 },
    { 0xd6f88ef1, SUC_lexicon_tags_7 },
    { 0x28588ef1, SUC_lexicon_tags_37 },
    { 0x46788ef1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22988efd, SUC_lexicon_tags_17 },
    { 0xbe188efd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b88f03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdfd88f05, SUC_lexicon_tags_31 },
    { 0xe9588f05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8788f09, SUC_lexicon_tags_1 },
    { 0x22d88f09, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b88f0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd88f14, SUC_lexicon_tags_2 },
    { 0xa0188f15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d388f20, SUC_lexicon_tags_8 },
    { 0xbfd88f20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac188f25, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e588f2b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4c388f2d, SUC_lexicon_tags_7 },
    { 0x24b88f2e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x96d88f30, SUC_lexicon_tags_33 },
    { 0xd9f88f31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec588f33, SUC_lexicon_tags_17 },
    { 0x3b588f34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9b88f36, SUC_lexicon_tags_87 },
    { 0x1c788f37, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c388f3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b88f42, SUC_lexicon_tags_64 },
    { 0x94b88f43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb88f45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd88f49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd88f56, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x72d88f59, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b88f5d, SUC_lexicon_tags_31 },
    { 0x8ab88f5e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe6d88f60, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5588f63, SUC_lexicon_tags_1 },
    { 0x11388f64, SUC_lexicon_tags_68 },
    { 0x68f88f64, SUC_lexicon_tags_47 },
    { 0x30388f66, SUC_lexicon_tags_2 },
    { 0x23388f63, SUC_lexicon_tags_7 },
    { 0x3c388f65, SUC_lexicon_tags_63 },
    { 0xc5388f69, SUC_lexicon_tags_2 },
    { 0xc7588f69, SUC_lexicon_tags_30 },
    { 0xd4388f6b, SUC_lexicon_tags_14 },
    { 0x24d88f6b, SUC_lexicon_tags_19 },
    { 0x5d788f6d, SUC_lexicon_tags_17 },
    { 0x37b88f6a, SUC_lexicon_tags_30 },
    { 0x58588f69, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a388f77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8188f7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbaf88f7d, SUC_lexicon_tags_45 },
    { 0x9f188f7d, SUC_lexicon_tags_25 },
    { 0xe3b88f7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b388f81, SUC_lexicon_tags_25 },
    { 0xc6988f81, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd88f88, SUC_lexicon_tags_26 },
    { 0xe588f88, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa188f8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75388f8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82388f90, SUC_lexicon_tags_11 },
    { 0xfdb88f91, SUC_lexicon_tags_0 },
    { 0xb0f88f92, SUC_lexicon_tags_60 },
    { 0x4bb88f93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeeb88f95, SUC_lexicon_tags_12 },
    { 0xbf88f96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64588f99, SUC_lexicon_tags_11 },
    { 0xd7788f99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd88f9e, SUC_lexicon_tags_33 },
    { 0xcd188f9e, SUC_lexicon_tags_17 },
    { 0x54b88fa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b88fa5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8a588fa7, SUC_lexicon_tags_15 },
    { 0xaa788fa7, SUC_lexicon_tags_51 },
    { 0xb0f88fa9, SUC_lexicon_tags_7 },
    { 0x7e788faa, SUC_lexicon_tags_7 },
    { 0x6988faa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18788fb6, SUC_lexicon_tags_0 },
    { 0xc2988fb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5588fbc, SUC_lexicon_tags_46 },
    { 0xfdb88fbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3588fbf, SUC_lexicon_tags_39 },
    { 0x3cd88fbf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b88fca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae188fcc, SUC_lexicon_tags_0 },
    { 0x49788fcd, SUC_lexicon_tags_2 },
    { 0x6c988fce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69588fd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7588fda, SUC_lexicon_tags_15 },
    { 0x33988fdb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d88fdf, SUC_lexicon_tags_37 },
    { 0x1e788fe0, SUC_lexicon_tags_1 },
    { 0xb9b88fe1, SUC_lexicon_tags_55 },
    { 0xd1d88fe2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb5588fe4, SUC_lexicon_tags_17 },
    { 0x8e988fe5, SUC_lexicon_tags_25 },
    { 0x43988fe4, SUC_lexicon_tags_4 },
    { 0xea788fe5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfef88fe9, SUC_lexicon_tags_17 },
    { 0x13b88fe9, SUC_lexicon_tags_8 },
    { 0x36188fe9, SUC_lexicon_tags_38 },
    { 0xcef88fec, SUC_lexicon_tags_22 },
    { 0x4e388fed, SUC_lexicon_tags_2 },
    { 0x6b988fed, SUC_lexicon_tags_19 },
    { 0x2f988fee, SUC_lexicon_tags_179 },
    { 0xa2588fee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x94788ff2, SUC_lexicon_tags_11 },
    { 0x16988ff3, SUC_lexicon_tags_49 },
    { 0xe3b88ff2, SUC_lexicon_tags_25 },
    { 0x7ed88ff2, SUC_lexicon_tags_17 },
    { 0x64588ff6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae788ff8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f88ffb, SUC_lexicon_tags_15 },
    { 0x6a188ffc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x83588ffe, SUC_lexicon_tags_8 },
    { 0x7a388fff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77f89001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1389006, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd8900d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf89011, SUC_lexicon_tags_2 },
    { 0x4c989012, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18989014, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x99189016, SUC_lexicon_tags_1 },
    { 0xb0789017, SUC_lexicon_tags_37 },
    { 0xb8989018, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf8901c, SUC_lexicon_tags_14 },
    { 0xd0b8901d, SUC_lexicon_tags_17 },
    { 0x8a58901e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d89023, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d89027, SUC_lexicon_tags_1 },
    { 0x43989028, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d8902e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79389032, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab8903a, SUC_lexicon_tags_15 },
    { 0xb258903a, SUC_lexicon_tags_11 },
    { 0x8478903c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d78903f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56f89041, SUC_lexicon_tags_63 },
    { 0xa2b89042, SUC_lexicon_tags_43 },
    { 0x8a389043, SUC_lexicon_tags_47 },
    { 0x42189043, SUC_lexicon_tags_15 },
    { 0xcd189043, SUC_lexicon_tags_25 },
    { 0xa2989046, SUC_lexicon_tags_61 },
    { 0x4a189045, SUC_lexicon_tags_49 },
    { 0x57189048, SUC_lexicon_tags_60 },
    { 0xb4189047, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb18904b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3eb8904d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2c58904f, SUC_lexicon_tags_21 },
    { 0x9dd89050, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b89056, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x2fd89058, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf218905d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d89063, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6189066, SUC_lexicon_tags_26 },
    { 0xd7189067, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35989069, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff8906c, SUC_lexicon_tags_47 },
    { 0x6bf8906c, SUC_lexicon_tags_7 },
    { 0x38b8906e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b8907b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d8907e, SUC_lexicon_tags_11 },
    { 0xd258907f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1189081, SUC_lexicon_tags_37 },
    { 0xcd189081, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d89085, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x98789088, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x6538908b, SUC_lexicon_tags_15 },
    { 0xb758908b, SUC_lexicon_tags_39 },
    { 0xc098908d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xa078908f, SUC_lexicon_tags_63 },
    { 0xd7d8908f, SUC_lexicon_tags_0 },
    { 0x9998908f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34589096, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x38f89098, SUC_lexicon_tags_37 },
    { 0xb0589099, SUC_lexicon_tags_33 },
    { 0x6d89098, SUC_lexicon_tags_6 },
    { 0x3a38909a, SUC_lexicon_tags_11 },
    { 0xff89098, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e38909f, SUC_lexicon_tags_27 },
    { 0x595890a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83890a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed890ac, SUC_lexicon_tags_17 },
    { 0x813890ac, SUC_lexicon_tags_30 },
    { 0xdf1890ae, SUC_lexicon_tags_27 },
    { 0x6af890af, SUC_lexicon_tags_6 },
    { 0xf77890af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49890b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7890b7, SUC_lexicon_tags_25 },
    { 0x543890b8, SUC_lexicon_tags_37 },
    { 0xc79890b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x757890bb, SUC_lexicon_tags_11 },
    { 0x8c1890bc, SUC_lexicon_tags_29 },
    { 0xf0f890bc, SUC_lexicon_tags_7 },
    { 0x625890be, SUC_lexicon_tags_17 },
    { 0x5d890bc, SUC_lexicon_tags_29 },
    { 0xd5b890c0, SUC_lexicon_tags_25 },
    { 0x793890c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d890c6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x299890c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7890cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53890ce, SUC_lexicon_tags_14 },
    { 0x73f890ce, SUC_lexicon_tags_43 },
    { 0x997890d0, SUC_lexicon_tags_1143 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835890d4, SUC_lexicon_tags_113 },
    { 0x133890d5, SUC_lexicon_tags_11 },
    { 0xe7d890d6, SUC_lexicon_tags_27 },
    { 0xb9f890d6, SUC_lexicon_tags_2 },
    { 0xdfb890d8, SUC_lexicon_tags_0 },
    { 0x8df890d7, SUC_lexicon_tags_37 },
    { 0x7bf890d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9890e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b890e3, SUC_lexicon_tags_43 },
    { 0xa8d890e3, SUC_lexicon_tags_5 },
    { 0xd7890e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997890e9, SUC_lexicon_tags_39 },
    { 0x92f890ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x947890ed, SUC_lexicon_tags_3 },
    { 0xc41890ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa63890f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacd890f4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b890f7, SUC_lexicon_tags_28 },
    { 0x9f890f8, SUC_lexicon_tags_37 },
    { 0x14d890f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x65890fc, SUC_lexicon_tags_10 },
    { 0x975890fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b89101, SUC_lexicon_tags_36 },
    { 0xf6b89102, SUC_lexicon_tags_77 },
    { 0x68189102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff89107, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24989109, SUC_lexicon_tags_11 },
    { 0x42f8910a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb89110, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9db89112, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa4189114, SUC_lexicon_tags_12 },
    { 0x3a189114, SUC_lexicon_tags_15 },
    { 0xefb89116, SUC_lexicon_tags_17 },
    { 0x95b89116, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2af89119, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf358911d, SUC_lexicon_tags_11 },
    { 0x7458911e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65789120, SUC_lexicon_tags_37 },
    { 0x29d89121, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2598912a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfbd8912c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1fd8912e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xded89130, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d589132, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89135, SUC_lexicon_tags_63 },
    { 0xbd589135, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccd89138, SUC_lexicon_tags_19 },
    { 0xa7d89138, SUC_lexicon_tags_38 },
    { 0xdab8913a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7798913d, SUC_lexicon_tags_31 },
    { 0x95f8913e, SUC_lexicon_tags_94 },
    { 0xcbd8913e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff89143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb138914b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad8914e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e789152, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9fb89154, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67b89157, SUC_lexicon_tags_19 },
    { 0x16789158, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1378915a, SUC_lexicon_tags_87 },
    { 0x4db8915b, SUC_lexicon_tags_14 },
    { 0xca58915c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68789160, SUC_lexicon_tags_45 },
    { 0x91589161, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13389164, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xceb89166, SUC_lexicon_tags_30 },
    { 0xc4f89166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72789169, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4758916e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b89171, SUC_lexicon_tags_7 },
    { 0x55989172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26d89174, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22189176, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6b89178, SUC_lexicon_tags_25 },
    { 0x71d89179, SUC_lexicon_tags_2 },
    { 0x1818917a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd658917c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa478917f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f89185, SUC_lexicon_tags_17 },
    { 0x2a989186, SUC_lexicon_tags_37 },
    { 0x55f89187, SUC_lexicon_tags_2 },
    { 0xf9b89188, SUC_lexicon_tags_1 },
    { 0x52589189, SUC_lexicon_tags_55 },
    { 0xbaf8918a, SUC_lexicon_tags_2 },
    { 0xa9d89186, SUC_lexicon_tags_37 },
    { 0xee58918c, SUC_lexicon_tags_2 },
    { 0xfc38918b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35989191, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d89196, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1589198, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x17d8919a, SUC_lexicon_tags_25 },
    { 0xe998919b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b891a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed891aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b891af, SUC_lexicon_tags_274 },
    { 0xae7891b0, SUC_lexicon_tags_2 },
    { 0x2ed891b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xae5891b3, SUC_lexicon_tags_25 },
    { 0xad9891b3, SUC_lexicon_tags_63 },
    { 0x4f9891b5, SUC_lexicon_tags_17 },
    { 0x221891b5, SUC_lexicon_tags_31 },
    { 0xccb891b7, SUC_lexicon_tags_15 },
    { 0xa2f891b4, SUC_lexicon_tags_11 },
    { 0x645891b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8f7891bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf891c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29891c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad891ca, SUC_lexicon_tags_65 },
    { 0x3ef891ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225891d4, SUC_lexicon_tags_7 },
    { 0x7a9891d5, SUC_lexicon_tags_12 },
    { 0x6e1891d6, SUC_lexicon_tags_25 },
    { 0x55b891d4, SUC_lexicon_tags_27 },
    { 0x6c7891d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445891dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82b891df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf891e2, SUC_lexicon_tags_2 },
    { 0xa95891e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1891e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b891ec, SUC_lexicon_tags_27 },
    { 0xcb1891ed, SUC_lexicon_tags_12 },
    { 0xe9d891ee, SUC_lexicon_tags_8 },
    { 0x37891ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x657891f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9891f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7891f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db891fc, SUC_lexicon_tags_17 },
    { 0x527891fd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x77d891ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f89202, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd9589204, SUC_lexicon_tags_19 },
    { 0x62589205, SUC_lexicon_tags_25 },
    { 0x49d89206, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8cd89208, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf8920b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1978920f, SUC_lexicon_tags_2 },
    { 0xb1989210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e589215, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f789219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad8921c, SUC_lexicon_tags_38 },
    { 0x7c18921d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b89226, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa5d89228, SUC_lexicon_tags_37 },
    { 0x63f89229, SUC_lexicon_tags_137 },
    { 0xdc18922a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99589233, SUC_lexicon_tags_25 },
    { 0x20f89234, SUC_lexicon_tags_54 },
    { 0xdcf89235, SUC_lexicon_tags_7 },
    { 0xca589236, SUC_lexicon_tags_31 },
    { 0x30389236, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed8923b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3ed8923d, SUC_lexicon_tags_2 },
    { 0xa3f8923e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb89242, SUC_lexicon_tags_0 },
    { 0xc9389243, SUC_lexicon_tags_43 },
    { 0xb5989243, SUC_lexicon_tags_4 },
    { 0x76989245, SUC_lexicon_tags_103 },
    { 0x46789246, SUC_lexicon_tags_17 },
    { 0xb6789243, SUC_lexicon_tags_15 },
    { 0xf6589245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85389253, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81789256, SUC_lexicon_tags_7 },
    { 0x21189257, SUC_lexicon_tags_22 },
    { 0x7f989258, SUC_lexicon_tags_25 },
    { 0xa2b89259, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd158925c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc58925f, SUC_lexicon_tags_84 },
    { 0xb2789260, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99989265, SUC_lexicon_tags_11 },
    { 0x43389266, SUC_lexicon_tags_12 },
    { 0xf2d89267, SUC_lexicon_tags_7 },
    { 0xaa589268, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b8926d, SUC_lexicon_tags_25 },
    { 0x8358926e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab89271, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30789275, SUC_lexicon_tags_7 },
    { 0xc6389276, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d78927a, SUC_lexicon_tags_30 },
    { 0x8cd8927b, SUC_lexicon_tags_63 },
    { 0x5398927b, SUC_lexicon_tags_28 },
    { 0x70f8927d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x42b8927f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8189281, SUC_lexicon_tags_4 },
    { 0x9f989281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6589288, SUC_lexicon_tags_15 },
    { 0xc0789289, SUC_lexicon_tags_2 },
    { 0x10d89288, SUC_lexicon_tags_27 },
    { 0x9ed8928b, SUC_lexicon_tags_31 },
    { 0x16d8928b, SUC_lexicon_tags_4 },
    { 0xa2b8928d, SUC_lexicon_tags_38 },
    { 0x93f8928e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24189291, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa389295, SUC_lexicon_tags_28 },
    { 0x4d989295, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed389298, SUC_lexicon_tags_84 },
    { 0xab789299, SUC_lexicon_tags_63 },
    { 0x58b8929a, SUC_lexicon_tags_9 },
    { 0x5b38929a, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb958929f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x613892a1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9892a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d892ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7892b1, SUC_lexicon_tags_30 },
    { 0xe63892b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd51892b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115892ba, SUC_lexicon_tags_12 },
    { 0xb53892bb, SUC_lexicon_tags_69 },
    { 0x921892bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x285892be, SUC_lexicon_tags_51 },
    { 0xb77892bf, SUC_lexicon_tags_29 },
    { 0xac1892c0, SUC_lexicon_tags_0 },
    { 0x613892c0, SUC_lexicon_tags_13 },
    { 0xf45892c0, SUC_lexicon_tags_15 },
    { 0xae9892c1, SUC_lexicon_tags_11 },
    { 0x6b5892c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1892c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb21892c9, SUC_lexicon_tags_7 },
    { 0x2d3892c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d892cd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5892d8, SUC_lexicon_tags_0 },
    { 0x8ad892d9, SUC_lexicon_tags_15 },
    { 0xd95892da, SUC_lexicon_tags_8 },
    { 0x1db892db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33892de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33892e0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xec7892e2, SUC_lexicon_tags_21 },
    { 0xa4b892e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7892e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2d892eb, SUC_lexicon_tags_49 },
    { 0x3f3892eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81892ef, SUC_lexicon_tags_15 },
    { 0xbc1892f0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1892f5, SUC_lexicon_tags_38 },
    { 0xaf9892f6, SUC_lexicon_tags_7 },
    { 0xc47892f7, SUC_lexicon_tags_29 },
    { 0xf37892f8, SUC_lexicon_tags_11 },
    { 0xb0b892f7, SUC_lexicon_tags_47 },
    { 0xee1892fa, SUC_lexicon_tags_10 },
    { 0xcdd892fb, SUC_lexicon_tags_15 },
    { 0xd0d892fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af89301, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xedd89303, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc389305, SUC_lexicon_tags_28 },
    { 0x34789306, SUC_lexicon_tags_38 },
    { 0xc2f89305, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf2f89309, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x49b8930b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d8930e, SUC_lexicon_tags_38 },
    { 0xb38930e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb189313, SUC_lexicon_tags_17 },
    { 0x12d89313, SUC_lexicon_tags_2 },
    { 0x11189315, SUC_lexicon_tags_3 },
    { 0xcf589314, SUC_lexicon_tags_71 },
    { 0x69b89315, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf8931c, SUC_lexicon_tags_26 },
    { 0xd578931c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa589320, SUC_lexicon_tags_18 },
    { 0x55d89321, SUC_lexicon_tags_17 },
    { 0x8ef89320, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b89325, SUC_lexicon_tags_25 },
    { 0x97989326, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97f89328, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd8932c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d89330, SUC_lexicon_tags_17 },
    { 0xc189330, SUC_lexicon_tags_7 },
    { 0x2189331, SUC_lexicon_tags_37 },
    { 0xe4389333, SUC_lexicon_tags_37 },
    { 0x83389331, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x49989337, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd338933d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x778933f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e189346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f8934b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f89352, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10789354, SUC_lexicon_tags_12 },
    { 0xb3b89354, SUC_lexicon_tags_17 },
    { 0x1c789355, SUC_lexicon_tags_25 },
    { 0x98989357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc758935b, SUC_lexicon_tags_38 },
    { 0x7bd8935c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2789363, SUC_lexicon_tags_43 },
    { 0x54589364, SUC_lexicon_tags_37 },
    { 0x6a589363, SUC_lexicon_tags_4 },
    { 0x23f89366, SUC_lexicon_tags_12 },
    { 0x12989367, SUC_lexicon_tags_7 },
    { 0x13f89364, SUC_lexicon_tags_37 },
    { 0x32389369, SUC_lexicon_tags_38 },
    { 0x7bd8936a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1798936e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86989371, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22989377, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x6d589379, SUC_lexicon_tags_7 },
    { 0xaab89379, SUC_lexicon_tags_65 },
    { 0x3d8937b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2389380, SUC_lexicon_tags_28 },
    { 0x95589381, SUC_lexicon_tags_29 },
    { 0x98989380, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x72589384, SUC_lexicon_tags_17 },
    { 0xae789385, SUC_lexicon_tags_2 },
    { 0xc3d89384, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xad89388, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5418938b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c18938e, SUC_lexicon_tags_19 },
    { 0x9558938f, SUC_lexicon_tags_26 },
    { 0xa0f89390, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfed89392, SUC_lexicon_tags_14 },
    { 0xb89393, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b189395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb18939a, SUC_lexicon_tags_151 },
    { 0x28b8939a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb8939e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28f893a0, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3893a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdb1893a7, SUC_lexicon_tags_160 },
    { 0x19b893a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b893aa, SUC_lexicon_tags_14 },
    { 0xcb3893ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93893ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x49b893b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323893b4, SUC_lexicon_tags_3 },
    { 0xe9d893b5, SUC_lexicon_tags_63 },
    { 0xfe3893b6, SUC_lexicon_tags_15 },
    { 0x14d893b6, SUC_lexicon_tags_15 },
    { 0x29b893b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9893bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9893c8, SUC_lexicon_tags_10 },
    { 0x1b5893c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x629893cc, SUC_lexicon_tags_11 },
    { 0x999893cc, SUC_lexicon_tags_12 },
    { 0x725893ce, SUC_lexicon_tags_155 },
    { 0xb65893cf, SUC_lexicon_tags_2 },
    { 0x25d893d0, SUC_lexicon_tags_13 },
    { 0xfd3893d1, SUC_lexicon_tags_37 },
    { 0x72d893d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9893d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7893da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b893df, SUC_lexicon_tags_9 },
    { 0x42f893e0, SUC_lexicon_tags_15 },
    { 0xc7893e0, SUC_lexicon_tags_1 },
    { 0xaed893e1, SUC_lexicon_tags_11 },
    { 0xbf5893e3, SUC_lexicon_tags_0 },
    { 0x4e3893e4, SUC_lexicon_tags_43 },
    { 0xbab893e4, SUC_lexicon_tags_2 },
    { 0x9b7893e6, SUC_lexicon_tags_38 },
    { 0xda7893e3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x48f893e9, SUC_lexicon_tags_14 },
    { 0x83d893ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9893ef, SUC_lexicon_tags_83 },
    { 0xa37893f0, SUC_lexicon_tags_7 },
    { 0xa1d893f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865893f5, SUC_lexicon_tags_12 },
    { 0x753893f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85893f8, SUC_lexicon_tags_47 },
    { 0x731893f9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41989400, SUC_lexicon_tags_49 },
    { 0x41989401, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x7e189403, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9789408, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65d8940b, SUC_lexicon_tags_15 },
    { 0x4618940c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbab8940e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf89411, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85389413, SUC_lexicon_tags_49 },
    { 0x4eb89414, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c589418, SUC_lexicon_tags_1 },
    { 0x35789418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d8941d, SUC_lexicon_tags_11 },
    { 0x1c98941e, SUC_lexicon_tags_33 },
    { 0xbe58941d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9989427, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb6f89429, SUC_lexicon_tags_25 },
    { 0x5d589429, SUC_lexicon_tags_33 },
    { 0xab58942b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x7dd8942d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19989432, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d989437, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe2989439, SUC_lexicon_tags_2 },
    { 0x57d8943a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf8943e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb89442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99d89444, SUC_lexicon_tags_7 },
    { 0x5af89444, SUC_lexicon_tags_26 },
    { 0x77b89446, SUC_lexicon_tags_38 },
    { 0x8bd89445, SUC_lexicon_tags_17 },
    { 0xb9989447, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb538944a, SUC_lexicon_tags_25 },
    { 0x8378944a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe658944d, SUC_lexicon_tags_37 },
    { 0x88d8944e, SUC_lexicon_tags_10 },
    { 0x3158944f, SUC_lexicon_tags_82 },
    { 0xed589450, SUC_lexicon_tags_19 },
    { 0x44189450, SUC_lexicon_tags_66 },
    { 0x13189450, SUC_lexicon_tags_1 },
    { 0x3ab89451, SUC_lexicon_tags_33 },
    { 0xb3f89450, SUC_lexicon_tags_65 },
    { 0xb9d89455, SUC_lexicon_tags_37 },
    { 0x4e789456, SUC_lexicon_tags_2 },
    { 0xa7589455, SUC_lexicon_tags_55 },
    { 0xbcb89450, SUC_lexicon_tags_17 },
    { 0xf6189456, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19389460, SUC_lexicon_tags_61 },
    { 0xce789461, SUC_lexicon_tags_1 },
    { 0x62589461, SUC_lexicon_tags_12 },
    { 0x84589463, SUC_lexicon_tags_29 },
    { 0x7ed89464, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a78946a, SUC_lexicon_tags_65 },
    { 0xaaf8946b, SUC_lexicon_tags_17 },
    { 0x3698946c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1198946e, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x97989470, SUC_lexicon_tags_105 },
    { 0xba389470, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d89474, SUC_lexicon_tags_25 },
    { 0xa9d89475, SUC_lexicon_tags_56 },
    { 0xf4989476, SUC_lexicon_tags_25 },
    { 0x29d89477, SUC_lexicon_tags_0 },
    { 0xdaf89478, SUC_lexicon_tags_16 },
    { 0xad789479, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa138947c, SUC_lexicon_tags_14 },
    { 0x6198947d, SUC_lexicon_tags_3 },
    { 0xe818947d, SUC_lexicon_tags_66 },
    { 0x1038947f, SUC_lexicon_tags_0 },
    { 0xd258947d, SUC_lexicon_tags_11 },
    { 0x52989481, SUC_lexicon_tags_63 },
    { 0x54789481, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe389484, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8af89486, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc958948c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7998948e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb89492, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f89498, SUC_lexicon_tags_6 },
    { 0x4dd89499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a78949d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce58949f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x229894a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd894a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9894b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b894be, SUC_lexicon_tags_6 },
    { 0x121894be, SUC_lexicon_tags_47 },
    { 0xced894bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b894c3, SUC_lexicon_tags_66 },
    { 0xbfb894c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945894cb, SUC_lexicon_tags_29 },
    { 0xd85894cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1894d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87d894d4, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9894d8, SUC_lexicon_tags_70 },
    { 0xc8f894d9, SUC_lexicon_tags_47 },
    { 0xd19894da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xce3894dd, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43894e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d894e4, SUC_lexicon_tags_15 },
    { 0x903894e4, SUC_lexicon_tags_2 },
    { 0x677894e5, SUC_lexicon_tags_15 },
    { 0xd894e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5894ea, SUC_lexicon_tags_23 },
    { 0xb93894ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27894ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb894f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1894f4, SUC_lexicon_tags_7 },
    { 0x2a9894f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6b894f7, SUC_lexicon_tags_15 },
    { 0xe8d894f7, SUC_lexicon_tags_31 },
    { 0x38f894f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb37894fb, SUC_lexicon_tags_23 },
    { 0x3b1894fb, SUC_lexicon_tags_15 },
    { 0x135894fc, SUC_lexicon_tags_31 },
    { 0xa19894fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2189505, SUC_lexicon_tags_3 },
    { 0x6f789505, SUC_lexicon_tags_17 },
    { 0xe6989507, SUC_lexicon_tags_23 },
    { 0xf4189507, SUC_lexicon_tags_485 },
    { 0x13389507, SUC_lexicon_tags_17 },
    { 0x75b89509, SUC_lexicon_tags_62 },
    { 0xdcf8950a, SUC_lexicon_tags_22 },
    { 0x12f8950c, SUC_lexicon_tags_79 },
    { 0x3b8950d, SUC_lexicon_tags_49 },
    { 0xca589507, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1189518, SUC_lexicon_tags_11 },
    { 0xb8d89519, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd8951c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c189520, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80589524, SUC_lexicon_tags_38 },
    { 0xfd89525, SUC_lexicon_tags_11 },
    { 0xdfd89525, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8958952b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b8952e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x21589530, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xa7b89532, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12989534, SUC_lexicon_tags_45 },
    { 0xb0d89535, SUC_lexicon_tags_26 },
    { 0xa9789535, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x82189538, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa778953b, SUC_lexicon_tags_87 },
    { 0x3db8953c, SUC_lexicon_tags_0 },
    { 0xd398953c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x47589540, SUC_lexicon_tags_7 },
    { 0xf1189540, SUC_lexicon_tags_31 },
    { 0x45589542, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f8954a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e989554, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9f589556, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x12389559, SUC_lexicon_tags_39 },
    { 0x8f989559, SUC_lexicon_tags_21 },
    { 0x3a989559, SUC_lexicon_tags_38 },
    { 0x1078955c, SUC_lexicon_tags_94 },
    { 0x2798955c, SUC_lexicon_tags_33 },
    { 0x7358955e, SUC_lexicon_tags_37 },
    { 0x55d8955e, SUC_lexicon_tags_39 },
    { 0x3e389560, SUC_lexicon_tags_28 },
    { 0xb958955c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60189565, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f89568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6538956a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd18956f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x8389572, SUC_lexicon_tags_2 },
    { 0x17f89573, SUC_lexicon_tags_33 },
    { 0x69589574, SUC_lexicon_tags_49 },
    { 0x9a789575, SUC_lexicon_tags_37 },
    { 0x4b789576, SUC_lexicon_tags_55 },
    { 0x80f89577, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b78957a, SUC_lexicon_tags_23 },
    { 0x9a78957b, SUC_lexicon_tags_11 },
    { 0xcc38957a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9389581, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd189583, SUC_lexicon_tags_31 },
    { 0xcd589584, SUC_lexicon_tags_7 },
    { 0xdb389584, SUC_lexicon_tags_43 },
    { 0xdb989586, SUC_lexicon_tags_37 },
    { 0x76d89585, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd6d89589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd438958c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96589590, SUC_lexicon_tags_108 },
    { 0xae389591, SUC_lexicon_tags_9 },
    { 0x64189590, SUC_lexicon_tags_26 },
    { 0x69389593, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4589596, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b89599, SUC_lexicon_tags_12 },
    { 0xb758959a, SUC_lexicon_tags_11 },
    { 0x6a38959b, SUC_lexicon_tags_15 },
    { 0x9658959c, SUC_lexicon_tags_6 },
    { 0x6db8959c, SUC_lexicon_tags_15 },
    { 0x7638959e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67b895a0, SUC_lexicon_tags_10 },
    { 0xa11895a1, SUC_lexicon_tags_60 },
    { 0x1d7895a2, SUC_lexicon_tags_87 },
    { 0xe2d895a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455895a8, SUC_lexicon_tags_26 },
    { 0x203895a8, SUC_lexicon_tags_37 },
    { 0x8e3895aa, SUC_lexicon_tags_26 },
    { 0xd17895aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b895af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d895b2, SUC_lexicon_tags_47 },
    { 0xb2f895b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x465895b5, SUC_lexicon_tags_25 },
    { 0x1c5895b6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xddb895b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b895bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b895bf, SUC_lexicon_tags_7 },
    { 0x673895bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e7895c2, SUC_lexicon_tags_11 },
    { 0x211895c3, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x27895c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5895c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7895cc, SUC_lexicon_tags_17 },
    { 0x4c3895cd, SUC_lexicon_tags_15 },
    { 0xf51895cd, SUC_lexicon_tags_2 },
    { 0x393895cf, SUC_lexicon_tags_39 },
    { 0xfe3895cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d895d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcbf895d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f895d9, SUC_lexicon_tags_61 },
    { 0xcc7895da, SUC_lexicon_tags_55 },
    { 0x35f895d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a3895dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47895e1, SUC_lexicon_tags_29 },
    { 0x401895e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983895e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579895ed, SUC_lexicon_tags_7 },
    { 0x285895ed, SUC_lexicon_tags_2 },
    { 0x321895ef, SUC_lexicon_tags_8 },
    { 0xbb1895f0, SUC_lexicon_tags_2 },
    { 0xa59895f0, SUC_lexicon_tags_10 },
    { 0xd5f895f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x677895f4, SUC_lexicon_tags_1 },
    { 0x385895f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871895ff, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39789604, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xec189606, SUC_lexicon_tags_29 },
    { 0x25189607, SUC_lexicon_tags_17 },
    { 0x59b89606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9838960a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d98960f, SUC_lexicon_tags_1 },
    { 0x73789610, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc989615, SUC_lexicon_tags_3 },
    { 0x88b89616, SUC_lexicon_tags_31 },
    { 0xd0f89615, SUC_lexicon_tags_2 },
    { 0xdc789618, SUC_lexicon_tags_12 },
    { 0x72589618, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xa518961b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d989620, SUC_lexicon_tags_37 },
    { 0x7f389620, SUC_lexicon_tags_25 },
    { 0x35189621, SUC_lexicon_tags_39 },
    { 0xa1b89621, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97189628, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdd38962a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6ad8962c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59189630, SUC_lexicon_tags_15 },
    { 0xe7789630, SUC_lexicon_tags_4 },
    { 0xa7789631, SUC_lexicon_tags_65 },
    { 0xfd89630, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc38963c, SUC_lexicon_tags_10 },
    { 0x3a78963c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6cf8963f, SUC_lexicon_tags_861 },
    { 0, NULL },
    { 0x2389641, SUC_lexicon_tags_1 },
    { 0x95989642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b789645, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5bb89647, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a18964c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf38964f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d89652, SUC_lexicon_tags_60 },
    { 0x97d89653, SUC_lexicon_tags_22 },
    { 0x35789654, SUC_lexicon_tags_49 },
    { 0xf9189655, SUC_lexicon_tags_47 },
    { 0x8ad89656, SUC_lexicon_tags_7 },
    { 0xf4b89657, SUC_lexicon_tags_84 },
    { 0xf3d89657, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa918965a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af8965d, SUC_lexicon_tags_1 },
    { 0xe458965e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22789661, SUC_lexicon_tags_6 },
    { 0xa6f89661, SUC_lexicon_tags_7 },
    { 0xeef89663, SUC_lexicon_tags_14 },
    { 0x6e589664, SUC_lexicon_tags_9 },
    { 0xa1389664, SUC_lexicon_tags_37 },
    { 0xadb89665, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d89669, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8389671, SUC_lexicon_tags_20 },
    { 0x22189672, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d589675, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2789677, SUC_lexicon_tags_0 },
    { 0x7ef89678, SUC_lexicon_tags_16 },
    { 0xa4389678, SUC_lexicon_tags_9 },
    { 0x77d8967a, SUC_lexicon_tags_79 },
    { 0x38b89679, SUC_lexicon_tags_108 },
    { 0xff38967c, SUC_lexicon_tags_17 },
    { 0x4218967d, SUC_lexicon_tags_17 },
    { 0x238967e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd389681, SUC_lexicon_tags_17 },
    { 0x9f589681, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0989690, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x27389693, SUC_lexicon_tags_42 },
    { 0x23589694, SUC_lexicon_tags_43 },
    { 0xf589695, SUC_lexicon_tags_1488 },
    { 0, NULL },
    { 0xcab89697, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d8969d, SUC_lexicon_tags_23 },
    { 0x7238969e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13896a4, SUC_lexicon_tags_14 },
    { 0xfb896a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x665896a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603896ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ed896ae, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x739896b0, SUC_lexicon_tags_48 },
    { 0xac7896b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d896b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21896bc, SUC_lexicon_tags_15 },
    { 0xed3896bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6d896bf, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf896c6, SUC_lexicon_tags_7 },
    { 0x71b896c7, SUC_lexicon_tags_384 },
    { 0x4b9896c8, SUC_lexicon_tags_7 },
    { 0x869896c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879896ce, SUC_lexicon_tags_10 },
    { 0x179896ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9896d2, SUC_lexicon_tags_23 },
    { 0x561896d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b896db, SUC_lexicon_tags_6 },
    { 0xfd9896dc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfbb896de, SUC_lexicon_tags_37 },
    { 0x54f896df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223896ea, SUC_lexicon_tags_37 },
    { 0xf4d896ea, SUC_lexicon_tags_12 },
    { 0x873896ec, SUC_lexicon_tags_15 },
    { 0x9dd896ec, SUC_lexicon_tags_25 },
    { 0x1a3896ec, SUC_lexicon_tags_30 },
    { 0xcd9896ef, SUC_lexicon_tags_27 },
    { 0xd2d896ef, SUC_lexicon_tags_11 },
    { 0xd53896ef, SUC_lexicon_tags_87 },
    { 0x99896ee, SUC_lexicon_tags_25 },
    { 0x3bb896f3, SUC_lexicon_tags_34 },
    { 0x4b896ea, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5f3896f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f896fb, SUC_lexicon_tags_0 },
    { 0xb69896fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09896ff, SUC_lexicon_tags_6 },
    { 0x92189700, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31189702, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74f89704, SUC_lexicon_tags_2 },
    { 0xee589705, SUC_lexicon_tags_1 },
    { 0xd989705, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c58970e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d89711, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd89714, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f8971c, SUC_lexicon_tags_17 },
    { 0x5a58971d, SUC_lexicon_tags_7 },
    { 0xe698971e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdd89720, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4989723, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d89728, SUC_lexicon_tags_1 },
    { 0x25189729, SUC_lexicon_tags_27 },
    { 0x7158972a, SUC_lexicon_tags_30 },
    { 0x5c78972b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3158972d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb89730, SUC_lexicon_tags_17 },
    { 0xce389731, SUC_lexicon_tags_2 },
    { 0x99389732, SUC_lexicon_tags_11 },
    { 0xd4389732, SUC_lexicon_tags_2 },
    { 0x76b89733, SUC_lexicon_tags_19 },
    { 0x1a189735, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d89740, SUC_lexicon_tags_45 },
    { 0xd8189741, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e989744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86189749, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x83f8974b, SUC_lexicon_tags_31 },
    { 0x5238974c, SUC_lexicon_tags_12 },
    { 0x9f58974d, SUC_lexicon_tags_14 },
    { 0xd218974d, SUC_lexicon_tags_43 },
    { 0x2e98974f, SUC_lexicon_tags_25 },
    { 0x6a58974c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11389755, SUC_lexicon_tags_12 },
    { 0x9b389756, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad8975a, SUC_lexicon_tags_25 },
    { 0x3898975a, SUC_lexicon_tags_15 },
    { 0x5ed8975b, SUC_lexicon_tags_33 },
    { 0x8b18975d, SUC_lexicon_tags_17 },
    { 0x45d8975b, SUC_lexicon_tags_34 },
    { 0x1eb8975f, SUC_lexicon_tags_68 },
    { 0xd0789760, SUC_lexicon_tags_17 },
    { 0x4818975e, SUC_lexicon_tags_30 },
    { 0x2f789762, SUC_lexicon_tags_12 },
    { 0x5e789762, SUC_lexicon_tags_1459 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff89767, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f8976b, SUC_lexicon_tags_49 },
    { 0xf6b8976c, SUC_lexicon_tags_21 },
    { 0x18b8976b, SUC_lexicon_tags_15 },
    { 0x1b18976d, SUC_lexicon_tags_32 },
    { 0x88b8976f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdc789771, SUC_lexicon_tags_38 },
    { 0xfef89772, SUC_lexicon_tags_37 },
    { 0xa8d89773, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e389776, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd89778, SUC_lexicon_tags_19 },
    { 0xf8989779, SUC_lexicon_tags_17 },
    { 0x81d89779, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8218977c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b8977f, SUC_lexicon_tags_10 },
    { 0xa0b89780, SUC_lexicon_tags_23 },
    { 0x4398977f, SUC_lexicon_tags_38 },
    { 0xb2789782, SUC_lexicon_tags_27 },
    { 0xee989783, SUC_lexicon_tags_47 },
    { 0x2dd89784, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc6f89786, SUC_lexicon_tags_201 },
    { 0xa1389787, SUC_lexicon_tags_17 },
    { 0x7c389788, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2998978a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2998978c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7238978e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1b89790, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf5589792, SUC_lexicon_tags_11 },
    { 0x3389793, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8979d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df897a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa45897a2, SUC_lexicon_tags_1 },
    { 0x403897a2, SUC_lexicon_tags_7 },
    { 0x9e5897a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d897a9, SUC_lexicon_tags_63 },
    { 0x717897aa, SUC_lexicon_tags_17 },
    { 0x3a9897ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93897af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e1897b1, SUC_lexicon_tags_25 },
    { 0x64f897b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719897b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9897bc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73897bf, SUC_lexicon_tags_60 },
    { 0xfdd897bf, SUC_lexicon_tags_2 },
    { 0x7e3897bf, SUC_lexicon_tags_14 },
    { 0xb9b897c2, SUC_lexicon_tags_8 },
    { 0x843897c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7897c6, SUC_lexicon_tags_26 },
    { 0xe1897c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553897d2, SUC_lexicon_tags_17 },
    { 0xcdf897d3, SUC_lexicon_tags_1 },
    { 0x655897d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb897d7, SUC_lexicon_tags_37 },
    { 0x4b7897d7, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925897dc, SUC_lexicon_tags_19 },
    { 0x889897dd, SUC_lexicon_tags_32 },
    { 0x31b897de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83897e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94b897e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf897ea, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f897ed, SUC_lexicon_tags_31 },
    { 0x83b897ee, SUC_lexicon_tags_21 },
    { 0x599897ef, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1897f2, SUC_lexicon_tags_7 },
    { 0xc53897f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45897f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63897fa, SUC_lexicon_tags_1 },
    { 0xa69897fb, SUC_lexicon_tags_6 },
    { 0xebb897fb, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xde5897fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3789804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86189807, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71189809, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78980e, SUC_lexicon_tags_1 },
    { 0x5cd8980e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1189812, SUC_lexicon_tags_0 },
    { 0x83589813, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x51389815, SUC_lexicon_tags_1551 },
    { 0x5f989816, SUC_lexicon_tags_11 },
    { 0x79189817, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xee789819, SUC_lexicon_tags_28 },
    { 0x4e389819, SUC_lexicon_tags_25 },
    { 0xbff8981b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe998981d, SUC_lexicon_tags_37 },
    { 0x4378981e, SUC_lexicon_tags_25 },
    { 0xd858981f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b89826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d89829, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa578982f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b89836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7f89838, SUC_lexicon_tags_9 },
    { 0x66d89839, SUC_lexicon_tags_35 },
    { 0xcb589838, SUC_lexicon_tags_38 },
    { 0xbd789838, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d89841, SUC_lexicon_tags_151 },
    { 0x64b89841, SUC_lexicon_tags_47 },
    { 0x7d89841, SUC_lexicon_tags_12 },
    { 0x73f89842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52b89846, SUC_lexicon_tags_11 },
    { 0xed589846, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c789849, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59189854, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3789864, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb3d89866, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57d89868, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b8986b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f8986e, SUC_lexicon_tags_11 },
    { 0x1f78986f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73789873, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b89876, SUC_lexicon_tags_14 },
    { 0xe5389876, SUC_lexicon_tags_1 },
    { 0xf589878, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c18987a, SUC_lexicon_tags_49 },
    { 0x4298987b, SUC_lexicon_tags_30 },
    { 0xd5f8987b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc789880, SUC_lexicon_tags_51 },
    { 0xb8989881, SUC_lexicon_tags_23 },
    { 0x31189881, SUC_lexicon_tags_2 },
    { 0x93189882, SUC_lexicon_tags_21 },
    { 0x68789884, SUC_lexicon_tags_7 },
    { 0xfcf89885, SUC_lexicon_tags_2 },
    { 0x4af89884, SUC_lexicon_tags_68 },
    { 0xf0d89884, SUC_lexicon_tags_12 },
    { 0x96f89888, SUC_lexicon_tags_43 },
    { 0x14789888, SUC_lexicon_tags_39 },
    { 0xef989889, SUC_lexicon_tags_0 },
    { 0x4c189888, SUC_lexicon_tags_25 },
    { 0x7a38988a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeeb8988e, SUC_lexicon_tags_29 },
    { 0xd678988f, SUC_lexicon_tags_2 },
    { 0xaf38988f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8789892, SUC_lexicon_tags_0 },
    { 0xa1b89892, SUC_lexicon_tags_47 },
    { 0x5ad89894, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15189898, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xde18989a, SUC_lexicon_tags_532 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f8989e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29d898a0, SUC_lexicon_tags_15 },
    { 0x2bf898a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xed5898a3, SUC_lexicon_tags_47 },
    { 0x6fd898a4, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0x291898a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f898ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81898b2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db898b5, SUC_lexicon_tags_10 },
    { 0x611898b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa69898b8, SUC_lexicon_tags_37 },
    { 0x53d898b8, SUC_lexicon_tags_25 },
    { 0x973898b8, SUC_lexicon_tags_63 },
    { 0xa5f898b8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x411898be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ff898c0, SUC_lexicon_tags_31 },
    { 0x395898c1, SUC_lexicon_tags_39 },
    { 0xd3f898c2, SUC_lexicon_tags_65 },
    { 0xb19898c2, SUC_lexicon_tags_25 },
    { 0x5e9898c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d898ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817898ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc3f898d0, SUC_lexicon_tags_11 },
    { 0x9e9898d1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219898d5, SUC_lexicon_tags_1 },
    { 0x7fb898d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245898db, SUC_lexicon_tags_39 },
    { 0x6c9898dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd898e0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5898e5, SUC_lexicon_tags_37 },
    { 0x4b7898e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801898f0, SUC_lexicon_tags_60 },
    { 0xe7898f1, SUC_lexicon_tags_55 },
    { 0x451898f2, SUC_lexicon_tags_37 },
    { 0xa01898f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693898f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x297898fb, SUC_lexicon_tags_48 },
    { 0x9d9898fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1789907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd989909, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb038990b, SUC_lexicon_tags_31 },
    { 0x8ad8990c, SUC_lexicon_tags_23 },
    { 0xc798990b, SUC_lexicon_tags_1 },
    { 0x2fd8990e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f89911, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a189917, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xdcf89919, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea78991b, SUC_lexicon_tags_15 },
    { 0xb678991c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3258991f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6189925, SUC_lexicon_tags_57 },
    { 0x16d89925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89b89928, SUC_lexicon_tags_23 },
    { 0x36589929, SUC_lexicon_tags_16 },
    { 0x9cb89929, SUC_lexicon_tags_26 },
    { 0x80389929, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d58992d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc78993f, SUC_lexicon_tags_11 },
    { 0x1f189940, SUC_lexicon_tags_15 },
    { 0x23f89940, SUC_lexicon_tags_27 },
    { 0x23989940, SUC_lexicon_tags_2 },
    { 0xd6789943, SUC_lexicon_tags_16 },
    { 0xd9189942, SUC_lexicon_tags_2 },
    { 0x7af89945, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b989949, SUC_lexicon_tags_11 },
    { 0xc2b89949, SUC_lexicon_tags_38 },
    { 0x14b8994b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd378994d, SUC_lexicon_tags_12 },
    { 0x4478994d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b89955, SUC_lexicon_tags_17 },
    { 0x52189955, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b8995e, SUC_lexicon_tags_591 },
    { 0xcb78995e, SUC_lexicon_tags_45 },
    { 0x43f89960, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7589962, SUC_lexicon_tags_39 },
    { 0xf2389963, SUC_lexicon_tags_55 },
    { 0x46d89963, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d589966, SUC_lexicon_tags_68 },
    { 0xc789967, SUC_lexicon_tags_17 },
    { 0x2cf89967, SUC_lexicon_tags_11 },
    { 0xbb389969, SUC_lexicon_tags_1 },
    { 0xf2789969, SUC_lexicon_tags_25 },
    { 0x6b89967, SUC_lexicon_tags_49 },
    { 0x12b89968, SUC_lexicon_tags_1 },
    { 0x5b38996d, SUC_lexicon_tags_15 },
    { 0x3878996e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe589972, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf538997a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0989982, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41789987, SUC_lexicon_tags_43 },
    { 0xf5589987, SUC_lexicon_tags_8 },
    { 0x5f389989, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f98998c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4789992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43f89994, SUC_lexicon_tags_7 },
    { 0x6e989994, SUC_lexicon_tags_38 },
    { 0x58789995, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb8999b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf558999d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683899a4, SUC_lexicon_tags_47 },
    { 0x751899a5, SUC_lexicon_tags_12 },
    { 0xc8f899a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5899aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687899af, SUC_lexicon_tags_26 },
    { 0xb7f899b0, SUC_lexicon_tags_14 },
    { 0xf6b899b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771899b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1bd899ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9899c6, SUC_lexicon_tags_11 },
    { 0xbe1899c6, SUC_lexicon_tags_38 },
    { 0x2e9899c8, SUC_lexicon_tags_12 },
    { 0xe1d899c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x397899cb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbed899cd, SUC_lexicon_tags_2 },
    { 0x29b899cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97899d1, SUC_lexicon_tags_11 },
    { 0x4bb899d1, SUC_lexicon_tags_25 },
    { 0x1a3899d3, SUC_lexicon_tags_37 },
    { 0x6d1899d3, SUC_lexicon_tags_8 },
    { 0x309899d5, SUC_lexicon_tags_11 },
    { 0x1e7899d1, SUC_lexicon_tags_12 },
    { 0x255899d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507899db, SUC_lexicon_tags_2 },
    { 0x589899db, SUC_lexicon_tags_0 },
    { 0x5d3899db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f899e0, SUC_lexicon_tags_17 },
    { 0xb59899e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f899e5, SUC_lexicon_tags_1 },
    { 0xac5899e6, SUC_lexicon_tags_26 },
    { 0x5d9899e6, SUC_lexicon_tags_15 },
    { 0x671899e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07899eb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615899ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5589a00, SUC_lexicon_tags_18 },
    { 0x4e789a00, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43589a09, SUC_lexicon_tags_25 },
    { 0x47389a0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca589a0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab89a12, SUC_lexicon_tags_37 },
    { 0x52789a12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16589a17, SUC_lexicon_tags_11 },
    { 0xfd989a18, SUC_lexicon_tags_26 },
    { 0x89189a17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xab989a1b, SUC_lexicon_tags_7 },
    { 0x6d989a1c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4dd89a1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x12189a21, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf89a24, SUC_lexicon_tags_87 },
    { 0x76189a25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x61789a27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42989a2d, SUC_lexicon_tags_12 },
    { 0xfad89a2e, SUC_lexicon_tags_87 },
    { 0x40189a2d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd89a32, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc3789a34, SUC_lexicon_tags_15 },
    { 0x68589a35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff89a38, SUC_lexicon_tags_65 },
    { 0x7e789a39, SUC_lexicon_tags_23 },
    { 0xcb389a3a, SUC_lexicon_tags_18 },
    { 0x2c589a3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c989a3d, SUC_lexicon_tags_82 },
    { 0x88789a3e, SUC_lexicon_tags_47 },
    { 0xddb89a3f, SUC_lexicon_tags_31 },
    { 0xee389a40, SUC_lexicon_tags_25 },
    { 0x60189a3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc189a48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2989a4a, SUC_lexicon_tags_1 },
    { 0xbeb89a4b, SUC_lexicon_tags_11 },
    { 0xadf89a4b, SUC_lexicon_tags_43 },
    { 0x99589a4b, SUC_lexicon_tags_31 },
    { 0xbe189a4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3789a51, SUC_lexicon_tags_1281 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f89a55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x12189a58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x73989a5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe2389a5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2789a62, SUC_lexicon_tags_15 },
    { 0xfe989a63, SUC_lexicon_tags_21 },
    { 0x1a389a63, SUC_lexicon_tags_19 },
    { 0x23789a65, SUC_lexicon_tags_185 },
    { 0x4a789a66, SUC_lexicon_tags_74 },
    { 0x6bb89a65, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x55789a69, SUC_lexicon_tags_17 },
    { 0x78189a6a, SUC_lexicon_tags_159 },
    { 0x18589a6b, SUC_lexicon_tags_47 },
    { 0x21789a6c, SUC_lexicon_tags_14 },
    { 0x79789a6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d89a73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x14189a76, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5189a79, SUC_lexicon_tags_7 },
    { 0x2e589a79, SUC_lexicon_tags_11 },
    { 0x1b989a79, SUC_lexicon_tags_15 },
    { 0xf4f89a7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78989a7e, SUC_lexicon_tags_15 },
    { 0xe5b89a7f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d789a83, SUC_lexicon_tags_25 },
    { 0x1f389a83, SUC_lexicon_tags_47 },
    { 0x81b89a84, SUC_lexicon_tags_0 },
    { 0x7989a84, SUC_lexicon_tags_15 },
    { 0xb9189a86, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b89a8b, SUC_lexicon_tags_25 },
    { 0xdc389a8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f89a8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba189a95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1189a97, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x35789a9a, SUC_lexicon_tags_60 },
    { 0x30589a9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7389aa2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe5789aa4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf389aa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d89aab, SUC_lexicon_tags_66 },
    { 0x38b89aac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb789ab4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x50189ab6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa189ab9, SUC_lexicon_tags_17 },
    { 0xd7989ab9, SUC_lexicon_tags_15 },
    { 0x5bd89abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7989abe, SUC_lexicon_tags_46 },
    { 0xff989abf, SUC_lexicon_tags_17 },
    { 0x98d89abe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c789ac3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x69d89ac6, SUC_lexicon_tags_3 },
    { 0xc0589ac7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d89aca, SUC_lexicon_tags_9 },
    { 0xd9789acb, SUC_lexicon_tags_33 },
    { 0x78789aca, SUC_lexicon_tags_17 },
    { 0xab789aca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x62b89acf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d189ad2, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0, NULL },
    { 0xf189ad5, SUC_lexicon_tags_37 },
    { 0x3d89ad5, SUC_lexicon_tags_15 },
    { 0xb4b89ad5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95989adb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfc389add, SUC_lexicon_tags_49 },
    { 0x2f389add, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x87589ae1, SUC_lexicon_tags_17 },
    { 0x5bf89ae2, SUC_lexicon_tags_1 },
    { 0xc9d89ae2, SUC_lexicon_tags_25 },
    { 0xed989ae4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b89ae8, SUC_lexicon_tags_15 },
    { 0x86b89ae9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d89af1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b89af4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71189afd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f589b06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb189b09, SUC_lexicon_tags_2 },
    { 0x71189b0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34389b11, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb789b15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d89b1c, SUC_lexicon_tags_25 },
    { 0x17f89b1d, SUC_lexicon_tags_60 },
    { 0x77d89b1e, SUC_lexicon_tags_27 },
    { 0xbb189b1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e189b26, SUC_lexicon_tags_47 },
    { 0x2589b27, SUC_lexicon_tags_15 },
    { 0x38d89b28, SUC_lexicon_tags_7 },
    { 0x1d189b28, SUC_lexicon_tags_48 },
    { 0xff589b26, SUC_lexicon_tags_1 },
    { 0x50789b2b, SUC_lexicon_tags_69 },
    { 0xa1789b29, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb89b31, SUC_lexicon_tags_15 },
    { 0xe2f89b32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfd89b34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xab989b36, SUC_lexicon_tags_16 },
    { 0xf8389b36, SUC_lexicon_tags_2 },
    { 0x25589b37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82b89b3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c389b41, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc4189b43, SUC_lexicon_tags_15 },
    { 0x9af89b44, SUC_lexicon_tags_7 },
    { 0xe8d89b45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d89b49, SUC_lexicon_tags_1 },
    { 0xdbb89b49, SUC_lexicon_tags_11 },
    { 0x98589b4b, SUC_lexicon_tags_25 },
    { 0xc6f89b4b, SUC_lexicon_tags_15 },
    { 0xc7f89b4b, SUC_lexicon_tags_351 },
    { 0xf2389b4e, SUC_lexicon_tags_30 },
    { 0x84b89b4f, SUC_lexicon_tags_12 },
    { 0xd4b89b4f, SUC_lexicon_tags_7 },
    { 0x3b789b4b, SUC_lexicon_tags_21 },
    { 0xac789b4b, SUC_lexicon_tags_68 },
    { 0x50789b49, SUC_lexicon_tags_37 },
    { 0xa2f89b54, SUC_lexicon_tags_32 },
    { 0x5df89b54, SUC_lexicon_tags_25 },
    { 0x9f89b53, SUC_lexicon_tags_17 },
    { 0x47989b57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4789b59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7389b5c, SUC_lexicon_tags_12 },
    { 0x56b89b5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5789b61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a789b64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82989b69, SUC_lexicon_tags_60 },
    { 0x75189b6a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf89b6d, SUC_lexicon_tags_12 },
    { 0x87b89b6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc389b72, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f89b75, SUC_lexicon_tags_45 },
    { 0x3989b75, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e389b79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57d89b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc989b7f, SUC_lexicon_tags_23 },
    { 0x8a589b80, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e989b82, SUC_lexicon_tags_43 },
    { 0xdfb89b83, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x42589b86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48989b89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fb89b8b, SUC_lexicon_tags_47 },
    { 0xd9189b8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf89b91, SUC_lexicon_tags_17 },
    { 0x13989b92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e789b9b, SUC_lexicon_tags_2 },
    { 0xe989b9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b89ba6, SUC_lexicon_tags_11 },
    { 0xf9f89ba7, SUC_lexicon_tags_3 },
    { 0x70b89ba7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80789bac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb89bb2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x71789bb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d789bbc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d389bbe, SUC_lexicon_tags_43 },
    { 0x13589bbe, SUC_lexicon_tags_15 },
    { 0xc4389bc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57189bc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bd89bc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd89bc7, SUC_lexicon_tags_80 },
    { 0x8cf89bc8, SUC_lexicon_tags_37 },
    { 0x75f89bc9, SUC_lexicon_tags_4 },
    { 0xd8389bc8, SUC_lexicon_tags_7 },
    { 0xddf89bcb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6989bd2, SUC_lexicon_tags_43 },
    { 0xd8f89bd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f89bd8, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95389bdc, SUC_lexicon_tags_2 },
    { 0xefd89bdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7589be1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a589be5, SUC_lexicon_tags_238 },
    { 0xbc789be6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce389be8, SUC_lexicon_tags_7 },
    { 0xbcf89be9, SUC_lexicon_tags_15 },
    { 0xf4589bea, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xae789bed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb89bf0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe3389bf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13989bf4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7d189bf6, SUC_lexicon_tags_8 },
    { 0x8f589bf6, SUC_lexicon_tags_47 },
    { 0x92789bf6, SUC_lexicon_tags_761 },
    { 0xf1d89bf6, SUC_lexicon_tags_17 },
    { 0x68f89bf9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbcf89bfc, SUC_lexicon_tags_34 },
    { 0xc2b89bfc, SUC_lexicon_tags_46 },
    { 0xcd389bfc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8989c03, SUC_lexicon_tags_11 },
    { 0x86989c04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33389c06, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9eb89c08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b89c0e, SUC_lexicon_tags_293 },
    { 0xc4d89c0f, SUC_lexicon_tags_8 },
    { 0x98b89c0f, SUC_lexicon_tags_27 },
    { 0xbc989c11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf89c17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d89c1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98989c21, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3af89c23, SUC_lexicon_tags_17 },
    { 0xde589c24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x78589c27, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf0589c29, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x18589c2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7df89c2e, SUC_lexicon_tags_25 },
    { 0xe2189c2f, SUC_lexicon_tags_11 },
    { 0xb2f89c30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa989c35, SUC_lexicon_tags_12 },
    { 0x2cd89c35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd989c3a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d989c3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd189c4b, SUC_lexicon_tags_19 },
    { 0x83389c4b, SUC_lexicon_tags_118 },
    { 0xd3f89c4d, SUC_lexicon_tags_79 },
    { 0x4a789c4d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcaf89c50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea989c5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3589c5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1389c62, SUC_lexicon_tags_10 },
    { 0x2eb89c63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2dd89c65, SUC_lexicon_tags_47 },
    { 0x9a189c66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d589c6c, SUC_lexicon_tags_38 },
    { 0x4e189c6d, SUC_lexicon_tags_15 },
    { 0x15389c6e, SUC_lexicon_tags_60 },
    { 0xc5b89c6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6189c77, SUC_lexicon_tags_25 },
    { 0xa1189c78, SUC_lexicon_tags_47 },
    { 0xc5989c78, SUC_lexicon_tags_0 },
    { 0xabf89c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad89c7e, SUC_lexicon_tags_23 },
    { 0xeb389c7e, SUC_lexicon_tags_38 },
    { 0x99589c80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed89c87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb89c8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x63189c8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d89c92, SUC_lexicon_tags_14 },
    { 0x69989c93, SUC_lexicon_tags_11 },
    { 0xa4f89c94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8589c9b, SUC_lexicon_tags_31 },
    { 0x63989c9c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad89cac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25189caf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3189cb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3589cb3, SUC_lexicon_tags_0 },
    { 0xd5b89cb3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb89cb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6989cbb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59389cc0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5c389cc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13d89cc4, SUC_lexicon_tags_25 },
    { 0xa2189cc5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef89ccd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0589cd0, SUC_lexicon_tags_11 },
    { 0x99589cd1, SUC_lexicon_tags_25 },
    { 0xc5989cd2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x74989cd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbff89cd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0389cd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f89ce2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b589ce8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x49789cea, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x6b789cec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d89cf1, SUC_lexicon_tags_48 },
    { 0x7af89cf2, SUC_lexicon_tags_15 },
    { 0x37f89cf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24589cfe, SUC_lexicon_tags_11 },
    { 0xfbd89cfe, SUC_lexicon_tags_38 },
    { 0x55189d00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbf89d02, SUC_lexicon_tags_29 },
    { 0xd1b89d03, SUC_lexicon_tags_37 },
    { 0x7d989d04, SUC_lexicon_tags_60 },
    { 0x9c389d05, SUC_lexicon_tags_31 },
    { 0x6a589d06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd189d08, SUC_lexicon_tags_45 },
    { 0x54f89d09, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d589d10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93389d1a, SUC_lexicon_tags_60 },
    { 0xe5589d1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d89d1f, SUC_lexicon_tags_2 },
    { 0xb1589d20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8789d27, SUC_lexicon_tags_17 },
    { 0xd8f89d28, SUC_lexicon_tags_17 },
    { 0x5dd89d28, SUC_lexicon_tags_2 },
    { 0x47189d2a, SUC_lexicon_tags_14 },
    { 0x31f89d2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed89d30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe189d32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb89d38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab89d3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb989d3f, SUC_lexicon_tags_6 },
    { 0xb9d89d40, SUC_lexicon_tags_7 },
    { 0x2ad89d41, SUC_lexicon_tags_28 },
    { 0x74f89d42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e189d45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f89d48, SUC_lexicon_tags_38 },
    { 0x50989d49, SUC_lexicon_tags_28 },
    { 0xb4789d49, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8f789d4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23789d58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5589d5b, SUC_lexicon_tags_25 },
    { 0x1ed89d5c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x48389d5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50389d63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40589d6c, SUC_lexicon_tags_79 },
    { 0x3b89d6d, SUC_lexicon_tags_25 },
    { 0xf6389d6d, SUC_lexicon_tags_45 },
    { 0x68f89d6e, SUC_lexicon_tags_26 },
    { 0xedd89d70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69789d74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf589d78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f789d7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa389d80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f89d82, SUC_lexicon_tags_2 },
    { 0xe7f89d82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f189d88, SUC_lexicon_tags_37 },
    { 0xa2b89d88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ab89d8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5389d91, SUC_lexicon_tags_31 },
    { 0x67389d92, SUC_lexicon_tags_49 },
    { 0xd6d89d93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d89d97, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f89d9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b89da1, SUC_lexicon_tags_12 },
    { 0xf5d89da1, SUC_lexicon_tags_8 },
    { 0x81589da3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb89da7, SUC_lexicon_tags_37 },
    { 0x94989da8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x10989daa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8cf89dac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f389dae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb389db1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8df89db3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88189db6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f189dbb, SUC_lexicon_tags_63 },
    { 0x49389dbc, SUC_lexicon_tags_45 },
    { 0x80b89dbd, SUC_lexicon_tags_6 },
    { 0x58d89dbb, SUC_lexicon_tags_27 },
    { 0xdb589dbd, SUC_lexicon_tags_39 },
    { 0x85989dbd, SUC_lexicon_tags_13 },
    { 0xf7589dc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f89dc6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4c989dc8, SUC_lexicon_tags_2 },
    { 0xd1589dc9, SUC_lexicon_tags_32 },
    { 0x15989dc8, SUC_lexicon_tags_40 },
    { 0x67789dca, SUC_lexicon_tags_133 },
    { 0x9f989dc9, SUC_lexicon_tags_23 },
    { 0x4c189dcb, SUC_lexicon_tags_15 },
    { 0xf3989dc8, SUC_lexicon_tags_10 },
    { 0x96589dc9, SUC_lexicon_tags_63 },
    { 0x19d89dd0, SUC_lexicon_tags_39 },
    { 0xfa789dd0, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65789dd5, SUC_lexicon_tags_7 },
    { 0x88989dd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0789dda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d389ddd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d189de4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe8d89de6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54589de8, SUC_lexicon_tags_58 },
    { 0xd2d89de8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71189df1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8589df3, SUC_lexicon_tags_33 },
    { 0x45f89df4, SUC_lexicon_tags_37 },
    { 0xed389df3, SUC_lexicon_tags_31 },
    { 0x7f89df6, SUC_lexicon_tags_35 },
    { 0x9a589df7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6589dfa, SUC_lexicon_tags_0 },
    { 0x88d89dfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f89dfe, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb8389e00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd589e05, SUC_lexicon_tags_25 },
    { 0x1b989e05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f89e0b, SUC_lexicon_tags_2 },
    { 0x47389e0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21189e13, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd89e17, SUC_lexicon_tags_2 },
    { 0xd1f89e18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39989e1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d89e20, SUC_lexicon_tags_26 },
    { 0xfc789e21, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b89e24, SUC_lexicon_tags_37 },
    { 0x89789e25, SUC_lexicon_tags_17 },
    { 0xacd89e26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1589e28, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b89e2d, SUC_lexicon_tags_11 },
    { 0xe5f89e2e, SUC_lexicon_tags_25 },
    { 0xb9b89e2f, SUC_lexicon_tags_46 },
    { 0xa2f89e30, SUC_lexicon_tags_38 },
    { 0x26d89e31, SUC_lexicon_tags_11 },
    { 0xbbd89e2f, SUC_lexicon_tags_8 },
    { 0xe9b89e33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d89e36, SUC_lexicon_tags_40 },
    { 0x9b989e36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed89e39, SUC_lexicon_tags_80 },
    { 0xc3f89e39, SUC_lexicon_tags_17 },
    { 0x4bb89e3b, SUC_lexicon_tags_47 },
    { 0xf2989e3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c789e40, SUC_lexicon_tags_9 },
    { 0x6e189e40, SUC_lexicon_tags_37 },
    { 0x72389e41, SUC_lexicon_tags_25 },
    { 0xeb589e42, SUC_lexicon_tags_8 },
    { 0xb7189e44, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d389e46, SUC_lexicon_tags_43 },
    { 0x6f589e47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b89e4e, SUC_lexicon_tags_1 },
    { 0x55789e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94789e52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8789e54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d589e5b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62589e60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9789e66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0589e69, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e189e6c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3989e6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b389e79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c189e7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7589e81, SUC_lexicon_tags_7 },
    { 0x4e189e82, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf89e88, SUC_lexicon_tags_32 },
    { 0x3d189e88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9589e8f, SUC_lexicon_tags_17 },
    { 0x97d89e8f, SUC_lexicon_tags_11 },
    { 0x37f89e91, SUC_lexicon_tags_19 },
    { 0x13389e90, SUC_lexicon_tags_9 },
    { 0xcf389e8f, SUC_lexicon_tags_19 },
    { 0xc1189e94, SUC_lexicon_tags_30 },
    { 0x3f589e8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffb89e97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68789e9b, SUC_lexicon_tags_7 },
    { 0xe789e9b, SUC_lexicon_tags_8 },
    { 0xc9f89e9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21589ea0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2b989ea2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1789ea5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c989ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeef89eaa, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f789eb0, SUC_lexicon_tags_21 },
    { 0x61189eb0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b89eb7, SUC_lexicon_tags_60 },
    { 0xf1d89eb8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7589ebb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2ed89ebd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd7989ebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30d89ec1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59789ec3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e189ec6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89989ec8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8389eca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9989ed1, SUC_lexicon_tags_25 },
    { 0x78389ed1, SUC_lexicon_tags_37 },
    { 0x11789ed3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a789ed9, SUC_lexicon_tags_47 },
    { 0xfe789ed9, SUC_lexicon_tags_0 },
    { 0x17989edb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfef89edd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf89ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b89ee4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c589ee6, SUC_lexicon_tags_11 },
    { 0xd9389ee6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f89eed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18989eef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d89ef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8389efa, SUC_lexicon_tags_15 },
    { 0xbeb89efa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35789f02, SUC_lexicon_tags_37 },
    { 0x78b89f02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe6989f05, SUC_lexicon_tags_11 },
    { 0x2f89f06, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa0989f08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1989f11, SUC_lexicon_tags_37 },
    { 0x3d189f12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f89f1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8789f1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x29589f20, SUC_lexicon_tags_26 },
    { 0xbd89f20, SUC_lexicon_tags_48 },
    { 0xa1989f20, SUC_lexicon_tags_15 },
    { 0x45389f23, SUC_lexicon_tags_1 },
    { 0x7c989f21, SUC_lexicon_tags_9 },
    { 0x23789f25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e589f28, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef189f2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf89f35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10589f3b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f89f42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef89f48, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xf389f4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8df89f4c, SUC_lexicon_tags_7 },
    { 0x55b89f4c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f789f50, SUC_lexicon_tags_17 },
    { 0xd4f89f51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f589f55, SUC_lexicon_tags_15 },
    { 0xf9189f55, SUC_lexicon_tags_64 },
    { 0x5589f57, SUC_lexicon_tags_1 },
    { 0x25d89f57, SUC_lexicon_tags_17 },
    { 0x76b89f59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde789f5d, SUC_lexicon_tags_62 },
    { 0xca989f5e, SUC_lexicon_tags_12 },
    { 0xdeb89f5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb89f62, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3989f65, SUC_lexicon_tags_46 },
    { 0xa6189f66, SUC_lexicon_tags_14 },
    { 0x4e389f67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7589f6c, SUC_lexicon_tags_2 },
    { 0xd0189f6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55589f6f, SUC_lexicon_tags_10 },
    { 0xfcb89f6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1389f73, SUC_lexicon_tags_12 },
    { 0xbf589f74, SUC_lexicon_tags_27 },
    { 0xfd389f75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7589f7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b589f7d, SUC_lexicon_tags_23 },
    { 0x6d989f7d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f389f87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d89f8a, SUC_lexicon_tags_3 },
    { 0x7c989f8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad89f91, SUC_lexicon_tags_1 },
    { 0x4c389f91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77589f96, SUC_lexicon_tags_25 },
    { 0xb9b89f97, SUC_lexicon_tags_11 },
    { 0xf589f98, SUC_lexicon_tags_14 },
    { 0xe2f89f98, SUC_lexicon_tags_15 },
    { 0x90989f9a, SUC_lexicon_tags_12 },
    { 0xc5b89f9a, SUC_lexicon_tags_10 },
    { 0xc0b89f9a, SUC_lexicon_tags_15 },
    { 0xe3b89f9d, SUC_lexicon_tags_45 },
    { 0x88b89f98, SUC_lexicon_tags_65 },
    { 0xe2589f9f, SUC_lexicon_tags_1 },
    { 0x1c789fa0, SUC_lexicon_tags_1 },
    { 0x48d89f9f, SUC_lexicon_tags_2 },
    { 0x24189fa2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd189fa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x38789fa9, SUC_lexicon_tags_14 },
    { 0xacf89fa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d89fb5, SUC_lexicon_tags_6 },
    { 0xa9d89fb6, SUC_lexicon_tags_47 },
    { 0x84389fb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73789fb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xed89fbc, SUC_lexicon_tags_65 },
    { 0x2c789fbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c189fc3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61b89fc5, SUC_lexicon_tags_54 },
    { 0xa3589fc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f89fc9, SUC_lexicon_tags_42 },
    { 0x9a189fc9, SUC_lexicon_tags_30 },
    { 0xe5b89fcb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9789fcd, SUC_lexicon_tags_206 },
    { 0x23189fce, SUC_lexicon_tags_49 },
    { 0xae189fce, SUC_lexicon_tags_37 },
    { 0xd4f89fd0, SUC_lexicon_tags_11 },
    { 0x3b89fd0, SUC_lexicon_tags_71 },
    { 0x22b89fd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f89fd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x67989fd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9989fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a389fe2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xca389fe5, SUC_lexicon_tags_2 },
    { 0x1cb89fe6, SUC_lexicon_tags_8 },
    { 0xfdd89fe7, SUC_lexicon_tags_1 },
    { 0x48b89fe7, SUC_lexicon_tags_46 },
    { 0x1bf89fe9, SUC_lexicon_tags_37 },
    { 0x75b89fea, SUC_lexicon_tags_11 },
    { 0x8589fea, SUC_lexicon_tags_25 },
    { 0x9a989feb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4389fee, SUC_lexicon_tags_21 },
    { 0xf2189fef, SUC_lexicon_tags_524 },
    { 0x59189ff0, SUC_lexicon_tags_11 },
    { 0xf989ff1, SUC_lexicon_tags_23 },
    { 0x30b89fee, SUC_lexicon_tags_35 },
    { 0x43d89ff0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d89ff6, SUC_lexicon_tags_45 },
    { 0x65989ff7, SUC_lexicon_tags_15 },
    { 0xf3389ff7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35789ffb, SUC_lexicon_tags_146 },
    { 0x32989ffb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe178a001, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d8a004, SUC_lexicon_tags_75 },
    { 0xacd8a004, SUC_lexicon_tags_7 },
    { 0x7538a006, SUC_lexicon_tags_59 },
    { 0x8078a007, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f8a00c, SUC_lexicon_tags_39 },
    { 0x4e38a00c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f8a014, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f8a018, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf58a01a, SUC_lexicon_tags_28 },
    { 0x94d8a01a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab8a020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef98a022, SUC_lexicon_tags_27 },
    { 0x3ab8a022, SUC_lexicon_tags_66 },
    { 0x9478a024, SUC_lexicon_tags_12 },
    { 0xc418a025, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4418a029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8a02d, SUC_lexicon_tags_38 },
    { 0xedf8a02e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9d8a030, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7f58a032, SUC_lexicon_tags_26 },
    { 0x4538a032, SUC_lexicon_tags_37 },
    { 0xd418a032, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b98a036, SUC_lexicon_tags_25 },
    { 0x8218a036, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcdb8a039, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f8a03c, SUC_lexicon_tags_46 },
    { 0xd0f8a03c, SUC_lexicon_tags_12 },
    { 0xbf18a03e, SUC_lexicon_tags_2 },
    { 0x80b8a03f, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x6db8a041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1278a043, SUC_lexicon_tags_17 },
    { 0xa998a044, SUC_lexicon_tags_26 },
    { 0x2c98a045, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc298a04b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff8a04e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9678a051, SUC_lexicon_tags_33 },
    { 0xda78a051, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4178a056, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d8a059, SUC_lexicon_tags_1 },
    { 0xe4d8a05a, SUC_lexicon_tags_2 },
    { 0x18d8a05b, SUC_lexicon_tags_21 },
    { 0xfa38a05c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb38a064, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6df8a066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa498a075, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b8a079, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd698a07e, SUC_lexicon_tags_29 },
    { 0xc958a07f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4c98a081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6178a084, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d78a087, SUC_lexicon_tags_17 },
    { 0x8f98a088, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed8a08d, SUC_lexicon_tags_12 },
    { 0x6258a08e, SUC_lexicon_tags_17 },
    { 0xe778a08f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf38a091, SUC_lexicon_tags_7 },
    { 0xc0d8a092, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6778a094, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x72f8a097, SUC_lexicon_tags_2 },
    { 0xe9f8a098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe318a09a, SUC_lexicon_tags_12 },
    { 0x6a78a09b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c98a0a0, SUC_lexicon_tags_11 },
    { 0x90b8a0a0, SUC_lexicon_tags_25 },
    { 0x38f8a0a2, SUC_lexicon_tags_2 },
    { 0x2cf8a0a3, SUC_lexicon_tags_17 },
    { 0x9918a0a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf8a0ad, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xee38a0af, SUC_lexicon_tags_21 },
    { 0xcb38a0b0, SUC_lexicon_tags_38 },
    { 0xe438a0b1, SUC_lexicon_tags_60 },
    { 0x9cd8a0b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e78a0b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9b38a0b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b38a0b8, SUC_lexicon_tags_7 },
    { 0xc458a0b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb8a0be, SUC_lexicon_tags_17 },
    { 0x9c58a0bf, SUC_lexicon_tags_39 },
    { 0xdbb8a0bf, SUC_lexicon_tags_17 },
    { 0x3d38a0c1, SUC_lexicon_tags_48 },
    { 0x7cd8a0c2, SUC_lexicon_tags_43 },
    { 0xc7d8a0c3, SUC_lexicon_tags_11 },
    { 0x2b18a0c1, SUC_lexicon_tags_28 },
    { 0x63f8a0c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458a0cc, SUC_lexicon_tags_28 },
    { 0xdd38a0cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d8a0d5, SUC_lexicon_tags_80 },
    { 0x4dd8a0d6, SUC_lexicon_tags_31 },
    { 0x7618a0d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd58a0e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3eb8a0e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1898a0e5, SUC_lexicon_tags_10 },
    { 0x5d18a0e6, SUC_lexicon_tags_17 },
    { 0x6f58a0e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1518a0ed, SUC_lexicon_tags_1 },
    { 0xdd38a0ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf598a0f3, SUC_lexicon_tags_12 },
    { 0xfdd8a0f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b8a0fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad38a100, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcf78a102, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7778a10c, SUC_lexicon_tags_39 },
    { 0x18d8a10c, SUC_lexicon_tags_53 },
    { 0x15b8a10d, SUC_lexicon_tags_60 },
    { 0x27b8a10f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd98a113, SUC_lexicon_tags_37 },
    { 0x8c38a114, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd378a11e, SUC_lexicon_tags_172 },
    { 0x1a58a11f, SUC_lexicon_tags_37 },
    { 0xc078a11e, SUC_lexicon_tags_1 },
    { 0xfc18a121, SUC_lexicon_tags_33 },
    { 0x67f8a11e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x6818a125, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4bd8a127, SUC_lexicon_tags_37 },
    { 0x6cb8a128, SUC_lexicon_tags_11 },
    { 0x8658a129, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f8a12f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d8a132, SUC_lexicon_tags_25 },
    { 0x2138a132, SUC_lexicon_tags_1048 },
    { 0x4af8a134, SUC_lexicon_tags_17 },
    { 0x21f8a134, SUC_lexicon_tags_12 },
    { 0x30b8a136, SUC_lexicon_tags_17 },
    { 0xf58a132, SUC_lexicon_tags_5 },
    { 0x7e78a138, SUC_lexicon_tags_15 },
    { 0x90d8a139, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1238a13c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d8a142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc78a145, SUC_lexicon_tags_92 },
    { 0x4d38a146, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x65b8a148, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7618a14b, SUC_lexicon_tags_2 },
    { 0x1858a14c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4198a14e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb38a153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5d8a155, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5818a159, SUC_lexicon_tags_0 },
    { 0x8df8a15a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6558a15e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f38a163, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9358a165, SUC_lexicon_tags_60 },
    { 0x6af8a166, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9038a16a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c38a171, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb8a174, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b8a177, SUC_lexicon_tags_11 },
    { 0x2b18a177, SUC_lexicon_tags_49 },
    { 0x7f78a177, SUC_lexicon_tags_49 },
    { 0xc678a178, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b18a17c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5398a180, SUC_lexicon_tags_8 },
    { 0xa478a181, SUC_lexicon_tags_28 },
    { 0xf8f8a182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9b8a184, SUC_lexicon_tags_7 },
    { 0x89f8a185, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe58a187, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4758a18a, SUC_lexicon_tags_15 },
    { 0xe418a18b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xebd8a18d, SUC_lexicon_tags_25 },
    { 0xf398a18e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe758a194, SUC_lexicon_tags_17 },
    { 0x4d98a194, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf18a198, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f8a19b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xcf8a19d, SUC_lexicon_tags_6 },
    { 0x45f8a19e, SUC_lexicon_tags_54 },
    { 0xe18a19f, SUC_lexicon_tags_12 },
    { 0x4e18a19f, SUC_lexicon_tags_1 },
    { 0xb3d8a1a1, SUC_lexicon_tags_1 },
    { 0xb718a19f, SUC_lexicon_tags_2 },
    { 0x81d8a1a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2818a1aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1d8a1ac, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xa9b8a1ae, SUC_lexicon_tags_11 },
    { 0xe838a1af, SUC_lexicon_tags_63 },
    { 0x9798a1b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4978a1b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2318a1c4, SUC_lexicon_tags_44 },
    { 0xeaf8a1c5, SUC_lexicon_tags_25 },
    { 0xad38a1c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e78a1c8, SUC_lexicon_tags_37 },
    { 0x36b8a1c9, SUC_lexicon_tags_198 },
    { 0x1318a1ca, SUC_lexicon_tags_4 },
    { 0xa3d8a1c9, SUC_lexicon_tags_37 },
    { 0x6178a1cc, SUC_lexicon_tags_12 },
    { 0xc0b8a1c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3ad8a1cf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae98a1d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d8a1d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x81f8a1d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xac58a1db, SUC_lexicon_tags_37 },
    { 0xcb98a1dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5598a1df, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x5758a1e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8598a1e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1618a1e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4538a1eb, SUC_lexicon_tags_19 },
    { 0x6c18a1ec, SUC_lexicon_tags_38 },
    { 0x4c18a1ed, SUC_lexicon_tags_8 },
    { 0xb4b8a1ed, SUC_lexicon_tags_40 },
    { 0x1d18a1ef, SUC_lexicon_tags_27 },
    { 0x1f18a1f0, SUC_lexicon_tags_11 },
    { 0x7e58a1ef, SUC_lexicon_tags_38 },
    { 0xe778a1f1, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b8a1f5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2358a1fa, SUC_lexicon_tags_4 },
    { 0x3258a1fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc998a1fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb218a204, SUC_lexicon_tags_8 },
    { 0x5758a205, SUC_lexicon_tags_11 },
    { 0x2db8a206, SUC_lexicon_tags_60 },
    { 0x1678a204, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3df8a209, SUC_lexicon_tags_1 },
    { 0x2f98a209, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b8a217, SUC_lexicon_tags_15 },
    { 0x4338a218, SUC_lexicon_tags_37 },
    { 0xbb38a219, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d58a221, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f98a227, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x25d8a229, SUC_lexicon_tags_1 },
    { 0x3e18a22a, SUC_lexicon_tags_1 },
    { 0x56d8a22b, SUC_lexicon_tags_17 },
    { 0xe0f8a22b, SUC_lexicon_tags_4 },
    { 0x4cd8a22a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5798a22f, SUC_lexicon_tags_2 },
    { 0x9998a230, SUC_lexicon_tags_33 },
    { 0x40b8a231, SUC_lexicon_tags_17 },
    { 0xd398a232, SUC_lexicon_tags_16 },
    { 0xb878a233, SUC_lexicon_tags_25 },
    { 0x1c38a232, SUC_lexicon_tags_16 },
    { 0x9c58a235, SUC_lexicon_tags_37 },
    { 0x87d8a232, SUC_lexicon_tags_12 },
    { 0x20b8a231, SUC_lexicon_tags_37 },
    { 0xa8f8a238, SUC_lexicon_tags_11 },
    { 0xdc78a233, SUC_lexicon_tags_16 },
    { 0x9958a236, SUC_lexicon_tags_12 },
    { 0x8d78a239, SUC_lexicon_tags_14 },
    { 0xda38a23c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf78a242, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd8a247, SUC_lexicon_tags_9 },
    { 0xd8a247, SUC_lexicon_tags_37 },
    { 0x7178a249, SUC_lexicon_tags_17 },
    { 0x6a98a24a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd978a24e, SUC_lexicon_tags_17 },
    { 0x7978a24f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x26f8a251, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7078a253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a78a255, SUC_lexicon_tags_7 },
    { 0xac98a256, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe78a258, SUC_lexicon_tags_33 },
    { 0x2698a259, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f8a260, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9098a266, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb638a269, SUC_lexicon_tags_25 },
    { 0x6978a26a, SUC_lexicon_tags_17 },
    { 0xd338a26a, SUC_lexicon_tags_23 },
    { 0x6498a26c, SUC_lexicon_tags_30 },
    { 0x7e58a26c, SUC_lexicon_tags_15 },
    { 0x54f8a26e, SUC_lexicon_tags_7 },
    { 0xc298a26c, SUC_lexicon_tags_17 },
    { 0x7c98a26d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82b8a272, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb58a275, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d8a27b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd4b8a27d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f8a285, SUC_lexicon_tags_10 },
    { 0xbad8a286, SUC_lexicon_tags_2 },
    { 0x52f8a285, SUC_lexicon_tags_116 },
    { 0xd18a288, SUC_lexicon_tags_5 },
    { 0xdb78a289, SUC_lexicon_tags_45 },
    { 0xc4d8a285, SUC_lexicon_tags_1 },
    { 0x5d18a28b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8a294, SUC_lexicon_tags_23 },
    { 0x1a18a295, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf8a299, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3d8a29b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df8a29e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9178a2a0, SUC_lexicon_tags_49 },
    { 0xfd38a2a1, SUC_lexicon_tags_29 },
    { 0xdb78a2a2, SUC_lexicon_tags_11 },
    { 0xb818a2a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d8a2a7, SUC_lexicon_tags_57 },
    { 0xd978a2a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb8a2ab, SUC_lexicon_tags_39 },
    { 0x2e78a2ac, SUC_lexicon_tags_27 },
    { 0xd6f8a2ac, SUC_lexicon_tags_82 },
    { 0x8a78a2ad, SUC_lexicon_tags_87 },
    { 0x2a78a2af, SUC_lexicon_tags_2 },
    { 0x8278a2ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d8a2b4, SUC_lexicon_tags_31 },
    { 0x5618a2b5, SUC_lexicon_tags_7 },
    { 0x9438a2b6, SUC_lexicon_tags_15 },
    { 0xc658a2b7, SUC_lexicon_tags_17 },
    { 0x64b8a2b8, SUC_lexicon_tags_14 },
    { 0xa438a2b7, SUC_lexicon_tags_10 },
    { 0xdb58a2b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f58a2bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8378a2bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc358a2c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2258a2c9, SUC_lexicon_tags_11 },
    { 0x9898a2ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d8a2d1, SUC_lexicon_tags_6 },
    { 0x46b8a2d2, SUC_lexicon_tags_104 },
    { 0xd3f8a2d3, SUC_lexicon_tags_98 },
    { 0xb1d8a2d2, SUC_lexicon_tags_19 },
    { 0x758a2d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4338a2da, SUC_lexicon_tags_25 },
    { 0x27d8a2db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21d8a2dd, SUC_lexicon_tags_49 },
    { 0xa298a2de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e38a2e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa198a2e6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7198a2ee, SUC_lexicon_tags_0 },
    { 0xa2b8a2ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcb18a2f1, SUC_lexicon_tags_25 },
    { 0xc4b8a2f1, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b8a2f5, SUC_lexicon_tags_26 },
    { 0x9b18a2f5, SUC_lexicon_tags_14 },
    { 0xc078a2f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf638a2fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf8a2fe, SUC_lexicon_tags_11 },
    { 0xf958a2ff, SUC_lexicon_tags_37 },
    { 0x158a2ff, SUC_lexicon_tags_25 },
    { 0x1158a301, SUC_lexicon_tags_43 },
    { 0x7b38a301, SUC_lexicon_tags_65 },
    { 0x9f18a302, SUC_lexicon_tags_19 },
    { 0x4478a302, SUC_lexicon_tags_21 },
    { 0x34f8a305, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88d8a307, SUC_lexicon_tags_14 },
    { 0xd938a307, SUC_lexicon_tags_14 },
    { 0xeb18a309, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5058a30c, SUC_lexicon_tags_37 },
    { 0xf0f8a30c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd298a30f, SUC_lexicon_tags_38 },
    { 0x8a38a310, SUC_lexicon_tags_30 },
    { 0xabb8a311, SUC_lexicon_tags_8 },
    { 0x36b8a312, SUC_lexicon_tags_17 },
    { 0x2058a313, SUC_lexicon_tags_47 },
    { 0x858a30f, SUC_lexicon_tags_8 },
    { 0x65b8a315, SUC_lexicon_tags_81 },
    { 0x9c98a316, SUC_lexicon_tags_11 },
    { 0xfab8a317, SUC_lexicon_tags_28 },
    { 0xb8f8a313, SUC_lexicon_tags_25 },
    { 0x45d8a319, SUC_lexicon_tags_27 },
    { 0x3ef8a311, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f98a31c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe078a31e, SUC_lexicon_tags_45 },
    { 0xe838a31e, SUC_lexicon_tags_1 },
    { 0x1118a320, SUC_lexicon_tags_1 },
    { 0xeab8a321, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f8a328, SUC_lexicon_tags_15 },
    { 0x4ef8a329, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1158a32d, SUC_lexicon_tags_23 },
    { 0x32f8a32d, SUC_lexicon_tags_11 },
    { 0x3238a32f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f8a336, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3f8a338, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6cf8a33a, SUC_lexicon_tags_37 },
    { 0x49d8a33b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb8a341, SUC_lexicon_tags_57 },
    { 0x4a58a342, SUC_lexicon_tags_37 },
    { 0x4f78a343, SUC_lexicon_tags_1 },
    { 0xd218a343, SUC_lexicon_tags_93 },
    { 0xc378a344, SUC_lexicon_tags_12 },
    { 0x28b8a344, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a58a34a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf698a34d, SUC_lexicon_tags_45 },
    { 0xcc18a34e, SUC_lexicon_tags_38 },
    { 0x83d8a34f, SUC_lexicon_tags_68 },
    { 0x78a350, SUC_lexicon_tags_17 },
    { 0x1cf8a350, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f78a353, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d58a358, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3198a35f, SUC_lexicon_tags_10 },
    { 0x9c98a360, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb6d8a362, SUC_lexicon_tags_25 },
    { 0x7018a363, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f8a366, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f8a369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf958a36d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd918a372, SUC_lexicon_tags_31 },
    { 0x54f8a373, SUC_lexicon_tags_27 },
    { 0xb8a373, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe18a378, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x3318a37b, SUC_lexicon_tags_37 },
    { 0x2d58a37c, SUC_lexicon_tags_41 },
    { 0x8898a37d, SUC_lexicon_tags_17 },
    { 0x2f98a37b, SUC_lexicon_tags_38 },
    { 0xd258a37f, SUC_lexicon_tags_21 },
    { 0xbf58a37f, SUC_lexicon_tags_17 },
    { 0x9498a37e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd038a387, SUC_lexicon_tags_10 },
    { 0x89d8a388, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d98a38c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8018a38e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f8a398, SUC_lexicon_tags_14 },
    { 0xdb98a398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9198a39b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc078a39d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7338a3a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bf8a3a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xabd8a3a8, SUC_lexicon_tags_25 },
    { 0x52d8a3a8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8d38a3ab, SUC_lexicon_tags_49 },
    { 0xb758a3ac, SUC_lexicon_tags_37 },
    { 0x9e78a3ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb8a3b0, SUC_lexicon_tags_37 },
    { 0x4cf8a3b0, SUC_lexicon_tags_30 },
    { 0x9dd8a3b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd8a3bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f58a3be, SUC_lexicon_tags_7 },
    { 0xb9f8a3be, SUC_lexicon_tags_1 },
    { 0x6b98a3c0, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f8a3c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e38a3c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8318a3cd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x7538a3cf, SUC_lexicon_tags_37 },
    { 0x46b8a3d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf8a3d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb198a3d7, SUC_lexicon_tags_12 },
    { 0x50b8a3d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd58a3dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f38a3de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f8a3e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86d8a3e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad8a3ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94f8a3ec, SUC_lexicon_tags_23 },
    { 0x5198a3ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5638a3ef, SUC_lexicon_tags_7 },
    { 0x3858a3f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a18a3f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd8a3f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb38a3fd, SUC_lexicon_tags_17 },
    { 0xec98a3fe, SUC_lexicon_tags_7 },
    { 0xc3f8a3ff, SUC_lexicon_tags_11 },
    { 0x5c98a400, SUC_lexicon_tags_2 },
    { 0x9998a3fe, SUC_lexicon_tags_49 },
    { 0xd318a402, SUC_lexicon_tags_14 },
    { 0x1398a403, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4278a406, SUC_lexicon_tags_11 },
    { 0x2998a407, SUC_lexicon_tags_84 },
    { 0x42b8a406, SUC_lexicon_tags_0 },
    { 0x30f8a409, SUC_lexicon_tags_16 },
    { 0x5cb8a40a, SUC_lexicon_tags_17 },
    { 0x4318a40b, SUC_lexicon_tags_15 },
    { 0x5858a40c, SUC_lexicon_tags_30 },
    { 0x95b8a40b, SUC_lexicon_tags_15 },
    { 0x70d8a40b, SUC_lexicon_tags_37 },
    { 0x1238a40b, SUC_lexicon_tags_12 },
    { 0x1fd8a409, SUC_lexicon_tags_34 },
    { 0x2db8a40a, SUC_lexicon_tags_9 },
    { 0x1eb8a412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb38a416, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3318a423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaad8a425, SUC_lexicon_tags_28 },
    { 0xce58a426, SUC_lexicon_tags_11 },
    { 0x8838a426, SUC_lexicon_tags_47 },
    { 0xb58a428, SUC_lexicon_tags_55 },
    { 0xf078a428, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff8a42e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df8a433, SUC_lexicon_tags_7 },
    { 0xd958a434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f8a438, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5158a43a, SUC_lexicon_tags_15 },
    { 0x7738a43b, SUC_lexicon_tags_7 },
    { 0xe58a43a, SUC_lexicon_tags_8 },
    { 0xfa58a43d, SUC_lexicon_tags_37 },
    { 0xa3d8a43e, SUC_lexicon_tags_1 },
    { 0x4718a43d, SUC_lexicon_tags_37 },
    { 0x69b8a440, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5698a444, SUC_lexicon_tags_7 },
    { 0x2a58a444, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1038a449, SUC_lexicon_tags_33 },
    { 0x5158a449, SUC_lexicon_tags_27 },
    { 0x5bd8a449, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4b8a44d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9978a451, SUC_lexicon_tags_11 },
    { 0x8e78a452, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8a455, SUC_lexicon_tags_32 },
    { 0x6698a455, SUC_lexicon_tags_31 },
    { 0x6cb8a456, SUC_lexicon_tags_92 },
    { 0x63b8a456, SUC_lexicon_tags_29 },
    { 0x98a459, SUC_lexicon_tags_30 },
    { 0x2358a459, SUC_lexicon_tags_16 },
    { 0x2318a456, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xad78a45d, SUC_lexicon_tags_17 },
    { 0xa018a45e, SUC_lexicon_tags_10 },
    { 0x6bd8a45f, SUC_lexicon_tags_27 },
    { 0x418a45d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd8a466, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x90d8a468, SUC_lexicon_tags_33 },
    { 0xb3d8a469, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3038a46c, SUC_lexicon_tags_20 },
    { 0x1238a46d, SUC_lexicon_tags_31 },
    { 0x2058a46d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x398a471, SUC_lexicon_tags_7 },
    { 0x6dd8a472, SUC_lexicon_tags_37 },
    { 0x92d8a473, SUC_lexicon_tags_1 },
    { 0x6198a474, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6098a476, SUC_lexicon_tags_15 },
    { 0x5a78a477, SUC_lexicon_tags_17 },
    { 0x9338a478, SUC_lexicon_tags_26 },
    { 0xd038a478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8a47d, SUC_lexicon_tags_2 },
    { 0x65f8a47d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe438a484, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a18a489, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8718a48c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd238a48e, SUC_lexicon_tags_29 },
    { 0xe7d8a48e, SUC_lexicon_tags_27 },
    { 0x6378a490, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x37d8a492, SUC_lexicon_tags_34 },
    { 0xedd8a493, SUC_lexicon_tags_39 },
    { 0x62f8a494, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea18a496, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6198a499, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf8a49e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf038a4a0, SUC_lexicon_tags_21 },
    { 0xe838a4a1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd58a4a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1198a4ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d8a4b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x81f8a4b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f38a4b8, SUC_lexicon_tags_7 },
    { 0xf718a4b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe658a4bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3018a4be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef8a4c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1018a4c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f8a4cf, SUC_lexicon_tags_30 },
    { 0x838a4d0, SUC_lexicon_tags_25 },
    { 0x658a4d1, SUC_lexicon_tags_11 },
    { 0x8178a4d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd698a4d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd98a4d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9558a4df, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0x1d98a4e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3878a4e5, SUC_lexicon_tags_12 },
    { 0x438a4e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb598a4e8, SUC_lexicon_tags_4 },
    { 0xdc18a4e9, SUC_lexicon_tags_37 },
    { 0x2138a4ea, SUC_lexicon_tags_65 },
    { 0xbb58a4eb, SUC_lexicon_tags_37 },
    { 0xfa78a4eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d58a4ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b98a4f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7178a4fd, SUC_lexicon_tags_18 },
    { 0x9278a4fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf198a500, SUC_lexicon_tags_11 },
    { 0x9578a501, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2858a504, SUC_lexicon_tags_38 },
    { 0xeeb8a505, SUC_lexicon_tags_14 },
    { 0xdfd8a506, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3a38a508, SUC_lexicon_tags_37 },
    { 0x7338a509, SUC_lexicon_tags_2 },
    { 0x13f8a50a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c18a50d, SUC_lexicon_tags_48 },
    { 0x2878a50e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb018a511, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xef58a513, SUC_lexicon_tags_12 },
    { 0xa38a513, SUC_lexicon_tags_80 },
    { 0x3d78a515, SUC_lexicon_tags_2 },
    { 0x12f8a515, SUC_lexicon_tags_18 },
    { 0xd3f8a517, SUC_lexicon_tags_17 },
    { 0x6cf8a515, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4db8a51a, SUC_lexicon_tags_11 },
    { 0x5218a51b, SUC_lexicon_tags_10 },
    { 0xe8b8a51c, SUC_lexicon_tags_14 },
    { 0x4d18a51d, SUC_lexicon_tags_17 },
    { 0xbc18a51e, SUC_lexicon_tags_1 },
    { 0x1df8a51f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x96f8a522, SUC_lexicon_tags_28 },
    { 0xa498a523, SUC_lexicon_tags_7 },
    { 0x42d8a522, SUC_lexicon_tags_15 },
    { 0x91d8a524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51b8a527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd8a52d, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7338a531, SUC_lexicon_tags_3 },
    { 0xd118a532, SUC_lexicon_tags_45 },
    { 0xf138a533, SUC_lexicon_tags_15 },
    { 0xf58a533, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8018a53e, SUC_lexicon_tags_1 },
    { 0xdc98a53e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe98a543, SUC_lexicon_tags_19 },
    { 0x3f18a544, SUC_lexicon_tags_15 },
    { 0x3d98a545, SUC_lexicon_tags_54 },
    { 0x9bd8a544, SUC_lexicon_tags_0 },
    { 0x7ed8a546, SUC_lexicon_tags_49 },
    { 0x3e58a545, SUC_lexicon_tags_23 },
    { 0xcd8a549, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe598a54b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8238a54e, SUC_lexicon_tags_0 },
    { 0x6f58a54f, SUC_lexicon_tags_0 },
    { 0xb658a54f, SUC_lexicon_tags_7 },
    { 0x6e98a551, SUC_lexicon_tags_1 },
    { 0x2cb8a54f, SUC_lexicon_tags_37 },
    { 0xa3d8a552, SUC_lexicon_tags_7 },
    { 0xb738a554, SUC_lexicon_tags_37 },
    { 0xaa18a554, SUC_lexicon_tags_17 },
    { 0xcab8a556, SUC_lexicon_tags_15 },
    { 0x5418a556, SUC_lexicon_tags_7 },
    { 0x5898a555, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b8a561, SUC_lexicon_tags_35 },
    { 0xb998a561, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf258a564, SUC_lexicon_tags_12 },
    { 0x7758a565, SUC_lexicon_tags_12 },
    { 0xa338a566, SUC_lexicon_tags_0 },
    { 0x878a566, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df8a56a, SUC_lexicon_tags_68 },
    { 0x4078a56b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3d8a56d, SUC_lexicon_tags_27 },
    { 0x1c38a56d, SUC_lexicon_tags_15 },
    { 0xef38a56f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf38a572, SUC_lexicon_tags_63 },
    { 0xe578a572, SUC_lexicon_tags_27 },
    { 0xb078a574, SUC_lexicon_tags_14 },
    { 0xac18a575, SUC_lexicon_tags_14 },
    { 0xa818a576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x79d8a578, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe58a57a, SUC_lexicon_tags_30 },
    { 0x5078a57b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb8a58c, SUC_lexicon_tags_33 },
    { 0xa258a58d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe218a58f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf378a599, SUC_lexicon_tags_2 },
    { 0x9bb8a59a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd18a59f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8a5a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f8a5ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2458a5b0, SUC_lexicon_tags_60 },
    { 0xb278a5b0, SUC_lexicon_tags_7 },
    { 0xbdf8a5b2, SUC_lexicon_tags_39 },
    { 0xe498a5b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf518a5b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1998a5b9, SUC_lexicon_tags_14 },
    { 0x8fb8a5ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d8a5bd, SUC_lexicon_tags_37 },
    { 0x4138a5be, SUC_lexicon_tags_68 },
    { 0x8498a5bf, SUC_lexicon_tags_11 },
    { 0x5d98a5c0, SUC_lexicon_tags_12 },
    { 0x1438a5bf, SUC_lexicon_tags_1 },
    { 0x8bf8a5c2, SUC_lexicon_tags_12 },
    { 0x9958a5c2, SUC_lexicon_tags_27 },
    { 0xd8b8a5c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a38a5c8, SUC_lexicon_tags_1 },
    { 0x778a5c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8418a5cb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbe38a5cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f8a5dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe238a5df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb258a5e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b8a5ea, SUC_lexicon_tags_23 },
    { 0x6438a5eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc598a5ee, SUC_lexicon_tags_2 },
    { 0x61f8a5ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa058a5f1, SUC_lexicon_tags_49 },
    { 0x5038a5f2, SUC_lexicon_tags_48 },
    { 0x6b8a5f1, SUC_lexicon_tags_45 },
    { 0x53d8a5f3, SUC_lexicon_tags_43 },
    { 0x2018a5f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd8a5ff, SUC_lexicon_tags_28 },
    { 0xa218a600, SUC_lexicon_tags_37 },
    { 0x1158a601, SUC_lexicon_tags_11 },
    { 0x338a602, SUC_lexicon_tags_47 },
    { 0x798a602, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db8a60a, SUC_lexicon_tags_28 },
    { 0x2c18a60b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d8a612, SUC_lexicon_tags_8 },
    { 0x6af8a613, SUC_lexicon_tags_27 },
    { 0x7db8a613, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xe178a617, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9778a61e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb38a622, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc18a626, SUC_lexicon_tags_37 },
    { 0x8218a627, SUC_lexicon_tags_15 },
    { 0x2618a628, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d8a62e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x19b8a630, SUC_lexicon_tags_27 },
    { 0xed58a631, SUC_lexicon_tags_37 },
    { 0x1378a632, SUC_lexicon_tags_77 },
    { 0x94b8a633, SUC_lexicon_tags_33 },
    { 0x638a631, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda98a639, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d8a63e, SUC_lexicon_tags_17 },
    { 0x2878a63f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa98a641, SUC_lexicon_tags_30 },
    { 0x44f8a642, SUC_lexicon_tags_17 },
    { 0x6b8a643, SUC_lexicon_tags_46 },
    { 0x1d18a641, SUC_lexicon_tags_30 },
    { 0x7018a645, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4578a647, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb78a64b, SUC_lexicon_tags_11 },
    { 0xe5b8a64b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb878a650, SUC_lexicon_tags_2 },
    { 0x9718a650, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x358a653, SUC_lexicon_tags_60 },
    { 0xdff8a653, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f8a659, SUC_lexicon_tags_11 },
    { 0xd678a659, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6eb8a65c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4538a665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d8a66c, SUC_lexicon_tags_60 },
    { 0xcbd8a66d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4138a66f, SUC_lexicon_tags_65 },
    { 0x7378a670, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8498a673, SUC_lexicon_tags_19 },
    { 0x28b8a674, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5478a676, SUC_lexicon_tags_12 },
    { 0xbdb8a677, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8518a679, SUC_lexicon_tags_25 },
    { 0xe3f8a679, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x30b8a67c, SUC_lexicon_tags_2 },
    { 0xc458a67d, SUC_lexicon_tags_7 },
    { 0xf4b8a67e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b8a682, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d8a687, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xda98a689, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a98a68b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf458a691, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8258a696, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d8a69a, SUC_lexicon_tags_65 },
    { 0xbc18a69b, SUC_lexicon_tags_28 },
    { 0xb7d8a69b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d8a6a2, SUC_lexicon_tags_5 },
    { 0x67f8a6a3, SUC_lexicon_tags_14 },
    { 0x6458a6a4, SUC_lexicon_tags_244 },
    { 0, NULL },
    { 0xa518a6a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb8a6ac, SUC_lexicon_tags_11 },
    { 0x21b8a6ad, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xa718a6af, SUC_lexicon_tags_27 },
    { 0xd718a6b0, SUC_lexicon_tags_45 },
    { 0x5a18a6b0, SUC_lexicon_tags_31 },
    { 0x46d8a6b0, SUC_lexicon_tags_38 },
    { 0xad78a6af, SUC_lexicon_tags_20 },
    { 0x3d58a6b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xef98a6b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd218a6b9, SUC_lexicon_tags_47 },
    { 0x2e58a6ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1878a6c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf198a6c4, SUC_lexicon_tags_38 },
    { 0xcc18a6c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd8a6c8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3af8a6ca, SUC_lexicon_tags_1 },
    { 0xbd18a6ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178a6cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7d8a6d1, SUC_lexicon_tags_1 },
    { 0x3f98a6d2, SUC_lexicon_tags_16 },
    { 0x1d8a6d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe198a6d6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d18a6dc, SUC_lexicon_tags_44 },
    { 0x6558a6dd, SUC_lexicon_tags_11 },
    { 0x8a98a6de, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc538a6e2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9518a6e4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbe78a6e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x11d8a6e8, SUC_lexicon_tags_17 },
    { 0xe598a6e8, SUC_lexicon_tags_40 },
    { 0x1758a6ea, SUC_lexicon_tags_6 },
    { 0x4c58a6e9, SUC_lexicon_tags_15 },
    { 0x9ff8a6e8, SUC_lexicon_tags_14 },
    { 0xbab8a6ed, SUC_lexicon_tags_2 },
    { 0xfb18a6eb, SUC_lexicon_tags_27 },
    { 0x6738a6ec, SUC_lexicon_tags_38 },
    { 0xb0f8a6ed, SUC_lexicon_tags_108 },
    { 0xec18a6f1, SUC_lexicon_tags_7 },
    { 0xf398a6f2, SUC_lexicon_tags_28 },
    { 0x9bb8a6f3, SUC_lexicon_tags_15 },
    { 0xf5d8a6f3, SUC_lexicon_tags_27 },
    { 0x1738a6f2, SUC_lexicon_tags_15 },
    { 0xee78a6ee, SUC_lexicon_tags_23 },
    { 0xf78a6f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf78a6f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b8a700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2378a703, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb958a70a, SUC_lexicon_tags_17 },
    { 0x1e58a70a, SUC_lexicon_tags_25 },
    { 0x88b8a70b, SUC_lexicon_tags_43 },
    { 0xdd78a70d, SUC_lexicon_tags_17 },
    { 0xe2d8a70d, SUC_lexicon_tags_15 },
    { 0x8598a70d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8138a716, SUC_lexicon_tags_46 },
    { 0xa2f8a716, SUC_lexicon_tags_1 },
    { 0xf6b8a717, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd8a71d, SUC_lexicon_tags_26 },
    { 0xaa78a71e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b8a722, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5478a72a, SUC_lexicon_tags_17 },
    { 0xed78a72b, SUC_lexicon_tags_11 },
    { 0xe878a72a, SUC_lexicon_tags_12 },
    { 0x798a72c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ad8a72f, SUC_lexicon_tags_1 },
    { 0xa838a72f, SUC_lexicon_tags_23 },
    { 0x4ff8a72f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf8a736, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a18a73a, SUC_lexicon_tags_11 },
    { 0x2498a73a, SUC_lexicon_tags_15 },
    { 0x7a78a73c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d8a748, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8158a757, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1d8a759, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xecb8a75c, SUC_lexicon_tags_0 },
    { 0x3d58a75d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d8a761, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b8a764, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f18a767, SUC_lexicon_tags_9 },
    { 0x8bb8a768, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8a76e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1218a770, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa298a776, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1cf8a778, SUC_lexicon_tags_31 },
    { 0x90d8a779, SUC_lexicon_tags_19 },
    { 0x3538a779, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b58a77f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d8a784, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6498a786, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4278a78a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8d8a78c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5218a78f, SUC_lexicon_tags_26 },
    { 0x9e58a790, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b8a795, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb238a797, SUC_lexicon_tags_25 },
    { 0x6e38a798, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbff8a79a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc218a79c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f58a7a6, SUC_lexicon_tags_8 },
    { 0x1e58a7a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe998a7aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f8a7ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf8a7b2, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x15b8a7b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff8a7bc, SUC_lexicon_tags_25 },
    { 0x7a78a7bd, SUC_lexicon_tags_15 },
    { 0xbd58a7bc, SUC_lexicon_tags_37 },
    { 0xd1b8a7bf, SUC_lexicon_tags_17 },
    { 0x9e18a7c0, SUC_lexicon_tags_7 },
    { 0x3f18a7be, SUC_lexicon_tags_37 },
    { 0x6918a7c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd858a7c6, SUC_lexicon_tags_87 },
    { 0x4858a7c7, SUC_lexicon_tags_12 },
    { 0xa378a7c8, SUC_lexicon_tags_32 },
    { 0xee58a7c6, SUC_lexicon_tags_109 },
    { 0x3838a7ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x47f8a7cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e38a7ce, SUC_lexicon_tags_2 },
    { 0xd838a7ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb0b8a7d1, SUC_lexicon_tags_48 },
    { 0x3038a7d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db8a7d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b8a7db, SUC_lexicon_tags_523 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc778a7df, SUC_lexicon_tags_29 },
    { 0x86b8a7e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf8a7e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c38a7e8, SUC_lexicon_tags_70 },
    { 0xc218a7e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7738a7ec, SUC_lexicon_tags_14 },
    { 0x8818a7ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6778a7f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3958a7f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3ff8a7fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178a7fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e98a800, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb78a802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa98a80e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a38a810, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a98a814, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc898a81a, SUC_lexicon_tags_1 },
    { 0x8c98a81b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdef8a81d, SUC_lexicon_tags_44 },
    { 0x2778a81e, SUC_lexicon_tags_23 },
    { 0x4638a81f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f8a822, SUC_lexicon_tags_60 },
    { 0x2bf8a823, SUC_lexicon_tags_30 },
    { 0x4a58a822, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb8a82a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3918a82c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd658a82e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf798a831, SUC_lexicon_tags_2 },
    { 0x9f18a832, SUC_lexicon_tags_87 },
    { 0xc278a833, SUC_lexicon_tags_47 },
    { 0x6d38a832, SUC_lexicon_tags_3 },
    { 0x5098a835, SUC_lexicon_tags_38 },
    { 0x7518a836, SUC_lexicon_tags_32 },
    { 0x2038a832, SUC_lexicon_tags_26 },
    { 0x89f8a832, SUC_lexicon_tags_14 },
    { 0xccb8a833, SUC_lexicon_tags_33 },
    { 0xf8d8a83a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938a841, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9878a843, SUC_lexicon_tags_30 },
    { 0x5ff8a844, SUC_lexicon_tags_12 },
    { 0x1038a845, SUC_lexicon_tags_16 },
    { 0x1db8a845, SUC_lexicon_tags_12 },
    { 0x3558a846, SUC_lexicon_tags_12 },
    { 0x7158a843, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb2d8a84a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff38a84d, SUC_lexicon_tags_30 },
    { 0xdbd8a84e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2398a850, SUC_lexicon_tags_43 },
    { 0xb278a851, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb8a85b, SUC_lexicon_tags_37 },
    { 0xd498a85c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf858a861, SUC_lexicon_tags_14 },
    { 0x6bb8a862, SUC_lexicon_tags_35 },
    { 0xb2d8a863, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc178a86c, SUC_lexicon_tags_10 },
    { 0x34d8a86d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f8a870, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d8a876, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad38a87f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8a884, SUC_lexicon_tags_47 },
    { 0x7df8a885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef98a887, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b8a88a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d18a88c, SUC_lexicon_tags_15 },
    { 0x7b78a88d, SUC_lexicon_tags_12 },
    { 0xd198a88e, SUC_lexicon_tags_33 },
    { 0x9f18a88f, SUC_lexicon_tags_32 },
    { 0x4ef8a88c, SUC_lexicon_tags_87 },
    { 0x14f8a88d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b8a898, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5818a89a, SUC_lexicon_tags_29 },
    { 0xbf98a89b, SUC_lexicon_tags_47 },
    { 0x9d38a89c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd8a8a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcdd8a8a8, SUC_lexicon_tags_37 },
    { 0x26f8a8a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2058a8ae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4dd8a8b0, SUC_lexicon_tags_8 },
    { 0xfb18a8b1, SUC_lexicon_tags_393 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6678a8b9, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d8a8bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd078a8bf, SUC_lexicon_tags_37 },
    { 0x4698a8bf, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc878a8c4, SUC_lexicon_tags_37 },
    { 0x458a8c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73b8a8c7, SUC_lexicon_tags_47 },
    { 0x6698a8c7, SUC_lexicon_tags_15 },
    { 0x4f38a8c9, SUC_lexicon_tags_49 },
    { 0x24f8a8ca, SUC_lexicon_tags_14 },
    { 0x9db8a8ca, SUC_lexicon_tags_31 },
    { 0xde38a8ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e78a8d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5298a8d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9478a8d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af8a8dc, SUC_lexicon_tags_12 },
    { 0xc38a8dc, SUC_lexicon_tags_68 },
    { 0x1a98a8de, SUC_lexicon_tags_43 },
    { 0x1038a8df, SUC_lexicon_tags_27 },
    { 0xadd8a8df, SUC_lexicon_tags_0 },
    { 0xc0f8a8dc, SUC_lexicon_tags_17 },
    { 0x218a8e2, SUC_lexicon_tags_25 },
    { 0x84b8a8dd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb8a8ed, SUC_lexicon_tags_63 },
    { 0x1c98a8ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8a8f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f18a8f6, SUC_lexicon_tags_43 },
    { 0x3138a8f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2058a8fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb8a8fd, SUC_lexicon_tags_31 },
    { 0xee18a8fe, SUC_lexicon_tags_6 },
    { 0x1078a8ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xce38a902, SUC_lexicon_tags_16 },
    { 0x66b8a903, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa238a905, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa038a907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x26d8a90a, SUC_lexicon_tags_26 },
    { 0xf258a90b, SUC_lexicon_tags_7 },
    { 0x7bb8a90b, SUC_lexicon_tags_68 },
    { 0x2ab8a90c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe58a90f, SUC_lexicon_tags_33 },
    { 0xa118a910, SUC_lexicon_tags_32 },
    { 0xc5b8a910, SUC_lexicon_tags_29 },
    { 0xc0d8a90f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd0d8a914, SUC_lexicon_tags_27 },
    { 0xe278a914, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa7b8a917, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9618a919, SUC_lexicon_tags_17 },
    { 0x998a919, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1bf8a91c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2138a920, SUC_lexicon_tags_48 },
    { 0x1e18a921, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4418a924, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x198a926, SUC_lexicon_tags_10 },
    { 0x9398a926, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc78a929, SUC_lexicon_tags_38 },
    { 0xc298a929, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb6b8a92c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1638a92f, SUC_lexicon_tags_11 },
    { 0x7738a930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecd8a933, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2558a939, SUC_lexicon_tags_25 },
    { 0x3038a93a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f18a93e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd398a943, SUC_lexicon_tags_95 },
    { 0x8f98a943, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8458a946, SUC_lexicon_tags_7 },
    { 0x3af8a946, SUC_lexicon_tags_54 },
    { 0xc798a948, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xac18a94a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7698a94f, SUC_lexicon_tags_8 },
    { 0x3af8a94f, SUC_lexicon_tags_37 },
    { 0xb2f8a950, SUC_lexicon_tags_34 },
    { 0x58f8a94f, SUC_lexicon_tags_6 },
    { 0x7638a951, SUC_lexicon_tags_10 },
    { 0xe938a954, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef8a958, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb518a95a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a78a95f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc038a961, SUC_lexicon_tags_25 },
    { 0xc38a962, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f8a965, SUC_lexicon_tags_12 },
    { 0xaa98a966, SUC_lexicon_tags_7 },
    { 0x2138a967, SUC_lexicon_tags_1 },
    { 0x9018a968, SUC_lexicon_tags_0 },
    { 0x89d8a969, SUC_lexicon_tags_2 },
    { 0x6b38a96a, SUC_lexicon_tags_82 },
    { 0xd838a96b, SUC_lexicon_tags_37 },
    { 0x5ed8a968, SUC_lexicon_tags_11 },
    { 0xac58a96d, SUC_lexicon_tags_220 },
    { 0x6a58a96e, SUC_lexicon_tags_7 },
    { 0x61d8a96f, SUC_lexicon_tags_17 },
    { 0x1b8a96f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8718a973, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4f8a975, SUC_lexicon_tags_8 },
    { 0xd178a975, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8438a978, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65b8a97a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa798a97e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b8a981, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb18a988, SUC_lexicon_tags_6 },
    { 0x1d8a989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9378a98b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f8a98e, SUC_lexicon_tags_27 },
    { 0x35d8a98e, SUC_lexicon_tags_49 },
    { 0xa3b8a98e, SUC_lexicon_tags_1 },
    { 0x1d38a991, SUC_lexicon_tags_843 },
    { 0xf878a991, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7598a995, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd8a99c, SUC_lexicon_tags_16 },
    { 0xd8f8a99d, SUC_lexicon_tags_31 },
    { 0xf418a99e, SUC_lexicon_tags_57 },
    { 0x4a98a99c, SUC_lexicon_tags_17 },
    { 0xf8f8a9a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b8a9a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a38a9a8, SUC_lexicon_tags_37 },
    { 0x66f8a9a8, SUC_lexicon_tags_17 },
    { 0xcd98a9a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f8a9ae, SUC_lexicon_tags_23 },
    { 0x9c78a9af, SUC_lexicon_tags_68 },
    { 0xc118a9ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x55b8a9b2, SUC_lexicon_tags_415 },
    { 0x9b8a9b2, SUC_lexicon_tags_62 },
    { 0xce18a9b2, SUC_lexicon_tags_2 },
    { 0xb9b8a9b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f8a9be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c58a9c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf998a9c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f8a9ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1898a9cc, SUC_lexicon_tags_17 },
    { 0x9098a9cd, SUC_lexicon_tags_30 },
    { 0x42d8a9ce, SUC_lexicon_tags_12 },
    { 0x21d8a9cf, SUC_lexicon_tags_37 },
    { 0xa38a9d0, SUC_lexicon_tags_11 },
    { 0xd98a9d1, SUC_lexicon_tags_91 },
    { 0x5458a9cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1398a9d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f8a9dc, SUC_lexicon_tags_39 },
    { 0xcf18a9dd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb938a9e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d8a9eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc658a9f0, SUC_lexicon_tags_30 },
    { 0x3a98a9f1, SUC_lexicon_tags_25 },
    { 0x16d8a9f1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2e58a9f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d58aa01, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbe78aa03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2df8aa05, SUC_lexicon_tags_18 },
    { 0x38aa06, SUC_lexicon_tags_21 },
    { 0xc998aa06, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x65b8aa09, SUC_lexicon_tags_25 },
    { 0x1098aa09, SUC_lexicon_tags_3 },
    { 0xed8aa0b, SUC_lexicon_tags_37 },
    { 0xf1b8aa0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfd38aa0e, SUC_lexicon_tags_11 },
    { 0x46f8aa0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdb8aa11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d8aa16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938aa1c, SUC_lexicon_tags_29 },
    { 0xd8f8aa1c, SUC_lexicon_tags_34 },
    { 0xfe38aa1e, SUC_lexicon_tags_17 },
    { 0xd6d8aa1f, SUC_lexicon_tags_17 },
    { 0xcfb8aa20, SUC_lexicon_tags_37 },
    { 0x4bf8aa1f, SUC_lexicon_tags_7 },
    { 0xb0d8aa1c, SUC_lexicon_tags_30 },
    { 0x9a38aa23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa698aa25, SUC_lexicon_tags_31 },
    { 0x87f8aa26, SUC_lexicon_tags_25 },
    { 0x25d8aa25, SUC_lexicon_tags_61 },
    { 0xc278aa27, SUC_lexicon_tags_37 },
    { 0x9e98aa26, SUC_lexicon_tags_23 },
    { 0x6818aa26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f58aa2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b8aa34, SUC_lexicon_tags_13 },
    { 0x6f38aa35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b8aa3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4978aa3c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe58aa3f, SUC_lexicon_tags_25 },
    { 0x1b98aa40, SUC_lexicon_tags_85 },
    { 0xa3f8aa41, SUC_lexicon_tags_38 },
    { 0x67f8aa3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf98aa45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x978aa47, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3038aa4c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf158aa50, SUC_lexicon_tags_48 },
    { 0x4bf8aa51, SUC_lexicon_tags_25 },
    { 0xbd18aa51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b8aa55, SUC_lexicon_tags_14 },
    { 0xcbb8aa56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1118aa58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b8aa5b, SUC_lexicon_tags_12 },
    { 0xd798aa5b, SUC_lexicon_tags_6 },
    { 0xe98aa5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf438aa63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5318aa65, SUC_lexicon_tags_1 },
    { 0x1418aa65, SUC_lexicon_tags_7 },
    { 0x7c98aa67, SUC_lexicon_tags_37 },
    { 0x6b58aa66, SUC_lexicon_tags_12 },
    { 0x3838aa69, SUC_lexicon_tags_12 },
    { 0x14f8aa6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf338aa70, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58aa76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1718aa79, SUC_lexicon_tags_1 },
    { 0xc9d8aa7a, SUC_lexicon_tags_6 },
    { 0xc858aa7a, SUC_lexicon_tags_7 },
    { 0x7f18aa7c, SUC_lexicon_tags_12 },
    { 0xa3d8aa7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3758aa80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a58aa83, SUC_lexicon_tags_25 },
    { 0xba18aa84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9278aa86, SUC_lexicon_tags_17 },
    { 0xfc18aa86, SUC_lexicon_tags_43 },
    { 0xd98aa88, SUC_lexicon_tags_30 },
    { 0xccf8aa86, SUC_lexicon_tags_2 },
    { 0x8838aa8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf838aa8c, SUC_lexicon_tags_17 },
    { 0xcc18aa8d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f8aa95, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf8aa9a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd558aa9c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d8aa9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xef18aaa1, SUC_lexicon_tags_2 },
    { 0x9258aaa2, SUC_lexicon_tags_33 },
    { 0x5198aaa3, SUC_lexicon_tags_17 },
    { 0x3638aaa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb878aaa7, SUC_lexicon_tags_7 },
    { 0xca78aaa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8aaad, SUC_lexicon_tags_17 },
    { 0x8258aaad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd578aab0, SUC_lexicon_tags_1 },
    { 0x18aab1, SUC_lexicon_tags_1 },
    { 0xa38aab2, SUC_lexicon_tags_27 },
    { 0x5dd8aab1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52f8aab5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad8aab9, SUC_lexicon_tags_38 },
    { 0xd858aab9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db8aabd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd98aac0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x58d8aac2, SUC_lexicon_tags_17 },
    { 0x7a98aac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4158aac7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d58aacf, SUC_lexicon_tags_60 },
    { 0x6b58aad0, SUC_lexicon_tags_38 },
    { 0x74d8aad0, SUC_lexicon_tags_11 },
    { 0xe7f8aad2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc238aad4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b8aad7, SUC_lexicon_tags_30 },
    { 0x5378aad8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb98aade, SUC_lexicon_tags_2 },
    { 0x5118aadf, SUC_lexicon_tags_37 },
    { 0xe478aae0, SUC_lexicon_tags_47 },
    { 0x6e98aae0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc058aae3, SUC_lexicon_tags_43 },
    { 0x46f8aae3, SUC_lexicon_tags_60 },
    { 0xc38aae5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b18aaed, SUC_lexicon_tags_14 },
    { 0x7fd8aaee, SUC_lexicon_tags_34 },
    { 0x8f38aaee, SUC_lexicon_tags_0 },
    { 0xbbb8aaef, SUC_lexicon_tags_12 },
    { 0xb278aaf1, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xa398aaf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9078aaf5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e18aafb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x298aafd, SUC_lexicon_tags_19 },
    { 0xabb8aafd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd518ab01, SUC_lexicon_tags_15 },
    { 0xd598ab02, SUC_lexicon_tags_19 },
    { 0x5138ab02, SUC_lexicon_tags_38 },
    { 0x3178ab04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8958ab06, SUC_lexicon_tags_60 },
    { 0xa438ab07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb8ab0e, SUC_lexicon_tags_124 },
    { 0x6678ab0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb8ab12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2a38ab14, SUC_lexicon_tags_14 },
    { 0x3378ab14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d8ab1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef8ab22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7c18ab24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b78ab26, SUC_lexicon_tags_25 },
    { 0xa478ab27, SUC_lexicon_tags_15 },
    { 0x79b8ab28, SUC_lexicon_tags_47 },
    { 0xd5b8ab27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa78ab2c, SUC_lexicon_tags_31 },
    { 0xb6d8ab2c, SUC_lexicon_tags_2 },
    { 0xbf98ab2e, SUC_lexicon_tags_19 },
    { 0x1d78ab2e, SUC_lexicon_tags_25 },
    { 0xee58ab30, SUC_lexicon_tags_15 },
    { 0xd0f8ab30, SUC_lexicon_tags_37 },
    { 0xe2f8ab2f, SUC_lexicon_tags_0 },
    { 0x8eb8ab33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7438ab37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64b8ab39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f8ab3d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f8ab41, SUC_lexicon_tags_29 },
    { 0xff58ab42, SUC_lexicon_tags_32 },
    { 0x4838ab43, SUC_lexicon_tags_1 },
    { 0x10b8ab44, SUC_lexicon_tags_32 },
    { 0xb338ab43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x818ab48, SUC_lexicon_tags_17 },
    { 0xc3d8ab49, SUC_lexicon_tags_8 },
    { 0x5438ab49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e38ab4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8ab57, SUC_lexicon_tags_12 },
    { 0xdad8ab57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe038ab5c, SUC_lexicon_tags_244 },
    { 0x1e38ab5d, SUC_lexicon_tags_17 },
    { 0xb3d8ab5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3998ab60, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd2f8ab62, SUC_lexicon_tags_11 },
    { 0x6c58ab63, SUC_lexicon_tags_25 },
    { 0xed38ab64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd8ab67, SUC_lexicon_tags_14 },
    { 0x6c58ab68, SUC_lexicon_tags_37 },
    { 0x2298ab69, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0x52f8ab6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8278ab70, SUC_lexicon_tags_32 },
    { 0x76d8ab70, SUC_lexicon_tags_25 },
    { 0x718ab72, SUC_lexicon_tags_11 },
    { 0xdc98ab71, SUC_lexicon_tags_32 },
    { 0xd198ab74, SUC_lexicon_tags_17 },
    { 0x80d8ab74, SUC_lexicon_tags_23 },
    { 0x62f8ab70, SUC_lexicon_tags_37 },
    { 0x4e18ab71, SUC_lexicon_tags_55 },
    { 0x91b8ab76, SUC_lexicon_tags_47 },
    { 0xe7d8ab79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2398ab7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d8ab81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918ab87, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbe78ab89, SUC_lexicon_tags_7 },
    { 0x29d8ab8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd38ab8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b8ab93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc078aba0, SUC_lexicon_tags_17 },
    { 0xaef8aba1, SUC_lexicon_tags_9 },
    { 0x7858aba1, SUC_lexicon_tags_82 },
    { 0xed78aba1, SUC_lexicon_tags_31 },
    { 0x9bb8aba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f8aba7, SUC_lexicon_tags_17 },
    { 0xb378aba7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x51f8abaa, SUC_lexicon_tags_25 },
    { 0x73d8abaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5078abaf, SUC_lexicon_tags_12 },
    { 0x9c18abb0, SUC_lexicon_tags_17 },
    { 0x71d8abb0, SUC_lexicon_tags_60 },
    { 0xdf8abb2, SUC_lexicon_tags_38 },
    { 0xcbf8abaf, SUC_lexicon_tags_46 },
    { 0x9538abb4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc18abb7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x438abbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa7d8abbe, SUC_lexicon_tags_12 },
    { 0x7a78abbf, SUC_lexicon_tags_37 },
    { 0xe778abbf, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf8abc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdad8abc5, SUC_lexicon_tags_37 },
    { 0x16b8abc6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfdb8abc8, SUC_lexicon_tags_12 },
    { 0x3038abc9, SUC_lexicon_tags_38 },
    { 0xecf8abc8, SUC_lexicon_tags_49 },
    { 0xd1b8abcb, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d8abce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb918abd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3d8abd6, SUC_lexicon_tags_31 },
    { 0x2f58abd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e58abda, SUC_lexicon_tags_43 },
    { 0x54d8abdb, SUC_lexicon_tags_23 },
    { 0x2618abdb, SUC_lexicon_tags_11 },
    { 0x6cd8abdb, SUC_lexicon_tags_60 },
    { 0x42f8abde, SUC_lexicon_tags_2 },
    { 0x6898abdf, SUC_lexicon_tags_60 },
    { 0x8218abde, SUC_lexicon_tags_114 },
    { 0x39d8abda, SUC_lexicon_tags_17 },
    { 0x88b8abe2, SUC_lexicon_tags_45 },
    { 0x7f38abdf, SUC_lexicon_tags_17 },
    { 0x1418abe4, SUC_lexicon_tags_25 },
    { 0x84d8abe0, SUC_lexicon_tags_138 },
    { 0x538abdb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6d8abe8, SUC_lexicon_tags_12 },
    { 0xd78abe9, SUC_lexicon_tags_37 },
    { 0x1298abea, SUC_lexicon_tags_57 },
    { 0xf598abeb, SUC_lexicon_tags_12 },
    { 0x6878abec, SUC_lexicon_tags_11 },
    { 0xcf38abed, SUC_lexicon_tags_27 },
    { 0x8b58abee, SUC_lexicon_tags_47 },
    { 0xe118abee, SUC_lexicon_tags_1 },
    { 0xf7f8abea, SUC_lexicon_tags_17 },
    { 0x9b38abee, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3798abf6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda38abfb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8158abfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6378ac01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef8ac04, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe738ac06, SUC_lexicon_tags_37 },
    { 0xbb98ac07, SUC_lexicon_tags_60 },
    { 0xe318ac07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa558ac0a, SUC_lexicon_tags_28 },
    { 0xd6d8ac0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaab8ac0d, SUC_lexicon_tags_49 },
    { 0x8638ac0e, SUC_lexicon_tags_60 },
    { 0x2738ac0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2f58ac11, SUC_lexicon_tags_8 },
    { 0xf2b8ac11, SUC_lexicon_tags_124 },
    { 0x5918ac13, SUC_lexicon_tags_29 },
    { 0x7bf8ac14, SUC_lexicon_tags_37 },
    { 0xa5f8ac11, SUC_lexicon_tags_43 },
    { 0xc758ac12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6038ac19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea58ac20, SUC_lexicon_tags_8 },
    { 0xa2f8ac21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9118ac27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3f8ac29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7f8ac2b, SUC_lexicon_tags_37 },
    { 0xaa98ac2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc78ac30, SUC_lexicon_tags_30 },
    { 0x4938ac30, SUC_lexicon_tags_12 },
    { 0x6e98ac32, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9138ac36, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f38ac3a, SUC_lexicon_tags_9 },
    { 0x7a58ac3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a18ac41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad38ac43, SUC_lexicon_tags_14 },
    { 0xc838ac44, SUC_lexicon_tags_29 },
    { 0x9bf8ac45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5af8ac47, SUC_lexicon_tags_17 },
    { 0x6ff8ac47, SUC_lexicon_tags_14 },
    { 0x87f8ac49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb8ac4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9398ac51, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c98ac54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9918ac56, SUC_lexicon_tags_37 },
    { 0x77d8ac57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb438ac5a, SUC_lexicon_tags_35 },
    { 0x93b8ac5a, SUC_lexicon_tags_1 },
    { 0x9178ac5c, SUC_lexicon_tags_15 },
    { 0xa618ac5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d8ac61, SUC_lexicon_tags_25 },
    { 0x8d8ac62, SUC_lexicon_tags_12 },
    { 0x1d58ac61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b8ac67, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb078ac69, SUC_lexicon_tags_43 },
    { 0xe778ac6a, SUC_lexicon_tags_10 },
    { 0x62b8ac6b, SUC_lexicon_tags_14 },
    { 0xdff8ac6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea58ac70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f8ac74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b58ac77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7278ac79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c98ac7f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9978ac83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd958ac85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338ac8a, SUC_lexicon_tags_45 },
    { 0xecb8ac8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf178ac8d, SUC_lexicon_tags_16 },
    { 0x9738ac8e, SUC_lexicon_tags_11 },
    { 0x5838ac8f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f8ac9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed98aca2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa958aca4, SUC_lexicon_tags_0 },
    { 0xd098aca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3558aca7, SUC_lexicon_tags_16 },
    { 0xd378aca7, SUC_lexicon_tags_37 },
    { 0xfb8aca9, SUC_lexicon_tags_21 },
    { 0x3798acaa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf698acac, SUC_lexicon_tags_11 },
    { 0xf2b8acad, SUC_lexicon_tags_33 },
    { 0x4218acae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc018acb1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb18acb5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7978acb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3cd8acb9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf8acbd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd58acc2, SUC_lexicon_tags_17 },
    { 0x6478acc2, SUC_lexicon_tags_17 },
    { 0xa018acc2, SUC_lexicon_tags_7 },
    { 0x7418acc5, SUC_lexicon_tags_30 },
    { 0x7b8acc6, SUC_lexicon_tags_8 },
    { 0x48f8acc6, SUC_lexicon_tags_25 },
    { 0xf298acc3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1f18acca, SUC_lexicon_tags_37 },
    { 0x7ab8acca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c78acce, SUC_lexicon_tags_39 },
    { 0xafb8accf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1958acd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb878acd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f8acda, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39f8acdc, SUC_lexicon_tags_45 },
    { 0x6e78acdd, SUC_lexicon_tags_3 },
    { 0xc338acde, SUC_lexicon_tags_17 },
    { 0x28b8acdf, SUC_lexicon_tags_27 },
    { 0xfd38acdf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xef98ace3, SUC_lexicon_tags_16 },
    { 0x9778ace4, SUC_lexicon_tags_7 },
    { 0x7538ace3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc18ace7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb8acea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb438acec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1598acef, SUC_lexicon_tags_3 },
    { 0xb418acf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe338acf4, SUC_lexicon_tags_25 },
    { 0xe018acf5, SUC_lexicon_tags_27 },
    { 0x478acf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x72b8acf8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18acfc, SUC_lexicon_tags_95 },
    { 0x7358acfc, SUC_lexicon_tags_15 },
    { 0x34f8acfc, SUC_lexicon_tags_87 },
    { 0xb658acff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5518ad01, SUC_lexicon_tags_6 },
    { 0x1fd8ad01, SUC_lexicon_tags_34 },
    { 0x88b8ad01, SUC_lexicon_tags_1 },
    { 0x138ad02, SUC_lexicon_tags_9 },
    { 0x9978ad05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x37f8ad07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed8ad0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4378ad0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5978ad11, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1d8ad13, SUC_lexicon_tags_25 },
    { 0x3238ad14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb18ad19, SUC_lexicon_tags_27 },
    { 0x96b8ad19, SUC_lexicon_tags_12 },
    { 0xe1d8ad19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2498ad20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ef8ad22, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf0f8ad24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d8ad29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e38ad2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd18ad31, SUC_lexicon_tags_11 },
    { 0x7838ad31, SUC_lexicon_tags_1 },
    { 0x43b8ad32, SUC_lexicon_tags_11 },
    { 0xef38ad33, SUC_lexicon_tags_2 },
    { 0xf258ad35, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f18ad3d, SUC_lexicon_tags_12 },
    { 0xc6d8ad3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb8ad43, SUC_lexicon_tags_15 },
    { 0x1918ad43, SUC_lexicon_tags_2 },
    { 0xe9b8ad45, SUC_lexicon_tags_46 },
    { 0x90b8ad45, SUC_lexicon_tags_28 },
    { 0x2518ad45, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118ad4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd398ad4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab18ad51, SUC_lexicon_tags_17 },
    { 0xc6f8ad52, SUC_lexicon_tags_567 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b18ad55, SUC_lexicon_tags_146 },
    { 0x18ad56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd8ad58, SUC_lexicon_tags_8 },
    { 0xba58ad58, SUC_lexicon_tags_37 },
    { 0xc0b8ad5a, SUC_lexicon_tags_47 },
    { 0xb598ad5b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x75b8ad5d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5958ad60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a98ad63, SUC_lexicon_tags_17 },
    { 0xc158ad64, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1e38ad66, SUC_lexicon_tags_44 },
    { 0x1618ad66, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6458ad69, SUC_lexicon_tags_16 },
    { 0xd338ad6a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb278ad6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd8ad70, SUC_lexicon_tags_32 },
    { 0xcb8ad70, SUC_lexicon_tags_17 },
    { 0x9e18ad72, SUC_lexicon_tags_8 },
    { 0xfc38ad73, SUC_lexicon_tags_17 },
    { 0x43b8ad73, SUC_lexicon_tags_23 },
    { 0xf838ad74, SUC_lexicon_tags_11 },
    { 0x81d8ad76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d8ad7a, SUC_lexicon_tags_15 },
    { 0xded8ad7a, SUC_lexicon_tags_27 },
    { 0xed8ad7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6718ad7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34b8ad81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa658ad84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x17b8ad86, SUC_lexicon_tags_25 },
    { 0xb458ad86, SUC_lexicon_tags_38 },
    { 0x2c18ad88, SUC_lexicon_tags_2 },
    { 0x8d18ad88, SUC_lexicon_tags_23 },
    { 0x1078ad89, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd8ad90, SUC_lexicon_tags_63 },
    { 0xe7d8ad91, SUC_lexicon_tags_7 },
    { 0xf558ad92, SUC_lexicon_tags_17 },
    { 0x7018ad91, SUC_lexicon_tags_49 },
    { 0x9b78ad90, SUC_lexicon_tags_37 },
    { 0x99d8ad95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738ad9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8958ada4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe18ada6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc638ada8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a78adab, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5a58adad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8778adb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5018adb3, SUC_lexicon_tags_21 },
    { 0x4a78adb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf878adbc, SUC_lexicon_tags_9 },
    { 0x6b18adbd, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xdbd8adbf, SUC_lexicon_tags_27 },
    { 0xd318adc0, SUC_lexicon_tags_40 },
    { 0x5638adc0, SUC_lexicon_tags_14 },
    { 0x8a18adc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff8adc5, SUC_lexicon_tags_11 },
    { 0x8558adc5, SUC_lexicon_tags_15 },
    { 0x1e18adc7, SUC_lexicon_tags_17 },
    { 0x62f8adc8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf298adcb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7878adce, SUC_lexicon_tags_17 },
    { 0x3418adce, SUC_lexicon_tags_9 },
    { 0xb658add0, SUC_lexicon_tags_47 },
    { 0x8bb8add0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b8add4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc078add6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2bb8add8, SUC_lexicon_tags_49 },
    { 0xdf8add9, SUC_lexicon_tags_10 },
    { 0x5358adda, SUC_lexicon_tags_12 },
    { 0xa718addb, SUC_lexicon_tags_47 },
    { 0x3238add9, SUC_lexicon_tags_25 },
    { 0x8f78add9, SUC_lexicon_tags_1353 },
    { 0x83f8adde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad8ade3, SUC_lexicon_tags_17 },
    { 0xaa18ade4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f58adeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8adee, SUC_lexicon_tags_37 },
    { 0x1fb8adee, SUC_lexicon_tags_29 },
    { 0xd298adf0, SUC_lexicon_tags_8 },
    { 0xb338adf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc918adf4, SUC_lexicon_tags_28 },
    { 0x5b58adf5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb8adf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc578adfc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b8adff, SUC_lexicon_tags_17 },
    { 0x5418ae00, SUC_lexicon_tags_0 },
    { 0xd938adff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef18ae03, SUC_lexicon_tags_47 },
    { 0xdb78ae03, SUC_lexicon_tags_0 },
    { 0xff38ae03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd8ae0a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2738ae0d, SUC_lexicon_tags_15 },
    { 0x7ab8ae0e, SUC_lexicon_tags_17 },
    { 0xfdb8ae0e, SUC_lexicon_tags_39 },
    { 0xcdd8ae0e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4058ae16, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9838ae18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa918ae1c, SUC_lexicon_tags_43 },
    { 0xc378ae1d, SUC_lexicon_tags_2 },
    { 0xacf8ae1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db8ae21, SUC_lexicon_tags_25 },
    { 0x8fd8ae21, SUC_lexicon_tags_13 },
    { 0x4978ae21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3d8ae25, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb878ae27, SUC_lexicon_tags_28 },
    { 0x97d8ae27, SUC_lexicon_tags_12 },
    { 0x4e78ae29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa238ae2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ae2f, SUC_lexicon_tags_30 },
    { 0xe978ae30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff8ae33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78ae36, SUC_lexicon_tags_12 },
    { 0xadf8ae37, SUC_lexicon_tags_3 },
    { 0xa178ae36, SUC_lexicon_tags_31 },
    { 0xd9d8ae37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba78ae3d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f58ae42, SUC_lexicon_tags_19 },
    { 0x4e78ae43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f8ae49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f58ae4b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfe18ae4d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b8ae53, SUC_lexicon_tags_1 },
    { 0x86b8ae54, SUC_lexicon_tags_45 },
    { 0x3018ae55, SUC_lexicon_tags_12 },
    { 0xf58ae55, SUC_lexicon_tags_17 },
    { 0xf878ae54, SUC_lexicon_tags_25 },
    { 0x6bf8ae58, SUC_lexicon_tags_9 },
    { 0xae78ae59, SUC_lexicon_tags_17 },
    { 0xccf8ae5a, SUC_lexicon_tags_1 },
    { 0x9518ae5b, SUC_lexicon_tags_68 },
    { 0x8178ae5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2018ae5f, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x6238ae61, SUC_lexicon_tags_98 },
    { 0xc3f8ae62, SUC_lexicon_tags_46 },
    { 0xf358ae63, SUC_lexicon_tags_8 },
    { 0xced8ae64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3518ae67, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9378ae6e, SUC_lexicon_tags_11 },
    { 0x1fd8ae6f, SUC_lexicon_tags_11 },
    { 0x938ae6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b8ae75, SUC_lexicon_tags_25 },
    { 0x8b38ae75, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xea18ae78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d98ae7c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f8ae7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeed8ae81, SUC_lexicon_tags_63 },
    { 0xc898ae82, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x46b8ae84, SUC_lexicon_tags_12 },
    { 0x32f8ae84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f8ae8b, SUC_lexicon_tags_11 },
    { 0xfcf8ae8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8658ae8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d8ae92, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x4b58ae94, SUC_lexicon_tags_79 },
    { 0xd258ae95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e18ae9a, SUC_lexicon_tags_47 },
    { 0x4f78ae9b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb8aea0, SUC_lexicon_tags_1 },
    { 0xa5f8aea0, SUC_lexicon_tags_1 },
    { 0xbd78aea1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8aea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef8aea8, SUC_lexicon_tags_32 },
    { 0xf5b8aea8, SUC_lexicon_tags_33 },
    { 0x9498aeaa, SUC_lexicon_tags_31 },
    { 0x2738aeaa, SUC_lexicon_tags_23 },
    { 0x1fd8aea9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6898aeb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f8aeb8, SUC_lexicon_tags_65 },
    { 0xb398aeb9, SUC_lexicon_tags_1 },
    { 0xf8f8aeb8, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8aebd, SUC_lexicon_tags_11 },
    { 0xba58aebd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9758aec0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b8aec3, SUC_lexicon_tags_17 },
    { 0xe7f8aec4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f98aec8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d38aecc, SUC_lexicon_tags_56 },
    { 0x3038aecd, SUC_lexicon_tags_27 },
    { 0x6238aecd, SUC_lexicon_tags_10 },
    { 0xa998aecf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb658aed2, SUC_lexicon_tags_12 },
    { 0x9418aed3, SUC_lexicon_tags_35 },
    { 0xb058aed4, SUC_lexicon_tags_15 },
    { 0xa738aed3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f8aed8, SUC_lexicon_tags_7 },
    { 0x1b58aed8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe338aedd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f8aee4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6f18aee6, SUC_lexicon_tags_1 },
    { 0xf8f8aee7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf8aeea, SUC_lexicon_tags_15 },
    { 0xd3d8aeeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf858aeef, SUC_lexicon_tags_11 },
    { 0x34b8aeef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdab8aef2, SUC_lexicon_tags_37 },
    { 0x38aef3, SUC_lexicon_tags_29 },
    { 0x1d58aef3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad8aef8, SUC_lexicon_tags_63 },
    { 0x278aef9, SUC_lexicon_tags_27 },
    { 0x8318aef9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8298aefd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8578aeff, SUC_lexicon_tags_47 },
    { 0x1758af00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d8af07, SUC_lexicon_tags_45 },
    { 0xf78af08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f8af0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f18af15, SUC_lexicon_tags_23 },
    { 0x9d38af16, SUC_lexicon_tags_46 },
    { 0xaaf8af16, SUC_lexicon_tags_21 },
    { 0xb838af16, SUC_lexicon_tags_65 },
    { 0x8178af19, SUC_lexicon_tags_24 },
    { 0xcef8af1a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7918af20, SUC_lexicon_tags_19 },
    { 0xc38af20, SUC_lexicon_tags_15 },
    { 0x54f8af22, SUC_lexicon_tags_7 },
    { 0xf2f8af22, SUC_lexicon_tags_101 },
    { 0xb138af24, SUC_lexicon_tags_39 },
    { 0xc7f8af25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d58af2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b8af2e, SUC_lexicon_tags_38 },
    { 0x3198af2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b8af3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xee18af3d, SUC_lexicon_tags_27 },
    { 0x3518af3e, SUC_lexicon_tags_5 },
    { 0xe038af3e, SUC_lexicon_tags_28 },
    { 0x5438af3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2cd8af42, SUC_lexicon_tags_7 },
    { 0xb078af43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff8af46, SUC_lexicon_tags_25 },
    { 0x1378af46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaed8af49, SUC_lexicon_tags_33 },
    { 0x58d8af4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf8af53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa18af5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcad8af5c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f8af60, SUC_lexicon_tags_7 },
    { 0xc6f8af61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3938af63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb8af65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc18af67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe138af6a, SUC_lexicon_tags_8 },
    { 0xcdd8af6b, SUC_lexicon_tags_0 },
    { 0x4fb8af6c, SUC_lexicon_tags_56 },
    { 0xbe58af6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6b8af6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd8af72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8af7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d8af81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc58af85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4b8af87, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe18af8b, SUC_lexicon_tags_11 },
    { 0x3e38af8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad8af90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd98af94, SUC_lexicon_tags_77 },
    { 0xf9f8af95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf8af98, SUC_lexicon_tags_11 },
    { 0x4d38af99, SUC_lexicon_tags_1 },
    { 0x2a38af9a, SUC_lexicon_tags_12 },
    { 0x9f58af98, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d98af9e, SUC_lexicon_tags_47 },
    { 0x2318af9f, SUC_lexicon_tags_17 },
    { 0x2098afa0, SUC_lexicon_tags_37 },
    { 0x7c38af9f, SUC_lexicon_tags_33 },
    { 0xe278afa2, SUC_lexicon_tags_532 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8218afa8, SUC_lexicon_tags_23 },
    { 0xfd78afa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3438afad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6fd8afaf, SUC_lexicon_tags_14 },
    { 0x73d8afaf, SUC_lexicon_tags_30 },
    { 0xe798afb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6258afb3, SUC_lexicon_tags_28 },
    { 0xb9b8afb4, SUC_lexicon_tags_37 },
    { 0xd678afb5, SUC_lexicon_tags_11 },
    { 0x1598afb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd38afba, SUC_lexicon_tags_15 },
    { 0xa1f8afbb, SUC_lexicon_tags_6 },
    { 0x6518afbb, SUC_lexicon_tags_38 },
    { 0xbdb8afba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe078afc0, SUC_lexicon_tags_39 },
    { 0x2ad8afc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6678afc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d78afd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b8afd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f8afda, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe518afe0, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0xc58afe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcf98afe5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4658afe7, SUC_lexicon_tags_45 },
    { 0x90f8afe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa878afea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2678aff1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b58aff4, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x2a18aff6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5438aff9, SUC_lexicon_tags_14 },
    { 0x5c58affa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f8b000, SUC_lexicon_tags_31 },
    { 0x6df8b001, SUC_lexicon_tags_6 },
    { 0x2418b002, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4138b006, SUC_lexicon_tags_38 },
    { 0xcad8b007, SUC_lexicon_tags_160 },
    { 0x83b8b008, SUC_lexicon_tags_47 },
    { 0x878b008, SUC_lexicon_tags_12 },
    { 0x6b78b00a, SUC_lexicon_tags_17 },
    { 0x50f8b00b, SUC_lexicon_tags_38 },
    { 0x7b8b00b, SUC_lexicon_tags_25 },
    { 0xcdb8b00d, SUC_lexicon_tags_12 },
    { 0xeff8b00c, SUC_lexicon_tags_13 },
    { 0x1e58b009, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xec18b012, SUC_lexicon_tags_1 },
    { 0xcd8b013, SUC_lexicon_tags_12 },
    { 0x9d18b014, SUC_lexicon_tags_49 },
    { 0x36d8b013, SUC_lexicon_tags_14 },
    { 0xd358b016, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d8b01b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5998b020, SUC_lexicon_tags_63 },
    { 0xb3b8b021, SUC_lexicon_tags_60 },
    { 0x60f8b022, SUC_lexicon_tags_6 },
    { 0xe618b020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1358b026, SUC_lexicon_tags_12 },
    { 0x6df8b027, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x24b8b029, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xab98b02c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f8b030, SUC_lexicon_tags_38 },
    { 0x6218b030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4458b035, SUC_lexicon_tags_45 },
    { 0xc8b8b035, SUC_lexicon_tags_25 },
    { 0x5918b036, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e18b039, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x61f8b03b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0d8b03d, SUC_lexicon_tags_15 },
    { 0xfa78b03e, SUC_lexicon_tags_77 },
    { 0x1178b03e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4d8b041, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca18b044, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff8b047, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x92d8b04a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb98b04e, SUC_lexicon_tags_12 },
    { 0x6118b04f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa578b052, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba78b056, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3c38b058, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf738b05a, SUC_lexicon_tags_48 },
    { 0xa6d8b05b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9598b05d, SUC_lexicon_tags_25 },
    { 0xfdf8b05d, SUC_lexicon_tags_16 },
    { 0x8db8b05e, SUC_lexicon_tags_23 },
    { 0x3538b060, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c38b065, SUC_lexicon_tags_19 },
    { 0x1358b065, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53d8b068, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6998b06a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad8b06d, SUC_lexicon_tags_25 },
    { 0x7558b06e, SUC_lexicon_tags_7 },
    { 0xa538b06f, SUC_lexicon_tags_4 },
    { 0x9638b070, SUC_lexicon_tags_27 },
    { 0x93f8b071, SUC_lexicon_tags_29 },
    { 0x2838b072, SUC_lexicon_tags_18 },
    { 0xddd8b072, SUC_lexicon_tags_28 },
    { 0x1238b074, SUC_lexicon_tags_1267 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e98b077, SUC_lexicon_tags_25 },
    { 0x398b078, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8b07e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7458b081, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69d8b083, SUC_lexicon_tags_23 },
    { 0x23d8b084, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8338b087, SUC_lexicon_tags_0 },
    { 0x28b8b087, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbb38b08a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe98b08c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf8b096, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x42d8b098, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5778b09a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff8b0a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f8b0a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a18b0ac, SUC_lexicon_tags_50 },
    { 0x72d8b0ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fb8b0af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x78b8b0b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9598b0b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d8b0b8, SUC_lexicon_tags_12 },
    { 0xc7d8b0b8, SUC_lexicon_tags_46 },
    { 0x1658b0ba, SUC_lexicon_tags_17 },
    { 0xc678b0bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd8b0c1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9358b0c4, SUC_lexicon_tags_11 },
    { 0x7d58b0c5, SUC_lexicon_tags_39 },
    { 0xc138b0c6, SUC_lexicon_tags_23 },
    { 0xcbd8b0c7, SUC_lexicon_tags_2 },
    { 0x93f8b0c5, SUC_lexicon_tags_11 },
    { 0x1838b0c9, SUC_lexicon_tags_39 },
    { 0xfcb8b0ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3f8b0cc, SUC_lexicon_tags_1 },
    { 0xfc98b0cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf638b0cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4978b0d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f38b0d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9338b0d9, SUC_lexicon_tags_603 },
    { 0x3c18b0da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d38b0dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x11b8b0de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7558b0e4, SUC_lexicon_tags_27 },
    { 0xf6b8b0e5, SUC_lexicon_tags_2 },
    { 0x2518b0e6, SUC_lexicon_tags_6 },
    { 0x89b8b0e6, SUC_lexicon_tags_5 },
    { 0x1e58b0e6, SUC_lexicon_tags_37 },
    { 0xf318b0e8, SUC_lexicon_tags_19 },
    { 0xe18b0e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e58b0ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d8b0f3, SUC_lexicon_tags_201 },
    { 0x59f8b0f4, SUC_lexicon_tags_7 },
    { 0x9378b0f5, SUC_lexicon_tags_29 },
    { 0xe098b0f4, SUC_lexicon_tags_63 },
    { 0x1d58b0f7, SUC_lexicon_tags_15 },
    { 0xbd8b0f5, SUC_lexicon_tags_28 },
    { 0x2f18b0f8, SUC_lexicon_tags_29 },
    { 0xc438b0f6, SUC_lexicon_tags_11 },
    { 0xe998b0fb, SUC_lexicon_tags_26 },
    { 0xacf8b0f7, SUC_lexicon_tags_37 },
    { 0x4ed8b0fb, SUC_lexicon_tags_15 },
    { 0x13d8b0fe, SUC_lexicon_tags_15 },
    { 0x598b0fe, SUC_lexicon_tags_7 },
    { 0x9278b100, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f78b104, SUC_lexicon_tags_11 },
    { 0x778b105, SUC_lexicon_tags_13 },
    { 0x4b58b104, SUC_lexicon_tags_12 },
    { 0x9498b106, SUC_lexicon_tags_15 },
    { 0xe4d8b105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc18b115, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f8b119, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa798b11b, SUC_lexicon_tags_0 },
    { 0x378b11c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6418b120, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6778b122, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa6f8b124, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xd458b127, SUC_lexicon_tags_12 },
    { 0x1d58b128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d8b131, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f58b134, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb38b138, SUC_lexicon_tags_30 },
    { 0x4ab8b139, SUC_lexicon_tags_23 },
    { 0x1bf8b13a, SUC_lexicon_tags_1 },
    { 0x94d8b13a, SUC_lexicon_tags_60 },
    { 0xdc58b139, SUC_lexicon_tags_2 },
    { 0x35b8b13a, SUC_lexicon_tags_25 },
    { 0x9718b13e, SUC_lexicon_tags_37 },
    { 0x57b8b13c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8b98b141, SUC_lexicon_tags_34 },
    { 0xed38b142, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2e58b144, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3318b148, SUC_lexicon_tags_16 },
    { 0x39f8b149, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe018b14f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7378b154, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f8b157, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6378b15c, SUC_lexicon_tags_19 },
    { 0x3cb8b15d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa6b8b15f, SUC_lexicon_tags_65 },
    { 0xfd38b160, SUC_lexicon_tags_14 },
    { 0x79f8b15f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad8b164, SUC_lexicon_tags_26 },
    { 0xe298b164, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7238b16a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf8b170, SUC_lexicon_tags_2 },
    { 0x1218b171, SUC_lexicon_tags_23 },
    { 0x7a98b171, SUC_lexicon_tags_11 },
    { 0x7978b172, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8818b176, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b8b17b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6ef8b17d, SUC_lexicon_tags_15 },
    { 0xa8d8b17e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1718b182, SUC_lexicon_tags_37 },
    { 0xf438b183, SUC_lexicon_tags_38 },
    { 0x2538b183, SUC_lexicon_tags_44 },
    { 0x22b8b184, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2058b187, SUC_lexicon_tags_31 },
    { 0x3c78b187, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb58b18a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x81d8b18c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17b8b18e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc718b191, SUC_lexicon_tags_87 },
    { 0x8258b192, SUC_lexicon_tags_12 },
    { 0x8938b193, SUC_lexicon_tags_63 },
    { 0x86f8b192, SUC_lexicon_tags_64 },
    { 0xc3d8b195, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45b8b197, SUC_lexicon_tags_30 },
    { 0x7b78b198, SUC_lexicon_tags_60 },
    { 0xa5b8b199, SUC_lexicon_tags_37 },
    { 0xeff8b19a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc418b19e, SUC_lexicon_tags_29 },
    { 0xf458b19e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdb8b1a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f8b1a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd18b1a7, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f98b1aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe818b1ad, SUC_lexicon_tags_12 },
    { 0xbd58b1ae, SUC_lexicon_tags_31 },
    { 0xc6d8b1af, SUC_lexicon_tags_11 },
    { 0x418b1ae, SUC_lexicon_tags_39 },
    { 0x5f18b1b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8b1b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e58b1b7, SUC_lexicon_tags_12 },
    { 0xcb38b1b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb38b1ba, SUC_lexicon_tags_25 },
    { 0xa238b1ba, SUC_lexicon_tags_11 },
    { 0xcab8b1bc, SUC_lexicon_tags_33 },
    { 0x5cd8b1bd, SUC_lexicon_tags_47 },
    { 0x778b1be, SUC_lexicon_tags_15 },
    { 0x62d8b1bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18b1c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb8b1c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe58b1d3, SUC_lexicon_tags_12 },
    { 0x6538b1d4, SUC_lexicon_tags_17 },
    { 0xb2b8b1d5, SUC_lexicon_tags_17 },
    { 0x28d8b1d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7738b1da, SUC_lexicon_tags_23 },
    { 0x3318b1db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2ab8b1dd, SUC_lexicon_tags_7 },
    { 0xfff8b1dd, SUC_lexicon_tags_33 },
    { 0xc438b1df, SUC_lexicon_tags_12 },
    { 0x3858b1e0, SUC_lexicon_tags_29 },
    { 0x4998b1e1, SUC_lexicon_tags_77 },
    { 0x138b1e2, SUC_lexicon_tags_3 },
    { 0xd1f8b1e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6638b1f3, SUC_lexicon_tags_37 },
    { 0x7978b1f4, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x8858b1f6, SUC_lexicon_tags_25 },
    { 0x8758b1f7, SUC_lexicon_tags_12 },
    { 0x95d8b1f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b8b1fe, SUC_lexicon_tags_7 },
    { 0x2b18b1ff, SUC_lexicon_tags_8 },
    { 0x9ff8b200, SUC_lexicon_tags_30 },
    { 0x2a18b200, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4f78b203, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb578b205, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b8b208, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb438b210, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1798b216, SUC_lexicon_tags_29 },
    { 0x7498b217, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef8b21d, SUC_lexicon_tags_7 },
    { 0xc898b21e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a58b223, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c18b226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71f8b228, SUC_lexicon_tags_37 },
    { 0x9818b229, SUC_lexicon_tags_25 },
    { 0xb1b8b22a, SUC_lexicon_tags_31 },
    { 0x11d8b22b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf8b231, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecb8b233, SUC_lexicon_tags_17 },
    { 0x8d8b234, SUC_lexicon_tags_15 },
    { 0x8978b235, SUC_lexicon_tags_27 },
    { 0xf158b236, SUC_lexicon_tags_36 },
    { 0x4a58b237, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd058b23f, SUC_lexicon_tags_11 },
    { 0x4c58b240, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6278b242, SUC_lexicon_tags_45 },
    { 0x6c18b243, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f8b246, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94f8b248, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2d8b24a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc038b24e, SUC_lexicon_tags_33 },
    { 0xdbd8b24e, SUC_lexicon_tags_46 },
    { 0x4ef8b250, SUC_lexicon_tags_19 },
    { 0xff98b24e, SUC_lexicon_tags_11 },
    { 0x6d98b252, SUC_lexicon_tags_16 },
    { 0x7538b251, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd8b256, SUC_lexicon_tags_15 },
    { 0xedf8b257, SUC_lexicon_tags_39 },
    { 0x4058b257, SUC_lexicon_tags_37 },
    { 0x5418b259, SUC_lexicon_tags_25 },
    { 0xb278b25a, SUC_lexicon_tags_7 },
    { 0x3d18b259, SUC_lexicon_tags_12 },
    { 0xae98b25c, SUC_lexicon_tags_10 },
    { 0xb0b8b259, SUC_lexicon_tags_69 },
    { 0x6358b25e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf58b260, SUC_lexicon_tags_15 },
    { 0x27f8b261, SUC_lexicon_tags_17 },
    { 0x6838b261, SUC_lexicon_tags_17 },
    { 0x6418b262, SUC_lexicon_tags_37 },
    { 0xf358b262, SUC_lexicon_tags_17 },
    { 0x558b265, SUC_lexicon_tags_7 },
    { 0x5f98b265, SUC_lexicon_tags_49 },
    { 0x2ad8b264, SUC_lexicon_tags_26 },
    { 0x638b268, SUC_lexicon_tags_25 },
    { 0x46f8b269, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x42b8b26b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xdf18b26d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef8b272, SUC_lexicon_tags_27 },
    { 0x8838b273, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b8b276, SUC_lexicon_tags_30 },
    { 0x8cf8b277, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed8b27b, SUC_lexicon_tags_17 },
    { 0x8198b27c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad8b281, SUC_lexicon_tags_27 },
    { 0x5778b282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb8b286, SUC_lexicon_tags_17 },
    { 0x5fd8b287, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c58b289, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8298b28b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6458b28d, SUC_lexicon_tags_77 },
    { 0x7a78b28e, SUC_lexicon_tags_17 },
    { 0xe958b28d, SUC_lexicon_tags_49 },
    { 0xce58b290, SUC_lexicon_tags_2 },
    { 0x8a98b290, SUC_lexicon_tags_9 },
    { 0xc218b291, SUC_lexicon_tags_1 },
    { 0xbcd8b293, SUC_lexicon_tags_0 },
    { 0x10d8b293, SUC_lexicon_tags_49 },
    { 0xdbb8b294, SUC_lexicon_tags_33 },
    { 0xf758b296, SUC_lexicon_tags_64 },
    { 0x2938b296, SUC_lexicon_tags_37 },
    { 0xbfb8b297, SUC_lexicon_tags_1 },
    { 0xc4b8b294, SUC_lexicon_tags_47 },
    { 0x5398b29a, SUC_lexicon_tags_27 },
    { 0xe758b28d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f8b29f, SUC_lexicon_tags_27 },
    { 0xcbf8b2a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44b8b2a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f8b2a6, SUC_lexicon_tags_33 },
    { 0x4938b2a7, SUC_lexicon_tags_26 },
    { 0x2498b2a8, SUC_lexicon_tags_14 },
    { 0xc238b2a9, SUC_lexicon_tags_12 },
    { 0xd178b2aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8b2af, SUC_lexicon_tags_1 },
    { 0x7478b2b0, SUC_lexicon_tags_9 },
    { 0x59d8b2b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c78b2b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf1d8b2ba, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d8b2c9, SUC_lexicon_tags_43 },
    { 0x17f8b2ca, SUC_lexicon_tags_116 },
    { 0x1c18b2cb, SUC_lexicon_tags_15 },
    { 0xf178b2cc, SUC_lexicon_tags_25 },
    { 0x5d98b2cd, SUC_lexicon_tags_25 },
    { 0xffb8b2ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a58b2d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7878b2d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3f8b2d6, SUC_lexicon_tags_37 },
    { 0x5f58b2d7, SUC_lexicon_tags_37 },
    { 0x6018b2d8, SUC_lexicon_tags_60 },
    { 0xc598b2d8, SUC_lexicon_tags_208 },
    { 0x55f8b2da, SUC_lexicon_tags_25 },
    { 0xe38b2d9, SUC_lexicon_tags_15 },
    { 0xbe18b2d7, SUC_lexicon_tags_55 },
    { 0x8238b2dd, SUC_lexicon_tags_11 },
    { 0x5ab8b2de, SUC_lexicon_tags_33 },
    { 0x3d58b2df, SUC_lexicon_tags_25 },
    { 0xca18b2e0, SUC_lexicon_tags_15 },
    { 0xa1f8b2e1, SUC_lexicon_tags_30 },
    { 0xf258b2e0, SUC_lexicon_tags_37 },
    { 0x2fd8b2e1, SUC_lexicon_tags_126 },
    { 0x9058b2e4, SUC_lexicon_tags_47 },
    { 0x2338b2e5, SUC_lexicon_tags_37 },
    { 0x9c98b2e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde98b2e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7178b2ec, SUC_lexicon_tags_54 },
    { 0xc738b2ec, SUC_lexicon_tags_17 },
    { 0x2c38b2ee, SUC_lexicon_tags_12 },
    { 0xff8b2ef, SUC_lexicon_tags_17 },
    { 0x13b8b2f0, SUC_lexicon_tags_25 },
    { 0x7878b2ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb8b2f5, SUC_lexicon_tags_87 },
    { 0x5dd8b2f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8138b2f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f38b2fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b8b2ff, SUC_lexicon_tags_12 },
    { 0x81d8b300, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a58b302, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfd8b304, SUC_lexicon_tags_37 },
    { 0xe798b304, SUC_lexicon_tags_26 },
    { 0xa818b306, SUC_lexicon_tags_19 },
    { 0x3cf8b307, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xed78b309, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3698b313, SUC_lexicon_tags_7 },
    { 0x1cd8b314, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab8b31a, SUC_lexicon_tags_25 },
    { 0x7698b31b, SUC_lexicon_tags_64 },
    { 0x43d8b31a, SUC_lexicon_tags_11 },
    { 0x4718b31d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f78b31f, SUC_lexicon_tags_13 },
    { 0x3538b320, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd38b324, SUC_lexicon_tags_46 },
    { 0x1af8b325, SUC_lexicon_tags_25 },
    { 0xc798b326, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4758b32d, SUC_lexicon_tags_25 },
    { 0x7878b32e, SUC_lexicon_tags_25 },
    { 0x2d18b32f, SUC_lexicon_tags_19 },
    { 0xb2f8b32d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd278b333, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e78b337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6df8b339, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b8b33d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb698b343, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xad98b346, SUC_lexicon_tags_3 },
    { 0xc938b347, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x38b8b34a, SUC_lexicon_tags_25 },
    { 0xbfb8b34b, SUC_lexicon_tags_2 },
    { 0xe258b34b, SUC_lexicon_tags_275 },
    { 0x5818b34c, SUC_lexicon_tags_1 },
    { 0x87b8b34c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe38b353, SUC_lexicon_tags_15 },
    { 0xe3d8b354, SUC_lexicon_tags_2 },
    { 0xa4f8b354, SUC_lexicon_tags_30 },
    { 0x6958b353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe778b35b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x278b35d, SUC_lexicon_tags_11 },
    { 0xadd8b35e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9778b364, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xeb98b366, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6878b36d, SUC_lexicon_tags_25 },
    { 0xfe78b36d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3738b373, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3758b37c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db8b37f, SUC_lexicon_tags_21 },
    { 0xac98b37f, SUC_lexicon_tags_7 },
    { 0x69f8b37f, SUC_lexicon_tags_29 },
    { 0x7158b37f, SUC_lexicon_tags_68 },
    { 0x62f8b381, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa418b385, SUC_lexicon_tags_17 },
    { 0x8c98b386, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd78b389, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xced8b38c, SUC_lexicon_tags_39 },
    { 0x8018b38d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd8b391, SUC_lexicon_tags_57 },
    { 0xd398b391, SUC_lexicon_tags_7 },
    { 0x4658b391, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3538b395, SUC_lexicon_tags_7 },
    { 0x86f8b396, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb0f8b398, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xddd8b39a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f8b3a5, SUC_lexicon_tags_4 },
    { 0xf78b3a6, SUC_lexicon_tags_12 },
    { 0x4a58b3a7, SUC_lexicon_tags_33 },
    { 0xda98b3a8, SUC_lexicon_tags_7 },
    { 0x3418b3a9, SUC_lexicon_tags_31 },
    { 0x17f8b3a7, SUC_lexicon_tags_11 },
    { 0xb5f8b3a8, SUC_lexicon_tags_60 },
    { 0x66b8b3ac, SUC_lexicon_tags_37 },
    { 0x2f78b3ac, SUC_lexicon_tags_15 },
    { 0xcd78b3ac, SUC_lexicon_tags_11 },
    { 0x7358b3af, SUC_lexicon_tags_14 },
    { 0xd678b3b0, SUC_lexicon_tags_9 },
    { 0x2538b3b1, SUC_lexicon_tags_77 },
    { 0x5c58b3b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ad8b3b4, SUC_lexicon_tags_43 },
    { 0xedb8b3b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfef8b3b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78b3ba, SUC_lexicon_tags_0 },
    { 0xa2f8b3bb, SUC_lexicon_tags_14 },
    { 0xc4f8b3bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3a18b3be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7418b3c0, SUC_lexicon_tags_31 },
    { 0x25b8b3c0, SUC_lexicon_tags_31 },
    { 0x8fb8b3c2, SUC_lexicon_tags_43 },
    { 0xdfd8b3c2, SUC_lexicon_tags_25 },
    { 0x3058b3c2, SUC_lexicon_tags_4 },
    { 0xe1b8b3c5, SUC_lexicon_tags_25 },
    { 0xbf38b3c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xef38b3c9, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce78b3ce, SUC_lexicon_tags_2 },
    { 0x49d8b3ce, SUC_lexicon_tags_2 },
    { 0xf9f8b3ce, SUC_lexicon_tags_7 },
    { 0x76f8b3d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4118b3d4, SUC_lexicon_tags_25 },
    { 0x8158b3d5, SUC_lexicon_tags_37 },
    { 0x51f8b3d5, SUC_lexicon_tags_0 },
    { 0xf378b3d5, SUC_lexicon_tags_37 },
    { 0x7cb8b3d8, SUC_lexicon_tags_15 },
    { 0x5e38b3d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b8b3df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6778b3e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf278b3e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b8b3e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3898b3e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d18b3ec, SUC_lexicon_tags_17 },
    { 0xc3f8b3ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb18b3f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a98b3f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a98b3f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xca58b3f9, SUC_lexicon_tags_44 },
    { 0x9df8b3fa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf38b3fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8798b404, SUC_lexicon_tags_252 },
    { 0xa5b8b404, SUC_lexicon_tags_282 },
    { 0x5678b406, SUC_lexicon_tags_63 },
    { 0x7f98b405, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e98b40e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458b415, SUC_lexicon_tags_22 },
    { 0x9c18b416, SUC_lexicon_tags_15 },
    { 0x6e58b416, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b8b41b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x21f8b41e, SUC_lexicon_tags_13 },
    { 0xdfb8b41f, SUC_lexicon_tags_8 },
    { 0xdb8b420, SUC_lexicon_tags_17 },
    { 0x1898b421, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c18b423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f8b427, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1738b42a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa878b42c, SUC_lexicon_tags_7 },
    { 0x7f58b42c, SUC_lexicon_tags_12 },
    { 0x7918b42c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b8b434, SUC_lexicon_tags_27 },
    { 0x2258b435, SUC_lexicon_tags_7 },
    { 0x3c58b434, SUC_lexicon_tags_37 },
    { 0x4ab8b434, SUC_lexicon_tags_45 },
    { 0xddf8b436, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1578b43b, SUC_lexicon_tags_57 },
    { 0x3958b43c, SUC_lexicon_tags_0 },
    { 0x49b8b43c, SUC_lexicon_tags_5 },
    { 0xad8b43e, SUC_lexicon_tags_14 },
    { 0x8378b43f, SUC_lexicon_tags_7 },
    { 0x2cb8b43e, SUC_lexicon_tags_60 },
    { 0x9eb8b43e, SUC_lexicon_tags_17 },
    { 0x1c18b442, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3cd8b444, SUC_lexicon_tags_37 },
    { 0xb258b444, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf898b449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3b8b44b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9458b44f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa238b453, SUC_lexicon_tags_17 },
    { 0x1758b454, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xed58b456, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d8b45a, SUC_lexicon_tags_0 },
    { 0xeef8b45b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd78b460, SUC_lexicon_tags_43 },
    { 0x3878b461, SUC_lexicon_tags_43 },
    { 0x23d8b460, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f8b466, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc58b46c, SUC_lexicon_tags_1 },
    { 0x8918b46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9018b46f, SUC_lexicon_tags_25 },
    { 0xeff8b470, SUC_lexicon_tags_0 },
    { 0x4c38b470, SUC_lexicon_tags_7 },
    { 0xd4b8b46f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7218b479, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf658b480, SUC_lexicon_tags_60 },
    { 0xabd8b481, SUC_lexicon_tags_8 },
    { 0x7118b482, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f8b488, SUC_lexicon_tags_15 },
    { 0xd038b489, SUC_lexicon_tags_27 },
    { 0xe5f8b48a, SUC_lexicon_tags_30 },
    { 0x7b8b48a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e38b48f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d18b493, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb38b495, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc458b497, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5598b499, SUC_lexicon_tags_2 },
    { 0xd5d8b49a, SUC_lexicon_tags_25 },
    { 0x4498b49a, SUC_lexicon_tags_19 },
    { 0xb058b49c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef78b4a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd8b4a7, SUC_lexicon_tags_65 },
    { 0xe538b4a7, SUC_lexicon_tags_27 },
    { 0x1cb8b4a8, SUC_lexicon_tags_10 },
    { 0xec98b4aa, SUC_lexicon_tags_30 },
    { 0x2ad8b4a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d78b4ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f58b4b0, SUC_lexicon_tags_1 },
    { 0x15b8b4b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4618b4b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f8b4ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5418b4be, SUC_lexicon_tags_49 },
    { 0xcd38b4bf, SUC_lexicon_tags_7 },
    { 0xeaf8b4c0, SUC_lexicon_tags_7 },
    { 0x74b8b4c1, SUC_lexicon_tags_49 },
    { 0x2d78b4c2, SUC_lexicon_tags_15 },
    { 0x27b8b4c3, SUC_lexicon_tags_0 },
    { 0xc378b4bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfdf8b4c6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6498b4ca, SUC_lexicon_tags_17 },
    { 0xf898b4cb, SUC_lexicon_tags_65 },
    { 0x1118b4cc, SUC_lexicon_tags_13 },
    { 0x3178b4cc, SUC_lexicon_tags_12 },
    { 0xb578b4ce, SUC_lexicon_tags_12 },
    { 0x68b8b4ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2c98b4d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9458b4d4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d8b4d8, SUC_lexicon_tags_17 },
    { 0xf198b4d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8b4df, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e98b4e2, SUC_lexicon_tags_17 },
    { 0x3678b4e2, SUC_lexicon_tags_7 },
    { 0x2238b4e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b38b4e6, SUC_lexicon_tags_27 },
    { 0x8b78b4e6, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xde18b4e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe618b4eb, SUC_lexicon_tags_37 },
    { 0x8cd8b4ec, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x3a98b4ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73f8b4f0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2fb8b4f2, SUC_lexicon_tags_6 },
    { 0x5b18b4f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd58b4f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d8b4f9, SUC_lexicon_tags_44 },
    { 0xabf8b4f9, SUC_lexicon_tags_27 },
    { 0x86f8b4fb, SUC_lexicon_tags_47 },
    { 0xdd58b4fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe458b501, SUC_lexicon_tags_37 },
    { 0xbbd8b501, SUC_lexicon_tags_17 },
    { 0xe4d8b503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8378b50b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc238b50e, SUC_lexicon_tags_15 },
    { 0x7a58b50f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb8b514, SUC_lexicon_tags_244 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a78b51a, SUC_lexicon_tags_38 },
    { 0x1c38b51a, SUC_lexicon_tags_27 },
    { 0xabf8b51c, SUC_lexicon_tags_13 },
    { 0x7fd8b51d, SUC_lexicon_tags_7 },
    { 0x5ab8b51d, SUC_lexicon_tags_9 },
    { 0xd9d8b51c, SUC_lexicon_tags_0 },
    { 0xd698b520, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa378b522, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64b8b524, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce38b529, SUC_lexicon_tags_2 },
    { 0x51d8b52a, SUC_lexicon_tags_14 },
    { 0xfab8b52b, SUC_lexicon_tags_30 },
    { 0xbb58b52a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c58b52e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa098b530, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c78b533, SUC_lexicon_tags_34 },
    { 0x1f8b534, SUC_lexicon_tags_2 },
    { 0x3558b535, SUC_lexicon_tags_64 },
    { 0x6278b533, SUC_lexicon_tags_37 },
    { 0x8f78b534, SUC_lexicon_tags_25 },
    { 0xb0f8b538, SUC_lexicon_tags_33 },
    { 0x28f8b539, SUC_lexicon_tags_7 },
    { 0x37d8b539, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb938b542, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xde38b545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb4b8b547, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3298b54a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7018b54e, SUC_lexicon_tags_12 },
    { 0x3678b54f, SUC_lexicon_tags_47 },
    { 0xb318b550, SUC_lexicon_tags_1 },
    { 0x9338b54f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac98b555, SUC_lexicon_tags_11 },
    { 0x82d8b555, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe38b559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9398b55c, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf18b560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x32f8b562, SUC_lexicon_tags_55 },
    { 0xb118b563, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c18b565, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1eb8b567, SUC_lexicon_tags_37 },
    { 0x59b8b568, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7578b56c, SUC_lexicon_tags_28 },
    { 0xfa98b56d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6898b573, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf38b576, SUC_lexicon_tags_46 },
    { 0xc0b8b577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac38b57f, SUC_lexicon_tags_7 },
    { 0x9a18b580, SUC_lexicon_tags_12 },
    { 0xccd8b581, SUC_lexicon_tags_1 },
    { 0x4b58b581, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd8b589, SUC_lexicon_tags_49 },
    { 0xae18b58a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b98b58c, SUC_lexicon_tags_69 },
    { 0xe358b58c, SUC_lexicon_tags_25 },
    { 0x81b8b58e, SUC_lexicon_tags_25 },
    { 0x8958b58f, SUC_lexicon_tags_27 },
    { 0x81f8b590, SUC_lexicon_tags_47 },
    { 0x3ab8b591, SUC_lexicon_tags_7 },
    { 0x1578b590, SUC_lexicon_tags_27 },
    { 0xff78b593, SUC_lexicon_tags_37 },
    { 0x82b8b593, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed8b59d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xee98b59f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b8b5a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb8b5a9, SUC_lexicon_tags_77 },
    { 0x8c18b5aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb38b5af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf8b5b2, SUC_lexicon_tags_7 },
    { 0x80b8b5b2, SUC_lexicon_tags_25 },
    { 0xfb18b5b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f8b5b8, SUC_lexicon_tags_25 },
    { 0xeb78b5b9, SUC_lexicon_tags_63 },
    { 0x2818b5ba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b98b5bd, SUC_lexicon_tags_1 },
    { 0x8d58b5bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4b8b5c0, SUC_lexicon_tags_25 },
    { 0xb3b8b5c1, SUC_lexicon_tags_48 },
    { 0xdc78b5c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf8b5cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b8b5d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7398b5d2, SUC_lexicon_tags_1 },
    { 0x6758b5d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x99f8b5d5, SUC_lexicon_tags_23 },
    { 0x6918b5d6, SUC_lexicon_tags_25 },
    { 0x9338b5d6, SUC_lexicon_tags_9 },
    { 0xbdb8b5d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f8b5db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x638b5dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9778b5e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b98b5e9, SUC_lexicon_tags_11 },
    { 0xafb8b5e9, SUC_lexicon_tags_47 },
    { 0x7838b5eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a58b5f3, SUC_lexicon_tags_46 },
    { 0x73b8b5f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d8b5f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9418b5fa, SUC_lexicon_tags_11 },
    { 0x8f8b5fb, SUC_lexicon_tags_29 },
    { 0x9b18b5fb, SUC_lexicon_tags_45 },
    { 0x3298b5fa, SUC_lexicon_tags_68 },
    { 0x8358b5fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc278b601, SUC_lexicon_tags_1 },
    { 0x6758b602, SUC_lexicon_tags_15 },
    { 0xab38b603, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xad98b605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf218b60d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e38b610, SUC_lexicon_tags_7 },
    { 0x1258b611, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc678b614, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138b61b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb18b620, SUC_lexicon_tags_11 },
    { 0x37d8b620, SUC_lexicon_tags_53 },
    { 0xe598b622, SUC_lexicon_tags_1 },
    { 0x69d8b623, SUC_lexicon_tags_82 },
    { 0xe4d8b620, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf8b627, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7db8b629, SUC_lexicon_tags_61 },
    { 0xf718b62a, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x8c18b62c, SUC_lexicon_tags_37 },
    { 0x81d8b62d, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0x378b630, SUC_lexicon_tags_23 },
    { 0x3f58b631, SUC_lexicon_tags_43 },
    { 0x6ab8b632, SUC_lexicon_tags_43 },
    { 0xd078b632, SUC_lexicon_tags_1 },
    { 0x4278b634, SUC_lexicon_tags_28 },
    { 0x3d78b632, SUC_lexicon_tags_32 },
    { 0xe0f8b635, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3258b638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef8b63b, SUC_lexicon_tags_15 },
    { 0x2118b63c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f8b644, SUC_lexicon_tags_2 },
    { 0x9ed8b645, SUC_lexicon_tags_25 },
    { 0xf8d8b646, SUC_lexicon_tags_19 },
    { 0xbd18b645, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa58b64a, SUC_lexicon_tags_14 },
    { 0xac58b64a, SUC_lexicon_tags_11 },
    { 0x4af8b64c, SUC_lexicon_tags_2 },
    { 0x75d8b64d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bf8b64f, SUC_lexicon_tags_12 },
    { 0xd1b8b650, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb658b652, SUC_lexicon_tags_7 },
    { 0xddd8b653, SUC_lexicon_tags_56 },
    { 0xb458b652, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed8b65e, SUC_lexicon_tags_23 },
    { 0xdb58b65f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2878b661, SUC_lexicon_tags_19 },
    { 0xdaf8b662, SUC_lexicon_tags_118 },
    { 0xccb8b662, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb778b666, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd8b66a, SUC_lexicon_tags_47 },
    { 0x9a98b66b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc38b66f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab8b677, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a78b682, SUC_lexicon_tags_37 },
    { 0xfa58b683, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc258b688, SUC_lexicon_tags_1 },
    { 0x818b689, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e18b68b, SUC_lexicon_tags_17 },
    { 0xf8b8b68c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f18b695, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd8b69b, SUC_lexicon_tags_45 },
    { 0x13b8b69c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4098b69f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af8b6a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf8b6a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b8b6aa, SUC_lexicon_tags_60 },
    { 0xa9d8b6ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf18b6b0, SUC_lexicon_tags_60 },
    { 0xaf8b6b1, SUC_lexicon_tags_7 },
    { 0x5af8b6b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ab8b6b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e58b6b9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfcf8b6bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa038b6be, SUC_lexicon_tags_1 },
    { 0x75f8b6bf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b8b6c2, SUC_lexicon_tags_23 },
    { 0xc698b6c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8598b6cc, SUC_lexicon_tags_6 },
    { 0x6238b6cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd8b6d0, SUC_lexicon_tags_806 },
    { 0x4318b6d1, SUC_lexicon_tags_37 },
    { 0x2b58b6d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf438b6dd, SUC_lexicon_tags_1 },
    { 0x8c38b6dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f8b6e2, SUC_lexicon_tags_1 },
    { 0x2218b6e3, SUC_lexicon_tags_1 },
    { 0x5218b6e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf8b6e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8f8b6e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x338b6ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd58b6ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8038b6f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d8b6f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8618b6fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3c98b6fd, SUC_lexicon_tags_14 },
    { 0xb398b6fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd8b702, SUC_lexicon_tags_2 },
    { 0x6df8b703, SUC_lexicon_tags_33 },
    { 0x5798b702, SUC_lexicon_tags_37 },
    { 0x7f8b702, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3558b70a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf8f8b70c, SUC_lexicon_tags_2 },
    { 0x1d18b70d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb498b712, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb8b717, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8638b71b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2998b720, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x3c58b722, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d8b726, SUC_lexicon_tags_1 },
    { 0x258b726, SUC_lexicon_tags_25 },
    { 0x6af8b727, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf8b72e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x78d8b730, SUC_lexicon_tags_7 },
    { 0xc2d8b730, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f8b740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d78b742, SUC_lexicon_tags_27 },
    { 0x218b743, SUC_lexicon_tags_3 },
    { 0xdcd8b744, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5ed8b746, SUC_lexicon_tags_17 },
    { 0xf9f8b747, SUC_lexicon_tags_45 },
    { 0x73d8b747, SUC_lexicon_tags_47 },
    { 0xb7f8b749, SUC_lexicon_tags_7 },
    { 0xc618b74a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b8b74f, SUC_lexicon_tags_11 },
    { 0xcc38b750, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0x1c78b752, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1938b756, SUC_lexicon_tags_37 },
    { 0xf598b756, SUC_lexicon_tags_11 },
    { 0xbcd8b757, SUC_lexicon_tags_49 },
    { 0x4a58b759, SUC_lexicon_tags_0 },
    { 0x7e18b75a, SUC_lexicon_tags_9 },
    { 0x2838b75b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc518b75d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f8b768, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfb8b76a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df8b771, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe898b774, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8b776, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f18b77d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x4dd8b77f, SUC_lexicon_tags_37 },
    { 0x4918b780, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f8b786, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xafd8b788, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2378b78a, SUC_lexicon_tags_11 },
    { 0x578b78b, SUC_lexicon_tags_1 },
    { 0xc158b78c, SUC_lexicon_tags_37 },
    { 0x4f8b78c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d8b791, SUC_lexicon_tags_43 },
    { 0xe118b792, SUC_lexicon_tags_43 },
    { 0xa538b793, SUC_lexicon_tags_27 },
    { 0x67b8b794, SUC_lexicon_tags_10 },
    { 0xbe78b795, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e58b797, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x50d8b799, SUC_lexicon_tags_14 },
    { 0x8fd8b79a, SUC_lexicon_tags_30 },
    { 0x4f78b79a, SUC_lexicon_tags_11 },
    { 0xbb18b79c, SUC_lexicon_tags_307 },
    { 0x49f8b79d, SUC_lexicon_tags_2 },
    { 0xab58b79d, SUC_lexicon_tags_19 },
    { 0xf638b79c, SUC_lexicon_tags_4 },
    { 0xdd98b79d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1cb8b7a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f58b7ab, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe2b8b7ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa78b7b0, SUC_lexicon_tags_27 },
    { 0x3958b7b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ed8b7b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b8b7b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7578b7ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb8b7bd, SUC_lexicon_tags_25 },
    { 0x2198b7be, SUC_lexicon_tags_37 },
    { 0xc118b7bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2798b7c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d8b7c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3898b7c9, SUC_lexicon_tags_31 },
    { 0xdff8b7c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b78b7cd, SUC_lexicon_tags_15 },
    { 0x2058b7cd, SUC_lexicon_tags_2 },
    { 0x2c98b7cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b8b7d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd778b7d7, SUC_lexicon_tags_37 },
    { 0x4458b7d8, SUC_lexicon_tags_0 },
    { 0x7518b7d7, SUC_lexicon_tags_37 },
    { 0x3b78b7da, SUC_lexicon_tags_3 },
    { 0x298b7d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe98b7e2, SUC_lexicon_tags_37 },
    { 0xa958b7e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e58b7e5, SUC_lexicon_tags_37 },
    { 0x4ab8b7e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f98b7e8, SUC_lexicon_tags_11 },
    { 0x4cf8b7e9, SUC_lexicon_tags_18 },
    { 0x3918b7ea, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb38b7ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb8b7f7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb938b801, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e58b805, SUC_lexicon_tags_757 },
    { 0x99b8b805, SUC_lexicon_tags_16 },
    { 0xfab8b806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6df8b809, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf6b8b80b, SUC_lexicon_tags_15 },
    { 0xfd38b80c, SUC_lexicon_tags_66 },
    { 0x83b8b80c, SUC_lexicon_tags_37 },
    { 0x1478b80d, SUC_lexicon_tags_49 },
    { 0x2398b80c, SUC_lexicon_tags_87 },
    { 0x79b8b810, SUC_lexicon_tags_15 },
    { 0xd1b8b811, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f58b813, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0x618b815, SUC_lexicon_tags_2 },
    { 0xf4d8b816, SUC_lexicon_tags_0 },
    { 0xa8f8b815, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f8b81f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb898b822, SUC_lexicon_tags_17 },
    { 0xdd18b823, SUC_lexicon_tags_8 },
    { 0x9ed8b824, SUC_lexicon_tags_12 },
    { 0x63b8b823, SUC_lexicon_tags_2 },
    { 0x6b38b826, SUC_lexicon_tags_12 },
    { 0x3dd8b826, SUC_lexicon_tags_31 },
    { 0xe258b827, SUC_lexicon_tags_66 },
    { 0x9198b829, SUC_lexicon_tags_33 },
    { 0x5498b82a, SUC_lexicon_tags_25 },
    { 0x398b825, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab8b82e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x878b831, SUC_lexicon_tags_7 },
    { 0x21f8b832, SUC_lexicon_tags_8 },
    { 0x1098b832, SUC_lexicon_tags_21 },
    { 0x5c18b832, SUC_lexicon_tags_7 },
    { 0xd518b835, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2738b837, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d8b83e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd8b847, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3178b84b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f8b84e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48f8b850, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c98b853, SUC_lexicon_tags_1 },
    { 0xb58b854, SUC_lexicon_tags_14 },
    { 0xc4b8b855, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xba58b857, SUC_lexicon_tags_35 },
    { 0xd358b858, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad58b85d, SUC_lexicon_tags_57 },
    { 0xf478b85e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7758b860, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc38b864, SUC_lexicon_tags_26 },
    { 0xc758b865, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf4d8b867, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdb18b869, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbc58b86b, SUC_lexicon_tags_46 },
    { 0x9198b86c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd238b86e, SUC_lexicon_tags_2 },
    { 0x2ef8b86f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b8b875, SUC_lexicon_tags_87 },
    { 0xcab8b875, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8558b878, SUC_lexicon_tags_3 },
    { 0x13d8b878, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8bd8b87b, SUC_lexicon_tags_11 },
    { 0x9b38b87c, SUC_lexicon_tags_37 },
    { 0x6c98b87d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a98b881, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x6358b884, SUC_lexicon_tags_28 },
    { 0x4018b884, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d8b889, SUC_lexicon_tags_19 },
    { 0x99f8b88a, SUC_lexicon_tags_17 },
    { 0x83d8b88b, SUC_lexicon_tags_15 },
    { 0xa158b88a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x938b88e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5158b891, SUC_lexicon_tags_7 },
    { 0xb6d8b891, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc758b894, SUC_lexicon_tags_1 },
    { 0xfcd8b895, SUC_lexicon_tags_45 },
    { 0xddf8b896, SUC_lexicon_tags_38 },
    { 0xba18b896, SUC_lexicon_tags_47 },
    { 0xa178b895, SUC_lexicon_tags_14 },
    { 0xdb58b899, SUC_lexicon_tags_47 },
    { 0x6158b89a, SUC_lexicon_tags_46 },
    { 0xfd18b89b, SUC_lexicon_tags_29 },
    { 0x4898b897, SUC_lexicon_tags_27 },
    { 0x3ff8b89d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9298b8a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2778b8a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa238b8a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2898b8ab, SUC_lexicon_tags_100 },
    { 0xad78b8ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43d8b8ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e18b8b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb8b8b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2818b8b8, SUC_lexicon_tags_1 },
    { 0xb658b8b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef8b8c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbef8b8c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4998b8c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5238b8c7, SUC_lexicon_tags_37 },
    { 0xb838b8c8, SUC_lexicon_tags_7 },
    { 0x7e18b8c8, SUC_lexicon_tags_17 },
    { 0x29b8b8ca, SUC_lexicon_tags_7 },
    { 0xe698b8c8, SUC_lexicon_tags_69 },
    { 0x8218b8cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x12b8b8ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xba58b8d1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8998b8d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4098b8db, SUC_lexicon_tags_42 },
    { 0xf398b8db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b8b8df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9318b8e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b8b8e7, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x9978b8e9, SUC_lexicon_tags_29 },
    { 0x4e78b8ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ab8b8ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3038b8f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3538b8f8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8958b8fc, SUC_lexicon_tags_17 },
    { 0xe238b8fc, SUC_lexicon_tags_17 },
    { 0xd758b8fe, SUC_lexicon_tags_25 },
    { 0xc8b8b8fe, SUC_lexicon_tags_23 },
    { 0x74b8b8fe, SUC_lexicon_tags_2 },
    { 0x7b98b8fc, SUC_lexicon_tags_15 },
    { 0x7af8b8ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf478b906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc918b90f, SUC_lexicon_tags_29 },
    { 0xdcd8b910, SUC_lexicon_tags_11 },
    { 0xb2b8b90f, SUC_lexicon_tags_30 },
    { 0x9e38b912, SUC_lexicon_tags_56 },
    { 0x7258b913, SUC_lexicon_tags_19 },
    { 0x9878b90f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4918b916, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xcf38b918, SUC_lexicon_tags_39 },
    { 0xfb98b918, SUC_lexicon_tags_11 },
    { 0xa758b91a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6658b91d, SUC_lexicon_tags_39 },
    { 0x7698b91e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8d8b920, SUC_lexicon_tags_38 },
    { 0x1d38b920, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86b8b924, SUC_lexicon_tags_15 },
    { 0xcf78b925, SUC_lexicon_tags_37 },
    { 0x3a78b924, SUC_lexicon_tags_47 },
    { 0x6218b924, SUC_lexicon_tags_30 },
    { 0x4238b928, SUC_lexicon_tags_45 },
    { 0x2c38b929, SUC_lexicon_tags_16 },
    { 0x2758b924, SUC_lexicon_tags_4 },
    { 0x9af8b92a, SUC_lexicon_tags_15 },
    { 0x5318b92c, SUC_lexicon_tags_12 },
    { 0xc6f8b92b, SUC_lexicon_tags_31 },
    { 0x5d38b92e, SUC_lexicon_tags_60 },
    { 0x1a58b92c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e78b932, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf8b935, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7338b939, SUC_lexicon_tags_38 },
    { 0xfed8b93a, SUC_lexicon_tags_11 },
    { 0x38d8b93b, SUC_lexicon_tags_1 },
    { 0x76b8b93c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d8b940, SUC_lexicon_tags_25 },
    { 0xfe58b941, SUC_lexicon_tags_2 },
    { 0xdfd8b940, SUC_lexicon_tags_31 },
    { 0xeb58b943, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98b946, SUC_lexicon_tags_83 },
    { 0xac38b947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18b94b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xc5d8b94d, SUC_lexicon_tags_1 },
    { 0x60b8b94e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d58b952, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf8b956, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4078b958, SUC_lexicon_tags_2 },
    { 0x3c98b959, SUC_lexicon_tags_33 },
    { 0x6898b95a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab8b95c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5898b960, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x42b8b962, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xef58b964, SUC_lexicon_tags_9 },
    { 0x6098b964, SUC_lexicon_tags_47 },
    { 0xd0d8b966, SUC_lexicon_tags_19 },
    { 0xa7f8b965, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1c58b969, SUC_lexicon_tags_17 },
    { 0xb798b96a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x438b96c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d8b96f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc478b972, SUC_lexicon_tags_11 },
    { 0xf598b972, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b18b977, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec78b97e, SUC_lexicon_tags_34 },
    { 0x78d8b97f, SUC_lexicon_tags_45 },
    { 0xa0f8b980, SUC_lexicon_tags_38 },
    { 0xe98b97e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd8b984, SUC_lexicon_tags_23 },
    { 0xe1b8b985, SUC_lexicon_tags_7 },
    { 0xd898b986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e98b98a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9b38b98c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1858b991, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac78b996, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa38b99c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8118b9a1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f38b9a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5238b9a8, SUC_lexicon_tags_17 },
    { 0x6418b9a9, SUC_lexicon_tags_13 },
    { 0xde78b9aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d8b9ae, SUC_lexicon_tags_23 },
    { 0x4fb8b9af, SUC_lexicon_tags_25 },
    { 0xc9f8b9b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe18b9b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218b9b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f8b9bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7838b9bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd438b9c9, SUC_lexicon_tags_37 },
    { 0x758b9ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3298b9ce, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b18b9d1, SUC_lexicon_tags_37 },
    { 0x6998b9d2, SUC_lexicon_tags_15 },
    { 0xe7d8b9d3, SUC_lexicon_tags_9 },
    { 0xd5d8b9d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfad8b9d6, SUC_lexicon_tags_25 },
    { 0xab8b9d7, SUC_lexicon_tags_0 },
    { 0x2b78b9d8, SUC_lexicon_tags_15 },
    { 0xdcb8b9d9, SUC_lexicon_tags_37 },
    { 0x91f8b9d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb238b9dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8798b9df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe518b9e6, SUC_lexicon_tags_25 },
    { 0xfa78b9e7, SUC_lexicon_tags_31 },
    { 0x84b8b9e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x518b9ef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc998b9f2, SUC_lexicon_tags_49 },
    { 0xa858b9f2, SUC_lexicon_tags_12 },
    { 0x1158b9f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb8b9f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42f8b9fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc238b9ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d8ba03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5858ba05, SUC_lexicon_tags_37 },
    { 0x1478ba06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc838ba08, SUC_lexicon_tags_11 },
    { 0xed18ba09, SUC_lexicon_tags_49 },
    { 0x98f8ba08, SUC_lexicon_tags_0 },
    { 0xbd58ba0b, SUC_lexicon_tags_60 },
    { 0xfe58ba08, SUC_lexicon_tags_30 },
    { 0x19b8ba0a, SUC_lexicon_tags_11 },
    { 0xb1f8ba0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8098ba10, SUC_lexicon_tags_1 },
    { 0xf9b8ba11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e78ba17, SUC_lexicon_tags_38 },
    { 0x858ba17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc0b8ba1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6278ba1c, SUC_lexicon_tags_68 },
    { 0xddf8ba1d, SUC_lexicon_tags_15 },
    { 0x93b8ba1d, SUC_lexicon_tags_10 },
    { 0xe498ba1f, SUC_lexicon_tags_37 },
    { 0xae98ba20, SUC_lexicon_tags_17 },
    { 0xf938ba20, SUC_lexicon_tags_0 },
    { 0xead8ba1c, SUC_lexicon_tags_17 },
    { 0x398ba22, SUC_lexicon_tags_7 },
    { 0x6ef8ba24, SUC_lexicon_tags_0 },
    { 0xac38ba25, SUC_lexicon_tags_47 },
    { 0x7a98ba1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6638ba28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe678ba2d, SUC_lexicon_tags_37 },
    { 0x18f8ba2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa698ba31, SUC_lexicon_tags_31 },
    { 0xea38ba32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd8ba37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d8ba3c, SUC_lexicon_tags_38 },
    { 0x1e78ba3d, SUC_lexicon_tags_16 },
    { 0x9a78ba3d, SUC_lexicon_tags_30 },
    { 0x7c58ba3d, SUC_lexicon_tags_25 },
    { 0xa458ba3d, SUC_lexicon_tags_37 },
    { 0x25f8ba41, SUC_lexicon_tags_12 },
    { 0x4dd8ba40, SUC_lexicon_tags_38 },
    { 0x32d8ba43, SUC_lexicon_tags_12 },
    { 0x5ab8ba44, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d8ba49, SUC_lexicon_tags_8 },
    { 0x37b8ba4a, SUC_lexicon_tags_12 },
    { 0x2518ba4a, SUC_lexicon_tags_27 },
    { 0xfd98ba4b, SUC_lexicon_tags_6 },
    { 0x3fd8ba4b, SUC_lexicon_tags_9 },
    { 0xaab8ba4e, SUC_lexicon_tags_17 },
    { 0x76f8ba4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef8ba52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad8ba57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x218ba5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6178ba5c, SUC_lexicon_tags_13 },
    { 0x6478ba5d, SUC_lexicon_tags_12 },
    { 0xf8f8ba5d, SUC_lexicon_tags_11 },
    { 0xedf8ba5f, SUC_lexicon_tags_17 },
    { 0x5898ba5f, SUC_lexicon_tags_27 },
    { 0xefb8ba5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4178ba64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf58ba68, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb278ba6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3458ba6e, SUC_lexicon_tags_43 },
    { 0x9658ba6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7738ba75, SUC_lexicon_tags_7 },
    { 0x9b8ba76, SUC_lexicon_tags_1 },
    { 0x4a18ba76, SUC_lexicon_tags_11 },
    { 0x16d8ba78, SUC_lexicon_tags_15 },
    { 0x998ba78, SUC_lexicon_tags_37 },
    { 0x2778ba7a, SUC_lexicon_tags_15 },
    { 0x8a78ba7b, SUC_lexicon_tags_1 },
    { 0x8798ba7c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7138ba80, SUC_lexicon_tags_47 },
    { 0xd38ba81, SUC_lexicon_tags_12 },
    { 0x4e98ba82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa398ba87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb658ba89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d8ba8c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe058ba90, SUC_lexicon_tags_0 },
    { 0x2fd8ba91, SUC_lexicon_tags_3 },
    { 0x2d58ba92, SUC_lexicon_tags_33 },
    { 0x6cb8ba93, SUC_lexicon_tags_2 },
    { 0x3538ba94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae98ba97, SUC_lexicon_tags_7 },
    { 0x6378ba98, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec78ba9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa458ba9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe758baa0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa998baa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd918baa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f8baaf, SUC_lexicon_tags_10 },
    { 0x9a18bab0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef8bab3, SUC_lexicon_tags_30 },
    { 0x8958bab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf498bacb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d78bad0, SUC_lexicon_tags_25 },
    { 0xcb8bad1, SUC_lexicon_tags_25 },
    { 0x5c58bad2, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd38bad8, SUC_lexicon_tags_17 },
    { 0x12b8bad9, SUC_lexicon_tags_17 },
    { 0x9798bad9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b8badd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb058bae3, SUC_lexicon_tags_28 },
    { 0x6e98bae4, SUC_lexicon_tags_8 },
    { 0x8a38bae4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9b8bae7, SUC_lexicon_tags_10 },
    { 0x9618bae7, SUC_lexicon_tags_21 },
    { 0x858bae8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef8baee, SUC_lexicon_tags_38 },
    { 0xb418baee, SUC_lexicon_tags_8 },
    { 0x7fb8baf0, SUC_lexicon_tags_8 },
    { 0x72f8baf1, SUC_lexicon_tags_48 },
    { 0x1f78baf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b8baf9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab8bafe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7638bb03, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xe098bb05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9458bb09, SUC_lexicon_tags_15 },
    { 0xec78bb0a, SUC_lexicon_tags_47 },
    { 0xa118bb0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb858bb0d, SUC_lexicon_tags_179 },
    { 0xe5d8bb0d, SUC_lexicon_tags_40 },
    { 0xd098bb0f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xefb8bb11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3978bb13, SUC_lexicon_tags_11 },
    { 0x758bb13, SUC_lexicon_tags_11 },
    { 0xf198bb13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39b8bb17, SUC_lexicon_tags_1 },
    { 0xb878bb18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3718bb1d, SUC_lexicon_tags_14 },
    { 0xcd58bb1d, SUC_lexicon_tags_14 },
    { 0x4798bb1d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2698bb22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdb8bb24, SUC_lexicon_tags_29 },
    { 0x75d8bb25, SUC_lexicon_tags_17 },
    { 0x5d98bb24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff8bb2d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x78d8bb30, SUC_lexicon_tags_68 },
    { 0xc038bb30, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f8bb35, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x56b8bb37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3138bb3a, SUC_lexicon_tags_3 },
    { 0xfe18bb3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf2b8bb3d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a78bb40, SUC_lexicon_tags_38 },
    { 0x31d8bb41, SUC_lexicon_tags_25 },
    { 0x7b8bb42, SUC_lexicon_tags_11 },
    { 0x9918bb42, SUC_lexicon_tags_15 },
    { 0xcb8bb42, SUC_lexicon_tags_23 },
    { 0xca98bb45, SUC_lexicon_tags_15 },
    { 0x4198bb46, SUC_lexicon_tags_17 },
    { 0x2318bb47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d8bb4a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb8bb4e, SUC_lexicon_tags_33 },
    { 0x2938bb4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5778bb52, SUC_lexicon_tags_33 },
    { 0x338bb53, SUC_lexicon_tags_254 },
    { 0xe578bb54, SUC_lexicon_tags_27 },
    { 0x61b8bb52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b8bb5a, SUC_lexicon_tags_12 },
    { 0x3998bb5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8bb60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb498bb63, SUC_lexicon_tags_14 },
    { 0xeb18bb63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14d8bb66, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db8bb69, SUC_lexicon_tags_91 },
    { 0x698bb6a, SUC_lexicon_tags_29 },
    { 0xd4f8bb6b, SUC_lexicon_tags_7 },
    { 0x5418bb69, SUC_lexicon_tags_46 },
    { 0x38d8bb6a, SUC_lexicon_tags_141 },
    { 0x3db8bb6b, SUC_lexicon_tags_2 },
    { 0xff8bb6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf738bb74, SUC_lexicon_tags_33 },
    { 0x64f8bb75, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f8bb7d, SUC_lexicon_tags_7 },
    { 0x52d8bb7e, SUC_lexicon_tags_49 },
    { 0xd478bb7f, SUC_lexicon_tags_0 },
    { 0x65d8bb80, SUC_lexicon_tags_15 },
    { 0xb9d8bb7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd58bb86, SUC_lexicon_tags_24 },
    { 0xc578bb87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73b8bb89, SUC_lexicon_tags_27 },
    { 0xaf58bb89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x758bb8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd8bb91, SUC_lexicon_tags_2 },
    { 0x1858bb92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5118bb94, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8bb98, SUC_lexicon_tags_38 },
    { 0xdfd8bb99, SUC_lexicon_tags_22 },
    { 0xa6b8bb9a, SUC_lexicon_tags_45 },
    { 0xcf18bb9a, SUC_lexicon_tags_47 },
    { 0x5658bb99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d8bba9, SUC_lexicon_tags_30 },
    { 0x1cb8bbaa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9eb8bbac, SUC_lexicon_tags_17 },
    { 0x9b8bbad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd418bbb4, SUC_lexicon_tags_43 },
    { 0x9eb8bbb4, SUC_lexicon_tags_47 },
    { 0xa738bbb6, SUC_lexicon_tags_27 },
    { 0x4f98bbb7, SUC_lexicon_tags_1 },
    { 0xc618bbb7, SUC_lexicon_tags_136 },
    { 0xf418bbb7, SUC_lexicon_tags_4 },
    { 0xeb78bbb8, SUC_lexicon_tags_25 },
    { 0x3658bbb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5998bbbd, SUC_lexicon_tags_15 },
    { 0x1138bbbd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc38bbc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81d8bbca, SUC_lexicon_tags_15 },
    { 0xcf98bbca, SUC_lexicon_tags_11 },
    { 0x97d8bbcc, SUC_lexicon_tags_11 },
    { 0x1958bbcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f8bbd0, SUC_lexicon_tags_17 },
    { 0x1238bbd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2798bbd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6998bbd7, SUC_lexicon_tags_17 },
    { 0x43f8bbd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf98bbda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba98bbdc, SUC_lexicon_tags_50 },
    { 0xeed8bbdc, SUC_lexicon_tags_15 },
    { 0x21d8bbdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8978bbe3, SUC_lexicon_tags_13 },
    { 0x5838bbe4, SUC_lexicon_tags_11 },
    { 0x3778bbe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd558bbeb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3a78bbed, SUC_lexicon_tags_13 },
    { 0xb918bbee, SUC_lexicon_tags_16 },
    { 0x2af8bbef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1b8bbf1, SUC_lexicon_tags_29 },
    { 0xa78bbf2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef8bbf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8bbfd, SUC_lexicon_tags_30 },
    { 0x8f98bbfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b8bc01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5098bc05, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9d78bc07, SUC_lexicon_tags_129 },
    { 0x3778bc08, SUC_lexicon_tags_27 },
    { 0xac18bc08, SUC_lexicon_tags_15 },
    { 0xa938bc0a, SUC_lexicon_tags_25 },
    { 0x4d18bc07, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc458bc0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1898bc12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7938bc16, SUC_lexicon_tags_11 },
    { 0xd178bc16, SUC_lexicon_tags_62 },
    { 0xdcb8bc17, SUC_lexicon_tags_15 },
    { 0x6fb8bc19, SUC_lexicon_tags_12 },
    { 0x9838bc1a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78bc1d, SUC_lexicon_tags_9 },
    { 0xeb8bc1d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d8bc23, SUC_lexicon_tags_17 },
    { 0x7558bc23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d98bc26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5698bc2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf8bc30, SUC_lexicon_tags_25 },
    { 0xf538bc30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2858bc35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9cf8bc37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9838bc3a, SUC_lexicon_tags_38 },
    { 0x2558bc3b, SUC_lexicon_tags_61 },
    { 0xb838bc3a, SUC_lexicon_tags_47 },
    { 0xc0b8bc3d, SUC_lexicon_tags_9 },
    { 0x7c78bc3d, SUC_lexicon_tags_9 },
    { 0x8f38bc3f, SUC_lexicon_tags_17 },
    { 0xdfd8bc3f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7558bc43, SUC_lexicon_tags_0 },
    { 0xb078bc43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3f8bc46, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x818bc49, SUC_lexicon_tags_27 },
    { 0xe218bc4a, SUC_lexicon_tags_1 },
    { 0xc058bc49, SUC_lexicon_tags_29 },
    { 0x1df8bc4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9e58bc4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8d38bc50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd838bc54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x45b8bc56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3718bc5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4cb8bc5c, SUC_lexicon_tags_29 },
    { 0x1158bc5d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf358bc61, SUC_lexicon_tags_2 },
    { 0x1958bc62, SUC_lexicon_tags_2 },
    { 0xcfd8bc61, SUC_lexicon_tags_1 },
    { 0x1858bc63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2e78bc66, SUC_lexicon_tags_31 },
    { 0x35d8bc67, SUC_lexicon_tags_19 },
    { 0xe9d8bc68, SUC_lexicon_tags_37 },
    { 0x17b8bc69, SUC_lexicon_tags_23 },
    { 0xd038bc6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b8bc6e, SUC_lexicon_tags_17 },
    { 0xf458bc6f, SUC_lexicon_tags_25 },
    { 0xacd8bc70, SUC_lexicon_tags_17 },
    { 0x2738bc71, SUC_lexicon_tags_63 },
    { 0x1578bc6f, SUC_lexicon_tags_43 },
    { 0xbc18bc70, SUC_lexicon_tags_11 },
    { 0x8838bc6f, SUC_lexicon_tags_7 },
    { 0xb3d8bc75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7898bc78, SUC_lexicon_tags_38 },
    { 0xf3b8bc79, SUC_lexicon_tags_2 },
    { 0xb538bc79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52b8bc7c, SUC_lexicon_tags_7 },
    { 0xe038bc7d, SUC_lexicon_tags_29 },
    { 0xeb78bc7e, SUC_lexicon_tags_700 },
    { 0xb918bc7e, SUC_lexicon_tags_23 },
    { 0x2cd8bc80, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x3518bc83, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb8bc86, SUC_lexicon_tags_63 },
    { 0x3078bc87, SUC_lexicon_tags_320 },
    { 0x7c98bc87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b8bc8a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xee98bc8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc358bc94, SUC_lexicon_tags_17 },
    { 0xc098bc94, SUC_lexicon_tags_17 },
    { 0x6fd8bc95, SUC_lexicon_tags_37 },
    { 0xd3d8bc97, SUC_lexicon_tags_17 },
    { 0x2138bc96, SUC_lexicon_tags_62 },
    { 0x7cf8bc99, SUC_lexicon_tags_14 },
    { 0xf8f8bc96, SUC_lexicon_tags_37 },
    { 0x91f8bc9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd8bc9d, SUC_lexicon_tags_37 },
    { 0x87d8bc9d, SUC_lexicon_tags_11 },
    { 0xdaf8bc9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8718bca4, SUC_lexicon_tags_4 },
    { 0xf518bca4, SUC_lexicon_tags_23 },
    { 0xb1b8bca4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9798bca9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8038bcae, SUC_lexicon_tags_0 },
    { 0xe098bcaf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed8bcb3, SUC_lexicon_tags_37 },
    { 0x8e58bcb3, SUC_lexicon_tags_7 },
    { 0x8518bcb4, SUC_lexicon_tags_13 },
    { 0x7098bcb6, SUC_lexicon_tags_2 },
    { 0x8078bcb7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc98bcb9, SUC_lexicon_tags_1 },
    { 0x7238bcba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a58bcbd, SUC_lexicon_tags_43 },
    { 0x1078bcbd, SUC_lexicon_tags_11 },
    { 0xdcd8bcbd, SUC_lexicon_tags_27 },
    { 0x5898bcc0, SUC_lexicon_tags_45 },
    { 0x3d78bcc1, SUC_lexicon_tags_37 },
    { 0x5918bcc2, SUC_lexicon_tags_87 },
    { 0x6f8bcc0, SUC_lexicon_tags_7 },
    { 0xa578bcc1, SUC_lexicon_tags_11 },
    { 0x7a78bcc5, SUC_lexicon_tags_31 },
    { 0xcb8bcc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c78bcc9, SUC_lexicon_tags_33 },
    { 0x5338bcca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc278bccf, SUC_lexicon_tags_5 },
    { 0x2cb8bcd0, SUC_lexicon_tags_12 },
    { 0x55f8bccf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4558bcd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74d8bcd5, SUC_lexicon_tags_47 },
    { 0xf58bcd6, SUC_lexicon_tags_7 },
    { 0x1db8bcd6, SUC_lexicon_tags_37 },
    { 0x9098bcd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8218bcda, SUC_lexicon_tags_11 },
    { 0xb5f8bcdb, SUC_lexicon_tags_11 },
    { 0x52d8bcdb, SUC_lexicon_tags_60 },
    { 0xfff8bcdd, SUC_lexicon_tags_15 },
    { 0x598bcde, SUC_lexicon_tags_1 },
    { 0xbb18bcde, SUC_lexicon_tags_7 },
    { 0x6e58bce0, SUC_lexicon_tags_37 },
    { 0x6198bce0, SUC_lexicon_tags_43 },
    { 0xe0f8bcde, SUC_lexicon_tags_7 },
    { 0x1258bcdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f8bce9, SUC_lexicon_tags_82 },
    { 0x7818bce9, SUC_lexicon_tags_63 },
    { 0x2f18bceb, SUC_lexicon_tags_1 },
    { 0x62b8bce9, SUC_lexicon_tags_11 },
    { 0x2fb8bced, SUC_lexicon_tags_37 },
    { 0x21d8bcee, SUC_lexicon_tags_38 },
    { 0xff8bcef, SUC_lexicon_tags_25 },
    { 0x28f8bcf0, SUC_lexicon_tags_9 },
    { 0x83d8bcea, SUC_lexicon_tags_38 },
    { 0x4d38bcf0, SUC_lexicon_tags_30 },
    { 0x4818bcf2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5298bcfc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x94f8bcfe, SUC_lexicon_tags_37 },
    { 0x8df8bcff, SUC_lexicon_tags_8 },
    { 0xd9f8bd00, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b8bd03, SUC_lexicon_tags_1 },
    { 0x6838bd04, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f78bd07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f8bd0c, SUC_lexicon_tags_25 },
    { 0x8a18bd0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd438bd13, SUC_lexicon_tags_15 },
    { 0x6658bd14, SUC_lexicon_tags_37 },
    { 0x9278bd15, SUC_lexicon_tags_1 },
    { 0x9cd8bd16, SUC_lexicon_tags_2 },
    { 0x2f38bd17, SUC_lexicon_tags_7 },
    { 0xe118bd15, SUC_lexicon_tags_11 },
    { 0xa838bd19, SUC_lexicon_tags_0 },
    { 0x13d8bd16, SUC_lexicon_tags_19 },
    { 0xddb8bd1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4d8bd1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb58bd21, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1358bd25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f8bd28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ef8bd2a, SUC_lexicon_tags_8 },
    { 0xb118bd2b, SUC_lexicon_tags_26 },
    { 0xfc38bd2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10f8bd2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ad8bd30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xebd8bd32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c98bd34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd58bd38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5138bd3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b8bd3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa538bd43, SUC_lexicon_tags_11 },
    { 0xf5b8bd44, SUC_lexicon_tags_11 },
    { 0x1638bd45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd8bd49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd8bd4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f8bd4e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe978bd51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb078bd53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b8bd59, SUC_lexicon_tags_26 },
    { 0x1018bd59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd8bd5f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bd64, SUC_lexicon_tags_1 },
    { 0xbdf8bd64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2058bd68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad78bd6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd18bd6e, SUC_lexicon_tags_19 },
    { 0x2a58bd6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe78bd71, SUC_lexicon_tags_60 },
    { 0x9878bd72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x47d8bd74, SUC_lexicon_tags_15 },
    { 0xb058bd75, SUC_lexicon_tags_47 },
    { 0xc6b8bd75, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec78bd7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b8bd7f, SUC_lexicon_tags_25 },
    { 0x4458bd80, SUC_lexicon_tags_17 },
    { 0x2618bd81, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x85f8bd84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d8bd8a, SUC_lexicon_tags_7 },
    { 0xc7f8bd8b, SUC_lexicon_tags_17 },
    { 0x2d18bd8b, SUC_lexicon_tags_46 },
    { 0x4398bd8d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbe38bd8f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e38bd96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d8bd9d, SUC_lexicon_tags_37 },
    { 0x53f8bd9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xed38bda0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd8bda3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd98bda6, SUC_lexicon_tags_6 },
    { 0xce58bda7, SUC_lexicon_tags_45 },
    { 0x1b78bda8, SUC_lexicon_tags_4 },
    { 0x56b8bda8, SUC_lexicon_tags_24 },
    { 0xad98bda6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c98bdad, SUC_lexicon_tags_65 },
    { 0x9498bdae, SUC_lexicon_tags_1 },
    { 0x7438bdaf, SUC_lexicon_tags_17 },
    { 0xc6f8bdb0, SUC_lexicon_tags_31 },
    { 0x1698bdb1, SUC_lexicon_tags_14 },
    { 0x2498bdb0, SUC_lexicon_tags_37 },
    { 0xed18bdb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab8bdb6, SUC_lexicon_tags_48 },
    { 0x2f78bdb7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd298bdc2, SUC_lexicon_tags_12 },
    { 0x1858bdc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1018bdc5, SUC_lexicon_tags_32 },
    { 0xc418bdc5, SUC_lexicon_tags_14 },
    { 0x75f8bdc7, SUC_lexicon_tags_12 },
    { 0xa1d8bdc7, SUC_lexicon_tags_65 },
    { 0xa618bdc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d8bdcb, SUC_lexicon_tags_15 },
    { 0x5c18bdcc, SUC_lexicon_tags_37 },
    { 0x7458bdcd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5818bdd0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4d18bdd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d8bdd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f58bdda, SUC_lexicon_tags_46 },
    { 0x8818bdda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7b8bddd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2cd8bddf, SUC_lexicon_tags_26 },
    { 0x37f8bde0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d8bde4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc18bde7, SUC_lexicon_tags_11 },
    { 0xba78bde7, SUC_lexicon_tags_38 },
    { 0x1198bde8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f8bded, SUC_lexicon_tags_44 },
    { 0x1078bded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf758bdf1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x35b8bdf3, SUC_lexicon_tags_87 },
    { 0x71f8bdf4, SUC_lexicon_tags_37 },
    { 0x9cf8bdf5, SUC_lexicon_tags_11 },
    { 0x1c38bdf5, SUC_lexicon_tags_23 },
    { 0xfb8bdf7, SUC_lexicon_tags_49 },
    { 0xcaf8bdf8, SUC_lexicon_tags_38 },
    { 0xc8b8bdf9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a58bdfb, SUC_lexicon_tags_47 },
    { 0x6a18bdfc, SUC_lexicon_tags_0 },
    { 0x4b78bdfd, SUC_lexicon_tags_1 },
    { 0x88d8bdfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd878be00, SUC_lexicon_tags_2 },
    { 0x7218be01, SUC_lexicon_tags_2 },
    { 0x6758be02, SUC_lexicon_tags_10 },
    { 0x9cb8be02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x298be07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4f8be09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7758be0e, SUC_lexicon_tags_56 },
    { 0x6138be0f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6678be18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c38be1f, SUC_lexicon_tags_75 },
    { 0x5538be1f, SUC_lexicon_tags_15 },
    { 0x6fd8be21, SUC_lexicon_tags_1 },
    { 0xb098be1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b58be24, SUC_lexicon_tags_87 },
    { 0x69f8be25, SUC_lexicon_tags_23 },
    { 0x4c18be26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd8be2a, SUC_lexicon_tags_2 },
    { 0x55b8be2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a98be2f, SUC_lexicon_tags_45 },
    { 0x91b8be30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f8be32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb8be38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd98be3c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x7458be3f, SUC_lexicon_tags_2 },
    { 0xd858be40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8be48, SUC_lexicon_tags_2 },
    { 0x6e98be49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c78be4e, SUC_lexicon_tags_82 },
    { 0x3a98be4e, SUC_lexicon_tags_11 },
    { 0x6818be4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd8be55, SUC_lexicon_tags_57 },
    { 0xb98be56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6298be5b, SUC_lexicon_tags_24 },
    { 0x59f8be5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4998be5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb8be61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b8be67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdff8be69, SUC_lexicon_tags_1 },
    { 0xaf18be6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x34f8be6c, SUC_lexicon_tags_62 },
    { 0x39d8be6c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6878be71, SUC_lexicon_tags_38 },
    { 0x49d8be72, SUC_lexicon_tags_7 },
    { 0x1bb8be72, SUC_lexicon_tags_7 },
    { 0xe8d8be71, SUC_lexicon_tags_12 },
    { 0xf738be73, SUC_lexicon_tags_7 },
    { 0x5258be74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe058be78, SUC_lexicon_tags_8 },
    { 0xbd78be78, SUC_lexicon_tags_26 },
    { 0x5ed8be78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe838be7d, SUC_lexicon_tags_12 },
    { 0x6458be7e, SUC_lexicon_tags_11 },
    { 0xfdb8be7f, SUC_lexicon_tags_49 },
    { 0x6c38be7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x88f8be82, SUC_lexicon_tags_33 },
    { 0x598be82, SUC_lexicon_tags_11 },
    { 0x7878be82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d8be8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa618be8f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f18be92, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xc758be94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad8be97, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfdd8be99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b8be9d, SUC_lexicon_tags_27 },
    { 0x3bb8be9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf8bea3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc338bea5, SUC_lexicon_tags_2 },
    { 0xf2f8bea6, SUC_lexicon_tags_63 },
    { 0x1458bea7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x6b38bea9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f18beac, SUC_lexicon_tags_7 },
    { 0x6298bead, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6218beb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5778beb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e98beb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb578beb9, SUC_lexicon_tags_11 },
    { 0xfb78beba, SUC_lexicon_tags_43 },
    { 0xbd58bebb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8f58bebd, SUC_lexicon_tags_53 },
    { 0x9c58bebd, SUC_lexicon_tags_25 },
    { 0xee78bebf, SUC_lexicon_tags_37 },
    { 0xf318bec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa638bec4, SUC_lexicon_tags_25 },
    { 0x9478bec4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c98beca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd8bed4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed8bed7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8bed9, SUC_lexicon_tags_27 },
    { 0x7158beda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xee78bedc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd58bede, SUC_lexicon_tags_21 },
    { 0x8118bede, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b8bee2, SUC_lexicon_tags_11 },
    { 0x2ef8bee2, SUC_lexicon_tags_113 },
    { 0x6b38bee4, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xffb8bee6, SUC_lexicon_tags_6 },
    { 0xae98bee7, SUC_lexicon_tags_25 },
    { 0xcbb8bee8, SUC_lexicon_tags_37 },
    { 0x2698bee8, SUC_lexicon_tags_15 },
    { 0xdc78bee7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae18beee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6af8bef6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b18befc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x80f8befe, SUC_lexicon_tags_8 },
    { 0x3b98beff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb998bf04, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd8bf07, SUC_lexicon_tags_33 },
    { 0xcf98bf08, SUC_lexicon_tags_2 },
    { 0xc7b8bf09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd758bf10, SUC_lexicon_tags_47 },
    { 0xf978bf10, SUC_lexicon_tags_23 },
    { 0xeb78bf12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed8bf17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf158bf19, SUC_lexicon_tags_17 },
    { 0x4fd8bf19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f98bf1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85b8bf1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf098bf27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ff8bf29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x12d8bf2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d8bf2f, SUC_lexicon_tags_39 },
    { 0x7138bf30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4618bf32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe918bf34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f8bf3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64d8bf3e, SUC_lexicon_tags_69 },
    { 0xf8f8bf3f, SUC_lexicon_tags_25 },
    { 0x3fb8bf40, SUC_lexicon_tags_25 },
    { 0xaf58bf40, SUC_lexicon_tags_27 },
    { 0x2958bf41, SUC_lexicon_tags_17 },
    { 0x2f18bf43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28f8bf45, SUC_lexicon_tags_29 },
    { 0xd238bf46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b8bf49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4698bf4e, SUC_lexicon_tags_57 },
    { 0x9318bf4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe638bf51, SUC_lexicon_tags_14 },
    { 0x91d8bf52, SUC_lexicon_tags_2 },
    { 0x538bf53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3058bf55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa378bf5a, SUC_lexicon_tags_12 },
    { 0xfe58bf5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda18bf5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6138bf60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878bf65, SUC_lexicon_tags_2 },
    { 0xfdf8bf66, SUC_lexicon_tags_15 },
    { 0xe218bf65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc738bf69, SUC_lexicon_tags_38 },
    { 0x7538bf69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f78bf71, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7df8bf74, SUC_lexicon_tags_12 },
    { 0x9538bf75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8bf78, SUC_lexicon_tags_45 },
    { 0xbf8bf79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8bf7f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x65f8bf81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd18bf83, SUC_lexicon_tags_30 },
    { 0x658bf84, SUC_lexicon_tags_0 },
    { 0x9af8bf85, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1bb8bf87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef8bf8a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3678bf8e, SUC_lexicon_tags_23 },
    { 0xca98bf8f, SUC_lexicon_tags_1 },
    { 0x1838bf8e, SUC_lexicon_tags_26 },
    { 0xf118bf91, SUC_lexicon_tags_30 },
    { 0xb138bf8f, SUC_lexicon_tags_32 },
    { 0x5ed8bf93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b38bf97, SUC_lexicon_tags_28 },
    { 0x6cf8bf98, SUC_lexicon_tags_12 },
    { 0x7958bf99, SUC_lexicon_tags_4 },
    { 0xf7d8bf9a, SUC_lexicon_tags_14 },
    { 0x3e78bf9a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf658bf9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d8bfa3, SUC_lexicon_tags_48 },
    { 0xc978bfa4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf038bfa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf8bfb6, SUC_lexicon_tags_43 },
    { 0x67d8bfb7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc458bfc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4238bfc6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9c38bfc8, SUC_lexicon_tags_23 },
    { 0xae98bfc9, SUC_lexicon_tags_4 },
    { 0xebf8bfc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd598bfcf, SUC_lexicon_tags_129 },
    { 0x45f8bfcf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b8bfd4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c18bfd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1018bfe1, SUC_lexicon_tags_17 },
    { 0xfd98bfe2, SUC_lexicon_tags_17 },
    { 0xa78bfe3, SUC_lexicon_tags_11 },
    { 0x3bb8bfe4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c38bff7, SUC_lexicon_tags_127 },
    { 0xc298bff7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8178bffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad98c000, SUC_lexicon_tags_1 },
    { 0xd078c001, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf8c004, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee78c007, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8d78c009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1718c00b, SUC_lexicon_tags_43 },
    { 0x9c18c00c, SUC_lexicon_tags_56 },
    { 0x3cf8c00d, SUC_lexicon_tags_11 },
    { 0xf478c00e, SUC_lexicon_tags_19 },
    { 0x33f8c00e, SUC_lexicon_tags_17 },
    { 0x3698c010, SUC_lexicon_tags_63 },
    { 0xa1f8c011, SUC_lexicon_tags_27 },
    { 0x2398c012, SUC_lexicon_tags_106 },
    { 0x66b8c00e, SUC_lexicon_tags_17 },
    { 0xb9b8c00d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a18c018, SUC_lexicon_tags_39 },
    { 0x13b8c019, SUC_lexicon_tags_7 },
    { 0x7df8c01a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x678c01f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb838c021, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc78c024, SUC_lexicon_tags_2 },
    { 0x7358c024, SUC_lexicon_tags_7 },
    { 0xa1f8c024, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d8c029, SUC_lexicon_tags_62 },
    { 0x75f8c02a, SUC_lexicon_tags_1 },
    { 0x2538c02b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeff8c02d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd8c031, SUC_lexicon_tags_30 },
    { 0x2db8c031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df8c035, SUC_lexicon_tags_60 },
    { 0x3678c036, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd138c03e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd8c041, SUC_lexicon_tags_39 },
    { 0x9dd8c042, SUC_lexicon_tags_0 },
    { 0x1658c043, SUC_lexicon_tags_11 },
    { 0x9358c044, SUC_lexicon_tags_47 },
    { 0x35f8c044, SUC_lexicon_tags_8 },
    { 0x47f8c046, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfd8c048, SUC_lexicon_tags_30 },
    { 0x3cd8c049, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdb18c04b, SUC_lexicon_tags_37 },
    { 0x60f8c04c, SUC_lexicon_tags_12 },
    { 0xe878c04c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdad8c04f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbed8c051, SUC_lexicon_tags_23 },
    { 0x91b8c051, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f18c055, SUC_lexicon_tags_11 },
    { 0xb7b8c055, SUC_lexicon_tags_28 },
    { 0x358c057, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa558c05a, SUC_lexicon_tags_7 },
    { 0xc798c05b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae78c05e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea78c062, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b18c064, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e38c067, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f78c06b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed8c06f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9038c075, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x118c077, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x71b8c079, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff8c07f, SUC_lexicon_tags_177 },
    { 0xaa58c07f, SUC_lexicon_tags_23 },
    { 0xbb8c081, SUC_lexicon_tags_1 },
    { 0xe138c081, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8c084, SUC_lexicon_tags_23 },
    { 0x8bb8c084, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d98c088, SUC_lexicon_tags_6 },
    { 0xe6d8c089, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8238c08b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa58c08d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a18c092, SUC_lexicon_tags_47 },
    { 0xc898c093, SUC_lexicon_tags_14 },
    { 0xdeb8c094, SUC_lexicon_tags_1 },
    { 0xe038c092, SUC_lexicon_tags_2 },
    { 0x7898c093, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc498c099, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9618c09b, SUC_lexicon_tags_31 },
    { 0x2ff8c09c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f78c09f, SUC_lexicon_tags_17 },
    { 0x1438c09f, SUC_lexicon_tags_27 },
    { 0x86d8c0a1, SUC_lexicon_tags_11 },
    { 0x39f8c0a0, SUC_lexicon_tags_9 },
    { 0x2b58c0a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf498c0a5, SUC_lexicon_tags_60 },
    { 0xb7f8c0a6, SUC_lexicon_tags_37 },
    { 0x8558c0a7, SUC_lexicon_tags_37 },
    { 0x2098c0a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b8c0ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2938c0ae, SUC_lexicon_tags_27 },
    { 0x5238c0ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f38c0b5, SUC_lexicon_tags_26 },
    { 0x89b8c0b6, SUC_lexicon_tags_14 },
    { 0xb4f8c0b7, SUC_lexicon_tags_27 },
    { 0x2838c0b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8878c0ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4f8c0bc, SUC_lexicon_tags_15 },
    { 0xadf8c0bd, SUC_lexicon_tags_17 },
    { 0x1418c0bd, SUC_lexicon_tags_37 },
    { 0x3398c0be, SUC_lexicon_tags_25 },
    { 0x5d18c0bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5f8c0c2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8338c0c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad58c0c8, SUC_lexicon_tags_107 },
    { 0xf58c0c8, SUC_lexicon_tags_39 },
    { 0x3ad8c0ca, SUC_lexicon_tags_84 },
    { 0x47d8c0cb, SUC_lexicon_tags_11 },
    { 0xbe78c0cc, SUC_lexicon_tags_33 },
    { 0xc6d8c0cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad8c0d1, SUC_lexicon_tags_7 },
    { 0xdfb8c0d1, SUC_lexicon_tags_26 },
    { 0xc718c0d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f8c0d6, SUC_lexicon_tags_7 },
    { 0x8718c0d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf78c0dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c18c0e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd858c0e5, SUC_lexicon_tags_47 },
    { 0x13d8c0e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xed78c0e9, SUC_lexicon_tags_23 },
    { 0xcc38c0e9, SUC_lexicon_tags_11 },
    { 0x4cf8c0e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb78c0f2, SUC_lexicon_tags_1 },
    { 0x29d8c0f3, SUC_lexicon_tags_17 },
    { 0xb638c0f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa2d8c0f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b8c0f9, SUC_lexicon_tags_12 },
    { 0x1318c0fa, SUC_lexicon_tags_37 },
    { 0x9d18c0f9, SUC_lexicon_tags_16 },
    { 0x10b8c0fc, SUC_lexicon_tags_27 },
    { 0xdbf8c0fd, SUC_lexicon_tags_1595 },
    { 0x8d98c0fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x638c101, SUC_lexicon_tags_1 },
    { 0x98f8c102, SUC_lexicon_tags_9 },
    { 0x3af8c103, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9838c107, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5118c109, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa498c10c, SUC_lexicon_tags_29 },
    { 0x3798c10c, SUC_lexicon_tags_15 },
    { 0x838c10c, SUC_lexicon_tags_47 },
    { 0x4ef8c10f, SUC_lexicon_tags_29 },
    { 0x3c18c10d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b8c115, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d8c120, SUC_lexicon_tags_9 },
    { 0x22f8c121, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf758c123, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd918c125, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc518c127, SUC_lexicon_tags_1 },
    { 0x2738c128, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb78c12a, SUC_lexicon_tags_15 },
    { 0xf178c12a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f8c131, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8738c133, SUC_lexicon_tags_60 },
    { 0x8b98c134, SUC_lexicon_tags_1 },
    { 0x2898c135, SUC_lexicon_tags_38 },
    { 0x9818c136, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d38c138, SUC_lexicon_tags_28 },
    { 0x1338c139, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9038c13b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc1d8c13d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x79d8c13f, SUC_lexicon_tags_1 },
    { 0x1458c140, SUC_lexicon_tags_1 },
    { 0x27b8c13f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578c146, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6858c14d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9098c150, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd718c152, SUC_lexicon_tags_63 },
    { 0xcb8c153, SUC_lexicon_tags_37 },
    { 0xe1f8c153, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b8c157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3858c159, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1f8c15b, SUC_lexicon_tags_35 },
    { 0x5198c15c, SUC_lexicon_tags_113 },
    { 0x9f78c15d, SUC_lexicon_tags_38 },
    { 0x84b8c15e, SUC_lexicon_tags_11 },
    { 0xed8c15e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf018c161, SUC_lexicon_tags_45 },
    { 0xe378c162, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3298c164, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f8c16b, SUC_lexicon_tags_17 },
    { 0x18b8c16c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4698c16e, SUC_lexicon_tags_2 },
    { 0x7718c16f, SUC_lexicon_tags_2 },
    { 0x1138c170, SUC_lexicon_tags_23 },
    { 0x7b18c171, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd78c179, SUC_lexicon_tags_354 },
    { 0, NULL },
    { 0xbdd8c17b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7398c17d, SUC_lexicon_tags_12 },
    { 0xea18c17d, SUC_lexicon_tags_29 },
    { 0x4ed8c17f, SUC_lexicon_tags_25 },
    { 0x1078c17d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1618c183, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe218c185, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba38c18f, SUC_lexicon_tags_11 },
    { 0xeaf8c18f, SUC_lexicon_tags_37 },
    { 0x5038c191, SUC_lexicon_tags_13 },
    { 0x29b8c190, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1078c197, SUC_lexicon_tags_87 },
    { 0x5878c197, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5298c19b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef8c1a0, SUC_lexicon_tags_25 },
    { 0xaad8c1a0, SUC_lexicon_tags_15 },
    { 0x8978c1a2, SUC_lexicon_tags_12 },
    { 0x38f8c1a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4538c1a5, SUC_lexicon_tags_29 },
    { 0xa218c1a6, SUC_lexicon_tags_37 },
    { 0xa858c1a7, SUC_lexicon_tags_10 },
    { 0xbdb8c1a8, SUC_lexicon_tags_27 },
    { 0xa58c1a5, SUC_lexicon_tags_37 },
    { 0x6ff8c1a8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa058c1b0, SUC_lexicon_tags_6 },
    { 0xa938c1b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec58c1b5, SUC_lexicon_tags_15 },
    { 0x8bb8c1b6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d8c1ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f8c1c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa58c1c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a18c1c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72b8c1c9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b8c1d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a38c1d5, SUC_lexicon_tags_2 },
    { 0xd358c1d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc718c1d8, SUC_lexicon_tags_25 },
    { 0x9f8c1d8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb8f8c1db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8978c1de, SUC_lexicon_tags_39 },
    { 0x38b8c1df, SUC_lexicon_tags_11 },
    { 0xe6f8c1e0, SUC_lexicon_tags_38 },
    { 0x52f8c1e1, SUC_lexicon_tags_15 },
    { 0xd018c1e2, SUC_lexicon_tags_12 },
    { 0xb478c1e3, SUC_lexicon_tags_57 },
    { 0x7f78c1e1, SUC_lexicon_tags_12 },
    { 0xe998c1e5, SUC_lexicon_tags_25 },
    { 0x3a58c1e6, SUC_lexicon_tags_0 },
    { 0xa858c1e5, SUC_lexicon_tags_7 },
    { 0xfdf8c1e6, SUC_lexicon_tags_65 },
    { 0x9fd8c1e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f8c1ec, SUC_lexicon_tags_2 },
    { 0x6c78c1ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4f8c1ef, SUC_lexicon_tags_11 },
    { 0x738c1f0, SUC_lexicon_tags_6 },
    { 0xab98c1f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1158c1f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb38c1fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc918c200, SUC_lexicon_tags_11 },
    { 0x1d38c201, SUC_lexicon_tags_37 },
    { 0xe9b8c200, SUC_lexicon_tags_32 },
    { 0xeaf8c202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe98c205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa158c20c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc98c20e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9718c213, SUC_lexicon_tags_37 },
    { 0x76b8c214, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2078c217, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb8c21a, SUC_lexicon_tags_15 },
    { 0x1398c21a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f8c21f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b8c223, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb218c225, SUC_lexicon_tags_28 },
    { 0x7258c225, SUC_lexicon_tags_1 },
    { 0x1538c227, SUC_lexicon_tags_40 },
    { 0xe0b8c228, SUC_lexicon_tags_37 },
    { 0xcfb8c228, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd238c22b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f8c22f, SUC_lexicon_tags_0 },
    { 0x59b8c230, SUC_lexicon_tags_35 },
    { 0xc578c230, SUC_lexicon_tags_17 },
    { 0xeab8c230, SUC_lexicon_tags_11 },
    { 0xcc98c233, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d8c237, SUC_lexicon_tags_63 },
    { 0xbd78c238, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd38c23b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d8c240, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf138c242, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f58c246, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad8c24f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18c255, SUC_lexicon_tags_12 },
    { 0x56d8c256, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1d8c258, SUC_lexicon_tags_61 },
    { 0xf58c259, SUC_lexicon_tags_27 },
    { 0x69b8c25a, SUC_lexicon_tags_47 },
    { 0x3838c25b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xef98c25d, SUC_lexicon_tags_15 },
    { 0x90f8c25d, SUC_lexicon_tags_38 },
    { 0xef38c25f, SUC_lexicon_tags_33 },
    { 0x4cf8c260, SUC_lexicon_tags_11 },
    { 0x4538c260, SUC_lexicon_tags_0 },
    { 0x58f8c260, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8c267, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe38c26a, SUC_lexicon_tags_1 },
    { 0xf7f8c26b, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb8c26e, SUC_lexicon_tags_0 },
    { 0x7ad8c26f, SUC_lexicon_tags_23 },
    { 0xcbb8c26f, SUC_lexicon_tags_14 },
    { 0x338c26e, SUC_lexicon_tags_7 },
    { 0x1ad8c26f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xddf8c274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3858c277, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xefd8c279, SUC_lexicon_tags_28 },
    { 0x3298c27a, SUC_lexicon_tags_25 },
    { 0xfe38c27a, SUC_lexicon_tags_17 },
    { 0xff58c27c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d58c285, SUC_lexicon_tags_45 },
    { 0x9638c285, SUC_lexicon_tags_87 },
    { 0xbb8c287, SUC_lexicon_tags_114 },
    { 0xa918c287, SUC_lexicon_tags_37 },
    { 0xc3f8c286, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9cb8c28b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x238c294, SUC_lexicon_tags_29 },
    { 0x4c18c295, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf18c29a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24f8c29c, SUC_lexicon_tags_33 },
    { 0x85b8c29d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2dd8c29f, SUC_lexicon_tags_7 },
    { 0xb4d8c2a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7558c2a5, SUC_lexicon_tags_12 },
    { 0xc8d8c2a6, SUC_lexicon_tags_13 },
    { 0x12f8c2a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4938c2ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfdb8c2ad, SUC_lexicon_tags_60 },
    { 0xfc78c2ae, SUC_lexicon_tags_63 },
    { 0x398c2af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9338c2b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf8c2b9, SUC_lexicon_tags_11 },
    { 0x8978c2ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5598c2bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb658c2c2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f8c2c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5198c2c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d38c2cb, SUC_lexicon_tags_1 },
    { 0x6298c2cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1838c2d1, SUC_lexicon_tags_0 },
    { 0xa5b8c2d2, SUC_lexicon_tags_11 },
    { 0x7b78c2d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4478c2d5, SUC_lexicon_tags_7 },
    { 0x76f8c2d6, SUC_lexicon_tags_45 },
    { 0x2358c2d7, SUC_lexicon_tags_37 },
    { 0x44b8c2d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378c2dc, SUC_lexicon_tags_1 },
    { 0x37f8c2dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6838c2e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc178c2e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f8c2eb, SUC_lexicon_tags_27 },
    { 0x6038c2eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa2d8c2ee, SUC_lexicon_tags_38 },
    { 0x7bb8c2ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd858c2f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2258c2f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde58c2f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3b38c2fb, SUC_lexicon_tags_49 },
    { 0x4fb8c2fc, SUC_lexicon_tags_60 },
    { 0x8a98c2fd, SUC_lexicon_tags_25 },
    { 0x1138c2fe, SUC_lexicon_tags_2 },
    { 0xf2f8c2fe, SUC_lexicon_tags_28 },
    { 0xa498c300, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d8c304, SUC_lexicon_tags_19 },
    { 0x8c78c305, SUC_lexicon_tags_28 },
    { 0x418c306, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b8c30b, SUC_lexicon_tags_29 },
    { 0x13b8c30c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d8c30f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa38c314, SUC_lexicon_tags_38 },
    { 0x3a38c315, SUC_lexicon_tags_23 },
    { 0x3738c315, SUC_lexicon_tags_33 },
    { 0x5f18c317, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb598c31a, SUC_lexicon_tags_37 },
    { 0x22d8c31b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed8c31e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6b38c320, SUC_lexicon_tags_12 },
    { 0xaf18c321, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe358c323, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10d8c325, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xccb8c327, SUC_lexicon_tags_37 },
    { 0x8098c328, SUC_lexicon_tags_25 },
    { 0x21b8c327, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x2af8c32b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7218c330, SUC_lexicon_tags_15 },
    { 0xaf58c331, SUC_lexicon_tags_11 },
    { 0x4a18c332, SUC_lexicon_tags_14 },
    { 0xa4f8c332, SUC_lexicon_tags_2 },
    { 0x3d8c330, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7ef8c336, SUC_lexicon_tags_15 },
    { 0x3658c337, SUC_lexicon_tags_37 },
    { 0xb038c336, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a58c33a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf8c33d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f8c344, SUC_lexicon_tags_2 },
    { 0x8ff8c344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae38c349, SUC_lexicon_tags_2 },
    { 0xbb78c34a, SUC_lexicon_tags_11 },
    { 0xda98c34a, SUC_lexicon_tags_29 },
    { 0x2158c34c, SUC_lexicon_tags_17 },
    { 0xff98c34d, SUC_lexicon_tags_46 },
    { 0xb438c34d, SUC_lexicon_tags_23 },
    { 0x2a38c34c, SUC_lexicon_tags_7 },
    { 0xdad8c350, SUC_lexicon_tags_6 },
    { 0xc9b8c350, SUC_lexicon_tags_17 },
    { 0xd758c352, SUC_lexicon_tags_25 },
    { 0x9c38c352, SUC_lexicon_tags_14 },
    { 0xbb78c354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf8c359, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d18c35e, SUC_lexicon_tags_11 },
    { 0x9d98c35f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b8c365, SUC_lexicon_tags_66 },
    { 0xec78c366, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d18c369, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e78c36e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf8c371, SUC_lexicon_tags_32 },
    { 0x4878c371, SUC_lexicon_tags_11 },
    { 0x5ed8c371, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf8c37b, SUC_lexicon_tags_61 },
    { 0xf578c37b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x118c380, SUC_lexicon_tags_2 },
    { 0x12d8c381, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2f8c383, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f78c387, SUC_lexicon_tags_17 },
    { 0x9898c387, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded8c392, SUC_lexicon_tags_30 },
    { 0x8758c393, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9438c395, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd318c399, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9598c39d, SUC_lexicon_tags_61 },
    { 0x5618c39d, SUC_lexicon_tags_15 },
    { 0x7f18c39d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd8c3a7, SUC_lexicon_tags_80 },
    { 0xd458c3a8, SUC_lexicon_tags_37 },
    { 0x1558c3a8, SUC_lexicon_tags_21 },
    { 0xd878c3a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d8c3b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d8c3b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4258c3b8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc318c3ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3978c3bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd8c3bf, SUC_lexicon_tags_32 },
    { 0x7f58c3c0, SUC_lexicon_tags_1 },
    { 0x84b8c3c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d8c3c5, SUC_lexicon_tags_12 },
    { 0xeed8c3c5, SUC_lexicon_tags_49 },
    { 0x4018c3c5, SUC_lexicon_tags_1 },
    { 0xedb8c3c8, SUC_lexicon_tags_12 },
    { 0xc6f8c3c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f8c3cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc478c3d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd858c3d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3b8c3da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf38c3dc, SUC_lexicon_tags_19 },
    { 0xd2d8c3dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3718c3e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4f8c3e3, SUC_lexicon_tags_23 },
    { 0xad98c3e4, SUC_lexicon_tags_29 },
    { 0xbc38c3e4, SUC_lexicon_tags_37 },
    { 0x1d58c3e6, SUC_lexicon_tags_17 },
    { 0x2718c3e7, SUC_lexicon_tags_2 },
    { 0x6178c3e6, SUC_lexicon_tags_7 },
    { 0x1dd8c3e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80b8c3eb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8718c3ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7bf8c3f1, SUC_lexicon_tags_15 },
    { 0xd458c3f2, SUC_lexicon_tags_45 },
    { 0xaa98c3f3, SUC_lexicon_tags_2 },
    { 0xf158c3f3, SUC_lexicon_tags_11 },
    { 0xd598c3f5, SUC_lexicon_tags_63 },
    { 0x2538c3f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f8c3f9, SUC_lexicon_tags_12 },
    { 0x83b8c3fa, SUC_lexicon_tags_15 },
    { 0x52d8c3fa, SUC_lexicon_tags_8 },
    { 0xfbb8c3f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2758c3fe, SUC_lexicon_tags_11 },
    { 0x5d38c3ff, SUC_lexicon_tags_37 },
    { 0xdb38c3ff, SUC_lexicon_tags_17 },
    { 0x8b58c401, SUC_lexicon_tags_6 },
    { 0x7838c401, SUC_lexicon_tags_15 },
    { 0x6b38c403, SUC_lexicon_tags_37 },
    { 0x2678c404, SUC_lexicon_tags_1 },
    { 0xd1d8c3ff, SUC_lexicon_tags_1 },
    { 0xca78c400, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x19b8c408, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9758c40c, SUC_lexicon_tags_9 },
    { 0x7218c40c, SUC_lexicon_tags_7 },
    { 0xaff8c40c, SUC_lexicon_tags_37 },
    { 0x6138c40d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d8c417, SUC_lexicon_tags_7 },
    { 0xe638c418, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9258c41b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2938c420, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc938c424, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb8c427, SUC_lexicon_tags_77 },
    { 0x2c58c428, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb58c42c, SUC_lexicon_tags_37 },
    { 0xf318c42d, SUC_lexicon_tags_47 },
    { 0x258c42e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1f8c430, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5098c432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb118c434, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed8c438, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9f98c43a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dd8c43c, SUC_lexicon_tags_0 },
    { 0x98d8c43d, SUC_lexicon_tags_37 },
    { 0xc4b8c43e, SUC_lexicon_tags_17 },
    { 0x5018c43e, SUC_lexicon_tags_17 },
    { 0x23f8c440, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5fb8c442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4258c444, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf058c446, SUC_lexicon_tags_13 },
    { 0xf5d8c447, SUC_lexicon_tags_1 },
    { 0x4c18c448, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d8c44b, SUC_lexicon_tags_60 },
    { 0xedf8c44c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf858c44f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa198c453, SUC_lexicon_tags_25 },
    { 0x6ed8c454, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf18c45d, SUC_lexicon_tags_15 },
    { 0x1cd8c45d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb458c463, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bb8c465, SUC_lexicon_tags_63 },
    { 0xbef8c466, SUC_lexicon_tags_12 },
    { 0x218c467, SUC_lexicon_tags_47 },
    { 0x24b8c466, SUC_lexicon_tags_60 },
    { 0xdaf8c468, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b78c46c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9178c472, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf8c478, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed8c47d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4078c481, SUC_lexicon_tags_19 },
    { 0x52b8c482, SUC_lexicon_tags_0 },
    { 0xdd58c482, SUC_lexicon_tags_9 },
    { 0x7578c483, SUC_lexicon_tags_2 },
    { 0xfc78c485, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4718c487, SUC_lexicon_tags_29 },
    { 0xd178c487, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x98f8c48a, SUC_lexicon_tags_57 },
    { 0x5a18c48b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf998c490, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb18c493, SUC_lexicon_tags_19 },
    { 0x50f8c493, SUC_lexicon_tags_33 },
    { 0x6778c494, SUC_lexicon_tags_23 },
    { 0xc278c496, SUC_lexicon_tags_14 },
    { 0x6478c497, SUC_lexicon_tags_19 },
    { 0xcb38c496, SUC_lexicon_tags_4 },
    { 0x63b8c496, SUC_lexicon_tags_11 },
    { 0x6918c496, SUC_lexicon_tags_17 },
    { 0xf058c499, SUC_lexicon_tags_8 },
    { 0x4718c49c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f8c49f, SUC_lexicon_tags_66 },
    { 0xb438c49f, SUC_lexicon_tags_41 },
    { 0xb478c49f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa98c4a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb8c4a5, SUC_lexicon_tags_7 },
    { 0xc9d8c4a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe918c4ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd4d8c4ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d8c4b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d18c4b7, SUC_lexicon_tags_21 },
    { 0xce18c4b8, SUC_lexicon_tags_7 },
    { 0xc998c4b9, SUC_lexicon_tags_15 },
    { 0xf158c4ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5678c4c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf998c4c6, SUC_lexicon_tags_26 },
    { 0x72f8c4c7, SUC_lexicon_tags_30 },
    { 0xd4b8c4c8, SUC_lexicon_tags_59 },
    { 0x2c38c4c8, SUC_lexicon_tags_17 },
    { 0xe678c4ca, SUC_lexicon_tags_65 },
    { 0xff8c4ca, SUC_lexicon_tags_35 },
    { 0x638c4cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32f8c4ce, SUC_lexicon_tags_44 },
    { 0x3b18c4cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xac18c4d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15f8c4d4, SUC_lexicon_tags_13 },
    { 0x8478c4d5, SUC_lexicon_tags_17 },
    { 0xb238c4d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f8c4dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf8c4e0, SUC_lexicon_tags_14 },
    { 0xb0f8c4e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4918c4e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x90d8c4e9, SUC_lexicon_tags_29 },
    { 0x74d8c4e9, SUC_lexicon_tags_2 },
    { 0x4438c4eb, SUC_lexicon_tags_37 },
    { 0x9ef8c4eb, SUC_lexicon_tags_7 },
    { 0x9478c4ea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4818c4f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d8c4f4, SUC_lexicon_tags_0 },
    { 0x3cb8c4f5, SUC_lexicon_tags_1 },
    { 0xc458c4f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b8c4fd, SUC_lexicon_tags_27 },
    { 0x51d8c4fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5618c503, SUC_lexicon_tags_28 },
    { 0xf3f8c504, SUC_lexicon_tags_37 },
    { 0x32d8c504, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9398c508, SUC_lexicon_tags_30 },
    { 0xe358c508, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5b8c50b, SUC_lexicon_tags_38 },
    { 0x1d58c50c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26b8c50e, SUC_lexicon_tags_63 },
    { 0xc5b8c50e, SUC_lexicon_tags_25 },
    { 0x7478c50f, SUC_lexicon_tags_33 },
    { 0xa1f8c511, SUC_lexicon_tags_15 },
    { 0x5ff8c512, SUC_lexicon_tags_0 },
    { 0x80f8c50e, SUC_lexicon_tags_4 },
    { 0x50f8c514, SUC_lexicon_tags_7 },
    { 0xf238c514, SUC_lexicon_tags_10 },
    { 0xbcf8c516, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe118c519, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2158c51f, SUC_lexicon_tags_38 },
    { 0x8118c520, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad8c526, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda78c52a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b98c52c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda58c52f, SUC_lexicon_tags_47 },
    { 0xcd78c530, SUC_lexicon_tags_238 },
    { 0xe858c531, SUC_lexicon_tags_7 },
    { 0x70b8c532, SUC_lexicon_tags_12 },
    { 0x1258c532, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55d8c535, SUC_lexicon_tags_63 },
    { 0x9fb8c536, SUC_lexicon_tags_37 },
    { 0xdb38c537, SUC_lexicon_tags_12 },
    { 0x1ad8c538, SUC_lexicon_tags_45 },
    { 0xad8c539, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac98c53f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9198c546, SUC_lexicon_tags_42 },
    { 0xc998c547, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed8c54b, SUC_lexicon_tags_7 },
    { 0xebf8c54b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b8c550, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f8c554, SUC_lexicon_tags_17 },
    { 0xf6d8c555, SUC_lexicon_tags_37 },
    { 0x6e38c556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f8c55a, SUC_lexicon_tags_7 },
    { 0x3278c55b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6498c55d, SUC_lexicon_tags_66 },
    { 0x21d8c55e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b8c566, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7098c568, SUC_lexicon_tags_41 },
    { 0xd38c569, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf338c56b, SUC_lexicon_tags_25 },
    { 0x6998c56c, SUC_lexicon_tags_15 },
    { 0x2a18c56d, SUC_lexicon_tags_98 },
    { 0xfa78c56c, SUC_lexicon_tags_10 },
    { 0x8e18c56f, SUC_lexicon_tags_17 },
    { 0x11d8c56f, SUC_lexicon_tags_1548 },
    { 0, NULL },
    { 0, NULL },
    { 0x7338c573, SUC_lexicon_tags_0 },
    { 0x1d8c573, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd378c576, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b58c57c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b8c57e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x38f8c580, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9978c582, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8198c588, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec78c58c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb8c594, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf538c597, SUC_lexicon_tags_28 },
    { 0x57d8c598, SUC_lexicon_tags_11 },
    { 0xcbb8c599, SUC_lexicon_tags_12 },
    { 0x4578c59a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d8c5a0, SUC_lexicon_tags_37 },
    { 0x85d8c5a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a18c5a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59d8c5a6, SUC_lexicon_tags_1 },
    { 0x2498c5a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2658c5a9, SUC_lexicon_tags_29 },
    { 0x8478c5a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaa98c5ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2978c5b3, SUC_lexicon_tags_33 },
    { 0x7ef8c5b4, SUC_lexicon_tags_9 },
    { 0xfc98c5b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d8c5b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb978c5bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f8c5c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2df8c5c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3198c5c6, SUC_lexicon_tags_12 },
    { 0x4738c5c7, SUC_lexicon_tags_8 },
    { 0x2b38c5c8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7598c5cb, SUC_lexicon_tags_14 },
    { 0x3f8c5cc, SUC_lexicon_tags_25 },
    { 0xabb8c5cd, SUC_lexicon_tags_107 },
    { 0x5bb8c5cd, SUC_lexicon_tags_6 },
    { 0x6df8c5cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d8c5d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f98c5da, SUC_lexicon_tags_10 },
    { 0x3718c5db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9578c5dd, SUC_lexicon_tags_12 },
    { 0xcdf8c5de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff8c5e1, SUC_lexicon_tags_8 },
    { 0xf438c5e2, SUC_lexicon_tags_37 },
    { 0xe238c5e2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878c5e7, SUC_lexicon_tags_11 },
    { 0x28d8c5e8, SUC_lexicon_tags_63 },
    { 0x61d8c5e8, SUC_lexicon_tags_7 },
    { 0x1e98c5e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fb8c5ec, SUC_lexicon_tags_7 },
    { 0x4d18c5ed, SUC_lexicon_tags_43 },
    { 0xaff8c5ee, SUC_lexicon_tags_23 },
    { 0xd018c5ed, SUC_lexicon_tags_26 },
    { 0x8478c5f0, SUC_lexicon_tags_7 },
    { 0xe0b8c5ed, SUC_lexicon_tags_23 },
    { 0x7878c5f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4938c5f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45d8c5f6, SUC_lexicon_tags_25 },
    { 0x5ff8c5f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72f8c602, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed8c606, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x7178c609, SUC_lexicon_tags_7 },
    { 0x2bd8c609, SUC_lexicon_tags_37 },
    { 0x4358c60b, SUC_lexicon_tags_12 },
    { 0x8dd8c60c, SUC_lexicon_tags_30 },
    { 0x3198c60d, SUC_lexicon_tags_2 },
    { 0xf7b8c60e, SUC_lexicon_tags_29 },
    { 0xe3f8c60e, SUC_lexicon_tags_7 },
    { 0x94d8c60a, SUC_lexicon_tags_60 },
    { 0x3c98c60a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6f8c613, SUC_lexicon_tags_7 },
    { 0x6b98c614, SUC_lexicon_tags_69 },
    { 0xafb8c614, SUC_lexicon_tags_1159 },
    { 0, NULL },
    { 0xb778c617, SUC_lexicon_tags_45 },
    { 0x2738c617, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81f8c61a, SUC_lexicon_tags_35 },
    { 0x26d8c61b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc978c61f, SUC_lexicon_tags_17 },
    { 0x5058c61f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf038c623, SUC_lexicon_tags_11 },
    { 0x1318c624, SUC_lexicon_tags_37 },
    { 0x1338c623, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ad8c627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5818c632, SUC_lexicon_tags_7 },
    { 0x4718c633, SUC_lexicon_tags_65 },
    { 0x23b8c633, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb8c638, SUC_lexicon_tags_3 },
    { 0x298c638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd78c63c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa438c63e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4258c647, SUC_lexicon_tags_4 },
    { 0x3218c648, SUC_lexicon_tags_15 },
    { 0x4d38c647, SUC_lexicon_tags_14 },
    { 0xd9d8c64a, SUC_lexicon_tags_25 },
    { 0xd778c64a, SUC_lexicon_tags_14 },
    { 0x9dd8c64c, SUC_lexicon_tags_47 },
    { 0x7b98c64d, SUC_lexicon_tags_39 },
    { 0x61f8c647, SUC_lexicon_tags_13 },
    { 0xf358c64f, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0x7398c651, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b18c655, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78c658, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d38c660, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa038c664, SUC_lexicon_tags_25 },
    { 0xa978c664, SUC_lexicon_tags_37 },
    { 0xc718c666, SUC_lexicon_tags_17 },
    { 0x53f8c665, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8c675, SUC_lexicon_tags_82 },
    { 0x5bf8c676, SUC_lexicon_tags_17 },
    { 0xb518c677, SUC_lexicon_tags_11 },
    { 0x5d8c678, SUC_lexicon_tags_13 },
    { 0x5b38c679, SUC_lexicon_tags_1 },
    { 0x4798c677, SUC_lexicon_tags_63 },
    { 0xc798c678, SUC_lexicon_tags_60 },
    { 0x63d8c677, SUC_lexicon_tags_61 },
    { 0xb518c67d, SUC_lexicon_tags_12 },
    { 0xa218c67d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98c684, SUC_lexicon_tags_30 },
    { 0xcff8c684, SUC_lexicon_tags_1 },
    { 0xfa58c684, SUC_lexicon_tags_26 },
    { 0x8558c686, SUC_lexicon_tags_25 },
    { 0x4738c688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97b8c68a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d8c68e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b58c691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd8c697, SUC_lexicon_tags_14 },
    { 0x2318c698, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7918c69a, SUC_lexicon_tags_9 },
    { 0xfe98c69b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7658c6a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb8c6a9, SUC_lexicon_tags_14 },
    { 0x84b8c6a9, SUC_lexicon_tags_25 },
    { 0x7a98c6a9, SUC_lexicon_tags_9 },
    { 0xd298c6ac, SUC_lexicon_tags_1 },
    { 0x46b8c6ab, SUC_lexicon_tags_37 },
    { 0x6e38c6ae, SUC_lexicon_tags_66 },
    { 0xf778c6aa, SUC_lexicon_tags_37 },
    { 0xbf38c6b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa198c6b2, SUC_lexicon_tags_11 },
    { 0x2d78c6b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b8c6bb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5298c6c3, SUC_lexicon_tags_47 },
    { 0x9838c6c4, SUC_lexicon_tags_2 },
    { 0x6db8c6c4, SUC_lexicon_tags_45 },
    { 0x5af8c6c3, SUC_lexicon_tags_2 },
    { 0xe238c6c3, SUC_lexicon_tags_7 },
    { 0x3d78c6c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9418c6cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec18c6ce, SUC_lexicon_tags_2 },
    { 0x1f98c6cf, SUC_lexicon_tags_47 },
    { 0x7198c6d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e18c6d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18d8c6d6, SUC_lexicon_tags_37 },
    { 0x9278c6d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb958c6d9, SUC_lexicon_tags_18 },
    { 0x9f98c6da, SUC_lexicon_tags_2 },
    { 0x2c18c6db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24d8c6de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7398c6e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc138c6e8, SUC_lexicon_tags_45 },
    { 0x65d8c6e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f8c6ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7798c6f0, SUC_lexicon_tags_1551 },
    { 0xad78c6f1, SUC_lexicon_tags_11 },
    { 0x7378c6f2, SUC_lexicon_tags_14 },
    { 0xfbb8c6f2, SUC_lexicon_tags_37 },
    { 0x3278c6f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3b8c6f6, SUC_lexicon_tags_33 },
    { 0x5138c6f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe518c6fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d8c6ff, SUC_lexicon_tags_37 },
    { 0x2db8c700, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xac98c702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b8c705, SUC_lexicon_tags_0 },
    { 0xc138c706, SUC_lexicon_tags_38 },
    { 0x8c58c706, SUC_lexicon_tags_40 },
    { 0x3af8c708, SUC_lexicon_tags_28 },
    { 0x2b78c709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1eb8c70b, SUC_lexicon_tags_45 },
    { 0x8a78c70c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf978c70f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9778c713, SUC_lexicon_tags_6 },
    { 0xcd58c714, SUC_lexicon_tags_43 },
    { 0x3738c715, SUC_lexicon_tags_7 },
    { 0xf1f8c715, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xac18c719, SUC_lexicon_tags_28 },
    { 0x6b98c719, SUC_lexicon_tags_1 },
    { 0xe818c71b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbc78c71d, SUC_lexicon_tags_25 },
    { 0xe78c71d, SUC_lexicon_tags_12 },
    { 0x2ad8c71e, SUC_lexicon_tags_11 },
    { 0x7798c720, SUC_lexicon_tags_2 },
    { 0x1e18c721, SUC_lexicon_tags_198 },
    { 0x69b8c722, SUC_lexicon_tags_7 },
    { 0x2ef8c71d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4a38c725, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb278c728, SUC_lexicon_tags_23 },
    { 0x74f8c728, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6038c72c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f8c72f, SUC_lexicon_tags_11 },
    { 0x4ad8c730, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2478c738, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a38c73b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1b8c73d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d38c73f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f8c742, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1818c744, SUC_lexicon_tags_12 },
    { 0x2638c745, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x64d8c747, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5358c74b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x22f8c74e, SUC_lexicon_tags_38 },
    { 0x5ab8c74f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd8c755, SUC_lexicon_tags_15 },
    { 0xd158c755, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe298c75b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef18c760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d78c762, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b8c769, SUC_lexicon_tags_48 },
    { 0x68f8c769, SUC_lexicon_tags_19 },
    { 0x1558c769, SUC_lexicon_tags_149 },
    { 0x51d8c76c, SUC_lexicon_tags_1 },
    { 0x1438c76d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd78c774, SUC_lexicon_tags_12 },
    { 0xb158c775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8b8c777, SUC_lexicon_tags_37 },
    { 0xb318c777, SUC_lexicon_tags_14 },
    { 0x70f8c779, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d98c77b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b8c781, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5258c783, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f8c78b, SUC_lexicon_tags_1 },
    { 0x7f78c78b, SUC_lexicon_tags_39 },
    { 0x6d38c78d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7278c790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe818c793, SUC_lexicon_tags_17 },
    { 0x24d8c794, SUC_lexicon_tags_45 },
    { 0x1518c795, SUC_lexicon_tags_1 },
    { 0xce38c796, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff18c798, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1418c7a0, SUC_lexicon_tags_2 },
    { 0x3cf8c7a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4bb8c7a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc578c7a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b8c7aa, SUC_lexicon_tags_107 },
    { 0x4958c7ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15d8c7ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc798c7af, SUC_lexicon_tags_57 },
    { 0xc0d8c7b0, SUC_lexicon_tags_23 },
    { 0x88b8c7b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb658c7b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e18c7ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8458c7bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b78c7be, SUC_lexicon_tags_45 },
    { 0xfa78c7bf, SUC_lexicon_tags_23 },
    { 0x58b8c7be, SUC_lexicon_tags_37 },
    { 0x3058c7c1, SUC_lexicon_tags_12 },
    { 0xd078c7c2, SUC_lexicon_tags_2 },
    { 0x7178c7c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2eb8c7c5, SUC_lexicon_tags_31 },
    { 0x6958c7c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x82d8c7c8, SUC_lexicon_tags_1 },
    { 0xe058c7c8, SUC_lexicon_tags_15 },
    { 0xbf78c7ca, SUC_lexicon_tags_71 },
    { 0x2e38c7cb, SUC_lexicon_tags_45 },
    { 0xa9b8c7cc, SUC_lexicon_tags_19 },
    { 0x38d8c7ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa958c7d3, SUC_lexicon_tags_336 },
    { 0xc958c7d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7198c7d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a58c7da, SUC_lexicon_tags_14 },
    { 0xd178c7db, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d8c7de, SUC_lexicon_tags_1 },
    { 0x6618c7de, SUC_lexicon_tags_9 },
    { 0x8dd8c7e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb8c7e3, SUC_lexicon_tags_8 },
    { 0xdcb8c7e4, SUC_lexicon_tags_33 },
    { 0xf1b8c7e5, SUC_lexicon_tags_255 },
    { 0x73b8c7e6, SUC_lexicon_tags_27 },
    { 0xe8d8c7e7, SUC_lexicon_tags_63 },
    { 0x7d78c7e7, SUC_lexicon_tags_37 },
    { 0x4638c7e9, SUC_lexicon_tags_47 },
    { 0x3478c7ea, SUC_lexicon_tags_37 },
    { 0xbc98c7e9, SUC_lexicon_tags_7 },
    { 0x86b8c7ec, SUC_lexicon_tags_33 },
    { 0x5078c7ea, SUC_lexicon_tags_17 },
    { 0xdf78c7e7, SUC_lexicon_tags_8 },
    { 0xaaf8c7ec, SUC_lexicon_tags_11 },
    { 0x5758c7ee, SUC_lexicon_tags_58 },
    { 0xbe58c7ef, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7158c7f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc438c7fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd5f8c7fe, SUC_lexicon_tags_17 },
    { 0x36b8c7ff, SUC_lexicon_tags_29 },
    { 0x1718c7ff, SUC_lexicon_tags_37 },
    { 0xc7b8c801, SUC_lexicon_tags_11 },
    { 0xbc18c801, SUC_lexicon_tags_12 },
    { 0x1978c800, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2818c807, SUC_lexicon_tags_25 },
    { 0xe538c808, SUC_lexicon_tags_1 },
    { 0x8998c809, SUC_lexicon_tags_31 },
    { 0xf998c80a, SUC_lexicon_tags_66 },
    { 0xb218c80b, SUC_lexicon_tags_12 },
    { 0x3558c80c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf8c812, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd98c815, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe358c817, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf18c81b, SUC_lexicon_tags_15 },
    { 0x7db8c81b, SUC_lexicon_tags_7 },
    { 0x2b78c81d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc198c824, SUC_lexicon_tags_37 },
    { 0x9f58c825, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9418c827, SUC_lexicon_tags_7 },
    { 0x67b8c828, SUC_lexicon_tags_60 },
    { 0x8238c828, SUC_lexicon_tags_11 },
    { 0xbab8c82a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8938c82e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1578c832, SUC_lexicon_tags_49 },
    { 0xf4b8c832, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d78c83a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x338c840, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b8c84c, SUC_lexicon_tags_2 },
    { 0x1818c84c, SUC_lexicon_tags_38 },
    { 0x88f8c84e, SUC_lexicon_tags_11 },
    { 0x4798c84e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5778c853, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3f8c855, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5018c857, SUC_lexicon_tags_15 },
    { 0x9598c858, SUC_lexicon_tags_11 },
    { 0xfd18c859, SUC_lexicon_tags_6 },
    { 0xc798c859, SUC_lexicon_tags_96 },
    { 0x9658c85b, SUC_lexicon_tags_1 },
    { 0x5d58c85c, SUC_lexicon_tags_2 },
    { 0x20b8c85d, SUC_lexicon_tags_7 },
    { 0xd898c85d, SUC_lexicon_tags_25 },
    { 0x84d8c85f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d38c864, SUC_lexicon_tags_14 },
    { 0x2d98c865, SUC_lexicon_tags_28 },
    { 0x5478c866, SUC_lexicon_tags_12 },
    { 0x3bb8c865, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5778c869, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f8c86d, SUC_lexicon_tags_15 },
    { 0x1dd8c86e, SUC_lexicon_tags_37 },
    { 0x13d8c86f, SUC_lexicon_tags_14 },
    { 0x738c86d, SUC_lexicon_tags_25 },
    { 0xfe78c86d, SUC_lexicon_tags_25 },
    { 0xd2b8c872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd8c879, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6238c87c, SUC_lexicon_tags_44 },
    { 0x4158c87d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd218c881, SUC_lexicon_tags_2 },
    { 0x70b8c882, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe058c88b, SUC_lexicon_tags_8 },
    { 0x71d8c88b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1098c88f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5e38c891, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b8c894, SUC_lexicon_tags_25 },
    { 0x3d58c895, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x18c897, SUC_lexicon_tags_47 },
    { 0x7258c898, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7558c8a0, SUC_lexicon_tags_1 },
    { 0xdff8c8a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xced8c8a3, SUC_lexicon_tags_33 },
    { 0x18d8c8a4, SUC_lexicon_tags_63 },
    { 0x30d8c8a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb8c8a8, SUC_lexicon_tags_38 },
    { 0xec18c8a8, SUC_lexicon_tags_15 },
    { 0xb8b8c8aa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd8f8c8ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf18c8b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf578c8b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e78c8bb, SUC_lexicon_tags_11 },
    { 0x3d8c8bc, SUC_lexicon_tags_39 },
    { 0xef38c8bd, SUC_lexicon_tags_11 },
    { 0xd518c8bb, SUC_lexicon_tags_6 },
    { 0x6218c8be, SUC_lexicon_tags_18 },
    { 0x7d98c8c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20b8c8c5, SUC_lexicon_tags_29 },
    { 0x4918c8c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf198c8d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf518c8d5, SUC_lexicon_tags_33 },
    { 0x9f58c8d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc78c8d9, SUC_lexicon_tags_47 },
    { 0xdef8c8da, SUC_lexicon_tags_33 },
    { 0x6fb8c8d9, SUC_lexicon_tags_1 },
    { 0x1cd8c8da, SUC_lexicon_tags_27 },
    { 0x1078c8dd, SUC_lexicon_tags_63 },
    { 0xbe18c8de, SUC_lexicon_tags_2 },
    { 0x2078c8df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4338c8e2, SUC_lexicon_tags_85 },
    { 0x7e18c8e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30d8c8e5, SUC_lexicon_tags_31 },
    { 0xc038c8e6, SUC_lexicon_tags_27 },
    { 0x1498c8e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8c8ee, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8f98c8f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1c98c8f2, SUC_lexicon_tags_45 },
    { 0x46d8c8f3, SUC_lexicon_tags_28 },
    { 0x5098c8f2, SUC_lexicon_tags_15 },
    { 0x5458c8f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a98c8f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f58c8fc, SUC_lexicon_tags_29 },
    { 0xa4d8c8fc, SUC_lexicon_tags_1 },
    { 0x7218c8fe, SUC_lexicon_tags_23 },
    { 0x71f8c8ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6418c903, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9698c905, SUC_lexicon_tags_37 },
    { 0x178c906, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec38c90a, SUC_lexicon_tags_105 },
    { 0xe8b8c90a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82f8c90d, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad8c910, SUC_lexicon_tags_11 },
    { 0xbbf8c910, SUC_lexicon_tags_11 },
    { 0x1398c911, SUC_lexicon_tags_15 },
    { 0xb4d8c912, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x15b8c915, SUC_lexicon_tags_12 },
    { 0xaa58c916, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa38c922, SUC_lexicon_tags_25 },
    { 0x4538c923, SUC_lexicon_tags_26 },
    { 0x9338c923, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77d8c926, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d8c92c, SUC_lexicon_tags_30 },
    { 0x6818c92d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9858c934, SUC_lexicon_tags_37 },
    { 0xd438c934, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe338c93c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58c93f, SUC_lexicon_tags_22 },
    { 0xb298c940, SUC_lexicon_tags_3 },
    { 0x4158c93f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7498c945, SUC_lexicon_tags_0 },
    { 0xc258c946, SUC_lexicon_tags_37 },
    { 0xf698c947, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xd6b8c949, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b8c952, SUC_lexicon_tags_11 },
    { 0xb538c953, SUC_lexicon_tags_1 },
    { 0xe018c953, SUC_lexicon_tags_66 },
    { 0x1fb8c953, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4538c958, SUC_lexicon_tags_13 },
    { 0xfed8c959, SUC_lexicon_tags_37 },
    { 0xd4f8c95a, SUC_lexicon_tags_12 },
    { 0xa5f8c95a, SUC_lexicon_tags_62 },
    { 0xe2b8c95a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b8c95f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cd8c961, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed8c968, SUC_lexicon_tags_23 },
    { 0xf718c969, SUC_lexicon_tags_9 },
    { 0x5618c96a, SUC_lexicon_tags_6 },
    { 0xdb8c96a, SUC_lexicon_tags_14 },
    { 0x7ab8c969, SUC_lexicon_tags_28 },
    { 0xdb98c96b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf218c976, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb8c97a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x90f8c97c, SUC_lexicon_tags_13 },
    { 0x6418c97d, SUC_lexicon_tags_7 },
    { 0x6bf8c97e, SUC_lexicon_tags_38 },
    { 0x6c38c97e, SUC_lexicon_tags_37 },
    { 0x2298c980, SUC_lexicon_tags_61 },
    { 0xbf98c980, SUC_lexicon_tags_21 },
    { 0x9798c980, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1d8c984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8918c987, SUC_lexicon_tags_61 },
    { 0x4cf8c988, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b8c98c, SUC_lexicon_tags_37 },
    { 0x6c38c98d, SUC_lexicon_tags_17 },
    { 0xc898c98e, SUC_lexicon_tags_17 },
    { 0xb0d8c98e, SUC_lexicon_tags_49 },
    { 0x9118c98e, SUC_lexicon_tags_7 },
    { 0xeaf8c991, SUC_lexicon_tags_37 },
    { 0xfdd8c992, SUC_lexicon_tags_11 },
    { 0xcf58c993, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1378c998, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf8c99e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc298c9a0, SUC_lexicon_tags_25 },
    { 0xce18c9a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc518c9a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb698c9aa, SUC_lexicon_tags_14 },
    { 0xa1b8c9aa, SUC_lexicon_tags_33 },
    { 0x8598c9ac, SUC_lexicon_tags_0 },
    { 0x6e58c9ad, SUC_lexicon_tags_229 },
    { 0xc398c9aa, SUC_lexicon_tags_45 },
    { 0x9dd8c9af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf078c9b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1738c9ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f8c9bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46f8c9bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde38c9c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x158c9c5, SUC_lexicon_tags_1 },
    { 0x6818c9c5, SUC_lexicon_tags_2 },
    { 0x6e58c9c5, SUC_lexicon_tags_25 },
    { 0x5c18c9c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7638c9ce, SUC_lexicon_tags_59 },
    { 0x88d8c9cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b8c9d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f8c9d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c98c9da, SUC_lexicon_tags_31 },
    { 0x1cd8c9db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1998c9de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd8c9e6, SUC_lexicon_tags_7 },
    { 0x9cd8c9e7, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0xf438c9e9, SUC_lexicon_tags_47 },
    { 0xc598c9ea, SUC_lexicon_tags_25 },
    { 0x8238c9eb, SUC_lexicon_tags_27 },
    { 0x81b8c9eb, SUC_lexicon_tags_63 },
    { 0xd478c9eb, SUC_lexicon_tags_1379 },
    { 0x7758c9ee, SUC_lexicon_tags_14 },
    { 0x69f8c9ef, SUC_lexicon_tags_27 },
    { 0x9018c9ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x938c9f3, SUC_lexicon_tags_8 },
    { 0x18c9f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef58c9f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90f8c9f8, SUC_lexicon_tags_84 },
    { 0xd658c9f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa398c9fc, SUC_lexicon_tags_35 },
    { 0x4918c9fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d8ca01, SUC_lexicon_tags_1 },
    { 0xf4d8ca02, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb2b8ca04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa198ca07, SUC_lexicon_tags_17 },
    { 0x20f8ca07, SUC_lexicon_tags_14 },
    { 0x4818ca08, SUC_lexicon_tags_2 },
    { 0xf858ca0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x44d8ca0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe778ca11, SUC_lexicon_tags_45 },
    { 0x878ca11, SUC_lexicon_tags_2 },
    { 0xf338ca13, SUC_lexicon_tags_12 },
    { 0x6db8ca14, SUC_lexicon_tags_25 },
    { 0xbcd8ca15, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb678ca17, SUC_lexicon_tags_48 },
    { 0x7098ca18, SUC_lexicon_tags_17 },
    { 0x6578ca17, SUC_lexicon_tags_17 },
    { 0x61d8ca1a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1078ca1d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc6b8ca1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6458ca25, SUC_lexicon_tags_23 },
    { 0xf8ca26, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6818ca28, SUC_lexicon_tags_12 },
    { 0xfa38ca28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91f8ca2b, SUC_lexicon_tags_82 },
    { 0xfbf8ca2b, SUC_lexicon_tags_25 },
    { 0x7e98ca2d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df8ca34, SUC_lexicon_tags_25 },
    { 0x4a98ca35, SUC_lexicon_tags_25 },
    { 0xb838ca35, SUC_lexicon_tags_593 },
    { 0xe338ca35, SUC_lexicon_tags_37 },
    { 0xf298ca38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe078ca3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca38ca3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb8ca42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8138ca44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718ca4b, SUC_lexicon_tags_27 },
    { 0x91d8ca4c, SUC_lexicon_tags_31 },
    { 0x24b8ca4d, SUC_lexicon_tags_27 },
    { 0x5d38ca4e, SUC_lexicon_tags_182 },
    { 0xdfb8ca4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f8ca55, SUC_lexicon_tags_2 },
    { 0xe698ca55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f38ca5b, SUC_lexicon_tags_25 },
    { 0xffb8ca5c, SUC_lexicon_tags_31 },
    { 0x4eb8ca5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b98ca62, SUC_lexicon_tags_37 },
    { 0xadb8ca63, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1018ca68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e98ca6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf8ca6d, SUC_lexicon_tags_25 },
    { 0x7d38ca6d, SUC_lexicon_tags_61 },
    { 0x5bd8ca6f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x24b8ca71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38ca74, SUC_lexicon_tags_31 },
    { 0x3a18ca75, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5338ca7d, SUC_lexicon_tags_19 },
    { 0xc0f8ca7e, SUC_lexicon_tags_15 },
    { 0xa918ca7f, SUC_lexicon_tags_15 },
    { 0xa178ca80, SUC_lexicon_tags_48 },
    { 0xa1d8ca81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf58ca83, SUC_lexicon_tags_37 },
    { 0xecf8ca83, SUC_lexicon_tags_37 },
    { 0xffd8ca85, SUC_lexicon_tags_1 },
    { 0x3cd8ca86, SUC_lexicon_tags_17 },
    { 0x4398ca87, SUC_lexicon_tags_23 },
    { 0x9af8ca84, SUC_lexicon_tags_11 },
    { 0xd338ca89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd018ca8e, SUC_lexicon_tags_26 },
    { 0x8038ca8f, SUC_lexicon_tags_11 },
    { 0xfef8ca8f, SUC_lexicon_tags_118 },
    { 0x43b8ca91, SUC_lexicon_tags_38 },
    { 0x7458ca92, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8178ca98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde38ca9a, SUC_lexicon_tags_17 },
    { 0x4498ca9a, SUC_lexicon_tags_49 },
    { 0xfab8ca9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd878caa3, SUC_lexicon_tags_11 },
    { 0x3e18caa4, SUC_lexicon_tags_4 },
    { 0x2638caa5, SUC_lexicon_tags_42 },
    { 0x5b58caa3, SUC_lexicon_tags_14 },
    { 0xf0f8caa7, SUC_lexicon_tags_28 },
    { 0x35f8caa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa58caaa, SUC_lexicon_tags_25 },
    { 0x3e98caaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9df8caad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca18cab4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f98cac2, SUC_lexicon_tags_7 },
    { 0x51b8cac3, SUC_lexicon_tags_11 },
    { 0xcf98cac3, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xad98cac6, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b38cac9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff98cacd, SUC_lexicon_tags_25 },
    { 0x6f18cace, SUC_lexicon_tags_14 },
    { 0xd498cacf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f8cad3, SUC_lexicon_tags_48 },
    { 0x4a58cad4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa78cad8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea98cadc, SUC_lexicon_tags_46 },
    { 0x79d8cadd, SUC_lexicon_tags_17 },
    { 0x5f78cade, SUC_lexicon_tags_17 },
    { 0x3458cadf, SUC_lexicon_tags_37 },
    { 0x8778cade, SUC_lexicon_tags_23 },
    { 0xbeb8cae0, SUC_lexicon_tags_7 },
    { 0x8b58cadd, SUC_lexicon_tags_38 },
    { 0xef58cae0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa58cae8, SUC_lexicon_tags_46 },
    { 0xe138cae9, SUC_lexicon_tags_8 },
    { 0xc518caea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc958caef, SUC_lexicon_tags_49 },
    { 0xc38caef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f18caf4, SUC_lexicon_tags_11 },
    { 0x6df8caf5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb798caf7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd8cafa, SUC_lexicon_tags_23 },
    { 0x7b98cafb, SUC_lexicon_tags_1 },
    { 0xd578cafb, SUC_lexicon_tags_12 },
    { 0x6f18cafb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde98cb00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6818cb04, SUC_lexicon_tags_59 },
    { 0x1d58cb05, SUC_lexicon_tags_23 },
    { 0x1d78cb04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d8cb0a, SUC_lexicon_tags_34 },
    { 0x7e18cb0b, SUC_lexicon_tags_1 },
    { 0x9758cb0c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7198cb0e, SUC_lexicon_tags_29 },
    { 0x5318cb0e, SUC_lexicon_tags_66 },
    { 0x1a38cb0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f8cb15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d8cb1a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f8cb1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc418cb20, SUC_lexicon_tags_11 },
    { 0xea98cb20, SUC_lexicon_tags_42 },
    { 0x8e98cb21, SUC_lexicon_tags_63 },
    { 0x7978cb22, SUC_lexicon_tags_1 },
    { 0xcbb8cb22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b98cb26, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4538cb29, SUC_lexicon_tags_6 },
    { 0x7bf8cb2a, SUC_lexicon_tags_19 },
    { 0x26b8cb2b, SUC_lexicon_tags_77 },
    { 0xa038cb2c, SUC_lexicon_tags_63 },
    { 0x32f8cb2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2998cb33, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0xe998cb35, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8218cb37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f8cb3c, SUC_lexicon_tags_27 },
    { 0x8ef8cb3d, SUC_lexicon_tags_0 },
    { 0x76d8cb3e, SUC_lexicon_tags_11 },
    { 0x598cb3f, SUC_lexicon_tags_12 },
    { 0xb8d8cb40, SUC_lexicon_tags_10 },
    { 0xb3f8cb40, SUC_lexicon_tags_31 },
    { 0xd378cb41, SUC_lexicon_tags_29 },
    { 0x2a58cb40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b8cb4d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6438cb51, SUC_lexicon_tags_30 },
    { 0xe5d8cb52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53f8cb54, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3958cb5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f58cb5e, SUC_lexicon_tags_49 },
    { 0xbb58cb5f, SUC_lexicon_tags_47 },
    { 0xb3d8cb60, SUC_lexicon_tags_17 },
    { 0xa878cb61, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d8cb65, SUC_lexicon_tags_27 },
    { 0xc58cb66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d8cb6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7578cb6f, SUC_lexicon_tags_26 },
    { 0xacf8cb70, SUC_lexicon_tags_2 },
    { 0x8b18cb71, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x24b8cb73, SUC_lexicon_tags_11 },
    { 0x5bf8cb74, SUC_lexicon_tags_37 },
    { 0xaad8cb74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b8cb79, SUC_lexicon_tags_12 },
    { 0x9b98cb7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb718cb7f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb8cb84, SUC_lexicon_tags_27 },
    { 0x92f8cb85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd8cb8f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc5b8cb91, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f38cb95, SUC_lexicon_tags_12 },
    { 0x2558cb96, SUC_lexicon_tags_31 },
    { 0xbf8cb97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc218cb9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c18cb9f, SUC_lexicon_tags_2 },
    { 0x3698cb9f, SUC_lexicon_tags_23 },
    { 0xb0d8cba1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd398cba4, SUC_lexicon_tags_1 },
    { 0x6a18cba5, SUC_lexicon_tags_1 },
    { 0x6fb8cba4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa918cba9, SUC_lexicon_tags_56 },
    { 0x70b8cbaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c98cbb0, SUC_lexicon_tags_11 },
    { 0xef18cbb0, SUC_lexicon_tags_17 },
    { 0x9cf8cbb2, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0x7678cbb5, SUC_lexicon_tags_6 },
    { 0x7f38cbb6, SUC_lexicon_tags_8 },
    { 0x3d98cbb5, SUC_lexicon_tags_28 },
    { 0x45b8cbb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b58cbbf, SUC_lexicon_tags_12 },
    { 0x5a18cbc0, SUC_lexicon_tags_39 },
    { 0x8d8cbc1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa698cbc3, SUC_lexicon_tags_19 },
    { 0xa5b8cbc4, SUC_lexicon_tags_3 },
    { 0xdf8cbc5, SUC_lexicon_tags_43 },
    { 0xaaf8cbc6, SUC_lexicon_tags_30 },
    { 0x7d18cbc7, SUC_lexicon_tags_7 },
    { 0x9f78cbc8, SUC_lexicon_tags_47 },
    { 0x39b8cbc9, SUC_lexicon_tags_24 },
    { 0xee18cbc5, SUC_lexicon_tags_1 },
    { 0xdab8cbcb, SUC_lexicon_tags_29 },
    { 0x1d58cbcc, SUC_lexicon_tags_7 },
    { 0x2418cbc5, SUC_lexicon_tags_11 },
    { 0xe738cbce, SUC_lexicon_tags_37 },
    { 0x3d98cbce, SUC_lexicon_tags_8 },
    { 0xa358cbca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f8cbd7, SUC_lexicon_tags_12 },
    { 0x8138cbd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b8cbe0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a58cbe3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd558cbe5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdc78cbe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f18cbeb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6c18cbed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc398cbef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42d8cbf1, SUC_lexicon_tags_47 },
    { 0xba58cbf1, SUC_lexicon_tags_47 },
    { 0x2878cbf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d8cbfb, SUC_lexicon_tags_7 },
    { 0xab98cbfc, SUC_lexicon_tags_1 },
    { 0xf978cbfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf058cbff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4698cc02, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x7578cc05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb78cc0c, SUC_lexicon_tags_20 },
    { 0x4bd8cc0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d98cc15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b18cc17, SUC_lexicon_tags_45 },
    { 0x1758cc18, SUC_lexicon_tags_31 },
    { 0xc2b8cc18, SUC_lexicon_tags_37 },
    { 0x4e58cc17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8cc21, SUC_lexicon_tags_15 },
    { 0x7138cc21, SUC_lexicon_tags_11 },
    { 0xf18cc21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef8cc30, SUC_lexicon_tags_230 },
    { 0xe638cc31, SUC_lexicon_tags_37 },
    { 0x21d8cc32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb778cc35, SUC_lexicon_tags_29 },
    { 0x6e18cc36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b8cc3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x158cc3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1238cc43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x52b8cc45, SUC_lexicon_tags_65 },
    { 0x9498cc46, SUC_lexicon_tags_85 },
    { 0x7a78cc46, SUC_lexicon_tags_37 },
    { 0x85f8cc48, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d8cc4d, SUC_lexicon_tags_60 },
    { 0xb3b8cc4e, SUC_lexicon_tags_31 },
    { 0x1878cc4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa418cc51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e18cc56, SUC_lexicon_tags_37 },
    { 0xc218cc57, SUC_lexicon_tags_39 },
    { 0xc9f8cc56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f8cc5f, SUC_lexicon_tags_13 },
    { 0xf18cc60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b58cc62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7918cc67, SUC_lexicon_tags_37 },
    { 0x8538cc67, SUC_lexicon_tags_27 },
    { 0xc418cc67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe158cc6c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf298cc6e, SUC_lexicon_tags_118 },
    { 0x9cb8cc6e, SUC_lexicon_tags_69 },
    { 0xac58cc70, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc378cc74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f8cc76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f8cc7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb98cc85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf1b8cc87, SUC_lexicon_tags_48 },
    { 0xda58cc88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa938cc8a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f8cc8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9d8cc8f, SUC_lexicon_tags_70 },
    { 0x3998cc8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b18cc93, SUC_lexicon_tags_39 },
    { 0xea58cc94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b18cc98, SUC_lexicon_tags_60 },
    { 0x5ab8cc99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd8cc9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca58cc9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9538cca0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1998cca2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d8cca6, SUC_lexicon_tags_47 },
    { 0xba18cca7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7598ccab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ccaf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x54f8ccb1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf8ccb5, SUC_lexicon_tags_15 },
    { 0xee58ccb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf238ccbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xecd8ccbf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc198ccc2, SUC_lexicon_tags_8 },
    { 0xc338ccc3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e38ccc7, SUC_lexicon_tags_48 },
    { 0x1798ccc8, SUC_lexicon_tags_40 },
    { 0x6778ccc7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed8cccd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b38cccf, SUC_lexicon_tags_29 },
    { 0x3978ccd0, SUC_lexicon_tags_19 },
    { 0xfd58ccd1, SUC_lexicon_tags_55 },
    { 0xb6b8ccd2, SUC_lexicon_tags_60 },
    { 0xc818ccd1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d8ccd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc298ccdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b8ccdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b8cce2, SUC_lexicon_tags_23 },
    { 0xd318cce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x218cce5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d8cce9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe238ccee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8398ccf0, SUC_lexicon_tags_11 },
    { 0x7798ccf1, SUC_lexicon_tags_0 },
    { 0x2b38ccf2, SUC_lexicon_tags_15 },
    { 0x2d8ccf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5498ccf5, SUC_lexicon_tags_45 },
    { 0xd038ccf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf638ccf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3cf8ccfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb498ccfc, SUC_lexicon_tags_37 },
    { 0x6498ccfd, SUC_lexicon_tags_55 },
    { 0x3cf8ccfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd678cd02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d78cd09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9618cd0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b38cd0f, SUC_lexicon_tags_118 },
    { 0xff18cd0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8cd17, SUC_lexicon_tags_11 },
    { 0x5558cd17, SUC_lexicon_tags_11 },
    { 0xca38cd17, SUC_lexicon_tags_27 },
    { 0xd2d8cd17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8d8cd1c, SUC_lexicon_tags_214 },
    { 0x8818cd1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f78cd20, SUC_lexicon_tags_45 },
    { 0x7d8cd20, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xec38cd24, SUC_lexicon_tags_25 },
    { 0x1098cd25, SUC_lexicon_tags_8 },
    { 0xc4f8cd24, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb8cd2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd38cd3a, SUC_lexicon_tags_204 },
    { 0x3178cd3b, SUC_lexicon_tags_38 },
    { 0xa058cd3b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe5b8cd3e, SUC_lexicon_tags_2 },
    { 0xdc98cd3e, SUC_lexicon_tags_28 },
    { 0xb38cd3e, SUC_lexicon_tags_11 },
    { 0xc398cd41, SUC_lexicon_tags_79 },
    { 0xc698cd42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b8cd48, SUC_lexicon_tags_2 },
    { 0xbd98cd48, SUC_lexicon_tags_2 },
    { 0x8a98cd4a, SUC_lexicon_tags_9 },
    { 0x7838cd4b, SUC_lexicon_tags_11 },
    { 0xca78cd4a, SUC_lexicon_tags_17 },
    { 0xc3f8cd4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe078cd4f, SUC_lexicon_tags_37 },
    { 0xd198cd4f, SUC_lexicon_tags_17 },
    { 0xe5d8cd4f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4878cd56, SUC_lexicon_tags_14 },
    { 0xd758cd57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b8cd5a, SUC_lexicon_tags_43 },
    { 0x2778cd5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5078cd5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f98cd5f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52b8cd63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4138cd68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b8cd6f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed8cd73, SUC_lexicon_tags_2 },
    { 0x1898cd74, SUC_lexicon_tags_75 },
    { 0xdd98cd75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4898cd77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6618cd79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe358cd7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe318cd7f, SUC_lexicon_tags_45 },
    { 0xc5b8cd80, SUC_lexicon_tags_144 },
    { 0xcfd8cd81, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd38cd86, SUC_lexicon_tags_60 },
    { 0x6958cd87, SUC_lexicon_tags_17 },
    { 0xd618cd88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29f8cd8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd598cd8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92f8cd90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9538cd95, SUC_lexicon_tags_37 },
    { 0x6618cd96, SUC_lexicon_tags_11 },
    { 0xf438cd97, SUC_lexicon_tags_49 },
    { 0x4758cd98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb218cd9a, SUC_lexicon_tags_15 },
    { 0x8458cd9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee78cd9f, SUC_lexicon_tags_25 },
    { 0x518cd9f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7658cda2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe198cda5, SUC_lexicon_tags_17 },
    { 0x3898cda5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa238cdb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9298cdb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa518cdbe, SUC_lexicon_tags_27 },
    { 0xb638cdbf, SUC_lexicon_tags_138 },
    { 0xf878cdc0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3258cdc2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6b78cdc4, SUC_lexicon_tags_37 },
    { 0xd8cdc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d8cdc8, SUC_lexicon_tags_16 },
    { 0xb978cdc8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe798cdcb, SUC_lexicon_tags_38 },
    { 0x4898cdcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f8cdcf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1418cdd1, SUC_lexicon_tags_11 },
    { 0x33d8cdd2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5838cdd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c58cddc, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a78cddf, SUC_lexicon_tags_63 },
    { 0x19b8cde0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e78cde3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b8cde8, SUC_lexicon_tags_11 },
    { 0x3758cde8, SUC_lexicon_tags_47 },
    { 0x26d8cde8, SUC_lexicon_tags_47 },
    { 0xab58cdeb, SUC_lexicon_tags_17 },
    { 0x44d8cdeb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d38cdee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d8cdff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4518ce02, SUC_lexicon_tags_17 },
    { 0xd18ce02, SUC_lexicon_tags_6 },
    { 0xeef8ce04, SUC_lexicon_tags_28 },
    { 0x1078ce05, SUC_lexicon_tags_12 },
    { 0x7138ce06, SUC_lexicon_tags_29 },
    { 0xca18ce06, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x3ed8ce09, SUC_lexicon_tags_30 },
    { 0x5d78ce0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9698ce0e, SUC_lexicon_tags_37 },
    { 0x2f98ce0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1b8ce11, SUC_lexicon_tags_7 },
    { 0x8f18ce12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7318ce19, SUC_lexicon_tags_15 },
    { 0x7058ce1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5758ce1e, SUC_lexicon_tags_57 },
    { 0x6b98ce1f, SUC_lexicon_tags_17 },
    { 0x5898ce1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xef18ce23, SUC_lexicon_tags_12 },
    { 0x2878ce23, SUC_lexicon_tags_65 },
    { 0xdf8ce23, SUC_lexicon_tags_17 },
    { 0xdcd8ce23, SUC_lexicon_tags_63 },
    { 0xf438ce27, SUC_lexicon_tags_19 },
    { 0x8d58ce28, SUC_lexicon_tags_8 },
    { 0x6f38ce28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51d8ce2b, SUC_lexicon_tags_60 },
    { 0xee18ce2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xedb8ce2e, SUC_lexicon_tags_29 },
    { 0xeed8ce2f, SUC_lexicon_tags_11 },
    { 0xb978ce30, SUC_lexicon_tags_21 },
    { 0x7ed8ce30, SUC_lexicon_tags_31 },
    { 0x3038ce2e, SUC_lexicon_tags_15 },
    { 0x78ce32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa38ce35, SUC_lexicon_tags_68 },
    { 0xb8ce36, SUC_lexicon_tags_19 },
    { 0x898ce37, SUC_lexicon_tags_45 },
    { 0x898ce38, SUC_lexicon_tags_1 },
    { 0xca18ce39, SUC_lexicon_tags_15 },
    { 0xbb78ce3a, SUC_lexicon_tags_9 },
    { 0xaed8ce3a, SUC_lexicon_tags_12 },
    { 0x44b8ce3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1458ce3e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd818ce42, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x1e98ce44, SUC_lexicon_tags_23 },
    { 0x8278ce45, SUC_lexicon_tags_60 },
    { 0x2f8ce46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f8ce4b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d8ce4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3298ce52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8ce55, SUC_lexicon_tags_33 },
    { 0x658ce56, SUC_lexicon_tags_17 },
    { 0x8fb8ce55, SUC_lexicon_tags_1 },
    { 0xb358ce58, SUC_lexicon_tags_12 },
    { 0x9bf8ce58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5638ce5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b8ce63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f8ce72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61f8ce74, SUC_lexicon_tags_15 },
    { 0xe578ce75, SUC_lexicon_tags_15 },
    { 0xddb8ce76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9658ce79, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f8ce7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb8ce82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd2f8ce84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd898ce86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5cf8ce88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb178ce8c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd8ce90, SUC_lexicon_tags_61 },
    { 0xbf58ce91, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa358ce95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe198ce97, SUC_lexicon_tags_37 },
    { 0xa238ce98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3f8ce9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7638ce9d, SUC_lexicon_tags_27 },
    { 0x4618ce9e, SUC_lexicon_tags_11 },
    { 0xbb8ce9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1838cea2, SUC_lexicon_tags_37 },
    { 0x6cf8cea3, SUC_lexicon_tags_2 },
    { 0x3498cea3, SUC_lexicon_tags_7 },
    { 0x8698cea4, SUC_lexicon_tags_60 },
    { 0x5418cea4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb658ceac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9598ceae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb78ceb1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf78ceb4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff8ceb7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f8ceba, SUC_lexicon_tags_23 },
    { 0x1618ceba, SUC_lexicon_tags_37 },
    { 0x9818cebc, SUC_lexicon_tags_15 },
    { 0x8718cebd, SUC_lexicon_tags_48 },
    { 0x9e38cebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7098cec2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f8cec8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f8cecb, SUC_lexicon_tags_6 },
    { 0xa1f8cecb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f8ced0, SUC_lexicon_tags_19 },
    { 0xbf58ced0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fd8ced3, SUC_lexicon_tags_19 },
    { 0x658ced4, SUC_lexicon_tags_37 },
    { 0xd0f8ced5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x59d8ced7, SUC_lexicon_tags_2 },
    { 0xfd18ced7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1738ceda, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb18cedd, SUC_lexicon_tags_19 },
    { 0x9698cede, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d8cee9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6998ceeb, SUC_lexicon_tags_2 },
    { 0x79f8ceec, SUC_lexicon_tags_37 },
    { 0x8f18ceed, SUC_lexicon_tags_45 },
    { 0x74d8ceee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf8cef2, SUC_lexicon_tags_25 },
    { 0x22d8cef3, SUC_lexicon_tags_1 },
    { 0x8778cef2, SUC_lexicon_tags_47 },
    { 0xf178cef3, SUC_lexicon_tags_21 },
    { 0x3038cef3, SUC_lexicon_tags_2 },
    { 0x9b78cef7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b38cefa, SUC_lexicon_tags_57 },
    { 0xe8b8cefb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6d8cefd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1798cf00, SUC_lexicon_tags_1 },
    { 0xc58cf00, SUC_lexicon_tags_23 },
    { 0x7ef8cf02, SUC_lexicon_tags_17 },
    { 0xe438cf01, SUC_lexicon_tags_7 },
    { 0x7418cf04, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd8cf12, SUC_lexicon_tags_25 },
    { 0x2878cf13, SUC_lexicon_tags_16 },
    { 0xf718cf13, SUC_lexicon_tags_55 },
    { 0x6d78cf15, SUC_lexicon_tags_25 },
    { 0x1158cf12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b38cf19, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe058cf20, SUC_lexicon_tags_37 },
    { 0x2c18cf21, SUC_lexicon_tags_37 },
    { 0x3058cf20, SUC_lexicon_tags_17 },
    { 0xc2d8cf23, SUC_lexicon_tags_664 },
    { 0x1118cf24, SUC_lexicon_tags_2 },
    { 0x8638cf24, SUC_lexicon_tags_11 },
    { 0x3558cf20, SUC_lexicon_tags_29 },
    { 0x9158cf27, SUC_lexicon_tags_1 },
    { 0x18f8cf28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9698cf2a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x978cf2c, SUC_lexicon_tags_11 },
    { 0xd0f8cf2d, SUC_lexicon_tags_4 },
    { 0x3d98cf2c, SUC_lexicon_tags_28 },
    { 0x6a98cf2f, SUC_lexicon_tags_38 },
    { 0x6898cf2e, SUC_lexicon_tags_23 },
    { 0x4ff8cf31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b8cf34, SUC_lexicon_tags_15 },
    { 0xec38cf35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2d98cf37, SUC_lexicon_tags_37 },
    { 0xe458cf37, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a78cf3d, SUC_lexicon_tags_65 },
    { 0x2cd8cf3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b8cf44, SUC_lexicon_tags_12 },
    { 0x4b38cf45, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3b38cf47, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5898cf49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df8cf52, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5758cf55, SUC_lexicon_tags_69 },
    { 0xdbb8cf56, SUC_lexicon_tags_39 },
    { 0xbff8cf55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a58cf5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd8cf5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b78cf61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3258cf63, SUC_lexicon_tags_37 },
    { 0x7d8cf64, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb8cf68, SUC_lexicon_tags_17 },
    { 0x5418cf68, SUC_lexicon_tags_38 },
    { 0x87f8cf6a, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f8cf6d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d8cf76, SUC_lexicon_tags_19 },
    { 0x5898cf77, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x818cf7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4cd8cf7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc038cf7e, SUC_lexicon_tags_25 },
    { 0xf218cf7e, SUC_lexicon_tags_55 },
    { 0x6c58cf7f, SUC_lexicon_tags_15 },
    { 0x6078cf81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b8cf8a, SUC_lexicon_tags_15 },
    { 0x3f18cf8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa498cf8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e98cf90, SUC_lexicon_tags_17 },
    { 0x1b18cf91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b8cf94, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb98cf97, SUC_lexicon_tags_29 },
    { 0xe278cf98, SUC_lexicon_tags_27 },
    { 0x49d8cf99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb698cf9d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xac18cf9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce38cfa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb8cfa7, SUC_lexicon_tags_11 },
    { 0xce98cfa8, SUC_lexicon_tags_124 },
    { 0xf8b8cfa8, SUC_lexicon_tags_23 },
    { 0xe618cfaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x818cfad, SUC_lexicon_tags_17 },
    { 0xb4f8cfae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce78cfb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4518cfb7, SUC_lexicon_tags_25 },
    { 0x8b78cfb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x66f8cfba, SUC_lexicon_tags_43 },
    { 0x2798cfbb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a18cfc1, SUC_lexicon_tags_47 },
    { 0xd4f8cfc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb58cfc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d8cfcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4738cfd3, SUC_lexicon_tags_47 },
    { 0xb658cfd4, SUC_lexicon_tags_43 },
    { 0x71d8cfd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed8cfda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b8cfdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2438cfe6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb8cfe9, SUC_lexicon_tags_1 },
    { 0x98cfea, SUC_lexicon_tags_27 },
    { 0xd1d8cfeb, SUC_lexicon_tags_38 },
    { 0xe158cfeb, SUC_lexicon_tags_1 },
    { 0x98d8cfe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe98cff2, SUC_lexicon_tags_15 },
    { 0xa038cff3, SUC_lexicon_tags_70 },
    { 0x82d8cff4, SUC_lexicon_tags_24 },
    { 0x92f8cff5, SUC_lexicon_tags_25 },
    { 0xb238cff6, SUC_lexicon_tags_47 },
    { 0xda38cff4, SUC_lexicon_tags_4 },
    { 0x9798cff8, SUC_lexicon_tags_17 },
    { 0x5ad8cff9, SUC_lexicon_tags_15 },
    { 0xf498cffa, SUC_lexicon_tags_47 },
    { 0x8558cffa, SUC_lexicon_tags_7 },
    { 0x1d18cff8, SUC_lexicon_tags_2 },
    { 0xe058cffd, SUC_lexicon_tags_17 },
    { 0x11d8cff7, SUC_lexicon_tags_39 },
    { 0x66d8cfff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa778d001, SUC_lexicon_tags_7 },
    { 0xdb78d001, SUC_lexicon_tags_65 },
    { 0x1818d003, SUC_lexicon_tags_43 },
    { 0x8dd8d004, SUC_lexicon_tags_66 },
    { 0x5cd8d003, SUC_lexicon_tags_11 },
    { 0x3cd8d006, SUC_lexicon_tags_11 },
    { 0x26b8d006, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf538d00c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c78d01a, SUC_lexicon_tags_1 },
    { 0x37b8d01a, SUC_lexicon_tags_14 },
    { 0xec18d01a, SUC_lexicon_tags_3 },
    { 0x5fd8d01d, SUC_lexicon_tags_12 },
    { 0x158d01d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x94d8d020, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf998d026, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23d8d028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1858d02d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2238d031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b8d034, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad98d037, SUC_lexicon_tags_37 },
    { 0x45d8d037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa918d03a, SUC_lexicon_tags_11 },
    { 0x8338d03b, SUC_lexicon_tags_7 },
    { 0x6538d03c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9158d03f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7478d048, SUC_lexicon_tags_37 },
    { 0x1bd8d049, SUC_lexicon_tags_8 },
    { 0x1018d04a, SUC_lexicon_tags_8 },
    { 0x5a38d04b, SUC_lexicon_tags_33 },
    { 0xc938d04c, SUC_lexicon_tags_35 },
    { 0x3d38d04c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6d38d04f, SUC_lexicon_tags_65 },
    { 0xde18d050, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b58d054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9238d057, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f8d05a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe758d05d, SUC_lexicon_tags_11 },
    { 0x1e38d05d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79b8d060, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd8d06e, SUC_lexicon_tags_37 },
    { 0x3e98d06f, SUC_lexicon_tags_4 },
    { 0x50b8d06f, SUC_lexicon_tags_160 },
    { 0x8798d06f, SUC_lexicon_tags_33 },
    { 0xb998d06f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x878d079, SUC_lexicon_tags_2 },
    { 0x4398d07a, SUC_lexicon_tags_43 },
    { 0xad8d079, SUC_lexicon_tags_37 },
    { 0x1ef8d07a, SUC_lexicon_tags_2 },
    { 0x7218d07d, SUC_lexicon_tags_55 },
    { 0x75f8d07e, SUC_lexicon_tags_19 },
    { 0xccf8d07d, SUC_lexicon_tags_14 },
    { 0x8298d080, SUC_lexicon_tags_12 },
    { 0x13f8d07a, SUC_lexicon_tags_11 },
    { 0x8bf8d082, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d8d085, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd8d08c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4bb8d08e, SUC_lexicon_tags_16 },
    { 0xb178d08f, SUC_lexicon_tags_7 },
    { 0x9ef8d090, SUC_lexicon_tags_15 },
    { 0x87b8d091, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3818d097, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc78d09b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f8d09e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f8d0a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b78d0a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb8d0ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55d8d0b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe318d0b2, SUC_lexicon_tags_25 },
    { 0x48f8d0b2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3d8d0b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f8d0b8, SUC_lexicon_tags_14 },
    { 0x5298d0b9, SUC_lexicon_tags_216 },
    { 0xac38d0ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f98d0bd, SUC_lexicon_tags_12 },
    { 0xa138d0be, SUC_lexicon_tags_7 },
    { 0x40f8d0be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2298d0c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb98d0c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b8d0ca, SUC_lexicon_tags_29 },
    { 0x6e98d0cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf8d0cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a98d0d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd138d0d6, SUC_lexicon_tags_17 },
    { 0x7a58d0d7, SUC_lexicon_tags_3 },
    { 0xf1f8d0d8, SUC_lexicon_tags_10 },
    { 0x3f78d0d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e58d0e0, SUC_lexicon_tags_9 },
    { 0x2238d0e1, SUC_lexicon_tags_45 },
    { 0x2cd8d0e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2078d0e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7d8d0e6, SUC_lexicon_tags_27 },
    { 0x6778d0e6, SUC_lexicon_tags_13 },
    { 0x36f8d0e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a38d0ee, SUC_lexicon_tags_33 },
    { 0xfd58d0ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf158d0f2, SUC_lexicon_tags_38 },
    { 0xdab8d0f3, SUC_lexicon_tags_37 },
    { 0x80b8d0f2, SUC_lexicon_tags_39 },
    { 0x5ef8d0f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb298d0f9, SUC_lexicon_tags_25 },
    { 0x76b8d0fa, SUC_lexicon_tags_79 },
    { 0x1f98d0f9, SUC_lexicon_tags_55 },
    { 0x6f78d0fc, SUC_lexicon_tags_17 },
    { 0x9dd8d0fc, SUC_lexicon_tags_25 },
    { 0x458d0fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x10b8d100, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8578d105, SUC_lexicon_tags_12 },
    { 0x8038d105, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe218d108, SUC_lexicon_tags_31 },
    { 0x118d108, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a98d10d, SUC_lexicon_tags_1 },
    { 0x3318d10e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b78d111, SUC_lexicon_tags_23 },
    { 0xe478d112, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7138d116, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab8d11c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1238d124, SUC_lexicon_tags_19 },
    { 0x5cb8d124, SUC_lexicon_tags_56 },
    { 0xa9f8d126, SUC_lexicon_tags_17 },
    { 0xe498d126, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f8d12a, SUC_lexicon_tags_33 },
    { 0x2c78d12b, SUC_lexicon_tags_14 },
    { 0x91b8d12b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d8d12f, SUC_lexicon_tags_49 },
    { 0xb178d130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b8d133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8858d136, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe618d138, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f8d13d, SUC_lexicon_tags_17 },
    { 0xf838d13e, SUC_lexicon_tags_49 },
    { 0xcbf8d13f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe38d143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f8d147, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d38d14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef8d14e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea78d150, SUC_lexicon_tags_25 },
    { 0x81b8d151, SUC_lexicon_tags_25 },
    { 0x77b8d152, SUC_lexicon_tags_9 },
    { 0x2178d151, SUC_lexicon_tags_1 },
    { 0xe9d8d153, SUC_lexicon_tags_15 },
    { 0xb358d155, SUC_lexicon_tags_25 },
    { 0xaf8d156, SUC_lexicon_tags_11 },
    { 0x8ed8d157, SUC_lexicon_tags_26 },
    { 0x7b78d158, SUC_lexicon_tags_25 },
    { 0x82f8d158, SUC_lexicon_tags_17 },
    { 0x5078d15a, SUC_lexicon_tags_3 },
    { 0x83b8d15a, SUC_lexicon_tags_17 },
    { 0xf018d15c, SUC_lexicon_tags_2 },
    { 0x4278d15c, SUC_lexicon_tags_37 },
    { 0xe598d15e, SUC_lexicon_tags_37 },
    { 0xc298d159, SUC_lexicon_tags_0 },
    { 0xa478d157, SUC_lexicon_tags_33 },
    { 0x4158d15f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4cd8d163, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf538d167, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb8d169, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd18d16c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e98d172, SUC_lexicon_tags_60 },
    { 0x3ed8d172, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee78d177, SUC_lexicon_tags_31 },
    { 0x1558d177, SUC_lexicon_tags_8 },
    { 0xeb18d177, SUC_lexicon_tags_27 },
    { 0xe878d17a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8d17f, SUC_lexicon_tags_27 },
    { 0x878d17f, SUC_lexicon_tags_40 },
    { 0xe5b8d181, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d38d189, SUC_lexicon_tags_7 },
    { 0xa898d18a, SUC_lexicon_tags_37 },
    { 0xe958d18b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3618d18d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffb8d18f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5218d191, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d8d199, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdb78d19b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458d19f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1ab8d1a1, SUC_lexicon_tags_23 },
    { 0x3eb8d1a2, SUC_lexicon_tags_38 },
    { 0x1658d1a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9478d1a5, SUC_lexicon_tags_18 },
    { 0x9a78d1a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb858d1aa, SUC_lexicon_tags_2 },
    { 0x3838d1ab, SUC_lexicon_tags_47 },
    { 0x62f8d1ac, SUC_lexicon_tags_11 },
    { 0x5198d1ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2bd8d1af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c78d1b3, SUC_lexicon_tags_49 },
    { 0x3db8d1b4, SUC_lexicon_tags_11 },
    { 0xf858d1b5, SUC_lexicon_tags_2 },
    { 0x4338d1b6, SUC_lexicon_tags_3 },
    { 0xdb98d1b7, SUC_lexicon_tags_1 },
    { 0x91b8d1b7, SUC_lexicon_tags_37 },
    { 0xde58d1b9, SUC_lexicon_tags_62 },
    { 0x4278d1ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff98d1bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc78d1c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab8d1c3, SUC_lexicon_tags_1 },
    { 0xcbb8d1c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4098d1c6, SUC_lexicon_tags_19 },
    { 0x5ad8d1c7, SUC_lexicon_tags_21 },
    { 0x5318d1c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb058d1ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd8d1ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa478d1d0, SUC_lexicon_tags_7 },
    { 0x6278d1d1, SUC_lexicon_tags_0 },
    { 0xd4b8d1d2, SUC_lexicon_tags_2 },
    { 0x5a38d1d3, SUC_lexicon_tags_10 },
    { 0x3ff8d1d2, SUC_lexicon_tags_46 },
    { 0xf358d1d2, SUC_lexicon_tags_23 },
    { 0x66f8d1d6, SUC_lexicon_tags_47 },
    { 0x45d8d1d7, SUC_lexicon_tags_48 },
    { 0x3df8d1d6, SUC_lexicon_tags_37 },
    { 0x2778d1d9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd8d1dd, SUC_lexicon_tags_2 },
    { 0xa658d1dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x67d8d1e0, SUC_lexicon_tags_37 },
    { 0x4f18d1e1, SUC_lexicon_tags_47 },
    { 0xff18d1e2, SUC_lexicon_tags_31 },
    { 0xcb78d1e1, SUC_lexicon_tags_37 },
    { 0xfe38d1e4, SUC_lexicon_tags_16 },
    { 0xa0f8d1e3, SUC_lexicon_tags_46 },
    { 0xe278d1e6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b8d1eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8998d1ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1878d1f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa098d1f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9b8d1fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe238d1fd, SUC_lexicon_tags_94 },
    { 0xed18d1fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd8d201, SUC_lexicon_tags_70 },
    { 0xf0f8d202, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5db8d204, SUC_lexicon_tags_96 },
    { 0x38f8d205, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff8d213, SUC_lexicon_tags_9 },
    { 0x9558d213, SUC_lexicon_tags_37 },
    { 0xeb78d215, SUC_lexicon_tags_33 },
    { 0x1fd8d215, SUC_lexicon_tags_47 },
    { 0x7318d217, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d8d222, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x69b8d224, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9f8d226, SUC_lexicon_tags_7 },
    { 0x54d8d227, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7558d230, SUC_lexicon_tags_16 },
    { 0x9738d230, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x178d233, SUC_lexicon_tags_25 },
    { 0x2458d233, SUC_lexicon_tags_12 },
    { 0x3eb8d235, SUC_lexicon_tags_11 },
    { 0x2778d236, SUC_lexicon_tags_60 },
    { 0xb158d237, SUC_lexicon_tags_39 },
    { 0x6f38d238, SUC_lexicon_tags_3 },
    { 0x1f8d238, SUC_lexicon_tags_27 },
    { 0x4658d237, SUC_lexicon_tags_37 },
    { 0xe698d23b, SUC_lexicon_tags_10 },
    { 0x61f8d23b, SUC_lexicon_tags_1 },
    { 0xb518d23c, SUC_lexicon_tags_15 },
    { 0xdb8d234, SUC_lexicon_tags_17 },
    { 0xffd8d23f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8618d242, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9f8d244, SUC_lexicon_tags_46 },
    { 0x3d98d245, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c58d248, SUC_lexicon_tags_136 },
    { 0xae38d249, SUC_lexicon_tags_37 },
    { 0x52f8d248, SUC_lexicon_tags_12 },
    { 0x4eb8d248, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd8d253, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc318d257, SUC_lexicon_tags_25 },
    { 0x54f8d257, SUC_lexicon_tags_14 },
    { 0xc918d259, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf8d25d, SUC_lexicon_tags_2 },
    { 0x96b8d25e, SUC_lexicon_tags_40 },
    { 0xaff8d25e, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x8e18d261, SUC_lexicon_tags_49 },
    { 0x8178d262, SUC_lexicon_tags_44 },
    { 0x9ab8d263, SUC_lexicon_tags_1 },
    { 0x44b8d262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2418d268, SUC_lexicon_tags_31 },
    { 0xc578d268, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6f78d26b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e38d26f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3998d271, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa958d274, SUC_lexicon_tags_25 },
    { 0xb4d8d275, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9f8d277, SUC_lexicon_tags_19 },
    { 0x20d8d278, SUC_lexicon_tags_28 },
    { 0xe6f8d279, SUC_lexicon_tags_7 },
    { 0xbb78d27a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7958d27f, SUC_lexicon_tags_2 },
    { 0x5db8d280, SUC_lexicon_tags_6 },
    { 0x8698d281, SUC_lexicon_tags_39 },
    { 0x57b8d280, SUC_lexicon_tags_37 },
    { 0x6c18d281, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x7c98d285, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2858d287, SUC_lexicon_tags_55 },
    { 0x2f98d288, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7f8d28a, SUC_lexicon_tags_218 },
    { 0xd498d28a, SUC_lexicon_tags_74 },
    { 0xab58d28b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d8d293, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4498d296, SUC_lexicon_tags_19 },
    { 0x6778d297, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b18d29e, SUC_lexicon_tags_92 },
    { 0xac58d29f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ed8d2a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7298d2a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7bf8d2a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b98d2a7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df8d2ac, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf8f8d2ae, SUC_lexicon_tags_17 },
    { 0x7d58d2af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf798d2b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa078d2b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d8d2bc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b8d2bf, SUC_lexicon_tags_37 },
    { 0xbd38d2bf, SUC_lexicon_tags_11 },
    { 0xea18d2bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf8d2c6, SUC_lexicon_tags_15 },
    { 0xf38d2c7, SUC_lexicon_tags_1 },
    { 0xfd78d2c8, SUC_lexicon_tags_25 },
    { 0x838d2c7, SUC_lexicon_tags_206 },
    { 0xf358d2c6, SUC_lexicon_tags_25 },
    { 0x8738d2c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9a38d2cd, SUC_lexicon_tags_11 },
    { 0xded8d2ce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b8d2d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb018d2d8, SUC_lexicon_tags_11 },
    { 0x4cd8d2d9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4738d2db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8078d2e0, SUC_lexicon_tags_61 },
    { 0xf358d2e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6558d2e7, SUC_lexicon_tags_31 },
    { 0xe478d2e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf18d2ea, SUC_lexicon_tags_77 },
    { 0xadd8d2eb, SUC_lexicon_tags_68 },
    { 0xb898d2ec, SUC_lexicon_tags_70 },
    { 0x76d8d2ed, SUC_lexicon_tags_55 },
    { 0xe2f8d2ee, SUC_lexicon_tags_25 },
    { 0x2838d2ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4278d2f2, SUC_lexicon_tags_25 },
    { 0x3998d2f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22f8d2f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd318d2f8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x1898d2fa, SUC_lexicon_tags_15 },
    { 0x4e98d2fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f8d2ff, SUC_lexicon_tags_17 },
    { 0x3918d2ff, SUC_lexicon_tags_108 },
    { 0x3938d301, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5138d304, SUC_lexicon_tags_17 },
    { 0x83f8d305, SUC_lexicon_tags_11 },
    { 0xfc38d304, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4638d309, SUC_lexicon_tags_0 },
    { 0x1738d30a, SUC_lexicon_tags_18 },
    { 0xbb98d309, SUC_lexicon_tags_11 },
    { 0x2098d30a, SUC_lexicon_tags_61 },
    { 0x9038d30d, SUC_lexicon_tags_37 },
    { 0x8cd8d30e, SUC_lexicon_tags_9 },
    { 0x27f8d30f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6878d311, SUC_lexicon_tags_37 },
    { 0x9eb8d311, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd978d317, SUC_lexicon_tags_15 },
    { 0x2db8d318, SUC_lexicon_tags_17 },
    { 0xd1f8d318, SUC_lexicon_tags_9 },
    { 0x18d8d317, SUC_lexicon_tags_8 },
    { 0xc3d8d31b, SUC_lexicon_tags_15 },
    { 0x6918d31c, SUC_lexicon_tags_60 },
    { 0x4298d31d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef18d31f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d78d321, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2f8d323, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd98d326, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce18d328, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5538d32a, SUC_lexicon_tags_12 },
    { 0x8578d32b, SUC_lexicon_tags_4 },
    { 0xe218d32b, SUC_lexicon_tags_10 },
    { 0x1d98d32d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d8d331, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xf218d333, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x598d335, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b8d338, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a78d33b, SUC_lexicon_tags_17 },
    { 0x1378d33b, SUC_lexicon_tags_37 },
    { 0xed58d33d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c58d33f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf8d345, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8858d34e, SUC_lexicon_tags_48 },
    { 0x7d98d34f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a58d354, SUC_lexicon_tags_25 },
    { 0x8f8d355, SUC_lexicon_tags_1 },
    { 0xb7d8d356, SUC_lexicon_tags_37 },
    { 0x8258d354, SUC_lexicon_tags_25 },
    { 0x57d8d355, SUC_lexicon_tags_13 },
    { 0x8438d356, SUC_lexicon_tags_25 },
    { 0x5e58d35a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8738d35c, SUC_lexicon_tags_45 },
    { 0xd278d35d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x5dd8d35f, SUC_lexicon_tags_32 },
    { 0x99d8d360, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb8d367, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed8d36b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c78d36e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f8d371, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f8d376, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff8d37c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd8d381, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f8d384, SUC_lexicon_tags_0 },
    { 0xa258d385, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf238d387, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb8d38a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xc58d38c, SUC_lexicon_tags_185 },
    { 0xa9f8d38d, SUC_lexicon_tags_124 },
    { 0xe338d38c, SUC_lexicon_tags_2 },
    { 0x2ab8d38c, SUC_lexicon_tags_2 },
    { 0x7318d38d, SUC_lexicon_tags_106 },
    { 0x7378d390, SUC_lexicon_tags_15 },
    { 0x9e98d392, SUC_lexicon_tags_1 },
    { 0x6498d393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda78d397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6418d39e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff38d3a0, SUC_lexicon_tags_80 },
    { 0x5d38d3a1, SUC_lexicon_tags_29 },
    { 0xc458d3a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2498d3a7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9378d3a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf538d3af, SUC_lexicon_tags_47 },
    { 0x8218d3b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x418d3b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc498d3b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2418d3b7, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x378d3bc, SUC_lexicon_tags_12 },
    { 0xa958d3bd, SUC_lexicon_tags_33 },
    { 0x8c58d3be, SUC_lexicon_tags_14 },
    { 0x8678d3be, SUC_lexicon_tags_19 },
    { 0xf718d3c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb478d3c6, SUC_lexicon_tags_19 },
    { 0x9fd8d3c7, SUC_lexicon_tags_17 },
    { 0xf9d8d3c8, SUC_lexicon_tags_17 },
    { 0x16b8d3c9, SUC_lexicon_tags_12 },
    { 0x3f8d3ca, SUC_lexicon_tags_7 },
    { 0x37f8d3cb, SUC_lexicon_tags_14 },
    { 0x80f8d3cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a18d3ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaad8d3d0, SUC_lexicon_tags_7 },
    { 0x3e98d3d1, SUC_lexicon_tags_25 },
    { 0x56b8d3d1, SUC_lexicon_tags_19 },
    { 0x9558d3d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b8d3da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58d8d3dc, SUC_lexicon_tags_550 },
    { 0x9258d3dc, SUC_lexicon_tags_49 },
    { 0xa918d3dc, SUC_lexicon_tags_15 },
    { 0xd478d3dd, SUC_lexicon_tags_65 },
    { 0x2598d3e0, SUC_lexicon_tags_17 },
    { 0x3b78d3e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d8d3e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c78d3eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b8d3f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf458d3f8, SUC_lexicon_tags_38 },
    { 0x6478d3f8, SUC_lexicon_tags_17 },
    { 0xc278d3f9, SUC_lexicon_tags_7 },
    { 0xdc58d3f9, SUC_lexicon_tags_12 },
    { 0xddf8d3fa, SUC_lexicon_tags_40 },
    { 0x9a18d3fd, SUC_lexicon_tags_48 },
    { 0xe0d8d3fe, SUC_lexicon_tags_37 },
    { 0x1418d3fb, SUC_lexicon_tags_9 },
    { 0x24f8d400, SUC_lexicon_tags_12 },
    { 0xf6f8d401, SUC_lexicon_tags_60 },
    { 0xb898d401, SUC_lexicon_tags_11 },
    { 0xd418d401, SUC_lexicon_tags_11 },
    { 0x18f8d404, SUC_lexicon_tags_45 },
    { 0xb758d404, SUC_lexicon_tags_21 },
    { 0xc6f8d404, SUC_lexicon_tags_127 },
    { 0xe538d407, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6898d412, SUC_lexicon_tags_31 },
    { 0x8cb8d413, SUC_lexicon_tags_27 },
    { 0x5738d412, SUC_lexicon_tags_2 },
    { 0x4658d415, SUC_lexicon_tags_1 },
    { 0x92f8d414, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbcb8d418, SUC_lexicon_tags_29 },
    { 0x3958d418, SUC_lexicon_tags_38 },
    { 0x8f8d418, SUC_lexicon_tags_47 },
    { 0x4018d418, SUC_lexicon_tags_12 },
    { 0x5a98d418, SUC_lexicon_tags_7 },
    { 0x72b8d41d, SUC_lexicon_tags_19 },
    { 0xe258d41d, SUC_lexicon_tags_0 },
    { 0x4138d41e, SUC_lexicon_tags_27 },
    { 0x898d418, SUC_lexicon_tags_18 },
    { 0x2a18d421, SUC_lexicon_tags_31 },
    { 0x1098d41f, SUC_lexicon_tags_25 },
    { 0xdf8d423, SUC_lexicon_tags_17 },
    { 0xe438d424, SUC_lexicon_tags_37 },
    { 0xa38d425, SUC_lexicon_tags_19 },
    { 0xcc58d41c, SUC_lexicon_tags_7 },
    { 0xbb38d427, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1138d429, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5918d42e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a38d430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9198d434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc718d437, SUC_lexicon_tags_79 },
    { 0xa58d438, SUC_lexicon_tags_112 },
    { 0xb438d439, SUC_lexicon_tags_1 },
    { 0xe658d43a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d8d43e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad8d442, SUC_lexicon_tags_45 },
    { 0xa298d443, SUC_lexicon_tags_23 },
    { 0xf338d444, SUC_lexicon_tags_12 },
    { 0x7f78d444, SUC_lexicon_tags_68 },
    { 0xfcd8d443, SUC_lexicon_tags_29 },
    { 0xcd78d443, SUC_lexicon_tags_2 },
    { 0x2bb8d448, SUC_lexicon_tags_39 },
    { 0x94f8d444, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe398d44b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbcf8d44d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f8d450, SUC_lexicon_tags_17 },
    { 0xad58d451, SUC_lexicon_tags_0 },
    { 0xa078d452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf8d460, SUC_lexicon_tags_17 },
    { 0x278d461, SUC_lexicon_tags_7 },
    { 0x2db8d461, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe18d465, SUC_lexicon_tags_12 },
    { 0xbd58d465, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b98d468, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3478d46f, SUC_lexicon_tags_9 },
    { 0xcdb8d470, SUC_lexicon_tags_17 },
    { 0x27b8d471, SUC_lexicon_tags_27 },
    { 0xec78d471, SUC_lexicon_tags_117 },
    { 0x6db8d472, SUC_lexicon_tags_7 },
    { 0x558d470, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d8d47d, SUC_lexicon_tags_9 },
    { 0xa78d47d, SUC_lexicon_tags_7 },
    { 0x91f8d47f, SUC_lexicon_tags_47 },
    { 0xf6d8d47d, SUC_lexicon_tags_23 },
    { 0x8bd8d481, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d98d483, SUC_lexicon_tags_7 },
    { 0x8f58d484, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a78d48c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d8d490, SUC_lexicon_tags_30 },
    { 0x8718d490, SUC_lexicon_tags_13 },
    { 0x4d58d490, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bd8d494, SUC_lexicon_tags_17 },
    { 0x238d495, SUC_lexicon_tags_17 },
    { 0x1f38d496, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf878d499, SUC_lexicon_tags_25 },
    { 0x6078d499, SUC_lexicon_tags_49 },
    { 0x6df8d499, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xba98d49d, SUC_lexicon_tags_10 },
    { 0x1e78d49e, SUC_lexicon_tags_7 },
    { 0x4e98d49f, SUC_lexicon_tags_43 },
    { 0x4118d49d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5618d4a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa858d4a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40f8d4a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f8d4ae, SUC_lexicon_tags_14 },
    { 0x4878d4af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8d4b4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x16d8d4b6, SUC_lexicon_tags_25 },
    { 0xcad8d4b7, SUC_lexicon_tags_62 },
    { 0x60f8d4b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb418d4bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d58d4bf, SUC_lexicon_tags_45 },
    { 0x7ed8d4bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca78d4c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9618d4c4, SUC_lexicon_tags_2 },
    { 0x1458d4c5, SUC_lexicon_tags_45 },
    { 0xaed8d4c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf8d4ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d8d4ce, SUC_lexicon_tags_11 },
    { 0x5ef8d4cf, SUC_lexicon_tags_47 },
    { 0x8d38d4d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef8d4d3, SUC_lexicon_tags_79 },
    { 0x1af8d4d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78d4d7, SUC_lexicon_tags_25 },
    { 0x9d18d4d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb258d4df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e58d4e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e78d4e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d8d4ea, SUC_lexicon_tags_91 },
    { 0xd8b8d4eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb8d4ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d8d4f1, SUC_lexicon_tags_30 },
    { 0x43d8d4f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa638d4f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7b8d4f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58d8d4f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x578d4fa, SUC_lexicon_tags_7 },
    { 0x5e18d4fb, SUC_lexicon_tags_2 },
    { 0x41b8d4fb, SUC_lexicon_tags_12 },
    { 0xdc38d4fa, SUC_lexicon_tags_2 },
    { 0xb798d4fe, SUC_lexicon_tags_11 },
    { 0xcd98d4ff, SUC_lexicon_tags_1 },
    { 0xb338d500, SUC_lexicon_tags_38 },
    { 0x9058d501, SUC_lexicon_tags_37 },
    { 0x4878d502, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d8d506, SUC_lexicon_tags_23 },
    { 0xc7f8d507, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8758d50c, SUC_lexicon_tags_87 },
    { 0xa118d50d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd78d513, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5d38d515, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb958d519, SUC_lexicon_tags_17 },
    { 0x8818d51a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d8d51e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe98d521, SUC_lexicon_tags_11 },
    { 0xaad8d522, SUC_lexicon_tags_11 },
    { 0x2758d523, SUC_lexicon_tags_34 },
    { 0xe3b8d522, SUC_lexicon_tags_11 },
    { 0x96b8d525, SUC_lexicon_tags_23 },
    { 0x17b8d526, SUC_lexicon_tags_12 },
    { 0x1d38d526, SUC_lexicon_tags_47 },
    { 0x9a98d527, SUC_lexicon_tags_37 },
    { 0x1478d527, SUC_lexicon_tags_33 },
    { 0x73d8d52a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9698d52d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f18d531, SUC_lexicon_tags_31 },
    { 0xe958d531, SUC_lexicon_tags_49 },
    { 0x958d533, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94d8d535, SUC_lexicon_tags_25 },
    { 0x6a58d536, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8118d53d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x178d542, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba18d544, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf8d547, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2238d549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b98d54d, SUC_lexicon_tags_47 },
    { 0xd5d8d54e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b8d552, SUC_lexicon_tags_30 },
    { 0x8498d552, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b8d55f, SUC_lexicon_tags_8 },
    { 0x4938d560, SUC_lexicon_tags_17 },
    { 0xe938d55f, SUC_lexicon_tags_37 },
    { 0x5138d561, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b8d566, SUC_lexicon_tags_11 },
    { 0x17f8d567, SUC_lexicon_tags_13 },
    { 0x5178d567, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb8d56e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb78d571, SUC_lexicon_tags_28 },
    { 0x8158d572, SUC_lexicon_tags_8 },
    { 0xbe58d573, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa98d578, SUC_lexicon_tags_9 },
    { 0x7278d579, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8d584, SUC_lexicon_tags_46 },
    { 0xfab8d584, SUC_lexicon_tags_20 },
    { 0x9778d586, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef8d589, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f8d58c, SUC_lexicon_tags_37 },
    { 0x2658d58d, SUC_lexicon_tags_25 },
    { 0xa9f8d58d, SUC_lexicon_tags_15 },
    { 0xd4d8d58e, SUC_lexicon_tags_13 },
    { 0x9918d590, SUC_lexicon_tags_11 },
    { 0x8178d591, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe318d597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a18d59a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa738d5a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9278d5a2, SUC_lexicon_tags_38 },
    { 0x8398d5a2, SUC_lexicon_tags_32 },
    { 0xa198d5a4, SUC_lexicon_tags_6 },
    { 0xc418d5a5, SUC_lexicon_tags_12 },
    { 0x26f8d5a6, SUC_lexicon_tags_37 },
    { 0x5158d5a7, SUC_lexicon_tags_7 },
    { 0x1d58d5a8, SUC_lexicon_tags_3 },
    { 0x8a18d5a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd618d5ab, SUC_lexicon_tags_21 },
    { 0x8e98d5ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa118d5b1, SUC_lexicon_tags_2 },
    { 0x2298d5b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4198d5b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d8d5b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfcd8d5bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f8d5bf, SUC_lexicon_tags_63 },
    { 0xcf8d5bf, SUC_lexicon_tags_10 },
    { 0x8458d5c1, SUC_lexicon_tags_27 },
    { 0x638d5c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b8d5c7, SUC_lexicon_tags_2 },
    { 0x6058d5c8, SUC_lexicon_tags_17 },
    { 0x94d8d5c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d8d5cc, SUC_lexicon_tags_19 },
    { 0xd438d5cd, SUC_lexicon_tags_16 },
    { 0x42f8d5cd, SUC_lexicon_tags_71 },
    { 0x9d98d5cf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b38d5d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d8d5db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe918d5de, SUC_lexicon_tags_25 },
    { 0xd698d5df, SUC_lexicon_tags_37 },
    { 0xa3f8d5e0, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xdef8d5e2, SUC_lexicon_tags_82 },
    { 0xedb8d5e2, SUC_lexicon_tags_15 },
    { 0xa938d5e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7118d5e7, SUC_lexicon_tags_27 },
    { 0xe4b8d5e7, SUC_lexicon_tags_49 },
    { 0xb718d5e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a58d5ec, SUC_lexicon_tags_53 },
    { 0xfc38d5ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3698d5f0, SUC_lexicon_tags_37 },
    { 0x8838d5f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf778d5f8, SUC_lexicon_tags_12 },
    { 0xcc58d5f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x51f8d5fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f8d5fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b8d600, SUC_lexicon_tags_14 },
    { 0x32d8d601, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x89b8d603, SUC_lexicon_tags_14 },
    { 0xb338d604, SUC_lexicon_tags_26 },
    { 0xe418d603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf938d60a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458d60e, SUC_lexicon_tags_37 },
    { 0x8338d60f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b8d613, SUC_lexicon_tags_37 },
    { 0x70d8d613, SUC_lexicon_tags_1 },
    { 0xc058d614, SUC_lexicon_tags_17 },
    { 0xd6b8d616, SUC_lexicon_tags_0 },
    { 0x5778d617, SUC_lexicon_tags_49 },
    { 0xac58d618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93b8d61a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c38d620, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb18d622, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x138d626, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d8d632, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8438d636, SUC_lexicon_tags_37 },
    { 0x9178d637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb8d63b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1b8d63d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xce18d63f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed8d644, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4938d64a, SUC_lexicon_tags_30 },
    { 0xc898d64b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6918d64d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x97b8d64f, SUC_lexicon_tags_17 },
    { 0x1938d64f, SUC_lexicon_tags_35 },
    { 0xf4b8d651, SUC_lexicon_tags_37 },
    { 0xb218d651, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b38d655, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb98d657, SUC_lexicon_tags_17 },
    { 0xd078d657, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8618d65c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc898d65f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xca18d662, SUC_lexicon_tags_0 },
    { 0x5698d662, SUC_lexicon_tags_1 },
    { 0xd1d8d664, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb18d666, SUC_lexicon_tags_43 },
    { 0x3598d667, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf138d66c, SUC_lexicon_tags_27 },
    { 0x4b98d66c, SUC_lexicon_tags_7 },
    { 0x6d98d66e, SUC_lexicon_tags_27 },
    { 0x8178d66e, SUC_lexicon_tags_8 },
    { 0x3f78d66c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb918d672, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e58d675, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6b58d677, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d8d67a, SUC_lexicon_tags_33 },
    { 0xa3b8d67a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b8d67e, SUC_lexicon_tags_32 },
    { 0x9598d67f, SUC_lexicon_tags_55 },
    { 0x60b8d67f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3658d683, SUC_lexicon_tags_11 },
    { 0xd298d684, SUC_lexicon_tags_47 },
    { 0x7a18d683, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed8d68d, SUC_lexicon_tags_37 },
    { 0x29d8d68e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c78d694, SUC_lexicon_tags_8 },
    { 0x2f58d695, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6ed8d697, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7978d69c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc958d6a8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc858d6ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x158d6ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa78d6b3, SUC_lexicon_tags_14 },
    { 0x6f98d6b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f78d6b6, SUC_lexicon_tags_10 },
    { 0xf018d6b6, SUC_lexicon_tags_17 },
    { 0x6e98d6b7, SUC_lexicon_tags_25 },
    { 0x2098d6b9, SUC_lexicon_tags_23 },
    { 0xeb8d6b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5f8d6bc, SUC_lexicon_tags_9 },
    { 0x7bd8d6bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b18d6c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f8d6c9, SUC_lexicon_tags_17 },
    { 0x2358d6ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4998d6cd, SUC_lexicon_tags_11 },
    { 0x2058d6ce, SUC_lexicon_tags_35 },
    { 0x8fb8d6cf, SUC_lexicon_tags_0 },
    { 0xb038d6cf, SUC_lexicon_tags_27 },
    { 0xcc58d6d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f58d6d5, SUC_lexicon_tags_25 },
    { 0xb98d6d6, SUC_lexicon_tags_38 },
    { 0x1058d6d6, SUC_lexicon_tags_47 },
    { 0x958d6d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb258d6dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1998d6de, SUC_lexicon_tags_25 },
    { 0xa0d8d6de, SUC_lexicon_tags_37 },
    { 0x2cd8d6de, SUC_lexicon_tags_23 },
    { 0x8418d6e0, SUC_lexicon_tags_46 },
    { 0xa7b8d6e2, SUC_lexicon_tags_10 },
    { 0xc4b8d6e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4518d6e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f8d6ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1338d6ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9198d6f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6578d6f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d8d700, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc838d707, SUC_lexicon_tags_12 },
    { 0xd0b8d708, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x19d8d70a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3438d70c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x11b8d70e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf438d711, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7798d713, SUC_lexicon_tags_12 },
    { 0xe778d713, SUC_lexicon_tags_23 },
    { 0x7b8d715, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0f8d717, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc098d719, SUC_lexicon_tags_37 },
    { 0xecb8d71a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f8d724, SUC_lexicon_tags_7 },
    { 0x3278d724, SUC_lexicon_tags_1 },
    { 0xd818d725, SUC_lexicon_tags_11 },
    { 0xd6b8d724, SUC_lexicon_tags_6 },
    { 0x9098d728, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f8d72d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c58d730, SUC_lexicon_tags_25 },
    { 0x7678d730, SUC_lexicon_tags_48 },
    { 0xbd8d731, SUC_lexicon_tags_47 },
    { 0x6bf8d733, SUC_lexicon_tags_15 },
    { 0x85d8d734, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd038d73a, SUC_lexicon_tags_7 },
    { 0xd558d73b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xed18d73e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xff58d741, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd8d744, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6538d747, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8678d74a, SUC_lexicon_tags_19 },
    { 0x1378d74a, SUC_lexicon_tags_17 },
    { 0xdf78d74c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf8d74f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d58d752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5458d755, SUC_lexicon_tags_15 },
    { 0xd7b8d756, SUC_lexicon_tags_33 },
    { 0xf598d755, SUC_lexicon_tags_16 },
    { 0xf4f8d758, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d38d75a, SUC_lexicon_tags_7 },
    { 0x4718d75b, SUC_lexicon_tags_15 },
    { 0x3278d75a, SUC_lexicon_tags_10 },
    { 0xf0f8d75d, SUC_lexicon_tags_39 },
    { 0x71f8d75d, SUC_lexicon_tags_60 },
    { 0x2b58d75b, SUC_lexicon_tags_33 },
    { 0xa9f8d75b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd8d764, SUC_lexicon_tags_28 },
    { 0x1558d765, SUC_lexicon_tags_37 },
    { 0x6eb8d765, SUC_lexicon_tags_39 },
    { 0xdf18d764, SUC_lexicon_tags_7 },
    { 0x5d98d768, SUC_lexicon_tags_12 },
    { 0x1e38d769, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cf8d76b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c18d76e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a18d775, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe538d778, SUC_lexicon_tags_47 },
    { 0x3ab8d779, SUC_lexicon_tags_66 },
    { 0x5298d779, SUC_lexicon_tags_13 },
    { 0xff38d778, SUC_lexicon_tags_17 },
    { 0xfb38d778, SUC_lexicon_tags_2 },
    { 0x498d779, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fd8d77f, SUC_lexicon_tags_25 },
    { 0xa058d77f, SUC_lexicon_tags_18 },
    { 0xe958d780, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bb8d783, SUC_lexicon_tags_12 },
    { 0x1698d784, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8d788, SUC_lexicon_tags_8 },
    { 0x14b8d788, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa478d78d, SUC_lexicon_tags_6 },
    { 0xfe98d78e, SUC_lexicon_tags_16 },
    { 0x3fd8d78d, SUC_lexicon_tags_7 },
    { 0xc9b8d78f, SUC_lexicon_tags_10 },
    { 0x18f8d790, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf98d794, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3098d798, SUC_lexicon_tags_17 },
    { 0x3198d799, SUC_lexicon_tags_12 },
    { 0x3b38d79a, SUC_lexicon_tags_14 },
    { 0x3078d79a, SUC_lexicon_tags_3 },
    { 0xeeb8d79c, SUC_lexicon_tags_35 },
    { 0xe1b8d79d, SUC_lexicon_tags_35 },
    { 0x7158d79d, SUC_lexicon_tags_61 },
    { 0x2718d79d, SUC_lexicon_tags_38 },
    { 0x1ad8d79d, SUC_lexicon_tags_1 },
    { 0x60f8d79c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c58d7a3, SUC_lexicon_tags_79 },
    { 0x8c18d7a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18d7a8, SUC_lexicon_tags_21 },
    { 0xb138d7a9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x218d7ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f8d7b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd18d7b3, SUC_lexicon_tags_16 },
    { 0xf118d7b4, SUC_lexicon_tags_4 },
    { 0xe138d7b5, SUC_lexicon_tags_63 },
    { 0xf578d7b6, SUC_lexicon_tags_87 },
    { 0xe918d7b5, SUC_lexicon_tags_33 },
    { 0x918d7b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d58d7c3, SUC_lexicon_tags_25 },
    { 0xee58d7c4, SUC_lexicon_tags_2 },
    { 0x4718d7c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5038d7c8, SUC_lexicon_tags_87 },
    { 0xe898d7c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8098d7ce, SUC_lexicon_tags_37 },
    { 0xc118d7ce, SUC_lexicon_tags_17 },
    { 0xe358d7d0, SUC_lexicon_tags_12 },
    { 0xe0d8d7d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2198d7d3, SUC_lexicon_tags_1 },
    { 0xcfb8d7d4, SUC_lexicon_tags_0 },
    { 0x3f78d7d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd8d7d8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1438d7df, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa558d7e3, SUC_lexicon_tags_5 },
    { 0x2138d7e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb8d7e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed8d7eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf18d7f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb798d7fa, SUC_lexicon_tags_7 },
    { 0xda98d7fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfe18d7fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd8d801, SUC_lexicon_tags_25 },
    { 0x2d8d801, SUC_lexicon_tags_35 },
    { 0xab78d801, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcef8d805, SUC_lexicon_tags_11 },
    { 0x1298d805, SUC_lexicon_tags_17 },
    { 0x32b8d807, SUC_lexicon_tags_47 },
    { 0xde38d808, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bd8d80a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x7358d80d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d8d811, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf58d814, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4318d816, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xad98d819, SUC_lexicon_tags_6 },
    { 0x2158d81a, SUC_lexicon_tags_28 },
    { 0xdf58d81b, SUC_lexicon_tags_23 },
    { 0x1298d81c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6698d821, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b38d827, SUC_lexicon_tags_15 },
    { 0x45d8d828, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb8d82d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdab8d82f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e18d832, SUC_lexicon_tags_17 },
    { 0xe378d833, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1058d835, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa158d83a, SUC_lexicon_tags_7 },
    { 0x2258d83b, SUC_lexicon_tags_25 },
    { 0xfd38d83c, SUC_lexicon_tags_7 },
    { 0xa638d83d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9618d83f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6718d841, SUC_lexicon_tags_35 },
    { 0x9f8d842, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8438d848, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24b8d84a, SUC_lexicon_tags_14 },
    { 0x6058d84b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4918d84d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa858d854, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb858d85a, SUC_lexicon_tags_19 },
    { 0xbdb8d85b, SUC_lexicon_tags_29 },
    { 0x9af8d85a, SUC_lexicon_tags_37 },
    { 0xe798d85d, SUC_lexicon_tags_37 },
    { 0x99b8d85c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7738d866, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x798d86c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b98d870, SUC_lexicon_tags_17 },
    { 0xc9b8d870, SUC_lexicon_tags_30 },
    { 0x87b8d871, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed78d876, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d8d87b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d8d880, SUC_lexicon_tags_1 },
    { 0x2a98d881, SUC_lexicon_tags_17 },
    { 0x6f18d880, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf8d886, SUC_lexicon_tags_7 },
    { 0xf338d886, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc058d88d, SUC_lexicon_tags_12 },
    { 0x8078d88d, SUC_lexicon_tags_7 },
    { 0xcd8d88f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c98d891, SUC_lexicon_tags_8 },
    { 0x58d892, SUC_lexicon_tags_49 },
    { 0xcaf8d892, SUC_lexicon_tags_23 },
    { 0xdfb8d894, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb8d899, SUC_lexicon_tags_25 },
    { 0xd7b8d89a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5638d89d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f8d8a3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb858d8a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618d8ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc38d8b0, SUC_lexicon_tags_10 },
    { 0xf7d8d8b1, SUC_lexicon_tags_7 },
    { 0xbdd8d8b2, SUC_lexicon_tags_21 },
    { 0xaef8d8b1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x31f8d8b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f8d8b9, SUC_lexicon_tags_2 },
    { 0x6918d8ba, SUC_lexicon_tags_7 },
    { 0xbd8d8bb, SUC_lexicon_tags_46 },
    { 0x5358d8bc, SUC_lexicon_tags_1 },
    { 0xbd8d8ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe398d8bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef18d8c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa238d8c8, SUC_lexicon_tags_11 },
    { 0x2bd8d8c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa918d8cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3478d8ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa958d8d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc9b8d8d2, SUC_lexicon_tags_27 },
    { 0x6ad8d8d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71d8d8d5, SUC_lexicon_tags_9 },
    { 0x67d8d8d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbd8d8d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3858d8da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcf98d8dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf3d8d8de, SUC_lexicon_tags_11 },
    { 0x3bb8d8df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5f8d8e1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x32b8d8e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc818d8e6, SUC_lexicon_tags_15 },
    { 0x86d8d8e6, SUC_lexicon_tags_19 },
    { 0x2d78d8e6, SUC_lexicon_tags_1385 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa618d8f2, SUC_lexicon_tags_64 },
    { 0xf9f8d8f2, SUC_lexicon_tags_1 },
    { 0x8a18d8f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d8d8fc, SUC_lexicon_tags_37 },
    { 0xdf58d8fd, SUC_lexicon_tags_17 },
    { 0xa958d8fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d58d901, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4698d905, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c18d90a, SUC_lexicon_tags_280 },
    { 0x3338d90a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8178d90e, SUC_lexicon_tags_87 },
    { 0x6438d90e, SUC_lexicon_tags_1 },
    { 0x5278d90e, SUC_lexicon_tags_25 },
    { 0xf378d90e, SUC_lexicon_tags_19 },
    { 0x88f8d912, SUC_lexicon_tags_14 },
    { 0x3018d913, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd8d916, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd8d919, SUC_lexicon_tags_0 },
    { 0x9ab8d91a, SUC_lexicon_tags_19 },
    { 0x29b8d91b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6b18d91d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d8d921, SUC_lexicon_tags_23 },
    { 0x9b8d922, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a78d927, SUC_lexicon_tags_12 },
    { 0x3b78d928, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc58d92a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23d8d92c, SUC_lexicon_tags_215 },
    { 0xfe38d92d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4058d934, SUC_lexicon_tags_15 },
    { 0xfef8d935, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a78d937, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d8d93a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8018d93c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf838d941, SUC_lexicon_tags_49 },
    { 0x398d941, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4358d946, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918d953, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdf8d955, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a38d958, SUC_lexicon_tags_54 },
    { 0xeff8d959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed8d95c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9ed8d95e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2db8d960, SUC_lexicon_tags_19 },
    { 0xad8d961, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f8d965, SUC_lexicon_tags_12 },
    { 0x9378d966, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f58d96a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f8d96d, SUC_lexicon_tags_21 },
    { 0x9b18d96e, SUC_lexicon_tags_2 },
    { 0x83d8d96f, SUC_lexicon_tags_7 },
    { 0x3cd8d970, SUC_lexicon_tags_2 },
    { 0xad58d971, SUC_lexicon_tags_29 },
    { 0xf9b8d972, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa118d974, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb8d97a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f8d97f, SUC_lexicon_tags_5 },
    { 0x2318d980, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d8d983, SUC_lexicon_tags_12 },
    { 0x1a38d984, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4b8d986, SUC_lexicon_tags_66 },
    { 0x1438d986, SUC_lexicon_tags_43 },
    { 0x14b8d987, SUC_lexicon_tags_87 },
    { 0x5d98d989, SUC_lexicon_tags_10 },
    { 0x638d989, SUC_lexicon_tags_15 },
    { 0x72f8d98b, SUC_lexicon_tags_8 },
    { 0x8c18d98b, SUC_lexicon_tags_17 },
    { 0x8c38d989, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1178d992, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1958d996, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c18d99b, SUC_lexicon_tags_28 },
    { 0xa358d99c, SUC_lexicon_tags_37 },
    { 0xb0d8d99c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3238d9a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa318d9a5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa418d9a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2718d9ab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc418d9ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5358d9b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xccb8d9b2, SUC_lexicon_tags_11 },
    { 0x2318d9b2, SUC_lexicon_tags_7 },
    { 0x7d38d9b4, SUC_lexicon_tags_0 },
    { 0xc4f8d9b5, SUC_lexicon_tags_2 },
    { 0xd18d9b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a58d9b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b18d9bb, SUC_lexicon_tags_45 },
    { 0x6d38d9bc, SUC_lexicon_tags_37 },
    { 0xdad8d9bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3818d9c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee98d9c5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcdb8d9c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79f8d9c9, SUC_lexicon_tags_16 },
    { 0xe958d9ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab8d9cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9b8d9cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d58d9d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1a38d9d4, SUC_lexicon_tags_7 },
    { 0x9658d9d4, SUC_lexicon_tags_16 },
    { 0xca78d9d6, SUC_lexicon_tags_48 },
    { 0x1258d9d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a58d9d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ed8d9db, SUC_lexicon_tags_9 },
    { 0x6a98d9dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5838d9de, SUC_lexicon_tags_1 },
    { 0x4598d9df, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6578d9e1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad58d9e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c38d9e9, SUC_lexicon_tags_12 },
    { 0x67b8d9e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbe78d9ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd18d9f0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d8d9f6, SUC_lexicon_tags_13 },
    { 0x538d9f7, SUC_lexicon_tags_37 },
    { 0xaf8d9f8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbdd8d9fa, SUC_lexicon_tags_14 },
    { 0x8ef8d9fb, SUC_lexicon_tags_29 },
    { 0xf358d9fc, SUC_lexicon_tags_69 },
    { 0x22b8d9fc, SUC_lexicon_tags_17 },
    { 0x5f98d9fc, SUC_lexicon_tags_4 },
    { 0x67d8d9ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd8da02, SUC_lexicon_tags_21 },
    { 0xf558da03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b8da08, SUC_lexicon_tags_43 },
    { 0x4ad8da09, SUC_lexicon_tags_1 },
    { 0x5b38da0a, SUC_lexicon_tags_15 },
    { 0xfe38da0b, SUC_lexicon_tags_25 },
    { 0x9318da0c, SUC_lexicon_tags_1 },
    { 0xcd38da0d, SUC_lexicon_tags_0 },
    { 0xff38da0e, SUC_lexicon_tags_32 },
    { 0xabf8da0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb618da12, SUC_lexicon_tags_2 },
    { 0xe038da13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9db8da15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d8da1f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9cd8da21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a18da23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f98da27, SUC_lexicon_tags_15 },
    { 0xdd18da27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69f8da2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e38da2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e58da2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff8da33, SUC_lexicon_tags_6 },
    { 0xca18da34, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6518da37, SUC_lexicon_tags_29 },
    { 0xcd98da38, SUC_lexicon_tags_21 },
    { 0x8a18da38, SUC_lexicon_tags_37 },
    { 0x7a18da3a, SUC_lexicon_tags_8 },
    { 0xdbf8da3a, SUC_lexicon_tags_2 },
    { 0x4ed8da38, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb8da40, SUC_lexicon_tags_11 },
    { 0xf278da41, SUC_lexicon_tags_38 },
    { 0x58d8da42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d8da45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf698da48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e18da50, SUC_lexicon_tags_1 },
    { 0x2818da50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2998da54, SUC_lexicon_tags_11 },
    { 0x2518da55, SUC_lexicon_tags_25 },
    { 0x9098da56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1038da58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x918da5b, SUC_lexicon_tags_15 },
    { 0x6998da5c, SUC_lexicon_tags_15 },
    { 0xa2d8da5d, SUC_lexicon_tags_65 },
    { 0x5b18da5c, SUC_lexicon_tags_15 },
    { 0xc9d8da5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7158da61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9058da65, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f78da6a, SUC_lexicon_tags_43 },
    { 0x6bf8da6a, SUC_lexicon_tags_104 },
    { 0x8d78da6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d58da71, SUC_lexicon_tags_8 },
    { 0x5b38da71, SUC_lexicon_tags_17 },
    { 0xf718da71, SUC_lexicon_tags_23 },
    { 0x9198da74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18da7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc38da80, SUC_lexicon_tags_17 },
    { 0xba18da81, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x91b8da83, SUC_lexicon_tags_58 },
    { 0x638da84, SUC_lexicon_tags_37 },
    { 0x89f8da85, SUC_lexicon_tags_45 },
    { 0xb218da86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9538da88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa98da8b, SUC_lexicon_tags_17 },
    { 0x1dd8da8c, SUC_lexicon_tags_9 },
    { 0xa998da8d, SUC_lexicon_tags_23 },
    { 0xb8da8c, SUC_lexicon_tags_79 },
    { 0x2d78da8c, SUC_lexicon_tags_54 },
    { 0x4938da90, SUC_lexicon_tags_27 },
    { 0xec98da90, SUC_lexicon_tags_38 },
    { 0xe5f8da8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46f8da95, SUC_lexicon_tags_28 },
    { 0x9078da96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b8da9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d58da9e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xced8daa1, SUC_lexicon_tags_7 },
    { 0xcf8daa2, SUC_lexicon_tags_25 },
    { 0x7258daa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f8daaa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf8daae, SUC_lexicon_tags_33 },
    { 0x9778daaf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x23d8dab1, SUC_lexicon_tags_3 },
    { 0x2838dab1, SUC_lexicon_tags_25 },
    { 0xd618dab1, SUC_lexicon_tags_37 },
    { 0xa6d8dab4, SUC_lexicon_tags_25 },
    { 0xa118dab5, SUC_lexicon_tags_15 },
    { 0x378dab5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6258dab8, SUC_lexicon_tags_12 },
    { 0x4418dab9, SUC_lexicon_tags_68 },
    { 0xa018dab8, SUC_lexicon_tags_61 },
    { 0x9618dabb, SUC_lexicon_tags_65 },
    { 0xad58dabc, SUC_lexicon_tags_47 },
    { 0x88d8dabc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9db8dabf, SUC_lexicon_tags_15 },
    { 0xed58dac0, SUC_lexicon_tags_18 },
    { 0xf638dac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa838dac4, SUC_lexicon_tags_11 },
    { 0x1d78dac4, SUC_lexicon_tags_15 },
    { 0x2e38dac5, SUC_lexicon_tags_15 },
    { 0x49f8dac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6d8dac9, SUC_lexicon_tags_25 },
    { 0x60f8daca, SUC_lexicon_tags_29 },
    { 0x4478dacb, SUC_lexicon_tags_45 },
    { 0x5b78dacb, SUC_lexicon_tags_18 },
    { 0x9638daca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc098dacf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f8dad2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f78dad4, SUC_lexicon_tags_15 },
    { 0xf718dad4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa538dadb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf278dade, SUC_lexicon_tags_26 },
    { 0x1f58dade, SUC_lexicon_tags_25 },
    { 0xa3f8dade, SUC_lexicon_tags_17 },
    { 0x25d8dae0, SUC_lexicon_tags_26 },
    { 0x9798dae2, SUC_lexicon_tags_47 },
    { 0x6e18dae3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d8daeb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6638daed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9398daf0, SUC_lexicon_tags_7 },
    { 0x7158daf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bf8daf3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38daf8, SUC_lexicon_tags_38 },
    { 0xf1d8daf8, SUC_lexicon_tags_7 },
    { 0x2d58dafa, SUC_lexicon_tags_17 },
    { 0xf4f8dafb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6b8dafd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x57f8daff, SUC_lexicon_tags_32 },
    { 0x6818db00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8438db02, SUC_lexicon_tags_17 },
    { 0x8878db03, SUC_lexicon_tags_14 },
    { 0x9498db03, SUC_lexicon_tags_38 },
    { 0xa3b8db03, SUC_lexicon_tags_15 },
    { 0xf8f8db03, SUC_lexicon_tags_32 },
    { 0xb998db07, SUC_lexicon_tags_65 },
    { 0xd378db08, SUC_lexicon_tags_160 },
    { 0x5c98db09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d98db0e, SUC_lexicon_tags_15 },
    { 0x9498db0f, SUC_lexicon_tags_37 },
    { 0x1bb8db10, SUC_lexicon_tags_53 },
    { 0x62b8db10, SUC_lexicon_tags_31 },
    { 0xb738db0f, SUC_lexicon_tags_38 },
    { 0xed58db13, SUC_lexicon_tags_7 },
    { 0x3958db14, SUC_lexicon_tags_8 },
    { 0xe8f8db13, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5398db1b, SUC_lexicon_tags_68 },
    { 0xaad8db1c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xa58db1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9198db21, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8db25, SUC_lexicon_tags_37 },
    { 0xc2b8db25, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf2f8db28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3d8db2a, SUC_lexicon_tags_25 },
    { 0xd9f8db2b, SUC_lexicon_tags_155 },
    { 0x50d8db2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2938db2e, SUC_lexicon_tags_55 },
    { 0xeb98db2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b8db32, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa358db36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa18db38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2378db3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f98db3f, SUC_lexicon_tags_83 },
    { 0x4398db40, SUC_lexicon_tags_43 },
    { 0xeff8db41, SUC_lexicon_tags_16 },
    { 0x8898db40, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1338db49, SUC_lexicon_tags_38 },
    { 0xead8db49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8798db4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a38db4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8738db56, SUC_lexicon_tags_60 },
    { 0xbd58db57, SUC_lexicon_tags_1 },
    { 0xd078db57, SUC_lexicon_tags_30 },
    { 0x8518db56, SUC_lexicon_tags_15 },
    { 0x678db5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe698db5c, SUC_lexicon_tags_38 },
    { 0xd858db5c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918db64, SUC_lexicon_tags_14 },
    { 0xdfb8db64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b8db77, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b8db7b, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xcd38db7d, SUC_lexicon_tags_37 },
    { 0x1818db7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a78db83, SUC_lexicon_tags_40 },
    { 0x76d8db83, SUC_lexicon_tags_25 },
    { 0x3058db85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b8db88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb438db8c, SUC_lexicon_tags_25 },
    { 0xa098db8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8838db90, SUC_lexicon_tags_7 },
    { 0x1058db91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7558db9a, SUC_lexicon_tags_23 },
    { 0x3398db9b, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x3a58db9d, SUC_lexicon_tags_12 },
    { 0xb118db9e, SUC_lexicon_tags_26 },
    { 0xbdd8db9f, SUC_lexicon_tags_27 },
    { 0x7e78db9d, SUC_lexicon_tags_38 },
    { 0xe038dba1, SUC_lexicon_tags_47 },
    { 0x4518dba2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb58dba4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5738dba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca78dbaa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd38dbad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa9f8dbaf, SUC_lexicon_tags_29 },
    { 0xe638dbb0, SUC_lexicon_tags_37 },
    { 0x6118dbb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f8dbb6, SUC_lexicon_tags_11 },
    { 0x2158dbb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xad58dbba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e78dbbd, SUC_lexicon_tags_8 },
    { 0x6cb8dbbd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf8dbc1, SUC_lexicon_tags_1 },
    { 0xacd8dbc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6218dbc4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x76b8dbc6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7f18dbc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f8dbcb, SUC_lexicon_tags_10 },
    { 0x6018dbcc, SUC_lexicon_tags_49 },
    { 0xa18dbcd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2598dbcf, SUC_lexicon_tags_12 },
    { 0xf018dbd0, SUC_lexicon_tags_172 },
    { 0x17b8dbd1, SUC_lexicon_tags_19 },
    { 0xedb8dbd2, SUC_lexicon_tags_1 },
    { 0x2df8dbd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55b8dbd5, SUC_lexicon_tags_25 },
    { 0x39f8dbd6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b8dbdb, SUC_lexicon_tags_2 },
    { 0x4b58dbdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e58dbdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf938dbe1, SUC_lexicon_tags_60 },
    { 0x3338dbe1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd738dbe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d78dbeb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5398dbf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc38dbf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8818dbf9, SUC_lexicon_tags_14 },
    { 0xc8b8dbfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd398dc01, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc038dc07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe718dc09, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf8dc0d, SUC_lexicon_tags_1 },
    { 0x7338dc0d, SUC_lexicon_tags_49 },
    { 0xa138dc0f, SUC_lexicon_tags_37 },
    { 0x6ad8dc10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd8dc16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc918dc19, SUC_lexicon_tags_47 },
    { 0xf8f8dc1a, SUC_lexicon_tags_37 },
    { 0xbfd8dc1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd8dc1d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x56b8dc1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1738dc22, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead8dc28, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf498dc2b, SUC_lexicon_tags_34 },
    { 0xb378dc2b, SUC_lexicon_tags_7 },
    { 0x30f8dc2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd8dc36, SUC_lexicon_tags_5 },
    { 0x1378dc37, SUC_lexicon_tags_38 },
    { 0xfc18dc38, SUC_lexicon_tags_7 },
    { 0x4018dc38, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb178dc3c, SUC_lexicon_tags_11 },
    { 0x5e18dc3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a78dc46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b8dc52, SUC_lexicon_tags_25 },
    { 0xf5d8dc52, SUC_lexicon_tags_0 },
    { 0xf718dc52, SUC_lexicon_tags_32 },
    { 0xdd38dc53, SUC_lexicon_tags_4 },
    { 0xad18dc56, SUC_lexicon_tags_7 },
    { 0x1738dc52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d8dc5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4378dc61, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3938dc64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd38dc68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd8dc6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8858dc72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf18dc76, SUC_lexicon_tags_57 },
    { 0xc438dc76, SUC_lexicon_tags_2 },
    { 0x2d78dc78, SUC_lexicon_tags_31 },
    { 0x8e18dc77, SUC_lexicon_tags_30 },
    { 0x3458dc7a, SUC_lexicon_tags_0 },
    { 0x9598dc7b, SUC_lexicon_tags_11 },
    { 0xbc78dc7a, SUC_lexicon_tags_15 },
    { 0xbf98dc7c, SUC_lexicon_tags_2 },
    { 0xfeb8dc77, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d38dc81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91d8dc83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc698dc88, SUC_lexicon_tags_23 },
    { 0xec18dc88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f8dc90, SUC_lexicon_tags_1 },
    { 0x2918dc91, SUC_lexicon_tags_30 },
    { 0x1978dc92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d8dc9b, SUC_lexicon_tags_11 },
    { 0x9238dc9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d8dc9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc58dca5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd218dca9, SUC_lexicon_tags_37 },
    { 0x13d8dcaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d8dcb1, SUC_lexicon_tags_0 },
    { 0x21d8dcb1, SUC_lexicon_tags_25 },
    { 0x42b8dcb3, SUC_lexicon_tags_37 },
    { 0x6478dcb3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc058dcb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db8dcba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b8dcbd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3f8dcbf, SUC_lexicon_tags_30 },
    { 0x9598dcc0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f8dcc4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6538dcc6, SUC_lexicon_tags_39 },
    { 0x18d8dcc7, SUC_lexicon_tags_25 },
    { 0x43d8dcc8, SUC_lexicon_tags_17 },
    { 0xe958dcc9, SUC_lexicon_tags_17 },
    { 0x598dcc8, SUC_lexicon_tags_43 },
    { 0xd698dccb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4578dccd, SUC_lexicon_tags_15 },
    { 0xe258dcce, SUC_lexicon_tags_11 },
    { 0x52d8dccd, SUC_lexicon_tags_87 },
    { 0x6238dccd, SUC_lexicon_tags_10 },
    { 0xf778dcd1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7318dcd3, SUC_lexicon_tags_25 },
    { 0xccf8dcd3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd498dcd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd558dcda, SUC_lexicon_tags_15 },
    { 0x5a38dcda, SUC_lexicon_tags_16 },
    { 0xfff8dcdc, SUC_lexicon_tags_61 },
    { 0x8c18dcdd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcc78dcdf, SUC_lexicon_tags_4 },
    { 0xa18dce0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7f8dce2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1778dce4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1858dce6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c18dce9, SUC_lexicon_tags_25 },
    { 0x1f78dcea, SUC_lexicon_tags_1 },
    { 0x898dceb, SUC_lexicon_tags_37 },
    { 0x73b8dce9, SUC_lexicon_tags_7 },
    { 0xf98dced, SUC_lexicon_tags_16 },
    { 0x2d78dcee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd8dcf3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b78dcf6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa18dcf8, SUC_lexicon_tags_30 },
    { 0xdcf8dcf8, SUC_lexicon_tags_6 },
    { 0x4cb8dcfa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4238dcfd, SUC_lexicon_tags_14 },
    { 0x6bb8dcfe, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xabd8dd00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f8dd04, SUC_lexicon_tags_49 },
    { 0x9658dd05, SUC_lexicon_tags_12 },
    { 0x9778dd06, SUC_lexicon_tags_43 },
    { 0x9d78dd06, SUC_lexicon_tags_25 },
    { 0x1a38dd06, SUC_lexicon_tags_33 },
    { 0xfcb8dd09, SUC_lexicon_tags_2 },
    { 0xef8dd05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff8dd0e, SUC_lexicon_tags_3 },
    { 0x68f8dd0e, SUC_lexicon_tags_45 },
    { 0x1f98dd0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa6d8dd12, SUC_lexicon_tags_17 },
    { 0x2af8dd12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2218dd18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7f8dd1a, SUC_lexicon_tags_88 },
    { 0x6658dd1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d38dd20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb138dd23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd8dd2b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a38dd2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d18dd31, SUC_lexicon_tags_8 },
    { 0x2b58dd31, SUC_lexicon_tags_6 },
    { 0x6938dd33, SUC_lexicon_tags_6 },
    { 0x40d8dd32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15b8dd36, SUC_lexicon_tags_17 },
    { 0x6018dd37, SUC_lexicon_tags_37 },
    { 0xcf8dd38, SUC_lexicon_tags_7 },
    { 0xb18dd36, SUC_lexicon_tags_27 },
    { 0x97f8dd3a, SUC_lexicon_tags_46 },
    { 0xc218dd38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6658dd41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d38dd47, SUC_lexicon_tags_17 },
    { 0x2f58dd48, SUC_lexicon_tags_26 },
    { 0x7d98dd49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x718dd50, SUC_lexicon_tags_25 },
    { 0xcc38dd51, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b38dd54, SUC_lexicon_tags_84 },
    { 0x8db8dd55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f8dd58, SUC_lexicon_tags_15 },
    { 0xa178dd59, SUC_lexicon_tags_26 },
    { 0x6398dd59, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2898dd5c, SUC_lexicon_tags_12 },
    { 0xf498dd5d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b18dd61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20f8dd65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee78dd68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d8dd70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf78dd74, SUC_lexicon_tags_47 },
    { 0x2a98dd75, SUC_lexicon_tags_84 },
    { 0x7978dd76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c18dd78, SUC_lexicon_tags_12 },
    { 0x1bf8dd78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd8dd7c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf958dd82, SUC_lexicon_tags_49 },
    { 0xc158dd83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5578dd85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb278dd89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b8dd8c, SUC_lexicon_tags_47 },
    { 0xb58dd8d, SUC_lexicon_tags_6 },
    { 0x3bf8dd8d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1618dd96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf598dd99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f78dd9c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb8dda0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1f38dda2, SUC_lexicon_tags_25 },
    { 0x7618dda2, SUC_lexicon_tags_41 },
    { 0x2df8dda4, SUC_lexicon_tags_15 },
    { 0x97b8dda5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d78dda8, SUC_lexicon_tags_63 },
    { 0x9298dda8, SUC_lexicon_tags_15 },
    { 0xd858ddaa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d8ddad, SUC_lexicon_tags_25 },
    { 0xb078ddad, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b8ddb3, SUC_lexicon_tags_26 },
    { 0xf538ddb3, SUC_lexicon_tags_37 },
    { 0x8858ddb4, SUC_lexicon_tags_37 },
    { 0x12f8ddb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d8ddbc, SUC_lexicon_tags_25 },
    { 0xfe58ddbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca58ddbf, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b8ddc2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1238ddc5, SUC_lexicon_tags_37 },
    { 0x4338ddc6, SUC_lexicon_tags_0 },
    { 0x8bf8ddc7, SUC_lexicon_tags_84 },
    { 0xb698ddc6, SUC_lexicon_tags_17 },
    { 0x6c98ddc9, SUC_lexicon_tags_17 },
    { 0x5db8ddca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a38ddce, SUC_lexicon_tags_27 },
    { 0x2ed8ddce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d8ddd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4dd8ddd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1218ddd9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b8dddc, SUC_lexicon_tags_31 },
    { 0x4d58dddd, SUC_lexicon_tags_17 },
    { 0xadb8ddde, SUC_lexicon_tags_12 },
    { 0x6438dddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd8dde8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeff8ddea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d38ddf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x578ddf6, SUC_lexicon_tags_2 },
    { 0x3b8ddf6, SUC_lexicon_tags_29 },
    { 0x8bf8ddf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9038ddfa, SUC_lexicon_tags_2 },
    { 0xcd38ddfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbb8ddfd, SUC_lexicon_tags_29 },
    { 0x45f8ddfe, SUC_lexicon_tags_85 },
    { 0xcc38ddfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5438de07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf78de0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd8de0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1618de11, SUC_lexicon_tags_2 },
    { 0x4c38de12, SUC_lexicon_tags_15 },
    { 0xbcf8de11, SUC_lexicon_tags_34 },
    { 0x3758de14, SUC_lexicon_tags_1 },
    { 0x51b8de11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1098de17, SUC_lexicon_tags_65 },
    { 0x6d78de18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88f8de1a, SUC_lexicon_tags_0 },
    { 0x51b8de1b, SUC_lexicon_tags_15 },
    { 0x28b8de1b, SUC_lexicon_tags_34 },
    { 0x9338de1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7718de22, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1638de24, SUC_lexicon_tags_46 },
    { 0x43f8de24, SUC_lexicon_tags_7 },
    { 0xd858de25, SUC_lexicon_tags_15 },
    { 0xe818de27, SUC_lexicon_tags_23 },
    { 0x6b38de25, SUC_lexicon_tags_25 },
    { 0x6db8de28, SUC_lexicon_tags_25 },
    { 0xe1f8de2a, SUC_lexicon_tags_37 },
    { 0xb618de2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f8de31, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1278de35, SUC_lexicon_tags_29 },
    { 0xd418de36, SUC_lexicon_tags_16 },
    { 0xbcf8de37, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf998de3b, SUC_lexicon_tags_27 },
    { 0x6b78de3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfaf8de3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb378de41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f58de43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9f8de45, SUC_lexicon_tags_2 },
    { 0xdaf8de46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd758de48, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb8de4e, SUC_lexicon_tags_40 },
    { 0x9998de4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb8de5a, SUC_lexicon_tags_12 },
    { 0x8178de5b, SUC_lexicon_tags_25 },
    { 0x7bb8de5c, SUC_lexicon_tags_29 },
    { 0x7cb8de5b, SUC_lexicon_tags_19 },
    { 0xccf8de5c, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x2878de60, SUC_lexicon_tags_12 },
    { 0x2258de61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd18de63, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6838de6d, SUC_lexicon_tags_63 },
    { 0x5ef8de6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e38de70, SUC_lexicon_tags_5 },
    { 0xc6b8de70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd8de75, SUC_lexicon_tags_7 },
    { 0x1f78de75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44d8de78, SUC_lexicon_tags_14 },
    { 0xe3d8de79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9178de7c, SUC_lexicon_tags_60 },
    { 0xed38de7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe1b8de7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef8de85, SUC_lexicon_tags_25 },
    { 0x6338de86, SUC_lexicon_tags_11 },
    { 0x8138de87, SUC_lexicon_tags_27 },
    { 0x3718de88, SUC_lexicon_tags_1 },
    { 0xa4d8de89, SUC_lexicon_tags_29 },
    { 0xd858de89, SUC_lexicon_tags_35 },
    { 0x2b38de8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5438de8d, SUC_lexicon_tags_19 },
    { 0x7758de8e, SUC_lexicon_tags_1 },
    { 0xadf8de8f, SUC_lexicon_tags_37 },
    { 0x3698de90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c18de96, SUC_lexicon_tags_11 },
    { 0x36d8de97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b78de9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6c58de9c, SUC_lexicon_tags_1 },
    { 0x57d8de9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf78de9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8998dea4, SUC_lexicon_tags_0 },
    { 0xb978dea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa098dea7, SUC_lexicon_tags_17 },
    { 0x3978dea7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3278deaf, SUC_lexicon_tags_2 },
    { 0xe9f8deb0, SUC_lexicon_tags_1 },
    { 0x9498deb1, SUC_lexicon_tags_39 },
    { 0xa038deaf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xea18deb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84f8deb6, SUC_lexicon_tags_6 },
    { 0x2898deb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df8deba, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf78debd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8dec3, SUC_lexicon_tags_10 },
    { 0x4578dec4, SUC_lexicon_tags_8 },
    { 0x9e78dec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58dec9, SUC_lexicon_tags_11 },
    { 0x82b8deca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46f8decc, SUC_lexicon_tags_15 },
    { 0xfad8decd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c38ded2, SUC_lexicon_tags_1 },
    { 0xcc98ded3, SUC_lexicon_tags_11 },
    { 0xe8f8ded3, SUC_lexicon_tags_45 },
    { 0xbbf8ded5, SUC_lexicon_tags_282 },
    { 0xcad8ded6, SUC_lexicon_tags_15 },
    { 0x3f38ded6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b8dedb, SUC_lexicon_tags_26 },
    { 0x2f8dedb, SUC_lexicon_tags_33 },
    { 0x6df8dedc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6458dedf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a78dee2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18dee9, SUC_lexicon_tags_9 },
    { 0x30f8dee9, SUC_lexicon_tags_7 },
    { 0x6758deeb, SUC_lexicon_tags_9 },
    { 0x82d8deec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xddd8deee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde18def2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad38def7, SUC_lexicon_tags_27 },
    { 0xf4b8def8, SUC_lexicon_tags_74 },
    { 0x5fd8def8, SUC_lexicon_tags_17 },
    { 0x358def8, SUC_lexicon_tags_25 },
    { 0xadd8def7, SUC_lexicon_tags_1 },
    { 0xfa18defb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa818deff, SUC_lexicon_tags_16 },
    { 0x34f8df00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc518df05, SUC_lexicon_tags_37 },
    { 0x2f8df06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b38df0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1178df0c, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x32b8df0e, SUC_lexicon_tags_27 },
    { 0xd8b8df0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9f8df11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f38df15, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9598df1a, SUC_lexicon_tags_2 },
    { 0x3a98df1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af8df1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc978df26, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe198df2a, SUC_lexicon_tags_12 },
    { 0x47d8df2b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1638df2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb8df34, SUC_lexicon_tags_2 },
    { 0xb298df34, SUC_lexicon_tags_17 },
    { 0x3c78df36, SUC_lexicon_tags_25 },
    { 0x3598df37, SUC_lexicon_tags_31 },
    { 0x3a98df38, SUC_lexicon_tags_61 },
    { 0x2878df34, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6658df3b, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b8df3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa258df42, SUC_lexicon_tags_11 },
    { 0x27d8df42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d8df52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d8df56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6638df58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b8df5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x938df5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe018df60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d8df63, SUC_lexicon_tags_1 },
    { 0x4bb8df64, SUC_lexicon_tags_35 },
    { 0xd698df65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x938df67, SUC_lexicon_tags_49 },
    { 0xdd78df68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf38df6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b8df6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf8df72, SUC_lexicon_tags_98 },
    { 0xce58df72, SUC_lexicon_tags_37 },
    { 0x3778df74, SUC_lexicon_tags_55 },
    { 0xdc58df75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e78df77, SUC_lexicon_tags_15 },
    { 0x4978df78, SUC_lexicon_tags_1 },
    { 0x3638df79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeff8df7b, SUC_lexicon_tags_53 },
    { 0x8c78df7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd8df7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2bf8df81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ef8df83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc98df89, SUC_lexicon_tags_47 },
    { 0x5bf8df8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9338df8c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf558df94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb478df97, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1558df99, SUC_lexicon_tags_49 },
    { 0x4af8df9a, SUC_lexicon_tags_25 },
    { 0x1f58df9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd998dfa0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x35d8dfa2, SUC_lexicon_tags_47 },
    { 0xd938dfa2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8158dfa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac38dfa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d98dfa9, SUC_lexicon_tags_56 },
    { 0x38f8dfaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b8dfb1, SUC_lexicon_tags_66 },
    { 0x6798dfb1, SUC_lexicon_tags_37 },
    { 0x69b8dfb3, SUC_lexicon_tags_6 },
    { 0x9a58dfb4, SUC_lexicon_tags_37 },
    { 0xebf8dfb5, SUC_lexicon_tags_6 },
    { 0x4498dfb6, SUC_lexicon_tags_6 },
    { 0x4458dfb6, SUC_lexicon_tags_11 },
    { 0x31f8dfb3, SUC_lexicon_tags_23 },
    { 0x9d38dfb3, SUC_lexicon_tags_37 },
    { 0xd938dfba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e58dfc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3638dfc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b78dfc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d78dfcc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f8dfd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f8dfd8, SUC_lexicon_tags_43 },
    { 0xc018dfd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab8dfdc, SUC_lexicon_tags_25 },
    { 0x2f58dfdc, SUC_lexicon_tags_1 },
    { 0x5e78dfdd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f8dfe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa298dfe4, SUC_lexicon_tags_2 },
    { 0xa698dfe5, SUC_lexicon_tags_3 },
    { 0x51f8dfe5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db8dfea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e78dfed, SUC_lexicon_tags_37 },
    { 0x6e78dfee, SUC_lexicon_tags_15 },
    { 0xfc58dfed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2738dff1, SUC_lexicon_tags_33 },
    { 0x3a98dff1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f58dff7, SUC_lexicon_tags_92 },
    { 0x1ad8dff7, SUC_lexicon_tags_15 },
    { 0xa1d8dff9, SUC_lexicon_tags_77 },
    { 0xd538dffa, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a78dffe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x418e002, SUC_lexicon_tags_17 },
    { 0xed38e003, SUC_lexicon_tags_15 },
    { 0xc858e004, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2218e007, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb558e00e, SUC_lexicon_tags_60 },
    { 0x7198e00f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f58e014, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b8e01f, SUC_lexicon_tags_14 },
    { 0x5878e020, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e58e025, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf7f8e027, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4378e02a, SUC_lexicon_tags_62 },
    { 0x45d8e02b, SUC_lexicon_tags_39 },
    { 0x7e38e02c, SUC_lexicon_tags_1 },
    { 0x1c18e02b, SUC_lexicon_tags_11 },
    { 0xe558e02e, SUC_lexicon_tags_62 },
    { 0x9d8e02f, SUC_lexicon_tags_37 },
    { 0x6258e02e, SUC_lexicon_tags_47 },
    { 0x29d8e031, SUC_lexicon_tags_7 },
    { 0xac38e031, SUC_lexicon_tags_25 },
    { 0x1058e02f, SUC_lexicon_tags_1 },
    { 0x8d18e02b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1518e038, SUC_lexicon_tags_23 },
    { 0xa6f8e039, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x18e03c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d8e047, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x15d8e04a, SUC_lexicon_tags_25 },
    { 0xb798e04b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d98e04e, SUC_lexicon_tags_37 },
    { 0xdb78e04f, SUC_lexicon_tags_7 },
    { 0x3118e050, SUC_lexicon_tags_25 },
    { 0xdd78e051, SUC_lexicon_tags_45 },
    { 0x3558e052, SUC_lexicon_tags_15 },
    { 0x1858e053, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa018e058, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5498e05a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e38e05c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x90f8e05e, SUC_lexicon_tags_45 },
    { 0xb718e05e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a78e063, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f8e068, SUC_lexicon_tags_65 },
    { 0xe078e069, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b8e06d, SUC_lexicon_tags_23 },
    { 0xf378e06e, SUC_lexicon_tags_47 },
    { 0x5b78e06f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe38e076, SUC_lexicon_tags_2 },
    { 0x9ad8e077, SUC_lexicon_tags_25 },
    { 0xb4f8e078, SUC_lexicon_tags_43 },
    { 0x52f8e076, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x57f8e07b, SUC_lexicon_tags_17 },
    { 0x1a38e07c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb878e07f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda98e083, SUC_lexicon_tags_47 },
    { 0xd898e084, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f8e088, SUC_lexicon_tags_45 },
    { 0xd598e089, SUC_lexicon_tags_25 },
    { 0xc178e089, SUC_lexicon_tags_25 },
    { 0x5558e08a, SUC_lexicon_tags_48 },
    { 0xf418e08c, SUC_lexicon_tags_98 },
    { 0x7358e089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0d8e08f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26f8e091, SUC_lexicon_tags_10 },
    { 0xb2f8e091, SUC_lexicon_tags_816 },
    { 0xdb98e093, SUC_lexicon_tags_37 },
    { 0xcd58e094, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9258e096, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba38e098, SUC_lexicon_tags_7 },
    { 0x2a18e099, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb838e09b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe038e0a1, SUC_lexicon_tags_25 },
    { 0xaa58e0a2, SUC_lexicon_tags_63 },
    { 0x79f8e0a3, SUC_lexicon_tags_37 },
    { 0x6b18e0a4, SUC_lexicon_tags_43 },
    { 0x58f8e0a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7158e0a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb8e0ac, SUC_lexicon_tags_17 },
    { 0x1538e0ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe098e0b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d8e0b8, SUC_lexicon_tags_4 },
    { 0xfa58e0b9, SUC_lexicon_tags_68 },
    { 0xf998e0ba, SUC_lexicon_tags_11 },
    { 0x8258e0ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6138e0bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec98e0c6, SUC_lexicon_tags_25 },
    { 0x4b78e0c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe458e0c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda78e0cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1218e0d1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xeb98e0d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7958e0d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d8e0df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf8e0e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4038e0e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d8e0ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe878e0ec, SUC_lexicon_tags_25 },
    { 0x578e0ed, SUC_lexicon_tags_66 },
    { 0xf458e0ed, SUC_lexicon_tags_7 },
    { 0x98f8e0ed, SUC_lexicon_tags_1 },
    { 0xb1b8e0ef, SUC_lexicon_tags_31 },
    { 0xad98e0ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xff38e0f4, SUC_lexicon_tags_43 },
    { 0x1498e0f5, SUC_lexicon_tags_6 },
    { 0x7438e0f6, SUC_lexicon_tags_11 },
    { 0xe018e0f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f8e0fa, SUC_lexicon_tags_25 },
    { 0x8e58e0fb, SUC_lexicon_tags_37 },
    { 0xb538e0fc, SUC_lexicon_tags_24 },
    { 0x1978e0fa, SUC_lexicon_tags_37 },
    { 0xaf8e0fe, SUC_lexicon_tags_1438 },
    { 0xa5d8e0fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e58e101, SUC_lexicon_tags_28 },
    { 0x2738e102, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d8e105, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7838e108, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc38e10c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d8e110, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc658e119, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd758e11b, SUC_lexicon_tags_31 },
    { 0xd638e11c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe1d8e11e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x53f8e120, SUC_lexicon_tags_12 },
    { 0x7938e120, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a38e123, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54d8e125, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd8e129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x478e12b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74f8e12f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c58e134, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb918e136, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b38e13c, SUC_lexicon_tags_37 },
    { 0x4438e13c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe898e13f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9758e141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e18e147, SUC_lexicon_tags_11 },
    { 0x3298e148, SUC_lexicon_tags_7 },
    { 0x8a98e148, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaeb8e14b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb8e14e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5278e152, SUC_lexicon_tags_17 },
    { 0x8118e152, SUC_lexicon_tags_17 },
    { 0xe0d8e152, SUC_lexicon_tags_49 },
    { 0xb8f8e152, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2e18e157, SUC_lexicon_tags_12 },
    { 0xdd8e157, SUC_lexicon_tags_23 },
    { 0xc978e158, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab8e165, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b8e169, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5b8e16b, SUC_lexicon_tags_7 },
    { 0x6818e16c, SUC_lexicon_tags_63 },
    { 0xb098e16d, SUC_lexicon_tags_7 },
    { 0xf178e16d, SUC_lexicon_tags_30 },
    { 0x7d78e16f, SUC_lexicon_tags_26 },
    { 0x2758e170, SUC_lexicon_tags_11 },
    { 0x7758e171, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb978e173, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4618e178, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d38e17b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d8e182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d8e189, SUC_lexicon_tags_64 },
    { 0x738e189, SUC_lexicon_tags_14 },
    { 0xc298e189, SUC_lexicon_tags_1 },
    { 0x7158e18c, SUC_lexicon_tags_83 },
    { 0xb6f8e18c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef8e191, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2378e19d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6498e1a0, SUC_lexicon_tags_62 },
    { 0x9ab8e1a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1598e1ae, SUC_lexicon_tags_23 },
    { 0xb398e1af, SUC_lexicon_tags_23 },
    { 0xd4f8e1b0, SUC_lexicon_tags_7 },
    { 0x1838e1b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9eb8e1b3, SUC_lexicon_tags_1 },
    { 0x5f18e1b3, SUC_lexicon_tags_25 },
    { 0xf5f8e1b5, SUC_lexicon_tags_7 },
    { 0xc618e1b5, SUC_lexicon_tags_12 },
    { 0xb2d8e1b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8038e1b9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4118e1be, SUC_lexicon_tags_2 },
    { 0xb6d8e1be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b98e1c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c18e1c3, SUC_lexicon_tags_91 },
    { 0x8dd8e1c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d8e1cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8638e1cf, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc18e1d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd6d8e1d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb8e1dd, SUC_lexicon_tags_25 },
    { 0x9098e1dd, SUC_lexicon_tags_17 },
    { 0x8e98e1dd, SUC_lexicon_tags_49 },
    { 0x6758e1e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9518e1e2, SUC_lexicon_tags_25 },
    { 0x8338e1e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b78e1e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d18e1e9, SUC_lexicon_tags_14 },
    { 0x9d98e1e9, SUC_lexicon_tags_17 },
    { 0xa958e1eb, SUC_lexicon_tags_1 },
    { 0xfed8e1eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f8e1ee, SUC_lexicon_tags_11 },
    { 0xb0f8e1ef, SUC_lexicon_tags_49 },
    { 0xa898e1f0, SUC_lexicon_tags_33 },
    { 0x4cf8e1f1, SUC_lexicon_tags_124 },
    { 0x8ab8e1f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7418e1f6, SUC_lexicon_tags_26 },
    { 0xbff8e1f7, SUC_lexicon_tags_2 },
    { 0xbc38e1f8, SUC_lexicon_tags_37 },
    { 0xfeb8e1f9, SUC_lexicon_tags_25 },
    { 0xd6f8e1fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b8e1fe, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4d78e200, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3498e202, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbff8e204, SUC_lexicon_tags_32 },
    { 0x838e204, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9738e20e, SUC_lexicon_tags_3 },
    { 0xe9f8e20f, SUC_lexicon_tags_234 },
    { 0xab58e210, SUC_lexicon_tags_1 },
    { 0x4a58e211, SUC_lexicon_tags_16 },
    { 0x6498e211, SUC_lexicon_tags_37 },
    { 0x45f8e213, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3dd8e215, SUC_lexicon_tags_17 },
    { 0xf2d8e216, SUC_lexicon_tags_37 },
    { 0xcb78e215, SUC_lexicon_tags_14 },
    { 0x3ef8e218, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4418e21d, SUC_lexicon_tags_87 },
    { 0x72b8e21e, SUC_lexicon_tags_37 },
    { 0x9358e21e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5718e221, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b8e224, SUC_lexicon_tags_7 },
    { 0xf6b8e225, SUC_lexicon_tags_19 },
    { 0x5e98e225, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df8e22b, SUC_lexicon_tags_63 },
    { 0x72b8e22b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff8e231, SUC_lexicon_tags_12 },
    { 0xeb8e232, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5318e235, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb8e238, SUC_lexicon_tags_0 },
    { 0x7978e239, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a78e23f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f18e245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd258e248, SUC_lexicon_tags_8 },
    { 0x9d58e249, SUC_lexicon_tags_17 },
    { 0x2258e24a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b78e250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bd8e252, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8778e257, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9fd8e259, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x838e25e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b8e265, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce98e267, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd8e26f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6018e271, SUC_lexicon_tags_17 },
    { 0xff18e271, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b8e278, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x61d8e27a, SUC_lexicon_tags_43 },
    { 0xa458e27b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d8e27e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5158e281, SUC_lexicon_tags_13 },
    { 0x85d8e282, SUC_lexicon_tags_4 },
    { 0x8bd8e282, SUC_lexicon_tags_25 },
    { 0x3958e284, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf418e288, SUC_lexicon_tags_4 },
    { 0x59f8e289, SUC_lexicon_tags_37 },
    { 0x1d58e28a, SUC_lexicon_tags_47 },
    { 0x3e18e28b, SUC_lexicon_tags_38 },
    { 0x3f8e28a, SUC_lexicon_tags_12 },
    { 0xd18e28c, SUC_lexicon_tags_12 },
    { 0xe238e28b, SUC_lexicon_tags_35 },
    { 0xb498e289, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b8e291, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9c58e293, SUC_lexicon_tags_7 },
    { 0x4698e293, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd8e297, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8378e299, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6058e29d, SUC_lexicon_tags_37 },
    { 0x1ed8e29e, SUC_lexicon_tags_38 },
    { 0xa7b8e29d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x70d8e2a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb738e2a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcb78e2a5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f8e2a9, SUC_lexicon_tags_17 },
    { 0x3b58e2aa, SUC_lexicon_tags_25 },
    { 0x1218e2a9, SUC_lexicon_tags_26 },
    { 0xad58e2ac, SUC_lexicon_tags_30 },
    { 0xcb38e2ac, SUC_lexicon_tags_236 },
    { 0x4238e2ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f8e2b9, SUC_lexicon_tags_25 },
    { 0xdff8e2ba, SUC_lexicon_tags_17 },
    { 0xf4f8e2b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb8e2c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc058e2c6, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x9178e2c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56f8e2cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd8e2ce, SUC_lexicon_tags_60 },
    { 0x7c58e2cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe38e2d1, SUC_lexicon_tags_41 },
    { 0xb178e2d1, SUC_lexicon_tags_11 },
    { 0x82d8e2d3, SUC_lexicon_tags_82 },
    { 0xb458e2d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d8e2dc, SUC_lexicon_tags_1 },
    { 0x958e2dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f8e2e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4b8e2e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8e2eb, SUC_lexicon_tags_46 },
    { 0xf9d8e2ec, SUC_lexicon_tags_47 },
    { 0x7d38e2ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6398e2f0, SUC_lexicon_tags_25 },
    { 0x1798e2f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd318e2f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55f8e2f8, SUC_lexicon_tags_69 },
    { 0x96d8e2f9, SUC_lexicon_tags_11 },
    { 0x93b8e2f9, SUC_lexicon_tags_25 },
    { 0xaf18e2fa, SUC_lexicon_tags_48 },
    { 0xd9d8e2fa, SUC_lexicon_tags_19 },
    { 0x91b8e2fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f8e300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3818e303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb78e307, SUC_lexicon_tags_15 },
    { 0x7558e307, SUC_lexicon_tags_21 },
    { 0xbb8e308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1458e310, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x8378e312, SUC_lexicon_tags_11 },
    { 0xe778e313, SUC_lexicon_tags_31 },
    { 0xca98e314, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x88d8e316, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b8e31a, SUC_lexicon_tags_7 },
    { 0x3278e31a, SUC_lexicon_tags_37 },
    { 0x2998e31c, SUC_lexicon_tags_65 },
    { 0x5618e31b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe58e324, SUC_lexicon_tags_11 },
    { 0xc218e324, SUC_lexicon_tags_37 },
    { 0xd258e326, SUC_lexicon_tags_15 },
    { 0x12f8e326, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed58e32b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb398e32e, SUC_lexicon_tags_11 },
    { 0x4c98e32e, SUC_lexicon_tags_1 },
    { 0x4a38e330, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f8e336, SUC_lexicon_tags_62 },
    { 0xbb58e336, SUC_lexicon_tags_23 },
    { 0x6378e337, SUC_lexicon_tags_17 },
    { 0xe438e336, SUC_lexicon_tags_84 },
    { 0x99f8e338, SUC_lexicon_tags_138 },
    { 0x18f8e33b, SUC_lexicon_tags_17 },
    { 0x5478e339, SUC_lexicon_tags_49 },
    { 0xb858e33d, SUC_lexicon_tags_33 },
    { 0x1058e33d, SUC_lexicon_tags_44 },
    { 0x48b8e33f, SUC_lexicon_tags_32 },
    { 0x6d38e33f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7638e342, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8218e344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6818e347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d8e34a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc78e34e, SUC_lexicon_tags_27 },
    { 0xc578e34f, SUC_lexicon_tags_23 },
    { 0x34f8e34f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3598e353, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc458e357, SUC_lexicon_tags_1 },
    { 0x9178e358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58e35d, SUC_lexicon_tags_25 },
    { 0x4d18e35e, SUC_lexicon_tags_47 },
    { 0xe4d8e35e, SUC_lexicon_tags_38 },
    { 0x8f8e360, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb8e366, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a98e368, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd498e36f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8e373, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf8e376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7318e379, SUC_lexicon_tags_1078 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc478e381, SUC_lexicon_tags_12 },
    { 0xa038e381, SUC_lexicon_tags_19 },
    { 0x1498e383, SUC_lexicon_tags_43 },
    { 0x71f8e384, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf8e386, SUC_lexicon_tags_15 },
    { 0xb918e387, SUC_lexicon_tags_7 },
    { 0x2318e388, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d8e38b, SUC_lexicon_tags_2 },
    { 0x2258e38c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f18e38e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a98e392, SUC_lexicon_tags_7 },
    { 0x7858e393, SUC_lexicon_tags_47 },
    { 0x7a38e394, SUC_lexicon_tags_11 },
    { 0xc18e395, SUC_lexicon_tags_25 },
    { 0x44d8e396, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x62b8e398, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32b8e39b, SUC_lexicon_tags_26 },
    { 0xa1f8e39c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738e3a0, SUC_lexicon_tags_13 },
    { 0x6138e3a0, SUC_lexicon_tags_45 },
    { 0xf438e3a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a58e3a6, SUC_lexicon_tags_26 },
    { 0xd078e3a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9d8e3a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b8e3b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe4d8e3b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6798e3b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfd8e3b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd38e3bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2538e3c0, SUC_lexicon_tags_17 },
    { 0xd878e3c1, SUC_lexicon_tags_37 },
    { 0xdb98e3c2, SUC_lexicon_tags_37 },
    { 0x2218e3c2, SUC_lexicon_tags_17 },
    { 0x87b8e3c1, SUC_lexicon_tags_37 },
    { 0x92b8e3c3, SUC_lexicon_tags_17 },
    { 0x4598e3c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd58e3ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b8e3d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5318e3d3, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0x7738e3d6, SUC_lexicon_tags_11 },
    { 0x1b8e3d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e58e3dd, SUC_lexicon_tags_7 },
    { 0x8e78e3de, SUC_lexicon_tags_69 },
    { 0x5998e3de, SUC_lexicon_tags_60 },
    { 0x6f58e3e0, SUC_lexicon_tags_1 },
    { 0x9698e3dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf718e3e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c38e3e7, SUC_lexicon_tags_2 },
    { 0x7b98e3e8, SUC_lexicon_tags_13 },
    { 0x2998e3e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd918e3ed, SUC_lexicon_tags_63 },
    { 0x5ab8e3ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d8e3f1, SUC_lexicon_tags_15 },
    { 0x4878e3f2, SUC_lexicon_tags_13 },
    { 0xc398e3f1, SUC_lexicon_tags_7 },
    { 0x24f8e3f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa178e3f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9998e3ff, SUC_lexicon_tags_6 },
    { 0xdb18e400, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd58e403, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5ff8e405, SUC_lexicon_tags_32 },
    { 0x7498e405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x74d8e409, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf178e40e, SUC_lexicon_tags_26 },
    { 0x8818e40e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb798e412, SUC_lexicon_tags_63 },
    { 0x858e413, SUC_lexicon_tags_7 },
    { 0x4158e414, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9558e416, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e18e422, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d8e426, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5298e428, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4998e42a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f8e42e, SUC_lexicon_tags_12 },
    { 0xdbb8e42f, SUC_lexicon_tags_43 },
    { 0xd1d8e42f, SUC_lexicon_tags_26 },
    { 0x5ed8e431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd8e436, SUC_lexicon_tags_1 },
    { 0x4878e437, SUC_lexicon_tags_37 },
    { 0x2b18e436, SUC_lexicon_tags_60 },
    { 0xf0b8e437, SUC_lexicon_tags_8 },
    { 0x3158e436, SUC_lexicon_tags_45 },
    { 0x4ef8e43a, SUC_lexicon_tags_37 },
    { 0xd738e43c, SUC_lexicon_tags_17 },
    { 0x8038e43d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2298e43f, SUC_lexicon_tags_60 },
    { 0x2c38e440, SUC_lexicon_tags_1 },
    { 0xa098e441, SUC_lexicon_tags_2 },
    { 0x4c18e442, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d8e446, SUC_lexicon_tags_43 },
    { 0x3c18e447, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f8e44a, SUC_lexicon_tags_55 },
    { 0x9e78e44b, SUC_lexicon_tags_28 },
    { 0x9658e44c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b8e44f, SUC_lexicon_tags_290 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f8e454, SUC_lexicon_tags_9 },
    { 0x2418e455, SUC_lexicon_tags_26 },
    { 0x2a78e455, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc858e458, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe18e45f, SUC_lexicon_tags_1 },
    { 0x76b8e460, SUC_lexicon_tags_33 },
    { 0xb978e461, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f8e464, SUC_lexicon_tags_1 },
    { 0x4dd8e465, SUC_lexicon_tags_43 },
    { 0xb718e466, SUC_lexicon_tags_17 },
    { 0x7f78e467, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8e46b, SUC_lexicon_tags_23 },
    { 0x81f8e46c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x738e470, SUC_lexicon_tags_7 },
    { 0x2db8e471, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b8e474, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd8e479, SUC_lexicon_tags_11 },
    { 0x69d8e479, SUC_lexicon_tags_27 },
    { 0xf58e479, SUC_lexicon_tags_15 },
    { 0x2a78e47c, SUC_lexicon_tags_37 },
    { 0xe8d8e47d, SUC_lexicon_tags_25 },
    { 0x5c18e47e, SUC_lexicon_tags_25 },
    { 0x4878e47f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab8e482, SUC_lexicon_tags_29 },
    { 0xce18e482, SUC_lexicon_tags_15 },
    { 0x3a58e483, SUC_lexicon_tags_7 },
    { 0x2e18e482, SUC_lexicon_tags_25 },
    { 0x7318e485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c38e489, SUC_lexicon_tags_10 },
    { 0x4718e48a, SUC_lexicon_tags_17 },
    { 0x1738e48b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d8e497, SUC_lexicon_tags_37 },
    { 0x5758e498, SUC_lexicon_tags_25 },
    { 0x2c58e497, SUC_lexicon_tags_30 },
    { 0xb678e49a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd98e4a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a38e4a3, SUC_lexicon_tags_1 },
    { 0xbfb8e4a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa78e4a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b8e4b2, SUC_lexicon_tags_15 },
    { 0x7418e4b3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4158e4b8, SUC_lexicon_tags_68 },
    { 0xdb8e4b9, SUC_lexicon_tags_12 },
    { 0xc238e4ba, SUC_lexicon_tags_23 },
    { 0x7af8e4bb, SUC_lexicon_tags_25 },
    { 0x8918e4ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe458e4c2, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x7578e4c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1738e4c7, SUC_lexicon_tags_30 },
    { 0xc058e4c8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b8e4cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1158e4d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4d8e4d4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f78e4da, SUC_lexicon_tags_25 },
    { 0x7e18e4db, SUC_lexicon_tags_11 },
    { 0x9d38e4da, SUC_lexicon_tags_7 },
    { 0x5698e4dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb8e4e3, SUC_lexicon_tags_25 },
    { 0x1f18e4e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf8e4ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4678e4f2, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xfcb8e4f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59b8e4f6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7638e4f9, SUC_lexicon_tags_15 },
    { 0xe758e4fa, SUC_lexicon_tags_30 },
    { 0x698e4fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3558e501, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5338e504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x918e508, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f8e50b, SUC_lexicon_tags_30 },
    { 0x4c98e50b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c78e510, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8178e513, SUC_lexicon_tags_39 },
    { 0x2138e514, SUC_lexicon_tags_1 },
    { 0x6ad8e514, SUC_lexicon_tags_27 },
    { 0x6298e516, SUC_lexicon_tags_29 },
    { 0x9e38e516, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4798e522, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8c78e524, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaf38e526, SUC_lexicon_tags_12 },
    { 0xf1f8e527, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8e52a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef8e52d, SUC_lexicon_tags_37 },
    { 0xc078e52e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb8e533, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2618e537, SUC_lexicon_tags_32 },
    { 0x8538e537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d78e53a, SUC_lexicon_tags_160 },
    { 0xbef8e53a, SUC_lexicon_tags_1 },
    { 0x4e58e53c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b8e53f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb438e542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1938e544, SUC_lexicon_tags_1 },
    { 0x3a98e545, SUC_lexicon_tags_2 },
    { 0x8e38e546, SUC_lexicon_tags_37 },
    { 0xd8b8e545, SUC_lexicon_tags_21 },
    { 0x85b8e548, SUC_lexicon_tags_17 },
    { 0xdfd8e545, SUC_lexicon_tags_70 },
    { 0x7378e54a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d8e550, SUC_lexicon_tags_63 },
    { 0x1358e551, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa578e555, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb278e558, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1378e55a, SUC_lexicon_tags_43 },
    { 0x7758e55b, SUC_lexicon_tags_16 },
    { 0x9038e55c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7698e561, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3058e563, SUC_lexicon_tags_14 },
    { 0x8bd8e563, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb8e56a, SUC_lexicon_tags_43 },
    { 0x7858e56b, SUC_lexicon_tags_15 },
    { 0x8e58e56a, SUC_lexicon_tags_6 },
    { 0x3cf8e56d, SUC_lexicon_tags_0 },
    { 0x3738e56e, SUC_lexicon_tags_29 },
    { 0xc218e56f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb218e573, SUC_lexicon_tags_66 },
    { 0xe0f8e574, SUC_lexicon_tags_2 },
    { 0xa338e575, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d8e578, SUC_lexicon_tags_31 },
    { 0x1198e579, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x27f8e57b, SUC_lexicon_tags_0 },
    { 0x5538e57b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe718e586, SUC_lexicon_tags_81 },
    { 0xced8e587, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d18e58c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b8e594, SUC_lexicon_tags_23 },
    { 0x3c18e595, SUC_lexicon_tags_60 },
    { 0x3318e596, SUC_lexicon_tags_1 },
    { 0x2f18e597, SUC_lexicon_tags_66 },
    { 0x2818e597, SUC_lexicon_tags_32 },
    { 0xcc38e599, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfcb8e59b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xff18e59e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e98e5a1, SUC_lexicon_tags_60 },
    { 0x338e5a2, SUC_lexicon_tags_25 },
    { 0x5058e5a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7358e5a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c58e5a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1318e5a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa38e5ac, SUC_lexicon_tags_3 },
    { 0x2558e5ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d8e5b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d98e5b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa018e5b4, SUC_lexicon_tags_23 },
    { 0x62f8e5b5, SUC_lexicon_tags_15 },
    { 0xcb18e5b5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc3d8e5b8, SUC_lexicon_tags_0 },
    { 0x4f18e5b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2f8e5bb, SUC_lexicon_tags_15 },
    { 0xf558e5bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b8e5c0, SUC_lexicon_tags_8 },
    { 0xd5f8e5c1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9978e5c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xedd8e5c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1358e5cd, SUC_lexicon_tags_8 },
    { 0xc78e5ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xab78e5d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf378e5d6, SUC_lexicon_tags_16 },
    { 0xfe78e5d6, SUC_lexicon_tags_2 },
    { 0x378e5d8, SUC_lexicon_tags_9 },
    { 0xaed8e5d9, SUC_lexicon_tags_25 },
    { 0xc8d8e5d9, SUC_lexicon_tags_33 },
    { 0x4278e5da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d8e5de, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda38e5e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42d8e5e4, SUC_lexicon_tags_13 },
    { 0x1f98e5e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb8e5e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f8e5ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4058e5f0, SUC_lexicon_tags_4 },
    { 0x5218e5f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f8e5f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb8e5f9, SUC_lexicon_tags_7 },
    { 0xacb8e5f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8e5fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2018e600, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b8e604, SUC_lexicon_tags_25 },
    { 0xa3d8e605, SUC_lexicon_tags_37 },
    { 0x30b8e604, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x358e60f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe678e616, SUC_lexicon_tags_11 },
    { 0x9638e617, SUC_lexicon_tags_85 },
    { 0x258e618, SUC_lexicon_tags_1 },
    { 0x4658e616, SUC_lexicon_tags_0 },
    { 0xbef8e617, SUC_lexicon_tags_37 },
    { 0x2d38e618, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86d8e61f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa038e622, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc518e627, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb078e629, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1f8e62b, SUC_lexicon_tags_25 },
    { 0xb258e62c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd478e630, SUC_lexicon_tags_23 },
    { 0xc8f8e631, SUC_lexicon_tags_0 },
    { 0x56d8e632, SUC_lexicon_tags_11 },
    { 0xfbb8e632, SUC_lexicon_tags_148 },
    { 0x8218e632, SUC_lexicon_tags_7 },
    { 0xe0b8e632, SUC_lexicon_tags_82 },
    { 0x6958e636, SUC_lexicon_tags_38 },
    { 0xc0d8e636, SUC_lexicon_tags_243 },
    { 0x86b8e637, SUC_lexicon_tags_11 },
    { 0xe918e639, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe498e63b, SUC_lexicon_tags_35 },
    { 0x6218e63c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb158e63f, SUC_lexicon_tags_55 },
    { 0xd98e640, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90f8e642, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9b8e644, SUC_lexicon_tags_23 },
    { 0x9d98e645, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf8e648, SUC_lexicon_tags_61 },
    { 0xe7f8e649, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf98e64c, SUC_lexicon_tags_1 },
    { 0xc1f8e64d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1638e650, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe4f8e652, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd0d8e654, SUC_lexicon_tags_25 },
    { 0x42f8e655, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f8e658, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db8e65b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc58e661, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5838e664, SUC_lexicon_tags_33 },
    { 0x8a58e665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6918e668, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f58e66c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f8e677, SUC_lexicon_tags_11 },
    { 0x3338e678, SUC_lexicon_tags_56 },
    { 0xa8d8e678, SUC_lexicon_tags_82 },
    { 0xa38e67a, SUC_lexicon_tags_37 },
    { 0x6098e67a, SUC_lexicon_tags_38 },
    { 0xf438e67c, SUC_lexicon_tags_46 },
    { 0x88b8e679, SUC_lexicon_tags_10 },
    { 0xb2d8e67e, SUC_lexicon_tags_62 },
    { 0x55b8e67b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe0d8e681, SUC_lexicon_tags_17 },
    { 0x2bf8e681, SUC_lexicon_tags_2 },
    { 0x9078e681, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf8e686, SUC_lexicon_tags_60 },
    { 0x49f8e687, SUC_lexicon_tags_46 },
    { 0xf558e688, SUC_lexicon_tags_8 },
    { 0xe238e689, SUC_lexicon_tags_33 },
    { 0x9ab8e689, SUC_lexicon_tags_126 },
    { 0x6ef8e68b, SUC_lexicon_tags_30 },
    { 0x8018e687, SUC_lexicon_tags_29 },
    { 0x8a78e68c, SUC_lexicon_tags_57 },
    { 0x6ab8e68e, SUC_lexicon_tags_1 },
    { 0xf958e68f, SUC_lexicon_tags_12 },
    { 0xf258e686, SUC_lexicon_tags_37 },
    { 0x9238e691, SUC_lexicon_tags_7 },
    { 0xd958e687, SUC_lexicon_tags_26 },
    { 0xddd8e693, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaf78e695, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3158e699, SUC_lexicon_tags_7 },
    { 0xd858e69a, SUC_lexicon_tags_27 },
    { 0xb078e69a, SUC_lexicon_tags_8 },
    { 0x67b8e69c, SUC_lexicon_tags_14 },
    { 0x3738e69d, SUC_lexicon_tags_17 },
    { 0xbcd8e69e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad8e6a4, SUC_lexicon_tags_60 },
    { 0x5518e6a4, SUC_lexicon_tags_25 },
    { 0xd098e6a6, SUC_lexicon_tags_15 },
    { 0xbf18e6a7, SUC_lexicon_tags_49 },
    { 0x7798e6a8, SUC_lexicon_tags_2 },
    { 0x2558e6a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x12d8e6ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ad8e6ad, SUC_lexicon_tags_1 },
    { 0x3ff8e6ae, SUC_lexicon_tags_47 },
    { 0xfbf8e6af, SUC_lexicon_tags_4 },
    { 0xac18e6b0, SUC_lexicon_tags_11 },
    { 0xcf98e6af, SUC_lexicon_tags_15 },
    { 0x17d8e6b2, SUC_lexicon_tags_10 },
    { 0x57d8e6af, SUC_lexicon_tags_15 },
    { 0x1e98e6ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7358e6b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd58e6b9, SUC_lexicon_tags_12 },
    { 0xbd18e6ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18e6be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x938e6c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d8e6c7, SUC_lexicon_tags_31 },
    { 0xf6f8e6c8, SUC_lexicon_tags_11 },
    { 0x6038e6c9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2158e6cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa918e6ce, SUC_lexicon_tags_14 },
    { 0x7938e6ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b78e6d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90f8e6d3, SUC_lexicon_tags_12 },
    { 0x938e6d4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd618e6d6, SUC_lexicon_tags_30 },
    { 0x88d8e6d7, SUC_lexicon_tags_92 },
    { 0x24d8e6d7, SUC_lexicon_tags_12 },
    { 0xa578e6d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb8e6de, SUC_lexicon_tags_37 },
    { 0xe3d8e6de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe738e6e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8078e6e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c38e6e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8398e6ec, SUC_lexicon_tags_43 },
    { 0x9838e6ec, SUC_lexicon_tags_23 },
    { 0xfe18e6ee, SUC_lexicon_tags_29 },
    { 0x7c18e6ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf558e6f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf8e6f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1f8e6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6398e6f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8e6fc, SUC_lexicon_tags_47 },
    { 0xbcb8e6fc, SUC_lexicon_tags_30 },
    { 0x27d8e6fe, SUC_lexicon_tags_33 },
    { 0x838e6ff, SUC_lexicon_tags_48 },
    { 0x9f58e700, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b8e703, SUC_lexicon_tags_31 },
    { 0x4fd8e704, SUC_lexicon_tags_2 },
    { 0x23b8e705, SUC_lexicon_tags_39 },
    { 0x1af8e705, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xa638e708, SUC_lexicon_tags_254 },
    { 0xcdb8e709, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b8e70d, SUC_lexicon_tags_47 },
    { 0xe218e70e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2638e710, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d8e713, SUC_lexicon_tags_75 },
    { 0x1cb8e714, SUC_lexicon_tags_27 },
    { 0xd6d8e713, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d8e719, SUC_lexicon_tags_28 },
    { 0x39d8e719, SUC_lexicon_tags_19 },
    { 0x3bf8e71b, SUC_lexicon_tags_37 },
    { 0xf1f8e71c, SUC_lexicon_tags_9 },
    { 0x4398e71a, SUC_lexicon_tags_0 },
    { 0xffd8e71e, SUC_lexicon_tags_26 },
    { 0xf5b8e71f, SUC_lexicon_tags_27 },
    { 0x7758e71f, SUC_lexicon_tags_25 },
    { 0x1a58e721, SUC_lexicon_tags_92 },
    { 0x6618e71e, SUC_lexicon_tags_15 },
    { 0x5258e719, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2438e727, SUC_lexicon_tags_0 },
    { 0x8ef8e728, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed8e72d, SUC_lexicon_tags_25 },
    { 0x7fd8e72d, SUC_lexicon_tags_17 },
    { 0x3f78e72f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d8e736, SUC_lexicon_tags_11 },
    { 0xe018e736, SUC_lexicon_tags_14 },
    { 0x47d8e738, SUC_lexicon_tags_79 },
    { 0x5398e739, SUC_lexicon_tags_44 },
    { 0x998e73a, SUC_lexicon_tags_31 },
    { 0xa858e738, SUC_lexicon_tags_70 },
    { 0xc838e73c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d98e73e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5a58e740, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0xd658e743, SUC_lexicon_tags_17 },
    { 0x68d8e743, SUC_lexicon_tags_2 },
    { 0xdff8e743, SUC_lexicon_tags_23 },
    { 0xa8f8e743, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f8e74b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6058e750, SUC_lexicon_tags_19 },
    { 0xd8e751, SUC_lexicon_tags_15 },
    { 0xb5d8e751, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde18e758, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1118e75a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd998e75d, SUC_lexicon_tags_19 },
    { 0x84f8e75e, SUC_lexicon_tags_11 },
    { 0xa7d8e75f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f8e762, SUC_lexicon_tags_15 },
    { 0x9718e763, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1458e767, SUC_lexicon_tags_0 },
    { 0xbc58e768, SUC_lexicon_tags_65 },
    { 0x7cb8e768, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5838e76c, SUC_lexicon_tags_15 },
    { 0x3b8e76d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa638e770, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf678e775, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xac18e777, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab8e77a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7558e77c, SUC_lexicon_tags_12 },
    { 0xa78e77c, SUC_lexicon_tags_12 },
    { 0x11f8e77c, SUC_lexicon_tags_1 },
    { 0xe978e77f, SUC_lexicon_tags_25 },
    { 0xf758e77f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d8e783, SUC_lexicon_tags_55 },
    { 0xa3f8e784, SUC_lexicon_tags_60 },
    { 0x1778e785, SUC_lexicon_tags_7 },
    { 0x2438e786, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa658e788, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b8e78a, SUC_lexicon_tags_6 },
    { 0xa0d8e78b, SUC_lexicon_tags_25 },
    { 0x9098e78c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x758e78e, SUC_lexicon_tags_12 },
    { 0x5bd8e78f, SUC_lexicon_tags_60 },
    { 0x9ad8e78e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c38e798, SUC_lexicon_tags_45 },
    { 0x4058e798, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3558e79c, SUC_lexicon_tags_1 },
    { 0xe118e79d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x318e7a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd638e7a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f8e7ab, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf7d8e7ad, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8c38e7af, SUC_lexicon_tags_29 },
    { 0x26f8e7b0, SUC_lexicon_tags_103 },
    { 0xe498e7b0, SUC_lexicon_tags_37 },
    { 0x3038e7b0, SUC_lexicon_tags_12 },
    { 0xc98e7b3, SUC_lexicon_tags_19 },
    { 0xb818e7b4, SUC_lexicon_tags_7 },
    { 0xaa38e7b5, SUC_lexicon_tags_4 },
    { 0xdf98e7b6, SUC_lexicon_tags_39 },
    { 0xa258e7b7, SUC_lexicon_tags_2 },
    { 0x6238e7b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e58e7ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d8e7bd, SUC_lexicon_tags_17 },
    { 0x9658e7bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8378e7c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c98e7c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb818e7c6, SUC_lexicon_tags_47 },
    { 0xbf58e7c6, SUC_lexicon_tags_9 },
    { 0xf298e7c8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a18e7cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x40f8e7ce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb998e7d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d8e7d6, SUC_lexicon_tags_7 },
    { 0x20b8e7d6, SUC_lexicon_tags_37 },
    { 0x9d98e7d7, SUC_lexicon_tags_1 },
    { 0x3d38e7d9, SUC_lexicon_tags_38 },
    { 0xdc38e7da, SUC_lexicon_tags_1 },
    { 0xd18e7da, SUC_lexicon_tags_31 },
    { 0xe58e7dc, SUC_lexicon_tags_4 },
    { 0x8e18e7d9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xb7b8e7df, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8298e7e1, SUC_lexicon_tags_2 },
    { 0xd738e7e2, SUC_lexicon_tags_46 },
    { 0x9e18e7e2, SUC_lexicon_tags_37 },
    { 0x8918e7e4, SUC_lexicon_tags_43 },
    { 0xc018e7e2, SUC_lexicon_tags_6 },
    { 0x9638e7e6, SUC_lexicon_tags_7 },
    { 0x7138e7e7, SUC_lexicon_tags_37 },
    { 0xb678e7e8, SUC_lexicon_tags_25 },
    { 0xca98e7e8, SUC_lexicon_tags_14 },
    { 0x2d18e7ea, SUC_lexicon_tags_28 },
    { 0x25d8e7e6, SUC_lexicon_tags_25 },
    { 0x2958e7e7, SUC_lexicon_tags_37 },
    { 0x7c58e7e6, SUC_lexicon_tags_17 },
    { 0xb3d8e7ee, SUC_lexicon_tags_12 },
    { 0x9938e7e7, SUC_lexicon_tags_14 },
    { 0xfef8e7e9, SUC_lexicon_tags_7 },
    { 0x6b78e7ee, SUC_lexicon_tags_7 },
    { 0x5e98e7eb, SUC_lexicon_tags_233 },
    { 0x2398e7f3, SUC_lexicon_tags_37 },
    { 0x99b8e7e2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5c58e7f6, SUC_lexicon_tags_11 },
    { 0x2ed8e7f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f8e7fd, SUC_lexicon_tags_25 },
    { 0x4758e7fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc338e801, SUC_lexicon_tags_15 },
    { 0x4098e802, SUC_lexicon_tags_15 },
    { 0x5bf8e803, SUC_lexicon_tags_56 },
    { 0x25f8e804, SUC_lexicon_tags_37 },
    { 0xfdb8e804, SUC_lexicon_tags_25 },
    { 0x1138e803, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd8e80a, SUC_lexicon_tags_37 },
    { 0xc1f8e80b, SUC_lexicon_tags_506 },
    { 0xe1d8e80c, SUC_lexicon_tags_43 },
    { 0xfbd8e80d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab8e813, SUC_lexicon_tags_23 },
    { 0xdf78e813, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9298e817, SUC_lexicon_tags_29 },
    { 0x9b38e817, SUC_lexicon_tags_38 },
    { 0x8818e818, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e78e81e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb18e821, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9138e823, SUC_lexicon_tags_42 },
    { 0xbe58e823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d8e827, SUC_lexicon_tags_13 },
    { 0xda18e827, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf8e82b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb018e82f, SUC_lexicon_tags_12 },
    { 0x5a78e830, SUC_lexicon_tags_1 },
    { 0xa438e831, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce58e835, SUC_lexicon_tags_63 },
    { 0xa018e835, SUC_lexicon_tags_19 },
    { 0xc0f8e837, SUC_lexicon_tags_79 },
    { 0x71f8e838, SUC_lexicon_tags_45 },
    { 0xcc98e835, SUC_lexicon_tags_11 },
    { 0xa078e835, SUC_lexicon_tags_148 },
    { 0xa578e837, SUC_lexicon_tags_6 },
    { 0xaf58e83c, SUC_lexicon_tags_238 },
    { 0x56d8e83d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f8e840, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xd998e842, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9eb8e844, SUC_lexicon_tags_25 },
    { 0x7778e845, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3898e848, SUC_lexicon_tags_71 },
    { 0xa778e848, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2318e84b, SUC_lexicon_tags_66 },
    { 0xa618e84c, SUC_lexicon_tags_55 },
    { 0x9838e84d, SUC_lexicon_tags_31 },
    { 0x3978e84d, SUC_lexicon_tags_19 },
    { 0xb758e84f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd8e856, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6798e859, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b8e85c, SUC_lexicon_tags_3 },
    { 0x9e38e85d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3378e861, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3178e863, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b8e86b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa338e86f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x258e875, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc718e878, SUC_lexicon_tags_2 },
    { 0x1b98e879, SUC_lexicon_tags_2 },
    { 0xbf78e87a, SUC_lexicon_tags_44 },
    { 0xe7f8e87a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab78e881, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x458e883, SUC_lexicon_tags_7 },
    { 0x598e883, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1898e886, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb938e888, SUC_lexicon_tags_17 },
    { 0x1778e888, SUC_lexicon_tags_45 },
    { 0xebd8e88a, SUC_lexicon_tags_17 },
    { 0x9f78e88a, SUC_lexicon_tags_37 },
    { 0xd0b8e88a, SUC_lexicon_tags_38 },
    { 0x5d18e88d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3398e891, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f8e894, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8738e896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd758e89a, SUC_lexicon_tags_542 },
    { 0xa378e89b, SUC_lexicon_tags_1 },
    { 0x7918e89c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83f8e89e, SUC_lexicon_tags_69 },
    { 0x62f8e89f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1038e8a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef78e8a7, SUC_lexicon_tags_25 },
    { 0x3818e8a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af8e8ab, SUC_lexicon_tags_2 },
    { 0xebd8e8ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d18e8ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb38e8b3, SUC_lexicon_tags_15 },
    { 0x1898e8b4, SUC_lexicon_tags_27 },
    { 0x1c18e8b5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x83b8e8b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5518e8bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5e98e8bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1238e8c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb638e8c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1278e8cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5f8e8ce, SUC_lexicon_tags_47 },
    { 0x7358e8cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b8e8d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee58e8da, SUC_lexicon_tags_21 },
    { 0x2b38e8db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c58e8df, SUC_lexicon_tags_9 },
    { 0xcc18e8e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3418e8e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc198e8e5, SUC_lexicon_tags_7 },
    { 0x35d8e8e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f98e8e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd798e8f1, SUC_lexicon_tags_39 },
    { 0x5698e8f2, SUC_lexicon_tags_37 },
    { 0xdff8e8f3, SUC_lexicon_tags_56 },
    { 0xbcf8e8f2, SUC_lexicon_tags_7 },
    { 0x6d18e8f5, SUC_lexicon_tags_17 },
    { 0xc38e8f6, SUC_lexicon_tags_49 },
    { 0xa3f8e8f7, SUC_lexicon_tags_15 },
    { 0x118e8f8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x2318e8fb, SUC_lexicon_tags_43 },
    { 0x4bd8e8fb, SUC_lexicon_tags_7 },
    { 0xbb78e8fd, SUC_lexicon_tags_37 },
    { 0xac98e8fc, SUC_lexicon_tags_66 },
    { 0x7a58e8ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5158e903, SUC_lexicon_tags_6 },
    { 0xabb8e904, SUC_lexicon_tags_0 },
    { 0xd3f8e905, SUC_lexicon_tags_2 },
    { 0xc738e903, SUC_lexicon_tags_15 },
    { 0x798e907, SUC_lexicon_tags_49 },
    { 0xb838e907, SUC_lexicon_tags_37 },
    { 0xb738e907, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1918e90b, SUC_lexicon_tags_25 },
    { 0x7b98e90b, SUC_lexicon_tags_1 },
    { 0xbb78e90d, SUC_lexicon_tags_13 },
    { 0x6378e90e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa038e913, SUC_lexicon_tags_60 },
    { 0x2f8e914, SUC_lexicon_tags_37 },
    { 0x7b58e913, SUC_lexicon_tags_2 },
    { 0xf938e916, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xae58e918, SUC_lexicon_tags_13 },
    { 0xf718e919, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x80b8e91b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bf8e91d, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb318e921, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d78e928, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd138e92a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b8e930, SUC_lexicon_tags_0 },
    { 0x6338e931, SUC_lexicon_tags_2 },
    { 0xc4d8e931, SUC_lexicon_tags_37 },
    { 0x2e18e933, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd438e935, SUC_lexicon_tags_15 },
    { 0x7578e936, SUC_lexicon_tags_11 },
    { 0xbc78e937, SUC_lexicon_tags_29 },
    { 0x1d18e936, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe698e93a, SUC_lexicon_tags_37 },
    { 0xcb18e93b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9958e941, SUC_lexicon_tags_9 },
    { 0xecb8e942, SUC_lexicon_tags_17 },
    { 0x1b78e942, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9418e947, SUC_lexicon_tags_28 },
    { 0xc878e948, SUC_lexicon_tags_37 },
    { 0x3a38e949, SUC_lexicon_tags_29 },
    { 0x14d8e94a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7258e94c, SUC_lexicon_tags_47 },
    { 0x5df8e94d, SUC_lexicon_tags_39 },
    { 0x8658e94e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6858e950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d58e952, SUC_lexicon_tags_8 },
    { 0xcfb8e953, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2418e956, SUC_lexicon_tags_37 },
    { 0x96b8e956, SUC_lexicon_tags_124 },
    { 0xc558e956, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4718e95a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d8e95f, SUC_lexicon_tags_26 },
    { 0x178e960, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd8e964, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x9f18e966, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf8e96d, SUC_lexicon_tags_47 },
    { 0x7c38e96e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d8e973, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd8e976, SUC_lexicon_tags_31 },
    { 0xf758e977, SUC_lexicon_tags_1 },
    { 0x2d58e978, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4558e97f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d8e983, SUC_lexicon_tags_48 },
    { 0xa6b8e983, SUC_lexicon_tags_42 },
    { 0xbef8e985, SUC_lexicon_tags_6 },
    { 0x8d58e986, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce58e988, SUC_lexicon_tags_57 },
    { 0x23d8e989, SUC_lexicon_tags_17 },
    { 0xcd8e98a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e58e98d, SUC_lexicon_tags_15 },
    { 0x25f8e98d, SUC_lexicon_tags_31 },
    { 0x6218e98f, SUC_lexicon_tags_8 },
    { 0xc6f8e990, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3138e993, SUC_lexicon_tags_44 },
    { 0x4d18e993, SUC_lexicon_tags_1 },
    { 0x1ff8e994, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b58e9a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xae38e9a4, SUC_lexicon_tags_25 },
    { 0x678e9a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa598e9a7, SUC_lexicon_tags_37 },
    { 0xf358e9a8, SUC_lexicon_tags_1 },
    { 0xa738e9a7, SUC_lexicon_tags_12 },
    { 0x3e98e9a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb8e9ae, SUC_lexicon_tags_48 },
    { 0xe398e9ae, SUC_lexicon_tags_2 },
    { 0xef58e9b0, SUC_lexicon_tags_17 },
    { 0xd458e9b0, SUC_lexicon_tags_29 },
    { 0x6fd8e9b2, SUC_lexicon_tags_12 },
    { 0x4b38e9b2, SUC_lexicon_tags_12 },
    { 0xd9f8e9b4, SUC_lexicon_tags_38 },
    { 0xb018e9b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f8e9bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6878e9c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf18e9c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb838e9c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1698e9cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8578e9cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb358e9d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb718e9d4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb8e9d9, SUC_lexicon_tags_1 },
    { 0xafb8e9d9, SUC_lexicon_tags_0 },
    { 0xbef8e9db, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc6b8e9dd, SUC_lexicon_tags_34 },
    { 0x7978e9de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86d8e9e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56b8e9e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb8e9e5, SUC_lexicon_tags_38 },
    { 0x69f8e9e6, SUC_lexicon_tags_25 },
    { 0x7bf8e9e7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6878e9ed, SUC_lexicon_tags_32 },
    { 0x2e78e9ed, SUC_lexicon_tags_6 },
    { 0x5d58e9ef, SUC_lexicon_tags_23 },
    { 0x3f58e9f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c18e9f7, SUC_lexicon_tags_31 },
    { 0xeff8e9f8, SUC_lexicon_tags_1 },
    { 0xd278e9f9, SUC_lexicon_tags_1 },
    { 0x8318e9fa, SUC_lexicon_tags_12 },
    { 0x2378e9fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe018e9fe, SUC_lexicon_tags_19 },
    { 0x2298e9ff, SUC_lexicon_tags_17 },
    { 0x12f8e9ff, SUC_lexicon_tags_12 },
    { 0xe058ea00, SUC_lexicon_tags_8 },
    { 0x32f8ea00, SUC_lexicon_tags_87 },
    { 0xfd8ea02, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xfc78ea05, SUC_lexicon_tags_48 },
    { 0x1ad8ea05, SUC_lexicon_tags_17 },
    { 0xf6b8ea07, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbbb8ea09, SUC_lexicon_tags_12 },
    { 0x2af8ea0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e98ea0c, SUC_lexicon_tags_37 },
    { 0x25f8ea0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf338ea10, SUC_lexicon_tags_22 },
    { 0x35d8ea10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb218ea13, SUC_lexicon_tags_28 },
    { 0x77b8ea13, SUC_lexicon_tags_62 },
    { 0x5818ea15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4058ea1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc058ea21, SUC_lexicon_tags_123 },
    { 0x2b98ea22, SUC_lexicon_tags_4 },
    { 0x1078ea23, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x558ea25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa518ea27, SUC_lexicon_tags_2 },
    { 0xb858ea27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e98ea2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9138ea30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6558ea32, SUC_lexicon_tags_1 },
    { 0xeb8ea33, SUC_lexicon_tags_60 },
    { 0xfdf8ea34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf338ea3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc878ea4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf658ea4f, SUC_lexicon_tags_28 },
    { 0x5bf8ea50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff8ea54, SUC_lexicon_tags_11 },
    { 0xed8ea55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ad8ea57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa78ea5a, SUC_lexicon_tags_68 },
    { 0xe958ea5b, SUC_lexicon_tags_1 },
    { 0x318ea5b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d8ea60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6438ea67, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x17d8ea69, SUC_lexicon_tags_87 },
    { 0x4af8ea6a, SUC_lexicon_tags_14 },
    { 0x2d58ea6b, SUC_lexicon_tags_12 },
    { 0x94f8ea6a, SUC_lexicon_tags_0 },
    { 0xa6f8ea6b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3b18ea6f, SUC_lexicon_tags_17 },
    { 0x2e58ea70, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5078ea73, SUC_lexicon_tags_29 },
    { 0x4398ea74, SUC_lexicon_tags_11 },
    { 0x80b8ea73, SUC_lexicon_tags_19 },
    { 0x6838ea76, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb8ea78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2638ea7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd98ea81, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6758ea88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x44f8ea8a, SUC_lexicon_tags_10 },
    { 0x69f8ea8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95d8ea8d, SUC_lexicon_tags_275 },
    { 0x5ed8ea8e, SUC_lexicon_tags_17 },
    { 0x9c38ea8f, SUC_lexicon_tags_17 },
    { 0x1a78ea90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87b8ea92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf3b8ea94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d8ea9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x318ea9d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9818eaa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc38eaa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc78eaab, SUC_lexicon_tags_11 },
    { 0x5738eaab, SUC_lexicon_tags_15 },
    { 0x4758eaad, SUC_lexicon_tags_87 },
    { 0xa9b8eaad, SUC_lexicon_tags_3 },
    { 0xbbf8eaae, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd8eab3, SUC_lexicon_tags_25 },
    { 0x7bf8eab3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8b8eab6, SUC_lexicon_tags_7 },
    { 0x3a38eab7, SUC_lexicon_tags_2 },
    { 0xb778eab6, SUC_lexicon_tags_37 },
    { 0xef98eab9, SUC_lexicon_tags_21 },
    { 0x1338eaba, SUC_lexicon_tags_27 },
    { 0x3098eab8, SUC_lexicon_tags_26 },
    { 0x5af8eab7, SUC_lexicon_tags_17 },
    { 0xc658eabc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58d8eabf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe618eac1, SUC_lexicon_tags_2 },
    { 0x5d8eac2, SUC_lexicon_tags_47 },
    { 0x4b78eac2, SUC_lexicon_tags_65 },
    { 0xc518eac1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d8eac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8938eac9, SUC_lexicon_tags_2 },
    { 0xbd18eaca, SUC_lexicon_tags_37 },
    { 0x25f8eaca, SUC_lexicon_tags_216 },
    { 0x9618eaca, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c58ead2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc178ead6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf258ead8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb8eadc, SUC_lexicon_tags_29 },
    { 0x7298eadd, SUC_lexicon_tags_7 },
    { 0x3918eade, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x818eae0, SUC_lexicon_tags_1 },
    { 0x99f8eae1, SUC_lexicon_tags_17 },
    { 0x25f8eae2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbff8eae4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b8eae8, SUC_lexicon_tags_20 },
    { 0x8038eae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b8eaec, SUC_lexicon_tags_38 },
    { 0x1318eaed, SUC_lexicon_tags_2 },
    { 0x5338eaee, SUC_lexicon_tags_43 },
    { 0xb0f8eaef, SUC_lexicon_tags_26 },
    { 0x1278eaee, SUC_lexicon_tags_14 },
    { 0x4db8eaef, SUC_lexicon_tags_17 },
    { 0xc8d8eaf2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f8eaf6, SUC_lexicon_tags_1665 },
    { 0, NULL },
    { 0, NULL },
    { 0x6178eaf9, SUC_lexicon_tags_1 },
    { 0x4038eafa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a78eafe, SUC_lexicon_tags_25 },
    { 0x1338eafe, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b8eb05, SUC_lexicon_tags_25 },
    { 0xda38eb06, SUC_lexicon_tags_8 },
    { 0xb298eb06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d58eb11, SUC_lexicon_tags_23 },
    { 0xe938eb11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d8eb19, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x3d18eb1b, SUC_lexicon_tags_19 },
    { 0x9998eb1c, SUC_lexicon_tags_37 },
    { 0x4f58eb1b, SUC_lexicon_tags_11 },
    { 0x3cb8eb1e, SUC_lexicon_tags_8 },
    { 0x8638eb1f, SUC_lexicon_tags_11 },
    { 0x4e38eb20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f8eb25, SUC_lexicon_tags_48 },
    { 0x9238eb26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f8eb2b, SUC_lexicon_tags_15 },
    { 0x9ad8eb2c, SUC_lexicon_tags_8 },
    { 0xb0d8eb2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50b8eb2f, SUC_lexicon_tags_33 },
    { 0xac58eb30, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x80d8eb33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe878eb35, SUC_lexicon_tags_23 },
    { 0x3ab8eb35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c98eb3c, SUC_lexicon_tags_37 },
    { 0xbad8eb3d, SUC_lexicon_tags_33 },
    { 0x3558eb3c, SUC_lexicon_tags_12 },
    { 0x3578eb3c, SUC_lexicon_tags_12 },
    { 0x9098eb3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9258eb42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7738eb48, SUC_lexicon_tags_15 },
    { 0xd698eb49, SUC_lexicon_tags_8 },
    { 0x33f8eb4a, SUC_lexicon_tags_29 },
    { 0x5998eb48, SUC_lexicon_tags_31 },
    { 0xe638eb49, SUC_lexicon_tags_25 },
    { 0xf3d8eb4d, SUC_lexicon_tags_10 },
    { 0x4838eb4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd018eb50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb818eb56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79b8eb58, SUC_lexicon_tags_14 },
    { 0xfc98eb58, SUC_lexicon_tags_15 },
    { 0x3058eb58, SUC_lexicon_tags_37 },
    { 0x87b8eb59, SUC_lexicon_tags_19 },
    { 0x30f8eb5c, SUC_lexicon_tags_0 },
    { 0x8838eb5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf8eb61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d8eb65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f8eb70, SUC_lexicon_tags_2 },
    { 0x1a38eb70, SUC_lexicon_tags_12 },
    { 0x4878eb72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f18eb74, SUC_lexicon_tags_23 },
    { 0xa038eb75, SUC_lexicon_tags_37 },
    { 0xb438eb76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee38eb7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf538eb80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7838eb84, SUC_lexicon_tags_31 },
    { 0x7ff8eb85, SUC_lexicon_tags_35 },
    { 0xaaf8eb86, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d8eb89, SUC_lexicon_tags_12 },
    { 0x8fd8eb8a, SUC_lexicon_tags_25 },
    { 0xdf98eb8b, SUC_lexicon_tags_3 },
    { 0x5d18eb8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2198eb8f, SUC_lexicon_tags_17 },
    { 0xc78eb90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d78eb92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda58eb94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9878eb9a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd158eb9c, SUC_lexicon_tags_784 },
    { 0x7378eb9d, SUC_lexicon_tags_23 },
    { 0xab18eb9e, SUC_lexicon_tags_47 },
    { 0xe3f8eb9f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7378eba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc858eba5, SUC_lexicon_tags_38 },
    { 0x61f8eba6, SUC_lexicon_tags_1 },
    { 0xc38eba7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea98ebad, SUC_lexicon_tags_21 },
    { 0xabd8ebae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1818ebb1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5258ebb7, SUC_lexicon_tags_7 },
    { 0x85f8ebb7, SUC_lexicon_tags_19 },
    { 0xb258ebb8, SUC_lexicon_tags_13 },
    { 0x6f78ebba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x72b8ebbc, SUC_lexicon_tags_15 },
    { 0x35b8ebbd, SUC_lexicon_tags_60 },
    { 0xe898ebbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf58ebc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4458ebc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d8ebcd, SUC_lexicon_tags_11 },
    { 0xe2b8ebce, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd8ebd3, SUC_lexicon_tags_18 },
    { 0x9c18ebd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6238ebda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8798ebdc, SUC_lexicon_tags_17 },
    { 0x7878ebdd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88b8ebdf, SUC_lexicon_tags_17 },
    { 0x41b8ebe0, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f8ebf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af8ebf7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea98ebfb, SUC_lexicon_tags_38 },
    { 0x62b8ebfc, SUC_lexicon_tags_2 },
    { 0xda38ebfd, SUC_lexicon_tags_33 },
    { 0xa858ebfe, SUC_lexicon_tags_9 },
    { 0x2058ebfe, SUC_lexicon_tags_87 },
    { 0x98b8ec00, SUC_lexicon_tags_15 },
    { 0xf5b8ec01, SUC_lexicon_tags_60 },
    { 0x2538ebfb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe338ec05, SUC_lexicon_tags_43 },
    { 0xf078ec06, SUC_lexicon_tags_84 },
    { 0xf118ec07, SUC_lexicon_tags_37 },
    { 0xd0f8ec08, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c98ec0f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xff8ec11, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf8ec15, SUC_lexicon_tags_6 },
    { 0xfff8ec16, SUC_lexicon_tags_1 },
    { 0x9398ec16, SUC_lexicon_tags_12 },
    { 0x1178ec18, SUC_lexicon_tags_87 },
    { 0xfaf8ec19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x22f8ec1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d58ec20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b8ec23, SUC_lexicon_tags_47 },
    { 0xf18ec23, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b8ec27, SUC_lexicon_tags_47 },
    { 0x84d8ec28, SUC_lexicon_tags_6 },
    { 0x47b8ec29, SUC_lexicon_tags_63 },
    { 0xcb78ec2a, SUC_lexicon_tags_55 },
    { 0x1a18ec2b, SUC_lexicon_tags_17 },
    { 0xf518ec2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b8ec30, SUC_lexicon_tags_25 },
    { 0xc998ec31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a98ec33, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa918ec36, SUC_lexicon_tags_0 },
    { 0x8f8ec37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea58ec3c, SUC_lexicon_tags_60 },
    { 0x1858ec3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xef78ec3f, SUC_lexicon_tags_0 },
    { 0x1238ec40, SUC_lexicon_tags_10 },
    { 0x5cb8ec41, SUC_lexicon_tags_8 },
    { 0xa8b8ec42, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1618ec46, SUC_lexicon_tags_8 },
    { 0xbc38ec47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb8ec4b, SUC_lexicon_tags_39 },
    { 0xf638ec4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11f8ec4e, SUC_lexicon_tags_11 },
    { 0x68d8ec4f, SUC_lexicon_tags_31 },
    { 0xae18ec50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad8ec55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe38ec58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f8ec5b, SUC_lexicon_tags_46 },
    { 0x38d8ec5c, SUC_lexicon_tags_12 },
    { 0xa838ec5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a18ec5f, SUC_lexicon_tags_1 },
    { 0xf5d8ec60, SUC_lexicon_tags_17 },
    { 0x1338ec61, SUC_lexicon_tags_60 },
    { 0x40f8ec61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d8ec68, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfc98ec6a, SUC_lexicon_tags_26 },
    { 0x8c98ec6b, SUC_lexicon_tags_17 },
    { 0x2998ec6c, SUC_lexicon_tags_37 },
    { 0xe118ec6d, SUC_lexicon_tags_57 },
    { 0x51f8ec6e, SUC_lexicon_tags_55 },
    { 0x9e58ec6f, SUC_lexicon_tags_15 },
    { 0x9ab8ec70, SUC_lexicon_tags_63 },
    { 0xa3d8ec6e, SUC_lexicon_tags_10 },
    { 0xe78ec72, SUC_lexicon_tags_37 },
    { 0xa478ec73, SUC_lexicon_tags_151 },
    { 0xd658ec74, SUC_lexicon_tags_104 },
    { 0x7af8ec72, SUC_lexicon_tags_29 },
    { 0x2bb8ec76, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf8ec7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8ec7e, SUC_lexicon_tags_33 },
    { 0xbd58ec7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd18ec81, SUC_lexicon_tags_17 },
    { 0x6b58ec81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb98ec87, SUC_lexicon_tags_25 },
    { 0xe618ec87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x578ec8b, SUC_lexicon_tags_15 },
    { 0x24b8ec8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f98ec8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xceb8ec91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x518ec93, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5ad8ec95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa78ec9c, SUC_lexicon_tags_37 },
    { 0x2558ec9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f78ec9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf058eca4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9d8eca6, SUC_lexicon_tags_9 },
    { 0xfad8eca6, SUC_lexicon_tags_17 },
    { 0xaa18eca6, SUC_lexicon_tags_17 },
    { 0x9ff8eca9, SUC_lexicon_tags_25 },
    { 0x36b8ecaa, SUC_lexicon_tags_47 },
    { 0x1e38eca9, SUC_lexicon_tags_37 },
    { 0x9538ecac, SUC_lexicon_tags_25 },
    { 0x2bf8eca8, SUC_lexicon_tags_15 },
    { 0x8978ecae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed8ecb1, SUC_lexicon_tags_38 },
    { 0x4618ecb2, SUC_lexicon_tags_37 },
    { 0xd7d8ecb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf058ecb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf658ecb9, SUC_lexicon_tags_17 },
    { 0x4f8ecb9, SUC_lexicon_tags_17 },
    { 0xfc98ecba, SUC_lexicon_tags_12 },
    { 0xabf8ecba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf318ecbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e38ecc3, SUC_lexicon_tags_23 },
    { 0x31f8ecc4, SUC_lexicon_tags_34 },
    { 0x46b8ecc3, SUC_lexicon_tags_0 },
    { 0xb3f8ecc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1898ecca, SUC_lexicon_tags_58 },
    { 0x6698eccb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6998eccd, SUC_lexicon_tags_2 },
    { 0xe3b8eccd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb898ecd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6518ecd3, SUC_lexicon_tags_63 },
    { 0x5d8ecd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf8ecd7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f8ecde, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa18ece2, SUC_lexicon_tags_1 },
    { 0xedf8ece3, SUC_lexicon_tags_11 },
    { 0xa078ece4, SUC_lexicon_tags_8 },
    { 0x9d78ece4, SUC_lexicon_tags_17 },
    { 0x2938ece6, SUC_lexicon_tags_45 },
    { 0x2ff8ece3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab8eceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe538ecf0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfd38ecf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8858ecf7, SUC_lexicon_tags_65 },
    { 0x2758ecf8, SUC_lexicon_tags_25 },
    { 0x82b8ecf9, SUC_lexicon_tags_14 },
    { 0x7a98ecf9, SUC_lexicon_tags_15 },
    { 0xdaf8ecf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d8ed01, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e18ed04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb438ed06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfa98ed08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6f8ed14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d8ed1c, SUC_lexicon_tags_19 },
    { 0x5bd8ed1d, SUC_lexicon_tags_2 },
    { 0xeb18ed1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cd8ed20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db8ed25, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5578ed28, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc318ed2d, SUC_lexicon_tags_11 },
    { 0x3af8ed2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1858ed33, SUC_lexicon_tags_7 },
    { 0x58d8ed34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7138ed36, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd858ed39, SUC_lexicon_tags_11 },
    { 0x7c78ed3a, SUC_lexicon_tags_10 },
    { 0x9ab8ed3b, SUC_lexicon_tags_37 },
    { 0x3758ed3b, SUC_lexicon_tags_64 },
    { 0xee18ed3d, SUC_lexicon_tags_30 },
    { 0xd878ed3d, SUC_lexicon_tags_48 },
    { 0xf3f8ed3e, SUC_lexicon_tags_13 },
    { 0x49d8ed40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa438ed42, SUC_lexicon_tags_37 },
    { 0x3378ed43, SUC_lexicon_tags_78 },
    { 0x3f98ed44, SUC_lexicon_tags_12 },
    { 0x3358ed42, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbd78ed47, SUC_lexicon_tags_17 },
    { 0x90f8ed47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x158ed4c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7bd8ed4e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6518ed50, SUC_lexicon_tags_64 },
    { 0xf658ed50, SUC_lexicon_tags_25 },
    { 0x3e78ed52, SUC_lexicon_tags_11 },
    { 0xd418ed53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8d8ed55, SUC_lexicon_tags_0 },
    { 0xb3b8ed56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5178ed58, SUC_lexicon_tags_17 },
    { 0xf5d8ed59, SUC_lexicon_tags_11 },
    { 0xebd8ed5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb8ed5d, SUC_lexicon_tags_103 },
    { 0x59f8ed5e, SUC_lexicon_tags_9 },
    { 0xb3f8ed5d, SUC_lexicon_tags_79 },
    { 0xbad8ed60, SUC_lexicon_tags_40 },
    { 0xe918ed60, SUC_lexicon_tags_25 },
    { 0xe338ed5e, SUC_lexicon_tags_11 },
    { 0x2178ed63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b8ed66, SUC_lexicon_tags_68 },
    { 0xcd18ed67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa98ed6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f8ed73, SUC_lexicon_tags_9 },
    { 0x698ed74, SUC_lexicon_tags_39 },
    { 0xe738ed73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d8ed7e, SUC_lexicon_tags_1 },
    { 0xbf38ed7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd38ed84, SUC_lexicon_tags_138 },
    { 0x4158ed85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b8ed88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66b8ed8a, SUC_lexicon_tags_7 },
    { 0xfcb8ed8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d98ed8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1478ed92, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xda18ed94, SUC_lexicon_tags_0 },
    { 0xe198ed95, SUC_lexicon_tags_63 },
    { 0xe1b8ed96, SUC_lexicon_tags_43 },
    { 0x83b8ed96, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb78ed9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99b8ed9c, SUC_lexicon_tags_39 },
    { 0x8498ed9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1438eda6, SUC_lexicon_tags_108 },
    { 0x8198eda7, SUC_lexicon_tags_27 },
    { 0x97d8eda6, SUC_lexicon_tags_17 },
    { 0x5378eda9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6618edab, SUC_lexicon_tags_17 },
    { 0x2978edab, SUC_lexicon_tags_65 },
    { 0xcb58edac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa78edb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd18edb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3718edb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8238edba, SUC_lexicon_tags_23 },
    { 0x1df8edbb, SUC_lexicon_tags_64 },
    { 0x14d8edbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa638edbf, SUC_lexicon_tags_25 },
    { 0x938edc0, SUC_lexicon_tags_31 },
    { 0xfdf8edbf, SUC_lexicon_tags_17 },
    { 0xe5f8edc2, SUC_lexicon_tags_45 },
    { 0xb98edc3, SUC_lexicon_tags_25 },
    { 0x9678edc4, SUC_lexicon_tags_104 },
    { 0x48f8edc4, SUC_lexicon_tags_1 },
    { 0x2378edbf, SUC_lexicon_tags_11 },
    { 0x1038edc7, SUC_lexicon_tags_47 },
    { 0x33f8edc8, SUC_lexicon_tags_65 },
    { 0x36d8edc9, SUC_lexicon_tags_4 },
    { 0x6918edc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73f8edcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd78edce, SUC_lexicon_tags_7 },
    { 0x2358edcf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x47b8edd1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd8edd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe698edd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc338edda, SUC_lexicon_tags_681 },
    { 0xf258eddb, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a18eddf, SUC_lexicon_tags_1 },
    { 0xe478eddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fb8ede2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f58ede4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc398ede7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b8edeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9978edee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2218edf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9618edf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f8edfd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe598ee01, SUC_lexicon_tags_12 },
    { 0x438ee02, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d8ee05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf8ee0a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9278ee0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff8ee15, SUC_lexicon_tags_6 },
    { 0x4358ee15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c98ee1b, SUC_lexicon_tags_46 },
    { 0x1ff8ee1c, SUC_lexicon_tags_10 },
    { 0xafd8ee1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b78ee26, SUC_lexicon_tags_11 },
    { 0x38f8ee27, SUC_lexicon_tags_37 },
    { 0x4238ee26, SUC_lexicon_tags_60 },
    { 0xa698ee29, SUC_lexicon_tags_37 },
    { 0x2eb8ee29, SUC_lexicon_tags_37 },
    { 0xb6f8ee29, SUC_lexicon_tags_37 },
    { 0xbdd8ee29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe258ee2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d8ee3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1778ee3d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf498ee3f, SUC_lexicon_tags_15 },
    { 0xd7f8ee40, SUC_lexicon_tags_2 },
    { 0xb618ee40, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4678ee45, SUC_lexicon_tags_27 },
    { 0xd9f8ee45, SUC_lexicon_tags_12 },
    { 0x958ee46, SUC_lexicon_tags_25 },
    { 0x5cf8ee48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed8ee4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2358ee4e, SUC_lexicon_tags_29 },
    { 0xd878ee4e, SUC_lexicon_tags_10 },
    { 0xad8ee4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3638ee52, SUC_lexicon_tags_32 },
    { 0xf298ee53, SUC_lexicon_tags_37 },
    { 0xfe78ee54, SUC_lexicon_tags_4 },
    { 0x6938ee55, SUC_lexicon_tags_295 },
    { 0xa18ee56, SUC_lexicon_tags_44 },
    { 0x4158ee57, SUC_lexicon_tags_0 },
    { 0x2f98ee56, SUC_lexicon_tags_26 },
    { 0x91f8ee58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf78ee5e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7798ee62, SUC_lexicon_tags_28 },
    { 0x6ff8ee63, SUC_lexicon_tags_56 },
    { 0x8718ee64, SUC_lexicon_tags_2 },
    { 0x9138ee65, SUC_lexicon_tags_45 },
    { 0xb718ee63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ff8ee68, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb98ee6f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a58ee74, SUC_lexicon_tags_25 },
    { 0x12d8ee75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d18ee7c, SUC_lexicon_tags_13 },
    { 0xc118ee7d, SUC_lexicon_tags_14 },
    { 0xa418ee7d, SUC_lexicon_tags_63 },
    { 0x2e98ee7e, SUC_lexicon_tags_37 },
    { 0xe318ee80, SUC_lexicon_tags_63 },
    { 0xcd78ee7f, SUC_lexicon_tags_29 },
    { 0x69b8ee82, SUC_lexicon_tags_0 },
    { 0x8198ee82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d8ee88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf058ee8b, SUC_lexicon_tags_63 },
    { 0x7278ee8c, SUC_lexicon_tags_19 },
    { 0xaf58ee8c, SUC_lexicon_tags_15 },
    { 0xcb98ee8e, SUC_lexicon_tags_25 },
    { 0x89f8ee8f, SUC_lexicon_tags_31 },
    { 0xcbf8ee90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a98ee95, SUC_lexicon_tags_61 },
    { 0xc4f8ee96, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd078ee9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9998ee9d, SUC_lexicon_tags_1 },
    { 0x22b8ee9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5758eead, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe1f8eeaf, SUC_lexicon_tags_12 },
    { 0xf38eeb0, SUC_lexicon_tags_49 },
    { 0x578eeb1, SUC_lexicon_tags_11 },
    { 0x3fd8eeb1, SUC_lexicon_tags_2 },
    { 0xc498eeb3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb9b8eeb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1818eebb, SUC_lexicon_tags_63 },
    { 0xbbd8eebb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4698eec1, SUC_lexicon_tags_19 },
    { 0xde18eec2, SUC_lexicon_tags_1 },
    { 0x6918eec2, SUC_lexicon_tags_619 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb118eec7, SUC_lexicon_tags_8 },
    { 0x5378eec7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d8eece, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc498eed7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4558eeda, SUC_lexicon_tags_25 },
    { 0x96d8eedb, SUC_lexicon_tags_17 },
    { 0x43d8eedc, SUC_lexicon_tags_33 },
    { 0x2a18eedc, SUC_lexicon_tags_2 },
    { 0x94f8eede, SUC_lexicon_tags_25 },
    { 0x29b8eedf, SUC_lexicon_tags_15 },
    { 0x82b8eedc, SUC_lexicon_tags_11 },
    { 0x5178eee1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f8eee4, SUC_lexicon_tags_37 },
    { 0x7978eee5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf8eeeb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3418eeed, SUC_lexicon_tags_9 },
    { 0xf538eeee, SUC_lexicon_tags_16 },
    { 0x23b8eeef, SUC_lexicon_tags_34 },
    { 0x9538eeef, SUC_lexicon_tags_47 },
    { 0xc9f8eef0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf98eef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e78eef8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa778ef00, SUC_lexicon_tags_37 },
    { 0xcfb8ef00, SUC_lexicon_tags_0 },
    { 0xfc58ef01, SUC_lexicon_tags_12 },
    { 0x86f8ef00, SUC_lexicon_tags_49 },
    { 0x2178ef04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df8ef0c, SUC_lexicon_tags_25 },
    { 0xdcf8ef0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5ed8ef0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d8ef17, SUC_lexicon_tags_63 },
    { 0x7178ef17, SUC_lexicon_tags_17 },
    { 0xcfd8ef19, SUC_lexicon_tags_60 },
    { 0xb118ef18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4378ef1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c58ef1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b78ef23, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3f38ef25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c38ef27, SUC_lexicon_tags_48 },
    { 0xdc18ef27, SUC_lexicon_tags_12 },
    { 0x6458ef29, SUC_lexicon_tags_26 },
    { 0x16d8ef2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2678ef2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4298ef34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca98ef39, SUC_lexicon_tags_12 },
    { 0x7bd8ef3a, SUC_lexicon_tags_26 },
    { 0xb858ef3b, SUC_lexicon_tags_29 },
    { 0x318ef3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa518ef3e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3878ef41, SUC_lexicon_tags_25 },
    { 0xbb58ef41, SUC_lexicon_tags_6 },
    { 0xfc38ef43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa6f8ef45, SUC_lexicon_tags_11 },
    { 0x28f8ef45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76b8ef48, SUC_lexicon_tags_25 },
    { 0x3798ef49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed18ef4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd18ef54, SUC_lexicon_tags_25 },
    { 0x4938ef55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9858ef58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x698ef5b, SUC_lexicon_tags_43 },
    { 0x50b8ef5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9318ef61, SUC_lexicon_tags_12 },
    { 0x11f8ef62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7b38ef64, SUC_lexicon_tags_98 },
    { 0x3398ef65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd38ef68, SUC_lexicon_tags_37 },
    { 0xddb8ef69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d8ef6f, SUC_lexicon_tags_43 },
    { 0x2238ef70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9638ef72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc658ef7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5158ef7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe818ef80, SUC_lexicon_tags_1 },
    { 0x3798ef80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f8ef84, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2198ef8b, SUC_lexicon_tags_7 },
    { 0xe5d8ef8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b58ef8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d18ef90, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f38ef93, SUC_lexicon_tags_60 },
    { 0x5cb8ef94, SUC_lexicon_tags_25 },
    { 0x5d78ef94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef78ef97, SUC_lexicon_tags_47 },
    { 0xf758ef98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb8ef9c, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c78ef9f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf98efa1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd8efa7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a18efa9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x118efac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x458efae, SUC_lexicon_tags_15 },
    { 0x6c38efae, SUC_lexicon_tags_30 },
    { 0x7c18efb0, SUC_lexicon_tags_23 },
    { 0x3978efb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa098efb4, SUC_lexicon_tags_6 },
    { 0xf1f8efb5, SUC_lexicon_tags_11 },
    { 0xe078efb6, SUC_lexicon_tags_87 },
    { 0xd498efb7, SUC_lexicon_tags_11 },
    { 0xf2d8efb8, SUC_lexicon_tags_7 },
    { 0xb318efb5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c38efbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d78efc4, SUC_lexicon_tags_1 },
    { 0x7158efc4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6618efc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97b8efcb, SUC_lexicon_tags_65 },
    { 0xa0f8efcb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4418efce, SUC_lexicon_tags_79 },
    { 0x55b8efce, SUC_lexicon_tags_37 },
    { 0x3b18efce, SUC_lexicon_tags_11 },
    { 0xcb78efd0, SUC_lexicon_tags_47 },
    { 0xeb8efd2, SUC_lexicon_tags_26 },
    { 0xdf18efd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed8efd8, SUC_lexicon_tags_7 },
    { 0x2f58efd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df8efdf, SUC_lexicon_tags_11 },
    { 0x90b8efe0, SUC_lexicon_tags_43 },
    { 0x1058efe1, SUC_lexicon_tags_37 },
    { 0xc5d8efe2, SUC_lexicon_tags_55 },
    { 0x4458efe0, SUC_lexicon_tags_0 },
    { 0x6fd8efe2, SUC_lexicon_tags_26 },
    { 0xe1b8efe5, SUC_lexicon_tags_23 },
    { 0x20b8efe6, SUC_lexicon_tags_48 },
    { 0x6838efe1, SUC_lexicon_tags_15 },
    { 0x2cb8efe8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7138efea, SUC_lexicon_tags_31 },
    { 0x8698efea, SUC_lexicon_tags_12 },
    { 0xd2f8efec, SUC_lexicon_tags_47 },
    { 0x7318efed, SUC_lexicon_tags_26 },
    { 0xa558efee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7838eff2, SUC_lexicon_tags_15 },
    { 0x7e58eff2, SUC_lexicon_tags_37 },
    { 0x2c38eff3, SUC_lexicon_tags_19 },
    { 0x5798eff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7978eff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e58eff9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4d58effb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf58effd, SUC_lexicon_tags_38 },
    { 0xa778effe, SUC_lexicon_tags_26 },
    { 0x33b8effe, SUC_lexicon_tags_26 },
    { 0x5738effe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4d8f002, SUC_lexicon_tags_10 },
    { 0xa818f002, SUC_lexicon_tags_15 },
    { 0xead8f004, SUC_lexicon_tags_43 },
    { 0xfdb8f005, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd078f00a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbbd8f00c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf8f00f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ed8f011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2958f014, SUC_lexicon_tags_2 },
    { 0x8078f015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd18f017, SUC_lexicon_tags_108 },
    { 0x54d8f018, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc38f01c, SUC_lexicon_tags_17 },
    { 0xb8f01d, SUC_lexicon_tags_12 },
    { 0x52b8f01d, SUC_lexicon_tags_12 },
    { 0xd2b8f01c, SUC_lexicon_tags_11 },
    { 0x4998f020, SUC_lexicon_tags_11 },
    { 0x818f020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7558f023, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1018f025, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53f8f027, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8978f029, SUC_lexicon_tags_55 },
    { 0x2238f029, SUC_lexicon_tags_11 },
    { 0xdf98f02b, SUC_lexicon_tags_15 },
    { 0xaa18f02c, SUC_lexicon_tags_49 },
    { 0x6b8f02c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3758f030, SUC_lexicon_tags_17 },
    { 0x1c78f030, SUC_lexicon_tags_7 },
    { 0xf518f032, SUC_lexicon_tags_7 },
    { 0x1a58f031, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a18f035, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b8f03b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed8f041, SUC_lexicon_tags_15 },
    { 0x28d8f042, SUC_lexicon_tags_11 },
    { 0xb978f041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83d8f045, SUC_lexicon_tags_6 },
    { 0xb5b8f046, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79f8f048, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab8f04c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e98f05b, SUC_lexicon_tags_1 },
    { 0xbf58f05b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd778f060, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff8f066, SUC_lexicon_tags_7 },
    { 0x4bf8f066, SUC_lexicon_tags_56 },
    { 0xbfd8f066, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xac18f06a, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd8f06e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e18f070, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db8f073, SUC_lexicon_tags_25 },
    { 0x8c78f074, SUC_lexicon_tags_11 },
    { 0x3718f075, SUC_lexicon_tags_28 },
    { 0xf9f8f076, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf238f07a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f07d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab8f083, SUC_lexicon_tags_32 },
    { 0xd158f083, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d8f089, SUC_lexicon_tags_47 },
    { 0x8778f089, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf138f08d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa718f093, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f58f09b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f8f0a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee58f0a9, SUC_lexicon_tags_28 },
    { 0x9d78f0aa, SUC_lexicon_tags_15 },
    { 0x7518f0ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1358f0ae, SUC_lexicon_tags_26 },
    { 0x7b8f0ae, SUC_lexicon_tags_35 },
    { 0xae78f0b0, SUC_lexicon_tags_32 },
    { 0xca38f0b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf98f0b5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab8f0ba, SUC_lexicon_tags_17 },
    { 0xcf58f0ba, SUC_lexicon_tags_7 },
    { 0x578f0bc, SUC_lexicon_tags_17 },
    { 0xf158f0ba, SUC_lexicon_tags_31 },
    { 0xbc78f0ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4418f0c1, SUC_lexicon_tags_12 },
    { 0xb958f0c1, SUC_lexicon_tags_47 },
    { 0x9758f0c3, SUC_lexicon_tags_26 },
    { 0x52f8f0c4, SUC_lexicon_tags_12 },
    { 0x1458f0c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4918f0cc, SUC_lexicon_tags_7 },
    { 0x1198f0cd, SUC_lexicon_tags_39 },
    { 0x20d8f0ce, SUC_lexicon_tags_47 },
    { 0x3778f0cc, SUC_lexicon_tags_35 },
    { 0x4bb8f0cc, SUC_lexicon_tags_12 },
    { 0xe7f8f0cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa98f0d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b8f0d7, SUC_lexicon_tags_35 },
    { 0x3798f0d8, SUC_lexicon_tags_1 },
    { 0x1b38f0d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6678f0dc, SUC_lexicon_tags_37 },
    { 0x1358f0dc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbd8f0df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa298f0e5, SUC_lexicon_tags_84 },
    { 0x63b8f0e6, SUC_lexicon_tags_14 },
    { 0x59b8f0e6, SUC_lexicon_tags_47 },
    { 0x8dd8f0e8, SUC_lexicon_tags_17 },
    { 0x1ab8f0e6, SUC_lexicon_tags_11 },
    { 0xd5d8f0e8, SUC_lexicon_tags_12 },
    { 0x9b38f0e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f8f0f3, SUC_lexicon_tags_4 },
    { 0x5c58f0f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa6d8f0f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b8f0f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf738f0fb, SUC_lexicon_tags_39 },
    { 0x69b8f0fc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6978f101, SUC_lexicon_tags_9 },
    { 0x88b8f101, SUC_lexicon_tags_17 },
    { 0x9ef8f103, SUC_lexicon_tags_39 },
    { 0x84f8f103, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d8f107, SUC_lexicon_tags_14 },
    { 0x40f8f108, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4bd8f10a, SUC_lexicon_tags_24 },
    { 0xd7d8f10a, SUC_lexicon_tags_37 },
    { 0xad78f10a, SUC_lexicon_tags_25 },
    { 0xfad8f10d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7d8f10f, SUC_lexicon_tags_2 },
    { 0xa278f10f, SUC_lexicon_tags_59 },
    { 0x7698f111, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb8f114, SUC_lexicon_tags_1 },
    { 0x9ad8f115, SUC_lexicon_tags_2 },
    { 0xbc78f116, SUC_lexicon_tags_46 },
    { 0x53d8f117, SUC_lexicon_tags_6 },
    { 0x8d78f118, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb598f11c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5198f121, SUC_lexicon_tags_60 },
    { 0x2498f122, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5198f125, SUC_lexicon_tags_64 },
    { 0x7358f125, SUC_lexicon_tags_14 },
    { 0x75b8f127, SUC_lexicon_tags_7 },
    { 0x7178f128, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4f8f12a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7af8f12d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7598f12f, SUC_lexicon_tags_25 },
    { 0x8498f130, SUC_lexicon_tags_10 },
    { 0x95b8f130, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2978f135, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b8f138, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa198f13a, SUC_lexicon_tags_21 },
    { 0x1ed8f13b, SUC_lexicon_tags_68 },
    { 0x8978f13c, SUC_lexicon_tags_6 },
    { 0x4058f13d, SUC_lexicon_tags_315 },
    { 0x8a78f13e, SUC_lexicon_tags_11 },
    { 0xc458f13d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9698f141, SUC_lexicon_tags_15 },
    { 0x24d8f141, SUC_lexicon_tags_45 },
    { 0x178f142, SUC_lexicon_tags_7 },
    { 0xdb98f144, SUC_lexicon_tags_38 },
    { 0xe778f144, SUC_lexicon_tags_68 },
    { 0x5518f146, SUC_lexicon_tags_38 },
    { 0xff78f147, SUC_lexicon_tags_0 },
    { 0x7ad8f142, SUC_lexicon_tags_63 },
    { 0xca78f146, SUC_lexicon_tags_57 },
    { 0x8e98f14a, SUC_lexicon_tags_12 },
    { 0xc6d8f14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x47f8f14d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa58f14f, SUC_lexicon_tags_27 },
    { 0xee38f150, SUC_lexicon_tags_37 },
    { 0xf698f151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9018f154, SUC_lexicon_tags_63 },
    { 0x3c58f154, SUC_lexicon_tags_17 },
    { 0x24b8f156, SUC_lexicon_tags_14 },
    { 0xab78f157, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf458f159, SUC_lexicon_tags_31 },
    { 0x8af8f159, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a18f15c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b58f161, SUC_lexicon_tags_57 },
    { 0x9538f162, SUC_lexicon_tags_38 },
    { 0x2db8f162, SUC_lexicon_tags_19 },
    { 0x2ad8f164, SUC_lexicon_tags_63 },
    { 0x9918f163, SUC_lexicon_tags_23 },
    { 0xd5d8f166, SUC_lexicon_tags_19 },
    { 0x5658f167, SUC_lexicon_tags_95 },
    { 0xb38f168, SUC_lexicon_tags_7 },
    { 0xdc38f169, SUC_lexicon_tags_2 },
    { 0x78f16a, SUC_lexicon_tags_23 },
    { 0xf698f16b, SUC_lexicon_tags_17 },
    { 0x78f16a, SUC_lexicon_tags_11 },
    { 0x2038f164, SUC_lexicon_tags_17 },
    { 0x2618f169, SUC_lexicon_tags_45 },
    { 0x8638f16f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b8f173, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5ff8f175, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc358f17d, SUC_lexicon_tags_1 },
    { 0xad78f17d, SUC_lexicon_tags_23 },
    { 0x2c18f17d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe718f182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6198f184, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc38f18b, SUC_lexicon_tags_2 },
    { 0xd478f18b, SUC_lexicon_tags_29 },
    { 0x2b18f18d, SUC_lexicon_tags_38 },
    { 0xea58f18d, SUC_lexicon_tags_28 },
    { 0x3078f18f, SUC_lexicon_tags_14 },
    { 0x45b8f190, SUC_lexicon_tags_65 },
    { 0xfad8f191, SUC_lexicon_tags_63 },
    { 0x5cd8f18f, SUC_lexicon_tags_2 },
    { 0x1918f191, SUC_lexicon_tags_44 },
    { 0x4298f194, SUC_lexicon_tags_23 },
    { 0x13d8f194, SUC_lexicon_tags_7 },
    { 0x1b38f196, SUC_lexicon_tags_17 },
    { 0xc5f8f192, SUC_lexicon_tags_25 },
    { 0x3bb8f193, SUC_lexicon_tags_38 },
    { 0xb858f18e, SUC_lexicon_tags_5 },
    { 0xc858f193, SUC_lexicon_tags_26 },
    { 0xaaf8f195, SUC_lexicon_tags_21 },
    { 0x4778f197, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2758f1a6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf8f1aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5fb8f1ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e78f1b1, SUC_lexicon_tags_31 },
    { 0x15f8f1b2, SUC_lexicon_tags_11 },
    { 0x6458f1b1, SUC_lexicon_tags_26 },
    { 0xa358f1b4, SUC_lexicon_tags_63 },
    { 0xd098f1b5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc98f1b8, SUC_lexicon_tags_17 },
    { 0x3f98f1b9, SUC_lexicon_tags_16 },
    { 0x3058f1b9, SUC_lexicon_tags_33 },
    { 0x41f8f1b9, SUC_lexicon_tags_45 },
    { 0x64d8f1ba, SUC_lexicon_tags_21 },
    { 0x1e98f1bb, SUC_lexicon_tags_43 },
    { 0xb978f1bc, SUC_lexicon_tags_3 },
    { 0xb358f1bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ef8f1c1, SUC_lexicon_tags_31 },
    { 0x5b78f1c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf218f1c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a18f1c8, SUC_lexicon_tags_7 },
    { 0x3858f1c8, SUC_lexicon_tags_45 },
    { 0x73b8f1c8, SUC_lexicon_tags_25 },
    { 0x60f8f1cb, SUC_lexicon_tags_17 },
    { 0x8c78f1c9, SUC_lexicon_tags_1 },
    { 0xad38f1cd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x19f8f1cf, SUC_lexicon_tags_37 },
    { 0xd858f1cf, SUC_lexicon_tags_14 },
    { 0xee78f1d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb218f1df, SUC_lexicon_tags_49 },
    { 0x6358f1e0, SUC_lexicon_tags_7 },
    { 0xbe18f1df, SUC_lexicon_tags_1 },
    { 0xa838f1e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f8f1e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3738f1ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4178f1f2, SUC_lexicon_tags_11 },
    { 0xa38f1f3, SUC_lexicon_tags_25 },
    { 0x8c78f1f4, SUC_lexicon_tags_37 },
    { 0x4db8f1f4, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xc998f1f7, SUC_lexicon_tags_26 },
    { 0x9978f1f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8098f1fb, SUC_lexicon_tags_25 },
    { 0x1798f1fc, SUC_lexicon_tags_12 },
    { 0x93d8f1fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa578f205, SUC_lexicon_tags_15 },
    { 0xc538f205, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x47b8f208, SUC_lexicon_tags_37 },
    { 0x6338f209, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd78f20d, SUC_lexicon_tags_37 },
    { 0x88f8f20e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1198f210, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db8f213, SUC_lexicon_tags_2 },
    { 0x6ed8f213, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38f216, SUC_lexicon_tags_37 },
    { 0x55d8f216, SUC_lexicon_tags_17 },
    { 0x4a58f218, SUC_lexicon_tags_0 },
    { 0xa738f219, SUC_lexicon_tags_0 },
    { 0x29b8f21a, SUC_lexicon_tags_27 },
    { 0x89b8f21a, SUC_lexicon_tags_46 },
    { 0x1e58f21b, SUC_lexicon_tags_9 },
    { 0xe38f217, SUC_lexicon_tags_28 },
    { 0x67d8f21c, SUC_lexicon_tags_11 },
    { 0x41d8f21f, SUC_lexicon_tags_7 },
    { 0x9618f220, SUC_lexicon_tags_12 },
    { 0xeab8f221, SUC_lexicon_tags_0 },
    { 0xf7b8f222, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf958f226, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c98f22c, SUC_lexicon_tags_23 },
    { 0xa118f22d, SUC_lexicon_tags_37 },
    { 0xfe18f22e, SUC_lexicon_tags_11 },
    { 0xf6d8f22d, SUC_lexicon_tags_27 },
    { 0x4218f230, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcf8f232, SUC_lexicon_tags_34 },
    { 0x5a98f233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcfb8f235, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x75d8f238, SUC_lexicon_tags_38 },
    { 0x9e98f239, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc618f23c, SUC_lexicon_tags_16 },
    { 0xa58f23c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a18f23f, SUC_lexicon_tags_33 },
    { 0x4bf8f23f, SUC_lexicon_tags_37 },
    { 0x12b8f241, SUC_lexicon_tags_17 },
    { 0xda58f242, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa2b8f244, SUC_lexicon_tags_13 },
    { 0x6018f245, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x66b8f247, SUC_lexicon_tags_5 },
    { 0x63b8f247, SUC_lexicon_tags_9 },
    { 0xa9d8f249, SUC_lexicon_tags_17 },
    { 0xa858f24a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd8f24e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc6f8f250, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b8f253, SUC_lexicon_tags_11 },
    { 0x6ed8f253, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef8f257, SUC_lexicon_tags_38 },
    { 0x30b8f258, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1c58f25a, SUC_lexicon_tags_11 },
    { 0x4738f25b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd978f260, SUC_lexicon_tags_15 },
    { 0x5638f261, SUC_lexicon_tags_37 },
    { 0xf3f8f261, SUC_lexicon_tags_8 },
    { 0x84f8f260, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2038f267, SUC_lexicon_tags_11 },
    { 0xb658f267, SUC_lexicon_tags_11 },
    { 0xc738f267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d98f26b, SUC_lexicon_tags_7 },
    { 0x3bf8f26c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8f270, SUC_lexicon_tags_17 },
    { 0x85b8f271, SUC_lexicon_tags_30 },
    { 0xe958f272, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbcd8f274, SUC_lexicon_tags_11 },
    { 0xb18f274, SUC_lexicon_tags_37 },
    { 0xb4b8f276, SUC_lexicon_tags_37 },
    { 0x1258f277, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11d8f279, SUC_lexicon_tags_9 },
    { 0xd8d8f279, SUC_lexicon_tags_17 },
    { 0x4d8f27a, SUC_lexicon_tags_1 },
    { 0x6d18f27c, SUC_lexicon_tags_15 },
    { 0x8538f27d, SUC_lexicon_tags_12 },
    { 0xe598f27d, SUC_lexicon_tags_12 },
    { 0xe378f27e, SUC_lexicon_tags_2 },
    { 0x7778f27e, SUC_lexicon_tags_12 },
    { 0x7618f27c, SUC_lexicon_tags_12 },
    { 0x3818f281, SUC_lexicon_tags_19 },
    { 0x8c18f283, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbf8f285, SUC_lexicon_tags_12 },
    { 0xa0d8f286, SUC_lexicon_tags_31 },
    { 0xbf98f287, SUC_lexicon_tags_47 },
    { 0x4d38f288, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd818f28e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1e38f290, SUC_lexicon_tags_17 },
    { 0xd638f291, SUC_lexicon_tags_17 },
    { 0xd8f8f290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2058f297, SUC_lexicon_tags_11 },
    { 0x6d78f298, SUC_lexicon_tags_12 },
    { 0xbb8f298, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfcb8f29b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf8f29e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd78f2a1, SUC_lexicon_tags_37 },
    { 0xb78f2a2, SUC_lexicon_tags_25 },
    { 0x4e98f2a3, SUC_lexicon_tags_4 },
    { 0x1738f2a1, SUC_lexicon_tags_43 },
    { 0x198f2a5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa1b8f2a7, SUC_lexicon_tags_27 },
    { 0x5758f2a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef58f2ae, SUC_lexicon_tags_12 },
    { 0x938f2af, SUC_lexicon_tags_17 },
    { 0xc478f2b0, SUC_lexicon_tags_27 },
    { 0x6db8f2b1, SUC_lexicon_tags_6 },
    { 0xb7f8f2b0, SUC_lexicon_tags_45 },
    { 0xe698f2b0, SUC_lexicon_tags_26 },
    { 0x6678f2b3, SUC_lexicon_tags_15 },
    { 0x8398f2b5, SUC_lexicon_tags_0 },
    { 0xd678f2b1, SUC_lexicon_tags_6 },
    { 0x8058f2b7, SUC_lexicon_tags_31 },
    { 0xad8f2b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6058f2c0, SUC_lexicon_tags_68 },
    { 0xab58f2c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcc78f2c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d8f2c9, SUC_lexicon_tags_17 },
    { 0x7698f2c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef8f2cd, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe58f2d6, SUC_lexicon_tags_13 },
    { 0x72d8f2d7, SUC_lexicon_tags_33 },
    { 0xc7f8f2d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b8f2dc, SUC_lexicon_tags_17 },
    { 0xfbd8f2dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5d8f2df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9918f2e2, SUC_lexicon_tags_11 },
    { 0xdf38f2e3, SUC_lexicon_tags_45 },
    { 0x86d8f2e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb8f2e7, SUC_lexicon_tags_2 },
    { 0x86b8f2e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f38f2ed, SUC_lexicon_tags_9 },
    { 0x1638f2ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb8f2f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x18f8f2f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b8f2f7, SUC_lexicon_tags_9 },
    { 0xcc98f2f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87f8f2fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad8f2ff, SUC_lexicon_tags_17 },
    { 0x9f58f300, SUC_lexicon_tags_1429 },
    { 0x91b8f301, SUC_lexicon_tags_25 },
    { 0x9d8f301, SUC_lexicon_tags_84 },
    { 0xea58f303, SUC_lexicon_tags_31 },
    { 0x40b8f304, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e98f30a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb8f30e, SUC_lexicon_tags_27 },
    { 0x4d8f30f, SUC_lexicon_tags_7 },
    { 0x4f98f30e, SUC_lexicon_tags_43 },
    { 0x50d8f30f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2598f313, SUC_lexicon_tags_38 },
    { 0xccd8f314, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x78f317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9118f31a, SUC_lexicon_tags_1 },
    { 0x1ed8f31a, SUC_lexicon_tags_0 },
    { 0xf3b8f31b, SUC_lexicon_tags_1 },
    { 0x3c78f31d, SUC_lexicon_tags_1 },
    { 0xc158f31e, SUC_lexicon_tags_2 },
    { 0x5198f31f, SUC_lexicon_tags_7 },
    { 0x7e18f320, SUC_lexicon_tags_14 },
    { 0xc758f31a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad8f324, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcaf8f326, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c98f329, SUC_lexicon_tags_3 },
    { 0x3018f32a, SUC_lexicon_tags_7 },
    { 0x2db8f32b, SUC_lexicon_tags_31 },
    { 0xdf58f32c, SUC_lexicon_tags_38 },
    { 0x2498f32c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e18f32f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x678f331, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a78f333, SUC_lexicon_tags_33 },
    { 0xd478f334, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b8f337, SUC_lexicon_tags_15 },
    { 0x6a78f338, SUC_lexicon_tags_26 },
    { 0xc598f339, SUC_lexicon_tags_30 },
    { 0x2c58f33a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec78f33c, SUC_lexicon_tags_43 },
    { 0xddd8f33d, SUC_lexicon_tags_15 },
    { 0x7658f33c, SUC_lexicon_tags_26 },
    { 0xebf8f33e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a98f344, SUC_lexicon_tags_7 },
    { 0x6178f345, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd698f34a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7358f34c, SUC_lexicon_tags_15 },
    { 0xae58f34d, SUC_lexicon_tags_65 },
    { 0xb5f8f34c, SUC_lexicon_tags_15 },
    { 0x4558f34f, SUC_lexicon_tags_11 },
    { 0xc9f8f34f, SUC_lexicon_tags_2 },
    { 0xa4d8f351, SUC_lexicon_tags_26 },
    { 0x4258f352, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad98f356, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8138f35a, SUC_lexicon_tags_26 },
    { 0x9978f35a, SUC_lexicon_tags_33 },
    { 0x7f58f35c, SUC_lexicon_tags_15 },
    { 0x3f18f35d, SUC_lexicon_tags_7 },
    { 0x1118f35a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa8f8f360, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe198f363, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd8f36b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b38f36d, SUC_lexicon_tags_15 },
    { 0xb2d8f36d, SUC_lexicon_tags_56 },
    { 0xf218f36f, SUC_lexicon_tags_7 },
    { 0xbcf8f370, SUC_lexicon_tags_25 },
    { 0xa058f370, SUC_lexicon_tags_29 },
    { 0xb0b8f372, SUC_lexicon_tags_17 },
    { 0x1498f373, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b8f376, SUC_lexicon_tags_17 },
    { 0x1df8f377, SUC_lexicon_tags_11 },
    { 0x3338f378, SUC_lexicon_tags_2 },
    { 0xb1d8f379, SUC_lexicon_tags_14 },
    { 0xe4f8f377, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbab8f37c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a78f380, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8738f386, SUC_lexicon_tags_2 },
    { 0xbe18f387, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad8f390, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xa718f392, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3938f39c, SUC_lexicon_tags_17 },
    { 0x7258f39d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf818f3a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf598f3ac, SUC_lexicon_tags_48 },
    { 0xca78f3ad, SUC_lexicon_tags_720 },
    { 0x3b58f3ad, SUC_lexicon_tags_15 },
    { 0x8df8f3af, SUC_lexicon_tags_19 },
    { 0xe138f3af, SUC_lexicon_tags_42 },
    { 0xd758f3af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b58f3b4, SUC_lexicon_tags_37 },
    { 0xa538f3b5, SUC_lexicon_tags_68 },
    { 0x4998f3b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd8f3b9, SUC_lexicon_tags_7 },
    { 0xb558f3b9, SUC_lexicon_tags_15 },
    { 0x2b58f3ba, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0x14b8f3bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7658f3c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa658f3c2, SUC_lexicon_tags_7 },
    { 0xd838f3c3, SUC_lexicon_tags_25 },
    { 0x3e78f3c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a78f3c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x70b8f3cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec38f3cf, SUC_lexicon_tags_277 },
    { 0x74f8f3d0, SUC_lexicon_tags_55 },
    { 0xe878f3d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f78f3d8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d8f3db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1978f3dd, SUC_lexicon_tags_44 },
    { 0x90f8f3de, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd8f3e2, SUC_lexicon_tags_7 },
    { 0xe158f3e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe698f3e7, SUC_lexicon_tags_37 },
    { 0x5938f3e7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa98f3f5, SUC_lexicon_tags_2 },
    { 0x31f8f3f6, SUC_lexicon_tags_30 },
    { 0xd38f3f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f8f3fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70b8f3fc, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc298f402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d8f405, SUC_lexicon_tags_42 },
    { 0xf9b8f406, SUC_lexicon_tags_19 },
    { 0xd058f406, SUC_lexicon_tags_30 },
    { 0x9418f408, SUC_lexicon_tags_14 },
    { 0x9f78f406, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1118f40c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76f8f411, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4058f413, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa398f416, SUC_lexicon_tags_29 },
    { 0x40d8f416, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x478f41c, SUC_lexicon_tags_15 },
    { 0x2c58f41c, SUC_lexicon_tags_7 },
    { 0x8e98f41d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2798f420, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8318f426, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x618f42b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x778f42d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3998f431, SUC_lexicon_tags_37 },
    { 0xf78f432, SUC_lexicon_tags_28 },
    { 0x49d8f432, SUC_lexicon_tags_44 },
    { 0x25f8f431, SUC_lexicon_tags_15 },
    { 0xc6b8f435, SUC_lexicon_tags_60 },
    { 0xf9f8f434, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3278f43a, SUC_lexicon_tags_33 },
    { 0x1198f43b, SUC_lexicon_tags_25 },
    { 0xd3d8f43b, SUC_lexicon_tags_1 },
    { 0x83f8f43d, SUC_lexicon_tags_37 },
    { 0x6338f43e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4318f441, SUC_lexicon_tags_16 },
    { 0x6e78f442, SUC_lexicon_tags_37 },
    { 0xa258f442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d8f447, SUC_lexicon_tags_31 },
    { 0x9ef8f447, SUC_lexicon_tags_1 },
    { 0xc5d8f449, SUC_lexicon_tags_17 },
    { 0x4338f449, SUC_lexicon_tags_7 },
    { 0x6278f44b, SUC_lexicon_tags_17 },
    { 0x6198f44a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xae58f44e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a58f450, SUC_lexicon_tags_29 },
    { 0x3878f450, SUC_lexicon_tags_1 },
    { 0xdad8f452, SUC_lexicon_tags_11 },
    { 0xedb8f453, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc178f455, SUC_lexicon_tags_33 },
    { 0xa58f455, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2858f45a, SUC_lexicon_tags_45 },
    { 0x7af8f45b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb8f462, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1ef8f464, SUC_lexicon_tags_19 },
    { 0x4f78f464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa58f467, SUC_lexicon_tags_47 },
    { 0x998f468, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39f8f46a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b8f46c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d8f470, SUC_lexicon_tags_26 },
    { 0x7a18f470, SUC_lexicon_tags_4 },
    { 0xdcd8f470, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf8f476, SUC_lexicon_tags_27 },
    { 0x1898f476, SUC_lexicon_tags_26 },
    { 0xf8b8f476, SUC_lexicon_tags_17 },
    { 0xac18f478, SUC_lexicon_tags_7 },
    { 0x2018f478, SUC_lexicon_tags_65 },
    { 0xb0b8f47b, SUC_lexicon_tags_48 },
    { 0x2318f477, SUC_lexicon_tags_21 },
    { 0x5158f47d, SUC_lexicon_tags_45 },
    { 0x7218f47d, SUC_lexicon_tags_17 },
    { 0xfc38f47c, SUC_lexicon_tags_47 },
    { 0x8d38f47c, SUC_lexicon_tags_39 },
    { 0xeed8f47b, SUC_lexicon_tags_39 },
    { 0xc698f482, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb8f485, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7778f488, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df8f490, SUC_lexicon_tags_15 },
    { 0x54d8f491, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac78f497, SUC_lexicon_tags_29 },
    { 0x14d8f497, SUC_lexicon_tags_14 },
    { 0xb878f497, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb58f49d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6b8f49f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33f8f4a1, SUC_lexicon_tags_25 },
    { 0x4e98f4a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d98f4a6, SUC_lexicon_tags_47 },
    { 0x3938f4a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5678f4ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6c98f4ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe378f4b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e38f4b2, SUC_lexicon_tags_17 },
    { 0x7eb8f4b3, SUC_lexicon_tags_11 },
    { 0x5cf8f4b4, SUC_lexicon_tags_49 },
    { 0x7f58f4b5, SUC_lexicon_tags_9 },
    { 0xbf18f4b6, SUC_lexicon_tags_17 },
    { 0xfbd8f4b6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9518f4ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd18f4be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x398f4c0, SUC_lexicon_tags_17 },
    { 0x5998f4c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa698f4c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af8f4c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b18f4cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe038f4ce, SUC_lexicon_tags_25 },
    { 0x2e38f4ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2478f4d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d78f4d5, SUC_lexicon_tags_15 },
    { 0xab58f4d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5218f4d8, SUC_lexicon_tags_1 },
    { 0xb18f4d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b8f4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8db8f4de, SUC_lexicon_tags_15 },
    { 0x3078f4df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf398f4e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7138f4e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd8f4e9, SUC_lexicon_tags_17 },
    { 0x9558f4e9, SUC_lexicon_tags_11 },
    { 0xf678f4ea, SUC_lexicon_tags_49 },
    { 0x9618f4ea, SUC_lexicon_tags_12 },
    { 0x9f8f4ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9078f4f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c58f4f4, SUC_lexicon_tags_16 },
    { 0xf5d8f4f5, SUC_lexicon_tags_6 },
    { 0x4438f4f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd8f4f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f58f4fb, SUC_lexicon_tags_14 },
    { 0xe6d8f4fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5678f4ff, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x3298f501, SUC_lexicon_tags_37 },
    { 0x5cb8f502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f8f508, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa918f50a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3d8f50c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2838f515, SUC_lexicon_tags_2 },
    { 0x8dd8f516, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b8f51b, SUC_lexicon_tags_22 },
    { 0x4218f51c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c58f51e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d8f525, SUC_lexicon_tags_14 },
    { 0xde98f525, SUC_lexicon_tags_63 },
    { 0x5f8f527, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f18f52b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f8f530, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95b8f532, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa238f534, SUC_lexicon_tags_25 },
    { 0x6678f535, SUC_lexicon_tags_47 },
    { 0xea78f534, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b8f539, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb218f53b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd898f53f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3298f543, SUC_lexicon_tags_55 },
    { 0x9bb8f543, SUC_lexicon_tags_11 },
    { 0xefd8f545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b8f54a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd0f8f54c, SUC_lexicon_tags_17 },
    { 0x96f8f54d, SUC_lexicon_tags_1 },
    { 0x2b18f54e, SUC_lexicon_tags_17 },
    { 0xdbf8f54f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5918f557, SUC_lexicon_tags_2 },
    { 0x2fb8f558, SUC_lexicon_tags_16 },
    { 0xd338f559, SUC_lexicon_tags_15 },
    { 0x54f8f55a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf8f560, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3158f565, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf18f568, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f98f56c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f8f56f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8978f571, SUC_lexicon_tags_7 },
    { 0x5cf8f571, SUC_lexicon_tags_49 },
    { 0x9e18f573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd58f575, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d8f579, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3878f57c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6798f57e, SUC_lexicon_tags_1462 },
    { 0x1298f57f, SUC_lexicon_tags_19 },
    { 0x28f8f580, SUC_lexicon_tags_47 },
    { 0x2958f580, SUC_lexicon_tags_27 },
    { 0x3218f582, SUC_lexicon_tags_43 },
    { 0x3ad8f583, SUC_lexicon_tags_2 },
    { 0x94f8f584, SUC_lexicon_tags_2 },
    { 0x398f580, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe118f588, SUC_lexicon_tags_33 },
    { 0x5198f588, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f8f58c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc378f591, SUC_lexicon_tags_37 },
    { 0xc6b8f592, SUC_lexicon_tags_49 },
    { 0x5338f593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18f596, SUC_lexicon_tags_25 },
    { 0x6d8f597, SUC_lexicon_tags_7 },
    { 0x2eb8f597, SUC_lexicon_tags_37 },
    { 0xc878f597, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b18f59b, SUC_lexicon_tags_23 },
    { 0x2ab8f59c, SUC_lexicon_tags_31 },
    { 0xee38f59d, SUC_lexicon_tags_25 },
    { 0x58f59e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae98f5a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5f8f5a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4398f5ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd858f5af, SUC_lexicon_tags_57 },
    { 0xa18f5b0, SUC_lexicon_tags_37 },
    { 0x78f5b1, SUC_lexicon_tags_66 },
    { 0x6398f5af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe658f5b5, SUC_lexicon_tags_10 },
    { 0x8a38f5b6, SUC_lexicon_tags_19 },
    { 0xb4f8f5b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d8f5bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3358f5c6, SUC_lexicon_tags_45 },
    { 0xfab8f5c7, SUC_lexicon_tags_23 },
    { 0x9418f5c6, SUC_lexicon_tags_25 },
    { 0x58f5c9, SUC_lexicon_tags_15 },
    { 0x5258f5ca, SUC_lexicon_tags_0 },
    { 0x9218f5cb, SUC_lexicon_tags_11 },
    { 0x6638f5ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7378f5cf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5298f5d2, SUC_lexicon_tags_37 },
    { 0xd578f5d3, SUC_lexicon_tags_11 },
    { 0xd638f5d3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d78f5dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd8f5df, SUC_lexicon_tags_59 },
    { 0x58b8f5e0, SUC_lexicon_tags_9 },
    { 0xa198f5e1, SUC_lexicon_tags_23 },
    { 0x7d58f5df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1878f5e6, SUC_lexicon_tags_8 },
    { 0x7f38f5e6, SUC_lexicon_tags_27 },
    { 0x9e98f5e7, SUC_lexicon_tags_70 },
    { 0xc158f5e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94b8f5ec, SUC_lexicon_tags_8 },
    { 0x4138f5ed, SUC_lexicon_tags_25 },
    { 0xff38f5ed, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e38f5f2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc218f5f7, SUC_lexicon_tags_1 },
    { 0xdc98f5f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5098f5fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e98f5fe, SUC_lexicon_tags_29 },
    { 0xabf8f5ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab8f605, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d8f60b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20f8f60d, SUC_lexicon_tags_38 },
    { 0x9a18f60e, SUC_lexicon_tags_11 },
    { 0x3e18f60f, SUC_lexicon_tags_38 },
    { 0x63f8f60d, SUC_lexicon_tags_92 },
    { 0x6278f610, SUC_lexicon_tags_35 },
    { 0x6978f612, SUC_lexicon_tags_14 },
    { 0xca78f613, SUC_lexicon_tags_49 },
    { 0x2bd8f614, SUC_lexicon_tags_25 },
    { 0xb438f614, SUC_lexicon_tags_11 },
    { 0x6258f615, SUC_lexicon_tags_14 },
    { 0xfe98f610, SUC_lexicon_tags_25 },
    { 0x45f8f618, SUC_lexicon_tags_9 },
    { 0x73d8f614, SUC_lexicon_tags_37 },
    { 0xe718f61a, SUC_lexicon_tags_17 },
    { 0xbe18f61b, SUC_lexicon_tags_25 },
    { 0xb18f60e, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xc278f61e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b8f624, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5718f628, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a18f62d, SUC_lexicon_tags_27 },
    { 0x1a58f62d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xe9f8f630, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb458f632, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ad8f634, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1358f636, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a98f63c, SUC_lexicon_tags_77 },
    { 0x1378f63d, SUC_lexicon_tags_15 },
    { 0xe618f63e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc38f647, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d8f64b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8518f64d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1798f650, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d38f654, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b8f658, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa338f65d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3378f661, SUC_lexicon_tags_66 },
    { 0x5d58f661, SUC_lexicon_tags_82 },
    { 0xfbf8f663, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d8f669, SUC_lexicon_tags_34 },
    { 0x698f669, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7678f66d, SUC_lexicon_tags_104 },
    { 0x5118f66e, SUC_lexicon_tags_0 },
    { 0x658f66e, SUC_lexicon_tags_25 },
    { 0xae58f66e, SUC_lexicon_tags_11 },
    { 0x7558f66d, SUC_lexicon_tags_25 },
    { 0x7e38f66e, SUC_lexicon_tags_0 },
    { 0x6fd8f673, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda58f675, SUC_lexicon_tags_1 },
    { 0xbcd8f675, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x5e58f678, SUC_lexicon_tags_21 },
    { 0x93d8f679, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc258f67b, SUC_lexicon_tags_14 },
    { 0xb218f67b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf8f67f, SUC_lexicon_tags_46 },
    { 0x34d8f680, SUC_lexicon_tags_37 },
    { 0x6b38f680, SUC_lexicon_tags_2 },
    { 0x7998f67f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9338f685, SUC_lexicon_tags_29 },
    { 0x33f8f686, SUC_lexicon_tags_26 },
    { 0x8858f685, SUC_lexicon_tags_49 },
    { 0x5038f688, SUC_lexicon_tags_64 },
    { 0x8cf8f689, SUC_lexicon_tags_25 },
    { 0xac98f68a, SUC_lexicon_tags_27 },
    { 0x9bd8f68b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb638f68e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6198f694, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f98f696, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2fd8f698, SUC_lexicon_tags_37 },
    { 0xa098f699, SUC_lexicon_tags_47 },
    { 0x4bf8f69a, SUC_lexicon_tags_2 },
    { 0xfed8f69b, SUC_lexicon_tags_14 },
    { 0xe978f69c, SUC_lexicon_tags_27 },
    { 0x7098f69d, SUC_lexicon_tags_37 },
    { 0x1ab8f69d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e78f6a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa138f6a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb518f6ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe118f6af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f98f6b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73d8f6b6, SUC_lexicon_tags_57 },
    { 0x4278f6b7, SUC_lexicon_tags_37 },
    { 0xf678f6b8, SUC_lexicon_tags_17 },
    { 0xbe78f6b8, SUC_lexicon_tags_37 },
    { 0xc178f6b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a38f6be, SUC_lexicon_tags_15 },
    { 0x338f6bf, SUC_lexicon_tags_13 },
    { 0x11f8f6c0, SUC_lexicon_tags_4 },
    { 0xd678f6c0, SUC_lexicon_tags_45 },
    { 0xb678f6c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff8f6c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fb8f6c8, SUC_lexicon_tags_11 },
    { 0x2bb8f6c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6378f6d0, SUC_lexicon_tags_25 },
    { 0xcef8f6d1, SUC_lexicon_tags_15 },
    { 0x9718f6d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf8f6d5, SUC_lexicon_tags_9 },
    { 0xf7f8f6d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa1f8f6d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb038f6da, SUC_lexicon_tags_60 },
    { 0xfdb8f6db, SUC_lexicon_tags_2 },
    { 0x41b8f6dc, SUC_lexicon_tags_7 },
    { 0xb058f6db, SUC_lexicon_tags_2 },
    { 0xe258f6de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d8f6e6, SUC_lexicon_tags_30 },
    { 0x9618f6e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad38f6eb, SUC_lexicon_tags_26 },
    { 0x48d8f6ec, SUC_lexicon_tags_10 },
    { 0xd5f8f6ed, SUC_lexicon_tags_49 },
    { 0x4518f6ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5278f6f1, SUC_lexicon_tags_28 },
    { 0x4118f6f2, SUC_lexicon_tags_34 },
    { 0x5558f6f1, SUC_lexicon_tags_25 },
    { 0x1a58f6f3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d8f6f7, SUC_lexicon_tags_47 },
    { 0xba58f6f8, SUC_lexicon_tags_7 },
    { 0xc458f6f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1418f6fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d8f6fe, SUC_lexicon_tags_2 },
    { 0x7358f6ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf8f703, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d8f706, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab8f70b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f98f70e, SUC_lexicon_tags_12 },
    { 0x30d8f70f, SUC_lexicon_tags_8 },
    { 0xbd98f710, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab8f715, SUC_lexicon_tags_57 },
    { 0x8158f715, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce38f721, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee38f723, SUC_lexicon_tags_87 },
    { 0xe638f723, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db8f727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5798f729, SUC_lexicon_tags_39 },
    { 0x7898f729, SUC_lexicon_tags_37 },
    { 0x2198f72b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab8f730, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3918f732, SUC_lexicon_tags_25 },
    { 0x8138f732, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f78f736, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa258f73a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x5998f73c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf698f73e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb8f745, SUC_lexicon_tags_4 },
    { 0x7a58f746, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f8f74f, SUC_lexicon_tags_9 },
    { 0x9818f74f, SUC_lexicon_tags_77 },
    { 0x2d8f751, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6db8f753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78b8f755, SUC_lexicon_tags_47 },
    { 0x2f78f755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d8f75f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf8f768, SUC_lexicon_tags_12 },
    { 0x4d78f769, SUC_lexicon_tags_12 },
    { 0x23f8f76a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcf8f76c, SUC_lexicon_tags_11 },
    { 0xc858f76d, SUC_lexicon_tags_16 },
    { 0xa38f76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71d8f770, SUC_lexicon_tags_27 },
    { 0x1578f771, SUC_lexicon_tags_8 },
    { 0x5bf8f770, SUC_lexicon_tags_26 },
    { 0xcf58f773, SUC_lexicon_tags_7 },
    { 0xccb8f773, SUC_lexicon_tags_11 },
    { 0x7b98f775, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x98f777, SUC_lexicon_tags_6 },
    { 0xff18f778, SUC_lexicon_tags_37 },
    { 0x238f778, SUC_lexicon_tags_17 },
    { 0x6258f777, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5438f780, SUC_lexicon_tags_19 },
    { 0xa0f8f781, SUC_lexicon_tags_19 },
    { 0xa3d8f782, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b98f785, SUC_lexicon_tags_12 },
    { 0xbf98f786, SUC_lexicon_tags_11 },
    { 0x3e38f785, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc58f78a, SUC_lexicon_tags_65 },
    { 0x6918f78b, SUC_lexicon_tags_7 },
    { 0x5358f78c, SUC_lexicon_tags_37 },
    { 0xab38f78d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb8f790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7318f796, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7938f79d, SUC_lexicon_tags_13 },
    { 0xf898f79e, SUC_lexicon_tags_49 },
    { 0xdd78f79f, SUC_lexicon_tags_47 },
    { 0x65b8f7a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xef8f7a2, SUC_lexicon_tags_8 },
    { 0xfa78f7a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f8f7a6, SUC_lexicon_tags_15 },
    { 0x8018f7a6, SUC_lexicon_tags_25 },
    { 0x6a98f7a8, SUC_lexicon_tags_27 },
    { 0x2138f7a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b8f7ad, SUC_lexicon_tags_15 },
    { 0xfe58f7ae, SUC_lexicon_tags_37 },
    { 0xf538f7ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac38f7b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f8f7bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4718f7bf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa258f7c1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2cb8f7c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb8f7c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x218f7c9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6b78f7cb, SUC_lexicon_tags_7 },
    { 0x4658f7cc, SUC_lexicon_tags_7 },
    { 0x96b8f7cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf838f7cf, SUC_lexicon_tags_12 },
    { 0x8c38f7d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8898f7d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d18f7d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf8f7d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3318f7db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebb8f7dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d8f7e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b38f7e2, SUC_lexicon_tags_37 },
    { 0x54f8f7e3, SUC_lexicon_tags_2 },
    { 0x2698f7e4, SUC_lexicon_tags_28 },
    { 0xd238f7e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9838f7e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2998f7e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcef8f7eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb38f7ed, SUC_lexicon_tags_60 },
    { 0x878f7ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff8f7f2, SUC_lexicon_tags_16 },
    { 0xaed8f7f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb8f7f6, SUC_lexicon_tags_0 },
    { 0x4218f7f6, SUC_lexicon_tags_38 },
    { 0xf378f7f8, SUC_lexicon_tags_11 },
    { 0x7df8f7f9, SUC_lexicon_tags_29 },
    { 0x9638f7f9, SUC_lexicon_tags_1 },
    { 0x8318f7fb, SUC_lexicon_tags_38 },
    { 0x9298f7fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b8f7ff, SUC_lexicon_tags_31 },
    { 0x1c18f7ff, SUC_lexicon_tags_25 },
    { 0x67f8f801, SUC_lexicon_tags_45 },
    { 0x6418f801, SUC_lexicon_tags_1 },
    { 0xcef8f803, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc98f806, SUC_lexicon_tags_161 },
    { 0xaf38f806, SUC_lexicon_tags_2 },
    { 0xbb8f808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc978f80a, SUC_lexicon_tags_10 },
    { 0x7098f80b, SUC_lexicon_tags_6 },
    { 0x2978f80b, SUC_lexicon_tags_32 },
    { 0xdf58f80c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x10d8f810, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b38f812, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2338f816, SUC_lexicon_tags_15 },
    { 0x9cf8f817, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd8f819, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3278f81b, SUC_lexicon_tags_37 },
    { 0xedb8f81c, SUC_lexicon_tags_37 },
    { 0xed58f81b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x70f8f81f, SUC_lexicon_tags_25 },
    { 0x2ff8f81f, SUC_lexicon_tags_0 },
    { 0xb7f8f821, SUC_lexicon_tags_39 },
    { 0xeb8f821, SUC_lexicon_tags_27 },
    { 0x4478f821, SUC_lexicon_tags_14 },
    { 0xd078f821, SUC_lexicon_tags_57 },
    { 0x2ff8f820, SUC_lexicon_tags_37 },
    { 0x2478f826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d8f829, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2418f82b, SUC_lexicon_tags_25 },
    { 0x8d38f82b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e58f82e, SUC_lexicon_tags_17 },
    { 0xb298f82f, SUC_lexicon_tags_63 },
    { 0x4cd8f82e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5838f833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd8f839, SUC_lexicon_tags_25 },
    { 0xe5f8f839, SUC_lexicon_tags_8 },
    { 0xf898f83b, SUC_lexicon_tags_17 },
    { 0x8198f83c, SUC_lexicon_tags_11 },
    { 0x3ab8f83c, SUC_lexicon_tags_17 },
    { 0xc358f83e, SUC_lexicon_tags_17 },
    { 0xc2b8f83e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4038f845, SUC_lexicon_tags_55 },
    { 0x8b78f846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f8f849, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d8f84c, SUC_lexicon_tags_2 },
    { 0x5478f84d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d58f851, SUC_lexicon_tags_14 },
    { 0x7d18f852, SUC_lexicon_tags_7 },
    { 0x278f851, SUC_lexicon_tags_37 },
    { 0x98f854, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x58d8f856, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb158f858, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f78f85c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd8f85f, SUC_lexicon_tags_47 },
    { 0xc1b8f860, SUC_lexicon_tags_28 },
    { 0xb618f861, SUC_lexicon_tags_33 },
    { 0xbe18f862, SUC_lexicon_tags_37 },
    { 0x2bb8f860, SUC_lexicon_tags_61 },
    { 0xaf58f864, SUC_lexicon_tags_11 },
    { 0x1358f865, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab8f86a, SUC_lexicon_tags_210 },
    { 0x2458f86a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd8f86e, SUC_lexicon_tags_18 },
    { 0x49d8f86f, SUC_lexicon_tags_7 },
    { 0x34b8f870, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb8f873, SUC_lexicon_tags_65 },
    { 0xd118f873, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c18f876, SUC_lexicon_tags_37 },
    { 0x66b8f876, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ad8f879, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d78f881, SUC_lexicon_tags_48 },
    { 0xc3f8f881, SUC_lexicon_tags_45 },
    { 0x8bd8f883, SUC_lexicon_tags_49 },
    { 0xead8f884, SUC_lexicon_tags_151 },
    { 0xedf8f885, SUC_lexicon_tags_47 },
    { 0xd818f886, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6278f88d, SUC_lexicon_tags_7 },
    { 0x5518f88e, SUC_lexicon_tags_43 },
    { 0xc798f88f, SUC_lexicon_tags_37 },
    { 0x5058f88f, SUC_lexicon_tags_15 },
    { 0xbe98f890, SUC_lexicon_tags_0 },
    { 0x2218f88f, SUC_lexicon_tags_17 },
    { 0xc6f8f88f, SUC_lexicon_tags_39 },
    { 0x17f8f893, SUC_lexicon_tags_47 },
    { 0x1e98f895, SUC_lexicon_tags_39 },
    { 0x5458f896, SUC_lexicon_tags_31 },
    { 0x4ad8f88e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa218f89a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf58f89c, SUC_lexicon_tags_7 },
    { 0x2ef8f89c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b8f8a2, SUC_lexicon_tags_17 },
    { 0x5d18f8a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd258f8a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x92b8f8a8, SUC_lexicon_tags_151 },
    { 0x4e78f8a9, SUC_lexicon_tags_7 },
    { 0x6538f8aa, SUC_lexicon_tags_70 },
    { 0x5b18f8aa, SUC_lexicon_tags_31 },
    { 0x7738f8ac, SUC_lexicon_tags_46 },
    { 0x2658f8ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9818f8af, SUC_lexicon_tags_45 },
    { 0x4e58f8b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb8f8b8, SUC_lexicon_tags_7 },
    { 0xb58f8b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3658f8bb, SUC_lexicon_tags_27 },
    { 0x2478f8bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf8f8c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8458f8c6, SUC_lexicon_tags_38 },
    { 0x2158f8c7, SUC_lexicon_tags_12 },
    { 0xabf8f8c8, SUC_lexicon_tags_45 },
    { 0xd998f8c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb8f8cd, SUC_lexicon_tags_10 },
    { 0xca78f8ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2678f8d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb58f8d4, SUC_lexicon_tags_7 },
    { 0x98f8d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb178f8d7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x7118f8da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf8f8dc, SUC_lexicon_tags_12 },
    { 0x198f8dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a58f8df, SUC_lexicon_tags_45 },
    { 0x8b98f8e0, SUC_lexicon_tags_25 },
    { 0xc5b8f8e1, SUC_lexicon_tags_23 },
    { 0x4dd8f8e2, SUC_lexicon_tags_10 },
    { 0x2358f8e3, SUC_lexicon_tags_9 },
    { 0x3178f8df, SUC_lexicon_tags_65 },
    { 0x51b8f8e4, SUC_lexicon_tags_1 },
    { 0xea18f8e5, SUC_lexicon_tags_39 },
    { 0x1718f8e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab98f8e9, SUC_lexicon_tags_42 },
    { 0xf8d8f8e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf98f8ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a98f8ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f8f8f1, SUC_lexicon_tags_33 },
    { 0xeaf8f8f2, SUC_lexicon_tags_37 },
    { 0x3c58f8f3, SUC_lexicon_tags_14 },
    { 0x65b8f8f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b8f8f6, SUC_lexicon_tags_15 },
    { 0x4cb8f8f7, SUC_lexicon_tags_2 },
    { 0x2238f8f8, SUC_lexicon_tags_37 },
    { 0x2678f8f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc58f901, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec18f909, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b8f90d, SUC_lexicon_tags_57 },
    { 0xa578f90e, SUC_lexicon_tags_1 },
    { 0x22b8f90f, SUC_lexicon_tags_1 },
    { 0x2598f910, SUC_lexicon_tags_68 },
    { 0x4318f911, SUC_lexicon_tags_10 },
    { 0x1a18f912, SUC_lexicon_tags_57 },
    { 0x3538f913, SUC_lexicon_tags_14 },
    { 0x1818f914, SUC_lexicon_tags_7 },
    { 0xc818f912, SUC_lexicon_tags_26 },
    { 0x3a18f916, SUC_lexicon_tags_33 },
    { 0x3698f910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db8f91a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc578f921, SUC_lexicon_tags_37 },
    { 0xba38f921, SUC_lexicon_tags_37 },
    { 0xd958f923, SUC_lexicon_tags_7 },
    { 0xb558f922, SUC_lexicon_tags_28 },
    { 0x6198f925, SUC_lexicon_tags_0 },
    { 0x2e78f926, SUC_lexicon_tags_7 },
    { 0x3538f921, SUC_lexicon_tags_43 },
    { 0x38f923, SUC_lexicon_tags_4 },
    { 0xc9f8f921, SUC_lexicon_tags_37 },
    { 0xcfb8f92a, SUC_lexicon_tags_17 },
    { 0x93b8f922, SUC_lexicon_tags_1041 },
    { 0x5ad8f92c, SUC_lexicon_tags_17 },
    { 0x3d98f925, SUC_lexicon_tags_66 },
    { 0x3778f92e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b8f931, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6b8f933, SUC_lexicon_tags_11 },
    { 0x3698f934, SUC_lexicon_tags_17 },
    { 0xfa38f934, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1338f939, SUC_lexicon_tags_11 },
    { 0x73d8f939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cd8f93c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b8f940, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x99f8f942, SUC_lexicon_tags_38 },
    { 0xb1d8f942, SUC_lexicon_tags_55 },
    { 0x1fd8f944, SUC_lexicon_tags_3 },
    { 0xf9b8f945, SUC_lexicon_tags_17 },
    { 0x3c58f945, SUC_lexicon_tags_25 },
    { 0x2498f947, SUC_lexicon_tags_47 },
    { 0xe818f944, SUC_lexicon_tags_37 },
    { 0x90f8f948, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdaf8f94b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e18f94d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b8f952, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bd8f954, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3638f956, SUC_lexicon_tags_11 },
    { 0xefd8f956, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a98f95a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e58f95c, SUC_lexicon_tags_37 },
    { 0x9a78f95d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b8f961, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f8f966, SUC_lexicon_tags_105 },
    { 0x86f8f967, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ed8f969, SUC_lexicon_tags_29 },
    { 0xa178f969, SUC_lexicon_tags_12 },
    { 0xa698f96b, SUC_lexicon_tags_11 },
    { 0x6dd8f96c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1738f971, SUC_lexicon_tags_60 },
    { 0x3d98f972, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba38f976, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6278f978, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b58f97b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb438f97e, SUC_lexicon_tags_7 },
    { 0x67f8f97f, SUC_lexicon_tags_12 },
    { 0x6458f97e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda78f987, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b8f991, SUC_lexicon_tags_2 },
    { 0x5678f992, SUC_lexicon_tags_87 },
    { 0xbcd8f992, SUC_lexicon_tags_26 },
    { 0x9c78f992, SUC_lexicon_tags_0 },
    { 0x1b58f995, SUC_lexicon_tags_17 },
    { 0x9858f992, SUC_lexicon_tags_11 },
    { 0x5c98f995, SUC_lexicon_tags_25 },
    { 0xadf8f996, SUC_lexicon_tags_7 },
    { 0x4098f999, SUC_lexicon_tags_131 },
    { 0xd138f993, SUC_lexicon_tags_49 },
    { 0x4578f99a, SUC_lexicon_tags_63 },
    { 0xe018f99c, SUC_lexicon_tags_14 },
    { 0xf2d8f99d, SUC_lexicon_tags_25 },
    { 0x4818f99e, SUC_lexicon_tags_7 },
    { 0x5078f99c, SUC_lexicon_tags_104 },
    { 0xd2f8f995, SUC_lexicon_tags_4 },
    { 0xdb8f9a1, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d8f9a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f8f9a9, SUC_lexicon_tags_17 },
    { 0x8338f9aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a78f9ac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb0d8f9ae, SUC_lexicon_tags_6 },
    { 0xc6b8f9af, SUC_lexicon_tags_2 },
    { 0xc978f9af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd538f9b3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d8f9b6, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed98f9bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61d8f9bf, SUC_lexicon_tags_12 },
    { 0x6c18f9c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x80d8f9c2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4598f9c4, SUC_lexicon_tags_13 },
    { 0x76f8f9c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8cd8f9c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb638f9cf, SUC_lexicon_tags_8 },
    { 0xc638f9cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5958f9d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9bd8f9d4, SUC_lexicon_tags_282 },
    { 0x1798f9d5, SUC_lexicon_tags_31 },
    { 0x418f9d6, SUC_lexicon_tags_14 },
    { 0x63f8f9d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d8f9dd, SUC_lexicon_tags_11 },
    { 0x6c58f9dd, SUC_lexicon_tags_37 },
    { 0x4a98f9df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb578f9e2, SUC_lexicon_tags_267 },
    { 0x2738f9e3, SUC_lexicon_tags_17 },
    { 0xf198f9e3, SUC_lexicon_tags_29 },
    { 0x2498f9e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f18f9ef, SUC_lexicon_tags_9 },
    { 0xfed8f9ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf8f9f3, SUC_lexicon_tags_6 },
    { 0xa3d8f9f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe18f9f6, SUC_lexicon_tags_2 },
    { 0x6538f9f6, SUC_lexicon_tags_14 },
    { 0x6498f9f6, SUC_lexicon_tags_1 },
    { 0xe7b8f9f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5e58f9fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2978f9ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43d8fa01, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x56b8fa04, SUC_lexicon_tags_13 },
    { 0xe858fa05, SUC_lexicon_tags_17 },
    { 0x82d8fa06, SUC_lexicon_tags_27 },
    { 0x3b58fa07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa618fa09, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6578fa0d, SUC_lexicon_tags_49 },
    { 0x6778fa0d, SUC_lexicon_tags_29 },
    { 0x6298fa0d, SUC_lexicon_tags_11 },
    { 0x78b8fa10, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2b38fa12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x458fa19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b18fa22, SUC_lexicon_tags_45 },
    { 0xb3f8fa22, SUC_lexicon_tags_17 },
    { 0x1c58fa23, SUC_lexicon_tags_34 },
    { 0xae98fa25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x718fa27, SUC_lexicon_tags_12 },
    { 0x35f8fa28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe038fa2a, SUC_lexicon_tags_0 },
    { 0xf058fa2a, SUC_lexicon_tags_12 },
    { 0xaf38fa2c, SUC_lexicon_tags_29 },
    { 0xd178fa2c, SUC_lexicon_tags_27 },
    { 0xf58fa2e, SUC_lexicon_tags_84 },
    { 0xafd8fa2e, SUC_lexicon_tags_49 },
    { 0x5398fa2d, SUC_lexicon_tags_19 },
    { 0xba38fa31, SUC_lexicon_tags_47 },
    { 0x8658fa2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d8fa38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa98fa3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc398fa3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b8fa41, SUC_lexicon_tags_27 },
    { 0x7a38fa42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c38fa47, SUC_lexicon_tags_9 },
    { 0xa438fa48, SUC_lexicon_tags_25 },
    { 0xd2d8fa48, SUC_lexicon_tags_2 },
    { 0x24d8fa47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bb8fa4c, SUC_lexicon_tags_25 },
    { 0xbed8fa4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6958fa4f, SUC_lexicon_tags_30 },
    { 0x37d8fa50, SUC_lexicon_tags_47 },
    { 0xedb8fa4f, SUC_lexicon_tags_49 },
    { 0x4e98fa52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa58fa55, SUC_lexicon_tags_8 },
    { 0xebf8fa56, SUC_lexicon_tags_47 },
    { 0x71d8fa55, SUC_lexicon_tags_49 },
    { 0x4f38fa58, SUC_lexicon_tags_12 },
    { 0xc58fa58, SUC_lexicon_tags_4 },
    { 0xc2f8fa55, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f8fa5e, SUC_lexicon_tags_25 },
    { 0xb398fa5e, SUC_lexicon_tags_14 },
    { 0xd238fa60, SUC_lexicon_tags_62 },
    { 0x5a98fa60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7518fa64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd078fa6b, SUC_lexicon_tags_2 },
    { 0xd9b8fa6c, SUC_lexicon_tags_23 },
    { 0xfeb8fa6c, SUC_lexicon_tags_23 },
    { 0xcdf8fa6b, SUC_lexicon_tags_19 },
    { 0xfa18fa6d, SUC_lexicon_tags_11 },
    { 0xa2b8fa70, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc078fa75, SUC_lexicon_tags_15 },
    { 0x2378fa76, SUC_lexicon_tags_7 },
    { 0xc418fa76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a38fa7b, SUC_lexicon_tags_1 },
    { 0xea18fa7c, SUC_lexicon_tags_26 },
    { 0xdd8fa7d, SUC_lexicon_tags_21 },
    { 0xfcf8fa7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe698fa80, SUC_lexicon_tags_37 },
    { 0x7ed8fa80, SUC_lexicon_tags_12 },
    { 0xe858fa80, SUC_lexicon_tags_9 },
    { 0xef78fa83, SUC_lexicon_tags_12 },
    { 0x5438fa84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf5b8fa86, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe1f8fa88, SUC_lexicon_tags_9 },
    { 0x3858fa89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa4b8fa8b, SUC_lexicon_tags_14 },
    { 0x5a58fa8b, SUC_lexicon_tags_9 },
    { 0xcb78fa8d, SUC_lexicon_tags_47 },
    { 0x2e98fa8d, SUC_lexicon_tags_38 },
    { 0xaff8fa8b, SUC_lexicon_tags_1 },
    { 0xe3b8fa8f, SUC_lexicon_tags_0 },
    { 0x84f8fa91, SUC_lexicon_tags_17 },
    { 0xe9f8fa92, SUC_lexicon_tags_7 },
    { 0xbf98fa93, SUC_lexicon_tags_11 },
    { 0xc778fa94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb8fa99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b8fa9e, SUC_lexicon_tags_21 },
    { 0xac18fa9f, SUC_lexicon_tags_30 },
    { 0x418fa9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a98faa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8218faa7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f8faab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x198fab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3478fab6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3f8fab8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2a58faba, SUC_lexicon_tags_79 },
    { 0xd0b8fabb, SUC_lexicon_tags_26 },
    { 0x2eb8fabc, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x5038fabe, SUC_lexicon_tags_46 },
    { 0xe6f8fabf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d8fac3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe818fac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d38faca, SUC_lexicon_tags_37 },
    { 0x6db8facb, SUC_lexicon_tags_38 },
    { 0xc218faca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3118fad3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f8fadc, SUC_lexicon_tags_6 },
    { 0xc278fadc, SUC_lexicon_tags_6 },
    { 0x2898fade, SUC_lexicon_tags_25 },
    { 0x30b8fadf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb58fae1, SUC_lexicon_tags_12 },
    { 0x9498fae1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd8faee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe58faf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8918faf9, SUC_lexicon_tags_12 },
    { 0x1fd8faf9, SUC_lexicon_tags_66 },
    { 0xb938fafa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed8fafe, SUC_lexicon_tags_28 },
    { 0x15f8faff, SUC_lexicon_tags_2 },
    { 0xb918faff, SUC_lexicon_tags_15 },
    { 0x4d98fafe, SUC_lexicon_tags_15 },
    { 0xb98fb02, SUC_lexicon_tags_7 },
    { 0xdd78fb03, SUC_lexicon_tags_49 },
    { 0x1618fb02, SUC_lexicon_tags_37 },
    { 0x54f8fb05, SUC_lexicon_tags_15 },
    { 0x3f58fb06, SUC_lexicon_tags_1 },
    { 0x31b8fb07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5258fb0c, SUC_lexicon_tags_37 },
    { 0xb6d8fb0c, SUC_lexicon_tags_8 },
    { 0x57b8fb0e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x31b8fb10, SUC_lexicon_tags_12 },
    { 0x82d8fb11, SUC_lexicon_tags_10 },
    { 0xc018fb12, SUC_lexicon_tags_15 },
    { 0x8378fb12, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xae38fb16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4398fb1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc6b8fb1d, SUC_lexicon_tags_17 },
    { 0x85d8fb1d, SUC_lexicon_tags_77 },
    { 0x8b18fb1f, SUC_lexicon_tags_38 },
    { 0x9078fb20, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb38fb24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd078fb26, SUC_lexicon_tags_11 },
    { 0xb238fb26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b8fb2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86f8fb2d, SUC_lexicon_tags_4 },
    { 0x8858fb2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead8fb33, SUC_lexicon_tags_37 },
    { 0xdeb8fb34, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b8fb38, SUC_lexicon_tags_1 },
    { 0x8e98fb38, SUC_lexicon_tags_10 },
    { 0x4658fb3a, SUC_lexicon_tags_1 },
    { 0x50f8fb3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa358fb42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe438fb47, SUC_lexicon_tags_43 },
    { 0x32d8fb48, SUC_lexicon_tags_8 },
    { 0xd438fb49, SUC_lexicon_tags_25 },
    { 0xc058fb4a, SUC_lexicon_tags_37 },
    { 0xf958fb4b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f8fb4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d8fb53, SUC_lexicon_tags_19 },
    { 0x8fd8fb54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab8fb57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa998fb5c, SUC_lexicon_tags_13 },
    { 0x7178fb5c, SUC_lexicon_tags_25 },
    { 0xd938fb5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef8fb61, SUC_lexicon_tags_15 },
    { 0x3498fb61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8478fb64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde38fb69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6638fb6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab8fb70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9d8fb72, SUC_lexicon_tags_12 },
    { 0xa6f8fb73, SUC_lexicon_tags_1 },
    { 0x5bb8fb73, SUC_lexicon_tags_11 },
    { 0x4698fb74, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc78fb79, SUC_lexicon_tags_38 },
    { 0x92d8fb7a, SUC_lexicon_tags_2 },
    { 0x5a78fb7b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9798fb84, SUC_lexicon_tags_84 },
    { 0xa738fb85, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d8fb8e, SUC_lexicon_tags_56 },
    { 0xa778fb8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b8fb92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb8fb97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f8fb9a, SUC_lexicon_tags_1 },
    { 0x8398fb9a, SUC_lexicon_tags_12 },
    { 0xc158fb9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x98d8fb9e, SUC_lexicon_tags_11 },
    { 0x1258fb9f, SUC_lexicon_tags_6 },
    { 0xea18fba0, SUC_lexicon_tags_3 },
    { 0x4298fb9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3578fba6, SUC_lexicon_tags_48 },
    { 0x33d8fba6, SUC_lexicon_tags_68 },
    { 0x45f8fba7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xe698fbab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb858fbae, SUC_lexicon_tags_37 },
    { 0xec78fbae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbd98fbb1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e18fbb4, SUC_lexicon_tags_43 },
    { 0x42d8fbb4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2218fbb7, SUC_lexicon_tags_17 },
    { 0x5e18fbb8, SUC_lexicon_tags_1 },
    { 0x81d8fbb7, SUC_lexicon_tags_38 },
    { 0x70d8fbba, SUC_lexicon_tags_38 },
    { 0x6598fbb9, SUC_lexicon_tags_70 },
    { 0xd338fbba, SUC_lexicon_tags_70 },
    { 0xa98fbbd, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab8fbc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb418fbc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2518fbc4, SUC_lexicon_tags_0 },
    { 0xcf58fbc5, SUC_lexicon_tags_12 },
    { 0xefb8fbc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f8fbca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f8fbd0, SUC_lexicon_tags_15 },
    { 0x37d8fbd0, SUC_lexicon_tags_6 },
    { 0x67d8fbd1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b98fbd6, SUC_lexicon_tags_12 },
    { 0xafd8fbd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe578fbda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b8fbdf, SUC_lexicon_tags_4 },
    { 0x9998fbe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1678fbe2, SUC_lexicon_tags_48 },
    { 0x6158fbe3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b8fbe7, SUC_lexicon_tags_0 },
    { 0x2138fbe8, SUC_lexicon_tags_48 },
    { 0x1cb8fbe9, SUC_lexicon_tags_33 },
    { 0x8e58fbea, SUC_lexicon_tags_37 },
    { 0xddb8fbeb, SUC_lexicon_tags_29 },
    { 0xdf78fbea, SUC_lexicon_tags_25 },
    { 0xdc78fbe7, SUC_lexicon_tags_37 },
    { 0xaa98fbee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab78fbf0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf8fbf7, SUC_lexicon_tags_14 },
    { 0x3d18fbf8, SUC_lexicon_tags_26 },
    { 0x7718fbf9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5598fbfc, SUC_lexicon_tags_33 },
    { 0xa4d8fbfd, SUC_lexicon_tags_39 },
    { 0xc438fbfe, SUC_lexicon_tags_37 },
    { 0xcc98fbff, SUC_lexicon_tags_37 },
    { 0x3b78fbfe, SUC_lexicon_tags_14 },
    { 0x11b8fc01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f78fc06, SUC_lexicon_tags_1 },
    { 0x2478fc06, SUC_lexicon_tags_11 },
    { 0xf878fc08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f18fc0b, SUC_lexicon_tags_17 },
    { 0xe098fc0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbef8fc0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5158fc10, SUC_lexicon_tags_11 },
    { 0xd078fc11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf698fc14, SUC_lexicon_tags_37 },
    { 0x6f38fc14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbe58fc17, SUC_lexicon_tags_37 },
    { 0x1838fc18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3c98fc1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa58fc1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddd8fc1f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xea78fc21, SUC_lexicon_tags_55 },
    { 0xd0b8fc21, SUC_lexicon_tags_23 },
    { 0xa938fc23, SUC_lexicon_tags_28 },
    { 0x4738fc21, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8178fc2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e18fc31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe558fc35, SUC_lexicon_tags_12 },
    { 0x24b8fc36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2538fc38, SUC_lexicon_tags_33 },
    { 0xbe98fc39, SUC_lexicon_tags_62 },
    { 0x5158fc38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2338fc3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb98fc3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe938fc44, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe138fc46, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5458fc49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeb8fc4b, SUC_lexicon_tags_37 },
    { 0xee38fc4c, SUC_lexicon_tags_1 },
    { 0x3358fc4d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf58fc50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9078fc52, SUC_lexicon_tags_63 },
    { 0xa238fc53, SUC_lexicon_tags_15 },
    { 0x54d8fc53, SUC_lexicon_tags_60 },
    { 0xa8f8fc55, SUC_lexicon_tags_10 },
    { 0xb1d8fc55, SUC_lexicon_tags_30 },
    { 0xb618fc56, SUC_lexicon_tags_14 },
    { 0xc78fc58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4678fc5b, SUC_lexicon_tags_19 },
    { 0x4df8fc5c, SUC_lexicon_tags_74 },
    { 0x3058fc5d, SUC_lexicon_tags_8 },
    { 0x358fc5e, SUC_lexicon_tags_30 },
    { 0x95b8fc5e, SUC_lexicon_tags_0 },
    { 0xb538fc5c, SUC_lexicon_tags_15 },
    { 0xab18fc61, SUC_lexicon_tags_47 },
    { 0x8db8fc62, SUC_lexicon_tags_15 },
    { 0x9398fc63, SUC_lexicon_tags_414 },
    { 0xa258fc64, SUC_lexicon_tags_87 },
    { 0xb178fc62, SUC_lexicon_tags_27 },
    { 0xdcf8fc62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae78fc6a, SUC_lexicon_tags_15 },
    { 0x7118fc6b, SUC_lexicon_tags_37 },
    { 0xa3f8fc6a, SUC_lexicon_tags_16 },
    { 0xe3f8fc6d, SUC_lexicon_tags_32 },
    { 0x4578fc6c, SUC_lexicon_tags_29 },
    { 0x2378fc6c, SUC_lexicon_tags_12 },
    { 0x3b8fc70, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf218fc73, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db8fc78, SUC_lexicon_tags_902 },
    { 0, NULL },
    { 0x7178fc7a, SUC_lexicon_tags_37 },
    { 0xd8b8fc7a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd8fc7e, SUC_lexicon_tags_33 },
    { 0x5598fc7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db8fc83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a18fc88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc18fc91, SUC_lexicon_tags_25 },
    { 0xd2f8fc92, SUC_lexicon_tags_59 },
    { 0x3138fc92, SUC_lexicon_tags_37 },
    { 0xc5b8fc94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x14b8fc96, SUC_lexicon_tags_65 },
    { 0xbe18fc96, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e58fca2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3618fca7, SUC_lexicon_tags_17 },
    { 0xd1d8fca8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f58fcb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c98fcb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3618fcba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd38fcbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f8fcc6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7f98fcc8, SUC_lexicon_tags_4 },
    { 0xa498fcc9, SUC_lexicon_tags_105 },
    { 0x1ad8fcc9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc18fcce, SUC_lexicon_tags_19 },
    { 0x9298fccf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2218fcd2, SUC_lexicon_tags_12 },
    { 0x2bd8fcd3, SUC_lexicon_tags_37 },
    { 0xae38fcd3, SUC_lexicon_tags_4 },
    { 0x5b98fcd4, SUC_lexicon_tags_1 },
    { 0xe638fcd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee38fcdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb518fce1, SUC_lexicon_tags_47 },
    { 0x72f8fce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1018fce4, SUC_lexicon_tags_160 },
    { 0x6e98fce4, SUC_lexicon_tags_1 },
    { 0x678fce5, SUC_lexicon_tags_12 },
    { 0x858fce7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa598fcef, SUC_lexicon_tags_49 },
    { 0xc9d8fcf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedb8fcf2, SUC_lexicon_tags_68 },
    { 0x2cf8fcf3, SUC_lexicon_tags_17 },
    { 0x9098fcf3, SUC_lexicon_tags_1 },
    { 0x2a98fcf5, SUC_lexicon_tags_54 },
    { 0x9fb8fcf5, SUC_lexicon_tags_49 },
    { 0x5218fcf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b38fcf9, SUC_lexicon_tags_1 },
    { 0x9a58fcf9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed8fd00, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad8fd03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1038fd05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa18fd09, SUC_lexicon_tags_27 },
    { 0xe318fd0a, SUC_lexicon_tags_17 },
    { 0x1818fd0a, SUC_lexicon_tags_19 },
    { 0xb538fd0b, SUC_lexicon_tags_25 },
    { 0x6078fd0d, SUC_lexicon_tags_40 },
    { 0xc658fd0c, SUC_lexicon_tags_7 },
    { 0x39f8fd0f, SUC_lexicon_tags_11 },
    { 0xc6f8fd10, SUC_lexicon_tags_19 },
    { 0x41b8fd0d, SUC_lexicon_tags_1 },
    { 0x1f38fd11, SUC_lexicon_tags_1 },
    { 0xc898fd13, SUC_lexicon_tags_17 },
    { 0x3878fd13, SUC_lexicon_tags_70 },
    { 0x4018fd0d, SUC_lexicon_tags_45 },
    { 0x3e18fd12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef8fd1a, SUC_lexicon_tags_6 },
    { 0x2eb8fd1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe478fd1e, SUC_lexicon_tags_26 },
    { 0xb0b8fd1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a38fd22, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7d38fd24, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f8fd28, SUC_lexicon_tags_30 },
    { 0x5398fd29, SUC_lexicon_tags_68 },
    { 0x4618fd28, SUC_lexicon_tags_9 },
    { 0x9718fd2b, SUC_lexicon_tags_25 },
    { 0x6d18fd2c, SUC_lexicon_tags_14 },
    { 0x9758fd2c, SUC_lexicon_tags_2 },
    { 0x9178fd2e, SUC_lexicon_tags_15 },
    { 0x4fb8fd2b, SUC_lexicon_tags_47 },
    { 0x7d18fd2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11d8fd32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bf8fd34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c38fd39, SUC_lexicon_tags_74 },
    { 0x2478fd3a, SUC_lexicon_tags_14 },
    { 0xd458fd3b, SUC_lexicon_tags_1 },
    { 0x79b8fd3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd8fd40, SUC_lexicon_tags_16 },
    { 0xc318fd40, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb338fd43, SUC_lexicon_tags_91 },
    { 0xb898fd44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6118fd48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc318fd4c, SUC_lexicon_tags_11 },
    { 0x4f58fd4d, SUC_lexicon_tags_17 },
    { 0x3238fd4e, SUC_lexicon_tags_1 },
    { 0x6618fd4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe218fd51, SUC_lexicon_tags_17 },
    { 0x59b8fd52, SUC_lexicon_tags_37 },
    { 0xcbf8fd53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfab8fd56, SUC_lexicon_tags_28 },
    { 0xf178fd57, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f8fd5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2d18fd5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe738fd60, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f8fd65, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff8fd6d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf418fd72, SUC_lexicon_tags_77 },
    { 0xd898fd72, SUC_lexicon_tags_8 },
    { 0x9c78fd72, SUC_lexicon_tags_25 },
    { 0x9ed8fd73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3738fd78, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd58fd7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc98fd7c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa478fd80, SUC_lexicon_tags_12 },
    { 0xf278fd80, SUC_lexicon_tags_8 },
    { 0x8b18fd82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5778fd84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda58fd86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7138fd8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca98fd90, SUC_lexicon_tags_33 },
    { 0x6d18fd90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1718fd94, SUC_lexicon_tags_25 },
    { 0x2378fd95, SUC_lexicon_tags_11 },
    { 0xfc78fd96, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4118fd99, SUC_lexicon_tags_7 },
    { 0xee18fd9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc338fd9d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb858fda0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b38fda3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d8fda6, SUC_lexicon_tags_25 },
    { 0x1598fda7, SUC_lexicon_tags_25 },
    { 0x2738fda8, SUC_lexicon_tags_1 },
    { 0xf6f8fda7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7698fdac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f8fdaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd8fdb2, SUC_lexicon_tags_11 },
    { 0x98d8fdb2, SUC_lexicon_tags_25 },
    { 0x8038fdb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad8fdba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6978fdbd, SUC_lexicon_tags_21 },
    { 0x6a78fdbd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d8fdc3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc518fdd0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd058fdd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3778fdda, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1638fde1, SUC_lexicon_tags_8 },
    { 0x22b8fde1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8478fde5, SUC_lexicon_tags_15 },
    { 0x3718fde6, SUC_lexicon_tags_8 },
    { 0x4db8fde7, SUC_lexicon_tags_1 },
    { 0x6878fde7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe858fdea, SUC_lexicon_tags_25 },
    { 0xe538fdeb, SUC_lexicon_tags_12 },
    { 0x9838fdeb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8ef8fdee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xba18fdf1, SUC_lexicon_tags_27 },
    { 0xd138fdf2, SUC_lexicon_tags_37 },
    { 0xb918fdf3, SUC_lexicon_tags_15 },
    { 0x878fdf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a18fdf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8398fdf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf8fdfd, SUC_lexicon_tags_33 },
    { 0xfad8fdfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab8fe01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc898fe03, SUC_lexicon_tags_0 },
    { 0xcc98fe04, SUC_lexicon_tags_15 },
    { 0xaed8fe05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf898fe0a, SUC_lexicon_tags_9 },
    { 0xd438fe0a, SUC_lexicon_tags_79 },
    { 0xa598fe0c, SUC_lexicon_tags_37 },
    { 0xe3f8fe0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf8fe13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c98fe18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61d8fe1a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa0b8fe1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c98fe2b, SUC_lexicon_tags_23 },
    { 0x6f38fe2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3798fe2e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5698fe32, SUC_lexicon_tags_38 },
    { 0x6d98fe32, SUC_lexicon_tags_9 },
    { 0x4818fe34, SUC_lexicon_tags_63 },
    { 0x5738fe35, SUC_lexicon_tags_43 },
    { 0xdb78fe36, SUC_lexicon_tags_1 },
    { 0xa158fe33, SUC_lexicon_tags_15 },
    { 0x2d38fe32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6598fe3f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5138fe42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf58fe47, SUC_lexicon_tags_1 },
    { 0xd6d8fe47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4358fe4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd518fe50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d8fe53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3938fe55, SUC_lexicon_tags_197 },
    { 0x3b58fe56, SUC_lexicon_tags_38 },
    { 0x6498fe57, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1778fe5b, SUC_lexicon_tags_23 },
    { 0xa7b8fe5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcb98fe5e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7998fe6a, SUC_lexicon_tags_38 },
    { 0xa618fe6a, SUC_lexicon_tags_9 },
    { 0x4b8fe6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf98fe72, SUC_lexicon_tags_23 },
    { 0x8f8fe72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f8fe76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1298fe7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d8fe82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce78fe86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd58fe8d, SUC_lexicon_tags_12 },
    { 0x1178fe8d, SUC_lexicon_tags_28 },
    { 0x90b8fe8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8fe95, SUC_lexicon_tags_37 },
    { 0x32f8fe95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe458fe9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef8fe9d, SUC_lexicon_tags_9 },
    { 0xbf98fe9e, SUC_lexicon_tags_14 },
    { 0xc778fe9f, SUC_lexicon_tags_35 },
    { 0x3c18fe9d, SUC_lexicon_tags_174 },
    { 0xb7d8fea1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b38fea4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6458fea6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f8fea9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe2f8feab, SUC_lexicon_tags_17 },
    { 0x2d18feac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b98feaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b58feb2, SUC_lexicon_tags_45 },
    { 0x9698feb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f38feb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d8feba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe1b8febc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5d8febe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4018fec0, SUC_lexicon_tags_17 },
    { 0x8058fec1, SUC_lexicon_tags_38 },
    { 0x56d8fec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd158fec5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b8fecb, SUC_lexicon_tags_12 },
    { 0x29b8fecc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7958fece, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb158fed2, SUC_lexicon_tags_28 },
    { 0xf0b8fed2, SUC_lexicon_tags_25 },
    { 0xb778fed4, SUC_lexicon_tags_37 },
    { 0x7e38fed5, SUC_lexicon_tags_60 },
    { 0xc7d8fed2, SUC_lexicon_tags_12 },
    { 0x10d8fed7, SUC_lexicon_tags_2 },
    { 0xa3b8fed5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f8fedc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb38fede, SUC_lexicon_tags_33 },
    { 0xdbf8fede, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a18fee3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8698fee5, SUC_lexicon_tags_43 },
    { 0x858fee6, SUC_lexicon_tags_11 },
    { 0xf1f8fee6, SUC_lexicon_tags_14 },
    { 0x83f8fee8, SUC_lexicon_tags_19 },
    { 0x6f98fee8, SUC_lexicon_tags_32 },
    { 0x1078fee8, SUC_lexicon_tags_17 },
    { 0x2e98feeb, SUC_lexicon_tags_25 },
    { 0xaf38feea, SUC_lexicon_tags_10 },
    { 0x9978fee6, SUC_lexicon_tags_17 },
    { 0xdf38feee, SUC_lexicon_tags_46 },
    { 0x3818feec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x70b8fef1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2678fef4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa158fef7, SUC_lexicon_tags_23 },
    { 0xe7b8fef8, SUC_lexicon_tags_8 },
    { 0xbe58fef7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fefe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6538ff02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4158ff0a, SUC_lexicon_tags_6 },
    { 0x4898ff0a, SUC_lexicon_tags_19 },
    { 0x1a78ff0a, SUC_lexicon_tags_57 },
    { 0xbc78ff0d, SUC_lexicon_tags_6 },
    { 0x2d38ff0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7c18ff10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf678ff14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed58ff18, SUC_lexicon_tags_0 },
    { 0xbcd8ff19, SUC_lexicon_tags_1 },
    { 0xae98ff1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5398ff1c, SUC_lexicon_tags_25 },
    { 0x7358ff1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8d8ff1f, SUC_lexicon_tags_27 },
    { 0x39b8ff20, SUC_lexicon_tags_0 },
    { 0x4e78ff21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb78ff29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d58ff31, SUC_lexicon_tags_8 },
    { 0xc498ff32, SUC_lexicon_tags_47 },
    { 0xc4b8ff33, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e58ff36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b8ff39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d98ff3e, SUC_lexicon_tags_60 },
    { 0x72d8ff3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ff42, SUC_lexicon_tags_1 },
    { 0xf098ff43, SUC_lexicon_tags_25 },
    { 0x618ff43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe238ff47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed8ff4e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb178ff50, SUC_lexicon_tags_2 },
    { 0xd158ff51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c38ff57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad8ff5a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf738ff5d, SUC_lexicon_tags_33 },
    { 0xe498ff5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd538ff66, SUC_lexicon_tags_15 },
    { 0xaa98ff67, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a58ff6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb718ff6d, SUC_lexicon_tags_0 },
    { 0x5618ff6e, SUC_lexicon_tags_2 },
    { 0xe3d8ff6f, SUC_lexicon_tags_17 },
    { 0xb078ff6f, SUC_lexicon_tags_9 },
    { 0x5978ff6f, SUC_lexicon_tags_2 },
    { 0x58b8ff6d, SUC_lexicon_tags_17 },
    { 0xddf8ff73, SUC_lexicon_tags_29 },
    { 0x2938ff73, SUC_lexicon_tags_60 },
    { 0xebd8ff73, SUC_lexicon_tags_32 },
    { 0x6318ff76, SUC_lexicon_tags_4 },
    { 0xc718ff77, SUC_lexicon_tags_60 },
    { 0x2a18ff78, SUC_lexicon_tags_15 },
    { 0xc418ff77, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b78ff82, SUC_lexicon_tags_11 },
    { 0x9278ff83, SUC_lexicon_tags_11 },
    { 0x3938ff84, SUC_lexicon_tags_2 },
    { 0xfe38ff85, SUC_lexicon_tags_91 },
    { 0xc6b8ff85, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f8ff8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f8ff8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x52b8ff91, SUC_lexicon_tags_11 },
    { 0xc8b8ff92, SUC_lexicon_tags_38 },
    { 0xd998ff92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f8ff96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd78ff9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d8ffa6, SUC_lexicon_tags_40 },
    { 0x8498ffa6, SUC_lexicon_tags_1 },
    { 0xc778ffa8, SUC_lexicon_tags_15 },
    { 0x59b8ffa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa958ffaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd078ffba, SUC_lexicon_tags_11 },
    { 0x1918ffba, SUC_lexicon_tags_65 },
    { 0xa098ffbb, SUC_lexicon_tags_31 },
    { 0x51f8ffbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97f8ffbf, SUC_lexicon_tags_27 },
    { 0x4ad8ffbf, SUC_lexicon_tags_129 },
    { 0xe938ffc0, SUC_lexicon_tags_11 },
    { 0xa838ffc2, SUC_lexicon_tags_0 },
    { 0x5418ffc3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xed18ffc6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9d8ffc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7ab8ffca, SUC_lexicon_tags_9 },
    { 0x5878ffcb, SUC_lexicon_tags_70 },
    { 0x11f8ffcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc98ffcf, SUC_lexicon_tags_55 },
    { 0x7678ffd0, SUC_lexicon_tags_47 },
    { 0x41f8ffcf, SUC_lexicon_tags_2 },
    { 0xc058ffd2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf478ffd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e98ffd9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x92f8ffdb, SUC_lexicon_tags_1 },
    { 0x5538ffdc, SUC_lexicon_tags_131 },
    { 0xc798ffdb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ef8ffdf, SUC_lexicon_tags_87 },
    { 0x6fd8ffe0, SUC_lexicon_tags_37 },
    { 0x2f78ffe0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b58ffe4, SUC_lexicon_tags_49 },
    { 0xafb8ffe5, SUC_lexicon_tags_1 },
    { 0x6898ffe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7798ffea, SUC_lexicon_tags_1 },
    { 0x1c38ffeb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1358fff2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4778fff4, SUC_lexicon_tags_15 },
    { 0x46b8fff4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf9b8fff7, SUC_lexicon_tags_20 },
    { 0xd5b8fff8, SUC_lexicon_tags_14 },
    { 0x4698fff9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8598ffff, SUC_lexicon_tags_11 },
    { 0x4818ffff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1af90002, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x45d90004, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d90008, SUC_lexicon_tags_13 },
    { 0x8dd90009, SUC_lexicon_tags_12 },
    { 0xd6d90008, SUC_lexicon_tags_32 },
    { 0x1e99000b, SUC_lexicon_tags_12 },
    { 0x55b9000c, SUC_lexicon_tags_7 },
    { 0x86f9000a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x49d9000f, SUC_lexicon_tags_61 },
    { 0x15190010, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4390012, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x24f90014, SUC_lexicon_tags_7 },
    { 0xfdf90015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b790017, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39001c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb239001e, SUC_lexicon_tags_15 },
    { 0xdaf9001e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80190029, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xfa99002b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb379002d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56d9002f, SUC_lexicon_tags_5 },
    { 0x3d990030, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d90033, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b90036, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f9003c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd39003e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8590045, SUC_lexicon_tags_49 },
    { 0x1ab90046, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f9004d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab790050, SUC_lexicon_tags_43 },
    { 0x75790051, SUC_lexicon_tags_17 },
    { 0x5e190051, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d390055, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x17190057, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6659005c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42790060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b90063, SUC_lexicon_tags_25 },
    { 0x52b90063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe399006a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d9006d, SUC_lexicon_tags_26 },
    { 0x2519006d, SUC_lexicon_tags_31 },
    { 0xccb9006f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0790071, SUC_lexicon_tags_37 },
    { 0x8c790072, SUC_lexicon_tags_1 },
    { 0x31d90073, SUC_lexicon_tags_25 },
    { 0x8c990074, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd190077, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x790079, SUC_lexicon_tags_25 },
    { 0x93b9007a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2f79007c, SUC_lexicon_tags_83 },
    { 0x5979007c, SUC_lexicon_tags_15 },
    { 0x5c39007c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55990081, SUC_lexicon_tags_60 },
    { 0x14b90082, SUC_lexicon_tags_45 },
    { 0xbd90083, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9190088, SUC_lexicon_tags_37 },
    { 0xb1390088, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9559008b, SUC_lexicon_tags_27 },
    { 0x87f9008c, SUC_lexicon_tags_14 },
    { 0xbb9008b, SUC_lexicon_tags_7 },
    { 0x48d9008b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d90092, SUC_lexicon_tags_1530 },
    { 0, NULL },
    { 0, NULL },
    { 0x82990095, SUC_lexicon_tags_11 },
    { 0x74f90096, SUC_lexicon_tags_11 },
    { 0x56d90096, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6fb90099, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x799009b, SUC_lexicon_tags_1 },
    { 0xac59009c, SUC_lexicon_tags_38 },
    { 0x419009d, SUC_lexicon_tags_12 },
    { 0x74f9009e, SUC_lexicon_tags_25 },
    { 0x8099009e, SUC_lexicon_tags_25 },
    { 0x55f9009b, SUC_lexicon_tags_61 },
    { 0x3b9900a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x645900a3, SUC_lexicon_tags_9 },
    { 0x549900a3, SUC_lexicon_tags_34 },
    { 0xe4f900a3, SUC_lexicon_tags_7 },
    { 0xdf9900a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd900aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9900af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d900b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x207900b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5900b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29900bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769900c5, SUC_lexicon_tags_8 },
    { 0xff3900c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x101900c8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7900ca, SUC_lexicon_tags_23 },
    { 0xca1900ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651900cf, SUC_lexicon_tags_37 },
    { 0x66f900d0, SUC_lexicon_tags_0 },
    { 0x251900d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5900d7, SUC_lexicon_tags_45 },
    { 0xfa1900d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf900dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9900e1, SUC_lexicon_tags_15 },
    { 0x1a7900e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d900e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x761900eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d9900ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe2b900ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x389900f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93900f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b900f7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9900fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d90105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7099010a, SUC_lexicon_tags_21 },
    { 0xe19010b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad90110, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bf90112, SUC_lexicon_tags_2 },
    { 0xe0390113, SUC_lexicon_tags_84 },
    { 0x20b90114, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73190116, SUC_lexicon_tags_17 },
    { 0xad990117, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3519011c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9159011e, SUC_lexicon_tags_7 },
    { 0xa819011e, SUC_lexicon_tags_79 },
    { 0xb7190120, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d90124, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc390127, SUC_lexicon_tags_1 },
    { 0xe9990128, SUC_lexicon_tags_21 },
    { 0x4bd90128, SUC_lexicon_tags_1 },
    { 0x69190129, SUC_lexicon_tags_37 },
    { 0xd239012a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf159012d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7d9012f, SUC_lexicon_tags_37 },
    { 0xccf90130, SUC_lexicon_tags_91 },
    { 0x8cb90131, SUC_lexicon_tags_9 },
    { 0x8b79012f, SUC_lexicon_tags_11 },
    { 0x7db90131, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f9013b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b9013e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6590141, SUC_lexicon_tags_47 },
    { 0xf790142, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf590144, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f90146, SUC_lexicon_tags_0 },
    { 0xe4990147, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd9014e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f90152, SUC_lexicon_tags_14 },
    { 0xde790153, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c390156, SUC_lexicon_tags_19 },
    { 0x26990157, SUC_lexicon_tags_7 },
    { 0xc1d90157, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6939015b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1190160, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdbb90162, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf990166, SUC_lexicon_tags_15 },
    { 0xdbd90167, SUC_lexicon_tags_7 },
    { 0x3df90168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1bd9016a, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e59016d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd90171, SUC_lexicon_tags_26 },
    { 0xd5b90172, SUC_lexicon_tags_17 },
    { 0x47390173, SUC_lexicon_tags_64 },
    { 0x14190172, SUC_lexicon_tags_1 },
    { 0x4a390174, SUC_lexicon_tags_2 },
    { 0xe2d90176, SUC_lexicon_tags_37 },
    { 0xe5990172, SUC_lexicon_tags_11 },
    { 0x6b90178, SUC_lexicon_tags_1 },
    { 0x14590177, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd9017c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb9b9017e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c390180, SUC_lexicon_tags_2 },
    { 0xe590181, SUC_lexicon_tags_2 },
    { 0xbf990180, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb90184, SUC_lexicon_tags_8 },
    { 0x91190185, SUC_lexicon_tags_17 },
    { 0xc4990186, SUC_lexicon_tags_38 },
    { 0xc0d90185, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6a990189, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc979018f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5eb90191, SUC_lexicon_tags_23 },
    { 0x5b990191, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa390195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a390197, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10190199, SUC_lexicon_tags_63 },
    { 0x22b9019a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157901a0, SUC_lexicon_tags_391 },
    { 0xcd9901a1, SUC_lexicon_tags_1 },
    { 0xc45901a0, SUC_lexicon_tags_11 },
    { 0xe8d901a3, SUC_lexicon_tags_25 },
    { 0xba1901a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d901a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125901ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff901af, SUC_lexicon_tags_17 },
    { 0xa2d901af, SUC_lexicon_tags_45 },
    { 0xf87901b0, SUC_lexicon_tags_15 },
    { 0xf0f901b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7901b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced901bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8d901be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc57901c0, SUC_lexicon_tags_23 },
    { 0x105901c1, SUC_lexicon_tags_1 },
    { 0xa09901c2, SUC_lexicon_tags_224 },
    { 0x285901c1, SUC_lexicon_tags_2 },
    { 0x7f3901c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71901c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35b901ca, SUC_lexicon_tags_44 },
    { 0xdf5901cb, SUC_lexicon_tags_28 },
    { 0x5af901cc, SUC_lexicon_tags_25 },
    { 0xcb7901cd, SUC_lexicon_tags_45 },
    { 0xd35901cd, SUC_lexicon_tags_25 },
    { 0xae5901cf, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x9df901d1, SUC_lexicon_tags_60 },
    { 0xe9d901d2, SUC_lexicon_tags_25 },
    { 0xc27901d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5901d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc9901d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d901e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9901e6, SUC_lexicon_tags_28 },
    { 0x757901e6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xbef901e9, SUC_lexicon_tags_15 },
    { 0x8e9901ea, SUC_lexicon_tags_7 },
    { 0x2f901eb, SUC_lexicon_tags_17 },
    { 0x6c7901ec, SUC_lexicon_tags_7 },
    { 0x50b901ed, SUC_lexicon_tags_17 },
    { 0x45d901ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdf5901f0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x293901f2, SUC_lexicon_tags_11 },
    { 0x2bd901f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8ab901f5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbab901f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x823901fa, SUC_lexicon_tags_13 },
    { 0x4fb901fb, SUC_lexicon_tags_45 },
    { 0xf2f901fc, SUC_lexicon_tags_37 },
    { 0x65901fd, SUC_lexicon_tags_37 },
    { 0xd1b901fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb90201, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13190208, SUC_lexicon_tags_1 },
    { 0x95790208, SUC_lexicon_tags_4 },
    { 0xb9b90208, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x4619020d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f390212, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78390214, SUC_lexicon_tags_2 },
    { 0xa8f90215, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7fb90217, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc19021a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9779021d, SUC_lexicon_tags_42 },
    { 0x1019021e, SUC_lexicon_tags_45 },
    { 0xd5f9021e, SUC_lexicon_tags_1 },
    { 0xe4990220, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b90223, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6e190225, SUC_lexicon_tags_23 },
    { 0xda390226, SUC_lexicon_tags_68 },
    { 0xb4190227, SUC_lexicon_tags_30 },
    { 0xa2990228, SUC_lexicon_tags_108 },
    { 0x83190228, SUC_lexicon_tags_28 },
    { 0xea790228, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49590232, SUC_lexicon_tags_23 },
    { 0x53990232, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed90238, SUC_lexicon_tags_37 },
    { 0xe8190239, SUC_lexicon_tags_1 },
    { 0x81790239, SUC_lexicon_tags_63 },
    { 0xb479023b, SUC_lexicon_tags_47 },
    { 0x8459023c, SUC_lexicon_tags_49 },
    { 0xceb9023d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa379023f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1390242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b90247, SUC_lexicon_tags_6 },
    { 0x23190248, SUC_lexicon_tags_38 },
    { 0x6e590249, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7f9024b, SUC_lexicon_tags_15 },
    { 0x7239024c, SUC_lexicon_tags_17 },
    { 0x3479024d, SUC_lexicon_tags_1 },
    { 0xf899024e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d90251, SUC_lexicon_tags_17 },
    { 0x76590252, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xba590255, SUC_lexicon_tags_11 },
    { 0xe4790256, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b90260, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d90266, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1239026b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd2f9026d, SUC_lexicon_tags_69 },
    { 0xe539026d, SUC_lexicon_tags_17 },
    { 0xf2f9026e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1190273, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a19027e, SUC_lexicon_tags_23 },
    { 0x8699027f, SUC_lexicon_tags_25 },
    { 0x6999027f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x23790283, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b90289, SUC_lexicon_tags_17 },
    { 0x5b39028a, SUC_lexicon_tags_30 },
    { 0xa9f9028b, SUC_lexicon_tags_31 },
    { 0x7dd9028c, SUC_lexicon_tags_7 },
    { 0xef19028b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48990292, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f590297, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1379029c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b902a2, SUC_lexicon_tags_38 },
    { 0x8bd902a2, SUC_lexicon_tags_2 },
    { 0xd77902a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb902a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91902ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1902b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533902bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4b902be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5902c1, SUC_lexicon_tags_30 },
    { 0x835902c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89902c9, SUC_lexicon_tags_1 },
    { 0xc6f902ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d902cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1902d1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x25902d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95902d6, SUC_lexicon_tags_68 },
    { 0xa9d902d6, SUC_lexicon_tags_43 },
    { 0x935902d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d1902da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec5902dc, SUC_lexicon_tags_25 },
    { 0x88d902dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xded902df, SUC_lexicon_tags_49 },
    { 0xdd7902e0, SUC_lexicon_tags_37 },
    { 0x1db902e1, SUC_lexicon_tags_23 },
    { 0x5c1902e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3902e8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5902ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65902f1, SUC_lexicon_tags_71 },
    { 0xc7d902f2, SUC_lexicon_tags_31 },
    { 0x451902f1, SUC_lexicon_tags_63 },
    { 0x761902f4, SUC_lexicon_tags_14 },
    { 0xff7902f2, SUC_lexicon_tags_39 },
    { 0x9f902f6, SUC_lexicon_tags_25 },
    { 0xb33902f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x717902fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763902ff, SUC_lexicon_tags_37 },
    { 0x92f902ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5590304, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x40590306, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf279030a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec390310, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x28f90312, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x98790315, SUC_lexicon_tags_179 },
    { 0x2c390316, SUC_lexicon_tags_2 },
    { 0xc9b90316, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b9031b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b9031f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30990321, SUC_lexicon_tags_10 },
    { 0xeaf90321, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f90327, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2199032e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5d90330, SUC_lexicon_tags_32 },
    { 0x38390331, SUC_lexicon_tags_17 },
    { 0x7d90332, SUC_lexicon_tags_11 },
    { 0x35790333, SUC_lexicon_tags_19 },
    { 0xdd790334, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b90337, SUC_lexicon_tags_25 },
    { 0xea590337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed9033a, SUC_lexicon_tags_25 },
    { 0x9df9033b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff90349, SUC_lexicon_tags_10 },
    { 0xee99034a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90790350, SUC_lexicon_tags_10 },
    { 0x3c990351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59990355, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59190361, SUC_lexicon_tags_32 },
    { 0xed190362, SUC_lexicon_tags_29 },
    { 0x58790363, SUC_lexicon_tags_68 },
    { 0x90790363, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2d90366, SUC_lexicon_tags_79 },
    { 0x58b90367, SUC_lexicon_tags_7 },
    { 0x16b90368, SUC_lexicon_tags_2 },
    { 0x41990369, SUC_lexicon_tags_37 },
    { 0xb0590369, SUC_lexicon_tags_2 },
    { 0x9f90367, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a59036f, SUC_lexicon_tags_91 },
    { 0x69990370, SUC_lexicon_tags_11 },
    { 0x29590370, SUC_lexicon_tags_47 },
    { 0xdc19036f, SUC_lexicon_tags_1 },
    { 0x7fb9036f, SUC_lexicon_tags_11 },
    { 0xf4b90374, SUC_lexicon_tags_27 },
    { 0x75390374, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa3590377, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd659037a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x82f9037d, SUC_lexicon_tags_11 },
    { 0xe2b9037d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96990380, SUC_lexicon_tags_47 },
    { 0x19d90381, SUC_lexicon_tags_25 },
    { 0x92f90380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59590388, SUC_lexicon_tags_16 },
    { 0xb9f90389, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4af9038b, SUC_lexicon_tags_12 },
    { 0xf119038c, SUC_lexicon_tags_66 },
    { 0xb719038d, SUC_lexicon_tags_43 },
    { 0x3319038e, SUC_lexicon_tags_9 },
    { 0x8e19038f, SUC_lexicon_tags_15 },
    { 0x63b90390, SUC_lexicon_tags_61 },
    { 0x7a79038f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29790393, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b190396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6339039b, SUC_lexicon_tags_14 },
    { 0x1bb9039c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa219039e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21903a1, SUC_lexicon_tags_17 },
    { 0x2cb903a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd31903a4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d903a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x793903ab, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcd5903ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f903b1, SUC_lexicon_tags_37 },
    { 0x679903b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53903b4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x925903b7, SUC_lexicon_tags_17 },
    { 0x507903b8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19903bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59903bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9903c3, SUC_lexicon_tags_30 },
    { 0x8c9903c4, SUC_lexicon_tags_4 },
    { 0x905903c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717903ce, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xadb903d0, SUC_lexicon_tags_2 },
    { 0xa7b903d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xff1903d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113903d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d903dc, SUC_lexicon_tags_37 },
    { 0x71d903dc, SUC_lexicon_tags_32 },
    { 0x5c1903de, SUC_lexicon_tags_29 },
    { 0x213903de, SUC_lexicon_tags_2 },
    { 0x801903de, SUC_lexicon_tags_27 },
    { 0xe75903dc, SUC_lexicon_tags_28 },
    { 0x931903df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7903e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x119903ea, SUC_lexicon_tags_33 },
    { 0x8e1903eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffb903ed, SUC_lexicon_tags_11 },
    { 0xc03903ed, SUC_lexicon_tags_11 },
    { 0xe79903ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef5903f1, SUC_lexicon_tags_37 },
    { 0xd31903f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51903f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55903ff, SUC_lexicon_tags_2 },
    { 0xa17903ff, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8790404, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d790407, SUC_lexicon_tags_46 },
    { 0x42190408, SUC_lexicon_tags_49 },
    { 0xeed90409, SUC_lexicon_tags_38 },
    { 0xcd39040a, SUC_lexicon_tags_77 },
    { 0x9a99040a, SUC_lexicon_tags_60 },
    { 0x55190407, SUC_lexicon_tags_15 },
    { 0xacd9040d, SUC_lexicon_tags_3 },
    { 0x8d39040e, SUC_lexicon_tags_61 },
    { 0x8459040f, SUC_lexicon_tags_15 },
    { 0x6039040d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef90415, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x25f90417, SUC_lexicon_tags_19 },
    { 0x81f90418, SUC_lexicon_tags_7 },
    { 0xbe590417, SUC_lexicon_tags_37 },
    { 0xa8190417, SUC_lexicon_tags_37 },
    { 0xc939041b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7959041f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbcd90421, SUC_lexicon_tags_3 },
    { 0xc3f90422, SUC_lexicon_tags_62 },
    { 0xae390423, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9390426, SUC_lexicon_tags_23 },
    { 0x6cb90426, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2539042c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa7b9042e, SUC_lexicon_tags_7 },
    { 0xe799042f, SUC_lexicon_tags_10 },
    { 0x60b9042e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x85190433, SUC_lexicon_tags_23 },
    { 0x2d790434, SUC_lexicon_tags_92 },
    { 0x1f590435, SUC_lexicon_tags_1 },
    { 0xbe590436, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b9043c, SUC_lexicon_tags_129 },
    { 0x7a19043d, SUC_lexicon_tags_9 },
    { 0xb7d9043e, SUC_lexicon_tags_10 },
    { 0xf919043d, SUC_lexicon_tags_15 },
    { 0x92d90440, SUC_lexicon_tags_17 },
    { 0x64590441, SUC_lexicon_tags_35 },
    { 0xd0190442, SUC_lexicon_tags_11 },
    { 0x78190443, SUC_lexicon_tags_55 },
    { 0xc0590443, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d90447, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5679044f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82790453, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74790455, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad90462, SUC_lexicon_tags_16 },
    { 0xf0f90462, SUC_lexicon_tags_49 },
    { 0xc8b90464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee19046d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d90472, SUC_lexicon_tags_37 },
    { 0x5f790473, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e990475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe099047b, SUC_lexicon_tags_12 },
    { 0xdfb9047c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c99047f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3590482, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb9d90484, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88590488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb559048a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf5b9048c, SUC_lexicon_tags_11 },
    { 0x479048d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b590491, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e99049a, SUC_lexicon_tags_25 },
    { 0xb6d9049b, SUC_lexicon_tags_25 },
    { 0x4739049b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63904a0, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x157904a2, SUC_lexicon_tags_12 },
    { 0xd1904a2, SUC_lexicon_tags_7 },
    { 0xc0b904a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71f904a6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa83904a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1904ad, SUC_lexicon_tags_7 },
    { 0x619904ad, SUC_lexicon_tags_28 },
    { 0xd27904ad, SUC_lexicon_tags_2 },
    { 0x2ef904b0, SUC_lexicon_tags_17 },
    { 0x48f904b0, SUC_lexicon_tags_14 },
    { 0xef1904b2, SUC_lexicon_tags_48 },
    { 0x21d904b2, SUC_lexicon_tags_43 },
    { 0x263904b1, SUC_lexicon_tags_37 },
    { 0x3db904b5, SUC_lexicon_tags_31 },
    { 0x3d9904b6, SUC_lexicon_tags_11 },
    { 0xc01904b7, SUC_lexicon_tags_45 },
    { 0x7c9904b5, SUC_lexicon_tags_25 },
    { 0xd0b904b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f7904bb, SUC_lexicon_tags_30 },
    { 0xda7904bc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x74f904bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205904c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcdb904c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915904ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe83904cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363904d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbe7904d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe73904d4, SUC_lexicon_tags_0 },
    { 0x4f9904d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86b904d7, SUC_lexicon_tags_15 },
    { 0x511904d8, SUC_lexicon_tags_19 },
    { 0xfd5904d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b1904db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71904e0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1904e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x333904e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d904ea, SUC_lexicon_tags_43 },
    { 0x39b904ea, SUC_lexicon_tags_7 },
    { 0x8b904ec, SUC_lexicon_tags_11 },
    { 0x493904ed, SUC_lexicon_tags_88 },
    { 0x7b3904ee, SUC_lexicon_tags_25 },
    { 0x3bf904ed, SUC_lexicon_tags_35 },
    { 0xbcd904ef, SUC_lexicon_tags_37 },
    { 0xa5b904f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18f904f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889904f8, SUC_lexicon_tags_45 },
    { 0x6b5904f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9904fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x469904fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5190502, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7790506, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d99050a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f59050d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ad9050f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6c190511, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56790516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b90520, SUC_lexicon_tags_160 },
    { 0x9190521, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf90524, SUC_lexicon_tags_56 },
    { 0x18390525, SUC_lexicon_tags_25 },
    { 0x69390526, SUC_lexicon_tags_49 },
    { 0xbd390526, SUC_lexicon_tags_298 },
    { 0x59790525, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f90531, SUC_lexicon_tags_37 },
    { 0xbe390532, SUC_lexicon_tags_9 },
    { 0x8d390533, SUC_lexicon_tags_26 },
    { 0x74f90533, SUC_lexicon_tags_11 },
    { 0x54990534, SUC_lexicon_tags_14 },
    { 0xd1190536, SUC_lexicon_tags_14 },
    { 0xaef90537, SUC_lexicon_tags_12 },
    { 0x95f90534, SUC_lexicon_tags_49 },
    { 0xec390532, SUC_lexicon_tags_49 },
    { 0x15d90533, SUC_lexicon_tags_9 },
    { 0xe4f9053b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13190541, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f590543, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b590547, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec390549, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2bd9054b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d9054f, SUC_lexicon_tags_66 },
    { 0x4f99054f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39055b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xabd9055d, SUC_lexicon_tags_70 },
    { 0xe2b9055e, SUC_lexicon_tags_2 },
    { 0x8599055d, SUC_lexicon_tags_27 },
    { 0xf4990560, SUC_lexicon_tags_47 },
    { 0x30190561, SUC_lexicon_tags_11 },
    { 0x5219055d, SUC_lexicon_tags_23 },
    { 0xa4590563, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f90566, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f990569, SUC_lexicon_tags_1 },
    { 0xca390569, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f90570, SUC_lexicon_tags_9 },
    { 0xca590571, SUC_lexicon_tags_0 },
    { 0x50990572, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc2d90574, SUC_lexicon_tags_2 },
    { 0x44390575, SUC_lexicon_tags_47 },
    { 0x2b390574, SUC_lexicon_tags_12 },
    { 0x54d90577, SUC_lexicon_tags_15 },
    { 0x9b390578, SUC_lexicon_tags_37 },
    { 0xa6390577, SUC_lexicon_tags_1571 },
    { 0xbd990577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd90581, SUC_lexicon_tags_25 },
    { 0x89590582, SUC_lexicon_tags_17 },
    { 0xad90583, SUC_lexicon_tags_2 },
    { 0x43d90582, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b90588, SUC_lexicon_tags_65 },
    { 0x69390588, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa999058d, SUC_lexicon_tags_37 },
    { 0xf279058e, SUC_lexicon_tags_2 },
    { 0x2759058e, SUC_lexicon_tags_7 },
    { 0x6659058e, SUC_lexicon_tags_11 },
    { 0x5239058d, SUC_lexicon_tags_26 },
    { 0xfef90592, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xb0990594, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9790597, SUC_lexicon_tags_37 },
    { 0xc7590597, SUC_lexicon_tags_11 },
    { 0x98990598, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb905a1, SUC_lexicon_tags_37 },
    { 0xf2b905a2, SUC_lexicon_tags_43 },
    { 0x893905a3, SUC_lexicon_tags_6 },
    { 0x23f905a4, SUC_lexicon_tags_17 },
    { 0x1d5905a4, SUC_lexicon_tags_12 },
    { 0x5d9905a6, SUC_lexicon_tags_46 },
    { 0x55905a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9af905a9, SUC_lexicon_tags_34 },
    { 0x8c7905a9, SUC_lexicon_tags_65 },
    { 0xd87905ab, SUC_lexicon_tags_25 },
    { 0xacf905ac, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x81905ae, SUC_lexicon_tags_17 },
    { 0x581905af, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x387905b2, SUC_lexicon_tags_11 },
    { 0xb1d905b2, SUC_lexicon_tags_63 },
    { 0x22f905b3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1905b7, SUC_lexicon_tags_1 },
    { 0x28d905b7, SUC_lexicon_tags_33 },
    { 0xc15905b8, SUC_lexicon_tags_12 },
    { 0x2a5905ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd39905bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb37905be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe7905c0, SUC_lexicon_tags_7 },
    { 0xc23905c1, SUC_lexicon_tags_28 },
    { 0xab5905c0, SUC_lexicon_tags_62 },
    { 0x8b1905c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1905c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x527905c9, SUC_lexicon_tags_215 },
    { 0xa55905ca, SUC_lexicon_tags_17 },
    { 0xbf905cb, SUC_lexicon_tags_13 },
    { 0xa87905cb, SUC_lexicon_tags_33 },
    { 0x135905cc, SUC_lexicon_tags_1 },
    { 0x239905ce, SUC_lexicon_tags_15 },
    { 0x889905ce, SUC_lexicon_tags_10 },
    { 0x6fd905cf, SUC_lexicon_tags_28 },
    { 0x9d9905cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dd905d3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfb5905d5, SUC_lexicon_tags_79 },
    { 0x407905d6, SUC_lexicon_tags_46 },
    { 0xa2f905d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133905db, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x48d905dd, SUC_lexicon_tags_15 },
    { 0x21b905de, SUC_lexicon_tags_45 },
    { 0x393905df, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x109905e1, SUC_lexicon_tags_16 },
    { 0x8e9905e2, SUC_lexicon_tags_47 },
    { 0x5b7905e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65905e7, SUC_lexicon_tags_12 },
    { 0x22b905e8, SUC_lexicon_tags_15 },
    { 0x8ed905e9, SUC_lexicon_tags_57 },
    { 0x13d905ea, SUC_lexicon_tags_23 },
    { 0xa45905e8, SUC_lexicon_tags_68 },
    { 0xfcd905ec, SUC_lexicon_tags_11 },
    { 0x3d5905ed, SUC_lexicon_tags_103 },
    { 0x6cb905ec, SUC_lexicon_tags_60 },
    { 0x153905e9, SUC_lexicon_tags_47 },
    { 0xc5f905f0, SUC_lexicon_tags_1 },
    { 0x53f905f1, SUC_lexicon_tags_11 },
    { 0x5b3905f2, SUC_lexicon_tags_63 },
    { 0xff7905f3, SUC_lexicon_tags_27 },
    { 0x3cb905f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x267905f6, SUC_lexicon_tags_2 },
    { 0xa07905f6, SUC_lexicon_tags_1 },
    { 0x82d905f8, SUC_lexicon_tags_16 },
    { 0x7b9905f9, SUC_lexicon_tags_9 },
    { 0x14d905fa, SUC_lexicon_tags_11 },
    { 0xeaf905fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34790600, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7790604, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed790608, SUC_lexicon_tags_33 },
    { 0xa6590609, SUC_lexicon_tags_11 },
    { 0x190609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c590614, SUC_lexicon_tags_29 },
    { 0x94590614, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa139061a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f9061d, SUC_lexicon_tags_11 },
    { 0x3d79061d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62990622, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1390627, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x10d90629, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d9062e, SUC_lexicon_tags_11 },
    { 0x98b9062e, SUC_lexicon_tags_12 },
    { 0xcb90630, SUC_lexicon_tags_45 },
    { 0x33590631, SUC_lexicon_tags_37 },
    { 0xe339062f, SUC_lexicon_tags_8 },
    { 0xb6990633, SUC_lexicon_tags_27 },
    { 0xa9990634, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46590638, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd119063b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd79063d, SUC_lexicon_tags_28 },
    { 0x9ad9063e, SUC_lexicon_tags_39 },
    { 0xe79063e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63990644, SUC_lexicon_tags_108 },
    { 0xebb90645, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5790649, SUC_lexicon_tags_26 },
    { 0xfe190649, SUC_lexicon_tags_11 },
    { 0x439064b, SUC_lexicon_tags_33 },
    { 0xca39064c, SUC_lexicon_tags_49 },
    { 0xd0b9064a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93390651, SUC_lexicon_tags_33 },
    { 0xdb590652, SUC_lexicon_tags_28 },
    { 0x44f90652, SUC_lexicon_tags_87 },
    { 0xfe990654, SUC_lexicon_tags_27 },
    { 0x94d90651, SUC_lexicon_tags_4 },
    { 0x96790654, SUC_lexicon_tags_12 },
    { 0x9b990657, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7f390659, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc359065b, SUC_lexicon_tags_17 },
    { 0xa379065c, SUC_lexicon_tags_13 },
    { 0xf8b9065d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39790660, SUC_lexicon_tags_7 },
    { 0xd7d90660, SUC_lexicon_tags_17 },
    { 0x7f590660, SUC_lexicon_tags_47 },
    { 0x2d590663, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab99066e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc390671, SUC_lexicon_tags_11 },
    { 0x1ef90672, SUC_lexicon_tags_1 },
    { 0xb9b90672, SUC_lexicon_tags_49 },
    { 0x9d190674, SUC_lexicon_tags_0 },
    { 0x5db90675, SUC_lexicon_tags_25 },
    { 0x13d90676, SUC_lexicon_tags_11 },
    { 0x58f90676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb90680, SUC_lexicon_tags_1 },
    { 0xd6d90681, SUC_lexicon_tags_17 },
    { 0xd190681, SUC_lexicon_tags_12 },
    { 0xa2790683, SUC_lexicon_tags_47 },
    { 0xd4f90684, SUC_lexicon_tags_37 },
    { 0x49790680, SUC_lexicon_tags_14 },
    { 0x16190682, SUC_lexicon_tags_3 },
    { 0x5ff90687, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b9068a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8d9068c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa519068f, SUC_lexicon_tags_652 },
    { 0x6c79068f, SUC_lexicon_tags_63 },
    { 0xae590691, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d90694, SUC_lexicon_tags_45 },
    { 0x57d90695, SUC_lexicon_tags_45 },
    { 0xe7990694, SUC_lexicon_tags_14 },
    { 0xe4b90694, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af906a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b906aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3906b0, SUC_lexicon_tags_151 },
    { 0xb9d906b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x937906b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c1906b5, SUC_lexicon_tags_0 },
    { 0xd89906b6, SUC_lexicon_tags_11 },
    { 0xc15906b7, SUC_lexicon_tags_12 },
    { 0x1e3906b7, SUC_lexicon_tags_15 },
    { 0x8bf906b9, SUC_lexicon_tags_30 },
    { 0xd9d906b9, SUC_lexicon_tags_37 },
    { 0x53b906b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d906c3, SUC_lexicon_tags_57 },
    { 0x1ed906c4, SUC_lexicon_tags_25 },
    { 0x43906c3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe5d906c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae7906c9, SUC_lexicon_tags_85 },
    { 0x14d906c9, SUC_lexicon_tags_2 },
    { 0x115906c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1906cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5d9906d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2e3906d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e7906d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97906d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd906de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5906e4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f906e8, SUC_lexicon_tags_12 },
    { 0xe5f906e8, SUC_lexicon_tags_17 },
    { 0xdc5906ea, SUC_lexicon_tags_6 },
    { 0x6e7906e9, SUC_lexicon_tags_21 },
    { 0x337906ec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9eb906ee, SUC_lexicon_tags_2 },
    { 0x8d9906ee, SUC_lexicon_tags_33 },
    { 0x315906ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1906f8, SUC_lexicon_tags_17 },
    { 0x7ad906f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f906fe, SUC_lexicon_tags_37 },
    { 0xfdd906fe, SUC_lexicon_tags_7 },
    { 0x4f90700, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2a990702, SUC_lexicon_tags_17 },
    { 0xff90703, SUC_lexicon_tags_37 },
    { 0xd9d90704, SUC_lexicon_tags_48 },
    { 0xa4790704, SUC_lexicon_tags_26 },
    { 0x5af90706, SUC_lexicon_tags_454 },
    { 0x73790707, SUC_lexicon_tags_37 },
    { 0xc9990707, SUC_lexicon_tags_14 },
    { 0xd8190709, SUC_lexicon_tags_43 },
    { 0x25f9070a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b99070f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb90712, SUC_lexicon_tags_35 },
    { 0xb2b90712, SUC_lexicon_tags_35 },
    { 0x5e190713, SUC_lexicon_tags_17 },
    { 0x3eb90715, SUC_lexicon_tags_3 },
    { 0x60790713, SUC_lexicon_tags_8 },
    { 0x8e990713, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd99071a, SUC_lexicon_tags_27 },
    { 0x28b9071b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf90724, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc019072d, SUC_lexicon_tags_30 },
    { 0xcb99072e, SUC_lexicon_tags_54 },
    { 0x3879072f, SUC_lexicon_tags_26 },
    { 0x30990730, SUC_lexicon_tags_23 },
    { 0xc9d90731, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5790734, SUC_lexicon_tags_15 },
    { 0x41790735, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a390739, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8b9073b, SUC_lexicon_tags_84 },
    { 0x3c39073b, SUC_lexicon_tags_19 },
    { 0x2eb9073c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14790741, SUC_lexicon_tags_7 },
    { 0x5df90742, SUC_lexicon_tags_46 },
    { 0x90b90743, SUC_lexicon_tags_2 },
    { 0x6c390744, SUC_lexicon_tags_15 },
    { 0x2ad90744, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd39074c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9959074e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d90754, SUC_lexicon_tags_2 },
    { 0x62790755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a39075a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5590762, SUC_lexicon_tags_131 },
    { 0xe0d90762, SUC_lexicon_tags_17 },
    { 0x6cf90764, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b90769, SUC_lexicon_tags_0 },
    { 0xf8990769, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d390772, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f90777, SUC_lexicon_tags_7 },
    { 0xca390778, SUC_lexicon_tags_2 },
    { 0x19390778, SUC_lexicon_tags_25 },
    { 0x4219077a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa659077d, SUC_lexicon_tags_15 },
    { 0x3a59077e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5790784, SUC_lexicon_tags_43 },
    { 0x79d90785, SUC_lexicon_tags_11 },
    { 0xaed90786, SUC_lexicon_tags_17 },
    { 0xf190785, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7999078b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4919078f, SUC_lexicon_tags_34 },
    { 0xf1990790, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b90793, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa479079a, SUC_lexicon_tags_46 },
    { 0xa579079a, SUC_lexicon_tags_832 },
    { 0x2d59079b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1907a0, SUC_lexicon_tags_61 },
    { 0x735907a1, SUC_lexicon_tags_37 },
    { 0x9907a2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93907a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d907aa, SUC_lexicon_tags_0 },
    { 0xbbb907ab, SUC_lexicon_tags_60 },
    { 0xf8f907ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3907b0, SUC_lexicon_tags_37 },
    { 0x1ff907b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ab907b3, SUC_lexicon_tags_7 },
    { 0xb81907b4, SUC_lexicon_tags_27 },
    { 0x629907b5, SUC_lexicon_tags_11 },
    { 0x719907b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561907ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xad1907bc, SUC_lexicon_tags_57 },
    { 0x6e7907bd, SUC_lexicon_tags_65 },
    { 0xa09907bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37907c1, SUC_lexicon_tags_25 },
    { 0x93b907c2, SUC_lexicon_tags_12 },
    { 0x955907c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x865907c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45907cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7907d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5907d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f907da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d907e1, SUC_lexicon_tags_28 },
    { 0x28f907e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9b907e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45d907e6, SUC_lexicon_tags_29 },
    { 0xd41907e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1907ed, SUC_lexicon_tags_14 },
    { 0xa1f907ed, SUC_lexicon_tags_13 },
    { 0x263907ef, SUC_lexicon_tags_23 },
    { 0xda9907f0, SUC_lexicon_tags_1 },
    { 0x977907ed, SUC_lexicon_tags_19 },
    { 0x803907ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47907f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f907fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5b907fc, SUC_lexicon_tags_15 },
    { 0x3c3907fd, SUC_lexicon_tags_7 },
    { 0x549907fe, SUC_lexicon_tags_14 },
    { 0xccf907ff, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xea590801, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3cb90803, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9390806, SUC_lexicon_tags_25 },
    { 0x7eb90807, SUC_lexicon_tags_2 },
    { 0xa0d90808, SUC_lexicon_tags_4 },
    { 0x65790809, SUC_lexicon_tags_418 },
    { 0x21f9080a, SUC_lexicon_tags_43 },
    { 0xdf79080b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8790819, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1590824, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d990828, SUC_lexicon_tags_15 },
    { 0x1f590829, SUC_lexicon_tags_23 },
    { 0x4859082a, SUC_lexicon_tags_47 },
    { 0xe8f90829, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb9082f, SUC_lexicon_tags_43 },
    { 0x7cd90830, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x47b90832, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec59083a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3790841, SUC_lexicon_tags_47 },
    { 0x64b90842, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36390846, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62790851, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6390853, SUC_lexicon_tags_15 },
    { 0x8b590854, SUC_lexicon_tags_31 },
    { 0x9f790855, SUC_lexicon_tags_19 },
    { 0xf1b90856, SUC_lexicon_tags_7 },
    { 0x29190856, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d9085d, SUC_lexicon_tags_32 },
    { 0x7359085e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b90861, SUC_lexicon_tags_11 },
    { 0x1fb90861, SUC_lexicon_tags_25 },
    { 0xf5b90863, SUC_lexicon_tags_15 },
    { 0x9af90864, SUC_lexicon_tags_57 },
    { 0xb8f90865, SUC_lexicon_tags_17 },
    { 0x6bd90865, SUC_lexicon_tags_61 },
    { 0x36190865, SUC_lexicon_tags_6 },
    { 0xc3590866, SUC_lexicon_tags_15 },
    { 0x13b90864, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5499086d, SUC_lexicon_tags_1 },
    { 0x8399086d, SUC_lexicon_tags_144 },
    { 0xfbd9086f, SUC_lexicon_tags_45 },
    { 0x45390870, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd90873, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff590875, SUC_lexicon_tags_77 },
    { 0x3b390876, SUC_lexicon_tags_14 },
    { 0xb1f90875, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb639087a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78d9087c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b90883, SUC_lexicon_tags_25 },
    { 0xbf790883, SUC_lexicon_tags_26 },
    { 0x2b590885, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1019088a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c19088c, SUC_lexicon_tags_25 },
    { 0xa699088d, SUC_lexicon_tags_49 },
    { 0x9ad9088e, SUC_lexicon_tags_10 },
    { 0xbb79088f, SUC_lexicon_tags_1 },
    { 0x2c590890, SUC_lexicon_tags_15 },
    { 0xb90891, SUC_lexicon_tags_49 },
    { 0xb0f90891, SUC_lexicon_tags_11 },
    { 0xca79088d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaeb90895, SUC_lexicon_tags_10 },
    { 0xd9190896, SUC_lexicon_tags_1 },
    { 0xdcd90896, SUC_lexicon_tags_23 },
    { 0xb8b90898, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7179089b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875908a0, SUC_lexicon_tags_2 },
    { 0xb47908a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d908a8, SUC_lexicon_tags_11 },
    { 0xc9b908a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823908ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d908b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88f908b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227908bc, SUC_lexicon_tags_32 },
    { 0x471908bc, SUC_lexicon_tags_38 },
    { 0xc81908bc, SUC_lexicon_tags_9 },
    { 0x791908bf, SUC_lexicon_tags_7 },
    { 0x555908c0, SUC_lexicon_tags_7 },
    { 0xb13908c1, SUC_lexicon_tags_1 },
    { 0xfbd908c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d908c7, SUC_lexicon_tags_37 },
    { 0x291908c8, SUC_lexicon_tags_15 },
    { 0x443908c8, SUC_lexicon_tags_11 },
    { 0x75d908c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x25908cd, SUC_lexicon_tags_27 },
    { 0xcc3908ce, SUC_lexicon_tags_2 },
    { 0x1b7908cd, SUC_lexicon_tags_1 },
    { 0xbb5908d0, SUC_lexicon_tags_25 },
    { 0xb1908cf, SUC_lexicon_tags_37 },
    { 0x24b908d2, SUC_lexicon_tags_37 },
    { 0xbc3908ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d908d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07908db, SUC_lexicon_tags_4 },
    { 0x60d908db, SUC_lexicon_tags_17 },
    { 0x9908dd, SUC_lexicon_tags_2 },
    { 0x519908dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3eb908e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7908e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x447908e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7908e9, SUC_lexicon_tags_37 },
    { 0x5cb908ea, SUC_lexicon_tags_45 },
    { 0xa69908ea, SUC_lexicon_tags_49 },
    { 0xc09908e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x409908ef, SUC_lexicon_tags_25 },
    { 0x4fd908ef, SUC_lexicon_tags_17 },
    { 0xa0d908ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72d908f3, SUC_lexicon_tags_12 },
    { 0xe8b908f4, SUC_lexicon_tags_15 },
    { 0x6af908f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab90900, SUC_lexicon_tags_25 },
    { 0xcc190901, SUC_lexicon_tags_71 },
    { 0xd9790902, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2a590904, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb90908, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa239090a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1590910, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcbb90912, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x56590915, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7939091b, SUC_lexicon_tags_49 },
    { 0x7ad9091c, SUC_lexicon_tags_37 },
    { 0xf379091c, SUC_lexicon_tags_10 },
    { 0x70d9091e, SUC_lexicon_tags_19 },
    { 0x4f59091f, SUC_lexicon_tags_16 },
    { 0x81990920, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc590924, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b9092a, SUC_lexicon_tags_14 },
    { 0xc539092a, SUC_lexicon_tags_53 },
    { 0xf899092c, SUC_lexicon_tags_12 },
    { 0x1779092d, SUC_lexicon_tags_1064 },
    { 0, NULL },
    { 0x679092f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f190932, SUC_lexicon_tags_351 },
    { 0x4cd90933, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f90938, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b9093e, SUC_lexicon_tags_25 },
    { 0xfaf9093f, SUC_lexicon_tags_49 },
    { 0x7599093e, SUC_lexicon_tags_31 },
    { 0x7d90941, SUC_lexicon_tags_9 },
    { 0x52d90941, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db90945, SUC_lexicon_tags_7 },
    { 0x5990946, SUC_lexicon_tags_1 },
    { 0x8b790946, SUC_lexicon_tags_12 },
    { 0xe3990948, SUC_lexicon_tags_12 },
    { 0xb1790949, SUC_lexicon_tags_25 },
    { 0xfab9094a, SUC_lexicon_tags_15 },
    { 0xe99094a, SUC_lexicon_tags_47 },
    { 0x2259094c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c390950, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d90955, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1190958, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2d59095a, SUC_lexicon_tags_32 },
    { 0xf499095b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc39095d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94f9095f, SUC_lexicon_tags_11 },
    { 0x1a99095f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1a990962, SUC_lexicon_tags_68 },
    { 0x14990963, SUC_lexicon_tags_17 },
    { 0x9fd90964, SUC_lexicon_tags_1 },
    { 0x72590964, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x92990967, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf919096c, SUC_lexicon_tags_37 },
    { 0x9e19096c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d190974, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f59097c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb790981, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xce390984, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0790986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d9098e, SUC_lexicon_tags_56 },
    { 0x5979098f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca790991, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d90995, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x13790998, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2799099a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d9099e, SUC_lexicon_tags_38 },
    { 0x7cb9099f, SUC_lexicon_tags_3 },
    { 0x1d19099e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5909a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129909a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1909ae, SUC_lexicon_tags_17 },
    { 0x55d909af, SUC_lexicon_tags_13 },
    { 0x2dd909b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03909b3, SUC_lexicon_tags_29 },
    { 0x381909b4, SUC_lexicon_tags_134 },
    { 0xabf909b4, SUC_lexicon_tags_32 },
    { 0x63b909b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc15909b8, SUC_lexicon_tags_3 },
    { 0x3c5909b9, SUC_lexicon_tags_14 },
    { 0x62b909ba, SUC_lexicon_tags_10 },
    { 0x973909b9, SUC_lexicon_tags_174 },
    { 0xd97909b8, SUC_lexicon_tags_31 },
    { 0xdc5909bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc0f909bf, SUC_lexicon_tags_17 },
    { 0xb5b909bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f909c5, SUC_lexicon_tags_7 },
    { 0x3af909c6, SUC_lexicon_tags_33 },
    { 0xdb909c7, SUC_lexicon_tags_3 },
    { 0x155909c8, SUC_lexicon_tags_37 },
    { 0xd3b909c8, SUC_lexicon_tags_17 },
    { 0x663909c9, SUC_lexicon_tags_11 },
    { 0x901909cb, SUC_lexicon_tags_1 },
    { 0x60f909cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x861909ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31909d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b909dc, SUC_lexicon_tags_27 },
    { 0x9fd909dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97909df, SUC_lexicon_tags_7 },
    { 0x9b7909e0, SUC_lexicon_tags_45 },
    { 0xc2f909e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x205909e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b909e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x495909e9, SUC_lexicon_tags_30 },
    { 0x2ad909ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f909f1, SUC_lexicon_tags_45 },
    { 0x16d909f2, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7909fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x287909fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36590a01, SUC_lexicon_tags_1 },
    { 0x4e790a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38990a04, SUC_lexicon_tags_151 },
    { 0xde390a05, SUC_lexicon_tags_11 },
    { 0x71b90a05, SUC_lexicon_tags_11 },
    { 0xefb90a06, SUC_lexicon_tags_7 },
    { 0xcf390a05, SUC_lexicon_tags_12 },
    { 0x1a790a09, SUC_lexicon_tags_15 },
    { 0xb9d90a09, SUC_lexicon_tags_25 },
    { 0xd5590a0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad90a11, SUC_lexicon_tags_28 },
    { 0xe6390a12, SUC_lexicon_tags_17 },
    { 0xaeb90a12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f90a18, SUC_lexicon_tags_16 },
    { 0xba790a18, SUC_lexicon_tags_47 },
    { 0x79190a19, SUC_lexicon_tags_159 },
    { 0x3cd90a1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x74f90a1d, SUC_lexicon_tags_15 },
    { 0x11990a1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab190a35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62b90a37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b190a3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61590a44, SUC_lexicon_tags_59 },
    { 0xd9b90a45, SUC_lexicon_tags_26 },
    { 0x7bb90a44, SUC_lexicon_tags_47 },
    { 0x8990a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50390a4b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9f790a4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf990a50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e590a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab90a56, SUC_lexicon_tags_38 },
    { 0xafd90a57, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7390a5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43d90a61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69b90a63, SUC_lexicon_tags_4 },
    { 0x88790a64, SUC_lexicon_tags_7 },
    { 0xfe190a65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29790a6a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b990a6f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d90a74, SUC_lexicon_tags_45 },
    { 0x43190a74, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xde590a77, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9790a7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af90a7d, SUC_lexicon_tags_48 },
    { 0x46390a7e, SUC_lexicon_tags_33 },
    { 0x1ab90a7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f90a84, SUC_lexicon_tags_49 },
    { 0x5ff90a84, SUC_lexicon_tags_8 },
    { 0xa4d90a85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b190a8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f590a8c, SUC_lexicon_tags_63 },
    { 0x97b90a8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5790a8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaad90a91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf90a95, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x66d90a97, SUC_lexicon_tags_4 },
    { 0x1b590a98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12390a9a, SUC_lexicon_tags_37 },
    { 0xa7790a9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b790a9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0f90a9f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc790aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa790aa7, SUC_lexicon_tags_25 },
    { 0xa4990aa8, SUC_lexicon_tags_40 },
    { 0xd0f90aa9, SUC_lexicon_tags_12 },
    { 0x7fb90aa9, SUC_lexicon_tags_49 },
    { 0xfd90aab, SUC_lexicon_tags_37 },
    { 0xd3590aac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff990aae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2d90ab0, SUC_lexicon_tags_10 },
    { 0xcb190ab1, SUC_lexicon_tags_38 },
    { 0xe790ab2, SUC_lexicon_tags_7 },
    { 0x82b90ab3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x990ab7, SUC_lexicon_tags_7 },
    { 0x8ef90ab7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x66590aba, SUC_lexicon_tags_31 },
    { 0x4b190abb, SUC_lexicon_tags_23 },
    { 0x22990abc, SUC_lexicon_tags_28 },
    { 0x64190abb, SUC_lexicon_tags_14 },
    { 0x19990abb, SUC_lexicon_tags_48 },
    { 0x3d790abd, SUC_lexicon_tags_10 },
    { 0xf5d90ac0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3990ac3, SUC_lexicon_tags_87 },
    { 0x1e790ac4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf90aca, SUC_lexicon_tags_2 },
    { 0x62390aca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfbd90acd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db90ad0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xded90ad2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45d90ad4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc190ad6, SUC_lexicon_tags_1 },
    { 0xb1190ad7, SUC_lexicon_tags_2 },
    { 0x4c790ad6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd90adb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d90ae0, SUC_lexicon_tags_99 },
    { 0xd6f90ae0, SUC_lexicon_tags_15 },
    { 0x51590ae0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbed90ae8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52d90af1, SUC_lexicon_tags_17 },
    { 0xee190af2, SUC_lexicon_tags_3 },
    { 0xa9f90af3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ff90af5, SUC_lexicon_tags_27 },
    { 0x25790af6, SUC_lexicon_tags_47 },
    { 0x68b90af5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a190b00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48790b02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x67790b04, SUC_lexicon_tags_77 },
    { 0xbf590b04, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x83f90b07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f90b0a, SUC_lexicon_tags_25 },
    { 0x5d990b0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31390b0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32190b0f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x70190b11, SUC_lexicon_tags_14 },
    { 0x65b90b11, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d90b17, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e990b1d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa5190b1f, SUC_lexicon_tags_25 },
    { 0xff390b1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b90b23, SUC_lexicon_tags_25 },
    { 0x19f90b23, SUC_lexicon_tags_0 },
    { 0x62b90b24, SUC_lexicon_tags_11 },
    { 0xeef90b24, SUC_lexicon_tags_47 },
    { 0x9db90b27, SUC_lexicon_tags_23 },
    { 0x2d90b28, SUC_lexicon_tags_37 },
    { 0x34390b27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d190b33, SUC_lexicon_tags_1 },
    { 0x10f90b33, SUC_lexicon_tags_27 },
    { 0xc6b90b35, SUC_lexicon_tags_11 },
    { 0xd990b36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92790b39, SUC_lexicon_tags_17 },
    { 0x19190b3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e990b40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7790b4a, SUC_lexicon_tags_8 },
    { 0xed790b4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc190b53, SUC_lexicon_tags_17 },
    { 0x62190b54, SUC_lexicon_tags_31 },
    { 0x9af90b54, SUC_lexicon_tags_37 },
    { 0x35d90b53, SUC_lexicon_tags_25 },
    { 0x66590b55, SUC_lexicon_tags_7 },
    { 0x55990b58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d90b5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d90b69, SUC_lexicon_tags_618 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3390b6c, SUC_lexicon_tags_39 },
    { 0x33d90b6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50f90b6f, SUC_lexicon_tags_16 },
    { 0x8f590b70, SUC_lexicon_tags_29 },
    { 0x86790b71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd90b79, SUC_lexicon_tags_25 },
    { 0x16190b79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d90b7d, SUC_lexicon_tags_7 },
    { 0xdb590b7e, SUC_lexicon_tags_15 },
    { 0x19390b7e, SUC_lexicon_tags_11 },
    { 0x4c190b80, SUC_lexicon_tags_12 },
    { 0x89b90b81, SUC_lexicon_tags_37 },
    { 0xa3790b80, SUC_lexicon_tags_1 },
    { 0x5b590b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21190b85, SUC_lexicon_tags_87 },
    { 0x88790b86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44990b89, SUC_lexicon_tags_11 },
    { 0x58190b8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b90b8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x75190b8f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2790b93, SUC_lexicon_tags_4 },
    { 0xe9f90b94, SUC_lexicon_tags_2 },
    { 0x76f90b94, SUC_lexicon_tags_0 },
    { 0xb9990b93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74990b98, SUC_lexicon_tags_7 },
    { 0x94590b99, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x25190b9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ff90b9d, SUC_lexicon_tags_37 },
    { 0xd1990b9e, SUC_lexicon_tags_7 },
    { 0x4c590b9e, SUC_lexicon_tags_38 },
    { 0x75590ba0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28990ba5, SUC_lexicon_tags_21 },
    { 0x76d90ba5, SUC_lexicon_tags_60 },
    { 0x69590ba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50790ba9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7990bad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce190bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8390bba, SUC_lexicon_tags_77 },
    { 0xa4790bbb, SUC_lexicon_tags_25 },
    { 0xf9790bbc, SUC_lexicon_tags_60 },
    { 0x21590bbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68990bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4390bc5, SUC_lexicon_tags_35 },
    { 0x92990bc5, SUC_lexicon_tags_60 },
    { 0x28590bc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9590bcc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b90bcf, SUC_lexicon_tags_2 },
    { 0x17590bd0, SUC_lexicon_tags_11 },
    { 0xc2590bd1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa3790bd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25790bd5, SUC_lexicon_tags_4 },
    { 0x78d90bd6, SUC_lexicon_tags_27 },
    { 0x9d790bd7, SUC_lexicon_tags_7 },
    { 0xf2390bd7, SUC_lexicon_tags_56 },
    { 0xd0d90bd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12590bdb, SUC_lexicon_tags_38 },
    { 0xffd90bdc, SUC_lexicon_tags_0 },
    { 0xab590bdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b90be0, SUC_lexicon_tags_15 },
    { 0x17390be1, SUC_lexicon_tags_11 },
    { 0x62b90be2, SUC_lexicon_tags_1 },
    { 0xab190be3, SUC_lexicon_tags_4 },
    { 0x40790be4, SUC_lexicon_tags_25 },
    { 0x83390be4, SUC_lexicon_tags_20 },
    { 0xd3390be6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1590bec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x71590bee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x83d90bf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf90bf5, SUC_lexicon_tags_17 },
    { 0x77d90bf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f90bf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59790bfc, SUC_lexicon_tags_25 },
    { 0xcef90bfc, SUC_lexicon_tags_84 },
    { 0xb1990bfc, SUC_lexicon_tags_46 },
    { 0xad390bff, SUC_lexicon_tags_33 },
    { 0x68d90bfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21d90c02, SUC_lexicon_tags_37 },
    { 0x18f90c02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x24b90c05, SUC_lexicon_tags_17 },
    { 0x2ef90c05, SUC_lexicon_tags_1 },
    { 0xcbb90c07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20790c0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b90c10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38190c13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b90c17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde790c1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc790c1e, SUC_lexicon_tags_60 },
    { 0x70990c1e, SUC_lexicon_tags_12 },
    { 0x3d790c20, SUC_lexicon_tags_35 },
    { 0x9d390c21, SUC_lexicon_tags_10 },
    { 0x25590c20, SUC_lexicon_tags_27 },
    { 0xb8590c20, SUC_lexicon_tags_28 },
    { 0x81190c23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c790c26, SUC_lexicon_tags_63 },
    { 0x61390c27, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b90c2c, SUC_lexicon_tags_0 },
    { 0xa3390c2c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db90c31, SUC_lexicon_tags_62 },
    { 0x79190c32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4790c34, SUC_lexicon_tags_2 },
    { 0x9d390c35, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d90c38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78190c3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x42590c3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8d90c41, SUC_lexicon_tags_19 },
    { 0xd3990c41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0590c47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4990c49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2db90c4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x60990c4e, SUC_lexicon_tags_12 },
    { 0x70390c4f, SUC_lexicon_tags_38 },
    { 0x5f390c4f, SUC_lexicon_tags_37 },
    { 0xf8790c50, SUC_lexicon_tags_11 },
    { 0x30790c52, SUC_lexicon_tags_30 },
    { 0x27f90c52, SUC_lexicon_tags_37 },
    { 0x2f190c4f, SUC_lexicon_tags_19 },
    { 0xf3990c52, SUC_lexicon_tags_12 },
    { 0x12990c56, SUC_lexicon_tags_43 },
    { 0x4f990c57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x11b90c59, SUC_lexicon_tags_46 },
    { 0xa9390c5a, SUC_lexicon_tags_25 },
    { 0xa2d90c59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8790c61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14190c67, SUC_lexicon_tags_12 },
    { 0x21b90c67, SUC_lexicon_tags_14 },
    { 0x18790c68, SUC_lexicon_tags_15 },
    { 0xcc990c6a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x31190c6c, SUC_lexicon_tags_47 },
    { 0xdfd90c6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1590c74, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d90c82, SUC_lexicon_tags_37 },
    { 0xeff90c83, SUC_lexicon_tags_30 },
    { 0xfc990c84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79790c8c, SUC_lexicon_tags_27 },
    { 0x7db90c8c, SUC_lexicon_tags_46 },
    { 0xb5790c8d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x92790c90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49590c95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe9990c97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c990c99, SUC_lexicon_tags_2 },
    { 0x7bf90c9a, SUC_lexicon_tags_2 },
    { 0xf9190c9b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa190c9e, SUC_lexicon_tags_1 },
    { 0xaef90c9f, SUC_lexicon_tags_37 },
    { 0xa6790c9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79990ca4, SUC_lexicon_tags_63 },
    { 0x88790ca4, SUC_lexicon_tags_95 },
    { 0xf3790ca5, SUC_lexicon_tags_37 },
    { 0x8a590ca4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74990cb5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad790cba, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8590cc2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd4d90cc4, SUC_lexicon_tags_23 },
    { 0xfd990cc5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f390cc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc990ccb, SUC_lexicon_tags_11 },
    { 0xb9d90ccc, SUC_lexicon_tags_0 },
    { 0xdd990ccd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1390cd3, SUC_lexicon_tags_27 },
    { 0x2990cd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad590cda, SUC_lexicon_tags_15 },
    { 0xd9590cdb, SUC_lexicon_tags_46 },
    { 0xa2990cdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6790cde, SUC_lexicon_tags_27 },
    { 0x6b90cdf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d90ce3, SUC_lexicon_tags_23 },
    { 0x5ef90ce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a190cee, SUC_lexicon_tags_37 },
    { 0xf2190cef, SUC_lexicon_tags_25 },
    { 0x4bf90cef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92d90cf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdf90cf4, SUC_lexicon_tags_7 },
    { 0x1b390cf4, SUC_lexicon_tags_13 },
    { 0x69b90cf6, SUC_lexicon_tags_14 },
    { 0xb0b90cf6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x70590cf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b90d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0b90d02, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x55590d04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4390d07, SUC_lexicon_tags_63 },
    { 0x93790d08, SUC_lexicon_tags_2 },
    { 0x6b390d09, SUC_lexicon_tags_33 },
    { 0xa4990d0a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8390d0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3190d11, SUC_lexicon_tags_37 },
    { 0xb6f90d12, SUC_lexicon_tags_27 },
    { 0x66b90d13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x23990d15, SUC_lexicon_tags_26 },
    { 0x9e990d15, SUC_lexicon_tags_31 },
    { 0x35990d16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1cd90d19, SUC_lexicon_tags_21 },
    { 0x48f90d19, SUC_lexicon_tags_14 },
    { 0x12390d1b, SUC_lexicon_tags_66 },
    { 0x1db90d1c, SUC_lexicon_tags_38 },
    { 0xf5790d1c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdcf90d1f, SUC_lexicon_tags_60 },
    { 0x45f90d1f, SUC_lexicon_tags_33 },
    { 0xbfb90d21, SUC_lexicon_tags_45 },
    { 0xd8190d21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd190d25, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xce390d27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d190d29, SUC_lexicon_tags_66 },
    { 0xc0590d2a, SUC_lexicon_tags_11 },
    { 0xc590d2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xddf90d2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57190d31, SUC_lexicon_tags_37 },
    { 0x29d90d31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79590d3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe790d3d, SUC_lexicon_tags_25 },
    { 0x5c990d3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x69f90d40, SUC_lexicon_tags_28 },
    { 0xf5590d40, SUC_lexicon_tags_25 },
    { 0xf190d42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d90d47, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a390d4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7190d4c, SUC_lexicon_tags_63 },
    { 0x7b190d4c, SUC_lexicon_tags_17 },
    { 0x26190d4e, SUC_lexicon_tags_7 },
    { 0x42390d4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda790d52, SUC_lexicon_tags_84 },
    { 0x7c590d52, SUC_lexicon_tags_15 },
    { 0x8ed90d54, SUC_lexicon_tags_2 },
    { 0x8b90d55, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf90d5c, SUC_lexicon_tags_5 },
    { 0x51d90d5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a990d62, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xba990d65, SUC_lexicon_tags_45 },
    { 0xa2590d66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f90d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb90d6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4db90d76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x29590d79, SUC_lexicon_tags_64 },
    { 0x79f90d7a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53190d7c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x82590d7e, SUC_lexicon_tags_15 },
    { 0x78f90d7f, SUC_lexicon_tags_3 },
    { 0xdcb90d80, SUC_lexicon_tags_1 },
    { 0xed190d7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8df90d83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d90d8c, SUC_lexicon_tags_17 },
    { 0x3f790d8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1d190d8f, SUC_lexicon_tags_27 },
    { 0xee390d90, SUC_lexicon_tags_37 },
    { 0x9e590d91, SUC_lexicon_tags_0 },
    { 0xed990d92, SUC_lexicon_tags_0 },
    { 0x94d90d93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb90d9a, SUC_lexicon_tags_49 },
    { 0xa9f90d9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5d90d9d, SUC_lexicon_tags_17 },
    { 0xa8d90d9e, SUC_lexicon_tags_23 },
    { 0x55790d9f, SUC_lexicon_tags_30 },
    { 0xef990da0, SUC_lexicon_tags_61 },
    { 0x97590d9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x52d90da3, SUC_lexicon_tags_27 },
    { 0xbb990da4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5990da6, SUC_lexicon_tags_7 },
    { 0x99390da7, SUC_lexicon_tags_15 },
    { 0x45190da8, SUC_lexicon_tags_15 },
    { 0xd2990da9, SUC_lexicon_tags_60 },
    { 0x64390daa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d90dae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd390dbc, SUC_lexicon_tags_45 },
    { 0xa8790dbd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa5590dbf, SUC_lexicon_tags_12 },
    { 0x38190dc0, SUC_lexicon_tags_11 },
    { 0xc6790dc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41590dc3, SUC_lexicon_tags_29 },
    { 0x15390dc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44590dc6, SUC_lexicon_tags_45 },
    { 0xd5590dc6, SUC_lexicon_tags_49 },
    { 0x2cb90dc8, SUC_lexicon_tags_0 },
    { 0xe6990dc9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x30390dcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49790dcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d90dd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77990ddc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8390ddf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59590de3, SUC_lexicon_tags_17 },
    { 0xa8590de4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd390de9, SUC_lexicon_tags_23 },
    { 0x4df90de9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bb90dec, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d90df0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d90df8, SUC_lexicon_tags_17 },
    { 0xb6f90df8, SUC_lexicon_tags_23 },
    { 0x3c190df9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41590e01, SUC_lexicon_tags_66 },
    { 0xf90e02, SUC_lexicon_tags_15 },
    { 0x56990e02, SUC_lexicon_tags_11 },
    { 0xbb390e04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad90e07, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x24590e09, SUC_lexicon_tags_37 },
    { 0x30990e09, SUC_lexicon_tags_14 },
    { 0x1f990e0a, SUC_lexicon_tags_55 },
    { 0x71590e0a, SUC_lexicon_tags_45 },
    { 0xc1990e0d, SUC_lexicon_tags_18 },
    { 0xad390e0e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25190e15, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69390e1a, SUC_lexicon_tags_4 },
    { 0x8e990e1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f90e2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab90e30, SUC_lexicon_tags_61 },
    { 0x95390e31, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x98190e34, SUC_lexicon_tags_60 },
    { 0x45590e34, SUC_lexicon_tags_37 },
    { 0x34590e36, SUC_lexicon_tags_1 },
    { 0x41190e37, SUC_lexicon_tags_26 },
    { 0xf0f90e37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc390e3a, SUC_lexicon_tags_68 },
    { 0x4990e3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x57990e3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92990e44, SUC_lexicon_tags_11 },
    { 0xbb390e45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb90e48, SUC_lexicon_tags_11 },
    { 0x81990e48, SUC_lexicon_tags_17 },
    { 0x98d90e48, SUC_lexicon_tags_27 },
    { 0xcb990e4b, SUC_lexicon_tags_17 },
    { 0x6d790e4c, SUC_lexicon_tags_27 },
    { 0x90790e4c, SUC_lexicon_tags_11 },
    { 0x1d390e4d, SUC_lexicon_tags_23 },
    { 0xc4990e4f, SUC_lexicon_tags_103 },
    { 0x15990e50, SUC_lexicon_tags_45 },
    { 0x20190e51, SUC_lexicon_tags_65 },
    { 0xe1b90e52, SUC_lexicon_tags_9 },
    { 0xaef90e52, SUC_lexicon_tags_87 },
    { 0x27d90e54, SUC_lexicon_tags_13 },
    { 0xfbb90e55, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0190e58, SUC_lexicon_tags_47 },
    { 0xeb790e58, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x39790e5b, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x74190e5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x36590e5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x54590e62, SUC_lexicon_tags_49 },
    { 0xe2190e62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72190e66, SUC_lexicon_tags_37 },
    { 0xb1f90e67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b90e6d, SUC_lexicon_tags_15 },
    { 0x5d990e6d, SUC_lexicon_tags_11 },
    { 0x1e790e6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc4f90e71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f90e75, SUC_lexicon_tags_8 },
    { 0x79190e75, SUC_lexicon_tags_38 },
    { 0x87390e77, SUC_lexicon_tags_60 },
    { 0x4990e78, SUC_lexicon_tags_0 },
    { 0x1ed90e76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4590e83, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x87990e86, SUC_lexicon_tags_138 },
    { 0xf2190e87, SUC_lexicon_tags_16 },
    { 0x80390e87, SUC_lexicon_tags_15 },
    { 0xb2990e89, SUC_lexicon_tags_26 },
    { 0xd0590e8a, SUC_lexicon_tags_37 },
    { 0x9b990e8b, SUC_lexicon_tags_33 },
    { 0xebd90e8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba590e8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdd590e90, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x47b90e92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92d90e9c, SUC_lexicon_tags_19 },
    { 0xae190e9d, SUC_lexicon_tags_17 },
    { 0xd390e9d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b90ea2, SUC_lexicon_tags_10 },
    { 0xc7990ea2, SUC_lexicon_tags_25 },
    { 0xbb190ea2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8390ea8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57790eae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34390eb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68990eb3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f90eb6, SUC_lexicon_tags_0 },
    { 0xe2f90eb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb90eb9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4990ebe, SUC_lexicon_tags_27 },
    { 0xdb590ebe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1190ec4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8190ecc, SUC_lexicon_tags_1 },
    { 0x20190ecd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb190ed0, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b90ed4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x20790ed6, SUC_lexicon_tags_1 },
    { 0xe3b90ed7, SUC_lexicon_tags_12 },
    { 0xc5990ed7, SUC_lexicon_tags_12 },
    { 0xba190ed8, SUC_lexicon_tags_12 },
    { 0x4790eda, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x46390edc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45390ee1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2990ee4, SUC_lexicon_tags_12 },
    { 0xf4f90ee4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4e990ee7, SUC_lexicon_tags_8 },
    { 0x17590ee7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad90eec, SUC_lexicon_tags_37 },
    { 0x43f90eed, SUC_lexicon_tags_79 },
    { 0x70190eee, SUC_lexicon_tags_15 },
    { 0x65990eef, SUC_lexicon_tags_18 },
    { 0x60d90ef0, SUC_lexicon_tags_107 },
    { 0xbf590eef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6790ef5, SUC_lexicon_tags_17 },
    { 0xd7590ef5, SUC_lexicon_tags_28 },
    { 0x14d90ef6, SUC_lexicon_tags_14 },
    { 0x50190ef8, SUC_lexicon_tags_2 },
    { 0x92190ef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83390f06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e790f08, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2b90f0a, SUC_lexicon_tags_15 },
    { 0x7790f0b, SUC_lexicon_tags_60 },
    { 0x9c990f0c, SUC_lexicon_tags_31 },
    { 0x3f590f0b, SUC_lexicon_tags_12 },
    { 0x55190f0e, SUC_lexicon_tags_24 },
    { 0xa9590f0f, SUC_lexicon_tags_19 },
    { 0x11990f0f, SUC_lexicon_tags_28 },
    { 0x84790f0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d90f15, SUC_lexicon_tags_25 },
    { 0xfbb90f16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x85f90f18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53d90f1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36b90f1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40990f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78b90f21, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x54190f23, SUC_lexicon_tags_17 },
    { 0xb8f90f23, SUC_lexicon_tags_25 },
    { 0x16390f25, SUC_lexicon_tags_2 },
    { 0x190f25, SUC_lexicon_tags_2 },
    { 0xbdf90f27, SUC_lexicon_tags_11 },
    { 0x8e990f28, SUC_lexicon_tags_29 },
    { 0x1bd90f28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37f90f2b, SUC_lexicon_tags_25 },
    { 0x70b90f2c, SUC_lexicon_tags_43 },
    { 0x32b90f2d, SUC_lexicon_tags_37 },
    { 0xb2190f2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b790f31, SUC_lexicon_tags_38 },
    { 0xbcd90f32, SUC_lexicon_tags_11 },
    { 0xa7590f33, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea190f3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe590f40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33590f45, SUC_lexicon_tags_11 },
    { 0xc5190f45, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a590f52, SUC_lexicon_tags_45 },
    { 0xd0f90f53, SUC_lexicon_tags_33 },
    { 0x8b390f54, SUC_lexicon_tags_9 },
    { 0x2ab90f55, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79590f5a, SUC_lexicon_tags_66 },
    { 0x37390f5b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f990f5f, SUC_lexicon_tags_19 },
    { 0xb9990f60, SUC_lexicon_tags_29 },
    { 0x93190f60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa990f64, SUC_lexicon_tags_14 },
    { 0x93190f64, SUC_lexicon_tags_17 },
    { 0x68f90f65, SUC_lexicon_tags_14 },
    { 0xf4990f67, SUC_lexicon_tags_25 },
    { 0x8e190f68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d90f6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x78f90f6d, SUC_lexicon_tags_25 },
    { 0x77990f6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5990f71, SUC_lexicon_tags_3 },
    { 0x72390f72, SUC_lexicon_tags_37 },
    { 0xe6190f73, SUC_lexicon_tags_11 },
    { 0x19590f73, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86790f78, SUC_lexicon_tags_38 },
    { 0xa1990f79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee390f80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef90f84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4b590f86, SUC_lexicon_tags_11 },
    { 0x11d90f86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb90f8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f990f92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4d90f94, SUC_lexicon_tags_15 },
    { 0x8e390f95, SUC_lexicon_tags_17 },
    { 0x24f90f95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f990f99, SUC_lexicon_tags_2 },
    { 0xd3990f99, SUC_lexicon_tags_15 },
    { 0x6b790f9a, SUC_lexicon_tags_54 },
    { 0x23790f9c, SUC_lexicon_tags_45 },
    { 0xfed90f9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee990fa6, SUC_lexicon_tags_28 },
    { 0x81d90fa6, SUC_lexicon_tags_27 },
    { 0xf7f90fa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa190fab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1390faf, SUC_lexicon_tags_25 },
    { 0x9eb90fb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a990fb5, SUC_lexicon_tags_235 },
    { 0x20190fb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb990fb8, SUC_lexicon_tags_11 },
    { 0x62190fb9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb8990fbb, SUC_lexicon_tags_7 },
    { 0x3b990fbc, SUC_lexicon_tags_7 },
    { 0xdaf90fbc, SUC_lexicon_tags_11 },
    { 0x43d90fbc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc4990fc0, SUC_lexicon_tags_30 },
    { 0xf3b90fc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb90fc4, SUC_lexicon_tags_14 },
    { 0x77790fc4, SUC_lexicon_tags_79 },
    { 0x47b90fc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2190fca, SUC_lexicon_tags_2 },
    { 0x7d590fcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd90fce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b590fd1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e190fd5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x39990fd8, SUC_lexicon_tags_7 },
    { 0xef990fd8, SUC_lexicon_tags_33 },
    { 0x3590fda, SUC_lexicon_tags_0 },
    { 0x3cf90fdb, SUC_lexicon_tags_11 },
    { 0x58190fdc, SUC_lexicon_tags_17 },
    { 0x61790fd8, SUC_lexicon_tags_60 },
    { 0x91190fdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6390fe0, SUC_lexicon_tags_2 },
    { 0x52390fe1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5a790fe3, SUC_lexicon_tags_60 },
    { 0x73b90fe4, SUC_lexicon_tags_9 },
    { 0x4e590fe3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff90fe8, SUC_lexicon_tags_37 },
    { 0x79790fe9, SUC_lexicon_tags_13 },
    { 0x2e990fea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58d90fec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1d90fee, SUC_lexicon_tags_12 },
    { 0x73b90fef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x58390ff1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9790ff4, SUC_lexicon_tags_8 },
    { 0x8190ff4, SUC_lexicon_tags_1598 },
    { 0x6fd90ff4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf90ff9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff90ffc, SUC_lexicon_tags_43 },
    { 0x7ed90ffd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xddb90fff, SUC_lexicon_tags_25 },
    { 0x9b91000, SUC_lexicon_tags_33 },
    { 0x50991001, SUC_lexicon_tags_7 },
    { 0xe9b91000, SUC_lexicon_tags_37 },
    { 0x2b991003, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82b91006, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c19100c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79791010, SUC_lexicon_tags_12 },
    { 0x95391011, SUC_lexicon_tags_6 },
    { 0xa9391010, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f91015, SUC_lexicon_tags_30 },
    { 0x42991016, SUC_lexicon_tags_11 },
    { 0x68b91015, SUC_lexicon_tags_46 },
    { 0xc4d91018, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f9101e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb91023, SUC_lexicon_tags_2 },
    { 0x13d91024, SUC_lexicon_tags_11 },
    { 0xd0391023, SUC_lexicon_tags_2 },
    { 0xb9391024, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c591028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4239102c, SUC_lexicon_tags_15 },
    { 0xd519102d, SUC_lexicon_tags_14 },
    { 0x8459102e, SUC_lexicon_tags_56 },
    { 0xde79102f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c591033, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83991044, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78191047, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b9104a, SUC_lexicon_tags_1 },
    { 0x87b9104b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa299104e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x62991051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d9105b, SUC_lexicon_tags_7 },
    { 0xea79105b, SUC_lexicon_tags_4 },
    { 0x9c99105d, SUC_lexicon_tags_28 },
    { 0xff99105e, SUC_lexicon_tags_2 },
    { 0x58d9105d, SUC_lexicon_tags_14 },
    { 0xa1591060, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f91064, SUC_lexicon_tags_39 },
    { 0x45591064, SUC_lexicon_tags_63 },
    { 0xd7b91065, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x99b91068, SUC_lexicon_tags_8 },
    { 0xe2b91069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbd19106b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ff9106d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf879106f, SUC_lexicon_tags_10 },
    { 0x7eb91070, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d91073, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2bd91075, SUC_lexicon_tags_15 },
    { 0xcf791076, SUC_lexicon_tags_33 },
    { 0xd1b91076, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9107a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc991081, SUC_lexicon_tags_19 },
    { 0xe7b91082, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6991084, SUC_lexicon_tags_25 },
    { 0x8bb91085, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79108b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6499108d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63f9108f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf91093, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32591098, SUC_lexicon_tags_11 },
    { 0x86d91099, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9999109c, SUC_lexicon_tags_271 },
    { 0, NULL },
    { 0x71d9109e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x981910a1, SUC_lexicon_tags_37 },
    { 0xaed910a2, SUC_lexicon_tags_15 },
    { 0xca1910a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc69910a5, SUC_lexicon_tags_31 },
    { 0x631910a6, SUC_lexicon_tags_12 },
    { 0x1c9910a7, SUC_lexicon_tags_17 },
    { 0x569910a5, SUC_lexicon_tags_34 },
    { 0x985910a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d910ae, SUC_lexicon_tags_160 },
    { 0x621910af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x755910b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d910b4, SUC_lexicon_tags_7 },
    { 0xb35910b5, SUC_lexicon_tags_25 },
    { 0x9ff910b5, SUC_lexicon_tags_45 },
    { 0xf91910b7, SUC_lexicon_tags_12 },
    { 0x3e3910b8, SUC_lexicon_tags_37 },
    { 0xeed910b7, SUC_lexicon_tags_2 },
    { 0x4b3910b5, SUC_lexicon_tags_14 },
    { 0x759910ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987910bf, SUC_lexicon_tags_57 },
    { 0x48b910c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781910c4, SUC_lexicon_tags_4 },
    { 0x157910c4, SUC_lexicon_tags_2 },
    { 0x5bb910c4, SUC_lexicon_tags_27 },
    { 0x509910c4, SUC_lexicon_tags_63 },
    { 0xa49910c7, SUC_lexicon_tags_66 },
    { 0xa1b910c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcf7910cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x303910ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf910d0, SUC_lexicon_tags_19 },
    { 0xb87910d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x623910d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xad1910d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f910da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb910df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6d910e1, SUC_lexicon_tags_11 },
    { 0x989910e1, SUC_lexicon_tags_2 },
    { 0x297910e1, SUC_lexicon_tags_21 },
    { 0xa41910e4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x829910e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75910e9, SUC_lexicon_tags_0 },
    { 0x891910e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x817910ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac7910ef, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701910f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df910f7, SUC_lexicon_tags_11 },
    { 0x275910f8, SUC_lexicon_tags_23 },
    { 0xab1910f7, SUC_lexicon_tags_19 },
    { 0x571910fa, SUC_lexicon_tags_37 },
    { 0x6d3910fb, SUC_lexicon_tags_82 },
    { 0x985910fc, SUC_lexicon_tags_17 },
    { 0xfff910fc, SUC_lexicon_tags_55 },
    { 0xcdd910fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57191100, SUC_lexicon_tags_21 },
    { 0xb3591101, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66d91103, SUC_lexicon_tags_25 },
    { 0x46d91104, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b191107, SUC_lexicon_tags_46 },
    { 0x36191107, SUC_lexicon_tags_7 },
    { 0xb9d91107, SUC_lexicon_tags_11 },
    { 0xf9b91108, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b9110d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d991117, SUC_lexicon_tags_17 },
    { 0xd5191118, SUC_lexicon_tags_3 },
    { 0xe6191118, SUC_lexicon_tags_25 },
    { 0x81d91118, SUC_lexicon_tags_11 },
    { 0x9279111b, SUC_lexicon_tags_17 },
    { 0xf6f9111c, SUC_lexicon_tags_17 },
    { 0xd8791117, SUC_lexicon_tags_11 },
    { 0xa719111e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8591126, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb91129, SUC_lexicon_tags_15 },
    { 0x2a79112a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f91130, SUC_lexicon_tags_11 },
    { 0xa6791131, SUC_lexicon_tags_7 },
    { 0xa7b91132, SUC_lexicon_tags_28 },
    { 0x65791133, SUC_lexicon_tags_23 },
    { 0xbb591134, SUC_lexicon_tags_1 },
    { 0xf391135, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d91138, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb899113b, SUC_lexicon_tags_7 },
    { 0x9c79113b, SUC_lexicon_tags_11 },
    { 0xef99113b, SUC_lexicon_tags_17 },
    { 0x13d9113c, SUC_lexicon_tags_1 },
    { 0x4a39113f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13791141, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd91144, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d91148, SUC_lexicon_tags_12 },
    { 0x77791149, SUC_lexicon_tags_12 },
    { 0x2b59114a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3879114e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x94591151, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df91154, SUC_lexicon_tags_38 },
    { 0xaab91155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc391157, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f9115b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f91165, SUC_lexicon_tags_14 },
    { 0x54591166, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db9116a, SUC_lexicon_tags_87 },
    { 0xcf9116b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8d9116d, SUC_lexicon_tags_37 },
    { 0x71b9116e, SUC_lexicon_tags_15 },
    { 0xbdd9116f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x991171, SUC_lexicon_tags_15 },
    { 0x7fb91172, SUC_lexicon_tags_62 },
    { 0x14991173, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7191175, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x83391177, SUC_lexicon_tags_100 },
    { 0xc3191178, SUC_lexicon_tags_37 },
    { 0xb4391177, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde19117d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7779117f, SUC_lexicon_tags_52 },
    { 0xb3991180, SUC_lexicon_tags_15 },
    { 0xd159117f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd1591183, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d191186, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb2591188, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e79118a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2699118e, SUC_lexicon_tags_19 },
    { 0xfcf9118f, SUC_lexicon_tags_21 },
    { 0x91d91190, SUC_lexicon_tags_30 },
    { 0xe8d91191, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12f91193, SUC_lexicon_tags_9 },
    { 0xa5591194, SUC_lexicon_tags_23 },
    { 0x5ff91194, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc6591197, SUC_lexicon_tags_25 },
    { 0x44d91198, SUC_lexicon_tags_31 },
    { 0x76591199, SUC_lexicon_tags_85 },
    { 0x3191198, SUC_lexicon_tags_2 },
    { 0xe791199, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6539119d, SUC_lexicon_tags_16 },
    { 0x3f19119e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b911a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x217911a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b911a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b911ac, SUC_lexicon_tags_1 },
    { 0xa7911ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321911b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x103911b3, SUC_lexicon_tags_21 },
    { 0xfbd911b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1911b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421911bc, SUC_lexicon_tags_77 },
    { 0x179911bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc57911bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53911c2, SUC_lexicon_tags_55 },
    { 0x1b1911c2, SUC_lexicon_tags_33 },
    { 0xb79911c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf911c7, SUC_lexicon_tags_9 },
    { 0x903911c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd87911ca, SUC_lexicon_tags_17 },
    { 0x16b911cb, SUC_lexicon_tags_49 },
    { 0xb5b911ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x267911ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1911d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1911d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd8f911d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x26f911db, SUC_lexicon_tags_6 },
    { 0x6f5911dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b911e0, SUC_lexicon_tags_26 },
    { 0x537911e1, SUC_lexicon_tags_17 },
    { 0x8bb911e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ef911e4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf6d911e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3911e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef9911ea, SUC_lexicon_tags_39 },
    { 0x911911ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3911ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355911f3, SUC_lexicon_tags_25 },
    { 0xa1b911f4, SUC_lexicon_tags_5 },
    { 0xd63911f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x531911f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdfb911f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x267911fc, SUC_lexicon_tags_25 },
    { 0x497911fc, SUC_lexicon_tags_2 },
    { 0xb9d911fc, SUC_lexicon_tags_17 },
    { 0x1ab911fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb91203, SUC_lexicon_tags_0 },
    { 0x4e191204, SUC_lexicon_tags_10 },
    { 0x8f591204, SUC_lexicon_tags_15 },
    { 0x6ed91205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b9120a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b9120d, SUC_lexicon_tags_34 },
    { 0x28d9120e, SUC_lexicon_tags_25 },
    { 0x58f9120f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b991214, SUC_lexicon_tags_47 },
    { 0x4191215, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f91219, SUC_lexicon_tags_12 },
    { 0xf6b91219, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x10b9121c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ab9121e, SUC_lexicon_tags_14 },
    { 0x3619121f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2791221, SUC_lexicon_tags_15 },
    { 0xccb91222, SUC_lexicon_tags_38 },
    { 0x92791223, SUC_lexicon_tags_95 },
    { 0x20391224, SUC_lexicon_tags_31 },
    { 0xb1f91223, SUC_lexicon_tags_11 },
    { 0x7e391224, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b9122b, SUC_lexicon_tags_2 },
    { 0xbc59122c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2af9122e, SUC_lexicon_tags_17 },
    { 0x45b9122e, SUC_lexicon_tags_11 },
    { 0x43791230, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65791238, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e19123b, SUC_lexicon_tags_11 },
    { 0xe839123b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b9123e, SUC_lexicon_tags_25 },
    { 0xa679123e, SUC_lexicon_tags_11 },
    { 0x86191240, SUC_lexicon_tags_47 },
    { 0x2e391241, SUC_lexicon_tags_29 },
    { 0x30d91242, SUC_lexicon_tags_19 },
    { 0xe4991242, SUC_lexicon_tags_15 },
    { 0xf3791244, SUC_lexicon_tags_63 },
    { 0xd0791245, SUC_lexicon_tags_39 },
    { 0x83191245, SUC_lexicon_tags_39 },
    { 0xbd391247, SUC_lexicon_tags_61 },
    { 0xf5791245, SUC_lexicon_tags_33 },
    { 0xdaf91249, SUC_lexicon_tags_8 },
    { 0xb9191244, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a79124d, SUC_lexicon_tags_55 },
    { 0xd239124d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe591251, SUC_lexicon_tags_19 },
    { 0x37991252, SUC_lexicon_tags_1 },
    { 0x9c391252, SUC_lexicon_tags_25 },
    { 0x3df91254, SUC_lexicon_tags_24 },
    { 0x12b91255, SUC_lexicon_tags_7 },
    { 0xc591252, SUC_lexicon_tags_27 },
    { 0x25b91251, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xfab91259, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xce99125b, SUC_lexicon_tags_21 },
    { 0xdff9125c, SUC_lexicon_tags_17 },
    { 0x61b9125d, SUC_lexicon_tags_35 },
    { 0x99d9125d, SUC_lexicon_tags_18 },
    { 0xb9f9125d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d791262, SUC_lexicon_tags_68 },
    { 0xf3b91263, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f91266, SUC_lexicon_tags_15 },
    { 0xc5591267, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9579126a, SUC_lexicon_tags_37 },
    { 0x5f99126b, SUC_lexicon_tags_1 },
    { 0x8699126c, SUC_lexicon_tags_4 },
    { 0xf999126d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4991270, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3191274, SUC_lexicon_tags_12 },
    { 0xe9191275, SUC_lexicon_tags_38 },
    { 0x93591274, SUC_lexicon_tags_35 },
    { 0xaed91277, SUC_lexicon_tags_25 },
    { 0x74d91277, SUC_lexicon_tags_60 },
    { 0xc5d91279, SUC_lexicon_tags_29 },
    { 0xc0991279, SUC_lexicon_tags_29 },
    { 0x599127a, SUC_lexicon_tags_1 },
    { 0x38591279, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd9127f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7391281, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43991286, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x24f91288, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb539128b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f59128f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c591291, SUC_lexicon_tags_33 },
    { 0x9b791292, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfd591294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7191297, SUC_lexicon_tags_45 },
    { 0x91191298, SUC_lexicon_tags_63 },
    { 0x13b91297, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb559129b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6659129d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453912a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9912a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf912a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32b912aa, SUC_lexicon_tags_48 },
    { 0x27d912ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d912b1, SUC_lexicon_tags_29 },
    { 0xba7912b1, SUC_lexicon_tags_12 },
    { 0x3f3912b3, SUC_lexicon_tags_15 },
    { 0x187912b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91d912b6, SUC_lexicon_tags_47 },
    { 0x73f912b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1912b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3b912bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71912c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59912c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfbd912c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x483912c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5912d5, SUC_lexicon_tags_37 },
    { 0x22d912d6, SUC_lexicon_tags_25 },
    { 0x2ab912d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567912db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9912e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b912e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7912e8, SUC_lexicon_tags_12 },
    { 0xf9d912e8, SUC_lexicon_tags_60 },
    { 0x473912ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3912ef, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x76d912f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf3912f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd912f9, SUC_lexicon_tags_0 },
    { 0xa7d912fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec7912fc, SUC_lexicon_tags_17 },
    { 0x507912fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5b912ff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde991303, SUC_lexicon_tags_47 },
    { 0xa5791304, SUC_lexicon_tags_59 },
    { 0x99591305, SUC_lexicon_tags_27 },
    { 0xbab91306, SUC_lexicon_tags_31 },
    { 0xffb91307, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6139130c, SUC_lexicon_tags_17 },
    { 0x46b9130c, SUC_lexicon_tags_31 },
    { 0x6eb9130e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6ab91310, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf91313, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f191316, SUC_lexicon_tags_17 },
    { 0x9ad91317, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b9131a, SUC_lexicon_tags_43 },
    { 0xc3d9131b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b9131e, SUC_lexicon_tags_21 },
    { 0x6f9131e, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa591323, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf91327, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf479132a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x55d9132c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d59132f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc791333, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x73191336, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e59133a, SUC_lexicon_tags_14 },
    { 0xf399133a, SUC_lexicon_tags_19 },
    { 0x71b9133a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6991341, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf91344, SUC_lexicon_tags_16 },
    { 0x63f91345, SUC_lexicon_tags_7 },
    { 0x56191346, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0x70b91349, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6791352, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x47d91354, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x63f91356, SUC_lexicon_tags_15 },
    { 0x7f91357, SUC_lexicon_tags_12 },
    { 0x17391357, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb319135b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f91364, SUC_lexicon_tags_2 },
    { 0xe191365, SUC_lexicon_tags_14 },
    { 0xb9d91366, SUC_lexicon_tags_37 },
    { 0xab791365, SUC_lexicon_tags_37 },
    { 0x7a791368, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf9136b, SUC_lexicon_tags_11 },
    { 0x8b39136b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6419136f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef9137b, SUC_lexicon_tags_6 },
    { 0xc3b9137b, SUC_lexicon_tags_25 },
    { 0x459137d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53991381, SUC_lexicon_tags_12 },
    { 0x4d91381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49591385, SUC_lexicon_tags_37 },
    { 0xdf991385, SUC_lexicon_tags_45 },
    { 0xfd591387, SUC_lexicon_tags_14 },
    { 0xef991387, SUC_lexicon_tags_47 },
    { 0xd3791387, SUC_lexicon_tags_23 },
    { 0x1259138a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0f9138c, SUC_lexicon_tags_25 },
    { 0xbb9138c, SUC_lexicon_tags_12 },
    { 0xb839138e, SUC_lexicon_tags_0 },
    { 0xf5d9138f, SUC_lexicon_tags_33 },
    { 0xa5d9138e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5379139b, SUC_lexicon_tags_11 },
    { 0xb519139c, SUC_lexicon_tags_38 },
    { 0x8239139d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699913a3, SUC_lexicon_tags_12 },
    { 0x9c5913a4, SUC_lexicon_tags_38 },
    { 0x477913a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf913a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3913ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x937913b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd913b3, SUC_lexicon_tags_33 },
    { 0x4cf913b3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95913b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae5913b9, SUC_lexicon_tags_10 },
    { 0x5a9913ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3913c4, SUC_lexicon_tags_0 },
    { 0x42d913c5, SUC_lexicon_tags_11 },
    { 0xf25913c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0b913c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71913cf, SUC_lexicon_tags_43 },
    { 0xf47913cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299913d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3913de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493913e4, SUC_lexicon_tags_49 },
    { 0x10d913e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79913e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245913f0, SUC_lexicon_tags_18 },
    { 0x81d913f0, SUC_lexicon_tags_32 },
    { 0xfe9913f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82f913f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d913f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe25913f9, SUC_lexicon_tags_37 },
    { 0x227913f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1d913fc, SUC_lexicon_tags_28 },
    { 0xa6b913fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f3913ff, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f91406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca59140a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5139140d, SUC_lexicon_tags_15 },
    { 0x2799140e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d9141a, SUC_lexicon_tags_10 },
    { 0xe59141b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d19141e, SUC_lexicon_tags_0 },
    { 0x3199141f, SUC_lexicon_tags_23 },
    { 0x8c391420, SUC_lexicon_tags_1 },
    { 0x4939141e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d91426, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d791428, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3199142d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd91431, SUC_lexicon_tags_46 },
    { 0x33d91431, SUC_lexicon_tags_1 },
    { 0x83b91433, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81191437, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a991443, SUC_lexicon_tags_7 },
    { 0x3c191444, SUC_lexicon_tags_0 },
    { 0xf6b91445, SUC_lexicon_tags_5 },
    { 0xb4591443, SUC_lexicon_tags_60 },
    { 0x73191447, SUC_lexicon_tags_45 },
    { 0xcc591447, SUC_lexicon_tags_8 },
    { 0x85d91449, SUC_lexicon_tags_43 },
    { 0x24791448, SUC_lexicon_tags_25 },
    { 0xa199144b, SUC_lexicon_tags_37 },
    { 0x8b39144c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb91450, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71591454, SUC_lexicon_tags_17 },
    { 0x7d791454, SUC_lexicon_tags_25 },
    { 0x39791456, SUC_lexicon_tags_47 },
    { 0xe4591457, SUC_lexicon_tags_19 },
    { 0x3ad91458, SUC_lexicon_tags_7 },
    { 0x6ff91459, SUC_lexicon_tags_27 },
    { 0xb5391456, SUC_lexicon_tags_47 },
    { 0xfa59145b, SUC_lexicon_tags_11 },
    { 0xd739145c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11591461, SUC_lexicon_tags_7 },
    { 0x11f91461, SUC_lexicon_tags_14 },
    { 0xc8b91463, SUC_lexicon_tags_11 },
    { 0xbd991464, SUC_lexicon_tags_10 },
    { 0xba391465, SUC_lexicon_tags_38 },
    { 0x7df91463, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a591469, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9179146f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13791471, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdfb91473, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57591477, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15191483, SUC_lexicon_tags_25 },
    { 0xc1b91484, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3591488, SUC_lexicon_tags_6 },
    { 0xfad91488, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd9148c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6759148e, SUC_lexicon_tags_37 },
    { 0x4b99148f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e991492, SUC_lexicon_tags_2 },
    { 0x9e991492, SUC_lexicon_tags_0 },
    { 0x41791494, SUC_lexicon_tags_33 },
    { 0x48d91495, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f9149b, SUC_lexicon_tags_26 },
    { 0x1359149c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45d9149e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81914aa, SUC_lexicon_tags_21 },
    { 0x341914ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237914b2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x79f914b4, SUC_lexicon_tags_69 },
    { 0x267914b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df914b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df914bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1914c7, SUC_lexicon_tags_15 },
    { 0xc9b914c7, SUC_lexicon_tags_12 },
    { 0x121914c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad914ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57914d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x227914d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5914dd, SUC_lexicon_tags_6 },
    { 0xa65914de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9eb914e0, SUC_lexicon_tags_4 },
    { 0xbf5914e0, SUC_lexicon_tags_11 },
    { 0x4d7914e2, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x883914e5, SUC_lexicon_tags_17 },
    { 0x637914e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315914ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59d914ee, SUC_lexicon_tags_60 },
    { 0xe03914ee, SUC_lexicon_tags_14 },
    { 0xf11914f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9914f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153914f8, SUC_lexicon_tags_1 },
    { 0x7fb914f9, SUC_lexicon_tags_10 },
    { 0xb6d914fa, SUC_lexicon_tags_17 },
    { 0x321914fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x195914fe, SUC_lexicon_tags_2 },
    { 0x143914fe, SUC_lexicon_tags_10 },
    { 0xb1191500, SUC_lexicon_tags_49 },
    { 0xfdf914ff, SUC_lexicon_tags_30 },
    { 0xba991502, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71191506, SUC_lexicon_tags_49 },
    { 0x53791507, SUC_lexicon_tags_31 },
    { 0xb7791508, SUC_lexicon_tags_11 },
    { 0x45791509, SUC_lexicon_tags_7 },
    { 0xc959150a, SUC_lexicon_tags_2 },
    { 0x44d9150b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e99150d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1991513, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xce591515, SUC_lexicon_tags_17 },
    { 0xfc191516, SUC_lexicon_tags_43 },
    { 0x72791515, SUC_lexicon_tags_37 },
    { 0x92191518, SUC_lexicon_tags_0 },
    { 0xe2791515, SUC_lexicon_tags_2 },
    { 0x1391518, SUC_lexicon_tags_2 },
    { 0x7839151b, SUC_lexicon_tags_10 },
    { 0xdef91517, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc559151f, SUC_lexicon_tags_1 },
    { 0x24f91520, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b91524, SUC_lexicon_tags_12 },
    { 0xf4991525, SUC_lexicon_tags_11 },
    { 0xe0b91526, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b9152a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc019152e, SUC_lexicon_tags_17 },
    { 0x2cf9152e, SUC_lexicon_tags_15 },
    { 0x4b99152f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf6b91532, SUC_lexicon_tags_177 },
    { 0xebb91533, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a191535, SUC_lexicon_tags_49 },
    { 0xd6191536, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x12d91538, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2b79153a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9153d, SUC_lexicon_tags_6 },
    { 0x899153d, SUC_lexicon_tags_37 },
    { 0x32f9153f, SUC_lexicon_tags_25 },
    { 0x2d191540, SUC_lexicon_tags_3 },
    { 0xe7b9153e, SUC_lexicon_tags_19 },
    { 0xaf39153e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5991545, SUC_lexicon_tags_2 },
    { 0x84391545, SUC_lexicon_tags_12 },
    { 0xbcf91547, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62191551, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61391554, SUC_lexicon_tags_33 },
    { 0xef191555, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99b91557, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7919155c, SUC_lexicon_tags_1 },
    { 0xd839155d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df91564, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f191568, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c191575, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75591582, SUC_lexicon_tags_47 },
    { 0xf0d91583, SUC_lexicon_tags_15 },
    { 0x4ed91584, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b59158b, SUC_lexicon_tags_131 },
    { 0x6bd9158b, SUC_lexicon_tags_37 },
    { 0x1079158b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4991590, SUC_lexicon_tags_25 },
    { 0x31191591, SUC_lexicon_tags_43 },
    { 0x7f991592, SUC_lexicon_tags_33 },
    { 0x30191591, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd791595, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d91598, SUC_lexicon_tags_11 },
    { 0xba791598, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f9159e, SUC_lexicon_tags_47 },
    { 0x7179159f, SUC_lexicon_tags_49 },
    { 0x551915a0, SUC_lexicon_tags_47 },
    { 0xeab915a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x14d915a3, SUC_lexicon_tags_47 },
    { 0xab5915a4, SUC_lexicon_tags_2 },
    { 0x299915a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x863915a7, SUC_lexicon_tags_11 },
    { 0xb01915a8, SUC_lexicon_tags_14 },
    { 0xd15915a8, SUC_lexicon_tags_47 },
    { 0x36b915a8, SUC_lexicon_tags_26 },
    { 0xe9d915ab, SUC_lexicon_tags_2 },
    { 0xd79915a7, SUC_lexicon_tags_16 },
    { 0x511915ad, SUC_lexicon_tags_17 },
    { 0x681915ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3915b1, SUC_lexicon_tags_47 },
    { 0x737915b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x12b915b4, SUC_lexicon_tags_0 },
    { 0xba5915b4, SUC_lexicon_tags_103 },
    { 0x3db915b6, SUC_lexicon_tags_11 },
    { 0x273915b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x877915b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x369915bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb915c2, SUC_lexicon_tags_14 },
    { 0xf7f915c3, SUC_lexicon_tags_170 },
    { 0x319915c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f915cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fd915cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af915d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd73915d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ad915d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9915dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5b1915de, SUC_lexicon_tags_2 },
    { 0xcbd915de, SUC_lexicon_tags_9 },
    { 0xb79915e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1915e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd67915e6, SUC_lexicon_tags_49 },
    { 0x57b915e6, SUC_lexicon_tags_12 },
    { 0xe1d915e8, SUC_lexicon_tags_23 },
    { 0x77f915e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b915f2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1915f5, SUC_lexicon_tags_79 },
    { 0xf3915f6, SUC_lexicon_tags_25 },
    { 0x6f1915f6, SUC_lexicon_tags_49 },
    { 0xd7b915f5, SUC_lexicon_tags_17 },
    { 0x3b1915f6, SUC_lexicon_tags_47 },
    { 0xe27915fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d915fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b91603, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5d791605, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1991607, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab9160a, SUC_lexicon_tags_7 },
    { 0xed19160b, SUC_lexicon_tags_48 },
    { 0x80d9160b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d91611, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b91619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe139161c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c191621, SUC_lexicon_tags_7 },
    { 0x5bd91621, SUC_lexicon_tags_20 },
    { 0x13991622, SUC_lexicon_tags_37 },
    { 0xe791621, SUC_lexicon_tags_19 },
    { 0x93f91624, SUC_lexicon_tags_17 },
    { 0x2f191621, SUC_lexicon_tags_0 },
    { 0x7c791627, SUC_lexicon_tags_25 },
    { 0xcb391628, SUC_lexicon_tags_17 },
    { 0x22191627, SUC_lexicon_tags_27 },
    { 0x8591625, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb91632, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0391637, SUC_lexicon_tags_12 },
    { 0x50191638, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8679163c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b91640, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xda191643, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd91648, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb39164b, SUC_lexicon_tags_15 },
    { 0xe5b9164c, SUC_lexicon_tags_63 },
    { 0x5479164c, SUC_lexicon_tags_8 },
    { 0x5519164c, SUC_lexicon_tags_13 },
    { 0xbcb9164f, SUC_lexicon_tags_0 },
    { 0xd819164c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f591652, SUC_lexicon_tags_1 },
    { 0x6a191653, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0xc5191655, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb9165a, SUC_lexicon_tags_60 },
    { 0xe39165b, SUC_lexicon_tags_34 },
    { 0x3dd9165c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf7f9165e, SUC_lexicon_tags_2 },
    { 0xb2d9165e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x12791662, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbd91664, SUC_lexicon_tags_38 },
    { 0x2ff91664, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4df91667, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xea591669, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fd9166b, SUC_lexicon_tags_7 },
    { 0xa719166c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9ad9166e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d91671, SUC_lexicon_tags_2 },
    { 0xdbf91671, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce391676, SUC_lexicon_tags_84 },
    { 0x9ed91677, SUC_lexicon_tags_11 },
    { 0x45991676, SUC_lexicon_tags_63 },
    { 0xa7791679, SUC_lexicon_tags_51 },
    { 0xf0b91679, SUC_lexicon_tags_77 },
    { 0x7439167b, SUC_lexicon_tags_43 },
    { 0x95f9167c, SUC_lexicon_tags_37 },
    { 0x7099167d, SUC_lexicon_tags_68 },
    { 0x8dd9167d, SUC_lexicon_tags_27 },
    { 0x8b19167d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6191685, SUC_lexicon_tags_1 },
    { 0xe0d91685, SUC_lexicon_tags_82 },
    { 0x1fb91687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f9168a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d39168c, SUC_lexicon_tags_37 },
    { 0xcad9168c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad91691, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x16191693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f91697, SUC_lexicon_tags_39 },
    { 0x15f91697, SUC_lexicon_tags_607 },
    { 0xed191699, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f9169c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7916a1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xce9916a3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x539916a5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x161916a8, SUC_lexicon_tags_38 },
    { 0xc05916a8, SUC_lexicon_tags_8 },
    { 0xa3916aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x11d916ac, SUC_lexicon_tags_11 },
    { 0x533916ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a9916af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9916b2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcd3916b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x99b916b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d916bb, SUC_lexicon_tags_17 },
    { 0x1a9916bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61d916be, SUC_lexicon_tags_23 },
    { 0x285916be, SUC_lexicon_tags_29 },
    { 0xadb916c0, SUC_lexicon_tags_11 },
    { 0xc01916c1, SUC_lexicon_tags_63 },
    { 0x97b916c2, SUC_lexicon_tags_50 },
    { 0xe59916c3, SUC_lexicon_tags_25 },
    { 0xf49916c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f916c8, SUC_lexicon_tags_9 },
    { 0xbd5916c8, SUC_lexicon_tags_47 },
    { 0x445916c9, SUC_lexicon_tags_17 },
    { 0xb2b916cb, SUC_lexicon_tags_7 },
    { 0x3b916cc, SUC_lexicon_tags_15 },
    { 0x6f9916cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3916d5, SUC_lexicon_tags_11 },
    { 0xafb916d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda3916d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5916dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5916e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297916e7, SUC_lexicon_tags_32 },
    { 0x541916e8, SUC_lexicon_tags_84 },
    { 0x44f916e9, SUC_lexicon_tags_2 },
    { 0x77b916ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b916ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11916f4, SUC_lexicon_tags_1 },
    { 0x6a7916f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13916fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf91702, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef91706, SUC_lexicon_tags_15 },
    { 0xa6d91706, SUC_lexicon_tags_3 },
    { 0xf6f91708, SUC_lexicon_tags_57 },
    { 0x42b91709, SUC_lexicon_tags_4 },
    { 0x1a19170a, SUC_lexicon_tags_68 },
    { 0x81d91709, SUC_lexicon_tags_1 },
    { 0xbfd91708, SUC_lexicon_tags_2 },
    { 0x8399170d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6c99170f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb91713, SUC_lexicon_tags_37 },
    { 0xe0391714, SUC_lexicon_tags_37 },
    { 0xc7f91715, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6791719, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a39171e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d391722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58d91724, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0191726, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b9172b, SUC_lexicon_tags_7 },
    { 0x7b59172c, SUC_lexicon_tags_38 },
    { 0xd919172d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97391732, SUC_lexicon_tags_25 },
    { 0x94191733, SUC_lexicon_tags_15 },
    { 0xed791734, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6991738, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc459173b, SUC_lexicon_tags_11 },
    { 0x6a79173c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc919173e, SUC_lexicon_tags_49 },
    { 0x9a39173f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef91749, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d91751, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x90591753, SUC_lexicon_tags_10 },
    { 0x11f91753, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff91757, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7059175b, SUC_lexicon_tags_28 },
    { 0xe5d9175c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1391761, SUC_lexicon_tags_37 },
    { 0x7191761, SUC_lexicon_tags_17 },
    { 0xf5791763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa391765, SUC_lexicon_tags_17 },
    { 0xdcd91765, SUC_lexicon_tags_87 },
    { 0xe3991767, SUC_lexicon_tags_25 },
    { 0xa3191767, SUC_lexicon_tags_37 },
    { 0x1e391766, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbad9176b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf2f9176d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b91770, SUC_lexicon_tags_0 },
    { 0xeb791770, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f91778, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6479177c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a79177e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13191780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77d91783, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x26591786, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9799178c, SUC_lexicon_tags_57 },
    { 0xf339178c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93991797, SUC_lexicon_tags_30 },
    { 0x7e991798, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb779179f, SUC_lexicon_tags_46 },
    { 0x9b1917a0, SUC_lexicon_tags_37 },
    { 0x6ed917a0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb917a4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa2d917a6, SUC_lexicon_tags_9 },
    { 0x3fd917a6, SUC_lexicon_tags_48 },
    { 0x81d917a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95917ac, SUC_lexicon_tags_17 },
    { 0xb51917ad, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x84d917b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33917b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461917bc, SUC_lexicon_tags_27 },
    { 0x30d917bd, SUC_lexicon_tags_8 },
    { 0x6f5917be, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87917c1, SUC_lexicon_tags_11 },
    { 0x901917c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb3917c4, SUC_lexicon_tags_63 },
    { 0x5c9917c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x143917c7, SUC_lexicon_tags_70 },
    { 0x897917c8, SUC_lexicon_tags_38 },
    { 0xd73917c9, SUC_lexicon_tags_63 },
    { 0xe85917c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7b917cc, SUC_lexicon_tags_27 },
    { 0x695917cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d917d2, SUC_lexicon_tags_37 },
    { 0x567917d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4af917d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb21917d7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31917db, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf917e1, SUC_lexicon_tags_25 },
    { 0x589917e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x671917e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xffd917e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489917ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b917ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x19b917f1, SUC_lexicon_tags_12 },
    { 0xa43917f2, SUC_lexicon_tags_15 },
    { 0x319917f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245917f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137917fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf791801, SUC_lexicon_tags_141 },
    { 0x20791802, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38191807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecb91809, SUC_lexicon_tags_43 },
    { 0x60791809, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae99180f, SUC_lexicon_tags_0 },
    { 0x9e59180f, SUC_lexicon_tags_12 },
    { 0xb6d9180f, SUC_lexicon_tags_12 },
    { 0x84f91811, SUC_lexicon_tags_26 },
    { 0x1859180f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x88d91816, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98191818, SUC_lexicon_tags_19 },
    { 0xfa991818, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffb9181b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb979181d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a191826, SUC_lexicon_tags_25 },
    { 0x5b191826, SUC_lexicon_tags_38 },
    { 0x49191827, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b9182f, SUC_lexicon_tags_84 },
    { 0x33d91830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fb91832, SUC_lexicon_tags_37 },
    { 0xed191833, SUC_lexicon_tags_2 },
    { 0xdc791834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd591836, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2039183d, SUC_lexicon_tags_23 },
    { 0x3f39183e, SUC_lexicon_tags_15 },
    { 0xa3f9183d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7391844, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xae191846, SUC_lexicon_tags_37 },
    { 0x99f91847, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f9184a, SUC_lexicon_tags_49 },
    { 0x1df9184a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51791851, SUC_lexicon_tags_15 },
    { 0xb9391852, SUC_lexicon_tags_0 },
    { 0x96391853, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d91856, SUC_lexicon_tags_7 },
    { 0x1e391857, SUC_lexicon_tags_7 },
    { 0xa9b91856, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf339185e, SUC_lexicon_tags_11 },
    { 0xd2b9185f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df91866, SUC_lexicon_tags_12 },
    { 0xb9791867, SUC_lexicon_tags_142 },
    { 0xf2391866, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8991870, SUC_lexicon_tags_38 },
    { 0xed591870, SUC_lexicon_tags_28 },
    { 0x99991872, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b91876, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b9187b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xac39187d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfa79187f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2991885, SUC_lexicon_tags_25 },
    { 0x72991886, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5991889, SUC_lexicon_tags_2 },
    { 0x2b59188a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db9188d, SUC_lexicon_tags_11 },
    { 0x9f99188d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1591896, SUC_lexicon_tags_214 },
    { 0x54991897, SUC_lexicon_tags_2 },
    { 0xf5b91898, SUC_lexicon_tags_2 },
    { 0x59b91898, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9189d, SUC_lexicon_tags_10 },
    { 0x3dd9189e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x349918a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x397918a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf918ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d918b1, SUC_lexicon_tags_2 },
    { 0x415918b2, SUC_lexicon_tags_84 },
    { 0xb3d918b3, SUC_lexicon_tags_0 },
    { 0x507918b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab7918b6, SUC_lexicon_tags_84 },
    { 0x6eb918b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7918bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d918c0, SUC_lexicon_tags_11 },
    { 0x535918c0, SUC_lexicon_tags_11 },
    { 0x6dd918c2, SUC_lexicon_tags_6 },
    { 0x839918c0, SUC_lexicon_tags_2 },
    { 0x9f5918c4, SUC_lexicon_tags_37 },
    { 0xcdb918c4, SUC_lexicon_tags_11 },
    { 0x95b918c5, SUC_lexicon_tags_11 },
    { 0x68d918c4, SUC_lexicon_tags_25 },
    { 0x9e9918c2, SUC_lexicon_tags_12 },
    { 0x4a9918c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ab918cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801918cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab918d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01918d9, SUC_lexicon_tags_63 },
    { 0x50b918da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f918e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x987918e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37918f0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661918f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1918fc, SUC_lexicon_tags_63 },
    { 0xd83918fd, SUC_lexicon_tags_15 },
    { 0xfcd918fe, SUC_lexicon_tags_11 },
    { 0x6dd918ff, SUC_lexicon_tags_133 },
    { 0x361918fe, SUC_lexicon_tags_32 },
    { 0x6a7918fe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc1f91903, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d91906, SUC_lexicon_tags_2 },
    { 0xe6391907, SUC_lexicon_tags_12 },
    { 0xe3591906, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d9190a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d91913, SUC_lexicon_tags_77 },
    { 0x6d791914, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83791918, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x97b9191a, SUC_lexicon_tags_17 },
    { 0x6779191b, SUC_lexicon_tags_27 },
    { 0x4279191c, SUC_lexicon_tags_25 },
    { 0xb239191c, SUC_lexicon_tags_48 },
    { 0xb939191e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacf91920, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9f91922, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf591926, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d9192c, SUC_lexicon_tags_0 },
    { 0xa419192d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c191930, SUC_lexicon_tags_0 },
    { 0x91391931, SUC_lexicon_tags_151 },
    { 0x47791932, SUC_lexicon_tags_68 },
    { 0xc7391931, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a191935, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e59193a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb879193f, SUC_lexicon_tags_17 },
    { 0xe0d91940, SUC_lexicon_tags_25 },
    { 0x51591941, SUC_lexicon_tags_25 },
    { 0xa9b91942, SUC_lexicon_tags_30 },
    { 0x40991943, SUC_lexicon_tags_25 },
    { 0x6c591940, SUC_lexicon_tags_32 },
    { 0xa6191945, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x75991947, SUC_lexicon_tags_55 },
    { 0xcd791948, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf479194b, SUC_lexicon_tags_463 },
    { 0x5039194b, SUC_lexicon_tags_2 },
    { 0x86d9194b, SUC_lexicon_tags_43 },
    { 0x3f19194c, SUC_lexicon_tags_63 },
    { 0xb0d9194e, SUC_lexicon_tags_17 },
    { 0xe099194b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8dd91952, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bd91954, SUC_lexicon_tags_6 },
    { 0x39591955, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a391959, SUC_lexicon_tags_15 },
    { 0x48d91959, SUC_lexicon_tags_15 },
    { 0x18d9195b, SUC_lexicon_tags_11 },
    { 0x65b9195b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d391960, SUC_lexicon_tags_862 },
    { 0xe4591960, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2991963, SUC_lexicon_tags_10 },
    { 0xd3d91963, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2791967, SUC_lexicon_tags_5 },
    { 0xf4591968, SUC_lexicon_tags_17 },
    { 0xdc791967, SUC_lexicon_tags_0 },
    { 0xd59196a, SUC_lexicon_tags_7 },
    { 0xad79196b, SUC_lexicon_tags_7 },
    { 0xe8d9196c, SUC_lexicon_tags_1 },
    { 0x2659196b, SUC_lexicon_tags_15 },
    { 0xf1d9196c, SUC_lexicon_tags_55 },
    { 0x73391969, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb91978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6179197c, SUC_lexicon_tags_28 },
    { 0x8819197d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e991980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20791983, SUC_lexicon_tags_31 },
    { 0x98391984, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f791987, SUC_lexicon_tags_63 },
    { 0x17391987, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f79198b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a391991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca991993, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbcf91995, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b9199c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2559199e, SUC_lexicon_tags_0 },
    { 0xe199199e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39919a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99919ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9919b0, SUC_lexicon_tags_9 },
    { 0x6b3919b1, SUC_lexicon_tags_12 },
    { 0x3ed919b0, SUC_lexicon_tags_37 },
    { 0x177919b3, SUC_lexicon_tags_39 },
    { 0xc33919b0, SUC_lexicon_tags_1 },
    { 0xccb919b5, SUC_lexicon_tags_2 },
    { 0xf6b919b6, SUC_lexicon_tags_1 },
    { 0x4a1919b7, SUC_lexicon_tags_29 },
    { 0x71f919b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x16b919ba, SUC_lexicon_tags_60 },
    { 0xcd3919ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xabd919bd, SUC_lexicon_tags_1 },
    { 0xe0b919be, SUC_lexicon_tags_63 },
    { 0x5e5919bd, SUC_lexicon_tags_19 },
    { 0xf8f919c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x447919c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b1919c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5919c8, SUC_lexicon_tags_15 },
    { 0x433919c9, SUC_lexicon_tags_88 },
    { 0xa07919c9, SUC_lexicon_tags_32 },
    { 0x511919cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb919d2, SUC_lexicon_tags_15 },
    { 0xd61919d3, SUC_lexicon_tags_26 },
    { 0xcd3919d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x735919d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x683919d8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7919dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd919e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x609919e2, SUC_lexicon_tags_0 },
    { 0x5db919e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f919e8, SUC_lexicon_tags_43 },
    { 0x563919e9, SUC_lexicon_tags_47 },
    { 0x94f919ea, SUC_lexicon_tags_37 },
    { 0x46f919eb, SUC_lexicon_tags_13 },
    { 0x347919ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07919ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395919f4, SUC_lexicon_tags_21 },
    { 0x50b919f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x523919f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435919fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d91a03, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c191a07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f191a09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28391a0d, SUC_lexicon_tags_25 },
    { 0xc1f91a0e, SUC_lexicon_tags_17 },
    { 0x68591a0f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc991a13, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8791a16, SUC_lexicon_tags_63 },
    { 0xe9191a17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d91a1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4ab91a1c, SUC_lexicon_tags_43 },
    { 0xa9b91a1d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x23191a20, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f91a24, SUC_lexicon_tags_29 },
    { 0x12791a25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacb91a29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d91a2c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c991a2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x591a31, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x87f91a33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba591a35, SUC_lexicon_tags_63 },
    { 0xf4591a36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f91a39, SUC_lexicon_tags_37 },
    { 0xd4d91a3a, SUC_lexicon_tags_0 },
    { 0xedd91a39, SUC_lexicon_tags_41 },
    { 0xd3791a3c, SUC_lexicon_tags_19 },
    { 0xe4391a3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5191a41, SUC_lexicon_tags_11 },
    { 0xa3b91a42, SUC_lexicon_tags_29 },
    { 0xd8991a43, SUC_lexicon_tags_17 },
    { 0xc9d91a42, SUC_lexicon_tags_12 },
    { 0x8b91a42, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb7591a47, SUC_lexicon_tags_63 },
    { 0x3a391a48, SUC_lexicon_tags_55 },
    { 0x27191a49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3ef91a4b, SUC_lexicon_tags_33 },
    { 0x56f91a4b, SUC_lexicon_tags_1 },
    { 0xe2391a4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91a51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x96991a54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf591a5a, SUC_lexicon_tags_28 },
    { 0x45191a5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd91a62, SUC_lexicon_tags_12 },
    { 0x80b91a63, SUC_lexicon_tags_14 },
    { 0x75b91a64, SUC_lexicon_tags_27 },
    { 0x59d91a65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72791a69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae791a6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a191a70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5591a75, SUC_lexicon_tags_7 },
    { 0xe5591a76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x78591a79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22591a7b, SUC_lexicon_tags_38 },
    { 0xf8991a7c, SUC_lexicon_tags_1 },
    { 0xf9d91a7c, SUC_lexicon_tags_68 },
    { 0x5a991a7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63991a81, SUC_lexicon_tags_25 },
    { 0xc3d91a82, SUC_lexicon_tags_1 },
    { 0x1b991a83, SUC_lexicon_tags_87 },
    { 0x35991a84, SUC_lexicon_tags_11 },
    { 0xa591a85, SUC_lexicon_tags_6 },
    { 0xbaf91a83, SUC_lexicon_tags_33 },
    { 0x12391a85, SUC_lexicon_tags_63 },
    { 0xb1791a88, SUC_lexicon_tags_9 },
    { 0x6e991a89, SUC_lexicon_tags_37 },
    { 0x93791a8a, SUC_lexicon_tags_12 },
    { 0x36791a8a, SUC_lexicon_tags_66 },
    { 0xaf91a8c, SUC_lexicon_tags_19 },
    { 0xe3191a8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f91a96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaff91a98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa1791a9a, SUC_lexicon_tags_1 },
    { 0x32791a9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb91aa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f791aa6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2db91aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b91aac, SUC_lexicon_tags_17 },
    { 0x36191aac, SUC_lexicon_tags_12 },
    { 0x8f391aae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d91ab2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf991ab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9991ab8, SUC_lexicon_tags_87 },
    { 0xc8f91ab8, SUC_lexicon_tags_141 },
    { 0x3f791ab9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91abf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d391ac2, SUC_lexicon_tags_8 },
    { 0xbf591ac3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad91ac9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f91acc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd91acf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e991ad2, SUC_lexicon_tags_23 },
    { 0xdcf91ad3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78791ad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74191adb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b91ae0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x31b91ae2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf4d91ae4, SUC_lexicon_tags_48 },
    { 0xbdb91ae5, SUC_lexicon_tags_53 },
    { 0xf2f91ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bd91ae8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d91aeb, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf91aee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf91af2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f91af8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b91afc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef591b00, SUC_lexicon_tags_60 },
    { 0x82d91b01, SUC_lexicon_tags_14 },
    { 0x4a591b01, SUC_lexicon_tags_30 },
    { 0xf591b03, SUC_lexicon_tags_23 },
    { 0x90b91b04, SUC_lexicon_tags_25 },
    { 0x47f91b05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0391b08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f91b0c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c791b17, SUC_lexicon_tags_23 },
    { 0xa991b17, SUC_lexicon_tags_39 },
    { 0xa591b19, SUC_lexicon_tags_14 },
    { 0x5cf91b1a, SUC_lexicon_tags_37 },
    { 0xd7b91b1b, SUC_lexicon_tags_17 },
    { 0xfa791b1c, SUC_lexicon_tags_17 },
    { 0xc591b1a, SUC_lexicon_tags_26 },
    { 0xf5991b17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f91b23, SUC_lexicon_tags_2 },
    { 0x64191b23, SUC_lexicon_tags_17 },
    { 0x26791b25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe2791b27, SUC_lexicon_tags_5 },
    { 0x9e991b28, SUC_lexicon_tags_7 },
    { 0x85791b27, SUC_lexicon_tags_31 },
    { 0xc2391b29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b91b2e, SUC_lexicon_tags_45 },
    { 0x7d591b2e, SUC_lexicon_tags_27 },
    { 0xbe791b2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d91b33, SUC_lexicon_tags_37 },
    { 0x6f391b34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94991b36, SUC_lexicon_tags_2 },
    { 0xa4591b37, SUC_lexicon_tags_1 },
    { 0xcc591b36, SUC_lexicon_tags_188 },
    { 0xb4991b39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0391b3d, SUC_lexicon_tags_6 },
    { 0x51791b3e, SUC_lexicon_tags_6 },
    { 0x1a791b3f, SUC_lexicon_tags_7 },
    { 0xc1591b40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32391b44, SUC_lexicon_tags_27 },
    { 0x5b91b44, SUC_lexicon_tags_25 },
    { 0x25d91b46, SUC_lexicon_tags_15 },
    { 0xd2f91b47, SUC_lexicon_tags_15 },
    { 0x66991b48, SUC_lexicon_tags_38 },
    { 0xd3991b49, SUC_lexicon_tags_19 },
    { 0xac791b49, SUC_lexicon_tags_17 },
    { 0x65591b4b, SUC_lexicon_tags_2 },
    { 0x9f791b49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47b91b4e, SUC_lexicon_tags_9 },
    { 0xe0391b4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6191b53, SUC_lexicon_tags_28 },
    { 0xfab91b54, SUC_lexicon_tags_4 },
    { 0x60b91b55, SUC_lexicon_tags_4 },
    { 0x99791b55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7d91b58, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ed91b5a, SUC_lexicon_tags_87 },
    { 0x33991b5b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b91b5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84991b64, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4591b67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x13b91b69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2991b70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f91b73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70d91b76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f91b80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b91b83, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x11591b86, SUC_lexicon_tags_82 },
    { 0xf591b86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b91b8b, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c991b92, SUC_lexicon_tags_25 },
    { 0x5cb91b92, SUC_lexicon_tags_37 },
    { 0x9d991b94, SUC_lexicon_tags_25 },
    { 0xdb591b95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8391b98, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7a591b9a, SUC_lexicon_tags_45 },
    { 0x31d91b9b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f191b9e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1c791ba0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f191ba2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b91ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f91baa, SUC_lexicon_tags_12 },
    { 0xbdb91bab, SUC_lexicon_tags_68 },
    { 0xc0f91bac, SUC_lexicon_tags_8 },
    { 0x8d191bad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52d91baf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcc191bb1, SUC_lexicon_tags_37 },
    { 0x3f91bb1, SUC_lexicon_tags_25 },
    { 0x52b91bb1, SUC_lexicon_tags_16 },
    { 0x11b91bb4, SUC_lexicon_tags_11 },
    { 0xeb191bb5, SUC_lexicon_tags_38 },
    { 0xaa791bb6, SUC_lexicon_tags_19 },
    { 0x4d191bb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd791bbb, SUC_lexicon_tags_9 },
    { 0x53f91bbc, SUC_lexicon_tags_30 },
    { 0xdd791bbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf791bbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91791bc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc791bc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e191bcb, SUC_lexicon_tags_27 },
    { 0x38f91bcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8591bd1, SUC_lexicon_tags_4 },
    { 0x6fd91bd1, SUC_lexicon_tags_15 },
    { 0x16f91bd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5391bd9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf91be6, SUC_lexicon_tags_25 },
    { 0x5d991be7, SUC_lexicon_tags_1 },
    { 0x87991be7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f91beb, SUC_lexicon_tags_14 },
    { 0xe9d91bec, SUC_lexicon_tags_38 },
    { 0x5ed91bec, SUC_lexicon_tags_17 },
    { 0xf8b91beb, SUC_lexicon_tags_37 },
    { 0xad91bef, SUC_lexicon_tags_4 },
    { 0xf8991bf0, SUC_lexicon_tags_49 },
    { 0x20b91bef, SUC_lexicon_tags_1 },
    { 0xd4391bf1, SUC_lexicon_tags_7 },
    { 0xb1791bf3, SUC_lexicon_tags_29 },
    { 0x53991bf4, SUC_lexicon_tags_7 },
    { 0x5bf91bf5, SUC_lexicon_tags_37 },
    { 0xd8591bf6, SUC_lexicon_tags_7 },
    { 0x78f91bf7, SUC_lexicon_tags_37 },
    { 0x591bf0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4a591bfa, SUC_lexicon_tags_60 },
    { 0xd6591bfb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe0591bfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59b91bff, SUC_lexicon_tags_63 },
    { 0x4a791c00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d991c02, SUC_lexicon_tags_47 },
    { 0xdc591c02, SUC_lexicon_tags_27 },
    { 0xfc591c04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac991c0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf91c11, SUC_lexicon_tags_23 },
    { 0x72591c11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32191c1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41791c1f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb191c25, SUC_lexicon_tags_39 },
    { 0xf0191c26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab91c2c, SUC_lexicon_tags_1 },
    { 0x45791c2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76791c2f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x41f91c31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd591c33, SUC_lexicon_tags_14 },
    { 0x20f91c34, SUC_lexicon_tags_10 },
    { 0x94791c35, SUC_lexicon_tags_37 },
    { 0x18391c36, SUC_lexicon_tags_1 },
    { 0x86d91c34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56791c3e, SUC_lexicon_tags_7 },
    { 0x14f91c3f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3591c43, SUC_lexicon_tags_0 },
    { 0x83b91c43, SUC_lexicon_tags_37 },
    { 0x58b91c43, SUC_lexicon_tags_2 },
    { 0xa5591c46, SUC_lexicon_tags_33 },
    { 0x55791c46, SUC_lexicon_tags_7 },
    { 0x4ad91c48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc591c4d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf3b91c4f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xee991c51, SUC_lexicon_tags_1 },
    { 0x4b91c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e391c54, SUC_lexicon_tags_23 },
    { 0xdfd91c55, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b91c5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd391c5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb91c63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4591c68, SUC_lexicon_tags_15 },
    { 0xb9591c69, SUC_lexicon_tags_10 },
    { 0xa4191c68, SUC_lexicon_tags_25 },
    { 0x45f91c6a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b91c70, SUC_lexicon_tags_16 },
    { 0x8bd91c71, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f91c74, SUC_lexicon_tags_43 },
    { 0x39191c75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf91c78, SUC_lexicon_tags_49 },
    { 0xdfd91c79, SUC_lexicon_tags_81 },
    { 0xff191c79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64b91c7d, SUC_lexicon_tags_2 },
    { 0xb9991c7e, SUC_lexicon_tags_16 },
    { 0x40991c7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d591c85, SUC_lexicon_tags_33 },
    { 0xca791c86, SUC_lexicon_tags_27 },
    { 0xc4991c87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a991c8a, SUC_lexicon_tags_25 },
    { 0x3a191c8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39991c8f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2791c93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb591c99, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a591ca4, SUC_lexicon_tags_106 },
    { 0x89d91ca5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33591cab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0591cae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdad91cb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66991cb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a391cb5, SUC_lexicon_tags_194 },
    { 0xef91cb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x52391cb9, SUC_lexicon_tags_23 },
    { 0x37391cba, SUC_lexicon_tags_2 },
    { 0x21d91cba, SUC_lexicon_tags_17 },
    { 0x50791cbc, SUC_lexicon_tags_2 },
    { 0xc8191cbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf91cc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31391cc6, SUC_lexicon_tags_47 },
    { 0x43f91cc7, SUC_lexicon_tags_12 },
    { 0x95991cc7, SUC_lexicon_tags_49 },
    { 0x61591cc9, SUC_lexicon_tags_25 },
    { 0x8d91cc9, SUC_lexicon_tags_14 },
    { 0x55191cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1d91ccd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f91cd0, SUC_lexicon_tags_13 },
    { 0x92b91cd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0791cd6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791cdc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d91ce8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x32391ceb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91991cef, SUC_lexicon_tags_38 },
    { 0x8f991cf0, SUC_lexicon_tags_12 },
    { 0x15191cf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8991cf3, SUC_lexicon_tags_11 },
    { 0xf2591cf4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d91cf8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaff91cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d791cfd, SUC_lexicon_tags_45 },
    { 0x62591cfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb991d00, SUC_lexicon_tags_15 },
    { 0x86391d00, SUC_lexicon_tags_37 },
    { 0xdf791d02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b791d09, SUC_lexicon_tags_14 },
    { 0x5e591d0a, SUC_lexicon_tags_25 },
    { 0x46d91d09, SUC_lexicon_tags_61 },
    { 0x3ff91d0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda591d0f, SUC_lexicon_tags_15 },
    { 0xfc191d10, SUC_lexicon_tags_2 },
    { 0x20d91d11, SUC_lexicon_tags_30 },
    { 0xc4391d12, SUC_lexicon_tags_77 },
    { 0xcb791d13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf91d18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bd91d1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cb91d20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3191d24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e591d26, SUC_lexicon_tags_33 },
    { 0xa2d91d26, SUC_lexicon_tags_0 },
    { 0xdff91d27, SUC_lexicon_tags_15 },
    { 0x1d791d29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3191d2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcef91d2e, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x6b91d30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60791d32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68991d36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6191d38, SUC_lexicon_tags_28 },
    { 0xd5b91d39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f91d3f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b91d43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc6191d45, SUC_lexicon_tags_25 },
    { 0xdad91d46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65391d49, SUC_lexicon_tags_25 },
    { 0xf5b91d4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37791d4c, SUC_lexicon_tags_17 },
    { 0x47591d4d, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6191d50, SUC_lexicon_tags_37 },
    { 0x57591d51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60391d59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf791d5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb91d60, SUC_lexicon_tags_4 },
    { 0xa4991d60, SUC_lexicon_tags_8 },
    { 0x74191d60, SUC_lexicon_tags_6 },
    { 0xa0b91d63, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92191d67, SUC_lexicon_tags_31 },
    { 0xb3d91d68, SUC_lexicon_tags_11 },
    { 0xb0191d69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d791d6b, SUC_lexicon_tags_29 },
    { 0x7e791d6b, SUC_lexicon_tags_2 },
    { 0xf4591d6d, SUC_lexicon_tags_37 },
    { 0x9d591d6e, SUC_lexicon_tags_15 },
    { 0x2f791d6f, SUC_lexicon_tags_2 },
    { 0xa6591d6d, SUC_lexicon_tags_23 },
    { 0xc4391d70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6591d77, SUC_lexicon_tags_11 },
    { 0x9c991d77, SUC_lexicon_tags_0 },
    { 0x1d591d79, SUC_lexicon_tags_14 },
    { 0xa8591d79, SUC_lexicon_tags_10 },
    { 0xd3591d7b, SUC_lexicon_tags_14 },
    { 0xabb91d7c, SUC_lexicon_tags_31 },
    { 0xad991d79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b91d80, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x26791d82, SUC_lexicon_tags_37 },
    { 0xcbd91d82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb91d85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b91d89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f91d8c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x43f91d8e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xac791d90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f91d96, SUC_lexicon_tags_27 },
    { 0x4f391d97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b91d9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f791d9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d91da0, SUC_lexicon_tags_2 },
    { 0x79191da1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b591da4, SUC_lexicon_tags_25 },
    { 0x9191da5, SUC_lexicon_tags_14 },
    { 0x2f391da6, SUC_lexicon_tags_33 },
    { 0x6e791da7, SUC_lexicon_tags_29 },
    { 0x11591da5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3b91daa, SUC_lexicon_tags_49 },
    { 0xb5991dab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9dd91dad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c191db0, SUC_lexicon_tags_116 },
    { 0x7c391db1, SUC_lexicon_tags_66 },
    { 0xe8f91db2, SUC_lexicon_tags_11 },
    { 0xaa991db3, SUC_lexicon_tags_2 },
    { 0x6a791db4, SUC_lexicon_tags_116 },
    { 0xfb391db5, SUC_lexicon_tags_27 },
    { 0x17591db4, SUC_lexicon_tags_84 },
    { 0xb3791db7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c391dba, SUC_lexicon_tags_7 },
    { 0xc0b91dba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59391dbd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eb91dc0, SUC_lexicon_tags_31 },
    { 0xa4b91dc1, SUC_lexicon_tags_37 },
    { 0x13391dc2, SUC_lexicon_tags_37 },
    { 0x21591dc2, SUC_lexicon_tags_60 },
    { 0xe1591dc4, SUC_lexicon_tags_7 },
    { 0xcf391dc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1191dc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57191dd2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x35391dd4, SUC_lexicon_tags_12 },
    { 0x5dd91dd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f391dd9, SUC_lexicon_tags_27 },
    { 0xde991dd9, SUC_lexicon_tags_79 },
    { 0x14d91dd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9b91ddd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65391de2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0b91de4, SUC_lexicon_tags_33 },
    { 0x5e591de5, SUC_lexicon_tags_56 },
    { 0x5e991de5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73791df3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb391df8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22991dfe, SUC_lexicon_tags_14 },
    { 0x94b91dff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b91e03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d91e07, SUC_lexicon_tags_34 },
    { 0xa0391e08, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x81f91e0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc591e0c, SUC_lexicon_tags_2 },
    { 0x65b91e0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf5791e0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f91e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51591e15, SUC_lexicon_tags_0 },
    { 0x13591e16, SUC_lexicon_tags_25 },
    { 0xe0791e17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f91e1a, SUC_lexicon_tags_63 },
    { 0xd6f91e1b, SUC_lexicon_tags_31 },
    { 0xeaf91e1c, SUC_lexicon_tags_12 },
    { 0x75991e1d, SUC_lexicon_tags_31 },
    { 0x65f91e1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bd91e27, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbf391e29, SUC_lexicon_tags_15 },
    { 0x5b391e2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3791e37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2391e3a, SUC_lexicon_tags_0 },
    { 0x52791e3b, SUC_lexicon_tags_21 },
    { 0x29591e3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b391e40, SUC_lexicon_tags_21 },
    { 0x3ad91e41, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x51991e44, SUC_lexicon_tags_15 },
    { 0x72391e45, SUC_lexicon_tags_1 },
    { 0xc3191e44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b91e4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76f91e4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7991e51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4af91e53, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab91e59, SUC_lexicon_tags_37 },
    { 0xa2791e59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2791e5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75191e5f, SUC_lexicon_tags_15 },
    { 0xf8391e60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f991e63, SUC_lexicon_tags_23 },
    { 0xb3791e64, SUC_lexicon_tags_26 },
    { 0x21391e65, SUC_lexicon_tags_0 },
    { 0xa8991e64, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xaa391e68, SUC_lexicon_tags_11 },
    { 0xcd391e69, SUC_lexicon_tags_12 },
    { 0xf7591e69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea591e6e, SUC_lexicon_tags_12 },
    { 0x18d91e6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b91e73, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7b91e75, SUC_lexicon_tags_38 },
    { 0xb1b91e76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb791e7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f191e81, SUC_lexicon_tags_77 },
    { 0x4a791e82, SUC_lexicon_tags_45 },
    { 0x2a191e81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0991e87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab191e8b, SUC_lexicon_tags_49 },
    { 0xf3b91e8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50991e91, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84991e96, SUC_lexicon_tags_26 },
    { 0x18991e97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58b91e99, SUC_lexicon_tags_25 },
    { 0x41b91e99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24791e9d, SUC_lexicon_tags_1 },
    { 0x7cb91e9e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb991ea6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41591eab, SUC_lexicon_tags_11 },
    { 0x65791eac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x30d91eaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9191eb1, SUC_lexicon_tags_66 },
    { 0xd0591eb2, SUC_lexicon_tags_60 },
    { 0xfeb91eb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a591eb6, SUC_lexicon_tags_15 },
    { 0x54191eb7, SUC_lexicon_tags_33 },
    { 0x85f91eb7, SUC_lexicon_tags_48 },
    { 0x7f91eb8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xac991ebb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e791ebd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f791ebf, SUC_lexicon_tags_30 },
    { 0xe0191ebf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22f91ec4, SUC_lexicon_tags_45 },
    { 0x89b91ec5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32b91ec7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f91ecf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd91ed7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef191edb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x87791ede, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x82591ee0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x39991ee3, SUC_lexicon_tags_60 },
    { 0x30791ee4, SUC_lexicon_tags_14 },
    { 0xf8391ee5, SUC_lexicon_tags_25 },
    { 0x7f591ee6, SUC_lexicon_tags_12 },
    { 0x6bf91ee3, SUC_lexicon_tags_6 },
    { 0x75791ee5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32591eec, SUC_lexicon_tags_172 },
    { 0x71191eed, SUC_lexicon_tags_25 },
    { 0xcc791eed, SUC_lexicon_tags_8 },
    { 0xf6591eef, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xde991ef1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f91ef7, SUC_lexicon_tags_25 },
    { 0x13391ef8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b91efb, SUC_lexicon_tags_33 },
    { 0xc8791efb, SUC_lexicon_tags_4 },
    { 0xe2591efc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a391f00, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3591f09, SUC_lexicon_tags_63 },
    { 0xbf991f0a, SUC_lexicon_tags_12 },
    { 0x56991f09, SUC_lexicon_tags_37 },
    { 0xdd991f09, SUC_lexicon_tags_12 },
    { 0xebb91f09, SUC_lexicon_tags_15 },
    { 0x13f91f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xedd91f10, SUC_lexicon_tags_23 },
    { 0x2b191f11, SUC_lexicon_tags_84 },
    { 0x8ed91f12, SUC_lexicon_tags_7 },
    { 0xdc391f13, SUC_lexicon_tags_28 },
    { 0x77d91f14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x94791f16, SUC_lexicon_tags_59 },
    { 0x91591f16, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa991f1a, SUC_lexicon_tags_23 },
    { 0xe2b91f1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5391f1e, SUC_lexicon_tags_17 },
    { 0x79991f1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb91f23, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x46d91f25, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x36f91f28, SUC_lexicon_tags_47 },
    { 0x9b791f28, SUC_lexicon_tags_1 },
    { 0xc2b91f29, SUC_lexicon_tags_47 },
    { 0x12591f2b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11791f32, SUC_lexicon_tags_7 },
    { 0xe1191f32, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x44791f35, SUC_lexicon_tags_23 },
    { 0x99b91f36, SUC_lexicon_tags_2 },
    { 0xd1991f36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb91f3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab91f3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e391f3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b91f45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2391f49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f91f4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced91f55, SUC_lexicon_tags_359 },
    { 0xce791f56, SUC_lexicon_tags_17 },
    { 0x75f91f57, SUC_lexicon_tags_39 },
    { 0x6a591f58, SUC_lexicon_tags_230 },
    { 0x1c791f58, SUC_lexicon_tags_11 },
    { 0x5f791f5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6391f68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4391f6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f591f6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc991f70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x50191f72, SUC_lexicon_tags_37 },
    { 0x94f91f73, SUC_lexicon_tags_27 },
    { 0x5ab91f74, SUC_lexicon_tags_23 },
    { 0xc8d91f72, SUC_lexicon_tags_15 },
    { 0x18591f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b91f79, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x60d91f7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e391f81, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83791f83, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b91f8c, SUC_lexicon_tags_11 },
    { 0xbbd91f8d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4db91f8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba391f94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a791f99, SUC_lexicon_tags_4 },
    { 0x66991f9a, SUC_lexicon_tags_65 },
    { 0xd6991f9b, SUC_lexicon_tags_12 },
    { 0xe5591f9c, SUC_lexicon_tags_17 },
    { 0x40991f99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff91fa1, SUC_lexicon_tags_2 },
    { 0x8f591fa2, SUC_lexicon_tags_11 },
    { 0x23f91fa3, SUC_lexicon_tags_35 },
    { 0xe4591fa3, SUC_lexicon_tags_7 },
    { 0x76b91fa3, SUC_lexicon_tags_7 },
    { 0xb0191fa6, SUC_lexicon_tags_15 },
    { 0x6c791fa7, SUC_lexicon_tags_8 },
    { 0xea791fa6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bd91faa, SUC_lexicon_tags_17 },
    { 0x6e591fab, SUC_lexicon_tags_15 },
    { 0x46d91faa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31791fb0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xce191fb2, SUC_lexicon_tags_2 },
    { 0x89591fb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ad91fb5, SUC_lexicon_tags_14 },
    { 0xe1991fb6, SUC_lexicon_tags_30 },
    { 0x58f91fb7, SUC_lexicon_tags_33 },
    { 0xe3391fb8, SUC_lexicon_tags_84 },
    { 0x4a991fb9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad791fbe, SUC_lexicon_tags_30 },
    { 0x43f91fbf, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x94991fc2, SUC_lexicon_tags_2 },
    { 0xa5791fc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xee791fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cd91fc7, SUC_lexicon_tags_14 },
    { 0x2f191fc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf91fca, SUC_lexicon_tags_17 },
    { 0xd3b91fcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e991fcd, SUC_lexicon_tags_7 },
    { 0x2c991fcd, SUC_lexicon_tags_66 },
    { 0x80191fcd, SUC_lexicon_tags_15 },
    { 0x22791fd0, SUC_lexicon_tags_52 },
    { 0xe8591fd1, SUC_lexicon_tags_39 },
    { 0x36d91fd2, SUC_lexicon_tags_11 },
    { 0x90f91fcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcf91fd5, SUC_lexicon_tags_30 },
    { 0x3f191fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d91fda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed91fde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3791fe2, SUC_lexicon_tags_17 },
    { 0x39791fe3, SUC_lexicon_tags_70 },
    { 0xa9391fe3, SUC_lexicon_tags_11 },
    { 0xe1d91fe5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b791fe9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xba391fec, SUC_lexicon_tags_19 },
    { 0xf9991fec, SUC_lexicon_tags_33 },
    { 0xba591fed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78191ff2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54b91ff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd91ff8, SUC_lexicon_tags_37 },
    { 0x1b91ff9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2591ffd, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xeb791fff, SUC_lexicon_tags_4 },
    { 0xd4f92000, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdeb92002, SUC_lexicon_tags_25 },
    { 0x56d92003, SUC_lexicon_tags_14 },
    { 0x40992002, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xad92006, SUC_lexicon_tags_1 },
    { 0x65f92007, SUC_lexicon_tags_83 },
    { 0x23392007, SUC_lexicon_tags_37 },
    { 0x7ff92009, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed9200c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a59200e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b92011, SUC_lexicon_tags_31 },
    { 0xd8b92012, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98992016, SUC_lexicon_tags_14 },
    { 0x81392017, SUC_lexicon_tags_47 },
    { 0x20f92018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b9201b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d92022, SUC_lexicon_tags_749 },
    { 0x8d592023, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdff92025, SUC_lexicon_tags_15 },
    { 0xd7592026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25992028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba19202d, SUC_lexicon_tags_19 },
    { 0xa319202e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x48d92030, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x49992033, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df9203c, SUC_lexicon_tags_7 },
    { 0x2ed9203c, SUC_lexicon_tags_10 },
    { 0xf459203c, SUC_lexicon_tags_37 },
    { 0xd799203f, SUC_lexicon_tags_12 },
    { 0x2ff9203d, SUC_lexicon_tags_27 },
    { 0x19d92041, SUC_lexicon_tags_7 },
    { 0x9a592042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d792047, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35792049, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f9204d, SUC_lexicon_tags_9 },
    { 0xfb59204d, SUC_lexicon_tags_0 },
    { 0xf159204f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12d92051, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f92054, SUC_lexicon_tags_15 },
    { 0xe5592055, SUC_lexicon_tags_44 },
    { 0x2cd92054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a392058, SUC_lexicon_tags_27 },
    { 0x2cd92059, SUC_lexicon_tags_25 },
    { 0x8259205a, SUC_lexicon_tags_11 },
    { 0xd759205b, SUC_lexicon_tags_11 },
    { 0x62f9205a, SUC_lexicon_tags_52 },
    { 0xdc192058, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1592060, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2f392062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f992064, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa3d92066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2499206b, SUC_lexicon_tags_9 },
    { 0x7cf9206c, SUC_lexicon_tags_11 },
    { 0x3cf9206c, SUC_lexicon_tags_1 },
    { 0x79f9206e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb92074, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81b92076, SUC_lexicon_tags_113 },
    { 0xfa792077, SUC_lexicon_tags_8 },
    { 0x58392078, SUC_lexicon_tags_23 },
    { 0xb2592079, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda99207e, SUC_lexicon_tags_28 },
    { 0x6b19207f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f92083, SUC_lexicon_tags_33 },
    { 0xbfb92084, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b92088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb9208f, SUC_lexicon_tags_25 },
    { 0xfa992090, SUC_lexicon_tags_45 },
    { 0x3a192091, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x55792094, SUC_lexicon_tags_30 },
    { 0xdbd92095, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x91b92097, SUC_lexicon_tags_60 },
    { 0x95992098, SUC_lexicon_tags_7 },
    { 0x94192098, SUC_lexicon_tags_63 },
    { 0xd3d92098, SUC_lexicon_tags_2 },
    { 0x7b992097, SUC_lexicon_tags_1 },
    { 0xa992097, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd59209e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x251920a1, SUC_lexicon_tags_69 },
    { 0x593920a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343920a9, SUC_lexicon_tags_7 },
    { 0x5b5920aa, SUC_lexicon_tags_37 },
    { 0x659920a9, SUC_lexicon_tags_1 },
    { 0x48b920a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x22b920ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103920b2, SUC_lexicon_tags_17 },
    { 0x9a3920b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87d920b5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x107920b7, SUC_lexicon_tags_1 },
    { 0x7cd920b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37920bc, SUC_lexicon_tags_12 },
    { 0x953920bd, SUC_lexicon_tags_11 },
    { 0xf1f920bd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d920c3, SUC_lexicon_tags_17 },
    { 0xec1920c4, SUC_lexicon_tags_25 },
    { 0xaa3920c5, SUC_lexicon_tags_27 },
    { 0xceb920c6, SUC_lexicon_tags_25 },
    { 0xba1920c7, SUC_lexicon_tags_23 },
    { 0x41d920c7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df920cb, SUC_lexicon_tags_37 },
    { 0x1df920cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9920d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d920d5, SUC_lexicon_tags_46 },
    { 0x84f920d6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7920df, SUC_lexicon_tags_15 },
    { 0x88f920df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55920e5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d920e9, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3920ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67920f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf920f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed920ff, SUC_lexicon_tags_17 },
    { 0xa7992100, SUC_lexicon_tags_159 },
    { 0x99f92101, SUC_lexicon_tags_38 },
    { 0x2e192102, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3392106, SUC_lexicon_tags_7 },
    { 0xad92107, SUC_lexicon_tags_48 },
    { 0xa7392106, SUC_lexicon_tags_45 },
    { 0x63b92106, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf9210e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7992110, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x87192112, SUC_lexicon_tags_14 },
    { 0x48592112, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d92117, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a19211a, SUC_lexicon_tags_42 },
    { 0x2f79211a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65792127, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b9212a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e79212e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb2f92130, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81d92132, SUC_lexicon_tags_2 },
    { 0x9c792133, SUC_lexicon_tags_30 },
    { 0x7eb92133, SUC_lexicon_tags_6 },
    { 0x1392135, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4819213b, SUC_lexicon_tags_15 },
    { 0x2db9213c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2592140, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1819214e, SUC_lexicon_tags_25 },
    { 0x9ad9214f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5d92151, SUC_lexicon_tags_37 },
    { 0xa2392151, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x65792154, SUC_lexicon_tags_25 },
    { 0xc9792155, SUC_lexicon_tags_30 },
    { 0x5c992155, SUC_lexicon_tags_31 },
    { 0x35592157, SUC_lexicon_tags_2 },
    { 0x6a392158, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9215b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x79d9215d, SUC_lexicon_tags_27 },
    { 0x3279215d, SUC_lexicon_tags_2 },
    { 0xb4d9215d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a592162, SUC_lexicon_tags_19 },
    { 0xd3592163, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d92166, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d92171, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98392177, SUC_lexicon_tags_92 },
    { 0x7cb92178, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc99217a, SUC_lexicon_tags_30 },
    { 0x2379217a, SUC_lexicon_tags_11 },
    { 0xe8b9217c, SUC_lexicon_tags_53 },
    { 0xea39217b, SUC_lexicon_tags_15 },
    { 0xf499217e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbe192180, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d92184, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef79218a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28192192, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f92195, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb92199, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6499219d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8579219f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9921a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b921aa, SUC_lexicon_tags_24 },
    { 0x8d7921aa, SUC_lexicon_tags_25 },
    { 0xbc1921ac, SUC_lexicon_tags_49 },
    { 0xf07921ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb81921af, SUC_lexicon_tags_12 },
    { 0xe39921b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b921b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51921b8, SUC_lexicon_tags_47 },
    { 0x1d7921b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x175921bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x46f921be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd921c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1921c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf921c8, SUC_lexicon_tags_8 },
    { 0xfa3921c8, SUC_lexicon_tags_63 },
    { 0xb83921ca, SUC_lexicon_tags_25 },
    { 0xe1d921cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x28d921cd, SUC_lexicon_tags_17 },
    { 0xab9921ce, SUC_lexicon_tags_1 },
    { 0x933921ce, SUC_lexicon_tags_9 },
    { 0x60d921d0, SUC_lexicon_tags_17 },
    { 0x493921d0, SUC_lexicon_tags_14 },
    { 0x13d921d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x25b921d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd49921d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xded921d9, SUC_lexicon_tags_1 },
    { 0xd35921da, SUC_lexicon_tags_158 },
    { 0xe15921d9, SUC_lexicon_tags_11 },
    { 0x8b9921d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d921e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ef921e2, SUC_lexicon_tags_31 },
    { 0x567921e3, SUC_lexicon_tags_48 },
    { 0x361921e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dd921e6, SUC_lexicon_tags_17 },
    { 0xf3d921e7, SUC_lexicon_tags_12 },
    { 0x583921e7, SUC_lexicon_tags_70 },
    { 0x85b921e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd921ef, SUC_lexicon_tags_4 },
    { 0x1e1921ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5921f3, SUC_lexicon_tags_11 },
    { 0xef9921f4, SUC_lexicon_tags_14 },
    { 0xdfb921f5, SUC_lexicon_tags_7 },
    { 0xac9921f6, SUC_lexicon_tags_49 },
    { 0x715921f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbed921fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2f7921fc, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xe85921fe, SUC_lexicon_tags_27 },
    { 0xe0f921ff, SUC_lexicon_tags_2 },
    { 0xb8b92200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd92202, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd592206, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x36992208, SUC_lexicon_tags_15 },
    { 0xb4592209, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2999220b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1192210, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb792212, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b9221a, SUC_lexicon_tags_13 },
    { 0xf7b9221b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b19222b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb299222d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4392231, SUC_lexicon_tags_19 },
    { 0xea592232, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d92235, SUC_lexicon_tags_12 },
    { 0x7592236, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f92244, SUC_lexicon_tags_29 },
    { 0xa4992245, SUC_lexicon_tags_56 },
    { 0x92192246, SUC_lexicon_tags_25 },
    { 0xccf92247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad9224d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb392256, SUC_lexicon_tags_25 },
    { 0xe8f92257, SUC_lexicon_tags_7 },
    { 0xd192258, SUC_lexicon_tags_15 },
    { 0x7d592258, SUC_lexicon_tags_43 },
    { 0x2eb92257, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7f39225c, SUC_lexicon_tags_31 },
    { 0x1d19225d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x54b9225f, SUC_lexicon_tags_44 },
    { 0xdf992260, SUC_lexicon_tags_35 },
    { 0x3a192261, SUC_lexicon_tags_14 },
    { 0xf2f92261, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e192264, SUC_lexicon_tags_25 },
    { 0xb6592264, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84792267, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc19226d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d92270, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f92274, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27592279, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6859227d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b39227f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90992281, SUC_lexicon_tags_11 },
    { 0x33792282, SUC_lexicon_tags_1 },
    { 0xf6592283, SUC_lexicon_tags_32 },
    { 0x25792284, SUC_lexicon_tags_2 },
    { 0x84f92283, SUC_lexicon_tags_19 },
    { 0xf4b92284, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x42992288, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e39228d, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef92290, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f92293, SUC_lexicon_tags_3 },
    { 0xf3192294, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5179229b, SUC_lexicon_tags_29 },
    { 0x18f9229c, SUC_lexicon_tags_61 },
    { 0x1679229c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3922a3, SUC_lexicon_tags_15 },
    { 0x7c9922a4, SUC_lexicon_tags_27 },
    { 0xc0f922a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa2b922a7, SUC_lexicon_tags_15 },
    { 0xcd1922a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc33922aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4b922ac, SUC_lexicon_tags_25 },
    { 0xaeb922ad, SUC_lexicon_tags_18 },
    { 0x2e3922ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb922b1, SUC_lexicon_tags_27 },
    { 0x53b922b2, SUC_lexicon_tags_11 },
    { 0x78b922b3, SUC_lexicon_tags_26 },
    { 0x701922b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdad922b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7922bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8ab922c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x617922c3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1922c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x76d922c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b922ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391922d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1922d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d922dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9922e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689922e5, SUC_lexicon_tags_37 },
    { 0x195922e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd67922e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b922ec, SUC_lexicon_tags_377 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f922f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d922f7, SUC_lexicon_tags_12 },
    { 0xd47922f7, SUC_lexicon_tags_2 },
    { 0xe6f922f7, SUC_lexicon_tags_17 },
    { 0xce3922f9, SUC_lexicon_tags_12 },
    { 0x871922fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69922ff, SUC_lexicon_tags_39 },
    { 0x2e9922ff, SUC_lexicon_tags_856 },
    { 0x87592301, SUC_lexicon_tags_17 },
    { 0xa4592302, SUC_lexicon_tags_77 },
    { 0xfa992303, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x76792306, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e392309, SUC_lexicon_tags_11 },
    { 0x7a192309, SUC_lexicon_tags_46 },
    { 0x5299230b, SUC_lexicon_tags_17 },
    { 0xb3f9230a, SUC_lexicon_tags_15 },
    { 0xb019230a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9d9230f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb92315, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b192319, SUC_lexicon_tags_21 },
    { 0xb4d9231a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d92321, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d92325, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd92328, SUC_lexicon_tags_1396 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f9232f, SUC_lexicon_tags_7 },
    { 0x639232f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11d92332, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed392334, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x80192337, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4d92339, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b9233e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x21b92341, SUC_lexicon_tags_11 },
    { 0x4192342, SUC_lexicon_tags_7 },
    { 0xe9592343, SUC_lexicon_tags_16 },
    { 0x10192341, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7299234a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3399234c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d59234e, SUC_lexicon_tags_47 },
    { 0x1f99234f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48f92352, SUC_lexicon_tags_27 },
    { 0x6c592353, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d92357, SUC_lexicon_tags_2 },
    { 0x8f592358, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c79235a, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95392360, SUC_lexicon_tags_0 },
    { 0x9ff92360, SUC_lexicon_tags_25 },
    { 0xb2792362, SUC_lexicon_tags_12 },
    { 0xac392361, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b19236c, SUC_lexicon_tags_36 },
    { 0x5319236c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18592378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8079237a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe2d9237c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d92383, SUC_lexicon_tags_37 },
    { 0xadd92384, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef9238a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa859238f, SUC_lexicon_tags_7 },
    { 0xe7992390, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c992393, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd1592395, SUC_lexicon_tags_38 },
    { 0x91392395, SUC_lexicon_tags_28 },
    { 0xf192397, SUC_lexicon_tags_28 },
    { 0x79992398, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5b9239a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb9239e, SUC_lexicon_tags_37 },
    { 0x23f9239e, SUC_lexicon_tags_15 },
    { 0x335923a0, SUC_lexicon_tags_60 },
    { 0x623923a0, SUC_lexicon_tags_63 },
    { 0x401923a2, SUC_lexicon_tags_37 },
    { 0x29923a3, SUC_lexicon_tags_38 },
    { 0x4d923a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb89923a6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb19923a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1923ae, SUC_lexicon_tags_15 },
    { 0x5cf923ae, SUC_lexicon_tags_23 },
    { 0x5a3923ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d923b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7923bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x457923bf, SUC_lexicon_tags_20 },
    { 0xf63923c0, SUC_lexicon_tags_87 },
    { 0xe43923c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b923c4, SUC_lexicon_tags_19 },
    { 0x613923c4, SUC_lexicon_tags_25 },
    { 0xcb7923c6, SUC_lexicon_tags_47 },
    { 0xbf9923c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x79b923c9, SUC_lexicon_tags_30 },
    { 0x815923ca, SUC_lexicon_tags_25 },
    { 0x911923cb, SUC_lexicon_tags_14 },
    { 0xd1f923cc, SUC_lexicon_tags_0 },
    { 0xcad923cd, SUC_lexicon_tags_60 },
    { 0x935923ce, SUC_lexicon_tags_60 },
    { 0xae9923cd, SUC_lexicon_tags_26 },
    { 0x3d3923c9, SUC_lexicon_tags_31 },
    { 0x1d7923d1, SUC_lexicon_tags_47 },
    { 0xe93923d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1923d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7b923d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849923dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7923de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3923e3, SUC_lexicon_tags_7 },
    { 0x8e1923e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3923eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb923f5, SUC_lexicon_tags_1 },
    { 0xb33923f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185923fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81923fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e392400, SUC_lexicon_tags_15 },
    { 0x5f992401, SUC_lexicon_tags_45 },
    { 0xd5b92402, SUC_lexicon_tags_0 },
    { 0xe9592403, SUC_lexicon_tags_17 },
    { 0x92d92400, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9392408, SUC_lexicon_tags_33 },
    { 0x23b92409, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb119240c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b792412, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80592414, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1799241b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8059241d, SUC_lexicon_tags_11 },
    { 0x8319241e, SUC_lexicon_tags_26 },
    { 0xf39241f, SUC_lexicon_tags_37 },
    { 0x53b9241d, SUC_lexicon_tags_15 },
    { 0x5f19241e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df92424, SUC_lexicon_tags_1 },
    { 0x7a392424, SUC_lexicon_tags_39 },
    { 0xe2d92424, SUC_lexicon_tags_0 },
    { 0xadd92426, SUC_lexicon_tags_17 },
    { 0x53392426, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f9242d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9a19242f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f192432, SUC_lexicon_tags_37 },
    { 0xe8992433, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d92437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c59243a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b9243d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb92444, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x61d92446, SUC_lexicon_tags_18 },
    { 0xbe992447, SUC_lexicon_tags_60 },
    { 0xcdf92446, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa9b9244a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf9244f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48592451, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x14392453, SUC_lexicon_tags_1 },
    { 0x14992453, SUC_lexicon_tags_64 },
    { 0x40392455, SUC_lexicon_tags_37 },
    { 0xe9b92456, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6792459, SUC_lexicon_tags_13 },
    { 0x1a39245a, SUC_lexicon_tags_15 },
    { 0xa399245b, SUC_lexicon_tags_4 },
    { 0x2119245c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b9245f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f92467, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5592469, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad9246c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e392474, SUC_lexicon_tags_116 },
    { 0x87b92475, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbfb92477, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xba192479, SUC_lexicon_tags_15 },
    { 0x1339247a, SUC_lexicon_tags_37 },
    { 0x2299247b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d59247d, SUC_lexicon_tags_43 },
    { 0x2499247e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c792482, SUC_lexicon_tags_14 },
    { 0x35992482, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77992485, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd92488, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc019248a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48592490, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8992492, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b92495, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xebb92497, SUC_lexicon_tags_27 },
    { 0x87792498, SUC_lexicon_tags_2 },
    { 0xfbf92499, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d19249e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7924a1, SUC_lexicon_tags_11 },
    { 0x9c7924a2, SUC_lexicon_tags_66 },
    { 0x49d924a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa9924a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x365924a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901924ab, SUC_lexicon_tags_61 },
    { 0x9f9924ac, SUC_lexicon_tags_17 },
    { 0xd8b924ad, SUC_lexicon_tags_60 },
    { 0x9dd924ad, SUC_lexicon_tags_1 },
    { 0x25b924af, SUC_lexicon_tags_23 },
    { 0x3b5924b0, SUC_lexicon_tags_82 },
    { 0x6d5924b1, SUC_lexicon_tags_14 },
    { 0x6ef924b1, SUC_lexicon_tags_16 },
    { 0xae9924b3, SUC_lexicon_tags_1 },
    { 0x20b924af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff7924b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7924bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x871924be, SUC_lexicon_tags_29 },
    { 0xee5924bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x37b924c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b924c8, SUC_lexicon_tags_57 },
    { 0x52b924c9, SUC_lexicon_tags_33 },
    { 0x28b924c8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03924cf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db924d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dd924d4, SUC_lexicon_tags_45 },
    { 0xc3f924d5, SUC_lexicon_tags_12 },
    { 0xde1924d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7924da, SUC_lexicon_tags_14 },
    { 0xaef924db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3924e2, SUC_lexicon_tags_11 },
    { 0x9d3924e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbeb924e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d924ea, SUC_lexicon_tags_17 },
    { 0xefb924eb, SUC_lexicon_tags_29 },
    { 0x52d924ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94b924ee, SUC_lexicon_tags_37 },
    { 0x727924ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x287924f1, SUC_lexicon_tags_6 },
    { 0xcbf924f2, SUC_lexicon_tags_49 },
    { 0xfbd924f3, SUC_lexicon_tags_37 },
    { 0x777924f4, SUC_lexicon_tags_3 },
    { 0x535924f4, SUC_lexicon_tags_37 },
    { 0xe01924f6, SUC_lexicon_tags_1 },
    { 0x6fb924f7, SUC_lexicon_tags_65 },
    { 0x517924f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa29924fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e99250e, SUC_lexicon_tags_26 },
    { 0xccb9250f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12392512, SUC_lexicon_tags_1 },
    { 0x8dd92512, SUC_lexicon_tags_17 },
    { 0x48792512, SUC_lexicon_tags_108 },
    { 0x84f92515, SUC_lexicon_tags_25 },
    { 0x2bd92515, SUC_lexicon_tags_25 },
    { 0xda192517, SUC_lexicon_tags_1 },
    { 0xd9992516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b9251c, SUC_lexicon_tags_21 },
    { 0xe6b9251d, SUC_lexicon_tags_31 },
    { 0x5179251e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9c992520, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x84d92522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d92525, SUC_lexicon_tags_45 },
    { 0x98392525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5399252b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9252e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c992532, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd992534, SUC_lexicon_tags_0 },
    { 0xcfd92535, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x14f92537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb9253a, SUC_lexicon_tags_3 },
    { 0x8e99253b, SUC_lexicon_tags_12 },
    { 0xc119253b, SUC_lexicon_tags_12 },
    { 0xf99253c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf792541, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f92544, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6b92546, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1792548, SUC_lexicon_tags_36 },
    { 0x5b192548, SUC_lexicon_tags_65 },
    { 0x7639254a, SUC_lexicon_tags_25 },
    { 0x2179254b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb39254e, SUC_lexicon_tags_25 },
    { 0xf7f9254e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49992553, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d92558, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d79255c, SUC_lexicon_tags_77 },
    { 0x1679255d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0392562, SUC_lexicon_tags_17 },
    { 0xa2792563, SUC_lexicon_tags_29 },
    { 0xd7792562, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c592567, SUC_lexicon_tags_11 },
    { 0x94f92568, SUC_lexicon_tags_27 },
    { 0xe0b92567, SUC_lexicon_tags_28 },
    { 0xb9792568, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1f39256c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf992570, SUC_lexicon_tags_11 },
    { 0xea592571, SUC_lexicon_tags_25 },
    { 0xc6792572, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a792575, SUC_lexicon_tags_25 },
    { 0x49f92575, SUC_lexicon_tags_7 },
    { 0x72d92577, SUC_lexicon_tags_11 },
    { 0x1e392578, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x279257b, SUC_lexicon_tags_11 },
    { 0xceb9257b, SUC_lexicon_tags_17 },
    { 0x61b9257c, SUC_lexicon_tags_33 },
    { 0x8ab9257d, SUC_lexicon_tags_7 },
    { 0x1f19257f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x91f92582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2192586, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12992588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa59258b, SUC_lexicon_tags_17 },
    { 0x8d9258c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x57d9258e, SUC_lexicon_tags_23 },
    { 0x8b99258e, SUC_lexicon_tags_14 },
    { 0x7099258f, SUC_lexicon_tags_84 },
    { 0xd9392590, SUC_lexicon_tags_49 },
    { 0x6959258e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b92596, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd792598, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1159259b, SUC_lexicon_tags_6 },
    { 0xd5f9259c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf6d9259e, SUC_lexicon_tags_4 },
    { 0xa819259f, SUC_lexicon_tags_45 },
    { 0xf95925a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb97925a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775925ac, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f925b0, SUC_lexicon_tags_45 },
    { 0x1c7925b0, SUC_lexicon_tags_28 },
    { 0xf45925b0, SUC_lexicon_tags_25 },
    { 0x9e7925b3, SUC_lexicon_tags_2 },
    { 0x311925b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x561925b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe21925b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59925bc, SUC_lexicon_tags_66 },
    { 0xb1f925bd, SUC_lexicon_tags_46 },
    { 0x5c5925be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92f925c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x269925c2, SUC_lexicon_tags_10 },
    { 0x233925c2, SUC_lexicon_tags_2 },
    { 0x329925c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1925c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263925cf, SUC_lexicon_tags_2 },
    { 0x76f925d0, SUC_lexicon_tags_60 },
    { 0x705925d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb27925d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x389925d6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831925db, SUC_lexicon_tags_29 },
    { 0xfbf925db, SUC_lexicon_tags_0 },
    { 0x501925dc, SUC_lexicon_tags_15 },
    { 0xd2b925db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255925e4, SUC_lexicon_tags_7 },
    { 0x43d925e5, SUC_lexicon_tags_45 },
    { 0xa55925e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5925e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cb925eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99925ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53925f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab925f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x545925fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa392604, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f92608, SUC_lexicon_tags_47 },
    { 0xe5f92609, SUC_lexicon_tags_23 },
    { 0x1899260a, SUC_lexicon_tags_7 },
    { 0x3fb9260a, SUC_lexicon_tags_17 },
    { 0x3ef9260b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5f9260e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d92615, SUC_lexicon_tags_37 },
    { 0x99392616, SUC_lexicon_tags_33 },
    { 0xe0f92616, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd359261c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2979261e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f92621, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x65d92623, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb92627, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d592629, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8392633, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b392635, SUC_lexicon_tags_61 },
    { 0xb0992635, SUC_lexicon_tags_1 },
    { 0xe1d92636, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb9263c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb899263e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f92642, SUC_lexicon_tags_11 },
    { 0x3b592643, SUC_lexicon_tags_37 },
    { 0xd7992644, SUC_lexicon_tags_37 },
    { 0x8eb92645, SUC_lexicon_tags_23 },
    { 0x30792643, SUC_lexicon_tags_0 },
    { 0xfd92642, SUC_lexicon_tags_19 },
    { 0xaad92648, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc879264a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4859264d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9992653, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d992658, SUC_lexicon_tags_19 },
    { 0x10592659, SUC_lexicon_tags_33 },
    { 0x7dd9265a, SUC_lexicon_tags_37 },
    { 0x7399265b, SUC_lexicon_tags_1 },
    { 0xf139265a, SUC_lexicon_tags_70 },
    { 0xa1f9265c, SUC_lexicon_tags_17 },
    { 0x2459265e, SUC_lexicon_tags_2 },
    { 0x93192658, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3592662, SUC_lexicon_tags_36 },
    { 0x41792663, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4ad92665, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86f92667, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3792669, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f9266d, SUC_lexicon_tags_158 },
    { 0x7b99266e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c592670, SUC_lexicon_tags_60 },
    { 0xd7d92671, SUC_lexicon_tags_15 },
    { 0xebf92671, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f92675, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c392679, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9267d, SUC_lexicon_tags_19 },
    { 0xc719267d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb8b92680, SUC_lexicon_tags_60 },
    { 0x62792681, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x16b92683, SUC_lexicon_tags_12 },
    { 0x98792683, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xaad92686, SUC_lexicon_tags_17 },
    { 0xa4b92687, SUC_lexicon_tags_17 },
    { 0xec992687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa759268a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759268f, SUC_lexicon_tags_1 },
    { 0xc1992690, SUC_lexicon_tags_15 },
    { 0xb992690, SUC_lexicon_tags_27 },
    { 0x6a39268f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d92696, SUC_lexicon_tags_37 },
    { 0x5392697, SUC_lexicon_tags_9 },
    { 0x61d92696, SUC_lexicon_tags_46 },
    { 0x23592699, SUC_lexicon_tags_12 },
    { 0x79f9269a, SUC_lexicon_tags_15 },
    { 0x89f9269a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f9269f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d3926a1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9926a7, SUC_lexicon_tags_33 },
    { 0x807926a7, SUC_lexicon_tags_19 },
    { 0xa81926a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65f926ab, SUC_lexicon_tags_31 },
    { 0xc7b926ac, SUC_lexicon_tags_37 },
    { 0x3f1926ac, SUC_lexicon_tags_33 },
    { 0x48f926ac, SUC_lexicon_tags_27 },
    { 0x3c9926ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93926b4, SUC_lexicon_tags_11 },
    { 0x6ad926b5, SUC_lexicon_tags_17 },
    { 0x6ed926b4, SUC_lexicon_tags_5 },
    { 0xb43926b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9eb926b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2ed926bb, SUC_lexicon_tags_82 },
    { 0x9c1926bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x391926bf, SUC_lexicon_tags_11 },
    { 0x609926bf, SUC_lexicon_tags_31 },
    { 0xc51926c1, SUC_lexicon_tags_68 },
    { 0x273926c2, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4f1926c4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f926c7, SUC_lexicon_tags_15 },
    { 0xae7926c7, SUC_lexicon_tags_37 },
    { 0xa37926c7, SUC_lexicon_tags_2 },
    { 0xef9926ca, SUC_lexicon_tags_25 },
    { 0x39d926cb, SUC_lexicon_tags_25 },
    { 0xdb9926c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39926d3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3926d6, SUC_lexicon_tags_46 },
    { 0xb5b926d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f926db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x535926dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b926e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499926ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637926f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb5926f4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x74f926f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5926fe, SUC_lexicon_tags_12 },
    { 0x943926ff, SUC_lexicon_tags_38 },
    { 0x9bd926ff, SUC_lexicon_tags_512 },
    { 0xc3792701, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84392707, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaef92709, SUC_lexicon_tags_17 },
    { 0x51392709, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2019270c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46792713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9792717, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a59271a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3599271c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d9271f, SUC_lexicon_tags_66 },
    { 0xe059271f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb792723, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e192725, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x55992727, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b9272c, SUC_lexicon_tags_17 },
    { 0x9bf9272d, SUC_lexicon_tags_13 },
    { 0x7f59272e, SUC_lexicon_tags_12 },
    { 0x9d9272d, SUC_lexicon_tags_19 },
    { 0xb7792730, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7db92732, SUC_lexicon_tags_16 },
    { 0xe9592732, SUC_lexicon_tags_37 },
    { 0x46192734, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0992737, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb9273d, SUC_lexicon_tags_23 },
    { 0x819273d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab92741, SUC_lexicon_tags_11 },
    { 0xb5592742, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52992746, SUC_lexicon_tags_7 },
    { 0xc6392746, SUC_lexicon_tags_63 },
    { 0xedd92748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda59274c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xab19274e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d92754, SUC_lexicon_tags_55 },
    { 0xbbb92755, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46192759, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f9275c, SUC_lexicon_tags_65 },
    { 0x6499275d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb792765, SUC_lexicon_tags_618 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe639276a, SUC_lexicon_tags_1 },
    { 0xd099276b, SUC_lexicon_tags_60 },
    { 0x2339276a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21392770, SUC_lexicon_tags_8 },
    { 0x7992771, SUC_lexicon_tags_15 },
    { 0x45392772, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e192776, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34592786, SUC_lexicon_tags_27 },
    { 0x33792787, SUC_lexicon_tags_84 },
    { 0xaad92788, SUC_lexicon_tags_26 },
    { 0xccb92789, SUC_lexicon_tags_38 },
    { 0x999278a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e992797, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd9279d, SUC_lexicon_tags_19 },
    { 0x42f9279e, SUC_lexicon_tags_14 },
    { 0xa399279d, SUC_lexicon_tags_1 },
    { 0x19d9279e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107927a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4ad927a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf927ad, SUC_lexicon_tags_11 },
    { 0x47b927ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa49927b0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793927b6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9f7927b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d927bb, SUC_lexicon_tags_1 },
    { 0x1c3927bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b927c0, SUC_lexicon_tags_12 },
    { 0x29d927c1, SUC_lexicon_tags_19 },
    { 0x7ef927c0, SUC_lexicon_tags_8 },
    { 0x5e7927c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b927c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d927cb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2ef927cd, SUC_lexicon_tags_27 },
    { 0x5b5927ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09927d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4bf927d4, SUC_lexicon_tags_11 },
    { 0xe61927d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375927da, SUC_lexicon_tags_25 },
    { 0x1a9927db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb927e0, SUC_lexicon_tags_11 },
    { 0xd81927e1, SUC_lexicon_tags_1 },
    { 0xccd927e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf927e4, SUC_lexicon_tags_6 },
    { 0x43f927e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaa5927e7, SUC_lexicon_tags_17 },
    { 0x505927e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59927ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf927f0, SUC_lexicon_tags_33 },
    { 0x40f927f0, SUC_lexicon_tags_37 },
    { 0xbad927f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f927f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32992800, SUC_lexicon_tags_23 },
    { 0xf8192800, SUC_lexicon_tags_49 },
    { 0xc6792800, SUC_lexicon_tags_29 },
    { 0x3db92803, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4792806, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdf92808, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe939280d, SUC_lexicon_tags_26 },
    { 0x9c19280e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b92812, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3cd92814, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cb92818, SUC_lexicon_tags_31 },
    { 0x2b192819, SUC_lexicon_tags_8 },
    { 0xd1392819, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57792820, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa9992822, SUC_lexicon_tags_5 },
    { 0x88d92823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9eb92825, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f92829, SUC_lexicon_tags_45 },
    { 0x8c19282a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe79282c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd92833, SUC_lexicon_tags_11 },
    { 0x23b92834, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf92837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26392839, SUC_lexicon_tags_17 },
    { 0xf079283a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x592842, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccf92845, SUC_lexicon_tags_9 },
    { 0x2ef92846, SUC_lexicon_tags_8 },
    { 0x24b92846, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a99284a, SUC_lexicon_tags_14 },
    { 0xc0f9284a, SUC_lexicon_tags_29 },
    { 0x7cf9284c, SUC_lexicon_tags_7 },
    { 0x9c99284c, SUC_lexicon_tags_37 },
    { 0xda59284e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd92853, SUC_lexicon_tags_1 },
    { 0x61592854, SUC_lexicon_tags_25 },
    { 0x9c992853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bd92857, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x10792859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9285d, SUC_lexicon_tags_15 },
    { 0x5539285d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b92863, SUC_lexicon_tags_17 },
    { 0x592864, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5992866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f92869, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f79286c, SUC_lexicon_tags_8 },
    { 0x15b9286d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x99592870, SUC_lexicon_tags_47 },
    { 0x6992871, SUC_lexicon_tags_15 },
    { 0x66992871, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f92875, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x792879, SUC_lexicon_tags_32 },
    { 0x6f79287a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b9287e, SUC_lexicon_tags_25 },
    { 0xd5b9287f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7992882, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d92887, SUC_lexicon_tags_7 },
    { 0x71992888, SUC_lexicon_tags_10 },
    { 0x67392889, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6992894, SUC_lexicon_tags_25 },
    { 0xff392894, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af92899, SUC_lexicon_tags_34 },
    { 0x2c79289a, SUC_lexicon_tags_2 },
    { 0x74f9289a, SUC_lexicon_tags_27 },
    { 0x4519289c, SUC_lexicon_tags_14 },
    { 0x1a19289b, SUC_lexicon_tags_37 },
    { 0x8ed9289e, SUC_lexicon_tags_45 },
    { 0x24f9289f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d928a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1928a7, SUC_lexicon_tags_29 },
    { 0x39f928a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x21928ab, SUC_lexicon_tags_17 },
    { 0x607928ac, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xacd928ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcd928b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5928b3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1928b9, SUC_lexicon_tags_12 },
    { 0xdb7928ba, SUC_lexicon_tags_37 },
    { 0x381928bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x60f928be, SUC_lexicon_tags_75 },
    { 0xb9f928bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d928c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31b928c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbed928c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199928ca, SUC_lexicon_tags_15 },
    { 0x72f928cb, SUC_lexicon_tags_37 },
    { 0x4f1928cb, SUC_lexicon_tags_45 },
    { 0x247928cc, SUC_lexicon_tags_37 },
    { 0x9f1928ce, SUC_lexicon_tags_37 },
    { 0x3d9928cb, SUC_lexicon_tags_23 },
    { 0x4b5928cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe1928d2, SUC_lexicon_tags_31 },
    { 0x6cb928d3, SUC_lexicon_tags_14 },
    { 0xac9928d4, SUC_lexicon_tags_2 },
    { 0x225928d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x27d928d7, SUC_lexicon_tags_12 },
    { 0x25f928d8, SUC_lexicon_tags_7 },
    { 0xc83928d9, SUC_lexicon_tags_409 },
    { 0, NULL },
    { 0, NULL },
    { 0x451928dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679928e0, SUC_lexicon_tags_6 },
    { 0x1b5928e1, SUC_lexicon_tags_59 },
    { 0x503928e0, SUC_lexicon_tags_512 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5928e9, SUC_lexicon_tags_160 },
    { 0x913928ea, SUC_lexicon_tags_33 },
    { 0x46b928eb, SUC_lexicon_tags_5 },
    { 0x2e7928eb, SUC_lexicon_tags_31 },
    { 0x539928ea, SUC_lexicon_tags_32 },
    { 0xca5928ee, SUC_lexicon_tags_15 },
    { 0xcd3928ef, SUC_lexicon_tags_37 },
    { 0xc19928ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45928f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385928fc, SUC_lexicon_tags_37 },
    { 0x9cb928fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7392901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41192907, SUC_lexicon_tags_1 },
    { 0xc7192907, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe839290b, SUC_lexicon_tags_15 },
    { 0x9939290c, SUC_lexicon_tags_2 },
    { 0x72d9290d, SUC_lexicon_tags_28 },
    { 0x1899290e, SUC_lexicon_tags_23 },
    { 0x2f99290b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x97f92912, SUC_lexicon_tags_0 },
    { 0xc192913, SUC_lexicon_tags_12 },
    { 0x31992914, SUC_lexicon_tags_29 },
    { 0x75592913, SUC_lexicon_tags_11 },
    { 0xb8392915, SUC_lexicon_tags_6 },
    { 0xe7b92917, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f39291b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb099291d, SUC_lexicon_tags_2 },
    { 0x3df9291e, SUC_lexicon_tags_0 },
    { 0xc599291d, SUC_lexicon_tags_37 },
    { 0xa8d92920, SUC_lexicon_tags_26 },
    { 0x2bb92921, SUC_lexicon_tags_1 },
    { 0x74992922, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x42b92924, SUC_lexicon_tags_31 },
    { 0x9d592924, SUC_lexicon_tags_33 },
    { 0x18f92926, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xacf92928, SUC_lexicon_tags_393 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b99292b, SUC_lexicon_tags_27 },
    { 0xdbd9292c, SUC_lexicon_tags_160 },
    { 0x6ab9292c, SUC_lexicon_tags_23 },
    { 0x52f9292e, SUC_lexicon_tags_65 },
    { 0x9e39292f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e792931, SUC_lexicon_tags_14 },
    { 0xc0992931, SUC_lexicon_tags_12 },
    { 0x5b92933, SUC_lexicon_tags_0 },
    { 0xd1f92934, SUC_lexicon_tags_1 },
    { 0x42192933, SUC_lexicon_tags_2 },
    { 0xdb192933, SUC_lexicon_tags_11 },
    { 0xbdd92937, SUC_lexicon_tags_25 },
    { 0x8ad92937, SUC_lexicon_tags_11 },
    { 0x34592939, SUC_lexicon_tags_15 },
    { 0xa5b92938, SUC_lexicon_tags_55 },
    { 0x7759293b, SUC_lexicon_tags_12 },
    { 0xf7b9293c, SUC_lexicon_tags_28 },
    { 0x9a192937, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f19293f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbf792941, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b992947, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d59294a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9294d, SUC_lexicon_tags_68 },
    { 0x6359294e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56f92950, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd92957, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59295f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b92964, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d9296b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd639296f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd92976, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d392981, SUC_lexicon_tags_31 },
    { 0xb1192982, SUC_lexicon_tags_60 },
    { 0xde792983, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3b792985, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3719298d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4192990, SUC_lexicon_tags_7 },
    { 0xad792991, SUC_lexicon_tags_49 },
    { 0x87792991, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d92996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90392998, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x339299a, SUC_lexicon_tags_12 },
    { 0xa0f9299b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16d929a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d929a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3929ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f929b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5929ba, SUC_lexicon_tags_2 },
    { 0x411929bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43929c1, SUC_lexicon_tags_27 },
    { 0x2f1929c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f929c6, SUC_lexicon_tags_60 },
    { 0x769929c7, SUC_lexicon_tags_47 },
    { 0xf35929c6, SUC_lexicon_tags_37 },
    { 0x64f929c9, SUC_lexicon_tags_0 },
    { 0x32d929c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x601929cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c9929cf, SUC_lexicon_tags_46 },
    { 0x1b1929cf, SUC_lexicon_tags_12 },
    { 0xb1f929d1, SUC_lexicon_tags_33 },
    { 0x6a3929d2, SUC_lexicon_tags_1 },
    { 0x155929d1, SUC_lexicon_tags_84 },
    { 0xae7929d3, SUC_lexicon_tags_17 },
    { 0x1a3929d5, SUC_lexicon_tags_30 },
    { 0xff5929d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5929d9, SUC_lexicon_tags_57 },
    { 0x7e7929da, SUC_lexicon_tags_12 },
    { 0x959929d9, SUC_lexicon_tags_29 },
    { 0xebf929db, SUC_lexicon_tags_38 },
    { 0xcf929dd, SUC_lexicon_tags_9 },
    { 0x90f929de, SUC_lexicon_tags_15 },
    { 0xcbb929df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecb929e1, SUC_lexicon_tags_15 },
    { 0xc4d929e1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x217929e4, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9929ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x62f929ec, SUC_lexicon_tags_66 },
    { 0xb1b929ed, SUC_lexicon_tags_37 },
    { 0x97b929ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4b929f0, SUC_lexicon_tags_37 },
    { 0x7e1929f0, SUC_lexicon_tags_37 },
    { 0xf47929f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3929f6, SUC_lexicon_tags_37 },
    { 0xf929f6, SUC_lexicon_tags_25 },
    { 0xbaf929f7, SUC_lexicon_tags_3 },
    { 0xbdd929f6, SUC_lexicon_tags_25 },
    { 0x80d929fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9929fe, SUC_lexicon_tags_25 },
    { 0xde1929ff, SUC_lexicon_tags_27 },
    { 0xd5792a00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b992a07, SUC_lexicon_tags_0 },
    { 0xa6d92a08, SUC_lexicon_tags_25 },
    { 0x23392a09, SUC_lexicon_tags_77 },
    { 0x5e192a09, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d792a0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f92a18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14592a1c, SUC_lexicon_tags_11 },
    { 0xd8992a1c, SUC_lexicon_tags_106 },
    { 0xd4592a1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d92a23, SUC_lexicon_tags_23 },
    { 0xb0792a24, SUC_lexicon_tags_27 },
    { 0x76792a24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d792a2a, SUC_lexicon_tags_7 },
    { 0xeb992a2b, SUC_lexicon_tags_11 },
    { 0xb2192a2c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d92a34, SUC_lexicon_tags_33 },
    { 0xfd992a35, SUC_lexicon_tags_229 },
    { 0x19592a35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d92a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f392a3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7592a40, SUC_lexicon_tags_15 },
    { 0x11992a41, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc192a44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde192a4c, SUC_lexicon_tags_37 },
    { 0x24f92a4c, SUC_lexicon_tags_35 },
    { 0x1592a4e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3992a51, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1c992a53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd92a56, SUC_lexicon_tags_4 },
    { 0x50192a57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab992a5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb792a5e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x12d92a60, SUC_lexicon_tags_37 },
    { 0x4d392a61, SUC_lexicon_tags_69 },
    { 0x7ff92a62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5592a64, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xf4792a66, SUC_lexicon_tags_15 },
    { 0x90792a67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f192a6a, SUC_lexicon_tags_13 },
    { 0x27792a6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b92a70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1a192a72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55792a74, SUC_lexicon_tags_14 },
    { 0x8ab92a75, SUC_lexicon_tags_6 },
    { 0x8cf92a75, SUC_lexicon_tags_11 },
    { 0xa5792a74, SUC_lexicon_tags_11 },
    { 0xd1192a78, SUC_lexicon_tags_26 },
    { 0xb2f92a78, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a592a7f, SUC_lexicon_tags_15 },
    { 0xaff92a80, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x63392a83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f792a87, SUC_lexicon_tags_9 },
    { 0xcd392a87, SUC_lexicon_tags_25 },
    { 0xfc992a89, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca792a8f, SUC_lexicon_tags_12 },
    { 0x31592a90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f192a95, SUC_lexicon_tags_37 },
    { 0x35992a95, SUC_lexicon_tags_15 },
    { 0xe7d92a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd92a9a, SUC_lexicon_tags_84 },
    { 0x7e792a9b, SUC_lexicon_tags_60 },
    { 0xf6f92a9a, SUC_lexicon_tags_27 },
    { 0x95992a9c, SUC_lexicon_tags_38 },
    { 0x9cb92a9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff92aa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ab92aa3, SUC_lexicon_tags_66 },
    { 0xea592aa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7192aa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9392aaa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f92aad, SUC_lexicon_tags_23 },
    { 0xa5d92aae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b92ab1, SUC_lexicon_tags_11 },
    { 0x68992ab2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x44992ab5, SUC_lexicon_tags_2 },
    { 0xc1192ab5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81392abc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe992abe, SUC_lexicon_tags_15 },
    { 0x11b92abf, SUC_lexicon_tags_1 },
    { 0xf7f92ac0, SUC_lexicon_tags_1 },
    { 0xe4792ac0, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xbb392ac3, SUC_lexicon_tags_9 },
    { 0x4792ac4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb392ac8, SUC_lexicon_tags_43 },
    { 0xff992ac8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec192ad1, SUC_lexicon_tags_37 },
    { 0xb6792ad1, SUC_lexicon_tags_63 },
    { 0x4ad92ad3, SUC_lexicon_tags_12 },
    { 0x51b92ad3, SUC_lexicon_tags_38 },
    { 0xd2192ad3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed92ad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5592ada, SUC_lexicon_tags_0 },
    { 0xd5d92adb, SUC_lexicon_tags_17 },
    { 0x3cf92adb, SUC_lexicon_tags_19 },
    { 0xf3b92add, SUC_lexicon_tags_11 },
    { 0xa2792ada, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3f92ae0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f92ae5, SUC_lexicon_tags_31 },
    { 0xbb92ae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8592aea, SUC_lexicon_tags_17 },
    { 0x5f192aeb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30192af0, SUC_lexicon_tags_17 },
    { 0x63592af1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df92af6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48792afb, SUC_lexicon_tags_17 },
    { 0x19592afc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a992aff, SUC_lexicon_tags_37 },
    { 0x8b192aff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d92b07, SUC_lexicon_tags_1 },
    { 0xd4f92b08, SUC_lexicon_tags_57 },
    { 0x20592b09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf3d92b0b, SUC_lexicon_tags_47 },
    { 0xe1f92b0b, SUC_lexicon_tags_235 },
    { 0x7ff92b0d, SUC_lexicon_tags_144 },
    { 0x99792b0d, SUC_lexicon_tags_25 },
    { 0x27b92b0c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x62d92b11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d592b13, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2dd92b15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7392b17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94192b1d, SUC_lexicon_tags_53 },
    { 0xa8f92b1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed92b26, SUC_lexicon_tags_44 },
    { 0x3f992b27, SUC_lexicon_tags_11 },
    { 0x1cd92b27, SUC_lexicon_tags_26 },
    { 0x48392b29, SUC_lexicon_tags_1 },
    { 0xc2d92b27, SUC_lexicon_tags_10 },
    { 0x5e592b26, SUC_lexicon_tags_37 },
    { 0x63392b2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7592b2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27392b34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c992b38, SUC_lexicon_tags_113 },
    { 0xdfb92b38, SUC_lexicon_tags_12 },
    { 0xff392b3a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb92b3d, SUC_lexicon_tags_9 },
    { 0xa392b3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff92b43, SUC_lexicon_tags_25 },
    { 0x67992b43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8392b47, SUC_lexicon_tags_38 },
    { 0x1fd92b47, SUC_lexicon_tags_30 },
    { 0x192b49, SUC_lexicon_tags_25 },
    { 0x6c992b48, SUC_lexicon_tags_2 },
    { 0xa4792b47, SUC_lexicon_tags_60 },
    { 0xa5192b4c, SUC_lexicon_tags_11 },
    { 0x39392b49, SUC_lexicon_tags_37 },
    { 0xe5d92b49, SUC_lexicon_tags_12 },
    { 0xfed92b47, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df92b52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xafd92b54, SUC_lexicon_tags_7 },
    { 0xf992b54, SUC_lexicon_tags_25 },
    { 0x54592b56, SUC_lexicon_tags_14 },
    { 0x5f192b57, SUC_lexicon_tags_2 },
    { 0x23f92b56, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0xaa392b5a, SUC_lexicon_tags_63 },
    { 0x89792b5b, SUC_lexicon_tags_61 },
    { 0x49192b5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab92b64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ab92b66, SUC_lexicon_tags_60 },
    { 0x39392b66, SUC_lexicon_tags_37 },
    { 0xf3592b68, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x13792b6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f92b6f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0992b75, SUC_lexicon_tags_12 },
    { 0xbaf92b76, SUC_lexicon_tags_43 },
    { 0x29392b77, SUC_lexicon_tags_15 },
    { 0x5a992b78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x10792b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41792b7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44792b83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x92392b85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6992b8d, SUC_lexicon_tags_15 },
    { 0x23d92b8d, SUC_lexicon_tags_2 },
    { 0xf5792b8f, SUC_lexicon_tags_351 },
    { 0xc7b92b90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e792b92, SUC_lexicon_tags_38 },
    { 0x90792b92, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2cf92b95, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c192b9c, SUC_lexicon_tags_43 },
    { 0xd8792b9d, SUC_lexicon_tags_17 },
    { 0xeed92b9c, SUC_lexicon_tags_55 },
    { 0x23592b9c, SUC_lexicon_tags_1433 },
    { 0xb0b92ba0, SUC_lexicon_tags_17 },
    { 0x28792ba1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde792ba6, SUC_lexicon_tags_12 },
    { 0xc0592ba7, SUC_lexicon_tags_9 },
    { 0x55f92ba7, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b92bac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3392baf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9592bb3, SUC_lexicon_tags_45 },
    { 0x9a592bb4, SUC_lexicon_tags_12 },
    { 0x3b92bb5, SUC_lexicon_tags_8 },
    { 0x86f92bb3, SUC_lexicon_tags_26 },
    { 0xe7b92bb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3e192bb9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb192bbc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d92bc4, SUC_lexicon_tags_37 },
    { 0x6a392bc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1fb92bc7, SUC_lexicon_tags_282 },
    { 0x58792bc8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d92bcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c392bd1, SUC_lexicon_tags_34 },
    { 0x7c392bd2, SUC_lexicon_tags_10 },
    { 0x2d992bd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x65992bd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65392bdc, SUC_lexicon_tags_11 },
    { 0xc2f92bdd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85992be0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26392be5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe192be7, SUC_lexicon_tags_2 },
    { 0xb9f92be7, SUC_lexicon_tags_15 },
    { 0xfd592be7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78f92beb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47b92bed, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc5592bef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53592bf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1992bf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb592bfc, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa7992bfe, SUC_lexicon_tags_65 },
    { 0xf0f92bff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d592c05, SUC_lexicon_tags_45 },
    { 0xe0f92c05, SUC_lexicon_tags_8 },
    { 0x3fd92c07, SUC_lexicon_tags_38 },
    { 0x44992c05, SUC_lexicon_tags_7 },
    { 0xc7992c05, SUC_lexicon_tags_45 },
    { 0xf1192c0a, SUC_lexicon_tags_14 },
    { 0x7af92c0a, SUC_lexicon_tags_10 },
    { 0x99192c0c, SUC_lexicon_tags_0 },
    { 0x29392c0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f92c12, SUC_lexicon_tags_33 },
    { 0xc7192c12, SUC_lexicon_tags_18 },
    { 0x93992c14, SUC_lexicon_tags_2 },
    { 0xd7392c15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8792c17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d92c1d, SUC_lexicon_tags_0 },
    { 0xb192c1e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef92c21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f92c24, SUC_lexicon_tags_23 },
    { 0x3dd92c25, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0xe8d92c27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f92c2f, SUC_lexicon_tags_32 },
    { 0xe0b92c2f, SUC_lexicon_tags_7 },
    { 0x19b92c30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf9f92c33, SUC_lexicon_tags_63 },
    { 0x2e392c33, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e392c39, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x45592c3b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3392c3f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4392c43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d92c4b, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd392c4e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x90392c51, SUC_lexicon_tags_14 },
    { 0x57792c52, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d92c55, SUC_lexicon_tags_7 },
    { 0xa992c55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3992c58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb7192c5a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d92c5e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19992c65, SUC_lexicon_tags_12 },
    { 0x66b92c66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf92c6d, SUC_lexicon_tags_39 },
    { 0xdb992c6d, SUC_lexicon_tags_14 },
    { 0xda792c6f, SUC_lexicon_tags_12 },
    { 0xba792c70, SUC_lexicon_tags_25 },
    { 0xe8f92c6d, SUC_lexicon_tags_11 },
    { 0xf6192c72, SUC_lexicon_tags_43 },
    { 0x4192c73, SUC_lexicon_tags_27 },
    { 0xbaf92c73, SUC_lexicon_tags_25 },
    { 0xe6992c75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9392c7a, SUC_lexicon_tags_25 },
    { 0x58392c7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45b92c7d, SUC_lexicon_tags_208 },
    { 0x16f92c7d, SUC_lexicon_tags_45 },
    { 0xf0992c7d, SUC_lexicon_tags_19 },
    { 0x5192c80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60592c88, SUC_lexicon_tags_160 },
    { 0xb2392c89, SUC_lexicon_tags_25 },
    { 0xd1f92c89, SUC_lexicon_tags_3 },
    { 0x7ad92c8b, SUC_lexicon_tags_25 },
    { 0x55792c8c, SUC_lexicon_tags_37 },
    { 0x46392c8d, SUC_lexicon_tags_68 },
    { 0x1a592c8e, SUC_lexicon_tags_17 },
    { 0x76b92c8f, SUC_lexicon_tags_11 },
    { 0xfd392c90, SUC_lexicon_tags_7 },
    { 0x63392c91, SUC_lexicon_tags_7 },
    { 0xbf92c88, SUC_lexicon_tags_57 },
    { 0xc792c93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x34592c96, SUC_lexicon_tags_10 },
    { 0x12392c97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98792c9d, SUC_lexicon_tags_12 },
    { 0x1f392c9d, SUC_lexicon_tags_12 },
    { 0x83d92c9f, SUC_lexicon_tags_17 },
    { 0xdd592c9f, SUC_lexicon_tags_87 },
    { 0x8d592ca1, SUC_lexicon_tags_9 },
    { 0xefd92c9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x26792ca5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb392ca7, SUC_lexicon_tags_11 },
    { 0xfdb92ca8, SUC_lexicon_tags_15 },
    { 0x2c392ca8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd5192cab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49b92cad, SUC_lexicon_tags_19 },
    { 0x72592cad, SUC_lexicon_tags_11 },
    { 0x47592caf, SUC_lexicon_tags_17 },
    { 0x7df92cad, SUC_lexicon_tags_48 },
    { 0x9ed92cb1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c192cb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8bb92cb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb92cc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3592cce, SUC_lexicon_tags_2 },
    { 0x22192ccf, SUC_lexicon_tags_170 },
    { 0xde992cd0, SUC_lexicon_tags_37 },
    { 0x1c992cd1, SUC_lexicon_tags_2 },
    { 0x6fb92cd2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x59792cd5, SUC_lexicon_tags_12 },
    { 0x9a992cd6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70792cdb, SUC_lexicon_tags_27 },
    { 0x4f192cdc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42992ce1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7d592ce3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4d992ce5, SUC_lexicon_tags_7 },
    { 0xe8992ce5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1cf92ce8, SUC_lexicon_tags_12 },
    { 0xc8592ce9, SUC_lexicon_tags_23 },
    { 0x87592ce9, SUC_lexicon_tags_48 },
    { 0x27192cea, SUC_lexicon_tags_40 },
    { 0xcdf92cea, SUC_lexicon_tags_13 },
    { 0xbbb92ce8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2392cf1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb792cf7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4b392cf9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x27592cfc, SUC_lexicon_tags_38 },
    { 0x8e592cfd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x93992cff, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab992d06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29192d0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77b92d0d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcdd92d0f, SUC_lexicon_tags_933 },
    { 0x65992d10, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f92d13, SUC_lexicon_tags_37 },
    { 0x57b92d13, SUC_lexicon_tags_37 },
    { 0x1cb92d15, SUC_lexicon_tags_7 },
    { 0xeeb92d16, SUC_lexicon_tags_14 },
    { 0xf3d92d17, SUC_lexicon_tags_11 },
    { 0xbcb92d17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b92d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4992d20, SUC_lexicon_tags_6 },
    { 0xf8592d21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af92d26, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x64992d29, SUC_lexicon_tags_25 },
    { 0x4fb92d29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d92d31, SUC_lexicon_tags_62 },
    { 0xdf592d32, SUC_lexicon_tags_1 },
    { 0xbed92d31, SUC_lexicon_tags_15 },
    { 0xa3192d34, SUC_lexicon_tags_43 },
    { 0x9f992d34, SUC_lexicon_tags_38 },
    { 0x18d92d36, SUC_lexicon_tags_39 },
    { 0x8eb92d37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb92d3c, SUC_lexicon_tags_16 },
    { 0x54d92d3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcad92d3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78592d41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a792d43, SUC_lexicon_tags_11 },
    { 0x2d792d43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2192d48, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd0192d4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85192d4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb92d53, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a192d56, SUC_lexicon_tags_60 },
    { 0x82f92d57, SUC_lexicon_tags_11 },
    { 0x5b192d58, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5792d5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x84392d60, SUC_lexicon_tags_69 },
    { 0x55b92d61, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd92d64, SUC_lexicon_tags_38 },
    { 0xfd92d65, SUC_lexicon_tags_15 },
    { 0x5b992d66, SUC_lexicon_tags_23 },
    { 0xef192d64, SUC_lexicon_tags_11 },
    { 0x67f92d64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66592d6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd992d75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0392d77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7792d81, SUC_lexicon_tags_29 },
    { 0x5f392d82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb0b92d84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8d792d86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd92d88, SUC_lexicon_tags_29 },
    { 0x84992d88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1b92d8b, SUC_lexicon_tags_12 },
    { 0x8af92d8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8592d90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd92d94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2792d96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b92d9a, SUC_lexicon_tags_2 },
    { 0xaad92d9a, SUC_lexicon_tags_9 },
    { 0x1d792d9c, SUC_lexicon_tags_17 },
    { 0x54792d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea792d9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5db92da1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2992da4, SUC_lexicon_tags_37 },
    { 0x45b92da5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8592da8, SUC_lexicon_tags_2 },
    { 0x79792da8, SUC_lexicon_tags_7 },
    { 0xdeb92da9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab92daf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbbb92db1, SUC_lexicon_tags_25 },
    { 0xb2792db2, SUC_lexicon_tags_9 },
    { 0x99592db2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c792db8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8592dbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96992dc1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66192dc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b192dca, SUC_lexicon_tags_15 },
    { 0x26b92dcb, SUC_lexicon_tags_45 },
    { 0x9b592dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79392dd2, SUC_lexicon_tags_28 },
    { 0xecd92dd3, SUC_lexicon_tags_14 },
    { 0xdf992dd3, SUC_lexicon_tags_37 },
    { 0x58992dd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a592dd9, SUC_lexicon_tags_15 },
    { 0x31792dda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf92ddf, SUC_lexicon_tags_2 },
    { 0x6e592de0, SUC_lexicon_tags_25 },
    { 0x3ed92de1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb92de8, SUC_lexicon_tags_14 },
    { 0x4b392de8, SUC_lexicon_tags_28 },
    { 0xdcf92dea, SUC_lexicon_tags_4 },
    { 0x7cf92deb, SUC_lexicon_tags_28 },
    { 0x31b92dec, SUC_lexicon_tags_2 },
    { 0x89392dec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f92df2, SUC_lexicon_tags_1 },
    { 0x35d92df3, SUC_lexicon_tags_37 },
    { 0x6a192df2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c992df9, SUC_lexicon_tags_80 },
    { 0x26192dfa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b92e00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa1b92e02, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f92e06, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xae792e08, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37192e0c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8b792e0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e392e11, SUC_lexicon_tags_17 },
    { 0x5592e12, SUC_lexicon_tags_47 },
    { 0x51392e11, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xdbd92e15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f992e17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0192e1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd992e24, SUC_lexicon_tags_37 },
    { 0x60f92e25, SUC_lexicon_tags_14 },
    { 0x52b92e26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4192e2b, SUC_lexicon_tags_11 },
    { 0xb9d92e2c, SUC_lexicon_tags_23 },
    { 0xf2f92e2c, SUC_lexicon_tags_7 },
    { 0xd0f92e2e, SUC_lexicon_tags_45 },
    { 0xb7992e2f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d92e32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2392e39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f92e3d, SUC_lexicon_tags_2 },
    { 0x5b192e3d, SUC_lexicon_tags_11 },
    { 0x46992e3f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7cd92e41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab92e46, SUC_lexicon_tags_12 },
    { 0x9f192e46, SUC_lexicon_tags_1 },
    { 0x68d92e46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77192e4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6992e51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43992e53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7192e57, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d792e5b, SUC_lexicon_tags_455 },
    { 0x5df92e5c, SUC_lexicon_tags_15 },
    { 0x7f992e5c, SUC_lexicon_tags_43 },
    { 0x9f192e5e, SUC_lexicon_tags_47 },
    { 0xc0192e5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cb92e61, SUC_lexicon_tags_60 },
    { 0x2c392e61, SUC_lexicon_tags_17 },
    { 0x64992e63, SUC_lexicon_tags_17 },
    { 0x22192e64, SUC_lexicon_tags_38 },
    { 0x74992e63, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x8fd92e67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53f92e69, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xca592e6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab92e6e, SUC_lexicon_tags_68 },
    { 0xff392e6e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6992e73, SUC_lexicon_tags_6 },
    { 0xacf92e74, SUC_lexicon_tags_27 },
    { 0xce592e75, SUC_lexicon_tags_480 },
    { 0x54d92e73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d92e7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b92e7f, SUC_lexicon_tags_11 },
    { 0xb5b92e80, SUC_lexicon_tags_65 },
    { 0xfc792e80, SUC_lexicon_tags_47 },
    { 0x70992e82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56792e84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60b92e86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffd92e88, SUC_lexicon_tags_62 },
    { 0xe9392e89, SUC_lexicon_tags_23 },
    { 0x32992e8a, SUC_lexicon_tags_2 },
    { 0x23d92e8b, SUC_lexicon_tags_7 },
    { 0x94d92e8a, SUC_lexicon_tags_26 },
    { 0xde992e8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6b92e94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6392e9b, SUC_lexicon_tags_1 },
    { 0x9ad92e9b, SUC_lexicon_tags_37 },
    { 0xe6b92e9c, SUC_lexicon_tags_45 },
    { 0xa4792e9e, SUC_lexicon_tags_60 },
    { 0xddb92e9e, SUC_lexicon_tags_15 },
    { 0xd2592e9f, SUC_lexicon_tags_14 },
    { 0x5f92ea0, SUC_lexicon_tags_11 },
    { 0x81b92ea2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f592ea8, SUC_lexicon_tags_28 },
    { 0x36592ea9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6192eab, SUC_lexicon_tags_37 },
    { 0x74d92eab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83392eae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0992eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1192eb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b92ebb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e792ebd, SUC_lexicon_tags_70 },
    { 0x7f192ebe, SUC_lexicon_tags_26 },
    { 0x49f92ebe, SUC_lexicon_tags_37 },
    { 0xb3592ec0, SUC_lexicon_tags_11 },
    { 0x85d92ec0, SUC_lexicon_tags_63 },
    { 0xbb792ec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d392ec4, SUC_lexicon_tags_37 },
    { 0x8ab92ec5, SUC_lexicon_tags_3 },
    { 0x55392ec4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b992ece, SUC_lexicon_tags_47 },
    { 0x1d392ecf, SUC_lexicon_tags_25 },
    { 0x3e992ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4992ed2, SUC_lexicon_tags_15 },
    { 0x3192ed3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5392eda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14392ede, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2392ee0, SUC_lexicon_tags_15 },
    { 0xdd192ee0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x32b92ee3, SUC_lexicon_tags_37 },
    { 0xf4f92ee4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d92ee8, SUC_lexicon_tags_7 },
    { 0xf8792ee8, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b92eec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdc792eee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f92ef1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d92ef8, SUC_lexicon_tags_17 },
    { 0xe8392ef9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6992efc, SUC_lexicon_tags_45 },
    { 0x6eb92efd, SUC_lexicon_tags_12 },
    { 0x49b92efe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61992f04, SUC_lexicon_tags_19 },
    { 0xeeb92f04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b92f09, SUC_lexicon_tags_2 },
    { 0xd3392f0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c592f0c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf92f10, SUC_lexicon_tags_38 },
    { 0x1c592f10, SUC_lexicon_tags_46 },
    { 0x32b92f12, SUC_lexicon_tags_15 },
    { 0x90192f11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44192f15, SUC_lexicon_tags_1 },
    { 0x16f92f16, SUC_lexicon_tags_17 },
    { 0x12192f16, SUC_lexicon_tags_85 },
    { 0xbc992f18, SUC_lexicon_tags_1 },
    { 0x3a592f19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5992f1b, SUC_lexicon_tags_9 },
    { 0x3d392f1c, SUC_lexicon_tags_37 },
    { 0xdef92f1b, SUC_lexicon_tags_39 },
    { 0xe5f92f1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ef92f20, SUC_lexicon_tags_12 },
    { 0xa4392f20, SUC_lexicon_tags_12 },
    { 0x8f792f21, SUC_lexicon_tags_33 },
    { 0xf3192f23, SUC_lexicon_tags_21 },
    { 0x31392f24, SUC_lexicon_tags_25 },
    { 0x47d92f20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd192f29, SUC_lexicon_tags_31 },
    { 0x90d92f2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b92f2e, SUC_lexicon_tags_6 },
    { 0x8b992f2f, SUC_lexicon_tags_14 },
    { 0xae392f30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded92f37, SUC_lexicon_tags_37 },
    { 0x60192f38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27192f3c, SUC_lexicon_tags_6 },
    { 0x22192f3d, SUC_lexicon_tags_21 },
    { 0xc5592f3d, SUC_lexicon_tags_15 },
    { 0xde192f3e, SUC_lexicon_tags_7 },
    { 0xcb992f3d, SUC_lexicon_tags_32 },
    { 0x90792f41, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1c392f43, SUC_lexicon_tags_2 },
    { 0x31d92f43, SUC_lexicon_tags_43 },
    { 0x6ab92f45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd92f48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef92f4c, SUC_lexicon_tags_85 },
    { 0xebb92f4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51f92f4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc0992f51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d992f5b, SUC_lexicon_tags_11 },
    { 0xe6f92f5c, SUC_lexicon_tags_25 },
    { 0xc6d92f5d, SUC_lexicon_tags_11 },
    { 0xbfb92f5e, SUC_lexicon_tags_2 },
    { 0x77d92f5e, SUC_lexicon_tags_37 },
    { 0xc0f92f60, SUC_lexicon_tags_16 },
    { 0x8cd92f61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf92f65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e192f67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9592f6a, SUC_lexicon_tags_11 },
    { 0xd2d92f6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bb92f6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f92f70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a792f76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3f392f78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x13392f7a, SUC_lexicon_tags_11 },
    { 0xebb92f7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef92f7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a592f80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce192f84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c792f89, SUC_lexicon_tags_1 },
    { 0xc992f8a, SUC_lexicon_tags_19 },
    { 0x33392f8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x32592f8e, SUC_lexicon_tags_7 },
    { 0x3fd92f8f, SUC_lexicon_tags_15 },
    { 0x77592f8e, SUC_lexicon_tags_60 },
    { 0x47b92f91, SUC_lexicon_tags_11 },
    { 0x24792f92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cf92f94, SUC_lexicon_tags_29 },
    { 0xdb392f95, SUC_lexicon_tags_34 },
    { 0x48b92f95, SUC_lexicon_tags_15 },
    { 0xe4792f94, SUC_lexicon_tags_33 },
    { 0xf2b92f98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e592f9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d92f9f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x52f92fa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d392fa4, SUC_lexicon_tags_14 },
    { 0xf2d92fa5, SUC_lexicon_tags_30 },
    { 0xb4f92fa4, SUC_lexicon_tags_7 },
    { 0x86792fa5, SUC_lexicon_tags_37 },
    { 0x62f92fa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79992fb1, SUC_lexicon_tags_39 },
    { 0xaa792fb2, SUC_lexicon_tags_12 },
    { 0x52b92fb3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72592fb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b92fb8, SUC_lexicon_tags_15 },
    { 0x65392fb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf792fbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce792fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d92fd1, SUC_lexicon_tags_37 },
    { 0xbb392fd2, SUC_lexicon_tags_35 },
    { 0x82392fd2, SUC_lexicon_tags_11 },
    { 0x75f92fd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3c792fd6, SUC_lexicon_tags_29 },
    { 0x7d592fd7, SUC_lexicon_tags_7 },
    { 0x31b92fd8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x85d92fda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f992fdd, SUC_lexicon_tags_63 },
    { 0xc9592fdd, SUC_lexicon_tags_33 },
    { 0x71392fdd, SUC_lexicon_tags_47 },
    { 0xab92fde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e192fe2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8d92fe4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4592fe7, SUC_lexicon_tags_47 },
    { 0xbed92fe7, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12792fed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcd592fef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf1792ff1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87992ffe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b93002, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44793006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91993008, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae99300e, SUC_lexicon_tags_5 },
    { 0xef79300f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a393013, SUC_lexicon_tags_15 },
    { 0x8b393013, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8793017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef9301a, SUC_lexicon_tags_31 },
    { 0xf559301a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e793028, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda59302e, SUC_lexicon_tags_30 },
    { 0x79b9302f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8793033, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa7f93035, SUC_lexicon_tags_25 },
    { 0xcdf93035, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57193038, SUC_lexicon_tags_1 },
    { 0x25f93039, SUC_lexicon_tags_48 },
    { 0x7e79303a, SUC_lexicon_tags_19 },
    { 0x6519303a, SUC_lexicon_tags_43 },
    { 0x3559303a, SUC_lexicon_tags_38 },
    { 0xb339303d, SUC_lexicon_tags_15 },
    { 0x9679303e, SUC_lexicon_tags_27 },
    { 0xce59303e, SUC_lexicon_tags_16 },
    { 0x50b93040, SUC_lexicon_tags_27 },
    { 0xccd9303d, SUC_lexicon_tags_7 },
    { 0x90d93041, SUC_lexicon_tags_12 },
    { 0xd0d93042, SUC_lexicon_tags_8 },
    { 0x62593040, SUC_lexicon_tags_27 },
    { 0x66593041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c393048, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd259304b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb93051, SUC_lexicon_tags_30 },
    { 0x75593052, SUC_lexicon_tags_10 },
    { 0xa5593053, SUC_lexicon_tags_63 },
    { 0x19393053, SUC_lexicon_tags_84 },
    { 0x78593055, SUC_lexicon_tags_11 },
    { 0x2d393056, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9ef93058, SUC_lexicon_tags_47 },
    { 0xfeb93059, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c59305c, SUC_lexicon_tags_27 },
    { 0x8fb9305d, SUC_lexicon_tags_17 },
    { 0x9f19305e, SUC_lexicon_tags_414 },
    { 0x9459305e, SUC_lexicon_tags_38 },
    { 0xe79305c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7593063, SUC_lexicon_tags_12 },
    { 0xae593064, SUC_lexicon_tags_1 },
    { 0xe5793065, SUC_lexicon_tags_29 },
    { 0x46393066, SUC_lexicon_tags_26 },
    { 0xaf593066, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf8793069, SUC_lexicon_tags_25 },
    { 0x49b9306a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf99306d, SUC_lexicon_tags_0 },
    { 0xd259306d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa793070, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb393072, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22793079, SUC_lexicon_tags_79 },
    { 0xb9193079, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd93080, SUC_lexicon_tags_30 },
    { 0xdd993080, SUC_lexicon_tags_2 },
    { 0xd8f93082, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27393084, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f93087, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f9308a, SUC_lexicon_tags_11 },
    { 0x5b9308a, SUC_lexicon_tags_11 },
    { 0x42f9308a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c99308f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd393093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80f93095, SUC_lexicon_tags_8 },
    { 0x6db93096, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x64393098, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab9309b, SUC_lexicon_tags_2 },
    { 0x36d9309c, SUC_lexicon_tags_7 },
    { 0x8fb9309c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf039309f, SUC_lexicon_tags_60 },
    { 0x4d930a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b930a3, SUC_lexicon_tags_45 },
    { 0xed9930a3, SUC_lexicon_tags_7 },
    { 0xdb3930a5, SUC_lexicon_tags_0 },
    { 0x191930a4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5930ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d930b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbb5930b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3930b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c7930b8, SUC_lexicon_tags_37 },
    { 0x7c5930b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa19930bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f930c0, SUC_lexicon_tags_1 },
    { 0x2ff930c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff930c7, SUC_lexicon_tags_25 },
    { 0x607930c7, SUC_lexicon_tags_63 },
    { 0xbeb930c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79930cc, SUC_lexicon_tags_27 },
    { 0x243930cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47930d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x699930d3, SUC_lexicon_tags_9 },
    { 0xb9d930d4, SUC_lexicon_tags_113 },
    { 0x73d930d5, SUC_lexicon_tags_11 },
    { 0xeb7930d6, SUC_lexicon_tags_37 },
    { 0x773930d7, SUC_lexicon_tags_72 },
    { 0xbdf930d8, SUC_lexicon_tags_7 },
    { 0x609930d9, SUC_lexicon_tags_7 },
    { 0xee1930da, SUC_lexicon_tags_6 },
    { 0xe55930db, SUC_lexicon_tags_47 },
    { 0x23d930d8, SUC_lexicon_tags_17 },
    { 0x6d930dd, SUC_lexicon_tags_37 },
    { 0xb75930de, SUC_lexicon_tags_37 },
    { 0x3ff930de, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49930e2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x489930e4, SUC_lexicon_tags_15 },
    { 0x111930e5, SUC_lexicon_tags_47 },
    { 0x55b930e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37930ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x25b930ee, SUC_lexicon_tags_2 },
    { 0x5f9930ee, SUC_lexicon_tags_87 },
    { 0xa8f930f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f930f5, SUC_lexicon_tags_4 },
    { 0x745930f6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1930f9, SUC_lexicon_tags_25 },
    { 0xb7d930f9, SUC_lexicon_tags_23 },
    { 0xe67930f9, SUC_lexicon_tags_25 },
    { 0x83d930fb, SUC_lexicon_tags_46 },
    { 0xa3f930fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd93100, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab93103, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd9310a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9d9310c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3319310e, SUC_lexicon_tags_1 },
    { 0x8359310f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb93114, SUC_lexicon_tags_87 },
    { 0x74d93115, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35993117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73393119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cb9311b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5793122, SUC_lexicon_tags_2 },
    { 0x9d193123, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10393129, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3939312b, SUC_lexicon_tags_7 },
    { 0xaf99312c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72793133, SUC_lexicon_tags_7 },
    { 0x1d593134, SUC_lexicon_tags_63 },
    { 0x7fd93134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af93138, SUC_lexicon_tags_1 },
    { 0x35d93138, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xee59313c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcef9313e, SUC_lexicon_tags_14 },
    { 0x39d9313e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8593143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b93147, SUC_lexicon_tags_38 },
    { 0x80193148, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7059314c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab193157, SUC_lexicon_tags_2 },
    { 0x2df93158, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1979315b, SUC_lexicon_tags_16 },
    { 0x1179315b, SUC_lexicon_tags_6 },
    { 0x7db9315d, SUC_lexicon_tags_49 },
    { 0xdfd9315e, SUC_lexicon_tags_12 },
    { 0xd4b9315b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xccd93161, SUC_lexicon_tags_15 },
    { 0x99193162, SUC_lexicon_tags_124 },
    { 0x6993163, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75b93165, SUC_lexicon_tags_17 },
    { 0xe4993166, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd393168, SUC_lexicon_tags_47 },
    { 0xa4993169, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b9316d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b93170, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d93179, SUC_lexicon_tags_7 },
    { 0xdfb9317a, SUC_lexicon_tags_79 },
    { 0x3819317b, SUC_lexicon_tags_31 },
    { 0xe6393179, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab793181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28393183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbf93185, SUC_lexicon_tags_46 },
    { 0xa8d93186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78b93188, SUC_lexicon_tags_23 },
    { 0x6ab93189, SUC_lexicon_tags_11 },
    { 0x7993189, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f9318d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b93192, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24393199, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf159319d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3931a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x85d931a3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b931a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9b931a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5ab931ab, SUC_lexicon_tags_11 },
    { 0x93931ac, SUC_lexicon_tags_49 },
    { 0x959931ad, SUC_lexicon_tags_6 },
    { 0x983931ae, SUC_lexicon_tags_23 },
    { 0xa5931af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x69b931b1, SUC_lexicon_tags_25 },
    { 0xf3d931b1, SUC_lexicon_tags_42 },
    { 0x40d931b3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x593931b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b931bb, SUC_lexicon_tags_82 },
    { 0xbb931bb, SUC_lexicon_tags_17 },
    { 0x49f931bc, SUC_lexicon_tags_25 },
    { 0xbc1931be, SUC_lexicon_tags_31 },
    { 0x2c1931be, SUC_lexicon_tags_6 },
    { 0x19b931bd, SUC_lexicon_tags_17 },
    { 0xf57931bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb931c7, SUC_lexicon_tags_2 },
    { 0xd8f931c8, SUC_lexicon_tags_34 },
    { 0x5df931c9, SUC_lexicon_tags_12 },
    { 0xe07931c7, SUC_lexicon_tags_17 },
    { 0xd71931cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd11931cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b931d2, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb931d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97931da, SUC_lexicon_tags_19 },
    { 0x789931db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b931df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b931e9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x905931eb, SUC_lexicon_tags_11 },
    { 0xeb3931ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2d1931ee, SUC_lexicon_tags_37 },
    { 0x99f931ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d9931f1, SUC_lexicon_tags_60 },
    { 0x219931f1, SUC_lexicon_tags_33 },
    { 0x927931f2, SUC_lexicon_tags_0 },
    { 0xbef931f1, SUC_lexicon_tags_14 },
    { 0xb47931f2, SUC_lexicon_tags_17 },
    { 0x5ed931f6, SUC_lexicon_tags_26 },
    { 0x59b931f7, SUC_lexicon_tags_6 },
    { 0x675931f8, SUC_lexicon_tags_60 },
    { 0x6d3931f9, SUC_lexicon_tags_29 },
    { 0x70f931fa, SUC_lexicon_tags_40 },
    { 0x727931fa, SUC_lexicon_tags_124 },
    { 0x5a7931fb, SUC_lexicon_tags_11 },
    { 0x679931f5, SUC_lexicon_tags_7 },
    { 0x7f9931f1, SUC_lexicon_tags_44 },
    { 0x5d931ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9793201, SUC_lexicon_tags_2 },
    { 0xa793201, SUC_lexicon_tags_46 },
    { 0x75393201, SUC_lexicon_tags_23 },
    { 0x56593201, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x66393207, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc2793209, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b9320c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb93213, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1393219, SUC_lexicon_tags_2 },
    { 0xfad9321a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5239321c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9321f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f93223, SUC_lexicon_tags_15 },
    { 0xebb93223, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x12f93226, SUC_lexicon_tags_7 },
    { 0xca193226, SUC_lexicon_tags_39 },
    { 0x48993228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b9322b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b93230, SUC_lexicon_tags_8 },
    { 0xb4393231, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69593233, SUC_lexicon_tags_1 },
    { 0xc2993233, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb9323a, SUC_lexicon_tags_79 },
    { 0xfd99323b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a59323f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc793241, SUC_lexicon_tags_11 },
    { 0x43593242, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f993245, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a793249, SUC_lexicon_tags_30 },
    { 0xc9b9324a, SUC_lexicon_tags_2 },
    { 0x79b9324b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d9324f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb93253, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95793257, SUC_lexicon_tags_15 },
    { 0x1e593257, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6399325b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce99325d, SUC_lexicon_tags_19 },
    { 0x7819325e, SUC_lexicon_tags_30 },
    { 0x3719325f, SUC_lexicon_tags_38 },
    { 0x79f9325f, SUC_lexicon_tags_37 },
    { 0x4b93261, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19b93263, SUC_lexicon_tags_83 },
    { 0x95793264, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b93267, SUC_lexicon_tags_21 },
    { 0x78b93267, SUC_lexicon_tags_12 },
    { 0xb5d93269, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xef79326c, SUC_lexicon_tags_12 },
    { 0x3a79326d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa659326f, SUC_lexicon_tags_11 },
    { 0x4cd93270, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cb93274, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x67793276, SUC_lexicon_tags_12 },
    { 0xe5d93277, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f9327b, SUC_lexicon_tags_11 },
    { 0x1ad9327c, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed93281, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x44793284, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xe0593286, SUC_lexicon_tags_2 },
    { 0x76f93287, SUC_lexicon_tags_47 },
    { 0x393288, SUC_lexicon_tags_37 },
    { 0x9cf93288, SUC_lexicon_tags_27 },
    { 0xd9993289, SUC_lexicon_tags_22 },
    { 0x65d9328b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb93291, SUC_lexicon_tags_3 },
    { 0xbef93292, SUC_lexicon_tags_43 },
    { 0x96b93293, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9193296, SUC_lexicon_tags_10 },
    { 0x26593297, SUC_lexicon_tags_17 },
    { 0x6c993298, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d79329b, SUC_lexicon_tags_0 },
    { 0x44d9329c, SUC_lexicon_tags_7 },
    { 0xbdf9329d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc159329f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd932a3, SUC_lexicon_tags_37 },
    { 0x883932a3, SUC_lexicon_tags_26 },
    { 0x691932a4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x13f932a7, SUC_lexicon_tags_27 },
    { 0x469932a8, SUC_lexicon_tags_64 },
    { 0xcc3932a9, SUC_lexicon_tags_12 },
    { 0x3c5932a9, SUC_lexicon_tags_37 },
    { 0x589932ab, SUC_lexicon_tags_12 },
    { 0x41f932a8, SUC_lexicon_tags_0 },
    { 0x79b932ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443932b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4dd932b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29932b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3932c1, SUC_lexicon_tags_53 },
    { 0xa25932c2, SUC_lexicon_tags_62 },
    { 0x3b932c3, SUC_lexicon_tags_14 },
    { 0xec3932c3, SUC_lexicon_tags_1 },
    { 0xb3d932c2, SUC_lexicon_tags_12 },
    { 0x5ed932c4, SUC_lexicon_tags_31 },
    { 0x987932c7, SUC_lexicon_tags_11 },
    { 0x165932c6, SUC_lexicon_tags_38 },
    { 0x10f932c9, SUC_lexicon_tags_12 },
    { 0xae9932c1, SUC_lexicon_tags_65 },
    { 0xa67932cb, SUC_lexicon_tags_6 },
    { 0x96d932c8, SUC_lexicon_tags_25 },
    { 0x195932cd, SUC_lexicon_tags_15 },
    { 0xca9932ce, SUC_lexicon_tags_62 },
    { 0x9b3932cf, SUC_lexicon_tags_9 },
    { 0x6b3932d0, SUC_lexicon_tags_62 },
    { 0xb2b932d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5932d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1fb932d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x587932d9, SUC_lexicon_tags_28 },
    { 0xa83932da, SUC_lexicon_tags_12 },
    { 0x183932db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af932e4, SUC_lexicon_tags_14 },
    { 0x8dd932e4, SUC_lexicon_tags_31 },
    { 0xb7932e4, SUC_lexicon_tags_17 },
    { 0x7ff932e7, SUC_lexicon_tags_33 },
    { 0x5d932e5, SUC_lexicon_tags_12 },
    { 0x3ab932e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc85932eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f7932ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb61932ef, SUC_lexicon_tags_4 },
    { 0x201932f0, SUC_lexicon_tags_37 },
    { 0xb1932f0, SUC_lexicon_tags_12 },
    { 0x113932f2, SUC_lexicon_tags_47 },
    { 0x453932f3, SUC_lexicon_tags_38 },
    { 0x6df932f3, SUC_lexicon_tags_49 },
    { 0xf2b932f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x139932f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa59932f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8d932fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda193301, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67793305, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f9330a, SUC_lexicon_tags_60 },
    { 0xeab9330b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf739330e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x71993310, SUC_lexicon_tags_11 },
    { 0x9e193310, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39b93313, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c393317, SUC_lexicon_tags_39 },
    { 0xe2593317, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4839331b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b99331e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b93326, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f9332b, SUC_lexicon_tags_7 },
    { 0xf6f9332c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd193336, SUC_lexicon_tags_49 },
    { 0xe7793336, SUC_lexicon_tags_15 },
    { 0x79993336, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b9333b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d93341, SUC_lexicon_tags_37 },
    { 0x1fd93342, SUC_lexicon_tags_15 },
    { 0xe0993343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40793346, SUC_lexicon_tags_27 },
    { 0xd5393346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c99334b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd79334d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14793351, SUC_lexicon_tags_17 },
    { 0x7f93352, SUC_lexicon_tags_38 },
    { 0xc6f93353, SUC_lexicon_tags_27 },
    { 0xf0d93352, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2bd93356, SUC_lexicon_tags_25 },
    { 0x34f93357, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10593359, SUC_lexicon_tags_23 },
    { 0x63993359, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3479335c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7d9335e, SUC_lexicon_tags_26 },
    { 0x3399335e, SUC_lexicon_tags_87 },
    { 0x4c19335e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf593362, SUC_lexicon_tags_25 },
    { 0x2cb93362, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6593368, SUC_lexicon_tags_1 },
    { 0xaad93369, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf9336d, SUC_lexicon_tags_63 },
    { 0x9b19336e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85d93373, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4039337b, SUC_lexicon_tags_14 },
    { 0x8599337c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdbf9337e, SUC_lexicon_tags_21 },
    { 0xabf9337e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c993381, SUC_lexicon_tags_27 },
    { 0xc7793382, SUC_lexicon_tags_15 },
    { 0xfdd93383, SUC_lexicon_tags_38 },
    { 0x19593382, SUC_lexicon_tags_10 },
    { 0x2ff93385, SUC_lexicon_tags_39 },
    { 0x3ff93381, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b93389, SUC_lexicon_tags_2 },
    { 0x5fd93389, SUC_lexicon_tags_56 },
    { 0xcd19338b, SUC_lexicon_tags_15 },
    { 0x4d19338c, SUC_lexicon_tags_25 },
    { 0x6a19338b, SUC_lexicon_tags_12 },
    { 0xc7f9338d, SUC_lexicon_tags_17 },
    { 0xe2d9338f, SUC_lexicon_tags_27 },
    { 0x3839338f, SUC_lexicon_tags_37 },
    { 0xcad9338b, SUC_lexicon_tags_12 },
    { 0x5a99338d, SUC_lexicon_tags_10 },
    { 0xdeb93393, SUC_lexicon_tags_1 },
    { 0x83593394, SUC_lexicon_tags_37 },
    { 0xc4793395, SUC_lexicon_tags_2 },
    { 0x8ff9338b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239339c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5759339e, SUC_lexicon_tags_25 },
    { 0xc459339f, SUC_lexicon_tags_15 },
    { 0x87d933a0, SUC_lexicon_tags_7 },
    { 0x233933a1, SUC_lexicon_tags_25 },
    { 0xc19933a2, SUC_lexicon_tags_25 },
    { 0xac1933a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd83933a5, SUC_lexicon_tags_11 },
    { 0xfe1933a6, SUC_lexicon_tags_13 },
    { 0xf45933a7, SUC_lexicon_tags_1 },
    { 0x48d933a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df933ab, SUC_lexicon_tags_43 },
    { 0xe8f933ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5933b0, SUC_lexicon_tags_2 },
    { 0x24d933b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd933b4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71933b7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3933bd, SUC_lexicon_tags_13 },
    { 0x7a3933be, SUC_lexicon_tags_46 },
    { 0xe7d933bf, SUC_lexicon_tags_28 },
    { 0xea1933c0, SUC_lexicon_tags_11 },
    { 0x87933c1, SUC_lexicon_tags_25 },
    { 0xd5d933c2, SUC_lexicon_tags_37 },
    { 0xdff933c0, SUC_lexicon_tags_50 },
    { 0x2a3933c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4b9933c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f933cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9c3933cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x829933cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b933d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f933d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287933dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8af933de, SUC_lexicon_tags_47 },
    { 0xc29933df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25933e5, SUC_lexicon_tags_15 },
    { 0x30b933e6, SUC_lexicon_tags_12 },
    { 0x82b933e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d933eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b933ee, SUC_lexicon_tags_25 },
    { 0x8d9933ef, SUC_lexicon_tags_16 },
    { 0xa57933ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5933f3, SUC_lexicon_tags_25 },
    { 0xfb933f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27933f8, SUC_lexicon_tags_11 },
    { 0x53f933f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5933fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af93403, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f93409, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x819340b, SUC_lexicon_tags_1 },
    { 0x5939340b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccd9340e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x46393410, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xca793413, SUC_lexicon_tags_31 },
    { 0xc8b93414, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1993417, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9341c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb99341f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x69f93422, SUC_lexicon_tags_282 },
    { 0x8e993422, SUC_lexicon_tags_35 },
    { 0xa0f93424, SUC_lexicon_tags_25 },
    { 0x83f93425, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b93428, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b9342b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4739342f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb93435, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5593437, SUC_lexicon_tags_15 },
    { 0x6e793437, SUC_lexicon_tags_19 },
    { 0x38b93437, SUC_lexicon_tags_7 },
    { 0xd2f93437, SUC_lexicon_tags_31 },
    { 0xda99343b, SUC_lexicon_tags_0 },
    { 0xb3f9343c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb19343e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x87993441, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b93444, SUC_lexicon_tags_7 },
    { 0x70393445, SUC_lexicon_tags_25 },
    { 0x2d193446, SUC_lexicon_tags_8 },
    { 0x4ff93447, SUC_lexicon_tags_12 },
    { 0xbd193448, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec39344f, SUC_lexicon_tags_12 },
    { 0xa959344f, SUC_lexicon_tags_31 },
    { 0x3ed93450, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb93458, SUC_lexicon_tags_4 },
    { 0x3d993459, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1759345b, SUC_lexicon_tags_25 },
    { 0x4819345c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9b9345e, SUC_lexicon_tags_6 },
    { 0xf8d9345f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x76f93462, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd93469, SUC_lexicon_tags_7 },
    { 0xd259346a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb9346d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f93472, SUC_lexicon_tags_6 },
    { 0xc7b93472, SUC_lexicon_tags_0 },
    { 0x20393474, SUC_lexicon_tags_1 },
    { 0x9b993472, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb9347b, SUC_lexicon_tags_53 },
    { 0xdcb9347b, SUC_lexicon_tags_66 },
    { 0xe919347d, SUC_lexicon_tags_103 },
    { 0x5f59347b, SUC_lexicon_tags_11 },
    { 0x6bf9347f, SUC_lexicon_tags_15 },
    { 0x45993480, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf3593482, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6e793484, SUC_lexicon_tags_2 },
    { 0x2cb93485, SUC_lexicon_tags_0 },
    { 0xfb393486, SUC_lexicon_tags_12 },
    { 0x74993487, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71d93489, SUC_lexicon_tags_11 },
    { 0x37593489, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17793490, SUC_lexicon_tags_19 },
    { 0x85593491, SUC_lexicon_tags_10 },
    { 0x60593492, SUC_lexicon_tags_3 },
    { 0x5193492, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ed93495, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2193499, SUC_lexicon_tags_11 },
    { 0x579349a, SUC_lexicon_tags_11 },
    { 0xf6f9349a, SUC_lexicon_tags_43 },
    { 0xd7b9349c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27934a0, SUC_lexicon_tags_2 },
    { 0x1cb934a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef934a9, SUC_lexicon_tags_33 },
    { 0x303934a9, SUC_lexicon_tags_49 },
    { 0xaf934ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x49934ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1fb934af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305934b3, SUC_lexicon_tags_27 },
    { 0xe63934b4, SUC_lexicon_tags_2 },
    { 0xe15934b4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3934b8, SUC_lexicon_tags_23 },
    { 0x885934b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf934bc, SUC_lexicon_tags_8 },
    { 0x8df934bc, SUC_lexicon_tags_15 },
    { 0x523934bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc83934c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1934c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x519934ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb934cf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905934d7, SUC_lexicon_tags_1 },
    { 0x171934d7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x209934da, SUC_lexicon_tags_29 },
    { 0x77934db, SUC_lexicon_tags_37 },
    { 0xd21934dc, SUC_lexicon_tags_33 },
    { 0xcf1934dd, SUC_lexicon_tags_14 },
    { 0x689934db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631934e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7934eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d934ee, SUC_lexicon_tags_12 },
    { 0xccb934ef, SUC_lexicon_tags_7 },
    { 0x73934ef, SUC_lexicon_tags_19 },
    { 0xb9934f1, SUC_lexicon_tags_30 },
    { 0xdfb934f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b934f6, SUC_lexicon_tags_26 },
    { 0x81d934f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x251934f9, SUC_lexicon_tags_48 },
    { 0x22b934fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd5d934fc, SUC_lexicon_tags_12 },
    { 0x101934fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2f3934ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f93505, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb93509, SUC_lexicon_tags_21 },
    { 0xf899350a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb9350d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd793514, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x64793517, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9793519, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x659351b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91d9351d, SUC_lexicon_tags_37 },
    { 0x4b9351e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19793521, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdaf93523, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e793525, SUC_lexicon_tags_17 },
    { 0x59f93526, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39352d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74193530, SUC_lexicon_tags_62 },
    { 0x47d93530, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b93537, SUC_lexicon_tags_66 },
    { 0x1ef93538, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a59353d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1993541, SUC_lexicon_tags_17 },
    { 0xb9393541, SUC_lexicon_tags_35 },
    { 0x4f593541, SUC_lexicon_tags_39 },
    { 0xf7b93544, SUC_lexicon_tags_15 },
    { 0x57193545, SUC_lexicon_tags_49 },
    { 0xe4d93546, SUC_lexicon_tags_61 },
    { 0x9eb93545, SUC_lexicon_tags_2 },
    { 0x7a793546, SUC_lexicon_tags_12 },
    { 0x19593548, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f59354d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8259354f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x62993551, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46793554, SUC_lexicon_tags_4 },
    { 0xbe993555, SUC_lexicon_tags_9 },
    { 0x62593556, SUC_lexicon_tags_14 },
    { 0xe3193557, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e79355b, SUC_lexicon_tags_11 },
    { 0xf9355b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6393560, SUC_lexicon_tags_39 },
    { 0x59d93561, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d9356b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef93570, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac593573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d593575, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f9357b, SUC_lexicon_tags_15 },
    { 0x699357c, SUC_lexicon_tags_30 },
    { 0xecb9357c, SUC_lexicon_tags_26 },
    { 0x8679357e, SUC_lexicon_tags_14 },
    { 0x5979357b, SUC_lexicon_tags_23 },
    { 0x58d93580, SUC_lexicon_tags_12 },
    { 0xe539357b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd93585, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb1393587, SUC_lexicon_tags_60 },
    { 0x4bf93587, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b9358b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf93592, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc1793594, SUC_lexicon_tags_60 },
    { 0xc7793595, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd93598, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1935a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2c5935a7, SUC_lexicon_tags_42 },
    { 0xbf3935a7, SUC_lexicon_tags_12 },
    { 0x1fb935a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x317935ab, SUC_lexicon_tags_1 },
    { 0x8b7935ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a5935ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1935b1, SUC_lexicon_tags_81 },
    { 0xf4b935b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393935b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9935c2, SUC_lexicon_tags_26 },
    { 0xd7935c3, SUC_lexicon_tags_35 },
    { 0xf55935c2, SUC_lexicon_tags_37 },
    { 0x7dd935c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x853935c7, SUC_lexicon_tags_38 },
    { 0xdd9935c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61b935ca, SUC_lexicon_tags_47 },
    { 0xdad935cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11f935cd, SUC_lexicon_tags_3 },
    { 0xc33935ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x631935d1, SUC_lexicon_tags_34 },
    { 0xddb935d2, SUC_lexicon_tags_17 },
    { 0xe73935d3, SUC_lexicon_tags_11 },
    { 0x5bf935d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293935d8, SUC_lexicon_tags_14 },
    { 0xa23935d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75935df, SUC_lexicon_tags_43 },
    { 0xd9b935e0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x95d935e2, SUC_lexicon_tags_38 },
    { 0xf45935e2, SUC_lexicon_tags_49 },
    { 0x30d935e4, SUC_lexicon_tags_27 },
    { 0xc6d935e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b935ea, SUC_lexicon_tags_62 },
    { 0x9ed935eb, SUC_lexicon_tags_43 },
    { 0x785935ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3935ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25935f3, SUC_lexicon_tags_27 },
    { 0x26f935f4, SUC_lexicon_tags_2 },
    { 0xd09935f5, SUC_lexicon_tags_123 },
    { 0x8ad935f4, SUC_lexicon_tags_17 },
    { 0x8f935f4, SUC_lexicon_tags_17 },
    { 0x9ff935f8, SUC_lexicon_tags_15 },
    { 0x1e7935f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79935fc, SUC_lexicon_tags_2 },
    { 0x821935fc, SUC_lexicon_tags_92 },
    { 0x727935fd, SUC_lexicon_tags_33 },
    { 0xe47935fd, SUC_lexicon_tags_47 },
    { 0x1df93600, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b93604, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf579360f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0393611, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42993613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd3393615, SUC_lexicon_tags_43 },
    { 0x7b193616, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a793619, SUC_lexicon_tags_39 },
    { 0x7eb9361a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf919361d, SUC_lexicon_tags_37 },
    { 0x23d9361e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8c793620, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf193625, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4819362c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe499362e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc793633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9363a, SUC_lexicon_tags_11 },
    { 0xd239363b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37593640, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2c193642, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcab93645, SUC_lexicon_tags_7 },
    { 0x3f193646, SUC_lexicon_tags_47 },
    { 0xfdf93646, SUC_lexicon_tags_25 },
    { 0xc6193645, SUC_lexicon_tags_25 },
    { 0x37793649, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4299364e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb993659, SUC_lexicon_tags_87 },
    { 0x4fb9365a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d9365f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1393662, SUC_lexicon_tags_15 },
    { 0x73993662, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd6f93665, SUC_lexicon_tags_49 },
    { 0xbe793665, SUC_lexicon_tags_0 },
    { 0xe8993667, SUC_lexicon_tags_23 },
    { 0x49793668, SUC_lexicon_tags_31 },
    { 0x5dd93669, SUC_lexicon_tags_184 },
    { 0x88793669, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xff39366d, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4593673, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f93676, SUC_lexicon_tags_34 },
    { 0x85d93677, SUC_lexicon_tags_55 },
    { 0xc3b93678, SUC_lexicon_tags_15 },
    { 0x5af93678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab59367b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf93680, SUC_lexicon_tags_26 },
    { 0x12593681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae393686, SUC_lexicon_tags_15 },
    { 0x69793686, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb9368a, SUC_lexicon_tags_2 },
    { 0xc9b9368a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5899368d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23793691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x34d93694, SUC_lexicon_tags_1 },
    { 0xd8f93695, SUC_lexicon_tags_33 },
    { 0x3ad93696, SUC_lexicon_tags_25 },
    { 0xff93696, SUC_lexicon_tags_25 },
    { 0x7f193697, SUC_lexicon_tags_0 },
    { 0x87393699, SUC_lexicon_tags_47 },
    { 0x19393699, SUC_lexicon_tags_1 },
    { 0x2c99369b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeed9369d, SUC_lexicon_tags_11 },
    { 0xa799369d, SUC_lexicon_tags_25 },
    { 0x8499369f, SUC_lexicon_tags_0 },
    { 0x65d936a0, SUC_lexicon_tags_19 },
    { 0xb539369f, SUC_lexicon_tags_49 },
    { 0x1939369e, SUC_lexicon_tags_71 },
    { 0x779369f, SUC_lexicon_tags_47 },
    { 0x5bf936a4, SUC_lexicon_tags_17 },
    { 0xf47936a5, SUC_lexicon_tags_38 },
    { 0x703936a5, SUC_lexicon_tags_15 },
    { 0xc61936a0, SUC_lexicon_tags_30 },
    { 0xfb1936a8, SUC_lexicon_tags_12 },
    { 0xcf5936a8, SUC_lexicon_tags_87 },
    { 0x129936aa, SUC_lexicon_tags_19 },
    { 0xd41936ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f936b1, SUC_lexicon_tags_28 },
    { 0x29b936b2, SUC_lexicon_tags_14 },
    { 0xa5d936b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd2b936b5, SUC_lexicon_tags_1 },
    { 0x125936b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b936ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a3936bc, SUC_lexicon_tags_49 },
    { 0x745936bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9936c4, SUC_lexicon_tags_17 },
    { 0xe47936c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49b936c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x459936c9, SUC_lexicon_tags_56 },
    { 0x99f936c9, SUC_lexicon_tags_37 },
    { 0xfc7936cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94d936cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1936d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1936d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f936db, SUC_lexicon_tags_23 },
    { 0xd1936db, SUC_lexicon_tags_28 },
    { 0xd15936dc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d936e2, SUC_lexicon_tags_1 },
    { 0x3e1936e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6f5936e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c9936e7, SUC_lexicon_tags_14 },
    { 0xfb5936e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f936eb, SUC_lexicon_tags_12 },
    { 0x639936ec, SUC_lexicon_tags_48 },
    { 0x12f936ed, SUC_lexicon_tags_2 },
    { 0xb31936ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed936f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9936f6, SUC_lexicon_tags_16 },
    { 0x5f5936f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3936fb, SUC_lexicon_tags_17 },
    { 0x797936fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ff936fe, SUC_lexicon_tags_118 },
    { 0x99b936ff, SUC_lexicon_tags_87 },
    { 0xa3f93700, SUC_lexicon_tags_25 },
    { 0x42d936ff, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xeb793703, SUC_lexicon_tags_12 },
    { 0xf5d93703, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x69193707, SUC_lexicon_tags_43 },
    { 0x45593708, SUC_lexicon_tags_11 },
    { 0x8cb93709, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa899370b, SUC_lexicon_tags_38 },
    { 0x86d9370b, SUC_lexicon_tags_17 },
    { 0xcf19370d, SUC_lexicon_tags_11 },
    { 0xa9d9370b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f93712, SUC_lexicon_tags_1 },
    { 0xcd593712, SUC_lexicon_tags_25 },
    { 0x6793714, SUC_lexicon_tags_40 },
    { 0xb8b93715, SUC_lexicon_tags_12 },
    { 0xb6393716, SUC_lexicon_tags_64 },
    { 0x89393717, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4e993719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4639371e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d93721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed93724, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7593726, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5d93728, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af93730, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd993736, SUC_lexicon_tags_11 },
    { 0x31d93736, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d9373b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2819373e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f393741, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfc793743, SUC_lexicon_tags_38 },
    { 0x8a993744, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x393746, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf593748, SUC_lexicon_tags_10 },
    { 0x7ad93749, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc879374b, SUC_lexicon_tags_25 },
    { 0x9819374b, SUC_lexicon_tags_15 },
    { 0x1019374c, SUC_lexicon_tags_16 },
    { 0x9c39374b, SUC_lexicon_tags_31 },
    { 0x9b99374d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xffb93751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96993753, SUC_lexicon_tags_8 },
    { 0xfc793754, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdb393756, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x68793758, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f9375c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd93768, SUC_lexicon_tags_27 },
    { 0x81f93769, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6879376e, SUC_lexicon_tags_23 },
    { 0x8e59376e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a993771, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56193773, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1993778, SUC_lexicon_tags_15 },
    { 0x9e993778, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb793786, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad93793, SUC_lexicon_tags_7 },
    { 0x53593794, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x15d93796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7993798, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995937a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb937a4, SUC_lexicon_tags_27 },
    { 0x721937a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd937a8, SUC_lexicon_tags_17 },
    { 0x2d5937a8, SUC_lexicon_tags_15 },
    { 0x17d937aa, SUC_lexicon_tags_37 },
    { 0x189937ab, SUC_lexicon_tags_118 },
    { 0xf33937ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5937b5, SUC_lexicon_tags_2 },
    { 0xcdd937b6, SUC_lexicon_tags_15 },
    { 0x6bd937b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x365937b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd937be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2d1937c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x241937c3, SUC_lexicon_tags_25 },
    { 0xa9937c4, SUC_lexicon_tags_46 },
    { 0x22f937c5, SUC_lexicon_tags_15 },
    { 0xc61937c4, SUC_lexicon_tags_37 },
    { 0x10f937c6, SUC_lexicon_tags_10 },
    { 0x261937c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d937cc, SUC_lexicon_tags_8 },
    { 0x1b937cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b937d0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xae1937d2, SUC_lexicon_tags_12 },
    { 0x24d937d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x21937d6, SUC_lexicon_tags_1 },
    { 0x9ed937d7, SUC_lexicon_tags_29 },
    { 0xf7b937d8, SUC_lexicon_tags_25 },
    { 0x4ff937d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647937e0, SUC_lexicon_tags_37 },
    { 0x5ff937e0, SUC_lexicon_tags_30 },
    { 0x7f3937e2, SUC_lexicon_tags_55 },
    { 0x21f937e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5937ea, SUC_lexicon_tags_82 },
    { 0xb09937ea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7937ee, SUC_lexicon_tags_10 },
    { 0x9c5937ef, SUC_lexicon_tags_1 },
    { 0x7937ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1937f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d9937f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b937fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4af93800, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddf93802, SUC_lexicon_tags_19 },
    { 0xf0b93803, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f93807, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b9380a, SUC_lexicon_tags_15 },
    { 0x2bf9380b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25993811, SUC_lexicon_tags_151 },
    { 0x9ff93812, SUC_lexicon_tags_11 },
    { 0x91193813, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b93817, SUC_lexicon_tags_25 },
    { 0xb2593818, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2db9381a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9593820, SUC_lexicon_tags_32 },
    { 0xf9393821, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x14d93823, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4593826, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4099382d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf193830, SUC_lexicon_tags_33 },
    { 0x97f93831, SUC_lexicon_tags_12 },
    { 0xf93830, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22f93834, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb93838, SUC_lexicon_tags_4 },
    { 0x31f93839, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8b79383b, SUC_lexicon_tags_11 },
    { 0x2e79383c, SUC_lexicon_tags_1 },
    { 0xc559383d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d93842, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7993845, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf639384c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5bf9384e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb93851, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4593855, SUC_lexicon_tags_12 },
    { 0x56993855, SUC_lexicon_tags_26 },
    { 0x8f993857, SUC_lexicon_tags_31 },
    { 0x4db93858, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af9385e, SUC_lexicon_tags_23 },
    { 0xb4f9385e, SUC_lexicon_tags_33 },
    { 0x5b93860, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1793866, SUC_lexicon_tags_0 },
    { 0xd0b93866, SUC_lexicon_tags_45 },
    { 0xd2393868, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5499386a, SUC_lexicon_tags_21 },
    { 0x1ff9386b, SUC_lexicon_tags_21 },
    { 0x1bd9386c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55993872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58393874, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x93993877, SUC_lexicon_tags_26 },
    { 0x5ed93877, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f9387b, SUC_lexicon_tags_11 },
    { 0x60b9387b, SUC_lexicon_tags_17 },
    { 0xb199387c, SUC_lexicon_tags_33 },
    { 0xd179387e, SUC_lexicon_tags_7 },
    { 0x4979387e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94393881, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc593884, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72593888, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1699388b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1699388e, SUC_lexicon_tags_3 },
    { 0x3f9388f, SUC_lexicon_tags_25 },
    { 0xad793890, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f93892, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd593897, SUC_lexicon_tags_10 },
    { 0x7a193897, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1679389b, SUC_lexicon_tags_27 },
    { 0xc059389b, SUC_lexicon_tags_36 },
    { 0x2699389d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4999389f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29938a2, SUC_lexicon_tags_25 },
    { 0x45d938a3, SUC_lexicon_tags_16 },
    { 0x613938a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b3938a6, SUC_lexicon_tags_25 },
    { 0x941938a7, SUC_lexicon_tags_15 },
    { 0xbd938a8, SUC_lexicon_tags_55 },
    { 0x797938a9, SUC_lexicon_tags_72 },
    { 0x691938aa, SUC_lexicon_tags_39 },
    { 0x1c9938a9, SUC_lexicon_tags_25 },
    { 0x97f938ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b938af, SUC_lexicon_tags_60 },
    { 0xb63938b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9938b7, SUC_lexicon_tags_15 },
    { 0x813938b7, SUC_lexicon_tags_17 },
    { 0xea7938b9, SUC_lexicon_tags_28 },
    { 0x56d938ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x987938bc, SUC_lexicon_tags_12 },
    { 0x181938bd, SUC_lexicon_tags_28 },
    { 0x2f3938bc, SUC_lexicon_tags_82 },
    { 0xddf938bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7938c3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x549938c6, SUC_lexicon_tags_60 },
    { 0x539938c7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7d938c9, SUC_lexicon_tags_106 },
    { 0x805938ca, SUC_lexicon_tags_15 },
    { 0x43938cb, SUC_lexicon_tags_17 },
    { 0x735938cc, SUC_lexicon_tags_68 },
    { 0x655938cc, SUC_lexicon_tags_62 },
    { 0x787938ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89938d4, SUC_lexicon_tags_9 },
    { 0xaef938d5, SUC_lexicon_tags_65 },
    { 0xf03938d5, SUC_lexicon_tags_15 },
    { 0xea9938d7, SUC_lexicon_tags_0 },
    { 0x41f938d8, SUC_lexicon_tags_25 },
    { 0xa39938d9, SUC_lexicon_tags_16 },
    { 0x9c3938da, SUC_lexicon_tags_7 },
    { 0xee5938db, SUC_lexicon_tags_6 },
    { 0xa17938db, SUC_lexicon_tags_14 },
    { 0x3fb938dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d938e0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xac9938e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9938e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4f938e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x299938eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7938ef, SUC_lexicon_tags_27 },
    { 0x30f938ef, SUC_lexicon_tags_12 },
    { 0xf3d938f1, SUC_lexicon_tags_8 },
    { 0x48f938f1, SUC_lexicon_tags_37 },
    { 0x3fd938f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975938f8, SUC_lexicon_tags_27 },
    { 0x509938f9, SUC_lexicon_tags_6 },
    { 0x1e5938fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3938fe, SUC_lexicon_tags_37 },
    { 0x469938fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0993902, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34993906, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8593908, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3099390a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1d79390c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf79390f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3bd93911, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c593914, SUC_lexicon_tags_60 },
    { 0x8193914, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97b93917, SUC_lexicon_tags_11 },
    { 0xc6d93918, SUC_lexicon_tags_22 },
    { 0xc2393919, SUC_lexicon_tags_12 },
    { 0xc4f93919, SUC_lexicon_tags_64 },
    { 0x45193918, SUC_lexicon_tags_37 },
    { 0xcaf9391c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b93920, SUC_lexicon_tags_27 },
    { 0xa3d93921, SUC_lexicon_tags_7 },
    { 0xa8393921, SUC_lexicon_tags_21 },
    { 0x3fd93923, SUC_lexicon_tags_23 },
    { 0x2b793921, SUC_lexicon_tags_62 },
    { 0x70993925, SUC_lexicon_tags_0 },
    { 0xc7393926, SUC_lexicon_tags_1 },
    { 0x57193927, SUC_lexicon_tags_0 },
    { 0x9c993924, SUC_lexicon_tags_6 },
    { 0x70b93929, SUC_lexicon_tags_25 },
    { 0x5cf9392a, SUC_lexicon_tags_17 },
    { 0x19b9392b, SUC_lexicon_tags_11 },
    { 0x5b193925, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd93931, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c993937, SUC_lexicon_tags_27 },
    { 0xdef93937, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x60b9393a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb39393c, SUC_lexicon_tags_56 },
    { 0xe79393c, SUC_lexicon_tags_1 },
    { 0x4ab9393e, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x5a593940, SUC_lexicon_tags_33 },
    { 0x29393940, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b393944, SUC_lexicon_tags_63 },
    { 0xc9593945, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89193947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f9394b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba59394d, SUC_lexicon_tags_33 },
    { 0xe1f9394e, SUC_lexicon_tags_38 },
    { 0xc0b9394d, SUC_lexicon_tags_12 },
    { 0x36b93950, SUC_lexicon_tags_14 },
    { 0xa2593951, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6193956, SUC_lexicon_tags_9 },
    { 0x6a793957, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d9395d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x26393960, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb93963, SUC_lexicon_tags_25 },
    { 0xd1b93964, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x67193967, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7d793969, SUC_lexicon_tags_124 },
    { 0xb139396a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa59396c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75993970, SUC_lexicon_tags_26 },
    { 0x5bf93971, SUC_lexicon_tags_9 },
    { 0x4e193972, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84393974, SUC_lexicon_tags_151 },
    { 0xcf593975, SUC_lexicon_tags_13 },
    { 0x35793976, SUC_lexicon_tags_12 },
    { 0x77993977, SUC_lexicon_tags_45 },
    { 0x3a593976, SUC_lexicon_tags_712 },
    { 0xa6793979, SUC_lexicon_tags_43 },
    { 0x85f93979, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32793980, SUC_lexicon_tags_15 },
    { 0x5bd93981, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8993984, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc593988, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b9398c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad93997, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd593999, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x139399c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb939a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a9939a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5939a8, SUC_lexicon_tags_15 },
    { 0xb75939a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87939ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5939b0, SUC_lexicon_tags_15 },
    { 0x2b5939b0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6a9939b3, SUC_lexicon_tags_1 },
    { 0xcf5939b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871939b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1b1939ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7939bd, SUC_lexicon_tags_17 },
    { 0x7d1939bd, SUC_lexicon_tags_37 },
    { 0x957939bd, SUC_lexicon_tags_11 },
    { 0x4b3939bd, SUC_lexicon_tags_47 },
    { 0xe83939bf, SUC_lexicon_tags_15 },
    { 0x2ab939bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db939c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x689939c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311939d3, SUC_lexicon_tags_25 },
    { 0xe81939d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x45b939d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269939dd, SUC_lexicon_tags_37 },
    { 0x95f939de, SUC_lexicon_tags_17 },
    { 0x911939df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb939e5, SUC_lexicon_tags_37 },
    { 0xf2b939e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x775939e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d939ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x795939ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d939f3, SUC_lexicon_tags_63 },
    { 0xf47939f4, SUC_lexicon_tags_31 },
    { 0x507939f4, SUC_lexicon_tags_1 },
    { 0x63f939f6, SUC_lexicon_tags_12 },
    { 0x595939f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x43b939f9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f939fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4f939fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a193a05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66393a0e, SUC_lexicon_tags_7 },
    { 0x9393a0f, SUC_lexicon_tags_10 },
    { 0x64d93a0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d93a13, SUC_lexicon_tags_37 },
    { 0xe5993a13, SUC_lexicon_tags_8 },
    { 0xbcd93a15, SUC_lexicon_tags_56 },
    { 0x76193a16, SUC_lexicon_tags_47 },
    { 0x8df93a17, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f793a1a, SUC_lexicon_tags_21 },
    { 0xf6193a1b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcd193a1d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f993a21, SUC_lexicon_tags_37 },
    { 0xe5f93a22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1793a28, SUC_lexicon_tags_11 },
    { 0x9ad93a29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93a35, SUC_lexicon_tags_19 },
    { 0x4d993a35, SUC_lexicon_tags_0 },
    { 0x63f93a37, SUC_lexicon_tags_4 },
    { 0xef593a38, SUC_lexicon_tags_27 },
    { 0x45b93a39, SUC_lexicon_tags_15 },
    { 0x30f93a39, SUC_lexicon_tags_43 },
    { 0xcef93a3a, SUC_lexicon_tags_2 },
    { 0x70393a3c, SUC_lexicon_tags_37 },
    { 0xc3993a38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5393a42, SUC_lexicon_tags_37 },
    { 0xc3f93a43, SUC_lexicon_tags_47 },
    { 0xc4f93a42, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc393a49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc393a50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df93a54, SUC_lexicon_tags_17 },
    { 0xc7d93a54, SUC_lexicon_tags_23 },
    { 0x50d93a56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f93a5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba793a5d, SUC_lexicon_tags_45 },
    { 0x91593a5d, SUC_lexicon_tags_17 },
    { 0xd5993a5f, SUC_lexicon_tags_55 },
    { 0xc2f93a5e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x25393a63, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x52f93a65, SUC_lexicon_tags_85 },
    { 0x2d193a65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b93a6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3d93a6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a793a6f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xef193a71, SUC_lexicon_tags_25 },
    { 0x95593a72, SUC_lexicon_tags_17 },
    { 0xbe193a71, SUC_lexicon_tags_0 },
    { 0x8ff93a74, SUC_lexicon_tags_220 },
    { 0x81593a74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd593a7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf393a7f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x99f93a81, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x81393a83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b93a86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6993a89, SUC_lexicon_tags_1063 },
    { 0, NULL },
    { 0, NULL },
    { 0x20393a8c, SUC_lexicon_tags_47 },
    { 0x96d93a8c, SUC_lexicon_tags_36 },
    { 0xa3393a8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f93a97, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x41193a9a, SUC_lexicon_tags_46 },
    { 0x6ff93a9a, SUC_lexicon_tags_65 },
    { 0x93d93a9a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x89b93a9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d93aa1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d93aa7, SUC_lexicon_tags_68 },
    { 0x66b93aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73b93aaa, SUC_lexicon_tags_29 },
    { 0x27b93aab, SUC_lexicon_tags_87 },
    { 0x1ab93aaa, SUC_lexicon_tags_113 },
    { 0xa7193aac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2393ab0, SUC_lexicon_tags_15 },
    { 0x54d93ab0, SUC_lexicon_tags_32 },
    { 0x39d93ab2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b93ab7, SUC_lexicon_tags_13 },
    { 0x61993ab7, SUC_lexicon_tags_10 },
    { 0x69193ab8, SUC_lexicon_tags_37 },
    { 0xb7593aba, SUC_lexicon_tags_26 },
    { 0x73393ab7, SUC_lexicon_tags_47 },
    { 0x99d93abc, SUC_lexicon_tags_29 },
    { 0x1b393abc, SUC_lexicon_tags_7 },
    { 0x5d593abb, SUC_lexicon_tags_23 },
    { 0xf9193abf, SUC_lexicon_tags_33 },
    { 0x26f93ac0, SUC_lexicon_tags_22 },
    { 0xcb193ab8, SUC_lexicon_tags_11 },
    { 0x5ed93ac2, SUC_lexicon_tags_33 },
    { 0x6ad93ac3, SUC_lexicon_tags_11 },
    { 0x8db93abb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x95593ac6, SUC_lexicon_tags_1 },
    { 0x18593ac7, SUC_lexicon_tags_30 },
    { 0x86193ac8, SUC_lexicon_tags_5 },
    { 0x60393ac9, SUC_lexicon_tags_66 },
    { 0xa7b93ac9, SUC_lexicon_tags_55 },
    { 0xce393aca, SUC_lexicon_tags_15 },
    { 0x66393ac8, SUC_lexicon_tags_37 },
    { 0xce993ac8, SUC_lexicon_tags_26 },
    { 0x78b93ac8, SUC_lexicon_tags_15 },
    { 0xad393acf, SUC_lexicon_tags_7 },
    { 0x53593ac7, SUC_lexicon_tags_37 },
    { 0xd6993acb, SUC_lexicon_tags_31 },
    { 0x3c593acf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x44b93ad4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d93ad7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48993adb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35593adf, SUC_lexicon_tags_11 },
    { 0x91193adf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd93ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf793ae9, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d93aec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf93af0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2393af2, SUC_lexicon_tags_28 },
    { 0x47793af3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e393af6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf5193af8, SUC_lexicon_tags_8 },
    { 0xf1993af9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x34993afb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77593b01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb2b93b03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45393b0c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4193b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff93b19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75193b1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58193b1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2593b1f, SUC_lexicon_tags_12 },
    { 0x52393b20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f93b27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92993b29, SUC_lexicon_tags_25 },
    { 0xd8b93b2a, SUC_lexicon_tags_12 },
    { 0xed793b2b, SUC_lexicon_tags_2 },
    { 0xa393b2b, SUC_lexicon_tags_28 },
    { 0x7d793b29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d793b2f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d993b38, SUC_lexicon_tags_63 },
    { 0x88b93b38, SUC_lexicon_tags_19 },
    { 0x2df93b3a, SUC_lexicon_tags_65 },
    { 0xfb793b3b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d593b3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b93b42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99793b46, SUC_lexicon_tags_3 },
    { 0x99193b47, SUC_lexicon_tags_27 },
    { 0xfa193b46, SUC_lexicon_tags_17 },
    { 0x29193b49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed93b4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf93b4f, SUC_lexicon_tags_60 },
    { 0x2df93b50, SUC_lexicon_tags_11 },
    { 0xb6b93b50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb193b53, SUC_lexicon_tags_27 },
    { 0xe6393b54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x91d93b56, SUC_lexicon_tags_2 },
    { 0x3a793b57, SUC_lexicon_tags_8 },
    { 0x25d93b58, SUC_lexicon_tags_66 },
    { 0xd4d93b59, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x47b93b5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1793b5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2193b60, SUC_lexicon_tags_14 },
    { 0xd2f93b61, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea393b66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf93b6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x94f93b6c, SUC_lexicon_tags_10 },
    { 0xa5593b6d, SUC_lexicon_tags_33 },
    { 0xc7993b6e, SUC_lexicon_tags_79 },
    { 0x4bd93b6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72d93b76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b93b7a, SUC_lexicon_tags_2 },
    { 0xa5d93b7b, SUC_lexicon_tags_1 },
    { 0xabf93b7b, SUC_lexicon_tags_56 },
    { 0xc5593b7c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8393b80, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x82993b82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0993b88, SUC_lexicon_tags_14 },
    { 0x41b93b88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77993b8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5993b8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28993b92, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d93b9e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe9993ba0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb193ba3, SUC_lexicon_tags_39 },
    { 0x29393ba3, SUC_lexicon_tags_23 },
    { 0x6e993ba3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc593ba7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5cb93ba9, SUC_lexicon_tags_83 },
    { 0x3b593baa, SUC_lexicon_tags_13 },
    { 0x76793bab, SUC_lexicon_tags_1 },
    { 0xbfb93bac, SUC_lexicon_tags_7 },
    { 0x29193bad, SUC_lexicon_tags_17 },
    { 0xa2193bab, SUC_lexicon_tags_2 },
    { 0x60393baf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x38393bb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3993bb7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x15f93bb9, SUC_lexicon_tags_33 },
    { 0xed93bb9, SUC_lexicon_tags_37 },
    { 0x58593bbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b93bbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x20d93bc1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99993bc5, SUC_lexicon_tags_27 },
    { 0xa2d93bc6, SUC_lexicon_tags_17 },
    { 0x55f93bc6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d93bcc, SUC_lexicon_tags_2 },
    { 0x6cb93bcd, SUC_lexicon_tags_1 },
    { 0x3c393bce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b93bd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b593bd7, SUC_lexicon_tags_0 },
    { 0x5b993bd8, SUC_lexicon_tags_84 },
    { 0xba593bd9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9193bdf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b93be2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5593be6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x47993be9, SUC_lexicon_tags_49 },
    { 0x2c193bea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e993bec, SUC_lexicon_tags_28 },
    { 0x1793bed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe5593bef, SUC_lexicon_tags_60 },
    { 0xa193bf0, SUC_lexicon_tags_5 },
    { 0x38793bf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91193bf3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb193bf8, SUC_lexicon_tags_3 },
    { 0xc7d93bf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d993bfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f93c02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe0d93c04, SUC_lexicon_tags_1 },
    { 0x5d193c05, SUC_lexicon_tags_1 },
    { 0x61b93c05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c593c0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf93c0d, SUC_lexicon_tags_65 },
    { 0x99393c0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3793c11, SUC_lexicon_tags_47 },
    { 0x69593c12, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b93c15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67793c17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b793c1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6bd93c1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd593c22, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa9d93c24, SUC_lexicon_tags_33 },
    { 0xfd193c25, SUC_lexicon_tags_15 },
    { 0xb6f93c26, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x16393c28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd793c2b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d393c30, SUC_lexicon_tags_1 },
    { 0x7a993c31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b93c36, SUC_lexicon_tags_14 },
    { 0x86b93c37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23993c39, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x86393c3b, SUC_lexicon_tags_15 },
    { 0xb0593c3c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb93c40, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x49193c43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1b93c45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf93c50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe3593c52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x54b93c54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1fb93c56, SUC_lexicon_tags_15 },
    { 0x45793c57, SUC_lexicon_tags_49 },
    { 0x7f593c58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b993c5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6393c5d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe7193c5f, SUC_lexicon_tags_7 },
    { 0xa0b93c60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d93c62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x17393c64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0193c68, SUC_lexicon_tags_27 },
    { 0x2b93c69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f93c6d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cf93c71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65393c74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff93c83, SUC_lexicon_tags_37 },
    { 0x67793c84, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf93c87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6993c8b, SUC_lexicon_tags_14 },
    { 0xe0193c8c, SUC_lexicon_tags_37 },
    { 0x71f93c8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52593c91, SUC_lexicon_tags_23 },
    { 0x98393c91, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd5193c94, SUC_lexicon_tags_1 },
    { 0xad93c95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb0b93c97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4193c9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1793ca1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd93ca3, SUC_lexicon_tags_11 },
    { 0xe1393ca4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f993ca6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf93ca9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42d93cab, SUC_lexicon_tags_11 },
    { 0x9bf93cac, SUC_lexicon_tags_2 },
    { 0x57b93cac, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd93cb9, SUC_lexicon_tags_63 },
    { 0xddd93cba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14f93cbc, SUC_lexicon_tags_48 },
    { 0xf5b93cbc, SUC_lexicon_tags_26 },
    { 0xa3b93cbd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b93cc1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf93cc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58593cc8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b93ccb, SUC_lexicon_tags_38 },
    { 0xf9393ccc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9cf93cce, SUC_lexicon_tags_33 },
    { 0x72d93ccf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d93cd3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30393cd9, SUC_lexicon_tags_17 },
    { 0x62193cd9, SUC_lexicon_tags_45 },
    { 0x47b93cdb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87793cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4193ce2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x17993ce4, SUC_lexicon_tags_37 },
    { 0x51793ce5, SUC_lexicon_tags_8 },
    { 0xa9d93ce5, SUC_lexicon_tags_8 },
    { 0x16993ce5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbad93ce9, SUC_lexicon_tags_31 },
    { 0xff93cea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad93ced, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c393cf0, SUC_lexicon_tags_23 },
    { 0x35d93cf1, SUC_lexicon_tags_506 },
    { 0x25b93cf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed193cf9, SUC_lexicon_tags_7 },
    { 0xf0d93cf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9593d03, SUC_lexicon_tags_48 },
    { 0x13993d04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f93d08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38193d0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79593d0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf93d12, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc1d93d14, SUC_lexicon_tags_7 },
    { 0x77d93d14, SUC_lexicon_tags_7 },
    { 0xfbb93d16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6cd93d18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x88f93d1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17993d20, SUC_lexicon_tags_12 },
    { 0x43f93d20, SUC_lexicon_tags_28 },
    { 0x58393d22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec193d24, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xad393d26, SUC_lexicon_tags_6 },
    { 0x18793d27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce993d29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7d93d2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8db93d2d, SUC_lexicon_tags_11 },
    { 0xabf93d2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e993d31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba993d34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ff93d36, SUC_lexicon_tags_25 },
    { 0x93793d36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c193d3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d593d3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f93d41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f93d49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6593d4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d93d50, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfa993d52, SUC_lexicon_tags_2 },
    { 0xc6993d52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe7193d55, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x24393d57, SUC_lexicon_tags_47 },
    { 0xd5193d57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2793d5a, SUC_lexicon_tags_30 },
    { 0x51b93d5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x91b93d5e, SUC_lexicon_tags_40 },
    { 0xa4f93d5f, SUC_lexicon_tags_16 },
    { 0x34793d5f, SUC_lexicon_tags_1279 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b93d65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x74b93d68, SUC_lexicon_tags_21 },
    { 0xb3b93d69, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf5d93d6b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf93d6f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27793d74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b93d78, SUC_lexicon_tags_11 },
    { 0xf0b93d78, SUC_lexicon_tags_1 },
    { 0x1e793d7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b93d7d, SUC_lexicon_tags_26 },
    { 0x93593d7d, SUC_lexicon_tags_47 },
    { 0xf4993d7d, SUC_lexicon_tags_8 },
    { 0x91d93d80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f593d83, SUC_lexicon_tags_29 },
    { 0xb7193d84, SUC_lexicon_tags_14 },
    { 0xcb593d85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa6793d87, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed193d8b, SUC_lexicon_tags_37 },
    { 0xb8593d8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ad93d8e, SUC_lexicon_tags_14 },
    { 0x47f93d8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x62993d92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x54393d94, SUC_lexicon_tags_33 },
    { 0x40b93d95, SUC_lexicon_tags_25 },
    { 0x31b93d96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31793d98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99b93d9a, SUC_lexicon_tags_19 },
    { 0xc9993d9a, SUC_lexicon_tags_12 },
    { 0x5e393d9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe593da2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa993da6, SUC_lexicon_tags_25 },
    { 0xfcd93da7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5593dac, SUC_lexicon_tags_47 },
    { 0xf9193dac, SUC_lexicon_tags_1 },
    { 0x70793dae, SUC_lexicon_tags_17 },
    { 0x4e993daf, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xfb93db1, SUC_lexicon_tags_17 },
    { 0x63f93db2, SUC_lexicon_tags_25 },
    { 0xf593db3, SUC_lexicon_tags_12 },
    { 0x57b93db4, SUC_lexicon_tags_7 },
    { 0x46593db5, SUC_lexicon_tags_2 },
    { 0xcdf93db4, SUC_lexicon_tags_11 },
    { 0x67793db4, SUC_lexicon_tags_1 },
    { 0x64993db3, SUC_lexicon_tags_37 },
    { 0xcad93db8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98593dbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd1d93dbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6bf93dc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd93dc4, SUC_lexicon_tags_136 },
    { 0x66193dc4, SUC_lexicon_tags_43 },
    { 0xf5193dc6, SUC_lexicon_tags_23 },
    { 0x8c193dc7, SUC_lexicon_tags_23 },
    { 0x98793dc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9593dca, SUC_lexicon_tags_21 },
    { 0x1bf93dcb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d93dd1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e393dd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1993ddb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b93ddf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e393de7, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f593df2, SUC_lexicon_tags_15 },
    { 0x47193df3, SUC_lexicon_tags_59 },
    { 0xddb93df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf93dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1993dfe, SUC_lexicon_tags_12 },
    { 0x33b93dfe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8ed93e01, SUC_lexicon_tags_46 },
    { 0x3a993e01, SUC_lexicon_tags_43 },
    { 0x87393e03, SUC_lexicon_tags_7 },
    { 0x4a193e04, SUC_lexicon_tags_63 },
    { 0xb3793e04, SUC_lexicon_tags_11 },
    { 0x5a193e03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22593e08, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6c393e0a, SUC_lexicon_tags_26 },
    { 0x4cf93e0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa0b93e0d, SUC_lexicon_tags_49 },
    { 0xa5d93e0e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc193e1a, SUC_lexicon_tags_11 },
    { 0x37593e1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0593e1f, SUC_lexicon_tags_43 },
    { 0x36793e20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62193e23, SUC_lexicon_tags_25 },
    { 0xbc793e23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f93e29, SUC_lexicon_tags_63 },
    { 0x2393e29, SUC_lexicon_tags_17 },
    { 0x28793e2b, SUC_lexicon_tags_48 },
    { 0x8e393e2b, SUC_lexicon_tags_64 },
    { 0xdbb93e2d, SUC_lexicon_tags_25 },
    { 0x49393e29, SUC_lexicon_tags_43 },
    { 0xe5593e2f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x51b93e31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d93e3c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf4f93e3e, SUC_lexicon_tags_15 },
    { 0x7bb93e3e, SUC_lexicon_tags_45 },
    { 0xc2d93e40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42993e4a, SUC_lexicon_tags_30 },
    { 0xafb93e4b, SUC_lexicon_tags_17 },
    { 0x4bb93e4c, SUC_lexicon_tags_11 },
    { 0x96593e4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43993e52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47793e55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c393e5e, SUC_lexicon_tags_2 },
    { 0xf7793e5f, SUC_lexicon_tags_47 },
    { 0x2dd93e60, SUC_lexicon_tags_25 },
    { 0x96793e61, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x47393e64, SUC_lexicon_tags_11 },
    { 0x29d93e65, SUC_lexicon_tags_87 },
    { 0x5e793e65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d93e69, SUC_lexicon_tags_45 },
    { 0x7a793e6a, SUC_lexicon_tags_26 },
    { 0xfa593e69, SUC_lexicon_tags_15 },
    { 0x28d93e6b, SUC_lexicon_tags_39 },
    { 0x4d193e6d, SUC_lexicon_tags_28 },
    { 0xbb993e6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54593e73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5cb93e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2393e78, SUC_lexicon_tags_6 },
    { 0x9ab93e79, SUC_lexicon_tags_17 },
    { 0xaf393e7a, SUC_lexicon_tags_60 },
    { 0x22f93e7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x26193e7e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd93e83, SUC_lexicon_tags_15 },
    { 0x99793e84, SUC_lexicon_tags_60 },
    { 0x2db93e84, SUC_lexicon_tags_33 },
    { 0xf6393e86, SUC_lexicon_tags_15 },
    { 0x79393e85, SUC_lexicon_tags_48 },
    { 0x5f993e88, SUC_lexicon_tags_2 },
    { 0xfeb93e88, SUC_lexicon_tags_1 },
    { 0x9e793e86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f93e93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xad593e95, SUC_lexicon_tags_27 },
    { 0x36193e96, SUC_lexicon_tags_17 },
    { 0x15193e97, SUC_lexicon_tags_19 },
    { 0xbf593e97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x56393e9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b93e9e, SUC_lexicon_tags_11 },
    { 0x13993e9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3593ea3, SUC_lexicon_tags_15 },
    { 0x31f93ea4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33393ea8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70393eaa, SUC_lexicon_tags_7 },
    { 0x9ff93eaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5993eaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8993eb3, SUC_lexicon_tags_36 },
    { 0xd6593eb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16b93eb6, SUC_lexicon_tags_0 },
    { 0x7bd93eb7, SUC_lexicon_tags_26 },
    { 0x16193eb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d93ebd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c993ebf, SUC_lexicon_tags_10 },
    { 0x3c793ec0, SUC_lexicon_tags_10 },
    { 0xdfb93ec1, SUC_lexicon_tags_37 },
    { 0x66993ec2, SUC_lexicon_tags_23 },
    { 0xdc793ec3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff993ec9, SUC_lexicon_tags_37 },
    { 0xd9593eca, SUC_lexicon_tags_27 },
    { 0x64993ecb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd93ecf, SUC_lexicon_tags_7 },
    { 0xff393ecf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d93ed5, SUC_lexicon_tags_30 },
    { 0x79593ed6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95993edc, SUC_lexicon_tags_29 },
    { 0x53393edd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7dd93edf, SUC_lexicon_tags_55 },
    { 0xe1393ee0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0593ee8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad93eee, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb393ef1, SUC_lexicon_tags_35 },
    { 0x33793ef1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf93ef9, SUC_lexicon_tags_47 },
    { 0x27b93efa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8b93efc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b593eff, SUC_lexicon_tags_56 },
    { 0xe0793f00, SUC_lexicon_tags_56 },
    { 0xb4593eff, SUC_lexicon_tags_15 },
    { 0xc2193f00, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b93f06, SUC_lexicon_tags_26 },
    { 0xa3193f06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9393f09, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3993f0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab93f10, SUC_lexicon_tags_37 },
    { 0x53593f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f93f1a, SUC_lexicon_tags_60 },
    { 0x66593f1b, SUC_lexicon_tags_13 },
    { 0x6cd93f1c, SUC_lexicon_tags_19 },
    { 0xb0f93f1d, SUC_lexicon_tags_38 },
    { 0xcbb93f1e, SUC_lexicon_tags_37 },
    { 0xed593f1c, SUC_lexicon_tags_12 },
    { 0x4793f20, SUC_lexicon_tags_28 },
    { 0xcc393f21, SUC_lexicon_tags_15 },
    { 0x40193f22, SUC_lexicon_tags_63 },
    { 0x6f993f22, SUC_lexicon_tags_15 },
    { 0xb8393f24, SUC_lexicon_tags_37 },
    { 0x70593f20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26f93f28, SUC_lexicon_tags_29 },
    { 0x64b93f28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6393f2b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a593f2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3f93f30, SUC_lexicon_tags_14 },
    { 0xc8f93f31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b93f36, SUC_lexicon_tags_0 },
    { 0x8c993f37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21993f3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x54193f40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97793f45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf193f4a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7193f4d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf93f51, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2793f56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5993f59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6df93f5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x40393f5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6793f60, SUC_lexicon_tags_11 },
    { 0x9cb93f61, SUC_lexicon_tags_31 },
    { 0x1a793f62, SUC_lexicon_tags_1 },
    { 0x9c193f63, SUC_lexicon_tags_37 },
    { 0xa3f93f63, SUC_lexicon_tags_33 },
    { 0x82393f61, SUC_lexicon_tags_0 },
    { 0x14b93f66, SUC_lexicon_tags_11 },
    { 0xd8993f66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1193f6e, SUC_lexicon_tags_17 },
    { 0xa6793f6e, SUC_lexicon_tags_11 },
    { 0xcf993f70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ff93f72, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x61793f75, SUC_lexicon_tags_25 },
    { 0x70193f75, SUC_lexicon_tags_7 },
    { 0x30b93f77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25793f7a, SUC_lexicon_tags_15 },
    { 0x2b593f7a, SUC_lexicon_tags_7 },
    { 0xb5193f7a, SUC_lexicon_tags_26 },
    { 0x35393f7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f93f81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c393f84, SUC_lexicon_tags_28 },
    { 0xaf793f85, SUC_lexicon_tags_45 },
    { 0xdc993f85, SUC_lexicon_tags_12 },
    { 0xb7f93f87, SUC_lexicon_tags_37 },
    { 0x61193f86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd93f8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x21593f90, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x91993f92, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7393f95, SUC_lexicon_tags_11 },
    { 0xc1393f96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7793f98, SUC_lexicon_tags_15 },
    { 0xdd393f99, SUC_lexicon_tags_21 },
    { 0x2e793f98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19993fa0, SUC_lexicon_tags_37 },
    { 0xc8f93fa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d793fa6, SUC_lexicon_tags_25 },
    { 0xc1993fa7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef193fb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd593fb5, SUC_lexicon_tags_37 },
    { 0x4ed93fb5, SUC_lexicon_tags_7 },
    { 0x23793fb6, SUC_lexicon_tags_28 },
    { 0x81d93fb8, SUC_lexicon_tags_31 },
    { 0x12d93fb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb9793fbb, SUC_lexicon_tags_4 },
    { 0x49193fbc, SUC_lexicon_tags_15 },
    { 0xa4993fbb, SUC_lexicon_tags_27 },
    { 0xfcb93fbb, SUC_lexicon_tags_12 },
    { 0x3993fbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd93fc8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90393fcd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c793fd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4193fd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78193fd4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98593fda, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69993fe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e793fe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6393fe6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xacd93fe8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb93ff0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x14f93ff2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93193ffa, SUC_lexicon_tags_19 },
    { 0x57993ffb, SUC_lexicon_tags_12 },
    { 0xa8793ffc, SUC_lexicon_tags_12 },
    { 0x1bf93ffd, SUC_lexicon_tags_17 },
    { 0xf8193ffe, SUC_lexicon_tags_1 },
    { 0xa0b93fff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73194003, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f9400b, SUC_lexicon_tags_7 },
    { 0xc219400c, SUC_lexicon_tags_15 },
    { 0xe999400c, SUC_lexicon_tags_12 },
    { 0x50f9400e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xeeb94010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b9401a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6779401c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d94022, SUC_lexicon_tags_17 },
    { 0x61794022, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d194028, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d9402b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9239402d, SUC_lexicon_tags_2 },
    { 0x25f9402e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd994038, SUC_lexicon_tags_17 },
    { 0xf2594038, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd559403d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef94045, SUC_lexicon_tags_33 },
    { 0x4394046, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f9404a, SUC_lexicon_tags_11 },
    { 0x139404b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db9404f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70994051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e994053, SUC_lexicon_tags_35 },
    { 0x6f394054, SUC_lexicon_tags_29 },
    { 0xc3d94055, SUC_lexicon_tags_46 },
    { 0xc1d94054, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d94059, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdf59405b, SUC_lexicon_tags_2 },
    { 0xd99405b, SUC_lexicon_tags_32 },
    { 0x38b9405d, SUC_lexicon_tags_14 },
    { 0x8ab9405e, SUC_lexicon_tags_17 },
    { 0x9139405f, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xb7194061, SUC_lexicon_tags_9 },
    { 0x8ad94062, SUC_lexicon_tags_2 },
    { 0xc9d94061, SUC_lexicon_tags_17 },
    { 0xafd94061, SUC_lexicon_tags_68 },
    { 0x36794065, SUC_lexicon_tags_12 },
    { 0x2e794064, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x53994068, SUC_lexicon_tags_1 },
    { 0xe8594069, SUC_lexicon_tags_16 },
    { 0xed19406a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b9406d, SUC_lexicon_tags_7 },
    { 0x6899406e, SUC_lexicon_tags_45 },
    { 0xac79406e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd194072, SUC_lexicon_tags_1 },
    { 0xfdb94072, SUC_lexicon_tags_10 },
    { 0x20794074, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58594078, SUC_lexicon_tags_3 },
    { 0x4fd94079, SUC_lexicon_tags_47 },
    { 0x43994078, SUC_lexicon_tags_37 },
    { 0x6b9407a, SUC_lexicon_tags_25 },
    { 0x17194079, SUC_lexicon_tags_32 },
    { 0x4cd9407d, SUC_lexicon_tags_49 },
    { 0x3cb9407e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f94082, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f9408a, SUC_lexicon_tags_27 },
    { 0x1979408a, SUC_lexicon_tags_1 },
    { 0x9c39408c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e194090, SUC_lexicon_tags_25 },
    { 0xb1f94091, SUC_lexicon_tags_7 },
    { 0x38394092, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84194097, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a59409a, SUC_lexicon_tags_47 },
    { 0x80b9409a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807940a1, SUC_lexicon_tags_77 },
    { 0x5c1940a2, SUC_lexicon_tags_27 },
    { 0x6ad940a3, SUC_lexicon_tags_63 },
    { 0x92d940a4, SUC_lexicon_tags_33 },
    { 0xbe5940a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46f940a7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417940ac, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x80b940ae, SUC_lexicon_tags_60 },
    { 0xe97940af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87940b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53940b6, SUC_lexicon_tags_17 },
    { 0xf4f940b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc69940b9, SUC_lexicon_tags_17 },
    { 0x475940ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539940be, SUC_lexicon_tags_11 },
    { 0x60b940bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7940c2, SUC_lexicon_tags_23 },
    { 0x29f940c3, SUC_lexicon_tags_1 },
    { 0x239940c4, SUC_lexicon_tags_19 },
    { 0xdad940c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b940c9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5940cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53940d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb39940db, SUC_lexicon_tags_17 },
    { 0x563940dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b940df, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x951940e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1940ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x665940ec, SUC_lexicon_tags_2 },
    { 0xf75940ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73940f0, SUC_lexicon_tags_38 },
    { 0x1c7940f0, SUC_lexicon_tags_12 },
    { 0x73f940f2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf940fa, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x4c7940fc, SUC_lexicon_tags_19 },
    { 0xa9f940fd, SUC_lexicon_tags_28 },
    { 0x23940fe, SUC_lexicon_tags_454 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f94101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78594103, SUC_lexicon_tags_1 },
    { 0xfc394103, SUC_lexicon_tags_12 },
    { 0x3c194105, SUC_lexicon_tags_15 },
    { 0x5cd94106, SUC_lexicon_tags_63 },
    { 0x8a194107, SUC_lexicon_tags_66 },
    { 0x8f194108, SUC_lexicon_tags_65 },
    { 0x83b94108, SUC_lexicon_tags_37 },
    { 0xcdd94103, SUC_lexicon_tags_15 },
    { 0x8e79410b, SUC_lexicon_tags_51 },
    { 0x3759410c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17394110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a594114, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4994118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2759411c, SUC_lexicon_tags_69 },
    { 0x3e79411d, SUC_lexicon_tags_1 },
    { 0x6879411c, SUC_lexicon_tags_15 },
    { 0xf099411e, SUC_lexicon_tags_25 },
    { 0x9159411f, SUC_lexicon_tags_2 },
    { 0xa7f94121, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7994123, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc194126, SUC_lexicon_tags_1 },
    { 0xc3194126, SUC_lexicon_tags_20 },
    { 0xf8b94128, SUC_lexicon_tags_37 },
    { 0x8ab94129, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5394131, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b394133, SUC_lexicon_tags_28 },
    { 0x12794134, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d94137, SUC_lexicon_tags_3 },
    { 0xb6994138, SUC_lexicon_tags_17 },
    { 0x49194138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x779413b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b94142, SUC_lexicon_tags_0 },
    { 0x6a794142, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66994146, SUC_lexicon_tags_68 },
    { 0x46594146, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1cf94149, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe059414d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f94152, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b9415a, SUC_lexicon_tags_2 },
    { 0x2ed9415b, SUC_lexicon_tags_66 },
    { 0x1c39415c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a99415e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f794160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57594163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa794169, SUC_lexicon_tags_27 },
    { 0xc3d94169, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xaab9416c, SUC_lexicon_tags_26 },
    { 0xd2b9416d, SUC_lexicon_tags_2 },
    { 0xaa99416d, SUC_lexicon_tags_6 },
    { 0x3799416f, SUC_lexicon_tags_60 },
    { 0x6bb9416f, SUC_lexicon_tags_1 },
    { 0x31394171, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaff94173, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc394176, SUC_lexicon_tags_1 },
    { 0xf6b94177, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4979417a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf9417e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a794181, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe394186, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xff394189, SUC_lexicon_tags_82 },
    { 0xb839418a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf94191, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf394193, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb94197, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f9419c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x9899419e, SUC_lexicon_tags_37 },
    { 0x459419e, SUC_lexicon_tags_72 },
    { 0x17f9419e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25941a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35941a7, SUC_lexicon_tags_57 },
    { 0xb6d941a8, SUC_lexicon_tags_32 },
    { 0xea3941a7, SUC_lexicon_tags_25 },
    { 0x3cd941a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9941b0, SUC_lexicon_tags_13 },
    { 0x567941b0, SUC_lexicon_tags_30 },
    { 0xec7941b2, SUC_lexicon_tags_17 },
    { 0x165941b3, SUC_lexicon_tags_40 },
    { 0xb89941b3, SUC_lexicon_tags_7 },
    { 0x8db941b5, SUC_lexicon_tags_23 },
    { 0x149941b6, SUC_lexicon_tags_87 },
    { 0x4b7941b6, SUC_lexicon_tags_11 },
    { 0xbbb941b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd941bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91b941bd, SUC_lexicon_tags_68 },
    { 0x837941be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1941c2, SUC_lexicon_tags_61 },
    { 0x16b941c3, SUC_lexicon_tags_8 },
    { 0x921941c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x565941c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829941cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f941d0, SUC_lexicon_tags_106 },
    { 0x42f941d0, SUC_lexicon_tags_29 },
    { 0x321941d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1941d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df941d9, SUC_lexicon_tags_25 },
    { 0x489941da, SUC_lexicon_tags_12 },
    { 0x745941db, SUC_lexicon_tags_12 },
    { 0x11f941dc, SUC_lexicon_tags_14 },
    { 0x98d941dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53941e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x185941e3, SUC_lexicon_tags_1 },
    { 0x1c3941e4, SUC_lexicon_tags_17 },
    { 0x9e7941e5, SUC_lexicon_tags_11 },
    { 0xa21941e4, SUC_lexicon_tags_82 },
    { 0xc2b941e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b941ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe77941ee, SUC_lexicon_tags_26 },
    { 0xc3b941ee, SUC_lexicon_tags_11 },
    { 0x3f5941ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd941f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x981941f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef941fa, SUC_lexicon_tags_27 },
    { 0xd69941fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17941fd, SUC_lexicon_tags_17 },
    { 0xe19941fe, SUC_lexicon_tags_11 },
    { 0x9b9941fd, SUC_lexicon_tags_14 },
    { 0x36994200, SUC_lexicon_tags_47 },
    { 0xeb94200, SUC_lexicon_tags_30 },
    { 0x38794202, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8394204, SUC_lexicon_tags_63 },
    { 0xa9594204, SUC_lexicon_tags_17 },
    { 0x7e794206, SUC_lexicon_tags_19 },
    { 0x32b94206, SUC_lexicon_tags_60 },
    { 0x41b94204, SUC_lexicon_tags_12 },
    { 0xe0194205, SUC_lexicon_tags_31 },
    { 0x9339420a, SUC_lexicon_tags_17 },
    { 0x73b9420b, SUC_lexicon_tags_60 },
    { 0xf1d9420c, SUC_lexicon_tags_72 },
    { 0xb8d9420a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65794210, SUC_lexicon_tags_33 },
    { 0xc5994211, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b194213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xacd94215, SUC_lexicon_tags_56 },
    { 0xfc394216, SUC_lexicon_tags_37 },
    { 0x42994217, SUC_lexicon_tags_12 },
    { 0x6b94217, SUC_lexicon_tags_17 },
    { 0x48d94217, SUC_lexicon_tags_25 },
    { 0x45f9421a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2139421c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5939421e, SUC_lexicon_tags_12 },
    { 0xe759421f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdff94221, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99422b, SUC_lexicon_tags_30 },
    { 0x80b9422c, SUC_lexicon_tags_23 },
    { 0xb319422d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1d99422f, SUC_lexicon_tags_23 },
    { 0x9179422f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xea194232, SUC_lexicon_tags_11 },
    { 0x35794233, SUC_lexicon_tags_26 },
    { 0x47b94233, SUC_lexicon_tags_59 },
    { 0xbfb94235, SUC_lexicon_tags_33 },
    { 0x3cf94235, SUC_lexicon_tags_47 },
    { 0x59394235, SUC_lexicon_tags_6 },
    { 0x35f94238, SUC_lexicon_tags_82 },
    { 0xab594238, SUC_lexicon_tags_0 },
    { 0xf3394232, SUC_lexicon_tags_60 },
    { 0x32f94237, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9423e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2394242, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b94246, SUC_lexicon_tags_107 },
    { 0x52194247, SUC_lexicon_tags_7 },
    { 0x59794246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a79424a, SUC_lexicon_tags_6 },
    { 0xe579424b, SUC_lexicon_tags_17 },
    { 0x8399424a, SUC_lexicon_tags_0 },
    { 0xad19424d, SUC_lexicon_tags_26 },
    { 0xed9424a, SUC_lexicon_tags_9 },
    { 0x2419424a, SUC_lexicon_tags_17 },
    { 0xb3d94250, SUC_lexicon_tags_35 },
    { 0xa1794251, SUC_lexicon_tags_37 },
    { 0x8499424b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1394254, SUC_lexicon_tags_17 },
    { 0x63794255, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4659425c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e99425f, SUC_lexicon_tags_33 },
    { 0x6d9425f, SUC_lexicon_tags_7 },
    { 0x57994261, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4599426a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6b9426c, SUC_lexicon_tags_26 },
    { 0x9c59426d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4fb9426f, SUC_lexicon_tags_47 },
    { 0xbc394270, SUC_lexicon_tags_6 },
    { 0x6579426f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b94275, SUC_lexicon_tags_11 },
    { 0x2794276, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf839427e, SUC_lexicon_tags_27 },
    { 0xcb59427f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24394283, SUC_lexicon_tags_14 },
    { 0x36194284, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4994289, SUC_lexicon_tags_84 },
    { 0xb879428a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30d9428c, SUC_lexicon_tags_23 },
    { 0xca19428d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f94292, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x64594294, SUC_lexicon_tags_25 },
    { 0x90594294, SUC_lexicon_tags_25 },
    { 0x9c994296, SUC_lexicon_tags_11 },
    { 0x5af94296, SUC_lexicon_tags_1 },
    { 0xad94297, SUC_lexicon_tags_11 },
    { 0x21994297, SUC_lexicon_tags_30 },
    { 0x93b9429a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb739429c, SUC_lexicon_tags_47 },
    { 0x2299429d, SUC_lexicon_tags_49 },
    { 0xbdf9429c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x775942a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1c5942a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x435942a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13942a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x817942ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x481942ad, SUC_lexicon_tags_12 },
    { 0xc9b942ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e9942b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x751942b3, SUC_lexicon_tags_11 },
    { 0xc7b942b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa3f942b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x877942b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bf942ba, SUC_lexicon_tags_34 },
    { 0x135942bb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8b942bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f3942bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f942c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591942c7, SUC_lexicon_tags_12 },
    { 0x201942c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b942d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77942d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19942df, SUC_lexicon_tags_47 },
    { 0xf97942df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59942e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859942e7, SUC_lexicon_tags_11 },
    { 0xc33942e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f942f1, SUC_lexicon_tags_37 },
    { 0xcb942f2, SUC_lexicon_tags_1 },
    { 0xe3942f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e1942f5, SUC_lexicon_tags_55 },
    { 0x59942f6, SUC_lexicon_tags_15 },
    { 0x9c9942f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x911942f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97942ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x49594301, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87194308, SUC_lexicon_tags_87 },
    { 0xb0794308, SUC_lexicon_tags_17 },
    { 0x1d39430a, SUC_lexicon_tags_31 },
    { 0x7179430b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9159430d, SUC_lexicon_tags_37 },
    { 0x1ad9430e, SUC_lexicon_tags_2 },
    { 0xded9430e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7f94311, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20394316, SUC_lexicon_tags_60 },
    { 0xbe794316, SUC_lexicon_tags_17 },
    { 0xc7594317, SUC_lexicon_tags_10 },
    { 0x99994318, SUC_lexicon_tags_11 },
    { 0x7959431a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c59431f, SUC_lexicon_tags_66 },
    { 0x1b94320, SUC_lexicon_tags_55 },
    { 0x98194321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff94329, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a39432e, SUC_lexicon_tags_22 },
    { 0xee79432f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d94333, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5bf94335, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66d94339, SUC_lexicon_tags_12 },
    { 0xb779433a, SUC_lexicon_tags_113 },
    { 0xdf39433b, SUC_lexicon_tags_15 },
    { 0xe999433c, SUC_lexicon_tags_15 },
    { 0xd199433d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49594341, SUC_lexicon_tags_2 },
    { 0xaf394342, SUC_lexicon_tags_25 },
    { 0xf8794342, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4594345, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa1194347, SUC_lexicon_tags_15 },
    { 0x44794347, SUC_lexicon_tags_8 },
    { 0x1e194349, SUC_lexicon_tags_14 },
    { 0x59d94349, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4794352, SUC_lexicon_tags_6 },
    { 0x52794353, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d94357, SUC_lexicon_tags_27 },
    { 0xb1194357, SUC_lexicon_tags_26 },
    { 0x44d94359, SUC_lexicon_tags_28 },
    { 0x7f194358, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2994360, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8594367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43194373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af94376, SUC_lexicon_tags_7 },
    { 0x81594377, SUC_lexicon_tags_26 },
    { 0x9b994376, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b9437b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2e59437d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4759437f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7194382, SUC_lexicon_tags_7 },
    { 0x95594383, SUC_lexicon_tags_46 },
    { 0x83594384, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa394388, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4019438b, SUC_lexicon_tags_9 },
    { 0x16f9438c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e79438e, SUC_lexicon_tags_12 },
    { 0x2539438f, SUC_lexicon_tags_15 },
    { 0x3f94390, SUC_lexicon_tags_40 },
    { 0x30794390, SUC_lexicon_tags_28 },
    { 0x8f19438e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb594395, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76394397, SUC_lexicon_tags_11 },
    { 0x34994397, SUC_lexicon_tags_17 },
    { 0xd7f94399, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7699439b, SUC_lexicon_tags_15 },
    { 0x8d19439c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b39439f, SUC_lexicon_tags_29 },
    { 0x1a39439f, SUC_lexicon_tags_6 },
    { 0x8c3943a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1943a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df943ab, SUC_lexicon_tags_25 },
    { 0x6b3943ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb943af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa7b943b1, SUC_lexicon_tags_55 },
    { 0x707943b2, SUC_lexicon_tags_17 },
    { 0x935943b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef943b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdef943b8, SUC_lexicon_tags_25 },
    { 0xf29943b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b943bd, SUC_lexicon_tags_33 },
    { 0x9b5943bd, SUC_lexicon_tags_68 },
    { 0xe73943be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2e9943c1, SUC_lexicon_tags_15 },
    { 0x3e9943c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71943cb, SUC_lexicon_tags_13 },
    { 0x361943cc, SUC_lexicon_tags_65 },
    { 0x48d943cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf81943cf, SUC_lexicon_tags_10 },
    { 0x15f943cf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xddb943d2, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67943d6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869943dc, SUC_lexicon_tags_27 },
    { 0xedb943dd, SUC_lexicon_tags_2 },
    { 0xa01943de, SUC_lexicon_tags_39 },
    { 0xc9943dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d943e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49943f1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf943f7, SUC_lexicon_tags_2 },
    { 0x63943f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47943fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x633943fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d94408, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa499440f, SUC_lexicon_tags_11 },
    { 0xd8f94410, SUC_lexicon_tags_10 },
    { 0xd7394411, SUC_lexicon_tags_37 },
    { 0x3d594412, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2639441b, SUC_lexicon_tags_124 },
    { 0xbb59441c, SUC_lexicon_tags_1 },
    { 0x9c39441b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b94423, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6794425, SUC_lexicon_tags_306 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf9442a, SUC_lexicon_tags_13 },
    { 0xda59442b, SUC_lexicon_tags_38 },
    { 0x1fb9442c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc1b9442e, SUC_lexicon_tags_7 },
    { 0xb199442f, SUC_lexicon_tags_38 },
    { 0xd0794430, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d94433, SUC_lexicon_tags_0 },
    { 0x69f94434, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f794438, SUC_lexicon_tags_11 },
    { 0x37794438, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e99443c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x30b9443e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6794442, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97594446, SUC_lexicon_tags_7 },
    { 0x3fd94447, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd19444a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8039444d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf94454, SUC_lexicon_tags_63 },
    { 0x8ab94455, SUC_lexicon_tags_87 },
    { 0xad994456, SUC_lexicon_tags_1 },
    { 0xbb394455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf9445a, SUC_lexicon_tags_26 },
    { 0x87f9445b, SUC_lexicon_tags_12 },
    { 0x459445a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0194461, SUC_lexicon_tags_1 },
    { 0x54d94462, SUC_lexicon_tags_27 },
    { 0x9c594461, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc194465, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69394469, SUC_lexicon_tags_2 },
    { 0xcaf9446a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd9446f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x72594472, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f9447d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f94483, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa059448b, SUC_lexicon_tags_0 },
    { 0xc959448c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x22d9448e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80994493, SUC_lexicon_tags_11 },
    { 0x44794493, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa394496, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb9b94498, SUC_lexicon_tags_26 },
    { 0xa994499, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7d59449b, SUC_lexicon_tags_1 },
    { 0x1a99449c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe659449e, SUC_lexicon_tags_18 },
    { 0x4a19449f, SUC_lexicon_tags_11 },
    { 0x60b9449e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9fd944a2, SUC_lexicon_tags_62 },
    { 0x667944a3, SUC_lexicon_tags_26 },
    { 0xc1f944a4, SUC_lexicon_tags_38 },
    { 0xd7944a5, SUC_lexicon_tags_33 },
    { 0x3b5944a6, SUC_lexicon_tags_7 },
    { 0x977944a7, SUC_lexicon_tags_9 },
    { 0x18d944a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd03944aa, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37944ae, SUC_lexicon_tags_1 },
    { 0xf31944af, SUC_lexicon_tags_18 },
    { 0x455944b0, SUC_lexicon_tags_39 },
    { 0xd11944b0, SUC_lexicon_tags_2 },
    { 0x4f3944b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91944b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81944ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x793944bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a3944be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51944c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d944ca, SUC_lexicon_tags_9 },
    { 0xdc7944cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2b9944cd, SUC_lexicon_tags_25 },
    { 0x611944ce, SUC_lexicon_tags_27 },
    { 0xcf7944cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819944db, SUC_lexicon_tags_12 },
    { 0xd3d944db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9944df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x109944e1, SUC_lexicon_tags_45 },
    { 0x933944e2, SUC_lexicon_tags_79 },
    { 0x867944e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x289944e6, SUC_lexicon_tags_14 },
    { 0xd07944e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63944ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9944f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xadb944f3, SUC_lexicon_tags_15 },
    { 0xf73944f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x88b944f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521944fa, SUC_lexicon_tags_17 },
    { 0x5f9944fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e7944fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6794502, SUC_lexicon_tags_31 },
    { 0x73994503, SUC_lexicon_tags_37 },
    { 0x9bf94504, SUC_lexicon_tags_26 },
    { 0x2b94503, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b94509, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19450b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f94511, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x51394514, SUC_lexicon_tags_11 },
    { 0xe4194515, SUC_lexicon_tags_26 },
    { 0x4b194516, SUC_lexicon_tags_1 },
    { 0xadd94517, SUC_lexicon_tags_87 },
    { 0xdd194515, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x939451b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7859451e, SUC_lexicon_tags_11 },
    { 0x5bf9451f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c394521, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff94526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcf94528, SUC_lexicon_tags_1 },
    { 0x90594529, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb39452b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2079452e, SUC_lexicon_tags_44 },
    { 0xadd9452f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64594531, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15394536, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1194539, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd99453b, SUC_lexicon_tags_57 },
    { 0x9759453c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2779453e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b94545, SUC_lexicon_tags_59 },
    { 0xabf94545, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b59454a, SUC_lexicon_tags_33 },
    { 0x9ed9454b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc459454e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8194553, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db94557, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b9455a, SUC_lexicon_tags_37 },
    { 0x8b79455a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd39455f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15594566, SUC_lexicon_tags_15 },
    { 0x46794567, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7fb94569, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1bd9456b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8794572, SUC_lexicon_tags_14 },
    { 0x4cf94573, SUC_lexicon_tags_7 },
    { 0x6a994574, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55994577, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1e194579, SUC_lexicon_tags_13 },
    { 0x9779457a, SUC_lexicon_tags_151 },
    { 0x2fb9457a, SUC_lexicon_tags_37 },
    { 0x1f39457c, SUC_lexicon_tags_82 },
    { 0xcf39457b, SUC_lexicon_tags_7 },
    { 0x7e19457e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f594584, SUC_lexicon_tags_2 },
    { 0x60994585, SUC_lexicon_tags_60 },
    { 0xffb94585, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6994589, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd94591, SUC_lexicon_tags_37 },
    { 0x9f394592, SUC_lexicon_tags_68 },
    { 0xc8d94592, SUC_lexicon_tags_19 },
    { 0x63994594, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f94598, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xde99459b, SUC_lexicon_tags_45 },
    { 0x1799459c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b945a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf945a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3945a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53945ad, SUC_lexicon_tags_7 },
    { 0xd1d945ae, SUC_lexicon_tags_33 },
    { 0xdf9945af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x221945b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x637945b4, SUC_lexicon_tags_17 },
    { 0x353945b5, SUC_lexicon_tags_15 },
    { 0x41d945b6, SUC_lexicon_tags_37 },
    { 0xe5b945b7, SUC_lexicon_tags_23 },
    { 0x8f1945b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59945be, SUC_lexicon_tags_17 },
    { 0x33f945bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17b945c1, SUC_lexicon_tags_11 },
    { 0x947945c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f945c6, SUC_lexicon_tags_1 },
    { 0xe0f945c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77945cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05945d0, SUC_lexicon_tags_26 },
    { 0x971945d1, SUC_lexicon_tags_17 },
    { 0x803945d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x141945d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac9945d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8b945d9, SUC_lexicon_tags_45 },
    { 0xa5b945da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27945e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x203945e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b945e5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b945f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1945f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0b945f5, SUC_lexicon_tags_60 },
    { 0xfb7945f5, SUC_lexicon_tags_37 },
    { 0x811945f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab945fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f94601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf4994603, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e194606, SUC_lexicon_tags_28 },
    { 0xaa194607, SUC_lexicon_tags_37 },
    { 0xf8394606, SUC_lexicon_tags_12 },
    { 0xff194609, SUC_lexicon_tags_35 },
    { 0xa899460a, SUC_lexicon_tags_47 },
    { 0xc9d9460b, SUC_lexicon_tags_25 },
    { 0x9119460c, SUC_lexicon_tags_6 },
    { 0x68394608, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdbb9460f, SUC_lexicon_tags_33 },
    { 0xc1d9460f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45994616, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3e994618, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64394623, SUC_lexicon_tags_2 },
    { 0xbb394623, SUC_lexicon_tags_11 },
    { 0x3df94625, SUC_lexicon_tags_10 },
    { 0xe0594626, SUC_lexicon_tags_37 },
    { 0x41194627, SUC_lexicon_tags_15 },
    { 0x22594627, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c39462b, SUC_lexicon_tags_15 },
    { 0xc4b9462b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27594633, SUC_lexicon_tags_37 },
    { 0xb1d94634, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26994638, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b9463b, SUC_lexicon_tags_27 },
    { 0x39f9463b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c19463e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa5794640, SUC_lexicon_tags_26 },
    { 0x1d194641, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19794645, SUC_lexicon_tags_48 },
    { 0xd4594646, SUC_lexicon_tags_37 },
    { 0x19d94647, SUC_lexicon_tags_2 },
    { 0x92794646, SUC_lexicon_tags_2 },
    { 0xfdb94646, SUC_lexicon_tags_23 },
    { 0xee394647, SUC_lexicon_tags_1 },
    { 0x7d59464b, SUC_lexicon_tags_47 },
    { 0x1699464b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x179464e, SUC_lexicon_tags_14 },
    { 0xc679464e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69594654, SUC_lexicon_tags_26 },
    { 0x4a194654, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x39794657, SUC_lexicon_tags_13 },
    { 0x30b94657, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe279465e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3d94660, SUC_lexicon_tags_82 },
    { 0xf0d94661, SUC_lexicon_tags_1 },
    { 0xcad94661, SUC_lexicon_tags_12 },
    { 0x25f94663, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f794665, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d994667, SUC_lexicon_tags_11 },
    { 0xb1194667, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f94673, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb94679, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf319467d, SUC_lexicon_tags_27 },
    { 0xcc19467d, SUC_lexicon_tags_49 },
    { 0x1c79467d, SUC_lexicon_tags_25 },
    { 0xfad94680, SUC_lexicon_tags_25 },
    { 0x8a594681, SUC_lexicon_tags_37 },
    { 0xfc194680, SUC_lexicon_tags_47 },
    { 0x5b994683, SUC_lexicon_tags_5 },
    { 0x9eb94683, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6594686, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb94689, SUC_lexicon_tags_33 },
    { 0x42b9468a, SUC_lexicon_tags_15 },
    { 0x4b39468b, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d94690, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2994694, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49594697, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd7994699, SUC_lexicon_tags_37 },
    { 0x99469a, SUC_lexicon_tags_1 },
    { 0x1e19469b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5059469e, SUC_lexicon_tags_25 },
    { 0x53d9469f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3946a7, SUC_lexicon_tags_68 },
    { 0xc21946a7, SUC_lexicon_tags_7 },
    { 0x69f946a9, SUC_lexicon_tags_2 },
    { 0xc17946a9, SUC_lexicon_tags_47 },
    { 0xbf3946a8, SUC_lexicon_tags_61 },
    { 0x7e7946a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1946b3, SUC_lexicon_tags_44 },
    { 0xa77946b3, SUC_lexicon_tags_11 },
    { 0x487946b5, SUC_lexicon_tags_17 },
    { 0x95946b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d946bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f946c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9946cc, SUC_lexicon_tags_34 },
    { 0x59946cd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x64f946cf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1946d3, SUC_lexicon_tags_38 },
    { 0xc57946d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2f1946d6, SUC_lexicon_tags_19 },
    { 0x44f946d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x967946da, SUC_lexicon_tags_8 },
    { 0xd7b946da, SUC_lexicon_tags_10 },
    { 0xa83946dc, SUC_lexicon_tags_6 },
    { 0xda1946db, SUC_lexicon_tags_1 },
    { 0xd3b946de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4df946e0, SUC_lexicon_tags_28 },
    { 0xda3946e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319946e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x25d946e7, SUC_lexicon_tags_11 },
    { 0x4c9946e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b946eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d1946ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3946f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x54f946f2, SUC_lexicon_tags_43 },
    { 0x7c9946f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741946f7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9946ff, SUC_lexicon_tags_17 },
    { 0x10d946ff, SUC_lexicon_tags_53 },
    { 0xe3394701, SUC_lexicon_tags_12 },
    { 0xb994700, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d94709, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeed9470b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd219470d, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0x65b9470f, SUC_lexicon_tags_12 },
    { 0xfcf94710, SUC_lexicon_tags_68 },
    { 0xebd9470f, SUC_lexicon_tags_7 },
    { 0xd394711, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27b94714, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa7d94716, SUC_lexicon_tags_11 },
    { 0x86b94717, SUC_lexicon_tags_8 },
    { 0x1bb94718, SUC_lexicon_tags_25 },
    { 0x58f94717, SUC_lexicon_tags_43 },
    { 0xf6f9471a, SUC_lexicon_tags_17 },
    { 0xcf594718, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2199471f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e394726, SUC_lexicon_tags_11 },
    { 0xc9594727, SUC_lexicon_tags_63 },
    { 0xac994728, SUC_lexicon_tags_11 },
    { 0x69994728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719472e, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27394732, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5e794734, SUC_lexicon_tags_25 },
    { 0x26f94735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d794737, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcef94739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9473e, SUC_lexicon_tags_29 },
    { 0xe719473e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b794742, SUC_lexicon_tags_17 },
    { 0xff994743, SUC_lexicon_tags_1 },
    { 0x4b594743, SUC_lexicon_tags_23 },
    { 0x70394745, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff19474b, SUC_lexicon_tags_0 },
    { 0x68b9474b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5979474e, SUC_lexicon_tags_1 },
    { 0x79b9474f, SUC_lexicon_tags_11 },
    { 0x80b9474e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd794757, SUC_lexicon_tags_17 },
    { 0x99794758, SUC_lexicon_tags_7 },
    { 0x9394759, SUC_lexicon_tags_11 },
    { 0xd2794759, SUC_lexicon_tags_11 },
    { 0x1f39475b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1f9475d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b94760, SUC_lexicon_tags_26 },
    { 0x8d594760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab94765, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d194767, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2639476d, SUC_lexicon_tags_7 },
    { 0xcef9476e, SUC_lexicon_tags_25 },
    { 0x60f9476f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d94778, SUC_lexicon_tags_7 },
    { 0x96994778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf9477e, SUC_lexicon_tags_17 },
    { 0xbb39477f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x72394782, SUC_lexicon_tags_37 },
    { 0xa6594782, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1b94785, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf794787, SUC_lexicon_tags_25 },
    { 0x9c794787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3db9478a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4194791, SUC_lexicon_tags_12 },
    { 0x3bd94792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f994794, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d94798, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe719479a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x91b9479c, SUC_lexicon_tags_7 },
    { 0x6c99479d, SUC_lexicon_tags_19 },
    { 0x8519479d, SUC_lexicon_tags_10 },
    { 0xfb99479f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95947a1, SUC_lexicon_tags_49 },
    { 0x845947a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01947a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4db947a7, SUC_lexicon_tags_14 },
    { 0xdc9947a8, SUC_lexicon_tags_25 },
    { 0x69947a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf947ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef947b2, SUC_lexicon_tags_7 },
    { 0x857947b3, SUC_lexicon_tags_11 },
    { 0xe5947b4, SUC_lexicon_tags_12 },
    { 0xf43947b5, SUC_lexicon_tags_8 },
    { 0xc61947b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913947ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe91947bc, SUC_lexicon_tags_25 },
    { 0x133947bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf947c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f947c7, SUC_lexicon_tags_63 },
    { 0x3e1947c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd947cc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453947d4, SUC_lexicon_tags_40 },
    { 0xf0b947d5, SUC_lexicon_tags_7 },
    { 0xb3b947d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcd947d8, SUC_lexicon_tags_19 },
    { 0x987947d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f9947db, SUC_lexicon_tags_16 },
    { 0xfcd947db, SUC_lexicon_tags_107 },
    { 0x5c9947dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92d947df, SUC_lexicon_tags_47 },
    { 0x709947e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7947e4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x257947e6, SUC_lexicon_tags_49 },
    { 0x4c7947e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3947eb, SUC_lexicon_tags_33 },
    { 0x1cf947ec, SUC_lexicon_tags_37 },
    { 0xe77947eb, SUC_lexicon_tags_25 },
    { 0x7d7947ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763947f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591947f6, SUC_lexicon_tags_38 },
    { 0x785947f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73947fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x149947fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21394800, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d94803, SUC_lexicon_tags_45 },
    { 0x30d94803, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4cd94806, SUC_lexicon_tags_16 },
    { 0x67d94807, SUC_lexicon_tags_61 },
    { 0x7ff94806, SUC_lexicon_tags_15 },
    { 0x2ff94808, SUC_lexicon_tags_17 },
    { 0x1a59480a, SUC_lexicon_tags_17 },
    { 0x9279480b, SUC_lexicon_tags_4 },
    { 0xbdd9480b, SUC_lexicon_tags_11 },
    { 0x6739480d, SUC_lexicon_tags_31 },
    { 0xa839480c, SUC_lexicon_tags_87 },
    { 0x5e39480f, SUC_lexicon_tags_82 },
    { 0xaed9480d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x22594813, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739481b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e794820, SUC_lexicon_tags_26 },
    { 0xfdd94820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c194823, SUC_lexicon_tags_25 },
    { 0xf9994823, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf79482a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x91b9482c, SUC_lexicon_tags_33 },
    { 0xab19482d, SUC_lexicon_tags_47 },
    { 0x98d9482e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xedf94830, SUC_lexicon_tags_17 },
    { 0xc7794831, SUC_lexicon_tags_0 },
    { 0x44994832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38b94834, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8d94839, SUC_lexicon_tags_43 },
    { 0x77b9483a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbad94842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f994845, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d94848, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf59484b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f9484e, SUC_lexicon_tags_60 },
    { 0x2759484f, SUC_lexicon_tags_49 },
    { 0x8ad94850, SUC_lexicon_tags_485 },
    { 0x9e994850, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2cb94853, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0xd4994855, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x72794857, SUC_lexicon_tags_11 },
    { 0xd6f94858, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb9485d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d94860, SUC_lexicon_tags_47 },
    { 0x6c194861, SUC_lexicon_tags_559 },
    { 0xb6194862, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x85b94864, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x13d94866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaf994868, SUC_lexicon_tags_38 },
    { 0x20194869, SUC_lexicon_tags_60 },
    { 0xbb99486a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4839486d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf94872, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4f94874, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfb94876, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd9487a, SUC_lexicon_tags_25 },
    { 0x32b9487b, SUC_lexicon_tags_47 },
    { 0x6519487a, SUC_lexicon_tags_39 },
    { 0x21f9487b, SUC_lexicon_tags_0 },
    { 0xce99487a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3db94880, SUC_lexicon_tags_27 },
    { 0x63f94881, SUC_lexicon_tags_36 },
    { 0x7794882, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f994885, SUC_lexicon_tags_92 },
    { 0xe5594886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bf94888, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b99488b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xea99488e, SUC_lexicon_tags_7 },
    { 0xb939488f, SUC_lexicon_tags_60 },
    { 0x5bb9488e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc0994892, SUC_lexicon_tags_9 },
    { 0xc0194893, SUC_lexicon_tags_49 },
    { 0x30194894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b94898, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6959489b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed948a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d948a3, SUC_lexicon_tags_7 },
    { 0x445948a3, SUC_lexicon_tags_18 },
    { 0x49b948a5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9948a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad1948aa, SUC_lexicon_tags_15 },
    { 0x9ab948aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357948b0, SUC_lexicon_tags_29 },
    { 0x633948b0, SUC_lexicon_tags_17 },
    { 0x2c1948b0, SUC_lexicon_tags_1 },
    { 0x549948b1, SUC_lexicon_tags_17 },
    { 0x217948b1, SUC_lexicon_tags_56 },
    { 0xf8b948b3, SUC_lexicon_tags_15 },
    { 0xf65948b6, SUC_lexicon_tags_30 },
    { 0x905948b6, SUC_lexicon_tags_23 },
    { 0x437948b8, SUC_lexicon_tags_4 },
    { 0xf3f948b6, SUC_lexicon_tags_31 },
    { 0x999948ba, SUC_lexicon_tags_25 },
    { 0x307948ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb83948bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3948c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4fb948c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155948cb, SUC_lexicon_tags_45 },
    { 0x97d948cc, SUC_lexicon_tags_49 },
    { 0x21d948cd, SUC_lexicon_tags_2 },
    { 0x1fd948cc, SUC_lexicon_tags_65 },
    { 0xa37948cf, SUC_lexicon_tags_104 },
    { 0xf6d948ce, SUC_lexicon_tags_1 },
    { 0xedf948d1, SUC_lexicon_tags_9 },
    { 0xaa7948d2, SUC_lexicon_tags_12 },
    { 0xb6f948d3, SUC_lexicon_tags_37 },
    { 0x40f948cf, SUC_lexicon_tags_33 },
    { 0xf9b948cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb948da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df948de, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x9b7948e0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x47b948e2, SUC_lexicon_tags_1 },
    { 0x507948e2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69948e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9948ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65948f2, SUC_lexicon_tags_27 },
    { 0x763948f3, SUC_lexicon_tags_8 },
    { 0x28d948f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x201948f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbf948f8, SUC_lexicon_tags_23 },
    { 0xec9948f8, SUC_lexicon_tags_7 },
    { 0x94d948f9, SUC_lexicon_tags_37 },
    { 0x301948fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdbb948fd, SUC_lexicon_tags_14 },
    { 0x4e7948fe, SUC_lexicon_tags_17 },
    { 0x9ed948fe, SUC_lexicon_tags_17 },
    { 0xc1594900, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52194904, SUC_lexicon_tags_135 },
    { 0x64194905, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb599490a, SUC_lexicon_tags_1 },
    { 0x79b9490b, SUC_lexicon_tags_2 },
    { 0x15f9490c, SUC_lexicon_tags_26 },
    { 0xd619490d, SUC_lexicon_tags_27 },
    { 0x399490e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3994916, SUC_lexicon_tags_68 },
    { 0x80d94917, SUC_lexicon_tags_30 },
    { 0x69b94917, SUC_lexicon_tags_0 },
    { 0xb4794917, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9639491b, SUC_lexicon_tags_17 },
    { 0xae59491b, SUC_lexicon_tags_8 },
    { 0xcb9491d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7759491f, SUC_lexicon_tags_10 },
    { 0x32394920, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc794923, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3194926, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x63994928, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d9492b, SUC_lexicon_tags_25 },
    { 0x9559492b, SUC_lexicon_tags_47 },
    { 0xebf9492d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68b9492f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b394933, SUC_lexicon_tags_17 },
    { 0x2a794933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef394939, SUC_lexicon_tags_25 },
    { 0xa5d94939, SUC_lexicon_tags_12 },
    { 0xe679493b, SUC_lexicon_tags_15 },
    { 0x98f9493c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1d9493e, SUC_lexicon_tags_37 },
    { 0x8fd9493f, SUC_lexicon_tags_27 },
    { 0xc7f94940, SUC_lexicon_tags_17 },
    { 0x22f94940, SUC_lexicon_tags_2 },
    { 0xd0f94942, SUC_lexicon_tags_11 },
    { 0x95f94943, SUC_lexicon_tags_43 },
    { 0xc194941, SUC_lexicon_tags_15 },
    { 0x85d94941, SUC_lexicon_tags_15 },
    { 0x74b94940, SUC_lexicon_tags_13 },
    { 0x3a794947, SUC_lexicon_tags_17 },
    { 0x8df94948, SUC_lexicon_tags_2 },
    { 0x7f594949, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf379494d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d94950, SUC_lexicon_tags_0 },
    { 0x16794951, SUC_lexicon_tags_63 },
    { 0x89194951, SUC_lexicon_tags_27 },
    { 0xf2794953, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3194955, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d594959, SUC_lexicon_tags_17 },
    { 0xd8d9495a, SUC_lexicon_tags_7 },
    { 0xfeb9495b, SUC_lexicon_tags_33 },
    { 0xed9495c, SUC_lexicon_tags_15 },
    { 0x919495d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f94960, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x35b94962, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e194967, SUC_lexicon_tags_17 },
    { 0xf4f94968, SUC_lexicon_tags_61 },
    { 0xe594969, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf99496b, SUC_lexicon_tags_25 },
    { 0xb559496b, SUC_lexicon_tags_55 },
    { 0xb239496d, SUC_lexicon_tags_7 },
    { 0xd999496d, SUC_lexicon_tags_15 },
    { 0xe2d9496f, SUC_lexicon_tags_1 },
    { 0xdcb94970, SUC_lexicon_tags_25 },
    { 0xf779496f, SUC_lexicon_tags_1 },
    { 0xa1d94972, SUC_lexicon_tags_23 },
    { 0x2df94973, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9194975, SUC_lexicon_tags_26 },
    { 0x5d994975, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b9497a, SUC_lexicon_tags_61 },
    { 0x37b9497a, SUC_lexicon_tags_2 },
    { 0x779497b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7059497f, SUC_lexicon_tags_63 },
    { 0x5a594980, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x78394982, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19594987, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf2594989, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e19498e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16b94991, SUC_lexicon_tags_33 },
    { 0x7d394991, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd94995, SUC_lexicon_tags_26 },
    { 0xc6394995, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda594998, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d9499d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x3799499f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3949a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9949a8, SUC_lexicon_tags_23 },
    { 0xa3d949a9, SUC_lexicon_tags_31 },
    { 0x5cb949aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x75949ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d949b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c1949b7, SUC_lexicon_tags_35 },
    { 0x1b3949b7, SUC_lexicon_tags_12 },
    { 0x95b949b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcfd949bb, SUC_lexicon_tags_7 },
    { 0x85b949bc, SUC_lexicon_tags_10 },
    { 0x3c7949bd, SUC_lexicon_tags_37 },
    { 0x36b949be, SUC_lexicon_tags_7 },
    { 0x62d949bd, SUC_lexicon_tags_39 },
    { 0x19d949be, SUC_lexicon_tags_6 },
    { 0x159949c1, SUC_lexicon_tags_38 },
    { 0xb9b949c1, SUC_lexicon_tags_47 },
    { 0xfe1949c1, SUC_lexicon_tags_27 },
    { 0xd87949bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad949c7, SUC_lexicon_tags_11 },
    { 0x6a1949c7, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb949d4, SUC_lexicon_tags_11 },
    { 0xd87949d4, SUC_lexicon_tags_62 },
    { 0xdd949d4, SUC_lexicon_tags_87 },
    { 0xe37949d4, SUC_lexicon_tags_57 },
    { 0xffb949d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f949dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x343949df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfc5949e1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85949e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf1d949e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92f949e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d949ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265949f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19b949f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x197949f7, SUC_lexicon_tags_25 },
    { 0x9c5949f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf949fd, SUC_lexicon_tags_14 },
    { 0x365949fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d94a02, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfdb94a04, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5394a07, SUC_lexicon_tags_28 },
    { 0x89594a08, SUC_lexicon_tags_7 },
    { 0x40d94a09, SUC_lexicon_tags_45 },
    { 0xd6394a09, SUC_lexicon_tags_0 },
    { 0x5194a0b, SUC_lexicon_tags_25 },
    { 0xd8994a07, SUC_lexicon_tags_8 },
    { 0x73f94a09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b94a0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5794a11, SUC_lexicon_tags_39 },
    { 0xce194a12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa794a15, SUC_lexicon_tags_1 },
    { 0xb1594a16, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcff94a18, SUC_lexicon_tags_37 },
    { 0xc5394a19, SUC_lexicon_tags_46 },
    { 0x58394a1a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2994a24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35394a26, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x46194a29, SUC_lexicon_tags_39 },
    { 0x8f194a2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58794a2e, SUC_lexicon_tags_12 },
    { 0xf994a2f, SUC_lexicon_tags_35 },
    { 0x2cf94a30, SUC_lexicon_tags_66 },
    { 0x2cb94a31, SUC_lexicon_tags_2 },
    { 0x16d94a32, SUC_lexicon_tags_37 },
    { 0xdbf94a2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef94a37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d94a41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb194a49, SUC_lexicon_tags_2 },
    { 0x81794a4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1194a4c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79194a51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d794a53, SUC_lexicon_tags_46 },
    { 0xcab94a54, SUC_lexicon_tags_15 },
    { 0xfd794a54, SUC_lexicon_tags_25 },
    { 0xe1d94a53, SUC_lexicon_tags_37 },
    { 0x42594a57, SUC_lexicon_tags_11 },
    { 0xd0994a54, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x88594a5b, SUC_lexicon_tags_17 },
    { 0xa6394a5c, SUC_lexicon_tags_58 },
    { 0xcf194a5d, SUC_lexicon_tags_27 },
    { 0x26394a5e, SUC_lexicon_tags_43 },
    { 0x84394a5f, SUC_lexicon_tags_38 },
    { 0x2394a60, SUC_lexicon_tags_23 },
    { 0xbab94a61, SUC_lexicon_tags_23 },
    { 0xfe594a62, SUC_lexicon_tags_2 },
    { 0x23d94a5e, SUC_lexicon_tags_8 },
    { 0xbdf94a62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f94a68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d794a6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb94a72, SUC_lexicon_tags_77 },
    { 0x1ab94a73, SUC_lexicon_tags_27 },
    { 0x99994a73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9794a7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67194a7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc6794a7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b194a84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d94a90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa194a92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f194a97, SUC_lexicon_tags_2 },
    { 0x3cd94a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2194a9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6b94a9e, SUC_lexicon_tags_2 },
    { 0x1b794a9f, SUC_lexicon_tags_29 },
    { 0xb5394a9f, SUC_lexicon_tags_25 },
    { 0x4d194aa1, SUC_lexicon_tags_15 },
    { 0x80594aa2, SUC_lexicon_tags_43 },
    { 0x80f94a9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7d94aa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10194aaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef94ab1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55994ab4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f94ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfad94ab9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15194abe, SUC_lexicon_tags_11 },
    { 0xd0594abf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e594ac5, SUC_lexicon_tags_12 },
    { 0xdf794ac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d794ac9, SUC_lexicon_tags_32 },
    { 0x6794ac9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x1394acd, SUC_lexicon_tags_11 },
    { 0x9f194acd, SUC_lexicon_tags_26 },
    { 0x66594acf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4594ad3, SUC_lexicon_tags_44 },
    { 0x8394ad4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf94ad9, SUC_lexicon_tags_33 },
    { 0xb7194ad9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94794adc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab94ae1, SUC_lexicon_tags_17 },
    { 0xb7794ae1, SUC_lexicon_tags_7 },
    { 0xe7794ae3, SUC_lexicon_tags_17 },
    { 0xc5194ae3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2794ae8, SUC_lexicon_tags_16 },
    { 0x25794ae9, SUC_lexicon_tags_11 },
    { 0x75f94aea, SUC_lexicon_tags_25 },
    { 0xf4d94aeb, SUC_lexicon_tags_80 },
    { 0x86b94aec, SUC_lexicon_tags_21 },
    { 0x77f94aeb, SUC_lexicon_tags_47 },
    { 0xb0594ae9, SUC_lexicon_tags_23 },
    { 0x30b94ae9, SUC_lexicon_tags_2 },
    { 0xc7794af0, SUC_lexicon_tags_1 },
    { 0xf2f94af0, SUC_lexicon_tags_7 },
    { 0x4e794af0, SUC_lexicon_tags_201 },
    { 0xdc394af3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf994af6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f594af9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39194afb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x12994afd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b94b02, SUC_lexicon_tags_25 },
    { 0xc5794b03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df94b06, SUC_lexicon_tags_11 },
    { 0x15994b07, SUC_lexicon_tags_19 },
    { 0xe3b94b08, SUC_lexicon_tags_49 },
    { 0xff994b06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bf94b0b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c994b0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca594b11, SUC_lexicon_tags_19 },
    { 0x45994b12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b94b16, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2394b19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb94b21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b994b24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2cf94b26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4994b28, SUC_lexicon_tags_15 },
    { 0xf5194b29, SUC_lexicon_tags_1 },
    { 0xd5194b2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed394b31, SUC_lexicon_tags_38 },
    { 0x81b94b31, SUC_lexicon_tags_40 },
    { 0x43b94b33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20d94b36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32f94b38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2d794b3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d94b3e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf7594b40, SUC_lexicon_tags_27 },
    { 0x7a594b40, SUC_lexicon_tags_31 },
    { 0xc3794b42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb194b46, SUC_lexicon_tags_38 },
    { 0xf7594b47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb994b4c, SUC_lexicon_tags_37 },
    { 0xa0594b4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d794b54, SUC_lexicon_tags_25 },
    { 0xe2d94b55, SUC_lexicon_tags_46 },
    { 0x25594b54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75194b59, SUC_lexicon_tags_45 },
    { 0xe3994b5a, SUC_lexicon_tags_17 },
    { 0x89d94b5a, SUC_lexicon_tags_25 },
    { 0xc8b94b59, SUC_lexicon_tags_45 },
    { 0x78194b5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3cb94b5f, SUC_lexicon_tags_15 },
    { 0xe1794b60, SUC_lexicon_tags_11 },
    { 0x1fd94b61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e394b6c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5394b73, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x88b94b75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0394b79, SUC_lexicon_tags_15 },
    { 0x3f194b7a, SUC_lexicon_tags_11 },
    { 0x6bd94b7a, SUC_lexicon_tags_46 },
    { 0xf2b94b7c, SUC_lexicon_tags_15 },
    { 0xd7994b7b, SUC_lexicon_tags_15 },
    { 0x94594b7e, SUC_lexicon_tags_47 },
    { 0xae594b7e, SUC_lexicon_tags_25 },
    { 0x77f94b80, SUC_lexicon_tags_27 },
    { 0x89d94b7e, SUC_lexicon_tags_136 },
    { 0xa4794b7c, SUC_lexicon_tags_12 },
    { 0xbc394b83, SUC_lexicon_tags_13 },
    { 0x3c194b7c, SUC_lexicon_tags_9 },
    { 0x4c994b84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2994b87, SUC_lexicon_tags_60 },
    { 0xc9394b88, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff94b8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0194b98, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b194b9d, SUC_lexicon_tags_15 },
    { 0xccb94b9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a194ba3, SUC_lexicon_tags_11 },
    { 0xeb194ba4, SUC_lexicon_tags_23 },
    { 0x2b394ba4, SUC_lexicon_tags_31 },
    { 0x44194ba5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2b794ba8, SUC_lexicon_tags_37 },
    { 0x64194ba8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62194bac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb794baf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f794bb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa594bb9, SUC_lexicon_tags_1 },
    { 0x73194bba, SUC_lexicon_tags_45 },
    { 0x6c794bb9, SUC_lexicon_tags_25 },
    { 0x22994bbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96594bc1, SUC_lexicon_tags_16 },
    { 0x5d94bc1, SUC_lexicon_tags_46 },
    { 0x22794bc3, SUC_lexicon_tags_2 },
    { 0x7dd94bc4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38594bc8, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd94bd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd794bd5, SUC_lexicon_tags_7 },
    { 0x3b594bd6, SUC_lexicon_tags_4 },
    { 0xa0d94bd5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd94bdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f594bdf, SUC_lexicon_tags_16 },
    { 0xaa594be0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d94be3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad94be8, SUC_lexicon_tags_32 },
    { 0x9df94be9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f94bec, SUC_lexicon_tags_14 },
    { 0xedd94bed, SUC_lexicon_tags_47 },
    { 0x7ed94bec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a394bf0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba194bfa, SUC_lexicon_tags_29 },
    { 0xca794bfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3794bfe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb94c01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44d94c03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x23b94c05, SUC_lexicon_tags_11 },
    { 0x9e794c06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4394c0c, SUC_lexicon_tags_12 },
    { 0xd7b94c0c, SUC_lexicon_tags_6 },
    { 0x51b94c0d, SUC_lexicon_tags_16 },
    { 0xe9394c0d, SUC_lexicon_tags_12 },
    { 0xe2d94c10, SUC_lexicon_tags_31 },
    { 0x46994c11, SUC_lexicon_tags_26 },
    { 0x26594c11, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd9b94c14, SUC_lexicon_tags_47 },
    { 0xe2594c15, SUC_lexicon_tags_2 },
    { 0x76594c16, SUC_lexicon_tags_12 },
    { 0x37f94c14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x32d94c19, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb5994c1b, SUC_lexicon_tags_11 },
    { 0x5f394c1b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7794c1e, SUC_lexicon_tags_172 },
    { 0x35794c1f, SUC_lexicon_tags_0 },
    { 0xfbd94c20, SUC_lexicon_tags_29 },
    { 0x3a394c20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38594c27, SUC_lexicon_tags_25 },
    { 0x54794c28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91194c2a, SUC_lexicon_tags_82 },
    { 0x18b94c2a, SUC_lexicon_tags_17 },
    { 0xd5b94c2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71794c32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x89d94c34, SUC_lexicon_tags_2 },
    { 0xad194c35, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d94c38, SUC_lexicon_tags_28 },
    { 0xcf994c39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bb94c3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84794c42, SUC_lexicon_tags_25 },
    { 0x84d94c42, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x53194c46, SUC_lexicon_tags_18 },
    { 0x18d94c47, SUC_lexicon_tags_28 },
    { 0x69994c48, SUC_lexicon_tags_70 },
    { 0x7d794c49, SUC_lexicon_tags_7 },
    { 0xb2594c4a, SUC_lexicon_tags_28 },
    { 0xbeb94c47, SUC_lexicon_tags_17 },
    { 0x69b94c4c, SUC_lexicon_tags_11 },
    { 0xdd794c4a, SUC_lexicon_tags_15 },
    { 0x44d94c4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac994c52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e194c54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d94c5a, SUC_lexicon_tags_1 },
    { 0xab994c5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd594c5f, SUC_lexicon_tags_47 },
    { 0x83194c60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3994c64, SUC_lexicon_tags_12 },
    { 0x64594c65, SUC_lexicon_tags_15 },
    { 0xc4f94c66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92194c68, SUC_lexicon_tags_10 },
    { 0xe7394c68, SUC_lexicon_tags_37 },
    { 0x67f94c6a, SUC_lexicon_tags_14 },
    { 0x5ab94c6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b94c6e, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18394c75, SUC_lexicon_tags_11 },
    { 0x3ed94c76, SUC_lexicon_tags_77 },
    { 0xd7394c77, SUC_lexicon_tags_17 },
    { 0x3d594c75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0794c7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x40594c7d, SUC_lexicon_tags_4 },
    { 0xf2194c7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a594c84, SUC_lexicon_tags_37 },
    { 0x47f94c84, SUC_lexicon_tags_16 },
    { 0x1a994c86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f94c89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6394c8b, SUC_lexicon_tags_0 },
    { 0xe1394c8c, SUC_lexicon_tags_1 },
    { 0x1f994c8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73994c8f, SUC_lexicon_tags_27 },
    { 0x88b94c90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16b94c92, SUC_lexicon_tags_17 },
    { 0xcd794c92, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d394c97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea394c99, SUC_lexicon_tags_37 },
    { 0x96794c9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44594c9e, SUC_lexicon_tags_204 },
    { 0xe1794c9f, SUC_lexicon_tags_12 },
    { 0x55394ca0, SUC_lexicon_tags_66 },
    { 0x7c994ca1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f94ca7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb94caa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8b94cac, SUC_lexicon_tags_21 },
    { 0x32d94cac, SUC_lexicon_tags_38 },
    { 0xf3b94cad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd794cb2, SUC_lexicon_tags_11 },
    { 0x82f94cb3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4994cb6, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f94cc0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb94cc4, SUC_lexicon_tags_3 },
    { 0x99f94cc5, SUC_lexicon_tags_7 },
    { 0x62d94cc4, SUC_lexicon_tags_65 },
    { 0x7594cc7, SUC_lexicon_tags_27 },
    { 0x32794cc4, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90994ccd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9794cd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf794cd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81394cd4, SUC_lexicon_tags_0 },
    { 0x8bb94cd4, SUC_lexicon_tags_11 },
    { 0xf994cd5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14194cdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ab94cde, SUC_lexicon_tags_10 },
    { 0x58594cdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46594ce1, SUC_lexicon_tags_55 },
    { 0xf9b94ce2, SUC_lexicon_tags_37 },
    { 0x1d94ce3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x10594ce6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d194cea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d94ced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35594cf2, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd94cf9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec594cfd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f94d02, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc194d06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b94d09, SUC_lexicon_tags_1 },
    { 0x65394d0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1994d0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x76994d0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5ad94d11, SUC_lexicon_tags_81 },
    { 0x1ed94d12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41994d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed94d1a, SUC_lexicon_tags_26 },
    { 0xa0b94d1b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b94d23, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x90d94d25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5bf94d27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9994d2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69794d2f, SUC_lexicon_tags_10 },
    { 0x4af94d30, SUC_lexicon_tags_39 },
    { 0x3f994d31, SUC_lexicon_tags_8 },
    { 0xcad94d32, SUC_lexicon_tags_30 },
    { 0xf9994d32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc994d35, SUC_lexicon_tags_1 },
    { 0xeff94d35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b394d3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79794d41, SUC_lexicon_tags_63 },
    { 0x5d394d41, SUC_lexicon_tags_1503 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0194d46, SUC_lexicon_tags_7 },
    { 0xcd194d46, SUC_lexicon_tags_12 },
    { 0xc9994d48, SUC_lexicon_tags_63 },
    { 0xc194d46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b94d4e, SUC_lexicon_tags_25 },
    { 0x8c994d4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37d94d51, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x81794d53, SUC_lexicon_tags_205 },
    { 0, NULL },
    { 0x61394d55, SUC_lexicon_tags_37 },
    { 0xf6994d55, SUC_lexicon_tags_12 },
    { 0xe3d94d57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9594d59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3594d5c, SUC_lexicon_tags_6 },
    { 0x1994d5d, SUC_lexicon_tags_37 },
    { 0x9e194d5d, SUC_lexicon_tags_11 },
    { 0xf2994d5f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5794d62, SUC_lexicon_tags_18 },
    { 0xad594d63, SUC_lexicon_tags_11 },
    { 0x48594d62, SUC_lexicon_tags_17 },
    { 0x52794d64, SUC_lexicon_tags_1 },
    { 0x1af94d66, SUC_lexicon_tags_68 },
    { 0xa3194d64, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xca594d69, SUC_lexicon_tags_37 },
    { 0xc8b94d6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x23d94d6c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc2394d6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8394d73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f94d77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c994d7d, SUC_lexicon_tags_6 },
    { 0xe4f94d7e, SUC_lexicon_tags_12 },
    { 0x84994d7d, SUC_lexicon_tags_56 },
    { 0xc9194d7f, SUC_lexicon_tags_8 },
    { 0x2a394d7d, SUC_lexicon_tags_26 },
    { 0x34b94d82, SUC_lexicon_tags_32 },
    { 0x1cd94d83, SUC_lexicon_tags_60 },
    { 0x90f94d7d, SUC_lexicon_tags_11 },
    { 0x97594d85, SUC_lexicon_tags_29 },
    { 0xe7194d85, SUC_lexicon_tags_17 },
    { 0x9db94d7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x90d94d89, SUC_lexicon_tags_14 },
    { 0xe8394d89, SUC_lexicon_tags_1 },
    { 0x2d94d8b, SUC_lexicon_tags_14 },
    { 0xa4594d8c, SUC_lexicon_tags_17 },
    { 0xfd794d8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdaf94d8f, SUC_lexicon_tags_1 },
    { 0x27b94d90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28394d93, SUC_lexicon_tags_25 },
    { 0x94f94d94, SUC_lexicon_tags_27 },
    { 0x5ef94d95, SUC_lexicon_tags_1 },
    { 0x7a594d96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95594d98, SUC_lexicon_tags_12 },
    { 0x3e394d99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df94d9d, SUC_lexicon_tags_26 },
    { 0x71994d9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc994da1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ef94da3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c394da7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda594dab, SUC_lexicon_tags_15 },
    { 0xb7d94dab, SUC_lexicon_tags_91 },
    { 0xbd794dad, SUC_lexicon_tags_0 },
    { 0x72d94dae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f94db8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x8994dbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda994dc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x89194dc4, SUC_lexicon_tags_63 },
    { 0xf0d94dc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f994dc8, SUC_lexicon_tags_1 },
    { 0x62f94dc9, SUC_lexicon_tags_11 },
    { 0xd9d94dc9, SUC_lexicon_tags_19 },
    { 0x28394dc8, SUC_lexicon_tags_49 },
    { 0xf7594dcc, SUC_lexicon_tags_17 },
    { 0x9fb94dcd, SUC_lexicon_tags_43 },
    { 0xc9194dcb, SUC_lexicon_tags_26 },
    { 0x52d94dcf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x80f94dd1, SUC_lexicon_tags_15 },
    { 0x6e194dd2, SUC_lexicon_tags_11 },
    { 0x9eb94dd3, SUC_lexicon_tags_37 },
    { 0x14194dd4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1194dd9, SUC_lexicon_tags_38 },
    { 0xbb394dda, SUC_lexicon_tags_6 },
    { 0x38994ddb, SUC_lexicon_tags_12 },
    { 0x48594ddc, SUC_lexicon_tags_25 },
    { 0xf7594ddc, SUC_lexicon_tags_2 },
    { 0x7eb94ddb, SUC_lexicon_tags_15 },
    { 0xe5394ddf, SUC_lexicon_tags_0 },
    { 0x34994de0, SUC_lexicon_tags_40 },
    { 0xe0d94dd9, SUC_lexicon_tags_56 },
    { 0xecf94de2, SUC_lexicon_tags_1 },
    { 0x61194de3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50994deb, SUC_lexicon_tags_27 },
    { 0xac994dec, SUC_lexicon_tags_17 },
    { 0x44794dec, SUC_lexicon_tags_60 },
    { 0xed594dec, SUC_lexicon_tags_37 },
    { 0x84594def, SUC_lexicon_tags_11 },
    { 0x6f994dec, SUC_lexicon_tags_55 },
    { 0x61b94df1, SUC_lexicon_tags_25 },
    { 0x52b94df2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4d194df4, SUC_lexicon_tags_62 },
    { 0xdef94df5, SUC_lexicon_tags_31 },
    { 0x12394df5, SUC_lexicon_tags_14 },
    { 0x91794df7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf194df9, SUC_lexicon_tags_65 },
    { 0x70d94dfa, SUC_lexicon_tags_43 },
    { 0xffd94dfb, SUC_lexicon_tags_25 },
    { 0x39394dfb, SUC_lexicon_tags_37 },
    { 0x62994dfb, SUC_lexicon_tags_26 },
    { 0xefd94df9, SUC_lexicon_tags_37 },
    { 0x35f94dff, SUC_lexicon_tags_39 },
    { 0x61994e00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d94e04, SUC_lexicon_tags_25 },
    { 0xf794e05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ad94e07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d94e0e, SUC_lexicon_tags_12 },
    { 0x35994e0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d394e11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd194e1b, SUC_lexicon_tags_49 },
    { 0x18d94e1b, SUC_lexicon_tags_111 },
    { 0x19394e1b, SUC_lexicon_tags_12 },
    { 0x16594e1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc6194e20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x80594e22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e594e24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a194e28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d94e2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2eb94e30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d94e33, SUC_lexicon_tags_43 },
    { 0xd5d94e33, SUC_lexicon_tags_17 },
    { 0xbd94e33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda594e39, SUC_lexicon_tags_33 },
    { 0x98194e39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x90f94e3d, SUC_lexicon_tags_11 },
    { 0x16194e3e, SUC_lexicon_tags_43 },
    { 0xdbf94e3e, SUC_lexicon_tags_7 },
    { 0x7f194e40, SUC_lexicon_tags_7 },
    { 0x60f94e41, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x17394e44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2394e49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6ff94e4b, SUC_lexicon_tags_14 },
    { 0xaff94e4b, SUC_lexicon_tags_14 },
    { 0x99594e4d, SUC_lexicon_tags_43 },
    { 0x56d94e4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e394e56, SUC_lexicon_tags_16 },
    { 0xd194e57, SUC_lexicon_tags_2 },
    { 0x60794e56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68794e61, SUC_lexicon_tags_25 },
    { 0xced94e61, SUC_lexicon_tags_48 },
    { 0x98994e63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5af94e65, SUC_lexicon_tags_3 },
    { 0x7b94e66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b94e6e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc794e70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22994e72, SUC_lexicon_tags_27 },
    { 0xa3194e73, SUC_lexicon_tags_32 },
    { 0xa5194e73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29394e7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e994e80, SUC_lexicon_tags_8 },
    { 0xced94e81, SUC_lexicon_tags_57 },
    { 0x7db94e82, SUC_lexicon_tags_23 },
    { 0x1db94e82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42f94e86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc194e8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb94e95, SUC_lexicon_tags_25 },
    { 0x67d94e96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42d94e9a, SUC_lexicon_tags_14 },
    { 0xacb94e9b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xae794e9d, SUC_lexicon_tags_25 },
    { 0x80794e9d, SUC_lexicon_tags_0 },
    { 0x70d94e9d, SUC_lexicon_tags_14 },
    { 0x13f94ea0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f94ea4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b94ea9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x6df94eab, SUC_lexicon_tags_19 },
    { 0x58b94eab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x64794eae, SUC_lexicon_tags_15 },
    { 0xb9394eae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df94eb3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f994eb8, SUC_lexicon_tags_6 },
    { 0xe0194eb9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4194ebd, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12394ec9, SUC_lexicon_tags_7 },
    { 0x4ab94eca, SUC_lexicon_tags_7 },
    { 0x22d94ec9, SUC_lexicon_tags_15 },
    { 0xeb794ecb, SUC_lexicon_tags_15 },
    { 0x35f94ecc, SUC_lexicon_tags_39 },
    { 0xc2394ece, SUC_lexicon_tags_49 },
    { 0x9194ece, SUC_lexicon_tags_32 },
    { 0xe0794ed0, SUC_lexicon_tags_11 },
    { 0x70d94ed1, SUC_lexicon_tags_12 },
    { 0x2b594ecb, SUC_lexicon_tags_15 },
    { 0x93f94ed0, SUC_lexicon_tags_7 },
    { 0xb1d94ec9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffd94ed6, SUC_lexicon_tags_23 },
    { 0x1a194ed7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1594ed9, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf94edc, SUC_lexicon_tags_12 },
    { 0x78994edd, SUC_lexicon_tags_65 },
    { 0xb9d94ede, SUC_lexicon_tags_2 },
    { 0x3e594edf, SUC_lexicon_tags_14 },
    { 0x6fd94ee0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38794ee2, SUC_lexicon_tags_27 },
    { 0xe2194ee3, SUC_lexicon_tags_16 },
    { 0x15194ee3, SUC_lexicon_tags_1 },
    { 0xab394ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11394ee8, SUC_lexicon_tags_6 },
    { 0x27394ee9, SUC_lexicon_tags_27 },
    { 0xba794eea, SUC_lexicon_tags_61 },
    { 0x80194eea, SUC_lexicon_tags_29 },
    { 0x49594eec, SUC_lexicon_tags_38 },
    { 0x30794ee8, SUC_lexicon_tags_17 },
    { 0x3af94eee, SUC_lexicon_tags_15 },
    { 0xff594eed, SUC_lexicon_tags_27 },
    { 0x65994ef0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d794ef5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77394ef9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38b94efd, SUC_lexicon_tags_39 },
    { 0xdeb94efe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff94f02, SUC_lexicon_tags_0 },
    { 0x4c194f03, SUC_lexicon_tags_27 },
    { 0xd9594f04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b94f08, SUC_lexicon_tags_12 },
    { 0x9e994f08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6dd94f0b, SUC_lexicon_tags_11 },
    { 0x52394f0b, SUC_lexicon_tags_9 },
    { 0x76d94f0c, SUC_lexicon_tags_23 },
    { 0xc3594f0e, SUC_lexicon_tags_38 },
    { 0x4f994f0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ff94f11, SUC_lexicon_tags_33 },
    { 0x48394f12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5194f19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa594f1d, SUC_lexicon_tags_12 },
    { 0x95394f1e, SUC_lexicon_tags_11 },
    { 0x4c994f1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x65d94f21, SUC_lexicon_tags_14 },
    { 0x9c394f21, SUC_lexicon_tags_17 },
    { 0x62594f21, SUC_lexicon_tags_57 },
    { 0x50594f24, SUC_lexicon_tags_71 },
    { 0xf0794f25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f94f2a, SUC_lexicon_tags_14 },
    { 0xe5d94f2a, SUC_lexicon_tags_6 },
    { 0xb3d94f2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b94f30, SUC_lexicon_tags_2 },
    { 0x9a794f31, SUC_lexicon_tags_55 },
    { 0xc2b94f32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32d94f34, SUC_lexicon_tags_2 },
    { 0x7a794f35, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b594f39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29394f3b, SUC_lexicon_tags_11 },
    { 0x9ef94f3b, SUC_lexicon_tags_15 },
    { 0x76b94f3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb594f41, SUC_lexicon_tags_1 },
    { 0xa0b94f41, SUC_lexicon_tags_11 },
    { 0x18194f43, SUC_lexicon_tags_37 },
    { 0x21594f44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46594f48, SUC_lexicon_tags_1 },
    { 0x8f94f49, SUC_lexicon_tags_7 },
    { 0x5c594f49, SUC_lexicon_tags_37 },
    { 0xcf594f4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cd94f4d, SUC_lexicon_tags_12 },
    { 0x3bf94f4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52594f50, SUC_lexicon_tags_47 },
    { 0x5f194f51, SUC_lexicon_tags_30 },
    { 0xe4994f51, SUC_lexicon_tags_15 },
    { 0x4b994f53, SUC_lexicon_tags_26 },
    { 0xf9194f53, SUC_lexicon_tags_299 },
    { 0xbff94f54, SUC_lexicon_tags_16 },
    { 0x58d94f54, SUC_lexicon_tags_25 },
    { 0x2ab94f53, SUC_lexicon_tags_60 },
    { 0xafb94f55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97d94f5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa794f60, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x57794f63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9d94f65, SUC_lexicon_tags_25 },
    { 0x8e394f66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a394f6c, SUC_lexicon_tags_25 },
    { 0xa5b94f6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56994f70, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4f94f79, SUC_lexicon_tags_30 },
    { 0x4bb94f7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd394f7d, SUC_lexicon_tags_34 },
    { 0x41d94f7d, SUC_lexicon_tags_7 },
    { 0xe3194f7e, SUC_lexicon_tags_19 },
    { 0xe6d94f80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd94f86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x30594f89, SUC_lexicon_tags_24 },
    { 0x6f194f8a, SUC_lexicon_tags_37 },
    { 0xa3194f8b, SUC_lexicon_tags_173 },
    { 0x5f394f8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f94f90, SUC_lexicon_tags_38 },
    { 0x8d194f90, SUC_lexicon_tags_17 },
    { 0xf794f92, SUC_lexicon_tags_0 },
    { 0xcab94f92, SUC_lexicon_tags_43 },
    { 0x81f94f90, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x5d94f96, SUC_lexicon_tags_12 },
    { 0x1cb94f96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd3994f99, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb94f9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69594f9e, SUC_lexicon_tags_15 },
    { 0x9f94f9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e194fa5, SUC_lexicon_tags_17 },
    { 0x39f94fa5, SUC_lexicon_tags_0 },
    { 0xdc394fa7, SUC_lexicon_tags_48 },
    { 0x77194fa8, SUC_lexicon_tags_28 },
    { 0x9994fa5, SUC_lexicon_tags_17 },
    { 0x20b94faa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d194fae, SUC_lexicon_tags_17 },
    { 0xfbb94faf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd994fb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d94fb5, SUC_lexicon_tags_21 },
    { 0xf3b94fb5, SUC_lexicon_tags_8 },
    { 0x9794fb7, SUC_lexicon_tags_31 },
    { 0x21f94fb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2594fbc, SUC_lexicon_tags_1 },
    { 0xb4d94fbc, SUC_lexicon_tags_33 },
    { 0xdf794fbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6f94fc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21794fc6, SUC_lexicon_tags_45 },
    { 0x1f394fc7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf94fcc, SUC_lexicon_tags_15 },
    { 0x52d94fcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd94fd0, SUC_lexicon_tags_29 },
    { 0xad594fd1, SUC_lexicon_tags_47 },
    { 0x3f394fd0, SUC_lexicon_tags_25 },
    { 0x1ab94fd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0194fd8, SUC_lexicon_tags_43 },
    { 0xb9394fd9, SUC_lexicon_tags_4 },
    { 0x4c194fd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97594fdc, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x18194fdf, SUC_lexicon_tags_25 },
    { 0x36994fe0, SUC_lexicon_tags_11 },
    { 0x18f94fe1, SUC_lexicon_tags_26 },
    { 0xf9394fe1, SUC_lexicon_tags_2 },
    { 0x1d394fdf, SUC_lexicon_tags_43 },
    { 0x6f94fe2, SUC_lexicon_tags_27 },
    { 0xbf594fe0, SUC_lexicon_tags_118 },
    { 0x57d94fe6, SUC_lexicon_tags_37 },
    { 0x7f94fe3, SUC_lexicon_tags_47 },
    { 0x61b94fe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64594fed, SUC_lexicon_tags_2 },
    { 0x1ed94fed, SUC_lexicon_tags_18 },
    { 0xb2394fef, SUC_lexicon_tags_1 },
    { 0x54d94fef, SUC_lexicon_tags_37 },
    { 0xf7f94ff0, SUC_lexicon_tags_11 },
    { 0x15f94fed, SUC_lexicon_tags_23 },
    { 0xfbf94ff0, SUC_lexicon_tags_656 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed94ff8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f94ffc, SUC_lexicon_tags_87 },
    { 0xc4594ffd, SUC_lexicon_tags_1 },
    { 0xee594ffd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d395002, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4395004, SUC_lexicon_tags_29 },
    { 0x97b95004, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7195008, SUC_lexicon_tags_11 },
    { 0x94d95008, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f19500f, SUC_lexicon_tags_148 },
    { 0x7b195010, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53995014, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x32195017, SUC_lexicon_tags_31 },
    { 0xedf95018, SUC_lexicon_tags_0 },
    { 0xc9f95019, SUC_lexicon_tags_14 },
    { 0xd6195019, SUC_lexicon_tags_30 },
    { 0xc319501b, SUC_lexicon_tags_47 },
    { 0xb5b9501c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f9501f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab395023, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e395025, SUC_lexicon_tags_11 },
    { 0xa6195025, SUC_lexicon_tags_12 },
    { 0x33d95027, SUC_lexicon_tags_37 },
    { 0xa995028, SUC_lexicon_tags_17 },
    { 0x10d95025, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf19502c, SUC_lexicon_tags_63 },
    { 0x8839502d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7319502f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62b95037, SUC_lexicon_tags_15 },
    { 0xcb795037, SUC_lexicon_tags_48 },
    { 0x4c195039, SUC_lexicon_tags_38 },
    { 0x7a995039, SUC_lexicon_tags_2 },
    { 0xdb79503b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2719503e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac395040, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d9504c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xecd9504e, SUC_lexicon_tags_11 },
    { 0x95b9504e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe8f95051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9579505a, SUC_lexicon_tags_25 },
    { 0xa179505b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2c79505d, SUC_lexicon_tags_61 },
    { 0x20d9505e, SUC_lexicon_tags_7 },
    { 0xdd19505f, SUC_lexicon_tags_7 },
    { 0x6b195060, SUC_lexicon_tags_136 },
    { 0x85f95061, SUC_lexicon_tags_29 },
    { 0x2cf95061, SUC_lexicon_tags_87 },
    { 0x60d95060, SUC_lexicon_tags_12 },
    { 0x25995064, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf9506b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b9506f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595073, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab95079, SUC_lexicon_tags_63 },
    { 0x35b95079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8219507c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16395081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10195084, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9395088, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x36d9508a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f39508e, SUC_lexicon_tags_17 },
    { 0x4ff9508f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c395092, SUC_lexicon_tags_1 },
    { 0x67795093, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb9509a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe119509c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb059509f, SUC_lexicon_tags_2 },
    { 0x2f19509f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9950a2, SUC_lexicon_tags_2 },
    { 0xe23950a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f9950a5, SUC_lexicon_tags_22 },
    { 0x7a3950a5, SUC_lexicon_tags_10 },
    { 0x1bd950a7, SUC_lexicon_tags_11 },
    { 0x225950a8, SUC_lexicon_tags_15 },
    { 0x59b950a9, SUC_lexicon_tags_3 },
    { 0x10d950a8, SUC_lexicon_tags_2 },
    { 0x413950aa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9950b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeb950b6, SUC_lexicon_tags_17 },
    { 0x9b1950b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b950ba, SUC_lexicon_tags_27 },
    { 0xfcd950bb, SUC_lexicon_tags_17 },
    { 0x429950bc, SUC_lexicon_tags_32 },
    { 0x76d950bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f950c0, SUC_lexicon_tags_37 },
    { 0xb51950c1, SUC_lexicon_tags_25 },
    { 0xfb3950c2, SUC_lexicon_tags_11 },
    { 0xcf5950c3, SUC_lexicon_tags_0 },
    { 0x4f5950c3, SUC_lexicon_tags_9 },
    { 0xdff950c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf950c8, SUC_lexicon_tags_47 },
    { 0x539950c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b950cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef950d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3950d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55b950db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b950e1, SUC_lexicon_tags_4 },
    { 0x5a3950e1, SUC_lexicon_tags_33 },
    { 0xec3950e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9950e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x135950eb, SUC_lexicon_tags_37 },
    { 0x1a5950ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ed950ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57950f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc9f950f9, SUC_lexicon_tags_65 },
    { 0x521950f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xed7950fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f95101, SUC_lexicon_tags_7 },
    { 0xd395102, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31995104, SUC_lexicon_tags_38 },
    { 0x53995105, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8499510b, SUC_lexicon_tags_11 },
    { 0xf199510c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8419510f, SUC_lexicon_tags_37 },
    { 0xcc19510f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52195113, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef95116, SUC_lexicon_tags_47 },
    { 0x73f95116, SUC_lexicon_tags_46 },
    { 0xef395116, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9019511c, SUC_lexicon_tags_27 },
    { 0x1639511d, SUC_lexicon_tags_7 },
    { 0xdaf9511d, SUC_lexicon_tags_7 },
    { 0xdd59511f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd95125, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb9512a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f79512c, SUC_lexicon_tags_27 },
    { 0x1f59512d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf395131, SUC_lexicon_tags_34 },
    { 0x37d95132, SUC_lexicon_tags_62 },
    { 0x93195133, SUC_lexicon_tags_27 },
    { 0x44f95134, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2195136, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf595138, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f9513e, SUC_lexicon_tags_17 },
    { 0xa359513f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc795145, SUC_lexicon_tags_34 },
    { 0x79795145, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d95149, SUC_lexicon_tags_12 },
    { 0x5339514a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ab95152, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x82595154, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec19515a, SUC_lexicon_tags_13 },
    { 0x98b9515a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2779515d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd95161, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89795166, SUC_lexicon_tags_37 },
    { 0x36d95167, SUC_lexicon_tags_0 },
    { 0xd5595166, SUC_lexicon_tags_46 },
    { 0xed595169, SUC_lexicon_tags_43 },
    { 0x71795169, SUC_lexicon_tags_2 },
    { 0xacf9516b, SUC_lexicon_tags_6 },
    { 0x42f9516c, SUC_lexicon_tags_16 },
    { 0x7b59516c, SUC_lexicon_tags_8 },
    { 0xbf59516c, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b95173, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52795175, SUC_lexicon_tags_38 },
    { 0xe9b95176, SUC_lexicon_tags_27 },
    { 0xc7195176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcad9517c, SUC_lexicon_tags_38 },
    { 0x4a99517c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d95188, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b9518f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d95195, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb759519a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633951a5, SUC_lexicon_tags_29 },
    { 0xdc9951a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6bb951a8, SUC_lexicon_tags_37 },
    { 0x2f5951a9, SUC_lexicon_tags_2 },
    { 0xdad951a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d951ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x159951af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x609951b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb951b5, SUC_lexicon_tags_17 },
    { 0x3f1951b6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xb59951b8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8ef951ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1df951be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcff951c0, SUC_lexicon_tags_60 },
    { 0xdd951c1, SUC_lexicon_tags_10 },
    { 0x393951c2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2ed951c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf91951c6, SUC_lexicon_tags_12 },
    { 0x923951c7, SUC_lexicon_tags_12 },
    { 0xedf951c8, SUC_lexicon_tags_29 },
    { 0x29f951c8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb951cc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x115951ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1951d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x64b951d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e3951d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5bb951d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f951dc, SUC_lexicon_tags_49 },
    { 0x1fd951dd, SUC_lexicon_tags_17 },
    { 0x54d951de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f1951e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b951e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891951e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97951ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691951ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x659951f1, SUC_lexicon_tags_1 },
    { 0x7c7951f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7951fb, SUC_lexicon_tags_23 },
    { 0xacf951fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa37951fe, SUC_lexicon_tags_114 },
    { 0xb71951fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39520a, SUC_lexicon_tags_12 },
    { 0x9c59520b, SUC_lexicon_tags_43 },
    { 0xc699520a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7079520e, SUC_lexicon_tags_23 },
    { 0x1d39520f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10195214, SUC_lexicon_tags_23 },
    { 0xcc195215, SUC_lexicon_tags_0 },
    { 0x12395216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7795218, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc39521b, SUC_lexicon_tags_33 },
    { 0x57f9521c, SUC_lexicon_tags_2 },
    { 0xfa19521c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73b9521f, SUC_lexicon_tags_37 },
    { 0x6f99521f, SUC_lexicon_tags_11 },
    { 0x55195221, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3d95223, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb95226, SUC_lexicon_tags_25 },
    { 0xa8b95227, SUC_lexicon_tags_17 },
    { 0x41195227, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6779522c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f95234, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9f95236, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa599523c, SUC_lexicon_tags_62 },
    { 0xdf59523d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb95245, SUC_lexicon_tags_43 },
    { 0x35195246, SUC_lexicon_tags_2 },
    { 0x85795247, SUC_lexicon_tags_26 },
    { 0xe5195248, SUC_lexicon_tags_43 },
    { 0xd9995247, SUC_lexicon_tags_48 },
    { 0x6179524a, SUC_lexicon_tags_30 },
    { 0x5959524b, SUC_lexicon_tags_37 },
    { 0x1199524c, SUC_lexicon_tags_47 },
    { 0x92b9524d, SUC_lexicon_tags_15 },
    { 0x4b39524e, SUC_lexicon_tags_1 },
    { 0x95b9524e, SUC_lexicon_tags_243 },
    { 0x6fd95246, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3395253, SUC_lexicon_tags_45 },
    { 0x9d95253, SUC_lexicon_tags_25 },
    { 0xe1f95255, SUC_lexicon_tags_43 },
    { 0x93d95256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19525c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x62d9525e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8bb95260, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9595264, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c395266, SUC_lexicon_tags_31 },
    { 0x5f195267, SUC_lexicon_tags_45 },
    { 0x22f95268, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22f9526a, SUC_lexicon_tags_49 },
    { 0xc579526b, SUC_lexicon_tags_12 },
    { 0x1399526b, SUC_lexicon_tags_2 },
    { 0x9099526d, SUC_lexicon_tags_31 },
    { 0x72d9526c, SUC_lexicon_tags_38 },
    { 0x7619526f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f395273, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9995279, SUC_lexicon_tags_1 },
    { 0x2eb95279, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5179527f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17195282, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x38595284, SUC_lexicon_tags_49 },
    { 0xa9f95285, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20195287, SUC_lexicon_tags_63 },
    { 0xf0d95288, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca39528a, SUC_lexicon_tags_26 },
    { 0x1e39528b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf019528e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaab95292, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b795297, SUC_lexicon_tags_9 },
    { 0x89d95298, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c39529e, SUC_lexicon_tags_25 },
    { 0xc459529f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41952a3, SUC_lexicon_tags_8 },
    { 0xc25952a4, SUC_lexicon_tags_25 },
    { 0x4b1952a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917952aa, SUC_lexicon_tags_27 },
    { 0x573952ab, SUC_lexicon_tags_44 },
    { 0x45d952ac, SUC_lexicon_tags_17 },
    { 0xb39952ad, SUC_lexicon_tags_1 },
    { 0x857952ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37952b1, SUC_lexicon_tags_15 },
    { 0x1e7952b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5952b5, SUC_lexicon_tags_1 },
    { 0x7cb952b6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91952ba, SUC_lexicon_tags_17 },
    { 0x173952bb, SUC_lexicon_tags_17 },
    { 0xa43952bc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f952bf, SUC_lexicon_tags_1722 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d952c6, SUC_lexicon_tags_0 },
    { 0xe85952c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb03952c9, SUC_lexicon_tags_63 },
    { 0xcb7952ca, SUC_lexicon_tags_10 },
    { 0xf2d952cb, SUC_lexicon_tags_30 },
    { 0x1fb952cc, SUC_lexicon_tags_17 },
    { 0x6df952ca, SUC_lexicon_tags_15 },
    { 0x40d952ce, SUC_lexicon_tags_65 },
    { 0x227952cf, SUC_lexicon_tags_6 },
    { 0x481952cf, SUC_lexicon_tags_60 },
    { 0xd47952cf, SUC_lexicon_tags_26 },
    { 0xe1952cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d952d8, SUC_lexicon_tags_85 },
    { 0x229952d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1952dc, SUC_lexicon_tags_2 },
    { 0xc4f952dd, SUC_lexicon_tags_10 },
    { 0xa33952de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1e7952e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f952e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7952ef, SUC_lexicon_tags_44 },
    { 0x719952f0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1952f7, SUC_lexicon_tags_2 },
    { 0x127952f7, SUC_lexicon_tags_25 },
    { 0x8c1952f9, SUC_lexicon_tags_0 },
    { 0x2bd952f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67952fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x99395300, SUC_lexicon_tags_2 },
    { 0x1a195301, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b195303, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe799530a, SUC_lexicon_tags_25 },
    { 0x48d9530a, SUC_lexicon_tags_48 },
    { 0x4d9530c, SUC_lexicon_tags_61 },
    { 0x5ad9530c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4b59530f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x60395311, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2b95313, SUC_lexicon_tags_1 },
    { 0x5a795314, SUC_lexicon_tags_2 },
    { 0x16795314, SUC_lexicon_tags_11 },
    { 0xa0d95316, SUC_lexicon_tags_11 },
    { 0x8b595317, SUC_lexicon_tags_7 },
    { 0xf7d95315, SUC_lexicon_tags_351 },
    { 0x7d95319, SUC_lexicon_tags_23 },
    { 0xf1f95314, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4159531f, SUC_lexicon_tags_15 },
    { 0xd5595320, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e795324, SUC_lexicon_tags_63 },
    { 0x2d195325, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98595329, SUC_lexicon_tags_68 },
    { 0x4679532a, SUC_lexicon_tags_12 },
    { 0xda99532b, SUC_lexicon_tags_184 },
    { 0x7079532c, SUC_lexicon_tags_26 },
    { 0x9d59532c, SUC_lexicon_tags_2 },
    { 0x1699532e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1195330, SUC_lexicon_tags_34 },
    { 0x1cd95331, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c995333, SUC_lexicon_tags_7 },
    { 0x4f395334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78595339, SUC_lexicon_tags_1 },
    { 0x1a395339, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcad9533c, SUC_lexicon_tags_16 },
    { 0xe0f9533d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b95340, SUC_lexicon_tags_25 },
    { 0x31195341, SUC_lexicon_tags_31 },
    { 0x27195342, SUC_lexicon_tags_12 },
    { 0x1e395341, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7c395345, SUC_lexicon_tags_12 },
    { 0xa8995346, SUC_lexicon_tags_34 },
    { 0x1a795347, SUC_lexicon_tags_47 },
    { 0x1f95347, SUC_lexicon_tags_12 },
    { 0xacf95346, SUC_lexicon_tags_25 },
    { 0x8a99534a, SUC_lexicon_tags_15 },
    { 0x9e59534b, SUC_lexicon_tags_19 },
    { 0x66395347, SUC_lexicon_tags_37 },
    { 0x2a395346, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef95354, SUC_lexicon_tags_19 },
    { 0xf0d95355, SUC_lexicon_tags_43 },
    { 0x52395356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc079535a, SUC_lexicon_tags_15 },
    { 0xe7b9535a, SUC_lexicon_tags_47 },
    { 0xce39535b, SUC_lexicon_tags_0 },
    { 0xcb9535c, SUC_lexicon_tags_3 },
    { 0x579535e, SUC_lexicon_tags_2 },
    { 0x52d9535c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed395365, SUC_lexicon_tags_12 },
    { 0x88995366, SUC_lexicon_tags_8 },
    { 0x89795367, SUC_lexicon_tags_17 },
    { 0x6f995367, SUC_lexicon_tags_15 },
    { 0x2f95366, SUC_lexicon_tags_1 },
    { 0x2b595366, SUC_lexicon_tags_7 },
    { 0x8119536b, SUC_lexicon_tags_4 },
    { 0x7d79536b, SUC_lexicon_tags_1 },
    { 0x6fb95367, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91595372, SUC_lexicon_tags_14 },
    { 0xe8d95372, SUC_lexicon_tags_7 },
    { 0x88795374, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf95377, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbf95379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b95380, SUC_lexicon_tags_64 },
    { 0x3a195381, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab95385, SUC_lexicon_tags_7 },
    { 0x8ab95386, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1139538b, SUC_lexicon_tags_9 },
    { 0xff99538c, SUC_lexicon_tags_66 },
    { 0x2459538c, SUC_lexicon_tags_27 },
    { 0x4d79538e, SUC_lexicon_tags_63 },
    { 0x88b9538b, SUC_lexicon_tags_26 },
    { 0x4ff9538c, SUC_lexicon_tags_79 },
    { 0xf3b95391, SUC_lexicon_tags_87 },
    { 0x17995391, SUC_lexicon_tags_65 },
    { 0x1479538e, SUC_lexicon_tags_25 },
    { 0xfd995394, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb95399, SUC_lexicon_tags_12 },
    { 0x1059539a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f953a6, SUC_lexicon_tags_1 },
    { 0x6b3953a6, SUC_lexicon_tags_29 },
    { 0xdab953a8, SUC_lexicon_tags_0 },
    { 0x70f953a7, SUC_lexicon_tags_2 },
    { 0x9e1953a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97953ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf953af, SUC_lexicon_tags_17 },
    { 0xdf5953b0, SUC_lexicon_tags_1 },
    { 0x581953b0, SUC_lexicon_tags_15 },
    { 0x7f7953b0, SUC_lexicon_tags_19 },
    { 0x531953af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7953bd, SUC_lexicon_tags_49 },
    { 0x959953be, SUC_lexicon_tags_7 },
    { 0x4b9953be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf31953c1, SUC_lexicon_tags_14 },
    { 0x69d953c1, SUC_lexicon_tags_2 },
    { 0x8af953c1, SUC_lexicon_tags_2 },
    { 0xd0d953c2, SUC_lexicon_tags_106 },
    { 0x887953c5, SUC_lexicon_tags_21 },
    { 0x661953c5, SUC_lexicon_tags_15 },
    { 0xf6d953c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1ed953c9, SUC_lexicon_tags_9 },
    { 0x939953ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x337953cc, SUC_lexicon_tags_44 },
    { 0x423953cc, SUC_lexicon_tags_12 },
    { 0xeb7953cd, SUC_lexicon_tags_77 },
    { 0x27f953ce, SUC_lexicon_tags_43 },
    { 0x9ad953cd, SUC_lexicon_tags_11 },
    { 0x23d953ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9953d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9953d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x257953dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47953e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb953e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea9953e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b953e9, SUC_lexicon_tags_37 },
    { 0x1b953ea, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x60d953ec, SUC_lexicon_tags_37 },
    { 0xa2b953ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd73953ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c1953f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf13953f3, SUC_lexicon_tags_91 },
    { 0x509953f4, SUC_lexicon_tags_38 },
    { 0x749953f5, SUC_lexicon_tags_16 },
    { 0x631953f5, SUC_lexicon_tags_26 },
    { 0x2bd953f5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15953fa, SUC_lexicon_tags_26 },
    { 0xd81953fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3953fe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb9d95400, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f995406, SUC_lexicon_tags_1 },
    { 0x81595406, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f19540c, SUC_lexicon_tags_30 },
    { 0x4259540d, SUC_lexicon_tags_12 },
    { 0x3799540d, SUC_lexicon_tags_30 },
    { 0x7cf9540f, SUC_lexicon_tags_8 },
    { 0xc0f9540d, SUC_lexicon_tags_1 },
    { 0x91f95411, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f95416, SUC_lexicon_tags_82 },
    { 0x5c795417, SUC_lexicon_tags_82 },
    { 0x25795418, SUC_lexicon_tags_87 },
    { 0x37795419, SUC_lexicon_tags_38 },
    { 0x54d9541a, SUC_lexicon_tags_18 },
    { 0x3e19541b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab195423, SUC_lexicon_tags_14 },
    { 0x6bf95424, SUC_lexicon_tags_0 },
    { 0x2b595423, SUC_lexicon_tags_19 },
    { 0xf5f95426, SUC_lexicon_tags_23 },
    { 0x1d795427, SUC_lexicon_tags_37 },
    { 0x2ad95428, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8bb9542a, SUC_lexicon_tags_1 },
    { 0x61b9542b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb95431, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x59995433, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ad95435, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa195437, SUC_lexicon_tags_25 },
    { 0xae195437, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3999543a, SUC_lexicon_tags_2 },
    { 0xdef9543b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a19543d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2195441, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1995444, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97795449, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d9544c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1f99544e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2019545b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c99545d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b95463, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1995466, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc519546d, SUC_lexicon_tags_25 },
    { 0x18b9546e, SUC_lexicon_tags_2 },
    { 0x8819546d, SUC_lexicon_tags_7 },
    { 0xafd9546e, SUC_lexicon_tags_25 },
    { 0x28d95471, SUC_lexicon_tags_87 },
    { 0xeb99546f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b95475, SUC_lexicon_tags_7 },
    { 0x24f95476, SUC_lexicon_tags_14 },
    { 0x93195475, SUC_lexicon_tags_25 },
    { 0x51395477, SUC_lexicon_tags_7 },
    { 0x1eb95479, SUC_lexicon_tags_34 },
    { 0x52595478, SUC_lexicon_tags_23 },
    { 0x2439547b, SUC_lexicon_tags_49 },
    { 0xab59547b, SUC_lexicon_tags_37 },
    { 0xdc99547c, SUC_lexicon_tags_55 },
    { 0x559547c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c195489, SUC_lexicon_tags_25 },
    { 0xf4795489, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cd9548c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19548f, SUC_lexicon_tags_15 },
    { 0x5cb95490, SUC_lexicon_tags_63 },
    { 0x595491, SUC_lexicon_tags_17 },
    { 0xf5795491, SUC_lexicon_tags_25 },
    { 0xc4995491, SUC_lexicon_tags_15 },
    { 0xa2595492, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28995498, SUC_lexicon_tags_27 },
    { 0xb5d95499, SUC_lexicon_tags_71 },
    { 0xccf9549a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef9549f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6af954a2, SUC_lexicon_tags_42 },
    { 0xf55954a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9954aa, SUC_lexicon_tags_63 },
    { 0x5db954aa, SUC_lexicon_tags_11 },
    { 0xdf9954ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231954b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a7954b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf3954b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183954be, SUC_lexicon_tags_25 },
    { 0xbbb954bf, SUC_lexicon_tags_37 },
    { 0x985954c0, SUC_lexicon_tags_37 },
    { 0x4e5954c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f954c4, SUC_lexicon_tags_45 },
    { 0x933954c5, SUC_lexicon_tags_1 },
    { 0xa11954c4, SUC_lexicon_tags_2 },
    { 0xf0b954c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x865954ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8f9954cc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355954d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d954d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1954d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x42b954da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfa7954dc, SUC_lexicon_tags_2 },
    { 0x931954dc, SUC_lexicon_tags_3 },
    { 0x83b954dc, SUC_lexicon_tags_11 },
    { 0x4b954df, SUC_lexicon_tags_29 },
    { 0xdb1954e0, SUC_lexicon_tags_26 },
    { 0x927954e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d954e4, SUC_lexicon_tags_28 },
    { 0x181954e4, SUC_lexicon_tags_27 },
    { 0x12f954e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b1954e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af954eb, SUC_lexicon_tags_0 },
    { 0x52f954eb, SUC_lexicon_tags_17 },
    { 0x437954eb, SUC_lexicon_tags_43 },
    { 0xfaf954ee, SUC_lexicon_tags_11 },
    { 0x311954ef, SUC_lexicon_tags_26 },
    { 0x817954f0, SUC_lexicon_tags_9 },
    { 0x665954f1, SUC_lexicon_tags_11 },
    { 0x601954ef, SUC_lexicon_tags_38 },
    { 0xacf954f3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d954f7, SUC_lexicon_tags_43 },
    { 0xa05954f8, SUC_lexicon_tags_17 },
    { 0x571954f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9bf954fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf954ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e595501, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7b95503, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa395506, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc995508, SUC_lexicon_tags_26 },
    { 0x21395509, SUC_lexicon_tags_136 },
    { 0x8cf9550a, SUC_lexicon_tags_7 },
    { 0xcfd9550b, SUC_lexicon_tags_15 },
    { 0xc639550c, SUC_lexicon_tags_84 },
    { 0x3e595509, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28195518, SUC_lexicon_tags_80 },
    { 0xd395518, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa59551b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0195522, SUC_lexicon_tags_29 },
    { 0xd3795522, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8a595525, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d95529, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67595532, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee995537, SUC_lexicon_tags_19 },
    { 0xcc595537, SUC_lexicon_tags_15 },
    { 0xb4d95539, SUC_lexicon_tags_47 },
    { 0x5239553a, SUC_lexicon_tags_37 },
    { 0x39d95539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b79553d, SUC_lexicon_tags_1 },
    { 0x7ef9553e, SUC_lexicon_tags_2 },
    { 0xe059553e, SUC_lexicon_tags_10 },
    { 0x8ad95540, SUC_lexicon_tags_30 },
    { 0xdc395540, SUC_lexicon_tags_1 },
    { 0x58b95540, SUC_lexicon_tags_26 },
    { 0xb2b9553f, SUC_lexicon_tags_25 },
    { 0x76595544, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d95548, SUC_lexicon_tags_11 },
    { 0xef95548, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x299554c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8679554e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d95555, SUC_lexicon_tags_15 },
    { 0x77995556, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x36995559, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f9555c, SUC_lexicon_tags_6 },
    { 0x6ed9555c, SUC_lexicon_tags_84 },
    { 0xf239555e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f95562, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78b95564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23395566, SUC_lexicon_tags_2 },
    { 0xef195567, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb139556a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd9556d, SUC_lexicon_tags_31 },
    { 0x8239556d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5595572, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d95576, SUC_lexicon_tags_30 },
    { 0x41f95576, SUC_lexicon_tags_7 },
    { 0xb7b95577, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd9557c, SUC_lexicon_tags_47 },
    { 0x8259557d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67995585, SUC_lexicon_tags_2 },
    { 0x86595585, SUC_lexicon_tags_11 },
    { 0x42b95587, SUC_lexicon_tags_136 },
    { 0x97195585, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a59558e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x44395591, SUC_lexicon_tags_15 },
    { 0x71d95592, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b95595, SUC_lexicon_tags_0 },
    { 0xb2395596, SUC_lexicon_tags_1 },
    { 0x65d95597, SUC_lexicon_tags_45 },
    { 0xa8795598, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x16f9559b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36f9559d, SUC_lexicon_tags_37 },
    { 0x1a59559e, SUC_lexicon_tags_46 },
    { 0xaa19559f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd81955a1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaf955a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73955ad, SUC_lexicon_tags_48 },
    { 0x5f1955ae, SUC_lexicon_tags_57 },
    { 0xcd1955af, SUC_lexicon_tags_31 },
    { 0xc39955b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26b955b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe8f955b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f955b7, SUC_lexicon_tags_11 },
    { 0x1a1955b7, SUC_lexicon_tags_14 },
    { 0xb83955b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b955be, SUC_lexicon_tags_25 },
    { 0xd79955be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfc7955c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1955c4, SUC_lexicon_tags_2 },
    { 0xf0f955c5, SUC_lexicon_tags_16 },
    { 0x859955c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8a1955c8, SUC_lexicon_tags_1 },
    { 0xb3f955c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23955d0, SUC_lexicon_tags_12 },
    { 0xc3955d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7955d4, SUC_lexicon_tags_25 },
    { 0x90b955d5, SUC_lexicon_tags_1 },
    { 0x5d9955d6, SUC_lexicon_tags_467 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed955dd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x129955df, SUC_lexicon_tags_14 },
    { 0x81b955df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1955e5, SUC_lexicon_tags_17 },
    { 0xb83955e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x133955e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed955ed, SUC_lexicon_tags_12 },
    { 0x89d955ee, SUC_lexicon_tags_17 },
    { 0xf77955ed, SUC_lexicon_tags_79 },
    { 0xa05955f0, SUC_lexicon_tags_19 },
    { 0x485955f0, SUC_lexicon_tags_17 },
    { 0xfcd955ee, SUC_lexicon_tags_16 },
    { 0x77955f1, SUC_lexicon_tags_47 },
    { 0x6bd955f4, SUC_lexicon_tags_0 },
    { 0x29955f5, SUC_lexicon_tags_35 },
    { 0xc1d955f6, SUC_lexicon_tags_65 },
    { 0x633955f6, SUC_lexicon_tags_17 },
    { 0x219955f2, SUC_lexicon_tags_84 },
    { 0x6ff955ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x727955fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb9955fe, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f95601, SUC_lexicon_tags_3 },
    { 0x32d95602, SUC_lexicon_tags_25 },
    { 0x8a995602, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcbf95605, SUC_lexicon_tags_11 },
    { 0xeb995606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39560c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6395614, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3bd95616, SUC_lexicon_tags_6 },
    { 0x62d95617, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c79561d, SUC_lexicon_tags_60 },
    { 0x4a59561e, SUC_lexicon_tags_8 },
    { 0xab9561e, SUC_lexicon_tags_2 },
    { 0x91595620, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2195622, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x35b95625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad9562a, SUC_lexicon_tags_31 },
    { 0xbd39562a, SUC_lexicon_tags_15 },
    { 0xa459562c, SUC_lexicon_tags_22 },
    { 0xb339562a, SUC_lexicon_tags_12 },
    { 0x9819562e, SUC_lexicon_tags_38 },
    { 0x8a19562c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25d95631, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb95637, SUC_lexicon_tags_11 },
    { 0x34595638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5059563b, SUC_lexicon_tags_17 },
    { 0x2b79563c, SUC_lexicon_tags_66 },
    { 0x7079563d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x46f9563f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2995644, SUC_lexicon_tags_11 },
    { 0xf195645, SUC_lexicon_tags_82 },
    { 0xf3195646, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd799564a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39564e, SUC_lexicon_tags_25 },
    { 0x79f9564f, SUC_lexicon_tags_25 },
    { 0x7f395650, SUC_lexicon_tags_37 },
    { 0x84395651, SUC_lexicon_tags_11 },
    { 0x6795652, SUC_lexicon_tags_15 },
    { 0xad595652, SUC_lexicon_tags_37 },
    { 0xa0b95651, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed95659, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f9565c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab95664, SUC_lexicon_tags_12 },
    { 0x23995665, SUC_lexicon_tags_12 },
    { 0x4bf95666, SUC_lexicon_tags_25 },
    { 0xd0195667, SUC_lexicon_tags_49 },
    { 0x9cd95666, SUC_lexicon_tags_25 },
    { 0xf5395669, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35d9566b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3479566f, SUC_lexicon_tags_60 },
    { 0xdf195670, SUC_lexicon_tags_11 },
    { 0x45d95671, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd395673, SUC_lexicon_tags_23 },
    { 0x40595674, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xff995677, SUC_lexicon_tags_82 },
    { 0xa1595677, SUC_lexicon_tags_43 },
    { 0xc8395679, SUC_lexicon_tags_45 },
    { 0x3ef95679, SUC_lexicon_tags_45 },
    { 0xdbb95677, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7cf9567d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9ed9567f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f595682, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b95686, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2db95688, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2479568c, SUC_lexicon_tags_12 },
    { 0xd1d9568d, SUC_lexicon_tags_25 },
    { 0x6839568e, SUC_lexicon_tags_26 },
    { 0xd519568d, SUC_lexicon_tags_25 },
    { 0xfcf95690, SUC_lexicon_tags_198 },
    { 0x3ed95690, SUC_lexicon_tags_45 },
    { 0x33795692, SUC_lexicon_tags_47 },
    { 0xa939568d, SUC_lexicon_tags_17 },
    { 0xc6995694, SUC_lexicon_tags_3 },
    { 0x59568e, SUC_lexicon_tags_14 },
    { 0x24595692, SUC_lexicon_tags_26 },
    { 0x4239568d, SUC_lexicon_tags_11 },
    { 0xbdb95696, SUC_lexicon_tags_26 },
    { 0x13595699, SUC_lexicon_tags_3 },
    { 0x20f95699, SUC_lexicon_tags_2 },
    { 0xfbb9569b, SUC_lexicon_tags_32 },
    { 0xc6b9569c, SUC_lexicon_tags_1 },
    { 0xca39569b, SUC_lexicon_tags_37 },
    { 0x259569a, SUC_lexicon_tags_1 },
    { 0x65f9569e, SUC_lexicon_tags_10 },
    { 0x5479568c, SUC_lexicon_tags_25 },
    { 0x46195691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8df956a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd95956a8, SUC_lexicon_tags_23 },
    { 0x2a1956a9, SUC_lexicon_tags_28 },
    { 0xd35956a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7956ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x369956af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83d956b1, SUC_lexicon_tags_45 },
    { 0xacb956b2, SUC_lexicon_tags_56 },
    { 0x78f956b2, SUC_lexicon_tags_25 },
    { 0x9f956b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427956ba, SUC_lexicon_tags_37 },
    { 0xf1b956ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf75956bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x62f956bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4eb956c1, SUC_lexicon_tags_1 },
    { 0x387956c1, SUC_lexicon_tags_45 },
    { 0xd8d956c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x947956c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3956ca, SUC_lexicon_tags_12 },
    { 0xd2f956cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xad5956cd, SUC_lexicon_tags_7 },
    { 0xd81956ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x367956d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf956d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665956de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x603956e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909956e5, SUC_lexicon_tags_80 },
    { 0x94d956e6, SUC_lexicon_tags_11 },
    { 0x4b9956e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5956ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd956f3, SUC_lexicon_tags_25 },
    { 0xf79956f4, SUC_lexicon_tags_7 },
    { 0x1b9956f4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x58b956f7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5956fa, SUC_lexicon_tags_14 },
    { 0x3956fb, SUC_lexicon_tags_14 },
    { 0x393956fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9956ff, SUC_lexicon_tags_37 },
    { 0x85d956ff, SUC_lexicon_tags_1 },
    { 0x1c195701, SUC_lexicon_tags_32 },
    { 0x6f595701, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bd95704, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb395707, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xea395709, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb59570b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb359570d, SUC_lexicon_tags_32 },
    { 0xe779570e, SUC_lexicon_tags_17 },
    { 0x84d9570f, SUC_lexicon_tags_37 },
    { 0xb995710, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f95714, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef95719, SUC_lexicon_tags_47 },
    { 0xcbf9571a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ad9571c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf95720, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe995722, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x99395724, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f95727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4739572a, SUC_lexicon_tags_1 },
    { 0x4339572a, SUC_lexicon_tags_1 },
    { 0x7999572c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xec59572f, SUC_lexicon_tags_19 },
    { 0x64d95730, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbf195732, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14795735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x19395738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90b9573a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9899573d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf395740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77195744, SUC_lexicon_tags_62 },
    { 0xe1995745, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f9574c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb9574e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbf95750, SUC_lexicon_tags_17 },
    { 0x14f95750, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bf95753, SUC_lexicon_tags_25 },
    { 0x84595754, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x24d95756, SUC_lexicon_tags_1 },
    { 0xa1795757, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdd795759, SUC_lexicon_tags_11 },
    { 0xb2d9575a, SUC_lexicon_tags_61 },
    { 0x9799575b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf59575e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f95763, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cf95765, SUC_lexicon_tags_30 },
    { 0x94995765, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdc395768, SUC_lexicon_tags_60 },
    { 0xff195769, SUC_lexicon_tags_31 },
    { 0x1ab9576a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2219576d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4919576f, SUC_lexicon_tags_11 },
    { 0xde195770, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf95773, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0xbff95776, SUC_lexicon_tags_17 },
    { 0x6fb95777, SUC_lexicon_tags_84 },
    { 0x18995776, SUC_lexicon_tags_27 },
    { 0x6d795776, SUC_lexicon_tags_146 },
    { 0xe9395777, SUC_lexicon_tags_49 },
    { 0x5b99577b, SUC_lexicon_tags_46 },
    { 0x5d99577c, SUC_lexicon_tags_15 },
    { 0x65d95777, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf95782, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde99578a, SUC_lexicon_tags_28 },
    { 0xa779578a, SUC_lexicon_tags_2 },
    { 0x37b9578c, SUC_lexicon_tags_9 },
    { 0xb579578d, SUC_lexicon_tags_15 },
    { 0x699578a, SUC_lexicon_tags_15 },
    { 0xac99578d, SUC_lexicon_tags_37 },
    { 0x9659578d, SUC_lexicon_tags_37 },
    { 0xaa195791, SUC_lexicon_tags_30 },
    { 0xb1d95792, SUC_lexicon_tags_25 },
    { 0x42b95791, SUC_lexicon_tags_19 },
    { 0xcab95791, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d95799, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd957a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b957a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9957a9, SUC_lexicon_tags_11 },
    { 0xe85957aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5957ae, SUC_lexicon_tags_35 },
    { 0xaab957ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ed957b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdf1957b3, SUC_lexicon_tags_15 },
    { 0xca1957b3, SUC_lexicon_tags_12 },
    { 0xc51957b5, SUC_lexicon_tags_15 },
    { 0x2e3957b5, SUC_lexicon_tags_60 },
    { 0x36b957b6, SUC_lexicon_tags_21 },
    { 0xd4d957b6, SUC_lexicon_tags_26 },
    { 0xfe7957b9, SUC_lexicon_tags_1 },
    { 0xf71957b4, SUC_lexicon_tags_25 },
    { 0x36b957b5, SUC_lexicon_tags_1 },
    { 0x2ff957bc, SUC_lexicon_tags_38 },
    { 0xa0d957b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d957c7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb957cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x447957ce, SUC_lexicon_tags_2 },
    { 0x1c9957ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf957d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5957d7, SUC_lexicon_tags_14 },
    { 0xe3957d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed957db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe03957dd, SUC_lexicon_tags_25 },
    { 0xa5b957de, SUC_lexicon_tags_11 },
    { 0x99f957dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x78f957e1, SUC_lexicon_tags_1 },
    { 0x47f957e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5957eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389957ef, SUC_lexicon_tags_7 },
    { 0x625957ef, SUC_lexicon_tags_37 },
    { 0x69d957f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9957f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761957fc, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16195800, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb299580d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff95810, SUC_lexicon_tags_46 },
    { 0x6ff95810, SUC_lexicon_tags_38 },
    { 0xcc795812, SUC_lexicon_tags_12 },
    { 0x63995812, SUC_lexicon_tags_25 },
    { 0x54795812, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc399581a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad9581d, SUC_lexicon_tags_14 },
    { 0x4959581e, SUC_lexicon_tags_1 },
    { 0x9f9581e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4795821, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83b95823, SUC_lexicon_tags_12 },
    { 0xbfb95824, SUC_lexicon_tags_11 },
    { 0xe0f95825, SUC_lexicon_tags_2 },
    { 0x30f95823, SUC_lexicon_tags_25 },
    { 0x9795827, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87595829, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b95830, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83b95832, SUC_lexicon_tags_47 },
    { 0xa7195832, SUC_lexicon_tags_63 },
    { 0x44595834, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b95839, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8b59583b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4395841, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74395844, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92f95850, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f95854, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4a195856, SUC_lexicon_tags_17 },
    { 0x30d95857, SUC_lexicon_tags_49 },
    { 0x88f95857, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14795863, SUC_lexicon_tags_37 },
    { 0x75b95864, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d795866, SUC_lexicon_tags_29 },
    { 0xce395866, SUC_lexicon_tags_10 },
    { 0xed795867, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3259586e, SUC_lexicon_tags_6 },
    { 0xbc79586f, SUC_lexicon_tags_65 },
    { 0xfab9586f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d595874, SUC_lexicon_tags_38 },
    { 0x9a795875, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5395878, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d9587b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f95885, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb9588a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5999588e, SUC_lexicon_tags_0 },
    { 0xb999588f, SUC_lexicon_tags_37 },
    { 0xc5f95890, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b395892, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32195895, SUC_lexicon_tags_6 },
    { 0x36b95895, SUC_lexicon_tags_3 },
    { 0x80195897, SUC_lexicon_tags_2 },
    { 0x65795895, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xea99589a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3119589c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9959589e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d958a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67958b0, SUC_lexicon_tags_27 },
    { 0x6e9958b1, SUC_lexicon_tags_47 },
    { 0x151958b2, SUC_lexicon_tags_26 },
    { 0x577958b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf35958b5, SUC_lexicon_tags_40 },
    { 0xfb958b6, SUC_lexicon_tags_9 },
    { 0x281958b5, SUC_lexicon_tags_23 },
    { 0x8d3958b8, SUC_lexicon_tags_27 },
    { 0x469958b5, SUC_lexicon_tags_15 },
    { 0x1b958ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f958c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67958c4, SUC_lexicon_tags_37 },
    { 0xf4d958c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413958ca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f958ce, SUC_lexicon_tags_16 },
    { 0xc67958ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d958d5, SUC_lexicon_tags_17 },
    { 0x637958d6, SUC_lexicon_tags_22 },
    { 0xc1f958d6, SUC_lexicon_tags_12 },
    { 0xcd5958d8, SUC_lexicon_tags_0 },
    { 0x74b958d9, SUC_lexicon_tags_37 },
    { 0x45b958d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1a3958dc, SUC_lexicon_tags_2 },
    { 0x7b5958dd, SUC_lexicon_tags_7 },
    { 0xf61958de, SUC_lexicon_tags_14 },
    { 0xddd958dd, SUC_lexicon_tags_14 },
    { 0xff3958dc, SUC_lexicon_tags_26 },
    { 0x8f958dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa1958e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d958ea, SUC_lexicon_tags_25 },
    { 0x883958eb, SUC_lexicon_tags_27 },
    { 0x539958ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91958f1, SUC_lexicon_tags_2 },
    { 0xdf1958f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b958fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb958fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f95900, SUC_lexicon_tags_7 },
    { 0xb9b95901, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa8995903, SUC_lexicon_tags_1 },
    { 0x5fd95904, SUC_lexicon_tags_77 },
    { 0x16b95904, SUC_lexicon_tags_22 },
    { 0x71195906, SUC_lexicon_tags_1 },
    { 0x8dd95907, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xea59590a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13395912, SUC_lexicon_tags_32 },
    { 0xed795913, SUC_lexicon_tags_1 },
    { 0xd2595914, SUC_lexicon_tags_26 },
    { 0x91995915, SUC_lexicon_tags_11 },
    { 0x7ad95916, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef9591b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19591f, SUC_lexicon_tags_37 },
    { 0x8d995920, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3db95923, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3279592b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6919592e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b195938, SUC_lexicon_tags_11 },
    { 0xbd95939, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40b9593b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9ab9593d, SUC_lexicon_tags_63 },
    { 0xf299593e, SUC_lexicon_tags_17 },
    { 0x6b19593f, SUC_lexicon_tags_7 },
    { 0x4719593e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82195942, SUC_lexicon_tags_2 },
    { 0xce195942, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33995945, SUC_lexicon_tags_37 },
    { 0xd2595946, SUC_lexicon_tags_37 },
    { 0xa4395947, SUC_lexicon_tags_66 },
    { 0xf3595948, SUC_lexicon_tags_17 },
    { 0xef995948, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf579594b, SUC_lexicon_tags_16 },
    { 0xce39594c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7219594f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2b95951, SUC_lexicon_tags_7 },
    { 0x4ff95952, SUC_lexicon_tags_15 },
    { 0xe5b95953, SUC_lexicon_tags_7 },
    { 0xaa395954, SUC_lexicon_tags_2 },
    { 0x53d95952, SUC_lexicon_tags_15 },
    { 0x8b95956, SUC_lexicon_tags_10 },
    { 0x44d95957, SUC_lexicon_tags_23 },
    { 0xb8f95955, SUC_lexicon_tags_7 },
    { 0x2bb95959, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5479595f, SUC_lexicon_tags_0 },
    { 0x23195960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d195962, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6395964, SUC_lexicon_tags_14 },
    { 0x3d95965, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5d95967, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4259596b, SUC_lexicon_tags_17 },
    { 0xbc79596c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3219596f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23595979, SUC_lexicon_tags_16 },
    { 0x2bb95979, SUC_lexicon_tags_31 },
    { 0x6159597b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff95981, SUC_lexicon_tags_0 },
    { 0x10b95982, SUC_lexicon_tags_37 },
    { 0x42995981, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde795987, SUC_lexicon_tags_11 },
    { 0x2ab95988, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b79598c, SUC_lexicon_tags_25 },
    { 0xb839598d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d795992, SUC_lexicon_tags_17 },
    { 0x57595993, SUC_lexicon_tags_15 },
    { 0x7dd95994, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x71b95996, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e395999, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2919599d, SUC_lexicon_tags_39 },
    { 0x2599599d, SUC_lexicon_tags_14 },
    { 0xc959599f, SUC_lexicon_tags_45 },
    { 0x347959a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957959a6, SUC_lexicon_tags_2 },
    { 0xa05959a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x249959aa, SUC_lexicon_tags_2 },
    { 0x601959ab, SUC_lexicon_tags_37 },
    { 0x285959ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x237959ae, SUC_lexicon_tags_70 },
    { 0x759959af, SUC_lexicon_tags_79 },
    { 0x2b1959b0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1959b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x913959b7, SUC_lexicon_tags_43 },
    { 0x3d9959b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x187959bb, SUC_lexicon_tags_12 },
    { 0xf97959bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1959c1, SUC_lexicon_tags_19 },
    { 0x119959c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1959c7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4f959cd, SUC_lexicon_tags_45 },
    { 0x56f959ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329959d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x507959d7, SUC_lexicon_tags_27 },
    { 0x769959d8, SUC_lexicon_tags_77 },
    { 0x45f959d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb959e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13959e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b959e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d959eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b959ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd3959f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa9959f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f959f7, SUC_lexicon_tags_48 },
    { 0x20d959f8, SUC_lexicon_tags_25 },
    { 0x749959f7, SUC_lexicon_tags_4 },
    { 0x573959fa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9a3959fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29f959fe, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7d995a00, SUC_lexicon_tags_95 },
    { 0x50195a00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d195a04, SUC_lexicon_tags_45 },
    { 0x2c395a04, SUC_lexicon_tags_7 },
    { 0x67795a06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b95a0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d595a10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x47595a13, SUC_lexicon_tags_9 },
    { 0x83595a14, SUC_lexicon_tags_17 },
    { 0x65d95a13, SUC_lexicon_tags_27 },
    { 0xb2d95a16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25195a1f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98195a25, SUC_lexicon_tags_6 },
    { 0xb5795a25, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8ed95a28, SUC_lexicon_tags_65 },
    { 0xec795a29, SUC_lexicon_tags_14 },
    { 0xf395a29, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1d95a2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28f95a30, SUC_lexicon_tags_21 },
    { 0xb0995a31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9995a33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b95a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34995a39, SUC_lexicon_tags_32 },
    { 0xd8795a39, SUC_lexicon_tags_0 },
    { 0x66395a3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d95a40, SUC_lexicon_tags_45 },
    { 0x71995a40, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d795a46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x34995a48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb95a4b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae595a4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x72595a52, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19395a56, SUC_lexicon_tags_3 },
    { 0x10b95a56, SUC_lexicon_tags_1 },
    { 0x65395a56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23995a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf95a64, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f95a6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6eb95a6c, SUC_lexicon_tags_61 },
    { 0x90395a6d, SUC_lexicon_tags_41 },
    { 0xb2595a6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd795a73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6dd95a75, SUC_lexicon_tags_48 },
    { 0x69d95a76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8595a7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f95a7f, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f95a88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xca995a8b, SUC_lexicon_tags_14 },
    { 0x9f95a8c, SUC_lexicon_tags_31 },
    { 0x93f95a8d, SUC_lexicon_tags_27 },
    { 0x26795a8e, SUC_lexicon_tags_60 },
    { 0xcb195a8d, SUC_lexicon_tags_43 },
    { 0xa3595a90, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea395a9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab95aa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d395aa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x27595aa8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20595aac, SUC_lexicon_tags_12 },
    { 0xef195aad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f95ab0, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xcdd95ab2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93395ab5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db95abd, SUC_lexicon_tags_14 },
    { 0xd0395abd, SUC_lexicon_tags_15 },
    { 0x28195abd, SUC_lexicon_tags_49 },
    { 0x3ef95abd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c795ac2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfbf95ac4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8995ac8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13595acd, SUC_lexicon_tags_6 },
    { 0xa3795ace, SUC_lexicon_tags_24 },
    { 0x35d95acf, SUC_lexicon_tags_63 },
    { 0x3e395acf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8d95ad2, SUC_lexicon_tags_15 },
    { 0x78b95ad3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c195ad5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64f95ad7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc195adc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb95ae4, SUC_lexicon_tags_7 },
    { 0xd7f95ae5, SUC_lexicon_tags_11 },
    { 0x32795ae6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d95aea, SUC_lexicon_tags_17 },
    { 0xc0d95aea, SUC_lexicon_tags_70 },
    { 0x3795aeb, SUC_lexicon_tags_7 },
    { 0x97595aeb, SUC_lexicon_tags_900 },
    { 0xb395aed, SUC_lexicon_tags_23 },
    { 0x5fd95aea, SUC_lexicon_tags_2 },
    { 0x2595af0, SUC_lexicon_tags_14 },
    { 0xa6f95af0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x35795af3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x11395af5, SUC_lexicon_tags_15 },
    { 0x3af95af6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f95afb, SUC_lexicon_tags_23 },
    { 0xa8595afc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d95b01, SUC_lexicon_tags_11 },
    { 0xab595b01, SUC_lexicon_tags_87 },
    { 0xd8195b03, SUC_lexicon_tags_39 },
    { 0x68595b01, SUC_lexicon_tags_7 },
    { 0x9f795b05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x73595b08, SUC_lexicon_tags_25 },
    { 0x56195b08, SUC_lexicon_tags_29 },
    { 0xdc595b0a, SUC_lexicon_tags_37 },
    { 0x5bf95b0a, SUC_lexicon_tags_37 },
    { 0xbd195b0c, SUC_lexicon_tags_12 },
    { 0xe995b0d, SUC_lexicon_tags_9 },
    { 0x1db95b0e, SUC_lexicon_tags_8 },
    { 0x96595b0f, SUC_lexicon_tags_14 },
    { 0x49d95b0d, SUC_lexicon_tags_8 },
    { 0xa1b95b11, SUC_lexicon_tags_8 },
    { 0xfe995b12, SUC_lexicon_tags_47 },
    { 0x22d95b13, SUC_lexicon_tags_23 },
    { 0xb6f95b10, SUC_lexicon_tags_230 },
    { 0xf5195b15, SUC_lexicon_tags_16 },
    { 0x11f95b16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b95b19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db95b1c, SUC_lexicon_tags_15 },
    { 0x79995b1d, SUC_lexicon_tags_25 },
    { 0xe2795b1e, SUC_lexicon_tags_2 },
    { 0x79d95b1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdff95b22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0395b24, SUC_lexicon_tags_38 },
    { 0xb7d95b24, SUC_lexicon_tags_45 },
    { 0x13f95b26, SUC_lexicon_tags_25 },
    { 0x78b95b27, SUC_lexicon_tags_25 },
    { 0xa6b95b27, SUC_lexicon_tags_15 },
    { 0x53995b28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab95b2f, SUC_lexicon_tags_19 },
    { 0x32f95b30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb95b34, SUC_lexicon_tags_4 },
    { 0xb5f95b35, SUC_lexicon_tags_37 },
    { 0xcb595b34, SUC_lexicon_tags_11 },
    { 0x4c795b37, SUC_lexicon_tags_7 },
    { 0xbfd95b35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd95b3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f95b41, SUC_lexicon_tags_7 },
    { 0x93995b42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd95b45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf95b48, SUC_lexicon_tags_21 },
    { 0xf1595b49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9b95b4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd395b4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfe395b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd95b54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe5f95b56, SUC_lexicon_tags_55 },
    { 0x2a995b57, SUC_lexicon_tags_25 },
    { 0x2d195b56, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8595b5b, SUC_lexicon_tags_23 },
    { 0x68195b5b, SUC_lexicon_tags_6 },
    { 0x71995b5d, SUC_lexicon_tags_30 },
    { 0x12395b5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf95b62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f995b65, SUC_lexicon_tags_7 },
    { 0x68d95b66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b595b69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c595b6b, SUC_lexicon_tags_27 },
    { 0xec395b6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2f95b6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93195b74, SUC_lexicon_tags_33 },
    { 0xda195b74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8595b81, SUC_lexicon_tags_29 },
    { 0xbf95b81, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdf595b84, SUC_lexicon_tags_1350 },
    { 0x6d395b84, SUC_lexicon_tags_2 },
    { 0x81995b84, SUC_lexicon_tags_12 },
    { 0x13b95b87, SUC_lexicon_tags_33 },
    { 0xcb795b87, SUC_lexicon_tags_28 },
    { 0x87995b89, SUC_lexicon_tags_6 },
    { 0x3b595b8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d195b8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36795b8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x37395b91, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5cb95b93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb95b97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba395ba2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd95ba7, SUC_lexicon_tags_9 },
    { 0x7eb95ba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30395baf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66b95bb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d995bbb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58195bc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9195bc5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d95bca, SUC_lexicon_tags_33 },
    { 0x88795bca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13195bcf, SUC_lexicon_tags_2 },
    { 0x2ef95bcf, SUC_lexicon_tags_23 },
    { 0xaab95bd1, SUC_lexicon_tags_48 },
    { 0x9ed95bd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92195bd6, SUC_lexicon_tags_37 },
    { 0x41395bd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29195bde, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d95be4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0395be9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27995bec, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x29f95bef, SUC_lexicon_tags_15 },
    { 0x5e395bf0, SUC_lexicon_tags_17 },
    { 0x70595bf1, SUC_lexicon_tags_416 },
    { 0x65f95bf2, SUC_lexicon_tags_60 },
    { 0xfc795bf1, SUC_lexicon_tags_65 },
    { 0x2a395bf4, SUC_lexicon_tags_2 },
    { 0x1995bf1, SUC_lexicon_tags_2 },
    { 0x75f95bf6, SUC_lexicon_tags_16 },
    { 0xbdb95bf6, SUC_lexicon_tags_16 },
    { 0xec395bef, SUC_lexicon_tags_37 },
    { 0x1b95bf9, SUC_lexicon_tags_45 },
    { 0x9bd95bfa, SUC_lexicon_tags_12 },
    { 0x9d195bfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d395bfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd95c05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd7195c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85995c0a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9c595c0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56395c0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa0195c11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd395c1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af95c24, SUC_lexicon_tags_37 },
    { 0x8bd95c25, SUC_lexicon_tags_61 },
    { 0x8ab95c26, SUC_lexicon_tags_45 },
    { 0x38795c27, SUC_lexicon_tags_37 },
    { 0x3c595c25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9a395c2a, SUC_lexicon_tags_7 },
    { 0x6c795c2b, SUC_lexicon_tags_17 },
    { 0xd4195c2c, SUC_lexicon_tags_2 },
    { 0x87f95c2d, SUC_lexicon_tags_25 },
    { 0x45395c2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75595c33, SUC_lexicon_tags_17 },
    { 0x2d95c34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28f95c36, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb95c3a, SUC_lexicon_tags_104 },
    { 0x3395c3b, SUC_lexicon_tags_45 },
    { 0x85795c3c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32595c40, SUC_lexicon_tags_31 },
    { 0x14d95c41, SUC_lexicon_tags_6 },
    { 0x87795c42, SUC_lexicon_tags_49 },
    { 0x16195c43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb195c4a, SUC_lexicon_tags_26 },
    { 0xad395c4a, SUC_lexicon_tags_55 },
    { 0x70995c4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc195c52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3595c54, SUC_lexicon_tags_32 },
    { 0x33d95c55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60195c57, SUC_lexicon_tags_19 },
    { 0xc3b95c57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd95c5d, SUC_lexicon_tags_65 },
    { 0xaa395c5e, SUC_lexicon_tags_7 },
    { 0xba995c5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1595c61, SUC_lexicon_tags_33 },
    { 0x3a395c62, SUC_lexicon_tags_14 },
    { 0x2e595c61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f595c65, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9595c68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5ff95c6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d95c6d, SUC_lexicon_tags_5 },
    { 0x87195c6e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb995c70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57395c72, SUC_lexicon_tags_108 },
    { 0xb4f95c73, SUC_lexicon_tags_7 },
    { 0xa9795c72, SUC_lexicon_tags_24 },
    { 0x84595c75, SUC_lexicon_tags_0 },
    { 0xfed95c76, SUC_lexicon_tags_15 },
    { 0x60f95c77, SUC_lexicon_tags_37 },
    { 0xd3395c78, SUC_lexicon_tags_37 },
    { 0x4ed95c77, SUC_lexicon_tags_21 },
    { 0x14b95c79, SUC_lexicon_tags_1 },
    { 0x48595c79, SUC_lexicon_tags_56 },
    { 0x28995c7c, SUC_lexicon_tags_17 },
    { 0x6b595c7c, SUC_lexicon_tags_9 },
    { 0x4bb95c7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4395c81, SUC_lexicon_tags_7 },
    { 0x15795c82, SUC_lexicon_tags_25 },
    { 0xd7595c83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d595c85, SUC_lexicon_tags_60 },
    { 0xb6395c86, SUC_lexicon_tags_19 },
    { 0xa6595c87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe3395c89, SUC_lexicon_tags_9 },
    { 0xd8395c8a, SUC_lexicon_tags_9 },
    { 0xfdb95c8b, SUC_lexicon_tags_65 },
    { 0x46195c8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a195c91, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26195c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9595ca2, SUC_lexicon_tags_37 },
    { 0x19b95ca3, SUC_lexicon_tags_10 },
    { 0xc6d95ca4, SUC_lexicon_tags_14 },
    { 0x8cd95ca5, SUC_lexicon_tags_12 },
    { 0x48d95ca6, SUC_lexicon_tags_31 },
    { 0x1a795ca7, SUC_lexicon_tags_11 },
    { 0x73195ca7, SUC_lexicon_tags_12 },
    { 0x13595ca7, SUC_lexicon_tags_1 },
    { 0x93995ca5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa595cac, SUC_lexicon_tags_2 },
    { 0x9a395cad, SUC_lexicon_tags_23 },
    { 0xb1795cae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb9b95cb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed95cb5, SUC_lexicon_tags_7 },
    { 0xf1d95cb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd795cba, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f195cbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca595cc5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27195cca, SUC_lexicon_tags_12 },
    { 0x64d95cca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x14795ccd, SUC_lexicon_tags_37 },
    { 0x8e195cce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb595cd4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7395cd8, SUC_lexicon_tags_25 },
    { 0x14995cd9, SUC_lexicon_tags_39 },
    { 0xdcf95cda, SUC_lexicon_tags_16 },
    { 0x87d95cdb, SUC_lexicon_tags_63 },
    { 0x9bf95cdc, SUC_lexicon_tags_2 },
    { 0x38795cdd, SUC_lexicon_tags_9 },
    { 0x8bd95cdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8995ce3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f95ce7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6cf95ce9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d95cec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e595cf1, SUC_lexicon_tags_30 },
    { 0xae595cf2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa4d95cf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795cf9, SUC_lexicon_tags_27 },
    { 0xe8d95cfa, SUC_lexicon_tags_7 },
    { 0x1d595cfa, SUC_lexicon_tags_7 },
    { 0x53395cfc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x65195cfe, SUC_lexicon_tags_47 },
    { 0x46395cff, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x69595d01, SUC_lexicon_tags_27 },
    { 0x2dd95d01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d95d06, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f995d09, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6395d13, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd95d16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41795d1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaaf95d1e, SUC_lexicon_tags_25 },
    { 0x84595d1e, SUC_lexicon_tags_25 },
    { 0xf9795d20, SUC_lexicon_tags_91 },
    { 0xa6d95d20, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x5595d23, SUC_lexicon_tags_7 },
    { 0xa2195d24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc995d26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce595d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfed95d2f, SUC_lexicon_tags_14 },
    { 0xe0995d30, SUC_lexicon_tags_10 },
    { 0xc9395d31, SUC_lexicon_tags_483 },
    { 0, NULL },
    { 0xb8595d33, SUC_lexicon_tags_6 },
    { 0xaf995d33, SUC_lexicon_tags_14 },
    { 0xd395d35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19f95d37, SUC_lexicon_tags_17 },
    { 0x31b95d38, SUC_lexicon_tags_19 },
    { 0x50f95d38, SUC_lexicon_tags_2 },
    { 0xa0995d3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c595d3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d795d3f, SUC_lexicon_tags_30 },
    { 0x35995d3f, SUC_lexicon_tags_1 },
    { 0x3a595d40, SUC_lexicon_tags_11 },
    { 0xbd95d41, SUC_lexicon_tags_45 },
    { 0xd6d95d43, SUC_lexicon_tags_63 },
    { 0xe5f95d43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e795d47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d395d4a, SUC_lexicon_tags_11 },
    { 0xb0795d4b, SUC_lexicon_tags_62 },
    { 0x67995d4b, SUC_lexicon_tags_38 },
    { 0x50195d4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5195d53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9e795d55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8f95d57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x43595d5a, SUC_lexicon_tags_40 },
    { 0xc8b95d5b, SUC_lexicon_tags_12 },
    { 0x3cb95d5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83195d60, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb95d66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6995d73, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f395d78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7bb95d7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d95d7d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41795d87, SUC_lexicon_tags_27 },
    { 0x95195d88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19b95d8a, SUC_lexicon_tags_43 },
    { 0xc5b95d8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc395d8d, SUC_lexicon_tags_8 },
    { 0xb8995d8e, SUC_lexicon_tags_49 },
    { 0xbcd95d8d, SUC_lexicon_tags_7 },
    { 0xd1d95d8e, SUC_lexicon_tags_23 },
    { 0x8dd95d8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x54395d93, SUC_lexicon_tags_27 },
    { 0x1db95d94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a795d96, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x21d95d98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f95d9b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e195d9e, SUC_lexicon_tags_38 },
    { 0x66b95d9f, SUC_lexicon_tags_23 },
    { 0xc3d95d9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23995da2, SUC_lexicon_tags_0 },
    { 0x96b95da3, SUC_lexicon_tags_19 },
    { 0x83d95da4, SUC_lexicon_tags_17 },
    { 0xa595da5, SUC_lexicon_tags_2 },
    { 0xbcd95da4, SUC_lexicon_tags_33 },
    { 0xd8395da7, SUC_lexicon_tags_30 },
    { 0x10995da8, SUC_lexicon_tags_6 },
    { 0x9bf95da9, SUC_lexicon_tags_32 },
    { 0xb7995daa, SUC_lexicon_tags_43 },
    { 0x10195da6, SUC_lexicon_tags_27 },
    { 0xc2595dac, SUC_lexicon_tags_12 },
    { 0xda595dad, SUC_lexicon_tags_2 },
    { 0x5ff95dae, SUC_lexicon_tags_60 },
    { 0xd5d95dad, SUC_lexicon_tags_2 },
    { 0x78995db0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x51595db2, SUC_lexicon_tags_17 },
    { 0xc2b95db3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfb795db5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x51195db8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6fd95dba, SUC_lexicon_tags_25 },
    { 0xf1795dba, SUC_lexicon_tags_37 },
    { 0xaa395dba, SUC_lexicon_tags_25 },
    { 0x84f95dbd, SUC_lexicon_tags_17 },
    { 0xbb795dbd, SUC_lexicon_tags_44 },
    { 0xdaf95dbf, SUC_lexicon_tags_6 },
    { 0x25f95dc0, SUC_lexicon_tags_2 },
    { 0x32f95dc1, SUC_lexicon_tags_10 },
    { 0xd3395dc2, SUC_lexicon_tags_12 },
    { 0xce995dc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x66395dc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b95dca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x64195dcc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cd95dd0, SUC_lexicon_tags_38 },
    { 0xd7195dd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b595dd3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2c595dd5, SUC_lexicon_tags_33 },
    { 0x40f95dd6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x62795dd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88b95ddb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66995ddd, SUC_lexicon_tags_25 },
    { 0x3db95ddd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74d95de2, SUC_lexicon_tags_1 },
    { 0x4795de3, SUC_lexicon_tags_27 },
    { 0x38395de3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af95de8, SUC_lexicon_tags_12 },
    { 0xb2f95de9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x12b95deb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7395ded, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d95df0, SUC_lexicon_tags_13 },
    { 0xe9195df0, SUC_lexicon_tags_27 },
    { 0x73195df0, SUC_lexicon_tags_48 },
    { 0xed95df3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad95e00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xce795e02, SUC_lexicon_tags_1 },
    { 0xf7595e02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38795e05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f95e08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x19b95e0a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb5395e0c, SUC_lexicon_tags_30 },
    { 0xb1b95e0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b95e12, SUC_lexicon_tags_15 },
    { 0xb8b95e13, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd5195e15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf9d95e17, SUC_lexicon_tags_7 },
    { 0x86f95e18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb595e26, SUC_lexicon_tags_5 },
    { 0x9ad95e26, SUC_lexicon_tags_71 },
    { 0x70f95e28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed795e2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x62f95e31, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62195e36, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x5ad95e38, SUC_lexicon_tags_25 },
    { 0xb8d95e39, SUC_lexicon_tags_8 },
    { 0x14795e3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d195e3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4195e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30595e43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9195e45, SUC_lexicon_tags_46 },
    { 0xc995e46, SUC_lexicon_tags_11 },
    { 0x8ff95e47, SUC_lexicon_tags_7 },
    { 0xaa195e47, SUC_lexicon_tags_37 },
    { 0x23d95e49, SUC_lexicon_tags_56 },
    { 0x64f95e4a, SUC_lexicon_tags_12 },
    { 0x68d95e4b, SUC_lexicon_tags_64 },
    { 0x4e995e4c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbef95e4e, SUC_lexicon_tags_23 },
    { 0xe7b95e4f, SUC_lexicon_tags_38 },
    { 0x73d95e4e, SUC_lexicon_tags_45 },
    { 0x2cb95e4f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x48595e53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f595e56, SUC_lexicon_tags_7 },
    { 0x22b95e56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf3995e59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75995e5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x68f95e5e, SUC_lexicon_tags_59 },
    { 0x3db95e5f, SUC_lexicon_tags_25 },
    { 0x60d95e5f, SUC_lexicon_tags_0 },
    { 0xb8195e60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf95e65, SUC_lexicon_tags_15 },
    { 0xa9d95e66, SUC_lexicon_tags_11 },
    { 0x20795e65, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb7795e69, SUC_lexicon_tags_6 },
    { 0xe5395e6a, SUC_lexicon_tags_201 },
    { 0x3d595e6b, SUC_lexicon_tags_45 },
    { 0xc6195e6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88395e6e, SUC_lexicon_tags_30 },
    { 0xf7995e6f, SUC_lexicon_tags_14 },
    { 0x9d395e6f, SUC_lexicon_tags_60 },
    { 0xfd995e6f, SUC_lexicon_tags_46 },
    { 0x9db95e72, SUC_lexicon_tags_37 },
    { 0xe6595e73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17795e82, SUC_lexicon_tags_19 },
    { 0xa5195e82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a795e87, SUC_lexicon_tags_7 },
    { 0xe5d95e87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86795e8a, SUC_lexicon_tags_88 },
    { 0xe3f95e8b, SUC_lexicon_tags_2 },
    { 0x4d795e8c, SUC_lexicon_tags_10 },
    { 0x49d95e8d, SUC_lexicon_tags_25 },
    { 0xd9f95e8e, SUC_lexicon_tags_23 },
    { 0x35795e8f, SUC_lexicon_tags_37 },
    { 0xa595e8f, SUC_lexicon_tags_45 },
    { 0xac995e8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62195e93, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b795e9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f95ead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58795eaf, SUC_lexicon_tags_17 },
    { 0x53795eb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c395ec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98195ec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9995eca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89795ece, SUC_lexicon_tags_25 },
    { 0x2cb95ecf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f95ed2, SUC_lexicon_tags_33 },
    { 0xcbf95ed3, SUC_lexicon_tags_15 },
    { 0x17995ed4, SUC_lexicon_tags_33 },
    { 0x1c795ed4, SUC_lexicon_tags_224 },
    { 0xc995ed5, SUC_lexicon_tags_14 },
    { 0xa2195ed2, SUC_lexicon_tags_7 },
    { 0x38795ed7, SUC_lexicon_tags_17 },
    { 0x4e595ed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15d95edb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db95ede, SUC_lexicon_tags_11 },
    { 0xa5995edf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e595ee2, SUC_lexicon_tags_27 },
    { 0xdd595ee3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd7d95ee5, SUC_lexicon_tags_8 },
    { 0xd4b95ee6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b995ee9, SUC_lexicon_tags_37 },
    { 0xa3795eea, SUC_lexicon_tags_55 },
    { 0xac195eeb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10595ef2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x11195ef5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66195ef8, SUC_lexicon_tags_57 },
    { 0x28f95ef9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2595efc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef95eff, SUC_lexicon_tags_584 },
    { 0x2ef95eff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7e195f02, SUC_lexicon_tags_8 },
    { 0xa9995f03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf5395f05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1f95f07, SUC_lexicon_tags_77 },
    { 0xc5d95f08, SUC_lexicon_tags_17 },
    { 0xbc595f09, SUC_lexicon_tags_23 },
    { 0x32b95f0a, SUC_lexicon_tags_58 },
    { 0x30995f09, SUC_lexicon_tags_27 },
    { 0xe8d95f0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23195f12, SUC_lexicon_tags_37 },
    { 0xa8b95f12, SUC_lexicon_tags_2 },
    { 0x89b95f12, SUC_lexicon_tags_27 },
    { 0xdeb95f15, SUC_lexicon_tags_17 },
    { 0x19d95f16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a395f1b, SUC_lexicon_tags_2 },
    { 0x29395f1b, SUC_lexicon_tags_27 },
    { 0xb2995f1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8b95f1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d95f28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa6995f2a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb0b95f2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb795f32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbeb95f34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b95f37, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d95f3a, SUC_lexicon_tags_68 },
    { 0xc9f95f3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9f995f3d, SUC_lexicon_tags_2 },
    { 0x9d595f3e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8995f43, SUC_lexicon_tags_15 },
    { 0xb7195f44, SUC_lexicon_tags_15 },
    { 0x1a995f45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5595f4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d95f53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50595f59, SUC_lexicon_tags_17 },
    { 0x83795f5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x32d95f5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7195f60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe795f65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bf95f67, SUC_lexicon_tags_54 },
    { 0xc1595f68, SUC_lexicon_tags_25 },
    { 0xa4d95f69, SUC_lexicon_tags_38 },
    { 0x8d395f6a, SUC_lexicon_tags_49 },
    { 0xa9995f6b, SUC_lexicon_tags_7 },
    { 0xeb595f69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35595f6e, SUC_lexicon_tags_12 },
    { 0x5a995f6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad95f72, SUC_lexicon_tags_63 },
    { 0xfb795f73, SUC_lexicon_tags_84 },
    { 0x88595f74, SUC_lexicon_tags_82 },
    { 0x24f95f75, SUC_lexicon_tags_18 },
    { 0x91795f75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5995f78, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb95f7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d95f87, SUC_lexicon_tags_427 },
    { 0x81995f87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8595f8b, SUC_lexicon_tags_55 },
    { 0xa195f8c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbc995f8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64795f91, SUC_lexicon_tags_25 },
    { 0xe8b95f91, SUC_lexicon_tags_11 },
    { 0x25795f93, SUC_lexicon_tags_7 },
    { 0xeb595f91, SUC_lexicon_tags_17 },
    { 0xe6595f95, SUC_lexicon_tags_4 },
    { 0x69195f96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b95f9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41195fa0, SUC_lexicon_tags_13 },
    { 0x57195fa1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa4795fa3, SUC_lexicon_tags_45 },
    { 0xf7f95fa3, SUC_lexicon_tags_45 },
    { 0x87d95fa5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d95fa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34d95fb0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4e795fb2, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea395fbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f95fc2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb95fc5, SUC_lexicon_tags_0 },
    { 0xb6795fc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7795fc8, SUC_lexicon_tags_87 },
    { 0x98595fc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d195fce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46f95fd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b95fd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac795fd4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0995fdd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfc195fdf, SUC_lexicon_tags_7 },
    { 0x49595fe0, SUC_lexicon_tags_11 },
    { 0x19f95fe1, SUC_lexicon_tags_37 },
    { 0x14b95fe1, SUC_lexicon_tags_19 },
    { 0x5d995fdf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b95fe6, SUC_lexicon_tags_2 },
    { 0x28f95fe7, SUC_lexicon_tags_13 },
    { 0x94d95fe8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4195fed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34595ff1, SUC_lexicon_tags_2 },
    { 0x35b95ff2, SUC_lexicon_tags_28 },
    { 0x2595ff3, SUC_lexicon_tags_45 },
    { 0xfe195ff3, SUC_lexicon_tags_48 },
    { 0x44b95ff5, SUC_lexicon_tags_1 },
    { 0x3af95ff6, SUC_lexicon_tags_63 },
    { 0x4bf95ff7, SUC_lexicon_tags_45 },
    { 0x8af95ff7, SUC_lexicon_tags_9 },
    { 0x8fb95ff9, SUC_lexicon_tags_55 },
    { 0x84f95ff2, SUC_lexicon_tags_11 },
    { 0x87795ffb, SUC_lexicon_tags_38 },
    { 0xcfd95ffb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef996001, SUC_lexicon_tags_47 },
    { 0x9a596002, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff96009, SUC_lexicon_tags_23 },
    { 0xc699600a, SUC_lexicon_tags_1 },
    { 0xb9996009, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2d96010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74196012, SUC_lexicon_tags_5 },
    { 0xd5f96012, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88996015, SUC_lexicon_tags_38 },
    { 0xee996015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce996018, SUC_lexicon_tags_4 },
    { 0xa8d96018, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4579601b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79601f, SUC_lexicon_tags_33 },
    { 0x3999601f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6996024, SUC_lexicon_tags_91 },
    { 0x47996025, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x47396028, SUC_lexicon_tags_49 },
    { 0x14596029, SUC_lexicon_tags_48 },
    { 0xf9b9602a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec19602c, SUC_lexicon_tags_26 },
    { 0xb0d9602c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60796036, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x34f96038, SUC_lexicon_tags_77 },
    { 0xfe196039, SUC_lexicon_tags_37 },
    { 0xb8b9603a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xab19603c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2796043, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6f396045, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1139604c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b96051, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x54b96054, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbaf96056, SUC_lexicon_tags_23 },
    { 0x56d96057, SUC_lexicon_tags_12 },
    { 0x11596057, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfed9605a, SUC_lexicon_tags_12 },
    { 0xa8f9605b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcf96060, SUC_lexicon_tags_37 },
    { 0x4ab96061, SUC_lexicon_tags_12 },
    { 0x5196062, SUC_lexicon_tags_17 },
    { 0xdcf96063, SUC_lexicon_tags_9 },
    { 0x9db96064, SUC_lexicon_tags_21 },
    { 0x52796065, SUC_lexicon_tags_37 },
    { 0xa2d96065, SUC_lexicon_tags_1 },
    { 0x89196066, SUC_lexicon_tags_15 },
    { 0x33d96068, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b9606b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xaeb9606d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5796075, SUC_lexicon_tags_26 },
    { 0x58596076, SUC_lexicon_tags_23 },
    { 0x92396076, SUC_lexicon_tags_5 },
    { 0xdef96076, SUC_lexicon_tags_7 },
    { 0x21596079, SUC_lexicon_tags_37 },
    { 0x60f96077, SUC_lexicon_tags_126 },
    { 0x6cd9607b, SUC_lexicon_tags_28 },
    { 0x4639607c, SUC_lexicon_tags_15 },
    { 0x7eb96075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3939607f, SUC_lexicon_tags_66 },
    { 0x86396080, SUC_lexicon_tags_16 },
    { 0x65396081, SUC_lexicon_tags_66 },
    { 0xfc396081, SUC_lexicon_tags_23 },
    { 0xc0f96083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e396086, SUC_lexicon_tags_3 },
    { 0xe5196087, SUC_lexicon_tags_15 },
    { 0xd0d96086, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe139608c, SUC_lexicon_tags_43 },
    { 0x2899608d, SUC_lexicon_tags_1 },
    { 0x10b9608e, SUC_lexicon_tags_1 },
    { 0xf2d9608c, SUC_lexicon_tags_11 },
    { 0xe6f96090, SUC_lexicon_tags_13 },
    { 0x71196090, SUC_lexicon_tags_68 },
    { 0x4b396091, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12f96094, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf96097, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f9609a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9379609c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183960a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67960a6, SUC_lexicon_tags_217 },
    { 0x849960a7, SUC_lexicon_tags_12 },
    { 0x9fd960a7, SUC_lexicon_tags_60 },
    { 0x1b3960a8, SUC_lexicon_tags_25 },
    { 0xc0b960a7, SUC_lexicon_tags_45 },
    { 0xe95960ab, SUC_lexicon_tags_1 },
    { 0x89b960ab, SUC_lexicon_tags_63 },
    { 0x9d1960ac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x427960af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62f960b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x479960b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7960c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b960c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5960cc, SUC_lexicon_tags_19 },
    { 0x93f960cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797960db, SUC_lexicon_tags_28 },
    { 0x4db960dc, SUC_lexicon_tags_29 },
    { 0xba7960db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a7960df, SUC_lexicon_tags_66 },
    { 0xd7b960df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xab1960e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad960e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cb960ea, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69960ed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909960f2, SUC_lexicon_tags_37 },
    { 0x62d960f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x199960f5, SUC_lexicon_tags_17 },
    { 0x39b960f6, SUC_lexicon_tags_11 },
    { 0x351960f6, SUC_lexicon_tags_37 },
    { 0xcd3960f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7960fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3960fe, SUC_lexicon_tags_7 },
    { 0xc93960fe, SUC_lexicon_tags_12 },
    { 0x18b96100, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b996105, SUC_lexicon_tags_25 },
    { 0xd6f96106, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b796108, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xd8d9610a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d9610d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9996113, SUC_lexicon_tags_38 },
    { 0x12596113, SUC_lexicon_tags_12 },
    { 0x6a596114, SUC_lexicon_tags_2 },
    { 0x76796114, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5219611a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f79611d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f96123, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5ff96125, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadf96127, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed19612a, SUC_lexicon_tags_35 },
    { 0x28d9612b, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9259612f, SUC_lexicon_tags_17 },
    { 0xadd96130, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f96132, SUC_lexicon_tags_63 },
    { 0x70d96133, SUC_lexicon_tags_66 },
    { 0xa0196132, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda596136, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4df96138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe319613c, SUC_lexicon_tags_19 },
    { 0x7639613d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52b9613f, SUC_lexicon_tags_49 },
    { 0x6eb96140, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde196143, SUC_lexicon_tags_46 },
    { 0x65b96144, SUC_lexicon_tags_184 },
    { 0x8bb96143, SUC_lexicon_tags_12 },
    { 0x34396146, SUC_lexicon_tags_104 },
    { 0x89f96147, SUC_lexicon_tags_14 },
    { 0x36796147, SUC_lexicon_tags_27 },
    { 0x19396149, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f79614b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3139614f, SUC_lexicon_tags_37 },
    { 0xf5196150, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc9196152, SUC_lexicon_tags_15 },
    { 0x97596152, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39396159, SUC_lexicon_tags_118 },
    { 0xeb9615a, SUC_lexicon_tags_47 },
    { 0xe1b9615a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7099615f, SUC_lexicon_tags_53 },
    { 0x8a996160, SUC_lexicon_tags_12 },
    { 0x45f96161, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1196165, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18196167, SUC_lexicon_tags_15 },
    { 0xab396167, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5279616a, SUC_lexicon_tags_6 },
    { 0x9d19616b, SUC_lexicon_tags_34 },
    { 0xd199616a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfb9616e, SUC_lexicon_tags_25 },
    { 0x3979616e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a196178, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59617e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef96181, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa4396183, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7d996185, SUC_lexicon_tags_37 },
    { 0x3cf96185, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9618a, SUC_lexicon_tags_11 },
    { 0x77f9618b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34796190, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f96193, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x20f96196, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf96199, SUC_lexicon_tags_7 },
    { 0x5279619a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1ab9619c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86f9619e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3961a0, SUC_lexicon_tags_129 },
    { 0x737961a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21d961a3, SUC_lexicon_tags_17 },
    { 0x8f5961a4, SUC_lexicon_tags_49 },
    { 0x6d961a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33961a9, SUC_lexicon_tags_27 },
    { 0x3f1961aa, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b961b4, SUC_lexicon_tags_1 },
    { 0x7e7961b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7961b9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303961be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x667961c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa49961c3, SUC_lexicon_tags_12 },
    { 0xa9f961c3, SUC_lexicon_tags_33 },
    { 0x19f961c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb6d961c7, SUC_lexicon_tags_66 },
    { 0x5d9961c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1961cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25961d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41961d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3961d9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x509961db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459961e3, SUC_lexicon_tags_25 },
    { 0xb4d961e4, SUC_lexicon_tags_37 },
    { 0xa01961e5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb49961e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf961ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb961f0, SUC_lexicon_tags_37 },
    { 0x2b3961f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab961f6, SUC_lexicon_tags_47 },
    { 0xf1d961f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5961fb, SUC_lexicon_tags_11 },
    { 0x9f961fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d196200, SUC_lexicon_tags_46 },
    { 0xf7b96200, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb96209, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e79620e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7796210, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9796214, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95996218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa959621d, SUC_lexicon_tags_29 },
    { 0x1e59621d, SUC_lexicon_tags_79 },
    { 0xbff9621f, SUC_lexicon_tags_38 },
    { 0x65796220, SUC_lexicon_tags_37 },
    { 0xff79621f, SUC_lexicon_tags_33 },
    { 0xd2f9621f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb96228, SUC_lexicon_tags_6 },
    { 0x3396229, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd479622b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c19622e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d96232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd196234, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4d96236, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc196238, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb9623c, SUC_lexicon_tags_61 },
    { 0x6919623d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b96240, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44f96242, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x30396245, SUC_lexicon_tags_15 },
    { 0x3cf96246, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac996249, SUC_lexicon_tags_43 },
    { 0x8119624a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90796252, SUC_lexicon_tags_26 },
    { 0xfa996253, SUC_lexicon_tags_49 },
    { 0x58796254, SUC_lexicon_tags_0 },
    { 0x7f396254, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x57596257, SUC_lexicon_tags_27 },
    { 0xf1996258, SUC_lexicon_tags_1059 },
    { 0x51d96259, SUC_lexicon_tags_11 },
    { 0xd0d9625a, SUC_lexicon_tags_13 },
    { 0x2ef9625b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3019625e, SUC_lexicon_tags_25 },
    { 0xbb99625e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9796265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c596267, SUC_lexicon_tags_17 },
    { 0x24d96268, SUC_lexicon_tags_11 },
    { 0xf5996268, SUC_lexicon_tags_14 },
    { 0xa199626a, SUC_lexicon_tags_26 },
    { 0x78b9626a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe259626d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5196275, SUC_lexicon_tags_2 },
    { 0xc4f96276, SUC_lexicon_tags_37 },
    { 0x18b96277, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df9627b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xf059627d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79596281, SUC_lexicon_tags_9 },
    { 0xce596282, SUC_lexicon_tags_6 },
    { 0xd5596283, SUC_lexicon_tags_32 },
    { 0xa2f96284, SUC_lexicon_tags_23 },
    { 0xb7b96285, SUC_lexicon_tags_10 },
    { 0xb2d96286, SUC_lexicon_tags_27 },
    { 0x98396287, SUC_lexicon_tags_17 },
    { 0xa9396283, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf59628b, SUC_lexicon_tags_45 },
    { 0xd2f9628b, SUC_lexicon_tags_12 },
    { 0xc2d9628d, SUC_lexicon_tags_11 },
    { 0x3039628d, SUC_lexicon_tags_17 },
    { 0x1659628f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d596291, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3eb96293, SUC_lexicon_tags_2 },
    { 0xca596294, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5196298, SUC_lexicon_tags_17 },
    { 0xa1b96299, SUC_lexicon_tags_17 },
    { 0x41d9629a, SUC_lexicon_tags_25 },
    { 0x8079629b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af962a0, SUC_lexicon_tags_23 },
    { 0x7f1962a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x489962a3, SUC_lexicon_tags_30 },
    { 0xdcf962a3, SUC_lexicon_tags_15 },
    { 0x6cd962a3, SUC_lexicon_tags_37 },
    { 0x603962a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b962ab, SUC_lexicon_tags_33 },
    { 0x421962ab, SUC_lexicon_tags_27 },
    { 0xa87962ad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b962b0, SUC_lexicon_tags_19 },
    { 0x72b962b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5962b4, SUC_lexicon_tags_11 },
    { 0x4e5962b5, SUC_lexicon_tags_29 },
    { 0x5b7962b6, SUC_lexicon_tags_69 },
    { 0xe8f962b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d962bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf962c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x18d962c3, SUC_lexicon_tags_37 },
    { 0x249962c4, SUC_lexicon_tags_46 },
    { 0x317962c4, SUC_lexicon_tags_15 },
    { 0x26b962c4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b962c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51962cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb962d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb5b962d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f962d8, SUC_lexicon_tags_96 },
    { 0xfdd962d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59962e0, SUC_lexicon_tags_4 },
    { 0xde7962e1, SUC_lexicon_tags_47 },
    { 0x723962e2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f962e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99962ec, SUC_lexicon_tags_113 },
    { 0x307962ec, SUC_lexicon_tags_12 },
    { 0x6b3962ec, SUC_lexicon_tags_8 },
    { 0x353962ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf23962f1, SUC_lexicon_tags_17 },
    { 0xd85962f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x46d962f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9962f7, SUC_lexicon_tags_43 },
    { 0xe3d962f8, SUC_lexicon_tags_38 },
    { 0xa2b962f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x58f962fb, SUC_lexicon_tags_25 },
    { 0x9962fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x143962fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd96301, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xa7b96303, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef96308, SUC_lexicon_tags_1 },
    { 0x31796309, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0796311, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d196319, SUC_lexicon_tags_49 },
    { 0x83d9631a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3499631f, SUC_lexicon_tags_17 },
    { 0xf596320, SUC_lexicon_tags_5 },
    { 0xf2596321, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b96325, SUC_lexicon_tags_11 },
    { 0xb8b96325, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3596328, SUC_lexicon_tags_2 },
    { 0xd2b96329, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3996330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4396333, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4196337, SUC_lexicon_tags_63 },
    { 0xc3996337, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcb9633c, SUC_lexicon_tags_65 },
    { 0xd5d9633d, SUC_lexicon_tags_8 },
    { 0x6f99633d, SUC_lexicon_tags_33 },
    { 0x1ab9633f, SUC_lexicon_tags_15 },
    { 0x55f9633f, SUC_lexicon_tags_37 },
    { 0x84996340, SUC_lexicon_tags_2 },
    { 0x3a196342, SUC_lexicon_tags_160 },
    { 0x76196343, SUC_lexicon_tags_65 },
    { 0x1cd96344, SUC_lexicon_tags_37 },
    { 0x57796345, SUC_lexicon_tags_29 },
    { 0x97b96346, SUC_lexicon_tags_17 },
    { 0x57796343, SUC_lexicon_tags_55 },
    { 0xf3596345, SUC_lexicon_tags_82 },
    { 0xa1f96344, SUC_lexicon_tags_12 },
    { 0x54596343, SUC_lexicon_tags_11 },
    { 0xd259634b, SUC_lexicon_tags_29 },
    { 0x79b9634c, SUC_lexicon_tags_11 },
    { 0xa7596342, SUC_lexicon_tags_1 },
    { 0xf519634e, SUC_lexicon_tags_37 },
    { 0xae19634e, SUC_lexicon_tags_38 },
    { 0xff196350, SUC_lexicon_tags_5 },
    { 0xf3b9634f, SUC_lexicon_tags_25 },
    { 0x8a196352, SUC_lexicon_tags_1 },
    { 0x9f596344, SUC_lexicon_tags_11 },
    { 0x21b9634c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dd96356, SUC_lexicon_tags_9 },
    { 0x91396357, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27996359, SUC_lexicon_tags_23 },
    { 0x7299635a, SUC_lexicon_tags_124 },
    { 0xcaf9635b, SUC_lexicon_tags_26 },
    { 0x9c79635a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac396360, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe0996362, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2796369, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af9636c, SUC_lexicon_tags_27 },
    { 0xf59636c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb659636f, SUC_lexicon_tags_28 },
    { 0x5a196370, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50996375, SUC_lexicon_tags_60 },
    { 0xa0796376, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcdf96378, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc19637e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d196382, SUC_lexicon_tags_31 },
    { 0xd1396382, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3796387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4e396389, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb319638e, SUC_lexicon_tags_65 },
    { 0xa339638e, SUC_lexicon_tags_8 },
    { 0x9c596390, SUC_lexicon_tags_37 },
    { 0x5d596390, SUC_lexicon_tags_102 },
    { 0, NULL },
    { 0x32596393, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x74196395, SUC_lexicon_tags_30 },
    { 0x2a396395, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e396398, SUC_lexicon_tags_25 },
    { 0xa2596399, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8959639d, SUC_lexicon_tags_25 },
    { 0xccb9639d, SUC_lexicon_tags_79 },
    { 0x779639f, SUC_lexicon_tags_15 },
    { 0xa339639f, SUC_lexicon_tags_37 },
    { 0x943963a1, SUC_lexicon_tags_17 },
    { 0x69963a1, SUC_lexicon_tags_63 },
    { 0xe4b963a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f963a7, SUC_lexicon_tags_19 },
    { 0x879963a8, SUC_lexicon_tags_19 },
    { 0xff9963a9, SUC_lexicon_tags_9 },
    { 0xa7d963a8, SUC_lexicon_tags_14 },
    { 0x89d963a9, SUC_lexicon_tags_43 },
    { 0x847963ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x963963af, SUC_lexicon_tags_38 },
    { 0x721963af, SUC_lexicon_tags_45 },
    { 0x1ef963b1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x80b963b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x88d963b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf01963b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9ed963ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271963c0, SUC_lexicon_tags_68 },
    { 0x5bd963c1, SUC_lexicon_tags_15 },
    { 0xa9b963c2, SUC_lexicon_tags_37 },
    { 0x6dd963c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5963c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc51963c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf3963cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x453963cd, SUC_lexicon_tags_60 },
    { 0xae7963ce, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9963d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4bb963d3, SUC_lexicon_tags_6 },
    { 0x135963d4, SUC_lexicon_tags_69 },
    { 0x4e3963d4, SUC_lexicon_tags_37 },
    { 0xdd963d4, SUC_lexicon_tags_27 },
    { 0xc55963d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d3963d9, SUC_lexicon_tags_7 },
    { 0x291963d9, SUC_lexicon_tags_37 },
    { 0xac3963db, SUC_lexicon_tags_165 },
    { 0x40d963dc, SUC_lexicon_tags_17 },
    { 0xdf3963db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa15963df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9963e2, SUC_lexicon_tags_2 },
    { 0xa59963e2, SUC_lexicon_tags_15 },
    { 0xc1b963e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d963e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7963ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff963f0, SUC_lexicon_tags_45 },
    { 0xff963f0, SUC_lexicon_tags_82 },
    { 0x1d1963f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5963fa, SUC_lexicon_tags_103 },
    { 0xc7d963fb, SUC_lexicon_tags_1 },
    { 0x115963fc, SUC_lexicon_tags_116 },
    { 0xa1963fd, SUC_lexicon_tags_47 },
    { 0xe2f963fe, SUC_lexicon_tags_37 },
    { 0x451963fe, SUC_lexicon_tags_17 },
    { 0x7eb96400, SUC_lexicon_tags_17 },
    { 0xb9796400, SUC_lexicon_tags_17 },
    { 0x83796402, SUC_lexicon_tags_45 },
    { 0xd2596400, SUC_lexicon_tags_37 },
    { 0x60396404, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb8b96406, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb6196408, SUC_lexicon_tags_17 },
    { 0x8796408, SUC_lexicon_tags_18 },
    { 0x1d396409, SUC_lexicon_tags_26 },
    { 0x4d19640b, SUC_lexicon_tags_112 },
    { 0x6b9640c, SUC_lexicon_tags_19 },
    { 0x4a39640d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d96411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40996413, SUC_lexicon_tags_14 },
    { 0xb3796413, SUC_lexicon_tags_10 },
    { 0x8bd96415, SUC_lexicon_tags_11 },
    { 0x1af96416, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9641a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb596420, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34396425, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4796429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8396431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x396433, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f96439, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef9643c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21d9643e, SUC_lexicon_tags_38 },
    { 0x1759643f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x84196442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2396444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb96447, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8399644d, SUC_lexicon_tags_43 },
    { 0xdbf9644e, SUC_lexicon_tags_17 },
    { 0x5419644f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd8d96451, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b96454, SUC_lexicon_tags_37 },
    { 0x42996455, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f196459, SUC_lexicon_tags_7 },
    { 0xf379645a, SUC_lexicon_tags_11 },
    { 0x8d59645b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50996463, SUC_lexicon_tags_37 },
    { 0xec996464, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed96468, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x65d9646a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2139646c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb39646f, SUC_lexicon_tags_28 },
    { 0xc619646f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5d96472, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d59647b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc396487, SUC_lexicon_tags_37 },
    { 0x1ab96488, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd919648a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2f9648c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f9648f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x47796492, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52796495, SUC_lexicon_tags_22 },
    { 0x4dd96496, SUC_lexicon_tags_11 },
    { 0x78396495, SUC_lexicon_tags_15 },
    { 0xf9196498, SUC_lexicon_tags_25 },
    { 0x7bb96498, SUC_lexicon_tags_3 },
    { 0x3179649a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed9649d, SUC_lexicon_tags_49 },
    { 0xc6f9649d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81964a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f964ac, SUC_lexicon_tags_13 },
    { 0xa27964ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x981964af, SUC_lexicon_tags_27 },
    { 0xc2f964b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d7964b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81964b9, SUC_lexicon_tags_1 },
    { 0x1a5964ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81f964bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f964bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead964c3, SUC_lexicon_tags_275 },
    { 0x267964c4, SUC_lexicon_tags_2 },
    { 0xa31964c3, SUC_lexicon_tags_87 },
    { 0xaf3964c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9964cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db964d3, SUC_lexicon_tags_653 },
    { 0x5e9964d4, SUC_lexicon_tags_30 },
    { 0xced964d5, SUC_lexicon_tags_60 },
    { 0xb29964d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef964d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x211964dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x773964df, SUC_lexicon_tags_79 },
    { 0xa2f964e0, SUC_lexicon_tags_17 },
    { 0xdef964e0, SUC_lexicon_tags_47 },
    { 0xa19964e2, SUC_lexicon_tags_43 },
    { 0x811964df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc7964e5, SUC_lexicon_tags_17 },
    { 0x19d964e6, SUC_lexicon_tags_12 },
    { 0x79d964e6, SUC_lexicon_tags_43 },
    { 0xbd964e5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x90d964ea, SUC_lexicon_tags_68 },
    { 0x34f964eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21964f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85964f6, SUC_lexicon_tags_15 },
    { 0xb63964f7, SUC_lexicon_tags_37 },
    { 0x4b5964f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x753964fb, SUC_lexicon_tags_27 },
    { 0xfe1964fb, SUC_lexicon_tags_0 },
    { 0xa6f964fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x295964ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9196503, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e596509, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b9650c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b96512, SUC_lexicon_tags_75 },
    { 0xf8b96513, SUC_lexicon_tags_11 },
    { 0xa9996512, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x55796516, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc979651b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8cb9651d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f196525, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b96528, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd9652b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3999652f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3ed96531, SUC_lexicon_tags_9 },
    { 0xab596532, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf596535, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3bf96537, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2899653b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa6f9653d, SUC_lexicon_tags_23 },
    { 0x3c99653e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd96547, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4b96549, SUC_lexicon_tags_14 },
    { 0x93396549, SUC_lexicon_tags_37 },
    { 0x69b9654a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbf79654d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4f19654f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af96552, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdcf96554, SUC_lexicon_tags_33 },
    { 0x4a796554, SUC_lexicon_tags_26 },
    { 0x80996556, SUC_lexicon_tags_13 },
    { 0xda796557, SUC_lexicon_tags_48 },
    { 0xaa596556, SUC_lexicon_tags_2 },
    { 0xad596554, SUC_lexicon_tags_25 },
    { 0xc1196559, SUC_lexicon_tags_39 },
    { 0x58796556, SUC_lexicon_tags_38 },
    { 0xba19655c, SUC_lexicon_tags_12 },
    { 0x2e19655a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x93596560, SUC_lexicon_tags_66 },
    { 0x7b96561, SUC_lexicon_tags_12 },
    { 0x71196562, SUC_lexicon_tags_27 },
    { 0x85396563, SUC_lexicon_tags_82 },
    { 0xc0d96561, SUC_lexicon_tags_10 },
    { 0xc6996565, SUC_lexicon_tags_2 },
    { 0xaa996564, SUC_lexicon_tags_7 },
    { 0xd2d96561, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x51196569, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a39656c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xcaf9656e, SUC_lexicon_tags_6 },
    { 0x4f99656f, SUC_lexicon_tags_39 },
    { 0xf1d96570, SUC_lexicon_tags_40 },
    { 0xdc396571, SUC_lexicon_tags_17 },
    { 0x26796572, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd6196574, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8f396576, SUC_lexicon_tags_43 },
    { 0xd9796577, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6359657c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf396584, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2796587, SUC_lexicon_tags_37 },
    { 0x85996588, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9d99658a, SUC_lexicon_tags_37 },
    { 0xe619658b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7196590, SUC_lexicon_tags_26 },
    { 0x88596591, SUC_lexicon_tags_49 },
    { 0xef96592, SUC_lexicon_tags_33 },
    { 0x37796592, SUC_lexicon_tags_25 },
    { 0xf8f96593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53796596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a596599, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bf9659b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf359659f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03965a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cf965a8, SUC_lexicon_tags_2 },
    { 0x541965a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209965b6, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361965ba, SUC_lexicon_tags_2 },
    { 0x49b965ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f5965bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x591965c0, SUC_lexicon_tags_11 },
    { 0xc1965c1, SUC_lexicon_tags_7 },
    { 0x29f965c1, SUC_lexicon_tags_39 },
    { 0x1a9965c0, SUC_lexicon_tags_17 },
    { 0x99965c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x64b965c6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9965cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345965cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cf965d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd965d4, SUC_lexicon_tags_19 },
    { 0xaf3965d4, SUC_lexicon_tags_14 },
    { 0x1ab965d6, SUC_lexicon_tags_9 },
    { 0x833965d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5965dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1965df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b965e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa11965e3, SUC_lexicon_tags_47 },
    { 0xae9965e3, SUC_lexicon_tags_68 },
    { 0xa2d965e5, SUC_lexicon_tags_4 },
    { 0x3f965e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x35b965e8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4c9965ea, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7965ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee9965f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d3965f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe3f965f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x737965f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x36f965f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca5965fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7965fe, SUC_lexicon_tags_37 },
    { 0xaad965fe, SUC_lexicon_tags_7 },
    { 0x1f7965fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2796603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b96608, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8559660a, SUC_lexicon_tags_77 },
    { 0x5299660b, SUC_lexicon_tags_160 },
    { 0x79b9660b, SUC_lexicon_tags_17 },
    { 0x1cb9660c, SUC_lexicon_tags_38 },
    { 0xfcb9660e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf196610, SUC_lexicon_tags_9 },
    { 0xd3596611, SUC_lexicon_tags_7 },
    { 0xf3f96612, SUC_lexicon_tags_19 },
    { 0x86d96610, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b996616, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xb0d96618, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd9661b, SUC_lexicon_tags_37 },
    { 0xbe59661c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9639661e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a396621, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6996629, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f9662e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5db96630, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x19196632, SUC_lexicon_tags_45 },
    { 0x77796632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad96637, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc39663b, SUC_lexicon_tags_1 },
    { 0x6099663b, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0xff39663f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b96647, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b9664c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15596653, SUC_lexicon_tags_17 },
    { 0xbed96654, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6196656, SUC_lexicon_tags_32 },
    { 0xb596657, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad9665f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc359666a, SUC_lexicon_tags_60 },
    { 0x979666b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa39666e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7796677, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8796679, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a99667d, SUC_lexicon_tags_7 },
    { 0xc4b9667e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7396686, SUC_lexicon_tags_2 },
    { 0x25d96687, SUC_lexicon_tags_19 },
    { 0x5b96688, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xddb9668b, SUC_lexicon_tags_15 },
    { 0xb599668c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9196693, SUC_lexicon_tags_7 },
    { 0x60596693, SUC_lexicon_tags_25 },
    { 0xef796695, SUC_lexicon_tags_1 },
    { 0xb8996695, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd279669d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1966a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3966a6, SUC_lexicon_tags_38 },
    { 0x7b7966a7, SUC_lexicon_tags_8 },
    { 0xe5f966a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89966ac, SUC_lexicon_tags_15 },
    { 0xb99966ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9966b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd966c0, SUC_lexicon_tags_12 },
    { 0xcf5966c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd87966c3, SUC_lexicon_tags_15 },
    { 0x249966c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3966c8, SUC_lexicon_tags_1 },
    { 0xf43966c9, SUC_lexicon_tags_11 },
    { 0xce5966c9, SUC_lexicon_tags_0 },
    { 0xbc7966ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519966d1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc9f966d3, SUC_lexicon_tags_47 },
    { 0x7e3966d4, SUC_lexicon_tags_47 },
    { 0xde9966d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x327966d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f966db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9af966dd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x431966df, SUC_lexicon_tags_25 },
    { 0xbcd966e0, SUC_lexicon_tags_2 },
    { 0x53d966e0, SUC_lexicon_tags_8 },
    { 0x13b966df, SUC_lexicon_tags_37 },
    { 0x5f3966e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f966ea, SUC_lexicon_tags_2 },
    { 0x26f966eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7cf966ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05966f2, SUC_lexicon_tags_1 },
    { 0x709966f2, SUC_lexicon_tags_8 },
    { 0x945966f4, SUC_lexicon_tags_15 },
    { 0x9966f5, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f966fb, SUC_lexicon_tags_555 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81396702, SUC_lexicon_tags_33 },
    { 0x87996703, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b96708, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3339670a, SUC_lexicon_tags_2 },
    { 0x1639670b, SUC_lexicon_tags_1 },
    { 0x7d39670b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb379670e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa9396710, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72196712, SUC_lexicon_tags_29 },
    { 0xf7996713, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x94796715, SUC_lexicon_tags_37 },
    { 0xf4396716, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d9671a, SUC_lexicon_tags_1 },
    { 0xc339671b, SUC_lexicon_tags_15 },
    { 0xcf79671c, SUC_lexicon_tags_29 },
    { 0x74d9671b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xf659671f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd96724, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b396727, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8839672f, SUC_lexicon_tags_0 },
    { 0xafb96730, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f96733, SUC_lexicon_tags_7 },
    { 0x6a996734, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9996739, SUC_lexicon_tags_1 },
    { 0xdf99673a, SUC_lexicon_tags_15 },
    { 0xd599673a, SUC_lexicon_tags_17 },
    { 0x55796739, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x27d9673f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d596742, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1396745, SUC_lexicon_tags_12 },
    { 0x6796746, SUC_lexicon_tags_77 },
    { 0xcf796746, SUC_lexicon_tags_11 },
    { 0x2c596748, SUC_lexicon_tags_17 },
    { 0xcff96748, SUC_lexicon_tags_4 },
    { 0xf539674a, SUC_lexicon_tags_37 },
    { 0xef19674b, SUC_lexicon_tags_6 },
    { 0xcb79674a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b96751, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3996755, SUC_lexicon_tags_6 },
    { 0x10d96756, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40b96759, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc939675b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6199675e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4df96760, SUC_lexicon_tags_12 },
    { 0x57996761, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d796766, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9676a, SUC_lexicon_tags_12 },
    { 0xa099676b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd9676d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd96779, SUC_lexicon_tags_0 },
    { 0xd6f96779, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xecf9677c, SUC_lexicon_tags_7 },
    { 0x6959677c, SUC_lexicon_tags_63 },
    { 0x79b9677e, SUC_lexicon_tags_16 },
    { 0xec39677e, SUC_lexicon_tags_19 },
    { 0x9d59677d, SUC_lexicon_tags_0 },
    { 0x90d9677c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb196784, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xffd96787, SUC_lexicon_tags_15 },
    { 0x16996787, SUC_lexicon_tags_7 },
    { 0xfc196789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccb9678c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4219678f, SUC_lexicon_tags_31 },
    { 0x9279678f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2396793, SUC_lexicon_tags_11 },
    { 0xcf996794, SUC_lexicon_tags_11 },
    { 0xd196795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6759679a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7e79679c, SUC_lexicon_tags_19 },
    { 0x4479679c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d967a0, SUC_lexicon_tags_15 },
    { 0xb9967a0, SUC_lexicon_tags_14 },
    { 0x913967a2, SUC_lexicon_tags_1 },
    { 0x1b967a3, SUC_lexicon_tags_60 },
    { 0x987967a2, SUC_lexicon_tags_2 },
    { 0xb77967a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe65967a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1967ab, SUC_lexicon_tags_25 },
    { 0x9b3967ac, SUC_lexicon_tags_27 },
    { 0x43d967ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d967b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x411967b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x637967b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff967ba, SUC_lexicon_tags_25 },
    { 0xe2f967bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb967bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ab967c1, SUC_lexicon_tags_10 },
    { 0xbd5967c1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457967c9, SUC_lexicon_tags_25 },
    { 0x1bd967ca, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9967cd, SUC_lexicon_tags_15 },
    { 0xfef967ce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x389967d1, SUC_lexicon_tags_63 },
    { 0x87b967d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927967d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d967db, SUC_lexicon_tags_1 },
    { 0x299967dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229967e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37967e9, SUC_lexicon_tags_37 },
    { 0xd7967e9, SUC_lexicon_tags_13 },
    { 0x8ef967eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd9967ed, SUC_lexicon_tags_68 },
    { 0x38f967ee, SUC_lexicon_tags_77 },
    { 0x603967ef, SUC_lexicon_tags_14 },
    { 0x8ad967ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x511967f2, SUC_lexicon_tags_43 },
    { 0x5e3967f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d967f7, SUC_lexicon_tags_26 },
    { 0x645967f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5967fb, SUC_lexicon_tags_27 },
    { 0xa15967fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d96800, SUC_lexicon_tags_25 },
    { 0xd9b96801, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9396806, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x71196808, SUC_lexicon_tags_60 },
    { 0x5a796809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb9680e, SUC_lexicon_tags_25 },
    { 0xe939680f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f796812, SUC_lexicon_tags_33 },
    { 0xa1596813, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa119681b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f9681f, SUC_lexicon_tags_47 },
    { 0x33796820, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f96827, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb9682b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8559682f, SUC_lexicon_tags_37 },
    { 0x3ed9682f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1796838, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28b9683a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8396844, SUC_lexicon_tags_92 },
    { 0x4596844, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ab96847, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e79684a, SUC_lexicon_tags_2 },
    { 0xdc79684b, SUC_lexicon_tags_27 },
    { 0x8259684c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9796851, SUC_lexicon_tags_51 },
    { 0xceb96852, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73d96854, SUC_lexicon_tags_13 },
    { 0xfa796854, SUC_lexicon_tags_1 },
    { 0xcb196854, SUC_lexicon_tags_1 },
    { 0x8db96857, SUC_lexicon_tags_26 },
    { 0xb1d96854, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa779685b, SUC_lexicon_tags_37 },
    { 0xa0d9685c, SUC_lexicon_tags_8 },
    { 0x2179685d, SUC_lexicon_tags_3 },
    { 0x2ef9685c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x24396860, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x94f96862, SUC_lexicon_tags_15 },
    { 0xf0b96862, SUC_lexicon_tags_37 },
    { 0x2cb96864, SUC_lexicon_tags_15 },
    { 0x1b596864, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9686c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4299686f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d96873, SUC_lexicon_tags_33 },
    { 0x25196874, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f596876, SUC_lexicon_tags_31 },
    { 0x11596877, SUC_lexicon_tags_25 },
    { 0xae596878, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7ef9687a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9687f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb996887, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb9688c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39688f, SUC_lexicon_tags_60 },
    { 0xab996890, SUC_lexicon_tags_63 },
    { 0x25f96890, SUC_lexicon_tags_12 },
    { 0x84d96892, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81196894, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55996896, SUC_lexicon_tags_15 },
    { 0xa5396897, SUC_lexicon_tags_2 },
    { 0x67d96896, SUC_lexicon_tags_37 },
    { 0x1d996896, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x51d9689c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd9689f, SUC_lexicon_tags_17 },
    { 0xae3968a0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x88f968a3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x17f968a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21968a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1ab968ab, SUC_lexicon_tags_7 },
    { 0xb5968ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f968b5, SUC_lexicon_tags_12 },
    { 0xa29968b6, SUC_lexicon_tags_74 },
    { 0x1bb968b7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5968bc, SUC_lexicon_tags_23 },
    { 0xad3968bd, SUC_lexicon_tags_11 },
    { 0x3b5968be, SUC_lexicon_tags_31 },
    { 0x9ff968bf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe6f968c1, SUC_lexicon_tags_31 },
    { 0x9cf968c2, SUC_lexicon_tags_7 },
    { 0x993968c1, SUC_lexicon_tags_37 },
    { 0xbbd968c3, SUC_lexicon_tags_9 },
    { 0x997968c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d968ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd968cd, SUC_lexicon_tags_17 },
    { 0xaa9968ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54d968d3, SUC_lexicon_tags_27 },
    { 0xa99968d4, SUC_lexicon_tags_17 },
    { 0x683968d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x581968d8, SUC_lexicon_tags_1 },
    { 0x451968d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5ad968db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x17b968de, SUC_lexicon_tags_3 },
    { 0x42f968de, SUC_lexicon_tags_12 },
    { 0xb47968de, SUC_lexicon_tags_2 },
    { 0xd9d968de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83968e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f968ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521968f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafb968f3, SUC_lexicon_tags_26 },
    { 0x6b9968f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb968f9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b9690b, SUC_lexicon_tags_11 },
    { 0xc6b9690c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad396911, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96596913, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7d196915, SUC_lexicon_tags_53 },
    { 0x6a996915, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5196919, SUC_lexicon_tags_16 },
    { 0xdfb9691a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ab9691c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5939691f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87396922, SUC_lexicon_tags_11 },
    { 0xb0b96923, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf9692b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae596930, SUC_lexicon_tags_27 },
    { 0x50f96931, SUC_lexicon_tags_17 },
    { 0xc9196931, SUC_lexicon_tags_1 },
    { 0x1a996930, SUC_lexicon_tags_60 },
    { 0x10396934, SUC_lexicon_tags_28 },
    { 0x1af96932, SUC_lexicon_tags_47 },
    { 0x41996930, SUC_lexicon_tags_47 },
    { 0xd4396937, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f96940, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91f96942, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76996944, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdbf96946, SUC_lexicon_tags_15 },
    { 0xba596947, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff9694a, SUC_lexicon_tags_63 },
    { 0xdb79694b, SUC_lexicon_tags_65 },
    { 0xdab9694c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d39694e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x21596950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad96954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f96959, SUC_lexicon_tags_25 },
    { 0xa539695a, SUC_lexicon_tags_1 },
    { 0x5796959, SUC_lexicon_tags_65 },
    { 0x6db9695c, SUC_lexicon_tags_15 },
    { 0x5679695a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfff9695f, SUC_lexicon_tags_7 },
    { 0xfc396960, SUC_lexicon_tags_12 },
    { 0xe8f96961, SUC_lexicon_tags_32 },
    { 0x1d996962, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d396968, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8119696a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f9696e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd796971, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x67796973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db96977, SUC_lexicon_tags_23 },
    { 0x9b96978, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3799697a, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19996983, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6179698c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7596990, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5996992, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff96998, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe619699a, SUC_lexicon_tags_17 },
    { 0xdfd9699b, SUC_lexicon_tags_17 },
    { 0x9339699a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5079699f, SUC_lexicon_tags_17 },
    { 0x899699f, SUC_lexicon_tags_38 },
    { 0x8e19699f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419969a6, SUC_lexicon_tags_37 },
    { 0x80d969a6, SUC_lexicon_tags_18 },
    { 0x887969a8, SUC_lexicon_tags_33 },
    { 0x8f5969a9, SUC_lexicon_tags_5 },
    { 0x203969aa, SUC_lexicon_tags_17 },
    { 0xb87969ab, SUC_lexicon_tags_37 },
    { 0x4d9969ac, SUC_lexicon_tags_17 },
    { 0x1f7969a7, SUC_lexicon_tags_7 },
    { 0x15f969ae, SUC_lexicon_tags_49 },
    { 0x3bd969af, SUC_lexicon_tags_43 },
    { 0x4e5969b0, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07969b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef969b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1bf969b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67d969bb, SUC_lexicon_tags_23 },
    { 0x933969bc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2c9969be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5969ca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x50f969cc, SUC_lexicon_tags_33 },
    { 0x45d969cc, SUC_lexicon_tags_38 },
    { 0xda7969cc, SUC_lexicon_tags_23 },
    { 0xc3969cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591969d3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8db969d5, SUC_lexicon_tags_8 },
    { 0x6f7969d6, SUC_lexicon_tags_14 },
    { 0x745969d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xad969da, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x161969dd, SUC_lexicon_tags_47 },
    { 0x4fb969de, SUC_lexicon_tags_13 },
    { 0xd7f969dd, SUC_lexicon_tags_7 },
    { 0x2f9969e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801969eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b969f1, SUC_lexicon_tags_11 },
    { 0x87f969f2, SUC_lexicon_tags_7 },
    { 0xb41969f3, SUC_lexicon_tags_12 },
    { 0x493969f3, SUC_lexicon_tags_7 },
    { 0xbbf969f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72b969f7, SUC_lexicon_tags_0 },
    { 0xdb1969f8, SUC_lexicon_tags_15 },
    { 0x731969f9, SUC_lexicon_tags_25 },
    { 0x43f969f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x229969fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaf969fe, SUC_lexicon_tags_37 },
    { 0x731969fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25796a01, SUC_lexicon_tags_26 },
    { 0x8ff96a01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab396a07, SUC_lexicon_tags_14 },
    { 0x70b96a07, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x88196a0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac396a0e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b196a11, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1796a14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4996a18, SUC_lexicon_tags_25 },
    { 0x4d196a19, SUC_lexicon_tags_7 },
    { 0xc8796a19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6596a1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9396a1f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c796a24, SUC_lexicon_tags_63 },
    { 0x44996a25, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5db96a29, SUC_lexicon_tags_0 },
    { 0xa4d96a29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a596a2c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe9b96a2e, SUC_lexicon_tags_17 },
    { 0xf4196a2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5596a34, SUC_lexicon_tags_0 },
    { 0xcdd96a35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad96a38, SUC_lexicon_tags_37 },
    { 0x74596a39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd396a3b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa596a3d, SUC_lexicon_tags_25 },
    { 0xd6f96a3e, SUC_lexicon_tags_7 },
    { 0xbc596a3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb96a48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab396a4c, SUC_lexicon_tags_17 },
    { 0x90596a4d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d96a52, SUC_lexicon_tags_37 },
    { 0x15796a53, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b96a56, SUC_lexicon_tags_1 },
    { 0x2ab96a57, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53596a5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c996a5e, SUC_lexicon_tags_37 },
    { 0x6f996a5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7596a63, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f96a66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x25196a69, SUC_lexicon_tags_11 },
    { 0x4cb96a6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe2796a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9396a71, SUC_lexicon_tags_65 },
    { 0xc6996a71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f796a77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6b96a79, SUC_lexicon_tags_38 },
    { 0x4cf96a79, SUC_lexicon_tags_17 },
    { 0x42396a7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x44196a7d, SUC_lexicon_tags_21 },
    { 0xa1396a7e, SUC_lexicon_tags_15 },
    { 0x43d96a7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d96a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67996a84, SUC_lexicon_tags_7 },
    { 0x21396a84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42796a87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf196a89, SUC_lexicon_tags_7 },
    { 0x35196a8a, SUC_lexicon_tags_4 },
    { 0xcd596a8a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85396a90, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb96a93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f96a98, SUC_lexicon_tags_31 },
    { 0x80b96a99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83d96a9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95196a9e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b96aa1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeef96aa3, SUC_lexicon_tags_8 },
    { 0x57f96aa4, SUC_lexicon_tags_37 },
    { 0xfa596aa5, SUC_lexicon_tags_49 },
    { 0x1a996aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b96aab, SUC_lexicon_tags_4 },
    { 0xd4996aab, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf4796aae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb96ab2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0d96ab4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f96aba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b96abd, SUC_lexicon_tags_49 },
    { 0xe196abe, SUC_lexicon_tags_33 },
    { 0xec996abf, SUC_lexicon_tags_33 },
    { 0xf2396ac0, SUC_lexicon_tags_55 },
    { 0x95396abe, SUC_lexicon_tags_30 },
    { 0xc9996ac0, SUC_lexicon_tags_7 },
    { 0xd0d96abf, SUC_lexicon_tags_11 },
    { 0x9d996ac4, SUC_lexicon_tags_37 },
    { 0x32f96ac2, SUC_lexicon_tags_12 },
    { 0x6b396ac6, SUC_lexicon_tags_343 },
    { 0xf6396ac3, SUC_lexicon_tags_23 },
    { 0xa3196ac8, SUC_lexicon_tags_26 },
    { 0x9e396ac9, SUC_lexicon_tags_43 },
    { 0x26b96ac3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b996acc, SUC_lexicon_tags_29 },
    { 0x7d796acd, SUC_lexicon_tags_47 },
    { 0x60b96ace, SUC_lexicon_tags_26 },
    { 0x11f96acf, SUC_lexicon_tags_2 },
    { 0x12996ace, SUC_lexicon_tags_11 },
    { 0x41996ad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85196ad4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d96ad8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x46196adb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6396ade, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x49d96ae0, SUC_lexicon_tags_5 },
    { 0x2cd96ae1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde196ae6, SUC_lexicon_tags_12 },
    { 0x2d796ae6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f96aee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d96af9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b96afc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b96b01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x49796b04, SUC_lexicon_tags_8 },
    { 0x6ad96b05, SUC_lexicon_tags_11 },
    { 0xad596b05, SUC_lexicon_tags_60 },
    { 0xdd996b07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b396b09, SUC_lexicon_tags_19 },
    { 0x60996b0a, SUC_lexicon_tags_17 },
    { 0x2bf96b0b, SUC_lexicon_tags_17 },
    { 0x21b96b0c, SUC_lexicon_tags_37 },
    { 0x46d96b0c, SUC_lexicon_tags_26 },
    { 0x37196b0e, SUC_lexicon_tags_7 },
    { 0x20396b0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33f96b11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7396b13, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2bb96b15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b596b18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x56996b1b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x98996b1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x33996b20, SUC_lexicon_tags_55 },
    { 0x8cb96b21, SUC_lexicon_tags_1 },
    { 0xda796b22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc596b27, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbed96b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b96b31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b96b34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d96b40, SUC_lexicon_tags_11 },
    { 0xbbf96b41, SUC_lexicon_tags_62 },
    { 0xbff96b42, SUC_lexicon_tags_15 },
    { 0x75f96b41, SUC_lexicon_tags_11 },
    { 0x2ef96b40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f96b48, SUC_lexicon_tags_17 },
    { 0x1f996b49, SUC_lexicon_tags_11 },
    { 0xe9d96b4a, SUC_lexicon_tags_25 },
    { 0xe3b96b4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x75396b4e, SUC_lexicon_tags_65 },
    { 0x16396b4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4396b51, SUC_lexicon_tags_12 },
    { 0xb7396b52, SUC_lexicon_tags_11 },
    { 0x1f196b51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd96b56, SUC_lexicon_tags_28 },
    { 0x8d196b56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd96b5b, SUC_lexicon_tags_2 },
    { 0xa3396b5b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf0796b5e, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf96b62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1796b65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7396b68, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7b396b6a, SUC_lexicon_tags_37 },
    { 0x5fd96b6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36f96b72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95996b74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d96b78, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaef96b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47596b7f, SUC_lexicon_tags_19 },
    { 0x74396b7f, SUC_lexicon_tags_56 },
    { 0xd1196b81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd396b8b, SUC_lexicon_tags_37 },
    { 0x76796b8c, SUC_lexicon_tags_19 },
    { 0x1fd96b8c, SUC_lexicon_tags_49 },
    { 0x4f796b8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf96b94, SUC_lexicon_tags_17 },
    { 0xfe396b94, SUC_lexicon_tags_15 },
    { 0xe6996b95, SUC_lexicon_tags_91 },
    { 0xbbd96b96, SUC_lexicon_tags_15 },
    { 0x12796b98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc0996b9a, SUC_lexicon_tags_26 },
    { 0x13396b9b, SUC_lexicon_tags_57 },
    { 0x3c196b9b, SUC_lexicon_tags_6 },
    { 0xce396b9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3196ba3, SUC_lexicon_tags_7 },
    { 0xa6196ba4, SUC_lexicon_tags_25 },
    { 0xbe596ba5, SUC_lexicon_tags_9 },
    { 0x44196ba5, SUC_lexicon_tags_28 },
    { 0xae596ba7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1d96ba9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94b96bad, SUC_lexicon_tags_38 },
    { 0x8b96bae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x396bb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc596bb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f96bb8, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4796bbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba596bc5, SUC_lexicon_tags_12 },
    { 0x63996bc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87996bca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5796bce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e796bd0, SUC_lexicon_tags_15 },
    { 0xcf196bd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd7396bd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18396bd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8396bd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad96bdf, SUC_lexicon_tags_12 },
    { 0x43196be0, SUC_lexicon_tags_49 },
    { 0xc1796be1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99596be7, SUC_lexicon_tags_8 },
    { 0xeb96be8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b96bec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f96bf4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90196bf9, SUC_lexicon_tags_15 },
    { 0x13996bfa, SUC_lexicon_tags_26 },
    { 0x45396bfb, SUC_lexicon_tags_15 },
    { 0x5ad96bfc, SUC_lexicon_tags_11 },
    { 0x79396bfc, SUC_lexicon_tags_2 },
    { 0xa2796bfb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d396c02, SUC_lexicon_tags_15 },
    { 0x49796c03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b96c06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69996c0b, SUC_lexicon_tags_27 },
    { 0x85196c0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3796c10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b96c15, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa9f96c17, SUC_lexicon_tags_37 },
    { 0x21196c17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9f96c1a, SUC_lexicon_tags_47 },
    { 0xda996c1b, SUC_lexicon_tags_7 },
    { 0xa9f96c1a, SUC_lexicon_tags_12 },
    { 0xb2d96c1d, SUC_lexicon_tags_84 },
    { 0x3c396c1e, SUC_lexicon_tags_14 },
    { 0xf196c1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78596c21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36d96c23, SUC_lexicon_tags_47 },
    { 0xd9f96c23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x79996c27, SUC_lexicon_tags_45 },
    { 0x65b96c27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e596c2b, SUC_lexicon_tags_14 },
    { 0xce796c2c, SUC_lexicon_tags_16 },
    { 0xa3796c2d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f96c35, SUC_lexicon_tags_17 },
    { 0x29f96c36, SUC_lexicon_tags_23 },
    { 0xe0796c35, SUC_lexicon_tags_12 },
    { 0x6d996c35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe196c3a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x75996c3c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa0796c3e, SUC_lexicon_tags_11 },
    { 0x21396c3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77996c46, SUC_lexicon_tags_23 },
    { 0xc2b96c47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77396c49, SUC_lexicon_tags_56 },
    { 0xa1196c49, SUC_lexicon_tags_37 },
    { 0x27996c4b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd96c50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf196c56, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5b796c58, SUC_lexicon_tags_12 },
    { 0xcef96c58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d96c5e, SUC_lexicon_tags_175 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1996c61, SUC_lexicon_tags_1 },
    { 0x79596c62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1196c67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0996c6b, SUC_lexicon_tags_60 },
    { 0xe0d96c6c, SUC_lexicon_tags_37 },
    { 0xcd996c6c, SUC_lexicon_tags_33 },
    { 0x78596c6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa796c70, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c796c73, SUC_lexicon_tags_439 },
    { 0xd5d96c74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x74796c77, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xde596c79, SUC_lexicon_tags_9 },
    { 0x3fb96c7a, SUC_lexicon_tags_25 },
    { 0x18b96c7b, SUC_lexicon_tags_16 },
    { 0x4a996c7c, SUC_lexicon_tags_38 },
    { 0x89796c7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf96c80, SUC_lexicon_tags_30 },
    { 0x41b96c80, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f96c86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0796c88, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x92f96c8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18196c90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x19996c93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b96c96, SUC_lexicon_tags_14 },
    { 0x98d96c97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5796c9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc8b96c9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37996ca2, SUC_lexicon_tags_25 },
    { 0x54996ca2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a196ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79796ca8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0396cad, SUC_lexicon_tags_6 },
    { 0x38996cae, SUC_lexicon_tags_64 },
    { 0x49f96cae, SUC_lexicon_tags_12 },
    { 0x12596caf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfb596cb2, SUC_lexicon_tags_63 },
    { 0x2ed96cb3, SUC_lexicon_tags_31 },
    { 0xff796cb2, SUC_lexicon_tags_2 },
    { 0xf0f96cb5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32196cba, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90f96cbe, SUC_lexicon_tags_1 },
    { 0x32d96cbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53196cc4, SUC_lexicon_tags_27 },
    { 0x29d96cc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d996cca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70396cce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e996cd1, SUC_lexicon_tags_30 },
    { 0xeb196cd2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ab96cd5, SUC_lexicon_tags_4 },
    { 0x90b96cd6, SUC_lexicon_tags_1 },
    { 0x5f996cd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27196cdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92196ce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d796ce4, SUC_lexicon_tags_26 },
    { 0xa0396ce5, SUC_lexicon_tags_66 },
    { 0xcc196ce4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41196ceb, SUC_lexicon_tags_37 },
    { 0x79196cec, SUC_lexicon_tags_1 },
    { 0x22796cec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x11796cf0, SUC_lexicon_tags_38 },
    { 0xe0396cf1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d96cf4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f196cf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35796cfa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x47196cfc, SUC_lexicon_tags_63 },
    { 0xd6596cfd, SUC_lexicon_tags_17 },
    { 0x5dd96cfd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e796d02, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe9196d04, SUC_lexicon_tags_28 },
    { 0x98b96d04, SUC_lexicon_tags_11 },
    { 0x87596d05, SUC_lexicon_tags_7 },
    { 0x18d96d07, SUC_lexicon_tags_49 },
    { 0xf8396d07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b96d0b, SUC_lexicon_tags_17 },
    { 0xf1196d0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14b96d10, SUC_lexicon_tags_17 },
    { 0x7af96d11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac596d13, SUC_lexicon_tags_41 },
    { 0xc8b96d14, SUC_lexicon_tags_37 },
    { 0xcf196d14, SUC_lexicon_tags_7 },
    { 0x78396d15, SUC_lexicon_tags_12 },
    { 0xd9f96d15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9196d19, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x57396d1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x27396d1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f996d23, SUC_lexicon_tags_25 },
    { 0x80f96d23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6796d2a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f796d2e, SUC_lexicon_tags_17 },
    { 0x76f96d2f, SUC_lexicon_tags_61 },
    { 0x3a196d30, SUC_lexicon_tags_25 },
    { 0x79596d31, SUC_lexicon_tags_32 },
    { 0x8f596d2e, SUC_lexicon_tags_57 },
    { 0xab96d33, SUC_lexicon_tags_1 },
    { 0x31796d34, SUC_lexicon_tags_47 },
    { 0xc3d96d35, SUC_lexicon_tags_23 },
    { 0xd0796d33, SUC_lexicon_tags_37 },
    { 0xb0596d37, SUC_lexicon_tags_38 },
    { 0x28596d38, SUC_lexicon_tags_30 },
    { 0xe8396d32, SUC_lexicon_tags_49 },
    { 0xc0396d2e, SUC_lexicon_tags_37 },
    { 0xc1f96d3b, SUC_lexicon_tags_16 },
    { 0x22f96d3b, SUC_lexicon_tags_15 },
    { 0xbb96d3b, SUC_lexicon_tags_107 },
    { 0x72f96d3e, SUC_lexicon_tags_16 },
    { 0xd3796d3e, SUC_lexicon_tags_39 },
    { 0x6f596d3e, SUC_lexicon_tags_47 },
    { 0x9f96d3c, SUC_lexicon_tags_34 },
    { 0xde596d3d, SUC_lexicon_tags_37 },
    { 0x8996d43, SUC_lexicon_tags_10 },
    { 0xa9396d44, SUC_lexicon_tags_17 },
    { 0x3b796d3b, SUC_lexicon_tags_11 },
    { 0xdad96d46, SUC_lexicon_tags_14 },
    { 0xd7d96d46, SUC_lexicon_tags_25 },
    { 0xcd996d48, SUC_lexicon_tags_39 },
    { 0x2c596d44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9196d4c, SUC_lexicon_tags_15 },
    { 0xd6596d4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35996d50, SUC_lexicon_tags_28 },
    { 0x5e196d51, SUC_lexicon_tags_12 },
    { 0x90996d52, SUC_lexicon_tags_15 },
    { 0xb0f96d53, SUC_lexicon_tags_15 },
    { 0xfa396d52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd396d58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2396d5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd96d61, SUC_lexicon_tags_25 },
    { 0xe7d96d61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b96d66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb96d69, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf2f96d6b, SUC_lexicon_tags_8 },
    { 0xf7d96d6c, SUC_lexicon_tags_11 },
    { 0x24f96d6b, SUC_lexicon_tags_8 },
    { 0xd0596d6e, SUC_lexicon_tags_25 },
    { 0x3796d6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x59196d71, SUC_lexicon_tags_19 },
    { 0x68396d72, SUC_lexicon_tags_7 },
    { 0x94d96d73, SUC_lexicon_tags_63 },
    { 0x42996d73, SUC_lexicon_tags_7 },
    { 0x84996d71, SUC_lexicon_tags_85 },
    { 0xf9596d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d96d7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b396d80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58f96d82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd7f96d84, SUC_lexicon_tags_14 },
    { 0x4c196d85, SUC_lexicon_tags_39 },
    { 0x2bd96d85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f996d8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60d96d8c, SUC_lexicon_tags_63 },
    { 0x8b596d8d, SUC_lexicon_tags_2 },
    { 0xc6196d8d, SUC_lexicon_tags_7 },
    { 0x37196d8f, SUC_lexicon_tags_3 },
    { 0x85796d8f, SUC_lexicon_tags_43 },
    { 0x12196d8d, SUC_lexicon_tags_1689 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee796d95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4996d99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1af96d9b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd3196d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99196d9f, SUC_lexicon_tags_17 },
    { 0x2a596d9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8996da5, SUC_lexicon_tags_9 },
    { 0xd3d96da5, SUC_lexicon_tags_15 },
    { 0xd2f96da7, SUC_lexicon_tags_27 },
    { 0x2cd96da8, SUC_lexicon_tags_2 },
    { 0x34196da8, SUC_lexicon_tags_7 },
    { 0xbd396daa, SUC_lexicon_tags_15 },
    { 0xc4d96da7, SUC_lexicon_tags_1 },
    { 0xd5996dac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36796daf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb996db3, SUC_lexicon_tags_25 },
    { 0x17d96db4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf2196db6, SUC_lexicon_tags_49 },
    { 0x3d396db6, SUC_lexicon_tags_29 },
    { 0x996db7, SUC_lexicon_tags_60 },
    { 0xeb196db9, SUC_lexicon_tags_118 },
    { 0xd1596db7, SUC_lexicon_tags_39 },
    { 0x89196db7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4196dbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a596dc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6596dcf, SUC_lexicon_tags_63 },
    { 0x28596dcf, SUC_lexicon_tags_23 },
    { 0xc2196dd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b796dd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18196dd8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x61596ddb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23196de0, SUC_lexicon_tags_17 },
    { 0xa4796de0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf596de4, SUC_lexicon_tags_14 },
    { 0x58f96de5, SUC_lexicon_tags_11 },
    { 0x19f96de6, SUC_lexicon_tags_138 },
    { 0x4fb96de7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf596dea, SUC_lexicon_tags_25 },
    { 0x42f96deb, SUC_lexicon_tags_47 },
    { 0xc2f96dea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f96df0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc196df3, SUC_lexicon_tags_151 },
    { 0x53196df4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d96dfa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x73396dfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11796e00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d96e06, SUC_lexicon_tags_34 },
    { 0x6cb96e06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b96e0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9b96e0e, SUC_lexicon_tags_23 },
    { 0x97196e0e, SUC_lexicon_tags_17 },
    { 0x1196e10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f396e16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd1796e18, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x28396e1b, SUC_lexicon_tags_23 },
    { 0x7d796e1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd96e22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd3796e24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdb96e26, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f196e2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5596e2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab196e30, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xabb96e32, SUC_lexicon_tags_66 },
    { 0x74b96e32, SUC_lexicon_tags_4 },
    { 0x3a996e32, SUC_lexicon_tags_63 },
    { 0x28d96e35, SUC_lexicon_tags_45 },
    { 0x8cd96e32, SUC_lexicon_tags_12 },
    { 0xcf996e37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3596e3a, SUC_lexicon_tags_17 },
    { 0xc1b96e3b, SUC_lexicon_tags_16 },
    { 0xd6196e3a, SUC_lexicon_tags_19 },
    { 0x7b396e3d, SUC_lexicon_tags_11 },
    { 0x50b96e3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0d96e40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5f96e42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d96e45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfef96e49, SUC_lexicon_tags_13 },
    { 0x65d96e49, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a796e50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80996e67, SUC_lexicon_tags_25 },
    { 0x70596e68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f96e6b, SUC_lexicon_tags_37 },
    { 0xb2996e6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfab96e6e, SUC_lexicon_tags_63 },
    { 0x45596e6f, SUC_lexicon_tags_15 },
    { 0x7bb96e70, SUC_lexicon_tags_37 },
    { 0xe6f96e70, SUC_lexicon_tags_14 },
    { 0xec996e72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd96e75, SUC_lexicon_tags_63 },
    { 0xccb96e76, SUC_lexicon_tags_19 },
    { 0x90996e75, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd6f96e79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79196e7b, SUC_lexicon_tags_17 },
    { 0x80d96e7c, SUC_lexicon_tags_56 },
    { 0x9bd96e7c, SUC_lexicon_tags_25 },
    { 0x17b96e7c, SUC_lexicon_tags_15 },
    { 0x9f196e7d, SUC_lexicon_tags_47 },
    { 0x3f796e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77396e82, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab96e85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1996e87, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xefb96e89, SUC_lexicon_tags_19 },
    { 0xdfd96e89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa996e8c, SUC_lexicon_tags_45 },
    { 0x3b996e8d, SUC_lexicon_tags_784 },
    { 0x49d96e8c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x64196e90, SUC_lexicon_tags_27 },
    { 0x4a596e91, SUC_lexicon_tags_33 },
    { 0xda396e91, SUC_lexicon_tags_4 },
    { 0x84b96e93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b96e96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9996e99, SUC_lexicon_tags_15 },
    { 0x13b96e9a, SUC_lexicon_tags_0 },
    { 0x71796e99, SUC_lexicon_tags_15 },
    { 0xbb796e9c, SUC_lexicon_tags_0 },
    { 0x95d96e9c, SUC_lexicon_tags_1005 },
    { 0x9df96e9d, SUC_lexicon_tags_8 },
    { 0xcf396e9f, SUC_lexicon_tags_8 },
    { 0xf9996e9f, SUC_lexicon_tags_47 },
    { 0x15f96ea1, SUC_lexicon_tags_37 },
    { 0x88996ea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f96ea8, SUC_lexicon_tags_70 },
    { 0xeb96ea9, SUC_lexicon_tags_31 },
    { 0x66f96eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37996eb1, SUC_lexicon_tags_74 },
    { 0xeb796eb2, SUC_lexicon_tags_26 },
    { 0x79196eb1, SUC_lexicon_tags_30 },
    { 0xd8196eb4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc8f96eb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d96ebd, SUC_lexicon_tags_88 },
    { 0xc7996ebe, SUC_lexicon_tags_15 },
    { 0xd0796ebe, SUC_lexicon_tags_37 },
    { 0xd4b96ebd, SUC_lexicon_tags_37 },
    { 0x95596ec1, SUC_lexicon_tags_1 },
    { 0xf2f96ebe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b96ec6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b96eca, SUC_lexicon_tags_107 },
    { 0x63996eca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb96ed4, SUC_lexicon_tags_47 },
    { 0x82996ed5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48196ed7, SUC_lexicon_tags_53 },
    { 0xc0996ed8, SUC_lexicon_tags_15 },
    { 0xb0196ed7, SUC_lexicon_tags_25 },
    { 0xe6196eda, SUC_lexicon_tags_21 },
    { 0xb9b96eda, SUC_lexicon_tags_1456 },
    { 0x95196ed9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8196ee0, SUC_lexicon_tags_1 },
    { 0x7396ee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63d96ee3, SUC_lexicon_tags_60 },
    { 0xde996ee4, SUC_lexicon_tags_56 },
    { 0xde196ee5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x21396ee7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d96eea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd96eed, SUC_lexicon_tags_15 },
    { 0xf9b96eee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85f96ef0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f96ef4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d96ef9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5396efe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b96f01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f96f07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea596f0e, SUC_lexicon_tags_17 },
    { 0xc5596f0f, SUC_lexicon_tags_19 },
    { 0xde196f0f, SUC_lexicon_tags_37 },
    { 0xe996f10, SUC_lexicon_tags_11 },
    { 0x5f396f12, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3e796f14, SUC_lexicon_tags_14 },
    { 0x98596f15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6196f1b, SUC_lexicon_tags_37 },
    { 0xb2396f1c, SUC_lexicon_tags_28 },
    { 0x1af96f1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2596f20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x94596f23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a596f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99996f2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55796f31, SUC_lexicon_tags_47 },
    { 0x2b196f32, SUC_lexicon_tags_23 },
    { 0x1d996f31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f96f39, SUC_lexicon_tags_19 },
    { 0x5b396f3a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x13996f3d, SUC_lexicon_tags_18 },
    { 0xbe596f3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9796f40, SUC_lexicon_tags_17 },
    { 0x5996f41, SUC_lexicon_tags_29 },
    { 0x34f96f41, SUC_lexicon_tags_40 },
    { 0x27796f43, SUC_lexicon_tags_0 },
    { 0x54196f44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f96f47, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50196f4c, SUC_lexicon_tags_0 },
    { 0x2eb96f4d, SUC_lexicon_tags_1 },
    { 0x3f596f4d, SUC_lexicon_tags_21 },
    { 0xac396f4e, SUC_lexicon_tags_17 },
    { 0xfd396f50, SUC_lexicon_tags_15 },
    { 0x81996f50, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x5c196f53, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d96f58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa9396f5a, SUC_lexicon_tags_11 },
    { 0xbdd96f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41796f5d, SUC_lexicon_tags_1 },
    { 0x77f96f5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ad96f60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54796f65, SUC_lexicon_tags_43 },
    { 0xd2d96f66, SUC_lexicon_tags_60 },
    { 0xb7d96f66, SUC_lexicon_tags_9 },
    { 0xf7d96f68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd3196f6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d96f70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c996f73, SUC_lexicon_tags_47 },
    { 0x15b96f73, SUC_lexicon_tags_1 },
    { 0xbd796f75, SUC_lexicon_tags_25 },
    { 0x69596f75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74196f78, SUC_lexicon_tags_835 },
    { 0, NULL },
    { 0x45f96f7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee396f7c, SUC_lexicon_tags_68 },
    { 0x45396f7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x95796f7f, SUC_lexicon_tags_17 },
    { 0x18396f80, SUC_lexicon_tags_91 },
    { 0x5ef96f81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71196f83, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x32796f86, SUC_lexicon_tags_25 },
    { 0xce196f87, SUC_lexicon_tags_9 },
    { 0xea796f86, SUC_lexicon_tags_46 },
    { 0x60f96f88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2396f91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x26196f93, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4196f96, SUC_lexicon_tags_19 },
    { 0x1ff96f96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b96f9b, SUC_lexicon_tags_0 },
    { 0x48d96f9c, SUC_lexicon_tags_12 },
    { 0xa4796f9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26596fa7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd9d96fa9, SUC_lexicon_tags_39 },
    { 0x50196faa, SUC_lexicon_tags_17 },
    { 0xb7196fab, SUC_lexicon_tags_63 },
    { 0xdb596faa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6796fae, SUC_lexicon_tags_38 },
    { 0xa2796faf, SUC_lexicon_tags_26 },
    { 0xf1396fb0, SUC_lexicon_tags_37 },
    { 0xb6396fb1, SUC_lexicon_tags_116 },
    { 0x5d396fb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f96fb7, SUC_lexicon_tags_25 },
    { 0x81596fb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a196fba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d96fbf, SUC_lexicon_tags_45 },
    { 0x9596fc0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d996fc5, SUC_lexicon_tags_37 },
    { 0x1ed96fc6, SUC_lexicon_tags_17 },
    { 0xa1596fc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x83996fca, SUC_lexicon_tags_37 },
    { 0xbf196fcb, SUC_lexicon_tags_17 },
    { 0xbe196fcb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x70f96fce, SUC_lexicon_tags_31 },
    { 0x15796fcf, SUC_lexicon_tags_37 },
    { 0xc4396fce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a196fd5, SUC_lexicon_tags_12 },
    { 0x6f196fd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3196fdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f96fe3, SUC_lexicon_tags_48 },
    { 0x15d96fe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b96fe7, SUC_lexicon_tags_7 },
    { 0xccf96fe8, SUC_lexicon_tags_5 },
    { 0x65b96fe9, SUC_lexicon_tags_33 },
    { 0x7e196fea, SUC_lexicon_tags_80 },
    { 0x26996fea, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d96fee, SUC_lexicon_tags_29 },
    { 0xdd196fef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8a996ff1, SUC_lexicon_tags_11 },
    { 0xaf796ff2, SUC_lexicon_tags_0 },
    { 0xc5396ff1, SUC_lexicon_tags_25 },
    { 0xad796ff4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeaf96ff6, SUC_lexicon_tags_38 },
    { 0x6bb96ff7, SUC_lexicon_tags_28 },
    { 0x9ab96ff8, SUC_lexicon_tags_37 },
    { 0x57d96ff9, SUC_lexicon_tags_7 },
    { 0x8996ffa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7796ffd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8596fff, SUC_lexicon_tags_2 },
    { 0x7f596fff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97997002, SUC_lexicon_tags_11 },
    { 0x4a997002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c397008, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2119700b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1619700f, SUC_lexicon_tags_17 },
    { 0x3cf97010, SUC_lexicon_tags_35 },
    { 0x99997010, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8397015, SUC_lexicon_tags_15 },
    { 0xe8797016, SUC_lexicon_tags_25 },
    { 0x8f797017, SUC_lexicon_tags_15 },
    { 0x9397018, SUC_lexicon_tags_61 },
    { 0x8db97019, SUC_lexicon_tags_15 },
    { 0x89397019, SUC_lexicon_tags_17 },
    { 0xfc197019, SUC_lexicon_tags_13 },
    { 0xde79701c, SUC_lexicon_tags_2 },
    { 0x4919701a, SUC_lexicon_tags_22 },
    { 0xfe997019, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe5d97020, SUC_lexicon_tags_1 },
    { 0x51f97021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72d97023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8797027, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b9702a, SUC_lexicon_tags_35 },
    { 0xfd9702b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d59702f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57597034, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1cb97036, SUC_lexicon_tags_0 },
    { 0x2197037, SUC_lexicon_tags_59 },
    { 0xba797038, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba99703c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88397042, SUC_lexicon_tags_9 },
    { 0x9a397043, SUC_lexicon_tags_23 },
    { 0x8d997044, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x95b97046, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0997048, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8479704d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb997050, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c997052, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4797054, SUC_lexicon_tags_17 },
    { 0xf4597055, SUC_lexicon_tags_15 },
    { 0xf6f97055, SUC_lexicon_tags_37 },
    { 0x61f97057, SUC_lexicon_tags_30 },
    { 0x98197054, SUC_lexicon_tags_124 },
    { 0xb4797059, SUC_lexicon_tags_7 },
    { 0x8379705a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf6f9705c, SUC_lexicon_tags_4 },
    { 0xed39705c, SUC_lexicon_tags_8 },
    { 0xe599705e, SUC_lexicon_tags_201 },
    { 0x23d9705e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df97066, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2759706a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe397070, SUC_lexicon_tags_17 },
    { 0xa997070, SUC_lexicon_tags_5 },
    { 0xfcb97072, SUC_lexicon_tags_7 },
    { 0xf4f97073, SUC_lexicon_tags_25 },
    { 0x1cb97073, SUC_lexicon_tags_2 },
    { 0x79997075, SUC_lexicon_tags_2 },
    { 0x34797070, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b19707a, SUC_lexicon_tags_10 },
    { 0xb299707b, SUC_lexicon_tags_28 },
    { 0x7459707c, SUC_lexicon_tags_84 },
    { 0x3999707b, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f97080, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae797082, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x41797085, SUC_lexicon_tags_65 },
    { 0x8e197085, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d97089, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1259708b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb19708d, SUC_lexicon_tags_6 },
    { 0xd639708e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db97091, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb97094, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d9709a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5970a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x459970a9, SUC_lexicon_tags_4 },
    { 0xd2b970aa, SUC_lexicon_tags_47 },
    { 0x1f9970aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x427970ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a1970af, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0xb4d970b1, SUC_lexicon_tags_47 },
    { 0x9fd970b2, SUC_lexicon_tags_11 },
    { 0xe67970b2, SUC_lexicon_tags_5 },
    { 0x339970b4, SUC_lexicon_tags_43 },
    { 0xb33970b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f970b8, SUC_lexicon_tags_0 },
    { 0xd09970b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88b970bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d970bf, SUC_lexicon_tags_12 },
    { 0x6ab970c0, SUC_lexicon_tags_33 },
    { 0xbd5970c1, SUC_lexicon_tags_63 },
    { 0xb0f970c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc3d970c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa55970c6, SUC_lexicon_tags_109 },
    { 0x9e9970c7, SUC_lexicon_tags_7 },
    { 0xb4d970c8, SUC_lexicon_tags_7 },
    { 0x64f970c9, SUC_lexicon_tags_31 },
    { 0x44b970c9, SUC_lexicon_tags_14 },
    { 0xf09970cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x281970cd, SUC_lexicon_tags_12 },
    { 0x26f970ce, SUC_lexicon_tags_27 },
    { 0xd61970ce, SUC_lexicon_tags_0 },
    { 0x235970d0, SUC_lexicon_tags_149 },
    { 0x6c1970d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x187970d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc13970d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b970d8, SUC_lexicon_tags_15 },
    { 0x62d970d9, SUC_lexicon_tags_12 },
    { 0xe81970d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9970e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1970e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f970ed, SUC_lexicon_tags_30 },
    { 0xba3970ee, SUC_lexicon_tags_23 },
    { 0x1cb970ef, SUC_lexicon_tags_38 },
    { 0x68d970ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7970f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf19970f5, SUC_lexicon_tags_11 },
    { 0x449970f5, SUC_lexicon_tags_61 },
    { 0x5bd970f5, SUC_lexicon_tags_32 },
    { 0xa5970f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b970fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71970fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85197100, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x67397102, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b997107, SUC_lexicon_tags_59 },
    { 0xcc197108, SUC_lexicon_tags_31 },
    { 0xe7b97108, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2197111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50797113, SUC_lexicon_tags_39 },
    { 0x7e597114, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db97119, SUC_lexicon_tags_25 },
    { 0xa959711a, SUC_lexicon_tags_11 },
    { 0xe0f9711a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae19711f, SUC_lexicon_tags_46 },
    { 0x65b97120, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x49797122, SUC_lexicon_tags_25 },
    { 0xe1b97123, SUC_lexicon_tags_13 },
    { 0x7ef97123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81397126, SUC_lexicon_tags_2 },
    { 0x6f597127, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91597132, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d97135, SUC_lexicon_tags_31 },
    { 0xa4997136, SUC_lexicon_tags_30 },
    { 0x18b97135, SUC_lexicon_tags_17 },
    { 0x63597137, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9459713c, SUC_lexicon_tags_48 },
    { 0xf599713c, SUC_lexicon_tags_12 },
    { 0x8d9713e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff97141, SUC_lexicon_tags_30 },
    { 0x70797142, SUC_lexicon_tags_0 },
    { 0x56797143, SUC_lexicon_tags_1 },
    { 0x5e797144, SUC_lexicon_tags_10 },
    { 0x75997145, SUC_lexicon_tags_65 },
    { 0xd1197146, SUC_lexicon_tags_12 },
    { 0xfb197145, SUC_lexicon_tags_15 },
    { 0x3fd97142, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f79714b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc179714e, SUC_lexicon_tags_47 },
    { 0xc919714f, SUC_lexicon_tags_11 },
    { 0x8df9714f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87197158, SUC_lexicon_tags_43 },
    { 0xd4197158, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41597161, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0197165, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5219716a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c79716c, SUC_lexicon_tags_47 },
    { 0xf399716c, SUC_lexicon_tags_1 },
    { 0x64f9716e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f97174, SUC_lexicon_tags_11 },
    { 0x30f97174, SUC_lexicon_tags_13 },
    { 0x12f97174, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc597179, SUC_lexicon_tags_26 },
    { 0x2f797179, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6599717e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b597181, SUC_lexicon_tags_11 },
    { 0xe9197182, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b97186, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x88797188, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239718e, SUC_lexicon_tags_2 },
    { 0xea99718f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b797193, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x18397195, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad97198, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff9719b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa919719d, SUC_lexicon_tags_11 },
    { 0xa419719e, SUC_lexicon_tags_37 },
    { 0xc0f9719e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1971a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x915971a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9971ac, SUC_lexicon_tags_27 },
    { 0x8b9971ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f971b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf971b6, SUC_lexicon_tags_0 },
    { 0x5e3971b6, SUC_lexicon_tags_37 },
    { 0x7f3971b8, SUC_lexicon_tags_23 },
    { 0xdb7971b6, SUC_lexicon_tags_17 },
    { 0xb55971b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f971c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69971c8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49971cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf77971ce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1971d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7971d6, SUC_lexicon_tags_6 },
    { 0x2d7971d7, SUC_lexicon_tags_11 },
    { 0x451971d8, SUC_lexicon_tags_7 },
    { 0x99b971d9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f971dc, SUC_lexicon_tags_19 },
    { 0x6c9971dc, SUC_lexicon_tags_10 },
    { 0xaf5971de, SUC_lexicon_tags_77 },
    { 0x47b971df, SUC_lexicon_tags_49 },
    { 0xfd3971de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5971e6, SUC_lexicon_tags_60 },
    { 0x95f971e6, SUC_lexicon_tags_46 },
    { 0xfb971e8, SUC_lexicon_tags_27 },
    { 0xbe1971e9, SUC_lexicon_tags_25 },
    { 0x4e7971ea, SUC_lexicon_tags_28 },
    { 0x1b971eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d971ee, SUC_lexicon_tags_15 },
    { 0x673971ee, SUC_lexicon_tags_14 },
    { 0xc37971f0, SUC_lexicon_tags_31 },
    { 0x1e5971f1, SUC_lexicon_tags_60 },
    { 0x9d3971f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d971f5, SUC_lexicon_tags_6 },
    { 0x42f971f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff971f9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1971ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1f997201, SUC_lexicon_tags_11 },
    { 0xe0b97202, SUC_lexicon_tags_17 },
    { 0x9f797203, SUC_lexicon_tags_7 },
    { 0x43197203, SUC_lexicon_tags_54 },
    { 0x9fd97205, SUC_lexicon_tags_15 },
    { 0xc4b97206, SUC_lexicon_tags_25 },
    { 0x3ef97207, SUC_lexicon_tags_17 },
    { 0x4597208, SUC_lexicon_tags_129 },
    { 0x1e997207, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfed9720b, SUC_lexicon_tags_12 },
    { 0x74b9720c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e597211, SUC_lexicon_tags_1 },
    { 0x66f97212, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x99797215, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf97218, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e79721c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb519721f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96797223, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf4997225, SUC_lexicon_tags_17 },
    { 0xde797225, SUC_lexicon_tags_30 },
    { 0xe8797227, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9739722b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2859722d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a59722f, SUC_lexicon_tags_0 },
    { 0x9ad9722f, SUC_lexicon_tags_6 },
    { 0xd0d9722f, SUC_lexicon_tags_5 },
    { 0x2999722f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2619723b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5239723d, SUC_lexicon_tags_37 },
    { 0xcfb9723e, SUC_lexicon_tags_1 },
    { 0x9379723f, SUC_lexicon_tags_0 },
    { 0x73197240, SUC_lexicon_tags_17 },
    { 0xa7f9723d, SUC_lexicon_tags_15 },
    { 0xe8997242, SUC_lexicon_tags_25 },
    { 0x41f97242, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd797247, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90997255, SUC_lexicon_tags_16 },
    { 0xa7397256, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4119725c, SUC_lexicon_tags_31 },
    { 0xa7b9725d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b97265, SUC_lexicon_tags_12 },
    { 0x38197265, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbbb97268, SUC_lexicon_tags_12 },
    { 0xcb597269, SUC_lexicon_tags_27 },
    { 0xf8b97269, SUC_lexicon_tags_17 },
    { 0x40b9726b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b9726f, SUC_lexicon_tags_2 },
    { 0xd859726f, SUC_lexicon_tags_87 },
    { 0xf7997271, SUC_lexicon_tags_15 },
    { 0x4899726f, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d97275, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e397282, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d99728b, SUC_lexicon_tags_7 },
    { 0x75b9728c, SUC_lexicon_tags_37 },
    { 0x1779728c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf97292, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85f97294, SUC_lexicon_tags_15 },
    { 0x5a397295, SUC_lexicon_tags_29 },
    { 0xd5397296, SUC_lexicon_tags_9 },
    { 0x69397297, SUC_lexicon_tags_27 },
    { 0x5e197298, SUC_lexicon_tags_9 },
    { 0x29d97296, SUC_lexicon_tags_16 },
    { 0x26797297, SUC_lexicon_tags_390 },
    { 0xc579729b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7059729d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d972a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757972a8, SUC_lexicon_tags_25 },
    { 0x1bd972a9, SUC_lexicon_tags_8 },
    { 0xf41972aa, SUC_lexicon_tags_25 },
    { 0xa95972a8, SUC_lexicon_tags_0 },
    { 0x13d972ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b1972ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f5972b0, SUC_lexicon_tags_33 },
    { 0x991972b1, SUC_lexicon_tags_68 },
    { 0x199972b1, SUC_lexicon_tags_33 },
    { 0x513972b2, SUC_lexicon_tags_37 },
    { 0x947972b4, SUC_lexicon_tags_37 },
    { 0x73d972b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1b972b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x301972b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7972bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85972c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87f972c4, SUC_lexicon_tags_10 },
    { 0xfab972c4, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945972d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6bb972d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f972d7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x425972d9, SUC_lexicon_tags_24 },
    { 0xa69972d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33972e2, SUC_lexicon_tags_7 },
    { 0x2f7972e3, SUC_lexicon_tags_7 },
    { 0xfdb972e3, SUC_lexicon_tags_24 },
    { 0xc23972e4, SUC_lexicon_tags_7 },
    { 0x267972e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42d972e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x797972ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79972ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf6b972f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7972f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5ff972f5, SUC_lexicon_tags_14 },
    { 0x5bd972f5, SUC_lexicon_tags_27 },
    { 0x69972f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91f972f9, SUC_lexicon_tags_27 },
    { 0x295972fa, SUC_lexicon_tags_82 },
    { 0x8a7972fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x981972fd, SUC_lexicon_tags_17 },
    { 0x717972fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d797304, SUC_lexicon_tags_25 },
    { 0x6ff97305, SUC_lexicon_tags_19 },
    { 0x75b97306, SUC_lexicon_tags_30 },
    { 0x51d97307, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a797309, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe679730b, SUC_lexicon_tags_0 },
    { 0x4ed9730c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8179730f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3a797311, SUC_lexicon_tags_17 },
    { 0x17797312, SUC_lexicon_tags_2 },
    { 0x87b97311, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd979731a, SUC_lexicon_tags_49 },
    { 0xf619731b, SUC_lexicon_tags_33 },
    { 0xe019731c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5797322, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8397324, SUC_lexicon_tags_15 },
    { 0x88397324, SUC_lexicon_tags_17 },
    { 0x6b797326, SUC_lexicon_tags_19 },
    { 0xe0797324, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3899732c, SUC_lexicon_tags_0 },
    { 0xfdf9732d, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x70f9732f, SUC_lexicon_tags_47 },
    { 0x8fd97330, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f597334, SUC_lexicon_tags_17 },
    { 0x55597335, SUC_lexicon_tags_27 },
    { 0xd397336, SUC_lexicon_tags_8 },
    { 0x2d397336, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d9733d, SUC_lexicon_tags_43 },
    { 0x2699733e, SUC_lexicon_tags_16 },
    { 0x7199733f, SUC_lexicon_tags_25 },
    { 0x6219733e, SUC_lexicon_tags_91 },
    { 0x26197341, SUC_lexicon_tags_5 },
    { 0xc8797342, SUC_lexicon_tags_47 },
    { 0x20397342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23797345, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa9b97347, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0d97349, SUC_lexicon_tags_0 },
    { 0x3c797349, SUC_lexicon_tags_27 },
    { 0x9bf9734a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe639734f, SUC_lexicon_tags_12 },
    { 0xfab97350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb97354, SUC_lexicon_tags_37 },
    { 0x2ab97354, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x27f97357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba19735b, SUC_lexicon_tags_37 },
    { 0xa9b9735c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f97361, SUC_lexicon_tags_17 },
    { 0x13997361, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x51597365, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc899736c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9f9736e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d97371, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f97376, SUC_lexicon_tags_25 },
    { 0x2f397377, SUC_lexicon_tags_30 },
    { 0xa5397377, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd359737b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28d9737d, SUC_lexicon_tags_27 },
    { 0x11d9737d, SUC_lexicon_tags_11 },
    { 0x3759737f, SUC_lexicon_tags_4 },
    { 0x41197380, SUC_lexicon_tags_23 },
    { 0x2fd97380, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49d97383, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4c797385, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xdbb97387, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25b97389, SUC_lexicon_tags_25 },
    { 0x7ad97389, SUC_lexicon_tags_84 },
    { 0xd239738b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe39738d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31197390, SUC_lexicon_tags_25 },
    { 0x55997390, SUC_lexicon_tags_63 },
    { 0x53f97390, SUC_lexicon_tags_32 },
    { 0xcdd97393, SUC_lexicon_tags_19 },
    { 0x6df97390, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2797396, SUC_lexicon_tags_49 },
    { 0x69997396, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x40397399, SUC_lexicon_tags_45 },
    { 0x6c79739a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8019739e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af973a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65973ad, SUC_lexicon_tags_2 },
    { 0x19f973ad, SUC_lexicon_tags_24 },
    { 0xe9d973af, SUC_lexicon_tags_2 },
    { 0xe21973af, SUC_lexicon_tags_15 },
    { 0x85f973b1, SUC_lexicon_tags_77 },
    { 0x93b973ae, SUC_lexicon_tags_66 },
    { 0x65973b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1973b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601973ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62d973bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85973c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b973cb, SUC_lexicon_tags_56 },
    { 0x705973cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf973d1, SUC_lexicon_tags_11 },
    { 0x265973d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39d973d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcff973d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16d973d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa27973da, SUC_lexicon_tags_23 },
    { 0xcb3973db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb03973dd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1973e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3973e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653973ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd77973f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe51973f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x283973f4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7973fc, SUC_lexicon_tags_33 },
    { 0x4ab973fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0197405, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d97412, SUC_lexicon_tags_13 },
    { 0x8f397412, SUC_lexicon_tags_15 },
    { 0x50597413, SUC_lexicon_tags_3 },
    { 0x38397415, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0797418, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc799741a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa159741f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff797423, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60397427, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc397429, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38997431, SUC_lexicon_tags_37 },
    { 0xd1397432, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f97436, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7597438, SUC_lexicon_tags_0 },
    { 0x88d97438, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb19743c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb597442, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf97448, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc39744c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc899744e, SUC_lexicon_tags_15 },
    { 0x54b9744f, SUC_lexicon_tags_2 },
    { 0xaa59744e, SUC_lexicon_tags_10 },
    { 0x76397451, SUC_lexicon_tags_14 },
    { 0x1e397452, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x80197454, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f9745a, SUC_lexicon_tags_25 },
    { 0xbd19745a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af97462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3179746e, SUC_lexicon_tags_7 },
    { 0x6e99746e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x67997471, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc7b97473, SUC_lexicon_tags_30 },
    { 0x54397474, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd1d97476, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a19747b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10397482, SUC_lexicon_tags_11 },
    { 0x38997482, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4597487, SUC_lexicon_tags_37 },
    { 0xee997488, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf299748d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa597491, SUC_lexicon_tags_7 },
    { 0xc8397492, SUC_lexicon_tags_1 },
    { 0x93f97493, SUC_lexicon_tags_14 },
    { 0x94197492, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x56797497, SUC_lexicon_tags_8 },
    { 0x9e397498, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af9749c, SUC_lexicon_tags_7 },
    { 0xd779749d, SUC_lexicon_tags_11 },
    { 0x7df9749e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd63974a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1c5974a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d974a4, SUC_lexicon_tags_15 },
    { 0xc0f974a5, SUC_lexicon_tags_1 },
    { 0xdff974a6, SUC_lexicon_tags_17 },
    { 0x4a5974a6, SUC_lexicon_tags_1 },
    { 0xff1974a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929974ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd91974ae, SUC_lexicon_tags_11 },
    { 0xf7b974af, SUC_lexicon_tags_27 },
    { 0x709974ae, SUC_lexicon_tags_17 },
    { 0xb13974b1, SUC_lexicon_tags_7 },
    { 0xb8b974b2, SUC_lexicon_tags_104 },
    { 0xad974b3, SUC_lexicon_tags_2 },
    { 0x6c7974b1, SUC_lexicon_tags_60 },
    { 0x945974af, SUC_lexicon_tags_0 },
    { 0xb11974b6, SUC_lexicon_tags_11 },
    { 0xf974b7, SUC_lexicon_tags_17 },
    { 0x847974b7, SUC_lexicon_tags_25 },
    { 0x34f974b7, SUC_lexicon_tags_11 },
    { 0x447974ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f974be, SUC_lexicon_tags_27 },
    { 0x645974bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1974c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b974cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f974d5, SUC_lexicon_tags_12 },
    { 0xa2f974d6, SUC_lexicon_tags_456 },
    { 0x97974d7, SUC_lexicon_tags_7 },
    { 0x1e5974d7, SUC_lexicon_tags_14 },
    { 0xac7974d8, SUC_lexicon_tags_26 },
    { 0xc3f974da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89974dd, SUC_lexicon_tags_9 },
    { 0x701974de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459974e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x795974e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed974f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bd974f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99974f6, SUC_lexicon_tags_133 },
    { 0xbeb974f6, SUC_lexicon_tags_12 },
    { 0x6af974f8, SUC_lexicon_tags_25 },
    { 0x6b9974f9, SUC_lexicon_tags_273 },
    { 0x727974fa, SUC_lexicon_tags_12 },
    { 0x233974f8, SUC_lexicon_tags_12 },
    { 0x7d974f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4597500, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed97503, SUC_lexicon_tags_2 },
    { 0x6b397503, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc9797506, SUC_lexicon_tags_15 },
    { 0x69597507, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d99750c, SUC_lexicon_tags_17 },
    { 0x7c79750d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3219750f, SUC_lexicon_tags_25 },
    { 0x31797510, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeab97515, SUC_lexicon_tags_15 },
    { 0x72397516, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9751b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1f79751d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18397520, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3997522, SUC_lexicon_tags_46 },
    { 0xe0b97523, SUC_lexicon_tags_69 },
    { 0x99997523, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f97529, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc39752c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95397530, SUC_lexicon_tags_30 },
    { 0xcab97531, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb97535, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f19753a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xcd79753c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5997544, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b97548, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2f79754a, SUC_lexicon_tags_14 },
    { 0x1199754b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50197554, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b97557, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa397559, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f9755c, SUC_lexicon_tags_14 },
    { 0x9c19755c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94f97560, SUC_lexicon_tags_37 },
    { 0xdb97560, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df97564, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d97567, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x15797569, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7797571, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d99757b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddb9757f, SUC_lexicon_tags_12 },
    { 0x12797580, SUC_lexicon_tags_229 },
    { 0x70d9757f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a397583, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f597588, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf59758d, SUC_lexicon_tags_44 },
    { 0xd79758e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5197592, SUC_lexicon_tags_23 },
    { 0x7b597593, SUC_lexicon_tags_12 },
    { 0x1c597593, SUC_lexicon_tags_87 },
    { 0x28797595, SUC_lexicon_tags_11 },
    { 0x15797593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1397599, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd9759e, SUC_lexicon_tags_17 },
    { 0xc059759e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b975a3, SUC_lexicon_tags_1618 },
    { 0x2bd975a4, SUC_lexicon_tags_12 },
    { 0x343975a4, SUC_lexicon_tags_12 },
    { 0x465975a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28b975a8, SUC_lexicon_tags_43 },
    { 0xe19975a9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729975af, SUC_lexicon_tags_15 },
    { 0x467975af, SUC_lexicon_tags_85 },
    { 0xcdf975af, SUC_lexicon_tags_28 },
    { 0xb9b975b2, SUC_lexicon_tags_60 },
    { 0x337975b2, SUC_lexicon_tags_13 },
    { 0xcaf975b3, SUC_lexicon_tags_1 },
    { 0x683975b4, SUC_lexicon_tags_25 },
    { 0x3a3975b6, SUC_lexicon_tags_19 },
    { 0x5a3975af, SUC_lexicon_tags_10 },
    { 0xdb975b8, SUC_lexicon_tags_17 },
    { 0x6fb975b4, SUC_lexicon_tags_25 },
    { 0xa0f975ba, SUC_lexicon_tags_25 },
    { 0x1ab975bb, SUC_lexicon_tags_26 },
    { 0x27b975b6, SUC_lexicon_tags_30 },
    { 0x791975bd, SUC_lexicon_tags_11 },
    { 0xa5d975be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139975c3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa25975c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851975d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ab975d4, SUC_lexicon_tags_91 },
    { 0xd4d975d4, SUC_lexicon_tags_25 },
    { 0x869975d4, SUC_lexicon_tags_11 },
    { 0x9d9975d7, SUC_lexicon_tags_44 },
    { 0xbc5975d4, SUC_lexicon_tags_60 },
    { 0x70d975d5, SUC_lexicon_tags_33 },
    { 0x2df975da, SUC_lexicon_tags_29 },
    { 0x81975d5, SUC_lexicon_tags_43 },
    { 0xeb975d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd21975de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd975e2, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f975e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651975ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b975ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x421975ef, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29975f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf975f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5975fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe87975fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeff975fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a797608, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd39760c, SUC_lexicon_tags_11 },
    { 0x6339760d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd97613, SUC_lexicon_tags_33 },
    { 0xdeb97614, SUC_lexicon_tags_37 },
    { 0x87797614, SUC_lexicon_tags_33 },
    { 0x24597616, SUC_lexicon_tags_11 },
    { 0x5df97616, SUC_lexicon_tags_11 },
    { 0x85b97618, SUC_lexicon_tags_17 },
    { 0x48b97619, SUC_lexicon_tags_1 },
    { 0x2539761a, SUC_lexicon_tags_25 },
    { 0xe19761b, SUC_lexicon_tags_63 },
    { 0x4b597616, SUC_lexicon_tags_29 },
    { 0xbed9761d, SUC_lexicon_tags_37 },
    { 0x7bf9761c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc197621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78797624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe397626, SUC_lexicon_tags_7 },
    { 0x7ab97627, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49397629, SUC_lexicon_tags_2 },
    { 0x6079762a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d9762c, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab97633, SUC_lexicon_tags_17 },
    { 0x1a197634, SUC_lexicon_tags_68 },
    { 0xbd597633, SUC_lexicon_tags_17 },
    { 0x97797635, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3397639, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36b9763b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf97645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f97649, SUC_lexicon_tags_61 },
    { 0x40197649, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8d9764c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c59764f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48597657, SUC_lexicon_tags_60 },
    { 0x1597658, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f9765c, SUC_lexicon_tags_37 },
    { 0xfe39765d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b79765f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d97663, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce797665, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb3397667, SUC_lexicon_tags_34 },
    { 0xd8d97667, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd9766d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b97674, SUC_lexicon_tags_25 },
    { 0x47797674, SUC_lexicon_tags_26 },
    { 0xcbd97674, SUC_lexicon_tags_12 },
    { 0xc5d97676, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159767c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f97681, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79768d, SUC_lexicon_tags_1164 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d397692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd97697, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe6197699, SUC_lexicon_tags_11 },
    { 0x4179769a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x7c39769c, SUC_lexicon_tags_37 },
    { 0x82b9769c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853976a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5976a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df976aa, SUC_lexicon_tags_77 },
    { 0x895976ab, SUC_lexicon_tags_45 },
    { 0x35f976aa, SUC_lexicon_tags_15 },
    { 0x6eb976ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f976b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd976b3, SUC_lexicon_tags_4 },
    { 0x4a5976b3, SUC_lexicon_tags_38 },
    { 0xa3d976b3, SUC_lexicon_tags_1 },
    { 0xd19976b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd29976b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913976bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb976bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf87976c1, SUC_lexicon_tags_12 },
    { 0x351976c2, SUC_lexicon_tags_130 },
    { 0, NULL },
    { 0xead976c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f976c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7976cb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf976ce, SUC_lexicon_tags_11 },
    { 0xa03976cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4f976d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7976d7, SUC_lexicon_tags_4 },
    { 0xe67976d7, SUC_lexicon_tags_63 },
    { 0x6ff976d9, SUC_lexicon_tags_32 },
    { 0x38b976da, SUC_lexicon_tags_7 },
    { 0x50f976db, SUC_lexicon_tags_19 },
    { 0x197976da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5976e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23f976e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7976e6, SUC_lexicon_tags_45 },
    { 0x237976e6, SUC_lexicon_tags_27 },
    { 0x3f5976e8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x855976eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd45976ed, SUC_lexicon_tags_140 },
    { 0x611976ee, SUC_lexicon_tags_8 },
    { 0x77976ee, SUC_lexicon_tags_42 },
    { 0x1c1976ed, SUC_lexicon_tags_17 },
    { 0xfbb976f1, SUC_lexicon_tags_47 },
    { 0x85976f2, SUC_lexicon_tags_11 },
    { 0x6c9976f3, SUC_lexicon_tags_37 },
    { 0xd09976f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b976fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99f976fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0597709, SUC_lexicon_tags_0 },
    { 0x30397709, SUC_lexicon_tags_7 },
    { 0x99b9770a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9770f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b197712, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2af97717, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379771e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4997721, SUC_lexicon_tags_17 },
    { 0x41997722, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d97729, SUC_lexicon_tags_2 },
    { 0xbeb9772a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a597730, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26797735, SUC_lexicon_tags_25 },
    { 0x7e797736, SUC_lexicon_tags_25 },
    { 0xca997735, SUC_lexicon_tags_17 },
    { 0xcf597737, SUC_lexicon_tags_48 },
    { 0x49d97739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb9773e, SUC_lexicon_tags_63 },
    { 0x9f59773e, SUC_lexicon_tags_87 },
    { 0x1c597740, SUC_lexicon_tags_46 },
    { 0x66997740, SUC_lexicon_tags_1 },
    { 0x39797741, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcc797744, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x43597746, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab9774b, SUC_lexicon_tags_66 },
    { 0x58b9774c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b97750, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x80d97752, SUC_lexicon_tags_17 },
    { 0x86397753, SUC_lexicon_tags_26 },
    { 0x59597754, SUC_lexicon_tags_17 },
    { 0x8bd97755, SUC_lexicon_tags_63 },
    { 0x5e397755, SUC_lexicon_tags_12 },
    { 0xd4b97755, SUC_lexicon_tags_0 },
    { 0x59997753, SUC_lexicon_tags_6 },
    { 0xe9997759, SUC_lexicon_tags_38 },
    { 0x2f99775a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f19775f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff97763, SUC_lexicon_tags_11 },
    { 0x92997763, SUC_lexicon_tags_61 },
    { 0x93f97765, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33597768, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b9776c, SUC_lexicon_tags_277 },
    { 0xc4d9776d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1997770, SUC_lexicon_tags_5 },
    { 0x83f97770, SUC_lexicon_tags_19 },
    { 0x2d97772, SUC_lexicon_tags_7 },
    { 0x99f97773, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x74997776, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19777a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa379777c, SUC_lexicon_tags_3 },
    { 0x24f9777c, SUC_lexicon_tags_27 },
    { 0xc979777e, SUC_lexicon_tags_15 },
    { 0xd4b9777f, SUC_lexicon_tags_1 },
    { 0x1259777e, SUC_lexicon_tags_2 },
    { 0x3df97781, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf97784, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac597788, SUC_lexicon_tags_37 },
    { 0x5b597789, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4819778e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8997790, SUC_lexicon_tags_7 },
    { 0x20f97791, SUC_lexicon_tags_1 },
    { 0x6c597791, SUC_lexicon_tags_27 },
    { 0xae597793, SUC_lexicon_tags_48 },
    { 0x20397791, SUC_lexicon_tags_15 },
    { 0x9a397795, SUC_lexicon_tags_30 },
    { 0x70197796, SUC_lexicon_tags_12 },
    { 0x4c997791, SUC_lexicon_tags_31 },
    { 0xc7d97798, SUC_lexicon_tags_11 },
    { 0x20197799, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x20f9779b, SUC_lexicon_tags_14 },
    { 0x7a59779c, SUC_lexicon_tags_11 },
    { 0xd1b9779d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7199779f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbeb977a1, SUC_lexicon_tags_43 },
    { 0xab9977a2, SUC_lexicon_tags_26 },
    { 0x47b977a3, SUC_lexicon_tags_14 },
    { 0x863977a4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff977a7, SUC_lexicon_tags_12 },
    { 0xa31977a7, SUC_lexicon_tags_26 },
    { 0x901977a7, SUC_lexicon_tags_68 },
    { 0xdc1977aa, SUC_lexicon_tags_18 },
    { 0x65f977ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d977ae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f977b2, SUC_lexicon_tags_25 },
    { 0xe0f977b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365977b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d977bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9977c1, SUC_lexicon_tags_16 },
    { 0xdc7977c2, SUC_lexicon_tags_15 },
    { 0x2cf977c2, SUC_lexicon_tags_49 },
    { 0xa15977c4, SUC_lexicon_tags_28 },
    { 0x167977c4, SUC_lexicon_tags_25 },
    { 0xb39977c6, SUC_lexicon_tags_47 },
    { 0xfb5977c7, SUC_lexicon_tags_7 },
    { 0xfd9977c8, SUC_lexicon_tags_100 },
    { 0x99977c6, SUC_lexicon_tags_37 },
    { 0x7eb977c4, SUC_lexicon_tags_17 },
    { 0xfb977cb, SUC_lexicon_tags_60 },
    { 0x791977cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b977d4, SUC_lexicon_tags_10 },
    { 0xec9977d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b1977d7, SUC_lexicon_tags_16 },
    { 0x647977d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x175977da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc5977dc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d977e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x81d977e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x85b977e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139977e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x50d977ea, SUC_lexicon_tags_10 },
    { 0xb73977eb, SUC_lexicon_tags_44 },
    { 0x5a9977ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf977f2, SUC_lexicon_tags_15 },
    { 0x50b977f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe03977f5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed977fc, SUC_lexicon_tags_17 },
    { 0xd51977fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc597800, SUC_lexicon_tags_44 },
    { 0xee597801, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd997803, SUC_lexicon_tags_12 },
    { 0x15997804, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f39780f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8597812, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47597814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd879781a, SUC_lexicon_tags_0 },
    { 0x95b9781b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa599781f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf97827, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7499782b, SUC_lexicon_tags_9 },
    { 0xe459782c, SUC_lexicon_tags_17 },
    { 0xea59782b, SUC_lexicon_tags_37 },
    { 0xbb79782e, SUC_lexicon_tags_12 },
    { 0x34d9782f, SUC_lexicon_tags_47 },
    { 0xa799782e, SUC_lexicon_tags_17 },
    { 0x99197831, SUC_lexicon_tags_35 },
    { 0x5a39782f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d397836, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf39783f, SUC_lexicon_tags_25 },
    { 0xaf197840, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d97847, SUC_lexicon_tags_17 },
    { 0x5e397847, SUC_lexicon_tags_37 },
    { 0x2c397847, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e99784e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c997850, SUC_lexicon_tags_26 },
    { 0xa3997851, SUC_lexicon_tags_27 },
    { 0x65597852, SUC_lexicon_tags_25 },
    { 0x1af97851, SUC_lexicon_tags_23 },
    { 0xda997854, SUC_lexicon_tags_37 },
    { 0xafb97855, SUC_lexicon_tags_1 },
    { 0x99b97850, SUC_lexicon_tags_25 },
    { 0x5af97853, SUC_lexicon_tags_11 },
    { 0x5b997858, SUC_lexicon_tags_8 },
    { 0x86997859, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f9785f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b97862, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cf97864, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e597868, SUC_lexicon_tags_17 },
    { 0xdf97869, SUC_lexicon_tags_61 },
    { 0xd159786a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea597870, SUC_lexicon_tags_11 },
    { 0x8a597871, SUC_lexicon_tags_25 },
    { 0xadf97872, SUC_lexicon_tags_11 },
    { 0x50397873, SUC_lexicon_tags_31 },
    { 0xc0597873, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c797877, SUC_lexicon_tags_9 },
    { 0xc2597878, SUC_lexicon_tags_47 },
    { 0xb5f97879, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c39787b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1799787d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3ad9787f, SUC_lexicon_tags_23 },
    { 0xdd397880, SUC_lexicon_tags_37 },
    { 0xcb97880, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecf97888, SUC_lexicon_tags_32 },
    { 0x4c397888, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7739788d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53197891, SUC_lexicon_tags_25 },
    { 0x68d97892, SUC_lexicon_tags_7 },
    { 0x1e397893, SUC_lexicon_tags_15 },
    { 0x6a997894, SUC_lexicon_tags_200 },
    { 0xa6d97895, SUC_lexicon_tags_37 },
    { 0xd3997892, SUC_lexicon_tags_2 },
    { 0xdf597897, SUC_lexicon_tags_15 },
    { 0x7f597897, SUC_lexicon_tags_7 },
    { 0xb6997894, SUC_lexicon_tags_13 },
    { 0x15397897, SUC_lexicon_tags_28 },
    { 0x11d97895, SUC_lexicon_tags_65 },
    { 0xa659789a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed978a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77978a9, SUC_lexicon_tags_13 },
    { 0x8e9978aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ad978ad, SUC_lexicon_tags_14 },
    { 0x4c9978ae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x705978b1, SUC_lexicon_tags_0 },
    { 0x699978b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5978b9, SUC_lexicon_tags_60 },
    { 0xa83978b9, SUC_lexicon_tags_19 },
    { 0xd13978bb, SUC_lexicon_tags_2 },
    { 0xa4b978bc, SUC_lexicon_tags_33 },
    { 0x5b9978bd, SUC_lexicon_tags_25 },
    { 0x253978be, SUC_lexicon_tags_5 },
    { 0xf3b978bf, SUC_lexicon_tags_98 },
    { 0xd91978bc, SUC_lexicon_tags_7 },
    { 0xd7d978c1, SUC_lexicon_tags_68 },
    { 0x90f978c1, SUC_lexicon_tags_45 },
    { 0x9df978c3, SUC_lexicon_tags_2 },
    { 0x45d978c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b978d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b978d8, SUC_lexicon_tags_2 },
    { 0x55d978d9, SUC_lexicon_tags_66 },
    { 0x6c9978d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783978df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493978e3, SUC_lexicon_tags_6 },
    { 0x411978e3, SUC_lexicon_tags_17 },
    { 0x629978e3, SUC_lexicon_tags_8 },
    { 0x161978e4, SUC_lexicon_tags_30 },
    { 0x751978e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa97978e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f978ec, SUC_lexicon_tags_44 },
    { 0xe4d978ec, SUC_lexicon_tags_19 },
    { 0xe2f978ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x117978f0, SUC_lexicon_tags_48 },
    { 0x72f978f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4a3978f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4d978f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d978fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e797901, SUC_lexicon_tags_25 },
    { 0x7b997902, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4597904, SUC_lexicon_tags_14 },
    { 0xccd97905, SUC_lexicon_tags_31 },
    { 0x21b97906, SUC_lexicon_tags_27 },
    { 0x9dd97907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e79790d, SUC_lexicon_tags_7 },
    { 0xa299790d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7397911, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f97914, SUC_lexicon_tags_0 },
    { 0xf6797915, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b9791a, SUC_lexicon_tags_11 },
    { 0xedb9791b, SUC_lexicon_tags_15 },
    { 0x14d9791c, SUC_lexicon_tags_7 },
    { 0xc6b9791d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df9792c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69f9792e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b197930, SUC_lexicon_tags_17 },
    { 0x1d797931, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a597934, SUC_lexicon_tags_63 },
    { 0xfb97934, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d97939, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7659793e, SUC_lexicon_tags_11 },
    { 0x1399793e, SUC_lexicon_tags_23 },
    { 0x76397940, SUC_lexicon_tags_27 },
    { 0x6f99793e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x35997943, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2597945, SUC_lexicon_tags_7 },
    { 0x91597946, SUC_lexicon_tags_15 },
    { 0x52197947, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad59794c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e397950, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d97955, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb97958, SUC_lexicon_tags_1 },
    { 0x9b997959, SUC_lexicon_tags_16 },
    { 0x5fb9795a, SUC_lexicon_tags_11 },
    { 0x4fb9795b, SUC_lexicon_tags_66 },
    { 0xe159795a, SUC_lexicon_tags_14 },
    { 0xa139795a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc939795f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44797963, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd797967, SUC_lexicon_tags_26 },
    { 0x54397967, SUC_lexicon_tags_23 },
    { 0x42797969, SUC_lexicon_tags_15 },
    { 0xa579796a, SUC_lexicon_tags_7 },
    { 0xc1f9796b, SUC_lexicon_tags_11 },
    { 0xbcf9796c, SUC_lexicon_tags_15 },
    { 0x5b9796c, SUC_lexicon_tags_37 },
    { 0x7479796e, SUC_lexicon_tags_1 },
    { 0x7b9796e, SUC_lexicon_tags_17 },
    { 0xae597969, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b97973, SUC_lexicon_tags_12 },
    { 0x6c197973, SUC_lexicon_tags_37 },
    { 0xb3397974, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1519797a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f9797d, SUC_lexicon_tags_28 },
    { 0x52f9797d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce997981, SUC_lexicon_tags_12 },
    { 0x87d97981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f97987, SUC_lexicon_tags_0 },
    { 0xcb397988, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd9798b, SUC_lexicon_tags_7 },
    { 0xc099798c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8399798f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17597991, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f9799b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee39799e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3979a3, SUC_lexicon_tags_38 },
    { 0xf21979a4, SUC_lexicon_tags_38 },
    { 0x1e7979a5, SUC_lexicon_tags_48 },
    { 0xd0b979a5, SUC_lexicon_tags_63 },
    { 0x457979a4, SUC_lexicon_tags_37 },
    { 0x36b979a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af979ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd979b1, SUC_lexicon_tags_25 },
    { 0xfd979b2, SUC_lexicon_tags_47 },
    { 0xdd979b2, SUC_lexicon_tags_12 },
    { 0xb4f979b4, SUC_lexicon_tags_17 },
    { 0x267979b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55979b8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc8b979ba, SUC_lexicon_tags_49 },
    { 0x69f979ba, SUC_lexicon_tags_19 },
    { 0xce7979bc, SUC_lexicon_tags_27 },
    { 0xc75979bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x89979bf, SUC_lexicon_tags_14 },
    { 0x9cd979c0, SUC_lexicon_tags_12 },
    { 0x475979c0, SUC_lexicon_tags_12 },
    { 0x70d979c2, SUC_lexicon_tags_11 },
    { 0x1cf979c2, SUC_lexicon_tags_25 },
    { 0xa2b979c4, SUC_lexicon_tags_11 },
    { 0xa65979c4, SUC_lexicon_tags_16 },
    { 0xed1979c6, SUC_lexicon_tags_31 },
    { 0x2f979c7, SUC_lexicon_tags_30 },
    { 0x701979c8, SUC_lexicon_tags_8 },
    { 0x51979c8, SUC_lexicon_tags_33 },
    { 0x627979ca, SUC_lexicon_tags_7 },
    { 0xee9979c6, SUC_lexicon_tags_63 },
    { 0xb71979c5, SUC_lexicon_tags_29 },
    { 0x5c5979cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689979d4, SUC_lexicon_tags_6 },
    { 0xaf979d4, SUC_lexicon_tags_19 },
    { 0xf2b979d6, SUC_lexicon_tags_11 },
    { 0xb9b979d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351979df, SUC_lexicon_tags_49 },
    { 0x8df979e0, SUC_lexicon_tags_1 },
    { 0x197979e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9979e4, SUC_lexicon_tags_15 },
    { 0x89d979e5, SUC_lexicon_tags_16 },
    { 0x699979e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd65979e8, SUC_lexicon_tags_0 },
    { 0xdcd979e9, SUC_lexicon_tags_0 },
    { 0x357979e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90f979ec, SUC_lexicon_tags_27 },
    { 0x8f7979ed, SUC_lexicon_tags_30 },
    { 0x6cd979ee, SUC_lexicon_tags_62 },
    { 0x413979ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f979f2, SUC_lexicon_tags_11 },
    { 0xc1979f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489979f7, SUC_lexicon_tags_32 },
    { 0x225979f8, SUC_lexicon_tags_23 },
    { 0xa53979f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f97a03, SUC_lexicon_tags_28 },
    { 0x7b797a04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78597a0c, SUC_lexicon_tags_13 },
    { 0x6b997a0d, SUC_lexicon_tags_25 },
    { 0xf9b97a0e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d97a13, SUC_lexicon_tags_19 },
    { 0xf4f97a14, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7597a1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb5397a1d, SUC_lexicon_tags_26 },
    { 0xb1b97a1d, SUC_lexicon_tags_33 },
    { 0x76797a1f, SUC_lexicon_tags_31 },
    { 0x10797a1f, SUC_lexicon_tags_11 },
    { 0xca797a21, SUC_lexicon_tags_12 },
    { 0xa3997a22, SUC_lexicon_tags_7 },
    { 0xa0797a22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e397a26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83197a2c, SUC_lexicon_tags_15 },
    { 0x74197a2d, SUC_lexicon_tags_37 },
    { 0xe3397a2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f97a35, SUC_lexicon_tags_29 },
    { 0x8eb97a36, SUC_lexicon_tags_4 },
    { 0x6e197a35, SUC_lexicon_tags_42 },
    { 0x48797a38, SUC_lexicon_tags_27 },
    { 0xf2797a39, SUC_lexicon_tags_25 },
    { 0x9ed97a3a, SUC_lexicon_tags_26 },
    { 0xbcf97a39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f597a3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24197a3f, SUC_lexicon_tags_8 },
    { 0x81797a3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x84797a43, SUC_lexicon_tags_17 },
    { 0xb8997a44, SUC_lexicon_tags_25 },
    { 0x38997a44, SUC_lexicon_tags_37 },
    { 0x1bf97a46, SUC_lexicon_tags_84 },
    { 0xb397a45, SUC_lexicon_tags_173 },
    { 0x97597a48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b797a4b, SUC_lexicon_tags_15 },
    { 0xbdd97a4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcab97a4e, SUC_lexicon_tags_12 },
    { 0x4597a4f, SUC_lexicon_tags_38 },
    { 0x24d97a4f, SUC_lexicon_tags_27 },
    { 0x52f97a51, SUC_lexicon_tags_56 },
    { 0xa3d97a50, SUC_lexicon_tags_45 },
    { 0x91197a4f, SUC_lexicon_tags_42 },
    { 0x2c197a4e, SUC_lexicon_tags_11 },
    { 0xec797a51, SUC_lexicon_tags_12 },
    { 0xa5397a56, SUC_lexicon_tags_37 },
    { 0x73b97a56, SUC_lexicon_tags_25 },
    { 0x95d97a4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa397a5a, SUC_lexicon_tags_38 },
    { 0xf5f97a5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab397a5e, SUC_lexicon_tags_14 },
    { 0x16197a5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23197a64, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb97a6a, SUC_lexicon_tags_68 },
    { 0x88d97a6a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41197a70, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdcb97a72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56397a79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb997a7c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xdfd97a7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb997a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98997a89, SUC_lexicon_tags_38 },
    { 0x9ff97a8a, SUC_lexicon_tags_32 },
    { 0x62b97a8b, SUC_lexicon_tags_38 },
    { 0x2e397a8c, SUC_lexicon_tags_151 },
    { 0xdb97a8a, SUC_lexicon_tags_12 },
    { 0x1ad97a8c, SUC_lexicon_tags_11 },
    { 0xe8f97a8f, SUC_lexicon_tags_15 },
    { 0x8ad97a90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2d97a92, SUC_lexicon_tags_1 },
    { 0x10397a93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f97a99, SUC_lexicon_tags_37 },
    { 0x4997a9a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x87597a9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b97aa1, SUC_lexicon_tags_38 },
    { 0x397aa1, SUC_lexicon_tags_19 },
    { 0xef797aa3, SUC_lexicon_tags_37 },
    { 0xdd997aa4, SUC_lexicon_tags_63 },
    { 0x3d97aa5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6997aa7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa6d97aa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b97aad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99797aaf, SUC_lexicon_tags_27 },
    { 0x3a597aaf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad97ab3, SUC_lexicon_tags_7 },
    { 0x11597ab3, SUC_lexicon_tags_7 },
    { 0x13797ab3, SUC_lexicon_tags_7 },
    { 0x4bd97ab6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1997aba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd97abe, SUC_lexicon_tags_17 },
    { 0x49f97abe, SUC_lexicon_tags_45 },
    { 0x6ff97ac0, SUC_lexicon_tags_65 },
    { 0x62797ac1, SUC_lexicon_tags_25 },
    { 0x92997ac1, SUC_lexicon_tags_4 },
    { 0x5f597ac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff97ac9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76797ad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88597ad4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9597ad9, SUC_lexicon_tags_35 },
    { 0x29b97ad9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x68197adc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55997ade, SUC_lexicon_tags_47 },
    { 0xc6197adf, SUC_lexicon_tags_10 },
    { 0xb0f97adf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8597ae4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x39d97ae7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df97aea, SUC_lexicon_tags_37 },
    { 0x98f97aea, SUC_lexicon_tags_23 },
    { 0x78b97aec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f597aef, SUC_lexicon_tags_37 },
    { 0xd5997af0, SUC_lexicon_tags_7 },
    { 0xba797af0, SUC_lexicon_tags_27 },
    { 0x19b97af2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94997af7, SUC_lexicon_tags_17 },
    { 0xbdd97af8, SUC_lexicon_tags_27 },
    { 0x63597af8, SUC_lexicon_tags_1 },
    { 0x60397afa, SUC_lexicon_tags_17 },
    { 0xf0597afb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf97aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe797b04, SUC_lexicon_tags_31 },
    { 0x92397b05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd97b0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9997b0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80797b11, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d97b15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47597b18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50597b1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf8d97b1e, SUC_lexicon_tags_33 },
    { 0x94797b1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f97b2d, SUC_lexicon_tags_15 },
    { 0x55797b2e, SUC_lexicon_tags_30 },
    { 0x40797b2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b97b34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3797b3d, SUC_lexicon_tags_17 },
    { 0xd0197b3d, SUC_lexicon_tags_14 },
    { 0x8cf97b3f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x48f97b41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x49f97b43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf97b48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17597b4b, SUC_lexicon_tags_49 },
    { 0xf2197b4c, SUC_lexicon_tags_23 },
    { 0xc4d97b4d, SUC_lexicon_tags_32 },
    { 0x6db97b4e, SUC_lexicon_tags_56 },
    { 0xd1397b4f, SUC_lexicon_tags_27 },
    { 0x32597b4c, SUC_lexicon_tags_17 },
    { 0x3a997b51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83597b53, SUC_lexicon_tags_37 },
    { 0x49797b54, SUC_lexicon_tags_37 },
    { 0x43397b55, SUC_lexicon_tags_0 },
    { 0x2a197b56, SUC_lexicon_tags_2 },
    { 0x98197b56, SUC_lexicon_tags_3 },
    { 0x52d97b54, SUC_lexicon_tags_49 },
    { 0x50b97b59, SUC_lexicon_tags_2 },
    { 0x16397b5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b197b5d, SUC_lexicon_tags_69 },
    { 0xbfb97b5e, SUC_lexicon_tags_26 },
    { 0xefb97b5e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe797b63, SUC_lexicon_tags_1 },
    { 0x28397b63, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7e597b66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b97b6c, SUC_lexicon_tags_12 },
    { 0x47797b6d, SUC_lexicon_tags_7 },
    { 0x34d97b6c, SUC_lexicon_tags_11 },
    { 0x4ff97b6f, SUC_lexicon_tags_6 },
    { 0x55b97b6f, SUC_lexicon_tags_60 },
    { 0x51d97b71, SUC_lexicon_tags_19 },
    { 0xf8997b71, SUC_lexicon_tags_60 },
    { 0x4cb97b6d, SUC_lexicon_tags_15 },
    { 0xca397b6d, SUC_lexicon_tags_37 },
    { 0xe1997b73, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd97b7c, SUC_lexicon_tags_83 },
    { 0xbeb97b7d, SUC_lexicon_tags_60 },
    { 0xdc997b7c, SUC_lexicon_tags_37 },
    { 0x5c797b7f, SUC_lexicon_tags_40 },
    { 0xf0397b80, SUC_lexicon_tags_6 },
    { 0xaa797b7c, SUC_lexicon_tags_68 },
    { 0xeeb97b81, SUC_lexicon_tags_9 },
    { 0x3bf97b83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd97b87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x34197b89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d197b8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4997b8e, SUC_lexicon_tags_8 },
    { 0xfd97b8f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d97b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db97b99, SUC_lexicon_tags_8 },
    { 0xf3397b9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f97ba0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd6b97ba2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaed97ba5, SUC_lexicon_tags_0 },
    { 0xbad97ba6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8197ba8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2b97bab, SUC_lexicon_tags_37 },
    { 0xddb97bab, SUC_lexicon_tags_7 },
    { 0x51397bad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2d97bb1, SUC_lexicon_tags_7 },
    { 0x49f97bb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12f97bb4, SUC_lexicon_tags_15 },
    { 0x78997bb5, SUC_lexicon_tags_47 },
    { 0x23797bb6, SUC_lexicon_tags_3 },
    { 0xb6f97bb6, SUC_lexicon_tags_2 },
    { 0x6c597bb6, SUC_lexicon_tags_33 },
    { 0xf1797bb8, SUC_lexicon_tags_19 },
    { 0x5e597bb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80597bc0, SUC_lexicon_tags_39 },
    { 0x58997bc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3db97bc3, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x53d97bc6, SUC_lexicon_tags_45 },
    { 0x72197bc6, SUC_lexicon_tags_4 },
    { 0xd9597bc8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a597bd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98d97bd4, SUC_lexicon_tags_25 },
    { 0x23b97bd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7dd97bd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3397bdb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7197bdf, SUC_lexicon_tags_1222 },
    { 0xfa797bdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d797be3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4997be8, SUC_lexicon_tags_26 },
    { 0x1cf97be9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xafd97beb, SUC_lexicon_tags_32 },
    { 0xd997bec, SUC_lexicon_tags_12 },
    { 0xa397bed, SUC_lexicon_tags_27 },
    { 0xa9397bed, SUC_lexicon_tags_61 },
    { 0x52197bee, SUC_lexicon_tags_29 },
    { 0xb9597bf0, SUC_lexicon_tags_12 },
    { 0xcef97bf0, SUC_lexicon_tags_26 },
    { 0xa0f97bf1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b197bf5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a197bfa, SUC_lexicon_tags_37 },
    { 0xab997bfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x89f97bfd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x27197c00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62797c06, SUC_lexicon_tags_12 },
    { 0x3b597c07, SUC_lexicon_tags_37 },
    { 0x8fd97c07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xad597c0a, SUC_lexicon_tags_2 },
    { 0x75d97c0b, SUC_lexicon_tags_11 },
    { 0x9a197c0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f97c12, SUC_lexicon_tags_29 },
    { 0x15d97c13, SUC_lexicon_tags_29 },
    { 0x4597c14, SUC_lexicon_tags_15 },
    { 0x16b97c14, SUC_lexicon_tags_32 },
    { 0xd5d97c16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c997c1f, SUC_lexicon_tags_1 },
    { 0x74f97c20, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x56f97c22, SUC_lexicon_tags_25 },
    { 0xe4d97c23, SUC_lexicon_tags_25 },
    { 0xe0b97c24, SUC_lexicon_tags_27 },
    { 0x1ff97c24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d797c28, SUC_lexicon_tags_7 },
    { 0xec197c29, SUC_lexicon_tags_17 },
    { 0xb7197c29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27797c2c, SUC_lexicon_tags_7 },
    { 0xe2197c2c, SUC_lexicon_tags_14 },
    { 0x25d97c2c, SUC_lexicon_tags_109 },
    { 0xd7b97c2f, SUC_lexicon_tags_47 },
    { 0x3a597c30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7797c33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f97c38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40197c3c, SUC_lexicon_tags_7 },
    { 0x63997c3c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85397c46, SUC_lexicon_tags_43 },
    { 0x19f97c46, SUC_lexicon_tags_25 },
    { 0x48397c48, SUC_lexicon_tags_12 },
    { 0x73f97c48, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17197c50, SUC_lexicon_tags_31 },
    { 0xd5197c51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90197c54, SUC_lexicon_tags_37 },
    { 0xeff97c55, SUC_lexicon_tags_17 },
    { 0x7bf97c55, SUC_lexicon_tags_13 },
    { 0x54597c57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1997c5b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec397c5f, SUC_lexicon_tags_16 },
    { 0x20397c5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe2d97c62, SUC_lexicon_tags_46 },
    { 0x1c997c63, SUC_lexicon_tags_37 },
    { 0x34b97c64, SUC_lexicon_tags_43 },
    { 0xbe597c65, SUC_lexicon_tags_27 },
    { 0x1c797c65, SUC_lexicon_tags_45 },
    { 0x20797c64, SUC_lexicon_tags_25 },
    { 0xb7197c68, SUC_lexicon_tags_14 },
    { 0x7fb97c69, SUC_lexicon_tags_29 },
    { 0x27997c6a, SUC_lexicon_tags_17 },
    { 0xb3597c6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x14797c6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d97c70, SUC_lexicon_tags_1 },
    { 0x58d97c70, SUC_lexicon_tags_32 },
    { 0x5ad97c72, SUC_lexicon_tags_19 },
    { 0x2d597c73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef97c76, SUC_lexicon_tags_21 },
    { 0x80997c77, SUC_lexicon_tags_17 },
    { 0xbed97c78, SUC_lexicon_tags_273 },
    { 0x20997c78, SUC_lexicon_tags_43 },
    { 0xc6197c78, SUC_lexicon_tags_0 },
    { 0xc5597c7b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xccb97c7d, SUC_lexicon_tags_1282 },
    { 0, NULL },
    { 0, NULL },
    { 0xba397c80, SUC_lexicon_tags_23 },
    { 0x48997c81, SUC_lexicon_tags_7 },
    { 0x87397c82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x90f97c84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e597c89, SUC_lexicon_tags_0 },
    { 0xf597c89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf997c8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d97c91, SUC_lexicon_tags_45 },
    { 0xad797c92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cf97c98, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d97c9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b97ca2, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x66d97ca4, SUC_lexicon_tags_45 },
    { 0x8b397ca5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b397caa, SUC_lexicon_tags_0 },
    { 0x4bd97cab, SUC_lexicon_tags_7 },
    { 0xb2797cab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x28197cae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf197cb1, SUC_lexicon_tags_77 },
    { 0x48797cb1, SUC_lexicon_tags_37 },
    { 0x64397cb3, SUC_lexicon_tags_28 },
    { 0x90397cb4, SUC_lexicon_tags_15 },
    { 0x61f97cb5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b97cb8, SUC_lexicon_tags_21 },
    { 0x1c397cb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73197cbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3db97cc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52997ccb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43597ccf, SUC_lexicon_tags_43 },
    { 0x1b797cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69d97cd2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a997cd5, SUC_lexicon_tags_45 },
    { 0xd7597cd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41797cd8, SUC_lexicon_tags_11 },
    { 0x70f97cd8, SUC_lexicon_tags_19 },
    { 0x75397cd9, SUC_lexicon_tags_12 },
    { 0x84397cda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44797cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8797ce1, SUC_lexicon_tags_2 },
    { 0x1af97ce1, SUC_lexicon_tags_37 },
    { 0x40b97ce2, SUC_lexicon_tags_2 },
    { 0xf6997ce1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb997ce9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27397cf2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91997cf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb797cf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f97cfc, SUC_lexicon_tags_4 },
    { 0x3fd97cfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d97d05, SUC_lexicon_tags_2 },
    { 0xba397d06, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfad97d0a, SUC_lexicon_tags_12 },
    { 0x27397d0b, SUC_lexicon_tags_6 },
    { 0x3197d0c, SUC_lexicon_tags_39 },
    { 0x81b97d0d, SUC_lexicon_tags_13 },
    { 0x9f797d0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x20997d10, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a997d15, SUC_lexicon_tags_26 },
    { 0xa4d97d16, SUC_lexicon_tags_82 },
    { 0xa8b97d15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f97d1d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12d97d1f, SUC_lexicon_tags_15 },
    { 0x21d97d20, SUC_lexicon_tags_48 },
    { 0x78197d1f, SUC_lexicon_tags_37 },
    { 0xad197d22, SUC_lexicon_tags_17 },
    { 0xedd97d22, SUC_lexicon_tags_87 },
    { 0x3bb97d24, SUC_lexicon_tags_25 },
    { 0x50d97d24, SUC_lexicon_tags_47 },
    { 0x8fd97d26, SUC_lexicon_tags_1 },
    { 0x75197d27, SUC_lexicon_tags_11 },
    { 0x35797d1f, SUC_lexicon_tags_25 },
    { 0xa597d29, SUC_lexicon_tags_21 },
    { 0xf4f97d21, SUC_lexicon_tags_2 },
    { 0xeb797d2b, SUC_lexicon_tags_27 },
    { 0xcd197d2b, SUC_lexicon_tags_37 },
    { 0xf1d97d2d, SUC_lexicon_tags_17 },
    { 0x30f97d2c, SUC_lexicon_tags_49 },
    { 0x64f97d2f, SUC_lexicon_tags_7 },
    { 0x44397d30, SUC_lexicon_tags_1 },
    { 0x6d997d31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec597d35, SUC_lexicon_tags_91 },
    { 0x2b597d35, SUC_lexicon_tags_15 },
    { 0x93797d36, SUC_lexicon_tags_25 },
    { 0x3a997d35, SUC_lexicon_tags_40 },
    { 0x93597d36, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3197d3f, SUC_lexicon_tags_27 },
    { 0xadb97d40, SUC_lexicon_tags_8 },
    { 0xaf597d40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b997d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d997d49, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f97d4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xebd97d4e, SUC_lexicon_tags_33 },
    { 0xa8d97d4f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b97d53, SUC_lexicon_tags_37 },
    { 0x51197d53, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x55597d56, SUC_lexicon_tags_26 },
    { 0xf797d57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x87597d5a, SUC_lexicon_tags_22 },
    { 0xf97d5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19397d66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c397d6d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a397d70, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b597d75, SUC_lexicon_tags_25 },
    { 0x91f97d76, SUC_lexicon_tags_14 },
    { 0x8a197d76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb97d80, SUC_lexicon_tags_39 },
    { 0x9c197d80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x73397d84, SUC_lexicon_tags_31 },
    { 0xdd997d84, SUC_lexicon_tags_33 },
    { 0x62d97d86, SUC_lexicon_tags_6 },
    { 0x6f397d87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d97d8b, SUC_lexicon_tags_33 },
    { 0x5197d8c, SUC_lexicon_tags_30 },
    { 0x9ad97d8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb597d91, SUC_lexicon_tags_2 },
    { 0xd7797d92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f197d96, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf2b97d98, SUC_lexicon_tags_37 },
    { 0x65797d99, SUC_lexicon_tags_17 },
    { 0x5bf97d99, SUC_lexicon_tags_65 },
    { 0x90197d99, SUC_lexicon_tags_15 },
    { 0xcf97d98, SUC_lexicon_tags_130 },
    { 0x3d197d9b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df97da0, SUC_lexicon_tags_42 },
    { 0x7b197da1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b197da3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36797da7, SUC_lexicon_tags_1 },
    { 0x32397da8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9d97daa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xedf97dac, SUC_lexicon_tags_29 },
    { 0x7c597dad, SUC_lexicon_tags_36 },
    { 0x6f797dae, SUC_lexicon_tags_25 },
    { 0xb3197daf, SUC_lexicon_tags_12 },
    { 0x57b97db0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa8b97db2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59197db6, SUC_lexicon_tags_15 },
    { 0x94597db6, SUC_lexicon_tags_12 },
    { 0x78f97db6, SUC_lexicon_tags_29 },
    { 0x9e797db8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6597dbc, SUC_lexicon_tags_66 },
    { 0xfd397dbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f797dc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61d97dc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c997dc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6eb97dc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd97dcb, SUC_lexicon_tags_17 },
    { 0x37397dcc, SUC_lexicon_tags_27 },
    { 0xcdd97dcd, SUC_lexicon_tags_37 },
    { 0xf8d97dce, SUC_lexicon_tags_23 },
    { 0x31397dcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f397dd2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d97dda, SUC_lexicon_tags_26 },
    { 0x34397dda, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f997de2, SUC_lexicon_tags_43 },
    { 0x29197de2, SUC_lexicon_tags_27 },
    { 0x46997de4, SUC_lexicon_tags_11 },
    { 0xfd797de4, SUC_lexicon_tags_45 },
    { 0x20d97de2, SUC_lexicon_tags_19 },
    { 0x5397de7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d97ded, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeb97df3, SUC_lexicon_tags_23 },
    { 0x88597df4, SUC_lexicon_tags_14 },
    { 0x58797df4, SUC_lexicon_tags_1 },
    { 0x21997df4, SUC_lexicon_tags_29 },
    { 0xb7797df3, SUC_lexicon_tags_7 },
    { 0xe3b97df8, SUC_lexicon_tags_87 },
    { 0x17d97df9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc997dfe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x79797e01, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6797e05, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x26797e07, SUC_lexicon_tags_9 },
    { 0x11797e08, SUC_lexicon_tags_1 },
    { 0xca197e09, SUC_lexicon_tags_7 },
    { 0x72397e0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b397e0c, SUC_lexicon_tags_37 },
    { 0x92d97e0d, SUC_lexicon_tags_8 },
    { 0xb4597e0d, SUC_lexicon_tags_48 },
    { 0x83997e0e, SUC_lexicon_tags_37 },
    { 0x7e197e10, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca397e14, SUC_lexicon_tags_37 },
    { 0x50997e15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52d97e18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22d97e1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f97e1e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xaf97e20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68797e24, SUC_lexicon_tags_61 },
    { 0xf3797e25, SUC_lexicon_tags_5 },
    { 0x76b97e26, SUC_lexicon_tags_10 },
    { 0xf8197e24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16397e2d, SUC_lexicon_tags_49 },
    { 0x99997e2e, SUC_lexicon_tags_38 },
    { 0xf597e2f, SUC_lexicon_tags_4 },
    { 0x60b97e2e, SUC_lexicon_tags_27 },
    { 0xb3997e31, SUC_lexicon_tags_25 },
    { 0xd4197e32, SUC_lexicon_tags_31 },
    { 0x56d97e32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b197e35, SUC_lexicon_tags_138 },
    { 0x16397e36, SUC_lexicon_tags_1 },
    { 0xc4d97e37, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1997e3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f97e46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x46d97e48, SUC_lexicon_tags_27 },
    { 0x9d597e49, SUC_lexicon_tags_68 },
    { 0xcb797e4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf97e4c, SUC_lexicon_tags_63 },
    { 0xd397e4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64397e50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94997e52, SUC_lexicon_tags_101 },
    { 0x24797e53, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfdf97e55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd597e59, SUC_lexicon_tags_37 },
    { 0x22f97e59, SUC_lexicon_tags_46 },
    { 0xbf97e5b, SUC_lexicon_tags_68 },
    { 0x11397e5b, SUC_lexicon_tags_11 },
    { 0xd1197e5d, SUC_lexicon_tags_4 },
    { 0xd5397e5d, SUC_lexicon_tags_23 },
    { 0x6f197e5e, SUC_lexicon_tags_49 },
    { 0x6997e5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5eb97e62, SUC_lexicon_tags_45 },
    { 0x48d97e62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef97e67, SUC_lexicon_tags_49 },
    { 0xbf797e67, SUC_lexicon_tags_65 },
    { 0x68d97e67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed97e6e, SUC_lexicon_tags_35 },
    { 0xccb97e6e, SUC_lexicon_tags_37 },
    { 0xab797e70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9797e72, SUC_lexicon_tags_11 },
    { 0x18397e73, SUC_lexicon_tags_56 },
    { 0x27797e74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3397e76, SUC_lexicon_tags_13 },
    { 0x4f797e77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef97e7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f97e84, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92997e89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59597e96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d97e99, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x40997e9b, SUC_lexicon_tags_27 },
    { 0x86797e9c, SUC_lexicon_tags_56 },
    { 0x7197e9d, SUC_lexicon_tags_68 },
    { 0xddf97e9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67197ea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cb97ea3, SUC_lexicon_tags_221 },
    { 0x27597ea3, SUC_lexicon_tags_30 },
    { 0x8a997ea4, SUC_lexicon_tags_23 },
    { 0x75d97ea3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d97eae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e797eb3, SUC_lexicon_tags_11 },
    { 0x23f97eb4, SUC_lexicon_tags_45 },
    { 0xa5397eb5, SUC_lexicon_tags_37 },
    { 0x94f97eb6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b97ebf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b97ec2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1597ec6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99997ec8, SUC_lexicon_tags_2 },
    { 0xae997ec8, SUC_lexicon_tags_33 },
    { 0x85597eca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xed197ecc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21397ed6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x95197ed9, SUC_lexicon_tags_12 },
    { 0xdd997eda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x64397edc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d597ee3, SUC_lexicon_tags_6 },
    { 0x4fd97ee4, SUC_lexicon_tags_38 },
    { 0x84597ee5, SUC_lexicon_tags_11 },
    { 0x7df97ee4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a397ee9, SUC_lexicon_tags_25 },
    { 0xf4b97ee9, SUC_lexicon_tags_11 },
    { 0x1c997ee9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x99397eed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcb97ef0, SUC_lexicon_tags_7 },
    { 0xc6b97ef0, SUC_lexicon_tags_11 },
    { 0x62d97ef2, SUC_lexicon_tags_37 },
    { 0xd1197ef1, SUC_lexicon_tags_2 },
    { 0xc3397ef4, SUC_lexicon_tags_49 },
    { 0x32d97ef5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbe797ef7, SUC_lexicon_tags_11 },
    { 0xa8397ef7, SUC_lexicon_tags_17 },
    { 0x29797ef9, SUC_lexicon_tags_45 },
    { 0x45b97efa, SUC_lexicon_tags_31 },
    { 0xcbd97ef8, SUC_lexicon_tags_46 },
    { 0x99d97ef9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6b97efe, SUC_lexicon_tags_9 },
    { 0x46397eff, SUC_lexicon_tags_12 },
    { 0x58797f00, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ef97f02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1eb97f04, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b397f0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8cb97f0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79197f10, SUC_lexicon_tags_43 },
    { 0xdc197f11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3f97f13, SUC_lexicon_tags_17 },
    { 0x6f97f14, SUC_lexicon_tags_7 },
    { 0x2ab97f15, SUC_lexicon_tags_8 },
    { 0x80f97f15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1797f1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91997f1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb597f1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x13997f21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44197f26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d97f28, SUC_lexicon_tags_9 },
    { 0x58997f28, SUC_lexicon_tags_69 },
    { 0xc6d97f2a, SUC_lexicon_tags_15 },
    { 0x29397f2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3d97f2d, SUC_lexicon_tags_37 },
    { 0x69b97f2e, SUC_lexicon_tags_8 },
    { 0x44d97f2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc197f34, SUC_lexicon_tags_45 },
    { 0xd5397f35, SUC_lexicon_tags_12 },
    { 0x2a397f36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb97f39, SUC_lexicon_tags_12 },
    { 0x88f97f3a, SUC_lexicon_tags_87 },
    { 0x93597f3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf97f41, SUC_lexicon_tags_0 },
    { 0xa3997f42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42597f46, SUC_lexicon_tags_25 },
    { 0xca797f47, SUC_lexicon_tags_43 },
    { 0xd5197f48, SUC_lexicon_tags_61 },
    { 0x38997f46, SUC_lexicon_tags_17 },
    { 0x4db97f4a, SUC_lexicon_tags_29 },
    { 0x72b97f4b, SUC_lexicon_tags_7 },
    { 0x9a997f4c, SUC_lexicon_tags_9 },
    { 0x3e397f46, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f97f50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72997f55, SUC_lexicon_tags_2 },
    { 0xad597f56, SUC_lexicon_tags_7 },
    { 0xbc997f57, SUC_lexicon_tags_1 },
    { 0xb997f58, SUC_lexicon_tags_0 },
    { 0x4c597f58, SUC_lexicon_tags_15 },
    { 0xdf997f5a, SUC_lexicon_tags_14 },
    { 0x6e197f5b, SUC_lexicon_tags_12 },
    { 0x41597f5b, SUC_lexicon_tags_17 },
    { 0xbc597f5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11397f60, SUC_lexicon_tags_25 },
    { 0xb3f97f61, SUC_lexicon_tags_3 },
    { 0xfdd97f62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec797f64, SUC_lexicon_tags_17 },
    { 0x36197f65, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8997f6e, SUC_lexicon_tags_7 },
    { 0x5f597f6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x89197f72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b97f75, SUC_lexicon_tags_7 },
    { 0xa8d97f76, SUC_lexicon_tags_17 },
    { 0xe1f97f77, SUC_lexicon_tags_37 },
    { 0x87b97f77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49397f7a, SUC_lexicon_tags_37 },
    { 0x6ab97f7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1597f80, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x60197f82, SUC_lexicon_tags_41 },
    { 0x76997f82, SUC_lexicon_tags_29 },
    { 0xe5d97f84, SUC_lexicon_tags_19 },
    { 0x98397f84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2af97f87, SUC_lexicon_tags_7 },
    { 0x4fb97f87, SUC_lexicon_tags_13 },
    { 0x94397f89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54597f8b, SUC_lexicon_tags_37 },
    { 0x8c197f8c, SUC_lexicon_tags_113 },
    { 0xd797f8d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d97f92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76197f94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb97f9a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x43597f9c, SUC_lexicon_tags_21 },
    { 0xb8997f9d, SUC_lexicon_tags_48 },
    { 0xfdf97f9d, SUC_lexicon_tags_2 },
    { 0xf4f97f9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56197fa1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78997fa3, SUC_lexicon_tags_38 },
    { 0x61b97fa4, SUC_lexicon_tags_7 },
    { 0xe6797fa4, SUC_lexicon_tags_14 },
    { 0x86997fa6, SUC_lexicon_tags_37 },
    { 0x6ff97fa6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7197fab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1797fad, SUC_lexicon_tags_7 },
    { 0x5c997fae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94397fb1, SUC_lexicon_tags_4 },
    { 0xf0997fb1, SUC_lexicon_tags_12 },
    { 0x57d97fb3, SUC_lexicon_tags_37 },
    { 0x16f97fb4, SUC_lexicon_tags_15 },
    { 0x6af97fb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde397fb7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf8197fb9, SUC_lexicon_tags_17 },
    { 0x53b97fba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24997fbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8f97fbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f97fc8, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f197fcc, SUC_lexicon_tags_91 },
    { 0xe0d97fcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d597fd0, SUC_lexicon_tags_28 },
    { 0xa1b97fd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcbd97fd3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6597fd7, SUC_lexicon_tags_66 },
    { 0x45397fd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde997fda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb397fde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb197fe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b97fe7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c797fec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1e597fee, SUC_lexicon_tags_31 },
    { 0xe1597fef, SUC_lexicon_tags_31 },
    { 0xc2f97fef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b97ff3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d97ff7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x78d97ff9, SUC_lexicon_tags_170 },
    { 0x73997ffa, SUC_lexicon_tags_14 },
    { 0x9c997ffb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x38f97ffd, SUC_lexicon_tags_16 },
    { 0x27197ffe, SUC_lexicon_tags_3 },
    { 0xb0597fff, SUC_lexicon_tags_15 },
    { 0x27597ffe, SUC_lexicon_tags_4 },
    { 0xc5f98000, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b398003, SUC_lexicon_tags_56 },
    { 0x4f198004, SUC_lexicon_tags_49 },
    { 0x11b98004, SUC_lexicon_tags_17 },
    { 0x98598005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3d98008, SUC_lexicon_tags_14 },
    { 0xd7198009, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41b9800c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7398010, SUC_lexicon_tags_27 },
    { 0xf6b98011, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a198014, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d9801a, SUC_lexicon_tags_26 },
    { 0xbd19801a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebb9801e, SUC_lexicon_tags_2 },
    { 0x6319801f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x58798022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb398025, SUC_lexicon_tags_11 },
    { 0xa9d98026, SUC_lexicon_tags_1 },
    { 0xb5b98027, SUC_lexicon_tags_1 },
    { 0x9f998027, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf439802e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33398031, SUC_lexicon_tags_1 },
    { 0x85d98032, SUC_lexicon_tags_37 },
    { 0x5df98032, SUC_lexicon_tags_25 },
    { 0x8d198033, SUC_lexicon_tags_23 },
    { 0x66b98033, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58598039, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3d19803b, SUC_lexicon_tags_7 },
    { 0x6939803c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92798041, SUC_lexicon_tags_2 },
    { 0x28198042, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b98045, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f398047, SUC_lexicon_tags_2 },
    { 0x14198048, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xabf9804a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e59804d, SUC_lexicon_tags_43 },
    { 0xf859804d, SUC_lexicon_tags_1 },
    { 0xe5d9804e, SUC_lexicon_tags_11 },
    { 0xefd98050, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df98053, SUC_lexicon_tags_23 },
    { 0x3e998054, SUC_lexicon_tags_9 },
    { 0xffb98055, SUC_lexicon_tags_38 },
    { 0x1398054, SUC_lexicon_tags_11 },
    { 0x15f98055, SUC_lexicon_tags_60 },
    { 0x8c798058, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2679805a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48398061, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed998067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7998069, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95f9806b, SUC_lexicon_tags_27 },
    { 0xba59806c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x759806e, SUC_lexicon_tags_23 },
    { 0x9379806f, SUC_lexicon_tags_7 },
    { 0x39398070, SUC_lexicon_tags_7 },
    { 0xf8b98071, SUC_lexicon_tags_6 },
    { 0xcf598071, SUC_lexicon_tags_27 },
    { 0x27d98072, SUC_lexicon_tags_25 },
    { 0x5d79806f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61598076, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3b198078, SUC_lexicon_tags_45 },
    { 0x75598079, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba19807b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x18d9807d, SUC_lexicon_tags_12 },
    { 0x8bf9807e, SUC_lexicon_tags_30 },
    { 0x3059807f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe398082, SUC_lexicon_tags_11 },
    { 0xecb98082, SUC_lexicon_tags_32 },
    { 0x91798084, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed98089, SUC_lexicon_tags_27 },
    { 0x9799808a, SUC_lexicon_tags_27 },
    { 0x8b99808a, SUC_lexicon_tags_0 },
    { 0xc939808a, SUC_lexicon_tags_31 },
    { 0x4a99808d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8d9808f, SUC_lexicon_tags_12 },
    { 0xf379808f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b98093, SUC_lexicon_tags_19 },
    { 0xd6798093, SUC_lexicon_tags_15 },
    { 0x8ef98095, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b98099, SUC_lexicon_tags_17 },
    { 0x1b98099, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa919809d, SUC_lexicon_tags_47 },
    { 0x6f99809e, SUC_lexicon_tags_15 },
    { 0x7639809e, SUC_lexicon_tags_34 },
    { 0x50d980a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461980a5, SUC_lexicon_tags_11 },
    { 0xa1f980a5, SUC_lexicon_tags_23 },
    { 0x2df980a7, SUC_lexicon_tags_7 },
    { 0x6b7980a8, SUC_lexicon_tags_14 },
    { 0xf53980a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd980af, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b980b7, SUC_lexicon_tags_79 },
    { 0x6f5980b8, SUC_lexicon_tags_62 },
    { 0x89f980b9, SUC_lexicon_tags_68 },
    { 0xbd3980b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5980bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb55980be, SUC_lexicon_tags_17 },
    { 0x677980be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e5980c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe07980c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275980c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf19980ca, SUC_lexicon_tags_3 },
    { 0xd7f980cb, SUC_lexicon_tags_104 },
    { 0xb11980cb, SUC_lexicon_tags_30 },
    { 0xbcd980cc, SUC_lexicon_tags_31 },
    { 0x51d980ca, SUC_lexicon_tags_7 },
    { 0xc99980ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x171980d2, SUC_lexicon_tags_30 },
    { 0x383980d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7980d7, SUC_lexicon_tags_31 },
    { 0xfb5980d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325980dd, SUC_lexicon_tags_38 },
    { 0x859980dd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785980e6, SUC_lexicon_tags_7 },
    { 0x3cb980e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d980ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x535980f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11980f3, SUC_lexicon_tags_79 },
    { 0xfd1980f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1980f8, SUC_lexicon_tags_7 },
    { 0x17d980f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b398100, SUC_lexicon_tags_15 },
    { 0x3f798101, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x72198104, SUC_lexicon_tags_12 },
    { 0x99b98105, SUC_lexicon_tags_11 },
    { 0xc998104, SUC_lexicon_tags_35 },
    { 0x6d98107, SUC_lexicon_tags_0 },
    { 0x12598108, SUC_lexicon_tags_17 },
    { 0x4ed98109, SUC_lexicon_tags_27 },
    { 0x84b9810a, SUC_lexicon_tags_11 },
    { 0x3179810a, SUC_lexicon_tags_9 },
    { 0xdbb98106, SUC_lexicon_tags_43 },
    { 0xbc39810d, SUC_lexicon_tags_79 },
    { 0xbaf9810d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f998112, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf98115, SUC_lexicon_tags_28 },
    { 0xd0598115, SUC_lexicon_tags_12 },
    { 0x98398117, SUC_lexicon_tags_70 },
    { 0xd6798117, SUC_lexicon_tags_19 },
    { 0xe0398116, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf99811b, SUC_lexicon_tags_14 },
    { 0xc279811b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb9811f, SUC_lexicon_tags_25 },
    { 0x25f98120, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab98128, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4319812c, SUC_lexicon_tags_47 },
    { 0xfc59812d, SUC_lexicon_tags_2 },
    { 0x90b9812c, SUC_lexicon_tags_37 },
    { 0xcaf9812f, SUC_lexicon_tags_2 },
    { 0x32b98130, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5798137, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd9813a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a19813f, SUC_lexicon_tags_12 },
    { 0x45f9813f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec798145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64398147, SUC_lexicon_tags_49 },
    { 0xdfb98147, SUC_lexicon_tags_19 },
    { 0x55198149, SUC_lexicon_tags_19 },
    { 0x9f198148, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed9814f, SUC_lexicon_tags_129 },
    { 0xb4b98150, SUC_lexicon_tags_7 },
    { 0x54998150, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2798155, SUC_lexicon_tags_7 },
    { 0x56798156, SUC_lexicon_tags_25 },
    { 0xee798157, SUC_lexicon_tags_15 },
    { 0xea598158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10198162, SUC_lexicon_tags_7 },
    { 0xa5198162, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x38d98166, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd9816c, SUC_lexicon_tags_2 },
    { 0xb3f9816d, SUC_lexicon_tags_11 },
    { 0x7b9816e, SUC_lexicon_tags_33 },
    { 0x20f9816f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8f98172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0998176, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d9817a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af9817d, SUC_lexicon_tags_10 },
    { 0x2119817e, SUC_lexicon_tags_9 },
    { 0xa919817d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16398182, SUC_lexicon_tags_47 },
    { 0x14f98183, SUC_lexicon_tags_23 },
    { 0x82598183, SUC_lexicon_tags_25 },
    { 0x94798185, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8059818d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c79818f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a998191, SUC_lexicon_tags_25 },
    { 0x71998192, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2779819a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef9819e, SUC_lexicon_tags_14 },
    { 0x4119819f, SUC_lexicon_tags_15 },
    { 0x5e9981a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205981a5, SUC_lexicon_tags_14 },
    { 0xf83981a5, SUC_lexicon_tags_46 },
    { 0xbd5981a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f981aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359981af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf981b3, SUC_lexicon_tags_38 },
    { 0x603981b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27981b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5981bb, SUC_lexicon_tags_14 },
    { 0xd63981bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab981c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x789981c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbcf981c6, SUC_lexicon_tags_33 },
    { 0x76d981c7, SUC_lexicon_tags_37 },
    { 0x4bf981c8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45981d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7981d8, SUC_lexicon_tags_8 },
    { 0xc7f981d8, SUC_lexicon_tags_13 },
    { 0x95b981da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27b981dc, SUC_lexicon_tags_2 },
    { 0x565981dd, SUC_lexicon_tags_25 },
    { 0x4b981dc, SUC_lexicon_tags_35 },
    { 0x8eb981df, SUC_lexicon_tags_17 },
    { 0xc99981e0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x895981e2, SUC_lexicon_tags_3 },
    { 0x327981e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d981e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285981ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71981f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957981fa, SUC_lexicon_tags_8 },
    { 0xdb9981fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8598202, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x68598205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebb98207, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb98210, SUC_lexicon_tags_40 },
    { 0x66998210, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8479821c, SUC_lexicon_tags_1 },
    { 0xc819821c, SUC_lexicon_tags_17 },
    { 0x8899821e, SUC_lexicon_tags_87 },
    { 0x7c79821e, SUC_lexicon_tags_10 },
    { 0xd5d98220, SUC_lexicon_tags_20 },
    { 0x7219821f, SUC_lexicon_tags_25 },
    { 0x2d19821c, SUC_lexicon_tags_30 },
    { 0x56798221, SUC_lexicon_tags_0 },
    { 0xd1598224, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xbab98227, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc19822c, SUC_lexicon_tags_33 },
    { 0xeb39822c, SUC_lexicon_tags_39 },
    { 0x1639822e, SUC_lexicon_tags_2 },
    { 0x20d9822e, SUC_lexicon_tags_1 },
    { 0xef39822c, SUC_lexicon_tags_14 },
    { 0xdbd98231, SUC_lexicon_tags_2 },
    { 0x24998232, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf98236, SUC_lexicon_tags_25 },
    { 0x17398237, SUC_lexicon_tags_37 },
    { 0xa2d98237, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xea79823a, SUC_lexicon_tags_38 },
    { 0x1899823a, SUC_lexicon_tags_15 },
    { 0x1659823a, SUC_lexicon_tags_17 },
    { 0x3659823b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd98241, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b98246, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5679824d, SUC_lexicon_tags_11 },
    { 0xde79824e, SUC_lexicon_tags_8 },
    { 0x2d19824e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b98253, SUC_lexicon_tags_23 },
    { 0x9c998254, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d39825c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4419825e, SUC_lexicon_tags_12 },
    { 0x65b9825f, SUC_lexicon_tags_11 },
    { 0xed98260, SUC_lexicon_tags_2 },
    { 0x8b398261, SUC_lexicon_tags_49 },
    { 0x37198261, SUC_lexicon_tags_31 },
    { 0x70998262, SUC_lexicon_tags_92 },
    { 0xa798264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d598266, SUC_lexicon_tags_11 },
    { 0x8e398267, SUC_lexicon_tags_80 },
    { 0x51398268, SUC_lexicon_tags_46 },
    { 0xc1d98266, SUC_lexicon_tags_0 },
    { 0xb239826a, SUC_lexicon_tags_37 },
    { 0xd679826a, SUC_lexicon_tags_14 },
    { 0x1859826c, SUC_lexicon_tags_11 },
    { 0x3179826d, SUC_lexicon_tags_31 },
    { 0xedd9826e, SUC_lexicon_tags_28 },
    { 0x3398266, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0198277, SUC_lexicon_tags_17 },
    { 0xecf98277, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39827b, SUC_lexicon_tags_74 },
    { 0xbe99827c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d398282, SUC_lexicon_tags_31 },
    { 0x7e598282, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f9828a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27598291, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeed98293, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d998295, SUC_lexicon_tags_37 },
    { 0x4c398296, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xedd98298, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdd59829a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8d9829e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f982a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xca1982a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeb3982a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x941982a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x227982aa, SUC_lexicon_tags_15 },
    { 0x209982ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x213982ae, SUC_lexicon_tags_1 },
    { 0xe23982af, SUC_lexicon_tags_46 },
    { 0x807982b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd982b6, SUC_lexicon_tags_25 },
    { 0x993982b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601982bc, SUC_lexicon_tags_30 },
    { 0x12f982bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef982c0, SUC_lexicon_tags_37 },
    { 0x323982c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf982c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7982c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153982cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f982d2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4a9982d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x383982d6, SUC_lexicon_tags_15 },
    { 0x311982d7, SUC_lexicon_tags_38 },
    { 0x4a5982d8, SUC_lexicon_tags_9 },
    { 0x6a7982d9, SUC_lexicon_tags_68 },
    { 0xce1982d6, SUC_lexicon_tags_60 },
    { 0xfbd982d6, SUC_lexicon_tags_25 },
    { 0x6e7982d7, SUC_lexicon_tags_9 },
    { 0x67982dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf982e4, SUC_lexicon_tags_45 },
    { 0xb1982e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f3982e7, SUC_lexicon_tags_12 },
    { 0x92f982e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9982ee, SUC_lexicon_tags_14 },
    { 0xec1982ef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7982f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd982f8, SUC_lexicon_tags_17 },
    { 0xb27982f8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed98309, SUC_lexicon_tags_15 },
    { 0xcc798309, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9759830f, SUC_lexicon_tags_23 },
    { 0xf1598310, SUC_lexicon_tags_1 },
    { 0x9d798311, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1759831a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d98320, SUC_lexicon_tags_57 },
    { 0x10d98321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f9832e, SUC_lexicon_tags_207 },
    { 0x6059832e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x14d98332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4398334, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af98338, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6798342, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b98345, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0d98347, SUC_lexicon_tags_25 },
    { 0x74d98348, SUC_lexicon_tags_1 },
    { 0x198349, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe9b9834b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4dd9834d, SUC_lexicon_tags_45 },
    { 0x57d9834e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a798352, SUC_lexicon_tags_27 },
    { 0xc3998352, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39798359, SUC_lexicon_tags_19 },
    { 0x6b39835a, SUC_lexicon_tags_33 },
    { 0x83598359, SUC_lexicon_tags_12 },
    { 0x90f9835c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe39835e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54398362, SUC_lexicon_tags_15 },
    { 0xeb798363, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x16998366, SUC_lexicon_tags_12 },
    { 0x2d998367, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8b98370, SUC_lexicon_tags_25 },
    { 0x8a398371, SUC_lexicon_tags_12 },
    { 0xd798372, SUC_lexicon_tags_113 },
    { 0x38198373, SUC_lexicon_tags_11 },
    { 0x4ff98374, SUC_lexicon_tags_23 },
    { 0x12798374, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45398379, SUC_lexicon_tags_5 },
    { 0xcff9837a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1b9837c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60d98384, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xad198387, SUC_lexicon_tags_37 },
    { 0xbe198387, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3439838a, SUC_lexicon_tags_11 },
    { 0xde39838a, SUC_lexicon_tags_17 },
    { 0xa19838c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfad9838f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9839c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9983a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d983a8, SUC_lexicon_tags_80 },
    { 0xdff983a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x107983ac, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed983af, SUC_lexicon_tags_91 },
    { 0x26b983b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af983b3, SUC_lexicon_tags_10 },
    { 0x8b1983b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddb983b6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xddd983b8, SUC_lexicon_tags_12 },
    { 0x395983b9, SUC_lexicon_tags_11 },
    { 0x619983ba, SUC_lexicon_tags_25 },
    { 0x513983b8, SUC_lexicon_tags_37 },
    { 0x6bf983bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5b983be, SUC_lexicon_tags_11 },
    { 0x86d983bf, SUC_lexicon_tags_160 },
    { 0xd6d983c0, SUC_lexicon_tags_81 },
    { 0xccb983c1, SUC_lexicon_tags_0 },
    { 0x575983c2, SUC_lexicon_tags_30 },
    { 0x8db983c2, SUC_lexicon_tags_8 },
    { 0xe63983c4, SUC_lexicon_tags_9 },
    { 0x42d983c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f983ca, SUC_lexicon_tags_891 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5983d2, SUC_lexicon_tags_21 },
    { 0x1f983d3, SUC_lexicon_tags_14 },
    { 0x41d983d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5983d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f983dc, SUC_lexicon_tags_11 },
    { 0x2ef983dd, SUC_lexicon_tags_39 },
    { 0xa27983de, SUC_lexicon_tags_17 },
    { 0xd49983de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329983e3, SUC_lexicon_tags_25 },
    { 0xd05983e3, SUC_lexicon_tags_15 },
    { 0xbb5983e5, SUC_lexicon_tags_49 },
    { 0x4a7983e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297983ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d983f0, SUC_lexicon_tags_15 },
    { 0x8d1983f1, SUC_lexicon_tags_45 },
    { 0x61f983f2, SUC_lexicon_tags_27 },
    { 0x785983f3, SUC_lexicon_tags_1 },
    { 0x6bf983f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781983f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435983ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9598401, SUC_lexicon_tags_24 },
    { 0x36798402, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed98406, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b9840a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76398410, SUC_lexicon_tags_65 },
    { 0x50398410, SUC_lexicon_tags_235 },
    { 0x69398411, SUC_lexicon_tags_93 },
    { 0x25598413, SUC_lexicon_tags_25 },
    { 0x9df98414, SUC_lexicon_tags_16 },
    { 0x3cd98414, SUC_lexicon_tags_7 },
    { 0x2a998416, SUC_lexicon_tags_13 },
    { 0xb4398417, SUC_lexicon_tags_4 },
    { 0xbb398417, SUC_lexicon_tags_17 },
    { 0x6198411, SUC_lexicon_tags_37 },
    { 0xfd98415, SUC_lexicon_tags_11 },
    { 0xce99841b, SUC_lexicon_tags_37 },
    { 0xd9f98412, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62b9841f, SUC_lexicon_tags_1 },
    { 0xd579841f, SUC_lexicon_tags_8 },
    { 0x50998420, SUC_lexicon_tags_55 },
    { 0x99198422, SUC_lexicon_tags_25 },
    { 0x1e198423, SUC_lexicon_tags_25 },
    { 0x37d98423, SUC_lexicon_tags_27 },
    { 0x3598423, SUC_lexicon_tags_26 },
    { 0xf3998426, SUC_lexicon_tags_26 },
    { 0x51398421, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47598429, SUC_lexicon_tags_1 },
    { 0xd539842a, SUC_lexicon_tags_2 },
    { 0xbdb9842b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24598431, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x84198433, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x70398436, SUC_lexicon_tags_43 },
    { 0xbb198437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c198439, SUC_lexicon_tags_46 },
    { 0x8919843a, SUC_lexicon_tags_9 },
    { 0x5e39843b, SUC_lexicon_tags_25 },
    { 0x93b98439, SUC_lexicon_tags_7 },
    { 0x58b9843d, SUC_lexicon_tags_12 },
    { 0x6ef9843c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x14998441, SUC_lexicon_tags_1 },
    { 0x52798442, SUC_lexicon_tags_15 },
    { 0x74d98443, SUC_lexicon_tags_37 },
    { 0x2dd98444, SUC_lexicon_tags_9 },
    { 0x72198442, SUC_lexicon_tags_14 },
    { 0xbf798446, SUC_lexicon_tags_38 },
    { 0x22f98446, SUC_lexicon_tags_87 },
    { 0x2ad98448, SUC_lexicon_tags_25 },
    { 0xe6f98448, SUC_lexicon_tags_113 },
    { 0xa8d98441, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8479844d, SUC_lexicon_tags_17 },
    { 0x1799844e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x16d98450, SUC_lexicon_tags_25 },
    { 0xe9598450, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe6798453, SUC_lexicon_tags_45 },
    { 0xd98454, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x22d98456, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3c798458, SUC_lexicon_tags_29 },
    { 0xdd198458, SUC_lexicon_tags_19 },
    { 0x8ff9845a, SUC_lexicon_tags_10 },
    { 0xd7198459, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9159845f, SUC_lexicon_tags_11 },
    { 0x9c198460, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd598464, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74998467, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8179846d, SUC_lexicon_tags_19 },
    { 0x6379846e, SUC_lexicon_tags_25 },
    { 0xdc79846f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb98477, SUC_lexicon_tags_12 },
    { 0x47b98478, SUC_lexicon_tags_17 },
    { 0x8f798477, SUC_lexicon_tags_37 },
    { 0xbcf98478, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b998486, SUC_lexicon_tags_47 },
    { 0x2fd98487, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff9848a, SUC_lexicon_tags_87 },
    { 0xe219848b, SUC_lexicon_tags_14 },
    { 0x1f19848c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6598492, SUC_lexicon_tags_12 },
    { 0x54198492, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79398498, SUC_lexicon_tags_2 },
    { 0xed598498, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x6a39849b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b9849e, SUC_lexicon_tags_7 },
    { 0x1fd9849f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf99984a1, SUC_lexicon_tags_11 },
    { 0xa93984a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47984a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9984a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373984ae, SUC_lexicon_tags_49 },
    { 0x277984af, SUC_lexicon_tags_40 },
    { 0x9c7984b0, SUC_lexicon_tags_7 },
    { 0x30b984b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x58b984b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x987984b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b984b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61984be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7984c2, SUC_lexicon_tags_26 },
    { 0x21d984c3, SUC_lexicon_tags_47 },
    { 0xe67984c4, SUC_lexicon_tags_1 },
    { 0x73f984c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3984c8, SUC_lexicon_tags_2 },
    { 0xbbb984c9, SUC_lexicon_tags_11 },
    { 0x391984ca, SUC_lexicon_tags_25 },
    { 0x93b984cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x339984ce, SUC_lexicon_tags_46 },
    { 0x295984cf, SUC_lexicon_tags_28 },
    { 0x693984cf, SUC_lexicon_tags_9 },
    { 0x4e7984d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83984d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607984db, SUC_lexicon_tags_8 },
    { 0xdf7984db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b984df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x10f984e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x951984e3, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85984e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x35b984ea, SUC_lexicon_tags_14 },
    { 0x61d984eb, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x979984ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfe1984f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95984f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59984f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673984fc, SUC_lexicon_tags_37 },
    { 0xf79984fd, SUC_lexicon_tags_28 },
    { 0xba7984fd, SUC_lexicon_tags_25 },
    { 0xb77984fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1998501, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb9850b, SUC_lexicon_tags_170 },
    { 0x38d9850c, SUC_lexicon_tags_25 },
    { 0xcb99850b, SUC_lexicon_tags_15 },
    { 0xe219850e, SUC_lexicon_tags_37 },
    { 0x9d79850b, SUC_lexicon_tags_9 },
    { 0x9d398510, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f79851a, SUC_lexicon_tags_7 },
    { 0x899851a, SUC_lexicon_tags_37 },
    { 0x13f9851b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb798522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7b98524, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5f98526, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35998529, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f9852c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d9852f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97198531, SUC_lexicon_tags_28 },
    { 0xa8f98532, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b98535, SUC_lexicon_tags_37 },
    { 0x4f198536, SUC_lexicon_tags_1 },
    { 0x98b98536, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b9853c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f39853e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64b98540, SUC_lexicon_tags_7 },
    { 0xbb598541, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa998547, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f98549, SUC_lexicon_tags_7 },
    { 0xab79854a, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe359854e, SUC_lexicon_tags_3 },
    { 0xb459854f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17398553, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb98559, SUC_lexicon_tags_11 },
    { 0x3259855a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6b9855c, SUC_lexicon_tags_141 },
    { 0x37b9855c, SUC_lexicon_tags_38 },
    { 0xc2d9855c, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f98562, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db98566, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb8b98568, SUC_lexicon_tags_37 },
    { 0xf5798569, SUC_lexicon_tags_1 },
    { 0x8159856a, SUC_lexicon_tags_0 },
    { 0x49d9856b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2f39856d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad998575, SUC_lexicon_tags_9 },
    { 0xb4b98576, SUC_lexicon_tags_11 },
    { 0xf7398575, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f9857c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5598580, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf998586, SUC_lexicon_tags_2 },
    { 0x7b98587, SUC_lexicon_tags_7 },
    { 0x72f98586, SUC_lexicon_tags_37 },
    { 0xfb98586, SUC_lexicon_tags_12 },
    { 0xd8b9858a, SUC_lexicon_tags_37 },
    { 0x4ed9858b, SUC_lexicon_tags_14 },
    { 0xc679858c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b9858f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a798596, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d19859e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37985a5, SUC_lexicon_tags_11 },
    { 0xde5985a6, SUC_lexicon_tags_7 },
    { 0xf5b985a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ab985a9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905985ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf985b1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x2c1985b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d985b7, SUC_lexicon_tags_12 },
    { 0x3a7985b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ad985ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7985bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x85f985bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x35d985c1, SUC_lexicon_tags_30 },
    { 0x429985c2, SUC_lexicon_tags_12 },
    { 0xfeb985c3, SUC_lexicon_tags_6 },
    { 0xfc1985c1, SUC_lexicon_tags_45 },
    { 0x3cb985c5, SUC_lexicon_tags_28 },
    { 0xb83985c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf6b985c8, SUC_lexicon_tags_37 },
    { 0x6b3985c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f985d3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967985d8, SUC_lexicon_tags_0 },
    { 0x4f5985d9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673985de, SUC_lexicon_tags_0 },
    { 0x8c1985df, SUC_lexicon_tags_1 },
    { 0x883985e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a3985e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7985e6, SUC_lexicon_tags_28 },
    { 0xe19985e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf985eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355985f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba9985f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb985f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1985f9, SUC_lexicon_tags_63 },
    { 0x4af985fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987985ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf98605, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad98607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4999860d, SUC_lexicon_tags_15 },
    { 0xed39860d, SUC_lexicon_tags_104 },
    { 0xce39860e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f98615, SUC_lexicon_tags_29 },
    { 0xbf398615, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef9861b, SUC_lexicon_tags_25 },
    { 0x7919861c, SUC_lexicon_tags_7 },
    { 0xccf9861c, SUC_lexicon_tags_23 },
    { 0xf639861b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d998622, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e598625, SUC_lexicon_tags_17 },
    { 0x1c798626, SUC_lexicon_tags_49 },
    { 0xe3d98627, SUC_lexicon_tags_0 },
    { 0xae998625, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b9862d, SUC_lexicon_tags_25 },
    { 0xc099862e, SUC_lexicon_tags_87 },
    { 0xe419862d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1798631, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31598635, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f98638, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3599863e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b398641, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f98648, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6f39864a, SUC_lexicon_tags_9 },
    { 0x41d9864b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3259864f, SUC_lexicon_tags_7 },
    { 0xc459864f, SUC_lexicon_tags_15 },
    { 0x67f98651, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc5d98653, SUC_lexicon_tags_3 },
    { 0xe0398654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc798657, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dd98659, SUC_lexicon_tags_23 },
    { 0x7d39865a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7599865d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a598661, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbef98664, SUC_lexicon_tags_11 },
    { 0x4f798665, SUC_lexicon_tags_9 },
    { 0x34398665, SUC_lexicon_tags_17 },
    { 0xc2198664, SUC_lexicon_tags_34 },
    { 0xc2b98664, SUC_lexicon_tags_37 },
    { 0x33b98669, SUC_lexicon_tags_11 },
    { 0xcb39866a, SUC_lexicon_tags_1 },
    { 0x6159866a, SUC_lexicon_tags_29 },
    { 0x5759866c, SUC_lexicon_tags_37 },
    { 0xf5f98668, SUC_lexicon_tags_65 },
    { 0xca39866e, SUC_lexicon_tags_23 },
    { 0xa4d9866e, SUC_lexicon_tags_7 },
    { 0x4319866f, SUC_lexicon_tags_37 },
    { 0xffd9866f, SUC_lexicon_tags_70 },
    { 0xa9398670, SUC_lexicon_tags_10 },
    { 0xaf798673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71798682, SUC_lexicon_tags_1 },
    { 0x6d598683, SUC_lexicon_tags_37 },
    { 0x18b98684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b98687, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa499868e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18798692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f19869b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1619869d, SUC_lexicon_tags_19 },
    { 0xb4f9869e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d986a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81986a8, SUC_lexicon_tags_26 },
    { 0x179986a8, SUC_lexicon_tags_1 },
    { 0xfaf986aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c7986ac, SUC_lexicon_tags_1 },
    { 0xb93986ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69f986af, SUC_lexicon_tags_20 },
    { 0x1df986b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbd986b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xead986b4, SUC_lexicon_tags_19 },
    { 0x147986b5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487986bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547986c2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1986c7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d986cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f986ce, SUC_lexicon_tags_19 },
    { 0xa6d986cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437986d5, SUC_lexicon_tags_31 },
    { 0x167986d6, SUC_lexicon_tags_23 },
    { 0x7f7986d7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xabb986d9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7986df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdab986e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x835986e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x191986e5, SUC_lexicon_tags_15 },
    { 0x14b986e6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x361986e9, SUC_lexicon_tags_14 },
    { 0x7e9986ea, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4c5986ec, SUC_lexicon_tags_46 },
    { 0x37f986ec, SUC_lexicon_tags_19 },
    { 0x609986ee, SUC_lexicon_tags_37 },
    { 0x185986ee, SUC_lexicon_tags_26 },
    { 0xe85986ec, SUC_lexicon_tags_25 },
    { 0xb63986ed, SUC_lexicon_tags_11 },
    { 0x2f3986ee, SUC_lexicon_tags_2 },
    { 0x517986f3, SUC_lexicon_tags_8 },
    { 0xf85986ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f986fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f986ff, SUC_lexicon_tags_42 },
    { 0x57398700, SUC_lexicon_tags_35 },
    { 0x37598701, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3998704, SUC_lexicon_tags_1 },
    { 0x5f998705, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1998707, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x20f98709, SUC_lexicon_tags_9 },
    { 0xcd198709, SUC_lexicon_tags_84 },
    { 0xc839870b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x58d9870d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7598717, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae798719, SUC_lexicon_tags_7 },
    { 0x9099871a, SUC_lexicon_tags_87 },
    { 0x6a19871a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13198727, SUC_lexicon_tags_1 },
    { 0x17198728, SUC_lexicon_tags_63 },
    { 0x79798728, SUC_lexicon_tags_32 },
    { 0xf5798729, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa198732, SUC_lexicon_tags_16 },
    { 0xbd198733, SUC_lexicon_tags_65 },
    { 0xf7f98734, SUC_lexicon_tags_87 },
    { 0x9a998735, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22b98737, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb598739, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a59873e, SUC_lexicon_tags_11 },
    { 0x60b9873f, SUC_lexicon_tags_23 },
    { 0x56b9873f, SUC_lexicon_tags_10 },
    { 0x91b9873f, SUC_lexicon_tags_42 },
    { 0x68998742, SUC_lexicon_tags_27 },
    { 0xedb9873f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef9874a, SUC_lexicon_tags_1 },
    { 0xd4f9874b, SUC_lexicon_tags_7 },
    { 0xeb99874a, SUC_lexicon_tags_33 },
    { 0x1c19874c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b98752, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb119875a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50d9875d, SUC_lexicon_tags_15 },
    { 0xe259875d, SUC_lexicon_tags_28 },
    { 0xf119875f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaef98767, SUC_lexicon_tags_37 },
    { 0xc2998767, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a59876f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb98773, SUC_lexicon_tags_44 },
    { 0x7db98774, SUC_lexicon_tags_47 },
    { 0x48d98775, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3598778, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b9877d, SUC_lexicon_tags_8 },
    { 0xeb59877d, SUC_lexicon_tags_17 },
    { 0x26b9877f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x65998782, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95798786, SUC_lexicon_tags_2 },
    { 0x84f98787, SUC_lexicon_tags_37 },
    { 0x7fb98786, SUC_lexicon_tags_1 },
    { 0x15198788, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf8f9878b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d98790, SUC_lexicon_tags_7 },
    { 0x8b598790, SUC_lexicon_tags_17 },
    { 0x73b98790, SUC_lexicon_tags_17 },
    { 0xbdb98791, SUC_lexicon_tags_14 },
    { 0xf398790, SUC_lexicon_tags_11 },
    { 0xfbf98795, SUC_lexicon_tags_45 },
    { 0xe9798796, SUC_lexicon_tags_25 },
    { 0xc0d98797, SUC_lexicon_tags_1 },
    { 0x26b98798, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1987a2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b987a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337987b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119987b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ff987b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d987bb, SUC_lexicon_tags_17 },
    { 0x599987bc, SUC_lexicon_tags_27 },
    { 0x735987bc, SUC_lexicon_tags_11 },
    { 0xb0b987be, SUC_lexicon_tags_32 },
    { 0xc3987be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x997987c1, SUC_lexicon_tags_43 },
    { 0x907987c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3987c9, SUC_lexicon_tags_42 },
    { 0xa7d987ca, SUC_lexicon_tags_29 },
    { 0x5fd987cb, SUC_lexicon_tags_29 },
    { 0x987987cc, SUC_lexicon_tags_15 },
    { 0x17b987cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7987d2, SUC_lexicon_tags_87 },
    { 0x8b5987d2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x323987d5, SUC_lexicon_tags_95 },
    { 0x5f9987d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429987e0, SUC_lexicon_tags_45 },
    { 0xa63987e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc73987e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d987e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x991987e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x307987ec, SUC_lexicon_tags_17 },
    { 0xfc9987ec, SUC_lexicon_tags_35 },
    { 0xcfd987ee, SUC_lexicon_tags_62 },
    { 0xe9f987ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x261987f1, SUC_lexicon_tags_12 },
    { 0x4cd987f1, SUC_lexicon_tags_23 },
    { 0x7f5987f1, SUC_lexicon_tags_65 },
    { 0x9ff987f4, SUC_lexicon_tags_0 },
    { 0xbb9987f5, SUC_lexicon_tags_2 },
    { 0x6fd987f6, SUC_lexicon_tags_17 },
    { 0xd63987f7, SUC_lexicon_tags_6 },
    { 0xe17987f8, SUC_lexicon_tags_23 },
    { 0xdd5987f8, SUC_lexicon_tags_15 },
    { 0x14b987f7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x11f987fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced98803, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f9880b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x5df9880d, SUC_lexicon_tags_39 },
    { 0xe1d9880e, SUC_lexicon_tags_23 },
    { 0xb6b9880f, SUC_lexicon_tags_11 },
    { 0x55d98810, SUC_lexicon_tags_31 },
    { 0x6ed9880f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0798813, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4798817, SUC_lexicon_tags_9 },
    { 0xa598818, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71f9881b, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x1219881d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36198824, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd9882a, SUC_lexicon_tags_23 },
    { 0x979882b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5879882f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d598834, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3eb98836, SUC_lexicon_tags_12 },
    { 0x68f98837, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaf798839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81f9883b, SUC_lexicon_tags_11 },
    { 0x6a79883b, SUC_lexicon_tags_11 },
    { 0xe139883d, SUC_lexicon_tags_15 },
    { 0x7af9883e, SUC_lexicon_tags_14 },
    { 0x8499883c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb98849, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1179884b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5f9884d, SUC_lexicon_tags_11 },
    { 0xd559884e, SUC_lexicon_tags_11 },
    { 0x1a99884e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5398852, SUC_lexicon_tags_34 },
    { 0x5e998853, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf3398855, SUC_lexicon_tags_38 },
    { 0xe2198856, SUC_lexicon_tags_33 },
    { 0x19998857, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbbb98859, SUC_lexicon_tags_271 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe579885d, SUC_lexicon_tags_201 },
    { 0x4439885e, SUC_lexicon_tags_9 },
    { 0xecf9885e, SUC_lexicon_tags_10 },
    { 0x9eb98860, SUC_lexicon_tags_17 },
    { 0xe2b98861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6998868, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb99886d, SUC_lexicon_tags_15 },
    { 0xe899886e, SUC_lexicon_tags_11 },
    { 0x1fb9886f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3198872, SUC_lexicon_tags_17 },
    { 0xa598872, SUC_lexicon_tags_63 },
    { 0xead98873, SUC_lexicon_tags_17 },
    { 0x5a998875, SUC_lexicon_tags_33 },
    { 0xd7f98876, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b198878, SUC_lexicon_tags_8 },
    { 0x12398879, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b79887c, SUC_lexicon_tags_32 },
    { 0x7319887d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9739887f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b98884, SUC_lexicon_tags_33 },
    { 0x62998885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab9888b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87198892, SUC_lexicon_tags_15 },
    { 0xdbb98893, SUC_lexicon_tags_13 },
    { 0xed798894, SUC_lexicon_tags_17 },
    { 0xe5998895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2798898, SUC_lexicon_tags_27 },
    { 0x3fd98898, SUC_lexicon_tags_26 },
    { 0x43d98898, SUC_lexicon_tags_17 },
    { 0xed9889b, SUC_lexicon_tags_0 },
    { 0x3af9889b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2a39889e, SUC_lexicon_tags_15 },
    { 0xa8f9889f, SUC_lexicon_tags_6 },
    { 0x8a7988a0, SUC_lexicon_tags_47 },
    { 0x977988a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ed988a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4ef988a5, SUC_lexicon_tags_17 },
    { 0x1b3988a6, SUC_lexicon_tags_56 },
    { 0x1d1988a5, SUC_lexicon_tags_2 },
    { 0x155988a8, SUC_lexicon_tags_25 },
    { 0xd9b988a8, SUC_lexicon_tags_0 },
    { 0xbf1988aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb95988ac, SUC_lexicon_tags_1 },
    { 0xaf988ad, SUC_lexicon_tags_23 },
    { 0x257988ac, SUC_lexicon_tags_12 },
    { 0x3a3988af, SUC_lexicon_tags_27 },
    { 0xf17988b0, SUC_lexicon_tags_28 },
    { 0x2a7988b1, SUC_lexicon_tags_1614 },
    { 0, NULL },
    { 0x24b988b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d988b5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa43988b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9988ba, SUC_lexicon_tags_19 },
    { 0x31988bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a9988bd, SUC_lexicon_tags_11 },
    { 0x5d7988be, SUC_lexicon_tags_33 },
    { 0x147988bf, SUC_lexicon_tags_37 },
    { 0x51b988be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1eb988c2, SUC_lexicon_tags_33 },
    { 0x417988c2, SUC_lexicon_tags_31 },
    { 0x687988c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x803988c7, SUC_lexicon_tags_37 },
    { 0x98b988c7, SUC_lexicon_tags_15 },
    { 0xa4b988c8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9988cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x507988cf, SUC_lexicon_tags_25 },
    { 0x909988d0, SUC_lexicon_tags_23 },
    { 0x1c3988d1, SUC_lexicon_tags_37 },
    { 0xbb3988d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f988d6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7988da, SUC_lexicon_tags_19 },
    { 0x2a5988db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441988e3, SUC_lexicon_tags_19 },
    { 0x8bd988e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe25988e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb29988e8, SUC_lexicon_tags_15 },
    { 0x8e3988e9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69988ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x343988f0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xde3988f2, SUC_lexicon_tags_37 },
    { 0x587988f3, SUC_lexicon_tags_65 },
    { 0x3f7988f4, SUC_lexicon_tags_63 },
    { 0xecb988f5, SUC_lexicon_tags_45 },
    { 0x39f988f6, SUC_lexicon_tags_7 },
    { 0x483988f6, SUC_lexicon_tags_49 },
    { 0xf21988f8, SUC_lexicon_tags_16 },
    { 0x955988f8, SUC_lexicon_tags_14 },
    { 0x787988f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23988fe, SUC_lexicon_tags_12 },
    { 0x32f988fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddb98901, SUC_lexicon_tags_12 },
    { 0xff198902, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f98908, SUC_lexicon_tags_11 },
    { 0xb0d98908, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f9890e, SUC_lexicon_tags_23 },
    { 0x4879890f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc1398911, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x36798913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa798915, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb98919, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d198922, SUC_lexicon_tags_87 },
    { 0x13798923, SUC_lexicon_tags_11 },
    { 0x33398924, SUC_lexicon_tags_87 },
    { 0x6f398922, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38398928, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12f98931, SUC_lexicon_tags_7 },
    { 0x48198932, SUC_lexicon_tags_84 },
    { 0x82f98933, SUC_lexicon_tags_25 },
    { 0x38798932, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67b98938, SUC_lexicon_tags_29 },
    { 0xe0f98939, SUC_lexicon_tags_69 },
    { 0xe6198939, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x519893d, SUC_lexicon_tags_30 },
    { 0x63d9893e, SUC_lexicon_tags_17 },
    { 0x6f9893e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x47b98941, SUC_lexicon_tags_5 },
    { 0x95398942, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf1b98944, SUC_lexicon_tags_25 },
    { 0x3f98945, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9f398947, SUC_lexicon_tags_12 },
    { 0x2a598947, SUC_lexicon_tags_7 },
    { 0x8ef98948, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3139894f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cf98951, SUC_lexicon_tags_194 },
    { 0x99598951, SUC_lexicon_tags_47 },
    { 0x5b98952, SUC_lexicon_tags_8 },
    { 0x1fb98952, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86b98959, SUC_lexicon_tags_15 },
    { 0x8599895a, SUC_lexicon_tags_2 },
    { 0x27b98959, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1798961, SUC_lexicon_tags_19 },
    { 0xac398962, SUC_lexicon_tags_70 },
    { 0x7bb98963, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb119896a, SUC_lexicon_tags_9 },
    { 0x9bd9896b, SUC_lexicon_tags_4 },
    { 0xde39896c, SUC_lexicon_tags_15 },
    { 0x5259896d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82798973, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f998979, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x619897c, SUC_lexicon_tags_11 },
    { 0x20d9897d, SUC_lexicon_tags_7 },
    { 0xdb99897e, SUC_lexicon_tags_56 },
    { 0x19b9897f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3398981, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67998984, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x58798987, SUC_lexicon_tags_25 },
    { 0x23798988, SUC_lexicon_tags_1 },
    { 0xfb398989, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfed9898b, SUC_lexicon_tags_37 },
    { 0x3cd9898c, SUC_lexicon_tags_60 },
    { 0x4859898d, SUC_lexicon_tags_1 },
    { 0xad9898e, SUC_lexicon_tags_37 },
    { 0xd799898c, SUC_lexicon_tags_11 },
    { 0xb0f9898b, SUC_lexicon_tags_48 },
    { 0xeab98990, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0b98993, SUC_lexicon_tags_27 },
    { 0x4598994, SUC_lexicon_tags_25 },
    { 0xb4998995, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b79899c, SUC_lexicon_tags_15 },
    { 0xa039899c, SUC_lexicon_tags_12 },
    { 0xce59899e, SUC_lexicon_tags_33 },
    { 0xacb9899c, SUC_lexicon_tags_7 },
    { 0xa479899e, SUC_lexicon_tags_30 },
    { 0xceb989a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x135989a4, SUC_lexicon_tags_12 },
    { 0xef9989a5, SUC_lexicon_tags_15 },
    { 0x58d989a6, SUC_lexicon_tags_17 },
    { 0xc6d989a7, SUC_lexicon_tags_39 },
    { 0xd6f989a5, SUC_lexicon_tags_26 },
    { 0x63b989a8, SUC_lexicon_tags_2 },
    { 0x121989aa, SUC_lexicon_tags_15 },
    { 0xe2b989ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x775989ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51989b6, SUC_lexicon_tags_1 },
    { 0x23b989b7, SUC_lexicon_tags_19 },
    { 0x517989b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803989bf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f989c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701989c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39989cb, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0xed989ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7dd989d0, SUC_lexicon_tags_7 },
    { 0x48b989d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43b989d3, SUC_lexicon_tags_15 },
    { 0xad5989d3, SUC_lexicon_tags_79 },
    { 0x357989d5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3989d9, SUC_lexicon_tags_10 },
    { 0x689989d9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77989de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8cf989e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3989e4, SUC_lexicon_tags_23 },
    { 0x64f989e5, SUC_lexicon_tags_31 },
    { 0x8e9989e6, SUC_lexicon_tags_1 },
    { 0x35989e6, SUC_lexicon_tags_11 },
    { 0xf1f989e6, SUC_lexicon_tags_17 },
    { 0x29b989e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d989ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b989f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd989f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb989f8, SUC_lexicon_tags_33 },
    { 0x5b3989f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7989fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c998a02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce798a04, SUC_lexicon_tags_17 },
    { 0xb3d98a04, SUC_lexicon_tags_60 },
    { 0xdc398a05, SUC_lexicon_tags_46 },
    { 0x8a398a06, SUC_lexicon_tags_38 },
    { 0xced98a08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d98a0c, SUC_lexicon_tags_37 },
    { 0xf198a0d, SUC_lexicon_tags_17 },
    { 0x5b598a0d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xca198a10, SUC_lexicon_tags_33 },
    { 0x3a198a10, SUC_lexicon_tags_1 },
    { 0x9ed98a12, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb6b98a14, SUC_lexicon_tags_324 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b98a17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d98a1a, SUC_lexicon_tags_49 },
    { 0xaad98a1b, SUC_lexicon_tags_1 },
    { 0x37398a1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c398a1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5598a20, SUC_lexicon_tags_37 },
    { 0x8cd98a21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1598a28, SUC_lexicon_tags_33 },
    { 0x5eb98a29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec398a2b, SUC_lexicon_tags_43 },
    { 0x5a998a2b, SUC_lexicon_tags_17 },
    { 0xbd598a2b, SUC_lexicon_tags_17 },
    { 0xb4198a2e, SUC_lexicon_tags_7 },
    { 0x99998a2b, SUC_lexicon_tags_14 },
    { 0x88398a30, SUC_lexicon_tags_12 },
    { 0x3f598a30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40198a3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf7798a3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7598a42, SUC_lexicon_tags_12 },
    { 0x54198a42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69198a46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47398a48, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d98a4b, SUC_lexicon_tags_17 },
    { 0xe0f98a4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3df98a4f, SUC_lexicon_tags_17 },
    { 0x43f98a50, SUC_lexicon_tags_37 },
    { 0x92798a51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb598a54, SUC_lexicon_tags_7 },
    { 0x5b798a54, SUC_lexicon_tags_45 },
    { 0xa598a56, SUC_lexicon_tags_25 },
    { 0x76f98a57, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2d198a59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43198a5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff998a5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8eb98a5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd98a62, SUC_lexicon_tags_17 },
    { 0xbc998a63, SUC_lexicon_tags_11 },
    { 0xb2998a63, SUC_lexicon_tags_27 },
    { 0x8f798a63, SUC_lexicon_tags_0 },
    { 0x3c598a66, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x62398a69, SUC_lexicon_tags_15 },
    { 0x75398a6a, SUC_lexicon_tags_27 },
    { 0x2cf98a6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3598a71, SUC_lexicon_tags_85 },
    { 0xd5998a72, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67998a77, SUC_lexicon_tags_1 },
    { 0x7af98a78, SUC_lexicon_tags_6 },
    { 0xef198a77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe7398a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd98a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b598a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3db98a84, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2198a87, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4398a89, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b98a8c, SUC_lexicon_tags_7 },
    { 0x198a8c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f98a91, SUC_lexicon_tags_55 },
    { 0x9fd98a91, SUC_lexicon_tags_48 },
    { 0x3a598a92, SUC_lexicon_tags_8 },
    { 0x1798a93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2f98a96, SUC_lexicon_tags_2 },
    { 0x9b98a97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22798a9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6798a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6398aa1, SUC_lexicon_tags_27 },
    { 0xaf598aa2, SUC_lexicon_tags_17 },
    { 0xe1598aa3, SUC_lexicon_tags_27 },
    { 0x8dd98aa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91798aa7, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd98ab3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9198ab5, SUC_lexicon_tags_17 },
    { 0xb4f98ab6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbbd98ab8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10998aba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed998ac0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d398ac3, SUC_lexicon_tags_11 },
    { 0xc3798ac4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81598acc, SUC_lexicon_tags_47 },
    { 0x3ef98acd, SUC_lexicon_tags_38 },
    { 0xb3398acd, SUC_lexicon_tags_16 },
    { 0x71b98acf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x42198ad1, SUC_lexicon_tags_56 },
    { 0xf1f98ad1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1798ad5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb598ad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b98ae0, SUC_lexicon_tags_15 },
    { 0x28798ae0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5c398ae3, SUC_lexicon_tags_12 },
    { 0x7b998ae4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda198ae6, SUC_lexicon_tags_19 },
    { 0x3798ae7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef98aea, SUC_lexicon_tags_12 },
    { 0xcbf98aea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3998af0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33198af2, SUC_lexicon_tags_12 },
    { 0xc1b98af3, SUC_lexicon_tags_11 },
    { 0xb4f98af4, SUC_lexicon_tags_28 },
    { 0x24998af3, SUC_lexicon_tags_1 },
    { 0x51398af6, SUC_lexicon_tags_11 },
    { 0x7e798af7, SUC_lexicon_tags_7 },
    { 0x3d598af8, SUC_lexicon_tags_8 },
    { 0x34998af8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed98afd, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x48198aff, SUC_lexicon_tags_37 },
    { 0xae598aff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x45398b03, SUC_lexicon_tags_13 },
    { 0xd5f98b04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad98b10, SUC_lexicon_tags_30 },
    { 0x35d98b10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d98b16, SUC_lexicon_tags_22 },
    { 0xf398b16, SUC_lexicon_tags_7 },
    { 0x6f998b18, SUC_lexicon_tags_25 },
    { 0x5e198b19, SUC_lexicon_tags_7 },
    { 0x4b398b1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73598b21, SUC_lexicon_tags_17 },
    { 0x95f98b21, SUC_lexicon_tags_2 },
    { 0x73d98b23, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1598b28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72f98b2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab398b2f, SUC_lexicon_tags_6 },
    { 0xb8b98b2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2598b35, SUC_lexicon_tags_63 },
    { 0xfa798b35, SUC_lexicon_tags_0 },
    { 0xde398b37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6998b3a, SUC_lexicon_tags_42 },
    { 0x12d98b3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b98b3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x57b98b40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0b98b42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b98b45, SUC_lexicon_tags_43 },
    { 0xc0998b45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51798b48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e998b4b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x49798b4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb198b51, SUC_lexicon_tags_37 },
    { 0x62798b51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52998b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5398b64, SUC_lexicon_tags_17 },
    { 0x7f998b65, SUC_lexicon_tags_7 },
    { 0xc2198b65, SUC_lexicon_tags_17 },
    { 0x89d98b65, SUC_lexicon_tags_23 },
    { 0x70b98b65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x43398b6b, SUC_lexicon_tags_11 },
    { 0x33598b6c, SUC_lexicon_tags_17 },
    { 0xe6998b6c, SUC_lexicon_tags_37 },
    { 0xad798b6d, SUC_lexicon_tags_47 },
    { 0xc9998b6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7398b74, SUC_lexicon_tags_7 },
    { 0xd2798b75, SUC_lexicon_tags_1 },
    { 0xae798b76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ed98b79, SUC_lexicon_tags_12 },
    { 0xe0d98b79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8b98b7d, SUC_lexicon_tags_17 },
    { 0x20198b7e, SUC_lexicon_tags_38 },
    { 0x6598b7f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x90598b82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x44998b85, SUC_lexicon_tags_14 },
    { 0x8b398b86, SUC_lexicon_tags_60 },
    { 0x5d598b86, SUC_lexicon_tags_39 },
    { 0x998b85, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x2f998b8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47598b8e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d98b91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61398b93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf98b96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb98b99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c598b9d, SUC_lexicon_tags_38 },
    { 0xc2f98b9e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1398ba1, SUC_lexicon_tags_79 },
    { 0x7cb98ba2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f98ba5, SUC_lexicon_tags_37 },
    { 0xe9798ba6, SUC_lexicon_tags_47 },
    { 0x67b98ba7, SUC_lexicon_tags_15 },
    { 0x89398ba8, SUC_lexicon_tags_11 },
    { 0x11398ba9, SUC_lexicon_tags_17 },
    { 0xc0d98ba9, SUC_lexicon_tags_28 },
    { 0xad798ba9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc598bae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bf98bb0, SUC_lexicon_tags_14 },
    { 0xcfd98bb1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbb198bb3, SUC_lexicon_tags_63 },
    { 0x1f98bb3, SUC_lexicon_tags_60 },
    { 0xd7598bb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af98bb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31398bbe, SUC_lexicon_tags_147 },
    { 0x60998bbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14198bc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa4398bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9198bca, SUC_lexicon_tags_62 },
    { 0xb0998bcb, SUC_lexicon_tags_68 },
    { 0x64198bcc, SUC_lexicon_tags_37 },
    { 0x3e598bcd, SUC_lexicon_tags_15 },
    { 0x20f98bca, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58d98bdf, SUC_lexicon_tags_60 },
    { 0xbab98be0, SUC_lexicon_tags_19 },
    { 0xd1d98be0, SUC_lexicon_tags_16 },
    { 0xe3f98be2, SUC_lexicon_tags_30 },
    { 0x34198be3, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa598be9, SUC_lexicon_tags_11 },
    { 0xfbf98bea, SUC_lexicon_tags_27 },
    { 0xc7798be9, SUC_lexicon_tags_14 },
    { 0xb1198beb, SUC_lexicon_tags_14 },
    { 0x90f98bed, SUC_lexicon_tags_204 },
    { 0xc6b98bed, SUC_lexicon_tags_35 },
    { 0xf3998bef, SUC_lexicon_tags_25 },
    { 0x5a798bef, SUC_lexicon_tags_28 },
    { 0xabf98bf0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1798bf3, SUC_lexicon_tags_11 },
    { 0x7a598bf3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d98bf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed98bfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb198c05, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6798c0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf8d98c0c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9ed98c0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1198c11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43998c13, SUC_lexicon_tags_7 },
    { 0x3f598c13, SUC_lexicon_tags_11 },
    { 0x3b198c15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bb98c17, SUC_lexicon_tags_9 },
    { 0xf9198c18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9198c1f, SUC_lexicon_tags_32 },
    { 0x87f98c1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b98c23, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x46598c26, SUC_lexicon_tags_26 },
    { 0x34d98c26, SUC_lexicon_tags_55 },
    { 0xfe198c26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d398c2d, SUC_lexicon_tags_25 },
    { 0xb4998c2e, SUC_lexicon_tags_6 },
    { 0x50f98c2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62f98c31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57398c35, SUC_lexicon_tags_35 },
    { 0x66398c36, SUC_lexicon_tags_12 },
    { 0x49998c36, SUC_lexicon_tags_0 },
    { 0xc8b98c38, SUC_lexicon_tags_7 },
    { 0x42998c35, SUC_lexicon_tags_17 },
    { 0xacb98c3a, SUC_lexicon_tags_8 },
    { 0xe9f98c37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52798c3d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe3198c3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf398c42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2998c47, SUC_lexicon_tags_11 },
    { 0x72b98c48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x46198c4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b998c53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x52598c55, SUC_lexicon_tags_7 },
    { 0xfb98c55, SUC_lexicon_tags_11 },
    { 0xb9f98c56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f98c5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x93998c5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec198c60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59398c67, SUC_lexicon_tags_37 },
    { 0xfef98c68, SUC_lexicon_tags_27 },
    { 0x8d998c68, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x5b598c6b, SUC_lexicon_tags_30 },
    { 0x7d798c6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78798c6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d598c78, SUC_lexicon_tags_12 },
    { 0x71798c78, SUC_lexicon_tags_37 },
    { 0xef398c7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68998c84, SUC_lexicon_tags_57 },
    { 0xc2198c85, SUC_lexicon_tags_2 },
    { 0xb798c86, SUC_lexicon_tags_29 },
    { 0xdb598c86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe598c89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a998c8c, SUC_lexicon_tags_49 },
    { 0x99f98c8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d98c91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f98c94, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1798c98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32398c9b, SUC_lexicon_tags_45 },
    { 0x9dd98c9b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf98ca9, SUC_lexicon_tags_9 },
    { 0x13598caa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f598cb4, SUC_lexicon_tags_33 },
    { 0x68598cb5, SUC_lexicon_tags_12 },
    { 0x33198cb4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7598cb9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3ab98cbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28798cc1, SUC_lexicon_tags_38 },
    { 0x8eb98cc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0998cc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea998cc9, SUC_lexicon_tags_7 },
    { 0xca198cca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff98ccd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x60398ccf, SUC_lexicon_tags_68 },
    { 0x15598ccf, SUC_lexicon_tags_7 },
    { 0xf3b98cd1, SUC_lexicon_tags_31 },
    { 0xbbb98cd1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92d98cd4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d398cd6, SUC_lexicon_tags_17 },
    { 0xaaf98cd6, SUC_lexicon_tags_17 },
    { 0xb5d98cd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44998cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c598cde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c398ce0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cf98ce2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73798ce5, SUC_lexicon_tags_1 },
    { 0x12b98ce6, SUC_lexicon_tags_85 },
    { 0x4c598ce7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33198cec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42d98cee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe998cf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ef98cf8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83998cfd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11598d03, SUC_lexicon_tags_30 },
    { 0xb4f98d04, SUC_lexicon_tags_43 },
    { 0xe4998d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee998d09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa5198d0b, SUC_lexicon_tags_61 },
    { 0x3c398d0b, SUC_lexicon_tags_1 },
    { 0xe2398d0c, SUC_lexicon_tags_2 },
    { 0xf8798d0e, SUC_lexicon_tags_27 },
    { 0x52398d0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25598d15, SUC_lexicon_tags_6 },
    { 0x93198d15, SUC_lexicon_tags_1 },
    { 0xad598d16, SUC_lexicon_tags_19 },
    { 0xced98d18, SUC_lexicon_tags_12 },
    { 0x45998d17, SUC_lexicon_tags_1 },
    { 0xad798d1a, SUC_lexicon_tags_11 },
    { 0xd2b98d1a, SUC_lexicon_tags_9 },
    { 0xcf198d1c, SUC_lexicon_tags_71 },
    { 0x37d98d19, SUC_lexicon_tags_14 },
    { 0xed998d1e, SUC_lexicon_tags_2 },
    { 0x1f98d16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1398d22, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xf8798d24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ad98d2f, SUC_lexicon_tags_37 },
    { 0x90198d2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f98d34, SUC_lexicon_tags_45 },
    { 0x76998d34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47198d38, SUC_lexicon_tags_71 },
    { 0xc5598d39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x38198d3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd998d3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d98d41, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb598d47, SUC_lexicon_tags_37 },
    { 0x6a598d48, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f98d4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91598d4f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x48598d51, SUC_lexicon_tags_12 },
    { 0xe3798d51, SUC_lexicon_tags_34 },
    { 0xab998d53, SUC_lexicon_tags_27 },
    { 0xfa198d52, SUC_lexicon_tags_6 },
    { 0xc5f98d55, SUC_lexicon_tags_113 },
    { 0x92b98d52, SUC_lexicon_tags_87 },
    { 0xe9f98d57, SUC_lexicon_tags_47 },
    { 0xb3f98d57, SUC_lexicon_tags_25 },
    { 0x7ff98d59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5998d5f, SUC_lexicon_tags_230 },
    { 0xeb198d5f, SUC_lexicon_tags_47 },
    { 0xf7798d61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27598d65, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7598d6a, SUC_lexicon_tags_30 },
    { 0xaeb98d6b, SUC_lexicon_tags_30 },
    { 0x99798d6b, SUC_lexicon_tags_31 },
    { 0xca998d6d, SUC_lexicon_tags_38 },
    { 0x3fd98d6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd0f98d70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55f98d73, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab98d79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x14998d7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xded98d7e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc2798d80, SUC_lexicon_tags_107 },
    { 0x44198d81, SUC_lexicon_tags_1 },
    { 0x6c398d82, SUC_lexicon_tags_26 },
    { 0x9db98d80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x46798d86, SUC_lexicon_tags_2 },
    { 0x6c598d87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18b98d8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d998d8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8398d8f, SUC_lexicon_tags_25 },
    { 0xb2398d8f, SUC_lexicon_tags_32 },
    { 0x14798d90, SUC_lexicon_tags_14 },
    { 0x4d998d92, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x60f98d94, SUC_lexicon_tags_15 },
    { 0xa2798d94, SUC_lexicon_tags_26 },
    { 0x6198d94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b98d98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96598d9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7198d9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88798da1, SUC_lexicon_tags_79 },
    { 0x61f98da2, SUC_lexicon_tags_25 },
    { 0xdcd98da3, SUC_lexicon_tags_38 },
    { 0xe4398da4, SUC_lexicon_tags_63 },
    { 0xc9798da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb98da9, SUC_lexicon_tags_15 },
    { 0x8a598daa, SUC_lexicon_tags_39 },
    { 0x31f98dab, SUC_lexicon_tags_927 },
    { 0xd9b98daa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x60b98dae, SUC_lexicon_tags_29 },
    { 0x97d98dae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a798db2, SUC_lexicon_tags_31 },
    { 0x9a198db3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a798dbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1798dc5, SUC_lexicon_tags_43 },
    { 0x59d98dc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28398dc9, SUC_lexicon_tags_9 },
    { 0xd7b98dca, SUC_lexicon_tags_17 },
    { 0x35b98dcb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92998dcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac598dd1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x88198dd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15198dd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4998ddb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5398dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc398de2, SUC_lexicon_tags_25 },
    { 0xae198de3, SUC_lexicon_tags_14 },
    { 0x4a998de2, SUC_lexicon_tags_37 },
    { 0x1ef98de4, SUC_lexicon_tags_18 },
    { 0xf5598de6, SUC_lexicon_tags_26 },
    { 0x57998de5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b998def, SUC_lexicon_tags_127 },
    { 0x9ed98df0, SUC_lexicon_tags_12 },
    { 0x62598df1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89798df7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cd98dfa, SUC_lexicon_tags_26 },
    { 0x45f98dfb, SUC_lexicon_tags_37 },
    { 0x2e198dfb, SUC_lexicon_tags_11 },
    { 0xc7598dfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd398e02, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b98e05, SUC_lexicon_tags_17 },
    { 0x31998e05, SUC_lexicon_tags_37 },
    { 0x57598e06, SUC_lexicon_tags_33 },
    { 0xa6198e07, SUC_lexicon_tags_11 },
    { 0x27398e09, SUC_lexicon_tags_25 },
    { 0xcc398e09, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b98e0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22798e0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b98e14, SUC_lexicon_tags_37 },
    { 0x6e198e15, SUC_lexicon_tags_32 },
    { 0x57998e16, SUC_lexicon_tags_118 },
    { 0xaf798e16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70d98e1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9998e20, SUC_lexicon_tags_23 },
    { 0x74398e21, SUC_lexicon_tags_15 },
    { 0x2e198e22, SUC_lexicon_tags_37 },
    { 0xe1998e23, SUC_lexicon_tags_7 },
    { 0xce998e21, SUC_lexicon_tags_68 },
    { 0x5ab98e25, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xeed98e27, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb98e2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d98e32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5398e39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad598e3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x18f98e3e, SUC_lexicon_tags_43 },
    { 0x6cf98e3e, SUC_lexicon_tags_25 },
    { 0x86d98e40, SUC_lexicon_tags_34 },
    { 0x4bb98e41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c598e45, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x12b98e47, SUC_lexicon_tags_11 },
    { 0x2d998e47, SUC_lexicon_tags_27 },
    { 0x6b198e49, SUC_lexicon_tags_6 },
    { 0x1ab98e48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40f98e4d, SUC_lexicon_tags_37 },
    { 0xfcf98e4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83798e52, SUC_lexicon_tags_25 },
    { 0xde398e52, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba798e55, SUC_lexicon_tags_2 },
    { 0x31798e56, SUC_lexicon_tags_17 },
    { 0x35998e55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f98e5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeeb98e5c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xce198e5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd98e61, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21398e66, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xf1798e68, SUC_lexicon_tags_17 },
    { 0xa1998e69, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x24398e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda998e6e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8398e71, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x37b98e73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c998e75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb98e78, SUC_lexicon_tags_15 },
    { 0xc3798e78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c198e7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb8598e7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77198e83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6d98e89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5b98e8b, SUC_lexicon_tags_1 },
    { 0xa9798e8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f98e8f, SUC_lexicon_tags_23 },
    { 0xb9d98e8f, SUC_lexicon_tags_37 },
    { 0x2f198e90, SUC_lexicon_tags_63 },
    { 0xd398e91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b398e94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10598e98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d98e9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdef98e9f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd98ea2, SUC_lexicon_tags_11 },
    { 0x5b998ea3, SUC_lexicon_tags_25 },
    { 0x98198ea3, SUC_lexicon_tags_21 },
    { 0x7a198ea5, SUC_lexicon_tags_46 },
    { 0xa6f98ea4, SUC_lexicon_tags_72 },
    { 0x53598ea7, SUC_lexicon_tags_9 },
    { 0x15798ea5, SUC_lexicon_tags_17 },
    { 0x42198ea9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6f98eab, SUC_lexicon_tags_10 },
    { 0xef198eab, SUC_lexicon_tags_25 },
    { 0x16b98ead, SUC_lexicon_tags_15 },
    { 0xf6998eac, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5198eb1, SUC_lexicon_tags_72 },
    { 0x41d98eb2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe798eb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8198eb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94998ebc, SUC_lexicon_tags_23 },
    { 0xccf98ebd, SUC_lexicon_tags_25 },
    { 0x70198ebe, SUC_lexicon_tags_14 },
    { 0x70798ebf, SUC_lexicon_tags_11 },
    { 0xb5398ec0, SUC_lexicon_tags_37 },
    { 0xb8398ec0, SUC_lexicon_tags_12 },
    { 0x53f98ebf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xa9f98ec4, SUC_lexicon_tags_9 },
    { 0x93f98ec5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65198ec9, SUC_lexicon_tags_9 },
    { 0xfb398eca, SUC_lexicon_tags_4 },
    { 0x64b98eca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68f98ecd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3598ed2, SUC_lexicon_tags_13 },
    { 0x3cd98ed3, SUC_lexicon_tags_0 },
    { 0x7e398ed4, SUC_lexicon_tags_14 },
    { 0x92d98ed3, SUC_lexicon_tags_23 },
    { 0x53198ed3, SUC_lexicon_tags_12 },
    { 0xa4798ed5, SUC_lexicon_tags_38 },
    { 0x47198ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89998edc, SUC_lexicon_tags_30 },
    { 0xc7b98edd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdb798edf, SUC_lexicon_tags_19 },
    { 0xf2398edf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38f98ee2, SUC_lexicon_tags_37 },
    { 0x5f798ee2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f98ee6, SUC_lexicon_tags_25 },
    { 0xf4b98ee6, SUC_lexicon_tags_29 },
    { 0xcfd98ee6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49798eeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d98eee, SUC_lexicon_tags_6 },
    { 0x8eb98eef, SUC_lexicon_tags_7 },
    { 0xfc398ef0, SUC_lexicon_tags_28 },
    { 0x35f98ef1, SUC_lexicon_tags_4 },
    { 0x2b998ef2, SUC_lexicon_tags_11 },
    { 0x19f98ef2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x13198ef5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x93b98ef7, SUC_lexicon_tags_5 },
    { 0x91d98ef7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd4d98efa, SUC_lexicon_tags_60 },
    { 0x8798efb, SUC_lexicon_tags_12 },
    { 0x61798efc, SUC_lexicon_tags_8 },
    { 0x75198efb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xff998f00, SUC_lexicon_tags_11 },
    { 0x44198f01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6198f03, SUC_lexicon_tags_63 },
    { 0xa2d98f03, SUC_lexicon_tags_11 },
    { 0x9d798f03, SUC_lexicon_tags_87 },
    { 0xc8f98f06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d98f09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3198f0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69198f0f, SUC_lexicon_tags_63 },
    { 0x6e398f10, SUC_lexicon_tags_47 },
    { 0x75198f0f, SUC_lexicon_tags_0 },
    { 0xf5598f10, SUC_lexicon_tags_2 },
    { 0x1e198f10, SUC_lexicon_tags_13 },
    { 0xb598f10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe398f1a, SUC_lexicon_tags_37 },
    { 0xcbb98f1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c198f1d, SUC_lexicon_tags_37 },
    { 0xe6d98f1e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b98f23, SUC_lexicon_tags_17 },
    { 0xcf398f24, SUC_lexicon_tags_23 },
    { 0xc1d98f25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b198f2b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x43b98f2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b598f31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8798f34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf98f39, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb998f3c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93f98f3e, SUC_lexicon_tags_66 },
    { 0x7d998f3f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x80798f41, SUC_lexicon_tags_10 },
    { 0x14998f41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23998f45, SUC_lexicon_tags_26 },
    { 0x37198f46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad398f49, SUC_lexicon_tags_47 },
    { 0x16f98f4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f98f4e, SUC_lexicon_tags_11 },
    { 0x78998f4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f98f54, SUC_lexicon_tags_44 },
    { 0x66598f55, SUC_lexicon_tags_11 },
    { 0xae998f54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f98f5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67398f5f, SUC_lexicon_tags_17 },
    { 0x1b598f60, SUC_lexicon_tags_8 },
    { 0xf798f61, SUC_lexicon_tags_17 },
    { 0xd1198f61, SUC_lexicon_tags_79 },
    { 0x52598f63, SUC_lexicon_tags_17 },
    { 0xc2598f64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b98f68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x54998f6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e998f6c, SUC_lexicon_tags_8 },
    { 0x3cb98f6d, SUC_lexicon_tags_14 },
    { 0xa0198f6e, SUC_lexicon_tags_179 },
    { 0xb2398f6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f98f71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56998f73, SUC_lexicon_tags_37 },
    { 0x48798f74, SUC_lexicon_tags_38 },
    { 0x2ad98f74, SUC_lexicon_tags_65 },
    { 0x2d198f75, SUC_lexicon_tags_38 },
    { 0xe0598f76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e598f79, SUC_lexicon_tags_26 },
    { 0xc0d98f79, SUC_lexicon_tags_63 },
    { 0x14798f7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d998f80, SUC_lexicon_tags_2 },
    { 0xa5398f80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8d598f83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85598f87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xec598f8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19798f90, SUC_lexicon_tags_65 },
    { 0x8ed98f91, SUC_lexicon_tags_15 },
    { 0xa9798f92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46798f94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91f98f96, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x73f98f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac398f9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85398f9d, SUC_lexicon_tags_126 },
    { 0x4b598f9e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d98fa2, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f98fa5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a598fad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d98fb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8798fbc, SUC_lexicon_tags_25 },
    { 0xb3198fbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf998fc6, SUC_lexicon_tags_6 },
    { 0xae398fc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x85598fc9, SUC_lexicon_tags_11 },
    { 0x4d98fc9, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71798fce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6df98fd0, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb598fd3, SUC_lexicon_tags_0 },
    { 0x8c798fd4, SUC_lexicon_tags_25 },
    { 0x45598fd5, SUC_lexicon_tags_55 },
    { 0x50b98fd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd798fd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e198fdd, SUC_lexicon_tags_17 },
    { 0x67598fdd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e598fe1, SUC_lexicon_tags_230 },
    { 0xacb98fe2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c798fe5, SUC_lexicon_tags_27 },
    { 0xf2b98fe6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0398fe8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d98feb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dd98fed, SUC_lexicon_tags_25 },
    { 0x23b98fed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x40598ff0, SUC_lexicon_tags_8 },
    { 0xb0598ff0, SUC_lexicon_tags_21 },
    { 0x73598ff2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b598ff4, SUC_lexicon_tags_37 },
    { 0x17398ff5, SUC_lexicon_tags_61 },
    { 0x5bf98ff4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x29798ff8, SUC_lexicon_tags_12 },
    { 0x60f98ff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c598ffc, SUC_lexicon_tags_60 },
    { 0xe4598ffd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfef98fff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa9399001, SUC_lexicon_tags_35 },
    { 0x76999002, SUC_lexicon_tags_8 },
    { 0x3e999002, SUC_lexicon_tags_60 },
    { 0x2a599002, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10f99008, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2999900a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f99015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f99018, SUC_lexicon_tags_1 },
    { 0x10599019, SUC_lexicon_tags_37 },
    { 0x84b99019, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3379901f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xa7b99021, SUC_lexicon_tags_15 },
    { 0x8799022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a399028, SUC_lexicon_tags_29 },
    { 0xf7d99029, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd079902c, SUC_lexicon_tags_6 },
    { 0xe739902d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0b9902f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7399031, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a799034, SUC_lexicon_tags_17 },
    { 0x7db99034, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3799037, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb099903a, SUC_lexicon_tags_84 },
    { 0xc7f9903b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45999042, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead99046, SUC_lexicon_tags_17 },
    { 0xab799047, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x75d99049, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x96d9904b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b39904f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x52b99052, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4199055, SUC_lexicon_tags_39 },
    { 0xc6799055, SUC_lexicon_tags_1 },
    { 0x93f99057, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42799059, SUC_lexicon_tags_103 },
    { 0xed799059, SUC_lexicon_tags_23 },
    { 0xe399905b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f9905e, SUC_lexicon_tags_25 },
    { 0x76b9905f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a399066, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb19906a, SUC_lexicon_tags_66 },
    { 0x239906b, SUC_lexicon_tags_47 },
    { 0x98b9906c, SUC_lexicon_tags_60 },
    { 0x7f19906d, SUC_lexicon_tags_94 },
    { 0x89b9906e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23199074, SUC_lexicon_tags_191 },
    { 0xcc799075, SUC_lexicon_tags_71 },
    { 0xc3b99075, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c39907b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b9907f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xcf799081, SUC_lexicon_tags_2 },
    { 0xf7d99082, SUC_lexicon_tags_0 },
    { 0x9af99083, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9908b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4799091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4399093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9179909b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f990a1, SUC_lexicon_tags_18 },
    { 0x93990a1, SUC_lexicon_tags_2 },
    { 0x73b990a3, SUC_lexicon_tags_17 },
    { 0xccd990a4, SUC_lexicon_tags_17 },
    { 0x621990a5, SUC_lexicon_tags_15 },
    { 0x9cd990a6, SUC_lexicon_tags_151 },
    { 0xdf1990a2, SUC_lexicon_tags_77 },
    { 0xd49990a8, SUC_lexicon_tags_49 },
    { 0xc8b990a9, SUC_lexicon_tags_30 },
    { 0x1cf990a3, SUC_lexicon_tags_5 },
    { 0xaeb990ab, SUC_lexicon_tags_27 },
    { 0x879990ac, SUC_lexicon_tags_15 },
    { 0x745990ad, SUC_lexicon_tags_7 },
    { 0x401990ae, SUC_lexicon_tags_37 },
    { 0x9b7990a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815990b5, SUC_lexicon_tags_12 },
    { 0xf41990b6, SUC_lexicon_tags_30 },
    { 0xc55990b7, SUC_lexicon_tags_23 },
    { 0xcc9990b7, SUC_lexicon_tags_57 },
    { 0x303990b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585990bd, SUC_lexicon_tags_10 },
    { 0xddf990be, SUC_lexicon_tags_79 },
    { 0xe25990be, SUC_lexicon_tags_8 },
    { 0x965990c0, SUC_lexicon_tags_21 },
    { 0xe2f990c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0d990c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd990c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365990cc, SUC_lexicon_tags_7 },
    { 0x68d990cd, SUC_lexicon_tags_7 },
    { 0xc27990ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9990d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f990e9, SUC_lexicon_tags_85 },
    { 0xfe3990e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x569990ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7990ef, SUC_lexicon_tags_45 },
    { 0x6df990ef, SUC_lexicon_tags_15 },
    { 0x27d990ef, SUC_lexicon_tags_43 },
    { 0x19d990f1, SUC_lexicon_tags_12 },
    { 0x1e3990f3, SUC_lexicon_tags_8 },
    { 0x183990f4, SUC_lexicon_tags_43 },
    { 0x157990ef, SUC_lexicon_tags_17 },
    { 0xc2d990f6, SUC_lexicon_tags_28 },
    { 0x95f990f7, SUC_lexicon_tags_2 },
    { 0xb13990f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3990fb, SUC_lexicon_tags_49 },
    { 0x207990fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x52d990fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a599102, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1339910b, SUC_lexicon_tags_5 },
    { 0xb559910c, SUC_lexicon_tags_37 },
    { 0xb699910c, SUC_lexicon_tags_56 },
    { 0xec99910c, SUC_lexicon_tags_17 },
    { 0xcd39910d, SUC_lexicon_tags_23 },
    { 0x39d99110, SUC_lexicon_tags_7 },
    { 0xd1999111, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc999119, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d99911c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92799120, SUC_lexicon_tags_17 },
    { 0x60199121, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c399123, SUC_lexicon_tags_101 },
    { 0xe6f99123, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x31399127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1639912c, SUC_lexicon_tags_14 },
    { 0x5179912d, SUC_lexicon_tags_144 },
    { 0xe699912e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d199133, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b599137, SUC_lexicon_tags_0 },
    { 0x34399138, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9fb9913a, SUC_lexicon_tags_37 },
    { 0xc139913b, SUC_lexicon_tags_31 },
    { 0x5b39913b, SUC_lexicon_tags_38 },
    { 0xe959913d, SUC_lexicon_tags_46 },
    { 0xa4d9913e, SUC_lexicon_tags_0 },
    { 0xa539913f, SUC_lexicon_tags_65 },
    { 0x9d59913a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d99147, SUC_lexicon_tags_8 },
    { 0xe4199148, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159914d, SUC_lexicon_tags_17 },
    { 0x1159914e, SUC_lexicon_tags_12 },
    { 0xb8b9914f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b99152, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf799159, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcf19915b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54199169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4339916f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db99172, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5799175, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f9917c, SUC_lexicon_tags_14 },
    { 0x9ab9917d, SUC_lexicon_tags_191 },
    { 0x71d9917e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7399181, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b399185, SUC_lexicon_tags_116 },
    { 0x93399186, SUC_lexicon_tags_60 },
    { 0xc6799187, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f399189, SUC_lexicon_tags_14 },
    { 0xb2f9918a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeaf9918c, SUC_lexicon_tags_38 },
    { 0x7339918d, SUC_lexicon_tags_32 },
    { 0x52f9918e, SUC_lexicon_tags_33 },
    { 0xe499918f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b99193, SUC_lexicon_tags_12 },
    { 0x46b99194, SUC_lexicon_tags_12 },
    { 0x6f599195, SUC_lexicon_tags_11 },
    { 0x50399195, SUC_lexicon_tags_14 },
    { 0x65599194, SUC_lexicon_tags_7 },
    { 0x5ff99194, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe539919a, SUC_lexicon_tags_25 },
    { 0x4719919b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1991a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80f991a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7991ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x741991ae, SUC_lexicon_tags_35 },
    { 0xb5991af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x189991b1, SUC_lexicon_tags_11 },
    { 0x3ef991b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1991b5, SUC_lexicon_tags_9 },
    { 0xd79991b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x401991b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3991bf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x981991c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8b991c4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f991c7, SUC_lexicon_tags_2 },
    { 0x903991c8, SUC_lexicon_tags_17 },
    { 0xc9f991c9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x89991cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9991ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33991d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf991d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7991db, SUC_lexicon_tags_6 },
    { 0xb0d991db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b991df, SUC_lexicon_tags_37 },
    { 0x625991e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63991e9, SUC_lexicon_tags_29 },
    { 0x12f991ea, SUC_lexicon_tags_10 },
    { 0x953991eb, SUC_lexicon_tags_45 },
    { 0x5bd991eb, SUC_lexicon_tags_40 },
    { 0x39f991ec, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x245991ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b1991f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f991f4, SUC_lexicon_tags_45 },
    { 0x5df991f4, SUC_lexicon_tags_67 },
    { 0xbb7991f4, SUC_lexicon_tags_33 },
    { 0x445991f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4df991f9, SUC_lexicon_tags_29 },
    { 0x49f991fa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c599203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62799205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f99920a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d59920d, SUC_lexicon_tags_17 },
    { 0xfbb9920e, SUC_lexicon_tags_37 },
    { 0x59920f, SUC_lexicon_tags_3 },
    { 0xb3199210, SUC_lexicon_tags_30 },
    { 0x4af9920f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee999216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1959921a, SUC_lexicon_tags_77 },
    { 0x4a19921b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe299921e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11799223, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc599228, SUC_lexicon_tags_82 },
    { 0xff399229, SUC_lexicon_tags_33 },
    { 0xadd99229, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf819922e, SUC_lexicon_tags_37 },
    { 0x1419922f, SUC_lexicon_tags_15 },
    { 0xa439922f, SUC_lexicon_tags_2 },
    { 0x53999231, SUC_lexicon_tags_771 },
    { 0xe6d99232, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91799238, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59923d, SUC_lexicon_tags_28 },
    { 0x4bf9923e, SUC_lexicon_tags_1 },
    { 0x8159923f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db99243, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d99248, SUC_lexicon_tags_1 },
    { 0x27599248, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee799251, SUC_lexicon_tags_37 },
    { 0xf99252, SUC_lexicon_tags_45 },
    { 0x56f99251, SUC_lexicon_tags_79 },
    { 0xf6f99253, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2999258, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f9925c, SUC_lexicon_tags_33 },
    { 0xc519925d, SUC_lexicon_tags_37 },
    { 0x699925d, SUC_lexicon_tags_9 },
    { 0xbeb9925f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x799261, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d99264, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d99268, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9926b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf259926e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6b199270, SUC_lexicon_tags_9 },
    { 0x21d99271, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x72b99274, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58f99278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa819927a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9719927d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe2b9927f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b99283, SUC_lexicon_tags_14 },
    { 0xba799283, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4799287, SUC_lexicon_tags_23 },
    { 0xd599288, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb9928b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1939928d, SUC_lexicon_tags_7 },
    { 0xd079928e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6599291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11399294, SUC_lexicon_tags_7 },
    { 0xb8199295, SUC_lexicon_tags_49 },
    { 0x1fb99295, SUC_lexicon_tags_15 },
    { 0xea999297, SUC_lexicon_tags_1 },
    { 0x54599298, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe599929a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b19929d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1992a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x43f992a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b992a6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd992a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc6f992ab, SUC_lexicon_tags_25 },
    { 0x9e9992ac, SUC_lexicon_tags_37 },
    { 0xa1b992ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85992b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d992b3, SUC_lexicon_tags_25 },
    { 0xc03992b4, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x60d992b6, SUC_lexicon_tags_35 },
    { 0x715992b7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb13992b9, SUC_lexicon_tags_60 },
    { 0xebf992ba, SUC_lexicon_tags_15 },
    { 0xbad992ba, SUC_lexicon_tags_43 },
    { 0xd13992bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50f992be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151992c6, SUC_lexicon_tags_38 },
    { 0x9d7992c6, SUC_lexicon_tags_11 },
    { 0xa69992c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0d992ca, SUC_lexicon_tags_264 },
    { 0xdfb992cb, SUC_lexicon_tags_25 },
    { 0x491992cc, SUC_lexicon_tags_49 },
    { 0x855992cd, SUC_lexicon_tags_47 },
    { 0xf2d992ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd992d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40d992d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1992dd, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x963992df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7992e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdc7992e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7a7992e6, SUC_lexicon_tags_37 },
    { 0x4e9992e7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67992ea, SUC_lexicon_tags_45 },
    { 0xdb3992eb, SUC_lexicon_tags_25 },
    { 0xb63992ec, SUC_lexicon_tags_47 },
    { 0x257992ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947992f1, SUC_lexicon_tags_11 },
    { 0xb37992f2, SUC_lexicon_tags_49 },
    { 0xc5992f2, SUC_lexicon_tags_56 },
    { 0x9d9992f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9992fa, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x167992fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacf99307, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5e799309, SUC_lexicon_tags_15 },
    { 0x1af9930a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9930f, SUC_lexicon_tags_37 },
    { 0x8bd99310, SUC_lexicon_tags_48 },
    { 0x9fb99311, SUC_lexicon_tags_23 },
    { 0x44799310, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f99321, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33599323, SUC_lexicon_tags_2 },
    { 0xd2799324, SUC_lexicon_tags_13 },
    { 0x61599324, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d99328, SUC_lexicon_tags_15 },
    { 0xa2399328, SUC_lexicon_tags_33 },
    { 0x3b59932a, SUC_lexicon_tags_14 },
    { 0xccb9932b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1619932d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9999330, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd99333, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xad99335, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff99338, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1439933b, SUC_lexicon_tags_7 },
    { 0xb339933c, SUC_lexicon_tags_7 },
    { 0x279933d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51599341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2999343, SUC_lexicon_tags_49 },
    { 0xb5f99344, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65999348, SUC_lexicon_tags_77 },
    { 0x2fd99349, SUC_lexicon_tags_28 },
    { 0x81d9934a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c39934d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64999351, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x82199354, SUC_lexicon_tags_47 },
    { 0xae199355, SUC_lexicon_tags_15 },
    { 0x6d799354, SUC_lexicon_tags_65 },
    { 0xdb199357, SUC_lexicon_tags_7 },
    { 0x2bf99358, SUC_lexicon_tags_37 },
    { 0xf9f99355, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b9935e, SUC_lexicon_tags_7 },
    { 0x7f79935f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43199361, SUC_lexicon_tags_29 },
    { 0x5ef99362, SUC_lexicon_tags_120 },
    { 0xf5399363, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd399367, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56399372, SUC_lexicon_tags_38 },
    { 0xf4999373, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf999376, SUC_lexicon_tags_6 },
    { 0xd3999376, SUC_lexicon_tags_85 },
    { 0x2bd99378, SUC_lexicon_tags_34 },
    { 0x4e999378, SUC_lexicon_tags_38 },
    { 0x7f399379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b9937f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad799381, SUC_lexicon_tags_19 },
    { 0xfeb99382, SUC_lexicon_tags_12 },
    { 0x36199381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d799387, SUC_lexicon_tags_45 },
    { 0xfad99388, SUC_lexicon_tags_37 },
    { 0xf8d99388, SUC_lexicon_tags_14 },
    { 0x97b9938a, SUC_lexicon_tags_17 },
    { 0x2f999387, SUC_lexicon_tags_29 },
    { 0xe479938a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36199393, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1619939b, SUC_lexicon_tags_60 },
    { 0xb4f9939c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1039939f, SUC_lexicon_tags_14 },
    { 0x1cf993a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1993a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6b993a8, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef993ad, SUC_lexicon_tags_27 },
    { 0x609993ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6d993b0, SUC_lexicon_tags_17 },
    { 0x153993b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603993b9, SUC_lexicon_tags_14 },
    { 0x80b993ba, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x579993bd, SUC_lexicon_tags_31 },
    { 0x1f1993be, SUC_lexicon_tags_37 },
    { 0x9a7993be, SUC_lexicon_tags_45 },
    { 0x275993c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e9993c2, SUC_lexicon_tags_37 },
    { 0x377993c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf79993c5, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375993ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f993d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487993de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d993e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1993e5, SUC_lexicon_tags_15 },
    { 0x51993e6, SUC_lexicon_tags_43 },
    { 0x77d993e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd51993e9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xadd993ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x179993ef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb993fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f993ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd99403, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f99408, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe119940b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdad9940e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x52999410, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde799412, SUC_lexicon_tags_68 },
    { 0xdaf99413, SUC_lexicon_tags_27 },
    { 0xb0d99413, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7f99416, SUC_lexicon_tags_45 },
    { 0x1c999417, SUC_lexicon_tags_37 },
    { 0x69999418, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc19941d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23999425, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2799428, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f99942a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb259942c, SUC_lexicon_tags_0 },
    { 0x9c39942d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5cd9942f, SUC_lexicon_tags_39 },
    { 0xec59942f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2399434, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xecf99437, SUC_lexicon_tags_26 },
    { 0x6a599438, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3759943c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85b9943e, SUC_lexicon_tags_2 },
    { 0x439943f, SUC_lexicon_tags_22 },
    { 0xab999440, SUC_lexicon_tags_17 },
    { 0x4339943e, SUC_lexicon_tags_7 },
    { 0x44399440, SUC_lexicon_tags_38 },
    { 0x8a399443, SUC_lexicon_tags_39 },
    { 0xa7d9943e, SUC_lexicon_tags_25 },
    { 0x73599445, SUC_lexicon_tags_49 },
    { 0xed99441, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b9944c, SUC_lexicon_tags_47 },
    { 0x2eb9944d, SUC_lexicon_tags_15 },
    { 0x4519944e, SUC_lexicon_tags_12 },
    { 0x6b99944d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7e599451, SUC_lexicon_tags_17 },
    { 0x25399451, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459945a, SUC_lexicon_tags_299 },
    { 0xbd59945b, SUC_lexicon_tags_55 },
    { 0x2119945b, SUC_lexicon_tags_35 },
    { 0xdb99945c, SUC_lexicon_tags_19 },
    { 0x86d9945d, SUC_lexicon_tags_1 },
    { 0xee79945f, SUC_lexicon_tags_16 },
    { 0xa999460, SUC_lexicon_tags_15 },
    { 0xb019945c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6d99468, SUC_lexicon_tags_15 },
    { 0x60799469, SUC_lexicon_tags_31 },
    { 0xff19946a, SUC_lexicon_tags_37 },
    { 0xdbd9946b, SUC_lexicon_tags_9 },
    { 0xee79946a, SUC_lexicon_tags_0 },
    { 0x95d9946d, SUC_lexicon_tags_42 },
    { 0x5a19946e, SUC_lexicon_tags_23 },
    { 0xf999946d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1599473, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd9947a, SUC_lexicon_tags_15 },
    { 0xbff9947b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f99947e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb599484, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc39948d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26799496, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd9949a, SUC_lexicon_tags_7 },
    { 0xd99949b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9199949d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbab9949f, SUC_lexicon_tags_17 },
    { 0xbe9994a0, SUC_lexicon_tags_38 },
    { 0x797994a0, SUC_lexicon_tags_57 },
    { 0x26d994a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f994a5, SUC_lexicon_tags_47 },
    { 0x31b994a6, SUC_lexicon_tags_17 },
    { 0x4f7994a7, SUC_lexicon_tags_63 },
    { 0x9ab994a8, SUC_lexicon_tags_23 },
    { 0xbcf994a9, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x113994ac, SUC_lexicon_tags_30 },
    { 0xbef994ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e1994af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5eb994b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f994b6, SUC_lexicon_tags_37 },
    { 0x8a3994b7, SUC_lexicon_tags_37 },
    { 0xec9994b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7f994ba, SUC_lexicon_tags_17 },
    { 0x2b994bb, SUC_lexicon_tags_27 },
    { 0xd75994bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b994c1, SUC_lexicon_tags_38 },
    { 0xa81994c2, SUC_lexicon_tags_37 },
    { 0x855994c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f994c7, SUC_lexicon_tags_17 },
    { 0x241994c8, SUC_lexicon_tags_66 },
    { 0xc8b994c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xced994cb, SUC_lexicon_tags_29 },
    { 0x889994cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7994d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcaf994d9, SUC_lexicon_tags_6 },
    { 0xdd994da, SUC_lexicon_tags_5 },
    { 0x6d7994db, SUC_lexicon_tags_47 },
    { 0x1ff994dc, SUC_lexicon_tags_25 },
    { 0xc83994dd, SUC_lexicon_tags_0 },
    { 0x875994dc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x15d994e0, SUC_lexicon_tags_1 },
    { 0x4ab994e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf994e4, SUC_lexicon_tags_33 },
    { 0xa99994e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89d994e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fd994e9, SUC_lexicon_tags_1 },
    { 0x22f994ea, SUC_lexicon_tags_1 },
    { 0x267994e9, SUC_lexicon_tags_27 },
    { 0x349994eb, SUC_lexicon_tags_19 },
    { 0x127994ed, SUC_lexicon_tags_62 },
    { 0x1b7994ea, SUC_lexicon_tags_33 },
    { 0x217994ef, SUC_lexicon_tags_6 },
    { 0x3b994f0, SUC_lexicon_tags_5 },
    { 0x7a7994f1, SUC_lexicon_tags_17 },
    { 0xa7b994f2, SUC_lexicon_tags_27 },
    { 0x673994ec, SUC_lexicon_tags_25 },
    { 0x96f994ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x627994f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x713994f9, SUC_lexicon_tags_0 },
    { 0x937994fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1994fd, SUC_lexicon_tags_12 },
    { 0x819994fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb399506, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11999508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7679950d, SUC_lexicon_tags_27 },
    { 0xe9f9950d, SUC_lexicon_tags_19 },
    { 0x74b9950f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd199517, SUC_lexicon_tags_2 },
    { 0x14999518, SUC_lexicon_tags_23 },
    { 0xd0f99519, SUC_lexicon_tags_14 },
    { 0xab19951a, SUC_lexicon_tags_17 },
    { 0x5e599518, SUC_lexicon_tags_12 },
    { 0xaf9951a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8f9951e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1dd99520, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f99524, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xae599527, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x19952a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f19952e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d199532, SUC_lexicon_tags_17 },
    { 0x64599533, SUC_lexicon_tags_17 },
    { 0x3e799534, SUC_lexicon_tags_29 },
    { 0x23f99535, SUC_lexicon_tags_14 },
    { 0xcbb99536, SUC_lexicon_tags_2 },
    { 0xd999537, SUC_lexicon_tags_2 },
    { 0xd8799535, SUC_lexicon_tags_35 },
    { 0x30799539, SUC_lexicon_tags_15 },
    { 0x42b9953a, SUC_lexicon_tags_2 },
    { 0xb399536, SUC_lexicon_tags_23 },
    { 0x9e19953c, SUC_lexicon_tags_31 },
    { 0xeab99539, SUC_lexicon_tags_9 },
    { 0x65f9953e, SUC_lexicon_tags_29 },
    { 0xddd99539, SUC_lexicon_tags_1 },
    { 0xec799540, SUC_lexicon_tags_45 },
    { 0xc4199534, SUC_lexicon_tags_6 },
    { 0x13f99532, SUC_lexicon_tags_7 },
    { 0xefd99543, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc1399545, SUC_lexicon_tags_15 },
    { 0xd6599545, SUC_lexicon_tags_30 },
    { 0x38599545, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9d99549, SUC_lexicon_tags_37 },
    { 0x1c99954a, SUC_lexicon_tags_1 },
    { 0xe399954b, SUC_lexicon_tags_65 },
    { 0x9419954b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf99551, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x23599554, SUC_lexicon_tags_25 },
    { 0x81399555, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba79955b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fd9955d, SUC_lexicon_tags_14 },
    { 0xb959955e, SUC_lexicon_tags_179 },
    { 0x3459955f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5199562, SUC_lexicon_tags_20 },
    { 0x44b99563, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x65399565, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f9956b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x45d9956e, SUC_lexicon_tags_11 },
    { 0xca79956f, SUC_lexicon_tags_23 },
    { 0xc2f9956f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x99399572, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52d99574, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfa399576, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1779957c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b399580, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2799583, SUC_lexicon_tags_19 },
    { 0x96799584, SUC_lexicon_tags_26 },
    { 0x3b199584, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9e399587, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b99590, SUC_lexicon_tags_7 },
    { 0xa2399591, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b599593, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1399596, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4b199598, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb599959b, SUC_lexicon_tags_14 },
    { 0xd8d9959c, SUC_lexicon_tags_28 },
    { 0x31b9959c, SUC_lexicon_tags_26 },
    { 0xd519959e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1f995a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd3995a2, SUC_lexicon_tags_60 },
    { 0xfeb995a2, SUC_lexicon_tags_23 },
    { 0xfdf995a4, SUC_lexicon_tags_14 },
    { 0x727995a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df995ac, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1ad995ae, SUC_lexicon_tags_1 },
    { 0x9a1995ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81b995b1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x93d995b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x561995b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9995bf, SUC_lexicon_tags_25 },
    { 0x57f995bf, SUC_lexicon_tags_37 },
    { 0xce9995c1, SUC_lexicon_tags_46 },
    { 0xd87995c0, SUC_lexicon_tags_7 },
    { 0xefb995bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd25995c5, SUC_lexicon_tags_6 },
    { 0x125995c6, SUC_lexicon_tags_14 },
    { 0xd31995c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7995ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b995d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x691995d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x18b995d7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb7f995d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5995dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa45995df, SUC_lexicon_tags_17 },
    { 0x2f3995e0, SUC_lexicon_tags_17 },
    { 0x619995e0, SUC_lexicon_tags_7 },
    { 0x15d995e0, SUC_lexicon_tags_7 },
    { 0xac5995e3, SUC_lexicon_tags_15 },
    { 0xbd7995e4, SUC_lexicon_tags_26 },
    { 0x6d995e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5995ed, SUC_lexicon_tags_60 },
    { 0xfd995ee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d995f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41995f7, SUC_lexicon_tags_12 },
    { 0x173995f8, SUC_lexicon_tags_56 },
    { 0x755995f8, SUC_lexicon_tags_19 },
    { 0xd15995f7, SUC_lexicon_tags_15 },
    { 0x8ad995fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb995fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb99603, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65599606, SUC_lexicon_tags_35 },
    { 0xff99607, SUC_lexicon_tags_60 },
    { 0x6a799608, SUC_lexicon_tags_21 },
    { 0x23999609, SUC_lexicon_tags_21 },
    { 0xb2d9960a, SUC_lexicon_tags_4 },
    { 0xdc39960b, SUC_lexicon_tags_26 },
    { 0x54d9960b, SUC_lexicon_tags_80 },
    { 0x41b9960d, SUC_lexicon_tags_17 },
    { 0x63b99609, SUC_lexicon_tags_28 },
    { 0x36f9960a, SUC_lexicon_tags_37 },
    { 0x239960d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92d99612, SUC_lexicon_tags_6 },
    { 0xf2f99613, SUC_lexicon_tags_2 },
    { 0x42199614, SUC_lexicon_tags_68 },
    { 0xc7599615, SUC_lexicon_tags_12 },
    { 0xd399615, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b99619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb9961c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d79961e, SUC_lexicon_tags_37 },
    { 0x3c39961f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb99623, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a999626, SUC_lexicon_tags_33 },
    { 0xa1399627, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d9962a, SUC_lexicon_tags_84 },
    { 0x5699962a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f9962f, SUC_lexicon_tags_47 },
    { 0x45799630, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70599635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd99638, SUC_lexicon_tags_7 },
    { 0x3d399639, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3479963e, SUC_lexicon_tags_6 },
    { 0xd439963f, SUC_lexicon_tags_61 },
    { 0xd5b99640, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc799642, SUC_lexicon_tags_2 },
    { 0x7c199643, SUC_lexicon_tags_39 },
    { 0xeab99644, SUC_lexicon_tags_1 },
    { 0xe799645, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff9964a, SUC_lexicon_tags_10 },
    { 0x4719964b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b99651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7399657, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45399659, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24b99660, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f399665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c59966b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf119966f, SUC_lexicon_tags_87 },
    { 0x72b99670, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x1b99672, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf99675, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb799677, SUC_lexicon_tags_39 },
    { 0xa6399678, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a59967e, SUC_lexicon_tags_38 },
    { 0xc199967f, SUC_lexicon_tags_35 },
    { 0xdfd9967e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11199684, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xaef99686, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b9968d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf99690, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2599694, SUC_lexicon_tags_63 },
    { 0x7b599695, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83996a0, SUC_lexicon_tags_17 },
    { 0xaa1996a1, SUC_lexicon_tags_11 },
    { 0xe6b996a0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9996a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9996b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x815996b6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5996bb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67996be, SUC_lexicon_tags_12 },
    { 0x127996be, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x357996c1, SUC_lexicon_tags_8 },
    { 0x4f9996c2, SUC_lexicon_tags_11 },
    { 0xf3b996c3, SUC_lexicon_tags_7 },
    { 0x461996c2, SUC_lexicon_tags_7 },
    { 0x3c5996c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22b996c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef996d1, SUC_lexicon_tags_17 },
    { 0x5dd996d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d996d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90d996de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135996e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87f996e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94d996e7, SUC_lexicon_tags_0 },
    { 0xb1f996e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1996eb, SUC_lexicon_tags_2 },
    { 0x709996eb, SUC_lexicon_tags_15 },
    { 0xc53996ed, SUC_lexicon_tags_0 },
    { 0x33b996ee, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6c3996f0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xee7996f2, SUC_lexicon_tags_28 },
    { 0x127996f2, SUC_lexicon_tags_7 },
    { 0x36b996f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15996f9, SUC_lexicon_tags_68 },
    { 0x133996f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71b996fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40599703, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe799707, SUC_lexicon_tags_11 },
    { 0x8fd99708, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f99970c, SUC_lexicon_tags_1 },
    { 0x13f9970c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2839970f, SUC_lexicon_tags_11 },
    { 0x4959970f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x68599712, SUC_lexicon_tags_49 },
    { 0xfed99713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81199716, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0x37799718, SUC_lexicon_tags_12 },
    { 0xc2799719, SUC_lexicon_tags_30 },
    { 0x8c39971a, SUC_lexicon_tags_11 },
    { 0xdef9971b, SUC_lexicon_tags_35 },
    { 0xe739971a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f199720, SUC_lexicon_tags_198 },
    { 0xdab99720, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69999726, SUC_lexicon_tags_14 },
    { 0xb9b99727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d79972f, SUC_lexicon_tags_15 },
    { 0x5799730, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x36b99733, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e599738, SUC_lexicon_tags_7 },
    { 0x1eb99739, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xadf9973c, SUC_lexicon_tags_15 },
    { 0x8759973c, SUC_lexicon_tags_17 },
    { 0x8af9973d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab99745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4f99747, SUC_lexicon_tags_15 },
    { 0x8c799748, SUC_lexicon_tags_11 },
    { 0x55799749, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21599751, SUC_lexicon_tags_39 },
    { 0x98399752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32799754, SUC_lexicon_tags_2 },
    { 0xc3199754, SUC_lexicon_tags_14 },
    { 0x1b599756, SUC_lexicon_tags_68 },
    { 0x70399755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc099975b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa59975d, SUC_lexicon_tags_7 },
    { 0xc739975d, SUC_lexicon_tags_63 },
    { 0x1a99975f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16599763, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b99768, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4379976c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b79976e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf9977c, SUC_lexicon_tags_25 },
    { 0xb1f9977d, SUC_lexicon_tags_27 },
    { 0xd779977d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x72599780, SUC_lexicon_tags_15 },
    { 0x3a599780, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df99788, SUC_lexicon_tags_0 },
    { 0xde199789, SUC_lexicon_tags_45 },
    { 0xa6199789, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f9978e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x34f99791, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e599794, SUC_lexicon_tags_37 },
    { 0x33199795, SUC_lexicon_tags_37 },
    { 0xbb399796, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98999799, SUC_lexicon_tags_6 },
    { 0xca99979a, SUC_lexicon_tags_17 },
    { 0xb499979b, SUC_lexicon_tags_37 },
    { 0xc779979c, SUC_lexicon_tags_2 },
    { 0x5f79979d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b997a2, SUC_lexicon_tags_47 },
    { 0x557997a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb63997a5, SUC_lexicon_tags_28 },
    { 0x247997a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x20b997a9, SUC_lexicon_tags_14 },
    { 0x79997a9, SUC_lexicon_tags_63 },
    { 0x187997ab, SUC_lexicon_tags_212 },
    { 0x89b997ab, SUC_lexicon_tags_7 },
    { 0xc7b997ac, SUC_lexicon_tags_47 },
    { 0x6fd997aa, SUC_lexicon_tags_0 },
    { 0x6c9997af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d997b4, SUC_lexicon_tags_25 },
    { 0xb5997b5, SUC_lexicon_tags_42 },
    { 0x2f997b4, SUC_lexicon_tags_37 },
    { 0x877997b7, SUC_lexicon_tags_37 },
    { 0x80f997b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x987997ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57997be, SUC_lexicon_tags_49 },
    { 0x127997bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34b997c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabf997c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7997cc, SUC_lexicon_tags_25 },
    { 0x715997cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d997d1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x30f997d4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5997d9, SUC_lexicon_tags_47 },
    { 0xdcb997d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91997e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d997e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbbf997e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x48f997e8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5997ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x681997f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b997f3, SUC_lexicon_tags_37 },
    { 0xfb1997f4, SUC_lexicon_tags_27 },
    { 0x24d997f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84b997f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9997fd, SUC_lexicon_tags_7 },
    { 0xc31997fe, SUC_lexicon_tags_43 },
    { 0x157997fe, SUC_lexicon_tags_28 },
    { 0xc6199800, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d199808, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d9980f, SUC_lexicon_tags_63 },
    { 0x24f9980f, SUC_lexicon_tags_11 },
    { 0x9c99980f, SUC_lexicon_tags_63 },
    { 0x33199810, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28b99816, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7599819, SUC_lexicon_tags_37 },
    { 0x9f19981a, SUC_lexicon_tags_37 },
    { 0xbf9981b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d99823, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c599826, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd9982d, SUC_lexicon_tags_12 },
    { 0xbf79982e, SUC_lexicon_tags_179 },
    { 0xcd19982d, SUC_lexicon_tags_49 },
    { 0xf5f99830, SUC_lexicon_tags_37 },
    { 0x75799830, SUC_lexicon_tags_12 },
    { 0x9b199832, SUC_lexicon_tags_28 },
    { 0x97f99830, SUC_lexicon_tags_37 },
    { 0xcfd99834, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2619983d, SUC_lexicon_tags_23 },
    { 0x3ad9983d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c799841, SUC_lexicon_tags_5 },
    { 0xf7799842, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69599845, SUC_lexicon_tags_1 },
    { 0xe8f99846, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf459984b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b99984d, SUC_lexicon_tags_11 },
    { 0x4839984e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97399855, SUC_lexicon_tags_26 },
    { 0xab399856, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b399858, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe739985c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80199861, SUC_lexicon_tags_17 },
    { 0x62d99861, SUC_lexicon_tags_27 },
    { 0x5a199863, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d999869, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e19986d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3799873, SUC_lexicon_tags_6 },
    { 0x8b399874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67799878, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff9987b, SUC_lexicon_tags_25 },
    { 0x1c59987b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa599883, SUC_lexicon_tags_85 },
    { 0xaf199884, SUC_lexicon_tags_25 },
    { 0x4bf99883, SUC_lexicon_tags_46 },
    { 0x53799884, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf479988a, SUC_lexicon_tags_9 },
    { 0x8859988b, SUC_lexicon_tags_0 },
    { 0x6cf9988c, SUC_lexicon_tags_11 },
    { 0xfbd9988c, SUC_lexicon_tags_11 },
    { 0x89d9988d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc999894, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f99898, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e99989c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f998a0, SUC_lexicon_tags_45 },
    { 0x4bf998a0, SUC_lexicon_tags_14 },
    { 0xbc1998a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901998a7, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91998aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ab998b1, SUC_lexicon_tags_25 },
    { 0xd07998b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d5998b4, SUC_lexicon_tags_45 },
    { 0x111998b4, SUC_lexicon_tags_4 },
    { 0xf8f998b6, SUC_lexicon_tags_18 },
    { 0x90f998b4, SUC_lexicon_tags_25 },
    { 0x3b7998b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef5998ba, SUC_lexicon_tags_15 },
    { 0x44d998bb, SUC_lexicon_tags_7 },
    { 0x58d998bb, SUC_lexicon_tags_9 },
    { 0x9b998bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5998c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc17998c2, SUC_lexicon_tags_12 },
    { 0xf39998c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcb1998c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7998cd, SUC_lexicon_tags_60 },
    { 0x8f7998ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b998d1, SUC_lexicon_tags_2 },
    { 0xd09998d1, SUC_lexicon_tags_27 },
    { 0xd33998d2, SUC_lexicon_tags_7 },
    { 0x6af998d2, SUC_lexicon_tags_68 },
    { 0xcd998d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x355998d7, SUC_lexicon_tags_29 },
    { 0x36f998d8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x449998db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829998df, SUC_lexicon_tags_28 },
    { 0x489998e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x55d998e2, SUC_lexicon_tags_11 },
    { 0x9fd998e3, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381998e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f998ef, SUC_lexicon_tags_11 },
    { 0xcef998ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25998f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x575998f8, SUC_lexicon_tags_13 },
    { 0x9d3998f9, SUC_lexicon_tags_101 },
    { 0xb55998fa, SUC_lexicon_tags_11 },
    { 0xba5998fa, SUC_lexicon_tags_37 },
    { 0x843998fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x61998ff, SUC_lexicon_tags_2 },
    { 0x45998ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b99904, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e999906, SUC_lexicon_tags_9 },
    { 0x75399907, SUC_lexicon_tags_8 },
    { 0x70d99906, SUC_lexicon_tags_87 },
    { 0xe1999906, SUC_lexicon_tags_26 },
    { 0xb419990a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e79990d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb99910, SUC_lexicon_tags_37 },
    { 0xa7399911, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x7fb99913, SUC_lexicon_tags_2 },
    { 0xd9d99914, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5df99917, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc279991b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f99921, SUC_lexicon_tags_12 },
    { 0x1eb99922, SUC_lexicon_tags_47 },
    { 0x9d399922, SUC_lexicon_tags_37 },
    { 0xc1599924, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86799929, SUC_lexicon_tags_11 },
    { 0x95399929, SUC_lexicon_tags_47 },
    { 0x399992b, SUC_lexicon_tags_27 },
    { 0xb299992c, SUC_lexicon_tags_2 },
    { 0xb2b9992b, SUC_lexicon_tags_85 },
    { 0xc3f9992e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddf99931, SUC_lexicon_tags_2 },
    { 0xb5f99931, SUC_lexicon_tags_5 },
    { 0x31b99933, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf999936, SUC_lexicon_tags_565 },
    { 0, NULL },
    { 0x27b99938, SUC_lexicon_tags_2 },
    { 0xcf399938, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf659993b, SUC_lexicon_tags_7 },
    { 0xe939993c, SUC_lexicon_tags_19 },
    { 0xd379993d, SUC_lexicon_tags_2 },
    { 0x7cb9993e, SUC_lexicon_tags_38 },
    { 0xda79993f, SUC_lexicon_tags_11 },
    { 0x75f99940, SUC_lexicon_tags_30 },
    { 0xc9199941, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3d99943, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4199949, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x679994c, SUC_lexicon_tags_54 },
    { 0x59b9994d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8759994f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8199951, SUC_lexicon_tags_37 },
    { 0x26b99952, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3599957, SUC_lexicon_tags_11 },
    { 0x1ed99958, SUC_lexicon_tags_33 },
    { 0xf0999958, SUC_lexicon_tags_25 },
    { 0x4b599958, SUC_lexicon_tags_8 },
    { 0xf5b9995b, SUC_lexicon_tags_25 },
    { 0x8df9995b, SUC_lexicon_tags_6 },
    { 0x5479995d, SUC_lexicon_tags_7 },
    { 0x2359995e, SUC_lexicon_tags_49 },
    { 0xa59995f, SUC_lexicon_tags_26 },
    { 0x5c99995e, SUC_lexicon_tags_1 },
    { 0xccb99961, SUC_lexicon_tags_11 },
    { 0x41999957, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c599966, SUC_lexicon_tags_49 },
    { 0xe7199967, SUC_lexicon_tags_65 },
    { 0x9f799967, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x1139996a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x26f9996c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd9996f, SUC_lexicon_tags_53 },
    { 0x87b99970, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd999975, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x39799977, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64f9997b, SUC_lexicon_tags_207 },
    { 0x5799997c, SUC_lexicon_tags_17 },
    { 0x5699997d, SUC_lexicon_tags_37 },
    { 0xa879997c, SUC_lexicon_tags_25 },
    { 0x20f9997e, SUC_lexicon_tags_25 },
    { 0x4879997b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9999982, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x15599985, SUC_lexicon_tags_3 },
    { 0xfad99986, SUC_lexicon_tags_14 },
    { 0xf7d99987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe19998e, SUC_lexicon_tags_17 },
    { 0xf99998f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34399997, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc59999f, SUC_lexicon_tags_19 },
    { 0xf039999f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cb999a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xab5999a5, SUC_lexicon_tags_104 },
    { 0x2b1999a5, SUC_lexicon_tags_26 },
    { 0x5b9999a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603999b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d3999b2, SUC_lexicon_tags_23 },
    { 0x5bd999b3, SUC_lexicon_tags_1 },
    { 0xd6f999b4, SUC_lexicon_tags_25 },
    { 0xfc5999b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f999b9, SUC_lexicon_tags_15 },
    { 0x585999ba, SUC_lexicon_tags_15 },
    { 0x9ff999bb, SUC_lexicon_tags_30 },
    { 0x9d999bc, SUC_lexicon_tags_7 },
    { 0xfe7999bd, SUC_lexicon_tags_30 },
    { 0xea7999be, SUC_lexicon_tags_63 },
    { 0x439999be, SUC_lexicon_tags_43 },
    { 0xda5999bd, SUC_lexicon_tags_49 },
    { 0xdcb999bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf93999c3, SUC_lexicon_tags_27 },
    { 0xbef999c3, SUC_lexicon_tags_47 },
    { 0x309999c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x773999c7, SUC_lexicon_tags_27 },
    { 0xc49999c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f999ce, SUC_lexicon_tags_10 },
    { 0x641999cf, SUC_lexicon_tags_38 },
    { 0xe31999cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6a9999d2, SUC_lexicon_tags_57 },
    { 0x6d999d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b7999d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3999d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05999dd, SUC_lexicon_tags_12 },
    { 0x5b999dd, SUC_lexicon_tags_11 },
    { 0xcf1999dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75999e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48f999e6, SUC_lexicon_tags_11 },
    { 0x2e1999e7, SUC_lexicon_tags_27 },
    { 0xbb9999e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339999ed, SUC_lexicon_tags_27 },
    { 0x8b999ee, SUC_lexicon_tags_27 },
    { 0x5af999ef, SUC_lexicon_tags_47 },
    { 0xf0b999f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a7999f2, SUC_lexicon_tags_33 },
    { 0xcd999f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x91b999f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9999f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b999fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c399a06, SUC_lexicon_tags_28 },
    { 0x31599a06, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x90399a09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb199a0d, SUC_lexicon_tags_15 },
    { 0x8799a0d, SUC_lexicon_tags_1 },
    { 0x1b599a0d, SUC_lexicon_tags_11 },
    { 0xaf999a10, SUC_lexicon_tags_7 },
    { 0xabf99a11, SUC_lexicon_tags_12 },
    { 0xa9b99a11, SUC_lexicon_tags_37 },
    { 0x3b199a0f, SUC_lexicon_tags_158 },
    { 0x9df99a14, SUC_lexicon_tags_15 },
    { 0x5e399a14, SUC_lexicon_tags_11 },
    { 0xe4199a15, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0xab99a19, SUC_lexicon_tags_74 },
    { 0x17999a1a, SUC_lexicon_tags_37 },
    { 0xe2399a1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94799a1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7599a21, SUC_lexicon_tags_25 },
    { 0x42d99a22, SUC_lexicon_tags_39 },
    { 0x3b999a23, SUC_lexicon_tags_11 },
    { 0x11999a24, SUC_lexicon_tags_12 },
    { 0xa0199a23, SUC_lexicon_tags_48 },
    { 0x17999a25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92d99a28, SUC_lexicon_tags_7 },
    { 0x9f99a28, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f99a2d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x74f99a2f, SUC_lexicon_tags_23 },
    { 0x13799a30, SUC_lexicon_tags_37 },
    { 0xde799a30, SUC_lexicon_tags_11 },
    { 0x11f99a31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c199a34, SUC_lexicon_tags_7 },
    { 0xaaf99a34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8599a39, SUC_lexicon_tags_15 },
    { 0xbe799a3a, SUC_lexicon_tags_37 },
    { 0xb8199a3a, SUC_lexicon_tags_11 },
    { 0x59199a3c, SUC_lexicon_tags_17 },
    { 0xdc599a3d, SUC_lexicon_tags_15 },
    { 0x62999a3e, SUC_lexicon_tags_11 },
    { 0xc9d99a3f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f599a42, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b199a45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4199a49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2399a4c, SUC_lexicon_tags_25 },
    { 0x1999a4c, SUC_lexicon_tags_48 },
    { 0x7bd99a4e, SUC_lexicon_tags_66 },
    { 0x63999a4c, SUC_lexicon_tags_11 },
    { 0x80b99a50, SUC_lexicon_tags_39 },
    { 0x3a999a51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b99a57, SUC_lexicon_tags_25 },
    { 0x91799a58, SUC_lexicon_tags_11 },
    { 0x28599a58, SUC_lexicon_tags_17 },
    { 0x4d599a58, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e999a5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4b99a61, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x15199a63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e399a66, SUC_lexicon_tags_45 },
    { 0x36f99a67, SUC_lexicon_tags_56 },
    { 0x9b99a68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64599a6a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xd0f99a6c, SUC_lexicon_tags_11 },
    { 0x49b99a6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb99a74, SUC_lexicon_tags_0 },
    { 0x7d599a75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75599a77, SUC_lexicon_tags_45 },
    { 0xed199a77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10d99a7f, SUC_lexicon_tags_12 },
    { 0x7599a80, SUC_lexicon_tags_11 },
    { 0x37999a81, SUC_lexicon_tags_30 },
    { 0x2bb99a7f, SUC_lexicon_tags_33 },
    { 0xf5199a83, SUC_lexicon_tags_12 },
    { 0x51999a84, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8f999a86, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3399a8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71d99a8c, SUC_lexicon_tags_17 },
    { 0x81799a8c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b99a93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3d99a95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27d99a97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab99a9e, SUC_lexicon_tags_66 },
    { 0x35799a9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f99aa6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16999ab0, SUC_lexicon_tags_43 },
    { 0x68399ab1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b199ab4, SUC_lexicon_tags_1274 },
    { 0x3e399ab5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92599abf, SUC_lexicon_tags_6 },
    { 0x39999abf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x80799ac3, SUC_lexicon_tags_2 },
    { 0x2db99ac4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b999ac6, SUC_lexicon_tags_29 },
    { 0xcdb99ac7, SUC_lexicon_tags_12 },
    { 0x39599ac8, SUC_lexicon_tags_17 },
    { 0x69999ac9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ad99acb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd199acd, SUC_lexicon_tags_7 },
    { 0xba799ace, SUC_lexicon_tags_19 },
    { 0xca199ace, SUC_lexicon_tags_55 },
    { 0xe0b99ad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e399ad4, SUC_lexicon_tags_44 },
    { 0x2f199ad5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a799ad7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d99adb, SUC_lexicon_tags_11 },
    { 0x50b99adc, SUC_lexicon_tags_23 },
    { 0xb9399add, SUC_lexicon_tags_10 },
    { 0x39799add, SUC_lexicon_tags_7 },
    { 0x32799adc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb4399ae1, SUC_lexicon_tags_7 },
    { 0x67999ae2, SUC_lexicon_tags_26 },
    { 0x5fd99ae3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2399aeb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24f99af0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af99afd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ef99aff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb99b04, SUC_lexicon_tags_11 },
    { 0x51599b05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4199b0c, SUC_lexicon_tags_2 },
    { 0x37f99b0d, SUC_lexicon_tags_15 },
    { 0x91599b0e, SUC_lexicon_tags_7 },
    { 0x2d399b0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf199b13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x69599b15, SUC_lexicon_tags_17 },
    { 0x3399b16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9599b20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a599b22, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd6599b24, SUC_lexicon_tags_26 },
    { 0xe4199b24, SUC_lexicon_tags_45 },
    { 0x7e999b26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef99b2a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df99b30, SUC_lexicon_tags_17 },
    { 0xa4f99b31, SUC_lexicon_tags_11 },
    { 0x1fd99b31, SUC_lexicon_tags_0 },
    { 0xf6f99b30, SUC_lexicon_tags_47 },
    { 0xff999b33, SUC_lexicon_tags_63 },
    { 0xc3999b30, SUC_lexicon_tags_17 },
    { 0xfdf99b35, SUC_lexicon_tags_37 },
    { 0x10b99b37, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd5999b39, SUC_lexicon_tags_37 },
    { 0x7dd99b3a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb2599b3c, SUC_lexicon_tags_7 },
    { 0x6cf99b3d, SUC_lexicon_tags_25 },
    { 0xad99b3e, SUC_lexicon_tags_28 },
    { 0x89999b3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd99b44, SUC_lexicon_tags_10 },
    { 0x84f99b44, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x78599b47, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d99b4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cd99b4c, SUC_lexicon_tags_25 },
    { 0x22999b4d, SUC_lexicon_tags_37 },
    { 0x6c799b4d, SUC_lexicon_tags_23 },
    { 0xcab99b4d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6799b57, SUC_lexicon_tags_4 },
    { 0x8b799b57, SUC_lexicon_tags_49 },
    { 0x97f99b59, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc599b5e, SUC_lexicon_tags_49 },
    { 0x9c799b5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d99b63, SUC_lexicon_tags_17 },
    { 0x6f99b64, SUC_lexicon_tags_23 },
    { 0xa4399b64, SUC_lexicon_tags_11 },
    { 0xa6199b65, SUC_lexicon_tags_82 },
    { 0x87799b67, SUC_lexicon_tags_40 },
    { 0x36799b68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x28999b6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c599b70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc399b73, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7199b78, SUC_lexicon_tags_3 },
    { 0xb2599b79, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f99b7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84f99b7e, SUC_lexicon_tags_43 },
    { 0x86f99b7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0999b86, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a799b89, SUC_lexicon_tags_7 },
    { 0x56d99b8a, SUC_lexicon_tags_37 },
    { 0x13d99b8a, SUC_lexicon_tags_2 },
    { 0xfd599b8c, SUC_lexicon_tags_2 },
    { 0x39799b8b, SUC_lexicon_tags_10 },
    { 0xb1f99b8d, SUC_lexicon_tags_25 },
    { 0xa599b8f, SUC_lexicon_tags_56 },
    { 0xab999b90, SUC_lexicon_tags_34 },
    { 0x49f99b8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x32199b94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfaf99b96, SUC_lexicon_tags_2 },
    { 0x2599b97, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b99b9a, SUC_lexicon_tags_1 },
    { 0xe1d99b9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f199b9e, SUC_lexicon_tags_31 },
    { 0xbd799b9f, SUC_lexicon_tags_15 },
    { 0x56999ba0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3d99ba2, SUC_lexicon_tags_33 },
    { 0xb2b99ba2, SUC_lexicon_tags_28 },
    { 0xf599ba4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xee999ba7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef99bad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32199baf, SUC_lexicon_tags_11 },
    { 0x39199bb0, SUC_lexicon_tags_33 },
    { 0x99199bb0, SUC_lexicon_tags_49 },
    { 0x23799bb2, SUC_lexicon_tags_37 },
    { 0xfeb99baf, SUC_lexicon_tags_7 },
    { 0x18999baf, SUC_lexicon_tags_17 },
    { 0xc4599bb5, SUC_lexicon_tags_5 },
    { 0xd3799bb6, SUC_lexicon_tags_17 },
    { 0x91599bb2, SUC_lexicon_tags_23 },
    { 0xf0f99bb4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d99bbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a599bbd, SUC_lexicon_tags_23 },
    { 0x44b99bbe, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0xac599bc1, SUC_lexicon_tags_15 },
    { 0xaed99bc2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d99bc7, SUC_lexicon_tags_16 },
    { 0x8c999bc8, SUC_lexicon_tags_7 },
    { 0xad799bc8, SUC_lexicon_tags_14 },
    { 0x9f999bc9, SUC_lexicon_tags_47 },
    { 0x83f99bcb, SUC_lexicon_tags_14 },
    { 0x85399bc7, SUC_lexicon_tags_29 },
    { 0x7e199bcd, SUC_lexicon_tags_7 },
    { 0x21399bc7, SUC_lexicon_tags_82 },
    { 0x1a199bce, SUC_lexicon_tags_15 },
    { 0x60999bc7, SUC_lexicon_tags_31 },
    { 0x2e399bcb, SUC_lexicon_tags_11 },
    { 0x67399bd2, SUC_lexicon_tags_19 },
    { 0x73799bd3, SUC_lexicon_tags_29 },
    { 0x43f99bd3, SUC_lexicon_tags_23 },
    { 0x85b99bcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92799bd9, SUC_lexicon_tags_112 },
    { 0xd3799bda, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcb99bdd, SUC_lexicon_tags_11 },
    { 0x7af99bde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f99be3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x46399be5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf399be7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf99bed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1199bf2, SUC_lexicon_tags_7 },
    { 0x1cd99bf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72d99bf5, SUC_lexicon_tags_77 },
    { 0x5fd99bf6, SUC_lexicon_tags_17 },
    { 0xc9599bf7, SUC_lexicon_tags_2 },
    { 0xdb99bf7, SUC_lexicon_tags_43 },
    { 0x57199bf8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9f199bfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7199bfd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa399c00, SUC_lexicon_tags_10 },
    { 0xd1399c01, SUC_lexicon_tags_63 },
    { 0xb7999c01, SUC_lexicon_tags_45 },
    { 0x81b99c01, SUC_lexicon_tags_37 },
    { 0x6c799c00, SUC_lexicon_tags_19 },
    { 0xde999c05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef99c08, SUC_lexicon_tags_11 },
    { 0xc8f99c09, SUC_lexicon_tags_1 },
    { 0x35599c09, SUC_lexicon_tags_17 },
    { 0xe6999c0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e399c0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b99c12, SUC_lexicon_tags_11 },
    { 0x6dd99c13, SUC_lexicon_tags_1 },
    { 0xc799c14, SUC_lexicon_tags_30 },
    { 0x83799c15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5ad99c17, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f199c1a, SUC_lexicon_tags_23 },
    { 0x54599c1b, SUC_lexicon_tags_11 },
    { 0xcab99c1b, SUC_lexicon_tags_47 },
    { 0xa599c1d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb199c20, SUC_lexicon_tags_17 },
    { 0xa2799c21, SUC_lexicon_tags_25 },
    { 0xf6599c22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b99c26, SUC_lexicon_tags_23 },
    { 0xaa399c26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79999c2b, SUC_lexicon_tags_0 },
    { 0x97999c2c, SUC_lexicon_tags_4 },
    { 0x49199c2d, SUC_lexicon_tags_35 },
    { 0x95199c2b, SUC_lexicon_tags_6 },
    { 0x24999c2f, SUC_lexicon_tags_23 },
    { 0x23799c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52599c35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed399c3b, SUC_lexicon_tags_60 },
    { 0xa599c3b, SUC_lexicon_tags_25 },
    { 0x71b99c3c, SUC_lexicon_tags_37 },
    { 0xe3999c3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5199c42, SUC_lexicon_tags_38 },
    { 0x51799c42, SUC_lexicon_tags_0 },
    { 0x94d99c44, SUC_lexicon_tags_43 },
    { 0xc4f99c44, SUC_lexicon_tags_15 },
    { 0xdb599c46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2d99c48, SUC_lexicon_tags_104 },
    { 0x20d99c49, SUC_lexicon_tags_27 },
    { 0x3c799c4a, SUC_lexicon_tags_0 },
    { 0x71d99c4a, SUC_lexicon_tags_47 },
    { 0x22399c4c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b99c51, SUC_lexicon_tags_15 },
    { 0xfc799c51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60799c57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe399c5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57199c61, SUC_lexicon_tags_23 },
    { 0xed599c61, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x91399c64, SUC_lexicon_tags_17 },
    { 0x66f99c65, SUC_lexicon_tags_11 },
    { 0xddd99c64, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd999c6b, SUC_lexicon_tags_2 },
    { 0x43399c6b, SUC_lexicon_tags_27 },
    { 0x5ad99c6d, SUC_lexicon_tags_27 },
    { 0x6fd99c6e, SUC_lexicon_tags_15 },
    { 0xdeb99c6f, SUC_lexicon_tags_25 },
    { 0x9db99c70, SUC_lexicon_tags_62 },
    { 0x71f99c70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72199c73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb599c76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f99c7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec199c86, SUC_lexicon_tags_38 },
    { 0x1c599c86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6199c8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x29399c8e, SUC_lexicon_tags_0 },
    { 0x57799c8f, SUC_lexicon_tags_23 },
    { 0xcf999c90, SUC_lexicon_tags_84 },
    { 0x1e799c8f, SUC_lexicon_tags_34 },
    { 0xccd99c92, SUC_lexicon_tags_12 },
    { 0x18f99c91, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16599c95, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d99c98, SUC_lexicon_tags_16 },
    { 0xeff99c99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab99ca0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0799ca4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc999ca6, SUC_lexicon_tags_27 },
    { 0xb7999ca7, SUC_lexicon_tags_11 },
    { 0xc0999ca7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23999cab, SUC_lexicon_tags_19 },
    { 0xfb599cac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab99cb1, SUC_lexicon_tags_27 },
    { 0xf5399cb2, SUC_lexicon_tags_25 },
    { 0xd7199cb1, SUC_lexicon_tags_46 },
    { 0x95999cb3, SUC_lexicon_tags_11 },
    { 0xa9599cb5, SUC_lexicon_tags_45 },
    { 0xf2d99cb5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45599cbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf999cc1, SUC_lexicon_tags_27 },
    { 0x77199cc2, SUC_lexicon_tags_17 },
    { 0x9f599cc3, SUC_lexicon_tags_14 },
    { 0x3db99cc2, SUC_lexicon_tags_37 },
    { 0x2d999cc5, SUC_lexicon_tags_27 },
    { 0xd6999cc2, SUC_lexicon_tags_47 },
    { 0x4df99cc7, SUC_lexicon_tags_37 },
    { 0x7d399cc8, SUC_lexicon_tags_15 },
    { 0xfbf99cc5, SUC_lexicon_tags_454 },
    { 0x8ad99cc8, SUC_lexicon_tags_14 },
    { 0x35b99cc4, SUC_lexicon_tags_17 },
    { 0x99f99ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb799ccf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b99cd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57399cd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0399cdd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d99ce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4199ce7, SUC_lexicon_tags_14 },
    { 0xbf799ce8, SUC_lexicon_tags_3 },
    { 0xd2f99ce9, SUC_lexicon_tags_8 },
    { 0xc0d99cea, SUC_lexicon_tags_50 },
    { 0xea999ce9, SUC_lexicon_tags_2 },
    { 0xa8d99ce7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0999cf1, SUC_lexicon_tags_170 },
    { 0xb7999cf2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c199cf5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5d399cf7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6bb99cf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9999cfc, SUC_lexicon_tags_38 },
    { 0x73599cfd, SUC_lexicon_tags_29 },
    { 0x18f99cfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a199d01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f99d04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1999d08, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b99d0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6199d11, SUC_lexicon_tags_12 },
    { 0xcaf99d11, SUC_lexicon_tags_30 },
    { 0x7d599d13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8399d18, SUC_lexicon_tags_62 },
    { 0xf3b99d19, SUC_lexicon_tags_10 },
    { 0x8eb99d19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1399d1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecb99d20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15799d22, SUC_lexicon_tags_3 },
    { 0xf2599d22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db99d28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd99d2b, SUC_lexicon_tags_25 },
    { 0x43799d2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd8799d2e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d199d31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe199d33, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcf999d35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef99d3c, SUC_lexicon_tags_48 },
    { 0xc0199d3d, SUC_lexicon_tags_108 },
    { 0x1ff99d3c, SUC_lexicon_tags_14 },
    { 0xe5d99d3f, SUC_lexicon_tags_14 },
    { 0x18599d3f, SUC_lexicon_tags_15 },
    { 0xa5399d3c, SUC_lexicon_tags_60 },
    { 0xc9599d42, SUC_lexicon_tags_1 },
    { 0x1c199d43, SUC_lexicon_tags_38 },
    { 0x94f99d3c, SUC_lexicon_tags_9 },
    { 0x94b99d45, SUC_lexicon_tags_7 },
    { 0x6f199d46, SUC_lexicon_tags_11 },
    { 0x38799d47, SUC_lexicon_tags_12 },
    { 0x86b99d48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb399d4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d99d4d, SUC_lexicon_tags_11 },
    { 0xca599d4e, SUC_lexicon_tags_7 },
    { 0xf3f99d4d, SUC_lexicon_tags_60 },
    { 0x54f99d50, SUC_lexicon_tags_11 },
    { 0xe9f99d4d, SUC_lexicon_tags_54 },
    { 0x7d999d4d, SUC_lexicon_tags_2 },
    { 0x7399d51, SUC_lexicon_tags_1 },
    { 0x8599d54, SUC_lexicon_tags_17 },
    { 0x98199d4f, SUC_lexicon_tags_14 },
    { 0x1c199d53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8999d5a, SUC_lexicon_tags_25 },
    { 0x74799d5b, SUC_lexicon_tags_12 },
    { 0x6b999d5b, SUC_lexicon_tags_45 },
    { 0x79399d5d, SUC_lexicon_tags_23 },
    { 0xdab99d5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f999d62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf599d68, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef399d6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d99d73, SUC_lexicon_tags_185 },
    { 0x22f99d74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b99d78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9f99d7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd99d7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4df99d82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb599d84, SUC_lexicon_tags_17 },
    { 0xc5d99d84, SUC_lexicon_tags_1 },
    { 0x2d199d86, SUC_lexicon_tags_160 },
    { 0xc5199d84, SUC_lexicon_tags_2 },
    { 0xa8d99d86, SUC_lexicon_tags_26 },
    { 0xb0d99d85, SUC_lexicon_tags_8 },
    { 0x69399d8a, SUC_lexicon_tags_17 },
    { 0xecb99d8b, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xb9b99d8d, SUC_lexicon_tags_46 },
    { 0xcd399d8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd99d92, SUC_lexicon_tags_38 },
    { 0x9f799d93, SUC_lexicon_tags_1 },
    { 0xad99d93, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61799d98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47b99d9a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x74199d9c, SUC_lexicon_tags_26 },
    { 0xeb199d9d, SUC_lexicon_tags_25 },
    { 0x9c999d9e, SUC_lexicon_tags_28 },
    { 0x26199d9f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d99da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69999dac, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf99db2, SUC_lexicon_tags_48 },
    { 0x4b999db3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41199db7, SUC_lexicon_tags_2 },
    { 0x8ed99db7, SUC_lexicon_tags_37 },
    { 0xc4b99db7, SUC_lexicon_tags_9 },
    { 0xdf99dba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd99dbd, SUC_lexicon_tags_60 },
    { 0x29d99dbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ff99dc0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x78999dc2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4599dca, SUC_lexicon_tags_25 },
    { 0x96d99dca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc799dd1, SUC_lexicon_tags_37 },
    { 0x58399dd1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b599dd7, SUC_lexicon_tags_2 },
    { 0x8f99dd7, SUC_lexicon_tags_19 },
    { 0x13d99dd7, SUC_lexicon_tags_7 },
    { 0xa1f99dda, SUC_lexicon_tags_12 },
    { 0xe0b99ddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b999de1, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0, NULL },
    { 0x84999de4, SUC_lexicon_tags_8 },
    { 0x7d799de5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbbd99de7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9999dec, SUC_lexicon_tags_37 },
    { 0x3c799ded, SUC_lexicon_tags_599 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed99df7, SUC_lexicon_tags_15 },
    { 0xd0199df8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef599dfc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91399e00, SUC_lexicon_tags_17 },
    { 0x80599e01, SUC_lexicon_tags_7 },
    { 0x2af99e00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec599e04, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xefd99e06, SUC_lexicon_tags_11 },
    { 0x6a799e06, SUC_lexicon_tags_1 },
    { 0x61b99e08, SUC_lexicon_tags_3 },
    { 0x75799e09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1399e0b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72599e12, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac599e18, SUC_lexicon_tags_63 },
    { 0x9f99e19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9ed99e1b, SUC_lexicon_tags_55 },
    { 0xa8d99e1c, SUC_lexicon_tags_25 },
    { 0x3b599e1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce399e1f, SUC_lexicon_tags_275 },
    { 0xe8b99e1f, SUC_lexicon_tags_6 },
    { 0x22b99e21, SUC_lexicon_tags_30 },
    { 0xbcb99e22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6999e25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48799e29, SUC_lexicon_tags_15 },
    { 0x9e599e2a, SUC_lexicon_tags_11 },
    { 0x55199e2b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58599e32, SUC_lexicon_tags_14 },
    { 0x2eb99e33, SUC_lexicon_tags_2 },
    { 0xdfd99e33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x62b99e36, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb1399e38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5cf99e3a, SUC_lexicon_tags_43 },
    { 0xa999e3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a599e3d, SUC_lexicon_tags_5 },
    { 0xa1999e3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e199e40, SUC_lexicon_tags_16 },
    { 0xdd999e40, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x32f99e44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75799e48, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c999e4d, SUC_lexicon_tags_60 },
    { 0xe4b99e4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78199e50, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70199e54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d99e57, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x93199e5a, SUC_lexicon_tags_31 },
    { 0x6b399e5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35d99e5d, SUC_lexicon_tags_47 },
    { 0x6b599e5e, SUC_lexicon_tags_39 },
    { 0xb8799e5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46199e62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb99e65, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce599e67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf599e69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f99e6e, SUC_lexicon_tags_64 },
    { 0xc5d99e6f, SUC_lexicon_tags_26 },
    { 0x78f99e6f, SUC_lexicon_tags_28 },
    { 0xb3b99e71, SUC_lexicon_tags_66 },
    { 0x69d99e71, SUC_lexicon_tags_0 },
    { 0x6b599e72, SUC_lexicon_tags_25 },
    { 0xb3599e74, SUC_lexicon_tags_30 },
    { 0x69799e75, SUC_lexicon_tags_7 },
    { 0x25199e74, SUC_lexicon_tags_1 },
    { 0x4a599e77, SUC_lexicon_tags_48 },
    { 0xf9f99e78, SUC_lexicon_tags_7 },
    { 0xfe799e78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1599e85, SUC_lexicon_tags_23 },
    { 0xc1999e86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35399e88, SUC_lexicon_tags_23 },
    { 0x3c999e89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9399e8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50199e8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd399e91, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xdc599e93, SUC_lexicon_tags_12 },
    { 0xa4399e94, SUC_lexicon_tags_44 },
    { 0x92f99e95, SUC_lexicon_tags_12 },
    { 0xd7599e94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81799e9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc799e9d, SUC_lexicon_tags_17 },
    { 0xc0d99e9e, SUC_lexicon_tags_17 },
    { 0xa8199e9f, SUC_lexicon_tags_1 },
    { 0x1af99e9e, SUC_lexicon_tags_7 },
    { 0x4ef99ea1, SUC_lexicon_tags_29 },
    { 0x43b99ea2, SUC_lexicon_tags_28 },
    { 0xe2f99ea3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4ed99ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d99ea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b99eac, SUC_lexicon_tags_23 },
    { 0x4a199ead, SUC_lexicon_tags_11 },
    { 0xda199eac, SUC_lexicon_tags_26 },
    { 0xc9d99eae, SUC_lexicon_tags_57 },
    { 0x76799eb0, SUC_lexicon_tags_15 },
    { 0xf8f99eb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25199ebb, SUC_lexicon_tags_19 },
    { 0xf8b99ebc, SUC_lexicon_tags_1 },
    { 0x1999ebd, SUC_lexicon_tags_3 },
    { 0x49b99ebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57799ec0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceb99ec4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b99ec7, SUC_lexicon_tags_37 },
    { 0x87399ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0f99eca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbf99ecc, SUC_lexicon_tags_37 },
    { 0x70d99ecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0999ed2, SUC_lexicon_tags_1 },
    { 0xe9d99ed3, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff99ee1, SUC_lexicon_tags_37 },
    { 0x8f199ee2, SUC_lexicon_tags_37 },
    { 0xb199ee3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6c399ee5, SUC_lexicon_tags_25 },
    { 0xe6999ee5, SUC_lexicon_tags_17 },
    { 0x59b99ee5, SUC_lexicon_tags_22 },
    { 0x96399ee6, SUC_lexicon_tags_43 },
    { 0x8cb99ee9, SUC_lexicon_tags_11 },
    { 0x53f99eea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98b99eec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f199eee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1799ef2, SUC_lexicon_tags_11 },
    { 0x90999ef2, SUC_lexicon_tags_0 },
    { 0xe6599ef3, SUC_lexicon_tags_27 },
    { 0x27599ef2, SUC_lexicon_tags_65 },
    { 0x7a999ef5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d99efb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cf99f01, SUC_lexicon_tags_25 },
    { 0x81999f02, SUC_lexicon_tags_16 },
    { 0x36d99f03, SUC_lexicon_tags_16 },
    { 0x62d99f04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x10999f06, SUC_lexicon_tags_80 },
    { 0x79d99f06, SUC_lexicon_tags_6 },
    { 0xf0199f08, SUC_lexicon_tags_65 },
    { 0x38d99f09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf399f0e, SUC_lexicon_tags_23 },
    { 0x3f599f0e, SUC_lexicon_tags_27 },
    { 0x5c999f10, SUC_lexicon_tags_42 },
    { 0xd7399f11, SUC_lexicon_tags_65 },
    { 0xbe799f12, SUC_lexicon_tags_37 },
    { 0x4ef99f0e, SUC_lexicon_tags_15 },
    { 0x10599f12, SUC_lexicon_tags_31 },
    { 0x70b99f15, SUC_lexicon_tags_11 },
    { 0xa2799f16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa599f1c, SUC_lexicon_tags_17 },
    { 0x58d99f1c, SUC_lexicon_tags_37 },
    { 0x94199f1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6199f21, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b99f24, SUC_lexicon_tags_106 },
    { 0x3b199f25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13399f27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb999f2b, SUC_lexicon_tags_60 },
    { 0xfd999f2c, SUC_lexicon_tags_37 },
    { 0xe4199f2d, SUC_lexicon_tags_11 },
    { 0x2d399f2c, SUC_lexicon_tags_28 },
    { 0x64399f2e, SUC_lexicon_tags_37 },
    { 0x1d599f30, SUC_lexicon_tags_17 },
    { 0x64b99f31, SUC_lexicon_tags_37 },
    { 0x66999f32, SUC_lexicon_tags_17 },
    { 0x22799f33, SUC_lexicon_tags_13 },
    { 0xdeb99f33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87f99f36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51799f38, SUC_lexicon_tags_11 },
    { 0xa7199f39, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcef99f3f, SUC_lexicon_tags_21 },
    { 0xfaf99f40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e199f42, SUC_lexicon_tags_38 },
    { 0x1df99f43, SUC_lexicon_tags_30 },
    { 0x5ed99f43, SUC_lexicon_tags_39 },
    { 0x91199f45, SUC_lexicon_tags_7 },
    { 0x6cf99f46, SUC_lexicon_tags_11 },
    { 0x18199f47, SUC_lexicon_tags_6 },
    { 0x65d99f46, SUC_lexicon_tags_11 },
    { 0x22599f48, SUC_lexicon_tags_45 },
    { 0x60f99f4a, SUC_lexicon_tags_37 },
    { 0xd6799f4b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa1d99f4d, SUC_lexicon_tags_43 },
    { 0x28b99f4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdaf99f50, SUC_lexicon_tags_10 },
    { 0x8399f51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf99f54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x31999f56, SUC_lexicon_tags_17 },
    { 0xe8f99f57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4799f5a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d99f5e, SUC_lexicon_tags_25 },
    { 0xbe999f5f, SUC_lexicon_tags_1 },
    { 0x72599f5f, SUC_lexicon_tags_15 },
    { 0x4bf99f61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9199f64, SUC_lexicon_tags_27 },
    { 0x85d99f65, SUC_lexicon_tags_25 },
    { 0xc6599f66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ad99f6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb9199f6c, SUC_lexicon_tags_11 },
    { 0xe8f99f6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1599f77, SUC_lexicon_tags_25 },
    { 0x1ab99f78, SUC_lexicon_tags_66 },
    { 0x67999f78, SUC_lexicon_tags_12 },
    { 0x3df99f7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab99f7e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd99f82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xea799f84, SUC_lexicon_tags_47 },
    { 0xea599f84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c199f87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccd99f89, SUC_lexicon_tags_17 },
    { 0x46799f89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8599f8e, SUC_lexicon_tags_30 },
    { 0xac399f8e, SUC_lexicon_tags_55 },
    { 0x9b99f90, SUC_lexicon_tags_17 },
    { 0x97799f90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96599f96, SUC_lexicon_tags_11 },
    { 0x97999f96, SUC_lexicon_tags_11 },
    { 0x42199f98, SUC_lexicon_tags_80 },
    { 0x3399f97, SUC_lexicon_tags_2 },
    { 0x73d99f99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4599f9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32d99f9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec999fa1, SUC_lexicon_tags_31 },
    { 0xabf99fa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ad99fa9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x45199fac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb99fb2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8a599fb4, SUC_lexicon_tags_27 },
    { 0x18999fb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f799fb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a599fc3, SUC_lexicon_tags_45 },
    { 0x73f99fc4, SUC_lexicon_tags_65 },
    { 0x97199fc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55199fc9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xee999fcc, SUC_lexicon_tags_23 },
    { 0x89199fcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8199fcf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42f99fd1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90999fd9, SUC_lexicon_tags_11 },
    { 0x9c799fd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f99fde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd199fe1, SUC_lexicon_tags_31 },
    { 0x22199fe2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f99ff3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1599ffa, SUC_lexicon_tags_1 },
    { 0x1f199ffb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4199a000, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6779a002, SUC_lexicon_tags_2 },
    { 0x6b79a003, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f9a006, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe379a008, SUC_lexicon_tags_3 },
    { 0x2279a009, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbf9a00b, SUC_lexicon_tags_59 },
    { 0xc4d9a00b, SUC_lexicon_tags_7 },
    { 0x4119a00c, SUC_lexicon_tags_45 },
    { 0xabf9a00e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9119a010, SUC_lexicon_tags_4 },
    { 0xc6f9a011, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcf9a017, SUC_lexicon_tags_1 },
    { 0x8cf9a017, SUC_lexicon_tags_60 },
    { 0x4d99a018, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9a01e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24d9a020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd39a024, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6559a026, SUC_lexicon_tags_1 },
    { 0xaff9a026, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc039a02a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1939a02c, SUC_lexicon_tags_3 },
    { 0x7939a02d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xced9a02f, SUC_lexicon_tags_37 },
    { 0x7659a030, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfb59a032, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xca39a035, SUC_lexicon_tags_25 },
    { 0xeaf9a036, SUC_lexicon_tags_17 },
    { 0x7ef9a037, SUC_lexicon_tags_21 },
    { 0x3bd9a038, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2159a03c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e19a03f, SUC_lexicon_tags_27 },
    { 0x859a040, SUC_lexicon_tags_29 },
    { 0x7479a040, SUC_lexicon_tags_29 },
    { 0xd819a040, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9979a046, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8699a049, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339a04d, SUC_lexicon_tags_87 },
    { 0xddf9a04d, SUC_lexicon_tags_37 },
    { 0x9a19a04f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xddf9a051, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xeb99a053, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b99a056, SUC_lexicon_tags_2 },
    { 0xcfd9a056, SUC_lexicon_tags_27 },
    { 0x5139a056, SUC_lexicon_tags_25 },
    { 0x6d19a059, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf9a05d, SUC_lexicon_tags_19 },
    { 0xaaf9a05e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe299a062, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f9a067, SUC_lexicon_tags_17 },
    { 0xff79a068, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2219a07a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc39a07d, SUC_lexicon_tags_19 },
    { 0xd0d9a07e, SUC_lexicon_tags_3 },
    { 0x8479a07f, SUC_lexicon_tags_11 },
    { 0xdcb9a080, SUC_lexicon_tags_11 },
    { 0x2999a07f, SUC_lexicon_tags_25 },
    { 0xdb19a082, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f9a086, SUC_lexicon_tags_29 },
    { 0x4ff9a087, SUC_lexicon_tags_15 },
    { 0x6f19a088, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9519a08e, SUC_lexicon_tags_15 },
    { 0x51f9a08e, SUC_lexicon_tags_31 },
    { 0xbd79a090, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd099a093, SUC_lexicon_tags_12 },
    { 0xa6b9a094, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8839a09a, SUC_lexicon_tags_31 },
    { 0x5b39a09a, SUC_lexicon_tags_1 },
    { 0x5579a09c, SUC_lexicon_tags_35 },
    { 0x5b79a09d, SUC_lexicon_tags_10 },
    { 0xe3b9a09d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xca79a0a1, SUC_lexicon_tags_17 },
    { 0x1239a0a1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc459a0a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6859a0a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ff9a0a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4259a0ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8679a0b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad9a0b6, SUC_lexicon_tags_22 },
    { 0x5b79a0b6, SUC_lexicon_tags_63 },
    { 0xf999a0b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5919a0bb, SUC_lexicon_tags_37 },
    { 0xf9d9a0bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df9a0c0, SUC_lexicon_tags_17 },
    { 0x6f79a0c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfcb9a0c3, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0xad59a0c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd19a0cb, SUC_lexicon_tags_26 },
    { 0x4db9a0cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5199a0ce, SUC_lexicon_tags_52 },
    { 0xa1b9a0cf, SUC_lexicon_tags_15 },
    { 0x6fb9a0cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a0d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2199a0d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4af9a0d9, SUC_lexicon_tags_25 },
    { 0x8359a0d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3bb9a0dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6339a0df, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b39a0e1, SUC_lexicon_tags_25 },
    { 0x3d59a0e2, SUC_lexicon_tags_1 },
    { 0x1e59a0e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f9a0e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff9a0ee, SUC_lexicon_tags_15 },
    { 0x3cf9a0ef, SUC_lexicon_tags_77 },
    { 0x94d9a0f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe2f9a0f2, SUC_lexicon_tags_21 },
    { 0x3899a0f3, SUC_lexicon_tags_2 },
    { 0x77f9a0f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f9a0f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa839a0fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96f9a0fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9079a102, SUC_lexicon_tags_19 },
    { 0xe099a103, SUC_lexicon_tags_1 },
    { 0x20d9a104, SUC_lexicon_tags_43 },
    { 0x7e39a104, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a108, SUC_lexicon_tags_7 },
    { 0x61b9a109, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7499a10b, SUC_lexicon_tags_8 },
    { 0x3f9a10c, SUC_lexicon_tags_65 },
    { 0xe479a10d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1859a110, SUC_lexicon_tags_11 },
    { 0x1df9a111, SUC_lexicon_tags_16 },
    { 0x9c19a111, SUC_lexicon_tags_0 },
    { 0xb099a112, SUC_lexicon_tags_63 },
    { 0xa5b9a114, SUC_lexicon_tags_43 },
    { 0xa9b9a111, SUC_lexicon_tags_65 },
    { 0x5af9a110, SUC_lexicon_tags_17 },
    { 0xb1d9a117, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa6b9a119, SUC_lexicon_tags_37 },
    { 0xcc99a119, SUC_lexicon_tags_25 },
    { 0x96f9a119, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d9a120, SUC_lexicon_tags_30 },
    { 0xbad9a120, SUC_lexicon_tags_0 },
    { 0x8559a122, SUC_lexicon_tags_28 },
    { 0xa3b9a120, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe59a125, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7079a128, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x1fb9a12a, SUC_lexicon_tags_37 },
    { 0x6319a12b, SUC_lexicon_tags_37 },
    { 0x60d9a12c, SUC_lexicon_tags_49 },
    { 0xaaf9a12b, SUC_lexicon_tags_21 },
    { 0x9959a12e, SUC_lexicon_tags_26 },
    { 0x6db9a12d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b9a132, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc0b9a134, SUC_lexicon_tags_38 },
    { 0xc339a134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9539a13a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa779a13c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2a99a13e, SUC_lexicon_tags_17 },
    { 0xd8b9a13e, SUC_lexicon_tags_87 },
    { 0xa359a140, SUC_lexicon_tags_2 },
    { 0xac39a141, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe2b9a143, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x46b9a145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2779a147, SUC_lexicon_tags_37 },
    { 0x30b9a148, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa559a14b, SUC_lexicon_tags_27 },
    { 0xd099a14c, SUC_lexicon_tags_15 },
    { 0xd199a14d, SUC_lexicon_tags_15 },
    { 0x6e19a14d, SUC_lexicon_tags_12 },
    { 0x4c59a14f, SUC_lexicon_tags_27 },
    { 0xccd9a14f, SUC_lexicon_tags_43 },
    { 0x7f39a14e, SUC_lexicon_tags_14 },
    { 0x19f9a152, SUC_lexicon_tags_12 },
    { 0xf6f9a152, SUC_lexicon_tags_14 },
    { 0x3df9a154, SUC_lexicon_tags_4 },
    { 0xf579a154, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf219a157, SUC_lexicon_tags_43 },
    { 0x4219a158, SUC_lexicon_tags_22 },
    { 0x95b9a158, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xb0b9a15b, SUC_lexicon_tags_24 },
    { 0xccd9a15c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d9a15f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xed79a161, SUC_lexicon_tags_15 },
    { 0x799a162, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4039a164, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1499a169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d9a16d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db9a170, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa799a172, SUC_lexicon_tags_30 },
    { 0xaa79a173, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe139a175, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31f9a177, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc419a181, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc139a183, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa979a186, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa019a18c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4619a18e, SUC_lexicon_tags_17 },
    { 0xfb19a18f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa4d9a191, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1339a193, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed9a198, SUC_lexicon_tags_14 },
    { 0x4bb9a199, SUC_lexicon_tags_28 },
    { 0x5f59a19a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d19a19d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe919a1a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb239a1a3, SUC_lexicon_tags_19 },
    { 0x8d99a1a4, SUC_lexicon_tags_43 },
    { 0xecd9a1a5, SUC_lexicon_tags_46 },
    { 0xe479a1a4, SUC_lexicon_tags_25 },
    { 0x1139a1a6, SUC_lexicon_tags_23 },
    { 0x459a1a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe019a1aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbb9a1ac, SUC_lexicon_tags_23 },
    { 0x34d9a1ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb9a1b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xebf9a1b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9099a1b7, SUC_lexicon_tags_14 },
    { 0x1f99a1b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbab9a1c1, SUC_lexicon_tags_1 },
    { 0x5239a1c2, SUC_lexicon_tags_37 },
    { 0x9d59a1c3, SUC_lexicon_tags_47 },
    { 0xce19a1c4, SUC_lexicon_tags_19 },
    { 0x9d99a1c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf9a1cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99a1d3, SUC_lexicon_tags_11 },
    { 0x7af9a1d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb199a1d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5c79a1d8, SUC_lexicon_tags_88 },
    { 0xb479a1d9, SUC_lexicon_tags_15 },
    { 0xb439a1da, SUC_lexicon_tags_23 },
    { 0x5859a1db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8bb9a1dd, SUC_lexicon_tags_19 },
    { 0x3659a1de, SUC_lexicon_tags_25 },
    { 0x7659a1df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf9a1e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f9a1e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xed39a1ea, SUC_lexicon_tags_49 },
    { 0xdb59a1eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46b9a1f7, SUC_lexicon_tags_11 },
    { 0x2359a1f8, SUC_lexicon_tags_33 },
    { 0x1f79a1f9, SUC_lexicon_tags_18 },
    { 0x9f59a1fa, SUC_lexicon_tags_2 },
    { 0xcab9a1fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5af9a1fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb779a203, SUC_lexicon_tags_31 },
    { 0x4e39a204, SUC_lexicon_tags_44 },
    { 0x5db9a205, SUC_lexicon_tags_0 },
    { 0x3cb9a205, SUC_lexicon_tags_12 },
    { 0x6f99a205, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xec99a209, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x24f9a20c, SUC_lexicon_tags_57 },
    { 0x89b9a20d, SUC_lexicon_tags_38 },
    { 0x599a20e, SUC_lexicon_tags_70 },
    { 0x48b9a20e, SUC_lexicon_tags_30 },
    { 0x8b39a210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c59a216, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7819a218, SUC_lexicon_tags_46 },
    { 0x7699a219, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0f9a21b, SUC_lexicon_tags_29 },
    { 0x24b9a21b, SUC_lexicon_tags_1 },
    { 0x2679a21c, SUC_lexicon_tags_2 },
    { 0x29f9a21e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4419a225, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa659a227, SUC_lexicon_tags_46 },
    { 0x8a19a228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd779a22a, SUC_lexicon_tags_37 },
    { 0x3539a22a, SUC_lexicon_tags_17 },
    { 0x1479a22b, SUC_lexicon_tags_2 },
    { 0x66b9a22d, SUC_lexicon_tags_49 },
    { 0x18b9a22d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d9a231, SUC_lexicon_tags_25 },
    { 0x44b9a232, SUC_lexicon_tags_48 },
    { 0xe459a232, SUC_lexicon_tags_43 },
    { 0x5979a234, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64d9a236, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ed9a238, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xced9a23d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb8b9a23f, SUC_lexicon_tags_37 },
    { 0xfbf9a23f, SUC_lexicon_tags_77 },
    { 0xad9a241, SUC_lexicon_tags_17 },
    { 0xa1b9a242, SUC_lexicon_tags_12 },
    { 0x3b99a242, SUC_lexicon_tags_23 },
    { 0x8c39a242, SUC_lexicon_tags_37 },
    { 0x7e39a23f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb099a247, SUC_lexicon_tags_1 },
    { 0x1339a248, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b9a250, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d9a257, SUC_lexicon_tags_0 },
    { 0x38b9a258, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b9a25d, SUC_lexicon_tags_37 },
    { 0xb899a25d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xac59a260, SUC_lexicon_tags_1080 },
    { 0, NULL },
    { 0x8739a262, SUC_lexicon_tags_33 },
    { 0xb599a263, SUC_lexicon_tags_23 },
    { 0xfb9a262, SUC_lexicon_tags_27 },
    { 0xe539a263, SUC_lexicon_tags_46 },
    { 0x9ef9a264, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58b9a26e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc579a273, SUC_lexicon_tags_17 },
    { 0x1d59a273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d9a279, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7739a27c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ef9a283, SUC_lexicon_tags_37 },
    { 0x84f9a283, SUC_lexicon_tags_1 },
    { 0xa199a283, SUC_lexicon_tags_91 },
    { 0x719a284, SUC_lexicon_tags_23 },
    { 0x5a39a287, SUC_lexicon_tags_37 },
    { 0xeff9a288, SUC_lexicon_tags_44 },
    { 0xe579a283, SUC_lexicon_tags_84 },
    { 0xb199a289, SUC_lexicon_tags_37 },
    { 0xd539a283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f9a28e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7679a293, SUC_lexicon_tags_13 },
    { 0x8159a294, SUC_lexicon_tags_12 },
    { 0xa0d9a294, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7819a297, SUC_lexicon_tags_21 },
    { 0xfbd9a298, SUC_lexicon_tags_19 },
    { 0x3db9a299, SUC_lexicon_tags_17 },
    { 0x2159a299, SUC_lexicon_tags_17 },
    { 0x7d99a298, SUC_lexicon_tags_2 },
    { 0x4a39a298, SUC_lexicon_tags_11 },
    { 0xa4f9a29d, SUC_lexicon_tags_38 },
    { 0x8d39a29e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39a2a6, SUC_lexicon_tags_25 },
    { 0x239a2a7, SUC_lexicon_tags_7 },
    { 0xa9f9a2a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4759a2aa, SUC_lexicon_tags_37 },
    { 0x7219a2ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d9a2b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc679a2b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe59a2b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6d9a2ba, SUC_lexicon_tags_2 },
    { 0xf1b9a2bb, SUC_lexicon_tags_23 },
    { 0xd659a2bc, SUC_lexicon_tags_37 },
    { 0xb819a2bd, SUC_lexicon_tags_11 },
    { 0xa259a2be, SUC_lexicon_tags_14 },
    { 0x5679a2be, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7739a2c6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5619a2ca, SUC_lexicon_tags_38 },
    { 0xc039a2cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb9a2cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa379a2d3, SUC_lexicon_tags_62 },
    { 0x6b39a2d4, SUC_lexicon_tags_29 },
    { 0x20d9a2d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa139a2d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2819a2e0, SUC_lexicon_tags_38 },
    { 0x77d9a2e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8399a2e3, SUC_lexicon_tags_47 },
    { 0x5ab9a2e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa39a2e6, SUC_lexicon_tags_33 },
    { 0xa719a2e7, SUC_lexicon_tags_0 },
    { 0xae59a2e7, SUC_lexicon_tags_37 },
    { 0x539a2e9, SUC_lexicon_tags_13 },
    { 0x6679a2ea, SUC_lexicon_tags_26 },
    { 0xc799a2e6, SUC_lexicon_tags_7 },
    { 0xab79a2eb, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x6099a2ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x31b9a2f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f9a2f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcdf9a2f7, SUC_lexicon_tags_84 },
    { 0x7c99a2f8, SUC_lexicon_tags_35 },
    { 0x2c59a2f7, SUC_lexicon_tags_1 },
    { 0x91d9a2fa, SUC_lexicon_tags_37 },
    { 0x999a2fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49b9a302, SUC_lexicon_tags_25 },
    { 0x9879a303, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x53b9a306, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7999a30b, SUC_lexicon_tags_4 },
    { 0xb919a30b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef9a311, SUC_lexicon_tags_32 },
    { 0xfc79a312, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3739a314, SUC_lexicon_tags_21 },
    { 0x1399a315, SUC_lexicon_tags_19 },
    { 0xcc39a314, SUC_lexicon_tags_17 },
    { 0x1d9a314, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb979a319, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1259a31b, SUC_lexicon_tags_1097 },
    { 0xa079a31c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6059a320, SUC_lexicon_tags_11 },
    { 0x4c39a320, SUC_lexicon_tags_60 },
    { 0xf659a322, SUC_lexicon_tags_15 },
    { 0x8e19a323, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9a326, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9339a328, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d19a32b, SUC_lexicon_tags_17 },
    { 0x7c19a32b, SUC_lexicon_tags_26 },
    { 0x5c19a32c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb9a333, SUC_lexicon_tags_37 },
    { 0xde99a333, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72b9a336, SUC_lexicon_tags_0 },
    { 0xbdd9a336, SUC_lexicon_tags_9 },
    { 0xcab9a336, SUC_lexicon_tags_415 },
    { 0x1bf9a339, SUC_lexicon_tags_14 },
    { 0x87d9a33a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa499a33e, SUC_lexicon_tags_108 },
    { 0x4fb9a33f, SUC_lexicon_tags_7 },
    { 0xa299a33f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bd9a344, SUC_lexicon_tags_14 },
    { 0xd3f9a345, SUC_lexicon_tags_12 },
    { 0x83d9a345, SUC_lexicon_tags_40 },
    { 0x75f9a345, SUC_lexicon_tags_11 },
    { 0x86b9a347, SUC_lexicon_tags_47 },
    { 0xe8d9a347, SUC_lexicon_tags_10 },
    { 0x4cf9a34a, SUC_lexicon_tags_0 },
    { 0x78d9a34b, SUC_lexicon_tags_61 },
    { 0x38f9a346, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf499a34e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed9a351, SUC_lexicon_tags_84 },
    { 0x8779a352, SUC_lexicon_tags_33 },
    { 0xfaf9a351, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab9a356, SUC_lexicon_tags_14 },
    { 0x3419a357, SUC_lexicon_tags_83 },
    { 0xe599a358, SUC_lexicon_tags_33 },
    { 0x99f9a357, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6439a35e, SUC_lexicon_tags_45 },
    { 0x11f9a35e, SUC_lexicon_tags_25 },
    { 0xa879a360, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xacf9a363, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf419a368, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f9a36b, SUC_lexicon_tags_25 },
    { 0x1dd9a36c, SUC_lexicon_tags_47 },
    { 0xebd9a36d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9079a370, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64f9a373, SUC_lexicon_tags_17 },
    { 0x3239a373, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c79a378, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x88b9a37a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f9a380, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f9a383, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6f59a385, SUC_lexicon_tags_16 },
    { 0x5339a386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf79a388, SUC_lexicon_tags_9 },
    { 0x5839a389, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe699a38c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa6d9a38e, SUC_lexicon_tags_37 },
    { 0x9079a38e, SUC_lexicon_tags_2 },
    { 0x4dd9a38f, SUC_lexicon_tags_24 },
    { 0x3f9a391, SUC_lexicon_tags_7 },
    { 0x91f9a392, SUC_lexicon_tags_37 },
    { 0x9739a390, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f9a397, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x5279a399, SUC_lexicon_tags_15 },
    { 0x5fb9a39a, SUC_lexicon_tags_17 },
    { 0x6779a39b, SUC_lexicon_tags_32 },
    { 0x6479a39c, SUC_lexicon_tags_20 },
    { 0x49b9a39d, SUC_lexicon_tags_31 },
    { 0x1819a39e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a19a3a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd279a3a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1299a3a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e19a3ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4519a3b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc19a3b3, SUC_lexicon_tags_9 },
    { 0xf8d9a3b3, SUC_lexicon_tags_17 },
    { 0x63b9a3b5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb879a3ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9839a3bd, SUC_lexicon_tags_45 },
    { 0xaf79a3bd, SUC_lexicon_tags_56 },
    { 0xff9a3bf, SUC_lexicon_tags_25 },
    { 0x359a3c0, SUC_lexicon_tags_37 },
    { 0x34b9a3bd, SUC_lexicon_tags_17 },
    { 0x59b9a3bf, SUC_lexicon_tags_12 },
    { 0x8bf9a3c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe879a3c7, SUC_lexicon_tags_17 },
    { 0x10d9a3c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d9a3cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89f9a3d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe359a3d3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8379a3d5, SUC_lexicon_tags_0 },
    { 0x3739a3d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1719a3d8, SUC_lexicon_tags_17 },
    { 0x9f9a3d9, SUC_lexicon_tags_11 },
    { 0xf099a3da, SUC_lexicon_tags_1 },
    { 0x1d99a3da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e79a3de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc579a3e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd19a3e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cd9a3e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb39a3e9, SUC_lexicon_tags_15 },
    { 0x8519a3ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb7d9a3ec, SUC_lexicon_tags_30 },
    { 0x1759a3ec, SUC_lexicon_tags_37 },
    { 0x10f9a3ed, SUC_lexicon_tags_63 },
    { 0x9e59a3ef, SUC_lexicon_tags_47 },
    { 0xbc39a3f0, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x9d19a3f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2819a3f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae19a3fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59a3ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0d9a401, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d9a404, SUC_lexicon_tags_15 },
    { 0xa059a405, SUC_lexicon_tags_65 },
    { 0x1419a405, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8f9a408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4259a40b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b9a40f, SUC_lexicon_tags_37 },
    { 0x1d99a410, SUC_lexicon_tags_25 },
    { 0x1739a411, SUC_lexicon_tags_37 },
    { 0x2079a410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac19a414, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4959a418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db9a41f, SUC_lexicon_tags_12 },
    { 0x8a79a420, SUC_lexicon_tags_33 },
    { 0xf9b9a421, SUC_lexicon_tags_33 },
    { 0x8319a422, SUC_lexicon_tags_47 },
    { 0x1c79a423, SUC_lexicon_tags_37 },
    { 0x8479a424, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3039a427, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x1d99a429, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d9a42d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b9a430, SUC_lexicon_tags_0 },
    { 0x2819a431, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8779a434, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa299a437, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff59a439, SUC_lexicon_tags_14 },
    { 0x4399a43a, SUC_lexicon_tags_0 },
    { 0xfa39a43a, SUC_lexicon_tags_47 },
    { 0x4b79a43c, SUC_lexicon_tags_37 },
    { 0x1779a43d, SUC_lexicon_tags_12 },
    { 0xdd19a439, SUC_lexicon_tags_13 },
    { 0xadb9a43e, SUC_lexicon_tags_23 },
    { 0x18d9a440, SUC_lexicon_tags_63 },
    { 0x82d9a441, SUC_lexicon_tags_2 },
    { 0xdd39a43a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xae59a445, SUC_lexicon_tags_60 },
    { 0x74d9a445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc259a448, SUC_lexicon_tags_43 },
    { 0x4e99a449, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3339a452, SUC_lexicon_tags_6 },
    { 0x5dd9a452, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf259a457, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39a45a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff9a45f, SUC_lexicon_tags_37 },
    { 0x28b9a45f, SUC_lexicon_tags_47 },
    { 0x7279a460, SUC_lexicon_tags_0 },
    { 0x5cb9a45f, SUC_lexicon_tags_3 },
    { 0xf059a461, SUC_lexicon_tags_14 },
    { 0xb099a464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8459a466, SUC_lexicon_tags_75 },
    { 0x28b9a467, SUC_lexicon_tags_57 },
    { 0xecb9a468, SUC_lexicon_tags_12 },
    { 0x3019a468, SUC_lexicon_tags_14 },
    { 0x6d79a467, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb419a46e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3959a472, SUC_lexicon_tags_15 },
    { 0x8ab9a473, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x599a475, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee59a47b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x56d9a47d, SUC_lexicon_tags_14 },
    { 0xc599a47e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb319a480, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce39a483, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb339a487, SUC_lexicon_tags_12 },
    { 0xccd9a488, SUC_lexicon_tags_30 },
    { 0x6b19a489, SUC_lexicon_tags_25 },
    { 0xa339a48a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4799a490, SUC_lexicon_tags_14 },
    { 0x4fb9a491, SUC_lexicon_tags_8 },
    { 0x3699a492, SUC_lexicon_tags_65 },
    { 0x76b9a493, SUC_lexicon_tags_26 },
    { 0x6a39a491, SUC_lexicon_tags_16 },
    { 0xd899a495, SUC_lexicon_tags_6 },
    { 0xad39a495, SUC_lexicon_tags_31 },
    { 0x9799a497, SUC_lexicon_tags_23 },
    { 0xe6b9a492, SUC_lexicon_tags_31 },
    { 0x6ff9a499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b9a49c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9a4a2, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f9a4a7, SUC_lexicon_tags_60 },
    { 0xc759a4a7, SUC_lexicon_tags_17 },
    { 0x6cf9a4a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f79a4ae, SUC_lexicon_tags_60 },
    { 0x24d9a4af, SUC_lexicon_tags_9 },
    { 0xa5d9a4af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca79a4b4, SUC_lexicon_tags_25 },
    { 0xacd9a4b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb019a4b9, SUC_lexicon_tags_33 },
    { 0xea59a4ba, SUC_lexicon_tags_14 },
    { 0x7939a4ba, SUC_lexicon_tags_6 },
    { 0xf279a4bc, SUC_lexicon_tags_17 },
    { 0x2279a4bd, SUC_lexicon_tags_11 },
    { 0x5699a4bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e59a4c1, SUC_lexicon_tags_1 },
    { 0x90b9a4c2, SUC_lexicon_tags_29 },
    { 0xf439a4c2, SUC_lexicon_tags_71 },
    { 0xad39a4c3, SUC_lexicon_tags_26 },
    { 0x77d9a4c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9a4c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8679a4cc, SUC_lexicon_tags_10 },
    { 0x7459a4cc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2239a4cf, SUC_lexicon_tags_0 },
    { 0x9f99a4d0, SUC_lexicon_tags_31 },
    { 0x36f9a4d0, SUC_lexicon_tags_38 },
    { 0x4cf9a4d1, SUC_lexicon_tags_77 },
    { 0xe659a4cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa799a4d6, SUC_lexicon_tags_7 },
    { 0x2b39a4d7, SUC_lexicon_tags_7 },
    { 0xe799a4d8, SUC_lexicon_tags_1 },
    { 0x8b39a4d8, SUC_lexicon_tags_8 },
    { 0xe7f9a4d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6659a4de, SUC_lexicon_tags_25 },
    { 0xbf9a4df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x94f9a4e1, SUC_lexicon_tags_30 },
    { 0x44f9a4e2, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2979a4e7, SUC_lexicon_tags_2 },
    { 0x43f9a4e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24b9a4ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa8f9a4ec, SUC_lexicon_tags_79 },
    { 0x2659a4ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1b9a4ef, SUC_lexicon_tags_62 },
    { 0xc099a4f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe939a4f3, SUC_lexicon_tags_38 },
    { 0x5d9a4f3, SUC_lexicon_tags_1 },
    { 0x2439a4f3, SUC_lexicon_tags_47 },
    { 0x7319a4f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2159a4f8, SUC_lexicon_tags_4 },
    { 0xdab9a4f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3159a4fb, SUC_lexicon_tags_37 },
    { 0xd979a4fb, SUC_lexicon_tags_12 },
    { 0x8119a4fc, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd879a503, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b9a508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2479a50c, SUC_lexicon_tags_74 },
    { 0x7c99a50c, SUC_lexicon_tags_7 },
    { 0xd8f9a50e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc239a511, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec39a513, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a79a516, SUC_lexicon_tags_43 },
    { 0x8e79a517, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4959a519, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6939a51b, SUC_lexicon_tags_37 },
    { 0x4cb9a51c, SUC_lexicon_tags_47 },
    { 0xf679a51d, SUC_lexicon_tags_23 },
    { 0x4499a51e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e79a522, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa99a525, SUC_lexicon_tags_12 },
    { 0x90b9a526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb99a52b, SUC_lexicon_tags_37 },
    { 0x1d79a52b, SUC_lexicon_tags_47 },
    { 0x2ad9a52d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1999a52f, SUC_lexicon_tags_5 },
    { 0xbb19a52f, SUC_lexicon_tags_32 },
    { 0x3eb9a531, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7839a533, SUC_lexicon_tags_14 },
    { 0x1d9a534, SUC_lexicon_tags_29 },
    { 0x8519a535, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaef9a537, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x939a539, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6399a53d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9199a542, SUC_lexicon_tags_37 },
    { 0xe559a542, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1db9a545, SUC_lexicon_tags_25 },
    { 0x51d9a546, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd4f9a548, SUC_lexicon_tags_56 },
    { 0xbb99a549, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x73b9a54b, SUC_lexicon_tags_46 },
    { 0xf959a54c, SUC_lexicon_tags_23 },
    { 0x19d9a54b, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf9a550, SUC_lexicon_tags_2 },
    { 0x85b9a550, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c39a556, SUC_lexicon_tags_159 },
    { 0x1e99a557, SUC_lexicon_tags_17 },
    { 0x28d9a558, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c99a55e, SUC_lexicon_tags_15 },
    { 0x8379a55f, SUC_lexicon_tags_11 },
    { 0x4f79a560, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b99a563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1439a565, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb539a568, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619a56d, SUC_lexicon_tags_23 },
    { 0x7ab9a56d, SUC_lexicon_tags_38 },
    { 0x5e59a56f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd659a572, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x9459a574, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc2d9a576, SUC_lexicon_tags_59 },
    { 0x23d9a576, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b9a57b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7839a57e, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xe339a581, SUC_lexicon_tags_10 },
    { 0x7799a582, SUC_lexicon_tags_9 },
    { 0xaa59a583, SUC_lexicon_tags_37 },
    { 0x3099a583, SUC_lexicon_tags_22 },
    { 0xac79a585, SUC_lexicon_tags_4 },
    { 0x1d79a585, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3099a588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679a58d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad39a593, SUC_lexicon_tags_26 },
    { 0x8439a593, SUC_lexicon_tags_18 },
    { 0x9319a593, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xd0b9a597, SUC_lexicon_tags_29 },
    { 0xc539a597, SUC_lexicon_tags_25 },
    { 0x8bb9a599, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb9a59d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4479a59f, SUC_lexicon_tags_39 },
    { 0x4559a5a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb9a5a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e99a5a4, SUC_lexicon_tags_45 },
    { 0x7eb9a5a4, SUC_lexicon_tags_178 },
    { 0x8e79a5a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3699a5a8, SUC_lexicon_tags_29 },
    { 0x7899a5a8, SUC_lexicon_tags_46 },
    { 0x66d9a5a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb9a5ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf199a5af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e39a5b1, SUC_lexicon_tags_60 },
    { 0xe579a5b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc959a5b4, SUC_lexicon_tags_17 },
    { 0x4879a5b5, SUC_lexicon_tags_31 },
    { 0x4779a5b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe699a5b8, SUC_lexicon_tags_23 },
    { 0x43f9a5b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43f9a5bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cb9a5bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2779a5c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3ff9a5c5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e99a5c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b99a5d1, SUC_lexicon_tags_47 },
    { 0x3779a5d2, SUC_lexicon_tags_63 },
    { 0xc39a5d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef79a5d5, SUC_lexicon_tags_25 },
    { 0x8879a5d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62f9a5d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf9a5dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59d9a5df, SUC_lexicon_tags_48 },
    { 0x42d9a5e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1919a5e5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f39a5e9, SUC_lexicon_tags_4 },
    { 0xf3f9a5ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c19a5ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7f19a5f1, SUC_lexicon_tags_19 },
    { 0x66b9a5f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe919a5f4, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xb6f9a5f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba99a5f8, SUC_lexicon_tags_12 },
    { 0xde19a5f9, SUC_lexicon_tags_5 },
    { 0x4ad9a5f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f9a601, SUC_lexicon_tags_6 },
    { 0xeb39a602, SUC_lexicon_tags_1 },
    { 0x6859a602, SUC_lexicon_tags_37 },
    { 0x90b9a602, SUC_lexicon_tags_25 },
    { 0xeb19a603, SUC_lexicon_tags_66 },
    { 0xf559a602, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1339a60b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff9a613, SUC_lexicon_tags_43 },
    { 0x4cf9a614, SUC_lexicon_tags_56 },
    { 0x1999a613, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec59a622, SUC_lexicon_tags_11 },
    { 0xf9b9a623, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b9a626, SUC_lexicon_tags_136 },
    { 0x33d9a627, SUC_lexicon_tags_30 },
    { 0xc219a628, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f19a62c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe079a631, SUC_lexicon_tags_87 },
    { 0xf9a632, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7119a63a, SUC_lexicon_tags_46 },
    { 0x139a63b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f9a63f, SUC_lexicon_tags_25 },
    { 0x4499a63f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb9a644, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f39a648, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5979a64b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1799a64e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9999a653, SUC_lexicon_tags_31 },
    { 0x8779a654, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f9a658, SUC_lexicon_tags_11 },
    { 0x8c99a658, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba19a65e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f9a663, SUC_lexicon_tags_37 },
    { 0xa7f9a664, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64f9a666, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4179a669, SUC_lexicon_tags_0 },
    { 0x3979a669, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4039a66f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa119a672, SUC_lexicon_tags_65 },
    { 0x2e39a673, SUC_lexicon_tags_35 },
    { 0x42b9a674, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc239a676, SUC_lexicon_tags_4 },
    { 0xee39a677, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30f9a67d, SUC_lexicon_tags_14 },
    { 0xb599a67d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5059a685, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f79a687, SUC_lexicon_tags_26 },
    { 0x7939a687, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8059a68b, SUC_lexicon_tags_15 },
    { 0xd419a68b, SUC_lexicon_tags_6 },
    { 0x37f9a68c, SUC_lexicon_tags_63 },
    { 0x6c19a68d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f9a693, SUC_lexicon_tags_9 },
    { 0x6159a694, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb259a699, SUC_lexicon_tags_11 },
    { 0x8e99a69a, SUC_lexicon_tags_45 },
    { 0x2e39a69b, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x98d9a69e, SUC_lexicon_tags_49 },
    { 0x38f9a69f, SUC_lexicon_tags_11 },
    { 0x2ff9a6a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad79a6a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5d9a6a6, SUC_lexicon_tags_82 },
    { 0xf1d9a6a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa59a6aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa919a6af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b99a6b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3b9a6b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5719a6b7, SUC_lexicon_tags_1 },
    { 0xbb99a6b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc959a6bf, SUC_lexicon_tags_23 },
    { 0x4e79a6bf, SUC_lexicon_tags_25 },
    { 0xfe79a6bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd99a6c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d9a6c7, SUC_lexicon_tags_19 },
    { 0x4059a6c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6659a6cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b9a6d0, SUC_lexicon_tags_17 },
    { 0x3cd9a6d0, SUC_lexicon_tags_9 },
    { 0x3679a6d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519a6d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6199a6d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb6b9a6da, SUC_lexicon_tags_63 },
    { 0x5859a6db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4579a6e1, SUC_lexicon_tags_7 },
    { 0x71d9a6e2, SUC_lexicon_tags_9 },
    { 0xe6d9a6e2, SUC_lexicon_tags_14 },
    { 0xdfd9a6e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe999a6e6, SUC_lexicon_tags_179 },
    { 0x64b9a6e6, SUC_lexicon_tags_39 },
    { 0x7ff9a6e6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3c39a6ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd79a6ed, SUC_lexicon_tags_138 },
    { 0x2b39a6ed, SUC_lexicon_tags_9 },
    { 0x2bd9a6ee, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x3559a6f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf419a6f7, SUC_lexicon_tags_71 },
    { 0x8919a6f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2359a6fb, SUC_lexicon_tags_17 },
    { 0xe6f9a6fb, SUC_lexicon_tags_52 },
    { 0xfcb9a6fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd9a707, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7939a70b, SUC_lexicon_tags_37 },
    { 0xc3b9a70c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8cd9a70e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x599a710, SUC_lexicon_tags_58 },
    { 0x27b9a711, SUC_lexicon_tags_19 },
    { 0x4e39a711, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1299a714, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb919a718, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4559a71a, SUC_lexicon_tags_48 },
    { 0xaa99a71a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff59a71d, SUC_lexicon_tags_678 },
    { 0xa5f9a71e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbab9a720, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4559a727, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e99a729, SUC_lexicon_tags_106 },
    { 0xadf9a72a, SUC_lexicon_tags_37 },
    { 0x1d99a72b, SUC_lexicon_tags_17 },
    { 0x6a39a72c, SUC_lexicon_tags_104 },
    { 0x7a79a72d, SUC_lexicon_tags_11 },
    { 0x8979a72e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc819a732, SUC_lexicon_tags_37 },
    { 0x3659a732, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d9a739, SUC_lexicon_tags_29 },
    { 0x8859a73a, SUC_lexicon_tags_15 },
    { 0xe499a73b, SUC_lexicon_tags_30 },
    { 0xef99a73c, SUC_lexicon_tags_37 },
    { 0x76d9a73d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4459a73f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6219a742, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x5dd9a744, SUC_lexicon_tags_47 },
    { 0x33f9a745, SUC_lexicon_tags_9 },
    { 0x4319a746, SUC_lexicon_tags_1 },
    { 0x2579a747, SUC_lexicon_tags_37 },
    { 0x79b9a748, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5279a74b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9a750, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc959a752, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x53d9a754, SUC_lexicon_tags_71 },
    { 0x7719a755, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79a759, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74b9a75e, SUC_lexicon_tags_63 },
    { 0xe879a75e, SUC_lexicon_tags_160 },
    { 0x8579a760, SUC_lexicon_tags_82 },
    { 0xddb9a761, SUC_lexicon_tags_42 },
    { 0xbf9a762, SUC_lexicon_tags_2 },
    { 0x75b9a763, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7559a766, SUC_lexicon_tags_11 },
    { 0x8079a767, SUC_lexicon_tags_13 },
    { 0xb299a766, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb19a76e, SUC_lexicon_tags_19 },
    { 0x3179a76f, SUC_lexicon_tags_12 },
    { 0x92d9a76e, SUC_lexicon_tags_35 },
    { 0xb1d9a771, SUC_lexicon_tags_37 },
    { 0x5f39a772, SUC_lexicon_tags_27 },
    { 0xbfb9a771, SUC_lexicon_tags_47 },
    { 0xab39a76e, SUC_lexicon_tags_9 },
    { 0x4e19a772, SUC_lexicon_tags_15 },
    { 0x9b99a776, SUC_lexicon_tags_30 },
    { 0x9a39a771, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f19a77e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49d9a780, SUC_lexicon_tags_8 },
    { 0xe959a780, SUC_lexicon_tags_55 },
    { 0x4059a782, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7b79a784, SUC_lexicon_tags_17 },
    { 0xc539a785, SUC_lexicon_tags_17 },
    { 0x9899a786, SUC_lexicon_tags_10 },
    { 0x6479a785, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xec59a78a, SUC_lexicon_tags_10 },
    { 0x84f9a78b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa639a790, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf79a793, SUC_lexicon_tags_16 },
    { 0xa179a794, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e59a79e, SUC_lexicon_tags_27 },
    { 0x3419a79e, SUC_lexicon_tags_27 },
    { 0x9a39a7a0, SUC_lexicon_tags_17 },
    { 0x87f9a7a0, SUC_lexicon_tags_2 },
    { 0xd199a7a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2399a7a4, SUC_lexicon_tags_2 },
    { 0xabf9a7a5, SUC_lexicon_tags_1 },
    { 0xacb9a7a5, SUC_lexicon_tags_10 },
    { 0x55f9a7a5, SUC_lexicon_tags_12 },
    { 0xc3d9a7a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb579a7aa, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9a7b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb39a7b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae79a7bb, SUC_lexicon_tags_17 },
    { 0x5a59a7bb, SUC_lexicon_tags_7 },
    { 0xd159a7bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40f9a7c3, SUC_lexicon_tags_11 },
    { 0x6259a7c3, SUC_lexicon_tags_17 },
    { 0xa6f9a7c5, SUC_lexicon_tags_37 },
    { 0x3fb9a7c6, SUC_lexicon_tags_26 },
    { 0x499a7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47f9a7c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fb9a7cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4679a7d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd9a7da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1679a7dd, SUC_lexicon_tags_60 },
    { 0x3279a7de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x79d9a7e1, SUC_lexicon_tags_30 },
    { 0x1679a7e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f9a7e5, SUC_lexicon_tags_17 },
    { 0xc179a7e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7b9a7e8, SUC_lexicon_tags_7 },
    { 0x9639a7e8, SUC_lexicon_tags_48 },
    { 0xe1b9a7ea, SUC_lexicon_tags_34 },
    { 0x80f9a7eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b9a7f2, SUC_lexicon_tags_77 },
    { 0x7f39a7f2, SUC_lexicon_tags_26 },
    { 0x7239a7f4, SUC_lexicon_tags_47 },
    { 0x7fd9a7f5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd7f9a7f7, SUC_lexicon_tags_31 },
    { 0xbbb9a7f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ab9a7fa, SUC_lexicon_tags_17 },
    { 0xfd99a7fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62b9a7fd, SUC_lexicon_tags_37 },
    { 0xf2d9a7fe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8159a802, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c59a804, SUC_lexicon_tags_15 },
    { 0x9d9a805, SUC_lexicon_tags_7 },
    { 0xa9d9a806, SUC_lexicon_tags_6 },
    { 0xf3b9a804, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2619a80a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe179a80c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x19f9a80e, SUC_lexicon_tags_1 },
    { 0xb5f9a80e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd9a814, SUC_lexicon_tags_95 },
    { 0x1199a815, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd319a817, SUC_lexicon_tags_1 },
    { 0x61d9a817, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x73b9a81a, SUC_lexicon_tags_37 },
    { 0xd019a81b, SUC_lexicon_tags_11 },
    { 0x38b9a81b, SUC_lexicon_tags_15 },
    { 0x4e19a81d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8859a820, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x74d9a822, SUC_lexicon_tags_37 },
    { 0x3899a822, SUC_lexicon_tags_63 },
    { 0x3459a823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f9a827, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2499a82c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x81b9a82f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4839a831, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb439a839, SUC_lexicon_tags_31 },
    { 0xb4d9a83a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1699a842, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd9a845, SUC_lexicon_tags_7 },
    { 0xf5f9a846, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa919a848, SUC_lexicon_tags_29 },
    { 0x5919a849, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb879a84c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc139a84e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9739a852, SUC_lexicon_tags_11 },
    { 0x6cb9a852, SUC_lexicon_tags_6 },
    { 0xf779a852, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d9a859, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3719a85c, SUC_lexicon_tags_23 },
    { 0x519a85d, SUC_lexicon_tags_9 },
    { 0xd6d9a85d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f9a862, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab9a865, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9a869, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1019a86b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef9a871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97b9a874, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b99a877, SUC_lexicon_tags_13 },
    { 0x94f9a877, SUC_lexicon_tags_27 },
    { 0xa179a877, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2319a87b, SUC_lexicon_tags_4 },
    { 0x4439a87b, SUC_lexicon_tags_25 },
    { 0xaad9a87c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd9a881, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6219a883, SUC_lexicon_tags_25 },
    { 0x3a99a884, SUC_lexicon_tags_33 },
    { 0xf6f9a885, SUC_lexicon_tags_17 },
    { 0x3d9a886, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe599a88e, SUC_lexicon_tags_37 },
    { 0x3d79a88f, SUC_lexicon_tags_39 },
    { 0x96d9a88f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed9a896, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0b9a89a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d9a89f, SUC_lexicon_tags_11 },
    { 0x2899a8a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56f9a8a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6379a8a4, SUC_lexicon_tags_25 },
    { 0x99a8a5, SUC_lexicon_tags_15 },
    { 0xadf9a8a5, SUC_lexicon_tags_42 },
    { 0xfc39a8a7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9ff9a8a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbed9a8ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec79a8ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2cb9a8af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x419a8b1, SUC_lexicon_tags_15 },
    { 0xc759a8b2, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x5319a8b4, SUC_lexicon_tags_17 },
    { 0x8d59a8b5, SUC_lexicon_tags_17 },
    { 0x699a8b6, SUC_lexicon_tags_74 },
    { 0x15f9a8b5, SUC_lexicon_tags_45 },
    { 0xab99a8b8, SUC_lexicon_tags_11 },
    { 0x7fb9a8b7, SUC_lexicon_tags_68 },
    { 0x5f19a8ba, SUC_lexicon_tags_2 },
    { 0xf999a8b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7659a8c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85b9a8c4, SUC_lexicon_tags_29 },
    { 0x479a8c4, SUC_lexicon_tags_37 },
    { 0x41b9a8c6, SUC_lexicon_tags_37 },
    { 0xa2d9a8c7, SUC_lexicon_tags_11 },
    { 0x1139a8c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5719a8d7, SUC_lexicon_tags_11 },
    { 0x68d9a8d8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x5879a8db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c59a8e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919a8e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f39a8ea, SUC_lexicon_tags_45 },
    { 0x2639a8ea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef39a8f6, SUC_lexicon_tags_11 },
    { 0xd699a8f7, SUC_lexicon_tags_47 },
    { 0x7439a8f7, SUC_lexicon_tags_27 },
    { 0x8459a8f9, SUC_lexicon_tags_2 },
    { 0xe319a8f8, SUC_lexicon_tags_23 },
    { 0xe7b9a8fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33f9a8fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb39a901, SUC_lexicon_tags_23 },
    { 0xc699a902, SUC_lexicon_tags_45 },
    { 0x8379a903, SUC_lexicon_tags_37 },
    { 0x6999a902, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd9a908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c99a90e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb9a915, SUC_lexicon_tags_21 },
    { 0x57b9a916, SUC_lexicon_tags_1 },
    { 0x4399a915, SUC_lexicon_tags_11 },
    { 0xb099a918, SUC_lexicon_tags_25 },
    { 0xfd99a917, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad99a91c, SUC_lexicon_tags_33 },
    { 0x2699a91d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c99a91f, SUC_lexicon_tags_11 },
    { 0xfe79a920, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f9a92a, SUC_lexicon_tags_92 },
    { 0x9d59a92b, SUC_lexicon_tags_17 },
    { 0x6939a92c, SUC_lexicon_tags_47 },
    { 0x8239a92b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b9a931, SUC_lexicon_tags_33 },
    { 0x3b19a931, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb9a935, SUC_lexicon_tags_21 },
    { 0xcc39a936, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf0f9a938, SUC_lexicon_tags_9 },
    { 0x60d9a939, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x7399a93b, SUC_lexicon_tags_32 },
    { 0x8cb9a93c, SUC_lexicon_tags_47 },
    { 0xb579a93d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a940, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d9a945, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ed9a947, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb019a94e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ed9a95b, SUC_lexicon_tags_6 },
    { 0xe279a95c, SUC_lexicon_tags_21 },
    { 0x6ab9a95c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b59a960, SUC_lexicon_tags_11 },
    { 0xfd19a961, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4f9a966, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c19a96a, SUC_lexicon_tags_7 },
    { 0xef99a96b, SUC_lexicon_tags_11 },
    { 0xb499a96a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5379a973, SUC_lexicon_tags_14 },
    { 0x6359a973, SUC_lexicon_tags_79 },
    { 0x21f9a974, SUC_lexicon_tags_77 },
    { 0x84b9a975, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b9a97b, SUC_lexicon_tags_7 },
    { 0x40f9a97b, SUC_lexicon_tags_6 },
    { 0xcb19a97c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2519a981, SUC_lexicon_tags_15 },
    { 0x2679a981, SUC_lexicon_tags_47 },
    { 0x90f9a983, SUC_lexicon_tags_85 },
    { 0x11b9a984, SUC_lexicon_tags_25 },
    { 0xba19a982, SUC_lexicon_tags_2 },
    { 0x7c39a986, SUC_lexicon_tags_12 },
    { 0x62b9a987, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d9a98b, SUC_lexicon_tags_47 },
    { 0x3b59a98c, SUC_lexicon_tags_7 },
    { 0xc059a98d, SUC_lexicon_tags_14 },
    { 0x9159a98e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7c99a990, SUC_lexicon_tags_37 },
    { 0xf579a990, SUC_lexicon_tags_36 },
    { 0x6379a992, SUC_lexicon_tags_25 },
    { 0x4df9a993, SUC_lexicon_tags_0 },
    { 0x7ed9a993, SUC_lexicon_tags_17 },
    { 0x1219a995, SUC_lexicon_tags_26 },
    { 0x6099a996, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfdd9a998, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4f9a99a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f9a99f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a9a3, SUC_lexicon_tags_37 },
    { 0x4379a9a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd9a9a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa099a9ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x19b9a9ae, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef9a9b1, SUC_lexicon_tags_45 },
    { 0xfb99a9b2, SUC_lexicon_tags_33 },
    { 0xb5f9a9b3, SUC_lexicon_tags_12 },
    { 0xeb19a9b4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd579a9bc, SUC_lexicon_tags_37 },
    { 0x6c79a9bd, SUC_lexicon_tags_30 },
    { 0x6959a9be, SUC_lexicon_tags_7 },
    { 0x5559a9bf, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb79a9c2, SUC_lexicon_tags_24 },
    { 0xd019a9c2, SUC_lexicon_tags_43 },
    { 0xe859a9c4, SUC_lexicon_tags_3 },
    { 0x1899a9c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179a9c9, SUC_lexicon_tags_1 },
    { 0xb3f9a9ca, SUC_lexicon_tags_37 },
    { 0x52b9a9c9, SUC_lexicon_tags_26 },
    { 0x7599a9ca, SUC_lexicon_tags_14 },
    { 0xf379a9cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd139a9cf, SUC_lexicon_tags_38 },
    { 0x8e99a9d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc39a9d6, SUC_lexicon_tags_25 },
    { 0xb0d9a9d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c39a9d9, SUC_lexicon_tags_11 },
    { 0xa0d9a9d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb79a9df, SUC_lexicon_tags_23 },
    { 0xfd79a9e0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf279a9e3, SUC_lexicon_tags_31 },
    { 0xf4d9a9e3, SUC_lexicon_tags_21 },
    { 0x1579a9e3, SUC_lexicon_tags_37 },
    { 0x7619a9e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7879a9e8, SUC_lexicon_tags_17 },
    { 0x3199a9e9, SUC_lexicon_tags_35 },
    { 0x7f19a9ea, SUC_lexicon_tags_12 },
    { 0x1e19a9eb, SUC_lexicon_tags_29 },
    { 0xd659a9ec, SUC_lexicon_tags_12 },
    { 0x8899a9ec, SUC_lexicon_tags_9 },
    { 0xfa79a9ee, SUC_lexicon_tags_15 },
    { 0x43f9a9ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b9a9f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9759a9f8, SUC_lexicon_tags_37 },
    { 0x7e39a9f9, SUC_lexicon_tags_23 },
    { 0x3dd9a9fa, SUC_lexicon_tags_1 },
    { 0x36d9a9fb, SUC_lexicon_tags_17 },
    { 0x3959a9fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5f9a9fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd39aa03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d59aa0c, SUC_lexicon_tags_11 },
    { 0xccf9aa0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf79aa12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf059aa16, SUC_lexicon_tags_23 },
    { 0x98f9aa17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b9aa1b, SUC_lexicon_tags_25 },
    { 0x91b9aa1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x16b9aa1e, SUC_lexicon_tags_55 },
    { 0xd319aa1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a59aa21, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xebd9aa23, SUC_lexicon_tags_32 },
    { 0x11f9aa23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c39aa27, SUC_lexicon_tags_39 },
    { 0x2599aa27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b19aa2e, SUC_lexicon_tags_37 },
    { 0x1b19aa2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1fb9aa31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5fb9aa33, SUC_lexicon_tags_6 },
    { 0xc4f9aa34, SUC_lexicon_tags_63 },
    { 0x5cb9aa35, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f9aa3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf59aa3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d79aa41, SUC_lexicon_tags_47 },
    { 0x3d19aa42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f9aa47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdbf9aa49, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef9aa4f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f9aa54, SUC_lexicon_tags_48 },
    { 0xe719aa55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7719aa57, SUC_lexicon_tags_23 },
    { 0x6fd9aa58, SUC_lexicon_tags_27 },
    { 0x2a99aa57, SUC_lexicon_tags_17 },
    { 0x5879aa5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3d9aa5c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd9aa60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48b9aa62, SUC_lexicon_tags_48 },
    { 0x18b9aa63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c99aa6a, SUC_lexicon_tags_34 },
    { 0xdcf9aa6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f9aa70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4459aa74, SUC_lexicon_tags_47 },
    { 0x119aa74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f9aa7c, SUC_lexicon_tags_15 },
    { 0xc8d9aa7d, SUC_lexicon_tags_17 },
    { 0xea79aa7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa139aa81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6339aa83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6719aa85, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed9aa8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3779aa8d, SUC_lexicon_tags_25 },
    { 0x6d19aa8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd519aa91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd79aa94, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe279aa99, SUC_lexicon_tags_17 },
    { 0xefb9aa99, SUC_lexicon_tags_17 },
    { 0x6b79aa99, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1059aaa2, SUC_lexicon_tags_49 },
    { 0xffd9aaa3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d9aaa6, SUC_lexicon_tags_19 },
    { 0xa8f9aaa6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f99aaac, SUC_lexicon_tags_27 },
    { 0x30f9aaad, SUC_lexicon_tags_48 },
    { 0xb5f9aaac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d9aab6, SUC_lexicon_tags_4 },
    { 0x6079aab7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbd39aab9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xda79aabc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb179aabe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd6b9aac0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e99aac3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b19aac8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x6259aacb, SUC_lexicon_tags_47 },
    { 0x86d9aacb, SUC_lexicon_tags_37 },
    { 0x7739aacb, SUC_lexicon_tags_12 },
    { 0x5ef9aacc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb59aad5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b19aae0, SUC_lexicon_tags_18 },
    { 0x199aae1, SUC_lexicon_tags_2 },
    { 0x2a59aae2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf299aae5, SUC_lexicon_tags_17 },
    { 0x64d9aae6, SUC_lexicon_tags_49 },
    { 0x85b9aae5, SUC_lexicon_tags_30 },
    { 0x779aae8, SUC_lexicon_tags_15 },
    { 0x5f9aae8, SUC_lexicon_tags_2 },
    { 0xc359aaea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff9aaf0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcf39aaf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1759aaf4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xedb9aaf6, SUC_lexicon_tags_1 },
    { 0xe019aaf7, SUC_lexicon_tags_28 },
    { 0x2b39aaf7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b9aafb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf9aafe, SUC_lexicon_tags_27 },
    { 0x9999aaff, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa4b9ab01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1259ab08, SUC_lexicon_tags_63 },
    { 0x39b9ab09, SUC_lexicon_tags_160 },
    { 0x7eb9ab0a, SUC_lexicon_tags_2 },
    { 0x2779ab0a, SUC_lexicon_tags_2 },
    { 0xb539ab0b, SUC_lexicon_tags_14 },
    { 0x37b9ab0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b9ab13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6259ab16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9dd9ab18, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfed9ab1a, SUC_lexicon_tags_11 },
    { 0x2359ab1a, SUC_lexicon_tags_11 },
    { 0x4419ab1c, SUC_lexicon_tags_12 },
    { 0x38d9ab1c, SUC_lexicon_tags_37 },
    { 0xca59ab1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc19ab25, SUC_lexicon_tags_0 },
    { 0xfef9ab25, SUC_lexicon_tags_17 },
    { 0x98d9ab27, SUC_lexicon_tags_12 },
    { 0x8df9ab27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7279ab2b, SUC_lexicon_tags_27 },
    { 0x94b9ab2c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6519ab2e, SUC_lexicon_tags_30 },
    { 0x1739ab2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59f9ab31, SUC_lexicon_tags_14 },
    { 0xad99ab32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc4b9ab34, SUC_lexicon_tags_31 },
    { 0x84b9ab34, SUC_lexicon_tags_27 },
    { 0x74f9ab36, SUC_lexicon_tags_11 },
    { 0x5e79ab37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe459ab3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f9ab3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd9ab42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4799ab46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27b9ab4b, SUC_lexicon_tags_30 },
    { 0xa5b9ab4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd9ab4f, SUC_lexicon_tags_25 },
    { 0x36b9ab4f, SUC_lexicon_tags_40 },
    { 0x1c99ab4f, SUC_lexicon_tags_11 },
    { 0x4c19ab52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b9ab56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4439ab58, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4a79ab5a, SUC_lexicon_tags_43 },
    { 0xcdf9ab5a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a19ab61, SUC_lexicon_tags_7 },
    { 0x7a79ab61, SUC_lexicon_tags_60 },
    { 0xba79ab63, SUC_lexicon_tags_19 },
    { 0x9bf9ab63, SUC_lexicon_tags_14 },
    { 0x8719ab63, SUC_lexicon_tags_79 },
    { 0xc5d9ab62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3219ab6b, SUC_lexicon_tags_37 },
    { 0xc4f9ab6c, SUC_lexicon_tags_13 },
    { 0x2219ab6d, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x4619ab6f, SUC_lexicon_tags_15 },
    { 0x6ad9ab70, SUC_lexicon_tags_17 },
    { 0xa519ab71, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5af9ab73, SUC_lexicon_tags_14 },
    { 0xfad9ab73, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xde59ab76, SUC_lexicon_tags_25 },
    { 0x7439ab77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f99ab79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed9ab7e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1799ab80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c99ab83, SUC_lexicon_tags_3 },
    { 0xeb9ab84, SUC_lexicon_tags_56 },
    { 0x22d9ab84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f59ab87, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36b9ab93, SUC_lexicon_tags_12 },
    { 0x6c59ab93, SUC_lexicon_tags_34 },
    { 0x8e99ab93, SUC_lexicon_tags_33 },
    { 0xe719ab96, SUC_lexicon_tags_124 },
    { 0x9b59ab96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39ab9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4659ab9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32b9abad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdcf9abaf, SUC_lexicon_tags_38 },
    { 0x1d79abb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd479abb2, SUC_lexicon_tags_1 },
    { 0xee99abb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e99abb9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef9abbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf9abc2, SUC_lexicon_tags_29 },
    { 0x7919abc3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8799abc7, SUC_lexicon_tags_27 },
    { 0x6839abc8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c79abcc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59abd4, SUC_lexicon_tags_1 },
    { 0xe859abd5, SUC_lexicon_tags_11 },
    { 0x3c59abd5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd519abd9, SUC_lexicon_tags_47 },
    { 0x3559abda, SUC_lexicon_tags_14 },
    { 0x68d9abd9, SUC_lexicon_tags_25 },
    { 0x6f9abdc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2099abe0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbf9abe2, SUC_lexicon_tags_114 },
    { 0x8179abe2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x68f9abe5, SUC_lexicon_tags_0 },
    { 0xbbf9abe6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdc39abe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb9abed, SUC_lexicon_tags_11 },
    { 0x719abee, SUC_lexicon_tags_2 },
    { 0x62d9abef, SUC_lexicon_tags_45 },
    { 0xb559abee, SUC_lexicon_tags_33 },
    { 0x8139abf0, SUC_lexicon_tags_1 },
    { 0x29f9abf0, SUC_lexicon_tags_17 },
    { 0xb4b9abf1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9919abf7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7199abfa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b9ac05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf9ac07, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9ac0b, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d9ac0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3819ac12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15d9ac14, SUC_lexicon_tags_15 },
    { 0x3c39ac14, SUC_lexicon_tags_1 },
    { 0x9af9ac16, SUC_lexicon_tags_56 },
    { 0xe7d9ac16, SUC_lexicon_tags_23 },
    { 0x2579ac18, SUC_lexicon_tags_47 },
    { 0x5d99ac16, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x61f9ac1c, SUC_lexicon_tags_37 },
    { 0xfc39ac1c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b19ac2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47f9ac2c, SUC_lexicon_tags_2 },
    { 0xa239ac2c, SUC_lexicon_tags_2 },
    { 0xcb79ac2e, SUC_lexicon_tags_2 },
    { 0x1d59ac2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabb9ac32, SUC_lexicon_tags_2 },
    { 0x1579ac33, SUC_lexicon_tags_17 },
    { 0x9379ac34, SUC_lexicon_tags_2 },
    { 0xd919ac35, SUC_lexicon_tags_10 },
    { 0xbcd9ac33, SUC_lexicon_tags_11 },
    { 0x5eb9ac37, SUC_lexicon_tags_7 },
    { 0xe4f9ac37, SUC_lexicon_tags_26 },
    { 0x1bf9ac37, SUC_lexicon_tags_11 },
    { 0x3e19ac3a, SUC_lexicon_tags_11 },
    { 0xa839ac3b, SUC_lexicon_tags_11 },
    { 0x4259ac37, SUC_lexicon_tags_11 },
    { 0xb919ac3d, SUC_lexicon_tags_7 },
    { 0x679ac3d, SUC_lexicon_tags_15 },
    { 0xf499ac3f, SUC_lexicon_tags_10 },
    { 0x6e59ac39, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xbf59ac42, SUC_lexicon_tags_54 },
    { 0xfcb9ac43, SUC_lexicon_tags_27 },
    { 0xf579ac44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7119ac47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7bf9ac49, SUC_lexicon_tags_33 },
    { 0x67d9ac4a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f9ac4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79ac53, SUC_lexicon_tags_26 },
    { 0x339ac54, SUC_lexicon_tags_14 },
    { 0xae19ac53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf9ac59, SUC_lexicon_tags_2 },
    { 0xef99ac59, SUC_lexicon_tags_21 },
    { 0x9e59ac5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd9ac5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xce99ac61, SUC_lexicon_tags_1 },
    { 0xaa79ac61, SUC_lexicon_tags_2 },
    { 0x3e19ac63, SUC_lexicon_tags_56 },
    { 0xab9ac64, SUC_lexicon_tags_23 },
    { 0xca19ac62, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x199ac67, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3b59ac69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b99ac6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3219ac71, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xea59ac73, SUC_lexicon_tags_6 },
    { 0xa8f9ac73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4959ac78, SUC_lexicon_tags_1 },
    { 0xc79ac78, SUC_lexicon_tags_12 },
    { 0x1079ac7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6979ac7c, SUC_lexicon_tags_25 },
    { 0xcc39ac7d, SUC_lexicon_tags_27 },
    { 0xd1f9ac7d, SUC_lexicon_tags_25 },
    { 0x73d9ac7f, SUC_lexicon_tags_25 },
    { 0xe199ac7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc819ac87, SUC_lexicon_tags_34 },
    { 0xfbf9ac87, SUC_lexicon_tags_14 },
    { 0x8ed9ac89, SUC_lexicon_tags_25 },
    { 0xd6d9ac89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x299ac8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8899ac94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d39ac98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb79ac9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3579ac9d, SUC_lexicon_tags_17 },
    { 0x6119ac9e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50d9aca5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfed9aca8, SUC_lexicon_tags_49 },
    { 0x359aca8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x55f9acab, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d9acae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a99acb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f9acb6, SUC_lexicon_tags_45 },
    { 0xf59acb7, SUC_lexicon_tags_45 },
    { 0xa99acb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb559acbb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f99acbf, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xb819acc2, SUC_lexicon_tags_8 },
    { 0x12f9acc3, SUC_lexicon_tags_14 },
    { 0x9a79acc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa319acca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbf39accc, SUC_lexicon_tags_44 },
    { 0xd659accd, SUC_lexicon_tags_12 },
    { 0x37f9acce, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb19acd3, SUC_lexicon_tags_23 },
    { 0x8a59acd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf939acd7, SUC_lexicon_tags_26 },
    { 0x5979acd7, SUC_lexicon_tags_8 },
    { 0x8a19acd9, SUC_lexicon_tags_65 },
    { 0x2019acda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d9acde, SUC_lexicon_tags_10 },
    { 0xe8b9acde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42b9ace2, SUC_lexicon_tags_7 },
    { 0x3479ace3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9159ace6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf19ace8, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xb079acea, SUC_lexicon_tags_27 },
    { 0xc219aceb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa079acf1, SUC_lexicon_tags_30 },
    { 0x3fd9acf2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca39acf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59acfb, SUC_lexicon_tags_14 },
    { 0xe6b9acfb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x90b9acff, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f9ad02, SUC_lexicon_tags_14 },
    { 0x1ff9ad03, SUC_lexicon_tags_12 },
    { 0xdf9ad04, SUC_lexicon_tags_17 },
    { 0xe999ad04, SUC_lexicon_tags_19 },
    { 0xc739ad06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa39ad0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d9ad12, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d9ad15, SUC_lexicon_tags_37 },
    { 0x8059ad16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8139ad18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a39ad1a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d19ad1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f9ad21, SUC_lexicon_tags_31 },
    { 0x1c39ad21, SUC_lexicon_tags_4 },
    { 0x9559ad23, SUC_lexicon_tags_1 },
    { 0x9759ad24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b99ad26, SUC_lexicon_tags_69 },
    { 0x7cd9ad27, SUC_lexicon_tags_12 },
    { 0xf79ad26, SUC_lexicon_tags_11 },
    { 0xfd19ad29, SUC_lexicon_tags_33 },
    { 0x1b19ad29, SUC_lexicon_tags_25 },
    { 0xc539ad2b, SUC_lexicon_tags_11 },
    { 0xc639ad29, SUC_lexicon_tags_91 },
    { 0xd019ad2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b9ad31, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x43f9ad33, SUC_lexicon_tags_37 },
    { 0x5eb9ad34, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x1a99ad36, SUC_lexicon_tags_14 },
    { 0x8f9ad36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa59ad3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd9ad42, SUC_lexicon_tags_37 },
    { 0xc0d9ad43, SUC_lexicon_tags_12 },
    { 0x51f9ad44, SUC_lexicon_tags_11 },
    { 0x6599ad45, SUC_lexicon_tags_11 },
    { 0xc679ad46, SUC_lexicon_tags_15 },
    { 0xaf79ad47, SUC_lexicon_tags_65 },
    { 0x7e99ad46, SUC_lexicon_tags_12 },
    { 0xc219ad42, SUC_lexicon_tags_12 },
    { 0x3e99ad44, SUC_lexicon_tags_19 },
    { 0xa079ad47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6259ad4d, SUC_lexicon_tags_2 },
    { 0x23b9ad4d, SUC_lexicon_tags_32 },
    { 0x80d9ad4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8f9ad51, SUC_lexicon_tags_12 },
    { 0x5439ad51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb9ad54, SUC_lexicon_tags_23 },
    { 0x6f99ad55, SUC_lexicon_tags_11 },
    { 0x27f9ad54, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xf399ad59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd8b9ad5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7d19ad5d, SUC_lexicon_tags_81 },
    { 0xdb99ad5e, SUC_lexicon_tags_7 },
    { 0x7f79ad5e, SUC_lexicon_tags_4 },
    { 0x7e99ad5f, SUC_lexicon_tags_7 },
    { 0xfc79ad61, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab9ad64, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9f99ad66, SUC_lexicon_tags_17 },
    { 0xb879ad67, SUC_lexicon_tags_17 },
    { 0x19ad68, SUC_lexicon_tags_4 },
    { 0x3e99ad66, SUC_lexicon_tags_11 },
    { 0xb7d9ad6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaab9ad6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5899ad76, SUC_lexicon_tags_15 },
    { 0xc5f9ad77, SUC_lexicon_tags_2 },
    { 0x88f9ad76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8d9ad81, SUC_lexicon_tags_8 },
    { 0xb319ad82, SUC_lexicon_tags_14 },
    { 0x1a39ad82, SUC_lexicon_tags_37 },
    { 0xed19ad84, SUC_lexicon_tags_63 },
    { 0xc979ad85, SUC_lexicon_tags_45 },
    { 0x6b19ad85, SUC_lexicon_tags_7 },
    { 0x7b79ad87, SUC_lexicon_tags_14 },
    { 0x1c59ad84, SUC_lexicon_tags_37 },
    { 0xb399ad88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b9ad8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd9ad8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd19ad91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c59ad97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e19ad9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab9ad9f, SUC_lexicon_tags_22 },
    { 0x28f9ada0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x40d9ada2, SUC_lexicon_tags_1 },
    { 0x239ada3, SUC_lexicon_tags_31 },
    { 0x84b9ada4, SUC_lexicon_tags_47 },
    { 0xc7d9ada5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee19adad, SUC_lexicon_tags_47 },
    { 0x3959adae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x259adb1, SUC_lexicon_tags_49 },
    { 0x6d9adb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa799adb4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5959adb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3279adba, SUC_lexicon_tags_47 },
    { 0x3bb9adbb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd819adbe, SUC_lexicon_tags_25 },
    { 0x3d9adbf, SUC_lexicon_tags_1 },
    { 0x6919adbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f19adc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1439adca, SUC_lexicon_tags_39 },
    { 0x33f9adcb, SUC_lexicon_tags_33 },
    { 0xd39adca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9f9adce, SUC_lexicon_tags_17 },
    { 0xfe59adce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6d9add1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd99add3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaed9add9, SUC_lexicon_tags_55 },
    { 0xa1d9adda, SUC_lexicon_tags_0 },
    { 0xfeb9adda, SUC_lexicon_tags_103 },
    { 0x3df9addb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d9ade5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x37b9ade7, SUC_lexicon_tags_17 },
    { 0xda59ade7, SUC_lexicon_tags_65 },
    { 0xe5f9ade9, SUC_lexicon_tags_10 },
    { 0x11f9ade9, SUC_lexicon_tags_27 },
    { 0xced9adeb, SUC_lexicon_tags_37 },
    { 0x7359adec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df9adf0, SUC_lexicon_tags_7 },
    { 0xdc99adf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bf9adf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e19adf5, SUC_lexicon_tags_42 },
    { 0x719adf6, SUC_lexicon_tags_13 },
    { 0xbd59adf7, SUC_lexicon_tags_0 },
    { 0x6179adf8, SUC_lexicon_tags_10 },
    { 0x47b9adf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5359adfc, SUC_lexicon_tags_27 },
    { 0xddf9adfd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f9ae00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ff9ae02, SUC_lexicon_tags_19 },
    { 0x46d9ae03, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf319ae05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf019ae0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe119ae12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7999ae14, SUC_lexicon_tags_14 },
    { 0xab59ae15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb9ae1d, SUC_lexicon_tags_33 },
    { 0x72f9ae1d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd6d9ae20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb9ae27, SUC_lexicon_tags_14 },
    { 0xc3f9ae28, SUC_lexicon_tags_32 },
    { 0xac19ae29, SUC_lexicon_tags_21 },
    { 0x2f99ae29, SUC_lexicon_tags_9 },
    { 0x8419ae2b, SUC_lexicon_tags_1 },
    { 0x4079ae2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9ae2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4859ae35, SUC_lexicon_tags_12 },
    { 0xd539ae35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699ae3a, SUC_lexicon_tags_37 },
    { 0xaf59ae3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b39ae3d, SUC_lexicon_tags_43 },
    { 0x5dd9ae3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2539ae40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac59ae43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae59ae48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc99ae4a, SUC_lexicon_tags_15 },
    { 0x8419ae4b, SUC_lexicon_tags_15 },
    { 0x1739ae4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb319ae4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5159ae50, SUC_lexicon_tags_30 },
    { 0xad99ae51, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f59ae54, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb399ae5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa079ae63, SUC_lexicon_tags_8 },
    { 0x2fd9ae64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd399ae69, SUC_lexicon_tags_14 },
    { 0x6b9ae69, SUC_lexicon_tags_66 },
    { 0x4d59ae6a, SUC_lexicon_tags_37 },
    { 0x8f59ae6c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9ae70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d9ae74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3499ae77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1679ae7a, SUC_lexicon_tags_47 },
    { 0xbf99ae7a, SUC_lexicon_tags_2 },
    { 0x84d9ae7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xde99ae7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c39ae82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd919ae86, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x57f9ae88, SUC_lexicon_tags_11 },
    { 0x79ae89, SUC_lexicon_tags_14 },
    { 0x7e19ae8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9399ae8c, SUC_lexicon_tags_11 },
    { 0x82b9ae8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9259ae98, SUC_lexicon_tags_14 },
    { 0x2399ae98, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xda59ae9c, SUC_lexicon_tags_8 },
    { 0x859ae9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb9aea9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaaf9aeab, SUC_lexicon_tags_1 },
    { 0x8d39aeac, SUC_lexicon_tags_2 },
    { 0xbc39aeab, SUC_lexicon_tags_28 },
    { 0x9659aeac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d79aeb4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe399aeb7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca79aebb, SUC_lexicon_tags_7 },
    { 0x69f9aebc, SUC_lexicon_tags_15 },
    { 0x1ed9aebb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe959aec3, SUC_lexicon_tags_2 },
    { 0x6159aec4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd559aec6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9399aecf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab9aed4, SUC_lexicon_tags_25 },
    { 0x49f9aed5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x219aed7, SUC_lexicon_tags_15 },
    { 0x6719aed7, SUC_lexicon_tags_17 },
    { 0xd019aed8, SUC_lexicon_tags_45 },
    { 0x639aed7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x32d9aedd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4759aedf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb9aee6, SUC_lexicon_tags_15 },
    { 0x1059aee6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa4b9aee9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99aeed, SUC_lexicon_tags_29 },
    { 0x4399aeee, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2859aef0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8959aef3, SUC_lexicon_tags_1 },
    { 0x3a79aef3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7b9aef6, SUC_lexicon_tags_16 },
    { 0x9219aef7, SUC_lexicon_tags_37 },
    { 0xc659aef6, SUC_lexicon_tags_23 },
    { 0xabd9aef6, SUC_lexicon_tags_63 },
    { 0x5c59aef8, SUC_lexicon_tags_11 },
    { 0x8ab9aefb, SUC_lexicon_tags_15 },
    { 0x20d9aefc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4539aefe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf9f9af00, SUC_lexicon_tags_11 },
    { 0xc759af01, SUC_lexicon_tags_15 },
    { 0x17f9af02, SUC_lexicon_tags_10 },
    { 0x3db9af03, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb619af05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef9af0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdab9af0c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1839af0e, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x4139af10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3759af14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5af9af16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d79af1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe39af1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb59af20, SUC_lexicon_tags_37 },
    { 0xbf59af20, SUC_lexicon_tags_28 },
    { 0x89d9af20, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x38b9af24, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f59af2b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef79af30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d9af33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d79af36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a99af39, SUC_lexicon_tags_11 },
    { 0x4279af39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2039af3d, SUC_lexicon_tags_25 },
    { 0x6359af3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa159af40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf619af43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e19af4a, SUC_lexicon_tags_10 },
    { 0xb759af4b, SUC_lexicon_tags_18 },
    { 0x8679af4a, SUC_lexicon_tags_63 },
    { 0x4639af4d, SUC_lexicon_tags_23 },
    { 0x62f9af4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc79af52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3879af55, SUC_lexicon_tags_17 },
    { 0x119af56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd99af58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9af5c, SUC_lexicon_tags_23 },
    { 0xc7b9af5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe319af5f, SUC_lexicon_tags_43 },
    { 0x9b79af60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b9af6a, SUC_lexicon_tags_34 },
    { 0xd639af6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3279af6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8d9af71, SUC_lexicon_tags_27 },
    { 0xce59af72, SUC_lexicon_tags_2 },
    { 0x5ff9af72, SUC_lexicon_tags_25 },
    { 0x9479af73, SUC_lexicon_tags_14 },
    { 0x3b99af71, SUC_lexicon_tags_12 },
    { 0x5f99af76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc539af79, SUC_lexicon_tags_29 },
    { 0x1c39af7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb9af7f, SUC_lexicon_tags_25 },
    { 0xce19af80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e19af82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d79af8b, SUC_lexicon_tags_37 },
    { 0x8a99af8c, SUC_lexicon_tags_33 },
    { 0x1319af8b, SUC_lexicon_tags_38 },
    { 0x79af8c, SUC_lexicon_tags_25 },
    { 0x6739af8f, SUC_lexicon_tags_0 },
    { 0xc0f9af90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc879af93, SUC_lexicon_tags_21 },
    { 0xd599af93, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2139af97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6b9af99, SUC_lexicon_tags_7 },
    { 0xfb99af9a, SUC_lexicon_tags_60 },
    { 0x4339af9b, SUC_lexicon_tags_17 },
    { 0xaf39af9c, SUC_lexicon_tags_10 },
    { 0xdd99af9d, SUC_lexicon_tags_439 },
    { 0x7f79af9e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xb799afa1, SUC_lexicon_tags_37 },
    { 0xeb19afa2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb579afa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23d9afa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf579afa9, SUC_lexicon_tags_15 },
    { 0x8979afa9, SUC_lexicon_tags_1 },
    { 0x7d19afaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd39afb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc659afb3, SUC_lexicon_tags_19 },
    { 0x8a39afb3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaf39afb6, SUC_lexicon_tags_37 },
    { 0xaf9afb7, SUC_lexicon_tags_25 },
    { 0xac19afb8, SUC_lexicon_tags_44 },
    { 0x5159afb9, SUC_lexicon_tags_49 },
    { 0x7b9afba, SUC_lexicon_tags_2 },
    { 0x7b79afba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f9afbe, SUC_lexicon_tags_415 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d39afc2, SUC_lexicon_tags_4 },
    { 0xb39afc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1859afc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9139afca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb9afce, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6919afd0, SUC_lexicon_tags_60 },
    { 0xaeb9afd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b39afd3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f9afda, SUC_lexicon_tags_17 },
    { 0xd279afdb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3319afde, SUC_lexicon_tags_15 },
    { 0x95f9afdf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f9afe2, SUC_lexicon_tags_27 },
    { 0xbe99afe2, SUC_lexicon_tags_12 },
    { 0x49b9afe4, SUC_lexicon_tags_14 },
    { 0xc239afe2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6939afe7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4279afea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe119afef, SUC_lexicon_tags_17 },
    { 0xe3b9aff0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd9aff3, SUC_lexicon_tags_27 },
    { 0x1e99aff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9aff9, SUC_lexicon_tags_38 },
    { 0xce59affa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f9affd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba99b003, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b9b007, SUC_lexicon_tags_28 },
    { 0xf819b008, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd759b00e, SUC_lexicon_tags_11 },
    { 0x4099b00e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8699b013, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4b9b017, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x95d9b019, SUC_lexicon_tags_0 },
    { 0xfe19b019, SUC_lexicon_tags_29 },
    { 0x5239b019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf19b01d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2b9b01f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b9b022, SUC_lexicon_tags_33 },
    { 0x4e39b023, SUC_lexicon_tags_29 },
    { 0xa119b024, SUC_lexicon_tags_37 },
    { 0x35f9b025, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af9b02a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3399b032, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3579b035, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd9b03d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4b39b03f, SUC_lexicon_tags_29 },
    { 0xcb79b040, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf9b046, SUC_lexicon_tags_11 },
    { 0x6539b047, SUC_lexicon_tags_87 },
    { 0xbdb9b047, SUC_lexicon_tags_16 },
    { 0xe59b047, SUC_lexicon_tags_63 },
    { 0x8ad9b04a, SUC_lexicon_tags_7 },
    { 0x8959b048, SUC_lexicon_tags_28 },
    { 0x6ed9b04c, SUC_lexicon_tags_2 },
    { 0xc779b04d, SUC_lexicon_tags_6 },
    { 0x6459b04e, SUC_lexicon_tags_1 },
    { 0xa879b04f, SUC_lexicon_tags_60 },
    { 0x8ed9b04e, SUC_lexicon_tags_60 },
    { 0xe9d9b051, SUC_lexicon_tags_38 },
    { 0xdf39b052, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3119b059, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf59b05b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3059b060, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3479b064, SUC_lexicon_tags_17 },
    { 0x8a79b065, SUC_lexicon_tags_25 },
    { 0xb199b066, SUC_lexicon_tags_47 },
    { 0x7959b065, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16f9b06c, SUC_lexicon_tags_23 },
    { 0x319b06d, SUC_lexicon_tags_11 },
    { 0xab39b06d, SUC_lexicon_tags_37 },
    { 0x9d59b06f, SUC_lexicon_tags_43 },
    { 0xb3f9b070, SUC_lexicon_tags_19 },
    { 0xe299b071, SUC_lexicon_tags_1 },
    { 0xc5d9b072, SUC_lexicon_tags_17 },
    { 0x4e59b072, SUC_lexicon_tags_12 },
    { 0x2639b06c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9b39b076, SUC_lexicon_tags_17 },
    { 0x6c59b077, SUC_lexicon_tags_16 },
    { 0x3ef9b078, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bd9b07a, SUC_lexicon_tags_1 },
    { 0xcad9b07b, SUC_lexicon_tags_33 },
    { 0x86b9b07b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb9b07f, SUC_lexicon_tags_7 },
    { 0xe1b9b080, SUC_lexicon_tags_2 },
    { 0x7899b080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb559b084, SUC_lexicon_tags_6 },
    { 0xbd79b085, SUC_lexicon_tags_30 },
    { 0x38b9b086, SUC_lexicon_tags_27 },
    { 0x7ab9b086, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6739b094, SUC_lexicon_tags_60 },
    { 0x6d19b094, SUC_lexicon_tags_17 },
    { 0x4279b096, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9f9b098, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3039b09a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5659b09c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe359b0a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f9b0a5, SUC_lexicon_tags_15 },
    { 0x6579b0a5, SUC_lexicon_tags_66 },
    { 0x9db9b0a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3439b0aa, SUC_lexicon_tags_37 },
    { 0x26d9b0aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f9b0ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f19b0af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df9b0b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939b0bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf879b0c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x10b9b0c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf59b0c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea39b0ca, SUC_lexicon_tags_27 },
    { 0x9e39b0cb, SUC_lexicon_tags_37 },
    { 0xd659b0ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcfb9b0ce, SUC_lexicon_tags_11 },
    { 0xe0d9b0cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb519b0d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d9b0d3, SUC_lexicon_tags_65 },
    { 0xb739b0d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c99b0d6, SUC_lexicon_tags_25 },
    { 0x8d9b0d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2919b0d9, SUC_lexicon_tags_2 },
    { 0x80d9b0da, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf379b0df, SUC_lexicon_tags_0 },
    { 0x4df9b0e0, SUC_lexicon_tags_17 },
    { 0x5099b0e0, SUC_lexicon_tags_33 },
    { 0x1d79b0e2, SUC_lexicon_tags_17 },
    { 0xab9b0e3, SUC_lexicon_tags_63 },
    { 0x8ff9b0e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb9b0e8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa799b0ed, SUC_lexicon_tags_160 },
    { 0x2c79b0ee, SUC_lexicon_tags_12 },
    { 0xe079b0ee, SUC_lexicon_tags_7 },
    { 0xcd99b0ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x58b9b0f2, SUC_lexicon_tags_31 },
    { 0xf179b0f2, SUC_lexicon_tags_4 },
    { 0x9e99b0f4, SUC_lexicon_tags_25 },
    { 0x49b9b0f3, SUC_lexicon_tags_12 },
    { 0xad79b0f6, SUC_lexicon_tags_1 },
    { 0x7db9b0f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b9b101, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2539b107, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f39b10e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2f9b112, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5539b117, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe539b11a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x12f9b11c, SUC_lexicon_tags_33 },
    { 0xbfb9b11d, SUC_lexicon_tags_7 },
    { 0xd2b9b11c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1639b120, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe919b123, SUC_lexicon_tags_37 },
    { 0x85b9b123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26b9b129, SUC_lexicon_tags_12 },
    { 0xb0b9b129, SUC_lexicon_tags_38 },
    { 0xd499b12b, SUC_lexicon_tags_49 },
    { 0xe179b12c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2059b12e, SUC_lexicon_tags_7 },
    { 0x2a99b12e, SUC_lexicon_tags_27 },
    { 0xb579b130, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd9b134, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd79b13c, SUC_lexicon_tags_7 },
    { 0x7419b13d, SUC_lexicon_tags_37 },
    { 0xb619b13d, SUC_lexicon_tags_11 },
    { 0x71d9b13e, SUC_lexicon_tags_13 },
    { 0xc3d9b13f, SUC_lexicon_tags_17 },
    { 0x9139b13d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28f9b143, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xee19b145, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xb199b147, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfb59b149, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b19b14d, SUC_lexicon_tags_1 },
    { 0x26d9b14e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f9b151, SUC_lexicon_tags_25 },
    { 0xdcd9b151, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xebf9b154, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d99b159, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa19b15c, SUC_lexicon_tags_37 },
    { 0xc239b15d, SUC_lexicon_tags_25 },
    { 0xe959b15d, SUC_lexicon_tags_117 },
    { 0x7179b15f, SUC_lexicon_tags_55 },
    { 0xd139b15c, SUC_lexicon_tags_16 },
    { 0x5f19b161, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd99b164, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf999b16a, SUC_lexicon_tags_6 },
    { 0xea39b16b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad39b16d, SUC_lexicon_tags_27 },
    { 0xc1f9b16e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe359b171, SUC_lexicon_tags_21 },
    { 0x34d9b172, SUC_lexicon_tags_1 },
    { 0xb259b172, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8659b177, SUC_lexicon_tags_63 },
    { 0x9c59b177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa079b17a, SUC_lexicon_tags_17 },
    { 0xf539b17b, SUC_lexicon_tags_21 },
    { 0xe1b9b17a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaf9b181, SUC_lexicon_tags_10 },
    { 0xe7f9b181, SUC_lexicon_tags_34 },
    { 0x60b9b181, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b9b186, SUC_lexicon_tags_43 },
    { 0xc0d9b187, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7859b18d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9819b18f, SUC_lexicon_tags_40 },
    { 0x9039b18f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xabd9b193, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b19b195, SUC_lexicon_tags_0 },
    { 0x2019b196, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb9b199, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4979b19f, SUC_lexicon_tags_15 },
    { 0x3819b19f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd99b1a4, SUC_lexicon_tags_43 },
    { 0x9139b1a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x899b1a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf559b1ab, SUC_lexicon_tags_38 },
    { 0xfe19b1ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9539b1ae, SUC_lexicon_tags_8 },
    { 0x12d9b1af, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf79b1b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1d9b1b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x11d9b1b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e79b1ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a19b1c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3059b1c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd59b1cc, SUC_lexicon_tags_38 },
    { 0xaf99b1cd, SUC_lexicon_tags_22 },
    { 0xc199b1cc, SUC_lexicon_tags_25 },
    { 0x4e79b1cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2a39b1d1, SUC_lexicon_tags_10 },
    { 0xf639b1d2, SUC_lexicon_tags_47 },
    { 0x50f9b1d2, SUC_lexicon_tags_174 },
    { 0xc299b1d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcf19b1d6, SUC_lexicon_tags_15 },
    { 0xe059b1d7, SUC_lexicon_tags_37 },
    { 0x1459b1d7, SUC_lexicon_tags_82 },
    { 0xec59b1d8, SUC_lexicon_tags_47 },
    { 0xe6d9b1da, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd659b1dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5339b1df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42b9b1e3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xded9b1e5, SUC_lexicon_tags_11 },
    { 0xf8b9b1e6, SUC_lexicon_tags_19 },
    { 0x1dd9b1e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db9b1ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8939b1f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf879b1f2, SUC_lexicon_tags_66 },
    { 0x2bb9b1f2, SUC_lexicon_tags_25 },
    { 0xe1f9b1f2, SUC_lexicon_tags_1 },
    { 0xfd39b1f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f79b1f7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8af9b204, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6bf9b206, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c79b208, SUC_lexicon_tags_29 },
    { 0xaa59b209, SUC_lexicon_tags_651 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd379b215, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa179b21a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdbb9b21c, SUC_lexicon_tags_7 },
    { 0xdf9b21d, SUC_lexicon_tags_27 },
    { 0x3319b21e, SUC_lexicon_tags_17 },
    { 0xfc79b21e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb99b221, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4959b224, SUC_lexicon_tags_17 },
    { 0x6179b224, SUC_lexicon_tags_37 },
    { 0xfc19b224, SUC_lexicon_tags_38 },
    { 0x84d9b225, SUC_lexicon_tags_30 },
    { 0xc019b228, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d39b22a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf59b230, SUC_lexicon_tags_37 },
    { 0x8ab9b230, SUC_lexicon_tags_5 },
    { 0xf919b232, SUC_lexicon_tags_1 },
    { 0x9379b231, SUC_lexicon_tags_16 },
    { 0x20f9b233, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6199b236, SUC_lexicon_tags_7 },
    { 0x5259b237, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb9b23b, SUC_lexicon_tags_12 },
    { 0xb0d9b23b, SUC_lexicon_tags_33 },
    { 0x7219b23d, SUC_lexicon_tags_7 },
    { 0xa8b9b23e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b59b242, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3019b244, SUC_lexicon_tags_11 },
    { 0x8839b244, SUC_lexicon_tags_38 },
    { 0xa979b246, SUC_lexicon_tags_11 },
    { 0x4799b247, SUC_lexicon_tags_12 },
    { 0x7999b248, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2799b24c, SUC_lexicon_tags_25 },
    { 0xb979b24d, SUC_lexicon_tags_74 },
    { 0xf039b24e, SUC_lexicon_tags_23 },
    { 0x8219b24d, SUC_lexicon_tags_9 },
    { 0xec59b24e, SUC_lexicon_tags_11 },
    { 0xbb19b251, SUC_lexicon_tags_45 },
    { 0x21b9b252, SUC_lexicon_tags_11 },
    { 0x2559b253, SUC_lexicon_tags_65 },
    { 0x9519b253, SUC_lexicon_tags_79 },
    { 0x1e19b24e, SUC_lexicon_tags_45 },
    { 0xecb9b256, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1fd9b258, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e19b25f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3d9b261, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd59b26a, SUC_lexicon_tags_2 },
    { 0xc59b26b, SUC_lexicon_tags_49 },
    { 0x8cd9b26c, SUC_lexicon_tags_14 },
    { 0x90d9b26b, SUC_lexicon_tags_61 },
    { 0xdd99b26d, SUC_lexicon_tags_35 },
    { 0x5639b26e, SUC_lexicon_tags_30 },
    { 0x51b9b270, SUC_lexicon_tags_16 },
    { 0xa4b9b271, SUC_lexicon_tags_45 },
    { 0x30d9b26b, SUC_lexicon_tags_10 },
    { 0x3359b273, SUC_lexicon_tags_21 },
    { 0x7599b272, SUC_lexicon_tags_11 },
    { 0xc999b26d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1319b27a, SUC_lexicon_tags_37 },
    { 0x5799b27b, SUC_lexicon_tags_600 },
    { 0xf299b27c, SUC_lexicon_tags_30 },
    { 0x4599b27c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b59b284, SUC_lexicon_tags_92 },
    { 0x6139b284, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39b288, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x49d9b28a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb9b28e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbb9b290, SUC_lexicon_tags_7 },
    { 0xbd79b290, SUC_lexicon_tags_60 },
    { 0x4059b291, SUC_lexicon_tags_14 },
    { 0x1379b293, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5c19b295, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e59b298, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf9b29b, SUC_lexicon_tags_55 },
    { 0xaaf9b29b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x639b29e, SUC_lexicon_tags_37 },
    { 0xf9f9b29f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d9b2a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7259b2a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb179b2a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd959b2ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519b2b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa639b2b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b39b2c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9f19b2c2, SUC_lexicon_tags_11 },
    { 0xa8d9b2c3, SUC_lexicon_tags_12 },
    { 0x4859b2c4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5299b2c6, SUC_lexicon_tags_25 },
    { 0xa19b2c7, SUC_lexicon_tags_19 },
    { 0x8bb9b2c7, SUC_lexicon_tags_2 },
    { 0x1859b2c9, SUC_lexicon_tags_2 },
    { 0x8f19b2ca, SUC_lexicon_tags_41 },
    { 0x5319b2ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e79b2ce, SUC_lexicon_tags_5 },
    { 0x9599b2ce, SUC_lexicon_tags_15 },
    { 0xfa19b2d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f9b2d3, SUC_lexicon_tags_45 },
    { 0x8119b2d4, SUC_lexicon_tags_6 },
    { 0x3639b2d4, SUC_lexicon_tags_30 },
    { 0x8459b2d4, SUC_lexicon_tags_28 },
    { 0xa299b2d6, SUC_lexicon_tags_1 },
    { 0xdfd9b2d8, SUC_lexicon_tags_23 },
    { 0x8239b2d9, SUC_lexicon_tags_15 },
    { 0x92f9b2da, SUC_lexicon_tags_12 },
    { 0x4b9b2d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa419b2e1, SUC_lexicon_tags_422 },
    { 0x80b9b2e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a39b2e5, SUC_lexicon_tags_33 },
    { 0xeab9b2e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6639b2ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf079b2ef, SUC_lexicon_tags_11 },
    { 0x62d9b2f0, SUC_lexicon_tags_15 },
    { 0x8199b2f1, SUC_lexicon_tags_15 },
    { 0x4c39b2f2, SUC_lexicon_tags_43 },
    { 0xc279b2f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x90d9b2f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa319b2fc, SUC_lexicon_tags_2 },
    { 0x2579b2fc, SUC_lexicon_tags_12 },
    { 0x9239b2fe, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9459b305, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa79b307, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6f9b30d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa419b311, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9b316, SUC_lexicon_tags_33 },
    { 0xf959b317, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9099b319, SUC_lexicon_tags_1 },
    { 0x3cf9b31a, SUC_lexicon_tags_11 },
    { 0xf3d9b31b, SUC_lexicon_tags_46 },
    { 0xd3f9b31c, SUC_lexicon_tags_17 },
    { 0xecf9b31d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b9b31f, SUC_lexicon_tags_37 },
    { 0x5139b320, SUC_lexicon_tags_1 },
    { 0x9159b31f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40b9b326, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d9b330, SUC_lexicon_tags_19 },
    { 0x9bd9b330, SUC_lexicon_tags_27 },
    { 0xedd9b332, SUC_lexicon_tags_46 },
    { 0x8b9b332, SUC_lexicon_tags_63 },
    { 0xf099b334, SUC_lexicon_tags_37 },
    { 0xf219b335, SUC_lexicon_tags_23 },
    { 0xb499b336, SUC_lexicon_tags_1 },
    { 0x58b9b330, SUC_lexicon_tags_16 },
    { 0x1279b338, SUC_lexicon_tags_37 },
    { 0xf0d9b339, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83d9b33c, SUC_lexicon_tags_39 },
    { 0x9659b33d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x29b9b33f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88b9b343, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2d9b345, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5339b348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9339b34b, SUC_lexicon_tags_17 },
    { 0xe419b34b, SUC_lexicon_tags_27 },
    { 0xd3b9b34d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4479b34f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3939b352, SUC_lexicon_tags_2 },
    { 0x2fd9b352, SUC_lexicon_tags_12 },
    { 0xf9b9b354, SUC_lexicon_tags_11 },
    { 0xb3f9b355, SUC_lexicon_tags_17 },
    { 0x77f9b356, SUC_lexicon_tags_14 },
    { 0x9c99b357, SUC_lexicon_tags_31 },
    { 0xd6b9b354, SUC_lexicon_tags_17 },
    { 0xc279b358, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3c39b35b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b79b35e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b9b361, SUC_lexicon_tags_31 },
    { 0xa739b361, SUC_lexicon_tags_64 },
    { 0xb1d9b361, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb9b36c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f9b370, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d9b373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e79b375, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5519b377, SUC_lexicon_tags_14 },
    { 0x3ef9b377, SUC_lexicon_tags_25 },
    { 0x5579b378, SUC_lexicon_tags_15 },
    { 0x9919b37a, SUC_lexicon_tags_47 },
    { 0x9a79b37b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2519b37f, SUC_lexicon_tags_66 },
    { 0xc919b380, SUC_lexicon_tags_0 },
    { 0x1039b380, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xbf9b383, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d9b388, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1d79b38a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1859b391, SUC_lexicon_tags_37 },
    { 0xa779b391, SUC_lexicon_tags_2 },
    { 0x7eb9b393, SUC_lexicon_tags_14 },
    { 0xf319b394, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bd9b396, SUC_lexicon_tags_0 },
    { 0xc919b397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d9b39c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1d59b39e, SUC_lexicon_tags_17 },
    { 0x2e79b39f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbf9b3a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2599b3aa, SUC_lexicon_tags_2 },
    { 0x36f9b3ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd119b3b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa2b9b3b8, SUC_lexicon_tags_27 },
    { 0xd4f9b3b9, SUC_lexicon_tags_16 },
    { 0x2f39b3ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf619b3bd, SUC_lexicon_tags_806 },
    { 0, NULL },
    { 0xf459b3bf, SUC_lexicon_tags_25 },
    { 0x579b3bf, SUC_lexicon_tags_16 },
    { 0xfb79b3c1, SUC_lexicon_tags_25 },
    { 0x58b9b3c2, SUC_lexicon_tags_43 },
    { 0x50b9b3c2, SUC_lexicon_tags_7 },
    { 0x1259b3c4, SUC_lexicon_tags_7 },
    { 0x27b9b3c5, SUC_lexicon_tags_37 },
    { 0x99b9b3c5, SUC_lexicon_tags_33 },
    { 0x2759b3c4, SUC_lexicon_tags_0 },
    { 0xd479b3c8, SUC_lexicon_tags_15 },
    { 0x27b9b3c9, SUC_lexicon_tags_207 },
    { 0xd7f9b3c1, SUC_lexicon_tags_1 },
    { 0x70d9b3c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f9b3ce, SUC_lexicon_tags_29 },
    { 0xacf9b3ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12b9b3d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4579b3d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4379b3d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd99b3d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4d9b3e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xce39b3e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea79b3e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb9b3ef, SUC_lexicon_tags_22 },
    { 0x2819b3f0, SUC_lexicon_tags_33 },
    { 0x3a19b3f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15f9b3f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xacd9b3f5, SUC_lexicon_tags_77 },
    { 0xf0d9b3f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd039b3fc, SUC_lexicon_tags_15 },
    { 0x2919b3fc, SUC_lexicon_tags_65 },
    { 0x3559b3fc, SUC_lexicon_tags_12 },
    { 0xdf79b3fd, SUC_lexicon_tags_9 },
    { 0xda99b3ff, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7439b40a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17d9b40c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff9b40f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8cf9b411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66b9b413, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf9b416, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdcf9b418, SUC_lexicon_tags_158 },
    { 0xbdd9b419, SUC_lexicon_tags_17 },
    { 0xe4b9b418, SUC_lexicon_tags_25 },
    { 0x759b41b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e99b41e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb959b423, SUC_lexicon_tags_17 },
    { 0x1619b424, SUC_lexicon_tags_6 },
    { 0x9cb9b424, SUC_lexicon_tags_31 },
    { 0xe59b425, SUC_lexicon_tags_37 },
    { 0x8699b424, SUC_lexicon_tags_7 },
    { 0xd3d9b423, SUC_lexicon_tags_27 },
    { 0xa5f9b429, SUC_lexicon_tags_25 },
    { 0x14b9b42a, SUC_lexicon_tags_35 },
    { 0x8d79b42a, SUC_lexicon_tags_45 },
    { 0x4179b42c, SUC_lexicon_tags_10 },
    { 0x2bd9b42d, SUC_lexicon_tags_8 },
    { 0xa1d9b42e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb79b435, SUC_lexicon_tags_45 },
    { 0xaef9b435, SUC_lexicon_tags_82 },
    { 0x5199b437, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e79b43a, SUC_lexicon_tags_39 },
    { 0xb899b43b, SUC_lexicon_tags_60 },
    { 0x4439b43b, SUC_lexicon_tags_47 },
    { 0x6b79b43c, SUC_lexicon_tags_43 },
    { 0x6f99b43e, SUC_lexicon_tags_23 },
    { 0x2d99b43e, SUC_lexicon_tags_25 },
    { 0x26b9b440, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9399b442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71b9b445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f99b450, SUC_lexicon_tags_18 },
    { 0x5b9b450, SUC_lexicon_tags_19 },
    { 0x9b79b452, SUC_lexicon_tags_7 },
    { 0x2179b453, SUC_lexicon_tags_6 },
    { 0x1839b450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b457, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e39b45d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d39b45f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb99b462, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b9b465, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3459b468, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d39b46b, SUC_lexicon_tags_197 },
    { 0x4b19b46c, SUC_lexicon_tags_27 },
    { 0x8379b46d, SUC_lexicon_tags_1 },
    { 0xdbd9b46e, SUC_lexicon_tags_17 },
    { 0x179b46d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce19b473, SUC_lexicon_tags_7 },
    { 0x5a99b474, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f79b477, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c39b47e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3779b480, SUC_lexicon_tags_14 },
    { 0x1419b481, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe579b484, SUC_lexicon_tags_31 },
    { 0x6159b484, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb9b48a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e19b48c, SUC_lexicon_tags_15 },
    { 0x4d39b48c, SUC_lexicon_tags_26 },
    { 0xf6b9b48e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf8d9b490, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeb39b492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb419b496, SUC_lexicon_tags_39 },
    { 0x55f9b497, SUC_lexicon_tags_7 },
    { 0xdbd9b497, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb759b49a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99b49d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4559b49f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf9b4a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94d9b4a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2639b4aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x41d9b4ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa339b4b0, SUC_lexicon_tags_17 },
    { 0x84d9b4b1, SUC_lexicon_tags_11 },
    { 0x9759b4b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359b4b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe19b4b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d39b4bb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f19b4be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5419b4c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd9b4c6, SUC_lexicon_tags_7 },
    { 0x53b9b4c7, SUC_lexicon_tags_1 },
    { 0xb579b4c8, SUC_lexicon_tags_11 },
    { 0x3459b4c9, SUC_lexicon_tags_0 },
    { 0x5479b4c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e59b4cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9659b4cf, SUC_lexicon_tags_11 },
    { 0x5699b4d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b19b4d7, SUC_lexicon_tags_37 },
    { 0xd699b4d8, SUC_lexicon_tags_33 },
    { 0x40b9b4d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe339b4db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb19b4dd, SUC_lexicon_tags_60 },
    { 0xb699b4dd, SUC_lexicon_tags_14 },
    { 0xa459b4de, SUC_lexicon_tags_6 },
    { 0xfe79b4dd, SUC_lexicon_tags_63 },
    { 0xa3d9b4e1, SUC_lexicon_tags_15 },
    { 0x9539b4e2, SUC_lexicon_tags_2 },
    { 0xa6f9b4e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3639b4e5, SUC_lexicon_tags_25 },
    { 0x1919b4e6, SUC_lexicon_tags_55 },
    { 0x6a59b4e5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6b19b4e9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff9b4ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33b9b4ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d9b4f2, SUC_lexicon_tags_17 },
    { 0xa6f9b4f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839b4fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe259b4ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc479b501, SUC_lexicon_tags_37 },
    { 0xe799b502, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfd19b504, SUC_lexicon_tags_525 },
    { 0xae19b505, SUC_lexicon_tags_174 },
    { 0xd339b506, SUC_lexicon_tags_37 },
    { 0xbf99b507, SUC_lexicon_tags_17 },
    { 0x6939b505, SUC_lexicon_tags_15 },
    { 0xa0f9b509, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3799b50b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb59b50f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3939b511, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd9b515, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b9b519, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7db9b51b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9599b51e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x21f9b520, SUC_lexicon_tags_15 },
    { 0x3559b520, SUC_lexicon_tags_39 },
    { 0x8d59b522, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x619b525, SUC_lexicon_tags_12 },
    { 0xb379b526, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb559b52b, SUC_lexicon_tags_19 },
    { 0xdd19b52b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee59b52f, SUC_lexicon_tags_29 },
    { 0xcd59b530, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x7b19b532, SUC_lexicon_tags_43 },
    { 0x7319b532, SUC_lexicon_tags_47 },
    { 0xac39b534, SUC_lexicon_tags_49 },
    { 0x1c19b533, SUC_lexicon_tags_30 },
    { 0x5ed9b536, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3f9b538, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2439b53d, SUC_lexicon_tags_107 },
    { 0x1d99b53e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4039b541, SUC_lexicon_tags_28 },
    { 0x5ed9b542, SUC_lexicon_tags_29 },
    { 0xcff9b543, SUC_lexicon_tags_2 },
    { 0xb899b544, SUC_lexicon_tags_95 },
    { 0xaed9b543, SUC_lexicon_tags_12 },
    { 0x9e39b542, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f9b549, SUC_lexicon_tags_34 },
    { 0x22b9b54a, SUC_lexicon_tags_92 },
    { 0x30f9b54b, SUC_lexicon_tags_10 },
    { 0x9699b54b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x73f9b54e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b9b554, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb39b558, SUC_lexicon_tags_7 },
    { 0x619b559, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56d9b55b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe919b55e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3439b565, SUC_lexicon_tags_52 },
    { 0x2b59b566, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdf19b568, SUC_lexicon_tags_12 },
    { 0xdc19b569, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb479b56d, SUC_lexicon_tags_23 },
    { 0xdef9b56e, SUC_lexicon_tags_2 },
    { 0x36d9b56f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5419b577, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda39b582, SUC_lexicon_tags_11 },
    { 0x1819b583, SUC_lexicon_tags_47 },
    { 0xb459b583, SUC_lexicon_tags_47 },
    { 0xfa99b585, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6599b588, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd519b58e, SUC_lexicon_tags_47 },
    { 0xe8f9b58f, SUC_lexicon_tags_9 },
    { 0x4579b58e, SUC_lexicon_tags_1 },
    { 0xb3d9b591, SUC_lexicon_tags_49 },
    { 0xa379b58f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb9b599, SUC_lexicon_tags_13 },
    { 0xe3b9b59a, SUC_lexicon_tags_7 },
    { 0x6279b59b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3979b5a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a99b5a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x98b9b5aa, SUC_lexicon_tags_15 },
    { 0xc4d9b5aa, SUC_lexicon_tags_11 },
    { 0x77b9b5ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe99b5b6, SUC_lexicon_tags_37 },
    { 0x22f9b5b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f9b5bc, SUC_lexicon_tags_11 },
    { 0x55f9b5bc, SUC_lexicon_tags_14 },
    { 0x8079b5bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f9b5c1, SUC_lexicon_tags_91 },
    { 0xcd9b5c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1079b5c4, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d59b5c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x58f9b5ca, SUC_lexicon_tags_12 },
    { 0x679b5cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a19b5ce, SUC_lexicon_tags_11 },
    { 0xb39b5cf, SUC_lexicon_tags_27 },
    { 0xead9b5d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7639b5d3, SUC_lexicon_tags_25 },
    { 0xd9f9b5d4, SUC_lexicon_tags_22 },
    { 0xfa99b5d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2579b5d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc459b5db, SUC_lexicon_tags_11 },
    { 0x8999b5dc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8819b5df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8659b5e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4119b5e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8619b5e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80d9b5ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79b5f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb459b5f4, SUC_lexicon_tags_29 },
    { 0xc299b5f5, SUC_lexicon_tags_17 },
    { 0xd99b5f5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf339b5fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f9b5ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0d9b606, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf9b609, SUC_lexicon_tags_61 },
    { 0x9d9b60a, SUC_lexicon_tags_8 },
    { 0x6d99b60a, SUC_lexicon_tags_19 },
    { 0xf6d9b609, SUC_lexicon_tags_38 },
    { 0x1799b60d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd9b613, SUC_lexicon_tags_70 },
    { 0xe999b614, SUC_lexicon_tags_6 },
    { 0x3bd9b615, SUC_lexicon_tags_33 },
    { 0xff59b616, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8bb9b618, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe639b61a, SUC_lexicon_tags_11 },
    { 0xfa39b61b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84b9b61d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f9b622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a19b624, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed9b628, SUC_lexicon_tags_19 },
    { 0x16b9b628, SUC_lexicon_tags_37 },
    { 0xf8b9b62a, SUC_lexicon_tags_39 },
    { 0x8479b62b, SUC_lexicon_tags_37 },
    { 0xf8b9b62c, SUC_lexicon_tags_17 },
    { 0xe759b62c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd19b633, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb9b636, SUC_lexicon_tags_1 },
    { 0xa319b636, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe559b63a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff9b63d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb59b640, SUC_lexicon_tags_2 },
    { 0xae39b641, SUC_lexicon_tags_0 },
    { 0x9cf9b642, SUC_lexicon_tags_27 },
    { 0x35f9b641, SUC_lexicon_tags_12 },
    { 0x48d9b641, SUC_lexicon_tags_55 },
    { 0x8ff9b641, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94d9b648, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4b59b64a, SUC_lexicon_tags_3 },
    { 0x4f19b64b, SUC_lexicon_tags_37 },
    { 0x7859b64a, SUC_lexicon_tags_1 },
    { 0x4e79b64b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a39b650, SUC_lexicon_tags_15 },
    { 0x9319b651, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d9b655, SUC_lexicon_tags_7 },
    { 0xff9b656, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc599b65c, SUC_lexicon_tags_18 },
    { 0xcb9b65d, SUC_lexicon_tags_23 },
    { 0x7099b65c, SUC_lexicon_tags_11 },
    { 0x1839b65f, SUC_lexicon_tags_1 },
    { 0x9399b65f, SUC_lexicon_tags_38 },
    { 0xb6f9b661, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b9b667, SUC_lexicon_tags_60 },
    { 0xf7b9b668, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xae19b66a, SUC_lexicon_tags_17 },
    { 0xe179b66b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec59b671, SUC_lexicon_tags_7 },
    { 0x82f9b671, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xda19b675, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc079b679, SUC_lexicon_tags_82 },
    { 0x3499b67a, SUC_lexicon_tags_54 },
    { 0xbe19b679, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc979b67d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cb9b683, SUC_lexicon_tags_27 },
    { 0x2f9b684, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8299b691, SUC_lexicon_tags_2 },
    { 0xed59b691, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead9b696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39b699, SUC_lexicon_tags_47 },
    { 0xe7f9b69a, SUC_lexicon_tags_17 },
    { 0x5e19b69b, SUC_lexicon_tags_4 },
    { 0xdcb9b69c, SUC_lexicon_tags_11 },
    { 0xacf9b69c, SUC_lexicon_tags_14 },
    { 0x1a59b69b, SUC_lexicon_tags_29 },
    { 0x88b9b69d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b9b6a3, SUC_lexicon_tags_16 },
    { 0xe3f9b6a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x259b6a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec99b6a9, SUC_lexicon_tags_13 },
    { 0x7499b6aa, SUC_lexicon_tags_2 },
    { 0x69d9b6ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb419b6ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f99b6b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa339b6b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8059b6b6, SUC_lexicon_tags_7 },
    { 0xb999b6b7, SUC_lexicon_tags_30 },
    { 0x1d39b6b8, SUC_lexicon_tags_7 },
    { 0x56b9b6b9, SUC_lexicon_tags_37 },
    { 0x2879b6ba, SUC_lexicon_tags_27 },
    { 0x3fd9b6b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe679b6bd, SUC_lexicon_tags_11 },
    { 0x5f9b6be, SUC_lexicon_tags_7 },
    { 0x2699b6bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa139b6c2, SUC_lexicon_tags_43 },
    { 0x8d59b6c2, SUC_lexicon_tags_23 },
    { 0x2e79b6c4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a79b6ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb899b6d2, SUC_lexicon_tags_10 },
    { 0x8419b6d3, SUC_lexicon_tags_23 },
    { 0xe9b9b6d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3719b6d7, SUC_lexicon_tags_16 },
    { 0x779b6d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3339b6db, SUC_lexicon_tags_49 },
    { 0x8cb9b6dc, SUC_lexicon_tags_25 },
    { 0x2bf9b6dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9b6e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb559b6ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdef9b6f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa079b6fa, SUC_lexicon_tags_49 },
    { 0x6359b6fb, SUC_lexicon_tags_2 },
    { 0x2399b6fc, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf9b706, SUC_lexicon_tags_14 },
    { 0x62b9b707, SUC_lexicon_tags_37 },
    { 0x4319b707, SUC_lexicon_tags_27 },
    { 0x21f9b707, SUC_lexicon_tags_25 },
    { 0x7ef9b70a, SUC_lexicon_tags_12 },
    { 0x8919b708, SUC_lexicon_tags_98 },
    { 0x6af9b70b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1679b70e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x36f9b710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f9b713, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xac39b716, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab9b71a, SUC_lexicon_tags_25 },
    { 0xdf99b71b, SUC_lexicon_tags_1 },
    { 0x70b9b71b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3af9b71e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x42b9b720, SUC_lexicon_tags_12 },
    { 0xeb79b720, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f9b724, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x739b726, SUC_lexicon_tags_23 },
    { 0xd299b726, SUC_lexicon_tags_17 },
    { 0xaad9b727, SUC_lexicon_tags_10 },
    { 0x6ad9b729, SUC_lexicon_tags_4 },
    { 0xe7b9b72a, SUC_lexicon_tags_43 },
    { 0xdf39b72b, SUC_lexicon_tags_0 },
    { 0x85f9b728, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x27f9b72e, SUC_lexicon_tags_2 },
    { 0x6d19b72f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6259b732, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb39b735, SUC_lexicon_tags_17 },
    { 0x2d99b736, SUC_lexicon_tags_1 },
    { 0x26f9b736, SUC_lexicon_tags_91 },
    { 0x8479b738, SUC_lexicon_tags_25 },
    { 0xeb99b735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66f9b73c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd439b73f, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0x6199b742, SUC_lexicon_tags_15 },
    { 0x8819b743, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed9b74c, SUC_lexicon_tags_38 },
    { 0x7e79b74d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f9b755, SUC_lexicon_tags_23 },
    { 0x9539b756, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe099b75a, SUC_lexicon_tags_10 },
    { 0xf679b75a, SUC_lexicon_tags_398 },
    { 0x99b9b75a, SUC_lexicon_tags_45 },
    { 0x99b75a, SUC_lexicon_tags_7 },
    { 0xf6f9b75e, SUC_lexicon_tags_14 },
    { 0x6b9b75a, SUC_lexicon_tags_31 },
    { 0x1359b75d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4e99b762, SUC_lexicon_tags_1 },
    { 0xd979b763, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7719b765, SUC_lexicon_tags_3 },
    { 0xb479b765, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb159b768, SUC_lexicon_tags_26 },
    { 0xae99b769, SUC_lexicon_tags_15 },
    { 0xb939b768, SUC_lexicon_tags_1 },
    { 0x43f9b768, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8f9b773, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe159b775, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xac19b778, SUC_lexicon_tags_2 },
    { 0x2f99b779, SUC_lexicon_tags_11 },
    { 0xa339b77a, SUC_lexicon_tags_2 },
    { 0xd099b77a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8399b77f, SUC_lexicon_tags_14 },
    { 0x53d9b780, SUC_lexicon_tags_30 },
    { 0xfc39b781, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c59b786, SUC_lexicon_tags_57 },
    { 0x1a99b787, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb59b789, SUC_lexicon_tags_38 },
    { 0x9f59b78a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4899b78c, SUC_lexicon_tags_14 },
    { 0xaa79b78d, SUC_lexicon_tags_17 },
    { 0x23b9b78e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf779b795, SUC_lexicon_tags_25 },
    { 0x619b795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacd9b799, SUC_lexicon_tags_10 },
    { 0x58f9b79a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf139b79c, SUC_lexicon_tags_17 },
    { 0xea99b79d, SUC_lexicon_tags_43 },
    { 0x5c19b79e, SUC_lexicon_tags_46 },
    { 0xd559b79d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6c19b7a1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe739b7a6, SUC_lexicon_tags_25 },
    { 0xd439b7a7, SUC_lexicon_tags_38 },
    { 0xfab9b7a7, SUC_lexicon_tags_19 },
    { 0x95b9b7a8, SUC_lexicon_tags_17 },
    { 0x8679b7a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f9b7ae, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x56d9b7b0, SUC_lexicon_tags_17 },
    { 0xc259b7b0, SUC_lexicon_tags_37 },
    { 0x5e19b7b1, SUC_lexicon_tags_17 },
    { 0x1cd9b7b3, SUC_lexicon_tags_25 },
    { 0xf8d9b7b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x36f9b7b6, SUC_lexicon_tags_2 },
    { 0x7ff9b7b7, SUC_lexicon_tags_45 },
    { 0x9cb9b7b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf9b7bd, SUC_lexicon_tags_37 },
    { 0x5099b7bd, SUC_lexicon_tags_11 },
    { 0xbab9b7bf, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b9b7c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed99b7cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee39b7cd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x22d9b7cf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xda79b7d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f9b7d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bd9b7d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8679b7de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17f9b7e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe339b7e9, SUC_lexicon_tags_37 },
    { 0x2639b7ea, SUC_lexicon_tags_37 },
    { 0x8e59b7ea, SUC_lexicon_tags_25 },
    { 0xcd9b7ec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23b9b7f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef9b7f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d9b7ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8439b808, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff39b810, SUC_lexicon_tags_431 },
    { 0x67f9b810, SUC_lexicon_tags_8 },
    { 0x7f99b811, SUC_lexicon_tags_25 },
    { 0xbc59b813, SUC_lexicon_tags_717 },
    { 0x8599b814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7599b817, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f9b81d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79b81f, SUC_lexicon_tags_45 },
    { 0x6899b820, SUC_lexicon_tags_37 },
    { 0x8cb9b821, SUC_lexicon_tags_47 },
    { 0x3859b821, SUC_lexicon_tags_1166 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3659b827, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2d9b829, SUC_lexicon_tags_31 },
    { 0xb459b82a, SUC_lexicon_tags_49 },
    { 0x3e39b82b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4359b832, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d9b835, SUC_lexicon_tags_11 },
    { 0x99b9b835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99b83b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9099b844, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x76f9b846, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30d9b84e, SUC_lexicon_tags_47 },
    { 0xec99b84e, SUC_lexicon_tags_25 },
    { 0x4559b850, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff19b852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6939b857, SUC_lexicon_tags_12 },
    { 0xd3b9b858, SUC_lexicon_tags_9 },
    { 0xcdd9b859, SUC_lexicon_tags_45 },
    { 0xfa19b858, SUC_lexicon_tags_11 },
    { 0xe019b857, SUC_lexicon_tags_19 },
    { 0x6719b859, SUC_lexicon_tags_13 },
    { 0x4fb9b85d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b9b860, SUC_lexicon_tags_17 },
    { 0xc39b861, SUC_lexicon_tags_4 },
    { 0x4559b862, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3159b865, SUC_lexicon_tags_7 },
    { 0x9bd9b865, SUC_lexicon_tags_10 },
    { 0xbb79b865, SUC_lexicon_tags_17 },
    { 0x5239b868, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb9b86b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf39b870, SUC_lexicon_tags_12 },
    { 0x9219b871, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xea39b873, SUC_lexicon_tags_47 },
    { 0x4cb9b873, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x32b9b876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9479b879, SUC_lexicon_tags_25 },
    { 0xf259b87a, SUC_lexicon_tags_11 },
    { 0x99f9b87b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a39b881, SUC_lexicon_tags_17 },
    { 0xfd99b882, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae59b88b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b9b88f, SUC_lexicon_tags_2 },
    { 0x4139b890, SUC_lexicon_tags_38 },
    { 0x6cd9b88f, SUC_lexicon_tags_87 },
    { 0x6839b891, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1559b895, SUC_lexicon_tags_15 },
    { 0x79b9b895, SUC_lexicon_tags_33 },
    { 0x5399b897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb9b89b, SUC_lexicon_tags_25 },
    { 0x14b9b89b, SUC_lexicon_tags_11 },
    { 0xfc59b89b, SUC_lexicon_tags_11 },
    { 0x4d59b89e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3479b8a1, SUC_lexicon_tags_9 },
    { 0xf79b8a2, SUC_lexicon_tags_11 },
    { 0x1ad9b8a2, SUC_lexicon_tags_9 },
    { 0x1e39b8a2, SUC_lexicon_tags_15 },
    { 0xae59b8a3, SUC_lexicon_tags_14 },
    { 0x8f19b8a4, SUC_lexicon_tags_17 },
    { 0xb659b8a4, SUC_lexicon_tags_7 },
    { 0x6739b8a1, SUC_lexicon_tags_46 },
    { 0xd139b8a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x42b9b8ab, SUC_lexicon_tags_235 },
    { 0x599b8ac, SUC_lexicon_tags_37 },
    { 0x1cf9b8ab, SUC_lexicon_tags_37 },
    { 0xfe99b8ae, SUC_lexicon_tags_1 },
    { 0x2a99b8ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6519b8b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb239b8b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4b9b8b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79b8bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86b9b8bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d9b8c1, SUC_lexicon_tags_29 },
    { 0xc179b8c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f9b8cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37f9b8cf, SUC_lexicon_tags_15 },
    { 0x4299b8cf, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9b8d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf79b8d8, SUC_lexicon_tags_23 },
    { 0x1459b8d8, SUC_lexicon_tags_25 },
    { 0x2b9b8da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fd9b8dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9499b8e0, SUC_lexicon_tags_57 },
    { 0xfa79b8e1, SUC_lexicon_tags_17 },
    { 0xe639b8e0, SUC_lexicon_tags_7 },
    { 0xc0f9b8e3, SUC_lexicon_tags_7 },
    { 0xbf79b8e4, SUC_lexicon_tags_17 },
    { 0xcc19b8e5, SUC_lexicon_tags_29 },
    { 0xd299b8e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf5f9b8e8, SUC_lexicon_tags_8 },
    { 0x7279b8e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b9b8f1, SUC_lexicon_tags_43 },
    { 0xc299b8f1, SUC_lexicon_tags_2 },
    { 0x9839b8f3, SUC_lexicon_tags_17 },
    { 0x53f9b8f2, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c19b8f7, SUC_lexicon_tags_7 },
    { 0x10d9b8f8, SUC_lexicon_tags_60 },
    { 0x4cf9b8f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde39b8fd, SUC_lexicon_tags_37 },
    { 0x80d9b8fe, SUC_lexicon_tags_8 },
    { 0x58b9b8fd, SUC_lexicon_tags_37 },
    { 0xf699b900, SUC_lexicon_tags_33 },
    { 0x2879b901, SUC_lexicon_tags_11 },
    { 0xfe19b902, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd9b906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1779b90b, SUC_lexicon_tags_8 },
    { 0xb279b90c, SUC_lexicon_tags_11 },
    { 0xa1f9b90d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf119b90f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba79b911, SUC_lexicon_tags_55 },
    { 0xf0f9b912, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde39b914, SUC_lexicon_tags_7 },
    { 0x3a39b915, SUC_lexicon_tags_2 },
    { 0xec19b916, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e19b91e, SUC_lexicon_tags_7 },
    { 0xf839b91f, SUC_lexicon_tags_2 },
    { 0x51b9b91f, SUC_lexicon_tags_2 },
    { 0xf2f9b921, SUC_lexicon_tags_7 },
    { 0x97b9b921, SUC_lexicon_tags_1 },
    { 0xdeb9b91f, SUC_lexicon_tags_29 },
    { 0x1b59b924, SUC_lexicon_tags_2 },
    { 0xa439b91e, SUC_lexicon_tags_23 },
    { 0x7739b926, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4679b929, SUC_lexicon_tags_12 },
    { 0x4479b92a, SUC_lexicon_tags_12 },
    { 0xf419b92b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc419b930, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xf679b932, SUC_lexicon_tags_35 },
    { 0x56f9b932, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5059b939, SUC_lexicon_tags_50 },
    { 0xb639b93a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15d9b93c, SUC_lexicon_tags_38 },
    { 0x9719b93d, SUC_lexicon_tags_2 },
    { 0x7319b93d, SUC_lexicon_tags_43 },
    { 0x77d9b93d, SUC_lexicon_tags_2 },
    { 0x20d9b940, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9839b943, SUC_lexicon_tags_0 },
    { 0x4479b944, SUC_lexicon_tags_15 },
    { 0x56f9b945, SUC_lexicon_tags_27 },
    { 0x8439b946, SUC_lexicon_tags_25 },
    { 0x42d9b947, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf399b949, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd9b94d, SUC_lexicon_tags_9 },
    { 0x5a99b94e, SUC_lexicon_tags_27 },
    { 0x40f9b94d, SUC_lexicon_tags_11 },
    { 0xbaf9b950, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3719b952, SUC_lexicon_tags_27 },
    { 0x5e39b953, SUC_lexicon_tags_68 },
    { 0x5699b954, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9939b957, SUC_lexicon_tags_15 },
    { 0x2bd9b958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6499b95b, SUC_lexicon_tags_10 },
    { 0x26d9b95b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb9b95f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8bb9b961, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef99b966, SUC_lexicon_tags_37 },
    { 0x28d9b966, SUC_lexicon_tags_37 },
    { 0xaf99b966, SUC_lexicon_tags_8 },
    { 0x3639b969, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x859b96b, SUC_lexicon_tags_32 },
    { 0x1af9b96c, SUC_lexicon_tags_0 },
    { 0xff19b96d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e79b970, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24d9b975, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3479b97a, SUC_lexicon_tags_19 },
    { 0xf019b97b, SUC_lexicon_tags_24 },
    { 0x4139b97b, SUC_lexicon_tags_0 },
    { 0x3c39b97d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f9b983, SUC_lexicon_tags_17 },
    { 0xe279b984, SUC_lexicon_tags_37 },
    { 0x2979b983, SUC_lexicon_tags_43 },
    { 0x25d9b983, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc39b98c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8f9b98e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a79b991, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc939b993, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xce79b995, SUC_lexicon_tags_12 },
    { 0xd439b996, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc539b998, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8779b99a, SUC_lexicon_tags_7 },
    { 0x8c79b99b, SUC_lexicon_tags_14 },
    { 0xff39b99c, SUC_lexicon_tags_71 },
    { 0x2519b99d, SUC_lexicon_tags_11 },
    { 0x2459b99e, SUC_lexicon_tags_39 },
    { 0x7ed9b99c, SUC_lexicon_tags_2 },
    { 0xf499b99b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d59b9a3, SUC_lexicon_tags_17 },
    { 0x5cb9b9a4, SUC_lexicon_tags_33 },
    { 0xd5f9b9a4, SUC_lexicon_tags_12 },
    { 0xf339b9a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44b9b9ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc39b9b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f9b9b3, SUC_lexicon_tags_45 },
    { 0xc3f9b9b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e39b9bd, SUC_lexicon_tags_26 },
    { 0x9839b9bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6a19b9c0, SUC_lexicon_tags_9 },
    { 0xd599b9c0, SUC_lexicon_tags_11 },
    { 0x9cf9b9c1, SUC_lexicon_tags_12 },
    { 0xa439b9c3, SUC_lexicon_tags_7 },
    { 0x15b9b9c0, SUC_lexicon_tags_46 },
    { 0xe399b9c5, SUC_lexicon_tags_27 },
    { 0x9139b9c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2059b9c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf39b9cb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1ed9b9cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8039b9cf, SUC_lexicon_tags_40 },
    { 0x8f19b9cf, SUC_lexicon_tags_25 },
    { 0x2519b9d1, SUC_lexicon_tags_17 },
    { 0x97d9b9d2, SUC_lexicon_tags_8 },
    { 0xb839b9cf, SUC_lexicon_tags_234 },
    { 0xe2f9b9d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3d9b9d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b79b9d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b9b9e1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f9b9e4, SUC_lexicon_tags_16 },
    { 0xad9b9e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7819b9ea, SUC_lexicon_tags_7 },
    { 0x519b9eb, SUC_lexicon_tags_151 },
    { 0xa6d9b9ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbd9b9ee, SUC_lexicon_tags_37 },
    { 0xc399b9ee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a59b9f4, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x6c39b9f6, SUC_lexicon_tags_31 },
    { 0x6c39b9f6, SUC_lexicon_tags_17 },
    { 0x20d9b9f8, SUC_lexicon_tags_23 },
    { 0xc699b9f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc119b9ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5c59ba01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd979ba03, SUC_lexicon_tags_25 },
    { 0xd139ba03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c79ba0a, SUC_lexicon_tags_2 },
    { 0xdbd9ba0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad79ba13, SUC_lexicon_tags_45 },
    { 0xef79ba14, SUC_lexicon_tags_15 },
    { 0x2059ba13, SUC_lexicon_tags_9 },
    { 0xf7b9ba16, SUC_lexicon_tags_12 },
    { 0x8659ba16, SUC_lexicon_tags_2 },
    { 0xfef9ba16, SUC_lexicon_tags_17 },
    { 0x9499ba16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc039ba1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f59ba1e, SUC_lexicon_tags_16 },
    { 0xdb19ba1e, SUC_lexicon_tags_1 },
    { 0x9819ba20, SUC_lexicon_tags_25 },
    { 0x3999ba20, SUC_lexicon_tags_12 },
    { 0x4d79ba20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a19ba24, SUC_lexicon_tags_38 },
    { 0xf019ba24, SUC_lexicon_tags_45 },
    { 0x8ef9ba25, SUC_lexicon_tags_7 },
    { 0x3d79ba27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b9ba2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e59ba2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c39ba2e, SUC_lexicon_tags_31 },
    { 0x9259ba2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1b9ba31, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad79ba37, SUC_lexicon_tags_13 },
    { 0x44b9ba38, SUC_lexicon_tags_14 },
    { 0x179ba39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64d9ba46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f99ba4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecb9ba4d, SUC_lexicon_tags_37 },
    { 0xd6b9ba4e, SUC_lexicon_tags_33 },
    { 0x7d19ba4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc099ba57, SUC_lexicon_tags_12 },
    { 0x5bb9ba57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8759ba5b, SUC_lexicon_tags_19 },
    { 0x9799ba5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9ba60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9ba65, SUC_lexicon_tags_38 },
    { 0x379ba66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d9ba69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b9ba6c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f9ba71, SUC_lexicon_tags_6 },
    { 0xaa59ba72, SUC_lexicon_tags_14 },
    { 0x8bd9ba72, SUC_lexicon_tags_151 },
    { 0x5139ba73, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d79ba76, SUC_lexicon_tags_37 },
    { 0xf7b9ba77, SUC_lexicon_tags_33 },
    { 0x8b59ba77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b9ba80, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc99ba83, SUC_lexicon_tags_25 },
    { 0x2559ba83, SUC_lexicon_tags_45 },
    { 0x1259ba83, SUC_lexicon_tags_1 },
    { 0x6019ba86, SUC_lexicon_tags_61 },
    { 0xa959ba83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba59ba8a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef9ba93, SUC_lexicon_tags_1 },
    { 0xb79ba93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd219ba97, SUC_lexicon_tags_25 },
    { 0x9b19ba98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa459ba9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2439baa1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd5f9baa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2559baa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd79baaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16b9bab1, SUC_lexicon_tags_15 },
    { 0xd2b9bab2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd9bab5, SUC_lexicon_tags_15 },
    { 0x95d9bab6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1379bab8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8279babf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8239bac1, SUC_lexicon_tags_21 },
    { 0xc4d9bac2, SUC_lexicon_tags_32 },
    { 0x8319bac2, SUC_lexicon_tags_63 },
    { 0x98b9bac1, SUC_lexicon_tags_26 },
    { 0x7559bac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f9baca, SUC_lexicon_tags_1 },
    { 0xf499bacb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb399bad2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3479bad4, SUC_lexicon_tags_30 },
    { 0x3879bad5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa579badc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb99badf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23b9bae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9d9bae3, SUC_lexicon_tags_19 },
    { 0xd59bae4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c59bae6, SUC_lexicon_tags_12 },
    { 0xa979bae6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3c19bae9, SUC_lexicon_tags_25 },
    { 0xb019baea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd4f9baec, SUC_lexicon_tags_17 },
    { 0x9bf9baed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2959baef, SUC_lexicon_tags_0 },
    { 0xe779baf0, SUC_lexicon_tags_64 },
    { 0xdd9baf0, SUC_lexicon_tags_17 },
    { 0x80d9baf2, SUC_lexicon_tags_17 },
    { 0x7db9baf2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3a99baf5, SUC_lexicon_tags_8 },
    { 0xa979baf6, SUC_lexicon_tags_16 },
    { 0x52f9baf5, SUC_lexicon_tags_201 },
    { 0x4f99baf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf9baff, SUC_lexicon_tags_37 },
    { 0x6199bb00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9679bb03, SUC_lexicon_tags_47 },
    { 0xd9bb03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54b9bb06, SUC_lexicon_tags_13 },
    { 0x9f59bb07, SUC_lexicon_tags_2 },
    { 0xd6f9bb07, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cb9bb0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa79bb11, SUC_lexicon_tags_22 },
    { 0xfff9bb12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x14f9bb14, SUC_lexicon_tags_12 },
    { 0xe879bb15, SUC_lexicon_tags_33 },
    { 0xd4f9bb16, SUC_lexicon_tags_84 },
    { 0x1939bb17, SUC_lexicon_tags_64 },
    { 0x3979bb14, SUC_lexicon_tags_35 },
    { 0x8c19bb19, SUC_lexicon_tags_12 },
    { 0x4039bb1a, SUC_lexicon_tags_1 },
    { 0xd059bb1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7f9bb1d, SUC_lexicon_tags_5 },
    { 0xc439bb1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf199bb23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9659bb2b, SUC_lexicon_tags_28 },
    { 0x1a39bb2c, SUC_lexicon_tags_23 },
    { 0x4959bb2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9459bb31, SUC_lexicon_tags_17 },
    { 0x4959bb31, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9379bb37, SUC_lexicon_tags_12 },
    { 0xb4d9bb37, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7df9bb3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d79bb3c, SUC_lexicon_tags_0 },
    { 0xce79bb3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6cb9bb3f, SUC_lexicon_tags_8 },
    { 0x2039bb40, SUC_lexicon_tags_8 },
    { 0x5f9bb40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb9bb46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b9bb4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1579bb50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dd9bb53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2f59bb55, SUC_lexicon_tags_37 },
    { 0x139bb56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4219bb59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9699bb5b, SUC_lexicon_tags_37 },
    { 0x4939bb5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa319bb5e, SUC_lexicon_tags_25 },
    { 0x67d9bb5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a59bb61, SUC_lexicon_tags_26 },
    { 0x8939bb62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1179bb67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5039bb69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4959bb6b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb9bb75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bb79, SUC_lexicon_tags_54 },
    { 0xa5f9bb7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x419bb7c, SUC_lexicon_tags_77 },
    { 0x8a59bb7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f99bb7f, SUC_lexicon_tags_30 },
    { 0xcad9bb80, SUC_lexicon_tags_13 },
    { 0xea99bb81, SUC_lexicon_tags_107 },
    { 0xb499bb82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf79bb89, SUC_lexicon_tags_12 },
    { 0xa3f9bb89, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7579bb8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcdb9bb8e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b9bb92, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e99bb95, SUC_lexicon_tags_12 },
    { 0xd699bb96, SUC_lexicon_tags_25 },
    { 0xbd19bb97, SUC_lexicon_tags_14 },
    { 0x1c59bb98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f9bba3, SUC_lexicon_tags_7 },
    { 0xf779bba4, SUC_lexicon_tags_6 },
    { 0xd979bba3, SUC_lexicon_tags_55 },
    { 0x9579bba6, SUC_lexicon_tags_17 },
    { 0xdbf9bba6, SUC_lexicon_tags_2 },
    { 0xb619bba7, SUC_lexicon_tags_38 },
    { 0xa919bba3, SUC_lexicon_tags_21 },
    { 0x1db9bbaa, SUC_lexicon_tags_11 },
    { 0xba19bbab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b59bbad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d9bbb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e59bbbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeff9bbbd, SUC_lexicon_tags_25 },
    { 0x6ef9bbbe, SUC_lexicon_tags_48 },
    { 0x8479bbbf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd639bbc4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe659bbcb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8559bbcd, SUC_lexicon_tags_60 },
    { 0xdf19bbce, SUC_lexicon_tags_12 },
    { 0x5a99bbcf, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cb9bbd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf819bbd6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc799bbdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe059bbdf, SUC_lexicon_tags_2 },
    { 0xa3f9bbe0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xca39bbe3, SUC_lexicon_tags_26 },
    { 0xd059bbe3, SUC_lexicon_tags_0 },
    { 0xbc79bbe4, SUC_lexicon_tags_21 },
    { 0x5b39bbe4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1039bbee, SUC_lexicon_tags_11 },
    { 0xe39bbef, SUC_lexicon_tags_33 },
    { 0x86b9bbef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f99bbf4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe939bbf8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3159bbfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac59bbfd, SUC_lexicon_tags_11 },
    { 0xd9bbfd, SUC_lexicon_tags_11 },
    { 0xa239bbfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee79bc02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf599bc07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b9bc0d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc99bc10, SUC_lexicon_tags_30 },
    { 0x1999bc11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7079bc14, SUC_lexicon_tags_34 },
    { 0x8219bc15, SUC_lexicon_tags_6 },
    { 0x8ad9bc14, SUC_lexicon_tags_43 },
    { 0x39f9bc16, SUC_lexicon_tags_56 },
    { 0x2499bc18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb9bc1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe419bc25, SUC_lexicon_tags_9 },
    { 0xe239bc26, SUC_lexicon_tags_2 },
    { 0xaf39bc26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d9bc2a, SUC_lexicon_tags_63 },
    { 0xf5b9bc2b, SUC_lexicon_tags_30 },
    { 0xea19bc2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea39bc2f, SUC_lexicon_tags_9 },
    { 0x79d9bc2f, SUC_lexicon_tags_26 },
    { 0xacd9bc31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9dd9bc33, SUC_lexicon_tags_63 },
    { 0x2299bc34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6639bc36, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x319bc39, SUC_lexicon_tags_17 },
    { 0xca39bc3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x819bc3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6db9bc3e, SUC_lexicon_tags_17 },
    { 0x9af9bc3f, SUC_lexicon_tags_17 },
    { 0xeb59bc3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xead9bc42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1819bc44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x78b9bc47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6199bc49, SUC_lexicon_tags_11 },
    { 0x9659bc4a, SUC_lexicon_tags_481 },
    { 0xa579bc4a, SUC_lexicon_tags_11 },
    { 0x6679bc4b, SUC_lexicon_tags_8 },
    { 0x599bc4d, SUC_lexicon_tags_37 },
    { 0x6e59bc4e, SUC_lexicon_tags_11 },
    { 0xcc79bc4f, SUC_lexicon_tags_11 },
    { 0xa2f9bc4c, SUC_lexicon_tags_17 },
    { 0xacb9bc49, SUC_lexicon_tags_43 },
    { 0x4a99bc52, SUC_lexicon_tags_38 },
    { 0x21f9bc4b, SUC_lexicon_tags_25 },
    { 0x3bf9bc54, SUC_lexicon_tags_1 },
    { 0x4219bc4a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff9bc5e, SUC_lexicon_tags_37 },
    { 0x5ff9bc5e, SUC_lexicon_tags_40 },
    { 0x1a39bc60, SUC_lexicon_tags_14 },
    { 0x7c79bc61, SUC_lexicon_tags_2 },
    { 0x39bc61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79bc64, SUC_lexicon_tags_2 },
    { 0x4779bc65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d39bc67, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68b9bc72, SUC_lexicon_tags_15 },
    { 0x6159bc72, SUC_lexicon_tags_87 },
    { 0xf059bc74, SUC_lexicon_tags_3 },
    { 0x3bf9bc74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf479bc77, SUC_lexicon_tags_54 },
    { 0xca59bc78, SUC_lexicon_tags_185 },
    { 0xf359bc79, SUC_lexicon_tags_28 },
    { 0x7fb9bc79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2759bc7e, SUC_lexicon_tags_25 },
    { 0xd3f9bc7e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f9bc82, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0xc519bc85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f9bc8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7919bc8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c99bc93, SUC_lexicon_tags_36 },
    { 0xb119bc94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7619bc96, SUC_lexicon_tags_37 },
    { 0xa159bc96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd359bc9c, SUC_lexicon_tags_12 },
    { 0x31d9bc9c, SUC_lexicon_tags_25 },
    { 0x3139bc9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e19bca4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb1b9bca6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc879bcaa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d9bcad, SUC_lexicon_tags_8 },
    { 0x4919bcae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xca59bcb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3559bcb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c79bcbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf099bcbe, SUC_lexicon_tags_49 },
    { 0x67b9bcbf, SUC_lexicon_tags_17 },
    { 0xb219bcc0, SUC_lexicon_tags_7 },
    { 0x19bcc1, SUC_lexicon_tags_0 },
    { 0x539bcbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d9bccb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x91b9bccd, SUC_lexicon_tags_63 },
    { 0x4399bcce, SUC_lexicon_tags_25 },
    { 0x7ef9bcce, SUC_lexicon_tags_25 },
    { 0xd159bcd0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xead9bcd5, SUC_lexicon_tags_25 },
    { 0x66b9bcd6, SUC_lexicon_tags_17 },
    { 0x4819bcd6, SUC_lexicon_tags_27 },
    { 0xd19bcd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b9bcdc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x759bcdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5099bce1, SUC_lexicon_tags_47 },
    { 0xdeb9bce2, SUC_lexicon_tags_7 },
    { 0x5519bce3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd9bce6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99b9bce8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d9bcec, SUC_lexicon_tags_11 },
    { 0x5b79bced, SUC_lexicon_tags_17 },
    { 0xea39bcec, SUC_lexicon_tags_2 },
    { 0x6ff9bcef, SUC_lexicon_tags_1 },
    { 0xbd99bcef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3599bcf2, SUC_lexicon_tags_17 },
    { 0xa1f9bcf2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2219bcf5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xccb9bcf7, SUC_lexicon_tags_1 },
    { 0xd0b9bcf8, SUC_lexicon_tags_47 },
    { 0x8ef9bcf9, SUC_lexicon_tags_12 },
    { 0xeff9bcfa, SUC_lexicon_tags_0 },
    { 0x4ab9bcfb, SUC_lexicon_tags_55 },
    { 0x7459bcfc, SUC_lexicon_tags_29 },
    { 0xd179bcf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e59bcff, SUC_lexicon_tags_16 },
    { 0x8c59bd00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f9bd04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39bd08, SUC_lexicon_tags_12 },
    { 0xe699bd08, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc099bd0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7819bd14, SUC_lexicon_tags_15 },
    { 0x34b9bd14, SUC_lexicon_tags_6 },
    { 0xa899bd16, SUC_lexicon_tags_104 },
    { 0x1799bd15, SUC_lexicon_tags_46 },
    { 0x8a79bd17, SUC_lexicon_tags_60 },
    { 0x9b19bd19, SUC_lexicon_tags_14 },
    { 0x9719bd1a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb9bd1f, SUC_lexicon_tags_37 },
    { 0x8b19bd20, SUC_lexicon_tags_32 },
    { 0x6199bd20, SUC_lexicon_tags_79 },
    { 0xe1b9bd21, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1339bd24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x259bd26, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc19bd2b, SUC_lexicon_tags_38 },
    { 0x6119bd2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d9bd2f, SUC_lexicon_tags_29 },
    { 0xfe79bd30, SUC_lexicon_tags_15 },
    { 0x3e19bd2f, SUC_lexicon_tags_11 },
    { 0x3479bd31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f9bd37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e99bd3c, SUC_lexicon_tags_25 },
    { 0xaf99bd3c, SUC_lexicon_tags_2 },
    { 0xb839bd3e, SUC_lexicon_tags_14 },
    { 0xbf79bd3e, SUC_lexicon_tags_0 },
    { 0x4899bd40, SUC_lexicon_tags_4 },
    { 0x6779bd3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc839bd4a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12b9bd4e, SUC_lexicon_tags_0 },
    { 0x9079bd4f, SUC_lexicon_tags_11 },
    { 0xe2d9bd4f, SUC_lexicon_tags_69 },
    { 0x1379bd4f, SUC_lexicon_tags_0 },
    { 0xd899bd52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8679bd54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f19bd56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a19bd58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf79bd5a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2299bd5c, SUC_lexicon_tags_57 },
    { 0x37d9bd5c, SUC_lexicon_tags_25 },
    { 0xc299bd5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7099bd60, SUC_lexicon_tags_12 },
    { 0xcd99bd61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6859bd64, SUC_lexicon_tags_49 },
    { 0x1b79bd65, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf819bd67, SUC_lexicon_tags_10 },
    { 0xa839bd67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd819bd6c, SUC_lexicon_tags_11 },
    { 0x9439bd6d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad9bd71, SUC_lexicon_tags_45 },
    { 0xacb9bd71, SUC_lexicon_tags_6 },
    { 0xb19bd73, SUC_lexicon_tags_49 },
    { 0xadb9bd73, SUC_lexicon_tags_37 },
    { 0x23f9bd71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9619bd78, SUC_lexicon_tags_26 },
    { 0x6a59bd78, SUC_lexicon_tags_25 },
    { 0xdad9bd7a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6959bd7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd919bd7e, SUC_lexicon_tags_7 },
    { 0x5499bd7f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c59bd82, SUC_lexicon_tags_7 },
    { 0xcf19bd83, SUC_lexicon_tags_1 },
    { 0x9439bd82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f9bd87, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9899bd8b, SUC_lexicon_tags_23 },
    { 0xa619bd8c, SUC_lexicon_tags_9 },
    { 0xa539bd8d, SUC_lexicon_tags_38 },
    { 0xb1d9bd8d, SUC_lexicon_tags_11 },
    { 0xe979bd8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe419bd91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57f9bd93, SUC_lexicon_tags_68 },
    { 0xcb59bd94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb9bd98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x359bd9b, SUC_lexicon_tags_25 },
    { 0xb3b9bd9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75b9bd9e, SUC_lexicon_tags_37 },
    { 0x3f99bd9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd9bda4, SUC_lexicon_tags_17 },
    { 0x1e99bda4, SUC_lexicon_tags_63 },
    { 0x7659bda4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8599bda8, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x359bdab, SUC_lexicon_tags_141 },
    { 0xc4d9bdab, SUC_lexicon_tags_103 },
    { 0x7539bdad, SUC_lexicon_tags_1 },
    { 0x3019bdae, SUC_lexicon_tags_57 },
    { 0x2af9bdac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad9bdb6, SUC_lexicon_tags_25 },
    { 0xf3f9bdb6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4299bdbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119bdc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe739bdc2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb9bdc5, SUC_lexicon_tags_15 },
    { 0x9ad9bdc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73f9bdc8, SUC_lexicon_tags_11 },
    { 0xf659bdc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4079bdcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd319bdcf, SUC_lexicon_tags_11 },
    { 0x8659bdcf, SUC_lexicon_tags_11 },
    { 0x3ab9bdd0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4699bdd6, SUC_lexicon_tags_25 },
    { 0x6779bdd6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xae39bdd9, SUC_lexicon_tags_49 },
    { 0x3739bdda, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1519bddc, SUC_lexicon_tags_49 },
    { 0x14d9bddd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7719bde3, SUC_lexicon_tags_11 },
    { 0x4879bde4, SUC_lexicon_tags_26 },
    { 0x8099bde4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x33f9bde8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8db9bdea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a59bdf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c59bdf3, SUC_lexicon_tags_28 },
    { 0x56d9bdf4, SUC_lexicon_tags_25 },
    { 0x6db9bdf5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d9bdf8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9eb9bdfa, SUC_lexicon_tags_15 },
    { 0x5819bdfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc919bdff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd9be02, SUC_lexicon_tags_23 },
    { 0xb579be03, SUC_lexicon_tags_2 },
    { 0x9a99be04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb819be0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1079be10, SUC_lexicon_tags_11 },
    { 0x9839be11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9599be13, SUC_lexicon_tags_0 },
    { 0x69d9be14, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb59be18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a39be1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6539be1d, SUC_lexicon_tags_32 },
    { 0xa559be1d, SUC_lexicon_tags_25 },
    { 0x29b9be1f, SUC_lexicon_tags_25 },
    { 0xf439be1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a99be24, SUC_lexicon_tags_1 },
    { 0x8a99be25, SUC_lexicon_tags_2 },
    { 0xb699be26, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x379be28, SUC_lexicon_tags_2 },
    { 0x18f9be28, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39be2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2279be34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3119be36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b9be3a, SUC_lexicon_tags_11 },
    { 0xd19be3b, SUC_lexicon_tags_12 },
    { 0xa199be3c, SUC_lexicon_tags_172 },
    { 0x9859be3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf9be41, SUC_lexicon_tags_19 },
    { 0x6479be42, SUC_lexicon_tags_14 },
    { 0xf519be42, SUC_lexicon_tags_16 },
    { 0x94b9be44, SUC_lexicon_tags_12 },
    { 0x7e59be44, SUC_lexicon_tags_7 },
    { 0x1339be41, SUC_lexicon_tags_12 },
    { 0x16d9be45, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x61b9be49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe99be4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b9be4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2919be51, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7579be55, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3f59be57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7f59be59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7b9be5b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee79be5f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x6839be61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8439be69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d9be6e, SUC_lexicon_tags_11 },
    { 0xae79be6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f9be75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6799be79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7839be7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90b9be84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb739be89, SUC_lexicon_tags_45 },
    { 0x19d9be8a, SUC_lexicon_tags_17 },
    { 0xcd99be8b, SUC_lexicon_tags_10 },
    { 0xa4b9be8c, SUC_lexicon_tags_7 },
    { 0x7db9be8d, SUC_lexicon_tags_47 },
    { 0x9199be8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219be92, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb9be9a, SUC_lexicon_tags_32 },
    { 0xfdb9be9a, SUC_lexicon_tags_11 },
    { 0x8cd9be9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf19be9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5cb9bea0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3099bea3, SUC_lexicon_tags_7 },
    { 0xcd79bea4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f9bea8, SUC_lexicon_tags_25 },
    { 0x4099bea9, SUC_lexicon_tags_1160 },
    { 0x3fd9beaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2919beaf, SUC_lexicon_tags_2 },
    { 0xf419beb0, SUC_lexicon_tags_19 },
    { 0x99b9beb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a19beb7, SUC_lexicon_tags_1 },
    { 0x68b9beb7, SUC_lexicon_tags_151 },
    { 0xb959beb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfcf9bebb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b9bec3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xf839bec6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b9becb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9bed2, SUC_lexicon_tags_11 },
    { 0x18d9bed3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed9bed6, SUC_lexicon_tags_7 },
    { 0xc379bed7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6139bedb, SUC_lexicon_tags_60 },
    { 0x9319bedc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc819bee0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98b9bee2, SUC_lexicon_tags_13 },
    { 0x4f59bee2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc759beed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6439bef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7619bef2, SUC_lexicon_tags_277 },
    { 0xbd19bef3, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3439befa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2079befc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xce79beff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b9bf0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xec39bf0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xddd9bf0f, SUC_lexicon_tags_14 },
    { 0xb099bf10, SUC_lexicon_tags_11 },
    { 0x89d9bf10, SUC_lexicon_tags_11 },
    { 0x8d39bf12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf959bf14, SUC_lexicon_tags_31 },
    { 0x3af9bf14, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b99bf19, SUC_lexicon_tags_33 },
    { 0x68b9bf1a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9219bf1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2299bf1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e39bf21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa399bf24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3699bf26, SUC_lexicon_tags_23 },
    { 0xa2b9bf27, SUC_lexicon_tags_17 },
    { 0x7599bf26, SUC_lexicon_tags_15 },
    { 0x5b39bf29, SUC_lexicon_tags_30 },
    { 0x2d99bf29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9239bf31, SUC_lexicon_tags_15 },
    { 0x5d19bf31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28d9bf3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52f9bf3d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b9bf4a, SUC_lexicon_tags_61 },
    { 0xd1b9bf4b, SUC_lexicon_tags_1 },
    { 0xbb9bf4c, SUC_lexicon_tags_37 },
    { 0x2bf9bf4c, SUC_lexicon_tags_43 },
    { 0x48d9bf4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0b9bf52, SUC_lexicon_tags_12 },
    { 0x6a99bf52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb979bf57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d9bf60, SUC_lexicon_tags_11 },
    { 0x7e99bf60, SUC_lexicon_tags_32 },
    { 0xc239bf60, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x34b9bf64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb099bf6b, SUC_lexicon_tags_27 },
    { 0x96d9bf6b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x96d9bf6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab9bf71, SUC_lexicon_tags_52 },
    { 0x37d9bf72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe979bf76, SUC_lexicon_tags_11 },
    { 0x159bf77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b9bf7b, SUC_lexicon_tags_15 },
    { 0xf0d9bf7c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd559bf7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d9bf81, SUC_lexicon_tags_33 },
    { 0xfd59bf82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bf9bf84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9bf89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc39bf96, SUC_lexicon_tags_45 },
    { 0x1959bf97, SUC_lexicon_tags_25 },
    { 0x7f19bf97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5659bf9b, SUC_lexicon_tags_49 },
    { 0xb1f9bf9c, SUC_lexicon_tags_47 },
    { 0x4dd9bf9d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3819bfa0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b79bfa3, SUC_lexicon_tags_47 },
    { 0xf2d9bfa4, SUC_lexicon_tags_2 },
    { 0x1259bfa5, SUC_lexicon_tags_17 },
    { 0x4af9bfa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a59bfa8, SUC_lexicon_tags_6 },
    { 0x8b19bfa9, SUC_lexicon_tags_0 },
    { 0xaeb9bfaa, SUC_lexicon_tags_0 },
    { 0xe559bfab, SUC_lexicon_tags_15 },
    { 0xc3b9bfab, SUC_lexicon_tags_31 },
    { 0xd819bfad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f39bfaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f39bfb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59bfba, SUC_lexicon_tags_9 },
    { 0x6479bfbb, SUC_lexicon_tags_33 },
    { 0xb499bfbb, SUC_lexicon_tags_7 },
    { 0xda59bfbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b39bfbf, SUC_lexicon_tags_27 },
    { 0x2539bfc0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b19bfc4, SUC_lexicon_tags_37 },
    { 0x2b79bfc4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9619bfc8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x2dd9bfca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4d39bfcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d39bfd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x339bfd2, SUC_lexicon_tags_33 },
    { 0x4f9bfd3, SUC_lexicon_tags_29 },
    { 0x6239bfd2, SUC_lexicon_tags_7 },
    { 0x4a79bfd5, SUC_lexicon_tags_27 },
    { 0x5019bfd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6079bfdc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e99bfdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6639bfe1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1e19bfe3, SUC_lexicon_tags_83 },
    { 0x2f9bfe3, SUC_lexicon_tags_45 },
    { 0xfc19bfe5, SUC_lexicon_tags_13 },
    { 0x5039bfe6, SUC_lexicon_tags_2 },
    { 0x6a39bfe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5019bfed, SUC_lexicon_tags_46 },
    { 0xaad9bfee, SUC_lexicon_tags_2 },
    { 0x32f9bfef, SUC_lexicon_tags_30 },
    { 0x1399bfef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0b9bff4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf39bffb, SUC_lexicon_tags_1 },
    { 0x1579bffb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fd9bffe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8439c000, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xda79c002, SUC_lexicon_tags_15 },
    { 0x3ed9c002, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2679c007, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3b9c009, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f9c00c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5219c00f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9c017, SUC_lexicon_tags_61 },
    { 0xa659c017, SUC_lexicon_tags_5 },
    { 0xef99c017, SUC_lexicon_tags_113 },
    { 0x7b19c01a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d9c01e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e19c020, SUC_lexicon_tags_14 },
    { 0x67d9c021, SUC_lexicon_tags_28 },
    { 0x77d9c020, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1af9c024, SUC_lexicon_tags_61 },
    { 0x49d9c025, SUC_lexicon_tags_1 },
    { 0xb3b9c025, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd819c02b, SUC_lexicon_tags_30 },
    { 0xd5d9c02b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc879c030, SUC_lexicon_tags_37 },
    { 0x2f99c030, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd399c036, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf679c03a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f9c03f, SUC_lexicon_tags_4 },
    { 0xd419c040, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x91d9c042, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd219c047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x80b9c049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd879c051, SUC_lexicon_tags_25 },
    { 0x3499c051, SUC_lexicon_tags_2 },
    { 0xc899c053, SUC_lexicon_tags_1 },
    { 0xe719c053, SUC_lexicon_tags_38 },
    { 0xfd99c055, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd979c057, SUC_lexicon_tags_17 },
    { 0x7d39c057, SUC_lexicon_tags_21 },
    { 0x459c058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c79c05c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9cb9c05e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf119c062, SUC_lexicon_tags_28 },
    { 0x5d99c063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeb9c067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6499c069, SUC_lexicon_tags_39 },
    { 0x2c59c069, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d9c06f, SUC_lexicon_tags_28 },
    { 0xf619c06f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x1739c073, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a79c077, SUC_lexicon_tags_38 },
    { 0x10d9c077, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x7ed9c07a, SUC_lexicon_tags_12 },
    { 0xe739c07b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb39c07d, SUC_lexicon_tags_14 },
    { 0xa679c07d, SUC_lexicon_tags_63 },
    { 0xda39c07f, SUC_lexicon_tags_30 },
    { 0x1b9c07f, SUC_lexicon_tags_14 },
    { 0xe779c081, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4919c084, SUC_lexicon_tags_26 },
    { 0x7779c085, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1819c087, SUC_lexicon_tags_11 },
    { 0x4cb9c088, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x51b9c08a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f9c08e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb719c094, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd9c097, SUC_lexicon_tags_27 },
    { 0xe019c098, SUC_lexicon_tags_25 },
    { 0xaa99c099, SUC_lexicon_tags_63 },
    { 0x5bd9c09a, SUC_lexicon_tags_17 },
    { 0x7679c09b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xde19c09e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9019c0a0, SUC_lexicon_tags_11 },
    { 0x1f39c0a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b99c0a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56b9c0ae, SUC_lexicon_tags_15 },
    { 0xae99c0ae, SUC_lexicon_tags_38 },
    { 0x559c0b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d19c0b3, SUC_lexicon_tags_47 },
    { 0xcf19c0b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x54f9c0b6, SUC_lexicon_tags_11 },
    { 0x1af9c0b7, SUC_lexicon_tags_47 },
    { 0x9fd9c0b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd659c0bb, SUC_lexicon_tags_28 },
    { 0x3699c0bc, SUC_lexicon_tags_87 },
    { 0xc7f9c0bd, SUC_lexicon_tags_60 },
    { 0x7739c0bb, SUC_lexicon_tags_27 },
    { 0xfa59c0bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc479c0c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4b9c0c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a99c0cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd379c0cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6099c0d1, SUC_lexicon_tags_37 },
    { 0xc319c0d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c99c0d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eb9c0db, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x4a99c0dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa619c0df, SUC_lexicon_tags_14 },
    { 0xaf39c0e0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc539c0ea, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5019c0ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa8d9c0f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9bb9c0f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d19c0f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f9c0fc, SUC_lexicon_tags_25 },
    { 0x9359c0fd, SUC_lexicon_tags_7 },
    { 0xe239c0fe, SUC_lexicon_tags_63 },
    { 0xc879c0fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x64d9c102, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4299c104, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee19c106, SUC_lexicon_tags_28 },
    { 0x7859c107, SUC_lexicon_tags_11 },
    { 0xe559c107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd99c10b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x59c10e, SUC_lexicon_tags_37 },
    { 0x9b39c10f, SUC_lexicon_tags_45 },
    { 0x5039c110, SUC_lexicon_tags_1 },
    { 0xa8d9c111, SUC_lexicon_tags_21 },
    { 0xcdd9c110, SUC_lexicon_tags_14 },
    { 0xbf99c113, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9219c117, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb619c11b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc99c120, SUC_lexicon_tags_82 },
    { 0x7c99c121, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879c12b, SUC_lexicon_tags_39 },
    { 0x1799c12c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ab9c137, SUC_lexicon_tags_25 },
    { 0xc6d9c137, SUC_lexicon_tags_15 },
    { 0x6f9c139, SUC_lexicon_tags_12 },
    { 0x1979c138, SUC_lexicon_tags_2 },
    { 0x1339c13b, SUC_lexicon_tags_1 },
    { 0x8e79c13c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8f9c141, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f9c144, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f79c148, SUC_lexicon_tags_25 },
    { 0x8e19c148, SUC_lexicon_tags_11 },
    { 0x6db9c14a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xfa19c14c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6239c15a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb719c15d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5939c160, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b9c163, SUC_lexicon_tags_26 },
    { 0x4859c164, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa39c166, SUC_lexicon_tags_44 },
    { 0x67f9c167, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b9c170, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7539c172, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9c176, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xebd9c178, SUC_lexicon_tags_38 },
    { 0x7259c179, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8419c17c, SUC_lexicon_tags_87 },
    { 0x6779c17d, SUC_lexicon_tags_7 },
    { 0xc059c17e, SUC_lexicon_tags_0 },
    { 0x5cb9c17f, SUC_lexicon_tags_25 },
    { 0xbff9c180, SUC_lexicon_tags_63 },
    { 0xbc39c181, SUC_lexicon_tags_10 },
    { 0x8dd9c182, SUC_lexicon_tags_49 },
    { 0xd0b9c181, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7859c186, SUC_lexicon_tags_0 },
    { 0x62d9c186, SUC_lexicon_tags_17 },
    { 0xebd9c186, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc739c18b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4059c192, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6599c195, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf799c197, SUC_lexicon_tags_12 },
    { 0xf299c198, SUC_lexicon_tags_37 },
    { 0x46b9c199, SUC_lexicon_tags_43 },
    { 0x3a79c19a, SUC_lexicon_tags_47 },
    { 0x5db9c19b, SUC_lexicon_tags_66 },
    { 0xe2f9c19b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c79c1a2, SUC_lexicon_tags_15 },
    { 0xd79c1a3, SUC_lexicon_tags_1 },
    { 0x4af9c1a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b9c1a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb119c1a9, SUC_lexicon_tags_2 },
    { 0x50f9c1aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b9c1ae, SUC_lexicon_tags_7 },
    { 0x7499c1af, SUC_lexicon_tags_25 },
    { 0x1579c1b0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x339c1b2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6859c1b4, SUC_lexicon_tags_37 },
    { 0x16f9c1b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa79c1b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91f9c1b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3399c1c3, SUC_lexicon_tags_2 },
    { 0x48b9c1c4, SUC_lexicon_tags_91 },
    { 0xf499c1c5, SUC_lexicon_tags_7 },
    { 0x6dd9c1c6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb9c1c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf9c1cc, SUC_lexicon_tags_34 },
    { 0x1f39c1cd, SUC_lexicon_tags_43 },
    { 0xe199c1cd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c99c1d8, SUC_lexicon_tags_17 },
    { 0xda99c1d9, SUC_lexicon_tags_97 },
    { 0xbbd9c1da, SUC_lexicon_tags_103 },
    { 0xcb9c1d8, SUC_lexicon_tags_13 },
    { 0xa039c1da, SUC_lexicon_tags_7 },
    { 0x5279c1d8, SUC_lexicon_tags_38 },
    { 0x7419c1dd, SUC_lexicon_tags_47 },
    { 0xa319c1dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb79c1e1, SUC_lexicon_tags_46 },
    { 0xc4d9c1e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af9c1e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d9c1ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc539c1ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39c1f2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8139c1f4, SUC_lexicon_tags_6 },
    { 0x2d9c1f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1699c1fa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7159c1ff, SUC_lexicon_tags_11 },
    { 0x1c59c200, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1b59c202, SUC_lexicon_tags_15 },
    { 0x74f9c203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e39c205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4379c209, SUC_lexicon_tags_11 },
    { 0x4499c20a, SUC_lexicon_tags_68 },
    { 0x88d9c20b, SUC_lexicon_tags_60 },
    { 0x9139c20c, SUC_lexicon_tags_17 },
    { 0x6599c20c, SUC_lexicon_tags_56 },
    { 0x89f9c20d, SUC_lexicon_tags_16 },
    { 0xcf59c20f, SUC_lexicon_tags_25 },
    { 0xd359c20e, SUC_lexicon_tags_48 },
    { 0x6779c20b, SUC_lexicon_tags_49 },
    { 0xa6d9c212, SUC_lexicon_tags_0 },
    { 0xf8f9c213, SUC_lexicon_tags_1 },
    { 0xfe79c214, SUC_lexicon_tags_60 },
    { 0xc379c210, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9319c217, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f99c219, SUC_lexicon_tags_6 },
    { 0x33b9c21a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd399c21e, SUC_lexicon_tags_5 },
    { 0x919c21f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb9c222, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda79c226, SUC_lexicon_tags_43 },
    { 0x8f39c227, SUC_lexicon_tags_83 },
    { 0x1299c228, SUC_lexicon_tags_29 },
    { 0xd1b9c229, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x92d9c22b, SUC_lexicon_tags_11 },
    { 0x3b19c22b, SUC_lexicon_tags_1 },
    { 0xb3b9c22d, SUC_lexicon_tags_1 },
    { 0x3559c22b, SUC_lexicon_tags_25 },
    { 0xfcd9c22c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5cd9c231, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe819c233, SUC_lexicon_tags_37 },
    { 0x8f9c234, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e79c239, SUC_lexicon_tags_2 },
    { 0x5cb9c239, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3f9c23d, SUC_lexicon_tags_60 },
    { 0xdbd9c23e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2779c244, SUC_lexicon_tags_9 },
    { 0x93d9c245, SUC_lexicon_tags_85 },
    { 0x7d59c246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe699c24f, SUC_lexicon_tags_12 },
    { 0xae99c250, SUC_lexicon_tags_8 },
    { 0x2b79c24f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59c254, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba39c258, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22d9c25d, SUC_lexicon_tags_46 },
    { 0xe259c25e, SUC_lexicon_tags_14 },
    { 0x5679c25e, SUC_lexicon_tags_1 },
    { 0xa579c25f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2419c264, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eb9c267, SUC_lexicon_tags_1151 },
    { 0x6259c267, SUC_lexicon_tags_14 },
    { 0x9919c269, SUC_lexicon_tags_12 },
    { 0x1059c26a, SUC_lexicon_tags_70 },
    { 0xf999c26b, SUC_lexicon_tags_21 },
    { 0x8199c26a, SUC_lexicon_tags_11 },
    { 0xa1f9c26d, SUC_lexicon_tags_11 },
    { 0x4799c26c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf9c272, SUC_lexicon_tags_40 },
    { 0x7ef9c272, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8379c276, SUC_lexicon_tags_1 },
    { 0x3f59c276, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f99c27b, SUC_lexicon_tags_17 },
    { 0x1d99c27c, SUC_lexicon_tags_77 },
    { 0xedd9c27d, SUC_lexicon_tags_9 },
    { 0x279c27e, SUC_lexicon_tags_49 },
    { 0xc319c27f, SUC_lexicon_tags_47 },
    { 0xe199c27e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36d9c284, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7999c287, SUC_lexicon_tags_15 },
    { 0x3619c288, SUC_lexicon_tags_17 },
    { 0xc339c288, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ed9c28c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44f9c290, SUC_lexicon_tags_7 },
    { 0xd219c291, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7119c294, SUC_lexicon_tags_2 },
    { 0x899c295, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdeb9c297, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc059c29c, SUC_lexicon_tags_1 },
    { 0x25f9c29d, SUC_lexicon_tags_15 },
    { 0x2339c29c, SUC_lexicon_tags_49 },
    { 0xe2f9c29f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb979c2a1, SUC_lexicon_tags_37 },
    { 0x1759c2a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9bd9c2a4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca19c2aa, SUC_lexicon_tags_2 },
    { 0xde79c2ab, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x9eb9c2ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d9c2b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ef9c2bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb9c2bf, SUC_lexicon_tags_46 },
    { 0x1839c2c0, SUC_lexicon_tags_23 },
    { 0xe5b9c2c0, SUC_lexicon_tags_351 },
    { 0xf759c2c2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9c2c7, SUC_lexicon_tags_68 },
    { 0xe3d9c2c8, SUC_lexicon_tags_11 },
    { 0xd0d9c2c9, SUC_lexicon_tags_7 },
    { 0x6ff9c2c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe739c2cd, SUC_lexicon_tags_37 },
    { 0xb039c2ce, SUC_lexicon_tags_37 },
    { 0x4899c2ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68b9c2d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x359c2d3, SUC_lexicon_tags_63 },
    { 0x2979c2d4, SUC_lexicon_tags_12 },
    { 0x53b9c2d4, SUC_lexicon_tags_66 },
    { 0xc7d9c2d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c39c2de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc279c2e1, SUC_lexicon_tags_38 },
    { 0xd3b9c2e1, SUC_lexicon_tags_45 },
    { 0x5939c2e2, SUC_lexicon_tags_27 },
    { 0x72f9c2e4, SUC_lexicon_tags_46 },
    { 0xe099c2e5, SUC_lexicon_tags_43 },
    { 0x8959c2e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2159c2e9, SUC_lexicon_tags_37 },
    { 0x6dd9c2ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84b9c2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xafd9c2f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf9c2f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb599c2f4, SUC_lexicon_tags_31 },
    { 0x13d9c2f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4739c2f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7659c2fb, SUC_lexicon_tags_45 },
    { 0x7239c2fb, SUC_lexicon_tags_11 },
    { 0xe439c2fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3479c300, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d9c30a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa359c310, SUC_lexicon_tags_6 },
    { 0x79b9c311, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfe59c313, SUC_lexicon_tags_49 },
    { 0xc419c313, SUC_lexicon_tags_23 },
    { 0x879c314, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b79c318, SUC_lexicon_tags_6 },
    { 0xa8d9c319, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe619c31d, SUC_lexicon_tags_32 },
    { 0xbcb9c31d, SUC_lexicon_tags_48 },
    { 0xcd59c31e, SUC_lexicon_tags_25 },
    { 0xda59c320, SUC_lexicon_tags_1 },
    { 0x5a39c321, SUC_lexicon_tags_25 },
    { 0x9719c321, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd9c326, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1779c32b, SUC_lexicon_tags_19 },
    { 0xdad9c32c, SUC_lexicon_tags_87 },
    { 0x7079c32c, SUC_lexicon_tags_37 },
    { 0xb359c32e, SUC_lexicon_tags_12 },
    { 0xe899c32c, SUC_lexicon_tags_39 },
    { 0xff19c32e, SUC_lexicon_tags_37 },
    { 0x49d9c331, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf9c334, SUC_lexicon_tags_14 },
    { 0xa259c334, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3739c33b, SUC_lexicon_tags_15 },
    { 0x2e99c33b, SUC_lexicon_tags_33 },
    { 0x6b9c33d, SUC_lexicon_tags_11 },
    { 0xff79c33e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1419c342, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1739c345, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b9c348, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d79c350, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaf9c354, SUC_lexicon_tags_113 },
    { 0x7959c354, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4339c35c, SUC_lexicon_tags_26 },
    { 0x9a19c35d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9419c363, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x52f9c366, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f59c369, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4959c36d, SUC_lexicon_tags_27 },
    { 0x6ad9c36e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b9c371, SUC_lexicon_tags_82 },
    { 0xb079c372, SUC_lexicon_tags_79 },
    { 0x7ef9c373, SUC_lexicon_tags_12 },
    { 0xdc59c374, SUC_lexicon_tags_79 },
    { 0xe319c375, SUC_lexicon_tags_17 },
    { 0xc079c376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e79c379, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbcd9c37b, SUC_lexicon_tags_56 },
    { 0x1139c37c, SUC_lexicon_tags_37 },
    { 0x3479c37d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f9c382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9af9c386, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c19c38a, SUC_lexicon_tags_25 },
    { 0xef9c38a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5bf9c38d, SUC_lexicon_tags_18 },
    { 0xaf79c38e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf9c391, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e19c393, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea79c397, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d9c39a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2619c39e, SUC_lexicon_tags_33 },
    { 0x97b9c39e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4dd9c3a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f9c3a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9c3a9, SUC_lexicon_tags_25 },
    { 0xb999c3a9, SUC_lexicon_tags_60 },
    { 0xcf39c3ab, SUC_lexicon_tags_47 },
    { 0xc679c3ac, SUC_lexicon_tags_37 },
    { 0xc3d9c3ad, SUC_lexicon_tags_17 },
    { 0x5b99c3ae, SUC_lexicon_tags_25 },
    { 0xe6f9c3af, SUC_lexicon_tags_33 },
    { 0xed19c3af, SUC_lexicon_tags_13 },
    { 0x8639c3b1, SUC_lexicon_tags_47 },
    { 0x3a79c3ae, SUC_lexicon_tags_15 },
    { 0x44b9c3b0, SUC_lexicon_tags_37 },
    { 0x9cb9c3b4, SUC_lexicon_tags_15 },
    { 0x7d9c3b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf9c3ba, SUC_lexicon_tags_2 },
    { 0x76b9c3ba, SUC_lexicon_tags_2 },
    { 0xcb19c3bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c99c3bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9c3c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9d9c3c5, SUC_lexicon_tags_33 },
    { 0x83f9c3c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b9c3cb, SUC_lexicon_tags_31 },
    { 0x479c3cc, SUC_lexicon_tags_1 },
    { 0xa7d9c3cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa199c3d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d9c3da, SUC_lexicon_tags_30 },
    { 0x83f9c3db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68f9c3dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2459c3e2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91d9c3e4, SUC_lexicon_tags_140 },
    { 0xe799c3e5, SUC_lexicon_tags_37 },
    { 0x4919c3e6, SUC_lexicon_tags_26 },
    { 0x299c3e6, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xfb59c3e9, SUC_lexicon_tags_323 },
    { 0xd59c3ea, SUC_lexicon_tags_229 },
    { 0x219c3eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2679c3ee, SUC_lexicon_tags_28 },
    { 0x5479c3ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4e79c3f1, SUC_lexicon_tags_8 },
    { 0x1839c3f2, SUC_lexicon_tags_11 },
    { 0x1a79c3f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb979c3f5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x4df9c3f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac79c3ff, SUC_lexicon_tags_11 },
    { 0x2b79c400, SUC_lexicon_tags_12 },
    { 0xd659c3ff, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9299c408, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1899c40a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c79c410, SUC_lexicon_tags_37 },
    { 0xe919c411, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9379c415, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc759c41d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9b9c420, SUC_lexicon_tags_107 },
    { 0xdc39c421, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b79c424, SUC_lexicon_tags_25 },
    { 0x6899c425, SUC_lexicon_tags_34 },
    { 0xbf19c425, SUC_lexicon_tags_30 },
    { 0x1819c427, SUC_lexicon_tags_12 },
    { 0xfd59c427, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74b9c42a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c79c42c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb119c42f, SUC_lexicon_tags_17 },
    { 0x3479c430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a19c435, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad19c43c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b9c443, SUC_lexicon_tags_13 },
    { 0x8a39c443, SUC_lexicon_tags_39 },
    { 0x2d59c445, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x59d9c447, SUC_lexicon_tags_23 },
    { 0x41f9c448, SUC_lexicon_tags_25 },
    { 0x6b99c448, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30b9c450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde79c452, SUC_lexicon_tags_60 },
    { 0xfeb9c452, SUC_lexicon_tags_48 },
    { 0x4ed9c454, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8279c45c, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78d9c460, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5219c462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e19c464, SUC_lexicon_tags_17 },
    { 0x7e39c465, SUC_lexicon_tags_14 },
    { 0x88b9c466, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e19c468, SUC_lexicon_tags_37 },
    { 0xb619c469, SUC_lexicon_tags_23 },
    { 0x8519c468, SUC_lexicon_tags_11 },
    { 0x4cf9c46b, SUC_lexicon_tags_25 },
    { 0xbd99c46c, SUC_lexicon_tags_37 },
    { 0xbfd9c46b, SUC_lexicon_tags_29 },
    { 0x8ef9c46e, SUC_lexicon_tags_25 },
    { 0x36f9c46f, SUC_lexicon_tags_21 },
    { 0xb1b9c470, SUC_lexicon_tags_30 },
    { 0xf519c470, SUC_lexicon_tags_68 },
    { 0xb159c46b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6259c474, SUC_lexicon_tags_17 },
    { 0x2c19c475, SUC_lexicon_tags_14 },
    { 0xb519c475, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9df9c479, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b79c47c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb79c482, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22b9c485, SUC_lexicon_tags_33 },
    { 0x4379c485, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb019c488, SUC_lexicon_tags_11 },
    { 0xab99c489, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2879c48d, SUC_lexicon_tags_21 },
    { 0x58b9c48e, SUC_lexicon_tags_23 },
    { 0x8979c48d, SUC_lexicon_tags_12 },
    { 0xf079c490, SUC_lexicon_tags_9 },
    { 0x9a59c48e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5d99c493, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1019c498, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xc7d9c49a, SUC_lexicon_tags_35 },
    { 0xc099c49a, SUC_lexicon_tags_37 },
    { 0x9f79c49b, SUC_lexicon_tags_47 },
    { 0x179c49c, SUC_lexicon_tags_9 },
    { 0x76b9c49e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2839c4a1, SUC_lexicon_tags_11 },
    { 0x8c99c4a1, SUC_lexicon_tags_57 },
    { 0xd399c4a1, SUC_lexicon_tags_9 },
    { 0xb799c4a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0f9c4a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b9c4ac, SUC_lexicon_tags_349 },
    { 0xad9c4ac, SUC_lexicon_tags_8 },
    { 0xdeb9c4ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b39c4b2, SUC_lexicon_tags_17 },
    { 0xa799c4b2, SUC_lexicon_tags_33 },
    { 0xfdd9c4b2, SUC_lexicon_tags_17 },
    { 0xa999c4b5, SUC_lexicon_tags_17 },
    { 0xc119c4b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8859c4be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6739c4c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f79c4ca, SUC_lexicon_tags_607 },
    { 0xd4b9c4cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x99d9c4cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2399c4d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9239c4d6, SUC_lexicon_tags_3 },
    { 0x6559c4d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb19c4d9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xee59c4dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6419c4de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed59c4e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd859c4e7, SUC_lexicon_tags_7 },
    { 0xd159c4e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e19c4ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b9c4f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8af9c4f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5b9c4f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0f9c4fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d19c502, SUC_lexicon_tags_29 },
    { 0x6f79c503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde79c506, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2379c508, SUC_lexicon_tags_7 },
    { 0x7c99c508, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafb9c50c, SUC_lexicon_tags_60 },
    { 0x6bf9c50d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6679c50f, SUC_lexicon_tags_65 },
    { 0xc199c50f, SUC_lexicon_tags_37 },
    { 0x59f9c511, SUC_lexicon_tags_49 },
    { 0xb599c512, SUC_lexicon_tags_84 },
    { 0x1559c513, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef9c516, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b9c51a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcab9c51f, SUC_lexicon_tags_28 },
    { 0x8379c520, SUC_lexicon_tags_6 },
    { 0x9439c521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e99c525, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d9c52a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea19c52c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8b9c52e, SUC_lexicon_tags_21 },
    { 0x23d9c52f, SUC_lexicon_tags_26 },
    { 0x65f9c530, SUC_lexicon_tags_25 },
    { 0x7b59c531, SUC_lexicon_tags_46 },
    { 0x3199c532, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf819c53c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd19c53f, SUC_lexicon_tags_14 },
    { 0x6ad9c53f, SUC_lexicon_tags_0 },
    { 0x6539c541, SUC_lexicon_tags_7 },
    { 0x9a79c541, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a19c547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe499c549, SUC_lexicon_tags_45 },
    { 0xdf59c54a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b9c550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa899c553, SUC_lexicon_tags_28 },
    { 0x4959c553, SUC_lexicon_tags_63 },
    { 0x5439c555, SUC_lexicon_tags_2 },
    { 0x3439c553, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8759c558, SUC_lexicon_tags_47 },
    { 0x9799c558, SUC_lexicon_tags_7 },
    { 0x2519c55a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819c563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9c568, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa319c56a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcf9c56e, SUC_lexicon_tags_93 },
    { 0x6cf9c56e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f9c577, SUC_lexicon_tags_23 },
    { 0xb7f9c577, SUC_lexicon_tags_47 },
    { 0xef9c579, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe39c57b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9399c57f, SUC_lexicon_tags_12 },
    { 0xe639c580, SUC_lexicon_tags_19 },
    { 0xc519c57f, SUC_lexicon_tags_37 },
    { 0xb599c582, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5399c584, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b9c586, SUC_lexicon_tags_16 },
    { 0xccb9c587, SUC_lexicon_tags_17 },
    { 0xb4f9c588, SUC_lexicon_tags_25 },
    { 0xb499c588, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b99c596, SUC_lexicon_tags_1 },
    { 0x7999c597, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2859c599, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f9c59d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b9c5a0, SUC_lexicon_tags_12 },
    { 0x6eb9c5a1, SUC_lexicon_tags_1 },
    { 0xb4d9c5a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa399c5a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9419c5aa, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a59c5ad, SUC_lexicon_tags_63 },
    { 0x6139c5ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x379c5b0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x59c5b2, SUC_lexicon_tags_47 },
    { 0xb959c5b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2b9c5ba, SUC_lexicon_tags_63 },
    { 0xee59c5ba, SUC_lexicon_tags_7 },
    { 0xd3f9c5bc, SUC_lexicon_tags_57 },
    { 0xdfb9c5bd, SUC_lexicon_tags_2 },
    { 0x39b9c5be, SUC_lexicon_tags_25 },
    { 0xadf9c5be, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9c19c5c1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x66b9c5c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f9c5ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf9c5ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1119c5d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7bf9c5d2, SUC_lexicon_tags_11 },
    { 0xaa79c5d3, SUC_lexicon_tags_25 },
    { 0xa59c5d3, SUC_lexicon_tags_11 },
    { 0x4179c5d5, SUC_lexicon_tags_43 },
    { 0x8cd9c5d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb9c5dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd9c5e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb959c5e5, SUC_lexicon_tags_15 },
    { 0xaff9c5e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d9c5ea, SUC_lexicon_tags_10 },
    { 0x2799c5ea, SUC_lexicon_tags_7 },
    { 0xf839c5ea, SUC_lexicon_tags_48 },
    { 0xe119c5ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e79c5f0, SUC_lexicon_tags_12 },
    { 0xc8f9c5f1, SUC_lexicon_tags_14 },
    { 0xf199c5f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4259c5f5, SUC_lexicon_tags_14 },
    { 0x539c5f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd9c5fb, SUC_lexicon_tags_31 },
    { 0xecd9c5fc, SUC_lexicon_tags_2 },
    { 0x19d9c5fd, SUC_lexicon_tags_25 },
    { 0xee99c5fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf9c606, SUC_lexicon_tags_25 },
    { 0x7319c607, SUC_lexicon_tags_27 },
    { 0x8979c607, SUC_lexicon_tags_97 },
    { 0x2499c608, SUC_lexicon_tags_11 },
    { 0x6119c60a, SUC_lexicon_tags_43 },
    { 0x2719c608, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93d9c60d, SUC_lexicon_tags_65 },
    { 0xe6f9c60e, SUC_lexicon_tags_29 },
    { 0x9ab9c60f, SUC_lexicon_tags_1 },
    { 0xaf9c60f, SUC_lexicon_tags_29 },
    { 0x519c60f, SUC_lexicon_tags_2 },
    { 0xd1d9c60f, SUC_lexicon_tags_45 },
    { 0xcb9c613, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd5f9c615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8959c619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda19c61b, SUC_lexicon_tags_32 },
    { 0x3079c61c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc9d9c61e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48d9c620, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x28d9c623, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4819c62f, SUC_lexicon_tags_77 },
    { 0x4439c630, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3159c632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf9c636, SUC_lexicon_tags_19 },
    { 0x8559c637, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5679c63a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8819c63c, SUC_lexicon_tags_47 },
    { 0x12d9c63d, SUC_lexicon_tags_121 },
    { 0xc9f9c63e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f9c644, SUC_lexicon_tags_7 },
    { 0x1459c645, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc739c647, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd719c649, SUC_lexicon_tags_7 },
    { 0x8b19c64a, SUC_lexicon_tags_31 },
    { 0xbd99c649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d9c650, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2279c652, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c59c656, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb9c664, SUC_lexicon_tags_14 },
    { 0x2679c665, SUC_lexicon_tags_38 },
    { 0xb799c665, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe939c66a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc419c66d, SUC_lexicon_tags_25 },
    { 0x9599c66e, SUC_lexicon_tags_37 },
    { 0x8bf9c66f, SUC_lexicon_tags_11 },
    { 0xeb99c670, SUC_lexicon_tags_15 },
    { 0xa759c66e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe159c67b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa039c67d, SUC_lexicon_tags_55 },
    { 0x70d9c67e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc39c681, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59f9c683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff9c689, SUC_lexicon_tags_67 },
    { 0x79b9c68a, SUC_lexicon_tags_0 },
    { 0xbf19c68b, SUC_lexicon_tags_42 },
    { 0x1a39c68a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd539c68f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc219c691, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x1899c694, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f9c69d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb19c6a1, SUC_lexicon_tags_32 },
    { 0x1899c6a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea79c6a6, SUC_lexicon_tags_18 },
    { 0x7939c6a7, SUC_lexicon_tags_45 },
    { 0xd979c6a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9159c6ae, SUC_lexicon_tags_15 },
    { 0xd2f9c6af, SUC_lexicon_tags_49 },
    { 0xdd19c6b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb9c6b4, SUC_lexicon_tags_12 },
    { 0x6359c6b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed9c6b8, SUC_lexicon_tags_31 },
    { 0x4db9c6b8, SUC_lexicon_tags_38 },
    { 0x5039c6b9, SUC_lexicon_tags_37 },
    { 0x19d9c6bb, SUC_lexicon_tags_9 },
    { 0xecb9c6bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f9c6c2, SUC_lexicon_tags_14 },
    { 0x479c6c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1939c6c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf979c6c7, SUC_lexicon_tags_1 },
    { 0x6dd9c6c8, SUC_lexicon_tags_2 },
    { 0x5319c6c7, SUC_lexicon_tags_37 },
    { 0xbc79c6c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb9c6cd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2519c6d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31d9c6d4, SUC_lexicon_tags_27 },
    { 0xff9c6d5, SUC_lexicon_tags_2 },
    { 0xa639c6d6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x31d9c6d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1699c6db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf999c6de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd079c6eb, SUC_lexicon_tags_37 },
    { 0xacd9c6eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5eb9c6ee, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe659c6f0, SUC_lexicon_tags_35 },
    { 0xac59c6f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8179c6f3, SUC_lexicon_tags_82 },
    { 0x6fb9c6f4, SUC_lexicon_tags_7 },
    { 0xc479c6f3, SUC_lexicon_tags_31 },
    { 0x2fd9c6f6, SUC_lexicon_tags_92 },
    { 0xbf79c6f7, SUC_lexicon_tags_17 },
    { 0x3cb9c6f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5379c6fa, SUC_lexicon_tags_1 },
    { 0xccf9c6fa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe659c6fe, SUC_lexicon_tags_31 },
    { 0x699c6ff, SUC_lexicon_tags_27 },
    { 0x1099c700, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf799c703, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf99c706, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67d9c708, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf899c70b, SUC_lexicon_tags_11 },
    { 0x4619c70c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e19c711, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d9c714, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3bb9c716, SUC_lexicon_tags_1 },
    { 0xcd99c717, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7439c71c, SUC_lexicon_tags_38 },
    { 0x44f9c71d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50f9c721, SUC_lexicon_tags_14 },
    { 0xd99c722, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda79c725, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a19c72c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa699c731, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc459c733, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b9c735, SUC_lexicon_tags_8 },
    { 0x4699c736, SUC_lexicon_tags_7 },
    { 0x6619c737, SUC_lexicon_tags_2 },
    { 0x4779c738, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa699c73c, SUC_lexicon_tags_0 },
    { 0x5f9c73d, SUC_lexicon_tags_11 },
    { 0x20b9c73e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1159c741, SUC_lexicon_tags_11 },
    { 0x3959c742, SUC_lexicon_tags_11 },
    { 0xcaf9c742, SUC_lexicon_tags_29 },
    { 0x4e79c744, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab9c746, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc19c74d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xefd9c74f, SUC_lexicon_tags_8 },
    { 0x2139c750, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a39c758, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ed9c75a, SUC_lexicon_tags_25 },
    { 0x7d99c75a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf739c75e, SUC_lexicon_tags_25 },
    { 0xda39c75e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xce79c761, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5659c765, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8b9c768, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b9c76b, SUC_lexicon_tags_15 },
    { 0x5bd9c76c, SUC_lexicon_tags_60 },
    { 0x8f99c76b, SUC_lexicon_tags_68 },
    { 0x4b19c76c, SUC_lexicon_tags_11 },
    { 0xad39c76f, SUC_lexicon_tags_39 },
    { 0xfb9c770, SUC_lexicon_tags_1 },
    { 0x9c59c770, SUC_lexicon_tags_65 },
    { 0x1ed9c772, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd9c77a, SUC_lexicon_tags_19 },
    { 0x89b9c77b, SUC_lexicon_tags_55 },
    { 0x39b9c77c, SUC_lexicon_tags_1 },
    { 0x6919c77b, SUC_lexicon_tags_9 },
    { 0xd239c77e, SUC_lexicon_tags_17 },
    { 0x1f39c77e, SUC_lexicon_tags_25 },
    { 0xcfd9c780, SUC_lexicon_tags_21 },
    { 0xee79c781, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9859c784, SUC_lexicon_tags_23 },
    { 0xcbf9c785, SUC_lexicon_tags_37 },
    { 0x8739c785, SUC_lexicon_tags_1 },
    { 0xc0d9c787, SUC_lexicon_tags_12 },
    { 0x9699c788, SUC_lexicon_tags_17 },
    { 0x2d39c789, SUC_lexicon_tags_1 },
    { 0x3ed9c789, SUC_lexicon_tags_17 },
    { 0x6039c787, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab9c791, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf19c795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1919c797, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba59c79e, SUC_lexicon_tags_26 },
    { 0x4bd9c79f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa319c7a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b79c7ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeb39c7ae, SUC_lexicon_tags_7 },
    { 0xde19c7af, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf9c7b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e79c7b8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0d9c7bc, SUC_lexicon_tags_17 },
    { 0xf8d9c7bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c79c7c0, SUC_lexicon_tags_14 },
    { 0xc099c7c0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1779c7c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb6d9c7c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc99c7cd, SUC_lexicon_tags_10 },
    { 0x4239c7ce, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab39c7d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d79c7dc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd159c7df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48b9c7e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x38d9c7e9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe79c7eb, SUC_lexicon_tags_25 },
    { 0xde99c7ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2299c7f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc79c7f7, SUC_lexicon_tags_12 },
    { 0xe8d9c7f8, SUC_lexicon_tags_7 },
    { 0x6e99c7f9, SUC_lexicon_tags_32 },
    { 0x82d9c7fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c99c7fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9c802, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc2f9c804, SUC_lexicon_tags_2 },
    { 0xd719c805, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd9c809, SUC_lexicon_tags_37 },
    { 0x8b19c80a, SUC_lexicon_tags_17 },
    { 0xab59c80b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5659c80d, SUC_lexicon_tags_37 },
    { 0xaf39c80d, SUC_lexicon_tags_15 },
    { 0x7a59c80f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe599c811, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8919c813, SUC_lexicon_tags_37 },
    { 0x24b9c814, SUC_lexicon_tags_17 },
    { 0xaa39c815, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc499c818, SUC_lexicon_tags_7 },
    { 0xafd9c818, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9c81c, SUC_lexicon_tags_48 },
    { 0x8d79c81d, SUC_lexicon_tags_7 },
    { 0x1c99c81e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb9c823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd799c82a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2959c82c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8039c82e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ad9c831, SUC_lexicon_tags_65 },
    { 0xc619c832, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda99c83c, SUC_lexicon_tags_12 },
    { 0x8af9c83d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d9c842, SUC_lexicon_tags_16 },
    { 0xcf59c842, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7659c845, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4f19c847, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe79c84b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4579c852, SUC_lexicon_tags_37 },
    { 0xd4d9c853, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7899c855, SUC_lexicon_tags_17 },
    { 0x6459c855, SUC_lexicon_tags_11 },
    { 0xb379c855, SUC_lexicon_tags_47 },
    { 0x7479c855, SUC_lexicon_tags_0 },
    { 0xced9c858, SUC_lexicon_tags_29 },
    { 0x679c85a, SUC_lexicon_tags_98 },
    { 0x45f9c856, SUC_lexicon_tags_46 },
    { 0xc7f9c85c, SUC_lexicon_tags_11 },
    { 0xa179c856, SUC_lexicon_tags_15 },
    { 0xa599c85e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3499c863, SUC_lexicon_tags_30 },
    { 0xc2f9c864, SUC_lexicon_tags_46 },
    { 0x3819c865, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe319c868, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf339c86d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9b9c86f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9b9c871, SUC_lexicon_tags_62 },
    { 0x7eb9c872, SUC_lexicon_tags_48 },
    { 0x7a79c873, SUC_lexicon_tags_74 },
    { 0xa379c873, SUC_lexicon_tags_49 },
    { 0x3cb9c872, SUC_lexicon_tags_19 },
    { 0xaf79c875, SUC_lexicon_tags_19 },
    { 0x39c877, SUC_lexicon_tags_7 },
    { 0x9399c876, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3999c87b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x8c19c87d, SUC_lexicon_tags_17 },
    { 0x34b9c87e, SUC_lexicon_tags_17 },
    { 0x3e39c87d, SUC_lexicon_tags_1 },
    { 0x6959c880, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa39c884, SUC_lexicon_tags_15 },
    { 0x5ff9c885, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6399c88c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db9c890, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfc59c892, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6d9c894, SUC_lexicon_tags_1 },
    { 0x7379c895, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4259c89b, SUC_lexicon_tags_17 },
    { 0x3139c89c, SUC_lexicon_tags_37 },
    { 0xb0f9c89d, SUC_lexicon_tags_17 },
    { 0xea59c89b, SUC_lexicon_tags_37 },
    { 0x2ab9c89c, SUC_lexicon_tags_26 },
    { 0xe219c8a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4119c8a4, SUC_lexicon_tags_1 },
    { 0xf1d9c8a4, SUC_lexicon_tags_8 },
    { 0xfaf9c8a6, SUC_lexicon_tags_43 },
    { 0x239c8a7, SUC_lexicon_tags_49 },
    { 0x62d9c8a7, SUC_lexicon_tags_15 },
    { 0xc6f9c8a5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b9c8ad, SUC_lexicon_tags_37 },
    { 0x1d59c8ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6119c8b1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd299c8b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ef9c8be, SUC_lexicon_tags_43 },
    { 0x2479c8bf, SUC_lexicon_tags_56 },
    { 0xa179c8c0, SUC_lexicon_tags_19 },
    { 0xc879c8c1, SUC_lexicon_tags_8 },
    { 0xc8f9c8c2, SUC_lexicon_tags_744 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359c8d2, SUC_lexicon_tags_15 },
    { 0xccb9c8d2, SUC_lexicon_tags_2 },
    { 0x47b9c8d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x79b9c8d6, SUC_lexicon_tags_60 },
    { 0xb1f9c8d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb099c8dc, SUC_lexicon_tags_2 },
    { 0x4299c8dc, SUC_lexicon_tags_7 },
    { 0x1ad9c8de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec79c8e2, SUC_lexicon_tags_14 },
    { 0x2639c8e2, SUC_lexicon_tags_0 },
    { 0xfeb9c8e4, SUC_lexicon_tags_25 },
    { 0x36f9c8e5, SUC_lexicon_tags_14 },
    { 0xa019c8e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3659c8ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f9c8ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9c8f2, SUC_lexicon_tags_15 },
    { 0x7a99c8f3, SUC_lexicon_tags_14 },
    { 0x8379c8f4, SUC_lexicon_tags_17 },
    { 0xbe59c8f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9c8f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe739c8fa, SUC_lexicon_tags_25 },
    { 0xc6d9c8fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1459c903, SUC_lexicon_tags_11 },
    { 0xf199c904, SUC_lexicon_tags_2 },
    { 0x9079c905, SUC_lexicon_tags_37 },
    { 0xcd99c906, SUC_lexicon_tags_21 },
    { 0x7e39c905, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7159c909, SUC_lexicon_tags_7 },
    { 0x6e39c90a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xc159c90d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b39c910, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1fd9c912, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb9c91a, SUC_lexicon_tags_11 },
    { 0xff79c91b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1019c91e, SUC_lexicon_tags_25 },
    { 0x7f59c91f, SUC_lexicon_tags_7 },
    { 0x8919c91e, SUC_lexicon_tags_22 },
    { 0x3ef9c91e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919c925, SUC_lexicon_tags_12 },
    { 0xe099c925, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1b79c928, SUC_lexicon_tags_45 },
    { 0x1959c929, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b99c92b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x47f9c92d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2239c930, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x37d9c932, SUC_lexicon_tags_10 },
    { 0x5a79c933, SUC_lexicon_tags_2 },
    { 0xe619c934, SUC_lexicon_tags_9 },
    { 0xaf99c935, SUC_lexicon_tags_25 },
    { 0x4cb9c936, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2299c939, SUC_lexicon_tags_7 },
    { 0x72d9c939, SUC_lexicon_tags_11 },
    { 0xf6d9c93b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec59c93f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7119c943, SUC_lexicon_tags_30 },
    { 0x1819c944, SUC_lexicon_tags_43 },
    { 0x7d59c945, SUC_lexicon_tags_27 },
    { 0x76b9c946, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb779c949, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb9c94e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x61b9c950, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9339c954, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eb9c959, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb9c95d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf5f9c95f, SUC_lexicon_tags_25 },
    { 0xc819c960, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd39c963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5159c96f, SUC_lexicon_tags_2 },
    { 0x55b9c970, SUC_lexicon_tags_23 },
    { 0x4cf9c970, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9719c976, SUC_lexicon_tags_55 },
    { 0x3e79c977, SUC_lexicon_tags_15 },
    { 0x3ad9c977, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd839c97a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ed9c980, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5999c982, SUC_lexicon_tags_14 },
    { 0x7bd9c983, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f9c986, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a59c988, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4139c98b, SUC_lexicon_tags_16 },
    { 0x9f19c98c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5659c98e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7659c994, SUC_lexicon_tags_11 },
    { 0x2b39c994, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c19c997, SUC_lexicon_tags_34 },
    { 0x3d9c998, SUC_lexicon_tags_37 },
    { 0xda59c999, SUC_lexicon_tags_12 },
    { 0xfa59c999, SUC_lexicon_tags_7 },
    { 0x87d9c998, SUC_lexicon_tags_23 },
    { 0x9bb9c99a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xae39c99f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9f59c9a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40b9c9a3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x81b9c9a5, SUC_lexicon_tags_27 },
    { 0x5439c9a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbd79c9a8, SUC_lexicon_tags_11 },
    { 0x2c59c9a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d59c9ac, SUC_lexicon_tags_1 },
    { 0x5779c9ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71b9c9af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78f9c9b4, SUC_lexicon_tags_39 },
    { 0xcb99c9b4, SUC_lexicon_tags_0 },
    { 0x5019c9b5, SUC_lexicon_tags_2 },
    { 0x9d59c9b6, SUC_lexicon_tags_27 },
    { 0x8ed9c9b8, SUC_lexicon_tags_11 },
    { 0xf759c9b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9c9bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d9c9c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2659c9c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd59c9c7, SUC_lexicon_tags_1 },
    { 0x3f99c9c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39c9cb, SUC_lexicon_tags_63 },
    { 0xb979c9cc, SUC_lexicon_tags_56 },
    { 0x5499c9cb, SUC_lexicon_tags_47 },
    { 0x3699c9ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8359c9d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f9c9dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6f9c9e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b9c9e9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2b99c9eb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x88b9c9ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb459c9f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b9c9f4, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe839c9f8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b9c9fb, SUC_lexicon_tags_26 },
    { 0x319c9fc, SUC_lexicon_tags_15 },
    { 0xed19c9fc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf9b9c9ff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa119ca03, SUC_lexicon_tags_17 },
    { 0xdb79ca04, SUC_lexicon_tags_37 },
    { 0x6cd9ca03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfeb9ca07, SUC_lexicon_tags_26 },
    { 0x4819ca08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86f9ca0a, SUC_lexicon_tags_42 },
    { 0xf939ca0b, SUC_lexicon_tags_45 },
    { 0x5679ca0c, SUC_lexicon_tags_63 },
    { 0x56f9ca0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2579ca12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8699ca16, SUC_lexicon_tags_25 },
    { 0x8739ca16, SUC_lexicon_tags_55 },
    { 0xd5f9ca16, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9ca1b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa619ca1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1799ca21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x639ca23, SUC_lexicon_tags_8 },
    { 0xad39ca24, SUC_lexicon_tags_95 },
    { 0xc99ca24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe179ca2f, SUC_lexicon_tags_14 },
    { 0x2c19ca30, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0xabd9ca32, SUC_lexicon_tags_37 },
    { 0x8619ca33, SUC_lexicon_tags_82 },
    { 0x92b9ca33, SUC_lexicon_tags_37 },
    { 0xbfb9ca34, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x18b9ca37, SUC_lexicon_tags_37 },
    { 0x7c99ca38, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f9ca3c, SUC_lexicon_tags_45 },
    { 0x1cb9ca3c, SUC_lexicon_tags_1 },
    { 0xac59ca3e, SUC_lexicon_tags_616 },
    { 0xe819ca3f, SUC_lexicon_tags_37 },
    { 0xc979ca3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf459ca49, SUC_lexicon_tags_25 },
    { 0xcb9ca4a, SUC_lexicon_tags_0 },
    { 0x299ca4a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b9ca53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1119ca55, SUC_lexicon_tags_2 },
    { 0xc2f9ca56, SUC_lexicon_tags_7 },
    { 0xd159ca57, SUC_lexicon_tags_28 },
    { 0x1b59ca58, SUC_lexicon_tags_29 },
    { 0x7e99ca59, SUC_lexicon_tags_1 },
    { 0x23d9ca59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d9ca5e, SUC_lexicon_tags_37 },
    { 0x40f9ca5f, SUC_lexicon_tags_37 },
    { 0xaef9ca5f, SUC_lexicon_tags_20 },
    { 0x86b9ca61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5459ca70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b9ca76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x30f9ca78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbfd9ca7a, SUC_lexicon_tags_17 },
    { 0xf719ca7b, SUC_lexicon_tags_12 },
    { 0x4159ca7b, SUC_lexicon_tags_23 },
    { 0x4c19ca7a, SUC_lexicon_tags_39 },
    { 0xd119ca7b, SUC_lexicon_tags_17 },
    { 0xbe99ca7f, SUC_lexicon_tags_11 },
    { 0xd4b9ca80, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b9ca84, SUC_lexicon_tags_45 },
    { 0x1f19ca85, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa19ca87, SUC_lexicon_tags_0 },
    { 0x8b99ca88, SUC_lexicon_tags_15 },
    { 0xf139ca88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff9ca8d, SUC_lexicon_tags_63 },
    { 0x9df9ca8e, SUC_lexicon_tags_19 },
    { 0x6999ca8e, SUC_lexicon_tags_23 },
    { 0x1319ca90, SUC_lexicon_tags_25 },
    { 0xfc39ca90, SUC_lexicon_tags_37 },
    { 0xa4f9ca90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19ca9d, SUC_lexicon_tags_23 },
    { 0x8cd9ca9d, SUC_lexicon_tags_10 },
    { 0x1eb9ca9d, SUC_lexicon_tags_11 },
    { 0xdd59ca9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54b9caa4, SUC_lexicon_tags_37 },
    { 0x3819caa4, SUC_lexicon_tags_1 },
    { 0x7dd9caa6, SUC_lexicon_tags_15 },
    { 0xb7b9caa6, SUC_lexicon_tags_47 },
    { 0x5519caa8, SUC_lexicon_tags_14 },
    { 0x6c79caa9, SUC_lexicon_tags_17 },
    { 0x2499caaa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3839caac, SUC_lexicon_tags_33 },
    { 0x80b9caad, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4959cab1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b9cab7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d9cabb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf059cabd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe819cac3, SUC_lexicon_tags_9 },
    { 0xa659cac4, SUC_lexicon_tags_39 },
    { 0x2c19cac5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9299cacb, SUC_lexicon_tags_11 },
    { 0x4879cacc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd9cacf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f9cad4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6859cad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e99cada, SUC_lexicon_tags_15 },
    { 0xf6d9cadb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6779cadd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d9cae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1499cae7, SUC_lexicon_tags_113 },
    { 0xa879cae8, SUC_lexicon_tags_11 },
    { 0xd759cae9, SUC_lexicon_tags_2 },
    { 0x25d9caea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x339caed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8159caf0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcbb9caf2, SUC_lexicon_tags_66 },
    { 0x9839caf3, SUC_lexicon_tags_14 },
    { 0x4cd9caf2, SUC_lexicon_tags_11 },
    { 0x61f9caf5, SUC_lexicon_tags_10 },
    { 0xec99caf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4679cafa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6e99cafc, SUC_lexicon_tags_37 },
    { 0x7939cafd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6319caff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef9cb09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26d9cb0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa7d9cb0f, SUC_lexicon_tags_15 },
    { 0xd3f9cb0f, SUC_lexicon_tags_11 },
    { 0x4639cb11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe9d9cb13, SUC_lexicon_tags_25 },
    { 0xc879cb14, SUC_lexicon_tags_53 },
    { 0x2079cb15, SUC_lexicon_tags_57 },
    { 0x4739cb16, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x33d9cb18, SUC_lexicon_tags_62 },
    { 0x839cb18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3579cb1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ad9cb1e, SUC_lexicon_tags_19 },
    { 0x70f9cb1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44d9cb23, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9579cb25, SUC_lexicon_tags_45 },
    { 0xa699cb26, SUC_lexicon_tags_11 },
    { 0x73d9cb27, SUC_lexicon_tags_4 },
    { 0x6979cb27, SUC_lexicon_tags_37 },
    { 0x7159cb25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x36d9cb2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5079cb2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc599cb35, SUC_lexicon_tags_8 },
    { 0xb459cb35, SUC_lexicon_tags_60 },
    { 0xef79cb36, SUC_lexicon_tags_1 },
    { 0x6e19cb38, SUC_lexicon_tags_35 },
    { 0x2699cb39, SUC_lexicon_tags_7 },
    { 0xe819cb39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb9cb40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc599cb44, SUC_lexicon_tags_49 },
    { 0x379cb45, SUC_lexicon_tags_11 },
    { 0x7339cb45, SUC_lexicon_tags_1 },
    { 0xdc79cb47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7259cb4a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1219cb4d, SUC_lexicon_tags_15 },
    { 0x6c19cb4d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9839cb5b, SUC_lexicon_tags_15 },
    { 0x3519cb5b, SUC_lexicon_tags_17 },
    { 0x5b9cb5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd9cb61, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5399cb64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde19cb67, SUC_lexicon_tags_2 },
    { 0x8a59cb68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81d9cb6f, SUC_lexicon_tags_61 },
    { 0xd359cb70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a59cb73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xae59cb76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d9cb7b, SUC_lexicon_tags_37 },
    { 0x31f9cb7c, SUC_lexicon_tags_2 },
    { 0xd4b9cb7d, SUC_lexicon_tags_4 },
    { 0x9cf9cb7d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xff39cb80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1259cb84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7299cb86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9cb89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea79cb8d, SUC_lexicon_tags_2 },
    { 0x7e19cb8d, SUC_lexicon_tags_37 },
    { 0x73d9cb8d, SUC_lexicon_tags_39 },
    { 0xad19cb8d, SUC_lexicon_tags_12 },
    { 0x51d9cb91, SUC_lexicon_tags_14 },
    { 0x4a19cb91, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd9cba0, SUC_lexicon_tags_13 },
    { 0xa2b9cba1, SUC_lexicon_tags_38 },
    { 0xb1f9cba2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xc479cba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1159cba6, SUC_lexicon_tags_2 },
    { 0xdb9cba6, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4499cbac, SUC_lexicon_tags_12 },
    { 0xfb79cbac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9539cbb2, SUC_lexicon_tags_60 },
    { 0xe6d9cbb3, SUC_lexicon_tags_79 },
    { 0xc699cbb3, SUC_lexicon_tags_1 },
    { 0xe1b9cbb5, SUC_lexicon_tags_47 },
    { 0xe159cbb6, SUC_lexicon_tags_25 },
    { 0x6c99cbb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f9cbba, SUC_lexicon_tags_7 },
    { 0xb979cbbb, SUC_lexicon_tags_17 },
    { 0x4b79cbbc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1359cbbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3699cbc0, SUC_lexicon_tags_7 },
    { 0xa279cbc1, SUC_lexicon_tags_25 },
    { 0xbb39cbc2, SUC_lexicon_tags_23 },
    { 0x439cbc3, SUC_lexicon_tags_10 },
    { 0xd259cbc4, SUC_lexicon_tags_17 },
    { 0xe9d9cbc5, SUC_lexicon_tags_28 },
    { 0x3f79cbc4, SUC_lexicon_tags_7 },
    { 0x32b9cbc6, SUC_lexicon_tags_25 },
    { 0xe4b9cbc5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x40d9cbcb, SUC_lexicon_tags_16 },
    { 0xa699cbcc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a99cbcf, SUC_lexicon_tags_37 },
    { 0x4f39cbd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe239cbd4, SUC_lexicon_tags_33 },
    { 0xa779cbd5, SUC_lexicon_tags_7 },
    { 0x4af9cbd5, SUC_lexicon_tags_55 },
    { 0xf639cbd5, SUC_lexicon_tags_49 },
    { 0xe899cbd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b9cbdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1779cbdd, SUC_lexicon_tags_39 },
    { 0x67d9cbde, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a19cbe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9f9cbe7, SUC_lexicon_tags_28 },
    { 0x1f9cbe8, SUC_lexicon_tags_14 },
    { 0xe59cbe7, SUC_lexicon_tags_38 },
    { 0xd479cbea, SUC_lexicon_tags_30 },
    { 0xded9cbea, SUC_lexicon_tags_11 },
    { 0x2439cbec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb459cbee, SUC_lexicon_tags_11 },
    { 0x1cf9cbef, SUC_lexicon_tags_30 },
    { 0x9d59cbf0, SUC_lexicon_tags_37 },
    { 0xc5d9cbf1, SUC_lexicon_tags_38 },
    { 0x3539cbf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe759cbf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdcf9cbf8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2699cbfc, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4479cc00, SUC_lexicon_tags_7 },
    { 0xc579cc01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8999cc07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10b9cc0a, SUC_lexicon_tags_8 },
    { 0xc219cc0b, SUC_lexicon_tags_25 },
    { 0xe7d9cc0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5879cc0f, SUC_lexicon_tags_60 },
    { 0x959cc10, SUC_lexicon_tags_1 },
    { 0xef19cc11, SUC_lexicon_tags_37 },
    { 0x87d9cc0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9679cc16, SUC_lexicon_tags_11 },
    { 0xf7b9cc17, SUC_lexicon_tags_13 },
    { 0x51f9cc18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7819cc1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe119cc21, SUC_lexicon_tags_2 },
    { 0xca99cc22, SUC_lexicon_tags_61 },
    { 0x3fd9cc23, SUC_lexicon_tags_2 },
    { 0x8b39cc22, SUC_lexicon_tags_1 },
    { 0x65b9cc24, SUC_lexicon_tags_2 },
    { 0x5719cc26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf219cc28, SUC_lexicon_tags_25 },
    { 0x319cc29, SUC_lexicon_tags_16 },
    { 0x1419cc2a, SUC_lexicon_tags_49 },
    { 0xce39cc2a, SUC_lexicon_tags_6 },
    { 0xc039cc2c, SUC_lexicon_tags_60 },
    { 0xe4f9cc2d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e39cc30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b59cc33, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc639cc38, SUC_lexicon_tags_6 },
    { 0xed99cc39, SUC_lexicon_tags_17 },
    { 0xfd39cc3a, SUC_lexicon_tags_38 },
    { 0x2a79cc39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf99cc3e, SUC_lexicon_tags_19 },
    { 0x1779cc3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f9cc42, SUC_lexicon_tags_43 },
    { 0x9ab9cc43, SUC_lexicon_tags_12 },
    { 0xd259cc44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4719cc47, SUC_lexicon_tags_57 },
    { 0x5419cc48, SUC_lexicon_tags_49 },
    { 0x85d9cc49, SUC_lexicon_tags_17 },
    { 0x5979cc4a, SUC_lexicon_tags_1444 },
    { 0xf359cc4a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f9cc50, SUC_lexicon_tags_30 },
    { 0x5079cc51, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe619cc53, SUC_lexicon_tags_37 },
    { 0x3bf9cc53, SUC_lexicon_tags_83 },
    { 0x6019cc55, SUC_lexicon_tags_27 },
    { 0x52b9cc53, SUC_lexicon_tags_60 },
    { 0x7959cc55, SUC_lexicon_tags_35 },
    { 0x8cd9cc54, SUC_lexicon_tags_14 },
    { 0x1699cc57, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfc79cc5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca59cc60, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x239cc63, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80b9cc6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd9cc72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef9cc75, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39cc78, SUC_lexicon_tags_68 },
    { 0x62f9cc79, SUC_lexicon_tags_17 },
    { 0x30f9cc7a, SUC_lexicon_tags_25 },
    { 0x85d9cc78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f39cc7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1779cc7f, SUC_lexicon_tags_25 },
    { 0x98b9cc80, SUC_lexicon_tags_37 },
    { 0x6f9cc81, SUC_lexicon_tags_37 },
    { 0xed79cc80, SUC_lexicon_tags_1 },
    { 0xe5f9cc81, SUC_lexicon_tags_15 },
    { 0x3059cc81, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1af9cc87, SUC_lexicon_tags_15 },
    { 0xec79cc87, SUC_lexicon_tags_7 },
    { 0xca39cc89, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd5f9cc8b, SUC_lexicon_tags_12 },
    { 0x8ed9cc8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb419cc8e, SUC_lexicon_tags_2 },
    { 0x2ed9cc8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb9cc92, SUC_lexicon_tags_48 },
    { 0x7219cc92, SUC_lexicon_tags_7 },
    { 0x6d19cc94, SUC_lexicon_tags_23 },
    { 0x279cc95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc7b9cc97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3099cc9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7119cca2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a39cca8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d59ccad, SUC_lexicon_tags_7 },
    { 0x8db9ccad, SUC_lexicon_tags_37 },
    { 0x85f9ccaf, SUC_lexicon_tags_23 },
    { 0xc859ccb0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6259ccb4, SUC_lexicon_tags_25 },
    { 0x1db9ccb5, SUC_lexicon_tags_47 },
    { 0x12b9ccb4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499ccba, SUC_lexicon_tags_7 },
    { 0x1f19ccbb, SUC_lexicon_tags_12 },
    { 0x8959ccbc, SUC_lexicon_tags_37 },
    { 0xca39ccbd, SUC_lexicon_tags_14 },
    { 0xb5b9ccbe, SUC_lexicon_tags_25 },
    { 0x3539ccbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9599ccc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdad9ccc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd9ccc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d79ccce, SUC_lexicon_tags_13 },
    { 0xa0f9cccf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e19ccd3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3c99ccd5, SUC_lexicon_tags_7 },
    { 0x3b9ccd5, SUC_lexicon_tags_2 },
    { 0x2f79ccd7, SUC_lexicon_tags_15 },
    { 0x2db9ccd8, SUC_lexicon_tags_41 },
    { 0xac59ccd9, SUC_lexicon_tags_37 },
    { 0x5319ccda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d9ccdd, SUC_lexicon_tags_37 },
    { 0x779ccde, SUC_lexicon_tags_25 },
    { 0xea99ccde, SUC_lexicon_tags_30 },
    { 0x5299cce0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x8d19cce2, SUC_lexicon_tags_45 },
    { 0xa439cce3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfef9cce5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfab9cce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8539cceb, SUC_lexicon_tags_0 },
    { 0x6c79ccec, SUC_lexicon_tags_25 },
    { 0x7139cced, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed79ccf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f9ccf9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3af9ccfb, SUC_lexicon_tags_12 },
    { 0x42b9ccfc, SUC_lexicon_tags_45 },
    { 0x7f19ccfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7579ccff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9cd07, SUC_lexicon_tags_21 },
    { 0xaf39cd08, SUC_lexicon_tags_37 },
    { 0xa0b9cd09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4799cd0f, SUC_lexicon_tags_1 },
    { 0x3899cd0f, SUC_lexicon_tags_11 },
    { 0x9c99cd10, SUC_lexicon_tags_8 },
    { 0xf159cd12, SUC_lexicon_tags_40 },
    { 0xe679cd13, SUC_lexicon_tags_47 },
    { 0x5419cd13, SUC_lexicon_tags_31 },
    { 0xa4f9cd15, SUC_lexicon_tags_191 },
    { 0xddd9cd15, SUC_lexicon_tags_27 },
    { 0x56f9cd17, SUC_lexicon_tags_23 },
    { 0xea39cd18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb79cd1e, SUC_lexicon_tags_2 },
    { 0x29d9cd1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed79cd22, SUC_lexicon_tags_98 },
    { 0x6239cd22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b9cd29, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x44f9cd2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1479cd32, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f99cd37, SUC_lexicon_tags_12 },
    { 0xc079cd38, SUC_lexicon_tags_25 },
    { 0x1599cd37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd19cd3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf039cd40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f59cd43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2839cd47, SUC_lexicon_tags_33 },
    { 0xfdf9cd47, SUC_lexicon_tags_25 },
    { 0xda19cd48, SUC_lexicon_tags_25 },
    { 0x8799cd4a, SUC_lexicon_tags_37 },
    { 0xc239cd48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ab9cd4e, SUC_lexicon_tags_15 },
    { 0x93b9cd4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ef9cd57, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59cd5e, SUC_lexicon_tags_3 },
    { 0xf599cd5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6659cd64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86d9cd66, SUC_lexicon_tags_87 },
    { 0xc979cd67, SUC_lexicon_tags_10 },
    { 0x9139cd68, SUC_lexicon_tags_7 },
    { 0xc219cd68, SUC_lexicon_tags_25 },
    { 0x6ab9cd66, SUC_lexicon_tags_2 },
    { 0x6879cd68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ef9cd6d, SUC_lexicon_tags_23 },
    { 0x4639cd6e, SUC_lexicon_tags_14 },
    { 0xb5f9cd6f, SUC_lexicon_tags_15 },
    { 0x6219cd70, SUC_lexicon_tags_4 },
    { 0x2e39cd71, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xefb9cd74, SUC_lexicon_tags_1 },
    { 0x6fb9cd75, SUC_lexicon_tags_17 },
    { 0x23d9cd74, SUC_lexicon_tags_36 },
    { 0x6139cd74, SUC_lexicon_tags_66 },
    { 0x2579cd78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9879cd7d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x419cd7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66b9cd87, SUC_lexicon_tags_2 },
    { 0xa499cd88, SUC_lexicon_tags_11 },
    { 0x3c79cd89, SUC_lexicon_tags_38 },
    { 0x7659cd89, SUC_lexicon_tags_26 },
    { 0x80f9cd8b, SUC_lexicon_tags_55 },
    { 0xe8f9cd8b, SUC_lexicon_tags_7 },
    { 0x1dd9cd8d, SUC_lexicon_tags_87 },
    { 0x6ab9cd8d, SUC_lexicon_tags_0 },
    { 0xbb99cd8b, SUC_lexicon_tags_15 },
    { 0x6a59cd89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2999cd9a, SUC_lexicon_tags_11 },
    { 0x3bb9cd9a, SUC_lexicon_tags_29 },
    { 0x48f9cd9c, SUC_lexicon_tags_15 },
    { 0xfad9cd9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d99cda3, SUC_lexicon_tags_14 },
    { 0x94d9cda4, SUC_lexicon_tags_2 },
    { 0xf819cda5, SUC_lexicon_tags_2 },
    { 0x3439cda5, SUC_lexicon_tags_16 },
    { 0xfb79cda7, SUC_lexicon_tags_17 },
    { 0x4039cda7, SUC_lexicon_tags_11 },
    { 0x44b9cda5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4819cdab, SUC_lexicon_tags_27 },
    { 0x36f9cdab, SUC_lexicon_tags_14 },
    { 0x5fb9cdad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3219cdaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d9cdb1, SUC_lexicon_tags_1 },
    { 0x7fb9cdb2, SUC_lexicon_tags_25 },
    { 0x6079cdb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd999cdb8, SUC_lexicon_tags_47 },
    { 0x8699cdb8, SUC_lexicon_tags_21 },
    { 0x4299cdba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9639cdbc, SUC_lexicon_tags_33 },
    { 0x67f9cdbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa759cdbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf679cdc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9cdc8, SUC_lexicon_tags_17 },
    { 0x88d9cdc9, SUC_lexicon_tags_39 },
    { 0x5bf9cdca, SUC_lexicon_tags_15 },
    { 0xc059cdca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1ab9cdcd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda79cdd7, SUC_lexicon_tags_47 },
    { 0xa79cdd8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb2b9cdda, SUC_lexicon_tags_10 },
    { 0x1f19cdda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1099cddd, SUC_lexicon_tags_7 },
    { 0xec99cddd, SUC_lexicon_tags_17 },
    { 0xf39cddf, SUC_lexicon_tags_29 },
    { 0x4cb9cddf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd9cdeb, SUC_lexicon_tags_11 },
    { 0x19f9cdec, SUC_lexicon_tags_33 },
    { 0x3599cdeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2d9cdf3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xca59cdf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ff9cdf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7159cdfe, SUC_lexicon_tags_25 },
    { 0xca39cdfe, SUC_lexicon_tags_25 },
    { 0xbdd9ce00, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b9ce07, SUC_lexicon_tags_4 },
    { 0x8f39ce08, SUC_lexicon_tags_9 },
    { 0x20b9ce09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bb9ce0b, SUC_lexicon_tags_25 },
    { 0x9bb9ce0c, SUC_lexicon_tags_1 },
    { 0xadb9ce0d, SUC_lexicon_tags_2 },
    { 0xde39ce0b, SUC_lexicon_tags_1 },
    { 0xc7b9ce0d, SUC_lexicon_tags_8 },
    { 0xf939ce10, SUC_lexicon_tags_45 },
    { 0x219ce10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd379ce14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf179ce18, SUC_lexicon_tags_107 },
    { 0xe0d9ce18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf9ce1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3b9ce1e, SUC_lexicon_tags_7 },
    { 0x159ce1f, SUC_lexicon_tags_7 },
    { 0x78f9ce20, SUC_lexicon_tags_12 },
    { 0x9f39ce20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc699ce24, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f9ce2c, SUC_lexicon_tags_29 },
    { 0x9039ce2d, SUC_lexicon_tags_11 },
    { 0x10d9ce2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8039ce31, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f19ce36, SUC_lexicon_tags_25 },
    { 0x8ef9ce37, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5199ce3a, SUC_lexicon_tags_15 },
    { 0x6579ce3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b9ce42, SUC_lexicon_tags_39 },
    { 0x13d9ce43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a79ce4a, SUC_lexicon_tags_37 },
    { 0x13f9ce4b, SUC_lexicon_tags_34 },
    { 0x3d59ce4c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x25f9ce4e, SUC_lexicon_tags_21 },
    { 0x30f9ce4f, SUC_lexicon_tags_25 },
    { 0x2a19ce50, SUC_lexicon_tags_28 },
    { 0x2d39ce51, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f59ce54, SUC_lexicon_tags_10 },
    { 0x7459ce54, SUC_lexicon_tags_9 },
    { 0x72b9ce56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb79ce58, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc259ce5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3f9ce5d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4259ce5f, SUC_lexicon_tags_17 },
    { 0x7959ce5f, SUC_lexicon_tags_63 },
    { 0x2319ce61, SUC_lexicon_tags_26 },
    { 0x1159ce61, SUC_lexicon_tags_7 },
    { 0xa2d9ce5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6119ce65, SUC_lexicon_tags_17 },
    { 0xd3b9ce66, SUC_lexicon_tags_3 },
    { 0xd939ce66, SUC_lexicon_tags_15 },
    { 0xebb9ce67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x92d9ce6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2939ce6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4af9ce75, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b19ce77, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x16d9ce79, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99ce7d, SUC_lexicon_tags_15 },
    { 0x7f99ce7d, SUC_lexicon_tags_7 },
    { 0x3819ce7f, SUC_lexicon_tags_15 },
    { 0xa379ce80, SUC_lexicon_tags_12 },
    { 0xee19ce81, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2759ce83, SUC_lexicon_tags_15 },
    { 0x1119ce83, SUC_lexicon_tags_612 },
    { 0x1f19ce85, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9419ce87, SUC_lexicon_tags_11 },
    { 0xa939ce88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f99ce91, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec19ce93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccb9ce9a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3679ce9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb0b9ce9e, SUC_lexicon_tags_29 },
    { 0xb2b9ce9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbcf9cea1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3d9cea3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c79cea9, SUC_lexicon_tags_14 },
    { 0xfaf9cea9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d99cead, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4219ceaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b79ceb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd19ceb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2159cebb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3499cebe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2659cec3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7a19cec5, SUC_lexicon_tags_45 },
    { 0xbef9cec5, SUC_lexicon_tags_4 },
    { 0xd4f9cec7, SUC_lexicon_tags_25 },
    { 0xd8f9cec5, SUC_lexicon_tags_12 },
    { 0x60b9cec9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa359cecb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7259ced0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4579ced2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3339ced9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd599cedc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc219cee0, SUC_lexicon_tags_0 },
    { 0xfd9cee0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf419cee3, SUC_lexicon_tags_25 },
    { 0x19d9cee4, SUC_lexicon_tags_38 },
    { 0x9399cee4, SUC_lexicon_tags_57 },
    { 0xfd19cee6, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b9ceeb, SUC_lexicon_tags_25 },
    { 0xc699ceec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1839cef0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe879cef3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa399cef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd9cef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93f9cefb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3139ceff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b9cf04, SUC_lexicon_tags_47 },
    { 0x7599cf05, SUC_lexicon_tags_1 },
    { 0x8d99cf05, SUC_lexicon_tags_27 },
    { 0x4ad9cf07, SUC_lexicon_tags_65 },
    { 0x62f9cf05, SUC_lexicon_tags_7 },
    { 0xf1d9cf06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8f9cf0b, SUC_lexicon_tags_17 },
    { 0x22d9cf0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d99cf15, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd8d9cf17, SUC_lexicon_tags_1 },
    { 0xe7f9cf18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf9cf1e, SUC_lexicon_tags_25 },
    { 0x9539cf1e, SUC_lexicon_tags_25 },
    { 0x7179cf1e, SUC_lexicon_tags_6 },
    { 0x7a59cf1e, SUC_lexicon_tags_15 },
    { 0xa139cf20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f39cf2d, SUC_lexicon_tags_1 },
    { 0x7ef9cf2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd419cf30, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4839cf38, SUC_lexicon_tags_32 },
    { 0x8919cf38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b9cf3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b9cf41, SUC_lexicon_tags_7 },
    { 0x9db9cf42, SUC_lexicon_tags_992 },
    { 0xe859cf43, SUC_lexicon_tags_82 },
    { 0xd739cf43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db9cf48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb519cf4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cb9cf4d, SUC_lexicon_tags_37 },
    { 0x4579cf4e, SUC_lexicon_tags_14 },
    { 0x1c79cf4e, SUC_lexicon_tags_37 },
    { 0x58b9cf4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f9cf53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5699cf56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7319cf58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50f9cf5b, SUC_lexicon_tags_12 },
    { 0x4db9cf5c, SUC_lexicon_tags_37 },
    { 0xa239cf5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96d9cf61, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeef9cf66, SUC_lexicon_tags_45 },
    { 0x9ef9cf67, SUC_lexicon_tags_47 },
    { 0x65f9cf68, SUC_lexicon_tags_7 },
    { 0x6fd9cf69, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5cf9cf6b, SUC_lexicon_tags_9 },
    { 0x7c19cf6c, SUC_lexicon_tags_27 },
    { 0xd9d9cf6c, SUC_lexicon_tags_12 },
    { 0x2659cf6e, SUC_lexicon_tags_63 },
    { 0x1399cf6f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319cf74, SUC_lexicon_tags_0 },
    { 0x7b79cf75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3059cf79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9239cf81, SUC_lexicon_tags_1 },
    { 0x3239cf81, SUC_lexicon_tags_19 },
    { 0x2679cf83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc99cf85, SUC_lexicon_tags_19 },
    { 0xa299cf86, SUC_lexicon_tags_84 },
    { 0x5039cf87, SUC_lexicon_tags_47 },
    { 0xc6f9cf88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd879cf8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc059cf8d, SUC_lexicon_tags_69 },
    { 0x3499cf8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x82b9cf90, SUC_lexicon_tags_2 },
    { 0xc2b9cf91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b9cf98, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f9cf9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x22d9cf9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2819cfa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc059cfa4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f9cfa9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68f9cfaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe539cfb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff9cfb9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5af9cfc8, SUC_lexicon_tags_15 },
    { 0x6ff9cfc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeef9cfcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e39cfcf, SUC_lexicon_tags_65 },
    { 0xc79cfd0, SUC_lexicon_tags_77 },
    { 0x7ab9cfd0, SUC_lexicon_tags_37 },
    { 0x4439cfd2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a39cfd6, SUC_lexicon_tags_6 },
    { 0x9c19cfd7, SUC_lexicon_tags_7 },
    { 0xbdd9cfd8, SUC_lexicon_tags_17 },
    { 0xc4d9cfd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc979cfdb, SUC_lexicon_tags_1 },
    { 0x739cfdb, SUC_lexicon_tags_31 },
    { 0x1d9cfdc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x359cfdf, SUC_lexicon_tags_45 },
    { 0x9919cfe0, SUC_lexicon_tags_37 },
    { 0x2199cfe0, SUC_lexicon_tags_15 },
    { 0x3819cfe0, SUC_lexicon_tags_63 },
    { 0x9dd9cfe3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb099cfe5, SUC_lexicon_tags_103 },
    { 0xcd79cfe6, SUC_lexicon_tags_2 },
    { 0x8af9cfe6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd099cfe9, SUC_lexicon_tags_17 },
    { 0x7279cfea, SUC_lexicon_tags_12 },
    { 0x57d9cfeb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eb9cfef, SUC_lexicon_tags_2 },
    { 0xcf79cfef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab39cff2, SUC_lexicon_tags_0 },
    { 0x3ad9cff2, SUC_lexicon_tags_1 },
    { 0xf039cff4, SUC_lexicon_tags_17 },
    { 0x2a79cff5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f9cff9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefd9cffb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a99cffe, SUC_lexicon_tags_12 },
    { 0x1dd9cfff, SUC_lexicon_tags_43 },
    { 0x4999cfff, SUC_lexicon_tags_15 },
    { 0x5a59d001, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12b9d003, SUC_lexicon_tags_198 },
    { 0x8d39d003, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7199d006, SUC_lexicon_tags_26 },
    { 0x6fd9d006, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf799d00b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42d9d00e, SUC_lexicon_tags_26 },
    { 0x2f39d00f, SUC_lexicon_tags_33 },
    { 0x6599d010, SUC_lexicon_tags_9 },
    { 0x5759d010, SUC_lexicon_tags_7 },
    { 0x8439d011, SUC_lexicon_tags_15 },
    { 0xe979d013, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2259d015, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe179d017, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2739d01c, SUC_lexicon_tags_62 },
    { 0xf699d01d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7999d01f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xddd9d021, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca99d027, SUC_lexicon_tags_25 },
    { 0x12d9d028, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6199d02c, SUC_lexicon_tags_0 },
    { 0x39b9d02c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a59d035, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xa519d038, SUC_lexicon_tags_4 },
    { 0x1459d039, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea79d03e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf39d040, SUC_lexicon_tags_48 },
    { 0xf5b9d041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4259d043, SUC_lexicon_tags_23 },
    { 0xf819d043, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3439d047, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11d9d04b, SUC_lexicon_tags_252 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af9d04e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc079d050, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d9d056, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xea79d058, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9b19d05a, SUC_lexicon_tags_17 },
    { 0xd1b9d05b, SUC_lexicon_tags_10 },
    { 0x49d9d05b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6119d061, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55b9d063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce19d066, SUC_lexicon_tags_70 },
    { 0x5bb9d067, SUC_lexicon_tags_69 },
    { 0x40d9d067, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d19d070, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d39d074, SUC_lexicon_tags_60 },
    { 0xdfd9d075, SUC_lexicon_tags_65 },
    { 0x9bd9d076, SUC_lexicon_tags_13 },
    { 0xd5b9d076, SUC_lexicon_tags_38 },
    { 0x2bf9d078, SUC_lexicon_tags_2 },
    { 0x2019d078, SUC_lexicon_tags_40 },
    { 0xb59d076, SUC_lexicon_tags_55 },
    { 0x3a99d075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1939d07d, SUC_lexicon_tags_52 },
    { 0xc6b9d07d, SUC_lexicon_tags_15 },
    { 0x7e39d07f, SUC_lexicon_tags_55 },
    { 0xd859d07f, SUC_lexicon_tags_11 },
    { 0xffb9d080, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6639d085, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfd9d087, SUC_lexicon_tags_15 },
    { 0xd359d088, SUC_lexicon_tags_26 },
    { 0x42d9d087, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc019d08b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb879d08e, SUC_lexicon_tags_19 },
    { 0x7cb9d08f, SUC_lexicon_tags_23 },
    { 0x8d79d08e, SUC_lexicon_tags_19 },
    { 0x4c99d08f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5fb9d093, SUC_lexicon_tags_37 },
    { 0x6b19d094, SUC_lexicon_tags_26 },
    { 0x44b9d095, SUC_lexicon_tags_45 },
    { 0x7a59d093, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2759d09d, SUC_lexicon_tags_12 },
    { 0xba79d09e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5579d0a0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xb159d0a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d99d0a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f99d0ad, SUC_lexicon_tags_46 },
    { 0xdcb9d0ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b9d0b6, SUC_lexicon_tags_15 },
    { 0x69f9d0b7, SUC_lexicon_tags_15 },
    { 0x9ed9d0b8, SUC_lexicon_tags_84 },
    { 0xab59d0b9, SUC_lexicon_tags_11 },
    { 0xe279d0b9, SUC_lexicon_tags_49 },
    { 0xebb9d0bb, SUC_lexicon_tags_53 },
    { 0x5bf9d0bb, SUC_lexicon_tags_7 },
    { 0xa559d0bd, SUC_lexicon_tags_49 },
    { 0xaf99d0bd, SUC_lexicon_tags_11 },
    { 0xfef9d0bf, SUC_lexicon_tags_14 },
    { 0x9519d0bd, SUC_lexicon_tags_31 },
    { 0xd239d0b9, SUC_lexicon_tags_8 },
    { 0x1dd9d0b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7119d0c4, SUC_lexicon_tags_17 },
    { 0xd8f9d0c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6239d0c8, SUC_lexicon_tags_35 },
    { 0xdbb9d0c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb59d0ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb9d0d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe59d0db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf9d0e0, SUC_lexicon_tags_1 },
    { 0x1039d0e0, SUC_lexicon_tags_14 },
    { 0xc579d0e0, SUC_lexicon_tags_37 },
    { 0xeaf9d0e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b9d0e6, SUC_lexicon_tags_1561 },
    { 0, NULL },
    { 0, NULL },
    { 0x6179d0e9, SUC_lexicon_tags_30 },
    { 0x6579d0ea, SUC_lexicon_tags_28 },
    { 0xeaf9d0ea, SUC_lexicon_tags_63 },
    { 0x78d9d0ec, SUC_lexicon_tags_2 },
    { 0x8ad9d0ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba39d0f0, SUC_lexicon_tags_37 },
    { 0x2219d0f1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9d0f4, SUC_lexicon_tags_11 },
    { 0x8619d0f5, SUC_lexicon_tags_12 },
    { 0xb4f9d0f5, SUC_lexicon_tags_2 },
    { 0x2919d0f7, SUC_lexicon_tags_11 },
    { 0xa999d0f8, SUC_lexicon_tags_37 },
    { 0x6039d0f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6979d0fd, SUC_lexicon_tags_46 },
    { 0xf779d0fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1839d102, SUC_lexicon_tags_11 },
    { 0x7b99d103, SUC_lexicon_tags_25 },
    { 0xe0f9d104, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2539d106, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6b9d108, SUC_lexicon_tags_15 },
    { 0xc219d109, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc659d10f, SUC_lexicon_tags_23 },
    { 0x74b9d10f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99d113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4b9d115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b9d11a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde59d11e, SUC_lexicon_tags_507 },
    { 0xd059d11f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad99d121, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d9d124, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59d12a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d9d12e, SUC_lexicon_tags_47 },
    { 0x81d9d12e, SUC_lexicon_tags_0 },
    { 0xdfd9d12f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd9d133, SUC_lexicon_tags_17 },
    { 0x6979d134, SUC_lexicon_tags_26 },
    { 0x7779d135, SUC_lexicon_tags_87 },
    { 0xa179d135, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f9d139, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af9d13c, SUC_lexicon_tags_49 },
    { 0xda79d13d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b9d141, SUC_lexicon_tags_31 },
    { 0xeb39d142, SUC_lexicon_tags_23 },
    { 0x58d9d143, SUC_lexicon_tags_11 },
    { 0xf099d144, SUC_lexicon_tags_31 },
    { 0x7579d142, SUC_lexicon_tags_7 },
    { 0xe39d146, SUC_lexicon_tags_2 },
    { 0xc859d147, SUC_lexicon_tags_4 },
    { 0x199d142, SUC_lexicon_tags_15 },
    { 0xd3d9d141, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b99d14e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x24d9d150, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a19d156, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf9d15a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379d160, SUC_lexicon_tags_27 },
    { 0x4539d160, SUC_lexicon_tags_60 },
    { 0x8a39d161, SUC_lexicon_tags_1 },
    { 0x1779d163, SUC_lexicon_tags_47 },
    { 0x68d9d164, SUC_lexicon_tags_53 },
    { 0x4159d162, SUC_lexicon_tags_1 },
    { 0xc979d163, SUC_lexicon_tags_13 },
    { 0xbfb9d163, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5299d169, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xea99d16c, SUC_lexicon_tags_11 },
    { 0x5f19d16c, SUC_lexicon_tags_37 },
    { 0x539d16e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b79d174, SUC_lexicon_tags_17 },
    { 0xfcb9d175, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e99d177, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x77d9d179, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe059d17d, SUC_lexicon_tags_2 },
    { 0x66d9d17d, SUC_lexicon_tags_1 },
    { 0xb3f9d17d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb759d182, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8879d18b, SUC_lexicon_tags_12 },
    { 0x6459d18c, SUC_lexicon_tags_1 },
    { 0x579d18d, SUC_lexicon_tags_15 },
    { 0x79d18e, SUC_lexicon_tags_29 },
    { 0x3c59d18f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e39d192, SUC_lexicon_tags_6 },
    { 0x3319d193, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaad9d195, SUC_lexicon_tags_7 },
    { 0x3399d195, SUC_lexicon_tags_17 },
    { 0xc479d197, SUC_lexicon_tags_31 },
    { 0x9479d195, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x76d9d19b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb799d19e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a59d1a3, SUC_lexicon_tags_27 },
    { 0x90b9d1a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5359d1a6, SUC_lexicon_tags_28 },
    { 0xfc19d1a7, SUC_lexicon_tags_9 },
    { 0x8839d1a6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a19d1ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6399d1b3, SUC_lexicon_tags_31 },
    { 0xda19d1b3, SUC_lexicon_tags_87 },
    { 0xcc59d1b5, SUC_lexicon_tags_25 },
    { 0xb1d9d1b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x96d9d1b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d9d1c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc719d1c2, SUC_lexicon_tags_14 },
    { 0xb359d1c2, SUC_lexicon_tags_23 },
    { 0x7b9d1c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9679d1c6, SUC_lexicon_tags_46 },
    { 0x7459d1c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6959d1c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c79d1cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5059d1cf, SUC_lexicon_tags_2 },
    { 0x3a79d1d0, SUC_lexicon_tags_47 },
    { 0xf5b9d1cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa979d1da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd159d1de, SUC_lexicon_tags_17 },
    { 0x7a59d1df, SUC_lexicon_tags_38 },
    { 0xc0d9d1e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d9d1e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa119d1e8, SUC_lexicon_tags_15 },
    { 0xdb59d1e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9d1f2, SUC_lexicon_tags_12 },
    { 0xff9d1f2, SUC_lexicon_tags_37 },
    { 0x3c99d1f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x97f9d1f6, SUC_lexicon_tags_1537 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb799d1fa, SUC_lexicon_tags_37 },
    { 0xb6d9d1fb, SUC_lexicon_tags_45 },
    { 0x3759d1fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2139d204, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe319d207, SUC_lexicon_tags_19 },
    { 0x6cb9d208, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe619d20a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79d20e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d19d211, SUC_lexicon_tags_17 },
    { 0xcb19d212, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f9d219, SUC_lexicon_tags_25 },
    { 0x2319d21a, SUC_lexicon_tags_23 },
    { 0xaf19d21b, SUC_lexicon_tags_16 },
    { 0xf979d21a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd9d221, SUC_lexicon_tags_15 },
    { 0xbe59d222, SUC_lexicon_tags_0 },
    { 0xe939d222, SUC_lexicon_tags_47 },
    { 0x70b9d224, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe519d226, SUC_lexicon_tags_63 },
    { 0xae59d227, SUC_lexicon_tags_60 },
    { 0x79d226, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f9d22c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe579d22f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea19d231, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb8b9d233, SUC_lexicon_tags_14 },
    { 0xc519d234, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1259d237, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc939d23f, SUC_lexicon_tags_15 },
    { 0xe59d23f, SUC_lexicon_tags_9 },
    { 0x13f9d23f, SUC_lexicon_tags_4 },
    { 0x2799d242, SUC_lexicon_tags_15 },
    { 0x4f59d243, SUC_lexicon_tags_17 },
    { 0x6659d243, SUC_lexicon_tags_33 },
    { 0x5a59d244, SUC_lexicon_tags_33 },
    { 0xd279d246, SUC_lexicon_tags_25 },
    { 0x5c99d244, SUC_lexicon_tags_11 },
    { 0xfbf9d247, SUC_lexicon_tags_37 },
    { 0x9a39d249, SUC_lexicon_tags_56 },
    { 0x5039d247, SUC_lexicon_tags_13 },
    { 0x39d24b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a59d24e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc859d250, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f9d254, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4419d257, SUC_lexicon_tags_87 },
    { 0x32b9d258, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf9d25c, SUC_lexicon_tags_60 },
    { 0xf519d25d, SUC_lexicon_tags_1 },
    { 0xbd79d25e, SUC_lexicon_tags_15 },
    { 0xa259d25e, SUC_lexicon_tags_37 },
    { 0xafb9d260, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f79d264, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b9d269, SUC_lexicon_tags_2 },
    { 0xcdf9d26a, SUC_lexicon_tags_17 },
    { 0x9959d26b, SUC_lexicon_tags_7 },
    { 0x8e59d26c, SUC_lexicon_tags_25 },
    { 0x79d26d, SUC_lexicon_tags_7 },
    { 0x4979d26e, SUC_lexicon_tags_47 },
    { 0xbc99d26c, SUC_lexicon_tags_17 },
    { 0x7cd9d270, SUC_lexicon_tags_29 },
    { 0x9f99d26f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93f9d27b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb739d284, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7fb9d286, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8539d28a, SUC_lexicon_tags_7 },
    { 0xd6b9d28b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d9d28f, SUC_lexicon_tags_4 },
    { 0x4ef9d290, SUC_lexicon_tags_49 },
    { 0x4459d291, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1459d293, SUC_lexicon_tags_15 },
    { 0x75b9d293, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb39d296, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159d29a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9d2ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79b9d2b7, SUC_lexicon_tags_16 },
    { 0x5699d2b8, SUC_lexicon_tags_21 },
    { 0x5e79d2b8, SUC_lexicon_tags_17 },
    { 0x8b9d2ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f9d2c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf9d2cc, SUC_lexicon_tags_30 },
    { 0x7ab9d2cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x86d9d2d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd899d2d2, SUC_lexicon_tags_19 },
    { 0xb479d2d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f9d2d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee19d2db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3099d2de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f9d2e3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda99d2e7, SUC_lexicon_tags_65 },
    { 0x2c79d2e7, SUC_lexicon_tags_68 },
    { 0x1519d2e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5499d2f1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xce19d2f3, SUC_lexicon_tags_11 },
    { 0x7859d2f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4b99d2f6, SUC_lexicon_tags_47 },
    { 0xc339d2f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf79d2fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c79d305, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9d30a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d59d312, SUC_lexicon_tags_23 },
    { 0xad9d313, SUC_lexicon_tags_8 },
    { 0xc6d9d313, SUC_lexicon_tags_0 },
    { 0xc099d312, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xced9d318, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c19d31b, SUC_lexicon_tags_66 },
    { 0xcaf9d31c, SUC_lexicon_tags_65 },
    { 0x5eb9d31d, SUC_lexicon_tags_27 },
    { 0x7639d31b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x82d9d321, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff9d324, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f9d326, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5d9d328, SUC_lexicon_tags_7 },
    { 0xd19d329, SUC_lexicon_tags_17 },
    { 0xa4b9d32a, SUC_lexicon_tags_47 },
    { 0xb359d32b, SUC_lexicon_tags_19 },
    { 0x43f9d32c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x49f9d32e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5f19d330, SUC_lexicon_tags_15 },
    { 0x6ff9d330, SUC_lexicon_tags_2 },
    { 0xd479d332, SUC_lexicon_tags_7 },
    { 0x18f9d330, SUC_lexicon_tags_14 },
    { 0x899d333, SUC_lexicon_tags_46 },
    { 0x30d9d335, SUC_lexicon_tags_25 },
    { 0x1f79d335, SUC_lexicon_tags_7 },
    { 0x3fd9d337, SUC_lexicon_tags_47 },
    { 0x9af9d338, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ad9d33b, SUC_lexicon_tags_8 },
    { 0x6a99d33b, SUC_lexicon_tags_23 },
    { 0x8d39d33d, SUC_lexicon_tags_0 },
    { 0x359d33e, SUC_lexicon_tags_25 },
    { 0xce79d33b, SUC_lexicon_tags_14 },
    { 0x4ff9d340, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb519d343, SUC_lexicon_tags_19 },
    { 0x74b9d343, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1459d346, SUC_lexicon_tags_7 },
    { 0x14b9d346, SUC_lexicon_tags_1 },
    { 0xdf59d348, SUC_lexicon_tags_60 },
    { 0xaed9d348, SUC_lexicon_tags_12 },
    { 0x8739d348, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9d34e, SUC_lexicon_tags_10 },
    { 0x7119d34e, SUC_lexicon_tags_7 },
    { 0xb7d9d34e, SUC_lexicon_tags_0 },
    { 0x1c39d350, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe879d359, SUC_lexicon_tags_17 },
    { 0xf419d359, SUC_lexicon_tags_25 },
    { 0xef79d35b, SUC_lexicon_tags_2 },
    { 0xa279d35c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9b99d35e, SUC_lexicon_tags_47 },
    { 0xd8b9d35f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x59d9d361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3699d363, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xb539d365, SUC_lexicon_tags_38 },
    { 0x1e99d366, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a19d368, SUC_lexicon_tags_47 },
    { 0xfd19d369, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe439d36c, SUC_lexicon_tags_60 },
    { 0xedb9d36d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b9d371, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d9d376, SUC_lexicon_tags_39 },
    { 0x50f9d376, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f9d37a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc179d37c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca79d381, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4239d383, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1239d387, SUC_lexicon_tags_7 },
    { 0x1179d388, SUC_lexicon_tags_23 },
    { 0xd9d9d389, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c39d38d, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56f9d392, SUC_lexicon_tags_158 },
    { 0xa759d393, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8439d398, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f9d39c, SUC_lexicon_tags_1 },
    { 0xee59d39d, SUC_lexicon_tags_11 },
    { 0x6e99d39e, SUC_lexicon_tags_14 },
    { 0x9699d39f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4599d3a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4619d3a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1279d3a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f9d3ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2739d3b0, SUC_lexicon_tags_7 },
    { 0xf1f9d3b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4739d3b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2019d3b6, SUC_lexicon_tags_30 },
    { 0x2899d3b7, SUC_lexicon_tags_17 },
    { 0xaf79d3b6, SUC_lexicon_tags_235 },
    { 0xd019d3b8, SUC_lexicon_tags_15 },
    { 0xe1d9d3b8, SUC_lexicon_tags_17 },
    { 0x9479d3b9, SUC_lexicon_tags_1 },
    { 0xf559d3b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xec79d3be, SUC_lexicon_tags_17 },
    { 0x9f19d3bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b9d3c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6dd9d3c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfbd9d3c7, SUC_lexicon_tags_37 },
    { 0xdd59d3c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc319d3cf, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa1d9d3d1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6599d3da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7999d3dc, SUC_lexicon_tags_6 },
    { 0x3ab9d3dc, SUC_lexicon_tags_47 },
    { 0x9df9d3de, SUC_lexicon_tags_17 },
    { 0x8099d3df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3f9d3e6, SUC_lexicon_tags_14 },
    { 0xff59d3e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ad9d3e9, SUC_lexicon_tags_19 },
    { 0x6b79d3ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1519d3f0, SUC_lexicon_tags_15 },
    { 0x4e19d3f1, SUC_lexicon_tags_15 },
    { 0xd3f9d3f1, SUC_lexicon_tags_12 },
    { 0x7439d3f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d9d3f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b9d3fa, SUC_lexicon_tags_28 },
    { 0xc439d3fb, SUC_lexicon_tags_12 },
    { 0x1e79d3fb, SUC_lexicon_tags_57 },
    { 0x55b9d3fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb339d401, SUC_lexicon_tags_10 },
    { 0x4f9d402, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4519d404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd9d409, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdab9d40f, SUC_lexicon_tags_37 },
    { 0x74d9d40f, SUC_lexicon_tags_25 },
    { 0x96d9d411, SUC_lexicon_tags_2 },
    { 0xa119d411, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8cd9d414, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa7f9d416, SUC_lexicon_tags_17 },
    { 0x1259d416, SUC_lexicon_tags_27 },
    { 0x51d9d418, SUC_lexicon_tags_1 },
    { 0xd1d9d416, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2af9d41b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5139d41e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cf9d423, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a99d426, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd1f9d428, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb399d42a, SUC_lexicon_tags_25 },
    { 0xebd9d42b, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x14b9d42d, SUC_lexicon_tags_10 },
    { 0xd0b9d42e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b9d432, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9d436, SUC_lexicon_tags_63 },
    { 0x2399d437, SUC_lexicon_tags_47 },
    { 0x6cb9d438, SUC_lexicon_tags_10 },
    { 0x6979d436, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9d43c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f9d43f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf439d442, SUC_lexicon_tags_900 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4019d44d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b9d451, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa059d453, SUC_lexicon_tags_7 },
    { 0xaff9d454, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2259d456, SUC_lexicon_tags_2 },
    { 0xf239d456, SUC_lexicon_tags_2 },
    { 0x2019d458, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18b9d45c, SUC_lexicon_tags_17 },
    { 0x6979d45d, SUC_lexicon_tags_7 },
    { 0x2579d45e, SUC_lexicon_tags_17 },
    { 0xa7d9d45e, SUC_lexicon_tags_69 },
    { 0xcf59d45f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d9d463, SUC_lexicon_tags_25 },
    { 0x9ad9d463, SUC_lexicon_tags_11 },
    { 0x80d9d464, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe339d467, SUC_lexicon_tags_17 },
    { 0x2b39d467, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d39d474, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43b9d478, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc279d47c, SUC_lexicon_tags_48 },
    { 0x38f9d47d, SUC_lexicon_tags_15 },
    { 0x69f9d47d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4679d480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e99d488, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5939d48f, SUC_lexicon_tags_49 },
    { 0x87b9d48f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa339d493, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b9d496, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119d49a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1479d49c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9519d4a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eb9d4a9, SUC_lexicon_tags_12 },
    { 0xf239d4a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe119d4af, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x6619d4b1, SUC_lexicon_tags_25 },
    { 0x3a79d4b2, SUC_lexicon_tags_18 },
    { 0xe239d4b3, SUC_lexicon_tags_7 },
    { 0x8879d4b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b59d4b7, SUC_lexicon_tags_11 },
    { 0x7df9d4b8, SUC_lexicon_tags_12 },
    { 0xfa99d4b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf9d4bf, SUC_lexicon_tags_60 },
    { 0x5e99d4bf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda19d4c4, SUC_lexicon_tags_8 },
    { 0xc0f9d4c4, SUC_lexicon_tags_4 },
    { 0x3cb9d4c6, SUC_lexicon_tags_12 },
    { 0x7419d4c4, SUC_lexicon_tags_39 },
    { 0x44d9d4c5, SUC_lexicon_tags_27 },
    { 0x4d59d4c8, SUC_lexicon_tags_17 },
    { 0x68d9d4c6, SUC_lexicon_tags_2 },
    { 0xea99d4cb, SUC_lexicon_tags_47 },
    { 0x2779d4cc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d19d4d0, SUC_lexicon_tags_25 },
    { 0xb119d4d0, SUC_lexicon_tags_27 },
    { 0x15d9d4d2, SUC_lexicon_tags_6 },
    { 0x82d9d4d3, SUC_lexicon_tags_10 },
    { 0xfbf9d4d4, SUC_lexicon_tags_9 },
    { 0x64f9d4d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc859d4d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9b9d4d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d9d4de, SUC_lexicon_tags_27 },
    { 0x1919d4de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e39d4eb, SUC_lexicon_tags_0 },
    { 0xaa19d4ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf9d4f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae39d4fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599d4fe, SUC_lexicon_tags_8 },
    { 0xa119d4ff, SUC_lexicon_tags_74 },
    { 0xb0d9d500, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f9d50b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e39d50d, SUC_lexicon_tags_43 },
    { 0xb599d50e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2b9d511, SUC_lexicon_tags_25 },
    { 0xfc79d512, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7659d51b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3b9d51d, SUC_lexicon_tags_25 },
    { 0x1679d51e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2239d527, SUC_lexicon_tags_3 },
    { 0xc6d9d527, SUC_lexicon_tags_56 },
    { 0x5379d528, SUC_lexicon_tags_25 },
    { 0x9019d52a, SUC_lexicon_tags_17 },
    { 0x1039d529, SUC_lexicon_tags_27 },
    { 0xcbb9d529, SUC_lexicon_tags_1 },
    { 0x8039d527, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc219d532, SUC_lexicon_tags_11 },
    { 0xac79d532, SUC_lexicon_tags_10 },
    { 0xaf99d534, SUC_lexicon_tags_10 },
    { 0x6539d532, SUC_lexicon_tags_8 },
    { 0x4959d532, SUC_lexicon_tags_28 },
    { 0xa999d537, SUC_lexicon_tags_1 },
    { 0x8279d537, SUC_lexicon_tags_25 },
    { 0xba79d536, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x3599d53c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8eb9d53e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7779d540, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3259d543, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe0f9d545, SUC_lexicon_tags_45 },
    { 0xb819d546, SUC_lexicon_tags_63 },
    { 0x26b9d546, SUC_lexicon_tags_6 },
    { 0x6739d548, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x38f9d54a, SUC_lexicon_tags_19 },
    { 0x8b39d54b, SUC_lexicon_tags_16 },
    { 0xecf9d54a, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e19d54f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b9d556, SUC_lexicon_tags_101 },
    { 0x7859d557, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff99d55a, SUC_lexicon_tags_25 },
    { 0x6af9d55b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b9d560, SUC_lexicon_tags_43 },
    { 0x2e79d561, SUC_lexicon_tags_42 },
    { 0x8579d560, SUC_lexicon_tags_4 },
    { 0xbbf9d561, SUC_lexicon_tags_63 },
    { 0x39f9d561, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a99d568, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f79d56b, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xf559d56d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7679d573, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b9d579, SUC_lexicon_tags_26 },
    { 0x79d57a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f99d57c, SUC_lexicon_tags_11 },
    { 0x1039d57d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2959d57f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d9d583, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf19d585, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab9d588, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3339d58c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b79d58e, SUC_lexicon_tags_30 },
    { 0x1f59d58f, SUC_lexicon_tags_14 },
    { 0x31b9d58f, SUC_lexicon_tags_14 },
    { 0xe1d9d591, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc019d593, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6719d596, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1259d59e, SUC_lexicon_tags_19 },
    { 0xf959d59f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf9d5a2, SUC_lexicon_tags_37 },
    { 0x48d9d5a2, SUC_lexicon_tags_84 },
    { 0xf479d5a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaed9d5a6, SUC_lexicon_tags_47 },
    { 0x96b9d5a7, SUC_lexicon_tags_49 },
    { 0xe5d9d5a8, SUC_lexicon_tags_65 },
    { 0x8bf9d5a7, SUC_lexicon_tags_31 },
    { 0x5079d5aa, SUC_lexicon_tags_2 },
    { 0x3919d5ab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb159d5ae, SUC_lexicon_tags_27 },
    { 0xb0f9d5af, SUC_lexicon_tags_47 },
    { 0xa919d5b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1799d5b8, SUC_lexicon_tags_37 },
    { 0xbcf9d5b8, SUC_lexicon_tags_310 },
    { 0x41f9d5b9, SUC_lexicon_tags_33 },
    { 0x8ed9d5ba, SUC_lexicon_tags_16 },
    { 0x4f9d5bc, SUC_lexicon_tags_12 },
    { 0xa59d5bc, SUC_lexicon_tags_48 },
    { 0x63b9d5bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f99d5c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb9d5c8, SUC_lexicon_tags_12 },
    { 0xe9b9d5c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf39d5cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x77b9d5cf, SUC_lexicon_tags_28 },
    { 0x31b9d5d0, SUC_lexicon_tags_28 },
    { 0xacb9d5cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea79d5d5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf59d5dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a99d5de, SUC_lexicon_tags_30 },
    { 0x4419d5df, SUC_lexicon_tags_7 },
    { 0xc1d9d5df, SUC_lexicon_tags_9 },
    { 0xf79d5e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9359d5e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ed9d5e6, SUC_lexicon_tags_87 },
    { 0xf5d9d5e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb039d5e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d99d5eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c59d5f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc439d5f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f39d5f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2d9d5f7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd59d5fc, SUC_lexicon_tags_7 },
    { 0xe8b9d5fd, SUC_lexicon_tags_12 },
    { 0xb059d5fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa319d60b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecd9d610, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ad9d613, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf559d61d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9d621, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b19d624, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9b59d626, SUC_lexicon_tags_14 },
    { 0x2119d627, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8599d62b, SUC_lexicon_tags_12 },
    { 0x8c79d62c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9219d634, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439d63b, SUC_lexicon_tags_15 },
    { 0x5619d63c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5559d63f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2319d644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18f9d64d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b9d655, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbd9d65a, SUC_lexicon_tags_17 },
    { 0x979d65b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x82f9d65d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8299d662, SUC_lexicon_tags_12 },
    { 0xd0b9d662, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d59d667, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59d66a, SUC_lexicon_tags_2 },
    { 0x8639d66a, SUC_lexicon_tags_27 },
    { 0x2199d66b, SUC_lexicon_tags_0 },
    { 0xc579d66d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca39d674, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4279d67b, SUC_lexicon_tags_27 },
    { 0x51d9d67c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3499d67e, SUC_lexicon_tags_83 },
    { 0xbd19d67e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed79d682, SUC_lexicon_tags_25 },
    { 0x8cf9d683, SUC_lexicon_tags_25 },
    { 0xdd19d684, SUC_lexicon_tags_15 },
    { 0x9f59d684, SUC_lexicon_tags_1 },
    { 0x7f9d686, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9b9d688, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb039d68c, SUC_lexicon_tags_34 },
    { 0x1439d68d, SUC_lexicon_tags_2 },
    { 0x9599d68d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa599d693, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f9d696, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf9d699, SUC_lexicon_tags_38 },
    { 0xeeb9d699, SUC_lexicon_tags_12 },
    { 0x2e19d69b, SUC_lexicon_tags_10 },
    { 0x84f9d69a, SUC_lexicon_tags_25 },
    { 0xe459d69c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e79d6a5, SUC_lexicon_tags_11 },
    { 0x89f9d6a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48f9d6ab, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa39d6b2, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f9d6b6, SUC_lexicon_tags_23 },
    { 0x27f9d6b6, SUC_lexicon_tags_29 },
    { 0x8e79d6b7, SUC_lexicon_tags_12 },
    { 0x1119d6b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb339d6bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b9d6ce, SUC_lexicon_tags_37 },
    { 0xf6d9d6ce, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f9d6d3, SUC_lexicon_tags_63 },
    { 0x1d79d6d3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d6d7, SUC_lexicon_tags_46 },
    { 0x9159d6d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46f9d6de, SUC_lexicon_tags_14 },
    { 0x1cd9d6df, SUC_lexicon_tags_80 },
    { 0x1379d6de, SUC_lexicon_tags_151 },
    { 0xc6f9d6df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8d9d6e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6159d6eb, SUC_lexicon_tags_40 },
    { 0x4759d6eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7319d6f1, SUC_lexicon_tags_49 },
    { 0x5399d6f2, SUC_lexicon_tags_11 },
    { 0x93b9d6f3, SUC_lexicon_tags_7 },
    { 0xe19d6f3, SUC_lexicon_tags_9 },
    { 0x2a99d6f5, SUC_lexicon_tags_29 },
    { 0x7c19d6f6, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f9d6f9, SUC_lexicon_tags_11 },
    { 0xc5b9d6fa, SUC_lexicon_tags_8 },
    { 0x3b9d6fb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8119d6fd, SUC_lexicon_tags_17 },
    { 0x6f9d6fd, SUC_lexicon_tags_12 },
    { 0x1db9d6ff, SUC_lexicon_tags_14 },
    { 0x2459d6fd, SUC_lexicon_tags_14 },
    { 0xd619d701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x939d704, SUC_lexicon_tags_7 },
    { 0x5619d705, SUC_lexicon_tags_24 },
    { 0xa8d9d704, SUC_lexicon_tags_25 },
    { 0x1639d704, SUC_lexicon_tags_14 },
    { 0x96d9d704, SUC_lexicon_tags_17 },
    { 0x4c79d709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe579d70d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d9d710, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8b9d717, SUC_lexicon_tags_25 },
    { 0x8999d718, SUC_lexicon_tags_45 },
    { 0xac99d719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa659d71d, SUC_lexicon_tags_15 },
    { 0xf59d71e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf859d723, SUC_lexicon_tags_9 },
    { 0xe3f9d723, SUC_lexicon_tags_1 },
    { 0x5c39d723, SUC_lexicon_tags_25 },
    { 0x759d723, SUC_lexicon_tags_1 },
    { 0x32d9d727, SUC_lexicon_tags_683 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9d72a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f99d736, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa6d9d738, SUC_lexicon_tags_6 },
    { 0x5839d739, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4279d73f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9919d743, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x839d745, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f9d748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc259d74c, SUC_lexicon_tags_8 },
    { 0xc999d74d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f9d756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad19d75a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x1f79d75c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9419d75e, SUC_lexicon_tags_2 },
    { 0x2ff9d75e, SUC_lexicon_tags_37 },
    { 0xe8d9d760, SUC_lexicon_tags_9 },
    { 0xa079d761, SUC_lexicon_tags_2 },
    { 0x6199d760, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x25d9d764, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62d9d767, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19d769, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x51b9d76b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6479d76f, SUC_lexicon_tags_17 },
    { 0xc1d9d76f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x239d773, SUC_lexicon_tags_26 },
    { 0x8979d774, SUC_lexicon_tags_104 },
    { 0x8f19d775, SUC_lexicon_tags_7 },
    { 0xb0b9d774, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd259d778, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b9d782, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a59d784, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x9df9d786, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5519d795, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52f9d79a, SUC_lexicon_tags_5 },
    { 0xab79d79b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4f9d79e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2d9d7a1, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa139d7a7, SUC_lexicon_tags_17 },
    { 0x6159d7a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f9d7ab, SUC_lexicon_tags_2 },
    { 0x67d9d7ac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1379d7ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda79d7b0, SUC_lexicon_tags_10 },
    { 0x6539d7b1, SUC_lexicon_tags_23 },
    { 0xd0f9d7b2, SUC_lexicon_tags_15 },
    { 0xbd59d7b2, SUC_lexicon_tags_52 },
    { 0x6479d7b2, SUC_lexicon_tags_85 },
    { 0x6439d7b3, SUC_lexicon_tags_7 },
    { 0xc7d9d7b3, SUC_lexicon_tags_13 },
    { 0x8159d7b3, SUC_lexicon_tags_26 },
    { 0x2079d7b8, SUC_lexicon_tags_17 },
    { 0x37b9d7b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x699d7bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9659d7be, SUC_lexicon_tags_45 },
    { 0xf799d7bf, SUC_lexicon_tags_25 },
    { 0x11f9d7bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b59d7c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f9d7cc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d9d7d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf319d7d6, SUC_lexicon_tags_1143 },
    { 0xdd59d7d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd39d7da, SUC_lexicon_tags_15 },
    { 0xf3f9d7db, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39d7de, SUC_lexicon_tags_47 },
    { 0x3219d7df, SUC_lexicon_tags_2 },
    { 0x2ab9d7df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4599d7e4, SUC_lexicon_tags_25 },
    { 0xd519d7e4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8319d7e8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa559d7ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9759d7ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3479d7f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f9d7f6, SUC_lexicon_tags_10 },
    { 0x7079d7f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6139d7f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfef9d7fc, SUC_lexicon_tags_11 },
    { 0x3299d7fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6719d802, SUC_lexicon_tags_17 },
    { 0xc459d803, SUC_lexicon_tags_26 },
    { 0x2cf9d803, SUC_lexicon_tags_43 },
    { 0xabb9d803, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d99d807, SUC_lexicon_tags_25 },
    { 0x7999d807, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8699d80b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2299d80e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x59b9d810, SUC_lexicon_tags_25 },
    { 0xa279d811, SUC_lexicon_tags_33 },
    { 0x50d9d810, SUC_lexicon_tags_11 },
    { 0xffb9d813, SUC_lexicon_tags_38 },
    { 0xbbb9d814, SUC_lexicon_tags_38 },
    { 0xacd9d815, SUC_lexicon_tags_12 },
    { 0x72b9d816, SUC_lexicon_tags_77 },
    { 0xd0d9d817, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfd9d819, SUC_lexicon_tags_28 },
    { 0x3a99d81a, SUC_lexicon_tags_57 },
    { 0xc159d81b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d39d828, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6019d82f, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999d833, SUC_lexicon_tags_17 },
    { 0x11d9d833, SUC_lexicon_tags_14 },
    { 0x1c99d835, SUC_lexicon_tags_1 },
    { 0xeb59d835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f9d83b, SUC_lexicon_tags_43 },
    { 0x2579d83c, SUC_lexicon_tags_31 },
    { 0x2d79d83d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1059d841, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7799d843, SUC_lexicon_tags_12 },
    { 0xfe99d843, SUC_lexicon_tags_7 },
    { 0x1139d845, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939d84c, SUC_lexicon_tags_1 },
    { 0x7e79d84c, SUC_lexicon_tags_63 },
    { 0x1079d84e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f9d851, SUC_lexicon_tags_43 },
    { 0x2579d852, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc99d856, SUC_lexicon_tags_44 },
    { 0xfab9d857, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc99d85b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1739d85d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c39d861, SUC_lexicon_tags_49 },
    { 0x7619d862, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc379d864, SUC_lexicon_tags_35 },
    { 0xfaf9d865, SUC_lexicon_tags_6 },
    { 0xc479d865, SUC_lexicon_tags_30 },
    { 0x6a19d867, SUC_lexicon_tags_0 },
    { 0xa079d866, SUC_lexicon_tags_25 },
    { 0x76d9d866, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc99d86c, SUC_lexicon_tags_25 },
    { 0x1b79d86d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8959d86f, SUC_lexicon_tags_6 },
    { 0xeb39d870, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaf9d873, SUC_lexicon_tags_6 },
    { 0xa5f9d874, SUC_lexicon_tags_25 },
    { 0x8699d874, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54b9d877, SUC_lexicon_tags_25 },
    { 0x3b19d878, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xde39d87b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb9d881, SUC_lexicon_tags_17 },
    { 0x45f9d882, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66f9d88b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0b9d88d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359d893, SUC_lexicon_tags_24 },
    { 0xc439d894, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a79d896, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2a19d898, SUC_lexicon_tags_54 },
    { 0xcb39d899, SUC_lexicon_tags_25 },
    { 0xbcb9d89a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xbfb9d89c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9579d89e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9259d8a0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x19b9d8a2, SUC_lexicon_tags_56 },
    { 0xddb9d8a3, SUC_lexicon_tags_19 },
    { 0x2c39d8a4, SUC_lexicon_tags_1 },
    { 0xab59d8a4, SUC_lexicon_tags_60 },
    { 0x61d9d8a6, SUC_lexicon_tags_27 },
    { 0xe3f9d8a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3459d8a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1259d8ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ed9d8af, SUC_lexicon_tags_25 },
    { 0x1679d8b0, SUC_lexicon_tags_25 },
    { 0xd319d8b1, SUC_lexicon_tags_30 },
    { 0x46b9d8b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b9d8b6, SUC_lexicon_tags_37 },
    { 0xf5d9d8b6, SUC_lexicon_tags_1 },
    { 0xbb19d8b8, SUC_lexicon_tags_2 },
    { 0xb099d8b9, SUC_lexicon_tags_306 },
    { 0xba99d8ba, SUC_lexicon_tags_46 },
    { 0xc59d8ba, SUC_lexicon_tags_23 },
    { 0x20f9d8bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc699d8be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb9d8c3, SUC_lexicon_tags_11 },
    { 0xc079d8c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed59d8c6, SUC_lexicon_tags_25 },
    { 0xc399d8c6, SUC_lexicon_tags_37 },
    { 0x6539d8c6, SUC_lexicon_tags_11 },
    { 0x5e19d8c9, SUC_lexicon_tags_11 },
    { 0x7899d8ca, SUC_lexicon_tags_37 },
    { 0x25b9d8cb, SUC_lexicon_tags_65 },
    { 0xfd59d8cc, SUC_lexicon_tags_25 },
    { 0xdd9d8cc, SUC_lexicon_tags_75 },
    { 0x539d8cd, SUC_lexicon_tags_31 },
    { 0x3339d8cf, SUC_lexicon_tags_15 },
    { 0x64f9d8cb, SUC_lexicon_tags_37 },
    { 0x3719d8c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf019d8d3, SUC_lexicon_tags_37 },
    { 0xb439d8d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f19d8db, SUC_lexicon_tags_28 },
    { 0x14b9d8dc, SUC_lexicon_tags_25 },
    { 0x70f9d8dd, SUC_lexicon_tags_7 },
    { 0x4df9d8de, SUC_lexicon_tags_77 },
    { 0x8ef9d8df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x54f9d8e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe399d8e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1ff9d8e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef19d8ed, SUC_lexicon_tags_7 },
    { 0x2499d8ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2ff9d8f0, SUC_lexicon_tags_23 },
    { 0x7119d8f0, SUC_lexicon_tags_1341 },
    { 0x119d8f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa439d8f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a99d8f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cd9d8fb, SUC_lexicon_tags_68 },
    { 0x7e19d8fb, SUC_lexicon_tags_25 },
    { 0xd919d8fb, SUC_lexicon_tags_12 },
    { 0x96b9d8fb, SUC_lexicon_tags_68 },
    { 0x9619d8ff, SUC_lexicon_tags_65 },
    { 0x3799d900, SUC_lexicon_tags_17 },
    { 0x2199d8fd, SUC_lexicon_tags_12 },
    { 0x33f9d902, SUC_lexicon_tags_37 },
    { 0x73d9d902, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6419d907, SUC_lexicon_tags_68 },
    { 0x8199d907, SUC_lexicon_tags_26 },
    { 0xc919d908, SUC_lexicon_tags_14 },
    { 0x6259d90a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd379d90f, SUC_lexicon_tags_17 },
    { 0x2199d90f, SUC_lexicon_tags_48 },
    { 0xbf39d911, SUC_lexicon_tags_6 },
    { 0x1dd9d912, SUC_lexicon_tags_25 },
    { 0xf599d911, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d99d917, SUC_lexicon_tags_60 },
    { 0x2e19d917, SUC_lexicon_tags_14 },
    { 0x219d919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa999d91d, SUC_lexicon_tags_47 },
    { 0xeb59d91e, SUC_lexicon_tags_6 },
    { 0x77f9d91f, SUC_lexicon_tags_17 },
    { 0xa0b9d920, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6919d926, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x14b9d929, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcf9d92b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2b9d92d, SUC_lexicon_tags_15 },
    { 0x5339d92e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2139d93a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf139d93e, SUC_lexicon_tags_14 },
    { 0xd419d93f, SUC_lexicon_tags_28 },
    { 0x139d940, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf9d943, SUC_lexicon_tags_7 },
    { 0x53d9d944, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x8c99d946, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb739d94a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd9d955, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3399d957, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc419d959, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ed9d95c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x13f9d95e, SUC_lexicon_tags_28 },
    { 0xdf19d95f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe739d962, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa199d965, SUC_lexicon_tags_27 },
    { 0x6399d966, SUC_lexicon_tags_11 },
    { 0xbb79d966, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd9d96c, SUC_lexicon_tags_31 },
    { 0xf7d9d96c, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0x78f9d96f, SUC_lexicon_tags_60 },
    { 0x5b59d970, SUC_lexicon_tags_33 },
    { 0x3279d971, SUC_lexicon_tags_11 },
    { 0xcd39d96f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9d975, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f59d97d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd39d97f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6719d987, SUC_lexicon_tags_19 },
    { 0x1e39d988, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59d9d991, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd19d993, SUC_lexicon_tags_47 },
    { 0xfdb9d993, SUC_lexicon_tags_15 },
    { 0x70d9d994, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2139d997, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba59d99b, SUC_lexicon_tags_1 },
    { 0x3dd9d99c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c79d99f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11f9d9a1, SUC_lexicon_tags_19 },
    { 0xebd9d9a2, SUC_lexicon_tags_1 },
    { 0xe139d9a3, SUC_lexicon_tags_17 },
    { 0x2ab9d9a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c39d9a9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf939d9ab, SUC_lexicon_tags_79 },
    { 0xf039d9ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a79d9af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf39d9b2, SUC_lexicon_tags_23 },
    { 0xa5b9d9b3, SUC_lexicon_tags_15 },
    { 0x119d9b2, SUC_lexicon_tags_17 },
    { 0xd819d9b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3419d9b7, SUC_lexicon_tags_38 },
    { 0x39b9d9b8, SUC_lexicon_tags_17 },
    { 0xe4f9d9b9, SUC_lexicon_tags_49 },
    { 0x1019d9ba, SUC_lexicon_tags_8 },
    { 0xfa39d9bb, SUC_lexicon_tags_61 },
    { 0x4bf9d9b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeeb9d9be, SUC_lexicon_tags_11 },
    { 0x2379d9bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc199d9ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x72b9d9cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9d9d1, SUC_lexicon_tags_1 },
    { 0x19b9d9d2, SUC_lexicon_tags_56 },
    { 0x8319d9d3, SUC_lexicon_tags_45 },
    { 0x9c19d9d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f19d9d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9d9db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x38d9d9dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8419d9df, SUC_lexicon_tags_11 },
    { 0x199d9e0, SUC_lexicon_tags_9 },
    { 0x4d99d9e0, SUC_lexicon_tags_978 },
    { 0x6e99d9e0, SUC_lexicon_tags_49 },
    { 0x3319d9e1, SUC_lexicon_tags_3 },
    { 0x98f9d9e4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3fd9d9e6, SUC_lexicon_tags_45 },
    { 0xb079d9e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5d9d9e9, SUC_lexicon_tags_37 },
    { 0x4f59d9ea, SUC_lexicon_tags_45 },
    { 0x4059d9e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fd9d9ed, SUC_lexicon_tags_10 },
    { 0x2439d9ee, SUC_lexicon_tags_28 },
    { 0x25b9d9ef, SUC_lexicon_tags_43 },
    { 0x4259d9ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b9d9f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf9d9f7, SUC_lexicon_tags_11 },
    { 0xc0f9d9f8, SUC_lexicon_tags_30 },
    { 0x3bb9d9f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8cf9d9fb, SUC_lexicon_tags_87 },
    { 0x3499d9fc, SUC_lexicon_tags_1 },
    { 0xa559d9fc, SUC_lexicon_tags_17 },
    { 0x54f9d9fd, SUC_lexicon_tags_49 },
    { 0xba19d9ff, SUC_lexicon_tags_11 },
    { 0x1d19d9fb, SUC_lexicon_tags_31 },
    { 0xd8d9da01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9559da05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9da0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1519da11, SUC_lexicon_tags_8 },
    { 0xc5f9da12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b9da1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2239da1c, SUC_lexicon_tags_1 },
    { 0xf059da1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x23b9da20, SUC_lexicon_tags_30 },
    { 0x4cd9da20, SUC_lexicon_tags_7 },
    { 0xfd99da22, SUC_lexicon_tags_38 },
    { 0x4839da22, SUC_lexicon_tags_19 },
    { 0x3199da24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d39da2a, SUC_lexicon_tags_17 },
    { 0x8a19da2b, SUC_lexicon_tags_1 },
    { 0x4cb9da2b, SUC_lexicon_tags_7 },
    { 0x4b9da2d, SUC_lexicon_tags_11 },
    { 0xa979da2d, SUC_lexicon_tags_45 },
    { 0x279da2c, SUC_lexicon_tags_7 },
    { 0x9359da30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb39da33, SUC_lexicon_tags_11 },
    { 0x3d39da33, SUC_lexicon_tags_17 },
    { 0x4d79da33, SUC_lexicon_tags_17 },
    { 0xfcd9da36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f9da39, SUC_lexicon_tags_25 },
    { 0xee19da3a, SUC_lexicon_tags_16 },
    { 0xd239da3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe879da3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf659da40, SUC_lexicon_tags_37 },
    { 0x7e39da41, SUC_lexicon_tags_23 },
    { 0x2819da42, SUC_lexicon_tags_15 },
    { 0x3119da43, SUC_lexicon_tags_2 },
    { 0x1919da43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa99da46, SUC_lexicon_tags_15 },
    { 0xb3b9da47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe419da49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a19da4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd019da4e, SUC_lexicon_tags_26 },
    { 0xf5b9da4f, SUC_lexicon_tags_12 },
    { 0xe5b9da4f, SUC_lexicon_tags_14 },
    { 0x4519da51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6d9da56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5939da59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8f99da5b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7499da60, SUC_lexicon_tags_7 },
    { 0x27d9da60, SUC_lexicon_tags_37 },
    { 0xd0f9da62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5719da64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa79da69, SUC_lexicon_tags_1303 },
    { 0x7459da6a, SUC_lexicon_tags_3 },
    { 0xe3f9da6a, SUC_lexicon_tags_25 },
    { 0xb239da6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c99da6f, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9939da74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7ed9da76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd9da79, SUC_lexicon_tags_47 },
    { 0x9819da79, SUC_lexicon_tags_1 },
    { 0x1519da7a, SUC_lexicon_tags_47 },
    { 0xe2b9da79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7939da83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b59da8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6719da8e, SUC_lexicon_tags_55 },
    { 0x4419da8f, SUC_lexicon_tags_33 },
    { 0x1839da8e, SUC_lexicon_tags_14 },
    { 0xbf19da8f, SUC_lexicon_tags_87 },
    { 0x51d9da92, SUC_lexicon_tags_12 },
    { 0xe839da93, SUC_lexicon_tags_12 },
    { 0x8219da94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe479da9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec19da9f, SUC_lexicon_tags_29 },
    { 0x1d9daa0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1b9daa2, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xde59daa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bb9daa6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf459daa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39daac, SUC_lexicon_tags_14 },
    { 0x84d9daad, SUC_lexicon_tags_26 },
    { 0x42d9daae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1919dab2, SUC_lexicon_tags_17 },
    { 0xa879dab3, SUC_lexicon_tags_37 },
    { 0xa139dab3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f9dac0, SUC_lexicon_tags_37 },
    { 0x78f9dac0, SUC_lexicon_tags_27 },
    { 0xa879dac1, SUC_lexicon_tags_55 },
    { 0x4ad9dac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd9dac9, SUC_lexicon_tags_2 },
    { 0xa499daca, SUC_lexicon_tags_17 },
    { 0x9019daca, SUC_lexicon_tags_18 },
    { 0x4559daca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb179dad1, SUC_lexicon_tags_44 },
    { 0xbcf9dad2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x50f9dad4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2039dad6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaf9dada, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe779dadd, SUC_lexicon_tags_12 },
    { 0xdf9dadd, SUC_lexicon_tags_63 },
    { 0x8a39dadd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1119dae1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad19dae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf639dae7, SUC_lexicon_tags_9 },
    { 0x7e39dae8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f9daf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5359daf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb9dafd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf9db00, SUC_lexicon_tags_1 },
    { 0x4359db01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd059db03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb019db09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4d9db0b, SUC_lexicon_tags_33 },
    { 0xab19db0c, SUC_lexicon_tags_25 },
    { 0x96d9db0d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79db12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb19db15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b39db22, SUC_lexicon_tags_26 },
    { 0xa659db23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4059db26, SUC_lexicon_tags_38 },
    { 0x8639db26, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5299db2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a79db2c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x7039db2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9db32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c79db37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2999db3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb79db3c, SUC_lexicon_tags_13 },
    { 0x3e19db3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359db42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58f9db44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd9db47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4079db4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7639db54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe959db5a, SUC_lexicon_tags_17 },
    { 0xc819db5b, SUC_lexicon_tags_66 },
    { 0xbb59db5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6d9db5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89d9db60, SUC_lexicon_tags_1 },
    { 0x5719db60, SUC_lexicon_tags_1 },
    { 0x9159db62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db9db67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x659db69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcad9db6c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x41b9db6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6699db72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b9db75, SUC_lexicon_tags_7 },
    { 0xe6f9db76, SUC_lexicon_tags_7 },
    { 0x4839db77, SUC_lexicon_tags_17 },
    { 0x1319db76, SUC_lexicon_tags_15 },
    { 0x6559db78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c19db7c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc939db80, SUC_lexicon_tags_3 },
    { 0x98f9db80, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f9db85, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0xbe59db87, SUC_lexicon_tags_29 },
    { 0xead9db87, SUC_lexicon_tags_37 },
    { 0x7099db88, SUC_lexicon_tags_37 },
    { 0xacd9db8a, SUC_lexicon_tags_9 },
    { 0xe1d9db88, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfd39db8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5799db92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f9db95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f9dba1, SUC_lexicon_tags_34 },
    { 0xc939dba2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4219dba6, SUC_lexicon_tags_63 },
    { 0xee79dba7, SUC_lexicon_tags_27 },
    { 0xcdb9dba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf9dbac, SUC_lexicon_tags_2 },
    { 0xa299dbad, SUC_lexicon_tags_190 },
    { 0x4ed9dbae, SUC_lexicon_tags_56 },
    { 0xde99dbaf, SUC_lexicon_tags_33 },
    { 0x8679dbaf, SUC_lexicon_tags_17 },
    { 0x20f9dbb1, SUC_lexicon_tags_25 },
    { 0xb5f9dbaf, SUC_lexicon_tags_14 },
    { 0x5759dbb3, SUC_lexicon_tags_63 },
    { 0xadf9dbb4, SUC_lexicon_tags_17 },
    { 0x43d9dbb5, SUC_lexicon_tags_92 },
    { 0x79d9dbb6, SUC_lexicon_tags_57 },
    { 0x4b19dbb7, SUC_lexicon_tags_28 },
    { 0x87d9dbb7, SUC_lexicon_tags_15 },
    { 0x4cd9dbb5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd179dbbb, SUC_lexicon_tags_31 },
    { 0x8499dbbc, SUC_lexicon_tags_2 },
    { 0x2a59dbbd, SUC_lexicon_tags_47 },
    { 0xe379dbbe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x80f9dbc1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb9dbc6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a39dbc9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf419dbcc, SUC_lexicon_tags_17 },
    { 0x65b9dbcd, SUC_lexicon_tags_9 },
    { 0x2fb9dbce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1599dbd0, SUC_lexicon_tags_15 },
    { 0x9239dbd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb179dbd3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db9dbd6, SUC_lexicon_tags_11 },
    { 0x5539dbd6, SUC_lexicon_tags_1 },
    { 0x2ad9dbd8, SUC_lexicon_tags_25 },
    { 0xe79dbd9, SUC_lexicon_tags_45 },
    { 0xe2d9dbd9, SUC_lexicon_tags_25 },
    { 0x8b79dbdb, SUC_lexicon_tags_26 },
    { 0x1419dbdc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4899dbe0, SUC_lexicon_tags_37 },
    { 0x6c39dbe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9319dbe4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa379dbe6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x679dbe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e79dbeb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe619dbee, SUC_lexicon_tags_8 },
    { 0x49d9dbee, SUC_lexicon_tags_1 },
    { 0xd979dbef, SUC_lexicon_tags_17 },
    { 0xf039dbf1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9dbf4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x39b9dbf7, SUC_lexicon_tags_17 },
    { 0x5079dbf8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59dbfc, SUC_lexicon_tags_15 },
    { 0xb119dbfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe039dc00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef9dc05, SUC_lexicon_tags_61 },
    { 0x3d59dc05, SUC_lexicon_tags_14 },
    { 0x64b9dc05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c79dc0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98f9dc0d, SUC_lexicon_tags_1 },
    { 0xad39dc0e, SUC_lexicon_tags_12 },
    { 0xffd9dc0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d9dc12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed9dc17, SUC_lexicon_tags_9 },
    { 0x40f9dc18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd279dc1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9039dc1d, SUC_lexicon_tags_13 },
    { 0xa8d9dc1d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc739dc20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5139dc29, SUC_lexicon_tags_37 },
    { 0xcbb9dc2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc539dc2d, SUC_lexicon_tags_30 },
    { 0xe3b9dc2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e79dc31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x37d9dc33, SUC_lexicon_tags_11 },
    { 0x5b19dc33, SUC_lexicon_tags_15 },
    { 0xc7d9dc35, SUC_lexicon_tags_27 },
    { 0x9699dc35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8899dc3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4259dc3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159dc42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f9dc46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff59dc48, SUC_lexicon_tags_3 },
    { 0x2a19dc49, SUC_lexicon_tags_92 },
    { 0x1f59dc48, SUC_lexicon_tags_12 },
    { 0x4559dc4b, SUC_lexicon_tags_63 },
    { 0x9d19dc4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4099dc4f, SUC_lexicon_tags_56 },
    { 0x2a19dc50, SUC_lexicon_tags_62 },
    { 0x88f9dc50, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b99dc55, SUC_lexicon_tags_37 },
    { 0xf1d9dc56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa99dc59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda99dc5c, SUC_lexicon_tags_37 },
    { 0x5779dc5d, SUC_lexicon_tags_34 },
    { 0xa7d9dc5e, SUC_lexicon_tags_17 },
    { 0x5cb9dc5f, SUC_lexicon_tags_14 },
    { 0x7699dc60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5b9dc63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f9dc67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff59dc6c, SUC_lexicon_tags_14 },
    { 0xa839dc6c, SUC_lexicon_tags_19 },
    { 0xa019dc6e, SUC_lexicon_tags_87 },
    { 0xed99dc6d, SUC_lexicon_tags_8 },
    { 0x82d9dc70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b9dc73, SUC_lexicon_tags_17 },
    { 0x4f59dc73, SUC_lexicon_tags_33 },
    { 0xee19dc75, SUC_lexicon_tags_8 },
    { 0x12d9dc75, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x399dc79, SUC_lexicon_tags_1 },
    { 0xac59dc7a, SUC_lexicon_tags_19 },
    { 0xad39dc7b, SUC_lexicon_tags_2 },
    { 0xa19dc7c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2b9dc81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f9dc84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35d9dc86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3479dc94, SUC_lexicon_tags_2 },
    { 0x4ff9dc95, SUC_lexicon_tags_15 },
    { 0x8899dc96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb9dc9c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb9dc9f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd9dca5, SUC_lexicon_tags_26 },
    { 0xd119dca6, SUC_lexicon_tags_29 },
    { 0x6779dca5, SUC_lexicon_tags_15 },
    { 0x9d39dca8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb959dcaa, SUC_lexicon_tags_21 },
    { 0x9bb9dcab, SUC_lexicon_tags_2 },
    { 0x5179dcac, SUC_lexicon_tags_158 },
    { 0xb939dcac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9f9dcaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc379dcb1, SUC_lexicon_tags_27 },
    { 0x2f39dcb1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9339dcb8, SUC_lexicon_tags_38 },
    { 0x8059dcb9, SUC_lexicon_tags_33 },
    { 0xa279dcba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc39dcbc, SUC_lexicon_tags_17 },
    { 0xf819dcbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd779dcbf, SUC_lexicon_tags_2 },
    { 0xaef9dcbf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e19dcc3, SUC_lexicon_tags_25 },
    { 0xcab9dcc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa359dcca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf9dccd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb379dccf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8f9dcd3, SUC_lexicon_tags_7 },
    { 0xb359dcd4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcad9dcd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d9dcdb, SUC_lexicon_tags_17 },
    { 0xf879dcdc, SUC_lexicon_tags_15 },
    { 0xeb79dcdc, SUC_lexicon_tags_8 },
    { 0xe639dcdd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c59dce4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf79dce6, SUC_lexicon_tags_26 },
    { 0x559dce7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed99dcec, SUC_lexicon_tags_17 },
    { 0x779dcec, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eb9dcf2, SUC_lexicon_tags_11 },
    { 0x28f9dcf3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x20f9dcf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6b9dcf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c39dcf9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8179dcfb, SUC_lexicon_tags_43 },
    { 0xd259dcfc, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe99dd00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0d9dd04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7859dd07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7759dd0b, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf659dd10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb79dd13, SUC_lexicon_tags_19 },
    { 0x39d9dd14, SUC_lexicon_tags_63 },
    { 0x3b19dd13, SUC_lexicon_tags_65 },
    { 0xc779dd16, SUC_lexicon_tags_9 },
    { 0x1a79dd17, SUC_lexicon_tags_20 },
    { 0x9c79dd14, SUC_lexicon_tags_60 },
    { 0xe779dd19, SUC_lexicon_tags_7 },
    { 0x29b9dd19, SUC_lexicon_tags_35 },
    { 0x9bd9dd1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f9dd1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8539dd22, SUC_lexicon_tags_49 },
    { 0xb4d9dd22, SUC_lexicon_tags_1 },
    { 0xda99dd24, SUC_lexicon_tags_9 },
    { 0x18f9dd25, SUC_lexicon_tags_25 },
    { 0xed19dd26, SUC_lexicon_tags_60 },
    { 0x7979dd26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff59dd29, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79dd2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f99dd2f, SUC_lexicon_tags_37 },
    { 0x3e59dd2f, SUC_lexicon_tags_2 },
    { 0x2e59dd2f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d9dd38, SUC_lexicon_tags_7 },
    { 0xb7f9dd39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f9dd3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf39dd40, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb9dd44, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7239dd46, SUC_lexicon_tags_31 },
    { 0x1539dd47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9dd4a, SUC_lexicon_tags_43 },
    { 0xf419dd4a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86f9dd4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0f9dd4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf9dd52, SUC_lexicon_tags_11 },
    { 0x7a59dd53, SUC_lexicon_tags_14 },
    { 0x72f9dd54, SUC_lexicon_tags_15 },
    { 0xc159dd54, SUC_lexicon_tags_17 },
    { 0xed79dd55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc239dd58, SUC_lexicon_tags_11 },
    { 0xae79dd59, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39dd5d, SUC_lexicon_tags_37 },
    { 0x3519dd5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5d9dd60, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd9dd64, SUC_lexicon_tags_17 },
    { 0xd0d9dd64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe059dd67, SUC_lexicon_tags_9 },
    { 0x42b9dd68, SUC_lexicon_tags_57 },
    { 0xf999dd69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38f9dd6f, SUC_lexicon_tags_37 },
    { 0xf99dd6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd99dd72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1479dd76, SUC_lexicon_tags_82 },
    { 0x1d39dd76, SUC_lexicon_tags_37 },
    { 0xc179dd78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14d9dd7d, SUC_lexicon_tags_7 },
    { 0xb019dd7d, SUC_lexicon_tags_38 },
    { 0x779dd7f, SUC_lexicon_tags_14 },
    { 0xceb9dd80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6af9dd82, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25b9dd88, SUC_lexicon_tags_16 },
    { 0xbed9dd89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f9dd8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a59dd97, SUC_lexicon_tags_13 },
    { 0x4259dd98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39b9dd9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9879dd9c, SUC_lexicon_tags_43 },
    { 0x6ed9dd9d, SUC_lexicon_tags_37 },
    { 0xbd39dd9d, SUC_lexicon_tags_17 },
    { 0x8ed9dd9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7899dda4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44d9dda6, SUC_lexicon_tags_39 },
    { 0x1d99dda6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa959ddaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9ddb8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb39ddbe, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe5f9ddc0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf879ddc5, SUC_lexicon_tags_15 },
    { 0x2419ddc6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x96b9ddc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9079ddcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6479ddcd, SUC_lexicon_tags_19 },
    { 0x4359ddce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4699ddd2, SUC_lexicon_tags_8 },
    { 0xf539ddd2, SUC_lexicon_tags_7 },
    { 0xec79ddd4, SUC_lexicon_tags_46 },
    { 0x6079ddd4, SUC_lexicon_tags_12 },
    { 0xd479ddd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xabf9ddd8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x69d9ddda, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf19dddc, SUC_lexicon_tags_46 },
    { 0x16d9dddd, SUC_lexicon_tags_1 },
    { 0xb819ddde, SUC_lexicon_tags_17 },
    { 0xb3d9dddf, SUC_lexicon_tags_19 },
    { 0xbf19dde0, SUC_lexicon_tags_74 },
    { 0x3219dde1, SUC_lexicon_tags_7 },
    { 0xa979dde2, SUC_lexicon_tags_15 },
    { 0xefd9dde3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9859ddec, SUC_lexicon_tags_11 },
    { 0x79b9dded, SUC_lexicon_tags_6 },
    { 0x6b9ddee, SUC_lexicon_tags_50 },
    { 0x3559dded, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7399ddf1, SUC_lexicon_tags_25 },
    { 0x37d9ddf1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2419ddf7, SUC_lexicon_tags_6 },
    { 0xe839ddf8, SUC_lexicon_tags_37 },
    { 0x38d9ddf9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe3d9ddfb, SUC_lexicon_tags_15 },
    { 0x1e19ddfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd59ddff, SUC_lexicon_tags_8 },
    { 0xccd9de00, SUC_lexicon_tags_43 },
    { 0xc0b9de01, SUC_lexicon_tags_98 },
    { 0x4319de00, SUC_lexicon_tags_23 },
    { 0xed79de03, SUC_lexicon_tags_11 },
    { 0xf2b9de04, SUC_lexicon_tags_25 },
    { 0x4199de00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11f9de07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff9de0a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d9de0d, SUC_lexicon_tags_47 },
    { 0x32d9de0d, SUC_lexicon_tags_50 },
    { 0x6819de0f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6319de11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d9de17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b39de1b, SUC_lexicon_tags_25 },
    { 0xb279de1c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d9de21, SUC_lexicon_tags_49 },
    { 0x16f9de21, SUC_lexicon_tags_25 },
    { 0x599de23, SUC_lexicon_tags_11 },
    { 0xe2f9de24, SUC_lexicon_tags_5 },
    { 0xedf9de25, SUC_lexicon_tags_37 },
    { 0xc159de26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe119de2b, SUC_lexicon_tags_14 },
    { 0xbc39de2b, SUC_lexicon_tags_12 },
    { 0x6959de2d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1b9de2f, SUC_lexicon_tags_32 },
    { 0xf99de30, SUC_lexicon_tags_98 },
    { 0x8ff9de30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e59de3a, SUC_lexicon_tags_32 },
    { 0xd6b9de3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4719de3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0b9de40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf879de43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1959de4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf59de4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc79de51, SUC_lexicon_tags_57 },
    { 0x2739de52, SUC_lexicon_tags_1 },
    { 0xb179de52, SUC_lexicon_tags_37 },
    { 0xbc79de51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c79de56, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x70f9de59, SUC_lexicon_tags_27 },
    { 0x9559de5a, SUC_lexicon_tags_39 },
    { 0x9cb9de5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3759de5d, SUC_lexicon_tags_9 },
    { 0x1379de5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf9b9de60, SUC_lexicon_tags_60 },
    { 0xbf39de61, SUC_lexicon_tags_47 },
    { 0x23f9de61, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b59de66, SUC_lexicon_tags_25 },
    { 0x9979de67, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8e19de69, SUC_lexicon_tags_49 },
    { 0xa919de6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5df9de6c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb9de72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8139de74, SUC_lexicon_tags_43 },
    { 0xf299de75, SUC_lexicon_tags_7 },
    { 0x279de75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2339de79, SUC_lexicon_tags_45 },
    { 0x1a59de7a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1eb9de7c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x37d9de7e, SUC_lexicon_tags_17 },
    { 0xa1d9de7e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac99de83, SUC_lexicon_tags_17 },
    { 0xad39de84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1399de86, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3719de88, SUC_lexicon_tags_17 },
    { 0xc079de88, SUC_lexicon_tags_1 },
    { 0xb039de89, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x8299de8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9de93, SUC_lexicon_tags_56 },
    { 0x9eb9de93, SUC_lexicon_tags_5 },
    { 0x6319de95, SUC_lexicon_tags_47 },
    { 0xf279de95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf19de9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccf9dea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad79dea5, SUC_lexicon_tags_47 },
    { 0x2899dea6, SUC_lexicon_tags_12 },
    { 0xb099dea7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98f9deac, SUC_lexicon_tags_17 },
    { 0x219dead, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3439deb5, SUC_lexicon_tags_2 },
    { 0x7f39deb6, SUC_lexicon_tags_1 },
    { 0xf139deb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf319deb9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8979debb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4f9debd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d19dec0, SUC_lexicon_tags_15 },
    { 0xcef9dec0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d9ded0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xef19ded2, SUC_lexicon_tags_38 },
    { 0xb79ded2, SUC_lexicon_tags_25 },
    { 0xe39ded2, SUC_lexicon_tags_86 },
    { 0xe899ded4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d9dedb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd819dede, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e79dee0, SUC_lexicon_tags_2 },
    { 0x16d9dee0, SUC_lexicon_tags_17 },
    { 0x9919dee2, SUC_lexicon_tags_33 },
    { 0xbd79dee1, SUC_lexicon_tags_12 },
    { 0x9f39dee4, SUC_lexicon_tags_7 },
    { 0x8659dee2, SUC_lexicon_tags_43 },
    { 0xcef9dee2, SUC_lexicon_tags_29 },
    { 0x92f9dee7, SUC_lexicon_tags_27 },
    { 0x6f79dee7, SUC_lexicon_tags_5 },
    { 0xfbb9dee9, SUC_lexicon_tags_12 },
    { 0x3639deea, SUC_lexicon_tags_2 },
    { 0x9dd9deeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3639deee, SUC_lexicon_tags_15 },
    { 0xb5d9deee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c39def3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1859def8, SUC_lexicon_tags_10 },
    { 0x1779def9, SUC_lexicon_tags_28 },
    { 0x1ed9defa, SUC_lexicon_tags_107 },
    { 0x5259defb, SUC_lexicon_tags_14 },
    { 0xe3d9defc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e79df02, SUC_lexicon_tags_7 },
    { 0xa4d9df03, SUC_lexicon_tags_27 },
    { 0x4679df04, SUC_lexicon_tags_28 },
    { 0x379df04, SUC_lexicon_tags_47 },
    { 0x2559df04, SUC_lexicon_tags_41 },
    { 0xdcf9df07, SUC_lexicon_tags_60 },
    { 0x7479df05, SUC_lexicon_tags_12 },
    { 0x79d9df09, SUC_lexicon_tags_234 },
    { 0x9c99df07, SUC_lexicon_tags_17 },
    { 0x3399df0b, SUC_lexicon_tags_47 },
    { 0x3f59df0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8f9df0f, SUC_lexicon_tags_11 },
    { 0x5b99df10, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb99df12, SUC_lexicon_tags_17 },
    { 0x6cd9df12, SUC_lexicon_tags_17 },
    { 0x7199df14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd999df16, SUC_lexicon_tags_23 },
    { 0xa59df17, SUC_lexicon_tags_21 },
    { 0x7339df17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf779df1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b9df1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6f9df20, SUC_lexicon_tags_2 },
    { 0xb9df21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3519df29, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff9df2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5219df30, SUC_lexicon_tags_1 },
    { 0xa0b9df31, SUC_lexicon_tags_63 },
    { 0x7739df31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c19df38, SUC_lexicon_tags_25 },
    { 0xcdf9df39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cd9df3d, SUC_lexicon_tags_37 },
    { 0xeb9df3d, SUC_lexicon_tags_1 },
    { 0x5b9df3f, SUC_lexicon_tags_49 },
    { 0x3a19df40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfb9df42, SUC_lexicon_tags_60 },
    { 0x2bf9df43, SUC_lexicon_tags_0 },
    { 0x51d9df44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa379df46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b19df4a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7619df4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f9df53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a99df56, SUC_lexicon_tags_60 },
    { 0xbb9df57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dd9df59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe299df60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239df64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cf9df6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe639df6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f9df72, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa199df75, SUC_lexicon_tags_38 },
    { 0xe679df75, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1c79df78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ab9df7a, SUC_lexicon_tags_4 },
    { 0xf039df7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe619df7d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd9df80, SUC_lexicon_tags_11 },
    { 0x5f79df80, SUC_lexicon_tags_7 },
    { 0x39b9df82, SUC_lexicon_tags_7 },
    { 0xad59df80, SUC_lexicon_tags_2 },
    { 0x3ab9df82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b19df90, SUC_lexicon_tags_26 },
    { 0x7ed9df91, SUC_lexicon_tags_46 },
    { 0x1639df90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x14f9df94, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4679df96, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e59df99, SUC_lexicon_tags_59 },
    { 0xee99df99, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff9df9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a19dfa4, SUC_lexicon_tags_11 },
    { 0xaf19dfa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e39dfa7, SUC_lexicon_tags_7 },
    { 0xc8f9dfa8, SUC_lexicon_tags_47 },
    { 0x1cd9dfa9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0d9dfaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2199dfb2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf9dfb6, SUC_lexicon_tags_11 },
    { 0x1039dfb7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac79dfc2, SUC_lexicon_tags_17 },
    { 0x4859dfc3, SUC_lexicon_tags_1 },
    { 0x8919dfc3, SUC_lexicon_tags_0 },
    { 0xb2b9dfc4, SUC_lexicon_tags_25 },
    { 0xe059dfc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df9dfc9, SUC_lexicon_tags_17 },
    { 0x7279dfca, SUC_lexicon_tags_10 },
    { 0x6df9dfca, SUC_lexicon_tags_11 },
    { 0x1319dfcc, SUC_lexicon_tags_15 },
    { 0x60d9dfcd, SUC_lexicon_tags_13 },
    { 0x7859dfcc, SUC_lexicon_tags_33 },
    { 0xc619dfcc, SUC_lexicon_tags_18 },
    { 0xab79dfd0, SUC_lexicon_tags_0 },
    { 0xe839dfd1, SUC_lexicon_tags_57 },
    { 0x73f9dfd2, SUC_lexicon_tags_17 },
    { 0x8059dfd2, SUC_lexicon_tags_25 },
    { 0xd019dfd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x879dfd6, SUC_lexicon_tags_7 },
    { 0xe019dfd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc19dfdd, SUC_lexicon_tags_31 },
    { 0xdb99dfde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5699dfe0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb919dfe2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4799dfe5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe19dfea, SUC_lexicon_tags_26 },
    { 0x2db9dfea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2f9dff4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa079dff6, SUC_lexicon_tags_32 },
    { 0x2119dff6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc319dff9, SUC_lexicon_tags_23 },
    { 0xf459dffa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cf9dffc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4139dffe, SUC_lexicon_tags_11 },
    { 0x1a99dfff, SUC_lexicon_tags_15 },
    { 0xd539e000, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2639e002, SUC_lexicon_tags_31 },
    { 0x7d9e003, SUC_lexicon_tags_28 },
    { 0xaf39e002, SUC_lexicon_tags_17 },
    { 0xe5f9e005, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24f9e007, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x26b9e009, SUC_lexicon_tags_17 },
    { 0x81f9e009, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1299e010, SUC_lexicon_tags_7 },
    { 0x2519e010, SUC_lexicon_tags_27 },
    { 0xaff9e012, SUC_lexicon_tags_234 },
    { 0xd4d9e013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3b9e015, SUC_lexicon_tags_33 },
    { 0x7f59e016, SUC_lexicon_tags_12 },
    { 0xfb99e015, SUC_lexicon_tags_30 },
    { 0x8139e015, SUC_lexicon_tags_60 },
    { 0x5539e019, SUC_lexicon_tags_478 },
    { 0xdbb9e017, SUC_lexicon_tags_29 },
    { 0xf139e01b, SUC_lexicon_tags_17 },
    { 0xf5d9e01b, SUC_lexicon_tags_19 },
    { 0x57b9e01b, SUC_lexicon_tags_32 },
    { 0x38d9e01e, SUC_lexicon_tags_6 },
    { 0x5b99e01e, SUC_lexicon_tags_0 },
    { 0x2099e01f, SUC_lexicon_tags_15 },
    { 0x99b9e021, SUC_lexicon_tags_25 },
    { 0x4779e01f, SUC_lexicon_tags_25 },
    { 0x9079e023, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1459e028, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83f9e02d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xabd9e02f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc2b9e031, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5659e037, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8b9e039, SUC_lexicon_tags_37 },
    { 0xa699e039, SUC_lexicon_tags_37 },
    { 0x4659e039, SUC_lexicon_tags_7 },
    { 0x1d9e03c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4519e046, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2959e048, SUC_lexicon_tags_32 },
    { 0x9199e048, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe959e04e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ed9e052, SUC_lexicon_tags_1 },
    { 0x7f99e053, SUC_lexicon_tags_25 },
    { 0xcaf9e054, SUC_lexicon_tags_350 },
    { 0x50d9e055, SUC_lexicon_tags_33 },
    { 0x55f9e054, SUC_lexicon_tags_11 },
    { 0x7159e055, SUC_lexicon_tags_11 },
    { 0xc3b9e058, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe99e05e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe239e066, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3099e06e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9d79e070, SUC_lexicon_tags_49 },
    { 0x1c79e070, SUC_lexicon_tags_27 },
    { 0x4e59e072, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b9e076, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8059e07a, SUC_lexicon_tags_17 },
    { 0x2599e07b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1079e07f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb9e08c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc519e090, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b79e093, SUC_lexicon_tags_7 },
    { 0xb159e093, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3359e09c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab9e09f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb19e0a1, SUC_lexicon_tags_31 },
    { 0x5319e0a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1579e0a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5199e0a8, SUC_lexicon_tags_38 },
    { 0x1039e0a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc759e0ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe619e0b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13f9e0b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb39e0b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x13f9e0bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5439e0be, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9f99e0c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b9e0cb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f9e0d1, SUC_lexicon_tags_25 },
    { 0x4679e0d1, SUC_lexicon_tags_49 },
    { 0xe7b9e0d2, SUC_lexicon_tags_37 },
    { 0xf599e0d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8d9e0d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e79e0d8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa899e0df, SUC_lexicon_tags_17 },
    { 0x2459e0df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xea19e0e2, SUC_lexicon_tags_63 },
    { 0x319e0e3, SUC_lexicon_tags_25 },
    { 0x83d9e0e2, SUC_lexicon_tags_17 },
    { 0x3e19e0e5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x80f9e0e7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5879e0ea, SUC_lexicon_tags_61 },
    { 0xfe99e0eb, SUC_lexicon_tags_25 },
    { 0xa759e0ec, SUC_lexicon_tags_61 },
    { 0xd9f9e0ed, SUC_lexicon_tags_11 },
    { 0x6bd9e0ec, SUC_lexicon_tags_10 },
    { 0xbbf9e0ef, SUC_lexicon_tags_7 },
    { 0xc299e0ef, SUC_lexicon_tags_60 },
    { 0xbe79e0ef, SUC_lexicon_tags_7 },
    { 0xcf39e0f1, SUC_lexicon_tags_7 },
    { 0xc279e0f3, SUC_lexicon_tags_11 },
    { 0x8e79e0f4, SUC_lexicon_tags_37 },
    { 0xa679e0f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1579e0f8, SUC_lexicon_tags_27 },
    { 0x4699e0f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6f99e0fb, SUC_lexicon_tags_124 },
    { 0xc019e0fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56f9e0ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4039e101, SUC_lexicon_tags_26 },
    { 0xae99e101, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x64b9e104, SUC_lexicon_tags_47 },
    { 0x7d79e104, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee39e107, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2979e10a, SUC_lexicon_tags_27 },
    { 0xb099e10b, SUC_lexicon_tags_25 },
    { 0x5859e10b, SUC_lexicon_tags_12 },
    { 0x9179e10a, SUC_lexicon_tags_85 },
    { 0x9e59e10e, SUC_lexicon_tags_0 },
    { 0x71b9e10f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6199e112, SUC_lexicon_tags_37 },
    { 0xf499e113, SUC_lexicon_tags_3 },
    { 0x8739e114, SUC_lexicon_tags_1 },
    { 0x679e115, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7659e117, SUC_lexicon_tags_25 },
    { 0xfe39e118, SUC_lexicon_tags_42 },
    { 0xc519e118, SUC_lexicon_tags_14 },
    { 0x9939e11a, SUC_lexicon_tags_21 },
    { 0x5939e11b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2f9e11d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2f9e124, SUC_lexicon_tags_25 },
    { 0x6519e125, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc3f9e127, SUC_lexicon_tags_15 },
    { 0xcbf9e127, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd079e12a, SUC_lexicon_tags_11 },
    { 0xc9b9e12a, SUC_lexicon_tags_37 },
    { 0xee99e12a, SUC_lexicon_tags_194 },
    { 0x9f19e12d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa979e130, SUC_lexicon_tags_37 },
    { 0x59f9e130, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd4b9e133, SUC_lexicon_tags_17 },
    { 0x4099e134, SUC_lexicon_tags_11 },
    { 0x13d9e135, SUC_lexicon_tags_43 },
    { 0x80d9e135, SUC_lexicon_tags_198 },
    { 0xcb99e137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3679e139, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xba19e13c, SUC_lexicon_tags_31 },
    { 0x3019e13d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ed9e13f, SUC_lexicon_tags_2 },
    { 0xf479e140, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34f9e144, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cf9e14a, SUC_lexicon_tags_62 },
    { 0x4d39e14b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9e14f, SUC_lexicon_tags_1 },
    { 0xc8f9e150, SUC_lexicon_tags_47 },
    { 0xcf99e150, SUC_lexicon_tags_4 },
    { 0x5599e152, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd9e15b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3219e15e, SUC_lexicon_tags_17 },
    { 0x33b9e15f, SUC_lexicon_tags_25 },
    { 0xf199e15e, SUC_lexicon_tags_1292 },
    { 0xa719e15e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5519e167, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6839e16b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d9e171, SUC_lexicon_tags_11 },
    { 0x7979e172, SUC_lexicon_tags_6 },
    { 0x3759e173, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9339e177, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b39e17b, SUC_lexicon_tags_47 },
    { 0x45d9e17b, SUC_lexicon_tags_63 },
    { 0x5979e17d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9619e182, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4879e185, SUC_lexicon_tags_1 },
    { 0x8399e185, SUC_lexicon_tags_27 },
    { 0x5479e187, SUC_lexicon_tags_10 },
    { 0x9239e188, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6519e18a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef9e18d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2959e18f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4a59e191, SUC_lexicon_tags_7 },
    { 0x24d9e191, SUC_lexicon_tags_19 },
    { 0x2b19e192, SUC_lexicon_tags_16 },
    { 0x8559e192, SUC_lexicon_tags_27 },
    { 0x5db9e195, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf439e197, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfb59e199, SUC_lexicon_tags_9 },
    { 0xb539e19a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa059e19d, SUC_lexicon_tags_30 },
    { 0x85d9e19e, SUC_lexicon_tags_11 },
    { 0x5f19e19e, SUC_lexicon_tags_37 },
    { 0xa1f9e1a0, SUC_lexicon_tags_1 },
    { 0x8c59e1a0, SUC_lexicon_tags_17 },
    { 0x479e1a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb019e1a4, SUC_lexicon_tags_37 },
    { 0x40d9e1a5, SUC_lexicon_tags_37 },
    { 0x7b39e1a6, SUC_lexicon_tags_15 },
    { 0xcf79e1a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6559e1a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9e1ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7599e1b2, SUC_lexicon_tags_15 },
    { 0x8af9e1b3, SUC_lexicon_tags_104 },
    { 0xeb39e1b2, SUC_lexicon_tags_39 },
    { 0x159e1b2, SUC_lexicon_tags_15 },
    { 0x8cf9e1b2, SUC_lexicon_tags_37 },
    { 0x3f99e1b4, SUC_lexicon_tags_0 },
    { 0x51f9e1b6, SUC_lexicon_tags_38 },
    { 0x3d79e1b7, SUC_lexicon_tags_37 },
    { 0xc39e1ba, SUC_lexicon_tags_17 },
    { 0xeed9e1bb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf259e1bf, SUC_lexicon_tags_45 },
    { 0xe8d9e1c0, SUC_lexicon_tags_1 },
    { 0xa79e1c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa739e1c5, SUC_lexicon_tags_10 },
    { 0xa779e1c6, SUC_lexicon_tags_11 },
    { 0xbad9e1c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3df9e1c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79e1cb, SUC_lexicon_tags_37 },
    { 0x1679e1cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1919e1d1, SUC_lexicon_tags_11 },
    { 0xca19e1d1, SUC_lexicon_tags_11 },
    { 0xdd9e1d3, SUC_lexicon_tags_0 },
    { 0x2ef9e1d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e1d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ab9e1dd, SUC_lexicon_tags_25 },
    { 0xe659e1de, SUC_lexicon_tags_60 },
    { 0x5019e1df, SUC_lexicon_tags_7 },
    { 0x59d9e1e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe919e1e2, SUC_lexicon_tags_26 },
    { 0x7359e1e2, SUC_lexicon_tags_9 },
    { 0xab59e1e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b9e1e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x43f9e1eb, SUC_lexicon_tags_63 },
    { 0xfbb9e1eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf999e1ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbe39e1f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e79e1f8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf0d9e1fa, SUC_lexicon_tags_41 },
    { 0xd59e1fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x83d9e1fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc539e200, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66d9e203, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf419e209, SUC_lexicon_tags_14 },
    { 0x3219e20a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc2b9e20c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4319e20e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8479e213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf59e215, SUC_lexicon_tags_14 },
    { 0x5f99e215, SUC_lexicon_tags_12 },
    { 0xb259e217, SUC_lexicon_tags_37 },
    { 0xd599e218, SUC_lexicon_tags_30 },
    { 0x699e217, SUC_lexicon_tags_26 },
    { 0x3d9e21a, SUC_lexicon_tags_37 },
    { 0x6a99e21b, SUC_lexicon_tags_11 },
    { 0x2019e21c, SUC_lexicon_tags_11 },
    { 0x6b79e21d, SUC_lexicon_tags_11 },
    { 0x9139e21e, SUC_lexicon_tags_27 },
    { 0xbe39e219, SUC_lexicon_tags_31 },
    { 0xd6f9e220, SUC_lexicon_tags_2 },
    { 0xbf79e220, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb459e225, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe059e22b, SUC_lexicon_tags_25 },
    { 0x6019e22c, SUC_lexicon_tags_28 },
    { 0x9639e22c, SUC_lexicon_tags_229 },
    { 0xa259e22e, SUC_lexicon_tags_11 },
    { 0x38b9e22d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb79e231, SUC_lexicon_tags_11 },
    { 0x82d9e231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19e235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4499e23a, SUC_lexicon_tags_16 },
    { 0xdff9e23b, SUC_lexicon_tags_14 },
    { 0xde59e23a, SUC_lexicon_tags_17 },
    { 0x4f39e23d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb9e244, SUC_lexicon_tags_5 },
    { 0xa359e245, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d9e249, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2f9e24b, SUC_lexicon_tags_15 },
    { 0x5ab9e24b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4fd9e24e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d9e253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9e25b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6439e25e, SUC_lexicon_tags_12 },
    { 0x4a19e25f, SUC_lexicon_tags_11 },
    { 0xc99e25e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8179e263, SUC_lexicon_tags_46 },
    { 0xc9b9e264, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7179e269, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e19e26b, SUC_lexicon_tags_7 },
    { 0x3a79e26b, SUC_lexicon_tags_7 },
    { 0x5c39e26d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b79e271, SUC_lexicon_tags_15 },
    { 0x3419e271, SUC_lexicon_tags_13 },
    { 0x4499e273, SUC_lexicon_tags_129 },
    { 0x8739e273, SUC_lexicon_tags_12 },
    { 0xaef9e275, SUC_lexicon_tags_48 },
    { 0xeeb9e272, SUC_lexicon_tags_7 },
    { 0x9099e277, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7039e27a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb9e282, SUC_lexicon_tags_60 },
    { 0x419e283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ef9e288, SUC_lexicon_tags_14 },
    { 0x4439e288, SUC_lexicon_tags_47 },
    { 0x8519e288, SUC_lexicon_tags_30 },
    { 0x2eb9e288, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf19e28d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe659e290, SUC_lexicon_tags_37 },
    { 0x6c99e291, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9539e294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1f9e296, SUC_lexicon_tags_38 },
    { 0x6f9e296, SUC_lexicon_tags_14 },
    { 0x2e79e296, SUC_lexicon_tags_70 },
    { 0x9459e298, SUC_lexicon_tags_1 },
    { 0xf4f9e298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79e29e, SUC_lexicon_tags_37 },
    { 0xac79e29e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9579e2a1, SUC_lexicon_tags_2 },
    { 0x9b39e2a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb679e2a4, SUC_lexicon_tags_2 },
    { 0x38b9e2a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd759e2a7, SUC_lexicon_tags_14 },
    { 0x9859e2a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9b9e2aa, SUC_lexicon_tags_7 },
    { 0xe9f9e2aa, SUC_lexicon_tags_38 },
    { 0x8419e2ac, SUC_lexicon_tags_25 },
    { 0x3319e2aa, SUC_lexicon_tags_2 },
    { 0xde79e2ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x27f9e2b1, SUC_lexicon_tags_14 },
    { 0x3c19e2b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d19e2b4, SUC_lexicon_tags_79 },
    { 0x3a19e2b5, SUC_lexicon_tags_47 },
    { 0xe399e2b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x71b9e2b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb39e2ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c99e2bf, SUC_lexicon_tags_15 },
    { 0x4339e2c0, SUC_lexicon_tags_37 },
    { 0x95b9e2c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b99e2c3, SUC_lexicon_tags_17 },
    { 0x9eb9e2c3, SUC_lexicon_tags_45 },
    { 0x1f99e2c5, SUC_lexicon_tags_14 },
    { 0xbff9e2c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3299e2cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa719e2d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7019e2d4, SUC_lexicon_tags_58 },
    { 0x60d9e2d4, SUC_lexicon_tags_15 },
    { 0x7239e2d6, SUC_lexicon_tags_28 },
    { 0x34f9e2d7, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2039e2dd, SUC_lexicon_tags_1 },
    { 0xe579e2de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4639e2e1, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa059e2e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32d9e2ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f19e2ee, SUC_lexicon_tags_16 },
    { 0x6139e2ef, SUC_lexicon_tags_18 },
    { 0x5879e2ee, SUC_lexicon_tags_25 },
    { 0x1f19e2f0, SUC_lexicon_tags_0 },
    { 0xf99e2f2, SUC_lexicon_tags_26 },
    { 0xd479e2ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4059e2f7, SUC_lexicon_tags_15 },
    { 0x8cd9e2f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc279e2fc, SUC_lexicon_tags_1 },
    { 0x6299e2fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb459e300, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9df9e302, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x419e304, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc199e306, SUC_lexicon_tags_43 },
    { 0xa7f9e307, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb859e309, SUC_lexicon_tags_6 },
    { 0x5d39e309, SUC_lexicon_tags_25 },
    { 0xe419e30b, SUC_lexicon_tags_49 },
    { 0xf479e30c, SUC_lexicon_tags_55 },
    { 0xbf9e30d, SUC_lexicon_tags_15 },
    { 0x9079e30c, SUC_lexicon_tags_43 },
    { 0x859e30b, SUC_lexicon_tags_29 },
    { 0x56b9e30f, SUC_lexicon_tags_15 },
    { 0xcb79e30c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x279e313, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd739e31a, SUC_lexicon_tags_47 },
    { 0xf359e31b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe439e31e, SUC_lexicon_tags_57 },
    { 0x1e79e31f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d9e326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc279e32c, SUC_lexicon_tags_37 },
    { 0x86f9e32c, SUC_lexicon_tags_17 },
    { 0x959e32d, SUC_lexicon_tags_3 },
    { 0x6b39e32e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6879e331, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92b9e337, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2719e339, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda79e347, SUC_lexicon_tags_0 },
    { 0x2ed9e348, SUC_lexicon_tags_48 },
    { 0xdcb9e349, SUC_lexicon_tags_45 },
    { 0xa3b9e349, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x57b9e34c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc779e34e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b9e355, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a79e357, SUC_lexicon_tags_8 },
    { 0x5499e358, SUC_lexicon_tags_2 },
    { 0x8019e359, SUC_lexicon_tags_1 },
    { 0xa6b9e358, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad39e35c, SUC_lexicon_tags_17 },
    { 0x1a99e35c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x85b9e360, SUC_lexicon_tags_6 },
    { 0x8d59e360, SUC_lexicon_tags_29 },
    { 0xaeb9e360, SUC_lexicon_tags_47 },
    { 0x12d9e362, SUC_lexicon_tags_38 },
    { 0x8539e364, SUC_lexicon_tags_37 },
    { 0xe059e365, SUC_lexicon_tags_37 },
    { 0x1a99e366, SUC_lexicon_tags_12 },
    { 0xfc59e367, SUC_lexicon_tags_25 },
    { 0xfdb9e368, SUC_lexicon_tags_25 },
    { 0x2659e369, SUC_lexicon_tags_17 },
    { 0xab99e36a, SUC_lexicon_tags_23 },
    { 0x67f9e36b, SUC_lexicon_tags_49 },
    { 0x9c19e36a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6659e36f, SUC_lexicon_tags_2 },
    { 0x4ad9e370, SUC_lexicon_tags_6 },
    { 0xbd99e371, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe19e373, SUC_lexicon_tags_29 },
    { 0xb19e374, SUC_lexicon_tags_37 },
    { 0x36f9e375, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46d9e37a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2559e37e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x80f9e380, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df9e386, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5319e389, SUC_lexicon_tags_59 },
    { 0xb359e38a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf759e38f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b9e394, SUC_lexicon_tags_65 },
    { 0x2ad9e394, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf79e397, SUC_lexicon_tags_7 },
    { 0xfd39e398, SUC_lexicon_tags_2 },
    { 0x9439e398, SUC_lexicon_tags_1 },
    { 0xc679e398, SUC_lexicon_tags_11 },
    { 0xd8b9e399, SUC_lexicon_tags_37 },
    { 0x39b9e39c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa159e3a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a79e3a7, SUC_lexicon_tags_6 },
    { 0x2359e3a7, SUC_lexicon_tags_7 },
    { 0x3b19e3a9, SUC_lexicon_tags_23 },
    { 0x5659e3aa, SUC_lexicon_tags_3 },
    { 0x779e3a8, SUC_lexicon_tags_25 },
    { 0x88d9e3aa, SUC_lexicon_tags_25 },
    { 0xfa59e3ac, SUC_lexicon_tags_89 },
    { 0x27f9e3ae, SUC_lexicon_tags_26 },
    { 0x79f9e3ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc79e3b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d9e3ba, SUC_lexicon_tags_47 },
    { 0x4179e3bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd799e3be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x88d9e3c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf319e3c6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d9e3cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3659e3d1, SUC_lexicon_tags_7 },
    { 0x87b9e3d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd9e3d7, SUC_lexicon_tags_66 },
    { 0x51f9e3d8, SUC_lexicon_tags_17 },
    { 0xebf9e3d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19e3dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf9e3e0, SUC_lexicon_tags_23 },
    { 0x9379e3e0, SUC_lexicon_tags_23 },
    { 0xd8b9e3e0, SUC_lexicon_tags_23 },
    { 0xa6b9e3e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4359e3e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad9e3eb, SUC_lexicon_tags_19 },
    { 0x9279e3ec, SUC_lexicon_tags_15 },
    { 0xaa99e3ed, SUC_lexicon_tags_1 },
    { 0x1999e3ee, SUC_lexicon_tags_8 },
    { 0x8af9e3ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd819e3f4, SUC_lexicon_tags_8 },
    { 0x3fb9e3f5, SUC_lexicon_tags_12 },
    { 0x14f9e3f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cd9e3f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3d9e3fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0d9e400, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfd39e402, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc019e404, SUC_lexicon_tags_23 },
    { 0xc9f9e405, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa359e409, SUC_lexicon_tags_2 },
    { 0x5079e40a, SUC_lexicon_tags_54 },
    { 0xc5b9e40b, SUC_lexicon_tags_299 },
    { 0xcc19e40c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db9e410, SUC_lexicon_tags_60 },
    { 0x7f59e411, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9359e413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9e41c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa419e420, SUC_lexicon_tags_1 },
    { 0x5b79e421, SUC_lexicon_tags_4 },
    { 0xd279e422, SUC_lexicon_tags_15 },
    { 0xdb19e423, SUC_lexicon_tags_31 },
    { 0x9ff9e424, SUC_lexicon_tags_17 },
    { 0xc399e421, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa159e42a, SUC_lexicon_tags_2 },
    { 0x8199e42b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3099e42d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf719e431, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd579e436, SUC_lexicon_tags_11 },
    { 0x81d9e436, SUC_lexicon_tags_17 },
    { 0x15f9e436, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11f9e43a, SUC_lexicon_tags_19 },
    { 0x2f59e43b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc779e43d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b9e440, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa79e442, SUC_lexicon_tags_437 },
    { 0x8d39e443, SUC_lexicon_tags_43 },
    { 0x4bd9e442, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e59e448, SUC_lexicon_tags_38 },
    { 0xb9d9e449, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f79e44b, SUC_lexicon_tags_8 },
    { 0x1839e44c, SUC_lexicon_tags_0 },
    { 0x1d59e44c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b9e451, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad9e454, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f9e45b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb19e45d, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79e461, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2019e465, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72b9e467, SUC_lexicon_tags_9 },
    { 0x20d9e468, SUC_lexicon_tags_2 },
    { 0x74f9e469, SUC_lexicon_tags_60 },
    { 0xb8b9e46a, SUC_lexicon_tags_25 },
    { 0x7f19e46b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b19e46f, SUC_lexicon_tags_19 },
    { 0xab9e470, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1459e473, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76b9e477, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb659e479, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70d9e47b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b9e480, SUC_lexicon_tags_15 },
    { 0x8699e480, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3999e483, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79e489, SUC_lexicon_tags_66 },
    { 0xea79e489, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d59e48e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa319e498, SUC_lexicon_tags_15 },
    { 0xa99e499, SUC_lexicon_tags_11 },
    { 0xfd59e49a, SUC_lexicon_tags_33 },
    { 0x4839e49b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5619e49e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x28f9e4a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1f9e4a2, SUC_lexicon_tags_14 },
    { 0x1739e4a2, SUC_lexicon_tags_27 },
    { 0x5539e4a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3539e4a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2af9e4ac, SUC_lexicon_tags_26 },
    { 0x5979e4ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x66d9e4af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12d9e4b2, SUC_lexicon_tags_26 },
    { 0x9439e4b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd9e4b6, SUC_lexicon_tags_37 },
    { 0x8839e4b7, SUC_lexicon_tags_6 },
    { 0x8ad9e4b6, SUC_lexicon_tags_12 },
    { 0xc5f9e4b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd9e4bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d9e4c6, SUC_lexicon_tags_61 },
    { 0x4859e4c7, SUC_lexicon_tags_12 },
    { 0xd4d9e4c8, SUC_lexicon_tags_45 },
    { 0x1339e4c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7499e4cc, SUC_lexicon_tags_11 },
    { 0x9b39e4cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19d9e4cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8059e4d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69d9e4d5, SUC_lexicon_tags_79 },
    { 0xd579e4d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf059e4e6, SUC_lexicon_tags_37 },
    { 0x32b9e4e6, SUC_lexicon_tags_15 },
    { 0x779e4e8, SUC_lexicon_tags_27 },
    { 0x879e4e9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7839e4eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5019e4f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb879e4f3, SUC_lexicon_tags_63 },
    { 0x6bf9e4f4, SUC_lexicon_tags_11 },
    { 0xd7d9e4f5, SUC_lexicon_tags_34 },
    { 0x2119e4f5, SUC_lexicon_tags_2 },
    { 0xb999e4f5, SUC_lexicon_tags_15 },
    { 0xe99e4f8, SUC_lexicon_tags_47 },
    { 0x8539e4f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf9e4ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd039e504, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb2d9e506, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d9e50e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7539e512, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7139e519, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2699e51c, SUC_lexicon_tags_77 },
    { 0xc679e51d, SUC_lexicon_tags_31 },
    { 0xfad9e51e, SUC_lexicon_tags_37 },
    { 0x819e51f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12d9e523, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d99e52c, SUC_lexicon_tags_38 },
    { 0xf999e52d, SUC_lexicon_tags_38 },
    { 0x6059e52e, SUC_lexicon_tags_1 },
    { 0x5d59e52d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xda99e531, SUC_lexicon_tags_100 },
    { 0x3039e532, SUC_lexicon_tags_37 },
    { 0x6b9e533, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x999e535, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f9e53c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7519e53e, SUC_lexicon_tags_35 },
    { 0xfb79e53e, SUC_lexicon_tags_25 },
    { 0x739e53f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f19e543, SUC_lexicon_tags_53 },
    { 0x1919e544, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d39e549, SUC_lexicon_tags_14 },
    { 0xb259e549, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b9e54e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3739e551, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3479e556, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x90d9e558, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa659e55a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99e55d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4899e564, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a39e566, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0d9e568, SUC_lexicon_tags_26 },
    { 0x9739e569, SUC_lexicon_tags_28 },
    { 0xeff9e56a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7859e56d, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0x7659e570, SUC_lexicon_tags_25 },
    { 0xba79e571, SUC_lexicon_tags_26 },
    { 0xead9e572, SUC_lexicon_tags_43 },
    { 0x6099e572, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe979e576, SUC_lexicon_tags_56 },
    { 0xa419e577, SUC_lexicon_tags_12 },
    { 0xb2d9e578, SUC_lexicon_tags_43 },
    { 0x7bf9e578, SUC_lexicon_tags_15 },
    { 0xf9d9e576, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x86d9e57c, SUC_lexicon_tags_1 },
    { 0xa0b9e57c, SUC_lexicon_tags_47 },
    { 0x4399e57e, SUC_lexicon_tags_9 },
    { 0x3a59e57f, SUC_lexicon_tags_70 },
    { 0xcff9e580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd79e585, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff9e588, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7519e58a, SUC_lexicon_tags_87 },
    { 0xa259e58b, SUC_lexicon_tags_37 },
    { 0x599e58b, SUC_lexicon_tags_507 },
    { 0xfa79e58d, SUC_lexicon_tags_2 },
    { 0xf7d9e58d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0b9e591, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf39e593, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd79e596, SUC_lexicon_tags_7 },
    { 0x50f9e596, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b79e59b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20d9e59f, SUC_lexicon_tags_29 },
    { 0x7479e59f, SUC_lexicon_tags_15 },
    { 0xbdf9e59f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeed9e5a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99e5b4, SUC_lexicon_tags_87 },
    { 0x2ad9e5b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39e5b8, SUC_lexicon_tags_13 },
    { 0x1219e5b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f79e5bc, SUC_lexicon_tags_4 },
    { 0x31d9e5bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d99e5c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f9e5c6, SUC_lexicon_tags_27 },
    { 0x1c19e5c7, SUC_lexicon_tags_7 },
    { 0x9d19e5c8, SUC_lexicon_tags_4 },
    { 0x3259e5c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9e5ce, SUC_lexicon_tags_19 },
    { 0x27d9e5cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9e5d4, SUC_lexicon_tags_19 },
    { 0xb939e5d5, SUC_lexicon_tags_21 },
    { 0xb019e5d6, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xa499e5d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6219e5db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ad9e5dd, SUC_lexicon_tags_63 },
    { 0x1b9e5dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9e5e7, SUC_lexicon_tags_26 },
    { 0x7d19e5e7, SUC_lexicon_tags_14 },
    { 0xf339e5e9, SUC_lexicon_tags_12 },
    { 0x8f19e5e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1799e5ee, SUC_lexicon_tags_60 },
    { 0xe439e5ef, SUC_lexicon_tags_4 },
    { 0x71d9e5ef, SUC_lexicon_tags_37 },
    { 0x71f9e5f0, SUC_lexicon_tags_16 },
    { 0x44d9e5f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8339e5f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0d9e5fb, SUC_lexicon_tags_30 },
    { 0x5859e5fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x89d9e5fe, SUC_lexicon_tags_21 },
    { 0x7419e5fe, SUC_lexicon_tags_37 },
    { 0x5fb9e600, SUC_lexicon_tags_25 },
    { 0x8a99e601, SUC_lexicon_tags_10 },
    { 0x1739e5ff, SUC_lexicon_tags_49 },
    { 0x9219e601, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd479e606, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6f9e608, SUC_lexicon_tags_41 },
    { 0xf9e608, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8519e60b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4639e60f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88f9e613, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d39e616, SUC_lexicon_tags_30 },
    { 0xad99e617, SUC_lexicon_tags_4 },
    { 0x1bd9e618, SUC_lexicon_tags_43 },
    { 0xe179e619, SUC_lexicon_tags_47 },
    { 0x7539e619, SUC_lexicon_tags_15 },
    { 0xf659e617, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xabb9e61d, SUC_lexicon_tags_3 },
    { 0x9e39e61e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84b9e621, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb419e625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe819e628, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f9e634, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b59e637, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x37b9e63a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1139e63d, SUC_lexicon_tags_65 },
    { 0xcd9e63e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff9e642, SUC_lexicon_tags_7 },
    { 0x9f59e642, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6459e645, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd939e64a, SUC_lexicon_tags_17 },
    { 0xe4d9e64a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd019e64d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f9e651, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6199e653, SUC_lexicon_tags_12 },
    { 0x38d9e654, SUC_lexicon_tags_11 },
    { 0xe7f9e653, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a79e657, SUC_lexicon_tags_38 },
    { 0xf4d9e658, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39e666, SUC_lexicon_tags_2 },
    { 0xf539e667, SUC_lexicon_tags_37 },
    { 0x3779e667, SUC_lexicon_tags_61 },
    { 0xba19e669, SUC_lexicon_tags_14 },
    { 0xb939e66a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94b9e66c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed19e674, SUC_lexicon_tags_15 },
    { 0xe279e675, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9139e678, SUC_lexicon_tags_27 },
    { 0xe6b9e678, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d9e67c, SUC_lexicon_tags_10 },
    { 0x50d9e67c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdef9e680, SUC_lexicon_tags_26 },
    { 0x4c59e681, SUC_lexicon_tags_40 },
    { 0x7c99e682, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa519e687, SUC_lexicon_tags_60 },
    { 0xc839e688, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x919e68b, SUC_lexicon_tags_48 },
    { 0x5259e68b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5079e693, SUC_lexicon_tags_82 },
    { 0x6839e693, SUC_lexicon_tags_70 },
    { 0xc219e695, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7d9e697, SUC_lexicon_tags_25 },
    { 0x8119e698, SUC_lexicon_tags_43 },
    { 0x9b59e698, SUC_lexicon_tags_63 },
    { 0x9b79e699, SUC_lexicon_tags_33 },
    { 0xfa19e69a, SUC_lexicon_tags_28 },
    { 0x56f9e699, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8259e69e, SUC_lexicon_tags_13 },
    { 0x4cb9e69f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf679e6a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b9e6a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5139e6ae, SUC_lexicon_tags_38 },
    { 0x8119e6af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf399e6b2, SUC_lexicon_tags_27 },
    { 0xa899e6b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82f9e6bb, SUC_lexicon_tags_1 },
    { 0x9719e6bc, SUC_lexicon_tags_10 },
    { 0xacb9e6bd, SUC_lexicon_tags_47 },
    { 0x5bb9e6bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b9e6c5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x65d9e6c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef9e6cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cb9e6cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6b9e6d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xea39e6d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5b9e6da, SUC_lexicon_tags_19 },
    { 0xcf19e6db, SUC_lexicon_tags_0 },
    { 0xd399e6db, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xe399e6de, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af9e6e2, SUC_lexicon_tags_2 },
    { 0xa599e6e3, SUC_lexicon_tags_31 },
    { 0x13b9e6e3, SUC_lexicon_tags_69 },
    { 0xfdb9e6e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5cd9e6e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1659e6ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4a19e6ec, SUC_lexicon_tags_7 },
    { 0x64b9e6ed, SUC_lexicon_tags_11 },
    { 0x7439e6ee, SUC_lexicon_tags_2 },
    { 0x4f39e6ee, SUC_lexicon_tags_1 },
    { 0x6119e6ef, SUC_lexicon_tags_15 },
    { 0xe8d9e6f1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4159e6f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d9e6fa, SUC_lexicon_tags_21 },
    { 0x43f9e6fa, SUC_lexicon_tags_11 },
    { 0x38f9e6fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc59e702, SUC_lexicon_tags_11 },
    { 0x7679e703, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8a79e705, SUC_lexicon_tags_11 },
    { 0x78f9e705, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb79e708, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e39e70d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ef9e710, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xec39e713, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc019e719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf799e71c, SUC_lexicon_tags_35 },
    { 0x3a19e71c, SUC_lexicon_tags_1405 },
    { 0, NULL },
    { 0xfe59e71f, SUC_lexicon_tags_79 },
    { 0xf119e720, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x26b9e723, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3059e727, SUC_lexicon_tags_11 },
    { 0xcf9e727, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb479e72c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf939e730, SUC_lexicon_tags_14 },
    { 0x7739e731, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79e736, SUC_lexicon_tags_2 },
    { 0x6839e737, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cd9e73a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b99e73d, SUC_lexicon_tags_1 },
    { 0x9859e73e, SUC_lexicon_tags_7 },
    { 0x7539e73f, SUC_lexicon_tags_57 },
    { 0x20f9e73d, SUC_lexicon_tags_48 },
    { 0xfd79e73d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6b9e744, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd9e749, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf9e74b, SUC_lexicon_tags_11 },
    { 0xda19e74c, SUC_lexicon_tags_70 },
    { 0x19d9e74d, SUC_lexicon_tags_2 },
    { 0xc6f9e74d, SUC_lexicon_tags_17 },
    { 0x57b9e74b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d9e756, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f59e75a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d9e763, SUC_lexicon_tags_11 },
    { 0x3eb9e764, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2db9e766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd9e771, SUC_lexicon_tags_62 },
    { 0x4a99e772, SUC_lexicon_tags_14 },
    { 0xe959e773, SUC_lexicon_tags_30 },
    { 0x8279e774, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf219e777, SUC_lexicon_tags_10 },
    { 0x8ed9e777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b39e77a, SUC_lexicon_tags_1 },
    { 0x6879e77a, SUC_lexicon_tags_15 },
    { 0xf8d9e77a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf959e77f, SUC_lexicon_tags_35 },
    { 0xc499e780, SUC_lexicon_tags_11 },
    { 0x2059e781, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9639e783, SUC_lexicon_tags_8 },
    { 0x5499e784, SUC_lexicon_tags_11 },
    { 0x42f9e785, SUC_lexicon_tags_7 },
    { 0x32d9e783, SUC_lexicon_tags_12 },
    { 0x4679e787, SUC_lexicon_tags_15 },
    { 0xdf79e787, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3419e78a, SUC_lexicon_tags_7 },
    { 0x7c59e78a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8459e78f, SUC_lexicon_tags_28 },
    { 0x7d59e790, SUC_lexicon_tags_31 },
    { 0x4779e791, SUC_lexicon_tags_17 },
    { 0x3399e791, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe999e797, SUC_lexicon_tags_39 },
    { 0x2979e797, SUC_lexicon_tags_38 },
    { 0x82d9e799, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9839e79b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d79e79d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c59e79f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59e7aa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439e7b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4d9e7b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3319e7b7, SUC_lexicon_tags_2 },
    { 0x61f9e7b7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6039e7bc, SUC_lexicon_tags_12 },
    { 0xffb9e7bc, SUC_lexicon_tags_77 },
    { 0x4bb9e7bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c39e7c1, SUC_lexicon_tags_14 },
    { 0x1199e7c2, SUC_lexicon_tags_15 },
    { 0x1e39e7c3, SUC_lexicon_tags_11 },
    { 0x1a39e7c4, SUC_lexicon_tags_33 },
    { 0x89d9e7c5, SUC_lexicon_tags_118 },
    { 0x919e7c6, SUC_lexicon_tags_33 },
    { 0x1df9e7c2, SUC_lexicon_tags_25 },
    { 0x8599e7c1, SUC_lexicon_tags_17 },
    { 0x7ad9e7c8, SUC_lexicon_tags_63 },
    { 0x6759e7c1, SUC_lexicon_tags_40 },
    { 0x43f9e7cb, SUC_lexicon_tags_71 },
    { 0x9539e7c5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6879e7d1, SUC_lexicon_tags_25 },
    { 0xc439e7d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbf9e7d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e39e7d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x30d9e7d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4239e7e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e99e7e3, SUC_lexicon_tags_60 },
    { 0x16f9e7e4, SUC_lexicon_tags_25 },
    { 0x4e19e7e5, SUC_lexicon_tags_33 },
    { 0x3819e7e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d9e7ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a19e7ec, SUC_lexicon_tags_47 },
    { 0x2419e7ed, SUC_lexicon_tags_17 },
    { 0x7a19e7ee, SUC_lexicon_tags_11 },
    { 0xa499e7ef, SUC_lexicon_tags_12 },
    { 0x94b9e7f0, SUC_lexicon_tags_39 },
    { 0x50b9e7ef, SUC_lexicon_tags_37 },
    { 0xe639e7ed, SUC_lexicon_tags_1 },
    { 0xddb9e7ef, SUC_lexicon_tags_7 },
    { 0xe539e7ef, SUC_lexicon_tags_1 },
    { 0xf139e7f5, SUC_lexicon_tags_20 },
    { 0xbfb9e7ee, SUC_lexicon_tags_17 },
    { 0x8c79e7f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb9e7fa, SUC_lexicon_tags_39 },
    { 0x3999e7fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf19e801, SUC_lexicon_tags_38 },
    { 0x8439e802, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b9e804, SUC_lexicon_tags_47 },
    { 0x1a19e805, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5739e809, SUC_lexicon_tags_127 },
    { 0xcaf9e80a, SUC_lexicon_tags_11 },
    { 0xd0d9e80b, SUC_lexicon_tags_17 },
    { 0x8ad9e80b, SUC_lexicon_tags_25 },
    { 0x49d9e80d, SUC_lexicon_tags_11 },
    { 0x159e80e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf39e810, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2bf9e812, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b9e815, SUC_lexicon_tags_63 },
    { 0xdb9e815, SUC_lexicon_tags_8 },
    { 0x4879e817, SUC_lexicon_tags_65 },
    { 0x8f9e818, SUC_lexicon_tags_1 },
    { 0x70d9e818, SUC_lexicon_tags_343 },
    { 0x5d19e81a, SUC_lexicon_tags_37 },
    { 0x8179e81b, SUC_lexicon_tags_32 },
    { 0x5399e81b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a19e81f, SUC_lexicon_tags_77 },
    { 0x6af9e81f, SUC_lexicon_tags_17 },
    { 0x10d9e821, SUC_lexicon_tags_14 },
    { 0x8d99e820, SUC_lexicon_tags_23 },
    { 0x1759e823, SUC_lexicon_tags_23 },
    { 0x5719e824, SUC_lexicon_tags_17 },
    { 0x8b79e81f, SUC_lexicon_tags_12 },
    { 0x3a79e826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc659e82b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1159e82d, SUC_lexicon_tags_28 },
    { 0x2e19e82e, SUC_lexicon_tags_2 },
    { 0xce19e82f, SUC_lexicon_tags_35 },
    { 0x6eb9e82d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d99e836, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c59e839, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b99e83e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d9e840, SUC_lexicon_tags_9 },
    { 0x8699e840, SUC_lexicon_tags_781 },
    { 0, NULL },
    { 0x9999e843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4099e846, SUC_lexicon_tags_14 },
    { 0x3e59e846, SUC_lexicon_tags_15 },
    { 0xfef9e847, SUC_lexicon_tags_92 },
    { 0x2639e848, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5619e84c, SUC_lexicon_tags_33 },
    { 0x26d9e84d, SUC_lexicon_tags_27 },
    { 0x4f59e84e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb759e851, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df9e857, SUC_lexicon_tags_12 },
    { 0x9479e858, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2f9e863, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf9e86b, SUC_lexicon_tags_45 },
    { 0x2e19e86c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa39e871, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb9e876, SUC_lexicon_tags_11 },
    { 0x9159e876, SUC_lexicon_tags_29 },
    { 0x42b9e878, SUC_lexicon_tags_63 },
    { 0x9899e879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef39e881, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ab9e886, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x20b9e888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29b9e88a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19e891, SUC_lexicon_tags_11 },
    { 0x3119e892, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd699e895, SUC_lexicon_tags_7 },
    { 0x47d9e896, SUC_lexicon_tags_31 },
    { 0xa7d9e896, SUC_lexicon_tags_2 },
    { 0x859e897, SUC_lexicon_tags_25 },
    { 0x1199e899, SUC_lexicon_tags_15 },
    { 0x5239e89a, SUC_lexicon_tags_26 },
    { 0x4199e89b, SUC_lexicon_tags_37 },
    { 0xed99e897, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x28b9e89f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc79e8a2, SUC_lexicon_tags_49 },
    { 0x1b99e8a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf19e8a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c39e8ac, SUC_lexicon_tags_23 },
    { 0x6619e8ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1259e8b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf179e8b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd899e8ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xded9e8bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2db9e8c1, SUC_lexicon_tags_70 },
    { 0x7e39e8c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb399e8c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf099e8ca, SUC_lexicon_tags_2 },
    { 0xf239e8cb, SUC_lexicon_tags_74 },
    { 0x2979e8ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d9e8d0, SUC_lexicon_tags_9 },
    { 0x4479e8d1, SUC_lexicon_tags_7 },
    { 0xad19e8d1, SUC_lexicon_tags_64 },
    { 0x9699e8d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b9e8d5, SUC_lexicon_tags_28 },
    { 0x6fd9e8d6, SUC_lexicon_tags_11 },
    { 0xbd99e8d7, SUC_lexicon_tags_46 },
    { 0xb299e8d8, SUC_lexicon_tags_12 },
    { 0x1659e8d9, SUC_lexicon_tags_0 },
    { 0x56d9e8d9, SUC_lexicon_tags_37 },
    { 0xeeb9e8db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe059e8df, SUC_lexicon_tags_17 },
    { 0x3f9e8e0, SUC_lexicon_tags_47 },
    { 0x34d9e8e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf399e8e7, SUC_lexicon_tags_26 },
    { 0xc7b9e8e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dd9e8eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabd9e8ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf9e8f1, SUC_lexicon_tags_47 },
    { 0xf619e8f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa539e8f4, SUC_lexicon_tags_23 },
    { 0xc9b9e8f5, SUC_lexicon_tags_21 },
    { 0x2959e8f6, SUC_lexicon_tags_15 },
    { 0xcf19e8f6, SUC_lexicon_tags_17 },
    { 0x4f99e8f4, SUC_lexicon_tags_65 },
    { 0xb079e8f6, SUC_lexicon_tags_12 },
    { 0x9cd9e8f4, SUC_lexicon_tags_14 },
    { 0xead9e8fb, SUC_lexicon_tags_15 },
    { 0x8ef9e8f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd939e8ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76b9e901, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7059e90f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb059e913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc639e918, SUC_lexicon_tags_70 },
    { 0x48b9e919, SUC_lexicon_tags_11 },
    { 0x45f9e91a, SUC_lexicon_tags_44 },
    { 0x1759e91b, SUC_lexicon_tags_11 },
    { 0x3d19e91c, SUC_lexicon_tags_17 },
    { 0x5e99e91d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x38f9e920, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35d9e923, SUC_lexicon_tags_65 },
    { 0xe059e924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb79e929, SUC_lexicon_tags_28 },
    { 0xb279e92a, SUC_lexicon_tags_11 },
    { 0x4e39e929, SUC_lexicon_tags_25 },
    { 0x5559e92b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78d9e92e, SUC_lexicon_tags_39 },
    { 0x53b9e92f, SUC_lexicon_tags_12 },
    { 0xb9e930, SUC_lexicon_tags_67 },
    { 0x82b9e931, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf979e936, SUC_lexicon_tags_14 },
    { 0xb559e937, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad79e93a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8879e93e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb79e941, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa379e946, SUC_lexicon_tags_15 },
    { 0x5bf9e947, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2119e94b, SUC_lexicon_tags_27 },
    { 0xadd9e94c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73f9e94e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1159e950, SUC_lexicon_tags_2 },
    { 0xdfb9e950, SUC_lexicon_tags_1 },
    { 0x6259e952, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf119e954, SUC_lexicon_tags_103 },
    { 0xc899e955, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9699e95b, SUC_lexicon_tags_19 },
    { 0x939e95c, SUC_lexicon_tags_37 },
    { 0xb4b9e95d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a79e961, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3279e968, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe459e96d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8119e972, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa319e975, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6379e977, SUC_lexicon_tags_1 },
    { 0xdef9e978, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8b9e97b, SUC_lexicon_tags_9 },
    { 0xe319e97c, SUC_lexicon_tags_33 },
    { 0x96b9e97b, SUC_lexicon_tags_38 },
    { 0x1d99e97b, SUC_lexicon_tags_49 },
    { 0x2b79e97c, SUC_lexicon_tags_26 },
    { 0xd579e97e, SUC_lexicon_tags_27 },
    { 0x61f9e97b, SUC_lexicon_tags_17 },
    { 0x4639e982, SUC_lexicon_tags_37 },
    { 0xaef9e983, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2319e985, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ff9e987, SUC_lexicon_tags_2 },
    { 0x3d59e987, SUC_lexicon_tags_37 },
    { 0x4519e989, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6939e98b, SUC_lexicon_tags_60 },
    { 0x2279e98c, SUC_lexicon_tags_184 },
    { 0x3179e98d, SUC_lexicon_tags_55 },
    { 0xcb39e98e, SUC_lexicon_tags_4 },
    { 0x8399e98f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff9e993, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb619e995, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb59e997, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe959e9a0, SUC_lexicon_tags_2 },
    { 0xa2d9e9a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e19e9a5, SUC_lexicon_tags_7 },
    { 0x92b9e9a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4119e9a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb39e9b1, SUC_lexicon_tags_21 },
    { 0x4599e9b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc919e9b8, SUC_lexicon_tags_15 },
    { 0xbf39e9b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x77d9e9bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b79e9bf, SUC_lexicon_tags_60 },
    { 0xdc39e9c0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa159e9c2, SUC_lexicon_tags_46 },
    { 0x9f9e9c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6359e9c5, SUC_lexicon_tags_30 },
    { 0x4179e9c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9919e9cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9e9d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9e9d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa3b9e9d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2799e9d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0d9e9e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1fd9e9e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b9e9e7, SUC_lexicon_tags_25 },
    { 0x1d9e9e7, SUC_lexicon_tags_60 },
    { 0x4e59e9e7, SUC_lexicon_tags_28 },
    { 0x7659e9ea, SUC_lexicon_tags_49 },
    { 0xb1d9e9e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6619e9ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2139e9f4, SUC_lexicon_tags_15 },
    { 0x6cb9e9f5, SUC_lexicon_tags_25 },
    { 0xd4d9e9f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42f9e9fa, SUC_lexicon_tags_17 },
    { 0xea99e9fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e99e9fd, SUC_lexicon_tags_11 },
    { 0xdb39e9fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79ea04, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe459ea07, SUC_lexicon_tags_37 },
    { 0xf0b9ea08, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f9ea0e, SUC_lexicon_tags_11 },
    { 0xe19ea0f, SUC_lexicon_tags_15 },
    { 0x5d39ea0f, SUC_lexicon_tags_55 },
    { 0xb559ea11, SUC_lexicon_tags_10 },
    { 0x8d39ea10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa419ea1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cb9ea1c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a39ea23, SUC_lexicon_tags_43 },
    { 0xea59ea24, SUC_lexicon_tags_42 },
    { 0x8839ea25, SUC_lexicon_tags_17 },
    { 0xffd9ea25, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d9ea2a, SUC_lexicon_tags_29 },
    { 0xdd19ea2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7eb9ea2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0d9ea32, SUC_lexicon_tags_11 },
    { 0xbf59ea33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7f79ea35, SUC_lexicon_tags_34 },
    { 0x7f59ea35, SUC_lexicon_tags_12 },
    { 0xfd59ea37, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6279ea3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f9ea3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5999ea43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac19ea47, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6679ea4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f9ea52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb9ea54, SUC_lexicon_tags_6 },
    { 0xd199ea55, SUC_lexicon_tags_61 },
    { 0xcfd9ea56, SUC_lexicon_tags_31 },
    { 0xa39ea56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6d9ea5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a39ea5e, SUC_lexicon_tags_7 },
    { 0x68d9ea5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2819ea6c, SUC_lexicon_tags_2 },
    { 0x819ea6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4959ea6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf199ea75, SUC_lexicon_tags_37 },
    { 0x5eb9ea76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d79ea7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9859ea7f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa759ea81, SUC_lexicon_tags_37 },
    { 0xea99ea82, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7459ea84, SUC_lexicon_tags_62 },
    { 0x3c99ea85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd739ea90, SUC_lexicon_tags_23 },
    { 0xb119ea91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa219ea98, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd9ea9c, SUC_lexicon_tags_1248 },
    { 0x9479ea9d, SUC_lexicon_tags_2 },
    { 0xca99ea9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9eaa3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4b9eaa7, SUC_lexicon_tags_7 },
    { 0xd6d9eaa8, SUC_lexicon_tags_43 },
    { 0xbd99eaa9, SUC_lexicon_tags_37 },
    { 0x4b9eaa9, SUC_lexicon_tags_37 },
    { 0x67d9eaaa, SUC_lexicon_tags_1 },
    { 0xb2d9eaaa, SUC_lexicon_tags_7 },
    { 0x9919eaaa, SUC_lexicon_tags_45 },
    { 0x44f9eaae, SUC_lexicon_tags_8 },
    { 0x6199eaac, SUC_lexicon_tags_12 },
    { 0x159eaab, SUC_lexicon_tags_15 },
    { 0xb7d9eab1, SUC_lexicon_tags_3 },
    { 0x42d9eaa9, SUC_lexicon_tags_33 },
    { 0x47d9eaa7, SUC_lexicon_tags_19 },
    { 0xa9d9eab4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e19eab7, SUC_lexicon_tags_12 },
    { 0x679eab8, SUC_lexicon_tags_257 },
    { 0x3bf9eab8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8779eabc, SUC_lexicon_tags_66 },
    { 0x579eabd, SUC_lexicon_tags_17 },
    { 0xa6d9eabc, SUC_lexicon_tags_66 },
    { 0x9d99eabf, SUC_lexicon_tags_2 },
    { 0x9b19eac0, SUC_lexicon_tags_7 },
    { 0x6559eac1, SUC_lexicon_tags_11 },
    { 0xf419eac2, SUC_lexicon_tags_63 },
    { 0xb6d9eac3, SUC_lexicon_tags_60 },
    { 0xd5d9eac4, SUC_lexicon_tags_19 },
    { 0x8b99eac5, SUC_lexicon_tags_74 },
    { 0x2cd9eac4, SUC_lexicon_tags_8 },
    { 0x8bb9eac1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd979eaca, SUC_lexicon_tags_79 },
    { 0xf839eaca, SUC_lexicon_tags_248 },
    { 0xb19eaca, SUC_lexicon_tags_17 },
    { 0x5639eacb, SUC_lexicon_tags_25 },
    { 0x13d9eace, SUC_lexicon_tags_17 },
    { 0x8e59eace, SUC_lexicon_tags_47 },
    { 0x7879ead0, SUC_lexicon_tags_60 },
    { 0x8ef9eacb, SUC_lexicon_tags_1 },
    { 0x6c39eace, SUC_lexicon_tags_37 },
    { 0xbdb9eacd, SUC_lexicon_tags_8 },
    { 0xe979ead4, SUC_lexicon_tags_84 },
    { 0x3419ead5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64b9eada, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b9eade, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6839eae2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xad99eae5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4799eae8, SUC_lexicon_tags_37 },
    { 0x7859eae9, SUC_lexicon_tags_8 },
    { 0x1cb9eaea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd9eaed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9099eaf5, SUC_lexicon_tags_23 },
    { 0x7c19eaf6, SUC_lexicon_tags_6 },
    { 0xf39eaf5, SUC_lexicon_tags_57 },
    { 0x5f79eaf6, SUC_lexicon_tags_39 },
    { 0x8999eaf9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b9eafe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f9eb02, SUC_lexicon_tags_28 },
    { 0x2c59eb02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd659eb08, SUC_lexicon_tags_12 },
    { 0x7bd9eb08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c39eb11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd59eb14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9139eb18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4159eb1b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd2f9eb1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b9eb20, SUC_lexicon_tags_1 },
    { 0xf479eb21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fb9eb23, SUC_lexicon_tags_40 },
    { 0xe619eb24, SUC_lexicon_tags_28 },
    { 0x4959eb23, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6db9eb27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a99eb2b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0b9eb2f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad19eb33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba99eb35, SUC_lexicon_tags_29 },
    { 0x9f99eb35, SUC_lexicon_tags_60 },
    { 0x7259eb37, SUC_lexicon_tags_25 },
    { 0x5439eb37, SUC_lexicon_tags_65 },
    { 0x7f79eb37, SUC_lexicon_tags_25 },
    { 0xfdd9eb3a, SUC_lexicon_tags_68 },
    { 0x6ad9eb3b, SUC_lexicon_tags_37 },
    { 0x56b9eb3a, SUC_lexicon_tags_2 },
    { 0xf199eb3d, SUC_lexicon_tags_43 },
    { 0xeed9eb3d, SUC_lexicon_tags_9 },
    { 0x9659eb3f, SUC_lexicon_tags_100 },
    { 0xcd59eb40, SUC_lexicon_tags_12 },
    { 0x3439eb41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e99eb46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39eb4e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbcf9eb50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86b9eb52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1399eb55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeed9eb58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a19eb5d, SUC_lexicon_tags_6 },
    { 0x4cf9eb5d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9eb64, SUC_lexicon_tags_37 },
    { 0xe9b9eb65, SUC_lexicon_tags_23 },
    { 0xa559eb65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xad59eb69, SUC_lexicon_tags_47 },
    { 0x69b9eb6a, SUC_lexicon_tags_39 },
    { 0x4019eb6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d39eb6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a59eb72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf9eb76, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d9eb7d, SUC_lexicon_tags_1 },
    { 0xfe59eb7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6499eb80, SUC_lexicon_tags_47 },
    { 0x7d99eb80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x479eb83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b79eb89, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xcc79eb8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12d9eb8d, SUC_lexicon_tags_1 },
    { 0x9319eb8d, SUC_lexicon_tags_17 },
    { 0x7af9eb8f, SUC_lexicon_tags_29 },
    { 0x6859eb90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6519eb95, SUC_lexicon_tags_218 },
    { 0xd659eb96, SUC_lexicon_tags_25 },
    { 0xf9eb95, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1cb9eb99, SUC_lexicon_tags_25 },
    { 0x1f19eb9a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a39eb9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2379eb9f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa639eba1, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0x6039eba4, SUC_lexicon_tags_19 },
    { 0xfb79eba5, SUC_lexicon_tags_12 },
    { 0x8899eba6, SUC_lexicon_tags_13 },
    { 0xff99eba6, SUC_lexicon_tags_15 },
    { 0x5e19eba5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef9ebac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4239ebae, SUC_lexicon_tags_14 },
    { 0xcfb9ebae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7579ebb2, SUC_lexicon_tags_12 },
    { 0xe6f9ebb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec39ebb8, SUC_lexicon_tags_7 },
    { 0x92b9ebb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x33d9ebbb, SUC_lexicon_tags_21 },
    { 0x2d79ebbc, SUC_lexicon_tags_11 },
    { 0x7999ebbc, SUC_lexicon_tags_31 },
    { 0x5959ebbe, SUC_lexicon_tags_11 },
    { 0x8799ebbd, SUC_lexicon_tags_15 },
    { 0xf1f9ebc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa99ebc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7719ebcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d9ebd1, SUC_lexicon_tags_25 },
    { 0xde99ebd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3659ebd6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0f9ebdb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcf99ebdd, SUC_lexicon_tags_12 },
    { 0x9239ebdd, SUC_lexicon_tags_37 },
    { 0xb459ebdf, SUC_lexicon_tags_39 },
    { 0xf579ebe0, SUC_lexicon_tags_2 },
    { 0x9859ebdd, SUC_lexicon_tags_25 },
    { 0x2219ebe2, SUC_lexicon_tags_11 },
    { 0x15b9ebdd, SUC_lexicon_tags_37 },
    { 0xc5b9ebe4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87f9ebe6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f79ebe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa379ebec, SUC_lexicon_tags_10 },
    { 0xdf9ebec, SUC_lexicon_tags_33 },
    { 0x6f79ebed, SUC_lexicon_tags_30 },
    { 0xdd19ebef, SUC_lexicon_tags_2 },
    { 0xaaf9ebef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdeb9ebf2, SUC_lexicon_tags_2 },
    { 0x1519ebf3, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b39ebf6, SUC_lexicon_tags_7 },
    { 0xc5b9ebf7, SUC_lexicon_tags_2 },
    { 0x2219ebf8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b99ebfe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b9ec06, SUC_lexicon_tags_9 },
    { 0x8819ec07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd9ec09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6599ec0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ed9ec12, SUC_lexicon_tags_61 },
    { 0x3239ec13, SUC_lexicon_tags_26 },
    { 0x4df9ec12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2319ec16, SUC_lexicon_tags_25 },
    { 0x38b9ec17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe999ec19, SUC_lexicon_tags_4 },
    { 0x2059ec1a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d79ec22, SUC_lexicon_tags_22 },
    { 0xb5f9ec23, SUC_lexicon_tags_45 },
    { 0x75f9ec22, SUC_lexicon_tags_25 },
    { 0x6359ec25, SUC_lexicon_tags_29 },
    { 0xe4d9ec26, SUC_lexicon_tags_57 },
    { 0xbb9ec27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa959ec31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe359ec34, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xdfb9ec36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86f9ec3a, SUC_lexicon_tags_19 },
    { 0x88b9ec3a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6279ec3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9979ec41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x439ec44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8379ec46, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d79ec4c, SUC_lexicon_tags_25 },
    { 0x17d9ec4d, SUC_lexicon_tags_30 },
    { 0xdab9ec4c, SUC_lexicon_tags_20 },
    { 0xbf79ec4f, SUC_lexicon_tags_0 },
    { 0x4459ec50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d99ec58, SUC_lexicon_tags_424 },
    { 0x8c39ec59, SUC_lexicon_tags_16 },
    { 0x2859ec59, SUC_lexicon_tags_12 },
    { 0xacb9ec5a, SUC_lexicon_tags_37 },
    { 0x6759ec5c, SUC_lexicon_tags_12 },
    { 0xf5f9ec5c, SUC_lexicon_tags_29 },
    { 0x1c99ec5d, SUC_lexicon_tags_28 },
    { 0x84b9ec5e, SUC_lexicon_tags_8 },
    { 0x99b9ec5b, SUC_lexicon_tags_46 },
    { 0x4439ec60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x34b9ec64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94f9ec68, SUC_lexicon_tags_55 },
    { 0x2099ec69, SUC_lexicon_tags_10 },
    { 0x6fd9ec68, SUC_lexicon_tags_17 },
    { 0x66f9ec6b, SUC_lexicon_tags_0 },
    { 0x5999ec6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb19ec6e, SUC_lexicon_tags_14 },
    { 0x7979ec6f, SUC_lexicon_tags_1 },
    { 0xd839ec70, SUC_lexicon_tags_37 },
    { 0x75f9ec71, SUC_lexicon_tags_17 },
    { 0x1399ec72, SUC_lexicon_tags_60 },
    { 0x5a59ec73, SUC_lexicon_tags_16 },
    { 0xca19ec73, SUC_lexicon_tags_4 },
    { 0xff39ec74, SUC_lexicon_tags_11 },
    { 0x6679ec6e, SUC_lexicon_tags_29 },
    { 0xb399ec77, SUC_lexicon_tags_37 },
    { 0x7459ec77, SUC_lexicon_tags_34 },
    { 0xa1d9ec79, SUC_lexicon_tags_27 },
    { 0x24f9ec79, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8e59ec7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc699ec7e, SUC_lexicon_tags_85 },
    { 0x55d9ec7f, SUC_lexicon_tags_45 },
    { 0x3099ec7e, SUC_lexicon_tags_65 },
    { 0x8b59ec80, SUC_lexicon_tags_27 },
    { 0x9499ec82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b79ec84, SUC_lexicon_tags_7 },
    { 0x1e59ec85, SUC_lexicon_tags_1 },
    { 0x5279ec84, SUC_lexicon_tags_31 },
    { 0x2d79ec84, SUC_lexicon_tags_2 },
    { 0x7f19ec87, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9459ec8f, SUC_lexicon_tags_37 },
    { 0x91f9ec90, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe299ec95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4799ec99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eb9ec9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70b9eca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7679eca4, SUC_lexicon_tags_25 },
    { 0xed79eca5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e19eca7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x63b9eca9, SUC_lexicon_tags_29 },
    { 0xef99ecaa, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xb739ecad, SUC_lexicon_tags_37 },
    { 0x6499ecad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9ecb1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7119ecb9, SUC_lexicon_tags_25 },
    { 0xfcd9ecba, SUC_lexicon_tags_27 },
    { 0xa939ecba, SUC_lexicon_tags_2 },
    { 0xc559ecbc, SUC_lexicon_tags_17 },
    { 0x86f9ecbc, SUC_lexicon_tags_1 },
    { 0xc219ecb9, SUC_lexicon_tags_25 },
    { 0x3319ecbf, SUC_lexicon_tags_26 },
    { 0xd679ecc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc599ecc2, SUC_lexicon_tags_27 },
    { 0x64d9ecc2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3a59ecc5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f19ecc8, SUC_lexicon_tags_9 },
    { 0x1819ecc8, SUC_lexicon_tags_49 },
    { 0xf479ecca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee59eccd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeb9ecd1, SUC_lexicon_tags_30 },
    { 0xdcf9ecd2, SUC_lexicon_tags_60 },
    { 0x69b9ecd3, SUC_lexicon_tags_19 },
    { 0xb699ecd3, SUC_lexicon_tags_11 },
    { 0x9a39ecd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x44b9ecd8, SUC_lexicon_tags_2 },
    { 0x3839ecd9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7919ecdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b9ece4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239ece8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f9eceb, SUC_lexicon_tags_29 },
    { 0x1b9eceb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8139ecf0, SUC_lexicon_tags_0 },
    { 0x9c79ecf1, SUC_lexicon_tags_37 },
    { 0x9f19ecf2, SUC_lexicon_tags_0 },
    { 0xd379ecf1, SUC_lexicon_tags_15 },
    { 0x599ecf2, SUC_lexicon_tags_14 },
    { 0x9139ecf3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd239ecf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1159ecfb, SUC_lexicon_tags_11 },
    { 0x4e19ecfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7259ed05, SUC_lexicon_tags_49 },
    { 0xca19ed06, SUC_lexicon_tags_2 },
    { 0x2af9ed07, SUC_lexicon_tags_3 },
    { 0x4699ed08, SUC_lexicon_tags_45 },
    { 0xd499ed08, SUC_lexicon_tags_47 },
    { 0xf139ed0a, SUC_lexicon_tags_17 },
    { 0xe4d9ed05, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf739ed0e, SUC_lexicon_tags_43 },
    { 0x24b9ed0f, SUC_lexicon_tags_47 },
    { 0xa5d9ed0f, SUC_lexicon_tags_39 },
    { 0x8859ed10, SUC_lexicon_tags_7 },
    { 0x5ed9ed12, SUC_lexicon_tags_21 },
    { 0x6bb9ed13, SUC_lexicon_tags_47 },
    { 0x2c99ed13, SUC_lexicon_tags_11 },
    { 0xdf39ed13, SUC_lexicon_tags_11 },
    { 0xf579ed16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb659ed18, SUC_lexicon_tags_17 },
    { 0x4459ed19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cd9ed1e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc259ed26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff59ed2b, SUC_lexicon_tags_7 },
    { 0x9a59ed2c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d9ed30, SUC_lexicon_tags_15 },
    { 0x54b9ed31, SUC_lexicon_tags_11 },
    { 0x1479ed31, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdcd9ed34, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839ed39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b9ed3d, SUC_lexicon_tags_11 },
    { 0x4b19ed3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d19ed45, SUC_lexicon_tags_37 },
    { 0x7319ed46, SUC_lexicon_tags_46 },
    { 0x46f9ed46, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c79ed4d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3839ed50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1319ed54, SUC_lexicon_tags_13 },
    { 0xde39ed54, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c19ed58, SUC_lexicon_tags_37 },
    { 0xc059ed59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f9ed64, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x4919ed66, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef9ed6b, SUC_lexicon_tags_17 },
    { 0x7739ed6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6079ed71, SUC_lexicon_tags_47 },
    { 0x5699ed72, SUC_lexicon_tags_66 },
    { 0x5679ed73, SUC_lexicon_tags_7 },
    { 0x21f9ed73, SUC_lexicon_tags_13 },
    { 0x1e99ed75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4179ed83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3519ed86, SUC_lexicon_tags_14 },
    { 0xb3b9ed87, SUC_lexicon_tags_33 },
    { 0xa699ed87, SUC_lexicon_tags_6 },
    { 0xd9b9ed89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb9ed8f, SUC_lexicon_tags_19 },
    { 0xf899ed90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e59ed93, SUC_lexicon_tags_26 },
    { 0x959ed93, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac19ed99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd419eda3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7799eda7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68f9eda9, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xf319edac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9edaf, SUC_lexicon_tags_19 },
    { 0x8e79edb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda59edbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7779edbd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f9edc6, SUC_lexicon_tags_17 },
    { 0x7379edc7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x36b9edc9, SUC_lexicon_tags_60 },
    { 0xdc79edc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb9edce, SUC_lexicon_tags_47 },
    { 0x1bb9edce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x22d9edd1, SUC_lexicon_tags_2 },
    { 0x8db9edd1, SUC_lexicon_tags_91 },
    { 0x64f9edd3, SUC_lexicon_tags_10 },
    { 0x16f9edd4, SUC_lexicon_tags_27 },
    { 0x84f9edd5, SUC_lexicon_tags_19 },
    { 0x5419edd4, SUC_lexicon_tags_8 },
    { 0x8839edd7, SUC_lexicon_tags_12 },
    { 0xa0f9edd7, SUC_lexicon_tags_39 },
    { 0xf2b9edd8, SUC_lexicon_tags_63 },
    { 0x8b59edd8, SUC_lexicon_tags_7 },
    { 0xcf9edd5, SUC_lexicon_tags_1 },
    { 0xbc79edd2, SUC_lexicon_tags_2 },
    { 0xd259eddd, SUC_lexicon_tags_15 },
    { 0xa219eddd, SUC_lexicon_tags_33 },
    { 0xc139eddf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db9ede4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f9ede8, SUC_lexicon_tags_11 },
    { 0x94f9ede9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd939eded, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1cd9edef, SUC_lexicon_tags_1 },
    { 0x9cf9edf0, SUC_lexicon_tags_31 },
    { 0x6299edf0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4759edf7, SUC_lexicon_tags_15 },
    { 0x4359edf8, SUC_lexicon_tags_32 },
    { 0x6919edf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2239edfe, SUC_lexicon_tags_15 },
    { 0xc499edff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a19ee05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdd19ee07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ad9ee0a, SUC_lexicon_tags_15 },
    { 0xfd79ee0b, SUC_lexicon_tags_6 },
    { 0x7bd9ee0c, SUC_lexicon_tags_17 },
    { 0x1879ee0d, SUC_lexicon_tags_25 },
    { 0x5679ee0d, SUC_lexicon_tags_37 },
    { 0xc239ee0f, SUC_lexicon_tags_7 },
    { 0x3799ee0d, SUC_lexicon_tags_47 },
    { 0x2bf9ee11, SUC_lexicon_tags_15 },
    { 0xa1f9ee12, SUC_lexicon_tags_12 },
    { 0xc0f9ee11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2d9ee17, SUC_lexicon_tags_91 },
    { 0xbd99ee18, SUC_lexicon_tags_2 },
    { 0xb839ee17, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd779ee1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99f9ee21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df9ee27, SUC_lexicon_tags_0 },
    { 0xd539ee28, SUC_lexicon_tags_15 },
    { 0xf0d9ee27, SUC_lexicon_tags_11 },
    { 0xe099ee28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe319ee31, SUC_lexicon_tags_11 },
    { 0xf419ee31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48b9ee35, SUC_lexicon_tags_46 },
    { 0x7b9ee35, SUC_lexicon_tags_37 },
    { 0x6bf9ee35, SUC_lexicon_tags_12 },
    { 0xfd79ee37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf579ee3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x439ee40, SUC_lexicon_tags_55 },
    { 0x6d79ee41, SUC_lexicon_tags_27 },
    { 0x2519ee42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cb9ee44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf8f9ee46, SUC_lexicon_tags_14 },
    { 0xb239ee47, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf9ee4d, SUC_lexicon_tags_11 },
    { 0x6639ee4e, SUC_lexicon_tags_2 },
    { 0xd879ee4f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe439ee53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x38f9ee55, SUC_lexicon_tags_25 },
    { 0x6f39ee55, SUC_lexicon_tags_12 },
    { 0xa8b9ee57, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6639ee5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f59ee5d, SUC_lexicon_tags_14 },
    { 0x1199ee5e, SUC_lexicon_tags_17 },
    { 0x12f9ee5d, SUC_lexicon_tags_4 },
    { 0x24d9ee60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6f9ee65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd39ee68, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9ee6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2259ee6f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a79ee74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b99ee77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e39ee7b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc3b9ee7d, SUC_lexicon_tags_15 },
    { 0x6119ee7d, SUC_lexicon_tags_15 },
    { 0xc299ee7f, SUC_lexicon_tags_58 },
    { 0xc379ee80, SUC_lexicon_tags_54 },
    { 0x1459ee7d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4db9ee84, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0x4979ee87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d99ee8b, SUC_lexicon_tags_37 },
    { 0x72b9ee8c, SUC_lexicon_tags_17 },
    { 0xe79ee8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d9ee95, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x9519ee97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f9ee9b, SUC_lexicon_tags_29 },
    { 0x9219ee9c, SUC_lexicon_tags_14 },
    { 0xd1f9ee9d, SUC_lexicon_tags_2 },
    { 0xd3d9ee9d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x5e39eea0, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x7d59eea2, SUC_lexicon_tags_13 },
    { 0xf599eea3, SUC_lexicon_tags_19 },
    { 0x10b9eea4, SUC_lexicon_tags_11 },
    { 0x3c19eea5, SUC_lexicon_tags_7 },
    { 0x5119eea5, SUC_lexicon_tags_7 },
    { 0x7119eea7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e79eea9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e79eeab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf9eeae, SUC_lexicon_tags_12 },
    { 0x8e79eeaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca99eeb8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2ef9eeba, SUC_lexicon_tags_63 },
    { 0x6cf9eeba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe679eebf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60b9eec5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2939eeca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93d9eecd, SUC_lexicon_tags_10 },
    { 0x6879eece, SUC_lexicon_tags_29 },
    { 0xba39eece, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe379eedc, SUC_lexicon_tags_17 },
    { 0x2139eedd, SUC_lexicon_tags_1 },
    { 0x8659eedd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb99eee1, SUC_lexicon_tags_48 },
    { 0xffd9eee1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf79eee4, SUC_lexicon_tags_60 },
    { 0x99d9eee5, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x1099eee8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x59b9eeea, SUC_lexicon_tags_47 },
    { 0xaf19eeea, SUC_lexicon_tags_2 },
    { 0x7779eeeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd99eeee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd439eef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb479eef3, SUC_lexicon_tags_25 },
    { 0x3a39eef3, SUC_lexicon_tags_1 },
    { 0xa6b9eef3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10b9eefb, SUC_lexicon_tags_7 },
    { 0xc7d9eefc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2819ef01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f19ef07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd719ef09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c79ef0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe959ef10, SUC_lexicon_tags_37 },
    { 0x1ad9ef10, SUC_lexicon_tags_47 },
    { 0xc899ef12, SUC_lexicon_tags_100 },
    { 0x3519ef12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79ef1c, SUC_lexicon_tags_9 },
    { 0xe9d9ef1d, SUC_lexicon_tags_32 },
    { 0x2099ef1c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2839ef20, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4d9ef25, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94d9ef2a, SUC_lexicon_tags_30 },
    { 0x7e19ef2b, SUC_lexicon_tags_47 },
    { 0xfd39ef2b, SUC_lexicon_tags_1 },
    { 0x6379ef2b, SUC_lexicon_tags_15 },
    { 0x4019ef2e, SUC_lexicon_tags_1 },
    { 0x1b79ef2e, SUC_lexicon_tags_77 },
    { 0xb899ef30, SUC_lexicon_tags_47 },
    { 0xdb9ef2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa199ef33, SUC_lexicon_tags_1 },
    { 0x1e59ef33, SUC_lexicon_tags_7 },
    { 0x51b9ef35, SUC_lexicon_tags_25 },
    { 0xbe79ef36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9ef39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ef9ef3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6f9ef3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d99ef41, SUC_lexicon_tags_17 },
    { 0xc9d9ef42, SUC_lexicon_tags_27 },
    { 0x7739ef43, SUC_lexicon_tags_60 },
    { 0xd559ef44, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b9ef47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef59ef49, SUC_lexicon_tags_2 },
    { 0xfc39ef4a, SUC_lexicon_tags_8 },
    { 0x4a59ef4a, SUC_lexicon_tags_14 },
    { 0x2159ef4c, SUC_lexicon_tags_25 },
    { 0x8e39ef4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d9ef51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x799ef54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xea19ef56, SUC_lexicon_tags_48 },
    { 0x5739ef57, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf519ef5e, SUC_lexicon_tags_47 },
    { 0xccd9ef5f, SUC_lexicon_tags_25 },
    { 0x95f9ef60, SUC_lexicon_tags_43 },
    { 0xdf59ef60, SUC_lexicon_tags_8 },
    { 0x6659ef62, SUC_lexicon_tags_28 },
    { 0x5199ef62, SUC_lexicon_tags_7 },
    { 0x2e79ef64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f9ef68, SUC_lexicon_tags_19 },
    { 0x1f79ef69, SUC_lexicon_tags_11 },
    { 0xfc59ef6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379ef6e, SUC_lexicon_tags_17 },
    { 0x1e99ef6f, SUC_lexicon_tags_43 },
    { 0xcaf9ef70, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8659ef72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8179ef7a, SUC_lexicon_tags_29 },
    { 0xce79ef7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73b9ef7d, SUC_lexicon_tags_7 },
    { 0xc279ef7e, SUC_lexicon_tags_25 },
    { 0x1ed9ef7f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe379ef83, SUC_lexicon_tags_15 },
    { 0x9dd9ef83, SUC_lexicon_tags_12 },
    { 0x1a99ef85, SUC_lexicon_tags_17 },
    { 0x1f99ef85, SUC_lexicon_tags_8 },
    { 0xaf79ef86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6239ef8b, SUC_lexicon_tags_37 },
    { 0x5cd9ef8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b59ef92, SUC_lexicon_tags_8 },
    { 0x3799ef93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7979ef95, SUC_lexicon_tags_85 },
    { 0xa1b9ef96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x85f9ef98, SUC_lexicon_tags_14 },
    { 0xb399ef99, SUC_lexicon_tags_43 },
    { 0xafb9ef9a, SUC_lexicon_tags_30 },
    { 0x6ff9ef9b, SUC_lexicon_tags_15 },
    { 0x4039ef9c, SUC_lexicon_tags_38 },
    { 0x8779ef9d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x8df9ef9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcd9efa2, SUC_lexicon_tags_8 },
    { 0x4559efa3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a99efa7, SUC_lexicon_tags_12 },
    { 0xc5d9efa7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2959efaa, SUC_lexicon_tags_29 },
    { 0x3839efab, SUC_lexicon_tags_35 },
    { 0x7999efac, SUC_lexicon_tags_2 },
    { 0x9a99efac, SUC_lexicon_tags_12 },
    { 0xb379efae, SUC_lexicon_tags_65 },
    { 0x1d39efaf, SUC_lexicon_tags_17 },
    { 0xb799efb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9efb4, SUC_lexicon_tags_11 },
    { 0xd359efb5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579efb9, SUC_lexicon_tags_10 },
    { 0x1479efba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6eb9efbc, SUC_lexicon_tags_30 },
    { 0xed79efbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1979efc1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x46b9efc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3839efcd, SUC_lexicon_tags_43 },
    { 0x9f79efce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c99efd0, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x71d9efd3, SUC_lexicon_tags_47 },
    { 0x2a59efd4, SUC_lexicon_tags_0 },
    { 0x43d9efd3, SUC_lexicon_tags_40 },
    { 0x2a39efd6, SUC_lexicon_tags_17 },
    { 0x84b9efd7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8059efd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e59efdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf259efde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc639efe1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa599efe4, SUC_lexicon_tags_19 },
    { 0x8af9efe4, SUC_lexicon_tags_124 },
    { 0x25b9efe6, SUC_lexicon_tags_21 },
    { 0x5279efe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe739efe9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d9efec, SUC_lexicon_tags_10 },
    { 0x1639efec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50b9eff0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5759eff6, SUC_lexicon_tags_28 },
    { 0xd479eff6, SUC_lexicon_tags_29 },
    { 0x5359eff6, SUC_lexicon_tags_25 },
    { 0x4e39eff7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd9effe, SUC_lexicon_tags_2 },
    { 0x6ef9effe, SUC_lexicon_tags_66 },
    { 0x6259effe, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x1279f002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b9f005, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb879f008, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x8979f00a, SUC_lexicon_tags_15 },
    { 0x7699f00b, SUC_lexicon_tags_7 },
    { 0x8319f00c, SUC_lexicon_tags_37 },
    { 0x99f9f00d, SUC_lexicon_tags_37 },
    { 0x6f79f00d, SUC_lexicon_tags_37 },
    { 0x4659f00e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5239f011, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e99f013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3899f016, SUC_lexicon_tags_14 },
    { 0xee19f016, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x65f9f019, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9af9f01c, SUC_lexicon_tags_47 },
    { 0x5719f01c, SUC_lexicon_tags_1 },
    { 0x8e99f01c, SUC_lexicon_tags_2 },
    { 0x7639f01f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d99f023, SUC_lexicon_tags_17 },
    { 0xf4f9f023, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb839f028, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb019f02b, SUC_lexicon_tags_1 },
    { 0x69d9f02c, SUC_lexicon_tags_17 },
    { 0x1559f02d, SUC_lexicon_tags_7 },
    { 0x8d79f02e, SUC_lexicon_tags_37 },
    { 0xddf9f02b, SUC_lexicon_tags_2 },
    { 0x9c79f030, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319f03b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa739f03d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd179f03f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c79f042, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd939f045, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xed9f047, SUC_lexicon_tags_15 },
    { 0xb979f047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef79f052, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6d79f054, SUC_lexicon_tags_11 },
    { 0x9b79f055, SUC_lexicon_tags_6 },
    { 0xc3f9f054, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb99f05d, SUC_lexicon_tags_1 },
    { 0xc259f05d, SUC_lexicon_tags_7 },
    { 0x1ab9f05f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe799f061, SUC_lexicon_tags_12 },
    { 0xf9d9f062, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7019f065, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe019f06a, SUC_lexicon_tags_12 },
    { 0x16f9f06b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4779f06d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe999f070, SUC_lexicon_tags_25 },
    { 0x2c19f070, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd59f076, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf9f07d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f9f081, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed9f083, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d79f085, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d9f089, SUC_lexicon_tags_33 },
    { 0x7f39f08a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99f090, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9f9f092, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d19f098, SUC_lexicon_tags_0 },
    { 0x5d9f099, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34b9f09b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a39f09d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x80d9f09f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b9f0a6, SUC_lexicon_tags_25 },
    { 0xacf9f0a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb839f0aa, SUC_lexicon_tags_29 },
    { 0xef59f0aa, SUC_lexicon_tags_7 },
    { 0x4759f0ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51d9f0ae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb699f0b1, SUC_lexicon_tags_1 },
    { 0x3d59f0b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2079f0b4, SUC_lexicon_tags_37 },
    { 0x6a59f0b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7639f0b9, SUC_lexicon_tags_2 },
    { 0x5ed9f0b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1c59f0bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae59f0c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d9f0c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8619f0c6, SUC_lexicon_tags_39 },
    { 0x9c19f0c7, SUC_lexicon_tags_2 },
    { 0x4fd9f0c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce79f0d4, SUC_lexicon_tags_19 },
    { 0x8af9f0d4, SUC_lexicon_tags_29 },
    { 0x8119f0d6, SUC_lexicon_tags_15 },
    { 0x2379f0d7, SUC_lexicon_tags_29 },
    { 0x3f39f0d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf599f0db, SUC_lexicon_tags_29 },
    { 0x5f99f0dc, SUC_lexicon_tags_6 },
    { 0x7f39f0dd, SUC_lexicon_tags_37 },
    { 0x7cd9f0dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4f59f0e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8839f0e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5859f0e6, SUC_lexicon_tags_60 },
    { 0x719f0e7, SUC_lexicon_tags_43 },
    { 0x6619f0e8, SUC_lexicon_tags_27 },
    { 0x2219f0e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68d9f0ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6119f0f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1839f0f5, SUC_lexicon_tags_29 },
    { 0x1d59f0f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f9f0f9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xafd9f0fc, SUC_lexicon_tags_7 },
    { 0x6799f0fc, SUC_lexicon_tags_43 },
    { 0x2919f0fe, SUC_lexicon_tags_15 },
    { 0x7d59f0fc, SUC_lexicon_tags_14 },
    { 0xf139f100, SUC_lexicon_tags_11 },
    { 0x5db9f101, SUC_lexicon_tags_27 },
    { 0xd79f0fe, SUC_lexicon_tags_25 },
    { 0x6099f0fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3019f106, SUC_lexicon_tags_38 },
    { 0xaf99f107, SUC_lexicon_tags_25 },
    { 0x4a79f106, SUC_lexicon_tags_46 },
    { 0x9839f107, SUC_lexicon_tags_12 },
    { 0xaff9f10a, SUC_lexicon_tags_17 },
    { 0xfcf9f109, SUC_lexicon_tags_13 },
    { 0x23b9f10a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cd9f110, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc939f112, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f9f115, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3b59f117, SUC_lexicon_tags_364 },
    { 0xad19f118, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d9f11c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a79f11f, SUC_lexicon_tags_1 },
    { 0xa5d9f120, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafb9f127, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1239f12c, SUC_lexicon_tags_902 },
    { 0, NULL },
    { 0, NULL },
    { 0x5059f12f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd639f137, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f19f13c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc679f140, SUC_lexicon_tags_11 },
    { 0x8e99f141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4739f143, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5db9f145, SUC_lexicon_tags_104 },
    { 0x3a79f145, SUC_lexicon_tags_4 },
    { 0x8359f147, SUC_lexicon_tags_1 },
    { 0x30d9f148, SUC_lexicon_tags_7 },
    { 0x56d9f149, SUC_lexicon_tags_30 },
    { 0xeb39f148, SUC_lexicon_tags_21 },
    { 0x32f9f146, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x899f14d, SUC_lexicon_tags_1 },
    { 0xdf79f14e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b19f153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d99f155, SUC_lexicon_tags_6 },
    { 0xcbb9f155, SUC_lexicon_tags_25 },
    { 0x8939f156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbf9f159, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xcf19f15b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2519f15d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ef9f164, SUC_lexicon_tags_49 },
    { 0x5919f165, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde99f16c, SUC_lexicon_tags_23 },
    { 0xf159f16c, SUC_lexicon_tags_44 },
    { 0x8e39f16e, SUC_lexicon_tags_7 },
    { 0xd99f16f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8359f174, SUC_lexicon_tags_11 },
    { 0xb39f175, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf399f177, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39f17b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2259f17e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78b9f182, SUC_lexicon_tags_45 },
    { 0x6e59f182, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7df9f185, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce79f188, SUC_lexicon_tags_17 },
    { 0xfdd9f189, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa1b9f18b, SUC_lexicon_tags_17 },
    { 0xb79f18b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbf9f18e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5b99f190, SUC_lexicon_tags_37 },
    { 0x3519f191, SUC_lexicon_tags_31 },
    { 0x1bf9f192, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cf9f195, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3199f199, SUC_lexicon_tags_47 },
    { 0xaa79f199, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x74f9f19c, SUC_lexicon_tags_16 },
    { 0xf5b9f19d, SUC_lexicon_tags_2 },
    { 0xb1f9f19d, SUC_lexicon_tags_15 },
    { 0x4a19f19e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x999f1a1, SUC_lexicon_tags_3 },
    { 0xdb9f1a2, SUC_lexicon_tags_27 },
    { 0x2559f1a1, SUC_lexicon_tags_1 },
    { 0x4719f1a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x659f1a7, SUC_lexicon_tags_12 },
    { 0x399f1a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaf9f1af, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb1d9f1b1, SUC_lexicon_tags_22 },
    { 0x8119f1b2, SUC_lexicon_tags_30 },
    { 0x1ed9f1b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98b9f1bc, SUC_lexicon_tags_25 },
    { 0xccd9f1bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1719f1c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc139f1c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b9f1ca, SUC_lexicon_tags_97 },
    { 0x199f1ca, SUC_lexicon_tags_15 },
    { 0xb299f1ca, SUC_lexicon_tags_17 },
    { 0xbcf9f1cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76d9f1d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8379f1d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7a99f1d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb39f1dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9919f1e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d99f1e5, SUC_lexicon_tags_60 },
    { 0x3959f1e6, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x12d9f1e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7f9f1ef, SUC_lexicon_tags_14 },
    { 0xedd9f1ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5199f1fa, SUC_lexicon_tags_46 },
    { 0x80b9f1fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f1ff, SUC_lexicon_tags_19 },
    { 0x399f1ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5e19f202, SUC_lexicon_tags_43 },
    { 0x3a59f203, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f39f206, SUC_lexicon_tags_57 },
    { 0x3119f207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b9f20a, SUC_lexicon_tags_37 },
    { 0x1c79f20a, SUC_lexicon_tags_2 },
    { 0xd159f20c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x499f20e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ab9f213, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89f9f215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca39f218, SUC_lexicon_tags_8 },
    { 0xd759f219, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3479f21b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e99f21e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb979f220, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe959f228, SUC_lexicon_tags_0 },
    { 0xe379f229, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6819f22b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8499f22e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x72d9f230, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb9f233, SUC_lexicon_tags_25 },
    { 0xf659f234, SUC_lexicon_tags_2 },
    { 0x47b9f235, SUC_lexicon_tags_21 },
    { 0x8cb9f236, SUC_lexicon_tags_17 },
    { 0x7d19f236, SUC_lexicon_tags_63 },
    { 0xb2b9f238, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1059f23b, SUC_lexicon_tags_7 },
    { 0x23b9f23c, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0x6e19f23e, SUC_lexicon_tags_25 },
    { 0x6bd9f23f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a79f243, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd9f24a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbf9f24c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x14f9f24e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81d9f250, SUC_lexicon_tags_33 },
    { 0x97b9f251, SUC_lexicon_tags_49 },
    { 0x7c79f251, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ed9f256, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd659f258, SUC_lexicon_tags_11 },
    { 0x9619f259, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3959f25b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1539f25d, SUC_lexicon_tags_27 },
    { 0xba39f25d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68d9f262, SUC_lexicon_tags_47 },
    { 0xef99f263, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8d9f267, SUC_lexicon_tags_1 },
    { 0xa539f268, SUC_lexicon_tags_87 },
    { 0xb119f269, SUC_lexicon_tags_37 },
    { 0xb3d9f269, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3039f26f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b9f279, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2b9f27c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb139f280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc839f283, SUC_lexicon_tags_26 },
    { 0x3ab9f284, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4af9f287, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3979f28a, SUC_lexicon_tags_7 },
    { 0xc219f28b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6699f28d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc79f290, SUC_lexicon_tags_23 },
    { 0xcef9f291, SUC_lexicon_tags_47 },
    { 0x1859f292, SUC_lexicon_tags_37 },
    { 0x4bb9f293, SUC_lexicon_tags_11 },
    { 0xb879f294, SUC_lexicon_tags_3 },
    { 0xb059f294, SUC_lexicon_tags_7 },
    { 0xc79f294, SUC_lexicon_tags_14 },
    { 0x5719f297, SUC_lexicon_tags_37 },
    { 0xb919f297, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8139f29a, SUC_lexicon_tags_7 },
    { 0x9339f29b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f9f29f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4139f2a7, SUC_lexicon_tags_17 },
    { 0xe739f2a7, SUC_lexicon_tags_65 },
    { 0xb6b9f2a9, SUC_lexicon_tags_11 },
    { 0x1519f2aa, SUC_lexicon_tags_1 },
    { 0xd519f2a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd9f2af, SUC_lexicon_tags_12 },
    { 0xcd19f2b0, SUC_lexicon_tags_37 },
    { 0x6159f2b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1019f2b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c79f2b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf9f2bd, SUC_lexicon_tags_12 },
    { 0xc5d9f2be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f19f2c2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe979f2c9, SUC_lexicon_tags_11 },
    { 0xbcd9f2c9, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xeed9f2cc, SUC_lexicon_tags_38 },
    { 0xe8b9f2cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41b9f2cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b9f2d3, SUC_lexicon_tags_27 },
    { 0x64d9f2d4, SUC_lexicon_tags_39 },
    { 0x6499f2d5, SUC_lexicon_tags_23 },
    { 0x2539f2d6, SUC_lexicon_tags_47 },
    { 0xbeb9f2d7, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x63f9f2d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff9f2dc, SUC_lexicon_tags_2 },
    { 0xfd99f2dc, SUC_lexicon_tags_47 },
    { 0xa819f2de, SUC_lexicon_tags_1 },
    { 0xfdb9f2de, SUC_lexicon_tags_2 },
    { 0x57d9f2e0, SUC_lexicon_tags_49 },
    { 0x78f9f2e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f9f2e4, SUC_lexicon_tags_37 },
    { 0xaf59f2e5, SUC_lexicon_tags_9 },
    { 0x4939f2e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3959f2e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f39f2f3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb119f2fb, SUC_lexicon_tags_37 },
    { 0x15f9f2fc, SUC_lexicon_tags_23 },
    { 0xc3d9f2fd, SUC_lexicon_tags_21 },
    { 0xbdb9f2fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x58d9f301, SUC_lexicon_tags_25 },
    { 0x7359f301, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf299f304, SUC_lexicon_tags_37 },
    { 0xb899f305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86f9f308, SUC_lexicon_tags_14 },
    { 0xdff9f309, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39f30e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4799f310, SUC_lexicon_tags_5 },
    { 0xe919f311, SUC_lexicon_tags_45 },
    { 0x1d19f312, SUC_lexicon_tags_17 },
    { 0x93d9f311, SUC_lexicon_tags_25 },
    { 0xf239f314, SUC_lexicon_tags_4 },
    { 0x8899f314, SUC_lexicon_tags_66 },
    { 0xe699f314, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa639f31b, SUC_lexicon_tags_2 },
    { 0x8b19f31c, SUC_lexicon_tags_17 },
    { 0xd659f31d, SUC_lexicon_tags_25 },
    { 0x46d9f31e, SUC_lexicon_tags_38 },
    { 0x71b9f31e, SUC_lexicon_tags_30 },
    { 0x7eb9f31e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22d9f322, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2f9f325, SUC_lexicon_tags_30 },
    { 0xc159f326, SUC_lexicon_tags_47 },
    { 0xd959f327, SUC_lexicon_tags_27 },
    { 0x18b9f325, SUC_lexicon_tags_15 },
    { 0x1d9f329, SUC_lexicon_tags_19 },
    { 0x9039f329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56d9f32e, SUC_lexicon_tags_28 },
    { 0xe5d9f32f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8579f335, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa139f337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d9f33d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c39f341, SUC_lexicon_tags_65 },
    { 0xf619f342, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f9f344, SUC_lexicon_tags_48 },
    { 0xc599f345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8799f347, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x12b9f349, SUC_lexicon_tags_21 },
    { 0x5e59f34a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b9f34d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa779f34f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd739f356, SUC_lexicon_tags_11 },
    { 0xd0d9f357, SUC_lexicon_tags_29 },
    { 0xc7b9f358, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b9f35c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff9f35e, SUC_lexicon_tags_6 },
    { 0xbff9f35f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb19f361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa839f364, SUC_lexicon_tags_27 },
    { 0x1139f364, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x60b9f367, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb199f369, SUC_lexicon_tags_11 },
    { 0xf559f36a, SUC_lexicon_tags_12 },
    { 0x9099f369, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf279f36e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccd9f373, SUC_lexicon_tags_7 },
    { 0x8899f374, SUC_lexicon_tags_30 },
    { 0xe299f374, SUC_lexicon_tags_30 },
    { 0x99f9f375, SUC_lexicon_tags_56 },
    { 0x999f377, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6399f37d, SUC_lexicon_tags_1 },
    { 0x3239f37e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbd9f380, SUC_lexicon_tags_27 },
    { 0x859f381, SUC_lexicon_tags_16 },
    { 0x5699f381, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa899f384, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf399f387, SUC_lexicon_tags_16 },
    { 0x3d39f388, SUC_lexicon_tags_23 },
    { 0x55f9f389, SUC_lexicon_tags_2 },
    { 0x9f99f389, SUC_lexicon_tags_19 },
    { 0x6eb9f38a, SUC_lexicon_tags_38 },
    { 0x3af9f38a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb99f391, SUC_lexicon_tags_15 },
    { 0x5c79f391, SUC_lexicon_tags_17 },
    { 0xd519f393, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8799f396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0f9f39a, SUC_lexicon_tags_47 },
    { 0xb8b9f39b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c39f39d, SUC_lexicon_tags_39 },
    { 0xb439f39e, SUC_lexicon_tags_65 },
    { 0xa479f39f, SUC_lexicon_tags_26 },
    { 0x6819f39e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1859f3a6, SUC_lexicon_tags_15 },
    { 0x4039f3a6, SUC_lexicon_tags_37 },
    { 0x6419f3a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xded9f3ab, SUC_lexicon_tags_37 },
    { 0x15d9f3ac, SUC_lexicon_tags_34 },
    { 0xe2f9f3ac, SUC_lexicon_tags_27 },
    { 0x7f39f3ae, SUC_lexicon_tags_16 },
    { 0x6699f3af, SUC_lexicon_tags_15 },
    { 0x3a79f3af, SUC_lexicon_tags_17 },
    { 0x8259f3b1, SUC_lexicon_tags_33 },
    { 0x4b99f3b2, SUC_lexicon_tags_33 },
    { 0x6199f3b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfeb9f3b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6399f3c1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x58f9f3c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f9f3ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a39f3cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e19f3d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf659f3d6, SUC_lexicon_tags_14 },
    { 0x8d79f3d7, SUC_lexicon_tags_47 },
    { 0x6f39f3d7, SUC_lexicon_tags_14 },
    { 0xecf9f3d9, SUC_lexicon_tags_87 },
    { 0xebb9f3d6, SUC_lexicon_tags_7 },
    { 0xc99f3da, SUC_lexicon_tags_0 },
    { 0xab59f3db, SUC_lexicon_tags_23 },
    { 0x46d9f3dd, SUC_lexicon_tags_15 },
    { 0x2f79f3de, SUC_lexicon_tags_12 },
    { 0x6639f3df, SUC_lexicon_tags_71 },
    { 0x76d9f3e0, SUC_lexicon_tags_9 },
    { 0x9319f3de, SUC_lexicon_tags_4 },
    { 0x6b99f3de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b59f3e4, SUC_lexicon_tags_25 },
    { 0xd0d9f3e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9f3f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4979f3f6, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9619f3fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb9f405, SUC_lexicon_tags_60 },
    { 0xe559f406, SUC_lexicon_tags_33 },
    { 0x52b9f406, SUC_lexicon_tags_15 },
    { 0xdd59f406, SUC_lexicon_tags_37 },
    { 0x3699f409, SUC_lexicon_tags_2 },
    { 0xb679f409, SUC_lexicon_tags_26 },
    { 0xfcb9f40b, SUC_lexicon_tags_1 },
    { 0xbf79f40a, SUC_lexicon_tags_10 },
    { 0x599f409, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4499f40f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe99f411, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe499f414, SUC_lexicon_tags_1 },
    { 0xa6f9f415, SUC_lexicon_tags_26 },
    { 0x8059f416, SUC_lexicon_tags_23 },
    { 0x7e19f414, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9859f41f, SUC_lexicon_tags_27 },
    { 0xb2d9f41f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x31b9f422, SUC_lexicon_tags_7 },
    { 0xf99f422, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7659f425, SUC_lexicon_tags_13 },
    { 0x1519f426, SUC_lexicon_tags_37 },
    { 0xffd9f427, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f79f42f, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0xf6f9f431, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf9f437, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x92f9f43a, SUC_lexicon_tags_12 },
    { 0x31d9f43b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xec99f43d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b39f43f, SUC_lexicon_tags_1 },
    { 0x3ef9f440, SUC_lexicon_tags_37 },
    { 0x9d59f440, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b59f444, SUC_lexicon_tags_12 },
    { 0x9fd9f445, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf9f448, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3919f44c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b79f44e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xae99f450, SUC_lexicon_tags_60 },
    { 0xff39f451, SUC_lexicon_tags_25 },
    { 0x759f450, SUC_lexicon_tags_9 },
    { 0x1679f453, SUC_lexicon_tags_15 },
    { 0x7bd9f454, SUC_lexicon_tags_37 },
    { 0x4e39f450, SUC_lexicon_tags_1 },
    { 0xbb59f455, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac59f45c, SUC_lexicon_tags_11 },
    { 0xd9f45d, SUC_lexicon_tags_10 },
    { 0x1559f45d, SUC_lexicon_tags_33 },
    { 0x1619f45d, SUC_lexicon_tags_79 },
    { 0x2c39f460, SUC_lexicon_tags_15 },
    { 0x5a79f45d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9239f464, SUC_lexicon_tags_6 },
    { 0xeb99f465, SUC_lexicon_tags_26 },
    { 0xe9d9f464, SUC_lexicon_tags_2 },
    { 0x8819f467, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe499f46e, SUC_lexicon_tags_84 },
    { 0xc7f9f46f, SUC_lexicon_tags_7 },
    { 0x3359f46f, SUC_lexicon_tags_11 },
    { 0xe859f470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac19f473, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7039f475, SUC_lexicon_tags_29 },
    { 0x4b59f476, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4d9f479, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc019f47d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xab39f480, SUC_lexicon_tags_14 },
    { 0x11d9f480, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1eb9f483, SUC_lexicon_tags_3 },
    { 0x46d9f483, SUC_lexicon_tags_1 },
    { 0x4719f485, SUC_lexicon_tags_7 },
    { 0x8d39f486, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb9f48d, SUC_lexicon_tags_27 },
    { 0xaa59f48e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe439f490, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70f9f494, SUC_lexicon_tags_7 },
    { 0x4999f495, SUC_lexicon_tags_21 },
    { 0xaf59f495, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c19f498, SUC_lexicon_tags_10 },
    { 0x2039f498, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c19f4a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb299f4a4, SUC_lexicon_tags_63 },
    { 0x7019f4a5, SUC_lexicon_tags_7 },
    { 0xbf9f4a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb879f4a9, SUC_lexicon_tags_301 },
    { 0xe559f4a9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x45f9f4ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa939f4b1, SUC_lexicon_tags_12 },
    { 0xab9f4b2, SUC_lexicon_tags_28 },
    { 0xd139f4b1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7999f4b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a19f4bb, SUC_lexicon_tags_26 },
    { 0x9359f4bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a99f4bf, SUC_lexicon_tags_11 },
    { 0xb2d9f4c0, SUC_lexicon_tags_37 },
    { 0xd299f4bf, SUC_lexicon_tags_29 },
    { 0x7399f4c0, SUC_lexicon_tags_9 },
    { 0xbb39f4c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13d9f4c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cd9f4c7, SUC_lexicon_tags_37 },
    { 0xdbf9f4c8, SUC_lexicon_tags_7 },
    { 0xced9f4c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa79f4cd, SUC_lexicon_tags_28 },
    { 0x23b9f4ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc099f4d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x619f4d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59f4d8, SUC_lexicon_tags_47 },
    { 0x23f9f4d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b9f4de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b9f4e1, SUC_lexicon_tags_5 },
    { 0x2839f4e2, SUC_lexicon_tags_38 },
    { 0xaa79f4e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa959f4e9, SUC_lexicon_tags_14 },
    { 0x6899f4e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9319f4ec, SUC_lexicon_tags_25 },
    { 0x5eb9f4ec, SUC_lexicon_tags_19 },
    { 0x6019f4ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd79f4f6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xed39f4f9, SUC_lexicon_tags_19 },
    { 0xcff9f4f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe299f4fd, SUC_lexicon_tags_12 },
    { 0x5339f4fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f39f501, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xac99f504, SUC_lexicon_tags_25 },
    { 0xca99f504, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc99f50a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99f50d, SUC_lexicon_tags_1052 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f9f512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9f516, SUC_lexicon_tags_17 },
    { 0x4b79f517, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf739f51a, SUC_lexicon_tags_2 },
    { 0x7f9f51a, SUC_lexicon_tags_4 },
    { 0x2bb9f51c, SUC_lexicon_tags_2 },
    { 0x3cf9f51b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4f9f522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1d79f524, SUC_lexicon_tags_4 },
    { 0xc0b9f525, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9039f537, SUC_lexicon_tags_56 },
    { 0xf3b9f538, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e59f53b, SUC_lexicon_tags_17 },
    { 0x4599f53c, SUC_lexicon_tags_6 },
    { 0xbb9f53c, SUC_lexicon_tags_17 },
    { 0x6699f53e, SUC_lexicon_tags_1 },
    { 0xe99f53f, SUC_lexicon_tags_11 },
    { 0xb039f540, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc559f542, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f59f546, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8d19f548, SUC_lexicon_tags_11 },
    { 0xf099f548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x69b9f54c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb39f554, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf9f557, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61d9f55a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc479f55e, SUC_lexicon_tags_11 },
    { 0xc939f55f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9f562, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6419f564, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b9f569, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf579f56e, SUC_lexicon_tags_15 },
    { 0xebf9f56f, SUC_lexicon_tags_37 },
    { 0x1f19f56e, SUC_lexicon_tags_43 },
    { 0xc6d9f571, SUC_lexicon_tags_25 },
    { 0x8c59f571, SUC_lexicon_tags_11 },
    { 0x1cb9f573, SUC_lexicon_tags_30 },
    { 0x3c39f574, SUC_lexicon_tags_0 },
    { 0xd319f575, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82d9f577, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x20d9f579, SUC_lexicon_tags_2 },
    { 0x5919f579, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b19f57f, SUC_lexicon_tags_33 },
    { 0x2f9f57f, SUC_lexicon_tags_25 },
    { 0xbcb9f57f, SUC_lexicon_tags_17 },
    { 0xd199f581, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x41f9f584, SUC_lexicon_tags_26 },
    { 0x9a39f585, SUC_lexicon_tags_61 },
    { 0x139f585, SUC_lexicon_tags_15 },
    { 0xba79f587, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb699f58b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5879f58e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9f590, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f9f598, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8759f59a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9899f59d, SUC_lexicon_tags_63 },
    { 0xd3f9f59e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b9f5a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd9f5aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b19f5ad, SUC_lexicon_tags_19 },
    { 0x4199f5ae, SUC_lexicon_tags_37 },
    { 0x7419f5af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82d9f5b3, SUC_lexicon_tags_2 },
    { 0x5299f5b3, SUC_lexicon_tags_37 },
    { 0x7d79f5b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7df9f5b7, SUC_lexicon_tags_107 },
    { 0x5fb9f5b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c19f5ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbd9f5bc, SUC_lexicon_tags_21 },
    { 0xd479f5bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c99f5bf, SUC_lexicon_tags_4 },
    { 0x8db9f5bf, SUC_lexicon_tags_37 },
    { 0xc359f5bf, SUC_lexicon_tags_21 },
    { 0x41f9f5c2, SUC_lexicon_tags_7 },
    { 0xa6d9f5c0, SUC_lexicon_tags_19 },
    { 0x5639f5c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd39f5c7, SUC_lexicon_tags_11 },
    { 0x89f9f5c8, SUC_lexicon_tags_11 },
    { 0xb999f5c9, SUC_lexicon_tags_37 },
    { 0xe039f5ca, SUC_lexicon_tags_30 },
    { 0x839f5cb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6099f5ce, SUC_lexicon_tags_11 },
    { 0x3839f5ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5879f5d4, SUC_lexicon_tags_18 },
    { 0x6a39f5d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2899f5d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde59f5d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c19f5db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b9f5de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc619f5e2, SUC_lexicon_tags_17 },
    { 0xda79f5e3, SUC_lexicon_tags_1 },
    { 0x5d19f5e4, SUC_lexicon_tags_12 },
    { 0x6e99f5e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8379f5e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c59f5ea, SUC_lexicon_tags_14 },
    { 0x7ad9f5ea, SUC_lexicon_tags_9 },
    { 0xab99f5ec, SUC_lexicon_tags_37 },
    { 0xf759f5ed, SUC_lexicon_tags_60 },
    { 0x64b9f5ee, SUC_lexicon_tags_9 },
    { 0x379f5ee, SUC_lexicon_tags_55 },
    { 0x759f5ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd079f5f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb219f5f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8959f5fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa939f600, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f9f605, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xfe19f607, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e39f609, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b79f60e, SUC_lexicon_tags_5 },
    { 0x21b9f60f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df9f613, SUC_lexicon_tags_15 },
    { 0x2f9f613, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65d9f619, SUC_lexicon_tags_12 },
    { 0xb099f61a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf379f61d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639f622, SUC_lexicon_tags_12 },
    { 0xb599f623, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d59f626, SUC_lexicon_tags_1 },
    { 0xecf9f627, SUC_lexicon_tags_61 },
    { 0xb639f626, SUC_lexicon_tags_7 },
    { 0x8dd9f629, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f9f62f, SUC_lexicon_tags_39 },
    { 0xe899f630, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed39f635, SUC_lexicon_tags_31 },
    { 0x2979f636, SUC_lexicon_tags_48 },
    { 0xad9f636, SUC_lexicon_tags_7 },
    { 0x7f39f638, SUC_lexicon_tags_12 },
    { 0x3f59f638, SUC_lexicon_tags_184 },
    { 0xecd9f638, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1859f63d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba99f641, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b99f643, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3219f646, SUC_lexicon_tags_48 },
    { 0xf479f647, SUC_lexicon_tags_17 },
    { 0xc519f648, SUC_lexicon_tags_26 },
    { 0x63b9f647, SUC_lexicon_tags_48 },
    { 0xff19f64a, SUC_lexicon_tags_30 },
    { 0x4639f64b, SUC_lexicon_tags_31 },
    { 0xe359f646, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc279f651, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd9f657, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc499f659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ed9f65b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2919f663, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9879f669, SUC_lexicon_tags_10 },
    { 0x1bd9f66a, SUC_lexicon_tags_11 },
    { 0xf699f669, SUC_lexicon_tags_26 },
    { 0xddb9f66c, SUC_lexicon_tags_12 },
    { 0x5df9f66c, SUC_lexicon_tags_1 },
    { 0x5b39f66e, SUC_lexicon_tags_15 },
    { 0xa1d9f66f, SUC_lexicon_tags_8 },
    { 0x32d9f66e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcf39f672, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9839f674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2299f67c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xae19f67e, SUC_lexicon_tags_68 },
    { 0xe559f67f, SUC_lexicon_tags_15 },
    { 0x6999f680, SUC_lexicon_tags_45 },
    { 0x56b9f681, SUC_lexicon_tags_7 },
    { 0xd959f682, SUC_lexicon_tags_12 },
    { 0x1f99f683, SUC_lexicon_tags_27 },
    { 0x4619f683, SUC_lexicon_tags_17 },
    { 0xff99f683, SUC_lexicon_tags_32 },
    { 0xc079f683, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9f68e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbb39f690, SUC_lexicon_tags_37 },
    { 0x5739f690, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60b9f694, SUC_lexicon_tags_19 },
    { 0xb779f695, SUC_lexicon_tags_12 },
    { 0x5db9f696, SUC_lexicon_tags_37 },
    { 0x1c99f697, SUC_lexicon_tags_63 },
    { 0x2e39f698, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d9f69e, SUC_lexicon_tags_32 },
    { 0x6f39f69e, SUC_lexicon_tags_2 },
    { 0x7319f6a0, SUC_lexicon_tags_28 },
    { 0xd499f69f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb9f6a4, SUC_lexicon_tags_17 },
    { 0x5039f6a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81b9f6a9, SUC_lexicon_tags_17 },
    { 0xbe59f6aa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x35f9f6ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd499f6af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a59f6b1, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x2579f6b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x679f6b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe99f6b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5399f6b9, SUC_lexicon_tags_19 },
    { 0x2e19f6ba, SUC_lexicon_tags_34 },
    { 0x48f9f6bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19f6bf, SUC_lexicon_tags_15 },
    { 0x1c39f6c0, SUC_lexicon_tags_14 },
    { 0xbdd9f6c1, SUC_lexicon_tags_174 },
    { 0xd699f6c2, SUC_lexicon_tags_29 },
    { 0xd619f6c0, SUC_lexicon_tags_15 },
    { 0x85b9f6c2, SUC_lexicon_tags_25 },
    { 0x4b19f6c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd139f6c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe759f6cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b99f6d3, SUC_lexicon_tags_7 },
    { 0x37b9f6d3, SUC_lexicon_tags_17 },
    { 0x9039f6d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb399f6d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xef19f6da, SUC_lexicon_tags_7 },
    { 0x9139f6db, SUC_lexicon_tags_44 },
    { 0xd839f6db, SUC_lexicon_tags_28 },
    { 0x90f9f6dd, SUC_lexicon_tags_7 },
    { 0x4f99f6de, SUC_lexicon_tags_74 },
    { 0x519f6dc, SUC_lexicon_tags_15 },
    { 0xc19f6dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9f6e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa39f6e8, SUC_lexicon_tags_17 },
    { 0x4419f6e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bb9f6eb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3f9f6ee, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x2df9f6f1, SUC_lexicon_tags_38 },
    { 0x2299f6f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecb9f6f6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xe1f9f6f8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8b59f6fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a39f6ff, SUC_lexicon_tags_11 },
    { 0xedd9f700, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e59f703, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0f9f705, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa579f708, SUC_lexicon_tags_9 },
    { 0x8af9f709, SUC_lexicon_tags_37 },
    { 0x4939f708, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad9f70f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaed9f711, SUC_lexicon_tags_37 },
    { 0x51f9f712, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9359f714, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8c19f716, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xad59f719, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb39f71f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f722, SUC_lexicon_tags_57 },
    { 0xd659f723, SUC_lexicon_tags_0 },
    { 0xbe19f722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfeb9f726, SUC_lexicon_tags_47 },
    { 0x6df9f727, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5179f72f, SUC_lexicon_tags_4 },
    { 0x9959f72f, SUC_lexicon_tags_25 },
    { 0x9999f731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a79f735, SUC_lexicon_tags_481 },
    { 0xd779f735, SUC_lexicon_tags_37 },
    { 0x5b9f736, SUC_lexicon_tags_70 },
    { 0xf499f737, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd699f73a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6b9f73e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa1f9f740, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2759f743, SUC_lexicon_tags_37 },
    { 0xf0f9f743, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb639f748, SUC_lexicon_tags_14 },
    { 0x6c99f748, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f9f751, SUC_lexicon_tags_38 },
    { 0xe139f751, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa539f756, SUC_lexicon_tags_37 },
    { 0x7f79f757, SUC_lexicon_tags_114 },
    { 0x3959f758, SUC_lexicon_tags_15 },
    { 0x6c39f758, SUC_lexicon_tags_11 },
    { 0x65b9f759, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28f9f75d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12f9f75f, SUC_lexicon_tags_27 },
    { 0x8f59f760, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x30b9f763, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde79f768, SUC_lexicon_tags_2 },
    { 0x5ad9f768, SUC_lexicon_tags_35 },
    { 0x47f9f76a, SUC_lexicon_tags_13 },
    { 0xa119f76b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6359f76e, SUC_lexicon_tags_7 },
    { 0x2a99f76f, SUC_lexicon_tags_12 },
    { 0xdeb9f770, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb79f77b, SUC_lexicon_tags_37 },
    { 0x439f77c, SUC_lexicon_tags_30 },
    { 0xc3d9f77b, SUC_lexicon_tags_1 },
    { 0x3219f77b, SUC_lexicon_tags_63 },
    { 0x2f39f77f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88d9f783, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa679f785, SUC_lexicon_tags_17 },
    { 0x2419f786, SUC_lexicon_tags_1 },
    { 0x4039f785, SUC_lexicon_tags_37 },
    { 0x7599f788, SUC_lexicon_tags_11 },
    { 0xa059f789, SUC_lexicon_tags_23 },
    { 0xe5f9f78a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d9f792, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd59f79b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1299f79e, SUC_lexicon_tags_16 },
    { 0xcb79f79e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61d9f7a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfd9f7a3, SUC_lexicon_tags_9 },
    { 0xd439f7a4, SUC_lexicon_tags_37 },
    { 0xdfb9f7a5, SUC_lexicon_tags_14 },
    { 0xf4d9f7a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd19f7ab, SUC_lexicon_tags_45 },
    { 0x5179f7ab, SUC_lexicon_tags_2 },
    { 0x1659f7ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b59f7af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f9f7b2, SUC_lexicon_tags_17 },
    { 0xd499f7b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd019f7b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1519f7ba, SUC_lexicon_tags_38 },
    { 0x4cf9f7ba, SUC_lexicon_tags_26 },
    { 0xe339f7bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe19f7be, SUC_lexicon_tags_12 },
    { 0xd699f7bf, SUC_lexicon_tags_25 },
    { 0x5e19f7bf, SUC_lexicon_tags_113 },
    { 0x69f9f7c1, SUC_lexicon_tags_33 },
    { 0x8019f7c0, SUC_lexicon_tags_56 },
    { 0x16f9f7bf, SUC_lexicon_tags_40 },
    { 0xccf9f7c3, SUC_lexicon_tags_15 },
    { 0x2279f7c2, SUC_lexicon_tags_23 },
    { 0x8859f7be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ad9f7ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcd9f7cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d39f7d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb459f7d7, SUC_lexicon_tags_38 },
    { 0x20b9f7d7, SUC_lexicon_tags_11 },
    { 0x1719f7d9, SUC_lexicon_tags_1 },
    { 0xc299f7d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d59f7dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9f7e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3d9f7e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x98b9f7e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6bb9f7e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x239f7ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd659f7f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2539f7f9, SUC_lexicon_tags_80 },
    { 0x1fb9f7f9, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d99f7ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcef9f801, SUC_lexicon_tags_37 },
    { 0x2c19f802, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd759f807, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4579f80a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a99f80c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb39f817, SUC_lexicon_tags_37 },
    { 0xb339f818, SUC_lexicon_tags_28 },
    { 0xeb59f819, SUC_lexicon_tags_12 },
    { 0xc039f81a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf859f81c, SUC_lexicon_tags_3 },
    { 0x6479f81d, SUC_lexicon_tags_17 },
    { 0xd179f81c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b99f822, SUC_lexicon_tags_30 },
    { 0x7f9f823, SUC_lexicon_tags_25 },
    { 0x47f9f824, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc9b9f826, SUC_lexicon_tags_29 },
    { 0x6f9f826, SUC_lexicon_tags_13 },
    { 0xc199f828, SUC_lexicon_tags_47 },
    { 0x2bb9f829, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8139f830, SUC_lexicon_tags_37 },
    { 0xe079f831, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c79f836, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x57b9f839, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2f9f83f, SUC_lexicon_tags_14 },
    { 0xd819f840, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3439f843, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f79f846, SUC_lexicon_tags_7 },
    { 0xfd79f847, SUC_lexicon_tags_14 },
    { 0x1439f848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5459f84a, SUC_lexicon_tags_27 },
    { 0x7799f84b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84d9f84f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdbd9f851, SUC_lexicon_tags_87 },
    { 0x5199f851, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xae19f855, SUC_lexicon_tags_95 },
    { 0x5df9f856, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6d99f858, SUC_lexicon_tags_39 },
    { 0x9ed9f859, SUC_lexicon_tags_77 },
    { 0x2af9f85a, SUC_lexicon_tags_473 },
    { 0x59b9f859, SUC_lexicon_tags_1 },
    { 0xb439f859, SUC_lexicon_tags_8 },
    { 0xbcb9f85d, SUC_lexicon_tags_95 },
    { 0x4b79f85c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9779f865, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2499f87c, SUC_lexicon_tags_19 },
    { 0xb559f87c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cd9f880, SUC_lexicon_tags_17 },
    { 0x8f99f881, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa59f886, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd219f88b, SUC_lexicon_tags_47 },
    { 0xd8d9f88b, SUC_lexicon_tags_16 },
    { 0x2139f88d, SUC_lexicon_tags_45 },
    { 0x3a99f88e, SUC_lexicon_tags_42 },
    { 0x23b9f88f, SUC_lexicon_tags_1 },
    { 0x3d19f88f, SUC_lexicon_tags_15 },
    { 0x5f39f890, SUC_lexicon_tags_27 },
    { 0x6d19f890, SUC_lexicon_tags_27 },
    { 0x5619f88e, SUC_lexicon_tags_14 },
    { 0x7c99f894, SUC_lexicon_tags_61 },
    { 0xf479f891, SUC_lexicon_tags_8 },
    { 0x86f9f891, SUC_lexicon_tags_46 },
    { 0xeeb9f897, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7739f899, SUC_lexicon_tags_1 },
    { 0x3379f899, SUC_lexicon_tags_17 },
    { 0x5919f89b, SUC_lexicon_tags_2 },
    { 0x6f99f89c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a19f8a0, SUC_lexicon_tags_9 },
    { 0x81b9f8a1, SUC_lexicon_tags_28 },
    { 0x6199f8a2, SUC_lexicon_tags_12 },
    { 0x3d9f8a3, SUC_lexicon_tags_5 },
    { 0x3b99f8a4, SUC_lexicon_tags_7 },
    { 0x7d99f8a2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7799f8a7, SUC_lexicon_tags_2 },
    { 0xa9b9f8a7, SUC_lexicon_tags_7 },
    { 0xba59f8a9, SUC_lexicon_tags_42 },
    { 0x6499f8a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4259f8ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6d9f8b0, SUC_lexicon_tags_15 },
    { 0x6c59f8b1, SUC_lexicon_tags_9 },
    { 0x3d19f8b1, SUC_lexicon_tags_0 },
    { 0xc2b9f8b3, SUC_lexicon_tags_22 },
    { 0x7e59f8b0, SUC_lexicon_tags_37 },
    { 0xb159f8b5, SUC_lexicon_tags_63 },
    { 0x6c59f8b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5f9f8b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b9f8bc, SUC_lexicon_tags_87 },
    { 0xdd39f8bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3019f8bf, SUC_lexicon_tags_14 },
    { 0xdf39f8c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef39f8c3, SUC_lexicon_tags_26 },
    { 0xc559f8c4, SUC_lexicon_tags_29 },
    { 0x5519f8c4, SUC_lexicon_tags_77 },
    { 0x27f9f8c6, SUC_lexicon_tags_92 },
    { 0x91f9f8c5, SUC_lexicon_tags_12 },
    { 0xfcd9f8c8, SUC_lexicon_tags_37 },
    { 0xe239f8c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4d9f8ce, SUC_lexicon_tags_38 },
    { 0xa699f8cf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1219f8d2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cf9f8d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f79f8df, SUC_lexicon_tags_25 },
    { 0xda59f8e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6759f8e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc059f8e7, SUC_lexicon_tags_34 },
    { 0xe199f8e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3479f8ea, SUC_lexicon_tags_33 },
    { 0xe9f9f8eb, SUC_lexicon_tags_38 },
    { 0xe219f8ec, SUC_lexicon_tags_11 },
    { 0x2239f8ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38d9f8f1, SUC_lexicon_tags_37 },
    { 0xfbf9f8f2, SUC_lexicon_tags_0 },
    { 0x9a39f8f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d9f8f6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x37f9f8f9, SUC_lexicon_tags_11 },
    { 0x3c99f8fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f39f8fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d99f900, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99f903, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda19f90b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc879f90d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff9f910, SUC_lexicon_tags_27 },
    { 0xecf9f911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f99f913, SUC_lexicon_tags_25 },
    { 0xd279f914, SUC_lexicon_tags_47 },
    { 0xf279f915, SUC_lexicon_tags_11 },
    { 0x3239f916, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8f9f918, SUC_lexicon_tags_25 },
    { 0x62f9f918, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6059f91d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81d9f91f, SUC_lexicon_tags_12 },
    { 0x7119f920, SUC_lexicon_tags_37 },
    { 0xc199f91f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd19f926, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x30b9f928, SUC_lexicon_tags_51 },
    { 0xa9b9f929, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb979f92d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e59f931, SUC_lexicon_tags_38 },
    { 0x1a99f931, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f59f935, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1e39f937, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd9f93a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579f93f, SUC_lexicon_tags_92 },
    { 0x4dd9f940, SUC_lexicon_tags_4 },
    { 0xde39f941, SUC_lexicon_tags_25 },
    { 0xf9b9f942, SUC_lexicon_tags_15 },
    { 0x5519f941, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x8c99f945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33d9f949, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd059f94b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd659f94f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd9f952, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d9f955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9879f958, SUC_lexicon_tags_14 },
    { 0xaaf9f958, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f99f95c, SUC_lexicon_tags_0 },
    { 0x1cd9f95d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8479f961, SUC_lexicon_tags_15 },
    { 0xc579f962, SUC_lexicon_tags_27 },
    { 0xd499f963, SUC_lexicon_tags_17 },
    { 0x65b9f961, SUC_lexicon_tags_30 },
    { 0xbe39f965, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdd99f967, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6299f96b, SUC_lexicon_tags_63 },
    { 0x3059f96c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe99f96e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec99f97d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7559f983, SUC_lexicon_tags_2 },
    { 0x8319f984, SUC_lexicon_tags_31 },
    { 0xcf79f984, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff9f988, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc659f991, SUC_lexicon_tags_34 },
    { 0x2d99f991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf19f99b, SUC_lexicon_tags_25 },
    { 0x50d9f99c, SUC_lexicon_tags_45 },
    { 0x939f99b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a19f9a1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f9a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b9f9a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa359f9aa, SUC_lexicon_tags_12 },
    { 0xbab9f9aa, SUC_lexicon_tags_19 },
    { 0x36b9f9aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7599f9b3, SUC_lexicon_tags_66 },
    { 0xe1b9f9b4, SUC_lexicon_tags_25 },
    { 0x4d9f9b3, SUC_lexicon_tags_19 },
    { 0x2659f9b3, SUC_lexicon_tags_87 },
    { 0x6ad9f9b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x79f9b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f9f9bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf159f9bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd59f9c3, SUC_lexicon_tags_82 },
    { 0xb39f9c4, SUC_lexicon_tags_37 },
    { 0x1ef9f9c3, SUC_lexicon_tags_63 },
    { 0x8859f9c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119f9ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d9f9d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4719f9d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd39f9db, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1099f9df, SUC_lexicon_tags_37 },
    { 0x5a79f9df, SUC_lexicon_tags_55 },
    { 0x5c39f9e0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d9f9e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d9f9ea, SUC_lexicon_tags_47 },
    { 0x2539f9ea, SUC_lexicon_tags_14 },
    { 0x6c99f9ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe299f9ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe459f9f0, SUC_lexicon_tags_12 },
    { 0xc4b9f9f0, SUC_lexicon_tags_25 },
    { 0xb9d9f9f2, SUC_lexicon_tags_30 },
    { 0xc879f9f0, SUC_lexicon_tags_55 },
    { 0xd379f9f4, SUC_lexicon_tags_11 },
    { 0xc999f9f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22b9f9fb, SUC_lexicon_tags_21 },
    { 0xc819f9fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc119f9ff, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6b9fa04, SUC_lexicon_tags_45 },
    { 0xbfd9fa04, SUC_lexicon_tags_45 },
    { 0x64b9fa06, SUC_lexicon_tags_12 },
    { 0x9c99fa07, SUC_lexicon_tags_63 },
    { 0x4d19fa07, SUC_lexicon_tags_15 },
    { 0xe099fa09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b9fa0d, SUC_lexicon_tags_37 },
    { 0x15f9fa0e, SUC_lexicon_tags_11 },
    { 0xd099fa0d, SUC_lexicon_tags_16 },
    { 0x54b9fa10, SUC_lexicon_tags_12 },
    { 0x8b59fa0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5239fa13, SUC_lexicon_tags_7 },
    { 0xeff9fa14, SUC_lexicon_tags_27 },
    { 0x6e79fa15, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe19fa19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7419fa1c, SUC_lexicon_tags_15 },
    { 0xe59fa1d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23f9fa21, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdcd9fa23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7499fa25, SUC_lexicon_tags_17 },
    { 0x4bf9fa26, SUC_lexicon_tags_25 },
    { 0xc219fa27, SUC_lexicon_tags_11 },
    { 0x4199fa26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9319fa2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d9fa2d, SUC_lexicon_tags_15 },
    { 0xb079fa2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1d9fa30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c59fa33, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd9fa36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x879fa38, SUC_lexicon_tags_7 },
    { 0x6079fa38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3119fa3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6d9fa45, SUC_lexicon_tags_28 },
    { 0x8179fa46, SUC_lexicon_tags_0 },
    { 0xb439fa46, SUC_lexicon_tags_23 },
    { 0xb659fa48, SUC_lexicon_tags_17 },
    { 0xfa79fa48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b9fa4d, SUC_lexicon_tags_2 },
    { 0x3ef9fa4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54f9fa54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5779fa56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f9fa5c, SUC_lexicon_tags_160 },
    { 0xd299fa5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf679fa5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed99fa62, SUC_lexicon_tags_12 },
    { 0x5db9fa63, SUC_lexicon_tags_9 },
    { 0x6459fa62, SUC_lexicon_tags_30 },
    { 0x6dd9fa65, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2899fa67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85d9fa69, SUC_lexicon_tags_4 },
    { 0x3ab9fa6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7eb9fa6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26f9fa71, SUC_lexicon_tags_16 },
    { 0xe9f9fa72, SUC_lexicon_tags_77 },
    { 0xd159fa73, SUC_lexicon_tags_23 },
    { 0x7b99fa74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1499fa76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xae19fa78, SUC_lexicon_tags_63 },
    { 0x21f9fa79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd219fa7b, SUC_lexicon_tags_27 },
    { 0xdb59fa7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79fa80, SUC_lexicon_tags_23 },
    { 0xde99fa81, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f19fa84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a79fa88, SUC_lexicon_tags_37 },
    { 0x5579fa88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbf9fa8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff9fa8f, SUC_lexicon_tags_63 },
    { 0xe379fa90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf79fa95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa399fa99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb9fa9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb99fa9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d9faa2, SUC_lexicon_tags_29 },
    { 0xd79faa3, SUC_lexicon_tags_39 },
    { 0x2579faa4, SUC_lexicon_tags_6 },
    { 0xe639faa5, SUC_lexicon_tags_37 },
    { 0x9a79faa4, SUC_lexicon_tags_47 },
    { 0xec39faa7, SUC_lexicon_tags_16 },
    { 0xab59faa8, SUC_lexicon_tags_17 },
    { 0x46f9faa8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1ef9faab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d9faae, SUC_lexicon_tags_138 },
    { 0x7d99faaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc019fab7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b9fabd, SUC_lexicon_tags_1 },
    { 0x26b9fabd, SUC_lexicon_tags_17 },
    { 0xe839fabf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e19fac3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf899fac8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6cd9faca, SUC_lexicon_tags_1 },
    { 0x25b9facb, SUC_lexicon_tags_14 },
    { 0x5199faca, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b19facf, SUC_lexicon_tags_63 },
    { 0x24b9fad0, SUC_lexicon_tags_47 },
    { 0x5d19fad1, SUC_lexicon_tags_27 },
    { 0xdb19fad1, SUC_lexicon_tags_17 },
    { 0x9f99fad2, SUC_lexicon_tags_37 },
    { 0xd979fad4, SUC_lexicon_tags_45 },
    { 0x30b9fad1, SUC_lexicon_tags_7 },
    { 0xcc19fad6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xde99fad9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb9fadd, SUC_lexicon_tags_2 },
    { 0x7d59fadd, SUC_lexicon_tags_12 },
    { 0x98f9fadf, SUC_lexicon_tags_49 },
    { 0xaaf9fadd, SUC_lexicon_tags_1 },
    { 0x4b99fadd, SUC_lexicon_tags_32 },
    { 0xba59fadf, SUC_lexicon_tags_63 },
    { 0x2779fae3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce59fae7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfc99fae9, SUC_lexicon_tags_12 },
    { 0x30f9faea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7d9faed, SUC_lexicon_tags_71 },
    { 0xaf59faee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c19faf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8779faf2, SUC_lexicon_tags_16 },
    { 0xe799faf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f39faf6, SUC_lexicon_tags_25 },
    { 0x69d9faf7, SUC_lexicon_tags_11 },
    { 0x5659faf8, SUC_lexicon_tags_32 },
    { 0x5459faf9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe1f9fafb, SUC_lexicon_tags_2 },
    { 0x50f9fafb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd9fb01, SUC_lexicon_tags_2 },
    { 0x6eb9fb01, SUC_lexicon_tags_7 },
    { 0x2b39fb03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4f9fb06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c39fb08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf99fb0a, SUC_lexicon_tags_19 },
    { 0x7179fb0b, SUC_lexicon_tags_7 },
    { 0xf799fb0c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2859fb0e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe919fb13, SUC_lexicon_tags_26 },
    { 0xc4b9fb14, SUC_lexicon_tags_3 },
    { 0x4319fb13, SUC_lexicon_tags_19 },
    { 0x11b9fb16, SUC_lexicon_tags_17 },
    { 0x6f9fb14, SUC_lexicon_tags_30 },
    { 0x9c39fb18, SUC_lexicon_tags_35 },
    { 0x1339fb19, SUC_lexicon_tags_17 },
    { 0xff9fb1a, SUC_lexicon_tags_463 },
    { 0xfe99fb18, SUC_lexicon_tags_25 },
    { 0xbd39fb1c, SUC_lexicon_tags_30 },
    { 0x82d9fb19, SUC_lexicon_tags_106 },
    { 0x2959fb18, SUC_lexicon_tags_26 },
    { 0xd879fb1f, SUC_lexicon_tags_14 },
    { 0x6639fb20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc119fb25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b9fb2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc39fb2e, SUC_lexicon_tags_15 },
    { 0x8359fb2f, SUC_lexicon_tags_732 },
    { 0x4919fb2f, SUC_lexicon_tags_47 },
    { 0x659fb2f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d79fb36, SUC_lexicon_tags_6 },
    { 0x7d59fb36, SUC_lexicon_tags_1 },
    { 0xf279fb38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0f9fb3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc059fb44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f9fb49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5799fb4b, SUC_lexicon_tags_9 },
    { 0x1f99fb4b, SUC_lexicon_tags_15 },
    { 0x8719fb4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3439fb56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4bf9fb58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e39fb5a, SUC_lexicon_tags_14 },
    { 0xe2b9fb5a, SUC_lexicon_tags_2 },
    { 0xd6f9fb5b, SUC_lexicon_tags_42 },
    { 0xb699fb5d, SUC_lexicon_tags_5 },
    { 0x3979fb5e, SUC_lexicon_tags_27 },
    { 0x1559fb5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b39fb62, SUC_lexicon_tags_33 },
    { 0x1a99fb63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf99fb65, SUC_lexicon_tags_16 },
    { 0x7799fb66, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9eb9fb68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2699fb6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe059fb6f, SUC_lexicon_tags_0 },
    { 0x8a79fb6f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa79fb75, SUC_lexicon_tags_18 },
    { 0x3bb9fb76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d19fb78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe19fb7b, SUC_lexicon_tags_25 },
    { 0x3499fb7c, SUC_lexicon_tags_11 },
    { 0xb59fb7c, SUC_lexicon_tags_10 },
    { 0x1eb9fb7c, SUC_lexicon_tags_30 },
    { 0xac99fb7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d39fb84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7659fb89, SUC_lexicon_tags_25 },
    { 0x6839fb8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75f9fb8c, SUC_lexicon_tags_31 },
    { 0x51b9fb8c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b9fb90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199fb95, SUC_lexicon_tags_25 },
    { 0x70b9fb96, SUC_lexicon_tags_85 },
    { 0x6339fb97, SUC_lexicon_tags_60 },
    { 0xfef9fb95, SUC_lexicon_tags_30 },
    { 0x23d9fb99, SUC_lexicon_tags_15 },
    { 0x28f9fb9a, SUC_lexicon_tags_1 },
    { 0xa2d9fb98, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9759fb9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4179fba2, SUC_lexicon_tags_8 },
    { 0x4499fba3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x12f9fba6, SUC_lexicon_tags_0 },
    { 0x96d9fba6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ef9fbae, SUC_lexicon_tags_594 },
    { 0xba79fbaf, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda99fbb7, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x19d9fbb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4979fbc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f19fbcd, SUC_lexicon_tags_32 },
    { 0x7539fbcd, SUC_lexicon_tags_1 },
    { 0x6499fbcf, SUC_lexicon_tags_46 },
    { 0x90b9fbd0, SUC_lexicon_tags_11 },
    { 0x5879fbd0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3899fbd3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a39fbd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa779fbe3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x87f9fbe6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb919fbea, SUC_lexicon_tags_55 },
    { 0x3059fbeb, SUC_lexicon_tags_13 },
    { 0xbfd9fbec, SUC_lexicon_tags_11 },
    { 0x46b9fbed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaad9fbf1, SUC_lexicon_tags_7 },
    { 0x5919fbf2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6379fbf4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d79fbf9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a39fbfe, SUC_lexicon_tags_24 },
    { 0xb2d9fbff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb359fc0b, SUC_lexicon_tags_7 },
    { 0x9739fc0c, SUC_lexicon_tags_84 },
    { 0x47f9fc0b, SUC_lexicon_tags_37 },
    { 0xdb19fc0e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6399fc10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff59fc14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a19fc18, SUC_lexicon_tags_12 },
    { 0x33b9fc19, SUC_lexicon_tags_113 },
    { 0xf119fc1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b39fc20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd539fc22, SUC_lexicon_tags_25 },
    { 0x6219fc22, SUC_lexicon_tags_33 },
    { 0xb659fc23, SUC_lexicon_tags_19 },
    { 0xb359fc22, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf79fc27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2939fc29, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe99fc2c, SUC_lexicon_tags_37 },
    { 0x8c79fc2d, SUC_lexicon_tags_37 },
    { 0xf8f9fc2e, SUC_lexicon_tags_15 },
    { 0x7339fc2f, SUC_lexicon_tags_37 },
    { 0xbf59fc2d, SUC_lexicon_tags_26 },
    { 0xc7b9fc2f, SUC_lexicon_tags_17 },
    { 0xf1b9fc32, SUC_lexicon_tags_33 },
    { 0xaf39fc2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x97d9fc36, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9fc3b, SUC_lexicon_tags_19 },
    { 0xfa99fc3b, SUC_lexicon_tags_95 },
    { 0xb9d9fc3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9fc42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65d9fc44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f9fc48, SUC_lexicon_tags_38 },
    { 0x5399fc48, SUC_lexicon_tags_12 },
    { 0x8e79fc4a, SUC_lexicon_tags_37 },
    { 0x62b9fc4a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84f9fc4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9119fc53, SUC_lexicon_tags_77 },
    { 0xfb19fc54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd639fc56, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd939fc5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb99fc5e, SUC_lexicon_tags_39 },
    { 0x57d9fc5f, SUC_lexicon_tags_30 },
    { 0x9bd9fc60, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x54d9fc63, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcb99fc65, SUC_lexicon_tags_25 },
    { 0x3119fc66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4439fc68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7f9fc6a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd699fc6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f9fc76, SUC_lexicon_tags_151 },
    { 0x80b9fc77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcd9fc7b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xc159fc7e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fc81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe759fc85, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb479fc87, SUC_lexicon_tags_12 },
    { 0xf4d9fc88, SUC_lexicon_tags_19 },
    { 0x8a59fc89, SUC_lexicon_tags_46 },
    { 0xd119fc89, SUC_lexicon_tags_77 },
    { 0xbb9fc8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3219fc8d, SUC_lexicon_tags_12 },
    { 0xddf9fc8d, SUC_lexicon_tags_21 },
    { 0xc4b9fc8f, SUC_lexicon_tags_2 },
    { 0xdc39fc8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3b9fc92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae59fc9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6b9fc9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f79fca1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e19fca6, SUC_lexicon_tags_77 },
    { 0x4939fca6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee39fcac, SUC_lexicon_tags_6 },
    { 0xc2b9fcac, SUC_lexicon_tags_11 },
    { 0xaf39fcae, SUC_lexicon_tags_10 },
    { 0xbad9fcaf, SUC_lexicon_tags_16 },
    { 0xdd79fcac, SUC_lexicon_tags_2 },
    { 0x9f9fcb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8f9fcbd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf359fcbf, SUC_lexicon_tags_1 },
    { 0x84d9fcc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x84f9fcc3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b9fcc9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf319fccd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61d9fcd1, SUC_lexicon_tags_19 },
    { 0x7c59fcd1, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8599fcd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3af9fcd8, SUC_lexicon_tags_57 },
    { 0x8959fcd8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6939fcdb, SUC_lexicon_tags_7 },
    { 0x2c39fcdc, SUC_lexicon_tags_27 },
    { 0xaf9fcdd, SUC_lexicon_tags_87 },
    { 0xd539fcdd, SUC_lexicon_tags_1 },
    { 0x74f9fcdf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b59fce7, SUC_lexicon_tags_15 },
    { 0x3df9fce8, SUC_lexicon_tags_60 },
    { 0x7979fce8, SUC_lexicon_tags_12 },
    { 0x2e59fce7, SUC_lexicon_tags_1 },
    { 0x2699fceb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf59fcf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ed9fcf2, SUC_lexicon_tags_11 },
    { 0x3539fcf3, SUC_lexicon_tags_2 },
    { 0x7499fcf4, SUC_lexicon_tags_25 },
    { 0x5039fcf4, SUC_lexicon_tags_8 },
    { 0x5719fcf2, SUC_lexicon_tags_17 },
    { 0xf59fcf7, SUC_lexicon_tags_17 },
    { 0x7c79fcf6, SUC_lexicon_tags_38 },
    { 0xda99fcf2, SUC_lexicon_tags_36 },
    { 0x5af9fcfa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb779fcff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d9fd08, SUC_lexicon_tags_49 },
    { 0xb2d9fd09, SUC_lexicon_tags_31 },
    { 0xac79fd09, SUC_lexicon_tags_85 },
    { 0xd399fd08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e79fd14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5779fd17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c99fd1a, SUC_lexicon_tags_14 },
    { 0xfcf9fd1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e59fd1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8719fd22, SUC_lexicon_tags_37 },
    { 0xd279fd22, SUC_lexicon_tags_17 },
    { 0x41d9fd23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4259fd28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f9fd2b, SUC_lexicon_tags_70 },
    { 0x4f9fd2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9319fd32, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d19fd39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc719fd3d, SUC_lexicon_tags_12 },
    { 0xfe79fd3e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc3b9fd40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2279fd43, SUC_lexicon_tags_33 },
    { 0xcd19fd43, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e79fd48, SUC_lexicon_tags_2 },
    { 0x9519fd48, SUC_lexicon_tags_79 },
    { 0x5559fd4a, SUC_lexicon_tags_1 },
    { 0x5af9fd4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db9fd4e, SUC_lexicon_tags_33 },
    { 0x99b9fd4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c19fd51, SUC_lexicon_tags_69 },
    { 0x43b9fd52, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xca39fd54, SUC_lexicon_tags_26 },
    { 0xc59fd55, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa959fd59, SUC_lexicon_tags_13 },
    { 0x1099fd59, SUC_lexicon_tags_719 },
    { 0, NULL },
    { 0, NULL },
    { 0x13b9fd5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2b9fd60, SUC_lexicon_tags_66 },
    { 0x5df9fd61, SUC_lexicon_tags_25 },
    { 0x2359fd61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f9fd69, SUC_lexicon_tags_10 },
    { 0xf419fd6a, SUC_lexicon_tags_15 },
    { 0xda99fd6a, SUC_lexicon_tags_11 },
    { 0x3c79fd6b, SUC_lexicon_tags_79 },
    { 0xd119fd6d, SUC_lexicon_tags_37 },
    { 0x5e79fd6e, SUC_lexicon_tags_15 },
    { 0xb9b9fd6f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2619fd73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb139fd77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf39fd79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd899fd7b, SUC_lexicon_tags_38 },
    { 0x51d9fd7c, SUC_lexicon_tags_25 },
    { 0x18d9fd7c, SUC_lexicon_tags_2 },
    { 0xfb19fd7e, SUC_lexicon_tags_17 },
    { 0xf439fd7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9b59fd81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8679fd83, SUC_lexicon_tags_34 },
    { 0x1799fd84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eb9fd87, SUC_lexicon_tags_15 },
    { 0xd2f9fd88, SUC_lexicon_tags_23 },
    { 0xb6d9fd87, SUC_lexicon_tags_922 },
    { 0, NULL },
    { 0x5a99fd8b, SUC_lexicon_tags_31 },
    { 0x66f9fd8c, SUC_lexicon_tags_33 },
    { 0x2559fd8b, SUC_lexicon_tags_11 },
    { 0x5c59fd8c, SUC_lexicon_tags_17 },
    { 0xddf9fd8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8d9fd91, SUC_lexicon_tags_9 },
    { 0x8b39fd92, SUC_lexicon_tags_31 },
    { 0xecf9fd92, SUC_lexicon_tags_55 },
    { 0xc39fd91, SUC_lexicon_tags_17 },
    { 0x23f9fd94, SUC_lexicon_tags_17 },
    { 0xac19fd96, SUC_lexicon_tags_1 },
    { 0x5439fd97, SUC_lexicon_tags_11 },
    { 0xa9d9fd95, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x6519fd9b, SUC_lexicon_tags_15 },
    { 0x38b9fd9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe059fd9f, SUC_lexicon_tags_12 },
    { 0x3779fda0, SUC_lexicon_tags_17 },
    { 0xf8d9fda1, SUC_lexicon_tags_39 },
    { 0x9c59fd9f, SUC_lexicon_tags_11 },
    { 0x5959fda3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e59fda8, SUC_lexicon_tags_11 },
    { 0x4e99fda8, SUC_lexicon_tags_55 },
    { 0xaad9fdaa, SUC_lexicon_tags_23 },
    { 0x6e59fdaa, SUC_lexicon_tags_14 },
    { 0xf9b9fdab, SUC_lexicon_tags_1 },
    { 0xbc39fda8, SUC_lexicon_tags_2 },
    { 0xf379fdae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd439fdb0, SUC_lexicon_tags_1 },
    { 0xd6b9fdb1, SUC_lexicon_tags_66 },
    { 0xf99fdb2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x4659fdb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb39fdb9, SUC_lexicon_tags_2 },
    { 0xca19fdba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4b9fdc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f9fdc7, SUC_lexicon_tags_9 },
    { 0x3ab9fdc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe59fdcf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf839fdd1, SUC_lexicon_tags_30 },
    { 0x2579fdd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6299fdd5, SUC_lexicon_tags_1 },
    { 0x4cd9fdd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe639fddb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8239fddd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff9fddf, SUC_lexicon_tags_193 },
    { 0x41f9fde0, SUC_lexicon_tags_3 },
    { 0x3bb9fde1, SUC_lexicon_tags_10 },
    { 0xad59fde2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d99fde5, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe519fdea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7219fdec, SUC_lexicon_tags_23 },
    { 0x8539fded, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d9fdef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3579fdf3, SUC_lexicon_tags_43 },
    { 0x4339fdf4, SUC_lexicon_tags_55 },
    { 0x9319fdf5, SUC_lexicon_tags_2 },
    { 0x5639fdf5, SUC_lexicon_tags_11 },
    { 0xac59fdf7, SUC_lexicon_tags_0 },
    { 0x4bd9fdf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef39fdfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe599fdff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7039fe02, SUC_lexicon_tags_45 },
    { 0xb359fe03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32f9fe0d, SUC_lexicon_tags_77 },
    { 0x3539fe0d, SUC_lexicon_tags_63 },
    { 0xa719fe0f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb539fe14, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f9fe17, SUC_lexicon_tags_33 },
    { 0xfcd9fe18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d59fe1c, SUC_lexicon_tags_38 },
    { 0xc999fe1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29d9fe26, SUC_lexicon_tags_1 },
    { 0x61f9fe26, SUC_lexicon_tags_25 },
    { 0x14f9fe27, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc859fe2b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb4f9fe2d, SUC_lexicon_tags_29 },
    { 0x7559fe2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xff59fe30, SUC_lexicon_tags_25 },
    { 0xfcd9fe31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf259fe33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe799fe38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf899fe3f, SUC_lexicon_tags_45 },
    { 0xef9fe40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cf9fe45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6f9fe47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x35f9fe49, SUC_lexicon_tags_30 },
    { 0xd039fe4a, SUC_lexicon_tags_56 },
    { 0xb2d9fe4b, SUC_lexicon_tags_17 },
    { 0x7699fe4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83b9fe4e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1899fe52, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b39fe55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba99fe57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9eb9fe59, SUC_lexicon_tags_47 },
    { 0xf9fe59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa539fe5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa479fe61, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc99fe64, SUC_lexicon_tags_5 },
    { 0xb859fe65, SUC_lexicon_tags_34 },
    { 0xbc19fe66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0f9fe70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd659fe75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b79fe78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c99fe7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c99fe7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd579fe7f, SUC_lexicon_tags_0 },
    { 0x23f9fe80, SUC_lexicon_tags_31 },
    { 0x9939fe7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbdb9fe83, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e39fe86, SUC_lexicon_tags_63 },
    { 0x5d9fe87, SUC_lexicon_tags_17 },
    { 0x4139fe88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1d9fe8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe2f9fe8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3239fe8e, SUC_lexicon_tags_27 },
    { 0xa99fe8f, SUC_lexicon_tags_38 },
    { 0x26f9fe8e, SUC_lexicon_tags_8 },
    { 0xc959fe8f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd9fe94, SUC_lexicon_tags_30 },
    { 0xb359fe95, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd779fe99, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc519fe9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87b9fe9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5a19fe9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xbb39fea1, SUC_lexicon_tags_28 },
    { 0xb2f9fea2, SUC_lexicon_tags_43 },
    { 0x1d59fea3, SUC_lexicon_tags_12 },
    { 0x9ed9fea1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8459fea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4359feab, SUC_lexicon_tags_39 },
    { 0x16d9feac, SUC_lexicon_tags_2 },
    { 0x3d9feac, SUC_lexicon_tags_12 },
    { 0xa099feae, SUC_lexicon_tags_33 },
    { 0x6059feae, SUC_lexicon_tags_45 },
    { 0xe1b9fead, SUC_lexicon_tags_4 },
    { 0x839feaf, SUC_lexicon_tags_7 },
    { 0xf699feb2, SUC_lexicon_tags_12 },
    { 0x4b39feb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b9feb7, SUC_lexicon_tags_19 },
    { 0xc579feb8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b99febe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46d9fec1, SUC_lexicon_tags_37 },
    { 0xd619fec2, SUC_lexicon_tags_37 },
    { 0xd939fec2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0d9fec5, SUC_lexicon_tags_49 },
    { 0xabd9fec6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5099fec8, SUC_lexicon_tags_47 },
    { 0x8839fec9, SUC_lexicon_tags_17 },
    { 0xff39fec9, SUC_lexicon_tags_17 },
    { 0xec99feca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72b9fed0, SUC_lexicon_tags_63 },
    { 0x6599fed1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82b9fed6, SUC_lexicon_tags_37 },
    { 0x3e59fed6, SUC_lexicon_tags_26 },
    { 0xeeb9fed7, SUC_lexicon_tags_39 },
    { 0x2fb9fed7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8519fedb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e39fedf, SUC_lexicon_tags_15 },
    { 0x4459fee0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd6f9fee2, SUC_lexicon_tags_56 },
    { 0xa059fee2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb99fee5, SUC_lexicon_tags_2 },
    { 0x53b9fee6, SUC_lexicon_tags_1 },
    { 0xf999fee7, SUC_lexicon_tags_3 },
    { 0x36f9fee8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x979feea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x16d9feed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9419fef0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba79fefc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaff9fefe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f19ff05, SUC_lexicon_tags_0 },
    { 0xadf9ff06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d59ff0a, SUC_lexicon_tags_12 },
    { 0xbb19ff0a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9b9ff0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4179ff0f, SUC_lexicon_tags_708 },
    { 0x1159ff10, SUC_lexicon_tags_12 },
    { 0xd5f9ff11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e19ff14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed99ff18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xded9ff1a, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x52d9ff1c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x739ff1f, SUC_lexicon_tags_60 },
    { 0x3159ff20, SUC_lexicon_tags_26 },
    { 0x5459ff1f, SUC_lexicon_tags_45 },
    { 0xff79ff22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd9ff27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6439ff2a, SUC_lexicon_tags_38 },
    { 0x3159ff2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4b9ff2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x49d9ff31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x699ff33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d19ff38, SUC_lexicon_tags_14 },
    { 0x3d79ff39, SUC_lexicon_tags_14 },
    { 0x6199ff3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1859ff46, SUC_lexicon_tags_37 },
    { 0x5c59ff47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff59ff51, SUC_lexicon_tags_25 },
    { 0x5879ff52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x419ff54, SUC_lexicon_tags_17 },
    { 0xf639ff55, SUC_lexicon_tags_1 },
    { 0xe719ff56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67f9ff59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8359ff5c, SUC_lexicon_tags_1 },
    { 0x1b59ff5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79ff66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2d9ff6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c59ff71, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8999ff77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca99ff79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf299ff7b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2339ff7e, SUC_lexicon_tags_8 },
    { 0x8cd9ff7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4799ff82, SUC_lexicon_tags_15 },
    { 0xb1d9ff83, SUC_lexicon_tags_17 },
    { 0x4399ff84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab79ff8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf19ff93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3959ff98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x18d9ff9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4059ff9d, SUC_lexicon_tags_31 },
    { 0x1539ff9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba99ffa3, SUC_lexicon_tags_6 },
    { 0x7fd9ffa4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe759ffa7, SUC_lexicon_tags_21 },
    { 0x9019ffa7, SUC_lexicon_tags_77 },
    { 0xc439ffa9, SUC_lexicon_tags_12 },
    { 0x3b99ffaa, SUC_lexicon_tags_15 },
    { 0x73b9ffab, SUC_lexicon_tags_11 },
    { 0xfdd9ffab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff9ffb0, SUC_lexicon_tags_7 },
    { 0x119ffb1, SUC_lexicon_tags_28 },
    { 0x259ffb2, SUC_lexicon_tags_160 },
    { 0xf699ffb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1739ffb5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1af9ffba, SUC_lexicon_tags_11 },
    { 0x53d9ffbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd9ffbf, SUC_lexicon_tags_63 },
    { 0x5019ffc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d39ffc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60f9ffce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x43d9ffd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6279ffd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21b9ffdf, SUC_lexicon_tags_77 },
    { 0x6ab9ffdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d99ffe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6159ffe7, SUC_lexicon_tags_26 },
    { 0x4519ffe8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc39ffea, SUC_lexicon_tags_14 },
    { 0x4779ffeb, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1279ffed, SUC_lexicon_tags_1 },
    { 0xe719ffee, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x7299fff0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf6b9fff2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7b99fff4, SUC_lexicon_tags_37 },
    { 0x4a99fff4, SUC_lexicon_tags_57 },
    { 0x1739fff6, SUC_lexicon_tags_8 },
    { 0xa6b9fff7, SUC_lexicon_tags_15 },
    { 0x9139fff7, SUC_lexicon_tags_0 },
    { 0x9899fff9, SUC_lexicon_tags_15 },
    { 0x7919fffa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8359fffe, SUC_lexicon_tags_12 },
    { 0xc7b9ffff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf29a0002, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd99a0004, SUC_lexicon_tags_15 },
    { 0xedfa0005, SUC_lexicon_tags_17 },
    { 0x383a0006, SUC_lexicon_tags_7 },
    { 0x409a0006, SUC_lexicon_tags_243 },
    { 0x1b5a0008, SUC_lexicon_tags_37 },
    { 0x2d1a0009, SUC_lexicon_tags_46 },
    { 0x7d3a0009, SUC_lexicon_tags_47 },
    { 0x913a000b, SUC_lexicon_tags_25 },
    { 0x7c1a000c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1a0011, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5a0014, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x765a0016, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x861a0019, SUC_lexicon_tags_11 },
    { 0x1a3a0019, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2ffa001c, SUC_lexicon_tags_26 },
    { 0xbd3a001d, SUC_lexicon_tags_25 },
    { 0x70fa001e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3a0021, SUC_lexicon_tags_29 },
    { 0x6b9a0022, SUC_lexicon_tags_2 },
    { 0x8e5a0023, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afa0029, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ba002c, SUC_lexicon_tags_31 },
    { 0x879a002d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4da0032, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47a0038, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69da003c, SUC_lexicon_tags_14 },
    { 0x73ba003c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835a0041, SUC_lexicon_tags_15 },
    { 0xd6da0042, SUC_lexicon_tags_19 },
    { 0xd13a0043, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5a004d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a0052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90fa0054, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ba0058, SUC_lexicon_tags_2 },
    { 0xd39a0059, SUC_lexicon_tags_12 },
    { 0xf0da0059, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03a0067, SUC_lexicon_tags_65 },
    { 0x255a0067, SUC_lexicon_tags_0 },
    { 0xe43a0068, SUC_lexicon_tags_33 },
    { 0xa61a006a, SUC_lexicon_tags_12 },
    { 0x32da006b, SUC_lexicon_tags_25 },
    { 0x81fa006c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2da0073, SUC_lexicon_tags_46 },
    { 0x699a0074, SUC_lexicon_tags_40 },
    { 0x1c9a0074, SUC_lexicon_tags_8 },
    { 0xb03a0076, SUC_lexicon_tags_0 },
    { 0xb35a0077, SUC_lexicon_tags_17 },
    { 0xf51a0073, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afa007e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x499a0081, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x72fa0083, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfffa0085, SUC_lexicon_tags_49 },
    { 0x995a0086, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xffda0088, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27a008b, SUC_lexicon_tags_12 },
    { 0xddba008c, SUC_lexicon_tags_23 },
    { 0x9c9a008d, SUC_lexicon_tags_26 },
    { 0xd41a008e, SUC_lexicon_tags_27 },
    { 0xa1ba008f, SUC_lexicon_tags_17 },
    { 0xcc5a008d, SUC_lexicon_tags_1 },
    { 0x613a0090, SUC_lexicon_tags_75 },
    { 0x5a9a008d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3cda0094, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x347a0097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x765a009a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ba009d, SUC_lexicon_tags_15 },
    { 0x395a009e, SUC_lexicon_tags_37 },
    { 0x723a009d, SUC_lexicon_tags_0 },
    { 0x329a00a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x257a00a3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fa00a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbfa00a8, SUC_lexicon_tags_6 },
    { 0x407a00a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7a00ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efa00b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19ba00b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3da00b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455a00bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463a00c2, SUC_lexicon_tags_79 },
    { 0xdb5a00c3, SUC_lexicon_tags_17 },
    { 0x63ba00c4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x515a00c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ffa00c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x465a00ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda00ce, SUC_lexicon_tags_27 },
    { 0x9e7a00ce, SUC_lexicon_tags_43 },
    { 0x2bba00d0, SUC_lexicon_tags_94 },
    { 0xda7a00d1, SUC_lexicon_tags_17 },
    { 0x3b9a00d2, SUC_lexicon_tags_11 },
    { 0x81a00cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7a00d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73a00dc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2e1a00de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6f5a00e0, SUC_lexicon_tags_11 },
    { 0x387a00e1, SUC_lexicon_tags_32 },
    { 0x64ba00e1, SUC_lexicon_tags_60 },
    { 0x791a00e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5a00e7, SUC_lexicon_tags_627 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743a00ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1a00f2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615a00f6, SUC_lexicon_tags_10 },
    { 0xb5a00f7, SUC_lexicon_tags_25 },
    { 0x1cda00f7, SUC_lexicon_tags_11 },
    { 0x461a00f9, SUC_lexicon_tags_11 },
    { 0x633a00fa, SUC_lexicon_tags_17 },
    { 0x539a00f9, SUC_lexicon_tags_9 },
    { 0x4c1a00fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dba00ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fa0104, SUC_lexicon_tags_17 },
    { 0xdeba0105, SUC_lexicon_tags_47 },
    { 0x46da0105, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371a010b, SUC_lexicon_tags_15 },
    { 0xe23a010c, SUC_lexicon_tags_33 },
    { 0x499a010d, SUC_lexicon_tags_11 },
    { 0x89ba010d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x805a0111, SUC_lexicon_tags_12 },
    { 0xd97a0112, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x53da0114, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb7da0116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63a011a, SUC_lexicon_tags_7 },
    { 0x8e7a011a, SUC_lexicon_tags_11 },
    { 0xabda011c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1a0127, SUC_lexicon_tags_37 },
    { 0x26fa0128, SUC_lexicon_tags_37 },
    { 0xf35a0129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda0137, SUC_lexicon_tags_39 },
    { 0xa7ba0137, SUC_lexicon_tags_38 },
    { 0x895a0138, SUC_lexicon_tags_7 },
    { 0xa6da0138, SUC_lexicon_tags_10 },
    { 0x5fa0139, SUC_lexicon_tags_15 },
    { 0xb63a013c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3a0143, SUC_lexicon_tags_60 },
    { 0xd31a0143, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1a0149, SUC_lexicon_tags_12 },
    { 0x309a0149, SUC_lexicon_tags_26 },
    { 0x497a014b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf9a014d, SUC_lexicon_tags_16 },
    { 0x71a014e, SUC_lexicon_tags_0 },
    { 0x637a014f, SUC_lexicon_tags_37 },
    { 0x8aba0150, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2a7a0152, SUC_lexicon_tags_7 },
    { 0x67a0153, SUC_lexicon_tags_26 },
    { 0xfe3a0154, SUC_lexicon_tags_7 },
    { 0x15da0155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93da0157, SUC_lexicon_tags_12 },
    { 0xfcfa0157, SUC_lexicon_tags_27 },
    { 0xb59a0159, SUC_lexicon_tags_8 },
    { 0xc6fa015a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eda015e, SUC_lexicon_tags_23 },
    { 0x373a015f, SUC_lexicon_tags_39 },
    { 0x1c9a015f, SUC_lexicon_tags_11 },
    { 0x493a0161, SUC_lexicon_tags_1 },
    { 0xa4ba0162, SUC_lexicon_tags_2 },
    { 0x457a0162, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8a9a0165, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9a0168, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x31fa016b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43a0171, SUC_lexicon_tags_8 },
    { 0xa77a0172, SUC_lexicon_tags_17 },
    { 0x755a0173, SUC_lexicon_tags_25 },
    { 0x31fa0172, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ba0177, SUC_lexicon_tags_26 },
    { 0xdeba0178, SUC_lexicon_tags_1 },
    { 0x55a0179, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3a017f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83a0182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1fa0184, SUC_lexicon_tags_14 },
    { 0xb49a0184, SUC_lexicon_tags_1 },
    { 0xa93a0186, SUC_lexicon_tags_1 },
    { 0xbcda0186, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa0191, SUC_lexicon_tags_12 },
    { 0x49a0192, SUC_lexicon_tags_21 },
    { 0x90ba0193, SUC_lexicon_tags_11 },
    { 0xe79a0192, SUC_lexicon_tags_34 },
    { 0x653a0193, SUC_lexicon_tags_146 },
    { 0x461a0192, SUC_lexicon_tags_1 },
    { 0x2d7a0193, SUC_lexicon_tags_49 },
    { 0xd43a0198, SUC_lexicon_tags_56 },
    { 0x7bda0192, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa71a019b, SUC_lexicon_tags_15 },
    { 0xf77a019b, SUC_lexicon_tags_8 },
    { 0x681a019d, SUC_lexicon_tags_65 },
    { 0x893a019d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757a01a4, SUC_lexicon_tags_11 },
    { 0xb1fa01a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45da01a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9afa01a9, SUC_lexicon_tags_16 },
    { 0xb01a01aa, SUC_lexicon_tags_1 },
    { 0x297a01ab, SUC_lexicon_tags_37 },
    { 0x59ba01ac, SUC_lexicon_tags_15 },
    { 0xa35a01ad, SUC_lexicon_tags_15 },
    { 0x809a01ac, SUC_lexicon_tags_129 },
    { 0x169a01ab, SUC_lexicon_tags_49 },
    { 0xab1a01af, SUC_lexicon_tags_46 },
    { 0xf45a01aa, SUC_lexicon_tags_17 },
    { 0x483a01ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa33a01b4, SUC_lexicon_tags_9 },
    { 0x101a01b4, SUC_lexicon_tags_29 },
    { 0x5a9a01b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdba01b9, SUC_lexicon_tags_15 },
    { 0x903a01ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63a01be, SUC_lexicon_tags_7 },
    { 0x6d1a01bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddda01c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa01c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4d9a01c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb15a01ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537a01d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffa01d6, SUC_lexicon_tags_14 },
    { 0x315a01d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x283a01d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x707a01dc, SUC_lexicon_tags_47 },
    { 0x447a01dd, SUC_lexicon_tags_37 },
    { 0x533a01dd, SUC_lexicon_tags_11 },
    { 0x745a01dc, SUC_lexicon_tags_2 },
    { 0x7bfa01e0, SUC_lexicon_tags_36 },
    { 0x969a01dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29a01e6, SUC_lexicon_tags_37 },
    { 0xa91a01e7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x70da01e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7a01f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdffa01f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa01fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661a01ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfa0206, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7a0209, SUC_lexicon_tags_14 },
    { 0x3a3a020a, SUC_lexicon_tags_29 },
    { 0x5cfa020a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ba0211, SUC_lexicon_tags_11 },
    { 0xfe5a0211, SUC_lexicon_tags_74 },
    { 0xf47a0213, SUC_lexicon_tags_60 },
    { 0x3dba0214, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6b9a0216, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145a021d, SUC_lexicon_tags_25 },
    { 0x533a021e, SUC_lexicon_tags_30 },
    { 0x52da021f, SUC_lexicon_tags_21 },
    { 0xb2fa0220, SUC_lexicon_tags_42 },
    { 0x567a021f, SUC_lexicon_tags_33 },
    { 0x383a0221, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1a0226, SUC_lexicon_tags_47 },
    { 0xd33a0227, SUC_lexicon_tags_0 },
    { 0x8ada0228, SUC_lexicon_tags_7 },
    { 0x931a0228, SUC_lexicon_tags_37 },
    { 0xcdda022a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57a022e, SUC_lexicon_tags_29 },
    { 0x127a022f, SUC_lexicon_tags_25 },
    { 0xa75a0230, SUC_lexicon_tags_5 },
    { 0xf9ba0230, SUC_lexicon_tags_31 },
    { 0x5fa0230, SUC_lexicon_tags_27 },
    { 0x9b3a0233, SUC_lexicon_tags_43 },
    { 0xff9a022f, SUC_lexicon_tags_107 },
    { 0x917a0235, SUC_lexicon_tags_28 },
    { 0x411a0236, SUC_lexicon_tags_28 },
    { 0x5ba0232, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585a023c, SUC_lexicon_tags_49 },
    { 0xa9a023d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x437a0240, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb47a0242, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9a024b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ba024e, SUC_lexicon_tags_14 },
    { 0xef3a024f, SUC_lexicon_tags_0 },
    { 0x2c1a0250, SUC_lexicon_tags_43 },
    { 0x747a0251, SUC_lexicon_tags_6 },
    { 0x817a0252, SUC_lexicon_tags_37 },
    { 0x839a0253, SUC_lexicon_tags_12 },
    { 0x687a0254, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x867a0256, SUC_lexicon_tags_11 },
    { 0x45fa0256, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fa025b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395a025f, SUC_lexicon_tags_37 },
    { 0x295a0260, SUC_lexicon_tags_48 },
    { 0xa7da0260, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ba0267, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ffa0269, SUC_lexicon_tags_37 },
    { 0xcffa0269, SUC_lexicon_tags_37 },
    { 0x65a026b, SUC_lexicon_tags_15 },
    { 0xd71a026c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33a0271, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3a7a0273, SUC_lexicon_tags_26 },
    { 0x77a0274, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc8ba0276, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7a027b, SUC_lexicon_tags_44 },
    { 0xc45a027c, SUC_lexicon_tags_10 },
    { 0xcd1a027c, SUC_lexicon_tags_37 },
    { 0x2dda027d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x421a0280, SUC_lexicon_tags_12 },
    { 0x947a0281, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9a0285, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383a0289, SUC_lexicon_tags_9 },
    { 0x609a0289, SUC_lexicon_tags_45 },
    { 0xf7fa028b, SUC_lexicon_tags_25 },
    { 0xabba028a, SUC_lexicon_tags_37 },
    { 0x2b9a028d, SUC_lexicon_tags_15 },
    { 0x105a028a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7a0291, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1a0295, SUC_lexicon_tags_46 },
    { 0x3d5a0296, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3a02a2, SUC_lexicon_tags_11 },
    { 0x843a02a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x583a02a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xebfa02a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a9a02a9, SUC_lexicon_tags_16 },
    { 0x2f7a02aa, SUC_lexicon_tags_33 },
    { 0xe03a02aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dba02af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ba02b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc1a02b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc71a02b6, SUC_lexicon_tags_39 },
    { 0xa2fa02b7, SUC_lexicon_tags_10 },
    { 0xedfa02b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9a02bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21da02c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd9a02c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417a02c6, SUC_lexicon_tags_14 },
    { 0x4bba02c7, SUC_lexicon_tags_65 },
    { 0xb91a02c8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb7ba02ca, SUC_lexicon_tags_26 },
    { 0xd5da02cb, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x5aba02cd, SUC_lexicon_tags_17 },
    { 0x417a02cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05a02d2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ba02d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9a02d8, SUC_lexicon_tags_11 },
    { 0xce5a02d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x605a02db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeba02df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa7da02e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda02e5, SUC_lexicon_tags_43 },
    { 0xa11a02e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x397a02e8, SUC_lexicon_tags_37 },
    { 0x6eda02e9, SUC_lexicon_tags_49 },
    { 0x459a02e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afa02ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777a02f3, SUC_lexicon_tags_60 },
    { 0xde9a02f4, SUC_lexicon_tags_37 },
    { 0x8ffa02f3, SUC_lexicon_tags_12 },
    { 0x51a02f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x26ba02f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ba02fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ba0301, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9dda0303, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x18fa0305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x783a0307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5a030a, SUC_lexicon_tags_65 },
    { 0xa1da030b, SUC_lexicon_tags_17 },
    { 0x30fa030c, SUC_lexicon_tags_16 },
    { 0xf19a030d, SUC_lexicon_tags_2 },
    { 0x445a030c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81a0310, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eda0317, SUC_lexicon_tags_15 },
    { 0x361a0317, SUC_lexicon_tags_27 },
    { 0x677a0319, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x84da031c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9a031f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a5a0321, SUC_lexicon_tags_25 },
    { 0x9bda0322, SUC_lexicon_tags_77 },
    { 0x3a7a0322, SUC_lexicon_tags_10 },
    { 0xbcfa0322, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1a0327, SUC_lexicon_tags_29 },
    { 0x16ba0328, SUC_lexicon_tags_23 },
    { 0x587a0328, SUC_lexicon_tags_17 },
    { 0x155a0329, SUC_lexicon_tags_81 },
    { 0xacba032b, SUC_lexicon_tags_12 },
    { 0x88ba032c, SUC_lexicon_tags_12 },
    { 0x745a032a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf07a032f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6da033a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13a0342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1a0345, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629a034a, SUC_lexicon_tags_14 },
    { 0x1ada034b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdeda034d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83a0352, SUC_lexicon_tags_0 },
    { 0xc91a0353, SUC_lexicon_tags_56 },
    { 0xc73a0354, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa37a0356, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7a0359, SUC_lexicon_tags_33 },
    { 0xf17a0359, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a035d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdf9a035f, SUC_lexicon_tags_19 },
    { 0x341a0360, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfa0366, SUC_lexicon_tags_19 },
    { 0x1c1a0367, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa41a0369, SUC_lexicon_tags_92 },
    { 0x7e9a036a, SUC_lexicon_tags_38 },
    { 0x217a036a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bda036d, SUC_lexicon_tags_7 },
    { 0xc6ba036e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x86da0371, SUC_lexicon_tags_23 },
    { 0x913a0371, SUC_lexicon_tags_37 },
    { 0xdc9a0373, SUC_lexicon_tags_45 },
    { 0x909a0374, SUC_lexicon_tags_38 },
    { 0x94ba0374, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91a037a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ba037c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x945a037e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903a0383, SUC_lexicon_tags_0 },
    { 0x9b9a0384, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x199a0386, SUC_lexicon_tags_25 },
    { 0x12da0387, SUC_lexicon_tags_11 },
    { 0xaefa0387, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323a038c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7da038e, SUC_lexicon_tags_15 },
    { 0x9d5a038e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x97da0391, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x873a0394, SUC_lexicon_tags_19 },
    { 0xfffa0394, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20da0399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9fa039b, SUC_lexicon_tags_37 },
    { 0x8c9a039b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ba03a1, SUC_lexicon_tags_25 },
    { 0x7da03a2, SUC_lexicon_tags_7 },
    { 0x14da03a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bba03a6, SUC_lexicon_tags_55 },
    { 0xcc3a03a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61a03ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9a03ae, SUC_lexicon_tags_1 },
    { 0xcb3a03af, SUC_lexicon_tags_7 },
    { 0x99ba03ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21da03b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5a03bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817a03c0, SUC_lexicon_tags_9 },
    { 0x247a03c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915a03d7, SUC_lexicon_tags_14 },
    { 0x757a03d8, SUC_lexicon_tags_68 },
    { 0xd91a03d8, SUC_lexicon_tags_25 },
    { 0xad3a03da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41a03dd, SUC_lexicon_tags_17 },
    { 0xf81a03de, SUC_lexicon_tags_28 },
    { 0x1ada03df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ba03e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x52fa03e4, SUC_lexicon_tags_63 },
    { 0xefba03e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7a03e9, SUC_lexicon_tags_33 },
    { 0x2aba03ea, SUC_lexicon_tags_194 },
    { 0xfba03e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97a03f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5a03f6, SUC_lexicon_tags_34 },
    { 0xe75a03f7, SUC_lexicon_tags_31 },
    { 0x2bda03f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x415a03fb, SUC_lexicon_tags_15 },
    { 0x607a03fc, SUC_lexicon_tags_17 },
    { 0xda9a03fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x365a0400, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x371a0403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95da0405, SUC_lexicon_tags_6 },
    { 0x7d3a0406, SUC_lexicon_tags_2 },
    { 0x1c3a0405, SUC_lexicon_tags_37 },
    { 0x8fda0408, SUC_lexicon_tags_48 },
    { 0x263a0409, SUC_lexicon_tags_37 },
    { 0x51da0409, SUC_lexicon_tags_12 },
    { 0x6fda0409, SUC_lexicon_tags_15 },
    { 0xa11a0407, SUC_lexicon_tags_15 },
    { 0xd4da0408, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a0411, SUC_lexicon_tags_7 },
    { 0xfcba0412, SUC_lexicon_tags_2 },
    { 0x385a0413, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb9a0415, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dda0419, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7a041e, SUC_lexicon_tags_17 },
    { 0x8cfa041f, SUC_lexicon_tags_0 },
    { 0xb7a0420, SUC_lexicon_tags_17 },
    { 0xc15a0420, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107a042f, SUC_lexicon_tags_19 },
    { 0x6eda042f, SUC_lexicon_tags_60 },
    { 0x545a0431, SUC_lexicon_tags_7 },
    { 0x871a0432, SUC_lexicon_tags_25 },
    { 0x905a0431, SUC_lexicon_tags_47 },
    { 0xab7a0433, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fa0437, SUC_lexicon_tags_48 },
    { 0x857a0438, SUC_lexicon_tags_15 },
    { 0xdbda0439, SUC_lexicon_tags_23 },
    { 0x82fa0439, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ba043d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fa0442, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fa0448, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd67a044a, SUC_lexicon_tags_8 },
    { 0x41fa044b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeda044e, SUC_lexicon_tags_38 },
    { 0x2fa044f, SUC_lexicon_tags_37 },
    { 0xd67a044f, SUC_lexicon_tags_12 },
    { 0x963a0451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99a0455, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cba0458, SUC_lexicon_tags_63 },
    { 0xb93a0458, SUC_lexicon_tags_15 },
    { 0x7f9a0459, SUC_lexicon_tags_15 },
    { 0xe83a045b, SUC_lexicon_tags_63 },
    { 0xbc7a045c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3a045f, SUC_lexicon_tags_48 },
    { 0xd91a045f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5a0465, SUC_lexicon_tags_28 },
    { 0x40ba0465, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8da046c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48da0474, SUC_lexicon_tags_77 },
    { 0x4e9a0475, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7a0479, SUC_lexicon_tags_38 },
    { 0x8a7a0479, SUC_lexicon_tags_60 },
    { 0x5e5a047b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409a0483, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45da0485, SUC_lexicon_tags_25 },
    { 0x4cfa0485, SUC_lexicon_tags_40 },
    { 0xde3a0487, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efa048b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739a048f, SUC_lexicon_tags_15 },
    { 0xc23a048f, SUC_lexicon_tags_65 },
    { 0xbd3a0491, SUC_lexicon_tags_17 },
    { 0x917a048f, SUC_lexicon_tags_131 },
    { 0x53fa0493, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45a049a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a04a2, SUC_lexicon_tags_63 },
    { 0xe2da04a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x68da04a5, SUC_lexicon_tags_49 },
    { 0xa5a04a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89a04aa, SUC_lexicon_tags_114 },
    { 0x6f3a04ab, SUC_lexicon_tags_31 },
    { 0x951a04ac, SUC_lexicon_tags_32 },
    { 0xbf9a04ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ba04b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957a04b4, SUC_lexicon_tags_66 },
    { 0x5cba04b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ba04b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3a04c0, SUC_lexicon_tags_49 },
    { 0x35da04c1, SUC_lexicon_tags_17 },
    { 0x7efa04c2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aba04c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eda04ca, SUC_lexicon_tags_7 },
    { 0x7e7a04cb, SUC_lexicon_tags_45 },
    { 0x709a04cb, SUC_lexicon_tags_7 },
    { 0xf45a04cb, SUC_lexicon_tags_11 },
    { 0xc1ba04cd, SUC_lexicon_tags_26 },
    { 0xeda04cf, SUC_lexicon_tags_9 },
    { 0xbda04cf, SUC_lexicon_tags_237 },
    { 0x64ba04cd, SUC_lexicon_tags_11 },
    { 0x459a04d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcfa04d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fa04da, SUC_lexicon_tags_38 },
    { 0x7fba04db, SUC_lexicon_tags_65 },
    { 0x3c1a04db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495a04e0, SUC_lexicon_tags_1 },
    { 0xd1fa04e1, SUC_lexicon_tags_31 },
    { 0x10da04e2, SUC_lexicon_tags_25 },
    { 0x931a04e3, SUC_lexicon_tags_9 },
    { 0x7b9a04e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x71da04e7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xee7a04e9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fa04ec, SUC_lexicon_tags_2 },
    { 0x2d3a04ed, SUC_lexicon_tags_65 },
    { 0x8b5a04ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9eda04f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97da04f5, SUC_lexicon_tags_47 },
    { 0x163a04f6, SUC_lexicon_tags_17 },
    { 0xcb1a04f6, SUC_lexicon_tags_17 },
    { 0x877a04f8, SUC_lexicon_tags_60 },
    { 0xf13a04f8, SUC_lexicon_tags_26 },
    { 0xb0fa04fa, SUC_lexicon_tags_34 },
    { 0x3ffa04f9, SUC_lexicon_tags_7 },
    { 0x193a04f5, SUC_lexicon_tags_17 },
    { 0xa09a04fd, SUC_lexicon_tags_31 },
    { 0x7bda04fe, SUC_lexicon_tags_11 },
    { 0x88da04ff, SUC_lexicon_tags_17 },
    { 0xf31a0500, SUC_lexicon_tags_0 },
    { 0x793a04fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ba0505, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa050b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d3a050d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x66ba050f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837a0514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6da051a, SUC_lexicon_tags_9 },
    { 0x3fba051a, SUC_lexicon_tags_12 },
    { 0x599a051c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd29a051e, SUC_lexicon_tags_39 },
    { 0x477a051e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x255a0521, SUC_lexicon_tags_45 },
    { 0x18fa0522, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23a052a, SUC_lexicon_tags_19 },
    { 0x5afa052a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffa0530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafba0535, SUC_lexicon_tags_17 },
    { 0x7d3a0536, SUC_lexicon_tags_27 },
    { 0xf73a0537, SUC_lexicon_tags_12 },
    { 0x357a0536, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x53da053b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x409a053d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe69a053f, SUC_lexicon_tags_12 },
    { 0x247a0540, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49a0544, SUC_lexicon_tags_30 },
    { 0x69ba0545, SUC_lexicon_tags_47 },
    { 0x57ba0544, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1bda0548, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97a054e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a0551, SUC_lexicon_tags_2 },
    { 0x5c1a0552, SUC_lexicon_tags_1 },
    { 0x315a0552, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x65ba0555, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc83a0557, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55a055c, SUC_lexicon_tags_25 },
    { 0xe7ba055d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x84da0560, SUC_lexicon_tags_22 },
    { 0x4e9a0561, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fa0564, SUC_lexicon_tags_63 },
    { 0xa89a0564, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdba056d, SUC_lexicon_tags_2 },
    { 0x60fa056e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57fa0570, SUC_lexicon_tags_12 },
    { 0x7a1a0571, SUC_lexicon_tags_14 },
    { 0x873a0572, SUC_lexicon_tags_17 },
    { 0xf4da0570, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xaa1a0575, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545a0584, SUC_lexicon_tags_35 },
    { 0x493a0585, SUC_lexicon_tags_23 },
    { 0x27fa0586, SUC_lexicon_tags_2 },
    { 0x15a0587, SUC_lexicon_tags_37 },
    { 0x3f7a0588, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ba058c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73a0590, SUC_lexicon_tags_11 },
    { 0x8bba0591, SUC_lexicon_tags_37 },
    { 0xed3a0590, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12da0595, SUC_lexicon_tags_1 },
    { 0x8f1a0595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6ba0598, SUC_lexicon_tags_32 },
    { 0xa65a0598, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfcfa059b, SUC_lexicon_tags_17 },
    { 0xa41a059c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x37da059e, SUC_lexicon_tags_34 },
    { 0xed5a059f, SUC_lexicon_tags_14 },
    { 0x65da059f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703a05b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x911a05b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba05bb, SUC_lexicon_tags_15 },
    { 0x275a05bc, SUC_lexicon_tags_11 },
    { 0x60da05bd, SUC_lexicon_tags_15 },
    { 0x95a05bd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1a05c5, SUC_lexicon_tags_47 },
    { 0x35a05c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5a05cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853a05d0, SUC_lexicon_tags_16 },
    { 0xad9a05d0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7a05d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x30da05db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x555a05dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4da05df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa05e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1a05e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dda05ea, SUC_lexicon_tags_37 },
    { 0x4a3a05eb, SUC_lexicon_tags_11 },
    { 0xf4fa05ec, SUC_lexicon_tags_38 },
    { 0xd07a05ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da05f4, SUC_lexicon_tags_11 },
    { 0x2a5a05f4, SUC_lexicon_tags_40 },
    { 0x5b1a05f6, SUC_lexicon_tags_1 },
    { 0xdbda05f4, SUC_lexicon_tags_15 },
    { 0xb17a05f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbfba05fa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707a0601, SUC_lexicon_tags_7 },
    { 0x837a0602, SUC_lexicon_tags_2 },
    { 0x98da0603, SUC_lexicon_tags_11 },
    { 0xe7a0604, SUC_lexicon_tags_10 },
    { 0x11ba0602, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fa0608, SUC_lexicon_tags_43 },
    { 0x163a0608, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa25a060b, SUC_lexicon_tags_0 },
    { 0xd41a060c, SUC_lexicon_tags_63 },
    { 0xb51a060c, SUC_lexicon_tags_56 },
    { 0xe5a060c, SUC_lexicon_tags_14 },
    { 0x691a060e, SUC_lexicon_tags_6 },
    { 0xa29a0610, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed7a0612, SUC_lexicon_tags_28 },
    { 0x497a0612, SUC_lexicon_tags_61 },
    { 0xc23a0614, SUC_lexicon_tags_43 },
    { 0xb5fa0612, SUC_lexicon_tags_25 },
    { 0x42ba0616, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeda061b, SUC_lexicon_tags_37 },
    { 0x7bda061c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57a0620, SUC_lexicon_tags_7 },
    { 0x5c3a0620, SUC_lexicon_tags_15 },
    { 0xfc3a0622, SUC_lexicon_tags_7 },
    { 0xdfda0623, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5a0627, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953a062b, SUC_lexicon_tags_30 },
    { 0xb9da062b, SUC_lexicon_tags_9 },
    { 0xc23a062d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153a0633, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731a0638, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd25a063a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7a063e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe51a0640, SUC_lexicon_tags_24 },
    { 0xe05a0641, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdbfa0643, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0fa0645, SUC_lexicon_tags_80 },
    { 0xdb5a0646, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x391a0648, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153a064e, SUC_lexicon_tags_11 },
    { 0xb81a064e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7a0652, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3a0655, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x97ba0657, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513a065c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177a0660, SUC_lexicon_tags_12 },
    { 0x239a0661, SUC_lexicon_tags_3 },
    { 0x763a0662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83a0665, SUC_lexicon_tags_6 },
    { 0xdf3a0665, SUC_lexicon_tags_12 },
    { 0xf89a0667, SUC_lexicon_tags_878 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453a0675, SUC_lexicon_tags_17 },
    { 0x62fa0675, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1da0678, SUC_lexicon_tags_25 },
    { 0x669a0678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fa067c, SUC_lexicon_tags_37 },
    { 0x291a067d, SUC_lexicon_tags_15 },
    { 0x223a067e, SUC_lexicon_tags_25 },
    { 0xd11a067f, SUC_lexicon_tags_38 },
    { 0x7b9a067d, SUC_lexicon_tags_17 },
    { 0x7ada0681, SUC_lexicon_tags_2 },
    { 0x9e1a0682, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47a0686, SUC_lexicon_tags_23 },
    { 0xe1ba0687, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdda068b, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0x467a068e, SUC_lexicon_tags_9 },
    { 0xd0ba068f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09a0695, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19a069a, SUC_lexicon_tags_37 },
    { 0xc4fa069b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xceda069d, SUC_lexicon_tags_27 },
    { 0xae5a069d, SUC_lexicon_tags_63 },
    { 0xb83a069d, SUC_lexicon_tags_757 },
    { 0x877a06a0, SUC_lexicon_tags_11 },
    { 0xf11a069e, SUC_lexicon_tags_17 },
    { 0x9fba06a1, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5a06a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3f1a06a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4d1a06aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8efa06ad, SUC_lexicon_tags_38 },
    { 0xcb9a06ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc3ba06b0, SUC_lexicon_tags_15 },
    { 0xbf9a06b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fa06b4, SUC_lexicon_tags_12 },
    { 0x8d1a06b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36da06ba, SUC_lexicon_tags_7 },
    { 0xa9fa06ba, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x577a06bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba06c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2fa06c2, SUC_lexicon_tags_17 },
    { 0xcd3a06c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3a06c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4dba06c8, SUC_lexicon_tags_47 },
    { 0x127a06c8, SUC_lexicon_tags_2 },
    { 0x7bda06ca, SUC_lexicon_tags_43 },
    { 0x29da06c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdda06cd, SUC_lexicon_tags_26 },
    { 0xf27a06ce, SUC_lexicon_tags_1 },
    { 0xd01a06ce, SUC_lexicon_tags_25 },
    { 0xb9a06cd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5a06d8, SUC_lexicon_tags_30 },
    { 0xe93a06d9, SUC_lexicon_tags_65 },
    { 0x941a06d8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x16da06dc, SUC_lexicon_tags_28 },
    { 0x1dfa06dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x517a06df, SUC_lexicon_tags_18 },
    { 0xb79a06e0, SUC_lexicon_tags_38 },
    { 0xcfa06e0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931a06e7, SUC_lexicon_tags_28 },
    { 0xf71a06e7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a06eb, SUC_lexicon_tags_27 },
    { 0x3a5a06eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf1a06ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fa06f1, SUC_lexicon_tags_17 },
    { 0xec9a06f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4eba06f4, SUC_lexicon_tags_25 },
    { 0x633a06f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6afa06f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965a06fd, SUC_lexicon_tags_29 },
    { 0xbd3a06fd, SUC_lexicon_tags_43 },
    { 0xef9a06ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x885a0701, SUC_lexicon_tags_4 },
    { 0xb79a0702, SUC_lexicon_tags_44 },
    { 0x97ba0702, SUC_lexicon_tags_60 },
    { 0x2aba0701, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb7a0706, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3a070d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455a0711, SUC_lexicon_tags_1 },
    { 0xb85a0711, SUC_lexicon_tags_49 },
    { 0x17da0712, SUC_lexicon_tags_26 },
    { 0x429a0714, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a0717, SUC_lexicon_tags_12 },
    { 0x361a0718, SUC_lexicon_tags_19 },
    { 0xdafa0719, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabda071c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1a0722, SUC_lexicon_tags_47 },
    { 0xc3ba0723, SUC_lexicon_tags_7 },
    { 0xf5fa0724, SUC_lexicon_tags_17 },
    { 0xacba0724, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc29a0727, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x13a0729, SUC_lexicon_tags_7 },
    { 0xf47a072a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9a072d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfda0732, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x291a0734, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eba073b, SUC_lexicon_tags_25 },
    { 0xfe5a073c, SUC_lexicon_tags_54 },
    { 0xcc3a073d, SUC_lexicon_tags_1 },
    { 0xc35a073e, SUC_lexicon_tags_25 },
    { 0x409a073e, SUC_lexicon_tags_30 },
    { 0x1f1a073c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1a0745, SUC_lexicon_tags_0 },
    { 0x36ba0746, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53a0749, SUC_lexicon_tags_1 },
    { 0x8a1a074a, SUC_lexicon_tags_55 },
    { 0x159a074b, SUC_lexicon_tags_32 },
    { 0x467a074b, SUC_lexicon_tags_25 },
    { 0xd69a074d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x81a074f, SUC_lexicon_tags_1 },
    { 0xab7a0750, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x547a0752, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb21a0754, SUC_lexicon_tags_37 },
    { 0xb35a0755, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x717a0758, SUC_lexicon_tags_25 },
    { 0x367a0759, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1a075c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fba075e, SUC_lexicon_tags_21 },
    { 0xa07a075f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73a0762, SUC_lexicon_tags_23 },
    { 0xd59a0762, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3a0766, SUC_lexicon_tags_35 },
    { 0x67a0766, SUC_lexicon_tags_23 },
    { 0xbba0766, SUC_lexicon_tags_16 },
    { 0x4bba0767, SUC_lexicon_tags_2 },
    { 0xa39a076a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x411a076d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x967a076f, SUC_lexicon_tags_39 },
    { 0xbffa0770, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65a0776, SUC_lexicon_tags_11 },
    { 0x4c7a0776, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117a077d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9a0780, SUC_lexicon_tags_7 },
    { 0x33da0781, SUC_lexicon_tags_23 },
    { 0xdada0780, SUC_lexicon_tags_1 },
    { 0x7d9a0783, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x845a0785, SUC_lexicon_tags_37 },
    { 0x279a0785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bba0789, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1a0794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9a0797, SUC_lexicon_tags_17 },
    { 0x987a0798, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x511a079b, SUC_lexicon_tags_72 },
    { 0x61fa079b, SUC_lexicon_tags_9 },
    { 0x947a079c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x949a079f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fa07a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x151a07a6, SUC_lexicon_tags_47 },
    { 0x701a07a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2ba07a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xebba07ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389a07b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0fa07b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bba07b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137a07bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc5a07bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ba07c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba1a07c6, SUC_lexicon_tags_39 },
    { 0xb5ba07c7, SUC_lexicon_tags_14 },
    { 0xb69a07c7, SUC_lexicon_tags_14 },
    { 0x31da07c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa07d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fa07d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33a07dc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xec9a07de, SUC_lexicon_tags_8 },
    { 0xf57a07df, SUC_lexicon_tags_68 },
    { 0x58ba07e0, SUC_lexicon_tags_3 },
    { 0x3b1a07e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ba07e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3e3a07e6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9a07e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1a07f1, SUC_lexicon_tags_39 },
    { 0x127a07f2, SUC_lexicon_tags_61 },
    { 0xe51a07f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a07f8, SUC_lexicon_tags_46 },
    { 0x3b3a07f9, SUC_lexicon_tags_1 },
    { 0x263a07f9, SUC_lexicon_tags_17 },
    { 0x3efa07fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x445a07fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x999a0800, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27da0803, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa0809, SUC_lexicon_tags_30 },
    { 0x40fa0809, SUC_lexicon_tags_37 },
    { 0xf95a080b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4eda080d, SUC_lexicon_tags_0 },
    { 0x663a080d, SUC_lexicon_tags_27 },
    { 0xab5a080e, SUC_lexicon_tags_25 },
    { 0x303a0810, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x119a0813, SUC_lexicon_tags_37 },
    { 0x4aba0814, SUC_lexicon_tags_10 },
    { 0xe0ba0815, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cda0818, SUC_lexicon_tags_100 },
    { 0x235a0818, SUC_lexicon_tags_37 },
    { 0x4bba0819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bda081c, SUC_lexicon_tags_12 },
    { 0xc9da081c, SUC_lexicon_tags_53 },
    { 0x453a081c, SUC_lexicon_tags_15 },
    { 0xa1ba081c, SUC_lexicon_tags_15 },
    { 0xac9a0820, SUC_lexicon_tags_65 },
    { 0xe95a0821, SUC_lexicon_tags_47 },
    { 0xef9a081d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aba0826, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53a082b, SUC_lexicon_tags_1 },
    { 0x35a082b, SUC_lexicon_tags_8 },
    { 0x86da082c, SUC_lexicon_tags_19 },
    { 0xf5fa082e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x705a0831, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721a0838, SUC_lexicon_tags_15 },
    { 0x219a0839, SUC_lexicon_tags_11 },
    { 0x7e5a0839, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacda0842, SUC_lexicon_tags_47 },
    { 0x5cda0842, SUC_lexicon_tags_68 },
    { 0x93a0844, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273a0849, SUC_lexicon_tags_47 },
    { 0x5b1a084a, SUC_lexicon_tags_12 },
    { 0x2d7a084a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77a084f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733a0853, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ba085a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x877a085d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0da0861, SUC_lexicon_tags_56 },
    { 0xfb9a0861, SUC_lexicon_tags_17 },
    { 0xaf3a0863, SUC_lexicon_tags_11 },
    { 0xf45a0864, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55a0867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fa086b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33a086e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ffa0870, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x345a0873, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcda0876, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fa0884, SUC_lexicon_tags_25 },
    { 0x6ffa0885, SUC_lexicon_tags_2 },
    { 0xb03a0886, SUC_lexicon_tags_60 },
    { 0x88fa0887, SUC_lexicon_tags_17 },
    { 0x22da0886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab7a088a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9a088f, SUC_lexicon_tags_1102 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1a0893, SUC_lexicon_tags_37 },
    { 0xc6fa0893, SUC_lexicon_tags_1 },
    { 0x635a0893, SUC_lexicon_tags_17 },
    { 0xc23a0896, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8d3a0898, SUC_lexicon_tags_7 },
    { 0xdf1a0899, SUC_lexicon_tags_17 },
    { 0x13a089a, SUC_lexicon_tags_25 },
    { 0xadba089b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793a089f, SUC_lexicon_tags_1 },
    { 0xb35a08a0, SUC_lexicon_tags_17 },
    { 0x1e7a08a0, SUC_lexicon_tags_43 },
    { 0x247a08a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x791a08a4, SUC_lexicon_tags_14 },
    { 0xeb3a08a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad7a08a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80ba08aa, SUC_lexicon_tags_7 },
    { 0x2e3a08aa, SUC_lexicon_tags_11 },
    { 0x9a7a08ac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91da08b1, SUC_lexicon_tags_1 },
    { 0xeeda08b2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5a08b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1a08be, SUC_lexicon_tags_27 },
    { 0x9c1a08be, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ba08c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa08ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fa08ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb81a08d0, SUC_lexicon_tags_33 },
    { 0xb73a08d0, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa71a08d3, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xa3ba08d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da08da, SUC_lexicon_tags_35 },
    { 0xfcfa08db, SUC_lexicon_tags_60 },
    { 0xd55a08dc, SUC_lexicon_tags_87 },
    { 0x54fa08dd, SUC_lexicon_tags_25 },
    { 0x6dfa08db, SUC_lexicon_tags_11 },
    { 0x4a3a08dc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3cda08e1, SUC_lexicon_tags_25 },
    { 0xdbfa08e2, SUC_lexicon_tags_45 },
    { 0x417a08e1, SUC_lexicon_tags_30 },
    { 0xb61a08e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19a08eb, SUC_lexicon_tags_14 },
    { 0x5f3a08eb, SUC_lexicon_tags_46 },
    { 0x2efa08ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dba08f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdf5a08f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x421a08f5, SUC_lexicon_tags_25 },
    { 0xdf3a08f6, SUC_lexicon_tags_17 },
    { 0xbe5a08f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7a0901, SUC_lexicon_tags_43 },
    { 0x83da0902, SUC_lexicon_tags_17 },
    { 0x9c1a0903, SUC_lexicon_tags_17 },
    { 0xa7ba0904, SUC_lexicon_tags_8 },
    { 0xa67a0903, SUC_lexicon_tags_27 },
    { 0x637a0906, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ba090a, SUC_lexicon_tags_6 },
    { 0x407a090b, SUC_lexicon_tags_12 },
    { 0xba1a090b, SUC_lexicon_tags_97 },
    { 0x221a090b, SUC_lexicon_tags_4 },
    { 0x629a090e, SUC_lexicon_tags_60 },
    { 0x46da090a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53a0914, SUC_lexicon_tags_30 },
    { 0xc39a0915, SUC_lexicon_tags_39 },
    { 0x33da0915, SUC_lexicon_tags_49 },
    { 0xc7fa0917, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd15a0919, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa091d, SUC_lexicon_tags_27 },
    { 0x725a091d, SUC_lexicon_tags_7 },
    { 0x4afa091f, SUC_lexicon_tags_1101 },
    { 0xff9a091f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xecfa0922, SUC_lexicon_tags_17 },
    { 0x36fa0923, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65a0926, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5a0929, SUC_lexicon_tags_23 },
    { 0x617a092a, SUC_lexicon_tags_25 },
    { 0xf91a092b, SUC_lexicon_tags_49 },
    { 0x32da092a, SUC_lexicon_tags_32 },
    { 0x177a092b, SUC_lexicon_tags_25 },
    { 0x55fa092e, SUC_lexicon_tags_11 },
    { 0x517a092e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13a0935, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbe7a0937, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933a093c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd39a093e, SUC_lexicon_tags_19 },
    { 0x89fa093e, SUC_lexicon_tags_17 },
    { 0xc11a0940, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xff1a0942, SUC_lexicon_tags_11 },
    { 0x3f1a0943, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15a0947, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdc3a0949, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547a0954, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3a095e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fa0964, SUC_lexicon_tags_68 },
    { 0xbdda0965, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9bba0967, SUC_lexicon_tags_19 },
    { 0x32da0968, SUC_lexicon_tags_2 },
    { 0x897a0968, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41a0974, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf45a0976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bda097d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73a0983, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1dfa0985, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ba0989, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fa098d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263a0991, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7f7a0993, SUC_lexicon_tags_19 },
    { 0x3c9a0993, SUC_lexicon_tags_262 },
    { 0x877a0993, SUC_lexicon_tags_37 },
    { 0xc19a0996, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe81a0998, SUC_lexicon_tags_27 },
    { 0x999a0999, SUC_lexicon_tags_26 },
    { 0x423a0998, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcba099d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3a09a7, SUC_lexicon_tags_7 },
    { 0x165a09a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7a09ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeba09ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97fa09b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x621a09b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afa09b9, SUC_lexicon_tags_12 },
    { 0x823a09ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x965a09bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fa09c7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1a09ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e5a09d0, SUC_lexicon_tags_1 },
    { 0x4e9a09d1, SUC_lexicon_tags_25 },
    { 0x39da09d2, SUC_lexicon_tags_25 },
    { 0xf61a09d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9a09d7, SUC_lexicon_tags_55 },
    { 0xdada09d8, SUC_lexicon_tags_113 },
    { 0x9a09d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfa09df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67a09e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895a09eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dda09ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5a09f3, SUC_lexicon_tags_7 },
    { 0x571a09f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31a09f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73a0a04, SUC_lexicon_tags_43 },
    { 0xe1ba0a04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x231a0a07, SUC_lexicon_tags_15 },
    { 0x8f7a0a08, SUC_lexicon_tags_11 },
    { 0xc63a0a09, SUC_lexicon_tags_26 },
    { 0x89fa0a0a, SUC_lexicon_tags_47 },
    { 0x8d1a0a0b, SUC_lexicon_tags_84 },
    { 0x6dda0a0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x675a0a0f, SUC_lexicon_tags_12 },
    { 0x811a0a10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a0a13, SUC_lexicon_tags_87 },
    { 0x461a0a14, SUC_lexicon_tags_231 },
    { 0xec9a0a14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9a0a18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ba0a1c, SUC_lexicon_tags_2 },
    { 0x919a0a1c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7a0a22, SUC_lexicon_tags_33 },
    { 0x5a7a0a22, SUC_lexicon_tags_17 },
    { 0xe35a0a22, SUC_lexicon_tags_7 },
    { 0x515a0a25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8dda0a27, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb19a0a29, SUC_lexicon_tags_28 },
    { 0xb49a0a2a, SUC_lexicon_tags_27 },
    { 0x82ba0a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bda0a2f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ba0a34, SUC_lexicon_tags_33 },
    { 0xb85a0a35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145a0a3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa29a0a3c, SUC_lexicon_tags_12 },
    { 0xfdfa0a3d, SUC_lexicon_tags_17 },
    { 0x9a5a0a3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfa0a47, SUC_lexicon_tags_17 },
    { 0x30da0a47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x225a0a4b, SUC_lexicon_tags_14 },
    { 0xde7a0a4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ba0a4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7a0a52, SUC_lexicon_tags_136 },
    { 0xf73a0a52, SUC_lexicon_tags_27 },
    { 0xfc7a0a54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7a0a5e, SUC_lexicon_tags_45 },
    { 0x487a0a5f, SUC_lexicon_tags_2 },
    { 0x6d9a0a60, SUC_lexicon_tags_43 },
    { 0x67da0a60, SUC_lexicon_tags_37 },
    { 0xd71a0a62, SUC_lexicon_tags_1 },
    { 0xc9a0a62, SUC_lexicon_tags_5 },
    { 0x381a0a63, SUC_lexicon_tags_12 },
    { 0x3d3a0a65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fba0a68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeda0a6b, SUC_lexicon_tags_32 },
    { 0x10da0a6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9a0a75, SUC_lexicon_tags_65 },
    { 0xcda0a75, SUC_lexicon_tags_32 },
    { 0x2c7a0a75, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefda0a7d, SUC_lexicon_tags_1 },
    { 0xe61a0a7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20ba0a80, SUC_lexicon_tags_63 },
    { 0x897a0a80, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1a0a84, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33a0a88, SUC_lexicon_tags_1464 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73a0a8d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x945a0a8f, SUC_lexicon_tags_25 },
    { 0x551a0a8f, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0x6d1a0a92, SUC_lexicon_tags_19 },
    { 0xac5a0a93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xedda0a96, SUC_lexicon_tags_15 },
    { 0x1b1a0a96, SUC_lexicon_tags_1 },
    { 0xf9ba0a98, SUC_lexicon_tags_11 },
    { 0x611a0a99, SUC_lexicon_tags_14 },
    { 0x837a0a99, SUC_lexicon_tags_37 },
    { 0x1c7a0a9a, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0xe7fa0a9d, SUC_lexicon_tags_38 },
    { 0x6bda0a9d, SUC_lexicon_tags_65 },
    { 0x80ba0a9f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fa0aa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa21a0aa5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33a0aa9, SUC_lexicon_tags_29 },
    { 0xeeda0aa9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbcda0aac, SUC_lexicon_tags_0 },
    { 0x935a0aad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ba0ab0, SUC_lexicon_tags_23 },
    { 0x623a0ab0, SUC_lexicon_tags_63 },
    { 0x87ba0ab2, SUC_lexicon_tags_12 },
    { 0xe23a0ab3, SUC_lexicon_tags_64 },
    { 0x943a0ab4, SUC_lexicon_tags_27 },
    { 0xc3da0ab3, SUC_lexicon_tags_60 },
    { 0xa65a0ab1, SUC_lexicon_tags_43 },
    { 0x49a0ab7, SUC_lexicon_tags_65 },
    { 0xbbba0ab8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fa0ac4, SUC_lexicon_tags_2 },
    { 0x867a0ac5, SUC_lexicon_tags_49 },
    { 0x757a0ac5, SUC_lexicon_tags_17 },
    { 0xc4da0ac7, SUC_lexicon_tags_49 },
    { 0x7a1a0ac8, SUC_lexicon_tags_1 },
    { 0x92fa0ac9, SUC_lexicon_tags_8 },
    { 0x679a0aca, SUC_lexicon_tags_84 },
    { 0xc8ba0acb, SUC_lexicon_tags_28 },
    { 0xd89a0ac8, SUC_lexicon_tags_1 },
    { 0x51fa0ac8, SUC_lexicon_tags_15 },
    { 0xc63a0ace, SUC_lexicon_tags_43 },
    { 0x52da0acb, SUC_lexicon_tags_56 },
    { 0x78da0ad0, SUC_lexicon_tags_31 },
    { 0xe2da0acc, SUC_lexicon_tags_33 },
    { 0x50ba0acb, SUC_lexicon_tags_15 },
    { 0xf33a0ad3, SUC_lexicon_tags_0 },
    { 0xca9a0ad3, SUC_lexicon_tags_68 },
    { 0x3efa0acb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3da0ada, SUC_lexicon_tags_25 },
    { 0x569a0adb, SUC_lexicon_tags_8 },
    { 0xa0ba0adc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cba0ae0, SUC_lexicon_tags_49 },
    { 0xce3a0ae1, SUC_lexicon_tags_38 },
    { 0x475a0ae2, SUC_lexicon_tags_0 },
    { 0xf73a0ae0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97da0ae7, SUC_lexicon_tags_38 },
    { 0xd03a0ae8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9a0aee, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xee5a0af0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205a0af4, SUC_lexicon_tags_0 },
    { 0x5b1a0af5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401a0af9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe05a0afb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x743a0afe, SUC_lexicon_tags_12 },
    { 0x8f5a0aff, SUC_lexicon_tags_23 },
    { 0x38da0b00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9a0b06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc5a0b08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe93a0b0a, SUC_lexicon_tags_19 },
    { 0x3d5a0b0b, SUC_lexicon_tags_8 },
    { 0x641a0b0c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x89ba0b0e, SUC_lexicon_tags_1 },
    { 0x9b7a0b0f, SUC_lexicon_tags_60 },
    { 0xf55a0b10, SUC_lexicon_tags_77 },
    { 0xf35a0b11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x479a0b13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xde3a0b15, SUC_lexicon_tags_11 },
    { 0xbbda0b16, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493a0b1b, SUC_lexicon_tags_47 },
    { 0xb21a0b1c, SUC_lexicon_tags_0 },
    { 0xca7a0b1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3a0b1f, SUC_lexicon_tags_12 },
    { 0x60ba0b20, SUC_lexicon_tags_63 },
    { 0x51da0b20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ba0b2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7a0b2f, SUC_lexicon_tags_25 },
    { 0x14ba0b2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3a0b33, SUC_lexicon_tags_2 },
    { 0x261a0b33, SUC_lexicon_tags_26 },
    { 0xaaba0b35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x129a0b37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819a0b3d, SUC_lexicon_tags_1 },
    { 0x709a0b3e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x207a0b40, SUC_lexicon_tags_6 },
    { 0xc3ba0b41, SUC_lexicon_tags_14 },
    { 0x703a0b41, SUC_lexicon_tags_33 },
    { 0x2e5a0b40, SUC_lexicon_tags_25 },
    { 0x2c1a0b44, SUC_lexicon_tags_17 },
    { 0xc31a0b45, SUC_lexicon_tags_25 },
    { 0xac9a0b46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x425a0b48, SUC_lexicon_tags_27 },
    { 0x29ba0b49, SUC_lexicon_tags_11 },
    { 0x89a0b4a, SUC_lexicon_tags_17 },
    { 0xe9da0b4b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e9a0b4d, SUC_lexicon_tags_7 },
    { 0x8f9a0b4e, SUC_lexicon_tags_15 },
    { 0x769a0b4f, SUC_lexicon_tags_17 },
    { 0x15da0b50, SUC_lexicon_tags_14 },
    { 0xffa0b51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba0b54, SUC_lexicon_tags_37 },
    { 0xbc7a0b54, SUC_lexicon_tags_46 },
    { 0xa33a0b55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817a0b5a, SUC_lexicon_tags_46 },
    { 0x68ba0b5b, SUC_lexicon_tags_23 },
    { 0xa17a0b5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc57a0b5e, SUC_lexicon_tags_4 },
    { 0x1bda0b5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd77a0b61, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fa0b64, SUC_lexicon_tags_37 },
    { 0x9dda0b65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbada0b68, SUC_lexicon_tags_12 },
    { 0x983a0b69, SUC_lexicon_tags_12 },
    { 0x7bba0b68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5a0b6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fa0b73, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x84fa0b75, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeba0b7a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecba0b7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485a0b88, SUC_lexicon_tags_0 },
    { 0x4ada0b89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x21a0b8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337a0b90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x533a0b92, SUC_lexicon_tags_17 },
    { 0xd57a0b93, SUC_lexicon_tags_17 },
    { 0x2e9a0b94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xda0b97, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55a0b9a, SUC_lexicon_tags_33 },
    { 0x363a0b9b, SUC_lexicon_tags_15 },
    { 0x9f3a0b9c, SUC_lexicon_tags_37 },
    { 0xc4ba0b9b, SUC_lexicon_tags_38 },
    { 0x101a0b9e, SUC_lexicon_tags_37 },
    { 0x425a0b9c, SUC_lexicon_tags_37 },
    { 0xdc7a0ba0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d3a0ba2, SUC_lexicon_tags_25 },
    { 0x97ba0ba3, SUC_lexicon_tags_55 },
    { 0x19fa0ba3, SUC_lexicon_tags_541 },
    { 0x271a0ba5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fa0ba9, SUC_lexicon_tags_21 },
    { 0x53ba0baa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5a0bae, SUC_lexicon_tags_16 },
    { 0x3eba0baf, SUC_lexicon_tags_1 },
    { 0x283a0baf, SUC_lexicon_tags_28 },
    { 0x2b7a0baf, SUC_lexicon_tags_19 },
    { 0x419a0bb2, SUC_lexicon_tags_88 },
    { 0xe51a0bae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853a0bbe, SUC_lexicon_tags_19 },
    { 0xc37a0bbf, SUC_lexicon_tags_2 },
    { 0xceda0bc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bda0bc3, SUC_lexicon_tags_82 },
    { 0x289a0bc4, SUC_lexicon_tags_37 },
    { 0x7fa0bc3, SUC_lexicon_tags_19 },
    { 0xcd7a0bc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341a0bcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x783a0bce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b3a0bd0, SUC_lexicon_tags_1 },
    { 0xd91a0bd0, SUC_lexicon_tags_47 },
    { 0x3e5a0bd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x697a0bd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca1a0bd6, SUC_lexicon_tags_4 },
    { 0x619a0bd7, SUC_lexicon_tags_25 },
    { 0xc15a0bd6, SUC_lexicon_tags_15 },
    { 0x389a0bd7, SUC_lexicon_tags_12 },
    { 0xdd7a0bd6, SUC_lexicon_tags_31 },
    { 0x27fa0bd9, SUC_lexicon_tags_37 },
    { 0x65ba0bda, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9a0bdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931a0be3, SUC_lexicon_tags_11 },
    { 0x397a0be3, SUC_lexicon_tags_17 },
    { 0x761a0be3, SUC_lexicon_tags_12 },
    { 0x9c9a0be6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513a0beb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105a0bef, SUC_lexicon_tags_6 },
    { 0xb25a0bf0, SUC_lexicon_tags_2 },
    { 0xfb7a0bf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8da0bf3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x285a0bf6, SUC_lexicon_tags_7 },
    { 0xc1ba0bf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x853a0bf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7a0bfc, SUC_lexicon_tags_10 },
    { 0x94fa0bfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb7a0bff, SUC_lexicon_tags_21 },
    { 0x551a0c00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9a0c05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795a0c0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x427a0c0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1a0c10, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fba0c13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5a0c19, SUC_lexicon_tags_12 },
    { 0x4ba0c19, SUC_lexicon_tags_9 },
    { 0xe75a0c19, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3a0c20, SUC_lexicon_tags_17 },
    { 0x82ba0c20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1a0c26, SUC_lexicon_tags_37 },
    { 0xe0da0c26, SUC_lexicon_tags_15 },
    { 0x943a0c27, SUC_lexicon_tags_48 },
    { 0x50ba0c29, SUC_lexicon_tags_25 },
    { 0x785a0c26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x993a0c2d, SUC_lexicon_tags_2 },
    { 0xa17a0c2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x271a0c30, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa9fa0c32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47ba0c35, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x90da0c38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa89a0c3a, SUC_lexicon_tags_64 },
    { 0x91da0c3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fa0c3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x725a0c42, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeba0c45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fa0c4d, SUC_lexicon_tags_6 },
    { 0xd4da0c4e, SUC_lexicon_tags_25 },
    { 0x235a0c4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b1a0c51, SUC_lexicon_tags_11 },
    { 0x925a0c52, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917a0c56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x297a0c58, SUC_lexicon_tags_74 },
    { 0xc6ba0c59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3a0c5c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ba0c61, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc37a0c63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93a0c67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x805a0c6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x771a0c6c, SUC_lexicon_tags_7 },
    { 0x957a0c6d, SUC_lexicon_tags_3 },
    { 0x757a0c6e, SUC_lexicon_tags_24 },
    { 0x59ba0c6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba0c75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3a0c79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1da0c7d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9a1a0c7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa87a0c81, SUC_lexicon_tags_7 },
    { 0x455a0c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205a0c86, SUC_lexicon_tags_63 },
    { 0xa27a0c86, SUC_lexicon_tags_37 },
    { 0xa95a0c88, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595a0c8c, SUC_lexicon_tags_104 },
    { 0xd13a0c8d, SUC_lexicon_tags_33 },
    { 0x723a0c8d, SUC_lexicon_tags_60 },
    { 0xc9fa0c8d, SUC_lexicon_tags_15 },
    { 0x2c7a0c90, SUC_lexicon_tags_2 },
    { 0xfdfa0c8f, SUC_lexicon_tags_28 },
    { 0x26ba0c92, SUC_lexicon_tags_12 },
    { 0xfa7a0c92, SUC_lexicon_tags_11 },
    { 0x865a0c8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7a0c98, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8aba0c9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fa0c9f, SUC_lexicon_tags_37 },
    { 0xb0da0ca0, SUC_lexicon_tags_68 },
    { 0x65fa0ca0, SUC_lexicon_tags_37 },
    { 0xabda0c9f, SUC_lexicon_tags_25 },
    { 0x3ba0c9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9c5a0ca5, SUC_lexicon_tags_72 },
    { 0xe87a0ca6, SUC_lexicon_tags_17 },
    { 0xc59a0ca7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5a0cad, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a0cb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x727a0cb2, SUC_lexicon_tags_0 },
    { 0x611a0cb3, SUC_lexicon_tags_23 },
    { 0xda1a0cb4, SUC_lexicon_tags_14 },
    { 0xaa9a0cb3, SUC_lexicon_tags_9 },
    { 0x519a0cb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61da0cb8, SUC_lexicon_tags_12 },
    { 0xedda0cb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a0cbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81a0cc3, SUC_lexicon_tags_63 },
    { 0x363a0cc4, SUC_lexicon_tags_0 },
    { 0xb67a0cc4, SUC_lexicon_tags_1 },
    { 0x931a0cc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fa0cc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x621a0ccb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfa0cd0, SUC_lexicon_tags_15 },
    { 0x2fba0cd1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4da0cd6, SUC_lexicon_tags_14 },
    { 0x783a0cd7, SUC_lexicon_tags_19 },
    { 0xfe9a0cd8, SUC_lexicon_tags_25 },
    { 0x309a0cd9, SUC_lexicon_tags_15 },
    { 0xc1ba0cda, SUC_lexicon_tags_24 },
    { 0xe8da0cdb, SUC_lexicon_tags_63 },
    { 0x359a0cdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fa0cdf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1a0ce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07a0ce5, SUC_lexicon_tags_11 },
    { 0xe7fa0ce5, SUC_lexicon_tags_2 },
    { 0x361a0ce5, SUC_lexicon_tags_7 },
    { 0xcc7a0ce8, SUC_lexicon_tags_45 },
    { 0x403a0ce8, SUC_lexicon_tags_14 },
    { 0x519a0cea, SUC_lexicon_tags_1 },
    { 0xbb1a0ceb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x653a0cee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcf5a0cf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x469a0cf3, SUC_lexicon_tags_7 },
    { 0x8a1a0cf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d1a0cf6, SUC_lexicon_tags_26 },
    { 0x3a3a0cf6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd63a0cf9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87a0cfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dda0cff, SUC_lexicon_tags_42 },
    { 0x1a0d00, SUC_lexicon_tags_7 },
    { 0xc99a0d01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeeda0d03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf53a0d05, SUC_lexicon_tags_34 },
    { 0xb8ba0d06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x617a0d09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59fa0d0b, SUC_lexicon_tags_12 },
    { 0x357a0d0c, SUC_lexicon_tags_59 },
    { 0x553a0d0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7a0d11, SUC_lexicon_tags_37 },
    { 0xb19a0d12, SUC_lexicon_tags_108 },
    { 0xc87a0d13, SUC_lexicon_tags_19 },
    { 0x4da0d12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737a0d19, SUC_lexicon_tags_70 },
    { 0xe29a0d1a, SUC_lexicon_tags_12 },
    { 0xc3ba0d1a, SUC_lexicon_tags_43 },
    { 0xe91a0d1c, SUC_lexicon_tags_61 },
    { 0x33a0d1b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x853a0d20, SUC_lexicon_tags_74 },
    { 0xe75a0d20, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xdf7a0d23, SUC_lexicon_tags_11 },
    { 0x79fa0d24, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fa0d27, SUC_lexicon_tags_25 },
    { 0x60fa0d28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05a0d2b, SUC_lexicon_tags_8 },
    { 0x81fa0d2c, SUC_lexicon_tags_25 },
    { 0xb2da0d2d, SUC_lexicon_tags_11 },
    { 0x381a0d2c, SUC_lexicon_tags_11 },
    { 0x479a0d2e, SUC_lexicon_tags_53 },
    { 0x2a3a0d30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11ba0d32, SUC_lexicon_tags_25 },
    { 0xc17a0d32, SUC_lexicon_tags_12 },
    { 0x7fa0d34, SUC_lexicon_tags_37 },
    { 0xb93a0d35, SUC_lexicon_tags_2 },
    { 0x171a0d36, SUC_lexicon_tags_19 },
    { 0x9ffa0d35, SUC_lexicon_tags_262 },
    { 0x921a0d33, SUC_lexicon_tags_37 },
    { 0xd45a0d35, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf05a0d3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x321a0d3d, SUC_lexicon_tags_0 },
    { 0xa75a0d3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfa0d41, SUC_lexicon_tags_25 },
    { 0x357a0d41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5a0d46, SUC_lexicon_tags_9 },
    { 0xf53a0d47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2fa0d49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51da0d4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9a0d4f, SUC_lexicon_tags_7 },
    { 0x8d3a0d50, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x551a0d52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x429a0d55, SUC_lexicon_tags_38 },
    { 0xf3ba0d55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaba0d59, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9a0d5e, SUC_lexicon_tags_0 },
    { 0x1dba0d5f, SUC_lexicon_tags_81 },
    { 0x8c5a0d60, SUC_lexicon_tags_25 },
    { 0x7ada0d61, SUC_lexicon_tags_11 },
    { 0x81da0d62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17a0d65, SUC_lexicon_tags_14 },
    { 0xeb7a0d66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe79a0d68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x10ba0d6a, SUC_lexicon_tags_39 },
    { 0xef5a0d6b, SUC_lexicon_tags_25 },
    { 0x76ba0d6a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa3a0d6e, SUC_lexicon_tags_11 },
    { 0x7a7a0d6e, SUC_lexicon_tags_25 },
    { 0x2ba0d6e, SUC_lexicon_tags_9 },
    { 0xdd3a0d71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf9a0d73, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbdba0d75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5a0d79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7a0d81, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9a0d85, SUC_lexicon_tags_12 },
    { 0x6afa0d86, SUC_lexicon_tags_14 },
    { 0xe61a0d86, SUC_lexicon_tags_19 },
    { 0x15fa0d87, SUC_lexicon_tags_2 },
    { 0x1fa0d89, SUC_lexicon_tags_32 },
    { 0x7bba0d8a, SUC_lexicon_tags_19 },
    { 0x859a0d8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ba0d8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a0d96, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6dba0d98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7da0d9c, SUC_lexicon_tags_4 },
    { 0xa75a0d9d, SUC_lexicon_tags_27 },
    { 0xc8da0d9e, SUC_lexicon_tags_14 },
    { 0x6dda0d9f, SUC_lexicon_tags_2 },
    { 0x979a0d9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38da0da2, SUC_lexicon_tags_17 },
    { 0x4f1a0da2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x553a0da5, SUC_lexicon_tags_7 },
    { 0xf07a0da6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dda0dab, SUC_lexicon_tags_28 },
    { 0xae5a0dab, SUC_lexicon_tags_33 },
    { 0x3e7a0dad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1a0db0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf77a0db2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd3a0db4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x935a0db6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141a0dba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab5a0dbc, SUC_lexicon_tags_25 },
    { 0x4d3a0dbd, SUC_lexicon_tags_57 },
    { 0xbaba0dbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x257a0dc1, SUC_lexicon_tags_23 },
    { 0xcf7a0dc2, SUC_lexicon_tags_1 },
    { 0x5dda0dc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x339a0dc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfada0dc9, SUC_lexicon_tags_37 },
    { 0x71fa0dc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1a0dcf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61da0dd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x941a0dd6, SUC_lexicon_tags_24 },
    { 0x51ba0dd7, SUC_lexicon_tags_83 },
    { 0xd27a0dd8, SUC_lexicon_tags_23 },
    { 0xf9a0dd8, SUC_lexicon_tags_25 },
    { 0x8b5a0dd9, SUC_lexicon_tags_31 },
    { 0xdf1a0dd6, SUC_lexicon_tags_1 },
    { 0x469a0ddc, SUC_lexicon_tags_82 },
    { 0x9ba0ddd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf75a0ddf, SUC_lexicon_tags_11 },
    { 0x949a0de0, SUC_lexicon_tags_37 },
    { 0x6b5a0ddf, SUC_lexicon_tags_106 },
    { 0x6ada0de2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379a0deb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195a0df1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49a0df6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3a0dfb, SUC_lexicon_tags_49 },
    { 0x557a0dfc, SUC_lexicon_tags_45 },
    { 0x7d3a0dfc, SUC_lexicon_tags_15 },
    { 0x681a0dfc, SUC_lexicon_tags_9 },
    { 0xe11a0dff, SUC_lexicon_tags_23 },
    { 0xaf7a0dff, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1a0e03, SUC_lexicon_tags_8 },
    { 0xd8fa0e04, SUC_lexicon_tags_7 },
    { 0xa9da0e05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a7a0e07, SUC_lexicon_tags_15 },
    { 0x77da0e08, SUC_lexicon_tags_1 },
    { 0xa31a0e09, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x40ba0e0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7a0e0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aba0e13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13a0e16, SUC_lexicon_tags_2 },
    { 0x34fa0e16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67a0e1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a0e22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fa0e26, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x153a0e28, SUC_lexicon_tags_63 },
    { 0x1f7a0e29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7a0e2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x993a0e2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x229a0e32, SUC_lexicon_tags_15 },
    { 0x655a0e33, SUC_lexicon_tags_25 },
    { 0x9a1a0e33, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35a0e37, SUC_lexicon_tags_13 },
    { 0xdd7a0e38, SUC_lexicon_tags_8 },
    { 0x179a0e38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a0e3f, SUC_lexicon_tags_83 },
    { 0xe8da0e40, SUC_lexicon_tags_37 },
    { 0x319a0e41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x353a0e43, SUC_lexicon_tags_11 },
    { 0xbc9a0e43, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x913a0e46, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759a0e4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3a0e53, SUC_lexicon_tags_14 },
    { 0xdcfa0e54, SUC_lexicon_tags_28 },
    { 0xf9da0e54, SUC_lexicon_tags_2 },
    { 0x523a0e56, SUC_lexicon_tags_33 },
    { 0x29da0e54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ba0e5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ba0e5f, SUC_lexicon_tags_1631 },
    { 0xfa3a0e60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadba0e65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15a0e6d, SUC_lexicon_tags_37 },
    { 0xcbda0e6e, SUC_lexicon_tags_77 },
    { 0xa13a0e6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63da0e71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81a0e77, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x295a0e7a, SUC_lexicon_tags_7 },
    { 0xf0fa0e7b, SUC_lexicon_tags_8 },
    { 0x91da0e7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9da0e83, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09a0e86, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21a0e8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefa0e91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66da0e96, SUC_lexicon_tags_26 },
    { 0xc95a0e97, SUC_lexicon_tags_11 },
    { 0xa89a0e96, SUC_lexicon_tags_47 },
    { 0x1cda0e97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dba0e9d, SUC_lexicon_tags_17 },
    { 0x13a0e9e, SUC_lexicon_tags_45 },
    { 0x88fa0e9e, SUC_lexicon_tags_39 },
    { 0xc8ba0ea0, SUC_lexicon_tags_27 },
    { 0xf49a0ea1, SUC_lexicon_tags_7 },
    { 0x887a0e9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9d7a0ea4, SUC_lexicon_tags_8 },
    { 0xed3a0ea5, SUC_lexicon_tags_17 },
    { 0x10fa0ea5, SUC_lexicon_tags_37 },
    { 0x807a0ea6, SUC_lexicon_tags_26 },
    { 0xcbda0ea8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd03a0eaa, SUC_lexicon_tags_15 },
    { 0xdb9a0eab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74da0eaf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3a0eb2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3a0eb6, SUC_lexicon_tags_9 },
    { 0x9ba0eb7, SUC_lexicon_tags_12 },
    { 0x467a0eb7, SUC_lexicon_tags_70 },
    { 0x42fa0eb6, SUC_lexicon_tags_14 },
    { 0x571a0eb6, SUC_lexicon_tags_10 },
    { 0xcafa0ebb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa51a0ebd, SUC_lexicon_tags_45 },
    { 0x817a0ebd, SUC_lexicon_tags_2 },
    { 0x257a0ebd, SUC_lexicon_tags_106 },
    { 0xdaba0ec0, SUC_lexicon_tags_0 },
    { 0x525a0ec0, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821a0ec9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fa0ed0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71da0ed2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15fa0ed4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaaba0ed6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa33a0ed8, SUC_lexicon_tags_49 },
    { 0xec1a0ed8, SUC_lexicon_tags_8 },
    { 0x591a0ed9, SUC_lexicon_tags_38 },
    { 0x9f3a0ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ba0ede, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a0ee5, SUC_lexicon_tags_27 },
    { 0x803a0ee5, SUC_lexicon_tags_13 },
    { 0x313a0ee6, SUC_lexicon_tags_15 },
    { 0xb4ba0ee7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5a0eee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ba0ef2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5a0ef5, SUC_lexicon_tags_60 },
    { 0xe8ba0ef5, SUC_lexicon_tags_42 },
    { 0x207a0ef6, SUC_lexicon_tags_37 },
    { 0xea3a0ef8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ba0efb, SUC_lexicon_tags_25 },
    { 0x7a5a0efc, SUC_lexicon_tags_48 },
    { 0x841a0efb, SUC_lexicon_tags_1 },
    { 0xa13a0efe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf4da0f00, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0xadfa0f02, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fa0f07, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3da0f12, SUC_lexicon_tags_2 },
    { 0xca9a0f12, SUC_lexicon_tags_17 },
    { 0x909a0f12, SUC_lexicon_tags_28 },
    { 0x8cda0f15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfa0f1b, SUC_lexicon_tags_35 },
    { 0xd17a0f1c, SUC_lexicon_tags_27 },
    { 0x3fda0f1d, SUC_lexicon_tags_54 },
    { 0x9f3a0f1b, SUC_lexicon_tags_121 },
    { 0xc43a0f1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3a0f2f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9a0f33, SUC_lexicon_tags_37 },
    { 0x861a0f34, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27ba0f36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x285a0f38, SUC_lexicon_tags_37 },
    { 0x911a0f38, SUC_lexicon_tags_11 },
    { 0xe07a0f3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fa0f43, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x98da0f45, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73a0f49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdc7a0f4b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9a0f4f, SUC_lexicon_tags_17 },
    { 0xe8fa0f4f, SUC_lexicon_tags_91 },
    { 0x93fa0f51, SUC_lexicon_tags_46 },
    { 0xf45a0f52, SUC_lexicon_tags_12 },
    { 0xd39a0f51, SUC_lexicon_tags_19 },
    { 0x491a0f54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90ba0f56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a1a0f58, SUC_lexicon_tags_11 },
    { 0xb8ba0f59, SUC_lexicon_tags_23 },
    { 0x247a0f5a, SUC_lexicon_tags_37 },
    { 0x12da0f5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fa0f61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5a0f68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ba0f6e, SUC_lexicon_tags_37 },
    { 0x851a0f6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5a0f73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x189a0f76, SUC_lexicon_tags_11 },
    { 0xa71a0f76, SUC_lexicon_tags_15 },
    { 0x605a0f77, SUC_lexicon_tags_6 },
    { 0xa43a0f77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569a0f80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x59a0f82, SUC_lexicon_tags_15 },
    { 0xacfa0f82, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfa0f86, SUC_lexicon_tags_5 },
    { 0xdd5a0f87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba0f91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda0f94, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ba0f9c, SUC_lexicon_tags_8 },
    { 0x979a0f9c, SUC_lexicon_tags_7 },
    { 0xd7ba0f9e, SUC_lexicon_tags_2 },
    { 0xa59a0f9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x869a0fa2, SUC_lexicon_tags_17 },
    { 0x9d3a0fa3, SUC_lexicon_tags_84 },
    { 0x79da0fa2, SUC_lexicon_tags_19 },
    { 0x239a0fa4, SUC_lexicon_tags_61 },
    { 0x58ba0fa6, SUC_lexicon_tags_2 },
    { 0x5b3a0fa7, SUC_lexicon_tags_30 },
    { 0xde5a0fa8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455a0fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89a0faf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79fa0fb1, SUC_lexicon_tags_17 },
    { 0x883a0fb1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x955a0fb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0da0fba, SUC_lexicon_tags_25 },
    { 0x82ba0fbb, SUC_lexicon_tags_30 },
    { 0xe3ba0fbc, SUC_lexicon_tags_11 },
    { 0xfe5a0fbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fa0fc2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x23ba0fc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fa0fc8, SUC_lexicon_tags_38 },
    { 0xa33a0fc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c3a0fcb, SUC_lexicon_tags_1 },
    { 0xb4da0fcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767a0fd0, SUC_lexicon_tags_1 },
    { 0x4eda0fd0, SUC_lexicon_tags_15 },
    { 0xa3fa0fd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22fa0fd4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561a0fd8, SUC_lexicon_tags_15 },
    { 0xc89a0fd9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x815a0fdb, SUC_lexicon_tags_84 },
    { 0xf27a0fdc, SUC_lexicon_tags_0 },
    { 0xd1a0fdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a7a0fdf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75a0fe9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfa0fed, SUC_lexicon_tags_25 },
    { 0xda3a0fee, SUC_lexicon_tags_1 },
    { 0x1fda0fef, SUC_lexicon_tags_25 },
    { 0xba7a0fef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7aba0ff2, SUC_lexicon_tags_14 },
    { 0xe1ba0ff2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5bba0ff5, SUC_lexicon_tags_75 },
    { 0x4cfa0ff5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56ba0ff8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59a0ffb, SUC_lexicon_tags_60 },
    { 0xc4fa0ffc, SUC_lexicon_tags_23 },
    { 0x663a0ffd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279a1001, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69a1004, SUC_lexicon_tags_2 },
    { 0x603a1004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a100d, SUC_lexicon_tags_952 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85a1010, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cba1013, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf77a1015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeafa1018, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ba101c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x863a101e, SUC_lexicon_tags_2 },
    { 0x5d3a101f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ba1022, SUC_lexicon_tags_49 },
    { 0xf77a1022, SUC_lexicon_tags_12 },
    { 0x6eda1024, SUC_lexicon_tags_11 },
    { 0xfc7a1023, SUC_lexicon_tags_23 },
    { 0x901a1022, SUC_lexicon_tags_37 },
    { 0xf41a1024, SUC_lexicon_tags_1 },
    { 0x6c9a1028, SUC_lexicon_tags_17 },
    { 0xcd1a1024, SUC_lexicon_tags_37 },
    { 0x481a1027, SUC_lexicon_tags_25 },
    { 0x611a102a, SUC_lexicon_tags_27 },
    { 0x573a102c, SUC_lexicon_tags_60 },
    { 0xd5ba1029, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579a1039, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13a103e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7a1041, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b9a1043, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05a1048, SUC_lexicon_tags_7 },
    { 0x13da1049, SUC_lexicon_tags_11 },
    { 0xa1ba104a, SUC_lexicon_tags_63 },
    { 0x82fa104b, SUC_lexicon_tags_37 },
    { 0xeaba104c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35a1050, SUC_lexicon_tags_25 },
    { 0x9afa1050, SUC_lexicon_tags_23 },
    { 0x29da1051, SUC_lexicon_tags_25 },
    { 0x89da1053, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151a1058, SUC_lexicon_tags_55 },
    { 0xe81a1059, SUC_lexicon_tags_38 },
    { 0x1cba105a, SUC_lexicon_tags_17 },
    { 0xf05a1058, SUC_lexicon_tags_37 },
    { 0xd37a105b, SUC_lexicon_tags_31 },
    { 0x603a1059, SUC_lexicon_tags_19 },
    { 0x3d1a105e, SUC_lexicon_tags_82 },
    { 0x30da105e, SUC_lexicon_tags_0 },
    { 0x1cda1059, SUC_lexicon_tags_47 },
    { 0xed9a1058, SUC_lexicon_tags_43 },
    { 0x447a1062, SUC_lexicon_tags_15 },
    { 0xa71a1059, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x781a1066, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5a106a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efa106d, SUC_lexicon_tags_0 },
    { 0xdbfa106e, SUC_lexicon_tags_25 },
    { 0x80ba106e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfa1073, SUC_lexicon_tags_0 },
    { 0xfafa1074, SUC_lexicon_tags_12 },
    { 0xbfa1073, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ba1078, SUC_lexicon_tags_23 },
    { 0x46da1079, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfa107e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1a1082, SUC_lexicon_tags_1 },
    { 0x82da1082, SUC_lexicon_tags_15 },
    { 0xe85a1082, SUC_lexicon_tags_12 },
    { 0x755a1085, SUC_lexicon_tags_37 },
    { 0xbcda1084, SUC_lexicon_tags_19 },
    { 0x83ba1082, SUC_lexicon_tags_14 },
    { 0xe89a1088, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cda1090, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe19a1092, SUC_lexicon_tags_8 },
    { 0x265a1092, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1a1097, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x689a109a, SUC_lexicon_tags_31 },
    { 0xe33a109b, SUC_lexicon_tags_27 },
    { 0x22da109b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7a9a109e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3a10a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5a10a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xec3a10a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73a10ab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883a10b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875a10b6, SUC_lexicon_tags_11 },
    { 0xbbfa10b6, SUC_lexicon_tags_17 },
    { 0xe47a10b8, SUC_lexicon_tags_27 },
    { 0x93ba10b9, SUC_lexicon_tags_25 },
    { 0xcc1a10ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6ba10bc, SUC_lexicon_tags_17 },
    { 0x63fa10bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fa10c2, SUC_lexicon_tags_37 },
    { 0x953a10c3, SUC_lexicon_tags_16 },
    { 0x6d5a10c3, SUC_lexicon_tags_31 },
    { 0x515a10c2, SUC_lexicon_tags_12 },
    { 0xe9da10c4, SUC_lexicon_tags_38 },
    { 0x8afa10c7, SUC_lexicon_tags_11 },
    { 0xfdfa10c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563a10cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fa10cf, SUC_lexicon_tags_10 },
    { 0x717a10d0, SUC_lexicon_tags_17 },
    { 0x40da10d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121a10d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x723a10d8, SUC_lexicon_tags_8 },
    { 0x87ba10d9, SUC_lexicon_tags_11 },
    { 0x17fa10d9, SUC_lexicon_tags_37 },
    { 0xdf3a10da, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x5c7a10dd, SUC_lexicon_tags_57 },
    { 0x85fa10de, SUC_lexicon_tags_66 },
    { 0x2a5a10df, SUC_lexicon_tags_25 },
    { 0x795a10e0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75a10e4, SUC_lexicon_tags_60 },
    { 0x567a10e5, SUC_lexicon_tags_12 },
    { 0x20fa10e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3a10e9, SUC_lexicon_tags_39 },
    { 0xe63a10e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa23a10ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751a10f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83da10f4, SUC_lexicon_tags_7 },
    { 0xaa3a10f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833a10fa, SUC_lexicon_tags_19 },
    { 0x21da10fa, SUC_lexicon_tags_0 },
    { 0x4b1a10fc, SUC_lexicon_tags_4 },
    { 0x41da10fd, SUC_lexicon_tags_7 },
    { 0xf87a10fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499a1103, SUC_lexicon_tags_10 },
    { 0x7e9a1103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f1a1106, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fa110a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x301a110d, SUC_lexicon_tags_12 },
    { 0x555a110e, SUC_lexicon_tags_1 },
    { 0x4c5a110f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x295a1111, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77a1115, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c5a1117, SUC_lexicon_tags_6 },
    { 0x303a1118, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdba111c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4c9a111e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fda1122, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b5a1124, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x337a1126, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7a1129, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7a112d, SUC_lexicon_tags_11 },
    { 0x503a112d, SUC_lexicon_tags_37 },
    { 0x5eda112d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ba1133, SUC_lexicon_tags_11 },
    { 0xce9a1133, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdd9a1136, SUC_lexicon_tags_11 },
    { 0x7c7a1137, SUC_lexicon_tags_27 },
    { 0x4c7a1138, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235a113c, SUC_lexicon_tags_87 },
    { 0x1cda113c, SUC_lexicon_tags_98 },
    { 0x235a113c, SUC_lexicon_tags_2 },
    { 0xf6fa113c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x535a1141, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3da1148, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xab1a114a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe75a114c, SUC_lexicon_tags_46 },
    { 0x2d1a114d, SUC_lexicon_tags_34 },
    { 0x9c3a114e, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a1153, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd81a1155, SUC_lexicon_tags_45 },
    { 0x1dba1155, SUC_lexicon_tags_1 },
    { 0x667a1157, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ba115a, SUC_lexicon_tags_9 },
    { 0x54da115b, SUC_lexicon_tags_25 },
    { 0xa09a115a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73a116c, SUC_lexicon_tags_15 },
    { 0x249a116c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9da1170, SUC_lexicon_tags_44 },
    { 0x555a1171, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e9a1173, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73a1177, SUC_lexicon_tags_11 },
    { 0xd31a1178, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfa117c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1a1184, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315a1188, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fa118d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1b9a118f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531a1197, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a119b, SUC_lexicon_tags_25 },
    { 0xc81a119b, SUC_lexicon_tags_11 },
    { 0xd9da119b, SUC_lexicon_tags_15 },
    { 0x307a119b, SUC_lexicon_tags_15 },
    { 0x989a119b, SUC_lexicon_tags_37 },
    { 0x6d7a119f, SUC_lexicon_tags_39 },
    { 0x9d9a11a1, SUC_lexicon_tags_6 },
    { 0xc0ba11a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38da11b2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419a11b6, SUC_lexicon_tags_49 },
    { 0x9c1a11b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603a11bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfba11be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x88ba11c0, SUC_lexicon_tags_1 },
    { 0xad7a11c0, SUC_lexicon_tags_25 },
    { 0x8b1a11c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x13ba11c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731a11c8, SUC_lexicon_tags_17 },
    { 0x963a11c9, SUC_lexicon_tags_10 },
    { 0x6a5a11c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ba11d1, SUC_lexicon_tags_17 },
    { 0x9b9a11d2, SUC_lexicon_tags_27 },
    { 0x203a11d2, SUC_lexicon_tags_55 },
    { 0x2ffa11d3, SUC_lexicon_tags_14 },
    { 0x57fa11d5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xab1a11d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x505a11d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ba11de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x75da11e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fa11e8, SUC_lexicon_tags_2 },
    { 0xe33a11e9, SUC_lexicon_tags_37 },
    { 0x749a11e9, SUC_lexicon_tags_4 },
    { 0x83fa11e9, SUC_lexicon_tags_37 },
    { 0x6b9a11ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc57a11ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37a11f5, SUC_lexicon_tags_11 },
    { 0x547a11f6, SUC_lexicon_tags_13 },
    { 0xa17a11f7, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eda11fa, SUC_lexicon_tags_37 },
    { 0x2ffa11fb, SUC_lexicon_tags_1 },
    { 0x53fa11fc, SUC_lexicon_tags_11 },
    { 0xe13a11fd, SUC_lexicon_tags_11 },
    { 0xde7a11fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x359a1200, SUC_lexicon_tags_0 },
    { 0x5cda1201, SUC_lexicon_tags_43 },
    { 0x503a1202, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x563a1205, SUC_lexicon_tags_30 },
    { 0x347a1206, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eba120c, SUC_lexicon_tags_37 },
    { 0x61a120d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73a1212, SUC_lexicon_tags_140 },
    { 0x8ada1212, SUC_lexicon_tags_84 },
    { 0x483a1214, SUC_lexicon_tags_12 },
    { 0xa3da1215, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4fa1217, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2a9a1219, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da121d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37a1223, SUC_lexicon_tags_63 },
    { 0xb25a1224, SUC_lexicon_tags_3 },
    { 0xd97a1225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5a122c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x31a122f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1a1233, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781a123d, SUC_lexicon_tags_7 },
    { 0xc5ba123e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffa1241, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749a1247, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x815a1249, SUC_lexicon_tags_17 },
    { 0x74ba124a, SUC_lexicon_tags_12 },
    { 0x903a124b, SUC_lexicon_tags_6 },
    { 0x293a124a, SUC_lexicon_tags_60 },
    { 0x73a124d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6b7a124f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83a1252, SUC_lexicon_tags_7 },
    { 0xf3a1252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef5a1255, SUC_lexicon_tags_14 },
    { 0xb33a1256, SUC_lexicon_tags_1 },
    { 0x153a1256, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97a125a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xddda125c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x931a125e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x66ba1260, SUC_lexicon_tags_49 },
    { 0x2cfa1261, SUC_lexicon_tags_41 },
    { 0x139a1262, SUC_lexicon_tags_25 },
    { 0xfd7a1263, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81a1269, SUC_lexicon_tags_4 },
    { 0xe1fa126a, SUC_lexicon_tags_46 },
    { 0x49fa126b, SUC_lexicon_tags_9 },
    { 0xd3fa126c, SUC_lexicon_tags_11 },
    { 0x827a126c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5a1270, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903a1275, SUC_lexicon_tags_47 },
    { 0xba1a1276, SUC_lexicon_tags_1 },
    { 0x24ba1277, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7c5a1279, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93a1281, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc61a1283, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5a5a1285, SUC_lexicon_tags_7 },
    { 0xd4da1286, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161a128a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61a128f, SUC_lexicon_tags_33 },
    { 0xd9da1290, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aba1295, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837a1299, SUC_lexicon_tags_19 },
    { 0x4fba129a, SUC_lexicon_tags_19 },
    { 0xe09a129a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x219a129d, SUC_lexicon_tags_11 },
    { 0x759a129e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afa12b4, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25a12b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fa12bd, SUC_lexicon_tags_0 },
    { 0xfb7a12be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9a12c4, SUC_lexicon_tags_2 },
    { 0xa1da12c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7a12ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efa12d2, SUC_lexicon_tags_77 },
    { 0xb9a12d3, SUC_lexicon_tags_17 },
    { 0xdf9a12d4, SUC_lexicon_tags_35 },
    { 0xa61a12d4, SUC_lexicon_tags_9 },
    { 0xfe9a12d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87a12db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ba12e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb9a12e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x903a12e4, SUC_lexicon_tags_8 },
    { 0xa69a12e5, SUC_lexicon_tags_15 },
    { 0x831a12e6, SUC_lexicon_tags_16 },
    { 0x8bfa12e4, SUC_lexicon_tags_26 },
    { 0xd25a12e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3a12ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe15a12f0, SUC_lexicon_tags_25 },
    { 0x3ba12f1, SUC_lexicon_tags_25 },
    { 0x9eba12f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa81a12f4, SUC_lexicon_tags_17 },
    { 0xb07a12f5, SUC_lexicon_tags_37 },
    { 0x743a12f6, SUC_lexicon_tags_17 },
    { 0x457a12f7, SUC_lexicon_tags_31 },
    { 0x44ba12f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95da12fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7a1302, SUC_lexicon_tags_37 },
    { 0x129a1303, SUC_lexicon_tags_37 },
    { 0xa21a1303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12da1309, SUC_lexicon_tags_25 },
    { 0xa59a1309, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a1318, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x26fa131a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9da131c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcdba131e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd25a1320, SUC_lexicon_tags_47 },
    { 0x73da1321, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1a1324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa41a1326, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3eba132b, SUC_lexicon_tags_38 },
    { 0xa5a132c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x22da132e, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433a1334, SUC_lexicon_tags_1 },
    { 0xa15a1334, SUC_lexicon_tags_7 },
    { 0x2f7a1336, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9a133b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x605a133d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x23a133f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb9a1341, SUC_lexicon_tags_17 },
    { 0x169a1342, SUC_lexicon_tags_34 },
    { 0xf5da1343, SUC_lexicon_tags_22 },
    { 0xa5fa1343, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3a1347, SUC_lexicon_tags_25 },
    { 0x1f1a1348, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78ba134c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73a134f, SUC_lexicon_tags_49 },
    { 0xd8da1350, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f5a1352, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fa1356, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x21da1359, SUC_lexicon_tags_15 },
    { 0x3f7a135a, SUC_lexicon_tags_1 },
    { 0xfcba135b, SUC_lexicon_tags_15 },
    { 0x7ada135c, SUC_lexicon_tags_27 },
    { 0xa69a135a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ba1366, SUC_lexicon_tags_59 },
    { 0x621a1366, SUC_lexicon_tags_11 },
    { 0xbd7a1368, SUC_lexicon_tags_25 },
    { 0x487a1366, SUC_lexicon_tags_14 },
    { 0x2e3a136a, SUC_lexicon_tags_37 },
    { 0x837a1367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5a1370, SUC_lexicon_tags_46 },
    { 0x49fa1371, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05a1375, SUC_lexicon_tags_151 },
    { 0xe1a1376, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x78da1378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x489a137a, SUC_lexicon_tags_25 },
    { 0x48ba137b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165a1380, SUC_lexicon_tags_13 },
    { 0x9b3a1380, SUC_lexicon_tags_66 },
    { 0x531a1382, SUC_lexicon_tags_63 },
    { 0x375a1382, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d3a1385, SUC_lexicon_tags_28 },
    { 0xecfa1385, SUC_lexicon_tags_18 },
    { 0x499a1386, SUC_lexicon_tags_77 },
    { 0x4eda1388, SUC_lexicon_tags_25 },
    { 0xb9fa1388, SUC_lexicon_tags_38 },
    { 0xfbda138a, SUC_lexicon_tags_32 },
    { 0x7bfa138b, SUC_lexicon_tags_37 },
    { 0x9dda138c, SUC_lexicon_tags_12 },
    { 0x3fda138c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ba1391, SUC_lexicon_tags_79 },
    { 0x985a1391, SUC_lexicon_tags_41 },
    { 0xa6ba1393, SUC_lexicon_tags_25 },
    { 0xd9da1393, SUC_lexicon_tags_1 },
    { 0x491a1393, SUC_lexicon_tags_1 },
    { 0x169a1396, SUC_lexicon_tags_56 },
    { 0xbeba1394, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe47a1399, SUC_lexicon_tags_15 },
    { 0xe59a139a, SUC_lexicon_tags_27 },
    { 0xda139a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfa139e, SUC_lexicon_tags_17 },
    { 0xb7da139f, SUC_lexicon_tags_12 },
    { 0x4f3a13a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37a13a5, SUC_lexicon_tags_4 },
    { 0x8fa13a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x819a13a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19a13ab, SUC_lexicon_tags_38 },
    { 0x905a13ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429a13b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269a13bc, SUC_lexicon_tags_8 },
    { 0x26da13bd, SUC_lexicon_tags_46 },
    { 0x72fa13be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5a13c1, SUC_lexicon_tags_25 },
    { 0x50fa13c2, SUC_lexicon_tags_26 },
    { 0x53ba13c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62da13c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29a13cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17ba13ce, SUC_lexicon_tags_19 },
    { 0x403a13cf, SUC_lexicon_tags_25 },
    { 0x221a13ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1ba13d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fda13d4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77da13de, SUC_lexicon_tags_17 },
    { 0xe15a13de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d5a13e1, SUC_lexicon_tags_17 },
    { 0x6d3a13e2, SUC_lexicon_tags_47 },
    { 0x517a13e3, SUC_lexicon_tags_45 },
    { 0x3afa13e2, SUC_lexicon_tags_37 },
    { 0xa87a13e2, SUC_lexicon_tags_31 },
    { 0x15a13e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ba13ea, SUC_lexicon_tags_85 },
    { 0x965a13eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793a13ef, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99da13f5, SUC_lexicon_tags_11 },
    { 0x403a13f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ba13fd, SUC_lexicon_tags_9 },
    { 0x405a13fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfa1401, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bba1404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cda1406, SUC_lexicon_tags_17 },
    { 0xba1406, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x199a1409, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fa140d, SUC_lexicon_tags_4 },
    { 0xd69a140d, SUC_lexicon_tags_15 },
    { 0x885a140f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355a1413, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafa1417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7a141e, SUC_lexicon_tags_2 },
    { 0x619a141f, SUC_lexicon_tags_36 },
    { 0x7bfa1420, SUC_lexicon_tags_17 },
    { 0x88ba1420, SUC_lexicon_tags_60 },
    { 0xec7a141e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9a1426, SUC_lexicon_tags_17 },
    { 0xf4fa1427, SUC_lexicon_tags_17 },
    { 0xee9a1428, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb51a142a, SUC_lexicon_tags_60 },
    { 0x3ffa142a, SUC_lexicon_tags_11 },
    { 0x1a3a142a, SUC_lexicon_tags_39 },
    { 0xfa1a142c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1a1431, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb9ba1433, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x479a1436, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x11da1438, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3a143b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6efa143d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ba1441, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993a1447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd77a1449, SUC_lexicon_tags_7 },
    { 0x6e7a144a, SUC_lexicon_tags_33 },
    { 0x135a144b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eda1450, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x687a1452, SUC_lexicon_tags_17 },
    { 0x34da1452, SUC_lexicon_tags_11 },
    { 0x5fba1454, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ba145f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x641a1461, SUC_lexicon_tags_37 },
    { 0x593a1462, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7bda1464, SUC_lexicon_tags_9 },
    { 0xaf7a1465, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5a1468, SUC_lexicon_tags_2 },
    { 0x637a1469, SUC_lexicon_tags_25 },
    { 0xd5da146a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcda146d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ba1471, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd0fa1473, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25a1476, SUC_lexicon_tags_34 },
    { 0x5d5a1476, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f1a1479, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fa147d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07a1481, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47da1483, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x23fa1485, SUC_lexicon_tags_0 },
    { 0x7f5a1486, SUC_lexicon_tags_17 },
    { 0x669a1487, SUC_lexicon_tags_15 },
    { 0xf49a1487, SUC_lexicon_tags_43 },
    { 0x3ffa1489, SUC_lexicon_tags_14 },
    { 0xfcfa1487, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fa1494, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbe5a1496, SUC_lexicon_tags_15 },
    { 0xe27a1497, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd95a1499, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76da149b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafa149e, SUC_lexicon_tags_47 },
    { 0x1a5a149f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x379a14a1, SUC_lexicon_tags_37 },
    { 0x8dda14a2, SUC_lexicon_tags_47 },
    { 0xa4fa14a3, SUC_lexicon_tags_113 },
    { 0x2c1a14a4, SUC_lexicon_tags_30 },
    { 0x9d7a14a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d9a14a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f7a14a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x507a14ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9a14b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfa14b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x771a14b4, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33a14bb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845a14bf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa14c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85a14ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55a14d0, SUC_lexicon_tags_48 },
    { 0xb2da14d1, SUC_lexicon_tags_47 },
    { 0xafa14d1, SUC_lexicon_tags_17 },
    { 0xee5a14d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf5a14d5, SUC_lexicon_tags_39 },
    { 0x2efa14d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3a14db, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8ffa14dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc87a14df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x157a14e1, SUC_lexicon_tags_15 },
    { 0x257a14e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857a14e6, SUC_lexicon_tags_14 },
    { 0x771a14e7, SUC_lexicon_tags_38 },
    { 0xb5da14e7, SUC_lexicon_tags_28 },
    { 0x111a14e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x771a14eb, SUC_lexicon_tags_23 },
    { 0x86fa14ec, SUC_lexicon_tags_17 },
    { 0xa4da14eb, SUC_lexicon_tags_37 },
    { 0x7b5a14eb, SUC_lexicon_tags_37 },
    { 0x49da14ed, SUC_lexicon_tags_7 },
    { 0xc53a14f0, SUC_lexicon_tags_23 },
    { 0x44ba14f1, SUC_lexicon_tags_1 },
    { 0xd79a14f0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xb5da14f4, SUC_lexicon_tags_63 },
    { 0x8f5a14f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969a14fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fa1502, SUC_lexicon_tags_14 },
    { 0x6a1a1502, SUC_lexicon_tags_0 },
    { 0x21a1502, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3a1507, SUC_lexicon_tags_1 },
    { 0xe13a1508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9a150b, SUC_lexicon_tags_15 },
    { 0x1f9a150c, SUC_lexicon_tags_68 },
    { 0xebfa150d, SUC_lexicon_tags_19 },
    { 0x6cfa150c, SUC_lexicon_tags_9 },
    { 0x79da150f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35a1512, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfa1517, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67fa1519, SUC_lexicon_tags_17 },
    { 0xda7a151a, SUC_lexicon_tags_37 },
    { 0x1ada151a, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0xb99a151d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x489a1520, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1a1523, SUC_lexicon_tags_7 },
    { 0x8f3a1524, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x66da1526, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255a152a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf3ba152c, SUC_lexicon_tags_1 },
    { 0x929a152c, SUC_lexicon_tags_15 },
    { 0xb83a152c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xaa5a1530, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bda1534, SUC_lexicon_tags_35 },
    { 0x705a1534, SUC_lexicon_tags_11 },
    { 0x2d1a1536, SUC_lexicon_tags_15 },
    { 0x547a1537, SUC_lexicon_tags_2 },
    { 0x87da1538, SUC_lexicon_tags_16 },
    { 0x93fa1534, SUC_lexicon_tags_7 },
    { 0xe1ba153a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57a1540, SUC_lexicon_tags_2 },
    { 0x16ba1541, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fa1545, SUC_lexicon_tags_18 },
    { 0x9dda1546, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x471a1548, SUC_lexicon_tags_2 },
    { 0xeeba1549, SUC_lexicon_tags_41 },
    { 0x31fa154a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3a1551, SUC_lexicon_tags_9 },
    { 0xb03a1552, SUC_lexicon_tags_16 },
    { 0xdd7a1553, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x59ba1555, SUC_lexicon_tags_30 },
    { 0x705a1556, SUC_lexicon_tags_16 },
    { 0x373a1557, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52da1561, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fa1566, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987a156e, SUC_lexicon_tags_7 },
    { 0xbfa156f, SUC_lexicon_tags_7 },
    { 0x36fa1570, SUC_lexicon_tags_77 },
    { 0xdefa1571, SUC_lexicon_tags_17 },
    { 0x7b5a1570, SUC_lexicon_tags_60 },
    { 0xa75a1572, SUC_lexicon_tags_11 },
    { 0x46fa156e, SUC_lexicon_tags_49 },
    { 0x4da1575, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0xd1da1577, SUC_lexicon_tags_537 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fa157a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641a157f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda1587, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fa158a, SUC_lexicon_tags_87 },
    { 0xf1a158b, SUC_lexicon_tags_107 },
    { 0xddda158c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1a158f, SUC_lexicon_tags_2 },
    { 0x38ba1590, SUC_lexicon_tags_2 },
    { 0x51ba1591, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfb1a1593, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21a1599, SUC_lexicon_tags_9 },
    { 0x6d7a1599, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7bba159c, SUC_lexicon_tags_19 },
    { 0x8d3a159d, SUC_lexicon_tags_63 },
    { 0xe1ba159c, SUC_lexicon_tags_33 },
    { 0x4efa159c, SUC_lexicon_tags_39 },
    { 0x27ba159e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x929a15a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdf3a15a5, SUC_lexicon_tags_31 },
    { 0x5bfa15a5, SUC_lexicon_tags_84 },
    { 0x609a15a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b9a15a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeda15af, SUC_lexicon_tags_91 },
    { 0xe6ba15af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xefba15b2, SUC_lexicon_tags_26 },
    { 0xec9a15b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03a15b6, SUC_lexicon_tags_11 },
    { 0xd0fa15b6, SUC_lexicon_tags_11 },
    { 0x53a15b6, SUC_lexicon_tags_2 },
    { 0x919a15b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26ba15bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfa15c0, SUC_lexicon_tags_7 },
    { 0x909a15c1, SUC_lexicon_tags_45 },
    { 0x69da15c2, SUC_lexicon_tags_25 },
    { 0x1dfa15c1, SUC_lexicon_tags_27 },
    { 0x695a15c3, SUC_lexicon_tags_49 },
    { 0x47a15c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615a15ca, SUC_lexicon_tags_26 },
    { 0x41da15ca, SUC_lexicon_tags_6 },
    { 0x4e1a15cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x927a15cf, SUC_lexicon_tags_17 },
    { 0x39fa15d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x469a15d2, SUC_lexicon_tags_1 },
    { 0xe6fa15d2, SUC_lexicon_tags_46 },
    { 0x355a15d2, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x3fa15d6, SUC_lexicon_tags_47 },
    { 0x479a15d7, SUC_lexicon_tags_12 },
    { 0x99fa15d8, SUC_lexicon_tags_17 },
    { 0xfdba15d9, SUC_lexicon_tags_37 },
    { 0xbefa15d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3a15df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a15e2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x431a15e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549a15e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x347a15eb, SUC_lexicon_tags_9 },
    { 0x975a15eb, SUC_lexicon_tags_23 },
    { 0x809a15ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x709a15ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5da15f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfa15f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fa15fc, SUC_lexicon_tags_49 },
    { 0xf5ba15fd, SUC_lexicon_tags_23 },
    { 0x68fa15fe, SUC_lexicon_tags_2 },
    { 0x191a15ff, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77a1606, SUC_lexicon_tags_33 },
    { 0x807a1606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91a160e, SUC_lexicon_tags_17 },
    { 0x945a160f, SUC_lexicon_tags_69 },
    { 0x76da160f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe37a1612, SUC_lexicon_tags_12 },
    { 0x3f1a1612, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4cfa1615, SUC_lexicon_tags_15 },
    { 0xfbba1616, SUC_lexicon_tags_43 },
    { 0xd17a1616, SUC_lexicon_tags_47 },
    { 0xfa7a1618, SUC_lexicon_tags_5 },
    { 0x663a1618, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x58da161c, SUC_lexicon_tags_10 },
    { 0xce7a161d, SUC_lexicon_tags_7 },
    { 0x713a161e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409a1622, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x561a1624, SUC_lexicon_tags_12 },
    { 0x465a1625, SUC_lexicon_tags_23 },
    { 0x233a1626, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21a162b, SUC_lexicon_tags_6 },
    { 0x5d9a162c, SUC_lexicon_tags_38 },
    { 0xeaba162d, SUC_lexicon_tags_29 },
    { 0x61a162d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9fa1630, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1a1637, SUC_lexicon_tags_12 },
    { 0x84fa1638, SUC_lexicon_tags_57 },
    { 0x92ba1639, SUC_lexicon_tags_0 },
    { 0xef9a1639, SUC_lexicon_tags_47 },
    { 0x40da1639, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1dda163d, SUC_lexicon_tags_207 },
    { 0x455a163e, SUC_lexicon_tags_28 },
    { 0xabfa163e, SUC_lexicon_tags_2 },
    { 0xdd9a1640, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01a1645, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec3a1647, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x49fa1649, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ba1651, SUC_lexicon_tags_15 },
    { 0xab5a1651, SUC_lexicon_tags_25 },
    { 0xb9fa1653, SUC_lexicon_tags_2 },
    { 0x153a1654, SUC_lexicon_tags_37 },
    { 0x901a1654, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3a1658, SUC_lexicon_tags_58 },
    { 0x4fa1658, SUC_lexicon_tags_37 },
    { 0x44da165a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa165e, SUC_lexicon_tags_31 },
    { 0xebba165f, SUC_lexicon_tags_7 },
    { 0x819a1660, SUC_lexicon_tags_63 },
    { 0xd9a165f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa1669, SUC_lexicon_tags_12 },
    { 0x173a166a, SUC_lexicon_tags_11 },
    { 0xf99a1669, SUC_lexicon_tags_25 },
    { 0xa4ba1669, SUC_lexicon_tags_12 },
    { 0x691a166d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dda166f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21a1673, SUC_lexicon_tags_37 },
    { 0x2f1a1674, SUC_lexicon_tags_64 },
    { 0x721a1673, SUC_lexicon_tags_63 },
    { 0xa47a1676, SUC_lexicon_tags_8 },
    { 0xcf5a1677, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1a167b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1a1681, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d5a1683, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa1688, SUC_lexicon_tags_17 },
    { 0x3eda1689, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x717a168b, SUC_lexicon_tags_25 },
    { 0x635a168b, SUC_lexicon_tags_2 },
    { 0x1e5a168c, SUC_lexicon_tags_27 },
    { 0xff7a168e, SUC_lexicon_tags_37 },
    { 0xc77a168f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa97a1691, SUC_lexicon_tags_38 },
    { 0x92fa1692, SUC_lexicon_tags_107 },
    { 0xde3a1692, SUC_lexicon_tags_38 },
    { 0xdb7a1692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ba1696, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2da1699, SUC_lexicon_tags_19 },
    { 0xaba1699, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ba169d, SUC_lexicon_tags_37 },
    { 0xcba169e, SUC_lexicon_tags_2 },
    { 0x4efa169f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ba16a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1bda16a6, SUC_lexicon_tags_63 },
    { 0x24da16a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99da16a9, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x2a7a16ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b1a16ad, SUC_lexicon_tags_25 },
    { 0x449a16ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33fa16b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77da16b9, SUC_lexicon_tags_21 },
    { 0x12ba16ba, SUC_lexicon_tags_6 },
    { 0x5a3a16b9, SUC_lexicon_tags_723 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ba16be, SUC_lexicon_tags_38 },
    { 0x20fa16bf, SUC_lexicon_tags_28 },
    { 0x5b9a16c0, SUC_lexicon_tags_49 },
    { 0x72ba16bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95a16c5, SUC_lexicon_tags_37 },
    { 0x8a1a16c6, SUC_lexicon_tags_17 },
    { 0x313a16c7, SUC_lexicon_tags_13 },
    { 0x99da16c8, SUC_lexicon_tags_79 },
    { 0xc35a16c9, SUC_lexicon_tags_38 },
    { 0x74da16ca, SUC_lexicon_tags_47 },
    { 0xd73a16cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fa16ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7a16d3, SUC_lexicon_tags_53 },
    { 0x647a16d3, SUC_lexicon_tags_28 },
    { 0x929a16d5, SUC_lexicon_tags_7 },
    { 0x39fa16d6, SUC_lexicon_tags_49 },
    { 0x55da16d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55a16da, SUC_lexicon_tags_47 },
    { 0x407a16db, SUC_lexicon_tags_4 },
    { 0xa5da16db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fa16e1, SUC_lexicon_tags_23 },
    { 0xbc1a16e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf29a16e4, SUC_lexicon_tags_37 },
    { 0xf1ba16e5, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a16e8, SUC_lexicon_tags_5 },
    { 0x2c1a16e9, SUC_lexicon_tags_41 },
    { 0xb1da16ea, SUC_lexicon_tags_43 },
    { 0x4bda16ea, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x5bda16ed, SUC_lexicon_tags_2 },
    { 0x971a16ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x475a16f0, SUC_lexicon_tags_32 },
    { 0x425a16f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd3a16f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1a16fc, SUC_lexicon_tags_7 },
    { 0x667a16fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x587a16ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3da1704, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa1709, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd67a170c, SUC_lexicon_tags_33 },
    { 0xc97a170d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3a1711, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16da1713, SUC_lexicon_tags_7 },
    { 0xc09a1714, SUC_lexicon_tags_35 },
    { 0x379a1714, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39a171d, SUC_lexicon_tags_17 },
    { 0x32ba171e, SUC_lexicon_tags_60 },
    { 0x15ba171e, SUC_lexicon_tags_6 },
    { 0x8f5a171d, SUC_lexicon_tags_15 },
    { 0xb3da1720, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf09a1723, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbada1729, SUC_lexicon_tags_11 },
    { 0xa0ba172a, SUC_lexicon_tags_11 },
    { 0xda1a1729, SUC_lexicon_tags_26 },
    { 0x2ba172c, SUC_lexicon_tags_36 },
    { 0xcb1a172d, SUC_lexicon_tags_1 },
    { 0x1ada172e, SUC_lexicon_tags_43 },
    { 0xda9a172e, SUC_lexicon_tags_15 },
    { 0xa69a172d, SUC_lexicon_tags_25 },
    { 0x5f7a1731, SUC_lexicon_tags_17 },
    { 0x29a1731, SUC_lexicon_tags_15 },
    { 0xa7ba1731, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15a173b, SUC_lexicon_tags_12 },
    { 0xe71a173c, SUC_lexicon_tags_45 },
    { 0xd7ba173d, SUC_lexicon_tags_13 },
    { 0x4e3a173d, SUC_lexicon_tags_36 },
    { 0x6fda173f, SUC_lexicon_tags_10 },
    { 0xc13a1740, SUC_lexicon_tags_54 },
    { 0xca1a1740, SUC_lexicon_tags_15 },
    { 0xacda1742, SUC_lexicon_tags_0 },
    { 0xf3fa173b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fa1746, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31a174f, SUC_lexicon_tags_38 },
    { 0xccba174f, SUC_lexicon_tags_17 },
    { 0x92ba1750, SUC_lexicon_tags_17 },
    { 0x22da1752, SUC_lexicon_tags_12 },
    { 0xbb7a1751, SUC_lexicon_tags_39 },
    { 0x23ba174f, SUC_lexicon_tags_15 },
    { 0x943a1755, SUC_lexicon_tags_55 },
    { 0x63a1756, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da175d, SUC_lexicon_tags_14 },
    { 0x6f3a175d, SUC_lexicon_tags_15 },
    { 0xfb7a175e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85da1761, SUC_lexicon_tags_37 },
    { 0x6d3a1762, SUC_lexicon_tags_2 },
    { 0xfe9a1762, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7a5a1765, SUC_lexicon_tags_112 },
    { 0xb99a1766, SUC_lexicon_tags_82 },
    { 0x32ba1767, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x33a1769, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975a176d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63a1772, SUC_lexicon_tags_38 },
    { 0x7d7a1772, SUC_lexicon_tags_15 },
    { 0x927a1774, SUC_lexicon_tags_38 },
    { 0xdc3a1775, SUC_lexicon_tags_71 },
    { 0x7e9a1776, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fa1779, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x259a177b, SUC_lexicon_tags_30 },
    { 0x6a9a177c, SUC_lexicon_tags_9 },
    { 0xa3a177d, SUC_lexicon_tags_7 },
    { 0xe4fa177b, SUC_lexicon_tags_31 },
    { 0x91da177f, SUC_lexicon_tags_15 },
    { 0x583a177c, SUC_lexicon_tags_17 },
    { 0x13da1781, SUC_lexicon_tags_16 },
    { 0xecba1780, SUC_lexicon_tags_7 },
    { 0xe67a1783, SUC_lexicon_tags_56 },
    { 0xec1a1784, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x26da1786, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5a1789, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87a178e, SUC_lexicon_tags_33 },
    { 0x3f3a178f, SUC_lexicon_tags_11 },
    { 0xfe9a1790, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87a1797, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x117a179a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375a179e, SUC_lexicon_tags_25 },
    { 0xdbba179f, SUC_lexicon_tags_12 },
    { 0x31da179f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2a1a17a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x129a17a5, SUC_lexicon_tags_25 },
    { 0x3b1a17a5, SUC_lexicon_tags_25 },
    { 0x5dfa17a7, SUC_lexicon_tags_15 },
    { 0x44da17a7, SUC_lexicon_tags_1 },
    { 0x7f5a17a7, SUC_lexicon_tags_37 },
    { 0x7a5a17aa, SUC_lexicon_tags_23 },
    { 0xf19a17ab, SUC_lexicon_tags_60 },
    { 0xc19a17ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7e3a17ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfba17b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819a17bb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfb5a17bd, SUC_lexicon_tags_2 },
    { 0xefda17be, SUC_lexicon_tags_11 },
    { 0xf81a17bd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7a17c2, SUC_lexicon_tags_67 },
    { 0xcbfa17c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88fa17c5, SUC_lexicon_tags_302 },
    { 0, NULL },
    { 0xbe9a17c7, SUC_lexicon_tags_19 },
    { 0xc6da17c8, SUC_lexicon_tags_1 },
    { 0xc9da17c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299a17ce, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62da17d5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9a17d8, SUC_lexicon_tags_12 },
    { 0xa2ba17d9, SUC_lexicon_tags_37 },
    { 0x867a17d8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7a17e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbd5a17e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbbda17eb, SUC_lexicon_tags_12 },
    { 0xd7da17eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fa17f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5cba17f2, SUC_lexicon_tags_14 },
    { 0x187a17f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba17fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27a17ff, SUC_lexicon_tags_15 },
    { 0x59fa17ff, SUC_lexicon_tags_63 },
    { 0x743a1801, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb1a1803, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x959a1806, SUC_lexicon_tags_21 },
    { 0xbf1a1807, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8eda1809, SUC_lexicon_tags_33 },
    { 0xd75a1809, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef9a180c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbda1810, SUC_lexicon_tags_49 },
    { 0xd77a1811, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x227a1813, SUC_lexicon_tags_25 },
    { 0x73fa1814, SUC_lexicon_tags_0 },
    { 0x9bfa1814, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ba181a, SUC_lexicon_tags_33 },
    { 0xc33a181b, SUC_lexicon_tags_47 },
    { 0xe3da181b, SUC_lexicon_tags_63 },
    { 0xa47a181d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x927a181f, SUC_lexicon_tags_25 },
    { 0x6d7a181f, SUC_lexicon_tags_63 },
    { 0xc2ba1821, SUC_lexicon_tags_35 },
    { 0x667a1822, SUC_lexicon_tags_2 },
    { 0x1a7a181f, SUC_lexicon_tags_48 },
    { 0x97da1824, SUC_lexicon_tags_25 },
    { 0x38ba1825, SUC_lexicon_tags_12 },
    { 0xf1a1825, SUC_lexicon_tags_4 },
    { 0xeefa1820, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9a182b, SUC_lexicon_tags_8 },
    { 0x303a182c, SUC_lexicon_tags_37 },
    { 0x741a182d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x221a1830, SUC_lexicon_tags_37 },
    { 0x69a1830, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7c5a1833, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fa1836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b5a1838, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd7a183a, SUC_lexicon_tags_8 },
    { 0xdf3a183a, SUC_lexicon_tags_10 },
    { 0x417a183c, SUC_lexicon_tags_25 },
    { 0xb7a183d, SUC_lexicon_tags_25 },
    { 0xaf3a183c, SUC_lexicon_tags_32 },
    { 0xc67a183f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x227a1842, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7a1845, SUC_lexicon_tags_11 },
    { 0xf53a1846, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fa1849, SUC_lexicon_tags_17 },
    { 0x9f9a184a, SUC_lexicon_tags_15 },
    { 0x9bda184a, SUC_lexicon_tags_6 },
    { 0xdb9a184a, SUC_lexicon_tags_9 },
    { 0xaf5a184d, SUC_lexicon_tags_37 },
    { 0xe0fa184e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x79da1850, SUC_lexicon_tags_2 },
    { 0x7d7a1850, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3a1857, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1a185a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a185d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x123a185f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a1862, SUC_lexicon_tags_29 },
    { 0x347a1863, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfaba1865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05a186c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e1a186e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3a1875, SUC_lexicon_tags_17 },
    { 0x11ba1876, SUC_lexicon_tags_12 },
    { 0x65a1877, SUC_lexicon_tags_33 },
    { 0xc37a1878, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x22fa187a, SUC_lexicon_tags_27 },
    { 0x2cda187a, SUC_lexicon_tags_7 },
    { 0xddba187b, SUC_lexicon_tags_85 },
    { 0x23fa187b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x32da187f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a188b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fa188e, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x30ba1890, SUC_lexicon_tags_39 },
    { 0x349a1891, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb77a1893, SUC_lexicon_tags_17 },
    { 0x4da1894, SUC_lexicon_tags_28 },
    { 0xcafa1895, SUC_lexicon_tags_17 },
    { 0x535a1893, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99a189a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133a189f, SUC_lexicon_tags_1 },
    { 0x4d9a189f, SUC_lexicon_tags_17 },
    { 0x66fa18a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4da18a4, SUC_lexicon_tags_0 },
    { 0xc5da18a5, SUC_lexicon_tags_45 },
    { 0x5a18a6, SUC_lexicon_tags_12 },
    { 0xef7a18a7, SUC_lexicon_tags_15 },
    { 0xaa5a18a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fa18ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da18af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5a18b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbada18b4, SUC_lexicon_tags_47 },
    { 0xa33a18b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x821a18b7, SUC_lexicon_tags_38 },
    { 0x9dfa18b8, SUC_lexicon_tags_17 },
    { 0xd49a18b7, SUC_lexicon_tags_17 },
    { 0xfb9a18ba, SUC_lexicon_tags_27 },
    { 0x81ba18ba, SUC_lexicon_tags_8 },
    { 0xe9da18ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb3fa18be, SUC_lexicon_tags_37 },
    { 0xb2da18bf, SUC_lexicon_tags_47 },
    { 0x26fa18bf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51a18c4, SUC_lexicon_tags_11 },
    { 0x617a18c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa65a18c7, SUC_lexicon_tags_14 },
    { 0x963a18c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a18cc, SUC_lexicon_tags_37 },
    { 0x217a18cd, SUC_lexicon_tags_25 },
    { 0x551a18ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d9a18d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeba18d3, SUC_lexicon_tags_19 },
    { 0x95ba18d3, SUC_lexicon_tags_19 },
    { 0x14fa18d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5da18d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11a18db, SUC_lexicon_tags_47 },
    { 0xcf5a18db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0da18e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfeda18e2, SUC_lexicon_tags_15 },
    { 0xabba18e3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7a18ea, SUC_lexicon_tags_6 },
    { 0x5e5a18eb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5a18ef, SUC_lexicon_tags_224 },
    { 0x611a18ef, SUC_lexicon_tags_14 },
    { 0xf7ba18ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59a18f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7a18f8, SUC_lexicon_tags_25 },
    { 0x19fa18f8, SUC_lexicon_tags_26 },
    { 0xdada18f9, SUC_lexicon_tags_23 },
    { 0x95ba18fb, SUC_lexicon_tags_17 },
    { 0xfbba18fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x889a18fe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9a1902, SUC_lexicon_tags_14 },
    { 0x7e3a1903, SUC_lexicon_tags_37 },
    { 0xd15a1902, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7a1909, SUC_lexicon_tags_38 },
    { 0x86fa190a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc9a190c, SUC_lexicon_tags_14 },
    { 0x2f1a190d, SUC_lexicon_tags_7 },
    { 0xadda190d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7a1912, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x729a1914, SUC_lexicon_tags_38 },
    { 0x8b7a1914, SUC_lexicon_tags_27 },
    { 0xaa7a1916, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697a191a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xed5a191c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3a191f, SUC_lexicon_tags_39 },
    { 0x671a1920, SUC_lexicon_tags_15 },
    { 0xdf1a1921, SUC_lexicon_tags_63 },
    { 0xa07a1921, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a1927, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11a192a, SUC_lexicon_tags_11 },
    { 0x5d1a192a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2cda192d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc91a192f, SUC_lexicon_tags_17 },
    { 0x6e1a1930, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd13a1932, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5e1a1934, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8da1936, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bba193b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a193e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9a1941, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfe5a1943, SUC_lexicon_tags_14 },
    { 0x4cba1944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x183a1946, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47a194c, SUC_lexicon_tags_25 },
    { 0x80fa194d, SUC_lexicon_tags_9 },
    { 0x24ba194e, SUC_lexicon_tags_113 },
    { 0xf2da194f, SUC_lexicon_tags_27 },
    { 0xf65a194f, SUC_lexicon_tags_12 },
    { 0x639a1951, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ba1955, SUC_lexicon_tags_65 },
    { 0x973a1955, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9a1959, SUC_lexicon_tags_47 },
    { 0x53da1959, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfefa195c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377a1961, SUC_lexicon_tags_27 },
    { 0xca9a1962, SUC_lexicon_tags_3 },
    { 0x327a1962, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1a1966, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfa196b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x217a196e, SUC_lexicon_tags_87 },
    { 0xdbda196f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3a1972, SUC_lexicon_tags_15 },
    { 0x86ba1972, SUC_lexicon_tags_14 },
    { 0xd65a1973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7da1977, SUC_lexicon_tags_14 },
    { 0xa01a1978, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe83a197a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3a1984, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15da1986, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9efa1988, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80da198a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fba198f, SUC_lexicon_tags_37 },
    { 0x451a198f, SUC_lexicon_tags_12 },
    { 0x81da1991, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87a1994, SUC_lexicon_tags_31 },
    { 0x471a1995, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xafba1998, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79da199c, SUC_lexicon_tags_11 },
    { 0xe77a199d, SUC_lexicon_tags_47 },
    { 0x44da199e, SUC_lexicon_tags_12 },
    { 0xdc3a199f, SUC_lexicon_tags_37 },
    { 0x6f5a19a0, SUC_lexicon_tags_140 },
    { 0xaf5a199c, SUC_lexicon_tags_25 },
    { 0x4f5a199f, SUC_lexicon_tags_25 },
    { 0xe25a19a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a19a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x13ba19a9, SUC_lexicon_tags_25 },
    { 0x9d7a19a9, SUC_lexicon_tags_2 },
    { 0x441a19ab, SUC_lexicon_tags_0 },
    { 0x12fa19ac, SUC_lexicon_tags_7 },
    { 0xf75a19ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc5fa19af, SUC_lexicon_tags_11 },
    { 0x487a19b0, SUC_lexicon_tags_44 },
    { 0x4b9a19b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeda19b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x373a19b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7a19b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x457a19bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaa7a19be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9b5a19c0, SUC_lexicon_tags_62 },
    { 0x485a19c1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xa93a19c3, SUC_lexicon_tags_30 },
    { 0x81a19c4, SUC_lexicon_tags_11 },
    { 0xe3ba19c5, SUC_lexicon_tags_141 },
    { 0x943a19c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ba19cc, SUC_lexicon_tags_2 },
    { 0x6c1a19cd, SUC_lexicon_tags_31 },
    { 0x617a19ce, SUC_lexicon_tags_32 },
    { 0xe1fa19cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbaba19d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963a19d8, SUC_lexicon_tags_27 },
    { 0xbbba19d9, SUC_lexicon_tags_1 },
    { 0x8aba19da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439a19de, SUC_lexicon_tags_15 },
    { 0x74ba19de, SUC_lexicon_tags_7 },
    { 0x6d3a19de, SUC_lexicon_tags_15 },
    { 0x305a19de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717a19e7, SUC_lexicon_tags_87 },
    { 0xf4ba19e8, SUC_lexicon_tags_46 },
    { 0xd79a19e7, SUC_lexicon_tags_27 },
    { 0x9c5a19ea, SUC_lexicon_tags_25 },
    { 0xb9fa19e7, SUC_lexicon_tags_33 },
    { 0x89da19e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21da19ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe5a19f1, SUC_lexicon_tags_30 },
    { 0x821a19f2, SUC_lexicon_tags_4 },
    { 0x427a19f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x555a19f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85ba19f7, SUC_lexicon_tags_7 },
    { 0xa4da19f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881a1a01, SUC_lexicon_tags_6 },
    { 0x17a1a02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea7a1a04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3a1a07, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xfba1a09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x309a1a0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287a1a10, SUC_lexicon_tags_9 },
    { 0xef1a1a10, SUC_lexicon_tags_34 },
    { 0x379a1a11, SUC_lexicon_tags_15 },
    { 0xa73a1a13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1a1a17, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x531a1a19, SUC_lexicon_tags_25 },
    { 0x375a1a19, SUC_lexicon_tags_17 },
    { 0x593a1a1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6b9a1a1d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3a1a20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa7a1a22, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fa1a27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb09a1a29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539a1a2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb1a1a2f, SUC_lexicon_tags_3 },
    { 0xdc3a1a30, SUC_lexicon_tags_15 },
    { 0x5e3a1a31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351a1a35, SUC_lexicon_tags_21 },
    { 0xa05a1a36, SUC_lexicon_tags_27 },
    { 0x849a1a37, SUC_lexicon_tags_12 },
    { 0xc6ba1a38, SUC_lexicon_tags_37 },
    { 0x245a1a39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9dba1a3b, SUC_lexicon_tags_2 },
    { 0xd25a1a3c, SUC_lexicon_tags_6 },
    { 0x13a1a3b, SUC_lexicon_tags_10 },
    { 0x371a1a3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6a5a1a40, SUC_lexicon_tags_11 },
    { 0x87da1a41, SUC_lexicon_tags_37 },
    { 0x1c1a1a40, SUC_lexicon_tags_62 },
    { 0x8fba1a43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e7a1a45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9a1a50, SUC_lexicon_tags_17 },
    { 0x941a1a50, SUC_lexicon_tags_25 },
    { 0xc63a1a52, SUC_lexicon_tags_23 },
    { 0x66fa1a52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fa1a57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a1a59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x521a1a5b, SUC_lexicon_tags_1 },
    { 0xdc5a1a5c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf95a1a5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc1ba1a60, SUC_lexicon_tags_11 },
    { 0xcbfa1a61, SUC_lexicon_tags_31 },
    { 0xc9da1a62, SUC_lexicon_tags_17 },
    { 0x1f3a1a62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x687a1a65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7a1a69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43a1a6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa1a6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfa1a71, SUC_lexicon_tags_25 },
    { 0xaa7a1a71, SUC_lexicon_tags_43 },
    { 0x2d1a1a71, SUC_lexicon_tags_1 },
    { 0x9c7a1a74, SUC_lexicon_tags_43 },
    { 0xd2da1a75, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe4fa1a77, SUC_lexicon_tags_37 },
    { 0x733a1a78, SUC_lexicon_tags_8 },
    { 0xe25a1a79, SUC_lexicon_tags_63 },
    { 0x10fa1a7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xed5a1a7c, SUC_lexicon_tags_46 },
    { 0x50fa1a7c, SUC_lexicon_tags_0 },
    { 0x361a1a7c, SUC_lexicon_tags_7 },
    { 0x2b7a1a7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ba1a83, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3a1a86, SUC_lexicon_tags_55 },
    { 0x45ba1a87, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa1a8b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4eda1a8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffa1a90, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x419a1a92, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167a1a97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e7a1a99, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1a9c, SUC_lexicon_tags_30 },
    { 0xdcfa1a9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35a1aa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfefa1aa3, SUC_lexicon_tags_65 },
    { 0xb27a1aa3, SUC_lexicon_tags_1 },
    { 0x9eda1aa5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x573a1aa7, SUC_lexicon_tags_17 },
    { 0x1f9a1aa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d3a1aaa, SUC_lexicon_tags_40 },
    { 0x2cfa1aab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7a1ab0, SUC_lexicon_tags_12 },
    { 0x699a1ab0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a1ab5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafa1ab9, SUC_lexicon_tags_46 },
    { 0x8fba1aba, SUC_lexicon_tags_3 },
    { 0xca9a1abb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525a1abf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ada1ac1, SUC_lexicon_tags_11 },
    { 0x9eba1ac2, SUC_lexicon_tags_27 },
    { 0x94fa1ac2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xecba1ac5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9a1ac8, SUC_lexicon_tags_25 },
    { 0xdf9a1ac8, SUC_lexicon_tags_2 },
    { 0x419a1aca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x175a1acc, SUC_lexicon_tags_25 },
    { 0x433a1acc, SUC_lexicon_tags_10 },
    { 0x655a1acd, SUC_lexicon_tags_221 },
    { 0xb63a1acf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cba1ad2, SUC_lexicon_tags_25 },
    { 0x585a1ad2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffa1ad6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc7a1ad8, SUC_lexicon_tags_14 },
    { 0xe1a1ad9, SUC_lexicon_tags_15 },
    { 0x1b7a1ad9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99fa1adc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41a1ae0, SUC_lexicon_tags_55 },
    { 0x73a1ae1, SUC_lexicon_tags_6 },
    { 0xe1ba1ae1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70da1ae4, SUC_lexicon_tags_24 },
    { 0xb6ba1ae4, SUC_lexicon_tags_14 },
    { 0xfffa1ae6, SUC_lexicon_tags_27 },
    { 0x2a1a1ae6, SUC_lexicon_tags_23 },
    { 0x3b1a1ae8, SUC_lexicon_tags_17 },
    { 0x25a1ae9, SUC_lexicon_tags_47 },
    { 0x783a1ae7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11a1aee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x515a1af0, SUC_lexicon_tags_17 },
    { 0x203a1af1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a1af8, SUC_lexicon_tags_53 },
    { 0xd1fa1af9, SUC_lexicon_tags_7 },
    { 0x2b3a1afa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x769a1afc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46da1b00, SUC_lexicon_tags_7 },
    { 0xcc1a1b01, SUC_lexicon_tags_15 },
    { 0xea7a1b02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747a1b08, SUC_lexicon_tags_26 },
    { 0xd03a1b08, SUC_lexicon_tags_43 },
    { 0x8aba1b0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x469a1b0c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4c9a1b0e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf2da1b10, SUC_lexicon_tags_46 },
    { 0x4c7a1b10, SUC_lexicon_tags_55 },
    { 0x845a1b11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x51a1b15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1ffa1b17, SUC_lexicon_tags_11 },
    { 0x17fa1b18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739a1b1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693a1b21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x53da1b23, SUC_lexicon_tags_1 },
    { 0x641a1b24, SUC_lexicon_tags_77 },
    { 0xd6da1b25, SUC_lexicon_tags_37 },
    { 0x829a1b25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fa1b29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9a1b2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x829a1b30, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa1b36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3a1b3e, SUC_lexicon_tags_15 },
    { 0xb45a1b3e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1ada1b41, SUC_lexicon_tags_14 },
    { 0xc61a1b42, SUC_lexicon_tags_11 },
    { 0xe39a1b43, SUC_lexicon_tags_17 },
    { 0xe41a1b44, SUC_lexicon_tags_25 },
    { 0xdda1b44, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487a1b49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dba1b4c, SUC_lexicon_tags_61 },
    { 0x75fa1b4d, SUC_lexicon_tags_27 },
    { 0x587a1b4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x687a1b50, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5a1b53, SUC_lexicon_tags_64 },
    { 0x2fa1b54, SUC_lexicon_tags_37 },
    { 0x3e7a1b55, SUC_lexicon_tags_65 },
    { 0x867a1b56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23da1b58, SUC_lexicon_tags_12 },
    { 0xe41a1b59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x585a1b5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3a1b5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3a1b68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9dfa1b6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fda1b6e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1a1b72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1bba1b74, SUC_lexicon_tags_208 },
    { 0x6e7a1b74, SUC_lexicon_tags_2 },
    { 0xe43a1b76, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeb1a1b78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5a1b7e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf99a1b80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ba1b84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5a1b8b, SUC_lexicon_tags_19 },
    { 0xd5fa1b8b, SUC_lexicon_tags_7 },
    { 0x515a1b8d, SUC_lexicon_tags_60 },
    { 0x747a1b8d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x10da1b90, SUC_lexicon_tags_17 },
    { 0x48fa1b91, SUC_lexicon_tags_7 },
    { 0x311a1b91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccba1b94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27a1b96, SUC_lexicon_tags_45 },
    { 0xb59a1b96, SUC_lexicon_tags_9 },
    { 0x8bfa1b98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1a1b9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbba1ba3, SUC_lexicon_tags_6 },
    { 0xb7ba1ba4, SUC_lexicon_tags_7 },
    { 0xc57a1ba5, SUC_lexicon_tags_13 },
    { 0xd0ba1ba4, SUC_lexicon_tags_45 },
    { 0xf7ba1ba7, SUC_lexicon_tags_17 },
    { 0x655a1ba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62da1baa, SUC_lexicon_tags_63 },
    { 0xc71a1bab, SUC_lexicon_tags_91 },
    { 0xa7fa1bac, SUC_lexicon_tags_31 },
    { 0xfa5a1bab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98da1baf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a1bb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fa1bb6, SUC_lexicon_tags_37 },
    { 0x615a1bb7, SUC_lexicon_tags_1 },
    { 0xa25a1bb8, SUC_lexicon_tags_2 },
    { 0x13fa1bb9, SUC_lexicon_tags_9 },
    { 0xbefa1bb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3a1bbd, SUC_lexicon_tags_11 },
    { 0xb23a1bbe, SUC_lexicon_tags_98 },
    { 0xe2da1bbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539a1bc4, SUC_lexicon_tags_47 },
    { 0x8a3a1bc4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7bda1bc7, SUC_lexicon_tags_49 },
    { 0x18fa1bc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f3a1bca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe65a1bcc, SUC_lexicon_tags_58 },
    { 0x3f5a1bcd, SUC_lexicon_tags_26 },
    { 0x21a1bce, SUC_lexicon_tags_1 },
    { 0x45ba1bcf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655a1bd6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd59a1bd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783a1be0, SUC_lexicon_tags_7 },
    { 0x2b9a1be0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fa1be4, SUC_lexicon_tags_37 },
    { 0x517a1be5, SUC_lexicon_tags_47 },
    { 0x3e9a1be6, SUC_lexicon_tags_1 },
    { 0x51ba1be5, SUC_lexicon_tags_17 },
    { 0xd73a1be8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1a1beb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd4ba1bed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fa1bf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9a1bf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcd7a1bfa, SUC_lexicon_tags_45 },
    { 0xfd3a1bfb, SUC_lexicon_tags_74 },
    { 0x689a1bfc, SUC_lexicon_tags_38 },
    { 0xa21a1bfd, SUC_lexicon_tags_8 },
    { 0x45fa1bfc, SUC_lexicon_tags_9 },
    { 0xcfba1bfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2dda1c01, SUC_lexicon_tags_17 },
    { 0x381a1c01, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x715a1c04, SUC_lexicon_tags_15 },
    { 0x9dba1c04, SUC_lexicon_tags_23 },
    { 0xa0ba1c04, SUC_lexicon_tags_27 },
    { 0x649a1c07, SUC_lexicon_tags_37 },
    { 0x711a1c07, SUC_lexicon_tags_39 },
    { 0xb5fa1c06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3da1c0f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7a1c15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39a1c19, SUC_lexicon_tags_17 },
    { 0x103a1c1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5a1c1d, SUC_lexicon_tags_37 },
    { 0x979a1c1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcba1c23, SUC_lexicon_tags_48 },
    { 0x113a1c24, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x853a1c26, SUC_lexicon_tags_12 },
    { 0xeb3a1c27, SUC_lexicon_tags_6 },
    { 0x55a1c28, SUC_lexicon_tags_27 },
    { 0xb07a1c29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b9a1c2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb7a1c2d, SUC_lexicon_tags_1 },
    { 0xfe1a1c2e, SUC_lexicon_tags_1 },
    { 0x3dfa1c2e, SUC_lexicon_tags_37 },
    { 0x73fa1c2f, SUC_lexicon_tags_1 },
    { 0x4b9a1c31, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x93ba1c33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfe1a1c35, SUC_lexicon_tags_38 },
    { 0x1bfa1c36, SUC_lexicon_tags_31 },
    { 0xff1a1c36, SUC_lexicon_tags_38 },
    { 0x9e3a1c38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641a1c3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc15a1c3f, SUC_lexicon_tags_12 },
    { 0xe07a1c40, SUC_lexicon_tags_28 },
    { 0x3d5a1c41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7a1c44, SUC_lexicon_tags_31 },
    { 0xc7da1c44, SUC_lexicon_tags_12 },
    { 0xcc5a1c45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaba1c49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9a1c4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x143a1c4f, SUC_lexicon_tags_17 },
    { 0x11a1c4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5a1c53, SUC_lexicon_tags_11 },
    { 0xbb1a1c54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xedda1c56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed3a1c58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09a1c5d, SUC_lexicon_tags_37 },
    { 0xd29a1c5d, SUC_lexicon_tags_3 },
    { 0xc17a1c5d, SUC_lexicon_tags_49 },
    { 0x9e3a1c5f, SUC_lexicon_tags_14 },
    { 0x84da1c61, SUC_lexicon_tags_23 },
    { 0x9aba1c62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7a1c6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0da1c74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9a1c79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cba1c7f, SUC_lexicon_tags_1 },
    { 0x161a1c80, SUC_lexicon_tags_43 },
    { 0x501a1c81, SUC_lexicon_tags_7 },
    { 0x769a1c82, SUC_lexicon_tags_477 },
    { 0x58ba1c82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9a1c88, SUC_lexicon_tags_12 },
    { 0x95da1c88, SUC_lexicon_tags_15 },
    { 0x815a1c89, SUC_lexicon_tags_63 },
    { 0xc9ba1c88, SUC_lexicon_tags_8 },
    { 0xcd5a1c89, SUC_lexicon_tags_28 },
    { 0x15a1c8d, SUC_lexicon_tags_49 },
    { 0x5c5a1c8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xccfa1c90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8fa1c92, SUC_lexicon_tags_43 },
    { 0xe89a1c93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacba1c95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x751a1c97, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ba1c99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59fa1c9b, SUC_lexicon_tags_7 },
    { 0x42da1c9c, SUC_lexicon_tags_17 },
    { 0x86ba1c9d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5a1ca1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc3da1ca3, SUC_lexicon_tags_2 },
    { 0xeda1ca3, SUC_lexicon_tags_37 },
    { 0xc13a1ca5, SUC_lexicon_tags_27 },
    { 0xbd3a1ca4, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x4c7a1ca8, SUC_lexicon_tags_11 },
    { 0x103a1ca8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1a1cac, SUC_lexicon_tags_16 },
    { 0x3ba1cac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaffa1caf, SUC_lexicon_tags_37 },
    { 0x973a1caf, SUC_lexicon_tags_17 },
    { 0x375a1cb1, SUC_lexicon_tags_15 },
    { 0x1f7a1cb1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ba1cbb, SUC_lexicon_tags_29 },
    { 0x9c7a1cbb, SUC_lexicon_tags_2 },
    { 0x271a1cbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fa1cc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96fa1cc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1a1cc8, SUC_lexicon_tags_15 },
    { 0x8d7a1cc9, SUC_lexicon_tags_30 },
    { 0x659a1cca, SUC_lexicon_tags_15 },
    { 0x413a1cca, SUC_lexicon_tags_1 },
    { 0xdeba1ccc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9a1cd7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33a1cdb, SUC_lexicon_tags_12 },
    { 0x309a1cdc, SUC_lexicon_tags_49 },
    { 0xf61a1cdd, SUC_lexicon_tags_11 },
    { 0xdda1cde, SUC_lexicon_tags_4 },
    { 0xf3ba1cdf, SUC_lexicon_tags_12 },
    { 0xd71a1cde, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x493a1ce3, SUC_lexicon_tags_10 },
    { 0x8c1a1ce4, SUC_lexicon_tags_8 },
    { 0x1b5a1ce4, SUC_lexicon_tags_7 },
    { 0x9bda1ce6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6d9a1ce8, SUC_lexicon_tags_12 },
    { 0x783a1ce9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc97a1ceb, SUC_lexicon_tags_17 },
    { 0xeffa1cec, SUC_lexicon_tags_0 },
    { 0x26ba1ced, SUC_lexicon_tags_15 },
    { 0xb6ba1cee, SUC_lexicon_tags_59 },
    { 0x599a1cee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x359a1cf2, SUC_lexicon_tags_37 },
    { 0xaafa1cf3, SUC_lexicon_tags_22 },
    { 0xdcba1cf4, SUC_lexicon_tags_31 },
    { 0x70fa1cf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba9a1cf7, SUC_lexicon_tags_15 },
    { 0x7c7a1cf8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc83a1cfa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69a1cfe, SUC_lexicon_tags_37 },
    { 0x9d9a1cff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0da1d06, SUC_lexicon_tags_11 },
    { 0xd21a1d07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15a1d0a, SUC_lexicon_tags_31 },
    { 0x34fa1d0a, SUC_lexicon_tags_95 },
    { 0x213a1d0c, SUC_lexicon_tags_4 },
    { 0x3a7a1d0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a1d13, SUC_lexicon_tags_1 },
    { 0xca5a1d14, SUC_lexicon_tags_26 },
    { 0xfb3a1d13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1a1d18, SUC_lexicon_tags_23 },
    { 0xe5a1d19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fa1d1c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3a1d26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1a1d2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645a1d30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x655a1d32, SUC_lexicon_tags_6 },
    { 0x1b5a1d32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1d37, SUC_lexicon_tags_15 },
    { 0x6c9a1d38, SUC_lexicon_tags_56 },
    { 0x4dfa1d38, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fa1d3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eba1d3f, SUC_lexicon_tags_17 },
    { 0x1cfa1d40, SUC_lexicon_tags_25 },
    { 0xb9a1d41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1a1d45, SUC_lexicon_tags_37 },
    { 0x52ba1d46, SUC_lexicon_tags_19 },
    { 0x3d1a1d47, SUC_lexicon_tags_2 },
    { 0x81da1d48, SUC_lexicon_tags_2 },
    { 0xbf5a1d49, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229a1d50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe03a1d52, SUC_lexicon_tags_2 },
    { 0x551a1d53, SUC_lexicon_tags_55 },
    { 0x1b3a1d54, SUC_lexicon_tags_6 },
    { 0xbd7a1d55, SUC_lexicon_tags_37 },
    { 0xbbda1d53, SUC_lexicon_tags_8 },
    { 0x8dda1d55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c1a1d59, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cba1d5f, SUC_lexicon_tags_37 },
    { 0x587a1d60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa25a1d62, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7a1d68, SUC_lexicon_tags_14 },
    { 0x99da1d69, SUC_lexicon_tags_47 },
    { 0x699a1d6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849a1d70, SUC_lexicon_tags_70 },
    { 0x8cfa1d71, SUC_lexicon_tags_4 },
    { 0x2c7a1d71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803a1d79, SUC_lexicon_tags_14 },
    { 0x31a1d79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41a1d7e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x68ba1d80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557a1d84, SUC_lexicon_tags_29 },
    { 0x2a5a1d85, SUC_lexicon_tags_49 },
    { 0x3cda1d86, SUC_lexicon_tags_23 },
    { 0x325a1d86, SUC_lexicon_tags_9 },
    { 0x969a1d84, SUC_lexicon_tags_15 },
    { 0x9d3a1d85, SUC_lexicon_tags_25 },
    { 0x60ba1d8a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787a1d93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x469a1d95, SUC_lexicon_tags_17 },
    { 0x539a1d96, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bba1d9b, SUC_lexicon_tags_11 },
    { 0x187a1d9c, SUC_lexicon_tags_7 },
    { 0x421a1d9d, SUC_lexicon_tags_19 },
    { 0xf89a1d9c, SUC_lexicon_tags_17 },
    { 0x94fa1d9f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x107a1da2, SUC_lexicon_tags_19 },
    { 0x393a1da3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fda1da7, SUC_lexicon_tags_91 },
    { 0x59da1da7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9c3a1daa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ba1dad, SUC_lexicon_tags_38 },
    { 0xa37a1dae, SUC_lexicon_tags_79 },
    { 0x7e5a1daf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc0ba1db1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x477a1db3, SUC_lexicon_tags_14 },
    { 0x1c1a1db4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87a1db8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9a1dbb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7a1dbe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01a1dc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3a1dc5, SUC_lexicon_tags_17 },
    { 0x911a1dc6, SUC_lexicon_tags_37 },
    { 0x827a1dc7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a1dcc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91a1dd0, SUC_lexicon_tags_26 },
    { 0x3f1a1dd1, SUC_lexicon_tags_60 },
    { 0x6b1a1dd1, SUC_lexicon_tags_17 },
    { 0x7cba1dd3, SUC_lexicon_tags_25 },
    { 0x165a1dd4, SUC_lexicon_tags_15 },
    { 0x893a1dd3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09a1dd8, SUC_lexicon_tags_56 },
    { 0xa8fa1dd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x741a1ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3ba1ddd, SUC_lexicon_tags_25 },
    { 0x6efa1dde, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x61da1de0, SUC_lexicon_tags_11 },
    { 0x7cfa1de0, SUC_lexicon_tags_1 },
    { 0x61a1de1, SUC_lexicon_tags_25 },
    { 0x803a1de2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3a1de7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1a1dea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8bda1dec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13a1def, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fa1df3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5a1df7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6da1df9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51a1dfc, SUC_lexicon_tags_12 },
    { 0x21da1dfc, SUC_lexicon_tags_65 },
    { 0x7b5a1dfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3ffa1e00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a1e03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3c3a1e05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fa1e09, SUC_lexicon_tags_10 },
    { 0xf4ba1e09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5dfa1e0c, SUC_lexicon_tags_28 },
    { 0x1b5a1e0c, SUC_lexicon_tags_43 },
    { 0x5b3a1e0e, SUC_lexicon_tags_47 },
    { 0xfcda1e0e, SUC_lexicon_tags_17 },
    { 0x40fa1e10, SUC_lexicon_tags_6 },
    { 0xf43a1e11, SUC_lexicon_tags_2 },
    { 0x1bda1e0f, SUC_lexicon_tags_26 },
    { 0x957a1e13, SUC_lexicon_tags_37 },
    { 0x707a1e14, SUC_lexicon_tags_30 },
    { 0x3f3a1e14, SUC_lexicon_tags_17 },
    { 0xd0da1e15, SUC_lexicon_tags_45 },
    { 0xcd9a1e14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5b9a1e19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fa1e1c, SUC_lexicon_tags_77 },
    { 0xf0fa1e1d, SUC_lexicon_tags_82 },
    { 0x939a1e1e, SUC_lexicon_tags_11 },
    { 0x637a1e1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab5a1e21, SUC_lexicon_tags_2 },
    { 0xab9a1e22, SUC_lexicon_tags_17 },
    { 0xc0ba1e23, SUC_lexicon_tags_49 },
    { 0xeba1e24, SUC_lexicon_tags_46 },
    { 0x64ba1e22, SUC_lexicon_tags_2 },
    { 0x50fa1e26, SUC_lexicon_tags_2 },
    { 0x6dda1e27, SUC_lexicon_tags_46 },
    { 0x415a1e27, SUC_lexicon_tags_49 },
    { 0xf07a1e23, SUC_lexicon_tags_38 },
    { 0x8f7a1e29, SUC_lexicon_tags_0 },
    { 0xd7fa1e2b, SUC_lexicon_tags_45 },
    { 0x765a1e2c, SUC_lexicon_tags_14 },
    { 0x8e1a1e2d, SUC_lexicon_tags_60 },
    { 0x21ba1e2e, SUC_lexicon_tags_43 },
    { 0xdb3a1e2c, SUC_lexicon_tags_7 },
    { 0x89ba1e30, SUC_lexicon_tags_17 },
    { 0x5b5a1e26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5a1e34, SUC_lexicon_tags_30 },
    { 0x56fa1e35, SUC_lexicon_tags_30 },
    { 0x849a1e36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x68fa1e39, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587a1e3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63a1e42, SUC_lexicon_tags_61 },
    { 0x391a1e43, SUC_lexicon_tags_48 },
    { 0xccba1e42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5a1e47, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9da1e4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3a1e54, SUC_lexicon_tags_25 },
    { 0xb79a1e55, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xa35a1e57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197a1e61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fa1e66, SUC_lexicon_tags_17 },
    { 0x6f7a1e67, SUC_lexicon_tags_0 },
    { 0x2e1a1e67, SUC_lexicon_tags_19 },
    { 0x7e7a1e68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeada1e6d, SUC_lexicon_tags_27 },
    { 0xdfa1e6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6da1e70, SUC_lexicon_tags_4 },
    { 0x80ba1e70, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fa1e77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ba1e7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce7a1e7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x659a1e80, SUC_lexicon_tags_38 },
    { 0x549a1e80, SUC_lexicon_tags_32 },
    { 0xbd3a1e82, SUC_lexicon_tags_17 },
    { 0x919a1e80, SUC_lexicon_tags_37 },
    { 0x189a1e84, SUC_lexicon_tags_17 },
    { 0x6e3a1e85, SUC_lexicon_tags_33 },
    { 0x873a1e86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x47fa1e88, SUC_lexicon_tags_12 },
    { 0xbba1e89, SUC_lexicon_tags_31 },
    { 0xc95a1e89, SUC_lexicon_tags_17 },
    { 0xfd5a1e89, SUC_lexicon_tags_86 },
    { 0x4dfa1e8c, SUC_lexicon_tags_25 },
    { 0x881a1e8d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ba1e91, SUC_lexicon_tags_38 },
    { 0x885a1e92, SUC_lexicon_tags_37 },
    { 0xa17a1e92, SUC_lexicon_tags_39 },
    { 0xcbba1e91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111a1e98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99da1e9d, SUC_lexicon_tags_17 },
    { 0x1dfa1e9d, SUC_lexicon_tags_11 },
    { 0x7b5a1e9f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ba1ea4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311a1eaa, SUC_lexicon_tags_25 },
    { 0x57a1eab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x78da1ead, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5a1eb0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185a1eb4, SUC_lexicon_tags_25 },
    { 0x27ba1eb5, SUC_lexicon_tags_14 },
    { 0x40fa1eb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x547a1eb9, SUC_lexicon_tags_12 },
    { 0xd85a1eba, SUC_lexicon_tags_25 },
    { 0xa91a1eb9, SUC_lexicon_tags_2 },
    { 0xb4fa1ebb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ba1ec6, SUC_lexicon_tags_0 },
    { 0x1c1a1ec7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b1a1ec9, SUC_lexicon_tags_103 },
    { 0x531a1eca, SUC_lexicon_tags_47 },
    { 0xcc5a1ecb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66da1ecd, SUC_lexicon_tags_170 },
    { 0x7d7a1ecd, SUC_lexicon_tags_7 },
    { 0x533a1ecf, SUC_lexicon_tags_15 },
    { 0x99da1ecf, SUC_lexicon_tags_49 },
    { 0x353a1ecf, SUC_lexicon_tags_15 },
    { 0x531a1ed2, SUC_lexicon_tags_33 },
    { 0xec3a1ecf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afa1ed7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1aba1ed9, SUC_lexicon_tags_63 },
    { 0xdb1a1ed9, SUC_lexicon_tags_6 },
    { 0xc27a1edb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x913a1ede, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1a1ee1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63a1ee8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9a1eeb, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xfffa1eed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51da1ef2, SUC_lexicon_tags_40 },
    { 0x2dfa1ef2, SUC_lexicon_tags_32 },
    { 0x19a1ef4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xee7a1ef6, SUC_lexicon_tags_17 },
    { 0xd6fa1ef7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265a1efb, SUC_lexicon_tags_38 },
    { 0xccba1efc, SUC_lexicon_tags_0 },
    { 0x991a1efd, SUC_lexicon_tags_1 },
    { 0x3c9a1efe, SUC_lexicon_tags_17 },
    { 0x9fda1eff, SUC_lexicon_tags_46 },
    { 0xaf9a1eff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855a1f07, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x19a1f09, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199a1f0e, SUC_lexicon_tags_2 },
    { 0xec1a1f0e, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xe91a1f11, SUC_lexicon_tags_15 },
    { 0x581a1f12, SUC_lexicon_tags_47 },
    { 0xb07a1f12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x601a1f15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a1f19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49a1f1c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc5a1f1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21a1f21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1a1f26, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x235a1f28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ba1f2c, SUC_lexicon_tags_25 },
    { 0xeba1f2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x769a1f30, SUC_lexicon_tags_1 },
    { 0x577a1f31, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691a1f36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x679a1f39, SUC_lexicon_tags_37 },
    { 0x8d1a1f3a, SUC_lexicon_tags_32 },
    { 0x3fa1f3b, SUC_lexicon_tags_48 },
    { 0xf41a1f39, SUC_lexicon_tags_82 },
    { 0xc81a1f3c, SUC_lexicon_tags_15 },
    { 0x74da1f3e, SUC_lexicon_tags_55 },
    { 0x33a1f3f, SUC_lexicon_tags_37 },
    { 0x93ba1f3c, SUC_lexicon_tags_11 },
    { 0x44ba1f41, SUC_lexicon_tags_37 },
    { 0x657a1f42, SUC_lexicon_tags_19 },
    { 0xcb9a1f42, SUC_lexicon_tags_17 },
    { 0xebfa1f41, SUC_lexicon_tags_47 },
    { 0x52fa1f43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x79ba1f47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ba1f4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a1f4c, SUC_lexicon_tags_12 },
    { 0x401a1f4d, SUC_lexicon_tags_6 },
    { 0xcda1f4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x901a1f50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe71a1f52, SUC_lexicon_tags_26 },
    { 0x501a1f52, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xbfa1f55, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x961a1f58, SUC_lexicon_tags_7 },
    { 0xdb7a1f58, SUC_lexicon_tags_2 },
    { 0x1dba1f5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf55a1f5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613a1f60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fa1f63, SUC_lexicon_tags_15 },
    { 0x5aba1f63, SUC_lexicon_tags_23 },
    { 0xd6fa1f65, SUC_lexicon_tags_25 },
    { 0xad5a1f64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5a1f6a, SUC_lexicon_tags_7 },
    { 0xcc1a1f6b, SUC_lexicon_tags_17 },
    { 0x185a1f6c, SUC_lexicon_tags_31 },
    { 0x159a1f6d, SUC_lexicon_tags_37 },
    { 0x719a1f6d, SUC_lexicon_tags_15 },
    { 0x5da1f6f, SUC_lexicon_tags_15 },
    { 0x851a1f70, SUC_lexicon_tags_7 },
    { 0x941a1f71, SUC_lexicon_tags_4 },
    { 0x33ba1f71, SUC_lexicon_tags_60 },
    { 0xbb9a1f6e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fa1f76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25da1f78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5a1f7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45a1f80, SUC_lexicon_tags_11 },
    { 0x3da1f81, SUC_lexicon_tags_29 },
    { 0xea3a1f82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3a1f88, SUC_lexicon_tags_61 },
    { 0xc7da1f89, SUC_lexicon_tags_33 },
    { 0x4b3a1f8a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x637a1f8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ba1f90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeda1f93, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x44fa1f95, SUC_lexicon_tags_7 },
    { 0xe6ba1f95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5a1f99, SUC_lexicon_tags_17 },
    { 0xd1fa1f99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705a1f9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bfa1fa0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfa1fa6, SUC_lexicon_tags_19 },
    { 0x7efa1fa6, SUC_lexicon_tags_0 },
    { 0xdc5a1fa8, SUC_lexicon_tags_47 },
    { 0x2dba1fa8, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x8dfa1fab, SUC_lexicon_tags_12 },
    { 0x8b5a1fac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5a1fb7, SUC_lexicon_tags_25 },
    { 0x3eda1fb8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9a1fbd, SUC_lexicon_tags_1 },
    { 0x2d3a1fbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ba1fc3, SUC_lexicon_tags_47 },
    { 0xe69a1fc4, SUC_lexicon_tags_68 },
    { 0x101a1fc5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46da1fc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb09a1fcb, SUC_lexicon_tags_37 },
    { 0xa71a1fcc, SUC_lexicon_tags_2 },
    { 0x7efa1fcc, SUC_lexicon_tags_37 },
    { 0xa8da1fce, SUC_lexicon_tags_25 },
    { 0xb55a1fcb, SUC_lexicon_tags_25 },
    { 0xfba1fcf, SUC_lexicon_tags_65 },
    { 0x3eba1fcb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3a1fd5, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fa1fdc, SUC_lexicon_tags_2 },
    { 0x971a1fdc, SUC_lexicon_tags_37 },
    { 0x6aba1fdc, SUC_lexicon_tags_77 },
    { 0x2f1a1fdf, SUC_lexicon_tags_65 },
    { 0xf43a1fe0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5a1fe3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27da1fe7, SUC_lexicon_tags_28 },
    { 0xc39a1fe8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fa1ff1, SUC_lexicon_tags_27 },
    { 0xf85a1ff1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x935a1ff5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9a1ffb, SUC_lexicon_tags_11 },
    { 0x54ba1ffc, SUC_lexicon_tags_53 },
    { 0x81ba1ffc, SUC_lexicon_tags_15 },
    { 0x325a1ffb, SUC_lexicon_tags_25 },
    { 0x1cfa1fff, SUC_lexicon_tags_37 },
    { 0x2d9a1ffd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605a200a, SUC_lexicon_tags_14 },
    { 0xe1da200a, SUC_lexicon_tags_57 },
    { 0x4a7a200a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x763a200e, SUC_lexicon_tags_28 },
    { 0x301a200f, SUC_lexicon_tags_82 },
    { 0x691a200f, SUC_lexicon_tags_12 },
    { 0xb41a2011, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffa2015, SUC_lexicon_tags_16 },
    { 0x7c5a2015, SUC_lexicon_tags_134 },
    { 0x8fa2016, SUC_lexicon_tags_7 },
    { 0xa35a2018, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a201b, SUC_lexicon_tags_17 },
    { 0xaafa201b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71fa201e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fa2025, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71a2029, SUC_lexicon_tags_0 },
    { 0x54da202a, SUC_lexicon_tags_15 },
    { 0x455a202a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53a2030, SUC_lexicon_tags_45 },
    { 0x10ba2031, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ada2035, SUC_lexicon_tags_7 },
    { 0xcb1a2036, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9a2039, SUC_lexicon_tags_25 },
    { 0xa07a203a, SUC_lexicon_tags_37 },
    { 0xb8da203b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38da203f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x769a2042, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83a2046, SUC_lexicon_tags_11 },
    { 0xa75a2047, SUC_lexicon_tags_15 },
    { 0xaaba2048, SUC_lexicon_tags_49 },
    { 0xea1a2048, SUC_lexicon_tags_7 },
    { 0xf73a204a, SUC_lexicon_tags_14 },
    { 0xf3a204b, SUC_lexicon_tags_45 },
    { 0x4c5a2047, SUC_lexicon_tags_55 },
    { 0x611a204d, SUC_lexicon_tags_14 },
    { 0xc2da204e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdba2051, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb3a2053, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fa2057, SUC_lexicon_tags_11 },
    { 0xfd3a2058, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xae1a205a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x739a205d, SUC_lexicon_tags_47 },
    { 0x5f5a205e, SUC_lexicon_tags_12 },
    { 0x821a205f, SUC_lexicon_tags_28 },
    { 0x4ada2060, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e9a2062, SUC_lexicon_tags_0 },
    { 0xf4da2062, SUC_lexicon_tags_95 },
    { 0x39a2064, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x857a2066, SUC_lexicon_tags_12 },
    { 0x2b7a2067, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697a206c, SUC_lexicon_tags_9 },
    { 0x555a206d, SUC_lexicon_tags_94 },
    { 0x787a206e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da2073, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fa2076, SUC_lexicon_tags_92 },
    { 0x241a2077, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x653a2079, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x66da207c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81a207f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x525a2081, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83a2085, SUC_lexicon_tags_95 },
    { 0x5d5a2085, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8fda2088, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc3a208a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309a208f, SUC_lexicon_tags_25 },
    { 0x29fa208f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ba2095, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73da209a, SUC_lexicon_tags_15 },
    { 0xec1a209b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9f5a209d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61a20a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd75a20a2, SUC_lexicon_tags_68 },
    { 0x69ba20a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11a20a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x791a20ac, SUC_lexicon_tags_84 },
    { 0xccda20ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc41a20af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x97fa20b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b7a20b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x271a20b5, SUC_lexicon_tags_38 },
    { 0x5e9a20b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43a20bc, SUC_lexicon_tags_37 },
    { 0xdffa20bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fba20c1, SUC_lexicon_tags_63 },
    { 0x979a20c2, SUC_lexicon_tags_8 },
    { 0xae1a20c3, SUC_lexicon_tags_12 },
    { 0xee1a20c4, SUC_lexicon_tags_25 },
    { 0xe07a20c5, SUC_lexicon_tags_11 },
    { 0xf35a20c6, SUC_lexicon_tags_25 },
    { 0xcfba20c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72da20d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x563a20d9, SUC_lexicon_tags_1 },
    { 0xb25a20da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eda20de, SUC_lexicon_tags_66 },
    { 0x4d1a20df, SUC_lexicon_tags_25 },
    { 0x427a20df, SUC_lexicon_tags_11 },
    { 0xdb5a20df, SUC_lexicon_tags_1 },
    { 0xff1a20e2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1eda20e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78fa20e6, SUC_lexicon_tags_47 },
    { 0xdc1a20e7, SUC_lexicon_tags_12 },
    { 0x483a20e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x159a20eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ba20f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fa20fa, SUC_lexicon_tags_28 },
    { 0x57ba20fb, SUC_lexicon_tags_0 },
    { 0x18fa20fc, SUC_lexicon_tags_37 },
    { 0x72fa20fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x469a2100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc57a2102, SUC_lexicon_tags_49 },
    { 0xeaba2103, SUC_lexicon_tags_14 },
    { 0x60fa2104, SUC_lexicon_tags_12 },
    { 0xbc3a2104, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15a210b, SUC_lexicon_tags_19 },
    { 0x53da210b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe4da210e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a2117, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5a211a, SUC_lexicon_tags_63 },
    { 0x9b5a211b, SUC_lexicon_tags_15 },
    { 0x4efa211c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75a211e, SUC_lexicon_tags_35 },
    { 0x8dda211e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ba2122, SUC_lexicon_tags_29 },
    { 0xf7da2123, SUC_lexicon_tags_49 },
    { 0xdba2123, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdba2127, SUC_lexicon_tags_1 },
    { 0x14da2127, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fa2134, SUC_lexicon_tags_12 },
    { 0x457a2135, SUC_lexicon_tags_11 },
    { 0x269a2136, SUC_lexicon_tags_84 },
    { 0x5b7a2135, SUC_lexicon_tags_17 },
    { 0xe09a2136, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9da213a, SUC_lexicon_tags_15 },
    { 0x337a213b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76da2140, SUC_lexicon_tags_37 },
    { 0x625a2140, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901a2149, SUC_lexicon_tags_11 },
    { 0x55da214a, SUC_lexicon_tags_27 },
    { 0x789a214b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7a2150, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3a2154, SUC_lexicon_tags_11 },
    { 0x855a2154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa0da2157, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x997a2159, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11a215c, SUC_lexicon_tags_7 },
    { 0x189a215d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957a2164, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7a2169, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x19a216c, SUC_lexicon_tags_55 },
    { 0x5c7a216d, SUC_lexicon_tags_37 },
    { 0x44fa216c, SUC_lexicon_tags_46 },
    { 0xb33a216e, SUC_lexicon_tags_46 },
    { 0x9bda216f, SUC_lexicon_tags_25 },
    { 0xf37a2170, SUC_lexicon_tags_39 },
    { 0xfda216e, SUC_lexicon_tags_68 },
    { 0x657a2173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c3a2175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd03a2177, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47da217d, SUC_lexicon_tags_38 },
    { 0xc65a217e, SUC_lexicon_tags_37 },
    { 0xc33a217f, SUC_lexicon_tags_2 },
    { 0xc89a217d, SUC_lexicon_tags_1 },
    { 0x6cfa217f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3a2185, SUC_lexicon_tags_7 },
    { 0xc91a2185, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aba2189, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x473a218c, SUC_lexicon_tags_1 },
    { 0xc25a218c, SUC_lexicon_tags_45 },
    { 0xd35a218e, SUC_lexicon_tags_1302 },
    { 0xd47a218e, SUC_lexicon_tags_37 },
    { 0xfeba218c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dda2193, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579a2198, SUC_lexicon_tags_1 },
    { 0x6afa2199, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7a21a1, SUC_lexicon_tags_8 },
    { 0xf29a21a2, SUC_lexicon_tags_7 },
    { 0x1e1a21a3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1da21a6, SUC_lexicon_tags_13 },
    { 0x3dfa21a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80da21b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42fa21b2, SUC_lexicon_tags_61 },
    { 0x86ba21b3, SUC_lexicon_tags_12 },
    { 0xb15a21b4, SUC_lexicon_tags_38 },
    { 0x217a21b4, SUC_lexicon_tags_55 },
    { 0xda3a21b5, SUC_lexicon_tags_27 },
    { 0x73ba21b7, SUC_lexicon_tags_25 },
    { 0x17fa21b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227a21bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31da21c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x495a21c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x369a21c4, SUC_lexicon_tags_25 },
    { 0xe8da21c5, SUC_lexicon_tags_28 },
    { 0x239a21c6, SUC_lexicon_tags_12 },
    { 0x663a21c7, SUC_lexicon_tags_38 },
    { 0x26ba21c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x493a21ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89a21d3, SUC_lexicon_tags_0 },
    { 0x431a21d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741a21d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509a21dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fa21df, SUC_lexicon_tags_1 },
    { 0xcd5a21e0, SUC_lexicon_tags_23 },
    { 0xff3a21e1, SUC_lexicon_tags_6 },
    { 0x553a21e2, SUC_lexicon_tags_15 },
    { 0xa91a21e3, SUC_lexicon_tags_2 },
    { 0x71fa21e2, SUC_lexicon_tags_11 },
    { 0x933a21e1, SUC_lexicon_tags_9 },
    { 0x2b9a21e6, SUC_lexicon_tags_12 },
    { 0x3f3a21e6, SUC_lexicon_tags_114 },
    { 0x2fda21e7, SUC_lexicon_tags_17 },
    { 0x2f9a21e9, SUC_lexicon_tags_23 },
    { 0x8e5a21e9, SUC_lexicon_tags_25 },
    { 0x977a21eb, SUC_lexicon_tags_104 },
    { 0x92ba21eb, SUC_lexicon_tags_47 },
    { 0x97a21e8, SUC_lexicon_tags_11 },
    { 0xba21ee, SUC_lexicon_tags_26 },
    { 0x7da21e9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1a21f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a21ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x955a2201, SUC_lexicon_tags_46 },
    { 0x84ba2201, SUC_lexicon_tags_23 },
    { 0x20da2203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fa2206, SUC_lexicon_tags_34 },
    { 0x643a2207, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe11a2209, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99fa220c, SUC_lexicon_tags_1 },
    { 0xb47a220c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffa2211, SUC_lexicon_tags_46 },
    { 0x509a2212, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x105a2215, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbd7a2217, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefa221b, SUC_lexicon_tags_12 },
    { 0xc0da221c, SUC_lexicon_tags_8 },
    { 0x1b9a221d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaefa221f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x833a2221, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5a2226, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xd39a2228, SUC_lexicon_tags_12 },
    { 0x5d1a2229, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85a222c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7a2235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9efa2238, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7a223e, SUC_lexicon_tags_7 },
    { 0xf59a223f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x469a2242, SUC_lexicon_tags_14 },
    { 0x61da2243, SUC_lexicon_tags_37 },
    { 0x4f1a2242, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91a2247, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1a224c, SUC_lexicon_tags_16 },
    { 0x6e5a224c, SUC_lexicon_tags_11 },
    { 0x249a224e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67a2254, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x971a2256, SUC_lexicon_tags_45 },
    { 0x16ba2256, SUC_lexicon_tags_43 },
    { 0x91fa2258, SUC_lexicon_tags_45 },
    { 0xbd1a2259, SUC_lexicon_tags_14 },
    { 0x597a225a, SUC_lexicon_tags_11 },
    { 0xd55a225b, SUC_lexicon_tags_12 },
    { 0xf29a2258, SUC_lexicon_tags_14 },
    { 0x167a2258, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31a225f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21a2263, SUC_lexicon_tags_197 },
    { 0x26ba2264, SUC_lexicon_tags_38 },
    { 0xfdda2265, SUC_lexicon_tags_77 },
    { 0x3cfa2264, SUC_lexicon_tags_25 },
    { 0x7ada2267, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fda226f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xefa2271, SUC_lexicon_tags_25 },
    { 0x5a2272, SUC_lexicon_tags_26 },
    { 0x88fa2273, SUC_lexicon_tags_112 },
    { 0x707a2274, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e1a2276, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319a227b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1a227e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15da2282, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b7a2284, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fa2287, SUC_lexicon_tags_1 },
    { 0x437a2288, SUC_lexicon_tags_31 },
    { 0x709a2288, SUC_lexicon_tags_17 },
    { 0xc1fa228a, SUC_lexicon_tags_46 },
    { 0x87ba2287, SUC_lexicon_tags_1 },
    { 0xc0da228c, SUC_lexicon_tags_46 },
    { 0x20da2289, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fa2290, SUC_lexicon_tags_15 },
    { 0xab3a2291, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffda2296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83a2298, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9c1a229a, SUC_lexicon_tags_26 },
    { 0x8d5a229b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbda229d, SUC_lexicon_tags_20 },
    { 0xb71a229e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cba22a1, SUC_lexicon_tags_7 },
    { 0x17ba22a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfe3a22a4, SUC_lexicon_tags_37 },
    { 0xf89a22a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ba22aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5a22af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd3da22b1, SUC_lexicon_tags_38 },
    { 0xb45a22b2, SUC_lexicon_tags_37 },
    { 0xc23a22b3, SUC_lexicon_tags_19 },
    { 0xd87a22b4, SUC_lexicon_tags_17 },
    { 0xe33a22b5, SUC_lexicon_tags_37 },
    { 0xf11a22b5, SUC_lexicon_tags_25 },
    { 0x261a22b5, SUC_lexicon_tags_11 },
    { 0x77fa22b1, SUC_lexicon_tags_63 },
    { 0xf57a22b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b7a22bb, SUC_lexicon_tags_6 },
    { 0xeb1a22bc, SUC_lexicon_tags_15 },
    { 0xa81a22bd, SUC_lexicon_tags_11 },
    { 0x15fa22be, SUC_lexicon_tags_37 },
    { 0x1b3a22bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ba22c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83a22c7, SUC_lexicon_tags_12 },
    { 0x75a22c8, SUC_lexicon_tags_9 },
    { 0x183a22c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395a22cd, SUC_lexicon_tags_55 },
    { 0x59a22cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07a22d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10da22d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03a22da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741a22e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x803a22e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x23da22e5, SUC_lexicon_tags_16 },
    { 0xdc5a22e5, SUC_lexicon_tags_23 },
    { 0x763a22e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657a22eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd09a22ed, SUC_lexicon_tags_37 },
    { 0xf7ba22ee, SUC_lexicon_tags_21 },
    { 0x75a22ef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x13ba22f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467a22f6, SUC_lexicon_tags_17 },
    { 0xa0fa22f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf9a22f9, SUC_lexicon_tags_7 },
    { 0xdd7a22f9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd5da22fc, SUC_lexicon_tags_26 },
    { 0x3b7a22fd, SUC_lexicon_tags_2 },
    { 0x47fa22fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71da2307, SUC_lexicon_tags_61 },
    { 0x70fa2308, SUC_lexicon_tags_11 },
    { 0x737a2308, SUC_lexicon_tags_12 },
    { 0x4da2309, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665a230e, SUC_lexicon_tags_21 },
    { 0x72da230f, SUC_lexicon_tags_38 },
    { 0x34fa2310, SUC_lexicon_tags_47 },
    { 0x65fa230f, SUC_lexicon_tags_64 },
    { 0x53a230e, SUC_lexicon_tags_60 },
    { 0xfd3a2313, SUC_lexicon_tags_7 },
    { 0x6efa2314, SUC_lexicon_tags_17 },
    { 0xaffa230e, SUC_lexicon_tags_11 },
    { 0xa9ba2316, SUC_lexicon_tags_11 },
    { 0xd7a2311, SUC_lexicon_tags_2 },
    { 0x59ba2318, SUC_lexicon_tags_7 },
    { 0x86da2319, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83da231b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7a231f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309a2324, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x873a2326, SUC_lexicon_tags_29 },
    { 0xe1ba2326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941a232d, SUC_lexicon_tags_30 },
    { 0x3e1a232e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63a2331, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc6da2333, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fa2336, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x743a2339, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6da233f, SUC_lexicon_tags_0 },
    { 0x56fa233f, SUC_lexicon_tags_21 },
    { 0x805a2341, SUC_lexicon_tags_25 },
    { 0x719a2342, SUC_lexicon_tags_37 },
    { 0x499a2340, SUC_lexicon_tags_37 },
    { 0x6b5a2344, SUC_lexicon_tags_2 },
    { 0x695a233f, SUC_lexicon_tags_43 },
    { 0x9aba2346, SUC_lexicon_tags_27 },
    { 0x181a2345, SUC_lexicon_tags_21 },
    { 0xa7a2348, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383a234c, SUC_lexicon_tags_138 },
    { 0xad1a234d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3a2354, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fa2358, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43a235b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeba2362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487a2366, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135a236d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x645a236f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885a2373, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d3a2375, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03a237d, SUC_lexicon_tags_14 },
    { 0x6da237d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfa2381, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf7fa2383, SUC_lexicon_tags_37 },
    { 0x90da2384, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd71a2386, SUC_lexicon_tags_43 },
    { 0x9c3a2387, SUC_lexicon_tags_13 },
    { 0x31fa2388, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57a238b, SUC_lexicon_tags_45 },
    { 0x3aba238b, SUC_lexicon_tags_1 },
    { 0xe43a238d, SUC_lexicon_tags_60 },
    { 0x90fa238b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7ba2390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9a2394, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915a239b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481a239f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x98da23a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65a23a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x419a23a9, SUC_lexicon_tags_12 },
    { 0x7d7a23a9, SUC_lexicon_tags_127 },
    { 0x57a23ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb2ba23ad, SUC_lexicon_tags_1 },
    { 0x2eba23ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x993a23b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcfa23b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259a23b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x18da23b9, SUC_lexicon_tags_28 },
    { 0xb73a23ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x39a23bc, SUC_lexicon_tags_2 },
    { 0x151a23bd, SUC_lexicon_tags_23 },
    { 0x19fa23be, SUC_lexicon_tags_37 },
    { 0x1bda23bf, SUC_lexicon_tags_63 },
    { 0x91ba23c0, SUC_lexicon_tags_11 },
    { 0x49da23c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x923a23c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671a23c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x987a23ca, SUC_lexicon_tags_7 },
    { 0xa5ba23ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ba23ce, SUC_lexicon_tags_26 },
    { 0x4e9a23ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccba23d3, SUC_lexicon_tags_6 },
    { 0x4eba23d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaf7a23d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74da23db, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x251a23dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fa23e0, SUC_lexicon_tags_15 },
    { 0x13ba23e1, SUC_lexicon_tags_10 },
    { 0x871a23e0, SUC_lexicon_tags_45 },
    { 0xaa7a23e1, SUC_lexicon_tags_47 },
    { 0x6bfa23e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1a23ed, SUC_lexicon_tags_25 },
    { 0xa1da23ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc53a23f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47a23f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddda23f6, SUC_lexicon_tags_38 },
    { 0xf7a23f6, SUC_lexicon_tags_2 },
    { 0x1c7a23f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5a23fd, SUC_lexicon_tags_30 },
    { 0x4e3a23fe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ba2402, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x439a2404, SUC_lexicon_tags_27 },
    { 0x181a2405, SUC_lexicon_tags_43 },
    { 0x8cba2406, SUC_lexicon_tags_1 },
    { 0xa91a2407, SUC_lexicon_tags_37 },
    { 0xa21a2404, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fa240e, SUC_lexicon_tags_7 },
    { 0x99fa240e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xba1a2411, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaba2415, SUC_lexicon_tags_34 },
    { 0x881a2415, SUC_lexicon_tags_45 },
    { 0x4bda2417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ba241b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5a2422, SUC_lexicon_tags_17 },
    { 0x205a2423, SUC_lexicon_tags_12 },
    { 0x3d3a2422, SUC_lexicon_tags_31 },
    { 0xd59a2425, SUC_lexicon_tags_26 },
    { 0x827a2426, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x23fa2428, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22da242a, SUC_lexicon_tags_45 },
    { 0x581a242a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x671a242d, SUC_lexicon_tags_12 },
    { 0x88da242d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa07a2430, SUC_lexicon_tags_2 },
    { 0xcaba2431, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x503a2434, SUC_lexicon_tags_1 },
    { 0x3dda2435, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd6da2437, SUC_lexicon_tags_16 },
    { 0xf5da2437, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xcf1a243a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xba9a243c, SUC_lexicon_tags_11 },
    { 0x2b9a243d, SUC_lexicon_tags_791 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2da2440, SUC_lexicon_tags_39 },
    { 0x76ba2441, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5a2446, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x329a2449, SUC_lexicon_tags_11 },
    { 0x429a244a, SUC_lexicon_tags_12 },
    { 0xdafa244b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcdda244d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xac9a244f, SUC_lexicon_tags_23 },
    { 0xf39a2450, SUC_lexicon_tags_37 },
    { 0x53a2450, SUC_lexicon_tags_1 },
    { 0xfeda2452, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa1ba2454, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ba245d, SUC_lexicon_tags_12 },
    { 0x5a9a245d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x693a2460, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a2464, SUC_lexicon_tags_46 },
    { 0x2ffa2464, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7a2468, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0xdeba246b, SUC_lexicon_tags_1 },
    { 0x2b9a246c, SUC_lexicon_tags_25 },
    { 0xe2ba246c, SUC_lexicon_tags_21 },
    { 0x635a246b, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43a2471, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8e3a2473, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07a2477, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05a247f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a1a2481, SUC_lexicon_tags_37 },
    { 0x935a2482, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823a2486, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbb9a2488, SUC_lexicon_tags_45 },
    { 0xef3a2488, SUC_lexicon_tags_295 },
    { 0xa8fa2488, SUC_lexicon_tags_46 },
    { 0x3c9a2488, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9a248e, SUC_lexicon_tags_30 },
    { 0xe0fa248e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffa2494, SUC_lexicon_tags_66 },
    { 0x157a2494, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc87a2497, SUC_lexicon_tags_33 },
    { 0xb89a2498, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ba24a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ba24a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eba24a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c3a24a8, SUC_lexicon_tags_31 },
    { 0x82ba24a9, SUC_lexicon_tags_19 },
    { 0x43da24a9, SUC_lexicon_tags_2 },
    { 0x6dfa24ab, SUC_lexicon_tags_38 },
    { 0x7b9a24ac, SUC_lexicon_tags_25 },
    { 0xeda24ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x481a24b0, SUC_lexicon_tags_60 },
    { 0xc41a24b1, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa24ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5a24bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb8fa24bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fa24c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3a24c9, SUC_lexicon_tags_6 },
    { 0xe0da24ca, SUC_lexicon_tags_15 },
    { 0x743a24c9, SUC_lexicon_tags_0 },
    { 0x6f7a24cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165a24d5, SUC_lexicon_tags_184 },
    { 0x687a24d5, SUC_lexicon_tags_87 },
    { 0x181a24d7, SUC_lexicon_tags_7 },
    { 0x673a24d8, SUC_lexicon_tags_60 },
    { 0x5c5a24d8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbba24dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x787a24df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc3a24e1, SUC_lexicon_tags_9 },
    { 0x915a24e1, SUC_lexicon_tags_33 },
    { 0x4b9a24e3, SUC_lexicon_tags_60 },
    { 0x5c1a24e4, SUC_lexicon_tags_63 },
    { 0x97fa24e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7a24ee, SUC_lexicon_tags_11 },
    { 0x4da24ef, SUC_lexicon_tags_10 },
    { 0x4dda24f0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27a24f4, SUC_lexicon_tags_19 },
    { 0x92ba24f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3a24fc, SUC_lexicon_tags_45 },
    { 0x389a24fd, SUC_lexicon_tags_1 },
    { 0xba7a24fd, SUC_lexicon_tags_29 },
    { 0xbeda24fe, SUC_lexicon_tags_0 },
    { 0x9efa24fd, SUC_lexicon_tags_25 },
    { 0x33fa2501, SUC_lexicon_tags_10 },
    { 0xaeba2502, SUC_lexicon_tags_14 },
    { 0x4cfa2500, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfa2507, SUC_lexicon_tags_7 },
    { 0x5eba2508, SUC_lexicon_tags_23 },
    { 0xd47a2508, SUC_lexicon_tags_17 },
    { 0xc67a250a, SUC_lexicon_tags_17 },
    { 0xd15a2507, SUC_lexicon_tags_42 },
    { 0xcaba250c, SUC_lexicon_tags_37 },
    { 0xcffa250b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x737a250f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x779a2511, SUC_lexicon_tags_11 },
    { 0x465a2512, SUC_lexicon_tags_488 },
    { 0, NULL },
    { 0x8c3a2514, SUC_lexicon_tags_20 },
    { 0xa51a2514, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36da2518, SUC_lexicon_tags_38 },
    { 0xdb3a2519, SUC_lexicon_tags_31 },
    { 0xc9a251a, SUC_lexicon_tags_47 },
    { 0xf81a251b, SUC_lexicon_tags_43 },
    { 0x759a2519, SUC_lexicon_tags_37 },
    { 0x963a251b, SUC_lexicon_tags_57 },
    { 0xf07a251d, SUC_lexicon_tags_26 },
    { 0xda7a251d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x825a2522, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7a2528, SUC_lexicon_tags_26 },
    { 0x5e7a2529, SUC_lexicon_tags_17 },
    { 0xefa2528, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1a252d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x253a252f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb7a2531, SUC_lexicon_tags_12 },
    { 0xd99a2532, SUC_lexicon_tags_15 },
    { 0xfafa2532, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399a2541, SUC_lexicon_tags_19 },
    { 0x19a2542, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x701a2545, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eda2548, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ba254c, SUC_lexicon_tags_12 },
    { 0x3a254d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x465a254f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1a2553, SUC_lexicon_tags_39 },
    { 0xe69a2553, SUC_lexicon_tags_25 },
    { 0xb59a2554, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x229a2558, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fa2560, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc4fa2562, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a2565, SUC_lexicon_tags_46 },
    { 0xdbba2566, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45a2568, SUC_lexicon_tags_25 },
    { 0xe45a2568, SUC_lexicon_tags_11 },
    { 0x397a2568, SUC_lexicon_tags_17 },
    { 0xbe7a256b, SUC_lexicon_tags_33 },
    { 0x95ba256c, SUC_lexicon_tags_17 },
    { 0x50ba256d, SUC_lexicon_tags_43 },
    { 0x1eda256a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a2573, SUC_lexicon_tags_12 },
    { 0x8a5a2573, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fa2578, SUC_lexicon_tags_39 },
    { 0xb43a2579, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd59a257b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679a2580, SUC_lexicon_tags_15 },
    { 0xdbfa2580, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ba258b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7a258f, SUC_lexicon_tags_26 },
    { 0x789a2590, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557a2594, SUC_lexicon_tags_3 },
    { 0xdcda2595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69fa2597, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb65a2599, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ba259d, SUC_lexicon_tags_12 },
    { 0xb3a259e, SUC_lexicon_tags_17 },
    { 0x80ba259f, SUC_lexicon_tags_25 },
    { 0xf4ba259f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97a25a6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x817a25a9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3a25ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7da25af, SUC_lexicon_tags_25 },
    { 0xe71a25b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fa25b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x685a25b7, SUC_lexicon_tags_7 },
    { 0x28ba25b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65a25bb, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xba7a25bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfa25c1, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3a25c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6afa25c6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93a25cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b9a25cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe81a25cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129a25d5, SUC_lexicon_tags_7 },
    { 0xc75a25d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x163a25d9, SUC_lexicon_tags_38 },
    { 0xaf5a25da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0ba25dc, SUC_lexicon_tags_37 },
    { 0xcf9a25dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ba25e1, SUC_lexicon_tags_7 },
    { 0x933a25e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67a25e8, SUC_lexicon_tags_14 },
    { 0xd4ba25e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fa25ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf79a25ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7a25f2, SUC_lexicon_tags_87 },
    { 0x15a25f3, SUC_lexicon_tags_11 },
    { 0x4bfa25f4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x49da25f6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5a25fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89a25ff, SUC_lexicon_tags_1 },
    { 0x4e1a2600, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe61a2602, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a2606, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fba260a, SUC_lexicon_tags_15 },
    { 0xd05a260b, SUC_lexicon_tags_15 },
    { 0x5d5a260c, SUC_lexicon_tags_29 },
    { 0x221a260a, SUC_lexicon_tags_23 },
    { 0x50da260e, SUC_lexicon_tags_7 },
    { 0xb69a260f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789a2613, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3a2616, SUC_lexicon_tags_7 },
    { 0x9a5a2616, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x555a2619, SUC_lexicon_tags_38 },
    { 0x6a5a261a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeda261c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xdd7a261e, SUC_lexicon_tags_11 },
    { 0x847a261f, SUC_lexicon_tags_17 },
    { 0x463a2620, SUC_lexicon_tags_402 },
    { 0x1cba261f, SUC_lexicon_tags_1 },
    { 0x367a2622, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x93da2624, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x127a2627, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x355a2629, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x67da262b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ba2632, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x295a2634, SUC_lexicon_tags_38 },
    { 0x1b7a2635, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x77da2638, SUC_lexicon_tags_7 },
    { 0x9b9a2639, SUC_lexicon_tags_553 },
    { 0xaf1a263a, SUC_lexicon_tags_37 },
    { 0x40ba2639, SUC_lexicon_tags_37 },
    { 0xdfba263c, SUC_lexicon_tags_37 },
    { 0xb49a263d, SUC_lexicon_tags_37 },
    { 0x7bda263d, SUC_lexicon_tags_92 },
    { 0xb3a263f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5a2642, SUC_lexicon_tags_9 },
    { 0x29fa2642, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd07a2645, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7a2648, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41a264e, SUC_lexicon_tags_15 },
    { 0x933a264e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x923a2652, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3a2656, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391a265b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97fa265d, SUC_lexicon_tags_37 },
    { 0x2eda265e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbda2661, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x68da2664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07a2669, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5d9a266b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ba266f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31a2673, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09a2678, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x495a267a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7a2680, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf47a2682, SUC_lexicon_tags_2 },
    { 0x8e9a2683, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8da268a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x335a268c, SUC_lexicon_tags_55 },
    { 0xa9a268d, SUC_lexicon_tags_17 },
    { 0x1ada268d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71da2690, SUC_lexicon_tags_54 },
    { 0xc3da2691, SUC_lexicon_tags_47 },
    { 0xd9da2691, SUC_lexicon_tags_32 },
    { 0x4bba2690, SUC_lexicon_tags_15 },
    { 0x341a2692, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90ba2698, SUC_lexicon_tags_7 },
    { 0x8dfa2699, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a269c, SUC_lexicon_tags_37 },
    { 0x9b7a269c, SUC_lexicon_tags_7 },
    { 0x2b1a269d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a26a2, SUC_lexicon_tags_25 },
    { 0xe15a26a2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601a26af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcd1a26b1, SUC_lexicon_tags_33 },
    { 0xa4fa26b1, SUC_lexicon_tags_45 },
    { 0xa5da26b3, SUC_lexicon_tags_55 },
    { 0xceda26b4, SUC_lexicon_tags_39 },
    { 0x68da26b3, SUC_lexicon_tags_12 },
    { 0xf93a26b6, SUC_lexicon_tags_9 },
    { 0x731a26b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb11a26b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x121a26bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x89ba26bd, SUC_lexicon_tags_60 },
    { 0x1cba26be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eba26c3, SUC_lexicon_tags_37 },
    { 0x225a26c4, SUC_lexicon_tags_33 },
    { 0x525a26c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba26c9, SUC_lexicon_tags_63 },
    { 0x749a26ca, SUC_lexicon_tags_106 },
    { 0xbc9a26ca, SUC_lexicon_tags_351 },
    { 0x295a26cb, SUC_lexicon_tags_25 },
    { 0x7a1a26c9, SUC_lexicon_tags_17 },
    { 0xf89a26ce, SUC_lexicon_tags_48 },
    { 0x651a26ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ada26d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x619a26d3, SUC_lexicon_tags_43 },
    { 0xeb3a26d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb45a26d6, SUC_lexicon_tags_1 },
    { 0xf01a26d6, SUC_lexicon_tags_37 },
    { 0x997a26d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1a26e1, SUC_lexicon_tags_15 },
    { 0x545a26e1, SUC_lexicon_tags_11 },
    { 0x415a26e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75da26e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabda26e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995a26ef, SUC_lexicon_tags_25 },
    { 0x11da26f0, SUC_lexicon_tags_29 },
    { 0x87da26f0, SUC_lexicon_tags_25 },
    { 0x3cfa26ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9da26f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57a26f8, SUC_lexicon_tags_60 },
    { 0xafa26f9, SUC_lexicon_tags_87 },
    { 0x6efa26f8, SUC_lexicon_tags_77 },
    { 0x315a26f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cba2701, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55a2705, SUC_lexicon_tags_25 },
    { 0xb03a2706, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfa270c, SUC_lexicon_tags_7 },
    { 0x88ba270d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x383a270f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1a2718, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593a271d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcefa271f, SUC_lexicon_tags_2 },
    { 0x153a271f, SUC_lexicon_tags_25 },
    { 0x16da2721, SUC_lexicon_tags_14 },
    { 0x6cda2722, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3a2725, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723a2729, SUC_lexicon_tags_45 },
    { 0x805a272a, SUC_lexicon_tags_35 },
    { 0x57fa272a, SUC_lexicon_tags_27 },
    { 0x713a272b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f5a272e, SUC_lexicon_tags_0 },
    { 0x95ba272f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38da2735, SUC_lexicon_tags_26 },
    { 0x21fa2736, SUC_lexicon_tags_27 },
    { 0x133a2735, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfa2740, SUC_lexicon_tags_26 },
    { 0xd39a2741, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d3a2743, SUC_lexicon_tags_23 },
    { 0x5c7a2744, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85a274a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85a2750, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37a2757, SUC_lexicon_tags_32 },
    { 0xdda2758, SUC_lexicon_tags_75 },
    { 0xc5ba2759, SUC_lexicon_tags_47 },
    { 0x5bba275a, SUC_lexicon_tags_2 },
    { 0xa87a2758, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f3a275d, SUC_lexicon_tags_25 },
    { 0xdada275e, SUC_lexicon_tags_25 },
    { 0x921a275f, SUC_lexicon_tags_2 },
    { 0x277a2760, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdda2765, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189a276b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e9a276d, SUC_lexicon_tags_25 },
    { 0x5efa276e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x255a2770, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed1a2772, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd51a2774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd89a2776, SUC_lexicon_tags_47 },
    { 0xf49a2777, SUC_lexicon_tags_77 },
    { 0x5bfa2778, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865a277c, SUC_lexicon_tags_37 },
    { 0x37ba277d, SUC_lexicon_tags_19 },
    { 0xd85a277d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1da2781, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03a278c, SUC_lexicon_tags_11 },
    { 0x54da278d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aba2791, SUC_lexicon_tags_39 },
    { 0xb6ba2791, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3a2798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba279c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fa279f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7a27a5, SUC_lexicon_tags_82 },
    { 0x34fa27a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341a27b1, SUC_lexicon_tags_59 },
    { 0x253a27b2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369a27b6, SUC_lexicon_tags_31 },
    { 0xe4da27b7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x201a27b9, SUC_lexicon_tags_25 },
    { 0xc2fa27ba, SUC_lexicon_tags_10 },
    { 0x731a27bb, SUC_lexicon_tags_6 },
    { 0x543a27b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5a27bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709a27cb, SUC_lexicon_tags_25 },
    { 0x4bba27cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcfa27ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdb7a27d0, SUC_lexicon_tags_47 },
    { 0xd39a27d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d9a27d3, SUC_lexicon_tags_35 },
    { 0x33fa27d3, SUC_lexicon_tags_63 },
    { 0x5f1a27d5, SUC_lexicon_tags_43 },
    { 0xedfa27d5, SUC_lexicon_tags_15 },
    { 0xf1ba27d7, SUC_lexicon_tags_16 },
    { 0xeb5a27d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25a27e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fa27e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ba27ea, SUC_lexicon_tags_25 },
    { 0x1b7a27eb, SUC_lexicon_tags_33 },
    { 0xa81a27ea, SUC_lexicon_tags_1 },
    { 0xeafa27eb, SUC_lexicon_tags_36 },
    { 0x2cba27ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9a27f2, SUC_lexicon_tags_10 },
    { 0x14da27f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeda27f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a27fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21a2800, SUC_lexicon_tags_66 },
    { 0xcd7a2801, SUC_lexicon_tags_2 },
    { 0x8f9a2801, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25a2806, SUC_lexicon_tags_15 },
    { 0xfaba2806, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9fa2809, SUC_lexicon_tags_103 },
    { 0x6a1a280a, SUC_lexicon_tags_55 },
    { 0x171a280a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7a280e, SUC_lexicon_tags_87 },
    { 0x6dda280f, SUC_lexicon_tags_12 },
    { 0xea3a2810, SUC_lexicon_tags_117 },
    { 0x495a280f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3a2814, SUC_lexicon_tags_15 },
    { 0xa3ba2815, SUC_lexicon_tags_2 },
    { 0xc09a2814, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xde7a2818, SUC_lexicon_tags_7 },
    { 0x599a2819, SUC_lexicon_tags_27 },
    { 0x58ba281a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcba281f, SUC_lexicon_tags_49 },
    { 0x935a2820, SUC_lexicon_tags_17 },
    { 0x375a281f, SUC_lexicon_tags_8 },
    { 0x4ffa2822, SUC_lexicon_tags_15 },
    { 0x34da2823, SUC_lexicon_tags_17 },
    { 0x8bba2824, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9a2828, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fa282c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7fa282e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb83a2830, SUC_lexicon_tags_39 },
    { 0x621a2831, SUC_lexicon_tags_116 },
    { 0xeb7a2830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ba2838, SUC_lexicon_tags_29 },
    { 0x249a2839, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33a283e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa2841, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01a2848, SUC_lexicon_tags_14 },
    { 0x563a2849, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13da284f, SUC_lexicon_tags_1 },
    { 0xb05a2850, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x155a2852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf09a2854, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd59a2856, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9a2859, SUC_lexicon_tags_11 },
    { 0xc8da285a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x88ba285c, SUC_lexicon_tags_7 },
    { 0xb61a285c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eba2860, SUC_lexicon_tags_37 },
    { 0x5c7a2861, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911a2867, SUC_lexicon_tags_1 },
    { 0x215a2867, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5a286b, SUC_lexicon_tags_2 },
    { 0x50fa286c, SUC_lexicon_tags_27 },
    { 0xf6fa286d, SUC_lexicon_tags_2 },
    { 0xc4da286e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x807a2871, SUC_lexicon_tags_64 },
    { 0x8e7a2872, SUC_lexicon_tags_1 },
    { 0xcf1a2871, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ba2877, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x473a2879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd5a287b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a1a287d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x753a287f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6da2882, SUC_lexicon_tags_47 },
    { 0xcb9a2883, SUC_lexicon_tags_25 },
    { 0x477a2882, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x573a2887, SUC_lexicon_tags_37 },
    { 0x8fa2888, SUC_lexicon_tags_1 },
    { 0xa75a2889, SUC_lexicon_tags_11 },
    { 0xd65a2889, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fba288d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9a2890, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871a2895, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947a2899, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3a289f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfa28a6, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1a28aa, SUC_lexicon_tags_14 },
    { 0xf0fa28ab, SUC_lexicon_tags_1 },
    { 0xf9da28ac, SUC_lexicon_tags_46 },
    { 0x84ba28ad, SUC_lexicon_tags_48 },
    { 0x461a28ad, SUC_lexicon_tags_1 },
    { 0x455a28af, SUC_lexicon_tags_7 },
    { 0xecda28b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7a28b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bba28ba, SUC_lexicon_tags_45 },
    { 0x977a28bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69a28be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb95a28c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127a28c6, SUC_lexicon_tags_63 },
    { 0x9afa28c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x385a28ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bba28cc, SUC_lexicon_tags_116 },
    { 0x827a28cc, SUC_lexicon_tags_49 },
    { 0xab9a28ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc39a28d0, SUC_lexicon_tags_47 },
    { 0xb39a28d1, SUC_lexicon_tags_21 },
    { 0x43ba28d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf13a28d4, SUC_lexicon_tags_7 },
    { 0xfe7a28d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1a28da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbffa28dc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ba28e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197a28e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fa28ed, SUC_lexicon_tags_1 },
    { 0x35a28ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aba28f2, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xd61a28f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309a28fb, SUC_lexicon_tags_12 },
    { 0x341a28fb, SUC_lexicon_tags_33 },
    { 0xc97a28fd, SUC_lexicon_tags_62 },
    { 0x311a28fd, SUC_lexicon_tags_17 },
    { 0x1b5a28ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3a2902, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88ba2904, SUC_lexicon_tags_11 },
    { 0xe95a2904, SUC_lexicon_tags_12 },
    { 0x831a2905, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1a2911, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x77ba2913, SUC_lexicon_tags_15 },
    { 0x9bba2914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9a291d, SUC_lexicon_tags_6 },
    { 0xfaba291e, SUC_lexicon_tags_0 },
    { 0x2ba291d, SUC_lexicon_tags_17 },
    { 0xa69a2920, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16da2925, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41da292d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5a2930, SUC_lexicon_tags_43 },
    { 0xf73a2930, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ba2935, SUC_lexicon_tags_15 },
    { 0xefda2935, SUC_lexicon_tags_4 },
    { 0x4cda2937, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1a293c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x31da293f, SUC_lexicon_tags_7 },
    { 0x689a2940, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb15a2942, SUC_lexicon_tags_37 },
    { 0xed5a2943, SUC_lexicon_tags_84 },
    { 0xf43a2944, SUC_lexicon_tags_75 },
    { 0x867a2942, SUC_lexicon_tags_148 },
    { 0x797a2946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41a2949, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x901a294c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ba2953, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9a2955, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15a2958, SUC_lexicon_tags_37 },
    { 0xac9a2958, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x797a295c, SUC_lexicon_tags_16 },
    { 0x7f9a295c, SUC_lexicon_tags_27 },
    { 0x4e3a295d, SUC_lexicon_tags_37 },
    { 0xa15a295f, SUC_lexicon_tags_37 },
    { 0x6d9a295f, SUC_lexicon_tags_16 },
    { 0xad1a2960, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61da296e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743a2972, SUC_lexicon_tags_33 },
    { 0x327a2972, SUC_lexicon_tags_70 },
    { 0x7eda2974, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923a297a, SUC_lexicon_tags_25 },
    { 0xa5da297b, SUC_lexicon_tags_45 },
    { 0x307a297b, SUC_lexicon_tags_1 },
    { 0x61da297b, SUC_lexicon_tags_17 },
    { 0x1e9a297e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57a2984, SUC_lexicon_tags_33 },
    { 0x655a2984, SUC_lexicon_tags_68 },
    { 0x8a9a2986, SUC_lexicon_tags_15 },
    { 0x6d3a2985, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x665a2989, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83a298e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bba2990, SUC_lexicon_tags_37 },
    { 0xdaba2990, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4da2994, SUC_lexicon_tags_25 },
    { 0x5bba2995, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b3a2997, SUC_lexicon_tags_15 },
    { 0xa81a2997, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451a29a0, SUC_lexicon_tags_92 },
    { 0x305a29a0, SUC_lexicon_tags_10 },
    { 0x40ba29a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f3a29a4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x821a29a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x72ba29a9, SUC_lexicon_tags_37 },
    { 0x175a29a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x779a29ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79a29b3, SUC_lexicon_tags_0 },
    { 0xa33a29b3, SUC_lexicon_tags_14 },
    { 0x55da29b5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadba29b9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591a29bd, SUC_lexicon_tags_4 },
    { 0x125a29be, SUC_lexicon_tags_7 },
    { 0xc9ba29be, SUC_lexicon_tags_0 },
    { 0x53fa29bd, SUC_lexicon_tags_87 },
    { 0x1da29c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa45a29c3, SUC_lexicon_tags_37 },
    { 0xe43a29c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83a29c8, SUC_lexicon_tags_10 },
    { 0x629a29c8, SUC_lexicon_tags_0 },
    { 0x5fa29c8, SUC_lexicon_tags_17 },
    { 0xabba29cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55a29ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27a29d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ba29d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd23a29d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34fa29d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x155a29db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ba29de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31ba29e0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28da29e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7a29e8, SUC_lexicon_tags_17 },
    { 0x7d9a29e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddba29ee, SUC_lexicon_tags_12 },
    { 0xefa29ee, SUC_lexicon_tags_47 },
    { 0xbafa29f0, SUC_lexicon_tags_8 },
    { 0x5e3a29f0, SUC_lexicon_tags_10 },
    { 0xa3da29f2, SUC_lexicon_tags_37 },
    { 0xbf3a29f2, SUC_lexicon_tags_15 },
    { 0x2eba29f4, SUC_lexicon_tags_60 },
    { 0xd39a29f1, SUC_lexicon_tags_7 },
    { 0xd85a29f2, SUC_lexicon_tags_0 },
    { 0xb6fa29f2, SUC_lexicon_tags_68 },
    { 0x4a9a29f5, SUC_lexicon_tags_48 },
    { 0x7a3a29f9, SUC_lexicon_tags_45 },
    { 0x84fa29fa, SUC_lexicon_tags_25 },
    { 0xc01a29f8, SUC_lexicon_tags_47 },
    { 0xadda29f6, SUC_lexicon_tags_17 },
    { 0xc81a29f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753a2a04, SUC_lexicon_tags_12 },
    { 0x75da2a04, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35a2a0c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x71da2a0e, SUC_lexicon_tags_2 },
    { 0x3d1a2a0f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xbd3a2a11, SUC_lexicon_tags_25 },
    { 0xb67a2a12, SUC_lexicon_tags_63 },
    { 0x669a2a13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19a2a17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x37fa2a1a, SUC_lexicon_tags_31 },
    { 0xbb9a2a1b, SUC_lexicon_tags_37 },
    { 0x565a2a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bba2a1e, SUC_lexicon_tags_11 },
    { 0xad5a2a1e, SUC_lexicon_tags_85 },
    { 0x3c1a2a20, SUC_lexicon_tags_2 },
    { 0x32fa2a21, SUC_lexicon_tags_25 },
    { 0xa67a2a22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x977a2a24, SUC_lexicon_tags_6 },
    { 0x2b5a2a25, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xf5a2a27, SUC_lexicon_tags_11 },
    { 0x1cba2a28, SUC_lexicon_tags_28 },
    { 0x911a2a29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe6fa2a2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157a2a2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a2a34, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaba2a3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387a2a40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3a2a4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271a2a50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da2a56, SUC_lexicon_tags_14 },
    { 0xa6fa2a57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc51a2a59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fa2a5b, SUC_lexicon_tags_27 },
    { 0x8eda2a5c, SUC_lexicon_tags_38 },
    { 0x65da2a5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x515a2a60, SUC_lexicon_tags_37 },
    { 0xdc7a2a60, SUC_lexicon_tags_17 },
    { 0x289a2a62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x475a2a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x397a2a66, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09a2a6a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1a2a6f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf5fa2a71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x87a2a74, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8dfa2a76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5a2a7a, SUC_lexicon_tags_37 },
    { 0x113a2a7a, SUC_lexicon_tags_14 },
    { 0xce7a2a7b, SUC_lexicon_tags_11 },
    { 0xd49a2a7b, SUC_lexicon_tags_27 },
    { 0xfe3a2a7e, SUC_lexicon_tags_25 },
    { 0xdbba2a7c, SUC_lexicon_tags_14 },
    { 0x9b7a2a80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x873a2a82, SUC_lexicon_tags_2 },
    { 0x4a7a2a83, SUC_lexicon_tags_235 },
    { 0x7d5a2a84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba2a87, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x123a2a89, SUC_lexicon_tags_0 },
    { 0x405a2a89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xee5a2a8c, SUC_lexicon_tags_375 },
    { 0x813a2a8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xee7a2a8f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fa2a92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32ba2a94, SUC_lexicon_tags_15 },
    { 0x6f1a2a95, SUC_lexicon_tags_46 },
    { 0xa4fa2a96, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fa2a9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fba2a9e, SUC_lexicon_tags_26 },
    { 0x62da2a9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a2aa2, SUC_lexicon_tags_44 },
    { 0x417a2aa2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe9ba2aa5, SUC_lexicon_tags_82 },
    { 0xa83a2aa5, SUC_lexicon_tags_47 },
    { 0xffda2aa6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151a2ab3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3da2ab5, SUC_lexicon_tags_24 },
    { 0x999a2ab6, SUC_lexicon_tags_27 },
    { 0x1d5a2ab7, SUC_lexicon_tags_63 },
    { 0x217a2ab7, SUC_lexicon_tags_11 },
    { 0x523a2ab9, SUC_lexicon_tags_11 },
    { 0xe9fa2ab9, SUC_lexicon_tags_7 },
    { 0x7afa2ab7, SUC_lexicon_tags_2 },
    { 0x385a2abb, SUC_lexicon_tags_43 },
    { 0x22da2abd, SUC_lexicon_tags_23 },
    { 0xed7a2abd, SUC_lexicon_tags_95 },
    { 0xb51a2ab5, SUC_lexicon_tags_11 },
    { 0xe9da2abd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x165a2ac3, SUC_lexicon_tags_49 },
    { 0x301a2ac4, SUC_lexicon_tags_23 },
    { 0xc95a2ac5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fa2ac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283a2acc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd5a2ace, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21a2ad4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa2ad7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43fa2ad9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359a2add, SUC_lexicon_tags_38 },
    { 0x2a5a2ade, SUC_lexicon_tags_77 },
    { 0x191a2add, SUC_lexicon_tags_11 },
    { 0x7dba2ae0, SUC_lexicon_tags_17 },
    { 0x963a2ae1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9a2ae6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139a2aec, SUC_lexicon_tags_14 },
    { 0x705a2aed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x909a2aef, SUC_lexicon_tags_10 },
    { 0x5d3a2aef, SUC_lexicon_tags_14 },
    { 0x9dfa2af1, SUC_lexicon_tags_15 },
    { 0x2e3a2af0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fa2af5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ba2af9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a3a2afb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ba2b00, SUC_lexicon_tags_26 },
    { 0x223a2b01, SUC_lexicon_tags_17 },
    { 0x201a2b01, SUC_lexicon_tags_37 },
    { 0xa31a2b00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3a2b08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1a2b0b, SUC_lexicon_tags_31 },
    { 0x949a2b0b, SUC_lexicon_tags_14 },
    { 0x275a2b0d, SUC_lexicon_tags_24 },
    { 0x505a2b0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385a2b19, SUC_lexicon_tags_28 },
    { 0x7e7a2b1a, SUC_lexicon_tags_5 },
    { 0x7da2b1a, SUC_lexicon_tags_15 },
    { 0x74da2b1a, SUC_lexicon_tags_7 },
    { 0x709a2b1c, SUC_lexicon_tags_17 },
    { 0x925a2b1b, SUC_lexicon_tags_9 },
    { 0xc49a2b1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fa2b23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b5a2b25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44da2b2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55a2b2d, SUC_lexicon_tags_37 },
    { 0x4d5a2b2d, SUC_lexicon_tags_43 },
    { 0xc39a2b2f, SUC_lexicon_tags_15 },
    { 0x3f1a2b2d, SUC_lexicon_tags_9 },
    { 0x5dba2b2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x195a2b33, SUC_lexicon_tags_15 },
    { 0x84ba2b34, SUC_lexicon_tags_15 },
    { 0xbb3a2b33, SUC_lexicon_tags_25 },
    { 0x407a2b36, SUC_lexicon_tags_26 },
    { 0x55ba2b34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x537a2b3a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x209a2b3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5cfa2b3f, SUC_lexicon_tags_12 },
    { 0xb89a2b40, SUC_lexicon_tags_33 },
    { 0xaf7a2b3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a2b45, SUC_lexicon_tags_2 },
    { 0xeeba2b45, SUC_lexicon_tags_16 },
    { 0xb91a2b47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fa2b4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1a2b57, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27a2b5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dda2b5d, SUC_lexicon_tags_47 },
    { 0x1d9a2b5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x66fa2b61, SUC_lexicon_tags_30 },
    { 0xb5da2b62, SUC_lexicon_tags_37 },
    { 0x727a2b62, SUC_lexicon_tags_9 },
    { 0x185a2b61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fda2b67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9a2b6b, SUC_lexicon_tags_1 },
    { 0x29da2b6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7a2b70, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ba2b74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51a2b77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839a2b7d, SUC_lexicon_tags_43 },
    { 0x60fa2b7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb9a2b80, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8cba2b82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79a2b89, SUC_lexicon_tags_26 },
    { 0x62ba2b89, SUC_lexicon_tags_12 },
    { 0x72da2b8b, SUC_lexicon_tags_29 },
    { 0x587a2b8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3da2b8e, SUC_lexicon_tags_38 },
    { 0x6d7a2b8f, SUC_lexicon_tags_46 },
    { 0x5c7a2b90, SUC_lexicon_tags_12 },
    { 0xdb1a2b91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7a2b95, SUC_lexicon_tags_17 },
    { 0x49ba2b96, SUC_lexicon_tags_2 },
    { 0xd21a2b96, SUC_lexicon_tags_17 },
    { 0x7c1a2b98, SUC_lexicon_tags_17 },
    { 0x9a7a2b99, SUC_lexicon_tags_31 },
    { 0x281a2b97, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7a2ba3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3a2ba7, SUC_lexicon_tags_15 },
    { 0xa9da2ba8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x675a2bab, SUC_lexicon_tags_19 },
    { 0xf1a2bac, SUC_lexicon_tags_57 },
    { 0x15da2bad, SUC_lexicon_tags_15 },
    { 0x3d9a2bad, SUC_lexicon_tags_25 },
    { 0xfcda2bad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fa2bb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x259a2bb5, SUC_lexicon_tags_56 },
    { 0xff7a2bb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479a2bba, SUC_lexicon_tags_37 },
    { 0xa63a2bba, SUC_lexicon_tags_37 },
    { 0xa8ba2bbb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1a2bc0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x995a2bc2, SUC_lexicon_tags_27 },
    { 0xb69a2bc2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9a2bc7, SUC_lexicon_tags_15 },
    { 0xbe9a2bc8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ba2bcc, SUC_lexicon_tags_151 },
    { 0x837a2bcc, SUC_lexicon_tags_37 },
    { 0x1d7a2bcc, SUC_lexicon_tags_47 },
    { 0x519a2bcf, SUC_lexicon_tags_7 },
    { 0xf81a2bcf, SUC_lexicon_tags_15 },
    { 0x10ba2bd1, SUC_lexicon_tags_22 },
    { 0xf83a2bd1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29a2bd7, SUC_lexicon_tags_60 },
    { 0x29da2bd7, SUC_lexicon_tags_11 },
    { 0xe29a2bd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ba2bdd, SUC_lexicon_tags_47 },
    { 0xafba2bdd, SUC_lexicon_tags_31 },
    { 0xb59a2bdf, SUC_lexicon_tags_17 },
    { 0x6efa2be0, SUC_lexicon_tags_45 },
    { 0xcfa2be1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93a2be5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857a2be9, SUC_lexicon_tags_54 },
    { 0xcc9a2bea, SUC_lexicon_tags_2 },
    { 0xa07a2beb, SUC_lexicon_tags_47 },
    { 0xc41a2bec, SUC_lexicon_tags_28 },
    { 0xa5fa2bed, SUC_lexicon_tags_17 },
    { 0x7a2bec, SUC_lexicon_tags_79 },
    { 0xd69a2bef, SUC_lexicon_tags_10 },
    { 0x4a1a2bea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9a2bf5, SUC_lexicon_tags_15 },
    { 0xb83a2bf6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfa2bfa, SUC_lexicon_tags_45 },
    { 0x1ada2bfb, SUC_lexicon_tags_15 },
    { 0x9d9a2bfc, SUC_lexicon_tags_37 },
    { 0xaeba2bfd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47a2c00, SUC_lexicon_tags_11 },
    { 0xa3fa2c01, SUC_lexicon_tags_47 },
    { 0x7d9a2c01, SUC_lexicon_tags_25 },
    { 0xa19a2c03, SUC_lexicon_tags_1676 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fa2c07, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747a2c0e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x751a2c10, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfa2c13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23a2c1c, SUC_lexicon_tags_7 },
    { 0x125a2c1d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7a2c23, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1a2c27, SUC_lexicon_tags_47 },
    { 0x477a2c27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbada2c2a, SUC_lexicon_tags_11 },
    { 0x329a2c2a, SUC_lexicon_tags_47 },
    { 0xb67a2c2b, SUC_lexicon_tags_17 },
    { 0xda2c2d, SUC_lexicon_tags_29 },
    { 0xb79a2c2d, SUC_lexicon_tags_28 },
    { 0xf9ba2c2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafda2c32, SUC_lexicon_tags_10 },
    { 0xd8da2c33, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1a2c38, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5a2c3c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7a2c40, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73a2c45, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xeb3a2c47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf9da2c49, SUC_lexicon_tags_112 },
    { 0x17da2c49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda2c4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1a2c51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ba2c5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x50fa2c61, SUC_lexicon_tags_45 },
    { 0x647a2c62, SUC_lexicon_tags_23 },
    { 0x29ba2c63, SUC_lexicon_tags_10 },
    { 0x4aba2c64, SUC_lexicon_tags_37 },
    { 0xb3fa2c64, SUC_lexicon_tags_84 },
    { 0x9c9a2c66, SUC_lexicon_tags_15 },
    { 0xfc5a2c67, SUC_lexicon_tags_60 },
    { 0x103a2c63, SUC_lexicon_tags_14 },
    { 0x397a2c64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x203a2c6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40da2c72, SUC_lexicon_tags_15 },
    { 0x841a2c72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa61a2c75, SUC_lexicon_tags_25 },
    { 0xe11a2c75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5a2c79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fa2c82, SUC_lexicon_tags_11 },
    { 0xcfba2c83, SUC_lexicon_tags_9 },
    { 0xdd1a2c84, SUC_lexicon_tags_38 },
    { 0x267a2c83, SUC_lexicon_tags_26 },
    { 0xc29a2c82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfa2c8c, SUC_lexicon_tags_15 },
    { 0xd4ba2c8d, SUC_lexicon_tags_2 },
    { 0xb3fa2c8e, SUC_lexicon_tags_3 },
    { 0x9eda2c8f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcf7a2c91, SUC_lexicon_tags_14 },
    { 0xfb9a2c92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfe1a2c94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fba2c9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfda2ca0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215a2ca4, SUC_lexicon_tags_43 },
    { 0x755a2ca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955a2cab, SUC_lexicon_tags_11 },
    { 0x5f9a2cac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefa2cb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd7a2cb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a2cb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579a2cba, SUC_lexicon_tags_34 },
    { 0xf13a2cbb, SUC_lexicon_tags_244 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913a2cbf, SUC_lexicon_tags_1 },
    { 0x389a2cc0, SUC_lexicon_tags_0 },
    { 0x5f1a2cc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7a2cc7, SUC_lexicon_tags_0 },
    { 0x219a2cc7, SUC_lexicon_tags_17 },
    { 0xd9ba2cc9, SUC_lexicon_tags_70 },
    { 0xe45a2cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea7a2ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda2ccf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9a2cd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ba2ce0, SUC_lexicon_tags_15 },
    { 0xec1a2ce1, SUC_lexicon_tags_14 },
    { 0x109a2ce2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9a2ce8, SUC_lexicon_tags_65 },
    { 0x415a2ce9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x401a2cec, SUC_lexicon_tags_43 },
    { 0xf77a2cec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8da2cef, SUC_lexicon_tags_11 },
    { 0xdaba2cef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865a2cfd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739a2d07, SUC_lexicon_tags_7 },
    { 0xa63a2d08, SUC_lexicon_tags_27 },
    { 0x8b7a2d08, SUC_lexicon_tags_25 },
    { 0x865a2d07, SUC_lexicon_tags_12 },
    { 0x43da2d08, SUC_lexicon_tags_12 },
    { 0xa99a2d0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa2d10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c1a2d12, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x14ba2d14, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fa2d19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23a2d1f, SUC_lexicon_tags_33 },
    { 0x613a2d20, SUC_lexicon_tags_31 },
    { 0x41ba2d20, SUC_lexicon_tags_28 },
    { 0xe63a2d21, SUC_lexicon_tags_91 },
    { 0x289a2d23, SUC_lexicon_tags_28 },
    { 0x46fa2d23, SUC_lexicon_tags_25 },
    { 0xf69a2d24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe3a2d27, SUC_lexicon_tags_15 },
    { 0xdf5a2d28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fa2d2c, SUC_lexicon_tags_11 },
    { 0xa0fa2d2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6da2d2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcba2d35, SUC_lexicon_tags_14 },
    { 0x271a2d36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81a2d39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5a2d3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73a2d45, SUC_lexicon_tags_11 },
    { 0x885a2d46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7a2d4a, SUC_lexicon_tags_30 },
    { 0x46da2d4b, SUC_lexicon_tags_17 },
    { 0x891a2d4b, SUC_lexicon_tags_61 },
    { 0xc45a2d4a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fa2d52, SUC_lexicon_tags_30 },
    { 0x17da2d53, SUC_lexicon_tags_17 },
    { 0xcb5a2d54, SUC_lexicon_tags_12 },
    { 0xb93a2d53, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9a2d58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73fa2d5a, SUC_lexicon_tags_1111 },
    { 0, NULL },
    { 0xb91a2d5c, SUC_lexicon_tags_60 },
    { 0x5cfa2d5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x131a2d60, SUC_lexicon_tags_43 },
    { 0xf39a2d60, SUC_lexicon_tags_27 },
    { 0x58ba2d62, SUC_lexicon_tags_61 },
    { 0xaeda2d63, SUC_lexicon_tags_17 },
    { 0xc6da2d60, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x515a2d66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ba2d6b, SUC_lexicon_tags_37 },
    { 0xbe9a2d6b, SUC_lexicon_tags_11 },
    { 0xf8da2d6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1a2d71, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x775a2d74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x375a2d77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7a2d7b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79a2d7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa25a2d81, SUC_lexicon_tags_7 },
    { 0x6b7a2d82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341a2d87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a2d8b, SUC_lexicon_tags_47 },
    { 0xe1a2d8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1a2d90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x713a2d93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc09a2d95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x493a2d97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a3a2d99, SUC_lexicon_tags_48 },
    { 0x4ffa2d9a, SUC_lexicon_tags_43 },
    { 0xda7a2d9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ba2d9e, SUC_lexicon_tags_12 },
    { 0x299a2d9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xacba2da2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65da2da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b3a2da6, SUC_lexicon_tags_60 },
    { 0xf45a2da7, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fa2daf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x967a2db2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ada2db6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbeda2db8, SUC_lexicon_tags_104 },
    { 0x29ba2db9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ba2dbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ba2dc0, SUC_lexicon_tags_63 },
    { 0xc65a2dc0, SUC_lexicon_tags_66 },
    { 0x21da2dc2, SUC_lexicon_tags_37 },
    { 0xb3ba2dc3, SUC_lexicon_tags_47 },
    { 0x429a2dc2, SUC_lexicon_tags_26 },
    { 0xeb9a2dc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfdda2dc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc5a2dc9, SUC_lexicon_tags_19 },
    { 0x715a2dca, SUC_lexicon_tags_38 },
    { 0x2f7a2dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc8da2dcd, SUC_lexicon_tags_38 },
    { 0x223a2dce, SUC_lexicon_tags_60 },
    { 0xd2fa2dcf, SUC_lexicon_tags_15 },
    { 0xba1a2dd0, SUC_lexicon_tags_27 },
    { 0x3afa2dd1, SUC_lexicon_tags_29 },
    { 0x16fa2dce, SUC_lexicon_tags_14 },
    { 0xaa5a2dd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47a2dd8, SUC_lexicon_tags_49 },
    { 0xc03a2dd9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fa2ddf, SUC_lexicon_tags_17 },
    { 0x961a2de0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x661a2de2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x851a2de4, SUC_lexicon_tags_37 },
    { 0x447a2de4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15a2de8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf99a2dea, SUC_lexicon_tags_7 },
    { 0x76fa2deb, SUC_lexicon_tags_12 },
    { 0xe9ba2dec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbd1a2dee, SUC_lexicon_tags_25 },
    { 0x1da2def, SUC_lexicon_tags_11 },
    { 0x2eba2dee, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09a2df3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9a2df6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb97a2df8, SUC_lexicon_tags_32 },
    { 0xf4da2df8, SUC_lexicon_tags_1 },
    { 0x5fa2dfa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7da2dfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29a2e00, SUC_lexicon_tags_27 },
    { 0x26fa2e01, SUC_lexicon_tags_15 },
    { 0xeb5a2e01, SUC_lexicon_tags_37 },
    { 0xe55a2e02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33a2e0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827a2e12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519a2e17, SUC_lexicon_tags_17 },
    { 0x523a2e18, SUC_lexicon_tags_37 },
    { 0xae1a2e19, SUC_lexicon_tags_61 },
    { 0x4ba2e19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975a2e1e, SUC_lexicon_tags_82 },
    { 0xc3ba2e1f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fa2e22, SUC_lexicon_tags_47 },
    { 0x1f7a2e23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6aba2e25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9ba2e27, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaba2e2b, SUC_lexicon_tags_87 },
    { 0x759a2e2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7a2e2f, SUC_lexicon_tags_12 },
    { 0x521a2e30, SUC_lexicon_tags_16 },
    { 0x495a2e31, SUC_lexicon_tags_57 },
    { 0xcf5a2e31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9a2e35, SUC_lexicon_tags_17 },
    { 0x661a2e36, SUC_lexicon_tags_0 },
    { 0x63fa2e37, SUC_lexicon_tags_15 },
    { 0xde7a2e37, SUC_lexicon_tags_30 },
    { 0x6a5a2e39, SUC_lexicon_tags_2 },
    { 0xa51a2e3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x375a2e3c, SUC_lexicon_tags_25 },
    { 0xc41a2e3d, SUC_lexicon_tags_14 },
    { 0xd93a2e3e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xba1a2e40, SUC_lexicon_tags_15 },
    { 0x8c1a2e40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x137a2e43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3a2e47, SUC_lexicon_tags_15 },
    { 0xb71a2e48, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3a2e4b, SUC_lexicon_tags_32 },
    { 0xecfa2e4c, SUC_lexicon_tags_31 },
    { 0xd5da2e4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54ba2e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x999a2e52, SUC_lexicon_tags_49 },
    { 0xce7a2e53, SUC_lexicon_tags_12 },
    { 0x5e3a2e53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf73a2e56, SUC_lexicon_tags_7 },
    { 0xf19a2e56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96da2e59, SUC_lexicon_tags_37 },
    { 0xbdba2e5a, SUC_lexicon_tags_27 },
    { 0xa2ba2e59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775a2e5f, SUC_lexicon_tags_2 },
    { 0x413a2e60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x685a2e62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51fa2e64, SUC_lexicon_tags_48 },
    { 0x55a2e65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279a2e6c, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09a2e6f, SUC_lexicon_tags_6 },
    { 0xc29a2e70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe59a2e72, SUC_lexicon_tags_25 },
    { 0x63da2e72, SUC_lexicon_tags_16 },
    { 0xf1fa2e74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bda2e76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aba2e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfa2e80, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3a2e83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fa2e86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa11a2e88, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ba2e8c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc6da2e8e, SUC_lexicon_tags_71 },
    { 0x193a2e8f, SUC_lexicon_tags_27 },
    { 0xf9a2e8e, SUC_lexicon_tags_7 },
    { 0x7f9a2e8f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e9a2e93, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x41da2e95, SUC_lexicon_tags_25 },
    { 0x5b5a2e96, SUC_lexicon_tags_23 },
    { 0xd5fa2e95, SUC_lexicon_tags_25 },
    { 0x403a2e97, SUC_lexicon_tags_2 },
    { 0xd29a2e99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57ba2e9b, SUC_lexicon_tags_32 },
    { 0xd17a2e9b, SUC_lexicon_tags_64 },
    { 0x2e9a2e9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa69a2e9f, SUC_lexicon_tags_2 },
    { 0x153a2e9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38da2ea5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131a2eaa, SUC_lexicon_tags_37 },
    { 0x855a2eab, SUC_lexicon_tags_45 },
    { 0x7b7a2eac, SUC_lexicon_tags_1 },
    { 0x5c3a2eac, SUC_lexicon_tags_49 },
    { 0x8eda2eae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849a2eb6, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479a2eba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23a2ebd, SUC_lexicon_tags_45 },
    { 0x5eba2ebd, SUC_lexicon_tags_38 },
    { 0xe3a2ebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3a2ec3, SUC_lexicon_tags_15 },
    { 0x631a2ec4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fa2ec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a2ecb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb11a2ecd, SUC_lexicon_tags_46 },
    { 0x95da2ece, SUC_lexicon_tags_37 },
    { 0x5f7a2ecf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9a2ed6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7a2edd, SUC_lexicon_tags_27 },
    { 0x5a1a2ede, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0da2ee1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3a2ee4, SUC_lexicon_tags_25 },
    { 0x115a2ee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x211a2ee7, SUC_lexicon_tags_70 },
    { 0x37ba2ee7, SUC_lexicon_tags_47 },
    { 0xd5a2ee9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3a2eec, SUC_lexicon_tags_6 },
    { 0xecda2eed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ba2ef1, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5a2ef5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc69a2ef7, SUC_lexicon_tags_57 },
    { 0xd37a2ef8, SUC_lexicon_tags_12 },
    { 0xbf5a2ef9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ba2eff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dfa2f01, SUC_lexicon_tags_17 },
    { 0x581a2f01, SUC_lexicon_tags_11 },
    { 0x64ba2f01, SUC_lexicon_tags_118 },
    { 0xcdfa2f03, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679a2f0f, SUC_lexicon_tags_30 },
    { 0xd6da2f0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x41ba2f12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xada2f14, SUC_lexicon_tags_25 },
    { 0xa9da2f14, SUC_lexicon_tags_32 },
    { 0xd19a2f16, SUC_lexicon_tags_15 },
    { 0xbd1a2f17, SUC_lexicon_tags_87 },
    { 0x2b1a2f18, SUC_lexicon_tags_47 },
    { 0x47ba2f17, SUC_lexicon_tags_2 },
    { 0x34fa2f1a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13a2f1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1a2f21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf3a2f23, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4da2f27, SUC_lexicon_tags_60 },
    { 0xb7ba2f27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97a2f2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfda2f32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7a2f35, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a2f38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x973a2f3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99a2f3d, SUC_lexicon_tags_16 },
    { 0xd31a2f3d, SUC_lexicon_tags_7 },
    { 0x3f5a2f3f, SUC_lexicon_tags_37 },
    { 0x9b3a2f40, SUC_lexicon_tags_17 },
    { 0x567a2f40, SUC_lexicon_tags_17 },
    { 0x73ba2f3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc81a2f44, SUC_lexicon_tags_17 },
    { 0xdafa2f45, SUC_lexicon_tags_31 },
    { 0x1fda2f45, SUC_lexicon_tags_17 },
    { 0x79fa2f47, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x579a2f49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35a2f4d, SUC_lexicon_tags_7 },
    { 0x1f3a2f4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aba2f53, SUC_lexicon_tags_28 },
    { 0xdb9a2f54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267a2f5b, SUC_lexicon_tags_15 },
    { 0x6afa2f5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcba2f60, SUC_lexicon_tags_15 },
    { 0x9c1a2f60, SUC_lexicon_tags_0 },
    { 0xa55a2f60, SUC_lexicon_tags_31 },
    { 0x283a2f63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x647a2f65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc67a2f67, SUC_lexicon_tags_15 },
    { 0xde7a2f67, SUC_lexicon_tags_9 },
    { 0x5f3a2f69, SUC_lexicon_tags_47 },
    { 0x201a2f6a, SUC_lexicon_tags_46 },
    { 0x6d3a2f68, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39a2f72, SUC_lexicon_tags_11 },
    { 0xadda2f73, SUC_lexicon_tags_12 },
    { 0xad3a2f74, SUC_lexicon_tags_15 },
    { 0x8f9a2f74, SUC_lexicon_tags_63 },
    { 0x17a2f75, SUC_lexicon_tags_1 },
    { 0xa21a2f77, SUC_lexicon_tags_2 },
    { 0x223a2f76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ba2f7b, SUC_lexicon_tags_1099 },
    { 0x229a2f7b, SUC_lexicon_tags_37 },
    { 0xd1da2f7d, SUC_lexicon_tags_28 },
    { 0x343a2f7d, SUC_lexicon_tags_1 },
    { 0xf31a2f7f, SUC_lexicon_tags_7 },
    { 0x511a2f7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b1a2f82, SUC_lexicon_tags_14 },
    { 0xfa9a2f83, SUC_lexicon_tags_25 },
    { 0xc53a2f84, SUC_lexicon_tags_6 },
    { 0xf19a2f85, SUC_lexicon_tags_19 },
    { 0x255a2f85, SUC_lexicon_tags_12 },
    { 0xfd3a2f86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb39a2f89, SUC_lexicon_tags_37 },
    { 0xfdba2f8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41a2f8f, SUC_lexicon_tags_35 },
    { 0xe5fa2f90, SUC_lexicon_tags_19 },
    { 0xe79a2f8f, SUC_lexicon_tags_0 },
    { 0x92da2f92, SUC_lexicon_tags_17 },
    { 0x2a1a2f93, SUC_lexicon_tags_136 },
    { 0x2f9a2f93, SUC_lexicon_tags_84 },
    { 0xccda2f94, SUC_lexicon_tags_15 },
    { 0x16fa2f95, SUC_lexicon_tags_17 },
    { 0xe85a2f97, SUC_lexicon_tags_15 },
    { 0x817a2f96, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ba2f9c, SUC_lexicon_tags_15 },
    { 0xd7ba2f9d, SUC_lexicon_tags_15 },
    { 0x7dda2f9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x787a2fa1, SUC_lexicon_tags_15 },
    { 0xd21a2fa1, SUC_lexicon_tags_77 },
    { 0x4e1a2fa3, SUC_lexicon_tags_60 },
    { 0xdfba2fa1, SUC_lexicon_tags_12 },
    { 0x43a2fa3, SUC_lexicon_tags_20 },
    { 0x7e5a2fa4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfa2fa9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7a2fac, SUC_lexicon_tags_37 },
    { 0xf7a2fad, SUC_lexicon_tags_27 },
    { 0x13fa2fae, SUC_lexicon_tags_7 },
    { 0xe6fa2faf, SUC_lexicon_tags_37 },
    { 0x565a2fb0, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621a2fb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561a2fbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa2fc1, SUC_lexicon_tags_38 },
    { 0xc39a2fc1, SUC_lexicon_tags_2 },
    { 0xcc1a2fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ba2fcd, SUC_lexicon_tags_15 },
    { 0x18ba2fcd, SUC_lexicon_tags_11 },
    { 0x3dfa2fce, SUC_lexicon_tags_2 },
    { 0x1d3a2fd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3a2fd5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379a2fdc, SUC_lexicon_tags_23 },
    { 0xc77a2fdc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdba2fe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x639a2fe3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9eba2fe5, SUC_lexicon_tags_33 },
    { 0xa7a2fe6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf07a2fe8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x32da2fea, SUC_lexicon_tags_35 },
    { 0x47ba2fea, SUC_lexicon_tags_27 },
    { 0x68da2fec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffa2ff0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x93ba2ff2, SUC_lexicon_tags_21 },
    { 0x6cba2ff2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aba2ff9, SUC_lexicon_tags_12 },
    { 0x841a2ffa, SUC_lexicon_tags_7 },
    { 0x319a2ffa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fa2fff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf03a3001, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf9a3003, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e7a3005, SUC_lexicon_tags_60 },
    { 0xe31a3006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1a300a, SUC_lexicon_tags_63 },
    { 0xd6ba300b, SUC_lexicon_tags_9 },
    { 0xee9a300c, SUC_lexicon_tags_25 },
    { 0xfaba300d, SUC_lexicon_tags_15 },
    { 0xdada300a, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5a3016, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dda3019, SUC_lexicon_tags_14 },
    { 0xb43a3019, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53a3020, SUC_lexicon_tags_27 },
    { 0xe69a3021, SUC_lexicon_tags_7 },
    { 0x809a3020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78ba3024, SUC_lexicon_tags_10 },
    { 0xa47a3025, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9a3028, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d1a302a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bba302c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83a302f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363a3039, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49a303c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ba3040, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97a3043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19a3047, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x745a3049, SUC_lexicon_tags_37 },
    { 0x21fa304a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09a304d, SUC_lexicon_tags_34 },
    { 0xb5fa304e, SUC_lexicon_tags_43 },
    { 0x97da304e, SUC_lexicon_tags_7 },
    { 0x3bba3050, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fa3055, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9a3059, SUC_lexicon_tags_15 },
    { 0x14ba305a, SUC_lexicon_tags_17 },
    { 0xcd1a305b, SUC_lexicon_tags_60 },
    { 0x203a305a, SUC_lexicon_tags_1 },
    { 0x29ba305a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ba3061, SUC_lexicon_tags_61 },
    { 0x723a3062, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x215a3065, SUC_lexicon_tags_69 },
    { 0x807a3066, SUC_lexicon_tags_618 },
    { 0, NULL },
    { 0x7afa3068, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaefa306a, SUC_lexicon_tags_15 },
    { 0xc5fa306a, SUC_lexicon_tags_2 },
    { 0xde9a306a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3da306f, SUC_lexicon_tags_14 },
    { 0xddfa3070, SUC_lexicon_tags_11 },
    { 0x5fba3071, SUC_lexicon_tags_37 },
    { 0xb45a306f, SUC_lexicon_tags_11 },
    { 0xeafa3072, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf5a3075, SUC_lexicon_tags_48 },
    { 0x2c5a3075, SUC_lexicon_tags_37 },
    { 0x8a1a3077, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x52fa3079, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9a307d, SUC_lexicon_tags_12 },
    { 0x607a307d, SUC_lexicon_tags_30 },
    { 0xfa5a307f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5a3085, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8d3a3087, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05a308a, SUC_lexicon_tags_65 },
    { 0x399a308b, SUC_lexicon_tags_30 },
    { 0xaf1a308c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x299a308f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18da3093, SUC_lexicon_tags_23 },
    { 0xe89a3094, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9a9a3096, SUC_lexicon_tags_8 },
    { 0x853a3097, SUC_lexicon_tags_174 },
    { 0x87ba3098, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ba309e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39a30a2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa30a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fa30ac, SUC_lexicon_tags_11 },
    { 0xfa7a30ac, SUC_lexicon_tags_58 },
    { 0x9bfa30ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda30b1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851a30b5, SUC_lexicon_tags_1 },
    { 0xc03a30b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da30b9, SUC_lexicon_tags_17 },
    { 0x855a30ba, SUC_lexicon_tags_42 },
    { 0x571a30bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5a30be, SUC_lexicon_tags_7 },
    { 0x799a30bf, SUC_lexicon_tags_8 },
    { 0x18da30bf, SUC_lexicon_tags_37 },
    { 0x5ada30c1, SUC_lexicon_tags_17 },
    { 0x1dda30c2, SUC_lexicon_tags_2 },
    { 0xaf7a30c2, SUC_lexicon_tags_31 },
    { 0x89da30c2, SUC_lexicon_tags_15 },
    { 0xe27a30c5, SUC_lexicon_tags_37 },
    { 0xad5a30c5, SUC_lexicon_tags_23 },
    { 0xe9da30c7, SUC_lexicon_tags_33 },
    { 0x22da30c5, SUC_lexicon_tags_14 },
    { 0x627a30c9, SUC_lexicon_tags_37 },
    { 0x343a30ca, SUC_lexicon_tags_37 },
    { 0x5f5a30cb, SUC_lexicon_tags_8 },
    { 0xecfa30ca, SUC_lexicon_tags_1 },
    { 0x14fa30cb, SUC_lexicon_tags_38 },
    { 0x9ada30cc, SUC_lexicon_tags_39 },
    { 0xab5a30cc, SUC_lexicon_tags_179 },
    { 0x30fa30cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a30d3, SUC_lexicon_tags_19 },
    { 0x6eba30d4, SUC_lexicon_tags_7 },
    { 0x44fa30d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597a30d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13a30de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba30e1, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55a30e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fa30e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81fa30ea, SUC_lexicon_tags_9 },
    { 0x8cba30ea, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x86ba30ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9a30f1, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0xfdfa30f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafa30f9, SUC_lexicon_tags_1549 },
    { 0, NULL },
    { 0x6f3a30fb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb91a30fd, SUC_lexicon_tags_0 },
    { 0x873a30fd, SUC_lexicon_tags_37 },
    { 0xfffa30ff, SUC_lexicon_tags_37 },
    { 0x95a30ff, SUC_lexicon_tags_26 },
    { 0x3cba3100, SUC_lexicon_tags_11 },
    { 0xfa3a3101, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813a3107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x203a310a, SUC_lexicon_tags_21 },
    { 0x1bda310a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907a3112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1a3116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d7a3118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc87a311a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455a3122, SUC_lexicon_tags_17 },
    { 0x349a3123, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8da3126, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1da3129, SUC_lexicon_tags_13 },
    { 0xe8da312a, SUC_lexicon_tags_49 },
    { 0xbbfa312b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3a312e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafa3131, SUC_lexicon_tags_8 },
    { 0x343a3132, SUC_lexicon_tags_26 },
    { 0x755a3133, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11a3136, SUC_lexicon_tags_25 },
    { 0xeeda3137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d3a3139, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x249a313b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28da3142, SUC_lexicon_tags_47 },
    { 0x93ba3143, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1a3147, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x647a314a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1da314f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dba3153, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb71a3156, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1a3158, SUC_lexicon_tags_7 },
    { 0xe09a3158, SUC_lexicon_tags_4 },
    { 0xe9fa315a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x911a315c, SUC_lexicon_tags_7 },
    { 0xdd1a315d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf95a315f, SUC_lexicon_tags_47 },
    { 0x557a3160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ba316d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9a3174, SUC_lexicon_tags_11 },
    { 0x96ba3174, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89a3178, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa87a317a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3a317d, SUC_lexicon_tags_11 },
    { 0x42ba317e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7a3180, SUC_lexicon_tags_12 },
    { 0xc79a3180, SUC_lexicon_tags_2 },
    { 0xf51a3182, SUC_lexicon_tags_49 },
    { 0x5f1a3183, SUC_lexicon_tags_1 },
    { 0x46da3184, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce1a3186, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1a3189, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1a318c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x757a318e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fda3193, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541a3199, SUC_lexicon_tags_17 },
    { 0x1d7a3199, SUC_lexicon_tags_30 },
    { 0x7eda319b, SUC_lexicon_tags_11 },
    { 0xca9a319c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68fa319f, SUC_lexicon_tags_15 },
    { 0xfc7a319f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x765a31a2, SUC_lexicon_tags_49 },
    { 0x417a31a2, SUC_lexicon_tags_14 },
    { 0x2da31a4, SUC_lexicon_tags_17 },
    { 0x905a31a5, SUC_lexicon_tags_15 },
    { 0x45a31a5, SUC_lexicon_tags_66 },
    { 0x9bfa31a7, SUC_lexicon_tags_34 },
    { 0xbbfa31a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dda31ab, SUC_lexicon_tags_80 },
    { 0xba5a31ab, SUC_lexicon_tags_17 },
    { 0xc4fa31ac, SUC_lexicon_tags_26 },
    { 0x839a31ad, SUC_lexicon_tags_1 },
    { 0x38da31af, SUC_lexicon_tags_11 },
    { 0xba31b0, SUC_lexicon_tags_18 },
    { 0x607a31af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1da31b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415a31c0, SUC_lexicon_tags_87 },
    { 0xfa31c0, SUC_lexicon_tags_95 },
    { 0x3b9a31c1, SUC_lexicon_tags_28 },
    { 0xbba31c1, SUC_lexicon_tags_25 },
    { 0x3ada31c1, SUC_lexicon_tags_17 },
    { 0x533a31c5, SUC_lexicon_tags_1 },
    { 0x26fa31c5, SUC_lexicon_tags_30 },
    { 0xf9da31c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03a31d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1ba31d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x741a31d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7a31d9, SUC_lexicon_tags_8 },
    { 0x929a31da, SUC_lexicon_tags_23 },
    { 0x7ba31db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59a31de, SUC_lexicon_tags_37 },
    { 0xc49a31df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39a31e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01a31f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95a31f8, SUC_lexicon_tags_30 },
    { 0x84fa31f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fa3200, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5a3206, SUC_lexicon_tags_11 },
    { 0xaffa3206, SUC_lexicon_tags_1063 },
    { 0, NULL },
    { 0x139a3209, SUC_lexicon_tags_38 },
    { 0x647a320a, SUC_lexicon_tags_25 },
    { 0xffda320b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545a3216, SUC_lexicon_tags_17 },
    { 0xe1fa3217, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3a321a, SUC_lexicon_tags_107 },
    { 0xa93a321a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x511a321e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x13a3221, SUC_lexicon_tags_17 },
    { 0xf8da3221, SUC_lexicon_tags_28 },
    { 0x64da3221, SUC_lexicon_tags_26 },
    { 0xdb7a3224, SUC_lexicon_tags_65 },
    { 0x6e3a3225, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7a322a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40da322f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42da3231, SUC_lexicon_tags_23 },
    { 0x629a3232, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d7a3234, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e3a3236, SUC_lexicon_tags_9 },
    { 0x1d5a3237, SUC_lexicon_tags_47 },
    { 0x245a3238, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ba323e, SUC_lexicon_tags_21 },
    { 0xf75a323f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2c5a3241, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x381a3243, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305a3248, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfdba324a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7a324d, SUC_lexicon_tags_17 },
    { 0xe79a324e, SUC_lexicon_tags_37 },
    { 0x7d3a324f, SUC_lexicon_tags_7 },
    { 0xf27a3250, SUC_lexicon_tags_66 },
    { 0x2a9a3251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29a3254, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07a3257, SUC_lexicon_tags_15 },
    { 0xd97a3258, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751a325c, SUC_lexicon_tags_11 },
    { 0x5fda325d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x59ba325f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895a326e, SUC_lexicon_tags_8 },
    { 0x42da326f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287a3275, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835a327a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9a327d, SUC_lexicon_tags_15 },
    { 0xa39a327d, SUC_lexicon_tags_46 },
    { 0x365a327f, SUC_lexicon_tags_45 },
    { 0x983a327e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9a3285, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fa328a, SUC_lexicon_tags_17 },
    { 0x55a328a, SUC_lexicon_tags_0 },
    { 0xa09a328c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x853a328f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x755a3292, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7a3295, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dda329a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f1a329c, SUC_lexicon_tags_25 },
    { 0x507a329d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xca3a329f, SUC_lexicon_tags_15 },
    { 0xcc3a32a0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xda5a32a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5b7a32a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecfa32a6, SUC_lexicon_tags_37 },
    { 0xe29a32a6, SUC_lexicon_tags_108 },
    { 0x1cfa32a8, SUC_lexicon_tags_31 },
    { 0xadba32a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x97fa32ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437a32b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23a32b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9a32b7, SUC_lexicon_tags_10 },
    { 0x283a32b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa73a32ba, SUC_lexicon_tags_21 },
    { 0xc79a32bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1a32c0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x79ba32c2, SUC_lexicon_tags_28 },
    { 0x215a32c3, SUC_lexicon_tags_32 },
    { 0x873a32c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeba32c8, SUC_lexicon_tags_127 },
    { 0x955a32c9, SUC_lexicon_tags_17 },
    { 0x9c3a32ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671a32ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75a32d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1a32d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf6ba32d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x967a32dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ba32e0, SUC_lexicon_tags_68 },
    { 0xb13a32e1, SUC_lexicon_tags_25 },
    { 0xb15a32e1, SUC_lexicon_tags_34 },
    { 0x263a32e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a3a32e5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe5ba32e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb03a32e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401a32ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbda32f0, SUC_lexicon_tags_38 },
    { 0x5f9a32f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775a32f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a32f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaeba32fb, SUC_lexicon_tags_0 },
    { 0xfb9a32fc, SUC_lexicon_tags_1 },
    { 0xd8ba32fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9a3300, SUC_lexicon_tags_33 },
    { 0xd53a3301, SUC_lexicon_tags_2 },
    { 0x197a3300, SUC_lexicon_tags_150 },
    { 0xc05a3303, SUC_lexicon_tags_17 },
    { 0xedba3302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afa3308, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a330c, SUC_lexicon_tags_15 },
    { 0xa71a330d, SUC_lexicon_tags_12 },
    { 0x68da330e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfc1a3310, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc05a3312, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1a3315, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497a3319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3a331c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x623a331e, SUC_lexicon_tags_4 },
    { 0x81fa331f, SUC_lexicon_tags_11 },
    { 0x1d9a3320, SUC_lexicon_tags_60 },
    { 0xd1a331e, SUC_lexicon_tags_33 },
    { 0x25fa3322, SUC_lexicon_tags_37 },
    { 0x91ba3323, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcda3326, SUC_lexicon_tags_49 },
    { 0xbafa3327, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9a332a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb3a332c, SUC_lexicon_tags_19 },
    { 0x305a332d, SUC_lexicon_tags_14 },
    { 0x14da332e, SUC_lexicon_tags_6 },
    { 0xd4da332f, SUC_lexicon_tags_63 },
    { 0xf41a3330, SUC_lexicon_tags_12 },
    { 0x7bda3330, SUC_lexicon_tags_170 },
    { 0x9cda3330, SUC_lexicon_tags_2 },
    { 0xd07a3333, SUC_lexicon_tags_13 },
    { 0xb23a332d, SUC_lexicon_tags_10 },
    { 0x327a332f, SUC_lexicon_tags_17 },
    { 0x76da3334, SUC_lexicon_tags_33 },
    { 0x2fa3337, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499a333c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa03a333e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253a3343, SUC_lexicon_tags_263 },
    { 0, NULL },
    { 0, NULL },
    { 0x847a3346, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6da3349, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x897a334b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeeda334d, SUC_lexicon_tags_33 },
    { 0xa97a334e, SUC_lexicon_tags_43 },
    { 0x15da334e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f9a3351, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9a3354, SUC_lexicon_tags_25 },
    { 0xdaba3355, SUC_lexicon_tags_0 },
    { 0xdefa3356, SUC_lexicon_tags_16 },
    { 0x2ba3356, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3a3368, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01a336b, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03a3372, SUC_lexicon_tags_17 },
    { 0x859a3372, SUC_lexicon_tags_12 },
    { 0xb7da3374, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcba3377, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x803a3379, SUC_lexicon_tags_11 },
    { 0xd75a3379, SUC_lexicon_tags_1 },
    { 0x2fa337b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d5a337d, SUC_lexicon_tags_17 },
    { 0x49fa337d, SUC_lexicon_tags_29 },
    { 0x973a337e, SUC_lexicon_tags_955 },
    { 0xb89a3380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47fa3382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x55a3385, SUC_lexicon_tags_65 },
    { 0x93da3385, SUC_lexicon_tags_23 },
    { 0xcf5a3387, SUC_lexicon_tags_32 },
    { 0xc6da3388, SUC_lexicon_tags_69 },
    { 0x803a3389, SUC_lexicon_tags_45 },
    { 0xe51a338a, SUC_lexicon_tags_11 },
    { 0x987a338b, SUC_lexicon_tags_14 },
    { 0x8a9a3388, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbda3394, SUC_lexicon_tags_28 },
    { 0xaa1a3395, SUC_lexicon_tags_4 },
    { 0xe0da3396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5a3398, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c3a339a, SUC_lexicon_tags_12 },
    { 0xe27a339a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9a33a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afa33a7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ba33ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44da33ae, SUC_lexicon_tags_12 },
    { 0x3efa33af, SUC_lexicon_tags_15 },
    { 0x79fa33b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x123a33b3, SUC_lexicon_tags_3 },
    { 0x2a7a33b4, SUC_lexicon_tags_10 },
    { 0xdb7a33b5, SUC_lexicon_tags_29 },
    { 0x445a33b6, SUC_lexicon_tags_55 },
    { 0x339a33b5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb8fa33b9, SUC_lexicon_tags_37 },
    { 0x485a33ba, SUC_lexicon_tags_7 },
    { 0x9cda33bb, SUC_lexicon_tags_17 },
    { 0xd8ba33bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dba33c0, SUC_lexicon_tags_17 },
    { 0xc0fa33c0, SUC_lexicon_tags_31 },
    { 0xcdfa33c1, SUC_lexicon_tags_29 },
    { 0x1d5a33c2, SUC_lexicon_tags_37 },
    { 0x4fa33c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ba33c7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5a33cd, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0x31fa33cf, SUC_lexicon_tags_1 },
    { 0x5c5a33cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ba33d3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721a33d8, SUC_lexicon_tags_2 },
    { 0xfa9a33d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201a33df, SUC_lexicon_tags_17 },
    { 0x9b1a33e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfa33e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65a33e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x567a33eb, SUC_lexicon_tags_15 },
    { 0x931a33eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d1a33ee, SUC_lexicon_tags_47 },
    { 0x901a33ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72fa33f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23da33f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe91a33f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45a33f8, SUC_lexicon_tags_31 },
    { 0x291a33f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x927a33fb, SUC_lexicon_tags_12 },
    { 0x66fa33fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9a33ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf77a3401, SUC_lexicon_tags_0 },
    { 0x1a3a3402, SUC_lexicon_tags_27 },
    { 0x84ba3403, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x74fa3406, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3e1a3408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ba340c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecba340e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a1a3410, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bba3413, SUC_lexicon_tags_21 },
    { 0x2f3a3414, SUC_lexicon_tags_6 },
    { 0xa57a3413, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9a3419, SUC_lexicon_tags_30 },
    { 0x161a341a, SUC_lexicon_tags_27 },
    { 0xcb9a341a, SUC_lexicon_tags_13 },
    { 0x723a341a, SUC_lexicon_tags_65 },
    { 0x8a5a341a, SUC_lexicon_tags_68 },
    { 0x2da341e, SUC_lexicon_tags_38 },
    { 0x1e1a341f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195a3423, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335a3428, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fa342b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2eda342d, SUC_lexicon_tags_27 },
    { 0x1c5a342d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655a3434, SUC_lexicon_tags_39 },
    { 0x5ba3435, SUC_lexicon_tags_284 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1a3438, SUC_lexicon_tags_16 },
    { 0xafa3439, SUC_lexicon_tags_35 },
    { 0x393a3439, SUC_lexicon_tags_30 },
    { 0x411a343b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18da3440, SUC_lexicon_tags_136 },
    { 0xeeba3440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ba3444, SUC_lexicon_tags_22 },
    { 0x1cda3444, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf8ba3447, SUC_lexicon_tags_14 },
    { 0x84da3447, SUC_lexicon_tags_25 },
    { 0x3f7a3449, SUC_lexicon_tags_17 },
    { 0xac9a344a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7a3452, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa3455, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x659a3457, SUC_lexicon_tags_37 },
    { 0xf91a3457, SUC_lexicon_tags_15 },
    { 0x845a3459, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1a345c, SUC_lexicon_tags_37 },
    { 0x931a345d, SUC_lexicon_tags_37 },
    { 0xfa1a345d, SUC_lexicon_tags_17 },
    { 0x19a345d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x931a3462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffba3477, SUC_lexicon_tags_17 },
    { 0x43fa3478, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ba347e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd07a3480, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ba3483, SUC_lexicon_tags_30 },
    { 0xd23a3484, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1a3488, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bfa348a, SUC_lexicon_tags_10 },
    { 0xa9da348b, SUC_lexicon_tags_11 },
    { 0x273a348c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c1a348e, SUC_lexicon_tags_11 },
    { 0x881a348f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1a3493, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ba3496, SUC_lexicon_tags_118 },
    { 0x69da3497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59a349c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x123a349f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xff7a34a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661a34a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3fa34ab, SUC_lexicon_tags_15 },
    { 0xc8da34ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x231a34af, SUC_lexicon_tags_13 },
    { 0xf11a34af, SUC_lexicon_tags_612 },
    { 0x7e5a34b1, SUC_lexicon_tags_17 },
    { 0x675a34b2, SUC_lexicon_tags_12 },
    { 0x251a34b3, SUC_lexicon_tags_63 },
    { 0xd9fa34b2, SUC_lexicon_tags_17 },
    { 0xfefa34b2, SUC_lexicon_tags_39 },
    { 0xb3ba34b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x793a34b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7a34bb, SUC_lexicon_tags_64 },
    { 0xb0da34bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fa34bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699a34c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xde7a34c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31a34c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfa34ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcda34d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x66da34d3, SUC_lexicon_tags_11 },
    { 0xbdfa34d3, SUC_lexicon_tags_7 },
    { 0x4cda34d4, SUC_lexicon_tags_3 },
    { 0xefba34d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449a34da, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651a34e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc07a34ea, SUC_lexicon_tags_12 },
    { 0x6d1a34eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a34ee, SUC_lexicon_tags_56 },
    { 0xc5ba34ee, SUC_lexicon_tags_43 },
    { 0xeeda34ef, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x735a34f3, SUC_lexicon_tags_1 },
    { 0x6d3a34f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x195a34f7, SUC_lexicon_tags_47 },
    { 0xd57a34f8, SUC_lexicon_tags_15 },
    { 0x151a34f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1a3500, SUC_lexicon_tags_11 },
    { 0xd8ba3501, SUC_lexicon_tags_29 },
    { 0xefa3502, SUC_lexicon_tags_25 },
    { 0x9efa3502, SUC_lexicon_tags_29 },
    { 0x88fa3504, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefba3506, SUC_lexicon_tags_63 },
    { 0x9d3a3507, SUC_lexicon_tags_7 },
    { 0x727a3508, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29fa350a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ba3513, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bba3516, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313a351b, SUC_lexicon_tags_17 },
    { 0x925a351c, SUC_lexicon_tags_17 },
    { 0x4f1a351d, SUC_lexicon_tags_1 },
    { 0x77a351e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ba3522, SUC_lexicon_tags_35 },
    { 0xec1a3522, SUC_lexicon_tags_0 },
    { 0x12ba3523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbda3526, SUC_lexicon_tags_11 },
    { 0x481a3527, SUC_lexicon_tags_43 },
    { 0x889a3528, SUC_lexicon_tags_15 },
    { 0x961a3528, SUC_lexicon_tags_7 },
    { 0x20fa352a, SUC_lexicon_tags_60 },
    { 0x363a3528, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543a3532, SUC_lexicon_tags_63 },
    { 0xb53a3533, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351a3537, SUC_lexicon_tags_0 },
    { 0x2a3a3537, SUC_lexicon_tags_45 },
    { 0xd87a3537, SUC_lexicon_tags_25 },
    { 0x881a353a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4da353f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x467a3541, SUC_lexicon_tags_17 },
    { 0x569a3542, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7a3549, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x979a354b, SUC_lexicon_tags_55 },
    { 0x65da354c, SUC_lexicon_tags_39 },
    { 0xe4da354d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31a3550, SUC_lexicon_tags_0 },
    { 0x37ba3551, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7a3556, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9a3559, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aba355c, SUC_lexicon_tags_37 },
    { 0x23a355d, SUC_lexicon_tags_12 },
    { 0xbe1a355d, SUC_lexicon_tags_27 },
    { 0x647a355f, SUC_lexicon_tags_12 },
    { 0x93a3560, SUC_lexicon_tags_17 },
    { 0x79fa355f, SUC_lexicon_tags_151 },
    { 0x815a355c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x333a3564, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9a3a3566, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ba3569, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a356c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd5da356e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5c1a3570, SUC_lexicon_tags_207 },
    { 0xe5a3571, SUC_lexicon_tags_15 },
    { 0x603a3572, SUC_lexicon_tags_39 },
    { 0x843a3570, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49a3578, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ffa357a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86ba357c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77a3581, SUC_lexicon_tags_19 },
    { 0xe19a3582, SUC_lexicon_tags_12 },
    { 0x10da3583, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229a3589, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7a358d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x747a3590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x511a3593, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x813a3595, SUC_lexicon_tags_17 },
    { 0xe5ba3595, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf1a3598, SUC_lexicon_tags_37 },
    { 0x39da3598, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4a5a359b, SUC_lexicon_tags_17 },
    { 0xbd1a359c, SUC_lexicon_tags_30 },
    { 0xe9ba359d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x379a359f, SUC_lexicon_tags_1 },
    { 0x3bfa35a0, SUC_lexicon_tags_15 },
    { 0x1ba35a1, SUC_lexicon_tags_43 },
    { 0x3ada35a1, SUC_lexicon_tags_63 },
    { 0x337a35a3, SUC_lexicon_tags_6 },
    { 0xc21a35a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19ba35a6, SUC_lexicon_tags_49 },
    { 0xb0fa35a7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ba35af, SUC_lexicon_tags_57 },
    { 0xbfa35b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72ba35b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2eda35b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a35ba, SUC_lexicon_tags_26 },
    { 0x49da35ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72ba35bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29a35c0, SUC_lexicon_tags_1 },
    { 0xa97a35c1, SUC_lexicon_tags_7 },
    { 0xf47a35c1, SUC_lexicon_tags_11 },
    { 0x685a35c3, SUC_lexicon_tags_2 },
    { 0xf3fa35c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5a3a35c6, SUC_lexicon_tags_11 },
    { 0x5eba35c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a35cc, SUC_lexicon_tags_29 },
    { 0x3ffa35cd, SUC_lexicon_tags_25 },
    { 0xe09a35cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x57fa35d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbba35d2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7a35d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83a35dc, SUC_lexicon_tags_27 },
    { 0x7e9a35dd, SUC_lexicon_tags_61 },
    { 0x78ba35de, SUC_lexicon_tags_26 },
    { 0x94fa35df, SUC_lexicon_tags_15 },
    { 0xdc7a35e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a35e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717a35ea, SUC_lexicon_tags_37 },
    { 0x20ba35ea, SUC_lexicon_tags_33 },
    { 0x13fa35eb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363a35f4, SUC_lexicon_tags_37 },
    { 0xacba35f5, SUC_lexicon_tags_15 },
    { 0x46ba35f6, SUC_lexicon_tags_27 },
    { 0xd1da35f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873a35fc, SUC_lexicon_tags_60 },
    { 0x241a35fd, SUC_lexicon_tags_25 },
    { 0x589a35fc, SUC_lexicon_tags_12 },
    { 0x167a35ff, SUC_lexicon_tags_4 },
    { 0xe83a3600, SUC_lexicon_tags_21 },
    { 0x4e9a3601, SUC_lexicon_tags_12 },
    { 0xa63a3602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0da3609, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x7e9a360b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fa3611, SUC_lexicon_tags_63 },
    { 0xd37a3611, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177a3616, SUC_lexicon_tags_17 },
    { 0xfcda3617, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x767a3619, SUC_lexicon_tags_0 },
    { 0x1a7a361a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ba361f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1a3622, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbffa3624, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61da3627, SUC_lexicon_tags_30 },
    { 0x2aba3628, SUC_lexicon_tags_29 },
    { 0xf55a3627, SUC_lexicon_tags_17 },
    { 0xe0da362a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x581a362d, SUC_lexicon_tags_43 },
    { 0x52ba362e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965a3638, SUC_lexicon_tags_42 },
    { 0x40fa3638, SUC_lexicon_tags_33 },
    { 0x397a363a, SUC_lexicon_tags_11 },
    { 0x981a363b, SUC_lexicon_tags_2 },
    { 0xca3a363b, SUC_lexicon_tags_39 },
    { 0x869a363a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fba3642, SUC_lexicon_tags_66 },
    { 0xb51a3643, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3646, SUC_lexicon_tags_39 },
    { 0xc4fa3647, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafa364a, SUC_lexicon_tags_25 },
    { 0xd41a364a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13a364e, SUC_lexicon_tags_32 },
    { 0xba3a364f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3da3651, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673a3656, SUC_lexicon_tags_38 },
    { 0x5bfa3657, SUC_lexicon_tags_10 },
    { 0xbb5a3658, SUC_lexicon_tags_1 },
    { 0x9c9a3658, SUC_lexicon_tags_65 },
    { 0x77da3658, SUC_lexicon_tags_25 },
    { 0x4bda365b, SUC_lexicon_tags_47 },
    { 0xf0fa365b, SUC_lexicon_tags_37 },
    { 0x1a9a365b, SUC_lexicon_tags_7 },
    { 0xe95a365e, SUC_lexicon_tags_8 },
    { 0x753a3657, SUC_lexicon_tags_26 },
    { 0xec3a3660, SUC_lexicon_tags_11 },
    { 0xbdda365f, SUC_lexicon_tags_16 },
    { 0x853a3662, SUC_lexicon_tags_25 },
    { 0x3afa3663, SUC_lexicon_tags_12 },
    { 0xa7da3662, SUC_lexicon_tags_42 },
    { 0xc09a3665, SUC_lexicon_tags_8 },
    { 0x4a3a3666, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x825a3669, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717a366d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5a3670, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffda3674, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21a367c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fa367f, SUC_lexicon_tags_47 },
    { 0xd49a3680, SUC_lexicon_tags_0 },
    { 0x84da367f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9a3684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2da3686, SUC_lexicon_tags_11 },
    { 0xdfda3686, SUC_lexicon_tags_47 },
    { 0x32da3687, SUC_lexicon_tags_37 },
    { 0xf59a3688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59da368b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11a3691, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a3a3693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95da3698, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x78fa369a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79a369d, SUC_lexicon_tags_27 },
    { 0xb91a369e, SUC_lexicon_tags_23 },
    { 0x905a369f, SUC_lexicon_tags_15 },
    { 0x91fa36a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7a36ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fa36b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5a36b2, SUC_lexicon_tags_2 },
    { 0xed3a36b3, SUC_lexicon_tags_7 },
    { 0xa23a36b4, SUC_lexicon_tags_30 },
    { 0xa8fa36b5, SUC_lexicon_tags_37 },
    { 0xe6da36b2, SUC_lexicon_tags_2 },
    { 0x259a36b7, SUC_lexicon_tags_16 },
    { 0x673a36b7, SUC_lexicon_tags_1 },
    { 0x34fa36b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d3a36bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x417a36be, SUC_lexicon_tags_1 },
    { 0x8bfa36bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9da36c1, SUC_lexicon_tags_2 },
    { 0x941a36c2, SUC_lexicon_tags_2 },
    { 0x399a36c3, SUC_lexicon_tags_25 },
    { 0xb73a36c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffa36c8, SUC_lexicon_tags_26 },
    { 0xdeba36c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efa36ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3a36d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3a36de, SUC_lexicon_tags_15 },
    { 0x1eda36df, SUC_lexicon_tags_14 },
    { 0xca1a36e0, SUC_lexicon_tags_29 },
    { 0x873a36e1, SUC_lexicon_tags_30 },
    { 0x163a36e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ada36e5, SUC_lexicon_tags_12 },
    { 0x77fa36e5, SUC_lexicon_tags_37 },
    { 0xfba36e5, SUC_lexicon_tags_15 },
    { 0x63da36e8, SUC_lexicon_tags_65 },
    { 0xb0ba36e9, SUC_lexicon_tags_9 },
    { 0xa75a36e6, SUC_lexicon_tags_37 },
    { 0xfdda36eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3a36f2, SUC_lexicon_tags_31 },
    { 0xcada36f2, SUC_lexicon_tags_14 },
    { 0x6c5a36f4, SUC_lexicon_tags_37 },
    { 0xdb5a36f4, SUC_lexicon_tags_12 },
    { 0x56ba36f6, SUC_lexicon_tags_47 },
    { 0x55a36f6, SUC_lexicon_tags_12 },
    { 0x1eba36f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1e7a36fa, SUC_lexicon_tags_1 },
    { 0xdcba36fb, SUC_lexicon_tags_47 },
    { 0x1d5a36fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79a3700, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efa3703, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11a3706, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107a370b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffa370e, SUC_lexicon_tags_65 },
    { 0x25a370f, SUC_lexicon_tags_47 },
    { 0xe71a370f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1a3714, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249a371a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327a371e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbba3723, SUC_lexicon_tags_84 },
    { 0xa4ba3723, SUC_lexicon_tags_30 },
    { 0x44da3725, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357a372c, SUC_lexicon_tags_8 },
    { 0xb6da372d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9a3730, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x781a3733, SUC_lexicon_tags_37 },
    { 0xd0ba3733, SUC_lexicon_tags_172 },
    { 0x3f5a3735, SUC_lexicon_tags_7 },
    { 0x8efa3736, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xa63a3738, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fa373e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3a3741, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x705a3744, SUC_lexicon_tags_4 },
    { 0x257a3745, SUC_lexicon_tags_48 },
    { 0xccfa3746, SUC_lexicon_tags_37 },
    { 0x62ba3745, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf3ba3749, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffa374d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5da3751, SUC_lexicon_tags_14 },
    { 0xf77a3752, SUC_lexicon_tags_9 },
    { 0x679a3751, SUC_lexicon_tags_3 },
    { 0xd97a3751, SUC_lexicon_tags_34 },
    { 0x837a3752, SUC_lexicon_tags_26 },
    { 0x54da3755, SUC_lexicon_tags_11 },
    { 0x607a3755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7a375a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5ba375c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123a3760, SUC_lexicon_tags_10 },
    { 0xbf9a3761, SUC_lexicon_tags_553 },
    { 0x7ada3760, SUC_lexicon_tags_15 },
    { 0xfbba3763, SUC_lexicon_tags_63 },
    { 0x68fa3760, SUC_lexicon_tags_64 },
    { 0x505a3765, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x53a3768, SUC_lexicon_tags_30 },
    { 0xe55a3769, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1a3770, SUC_lexicon_tags_11 },
    { 0x311a3770, SUC_lexicon_tags_16 },
    { 0x3ba3772, SUC_lexicon_tags_25 },
    { 0xac5a3773, SUC_lexicon_tags_11 },
    { 0x38da3771, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a377a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x42da377d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcba3781, SUC_lexicon_tags_87 },
    { 0xeafa3782, SUC_lexicon_tags_16 },
    { 0x9ada3783, SUC_lexicon_tags_38 },
    { 0x479a3783, SUC_lexicon_tags_55 },
    { 0xed9a3785, SUC_lexicon_tags_63 },
    { 0x53da3781, SUC_lexicon_tags_19 },
    { 0xd13a3787, SUC_lexicon_tags_45 },
    { 0xeada3784, SUC_lexicon_tags_7 },
    { 0xc45a3789, SUC_lexicon_tags_6 },
    { 0xfaba378a, SUC_lexicon_tags_11 },
    { 0xc3ba378b, SUC_lexicon_tags_12 },
    { 0xa0da378c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757a3793, SUC_lexicon_tags_27 },
    { 0x817a3793, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93a379a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xefda379c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991a37a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7a37aa, SUC_lexicon_tags_28 },
    { 0xa89a37ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb73a37ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67ba37af, SUC_lexicon_tags_1 },
    { 0xba37af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5a37b4, SUC_lexicon_tags_25 },
    { 0x75da37b5, SUC_lexicon_tags_47 },
    { 0xb7a37b4, SUC_lexicon_tags_42 },
    { 0x6b1a37b7, SUC_lexicon_tags_12 },
    { 0x57fa37b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11a37ba, SUC_lexicon_tags_77 },
    { 0x5bda37bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9a37c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fa37ca, SUC_lexicon_tags_15 },
    { 0x109a37cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x855a37ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7a37d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797a37e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x695a37e2, SUC_lexicon_tags_37 },
    { 0xcfa37e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5a37e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979a37ef, SUC_lexicon_tags_37 },
    { 0x329a37ef, SUC_lexicon_tags_15 },
    { 0x945a37f1, SUC_lexicon_tags_8 },
    { 0x701a37f1, SUC_lexicon_tags_17 },
    { 0x9e7a37f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5a37f6, SUC_lexicon_tags_46 },
    { 0xc49a37f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x975a37f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515a3801, SUC_lexicon_tags_7 },
    { 0xce9a3802, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1a3806, SUC_lexicon_tags_2 },
    { 0x22da3806, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9a380a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa93a380c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf0ba380e, SUC_lexicon_tags_27 },
    { 0x96da380e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fda3812, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x857a3815, SUC_lexicon_tags_45 },
    { 0x72ba3815, SUC_lexicon_tags_0 },
    { 0xb9a3817, SUC_lexicon_tags_16 },
    { 0x161a3818, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x291a381a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96ba381c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x679a381e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e1a3820, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x551a3822, SUC_lexicon_tags_45 },
    { 0x87da3823, SUC_lexicon_tags_25 },
    { 0x9d5a3822, SUC_lexicon_tags_49 },
    { 0x27a3823, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffa382c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfeba382e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x141a3830, SUC_lexicon_tags_32 },
    { 0x679a3831, SUC_lexicon_tags_29 },
    { 0x47fa3832, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x165a3834, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491a3839, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde9a383b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfa383e, SUC_lexicon_tags_8 },
    { 0xf07a383f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c7a3841, SUC_lexicon_tags_31 },
    { 0x2e3a3841, SUC_lexicon_tags_37 },
    { 0x82da3842, SUC_lexicon_tags_2 },
    { 0x52ba3842, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fa3849, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861a384e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fda3850, SUC_lexicon_tags_45 },
    { 0xe2da3851, SUC_lexicon_tags_8 },
    { 0x9eda3852, SUC_lexicon_tags_2 },
    { 0x4d1a3851, SUC_lexicon_tags_37 },
    { 0xcda3851, SUC_lexicon_tags_207 },
    { 0x49fa3851, SUC_lexicon_tags_7 },
    { 0x3eda3854, SUC_lexicon_tags_43 },
    { 0x941a3855, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2da3864, SUC_lexicon_tags_15 },
    { 0x4fba3864, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x13da3867, SUC_lexicon_tags_87 },
    { 0xb1ba3867, SUC_lexicon_tags_62 },
    { 0xf21a3869, SUC_lexicon_tags_15 },
    { 0xba3868, SUC_lexicon_tags_26 },
    { 0x579a386a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867a386f, SUC_lexicon_tags_2 },
    { 0x79fa3870, SUC_lexicon_tags_11 },
    { 0xf8da3871, SUC_lexicon_tags_2 },
    { 0x72ba3871, SUC_lexicon_tags_53 },
    { 0x74da3873, SUC_lexicon_tags_2 },
    { 0xcd5a3874, SUC_lexicon_tags_17 },
    { 0x113a3873, SUC_lexicon_tags_25 },
    { 0xacda3874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a387d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ba3883, SUC_lexicon_tags_11 },
    { 0x865a3883, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9a388b, SUC_lexicon_tags_6 },
    { 0x987a388c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fa3891, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ba3896, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3a3899, SUC_lexicon_tags_23 },
    { 0x4a9a389a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x539a389c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdada389f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609a38a3, SUC_lexicon_tags_19 },
    { 0x573a38a3, SUC_lexicon_tags_48 },
    { 0x2f3a38a5, SUC_lexicon_tags_19 },
    { 0xf9fa38a4, SUC_lexicon_tags_25 },
    { 0xc73a38a7, SUC_lexicon_tags_5 },
    { 0x8a9a38a6, SUC_lexicon_tags_28 },
    { 0x59fa38a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dba38ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29a38b0, SUC_lexicon_tags_38 },
    { 0xbb7a38b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae9a38b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35da38b5, SUC_lexicon_tags_29 },
    { 0xffda38b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491a38ba, SUC_lexicon_tags_25 },
    { 0xe11a38bb, SUC_lexicon_tags_60 },
    { 0x36ba38bc, SUC_lexicon_tags_33 },
    { 0x909a38bc, SUC_lexicon_tags_54 },
    { 0xea1a38bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3a38c3, SUC_lexicon_tags_54 },
    { 0xb3ba38c3, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfa38c8, SUC_lexicon_tags_21 },
    { 0x4cda38c8, SUC_lexicon_tags_27 },
    { 0x29fa38c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7a7a38cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a38d6, SUC_lexicon_tags_26 },
    { 0xe09a38d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a38da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x733a38dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a3a38de, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01a38e1, SUC_lexicon_tags_37 },
    { 0xb29a38e1, SUC_lexicon_tags_37 },
    { 0x2fa38e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859a38ed, SUC_lexicon_tags_25 },
    { 0x1e3a38ee, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbfa38f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8da38f3, SUC_lexicon_tags_1 },
    { 0x5d7a38f3, SUC_lexicon_tags_31 },
    { 0xc6ba38f5, SUC_lexicon_tags_4 },
    { 0xde9a38f6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15a38ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a1a3901, SUC_lexicon_tags_11 },
    { 0xf71a3902, SUC_lexicon_tags_19 },
    { 0xf75a3903, SUC_lexicon_tags_15 },
    { 0xa7ba3903, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819a3908, SUC_lexicon_tags_91 },
    { 0xcb3a3909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dda390b, SUC_lexicon_tags_16 },
    { 0x235a390c, SUC_lexicon_tags_33 },
    { 0x37a390c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1a3910, SUC_lexicon_tags_15 },
    { 0xa1a3910, SUC_lexicon_tags_7 },
    { 0x195a3911, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49da391b, SUC_lexicon_tags_35 },
    { 0x3e5a391b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xabda391e, SUC_lexicon_tags_6 },
    { 0xa57a391f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1f1a3921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x277a3923, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83da3928, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67a392d, SUC_lexicon_tags_2 },
    { 0x883a392d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53a3937, SUC_lexicon_tags_61 },
    { 0xc0fa3937, SUC_lexicon_tags_39 },
    { 0x4f1a3939, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x117a393b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ba393f, SUC_lexicon_tags_39 },
    { 0x5ba3940, SUC_lexicon_tags_2 },
    { 0x9e3a3941, SUC_lexicon_tags_19 },
    { 0x361a3942, SUC_lexicon_tags_33 },
    { 0x3b3a393f, SUC_lexicon_tags_3 },
    { 0x545a393f, SUC_lexicon_tags_17 },
    { 0x267a3945, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fda3948, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859a394c, SUC_lexicon_tags_125 },
    { 0x89a394d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe3da394f, SUC_lexicon_tags_0 },
    { 0x7d1a3950, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969a3957, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351a395d, SUC_lexicon_tags_11 },
    { 0x61a395e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37da3968, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x283a396b, SUC_lexicon_tags_16 },
    { 0xb1ba396c, SUC_lexicon_tags_7 },
    { 0x2fda396d, SUC_lexicon_tags_7 },
    { 0xfcda396c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25a3971, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40ba3973, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1a3976, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67a397b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaba3984, SUC_lexicon_tags_11 },
    { 0xc19a3984, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231a398c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x105a398f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9a3992, SUC_lexicon_tags_151 },
    { 0xf21a3992, SUC_lexicon_tags_7 },
    { 0xc7ba3993, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37da3997, SUC_lexicon_tags_37 },
    { 0xddfa3997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x473a399b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3a39a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89da39a4, SUC_lexicon_tags_7 },
    { 0xa21a39a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1da39a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdbfa39ab, SUC_lexicon_tags_49 },
    { 0xbc1a39ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4da39b8, SUC_lexicon_tags_7 },
    { 0x61fa39b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49da39bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ba39be, SUC_lexicon_tags_27 },
    { 0x9fa39bf, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ada39c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7a39ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa39d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55a39d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6da39de, SUC_lexicon_tags_55 },
    { 0x955a39de, SUC_lexicon_tags_11 },
    { 0x1b9a39e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x859a39e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bba39e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x223a39e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7a39ed, SUC_lexicon_tags_7 },
    { 0xcc1a39ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xb09a39f0, SUC_lexicon_tags_66 },
    { 0x13a39f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x587a39f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x499a39f5, SUC_lexicon_tags_7 },
    { 0x803a39f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x643a39f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1a39fb, SUC_lexicon_tags_1 },
    { 0x627a39fc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fa3a01, SUC_lexicon_tags_64 },
    { 0xde7a3a02, SUC_lexicon_tags_31 },
    { 0xaba3a03, SUC_lexicon_tags_11 },
    { 0x281a3a01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9a3a08, SUC_lexicon_tags_9 },
    { 0xb7a3a08, SUC_lexicon_tags_7 },
    { 0x391a3a0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a3a0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da3a17, SUC_lexicon_tags_7 },
    { 0xc3a3a18, SUC_lexicon_tags_37 },
    { 0x6c7a3a19, SUC_lexicon_tags_2 },
    { 0xcfda3a1a, SUC_lexicon_tags_9 },
    { 0xa27a3a1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfba3a20, SUC_lexicon_tags_31 },
    { 0x5d7a3a20, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69a3a2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c1a3a2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3a3a33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3a3a36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8b5a3a38, SUC_lexicon_tags_2 },
    { 0xd81a3a38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55ba3a3b, SUC_lexicon_tags_14 },
    { 0x7f5a3a3c, SUC_lexicon_tags_28 },
    { 0xa8da3a3d, SUC_lexicon_tags_7 },
    { 0xd4fa3a3d, SUC_lexicon_tags_43 },
    { 0x35ba3a3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3a3a44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x841a3a47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cfa3a49, SUC_lexicon_tags_14 },
    { 0x2b9a3a4a, SUC_lexicon_tags_17 },
    { 0xbfda3a4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7a3a4e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc7da3a50, SUC_lexicon_tags_6 },
    { 0x5eda3a51, SUC_lexicon_tags_0 },
    { 0x86ba3a52, SUC_lexicon_tags_42 },
    { 0xeefa3a51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a3a58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x271a3a5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb7a3a5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5eba3a5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf61a3a60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da3a64, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cda3a67, SUC_lexicon_tags_11 },
    { 0x8fa3a68, SUC_lexicon_tags_17 },
    { 0xd2da3a67, SUC_lexicon_tags_17 },
    { 0xd17a3a68, SUC_lexicon_tags_11 },
    { 0x989a3a67, SUC_lexicon_tags_37 },
    { 0xd6da3a68, SUC_lexicon_tags_32 },
    { 0x9d7a3a6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4dfa3a6f, SUC_lexicon_tags_63 },
    { 0x2f9a3a70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2eba3a72, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75a3a75, SUC_lexicon_tags_82 },
    { 0xdbfa3a76, SUC_lexicon_tags_7 },
    { 0xfb5a3a76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9a3a7a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb97a3a7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469a3a82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x319a3a84, SUC_lexicon_tags_17 },
    { 0x50da3a85, SUC_lexicon_tags_49 },
    { 0x815a3a85, SUC_lexicon_tags_17 },
    { 0xf8da3a87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x701a3a89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x96da3a8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32ba3a8d, SUC_lexicon_tags_2 },
    { 0x70da3a8d, SUC_lexicon_tags_1 },
    { 0xd3a3a8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26fa3a91, SUC_lexicon_tags_47 },
    { 0xf4da3a91, SUC_lexicon_tags_2 },
    { 0xbdda3a93, SUC_lexicon_tags_16 },
    { 0x7dfa3a94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15a3a9a, SUC_lexicon_tags_0 },
    { 0x1a3a9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfa3aa0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34ba3aa2, SUC_lexicon_tags_37 },
    { 0x167a3aa3, SUC_lexicon_tags_47 },
    { 0xa77a3aa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd57a3aa6, SUC_lexicon_tags_1 },
    { 0xd57a3aa6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfda3aaa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21a3ab2, SUC_lexicon_tags_15 },
    { 0xe05a3ab3, SUC_lexicon_tags_47 },
    { 0x6aba3ab4, SUC_lexicon_tags_42 },
    { 0xedda3ab5, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25a3ab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2da3aba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfbda3abc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bda3ac0, SUC_lexicon_tags_15 },
    { 0xd0fa3ac0, SUC_lexicon_tags_12 },
    { 0xcb9a3ac2, SUC_lexicon_tags_160 },
    { 0x82ba3ac3, SUC_lexicon_tags_1 },
    { 0xb85a3ac4, SUC_lexicon_tags_37 },
    { 0xbd5a3ac5, SUC_lexicon_tags_14 },
    { 0xd3a3ac6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd7a3ac8, SUC_lexicon_tags_63 },
    { 0xf39a3ac9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x497a3acc, SUC_lexicon_tags_0 },
    { 0xffa3acc, SUC_lexicon_tags_591 },
    { 0x713a3acc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bda3ad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7a3ad4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449a3adc, SUC_lexicon_tags_37 },
    { 0xa31a3adc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21a3ae1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa5ba3ae3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbba3ae7, SUC_lexicon_tags_7 },
    { 0x6c5a3ae7, SUC_lexicon_tags_31 },
    { 0xaa9a3ae9, SUC_lexicon_tags_2 },
    { 0x49ba3ae8, SUC_lexicon_tags_49 },
    { 0x95ba3aeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xffda3aee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25a3af1, SUC_lexicon_tags_23 },
    { 0xe69a3af1, SUC_lexicon_tags_62 },
    { 0x27da3af3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9a3af6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7b3a3af8, SUC_lexicon_tags_38 },
    { 0xfb9a3af8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3a3afe, SUC_lexicon_tags_7 },
    { 0xbf1a3afe, SUC_lexicon_tags_39 },
    { 0xda3b00, SUC_lexicon_tags_55 },
    { 0xc43a3b01, SUC_lexicon_tags_8 },
    { 0xe17a3b02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd99a3b04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xada3b06, SUC_lexicon_tags_23 },
    { 0xdafa3b06, SUC_lexicon_tags_84 },
    { 0x2fba3b08, SUC_lexicon_tags_12 },
    { 0xc3ba3b08, SUC_lexicon_tags_26 },
    { 0x19da3b09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fa3b10, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x175a3b13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x231a3b16, SUC_lexicon_tags_7 },
    { 0x1c3a3b17, SUC_lexicon_tags_1 },
    { 0x979a3b17, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83a3b1b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a3b20, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x9e7a3b22, SUC_lexicon_tags_15 },
    { 0x693a3b22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b1a3b25, SUC_lexicon_tags_23 },
    { 0x537a3b26, SUC_lexicon_tags_25 },
    { 0xca9a3b27, SUC_lexicon_tags_37 },
    { 0x161a3b25, SUC_lexicon_tags_53 },
    { 0x88da3b29, SUC_lexicon_tags_63 },
    { 0x705a3b27, SUC_lexicon_tags_12 },
    { 0xbc1a3b25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9a3b31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55a3b3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5a3b40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8eba3b42, SUC_lexicon_tags_0 },
    { 0xb75a3b43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb1a3b45, SUC_lexicon_tags_79 },
    { 0xabfa3b46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x105a3b48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x457a3b4a, SUC_lexicon_tags_48 },
    { 0x941a3b4a, SUC_lexicon_tags_7 },
    { 0xa79a3b4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x267a3b4e, SUC_lexicon_tags_37 },
    { 0x70da3b4f, SUC_lexicon_tags_11 },
    { 0x80da3b50, SUC_lexicon_tags_28 },
    { 0x1d1a3b51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01a3b54, SUC_lexicon_tags_818 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fa3b58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fa3b5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199a3b5f, SUC_lexicon_tags_77 },
    { 0x177a3b60, SUC_lexicon_tags_30 },
    { 0xc9da3b61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfa3b66, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x417a3b69, SUC_lexicon_tags_27 },
    { 0xe75a3b6a, SUC_lexicon_tags_1069 },
    { 0x2dba3b6b, SUC_lexicon_tags_31 },
    { 0xcafa3b6c, SUC_lexicon_tags_11 },
    { 0xdcda3b6d, SUC_lexicon_tags_28 },
    { 0x68da3b6e, SUC_lexicon_tags_17 },
    { 0x5aba3b6f, SUC_lexicon_tags_12 },
    { 0xb15a3b6d, SUC_lexicon_tags_9 },
    { 0xf69a3b6d, SUC_lexicon_tags_11 },
    { 0x1b1a3b72, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb11a3b74, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fda3b77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeda3b7a, SUC_lexicon_tags_0 },
    { 0x49ba3b7a, SUC_lexicon_tags_30 },
    { 0x4b5a3b7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319a3b80, SUC_lexicon_tags_25 },
    { 0x81fa3b81, SUC_lexicon_tags_47 },
    { 0x41fa3b82, SUC_lexicon_tags_12 },
    { 0x593a3b81, SUC_lexicon_tags_12 },
    { 0x6e1a3b84, SUC_lexicon_tags_0 },
    { 0xe29a3b82, SUC_lexicon_tags_46 },
    { 0xc3a3b83, SUC_lexicon_tags_23 },
    { 0xa13a3b81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x947a3b8a, SUC_lexicon_tags_29 },
    { 0xaf3a3b8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5a3b92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43da3b97, SUC_lexicon_tags_13 },
    { 0x22da3b98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bda3b9f, SUC_lexicon_tags_26 },
    { 0xbafa3ba0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeafa3ba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce3a3ba6, SUC_lexicon_tags_17 },
    { 0xe4fa3ba7, SUC_lexicon_tags_23 },
    { 0x7a1a3ba6, SUC_lexicon_tags_38 },
    { 0x9c9a3ba9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9a3bac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27da3bb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377a3bba, SUC_lexicon_tags_12 },
    { 0x7b9a3bbb, SUC_lexicon_tags_11 },
    { 0xad7a3bbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663a3bc0, SUC_lexicon_tags_37 },
    { 0x257a3bc0, SUC_lexicon_tags_7 },
    { 0x95a3bc2, SUC_lexicon_tags_4 },
    { 0x621a3bc3, SUC_lexicon_tags_37 },
    { 0xb35a3bc4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1da3bc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da3bcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ba3bd0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fa3bd3, SUC_lexicon_tags_0 },
    { 0xa91a3bd3, SUC_lexicon_tags_9 },
    { 0xb21a3bd5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a3bd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3bdd, SUC_lexicon_tags_39 },
    { 0x3d9a3bde, SUC_lexicon_tags_45 },
    { 0x7bda3bdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x307a3be1, SUC_lexicon_tags_82 },
    { 0x2cfa3be1, SUC_lexicon_tags_32 },
    { 0x143a3be3, SUC_lexicon_tags_11 },
    { 0x143a3be4, SUC_lexicon_tags_15 },
    { 0xc05a3be5, SUC_lexicon_tags_27 },
    { 0xc15a3be4, SUC_lexicon_tags_0 },
    { 0x605a3be7, SUC_lexicon_tags_11 },
    { 0xaf9a3be8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5a3bed, SUC_lexicon_tags_48 },
    { 0xdfa3bed, SUC_lexicon_tags_264 },
    { 0x483a3bef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3a3bf8, SUC_lexicon_tags_10 },
    { 0xedba3bf9, SUC_lexicon_tags_17 },
    { 0x267a3bf9, SUC_lexicon_tags_45 },
    { 0x5eba3bf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdfa3bfd, SUC_lexicon_tags_17 },
    { 0x767a3bfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9a3c01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19a3c06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967a3c0a, SUC_lexicon_tags_17 },
    { 0x74da3c0b, SUC_lexicon_tags_25 },
    { 0x8fa3c0c, SUC_lexicon_tags_34 },
    { 0x9dfa3c0d, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0x415a3c0f, SUC_lexicon_tags_611 },
    { 0x8ada3c0f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x757a3c13, SUC_lexicon_tags_46 },
    { 0x981a3c13, SUC_lexicon_tags_47 },
    { 0xadda3c15, SUC_lexicon_tags_33 },
    { 0x47da3c16, SUC_lexicon_tags_24 },
    { 0x2b5a3c16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x50fa3c19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925a3c1e, SUC_lexicon_tags_38 },
    { 0xfb9a3c1f, SUC_lexicon_tags_1 },
    { 0x6c5a3c20, SUC_lexicon_tags_37 },
    { 0xd5fa3c21, SUC_lexicon_tags_61 },
    { 0x6fda3c22, SUC_lexicon_tags_14 },
    { 0x7c9a3c23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9a3c28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159a3c2c, SUC_lexicon_tags_11 },
    { 0x27fa3c2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93da3c34, SUC_lexicon_tags_60 },
    { 0x82ba3c35, SUC_lexicon_tags_17 },
    { 0x723a3c35, SUC_lexicon_tags_9 },
    { 0x977a3c37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x98da3c3a, SUC_lexicon_tags_48 },
    { 0x35ba3c3a, SUC_lexicon_tags_17 },
    { 0xad1a3c3c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eba3c3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8da3c44, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf09a3c46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fa3c4b, SUC_lexicon_tags_32 },
    { 0xdf7a3c4b, SUC_lexicon_tags_2 },
    { 0x819a3c4d, SUC_lexicon_tags_27 },
    { 0x8afa3c4d, SUC_lexicon_tags_38 },
    { 0xeb9a3c4d, SUC_lexicon_tags_33 },
    { 0x90ba3c50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3ba3c52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93a3c55, SUC_lexicon_tags_87 },
    { 0xd65a3c56, SUC_lexicon_tags_1 },
    { 0xaf1a3c57, SUC_lexicon_tags_11 },
    { 0xc69a3c57, SUC_lexicon_tags_66 },
    { 0xbeba3c59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03a3c5e, SUC_lexicon_tags_15 },
    { 0x875a3c5f, SUC_lexicon_tags_17 },
    { 0xa5fa3c5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x493a3c63, SUC_lexicon_tags_43 },
    { 0xc51a3c64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287a3c70, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc07a3c72, SUC_lexicon_tags_1 },
    { 0x73fa3c73, SUC_lexicon_tags_72 },
    { 0x47da3c74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x383a3c76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97da3c7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67a3c7d, SUC_lexicon_tags_25 },
    { 0x9bba3c7e, SUC_lexicon_tags_63 },
    { 0xea9a3c7f, SUC_lexicon_tags_15 },
    { 0x7d5a3c80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x987a3c82, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ba3c86, SUC_lexicon_tags_61 },
    { 0xb5da3c87, SUC_lexicon_tags_0 },
    { 0xe8da3c88, SUC_lexicon_tags_63 },
    { 0x60da3c87, SUC_lexicon_tags_38 },
    { 0x485a3c88, SUC_lexicon_tags_17 },
    { 0x91da3c8a, SUC_lexicon_tags_61 },
    { 0xb0da3c86, SUC_lexicon_tags_25 },
    { 0xf51a3c89, SUC_lexicon_tags_13 },
    { 0x48ba3c8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355a3c92, SUC_lexicon_tags_29 },
    { 0x559a3c93, SUC_lexicon_tags_17 },
    { 0xb6da3c94, SUC_lexicon_tags_37 },
    { 0xe8fa3c95, SUC_lexicon_tags_7 },
    { 0xe51a3c94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeffa3c98, SUC_lexicon_tags_30 },
    { 0xa11a3c99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe55a3c9b, SUC_lexicon_tags_19 },
    { 0xd79a3c9b, SUC_lexicon_tags_15 },
    { 0x805a3c9b, SUC_lexicon_tags_14 },
    { 0x531a3c9b, SUC_lexicon_tags_23 },
    { 0x221a3c9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda3ca3, SUC_lexicon_tags_2 },
    { 0x1c5a3ca4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x14ba3ca6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x371a3ca8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11da3caa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xadda3cac, SUC_lexicon_tags_31 },
    { 0xd3fa3cac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x861a3cb0, SUC_lexicon_tags_33 },
    { 0x15da3cb1, SUC_lexicon_tags_14 },
    { 0x4fa3cb1, SUC_lexicon_tags_22 },
    { 0x71a3cb3, SUC_lexicon_tags_25 },
    { 0x2fba3cb4, SUC_lexicon_tags_1 },
    { 0xf9ba3cb5, SUC_lexicon_tags_31 },
    { 0xf75a3cb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881a3cbb, SUC_lexicon_tags_11 },
    { 0xcf9a3cbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913a3cc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x571a3cc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3a3cc5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43a3cc9, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9a3ccc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409a3cd0, SUC_lexicon_tags_15 },
    { 0x72da3cd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd41a3cd3, SUC_lexicon_tags_33 },
    { 0xdd5a3cd3, SUC_lexicon_tags_1 },
    { 0x41ba3cd4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xce5a3cd7, SUC_lexicon_tags_2 },
    { 0xf59a3cd7, SUC_lexicon_tags_135 },
    { 0x13ba3cd9, SUC_lexicon_tags_1 },
    { 0x991a3cd9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a3ce0, SUC_lexicon_tags_75 },
    { 0x729a3ce1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ba3ce4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dda3ce7, SUC_lexicon_tags_32 },
    { 0x2cda3ce8, SUC_lexicon_tags_25 },
    { 0x8e3a3ce8, SUC_lexicon_tags_2 },
    { 0xd9a3ce8, SUC_lexicon_tags_25 },
    { 0xf9da3cea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5a3cef, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fa3cf3, SUC_lexicon_tags_28 },
    { 0x7e5a3cf4, SUC_lexicon_tags_37 },
    { 0x305a3cf5, SUC_lexicon_tags_35 },
    { 0xdfba3cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee1a3cf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa55a3cfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217a3d03, SUC_lexicon_tags_37 },
    { 0x68da3d04, SUC_lexicon_tags_48 },
    { 0xd9fa3d03, SUC_lexicon_tags_2 },
    { 0x70ba3d06, SUC_lexicon_tags_17 },
    { 0x539a3d07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d3a3d09, SUC_lexicon_tags_33 },
    { 0x559a3d09, SUC_lexicon_tags_28 },
    { 0xad9a3d0b, SUC_lexicon_tags_7 },
    { 0xff7a3d0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa75a3d0e, SUC_lexicon_tags_8 },
    { 0xa9ba3d0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb7a3d11, SUC_lexicon_tags_1 },
    { 0x735a3d12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9a3d15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffa3d19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5a3d1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3da3d1f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7a3d23, SUC_lexicon_tags_2 },
    { 0x653a3d23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3a3d29, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23a3d2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7a3d31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x80da3d34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8f5a3d36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfa3d3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa49a3d3e, SUC_lexicon_tags_11 },
    { 0x149a3d3e, SUC_lexicon_tags_15 },
    { 0xd53a3d40, SUC_lexicon_tags_17 },
    { 0xfb1a3d41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd45a3d43, SUC_lexicon_tags_16 },
    { 0x745a3d43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x95ba3d46, SUC_lexicon_tags_1 },
    { 0x8f9a3d46, SUC_lexicon_tags_10 },
    { 0xf49a3d46, SUC_lexicon_tags_3 },
    { 0x3c5a3d49, SUC_lexicon_tags_28 },
    { 0x251a3d4a, SUC_lexicon_tags_11 },
    { 0x9cfa3d4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efa3d58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a3d5d, SUC_lexicon_tags_8 },
    { 0xb7fa3d5e, SUC_lexicon_tags_15 },
    { 0xa01a3d5e, SUC_lexicon_tags_55 },
    { 0x91ba3d60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x441a3d62, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ba3d66, SUC_lexicon_tags_33 },
    { 0x495a3d66, SUC_lexicon_tags_47 },
    { 0xc01a3d66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fa3d6b, SUC_lexicon_tags_34 },
    { 0xa81a3d6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x31fa3d6f, SUC_lexicon_tags_19 },
    { 0x773a3d6f, SUC_lexicon_tags_25 },
    { 0xbf7a3d71, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbcda3d73, SUC_lexicon_tags_12 },
    { 0x689a3d73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafa3d78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35da3d7c, SUC_lexicon_tags_84 },
    { 0x1a7a3d7d, SUC_lexicon_tags_49 },
    { 0x48da3d7d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9da3d89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887a3d91, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7a3d96, SUC_lexicon_tags_7 },
    { 0x523a3d96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa3d9b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1a3d9e, SUC_lexicon_tags_25 },
    { 0x503a3d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25a3da2, SUC_lexicon_tags_112 },
    { 0x397a3da2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9a3db1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9da3db4, SUC_lexicon_tags_7 },
    { 0x343a3db5, SUC_lexicon_tags_47 },
    { 0xe95a3db6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd81a3db8, SUC_lexicon_tags_17 },
    { 0xba9a3db9, SUC_lexicon_tags_57 },
    { 0x34da3dba, SUC_lexicon_tags_65 },
    { 0xc71a3dbb, SUC_lexicon_tags_37 },
    { 0xee3a3dbc, SUC_lexicon_tags_15 },
    { 0x58da3dbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ba3dc0, SUC_lexicon_tags_6 },
    { 0xf31a3dc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b9a3dc3, SUC_lexicon_tags_191 },
    { 0x17da3dc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fba3dc7, SUC_lexicon_tags_20 },
    { 0x10da3dc7, SUC_lexicon_tags_37 },
    { 0x8c5a3dc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9a3dcd, SUC_lexicon_tags_34 },
    { 0x5c9a3dce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x859a3dd0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efa3dd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ba3dda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8da3ddc, SUC_lexicon_tags_14 },
    { 0x6c5a3ddd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x125a3de0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7a3de3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50da3deb, SUC_lexicon_tags_11 },
    { 0x44da3deb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69a3df0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9a3df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5da3df5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605a3df9, SUC_lexicon_tags_37 },
    { 0x40ba3df9, SUC_lexicon_tags_12 },
    { 0x9d5a3dfb, SUC_lexicon_tags_17 },
    { 0xa3a3dfc, SUC_lexicon_tags_1 },
    { 0xfeba3dfd, SUC_lexicon_tags_25 },
    { 0xaa1a3dfc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f5a3e00, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdf1a3e02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577a3e09, SUC_lexicon_tags_1 },
    { 0x231a3e0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981a3e10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b3a3e12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31da3e16, SUC_lexicon_tags_25 },
    { 0x733a3e17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff9a3e19, SUC_lexicon_tags_25 },
    { 0x255a3e1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7a3e1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f3a3e1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aba3e22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b5a3e24, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55a3e27, SUC_lexicon_tags_8 },
    { 0x93da3e27, SUC_lexicon_tags_40 },
    { 0x1d3a3e29, SUC_lexicon_tags_37 },
    { 0x8a1a3e29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf23a3e2c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x51fa3e2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4da3e35, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9a3e39, SUC_lexicon_tags_6 },
    { 0xafa3e3a, SUC_lexicon_tags_37 },
    { 0xa21a3e3a, SUC_lexicon_tags_19 },
    { 0x645a3e3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a3e44, SUC_lexicon_tags_7 },
    { 0x23fa3e44, SUC_lexicon_tags_46 },
    { 0x13fa3e44, SUC_lexicon_tags_7 },
    { 0xf8ba3e47, SUC_lexicon_tags_39 },
    { 0x4e1a3e46, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x795a3e4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afa3e4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1a3e50, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eda3e54, SUC_lexicon_tags_75 },
    { 0x253a3e55, SUC_lexicon_tags_11 },
    { 0x579a3e56, SUC_lexicon_tags_7 },
    { 0x6cda3e57, SUC_lexicon_tags_43 },
    { 0x3dba3e58, SUC_lexicon_tags_37 },
    { 0xf2ba3e59, SUC_lexicon_tags_81 },
    { 0x577a3e5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7a3e5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28ba3e64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbda3e67, SUC_lexicon_tags_33 },
    { 0xdc9a3e67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x247a3e6a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21a3e6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba5a3e6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3a3e74, SUC_lexicon_tags_48 },
    { 0xca1a3e74, SUC_lexicon_tags_68 },
    { 0xcafa3e74, SUC_lexicon_tags_56 },
    { 0xfaba3e77, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a3e7b, SUC_lexicon_tags_17 },
    { 0x7bba3e7c, SUC_lexicon_tags_33 },
    { 0xfbfa3e7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x967a3e80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39a3e85, SUC_lexicon_tags_23 },
    { 0x27ba3e86, SUC_lexicon_tags_32 },
    { 0xd49a3e87, SUC_lexicon_tags_1 },
    { 0x1fba3e87, SUC_lexicon_tags_2 },
    { 0x8bba3e89, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643a3e90, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ba3e94, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9a3e99, SUC_lexicon_tags_25 },
    { 0x49da3e9a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe7ba3e9c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x73da3e9e, SUC_lexicon_tags_15 },
    { 0xf25a3e9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfa3ea5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fa3ea9, SUC_lexicon_tags_1 },
    { 0x199a3ea9, SUC_lexicon_tags_3 },
    { 0x853a3eaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a3eae, SUC_lexicon_tags_21 },
    { 0x9a5a3eaf, SUC_lexicon_tags_1 },
    { 0x88ba3eb0, SUC_lexicon_tags_9 },
    { 0x9efa3eb0, SUC_lexicon_tags_25 },
    { 0x2a3a3eb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09a3eb7, SUC_lexicon_tags_14 },
    { 0x60fa3eb7, SUC_lexicon_tags_77 },
    { 0x1f9a3eb9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x437a3ebb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ba3ebf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x723a3ec1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a3ec6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605a3eca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fa3ece, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x973a3ed0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987a3ed5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13a3ed9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8eba3edb, SUC_lexicon_tags_33 },
    { 0xeba3edc, SUC_lexicon_tags_0 },
    { 0xae3a3edc, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34da3ee1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609a3ee6, SUC_lexicon_tags_12 },
    { 0x497a3ee7, SUC_lexicon_tags_7 },
    { 0xe7ba3ee8, SUC_lexicon_tags_37 },
    { 0x2eba3ee6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x647a3eec, SUC_lexicon_tags_2 },
    { 0x5a3eed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d7a3eef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5a3ef5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x17a3ef7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245a3efb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9a5a3efd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa31a3eff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7a3f05, SUC_lexicon_tags_37 },
    { 0xd33a3f06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479a3f0c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27a3f0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb13a3f11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x991a3f13, SUC_lexicon_tags_11 },
    { 0x3dfa3f13, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x27a3f17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb07a3f19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3a3f23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba9a3f25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfeba3f27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71a3f2d, SUC_lexicon_tags_31 },
    { 0x81a3f2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1a3f31, SUC_lexicon_tags_106 },
    { 0xb5a3f31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ba3f35, SUC_lexicon_tags_65 },
    { 0x64fa3f36, SUC_lexicon_tags_19 },
    { 0xe05a3f37, SUC_lexicon_tags_91 },
    { 0xd77a3f38, SUC_lexicon_tags_8 },
    { 0x285a3f39, SUC_lexicon_tags_32 },
    { 0x6ffa3f36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cba3f3d, SUC_lexicon_tags_7 },
    { 0x7a3a3f3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdada3f43, SUC_lexicon_tags_11 },
    { 0x3e1a3f44, SUC_lexicon_tags_17 },
    { 0x3efa3f45, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x619a3f47, SUC_lexicon_tags_17 },
    { 0xf31a3f48, SUC_lexicon_tags_11 },
    { 0xc99a3f47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x931a3f4c, SUC_lexicon_tags_37 },
    { 0x377a3f4d, SUC_lexicon_tags_63 },
    { 0xa87a3f4d, SUC_lexicon_tags_57 },
    { 0x727a3f4f, SUC_lexicon_tags_2 },
    { 0x987a3f4e, SUC_lexicon_tags_45 },
    { 0xd15a3f51, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x88ba3f53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99da3f59, SUC_lexicon_tags_12 },
    { 0xe0fa3f5a, SUC_lexicon_tags_26 },
    { 0x1d9a3f59, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5dfa3f5d, SUC_lexicon_tags_37 },
    { 0x60fa3f5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x335a3f60, SUC_lexicon_tags_43 },
    { 0x997a3f61, SUC_lexicon_tags_35 },
    { 0xcbfa3f62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b1a3f64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc41a3f66, SUC_lexicon_tags_66 },
    { 0x17fa3f67, SUC_lexicon_tags_18 },
    { 0x35ba3f68, SUC_lexicon_tags_29 },
    { 0x87a3f69, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fa3f70, SUC_lexicon_tags_12 },
    { 0xf3da3f71, SUC_lexicon_tags_1 },
    { 0x595a3f71, SUC_lexicon_tags_18 },
    { 0x8b1a3f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x851a3f75, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753a3f79, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5a3f7c, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56da3f82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fba3f84, SUC_lexicon_tags_72 },
    { 0x1a1a3f85, SUC_lexicon_tags_17 },
    { 0xfda3f86, SUC_lexicon_tags_15 },
    { 0x883a3f86, SUC_lexicon_tags_37 },
    { 0xf89a3f86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde3a3f8a, SUC_lexicon_tags_45 },
    { 0xe55a3f8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x839a3f8d, SUC_lexicon_tags_49 },
    { 0x423a3f8d, SUC_lexicon_tags_7 },
    { 0x7cfa3f8d, SUC_lexicon_tags_0 },
    { 0x22fa3f90, SUC_lexicon_tags_31 },
    { 0x76da3f91, SUC_lexicon_tags_7 },
    { 0x6b5a3f8e, SUC_lexicon_tags_2 },
    { 0xd41a3f91, SUC_lexicon_tags_29 },
    { 0x397a3f8e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x133a3f96, SUC_lexicon_tags_9 },
    { 0xd6fa3f97, SUC_lexicon_tags_0 },
    { 0xefa3f98, SUC_lexicon_tags_2 },
    { 0x543a3f99, SUC_lexicon_tags_0 },
    { 0x805a3f9a, SUC_lexicon_tags_27 },
    { 0xdf7a3f97, SUC_lexicon_tags_30 },
    { 0xd0fa3f9c, SUC_lexicon_tags_1 },
    { 0x8f5a3f9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5a3fa2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ada3fa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583a3fab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53a3fad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3da3fb1, SUC_lexicon_tags_8 },
    { 0x4e1a3fb2, SUC_lexicon_tags_26 },
    { 0x743a3fb2, SUC_lexicon_tags_60 },
    { 0x263a3fb1, SUC_lexicon_tags_17 },
    { 0x289a3fb2, SUC_lexicon_tags_12 },
    { 0x907a3fb6, SUC_lexicon_tags_33 },
    { 0xccba3fb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ba3fbb, SUC_lexicon_tags_11 },
    { 0xb0da3fbb, SUC_lexicon_tags_12 },
    { 0xb8da3fbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9a3fc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dda3fc4, SUC_lexicon_tags_1 },
    { 0xba5a3fc4, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41a3fca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3da3fcc, SUC_lexicon_tags_7 },
    { 0x9b7a3fcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a3fd1, SUC_lexicon_tags_26 },
    { 0xf4ba3fd1, SUC_lexicon_tags_7 },
    { 0xb47a3fd3, SUC_lexicon_tags_28 },
    { 0x6c1a3fd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bba3fd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ba3fdb, SUC_lexicon_tags_47 },
    { 0xbd9a3fdc, SUC_lexicon_tags_39 },
    { 0xe5fa3fdb, SUC_lexicon_tags_39 },
    { 0xc4da3fde, SUC_lexicon_tags_25 },
    { 0xc63a3fdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31fa3fe1, SUC_lexicon_tags_15 },
    { 0x513a3fe2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa3fe7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x96da3fe9, SUC_lexicon_tags_15 },
    { 0x479a3fea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5a3fee, SUC_lexicon_tags_38 },
    { 0x79da3fef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925a3ff5, SUC_lexicon_tags_45 },
    { 0xbf5a3ff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1a3ffc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8aba3ffe, SUC_lexicon_tags_14 },
    { 0xf01a3fff, SUC_lexicon_tags_3 },
    { 0x2efa4000, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x339a4002, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x369a4005, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ba4009, SUC_lexicon_tags_12 },
    { 0x2aba400a, SUC_lexicon_tags_19 },
    { 0x4b3a4009, SUC_lexicon_tags_17 },
    { 0x7b3a400a, SUC_lexicon_tags_25 },
    { 0x2cfa400d, SUC_lexicon_tags_2 },
    { 0xf7ba400d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x655a4010, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9a4015, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x699a4017, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbda401c, SUC_lexicon_tags_23 },
    { 0x12da401c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fa4022, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19da4024, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf07a4026, SUC_lexicon_tags_37 },
    { 0x60ba4027, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fa402e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9a4031, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7a4035, SUC_lexicon_tags_27 },
    { 0x44fa4035, SUC_lexicon_tags_49 },
    { 0xa6fa4037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x649a4039, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47a403c, SUC_lexicon_tags_25 },
    { 0x97fa403d, SUC_lexicon_tags_45 },
    { 0x409a403c, SUC_lexicon_tags_37 },
    { 0x33ba403c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7a4042, SUC_lexicon_tags_25 },
    { 0x7d9a4043, SUC_lexicon_tags_68 },
    { 0x289a4043, SUC_lexicon_tags_37 },
    { 0xfa5a4045, SUC_lexicon_tags_27 },
    { 0x2a3a4043, SUC_lexicon_tags_9 },
    { 0xdefa4047, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26da404c, SUC_lexicon_tags_17 },
    { 0x829a404d, SUC_lexicon_tags_27 },
    { 0xefa404e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ba4052, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa75a4054, SUC_lexicon_tags_11 },
    { 0xa01a4055, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x205a4058, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a5a405a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1a4066, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa51a4069, SUC_lexicon_tags_8 },
    { 0x46ba406a, SUC_lexicon_tags_12 },
    { 0xaf7a406a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33a406e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81a4072, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867a4076, SUC_lexicon_tags_12 },
    { 0xb5a4077, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913a407d, SUC_lexicon_tags_48 },
    { 0x7eda407e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3fa4080, SUC_lexicon_tags_2 },
    { 0xeb3a4081, SUC_lexicon_tags_95 },
    { 0x557a4082, SUC_lexicon_tags_14 },
    { 0xfd1a4081, SUC_lexicon_tags_63 },
    { 0x737a4084, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd85a4086, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ba4090, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c1a4092, SUC_lexicon_tags_27 },
    { 0x1e7a4093, SUC_lexicon_tags_43 },
    { 0x3bda4094, SUC_lexicon_tags_11 },
    { 0x9bfa4094, SUC_lexicon_tags_57 },
    { 0xaeba4096, SUC_lexicon_tags_47 },
    { 0x7eda4092, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfa409a, SUC_lexicon_tags_8 },
    { 0x86ba409b, SUC_lexicon_tags_7 },
    { 0x85ba409b, SUC_lexicon_tags_37 },
    { 0xe09a409c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd87a409f, SUC_lexicon_tags_27 },
    { 0x511a40a0, SUC_lexicon_tags_12 },
    { 0x575a40a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xeba40a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fa40a7, SUC_lexicon_tags_1 },
    { 0xbffa40a8, SUC_lexicon_tags_11 },
    { 0xe9a40a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa43a40ab, SUC_lexicon_tags_37 },
    { 0xcf3a40ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x153a40af, SUC_lexicon_tags_37 },
    { 0xb9da40af, SUC_lexicon_tags_56 },
    { 0x82fa40b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735a40b5, SUC_lexicon_tags_48 },
    { 0x1efa40b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x555a40b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1a40bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ba40c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x46fa40c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3eba40c6, SUC_lexicon_tags_25 },
    { 0x97fa40c7, SUC_lexicon_tags_30 },
    { 0x1b7a40c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd97a40ca, SUC_lexicon_tags_12 },
    { 0xd3fa40ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfa40cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd15a40d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ba40d7, SUC_lexicon_tags_14 },
    { 0xd4fa40d7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53a40db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ba40e0, SUC_lexicon_tags_0 },
    { 0x831a40e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81a40e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51ba40e7, SUC_lexicon_tags_15 },
    { 0x7c1a40e8, SUC_lexicon_tags_11 },
    { 0x90da40e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc77a40eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc2fa40ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fa40f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fa40f3, SUC_lexicon_tags_7 },
    { 0x9f7a40f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe79a40f6, SUC_lexicon_tags_25 },
    { 0x727a40f7, SUC_lexicon_tags_68 },
    { 0x7dba40f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7bda40fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09a40ff, SUC_lexicon_tags_7 },
    { 0xd4da4100, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x761a4102, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63a4106, SUC_lexicon_tags_0 },
    { 0x543a4107, SUC_lexicon_tags_25 },
    { 0x425a4108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfda410c, SUC_lexicon_tags_17 },
    { 0xea7a410d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x373a410f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x499a4111, SUC_lexicon_tags_71 },
    { 0xcc1a4111, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5a4115, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x129a4117, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525a411b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203a4120, SUC_lexicon_tags_45 },
    { 0xd13a4121, SUC_lexicon_tags_37 },
    { 0x47fa4121, SUC_lexicon_tags_37 },
    { 0xc61a4123, SUC_lexicon_tags_60 },
    { 0x529a4124, SUC_lexicon_tags_36 },
    { 0x641a4124, SUC_lexicon_tags_25 },
    { 0xa0ba4124, SUC_lexicon_tags_30 },
    { 0x247a4126, SUC_lexicon_tags_7 },
    { 0x2aba4128, SUC_lexicon_tags_11 },
    { 0x8f7a4129, SUC_lexicon_tags_27 },
    { 0xb45a4128, SUC_lexicon_tags_11 },
    { 0x6e3a412a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x753a412e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9da4138, SUC_lexicon_tags_47 },
    { 0xe21a4139, SUC_lexicon_tags_1 },
    { 0xf69a413a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb19a413c, SUC_lexicon_tags_33 },
    { 0x37fa413d, SUC_lexicon_tags_107 },
    { 0x581a413e, SUC_lexicon_tags_25 },
    { 0x3d3a413e, SUC_lexicon_tags_28 },
    { 0xab3a4140, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43ba4142, SUC_lexicon_tags_31 },
    { 0xcafa4142, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43a4146, SUC_lexicon_tags_34 },
    { 0x231a4146, SUC_lexicon_tags_47 },
    { 0xc01a4147, SUC_lexicon_tags_49 },
    { 0xb6da4146, SUC_lexicon_tags_12 },
    { 0xa43a414a, SUC_lexicon_tags_47 },
    { 0xa05a4147, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3a414f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435a4155, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5a4158, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ba415d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455a4161, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x615a4163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27da4165, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc81a4167, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa49a4169, SUC_lexicon_tags_19 },
    { 0x957a416a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x23ba416c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc1a416e, SUC_lexicon_tags_156 },
    { 0x87ba416f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa21a4171, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb9fa4173, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0x2f9a4175, SUC_lexicon_tags_25 },
    { 0x323a4176, SUC_lexicon_tags_14 },
    { 0x261a4175, SUC_lexicon_tags_11 },
    { 0x871a4178, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf65a417a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aba4180, SUC_lexicon_tags_8 },
    { 0x4c9a4180, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xff5a4183, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5a4189, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x289a418b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215a418f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fa4195, SUC_lexicon_tags_11 },
    { 0x8e9a4195, SUC_lexicon_tags_19 },
    { 0x587a4197, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x423a419a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fda419d, SUC_lexicon_tags_38 },
    { 0x441a419d, SUC_lexicon_tags_26 },
    { 0xc05a419d, SUC_lexicon_tags_7 },
    { 0x901a41a0, SUC_lexicon_tags_33 },
    { 0xca7a41a1, SUC_lexicon_tags_31 },
    { 0xe5ba41a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981a41a7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73a41ac, SUC_lexicon_tags_37 },
    { 0xde5a41ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffa41b1, SUC_lexicon_tags_27 },
    { 0xa35a41b2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9a41b7, SUC_lexicon_tags_7 },
    { 0x577a41b8, SUC_lexicon_tags_17 },
    { 0xe69a41b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3a41bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737a41c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa41c4, SUC_lexicon_tags_15 },
    { 0x835a41c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53da41c7, SUC_lexicon_tags_17 },
    { 0x19da41c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffa41cb, SUC_lexicon_tags_45 },
    { 0x891a41cc, SUC_lexicon_tags_7 },
    { 0xfd1a41cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5a41d0, SUC_lexicon_tags_60 },
    { 0x7bda41d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49da41d5, SUC_lexicon_tags_1 },
    { 0xb7da41d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3a41d9, SUC_lexicon_tags_9 },
    { 0xc15a41da, SUC_lexicon_tags_2 },
    { 0x5bba41db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ba41e3, SUC_lexicon_tags_15 },
    { 0x609a41e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa19a41e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9da41ec, SUC_lexicon_tags_17 },
    { 0xd9da41ec, SUC_lexicon_tags_37 },
    { 0xb09a41ee, SUC_lexicon_tags_32 },
    { 0x88da41ec, SUC_lexicon_tags_172 },
    { 0xfa1a41ed, SUC_lexicon_tags_25 },
    { 0x2e3a41f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3da41f4, SUC_lexicon_tags_68 },
    { 0x69ba41f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdda41f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ba41ff, SUC_lexicon_tags_32 },
    { 0xb59a4200, SUC_lexicon_tags_37 },
    { 0xba5a4201, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595a4205, SUC_lexicon_tags_6 },
    { 0xcfda4206, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699a420d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9a4210, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dba4214, SUC_lexicon_tags_46 },
    { 0x9d9a4214, SUC_lexicon_tags_11 },
    { 0xcb5a4215, SUC_lexicon_tags_7 },
    { 0x449a4214, SUC_lexicon_tags_7 },
    { 0x9fda4218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ba421e, SUC_lexicon_tags_30 },
    { 0x9d3a421f, SUC_lexicon_tags_9 },
    { 0x52da4220, SUC_lexicon_tags_33 },
    { 0xd35a421e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dba4225, SUC_lexicon_tags_11 },
    { 0xba4226, SUC_lexicon_tags_31 },
    { 0x873a4227, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139a422d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623a4233, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487a4238, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0x43fa423a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x273a423c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383a4241, SUC_lexicon_tags_7 },
    { 0x661a4241, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a4248, SUC_lexicon_tags_12 },
    { 0xb83a4249, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28fa424b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8fa424d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x50da4250, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9a4255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763a425a, SUC_lexicon_tags_2 },
    { 0x4d3a425b, SUC_lexicon_tags_45 },
    { 0xcf9a425a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4dfa425e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a4262, SUC_lexicon_tags_61 },
    { 0x4afa4263, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0da4265, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07a4268, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x115a426a, SUC_lexicon_tags_9 },
    { 0xcc9a426a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1cfa426d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfa4271, SUC_lexicon_tags_15 },
    { 0x213a4271, SUC_lexicon_tags_9 },
    { 0x5fba4273, SUC_lexicon_tags_0 },
    { 0x37ba4274, SUC_lexicon_tags_4 },
    { 0x6f5a4275, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565a4279, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709a4280, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7a4284, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda5a4286, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x629a4288, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4da428c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x487a428e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7a4294, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe3da4296, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1a429a, SUC_lexicon_tags_17 },
    { 0xe07a429b, SUC_lexicon_tags_45 },
    { 0xe77a429c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10da429f, SUC_lexicon_tags_2 },
    { 0x6a3a429f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6eba42a2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa42a5, SUC_lexicon_tags_42 },
    { 0x477a42a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x42fa42a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ba42ac, SUC_lexicon_tags_15 },
    { 0xda9a42ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9ba42af, SUC_lexicon_tags_1 },
    { 0x9fa42b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fa42b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7fa42b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27a42ba, SUC_lexicon_tags_23 },
    { 0x815a42ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ba42bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf7a42c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafa42c4, SUC_lexicon_tags_23 },
    { 0x8efa42c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb9ba42c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295a42cd, SUC_lexicon_tags_29 },
    { 0x549a42ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x65da42d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9a42dc, SUC_lexicon_tags_31 },
    { 0x785a42dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ba42e6, SUC_lexicon_tags_17 },
    { 0xffda42e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dba42ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x41a42f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7a42f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6da42f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x889a42fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x76fa42fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82da4307, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3a4310, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x57da4312, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3a4317, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcba4319, SUC_lexicon_tags_12 },
    { 0xf39a431a, SUC_lexicon_tags_17 },
    { 0x60fa431a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fa4324, SUC_lexicon_tags_47 },
    { 0xe91a4325, SUC_lexicon_tags_26 },
    { 0xa21a4326, SUC_lexicon_tags_35 },
    { 0x649a4327, SUC_lexicon_tags_12 },
    { 0x453a4325, SUC_lexicon_tags_57 },
    { 0x6cda4325, SUC_lexicon_tags_4 },
    { 0xcbba4325, SUC_lexicon_tags_11 },
    { 0xefda432b, SUC_lexicon_tags_12 },
    { 0x641a432c, SUC_lexicon_tags_1 },
    { 0x261a432a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x373a432f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fda4331, SUC_lexicon_tags_43 },
    { 0x2a9a4331, SUC_lexicon_tags_19 },
    { 0x73fa4333, SUC_lexicon_tags_2 },
    { 0xac5a4331, SUC_lexicon_tags_11 },
    { 0x95a4335, SUC_lexicon_tags_14 },
    { 0xdd1a4336, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59a4339, SUC_lexicon_tags_37 },
    { 0x85da433a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fa4342, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa4345, SUC_lexicon_tags_33 },
    { 0x2e1a4346, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5bda4348, SUC_lexicon_tags_7 },
    { 0x6e1a4348, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb01a434b, SUC_lexicon_tags_47 },
    { 0x6d5a434c, SUC_lexicon_tags_19 },
    { 0xf45a434d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x99ba434f, SUC_lexicon_tags_17 },
    { 0x69a434f, SUC_lexicon_tags_31 },
    { 0x9cfa4351, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13a4354, SUC_lexicon_tags_12 },
    { 0x6c1a4355, SUC_lexicon_tags_6 },
    { 0x6ba4355, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x975a4358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1a435b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9a4360, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1bfa4362, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fa4365, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x213a4367, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335a436c, SUC_lexicon_tags_7 },
    { 0x3e9a436d, SUC_lexicon_tags_29 },
    { 0xa99a436d, SUC_lexicon_tags_27 },
    { 0x2b1a436f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577a4373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14da4377, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a9a4379, SUC_lexicon_tags_768 },
    { 0xc69a437a, SUC_lexicon_tags_70 },
    { 0xd1fa4379, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa61a437d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfa4382, SUC_lexicon_tags_1 },
    { 0x89a4383, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14da4387, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f5a4389, SUC_lexicon_tags_37 },
    { 0x7cda4389, SUC_lexicon_tags_51 },
    { 0x373a438b, SUC_lexicon_tags_19 },
    { 0x2d3a438c, SUC_lexicon_tags_57 },
    { 0x151a438d, SUC_lexicon_tags_2 },
    { 0xc99a4389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x395a4391, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97a4394, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37a4397, SUC_lexicon_tags_2 },
    { 0x61da4398, SUC_lexicon_tags_2 },
    { 0xb65a4399, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bba43a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6da43a5, SUC_lexicon_tags_2 },
    { 0x4c7a43a5, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xce9a43a8, SUC_lexicon_tags_47 },
    { 0xc7da43a8, SUC_lexicon_tags_1 },
    { 0x1a1a43aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221a43ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3a43b3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ba43b7, SUC_lexicon_tags_16 },
    { 0x921a43b7, SUC_lexicon_tags_38 },
    { 0x3aba43b8, SUC_lexicon_tags_11 },
    { 0xc47a43b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95a43bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49a43c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89a43c4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fa43c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a43d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd45a43d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x469a43d5, SUC_lexicon_tags_7 },
    { 0xb51a43d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35a43d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x47ba43db, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ba43de, SUC_lexicon_tags_12 },
    { 0xb55a43df, SUC_lexicon_tags_19 },
    { 0xe05a43df, SUC_lexicon_tags_25 },
    { 0xe47a43de, SUC_lexicon_tags_37 },
    { 0x9bda43e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3a43e5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2da43ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x913a43ed, SUC_lexicon_tags_38 },
    { 0xdb1a43ee, SUC_lexicon_tags_1 },
    { 0x2d1a43ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ba43f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x287a43f7, SUC_lexicon_tags_47 },
    { 0x731a43f8, SUC_lexicon_tags_1 },
    { 0x94ba43f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fda43fc, SUC_lexicon_tags_62 },
    { 0x7c7a43fd, SUC_lexicon_tags_60 },
    { 0x73a43fe, SUC_lexicon_tags_12 },
    { 0x633a43ff, SUC_lexicon_tags_15 },
    { 0xb17a43ff, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xfa9a4402, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313a4406, SUC_lexicon_tags_37 },
    { 0x31a4406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565a440d, SUC_lexicon_tags_12 },
    { 0x443a440d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa4411, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5a4415, SUC_lexicon_tags_19 },
    { 0x67ba4416, SUC_lexicon_tags_17 },
    { 0xcb5a4417, SUC_lexicon_tags_2 },
    { 0x9da4418, SUC_lexicon_tags_16 },
    { 0x487a4418, SUC_lexicon_tags_55 },
    { 0xe0fa4419, SUC_lexicon_tags_25 },
    { 0xfb1a441b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ba4420, SUC_lexicon_tags_37 },
    { 0xa49a4421, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x193a4423, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x395a4425, SUC_lexicon_tags_84 },
    { 0x6cfa4426, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ba442a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x853a442c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba4430, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a4432, SUC_lexicon_tags_37 },
    { 0x6c3a4433, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x359a4436, SUC_lexicon_tags_12 },
    { 0x4a7a4437, SUC_lexicon_tags_2 },
    { 0xa41a4437, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5da443a, SUC_lexicon_tags_49 },
    { 0xaf9a443b, SUC_lexicon_tags_2 },
    { 0x49ba443c, SUC_lexicon_tags_38 },
    { 0xab1a443c, SUC_lexicon_tags_16 },
    { 0xc45a443e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe55a4440, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2dfa4442, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcf7a4444, SUC_lexicon_tags_19 },
    { 0xf0fa4445, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fa4448, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0da444c, SUC_lexicon_tags_43 },
    { 0xe89a444d, SUC_lexicon_tags_27 },
    { 0xd65a444e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1a4453, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89a4455, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87a4459, SUC_lexicon_tags_25 },
    { 0x533a4459, SUC_lexicon_tags_38 },
    { 0xdb9a445b, SUC_lexicon_tags_8 },
    { 0x877a445b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x61da445f, SUC_lexicon_tags_75 },
    { 0x507a445f, SUC_lexicon_tags_31 },
    { 0xb89a4461, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f3a4463, SUC_lexicon_tags_12 },
    { 0xdf3a4464, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855a4468, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ba446d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x673a4470, SUC_lexicon_tags_12 },
    { 0x36fa4471, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7a4474, SUC_lexicon_tags_39 },
    { 0x2fba4475, SUC_lexicon_tags_226 },
    { 0xac5a4476, SUC_lexicon_tags_46 },
    { 0x60da4475, SUC_lexicon_tags_11 },
    { 0x673a4477, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03a447b, SUC_lexicon_tags_54 },
    { 0x78ba447b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3a4483, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7efa4485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505a448f, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x37a4491, SUC_lexicon_tags_489 },
    { 0xc0da4492, SUC_lexicon_tags_2 },
    { 0x949a4493, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeeba4495, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679a4499, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x11ba449c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffa44a5, SUC_lexicon_tags_29 },
    { 0x801a44a5, SUC_lexicon_tags_12 },
    { 0xee1a44a6, SUC_lexicon_tags_25 },
    { 0x6eda44a5, SUC_lexicon_tags_32 },
    { 0x4ba44a6, SUC_lexicon_tags_1 },
    { 0x841a44aa, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4da44b1, SUC_lexicon_tags_11 },
    { 0x97fa44b1, SUC_lexicon_tags_68 },
    { 0xc7ba44b3, SUC_lexicon_tags_19 },
    { 0x11fa44b4, SUC_lexicon_tags_2 },
    { 0x35da44b4, SUC_lexicon_tags_47 },
    { 0x999a44b5, SUC_lexicon_tags_19 },
    { 0x329a44b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc89a44b9, SUC_lexicon_tags_2 },
    { 0x739a44ba, SUC_lexicon_tags_37 },
    { 0xfa7a44bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x19ba44bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473a44c2, SUC_lexicon_tags_37 },
    { 0xc15a44c2, SUC_lexicon_tags_60 },
    { 0xd8ba44c3, SUC_lexicon_tags_11 },
    { 0x40ba44c5, SUC_lexicon_tags_1 },
    { 0xd77a44c2, SUC_lexicon_tags_11 },
    { 0x261a44c7, SUC_lexicon_tags_27 },
    { 0xedda44c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa15a44ca, SUC_lexicon_tags_28 },
    { 0xef7a44cb, SUC_lexicon_tags_33 },
    { 0x9efa44cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x88fa44ce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6b1a44d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb9a44d2, SUC_lexicon_tags_28 },
    { 0xaa3a44d2, SUC_lexicon_tags_46 },
    { 0x383a44d2, SUC_lexicon_tags_9 },
    { 0xde3a44d3, SUC_lexicon_tags_17 },
    { 0x6f1a44d6, SUC_lexicon_tags_25 },
    { 0xf71a44d7, SUC_lexicon_tags_46 },
    { 0x297a44d6, SUC_lexicon_tags_6 },
    { 0x8c5a44d9, SUC_lexicon_tags_0 },
    { 0x79ba44d4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfa44e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd29a44e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57a44e8, SUC_lexicon_tags_25 },
    { 0xaffa44e9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9a44ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedfa44ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213a44f3, SUC_lexicon_tags_45 },
    { 0x429a44f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeffa44f6, SUC_lexicon_tags_92 },
    { 0x10ba44f7, SUC_lexicon_tags_15 },
    { 0x5c7a44f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fa44fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5a4503, SUC_lexicon_tags_11 },
    { 0xbcfa4504, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba4506, SUC_lexicon_tags_17 },
    { 0xa5fa4506, SUC_lexicon_tags_11 },
    { 0x813a4506, SUC_lexicon_tags_2 },
    { 0xe8da4508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dda450d, SUC_lexicon_tags_1 },
    { 0x57fa450e, SUC_lexicon_tags_56 },
    { 0x95fa450e, SUC_lexicon_tags_17 },
    { 0xecda4510, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc97a4512, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x247a4514, SUC_lexicon_tags_14 },
    { 0x6d5a4514, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3a4519, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9da451c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfcba451e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7a4521, SUC_lexicon_tags_39 },
    { 0xf41a4521, SUC_lexicon_tags_10 },
    { 0xd33a4522, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eba4531, SUC_lexicon_tags_7 },
    { 0x567a4531, SUC_lexicon_tags_10 },
    { 0x72da4533, SUC_lexicon_tags_12 },
    { 0x55a4534, SUC_lexicon_tags_0 },
    { 0xbba4533, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afa453c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dba4541, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69da4543, SUC_lexicon_tags_29 },
    { 0x3eda4543, SUC_lexicon_tags_11 },
    { 0xd61a4545, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5a4549, SUC_lexicon_tags_25 },
    { 0x943a4549, SUC_lexicon_tags_28 },
    { 0x3aba4549, SUC_lexicon_tags_55 },
    { 0x667a454c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8d5a454e, SUC_lexicon_tags_37 },
    { 0x39da454f, SUC_lexicon_tags_38 },
    { 0xde5a4550, SUC_lexicon_tags_0 },
    { 0x5a454e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189a4555, SUC_lexicon_tags_25 },
    { 0xdb9a4555, SUC_lexicon_tags_17 },
    { 0xa13a4556, SUC_lexicon_tags_11 },
    { 0x7ffa4558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddba455f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47a4569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1a456c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1a456f, SUC_lexicon_tags_7 },
    { 0x761a456f, SUC_lexicon_tags_7 },
    { 0x831a456f, SUC_lexicon_tags_1 },
    { 0xd3ba4571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce9a4574, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5a4577, SUC_lexicon_tags_47 },
    { 0x9ba4578, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c5a457a, SUC_lexicon_tags_38 },
    { 0x4dba457a, SUC_lexicon_tags_1 },
    { 0x2b7a457a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43a457f, SUC_lexicon_tags_12 },
    { 0x5dba4580, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1a4582, SUC_lexicon_tags_12 },
    { 0xdf5a4583, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x305a4586, SUC_lexicon_tags_2 },
    { 0x4bda4587, SUC_lexicon_tags_1 },
    { 0x8e5a4587, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x657a458a, SUC_lexicon_tags_17 },
    { 0x71da458b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9a4591, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93a4594, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1fa4596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dda459b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa19a459d, SUC_lexicon_tags_84 },
    { 0xfd9a459d, SUC_lexicon_tags_79 },
    { 0x4f3a459f, SUC_lexicon_tags_120 },
    { 0xa35a459e, SUC_lexicon_tags_11 },
    { 0x8da45a1, SUC_lexicon_tags_63 },
    { 0xb65a459d, SUC_lexicon_tags_45 },
    { 0x5dba45a2, SUC_lexicon_tags_65 },
    { 0x6dda459d, SUC_lexicon_tags_0 },
    { 0xbfba45a0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50da45af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x133a45b2, SUC_lexicon_tags_26 },
    { 0x89ba45b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ada45b5, SUC_lexicon_tags_61 },
    { 0x5d3a45b5, SUC_lexicon_tags_49 },
    { 0xc13a45b7, SUC_lexicon_tags_46 },
    { 0x96ba45b8, SUC_lexicon_tags_17 },
    { 0x4eba45b7, SUC_lexicon_tags_37 },
    { 0xc1a45b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x225a45bc, SUC_lexicon_tags_2 },
    { 0xdeda45bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271a45c1, SUC_lexicon_tags_12 },
    { 0x49da45c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a9a45c4, SUC_lexicon_tags_39 },
    { 0xf83a45c5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13a45cb, SUC_lexicon_tags_17 },
    { 0x27ba45cc, SUC_lexicon_tags_18 },
    { 0xd35a45cc, SUC_lexicon_tags_33 },
    { 0xcdfa45cb, SUC_lexicon_tags_17 },
    { 0xc3da45cd, SUC_lexicon_tags_16 },
    { 0xb05a45d0, SUC_lexicon_tags_37 },
    { 0x58fa45d1, SUC_lexicon_tags_15 },
    { 0xd13a45d2, SUC_lexicon_tags_8 },
    { 0x6eda45d3, SUC_lexicon_tags_43 },
    { 0x8a1a45d1, SUC_lexicon_tags_10 },
    { 0x4f7a45d4, SUC_lexicon_tags_17 },
    { 0xecba45d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4efa45d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7bda45da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc5a45dc, SUC_lexicon_tags_8 },
    { 0x4b1a45dd, SUC_lexicon_tags_19 },
    { 0xb63a45dd, SUC_lexicon_tags_23 },
    { 0x1da45df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa45e4, SUC_lexicon_tags_28 },
    { 0x23da45e5, SUC_lexicon_tags_12 },
    { 0xfa1a45e5, SUC_lexicon_tags_37 },
    { 0xeb5a45e6, SUC_lexicon_tags_46 },
    { 0xa75a45e8, SUC_lexicon_tags_68 },
    { 0x625a45e8, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x393a45eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9da45ee, SUC_lexicon_tags_39 },
    { 0x719a45ef, SUC_lexicon_tags_8 },
    { 0x265a45ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98fa45f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83a45f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada45fd, SUC_lexicon_tags_12 },
    { 0xf15a45fe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5a4603, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x83ba4605, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fa4609, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd91a460b, SUC_lexicon_tags_30 },
    { 0xb3da460b, SUC_lexicon_tags_42 },
    { 0x9f1a460d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631a4611, SUC_lexicon_tags_40 },
    { 0x333a4612, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963a4618, SUC_lexicon_tags_25 },
    { 0x95ba4618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfa461d, SUC_lexicon_tags_12 },
    { 0xfd7a461e, SUC_lexicon_tags_25 },
    { 0xe2fa461f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf53a4621, SUC_lexicon_tags_28 },
    { 0xa11a4621, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7da4624, SUC_lexicon_tags_33 },
    { 0x4e1a4625, SUC_lexicon_tags_47 },
    { 0x91a4626, SUC_lexicon_tags_15 },
    { 0x511a4627, SUC_lexicon_tags_1 },
    { 0x72da4625, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x597a462a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53ba462c, SUC_lexicon_tags_8 },
    { 0x237a462c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efa4636, SUC_lexicon_tags_60 },
    { 0x4f5a4637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x467a4639, SUC_lexicon_tags_13 },
    { 0x70ba463a, SUC_lexicon_tags_12 },
    { 0x503a463a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265a4640, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x963a4642, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a9a4644, SUC_lexicon_tags_60 },
    { 0x12fa4645, SUC_lexicon_tags_2 },
    { 0xa0da4646, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x41fa4648, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79a464c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3a464f, SUC_lexicon_tags_63 },
    { 0x3a4650, SUC_lexicon_tags_49 },
    { 0xb65a4650, SUC_lexicon_tags_47 },
    { 0x78fa4652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279a4656, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47a465a, SUC_lexicon_tags_38 },
    { 0x897a465b, SUC_lexicon_tags_68 },
    { 0xbbba465a, SUC_lexicon_tags_25 },
    { 0xff1a465b, SUC_lexicon_tags_43 },
    { 0x4c3a465c, SUC_lexicon_tags_37 },
    { 0xd73a465f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ba4664, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fa466c, SUC_lexicon_tags_47 },
    { 0xcdda466d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x16ba4670, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2da4674, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5a4677, SUC_lexicon_tags_13 },
    { 0x9ffa4678, SUC_lexicon_tags_37 },
    { 0xbb1a4679, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3a4681, SUC_lexicon_tags_29 },
    { 0xd99a4682, SUC_lexicon_tags_1 },
    { 0x171a4682, SUC_lexicon_tags_15 },
    { 0x8aba4683, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545a4688, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd5a468a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1a468e, SUC_lexicon_tags_38 },
    { 0x879a468f, SUC_lexicon_tags_63 },
    { 0xdafa4690, SUC_lexicon_tags_1 },
    { 0x3c5a4691, SUC_lexicon_tags_15 },
    { 0x771a4690, SUC_lexicon_tags_45 },
    { 0x215a4693, SUC_lexicon_tags_66 },
    { 0xee5a4694, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81a469e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23a46a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x459a46a6, SUC_lexicon_tags_2 },
    { 0x8da46a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfa46aa, SUC_lexicon_tags_1 },
    { 0x15ba46ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x451a46ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x275a46b0, SUC_lexicon_tags_35 },
    { 0x30da46b0, SUC_lexicon_tags_27 },
    { 0x35a46b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ba46b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1a46ba, SUC_lexicon_tags_11 },
    { 0x887a46bb, SUC_lexicon_tags_17 },
    { 0xcffa46bb, SUC_lexicon_tags_25 },
    { 0x115a46bc, SUC_lexicon_tags_45 },
    { 0xa01a46be, SUC_lexicon_tags_62 },
    { 0xa79a46bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9a46c2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81da46c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c5a46c8, SUC_lexicon_tags_1 },
    { 0xc73a46c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943a46ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x853a46d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41fa46d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1a46d8, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ada46db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173a46e2, SUC_lexicon_tags_7 },
    { 0xd65a46e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ffa46e5, SUC_lexicon_tags_17 },
    { 0x609a46e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cba46ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56fa46ec, SUC_lexicon_tags_9 },
    { 0x3e7a46ed, SUC_lexicon_tags_25 },
    { 0x9b7a46ee, SUC_lexicon_tags_900 },
    { 0, NULL },
    { 0x4cfa46f0, SUC_lexicon_tags_49 },
    { 0x7a9a46f1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7a46f7, SUC_lexicon_tags_37 },
    { 0x563a46f8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbb7a46fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x67da46fd, SUC_lexicon_tags_21 },
    { 0x665a46fe, SUC_lexicon_tags_21 },
    { 0xf4da46ff, SUC_lexicon_tags_38 },
    { 0xc8fa4700, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x839a4702, SUC_lexicon_tags_29 },
    { 0x475a4702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ba4705, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x563a4707, SUC_lexicon_tags_53 },
    { 0xfda4708, SUC_lexicon_tags_37 },
    { 0xdc9a4709, SUC_lexicon_tags_0 },
    { 0xf15a470a, SUC_lexicon_tags_82 },
    { 0xab5a470b, SUC_lexicon_tags_29 },
    { 0xf5fa470b, SUC_lexicon_tags_39 },
    { 0x79a470b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9a4710, SUC_lexicon_tags_10 },
    { 0x13fa4711, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da4716, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ba471b, SUC_lexicon_tags_32 },
    { 0xf9fa471c, SUC_lexicon_tags_1 },
    { 0xb7da471d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ba4720, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fa4723, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1a4727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49da472a, SUC_lexicon_tags_37 },
    { 0x469a472b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9a472e, SUC_lexicon_tags_14 },
    { 0x697a472f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2da4732, SUC_lexicon_tags_11 },
    { 0x499a4733, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fa4736, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa21a4738, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefa473c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17a4741, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeda4746, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a4749, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfa474d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x709a474f, SUC_lexicon_tags_31 },
    { 0xa87a4750, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899a475d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cba4762, SUC_lexicon_tags_28 },
    { 0x471a4763, SUC_lexicon_tags_12 },
    { 0x76ba4764, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37a4767, SUC_lexicon_tags_15 },
    { 0x969a4768, SUC_lexicon_tags_7 },
    { 0xe01a4768, SUC_lexicon_tags_7 },
    { 0x2ba476a, SUC_lexicon_tags_11 },
    { 0x793a476a, SUC_lexicon_tags_47 },
    { 0xd1a476a, SUC_lexicon_tags_11 },
    { 0xf99a476b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3a4774, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x221a4776, SUC_lexicon_tags_1 },
    { 0xdda4776, SUC_lexicon_tags_45 },
    { 0x7b9a4778, SUC_lexicon_tags_68 },
    { 0x49da4779, SUC_lexicon_tags_222 },
    { 0xcf1a477a, SUC_lexicon_tags_15 },
    { 0x9bba4776, SUC_lexicon_tags_84 },
    { 0x4e1a477c, SUC_lexicon_tags_43 },
    { 0x9eba477c, SUC_lexicon_tags_17 },
    { 0xec7a477a, SUC_lexicon_tags_12 },
    { 0x2f7a4777, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x715a4781, SUC_lexicon_tags_224 },
    { 0x8a7a4782, SUC_lexicon_tags_1 },
    { 0x961a4782, SUC_lexicon_tags_37 },
    { 0x431a4784, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee5a4786, SUC_lexicon_tags_11 },
    { 0x5a3a4787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1a478b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a4791, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fda4794, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88fa4796, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaada479a, SUC_lexicon_tags_17 },
    { 0x3a7a479b, SUC_lexicon_tags_45 },
    { 0x44da479c, SUC_lexicon_tags_27 },
    { 0xec7a479d, SUC_lexicon_tags_17 },
    { 0x5dfa479c, SUC_lexicon_tags_15 },
    { 0xfafa479f, SUC_lexicon_tags_25 },
    { 0x87ba47a0, SUC_lexicon_tags_2 },
    { 0xcc3a47a1, SUC_lexicon_tags_17 },
    { 0x96fa47a2, SUC_lexicon_tags_1 },
    { 0xe9a479e, SUC_lexicon_tags_7 },
    { 0x2f5a47a4, SUC_lexicon_tags_4 },
    { 0x4d9a47a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9a7a47a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09a47ab, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ba47b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5a47b7, SUC_lexicon_tags_17 },
    { 0xca9a47b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11a47bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3a47c2, SUC_lexicon_tags_40 },
    { 0xacda47c3, SUC_lexicon_tags_63 },
    { 0x133a47c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9fa47c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3cfa47c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31a47cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7a47d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd37a47d4, SUC_lexicon_tags_39 },
    { 0xacda47d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x675a47d7, SUC_lexicon_tags_14 },
    { 0xb83a47d8, SUC_lexicon_tags_60 },
    { 0x2c3a47d8, SUC_lexicon_tags_21 },
    { 0x5ba47da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3da47dd, SUC_lexicon_tags_38 },
    { 0x9d1a47de, SUC_lexicon_tags_11 },
    { 0x78da47de, SUC_lexicon_tags_79 },
    { 0x66fa47e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xce3a47e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71a47e7, SUC_lexicon_tags_45 },
    { 0x9dda47e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717a47ed, SUC_lexicon_tags_31 },
    { 0xcbda47ed, SUC_lexicon_tags_16 },
    { 0xe23a47ef, SUC_lexicon_tags_27 },
    { 0x305a47ef, SUC_lexicon_tags_66 },
    { 0xf49a47f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73a47f6, SUC_lexicon_tags_25 },
    { 0xf59a47f6, SUC_lexicon_tags_25 },
    { 0xb5fa47f8, SUC_lexicon_tags_19 },
    { 0x64ba47f8, SUC_lexicon_tags_30 },
    { 0x6fba47fa, SUC_lexicon_tags_27 },
    { 0x37a47fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x50fa47fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf9a47ff, SUC_lexicon_tags_31 },
    { 0x709a47ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a4804, SUC_lexicon_tags_17 },
    { 0x117a4804, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fda4809, SUC_lexicon_tags_16 },
    { 0x925a480a, SUC_lexicon_tags_23 },
    { 0x4c9a4809, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41a4810, SUC_lexicon_tags_11 },
    { 0x731a4811, SUC_lexicon_tags_61 },
    { 0xf9a4812, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d3a4814, SUC_lexicon_tags_37 },
    { 0x39a4815, SUC_lexicon_tags_23 },
    { 0xc7da4816, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf63a4818, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483a481f, SUC_lexicon_tags_60 },
    { 0x30ba4820, SUC_lexicon_tags_21 },
    { 0x4a3a4821, SUC_lexicon_tags_37 },
    { 0x9da481f, SUC_lexicon_tags_34 },
    { 0xf63a4823, SUC_lexicon_tags_17 },
    { 0x1eda4824, SUC_lexicon_tags_2 },
    { 0xbfda4825, SUC_lexicon_tags_1 },
    { 0x18da4825, SUC_lexicon_tags_7 },
    { 0x3afa4827, SUC_lexicon_tags_17 },
    { 0x71ba4821, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3a4831, SUC_lexicon_tags_35 },
    { 0x659a4832, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69a4838, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69a483b, SUC_lexicon_tags_17 },
    { 0x691a483c, SUC_lexicon_tags_37 },
    { 0x1cda483d, SUC_lexicon_tags_101 },
    { 0x32ba483c, SUC_lexicon_tags_33 },
    { 0x49fa483f, SUC_lexicon_tags_28 },
    { 0x715a483f, SUC_lexicon_tags_20 },
    { 0xa21a483f, SUC_lexicon_tags_37 },
    { 0x7f1a4842, SUC_lexicon_tags_70 },
    { 0x7f9a483b, SUC_lexicon_tags_0 },
    { 0x25fa4844, SUC_lexicon_tags_1 },
    { 0x68ba483b, SUC_lexicon_tags_11 },
    { 0xe4ba4846, SUC_lexicon_tags_32 },
    { 0xd87a483b, SUC_lexicon_tags_14 },
    { 0x137a4848, SUC_lexicon_tags_2 },
    { 0x2dfa4843, SUC_lexicon_tags_14 },
    { 0x443a484a, SUC_lexicon_tags_21 },
    { 0x541a483d, SUC_lexicon_tags_4 },
    { 0xddfa484c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35a4857, SUC_lexicon_tags_12 },
    { 0x49ba4857, SUC_lexicon_tags_49 },
    { 0x2b7a4859, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf53a485b, SUC_lexicon_tags_14 },
    { 0xd7da485b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefba4861, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5a4864, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa1ba4866, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa33a4868, SUC_lexicon_tags_26 },
    { 0x24ba4869, SUC_lexicon_tags_15 },
    { 0x269a486a, SUC_lexicon_tags_27 },
    { 0x497a486b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5a486d, SUC_lexicon_tags_10 },
    { 0xe01a486e, SUC_lexicon_tags_28 },
    { 0xfaba486f, SUC_lexicon_tags_23 },
    { 0x8c5a486e, SUC_lexicon_tags_43 },
    { 0x99da4870, SUC_lexicon_tags_12 },
    { 0xa83a4872, SUC_lexicon_tags_39 },
    { 0xd87a486f, SUC_lexicon_tags_14 },
    { 0x30ba4873, SUC_lexicon_tags_57 },
    { 0xad7a4875, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477a487b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x613a487d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efa4881, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf5da4883, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5a4889, SUC_lexicon_tags_7 },
    { 0x7dba488a, SUC_lexicon_tags_2 },
    { 0xbeba4889, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61da488d, SUC_lexicon_tags_11 },
    { 0x737a488e, SUC_lexicon_tags_6 },
    { 0x671a488f, SUC_lexicon_tags_17 },
    { 0xb35a4890, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x875a4892, SUC_lexicon_tags_12 },
    { 0xb39a4893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d9a4895, SUC_lexicon_tags_77 },
    { 0xab5a4895, SUC_lexicon_tags_15 },
    { 0xb5ba4897, SUC_lexicon_tags_39 },
    { 0xc17a4896, SUC_lexicon_tags_2 },
    { 0xf59a4899, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbba489e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69a48a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8da48a4, SUC_lexicon_tags_2 },
    { 0x6dba48a5, SUC_lexicon_tags_21 },
    { 0x5b1a48a5, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xe03a48a8, SUC_lexicon_tags_7 },
    { 0x827a48a9, SUC_lexicon_tags_80 },
    { 0x4b1a48a9, SUC_lexicon_tags_69 },
    { 0xf91a48a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x171a48ad, SUC_lexicon_tags_22 },
    { 0xd7a48ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x227a48b1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a48b4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x995a48b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x279a48b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eda48bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e3a48bf, SUC_lexicon_tags_32 },
    { 0x9c9a48c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fda48c2, SUC_lexicon_tags_14 },
    { 0x527a48c3, SUC_lexicon_tags_8 },
    { 0x7da48c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835a48c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4f9a48cb, SUC_lexicon_tags_25 },
    { 0x39fa48cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa48cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x249a48d2, SUC_lexicon_tags_68 },
    { 0xa9a48d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe51a48d5, SUC_lexicon_tags_17 },
    { 0x58da48d6, SUC_lexicon_tags_25 },
    { 0x1d9a48d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521a48dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ba48e3, SUC_lexicon_tags_17 },
    { 0x239a48e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ba48e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa48eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271a48f2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5a48fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fa4900, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84fa4902, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297a4909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5cba490b, SUC_lexicon_tags_0 },
    { 0xfefa490b, SUC_lexicon_tags_2 },
    { 0x79fa490d, SUC_lexicon_tags_15 },
    { 0xd63a490e, SUC_lexicon_tags_56 },
    { 0xe99a490f, SUC_lexicon_tags_12 },
    { 0xf95a490d, SUC_lexicon_tags_1 },
    { 0x33a4911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7a4917, SUC_lexicon_tags_33 },
    { 0xe9ba4918, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33a4920, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7a4926, SUC_lexicon_tags_11 },
    { 0xbd7a4926, SUC_lexicon_tags_35 },
    { 0x251a4926, SUC_lexicon_tags_43 },
    { 0x50fa4928, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf91a492b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613a492f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a4933, SUC_lexicon_tags_17 },
    { 0xfb9a4934, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5a4939, SUC_lexicon_tags_7 },
    { 0x683a493a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x171a493c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x733a493e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3fa4940, SUC_lexicon_tags_37 },
    { 0xd45a4941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41a4944, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33a494f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc5da4951, SUC_lexicon_tags_68 },
    { 0x345a4952, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaba4956, SUC_lexicon_tags_62 },
    { 0x61fa4956, SUC_lexicon_tags_14 },
    { 0x767a4958, SUC_lexicon_tags_23 },
    { 0x4ada4959, SUC_lexicon_tags_105 },
    { 0x4bba495a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5a4961, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfda4964, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd61a4966, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x86da4969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22fa496b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ba496e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xde5a4970, SUC_lexicon_tags_19 },
    { 0x63a4970, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1a4975, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ba4978, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2eba497b, SUC_lexicon_tags_7 },
    { 0x723a497c, SUC_lexicon_tags_11 },
    { 0xbbda497b, SUC_lexicon_tags_518 },
    { 0x509a497e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a498a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917a498e, SUC_lexicon_tags_39 },
    { 0x6c7a498f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3a4994, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9a499e, SUC_lexicon_tags_37 },
    { 0x5a9a499f, SUC_lexicon_tags_25 },
    { 0xcf5a49a0, SUC_lexicon_tags_11 },
    { 0xaffa49a1, SUC_lexicon_tags_15 },
    { 0x119a49a0, SUC_lexicon_tags_49 },
    { 0xcffa499f, SUC_lexicon_tags_77 },
    { 0x1d7a49a4, SUC_lexicon_tags_6 },
    { 0x385a499f, SUC_lexicon_tags_43 },
    { 0xf99a49a6, SUC_lexicon_tags_23 },
    { 0x3afa49a7, SUC_lexicon_tags_29 },
    { 0xe4da49a8, SUC_lexicon_tags_29 },
    { 0xa25a49a9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa7ba49ab, SUC_lexicon_tags_63 },
    { 0x3dfa49ac, SUC_lexicon_tags_42 },
    { 0x465a49ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9a49b0, SUC_lexicon_tags_63 },
    { 0xe13a49b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fa49b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ba49ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf2fa49bc, SUC_lexicon_tags_38 },
    { 0x8d1a49bd, SUC_lexicon_tags_2 },
    { 0x5c3a49bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1a49c1, SUC_lexicon_tags_85 },
    { 0xc8ba49c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x343a49c4, SUC_lexicon_tags_31 },
    { 0xf05a49c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a49c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bba49ca, SUC_lexicon_tags_15 },
    { 0xb1ba49ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3a49ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17a49d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3a49d6, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727a49dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x643a49df, SUC_lexicon_tags_46 },
    { 0x59fa49df, SUC_lexicon_tags_1 },
    { 0xa67a49e1, SUC_lexicon_tags_11 },
    { 0x633a49e2, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3a49e5, SUC_lexicon_tags_2 },
    { 0x9c5a49e6, SUC_lexicon_tags_68 },
    { 0x1a1a49e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35da49ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb6fa49ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x10da49f1, SUC_lexicon_tags_55 },
    { 0xeffa49f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3da49f5, SUC_lexicon_tags_87 },
    { 0x8d9a49f6, SUC_lexicon_tags_27 },
    { 0x3cba49f6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9a49fc, SUC_lexicon_tags_19 },
    { 0x851a49fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e9a49ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1a4a05, SUC_lexicon_tags_39 },
    { 0xc6fa4a06, SUC_lexicon_tags_12 },
    { 0x1bfa4a07, SUC_lexicon_tags_75 },
    { 0x1ada4a07, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x79a4a0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfcda4a0c, SUC_lexicon_tags_28 },
    { 0x785a4a0d, SUC_lexicon_tags_23 },
    { 0xaa1a4a0c, SUC_lexicon_tags_60 },
    { 0xdb7a4a0e, SUC_lexicon_tags_30 },
    { 0x271a4a0d, SUC_lexicon_tags_4 },
    { 0x4cfa4a10, SUC_lexicon_tags_48 },
    { 0xcb1a4a11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487a4a16, SUC_lexicon_tags_79 },
    { 0xe5da4a17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdeda4a19, SUC_lexicon_tags_60 },
    { 0x7e5a4a1a, SUC_lexicon_tags_14 },
    { 0xd91a4a1b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x519a4a1d, SUC_lexicon_tags_17 },
    { 0xecfa4a1e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafa4a23, SUC_lexicon_tags_37 },
    { 0xd65a4a24, SUC_lexicon_tags_2 },
    { 0xcfa4a23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fba4a29, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x81fa4a2b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a4a2e, SUC_lexicon_tags_46 },
    { 0x5c5a4a2f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1c5a4a31, SUC_lexicon_tags_42 },
    { 0x8d1a4a32, SUC_lexicon_tags_19 },
    { 0xe03a4a32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x411a4a35, SUC_lexicon_tags_11 },
    { 0x7b7a4a36, SUC_lexicon_tags_7 },
    { 0x121a4a35, SUC_lexicon_tags_1 },
    { 0x235a4a35, SUC_lexicon_tags_12 },
    { 0xc37a4a39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x123a4a3c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x587a4a3e, SUC_lexicon_tags_200 },
    { 0xd93a4a3f, SUC_lexicon_tags_37 },
    { 0xcf5a4a40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47a4a42, SUC_lexicon_tags_9 },
    { 0xbf9a4a43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9a4a4a, SUC_lexicon_tags_1 },
    { 0x59a4a4b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x863a4a4d, SUC_lexicon_tags_25 },
    { 0xdc5a4a4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92da4a50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c1a4a52, SUC_lexicon_tags_38 },
    { 0x171a4a52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ba4a5b, SUC_lexicon_tags_16 },
    { 0x39ba4a5c, SUC_lexicon_tags_15 },
    { 0xa35a4a5c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x2c5a4a5f, SUC_lexicon_tags_17 },
    { 0x54da4a60, SUC_lexicon_tags_25 },
    { 0xbf7a4a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd9a4a63, SUC_lexicon_tags_9 },
    { 0x947a4a64, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1f7a4a66, SUC_lexicon_tags_15 },
    { 0xb31a4a67, SUC_lexicon_tags_17 },
    { 0x113a4a68, SUC_lexicon_tags_1 },
    { 0x453a4a68, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x717a4a6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x509a4a6e, SUC_lexicon_tags_48 },
    { 0x33a4a6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e1a4a71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa65a4a73, SUC_lexicon_tags_27 },
    { 0x8fda4a73, SUC_lexicon_tags_17 },
    { 0x3b7a4a75, SUC_lexicon_tags_17 },
    { 0xceda4a76, SUC_lexicon_tags_2 },
    { 0x57da4a74, SUC_lexicon_tags_26 },
    { 0xbffa4a78, SUC_lexicon_tags_17 },
    { 0x743a4a79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31a4a7d, SUC_lexicon_tags_23 },
    { 0x723a4a7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc81a4a80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7a4a85, SUC_lexicon_tags_32 },
    { 0x981a4a86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fa4a8a, SUC_lexicon_tags_91 },
    { 0x9efa4a8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dda4a8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eba4a96, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5a4a9d, SUC_lexicon_tags_11 },
    { 0x2efa4a9d, SUC_lexicon_tags_7 },
    { 0xe4fa4a9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x151a4aa1, SUC_lexicon_tags_37 },
    { 0x7eda4aa2, SUC_lexicon_tags_27 },
    { 0x94fa4aa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3a4aa6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x583a4aa9, SUC_lexicon_tags_29 },
    { 0xcf9a4aa9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43a4ab1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a4abc, SUC_lexicon_tags_12 },
    { 0x59da4abc, SUC_lexicon_tags_33 },
    { 0x3bfa4abd, SUC_lexicon_tags_25 },
    { 0x8cba4abd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25a4ac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59a4ad2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37a4ad6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41a4adb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5a4ae2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf7a4ae4, SUC_lexicon_tags_2 },
    { 0x3fa4ae5, SUC_lexicon_tags_33 },
    { 0x49a4ae6, SUC_lexicon_tags_43 },
    { 0x8cda4ae5, SUC_lexicon_tags_7 },
    { 0x723a4ae8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7a4aeb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2eba4aef, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xe4fa4af1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x845a4af3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb93a4af5, SUC_lexicon_tags_17 },
    { 0xfc3a4af6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475a4afd, SUC_lexicon_tags_43 },
    { 0xdf9a4afe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfefa4b01, SUC_lexicon_tags_43 },
    { 0x3a3a4b01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657a4b06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7a4b0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1fa4b0c, SUC_lexicon_tags_38 },
    { 0xd7a4b0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74ba4b0f, SUC_lexicon_tags_28 },
    { 0x29a4b10, SUC_lexicon_tags_11 },
    { 0x85fa4b11, SUC_lexicon_tags_14 },
    { 0xf7da4b10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305a4b19, SUC_lexicon_tags_2 },
    { 0x465a4b1a, SUC_lexicon_tags_11 },
    { 0xea9a4b1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21a4b1f, SUC_lexicon_tags_12 },
    { 0x5c5a4b20, SUC_lexicon_tags_27 },
    { 0xaf1a4b20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42ba4b23, SUC_lexicon_tags_27 },
    { 0xf7fa4b24, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa25a4b26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x499a4b28, SUC_lexicon_tags_31 },
    { 0x53fa4b28, SUC_lexicon_tags_112 },
    { 0x763a4b2a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdffa4b2c, SUC_lexicon_tags_1 },
    { 0x87a4b2c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x84da4b30, SUC_lexicon_tags_31 },
    { 0x82da4b31, SUC_lexicon_tags_14 },
    { 0x663a4b32, SUC_lexicon_tags_17 },
    { 0xa0ba4b33, SUC_lexicon_tags_11 },
    { 0x811a4b33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdba4b36, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a4b40, SUC_lexicon_tags_25 },
    { 0xfa4b41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x277a4b44, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70da4b48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba9a4b4a, SUC_lexicon_tags_2 },
    { 0x955a4b4b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21a4b4f, SUC_lexicon_tags_38 },
    { 0xddda4b50, SUC_lexicon_tags_15 },
    { 0xf0da4b50, SUC_lexicon_tags_9 },
    { 0x555a4b52, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fa4b56, SUC_lexicon_tags_1 },
    { 0xe25a4b57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665a4b5c, SUC_lexicon_tags_7 },
    { 0x151a4b5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55a4b60, SUC_lexicon_tags_32 },
    { 0xa1ba4b61, SUC_lexicon_tags_17 },
    { 0xc65a4b60, SUC_lexicon_tags_7 },
    { 0xb61a4b63, SUC_lexicon_tags_60 },
    { 0xecfa4b64, SUC_lexicon_tags_27 },
    { 0x8cda4b65, SUC_lexicon_tags_12 },
    { 0xa6da4b60, SUC_lexicon_tags_37 },
    { 0x6ffa4b67, SUC_lexicon_tags_17 },
    { 0xf19a4b68, SUC_lexicon_tags_15 },
    { 0x8d5a4b63, SUC_lexicon_tags_1 },
    { 0x8eba4b6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x301a4b6c, SUC_lexicon_tags_2 },
    { 0x627a4b6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc71a4b6f, SUC_lexicon_tags_11 },
    { 0x6eba4b70, SUC_lexicon_tags_70 },
    { 0xdffa4b71, SUC_lexicon_tags_1 },
    { 0xc0fa4b70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x36fa4b75, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ada4b79, SUC_lexicon_tags_10 },
    { 0x523a4b7a, SUC_lexicon_tags_13 },
    { 0x3fda4b79, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x39ba4b7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xabda4b7f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5da4b83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x84ba4b85, SUC_lexicon_tags_2 },
    { 0xd45a4b85, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xd15a4b88, SUC_lexicon_tags_65 },
    { 0x239a4b89, SUC_lexicon_tags_7 },
    { 0x9e5a4b89, SUC_lexicon_tags_15 },
    { 0x78da4b8a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5a4b8e, SUC_lexicon_tags_3 },
    { 0xfcba4b8f, SUC_lexicon_tags_12 },
    { 0xca9a4b8f, SUC_lexicon_tags_17 },
    { 0x97a4b8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb59a4b93, SUC_lexicon_tags_0 },
    { 0x5ffa4b93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa1ba4b96, SUC_lexicon_tags_25 },
    { 0x8efa4b97, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x26fa4b99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbba4b9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5a4b9f, SUC_lexicon_tags_77 },
    { 0x72da4ba0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3a4ba5, SUC_lexicon_tags_32 },
    { 0x345a4ba6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51fa4ba8, SUC_lexicon_tags_17 },
    { 0x563a4ba8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a4bac, SUC_lexicon_tags_26 },
    { 0xbd7a4bac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa4bb4, SUC_lexicon_tags_15 },
    { 0xf31a4bb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bba4bb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb77a4bbb, SUC_lexicon_tags_15 },
    { 0xcffa4bbc, SUC_lexicon_tags_37 },
    { 0xeb5a4bbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfa4bc2, SUC_lexicon_tags_49 },
    { 0x1e7a4bc3, SUC_lexicon_tags_27 },
    { 0x82fa4bc3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33a4bca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5afa4bcc, SUC_lexicon_tags_12 },
    { 0x907a4bcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15a4bd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf5a4bd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5a4bdd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ba4be2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5a4be7, SUC_lexicon_tags_0 },
    { 0x11a4be8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501a4bed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10da4bf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffa4bf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69a4bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3a4c06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x937a4c09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957a4c13, SUC_lexicon_tags_17 },
    { 0x343a4c14, SUC_lexicon_tags_17 },
    { 0xc97a4c14, SUC_lexicon_tags_0 },
    { 0x659a4c16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa71a4c18, SUC_lexicon_tags_60 },
    { 0x279a4c19, SUC_lexicon_tags_30 },
    { 0x591a4c19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ba4c1d, SUC_lexicon_tags_1 },
    { 0xde1a4c1d, SUC_lexicon_tags_87 },
    { 0x3d7a4c1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61da4c26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfa4c29, SUC_lexicon_tags_17 },
    { 0x659a4c2a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x623a4c2c, SUC_lexicon_tags_64 },
    { 0x7cba4c2d, SUC_lexicon_tags_8 },
    { 0x9eda4c2d, SUC_lexicon_tags_3 },
    { 0x7a4c2f, SUC_lexicon_tags_11 },
    { 0x79fa4c2f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x393a4c32, SUC_lexicon_tags_1171 },
    { 0, NULL },
    { 0x14fa4c34, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa63a4c36, SUC_lexicon_tags_0 },
    { 0xfda4c37, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afa4c3c, SUC_lexicon_tags_25 },
    { 0x5b9a4c3d, SUC_lexicon_tags_47 },
    { 0x123a4c3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeda4c42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62da4c44, SUC_lexicon_tags_10 },
    { 0xff5a4c44, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x821a4c47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67da4c49, SUC_lexicon_tags_85 },
    { 0xf39a4c4a, SUC_lexicon_tags_33 },
    { 0xe05a4c4b, SUC_lexicon_tags_25 },
    { 0xed7a4c4b, SUC_lexicon_tags_1 },
    { 0xef1a4c4d, SUC_lexicon_tags_2 },
    { 0xde7a4c4c, SUC_lexicon_tags_1 },
    { 0x12ba4c4f, SUC_lexicon_tags_64 },
    { 0x7eba4c4a, SUC_lexicon_tags_17 },
    { 0xdc3a4c4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x365a4c53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905a4c59, SUC_lexicon_tags_12 },
    { 0x9ffa4c5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffa4c5d, SUC_lexicon_tags_53 },
    { 0x81ba4c5e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8d1a4c60, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x355a4c62, SUC_lexicon_tags_11 },
    { 0xab1a4c63, SUC_lexicon_tags_37 },
    { 0x3a1a4c63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1a4c67, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffda4c6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x469a4c71, SUC_lexicon_tags_17 },
    { 0x19da4c72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237a4c76, SUC_lexicon_tags_25 },
    { 0xf7a4c77, SUC_lexicon_tags_15 },
    { 0xdf9a4c77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23ba4c7a, SUC_lexicon_tags_18 },
    { 0x8f5a4c7b, SUC_lexicon_tags_2 },
    { 0x8eda4c7c, SUC_lexicon_tags_25 },
    { 0xeeba4c7d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79a4c82, SUC_lexicon_tags_4 },
    { 0xf4da4c83, SUC_lexicon_tags_0 },
    { 0xd9da4c84, SUC_lexicon_tags_14 },
    { 0x6bda4c85, SUC_lexicon_tags_2 },
    { 0x11ba4c86, SUC_lexicon_tags_23 },
    { 0xdd5a4c86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50da4c89, SUC_lexicon_tags_12 },
    { 0x193a4c8a, SUC_lexicon_tags_33 },
    { 0xc5a4c8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9a4c8e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3a4c94, SUC_lexicon_tags_26 },
    { 0x7c1a4c94, SUC_lexicon_tags_25 },
    { 0xd41a4c95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229a4c9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a4c9e, SUC_lexicon_tags_17 },
    { 0xbfda4c9f, SUC_lexicon_tags_28 },
    { 0xe15a4ca0, SUC_lexicon_tags_79 },
    { 0xce9a4ca1, SUC_lexicon_tags_56 },
    { 0xc95a4ca2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x941a4ca5, SUC_lexicon_tags_57 },
    { 0x7a4ca6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf79a4ca8, SUC_lexicon_tags_25 },
    { 0x197a4ca9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7a4cad, SUC_lexicon_tags_14 },
    { 0xfeba4cae, SUC_lexicon_tags_37 },
    { 0x395a4cae, SUC_lexicon_tags_7 },
    { 0x62ba4cb0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba4cb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd57a4cb6, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a4cbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fa4cc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc9a4cc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765a4cc6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x767a4cc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9a4cce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5fa4cd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x273a4cd2, SUC_lexicon_tags_27 },
    { 0xf89a4cd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c5a4cd5, SUC_lexicon_tags_17 },
    { 0xf5a4cd6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79a4cdd, SUC_lexicon_tags_8 },
    { 0xe5da4cde, SUC_lexicon_tags_11 },
    { 0x409a4cdf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x871a4ce2, SUC_lexicon_tags_1 },
    { 0x4dda4ce3, SUC_lexicon_tags_63 },
    { 0xc29a4ce4, SUC_lexicon_tags_10 },
    { 0xa01a4ce4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69a4ce7, SUC_lexicon_tags_22 },
    { 0x503a4ce7, SUC_lexicon_tags_12 },
    { 0xb25a4ce9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb17a4ceb, SUC_lexicon_tags_87 },
    { 0xe93a4cec, SUC_lexicon_tags_1 },
    { 0x4f7a4ceb, SUC_lexicon_tags_54 },
    { 0xeb7a4cee, SUC_lexicon_tags_0 },
    { 0xc2ba4cec, SUC_lexicon_tags_45 },
    { 0x9a1a4cf0, SUC_lexicon_tags_12 },
    { 0x7eda4cf1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bda4cf4, SUC_lexicon_tags_2 },
    { 0xc21a4cf4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x819a4cf8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da4d00, SUC_lexicon_tags_2 },
    { 0x451a4d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5a4d04, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x463a4d06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5a4d0c, SUC_lexicon_tags_1 },
    { 0xd03a4d0d, SUC_lexicon_tags_29 },
    { 0x3cfa4d0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c1a4d10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ba4d13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01a4d18, SUC_lexicon_tags_14 },
    { 0x65a4d19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x839a4d1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeba4d1f, SUC_lexicon_tags_7 },
    { 0xd97a4d20, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfa4d24, SUC_lexicon_tags_10 },
    { 0xf33a4d24, SUC_lexicon_tags_15 },
    { 0xb37a4d24, SUC_lexicon_tags_43 },
    { 0x2c1a4d27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9a4d2c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9b7a4d2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9a4d33, SUC_lexicon_tags_25 },
    { 0x1a1a4d33, SUC_lexicon_tags_19 },
    { 0x895a4d34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x663a4d38, SUC_lexicon_tags_1 },
    { 0x5a3a4d39, SUC_lexicon_tags_19 },
    { 0xf7fa4d3a, SUC_lexicon_tags_27 },
    { 0xd45a4d3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7a4d3e, SUC_lexicon_tags_25 },
    { 0x9a5a4d3e, SUC_lexicon_tags_159 },
    { 0x78da4d40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x707a4d42, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x411a4d45, SUC_lexicon_tags_25 },
    { 0x445a4d46, SUC_lexicon_tags_87 },
    { 0xd65a4d46, SUC_lexicon_tags_2 },
    { 0x15fa4d47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da4d4e, SUC_lexicon_tags_11 },
    { 0xf29a4d4f, SUC_lexicon_tags_2 },
    { 0x5d9a4d50, SUC_lexicon_tags_21 },
    { 0xce3a4d50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5a4d54, SUC_lexicon_tags_25 },
    { 0x94da4d55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96da4d58, SUC_lexicon_tags_15 },
    { 0x463a4d59, SUC_lexicon_tags_25 },
    { 0x17da4d59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7a4d5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7a4d62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4da4d65, SUC_lexicon_tags_12 },
    { 0x747a4d65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1a4d6b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1a4d6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x79a4d71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x825a4d74, SUC_lexicon_tags_25 },
    { 0xeba4d75, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x31fa4d77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x609a4d79, SUC_lexicon_tags_7 },
    { 0x60fa4d7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97a4d7f, SUC_lexicon_tags_2 },
    { 0xfe5a4d80, SUC_lexicon_tags_12 },
    { 0x8b1a4d81, SUC_lexicon_tags_30 },
    { 0x92ba4d81, SUC_lexicon_tags_11 },
    { 0x989a4d82, SUC_lexicon_tags_6 },
    { 0xc17a4d84, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3a4d89, SUC_lexicon_tags_7 },
    { 0xe81a4d8a, SUC_lexicon_tags_15 },
    { 0x297a4d8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ba4d8e, SUC_lexicon_tags_38 },
    { 0xcf1a4d8f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41a4d93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ba4d96, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97a4d9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da4d9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91da4da2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9a4da5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x463a4da7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b3a4da9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ba4dac, SUC_lexicon_tags_37 },
    { 0xbf7a4dad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfa4db0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd45a4db2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3fa4db4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fa4db9, SUC_lexicon_tags_7 },
    { 0xdb9a4dba, SUC_lexicon_tags_31 },
    { 0x107a4dbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe85a4dbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69a4dc1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25a4dc7, SUC_lexicon_tags_61 },
    { 0x81a4dc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x607a4dca, SUC_lexicon_tags_9 },
    { 0xfa9a4dcb, SUC_lexicon_tags_18 },
    { 0x6afa4dcc, SUC_lexicon_tags_25 },
    { 0xf17a4dcc, SUC_lexicon_tags_17 },
    { 0xc0fa4dce, SUC_lexicon_tags_37 },
    { 0x71fa4dcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3a4dd3, SUC_lexicon_tags_7 },
    { 0x5e9a4dd4, SUC_lexicon_tags_84 },
    { 0xa0ba4dd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf51a4dd7, SUC_lexicon_tags_9 },
    { 0xb6da4dd8, SUC_lexicon_tags_6 },
    { 0x325a4dd9, SUC_lexicon_tags_46 },
    { 0x187a4dd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29a4de5, SUC_lexicon_tags_2 },
    { 0x5ba4de5, SUC_lexicon_tags_19 },
    { 0x3e3a4de7, SUC_lexicon_tags_37 },
    { 0xf6ba4de7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65a4deb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1e3a4ded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe27a4def, SUC_lexicon_tags_43 },
    { 0xcd5a4def, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11a4df3, SUC_lexicon_tags_65 },
    { 0xfb7a4df3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb59a4df6, SUC_lexicon_tags_17 },
    { 0xa75a4df7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe5a4df9, SUC_lexicon_tags_29 },
    { 0xfd5a4dfa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dba4dfe, SUC_lexicon_tags_6 },
    { 0x985a4dff, SUC_lexicon_tags_8 },
    { 0x2cba4e00, SUC_lexicon_tags_12 },
    { 0xd1da4e01, SUC_lexicon_tags_33 },
    { 0x5a1a4e00, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a4e05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86ba4e07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273a4e0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335a4e13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x535a4e15, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xee5a4e17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66da4e1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdda4e25, SUC_lexicon_tags_37 },
    { 0x213a4e25, SUC_lexicon_tags_37 },
    { 0xc49a4e25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffba4e2b, SUC_lexicon_tags_2 },
    { 0xca9a4e2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a4e2f, SUC_lexicon_tags_8 },
    { 0xf9fa4e30, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x36da4e33, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x39ba4e35, SUC_lexicon_tags_37 },
    { 0xa35a4e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ba4e39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x405a4e3b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfb1a4e3d, SUC_lexicon_tags_47 },
    { 0xea3a4e3d, SUC_lexicon_tags_33 },
    { 0x4dba4e3e, SUC_lexicon_tags_15 },
    { 0x875a4e40, SUC_lexicon_tags_15 },
    { 0xdc1a4e3e, SUC_lexicon_tags_28 },
    { 0x1b1a4e40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2ba4e44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a4e49, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6cfa4e4b, SUC_lexicon_tags_207 },
    { 0x58ba4e4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x965a4e4e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5a4e54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1a4e57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ba4e5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fda4e60, SUC_lexicon_tags_37 },
    { 0xb0ba4e61, SUC_lexicon_tags_4 },
    { 0x401a4e61, SUC_lexicon_tags_26 },
    { 0xc51a4e62, SUC_lexicon_tags_7 },
    { 0xa9da4e64, SUC_lexicon_tags_27 },
    { 0x947a4e64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x497a4e67, SUC_lexicon_tags_43 },
    { 0x25fa4e67, SUC_lexicon_tags_15 },
    { 0x465a4e69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27a4e6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x483a4e6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4da4e72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x987a4e75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9a4e7b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe6da4e7d, SUC_lexicon_tags_56 },
    { 0xb85a4e7d, SUC_lexicon_tags_7 },
    { 0xabfa4e7f, SUC_lexicon_tags_27 },
    { 0xc83a4e7f, SUC_lexicon_tags_12 },
    { 0xe0fa4e81, SUC_lexicon_tags_11 },
    { 0xc7a4e82, SUC_lexicon_tags_33 },
    { 0xe6ba4e7e, SUC_lexicon_tags_17 },
    { 0x567a4e84, SUC_lexicon_tags_49 },
    { 0x185a4e82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdda4e8e, SUC_lexicon_tags_37 },
    { 0xf9ba4e8f, SUC_lexicon_tags_2 },
    { 0xd49a4e90, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeda4e97, SUC_lexicon_tags_15 },
    { 0xc41a4e97, SUC_lexicon_tags_55 },
    { 0x72da4e99, SUC_lexicon_tags_28 },
    { 0xa61a4e9a, SUC_lexicon_tags_68 },
    { 0x86ba4e9b, SUC_lexicon_tags_12 },
    { 0x7f3a4e9c, SUC_lexicon_tags_29 },
    { 0xf1a4e9c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd6ba4e9f, SUC_lexicon_tags_37 },
    { 0x18da4ea0, SUC_lexicon_tags_15 },
    { 0xbb9a4ea0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ba4ea4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa4ea7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79a4eaa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5a4ead, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3a4eb0, SUC_lexicon_tags_37 },
    { 0x27da4eb1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2da4eb5, SUC_lexicon_tags_11 },
    { 0xbda4eb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd47a4eb8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x38da4eba, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xe69a4ebc, SUC_lexicon_tags_17 },
    { 0x1c7a4ebd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb51a4ebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x493a4ec1, SUC_lexicon_tags_7 },
    { 0x6efa4ec2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9fa4ec4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fa4ec8, SUC_lexicon_tags_17 },
    { 0x387a4ec8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73a4ecc, SUC_lexicon_tags_57 },
    { 0xbdfa4ecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9a4ed0, SUC_lexicon_tags_0 },
    { 0x7f5a4ed1, SUC_lexicon_tags_21 },
    { 0xbfba4ed2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba4ed8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98da4edd, SUC_lexicon_tags_27 },
    { 0xd9da4edd, SUC_lexicon_tags_25 },
    { 0x519a4edf, SUC_lexicon_tags_80 },
    { 0xeb5a4ee0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe79a4ee2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57da4eea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5a4eed, SUC_lexicon_tags_2 },
    { 0x4a1a4eed, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201a4ef4, SUC_lexicon_tags_19 },
    { 0x737a4ef4, SUC_lexicon_tags_17 },
    { 0x65ba4ef6, SUC_lexicon_tags_23 },
    { 0x4f9a4ef6, SUC_lexicon_tags_61 },
    { 0x901a4ef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5a4efd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3aba4eff, SUC_lexicon_tags_6 },
    { 0x9d5a4f00, SUC_lexicon_tags_15 },
    { 0xad9a4f01, SUC_lexicon_tags_15 },
    { 0xf65a4f01, SUC_lexicon_tags_39 },
    { 0x5dfa4f01, SUC_lexicon_tags_15 },
    { 0xf5da4f03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x871a4f07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fa4f09, SUC_lexicon_tags_37 },
    { 0xe0da4f0a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fa4f0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fa4f14, SUC_lexicon_tags_16 },
    { 0xbfa4f14, SUC_lexicon_tags_47 },
    { 0xf87a4f16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab5a4f18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x769a4f1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x47ba4f1c, SUC_lexicon_tags_8 },
    { 0x289a4f1d, SUC_lexicon_tags_25 },
    { 0x25a4f1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897a4f22, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69a4f29, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899a4f2e, SUC_lexicon_tags_1 },
    { 0xde5a4f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627a4f33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637a4f37, SUC_lexicon_tags_57 },
    { 0x519a4f38, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1a4f3b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xebba4f3e, SUC_lexicon_tags_3 },
    { 0x99da4f3e, SUC_lexicon_tags_2 },
    { 0x397a4f3e, SUC_lexicon_tags_33 },
    { 0x4a7a4f41, SUC_lexicon_tags_34 },
    { 0x23a4f3e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a4f4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cba4f4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c7a4f51, SUC_lexicon_tags_9 },
    { 0xd1ba4f52, SUC_lexicon_tags_17 },
    { 0xe1ba4f53, SUC_lexicon_tags_11 },
    { 0x417a4f54, SUC_lexicon_tags_37 },
    { 0xcf5a4f55, SUC_lexicon_tags_28 },
    { 0xf6ba4f56, SUC_lexicon_tags_27 },
    { 0x57ba4f57, SUC_lexicon_tags_68 },
    { 0x133a4f56, SUC_lexicon_tags_27 },
    { 0x213a4f57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e9a4f5b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x63fa4f5e, SUC_lexicon_tags_15 },
    { 0x789a4f5f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807a4f63, SUC_lexicon_tags_37 },
    { 0x32da4f64, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ada4f6b, SUC_lexicon_tags_2 },
    { 0xc73a4f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6da4f6f, SUC_lexicon_tags_6 },
    { 0xda4f70, SUC_lexicon_tags_2 },
    { 0x737a4f71, SUC_lexicon_tags_37 },
    { 0x687a4f72, SUC_lexicon_tags_29 },
    { 0x257a4f71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc7ba4f75, SUC_lexicon_tags_39 },
    { 0x8afa4f75, SUC_lexicon_tags_7 },
    { 0x5e9a4f75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11a4f7a, SUC_lexicon_tags_7 },
    { 0x277a4f7b, SUC_lexicon_tags_2 },
    { 0x7b3a4f7c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x10fa4f7e, SUC_lexicon_tags_37 },
    { 0x18fa4f7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0fa4f81, SUC_lexicon_tags_2 },
    { 0x8f7a4f82, SUC_lexicon_tags_1 },
    { 0x2a3a4f83, SUC_lexicon_tags_7 },
    { 0xf13a4f83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbba4f87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67a4f8a, SUC_lexicon_tags_15 },
    { 0xa15a4f8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45a4f8d, SUC_lexicon_tags_33 },
    { 0xd01a4f8e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fa4f93, SUC_lexicon_tags_70 },
    { 0x9f1a4f93, SUC_lexicon_tags_11 },
    { 0x2cda4f95, SUC_lexicon_tags_47 },
    { 0x203a4f96, SUC_lexicon_tags_9 },
    { 0xdb7a4f97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf97a4f99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78da4f9c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9a4fa0, SUC_lexicon_tags_17 },
    { 0xe95a4fa1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43a4fa5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10da4fa7, SUC_lexicon_tags_171 },
    { 0xd93a4fa7, SUC_lexicon_tags_79 },
    { 0x941a4fa9, SUC_lexicon_tags_12 },
    { 0xc75a4faa, SUC_lexicon_tags_71 },
    { 0x6c9a4fab, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07a4fae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cba4fb8, SUC_lexicon_tags_15 },
    { 0xc61a4fb9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cba4fc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9a4fc4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7a4fcb, SUC_lexicon_tags_47 },
    { 0x73ba4fcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x261a4fcf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5a4fd2, SUC_lexicon_tags_1 },
    { 0xf0da4fd3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa5da4fd5, SUC_lexicon_tags_17 },
    { 0xeb5a4fd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17a4fd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4eba4fda, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5a4fdd, SUC_lexicon_tags_99 },
    { 0xd79a4fde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5a4fe4, SUC_lexicon_tags_65 },
    { 0xc07a4fe5, SUC_lexicon_tags_39 },
    { 0xc21a4fe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0da4fea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573a4ff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39a4ff5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbfda4ff7, SUC_lexicon_tags_46 },
    { 0x25a4ff8, SUC_lexicon_tags_15 },
    { 0xbf3a4ff9, SUC_lexicon_tags_33 },
    { 0x8a1a4ffa, SUC_lexicon_tags_45 },
    { 0x3e9a4ff9, SUC_lexicon_tags_1 },
    { 0xb71a4ff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa95a4ffe, SUC_lexicon_tags_2 },
    { 0x67fa4ffe, SUC_lexicon_tags_45 },
    { 0x8cfa4fff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3dba5002, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba5005, SUC_lexicon_tags_9 },
    { 0xb0fa5005, SUC_lexicon_tags_31 },
    { 0xa83a5007, SUC_lexicon_tags_7 },
    { 0x5e7a5006, SUC_lexicon_tags_63 },
    { 0x585a5008, SUC_lexicon_tags_1 },
    { 0x3f9a5006, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x929a500d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7fba500f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9a5012, SUC_lexicon_tags_6 },
    { 0x1d1a5013, SUC_lexicon_tags_16 },
    { 0x273a5012, SUC_lexicon_tags_39 },
    { 0x133a5014, SUC_lexicon_tags_48 },
    { 0xdb1a5016, SUC_lexicon_tags_17 },
    { 0xedda5016, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x341a501a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991a501e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd13a5020, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xc21a5022, SUC_lexicon_tags_1 },
    { 0x955a5023, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x663a5026, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x549a5029, SUC_lexicon_tags_25 },
    { 0x941a5029, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x463a502d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453a5032, SUC_lexicon_tags_17 },
    { 0x8f7a5032, SUC_lexicon_tags_35 },
    { 0xebda5034, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123a503c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715a5043, SUC_lexicon_tags_30 },
    { 0x151a5044, SUC_lexicon_tags_11 },
    { 0x72da5045, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fa5048, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7a504e, SUC_lexicon_tags_1 },
    { 0xcada504e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4da5053, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a5059, SUC_lexicon_tags_53 },
    { 0xc2da505a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aba505d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ba5061, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x151a5064, SUC_lexicon_tags_11 },
    { 0x183a5065, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ba506c, SUC_lexicon_tags_12 },
    { 0x4f1a506d, SUC_lexicon_tags_12 },
    { 0x9c5a506d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8ba5070, SUC_lexicon_tags_15 },
    { 0x74da5071, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3a5076, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eda5079, SUC_lexicon_tags_47 },
    { 0xcc3a507a, SUC_lexicon_tags_27 },
    { 0x1b5a5079, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5a507f, SUC_lexicon_tags_17 },
    { 0x917a5080, SUC_lexicon_tags_33 },
    { 0xca7a507f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01a5087, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d9a5089, SUC_lexicon_tags_92 },
    { 0x311a508a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9a508e, SUC_lexicon_tags_33 },
    { 0xd99a508f, SUC_lexicon_tags_74 },
    { 0x913a5090, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xebda5092, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x439a5095, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2da509c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efa50a1, SUC_lexicon_tags_2 },
    { 0xaa9a50a2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x403a50a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc47a50a7, SUC_lexicon_tags_29 },
    { 0x7c1a50a8, SUC_lexicon_tags_12 },
    { 0x71a50a9, SUC_lexicon_tags_32 },
    { 0xba50a9, SUC_lexicon_tags_4 },
    { 0xe45a50a9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2d5a50ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1a50b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639a50b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375a50b9, SUC_lexicon_tags_11 },
    { 0x687a50ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a50c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b5a50c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557a50c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ba50cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ada50ce, SUC_lexicon_tags_61 },
    { 0x3dfa50cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x415a50d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcda50d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3a50dc, SUC_lexicon_tags_6 },
    { 0x6d5a50dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237a50e3, SUC_lexicon_tags_2 },
    { 0x67da50e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cda50e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1a50ec, SUC_lexicon_tags_2 },
    { 0x711a50ec, SUC_lexicon_tags_32 },
    { 0x51da50ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13a50f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1a50f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc89a50fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fa510d, SUC_lexicon_tags_57 },
    { 0x5b5a510e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29ba5110, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7a5113, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5a5118, SUC_lexicon_tags_12 },
    { 0xa07a5119, SUC_lexicon_tags_16 },
    { 0xe19a511a, SUC_lexicon_tags_63 },
    { 0xddba5118, SUC_lexicon_tags_15 },
    { 0xfc1a511a, SUC_lexicon_tags_19 },
    { 0x411a511d, SUC_lexicon_tags_12 },
    { 0xe7ba511b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3a5122, SUC_lexicon_tags_30 },
    { 0x177a5123, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bba5126, SUC_lexicon_tags_1 },
    { 0xfbfa5127, SUC_lexicon_tags_4 },
    { 0x387a5128, SUC_lexicon_tags_12 },
    { 0xe73a5129, SUC_lexicon_tags_15 },
    { 0xb69a5127, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25da512d, SUC_lexicon_tags_63 },
    { 0x3c7a512e, SUC_lexicon_tags_25 },
    { 0x865a512f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efa5132, SUC_lexicon_tags_11 },
    { 0xa75a5132, SUC_lexicon_tags_2 },
    { 0xf53a5134, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5a5139, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xadfa513b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83a513f, SUC_lexicon_tags_46 },
    { 0xab1a5140, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54da5146, SUC_lexicon_tags_1 },
    { 0x5f7a5147, SUC_lexicon_tags_2 },
    { 0xf13a5146, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89a514e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9a5154, SUC_lexicon_tags_25 },
    { 0xdd5a5155, SUC_lexicon_tags_17 },
    { 0x9c7a5155, SUC_lexicon_tags_38 },
    { 0xf43a5157, SUC_lexicon_tags_14 },
    { 0xcba5157, SUC_lexicon_tags_30 },
    { 0x93da5159, SUC_lexicon_tags_10 },
    { 0x1cba5154, SUC_lexicon_tags_7 },
    { 0x427a515b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9a5161, SUC_lexicon_tags_37 },
    { 0x603a5162, SUC_lexicon_tags_63 },
    { 0x6b1a5163, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ba5166, SUC_lexicon_tags_14 },
    { 0x97fa5167, SUC_lexicon_tags_30 },
    { 0x1a9a5168, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x651a516b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321a5172, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807a5176, SUC_lexicon_tags_37 },
    { 0x969a5176, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7afa517a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73a517d, SUC_lexicon_tags_279 },
    { 0x853a517d, SUC_lexicon_tags_37 },
    { 0xfc5a517f, SUC_lexicon_tags_31 },
    { 0x86ba517f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba5184, SUC_lexicon_tags_23 },
    { 0x5dba5185, SUC_lexicon_tags_14 },
    { 0x1e7a5185, SUC_lexicon_tags_12 },
    { 0x8c7a5187, SUC_lexicon_tags_37 },
    { 0x2da5188, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ba518c, SUC_lexicon_tags_39 },
    { 0xf6ba518c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9a5192, SUC_lexicon_tags_2 },
    { 0x313a5193, SUC_lexicon_tags_65 },
    { 0xd1ba5194, SUC_lexicon_tags_113 },
    { 0xc07a5194, SUC_lexicon_tags_39 },
    { 0x7b3a5194, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443a519e, SUC_lexicon_tags_68 },
    { 0xb75a519f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bba51a2, SUC_lexicon_tags_11 },
    { 0xf3da51a3, SUC_lexicon_tags_7 },
    { 0xd1da51a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8ba51a6, SUC_lexicon_tags_32 },
    { 0xd9da51a7, SUC_lexicon_tags_7 },
    { 0xcbfa51a8, SUC_lexicon_tags_62 },
    { 0xd5a51a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e9a51ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xca1a51ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963a51b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x633a51b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x167a51b5, SUC_lexicon_tags_31 },
    { 0xac3a51b6, SUC_lexicon_tags_14 },
    { 0x38ba51b6, SUC_lexicon_tags_37 },
    { 0xcb3a51b8, SUC_lexicon_tags_43 },
    { 0x15ba51b8, SUC_lexicon_tags_22 },
    { 0xeeba51ba, SUC_lexicon_tags_21 },
    { 0xbdfa51bb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdcda51bd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x709a51bf, SUC_lexicon_tags_45 },
    { 0xe7fa51c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fa51c4, SUC_lexicon_tags_11 },
    { 0x32da51c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fa51ce, SUC_lexicon_tags_43 },
    { 0xd63a51cf, SUC_lexicon_tags_33 },
    { 0x7bfa51d0, SUC_lexicon_tags_27 },
    { 0x3b9a51d1, SUC_lexicon_tags_6 },
    { 0x87da51d0, SUC_lexicon_tags_43 },
    { 0xf95a51cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16da51d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x237a51d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1a51dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869a51e1, SUC_lexicon_tags_11 },
    { 0x6efa51e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3a51e6, SUC_lexicon_tags_15 },
    { 0x945a51e7, SUC_lexicon_tags_31 },
    { 0x377a51e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85a51eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9a51f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed1a51f5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13a51f9, SUC_lexicon_tags_24 },
    { 0x9e9a51f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91a51ff, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7a5202, SUC_lexicon_tags_14 },
    { 0x741a5203, SUC_lexicon_tags_28 },
    { 0x76fa5204, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5a5208, SUC_lexicon_tags_11 },
    { 0x39fa5208, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a520f, SUC_lexicon_tags_11 },
    { 0xc15a520f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x12da5213, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13a5216, SUC_lexicon_tags_1 },
    { 0x999a5217, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69da5220, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73a5224, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2fda5226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x929a5229, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3a522d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a5232, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a9a5234, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdda5237, SUC_lexicon_tags_129 },
    { 0x33ba5238, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3a523c, SUC_lexicon_tags_0 },
    { 0xf23a523c, SUC_lexicon_tags_17 },
    { 0x33a523e, SUC_lexicon_tags_43 },
    { 0xb6fa523d, SUC_lexicon_tags_1 },
    { 0x203a523e, SUC_lexicon_tags_12 },
    { 0x17da5241, SUC_lexicon_tags_437 },
    { 0xf6fa5240, SUC_lexicon_tags_7 },
    { 0x907a5243, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x113a5245, SUC_lexicon_tags_17 },
    { 0x42fa5245, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09a524a, SUC_lexicon_tags_1 },
    { 0xf5ba524b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1a5254, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x317a5256, SUC_lexicon_tags_118 },
    { 0x199a5257, SUC_lexicon_tags_49 },
    { 0x101a5257, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41da525e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb03a5260, SUC_lexicon_tags_25 },
    { 0x747a5260, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da5265, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x767a5267, SUC_lexicon_tags_15 },
    { 0xc6da5267, SUC_lexicon_tags_7 },
    { 0xb5da5269, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x223a526b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefa5270, SUC_lexicon_tags_11 },
    { 0xe35a5270, SUC_lexicon_tags_11 },
    { 0x777a5272, SUC_lexicon_tags_49 },
    { 0xd57a5273, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11a527a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cba5281, SUC_lexicon_tags_42 },
    { 0x399a5282, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x425a5284, SUC_lexicon_tags_133 },
    { 0xf2da5285, SUC_lexicon_tags_10 },
    { 0x21fa5286, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbfa5288, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd91a528a, SUC_lexicon_tags_1 },
    { 0x26da528b, SUC_lexicon_tags_43 },
    { 0xc5ba528b, SUC_lexicon_tags_2 },
    { 0x1b7a528d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdda528f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cda529a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10ba529c, SUC_lexicon_tags_19 },
    { 0x11da529d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfba52a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe31a52a2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11a52a8, SUC_lexicon_tags_25 },
    { 0x281a52a9, SUC_lexicon_tags_63 },
    { 0xa73a52aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ba52af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1a52b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5a52b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9a52b8, SUC_lexicon_tags_32 },
    { 0xc27a52b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x919a52bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25a52be, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x11fa52c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x36fa52c2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a52c9, SUC_lexicon_tags_15 },
    { 0xa39a52ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747a52d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fa52df, SUC_lexicon_tags_1 },
    { 0xfd3a52df, SUC_lexicon_tags_7 },
    { 0x75da52e1, SUC_lexicon_tags_15 },
    { 0x689a52e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x977a52e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93a52ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x23a52ec, SUC_lexicon_tags_18 },
    { 0xcc7a52ed, SUC_lexicon_tags_129 },
    { 0xa3da52ec, SUC_lexicon_tags_33 },
    { 0xedba52ef, SUC_lexicon_tags_43 },
    { 0x9f3a52ef, SUC_lexicon_tags_43 },
    { 0x45da52ee, SUC_lexicon_tags_37 },
    { 0x771a52f0, SUC_lexicon_tags_46 },
    { 0xfe5a52ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fa52fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa5300, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x7e7a5302, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8b5a5304, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x819a5306, SUC_lexicon_tags_11 },
    { 0x2e9a5307, SUC_lexicon_tags_56 },
    { 0xd41a5308, SUC_lexicon_tags_25 },
    { 0xaa5a5308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfa5311, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe51a5313, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x231a5316, SUC_lexicon_tags_12 },
    { 0xa77a5316, SUC_lexicon_tags_49 },
    { 0x22fa5317, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x629a531a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3a5321, SUC_lexicon_tags_44 },
    { 0xfbba5322, SUC_lexicon_tags_31 },
    { 0xc27a5321, SUC_lexicon_tags_12 },
    { 0xba3a5324, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf6ba5326, SUC_lexicon_tags_2 },
    { 0x30ba5327, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203a532c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x115a532e, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0xbfda5330, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863a5338, SUC_lexicon_tags_49 },
    { 0xe61a5339, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x827a533b, SUC_lexicon_tags_25 },
    { 0x5eba533b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x361a533f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xfa5a5341, SUC_lexicon_tags_12 },
    { 0xb63a5342, SUC_lexicon_tags_4 },
    { 0xb5ba5343, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x769a5345, SUC_lexicon_tags_7 },
    { 0xa8ba5346, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fa5351, SUC_lexicon_tags_4 },
    { 0x8e5a5351, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x56ba5354, SUC_lexicon_tags_60 },
    { 0xc17a5355, SUC_lexicon_tags_25 },
    { 0x9d9a5354, SUC_lexicon_tags_25 },
    { 0xb7fa5357, SUC_lexicon_tags_37 },
    { 0xfbfa5358, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf9a535a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x391a535c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80fa535e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd01a5360, SUC_lexicon_tags_2 },
    { 0xdb9a5361, SUC_lexicon_tags_17 },
    { 0xbbba5362, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191a5366, SUC_lexicon_tags_49 },
    { 0x3eda5367, SUC_lexicon_tags_45 },
    { 0x7a1a5368, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x17ba536a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba536f, SUC_lexicon_tags_11 },
    { 0xb9ba5370, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28ba5372, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffa5375, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ba537c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc6da537e, SUC_lexicon_tags_2 },
    { 0x9cfa537e, SUC_lexicon_tags_47 },
    { 0x5afa5380, SUC_lexicon_tags_65 },
    { 0xb9ba5380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fa5385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x225a5388, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ba538b, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xb37a538d, SUC_lexicon_tags_47 },
    { 0xefba538e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa3a5390, SUC_lexicon_tags_2 },
    { 0x4b5a5391, SUC_lexicon_tags_13 },
    { 0x4c5a5392, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4d5a5394, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefa5399, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x88fa539b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x513a539d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9a53a0, SUC_lexicon_tags_37 },
    { 0x2eda53a0, SUC_lexicon_tags_15 },
    { 0xaf7a53a0, SUC_lexicon_tags_14 },
    { 0x1e9a53a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eda53a7, SUC_lexicon_tags_37 },
    { 0x135a53a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x591a53aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x243a53ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c3a53af, SUC_lexicon_tags_36 },
    { 0x4eda53b0, SUC_lexicon_tags_45 },
    { 0xb03a53b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x21a53b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x323a53b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7a53c0, SUC_lexicon_tags_37 },
    { 0xbc3a53c1, SUC_lexicon_tags_65 },
    { 0x22ba53c0, SUC_lexicon_tags_1 },
    { 0xc5da53c1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcd1a53c5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729a53c9, SUC_lexicon_tags_47 },
    { 0x25da53ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fa53cd, SUC_lexicon_tags_42 },
    { 0x763a53ce, SUC_lexicon_tags_33 },
    { 0xc9a53cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9a53d3, SUC_lexicon_tags_25 },
    { 0xcda53d4, SUC_lexicon_tags_7 },
    { 0xf87a53d3, SUC_lexicon_tags_15 },
    { 0x807a53d6, SUC_lexicon_tags_27 },
    { 0x44da53d3, SUC_lexicon_tags_57 },
    { 0x301a53d8, SUC_lexicon_tags_27 },
    { 0xd25a53d9, SUC_lexicon_tags_27 },
    { 0xca3a53d7, SUC_lexicon_tags_9 },
    { 0x5ba53db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27da53dd, SUC_lexicon_tags_11 },
    { 0x97da53dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949a53ec, SUC_lexicon_tags_65 },
    { 0x19da53ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fa53f0, SUC_lexicon_tags_11 },
    { 0xb2da53f0, SUC_lexicon_tags_74 },
    { 0xf7a53f2, SUC_lexicon_tags_12 },
    { 0xa49a53f3, SUC_lexicon_tags_26 },
    { 0xc23a53f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x887a53f6, SUC_lexicon_tags_63 },
    { 0x611a53f7, SUC_lexicon_tags_47 },
    { 0x151a53f8, SUC_lexicon_tags_11 },
    { 0xe2da53f9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83a53fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa9a5400, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fa5404, SUC_lexicon_tags_6 },
    { 0x193a5404, SUC_lexicon_tags_31 },
    { 0xb49a5404, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5a540b, SUC_lexicon_tags_0 },
    { 0x251a540c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7a540e, SUC_lexicon_tags_19 },
    { 0xe95a540f, SUC_lexicon_tags_15 },
    { 0xc93a540f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x239a5412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x433a5415, SUC_lexicon_tags_10 },
    { 0x4bfa5415, SUC_lexicon_tags_30 },
    { 0x24ba5416, SUC_lexicon_tags_39 },
    { 0x635a5418, SUC_lexicon_tags_57 },
    { 0xa09a5419, SUC_lexicon_tags_1 },
    { 0x725a541a, SUC_lexicon_tags_7 },
    { 0xf87a5418, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bda541e, SUC_lexicon_tags_26 },
    { 0x6bfa541f, SUC_lexicon_tags_336 },
    { 0x5e1a541f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebba5425, SUC_lexicon_tags_12 },
    { 0x8a7a5426, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d3a5428, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3a542b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4c3a542d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x585a5430, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe13a5432, SUC_lexicon_tags_25 },
    { 0xda5433, SUC_lexicon_tags_47 },
    { 0x651a5434, SUC_lexicon_tags_25 },
    { 0xb6fa5432, SUC_lexicon_tags_15 },
    { 0x6cba5436, SUC_lexicon_tags_0 },
    { 0x37a5434, SUC_lexicon_tags_33 },
    { 0xfb9a5438, SUC_lexicon_tags_59 },
    { 0x6d9a5438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5da543e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6cfa5440, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf05a5442, SUC_lexicon_tags_19 },
    { 0xb5da5443, SUC_lexicon_tags_30 },
    { 0x67fa5443, SUC_lexicon_tags_10 },
    { 0xa41a5443, SUC_lexicon_tags_13 },
    { 0xa3ba5446, SUC_lexicon_tags_25 },
    { 0xfa9a5444, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ba544a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6fa544c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cba5450, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ba5458, SUC_lexicon_tags_15 },
    { 0x629a5459, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6cfa545b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fa5462, SUC_lexicon_tags_0 },
    { 0x59a5463, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7bfa5465, SUC_lexicon_tags_19 },
    { 0x3e9a5466, SUC_lexicon_tags_26 },
    { 0x3bba5467, SUC_lexicon_tags_12 },
    { 0xa7fa5467, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7aba546a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd7fa546c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7efa546e, SUC_lexicon_tags_47 },
    { 0x9eda546e, SUC_lexicon_tags_17 },
    { 0x15da546f, SUC_lexicon_tags_38 },
    { 0x887a5471, SUC_lexicon_tags_2 },
    { 0x67da5472, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fa547a, SUC_lexicon_tags_26 },
    { 0x685a547b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x409a547e, SUC_lexicon_tags_8 },
    { 0x4efa547f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5a5482, SUC_lexicon_tags_87 },
    { 0xb09a5483, SUC_lexicon_tags_82 },
    { 0x56ba5483, SUC_lexicon_tags_9 },
    { 0x153a5483, SUC_lexicon_tags_15 },
    { 0x5eba5486, SUC_lexicon_tags_49 },
    { 0x89fa5487, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ba5495, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbfa5497, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87a5499, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9a549f, SUC_lexicon_tags_15 },
    { 0xe87a54a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53a54a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43a54a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369a54af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x78fa54b1, SUC_lexicon_tags_45 },
    { 0x27a54b2, SUC_lexicon_tags_8 },
    { 0x7b9a54b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33a54b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x81a54bb, SUC_lexicon_tags_30 },
    { 0xaf3a54bc, SUC_lexicon_tags_2 },
    { 0x477a54bd, SUC_lexicon_tags_2 },
    { 0xd21a54be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x153a54c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7a54c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39a54cb, SUC_lexicon_tags_14 },
    { 0x303a54cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92da54d1, SUC_lexicon_tags_1 },
    { 0x353a54d1, SUC_lexicon_tags_37 },
    { 0x489a54d3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2eba54d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f7a54d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x573a54d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fa54de, SUC_lexicon_tags_7 },
    { 0xe75a54de, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ba54e3, SUC_lexicon_tags_17 },
    { 0xf97a54e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfe3a54e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x399a54e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5da54ee, SUC_lexicon_tags_10 },
    { 0x82fa54ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fa54f2, SUC_lexicon_tags_17 },
    { 0x70da54f3, SUC_lexicon_tags_28 },
    { 0x147a54f2, SUC_lexicon_tags_11 },
    { 0x41da54f4, SUC_lexicon_tags_11 },
    { 0x6e7a54f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4da54f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8eba54fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731a54ff, SUC_lexicon_tags_12 },
    { 0x287a54ff, SUC_lexicon_tags_6 },
    { 0xd63a5501, SUC_lexicon_tags_1 },
    { 0x783a54ff, SUC_lexicon_tags_29 },
    { 0x6bda5502, SUC_lexicon_tags_19 },
    { 0xbd1a5501, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17a5507, SUC_lexicon_tags_17 },
    { 0xc5fa5508, SUC_lexicon_tags_38 },
    { 0x8d9a5509, SUC_lexicon_tags_25 },
    { 0xadda550a, SUC_lexicon_tags_14 },
    { 0x3da550b, SUC_lexicon_tags_254 },
    { 0xd67a550c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x585a550e, SUC_lexicon_tags_68 },
    { 0xdeba550f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fa5512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe83a5514, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x871a5517, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecda551a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x473a551c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31a551f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41da5523, SUC_lexicon_tags_15 },
    { 0xa97a5524, SUC_lexicon_tags_75 },
    { 0x303a5525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdba5527, SUC_lexicon_tags_138 },
    { 0x309a5527, SUC_lexicon_tags_37 },
    { 0xbbda5528, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fa5531, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe1ba5533, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513a5542, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3a554a, SUC_lexicon_tags_46 },
    { 0x6dfa554b, SUC_lexicon_tags_11 },
    { 0xacda554b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x873a554f, SUC_lexicon_tags_23 },
    { 0x835a5550, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd01a5552, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0da5555, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x339a5557, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc97a5559, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x303a555b, SUC_lexicon_tags_43 },
    { 0xd61a555c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51a5561, SUC_lexicon_tags_17 },
    { 0x34da5561, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ba5566, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77da556a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45a556e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x699a5570, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85a5577, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5a557b, SUC_lexicon_tags_11 },
    { 0xcb5a557c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x595a557e, SUC_lexicon_tags_2 },
    { 0xd07a557f, SUC_lexicon_tags_19 },
    { 0x4eba5580, SUC_lexicon_tags_2 },
    { 0x877a5580, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x937a5584, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf73a5586, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3a5589, SUC_lexicon_tags_77 },
    { 0xb83a5589, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77a558e, SUC_lexicon_tags_55 },
    { 0xc29a558f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9e7a5591, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3fa5593, SUC_lexicon_tags_45 },
    { 0x869a5594, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x37da5597, SUC_lexicon_tags_37 },
    { 0x63da5598, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1a559f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3a55a3, SUC_lexicon_tags_19 },
    { 0xe59a55a4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ba55ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ba55b0, SUC_lexicon_tags_2 },
    { 0xb9a55b0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc23a55b3, SUC_lexicon_tags_46 },
    { 0x44da55b4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd7a55b6, SUC_lexicon_tags_28 },
    { 0x109a55b7, SUC_lexicon_tags_14 },
    { 0xdbda55b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1eba55ba, SUC_lexicon_tags_37 },
    { 0x195a55bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf3a55bd, SUC_lexicon_tags_38 },
    { 0x4ada55be, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafa55c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da55c6, SUC_lexicon_tags_22 },
    { 0x44da55c7, SUC_lexicon_tags_15 },
    { 0xe2fa55c8, SUC_lexicon_tags_25 },
    { 0xfe5a55c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5a55cd, SUC_lexicon_tags_13 },
    { 0x9c1a55ce, SUC_lexicon_tags_224 },
    { 0xd27a55ce, SUC_lexicon_tags_25 },
    { 0xae9a55ce, SUC_lexicon_tags_11 },
    { 0x38da55cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7bda55d3, SUC_lexicon_tags_12 },
    { 0x5f3a55d4, SUC_lexicon_tags_12 },
    { 0x589a55d5, SUC_lexicon_tags_29 },
    { 0xe81a55d5, SUC_lexicon_tags_48 },
    { 0x3efa55d5, SUC_lexicon_tags_57 },
    { 0xa99a55d6, SUC_lexicon_tags_77 },
    { 0xf0da55d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32da55e0, SUC_lexicon_tags_12 },
    { 0xa01a55e0, SUC_lexicon_tags_12 },
    { 0x371a55e2, SUC_lexicon_tags_15 },
    { 0x75a55e3, SUC_lexicon_tags_8 },
    { 0x7e9a55e2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9a55ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x141a55ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ada55ee, SUC_lexicon_tags_26 },
    { 0xb19a55ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26da55f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5a55f6, SUC_lexicon_tags_16 },
    { 0x6ffa55f7, SUC_lexicon_tags_238 },
    { 0x137a55f6, SUC_lexicon_tags_11 },
    { 0x4a1a55f8, SUC_lexicon_tags_15 },
    { 0xb83a55fa, SUC_lexicon_tags_8 },
    { 0xb55a55f9, SUC_lexicon_tags_25 },
    { 0xcb9a55f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40da55fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec9a5600, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09a5603, SUC_lexicon_tags_25 },
    { 0x3ffa5603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357a5608, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1a560b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21a560e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x601a5611, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673a5615, SUC_lexicon_tags_37 },
    { 0x815a5616, SUC_lexicon_tags_47 },
    { 0x8c9a5617, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afa561a, SUC_lexicon_tags_12 },
    { 0xd1da561b, SUC_lexicon_tags_37 },
    { 0xc5fa561b, SUC_lexicon_tags_13 },
    { 0x35fa561d, SUC_lexicon_tags_63 },
    { 0x739a561e, SUC_lexicon_tags_151 },
    { 0xbdba561f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99a5624, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x547a5626, SUC_lexicon_tags_19 },
    { 0xa79a5627, SUC_lexicon_tags_15 },
    { 0x92da5627, SUC_lexicon_tags_11 },
    { 0xb1a5627, SUC_lexicon_tags_11 },
    { 0xc85a5628, SUC_lexicon_tags_65 },
    { 0x547a5629, SUC_lexicon_tags_17 },
    { 0xcf1a562c, SUC_lexicon_tags_26 },
    { 0x789a562d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49fa562f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629a5633, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75a5639, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579a5641, SUC_lexicon_tags_25 },
    { 0x85ba5641, SUC_lexicon_tags_26 },
    { 0xa95a5641, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9a5647, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdada564e, SUC_lexicon_tags_27 },
    { 0x331a564f, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63a5652, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3a5658, SUC_lexicon_tags_9 },
    { 0xf61a5659, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1a5662, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe4da5664, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x449a5666, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3da5668, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823a566d, SUC_lexicon_tags_25 },
    { 0x2bba566d, SUC_lexicon_tags_43 },
    { 0xb93a566f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ba5674, SUC_lexicon_tags_37 },
    { 0x979a5675, SUC_lexicon_tags_1 },
    { 0x751a5676, SUC_lexicon_tags_37 },
    { 0xea1a5674, SUC_lexicon_tags_45 },
    { 0xb33a5678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7fa567a, SUC_lexicon_tags_37 },
    { 0x4f5a567a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe19a567d, SUC_lexicon_tags_23 },
    { 0xf1a567d, SUC_lexicon_tags_37 },
    { 0xe07a567f, SUC_lexicon_tags_11 },
    { 0xbbda5680, SUC_lexicon_tags_37 },
    { 0x473a567e, SUC_lexicon_tags_37 },
    { 0x877a5682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3a5689, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33da5690, SUC_lexicon_tags_7 },
    { 0xfb9a5690, SUC_lexicon_tags_47 },
    { 0x621a5692, SUC_lexicon_tags_27 },
    { 0x7c1a5692, SUC_lexicon_tags_140 },
    { 0x35da5694, SUC_lexicon_tags_7 },
    { 0x3a7a5694, SUC_lexicon_tags_17 },
    { 0x2ba5696, SUC_lexicon_tags_25 },
    { 0xdd1a5697, SUC_lexicon_tags_37 },
    { 0x6fa5698, SUC_lexicon_tags_12 },
    { 0x845a5696, SUC_lexicon_tags_61 },
    { 0xdf1a5693, SUC_lexicon_tags_39 },
    { 0xa57a569b, SUC_lexicon_tags_146 },
    { 0x4eba569c, SUC_lexicon_tags_108 },
    { 0x871a569b, SUC_lexicon_tags_65 },
    { 0x971a5691, SUC_lexicon_tags_45 },
    { 0x1e5a5691, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x499a56a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8da56a6, SUC_lexicon_tags_31 },
    { 0x19da56a7, SUC_lexicon_tags_12 },
    { 0x90fa56a8, SUC_lexicon_tags_14 },
    { 0x68ba56a9, SUC_lexicon_tags_37 },
    { 0x2e9a56a8, SUC_lexicon_tags_6 },
    { 0x1cda56ab, SUC_lexicon_tags_149 },
    { 0xf3ba56ab, SUC_lexicon_tags_7 },
    { 0x99fa56ac, SUC_lexicon_tags_43 },
    { 0x8f7a56a6, SUC_lexicon_tags_33 },
    { 0x825a56af, SUC_lexicon_tags_23 },
    { 0x13ba56b0, SUC_lexicon_tags_27 },
    { 0xa31a56b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fa56ba, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfa56bd, SUC_lexicon_tags_382 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39da56c1, SUC_lexicon_tags_17 },
    { 0x585a56c1, SUC_lexicon_tags_11 },
    { 0x743a56c3, SUC_lexicon_tags_17 },
    { 0xf1da56c4, SUC_lexicon_tags_17 },
    { 0x8a9a56c1, SUC_lexicon_tags_8 },
    { 0x8e1a56c6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3a56ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x319a56cc, SUC_lexicon_tags_15 },
    { 0x177a56cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe81a56cf, SUC_lexicon_tags_7 },
    { 0xa0ba56d0, SUC_lexicon_tags_1 },
    { 0xeffa56d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123a56d6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xe81a56d8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ba56dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95a56e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741a56e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xeefa56e9, SUC_lexicon_tags_27 },
    { 0x4e3a56ea, SUC_lexicon_tags_63 },
    { 0xa79a56eb, SUC_lexicon_tags_7 },
    { 0x221a56ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7a56ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfba56f3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755a56f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x159a56fc, SUC_lexicon_tags_15 },
    { 0x119a56fd, SUC_lexicon_tags_1 },
    { 0xc09a56fe, SUC_lexicon_tags_15 },
    { 0xc19a56ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaba5704, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x199a5706, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9a5711, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x62ba5713, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aba5716, SUC_lexicon_tags_12 },
    { 0xc4ba5717, SUC_lexicon_tags_37 },
    { 0x549a5717, SUC_lexicon_tags_27 },
    { 0xd23a5719, SUC_lexicon_tags_12 },
    { 0xe81a571a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5a571f, SUC_lexicon_tags_12 },
    { 0xbdfa571f, SUC_lexicon_tags_15 },
    { 0xf95a5721, SUC_lexicon_tags_11 },
    { 0x3c7a5722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x727a5724, SUC_lexicon_tags_418 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25a5729, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607a572d, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a5733, SUC_lexicon_tags_11 },
    { 0x8dba5733, SUC_lexicon_tags_30 },
    { 0xc8fa5734, SUC_lexicon_tags_17 },
    { 0x4da5735, SUC_lexicon_tags_11 },
    { 0x2f1a5737, SUC_lexicon_tags_23 },
    { 0xb11a5736, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57a573d, SUC_lexicon_tags_47 },
    { 0x1bda573d, SUC_lexicon_tags_60 },
    { 0xd35a573f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb77a5741, SUC_lexicon_tags_10 },
    { 0xa31a5742, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9a5748, SUC_lexicon_tags_12 },
    { 0x5aba5749, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2fa574b, SUC_lexicon_tags_37 },
    { 0xafa574b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d3a574e, SUC_lexicon_tags_33 },
    { 0xaf7a574f, SUC_lexicon_tags_2 },
    { 0x90fa5750, SUC_lexicon_tags_30 },
    { 0xdb7a5751, SUC_lexicon_tags_25 },
    { 0xf1a5752, SUC_lexicon_tags_30 },
    { 0xc35a5753, SUC_lexicon_tags_37 },
    { 0xda9a5754, SUC_lexicon_tags_37 },
    { 0xc1da5750, SUC_lexicon_tags_60 },
    { 0x9eba5756, SUC_lexicon_tags_70 },
    { 0x687a574f, SUC_lexicon_tags_15 },
    { 0x49fa5758, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fa575b, SUC_lexicon_tags_16 },
    { 0xd3fa575c, SUC_lexicon_tags_12 },
    { 0xd6ba575d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1a5760, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91a5762, SUC_lexicon_tags_87 },
    { 0xeeba5762, SUC_lexicon_tags_19 },
    { 0x733a5763, SUC_lexicon_tags_31 },
    { 0x86fa5762, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x575a5767, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa29a5769, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dba576c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57a5773, SUC_lexicon_tags_10 },
    { 0xae3a5774, SUC_lexicon_tags_33 },
    { 0xb7a5775, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x273a5777, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd89a5779, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813a577d, SUC_lexicon_tags_10 },
    { 0x3e3a577e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203a5783, SUC_lexicon_tags_27 },
    { 0x1a5a5783, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1fa5786, SUC_lexicon_tags_19 },
    { 0x91fa5787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bda578a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x49a578c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5fa578e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269a5793, SUC_lexicon_tags_1 },
    { 0xb6fa5794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fa579a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcada579c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a579f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fa57a4, SUC_lexicon_tags_68 },
    { 0xb21a57a4, SUC_lexicon_tags_27 },
    { 0xab9a57a6, SUC_lexicon_tags_11 },
    { 0x791a57a6, SUC_lexicon_tags_66 },
    { 0xa57a57a5, SUC_lexicon_tags_25 },
    { 0x2e7a57a4, SUC_lexicon_tags_39 },
    { 0xc33a57aa, SUC_lexicon_tags_25 },
    { 0x9d3a57ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x461a57ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfa57af, SUC_lexicon_tags_17 },
    { 0xbe3a57b0, SUC_lexicon_tags_21 },
    { 0x903a57b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413a57b9, SUC_lexicon_tags_79 },
    { 0x909a57ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebda57bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a3a57c1, SUC_lexicon_tags_15 },
    { 0x9d1a57c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3bba57c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ba57c8, SUC_lexicon_tags_12 },
    { 0xe95a57c9, SUC_lexicon_tags_60 },
    { 0xf1a57c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997a57ce, SUC_lexicon_tags_15 },
    { 0x417a57cf, SUC_lexicon_tags_27 },
    { 0xda3a57ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fa57d4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xcc7a57d6, SUC_lexicon_tags_27 },
    { 0xbd1a57d6, SUC_lexicon_tags_87 },
    { 0x4d1a57d8, SUC_lexicon_tags_49 },
    { 0x1cfa57d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973a57e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6ba57e7, SUC_lexicon_tags_17 },
    { 0xa7da57e8, SUC_lexicon_tags_11 },
    { 0x3d9a57e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09a57ed, SUC_lexicon_tags_37 },
    { 0xc6da57ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6fa57f0, SUC_lexicon_tags_1 },
    { 0x2a3a57f1, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0x407a57f3, SUC_lexicon_tags_30 },
    { 0x1d1a57f4, SUC_lexicon_tags_23 },
    { 0x617a57f4, SUC_lexicon_tags_2 },
    { 0x73ba57f5, SUC_lexicon_tags_15 },
    { 0xb85a57f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x519a57f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x411a57fb, SUC_lexicon_tags_17 },
    { 0x999a57fb, SUC_lexicon_tags_0 },
    { 0x24fa57fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9a5804, SUC_lexicon_tags_43 },
    { 0x27ba5805, SUC_lexicon_tags_37 },
    { 0x5aba5804, SUC_lexicon_tags_11 },
    { 0xfcfa5807, SUC_lexicon_tags_1 },
    { 0xb75a5808, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x141a580a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa5811, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3a5815, SUC_lexicon_tags_17 },
    { 0xa09a5816, SUC_lexicon_tags_31 },
    { 0xaada5815, SUC_lexicon_tags_12 },
    { 0xb25a5815, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe3a581a, SUC_lexicon_tags_7 },
    { 0xb31a581b, SUC_lexicon_tags_45 },
    { 0xe7fa581b, SUC_lexicon_tags_21 },
    { 0xa73a581d, SUC_lexicon_tags_0 },
    { 0x80da581e, SUC_lexicon_tags_12 },
    { 0x5fa581e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe4ba5821, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x163a5823, SUC_lexicon_tags_28 },
    { 0xd7a5823, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x279a5826, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48ba5828, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x719a582a, SUC_lexicon_tags_1638 },
    { 0x25da582b, SUC_lexicon_tags_3 },
    { 0x55da582c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x335a582e, SUC_lexicon_tags_1 },
    { 0xc25a582e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x731a5831, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd9fa5833, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63da583b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92fa583d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd53a583f, SUC_lexicon_tags_11 },
    { 0x375a5840, SUC_lexicon_tags_14 },
    { 0xc89a5840, SUC_lexicon_tags_0 },
    { 0x59ba5840, SUC_lexicon_tags_27 },
    { 0xe9da5840, SUC_lexicon_tags_37 },
    { 0xcf7a5844, SUC_lexicon_tags_28 },
    { 0xd31a5842, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfa584e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc5a5850, SUC_lexicon_tags_10 },
    { 0x197a5851, SUC_lexicon_tags_17 },
    { 0x77ba5850, SUC_lexicon_tags_0 },
    { 0x991a5852, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfa5859, SUC_lexicon_tags_39 },
    { 0xd05a585a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x699a585c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49a5860, SUC_lexicon_tags_2 },
    { 0xcd5a5860, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83a5866, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcd7a5868, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6d1a586a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfba5870, SUC_lexicon_tags_25 },
    { 0xdba5871, SUC_lexicon_tags_12 },
    { 0xfc7a5872, SUC_lexicon_tags_12 },
    { 0xb67a5872, SUC_lexicon_tags_33 },
    { 0x123a5874, SUC_lexicon_tags_17 },
    { 0x1b5a5875, SUC_lexicon_tags_21 },
    { 0xf9a5876, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73a5879, SUC_lexicon_tags_29 },
    { 0x307a587a, SUC_lexicon_tags_25 },
    { 0x211a587b, SUC_lexicon_tags_28 },
    { 0x39a587c, SUC_lexicon_tags_17 },
    { 0x7a5a5879, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x41a5880, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f7a5882, SUC_lexicon_tags_37 },
    { 0x5f5a5883, SUC_lexicon_tags_38 },
    { 0x849a5884, SUC_lexicon_tags_45 },
    { 0xc5fa5883, SUC_lexicon_tags_23 },
    { 0x9fa5884, SUC_lexicon_tags_14 },
    { 0x313a5886, SUC_lexicon_tags_7 },
    { 0x77da5884, SUC_lexicon_tags_3 },
    { 0x467a5886, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x71da588c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bba5890, SUC_lexicon_tags_37 },
    { 0xc21a5891, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f5a5893, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fa5896, SUC_lexicon_tags_49 },
    { 0x913a5897, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6c5a5899, SUC_lexicon_tags_26 },
    { 0x1dfa5899, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fa58a1, SUC_lexicon_tags_35 },
    { 0x59ba58a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fa58a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb51a58a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3a58ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1a58b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fa58b3, SUC_lexicon_tags_14 },
    { 0x227a58b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ada58b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x925a58b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2ba58bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaba58be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b9a58c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x36ba58c3, SUC_lexicon_tags_11 },
    { 0x11ba58c4, SUC_lexicon_tags_39 },
    { 0x803a58c4, SUC_lexicon_tags_47 },
    { 0x42da58c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541a58d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x437a58da, SUC_lexicon_tags_27 },
    { 0xb75a58db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387a58e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcba58e6, SUC_lexicon_tags_15 },
    { 0xbf7a58e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcda58ef, SUC_lexicon_tags_34 },
    { 0xfba58f0, SUC_lexicon_tags_3 },
    { 0x19ba58f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863a58f6, SUC_lexicon_tags_60 },
    { 0xdcda58f6, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0xc59a58f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c7a58fb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79a58fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79a5903, SUC_lexicon_tags_37 },
    { 0xf25a5903, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefda590a, SUC_lexicon_tags_37 },
    { 0xfe9a590b, SUC_lexicon_tags_14 },
    { 0xdf3a590b, SUC_lexicon_tags_47 },
    { 0xb1da590c, SUC_lexicon_tags_10 },
    { 0xb03a590e, SUC_lexicon_tags_17 },
    { 0xb0ba590e, SUC_lexicon_tags_7 },
    { 0x8e3a5910, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb0ba5912, SUC_lexicon_tags_35 },
    { 0x1d1a5913, SUC_lexicon_tags_13 },
    { 0x55fa5912, SUC_lexicon_tags_37 },
    { 0xff9a5915, SUC_lexicon_tags_25 },
    { 0xb3fa5916, SUC_lexicon_tags_37 },
    { 0x901a5917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9a591c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcfba591e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fa5924, SUC_lexicon_tags_13 },
    { 0x67a5925, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa09a5927, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x969a592a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49a592f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeda5933, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9a5939, SUC_lexicon_tags_7 },
    { 0x46fa593a, SUC_lexicon_tags_37 },
    { 0x829a593b, SUC_lexicon_tags_11 },
    { 0xe55a593b, SUC_lexicon_tags_9 },
    { 0x249a593d, SUC_lexicon_tags_60 },
    { 0xdc5a5939, SUC_lexicon_tags_32 },
    { 0x2d9a593f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cba5944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2fa5946, SUC_lexicon_tags_17 },
    { 0x847a5947, SUC_lexicon_tags_32 },
    { 0x20da5948, SUC_lexicon_tags_7 },
    { 0xfefa5949, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacba594d, SUC_lexicon_tags_47 },
    { 0x397a594e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x65a5950, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfa5955, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x679a5957, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ba595b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5a595e, SUC_lexicon_tags_15 },
    { 0xe9fa595f, SUC_lexicon_tags_11 },
    { 0x83a595e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85a5963, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1a5966, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa43a5968, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1c5a596a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d3a596c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ba5971, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97fa5976, SUC_lexicon_tags_38 },
    { 0xaa7a5977, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a1a5979, SUC_lexicon_tags_19 },
    { 0xb9da597a, SUC_lexicon_tags_12 },
    { 0xf95a597b, SUC_lexicon_tags_47 },
    { 0x23a597b, SUC_lexicon_tags_60 },
    { 0xc17a597b, SUC_lexicon_tags_25 },
    { 0x3e7a597e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7da5982, SUC_lexicon_tags_1 },
    { 0x72ba5983, SUC_lexicon_tags_39 },
    { 0x7e9a5984, SUC_lexicon_tags_38 },
    { 0xafda5985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971a5989, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67a5990, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54ba5992, SUC_lexicon_tags_60 },
    { 0x88da5993, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9a5996, SUC_lexicon_tags_12 },
    { 0x801a5997, SUC_lexicon_tags_25 },
    { 0x103a5998, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fa599b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18fa599e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x709a59a0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95a59a9, SUC_lexicon_tags_14 },
    { 0x54fa59aa, SUC_lexicon_tags_151 },
    { 0x179a59a9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7a59b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdcda59b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffa59b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09a59b9, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xa59a59bb, SUC_lexicon_tags_25 },
    { 0xab3a59bc, SUC_lexicon_tags_0 },
    { 0xd71a59bc, SUC_lexicon_tags_38 },
    { 0x4dba59bc, SUC_lexicon_tags_27 },
    { 0xbd9a59bf, SUC_lexicon_tags_35 },
    { 0x4e9a59bf, SUC_lexicon_tags_8 },
    { 0xe81a59c1, SUC_lexicon_tags_2 },
    { 0xd99a59c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x711a59c4, SUC_lexicon_tags_11 },
    { 0x589a59c4, SUC_lexicon_tags_25 },
    { 0x5e7a59c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x479a59c9, SUC_lexicon_tags_26 },
    { 0x42da59ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73ba59cc, SUC_lexicon_tags_25 },
    { 0xb85a59cc, SUC_lexicon_tags_17 },
    { 0xd13a59ce, SUC_lexicon_tags_1 },
    { 0x74fa59cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x699a59d2, SUC_lexicon_tags_12 },
    { 0xbd1a59d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805a59d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1a59dc, SUC_lexicon_tags_69 },
    { 0xb13a59dd, SUC_lexicon_tags_11 },
    { 0xfafa59dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ba59e3, SUC_lexicon_tags_17 },
    { 0xef3a59e4, SUC_lexicon_tags_1 },
    { 0x733a59e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x74fa59e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7da59ed, SUC_lexicon_tags_6 },
    { 0x469a59ed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5a59f2, SUC_lexicon_tags_6 },
    { 0xcafa59f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xccda59f6, SUC_lexicon_tags_43 },
    { 0xe67a59f7, SUC_lexicon_tags_65 },
    { 0x971a59f8, SUC_lexicon_tags_2 },
    { 0x6cba59f8, SUC_lexicon_tags_21 },
    { 0xe9fa59fa, SUC_lexicon_tags_7 },
    { 0x7f9a59f7, SUC_lexicon_tags_2 },
    { 0xd39a59fc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01a5a06, SUC_lexicon_tags_17 },
    { 0x9f3a5a07, SUC_lexicon_tags_1 },
    { 0x76ba5a08, SUC_lexicon_tags_11 },
    { 0x1afa5a09, SUC_lexicon_tags_37 },
    { 0x5dda5a08, SUC_lexicon_tags_6 },
    { 0x789a5a0b, SUC_lexicon_tags_15 },
    { 0x349a5a0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a5a12, SUC_lexicon_tags_37 },
    { 0x767a5a13, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a5a20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x327a5a22, SUC_lexicon_tags_21 },
    { 0xe49a5a22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfa5a26, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5a5a29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39a5a30, SUC_lexicon_tags_12 },
    { 0x97da5a31, SUC_lexicon_tags_7 },
    { 0xd39a5a30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491a5a37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x153a5a3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x261a5a3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83fa5a3f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591a5a4b, SUC_lexicon_tags_37 },
    { 0xba5a4b, SUC_lexicon_tags_65 },
    { 0x723a5a4d, SUC_lexicon_tags_6 },
    { 0xf77a5a4e, SUC_lexicon_tags_31 },
    { 0x23ba5a4e, SUC_lexicon_tags_25 },
    { 0x94da5a4f, SUC_lexicon_tags_25 },
    { 0xf11a5a51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x453a5a53, SUC_lexicon_tags_9 },
    { 0xfcba5a54, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5a5a59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad3a5a5b, SUC_lexicon_tags_39 },
    { 0x2a1a5a5c, SUC_lexicon_tags_46 },
    { 0x37da5a5d, SUC_lexicon_tags_27 },
    { 0xcfba5a5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12da5a62, SUC_lexicon_tags_30 },
    { 0x5aba5a63, SUC_lexicon_tags_11 },
    { 0xb33a5a64, SUC_lexicon_tags_9 },
    { 0x169a5a62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57a5a68, SUC_lexicon_tags_11 },
    { 0x915a5a69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127a5a6d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x153a5a70, SUC_lexicon_tags_12 },
    { 0x9e9a5a71, SUC_lexicon_tags_77 },
    { 0xeb5a5a70, SUC_lexicon_tags_19 },
    { 0xde3a5a73, SUC_lexicon_tags_28 },
    { 0xb61a5a74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x739a5a77, SUC_lexicon_tags_2 },
    { 0x293a5a78, SUC_lexicon_tags_4 },
    { 0x775a5a79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113a5a81, SUC_lexicon_tags_31 },
    { 0x721a5a82, SUC_lexicon_tags_31 },
    { 0xbe5a5a83, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67a5a86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda5a89, SUC_lexicon_tags_12 },
    { 0x41ba5a8a, SUC_lexicon_tags_37 },
    { 0x33a5a8b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29a5a8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbeba5a90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba5a92, SUC_lexicon_tags_31 },
    { 0xb55a5a93, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x29a5a95, SUC_lexicon_tags_32 },
    { 0x76da5a96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffa5a99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1a5aa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ba5aa7, SUC_lexicon_tags_66 },
    { 0xe77a5aa8, SUC_lexicon_tags_25 },
    { 0xe75a5aa9, SUC_lexicon_tags_37 },
    { 0xb09a5aa9, SUC_lexicon_tags_14 },
    { 0xc6fa5aa9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ba5ab0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3a5ab7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96fa5ab9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x93ba5abb, SUC_lexicon_tags_25 },
    { 0x71ba5abc, SUC_lexicon_tags_66 },
    { 0xce1a5abc, SUC_lexicon_tags_2 },
    { 0x8efa5abe, SUC_lexicon_tags_51 },
    { 0x21ba5abf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45ba5ac1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48fa5ac6, SUC_lexicon_tags_49 },
    { 0x7c7a5ac6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f5a5ac9, SUC_lexicon_tags_14 },
    { 0xe97a5aca, SUC_lexicon_tags_17 },
    { 0xd1fa5aca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5acf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3fa5ad1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66da5ad5, SUC_lexicon_tags_30 },
    { 0x79da5ad6, SUC_lexicon_tags_808 },
    { 0xf05a5ad7, SUC_lexicon_tags_12 },
    { 0x1efa5ad7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3a5adb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5a5ae0, SUC_lexicon_tags_43 },
    { 0x2afa5ae0, SUC_lexicon_tags_17 },
    { 0x5a5ae0, SUC_lexicon_tags_7 },
    { 0xb7da5ae3, SUC_lexicon_tags_27 },
    { 0xd31a5ae4, SUC_lexicon_tags_25 },
    { 0xa47a5ae5, SUC_lexicon_tags_49 },
    { 0x2f1a5ae5, SUC_lexicon_tags_17 },
    { 0x2ffa5ae6, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13a5aeb, SUC_lexicon_tags_25 },
    { 0x95ba5aeb, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85a5af3, SUC_lexicon_tags_5 },
    { 0x7fba5af3, SUC_lexicon_tags_11 },
    { 0xb17a5af3, SUC_lexicon_tags_63 },
    { 0x99a5af6, SUC_lexicon_tags_68 },
    { 0xe2fa5af7, SUC_lexicon_tags_916 },
    { 0xe57a5af7, SUC_lexicon_tags_55 },
    { 0x271a5af9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23a5b00, SUC_lexicon_tags_48 },
    { 0x305a5b01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23da5b03, SUC_lexicon_tags_25 },
    { 0x3efa5b03, SUC_lexicon_tags_11 },
    { 0x397a5b05, SUC_lexicon_tags_48 },
    { 0xf0fa5b06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c3a5b08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ba5b0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x73da5b10, SUC_lexicon_tags_47 },
    { 0xc2fa5b11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x427a5b13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cda5b1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b1a5b1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5da5b1e, SUC_lexicon_tags_17 },
    { 0xbf7a5b1e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5a5b22, SUC_lexicon_tags_27 },
    { 0x975a5b22, SUC_lexicon_tags_12 },
    { 0x7a3a5b24, SUC_lexicon_tags_30 },
    { 0xb6da5b25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6da5b28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf93a5b2a, SUC_lexicon_tags_14 },
    { 0x321a5b2b, SUC_lexicon_tags_2 },
    { 0xe2da5b2b, SUC_lexicon_tags_25 },
    { 0x523a5b2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ba5b33, SUC_lexicon_tags_27 },
    { 0x201a5b33, SUC_lexicon_tags_3 },
    { 0xa7ba5b35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ba5b3d, SUC_lexicon_tags_7 },
    { 0xdc7a5b3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc53a5b40, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xd91a5b42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fa5b47, SUC_lexicon_tags_1 },
    { 0x7c5a5b47, SUC_lexicon_tags_12 },
    { 0xb6fa5b49, SUC_lexicon_tags_25 },
    { 0xd2da5b4a, SUC_lexicon_tags_480 },
    { 0xaafa5b48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cfa5b4d, SUC_lexicon_tags_27 },
    { 0x30ba5b4d, SUC_lexicon_tags_32 },
    { 0xab5a5b4d, SUC_lexicon_tags_74 },
    { 0x217a5b50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa5b57, SUC_lexicon_tags_33 },
    { 0xabda5b57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ba5b5b, SUC_lexicon_tags_15 },
    { 0xafba5b5c, SUC_lexicon_tags_49 },
    { 0x9eda5b5d, SUC_lexicon_tags_27 },
    { 0x2b9a5b5e, SUC_lexicon_tags_1 },
    { 0xb11a5b5f, SUC_lexicon_tags_28 },
    { 0x2fba5b5d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479a5b64, SUC_lexicon_tags_1 },
    { 0xf45a5b64, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fa5b69, SUC_lexicon_tags_1 },
    { 0xeada5b69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x83fa5b6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99a5b72, SUC_lexicon_tags_64 },
    { 0xd91a5b73, SUC_lexicon_tags_9 },
    { 0x147a5b74, SUC_lexicon_tags_43 },
    { 0x485a5b74, SUC_lexicon_tags_37 },
    { 0x4dfa5b76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91a5b7c, SUC_lexicon_tags_17 },
    { 0x1f7a5b7d, SUC_lexicon_tags_17 },
    { 0x75ba5b7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757a5b85, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101a5b89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x273a5b8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a5b8d, SUC_lexicon_tags_1 },
    { 0x943a5b8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd29a5b90, SUC_lexicon_tags_14 },
    { 0x855a5b90, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x77fa5b93, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x467a5b95, SUC_lexicon_tags_1 },
    { 0x73da5b96, SUC_lexicon_tags_31 },
    { 0x3ada5b97, SUC_lexicon_tags_4 },
    { 0x509a5b97, SUC_lexicon_tags_12 },
    { 0x179a5b99, SUC_lexicon_tags_37 },
    { 0xf1ba5b9a, SUC_lexicon_tags_43 },
    { 0x97ba5b9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fa5b9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1da5ba6, SUC_lexicon_tags_12 },
    { 0xef3a5ba7, SUC_lexicon_tags_1 },
    { 0xdf3a5ba8, SUC_lexicon_tags_33 },
    { 0xeb7a5ba8, SUC_lexicon_tags_23 },
    { 0x1bba5baa, SUC_lexicon_tags_1 },
    { 0x27a5baa, SUC_lexicon_tags_2 },
    { 0x6b7a5bac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bda5bae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3a5bb4, SUC_lexicon_tags_30 },
    { 0xac3a5bb4, SUC_lexicon_tags_2 },
    { 0xd13a5bb6, SUC_lexicon_tags_7 },
    { 0x6a1a5bb7, SUC_lexicon_tags_28 },
    { 0xa6fa5bb7, SUC_lexicon_tags_22 },
    { 0xb09a5bb9, SUC_lexicon_tags_1 },
    { 0xed3a5bb8, SUC_lexicon_tags_27 },
    { 0xbf7a5bb7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x531a5bbd, SUC_lexicon_tags_29 },
    { 0xaada5bbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfa5bc1, SUC_lexicon_tags_7 },
    { 0xd21a5bc2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445a5bc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35da5bc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc67a5bcb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4da5bd1, SUC_lexicon_tags_12 },
    { 0xedda5bd2, SUC_lexicon_tags_11 },
    { 0xcc1a5bd2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb09a5bd5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x787a5bd7, SUC_lexicon_tags_79 },
    { 0x213a5bd8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3da5bde, SUC_lexicon_tags_11 },
    { 0xeb9a5bdf, SUC_lexicon_tags_1 },
    { 0x9dfa5be0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x53fa5be2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ba5be5, SUC_lexicon_tags_38 },
    { 0x9fba5be6, SUC_lexicon_tags_65 },
    { 0x5d5a5be7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5a5bed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ba5bf2, SUC_lexicon_tags_31 },
    { 0xa77a5bf2, SUC_lexicon_tags_11 },
    { 0x67da5bf4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x605a5bf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9a5bff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3a5c05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467a5c0a, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x2f1a5c0c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3a5c11, SUC_lexicon_tags_15 },
    { 0xe21a5c12, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xddda5c14, SUC_lexicon_tags_29 },
    { 0x665a5c15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9a5c19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99a5c1c, SUC_lexicon_tags_1 },
    { 0x63ba5c1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ba5c21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7a5c27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88fa5c29, SUC_lexicon_tags_17 },
    { 0xa6da5c2a, SUC_lexicon_tags_17 },
    { 0x24ba5c2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7a5c33, SUC_lexicon_tags_26 },
    { 0x9a3a5c34, SUC_lexicon_tags_11 },
    { 0x9bda5c34, SUC_lexicon_tags_7 },
    { 0x2e9a5c35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197a5c3b, SUC_lexicon_tags_6 },
    { 0x60fa5c3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ba5c45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdbda5c47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8b1a5c49, SUC_lexicon_tags_25 },
    { 0x68ba5c4a, SUC_lexicon_tags_33 },
    { 0xd53a5c4b, SUC_lexicon_tags_10 },
    { 0xc3ba5c4c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeba5c51, SUC_lexicon_tags_7 },
    { 0xa3a5c52, SUC_lexicon_tags_25 },
    { 0x837a5c52, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfa5c59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dba5c68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7a5c6b, SUC_lexicon_tags_47 },
    { 0x5afa5c6b, SUC_lexicon_tags_17 },
    { 0x24fa5c6d, SUC_lexicon_tags_11 },
    { 0xbb3a5c6e, SUC_lexicon_tags_12 },
    { 0x6c1a5c6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5a5c74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58ba5c78, SUC_lexicon_tags_2 },
    { 0xd07a5c79, SUC_lexicon_tags_54 },
    { 0xbba5c7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fda5c7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecba5c81, SUC_lexicon_tags_30 },
    { 0xcada5c82, SUC_lexicon_tags_21 },
    { 0x617a5c83, SUC_lexicon_tags_17 },
    { 0x227a5c83, SUC_lexicon_tags_25 },
    { 0xd45a5c84, SUC_lexicon_tags_60 },
    { 0xf0fa5c83, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07a5c8e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9a5c92, SUC_lexicon_tags_6 },
    { 0x753a5c93, SUC_lexicon_tags_23 },
    { 0x9afa5c93, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x999a5c97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6bfa5c99, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefa5c9c, SUC_lexicon_tags_0 },
    { 0xa3a5c9d, SUC_lexicon_tags_19 },
    { 0x789a5c9e, SUC_lexicon_tags_27 },
    { 0x1bba5c9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5a5ca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ba5ca9, SUC_lexicon_tags_37 },
    { 0x41fa5caa, SUC_lexicon_tags_1127 },
    { 0xc4fa5ca9, SUC_lexicon_tags_15 },
    { 0x86da5cac, SUC_lexicon_tags_17 },
    { 0x5efa5cad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75a5cb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x253a5cb3, SUC_lexicon_tags_11 },
    { 0xf91a5cb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ba5cb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5a5cc5, SUC_lexicon_tags_37 },
    { 0xd99a5cc5, SUC_lexicon_tags_1 },
    { 0xa87a5cc7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5da5ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5da5cd7, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19a5cda, SUC_lexicon_tags_2 },
    { 0x24fa5cdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a5ce0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9a5ce4, SUC_lexicon_tags_3 },
    { 0xed9a5ce4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc41a5ce7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x753a5ce9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x427a5ceb, SUC_lexicon_tags_14 },
    { 0xc97a5cec, SUC_lexicon_tags_47 },
    { 0x773a5cec, SUC_lexicon_tags_17 },
    { 0x361a5ceb, SUC_lexicon_tags_79 },
    { 0x3c1a5ceb, SUC_lexicon_tags_1 },
    { 0xec1a5ceb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x11fa5cf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949a5cf6, SUC_lexicon_tags_8 },
    { 0x247a5cf6, SUC_lexicon_tags_11 },
    { 0x293a5cf6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe29a5cfa, SUC_lexicon_tags_12 },
    { 0xd0da5cfb, SUC_lexicon_tags_47 },
    { 0x4e1a5cfc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd7da5cfe, SUC_lexicon_tags_27 },
    { 0xa5da5cff, SUC_lexicon_tags_33 },
    { 0xae1a5d00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75fa5d02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x343a5d05, SUC_lexicon_tags_48 },
    { 0xf4fa5d06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa95a5d08, SUC_lexicon_tags_17 },
    { 0xb27a5d09, SUC_lexicon_tags_14 },
    { 0xa77a5d0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa45a5d0c, SUC_lexicon_tags_31 },
    { 0xe71a5d0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf25a5d0f, SUC_lexicon_tags_0 },
    { 0xe5da5d0f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1da5d12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x913a5d15, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x233a5d17, SUC_lexicon_tags_158 },
    { 0xc8fa5d18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5a5d1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fa5d20, SUC_lexicon_tags_40 },
    { 0xc07a5d21, SUC_lexicon_tags_17 },
    { 0xcc9a5d22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ba5d25, SUC_lexicon_tags_7 },
    { 0xf57a5d26, SUC_lexicon_tags_25 },
    { 0xf0ba5d27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ba5d2f, SUC_lexicon_tags_38 },
    { 0x671a5d30, SUC_lexicon_tags_21 },
    { 0x4bfa5d2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x17a5d33, SUC_lexicon_tags_37 },
    { 0x86fa5d34, SUC_lexicon_tags_6 },
    { 0xec5a5d33, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71a5d3a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eda5d3f, SUC_lexicon_tags_31 },
    { 0xa41a5d40, SUC_lexicon_tags_38 },
    { 0xa09a5d41, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec3a5d43, SUC_lexicon_tags_25 },
    { 0xfe3a5d43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91a5d49, SUC_lexicon_tags_25 },
    { 0x463a5d4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7a5d4f, SUC_lexicon_tags_46 },
    { 0x749a5d50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x81fa5d53, SUC_lexicon_tags_29 },
    { 0xa87a5d54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfa5d58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x947a5d5b, SUC_lexicon_tags_49 },
    { 0x4ada5d5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x309a5d5e, SUC_lexicon_tags_14 },
    { 0x94fa5d5f, SUC_lexicon_tags_17 },
    { 0xb5fa5d5e, SUC_lexicon_tags_15 },
    { 0xe87a5d61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01a5d68, SUC_lexicon_tags_15 },
    { 0x75a5d69, SUC_lexicon_tags_43 },
    { 0x34ba5d69, SUC_lexicon_tags_6 },
    { 0x295a5d6b, SUC_lexicon_tags_11 },
    { 0x269a5d6a, SUC_lexicon_tags_25 },
    { 0xeba5d6d, SUC_lexicon_tags_47 },
    { 0xa89a5d6d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x751a5d71, SUC_lexicon_tags_84 },
    { 0x277a5d72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee3a5d74, SUC_lexicon_tags_6 },
    { 0x581a5d74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9a5d7a, SUC_lexicon_tags_15 },
    { 0xf39a5d7b, SUC_lexicon_tags_3 },
    { 0xaa9a5d7a, SUC_lexicon_tags_43 },
    { 0x969a5d7d, SUC_lexicon_tags_25 },
    { 0xdbda5d7d, SUC_lexicon_tags_46 },
    { 0xa01a5d7f, SUC_lexicon_tags_15 },
    { 0x23a5d7f, SUC_lexicon_tags_15 },
    { 0x915a5d7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x431a5d83, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ba5d86, SUC_lexicon_tags_17 },
    { 0x813a5d87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13a5d8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aba5d8f, SUC_lexicon_tags_31 },
    { 0x55ba5d90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe37a5d92, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9a5d97, SUC_lexicon_tags_29 },
    { 0x809a5d98, SUC_lexicon_tags_1 },
    { 0xb53a5d97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07a5d9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fa5d9f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x849a5da2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9a5da6, SUC_lexicon_tags_15 },
    { 0x693a5da7, SUC_lexicon_tags_15 },
    { 0x863a5da8, SUC_lexicon_tags_45 },
    { 0x82da5da6, SUC_lexicon_tags_29 },
    { 0x5d1a5da8, SUC_lexicon_tags_37 },
    { 0x125a5dab, SUC_lexicon_tags_12 },
    { 0x905a5dab, SUC_lexicon_tags_43 },
    { 0x981a5dab, SUC_lexicon_tags_1 },
    { 0xff3a5dae, SUC_lexicon_tags_16 },
    { 0xee3a5daf, SUC_lexicon_tags_3 },
    { 0x82ba5db0, SUC_lexicon_tags_47 },
    { 0xaa3a5db1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7a5db5, SUC_lexicon_tags_23 },
    { 0x39a5db6, SUC_lexicon_tags_28 },
    { 0xcb5a5db7, SUC_lexicon_tags_670 },
    { 0x895a5db8, SUC_lexicon_tags_25 },
    { 0x42fa5db8, SUC_lexicon_tags_37 },
    { 0xa55a5db9, SUC_lexicon_tags_11 },
    { 0xcf7a5dbb, SUC_lexicon_tags_2 },
    { 0x9f7a5dbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffa5dbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627a5dc3, SUC_lexicon_tags_23 },
    { 0x247a5dc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5a5dcc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53a5dcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13fa5dd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd2da5dd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75a5dd9, SUC_lexicon_tags_57 },
    { 0x6d5a5dda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fa5ddd, SUC_lexicon_tags_37 },
    { 0x767a5dde, SUC_lexicon_tags_12 },
    { 0x27fa5ddd, SUC_lexicon_tags_28 },
    { 0x71ba5de0, SUC_lexicon_tags_12 },
    { 0x4efa5de1, SUC_lexicon_tags_64 },
    { 0x6e9a5de1, SUC_lexicon_tags_6 },
    { 0x397a5de3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ba5dec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefa5df9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf97a5dfb, SUC_lexicon_tags_45 },
    { 0x9bda5dfb, SUC_lexicon_tags_15 },
    { 0x5a1a5dfd, SUC_lexicon_tags_15 },
    { 0x1b7a5dfe, SUC_lexicon_tags_25 },
    { 0x1cda5dfe, SUC_lexicon_tags_0 },
    { 0x35fa5dff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fa5e04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fa5e08, SUC_lexicon_tags_46 },
    { 0x131a5e08, SUC_lexicon_tags_29 },
    { 0xd3fa5e0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f5a5e0c, SUC_lexicon_tags_60 },
    { 0xba7a5e0d, SUC_lexicon_tags_0 },
    { 0xa29a5e0e, SUC_lexicon_tags_12 },
    { 0xa6ba5e0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5a5e14, SUC_lexicon_tags_12 },
    { 0xed7a5e14, SUC_lexicon_tags_37 },
    { 0xcffa5e14, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83a5e19, SUC_lexicon_tags_16 },
    { 0x6a5a5e19, SUC_lexicon_tags_47 },
    { 0xa3da5e1b, SUC_lexicon_tags_43 },
    { 0x5b9a5e1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x855a5e1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x143a5e21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda1a5e23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ba5e27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ba5e2f, SUC_lexicon_tags_396 },
    { 0x4e3a5e30, SUC_lexicon_tags_4 },
    { 0xc79a5e31, SUC_lexicon_tags_37 },
    { 0x943a5e32, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463a5e38, SUC_lexicon_tags_11 },
    { 0x4e5a5e39, SUC_lexicon_tags_11 },
    { 0xfbba5e3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfa5e3e, SUC_lexicon_tags_2 },
    { 0x7a1a5e3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ba5e42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fa5e45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ba5e4a, SUC_lexicon_tags_11 },
    { 0xc43a5e4b, SUC_lexicon_tags_37 },
    { 0xd17a5e4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffa5e4f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5a5e55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327a5e5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fa5e63, SUC_lexicon_tags_0 },
    { 0x14fa5e64, SUC_lexicon_tags_0 },
    { 0x121a5e65, SUC_lexicon_tags_12 },
    { 0x33fa5e66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35da5e6d, SUC_lexicon_tags_37 },
    { 0xa1fa5e6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ba5e71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545a5e79, SUC_lexicon_tags_7 },
    { 0x1afa5e7a, SUC_lexicon_tags_44 },
    { 0xa85a5e7b, SUC_lexicon_tags_37 },
    { 0xe0fa5e7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639a5e80, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd9a5e82, SUC_lexicon_tags_31 },
    { 0xb5da5e83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7a5e87, SUC_lexicon_tags_11 },
    { 0x4a7a5e88, SUC_lexicon_tags_25 },
    { 0xeb1a5e89, SUC_lexicon_tags_25 },
    { 0x489a5e8a, SUC_lexicon_tags_257 },
    { 0x357a5e8a, SUC_lexicon_tags_23 },
    { 0xa1fa5e8c, SUC_lexicon_tags_7 },
    { 0xebba5e8c, SUC_lexicon_tags_27 },
    { 0x261a5e8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7a5e91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeb5a5e93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efa5e9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb07a5e9e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1a5ea1, SUC_lexicon_tags_11 },
    { 0xbfa5ea2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ba5ea5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26da5ea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9a5eac, SUC_lexicon_tags_6 },
    { 0x2c9a5ead, SUC_lexicon_tags_25 },
    { 0x335a5ead, SUC_lexicon_tags_68 },
    { 0xda5a5eaf, SUC_lexicon_tags_43 },
    { 0xdb1a5eb0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb5da5eb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7a5eb6, SUC_lexicon_tags_2 },
    { 0xeeba5eb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fa5eba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79a5ebe, SUC_lexicon_tags_2 },
    { 0x7d1a5ebe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3a5ec5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5a5ec9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfa5ed1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedba5ed7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaeba5ed9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fa5edf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835a5ee3, SUC_lexicon_tags_65 },
    { 0x605a5ee4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11a5ee8, SUC_lexicon_tags_19 },
    { 0x475a5ee9, SUC_lexicon_tags_25 },
    { 0x281a5eea, SUC_lexicon_tags_21 },
    { 0xa05a5ee9, SUC_lexicon_tags_17 },
    { 0x1b3a5eea, SUC_lexicon_tags_28 },
    { 0x963a5eed, SUC_lexicon_tags_2 },
    { 0x361a5eee, SUC_lexicon_tags_25 },
    { 0x88ba5eef, SUC_lexicon_tags_25 },
    { 0xd7ba5eef, SUC_lexicon_tags_98 },
    { 0x3ffa5ef1, SUC_lexicon_tags_25 },
    { 0x4e1a5ef2, SUC_lexicon_tags_29 },
    { 0xa2fa5ef3, SUC_lexicon_tags_30 },
    { 0xea1a5ef4, SUC_lexicon_tags_33 },
    { 0xc0fa5ef5, SUC_lexicon_tags_25 },
    { 0xfa7a5ef4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xada5efa, SUC_lexicon_tags_87 },
    { 0x1cda5efb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aba5eff, SUC_lexicon_tags_14 },
    { 0x9b9a5f00, SUC_lexicon_tags_27 },
    { 0x20ba5f01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29a5f04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51a5f07, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac9a5f09, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe63a5f0b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75a5f10, SUC_lexicon_tags_23 },
    { 0xdf5a5f10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe0fa5f13, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0x37fa5f15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbe3a5f17, SUC_lexicon_tags_47 },
    { 0x2bda5f18, SUC_lexicon_tags_40 },
    { 0x15ba5f19, SUC_lexicon_tags_45 },
    { 0x365a5f19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3c1a5f1c, SUC_lexicon_tags_15 },
    { 0xdfa5f1c, SUC_lexicon_tags_4 },
    { 0x3da5f1e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a5f21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x315a5f23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425a5f27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1a5f2a, SUC_lexicon_tags_6 },
    { 0x921a5f2a, SUC_lexicon_tags_87 },
    { 0x30da5f2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x707a5f2e, SUC_lexicon_tags_15 },
    { 0x849a5f2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ba5f32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fa5f36, SUC_lexicon_tags_17 },
    { 0x75ba5f37, SUC_lexicon_tags_82 },
    { 0x97da5f36, SUC_lexicon_tags_17 },
    { 0x839a5f39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395a5f3f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7da5f42, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x767a5f44, SUC_lexicon_tags_6 },
    { 0x44ba5f45, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x22fa5f47, SUC_lexicon_tags_1 },
    { 0xf83a5f47, SUC_lexicon_tags_17 },
    { 0x875a5f49, SUC_lexicon_tags_17 },
    { 0x6efa5f49, SUC_lexicon_tags_14 },
    { 0x6b7a5f48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfefa5f4d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3a5f50, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447a5f56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c3a5f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfa5f5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xde1a5f5f, SUC_lexicon_tags_11 },
    { 0x615a5f60, SUC_lexicon_tags_151 },
    { 0xc2da5f61, SUC_lexicon_tags_25 },
    { 0x847a5f61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7a5f65, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fa5f6a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1a5f72, SUC_lexicon_tags_23 },
    { 0x25da5f72, SUC_lexicon_tags_33 },
    { 0xcda5f74, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7a5f79, SUC_lexicon_tags_1 },
    { 0x291a5f7a, SUC_lexicon_tags_2 },
    { 0x6bba5f7b, SUC_lexicon_tags_29 },
    { 0x90da5f7c, SUC_lexicon_tags_25 },
    { 0xc5fa5f7b, SUC_lexicon_tags_6 },
    { 0xac5a5f7e, SUC_lexicon_tags_28 },
    { 0x941a5f7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da5f86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba1a5f88, SUC_lexicon_tags_12 },
    { 0x903a5f89, SUC_lexicon_tags_63 },
    { 0xada5f88, SUC_lexicon_tags_25 },
    { 0x5b7a5f89, SUC_lexicon_tags_25 },
    { 0xb1a5f89, SUC_lexicon_tags_55 },
    { 0xc47a5f8b, SUC_lexicon_tags_35 },
    { 0xfaba5f8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ba5f96, SUC_lexicon_tags_34 },
    { 0xfc7a5f97, SUC_lexicon_tags_47 },
    { 0x3f7a5f98, SUC_lexicon_tags_2 },
    { 0x6f5a5f99, SUC_lexicon_tags_2 },
    { 0x683a5f9a, SUC_lexicon_tags_1 },
    { 0xecda5f96, SUC_lexicon_tags_32 },
    { 0x543a5f9c, SUC_lexicon_tags_7 },
    { 0x4b7a5f9d, SUC_lexicon_tags_0 },
    { 0xb5fa5f9e, SUC_lexicon_tags_87 },
    { 0x219a5f9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7a5fa2, SUC_lexicon_tags_85 },
    { 0xeeda5fa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1a5fa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60da5fae, SUC_lexicon_tags_25 },
    { 0x869a5faf, SUC_lexicon_tags_15 },
    { 0xc77a5fb0, SUC_lexicon_tags_65 },
    { 0x4d1a5fb1, SUC_lexicon_tags_66 },
    { 0x20da5fb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31a5fb5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcdda5fb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9a5fbc, SUC_lexicon_tags_32 },
    { 0xb71a5fbc, SUC_lexicon_tags_35 },
    { 0x325a5fbe, SUC_lexicon_tags_1 },
    { 0xcfba5fbf, SUC_lexicon_tags_60 },
    { 0xa05a5fbf, SUC_lexicon_tags_33 },
    { 0x4a5a5fc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783a5fcc, SUC_lexicon_tags_40 },
    { 0x679a5fcc, SUC_lexicon_tags_23 },
    { 0x54da5fce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x26da5fd0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xaba5fd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565a5fea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03a5ff4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7a5ff7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xceba5ffa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7a6002, SUC_lexicon_tags_14 },
    { 0x521a6003, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54ba6005, SUC_lexicon_tags_39 },
    { 0x575a6005, SUC_lexicon_tags_37 },
    { 0x5c9a6007, SUC_lexicon_tags_61 },
    { 0xf01a6008, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xec9a600a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba600d, SUC_lexicon_tags_39 },
    { 0xe67a600d, SUC_lexicon_tags_14 },
    { 0x411a600f, SUC_lexicon_tags_11 },
    { 0x557a600d, SUC_lexicon_tags_11 },
    { 0x7dfa600f, SUC_lexicon_tags_8 },
    { 0xdafa6012, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5a6016, SUC_lexicon_tags_12 },
    { 0x507a6017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27a601b, SUC_lexicon_tags_40 },
    { 0xa73a601c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x173a601e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8efa6020, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x24ba6022, SUC_lexicon_tags_0 },
    { 0x7e7a6022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9f9a6025, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ba602b, SUC_lexicon_tags_29 },
    { 0x6b5a602c, SUC_lexicon_tags_17 },
    { 0x92ba602d, SUC_lexicon_tags_7 },
    { 0xb69a602c, SUC_lexicon_tags_32 },
    { 0x7bda602f, SUC_lexicon_tags_47 },
    { 0xd77a602b, SUC_lexicon_tags_39 },
    { 0x57da602d, SUC_lexicon_tags_27 },
    { 0x741a6032, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831a6036, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47a6039, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53da603b, SUC_lexicon_tags_39 },
    { 0x14fa603b, SUC_lexicon_tags_17 },
    { 0xa8ba603b, SUC_lexicon_tags_30 },
    { 0x249a603e, SUC_lexicon_tags_15 },
    { 0x165a603f, SUC_lexicon_tags_15 },
    { 0x6bda6040, SUC_lexicon_tags_6 },
    { 0x385a6041, SUC_lexicon_tags_25 },
    { 0x1a1a6042, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ba6049, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fba604c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fa6050, SUC_lexicon_tags_2 },
    { 0xecda6051, SUC_lexicon_tags_77 },
    { 0x563a6051, SUC_lexicon_tags_2 },
    { 0xd11a6051, SUC_lexicon_tags_2 },
    { 0x787a6054, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdada6056, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579a605d, SUC_lexicon_tags_21 },
    { 0x517a605e, SUC_lexicon_tags_15 },
    { 0x35fa605f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfba6068, SUC_lexicon_tags_15 },
    { 0x4fa6069, SUC_lexicon_tags_37 },
    { 0x79a6069, SUC_lexicon_tags_15 },
    { 0x701a606b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66da6070, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc43a6072, SUC_lexicon_tags_11 },
    { 0xcb7a6073, SUC_lexicon_tags_23 },
    { 0x20ba6074, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ba6078, SUC_lexicon_tags_17 },
    { 0xb8fa6079, SUC_lexicon_tags_47 },
    { 0x4b9a6078, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x817a607c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x701a607e, SUC_lexicon_tags_55 },
    { 0x92ba607f, SUC_lexicon_tags_43 },
    { 0x607a6080, SUC_lexicon_tags_15 },
    { 0xd0fa607e, SUC_lexicon_tags_63 },
    { 0x31ba6082, SUC_lexicon_tags_29 },
    { 0x111a6083, SUC_lexicon_tags_1 },
    { 0x5bba6083, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8bfa6086, SUC_lexicon_tags_27 },
    { 0x87fa6087, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf53a6089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32fa608c, SUC_lexicon_tags_70 },
    { 0xcc7a608c, SUC_lexicon_tags_2 },
    { 0x1e3a608e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7a6094, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1a6097, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695a609c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf83a609e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7a60a2, SUC_lexicon_tags_10 },
    { 0x473a60a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadda60a8, SUC_lexicon_tags_17 },
    { 0xfcba60a9, SUC_lexicon_tags_27 },
    { 0x20fa60aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x675a60ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fa60b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5a60b4, SUC_lexicon_tags_11 },
    { 0x513a60b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1da60b7, SUC_lexicon_tags_17 },
    { 0x71fa60b8, SUC_lexicon_tags_11 },
    { 0xad1a60b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x233a60bb, SUC_lexicon_tags_60 },
    { 0xf95a60bc, SUC_lexicon_tags_34 },
    { 0xa6fa60bb, SUC_lexicon_tags_12 },
    { 0xcd3a60bd, SUC_lexicon_tags_27 },
    { 0x9c7a60bf, SUC_lexicon_tags_47 },
    { 0x659a60be, SUC_lexicon_tags_17 },
    { 0x8f3a60c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a60c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25a60c8, SUC_lexicon_tags_46 },
    { 0x555a60c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7a60cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc8da60ce, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x193a60d0, SUC_lexicon_tags_29 },
    { 0x3bfa60d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefa60d3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78da60e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87a60e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bba60e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab3a60eb, SUC_lexicon_tags_37 },
    { 0x26fa60eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x489a60ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3a60f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf99a60f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a60f9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7d9a60fb, SUC_lexicon_tags_66 },
    { 0xabba60fc, SUC_lexicon_tags_11 },
    { 0xe3ba60fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x471a60ff, SUC_lexicon_tags_25 },
    { 0xb61a60ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ba6105, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7a6109, SUC_lexicon_tags_7 },
    { 0xa4ba610a, SUC_lexicon_tags_0 },
    { 0x94ba610b, SUC_lexicon_tags_14 },
    { 0x5fba610c, SUC_lexicon_tags_127 },
    { 0x8d7a610c, SUC_lexicon_tags_9 },
    { 0xe21a610a, SUC_lexicon_tags_17 },
    { 0x543a610f, SUC_lexicon_tags_15 },
    { 0x1cfa610f, SUC_lexicon_tags_7 },
    { 0x961a6111, SUC_lexicon_tags_37 },
    { 0x759a6112, SUC_lexicon_tags_38 },
    { 0x227a6112, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea9a6115, SUC_lexicon_tags_1 },
    { 0xe51a6116, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a6119, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ada611e, SUC_lexicon_tags_63 },
    { 0x54da611f, SUC_lexicon_tags_2 },
    { 0xb37a611e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb1a6122, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9a6127, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x997a6129, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfe1a612b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731a612f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85a6132, SUC_lexicon_tags_12 },
    { 0x77da6133, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3da6138, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a613f, SUC_lexicon_tags_2 },
    { 0x6dda613f, SUC_lexicon_tags_15 },
    { 0xd45a6141, SUC_lexicon_tags_15 },
    { 0x56ba6142, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53da6147, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8ffa6149, SUC_lexicon_tags_8 },
    { 0x80ba6149, SUC_lexicon_tags_91 },
    { 0xe6da614b, SUC_lexicon_tags_43 },
    { 0x327a614b, SUC_lexicon_tags_31 },
    { 0x973a614d, SUC_lexicon_tags_15 },
    { 0x8c7a614e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1a6152, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x453a6154, SUC_lexicon_tags_37 },
    { 0xca7a6155, SUC_lexicon_tags_9 },
    { 0x123a6156, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7a6158, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1a615c, SUC_lexicon_tags_25 },
    { 0x1f3a615c, SUC_lexicon_tags_17 },
    { 0xd7a615e, SUC_lexicon_tags_6 },
    { 0x6b5a615f, SUC_lexicon_tags_7 },
    { 0xe3ba615f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcefa6162, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61a6165, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbd7a6167, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe45a6169, SUC_lexicon_tags_69 },
    { 0x46da616a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7a616d, SUC_lexicon_tags_25 },
    { 0xea5a616e, SUC_lexicon_tags_17 },
    { 0xf17a616f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ba6173, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62da6175, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97a6177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f1a6179, SUC_lexicon_tags_15 },
    { 0x97ba617a, SUC_lexicon_tags_60 },
    { 0x799a617b, SUC_lexicon_tags_46 },
    { 0xdd1a6179, SUC_lexicon_tags_16 },
    { 0xebba617d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ada6180, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81a6185, SUC_lexicon_tags_37 },
    { 0x579a6186, SUC_lexicon_tags_16 },
    { 0xbdda6185, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1a618c, SUC_lexicon_tags_33 },
    { 0xad7a618d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc87a618f, SUC_lexicon_tags_37 },
    { 0x60fa6190, SUC_lexicon_tags_6 },
    { 0xb99a6190, SUC_lexicon_tags_17 },
    { 0x8e1a6192, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29a6199, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x237a619c, SUC_lexicon_tags_7 },
    { 0xfefa619d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c3a619f, SUC_lexicon_tags_26 },
    { 0xabba61a0, SUC_lexicon_tags_29 },
    { 0x103a61a1, SUC_lexicon_tags_30 },
    { 0xda7a61a2, SUC_lexicon_tags_33 },
    { 0x271a61a3, SUC_lexicon_tags_39 },
    { 0xc1ba61a4, SUC_lexicon_tags_37 },
    { 0xfdfa61a4, SUC_lexicon_tags_17 },
    { 0xee5a61a6, SUC_lexicon_tags_16 },
    { 0x9d5a61a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3a61b0, SUC_lexicon_tags_27 },
    { 0xf49a61b1, SUC_lexicon_tags_7 },
    { 0xdcda61b2, SUC_lexicon_tags_11 },
    { 0x7f7a61b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79a61b7, SUC_lexicon_tags_92 },
    { 0xde5a61b8, SUC_lexicon_tags_12 },
    { 0xba3a61b9, SUC_lexicon_tags_25 },
    { 0x421a61b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15a61c3, SUC_lexicon_tags_25 },
    { 0xf29a61c3, SUC_lexicon_tags_129 },
    { 0x4d1a61c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93ba61c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3a61cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1da61cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfa61d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x901a61d4, SUC_lexicon_tags_15 },
    { 0x2ffa61d4, SUC_lexicon_tags_10 },
    { 0xad1a61d6, SUC_lexicon_tags_7 },
    { 0x55da61d7, SUC_lexicon_tags_1 },
    { 0xd49a61d8, SUC_lexicon_tags_29 },
    { 0xec3a61d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1a61dd, SUC_lexicon_tags_17 },
    { 0xee9a61de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9a61e4, SUC_lexicon_tags_47 },
    { 0x5aba61e5, SUC_lexicon_tags_1 },
    { 0x237a61e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f7a61e8, SUC_lexicon_tags_6 },
    { 0xafda61e8, SUC_lexicon_tags_43 },
    { 0x2e1a61e9, SUC_lexicon_tags_0 },
    { 0x3f5a61eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fa61ef, SUC_lexicon_tags_37 },
    { 0xa0ba61ef, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x499a61f2, SUC_lexicon_tags_26 },
    { 0x52da61f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ffa61f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86da61fb, SUC_lexicon_tags_7 },
    { 0xcf9a61fc, SUC_lexicon_tags_1 },
    { 0x3e1a61fb, SUC_lexicon_tags_27 },
    { 0x6c9a61fb, SUC_lexicon_tags_2 },
    { 0x7eda61ff, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09a6206, SUC_lexicon_tags_25 },
    { 0xf1fa6206, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x639a6209, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ba620f, SUC_lexicon_tags_11 },
    { 0x3e5a6210, SUC_lexicon_tags_14 },
    { 0xed9a6210, SUC_lexicon_tags_15 },
    { 0x54ba6212, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117a6216, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fa621c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x94fa621e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9a6221, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bda6225, SUC_lexicon_tags_39 },
    { 0x883a6225, SUC_lexicon_tags_43 },
    { 0x85fa6226, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbda6229, SUC_lexicon_tags_2 },
    { 0x465a6229, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ba622f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf25a6231, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20da6235, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbf5a6237, SUC_lexicon_tags_87 },
    { 0x3f7a6238, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd51a623a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x239a623d, SUC_lexicon_tags_33 },
    { 0xcc7a623d, SUC_lexicon_tags_70 },
    { 0x1d3a623f, SUC_lexicon_tags_26 },
    { 0xe59a6240, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ba6244, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfd1a6246, SUC_lexicon_tags_11 },
    { 0xec5a6247, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x11da6249, SUC_lexicon_tags_8 },
    { 0x2c5a624a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9a624d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01a6255, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49a6258, SUC_lexicon_tags_1 },
    { 0x4d5a6259, SUC_lexicon_tags_33 },
    { 0x7d7a625a, SUC_lexicon_tags_37 },
    { 0xfd7a6258, SUC_lexicon_tags_12 },
    { 0x59fa625c, SUC_lexicon_tags_2 },
    { 0x9bba625d, SUC_lexicon_tags_18 },
    { 0xeada625e, SUC_lexicon_tags_11 },
    { 0x947a625f, SUC_lexicon_tags_23 },
    { 0xbf5a625f, SUC_lexicon_tags_146 },
    { 0x4fa625c, SUC_lexicon_tags_48 },
    { 0xa67a6262, SUC_lexicon_tags_25 },
    { 0x611a6263, SUC_lexicon_tags_2 },
    { 0x211a6264, SUC_lexicon_tags_30 },
    { 0x951a6262, SUC_lexicon_tags_12 },
    { 0xaa1a6266, SUC_lexicon_tags_32 },
    { 0x13da6267, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597a626f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1a6279, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x719a627b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x83a627d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ba628a, SUC_lexicon_tags_17 },
    { 0x583a628b, SUC_lexicon_tags_0 },
    { 0x5a3a628b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09a628f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39da6293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8da6295, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa47a6297, SUC_lexicon_tags_11 },
    { 0x4bfa6298, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5a629b, SUC_lexicon_tags_0 },
    { 0x4a5a629b, SUC_lexicon_tags_8 },
    { 0x4a5a629c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeba629f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x117a62a2, SUC_lexicon_tags_38 },
    { 0xd4ba62a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa59a62a5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x683a62a7, SUC_lexicon_tags_33 },
    { 0xcf9a62a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3a62b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ada62b3, SUC_lexicon_tags_15 },
    { 0xbe7a62b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a62ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29a62be, SUC_lexicon_tags_7 },
    { 0xa91a62be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x225a62c2, SUC_lexicon_tags_19 },
    { 0xb45a62c3, SUC_lexicon_tags_4 },
    { 0x6cda62c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1a62cb, SUC_lexicon_tags_17 },
    { 0xc17a62cc, SUC_lexicon_tags_13 },
    { 0xa6ba62cd, SUC_lexicon_tags_61 },
    { 0xf25a62ce, SUC_lexicon_tags_23 },
    { 0x321a62cf, SUC_lexicon_tags_12 },
    { 0xd41a62cb, SUC_lexicon_tags_25 },
    { 0x4a9a62d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x717a62d3, SUC_lexicon_tags_29 },
    { 0xc3a62d3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633a62d8, SUC_lexicon_tags_14 },
    { 0xab7a62d9, SUC_lexicon_tags_40 },
    { 0xf4ba62d8, SUC_lexicon_tags_25 },
    { 0xe7da62db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da62df, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5a62e2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa63a62e4, SUC_lexicon_tags_7 },
    { 0x6c9a62e5, SUC_lexicon_tags_6 },
    { 0xc59a62e6, SUC_lexicon_tags_14 },
    { 0x69da62e7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19a62eb, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97a62ee, SUC_lexicon_tags_61 },
    { 0xd2da62ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb91a62f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bfa62f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3da62f6, SUC_lexicon_tags_7 },
    { 0x4d3a62f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xffba62f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a5a62fb, SUC_lexicon_tags_84 },
    { 0xb0ba62fb, SUC_lexicon_tags_15 },
    { 0xda62fd, SUC_lexicon_tags_46 },
    { 0x21fa62fe, SUC_lexicon_tags_15 },
    { 0xf7fa62ff, SUC_lexicon_tags_1 },
    { 0x883a62fc, SUC_lexicon_tags_27 },
    { 0xe09a6301, SUC_lexicon_tags_2 },
    { 0xb8ba6301, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cda6308, SUC_lexicon_tags_15 },
    { 0x8b9a6309, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fba630c, SUC_lexicon_tags_16 },
    { 0x817a630c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ba6312, SUC_lexicon_tags_25 },
    { 0x2bda6313, SUC_lexicon_tags_66 },
    { 0xb21a6314, SUC_lexicon_tags_17 },
    { 0x55ba6315, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa631d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbdfa631f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fa6324, SUC_lexicon_tags_77 },
    { 0x95da6324, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffa6329, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01a632c, SUC_lexicon_tags_37 },
    { 0xe01a632c, SUC_lexicon_tags_25 },
    { 0x129a632e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x381a6330, SUC_lexicon_tags_17 },
    { 0x9ba6330, SUC_lexicon_tags_37 },
    { 0xc53a6331, SUC_lexicon_tags_49 },
    { 0xc07a6333, SUC_lexicon_tags_3 },
    { 0xa87a6333, SUC_lexicon_tags_8 },
    { 0x37a6335, SUC_lexicon_tags_28 },
    { 0xfeba6335, SUC_lexicon_tags_60 },
    { 0x649a6337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57a633c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efa6343, SUC_lexicon_tags_27 },
    { 0x76fa6343, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce3a6346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ba634b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fa634e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfa6353, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3a6356, SUC_lexicon_tags_25 },
    { 0xbd7a6357, SUC_lexicon_tags_10 },
    { 0xc41a6356, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x139a635b, SUC_lexicon_tags_25 },
    { 0x93fa635c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1a635f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447a6364, SUC_lexicon_tags_27 },
    { 0xb5da6364, SUC_lexicon_tags_25 },
    { 0x8aba6366, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fa636c, SUC_lexicon_tags_14 },
    { 0xf63a636d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fa637c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3a6381, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5a6387, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207a638b, SUC_lexicon_tags_6 },
    { 0x2dda638b, SUC_lexicon_tags_10 },
    { 0x9ffa638c, SUC_lexicon_tags_17 },
    { 0xc89a638e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891a6392, SUC_lexicon_tags_2 },
    { 0xb1fa6393, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787a6399, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd67a639c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0fa639e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a63a2, SUC_lexicon_tags_25 },
    { 0x4c9a63a2, SUC_lexicon_tags_12 },
    { 0x5e5a63a3, SUC_lexicon_tags_39 },
    { 0x525a63a5, SUC_lexicon_tags_37 },
    { 0xecba63a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61a63aa, SUC_lexicon_tags_65 },
    { 0x67fa63ab, SUC_lexicon_tags_37 },
    { 0xdc5a63ac, SUC_lexicon_tags_1 },
    { 0x69ba63ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bda63b0, SUC_lexicon_tags_25 },
    { 0xc01a63b1, SUC_lexicon_tags_11 },
    { 0x18da63b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fba63b4, SUC_lexicon_tags_6 },
    { 0xd0ba63b5, SUC_lexicon_tags_35 },
    { 0x17da63b6, SUC_lexicon_tags_15 },
    { 0x365a63b6, SUC_lexicon_tags_31 },
    { 0x625a63b8, SUC_lexicon_tags_17 },
    { 0xc35a63b7, SUC_lexicon_tags_4 },
    { 0xe3a63b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445a63be, SUC_lexicon_tags_14 },
    { 0x9e3a63bf, SUC_lexicon_tags_12 },
    { 0x715a63c0, SUC_lexicon_tags_33 },
    { 0x70da63c1, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x463a63c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bba63ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229a63d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9a63d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdfa63d9, SUC_lexicon_tags_14 },
    { 0xfe3a63da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eda63e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9a63e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1a63e9, SUC_lexicon_tags_37 },
    { 0xeb7a63ea, SUC_lexicon_tags_33 },
    { 0xc11a63eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9a63ee, SUC_lexicon_tags_45 },
    { 0x7e7a63ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403a63f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28da63f7, SUC_lexicon_tags_15 },
    { 0x28da63f8, SUC_lexicon_tags_11 },
    { 0x5c1a63f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffa63fd, SUC_lexicon_tags_17 },
    { 0x2e9a63fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84da6403, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e7a6405, SUC_lexicon_tags_23 },
    { 0x551a6406, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x15fa6408, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x909a640b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61a640e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21a6415, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b3a6417, SUC_lexicon_tags_15 },
    { 0x8dba6418, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5e5a641a, SUC_lexicon_tags_38 },
    { 0x8ada641a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837a6420, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5a6423, SUC_lexicon_tags_24 },
    { 0xb2da6424, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x26da6426, SUC_lexicon_tags_60 },
    { 0x10fa6426, SUC_lexicon_tags_62 },
    { 0xc9da6427, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xada642a, SUC_lexicon_tags_2 },
    { 0xf77a642b, SUC_lexicon_tags_15 },
    { 0x45ba642a, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0xf0ba642e, SUC_lexicon_tags_66 },
    { 0x48ba642f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3a6437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3a643a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13a643f, SUC_lexicon_tags_26 },
    { 0xab1a6440, SUC_lexicon_tags_193 },
    { 0x739a643f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8ba6443, SUC_lexicon_tags_7 },
    { 0xd8da6444, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefa6448, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9a644e, SUC_lexicon_tags_17 },
    { 0x32da644f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fa6454, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fa645a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb33a645c, SUC_lexicon_tags_30 },
    { 0x901a645d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753a6462, SUC_lexicon_tags_14 },
    { 0xa31a6463, SUC_lexicon_tags_1 },
    { 0x47fa6464, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603a646a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52da646f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x70ba6471, SUC_lexicon_tags_63 },
    { 0xfba6472, SUC_lexicon_tags_1 },
    { 0xd0da6473, SUC_lexicon_tags_1 },
    { 0x591a6472, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ba6479, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x585a647c, SUC_lexicon_tags_66 },
    { 0xc6ba647c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377a6481, SUC_lexicon_tags_29 },
    { 0x37a6482, SUC_lexicon_tags_17 },
    { 0x88ba6483, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb2ba6485, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dba6488, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999a648c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d7a648e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53da6493, SUC_lexicon_tags_25 },
    { 0x413a6493, SUC_lexicon_tags_29 },
    { 0xd67a6495, SUC_lexicon_tags_37 },
    { 0x1b3a6495, SUC_lexicon_tags_11 },
    { 0x91a6496, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc19a6499, SUC_lexicon_tags_11 },
    { 0xbaba6499, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf87a649c, SUC_lexicon_tags_2 },
    { 0xf5da649c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeba64a0, SUC_lexicon_tags_33 },
    { 0x3c5a64a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcba64a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5a64a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eda64aa, SUC_lexicon_tags_39 },
    { 0xcf3a64ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaeda64ad, SUC_lexicon_tags_6 },
    { 0x617a64ad, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9a64b1, SUC_lexicon_tags_11 },
    { 0x8e9a64b1, SUC_lexicon_tags_37 },
    { 0x8dba64b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bba64b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9a64b9, SUC_lexicon_tags_2 },
    { 0xc01a64b9, SUC_lexicon_tags_37 },
    { 0xadba64bb, SUC_lexicon_tags_21 },
    { 0x801a64bc, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7a64c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x881a64c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f1a64c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81a64c9, SUC_lexicon_tags_35 },
    { 0x1d1a64ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8bda64cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeeba64ce, SUC_lexicon_tags_83 },
    { 0xce5a64ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3a64da, SUC_lexicon_tags_56 },
    { 0x73fa64db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e3a64dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x701a64e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd1a64e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52da64e4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23da64e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fa64f0, SUC_lexicon_tags_60 },
    { 0x2a1a64f0, SUC_lexicon_tags_7 },
    { 0x7aba64f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215a64f6, SUC_lexicon_tags_17 },
    { 0xf49a64f6, SUC_lexicon_tags_37 },
    { 0x747a64f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591a64fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa45a64fe, SUC_lexicon_tags_12 },
    { 0x8f7a64fe, SUC_lexicon_tags_30 },
    { 0xf3da6500, SUC_lexicon_tags_27 },
    { 0x65da6501, SUC_lexicon_tags_12 },
    { 0x35a6502, SUC_lexicon_tags_12 },
    { 0xff9a6503, SUC_lexicon_tags_0 },
    { 0x24fa6503, SUC_lexicon_tags_14 },
    { 0xd3fa6505, SUC_lexicon_tags_59 },
    { 0x3a3a6506, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x425a6508, SUC_lexicon_tags_33 },
    { 0x6dba6509, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fa650e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ba6513, SUC_lexicon_tags_27 },
    { 0x329a6514, SUC_lexicon_tags_39 },
    { 0x52ba6515, SUC_lexicon_tags_27 },
    { 0x7f9a6516, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61a651c, SUC_lexicon_tags_151 },
    { 0x7c5a651c, SUC_lexicon_tags_29 },
    { 0x963a651e, SUC_lexicon_tags_9 },
    { 0x3a5a651f, SUC_lexicon_tags_23 },
    { 0xb3ba651f, SUC_lexicon_tags_37 },
    { 0xa25a651d, SUC_lexicon_tags_2 },
    { 0x1e9a651c, SUC_lexicon_tags_9 },
    { 0xd33a6523, SUC_lexicon_tags_12 },
    { 0x50ba6523, SUC_lexicon_tags_14 },
    { 0xa5a6524, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99ba6527, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457a652c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdada652f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05a6532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c5a6534, SUC_lexicon_tags_10 },
    { 0xbba6535, SUC_lexicon_tags_14 },
    { 0x7d5a6535, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761a653a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x409a653d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7a6540, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fa6543, SUC_lexicon_tags_7 },
    { 0x591a6543, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfba6548, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29a654a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x52da654c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x39a654f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x607a6551, SUC_lexicon_tags_1 },
    { 0x75fa6552, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc23a6554, SUC_lexicon_tags_37 },
    { 0x29da6555, SUC_lexicon_tags_37 },
    { 0xf2da6556, SUC_lexicon_tags_19 },
    { 0x38fa6556, SUC_lexicon_tags_65 },
    { 0x147a6555, SUC_lexicon_tags_15 },
    { 0xbcfa6559, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xaf9a655b, SUC_lexicon_tags_16 },
    { 0x673a655b, SUC_lexicon_tags_17 },
    { 0x40fa655d, SUC_lexicon_tags_15 },
    { 0x1b1a655e, SUC_lexicon_tags_2 },
    { 0x33ba655f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc15a6561, SUC_lexicon_tags_23 },
    { 0x76da6562, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681a6567, SUC_lexicon_tags_2 },
    { 0xf49a6568, SUC_lexicon_tags_17 },
    { 0xf3fa6567, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1a656f, SUC_lexicon_tags_26 },
    { 0xcdda6570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1ba6572, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cba6576, SUC_lexicon_tags_30 },
    { 0xa41a6576, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x77ba6579, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423a6583, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77a6586, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x373a6589, SUC_lexicon_tags_2 },
    { 0x97a658a, SUC_lexicon_tags_82 },
    { 0xb07a658b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x975a658d, SUC_lexicon_tags_63 },
    { 0x315a658e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fa6592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7a6596, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x28fa6598, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1a659f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fa65a5, SUC_lexicon_tags_12 },
    { 0xee7a65a6, SUC_lexicon_tags_29 },
    { 0x315a65a7, SUC_lexicon_tags_7 },
    { 0xfdda65a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3a65ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93a65af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a65b6, SUC_lexicon_tags_0 },
    { 0x3f5a65b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfaba65b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61a65bf, SUC_lexicon_tags_17 },
    { 0x557a65bf, SUC_lexicon_tags_43 },
    { 0xb6ba65c1, SUC_lexicon_tags_37 },
    { 0x6e7a65c0, SUC_lexicon_tags_1 },
    { 0xf3a65c3, SUC_lexicon_tags_37 },
    { 0xf8fa65c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279a65c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x673a65cc, SUC_lexicon_tags_68 },
    { 0xc2fa65cd, SUC_lexicon_tags_17 },
    { 0x28fa65cc, SUC_lexicon_tags_2 },
    { 0x61a65cf, SUC_lexicon_tags_55 },
    { 0xf4fa65d0, SUC_lexicon_tags_2 },
    { 0x179a65d1, SUC_lexicon_tags_39 },
    { 0x461a65ce, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xdc7a65d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb7a65d6, SUC_lexicon_tags_11 },
    { 0xd83a65d7, SUC_lexicon_tags_11 },
    { 0xdd1a65d8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3a65de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fa65e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e9a65e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x123a65ea, SUC_lexicon_tags_6 },
    { 0x817a65eb, SUC_lexicon_tags_37 },
    { 0x2cba65ec, SUC_lexicon_tags_0 },
    { 0x575a65ec, SUC_lexicon_tags_39 },
    { 0x8c1a65ee, SUC_lexicon_tags_26 },
    { 0x7e9a65ef, SUC_lexicon_tags_53 },
    { 0x8b9a65f0, SUC_lexicon_tags_17 },
    { 0x4c7a65f1, SUC_lexicon_tags_17 },
    { 0x167a65f2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ba65f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145a65fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72da65ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e1a6601, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dda6604, SUC_lexicon_tags_68 },
    { 0x2e3a6605, SUC_lexicon_tags_11 },
    { 0xff3a6606, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x581a6608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fa660e, SUC_lexicon_tags_12 },
    { 0x5e9a660f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bba6611, SUC_lexicon_tags_17 },
    { 0x303a6611, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5f5a6614, SUC_lexicon_tags_33 },
    { 0x881a6614, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x649a6617, SUC_lexicon_tags_66 },
    { 0x7ada6617, SUC_lexicon_tags_12 },
    { 0x823a6619, SUC_lexicon_tags_92 },
    { 0xf07a661a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaba661f, SUC_lexicon_tags_29 },
    { 0x517a6620, SUC_lexicon_tags_60 },
    { 0xff5a6621, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x337a6623, SUC_lexicon_tags_1 },
    { 0xfc3a6624, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109a6629, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5a662c, SUC_lexicon_tags_1 },
    { 0x70fa662d, SUC_lexicon_tags_1 },
    { 0x99fa662c, SUC_lexicon_tags_32 },
    { 0x143a662f, SUC_lexicon_tags_38 },
    { 0x28da6630, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51a6636, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5a5a6638, SUC_lexicon_tags_39 },
    { 0x71ba6639, SUC_lexicon_tags_11 },
    { 0x44ba663a, SUC_lexicon_tags_43 },
    { 0xf43a663b, SUC_lexicon_tags_45 },
    { 0xb6ba663b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc65a663e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd67a6640, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x565a6642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb75a6644, SUC_lexicon_tags_6 },
    { 0xad1a6645, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x133a6647, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f1a6649, SUC_lexicon_tags_38 },
    { 0x7dba6649, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5a664d, SUC_lexicon_tags_15 },
    { 0xeb5a664e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd7a6650, SUC_lexicon_tags_37 },
    { 0xf99a6651, SUC_lexicon_tags_106 },
    { 0xb3a6652, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1a6658, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dda665d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3a6660, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20da6668, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1a666c, SUC_lexicon_tags_37 },
    { 0xbe1a666d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4ba666f, SUC_lexicon_tags_21 },
    { 0x1d7a6670, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeba6673, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c3a6675, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x163a6677, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x389a667a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3a667e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbda6681, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5a6684, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6ffa6686, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x265a6688, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceba668f, SUC_lexicon_tags_7 },
    { 0x1ada668f, SUC_lexicon_tags_29 },
    { 0x25da6691, SUC_lexicon_tags_61 },
    { 0xf57a6691, SUC_lexicon_tags_37 },
    { 0xe2da6691, SUC_lexicon_tags_65 },
    { 0x593a6690, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15a669c, SUC_lexicon_tags_63 },
    { 0xb47a669d, SUC_lexicon_tags_158 },
    { 0x615a669c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ba66a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefa66a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69da66aa, SUC_lexicon_tags_15 },
    { 0x9cfa66ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7a66ae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eba66b1, SUC_lexicon_tags_26 },
    { 0xc8fa66b2, SUC_lexicon_tags_8 },
    { 0x463a66b3, SUC_lexicon_tags_31 },
    { 0xc3ba66b4, SUC_lexicon_tags_32 },
    { 0x5b7a66b2, SUC_lexicon_tags_3 },
    { 0x4b1a66b4, SUC_lexicon_tags_17 },
    { 0xde9a66b7, SUC_lexicon_tags_33 },
    { 0xa53a66b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9cfa66ba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x771a66bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fa66c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x473a66c2, SUC_lexicon_tags_7 },
    { 0xcf3a66c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29fa66c7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfefa66c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21a66ce, SUC_lexicon_tags_2 },
    { 0xed5a66ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41a66d4, SUC_lexicon_tags_47 },
    { 0xd29a66d5, SUC_lexicon_tags_25 },
    { 0x2a9a66d6, SUC_lexicon_tags_17 },
    { 0x13a66d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x671a66d9, SUC_lexicon_tags_1 },
    { 0xb89a66da, SUC_lexicon_tags_7 },
    { 0x981a66da, SUC_lexicon_tags_23 },
    { 0xc87a66da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2eba66de, SUC_lexicon_tags_23 },
    { 0x215a66de, SUC_lexicon_tags_2 },
    { 0xe11a66e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fda66e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfba66e5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x5e9a66e7, SUC_lexicon_tags_36 },
    { 0x15da66e8, SUC_lexicon_tags_7 },
    { 0x905a66e9, SUC_lexicon_tags_31 },
    { 0x757a66e8, SUC_lexicon_tags_23 },
    { 0xb27a66e7, SUC_lexicon_tags_25 },
    { 0x1b1a66e9, SUC_lexicon_tags_15 },
    { 0x701a66ed, SUC_lexicon_tags_37 },
    { 0xb5ba66ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69a66f3, SUC_lexicon_tags_65 },
    { 0x517a66f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x15ba66f6, SUC_lexicon_tags_14 },
    { 0x7efa66f7, SUC_lexicon_tags_43 },
    { 0xeda66f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe9da66fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ba66fd, SUC_lexicon_tags_2 },
    { 0x9a9a66fd, SUC_lexicon_tags_13 },
    { 0x915a66ff, SUC_lexicon_tags_2 },
    { 0x589a66ff, SUC_lexicon_tags_17 },
    { 0xe53a6700, SUC_lexicon_tags_37 },
    { 0xbffa66fe, SUC_lexicon_tags_35 },
    { 0x60da6703, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1a6707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2da6709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda5a670b, SUC_lexicon_tags_2 },
    { 0x5d1a670c, SUC_lexicon_tags_38 },
    { 0xfd5a670b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc1a670f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4a9a6711, SUC_lexicon_tags_47 },
    { 0x62da6711, SUC_lexicon_tags_22 },
    { 0x2fa6713, SUC_lexicon_tags_56 },
    { 0xbba6713, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1a671a, SUC_lexicon_tags_7 },
    { 0xe45a671b, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xf41a671d, SUC_lexicon_tags_84 },
    { 0x955a671d, SUC_lexicon_tags_49 },
    { 0xe03a671f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a9a6721, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1da6726, SUC_lexicon_tags_65 },
    { 0xa9fa6727, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7a6729, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x373a672c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9a6731, SUC_lexicon_tags_1 },
    { 0xee7a6731, SUC_lexicon_tags_30 },
    { 0xf6da6731, SUC_lexicon_tags_43 },
    { 0xb61a6734, SUC_lexicon_tags_9 },
    { 0xc1fa6735, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x215a6737, SUC_lexicon_tags_7 },
    { 0xa67a6738, SUC_lexicon_tags_25 },
    { 0xed3a6739, SUC_lexicon_tags_12 },
    { 0xac3a673a, SUC_lexicon_tags_33 },
    { 0xaada673b, SUC_lexicon_tags_65 },
    { 0x2f3a673b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x39fa673e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9a6742, SUC_lexicon_tags_45 },
    { 0xf9a6743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x835a6745, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x419a6748, SUC_lexicon_tags_33 },
    { 0x95a6748, SUC_lexicon_tags_17 },
    { 0xd65a6748, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11a6757, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3a6759, SUC_lexicon_tags_17 },
    { 0x4e9a675a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ba675d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x723a6760, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37a6763, SUC_lexicon_tags_7 },
    { 0x7cfa6763, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8da676a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ba676f, SUC_lexicon_tags_87 },
    { 0x145a6770, SUC_lexicon_tags_11 },
    { 0x513a6771, SUC_lexicon_tags_20 },
    { 0x4c9a6772, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747a6776, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fa677d, SUC_lexicon_tags_21 },
    { 0xfa3a677e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x12da6780, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769a6784, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4da6787, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72da678a, SUC_lexicon_tags_7 },
    { 0xfe3a678b, SUC_lexicon_tags_23 },
    { 0xd7da678c, SUC_lexicon_tags_2 },
    { 0xdd5a678c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb3a678f, SUC_lexicon_tags_11 },
    { 0x14da6790, SUC_lexicon_tags_37 },
    { 0xcbba6790, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1a6794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fa6799, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6bba679b, SUC_lexicon_tags_7 },
    { 0x951a679c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x779a679e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x757a67a0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93a67a6, SUC_lexicon_tags_12 },
    { 0xf53a67a6, SUC_lexicon_tags_43 },
    { 0x201a67a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5a67ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x467a67af, SUC_lexicon_tags_37 },
    { 0x5fba67b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37a67b4, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9a67b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a7a67bb, SUC_lexicon_tags_7 },
    { 0x4a9a67bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b9a67be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fa67c1, SUC_lexicon_tags_31 },
    { 0xedfa67c2, SUC_lexicon_tags_47 },
    { 0x7ba67c3, SUC_lexicon_tags_29 },
    { 0x429a67c3, SUC_lexicon_tags_22 },
    { 0x2dda67c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47da67c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cda67cb, SUC_lexicon_tags_31 },
    { 0x72ba67cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef7a67ce, SUC_lexicon_tags_17 },
    { 0x1a7a67ce, SUC_lexicon_tags_11 },
    { 0x4d5a67ce, SUC_lexicon_tags_129 },
    { 0x675a67d1, SUC_lexicon_tags_12 },
    { 0x5f3a67d2, SUC_lexicon_tags_32 },
    { 0xe2da67d2, SUC_lexicon_tags_15 },
    { 0x643a67d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93a67da, SUC_lexicon_tags_15 },
    { 0xbf1a67db, SUC_lexicon_tags_32 },
    { 0xa3fa67dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcada67de, SUC_lexicon_tags_26 },
    { 0x69fa67de, SUC_lexicon_tags_29 },
    { 0x53ba67e0, SUC_lexicon_tags_37 },
    { 0xea9a67e1, SUC_lexicon_tags_87 },
    { 0x7e7a67e1, SUC_lexicon_tags_15 },
    { 0xe0ba67e3, SUC_lexicon_tags_25 },
    { 0x983a67e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x865a67e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdb9a67e8, SUC_lexicon_tags_16 },
    { 0x5e7a67e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x181a67ec, SUC_lexicon_tags_14 },
    { 0xa5a67ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42da67f0, SUC_lexicon_tags_42 },
    { 0x6c5a67f1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x37a67f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa67f9, SUC_lexicon_tags_25 },
    { 0xe2da67fa, SUC_lexicon_tags_17 },
    { 0x327a67fb, SUC_lexicon_tags_63 },
    { 0x279a67fb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cba6807, SUC_lexicon_tags_1682 },
    { 0, NULL },
    { 0x7e5a6809, SUC_lexicon_tags_68 },
    { 0xb2fa6809, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7a680d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ba6812, SUC_lexicon_tags_11 },
    { 0x935a6813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7ba6815, SUC_lexicon_tags_23 },
    { 0x281a6815, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3fa6818, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159a681e, SUC_lexicon_tags_25 },
    { 0x7cda681f, SUC_lexicon_tags_25 },
    { 0xc1a6820, SUC_lexicon_tags_25 },
    { 0x929a6820, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x373a6823, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5a6826, SUC_lexicon_tags_17 },
    { 0x6bfa6827, SUC_lexicon_tags_83 },
    { 0x84fa6826, SUC_lexicon_tags_17 },
    { 0x6d3a6829, SUC_lexicon_tags_25 },
    { 0x89da682a, SUC_lexicon_tags_17 },
    { 0x7c3a682b, SUC_lexicon_tags_23 },
    { 0x981a6827, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fa6831, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xad7a6833, SUC_lexicon_tags_12 },
    { 0xf53a6834, SUC_lexicon_tags_0 },
    { 0x235a6835, SUC_lexicon_tags_8 },
    { 0xff5a6835, SUC_lexicon_tags_15 },
    { 0xfbda6837, SUC_lexicon_tags_82 },
    { 0xa09a6838, SUC_lexicon_tags_15 },
    { 0xf8ba6838, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4dfa683b, SUC_lexicon_tags_6 },
    { 0x457a683c, SUC_lexicon_tags_630 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15a6840, SUC_lexicon_tags_11 },
    { 0xaa3a6841, SUC_lexicon_tags_118 },
    { 0x623a6841, SUC_lexicon_tags_2 },
    { 0xc7ba6843, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x56ba6845, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95a6848, SUC_lexicon_tags_63 },
    { 0x361a6848, SUC_lexicon_tags_27 },
    { 0x607a6848, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaefa684d, SUC_lexicon_tags_37 },
    { 0xa31a684e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe51a6850, SUC_lexicon_tags_38 },
    { 0x32da6850, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x597a6854, SUC_lexicon_tags_37 },
    { 0xb0da6855, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba685a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9a685f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9da6861, SUC_lexicon_tags_11 },
    { 0x223a6861, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efa6867, SUC_lexicon_tags_30 },
    { 0xdf7a6867, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1a686d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663a6872, SUC_lexicon_tags_25 },
    { 0x4b9a6873, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8a7a6875, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1a6879, SUC_lexicon_tags_1602 },
    { 0x8efa687a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24da687c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03a687f, SUC_lexicon_tags_15 },
    { 0x791a6880, SUC_lexicon_tags_68 },
    { 0xb4ba6881, SUC_lexicon_tags_37 },
    { 0xde5a6880, SUC_lexicon_tags_23 },
    { 0x987a6880, SUC_lexicon_tags_19 },
    { 0x175a6884, SUC_lexicon_tags_33 },
    { 0xcc1a6885, SUC_lexicon_tags_40 },
    { 0x4eda6885, SUC_lexicon_tags_2 },
    { 0xe5fa6882, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67a688c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805a6893, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x691a6895, SUC_lexicon_tags_23 },
    { 0x2d3a6896, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4cba6898, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fa689b, SUC_lexicon_tags_12 },
    { 0x84ba689c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96da68a3, SUC_lexicon_tags_47 },
    { 0xa15a68a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x19fa68a6, SUC_lexicon_tags_66 },
    { 0x317a68a6, SUC_lexicon_tags_25 },
    { 0x41a68a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7a68ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99a68b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25a68b4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc33a68b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeefa68b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdfa68ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25a68bc, SUC_lexicon_tags_30 },
    { 0x7ffa68bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x519a68bf, SUC_lexicon_tags_29 },
    { 0x8ffa68c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x347a68c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x795a68c5, SUC_lexicon_tags_2 },
    { 0xf4ba68c6, SUC_lexicon_tags_84 },
    { 0xcada68c7, SUC_lexicon_tags_87 },
    { 0xbdba68c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b7a68ca, SUC_lexicon_tags_37 },
    { 0x597a68cb, SUC_lexicon_tags_37 },
    { 0x9dba68ca, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ba68d3, SUC_lexicon_tags_33 },
    { 0x99ba68d3, SUC_lexicon_tags_7 },
    { 0x291a68d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815a68da, SUC_lexicon_tags_0 },
    { 0x567a68db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f5a68dd, SUC_lexicon_tags_79 },
    { 0x79da68dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ba68e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1a68eb, SUC_lexicon_tags_42 },
    { 0x9fba68eb, SUC_lexicon_tags_39 },
    { 0xb19a68ed, SUC_lexicon_tags_38 },
    { 0xeeda68ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151a68f2, SUC_lexicon_tags_87 },
    { 0xae7a68f2, SUC_lexicon_tags_2 },
    { 0xb65a68f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdda68f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fa68fb, SUC_lexicon_tags_29 },
    { 0x2fda68fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe85a68fe, SUC_lexicon_tags_25 },
    { 0x653a68fe, SUC_lexicon_tags_38 },
    { 0x1a1a6900, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dda6903, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xb51a6905, SUC_lexicon_tags_15 },
    { 0xed7a6905, SUC_lexicon_tags_26 },
    { 0x39da6907, SUC_lexicon_tags_37 },
    { 0x47da6907, SUC_lexicon_tags_38 },
    { 0x6cba6908, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9a690c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5a6914, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe8fa6916, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11a691a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x853a691c, SUC_lexicon_tags_55 },
    { 0x835a691c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf09a691f, SUC_lexicon_tags_31 },
    { 0x127a691f, SUC_lexicon_tags_60 },
    { 0x811a6921, SUC_lexicon_tags_0 },
    { 0xdc9a6921, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5a6927, SUC_lexicon_tags_28 },
    { 0x135a6928, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7da692d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdba6931, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf1a6933, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70ba6935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3a6939, SUC_lexicon_tags_17 },
    { 0xd33a693a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3a693e, SUC_lexicon_tags_25 },
    { 0xa3fa693f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ba6942, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749a6949, SUC_lexicon_tags_11 },
    { 0x871a694a, SUC_lexicon_tags_92 },
    { 0x8d5a694b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fa6950, SUC_lexicon_tags_37 },
    { 0x93ba6951, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61a6954, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fa695b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48fa695d, SUC_lexicon_tags_33 },
    { 0x16ba695e, SUC_lexicon_tags_15 },
    { 0x9a9a695f, SUC_lexicon_tags_11 },
    { 0xb33a695f, SUC_lexicon_tags_552 },
    { 0xbffa6961, SUC_lexicon_tags_33 },
    { 0xd59a695d, SUC_lexicon_tags_43 },
    { 0x5f3a695d, SUC_lexicon_tags_15 },
    { 0xfd7a6964, SUC_lexicon_tags_68 },
    { 0x4fda6965, SUC_lexicon_tags_5 },
    { 0x50da6960, SUC_lexicon_tags_12 },
    { 0x9f3a6967, SUC_lexicon_tags_37 },
    { 0xacda6968, SUC_lexicon_tags_12 },
    { 0x2cda6969, SUC_lexicon_tags_252 },
    { 0x94fa696a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aba696f, SUC_lexicon_tags_14 },
    { 0xbe5a6970, SUC_lexicon_tags_35 },
    { 0x171a6970, SUC_lexicon_tags_15 },
    { 0x621a6972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3a6979, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x82fa697b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ba6985, SUC_lexicon_tags_38 },
    { 0x751a6985, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaada6988, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75a698a, SUC_lexicon_tags_33 },
    { 0x8c7a698b, SUC_lexicon_tags_11 },
    { 0x155a698c, SUC_lexicon_tags_17 },
    { 0x29fa698b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefda698f, SUC_lexicon_tags_11 },
    { 0xee5a6990, SUC_lexicon_tags_38 },
    { 0x473a6991, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fa6996, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7a699a, SUC_lexicon_tags_25 },
    { 0x811a699b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa699e, SUC_lexicon_tags_68 },
    { 0x5eba699f, SUC_lexicon_tags_55 },
    { 0xce9a69a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7a69a4, SUC_lexicon_tags_299 },
    { 0x55ba69a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0da69a7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x469a69aa, SUC_lexicon_tags_61 },
    { 0xfc7a69ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fa69ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3a69b1, SUC_lexicon_tags_38 },
    { 0xca5a69b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7a69c0, SUC_lexicon_tags_14 },
    { 0xf7a69c1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbeba69c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36fa69c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535a69cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465a69d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7a69d4, SUC_lexicon_tags_27 },
    { 0x24da69d4, SUC_lexicon_tags_19 },
    { 0xa7ba69d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22da69db, SUC_lexicon_tags_1 },
    { 0x809a69dc, SUC_lexicon_tags_25 },
    { 0xe57a69dd, SUC_lexicon_tags_12 },
    { 0x25fa69dd, SUC_lexicon_tags_17 },
    { 0x9b3a69de, SUC_lexicon_tags_68 },
    { 0x8bda69dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x567a69e2, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x243a69e4, SUC_lexicon_tags_30 },
    { 0xd33a69e5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1a69f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdda69f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97a69fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcefa69fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61a6a01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84da6a04, SUC_lexicon_tags_26 },
    { 0x3e5a6a05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25fa6a08, SUC_lexicon_tags_30 },
    { 0xfb3a6a08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bba6a0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941a6a10, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681a6a14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7fa6a16, SUC_lexicon_tags_11 },
    { 0xc7a6a16, SUC_lexicon_tags_6 },
    { 0x443a6a18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31fa6a1a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xd47a6a1c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39a6a2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ba6a39, SUC_lexicon_tags_37 },
    { 0x723a6a3a, SUC_lexicon_tags_25 },
    { 0x13ba6a3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdffa6a3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44da6a41, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x471a6a44, SUC_lexicon_tags_27 },
    { 0x85da6a45, SUC_lexicon_tags_2 },
    { 0x61ba6a45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad7a6a48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a1a6a4a, SUC_lexicon_tags_34 },
    { 0x71ba6a4b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dda6a4e, SUC_lexicon_tags_17 },
    { 0xab7a6a4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381a6a54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ba6a58, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe15a6a5a, SUC_lexicon_tags_11 },
    { 0x823a6a5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xea7a6a5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3a6a60, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x945a6a62, SUC_lexicon_tags_12 },
    { 0x75ba6a63, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fa6a66, SUC_lexicon_tags_63 },
    { 0x9d3a6a66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647a6a6b, SUC_lexicon_tags_236 },
    { 0x81a6a6c, SUC_lexicon_tags_27 },
    { 0x7fda6a6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55a6a70, SUC_lexicon_tags_29 },
    { 0x58da6a71, SUC_lexicon_tags_14 },
    { 0x1eba6a72, SUC_lexicon_tags_17 },
    { 0xd23a6a73, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x901a6a75, SUC_lexicon_tags_17 },
    { 0x9fba6a75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9a6a7a, SUC_lexicon_tags_87 },
    { 0x593a6a7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fa6a7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba3a6a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x315a6a82, SUC_lexicon_tags_1 },
    { 0x2eda6a83, SUC_lexicon_tags_66 },
    { 0x82fa6a83, SUC_lexicon_tags_11 },
    { 0xeb3a6a84, SUC_lexicon_tags_19 },
    { 0xe37a6a86, SUC_lexicon_tags_48 },
    { 0xf6ba6a85, SUC_lexicon_tags_25 },
    { 0x6c5a6a87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9a6a8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeba6a8e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9a6a93, SUC_lexicon_tags_38 },
    { 0xf2fa6a93, SUC_lexicon_tags_29 },
    { 0xf3a6a95, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1a6a98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81a6a9d, SUC_lexicon_tags_17 },
    { 0xf43a6a9d, SUC_lexicon_tags_47 },
    { 0xd25a6a9e, SUC_lexicon_tags_47 },
    { 0x339a6aa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf49a6aa2, SUC_lexicon_tags_21 },
    { 0xe71a6aa2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1da6aa8, SUC_lexicon_tags_38 },
    { 0xcb7a6aa8, SUC_lexicon_tags_1 },
    { 0xbe7a6aa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7a6aad, SUC_lexicon_tags_23 },
    { 0x8efa6aad, SUC_lexicon_tags_43 },
    { 0x8b5a6aaf, SUC_lexicon_tags_17 },
    { 0x125a6ab0, SUC_lexicon_tags_28 },
    { 0x5cfa6ab1, SUC_lexicon_tags_2 },
    { 0xfe7a6ab2, SUC_lexicon_tags_4 },
    { 0x523a6ab3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ba6ab8, SUC_lexicon_tags_82 },
    { 0x19da6ab8, SUC_lexicon_tags_25 },
    { 0x369a6aba, SUC_lexicon_tags_37 },
    { 0x415a6ab9, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545a6abf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7a6ac5, SUC_lexicon_tags_7 },
    { 0x837a6ac5, SUC_lexicon_tags_47 },
    { 0xa3ba6ac7, SUC_lexicon_tags_99 },
    { 0x4dfa6ac5, SUC_lexicon_tags_112 },
    { 0x813a6ac9, SUC_lexicon_tags_12 },
    { 0xfeda6aca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x409a6acc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99a6ad6, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cba6adb, SUC_lexicon_tags_9 },
    { 0xbfba6adc, SUC_lexicon_tags_17 },
    { 0xc55a6add, SUC_lexicon_tags_11 },
    { 0x1efa6ade, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57da6ae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ba6af0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5bda6af2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ba6af5, SUC_lexicon_tags_26 },
    { 0x8cda6af5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fa6afa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8b9a6afc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3a6b00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9a6b0a, SUC_lexicon_tags_28 },
    { 0xde3a6b0b, SUC_lexicon_tags_12 },
    { 0x277a6b0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28da6b11, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8e7a6b13, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x36ba6b15, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x195a6b17, SUC_lexicon_tags_46 },
    { 0x413a6b17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb7a6b1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985a6b1f, SUC_lexicon_tags_9 },
    { 0x1d3a6b20, SUC_lexicon_tags_15 },
    { 0x58da6b21, SUC_lexicon_tags_15 },
    { 0x94fa6b22, SUC_lexicon_tags_17 },
    { 0xf93a6b23, SUC_lexicon_tags_2 },
    { 0x6c7a6b1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359a6b29, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48da6b2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aba6b31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75fa6b33, SUC_lexicon_tags_48 },
    { 0x16ba6b34, SUC_lexicon_tags_19 },
    { 0xb07a6b34, SUC_lexicon_tags_12 },
    { 0x8f5a6b34, SUC_lexicon_tags_23 },
    { 0x2cba6b33, SUC_lexicon_tags_14 },
    { 0x947a6b38, SUC_lexicon_tags_32 },
    { 0x571a6b39, SUC_lexicon_tags_39 },
    { 0xb91a6b3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dda6b3e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff5a6b40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437a6b45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bba6b4a, SUC_lexicon_tags_16 },
    { 0x969a6b4b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19a6b4f, SUC_lexicon_tags_253 },
    { 0x11da6b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdffa6b52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77a6b56, SUC_lexicon_tags_27 },
    { 0xfefa6b57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ba6b5b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7dda6b5d, SUC_lexicon_tags_12 },
    { 0x2fba6b5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fa6b61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97fa6b63, SUC_lexicon_tags_38 },
    { 0xc05a6b64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511a6b6a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4dba6b6c, SUC_lexicon_tags_26 },
    { 0xf71a6b6c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe27a6b6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36ba6b71, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x34ba6b73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafa6b76, SUC_lexicon_tags_19 },
    { 0x469a6b76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445a6b7e, SUC_lexicon_tags_6 },
    { 0x3b3a6b7e, SUC_lexicon_tags_60 },
    { 0xbf3a6b80, SUC_lexicon_tags_15 },
    { 0xf7da6b80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05a6b8c, SUC_lexicon_tags_47 },
    { 0xed9a6b8c, SUC_lexicon_tags_12 },
    { 0x61da6b8e, SUC_lexicon_tags_7 },
    { 0xfb1a6b8f, SUC_lexicon_tags_0 },
    { 0x397a6b90, SUC_lexicon_tags_7 },
    { 0x68ba6b8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13a6b94, SUC_lexicon_tags_2 },
    { 0xe8fa6b95, SUC_lexicon_tags_11 },
    { 0x877a6b96, SUC_lexicon_tags_6 },
    { 0x1f7a6b96, SUC_lexicon_tags_19 },
    { 0xc75a6b94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3a6b9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe37a6b9e, SUC_lexicon_tags_77 },
    { 0x5b1a6b9f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51a6ba2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287a6baa, SUC_lexicon_tags_2 },
    { 0x3cba6bab, SUC_lexicon_tags_45 },
    { 0x581a6bac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x543a6bae, SUC_lexicon_tags_37 },
    { 0xf23a6bae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x66da6bb2, SUC_lexicon_tags_15 },
    { 0x5f9a6bb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0ba6bb5, SUC_lexicon_tags_11 },
    { 0xb9fa6bb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35a6bba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29da6bbe, SUC_lexicon_tags_11 },
    { 0xf29a6bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1a6bc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ba6bcc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x10da6bcf, SUC_lexicon_tags_41 },
    { 0x31a6bd0, SUC_lexicon_tags_59 },
    { 0x157a6bd0, SUC_lexicon_tags_16 },
    { 0xfd9a6bd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9a6bd7, SUC_lexicon_tags_163 },
    { 0x519a6bd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5a6bdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x33fa6be1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ba6be4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939a6beb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc5a6bed, SUC_lexicon_tags_37 },
    { 0x337a6bee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2d7a6bf0, SUC_lexicon_tags_49 },
    { 0x41ba6bf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19a6bf6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175a6bfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc21a6bfd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x36da6bff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x849a6c01, SUC_lexicon_tags_27 },
    { 0xcada6c01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x797a6c05, SUC_lexicon_tags_45 },
    { 0x23a6c06, SUC_lexicon_tags_27 },
    { 0xf9da6c07, SUC_lexicon_tags_25 },
    { 0x5d1a6c08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51a6c10, SUC_lexicon_tags_63 },
    { 0x2c3a6c11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1a6c17, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17da6c1c, SUC_lexicon_tags_49 },
    { 0x617a6c1c, SUC_lexicon_tags_1 },
    { 0xe97a6c1d, SUC_lexicon_tags_47 },
    { 0x637a6c1f, SUC_lexicon_tags_15 },
    { 0xd2ba6c1d, SUC_lexicon_tags_19 },
    { 0xeafa6c1d, SUC_lexicon_tags_7 },
    { 0x5dda6c1e, SUC_lexicon_tags_1 },
    { 0xdcba6c1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ada6c27, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb93a6c29, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed3a6c2b, SUC_lexicon_tags_37 },
    { 0x7f1a6c2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea9a6c2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfa6c31, SUC_lexicon_tags_61 },
    { 0x73da6c32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3a6c3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ba6c41, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcffa6c43, SUC_lexicon_tags_87 },
    { 0xeb7a6c44, SUC_lexicon_tags_39 },
    { 0x227a6c45, SUC_lexicon_tags_25 },
    { 0x681a6c45, SUC_lexicon_tags_47 },
    { 0x789a6c47, SUC_lexicon_tags_8 },
    { 0x173a6c48, SUC_lexicon_tags_65 },
    { 0xf1ba6c43, SUC_lexicon_tags_34 },
    { 0x967a6c4a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efa6c4e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd3da6c50, SUC_lexicon_tags_17 },
    { 0x68ba6c51, SUC_lexicon_tags_25 },
    { 0x743a6c52, SUC_lexicon_tags_11 },
    { 0xefda6c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541a6c5d, SUC_lexicon_tags_12 },
    { 0x9d9a6c5e, SUC_lexicon_tags_25 },
    { 0x21ba6c5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a1a6c61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ba6c65, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1a6c69, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffa6c6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ba6c6f, SUC_lexicon_tags_2 },
    { 0x77ba6c70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa99a6c72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5da6c75, SUC_lexicon_tags_9 },
    { 0xe91a6c76, SUC_lexicon_tags_87 },
    { 0x15da6c77, SUC_lexicon_tags_0 },
    { 0x885a6c78, SUC_lexicon_tags_1 },
    { 0xe3fa6c79, SUC_lexicon_tags_5 },
    { 0x9f7a6c78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ba6c7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92ba6c7f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x379a6c82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcffa6c84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd03a6c86, SUC_lexicon_tags_17 },
    { 0xaaba6c86, SUC_lexicon_tags_1 },
    { 0x5bba6c86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e9a6c8a, SUC_lexicon_tags_17 },
    { 0x201a6c8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6aba6c8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5a6c90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a6c95, SUC_lexicon_tags_11 },
    { 0x8e3a6c96, SUC_lexicon_tags_17 },
    { 0x4ba6c95, SUC_lexicon_tags_30 },
    { 0x907a6c98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9a5a6c9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x65fa6c9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ba6ca1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7da6ca4, SUC_lexicon_tags_17 },
    { 0xf49a6ca5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3ba6ca7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ba6cab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113a6cb1, SUC_lexicon_tags_77 },
    { 0x455a6cb2, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219a6cc1, SUC_lexicon_tags_37 },
    { 0x94fa6cc2, SUC_lexicon_tags_17 },
    { 0xed7a6cc2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1a6cc8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ba6cce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5da6cd3, SUC_lexicon_tags_17 },
    { 0x6bda6cd4, SUC_lexicon_tags_17 },
    { 0x573a6cd5, SUC_lexicon_tags_12 },
    { 0xc2da6cd5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fa6cda, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xbd9a6cdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27a6cdf, SUC_lexicon_tags_33 },
    { 0xb55a6cdf, SUC_lexicon_tags_85 },
    { 0xcffa6ce1, SUC_lexicon_tags_33 },
    { 0xadda6ce2, SUC_lexicon_tags_3 },
    { 0x241a6ce3, SUC_lexicon_tags_11 },
    { 0x5a1a6ce2, SUC_lexicon_tags_12 },
    { 0x93da6ce0, SUC_lexicon_tags_37 },
    { 0x92fa6ce4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355a6cea, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x725a6ced, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57da6cf1, SUC_lexicon_tags_11 },
    { 0xf55a6cf2, SUC_lexicon_tags_2 },
    { 0x71fa6cf3, SUC_lexicon_tags_17 },
    { 0x2cfa6cf4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x49a6cf6, SUC_lexicon_tags_7 },
    { 0x6bfa6cf7, SUC_lexicon_tags_37 },
    { 0x229a6cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a6cfe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83a6d01, SUC_lexicon_tags_367 },
    { 0, NULL },
    { 0, NULL },
    { 0x113a6d04, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45a6d0b, SUC_lexicon_tags_63 },
    { 0x4bba6d0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x691a6d0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87a6d10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfa6d15, SUC_lexicon_tags_17 },
    { 0x1dda6d16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8c7a6d18, SUC_lexicon_tags_25 },
    { 0xba7a6d18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x519a6d1b, SUC_lexicon_tags_7 },
    { 0x197a6d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde1a6d1e, SUC_lexicon_tags_17 },
    { 0x4a3a6d1f, SUC_lexicon_tags_25 },
    { 0x245a6d1f, SUC_lexicon_tags_25 },
    { 0x2cda6d1f, SUC_lexicon_tags_7 },
    { 0x9a9a6d22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213a6d27, SUC_lexicon_tags_15 },
    { 0xaba6d27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eba6d2c, SUC_lexicon_tags_47 },
    { 0xa6fa6d2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x45fa6d30, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6da6d39, SUC_lexicon_tags_201 },
    { 0x891a6d3a, SUC_lexicon_tags_14 },
    { 0x91da6d3b, SUC_lexicon_tags_11 },
    { 0xd81a6d3b, SUC_lexicon_tags_60 },
    { 0xf2ba6d3a, SUC_lexicon_tags_7 },
    { 0x75da6d3e, SUC_lexicon_tags_11 },
    { 0x2aba6d3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfa6d46, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ba6d4a, SUC_lexicon_tags_1109 },
    { 0, NULL },
    { 0x95da6d4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87a6d4f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x85da6d51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9a6d54, SUC_lexicon_tags_10 },
    { 0x55fa6d55, SUC_lexicon_tags_17 },
    { 0xc1a6d56, SUC_lexicon_tags_38 },
    { 0x20fa6d57, SUC_lexicon_tags_17 },
    { 0xd75a6d56, SUC_lexicon_tags_37 },
    { 0xca3a6d59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x23a6d5c, SUC_lexicon_tags_1 },
    { 0x807a6d5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dba6d61, SUC_lexicon_tags_25 },
    { 0x325a6d62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197a6d68, SUC_lexicon_tags_26 },
    { 0x291a6d69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7a6d6e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfa6d71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1a6d75, SUC_lexicon_tags_14 },
    { 0x241a6d76, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x82fa6d78, SUC_lexicon_tags_13 },
    { 0xbb9a6d79, SUC_lexicon_tags_25 },
    { 0xbd7a6d7a, SUC_lexicon_tags_14 },
    { 0x13a6d7b, SUC_lexicon_tags_15 },
    { 0x81fa6d79, SUC_lexicon_tags_49 },
    { 0x4f5a6d7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa6d80, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd2da6d82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6b9a6d84, SUC_lexicon_tags_37 },
    { 0x7d7a6d85, SUC_lexicon_tags_16 },
    { 0xe9a6d86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a6d89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d5a6d8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19a6d8e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x461a6d91, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ba6d99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ba6d9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eda6da5, SUC_lexicon_tags_34 },
    { 0x81ba6da6, SUC_lexicon_tags_37 },
    { 0x40da6da5, SUC_lexicon_tags_26 },
    { 0x301a6da8, SUC_lexicon_tags_11 },
    { 0xa1fa6da7, SUC_lexicon_tags_1 },
    { 0x2bba6da5, SUC_lexicon_tags_37 },
    { 0xa7da6daa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dba6db5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd7da6db7, SUC_lexicon_tags_15 },
    { 0xcbda6db8, SUC_lexicon_tags_11 },
    { 0x1f3a6db9, SUC_lexicon_tags_38 },
    { 0x99da6db9, SUC_lexicon_tags_45 },
    { 0xa75a6dbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231a6dc4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59a6dc7, SUC_lexicon_tags_11 },
    { 0xb35a6dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1a6dcf, SUC_lexicon_tags_68 },
    { 0x8c1a6dd0, SUC_lexicon_tags_216 },
    { 0x5d7a6dd1, SUC_lexicon_tags_11 },
    { 0x625a6dd2, SUC_lexicon_tags_11 },
    { 0xa7fa6dd3, SUC_lexicon_tags_15 },
    { 0x353a6dd4, SUC_lexicon_tags_7 },
    { 0x447a6dd1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x609a6dd8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x4da6dda, SUC_lexicon_tags_3 },
    { 0x8bda6ddb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbba6de1, SUC_lexicon_tags_33 },
    { 0x5b3a6de1, SUC_lexicon_tags_34 },
    { 0x2a1a6de3, SUC_lexicon_tags_19 },
    { 0xbb3a6de1, SUC_lexicon_tags_45 },
    { 0xfdfa6de5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93a6deb, SUC_lexicon_tags_19 },
    { 0xef9a6dec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90da6dee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x523a6df0, SUC_lexicon_tags_10 },
    { 0xb5ba6df0, SUC_lexicon_tags_17 },
    { 0x8b5a6df2, SUC_lexicon_tags_15 },
    { 0xbdfa6df0, SUC_lexicon_tags_12 },
    { 0x957a6df4, SUC_lexicon_tags_25 },
    { 0x5e5a6df4, SUC_lexicon_tags_33 },
    { 0xd6ba6df6, SUC_lexicon_tags_7 },
    { 0xa4da6df5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x263a6df9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915a6dff, SUC_lexicon_tags_27 },
    { 0x64fa6dff, SUC_lexicon_tags_12 },
    { 0xba9a6e01, SUC_lexicon_tags_37 },
    { 0x37ba6e02, SUC_lexicon_tags_1 },
    { 0x6a1a6e02, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x35da6e06, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1a6e09, SUC_lexicon_tags_43 },
    { 0x14fa6e0a, SUC_lexicon_tags_27 },
    { 0x78ba6e0b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8dba6e0d, SUC_lexicon_tags_15 },
    { 0x7c1a6e0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99fa6e10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53a6e14, SUC_lexicon_tags_8 },
    { 0xcb5a6e15, SUC_lexicon_tags_63 },
    { 0xe3a6e16, SUC_lexicon_tags_37 },
    { 0xcba6e17, SUC_lexicon_tags_37 },
    { 0x65ba6e18, SUC_lexicon_tags_0 },
    { 0x907a6e19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42da6e1b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da6e1f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x20fa6e21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305a6e25, SUC_lexicon_tags_1 },
    { 0xd09a6e26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51a6e29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587a6e2f, SUC_lexicon_tags_12 },
    { 0x1b9a6e30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7a6e37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcda6e3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd75a6e3e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb87a6e40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2fa6e42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83a6e46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x975a6e48, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xfb3a6e4a, SUC_lexicon_tags_43 },
    { 0x5da6e4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75ba6e4d, SUC_lexicon_tags_68 },
    { 0x4dda6e4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3a6e51, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39a6e58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ba6e5c, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fa6e62, SUC_lexicon_tags_0 },
    { 0xeefa6e63, SUC_lexicon_tags_11 },
    { 0xfa9a6e64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x383a6e67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd9a6e69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe7a6e6b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcc7a6e6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ba6e70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3a6e73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec9a6e75, SUC_lexicon_tags_29 },
    { 0x1e3a6e75, SUC_lexicon_tags_12 },
    { 0x6f9a6e75, SUC_lexicon_tags_12 },
    { 0x183a6e78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fa6e7b, SUC_lexicon_tags_49 },
    { 0x23da6e7c, SUC_lexicon_tags_29 },
    { 0x2e9a6e7d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x55da6e7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ba6e83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x227a6e85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d3a6e87, SUC_lexicon_tags_60 },
    { 0x63a6e87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad9a6e8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffa6e91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd07a6e93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fa6e99, SUC_lexicon_tags_25 },
    { 0x69da6e9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ba6e9d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xca9a6e9f, SUC_lexicon_tags_0 },
    { 0x4aba6ea0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7a6ea3, SUC_lexicon_tags_26 },
    { 0x233a6ea4, SUC_lexicon_tags_17 },
    { 0xb4ba6ea5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd73a6ea7, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x1e7a6ea9, SUC_lexicon_tags_7 },
    { 0x4cfa6eaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafa6eae, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xcb5a6eb0, SUC_lexicon_tags_68 },
    { 0x115a6eb0, SUC_lexicon_tags_11 },
    { 0x349a6eb2, SUC_lexicon_tags_1 },
    { 0xe67a6eb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x421a6eb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7a6eb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fa6ec3, SUC_lexicon_tags_11 },
    { 0x925a6ec4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc67a6ec6, SUC_lexicon_tags_7 },
    { 0x7d7a6ec7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29a6eca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ba6ecd, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba6ed3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x947a6ed5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xde7a6ed7, SUC_lexicon_tags_6 },
    { 0x725a6ed8, SUC_lexicon_tags_37 },
    { 0x227a6ed9, SUC_lexicon_tags_15 },
    { 0x47a6ed8, SUC_lexicon_tags_38 },
    { 0xc5a6edb, SUC_lexicon_tags_7 },
    { 0xc81a6edc, SUC_lexicon_tags_38 },
    { 0xff3a6edc, SUC_lexicon_tags_2 },
    { 0x219a6edc, SUC_lexicon_tags_0 },
    { 0x35ba6edf, SUC_lexicon_tags_45 },
    { 0x1c3a6ee0, SUC_lexicon_tags_63 },
    { 0xc69a6edd, SUC_lexicon_tags_63 },
    { 0xb7ba6ee2, SUC_lexicon_tags_7 },
    { 0xe9a6ee3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efa6eea, SUC_lexicon_tags_2 },
    { 0x505a6eea, SUC_lexicon_tags_0 },
    { 0x49a6eea, SUC_lexicon_tags_8 },
    { 0x389a6eed, SUC_lexicon_tags_9 },
    { 0x185a6eed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59fa6ef1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ba6ef4, SUC_lexicon_tags_31 },
    { 0xf83a6ef5, SUC_lexicon_tags_64 },
    { 0x639a6ef6, SUC_lexicon_tags_41 },
    { 0x64ba6ef7, SUC_lexicon_tags_37 },
    { 0xdf3a6ef6, SUC_lexicon_tags_26 },
    { 0xcbda6ef9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725a6f05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fa6f09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65da6f0d, SUC_lexicon_tags_47 },
    { 0xf93a6f0e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553a6f12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9a6f1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5a6f25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd37a6f28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67a6f2a, SUC_lexicon_tags_11 },
    { 0xb3ba6f2a, SUC_lexicon_tags_11 },
    { 0x705a6f2a, SUC_lexicon_tags_7 },
    { 0xc17a6f2d, SUC_lexicon_tags_11 },
    { 0x691a6f2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc5a6f30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75a6f37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7da6f3a, SUC_lexicon_tags_64 },
    { 0x675a6f3b, SUC_lexicon_tags_25 },
    { 0xed7a6f3a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc99a6f3e, SUC_lexicon_tags_0 },
    { 0x6c5a6f3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c5a6f41, SUC_lexicon_tags_25 },
    { 0x7c5a6f42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ba6f45, SUC_lexicon_tags_201 },
    { 0xc03a6f45, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xabba6f49, SUC_lexicon_tags_34 },
    { 0xd29a6f4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7a6f4e, SUC_lexicon_tags_19 },
    { 0x975a6f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafa6f52, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa6f55, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xad7a6f57, SUC_lexicon_tags_43 },
    { 0x3a3a6f58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea5a6f5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfa6f5e, SUC_lexicon_tags_25 },
    { 0x8fda6f5f, SUC_lexicon_tags_38 },
    { 0x27da6f5f, SUC_lexicon_tags_30 },
    { 0x37fa6f5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17a6f63, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253a6f67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x59fa6f69, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfa6f73, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0da6f77, SUC_lexicon_tags_38 },
    { 0xce3a6f77, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17a6f7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fa6f83, SUC_lexicon_tags_2 },
    { 0xf7a6f83, SUC_lexicon_tags_12 },
    { 0x3d1a6f84, SUC_lexicon_tags_11 },
    { 0x787a6f86, SUC_lexicon_tags_9 },
    { 0x441a6f86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedda6f92, SUC_lexicon_tags_2 },
    { 0x7cfa6f93, SUC_lexicon_tags_45 },
    { 0x3ffa6f94, SUC_lexicon_tags_19 },
    { 0x4c7a6f92, SUC_lexicon_tags_11 },
    { 0xdd1a6f96, SUC_lexicon_tags_38 },
    { 0xc71a6f97, SUC_lexicon_tags_29 },
    { 0x2ada6f97, SUC_lexicon_tags_70 },
    { 0x5e9a6f99, SUC_lexicon_tags_48 },
    { 0xa0ba6f9a, SUC_lexicon_tags_171 },
    { 0x735a6f9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3a6fa7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2da6faa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x415a6fad, SUC_lexicon_tags_8 },
    { 0x90fa6fae, SUC_lexicon_tags_29 },
    { 0x993a6fae, SUC_lexicon_tags_13 },
    { 0xacda6fb0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11a6fb4, SUC_lexicon_tags_23 },
    { 0x9e1a6fb5, SUC_lexicon_tags_15 },
    { 0x21a6fb6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x85da6fb9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33a6fbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x601a6fc0, SUC_lexicon_tags_30 },
    { 0x525a6fc1, SUC_lexicon_tags_9 },
    { 0xb03a6fc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xacda6fc4, SUC_lexicon_tags_27 },
    { 0xee7a6fc4, SUC_lexicon_tags_17 },
    { 0xe73a6fc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfa6fca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7a6fd0, SUC_lexicon_tags_49 },
    { 0x1d3a6fd1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8da6fd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba6fd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ba6fdb, SUC_lexicon_tags_15 },
    { 0x7ba6fdc, SUC_lexicon_tags_27 },
    { 0x47ba6fdd, SUC_lexicon_tags_12 },
    { 0x891a6fdd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fa6fe2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x959a6fe4, SUC_lexicon_tags_38 },
    { 0xedfa6fe5, SUC_lexicon_tags_1 },
    { 0x355a6fe4, SUC_lexicon_tags_46 },
    { 0xfcfa6fe7, SUC_lexicon_tags_45 },
    { 0xf9da6fe8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfa6feb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593a6ff2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x937a6ff5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x10da6ff8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11a6ffc, SUC_lexicon_tags_28 },
    { 0x517a6ffc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7a7002, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcda7004, SUC_lexicon_tags_21 },
    { 0x615a7005, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfa7008, SUC_lexicon_tags_55 },
    { 0x40fa7009, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57a700e, SUC_lexicon_tags_45 },
    { 0x20da700f, SUC_lexicon_tags_15 },
    { 0xe1da700f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfeba7012, SUC_lexicon_tags_4 },
    { 0x15ba7012, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ada701a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fa701d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb0fa701f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x741a7022, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ba7029, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd4ba702b, SUC_lexicon_tags_1 },
    { 0x7c9a702c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ba702f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61a7036, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34da703c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x615a703e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439a7042, SUC_lexicon_tags_43 },
    { 0x78da7042, SUC_lexicon_tags_2 },
    { 0xddfa7044, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811a704a, SUC_lexicon_tags_14 },
    { 0xc77a704a, SUC_lexicon_tags_77 },
    { 0xc1a704c, SUC_lexicon_tags_17 },
    { 0x635a704d, SUC_lexicon_tags_57 },
    { 0x3c1a704a, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcda7054, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eba7057, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x461a7059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bba705e, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0x551a7060, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7a7063, SUC_lexicon_tags_19 },
    { 0xd87a7063, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637a706b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe6ba706d, SUC_lexicon_tags_11 },
    { 0x917a706e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x881a7071, SUC_lexicon_tags_7 },
    { 0x311a7072, SUC_lexicon_tags_11 },
    { 0x9e7a7073, SUC_lexicon_tags_39 },
    { 0x9dba7071, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9a7078, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x623a707a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fa707d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507a7082, SUC_lexicon_tags_37 },
    { 0xc4ba7083, SUC_lexicon_tags_77 },
    { 0x39fa7083, SUC_lexicon_tags_26 },
    { 0xb6da7084, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207a708a, SUC_lexicon_tags_37 },
    { 0xf9fa708b, SUC_lexicon_tags_40 },
    { 0x981a708b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fa708f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x473a7092, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x62da7094, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ba7099, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50fa709b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51a70a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653a70a5, SUC_lexicon_tags_25 },
    { 0xffda70a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a70aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537a70ae, SUC_lexicon_tags_26 },
    { 0xbc7a70ae, SUC_lexicon_tags_7 },
    { 0x2a5a70b0, SUC_lexicon_tags_46 },
    { 0x961a70b1, SUC_lexicon_tags_43 },
    { 0x7a7a70b1, SUC_lexicon_tags_12 },
    { 0xc25a70b3, SUC_lexicon_tags_11 },
    { 0x2c7a70b4, SUC_lexicon_tags_1 },
    { 0xe0ba70ae, SUC_lexicon_tags_17 },
    { 0x7dba70af, SUC_lexicon_tags_33 },
    { 0x8ada70b7, SUC_lexicon_tags_2 },
    { 0xaafa70b8, SUC_lexicon_tags_2 },
    { 0x6d7a70b9, SUC_lexicon_tags_2 },
    { 0xedba70ba, SUC_lexicon_tags_63 },
    { 0x479a70bb, SUC_lexicon_tags_61 },
    { 0xfafa70bb, SUC_lexicon_tags_107 },
    { 0x82da70bd, SUC_lexicon_tags_55 },
    { 0xab9a70bb, SUC_lexicon_tags_32 },
    { 0xb1a70bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bba70c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dda70c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9a70cd, SUC_lexicon_tags_27 },
    { 0x445a70ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643a70d3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683a70d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559a70de, SUC_lexicon_tags_7 },
    { 0xa2da70de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2dfa70e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x725a70e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a1a70e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fa70e8, SUC_lexicon_tags_12 },
    { 0xe9da70e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7da70ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec1a70ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11a70f1, SUC_lexicon_tags_15 },
    { 0x48da70f2, SUC_lexicon_tags_22 },
    { 0xb7fa70f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc11a70f5, SUC_lexicon_tags_14 },
    { 0x1e3a70f6, SUC_lexicon_tags_12 },
    { 0xb83a70f7, SUC_lexicon_tags_11 },
    { 0xbe1a70f8, SUC_lexicon_tags_17 },
    { 0xccba70f8, SUC_lexicon_tags_6 },
    { 0xeb9a70fa, SUC_lexicon_tags_11 },
    { 0xf39a70fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1a70ff, SUC_lexicon_tags_12 },
    { 0x489a7100, SUC_lexicon_tags_47 },
    { 0x4b3a7101, SUC_lexicon_tags_16 },
    { 0xd5fa7102, SUC_lexicon_tags_11 },
    { 0x8cfa7103, SUC_lexicon_tags_47 },
    { 0xa19a7104, SUC_lexicon_tags_39 },
    { 0x725a7105, SUC_lexicon_tags_37 },
    { 0x1d5a7105, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ba7111, SUC_lexicon_tags_11 },
    { 0x711a7111, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925a7116, SUC_lexicon_tags_2 },
    { 0x31ba7117, SUC_lexicon_tags_37 },
    { 0xac9a7118, SUC_lexicon_tags_25 },
    { 0xcfa7119, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3a711e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x34fa7120, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275a7124, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b1a7126, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe87a7128, SUC_lexicon_tags_1 },
    { 0x28da7129, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983a712e, SUC_lexicon_tags_7 },
    { 0xd83a712f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadda7131, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2da7134, SUC_lexicon_tags_56 },
    { 0x9fa7135, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57ba7137, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf63a7139, SUC_lexicon_tags_99 },
    { 0xd27a713a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe0ba713c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83a713f, SUC_lexicon_tags_10 },
    { 0x469a7140, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x187a7143, SUC_lexicon_tags_37 },
    { 0xcc3a7144, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x729a7146, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf91a7148, SUC_lexicon_tags_49 },
    { 0x13a7149, SUC_lexicon_tags_1 },
    { 0xf87a714a, SUC_lexicon_tags_1 },
    { 0x259a714a, SUC_lexicon_tags_26 },
    { 0xd9fa714b, SUC_lexicon_tags_30 },
    { 0x5bda714c, SUC_lexicon_tags_33 },
    { 0xe13a714e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9a7152, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3a7158, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91a715d, SUC_lexicon_tags_37 },
    { 0x953a715e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x943a7160, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf15a7162, SUC_lexicon_tags_17 },
    { 0x8afa7163, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x23ba7165, SUC_lexicon_tags_17 },
    { 0x6b9a7166, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5a7171, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x839a7174, SUC_lexicon_tags_7 },
    { 0xbd1a7174, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9afa7177, SUC_lexicon_tags_12 },
    { 0xe5ba7178, SUC_lexicon_tags_11 },
    { 0x787a7179, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a717d, SUC_lexicon_tags_25 },
    { 0x9f1a717d, SUC_lexicon_tags_87 },
    { 0x9e3a717e, SUC_lexicon_tags_37 },
    { 0x9e7a717f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fa7184, SUC_lexicon_tags_1 },
    { 0x447a7184, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x511a7187, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75a718b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x849a718d, SUC_lexicon_tags_2 },
    { 0xd47a718d, SUC_lexicon_tags_2 },
    { 0x523a718f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dda7193, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x859a7196, SUC_lexicon_tags_26 },
    { 0xdeda7197, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747a719e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa65a71a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xde5a71a2, SUC_lexicon_tags_8 },
    { 0xa43a71a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3cfa71a5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd23a71a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fa71ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x199a71ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17a71b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda71b2, SUC_lexicon_tags_1 },
    { 0xb9da71b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57a71b6, SUC_lexicon_tags_25 },
    { 0x7d1a71b6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7a71c1, SUC_lexicon_tags_40 },
    { 0xb19a71c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a71ca, SUC_lexicon_tags_1 },
    { 0xb3fa71cb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679a71d1, SUC_lexicon_tags_11 },
    { 0xdd5a71d1, SUC_lexicon_tags_63 },
    { 0xb99a71d3, SUC_lexicon_tags_899 },
    { 0xcc5a71d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeda71d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cda71de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5d1a71e0, SUC_lexicon_tags_38 },
    { 0x77ba71e0, SUC_lexicon_tags_37 },
    { 0xbcfa71e0, SUC_lexicon_tags_12 },
    { 0xccda71e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5d5a71e5, SUC_lexicon_tags_63 },
    { 0x251a71e6, SUC_lexicon_tags_14 },
    { 0x24ba71e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfcfa71e9, SUC_lexicon_tags_15 },
    { 0x32ba71ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9a71ed, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x85a71ef, SUC_lexicon_tags_17 },
    { 0x6eba71f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3a71f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x83ba71f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfa71f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83a7200, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7a7205, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7a720c, SUC_lexicon_tags_7 },
    { 0x73da720c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2fa720f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99a7211, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fda7214, SUC_lexicon_tags_17 },
    { 0x1dfa7215, SUC_lexicon_tags_10 },
    { 0x4a9a7216, SUC_lexicon_tags_9 },
    { 0x8c1a7217, SUC_lexicon_tags_2 },
    { 0x4b5a7216, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc49a721a, SUC_lexicon_tags_21 },
    { 0x3a721b, SUC_lexicon_tags_37 },
    { 0x329a721b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb53a721e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c5a7220, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd37a7222, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfa7228, SUC_lexicon_tags_11 },
    { 0xd53a7229, SUC_lexicon_tags_47 },
    { 0xbb9a722a, SUC_lexicon_tags_2 },
    { 0x315a722b, SUC_lexicon_tags_31 },
    { 0x79fa7228, SUC_lexicon_tags_17 },
    { 0x16da722d, SUC_lexicon_tags_41 },
    { 0x213a722c, SUC_lexicon_tags_0 },
    { 0x52da722e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbda7232, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb03a7234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae5a7236, SUC_lexicon_tags_37 },
    { 0x7d7a7237, SUC_lexicon_tags_15 },
    { 0x5eda7237, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5a723b, SUC_lexicon_tags_37 },
    { 0x263a723b, SUC_lexicon_tags_7 },
    { 0x4f1a723b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55a7240, SUC_lexicon_tags_12 },
    { 0x53ba7241, SUC_lexicon_tags_16 },
    { 0xa55a7242, SUC_lexicon_tags_2 },
    { 0x3a7a7243, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad3a7245, SUC_lexicon_tags_2 },
    { 0x535a7245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x849a7248, SUC_lexicon_tags_2 },
    { 0xce7a7249, SUC_lexicon_tags_48 },
    { 0xb13a724a, SUC_lexicon_tags_14 },
    { 0x795a724b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe75a724d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615a7254, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffa7257, SUC_lexicon_tags_17 },
    { 0xd6ba7258, SUC_lexicon_tags_11 },
    { 0x931a7257, SUC_lexicon_tags_17 },
    { 0x277a725a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x579a725c, SUC_lexicon_tags_8 },
    { 0xda1a725d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9a7261, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25a7265, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3a726a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487a726f, SUC_lexicon_tags_21 },
    { 0x939a7270, SUC_lexicon_tags_1 },
    { 0xaeda7270, SUC_lexicon_tags_63 },
    { 0xb51a7271, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afa7275, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751a727d, SUC_lexicon_tags_1 },
    { 0xddfa727e, SUC_lexicon_tags_0 },
    { 0xda1a727f, SUC_lexicon_tags_11 },
    { 0xb79a727f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a7283, SUC_lexicon_tags_17 },
    { 0xae7a7284, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dba728b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7a7291, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x517a7293, SUC_lexicon_tags_15 },
    { 0x93ba7294, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7a1a7296, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49ba7298, SUC_lexicon_tags_15 },
    { 0xba7a7299, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x58fa729b, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfa729e, SUC_lexicon_tags_27 },
    { 0xe2ba729f, SUC_lexicon_tags_168 },
    { 0x1aba72a0, SUC_lexicon_tags_17 },
    { 0xd83a72a0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1a72a6, SUC_lexicon_tags_25 },
    { 0x9ffa72a6, SUC_lexicon_tags_27 },
    { 0xaf1a72a8, SUC_lexicon_tags_33 },
    { 0xb3ba72a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79a72ab, SUC_lexicon_tags_27 },
    { 0x98da72ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97a72b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a72b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1a72b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf53a72b9, SUC_lexicon_tags_17 },
    { 0x481a72ba, SUC_lexicon_tags_65 },
    { 0x18fa72ba, SUC_lexicon_tags_9 },
    { 0x329a72ba, SUC_lexicon_tags_11 },
    { 0x737a72bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1a72c1, SUC_lexicon_tags_87 },
    { 0xffa72c2, SUC_lexicon_tags_148 },
    { 0xbc1a72c1, SUC_lexicon_tags_55 },
    { 0x4c3a72c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9a72c7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x34ba72c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x955a72cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa51a72cd, SUC_lexicon_tags_47 },
    { 0x839a72ce, SUC_lexicon_tags_33 },
    { 0xe5a72cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159a72d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98da72d5, SUC_lexicon_tags_66 },
    { 0xae5a72d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x777a72d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3a72dc, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7a72e0, SUC_lexicon_tags_1 },
    { 0xea9a72e1, SUC_lexicon_tags_12 },
    { 0xaeda72e2, SUC_lexicon_tags_1 },
    { 0xa43a72e3, SUC_lexicon_tags_1 },
    { 0xee1a72e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc3a72e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539a72eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1a72f1, SUC_lexicon_tags_37 },
    { 0x98fa72f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba72f8, SUC_lexicon_tags_14 },
    { 0xd87a72f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x135a72fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8c1a72fe, SUC_lexicon_tags_2 },
    { 0xcdba72fe, SUC_lexicon_tags_28 },
    { 0x141a7300, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eba7305, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfdba7307, SUC_lexicon_tags_17 },
    { 0x54ba7307, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21a730d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1da730f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacba7317, SUC_lexicon_tags_1 },
    { 0xb29a7318, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x845a731a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83a731d, SUC_lexicon_tags_38 },
    { 0x8dfa731e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf95a7320, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x30da7323, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6dda7325, SUC_lexicon_tags_17 },
    { 0x35fa7326, SUC_lexicon_tags_25 },
    { 0x379a7327, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc7a7329, SUC_lexicon_tags_23 },
    { 0x515a7329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dba732d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x84ba732f, SUC_lexicon_tags_7 },
    { 0x2a5a7330, SUC_lexicon_tags_60 },
    { 0xc63a732f, SUC_lexicon_tags_29 },
    { 0xab1a7332, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981a7338, SUC_lexicon_tags_37 },
    { 0xa45a7338, SUC_lexicon_tags_37 },
    { 0x32ba7339, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95a7344, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdfa7346, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4fda7348, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3ba734a, SUC_lexicon_tags_12 },
    { 0x3aba734b, SUC_lexicon_tags_27 },
    { 0x23ba734b, SUC_lexicon_tags_5 },
    { 0x577a734c, SUC_lexicon_tags_11 },
    { 0x803a734b, SUC_lexicon_tags_7 },
    { 0xc17a734f, SUC_lexicon_tags_43 },
    { 0x627a734a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x157a7353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8ba7355, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81a7358, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2da7360, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x813a7362, SUC_lexicon_tags_1 },
    { 0xff7a7362, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x21fa7366, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x49ba7368, SUC_lexicon_tags_17 },
    { 0xf29a7369, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7a736c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8ba736e, SUC_lexicon_tags_26 },
    { 0xff5a736f, SUC_lexicon_tags_11 },
    { 0x5da7370, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfa7373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ba7376, SUC_lexicon_tags_2 },
    { 0x13a7377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9a737f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ba7382, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73a7387, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291a738e, SUC_lexicon_tags_2 },
    { 0x7eba738e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x72fa7392, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309a7399, SUC_lexicon_tags_2 },
    { 0x715a7399, SUC_lexicon_tags_2 },
    { 0x149a739b, SUC_lexicon_tags_1 },
    { 0x375a739c, SUC_lexicon_tags_23 },
    { 0x143a739b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51ba739f, SUC_lexicon_tags_27 },
    { 0xc49a73a0, SUC_lexicon_tags_15 },
    { 0x129a739f, SUC_lexicon_tags_25 },
    { 0xcb3a73a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dba73a5, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ba73aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x855a73ac, SUC_lexicon_tags_37 },
    { 0x983a73ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafba73b3, SUC_lexicon_tags_1416 },
    { 0xe93a73b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x245a73b6, SUC_lexicon_tags_11 },
    { 0x2f5a73b7, SUC_lexicon_tags_0 },
    { 0x61ba73b6, SUC_lexicon_tags_0 },
    { 0x17fa73b7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7a9a73bb, SUC_lexicon_tags_38 },
    { 0xc9a73bc, SUC_lexicon_tags_16 },
    { 0x221a73bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701a73c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7a73ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5a73cf, SUC_lexicon_tags_16 },
    { 0x523a73cf, SUC_lexicon_tags_18 },
    { 0x3f3a73d1, SUC_lexicon_tags_45 },
    { 0xdaba73d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81a73d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x681a73d7, SUC_lexicon_tags_37 },
    { 0x60fa73d8, SUC_lexicon_tags_9 },
    { 0x539a73d8, SUC_lexicon_tags_59 },
    { 0x2b1a73da, SUC_lexicon_tags_2 },
    { 0x8eba73db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf7a73dd, SUC_lexicon_tags_10 },
    { 0xdb1a73dd, SUC_lexicon_tags_17 },
    { 0x9b7a73df, SUC_lexicon_tags_7 },
    { 0x471a73dd, SUC_lexicon_tags_1 },
    { 0x421a73de, SUC_lexicon_tags_12 },
    { 0xa3fa73e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b9a73e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9a73e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9eda73ea, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3a73ed, SUC_lexicon_tags_17 },
    { 0x3da73ee, SUC_lexicon_tags_1 },
    { 0xe47a73ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x94da73f1, SUC_lexicon_tags_25 },
    { 0x3b7a73f1, SUC_lexicon_tags_2 },
    { 0x425a73f3, SUC_lexicon_tags_11 },
    { 0x293a73f4, SUC_lexicon_tags_8 },
    { 0x51fa73f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55a73fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x729a73fc, SUC_lexicon_tags_46 },
    { 0x287a73fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1dfa73ff, SUC_lexicon_tags_1 },
    { 0xe6ba7400, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53a7403, SUC_lexicon_tags_15 },
    { 0xbfa7403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67ba7406, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51da740a, SUC_lexicon_tags_0 },
    { 0x861a740b, SUC_lexicon_tags_37 },
    { 0x117a740b, SUC_lexicon_tags_9 },
    { 0x5d3a740d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aba7411, SUC_lexicon_tags_15 },
    { 0x24fa7412, SUC_lexicon_tags_0 },
    { 0x6cba7413, SUC_lexicon_tags_23 },
    { 0xfa7412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x643a7416, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf57a7418, SUC_lexicon_tags_82 },
    { 0x637a7419, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dba741c, SUC_lexicon_tags_11 },
    { 0x7bba741d, SUC_lexicon_tags_21 },
    { 0xb53a741c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1a7424, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3e7a7426, SUC_lexicon_tags_47 },
    { 0xc7da7427, SUC_lexicon_tags_7 },
    { 0xb35a7426, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6aba742a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x86fa742c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187a7432, SUC_lexicon_tags_31 },
    { 0x191a7432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5fa7435, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967a7439, SUC_lexicon_tags_37 },
    { 0x1c7a743a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x413a743d, SUC_lexicon_tags_47 },
    { 0x2ffa743d, SUC_lexicon_tags_37 },
    { 0x415a743d, SUC_lexicon_tags_19 },
    { 0xda7a7440, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8da7444, SUC_lexicon_tags_34 },
    { 0x12fa7445, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1a744d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39a7453, SUC_lexicon_tags_16 },
    { 0x6f5a7453, SUC_lexicon_tags_7 },
    { 0xf39a7455, SUC_lexicon_tags_30 },
    { 0xfd3a7453, SUC_lexicon_tags_65 },
    { 0x2f1a7457, SUC_lexicon_tags_2 },
    { 0xd8fa7458, SUC_lexicon_tags_37 },
    { 0x4e3a7459, SUC_lexicon_tags_82 },
    { 0x3a9a7458, SUC_lexicon_tags_17 },
    { 0x191a7456, SUC_lexicon_tags_12 },
    { 0x13da745c, SUC_lexicon_tags_65 },
    { 0xac3a745d, SUC_lexicon_tags_29 },
    { 0x683a745e, SUC_lexicon_tags_17 },
    { 0x435a745e, SUC_lexicon_tags_0 },
    { 0x917a745e, SUC_lexicon_tags_9 },
    { 0x683a7458, SUC_lexicon_tags_1 },
    { 0xf91a7460, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x863a7464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfa7467, SUC_lexicon_tags_0 },
    { 0x7eba7467, SUC_lexicon_tags_60 },
    { 0x947a7469, SUC_lexicon_tags_43 },
    { 0x81da746a, SUC_lexicon_tags_1 },
    { 0x889a746b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x177a746d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5a7472, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x105a7475, SUC_lexicon_tags_11 },
    { 0x361a7476, SUC_lexicon_tags_77 },
    { 0xc11a7477, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5a747b, SUC_lexicon_tags_248 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fda7480, SUC_lexicon_tags_55 },
    { 0x549a7480, SUC_lexicon_tags_14 },
    { 0x8c1a7482, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x43a7485, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bba748e, SUC_lexicon_tags_49 },
    { 0xa37a748f, SUC_lexicon_tags_37 },
    { 0xa13a7490, SUC_lexicon_tags_31 },
    { 0x525a748f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9a7495, SUC_lexicon_tags_11 },
    { 0x3fa7496, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ba749a, SUC_lexicon_tags_16 },
    { 0x463a749b, SUC_lexicon_tags_1 },
    { 0xda3a749a, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83da74a5, SUC_lexicon_tags_17 },
    { 0x875a74a6, SUC_lexicon_tags_17 },
    { 0x92ba74a7, SUC_lexicon_tags_25 },
    { 0x631a74a8, SUC_lexicon_tags_14 },
    { 0x45ba74a8, SUC_lexicon_tags_17 },
    { 0x6d7a74a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3a74ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4f9a74af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719a74b4, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0x25fa74b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a74b9, SUC_lexicon_tags_4 },
    { 0x7fba74ba, SUC_lexicon_tags_8 },
    { 0x93ba74bb, SUC_lexicon_tags_17 },
    { 0x74fa74b9, SUC_lexicon_tags_46 },
    { 0x127a74bd, SUC_lexicon_tags_37 },
    { 0xf4fa74be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4bba74c0, SUC_lexicon_tags_26 },
    { 0x3d9a74c1, SUC_lexicon_tags_37 },
    { 0x1e3a74c2, SUC_lexicon_tags_25 },
    { 0x1efa74c3, SUC_lexicon_tags_17 },
    { 0x925a74c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fa74c7, SUC_lexicon_tags_10 },
    { 0xdfa74c7, SUC_lexicon_tags_39 },
    { 0x1d7a74c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83a74d3, SUC_lexicon_tags_38 },
    { 0x7fba74d4, SUC_lexicon_tags_17 },
    { 0x28ba74d5, SUC_lexicon_tags_37 },
    { 0x871a74d6, SUC_lexicon_tags_92 },
    { 0xb7a74d5, SUC_lexicon_tags_23 },
    { 0xb09a74d3, SUC_lexicon_tags_25 },
    { 0x1fba74d9, SUC_lexicon_tags_15 },
    { 0x97fa74d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dda74e0, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da74e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe4da74e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf6da74e9, SUC_lexicon_tags_14 },
    { 0xb3da74ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cba74f3, SUC_lexicon_tags_25 },
    { 0x56fa74f4, SUC_lexicon_tags_11 },
    { 0x27da74f3, SUC_lexicon_tags_38 },
    { 0x2e3a74f4, SUC_lexicon_tags_19 },
    { 0x141a74f6, SUC_lexicon_tags_37 },
    { 0xd09a74f8, SUC_lexicon_tags_25 },
    { 0x495a74f4, SUC_lexicon_tags_4 },
    { 0x82ba74f4, SUC_lexicon_tags_17 },
    { 0xed5a74f3, SUC_lexicon_tags_45 },
    { 0x29ba74f6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf7fa74fe, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2da7502, SUC_lexicon_tags_33 },
    { 0x43da7503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fa7505, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87a750b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb5a750d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x843a750f, SUC_lexicon_tags_17 },
    { 0xa53a750f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6ffa7512, SUC_lexicon_tags_25 },
    { 0x4afa7513, SUC_lexicon_tags_2 },
    { 0x8dba7514, SUC_lexicon_tags_35 },
    { 0xb77a7513, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x79da7517, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a3a7519, SUC_lexicon_tags_17 },
    { 0xea7a751a, SUC_lexicon_tags_37 },
    { 0xc3a7519, SUC_lexicon_tags_23 },
    { 0xf79a751b, SUC_lexicon_tags_6 },
    { 0xfbda751c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9a7521, SUC_lexicon_tags_17 },
    { 0xc1fa7522, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1fa7524, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7a752e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbeba7530, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1a7537, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fba753a, SUC_lexicon_tags_19 },
    { 0x7e7a753b, SUC_lexicon_tags_68 },
    { 0xaaba753a, SUC_lexicon_tags_8 },
    { 0x77a753d, SUC_lexicon_tags_15 },
    { 0x64da753e, SUC_lexicon_tags_11 },
    { 0xeda753e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87a7542, SUC_lexicon_tags_15 },
    { 0xe9a7543, SUC_lexicon_tags_33 },
    { 0x2d9a7544, SUC_lexicon_tags_141 },
    { 0x6b1a7543, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fa7549, SUC_lexicon_tags_25 },
    { 0x60fa7549, SUC_lexicon_tags_28 },
    { 0xdd9a754a, SUC_lexicon_tags_23 },
    { 0x225a754b, SUC_lexicon_tags_28 },
    { 0x6f5a754d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83a7554, SUC_lexicon_tags_19 },
    { 0xda7a7555, SUC_lexicon_tags_27 },
    { 0xbcda7556, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x143a7559, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f1a755b, SUC_lexicon_tags_12 },
    { 0xe1a755c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79a7560, SUC_lexicon_tags_25 },
    { 0xf2fa7561, SUC_lexicon_tags_11 },
    { 0xa4ba7562, SUC_lexicon_tags_19 },
    { 0xf65a7563, SUC_lexicon_tags_91 },
    { 0xca5a7564, SUC_lexicon_tags_17 },
    { 0x4cba7565, SUC_lexicon_tags_12 },
    { 0x98da7562, SUC_lexicon_tags_15 },
    { 0x8a9a7567, SUC_lexicon_tags_47 },
    { 0xe5fa7564, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefa756b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13a7572, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fa7576, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7da757d, SUC_lexicon_tags_7 },
    { 0xe57a757d, SUC_lexicon_tags_29 },
    { 0x9a9a757d, SUC_lexicon_tags_1 },
    { 0xc55a7580, SUC_lexicon_tags_37 },
    { 0x545a757f, SUC_lexicon_tags_37 },
    { 0x1a3a7582, SUC_lexicon_tags_12 },
    { 0x13a757e, SUC_lexicon_tags_9 },
    { 0xfdda757e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431a7588, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x529a758a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aba7594, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ba7596, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ba759e, SUC_lexicon_tags_179 },
    { 0x205a759f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03a75a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3a75ab, SUC_lexicon_tags_20 },
    { 0x2f9a75ac, SUC_lexicon_tags_19 },
    { 0x483a75ad, SUC_lexicon_tags_7 },
    { 0x6bda75ae, SUC_lexicon_tags_27 },
    { 0x351a75af, SUC_lexicon_tags_19 },
    { 0x71a75af, SUC_lexicon_tags_25 },
    { 0x195a75b1, SUC_lexicon_tags_14 },
    { 0xf61a75b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbba75b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19a75ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9a75be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691a75c3, SUC_lexicon_tags_25 },
    { 0x333a75c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149a75c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa19a75ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba75cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01a75d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423a75d7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x669a75d9, SUC_lexicon_tags_25 },
    { 0xe6ba75d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203a75df, SUC_lexicon_tags_7 },
    { 0xa79a75e0, SUC_lexicon_tags_37 },
    { 0x1cfa75e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4ba75e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ba75e9, SUC_lexicon_tags_82 },
    { 0x96fa75ea, SUC_lexicon_tags_38 },
    { 0x7d9a75eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x983a75ed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x84ba75ef, SUC_lexicon_tags_11 },
    { 0x4d9a75ef, SUC_lexicon_tags_6 },
    { 0xd5fa75ef, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x993a75f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9aba75f6, SUC_lexicon_tags_12 },
    { 0xb21a75f6, SUC_lexicon_tags_2 },
    { 0xf55a75f8, SUC_lexicon_tags_129 },
    { 0x7eba75f9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3a75fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa1a75fe, SUC_lexicon_tags_27 },
    { 0x9b5a75ff, SUC_lexicon_tags_2 },
    { 0x155a7600, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853a760c, SUC_lexicon_tags_7 },
    { 0xcafa760d, SUC_lexicon_tags_15 },
    { 0xd05a760c, SUC_lexicon_tags_46 },
    { 0xab9a760f, SUC_lexicon_tags_10 },
    { 0xf93a7610, SUC_lexicon_tags_7 },
    { 0xbb3a7611, SUC_lexicon_tags_23 },
    { 0x8b9a7612, SUC_lexicon_tags_65 },
    { 0x701a760f, SUC_lexicon_tags_80 },
    { 0xb7a7614, SUC_lexicon_tags_47 },
    { 0xd3ba7614, SUC_lexicon_tags_10 },
    { 0x3c7a760f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5a7619, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7eda761b, SUC_lexicon_tags_32 },
    { 0xf5fa761c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fa7620, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2fa7622, SUC_lexicon_tags_4 },
    { 0x941a7622, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x159a7626, SUC_lexicon_tags_15 },
    { 0xa4fa7627, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fa762a, SUC_lexicon_tags_43 },
    { 0x983a762b, SUC_lexicon_tags_35 },
    { 0x8b9a762c, SUC_lexicon_tags_53 },
    { 0x661a762d, SUC_lexicon_tags_11 },
    { 0x4e1a762b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711a7633, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7a7637, SUC_lexicon_tags_9 },
    { 0x29fa7637, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3a763f, SUC_lexicon_tags_47 },
    { 0x745a7640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a7642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x441a7644, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d1a7646, SUC_lexicon_tags_264 },
    { 0x8a5a7646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9a764c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfa7652, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89a7658, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115a765d, SUC_lexicon_tags_0 },
    { 0x1bda765e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a7662, SUC_lexicon_tags_7 },
    { 0x9e3a7663, SUC_lexicon_tags_4 },
    { 0xf01a7664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55a7667, SUC_lexicon_tags_7 },
    { 0x93fa7667, SUC_lexicon_tags_31 },
    { 0x60ba7669, SUC_lexicon_tags_8 },
    { 0x15ba766a, SUC_lexicon_tags_2 },
    { 0xa99a7669, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x767a766d, SUC_lexicon_tags_12 },
    { 0x89fa766d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2da7675, SUC_lexicon_tags_7 },
    { 0xc7da7676, SUC_lexicon_tags_2 },
    { 0xb19a7677, SUC_lexicon_tags_17 },
    { 0x85a7678, SUC_lexicon_tags_17 },
    { 0x2c9a7677, SUC_lexicon_tags_45 },
    { 0xaefa767a, SUC_lexicon_tags_70 },
    { 0x317a767a, SUC_lexicon_tags_30 },
    { 0x163a767b, SUC_lexicon_tags_37 },
    { 0x1f9a7677, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb6ba767f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5a7683, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67a7686, SUC_lexicon_tags_31 },
    { 0x56da7687, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23a768d, SUC_lexicon_tags_37 },
    { 0x569a768e, SUC_lexicon_tags_7 },
    { 0xe11a768f, SUC_lexicon_tags_25 },
    { 0xb49a7690, SUC_lexicon_tags_17 },
    { 0x2f1a7691, SUC_lexicon_tags_37 },
    { 0xac1a7692, SUC_lexicon_tags_16 },
    { 0x92ba7693, SUC_lexicon_tags_12 },
    { 0xb21a7692, SUC_lexicon_tags_9 },
    { 0xee7a7691, SUC_lexicon_tags_182 },
    { 0xfc5a7693, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x973a7699, SUC_lexicon_tags_33 },
    { 0xf3ba769a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7a76a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x68ba76a2, SUC_lexicon_tags_7 },
    { 0x5ba76a3, SUC_lexicon_tags_28 },
    { 0x4f9a76a4, SUC_lexicon_tags_12 },
    { 0xff7a76a5, SUC_lexicon_tags_65 },
    { 0x43da76a4, SUC_lexicon_tags_66 },
    { 0xaa5a76a4, SUC_lexicon_tags_1 },
    { 0x57ba76a7, SUC_lexicon_tags_1 },
    { 0xe2da76a9, SUC_lexicon_tags_37 },
    { 0x83da76aa, SUC_lexicon_tags_2 },
    { 0xa5da76a2, SUC_lexicon_tags_37 },
    { 0x2ba76a5, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0xa97a76ae, SUC_lexicon_tags_15 },
    { 0xaeba76af, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x821a76b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9c9a76b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe47a76b6, SUC_lexicon_tags_11 },
    { 0x45a76b7, SUC_lexicon_tags_2 },
    { 0xfd9a76b7, SUC_lexicon_tags_62 },
    { 0x597a76b8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x87ba76bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecda76bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfba76cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7a76d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fa76d9, SUC_lexicon_tags_47 },
    { 0xbf3a76da, SUC_lexicon_tags_2 },
    { 0xbb1a76db, SUC_lexicon_tags_17 },
    { 0x787a76da, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cba76df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8da76e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1c3a76e4, SUC_lexicon_tags_60 },
    { 0xd27a76e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb7a76e7, SUC_lexicon_tags_65 },
    { 0xc37a76e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x86fa76ea, SUC_lexicon_tags_12 },
    { 0x24da76ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27a76ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387a76f2, SUC_lexicon_tags_42 },
    { 0x471a76f3, SUC_lexicon_tags_25 },
    { 0xec1a76f4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bba76fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361a76ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975a7703, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd2ba7705, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd8fa7707, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xafda7709, SUC_lexicon_tags_1 },
    { 0x383a770a, SUC_lexicon_tags_65 },
    { 0xef9a770b, SUC_lexicon_tags_33 },
    { 0xb87a770b, SUC_lexicon_tags_25 },
    { 0x65ba770b, SUC_lexicon_tags_23 },
    { 0x959a770b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fda7710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x795a7713, SUC_lexicon_tags_15 },
    { 0xaa1a7714, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc9a7716, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1a771a, SUC_lexicon_tags_15 },
    { 0x513a771a, SUC_lexicon_tags_10 },
    { 0x78ba771c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ba7720, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd61a7722, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691a7726, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3a772d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e7a772f, SUC_lexicon_tags_23 },
    { 0x6dfa772f, SUC_lexicon_tags_63 },
    { 0xda7a7730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31da7733, SUC_lexicon_tags_11 },
    { 0x91fa7733, SUC_lexicon_tags_2 },
    { 0xe5da7735, SUC_lexicon_tags_48 },
    { 0x85a7735, SUC_lexicon_tags_37 },
    { 0xe2da7737, SUC_lexicon_tags_12 },
    { 0x657a7735, SUC_lexicon_tags_7 },
    { 0xb01a7739, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x339a773c, SUC_lexicon_tags_26 },
    { 0x737a773d, SUC_lexicon_tags_35 },
    { 0xd47a773e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9a7741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1a7744, SUC_lexicon_tags_8 },
    { 0x10ba7745, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeba774c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3a7752, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827a775a, SUC_lexicon_tags_8 },
    { 0x4d7a775b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6d1a775d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59a7761, SUC_lexicon_tags_25 },
    { 0xe6ba7761, SUC_lexicon_tags_47 },
    { 0x95ba7763, SUC_lexicon_tags_17 },
    { 0xb7fa7764, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afa7767, SUC_lexicon_tags_25 },
    { 0x3fda7768, SUC_lexicon_tags_14 },
    { 0x57da7768, SUC_lexicon_tags_11 },
    { 0xb9fa776a, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fba776d, SUC_lexicon_tags_28 },
    { 0x6aba776e, SUC_lexicon_tags_48 },
    { 0xb13a776d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3a7776, SUC_lexicon_tags_25 },
    { 0x5b1a7777, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eba777f, SUC_lexicon_tags_31 },
    { 0x26fa7780, SUC_lexicon_tags_17 },
    { 0x989a7781, SUC_lexicon_tags_45 },
    { 0x63ba7781, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fa7789, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53a7791, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecba7795, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5a779b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda77a0, SUC_lexicon_tags_11 },
    { 0x27a77a0, SUC_lexicon_tags_49 },
    { 0x527a77a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835a77a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ba77b2, SUC_lexicon_tags_12 },
    { 0xedba77b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b3a77b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x631a77b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4da77bb, SUC_lexicon_tags_17 },
    { 0x9c7a77bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fba77c0, SUC_lexicon_tags_17 },
    { 0x81fa77c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997a77c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5a77ca, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299a77cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xec7a77d1, SUC_lexicon_tags_10 },
    { 0xf57a77d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37a77d8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf05a77da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27a77e1, SUC_lexicon_tags_25 },
    { 0x9eba77e2, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x789a77e4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fa77e7, SUC_lexicon_tags_37 },
    { 0x979a77e8, SUC_lexicon_tags_12 },
    { 0x221a77e9, SUC_lexicon_tags_25 },
    { 0xc57a77ea, SUC_lexicon_tags_59 },
    { 0x4dba77ea, SUC_lexicon_tags_80 },
    { 0x483a77eb, SUC_lexicon_tags_53 },
    { 0xa4da77e7, SUC_lexicon_tags_15 },
    { 0xb6da77eb, SUC_lexicon_tags_11 },
    { 0xac3a77e8, SUC_lexicon_tags_0 },
    { 0x807a77eb, SUC_lexicon_tags_15 },
    { 0x3e3a77ee, SUC_lexicon_tags_53 },
    { 0x85ba77e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80ba77f4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbba77f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efa77f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfa77fc, SUC_lexicon_tags_7 },
    { 0x3fa77fc, SUC_lexicon_tags_7 },
    { 0xcbba77fe, SUC_lexicon_tags_65 },
    { 0x359a77ff, SUC_lexicon_tags_82 },
    { 0x29ba77fe, SUC_lexicon_tags_31 },
    { 0x88ba77fd, SUC_lexicon_tags_11 },
    { 0x7c9a7800, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dda7807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17fa7809, SUC_lexicon_tags_12 },
    { 0x347a780a, SUC_lexicon_tags_17 },
    { 0xb3fa780b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x21fa780d, SUC_lexicon_tags_15 },
    { 0x297a780e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9a7812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa7815, SUC_lexicon_tags_26 },
    { 0x423a7815, SUC_lexicon_tags_47 },
    { 0x5b3a7817, SUC_lexicon_tags_27 },
    { 0x60da7815, SUC_lexicon_tags_0 },
    { 0x309a7819, SUC_lexicon_tags_11 },
    { 0x1e9a7819, SUC_lexicon_tags_27 },
    { 0xc1fa7819, SUC_lexicon_tags_27 },
    { 0xc8fa781b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fa7821, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x997a7823, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3a7827, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3a782b, SUC_lexicon_tags_15 },
    { 0xfcfa782c, SUC_lexicon_tags_60 },
    { 0x765a782c, SUC_lexicon_tags_63 },
    { 0x36ba782c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x175a7831, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5bba7833, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495a7837, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2a7a7839, SUC_lexicon_tags_19 },
    { 0x4dfa783a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x965a783c, SUC_lexicon_tags_19 },
    { 0x1f3a783c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5a7842, SUC_lexicon_tags_8 },
    { 0xf77a7843, SUC_lexicon_tags_2 },
    { 0xd29a7842, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5cda7846, SUC_lexicon_tags_1 },
    { 0x185a7847, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29a784d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7a7850, SUC_lexicon_tags_12 },
    { 0xfe9a7850, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x163a7854, SUC_lexicon_tags_45 },
    { 0x429a7855, SUC_lexicon_tags_47 },
    { 0xd99a7854, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153a785a, SUC_lexicon_tags_15 },
    { 0xe65a785b, SUC_lexicon_tags_15 },
    { 0xeeba785c, SUC_lexicon_tags_60 },
    { 0xeb3a785d, SUC_lexicon_tags_65 },
    { 0xa29a785e, SUC_lexicon_tags_9 },
    { 0xeb1a785e, SUC_lexicon_tags_25 },
    { 0xf05a785a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2ba7862, SUC_lexicon_tags_46 },
    { 0xbcda7863, SUC_lexicon_tags_23 },
    { 0x6bfa7864, SUC_lexicon_tags_1 },
    { 0x48fa7865, SUC_lexicon_tags_17 },
    { 0xcdba7865, SUC_lexicon_tags_11 },
    { 0x497a7867, SUC_lexicon_tags_39 },
    { 0xb13a7865, SUC_lexicon_tags_8 },
    { 0x87ba7869, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29a786c, SUC_lexicon_tags_11 },
    { 0x1c9a786d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3da7870, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb39a7872, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x723a7874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40da7878, SUC_lexicon_tags_15 },
    { 0x197a7878, SUC_lexicon_tags_7 },
    { 0x70fa787a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ba7880, SUC_lexicon_tags_11 },
    { 0x685a7880, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffa7884, SUC_lexicon_tags_26 },
    { 0x14fa7885, SUC_lexicon_tags_2 },
    { 0xb97a7886, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1a788c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7a7890, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afa7896, SUC_lexicon_tags_19 },
    { 0x52fa7897, SUC_lexicon_tags_7 },
    { 0x2f5a7897, SUC_lexicon_tags_1 },
    { 0x88da7899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3a789c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5a78a0, SUC_lexicon_tags_2 },
    { 0xf11a78a1, SUC_lexicon_tags_40 },
    { 0x369a78a2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1da78aa, SUC_lexicon_tags_63 },
    { 0xaa5a78aa, SUC_lexicon_tags_1 },
    { 0xfd1a78ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2fa78ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297a78b6, SUC_lexicon_tags_17 },
    { 0xa2fa78b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23a78bd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3a78c0, SUC_lexicon_tags_17 },
    { 0x7eba78c0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x58da78c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891a78ce, SUC_lexicon_tags_11 },
    { 0x4b3a78ce, SUC_lexicon_tags_23 },
    { 0xe53a78d0, SUC_lexicon_tags_11 },
    { 0xeba78ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7a78d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x783a78d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x63da78d9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x263a78dc, SUC_lexicon_tags_57 },
    { 0xde1a78dc, SUC_lexicon_tags_15 },
    { 0x9ffa78de, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cba78e5, SUC_lexicon_tags_26 },
    { 0xa9da78e6, SUC_lexicon_tags_2 },
    { 0x307a78e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9f1a78e9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe3ba78eb, SUC_lexicon_tags_14 },
    { 0x973a78eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x665a78ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efa78f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bda7900, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaada7902, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37a7906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x395a7909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5a7910, SUC_lexicon_tags_43 },
    { 0x2cba7911, SUC_lexicon_tags_45 },
    { 0xc81a7910, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765a7916, SUC_lexicon_tags_31 },
    { 0xf93a7917, SUC_lexicon_tags_0 },
    { 0xff9a7918, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x251a791b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb69a791d, SUC_lexicon_tags_6 },
    { 0xc13a791e, SUC_lexicon_tags_45 },
    { 0xba1a791e, SUC_lexicon_tags_84 },
    { 0xae1a7920, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7a7923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9a792d, SUC_lexicon_tags_66 },
    { 0xc9da792e, SUC_lexicon_tags_27 },
    { 0x35fa792f, SUC_lexicon_tags_82 },
    { 0x307a7930, SUC_lexicon_tags_66 },
    { 0xaa3a792f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb2fa7933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebda7937, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1aba7939, SUC_lexicon_tags_6 },
    { 0xa11a793a, SUC_lexicon_tags_2 },
    { 0xf71a793b, SUC_lexicon_tags_124 },
    { 0x31a793b, SUC_lexicon_tags_75 },
    { 0x117a793a, SUC_lexicon_tags_0 },
    { 0x977a7939, SUC_lexicon_tags_2 },
    { 0xda1a793f, SUC_lexicon_tags_63 },
    { 0xee1a7940, SUC_lexicon_tags_27 },
    { 0xcbfa7941, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3da7943, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901a7948, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997a7950, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3a7954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd5a7956, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x423a7959, SUC_lexicon_tags_16 },
    { 0xadda795a, SUC_lexicon_tags_47 },
    { 0xc0fa795b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643a7961, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5cba7963, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7a7967, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x88da7969, SUC_lexicon_tags_25 },
    { 0x6d1a7969, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbafa796c, SUC_lexicon_tags_296 },
    { 0x32da796c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9a7972, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xffa7974, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfa797a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bba797f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7a7987, SUC_lexicon_tags_15 },
    { 0x5a9a7988, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69a798b, SUC_lexicon_tags_11 },
    { 0x215a798b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x891a798f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a7991, SUC_lexicon_tags_29 },
    { 0x755a7992, SUC_lexicon_tags_7 },
    { 0x34da7992, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09a7997, SUC_lexicon_tags_16 },
    { 0xf59a7998, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacda79a2, SUC_lexicon_tags_23 },
    { 0x83a79a3, SUC_lexicon_tags_8 },
    { 0x305a79a2, SUC_lexicon_tags_25 },
    { 0x4b1a79a5, SUC_lexicon_tags_64 },
    { 0xbdba79a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2da79a8, SUC_lexicon_tags_37 },
    { 0x2cda79a9, SUC_lexicon_tags_25 },
    { 0x43da79aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31a79ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x925a79b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfa79b3, SUC_lexicon_tags_11 },
    { 0x60ba79b3, SUC_lexicon_tags_29 },
    { 0x26da79b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95ba79b7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x5e9a79b9, SUC_lexicon_tags_7 },
    { 0x949a79ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13a79bf, SUC_lexicon_tags_2 },
    { 0x931a79bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x173a79c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbda79c5, SUC_lexicon_tags_17 },
    { 0x6c7a79c6, SUC_lexicon_tags_17 },
    { 0xc6fa79c7, SUC_lexicon_tags_42 },
    { 0x3a79c6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97a79d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb1a79d5, SUC_lexicon_tags_45 },
    { 0x495a79d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x689a79d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97a79e2, SUC_lexicon_tags_19 },
    { 0x429a79e2, SUC_lexicon_tags_33 },
    { 0x80fa79e4, SUC_lexicon_tags_1 },
    { 0x5eba79e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565a79ea, SUC_lexicon_tags_18 },
    { 0x615a79eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57da79ef, SUC_lexicon_tags_5 },
    { 0xcada79ef, SUC_lexicon_tags_2 },
    { 0x777a79f1, SUC_lexicon_tags_87 },
    { 0xba5a79f2, SUC_lexicon_tags_35 },
    { 0x3c1a79f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x321a79f5, SUC_lexicon_tags_25 },
    { 0xe71a79f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe57a79f8, SUC_lexicon_tags_1 },
    { 0x9bfa79f9, SUC_lexicon_tags_16 },
    { 0x92fa79f9, SUC_lexicon_tags_3 },
    { 0x4bba79fb, SUC_lexicon_tags_25 },
    { 0xcb7a79fb, SUC_lexicon_tags_2 },
    { 0xac1a79fc, SUC_lexicon_tags_4 },
    { 0xe9a79fe, SUC_lexicon_tags_17 },
    { 0x609a79ff, SUC_lexicon_tags_359 },
    { 0xa6fa7a00, SUC_lexicon_tags_66 },
    { 0x71da7a00, SUC_lexicon_tags_12 },
    { 0x1f7a7a00, SUC_lexicon_tags_11 },
    { 0x5efa7a03, SUC_lexicon_tags_14 },
    { 0xf81a7a04, SUC_lexicon_tags_27 },
    { 0xc6fa7a03, SUC_lexicon_tags_174 },
    { 0x6e1a7a04, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fa7a0c, SUC_lexicon_tags_11 },
    { 0xdda7a0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x461a7a0f, SUC_lexicon_tags_25 },
    { 0xb6fa7a10, SUC_lexicon_tags_15 },
    { 0x949a7a11, SUC_lexicon_tags_8 },
    { 0xec5a7a11, SUC_lexicon_tags_37 },
    { 0x1d1a7a11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3a7a16, SUC_lexicon_tags_2 },
    { 0xab9a7a17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x923a7a1a, SUC_lexicon_tags_6 },
    { 0xef5a7a1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53a7a1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfa7a23, SUC_lexicon_tags_30 },
    { 0xa47a7a24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59da7a27, SUC_lexicon_tags_476 },
    { 0x803a7a27, SUC_lexicon_tags_37 },
    { 0x79da7a27, SUC_lexicon_tags_33 },
    { 0x70da7a2a, SUC_lexicon_tags_0 },
    { 0xd85a7a2b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bba7a2e, SUC_lexicon_tags_21 },
    { 0x349a7a2f, SUC_lexicon_tags_17 },
    { 0xc67a7a30, SUC_lexicon_tags_10 },
    { 0x15ba7a31, SUC_lexicon_tags_31 },
    { 0xdf5a7a31, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181a7a3a, SUC_lexicon_tags_53 },
    { 0x947a7a3b, SUC_lexicon_tags_49 },
    { 0xf5da7a3b, SUC_lexicon_tags_10 },
    { 0xce7a7a3c, SUC_lexicon_tags_8 },
    { 0xaf3a7a3e, SUC_lexicon_tags_1 },
    { 0xe7da7a3d, SUC_lexicon_tags_6 },
    { 0xc8ba7a40, SUC_lexicon_tags_17 },
    { 0x309a7a40, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x515a7a44, SUC_lexicon_tags_29 },
    { 0x7d3a7a45, SUC_lexicon_tags_43 },
    { 0x849a7a44, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fa7a4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43a7a4e, SUC_lexicon_tags_62 },
    { 0x52da7a4e, SUC_lexicon_tags_11 },
    { 0x3cfa7a50, SUC_lexicon_tags_29 },
    { 0xc6da7a51, SUC_lexicon_tags_15 },
    { 0x435a7a51, SUC_lexicon_tags_25 },
    { 0xc1da7a53, SUC_lexicon_tags_37 },
    { 0xbbba7a4e, SUC_lexicon_tags_375 },
    { 0xc61a7a51, SUC_lexicon_tags_8 },
    { 0xb6ba7a56, SUC_lexicon_tags_2 },
    { 0xeb3a7a51, SUC_lexicon_tags_37 },
    { 0x7efa7a53, SUC_lexicon_tags_26 },
    { 0x3ada7a59, SUC_lexicon_tags_47 },
    { 0x135a7a5a, SUC_lexicon_tags_13 },
    { 0x829a7a5b, SUC_lexicon_tags_7 },
    { 0x58fa7a5c, SUC_lexicon_tags_3 },
    { 0x9ffa7a5c, SUC_lexicon_tags_15 },
    { 0xe5da7a59, SUC_lexicon_tags_12 },
    { 0xcffa7a5b, SUC_lexicon_tags_27 },
    { 0xde3a7a60, SUC_lexicon_tags_49 },
    { 0xb1da7a61, SUC_lexicon_tags_31 },
    { 0x3bda7a5c, SUC_lexicon_tags_43 },
    { 0x399a7a63, SUC_lexicon_tags_87 },
    { 0xba9a7a64, SUC_lexicon_tags_37 },
    { 0x331a7a50, SUC_lexicon_tags_12 },
    { 0x439a7a5a, SUC_lexicon_tags_2 },
    { 0x793a7a5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5afa7a69, SUC_lexicon_tags_113 },
    { 0xa53a7a6a, SUC_lexicon_tags_12 },
    { 0x57fa7a6b, SUC_lexicon_tags_25 },
    { 0x673a7a6b, SUC_lexicon_tags_17 },
    { 0xadfa7a6d, SUC_lexicon_tags_6 },
    { 0xf29a7a6e, SUC_lexicon_tags_2 },
    { 0xc45a7a6f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dda7a7c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x415a7a7f, SUC_lexicon_tags_25 },
    { 0x44da7a7f, SUC_lexicon_tags_39 },
    { 0x6ffa7a7f, SUC_lexicon_tags_68 },
    { 0x7dfa7a81, SUC_lexicon_tags_37 },
    { 0xc65a7a83, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xc59a7a85, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65a7a88, SUC_lexicon_tags_7 },
    { 0x137a7a89, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xdc3a7a8b, SUC_lexicon_tags_37 },
    { 0xda5a7a8c, SUC_lexicon_tags_11 },
    { 0x4fda7a8b, SUC_lexicon_tags_17 },
    { 0xa0da7a8e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x93fa7a90, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51a7a95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ba7a98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x929a7a9b, SUC_lexicon_tags_2 },
    { 0x9e5a7a9c, SUC_lexicon_tags_44 },
    { 0xd33a7a9d, SUC_lexicon_tags_12 },
    { 0x7e9a7a9b, SUC_lexicon_tags_98 },
    { 0x211a7a9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2da7aa1, SUC_lexicon_tags_17 },
    { 0x951a7aa2, SUC_lexicon_tags_28 },
    { 0xf21a7aa2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc41a7aa5, SUC_lexicon_tags_15 },
    { 0xf6fa7aa6, SUC_lexicon_tags_15 },
    { 0x5afa7aa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61a7aaf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117a7ab4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda3a7ab6, SUC_lexicon_tags_12 },
    { 0x547a7ab7, SUC_lexicon_tags_2 },
    { 0xbe5a7ab7, SUC_lexicon_tags_37 },
    { 0xb2ba7ab9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55a7abe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x215a7ac1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1a7acb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdba7ad1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fda7ad4, SUC_lexicon_tags_30 },
    { 0x2eba7ad5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ba7ada, SUC_lexicon_tags_11 },
    { 0x4a3a7adb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05a7ade, SUC_lexicon_tags_45 },
    { 0xe6ba7ade, SUC_lexicon_tags_19 },
    { 0x13fa7adf, SUC_lexicon_tags_15 },
    { 0x2dfa7ae0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95a7ae5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x121a7ae7, SUC_lexicon_tags_11 },
    { 0x949a7ae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dfa7aea, SUC_lexicon_tags_2 },
    { 0x34da7aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1a7aee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7a7af0, SUC_lexicon_tags_37 },
    { 0xb05a7af1, SUC_lexicon_tags_60 },
    { 0x53da7af2, SUC_lexicon_tags_45 },
    { 0xb61a7af0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3e1a7af5, SUC_lexicon_tags_146 },
    { 0xf41a7af5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd41a7af8, SUC_lexicon_tags_27 },
    { 0xc47a7af9, SUC_lexicon_tags_42 },
    { 0xda1a7af9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaa3a7afc, SUC_lexicon_tags_63 },
    { 0xa3a7afc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56da7aff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741a7b05, SUC_lexicon_tags_63 },
    { 0xd83a7b06, SUC_lexicon_tags_161 },
    { 0x203a7b06, SUC_lexicon_tags_107 },
    { 0xff3a7b05, SUC_lexicon_tags_7 },
    { 0xe81a7b09, SUC_lexicon_tags_11 },
    { 0x73fa7b0a, SUC_lexicon_tags_28 },
    { 0xbfa7b0b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ba7b0e, SUC_lexicon_tags_68 },
    { 0xae1a7b0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3a7b13, SUC_lexicon_tags_37 },
    { 0xfa7a7b13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5aba7b18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfa7b1b, SUC_lexicon_tags_7 },
    { 0xbeba7b1b, SUC_lexicon_tags_2 },
    { 0x3eda7b1c, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ba7b22, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83da7b2e, SUC_lexicon_tags_60 },
    { 0x333a7b2e, SUC_lexicon_tags_60 },
    { 0x5ffa7b30, SUC_lexicon_tags_27 },
    { 0x835a7b2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5a7b37, SUC_lexicon_tags_15 },
    { 0xc99a7b38, SUC_lexicon_tags_1 },
    { 0x9e9a7b39, SUC_lexicon_tags_7 },
    { 0x3cda7b3a, SUC_lexicon_tags_11 },
    { 0x20da7b3b, SUC_lexicon_tags_25 },
    { 0xd69a7b3c, SUC_lexicon_tags_47 },
    { 0x441a7b3a, SUC_lexicon_tags_35 },
    { 0xf07a7b3e, SUC_lexicon_tags_31 },
    { 0xabba7b3e, SUC_lexicon_tags_57 },
    { 0xc65a7b3f, SUC_lexicon_tags_9 },
    { 0x985a7b41, SUC_lexicon_tags_65 },
    { 0xbefa7b42, SUC_lexicon_tags_19 },
    { 0x537a7b3f, SUC_lexicon_tags_11 },
    { 0x90da7b44, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8da7b4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887a7b4f, SUC_lexicon_tags_12 },
    { 0xf75a7b4f, SUC_lexicon_tags_11 },
    { 0x73ba7b51, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x82da7b53, SUC_lexicon_tags_30 },
    { 0x6eda7b54, SUC_lexicon_tags_61 },
    { 0xeba7b55, SUC_lexicon_tags_15 },
    { 0x239a7b53, SUC_lexicon_tags_14 },
    { 0x1fda7b55, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15a7b5b, SUC_lexicon_tags_13 },
    { 0x16ba7b5c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9a7b60, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa7b64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb7a7b66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x359a7b69, SUC_lexicon_tags_12 },
    { 0x811a7b69, SUC_lexicon_tags_53 },
    { 0xe27a7b6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51a7b72, SUC_lexicon_tags_7 },
    { 0x7cba7b73, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17a7b7d, SUC_lexicon_tags_60 },
    { 0x3c3a7b7e, SUC_lexicon_tags_2 },
    { 0x4f7a7b7f, SUC_lexicon_tags_2 },
    { 0x459a7b7d, SUC_lexicon_tags_14 },
    { 0x887a7b7e, SUC_lexicon_tags_29 },
    { 0x233a7b80, SUC_lexicon_tags_29 },
    { 0x3aba7b7d, SUC_lexicon_tags_66 },
    { 0x173a7b7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x82da7b86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57a7b90, SUC_lexicon_tags_62 },
    { 0x89fa7b91, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfa1a7b93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9a7b9b, SUC_lexicon_tags_66 },
    { 0x5a1a7b9c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1a7ba2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xccda7ba4, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x49da7ba6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x517a7ba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc27a7baa, SUC_lexicon_tags_39 },
    { 0xf41a7baa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491a7bb7, SUC_lexicon_tags_17 },
    { 0xb43a7bb8, SUC_lexicon_tags_2 },
    { 0x465a7bb8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cba7bbc, SUC_lexicon_tags_47 },
    { 0xf1fa7bbd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fa7bc2, SUC_lexicon_tags_79 },
    { 0xf07a7bc3, SUC_lexicon_tags_25 },
    { 0x283a7bc3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x751a7bc7, SUC_lexicon_tags_17 },
    { 0xbaba7bc8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa49a7bca, SUC_lexicon_tags_11 },
    { 0xf89a7bca, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a7bd6, SUC_lexicon_tags_19 },
    { 0xa89a7bd7, SUC_lexicon_tags_2 },
    { 0x66da7bd7, SUC_lexicon_tags_11 },
    { 0x871a7bd9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x797a7bdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7a7bde, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x257a7be0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5eda7be2, SUC_lexicon_tags_57 },
    { 0x321a7be2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74ba7be5, SUC_lexicon_tags_104 },
    { 0xc6da7be6, SUC_lexicon_tags_101 },
    { 0x6b9a7be7, SUC_lexicon_tags_2 },
    { 0x439a7be8, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9a7beb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf05a7bed, SUC_lexicon_tags_30 },
    { 0xa75a7bee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5a7bf2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xae1a7bf4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbf5a7bf6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6d5a7bf8, SUC_lexicon_tags_45 },
    { 0x11da7bf8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7a7bfc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x253a7bff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15a7c05, SUC_lexicon_tags_37 },
    { 0x6f5a7c06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x425a7c09, SUC_lexicon_tags_7 },
    { 0x541a7c0a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x429a7c0c, SUC_lexicon_tags_1 },
    { 0x781a7c0d, SUC_lexicon_tags_7 },
    { 0xceda7c0c, SUC_lexicon_tags_7 },
    { 0xb01a7c0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ba7c16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbba7c18, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf7a7c1a, SUC_lexicon_tags_11 },
    { 0x1dda7c1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7fa7c1d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x98da7c20, SUC_lexicon_tags_35 },
    { 0xf01a7c21, SUC_lexicon_tags_37 },
    { 0xd45a7c20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x311a7c24, SUC_lexicon_tags_27 },
    { 0xa6da7c24, SUC_lexicon_tags_0 },
    { 0xd47a7c26, SUC_lexicon_tags_1 },
    { 0x6bfa7c27, SUC_lexicon_tags_11 },
    { 0x337a7c28, SUC_lexicon_tags_1 },
    { 0x4e1a7c29, SUC_lexicon_tags_38 },
    { 0x14da7c29, SUC_lexicon_tags_56 },
    { 0x923a7c2b, SUC_lexicon_tags_2 },
    { 0xaf5a7c26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e3a7c2e, SUC_lexicon_tags_37 },
    { 0xa13a7c2f, SUC_lexicon_tags_0 },
    { 0x13da7c30, SUC_lexicon_tags_45 },
    { 0x5dda7c2f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x591a7c33, SUC_lexicon_tags_38 },
    { 0x36ba7c33, SUC_lexicon_tags_37 },
    { 0xec1a7c33, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf53a7c37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbda7c3c, SUC_lexicon_tags_29 },
    { 0xd31a7c3d, SUC_lexicon_tags_29 },
    { 0x78ba7c3e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d5a7c40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc77a7c42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ba7c46, SUC_lexicon_tags_17 },
    { 0xb4da7c47, SUC_lexicon_tags_48 },
    { 0x53a7c47, SUC_lexicon_tags_37 },
    { 0x711a7c49, SUC_lexicon_tags_14 },
    { 0x777a7c4a, SUC_lexicon_tags_31 },
    { 0x7b1a7c49, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03a7c52, SUC_lexicon_tags_25 },
    { 0x6fba7c53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x401a7c55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fa7c5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x88da7c5f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5a7c67, SUC_lexicon_tags_377 },
    { 0, NULL },
    { 0xad3a7c69, SUC_lexicon_tags_10 },
    { 0xb57a7c6a, SUC_lexicon_tags_66 },
    { 0xcba7c6a, SUC_lexicon_tags_68 },
    { 0x2f7a7c6c, SUC_lexicon_tags_87 },
    { 0xbd1a7c6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7da7c70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fa7c73, SUC_lexicon_tags_37 },
    { 0x6e9a7c74, SUC_lexicon_tags_26 },
    { 0x2b1a7c74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fa7c78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fa7c7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c9a7c7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61a7c82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1a7c86, SUC_lexicon_tags_12 },
    { 0xed5a7c86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x86ba7c89, SUC_lexicon_tags_1233 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1a7c8e, SUC_lexicon_tags_10 },
    { 0xcffa7c8f, SUC_lexicon_tags_57 },
    { 0xe73a7c90, SUC_lexicon_tags_24 },
    { 0x1d1a7c91, SUC_lexicon_tags_37 },
    { 0x763a7c91, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3a7c96, SUC_lexicon_tags_65 },
    { 0xe39a7c96, SUC_lexicon_tags_0 },
    { 0x56fa7c98, SUC_lexicon_tags_17 },
    { 0x657a7c99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f3a7c9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65a7ca4, SUC_lexicon_tags_38 },
    { 0x6b1a7ca4, SUC_lexicon_tags_1 },
    { 0x171a7ca4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc63a7ca8, SUC_lexicon_tags_19 },
    { 0xf59a7ca9, SUC_lexicon_tags_2 },
    { 0x64da7caa, SUC_lexicon_tags_19 },
    { 0x859a7cab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3a7cae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99da7cb0, SUC_lexicon_tags_26 },
    { 0xe89a7cb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17a7cb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725a7cba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x689a7cbd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x7d1a7cbf, SUC_lexicon_tags_47 },
    { 0xf67a7cc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ba7cc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x261a7cc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffa7cc8, SUC_lexicon_tags_11 },
    { 0xdb5a7cc9, SUC_lexicon_tags_12 },
    { 0x145a7cc9, SUC_lexicon_tags_1 },
    { 0xd9fa7ccb, SUC_lexicon_tags_37 },
    { 0x959a7ccb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44da7cce, SUC_lexicon_tags_45 },
    { 0xd11a7ccf, SUC_lexicon_tags_38 },
    { 0xaa9a7cd0, SUC_lexicon_tags_38 },
    { 0xb1da7cd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafa7cd7, SUC_lexicon_tags_37 },
    { 0x847a7cd7, SUC_lexicon_tags_2 },
    { 0x189a7cd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83a7cdc, SUC_lexicon_tags_10 },
    { 0xec5a7cdd, SUC_lexicon_tags_47 },
    { 0x185a7cde, SUC_lexicon_tags_31 },
    { 0xae5a7cdf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e9a7ce1, SUC_lexicon_tags_27 },
    { 0x817a7ce2, SUC_lexicon_tags_101 },
    { 0x9dba7ce3, SUC_lexicon_tags_12 },
    { 0xf1ba7ce4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59a7ce7, SUC_lexicon_tags_15 },
    { 0x34ba7ce7, SUC_lexicon_tags_25 },
    { 0x423a7ce9, SUC_lexicon_tags_12 },
    { 0x81da7cea, SUC_lexicon_tags_49 },
    { 0xd83a7ceb, SUC_lexicon_tags_11 },
    { 0xaf9a7cec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aba7cf0, SUC_lexicon_tags_47 },
    { 0x459a7cf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5a7cf4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97a7cf8, SUC_lexicon_tags_25 },
    { 0x67fa7cf9, SUC_lexicon_tags_32 },
    { 0x4bda7cfa, SUC_lexicon_tags_32 },
    { 0x7f5a7cfb, SUC_lexicon_tags_29 },
    { 0x197a7cfb, SUC_lexicon_tags_17 },
    { 0x8ba7cfd, SUC_lexicon_tags_43 },
    { 0x227a7cfe, SUC_lexicon_tags_57 },
    { 0x199a7cff, SUC_lexicon_tags_1 },
    { 0xa75a7d00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5bfa7d02, SUC_lexicon_tags_4 },
    { 0xd83a7d03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12da7d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x265a7d09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7a7d0f, SUC_lexicon_tags_14 },
    { 0xd6da7d10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7a7d15, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x999a7d18, SUC_lexicon_tags_1 },
    { 0x143a7d19, SUC_lexicon_tags_47 },
    { 0x86da7d19, SUC_lexicon_tags_6 },
    { 0x18ba7d18, SUC_lexicon_tags_15 },
    { 0xdd7a7d18, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x94da7d1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93a7d26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x841a7d28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9a7d2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63a7d32, SUC_lexicon_tags_43 },
    { 0x8d9a7d33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb53a7d35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ba7d38, SUC_lexicon_tags_14 },
    { 0xb55a7d39, SUC_lexicon_tags_17 },
    { 0xc5ba7d39, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9a7d3e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x457a7d41, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425a7d45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcba7d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x871a7d49, SUC_lexicon_tags_45 },
    { 0x827a7d49, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x701a7d4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9a7d4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2da7d55, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fa7d59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd17a7d5b, SUC_lexicon_tags_19 },
    { 0x7ba7d5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x249a7d5f, SUC_lexicon_tags_11 },
    { 0x9c3a7d60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ba7d64, SUC_lexicon_tags_65 },
    { 0x5b7a7d65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fa7d68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x321a7d6a, SUC_lexicon_tags_7 },
    { 0xfc5a7d6b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87a7d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41ba7d75, SUC_lexicon_tags_6 },
    { 0x9d9a7d76, SUC_lexicon_tags_99 },
    { 0xbb9a7d76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24da7d7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5a7d86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5aba7d88, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ba7d8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe05a7d8f, SUC_lexicon_tags_37 },
    { 0x3c7a7d90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfdba7d92, SUC_lexicon_tags_4 },
    { 0x8a1a7d93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3a7d96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7a7da2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x247a7da4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcb5a7da6, SUC_lexicon_tags_85 },
    { 0xc95a7da6, SUC_lexicon_tags_11 },
    { 0xc67a7da8, SUC_lexicon_tags_37 },
    { 0x5ba7da9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7a7db0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad5a7db2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1a7db8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x687a7dba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19a7dbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10ba7dbf, SUC_lexicon_tags_7 },
    { 0x741a7dbf, SUC_lexicon_tags_1 },
    { 0x3c1a7dc1, SUC_lexicon_tags_2 },
    { 0xf45a7dc1, SUC_lexicon_tags_1 },
    { 0x51da7dc1, SUC_lexicon_tags_57 },
    { 0x1b1a7dc4, SUC_lexicon_tags_15 },
    { 0xdd7a7dc4, SUC_lexicon_tags_27 },
    { 0xc3a7dc6, SUC_lexicon_tags_27 },
    { 0x787a7dc7, SUC_lexicon_tags_37 },
    { 0x7fba7dc8, SUC_lexicon_tags_29 },
    { 0xe49a7dc9, SUC_lexicon_tags_19 },
    { 0xd43a7dc9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fa7dce, SUC_lexicon_tags_23 },
    { 0xd2da7dcf, SUC_lexicon_tags_31 },
    { 0xaeda7dcf, SUC_lexicon_tags_19 },
    { 0xb3da7dd1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83a7dd5, SUC_lexicon_tags_25 },
    { 0x8f5a7dd5, SUC_lexicon_tags_56 },
    { 0x3f9a7dd7, SUC_lexicon_tags_25 },
    { 0x52fa7dd5, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91a7ddb, SUC_lexicon_tags_33 },
    { 0x667a7ddb, SUC_lexicon_tags_2 },
    { 0x7b7a7ddb, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xa95a7ddf, SUC_lexicon_tags_38 },
    { 0x263a7ddf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1f5a7de2, SUC_lexicon_tags_14 },
    { 0xabba7de3, SUC_lexicon_tags_11 },
    { 0x8d7a7de4, SUC_lexicon_tags_12 },
    { 0xec1a7de5, SUC_lexicon_tags_11 },
    { 0x48ba7de6, SUC_lexicon_tags_1 },
    { 0x22da7de6, SUC_lexicon_tags_295 },
    { 0xde9a7de7, SUC_lexicon_tags_63 },
    { 0xa6fa7de9, SUC_lexicon_tags_47 },
    { 0xf21a7de9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767a7dee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17a7df3, SUC_lexicon_tags_26 },
    { 0x853a7df3, SUC_lexicon_tags_87 },
    { 0x39da7df5, SUC_lexicon_tags_37 },
    { 0x9fba7df5, SUC_lexicon_tags_11 },
    { 0x1da7df7, SUC_lexicon_tags_1 },
    { 0xa0da7df8, SUC_lexicon_tags_0 },
    { 0x949a7df9, SUC_lexicon_tags_0 },
    { 0x5d7a7df6, SUC_lexicon_tags_49 },
    { 0x6c3a7dfb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc05a7dfd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xef7a7dff, SUC_lexicon_tags_33 },
    { 0xf37a7e00, SUC_lexicon_tags_14 },
    { 0x9c3a7e00, SUC_lexicon_tags_49 },
    { 0xbeba7e02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41a7e05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9a7e09, SUC_lexicon_tags_31 },
    { 0xdefa7e0a, SUC_lexicon_tags_37 },
    { 0xd33a7e0b, SUC_lexicon_tags_37 },
    { 0xbeba7e0b, SUC_lexicon_tags_4 },
    { 0x4a9a7e0a, SUC_lexicon_tags_64 },
    { 0x611a7e0e, SUC_lexicon_tags_55 },
    { 0xb23a7e0e, SUC_lexicon_tags_6 },
    { 0x397a7e0d, SUC_lexicon_tags_11 },
    { 0x5e3a7e11, SUC_lexicon_tags_1 },
    { 0x559a7e12, SUC_lexicon_tags_25 },
    { 0xd09a7e0e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3a7e16, SUC_lexicon_tags_1 },
    { 0xd99a7e17, SUC_lexicon_tags_13 },
    { 0xe6fa7e18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fda7e22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceda7e29, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7fa7e2b, SUC_lexicon_tags_44 },
    { 0x9cfa7e2b, SUC_lexicon_tags_28 },
    { 0xa9da7e2b, SUC_lexicon_tags_1 },
    { 0xc7a7e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3a7e32, SUC_lexicon_tags_12 },
    { 0x60fa7e32, SUC_lexicon_tags_1 },
    { 0x815a7e34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f1a7e36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49a7e39, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcdba7e3b, SUC_lexicon_tags_56 },
    { 0x99ba7e3c, SUC_lexicon_tags_11 },
    { 0xb83a7e3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1cba7e3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1d3a7e41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783a7e48, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xfe5a7e4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd2da7e4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7a7e50, SUC_lexicon_tags_16 },
    { 0xfe9a7e51, SUC_lexicon_tags_8 },
    { 0xb1ba7e51, SUC_lexicon_tags_23 },
    { 0xa1ba7e52, SUC_lexicon_tags_31 },
    { 0x1d9a7e53, SUC_lexicon_tags_17 },
    { 0x7a1a7e54, SUC_lexicon_tags_17 },
    { 0xe73a7e56, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x83fa7e58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61a7e5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67a7e65, SUC_lexicon_tags_37 },
    { 0x395a7e66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5a7e69, SUC_lexicon_tags_37 },
    { 0xf69a7e69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61a7e71, SUC_lexicon_tags_38 },
    { 0x72ba7e71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb93a7e74, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffa7e77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fa7e7b, SUC_lexicon_tags_8 },
    { 0xbcfa7e7c, SUC_lexicon_tags_54 },
    { 0xb3a7e7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9a7e84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x15ba7e86, SUC_lexicon_tags_4 },
    { 0x449a7e86, SUC_lexicon_tags_46 },
    { 0x2d3a7e86, SUC_lexicon_tags_1 },
    { 0x65da7e89, SUC_lexicon_tags_12 },
    { 0x8fba7e89, SUC_lexicon_tags_1 },
    { 0x9d9a7e8b, SUC_lexicon_tags_21 },
    { 0xc89a7e8b, SUC_lexicon_tags_2 },
    { 0xe45a7e8b, SUC_lexicon_tags_28 },
    { 0xd2da7e8b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xa59a7e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7a7e93, SUC_lexicon_tags_37 },
    { 0x813a7e94, SUC_lexicon_tags_39 },
    { 0x8cda7e95, SUC_lexicon_tags_60 },
    { 0x8ada7e96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x847a7e99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49da7ea1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x87a7ea3, SUC_lexicon_tags_11 },
    { 0xfe9a7ea4, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xaffa7ea6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ba7eab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57fa7ead, SUC_lexicon_tags_151 },
    { 0xe97a7eae, SUC_lexicon_tags_31 },
    { 0x9cba7ead, SUC_lexicon_tags_8 },
    { 0x357a7eb0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191a7eb5, SUC_lexicon_tags_12 },
    { 0x759a7eb6, SUC_lexicon_tags_1 },
    { 0x63ba7eb7, SUC_lexicon_tags_15 },
    { 0x1b1a7eb6, SUC_lexicon_tags_14 },
    { 0xac5a7eb5, SUC_lexicon_tags_31 },
    { 0xf61a7eba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54ba7ebc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x785a7ebe, SUC_lexicon_tags_9 },
    { 0x181a7ebe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa11a7ec1, SUC_lexicon_tags_14 },
    { 0x63ba7ec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b5a7ec4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0ba7ec6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x565a7ec9, SUC_lexicon_tags_11 },
    { 0xcada7ec9, SUC_lexicon_tags_48 },
    { 0xee5a7ecb, SUC_lexicon_tags_45 },
    { 0x151a7ecc, SUC_lexicon_tags_15 },
    { 0x249a7eca, SUC_lexicon_tags_47 },
    { 0x75ba7ecc, SUC_lexicon_tags_38 },
    { 0x97da7ecf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xabfa7ed1, SUC_lexicon_tags_60 },
    { 0x92ba7ed2, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x571a7ed5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3a7ed9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5a7edd, SUC_lexicon_tags_37 },
    { 0x103a7ede, SUC_lexicon_tags_37 },
    { 0x63da7edf, SUC_lexicon_tags_11 },
    { 0x71da7ee0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x19da7ee3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bda7ee6, SUC_lexicon_tags_15 },
    { 0xc0fa7ee7, SUC_lexicon_tags_25 },
    { 0xeeba7ee6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ba7ef0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed5a7ef2, SUC_lexicon_tags_2 },
    { 0x157a7ef2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951a7ef9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe77a7efb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7eda7efd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91a7f00, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x4b7a7f02, SUC_lexicon_tags_15 },
    { 0xa9fa7f03, SUC_lexicon_tags_33 },
    { 0x3c7a7f04, SUC_lexicon_tags_60 },
    { 0x859a7f03, SUC_lexicon_tags_32 },
    { 0x963a7f06, SUC_lexicon_tags_16 },
    { 0x9c3a7f06, SUC_lexicon_tags_11 },
    { 0x9efa7f02, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadda7f10, SUC_lexicon_tags_57 },
    { 0xff3a7f11, SUC_lexicon_tags_30 },
    { 0x1f5a7f11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23a7f18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1a7f23, SUC_lexicon_tags_17 },
    { 0xf9fa7f23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7f27, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1cba7f29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec7a7f2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a7a7f2d, SUC_lexicon_tags_47 },
    { 0x9aba7f2d, SUC_lexicon_tags_25 },
    { 0xc17a7f2f, SUC_lexicon_tags_46 },
    { 0x6a5a7f2f, SUC_lexicon_tags_15 },
    { 0xecda7f31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x967a7f33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x403a7f35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6da7f38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cba7f3d, SUC_lexicon_tags_17 },
    { 0xac9a7f3e, SUC_lexicon_tags_12 },
    { 0x8f3a7f3f, SUC_lexicon_tags_55 },
    { 0x489a7f40, SUC_lexicon_tags_30 },
    { 0x873a7f41, SUC_lexicon_tags_4 },
    { 0x7a3a7f42, SUC_lexicon_tags_17 },
    { 0x1dba7f3e, SUC_lexicon_tags_37 },
    { 0xe21a7f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfa7f4e, SUC_lexicon_tags_7 },
    { 0xe0ba7f4f, SUC_lexicon_tags_45 },
    { 0xe5a7f50, SUC_lexicon_tags_83 },
    { 0xc71a7f4f, SUC_lexicon_tags_11 },
    { 0xd53a7f52, SUC_lexicon_tags_37 },
    { 0x4eba7f53, SUC_lexicon_tags_43 },
    { 0xb1ba7f54, SUC_lexicon_tags_15 },
    { 0x939a7f50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd9a7f57, SUC_lexicon_tags_15 },
    { 0xd69a7f57, SUC_lexicon_tags_2 },
    { 0xf2fa7f59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x763a7f5c, SUC_lexicon_tags_37 },
    { 0xa57a7f5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ada7f60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x307a7f63, SUC_lexicon_tags_37 },
    { 0xceba7f64, SUC_lexicon_tags_29 },
    { 0x661a7f65, SUC_lexicon_tags_60 },
    { 0x59da7f65, SUC_lexicon_tags_17 },
    { 0x893a7f64, SUC_lexicon_tags_11 },
    { 0x38fa7f65, SUC_lexicon_tags_38 },
    { 0x661a7f68, SUC_lexicon_tags_14 },
    { 0xa99a7f64, SUC_lexicon_tags_49 },
    { 0xd1da7f65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4e9a7f6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25a7f70, SUC_lexicon_tags_25 },
    { 0xaeda7f71, SUC_lexicon_tags_7 },
    { 0x2afa7f72, SUC_lexicon_tags_104 },
    { 0xaeba7f72, SUC_lexicon_tags_0 },
    { 0x551a7f72, SUC_lexicon_tags_15 },
    { 0xf3da7f73, SUC_lexicon_tags_5 },
    { 0xb1a7f76, SUC_lexicon_tags_37 },
    { 0x489a7f77, SUC_lexicon_tags_23 },
    { 0xe69a7f78, SUC_lexicon_tags_47 },
    { 0x391a7f71, SUC_lexicon_tags_1 },
    { 0x597a7f7a, SUC_lexicon_tags_12 },
    { 0x935a7f7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903a7f85, SUC_lexicon_tags_12 },
    { 0xe29a7f86, SUC_lexicon_tags_1 },
    { 0x9da7f87, SUC_lexicon_tags_16 },
    { 0x77fa7f88, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1a7f8b, SUC_lexicon_tags_11 },
    { 0x8e3a7f8b, SUC_lexicon_tags_23 },
    { 0x315a7f8d, SUC_lexicon_tags_63 },
    { 0xb8da7f8c, SUC_lexicon_tags_25 },
    { 0x82da7f8f, SUC_lexicon_tags_25 },
    { 0xf87a7f8b, SUC_lexicon_tags_39 },
    { 0x36ba7f91, SUC_lexicon_tags_21 },
    { 0x1d3a7f92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21a7f95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d5a7f97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cda7f9d, SUC_lexicon_tags_17 },
    { 0x2a1a7f9e, SUC_lexicon_tags_25 },
    { 0xefda7f9f, SUC_lexicon_tags_60 },
    { 0x533a7f9d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821a7fa8, SUC_lexicon_tags_11 },
    { 0xaf1a7fa9, SUC_lexicon_tags_2 },
    { 0x229a7faa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233a7fb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fa7fb7, SUC_lexicon_tags_25 },
    { 0x9dda7fb7, SUC_lexicon_tags_47 },
    { 0x6a9a7fb9, SUC_lexicon_tags_27 },
    { 0xbfba7fba, SUC_lexicon_tags_7 },
    { 0x33ba7fb8, SUC_lexicon_tags_7 },
    { 0xb91a7fbc, SUC_lexicon_tags_3 },
    { 0xc25a7fbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x937a7fbf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafda7fc3, SUC_lexicon_tags_12 },
    { 0x9c5a7fc3, SUC_lexicon_tags_2 },
    { 0x687a7fc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdda7fc8, SUC_lexicon_tags_9 },
    { 0xf87a7fc9, SUC_lexicon_tags_1 },
    { 0xa19a7fc8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fa7fd0, SUC_lexicon_tags_27 },
    { 0x89da7fd0, SUC_lexicon_tags_28 },
    { 0xce9a7fd2, SUC_lexicon_tags_14 },
    { 0x29da7fd1, SUC_lexicon_tags_25 },
    { 0x86da7fd2, SUC_lexicon_tags_37 },
    { 0x515a7fd1, SUC_lexicon_tags_38 },
    { 0x175a7fd6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811a7fdb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf71a7fdd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261a7fe1, SUC_lexicon_tags_30 },
    { 0xa2fa7fe2, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47a7fe6, SUC_lexicon_tags_12 },
    { 0xc25a7fe6, SUC_lexicon_tags_11 },
    { 0x1bba7fe8, SUC_lexicon_tags_26 },
    { 0x7bba7fe8, SUC_lexicon_tags_25 },
    { 0x191a7fe8, SUC_lexicon_tags_12 },
    { 0x961a7feb, SUC_lexicon_tags_15 },
    { 0x1ffa7fec, SUC_lexicon_tags_17 },
    { 0xbcfa7fec, SUC_lexicon_tags_37 },
    { 0x463a7fe8, SUC_lexicon_tags_28 },
    { 0xebda7feb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x369a7ff1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc91a7ff3, SUC_lexicon_tags_55 },
    { 0x1ada7ff3, SUC_lexicon_tags_1 },
    { 0xe9da7ff3, SUC_lexicon_tags_1 },
    { 0x5b3a7ff4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8fa7ff8, SUC_lexicon_tags_9 },
    { 0x259a7ff9, SUC_lexicon_tags_65 },
    { 0x13a7ff9, SUC_lexicon_tags_25 },
    { 0xa39a7ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5a7fff, SUC_lexicon_tags_1 },
    { 0xa79a8000, SUC_lexicon_tags_7 },
    { 0x7e9a8001, SUC_lexicon_tags_38 },
    { 0x491a7fff, SUC_lexicon_tags_45 },
    { 0x141a8001, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xc1fa8005, SUC_lexicon_tags_87 },
    { 0x9d1a8006, SUC_lexicon_tags_4 },
    { 0xc19a8007, SUC_lexicon_tags_43 },
    { 0x567a8007, SUC_lexicon_tags_17 },
    { 0x699a8006, SUC_lexicon_tags_27 },
    { 0xedda800a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8a7a800c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13a8011, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bda8014, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6da8017, SUC_lexicon_tags_29 },
    { 0xe99a8018, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefa801e, SUC_lexicon_tags_60 },
    { 0xa61a801f, SUC_lexicon_tags_0 },
    { 0x6dda801e, SUC_lexicon_tags_45 },
    { 0x619a8021, SUC_lexicon_tags_14 },
    { 0xf3da8022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7da8024, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55a802d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccfa802f, SUC_lexicon_tags_38 },
    { 0xd95a8030, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda8035, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb07a8037, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8b1a8039, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8da803e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1a8041, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3c1a8043, SUC_lexicon_tags_17 },
    { 0x677a8044, SUC_lexicon_tags_15 },
    { 0x4f5a8044, SUC_lexicon_tags_129 },
    { 0xe31a8046, SUC_lexicon_tags_37 },
    { 0x8fda8046, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81a804b, SUC_lexicon_tags_17 },
    { 0xc9a804b, SUC_lexicon_tags_15 },
    { 0xe35a804d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b7a804f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37a8052, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf57a8054, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf5a8056, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7a805b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x66da805e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x665a8060, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0xbe7a8062, SUC_lexicon_tags_15 },
    { 0x2ffa8063, SUC_lexicon_tags_17 },
    { 0x25da8064, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xaafa8066, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa7ba8068, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903a806c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189a8070, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161a8075, SUC_lexicon_tags_17 },
    { 0x513a8076, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x211a8079, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x98fa807b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33a807e, SUC_lexicon_tags_28 },
    { 0x4b9a807f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15a8083, SUC_lexicon_tags_10 },
    { 0xe87a8084, SUC_lexicon_tags_26 },
    { 0x103a8084, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ba8089, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x82ba808b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfa8090, SUC_lexicon_tags_27 },
    { 0xe4fa8090, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x26ba8093, SUC_lexicon_tags_17 },
    { 0xa61a8093, SUC_lexicon_tags_11 },
    { 0x229a8094, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49a8099, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35da809c, SUC_lexicon_tags_11 },
    { 0x37a809d, SUC_lexicon_tags_8 },
    { 0xcdfa809e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a80a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1a80a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5a80a8, SUC_lexicon_tags_25 },
    { 0xb81a80a9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927a80b6, SUC_lexicon_tags_7 },
    { 0xcffa80b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x971a80b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45da80bc, SUC_lexicon_tags_5 },
    { 0x41a80bd, SUC_lexicon_tags_49 },
    { 0x779a80bd, SUC_lexicon_tags_8 },
    { 0xe03a80bf, SUC_lexicon_tags_11 },
    { 0xb4da80bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181a80c6, SUC_lexicon_tags_17 },
    { 0xb47a80c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7a80cb, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a80cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeba80d3, SUC_lexicon_tags_16 },
    { 0x377a80d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x111a80d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735a80da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fa80dd, SUC_lexicon_tags_47 },
    { 0xa9fa80de, SUC_lexicon_tags_17 },
    { 0xa01a80dd, SUC_lexicon_tags_16 },
    { 0x209a80dd, SUC_lexicon_tags_12 },
    { 0x1bda80e0, SUC_lexicon_tags_60 },
    { 0xebfa80de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19a80e5, SUC_lexicon_tags_9 },
    { 0xe43a80e6, SUC_lexicon_tags_31 },
    { 0xbdba80e6, SUC_lexicon_tags_17 },
    { 0x359a80e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc45a80ea, SUC_lexicon_tags_46 },
    { 0xa7ba80eb, SUC_lexicon_tags_1 },
    { 0xfd5a80ea, SUC_lexicon_tags_14 },
    { 0xfc7a80ea, SUC_lexicon_tags_11 },
    { 0x9f1a80ed, SUC_lexicon_tags_10 },
    { 0x533a80ef, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dda80f2, SUC_lexicon_tags_11 },
    { 0x47a80f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xde5a80f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x579a80f7, SUC_lexicon_tags_7 },
    { 0x75da80f8, SUC_lexicon_tags_30 },
    { 0x2a7a80f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19a8101, SUC_lexicon_tags_37 },
    { 0xf65a8102, SUC_lexicon_tags_114 },
    { 0x289a8103, SUC_lexicon_tags_37 },
    { 0x59ba8104, SUC_lexicon_tags_37 },
    { 0x4b9a8105, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fa8108, SUC_lexicon_tags_28 },
    { 0x8e7a8109, SUC_lexicon_tags_17 },
    { 0x84da810a, SUC_lexicon_tags_15 },
    { 0xcb5a810b, SUC_lexicon_tags_63 },
    { 0x295a810a, SUC_lexicon_tags_27 },
    { 0xa57a810b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x609a8110, SUC_lexicon_tags_45 },
    { 0xdf3a8111, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xed5a8113, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95a811b, SUC_lexicon_tags_26 },
    { 0x137a811b, SUC_lexicon_tags_37 },
    { 0x255a811d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157a8126, SUC_lexicon_tags_31 },
    { 0xeba8127, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9a812e, SUC_lexicon_tags_2 },
    { 0x5b9a812f, SUC_lexicon_tags_14 },
    { 0xd97a812f, SUC_lexicon_tags_98 },
    { 0xae9a812f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1a813a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59ba813c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba8140, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85a8143, SUC_lexicon_tags_11 },
    { 0x21a8143, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6da8149, SUC_lexicon_tags_14 },
    { 0xa87a8149, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1a8156, SUC_lexicon_tags_69 },
    { 0xeeba8156, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299a815f, SUC_lexicon_tags_31 },
    { 0x8f3a8160, SUC_lexicon_tags_201 },
    { 0x223a8161, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba8165, SUC_lexicon_tags_27 },
    { 0x74da8165, SUC_lexicon_tags_160 },
    { 0x44da8167, SUC_lexicon_tags_15 },
    { 0x48ba8167, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x901a816b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535a816f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1a8173, SUC_lexicon_tags_27 },
    { 0xd15a8174, SUC_lexicon_tags_2 },
    { 0xacfa8173, SUC_lexicon_tags_28 },
    { 0x31da8176, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x309a8178, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25a817c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12ba817e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65a8185, SUC_lexicon_tags_37 },
    { 0x535a8186, SUC_lexicon_tags_21 },
    { 0xbb3a8185, SUC_lexicon_tags_49 },
    { 0xefda8185, SUC_lexicon_tags_17 },
    { 0x9a5a8189, SUC_lexicon_tags_37 },
    { 0xefa818a, SUC_lexicon_tags_1077 },
    { 0x31fa818b, SUC_lexicon_tags_28 },
    { 0x391a8186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353a8190, SUC_lexicon_tags_68 },
    { 0xecfa8191, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39a8196, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc4fa8198, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x879a819a, SUC_lexicon_tags_27 },
    { 0xd73a819b, SUC_lexicon_tags_37 },
    { 0x98fa819b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x23ba819e, SUC_lexicon_tags_37 },
    { 0x5bda819e, SUC_lexicon_tags_37 },
    { 0xa75a819e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73da81a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x695a81a9, SUC_lexicon_tags_3 },
    { 0xd53a81aa, SUC_lexicon_tags_11 },
    { 0x721a81ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9ba81ad, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1a81b6, SUC_lexicon_tags_25 },
    { 0xb1fa81b6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x387a81ba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7a81bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7a81c0, SUC_lexicon_tags_34 },
    { 0x873a81c1, SUC_lexicon_tags_1092 },
    { 0xab9a81c2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a81c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x80fa81c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933a81d1, SUC_lexicon_tags_0 },
    { 0x163a81d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9a81d5, SUC_lexicon_tags_37 },
    { 0x40fa81d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x197a81d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5a81e2, SUC_lexicon_tags_11 },
    { 0x56ba81e3, SUC_lexicon_tags_1 },
    { 0x289a81e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69a81e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeada81ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fa81ed, SUC_lexicon_tags_23 },
    { 0x2bfa81ee, SUC_lexicon_tags_0 },
    { 0xf29a81ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xceba81f1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa09a81f3, SUC_lexicon_tags_23 },
    { 0x76fa81f4, SUC_lexicon_tags_56 },
    { 0x8fba81f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x639a81f7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeb7a81f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fa81fe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ba8201, SUC_lexicon_tags_43 },
    { 0xca7a8201, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xafda8204, SUC_lexicon_tags_33 },
    { 0x6ffa8205, SUC_lexicon_tags_45 },
    { 0x545a8206, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8bba8208, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x701a820a, SUC_lexicon_tags_37 },
    { 0xe05a820a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x759a820d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa05a820f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fba8212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe65a8214, SUC_lexicon_tags_15 },
    { 0x72da8215, SUC_lexicon_tags_37 },
    { 0x7aba8215, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38da821b, SUC_lexicon_tags_8 },
    { 0xa9da821b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x751a821e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x245a8221, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ba8225, SUC_lexicon_tags_56 },
    { 0xaffa8226, SUC_lexicon_tags_43 },
    { 0xfb5a8227, SUC_lexicon_tags_66 },
    { 0x453a8226, SUC_lexicon_tags_12 },
    { 0xcba8229, SUC_lexicon_tags_37 },
    { 0x33da8226, SUC_lexicon_tags_172 },
    { 0xf4ba822b, SUC_lexicon_tags_28 },
    { 0xfa3a8225, SUC_lexicon_tags_1 },
    { 0x127a822d, SUC_lexicon_tags_60 },
    { 0x695a8227, SUC_lexicon_tags_91 },
    { 0xe2da822e, SUC_lexicon_tags_26 },
    { 0xe33a8230, SUC_lexicon_tags_55 },
    { 0xd57a8230, SUC_lexicon_tags_15 },
    { 0x7e5a8232, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eba8235, SUC_lexicon_tags_37 },
    { 0x873a8235, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ada8239, SUC_lexicon_tags_0 },
    { 0xcd5a823a, SUC_lexicon_tags_60 },
    { 0x77da8239, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fa823f, SUC_lexicon_tags_27 },
    { 0x67ba8240, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735a8244, SUC_lexicon_tags_1 },
    { 0xdbfa8244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85a8248, SUC_lexicon_tags_23 },
    { 0x359a8248, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x613a824b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a824e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc97a8250, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf73a8252, SUC_lexicon_tags_17 },
    { 0x161a8252, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391a825b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcda825f, SUC_lexicon_tags_37 },
    { 0x8e7a8260, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x519a8263, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcada8269, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x681a826c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8dda826e, SUC_lexicon_tags_27 },
    { 0x379a826e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31a8272, SUC_lexicon_tags_12 },
    { 0xbb7a8273, SUC_lexicon_tags_47 },
    { 0xadda8274, SUC_lexicon_tags_55 },
    { 0x4dda8274, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x257a8277, SUC_lexicon_tags_30 },
    { 0xb07a8278, SUC_lexicon_tags_72 },
    { 0xdaba8278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79da827f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23da8281, SUC_lexicon_tags_65 },
    { 0x649a8282, SUC_lexicon_tags_23 },
    { 0xaeda8283, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92ba8285, SUC_lexicon_tags_25 },
    { 0xae7a8285, SUC_lexicon_tags_39 },
    { 0xf23a8285, SUC_lexicon_tags_13 },
    { 0xa5fa8288, SUC_lexicon_tags_12 },
    { 0x47a8289, SUC_lexicon_tags_60 },
    { 0xab7a8289, SUC_lexicon_tags_35 },
    { 0xf03a828b, SUC_lexicon_tags_7 },
    { 0xbf1a828c, SUC_lexicon_tags_37 },
    { 0xfb5a828a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdba828f, SUC_lexicon_tags_68 },
    { 0x255a8290, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92da8292, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ba8295, SUC_lexicon_tags_37 },
    { 0x135a8296, SUC_lexicon_tags_19 },
    { 0x7b5a8297, SUC_lexicon_tags_15 },
    { 0x92fa8297, SUC_lexicon_tags_16 },
    { 0x29fa8299, SUC_lexicon_tags_15 },
    { 0xa5a829a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59a829e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ba82a1, SUC_lexicon_tags_32 },
    { 0xc97a82a2, SUC_lexicon_tags_7 },
    { 0xbbba82a2, SUC_lexicon_tags_17 },
    { 0xe95a82a2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x197a82a7, SUC_lexicon_tags_23 },
    { 0xb31a82a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3eda82aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb07a82ac, SUC_lexicon_tags_1 },
    { 0x43da82ac, SUC_lexicon_tags_7 },
    { 0x793a82ad, SUC_lexicon_tags_63 },
    { 0xe31a82af, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x41da82b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x811a82b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x231a82b8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x441a82bb, SUC_lexicon_tags_17 },
    { 0x6aba82bc, SUC_lexicon_tags_10 },
    { 0x253a82bd, SUC_lexicon_tags_32 },
    { 0x399a82bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x74fa82c1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4cda82c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4da82c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5a82cb, SUC_lexicon_tags_2 },
    { 0xd31a82cc, SUC_lexicon_tags_33 },
    { 0xa33a82cd, SUC_lexicon_tags_2 },
    { 0xa37a82ce, SUC_lexicon_tags_17 },
    { 0x723a82cf, SUC_lexicon_tags_54 },
    { 0x23a82cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483a82d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaba82e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1a82ed, SUC_lexicon_tags_12 },
    { 0x4afa82ed, SUC_lexicon_tags_15 },
    { 0xbb7a82ef, SUC_lexicon_tags_38 },
    { 0x20ba82ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bba82f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9a82fb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621a8300, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887a8306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ba8309, SUC_lexicon_tags_534 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617a830f, SUC_lexicon_tags_1 },
    { 0xb67a8310, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95a8314, SUC_lexicon_tags_60 },
    { 0x13a8315, SUC_lexicon_tags_17 },
    { 0x47fa8316, SUC_lexicon_tags_47 },
    { 0xada8317, SUC_lexicon_tags_33 },
    { 0x21fa8316, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5a831d, SUC_lexicon_tags_9 },
    { 0x2b5a831e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76da8320, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ba8325, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x173a8328, SUC_lexicon_tags_12 },
    { 0x2cda8328, SUC_lexicon_tags_1 },
    { 0x95fa8328, SUC_lexicon_tags_2 },
    { 0x86fa8329, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5bfa832d, SUC_lexicon_tags_47 },
    { 0x237a832d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53a8331, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15a8335, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc39a8337, SUC_lexicon_tags_15 },
    { 0xe5da8337, SUC_lexicon_tags_27 },
    { 0x36fa8339, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3a8340, SUC_lexicon_tags_47 },
    { 0x45a8341, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec9a8343, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x259a8345, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3a834c, SUC_lexicon_tags_14 },
    { 0x2f7a834d, SUC_lexicon_tags_29 },
    { 0x4a7a834e, SUC_lexicon_tags_8 },
    { 0x8f1a834e, SUC_lexicon_tags_12 },
    { 0xb9a8350, SUC_lexicon_tags_14 },
    { 0xfd9a8351, SUC_lexicon_tags_45 },
    { 0x34da8352, SUC_lexicon_tags_55 },
    { 0xdb9a8351, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce3a8355, SUC_lexicon_tags_23 },
    { 0xbe3a8355, SUC_lexicon_tags_25 },
    { 0x1c1a8355, SUC_lexicon_tags_21 },
    { 0xa75a8358, SUC_lexicon_tags_19 },
    { 0xd3ba8359, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4da835c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfa8363, SUC_lexicon_tags_32 },
    { 0x89da8364, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fa8367, SUC_lexicon_tags_8 },
    { 0xcb1a8368, SUC_lexicon_tags_1 },
    { 0x7e1a8369, SUC_lexicon_tags_17 },
    { 0xa27a8369, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x649a836c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x881a836e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f7a8370, SUC_lexicon_tags_37 },
    { 0xcd3a8370, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf9a8373, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaf3a8375, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7a837a, SUC_lexicon_tags_6 },
    { 0x921a837a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5a8388, SUC_lexicon_tags_48 },
    { 0xa3fa8388, SUC_lexicon_tags_63 },
    { 0xa2ba838a, SUC_lexicon_tags_25 },
    { 0xb15a838b, SUC_lexicon_tags_38 },
    { 0x827a838b, SUC_lexicon_tags_37 },
    { 0x6eba838d, SUC_lexicon_tags_45 },
    { 0x14da838d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe15a8390, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x839a8393, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5fda8395, SUC_lexicon_tags_62 },
    { 0x191a8396, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175a839c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfcfa839e, SUC_lexicon_tags_12 },
    { 0x691a839f, SUC_lexicon_tags_25 },
    { 0x1b5a839f, SUC_lexicon_tags_10 },
    { 0x389a83a1, SUC_lexicon_tags_14 },
    { 0x36da83a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x24fa83a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9a83a7, SUC_lexicon_tags_37 },
    { 0x8f5a83a8, SUC_lexicon_tags_7 },
    { 0xed3a83a9, SUC_lexicon_tags_60 },
    { 0x75da83a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8da83ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59a83b1, SUC_lexicon_tags_63 },
    { 0x7fda83b2, SUC_lexicon_tags_12 },
    { 0x4b9a83b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8b1a83b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fba83b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a83be, SUC_lexicon_tags_0 },
    { 0xec7a83be, SUC_lexicon_tags_9 },
    { 0x1cda83c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc5a83c2, SUC_lexicon_tags_11 },
    { 0x735a83c2, SUC_lexicon_tags_25 },
    { 0xc95a83c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5a83cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fa83d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5a83d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64fa83d6, SUC_lexicon_tags_11 },
    { 0xa31a83d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43a83d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbda83de, SUC_lexicon_tags_23 },
    { 0x31da83df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73a83ea, SUC_lexicon_tags_45 },
    { 0x80fa83ea, SUC_lexicon_tags_30 },
    { 0x9eda83ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99a83ef, SUC_lexicon_tags_25 },
    { 0x225a83ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da83f5, SUC_lexicon_tags_37 },
    { 0x3e5a83f6, SUC_lexicon_tags_48 },
    { 0x55a83f6, SUC_lexicon_tags_7 },
    { 0x633a83f8, SUC_lexicon_tags_55 },
    { 0xad9a83f7, SUC_lexicon_tags_39 },
    { 0xba7a83f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3a8401, SUC_lexicon_tags_25 },
    { 0x981a8402, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x663a8405, SUC_lexicon_tags_68 },
    { 0xf13a8406, SUC_lexicon_tags_15 },
    { 0xb55a8406, SUC_lexicon_tags_63 },
    { 0xb59a8408, SUC_lexicon_tags_25 },
    { 0xaefa8408, SUC_lexicon_tags_63 },
    { 0xd8da840a, SUC_lexicon_tags_25 },
    { 0x2fba8407, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11a8411, SUC_lexicon_tags_15 },
    { 0x165a8412, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dda8415, SUC_lexicon_tags_2 },
    { 0x377a8416, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e7a8418, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2da841c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937a8420, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33a8423, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3a8426, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5da8428, SUC_lexicon_tags_2 },
    { 0x93fa8428, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1a842d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59a8433, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eba8439, SUC_lexicon_tags_25 },
    { 0x99a843a, SUC_lexicon_tags_3 },
    { 0x39da843b, SUC_lexicon_tags_48 },
    { 0xe75a843c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39a8440, SUC_lexicon_tags_11 },
    { 0x2e3a8441, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xafa8444, SUC_lexicon_tags_87 },
    { 0x35ba8445, SUC_lexicon_tags_29 },
    { 0xa45a8446, SUC_lexicon_tags_6 },
    { 0x195a8446, SUC_lexicon_tags_14 },
    { 0xd61a8446, SUC_lexicon_tags_26 },
    { 0xff3a8449, SUC_lexicon_tags_23 },
    { 0x7fda844a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b7a844c, SUC_lexicon_tags_7 },
    { 0x811a844d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0da8452, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef9a8454, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x267a8457, SUC_lexicon_tags_7 },
    { 0x3afa8458, SUC_lexicon_tags_2 },
    { 0x973a8459, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bda845f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x163a8462, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf9a8464, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc5a8466, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x581a8468, SUC_lexicon_tags_33 },
    { 0xc2ba8469, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2ba846b, SUC_lexicon_tags_33 },
    { 0xc09a846b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaba8473, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3a8476, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x195a8478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffa847b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9a847e, SUC_lexicon_tags_28 },
    { 0x8bfa847e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xecda8482, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x831a8485, SUC_lexicon_tags_16 },
    { 0x907a8486, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27da8492, SUC_lexicon_tags_27 },
    { 0x72da8493, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa81a8495, SUC_lexicon_tags_38 },
    { 0xa27a8495, SUC_lexicon_tags_63 },
    { 0x619a8497, SUC_lexicon_tags_3 },
    { 0x9e3a8498, SUC_lexicon_tags_77 },
    { 0x733a8498, SUC_lexicon_tags_17 },
    { 0x7cfa849a, SUC_lexicon_tags_27 },
    { 0xce1a8496, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfa849e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447a84a6, SUC_lexicon_tags_30 },
    { 0x671a84a7, SUC_lexicon_tags_43 },
    { 0x353a84a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa5a84aa, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739a84af, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7d5a84b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ba84b6, SUC_lexicon_tags_1525 },
    { 0x83fa84b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509a84bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651a84c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x171a84c3, SUC_lexicon_tags_7 },
    { 0xb95a84c4, SUC_lexicon_tags_8 },
    { 0xb09a84c5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483a84cb, SUC_lexicon_tags_27 },
    { 0x76fa84cc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x91fa84ce, SUC_lexicon_tags_7 },
    { 0x315a84cf, SUC_lexicon_tags_11 },
    { 0xab7a84d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x349a84d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7a84d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbdda84d8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7a84dd, SUC_lexicon_tags_3 },
    { 0x25ba84de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x60fa84e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407a84e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x215a84e6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbaba84e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cda84eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x863a84ed, SUC_lexicon_tags_47 },
    { 0xf8fa84ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43a84f2, SUC_lexicon_tags_26 },
    { 0x919a84f3, SUC_lexicon_tags_0 },
    { 0x64ba84f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde1a84f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa84fc, SUC_lexicon_tags_25 },
    { 0xdfa84fd, SUC_lexicon_tags_17 },
    { 0xc7ba84fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d3a8500, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73a8506, SUC_lexicon_tags_45 },
    { 0x3e5a8506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2efa8509, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa1a850b, SUC_lexicon_tags_14 },
    { 0x161a850c, SUC_lexicon_tags_17 },
    { 0xbd5a850d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8e1a850f, SUC_lexicon_tags_87 },
    { 0x1a9a8510, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fa8513, SUC_lexicon_tags_5 },
    { 0x221a8514, SUC_lexicon_tags_13 },
    { 0x94fa8515, SUC_lexicon_tags_11 },
    { 0xedfa8516, SUC_lexicon_tags_43 },
    { 0xbb5a8517, SUC_lexicon_tags_15 },
    { 0x761a8518, SUC_lexicon_tags_1 },
    { 0x63da8515, SUC_lexicon_tags_33 },
    { 0x713a851a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79da851c, SUC_lexicon_tags_11 },
    { 0xba851c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa53a851f, SUC_lexicon_tags_15 },
    { 0xa95a8520, SUC_lexicon_tags_12 },
    { 0x235a8520, SUC_lexicon_tags_37 },
    { 0x89da8520, SUC_lexicon_tags_8 },
    { 0x771a8523, SUC_lexicon_tags_14 },
    { 0x2d1a8523, SUC_lexicon_tags_25 },
    { 0xfcba8525, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59da852b, SUC_lexicon_tags_14 },
    { 0x36da852c, SUC_lexicon_tags_29 },
    { 0xbcba852d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7da853b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x509a853e, SUC_lexicon_tags_1 },
    { 0xdc1a853f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247a8543, SUC_lexicon_tags_43 },
    { 0x7f5a8544, SUC_lexicon_tags_19 },
    { 0x9bda8543, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3a8548, SUC_lexicon_tags_15 },
    { 0x647a8548, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf87a854b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9a8555, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b1a8557, SUC_lexicon_tags_47 },
    { 0xfe1a8557, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fa855b, SUC_lexicon_tags_30 },
    { 0x1f5a855b, SUC_lexicon_tags_0 },
    { 0xde7a855c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x60ba855f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3da8564, SUC_lexicon_tags_45 },
    { 0x9b3a8565, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x19a8567, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a856a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd7da856c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5a8578, SUC_lexicon_tags_11 },
    { 0xf1da8579, SUC_lexicon_tags_27 },
    { 0xea5a8578, SUC_lexicon_tags_9 },
    { 0x8f7a8578, SUC_lexicon_tags_37 },
    { 0xa01a857c, SUC_lexicon_tags_29 },
    { 0xbb9a8578, SUC_lexicon_tags_9 },
    { 0x3b9a857e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ba8582, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7dfa8584, SUC_lexicon_tags_38 },
    { 0xc0da8585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66fa8587, SUC_lexicon_tags_63 },
    { 0xebfa8587, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9a858b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e5a858d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x457a8590, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7a8596, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369a859f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x62da85a1, SUC_lexicon_tags_0 },
    { 0x19da85a1, SUC_lexicon_tags_38 },
    { 0x617a85a1, SUC_lexicon_tags_33 },
    { 0x899a85a4, SUC_lexicon_tags_49 },
    { 0x6a5a85a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a85aa, SUC_lexicon_tags_31 },
    { 0xdeda85aa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x15ba85ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bba85b0, SUC_lexicon_tags_14 },
    { 0x55fa85b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9a85b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5a85b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7a85bb, SUC_lexicon_tags_613 },
    { 0x9e3a85bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf81a85be, SUC_lexicon_tags_12 },
    { 0x5f7a85bf, SUC_lexicon_tags_37 },
    { 0xfd1a85c0, SUC_lexicon_tags_32 },
    { 0x34ba85c1, SUC_lexicon_tags_35 },
    { 0xdada85c2, SUC_lexicon_tags_17 },
    { 0x517a85c3, SUC_lexicon_tags_9 },
    { 0xb5fa85c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fa85c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd1da85ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc19a85cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x13ba85ce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115a85d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33a85d5, SUC_lexicon_tags_3 },
    { 0x56ba85d5, SUC_lexicon_tags_15 },
    { 0x4cfa85d5, SUC_lexicon_tags_46 },
    { 0x67a85d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4c7a85da, SUC_lexicon_tags_30 },
    { 0x45da85db, SUC_lexicon_tags_12 },
    { 0xc9da85db, SUC_lexicon_tags_17 },
    { 0x371a85dc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915a85e3, SUC_lexicon_tags_27 },
    { 0xd97a85e4, SUC_lexicon_tags_6 },
    { 0x8e9a85e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb19a85e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa89a85e9, SUC_lexicon_tags_47 },
    { 0x8eba85ea, SUC_lexicon_tags_63 },
    { 0x269a85ea, SUC_lexicon_tags_58 },
    { 0x315a85eb, SUC_lexicon_tags_9 },
    { 0x959a85eb, SUC_lexicon_tags_9 },
    { 0xdda85ee, SUC_lexicon_tags_27 },
    { 0xa4da85ef, SUC_lexicon_tags_1 },
    { 0x49fa85ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fa85f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21da85f5, SUC_lexicon_tags_4 },
    { 0xbbfa85f6, SUC_lexicon_tags_27 },
    { 0x65a85f7, SUC_lexicon_tags_17 },
    { 0xc95a85f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37a85fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x443a85fe, SUC_lexicon_tags_11 },
    { 0x2cfa85ff, SUC_lexicon_tags_0 },
    { 0x949a8600, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7a8604, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9da8607, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bba860b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fa860e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19a8613, SUC_lexicon_tags_107 },
    { 0x9d1a8614, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7bda8616, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51a861c, SUC_lexicon_tags_2 },
    { 0xa4ba861d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dda8620, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7a8627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68da862a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379a8630, SUC_lexicon_tags_37 },
    { 0x7afa8631, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xffa8634, SUC_lexicon_tags_43 },
    { 0x2fba8635, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb71a8638, SUC_lexicon_tags_29 },
    { 0x9d5a8639, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3a863e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ada8642, SUC_lexicon_tags_17 },
    { 0x417a8643, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa864a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbf9a864c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7a8651, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfa865d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcd1a865f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fa8662, SUC_lexicon_tags_37 },
    { 0x5f9a8663, SUC_lexicon_tags_28 },
    { 0x161a8663, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fa8667, SUC_lexicon_tags_37 },
    { 0xa11a8668, SUC_lexicon_tags_9 },
    { 0xafa8669, SUC_lexicon_tags_60 },
    { 0xbc9a8669, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xadba866d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb15a866f, SUC_lexicon_tags_2 },
    { 0xd73a8670, SUC_lexicon_tags_2 },
    { 0xaf5a866f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfa8674, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9a8677, SUC_lexicon_tags_7 },
    { 0xc53a8677, SUC_lexicon_tags_38 },
    { 0x22da8679, SUC_lexicon_tags_37 },
    { 0x5bda867a, SUC_lexicon_tags_9 },
    { 0x38fa867b, SUC_lexicon_tags_2 },
    { 0x181a867c, SUC_lexicon_tags_15 },
    { 0xa61a867b, SUC_lexicon_tags_25 },
    { 0xb7ba8678, SUC_lexicon_tags_33 },
    { 0x8fa867c, SUC_lexicon_tags_19 },
    { 0xcf9a8680, SUC_lexicon_tags_82 },
    { 0x161a8680, SUC_lexicon_tags_1 },
    { 0xc43a8682, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0da8687, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da868b, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x101a868d, SUC_lexicon_tags_47 },
    { 0x5dda868d, SUC_lexicon_tags_12 },
    { 0x2dba868f, SUC_lexicon_tags_11 },
    { 0x6cfa868f, SUC_lexicon_tags_37 },
    { 0xf87a8691, SUC_lexicon_tags_53 },
    { 0x50da868f, SUC_lexicon_tags_30 },
    { 0x727a8693, SUC_lexicon_tags_103 },
    { 0xedfa8690, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x763a8696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb87a8698, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb63a869a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ba869d, SUC_lexicon_tags_37 },
    { 0xf1ba869d, SUC_lexicon_tags_37 },
    { 0x6eba869d, SUC_lexicon_tags_14 },
    { 0xb01a86a0, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x941a86a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x577a86a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3a86a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937a86ac, SUC_lexicon_tags_14 },
    { 0xd39a86ad, SUC_lexicon_tags_72 },
    { 0xd3a86ae, SUC_lexicon_tags_12 },
    { 0xf1fa86af, SUC_lexicon_tags_37 },
    { 0x8c5a86ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x509a86b2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9fa86b4, SUC_lexicon_tags_15 },
    { 0x157a86b5, SUC_lexicon_tags_49 },
    { 0xf0da86b6, SUC_lexicon_tags_17 },
    { 0xd83a86b4, SUC_lexicon_tags_33 },
    { 0x693a86b6, SUC_lexicon_tags_17 },
    { 0x637a86b6, SUC_lexicon_tags_7 },
    { 0x635a86ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29a86be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7a86c5, SUC_lexicon_tags_31 },
    { 0x2da86c6, SUC_lexicon_tags_1 },
    { 0x617a86c7, SUC_lexicon_tags_15 },
    { 0x69da86c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ba86cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7a86d4, SUC_lexicon_tags_2 },
    { 0x9bba86d4, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0x7d9a86d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539a86dd, SUC_lexicon_tags_12 },
    { 0x9e3a86dd, SUC_lexicon_tags_63 },
    { 0x8d5a86df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cfa86e1, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dba86e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7a86e9, SUC_lexicon_tags_1 },
    { 0x591a86e9, SUC_lexicon_tags_2 },
    { 0x3ada86eb, SUC_lexicon_tags_38 },
    { 0x293a86ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fda86ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7a86f2, SUC_lexicon_tags_34 },
    { 0x2eba86f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379a86f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe83a86fb, SUC_lexicon_tags_25 },
    { 0x393a86fc, SUC_lexicon_tags_39 },
    { 0xd3ba86fc, SUC_lexicon_tags_63 },
    { 0x573a86fb, SUC_lexicon_tags_7 },
    { 0x93a86fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dda8702, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60fa8704, SUC_lexicon_tags_55 },
    { 0x965a8705, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa31a8707, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x79ba8709, SUC_lexicon_tags_17 },
    { 0xc45a870a, SUC_lexicon_tags_38 },
    { 0x5d1a870b, SUC_lexicon_tags_10 },
    { 0x3cfa870c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe65a870e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x67da8710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4eba8714, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33a8718, SUC_lexicon_tags_49 },
    { 0xd5a8719, SUC_lexicon_tags_6 },
    { 0x56fa8719, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aba871e, SUC_lexicon_tags_55 },
    { 0xb2da871f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x837a8722, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3dda8724, SUC_lexicon_tags_27 },
    { 0x3b3a8725, SUC_lexicon_tags_26 },
    { 0x3cfa8724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fa872a, SUC_lexicon_tags_7 },
    { 0xbfba872b, SUC_lexicon_tags_19 },
    { 0x70ba872c, SUC_lexicon_tags_46 },
    { 0x7dda872d, SUC_lexicon_tags_37 },
    { 0x6fda872c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cba8731, SUC_lexicon_tags_31 },
    { 0xfbfa8732, SUC_lexicon_tags_29 },
    { 0xb61a8733, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae5a8735, SUC_lexicon_tags_21 },
    { 0xa75a8736, SUC_lexicon_tags_49 },
    { 0xc0da8735, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9a873a, SUC_lexicon_tags_37 },
    { 0x12da873a, SUC_lexicon_tags_12 },
    { 0x2a9a873c, SUC_lexicon_tags_60 },
    { 0x8aba873d, SUC_lexicon_tags_45 },
    { 0x9a5a873e, SUC_lexicon_tags_7 },
    { 0x9d3a873f, SUC_lexicon_tags_11 },
    { 0xa1a873f, SUC_lexicon_tags_17 },
    { 0xb51a873e, SUC_lexicon_tags_5 },
    { 0xdd1a8742, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6ada8744, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ba8747, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8da874c, SUC_lexicon_tags_2 },
    { 0x3c9a874d, SUC_lexicon_tags_11 },
    { 0x77ba874e, SUC_lexicon_tags_33 },
    { 0x657a874f, SUC_lexicon_tags_11 },
    { 0x43ba8750, SUC_lexicon_tags_11 },
    { 0x7d1a8750, SUC_lexicon_tags_16 },
    { 0x9cba8752, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x895a8755, SUC_lexicon_tags_45 },
    { 0xa65a8756, SUC_lexicon_tags_33 },
    { 0x5cda8755, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xec9a8759, SUC_lexicon_tags_2 },
    { 0x73a8759, SUC_lexicon_tags_65 },
    { 0xdf9a8759, SUC_lexicon_tags_40 },
    { 0x3b9a875a, SUC_lexicon_tags_116 },
    { 0x987a875d, SUC_lexicon_tags_0 },
    { 0x6fa875d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89a8761, SUC_lexicon_tags_275 },
    { 0x4d9a8762, SUC_lexicon_tags_27 },
    { 0x669a8763, SUC_lexicon_tags_27 },
    { 0xbeba8761, SUC_lexicon_tags_11 },
    { 0xec5a8762, SUC_lexicon_tags_17 },
    { 0x959a8766, SUC_lexicon_tags_1 },
    { 0x465a8766, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x141a876a, SUC_lexicon_tags_27 },
    { 0x36ba876a, SUC_lexicon_tags_9 },
    { 0x71da876c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6d7a876e, SUC_lexicon_tags_23 },
    { 0xc1fa876f, SUC_lexicon_tags_9 },
    { 0xb3da8770, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbdfa8772, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7a8775, SUC_lexicon_tags_3 },
    { 0xa51a8776, SUC_lexicon_tags_17 },
    { 0x81ba8776, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xfe7a8779, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97da877d, SUC_lexicon_tags_45 },
    { 0x955a877e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3a8783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91da8786, SUC_lexicon_tags_56 },
    { 0x1a3a8787, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea7a8789, SUC_lexicon_tags_28 },
    { 0xf99a878a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785a8792, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1a8795, SUC_lexicon_tags_27 },
    { 0x55da8796, SUC_lexicon_tags_21 },
    { 0x64da8796, SUC_lexicon_tags_23 },
    { 0x863a8798, SUC_lexicon_tags_29 },
    { 0x429a8796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc59a879b, SUC_lexicon_tags_11 },
    { 0x8a3a879c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43a87a0, SUC_lexicon_tags_27 },
    { 0x7b3a87a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9a87a4, SUC_lexicon_tags_45 },
    { 0x507a87a5, SUC_lexicon_tags_1 },
    { 0xd2da87a6, SUC_lexicon_tags_6 },
    { 0x44fa87a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1a87aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfa87ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23a87b1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99a87b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45a87be, SUC_lexicon_tags_17 },
    { 0xbc5a87be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69a87c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17a87c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x77ba87c9, SUC_lexicon_tags_37 },
    { 0x6c3a87c9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01a87cd, SUC_lexicon_tags_26 },
    { 0x5c5a87ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffba87d2, SUC_lexicon_tags_17 },
    { 0x4f7a87d3, SUC_lexicon_tags_19 },
    { 0x263a87d3, SUC_lexicon_tags_38 },
    { 0xdd7a87d5, SUC_lexicon_tags_47 },
    { 0xa8ba87d6, SUC_lexicon_tags_5 },
    { 0x49da87d7, SUC_lexicon_tags_8 },
    { 0x5cda87d8, SUC_lexicon_tags_15 },
    { 0xa55a87d8, SUC_lexicon_tags_11 },
    { 0x499a87da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dba87df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf5a87e1, SUC_lexicon_tags_49 },
    { 0x813a87e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eda87e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x571a87e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x523a87ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311a87f2, SUC_lexicon_tags_37 },
    { 0xa7fa87f3, SUC_lexicon_tags_31 },
    { 0xb29a87f4, SUC_lexicon_tags_17 },
    { 0x1bfa87f5, SUC_lexicon_tags_45 },
    { 0x32fa87f6, SUC_lexicon_tags_15 },
    { 0x6b7a87f7, SUC_lexicon_tags_29 },
    { 0x3aba87f8, SUC_lexicon_tags_17 },
    { 0x791a87f9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbba87fe, SUC_lexicon_tags_49 },
    { 0xcb3a87fe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ba8804, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5a8807, SUC_lexicon_tags_15 },
    { 0x359a8808, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3a880b, SUC_lexicon_tags_38 },
    { 0xd45a880b, SUC_lexicon_tags_9 },
    { 0xde9a880d, SUC_lexicon_tags_25 },
    { 0xaf7a880d, SUC_lexicon_tags_11 },
    { 0x815a880f, SUC_lexicon_tags_4 },
    { 0xd73a880f, SUC_lexicon_tags_15 },
    { 0xa07a8810, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ba8815, SUC_lexicon_tags_9 },
    { 0x9fa8816, SUC_lexicon_tags_0 },
    { 0xde5a8817, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x68da881a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x681a881d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e7a881f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cda8823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x783a8825, SUC_lexicon_tags_0 },
    { 0xec1a8826, SUC_lexicon_tags_34 },
    { 0x13ba8826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcda882a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bda882d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ba8833, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x733a8835, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e3a8837, SUC_lexicon_tags_37 },
    { 0x3f9a8838, SUC_lexicon_tags_25 },
    { 0x925a8837, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fa883c, SUC_lexicon_tags_138 },
    { 0x903a883d, SUC_lexicon_tags_28 },
    { 0x45a883c, SUC_lexicon_tags_5 },
    { 0x917a883c, SUC_lexicon_tags_10 },
    { 0xea5a8840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afa8844, SUC_lexicon_tags_45 },
    { 0x30da8845, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x937a8847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4ba8849, SUC_lexicon_tags_37 },
    { 0x11fa884a, SUC_lexicon_tags_55 },
    { 0x9e9a884b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc11a884d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc3a884f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7a8856, SUC_lexicon_tags_63 },
    { 0x803a8857, SUC_lexicon_tags_62 },
    { 0x36fa8858, SUC_lexicon_tags_26 },
    { 0xf9a8859, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8da885c, SUC_lexicon_tags_2 },
    { 0xa29a885d, SUC_lexicon_tags_15 },
    { 0xadda885e, SUC_lexicon_tags_30 },
    { 0x4b9a885c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ba886a, SUC_lexicon_tags_60 },
    { 0xb89a886a, SUC_lexicon_tags_6 },
    { 0x615a886a, SUC_lexicon_tags_12 },
    { 0xe3a886d, SUC_lexicon_tags_17 },
    { 0x5f1a886d, SUC_lexicon_tags_25 },
    { 0xaefa886f, SUC_lexicon_tags_25 },
    { 0x14fa886a, SUC_lexicon_tags_17 },
    { 0x9b9a8871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1a8875, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ba8877, SUC_lexicon_tags_37 },
    { 0xba3a8877, SUC_lexicon_tags_43 },
    { 0x7ada8878, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa0ba887b, SUC_lexicon_tags_11 },
    { 0x85fa887c, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xe91a887e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xffba8880, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735a8885, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefa888a, SUC_lexicon_tags_14 },
    { 0xb21a888b, SUC_lexicon_tags_25 },
    { 0xbbda888c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c9a888e, SUC_lexicon_tags_127 },
    { 0x847a888f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5a8894, SUC_lexicon_tags_8 },
    { 0xeaba8895, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fa8898, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3bfa889a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc1a889c, SUC_lexicon_tags_60 },
    { 0x23a889d, SUC_lexicon_tags_27 },
    { 0x13da889d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9a88a2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bda88a5, SUC_lexicon_tags_47 },
    { 0x57a88a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x495a88a8, SUC_lexicon_tags_1 },
    { 0xec9a88a9, SUC_lexicon_tags_0 },
    { 0xd1da88a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x659a88ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3a88b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33a88b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fa88b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x411a88ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e7a88bc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a88bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fa88c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57da88c6, SUC_lexicon_tags_7 },
    { 0xda3a88c7, SUC_lexicon_tags_25 },
    { 0x6fda88c8, SUC_lexicon_tags_17 },
    { 0xadfa88c9, SUC_lexicon_tags_15 },
    { 0x819a88c9, SUC_lexicon_tags_1 },
    { 0x76ba88cb, SUC_lexicon_tags_12 },
    { 0x2e9a88cc, SUC_lexicon_tags_25 },
    { 0x31da88cc, SUC_lexicon_tags_6 },
    { 0x325a88ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf61a88d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x605a88d3, SUC_lexicon_tags_2 },
    { 0x6bda88d4, SUC_lexicon_tags_9 },
    { 0xe33a88d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefa88db, SUC_lexicon_tags_524 },
    { 0x9c9a88dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67a88de, SUC_lexicon_tags_30 },
    { 0x287a88de, SUC_lexicon_tags_15 },
    { 0x701a88de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6bfa88e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x39da88e4, SUC_lexicon_tags_0 },
    { 0xcda88e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68da88ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55a88f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ba88f6, SUC_lexicon_tags_11 },
    { 0x1c5a88f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fa88fb, SUC_lexicon_tags_60 },
    { 0xd83a88fb, SUC_lexicon_tags_6 },
    { 0x84ba88fb, SUC_lexicon_tags_26 },
    { 0x32fa88fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78da8900, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73a8906, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7da890e, SUC_lexicon_tags_12 },
    { 0x757a890f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ba8912, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21a8916, SUC_lexicon_tags_0 },
    { 0x85a8916, SUC_lexicon_tags_37 },
    { 0xea1a8918, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef9a891b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa8da891d, SUC_lexicon_tags_2 },
    { 0xc0da891e, SUC_lexicon_tags_26 },
    { 0xb9da891d, SUC_lexicon_tags_17 },
    { 0x2b3a8920, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1a8925, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113a892a, SUC_lexicon_tags_57 },
    { 0x6a9a892a, SUC_lexicon_tags_15 },
    { 0xa91a892b, SUC_lexicon_tags_28 },
    { 0x5cba892c, SUC_lexicon_tags_6 },
    { 0xe97a892e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16da8931, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaba8935, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ba893c, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49a8943, SUC_lexicon_tags_31 },
    { 0xc8ba8944, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ba8948, SUC_lexicon_tags_7 },
    { 0x18ba8949, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1a8950, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ba8954, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffda895a, SUC_lexicon_tags_23 },
    { 0x2c7a895a, SUC_lexicon_tags_37 },
    { 0xaa9a895c, SUC_lexicon_tags_0 },
    { 0x529a895a, SUC_lexicon_tags_31 },
    { 0x6f7a895e, SUC_lexicon_tags_1 },
    { 0x3fa895f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1fba8961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ba8966, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe47a8968, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7bba896a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1a8971, SUC_lexicon_tags_29 },
    { 0x99da8972, SUC_lexicon_tags_279 },
    { 0x7e9a8973, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93a8978, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73a897d, SUC_lexicon_tags_12 },
    { 0xd11a897e, SUC_lexicon_tags_1 },
    { 0x6aba897e, SUC_lexicon_tags_27 },
    { 0x1e5a897f, SUC_lexicon_tags_47 },
    { 0xd3a8980, SUC_lexicon_tags_33 },
    { 0x2eda897f, SUC_lexicon_tags_84 },
    { 0x737a897d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc39a8985, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1a8989, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f3a898b, SUC_lexicon_tags_27 },
    { 0x7b5a898c, SUC_lexicon_tags_7 },
    { 0xb55a898b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dba8990, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7a8993, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fda8996, SUC_lexicon_tags_87 },
    { 0x7ba8997, SUC_lexicon_tags_83 },
    { 0x37da8997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931a899c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dfa899e, SUC_lexicon_tags_0 },
    { 0x6bfa899e, SUC_lexicon_tags_7 },
    { 0xf43a899e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x129a89a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3a89a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8da89ab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35a89b7, SUC_lexicon_tags_17 },
    { 0xd1da89b8, SUC_lexicon_tags_141 },
    { 0x16ba89b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x497a89bb, SUC_lexicon_tags_33 },
    { 0x2bba89bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcda89bf, SUC_lexicon_tags_27 },
    { 0xf6ba89c0, SUC_lexicon_tags_37 },
    { 0x735a89c1, SUC_lexicon_tags_8 },
    { 0x4fba89c0, SUC_lexicon_tags_1 },
    { 0x699a89c0, SUC_lexicon_tags_9 },
    { 0x73a89c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x59ba89c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3da89ca, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4da89d0, SUC_lexicon_tags_25 },
    { 0xf7fa89d1, SUC_lexicon_tags_65 },
    { 0x26ba89d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3da89d5, SUC_lexicon_tags_23 },
    { 0x6e9a89d5, SUC_lexicon_tags_0 },
    { 0x9bda89d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3a89da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf21a89dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a89e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x229a89e3, SUC_lexicon_tags_6 },
    { 0xd79a89e4, SUC_lexicon_tags_32 },
    { 0x231a89e5, SUC_lexicon_tags_27 },
    { 0xbd3a89e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9a89e9, SUC_lexicon_tags_11 },
    { 0xb15a89ea, SUC_lexicon_tags_9 },
    { 0xb6fa89eb, SUC_lexicon_tags_19 },
    { 0xbf1a89ec, SUC_lexicon_tags_2 },
    { 0xa99a89ec, SUC_lexicon_tags_7 },
    { 0xd91a89ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57a89f4, SUC_lexicon_tags_14 },
    { 0x925a89f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x433a89f8, SUC_lexicon_tags_27 },
    { 0xd6fa89f9, SUC_lexicon_tags_11 },
    { 0xf03a89f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43a89fe, SUC_lexicon_tags_1 },
    { 0x35ba89ff, SUC_lexicon_tags_14 },
    { 0xefba8a00, SUC_lexicon_tags_44 },
    { 0xcbba8a01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62fa8a03, SUC_lexicon_tags_61 },
    { 0x1bda8a04, SUC_lexicon_tags_16 },
    { 0xed5a8a05, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10da8a0a, SUC_lexicon_tags_2 },
    { 0x765a8a0b, SUC_lexicon_tags_45 },
    { 0x50fa8a0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10ba8a0e, SUC_lexicon_tags_31 },
    { 0x90da8a0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda5a8a11, SUC_lexicon_tags_0 },
    { 0x9a1a8a12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5a8a1a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1a1a8a1c, SUC_lexicon_tags_10 },
    { 0x3f7a8a1d, SUC_lexicon_tags_45 },
    { 0x84ba8a1e, SUC_lexicon_tags_45 },
    { 0x33ba8a1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727a8a25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7a8a2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x735a8a2d, SUC_lexicon_tags_15 },
    { 0xbffa8a2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x669a8a30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ba8a33, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ba8a36, SUC_lexicon_tags_37 },
    { 0xad5a8a37, SUC_lexicon_tags_6 },
    { 0x5b7a8a37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25a8a3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649a8a3f, SUC_lexicon_tags_15 },
    { 0x9ada8a3f, SUC_lexicon_tags_29 },
    { 0xfcda8a3f, SUC_lexicon_tags_1 },
    { 0x6c1a8a3f, SUC_lexicon_tags_18 },
    { 0xedfa8a43, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x627a8a46, SUC_lexicon_tags_49 },
    { 0xbdda8a47, SUC_lexicon_tags_7 },
    { 0x913a8a48, SUC_lexicon_tags_11 },
    { 0xe2da8a49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97a8a4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x225a8a51, SUC_lexicon_tags_11 },
    { 0xa9a8a51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x499a8a55, SUC_lexicon_tags_25 },
    { 0xb8ba8a56, SUC_lexicon_tags_33 },
    { 0x113a8a57, SUC_lexicon_tags_118 },
    { 0x50fa8a57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9a8a5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3a8a62, SUC_lexicon_tags_1 },
    { 0x851a8a63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xacda8a65, SUC_lexicon_tags_28 },
    { 0xb79a8a66, SUC_lexicon_tags_25 },
    { 0x16ba8a67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ba8a6f, SUC_lexicon_tags_25 },
    { 0xb25a8a70, SUC_lexicon_tags_56 },
    { 0x3cba8a71, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93a8a75, SUC_lexicon_tags_29 },
    { 0x627a8a76, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03a8a7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ba8a80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x689a8a83, SUC_lexicon_tags_27 },
    { 0xe25a8a84, SUC_lexicon_tags_11 },
    { 0xd49a8a85, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfa8a8a, SUC_lexicon_tags_47 },
    { 0x43da8a8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703a8a90, SUC_lexicon_tags_14 },
    { 0x93a8a91, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51a8a96, SUC_lexicon_tags_84 },
    { 0x8fba8a97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fba8a9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1da8aa2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x517a8aa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11da8aa7, SUC_lexicon_tags_2 },
    { 0xf6fa8aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x537a8aab, SUC_lexicon_tags_16 },
    { 0x3c7a8aac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x799a8aaf, SUC_lexicon_tags_38 },
    { 0x2a1a8ab0, SUC_lexicon_tags_1 },
    { 0xa0fa8ab1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ba8ab4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fa8ab7, SUC_lexicon_tags_25 },
    { 0x56da8ab7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5a8ac0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1a8ac4, SUC_lexicon_tags_17 },
    { 0x3f1a8ac5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfa8ace, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39a8ad4, SUC_lexicon_tags_19 },
    { 0xd83a8ad4, SUC_lexicon_tags_25 },
    { 0xa11a8ad6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7da8ad8, SUC_lexicon_tags_11 },
    { 0x43a8ad9, SUC_lexicon_tags_32 },
    { 0x1c5a8ad8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d5a8adc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fa8ae6, SUC_lexicon_tags_17 },
    { 0xde9a8ae7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba8af0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfa8af4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fa8afe, SUC_lexicon_tags_37 },
    { 0x369a8aff, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5a8b03, SUC_lexicon_tags_1 },
    { 0x83a8b03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9da8b09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x873a8b0c, SUC_lexicon_tags_63 },
    { 0x365a8b0d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x84da8b10, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x50da8b12, SUC_lexicon_tags_11 },
    { 0xd33a8b13, SUC_lexicon_tags_43 },
    { 0xa97a8b14, SUC_lexicon_tags_60 },
    { 0x5a5a8b15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbda8b17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29a8b1c, SUC_lexicon_tags_17 },
    { 0xf5ba8b1c, SUC_lexicon_tags_21 },
    { 0xa8ba8b1d, SUC_lexicon_tags_179 },
    { 0xd97a8b1f, SUC_lexicon_tags_44 },
    { 0x26ba8b1f, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0xba7a8b22, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27a8b26, SUC_lexicon_tags_19 },
    { 0xe3da8b27, SUC_lexicon_tags_7 },
    { 0x489a8b27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33a8b2b, SUC_lexicon_tags_11 },
    { 0xcb9a8b2c, SUC_lexicon_tags_29 },
    { 0x181a8b2c, SUC_lexicon_tags_8 },
    { 0xdaba8b2e, SUC_lexicon_tags_84 },
    { 0x3eba8b2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7a8b33, SUC_lexicon_tags_2 },
    { 0xf2da8b34, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xda7a8b36, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615a8b3a, SUC_lexicon_tags_68 },
    { 0x277a8b3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc37a8b3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x837a8b40, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x92ba8b42, SUC_lexicon_tags_31 },
    { 0xd33a8b43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b5a8b45, SUC_lexicon_tags_32 },
    { 0xd5ba8b46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6ba8b48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853a8b4c, SUC_lexicon_tags_2 },
    { 0x9b1a8b4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fa8b50, SUC_lexicon_tags_25 },
    { 0x94ba8b51, SUC_lexicon_tags_68 },
    { 0xce7a8b52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5a8b56, SUC_lexicon_tags_25 },
    { 0x505a8b57, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59a8b5d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fa8b62, SUC_lexicon_tags_1 },
    { 0xe77a8b63, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdfda8b65, SUC_lexicon_tags_16 },
    { 0xc69a8b66, SUC_lexicon_tags_17 },
    { 0x897a8b66, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3e1a8b69, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x29da8b6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc13a8b6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7a8b72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3a8b79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3a8b7c, SUC_lexicon_tags_8 },
    { 0xfb7a8b7c, SUC_lexicon_tags_38 },
    { 0xe0fa8b7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffa8b82, SUC_lexicon_tags_7 },
    { 0xb5da8b83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0da8b85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aba8b89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4a9a8b8b, SUC_lexicon_tags_8 },
    { 0xd25a8b8b, SUC_lexicon_tags_26 },
    { 0x8f3a8b8d, SUC_lexicon_tags_45 },
    { 0x5a8b8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfa8b92, SUC_lexicon_tags_29 },
    { 0xfd9a8b93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3a8b99, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9a8ba1, SUC_lexicon_tags_6 },
    { 0x913a8ba1, SUC_lexicon_tags_30 },
    { 0x769a8ba1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3a8ba6, SUC_lexicon_tags_21 },
    { 0x417a8ba7, SUC_lexicon_tags_1 },
    { 0x19a8ba6, SUC_lexicon_tags_17 },
    { 0xda7a8ba9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9cba8bab, SUC_lexicon_tags_65 },
    { 0xa5fa8bac, SUC_lexicon_tags_15 },
    { 0x211a8bad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fa8bb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213a8bb8, SUC_lexicon_tags_15 },
    { 0x275a8bb9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x147a8bbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1a8bbf, SUC_lexicon_tags_39 },
    { 0x585a8bc0, SUC_lexicon_tags_33 },
    { 0x895a8bc1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5a8bc5, SUC_lexicon_tags_31 },
    { 0x413a8bc6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x76fa8bc8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x17a8bcb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713a8bd0, SUC_lexicon_tags_28 },
    { 0x8fa8bd1, SUC_lexicon_tags_17 },
    { 0x407a8bd1, SUC_lexicon_tags_8 },
    { 0x911a8bd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf7a8bd5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73da8bda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x103a8bdd, SUC_lexicon_tags_1 },
    { 0x6da8bdd, SUC_lexicon_tags_37 },
    { 0x853a8bdf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3a8be4, SUC_lexicon_tags_23 },
    { 0x129a8be5, SUC_lexicon_tags_37 },
    { 0x9b9a8be6, SUC_lexicon_tags_27 },
    { 0xcbfa8be7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa95a8be9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x67da8bec, SUC_lexicon_tags_33 },
    { 0x629a8bed, SUC_lexicon_tags_11 },
    { 0xa9ba8bed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9a8bf1, SUC_lexicon_tags_174 },
    { 0xea5a8bf2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x97a8bf4, SUC_lexicon_tags_37 },
    { 0x9dba8bf5, SUC_lexicon_tags_39 },
    { 0x29a8bf5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7fda8bf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5a8c01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35a8c06, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09a8c12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfba8c19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x345a8c1b, SUC_lexicon_tags_1 },
    { 0x4d3a8c1b, SUC_lexicon_tags_17 },
    { 0x315a8c1b, SUC_lexicon_tags_2 },
    { 0x5aba8c1d, SUC_lexicon_tags_7 },
    { 0x92ba8c1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ba8c22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361a8c29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe65a8c2b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcdda8c2d, SUC_lexicon_tags_1401 },
    { 0x31a8c2e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fba8c31, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x133a8c34, SUC_lexicon_tags_7 },
    { 0x96ba8c34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x873a8c37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcfa8c39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475a8c41, SUC_lexicon_tags_17 },
    { 0x4eba8c41, SUC_lexicon_tags_2 },
    { 0xa55a8c43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05a8c46, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ba8c51, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x46da8c53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5a8c56, SUC_lexicon_tags_55 },
    { 0x771a8c56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47a8c5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ba8c61, SUC_lexicon_tags_17 },
    { 0x469a8c61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3a8c68, SUC_lexicon_tags_25 },
    { 0xf3ba8c69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3a8c6f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x94da8c72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd27a8c74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeba8c78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401a8c7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x35da8c7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5a8c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x715a8c85, SUC_lexicon_tags_25 },
    { 0x8bba8c86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdada8c8a, SUC_lexicon_tags_53 },
    { 0xe7a8c8b, SUC_lexicon_tags_141 },
    { 0xfba8c8c, SUC_lexicon_tags_48 },
    { 0xccba8c8c, SUC_lexicon_tags_30 },
    { 0xb0fa8c8e, SUC_lexicon_tags_10 },
    { 0xedda8c8f, SUC_lexicon_tags_29 },
    { 0x2cda8c8f, SUC_lexicon_tags_14 },
    { 0x2d3a8c90, SUC_lexicon_tags_26 },
    { 0x539a8c8a, SUC_lexicon_tags_19 },
    { 0xba5a8c8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb79a8c95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeda8c9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177a8ca6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x147a8ca8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa27a8caa, SUC_lexicon_tags_60 },
    { 0xdc3a8cab, SUC_lexicon_tags_37 },
    { 0x2e3a8cab, SUC_lexicon_tags_11 },
    { 0x355a8cad, SUC_lexicon_tags_16 },
    { 0x531a8cad, SUC_lexicon_tags_47 },
    { 0xf71a8cab, SUC_lexicon_tags_11 },
    { 0xf1ba8cb0, SUC_lexicon_tags_33 },
    { 0x69da8caa, SUC_lexicon_tags_129 },
    { 0xbd3a8cb2, SUC_lexicon_tags_31 },
    { 0x1f7a8cb3, SUC_lexicon_tags_28 },
    { 0xc37a8cb4, SUC_lexicon_tags_31 },
    { 0xe37a8cb5, SUC_lexicon_tags_1 },
    { 0xbe7a8cb6, SUC_lexicon_tags_33 },
    { 0x165a8cb7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9a8cbb, SUC_lexicon_tags_30 },
    { 0xdcfa8cbb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x54fa8cbe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb49a8cc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebda8cc4, SUC_lexicon_tags_4 },
    { 0x29da8cc5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5a8cc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23a8ccc, SUC_lexicon_tags_4 },
    { 0xc4da8ccc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x833a8ccf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x365a8cd1, SUC_lexicon_tags_15 },
    { 0x463a8cd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfafa8cd4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbda8cd6, SUC_lexicon_tags_47 },
    { 0x29ba8cd7, SUC_lexicon_tags_49 },
    { 0x5bda8cd8, SUC_lexicon_tags_15 },
    { 0x975a8cd9, SUC_lexicon_tags_18 },
    { 0xdfba8cd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5a8ce1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdda8ce8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x393a8ceb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeba8cee, SUC_lexicon_tags_37 },
    { 0x96fa8cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcba8cf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6efa8cf4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71a8cf7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579a8cfc, SUC_lexicon_tags_37 },
    { 0xba5a8cfc, SUC_lexicon_tags_7 },
    { 0x7a9a8cfc, SUC_lexicon_tags_1 },
    { 0x8a3a8cfd, SUC_lexicon_tags_26 },
    { 0x2c3a8d00, SUC_lexicon_tags_16 },
    { 0xed1a8d01, SUC_lexicon_tags_1 },
    { 0xcb9a8d02, SUC_lexicon_tags_29 },
    { 0x8a5a8cfd, SUC_lexicon_tags_25 },
    { 0x253a8d04, SUC_lexicon_tags_60 },
    { 0xcffa8d05, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ba8d10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dba8d14, SUC_lexicon_tags_49 },
    { 0x43ba8d14, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb41a8d17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505a8d1b, SUC_lexicon_tags_17 },
    { 0xc5a8d1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x86da8d1e, SUC_lexicon_tags_15 },
    { 0x3ba8d1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a1a8d21, SUC_lexicon_tags_37 },
    { 0xbfba8d22, SUC_lexicon_tags_26 },
    { 0xdba8d22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efa8d28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ba8d2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb09a8d2e, SUC_lexicon_tags_37 },
    { 0x3d9a8d2f, SUC_lexicon_tags_69 },
    { 0x3ba8d30, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc89a8d32, SUC_lexicon_tags_27 },
    { 0x5d3a8d33, SUC_lexicon_tags_17 },
    { 0xe35a8d34, SUC_lexicon_tags_12 },
    { 0x92fa8d34, SUC_lexicon_tags_37 },
    { 0x64ba8d36, SUC_lexicon_tags_11 },
    { 0x1f9a8d32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x481a8d3a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xafba8d3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671a8d41, SUC_lexicon_tags_12 },
    { 0x989a8d42, SUC_lexicon_tags_12 },
    { 0xfb1a8d43, SUC_lexicon_tags_11 },
    { 0xd41a8d41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ba8d47, SUC_lexicon_tags_0 },
    { 0x19ba8d47, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x21da8d4a, SUC_lexicon_tags_0 },
    { 0x321a8d4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x183a8d4d, SUC_lexicon_tags_47 },
    { 0xa9fa8d4d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0da8d57, SUC_lexicon_tags_0 },
    { 0x20da8d57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f5a8d5a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc73a8d5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45a8d5f, SUC_lexicon_tags_11 },
    { 0xe41a8d5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0da8d62, SUC_lexicon_tags_6 },
    { 0xcc5a8d63, SUC_lexicon_tags_38 },
    { 0x9afa8d63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3a8d68, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xebfa8d6a, SUC_lexicon_tags_8 },
    { 0xcfa8d6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfda8d6f, SUC_lexicon_tags_47 },
    { 0xbafa8d6f, SUC_lexicon_tags_37 },
    { 0xadda8d71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffa8d76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113a8d7a, SUC_lexicon_tags_7 },
    { 0xdf5a8d7a, SUC_lexicon_tags_4 },
    { 0x861a8d7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bba8d84, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dba8d8b, SUC_lexicon_tags_46 },
    { 0x8b7a8d8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ba8d8f, SUC_lexicon_tags_37 },
    { 0x41a8d90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x621a8d92, SUC_lexicon_tags_17 },
    { 0x555a8d92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9a8d97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663a8d9b, SUC_lexicon_tags_25 },
    { 0x467a8d9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fa8da0, SUC_lexicon_tags_363 },
    { 0x849a8da0, SUC_lexicon_tags_38 },
    { 0xdaba8da2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x185a8da4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953a8da9, SUC_lexicon_tags_56 },
    { 0xae9a8da9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fa8db0, SUC_lexicon_tags_27 },
    { 0x727a8db0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37a8db5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7a8dba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27a8dbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x697a8dc0, SUC_lexicon_tags_12 },
    { 0x581a8dc1, SUC_lexicon_tags_7 },
    { 0x25ba8dc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x741a8dc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417a8dc9, SUC_lexicon_tags_17 },
    { 0x7cba8dca, SUC_lexicon_tags_2 },
    { 0x343a8dc9, SUC_lexicon_tags_25 },
    { 0x5b9a8dc9, SUC_lexicon_tags_15 },
    { 0xebba8dcd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247a8dd8, SUC_lexicon_tags_8 },
    { 0x79da8dd8, SUC_lexicon_tags_46 },
    { 0x111a8dd8, SUC_lexicon_tags_1 },
    { 0x2efa8dd9, SUC_lexicon_tags_12 },
    { 0x2f9a8ddc, SUC_lexicon_tags_37 },
    { 0x76ba8ddb, SUC_lexicon_tags_9 },
    { 0xb91a8dde, SUC_lexicon_tags_26 },
    { 0xb41a8dde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ba8de6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105a8df2, SUC_lexicon_tags_12 },
    { 0xd3da8df3, SUC_lexicon_tags_25 },
    { 0xea3a8df4, SUC_lexicon_tags_305 },
    { 0x793a8df3, SUC_lexicon_tags_14 },
    { 0xa1da8df6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bfa8df8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1da8dfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc41a8dfe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9a8e04, SUC_lexicon_tags_37 },
    { 0x90da8e05, SUC_lexicon_tags_25 },
    { 0x42da8e06, SUC_lexicon_tags_63 },
    { 0x29ba8e07, SUC_lexicon_tags_11 },
    { 0xc33a8e06, SUC_lexicon_tags_17 },
    { 0x9ba8e09, SUC_lexicon_tags_25 },
    { 0xd87a8e0a, SUC_lexicon_tags_28 },
    { 0xc7ba8e0b, SUC_lexicon_tags_45 },
    { 0xed7a8e0a, SUC_lexicon_tags_30 },
    { 0x517a8e09, SUC_lexicon_tags_48 },
    { 0x101a8e04, SUC_lexicon_tags_82 },
    { 0xe59a8e0c, SUC_lexicon_tags_22 },
    { 0xb19a8e06, SUC_lexicon_tags_30 },
    { 0x5b1a8e06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457a8e1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x635a8e1c, SUC_lexicon_tags_15 },
    { 0x159a8e1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e3a8e1f, SUC_lexicon_tags_65 },
    { 0x341a8e20, SUC_lexicon_tags_33 },
    { 0x2e3a8e20, SUC_lexicon_tags_60 },
    { 0x71a8e22, SUC_lexicon_tags_15 },
    { 0x157a8e23, SUC_lexicon_tags_15 },
    { 0x2e9a8e23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3a8e2f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x30da8e31, SUC_lexicon_tags_8 },
    { 0x77a8e32, SUC_lexicon_tags_25 },
    { 0x52fa8e31, SUC_lexicon_tags_3 },
    { 0xd85a8e34, SUC_lexicon_tags_30 },
    { 0xf9a8e34, SUC_lexicon_tags_37 },
    { 0x317a8e33, SUC_lexicon_tags_1 },
    { 0x8eda8e37, SUC_lexicon_tags_56 },
    { 0x24fa8e32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x803a8e3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7a8e3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5afa8e40, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x739a8e42, SUC_lexicon_tags_15 },
    { 0x29fa8e43, SUC_lexicon_tags_37 },
    { 0xfd9a8e43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c1a8e46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8cba8e48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9a8e4e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x789a8e50, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x92da8e52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693a8e58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ffa8e5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fa8e62, SUC_lexicon_tags_20 },
    { 0xa55a8e63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a8e67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c5a8e69, SUC_lexicon_tags_15 },
    { 0x845a8e6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x625a8e6c, SUC_lexicon_tags_12 },
    { 0xbafa8e6c, SUC_lexicon_tags_28 },
    { 0x9b9a8e6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405a8e72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a7a8e74, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641a8e79, SUC_lexicon_tags_37 },
    { 0x3dfa8e7a, SUC_lexicon_tags_25 },
    { 0xb57a8e7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64da8e7d, SUC_lexicon_tags_19 },
    { 0xa57a8e7e, SUC_lexicon_tags_34 },
    { 0x129a8e7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51da8e81, SUC_lexicon_tags_38 },
    { 0x881a8e82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfa8e85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceba8e8d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fa8e91, SUC_lexicon_tags_37 },
    { 0x251a8e92, SUC_lexicon_tags_11 },
    { 0xcd3a8e93, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xe37a8e95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fda8e97, SUC_lexicon_tags_37 },
    { 0x439a8e97, SUC_lexicon_tags_7 },
    { 0x781a8e99, SUC_lexicon_tags_1 },
    { 0xff5a8e9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3da8e9c, SUC_lexicon_tags_26 },
    { 0x877a8e9d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x75a8ea0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5a8ea3, SUC_lexicon_tags_60 },
    { 0x725a8ea3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27a8ea6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfa8eab, SUC_lexicon_tags_31 },
    { 0xdcba8eac, SUC_lexicon_tags_60 },
    { 0xf3da8ead, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfba8eb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x351a8eb2, SUC_lexicon_tags_7 },
    { 0xe3fa8eb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x93fa8eb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33a8ebb, SUC_lexicon_tags_7 },
    { 0x901a8ebb, SUC_lexicon_tags_7 },
    { 0xc61a8ebd, SUC_lexicon_tags_37 },
    { 0xcf7a8ebb, SUC_lexicon_tags_12 },
    { 0xea5a8ebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807a8ec4, SUC_lexicon_tags_0 },
    { 0x349a8ec4, SUC_lexicon_tags_57 },
    { 0xfba8ec6, SUC_lexicon_tags_28 },
    { 0xe3fa8ec6, SUC_lexicon_tags_43 },
    { 0xf2ba8ec8, SUC_lexicon_tags_91 },
    { 0xe9da8ec8, SUC_lexicon_tags_5 },
    { 0xc4fa8ec5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf59a8ecc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x731a8ecf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ba8ed2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49a8ed5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xefba8ed7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdc3a8ed9, SUC_lexicon_tags_104 },
    { 0xdafa8eda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29a8ee3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47a8ee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfa8eeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17a8eef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56fa8ef1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x853a8ef3, SUC_lexicon_tags_2 },
    { 0xe5da8ef4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x51ba8ef6, SUC_lexicon_tags_7 },
    { 0x5f3a8ef7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccba8efa, SUC_lexicon_tags_25 },
    { 0x7a9a8efb, SUC_lexicon_tags_9 },
    { 0x15a8efa, SUC_lexicon_tags_9 },
    { 0x859a8efd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969a8f02, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19a8f07, SUC_lexicon_tags_11 },
    { 0x37fa8f07, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x54fa8f0a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdada8f0c, SUC_lexicon_tags_2 },
    { 0x8d5a8f0c, SUC_lexicon_tags_12 },
    { 0x66da8f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba8f12, SUC_lexicon_tags_8 },
    { 0x8b1a8f13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x687a8f16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519a8f1a, SUC_lexicon_tags_42 },
    { 0x31ba8f1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29fa8f1d, SUC_lexicon_tags_7 },
    { 0xc8da8f1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4eda8f20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3a8f24, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541a8f28, SUC_lexicon_tags_23 },
    { 0xfada8f29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3ba8f2b, SUC_lexicon_tags_17 },
    { 0xdb5a8f2c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafa8f2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25a8f38, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805a8f3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22da8f3e, SUC_lexicon_tags_25 },
    { 0x9c3a8f3f, SUC_lexicon_tags_2 },
    { 0x413a8f40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dba8f42, SUC_lexicon_tags_27 },
    { 0x4f7a8f42, SUC_lexicon_tags_37 },
    { 0x19fa8f43, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1a8f47, SUC_lexicon_tags_9 },
    { 0xe83a8f47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721a8f4c, SUC_lexicon_tags_37 },
    { 0x497a8f4c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ba8f53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61a8f58, SUC_lexicon_tags_7 },
    { 0x12da8f59, SUC_lexicon_tags_6 },
    { 0x507a8f59, SUC_lexicon_tags_23 },
    { 0x45da8f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3a8f5e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547a8f62, SUC_lexicon_tags_6 },
    { 0x663a8f62, SUC_lexicon_tags_42 },
    { 0xb7da8f64, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x8c9a8f66, SUC_lexicon_tags_15 },
    { 0x8afa8f67, SUC_lexicon_tags_27 },
    { 0x477a8f68, SUC_lexicon_tags_2 },
    { 0xa51a8f69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x927a8f6b, SUC_lexicon_tags_22 },
    { 0xd9fa8f6b, SUC_lexicon_tags_42 },
    { 0xdfa8f6d, SUC_lexicon_tags_31 },
    { 0x421a8f6e, SUC_lexicon_tags_87 },
    { 0xf9da8f6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283a8f73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707a8f79, SUC_lexicon_tags_25 },
    { 0x59da8f79, SUC_lexicon_tags_21 },
    { 0x4a9a8f7b, SUC_lexicon_tags_9 },
    { 0x37a8f7c, SUC_lexicon_tags_2 },
    { 0x82da8f7c, SUC_lexicon_tags_0 },
    { 0x281a8f7e, SUC_lexicon_tags_33 },
    { 0xf1da8f7f, SUC_lexicon_tags_1 },
    { 0xc87a8f7e, SUC_lexicon_tags_62 },
    { 0x46fa8f81, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe29a8f83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79a8f86, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afa8f8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x671a8f8e, SUC_lexicon_tags_25 },
    { 0x4d7a8f8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1da8f92, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa93a8f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2aba8f96, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ba8f99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x29a8f9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3a8fa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x267a8fa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6da8faa, SUC_lexicon_tags_25 },
    { 0x33da8faa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5eba8fae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da8fb1, SUC_lexicon_tags_1 },
    { 0x6eba8fb2, SUC_lexicon_tags_37 },
    { 0xac9a8fb3, SUC_lexicon_tags_25 },
    { 0x64da8fb4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x821a8fb6, SUC_lexicon_tags_47 },
    { 0x649a8fb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x32ba8fb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x88fa8fbb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb77a8fbd, SUC_lexicon_tags_17 },
    { 0x233a8fbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x471a8fc0, SUC_lexicon_tags_7 },
    { 0xb41a8fc1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe95a8fc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbcba8fc5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x85a8fc8, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0x55fa8fca, SUC_lexicon_tags_37 },
    { 0x457a8fcb, SUC_lexicon_tags_8 },
    { 0x9a3a8fcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb95a8fce, SUC_lexicon_tags_27 },
    { 0x1e7a8fcf, SUC_lexicon_tags_38 },
    { 0x5f5a8fce, SUC_lexicon_tags_11 },
    { 0xf0fa8fcf, SUC_lexicon_tags_63 },
    { 0x9e1a8fd2, SUC_lexicon_tags_4 },
    { 0x8f7a8fd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833a8fe3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81a8fe8, SUC_lexicon_tags_33 },
    { 0x951a8fe8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x211a8fec, SUC_lexicon_tags_4 },
    { 0xe05a8fed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x257a8ff0, SUC_lexicon_tags_11 },
    { 0xe2da8ff0, SUC_lexicon_tags_3 },
    { 0x6eba8ff1, SUC_lexicon_tags_479 },
    { 0xc49a8ff3, SUC_lexicon_tags_15 },
    { 0x6a1a8ff4, SUC_lexicon_tags_17 },
    { 0xb35a8ff5, SUC_lexicon_tags_66 },
    { 0x85fa8ff2, SUC_lexicon_tags_8 },
    { 0xa69a8ff7, SUC_lexicon_tags_31 },
    { 0x72ba8ff8, SUC_lexicon_tags_12 },
    { 0x23a8ff9, SUC_lexicon_tags_28 },
    { 0x349a8ff7, SUC_lexicon_tags_38 },
    { 0x801a8ffb, SUC_lexicon_tags_1 },
    { 0x5e5a8ffc, SUC_lexicon_tags_37 },
    { 0x2d7a8ffb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed9a8fff, SUC_lexicon_tags_32 },
    { 0x327a9000, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x273a9002, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd93a9004, SUC_lexicon_tags_2 },
    { 0x831a9005, SUC_lexicon_tags_7 },
    { 0xd57a9006, SUC_lexicon_tags_30 },
    { 0x41ba9006, SUC_lexicon_tags_25 },
    { 0x21fa9004, SUC_lexicon_tags_496 },
    { 0xab1a9005, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x981a900b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7a9012, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71ba9014, SUC_lexicon_tags_37 },
    { 0xeb1a9015, SUC_lexicon_tags_31 },
    { 0x813a9014, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x729a9019, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415a901d, SUC_lexicon_tags_17 },
    { 0xe3da901d, SUC_lexicon_tags_82 },
    { 0x64fa901f, SUC_lexicon_tags_50 },
    { 0x6dba9020, SUC_lexicon_tags_30 },
    { 0x373a9020, SUC_lexicon_tags_31 },
    { 0x7b1a9022, SUC_lexicon_tags_219 },
    { 0x86ba9022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f7a9025, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbba9027, SUC_lexicon_tags_45 },
    { 0x8bba9028, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3a902b, SUC_lexicon_tags_1 },
    { 0x5eda902b, SUC_lexicon_tags_27 },
    { 0x8f7a902b, SUC_lexicon_tags_15 },
    { 0xa7fa902c, SUC_lexicon_tags_56 },
    { 0x305a902f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2dda9031, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x49da9034, SUC_lexicon_tags_43 },
    { 0x3ada9035, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x203a9038, SUC_lexicon_tags_47 },
    { 0x3f5a9039, SUC_lexicon_tags_8 },
    { 0xe5da9039, SUC_lexicon_tags_7 },
    { 0xc1ba903a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x97a903d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47ba903f, SUC_lexicon_tags_25 },
    { 0x17da9040, SUC_lexicon_tags_104 },
    { 0x98fa9041, SUC_lexicon_tags_11 },
    { 0x283a9042, SUC_lexicon_tags_25 },
    { 0x10fa9043, SUC_lexicon_tags_17 },
    { 0xc6ba9042, SUC_lexicon_tags_33 },
    { 0x239a9042, SUC_lexicon_tags_28 },
    { 0x585a903f, SUC_lexicon_tags_7 },
    { 0x89da9041, SUC_lexicon_tags_9 },
    { 0x9f7a9048, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7a904d, SUC_lexicon_tags_2 },
    { 0xb0da904e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x509a9050, SUC_lexicon_tags_7 },
    { 0x38fa9051, SUC_lexicon_tags_0 },
    { 0xf35a9052, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931a9056, SUC_lexicon_tags_17 },
    { 0x4fba9057, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x477a9059, SUC_lexicon_tags_38 },
    { 0x941a905a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x17a905c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a905f, SUC_lexicon_tags_15 },
    { 0x797a905f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63a9063, SUC_lexicon_tags_37 },
    { 0x457a9064, SUC_lexicon_tags_37 },
    { 0xfa3a9063, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x691a9067, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fba9069, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cba906c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4a1a906e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7a9071, SUC_lexicon_tags_39 },
    { 0xc09a9071, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ba9077, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4fda9079, SUC_lexicon_tags_37 },
    { 0xb1fa907a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33ba907c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2da907f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9a9082, SUC_lexicon_tags_18 },
    { 0x2afa9082, SUC_lexicon_tags_31 },
    { 0xb2fa9084, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ba908a, SUC_lexicon_tags_23 },
    { 0x3b3a908b, SUC_lexicon_tags_11 },
    { 0x2c9a908c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x317a908e, SUC_lexicon_tags_43 },
    { 0xa65a908f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x21a9091, SUC_lexicon_tags_25 },
    { 0x37ba9091, SUC_lexicon_tags_7 },
    { 0x99ba9093, SUC_lexicon_tags_25 },
    { 0x83da9094, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5d9a9096, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19a909a, SUC_lexicon_tags_49 },
    { 0x217a909b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5a90a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dda90a7, SUC_lexicon_tags_27 },
    { 0x1cda90a8, SUC_lexicon_tags_17 },
    { 0x31da90a9, SUC_lexicon_tags_19 },
    { 0x2a7a90a8, SUC_lexicon_tags_63 },
    { 0xdf5a90ab, SUC_lexicon_tags_25 },
    { 0x5c1a90ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507a90b4, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x507a90b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5a90b9, SUC_lexicon_tags_160 },
    { 0xbb1a90ba, SUC_lexicon_tags_11 },
    { 0x4e1a90b9, SUC_lexicon_tags_11 },
    { 0x6dfa90bc, SUC_lexicon_tags_37 },
    { 0x225a90bd, SUC_lexicon_tags_15 },
    { 0xab7a90ba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x87da90c1, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x9afa90c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a1a90c5, SUC_lexicon_tags_9 },
    { 0x72ba90c6, SUC_lexicon_tags_80 },
    { 0xbdba90c6, SUC_lexicon_tags_33 },
    { 0x381a90c7, SUC_lexicon_tags_2 },
    { 0x45fa90c9, SUC_lexicon_tags_15 },
    { 0x14ba90ca, SUC_lexicon_tags_11 },
    { 0x131a90cb, SUC_lexicon_tags_16 },
    { 0x1b3a90cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffa90d0, SUC_lexicon_tags_30 },
    { 0x341a90d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ba90d4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61a90d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3a90de, SUC_lexicon_tags_11 },
    { 0x94ba90df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b9a90e1, SUC_lexicon_tags_7 },
    { 0xd5a90e1, SUC_lexicon_tags_33 },
    { 0x629a90e3, SUC_lexicon_tags_9 },
    { 0xf8fa90e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd51a90e6, SUC_lexicon_tags_30 },
    { 0x8dfa90e6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x157a90ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967a90f4, SUC_lexicon_tags_37 },
    { 0x3e7a90f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfa90fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efa9101, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fba9104, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcdda9106, SUC_lexicon_tags_82 },
    { 0x53ba9107, SUC_lexicon_tags_21 },
    { 0x631a9107, SUC_lexicon_tags_9 },
    { 0x1c9a9107, SUC_lexicon_tags_7 },
    { 0xe7da910a, SUC_lexicon_tags_39 },
    { 0x589a910b, SUC_lexicon_tags_17 },
    { 0xdefa910b, SUC_lexicon_tags_32 },
    { 0x635a910d, SUC_lexicon_tags_12 },
    { 0xd1da9109, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cba9111, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x99fa9114, SUC_lexicon_tags_7 },
    { 0xf7fa9114, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd9a9117, SUC_lexicon_tags_15 },
    { 0x681a9118, SUC_lexicon_tags_15 },
    { 0xb19a9118, SUC_lexicon_tags_32 },
    { 0x367a9117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfba911c, SUC_lexicon_tags_7 },
    { 0xd11a911d, SUC_lexicon_tags_25 },
    { 0x66da911d, SUC_lexicon_tags_33 },
    { 0x703a911f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1a9121, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x495a9123, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4da9128, SUC_lexicon_tags_6 },
    { 0xb41a9129, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3a912d, SUC_lexicon_tags_47 },
    { 0x84fa912d, SUC_lexicon_tags_49 },
    { 0x723a912f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7cfa9131, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe4ba9133, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fda9138, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6b3a913a, SUC_lexicon_tags_30 },
    { 0x155a913b, SUC_lexicon_tags_11 },
    { 0xd9ba913c, SUC_lexicon_tags_25 },
    { 0x569a913c, SUC_lexicon_tags_38 },
    { 0x52ba913b, SUC_lexicon_tags_15 },
    { 0xbf3a913e, SUC_lexicon_tags_1 },
    { 0x68ba913c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ba9145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66fa9147, SUC_lexicon_tags_66 },
    { 0x309a9148, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x41da914b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a9150, SUC_lexicon_tags_15 },
    { 0x825a9151, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fa9156, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x85a9158, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x737a915a, SUC_lexicon_tags_15 },
    { 0xbb5a915b, SUC_lexicon_tags_33 },
    { 0x443a915c, SUC_lexicon_tags_37 },
    { 0xa3da915a, SUC_lexicon_tags_0 },
    { 0x1cba915e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa41a9160, SUC_lexicon_tags_9 },
    { 0x777a9161, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x499a9163, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5a9168, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19a916c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39a9171, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7a9174, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57a9178, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409a917c, SUC_lexicon_tags_7 },
    { 0x56da917d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7a9183, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x497a9185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55a918b, SUC_lexicon_tags_63 },
    { 0x8b9a918b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5fda918e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84fa9190, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa1da9192, SUC_lexicon_tags_30 },
    { 0xc43a9193, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbda9197, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ba919a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ba919e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77a91a4, SUC_lexicon_tags_11 },
    { 0x38fa91a5, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73a91a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3a91ac, SUC_lexicon_tags_15 },
    { 0xc33a91ad, SUC_lexicon_tags_63 },
    { 0x5b7a91ae, SUC_lexicon_tags_37 },
    { 0xaaba91af, SUC_lexicon_tags_38 },
    { 0xa2fa91ae, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8dfa91b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3a91b5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5a91b9, SUC_lexicon_tags_25 },
    { 0xd77a91ba, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ada91bf, SUC_lexicon_tags_12 },
    { 0x975a91c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x501a91c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353a91c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5a91ca, SUC_lexicon_tags_37 },
    { 0xc1fa91cb, SUC_lexicon_tags_39 },
    { 0xc4da91cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ffa91ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983a91d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x923a91da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3a91dd, SUC_lexicon_tags_15 },
    { 0x54da91de, SUC_lexicon_tags_31 },
    { 0x5e1a91df, SUC_lexicon_tags_96 },
    { 0xda7a91e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc8ba91e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5a91e4, SUC_lexicon_tags_33 },
    { 0xee7a91e4, SUC_lexicon_tags_25 },
    { 0x359a91e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf63a91e8, SUC_lexicon_tags_2 },
    { 0x92ba91e8, SUC_lexicon_tags_4 },
    { 0xf8ba91e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85a91ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x997a91f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x481a91f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7a91fb, SUC_lexicon_tags_7 },
    { 0xf5fa91fc, SUC_lexicon_tags_1 },
    { 0x923a91fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fa9201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833a9205, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xad7a9207, SUC_lexicon_tags_25 },
    { 0x4a7a9207, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x687a920a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bba9210, SUC_lexicon_tags_92 },
    { 0x8bfa9211, SUC_lexicon_tags_12 },
    { 0x2f9a9211, SUC_lexicon_tags_31 },
    { 0x7dda9213, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x449a9215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7a9218, SUC_lexicon_tags_32 },
    { 0x521a9219, SUC_lexicon_tags_85 },
    { 0x8fba921a, SUC_lexicon_tags_106 },
    { 0x50fa921b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ffa921d, SUC_lexicon_tags_70 },
    { 0xad3a921e, SUC_lexicon_tags_1 },
    { 0x173a921e, SUC_lexicon_tags_17 },
    { 0x235a921d, SUC_lexicon_tags_17 },
    { 0x1c9a9221, SUC_lexicon_tags_21 },
    { 0xbefa9222, SUC_lexicon_tags_25 },
    { 0xcdba9222, SUC_lexicon_tags_15 },
    { 0x637a9224, SUC_lexicon_tags_22 },
    { 0x919a9225, SUC_lexicon_tags_11 },
    { 0x5e5a9226, SUC_lexicon_tags_49 },
    { 0x385a9227, SUC_lexicon_tags_2 },
    { 0x171a9222, SUC_lexicon_tags_33 },
    { 0x3e1a9229, SUC_lexicon_tags_1 },
    { 0x77a921d, SUC_lexicon_tags_9 },
    { 0xa57a9226, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x495a922d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc67a922f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x969a9231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eda9236, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ba923a, SUC_lexicon_tags_11 },
    { 0xe8fa923a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbda923f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaf9a9241, SUC_lexicon_tags_84 },
    { 0x9d7a9241, SUC_lexicon_tags_2 },
    { 0x69ba9241, SUC_lexicon_tags_8 },
    { 0x6b1a9244, SUC_lexicon_tags_57 },
    { 0x675a9244, SUC_lexicon_tags_14 },
    { 0xa87a9241, SUC_lexicon_tags_2 },
    { 0xacfa9245, SUC_lexicon_tags_15 },
    { 0x5cfa9248, SUC_lexicon_tags_38 },
    { 0xb59a9248, SUC_lexicon_tags_37 },
    { 0x513a9246, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7a9251, SUC_lexicon_tags_1 },
    { 0x7dfa9252, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afa9256, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2ffa9258, SUC_lexicon_tags_10 },
    { 0x991a9259, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dda925d, SUC_lexicon_tags_21 },
    { 0x1e1a925e, SUC_lexicon_tags_37 },
    { 0x75da925e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ba9263, SUC_lexicon_tags_63 },
    { 0x69a9263, SUC_lexicon_tags_84 },
    { 0x33a9265, SUC_lexicon_tags_138 },
    { 0x2d7a9265, SUC_lexicon_tags_23 },
    { 0x671a9265, SUC_lexicon_tags_15 },
    { 0x671a9267, SUC_lexicon_tags_27 },
    { 0xaf3a9265, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fa926e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3da9270, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9fa9272, SUC_lexicon_tags_21 },
    { 0xbba9273, SUC_lexicon_tags_27 },
    { 0x1e9a9273, SUC_lexicon_tags_39 },
    { 0x447a9274, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf07a9277, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ba927d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6da927f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb03a9281, SUC_lexicon_tags_14 },
    { 0x681a9282, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31a9287, SUC_lexicon_tags_1 },
    { 0xfb3a9288, SUC_lexicon_tags_15 },
    { 0x1ada9288, SUC_lexicon_tags_27 },
    { 0x785a928a, SUC_lexicon_tags_17 },
    { 0x11a9288, SUC_lexicon_tags_68 },
    { 0x5a1a9288, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3a9291, SUC_lexicon_tags_7 },
    { 0xc55a9292, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcba9294, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505a9298, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafa929d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47a92a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99a92a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3a92ac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xca3a92ae, SUC_lexicon_tags_14 },
    { 0xd65a92ae, SUC_lexicon_tags_7 },
    { 0x4dfa92ae, SUC_lexicon_tags_45 },
    { 0x5ba92b1, SUC_lexicon_tags_14 },
    { 0xc11a92b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679a92b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefa92ba, SUC_lexicon_tags_92 },
    { 0x3f1a92ba, SUC_lexicon_tags_23 },
    { 0x2da92bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65a92c0, SUC_lexicon_tags_25 },
    { 0x26fa92c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbffa92c3, SUC_lexicon_tags_10 },
    { 0x92ba92c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x669a92c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507a92cb, SUC_lexicon_tags_10 },
    { 0x893a92cc, SUC_lexicon_tags_23 },
    { 0x8dba92cc, SUC_lexicon_tags_29 },
    { 0x6cfa92ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee5a92d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc31a92d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985a92d6, SUC_lexicon_tags_11 },
    { 0x29da92d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ba92de, SUC_lexicon_tags_1009 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1a92e2, SUC_lexicon_tags_11 },
    { 0xbd1a92e2, SUC_lexicon_tags_1 },
    { 0xc73a92e4, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xc7fa92e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa59a92e8, SUC_lexicon_tags_10 },
    { 0x77fa92e9, SUC_lexicon_tags_38 },
    { 0x8a3a92e9, SUC_lexicon_tags_15 },
    { 0xe45a92eb, SUC_lexicon_tags_17 },
    { 0x289a92e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x717a92ef, SUC_lexicon_tags_47 },
    { 0xfa3a92f0, SUC_lexicon_tags_124 },
    { 0x7fa92f1, SUC_lexicon_tags_25 },
    { 0x7dba92ef, SUC_lexicon_tags_37 },
    { 0x823a92f3, SUC_lexicon_tags_62 },
    { 0x5cba92f4, SUC_lexicon_tags_48 },
    { 0x27a92f5, SUC_lexicon_tags_27 },
    { 0x709a92f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35a92fa, SUC_lexicon_tags_7 },
    { 0x927a92fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe19a92fd, SUC_lexicon_tags_7 },
    { 0xf05a92fd, SUC_lexicon_tags_1 },
    { 0x91ba92fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x285a9301, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337a9305, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x687a9308, SUC_lexicon_tags_16 },
    { 0x17fa9309, SUC_lexicon_tags_48 },
    { 0x9cfa930a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fa930f, SUC_lexicon_tags_33 },
    { 0xc93a9310, SUC_lexicon_tags_32 },
    { 0xd65a9310, SUC_lexicon_tags_6 },
    { 0xdbfa9311, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfa9315, SUC_lexicon_tags_15 },
    { 0x709a9316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fa931a, SUC_lexicon_tags_10 },
    { 0x815a931b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553a9329, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x621a932b, SUC_lexicon_tags_1 },
    { 0xb4fa932c, SUC_lexicon_tags_7 },
    { 0x685a932d, SUC_lexicon_tags_15 },
    { 0x3d3a932e, SUC_lexicon_tags_85 },
    { 0xf0ba932b, SUC_lexicon_tags_15 },
    { 0xc57a9330, SUC_lexicon_tags_87 },
    { 0x985a932e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809a9335, SUC_lexicon_tags_84 },
    { 0xd3fa9336, SUC_lexicon_tags_7 },
    { 0xb35a9337, SUC_lexicon_tags_7 },
    { 0xba7a9338, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efa933b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x737a933d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ba9343, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd67a9345, SUC_lexicon_tags_47 },
    { 0x71a9346, SUC_lexicon_tags_25 },
    { 0x33da9347, SUC_lexicon_tags_25 },
    { 0xc5fa9347, SUC_lexicon_tags_8 },
    { 0x87a9348, SUC_lexicon_tags_12 },
    { 0xb67a934a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63a934d, SUC_lexicon_tags_12 },
    { 0x5c9a934d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93a9352, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719a9356, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757a935b, SUC_lexicon_tags_15 },
    { 0x61fa935c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f1a935e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x81da9360, SUC_lexicon_tags_12 },
    { 0xfb1a9361, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03a9366, SUC_lexicon_tags_64 },
    { 0x4b9a9367, SUC_lexicon_tags_87 },
    { 0x10fa9367, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fa936e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85a9371, SUC_lexicon_tags_1 },
    { 0x62ba9372, SUC_lexicon_tags_62 },
    { 0xceba9372, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1a937a, SUC_lexicon_tags_49 },
    { 0x199a937b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe63a937d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab5a937f, SUC_lexicon_tags_25 },
    { 0x895a9380, SUC_lexicon_tags_56 },
    { 0x5f9a9381, SUC_lexicon_tags_2 },
    { 0x56da9381, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eba9385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43a9388, SUC_lexicon_tags_27 },
    { 0x29ba9388, SUC_lexicon_tags_38 },
    { 0x255a938a, SUC_lexicon_tags_538 },
    { 0, NULL },
    { 0xc09a938c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9a9390, SUC_lexicon_tags_15 },
    { 0x1cba9390, SUC_lexicon_tags_45 },
    { 0x20fa9392, SUC_lexicon_tags_31 },
    { 0xac9a9393, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7a9398, SUC_lexicon_tags_25 },
    { 0x693a9399, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafa93a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1a93a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dfa93aa, SUC_lexicon_tags_17 },
    { 0xd6fa93aa, SUC_lexicon_tags_87 },
    { 0x3b5a93ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0ba93ae, SUC_lexicon_tags_34 },
    { 0xbfda93af, SUC_lexicon_tags_45 },
    { 0xe71a93af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21a93b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ba93bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359a93bf, SUC_lexicon_tags_17 },
    { 0x365a93c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717a93cd, SUC_lexicon_tags_0 },
    { 0x4fba93ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3a93d4, SUC_lexicon_tags_60 },
    { 0xe23a93d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897a93da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x477a93dc, SUC_lexicon_tags_0 },
    { 0x569a93dd, SUC_lexicon_tags_25 },
    { 0x445a93dd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587a93e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5a93e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab5a93e7, SUC_lexicon_tags_37 },
    { 0x8b3a93e8, SUC_lexicon_tags_31 },
    { 0x19a93e7, SUC_lexicon_tags_60 },
    { 0xb01a93ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57a93ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47da93f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53a93f3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93da93f8, SUC_lexicon_tags_25 },
    { 0xd87a93f9, SUC_lexicon_tags_25 },
    { 0xbffa93fa, SUC_lexicon_tags_0 },
    { 0x3e7a93fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafa93fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521a9405, SUC_lexicon_tags_127 },
    { 0xe91a9406, SUC_lexicon_tags_27 },
    { 0x1b5a9406, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x969a9409, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287a940e, SUC_lexicon_tags_46 },
    { 0x86ba940f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x913a9412, SUC_lexicon_tags_56 },
    { 0x72ba9413, SUC_lexicon_tags_37 },
    { 0x691a9412, SUC_lexicon_tags_17 },
    { 0x2b7a9415, SUC_lexicon_tags_15 },
    { 0x127a9416, SUC_lexicon_tags_31 },
    { 0x573a9414, SUC_lexicon_tags_29 },
    { 0xc95a9418, SUC_lexicon_tags_32 },
    { 0xe51a9419, SUC_lexicon_tags_28 },
    { 0x43ba9419, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc53a941c, SUC_lexicon_tags_1 },
    { 0xf1da941d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9dba941f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513a9424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6da9428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9a942b, SUC_lexicon_tags_31 },
    { 0x825a942c, SUC_lexicon_tags_45 },
    { 0x28da942d, SUC_lexicon_tags_2 },
    { 0xdcfa942e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x401a9430, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ba9438, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x313a943b, SUC_lexicon_tags_1 },
    { 0x9c9a943c, SUC_lexicon_tags_2 },
    { 0x61a943d, SUC_lexicon_tags_7 },
    { 0x7a3a943e, SUC_lexicon_tags_17 },
    { 0x62ba943f, SUC_lexicon_tags_47 },
    { 0xc87a943c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fa9444, SUC_lexicon_tags_4 },
    { 0xf97a9444, SUC_lexicon_tags_15 },
    { 0x635a9444, SUC_lexicon_tags_17 },
    { 0xb67a9447, SUC_lexicon_tags_25 },
    { 0x5b5a9448, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x389a944b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8a7a944d, SUC_lexicon_tags_25 },
    { 0xb75a944e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9a9453, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699a9457, SUC_lexicon_tags_11 },
    { 0xe53a9458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5a945b, SUC_lexicon_tags_25 },
    { 0x8a9a945c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9a9463, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7e3a9465, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7a946a, SUC_lexicon_tags_9 },
    { 0x607a946b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5a946f, SUC_lexicon_tags_37 },
    { 0x873a946f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7efa9472, SUC_lexicon_tags_17 },
    { 0x6ada9472, SUC_lexicon_tags_14 },
    { 0xa3ba9472, SUC_lexicon_tags_37 },
    { 0x82da9475, SUC_lexicon_tags_518 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefba947e, SUC_lexicon_tags_33 },
    { 0x3c1a947e, SUC_lexicon_tags_9 },
    { 0x70ba9480, SUC_lexicon_tags_55 },
    { 0x3c3a9480, SUC_lexicon_tags_15 },
    { 0x155a9482, SUC_lexicon_tags_12 },
    { 0xd99a9483, SUC_lexicon_tags_26 },
    { 0xc9da9483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9a9488, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ba948b, SUC_lexicon_tags_22 },
    { 0xb37a948b, SUC_lexicon_tags_17 },
    { 0x60da948d, SUC_lexicon_tags_95 },
    { 0x1d9a948e, SUC_lexicon_tags_26 },
    { 0x4e7a948c, SUC_lexicon_tags_17 },
    { 0x95a948b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393a9495, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfdfa9497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59a949d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9a94a1, SUC_lexicon_tags_63 },
    { 0x86da94a1, SUC_lexicon_tags_12 },
    { 0x1ada94a3, SUC_lexicon_tags_37 },
    { 0x513a94a2, SUC_lexicon_tags_2 },
    { 0x801a94a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82da94aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd87a94ac, SUC_lexicon_tags_1 },
    { 0x83a94ad, SUC_lexicon_tags_13 },
    { 0xe85a94ac, SUC_lexicon_tags_63 },
    { 0xcbda94ac, SUC_lexicon_tags_63 },
    { 0x591a94b0, SUC_lexicon_tags_11 },
    { 0xd11a94ae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xff3a94b3, SUC_lexicon_tags_21 },
    { 0xcd3a94b4, SUC_lexicon_tags_104 },
    { 0x369a94b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7a94b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x76da94ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x73a94bd, SUC_lexicon_tags_14 },
    { 0xb2ba94be, SUC_lexicon_tags_36 },
    { 0x28ba94be, SUC_lexicon_tags_87 },
    { 0xb67a94c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x897a94c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239a94c7, SUC_lexicon_tags_11 },
    { 0x209a94c8, SUC_lexicon_tags_37 },
    { 0x6dda94c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cba94cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82fa94ce, SUC_lexicon_tags_12 },
    { 0x453a94cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13da94d6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107a94da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541a94e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55a94eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9e1a94ed, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4da94f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ba94f4, SUC_lexicon_tags_7 },
    { 0xf07a94f4, SUC_lexicon_tags_1 },
    { 0xd05a94f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe5a94f8, SUC_lexicon_tags_30 },
    { 0x235a94f9, SUC_lexicon_tags_19 },
    { 0xfda94f8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fa94fd, SUC_lexicon_tags_37 },
    { 0x315a94fe, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0x5e7a9500, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa5a9502, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffa9505, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6bfa9507, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9a950a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dda950e, SUC_lexicon_tags_17 },
    { 0x1d3a950f, SUC_lexicon_tags_83 },
    { 0xda7a9510, SUC_lexicon_tags_3 },
    { 0xbd7a9510, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x791a9513, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f7a9515, SUC_lexicon_tags_17 },
    { 0xa89a9516, SUC_lexicon_tags_33 },
    { 0xaafa9516, SUC_lexicon_tags_59 },
    { 0x813a9518, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0fa951a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95a951f, SUC_lexicon_tags_15 },
    { 0x605a9520, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8efa9522, SUC_lexicon_tags_2 },
    { 0xe3ba9522, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3a9533, SUC_lexicon_tags_3 },
    { 0x77a9534, SUC_lexicon_tags_23 },
    { 0x867a9533, SUC_lexicon_tags_1 },
    { 0x213a9536, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ba9539, SUC_lexicon_tags_26 },
    { 0xdd1a953a, SUC_lexicon_tags_215 },
    { 0xd0da953a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ba9540, SUC_lexicon_tags_19 },
    { 0xb81a9541, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfa9544, SUC_lexicon_tags_61 },
    { 0xc45a9545, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fa9548, SUC_lexicon_tags_25 },
    { 0xe15a9549, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xaa5a954b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379a954f, SUC_lexicon_tags_37 },
    { 0x889a9550, SUC_lexicon_tags_82 },
    { 0xcc9a9550, SUC_lexicon_tags_350 },
    { 0x213a9552, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa0fa9554, SUC_lexicon_tags_21 },
    { 0x7e9a9555, SUC_lexicon_tags_1 },
    { 0x23da9556, SUC_lexicon_tags_45 },
    { 0x3a1a9556, SUC_lexicon_tags_2 },
    { 0xb0da9558, SUC_lexicon_tags_25 },
    { 0x537a9557, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49a955c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda9562, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699a9568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19a956b, SUC_lexicon_tags_23 },
    { 0x427a956c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fa956f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb7fa9571, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1a957a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x223a957d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffa9580, SUC_lexicon_tags_25 },
    { 0x655a9580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb27a9583, SUC_lexicon_tags_14 },
    { 0xd23a9584, SUC_lexicon_tags_7 },
    { 0x41da9585, SUC_lexicon_tags_25 },
    { 0x27fa9585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfeba9588, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91a958d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ba9599, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xee9a959b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x581a959e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439a95a5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fa95a8, SUC_lexicon_tags_25 },
    { 0x491a95a9, SUC_lexicon_tags_1209 },
    { 0x729a95a8, SUC_lexicon_tags_37 },
    { 0x9da95ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05a95ae, SUC_lexicon_tags_30 },
    { 0x6dda95af, SUC_lexicon_tags_26 },
    { 0xbf9a95b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123a95b4, SUC_lexicon_tags_2 },
    { 0xc19a95b5, SUC_lexicon_tags_23 },
    { 0x393a95b6, SUC_lexicon_tags_224 },
    { 0x92ba95b7, SUC_lexicon_tags_37 },
    { 0x4b9a95b7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedda95c3, SUC_lexicon_tags_11 },
    { 0x2d7a95c4, SUC_lexicon_tags_25 },
    { 0x16ba95c5, SUC_lexicon_tags_65 },
    { 0x8aba95c3, SUC_lexicon_tags_6 },
    { 0x451a95c5, SUC_lexicon_tags_47 },
    { 0xe07a95c8, SUC_lexicon_tags_39 },
    { 0xe63a95c6, SUC_lexicon_tags_14 },
    { 0x8cba95c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3a95d0, SUC_lexicon_tags_17 },
    { 0xb8fa95d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3a95d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9a95da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x10fa95dc, SUC_lexicon_tags_1 },
    { 0xc43a95dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x79fa95df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b9a95e1, SUC_lexicon_tags_23 },
    { 0xcf9a95e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdaba95e4, SUC_lexicon_tags_4 },
    { 0x479a95e5, SUC_lexicon_tags_37 },
    { 0x377a95e6, SUC_lexicon_tags_7 },
    { 0xb41a95e7, SUC_lexicon_tags_1 },
    { 0x443a95e8, SUC_lexicon_tags_38 },
    { 0xd8ba95e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97a95f1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x349a95f3, SUC_lexicon_tags_6 },
    { 0xaf7a95f4, SUC_lexicon_tags_10 },
    { 0xa1fa95f5, SUC_lexicon_tags_33 },
    { 0x76fa95f6, SUC_lexicon_tags_1 },
    { 0x991a95f6, SUC_lexicon_tags_0 },
    { 0x1ada95f7, SUC_lexicon_tags_25 },
    { 0xbb9a95f9, SUC_lexicon_tags_12 },
    { 0x5c3a95f9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe3ba95fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee7a95fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fa9601, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3a9607, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3da960d, SUC_lexicon_tags_1634 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5a961a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xcada961d, SUC_lexicon_tags_140 },
    { 0x61fa961e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb69a9620, SUC_lexicon_tags_11 },
    { 0x67da9621, SUC_lexicon_tags_47 },
    { 0x681a9620, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fa9624, SUC_lexicon_tags_5 },
    { 0x3b1a9625, SUC_lexicon_tags_7 },
    { 0xada9626, SUC_lexicon_tags_33 },
    { 0xe3ba9627, SUC_lexicon_tags_34 },
    { 0xb47a9628, SUC_lexicon_tags_31 },
    { 0x401a9626, SUC_lexicon_tags_2 },
    { 0x87fa9624, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495a962e, SUC_lexicon_tags_65 },
    { 0xa4fa962e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fa9633, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff3a9635, SUC_lexicon_tags_9 },
    { 0x4bda9635, SUC_lexicon_tags_38 },
    { 0x7d3a9637, SUC_lexicon_tags_31 },
    { 0xc7fa9637, SUC_lexicon_tags_11 },
    { 0x4c7a9639, SUC_lexicon_tags_33 },
    { 0x5c9a9638, SUC_lexicon_tags_49 },
    { 0xfb7a963b, SUC_lexicon_tags_27 },
    { 0x375a963a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbda963f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ba9642, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefa9647, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x219a9649, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aba9653, SUC_lexicon_tags_31 },
    { 0x1f7a9654, SUC_lexicon_tags_2 },
    { 0xe0ba9655, SUC_lexicon_tags_43 },
    { 0xf9ba9655, SUC_lexicon_tags_29 },
    { 0xb4ba9654, SUC_lexicon_tags_38 },
    { 0x39a9658, SUC_lexicon_tags_2 },
    { 0x1f9a9658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1a965c, SUC_lexicon_tags_27 },
    { 0x36da965d, SUC_lexicon_tags_60 },
    { 0xf93a965c, SUC_lexicon_tags_25 },
    { 0x9ada965d, SUC_lexicon_tags_80 },
    { 0xcd1a9660, SUC_lexicon_tags_33 },
    { 0x9b3a9661, SUC_lexicon_tags_14 },
    { 0x29ba9660, SUC_lexicon_tags_12 },
    { 0x153a9663, SUC_lexicon_tags_63 },
    { 0xe87a9664, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1a9668, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x705a966a, SUC_lexicon_tags_7 },
    { 0x92fa966a, SUC_lexicon_tags_692 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3da966e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7a9675, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83a9679, SUC_lexicon_tags_1 },
    { 0x6afa967a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61a967e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fa9685, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187a9691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f5a9693, SUC_lexicon_tags_33 },
    { 0x10da9694, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcf1a9696, SUC_lexicon_tags_15 },
    { 0x4b5a9697, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8da969b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867a96a7, SUC_lexicon_tags_15 },
    { 0xcc7a96a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42da96aa, SUC_lexicon_tags_250 },
    { 0xe59a96ab, SUC_lexicon_tags_6 },
    { 0x7cba96ac, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2da96af, SUC_lexicon_tags_43 },
    { 0x24da96af, SUC_lexicon_tags_66 },
    { 0x1bba96af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeefa96b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39a96ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x409a96bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x833a96bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7fa96c1, SUC_lexicon_tags_11 },
    { 0x405a96c2, SUC_lexicon_tags_37 },
    { 0xb3ba96c1, SUC_lexicon_tags_710 },
    { 0xbaba96c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05a96cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x11da96cf, SUC_lexicon_tags_12 },
    { 0xd1a96d0, SUC_lexicon_tags_29 },
    { 0x207a96d0, SUC_lexicon_tags_17 },
    { 0x9e7a96d2, SUC_lexicon_tags_29 },
    { 0x57ba96d3, SUC_lexicon_tags_12 },
    { 0x877a96d1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9a96d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54da96d9, SUC_lexicon_tags_37 },
    { 0x4e3a96d9, SUC_lexicon_tags_30 },
    { 0xde7a96d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x555a96dd, SUC_lexicon_tags_31 },
    { 0xa67a96dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5aba96e0, SUC_lexicon_tags_14 },
    { 0x89da96e1, SUC_lexicon_tags_10 },
    { 0x8e3a96e1, SUC_lexicon_tags_43 },
    { 0x8fba96e3, SUC_lexicon_tags_194 },
    { 0x117a96e4, SUC_lexicon_tags_12 },
    { 0xdc7a96e4, SUC_lexicon_tags_2 },
    { 0x42da96e6, SUC_lexicon_tags_87 },
    { 0xb17a96e7, SUC_lexicon_tags_25 },
    { 0x6b5a96e8, SUC_lexicon_tags_7 },
    { 0x64da96e9, SUC_lexicon_tags_38 },
    { 0x863a96ea, SUC_lexicon_tags_7 },
    { 0x70da96e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7a96ee, SUC_lexicon_tags_30 },
    { 0x617a96ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385a96f5, SUC_lexicon_tags_14 },
    { 0x971a96f6, SUC_lexicon_tags_23 },
    { 0x85a96f7, SUC_lexicon_tags_2 },
    { 0x6b9a96f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xffa96fa, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1da96fe, SUC_lexicon_tags_49 },
    { 0xfd3a96fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a9701, SUC_lexicon_tags_23 },
    { 0x56ba9702, SUC_lexicon_tags_7 },
    { 0x831a9703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ba9706, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bda9708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fa970d, SUC_lexicon_tags_25 },
    { 0xc8da970e, SUC_lexicon_tags_64 },
    { 0x5d9a970e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x711a9711, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x39a9713, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2da9718, SUC_lexicon_tags_2 },
    { 0xd47a9719, SUC_lexicon_tags_63 },
    { 0xd53a971a, SUC_lexicon_tags_10 },
    { 0xed5a971b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27a9723, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fa9726, SUC_lexicon_tags_11 },
    { 0x7c3a9727, SUC_lexicon_tags_25 },
    { 0x853a9727, SUC_lexicon_tags_45 },
    { 0xd3ba9727, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3da972c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3eba972e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fa9730, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971a9737, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ba973e, SUC_lexicon_tags_33 },
    { 0x76da973e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbda9743, SUC_lexicon_tags_2 },
    { 0x5f5a9744, SUC_lexicon_tags_0 },
    { 0xed5a9745, SUC_lexicon_tags_17 },
    { 0xe45a9746, SUC_lexicon_tags_17 },
    { 0x81a9743, SUC_lexicon_tags_38 },
    { 0xccba9748, SUC_lexicon_tags_29 },
    { 0xaf3a9748, SUC_lexicon_tags_60 },
    { 0xcbba974a, SUC_lexicon_tags_43 },
    { 0xb1da974a, SUC_lexicon_tags_19 },
    { 0x751a9749, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69a9755, SUC_lexicon_tags_63 },
    { 0x865a9756, SUC_lexicon_tags_8 },
    { 0x287a9756, SUC_lexicon_tags_25 },
    { 0xcc7a9756, SUC_lexicon_tags_38 },
    { 0x769a9756, SUC_lexicon_tags_5 },
    { 0x451a975a, SUC_lexicon_tags_40 },
    { 0x37da975a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e1a975d, SUC_lexicon_tags_37 },
    { 0x23a975e, SUC_lexicon_tags_43 },
    { 0xa13a975f, SUC_lexicon_tags_65 },
    { 0x9e5a9760, SUC_lexicon_tags_4 },
    { 0xd41a975f, SUC_lexicon_tags_34 },
    { 0x125a9761, SUC_lexicon_tags_37 },
    { 0x795a975f, SUC_lexicon_tags_25 },
    { 0xacfa9761, SUC_lexicon_tags_160 },
    { 0x74fa975d, SUC_lexicon_tags_2 },
    { 0xe05a9766, SUC_lexicon_tags_23 },
    { 0x83da9767, SUC_lexicon_tags_11 },
    { 0x10da9768, SUC_lexicon_tags_49 },
    { 0x423a9769, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425a976d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf31a9770, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4da977a, SUC_lexicon_tags_0 },
    { 0xf13a977b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5a977e, SUC_lexicon_tags_7 },
    { 0xea5a977f, SUC_lexicon_tags_47 },
    { 0x92fa9780, SUC_lexicon_tags_11 },
    { 0xecda9780, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da9784, SUC_lexicon_tags_1 },
    { 0xb09a9784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaba9789, SUC_lexicon_tags_1 },
    { 0x147a978a, SUC_lexicon_tags_60 },
    { 0xa3da9789, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x505a978d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x299a978f, SUC_lexicon_tags_141 },
    { 0x331a9790, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6da9792, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdcba9794, SUC_lexicon_tags_15 },
    { 0xdf9a9794, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3a9798, SUC_lexicon_tags_4 },
    { 0x27a9798, SUC_lexicon_tags_11 },
    { 0x10ba9798, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdda979d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5a97a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59da97a6, SUC_lexicon_tags_2 },
    { 0x513a97a6, SUC_lexicon_tags_21 },
    { 0xcfda97a8, SUC_lexicon_tags_17 },
    { 0x4eba97a8, SUC_lexicon_tags_75 },
    { 0x2fba97aa, SUC_lexicon_tags_11 },
    { 0x93a97ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afa97b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e9a97b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9a3a97b7, SUC_lexicon_tags_1 },
    { 0x123a97b7, SUC_lexicon_tags_45 },
    { 0x92da97b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8ba97bb, SUC_lexicon_tags_88 },
    { 0x411a97bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ba97c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7a97c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469a97cd, SUC_lexicon_tags_1 },
    { 0xa9fa97ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657a97d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x235a97d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b7a97d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24da97e0, SUC_lexicon_tags_23 },
    { 0xec3a97e1, SUC_lexicon_tags_28 },
    { 0xe1da97e1, SUC_lexicon_tags_7 },
    { 0xc31a97e1, SUC_lexicon_tags_9 },
    { 0x551a97e4, SUC_lexicon_tags_13 },
    { 0x545a97e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7a97e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ba97ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ba97f4, SUC_lexicon_tags_1 },
    { 0x14fa97f5, SUC_lexicon_tags_21 },
    { 0xe01a97f6, SUC_lexicon_tags_45 },
    { 0x8d5a97f7, SUC_lexicon_tags_6 },
    { 0x353a97f8, SUC_lexicon_tags_15 },
    { 0x54da97f7, SUC_lexicon_tags_56 },
    { 0xb3ba97fa, SUC_lexicon_tags_1 },
    { 0xcfba97f8, SUC_lexicon_tags_47 },
    { 0xbbfa97f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x481a97fe, SUC_lexicon_tags_63 },
    { 0xeada97fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4da9801, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fa9805, SUC_lexicon_tags_37 },
    { 0x9fda9805, SUC_lexicon_tags_26 },
    { 0x219a9807, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3a980d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfa9810, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35a9814, SUC_lexicon_tags_30 },
    { 0xf99a9815, SUC_lexicon_tags_2 },
    { 0xcfda9816, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79a9819, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ba981f, SUC_lexicon_tags_21 },
    { 0x23ba9820, SUC_lexicon_tags_77 },
    { 0x1e1a9821, SUC_lexicon_tags_26 },
    { 0x889a9820, SUC_lexicon_tags_6 },
    { 0x97fa9823, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3a9828, SUC_lexicon_tags_7 },
    { 0xd1a9829, SUC_lexicon_tags_7 },
    { 0x8ffa982a, SUC_lexicon_tags_16 },
    { 0x6c3a982a, SUC_lexicon_tags_48 },
    { 0x3d9a982b, SUC_lexicon_tags_25 },
    { 0x925a982c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfa9830, SUC_lexicon_tags_19 },
    { 0xc65a9831, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeda9836, SUC_lexicon_tags_49 },
    { 0xf39a9837, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b1a9839, SUC_lexicon_tags_6 },
    { 0xb97a983a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671a9843, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6da9849, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ba984d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba9852, SUC_lexicon_tags_60 },
    { 0x603a9852, SUC_lexicon_tags_37 },
    { 0xf03a9854, SUC_lexicon_tags_68 },
    { 0xc8da9854, SUC_lexicon_tags_1 },
    { 0xd79a9854, SUC_lexicon_tags_37 },
    { 0xde1a9857, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xedba9859, SUC_lexicon_tags_1 },
    { 0x35da9859, SUC_lexicon_tags_21 },
    { 0xc5fa985b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83da985f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eda9866, SUC_lexicon_tags_2 },
    { 0xc39a9866, SUC_lexicon_tags_7 },
    { 0x3a7a9868, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfa9874, SUC_lexicon_tags_60 },
    { 0x6e1a9874, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b5a9877, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fa9882, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217a9888, SUC_lexicon_tags_33 },
    { 0x1e1a9888, SUC_lexicon_tags_9 },
    { 0x2c7a988a, SUC_lexicon_tags_63 },
    { 0x20ba988b, SUC_lexicon_tags_37 },
    { 0x1bfa988c, SUC_lexicon_tags_12 },
    { 0xe5ba988c, SUC_lexicon_tags_17 },
    { 0x2da988e, SUC_lexicon_tags_17 },
    { 0x4ffa988f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x541a9891, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c1a9893, SUC_lexicon_tags_9 },
    { 0xecfa9894, SUC_lexicon_tags_38 },
    { 0x491a9895, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa21a9897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x497a989a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49a98a2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3ba98a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7a98a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5ba98a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2ba98ab, SUC_lexicon_tags_8 },
    { 0xd53a98ac, SUC_lexicon_tags_1 },
    { 0xfb3a98ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8eba98b0, SUC_lexicon_tags_27 },
    { 0x95a98b1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813a98b6, SUC_lexicon_tags_27 },
    { 0xd9da98b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09a98bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ba98bf, SUC_lexicon_tags_8 },
    { 0x7c1a98c0, SUC_lexicon_tags_11 },
    { 0xf33a98c0, SUC_lexicon_tags_47 },
    { 0xa8fa98c1, SUC_lexicon_tags_26 },
    { 0xd31a98c3, SUC_lexicon_tags_1 },
    { 0x3b9a98c3, SUC_lexicon_tags_28 },
    { 0x323a98c3, SUC_lexicon_tags_27 },
    { 0xb61a98c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99a98ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9a98ce, SUC_lexicon_tags_15 },
    { 0xd77a98cf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85a98d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x885a98d4, SUC_lexicon_tags_8 },
    { 0xd6da98d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55a98da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1fa98dc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd75a98de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x357a98e0, SUC_lexicon_tags_25 },
    { 0x6a9a98e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x177a98e3, SUC_lexicon_tags_27 },
    { 0x18da98e4, SUC_lexicon_tags_47 },
    { 0xddba98e5, SUC_lexicon_tags_2 },
    { 0xd6da98e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483a98ea, SUC_lexicon_tags_37 },
    { 0xe5fa98ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23da98ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ada98f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749a98f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f5a98f8, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xebda98fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3a98ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8f1a9901, SUC_lexicon_tags_1 },
    { 0x32fa9902, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ba9906, SUC_lexicon_tags_38 },
    { 0x485a9906, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x947a990a, SUC_lexicon_tags_17 },
    { 0x763a990b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x93da990d, SUC_lexicon_tags_14 },
    { 0xc0ba990d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141a9913, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x71a9916, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x527a9919, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x855a991c, SUC_lexicon_tags_11 },
    { 0x6bba991d, SUC_lexicon_tags_7 },
    { 0x77da991e, SUC_lexicon_tags_60 },
    { 0x3b1a991f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x597a9921, SUC_lexicon_tags_15 },
    { 0xe97a9922, SUC_lexicon_tags_11 },
    { 0xf4ba9923, SUC_lexicon_tags_15 },
    { 0x971a9922, SUC_lexicon_tags_79 },
    { 0x28fa9925, SUC_lexicon_tags_1 },
    { 0x547a9926, SUC_lexicon_tags_17 },
    { 0x489a9922, SUC_lexicon_tags_37 },
    { 0x77da9928, SUC_lexicon_tags_1 },
    { 0xdd9a9929, SUC_lexicon_tags_63 },
    { 0x8e5a9929, SUC_lexicon_tags_236 },
    { 0xdc5a992b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5a992f, SUC_lexicon_tags_8 },
    { 0x6b3a9930, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1a9933, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935a993a, SUC_lexicon_tags_17 },
    { 0xa39a993b, SUC_lexicon_tags_9 },
    { 0xe8fa993a, SUC_lexicon_tags_12 },
    { 0x5a3a993d, SUC_lexicon_tags_2 },
    { 0x4ada993e, SUC_lexicon_tags_48 },
    { 0xd0fa993f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x365a9942, SUC_lexicon_tags_17 },
    { 0x8a5a9943, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccda9947, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1a994b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff5a994d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665a9957, SUC_lexicon_tags_27 },
    { 0x1b7a9958, SUC_lexicon_tags_12 },
    { 0xeeda9959, SUC_lexicon_tags_25 },
    { 0x7b9a9959, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfafa995c, SUC_lexicon_tags_38 },
    { 0x22ba995d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x221a9960, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60da9962, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18fa9964, SUC_lexicon_tags_17 },
    { 0x4efa9965, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9a996c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x681a996f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1a1a9971, SUC_lexicon_tags_47 },
    { 0xeefa9971, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd39a9974, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29a9977, SUC_lexicon_tags_45 },
    { 0xc1da9978, SUC_lexicon_tags_37 },
    { 0xd19a9979, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x767a997b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63a9983, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7a9986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411a998b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefa998e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7a9992, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561a9996, SUC_lexicon_tags_84 },
    { 0x629a9997, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9a999a, SUC_lexicon_tags_60 },
    { 0xab9a999a, SUC_lexicon_tags_30 },
    { 0xfb1a999c, SUC_lexicon_tags_11 },
    { 0xd77a999d, SUC_lexicon_tags_17 },
    { 0x91ba999e, SUC_lexicon_tags_37 },
    { 0x6e7a999f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfa99a3, SUC_lexicon_tags_11 },
    { 0xc01a99a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557a99a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ba99ae, SUC_lexicon_tags_25 },
    { 0x155a99af, SUC_lexicon_tags_23 },
    { 0xda5a99af, SUC_lexicon_tags_11 },
    { 0x65fa99b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeba99b4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x777a99b7, SUC_lexicon_tags_17 },
    { 0xb29a99b8, SUC_lexicon_tags_55 },
    { 0x561a99b9, SUC_lexicon_tags_12 },
    { 0xa97a99b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeeba99bc, SUC_lexicon_tags_47 },
    { 0xb6fa99bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdada99c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd63a99c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x231a99c6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1a99c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33a99cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc35a99cf, SUC_lexicon_tags_38 },
    { 0x759a99d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87a99d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a99d8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17a99db, SUC_lexicon_tags_11 },
    { 0x381a99db, SUC_lexicon_tags_31 },
    { 0x83fa99dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23a99e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xae1a99e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc3a99e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65a99ec, SUC_lexicon_tags_37 },
    { 0x601a99ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8ada99ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77a99f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43a99f6, SUC_lexicon_tags_15 },
    { 0xd93a99f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eda99fb, SUC_lexicon_tags_47 },
    { 0xd23a99fc, SUC_lexicon_tags_11 },
    { 0xb41a99fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14fa99ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5a9a04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495a9a0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc5ba9a0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47a9a12, SUC_lexicon_tags_2 },
    { 0x32ba9a13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86ba9a15, SUC_lexicon_tags_49 },
    { 0xf3ba9a16, SUC_lexicon_tags_39 },
    { 0x2b7a9a17, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41a9a1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac9a9a1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcada9a1f, SUC_lexicon_tags_38 },
    { 0x423a9a20, SUC_lexicon_tags_6 },
    { 0x96ba9a20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfa9a24, SUC_lexicon_tags_25 },
    { 0xa95a9a25, SUC_lexicon_tags_11 },
    { 0xe13a9a24, SUC_lexicon_tags_32 },
    { 0x689a9a25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1a9a2a, SUC_lexicon_tags_15 },
    { 0x497a9a2b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x64fa9a2d, SUC_lexicon_tags_5 },
    { 0x315a9a2e, SUC_lexicon_tags_82 },
    { 0x13da9a2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x737a9a32, SUC_lexicon_tags_38 },
    { 0x135a9a33, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3a9a36, SUC_lexicon_tags_66 },
    { 0xf13a9a37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x157a9a39, SUC_lexicon_tags_28 },
    { 0x25fa9a39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9a9a41, SUC_lexicon_tags_15 },
    { 0x711a9a41, SUC_lexicon_tags_30 },
    { 0xd33a9a43, SUC_lexicon_tags_19 },
    { 0x64ba9a42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x217a9a46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fa9a49, SUC_lexicon_tags_17 },
    { 0x6b1a9a49, SUC_lexicon_tags_46 },
    { 0xf5a9a4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x637a9a4e, SUC_lexicon_tags_43 },
    { 0x249a9a4e, SUC_lexicon_tags_37 },
    { 0x4bfa9a50, SUC_lexicon_tags_49 },
    { 0x807a9a50, SUC_lexicon_tags_31 },
    { 0xbb1a9a52, SUC_lexicon_tags_37 },
    { 0x207a9a53, SUC_lexicon_tags_92 },
    { 0xc33a9a54, SUC_lexicon_tags_45 },
    { 0x231a9a54, SUC_lexicon_tags_11 },
    { 0xa3ba9a56, SUC_lexicon_tags_172 },
    { 0x631a9a56, SUC_lexicon_tags_0 },
    { 0x57a9a56, SUC_lexicon_tags_28 },
    { 0xfe9a9a56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7a9a5d, SUC_lexicon_tags_33 },
    { 0x1ffa9a5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x35da9a60, SUC_lexicon_tags_47 },
    { 0x1b1a9a60, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x99a9a64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5a9a6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7a9a6d, SUC_lexicon_tags_61 },
    { 0xd4ba9a6e, SUC_lexicon_tags_31 },
    { 0xbe9a9a6f, SUC_lexicon_tags_1 },
    { 0x915a9a6d, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35a9a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467a9a7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe93a9a7d, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9da9a87, SUC_lexicon_tags_43 },
    { 0x729a9a88, SUC_lexicon_tags_1 },
    { 0x819a9a89, SUC_lexicon_tags_42 },
    { 0x457a9a8a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fda9a8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1a9a92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4da9a94, SUC_lexicon_tags_6 },
    { 0x929a9a95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd75a9a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ada9a9a, SUC_lexicon_tags_17 },
    { 0x83a9a9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231a9aa0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a9aa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dba9aa4, SUC_lexicon_tags_4 },
    { 0x46da9aa5, SUC_lexicon_tags_12 },
    { 0x2bba9aa6, SUC_lexicon_tags_1 },
    { 0x155a9aa5, SUC_lexicon_tags_19 },
    { 0x6dfa9aa6, SUC_lexicon_tags_19 },
    { 0xdf3a9aa4, SUC_lexicon_tags_25 },
    { 0xfda9aa7, SUC_lexicon_tags_1229 },
    { 0xedba9aa9, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x8b3a9aad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4e3a9aaf, SUC_lexicon_tags_88 },
    { 0x989a9ab0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8da9ab3, SUC_lexicon_tags_37 },
    { 0x4a5a9ab4, SUC_lexicon_tags_7 },
    { 0xbf9a9ab4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad5a9ab7, SUC_lexicon_tags_37 },
    { 0x1dda9ab7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773a9abe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x927a9ac1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ba9acc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91a9ad1, SUC_lexicon_tags_63 },
    { 0x393a9ad1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfa9ad8, SUC_lexicon_tags_37 },
    { 0x2b9a9ad8, SUC_lexicon_tags_952 },
    { 0x2efa9ada, SUC_lexicon_tags_7 },
    { 0x8cda9adb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fa9ae1, SUC_lexicon_tags_14 },
    { 0x895a9ae1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287a9ae6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afa9aeb, SUC_lexicon_tags_23 },
    { 0x2eda9aeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x829a9aef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7a9af2, SUC_lexicon_tags_112 },
    { 0x8f1a9af3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x271a9af5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfa9af9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf55a9afb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57a9afe, SUC_lexicon_tags_15 },
    { 0xd49a9afe, SUC_lexicon_tags_12 },
    { 0x171a9b00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3ba9b02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x587a9b04, SUC_lexicon_tags_25 },
    { 0xd67a9b05, SUC_lexicon_tags_1 },
    { 0x879a9b04, SUC_lexicon_tags_1 },
    { 0xf91a9b06, SUC_lexicon_tags_23 },
    { 0x39a9b08, SUC_lexicon_tags_1 },
    { 0x51ba9b08, SUC_lexicon_tags_39 },
    { 0x4f9a9b05, SUC_lexicon_tags_34 },
    { 0x1a9a9b0b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc97a9b0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19a9b12, SUC_lexicon_tags_17 },
    { 0x71da9b13, SUC_lexicon_tags_11 },
    { 0x6e5a9b12, SUC_lexicon_tags_28 },
    { 0x73fa9b14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcda9b19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9a9b1d, SUC_lexicon_tags_17 },
    { 0x327a9b1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5a9b23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd3a9b25, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdba9b2b, SUC_lexicon_tags_17 },
    { 0x31ba9b2b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3a9b2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fa9b35, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9a9b39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99ba9b3b, SUC_lexicon_tags_17 },
    { 0xf95a9b3c, SUC_lexicon_tags_28 },
    { 0x715a9b3d, SUC_lexicon_tags_68 },
    { 0xca1a9b3b, SUC_lexicon_tags_39 },
    { 0xb33a9b3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x503a9b41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3bda9b43, SUC_lexicon_tags_12 },
    { 0x769a9b43, SUC_lexicon_tags_19 },
    { 0xa59a9b44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae5a9b47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x533a9b49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98da9b4f, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ba9b53, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fa9b57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ba9b5a, SUC_lexicon_tags_2 },
    { 0x42fa9b5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafa9b60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf25a9b62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcfa9b64, SUC_lexicon_tags_25 },
    { 0xb67a9b65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x611a9b68, SUC_lexicon_tags_12 },
    { 0x741a9b68, SUC_lexicon_tags_37 },
    { 0xbfba9b6a, SUC_lexicon_tags_11 },
    { 0x21fa9b6b, SUC_lexicon_tags_47 },
    { 0xd11a9b6c, SUC_lexicon_tags_43 },
    { 0x19ba9b6d, SUC_lexicon_tags_48 },
    { 0x629a9b6e, SUC_lexicon_tags_15 },
    { 0x399a9b6f, SUC_lexicon_tags_7 },
    { 0x7f5a9b70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135a9b74, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x1a3a9b76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591a9b7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75a9b7f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7a9b86, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3a9b8a, SUC_lexicon_tags_17 },
    { 0xb25a9b8b, SUC_lexicon_tags_26 },
    { 0xfa9b8b, SUC_lexicon_tags_37 },
    { 0xb01a9b8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77a9b90, SUC_lexicon_tags_2 },
    { 0xde9a9b91, SUC_lexicon_tags_108 },
    { 0xcb3a9b92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163a9b9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75a9ba1, SUC_lexicon_tags_7 },
    { 0xb7fa9ba2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265a9bac, SUC_lexicon_tags_31 },
    { 0xa47a9bad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9a9bb2, SUC_lexicon_tags_17 },
    { 0x667a9bb3, SUC_lexicon_tags_7 },
    { 0x9ffa9bb3, SUC_lexicon_tags_47 },
    { 0xafda9bb5, SUC_lexicon_tags_17 },
    { 0x8e7a9bb6, SUC_lexicon_tags_68 },
    { 0x9b3a9bb7, SUC_lexicon_tags_25 },
    { 0x8d7a9bb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7bfa9bba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x189a9bbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fa9bc2, SUC_lexicon_tags_14 },
    { 0x4d9a9bc3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x869a9bc6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3a9bcb, SUC_lexicon_tags_60 },
    { 0xf5fa9bcb, SUC_lexicon_tags_31 },
    { 0x5f7a9bcc, SUC_lexicon_tags_43 },
    { 0x39a9bce, SUC_lexicon_tags_15 },
    { 0x945a9bcf, SUC_lexicon_tags_11 },
    { 0xcc9a9bce, SUC_lexicon_tags_14 },
    { 0xb83a9bcd, SUC_lexicon_tags_32 },
    { 0x39da9bcd, SUC_lexicon_tags_2 },
    { 0xacda9bd3, SUC_lexicon_tags_43 },
    { 0xd63a9bd3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x923a9bd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3a9bd9, SUC_lexicon_tags_856 },
    { 0x6bfa9bd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x305a9bdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03a9be0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x297a9be2, SUC_lexicon_tags_2 },
    { 0x6b9a9be2, SUC_lexicon_tags_17 },
    { 0xaa5a9be4, SUC_lexicon_tags_29 },
    { 0xe67a9be3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe03a9be7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13a9bf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcafa9bf2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703a9bf6, SUC_lexicon_tags_23 },
    { 0x865a9bf7, SUC_lexicon_tags_17 },
    { 0xe13a9bf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83a9bff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9c03, SUC_lexicon_tags_77 },
    { 0x373a9c03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7a9c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdeba9c09, SUC_lexicon_tags_33 },
    { 0xae1a9c0a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xb4da9c0c, SUC_lexicon_tags_42 },
    { 0x267a9c0d, SUC_lexicon_tags_7 },
    { 0xbda9c0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x761a9c10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621a9c15, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5da9c1a, SUC_lexicon_tags_21 },
    { 0xd8da9c1b, SUC_lexicon_tags_25 },
    { 0x371a9c1a, SUC_lexicon_tags_7 },
    { 0x5eba9c1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7a9c20, SUC_lexicon_tags_71 },
    { 0xc0ba9c21, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67a9c25, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9a9c28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9a9c2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9a9c32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6f7a9c34, SUC_lexicon_tags_25 },
    { 0x549a9c34, SUC_lexicon_tags_0 },
    { 0x2bba9c35, SUC_lexicon_tags_25 },
    { 0xe6ba9c37, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x34da9c39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x205a9c3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29a9c40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187a9c44, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf5a9c46, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5a9c4b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd29a9c4d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bda9c52, SUC_lexicon_tags_43 },
    { 0x5fa9c52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a3a9c55, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x353a9c58, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3a9c5c, SUC_lexicon_tags_21 },
    { 0x999a9c5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefba9c68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fa9c6c, SUC_lexicon_tags_17 },
    { 0xe1da9c6c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x90da9c70, SUC_lexicon_tags_69 },
    { 0xbaba9c71, SUC_lexicon_tags_2 },
    { 0x8a1a9c72, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfa9c78, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fda9c7b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eda9c7e, SUC_lexicon_tags_6 },
    { 0xabba9c7e, SUC_lexicon_tags_39 },
    { 0x4ada9c80, SUC_lexicon_tags_17 },
    { 0xc97a9c81, SUC_lexicon_tags_10 },
    { 0xa65a9c81, SUC_lexicon_tags_30 },
    { 0xfba9c83, SUC_lexicon_tags_25 },
    { 0xc0ba9c84, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77a9c87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x619a9c8a, SUC_lexicon_tags_7 },
    { 0xf81a9c8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3a9c92, SUC_lexicon_tags_6 },
    { 0x197a9c93, SUC_lexicon_tags_21 },
    { 0xc49a9c94, SUC_lexicon_tags_27 },
    { 0x897a9c95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fa9c9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x431a9c9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43a9ca4, SUC_lexicon_tags_23 },
    { 0x61ba9ca5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcba9caa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505a9caf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327a9cb9, SUC_lexicon_tags_60 },
    { 0xaada9cb9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9cbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bda9cc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73a9cc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d9a9cc9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5a9ccd, SUC_lexicon_tags_37 },
    { 0x85a9cce, SUC_lexicon_tags_34 },
    { 0xb3da9ccf, SUC_lexicon_tags_60 },
    { 0x933a9cd0, SUC_lexicon_tags_118 },
    { 0xb2da9ccd, SUC_lexicon_tags_7 },
    { 0x5d7a9ccd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7a9cd7, SUC_lexicon_tags_17 },
    { 0xea3a9cd8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fa9ce0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fa9ce7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69a9ceb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd7a9ced, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fa9cf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfafa9cf6, SUC_lexicon_tags_11 },
    { 0x135a9cf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98ba9cfc, SUC_lexicon_tags_63 },
    { 0x2f1a9cfd, SUC_lexicon_tags_11 },
    { 0x199a9cfe, SUC_lexicon_tags_9 },
    { 0x15fa9cff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1a9d02, SUC_lexicon_tags_23 },
    { 0x1f7a9d02, SUC_lexicon_tags_87 },
    { 0x8dba9d04, SUC_lexicon_tags_17 },
    { 0x50ba9d05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc91a9d07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93a9d0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7a9d10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7a9d14, SUC_lexicon_tags_27 },
    { 0xc97a9d14, SUC_lexicon_tags_5 },
    { 0x237a9d14, SUC_lexicon_tags_37 },
    { 0x631a9d15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x727a9d1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1a9d20, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ba9d25, SUC_lexicon_tags_122 },
    { 0x68da9d25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89a9d29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec1a9d2b, SUC_lexicon_tags_65 },
    { 0xf51a9d2b, SUC_lexicon_tags_22 },
    { 0x10ba9d2c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509a9d34, SUC_lexicon_tags_33 },
    { 0x1fda9d35, SUC_lexicon_tags_12 },
    { 0x7d3a9d35, SUC_lexicon_tags_47 },
    { 0x629a9d35, SUC_lexicon_tags_49 },
    { 0x1fda9d38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ba9d3b, SUC_lexicon_tags_85 },
    { 0x87ba9d3c, SUC_lexicon_tags_38 },
    { 0x781a9d3d, SUC_lexicon_tags_27 },
    { 0xaeba9d3d, SUC_lexicon_tags_5 },
    { 0x937a9d3f, SUC_lexicon_tags_30 },
    { 0x8bba9d3e, SUC_lexicon_tags_37 },
    { 0x703a9d3e, SUC_lexicon_tags_2 },
    { 0xe59a9d3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3a9d46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x675a9d48, SUC_lexicon_tags_3 },
    { 0x9c1a9d49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c9a9d4b, SUC_lexicon_tags_17 },
    { 0x683a9d4b, SUC_lexicon_tags_7 },
    { 0xf5fa9d4d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43a9d50, SUC_lexicon_tags_17 },
    { 0xdfba9d51, SUC_lexicon_tags_7 },
    { 0x4c7a9d52, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927a9d57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x849a9d59, SUC_lexicon_tags_2 },
    { 0x3f1a9d5a, SUC_lexicon_tags_9 },
    { 0xfdfa9d59, SUC_lexicon_tags_15 },
    { 0x2ba9d5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9a9d5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca5a9d60, SUC_lexicon_tags_25 },
    { 0xc41a9d61, SUC_lexicon_tags_17 },
    { 0xd5ba9d62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343a9d6a, SUC_lexicon_tags_7 },
    { 0xf4ba9d6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfda9d74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9a9d77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6cfa9d79, SUC_lexicon_tags_2 },
    { 0xd8da9d79, SUC_lexicon_tags_2 },
    { 0x863a9d79, SUC_lexicon_tags_68 },
    { 0xf1da9d7a, SUC_lexicon_tags_25 },
    { 0xa3ba9d7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaba9d80, SUC_lexicon_tags_1 },
    { 0x5dda9d81, SUC_lexicon_tags_65 },
    { 0x607a9d82, SUC_lexicon_tags_23 },
    { 0xa29a9d83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7a9d88, SUC_lexicon_tags_14 },
    { 0x5b9a9d89, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefa9d8e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85a9d97, SUC_lexicon_tags_63 },
    { 0x189a9d98, SUC_lexicon_tags_0 },
    { 0x2bfa9d99, SUC_lexicon_tags_12 },
    { 0x29ba9d9a, SUC_lexicon_tags_2 },
    { 0x7d9a9d97, SUC_lexicon_tags_60 },
    { 0x2e7a9d99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9a9d9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x845a9da1, SUC_lexicon_tags_60 },
    { 0x923a9da2, SUC_lexicon_tags_26 },
    { 0x667a9da1, SUC_lexicon_tags_135 },
    { 0x625a9da3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6da9da7, SUC_lexicon_tags_37 },
    { 0x389a9da8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9a9dab, SUC_lexicon_tags_14 },
    { 0x199a9dac, SUC_lexicon_tags_23 },
    { 0xe81a9dac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12ba9daf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x401a9db2, SUC_lexicon_tags_25 },
    { 0x927a9db3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x153a9db5, SUC_lexicon_tags_82 },
    { 0x7b7a9db5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01a9dbb, SUC_lexicon_tags_70 },
    { 0xcc3a9dbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb85a9dbe, SUC_lexicon_tags_17 },
    { 0xad7a9dbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195a9dc4, SUC_lexicon_tags_15 },
    { 0x9a5a9dc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd49a9dc7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347a9dcb, SUC_lexicon_tags_17 },
    { 0xce1a9dcc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x817a9dce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7a9dda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1a9ddf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7efa9de2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fba9dea, SUC_lexicon_tags_25 },
    { 0x5e3a9deb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x875a9ded, SUC_lexicon_tags_68 },
    { 0x76da9ded, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x79da9df0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x683a9df3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x24fa9df5, SUC_lexicon_tags_7 },
    { 0xbb3a9df6, SUC_lexicon_tags_7 },
    { 0x5e9a9df7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f1a9df9, SUC_lexicon_tags_12 },
    { 0x521a9dfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ba9dff, SUC_lexicon_tags_25 },
    { 0x553a9e00, SUC_lexicon_tags_11 },
    { 0x215a9e00, SUC_lexicon_tags_25 },
    { 0x71fa9e02, SUC_lexicon_tags_25 },
    { 0xcb3a9e03, SUC_lexicon_tags_7 },
    { 0xcffa9e01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695a9e0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xde9a9e0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfa9e14, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfa9e19, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713a9e1e, SUC_lexicon_tags_60 },
    { 0x995a9e1f, SUC_lexicon_tags_16 },
    { 0x7e1a9e1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7a9e25, SUC_lexicon_tags_33 },
    { 0x76fa9e25, SUC_lexicon_tags_28 },
    { 0x977a9e27, SUC_lexicon_tags_25 },
    { 0xe33a9e28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701a9e2c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587a9e33, SUC_lexicon_tags_11 },
    { 0xd2da9e34, SUC_lexicon_tags_30 },
    { 0x869a9e34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49a9e39, SUC_lexicon_tags_5 },
    { 0xcf5a9e3a, SUC_lexicon_tags_15 },
    { 0xe35a9e3b, SUC_lexicon_tags_4 },
    { 0xa4da9e3b, SUC_lexicon_tags_47 },
    { 0x3cda9e3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12ba9e3f, SUC_lexicon_tags_25 },
    { 0xc43a9e3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x687a9e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfa9e48, SUC_lexicon_tags_10 },
    { 0x31ba9e48, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9a9e4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10da9e4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5a9e54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285a9e64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535a9e69, SUC_lexicon_tags_43 },
    { 0x5f7a9e6a, SUC_lexicon_tags_15 },
    { 0x209a9e6a, SUC_lexicon_tags_56 },
    { 0xf2ba9e6c, SUC_lexicon_tags_82 },
    { 0x109a9e6d, SUC_lexicon_tags_17 },
    { 0x595a9e6e, SUC_lexicon_tags_27 },
    { 0x443a9e6f, SUC_lexicon_tags_14 },
    { 0x97a9e6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdda9e75, SUC_lexicon_tags_11 },
    { 0x9fda9e76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fa9e7c, SUC_lexicon_tags_17 },
    { 0x9e1a9e7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71a9e81, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ba9e85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e9a9e87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507a9e8c, SUC_lexicon_tags_17 },
    { 0x3fda9e8d, SUC_lexicon_tags_44 },
    { 0x267a9e8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fa9e92, SUC_lexicon_tags_43 },
    { 0xc09a9e92, SUC_lexicon_tags_151 },
    { 0x60fa9e94, SUC_lexicon_tags_25 },
    { 0xe35a9e95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5a9e98, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fa9e9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545a9ea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325a9ea8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72da9eab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979a9eb0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7efa9eb3, SUC_lexicon_tags_65 },
    { 0x9ffa9eb4, SUC_lexicon_tags_17 },
    { 0xfc3a9eb5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfafa9eb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x365a9eb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49a9ebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17a9ec3, SUC_lexicon_tags_48 },
    { 0xe7a9ec3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xd39a9ec6, SUC_lexicon_tags_11 },
    { 0x96fa9ec7, SUC_lexicon_tags_15 },
    { 0x775a9ec8, SUC_lexicon_tags_7 },
    { 0xa43a9ec9, SUC_lexicon_tags_33 },
    { 0x71ba9ec9, SUC_lexicon_tags_43 },
    { 0x755a9ec9, SUC_lexicon_tags_12 },
    { 0x8efa9ecc, SUC_lexicon_tags_3 },
    { 0x505a9ecd, SUC_lexicon_tags_38 },
    { 0xc7da9ecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501a9ed8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1da9edb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71a9ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831a9ee5, SUC_lexicon_tags_49 },
    { 0x83ba9ee6, SUC_lexicon_tags_1685 },
    { 0x469a9ee7, SUC_lexicon_tags_1 },
    { 0x525a9ee8, SUC_lexicon_tags_27 },
    { 0x599a9ee9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9a9ef2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x45a9ef4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc65a9ef6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fa9efc, SUC_lexicon_tags_49 },
    { 0x7ba9efd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc9ba9eff, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fda9f07, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45a9f0d, SUC_lexicon_tags_25 },
    { 0x801a9f0e, SUC_lexicon_tags_17 },
    { 0x913a9f0d, SUC_lexicon_tags_17 },
    { 0x4d3a9f10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc53a9f12, SUC_lexicon_tags_37 },
    { 0xf21a9f13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eda9f18, SUC_lexicon_tags_4 },
    { 0x62ba9f19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80fa9f1b, SUC_lexicon_tags_12 },
    { 0x233a9f1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd3a9f1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939a9f22, SUC_lexicon_tags_28 },
    { 0xd25a9f22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fa9f2b, SUC_lexicon_tags_6 },
    { 0x9c3a9f2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21a9f30, SUC_lexicon_tags_33 },
    { 0x5a3a9f30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce5a9f33, SUC_lexicon_tags_12 },
    { 0xfd9a9f34, SUC_lexicon_tags_15 },
    { 0xe59a9f35, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x7a1a9f37, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fa9f3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a9f3e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x523a9f41, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbdfa9f43, SUC_lexicon_tags_7 },
    { 0x853a9f43, SUC_lexicon_tags_129 },
    { 0xb67a9f45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509a9f49, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77a9f4f, SUC_lexicon_tags_17 },
    { 0x9f1a9f4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791a9f5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e5a9f5c, SUC_lexicon_tags_0 },
    { 0xbd9a9f5c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x92da9f5f, SUC_lexicon_tags_7 },
    { 0x521a9f5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacba9f62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1a9f65, SUC_lexicon_tags_7 },
    { 0x47da9f65, SUC_lexicon_tags_33 },
    { 0xa73a9f65, SUC_lexicon_tags_25 },
    { 0xb69a9f68, SUC_lexicon_tags_15 },
    { 0x75da9f68, SUC_lexicon_tags_23 },
    { 0x801a9f6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7c9a9f6c, SUC_lexicon_tags_44 },
    { 0x2bfa9f6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19a9f70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bfa9f72, SUC_lexicon_tags_17 },
    { 0x7e9a9f73, SUC_lexicon_tags_14 },
    { 0x6a1a9f74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe05a9f76, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1dda9f78, SUC_lexicon_tags_11 },
    { 0xc95a9f79, SUC_lexicon_tags_38 },
    { 0x703a9f7a, SUC_lexicon_tags_2 },
    { 0x48fa9f79, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5a9f7f, SUC_lexicon_tags_37 },
    { 0xee3a9f80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1a9f84, SUC_lexicon_tags_55 },
    { 0x16ba9f85, SUC_lexicon_tags_48 },
    { 0x123a9f85, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaa1a9f88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ba9f8c, SUC_lexicon_tags_48 },
    { 0x163a9f8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dba9f92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafda9f98, SUC_lexicon_tags_17 },
    { 0x60ba9f98, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2da9f9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x475a9f9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb4da9fa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x197a9fa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7a9fa5, SUC_lexicon_tags_19 },
    { 0xec1a9fa6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x95a9fa9, SUC_lexicon_tags_31 },
    { 0x41ba9faa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7a9fb0, SUC_lexicon_tags_17 },
    { 0x4e9a9fb1, SUC_lexicon_tags_6 },
    { 0x6c3a9fb2, SUC_lexicon_tags_6 },
    { 0x969a9fb0, SUC_lexicon_tags_31 },
    { 0xd6da9fb4, SUC_lexicon_tags_55 },
    { 0x841a9fb1, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05a9fba, SUC_lexicon_tags_408 },
    { 0xc13a9fbb, SUC_lexicon_tags_19 },
    { 0xc3a9fbc, SUC_lexicon_tags_11 },
    { 0x6d1a9fbd, SUC_lexicon_tags_1 },
    { 0xb47a9fbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53a9fc0, SUC_lexicon_tags_12 },
    { 0x8c9a9fc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x235a9fc3, SUC_lexicon_tags_2 },
    { 0xf69a9fc4, SUC_lexicon_tags_2 },
    { 0xeaba9fc5, SUC_lexicon_tags_30 },
    { 0x57a9fc5, SUC_lexicon_tags_15 },
    { 0xe5fa9fc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fa9fd0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843a9fd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31a9fd9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31a9fdc, SUC_lexicon_tags_15 },
    { 0x349a9fdd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x735a9fe0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9a9fe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x199a9fe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47a9fec, SUC_lexicon_tags_27 },
    { 0xb35a9fed, SUC_lexicon_tags_15 },
    { 0x487a9fee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ba9ff1, SUC_lexicon_tags_6 },
    { 0x9efa9ff1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbc5a9ff4, SUC_lexicon_tags_37 },
    { 0x517a9ff4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17a9ffc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd53a9ffe, SUC_lexicon_tags_95 },
    { 0x47a9ffe, SUC_lexicon_tags_12 },
    { 0xf8faa000, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95aa007, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cbaa009, SUC_lexicon_tags_38 },
    { 0xf71aa00a, SUC_lexicon_tags_25 },
    { 0x233aa00b, SUC_lexicon_tags_7 },
    { 0x6baa00a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce3aa00e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e1aa010, SUC_lexicon_tags_26 },
    { 0xdb5aa010, SUC_lexicon_tags_11 },
    { 0xaa7aa011, SUC_lexicon_tags_437 },
    { 0x86baa013, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff1aa015, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5aa019, SUC_lexicon_tags_37 },
    { 0x769aa01a, SUC_lexicon_tags_1 },
    { 0x73aa01a, SUC_lexicon_tags_9 },
    { 0xb67aa01c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a3aa01e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x677aa020, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13aa023, SUC_lexicon_tags_23 },
    { 0x9cdaa024, SUC_lexicon_tags_33 },
    { 0x981aa023, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301aa029, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253aa02d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbaa045, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83daa047, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a7aa049, SUC_lexicon_tags_47 },
    { 0x4f1aa04a, SUC_lexicon_tags_25 },
    { 0xef3aa04b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53aa04e, SUC_lexicon_tags_37 },
    { 0x213aa04f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1faa052, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf4daa054, SUC_lexicon_tags_37 },
    { 0x78daa055, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x605aa057, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7aa05b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x55baa05e, SUC_lexicon_tags_7 },
    { 0x91daa05e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761aa063, SUC_lexicon_tags_16 },
    { 0x929aa063, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadaa068, SUC_lexicon_tags_7 },
    { 0x7d5aa069, SUC_lexicon_tags_15 },
    { 0x9baa06a, SUC_lexicon_tags_17 },
    { 0x631aa06b, SUC_lexicon_tags_57 },
    { 0x1d5aa06b, SUC_lexicon_tags_14 },
    { 0x331aa06d, SUC_lexicon_tags_44 },
    { 0x449aa06e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d9aa070, SUC_lexicon_tags_15 },
    { 0xd27aa071, SUC_lexicon_tags_3 },
    { 0xb99aa072, SUC_lexicon_tags_12 },
    { 0xdbfaa071, SUC_lexicon_tags_25 },
    { 0x887aa074, SUC_lexicon_tags_11 },
    { 0xe5daa075, SUC_lexicon_tags_45 },
    { 0x853aa076, SUC_lexicon_tags_37 },
    { 0xe47aa077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53aa084, SUC_lexicon_tags_60 },
    { 0xa17aa085, SUC_lexicon_tags_65 },
    { 0xfd9aa084, SUC_lexicon_tags_14 },
    { 0x995aa087, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fbaa089, SUC_lexicon_tags_17 },
    { 0x68daa089, SUC_lexicon_tags_27 },
    { 0xe27aa08b, SUC_lexicon_tags_1 },
    { 0x39daa08c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a7aa08e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb7baa090, SUC_lexicon_tags_37 },
    { 0x39baa091, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xddbaa093, SUC_lexicon_tags_43 },
    { 0xe83aa093, SUC_lexicon_tags_26 },
    { 0xf7faa093, SUC_lexicon_tags_2 },
    { 0xb65aa094, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57aa09d, SUC_lexicon_tags_2 },
    { 0x99baa09e, SUC_lexicon_tags_15 },
    { 0xdafaa09f, SUC_lexicon_tags_2 },
    { 0xa8faa09f, SUC_lexicon_tags_47 },
    { 0xe37aa0a0, SUC_lexicon_tags_27 },
    { 0x56faa0a1, SUC_lexicon_tags_584 },
    { 0, NULL },
    { 0, NULL },
    { 0x58faa0a5, SUC_lexicon_tags_31 },
    { 0x165aa0a6, SUC_lexicon_tags_43 },
    { 0x36daa0a7, SUC_lexicon_tags_25 },
    { 0x1b5aa0a5, SUC_lexicon_tags_45 },
    { 0x191aa0a8, SUC_lexicon_tags_0 },
    { 0x7baa0aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x723aa0ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e1aa0ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7aa0b2, SUC_lexicon_tags_11 },
    { 0x591aa0b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec5aa0b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5aa0b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07aa0be, SUC_lexicon_tags_9 },
    { 0xc71aa0bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x80baa0c2, SUC_lexicon_tags_55 },
    { 0x8edaa0c3, SUC_lexicon_tags_29 },
    { 0x2d3aa0c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x217aa0c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbaa0ca, SUC_lexicon_tags_43 },
    { 0xb77aa0ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7baa0cf, SUC_lexicon_tags_37 },
    { 0x89daa0d0, SUC_lexicon_tags_726 },
    { 0x909aa0d0, SUC_lexicon_tags_1 },
    { 0x3d7aa0cf, SUC_lexicon_tags_1 },
    { 0xaddaa0d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377aa0d7, SUC_lexicon_tags_23 },
    { 0x80baa0d7, SUC_lexicon_tags_82 },
    { 0xc01aa0d9, SUC_lexicon_tags_25 },
    { 0x1ddaa0da, SUC_lexicon_tags_55 },
    { 0xce1aa0d9, SUC_lexicon_tags_17 },
    { 0x955aa0d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x21baa0df, SUC_lexicon_tags_39 },
    { 0xdc7aa0e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x87baa0e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70daa0e5, SUC_lexicon_tags_25 },
    { 0x4abaa0e6, SUC_lexicon_tags_23 },
    { 0x835aa0e7, SUC_lexicon_tags_47 },
    { 0x993aa0e7, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85aa0eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311aa0f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3aa0f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe57aa0f8, SUC_lexicon_tags_49 },
    { 0x53baa0f9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afaa0fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xefbaa0fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119aa102, SUC_lexicon_tags_26 },
    { 0x921aa103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3baa107, SUC_lexicon_tags_55 },
    { 0x995aa108, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe63aa10a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddaa10d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x415aa10f, SUC_lexicon_tags_11 },
    { 0x80daa10f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32daa112, SUC_lexicon_tags_15 },
    { 0x7bfaa113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x283aa115, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebfaa118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x617aa11a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5aa11f, SUC_lexicon_tags_1 },
    { 0x7ffaa11f, SUC_lexicon_tags_37 },
    { 0x61faa121, SUC_lexicon_tags_66 },
    { 0x52daa121, SUC_lexicon_tags_28 },
    { 0xd1faa120, SUC_lexicon_tags_37 },
    { 0x90faa124, SUC_lexicon_tags_911 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411aa12a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95daa12c, SUC_lexicon_tags_218 },
    { 0xd7aa12c, SUC_lexicon_tags_11 },
    { 0x1adaa12e, SUC_lexicon_tags_47 },
    { 0xe09aa12c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x281aa132, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x797aa135, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a3aa137, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05aa13c, SUC_lexicon_tags_1 },
    { 0xaedaa13d, SUC_lexicon_tags_25 },
    { 0x981aa13e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81aa141, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47daa145, SUC_lexicon_tags_19 },
    { 0x5afaa145, SUC_lexicon_tags_37 },
    { 0xee9aa145, SUC_lexicon_tags_2 },
    { 0xd47aa146, SUC_lexicon_tags_6 },
    { 0x72daa149, SUC_lexicon_tags_49 },
    { 0x5c9aa14a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88daa14d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777aa152, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69daa159, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xef1aa15b, SUC_lexicon_tags_68 },
    { 0x8a1aa15c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75baa160, SUC_lexicon_tags_77 },
    { 0xeefaa161, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdfbaa163, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845aa168, SUC_lexicon_tags_12 },
    { 0xc5daa168, SUC_lexicon_tags_64 },
    { 0xfbdaa16a, SUC_lexicon_tags_2 },
    { 0x37daa16b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfd1aa16d, SUC_lexicon_tags_21 },
    { 0x757aa16d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1aa173, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57aa17a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3aa17d, SUC_lexicon_tags_116 },
    { 0xd11aa17e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847aa189, SUC_lexicon_tags_9 },
    { 0xdd1aa18a, SUC_lexicon_tags_11 },
    { 0xf6faa189, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9aa193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21aa19a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1aa1a0, SUC_lexicon_tags_1 },
    { 0x3c1aa1a1, SUC_lexicon_tags_33 },
    { 0x709aa1a0, SUC_lexicon_tags_2 },
    { 0xf49aa1a0, SUC_lexicon_tags_26 },
    { 0x719aa1a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fbaa1a6, SUC_lexicon_tags_2 },
    { 0xbcbaa1a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03aa1b5, SUC_lexicon_tags_37 },
    { 0xd13aa1b5, SUC_lexicon_tags_15 },
    { 0xc5daa1b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523aa1bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x161aa1be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33aa1c1, SUC_lexicon_tags_7 },
    { 0x333aa1c2, SUC_lexicon_tags_10 },
    { 0xb19aa1c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe93aa1c5, SUC_lexicon_tags_1 },
    { 0x3f1aa1c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3aa1c9, SUC_lexicon_tags_17 },
    { 0x8c3aa1c9, SUC_lexicon_tags_7 },
    { 0xa71aa1ca, SUC_lexicon_tags_37 },
    { 0xf01aa1cc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9c3aa1ce, SUC_lexicon_tags_33 },
    { 0x777aa1ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbaa1d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x52baa1d4, SUC_lexicon_tags_19 },
    { 0xb37aa1d5, SUC_lexicon_tags_8 },
    { 0xa1daa1d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9aa1da, SUC_lexicon_tags_15 },
    { 0x56baa1db, SUC_lexicon_tags_28 },
    { 0x7b9aa1dc, SUC_lexicon_tags_11 },
    { 0x2f5aa1dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x68baa1e0, SUC_lexicon_tags_25 },
    { 0x20baa1e1, SUC_lexicon_tags_74 },
    { 0xec9aa1e1, SUC_lexicon_tags_31 },
    { 0x49aa1e3, SUC_lexicon_tags_12 },
    { 0xb5baa1e4, SUC_lexicon_tags_17 },
    { 0xdd9aa1e3, SUC_lexicon_tags_0 },
    { 0xe95aa1e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xab7aa1e8, SUC_lexicon_tags_713 },
    { 0, NULL },
    { 0xce1aa1ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x121aa1ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x991aa1ee, SUC_lexicon_tags_31 },
    { 0x6baa1ef, SUC_lexicon_tags_65 },
    { 0x689aa1f0, SUC_lexicon_tags_29 },
    { 0x96daa1f1, SUC_lexicon_tags_37 },
    { 0x529aa1f2, SUC_lexicon_tags_87 },
    { 0xc31aa1ee, SUC_lexicon_tags_31 },
    { 0xdcfaa1f4, SUC_lexicon_tags_45 },
    { 0xbe3aa1f4, SUC_lexicon_tags_6 },
    { 0x33aa1f6, SUC_lexicon_tags_19 },
    { 0x35aa1f4, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0x601aa1fa, SUC_lexicon_tags_29 },
    { 0x9e7aa1fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2c7aa1fd, SUC_lexicon_tags_37 },
    { 0x1fbaa1fd, SUC_lexicon_tags_7 },
    { 0xab5aa1fd, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161aa204, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x689aa207, SUC_lexicon_tags_45 },
    { 0x887aa208, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x481aa20a, SUC_lexicon_tags_2 },
    { 0x9a7aa20b, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbaa215, SUC_lexicon_tags_7 },
    { 0xeb1aa216, SUC_lexicon_tags_17 },
    { 0x9b1aa216, SUC_lexicon_tags_25 },
    { 0x6d3aa215, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfa5aa21a, SUC_lexicon_tags_37 },
    { 0x1dbaa21b, SUC_lexicon_tags_11 },
    { 0xec1aa21c, SUC_lexicon_tags_39 },
    { 0xba1aa21c, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85aa221, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdaa224, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47aa228, SUC_lexicon_tags_2 },
    { 0x5e5aa228, SUC_lexicon_tags_46 },
    { 0x16baa22a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7faa22d, SUC_lexicon_tags_0 },
    { 0xc23aa22e, SUC_lexicon_tags_53 },
    { 0x9a7aa22f, SUC_lexicon_tags_2 },
    { 0x2e5aa230, SUC_lexicon_tags_48 },
    { 0xebfaa230, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x747aa233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1aa236, SUC_lexicon_tags_17 },
    { 0x7f9aa237, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x25daa23a, SUC_lexicon_tags_15 },
    { 0x4ebaa23b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1aa23f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbaa242, SUC_lexicon_tags_1 },
    { 0x323aa243, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfaa246, SUC_lexicon_tags_21 },
    { 0x60faa247, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfaa24e, SUC_lexicon_tags_33 },
    { 0x13daa24f, SUC_lexicon_tags_11 },
    { 0xe83aa250, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebaa254, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581aa25a, SUC_lexicon_tags_31 },
    { 0x1bfaa25a, SUC_lexicon_tags_37 },
    { 0x4f1aa25a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4daa260, SUC_lexicon_tags_46 },
    { 0x95daa261, SUC_lexicon_tags_33 },
    { 0xecdaa261, SUC_lexicon_tags_149 },
    { 0xbb7aa263, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1aa268, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbaa26b, SUC_lexicon_tags_17 },
    { 0x3d1aa26c, SUC_lexicon_tags_3 },
    { 0x605aa26d, SUC_lexicon_tags_38 },
    { 0xa53aa26d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327aa272, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4daa275, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbaa278, SUC_lexicon_tags_23 },
    { 0x881aa279, SUC_lexicon_tags_15 },
    { 0x5e9aa27a, SUC_lexicon_tags_66 },
    { 0x391aa278, SUC_lexicon_tags_15 },
    { 0x87faa27c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6faa280, SUC_lexicon_tags_2 },
    { 0xfa9aa281, SUC_lexicon_tags_60 },
    { 0xed5aa282, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedaa289, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x75aa28b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05aa28e, SUC_lexicon_tags_14 },
    { 0x15daa28f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497aa295, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc23aa297, SUC_lexicon_tags_11 },
    { 0x61baa297, SUC_lexicon_tags_15 },
    { 0x15faa299, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9aa29f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x317aa2a1, SUC_lexicon_tags_37 },
    { 0xc3faa2a2, SUC_lexicon_tags_1 },
    { 0x67faa2a3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1aa2a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d7aa2a8, SUC_lexicon_tags_2 },
    { 0x231aa2a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3aa2ae, SUC_lexicon_tags_1 },
    { 0xaffaa2af, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x411aa2b1, SUC_lexicon_tags_948 },
    { 0, NULL },
    { 0xc31aa2b3, SUC_lexicon_tags_28 },
    { 0x869aa2b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7aa2bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x941aa2bd, SUC_lexicon_tags_27 },
    { 0x197aa2be, SUC_lexicon_tags_9 },
    { 0x55faa2be, SUC_lexicon_tags_11 },
    { 0x52faa2bf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x871aa2c3, SUC_lexicon_tags_45 },
    { 0xc8baa2c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821aa2ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91aa2cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60faa2d3, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x8d1aa2d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b3aa2d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc2baa2d9, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0x35daa2db, SUC_lexicon_tags_87 },
    { 0x7fdaa2dc, SUC_lexicon_tags_38 },
    { 0x32baa2dd, SUC_lexicon_tags_63 },
    { 0xdffaa2de, SUC_lexicon_tags_25 },
    { 0x22faa2df, SUC_lexicon_tags_7 },
    { 0x415aa2df, SUC_lexicon_tags_60 },
    { 0x1d9aa2e1, SUC_lexicon_tags_60 },
    { 0xf1aa2e2, SUC_lexicon_tags_17 },
    { 0xdc5aa2e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9aa2e6, SUC_lexicon_tags_12 },
    { 0x5d7aa2e6, SUC_lexicon_tags_11 },
    { 0x1bbaa2e8, SUC_lexicon_tags_10 },
    { 0x521aa2e7, SUC_lexicon_tags_25 },
    { 0x575aa2e8, SUC_lexicon_tags_37 },
    { 0xcfaa2eb, SUC_lexicon_tags_25 },
    { 0x5c1aa2ec, SUC_lexicon_tags_12 },
    { 0xf0daa2e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x411aa2f0, SUC_lexicon_tags_0 },
    { 0xf8baa2f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5aa2f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1aa2f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xea5aa2fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd4baa2fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29aa2ff, SUC_lexicon_tags_11 },
    { 0xca5aa300, SUC_lexicon_tags_26 },
    { 0x409aa301, SUC_lexicon_tags_2 },
    { 0x20daa302, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa83aa304, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43aa308, SUC_lexicon_tags_14 },
    { 0x349aa308, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211aa30e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf59aa311, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffaa315, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xc97aa317, SUC_lexicon_tags_0 },
    { 0x1bbaa317, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x89baa31a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5faa31d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbaa321, SUC_lexicon_tags_1 },
    { 0xa2daa322, SUC_lexicon_tags_8 },
    { 0xf35aa323, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x749aa326, SUC_lexicon_tags_29 },
    { 0x72baa326, SUC_lexicon_tags_24 },
    { 0xd5daa326, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5baa32b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xd3baa32d, SUC_lexicon_tags_14 },
    { 0xc37aa32d, SUC_lexicon_tags_31 },
    { 0x667aa32f, SUC_lexicon_tags_17 },
    { 0xeb5aa32e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565aa335, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4baa33a, SUC_lexicon_tags_21 },
    { 0x77baa33a, SUC_lexicon_tags_434 },
    { 0, NULL },
    { 0xe01aa33d, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0x875aa33f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8afaa342, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5baa346, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2faa349, SUC_lexicon_tags_2 },
    { 0x3b5aa34a, SUC_lexicon_tags_23 },
    { 0xa03aa34a, SUC_lexicon_tags_9 },
    { 0x7f7aa34c, SUC_lexicon_tags_25 },
    { 0x7c9aa34d, SUC_lexicon_tags_34 },
    { 0xab7aa34e, SUC_lexicon_tags_7 },
    { 0x20daa34e, SUC_lexicon_tags_0 },
    { 0x229aa34a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899aa355, SUC_lexicon_tags_13 },
    { 0x35daa356, SUC_lexicon_tags_23 },
    { 0xfa5aa356, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd93aa35a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efaa35f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x54faa362, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcf9aa364, SUC_lexicon_tags_35 },
    { 0x17baa365, SUC_lexicon_tags_7 },
    { 0x921aa366, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5faa36b, SUC_lexicon_tags_37 },
    { 0xab7aa36c, SUC_lexicon_tags_17 },
    { 0x8f5aa36d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x913aa36f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbaa373, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x259aa376, SUC_lexicon_tags_37 },
    { 0x7cdaa377, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x49daa37a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x381aa37c, SUC_lexicon_tags_25 },
    { 0xc51aa37d, SUC_lexicon_tags_27 },
    { 0x663aa37e, SUC_lexicon_tags_11 },
    { 0x3aa37f, SUC_lexicon_tags_7 },
    { 0x839aa37f, SUC_lexicon_tags_25 },
    { 0x8abaa37d, SUC_lexicon_tags_45 },
    { 0x8a3aa382, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf83aa384, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1bbaa386, SUC_lexicon_tags_48 },
    { 0x575aa387, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edaa38a, SUC_lexicon_tags_18 },
    { 0x4a5aa38b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x529aa38e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61daa390, SUC_lexicon_tags_47 },
    { 0xfc1aa391, SUC_lexicon_tags_12 },
    { 0xbd5aa392, SUC_lexicon_tags_27 },
    { 0x1a3aa392, SUC_lexicon_tags_29 },
    { 0xccdaa394, SUC_lexicon_tags_14 },
    { 0x731aa395, SUC_lexicon_tags_12 },
    { 0xe25aa394, SUC_lexicon_tags_0 },
    { 0x6f1aa397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51aa39a, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0x13daa39c, SUC_lexicon_tags_1 },
    { 0x405aa39c, SUC_lexicon_tags_13 },
    { 0xdc1aa39e, SUC_lexicon_tags_11 },
    { 0xee5aa39c, SUC_lexicon_tags_26 },
    { 0xc71aa39d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9aa3a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x723aa3a5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303aa3ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25aa3ae, SUC_lexicon_tags_46 },
    { 0xe19aa3af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb89aa3b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29aa3b5, SUC_lexicon_tags_7 },
    { 0x63baa3b6, SUC_lexicon_tags_25 },
    { 0x5c3aa3b7, SUC_lexicon_tags_19 },
    { 0xddfaa3b8, SUC_lexicon_tags_23 },
    { 0xd93aa3b9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x81daa3bc, SUC_lexicon_tags_11 },
    { 0x61aa3bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3baa3c3, SUC_lexicon_tags_15 },
    { 0xb49aa3c4, SUC_lexicon_tags_2 },
    { 0x4d1aa3c5, SUC_lexicon_tags_37 },
    { 0xbbbaa3c6, SUC_lexicon_tags_46 },
    { 0x8dbaa3c5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5aa3ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77aa3cf, SUC_lexicon_tags_7 },
    { 0x30faa3cf, SUC_lexicon_tags_12 },
    { 0x1a3aa3d1, SUC_lexicon_tags_1 },
    { 0x671aa3d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb45aa3d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31aa3d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x295aa3da, SUC_lexicon_tags_47 },
    { 0xcebaa3da, SUC_lexicon_tags_28 },
    { 0x155aa3dc, SUC_lexicon_tags_46 },
    { 0x2abaa3dd, SUC_lexicon_tags_15 },
    { 0xa4faa3de, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x825aa3e0, SUC_lexicon_tags_43 },
    { 0x193aa3e1, SUC_lexicon_tags_12 },
    { 0xe89aa3e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd9daa3e4, SUC_lexicon_tags_25 },
    { 0x8daa3e5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53daa3e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759aa3ed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb73aa3ef, SUC_lexicon_tags_16 },
    { 0x707aa3f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf61aa3f2, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbaa3f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1aa3fc, SUC_lexicon_tags_11 },
    { 0x127aa3fd, SUC_lexicon_tags_17 },
    { 0xd07aa3fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x177aa400, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1aa405, SUC_lexicon_tags_27 },
    { 0x6ebaa405, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xcd5aa408, SUC_lexicon_tags_0 },
    { 0xdbaa409, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddaa40d, SUC_lexicon_tags_17 },
    { 0xf7daa40d, SUC_lexicon_tags_25 },
    { 0x431aa40f, SUC_lexicon_tags_25 },
    { 0x31baa40e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdaa417, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbcbaa419, SUC_lexicon_tags_7 },
    { 0x2afaa41a, SUC_lexicon_tags_12 },
    { 0xabdaa41a, SUC_lexicon_tags_16 },
    { 0xa5baa41c, SUC_lexicon_tags_15 },
    { 0xf95aa41d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89baa420, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdaa425, SUC_lexicon_tags_11 },
    { 0xe1baa425, SUC_lexicon_tags_17 },
    { 0xb1aa426, SUC_lexicon_tags_100 },
    { 0x7e3aa428, SUC_lexicon_tags_27 },
    { 0xa3baa425, SUC_lexicon_tags_25 },
    { 0x3b7aa42a, SUC_lexicon_tags_16 },
    { 0x751aa42b, SUC_lexicon_tags_80 },
    { 0xfbbaa42c, SUC_lexicon_tags_17 },
    { 0x9d1aa42b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x447aa42f, SUC_lexicon_tags_2 },
    { 0x787aa430, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1aa437, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x133aa439, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69aa43c, SUC_lexicon_tags_15 },
    { 0x289aa43d, SUC_lexicon_tags_27 },
    { 0xdb5aa43d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69aa442, SUC_lexicon_tags_11 },
    { 0xe19aa442, SUC_lexicon_tags_27 },
    { 0xee3aa444, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe15aa446, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf9aa448, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81aa44a, SUC_lexicon_tags_25 },
    { 0x75baa44b, SUC_lexicon_tags_17 },
    { 0xb29aa44b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17baa450, SUC_lexicon_tags_37 },
    { 0x4e1aa451, SUC_lexicon_tags_7 },
    { 0xb7daa452, SUC_lexicon_tags_17 },
    { 0x77faa453, SUC_lexicon_tags_11 },
    { 0x629aa452, SUC_lexicon_tags_31 },
    { 0xc6daa451, SUC_lexicon_tags_37 },
    { 0x4a9aa456, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x15daa458, SUC_lexicon_tags_23 },
    { 0xe67aa459, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5aa460, SUC_lexicon_tags_37 },
    { 0xeebaa461, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701aa468, SUC_lexicon_tags_17 },
    { 0x90baa469, SUC_lexicon_tags_68 },
    { 0xc3aa46a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x531aa46c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5aa473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57aa47b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5aa487, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c7aa489, SUC_lexicon_tags_27 },
    { 0xfd3aa48a, SUC_lexicon_tags_27 },
    { 0x58daa48b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83aa48f, SUC_lexicon_tags_47 },
    { 0x309aa490, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7baa494, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1aa497, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67aa4a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x75aa4a3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd15aa4a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69faa4ab, SUC_lexicon_tags_2 },
    { 0xa5baa4ac, SUC_lexicon_tags_37 },
    { 0xc29aa4ab, SUC_lexicon_tags_46 },
    { 0x97faa4ad, SUC_lexicon_tags_38 },
    { 0x71faa4af, SUC_lexicon_tags_17 },
    { 0x9ebaa4b0, SUC_lexicon_tags_37 },
    { 0xf47aa4b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf11aa4b3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8daa4ba, SUC_lexicon_tags_47 },
    { 0x6f5aa4ba, SUC_lexicon_tags_43 },
    { 0xc0faa4ba, SUC_lexicon_tags_14 },
    { 0x4a9aa4bc, SUC_lexicon_tags_2 },
    { 0xc6daa4be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebaa4c3, SUC_lexicon_tags_64 },
    { 0xa67aa4c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3baa4c6, SUC_lexicon_tags_17 },
    { 0xf8faa4c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddaa4cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbaa4cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x883aa4d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x131aa4d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6faa4d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5faa4d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x68faa4db, SUC_lexicon_tags_17 },
    { 0xe6faa4db, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683aa4e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8daa4e9, SUC_lexicon_tags_17 },
    { 0x7ebaa4e9, SUC_lexicon_tags_4 },
    { 0x94daa4eb, SUC_lexicon_tags_31 },
    { 0xd6faa4eb, SUC_lexicon_tags_17 },
    { 0xb5aa4ed, SUC_lexicon_tags_84 },
    { 0x2f3aa4ec, SUC_lexicon_tags_7 },
    { 0x991aa4ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x53aa4f2, SUC_lexicon_tags_63 },
    { 0x941aa4f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x411aa4f6, SUC_lexicon_tags_11 },
    { 0x7bdaa4f6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfaa4fa, SUC_lexicon_tags_2 },
    { 0x811aa4fb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31aa4ff, SUC_lexicon_tags_1 },
    { 0xf7daa4ff, SUC_lexicon_tags_12 },
    { 0x8f3aa501, SUC_lexicon_tags_15 },
    { 0x663aa502, SUC_lexicon_tags_22 },
    { 0x4bbaa500, SUC_lexicon_tags_48 },
    { 0xe7daa501, SUC_lexicon_tags_12 },
    { 0xac9aa505, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdaa509, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3aa50b, SUC_lexicon_tags_61 },
    { 0x12faa50b, SUC_lexicon_tags_11 },
    { 0x7e5aa50c, SUC_lexicon_tags_12 },
    { 0x27baa50b, SUC_lexicon_tags_31 },
    { 0x831aa50f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3faa513, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5daa520, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x737aa523, SUC_lexicon_tags_38 },
    { 0xea1aa524, SUC_lexicon_tags_15 },
    { 0xb19aa524, SUC_lexicon_tags_0 },
    { 0xadbaa524, SUC_lexicon_tags_2 },
    { 0xe61aa527, SUC_lexicon_tags_25 },
    { 0x811aa528, SUC_lexicon_tags_23 },
    { 0x41aa526, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedaa52e, SUC_lexicon_tags_46 },
    { 0xc43aa52f, SUC_lexicon_tags_19 },
    { 0x765aa52f, SUC_lexicon_tags_2 },
    { 0x283aa531, SUC_lexicon_tags_49 },
    { 0x3d3aa531, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27aa535, SUC_lexicon_tags_47 },
    { 0xae3aa536, SUC_lexicon_tags_0 },
    { 0x979aa535, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1aa53b, SUC_lexicon_tags_1 },
    { 0x32daa53c, SUC_lexicon_tags_23 },
    { 0x103aa53d, SUC_lexicon_tags_31 },
    { 0x181aa53e, SUC_lexicon_tags_15 },
    { 0x95faa53b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x627aa541, SUC_lexicon_tags_66 },
    { 0xaf3aa541, SUC_lexicon_tags_9 },
    { 0x129aa542, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x79faa546, SUC_lexicon_tags_43 },
    { 0x897aa546, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4baa54d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xdfaa54f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x789aa551, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa21aa553, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90baa558, SUC_lexicon_tags_29 },
    { 0x51aa559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc25aa55b, SUC_lexicon_tags_17 },
    { 0x11aa55c, SUC_lexicon_tags_25 },
    { 0xf6faa55d, SUC_lexicon_tags_25 },
    { 0xd85aa55e, SUC_lexicon_tags_12 },
    { 0xcbbaa55f, SUC_lexicon_tags_25 },
    { 0x39baa560, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbdbaa562, SUC_lexicon_tags_46 },
    { 0xb5faa562, SUC_lexicon_tags_26 },
    { 0xb1daa564, SUC_lexicon_tags_25 },
    { 0x727aa562, SUC_lexicon_tags_17 },
    { 0xc9baa564, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1dfaa568, SUC_lexicon_tags_27 },
    { 0x911aa569, SUC_lexicon_tags_15 },
    { 0x12daa56a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x557aa56d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1daa570, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddaa573, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9daa575, SUC_lexicon_tags_65 },
    { 0xf45aa576, SUC_lexicon_tags_26 },
    { 0x679aa576, SUC_lexicon_tags_30 },
    { 0x2c1aa576, SUC_lexicon_tags_15 },
    { 0x737aa579, SUC_lexicon_tags_57 },
    { 0xcd5aa57a, SUC_lexicon_tags_27 },
    { 0x65baa575, SUC_lexicon_tags_25 },
    { 0xd25aa57c, SUC_lexicon_tags_87 },
    { 0x31baa57c, SUC_lexicon_tags_3 },
    { 0xaebaa576, SUC_lexicon_tags_126 },
    { 0x9cbaa57d, SUC_lexicon_tags_6 },
    { 0x3bdaa57a, SUC_lexicon_tags_1 },
    { 0xa8baa57b, SUC_lexicon_tags_87 },
    { 0x8abaa57b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10faa585, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37aa58c, SUC_lexicon_tags_1 },
    { 0xbffaa58d, SUC_lexicon_tags_30 },
    { 0xb5faa58d, SUC_lexicon_tags_11 },
    { 0xf65aa58d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb29aa591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5faa593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f5aa595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31aa5a4, SUC_lexicon_tags_15 },
    { 0x2cbaa5a5, SUC_lexicon_tags_29 },
    { 0x4d7aa5a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7aa5a9, SUC_lexicon_tags_17 },
    { 0x409aa5aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01aa5b0, SUC_lexicon_tags_15 },
    { 0x1f5aa5b1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6bfaa5b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x661aa5b6, SUC_lexicon_tags_25 },
    { 0x3efaa5b7, SUC_lexicon_tags_35 },
    { 0xe59aa5b6, SUC_lexicon_tags_57 },
    { 0x109aa5b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x809aa5bc, SUC_lexicon_tags_25 },
    { 0x979aa5bd, SUC_lexicon_tags_12 },
    { 0xd7daa5be, SUC_lexicon_tags_38 },
    { 0xf69aa5bf, SUC_lexicon_tags_10 },
    { 0x4d9aa5bd, SUC_lexicon_tags_60 },
    { 0xefdaa5be, SUC_lexicon_tags_11 },
    { 0xbc1aa5c2, SUC_lexicon_tags_6 },
    { 0x363aa5bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95aa5c8, SUC_lexicon_tags_27 },
    { 0xc5daa5c8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf3aa5cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669aa5d0, SUC_lexicon_tags_3 },
    { 0x92baa5d1, SUC_lexicon_tags_15 },
    { 0x9a3aa5d0, SUC_lexicon_tags_12 },
    { 0x2ddaa5d3, SUC_lexicon_tags_5 },
    { 0xe61aa5d4, SUC_lexicon_tags_17 },
    { 0xf27aa5d2, SUC_lexicon_tags_14 },
    { 0x8dfaa5d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89aa5d9, SUC_lexicon_tags_60 },
    { 0x245aa5da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90faa5df, SUC_lexicon_tags_2 },
    { 0x31faa5e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5aa5e5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x919aa5e7, SUC_lexicon_tags_12 },
    { 0xbe3aa5e8, SUC_lexicon_tags_2 },
    { 0xc4faa5e8, SUC_lexicon_tags_11 },
    { 0xc0faa5ea, SUC_lexicon_tags_49 },
    { 0x1e9aa5eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95aa5f0, SUC_lexicon_tags_17 },
    { 0x65faa5f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7aa5f4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3c3aa5f6, SUC_lexicon_tags_26 },
    { 0xfffaa5f7, SUC_lexicon_tags_49 },
    { 0x98daa5f7, SUC_lexicon_tags_49 },
    { 0xb7daa5f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3aa5fd, SUC_lexicon_tags_25 },
    { 0x771aa5fe, SUC_lexicon_tags_1 },
    { 0xa19aa5ff, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x559aa601, SUC_lexicon_tags_17 },
    { 0xa27aa602, SUC_lexicon_tags_9 },
    { 0xec7aa602, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x953aa605, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7aa60d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913aa614, SUC_lexicon_tags_7 },
    { 0xac1aa615, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd8faa617, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391aa61b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x427aa61d, SUC_lexicon_tags_12 },
    { 0x3f5aa61d, SUC_lexicon_tags_14 },
    { 0xc51aa61f, SUC_lexicon_tags_68 },
    { 0x6baa620, SUC_lexicon_tags_27 },
    { 0x1a7aa621, SUC_lexicon_tags_12 },
    { 0x169aa621, SUC_lexicon_tags_31 },
    { 0x599aa623, SUC_lexicon_tags_17 },
    { 0xf61aa620, SUC_lexicon_tags_7 },
    { 0x3aa625, SUC_lexicon_tags_15 },
    { 0xd1faa625, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665aa62a, SUC_lexicon_tags_17 },
    { 0x191aa62b, SUC_lexicon_tags_29 },
    { 0xd73aa62c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9baa62e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6f7aa630, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x587aa633, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9e5aa635, SUC_lexicon_tags_2 },
    { 0x6e7aa635, SUC_lexicon_tags_37 },
    { 0xc39aa637, SUC_lexicon_tags_2 },
    { 0x41daa638, SUC_lexicon_tags_33 },
    { 0x971aa635, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2daa63c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e7aa63e, SUC_lexicon_tags_79 },
    { 0xb67aa63f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xebfaa641, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82daa645, SUC_lexicon_tags_27 },
    { 0xe43aa646, SUC_lexicon_tags_15 },
    { 0x979aa647, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729aa651, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9aa654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43faa659, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b9aa65b, SUC_lexicon_tags_19 },
    { 0xdc5aa65b, SUC_lexicon_tags_47 },
    { 0xe07aa65d, SUC_lexicon_tags_38 },
    { 0x4f5aa65b, SUC_lexicon_tags_47 },
    { 0xc7daa65f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67aa663, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe17aa665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf3aa667, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76daa66c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x459aa66e, SUC_lexicon_tags_2 },
    { 0x5e7aa66f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87daa676, SUC_lexicon_tags_9 },
    { 0x64baa677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87daa67d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957aa684, SUC_lexicon_tags_0 },
    { 0x753aa684, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x58daa687, SUC_lexicon_tags_55 },
    { 0xc55aa688, SUC_lexicon_tags_23 },
    { 0x4c1aa689, SUC_lexicon_tags_2 },
    { 0x45baa68a, SUC_lexicon_tags_60 },
    { 0x7fdaa68b, SUC_lexicon_tags_30 },
    { 0x5f7aa68c, SUC_lexicon_tags_21 },
    { 0x4cbaa68a, SUC_lexicon_tags_17 },
    { 0x865aa68b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fbaa690, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1aa69b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf09aa69d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5afaa69f, SUC_lexicon_tags_1 },
    { 0x771aa6a0, SUC_lexicon_tags_79 },
    { 0x3c5aa6a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffaa6a4, SUC_lexicon_tags_15 },
    { 0x63faa6a5, SUC_lexicon_tags_7 },
    { 0xef3aa6a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff3aa6a8, SUC_lexicon_tags_6 },
    { 0xaa1aa6a9, SUC_lexicon_tags_1 },
    { 0x613aa6aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff7aa6ac, SUC_lexicon_tags_0 },
    { 0x5ffaa6ac, SUC_lexicon_tags_2 },
    { 0xb3baa6ae, SUC_lexicon_tags_47 },
    { 0xc7baa6af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd33aa6b1, SUC_lexicon_tags_25 },
    { 0x1ffaa6b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f1aa6b4, SUC_lexicon_tags_17 },
    { 0x713aa6b4, SUC_lexicon_tags_0 },
    { 0x66daa6b5, SUC_lexicon_tags_16 },
    { 0x9faa6b7, SUC_lexicon_tags_43 },
    { 0xf17aa6b8, SUC_lexicon_tags_47 },
    { 0xe93aa6b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6faa6bc, SUC_lexicon_tags_26 },
    { 0xf0baa6bd, SUC_lexicon_tags_37 },
    { 0x957aa6be, SUC_lexicon_tags_12 },
    { 0x895aa6bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3aa6c2, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607aa6c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x37aa6cb, SUC_lexicon_tags_57 },
    { 0x1b3aa6cc, SUC_lexicon_tags_19 },
    { 0xcbfaa6cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfabaa6d0, SUC_lexicon_tags_12 },
    { 0xde3aa6d1, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1aa6d4, SUC_lexicon_tags_47 },
    { 0xa97aa6d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83aa6d8, SUC_lexicon_tags_23 },
    { 0xbaa6d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09aa6dc, SUC_lexicon_tags_14 },
    { 0xdb9aa6dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x743aa6e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5aa6e6, SUC_lexicon_tags_33 },
    { 0xf19aa6e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adaa6ea, SUC_lexicon_tags_8 },
    { 0xf17aa6ea, SUC_lexicon_tags_7 },
    { 0xbc3aa6ec, SUC_lexicon_tags_25 },
    { 0x459aa6ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x721aa6f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1aa6f5, SUC_lexicon_tags_47 },
    { 0xc03aa6f6, SUC_lexicon_tags_55 },
    { 0xcddaa6f7, SUC_lexicon_tags_1 },
    { 0x1cdaa6f8, SUC_lexicon_tags_19 },
    { 0x997aa6f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903aa702, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5daa706, SUC_lexicon_tags_1 },
    { 0xe9faa707, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efaa70b, SUC_lexicon_tags_30 },
    { 0xe25aa70b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455aa710, SUC_lexicon_tags_37 },
    { 0xc95aa710, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72daa719, SUC_lexicon_tags_26 },
    { 0x999aa71a, SUC_lexicon_tags_84 },
    { 0xc2baa71a, SUC_lexicon_tags_11 },
    { 0x9f5aa71c, SUC_lexicon_tags_27 },
    { 0x1baa71c, SUC_lexicon_tags_55 },
    { 0x1b3aa71e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbaa724, SUC_lexicon_tags_7 },
    { 0xb31aa725, SUC_lexicon_tags_17 },
    { 0xe2baa725, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1aa72b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdd5aa72d, SUC_lexicon_tags_16 },
    { 0x41faa72e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1aa737, SUC_lexicon_tags_1 },
    { 0xf9aa737, SUC_lexicon_tags_59 },
    { 0x325aa738, SUC_lexicon_tags_11 },
    { 0x7b1aa738, SUC_lexicon_tags_14 },
    { 0x7d1aa738, SUC_lexicon_tags_17 },
    { 0x485aa73c, SUC_lexicon_tags_63 },
    { 0x9ddaa738, SUC_lexicon_tags_1 },
    { 0xafaa73e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x161aa740, SUC_lexicon_tags_14 },
    { 0x64daa741, SUC_lexicon_tags_7 },
    { 0x3fdaa741, SUC_lexicon_tags_7 },
    { 0x3bbaa741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe99aa745, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1a3aa747, SUC_lexicon_tags_46 },
    { 0xfa7aa748, SUC_lexicon_tags_38 },
    { 0xa77aa747, SUC_lexicon_tags_7 },
    { 0xebaa747, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaefaa74c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825aa750, SUC_lexicon_tags_12 },
    { 0xe5daa750, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91faa754, SUC_lexicon_tags_1 },
    { 0x50faa755, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x83daa757, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfe9aa759, SUC_lexicon_tags_54 },
    { 0xc59aa75a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dfaa75c, SUC_lexicon_tags_43 },
    { 0xdfaa75d, SUC_lexicon_tags_0 },
    { 0xfb7aa75c, SUC_lexicon_tags_15 },
    { 0x7a1aa75f, SUC_lexicon_tags_43 },
    { 0xf97aa75e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62faa765, SUC_lexicon_tags_29 },
    { 0xb1faa765, SUC_lexicon_tags_2 },
    { 0xb4faa766, SUC_lexicon_tags_33 },
    { 0x3d9aa768, SUC_lexicon_tags_114 },
    { 0x5e9aa768, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x245aa76c, SUC_lexicon_tags_25 },
    { 0x3c1aa76d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53aa773, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe1daa775, SUC_lexicon_tags_32 },
    { 0x23baa776, SUC_lexicon_tags_46 },
    { 0xa5faa777, SUC_lexicon_tags_16 },
    { 0x9faa778, SUC_lexicon_tags_56 },
    { 0x6bfaa779, SUC_lexicon_tags_14 },
    { 0x46baa776, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9baa77d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeafaa77f, SUC_lexicon_tags_14 },
    { 0xa79aa780, SUC_lexicon_tags_17 },
    { 0xa5baa781, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809aa785, SUC_lexicon_tags_38 },
    { 0x3e9aa786, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5b7aa788, SUC_lexicon_tags_9 },
    { 0x107aa788, SUC_lexicon_tags_37 },
    { 0x235aa78a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9aa78d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x59aa790, SUC_lexicon_tags_25 },
    { 0x149aa791, SUC_lexicon_tags_91 },
    { 0x11daa792, SUC_lexicon_tags_49 },
    { 0xd35aa793, SUC_lexicon_tags_55 },
    { 0x3f1aa794, SUC_lexicon_tags_92 },
    { 0xd6faa791, SUC_lexicon_tags_1 },
    { 0x74baa793, SUC_lexicon_tags_62 },
    { 0xbd9aa797, SUC_lexicon_tags_25 },
    { 0x869aa792, SUC_lexicon_tags_23 },
    { 0xe75aa797, SUC_lexicon_tags_15 },
    { 0xef7aa794, SUC_lexicon_tags_7 },
    { 0x1bbaa79b, SUC_lexicon_tags_25 },
    { 0x867aa79c, SUC_lexicon_tags_11 },
    { 0x95faa79d, SUC_lexicon_tags_43 },
    { 0xd05aa796, SUC_lexicon_tags_11 },
    { 0x389aa790, SUC_lexicon_tags_26 },
    { 0xe65aa7a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x447aa7a3, SUC_lexicon_tags_45 },
    { 0x8bbaa7a4, SUC_lexicon_tags_64 },
    { 0x4a7aa7a3, SUC_lexicon_tags_2 },
    { 0x2c3aa7a6, SUC_lexicon_tags_37 },
    { 0xed9aa7a6, SUC_lexicon_tags_17 },
    { 0xd93aa7a8, SUC_lexicon_tags_49 },
    { 0xbf3aa7a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdaa7ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd6baa7ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61daa7b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaa7b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15aa7b6, SUC_lexicon_tags_56 },
    { 0x99baa7b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcabaa7b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1aa7bc, SUC_lexicon_tags_26 },
    { 0xa71aa7bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1aa7c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205aa7c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbaa7ca, SUC_lexicon_tags_53 },
    { 0x17aa7ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa29aa7cd, SUC_lexicon_tags_31 },
    { 0x387aa7ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1aa7d5, SUC_lexicon_tags_27 },
    { 0x621aa7d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639aa7db, SUC_lexicon_tags_25 },
    { 0x253aa7dc, SUC_lexicon_tags_25 },
    { 0x815aa7dd, SUC_lexicon_tags_2 },
    { 0xc83aa7de, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xab9aa7e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2afaa7e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41faa7e6, SUC_lexicon_tags_4 },
    { 0x225aa7e7, SUC_lexicon_tags_26 },
    { 0xe57aa7e8, SUC_lexicon_tags_38 },
    { 0x757aa7e6, SUC_lexicon_tags_25 },
    { 0x7ffaa7e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9aa7ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf3aa7f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x44baa7f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1faa7f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30daa7fd, SUC_lexicon_tags_37 },
    { 0xf95aa7fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa67aa800, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53daa807, SUC_lexicon_tags_46 },
    { 0x2d7aa807, SUC_lexicon_tags_15 },
    { 0xfe1aa807, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe79aa80b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x691aa80e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03aa811, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33aa814, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483aa819, SUC_lexicon_tags_25 },
    { 0xbb7aa81a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x329aa81c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3aa821, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891aa825, SUC_lexicon_tags_27 },
    { 0x4cdaa825, SUC_lexicon_tags_48 },
    { 0x7f1aa827, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79aa82e, SUC_lexicon_tags_2 },
    { 0xadaa82f, SUC_lexicon_tags_45 },
    { 0x58daa830, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfaa833, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39aa83a, SUC_lexicon_tags_38 },
    { 0xf97aa83b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5aa840, SUC_lexicon_tags_38 },
    { 0x5f7aa841, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1aa844, SUC_lexicon_tags_2 },
    { 0x62baa844, SUC_lexicon_tags_11 },
    { 0x29baa846, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157aa84d, SUC_lexicon_tags_26 },
    { 0x39aa84e, SUC_lexicon_tags_37 },
    { 0x70daa84f, SUC_lexicon_tags_87 },
    { 0xe15aa850, SUC_lexicon_tags_47 },
    { 0x6baa850, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6faa859, SUC_lexicon_tags_17 },
    { 0x8ddaa85a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4daa861, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4cfaa863, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x645aa865, SUC_lexicon_tags_12 },
    { 0xa01aa865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8faa86d, SUC_lexicon_tags_1 },
    { 0xae9aa86e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x94baa870, SUC_lexicon_tags_33 },
    { 0xd1faa871, SUC_lexicon_tags_12 },
    { 0x7d7aa872, SUC_lexicon_tags_43 },
    { 0x407aa870, SUC_lexicon_tags_58 },
    { 0x359aa871, SUC_lexicon_tags_15 },
    { 0xe41aa870, SUC_lexicon_tags_23 },
    { 0xebdaa876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab5aa878, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x427aa87a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x461aa87d, SUC_lexicon_tags_26 },
    { 0xc89aa87e, SUC_lexicon_tags_2 },
    { 0x8f7aa87e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xddfaa881, SUC_lexicon_tags_37 },
    { 0xaffaa882, SUC_lexicon_tags_23 },
    { 0x8cdaa882, SUC_lexicon_tags_108 },
    { 0xc01aa883, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243aa888, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1aa88d, SUC_lexicon_tags_17 },
    { 0x439aa88e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95aa892, SUC_lexicon_tags_7 },
    { 0x853aa892, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621aa897, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62faa89c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89aa8a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88daa8ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8f1aa8b0, SUC_lexicon_tags_11 },
    { 0xa61aa8b1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7aa8b7, SUC_lexicon_tags_11 },
    { 0xc83aa8b8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe97aa8ba, SUC_lexicon_tags_11 },
    { 0xc43aa8ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf1aa8bd, SUC_lexicon_tags_7 },
    { 0x375aa8be, SUC_lexicon_tags_7 },
    { 0xde3aa8be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43aa8c4, SUC_lexicon_tags_37 },
    { 0x629aa8c4, SUC_lexicon_tags_12 },
    { 0xecdaa8c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efaa8c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1aa8cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x299aa8ce, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317aa8d4, SUC_lexicon_tags_39 },
    { 0x131aa8d5, SUC_lexicon_tags_25 },
    { 0xde1aa8d4, SUC_lexicon_tags_14 },
    { 0xc15aa8d4, SUC_lexicon_tags_47 },
    { 0xe05aa8d8, SUC_lexicon_tags_37 },
    { 0x4ddaa8d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb2daa8db, SUC_lexicon_tags_12 },
    { 0xc5faa8db, SUC_lexicon_tags_25 },
    { 0x33baa8db, SUC_lexicon_tags_12 },
    { 0xb11aa8de, SUC_lexicon_tags_15 },
    { 0xd5baa8df, SUC_lexicon_tags_17 },
    { 0x88faa8df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41aa8e3, SUC_lexicon_tags_23 },
    { 0x899aa8e3, SUC_lexicon_tags_14 },
    { 0x6b5aa8e5, SUC_lexicon_tags_37 },
    { 0x2f1aa8e6, SUC_lexicon_tags_1 },
    { 0x17daa8e5, SUC_lexicon_tags_0 },
    { 0x2cdaa8e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5aa8eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37faa8ee, SUC_lexicon_tags_25 },
    { 0x903aa8ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a5aa8f1, SUC_lexicon_tags_38 },
    { 0xd1aa8f2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x685aa8f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975aa8f9, SUC_lexicon_tags_17 },
    { 0x595aa8f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429aa900, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3aa904, SUC_lexicon_tags_71 },
    { 0x3a7aa905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907aa90a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x767aa90d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x19baa910, SUC_lexicon_tags_60 },
    { 0xbbaa911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aa916, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1aa91b, SUC_lexicon_tags_1 },
    { 0x333aa91c, SUC_lexicon_tags_17 },
    { 0x8c5aa91d, SUC_lexicon_tags_6 },
    { 0xdfbaa91e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4baa920, SUC_lexicon_tags_25 },
    { 0x319aa921, SUC_lexicon_tags_4 },
    { 0x8a7aa921, SUC_lexicon_tags_19 },
    { 0x97daa922, SUC_lexicon_tags_0 },
    { 0x6e5aa923, SUC_lexicon_tags_25 },
    { 0xee9aa921, SUC_lexicon_tags_85 },
    { 0xedaa921, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0x96baa928, SUC_lexicon_tags_12 },
    { 0xa77aa929, SUC_lexicon_tags_37 },
    { 0xc6daa92a, SUC_lexicon_tags_46 },
    { 0x8cfaa92b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x54faa92d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x89aa92f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937aa934, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617aa938, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92baa942, SUC_lexicon_tags_25 },
    { 0xa77aa942, SUC_lexicon_tags_27 },
    { 0xa1faa944, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617aa949, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb19aa94b, SUC_lexicon_tags_0 },
    { 0x7d7aa94c, SUC_lexicon_tags_65 },
    { 0x48baa94b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x915aa950, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725aa954, SUC_lexicon_tags_45 },
    { 0x52daa954, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1aa959, SUC_lexicon_tags_75 },
    { 0x49baa959, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69aa95d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x59faa960, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1aa965, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43aa969, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51aa970, SUC_lexicon_tags_33 },
    { 0x843aa971, SUC_lexicon_tags_46 },
    { 0x3ddaa972, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x37baa974, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54baa978, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7aa97b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257aa97f, SUC_lexicon_tags_2 },
    { 0xb87aa97f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4c9aa982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95aa984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1aa988, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3aa98c, SUC_lexicon_tags_2 },
    { 0xc5daa98d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbaa991, SUC_lexicon_tags_17 },
    { 0x23aa992, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa1aa994, SUC_lexicon_tags_23 },
    { 0x6f3aa995, SUC_lexicon_tags_12 },
    { 0xb03aa996, SUC_lexicon_tags_12 },
    { 0x86daa997, SUC_lexicon_tags_15 },
    { 0x94faa998, SUC_lexicon_tags_14 },
    { 0xf29aa996, SUC_lexicon_tags_15 },
    { 0x4e3aa99a, SUC_lexicon_tags_45 },
    { 0x5dbaa996, SUC_lexicon_tags_11 },
    { 0x2d1aa99c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9faa9a0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807aa9a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1daa9a9, SUC_lexicon_tags_7 },
    { 0x767aa9aa, SUC_lexicon_tags_17 },
    { 0xa8baa9ab, SUC_lexicon_tags_17 },
    { 0xaadaa9ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6abaa9af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0faa9b3, SUC_lexicon_tags_160 },
    { 0x337aa9b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x365aa9b7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe73aa9b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf5aa9bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc63aa9bd, SUC_lexicon_tags_11 },
    { 0xd75aa9bd, SUC_lexicon_tags_46 },
    { 0x3adaa9bf, SUC_lexicon_tags_17 },
    { 0x69faa9bd, SUC_lexicon_tags_25 },
    { 0x5e9aa9bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa77aa9c3, SUC_lexicon_tags_27 },
    { 0xc01aa9c4, SUC_lexicon_tags_49 },
    { 0x4cfaa9c5, SUC_lexicon_tags_14 },
    { 0x77faa9c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x443aa9c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbaa9cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd51aa9ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9aa9d1, SUC_lexicon_tags_185 },
    { 0xb95aa9d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31daa9d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x123aa9d7, SUC_lexicon_tags_26 },
    { 0x115aa9d7, SUC_lexicon_tags_0 },
    { 0x835aa9d7, SUC_lexicon_tags_14 },
    { 0xa15aa9d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x391aa9dc, SUC_lexicon_tags_15 },
    { 0x23aa9dd, SUC_lexicon_tags_61 },
    { 0xb9daa9de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64faa9e4, SUC_lexicon_tags_48 },
    { 0x521aa9e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67faa9e9, SUC_lexicon_tags_9 },
    { 0x503aa9ea, SUC_lexicon_tags_29 },
    { 0x9fbaa9eb, SUC_lexicon_tags_11 },
    { 0xab5aa9eb, SUC_lexicon_tags_28 },
    { 0x27faa9ed, SUC_lexicon_tags_11 },
    { 0x169aa9ed, SUC_lexicon_tags_2 },
    { 0x50faa9ef, SUC_lexicon_tags_2 },
    { 0x343aa9ee, SUC_lexicon_tags_37 },
    { 0x539aa9f1, SUC_lexicon_tags_19 },
    { 0xdc3aa9f0, SUC_lexicon_tags_15 },
    { 0xf7baa9f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afaa9fa, SUC_lexicon_tags_41 },
    { 0x999aa9fa, SUC_lexicon_tags_9 },
    { 0xbf5aa9fa, SUC_lexicon_tags_8 },
    { 0x1cbaa9fd, SUC_lexicon_tags_25 },
    { 0x67aa9fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769aaa06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x503aaa09, SUC_lexicon_tags_17 },
    { 0x66daaa0a, SUC_lexicon_tags_17 },
    { 0x4f1aaa0a, SUC_lexicon_tags_11 },
    { 0xe57aaa0c, SUC_lexicon_tags_27 },
    { 0x863aaa0c, SUC_lexicon_tags_38 },
    { 0x8edaaa0b, SUC_lexicon_tags_79 },
    { 0x6e5aaa0f, SUC_lexicon_tags_37 },
    { 0x373aaa10, SUC_lexicon_tags_15 },
    { 0x4f1aaa0a, SUC_lexicon_tags_165 },
    { 0x197aaa12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61daaa17, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x14faaa19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421aaa20, SUC_lexicon_tags_63 },
    { 0x5bdaaa21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x135aaa23, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1baaa28, SUC_lexicon_tags_25 },
    { 0x475aaa28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3daaa32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x517aaa34, SUC_lexicon_tags_7 },
    { 0xa2faaa34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94faaa37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25faaa43, SUC_lexicon_tags_39 },
    { 0xc17aaa43, SUC_lexicon_tags_34 },
    { 0x763aaa43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb53aaa48, SUC_lexicon_tags_25 },
    { 0x60faaa49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7aaa4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5daaa50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57aaa57, SUC_lexicon_tags_79 },
    { 0x2bdaaa58, SUC_lexicon_tags_1 },
    { 0x7f3aaa57, SUC_lexicon_tags_14 },
    { 0x843aaa5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7aaa5e, SUC_lexicon_tags_8 },
    { 0x249aaa5f, SUC_lexicon_tags_23 },
    { 0xe89aaa60, SUC_lexicon_tags_30 },
    { 0x547aaa61, SUC_lexicon_tags_43 },
    { 0x64faaa62, SUC_lexicon_tags_45 },
    { 0x58faaa5f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895aaa69, SUC_lexicon_tags_47 },
    { 0x35baaa69, SUC_lexicon_tags_47 },
    { 0xba3aaa6b, SUC_lexicon_tags_25 },
    { 0x28baaa6a, SUC_lexicon_tags_17 },
    { 0xeb7aaa6d, SUC_lexicon_tags_12 },
    { 0x1f3aaa6a, SUC_lexicon_tags_173 },
    { 0, NULL },
    { 0x8d3aaa70, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x267aaa72, SUC_lexicon_tags_1 },
    { 0x367aaa73, SUC_lexicon_tags_1 },
    { 0xfbfaaa73, SUC_lexicon_tags_37 },
    { 0x321aaa74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe8baaa77, SUC_lexicon_tags_37 },
    { 0x311aaa78, SUC_lexicon_tags_12 },
    { 0x455aaa78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfaaa7e, SUC_lexicon_tags_17 },
    { 0xca9aaa7e, SUC_lexicon_tags_27 },
    { 0x735aaa7f, SUC_lexicon_tags_2 },
    { 0x86faaa80, SUC_lexicon_tags_33 },
    { 0xd7faaa80, SUC_lexicon_tags_106 },
    { 0xebbaaa7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea3aaa85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x423aaa87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17faaa8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15aaa91, SUC_lexicon_tags_65 },
    { 0xd8daaa92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adaaa95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1daaa9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad9aaa9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3aaaa1, SUC_lexicon_tags_31 },
    { 0xc41aaaa2, SUC_lexicon_tags_65 },
    { 0x6a1aaaa3, SUC_lexicon_tags_25 },
    { 0xcefaaaa4, SUC_lexicon_tags_485 },
    { 0x1faaaa5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x289aaaa8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15aaaab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75baaaad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56faaab2, SUC_lexicon_tags_6 },
    { 0x4c9aaab3, SUC_lexicon_tags_13 },
    { 0x4cdaaab4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x361aaab6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505aaabb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffaaac5, SUC_lexicon_tags_0 },
    { 0x9f3aaac6, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0x893aaac9, SUC_lexicon_tags_14 },
    { 0x4d7aaaca, SUC_lexicon_tags_17 },
    { 0x223aaacb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113aaad0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58daaad2, SUC_lexicon_tags_55 },
    { 0x3b5aaad3, SUC_lexicon_tags_38 },
    { 0xb0faaad2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x13aaad7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x99faaad9, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x13daaadb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35faaadf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x193aaae1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x957aaae3, SUC_lexicon_tags_1 },
    { 0xcb9aaae4, SUC_lexicon_tags_19 },
    { 0xf19aaae3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaddaaae7, SUC_lexicon_tags_60 },
    { 0x9a1aaae8, SUC_lexicon_tags_9 },
    { 0xb55aaae9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66faaaec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc9baaaee, SUC_lexicon_tags_4 },
    { 0x749aaaef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd75aaaf1, SUC_lexicon_tags_14 },
    { 0x1c3aaaf2, SUC_lexicon_tags_11 },
    { 0x437aaaf3, SUC_lexicon_tags_38 },
    { 0x79faaaf2, SUC_lexicon_tags_14 },
    { 0xbedaaaf5, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7daaafa, SUC_lexicon_tags_13 },
    { 0xed3aaafb, SUC_lexicon_tags_30 },
    { 0xdffaaafc, SUC_lexicon_tags_7 },
    { 0xe09aaafc, SUC_lexicon_tags_8 },
    { 0xd43aaafe, SUC_lexicon_tags_46 },
    { 0x9c5aaaff, SUC_lexicon_tags_12 },
    { 0x473aab00, SUC_lexicon_tags_11 },
    { 0xa59aaafe, SUC_lexicon_tags_66 },
    { 0x7f3aaaff, SUC_lexicon_tags_1086 },
    { 0x957aaafd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8baab05, SUC_lexicon_tags_25 },
    { 0x149aab06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29aab10, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x727aab13, SUC_lexicon_tags_37 },
    { 0x5d7aab14, SUC_lexicon_tags_15 },
    { 0xf3aab13, SUC_lexicon_tags_7 },
    { 0xbafaab15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xadbaab18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xecfaab1a, SUC_lexicon_tags_19 },
    { 0x2fdaab1a, SUC_lexicon_tags_17 },
    { 0x27baab1c, SUC_lexicon_tags_106 },
    { 0xeafaab1d, SUC_lexicon_tags_11 },
    { 0xee5aab1e, SUC_lexicon_tags_30 },
    { 0x389aab1e, SUC_lexicon_tags_92 },
    { 0x50baab20, SUC_lexicon_tags_25 },
    { 0xc23aab20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abaab24, SUC_lexicon_tags_14 },
    { 0x9a3aab25, SUC_lexicon_tags_38 },
    { 0x4ffaab26, SUC_lexicon_tags_1 },
    { 0xc9faab26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaab2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe79aab2e, SUC_lexicon_tags_15 },
    { 0x449aab2e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdaab33, SUC_lexicon_tags_11 },
    { 0xc73aab34, SUC_lexicon_tags_25 },
    { 0xedaab34, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4cdaab37, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfaab3b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1aab3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc67aab41, SUC_lexicon_tags_63 },
    { 0x357aab42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21aab45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x341aab48, SUC_lexicon_tags_39 },
    { 0xe45aab49, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5aab56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5faab5a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe69aab5c, SUC_lexicon_tags_594 },
    { 0xa29aab5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103aab64, SUC_lexicon_tags_37 },
    { 0x589aab65, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x313aab68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9daab6b, SUC_lexicon_tags_37 },
    { 0x6a5aab6b, SUC_lexicon_tags_14 },
    { 0x17daab6d, SUC_lexicon_tags_81 },
    { 0x6e5aab6e, SUC_lexicon_tags_61 },
    { 0xb8daab6f, SUC_lexicon_tags_15 },
    { 0xd49aab70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97faab77, SUC_lexicon_tags_79 },
    { 0x38daab77, SUC_lexicon_tags_27 },
    { 0xe7daab79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x75daab7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5aab7d, SUC_lexicon_tags_19 },
    { 0x3f1aab7d, SUC_lexicon_tags_12 },
    { 0xa79aab7f, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adaab83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32daab87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8baab8b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9aab9b, SUC_lexicon_tags_37 },
    { 0xde3aab9b, SUC_lexicon_tags_62 },
    { 0x4b3aab9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf43aab9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5d5aaba1, SUC_lexicon_tags_2 },
    { 0x397aaba2, SUC_lexicon_tags_17 },
    { 0xbd5aaba3, SUC_lexicon_tags_60 },
    { 0x799aaba3, SUC_lexicon_tags_11 },
    { 0x473aaba4, SUC_lexicon_tags_7 },
    { 0x18faaba6, SUC_lexicon_tags_1 },
    { 0xb03aaba7, SUC_lexicon_tags_15 },
    { 0x94daaba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45faabaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfaabae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43aabb2, SUC_lexicon_tags_8 },
    { 0xa65aabb3, SUC_lexicon_tags_47 },
    { 0xbfdaabb2, SUC_lexicon_tags_7 },
    { 0x741aabb5, SUC_lexicon_tags_46 },
    { 0x961aabb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6faabb8, SUC_lexicon_tags_33 },
    { 0xbe7aabb9, SUC_lexicon_tags_14 },
    { 0x6f7aabba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39aabbf, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xd01aabc1, SUC_lexicon_tags_0 },
    { 0x70baabc2, SUC_lexicon_tags_13 },
    { 0xb25aabc3, SUC_lexicon_tags_17 },
    { 0xe1daabc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5aabc9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5bbaabcb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345aabd1, SUC_lexicon_tags_17 },
    { 0xf87aabd2, SUC_lexicon_tags_396 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59aabd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x773aabd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69aabdc, SUC_lexicon_tags_17 },
    { 0x8c9aabdc, SUC_lexicon_tags_28 },
    { 0x351aabde, SUC_lexicon_tags_2 },
    { 0xd25aabdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1daabe1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbdaabe3, SUC_lexicon_tags_7 },
    { 0xab7aabe3, SUC_lexicon_tags_230 },
    { 0x3d5aabe4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc57aabe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x985aabe9, SUC_lexicon_tags_10 },
    { 0xf6faabea, SUC_lexicon_tags_17 },
    { 0xb6faabe9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc9aabed, SUC_lexicon_tags_63 },
    { 0xd91aabed, SUC_lexicon_tags_66 },
    { 0x91baabed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269aabf4, SUC_lexicon_tags_25 },
    { 0x3bdaabf5, SUC_lexicon_tags_17 },
    { 0xc89aabf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x329aabf9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x751aabfb, SUC_lexicon_tags_15 },
    { 0x843aabfc, SUC_lexicon_tags_33 },
    { 0x837aabfc, SUC_lexicon_tags_11 },
    { 0x339aabfe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93faac00, SUC_lexicon_tags_49 },
    { 0xa77aac00, SUC_lexicon_tags_2 },
    { 0x52baac01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899aac09, SUC_lexicon_tags_2 },
    { 0xe3aac0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xde7aac0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5aac11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65aac15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80daac1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503aac20, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa7baac22, SUC_lexicon_tags_11 },
    { 0x973aac23, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefaac2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1aac2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddaac32, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x479aac35, SUC_lexicon_tags_37 },
    { 0x3e5aac36, SUC_lexicon_tags_25 },
    { 0xb4baac37, SUC_lexicon_tags_37 },
    { 0xb37aac37, SUC_lexicon_tags_28 },
    { 0xe4faac35, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819aac3d, SUC_lexicon_tags_14 },
    { 0x3d1aac3d, SUC_lexicon_tags_49 },
    { 0xf55aac3f, SUC_lexicon_tags_56 },
    { 0xfbaac40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35aac44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731aac48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe35aac4a, SUC_lexicon_tags_2 },
    { 0x6b9aac4b, SUC_lexicon_tags_1 },
    { 0xaa3aac4b, SUC_lexicon_tags_3 },
    { 0xa25aac4d, SUC_lexicon_tags_28 },
    { 0x483aac4e, SUC_lexicon_tags_15 },
    { 0x151aac4b, SUC_lexicon_tags_15 },
    { 0xb8daac50, SUC_lexicon_tags_11 },
    { 0x72faac4c, SUC_lexicon_tags_12 },
    { 0xc8baac52, SUC_lexicon_tags_25 },
    { 0x617aac52, SUC_lexicon_tags_11 },
    { 0xf95aac54, SUC_lexicon_tags_113 },
    { 0xfa7aac54, SUC_lexicon_tags_15 },
    { 0x2a3aac56, SUC_lexicon_tags_60 },
    { 0x681aac56, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x58daac59, SUC_lexicon_tags_11 },
    { 0x641aac5a, SUC_lexicon_tags_14 },
    { 0xa41aac59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea7aac5d, SUC_lexicon_tags_45 },
    { 0xf3faac5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6daac61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x541aac64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x961aac66, SUC_lexicon_tags_182 },
    { 0xaedaac66, SUC_lexicon_tags_63 },
    { 0xd33aac68, SUC_lexicon_tags_33 },
    { 0x63faac66, SUC_lexicon_tags_17 },
    { 0xbe5aac6a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x427aac6d, SUC_lexicon_tags_25 },
    { 0x4ddaac6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4faac71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe25aac73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3aac7a, SUC_lexicon_tags_27 },
    { 0x431aac7b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1aac83, SUC_lexicon_tags_37 },
    { 0x571aac84, SUC_lexicon_tags_11 },
    { 0xa81aac85, SUC_lexicon_tags_11 },
    { 0x883aac86, SUC_lexicon_tags_33 },
    { 0x595aac87, SUC_lexicon_tags_37 },
    { 0xd41aac87, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43faac8a, SUC_lexicon_tags_17 },
    { 0xb9faac8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25daac92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5baac94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73daac96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1aac99, SUC_lexicon_tags_15 },
    { 0x673aac99, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb01aac9c, SUC_lexicon_tags_12 },
    { 0x60faac9d, SUC_lexicon_tags_63 },
    { 0xa8daac9e, SUC_lexicon_tags_37 },
    { 0xeebaac9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b5aaca1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13aaca4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3baacaa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf1faacac, SUC_lexicon_tags_39 },
    { 0xc67aacad, SUC_lexicon_tags_65 },
    { 0x2cdaacae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61aacb3, SUC_lexicon_tags_49 },
    { 0x369aacb4, SUC_lexicon_tags_28 },
    { 0x339aacb3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3aacbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87aacbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41daacc4, SUC_lexicon_tags_19 },
    { 0x2d1aacc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x949aacc8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2cbaacca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1aaccf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdaacd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c5aacd5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5daacd8, SUC_lexicon_tags_50 },
    { 0x59faacd9, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9aacdd, SUC_lexicon_tags_15 },
    { 0x363aacdd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3daace0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x223aace3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3daace6, SUC_lexicon_tags_12 },
    { 0x76baace7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x699aacea, SUC_lexicon_tags_8 },
    { 0x147aaceb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487aacf0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x261aacf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf03aacf4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99aacff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7aad04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a9aad06, SUC_lexicon_tags_37 },
    { 0xb9faad06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0baad0e, SUC_lexicon_tags_43 },
    { 0x3e3aad0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50baad14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3faad16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbaad1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7aad1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6daad25, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x805aad27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c3aad29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1aad2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aad30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499aad35, SUC_lexicon_tags_12 },
    { 0x911aad36, SUC_lexicon_tags_19 },
    { 0x6cbaad36, SUC_lexicon_tags_33 },
    { 0x289aad38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691aad3f, SUC_lexicon_tags_0 },
    { 0xe1faad40, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x73baad42, SUC_lexicon_tags_45 },
    { 0x5c9aad43, SUC_lexicon_tags_15 },
    { 0x4bbaad44, SUC_lexicon_tags_19 },
    { 0xbd7aad45, SUC_lexicon_tags_15 },
    { 0x981aad46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289aad4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63faad4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x387aad51, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5aad59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39aad5c, SUC_lexicon_tags_37 },
    { 0x6e7aad5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc0baad5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfaad66, SUC_lexicon_tags_1 },
    { 0x45baad67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x965aad69, SUC_lexicon_tags_158 },
    { 0xf57aad69, SUC_lexicon_tags_11 },
    { 0xb8daad69, SUC_lexicon_tags_37 },
    { 0xa87aad6b, SUC_lexicon_tags_37 },
    { 0xe23aad6d, SUC_lexicon_tags_45 },
    { 0xa49aad6d, SUC_lexicon_tags_82 },
    { 0x973aad6f, SUC_lexicon_tags_14 },
    { 0x5c9aad6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7f5aad72, SUC_lexicon_tags_23 },
    { 0x12baad72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37aad76, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97aad79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1aad7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93faad80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf13aad82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1faad87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8daad8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45aad8d, SUC_lexicon_tags_17 },
    { 0xd61aad8e, SUC_lexicon_tags_12 },
    { 0xb79aad8e, SUC_lexicon_tags_17 },
    { 0x7a1aad90, SUC_lexicon_tags_46 },
    { 0x19daad90, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5aad94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1bbaad96, SUC_lexicon_tags_17 },
    { 0x747aad96, SUC_lexicon_tags_37 },
    { 0xd79aad97, SUC_lexicon_tags_37 },
    { 0x43faad97, SUC_lexicon_tags_1 },
    { 0xa5baad9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffaada1, SUC_lexicon_tags_7 },
    { 0x887aada1, SUC_lexicon_tags_104 },
    { 0xba7aada2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xab1aada5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efaada8, SUC_lexicon_tags_37 },
    { 0x833aada9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90baadb0, SUC_lexicon_tags_45 },
    { 0xb0faadb1, SUC_lexicon_tags_1 },
    { 0x157aadb0, SUC_lexicon_tags_15 },
    { 0xbadaadb3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1aadb6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7faadbc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53aadc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ddaadc2, SUC_lexicon_tags_17 },
    { 0xc71aadc2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9aadc6, SUC_lexicon_tags_87 },
    { 0xb6baadc7, SUC_lexicon_tags_11 },
    { 0x4c3aadc7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5c7aadca, SUC_lexicon_tags_2 },
    { 0x8abaadcb, SUC_lexicon_tags_11 },
    { 0x53daadcc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2abaadce, SUC_lexicon_tags_27 },
    { 0x25aadcf, SUC_lexicon_tags_37 },
    { 0x917aadcf, SUC_lexicon_tags_107 },
    { 0xf75aadce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981aadd5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2bdaadd7, SUC_lexicon_tags_7 },
    { 0x9e1aadd8, SUC_lexicon_tags_23 },
    { 0x42faadd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc8faaddb, SUC_lexicon_tags_6 },
    { 0x5e5aaddc, SUC_lexicon_tags_11 },
    { 0xa05aaddd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1aade2, SUC_lexicon_tags_60 },
    { 0x45baade3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xda1aade5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17aadeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29daadf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1aadf3, SUC_lexicon_tags_7 },
    { 0x349aadf3, SUC_lexicon_tags_1 },
    { 0xf57aadf5, SUC_lexicon_tags_63 },
    { 0xdf1aadf4, SUC_lexicon_tags_21 },
    { 0xbe3aadf6, SUC_lexicon_tags_118 },
    { 0xf61aadf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357aadfd, SUC_lexicon_tags_12 },
    { 0xb1baadfd, SUC_lexicon_tags_39 },
    { 0x59daadff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57aae03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7aae0b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x261aae0e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x84faae11, SUC_lexicon_tags_43 },
    { 0xe97aae11, SUC_lexicon_tags_2 },
    { 0x6f5aae13, SUC_lexicon_tags_7 },
    { 0xf29aae14, SUC_lexicon_tags_11 },
    { 0x823aae15, SUC_lexicon_tags_37 },
    { 0x2cdaae16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbaae19, SUC_lexicon_tags_0 },
    { 0xe0faae19, SUC_lexicon_tags_37 },
    { 0xc47aae1b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x959aae1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9aae23, SUC_lexicon_tags_8 },
    { 0x985aae24, SUC_lexicon_tags_65 },
    { 0xe0baae24, SUC_lexicon_tags_26 },
    { 0x171aae26, SUC_lexicon_tags_25 },
    { 0x923aae27, SUC_lexicon_tags_26 },
    { 0xeffaae23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9faae2c, SUC_lexicon_tags_38 },
    { 0x21faae2d, SUC_lexicon_tags_17 },
    { 0x1f5aae2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffaae31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9aae36, SUC_lexicon_tags_17 },
    { 0x827aae37, SUC_lexicon_tags_25 },
    { 0xf51aae38, SUC_lexicon_tags_30 },
    { 0x315aae39, SUC_lexicon_tags_12 },
    { 0xab3aae39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9daae3d, SUC_lexicon_tags_46 },
    { 0xaedaae3e, SUC_lexicon_tags_11 },
    { 0xd89aae3e, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baae43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfaae46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x75aae48, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699aae4e, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x75aae50, SUC_lexicon_tags_46 },
    { 0xc3faae51, SUC_lexicon_tags_45 },
    { 0xdb1aae52, SUC_lexicon_tags_57 },
    { 0xda1aae52, SUC_lexicon_tags_66 },
    { 0x3fdaae54, SUC_lexicon_tags_9 },
    { 0xf31aae55, SUC_lexicon_tags_17 },
    { 0xc5faae52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e3aae58, SUC_lexicon_tags_65 },
    { 0xd35aae59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8cdaae5b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7daae5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1baae61, SUC_lexicon_tags_7 },
    { 0x1dbaae62, SUC_lexicon_tags_33 },
    { 0x627aae61, SUC_lexicon_tags_37 },
    { 0xbbfaae64, SUC_lexicon_tags_11 },
    { 0xecfaae65, SUC_lexicon_tags_17 },
    { 0x7bdaae64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffaae6a, SUC_lexicon_tags_39 },
    { 0x8a1aae6a, SUC_lexicon_tags_0 },
    { 0xebdaae6a, SUC_lexicon_tags_32 },
    { 0xf6faae6c, SUC_lexicon_tags_30 },
    { 0x5bdaae6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x699aae70, SUC_lexicon_tags_11 },
    { 0x433aae71, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83aae74, SUC_lexicon_tags_81 },
    { 0xdf5aae75, SUC_lexicon_tags_17 },
    { 0x851aae76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebfaae78, SUC_lexicon_tags_17 },
    { 0x3e5aae78, SUC_lexicon_tags_47 },
    { 0xf71aae7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93aae85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbaae88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4faae8b, SUC_lexicon_tags_20 },
    { 0x969aae8c, SUC_lexicon_tags_6 },
    { 0xbdaae8c, SUC_lexicon_tags_25 },
    { 0x593aae8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x505aae90, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x7b1aae92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8faae94, SUC_lexicon_tags_37 },
    { 0x9e9aae94, SUC_lexicon_tags_45 },
    { 0x7f7aae96, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcdaae98, SUC_lexicon_tags_17 },
    { 0x67aae99, SUC_lexicon_tags_7 },
    { 0x16baae9a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x2b9aae9c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x485aae9e, SUC_lexicon_tags_15 },
    { 0x67baae9f, SUC_lexicon_tags_29 },
    { 0xe9faaea0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x48faaea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1baaea5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x74daaea8, SUC_lexicon_tags_43 },
    { 0x6f9aaea9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869aaeae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x539aaeb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x547aaeb3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1aaeb8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x165aaeba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65aaebd, SUC_lexicon_tags_11 },
    { 0x92faaebe, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x847aaec1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103aaec6, SUC_lexicon_tags_1 },
    { 0x1f7aaec7, SUC_lexicon_tags_56 },
    { 0x929aaec7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1daaecb, SUC_lexicon_tags_17 },
    { 0x3cbaaecc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedaaed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffaaed4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x321aaed6, SUC_lexicon_tags_15 },
    { 0x477aaed7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf03aaed9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa41aaedb, SUC_lexicon_tags_1 },
    { 0xd7baaedc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edaaedf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14daaee4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x905aaee7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49baaeea, SUC_lexicon_tags_9 },
    { 0x3b7aaeeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x97aaeee, SUC_lexicon_tags_1 },
    { 0x6b7aaeef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc8faaef1, SUC_lexicon_tags_15 },
    { 0xc05aaef2, SUC_lexicon_tags_11 },
    { 0x905aaef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8baaef6, SUC_lexicon_tags_37 },
    { 0xa09aaef6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x339aaef9, SUC_lexicon_tags_23 },
    { 0x731aaefa, SUC_lexicon_tags_30 },
    { 0xebaaefb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x17aaefe, SUC_lexicon_tags_23 },
    { 0x47faaeff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3fbaaf01, SUC_lexicon_tags_25 },
    { 0xeabaaf02, SUC_lexicon_tags_1 },
    { 0x703aaf01, SUC_lexicon_tags_46 },
    { 0x159aaf04, SUC_lexicon_tags_33 },
    { 0x3cfaaf03, SUC_lexicon_tags_68 },
    { 0xd69aaf06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe1aaf08, SUC_lexicon_tags_1 },
    { 0x715aaf08, SUC_lexicon_tags_11 },
    { 0x205aaf0a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3baaf12, SUC_lexicon_tags_54 },
    { 0x249aaf13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82faaf16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8edaaf18, SUC_lexicon_tags_7 },
    { 0x5aaf19, SUC_lexicon_tags_6 },
    { 0xd25aaf18, SUC_lexicon_tags_14 },
    { 0x545aaf19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdaaf1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2faaf23, SUC_lexicon_tags_43 },
    { 0xebaaf24, SUC_lexicon_tags_17 },
    { 0x201aaf23, SUC_lexicon_tags_68 },
    { 0xa3daaf26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x841aaf28, SUC_lexicon_tags_15 },
    { 0x85faaf29, SUC_lexicon_tags_17 },
    { 0x997aaf2a, SUC_lexicon_tags_12 },
    { 0x40faaf2a, SUC_lexicon_tags_11 },
    { 0xd51aaf2b, SUC_lexicon_tags_15 },
    { 0xd2baaf2c, SUC_lexicon_tags_47 },
    { 0xb61aaf2e, SUC_lexicon_tags_17 },
    { 0xb65aaf2e, SUC_lexicon_tags_37 },
    { 0x779aaf30, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725aaf35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37faaf37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcb3aaf39, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa01aaf3b, SUC_lexicon_tags_17 },
    { 0x485aaf3c, SUC_lexicon_tags_64 },
    { 0xef7aaf3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x853aaf3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1aaf42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51aaf4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x599aaf4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abaaf51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469aaf55, SUC_lexicon_tags_17 },
    { 0x4b1aaf56, SUC_lexicon_tags_37 },
    { 0x249aaf56, SUC_lexicon_tags_12 },
    { 0x195aaf56, SUC_lexicon_tags_43 },
    { 0xb8faaf59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3aaf61, SUC_lexicon_tags_25 },
    { 0xc07aaf62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57daaf64, SUC_lexicon_tags_10 },
    { 0x97faaf65, SUC_lexicon_tags_31 },
    { 0x849aaf65, SUC_lexicon_tags_12 },
    { 0xc9aaf67, SUC_lexicon_tags_4 },
    { 0x94baaf66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3aaf6d, SUC_lexicon_tags_25 },
    { 0x9daaf6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9aaf73, SUC_lexicon_tags_17 },
    { 0x6cfaaf74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645aaf7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977aaf82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a3aaf84, SUC_lexicon_tags_39 },
    { 0xe5daaf84, SUC_lexicon_tags_65 },
    { 0xcfaaf84, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3daaf89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fbaaf8b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79aaf8e, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfaaf91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9aaf95, SUC_lexicon_tags_25 },
    { 0xe2daaf96, SUC_lexicon_tags_14 },
    { 0x505aaf95, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2baaf99, SUC_lexicon_tags_7 },
    { 0xcddaaf99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1aaf9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39aafa5, SUC_lexicon_tags_4 },
    { 0xbd9aafa5, SUC_lexicon_tags_37 },
    { 0xaafaafa7, SUC_lexicon_tags_2 },
    { 0xaebaafa6, SUC_lexicon_tags_794 },
    { 0xe41aafa9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc5aafab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa89aafad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x87faafaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419aafb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x143aafb5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x77faafb8, SUC_lexicon_tags_112 },
    { 0xfe1aafb9, SUC_lexicon_tags_19 },
    { 0x17aafba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7aafbf, SUC_lexicon_tags_35 },
    { 0x373aafc0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x41baafc3, SUC_lexicon_tags_49 },
    { 0xdfaafc4, SUC_lexicon_tags_47 },
    { 0x377aafc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe47aafc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c7aafc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb4baafcb, SUC_lexicon_tags_7 },
    { 0xb1baafcb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x96baafce, SUC_lexicon_tags_27 },
    { 0x18daafcf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edaafd6, SUC_lexicon_tags_15 },
    { 0x167aafd6, SUC_lexicon_tags_25 },
    { 0xcc7aafd7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x513aafdb, SUC_lexicon_tags_60 },
    { 0xb1faafdb, SUC_lexicon_tags_17 },
    { 0xd51aafdd, SUC_lexicon_tags_0 },
    { 0xef1aafdd, SUC_lexicon_tags_27 },
    { 0x2c9aafdf, SUC_lexicon_tags_2 },
    { 0x6e7aafe0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8baafe3, SUC_lexicon_tags_47 },
    { 0x229aafe4, SUC_lexicon_tags_19 },
    { 0xbfdaafe5, SUC_lexicon_tags_79 },
    { 0x181aafe6, SUC_lexicon_tags_47 },
    { 0xd0baafe6, SUC_lexicon_tags_49 },
    { 0x94faafe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b9aafea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871aafee, SUC_lexicon_tags_14 },
    { 0xe67aafee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x77faaff1, SUC_lexicon_tags_16 },
    { 0xcb3aaff2, SUC_lexicon_tags_30 },
    { 0x1abaaff3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8baaff6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1aaffa, SUC_lexicon_tags_37 },
    { 0x7f3aaffa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41aaffe, SUC_lexicon_tags_46 },
    { 0x273aaffe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbab003, SUC_lexicon_tags_23 },
    { 0x495ab004, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5ab00d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb55ab00f, SUC_lexicon_tags_202 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdab014, SUC_lexicon_tags_7 },
    { 0xea9ab015, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6bbab017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9ab020, SUC_lexicon_tags_11 },
    { 0x97dab021, SUC_lexicon_tags_30 },
    { 0xac3ab022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6afab024, SUC_lexicon_tags_23 },
    { 0x495ab025, SUC_lexicon_tags_0 },
    { 0xfedab025, SUC_lexicon_tags_21 },
    { 0xde3ab025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5ab02a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ab02d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3ab030, SUC_lexicon_tags_63 },
    { 0xea7ab031, SUC_lexicon_tags_9 },
    { 0x19bab032, SUC_lexicon_tags_41 },
    { 0x529ab033, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5ab038, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe81ab03a, SUC_lexicon_tags_11 },
    { 0xac3ab03a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fab040, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3ab045, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x765ab047, SUC_lexicon_tags_2 },
    { 0x433ab048, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6b7ab04a, SUC_lexicon_tags_14 },
    { 0xc6dab04a, SUC_lexicon_tags_25 },
    { 0xc57ab04b, SUC_lexicon_tags_7 },
    { 0xddbab04d, SUC_lexicon_tags_14 },
    { 0x81bab04b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edab052, SUC_lexicon_tags_481 },
    { 0x66bab052, SUC_lexicon_tags_33 },
    { 0x1c1ab054, SUC_lexicon_tags_61 },
    { 0xc47ab052, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbab059, SUC_lexicon_tags_68 },
    { 0x9cdab05a, SUC_lexicon_tags_29 },
    { 0xd55ab05b, SUC_lexicon_tags_17 },
    { 0x285ab05c, SUC_lexicon_tags_26 },
    { 0x717ab05a, SUC_lexicon_tags_12 },
    { 0x355ab05d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fab063, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdab066, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fab06a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9ab06f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfafab071, SUC_lexicon_tags_198 },
    { 0x6fbab072, SUC_lexicon_tags_26 },
    { 0x5fab073, SUC_lexicon_tags_1 },
    { 0xddbab073, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81ab078, SUC_lexicon_tags_27 },
    { 0x195ab079, SUC_lexicon_tags_1 },
    { 0xfd1ab079, SUC_lexicon_tags_37 },
    { 0x417ab078, SUC_lexicon_tags_68 },
    { 0x49ab07c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7ab083, SUC_lexicon_tags_14 },
    { 0x76bab084, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x357ab086, SUC_lexicon_tags_37 },
    { 0x255ab087, SUC_lexicon_tags_0 },
    { 0xbc9ab088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45ab08f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3ab092, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921ab098, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe07ab09a, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5ab0a0, SUC_lexicon_tags_21 },
    { 0xcbfab0a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1ab0a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x425ab0ab, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x74bab0ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543ab0b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18dab0b5, SUC_lexicon_tags_12 },
    { 0xca7ab0b6, SUC_lexicon_tags_8 },
    { 0x2ddab0b7, SUC_lexicon_tags_31 },
    { 0x523ab0b8, SUC_lexicon_tags_25 },
    { 0x2d5ab0b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb1ab0bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xabab0bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d5ab0bf, SUC_lexicon_tags_80 },
    { 0x6cbab0c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdab0c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc79ab0c5, SUC_lexicon_tags_2 },
    { 0x18bab0c5, SUC_lexicon_tags_15 },
    { 0xe4dab0c6, SUC_lexicon_tags_29 },
    { 0x743ab0c5, SUC_lexicon_tags_37 },
    { 0x17bab0c9, SUC_lexicon_tags_25 },
    { 0xb9dab0c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457ab0d2, SUC_lexicon_tags_29 },
    { 0xf67ab0d2, SUC_lexicon_tags_107 },
    { 0x6d7ab0d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9ab0d8, SUC_lexicon_tags_37 },
    { 0xe01ab0d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fab0de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x659ab0e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95ab0e4, SUC_lexicon_tags_113 },
    { 0x8c7ab0e5, SUC_lexicon_tags_7 },
    { 0x3b7ab0e5, SUC_lexicon_tags_11 },
    { 0x64fab0e5, SUC_lexicon_tags_25 },
    { 0xab5ab0e8, SUC_lexicon_tags_44 },
    { 0xcebab0e9, SUC_lexicon_tags_12 },
    { 0xc6dab0e9, SUC_lexicon_tags_29 },
    { 0x34fab0e6, SUC_lexicon_tags_25 },
    { 0x97ab0eb, SUC_lexicon_tags_11 },
    { 0xbfdab0ed, SUC_lexicon_tags_1 },
    { 0x313ab0eb, SUC_lexicon_tags_25 },
    { 0xdf7ab0ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3ab0f2, SUC_lexicon_tags_9 },
    { 0xaebab0f3, SUC_lexicon_tags_19 },
    { 0x2fbab0f4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31ab103, SUC_lexicon_tags_37 },
    { 0x643ab104, SUC_lexicon_tags_31 },
    { 0x383ab105, SUC_lexicon_tags_12 },
    { 0xc63ab106, SUC_lexicon_tags_11 },
    { 0x839ab106, SUC_lexicon_tags_14 },
    { 0xe61ab108, SUC_lexicon_tags_25 },
    { 0xda5ab109, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461ab10e, SUC_lexicon_tags_17 },
    { 0x8cfab10f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1c9ab111, SUC_lexicon_tags_506 },
    { 0x56dab112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1ab116, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab119, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765ab11d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x20dab11f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd87ab121, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xbdfab123, SUC_lexicon_tags_7 },
    { 0xf3fab124, SUC_lexicon_tags_8 },
    { 0x495ab125, SUC_lexicon_tags_49 },
    { 0xabbab126, SUC_lexicon_tags_14 },
    { 0x609ab127, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673ab12b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x353ab12e, SUC_lexicon_tags_61 },
    { 0xe97ab12f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2bab131, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249ab135, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc85ab137, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x527ab139, SUC_lexicon_tags_13 },
    { 0x785ab13a, SUC_lexicon_tags_17 },
    { 0x731ab13a, SUC_lexicon_tags_8 },
    { 0x797ab13c, SUC_lexicon_tags_17 },
    { 0x617ab13c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427ab141, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbb5ab143, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x267ab146, SUC_lexicon_tags_38 },
    { 0x661ab147, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabab14a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x74bab14c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce3ab14e, SUC_lexicon_tags_7 },
    { 0x5cfab14f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbab156, SUC_lexicon_tags_28 },
    { 0x39ab156, SUC_lexicon_tags_37 },
    { 0x101ab157, SUC_lexicon_tags_63 },
    { 0x809ab158, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x609ab15b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeefab15d, SUC_lexicon_tags_39 },
    { 0x8adab15d, SUC_lexicon_tags_18 },
    { 0xf27ab15f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bab161, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13ab165, SUC_lexicon_tags_10 },
    { 0x27bab165, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37ab169, SUC_lexicon_tags_37 },
    { 0x52dab16a, SUC_lexicon_tags_25 },
    { 0x4b7ab16b, SUC_lexicon_tags_23 },
    { 0x2d3ab16c, SUC_lexicon_tags_13 },
    { 0x85dab16d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfab171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fab176, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x515ab179, SUC_lexicon_tags_23 },
    { 0x9e5ab17a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdab17f, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0xa3ab181, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ab184, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8b3ab186, SUC_lexicon_tags_1 },
    { 0x953ab186, SUC_lexicon_tags_290 },
    { 0, NULL },
    { 0x927ab189, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x557ab18c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x137ab18f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2dfab191, SUC_lexicon_tags_37 },
    { 0x76bab191, SUC_lexicon_tags_6 },
    { 0xb07ab193, SUC_lexicon_tags_62 },
    { 0xaefab191, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59ab19b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675ab19f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x6dab1a1, SUC_lexicon_tags_15 },
    { 0x767ab1a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe21ab1a4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd33ab1a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ab1a9, SUC_lexicon_tags_329 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43ab1ac, SUC_lexicon_tags_37 },
    { 0x74fab1ad, SUC_lexicon_tags_11 },
    { 0x6f1ab1ae, SUC_lexicon_tags_7 },
    { 0x173ab1ad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fab1b4, SUC_lexicon_tags_66 },
    { 0x4c1ab1b4, SUC_lexicon_tags_0 },
    { 0xb7fab1b5, SUC_lexicon_tags_8 },
    { 0x81ab1b4, SUC_lexicon_tags_37 },
    { 0xc83ab1b8, SUC_lexicon_tags_15 },
    { 0x109ab1b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13ab1bc, SUC_lexicon_tags_11 },
    { 0xbc3ab1bd, SUC_lexicon_tags_12 },
    { 0x7adab1be, SUC_lexicon_tags_2 },
    { 0xdc1ab1bf, SUC_lexicon_tags_25 },
    { 0x18dab1c0, SUC_lexicon_tags_2 },
    { 0xdcfab1c1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97ab1c6, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dab1ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09ab1cd, SUC_lexicon_tags_7 },
    { 0xb73ab1cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa43ab1d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddab1d3, SUC_lexicon_tags_82 },
    { 0x31ab1d3, SUC_lexicon_tags_9 },
    { 0xdf3ab1d3, SUC_lexicon_tags_14 },
    { 0x2bfab1d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b1ab1d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xab1ab1db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723ab1e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x555ab1e2, SUC_lexicon_tags_0 },
    { 0x79dab1e2, SUC_lexicon_tags_1 },
    { 0x2b9ab1e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bab1e7, SUC_lexicon_tags_15 },
    { 0xa43ab1e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7ab1ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729ab1f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43ab1fa, SUC_lexicon_tags_28 },
    { 0x5ebab1fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x671ab1fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab200, SUC_lexicon_tags_37 },
    { 0x29fab201, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ab205, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ab20a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727ab213, SUC_lexicon_tags_26 },
    { 0xcb5ab213, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abab218, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbab221, SUC_lexicon_tags_6 },
    { 0x4e9ab221, SUC_lexicon_tags_15 },
    { 0x851ab223, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67bab225, SUC_lexicon_tags_19 },
    { 0x295ab226, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60dab228, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11ab22a, SUC_lexicon_tags_1 },
    { 0x11bab22a, SUC_lexicon_tags_21 },
    { 0x1dfab22c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78dab22e, SUC_lexicon_tags_27 },
    { 0xd8bab22e, SUC_lexicon_tags_15 },
    { 0xbf3ab230, SUC_lexicon_tags_1 },
    { 0x971ab231, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xadfab233, SUC_lexicon_tags_1 },
    { 0x7f1ab234, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf6dab236, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafdab239, SUC_lexicon_tags_15 },
    { 0x6c7ab23a, SUC_lexicon_tags_55 },
    { 0xeebab23b, SUC_lexicon_tags_87 },
    { 0x90fab23b, SUC_lexicon_tags_9 },
    { 0x51bab23a, SUC_lexicon_tags_65 },
    { 0x8c3ab23e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5ab243, SUC_lexicon_tags_37 },
    { 0x46dab244, SUC_lexicon_tags_13 },
    { 0xa35ab244, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x207ab248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bab24c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3efab24e, SUC_lexicon_tags_47 },
    { 0x2b9ab24e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1ab254, SUC_lexicon_tags_87 },
    { 0x62dab255, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09ab25a, SUC_lexicon_tags_17 },
    { 0x471ab25a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dab265, SUC_lexicon_tags_27 },
    { 0xd7dab266, SUC_lexicon_tags_37 },
    { 0xa9dab266, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x21fab26a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf01ab26c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x221ab26f, SUC_lexicon_tags_33 },
    { 0xd99ab270, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3ab275, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb81ab277, SUC_lexicon_tags_13 },
    { 0x987ab278, SUC_lexicon_tags_15 },
    { 0xa35ab278, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbb3ab27b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295ab27f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fab285, SUC_lexicon_tags_25 },
    { 0x935ab286, SUC_lexicon_tags_2 },
    { 0xb03ab286, SUC_lexicon_tags_11 },
    { 0x681ab288, SUC_lexicon_tags_28 },
    { 0xec5ab288, SUC_lexicon_tags_9 },
    { 0x2e5ab28a, SUC_lexicon_tags_27 },
    { 0x749ab28a, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dab28e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcdfab290, SUC_lexicon_tags_8 },
    { 0xc73ab291, SUC_lexicon_tags_46 },
    { 0x9e1ab291, SUC_lexicon_tags_26 },
    { 0xbfbab293, SUC_lexicon_tags_27 },
    { 0xe57ab294, SUC_lexicon_tags_3 },
    { 0x377ab295, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5ab299, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dab29d, SUC_lexicon_tags_39 },
    { 0x601ab29d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5b7ab2a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x725ab2a2, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dab2a8, SUC_lexicon_tags_6 },
    { 0x2bab2a9, SUC_lexicon_tags_25 },
    { 0x6fab2a9, SUC_lexicon_tags_45 },
    { 0xf43ab2ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fab2b1, SUC_lexicon_tags_27 },
    { 0xe47ab2b2, SUC_lexicon_tags_113 },
    { 0x135ab2b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2f5ab2b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73ab2b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bab2bb, SUC_lexicon_tags_15 },
    { 0x887ab2bc, SUC_lexicon_tags_17 },
    { 0x4b1ab2bb, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547ab2c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd0dab2c6, SUC_lexicon_tags_47 },
    { 0x40dab2c7, SUC_lexicon_tags_14 },
    { 0x6bbab2c8, SUC_lexicon_tags_25 },
    { 0xd4dab2c9, SUC_lexicon_tags_32 },
    { 0x3abab2c7, SUC_lexicon_tags_37 },
    { 0x4a9ab2cb, SUC_lexicon_tags_39 },
    { 0xa15ab2cc, SUC_lexicon_tags_17 },
    { 0x51dab2ca, SUC_lexicon_tags_2 },
    { 0xd05ab2ce, SUC_lexicon_tags_38 },
    { 0xc87ab2c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5fab2d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe5ab2d3, SUC_lexicon_tags_28 },
    { 0x2a3ab2d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x471ab2d6, SUC_lexicon_tags_60 },
    { 0xba3ab2d7, SUC_lexicon_tags_37 },
    { 0xf2bab2d8, SUC_lexicon_tags_48 },
    { 0x5cdab2d9, SUC_lexicon_tags_31 },
    { 0x3f1ab2da, SUC_lexicon_tags_60 },
    { 0xd69ab2d8, SUC_lexicon_tags_14 },
    { 0xe2dab2da, SUC_lexicon_tags_17 },
    { 0x21bab2dd, SUC_lexicon_tags_7 },
    { 0x7c1ab2de, SUC_lexicon_tags_17 },
    { 0x377ab2dd, SUC_lexicon_tags_17 },
    { 0x8edab2df, SUC_lexicon_tags_26 },
    { 0x679ab2e1, SUC_lexicon_tags_4 },
    { 0x98dab2e2, SUC_lexicon_tags_141 },
    { 0x579ab2e2, SUC_lexicon_tags_64 },
    { 0x78dab2e4, SUC_lexicon_tags_18 },
    { 0xca9ab2e5, SUC_lexicon_tags_11 },
    { 0x113ab2e6, SUC_lexicon_tags_7 },
    { 0x95ab2e5, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1ab2ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3ab2ef, SUC_lexicon_tags_38 },
    { 0xce3ab2f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbab2f4, SUC_lexicon_tags_17 },
    { 0xc5ab2f5, SUC_lexicon_tags_23 },
    { 0xc33ab2f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbab2f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x34dab2fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc73ab2fe, SUC_lexicon_tags_40 },
    { 0x795ab2ff, SUC_lexicon_tags_0 },
    { 0xae9ab300, SUC_lexicon_tags_4 },
    { 0x20bab301, SUC_lexicon_tags_25 },
    { 0xe55ab2fe, SUC_lexicon_tags_33 },
    { 0x7cdab303, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ab307, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x599ab30a, SUC_lexicon_tags_1 },
    { 0x3dab30a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x757ab30d, SUC_lexicon_tags_12 },
    { 0x84fab30e, SUC_lexicon_tags_25 },
    { 0x667ab30f, SUC_lexicon_tags_19 },
    { 0xbe3ab310, SUC_lexicon_tags_16 },
    { 0x7f3ab311, SUC_lexicon_tags_33 },
    { 0x681ab311, SUC_lexicon_tags_87 },
    { 0x927ab310, SUC_lexicon_tags_14 },
    { 0x96bab310, SUC_lexicon_tags_11 },
    { 0x9b9ab315, SUC_lexicon_tags_1 },
    { 0xd7bab315, SUC_lexicon_tags_62 },
    { 0xa47ab314, SUC_lexicon_tags_11 },
    { 0x395ab318, SUC_lexicon_tags_25 },
    { 0xebdab314, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdab31b, SUC_lexicon_tags_44 },
    { 0x567ab31c, SUC_lexicon_tags_35 },
    { 0x487ab31b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3ab320, SUC_lexicon_tags_60 },
    { 0x2f9ab321, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdab324, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdab329, SUC_lexicon_tags_12 },
    { 0xb5ab32a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bfab32c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947ab331, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1ab336, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc5ab338, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x905ab33a, SUC_lexicon_tags_2 },
    { 0x673ab33b, SUC_lexicon_tags_11 },
    { 0xf11ab33c, SUC_lexicon_tags_62 },
    { 0x15dab33d, SUC_lexicon_tags_14 },
    { 0xf9ab33d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc7ab340, SUC_lexicon_tags_85 },
    { 0xd79ab341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bab344, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x565ab346, SUC_lexicon_tags_66 },
    { 0x943ab346, SUC_lexicon_tags_2 },
    { 0xf31ab346, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc69ab34a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddab350, SUC_lexicon_tags_25 },
    { 0xfb1ab350, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01ab354, SUC_lexicon_tags_48 },
    { 0x84bab355, SUC_lexicon_tags_63 },
    { 0xbf3ab356, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773ab361, SUC_lexicon_tags_17 },
    { 0x101ab362, SUC_lexicon_tags_17 },
    { 0x809ab363, SUC_lexicon_tags_25 },
    { 0x3a9ab364, SUC_lexicon_tags_46 },
    { 0xe45ab361, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa61ab367, SUC_lexicon_tags_38 },
    { 0xb77ab367, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb4bab36a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x899ab36d, SUC_lexicon_tags_70 },
    { 0x737ab36e, SUC_lexicon_tags_17 },
    { 0x6c9ab36e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1ab373, SUC_lexicon_tags_55 },
    { 0xa8dab374, SUC_lexicon_tags_60 },
    { 0x8d5ab374, SUC_lexicon_tags_29 },
    { 0xd1dab375, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3ab37c, SUC_lexicon_tags_37 },
    { 0xf09ab37c, SUC_lexicon_tags_9 },
    { 0x269ab37e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243ab389, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedab38f, SUC_lexicon_tags_6 },
    { 0x1dbab390, SUC_lexicon_tags_66 },
    { 0x305ab390, SUC_lexicon_tags_38 },
    { 0xaafab392, SUC_lexicon_tags_17 },
    { 0x4cfab392, SUC_lexicon_tags_46 },
    { 0x73ab394, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81ab397, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bdab399, SUC_lexicon_tags_11 },
    { 0xee1ab39a, SUC_lexicon_tags_17 },
    { 0x4b9ab39b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfab39e, SUC_lexicon_tags_25 },
    { 0xf83ab39e, SUC_lexicon_tags_39 },
    { 0xe9bab39e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53ab3a6, SUC_lexicon_tags_15 },
    { 0x157ab3a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bab3ab, SUC_lexicon_tags_1 },
    { 0x8b7ab3ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdadab3ae, SUC_lexicon_tags_7 },
    { 0x8a3ab3ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bab3b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13ab3b8, SUC_lexicon_tags_351 },
    { 0xf45ab3b9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb37ab3bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd1ab3bd, SUC_lexicon_tags_17 },
    { 0xe63ab3be, SUC_lexicon_tags_17 },
    { 0x2d9ab3bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5ab3c2, SUC_lexicon_tags_15 },
    { 0xd37ab3c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bab3ce, SUC_lexicon_tags_12 },
    { 0x63bab3cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbab3d3, SUC_lexicon_tags_6 },
    { 0x7e9ab3d4, SUC_lexicon_tags_47 },
    { 0xd7bab3d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5dbab3d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3ab3da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ab3de, SUC_lexicon_tags_19 },
    { 0x40bab3df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9ab3e4, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xf35ab3e6, SUC_lexicon_tags_57 },
    { 0x55dab3e6, SUC_lexicon_tags_37 },
    { 0xa1bab3e7, SUC_lexicon_tags_11 },
    { 0x727ab3e6, SUC_lexicon_tags_55 },
    { 0x113ab3ea, SUC_lexicon_tags_62 },
    { 0xf17ab3e7, SUC_lexicon_tags_27 },
    { 0x37fab3ec, SUC_lexicon_tags_11 },
    { 0x67bab3ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545ab3f1, SUC_lexicon_tags_35 },
    { 0x58fab3f2, SUC_lexicon_tags_17 },
    { 0xdffab3f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3ab3fb, SUC_lexicon_tags_15 },
    { 0x5b5ab3fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13ab3ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19dab402, SUC_lexicon_tags_55 },
    { 0xb63ab403, SUC_lexicon_tags_15 },
    { 0x785ab402, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73bab406, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fab409, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbfab40b, SUC_lexicon_tags_23 },
    { 0x19fab40c, SUC_lexicon_tags_31 },
    { 0xd77ab40d, SUC_lexicon_tags_25 },
    { 0xb63ab40c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x455ab410, SUC_lexicon_tags_49 },
    { 0x721ab410, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdab414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddab417, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x25fab41a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475ab41f, SUC_lexicon_tags_33 },
    { 0x92fab420, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403ab428, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7c7ab42a, SUC_lexicon_tags_40 },
    { 0xafdab42b, SUC_lexicon_tags_17 },
    { 0x389ab42c, SUC_lexicon_tags_33 },
    { 0xb7dab42a, SUC_lexicon_tags_8 },
    { 0xcd9ab42e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7fab430, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fab435, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf8dab437, SUC_lexicon_tags_2 },
    { 0xdb3ab437, SUC_lexicon_tags_57 },
    { 0xc93ab439, SUC_lexicon_tags_37 },
    { 0xfebab439, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x419ab43d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fab440, SUC_lexicon_tags_47 },
    { 0x103ab441, SUC_lexicon_tags_1 },
    { 0x3d3ab442, SUC_lexicon_tags_7 },
    { 0xf85ab441, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbab448, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7ab44b, SUC_lexicon_tags_2 },
    { 0x2bfab44b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dab452, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdab455, SUC_lexicon_tags_17 },
    { 0xed9ab456, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3ab45a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253ab460, SUC_lexicon_tags_33 },
    { 0x33dab461, SUC_lexicon_tags_7 },
    { 0x621ab462, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65bab464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd65ab466, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4fdab468, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbab46b, SUC_lexicon_tags_69 },
    { 0x167ab46c, SUC_lexicon_tags_17 },
    { 0x55dab46b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xafbab46f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7ab476, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb27ab478, SUC_lexicon_tags_23 },
    { 0x8a7ab478, SUC_lexicon_tags_23 },
    { 0x7ebab478, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbe7ab47c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbc9ab47e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fab484, SUC_lexicon_tags_17 },
    { 0x929ab485, SUC_lexicon_tags_44 },
    { 0x1abab485, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559ab48b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733ab492, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa07ab494, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8adab496, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1ab499, SUC_lexicon_tags_107 },
    { 0xa03ab499, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbab49d, SUC_lexicon_tags_26 },
    { 0x94fab49e, SUC_lexicon_tags_35 },
    { 0x789ab49f, SUC_lexicon_tags_25 },
    { 0x5fbab4a0, SUC_lexicon_tags_75 },
    { 0xc6bab4a0, SUC_lexicon_tags_45 },
    { 0x305ab4a2, SUC_lexicon_tags_17 },
    { 0x4cfab4a1, SUC_lexicon_tags_77 },
    { 0x72dab49e, SUC_lexicon_tags_25 },
    { 0x1b5ab4a5, SUC_lexicon_tags_46 },
    { 0xbbbab4a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bab4a9, SUC_lexicon_tags_15 },
    { 0xd01ab4aa, SUC_lexicon_tags_46 },
    { 0x957ab4ab, SUC_lexicon_tags_17 },
    { 0x2f9ab4ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981ab4b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7efab4b3, SUC_lexicon_tags_150 },
    { 0x931ab4b3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x605ab4b7, SUC_lexicon_tags_15 },
    { 0x1a7ab4b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbab4bb, SUC_lexicon_tags_16 },
    { 0xea9ab4bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bab4c0, SUC_lexicon_tags_7 },
    { 0x269ab4c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ab4c4, SUC_lexicon_tags_1 },
    { 0xc3fab4c5, SUC_lexicon_tags_17 },
    { 0x4c3ab4c5, SUC_lexicon_tags_25 },
    { 0x207ab4c7, SUC_lexicon_tags_68 },
    { 0xd55ab4c4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7dab4ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fab4ce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbb3ab4d0, SUC_lexicon_tags_10 },
    { 0xe87ab4d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1ab4d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5fab4d6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fab4de, SUC_lexicon_tags_11 },
    { 0x575ab4de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x591ab4e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bab4f0, SUC_lexicon_tags_12 },
    { 0x475ab4f1, SUC_lexicon_tags_17 },
    { 0xef3ab4f1, SUC_lexicon_tags_11 },
    { 0x71fab4f1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bab4fa, SUC_lexicon_tags_1 },
    { 0xf1bab4fb, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7ab4fe, SUC_lexicon_tags_15 },
    { 0xdd3ab4ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9ab504, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5bab506, SUC_lexicon_tags_15 },
    { 0x655ab507, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adab50b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239ab50f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dab515, SUC_lexicon_tags_37 },
    { 0xd65ab516, SUC_lexicon_tags_1 },
    { 0x497ab517, SUC_lexicon_tags_1 },
    { 0x569ab517, SUC_lexicon_tags_9 },
    { 0xe0bab518, SUC_lexicon_tags_45 },
    { 0xa47ab51a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x559ab51d, SUC_lexicon_tags_23 },
    { 0xf19ab51d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77ab521, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb93ab523, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe15ab525, SUC_lexicon_tags_26 },
    { 0x49bab526, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17ab52c, SUC_lexicon_tags_55 },
    { 0xf1bab52d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39ab533, SUC_lexicon_tags_34 },
    { 0x8b1ab534, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dab538, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x913ab53a, SUC_lexicon_tags_37 },
    { 0xbbab53b, SUC_lexicon_tags_37 },
    { 0x8d5ab53b, SUC_lexicon_tags_15 },
    { 0x525ab53d, SUC_lexicon_tags_28 },
    { 0xccdab53c, SUC_lexicon_tags_38 },
    { 0x703ab53f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83ab544, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5ab54d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6b1ab54f, SUC_lexicon_tags_11 },
    { 0x55dab550, SUC_lexicon_tags_6 },
    { 0xe19ab551, SUC_lexicon_tags_8 },
    { 0xabdab550, SUC_lexicon_tags_9 },
    { 0x8fab553, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbab557, SUC_lexicon_tags_25 },
    { 0x6ffab558, SUC_lexicon_tags_230 },
    { 0xd49ab558, SUC_lexicon_tags_12 },
    { 0xf39ab55a, SUC_lexicon_tags_17 },
    { 0x605ab55a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ab560, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7afab563, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2c7ab565, SUC_lexicon_tags_33 },
    { 0xa63ab566, SUC_lexicon_tags_14 },
    { 0x86fab565, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1ab56e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99ab571, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x389ab574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc03ab576, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5ab579, SUC_lexicon_tags_1 },
    { 0x559ab57a, SUC_lexicon_tags_27 },
    { 0x3a1ab57a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbcdab57d, SUC_lexicon_tags_28 },
    { 0x4efab57e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bab582, SUC_lexicon_tags_15 },
    { 0x7cfab583, SUC_lexicon_tags_528 },
    { 0xcbab584, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351ab588, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3ab58b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3ab591, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdab597, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x71bab599, SUC_lexicon_tags_11 },
    { 0x39ab59a, SUC_lexicon_tags_2 },
    { 0xf33ab59b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1ab5a1, SUC_lexicon_tags_26 },
    { 0x265ab5a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb9ab5a4, SUC_lexicon_tags_37 },
    { 0x329ab5a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747ab5a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6dfab5ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ab5af, SUC_lexicon_tags_38 },
    { 0xcb5ab5b0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb8fab5b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7ab5b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7ab5ba, SUC_lexicon_tags_133 },
    { 0xf5fab5bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39ab5bf, SUC_lexicon_tags_15 },
    { 0xfa3ab5bf, SUC_lexicon_tags_2 },
    { 0x449ab5c1, SUC_lexicon_tags_45 },
    { 0x17bab5c2, SUC_lexicon_tags_31 },
    { 0x525ab5c0, SUC_lexicon_tags_12 },
    { 0x18bab5c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dab5c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x175ab5ca, SUC_lexicon_tags_14 },
    { 0xd0bab5cb, SUC_lexicon_tags_7 },
    { 0x14bab5cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x179ab5cf, SUC_lexicon_tags_10 },
    { 0x2efab5d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69ab5d5, SUC_lexicon_tags_11 },
    { 0xdd3ab5d6, SUC_lexicon_tags_27 },
    { 0xc39ab5d5, SUC_lexicon_tags_47 },
    { 0x9edab5d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81ab5dc, SUC_lexicon_tags_45 },
    { 0xaffab5dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47ab5e0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407ab5e4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481ab5ed, SUC_lexicon_tags_29 },
    { 0x835ab5ee, SUC_lexicon_tags_17 },
    { 0x543ab5ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x617ab5f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdab5f4, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91ab5f8, SUC_lexicon_tags_56 },
    { 0xa01ab5f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563ab5fd, SUC_lexicon_tags_17 },
    { 0xd8bab5fd, SUC_lexicon_tags_30 },
    { 0x2c7ab5ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x187ab602, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91ab609, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0x731ab60c, SUC_lexicon_tags_12 },
    { 0x2b9ab60c, SUC_lexicon_tags_62 },
    { 0xbd9ab60e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb1ab610, SUC_lexicon_tags_11 },
    { 0xc9ab611, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebab619, SUC_lexicon_tags_0 },
    { 0x6d9ab619, SUC_lexicon_tags_11 },
    { 0x305ab61a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bab61f, SUC_lexicon_tags_11 },
    { 0x73dab620, SUC_lexicon_tags_234 },
    { 0xa7dab621, SUC_lexicon_tags_38 },
    { 0x7fab622, SUC_lexicon_tags_47 },
    { 0xe09ab623, SUC_lexicon_tags_49 },
    { 0x711ab622, SUC_lexicon_tags_23 },
    { 0x539ab625, SUC_lexicon_tags_10 },
    { 0x2c7ab620, SUC_lexicon_tags_77 },
    { 0x96bab625, SUC_lexicon_tags_29 },
    { 0x323ab628, SUC_lexicon_tags_11 },
    { 0x73ab622, SUC_lexicon_tags_27 },
    { 0x131ab62a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bab62d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4f3ab62f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ab638, SUC_lexicon_tags_12 },
    { 0x82bab639, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x821ab63b, SUC_lexicon_tags_1 },
    { 0x165ab63c, SUC_lexicon_tags_13 },
    { 0xbbfab63d, SUC_lexicon_tags_11 },
    { 0xf75ab63c, SUC_lexicon_tags_25 },
    { 0x955ab63e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ab642, SUC_lexicon_tags_768 },
    { 0, NULL },
    { 0xfcfab644, SUC_lexicon_tags_1 },
    { 0xdadab644, SUC_lexicon_tags_38 },
    { 0x8c5ab645, SUC_lexicon_tags_28 },
    { 0xf1bab644, SUC_lexicon_tags_48 },
    { 0x20dab648, SUC_lexicon_tags_11 },
    { 0xbb5ab649, SUC_lexicon_tags_17 },
    { 0xca7ab648, SUC_lexicon_tags_1 },
    { 0xe1ab64a, SUC_lexicon_tags_15 },
    { 0x19bab649, SUC_lexicon_tags_34 },
    { 0x9abab64a, SUC_lexicon_tags_37 },
    { 0x8c5ab649, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3ab652, SUC_lexicon_tags_39 },
    { 0xb8bab653, SUC_lexicon_tags_7 },
    { 0x3bfab654, SUC_lexicon_tags_65 },
    { 0x24bab655, SUC_lexicon_tags_15 },
    { 0x155ab656, SUC_lexicon_tags_14 },
    { 0x7ddab655, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3ab65a, SUC_lexicon_tags_22 },
    { 0x321ab65b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fab665, SUC_lexicon_tags_17 },
    { 0x2e7ab665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7ab669, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdab66c, SUC_lexicon_tags_15 },
    { 0x909ab66c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf03ab66f, SUC_lexicon_tags_46 },
    { 0xeafab670, SUC_lexicon_tags_17 },
    { 0xf99ab671, SUC_lexicon_tags_50 },
    { 0x16dab672, SUC_lexicon_tags_301 },
    { 0xb71ab670, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x51fab675, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc13ab677, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8dab679, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55ab67d, SUC_lexicon_tags_16 },
    { 0x28bab67d, SUC_lexicon_tags_11 },
    { 0x365ab67f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3dbab681, SUC_lexicon_tags_12 },
    { 0x8ddab682, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x991ab684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1ab68f, SUC_lexicon_tags_12 },
    { 0x5d5ab68f, SUC_lexicon_tags_15 },
    { 0xb85ab691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7ab694, SUC_lexicon_tags_6 },
    { 0x463ab695, SUC_lexicon_tags_2 },
    { 0xd3fab696, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99ab69a, SUC_lexicon_tags_7 },
    { 0xf8fab69b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabab69f, SUC_lexicon_tags_2 },
    { 0xca9ab6a0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129ab6a5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07ab6a9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01ab6ac, SUC_lexicon_tags_17 },
    { 0x81fab6ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d7ab6af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61ab6b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1ab6ba, SUC_lexicon_tags_25 },
    { 0xa4bab6ba, SUC_lexicon_tags_2 },
    { 0x723ab6bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5ab6c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x69dab6c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fab6ca, SUC_lexicon_tags_29 },
    { 0xf21ab6cb, SUC_lexicon_tags_1 },
    { 0xedab6ca, SUC_lexicon_tags_25 },
    { 0xbadab6cc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9ab6d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x673ab6d2, SUC_lexicon_tags_17 },
    { 0x483ab6d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bab6dc, SUC_lexicon_tags_1 },
    { 0xb73ab6dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x443ab6e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afab6e9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dab6ec, SUC_lexicon_tags_37 },
    { 0x713ab6ed, SUC_lexicon_tags_23 },
    { 0x753ab6ee, SUC_lexicon_tags_23 },
    { 0x3cbab6ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afab6f3, SUC_lexicon_tags_15 },
    { 0x7f3ab6f4, SUC_lexicon_tags_47 },
    { 0x179ab6f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a5ab6f7, SUC_lexicon_tags_11 },
    { 0x4b9ab6f8, SUC_lexicon_tags_46 },
    { 0xa1bab6f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5f5ab6fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9ab703, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x983ab705, SUC_lexicon_tags_1 },
    { 0x245ab706, SUC_lexicon_tags_25 },
    { 0x9abab707, SUC_lexicon_tags_27 },
    { 0xa6fab708, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73ab70d, SUC_lexicon_tags_29 },
    { 0x2a1ab70d, SUC_lexicon_tags_63 },
    { 0xfd1ab70f, SUC_lexicon_tags_37 },
    { 0xcf9ab70f, SUC_lexicon_tags_15 },
    { 0x8bab70d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29fab718, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdab71b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5ab722, SUC_lexicon_tags_12 },
    { 0x253ab722, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5adab725, SUC_lexicon_tags_7 },
    { 0x19fab726, SUC_lexicon_tags_214 },
    { 0x1fdab727, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda1ab729, SUC_lexicon_tags_33 },
    { 0x2c9ab72a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41fab72d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x519ab730, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79dab732, SUC_lexicon_tags_31 },
    { 0xabab732, SUC_lexicon_tags_68 },
    { 0x29bab732, SUC_lexicon_tags_8 },
    { 0x527ab735, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1ab738, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd75ab73a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5ab73d, SUC_lexicon_tags_15 },
    { 0x625ab73e, SUC_lexicon_tags_17 },
    { 0xd73ab73e, SUC_lexicon_tags_17 },
    { 0x3cbab73e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dab74c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41ab750, SUC_lexicon_tags_19 },
    { 0x5f3ab750, SUC_lexicon_tags_46 },
    { 0x27bab750, SUC_lexicon_tags_6 },
    { 0x853ab753, SUC_lexicon_tags_27 },
    { 0xc59ab753, SUC_lexicon_tags_33 },
    { 0x6f1ab755, SUC_lexicon_tags_47 },
    { 0xc49ab756, SUC_lexicon_tags_19 },
    { 0xb67ab757, SUC_lexicon_tags_17 },
    { 0xb59ab757, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x199ab75b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x583ab75e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x627ab760, SUC_lexicon_tags_47 },
    { 0x73fab761, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x909ab763, SUC_lexicon_tags_25 },
    { 0x209ab764, SUC_lexicon_tags_25 },
    { 0x417ab765, SUC_lexicon_tags_12 },
    { 0x89dab765, SUC_lexicon_tags_14 },
    { 0x3e1ab766, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fab76a, SUC_lexicon_tags_61 },
    { 0x527ab76a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647ab771, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd6dab773, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353ab778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487ab77e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdab783, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bab78b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdab78e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923ab794, SUC_lexicon_tags_38 },
    { 0xfd5ab795, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf6bab797, SUC_lexicon_tags_37 },
    { 0x2e1ab798, SUC_lexicon_tags_83 },
    { 0xbf5ab799, SUC_lexicon_tags_47 },
    { 0xf83ab798, SUC_lexicon_tags_53 },
    { 0x793ab79b, SUC_lexicon_tags_47 },
    { 0xeefab79c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09ab79f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc81ab7a1, SUC_lexicon_tags_63 },
    { 0x899ab7a2, SUC_lexicon_tags_29 },
    { 0x8d1ab7a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfdfab7a5, SUC_lexicon_tags_7 },
    { 0x40bab7a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45ab7ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x463ab7ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5ab7b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfab7ba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x70fab7bc, SUC_lexicon_tags_33 },
    { 0xc1ab7bd, SUC_lexicon_tags_12 },
    { 0x767ab7be, SUC_lexicon_tags_16 },
    { 0x479ab7bd, SUC_lexicon_tags_47 },
    { 0xb57ab7c0, SUC_lexicon_tags_25 },
    { 0x7fab7c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ab7c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09ab7cf, SUC_lexicon_tags_47 },
    { 0x5b1ab7d0, SUC_lexicon_tags_37 },
    { 0xecfab7cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf21ab7d3, SUC_lexicon_tags_57 },
    { 0xb73ab7d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbab7d7, SUC_lexicon_tags_15 },
    { 0xcb7ab7d7, SUC_lexicon_tags_23 },
    { 0x455ab7d9, SUC_lexicon_tags_64 },
    { 0xd7ab7d9, SUC_lexicon_tags_23 },
    { 0x971ab7db, SUC_lexicon_tags_15 },
    { 0x46bab7dc, SUC_lexicon_tags_11 },
    { 0x137ab7d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x20bab7df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303ab7e4, SUC_lexicon_tags_45 },
    { 0x65bab7e4, SUC_lexicon_tags_47 },
    { 0x3c7ab7e6, SUC_lexicon_tags_33 },
    { 0xcd1ab7e7, SUC_lexicon_tags_2 },
    { 0xe79ab7e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dab7ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81ab7f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bab7f6, SUC_lexicon_tags_103 },
    { 0x1dfab7f6, SUC_lexicon_tags_37 },
    { 0x8c3ab7f8, SUC_lexicon_tags_55 },
    { 0xa1dab7f9, SUC_lexicon_tags_23 },
    { 0xcb7ab7f8, SUC_lexicon_tags_25 },
    { 0x799ab7f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49ab802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07ab806, SUC_lexicon_tags_42 },
    { 0x1cdab807, SUC_lexicon_tags_148 },
    { 0xd5fab807, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dab80d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35ab812, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53ab816, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55ab81a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdab81e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ab829, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1ab82c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fab82e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf23ab830, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x117ab833, SUC_lexicon_tags_12 },
    { 0x2c9ab834, SUC_lexicon_tags_7 },
    { 0xb4bab835, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b1ab837, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x429ab83a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5dab83c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7ab83f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57ab842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1ab846, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589ab84b, SUC_lexicon_tags_11 },
    { 0x563ab84c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2bfab84e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1ab851, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x825ab853, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97ab856, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4d9ab858, SUC_lexicon_tags_8 },
    { 0x593ab859, SUC_lexicon_tags_7 },
    { 0x8a7ab85a, SUC_lexicon_tags_11 },
    { 0x3e1ab85b, SUC_lexicon_tags_7 },
    { 0x3a9ab859, SUC_lexicon_tags_31 },
    { 0x397ab85d, SUC_lexicon_tags_11 },
    { 0x565ab85e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fab865, SUC_lexicon_tags_14 },
    { 0x789ab866, SUC_lexicon_tags_19 },
    { 0x133ab867, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xafab86a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9ab86d, SUC_lexicon_tags_19 },
    { 0xdd7ab86e, SUC_lexicon_tags_12 },
    { 0xe1dab86f, SUC_lexicon_tags_32 },
    { 0x211ab870, SUC_lexicon_tags_15 },
    { 0x573ab870, SUC_lexicon_tags_46 },
    { 0xaddab86f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461ab876, SUC_lexicon_tags_17 },
    { 0x2e1ab877, SUC_lexicon_tags_87 },
    { 0x223ab878, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe05ab87a, SUC_lexicon_tags_0 },
    { 0x6cbab87b, SUC_lexicon_tags_1 },
    { 0xa4bab87c, SUC_lexicon_tags_19 },
    { 0x399ab87c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa67ab87f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5ab883, SUC_lexicon_tags_60 },
    { 0x507ab883, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865ab889, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x101ab88c, SUC_lexicon_tags_63 },
    { 0xb4bab88c, SUC_lexicon_tags_25 },
    { 0xc97ab88e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe97ab890, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3ab897, SUC_lexicon_tags_7 },
    { 0xac5ab897, SUC_lexicon_tags_17 },
    { 0xd4dab898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25ab89e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29bab8a0, SUC_lexicon_tags_17 },
    { 0xd59ab8a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd11ab8a3, SUC_lexicon_tags_866 },
    { 0, NULL },
    { 0x193ab8a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abab8a8, SUC_lexicon_tags_17 },
    { 0x9e3ab8a9, SUC_lexicon_tags_37 },
    { 0x1fab8a8, SUC_lexicon_tags_8 },
    { 0x3fdab8ab, SUC_lexicon_tags_25 },
    { 0x9a9ab8ac, SUC_lexicon_tags_12 },
    { 0xc7dab8aa, SUC_lexicon_tags_30 },
    { 0x995ab8ae, SUC_lexicon_tags_97 },
    { 0xacfab8af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343ab8b3, SUC_lexicon_tags_37 },
    { 0x2a1ab8b4, SUC_lexicon_tags_7 },
    { 0x49fab8b5, SUC_lexicon_tags_48 },
    { 0x70fab8b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa85ab8b8, SUC_lexicon_tags_17 },
    { 0xe25ab8b9, SUC_lexicon_tags_25 },
    { 0xe25ab8ba, SUC_lexicon_tags_11 },
    { 0x3efab8ba, SUC_lexicon_tags_31 },
    { 0xb4dab8b9, SUC_lexicon_tags_87 },
    { 0xa3ab8b9, SUC_lexicon_tags_32 },
    { 0x9f1ab8bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x933ab8c0, SUC_lexicon_tags_25 },
    { 0x7efab8c0, SUC_lexicon_tags_11 },
    { 0xe7bab8c2, SUC_lexicon_tags_11 },
    { 0x7b7ab8c3, SUC_lexicon_tags_12 },
    { 0x3f3ab8c3, SUC_lexicon_tags_25 },
    { 0x39bab8c4, SUC_lexicon_tags_48 },
    { 0x46fab8c6, SUC_lexicon_tags_37 },
    { 0xa1ab8c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515ab8cd, SUC_lexicon_tags_65 },
    { 0xbd5ab8cd, SUC_lexicon_tags_42 },
    { 0x175ab8cf, SUC_lexicon_tags_86 },
    { 0xec7ab8ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709ab8d9, SUC_lexicon_tags_65 },
    { 0x90dab8da, SUC_lexicon_tags_0 },
    { 0xbb3ab8d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc01ab8dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdab8e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edab8ec, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf37ab8ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b3ab8f0, SUC_lexicon_tags_17 },
    { 0x8f9ab8f1, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x45dab8f4, SUC_lexicon_tags_0 },
    { 0x8d3ab8f5, SUC_lexicon_tags_15 },
    { 0x72dab8f4, SUC_lexicon_tags_37 },
    { 0xc35ab8f7, SUC_lexicon_tags_19 },
    { 0xa61ab8f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9c5ab8fa, SUC_lexicon_tags_13 },
    { 0x81ab8fb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x357ab8fd, SUC_lexicon_tags_27 },
    { 0x921ab8fd, SUC_lexicon_tags_25 },
    { 0x5afab8ff, SUC_lexicon_tags_37 },
    { 0xd8dab900, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575ab90a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667ab918, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x63bab91a, SUC_lexicon_tags_2 },
    { 0xf8fab91b, SUC_lexicon_tags_7 },
    { 0x83fab91a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x655ab91f, SUC_lexicon_tags_7 },
    { 0x541ab920, SUC_lexicon_tags_27 },
    { 0x27fab91f, SUC_lexicon_tags_37 },
    { 0x121ab922, SUC_lexicon_tags_2 },
    { 0x8efab921, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7ab927, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21ab92b, SUC_lexicon_tags_15 },
    { 0xca9ab92c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfcbab92e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ab932, SUC_lexicon_tags_1 },
    { 0xdd3ab933, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x76dab935, SUC_lexicon_tags_33 },
    { 0xdfbab936, SUC_lexicon_tags_25 },
    { 0x8fdab937, SUC_lexicon_tags_37 },
    { 0x44dab938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113ab93c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x41bab93e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93ab944, SUC_lexicon_tags_6 },
    { 0xb57ab945, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7ab948, SUC_lexicon_tags_2 },
    { 0x615ab949, SUC_lexicon_tags_11 },
    { 0x797ab949, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03ab94d, SUC_lexicon_tags_7 },
    { 0x121ab94e, SUC_lexicon_tags_42 },
    { 0xe73ab94e, SUC_lexicon_tags_49 },
    { 0xef3ab950, SUC_lexicon_tags_31 },
    { 0x99bab951, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bab954, SUC_lexicon_tags_49 },
    { 0xe3ab955, SUC_lexicon_tags_14 },
    { 0xab7ab954, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc77ab959, SUC_lexicon_tags_17 },
    { 0x5c5ab95a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfab95f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565ab968, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9fab96a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dab96f, SUC_lexicon_tags_33 },
    { 0x7a9ab970, SUC_lexicon_tags_62 },
    { 0xeb3ab971, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x14bab973, SUC_lexicon_tags_25 },
    { 0x8e7ab973, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7ab97a, SUC_lexicon_tags_28 },
    { 0xf4bab97a, SUC_lexicon_tags_23 },
    { 0x78dab97a, SUC_lexicon_tags_25 },
    { 0x97fab97b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xb11ab97f, SUC_lexicon_tags_37 },
    { 0xd1bab980, SUC_lexicon_tags_147 },
    { 0xe2dab97f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15dab983, SUC_lexicon_tags_69 },
    { 0x4bdab984, SUC_lexicon_tags_10 },
    { 0x8bbab984, SUC_lexicon_tags_63 },
    { 0xecbab983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x961ab988, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbbab98a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x74fab98c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43ab98f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd5dab991, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x525ab993, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5ab999, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1ab99e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4fab9a0, SUC_lexicon_tags_37 },
    { 0x82bab9a1, SUC_lexicon_tags_60 },
    { 0xc4fab9a2, SUC_lexicon_tags_2 },
    { 0x4dab9a1, SUC_lexicon_tags_25 },
    { 0x7a7ab9a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bab9a8, SUC_lexicon_tags_2 },
    { 0x49fab9a9, SUC_lexicon_tags_87 },
    { 0xd65ab9aa, SUC_lexicon_tags_36 },
    { 0x6f3ab9ab, SUC_lexicon_tags_34 },
    { 0x903ab9ab, SUC_lexicon_tags_17 },
    { 0xb0bab9ad, SUC_lexicon_tags_7 },
    { 0x323ab9ad, SUC_lexicon_tags_37 },
    { 0x337ab9af, SUC_lexicon_tags_63 },
    { 0x31bab9aa, SUC_lexicon_tags_12 },
    { 0x259ab9b1, SUC_lexicon_tags_2 },
    { 0x311ab9b2, SUC_lexicon_tags_46 },
    { 0x481ab9ac, SUC_lexicon_tags_24 },
    { 0x455ab9a8, SUC_lexicon_tags_17 },
    { 0x27bab9b5, SUC_lexicon_tags_19 },
    { 0x1afab9b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x397ab9b8, SUC_lexicon_tags_25 },
    { 0x745ab9b9, SUC_lexicon_tags_11 },
    { 0xd55ab9ba, SUC_lexicon_tags_2 },
    { 0xf17ab9bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dab9be, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdb7ab9c0, SUC_lexicon_tags_33 },
    { 0x67bab9c1, SUC_lexicon_tags_2 },
    { 0xb99ab9c0, SUC_lexicon_tags_45 },
    { 0xbe3ab9c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3ab9c8, SUC_lexicon_tags_17 },
    { 0xe73ab9c9, SUC_lexicon_tags_29 },
    { 0xfbdab9c9, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7ab9cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3ab9d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ab9d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x50bab9d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb5ab9d8, SUC_lexicon_tags_10 },
    { 0xd95ab9d9, SUC_lexicon_tags_17 },
    { 0x6c3ab9d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3ab9dd, SUC_lexicon_tags_17 },
    { 0x38dab9dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bab9e2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbab9e7, SUC_lexicon_tags_39 },
    { 0xcfab9e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x60dab9eb, SUC_lexicon_tags_1 },
    { 0x2f5ab9ec, SUC_lexicon_tags_37 },
    { 0x135ab9ec, SUC_lexicon_tags_43 },
    { 0x7cdab9ee, SUC_lexicon_tags_47 },
    { 0x18bab9ee, SUC_lexicon_tags_63 },
    { 0x781ab9f0, SUC_lexicon_tags_18 },
    { 0x82fab9f1, SUC_lexicon_tags_138 },
    { 0xc45ab9f2, SUC_lexicon_tags_1 },
    { 0x3fab9f3, SUC_lexicon_tags_10 },
    { 0xfdbab9f3, SUC_lexicon_tags_2 },
    { 0x589ab9f2, SUC_lexicon_tags_11 },
    { 0x603ab9ee, SUC_lexicon_tags_15 },
    { 0xa7dab9f7, SUC_lexicon_tags_15 },
    { 0x9a1ab9f7, SUC_lexicon_tags_33 },
    { 0x9cdab9f6, SUC_lexicon_tags_25 },
    { 0xcabab9f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x313ab9fc, SUC_lexicon_tags_12 },
    { 0x4b3ab9fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b3ab9ff, SUC_lexicon_tags_43 },
    { 0x2c3ab9ff, SUC_lexicon_tags_23 },
    { 0x28daba01, SUC_lexicon_tags_11 },
    { 0xf83aba02, SUC_lexicon_tags_17 },
    { 0x3dbaba03, SUC_lexicon_tags_60 },
    { 0x4baba02, SUC_lexicon_tags_3 },
    { 0x469aba05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x313aba08, SUC_lexicon_tags_33 },
    { 0xc13aba09, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadaba0f, SUC_lexicon_tags_11 },
    { 0x7f7aba10, SUC_lexicon_tags_215 },
    { 0xb81aba0f, SUC_lexicon_tags_2 },
    { 0xfc5aba10, SUC_lexicon_tags_27 },
    { 0xb19aba13, SUC_lexicon_tags_48 },
    { 0xfb7aba11, SUC_lexicon_tags_31 },
    { 0x9afaba15, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x80faba18, SUC_lexicon_tags_66 },
    { 0xa0faba18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbaba1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x57aba20, SUC_lexicon_tags_11 },
    { 0x51aba21, SUC_lexicon_tags_25 },
    { 0x127aba21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9daba28, SUC_lexicon_tags_35 },
    { 0xd77aba28, SUC_lexicon_tags_479 },
    { 0x86faba2a, SUC_lexicon_tags_1 },
    { 0xad1aba28, SUC_lexicon_tags_2 },
    { 0xbfaba29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865aba30, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x691aba33, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7aba39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b5aba3b, SUC_lexicon_tags_12 },
    { 0x6f7aba3c, SUC_lexicon_tags_7 },
    { 0xa13aba3c, SUC_lexicon_tags_14 },
    { 0x675aba3d, SUC_lexicon_tags_12 },
    { 0x337aba3f, SUC_lexicon_tags_63 },
    { 0x2bfaba40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77aba44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edaba4a, SUC_lexicon_tags_2 },
    { 0xf3faba4b, SUC_lexicon_tags_18 },
    { 0x1ddaba4c, SUC_lexicon_tags_63 },
    { 0xb7daba4d, SUC_lexicon_tags_27 },
    { 0xa1aba4d, SUC_lexicon_tags_31 },
    { 0x33aba4f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9aba52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65aba58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebaba60, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789aba65, SUC_lexicon_tags_1 },
    { 0x2d1aba66, SUC_lexicon_tags_30 },
    { 0xb9faba66, SUC_lexicon_tags_11 },
    { 0x20daba68, SUC_lexicon_tags_30 },
    { 0xf3faba68, SUC_lexicon_tags_43 },
    { 0xa89aba68, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x107aba6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34faba6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbffaba70, SUC_lexicon_tags_19 },
    { 0x4b3aba70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9cbaba73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295aba77, SUC_lexicon_tags_39 },
    { 0xacfaba78, SUC_lexicon_tags_57 },
    { 0xaababa79, SUC_lexicon_tags_29 },
    { 0x199aba7a, SUC_lexicon_tags_30 },
    { 0x411aba79, SUC_lexicon_tags_33 },
    { 0x26daba7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbaba7f, SUC_lexicon_tags_49 },
    { 0x243aba80, SUC_lexicon_tags_29 },
    { 0x689aba81, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x773aba83, SUC_lexicon_tags_10 },
    { 0x26baba84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0faba86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5aba8b, SUC_lexicon_tags_25 },
    { 0xf1faba8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35faba92, SUC_lexicon_tags_48 },
    { 0xc43aba92, SUC_lexicon_tags_55 },
    { 0x4adaba94, SUC_lexicon_tags_33 },
    { 0x33baba95, SUC_lexicon_tags_1 },
    { 0xf83aba92, SUC_lexicon_tags_12 },
    { 0x145aba97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39daba9c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5abaa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f7abaa9, SUC_lexicon_tags_55 },
    { 0x247abaa9, SUC_lexicon_tags_40 },
    { 0xd7dabaab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x277abaad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dabab1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443abab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabfababb, SUC_lexicon_tags_3 },
    { 0xff3ababc, SUC_lexicon_tags_250 },
    { 0x1a7ababb, SUC_lexicon_tags_29 },
    { 0xd79ababe, SUC_lexicon_tags_1 },
    { 0x6bfababc, SUC_lexicon_tags_37 },
    { 0x8b9abac0, SUC_lexicon_tags_25 },
    { 0x495abac1, SUC_lexicon_tags_12 },
    { 0xd4dababb, SUC_lexicon_tags_1 },
    { 0x899ababe, SUC_lexicon_tags_12 },
    { 0xac7abac1, SUC_lexicon_tags_14 },
    { 0x627abac5, SUC_lexicon_tags_0 },
    { 0xdcfabac6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9abaca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x245abacc, SUC_lexicon_tags_47 },
    { 0xd51abacc, SUC_lexicon_tags_21 },
    { 0xae9abace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbabad7, SUC_lexicon_tags_15 },
    { 0x867abad8, SUC_lexicon_tags_12 },
    { 0x15babad9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7abadd, SUC_lexicon_tags_10 },
    { 0x229abade, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dabae2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865abae9, SUC_lexicon_tags_27 },
    { 0x223abaea, SUC_lexicon_tags_29 },
    { 0x559abaeb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x46fabaed, SUC_lexicon_tags_14 },
    { 0x8d1abaee, SUC_lexicon_tags_25 },
    { 0xbcdabaef, SUC_lexicon_tags_2 },
    { 0x563abaf0, SUC_lexicon_tags_25 },
    { 0xd55abaed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1abaf5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89abafb, SUC_lexicon_tags_27 },
    { 0xf1fabafc, SUC_lexicon_tags_34 },
    { 0xd1dabafb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43abb01, SUC_lexicon_tags_17 },
    { 0x4f9abb02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x963abb04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15abb09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x639abb0c, SUC_lexicon_tags_16 },
    { 0x535abb0d, SUC_lexicon_tags_12 },
    { 0x561abb0c, SUC_lexicon_tags_37 },
    { 0xbd3abb0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59abb1a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159abb22, SUC_lexicon_tags_7 },
    { 0xaf9abb22, SUC_lexicon_tags_2 },
    { 0xfa7abb24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423abb29, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2d9abb2b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4edabb2d, SUC_lexicon_tags_43 },
    { 0x217abb2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53abb31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d3abb33, SUC_lexicon_tags_25 },
    { 0x391abb34, SUC_lexicon_tags_87 },
    { 0xe11abb35, SUC_lexicon_tags_31 },
    { 0xb19abb36, SUC_lexicon_tags_11 },
    { 0xf1babb35, SUC_lexicon_tags_25 },
    { 0x77babb38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0babb3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7abb3e, SUC_lexicon_tags_7 },
    { 0x2bbabb3e, SUC_lexicon_tags_7 },
    { 0x8dabb3e, SUC_lexicon_tags_55 },
    { 0x585abb41, SUC_lexicon_tags_37 },
    { 0xf67abb41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23fabb44, SUC_lexicon_tags_11 },
    { 0x4bbabb45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7dbabb47, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x633abb49, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdabb4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x29fabb51, SUC_lexicon_tags_17 },
    { 0x881abb52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abb57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69abb5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x367abb5e, SUC_lexicon_tags_14 },
    { 0xd03abb5f, SUC_lexicon_tags_25 },
    { 0x9d5abb60, SUC_lexicon_tags_48 },
    { 0x763abb61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdabb64, SUC_lexicon_tags_170 },
    { 0x4dabb65, SUC_lexicon_tags_234 },
    { 0xab1abb66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfabb69, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3dbabb6b, SUC_lexicon_tags_7 },
    { 0x37dabb6b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x65abb6f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3abb77, SUC_lexicon_tags_37 },
    { 0xdfabb78, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9abb7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fabb81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fabb8a, SUC_lexicon_tags_6 },
    { 0xc3dabb8a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521abb8f, SUC_lexicon_tags_49 },
    { 0x701abb90, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x293abb92, SUC_lexicon_tags_17 },
    { 0x6b7abb93, SUC_lexicon_tags_27 },
    { 0x3e1abb92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77abb9c, SUC_lexicon_tags_48 },
    { 0x193abb9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591abba7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdfabba9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03abbae, SUC_lexicon_tags_0 },
    { 0x6d9abbaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbabbb2, SUC_lexicon_tags_17 },
    { 0x34fabbb3, SUC_lexicon_tags_11 },
    { 0xbebabbb3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x747abbb7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5abbc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc3dabbc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xff5abbc4, SUC_lexicon_tags_37 },
    { 0x2a7abbc5, SUC_lexicon_tags_60 },
    { 0xb77abbc5, SUC_lexicon_tags_2 },
    { 0xeffabbc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbabbca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51abbcd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1afabbcf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53abbd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fabbd5, SUC_lexicon_tags_1 },
    { 0xc49abbd6, SUC_lexicon_tags_215 },
    { 0x3afabbd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadabbe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28babbe4, SUC_lexicon_tags_12 },
    { 0xa4dabbe5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x887abbe7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x257abbe9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7dabbeb, SUC_lexicon_tags_1 },
    { 0x7efabbec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa65abbee, SUC_lexicon_tags_28 },
    { 0x95abbef, SUC_lexicon_tags_30 },
    { 0xe4dabbef, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x341abbf3, SUC_lexicon_tags_13 },
    { 0xbe9abbf3, SUC_lexicon_tags_6 },
    { 0x201abbf5, SUC_lexicon_tags_47 },
    { 0xc17abbf4, SUC_lexicon_tags_30 },
    { 0x82babbf7, SUC_lexicon_tags_16 },
    { 0x94fabbf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fabbfb, SUC_lexicon_tags_17 },
    { 0x6edabbfb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9abc00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681abc04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7abc08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xba9abc0a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x301abc0d, SUC_lexicon_tags_148 },
    { 0x229abc0e, SUC_lexicon_tags_25 },
    { 0x7ababc0e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737abc13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41dabc15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5abc18, SUC_lexicon_tags_60 },
    { 0xa45abc18, SUC_lexicon_tags_34 },
    { 0xbd1abc18, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681abc24, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823abc29, SUC_lexicon_tags_1 },
    { 0x17fabc29, SUC_lexicon_tags_11 },
    { 0x725abc29, SUC_lexicon_tags_82 },
    { 0x42fabc29, SUC_lexicon_tags_85 },
    { 0x8d5abc2d, SUC_lexicon_tags_23 },
    { 0x63babc2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fabc33, SUC_lexicon_tags_25 },
    { 0x447abc34, SUC_lexicon_tags_7 },
    { 0x2c7abc34, SUC_lexicon_tags_45 },
    { 0x31dabc33, SUC_lexicon_tags_15 },
    { 0x17abc33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81abc3a, SUC_lexicon_tags_23 },
    { 0x35babc3a, SUC_lexicon_tags_61 },
    { 0xcb7abc3b, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505abc43, SUC_lexicon_tags_8 },
    { 0x3b3abc44, SUC_lexicon_tags_17 },
    { 0x8ddabc45, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8dbabc47, SUC_lexicon_tags_27 },
    { 0x905abc48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21abc50, SUC_lexicon_tags_40 },
    { 0xb47abc50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cfabc53, SUC_lexicon_tags_43 },
    { 0x203abc54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x423abc56, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x7b7abc58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9abc5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x95dabc5e, SUC_lexicon_tags_37 },
    { 0xddfabc5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1abc63, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9abc66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacfabc68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1abc6c, SUC_lexicon_tags_28 },
    { 0xe4fabc6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x307abc70, SUC_lexicon_tags_15 },
    { 0x3cbabc71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35dabc74, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3abc78, SUC_lexicon_tags_63 },
    { 0xa8babc79, SUC_lexicon_tags_12 },
    { 0x8e1abc7a, SUC_lexicon_tags_19 },
    { 0x3afabc7b, SUC_lexicon_tags_31 },
    { 0x38babc7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55abc87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99abc8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a7abc8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99abc92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50babc96, SUC_lexicon_tags_11 },
    { 0x617abc96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dabc9e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x801abca0, SUC_lexicon_tags_7 },
    { 0x205abca1, SUC_lexicon_tags_27 },
    { 0x867abca2, SUC_lexicon_tags_48 },
    { 0x165abca1, SUC_lexicon_tags_45 },
    { 0x707abca0, SUC_lexicon_tags_57 },
    { 0x52dabca3, SUC_lexicon_tags_11 },
    { 0x859abca6, SUC_lexicon_tags_2 },
    { 0xa83abca4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x349abca9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dabcaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dabcb3, SUC_lexicon_tags_69 },
    { 0xf71abcb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98babcb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fabcbb, SUC_lexicon_tags_37 },
    { 0x7b7abcbc, SUC_lexicon_tags_3 },
    { 0xf61abcbd, SUC_lexicon_tags_43 },
    { 0x669abcbd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb95abcc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7fdabcc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfebabcc4, SUC_lexicon_tags_43 },
    { 0xcb3abcc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x657abcc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12babcc9, SUC_lexicon_tags_14 },
    { 0x435abcc9, SUC_lexicon_tags_25 },
    { 0xd5dabcc9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fabccf, SUC_lexicon_tags_2 },
    { 0x30babcd0, SUC_lexicon_tags_49 },
    { 0xef7abcd0, SUC_lexicon_tags_25 },
    { 0xe0dabcd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeafabcd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1abcd9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb87abcdb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5abce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbabce7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3abced, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefdabcef, SUC_lexicon_tags_11 },
    { 0x4d7abcf0, SUC_lexicon_tags_15 },
    { 0x1f1abcef, SUC_lexicon_tags_17 },
    { 0x5e3abcf2, SUC_lexicon_tags_10 },
    { 0x83fabcf1, SUC_lexicon_tags_47 },
    { 0xc6dabcf2, SUC_lexicon_tags_56 },
    { 0xd97abcf1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x711abcf7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9babcfa, SUC_lexicon_tags_49 },
    { 0x35abcfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x92babcfd, SUC_lexicon_tags_17 },
    { 0x7b7abcfd, SUC_lexicon_tags_37 },
    { 0x521abcff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdddabd01, SUC_lexicon_tags_17 },
    { 0x215abd02, SUC_lexicon_tags_32 },
    { 0x6f1abd03, SUC_lexicon_tags_66 },
    { 0xbefabd02, SUC_lexicon_tags_43 },
    { 0x277abd02, SUC_lexicon_tags_1378 },
    { 0x9f1abd04, SUC_lexicon_tags_9 },
    { 0x5d9abd07, SUC_lexicon_tags_7 },
    { 0x90dabd07, SUC_lexicon_tags_15 },
    { 0xd29abd09, SUC_lexicon_tags_28 },
    { 0xb45abd0a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27babd0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185abd13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe37abd15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5abd19, SUC_lexicon_tags_23 },
    { 0x751abd1a, SUC_lexicon_tags_37 },
    { 0xe8dabd1b, SUC_lexicon_tags_17 },
    { 0x55babd19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd87abd1e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd61abd20, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xad3abd22, SUC_lexicon_tags_40 },
    { 0x4d1abd23, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdabd28, SUC_lexicon_tags_7 },
    { 0xa83abd28, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcbabd2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf5babd2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9abd30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77abd34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f3abd36, SUC_lexicon_tags_11 },
    { 0x4ffabd36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3abd3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9babd3d, SUC_lexicon_tags_31 },
    { 0x735abd3d, SUC_lexicon_tags_439 },
    { 0x923abd3f, SUC_lexicon_tags_1 },
    { 0x5b5abd40, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15abd43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afabd47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761abd4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a5abd4d, SUC_lexicon_tags_11 },
    { 0x4c5abd4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xac1abd50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fabd58, SUC_lexicon_tags_17 },
    { 0x1d9abd58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919abd5f, SUC_lexicon_tags_25 },
    { 0xd59abd60, SUC_lexicon_tags_37 },
    { 0x1b9abd61, SUC_lexicon_tags_12 },
    { 0x8ddabd60, SUC_lexicon_tags_25 },
    { 0x85fabd63, SUC_lexicon_tags_1 },
    { 0x203abd61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81abd6b, SUC_lexicon_tags_30 },
    { 0xff3abd6c, SUC_lexicon_tags_31 },
    { 0xafdabd6d, SUC_lexicon_tags_37 },
    { 0x961abd6e, SUC_lexicon_tags_1 },
    { 0x137abd6f, SUC_lexicon_tags_15 },
    { 0xe19abd70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fabd74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591abd79, SUC_lexicon_tags_25 },
    { 0xe13abd79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337abd7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x869abd81, SUC_lexicon_tags_26 },
    { 0x473abd82, SUC_lexicon_tags_47 },
    { 0xc4dabd83, SUC_lexicon_tags_28 },
    { 0xc5abd84, SUC_lexicon_tags_37 },
    { 0x597abd84, SUC_lexicon_tags_43 },
    { 0x871abd86, SUC_lexicon_tags_21 },
    { 0xd3abd85, SUC_lexicon_tags_38 },
    { 0xbd1abd88, SUC_lexicon_tags_11 },
    { 0xe53abd88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495abd8e, SUC_lexicon_tags_30 },
    { 0x957abd8f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc95abd91, SUC_lexicon_tags_30 },
    { 0x6efabd92, SUC_lexicon_tags_33 },
    { 0xdd1abd91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3abda5, SUC_lexicon_tags_37 },
    { 0x33fabda5, SUC_lexicon_tags_29 },
    { 0x35babda6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x943abda9, SUC_lexicon_tags_33 },
    { 0x193abda9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1abdad, SUC_lexicon_tags_11 },
    { 0x28dabdad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855abdb2, SUC_lexicon_tags_9 },
    { 0x607abdb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9abdb6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x36fabdb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x239abdbb, SUC_lexicon_tags_2 },
    { 0x6afabdbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd63abdbe, SUC_lexicon_tags_15 },
    { 0x77dabdbf, SUC_lexicon_tags_11 },
    { 0xe97abdc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeadabdc2, SUC_lexicon_tags_27 },
    { 0xebbabdc3, SUC_lexicon_tags_60 },
    { 0x3e9abdc2, SUC_lexicon_tags_37 },
    { 0x337abdc5, SUC_lexicon_tags_16 },
    { 0xfe3abdc5, SUC_lexicon_tags_12 },
    { 0x61babdc5, SUC_lexicon_tags_25 },
    { 0x7b1abdc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29abdcb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33babdd7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409abde0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x397abde3, SUC_lexicon_tags_88 },
    { 0xbb7abde4, SUC_lexicon_tags_54 },
    { 0x55dabde3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x13babde7, SUC_lexicon_tags_15 },
    { 0x79dabde8, SUC_lexicon_tags_60 },
    { 0x1a5abde9, SUC_lexicon_tags_8 },
    { 0x453abdea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd49abdec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27abdf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9fabdfb, SUC_lexicon_tags_17 },
    { 0x6f7abdfb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35abe00, SUC_lexicon_tags_1 },
    { 0x8f1abe00, SUC_lexicon_tags_25 },
    { 0x905abe02, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61abe05, SUC_lexicon_tags_9 },
    { 0x4b1abe06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7abe0b, SUC_lexicon_tags_17 },
    { 0x849abe0c, SUC_lexicon_tags_37 },
    { 0xbbbabe0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efabe14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dabe19, SUC_lexicon_tags_37 },
    { 0xf51abe1a, SUC_lexicon_tags_282 },
    { 0xf9abe1b, SUC_lexicon_tags_27 },
    { 0x799abe1a, SUC_lexicon_tags_2 },
    { 0x3b3abe1d, SUC_lexicon_tags_23 },
    { 0xaa7abe1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85abe21, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5abe27, SUC_lexicon_tags_26 },
    { 0x9dfabe27, SUC_lexicon_tags_27 },
    { 0xb03abe29, SUC_lexicon_tags_2 },
    { 0xae5abe29, SUC_lexicon_tags_17 },
    { 0x41abe2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x551abe2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91abe31, SUC_lexicon_tags_36 },
    { 0x8babe31, SUC_lexicon_tags_17 },
    { 0x8fdabe33, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3abe38, SUC_lexicon_tags_33 },
    { 0xdefabe38, SUC_lexicon_tags_49 },
    { 0x8ffabe3a, SUC_lexicon_tags_77 },
    { 0x8c7abe3a, SUC_lexicon_tags_8 },
    { 0xe3babe3c, SUC_lexicon_tags_15 },
    { 0xad3abe3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x173abe40, SUC_lexicon_tags_25 },
    { 0xe3abe41, SUC_lexicon_tags_0 },
    { 0x431abe42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fabe47, SUC_lexicon_tags_46 },
    { 0x999abe48, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5e9abe4a, SUC_lexicon_tags_3 },
    { 0x9e9abe4b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3a7abe4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95abe50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1abe55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x237abe58, SUC_lexicon_tags_2 },
    { 0x3b7abe58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1abe5f, SUC_lexicon_tags_12 },
    { 0xb17abe60, SUC_lexicon_tags_2 },
    { 0xdddabe61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83abe65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39abe69, SUC_lexicon_tags_35 },
    { 0xb5babe6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb7abe6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5abe6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9abe74, SUC_lexicon_tags_31 },
    { 0x64babe74, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69abe78, SUC_lexicon_tags_48 },
    { 0x465abe78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x87abe7c, SUC_lexicon_tags_47 },
    { 0x4b7abe7d, SUC_lexicon_tags_7 },
    { 0x3c1abe7d, SUC_lexicon_tags_31 },
    { 0xc27abe7d, SUC_lexicon_tags_17 },
    { 0x79babe7f, SUC_lexicon_tags_43 },
    { 0x8bbabe81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dabe89, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa1dabe8b, SUC_lexicon_tags_46 },
    { 0xa0dabe8c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589abe91, SUC_lexicon_tags_11 },
    { 0xa8babe92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfabe9d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x42babe9f, SUC_lexicon_tags_1 },
    { 0xfe9abea0, SUC_lexicon_tags_15 },
    { 0xa0babea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45abea5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99abeac, SUC_lexicon_tags_25 },
    { 0x937abead, SUC_lexicon_tags_21 },
    { 0x773abeae, SUC_lexicon_tags_31 },
    { 0x6d9abeaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x505abeb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0babeb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x793abeb8, SUC_lexicon_tags_45 },
    { 0x91abeb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18fabebb, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7abec3, SUC_lexicon_tags_25 },
    { 0xfffabec3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x643abec6, SUC_lexicon_tags_60 },
    { 0x2c1abec7, SUC_lexicon_tags_70 },
    { 0x6a3abec7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfabecb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187abed2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c5abed4, SUC_lexicon_tags_31 },
    { 0x2d1abed5, SUC_lexicon_tags_12 },
    { 0xc8dabed5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffabed9, SUC_lexicon_tags_9 },
    { 0x9d3abed9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5abede, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23abee1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf31abee3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x715abee6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x469abee9, SUC_lexicon_tags_27 },
    { 0xddabeea, SUC_lexicon_tags_7 },
    { 0x477abeea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf65abeee, SUC_lexicon_tags_2 },
    { 0x2ffabeef, SUC_lexicon_tags_37 },
    { 0xecbabeee, SUC_lexicon_tags_21 },
    { 0x84babef1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf0dabef3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21abef6, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x9d9abef8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe13abefa, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabefe, SUC_lexicon_tags_15 },
    { 0x6c7abeff, SUC_lexicon_tags_2 },
    { 0xeadabf00, SUC_lexicon_tags_27 },
    { 0xf4dabf01, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc3dabf03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xefbabf05, SUC_lexicon_tags_0 },
    { 0xd1dabf06, SUC_lexicon_tags_11 },
    { 0xdddabf07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x775abf0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9abf0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abf12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8fabf14, SUC_lexicon_tags_28 },
    { 0x7ababf15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1babf17, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529abf1b, SUC_lexicon_tags_23 },
    { 0xee1abf1c, SUC_lexicon_tags_15 },
    { 0x38fabf1d, SUC_lexicon_tags_23 },
    { 0x35babf1c, SUC_lexicon_tags_25 },
    { 0x1dfabf1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dabf22, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x379abf25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67abf2d, SUC_lexicon_tags_25 },
    { 0xa11abf2d, SUC_lexicon_tags_17 },
    { 0xaafabf2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dabf34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bbabf36, SUC_lexicon_tags_34 },
    { 0xc93abf36, SUC_lexicon_tags_15 },
    { 0xd0fabf38, SUC_lexicon_tags_63 },
    { 0xc1fabf39, SUC_lexicon_tags_11 },
    { 0xff3abf3a, SUC_lexicon_tags_43 },
    { 0x9b3abf3a, SUC_lexicon_tags_60 },
    { 0x623abf3c, SUC_lexicon_tags_37 },
    { 0x357abf3a, SUC_lexicon_tags_11 },
    { 0x4dfabf3c, SUC_lexicon_tags_27 },
    { 0x163abf3a, SUC_lexicon_tags_15 },
    { 0xd13abf40, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87abf45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x599abf47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c7abf49, SUC_lexicon_tags_25 },
    { 0x717abf49, SUC_lexicon_tags_9 },
    { 0xe3babf49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dabf50, SUC_lexicon_tags_21 },
    { 0xa6babf51, SUC_lexicon_tags_74 },
    { 0xf43abf51, SUC_lexicon_tags_11 },
    { 0x935abf53, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131abf62, SUC_lexicon_tags_37 },
    { 0xc4babf62, SUC_lexicon_tags_2 },
    { 0x1c3abf63, SUC_lexicon_tags_37 },
    { 0x5cbabf62, SUC_lexicon_tags_47 },
    { 0xe05abf66, SUC_lexicon_tags_30 },
    { 0x237abf67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85abf6b, SUC_lexicon_tags_12 },
    { 0x4b7abf6c, SUC_lexicon_tags_9 },
    { 0x2b1abf6d, SUC_lexicon_tags_11 },
    { 0x993abf6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x521abf70, SUC_lexicon_tags_49 },
    { 0xff1abf71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x687abf73, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1babf76, SUC_lexicon_tags_17 },
    { 0x93dabf77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf9abf79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x101abf7b, SUC_lexicon_tags_14 },
    { 0x8fabf7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8babf81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9abf85, SUC_lexicon_tags_37 },
    { 0x65abf86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x911abf88, SUC_lexicon_tags_0 },
    { 0x253abf89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ababf8c, SUC_lexicon_tags_23 },
    { 0xffdabf8c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ffabf8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53abf94, SUC_lexicon_tags_4 },
    { 0x5c7abf95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9babf9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa3dabf9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18babfa1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67abfa4, SUC_lexicon_tags_39 },
    { 0x985abfa4, SUC_lexicon_tags_1 },
    { 0x3c7abfa6, SUC_lexicon_tags_8 },
    { 0xd7fabfa6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x8fabfa9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xebabfab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24babfae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51abfb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdabfb7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xccbabfb9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3abfbc, SUC_lexicon_tags_12 },
    { 0x8afabfbd, SUC_lexicon_tags_8 },
    { 0xc0babfbc, SUC_lexicon_tags_47 },
    { 0xf67abfbe, SUC_lexicon_tags_29 },
    { 0x7d7abfbd, SUC_lexicon_tags_31 },
    { 0x1c7abfc1, SUC_lexicon_tags_33 },
    { 0xe95abfbe, SUC_lexicon_tags_1 },
    { 0xfb7abfbf, SUC_lexicon_tags_1 },
    { 0x5cdabfbf, SUC_lexicon_tags_82 },
    { 0xf3abfc0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x17babfc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433abfce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1abfd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf5abfd3, SUC_lexicon_tags_11 },
    { 0xbbfabfd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2dabfd6, SUC_lexicon_tags_37 },
    { 0xb3abfd6, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x47fabfd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7abfdb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fabfdf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x39babfe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x51abfe4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9babfe6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba1abfe8, SUC_lexicon_tags_45 },
    { 0x31babfe8, SUC_lexicon_tags_37 },
    { 0xef5abfea, SUC_lexicon_tags_46 },
    { 0x40fabfe8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddabfef, SUC_lexicon_tags_39 },
    { 0x797abff0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5abff7, SUC_lexicon_tags_38 },
    { 0x59dabff8, SUC_lexicon_tags_7 },
    { 0xbcfabff7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9abffe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33ac003, SUC_lexicon_tags_85 },
    { 0xbc5ac003, SUC_lexicon_tags_0 },
    { 0x305ac004, SUC_lexicon_tags_27 },
    { 0x755ac003, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967ac00a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29bac00c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x679ac00f, SUC_lexicon_tags_12 },
    { 0xd43ac010, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fac017, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabac01e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97ac021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9ac024, SUC_lexicon_tags_21 },
    { 0xc9bac025, SUC_lexicon_tags_1 },
    { 0x621ac024, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83ac02d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ac032, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211ac037, SUC_lexicon_tags_14 },
    { 0x3e5ac038, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x899ac03a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887ac044, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce9ac046, SUC_lexicon_tags_19 },
    { 0x5adac047, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x18bac049, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca7ac04b, SUC_lexicon_tags_7 },
    { 0x4b3ac04b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bac052, SUC_lexicon_tags_15 },
    { 0x36bac053, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887ac05a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d3ac05c, SUC_lexicon_tags_11 },
    { 0xe39ac05d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fac061, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0xe4dac063, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67ac069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bac06d, SUC_lexicon_tags_1 },
    { 0xe6fac06e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51ac071, SUC_lexicon_tags_60 },
    { 0xcd3ac071, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41dac074, SUC_lexicon_tags_64 },
    { 0x701ac075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fac07a, SUC_lexicon_tags_12 },
    { 0xbdfac07b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495ac080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x98dac083, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x60dac086, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaafac088, SUC_lexicon_tags_47 },
    { 0x74bac088, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7ac08c, SUC_lexicon_tags_23 },
    { 0xed7ac08d, SUC_lexicon_tags_7 },
    { 0x60dac08c, SUC_lexicon_tags_19 },
    { 0x4b7ac08c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bac094, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x7a7ac096, SUC_lexicon_tags_1 },
    { 0x67fac097, SUC_lexicon_tags_2 },
    { 0x975ac098, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1ac09b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447ac0a1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x33ac0a3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdac0a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8e1ac0a8, SUC_lexicon_tags_17 },
    { 0x295ac0a9, SUC_lexicon_tags_19 },
    { 0xa77ac0aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bac0af, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x563ac0b1, SUC_lexicon_tags_19 },
    { 0xfafac0b2, SUC_lexicon_tags_17 },
    { 0x8d3ac0b3, SUC_lexicon_tags_2 },
    { 0x87bac0b4, SUC_lexicon_tags_66 },
    { 0x59ac0b4, SUC_lexicon_tags_262 },
    { 0xf43ac0b6, SUC_lexicon_tags_47 },
    { 0x901ac0b6, SUC_lexicon_tags_11 },
    { 0xa87ac0b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x737ac0ba, SUC_lexicon_tags_17 },
    { 0x653ac0ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501ac0c0, SUC_lexicon_tags_23 },
    { 0xaa3ac0c1, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xff7ac0c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x737ac0c5, SUC_lexicon_tags_35 },
    { 0x3ddac0c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfac0ca, SUC_lexicon_tags_17 },
    { 0xacbac0cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bac0d0, SUC_lexicon_tags_57 },
    { 0x4a5ac0d1, SUC_lexicon_tags_7 },
    { 0xbd7ac0d1, SUC_lexicon_tags_31 },
    { 0x9a3ac0d3, SUC_lexicon_tags_39 },
    { 0xeb9ac0d4, SUC_lexicon_tags_25 },
    { 0x705ac0d2, SUC_lexicon_tags_61 },
    { 0x14bac0d6, SUC_lexicon_tags_47 },
    { 0x2ddac0d6, SUC_lexicon_tags_53 },
    { 0x461ac0d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ffac0da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebac0dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc77ac0df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03ac0e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85ac0e9, SUC_lexicon_tags_23 },
    { 0x1e7ac0ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3ac0ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddac0f1, SUC_lexicon_tags_17 },
    { 0xf75ac0f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295ac0f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bac0fb, SUC_lexicon_tags_425 },
    { 0xb7dac0fc, SUC_lexicon_tags_28 },
    { 0x9d5ac0fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f5ac0ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325ac104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dac10a, SUC_lexicon_tags_17 },
    { 0x5a5ac10b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dac10f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35ac114, SUC_lexicon_tags_37 },
    { 0x51fac114, SUC_lexicon_tags_12 },
    { 0x941ac114, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d1ac118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc79ac11a, SUC_lexicon_tags_15 },
    { 0x28fac11a, SUC_lexicon_tags_12 },
    { 0x10fac11c, SUC_lexicon_tags_754 },
    { 0xd47ac11c, SUC_lexicon_tags_4 },
    { 0x62dac11a, SUC_lexicon_tags_7 },
    { 0x189ac11c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8bdac121, SUC_lexicon_tags_49 },
    { 0x437ac121, SUC_lexicon_tags_37 },
    { 0x481ac122, SUC_lexicon_tags_8 },
    { 0xb8bac124, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x273ac127, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659ac12e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x729ac130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbac133, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdac138, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5ac13b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35ac13f, SUC_lexicon_tags_60 },
    { 0xe69ac140, SUC_lexicon_tags_37 },
    { 0x205ac141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41ac146, SUC_lexicon_tags_11 },
    { 0xfc7ac147, SUC_lexicon_tags_12 },
    { 0xcc3ac148, SUC_lexicon_tags_37 },
    { 0xe1bac149, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83bac14c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd67ac14e, SUC_lexicon_tags_7 },
    { 0x427ac14e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9ac152, SUC_lexicon_tags_7 },
    { 0x7c5ac153, SUC_lexicon_tags_0 },
    { 0xc5fac154, SUC_lexicon_tags_13 },
    { 0xea1ac154, SUC_lexicon_tags_26 },
    { 0xd7fac156, SUC_lexicon_tags_12 },
    { 0xb2bac157, SUC_lexicon_tags_45 },
    { 0xef9ac156, SUC_lexicon_tags_114 },
    { 0x671ac155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25ac15d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x425ac15f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5ac165, SUC_lexicon_tags_7 },
    { 0x16fac166, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfac169, SUC_lexicon_tags_7 },
    { 0x9ffac16a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xae1ac16d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbac171, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ac175, SUC_lexicon_tags_17 },
    { 0x411ac176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31ac179, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5dfac17b, SUC_lexicon_tags_25 },
    { 0xdbac17c, SUC_lexicon_tags_64 },
    { 0x45fac17d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ac182, SUC_lexicon_tags_39 },
    { 0xaedac183, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x301ac185, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71dac187, SUC_lexicon_tags_17 },
    { 0xbcfac187, SUC_lexicon_tags_39 },
    { 0xab9ac188, SUC_lexicon_tags_69 },
    { 0x76dac188, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9ac18d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dac194, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9ac19a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xeedac19c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bac1a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9ac1a8, SUC_lexicon_tags_25 },
    { 0x243ac1a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fac1b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07ac1b6, SUC_lexicon_tags_113 },
    { 0x717ac1b7, SUC_lexicon_tags_60 },
    { 0xe71ac1b7, SUC_lexicon_tags_55 },
    { 0xea9ac1b9, SUC_lexicon_tags_1 },
    { 0x2f5ac1b7, SUC_lexicon_tags_47 },
    { 0xfcbac1b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50dac1bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bac1c2, SUC_lexicon_tags_15 },
    { 0x193ac1c2, SUC_lexicon_tags_19 },
    { 0x6fdac1c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefac1c8, SUC_lexicon_tags_47 },
    { 0x137ac1c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee5ac1cb, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dac1cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f3ac1d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x107ac1d3, SUC_lexicon_tags_87 },
    { 0x52fac1d4, SUC_lexicon_tags_49 },
    { 0x821ac1d5, SUC_lexicon_tags_7 },
    { 0x36dac1d4, SUC_lexicon_tags_37 },
    { 0x96dac1d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17bac1d9, SUC_lexicon_tags_95 },
    { 0xea3ac1da, SUC_lexicon_tags_45 },
    { 0xc1dac1db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x37dac1de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5ac1e2, SUC_lexicon_tags_1 },
    { 0x5adac1e3, SUC_lexicon_tags_35 },
    { 0x3c5ac1e3, SUC_lexicon_tags_66 },
    { 0x451ac1e3, SUC_lexicon_tags_45 },
    { 0x407ac1e6, SUC_lexicon_tags_17 },
    { 0x439ac1e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9ac1ea, SUC_lexicon_tags_37 },
    { 0x4a3ac1eb, SUC_lexicon_tags_25 },
    { 0x3abac1ea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ac1f0, SUC_lexicon_tags_37 },
    { 0x411ac1f1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afac1f7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf85ac1f9, SUC_lexicon_tags_29 },
    { 0x895ac1fa, SUC_lexicon_tags_11 },
    { 0x891ac1fb, SUC_lexicon_tags_4 },
    { 0x795ac1fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efac200, SUC_lexicon_tags_12 },
    { 0xd4fac201, SUC_lexicon_tags_17 },
    { 0x68bac200, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391ac208, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987ac20c, SUC_lexicon_tags_7 },
    { 0x9a5ac20d, SUC_lexicon_tags_39 },
    { 0x323ac20c, SUC_lexicon_tags_12 },
    { 0xc33ac20d, SUC_lexicon_tags_40 },
    { 0xb6dac210, SUC_lexicon_tags_0 },
    { 0xfb5ac20d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3ac215, SUC_lexicon_tags_17 },
    { 0xf7bac216, SUC_lexicon_tags_17 },
    { 0x79ac217, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73ac21b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1ac21f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9ac223, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2bac225, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5ac229, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479ac22e, SUC_lexicon_tags_1 },
    { 0x317ac22f, SUC_lexicon_tags_11 },
    { 0x889ac22e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbac234, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425ac238, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fac23b, SUC_lexicon_tags_37 },
    { 0x49fac23c, SUC_lexicon_tags_7 },
    { 0x6fdac23d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5efac23f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbbac241, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bac244, SUC_lexicon_tags_4 },
    { 0xe07ac245, SUC_lexicon_tags_1 },
    { 0x3a3ac246, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fac24b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x517ac24d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34bac250, SUC_lexicon_tags_27 },
    { 0xeffac251, SUC_lexicon_tags_12 },
    { 0xde9ac250, SUC_lexicon_tags_45 },
    { 0x1e5ac253, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc07ac255, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x88fac257, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x837ac259, SUC_lexicon_tags_15 },
    { 0x47bac259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1ac25d, SUC_lexicon_tags_46 },
    { 0x85ac25e, SUC_lexicon_tags_17 },
    { 0x12dac25e, SUC_lexicon_tags_87 },
    { 0x4bdac25e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ac264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dac26e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x747ac271, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58fac273, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe77ac275, SUC_lexicon_tags_15 },
    { 0xc79ac276, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x191ac278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbfac27a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5ac27e, SUC_lexicon_tags_2 },
    { 0x2a3ac27f, SUC_lexicon_tags_30 },
    { 0x3afac280, SUC_lexicon_tags_49 },
    { 0x713ac27f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07ac285, SUC_lexicon_tags_12 },
    { 0xdcfac285, SUC_lexicon_tags_37 },
    { 0x991ac287, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bac28d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9adac28f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9ac297, SUC_lexicon_tags_45 },
    { 0x147ac298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb3dac29a, SUC_lexicon_tags_138 },
    { 0x5e1ac29a, SUC_lexicon_tags_25 },
    { 0xebbac29c, SUC_lexicon_tags_37 },
    { 0xb81ac29a, SUC_lexicon_tags_2 },
    { 0x2efac29e, SUC_lexicon_tags_33 },
    { 0xad7ac29f, SUC_lexicon_tags_29 },
    { 0x687ac2a0, SUC_lexicon_tags_6 },
    { 0x9c1ac2a1, SUC_lexicon_tags_37 },
    { 0x461ac2a2, SUC_lexicon_tags_14 },
    { 0x321ac2a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dac2a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367ac2b4, SUC_lexicon_tags_79 },
    { 0xbbac2b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb6dac2b7, SUC_lexicon_tags_46 },
    { 0x1c5ac2b8, SUC_lexicon_tags_25 },
    { 0xb33ac2b8, SUC_lexicon_tags_47 },
    { 0x637ac2ba, SUC_lexicon_tags_23 },
    { 0xebfac2ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ac2c2, SUC_lexicon_tags_29 },
    { 0xc1dac2c3, SUC_lexicon_tags_37 },
    { 0x6e9ac2c3, SUC_lexicon_tags_17 },
    { 0xc83ac2c5, SUC_lexicon_tags_2 },
    { 0xbafac2c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335ac2ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fac2d0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fac2d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ac2d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x409ac2da, SUC_lexicon_tags_38 },
    { 0xfbbac2da, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81ac2e0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x80bac2e2, SUC_lexicon_tags_12 },
    { 0x7c7ac2e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adac2e9, SUC_lexicon_tags_31 },
    { 0x577ac2ea, SUC_lexicon_tags_37 },
    { 0xac5ac2eb, SUC_lexicon_tags_11 },
    { 0x90bac2eb, SUC_lexicon_tags_1 },
    { 0x98fac2ed, SUC_lexicon_tags_38 },
    { 0xcddac2ed, SUC_lexicon_tags_25 },
    { 0x38bac2ef, SUC_lexicon_tags_40 },
    { 0xa63ac2ef, SUC_lexicon_tags_17 },
    { 0xa2fac2ed, SUC_lexicon_tags_0 },
    { 0xf45ac2ea, SUC_lexicon_tags_3 },
    { 0xd9fac2f3, SUC_lexicon_tags_7 },
    { 0x355ac2f4, SUC_lexicon_tags_17 },
    { 0xd2dac2e9, SUC_lexicon_tags_55 },
    { 0x7dbac2f2, SUC_lexicon_tags_104 },
    { 0xc25ac2eb, SUC_lexicon_tags_17 },
    { 0x3e1ac2f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821ac2fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fac302, SUC_lexicon_tags_27 },
    { 0x14fac303, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x789ac306, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x58dac309, SUC_lexicon_tags_7 },
    { 0x471ac309, SUC_lexicon_tags_23 },
    { 0xb19ac309, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1ac30f, SUC_lexicon_tags_0 },
    { 0x565ac30f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbb5ac312, SUC_lexicon_tags_47 },
    { 0xa37ac313, SUC_lexicon_tags_47 },
    { 0x94fac314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489ac318, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c9ac31a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbfac31c, SUC_lexicon_tags_37 },
    { 0xaebac31d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x13fac31f, SUC_lexicon_tags_17 },
    { 0x85fac320, SUC_lexicon_tags_45 },
    { 0x9d5ac31f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dac327, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7ac32b, SUC_lexicon_tags_37 },
    { 0x28fac32b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169ac330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bac334, SUC_lexicon_tags_33 },
    { 0x1b1ac335, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x737ac338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0dac33a, SUC_lexicon_tags_31 },
    { 0xbedac33a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bdac33d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fac340, SUC_lexicon_tags_0 },
    { 0xd2dac340, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x141ac344, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfac347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5ac34a, SUC_lexicon_tags_17 },
    { 0x75dac34a, SUC_lexicon_tags_33 },
    { 0xf5fac34c, SUC_lexicon_tags_21 },
    { 0xd21ac34d, SUC_lexicon_tags_38 },
    { 0x367ac34e, SUC_lexicon_tags_37 },
    { 0x7c1ac34e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779ac354, SUC_lexicon_tags_6 },
    { 0x381ac355, SUC_lexicon_tags_61 },
    { 0xae3ac354, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x407ac359, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1ac35c, SUC_lexicon_tags_28 },
    { 0x97bac35c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe85ac35f, SUC_lexicon_tags_12 },
    { 0x323ac35f, SUC_lexicon_tags_17 },
    { 0xa71ac361, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55ac365, SUC_lexicon_tags_1 },
    { 0x58bac366, SUC_lexicon_tags_2 },
    { 0x165ac366, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507ac36e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x943ac370, SUC_lexicon_tags_2 },
    { 0x297ac370, SUC_lexicon_tags_48 },
    { 0x883ac372, SUC_lexicon_tags_25 },
    { 0xc1ac372, SUC_lexicon_tags_1 },
    { 0x9dfac370, SUC_lexicon_tags_12 },
    { 0xc93ac375, SUC_lexicon_tags_7 },
    { 0x779ac371, SUC_lexicon_tags_26 },
    { 0x941ac372, SUC_lexicon_tags_37 },
    { 0x1afac378, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11ac37e, SUC_lexicon_tags_28 },
    { 0x533ac37f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1ac383, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfddac385, SUC_lexicon_tags_17 },
    { 0x381ac386, SUC_lexicon_tags_25 },
    { 0x479ac385, SUC_lexicon_tags_56 },
    { 0x6e3ac388, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffac38c, SUC_lexicon_tags_2 },
    { 0xf45ac38d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x697ac38f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x821ac391, SUC_lexicon_tags_38 },
    { 0xd25ac391, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x969ac394, SUC_lexicon_tags_19 },
    { 0xf97ac395, SUC_lexicon_tags_17 },
    { 0x865ac394, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abac399, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dac39d, SUC_lexicon_tags_17 },
    { 0xa0fac39e, SUC_lexicon_tags_7 },
    { 0xa7dac39f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2fdac3a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1ac3a5, SUC_lexicon_tags_37 },
    { 0xc9dac3a6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17ac3a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451ac3ad, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x925ac3af, SUC_lexicon_tags_14 },
    { 0x37ac3af, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x187ac3b2, SUC_lexicon_tags_23 },
    { 0x59fac3b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x993ac3b5, SUC_lexicon_tags_25 },
    { 0x51dac3b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa51ac3b8, SUC_lexicon_tags_29 },
    { 0x55ac3b9, SUC_lexicon_tags_12 },
    { 0xfcfac3ba, SUC_lexicon_tags_104 },
    { 0x99bac3bb, SUC_lexicon_tags_2 },
    { 0xdbfac3bb, SUC_lexicon_tags_2 },
    { 0xf85ac3b9, SUC_lexicon_tags_29 },
    { 0x7ddac3be, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89ac3c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63ac3c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fac3ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bfac3cc, SUC_lexicon_tags_25 },
    { 0x55bac3cd, SUC_lexicon_tags_17 },
    { 0x6c9ac3ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd51ac3d0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4b1ac3d2, SUC_lexicon_tags_12 },
    { 0x13fac3d3, SUC_lexicon_tags_25 },
    { 0xfe7ac3d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7ac3d9, SUC_lexicon_tags_12 },
    { 0xffbac3da, SUC_lexicon_tags_2 },
    { 0x85fac3db, SUC_lexicon_tags_66 },
    { 0x483ac3db, SUC_lexicon_tags_37 },
    { 0x87bac3db, SUC_lexicon_tags_8 },
    { 0xaefac3de, SUC_lexicon_tags_19 },
    { 0xf1bac3db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917ac3e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeebac3e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85ac3e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fac3eb, SUC_lexicon_tags_74 },
    { 0x285ac3ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffac3ef, SUC_lexicon_tags_17 },
    { 0x781ac3ef, SUC_lexicon_tags_17 },
    { 0xe3bac3f1, SUC_lexicon_tags_38 },
    { 0x131ac3f2, SUC_lexicon_tags_17 },
    { 0xe3ac3f3, SUC_lexicon_tags_28 },
    { 0xf2bac3f4, SUC_lexicon_tags_17 },
    { 0xc97ac3f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdac3f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675ac3fe, SUC_lexicon_tags_30 },
    { 0x6abac3fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x11ac401, SUC_lexicon_tags_29 },
    { 0xf2dac402, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf15ac404, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bac407, SUC_lexicon_tags_19 },
    { 0x825ac408, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737ac40e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x485ac410, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bac413, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65ac417, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaffac419, SUC_lexicon_tags_29 },
    { 0xe0fac41a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3ac41c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x467ac41e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39ac423, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbac428, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dac42c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x72fac42e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x497ac430, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbac437, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67ac440, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd3bac442, SUC_lexicon_tags_47 },
    { 0x6d1ac442, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x397ac445, SUC_lexicon_tags_60 },
    { 0x7dac446, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569ac44a, SUC_lexicon_tags_45 },
    { 0xd7dac44b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fac451, SUC_lexicon_tags_14 },
    { 0x37ac451, SUC_lexicon_tags_25 },
    { 0x999ac452, SUC_lexicon_tags_17 },
    { 0xa89ac451, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bbac456, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a5ac458, SUC_lexicon_tags_15 },
    { 0xed5ac459, SUC_lexicon_tags_17 },
    { 0x91fac45a, SUC_lexicon_tags_25 },
    { 0x27dac458, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x543ac45d, SUC_lexicon_tags_25 },
    { 0x665ac45e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fac462, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1ac46c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebac46f, SUC_lexicon_tags_37 },
    { 0xa43ac470, SUC_lexicon_tags_17 },
    { 0xb53ac471, SUC_lexicon_tags_23 },
    { 0xc6dac472, SUC_lexicon_tags_7 },
    { 0x327ac471, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf51ac475, SUC_lexicon_tags_25 },
    { 0x6c1ac476, SUC_lexicon_tags_15 },
    { 0x635ac477, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdf5ac479, SUC_lexicon_tags_2 },
    { 0xd05ac479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ac47e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769ac489, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8fac48b, SUC_lexicon_tags_0 },
    { 0xd95ac48c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x96bac48f, SUC_lexicon_tags_15 },
    { 0xdc1ac490, SUC_lexicon_tags_13 },
    { 0x64bac490, SUC_lexicon_tags_7 },
    { 0x5c5ac48f, SUC_lexicon_tags_27 },
    { 0x601ac491, SUC_lexicon_tags_56 },
    { 0xc95ac494, SUC_lexicon_tags_17 },
    { 0x511ac495, SUC_lexicon_tags_65 },
    { 0xd21ac495, SUC_lexicon_tags_7 },
    { 0xcabac491, SUC_lexicon_tags_7 },
    { 0x271ac490, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb49ac49b, SUC_lexicon_tags_15 },
    { 0x7d7ac49b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bac49f, SUC_lexicon_tags_7 },
    { 0xb8fac4a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa8bac4a2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xed5ac4a4, SUC_lexicon_tags_336 },
    { 0x51ac4a5, SUC_lexicon_tags_38 },
    { 0x983ac4a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fac4aa, SUC_lexicon_tags_25 },
    { 0x217ac4ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x509ac4ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a3ac4af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63ac4b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeffac4b4, SUC_lexicon_tags_32 },
    { 0xbf3ac4b5, SUC_lexicon_tags_49 },
    { 0x595ac4b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edac4ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc1ac4bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfac4c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb9ac4c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9ac4cc, SUC_lexicon_tags_12 },
    { 0x273ac4cd, SUC_lexicon_tags_33 },
    { 0x81fac4ce, SUC_lexicon_tags_17 },
    { 0x465ac4cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135ac4d7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x2adac4da, SUC_lexicon_tags_160 },
    { 0x853ac4db, SUC_lexicon_tags_1 },
    { 0xcd9ac4dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x70dac4de, SUC_lexicon_tags_48 },
    { 0x255ac4df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597ac4e5, SUC_lexicon_tags_48 },
    { 0xb1bac4e6, SUC_lexicon_tags_61 },
    { 0x655ac4e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81ac4ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463ac4f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fac4fb, SUC_lexicon_tags_39 },
    { 0x661ac4fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x96dac4ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fac502, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19ac507, SUC_lexicon_tags_37 },
    { 0x79fac508, SUC_lexicon_tags_17 },
    { 0xe8bac507, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xda5ac50b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5ac50f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169ac513, SUC_lexicon_tags_1 },
    { 0xca3ac514, SUC_lexicon_tags_12 },
    { 0x89dac514, SUC_lexicon_tags_14 },
    { 0xa73ac516, SUC_lexicon_tags_30 },
    { 0xbe5ac517, SUC_lexicon_tags_46 },
    { 0x98dac516, SUC_lexicon_tags_30 },
    { 0x25fac519, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451ac524, SUC_lexicon_tags_1 },
    { 0x6b9ac525, SUC_lexicon_tags_17 },
    { 0xf9fac525, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x477ac528, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5ac531, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fac534, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x765ac536, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b5ac538, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57ac53c, SUC_lexicon_tags_9 },
    { 0x93fac53d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d9ac53f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bac543, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ac547, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103ac54b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555ac550, SUC_lexicon_tags_23 },
    { 0xa83ac551, SUC_lexicon_tags_37 },
    { 0x2e1ac552, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbac555, SUC_lexicon_tags_11 },
    { 0x3a9ac556, SUC_lexicon_tags_7 },
    { 0xb61ac556, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3ac55b, SUC_lexicon_tags_65 },
    { 0x611ac55c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6cbac55e, SUC_lexicon_tags_17 },
    { 0xf11ac55f, SUC_lexicon_tags_11 },
    { 0x99dac560, SUC_lexicon_tags_12 },
    { 0xc95ac560, SUC_lexicon_tags_15 },
    { 0xedac560, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaddac565, SUC_lexicon_tags_37 },
    { 0x79dac566, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x831ac568, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbac56b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1ac573, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639ac579, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bac57c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fac58a, SUC_lexicon_tags_14 },
    { 0x71bac58b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x335ac58d, SUC_lexicon_tags_12 },
    { 0xcbac58d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545ac594, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x315ac596, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x139ac599, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65ac59f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3ac5a3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edac5a7, SUC_lexicon_tags_38 },
    { 0x5b9ac5a7, SUC_lexicon_tags_49 },
    { 0x2fbac5a9, SUC_lexicon_tags_46 },
    { 0x9cdac5aa, SUC_lexicon_tags_15 },
    { 0x5d1ac5ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71ac5af, SUC_lexicon_tags_59 },
    { 0xe03ac5b0, SUC_lexicon_tags_63 },
    { 0xfa7ac5b0, SUC_lexicon_tags_2 },
    { 0x45bac5b2, SUC_lexicon_tags_17 },
    { 0xb83ac5b2, SUC_lexicon_tags_7 },
    { 0x989ac5b4, SUC_lexicon_tags_14 },
    { 0x351ac5b4, SUC_lexicon_tags_6 },
    { 0x63fac5b6, SUC_lexicon_tags_15 },
    { 0xe2fac5b6, SUC_lexicon_tags_28 },
    { 0xdb3ac5b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x20dac5ba, SUC_lexicon_tags_2 },
    { 0x9c9ac5bb, SUC_lexicon_tags_25 },
    { 0x181ac5bc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xf0fac5be, SUC_lexicon_tags_37 },
    { 0x27fac5bf, SUC_lexicon_tags_75 },
    { 0xb91ac5bf, SUC_lexicon_tags_12 },
    { 0xe5fac5c1, SUC_lexicon_tags_17 },
    { 0x683ac5be, SUC_lexicon_tags_11 },
    { 0x391ac5bf, SUC_lexicon_tags_43 },
    { 0xff3ac5c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201ac5c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x34bac5ca, SUC_lexicon_tags_28 },
    { 0xba7ac5cb, SUC_lexicon_tags_14 },
    { 0x5edac5cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3ac5cf, SUC_lexicon_tags_2 },
    { 0x8e5ac5d0, SUC_lexicon_tags_65 },
    { 0x7d5ac5d1, SUC_lexicon_tags_37 },
    { 0x5edac5d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf6dac5d4, SUC_lexicon_tags_2 },
    { 0x36bac5d4, SUC_lexicon_tags_25 },
    { 0x311ac5d6, SUC_lexicon_tags_30 },
    { 0x9b5ac5d6, SUC_lexicon_tags_6 },
    { 0x3cfac5d7, SUC_lexicon_tags_17 },
    { 0xaebac5d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dac5dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x183ac5df, SUC_lexicon_tags_45 },
    { 0x751ac5e0, SUC_lexicon_tags_6 },
    { 0x433ac5df, SUC_lexicon_tags_17 },
    { 0x787ac5e2, SUC_lexicon_tags_65 },
    { 0xb1fac5df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577ac5e9, SUC_lexicon_tags_7 },
    { 0xe9bac5ea, SUC_lexicon_tags_43 },
    { 0x4dfac5eb, SUC_lexicon_tags_8 },
    { 0xff7ac5e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbcfac5ee, SUC_lexicon_tags_12 },
    { 0x34fac5ef, SUC_lexicon_tags_11 },
    { 0x4cdac5ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31ac5f4, SUC_lexicon_tags_29 },
    { 0x4c7ac5f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73ac5fd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb59ac5ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bac607, SUC_lexicon_tags_23 },
    { 0xcc1ac607, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99ac60c, SUC_lexicon_tags_0 },
    { 0x65dac60c, SUC_lexicon_tags_37 },
    { 0xa73ac60e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xce7ac610, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1ac614, SUC_lexicon_tags_17 },
    { 0x9b9ac615, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x525ac618, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dac61e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x617ac621, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b5ac623, SUC_lexicon_tags_2 },
    { 0xf41ac624, SUC_lexicon_tags_6 },
    { 0xcf9ac624, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4d3ac627, SUC_lexicon_tags_84 },
    { 0x13ac628, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5ac62d, SUC_lexicon_tags_1 },
    { 0x369ac62d, SUC_lexicon_tags_37 },
    { 0xdd9ac62d, SUC_lexicon_tags_60 },
    { 0x9f5ac630, SUC_lexicon_tags_335 },
    { 0xbc7ac62e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5ac634, SUC_lexicon_tags_8 },
    { 0x69bac635, SUC_lexicon_tags_42 },
    { 0x44dac634, SUC_lexicon_tags_15 },
    { 0x7bdac637, SUC_lexicon_tags_33 },
    { 0x395ac637, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x12dac63a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1ac63f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411ac644, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2abac646, SUC_lexicon_tags_37 },
    { 0x473ac647, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dac64b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2a1ac64d, SUC_lexicon_tags_85 },
    { 0xcbbac64e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125ac653, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1ac656, SUC_lexicon_tags_8 },
    { 0xe43ac657, SUC_lexicon_tags_25 },
    { 0x49fac657, SUC_lexicon_tags_37 },
    { 0x7b9ac659, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x40bac65b, SUC_lexicon_tags_6 },
    { 0x779ac65c, SUC_lexicon_tags_27 },
    { 0x9ffac65d, SUC_lexicon_tags_14 },
    { 0xee3ac65d, SUC_lexicon_tags_8 },
    { 0xd1bac65f, SUC_lexicon_tags_7 },
    { 0xd3fac660, SUC_lexicon_tags_2 },
    { 0x6cbac661, SUC_lexicon_tags_1 },
    { 0x813ac661, SUC_lexicon_tags_37 },
    { 0x38dac65b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf63ac665, SUC_lexicon_tags_17 },
    { 0x937ac666, SUC_lexicon_tags_17 },
    { 0xa49ac667, SUC_lexicon_tags_17 },
    { 0xdfac668, SUC_lexicon_tags_43 },
    { 0xdb9ac669, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fac66d, SUC_lexicon_tags_60 },
    { 0x3a3ac66d, SUC_lexicon_tags_37 },
    { 0x511ac66f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47ac676, SUC_lexicon_tags_1 },
    { 0x9c3ac676, SUC_lexicon_tags_1 },
    { 0x50fac678, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ac67c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7ac67f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xed3ac681, SUC_lexicon_tags_13 },
    { 0x539ac681, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ac689, SUC_lexicon_tags_82 },
    { 0x91fac689, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x119ac68d, SUC_lexicon_tags_25 },
    { 0x9d3ac68e, SUC_lexicon_tags_61 },
    { 0x279ac68e, SUC_lexicon_tags_37 },
    { 0x617ac68e, SUC_lexicon_tags_32 },
    { 0x2d1ac68e, SUC_lexicon_tags_33 },
    { 0xc95ac691, SUC_lexicon_tags_11 },
    { 0xbfdac693, SUC_lexicon_tags_1 },
    { 0x6fdac693, SUC_lexicon_tags_61 },
    { 0xc61ac690, SUC_lexicon_tags_14 },
    { 0xa99ac692, SUC_lexicon_tags_23 },
    { 0xb5ac690, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295ac69e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x495ac6a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ac6a4, SUC_lexicon_tags_29 },
    { 0xc3ac6a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ac6aa, SUC_lexicon_tags_87 },
    { 0x245ac6aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185ac6af, SUC_lexicon_tags_1 },
    { 0x5adac6af, SUC_lexicon_tags_3 },
    { 0x8a1ac6b1, SUC_lexicon_tags_87 },
    { 0xdd1ac6b2, SUC_lexicon_tags_2 },
    { 0xcefac6b0, SUC_lexicon_tags_12 },
    { 0x8bbac6b4, SUC_lexicon_tags_16 },
    { 0x9edac6b4, SUC_lexicon_tags_2 },
    { 0xde3ac6b6, SUC_lexicon_tags_17 },
    { 0x59dac6b7, SUC_lexicon_tags_29 },
    { 0x62bac6b4, SUC_lexicon_tags_9 },
    { 0xde9ac6b2, SUC_lexicon_tags_23 },
    { 0x6a1ac6b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59bac6bc, SUC_lexicon_tags_1 },
    { 0x2c1ac6bd, SUC_lexicon_tags_15 },
    { 0x5edac6be, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757ac6c6, SUC_lexicon_tags_49 },
    { 0x7c3ac6c7, SUC_lexicon_tags_15 },
    { 0xb13ac6c7, SUC_lexicon_tags_38 },
    { 0x715ac6c7, SUC_lexicon_tags_12 },
    { 0xba9ac6c7, SUC_lexicon_tags_0 },
    { 0x559ac6cb, SUC_lexicon_tags_27 },
    { 0x737ac6cc, SUC_lexicon_tags_25 },
    { 0x8e7ac6cd, SUC_lexicon_tags_39 },
    { 0xbbac6c7, SUC_lexicon_tags_2 },
    { 0x963ac6c6, SUC_lexicon_tags_1 },
    { 0x795ac6c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x85bac6d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x555ac6d5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753ac6dd, SUC_lexicon_tags_47 },
    { 0x2bdac6de, SUC_lexicon_tags_49 },
    { 0xdadac6de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbac6e2, SUC_lexicon_tags_38 },
    { 0x43bac6e3, SUC_lexicon_tags_11 },
    { 0x1b7ac6e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d3ac6e6, SUC_lexicon_tags_17 },
    { 0x181ac6e7, SUC_lexicon_tags_32 },
    { 0xcc1ac6e8, SUC_lexicon_tags_38 },
    { 0xdedac6e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x4adac6eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1ac6ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457ac6f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79ac6fd, SUC_lexicon_tags_31 },
    { 0xf6bac6fd, SUC_lexicon_tags_25 },
    { 0xf67ac6ff, SUC_lexicon_tags_68 },
    { 0xda1ac700, SUC_lexicon_tags_12 },
    { 0xeddac701, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f3ac703, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39ac70d, SUC_lexicon_tags_65 },
    { 0x327ac70e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7ac713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdac716, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97ac71a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ac71d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fac720, SUC_lexicon_tags_33 },
    { 0x7a9ac720, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaddac723, SUC_lexicon_tags_28 },
    { 0x72bac724, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffbac727, SUC_lexicon_tags_63 },
    { 0xe9fac728, SUC_lexicon_tags_14 },
    { 0x737ac729, SUC_lexicon_tags_9 },
    { 0x281ac72a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe99ac72c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bac730, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x82bac732, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x331ac734, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x665ac736, SUC_lexicon_tags_10 },
    { 0x6cdac737, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7ac73a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x169ac73c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881ac740, SUC_lexicon_tags_17 },
    { 0x9ddac740, SUC_lexicon_tags_9 },
    { 0x477ac741, SUC_lexicon_tags_20 },
    { 0x983ac742, SUC_lexicon_tags_7 },
    { 0xedac740, SUC_lexicon_tags_25 },
    { 0x31fac745, SUC_lexicon_tags_58 },
    { 0xa15ac746, SUC_lexicon_tags_0 },
    { 0xbfbac745, SUC_lexicon_tags_28 },
    { 0x1afac743, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe2fac74a, SUC_lexicon_tags_15 },
    { 0x6a3ac74b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75ac756, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x735ac758, SUC_lexicon_tags_6 },
    { 0xe2fac759, SUC_lexicon_tags_1 },
    { 0xb1ac759, SUC_lexicon_tags_14 },
    { 0x189ac759, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3ac760, SUC_lexicon_tags_60 },
    { 0x7d7ac761, SUC_lexicon_tags_37 },
    { 0xe3dac762, SUC_lexicon_tags_34 },
    { 0xc8fac763, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3ac766, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dac76a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8d7ac76c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x927ac76e, SUC_lexicon_tags_23 },
    { 0x965ac76e, SUC_lexicon_tags_6 },
    { 0x647ac770, SUC_lexicon_tags_1 },
    { 0x4fdac771, SUC_lexicon_tags_12 },
    { 0x14dac771, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211ac778, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x37dac77a, SUC_lexicon_tags_23 },
    { 0x855ac77b, SUC_lexicon_tags_26 },
    { 0x4a7ac77b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8cfac77e, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x97fac781, SUC_lexicon_tags_8 },
    { 0xb87ac781, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbffac784, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbac78d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x753ac78f, SUC_lexicon_tags_37 },
    { 0x2ebac790, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3ac797, SUC_lexicon_tags_18 },
    { 0x6a1ac798, SUC_lexicon_tags_7 },
    { 0x601ac799, SUC_lexicon_tags_25 },
    { 0x581ac79a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1ac7a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ac7a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x917ac7ab, SUC_lexicon_tags_4 },
    { 0xa53ac7ac, SUC_lexicon_tags_9 },
    { 0xfc9ac7ad, SUC_lexicon_tags_17 },
    { 0x16fac7ab, SUC_lexicon_tags_9 },
    { 0x6edac7af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abac7b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657ac7b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3ac7bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9ddac7be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5ac7c2, SUC_lexicon_tags_1 },
    { 0x4fdac7c3, SUC_lexicon_tags_2 },
    { 0xb6bac7c4, SUC_lexicon_tags_29 },
    { 0x76dac7c4, SUC_lexicon_tags_6 },
    { 0x6c5ac7c3, SUC_lexicon_tags_48 },
    { 0xf29ac7c7, SUC_lexicon_tags_47 },
    { 0x1fac7c8, SUC_lexicon_tags_2 },
    { 0xa7bac7c9, SUC_lexicon_tags_82 },
    { 0x6bdac7ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x781ac7cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51ac7d2, SUC_lexicon_tags_1 },
    { 0x433ac7d3, SUC_lexicon_tags_63 },
    { 0x1bfac7d4, SUC_lexicon_tags_25 },
    { 0xb79ac7d5, SUC_lexicon_tags_49 },
    { 0xbcbac7d4, SUC_lexicon_tags_29 },
    { 0xee9ac7d7, SUC_lexicon_tags_0 },
    { 0xd43ac7d8, SUC_lexicon_tags_1 },
    { 0x753ac7d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785ac7df, SUC_lexicon_tags_9 },
    { 0xb47ac7e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959ac7e8, SUC_lexicon_tags_25 },
    { 0xf41ac7e8, SUC_lexicon_tags_1 },
    { 0xc9dac7e9, SUC_lexicon_tags_60 },
    { 0x9cfac7eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9fac7ed, SUC_lexicon_tags_11 },
    { 0xaafac7ee, SUC_lexicon_tags_17 },
    { 0x4b7ac7ef, SUC_lexicon_tags_45 },
    { 0x501ac7f0, SUC_lexicon_tags_23 },
    { 0xa8bac7f0, SUC_lexicon_tags_32 },
    { 0x81ac7ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd13ac7f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4c3ac7f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bac801, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd75ac803, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ac807, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x563ac80a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09ac80e, SUC_lexicon_tags_27 },
    { 0xf3fac80f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd7ac811, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x645ac814, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa0bac816, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f3ac818, SUC_lexicon_tags_55 },
    { 0x637ac818, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadac81d, SUC_lexicon_tags_0 },
    { 0xed3ac81d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7ac822, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85ac826, SUC_lexicon_tags_17 },
    { 0x211ac826, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e9ac829, SUC_lexicon_tags_27 },
    { 0x86bac82a, SUC_lexicon_tags_1 },
    { 0x5c5ac82b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45dac82d, SUC_lexicon_tags_1 },
    { 0xbe7ac82e, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefac834, SUC_lexicon_tags_7 },
    { 0x39dac835, SUC_lexicon_tags_15 },
    { 0x6e1ac835, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x35dac838, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5ac83f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555ac844, SUC_lexicon_tags_17 },
    { 0xee5ac844, SUC_lexicon_tags_14 },
    { 0x325ac846, SUC_lexicon_tags_63 },
    { 0x41ac846, SUC_lexicon_tags_25 },
    { 0x5c7ac846, SUC_lexicon_tags_12 },
    { 0xed9ac849, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ac84d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8b3ac84f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83ac854, SUC_lexicon_tags_12 },
    { 0xb77ac854, SUC_lexicon_tags_15 },
    { 0x75fac856, SUC_lexicon_tags_49 },
    { 0xd4fac857, SUC_lexicon_tags_11 },
    { 0xe63ac858, SUC_lexicon_tags_26 },
    { 0x291ac859, SUC_lexicon_tags_35 },
    { 0x665ac85a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x80fac85c, SUC_lexicon_tags_2 },
    { 0x2cbac85d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75ac863, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefac86a, SUC_lexicon_tags_138 },
    { 0x463ac86a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe85ac86d, SUC_lexicon_tags_11 },
    { 0xb1ac86d, SUC_lexicon_tags_1 },
    { 0xc13ac86f, SUC_lexicon_tags_15 },
    { 0x8c7ac870, SUC_lexicon_tags_25 },
    { 0xad9ac871, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37ac876, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3ac87b, SUC_lexicon_tags_34 },
    { 0xa69ac87b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6abac87e, SUC_lexicon_tags_15 },
    { 0x5d9ac87f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dac882, SUC_lexicon_tags_2 },
    { 0x94dac883, SUC_lexicon_tags_11 },
    { 0x21bac882, SUC_lexicon_tags_11 },
    { 0xca7ac885, SUC_lexicon_tags_25 },
    { 0x66bac886, SUC_lexicon_tags_37 },
    { 0x5d3ac886, SUC_lexicon_tags_31 },
    { 0xb27ac888, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x95dac88a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1dac88c, SUC_lexicon_tags_17 },
    { 0xe4fac88c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bac894, SUC_lexicon_tags_8 },
    { 0x2dbac895, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b7ac897, SUC_lexicon_tags_4 },
    { 0x51fac897, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1ac89a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbefac89c, SUC_lexicon_tags_15 },
    { 0x9e3ac89d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7ac8a0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa6bac8a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdedac8a4, SUC_lexicon_tags_33 },
    { 0x683ac8a5, SUC_lexicon_tags_2 },
    { 0xb99ac8a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfac8ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bac8b4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09ac8b8, SUC_lexicon_tags_17 },
    { 0x663ac8b9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x62dac8bc, SUC_lexicon_tags_7 },
    { 0x5a7ac8bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdafac8bf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2bac8c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8fac8c3, SUC_lexicon_tags_0 },
    { 0xb15ac8c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfac8cd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39ac8d0, SUC_lexicon_tags_14 },
    { 0x2a9ac8d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dac8d6, SUC_lexicon_tags_47 },
    { 0x5adac8d7, SUC_lexicon_tags_54 },
    { 0x88fac8d6, SUC_lexicon_tags_11 },
    { 0xd1dac8d9, SUC_lexicon_tags_38 },
    { 0x3f9ac8da, SUC_lexicon_tags_25 },
    { 0x357ac8da, SUC_lexicon_tags_43 },
    { 0x53ac8da, SUC_lexicon_tags_9 },
    { 0x505ac8dd, SUC_lexicon_tags_23 },
    { 0xccbac8d9, SUC_lexicon_tags_13 },
    { 0xbfac8d9, SUC_lexicon_tags_47 },
    { 0xae7ac8d9, SUC_lexicon_tags_7 },
    { 0x6c1ac8e1, SUC_lexicon_tags_45 },
    { 0xefdac8dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f9ac8e4, SUC_lexicon_tags_11 },
    { 0x1c1ac8e5, SUC_lexicon_tags_12 },
    { 0x117ac8e6, SUC_lexicon_tags_2 },
    { 0xc0fac8e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x437ac8e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5ac8ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55ac8f3, SUC_lexicon_tags_17 },
    { 0x4ebac8f4, SUC_lexicon_tags_25 },
    { 0xc3dac8f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x159ac8f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x669ac8f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf03ac8fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc33ac8fd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ac902, SUC_lexicon_tags_33 },
    { 0x593ac903, SUC_lexicon_tags_1 },
    { 0xe5dac904, SUC_lexicon_tags_87 },
    { 0x7adac905, SUC_lexicon_tags_11 },
    { 0xfa9ac906, SUC_lexicon_tags_11 },
    { 0xb01ac905, SUC_lexicon_tags_10 },
    { 0xdfdac903, SUC_lexicon_tags_31 },
    { 0x13bac909, SUC_lexicon_tags_31 },
    { 0xdcfac90a, SUC_lexicon_tags_269 },
    { 0xf0bac90b, SUC_lexicon_tags_10 },
    { 0xb65ac903, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x831ac90e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x495ac911, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5ac915, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x709ac917, SUC_lexicon_tags_17 },
    { 0xe51ac917, SUC_lexicon_tags_28 },
    { 0x7efac919, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35ac91e, SUC_lexicon_tags_12 },
    { 0x57bac91f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dac922, SUC_lexicon_tags_77 },
    { 0xd7ac923, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x721ac926, SUC_lexicon_tags_17 },
    { 0x7a7ac927, SUC_lexicon_tags_30 },
    { 0x1f5ac926, SUC_lexicon_tags_23 },
    { 0x2efac926, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfac92c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b3ac92e, SUC_lexicon_tags_15 },
    { 0xb35ac92f, SUC_lexicon_tags_45 },
    { 0xf27ac930, SUC_lexicon_tags_61 },
    { 0xbe3ac92e, SUC_lexicon_tags_43 },
    { 0xbafac932, SUC_lexicon_tags_17 },
    { 0xd21ac933, SUC_lexicon_tags_46 },
    { 0x7dbac930, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17ac93d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fac940, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995ac944, SUC_lexicon_tags_11 },
    { 0xf85ac944, SUC_lexicon_tags_23 },
    { 0x95ac946, SUC_lexicon_tags_12 },
    { 0x895ac946, SUC_lexicon_tags_60 },
    { 0x5b3ac948, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x881ac94a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889ac94f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9ac953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71ac958, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7ac95d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5ac961, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7ac969, SUC_lexicon_tags_80 },
    { 0xaadac96a, SUC_lexicon_tags_61 },
    { 0xf7fac969, SUC_lexicon_tags_0 },
    { 0x7b1ac96c, SUC_lexicon_tags_7 },
    { 0x375ac96c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf9dac96f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475ac975, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07ac978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfac984, SUC_lexicon_tags_45 },
    { 0xd33ac984, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf67ac987, SUC_lexicon_tags_25 },
    { 0x3ebac988, SUC_lexicon_tags_11 },
    { 0xd2fac989, SUC_lexicon_tags_26 },
    { 0x99fac98a, SUC_lexicon_tags_45 },
    { 0x93bac98b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfac98e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3fac990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f7ac992, SUC_lexicon_tags_15 },
    { 0xf77ac992, SUC_lexicon_tags_48 },
    { 0x211ac994, SUC_lexicon_tags_1 },
    { 0x8ebac992, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335ac99b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x551ac99d, SUC_lexicon_tags_11 },
    { 0x40dac99e, SUC_lexicon_tags_38 },
    { 0x20fac99d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fac9a1, SUC_lexicon_tags_17 },
    { 0xe11ac9a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ac9a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ac9a9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fac9b0, SUC_lexicon_tags_40 },
    { 0x9e1ac9b1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7ac9bb, SUC_lexicon_tags_14 },
    { 0xe43ac9bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4b9ac9be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc79ac9c0, SUC_lexicon_tags_47 },
    { 0x439ac9c1, SUC_lexicon_tags_25 },
    { 0xe67ac9c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea3ac9c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945ac9cc, SUC_lexicon_tags_12 },
    { 0x737ac9cc, SUC_lexicon_tags_1 },
    { 0x4c9ac9cc, SUC_lexicon_tags_37 },
    { 0x11dac9cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5ac9d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xadfac9d5, SUC_lexicon_tags_22 },
    { 0xb55ac9d5, SUC_lexicon_tags_7 },
    { 0xfc9ac9d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x425ac9d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6f1ac9db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31ac9df, SUC_lexicon_tags_107 },
    { 0x461ac9df, SUC_lexicon_tags_31 },
    { 0xedfac9e1, SUC_lexicon_tags_43 },
    { 0xb9bac9e2, SUC_lexicon_tags_17 },
    { 0xadac9e2, SUC_lexicon_tags_17 },
    { 0x5cdac9e3, SUC_lexicon_tags_9 },
    { 0x677ac9e4, SUC_lexicon_tags_47 },
    { 0x2d7ac9e2, SUC_lexicon_tags_63 },
    { 0x47dac9e5, SUC_lexicon_tags_23 },
    { 0x2e9ac9e8, SUC_lexicon_tags_37 },
    { 0x43dac9e9, SUC_lexicon_tags_8 },
    { 0x639ac9ea, SUC_lexicon_tags_17 },
    { 0xabbac9ea, SUC_lexicon_tags_60 },
    { 0xaedac9ea, SUC_lexicon_tags_45 },
    { 0x39fac9e8, SUC_lexicon_tags_17 },
    { 0x23dac9e8, SUC_lexicon_tags_17 },
    { 0xda3ac9ea, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61ac9f5, SUC_lexicon_tags_60 },
    { 0x84bac9f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x645ac9f9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x231ac9fb, SUC_lexicon_tags_7 },
    { 0x4cdac9fb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45ac9ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbaca05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861aca09, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5aca0c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23aca0f, SUC_lexicon_tags_87 },
    { 0x3bdaca10, SUC_lexicon_tags_30 },
    { 0xfa3aca10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25baca13, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x295aca15, SUC_lexicon_tags_25 },
    { 0x8edaca16, SUC_lexicon_tags_182 },
    { 0x811aca17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x809aca19, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdaca1f, SUC_lexicon_tags_65 },
    { 0xbc7aca20, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63aca23, SUC_lexicon_tags_17 },
    { 0x785aca24, SUC_lexicon_tags_1 },
    { 0xbedaca25, SUC_lexicon_tags_60 },
    { 0xa9aca26, SUC_lexicon_tags_7 },
    { 0xb31aca27, SUC_lexicon_tags_25 },
    { 0x415aca26, SUC_lexicon_tags_2 },
    { 0xa6baca29, SUC_lexicon_tags_12 },
    { 0xc9daca27, SUC_lexicon_tags_47 },
    { 0x183aca2b, SUC_lexicon_tags_11 },
    { 0x403aca29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29daca2f, SUC_lexicon_tags_40 },
    { 0xe7faca30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x627aca32, SUC_lexicon_tags_28 },
    { 0x167aca33, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd39aca35, SUC_lexicon_tags_1 },
    { 0x817aca35, SUC_lexicon_tags_56 },
    { 0x337aca37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baca3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569aca3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x681aca42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x485aca45, SUC_lexicon_tags_27 },
    { 0x16baca46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46faca4f, SUC_lexicon_tags_92 },
    { 0xa71aca50, SUC_lexicon_tags_1 },
    { 0x199aca51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175aca57, SUC_lexicon_tags_8 },
    { 0x8faca58, SUC_lexicon_tags_2 },
    { 0xa8baca59, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddaca62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdaca65, SUC_lexicon_tags_26 },
    { 0x6ddaca66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45daca68, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xda1aca6a, SUC_lexicon_tags_46 },
    { 0xcbbaca6a, SUC_lexicon_tags_1 },
    { 0x419aca6b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4fbaca6e, SUC_lexicon_tags_12 },
    { 0xf91aca6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601aca75, SUC_lexicon_tags_50 },
    { 0x905aca76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7aca79, SUC_lexicon_tags_11 },
    { 0xb0baca79, SUC_lexicon_tags_2 },
    { 0x5c7aca7a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17aca7e, SUC_lexicon_tags_25 },
    { 0xe6daca7e, SUC_lexicon_tags_11 },
    { 0x353aca80, SUC_lexicon_tags_29 },
    { 0x3e7aca81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335aca86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993aca8e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587aca92, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63daca9c, SUC_lexicon_tags_44 },
    { 0x7faca9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbacaa1, SUC_lexicon_tags_45 },
    { 0x42dacaa1, SUC_lexicon_tags_11 },
    { 0xadbacaa1, SUC_lexicon_tags_14 },
    { 0x75dacaa1, SUC_lexicon_tags_37 },
    { 0xc31acaa5, SUC_lexicon_tags_11 },
    { 0xe55acaa5, SUC_lexicon_tags_1 },
    { 0x933acaa7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x283acaa9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411acaae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbcfacab0, SUC_lexicon_tags_28 },
    { 0xc23acab1, SUC_lexicon_tags_9 },
    { 0xa81acab2, SUC_lexicon_tags_39 },
    { 0xf49acab3, SUC_lexicon_tags_37 },
    { 0xc8facab2, SUC_lexicon_tags_17 },
    { 0x5d9acab5, SUC_lexicon_tags_56 },
    { 0x5dacab6, SUC_lexicon_tags_37 },
    { 0x4c7acab7, SUC_lexicon_tags_11 },
    { 0x67acab6, SUC_lexicon_tags_32 },
    { 0x22bacab9, SUC_lexicon_tags_34 },
    { 0xeedacab7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf99acabc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5acac0, SUC_lexicon_tags_30 },
    { 0xe9bacac0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x52bacac4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bacac8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x345acacb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb7acacd, SUC_lexicon_tags_23 },
    { 0x41dacacd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x413acad1, SUC_lexicon_tags_23 },
    { 0x285acad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f3acad4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xecbacad6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31acad9, SUC_lexicon_tags_1 },
    { 0x679acada, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5acae0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a7acae2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3acaf2, SUC_lexicon_tags_2 },
    { 0xe11acaf3, SUC_lexicon_tags_7 },
    { 0x48bacaf4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bacaf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84facafc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1facaff, SUC_lexicon_tags_45 },
    { 0x7e5acaff, SUC_lexicon_tags_0 },
    { 0x581acb01, SUC_lexicon_tags_55 },
    { 0x3abacb02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x925acb04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeedacb06, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665acb0c, SUC_lexicon_tags_9 },
    { 0xb73acb0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf4bacb0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75acb14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bdacb16, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dacb19, SUC_lexicon_tags_37 },
    { 0x12bacb19, SUC_lexicon_tags_30 },
    { 0x269acb1b, SUC_lexicon_tags_39 },
    { 0xd2facb1b, SUC_lexicon_tags_12 },
    { 0x9f3acb1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855acb21, SUC_lexicon_tags_113 },
    { 0xa43acb21, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dacb25, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3b1acb27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41acb31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bacb38, SUC_lexicon_tags_17 },
    { 0x1c7acb39, SUC_lexicon_tags_77 },
    { 0xe65acb3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315acb3e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdd5acb40, SUC_lexicon_tags_15 },
    { 0xdcfacb41, SUC_lexicon_tags_0 },
    { 0x719acb40, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x895acb45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5acb48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddacb55, SUC_lexicon_tags_12 },
    { 0x6abacb56, SUC_lexicon_tags_11 },
    { 0x47acb57, SUC_lexicon_tags_14 },
    { 0x447acb58, SUC_lexicon_tags_17 },
    { 0xba3acb56, SUC_lexicon_tags_75 },
    { 0xccfacb5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123acb60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9acb68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x631acb6a, SUC_lexicon_tags_19 },
    { 0x8e3acb6b, SUC_lexicon_tags_30 },
    { 0xab7acb6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7facb6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1acb76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97acb7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5acb82, SUC_lexicon_tags_49 },
    { 0xcddacb83, SUC_lexicon_tags_17 },
    { 0x9b1acb82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189acb88, SUC_lexicon_tags_25 },
    { 0x27acb89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7acb8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeb3acb91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x851acb93, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2facb96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x93dacb99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9acb9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdacba1, SUC_lexicon_tags_6 },
    { 0xd85acba2, SUC_lexicon_tags_10 },
    { 0x861acba3, SUC_lexicon_tags_23 },
    { 0xfebacba2, SUC_lexicon_tags_33 },
    { 0x991acba4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43acbad, SUC_lexicon_tags_15 },
    { 0xe23acbad, SUC_lexicon_tags_19 },
    { 0x8efacbaf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x457acbb1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x47facbb3, SUC_lexicon_tags_26 },
    { 0x197acbb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9acbbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99facbbe, SUC_lexicon_tags_37 },
    { 0xf4dacbbf, SUC_lexicon_tags_12 },
    { 0x4bdacbc0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x709acbc3, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30dacbc7, SUC_lexicon_tags_45 },
    { 0xfacbc8, SUC_lexicon_tags_37 },
    { 0xa27acbc8, SUC_lexicon_tags_63 },
    { 0x88bacbca, SUC_lexicon_tags_52 },
    { 0x32facbcb, SUC_lexicon_tags_11 },
    { 0xadacbcc, SUC_lexicon_tags_4 },
    { 0x50dacbcc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dacbd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bacbd6, SUC_lexicon_tags_126 },
    { 0xe85acbd6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7acbda, SUC_lexicon_tags_32 },
    { 0xe67acbda, SUC_lexicon_tags_37 },
    { 0xe05acbdb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x441acbdf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x935acbe1, SUC_lexicon_tags_0 },
    { 0xe0dacbe2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3cdacbe4, SUC_lexicon_tags_17 },
    { 0xca9acbe5, SUC_lexicon_tags_68 },
    { 0x10facbe4, SUC_lexicon_tags_68 },
    { 0x28bacbe6, SUC_lexicon_tags_49 },
    { 0xcdbacbe6, SUC_lexicon_tags_7 },
    { 0x543acbe6, SUC_lexicon_tags_42 },
    { 0x5fbacbe8, SUC_lexicon_tags_25 },
    { 0xaddacbe7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577acbf0, SUC_lexicon_tags_12 },
    { 0x62dacbf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe29acbf3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715acbfa, SUC_lexicon_tags_0 },
    { 0xea1acbfb, SUC_lexicon_tags_12 },
    { 0xe87acbfb, SUC_lexicon_tags_15 },
    { 0x7d3acbfa, SUC_lexicon_tags_9 },
    { 0xd97acbfe, SUC_lexicon_tags_1 },
    { 0x1cfacbfe, SUC_lexicon_tags_9 },
    { 0x903acc00, SUC_lexicon_tags_11 },
    { 0x41facc01, SUC_lexicon_tags_2 },
    { 0x427acc00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81acc08, SUC_lexicon_tags_21 },
    { 0x4e9acc09, SUC_lexicon_tags_11 },
    { 0x56facc0a, SUC_lexicon_tags_142 },
    { 0x1a9acc08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf9bacc0d, SUC_lexicon_tags_25 },
    { 0x29acc0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1acc12, SUC_lexicon_tags_15 },
    { 0x75acc12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e9acc15, SUC_lexicon_tags_27 },
    { 0x64facc16, SUC_lexicon_tags_49 },
    { 0xeafacc15, SUC_lexicon_tags_1 },
    { 0x799acc16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x357acc1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dacc21, SUC_lexicon_tags_29 },
    { 0x44dacc22, SUC_lexicon_tags_7 },
    { 0xe73acc23, SUC_lexicon_tags_4 },
    { 0xe1facc22, SUC_lexicon_tags_63 },
    { 0x239acc23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16facc27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c3acc29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335acc2d, SUC_lexicon_tags_12 },
    { 0xd65acc2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63acc31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba7acc33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x215acc36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b1acc38, SUC_lexicon_tags_19 },
    { 0xcdbacc39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bacc40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07acc46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x869acc49, SUC_lexicon_tags_11 },
    { 0x7facc4a, SUC_lexicon_tags_12 },
    { 0xb09acc4a, SUC_lexicon_tags_84 },
    { 0x6c1acc4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9acc50, SUC_lexicon_tags_37 },
    { 0xa15acc51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1acc56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddacc5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7acc60, SUC_lexicon_tags_60 },
    { 0x7c5acc61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8edacc63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd5acc65, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffacc6a, SUC_lexicon_tags_60 },
    { 0xeddacc6a, SUC_lexicon_tags_1 },
    { 0xa95acc6c, SUC_lexicon_tags_55 },
    { 0x85facc6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe15acc6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15acc72, SUC_lexicon_tags_45 },
    { 0x73facc73, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bacc76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39acc7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bacc80, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485acc86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edacc89, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x41bacc8b, SUC_lexicon_tags_25 },
    { 0x8d7acc8b, SUC_lexicon_tags_9 },
    { 0xea9acc8d, SUC_lexicon_tags_37 },
    { 0xd3facc8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307acc92, SUC_lexicon_tags_15 },
    { 0x925acc93, SUC_lexicon_tags_25 },
    { 0xd6facc94, SUC_lexicon_tags_63 },
    { 0x269acc95, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xed7acc97, SUC_lexicon_tags_60 },
    { 0x3acc98, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3acc9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7acc9f, SUC_lexicon_tags_60 },
    { 0x713acc9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x859acca2, SUC_lexicon_tags_7 },
    { 0x425acca3, SUC_lexicon_tags_25 },
    { 0xb97acca4, SUC_lexicon_tags_79 },
    { 0xed7acca3, SUC_lexicon_tags_25 },
    { 0xf69acca5, SUC_lexicon_tags_47 },
    { 0x1d9acca4, SUC_lexicon_tags_15 },
    { 0xf1bacca6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x79faccaa, SUC_lexicon_tags_40 },
    { 0x5dfaccab, SUC_lexicon_tags_7 },
    { 0xf79accab, SUC_lexicon_tags_14 },
    { 0x4d3accad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x101accaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x41daccb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x555accb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf67accb6, SUC_lexicon_tags_6 },
    { 0x6f3accb7, SUC_lexicon_tags_25 },
    { 0x84baccb6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfaccbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41accc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71baccc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a9accc6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x96daccc8, SUC_lexicon_tags_299 },
    { 0x19accc9, SUC_lexicon_tags_25 },
    { 0xf47accca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72daccce, SUC_lexicon_tags_37 },
    { 0x6edacccf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0daccd4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x519accd7, SUC_lexicon_tags_43 },
    { 0x327accd7, SUC_lexicon_tags_343 },
    { 0x35faccd9, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x74baccdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbacce1, SUC_lexicon_tags_23 },
    { 0x24facce1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19acce5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c7acce7, SUC_lexicon_tags_17 },
    { 0xd89acce8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e9accea, SUC_lexicon_tags_66 },
    { 0x13daccea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdaccee, SUC_lexicon_tags_21 },
    { 0x2d5accee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x659accf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277accf5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xadbaccf7, SUC_lexicon_tags_17 },
    { 0xbe7accf7, SUC_lexicon_tags_7 },
    { 0xbb7accf8, SUC_lexicon_tags_1 },
    { 0xd1baccf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x281accfd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9c3accff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90dacd02, SUC_lexicon_tags_15 },
    { 0xddbacd02, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7facd06, SUC_lexicon_tags_31 },
    { 0x41facd06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbafacd09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x555acd0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x553acd0f, SUC_lexicon_tags_61 },
    { 0x9e1acd10, SUC_lexicon_tags_2 },
    { 0x4b5acd0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625acd15, SUC_lexicon_tags_37 },
    { 0x41dacd16, SUC_lexicon_tags_8 },
    { 0x98facd17, SUC_lexicon_tags_7 },
    { 0x287acd18, SUC_lexicon_tags_47 },
    { 0x2e3acd19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x267acd1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdacd22, SUC_lexicon_tags_29 },
    { 0x1b5acd23, SUC_lexicon_tags_27 },
    { 0x2ddacd24, SUC_lexicon_tags_48 },
    { 0x57bacd23, SUC_lexicon_tags_17 },
    { 0x9a3acd26, SUC_lexicon_tags_0 },
    { 0x5f1acd22, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17acd2f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bacd32, SUC_lexicon_tags_15 },
    { 0x715acd33, SUC_lexicon_tags_30 },
    { 0x16facd33, SUC_lexicon_tags_6 },
    { 0x803acd35, SUC_lexicon_tags_2 },
    { 0x1cdacd36, SUC_lexicon_tags_21 },
    { 0x12dacd36, SUC_lexicon_tags_29 },
    { 0xeffacd38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x625acd3a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x55dacd3c, SUC_lexicon_tags_23 },
    { 0x1b7acd3d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5acd41, SUC_lexicon_tags_1 },
    { 0xcbacd42, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3acd45, SUC_lexicon_tags_7 },
    { 0x9facd46, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7acd49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64dacd4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c3acd50, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebacd55, SUC_lexicon_tags_23 },
    { 0x66facd56, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3acd5b, SUC_lexicon_tags_17 },
    { 0x995acd5c, SUC_lexicon_tags_37 },
    { 0xe0dacd5b, SUC_lexicon_tags_11 },
    { 0x877acd5e, SUC_lexicon_tags_30 },
    { 0x209acd5f, SUC_lexicon_tags_423 },
    { 0x487acd5f, SUC_lexicon_tags_7 },
    { 0x139acd61, SUC_lexicon_tags_2 },
    { 0x867acd61, SUC_lexicon_tags_47 },
    { 0xfd1acd5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6facd65, SUC_lexicon_tags_12 },
    { 0x593acd66, SUC_lexicon_tags_69 },
    { 0xc39acd67, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889acd6d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5acd6f, SUC_lexicon_tags_7 },
    { 0xca9acd6f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x951acd72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d7acd74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20facd77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddacd7b, SUC_lexicon_tags_37 },
    { 0x56bacd7b, SUC_lexicon_tags_31 },
    { 0x295acd7d, SUC_lexicon_tags_12 },
    { 0x46bacd7e, SUC_lexicon_tags_23 },
    { 0xc59acd7d, SUC_lexicon_tags_17 },
    { 0x41dacd7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90facd82, SUC_lexicon_tags_15 },
    { 0x16facd82, SUC_lexicon_tags_30 },
    { 0x121acd83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83acd87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97acd8e, SUC_lexicon_tags_17 },
    { 0x64facd8f, SUC_lexicon_tags_1 },
    { 0x1c9acd90, SUC_lexicon_tags_29 },
    { 0x143acd91, SUC_lexicon_tags_11 },
    { 0x1a5acd91, SUC_lexicon_tags_315 },
    { 0x131acd93, SUC_lexicon_tags_15 },
    { 0x52bacd94, SUC_lexicon_tags_43 },
    { 0xbb5acd95, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3acd99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85acd9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfacda0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x23facda2, SUC_lexicon_tags_17 },
    { 0x6adacda3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1acdab, SUC_lexicon_tags_27 },
    { 0x261acdac, SUC_lexicon_tags_26 },
    { 0x333acdac, SUC_lexicon_tags_48 },
    { 0x685acdae, SUC_lexicon_tags_1 },
    { 0x7fbacdad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195acdb4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x491acdb6, SUC_lexicon_tags_16 },
    { 0x883acdb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5dacdb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4e5acdbb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19acdbf, SUC_lexicon_tags_2 },
    { 0x80bacdbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81acdc3, SUC_lexicon_tags_39 },
    { 0xf9facdc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d7acdc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e3acdc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5acdcc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xac1acdce, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333acdd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebacdd5, SUC_lexicon_tags_17 },
    { 0xe9dacdd5, SUC_lexicon_tags_27 },
    { 0x405acdd7, SUC_lexicon_tags_7 },
    { 0xa8dacdd7, SUC_lexicon_tags_13 },
    { 0x6c3acdd9, SUC_lexicon_tags_1 },
    { 0x6c7acdda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa99acddc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29acddf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3dacde1, SUC_lexicon_tags_63 },
    { 0xe8bacde2, SUC_lexicon_tags_17 },
    { 0x1c9acde3, SUC_lexicon_tags_11 },
    { 0xa9dacde4, SUC_lexicon_tags_23 },
    { 0xa5dacde5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87acdec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31dacdf1, SUC_lexicon_tags_19 },
    { 0x411acdf1, SUC_lexicon_tags_63 },
    { 0xb8bacdf3, SUC_lexicon_tags_23 },
    { 0x1bacdf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4dacdf6, SUC_lexicon_tags_37 },
    { 0xf2dacdf7, SUC_lexicon_tags_14 },
    { 0x18bacdf6, SUC_lexicon_tags_14 },
    { 0x79facdf9, SUC_lexicon_tags_17 },
    { 0x683acdf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dacdfd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29ace00, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x74dace02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a7ace04, SUC_lexicon_tags_17 },
    { 0x8c1ace05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb35ace07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x483ace09, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ace0e, SUC_lexicon_tags_37 },
    { 0xde1ace0f, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xd17ace11, SUC_lexicon_tags_33 },
    { 0x9a7ace12, SUC_lexicon_tags_342 },
    { 0x53dace12, SUC_lexicon_tags_26 },
    { 0x493ace14, SUC_lexicon_tags_26 },
    { 0x103ace15, SUC_lexicon_tags_17 },
    { 0x7a1ace16, SUC_lexicon_tags_47 },
    { 0x91ace16, SUC_lexicon_tags_19 },
    { 0xc31ace18, SUC_lexicon_tags_2 },
    { 0xdf7ace18, SUC_lexicon_tags_17 },
    { 0x8aface1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983ace26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95ace29, SUC_lexicon_tags_17 },
    { 0x2bface2a, SUC_lexicon_tags_27 },
    { 0xac3ace2a, SUC_lexicon_tags_9 },
    { 0x33bace29, SUC_lexicon_tags_11 },
    { 0x9e5ace2c, SUC_lexicon_tags_11 },
    { 0x1dace2d, SUC_lexicon_tags_12 },
    { 0x78face2f, SUC_lexicon_tags_37 },
    { 0x41ace29, SUC_lexicon_tags_27 },
    { 0x3e9ace2e, SUC_lexicon_tags_12 },
    { 0xd57ace30, SUC_lexicon_tags_25 },
    { 0xd0bace33, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9ace3a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x139ace3c, SUC_lexicon_tags_10 },
    { 0xc3ace3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473ace42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bace46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dace49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbace4d, SUC_lexicon_tags_37 },
    { 0x20dace4d, SUC_lexicon_tags_47 },
    { 0x105ace4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73ace53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27ace56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x777ace58, SUC_lexicon_tags_13 },
    { 0x4b1ace59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ace5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29ace5e, SUC_lexicon_tags_1 },
    { 0x205ace5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbace65, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bace6a, SUC_lexicon_tags_87 },
    { 0x455ace6b, SUC_lexicon_tags_1 },
    { 0x8b1ace6a, SUC_lexicon_tags_1 },
    { 0xa49ace6d, SUC_lexicon_tags_1 },
    { 0x7a9ace6a, SUC_lexicon_tags_485 },
    { 0x341ace6f, SUC_lexicon_tags_30 },
    { 0x909ace70, SUC_lexicon_tags_1 },
    { 0x3b7ace71, SUC_lexicon_tags_11 },
    { 0xab5ace72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x357ace75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1ace78, SUC_lexicon_tags_25 },
    { 0xfa5ace79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347ace83, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x9adace85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9ace8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10face8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x999ace92, SUC_lexicon_tags_21 },
    { 0x4e3ace92, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885ace97, SUC_lexicon_tags_2 },
    { 0x72bace98, SUC_lexicon_tags_11 },
    { 0xd05ace99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2e3ace9b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x913ace9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a7acea0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87acea4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05acea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefaceab, SUC_lexicon_tags_48 },
    { 0xfefaceab, SUC_lexicon_tags_15 },
    { 0xc55aceac, SUC_lexicon_tags_14 },
    { 0xaabaceab, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0x447aceb0, SUC_lexicon_tags_12 },
    { 0xd8baceb1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3daceb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7aceb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a7aceb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc6bacebb, SUC_lexicon_tags_7 },
    { 0x971acebc, SUC_lexicon_tags_9 },
    { 0xdb3acebd, SUC_lexicon_tags_20 },
    { 0xf9acebe, SUC_lexicon_tags_45 },
    { 0xa3facebe, SUC_lexicon_tags_8 },
    { 0xa1dacec0, SUC_lexicon_tags_5 },
    { 0xb4facec1, SUC_lexicon_tags_31 },
    { 0x8f3acec2, SUC_lexicon_tags_25 },
    { 0xeefacec3, SUC_lexicon_tags_8 },
    { 0xf63acec0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1acecc, SUC_lexicon_tags_35 },
    { 0xe75acecc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbaced2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffaceda, SUC_lexicon_tags_11 },
    { 0x501aceda, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1acee6, SUC_lexicon_tags_17 },
    { 0x6a5acee7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5aceec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2daceee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x583acef1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36facefa, SUC_lexicon_tags_200 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199acefe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305acf02, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0facf05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3acf07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1facf0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5edacf0d, SUC_lexicon_tags_11 },
    { 0x579acf0e, SUC_lexicon_tags_11 },
    { 0x7acf0f, SUC_lexicon_tags_37 },
    { 0xc51acf10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdbacf12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dacf15, SUC_lexicon_tags_17 },
    { 0x485acf16, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365acf1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99acf1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x123acf1f, SUC_lexicon_tags_63 },
    { 0x18dacf20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81acf27, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb49acf29, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd47acf2b, SUC_lexicon_tags_1 },
    { 0x929acf2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9acf34, SUC_lexicon_tags_40 },
    { 0x2efacf34, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5acf3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bacf41, SUC_lexicon_tags_47 },
    { 0x73acf41, SUC_lexicon_tags_17 },
    { 0x8dfacf41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dacf47, SUC_lexicon_tags_15 },
    { 0x57facf48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x345acf4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1cbacf4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79acf50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49acf55, SUC_lexicon_tags_11 },
    { 0x3bfacf55, SUC_lexicon_tags_33 },
    { 0x6efacf56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcb5acf59, SUC_lexicon_tags_11 },
    { 0xf3facf5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x183acf5c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5acf60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc7facf62, SUC_lexicon_tags_29 },
    { 0xcafacf63, SUC_lexicon_tags_29 },
    { 0x729acf64, SUC_lexicon_tags_17 },
    { 0xdbdacf65, SUC_lexicon_tags_37 },
    { 0x4d7acf62, SUC_lexicon_tags_12 },
    { 0xe59acf66, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x1acf69, SUC_lexicon_tags_8 },
    { 0x269acf6a, SUC_lexicon_tags_10 },
    { 0xb6bacf6b, SUC_lexicon_tags_44 },
    { 0xbf3acf6a, SUC_lexicon_tags_244 },
    { 0x557acf6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937acf71, SUC_lexicon_tags_31 },
    { 0x32facf71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2d3acf74, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1d9acf76, SUC_lexicon_tags_17 },
    { 0x721acf77, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3acf7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7acf86, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631acf8e, SUC_lexicon_tags_15 },
    { 0xc99acf8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffacf93, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24facf9d, SUC_lexicon_tags_11 },
    { 0xf47acf9e, SUC_lexicon_tags_60 },
    { 0xe91acf9f, SUC_lexicon_tags_33 },
    { 0x88bacfa0, SUC_lexicon_tags_193 },
    { 0xd67acf9e, SUC_lexicon_tags_47 },
    { 0x26dacfa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x74bacfa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xedfacfa7, SUC_lexicon_tags_15 },
    { 0x73acfa8, SUC_lexicon_tags_15 },
    { 0xb4dacfa9, SUC_lexicon_tags_48 },
    { 0xbe3acfa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65acfae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffacfb3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc87acfb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccfacfb7, SUC_lexicon_tags_26 },
    { 0xe7facfb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37facfbc, SUC_lexicon_tags_17 },
    { 0xad7acfbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bacfc4, SUC_lexicon_tags_68 },
    { 0x7acfc4, SUC_lexicon_tags_62 },
    { 0xfa7acfc4, SUC_lexicon_tags_37 },
    { 0x565acfc7, SUC_lexicon_tags_11 },
    { 0x7dacfc7, SUC_lexicon_tags_79 },
    { 0xe7bacfc9, SUC_lexicon_tags_60 },
    { 0x2d1acfca, SUC_lexicon_tags_60 },
    { 0x92dacfc4, SUC_lexicon_tags_27 },
    { 0xf81acfcc, SUC_lexicon_tags_11 },
    { 0x483acfcb, SUC_lexicon_tags_12 },
    { 0x631acfc6, SUC_lexicon_tags_42 },
    { 0x1ebacfcb, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfacfd2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xff9acfd4, SUC_lexicon_tags_19 },
    { 0x15acfd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41acfd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x869acfdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9acfe0, SUC_lexicon_tags_57 },
    { 0xcbbacfe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d5acfe3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfb5acfe5, SUC_lexicon_tags_10 },
    { 0x405acfe6, SUC_lexicon_tags_11 },
    { 0xfbdacfe6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x533acfe9, SUC_lexicon_tags_17 },
    { 0xf3facfea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483acfee, SUC_lexicon_tags_7 },
    { 0xd51acfef, SUC_lexicon_tags_56 },
    { 0xc37acfef, SUC_lexicon_tags_14 },
    { 0xed5acff1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bacff5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x34facff7, SUC_lexicon_tags_13 },
    { 0x7adacff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7facffc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffacfff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35dad001, SUC_lexicon_tags_2 },
    { 0xdf5ad001, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9fad004, SUC_lexicon_tags_25 },
    { 0xe6dad005, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5ad00a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbad010, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efad019, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x987ad01c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3ad025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2cbad027, SUC_lexicon_tags_5 },
    { 0x73bad028, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7ad02c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1ad02f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87bad031, SUC_lexicon_tags_3 },
    { 0x59bad032, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321ad036, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5ad03a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9ad044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dad047, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bad04a, SUC_lexicon_tags_30 },
    { 0x9cdad04b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf29ad04e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461ad05d, SUC_lexicon_tags_27 },
    { 0xc3fad05e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfad067, SUC_lexicon_tags_11 },
    { 0x265ad068, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47ad06b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bad06f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637ad07b, SUC_lexicon_tags_7 },
    { 0xb31ad07b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x801ad07e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd93ad081, SUC_lexicon_tags_11 },
    { 0x9ddad082, SUC_lexicon_tags_49 },
    { 0x331ad082, SUC_lexicon_tags_65 },
    { 0x885ad081, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511ad08d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xce9ad08f, SUC_lexicon_tags_4 },
    { 0xd6bad08f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x629ad093, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x149ad095, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a5ad097, SUC_lexicon_tags_10 },
    { 0xd89ad098, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x951ad09b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7fad09d, SUC_lexicon_tags_1 },
    { 0x3b9ad09e, SUC_lexicon_tags_77 },
    { 0x957ad09f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x309ad0a2, SUC_lexicon_tags_14 },
    { 0x751ad0a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27ad0a9, SUC_lexicon_tags_7 },
    { 0xa53ad0aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ad0ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ffad0af, SUC_lexicon_tags_17 },
    { 0xd6fad0b0, SUC_lexicon_tags_77 },
    { 0x93ad0af, SUC_lexicon_tags_116 },
    { 0x29bad0b2, SUC_lexicon_tags_2 },
    { 0xa7dad0b3, SUC_lexicon_tags_25 },
    { 0x8dbad0b4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667ad0bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa31ad0bd, SUC_lexicon_tags_61 },
    { 0x125ad0bd, SUC_lexicon_tags_2 },
    { 0x603ad0bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ad0c5, SUC_lexicon_tags_1 },
    { 0xd69ad0c6, SUC_lexicon_tags_12 },
    { 0xc83ad0c7, SUC_lexicon_tags_63 },
    { 0x5f1ad0c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9cdad0ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1ad0ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dad0d2, SUC_lexicon_tags_12 },
    { 0x93ad0d2, SUC_lexicon_tags_0 },
    { 0x961ad0d4, SUC_lexicon_tags_37 },
    { 0xcfad0d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdad0d9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa25ad0db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x84dad0de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fad0e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6dad0e4, SUC_lexicon_tags_28 },
    { 0x737ad0e5, SUC_lexicon_tags_1 },
    { 0x273ad0e6, SUC_lexicon_tags_11 },
    { 0x719ad0e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9ad0e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fad0f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59ad0fb, SUC_lexicon_tags_37 },
    { 0xbc3ad0fc, SUC_lexicon_tags_12 },
    { 0xdd9ad0fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781ad101, SUC_lexicon_tags_7 },
    { 0x4b3ad101, SUC_lexicon_tags_64 },
    { 0xd71ad103, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x351ad105, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7adad107, SUC_lexicon_tags_26 },
    { 0x1c9ad108, SUC_lexicon_tags_33 },
    { 0xab5ad108, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x347ad10c, SUC_lexicon_tags_7 },
    { 0x24bad10d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd89ad10f, SUC_lexicon_tags_91 },
    { 0xc1ad10f, SUC_lexicon_tags_27 },
    { 0x211ad111, SUC_lexicon_tags_46 },
    { 0x3dad112, SUC_lexicon_tags_13 },
    { 0xb9ad112, SUC_lexicon_tags_129 },
    { 0x607ad114, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5d5ad116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb9ad118, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbad11b, SUC_lexicon_tags_60 },
    { 0xf7fad11c, SUC_lexicon_tags_15 },
    { 0x12dad11b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98dad122, SUC_lexicon_tags_1 },
    { 0x5d7ad122, SUC_lexicon_tags_4 },
    { 0x34fad124, SUC_lexicon_tags_3 },
    { 0xa3ad124, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedad12b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd9dad12d, SUC_lexicon_tags_30 },
    { 0xd6dad12d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x395ad131, SUC_lexicon_tags_1 },
    { 0x30fad132, SUC_lexicon_tags_17 },
    { 0xe0fad132, SUC_lexicon_tags_32 },
    { 0x65bad134, SUC_lexicon_tags_1 },
    { 0x1d9ad135, SUC_lexicon_tags_23 },
    { 0x1c7ad136, SUC_lexicon_tags_11 },
    { 0xf97ad136, SUC_lexicon_tags_62 },
    { 0x88bad138, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703ad142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81ad146, SUC_lexicon_tags_30 },
    { 0x18fad146, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7ad14d, SUC_lexicon_tags_82 },
    { 0x6cdad14e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1ad153, SUC_lexicon_tags_15 },
    { 0xaebad154, SUC_lexicon_tags_13 },
    { 0x3d5ad153, SUC_lexicon_tags_17 },
    { 0xa5fad156, SUC_lexicon_tags_33 },
    { 0x245ad156, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fad15a, SUC_lexicon_tags_17 },
    { 0x713ad15b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89ad160, SUC_lexicon_tags_60 },
    { 0x4b5ad160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21ad165, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afad16a, SUC_lexicon_tags_30 },
    { 0xa0dad16a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed9ad16d, SUC_lexicon_tags_47 },
    { 0x219ad16e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3ad173, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62bad175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x709ad177, SUC_lexicon_tags_7 },
    { 0x345ad178, SUC_lexicon_tags_21 },
    { 0xf41ad177, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43ad17d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fad181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bad188, SUC_lexicon_tags_27 },
    { 0xa3ad189, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17ad18e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x30bad191, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947ad19c, SUC_lexicon_tags_30 },
    { 0xce7ad19d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5ad1a0, SUC_lexicon_tags_27 },
    { 0x909ad1a1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x12bad1a3, SUC_lexicon_tags_35 },
    { 0xdf7ad1a4, SUC_lexicon_tags_72 },
    { 0x899ad1a4, SUC_lexicon_tags_21 },
    { 0xe1ad1a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadbad1a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x859ad1aa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3ad1b2, SUC_lexicon_tags_11 },
    { 0x181ad1b3, SUC_lexicon_tags_15 },
    { 0xce9ad1b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dad1b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa35ad1b9, SUC_lexicon_tags_17 },
    { 0x2d9ad1ba, SUC_lexicon_tags_23 },
    { 0xd09ad1bb, SUC_lexicon_tags_3 },
    { 0x141ad1bc, SUC_lexicon_tags_15 },
    { 0x431ad1bd, SUC_lexicon_tags_12 },
    { 0xd3bad1bd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x27fad1c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41ad1c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1ad1cb, SUC_lexicon_tags_17 },
    { 0x2e3ad1cc, SUC_lexicon_tags_11 },
    { 0xf07ad1cc, SUC_lexicon_tags_17 },
    { 0xd63ad1cd, SUC_lexicon_tags_49 },
    { 0x301ad1cf, SUC_lexicon_tags_125 },
    { 0xefad1d0, SUC_lexicon_tags_0 },
    { 0x55bad1cc, SUC_lexicon_tags_68 },
    { 0xfc7ad1cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8c1ad1d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b3ad1d6, SUC_lexicon_tags_66 },
    { 0xa3fad1d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc65ad1d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97ad1dd, SUC_lexicon_tags_28 },
    { 0xd61ad1dd, SUC_lexicon_tags_7 },
    { 0x7c9ad1de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3efad1e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc81ad1e3, SUC_lexicon_tags_47 },
    { 0x123ad1e3, SUC_lexicon_tags_7 },
    { 0x6efad1e5, SUC_lexicon_tags_21 },
    { 0x231ad1e6, SUC_lexicon_tags_60 },
    { 0x93dad1e7, SUC_lexicon_tags_17 },
    { 0xcd3ad1e8, SUC_lexicon_tags_10 },
    { 0x7f1ad1e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x809ad1ec, SUC_lexicon_tags_39 },
    { 0xcc1ad1ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ad1f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bad1f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ad1fe, SUC_lexicon_tags_12 },
    { 0xc79ad1ff, SUC_lexicon_tags_2 },
    { 0x8d9ad200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edad206, SUC_lexicon_tags_77 },
    { 0xd11ad206, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9ad20a, SUC_lexicon_tags_35 },
    { 0x23bad20b, SUC_lexicon_tags_14 },
    { 0xed1ad20b, SUC_lexicon_tags_15 },
    { 0xc27ad20d, SUC_lexicon_tags_578 },
    { 0xc09ad20d, SUC_lexicon_tags_63 },
    { 0x7abad20f, SUC_lexicon_tags_37 },
    { 0x2e3ad20a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4dad212, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x579ad215, SUC_lexicon_tags_10 },
    { 0xcc9ad216, SUC_lexicon_tags_11 },
    { 0xab1ad217, SUC_lexicon_tags_27 },
    { 0xf63ad218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b7ad21a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01ad21d, SUC_lexicon_tags_11 },
    { 0x573ad21d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95ad221, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fad227, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fad22e, SUC_lexicon_tags_84 },
    { 0x715ad22e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc95ad231, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x435ad233, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3ad237, SUC_lexicon_tags_37 },
    { 0x175ad237, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddad23b, SUC_lexicon_tags_15 },
    { 0x955ad23c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3ad241, SUC_lexicon_tags_11 },
    { 0x17dad242, SUC_lexicon_tags_0 },
    { 0x4e3ad243, SUC_lexicon_tags_47 },
    { 0x799ad244, SUC_lexicon_tags_26 },
    { 0x207ad245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bad24a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb93ad24d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ad251, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x151ad253, SUC_lexicon_tags_45 },
    { 0x2efad254, SUC_lexicon_tags_17 },
    { 0x4bad254, SUC_lexicon_tags_1 },
    { 0x185ad256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bad25c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x745ad25f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3ad262, SUC_lexicon_tags_2 },
    { 0xf7bad263, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9ad267, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3bfad269, SUC_lexicon_tags_1 },
    { 0x799ad26a, SUC_lexicon_tags_1 },
    { 0x5dbad26a, SUC_lexicon_tags_6 },
    { 0x51fad26c, SUC_lexicon_tags_38 },
    { 0x493ad26b, SUC_lexicon_tags_54 },
    { 0xd23ad26d, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ad271, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9ad277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdad27b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bad280, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45ad284, SUC_lexicon_tags_37 },
    { 0x3dbad285, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6dad287, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331ad28b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5ad28e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbad292, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xce5ad294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x743ad297, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b7ad299, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadad2a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc9ad2a3, SUC_lexicon_tags_112 },
    { 0x563ad2a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93ad2b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179ad2b4, SUC_lexicon_tags_1 },
    { 0x315ad2b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729ad2b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3ad2bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fad2c1, SUC_lexicon_tags_11 },
    { 0xcd1ad2c1, SUC_lexicon_tags_26 },
    { 0x1b1ad2c1, SUC_lexicon_tags_1 },
    { 0x931ad2c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7ad2c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63ad2d0, SUC_lexicon_tags_14 },
    { 0x4bdad2d1, SUC_lexicon_tags_19 },
    { 0xa0fad2d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bad2d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5ad2d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfad2dc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x1c5ad2de, SUC_lexicon_tags_47 },
    { 0x419ad2df, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ad2e3, SUC_lexicon_tags_9 },
    { 0x4afad2e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385ad2ec, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xffad2ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x623ad2f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347ad2f6, SUC_lexicon_tags_13 },
    { 0xfd1ad2f7, SUC_lexicon_tags_6 },
    { 0x7d9ad2f8, SUC_lexicon_tags_16 },
    { 0xd4dad2f9, SUC_lexicon_tags_29 },
    { 0x639ad2fa, SUC_lexicon_tags_15 },
    { 0xd2dad2fb, SUC_lexicon_tags_2 },
    { 0x9cdad2fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733ad302, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x871ad305, SUC_lexicon_tags_19 },
    { 0x75fad305, SUC_lexicon_tags_32 },
    { 0xb27ad306, SUC_lexicon_tags_25 },
    { 0x5c3ad308, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfad30b, SUC_lexicon_tags_55 },
    { 0xfd1ad30c, SUC_lexicon_tags_60 },
    { 0xa75ad30b, SUC_lexicon_tags_7 },
    { 0x74bad30e, SUC_lexicon_tags_11 },
    { 0x9a9ad30f, SUC_lexicon_tags_12 },
    { 0x1f1ad310, SUC_lexicon_tags_37 },
    { 0x505ad311, SUC_lexicon_tags_37 },
    { 0xefbad312, SUC_lexicon_tags_68 },
    { 0x757ad313, SUC_lexicon_tags_14 },
    { 0xacdad30b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9ad318, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787ad31c, SUC_lexicon_tags_17 },
    { 0x389ad31d, SUC_lexicon_tags_2 },
    { 0x757ad31c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x68dad320, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebad328, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5ad32d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885ad333, SUC_lexicon_tags_32 },
    { 0x37dad334, SUC_lexicon_tags_32 },
    { 0x503ad335, SUC_lexicon_tags_27 },
    { 0xdf3ad335, SUC_lexicon_tags_27 },
    { 0x609ad336, SUC_lexicon_tags_12 },
    { 0xfdbad338, SUC_lexicon_tags_7 },
    { 0xbdad338, SUC_lexicon_tags_0 },
    { 0xed9ad338, SUC_lexicon_tags_37 },
    { 0xc59ad335, SUC_lexicon_tags_25 },
    { 0x4b5ad33c, SUC_lexicon_tags_32 },
    { 0xa6fad335, SUC_lexicon_tags_26 },
    { 0x19fad338, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5ad342, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347ad348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfad34b, SUC_lexicon_tags_11 },
    { 0x661ad34b, SUC_lexicon_tags_56 },
    { 0xb59ad34d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fad351, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x58fad353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x459ad356, SUC_lexicon_tags_39 },
    { 0xb95ad357, SUC_lexicon_tags_12 },
    { 0x8f3ad357, SUC_lexicon_tags_63 },
    { 0xa55ad357, SUC_lexicon_tags_1 },
    { 0xd4bad35a, SUC_lexicon_tags_27 },
    { 0xb7bad35a, SUC_lexicon_tags_23 },
    { 0x183ad35c, SUC_lexicon_tags_23 },
    { 0xf9dad35d, SUC_lexicon_tags_17 },
    { 0x565ad35e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15ad361, SUC_lexicon_tags_31 },
    { 0xa19ad362, SUC_lexicon_tags_35 },
    { 0xc29ad361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x599ad365, SUC_lexicon_tags_17 },
    { 0x36dad366, SUC_lexicon_tags_25 },
    { 0xe51ad367, SUC_lexicon_tags_51 },
    { 0xae7ad367, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ad36a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x603ad36d, SUC_lexicon_tags_35 },
    { 0x1e1ad36d, SUC_lexicon_tags_37 },
    { 0xcfdad36f, SUC_lexicon_tags_0 },
    { 0x503ad36f, SUC_lexicon_tags_56 },
    { 0xe0bad371, SUC_lexicon_tags_245 },
    { 0x3dad371, SUC_lexicon_tags_27 },
    { 0xb2dad370, SUC_lexicon_tags_87 },
    { 0x6f7ad374, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fad378, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fad37f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7ad382, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7ad386, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x68dad388, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x23dad38b, SUC_lexicon_tags_25 },
    { 0xfb3ad38c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3ad392, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87ad395, SUC_lexicon_tags_33 },
    { 0xeb9ad395, SUC_lexicon_tags_11 },
    { 0x619ad397, SUC_lexicon_tags_43 },
    { 0xf63ad397, SUC_lexicon_tags_7 },
    { 0x48bad396, SUC_lexicon_tags_15 },
    { 0x993ad39a, SUC_lexicon_tags_0 },
    { 0x4c1ad39b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee9ad39d, SUC_lexicon_tags_65 },
    { 0xd53ad39e, SUC_lexicon_tags_25 },
    { 0x321ad39e, SUC_lexicon_tags_17 },
    { 0x32fad39d, SUC_lexicon_tags_37 },
    { 0x265ad3a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03ad3a8, SUC_lexicon_tags_19 },
    { 0x8a5ad3a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505ad3ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fad3b2, SUC_lexicon_tags_7 },
    { 0x6b7ad3b2, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadad3bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3ad3c2, SUC_lexicon_tags_14 },
    { 0x6cbad3c3, SUC_lexicon_tags_33 },
    { 0xeb5ad3c4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3ad3cc, SUC_lexicon_tags_33 },
    { 0xb79ad3cd, SUC_lexicon_tags_14 },
    { 0x58fad3ce, SUC_lexicon_tags_68 },
    { 0xf5ad3ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861ad3d3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x749ad3d6, SUC_lexicon_tags_6 },
    { 0x417ad3d7, SUC_lexicon_tags_30 },
    { 0x98dad3d8, SUC_lexicon_tags_46 },
    { 0xae3ad3d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991ad3e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x55dad3e4, SUC_lexicon_tags_15 },
    { 0x513ad3e5, SUC_lexicon_tags_11 },
    { 0xbc5ad3e6, SUC_lexicon_tags_28 },
    { 0xdfbad3e7, SUC_lexicon_tags_14 },
    { 0x289ad3e5, SUC_lexicon_tags_1 },
    { 0x85ad3e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x577ad3eb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x8a7ad3ed, SUC_lexicon_tags_33 },
    { 0x9d1ad3ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1ad3f5, SUC_lexicon_tags_28 },
    { 0x65bad3f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2edad3f8, SUC_lexicon_tags_25 },
    { 0xe71ad3f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1bad3fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149ad3ff, SUC_lexicon_tags_5 },
    { 0xdcbad400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7dad402, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf43ad404, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf35ad406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x179ad408, SUC_lexicon_tags_0 },
    { 0x54bad408, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x613ad40b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4a9ad40d, SUC_lexicon_tags_2 },
    { 0x12bad40d, SUC_lexicon_tags_17 },
    { 0x163ad40e, SUC_lexicon_tags_61 },
    { 0x26dad40e, SUC_lexicon_tags_1 },
    { 0xf23ad40d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9ad414, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fad417, SUC_lexicon_tags_37 },
    { 0x4e3ad418, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fad41c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bad41f, SUC_lexicon_tags_1 },
    { 0x6bfad420, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bad425, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fad428, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35ad42f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0bad431, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899ad435, SUC_lexicon_tags_67 },
    { 0x555ad435, SUC_lexicon_tags_1 },
    { 0xc9fad437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f1ad439, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf01ad43b, SUC_lexicon_tags_31 },
    { 0xef9ad43c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7ad440, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5ad447, SUC_lexicon_tags_31 },
    { 0x81dad448, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bad44b, SUC_lexicon_tags_25 },
    { 0xd3dad44c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54dad450, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebad452, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x8e5ad454, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x631ad457, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x681ad459, SUC_lexicon_tags_37 },
    { 0xcdad45a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45ad45d, SUC_lexicon_tags_25 },
    { 0x51ad45e, SUC_lexicon_tags_37 },
    { 0xb43ad45f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45ad465, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4dad467, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efad46b, SUC_lexicon_tags_56 },
    { 0xf2bad46c, SUC_lexicon_tags_11 },
    { 0xcd9ad46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fad470, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dad474, SUC_lexicon_tags_12 },
    { 0x71fad475, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbad47a, SUC_lexicon_tags_46 },
    { 0xeefad47a, SUC_lexicon_tags_30 },
    { 0x3d5ad47a, SUC_lexicon_tags_11 },
    { 0x51fad47b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bad480, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3ad48a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755ad48f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59fad492, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d9ad494, SUC_lexicon_tags_2 },
    { 0x81bad495, SUC_lexicon_tags_1 },
    { 0x549ad494, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1ad49b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x573ad49d, SUC_lexicon_tags_1151 },
    { 0x181ad49d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85ad4a2, SUC_lexicon_tags_11 },
    { 0xd3bad4a3, SUC_lexicon_tags_27 },
    { 0xd5dad4a4, SUC_lexicon_tags_39 },
    { 0x1a5ad4a5, SUC_lexicon_tags_12 },
    { 0x4dbad4a6, SUC_lexicon_tags_23 },
    { 0xf5bad4a6, SUC_lexicon_tags_7 },
    { 0x8a1ad4a3, SUC_lexicon_tags_17 },
    { 0x931ad4a9, SUC_lexicon_tags_4 },
    { 0xd9bad4aa, SUC_lexicon_tags_1 },
    { 0x7e7ad4a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7ad4ae, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9ad4ba, SUC_lexicon_tags_7 },
    { 0xa67ad4bb, SUC_lexicon_tags_14 },
    { 0xb77ad4bb, SUC_lexicon_tags_1 },
    { 0x42bad4bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x811ad4bf, SUC_lexicon_tags_1 },
    { 0xf93ad4c0, SUC_lexicon_tags_6 },
    { 0xc35ad4c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddad4c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565ad4c8, SUC_lexicon_tags_37 },
    { 0x9ffad4c9, SUC_lexicon_tags_29 },
    { 0x36dad4ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5ad4cd, SUC_lexicon_tags_1 },
    { 0x133ad4cd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fad4d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bad4d5, SUC_lexicon_tags_19 },
    { 0x35fad4d6, SUC_lexicon_tags_2 },
    { 0x343ad4d6, SUC_lexicon_tags_37 },
    { 0xa69ad4d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5ad4df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6c7ad4e1, SUC_lexicon_tags_17 },
    { 0xc71ad4e2, SUC_lexicon_tags_38 },
    { 0x803ad4e2, SUC_lexicon_tags_47 },
    { 0x367ad4e4, SUC_lexicon_tags_1 },
    { 0x7ebad4e1, SUC_lexicon_tags_11 },
    { 0x4e7ad4e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd9ad4e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fad4ec, SUC_lexicon_tags_12 },
    { 0x859ad4ec, SUC_lexicon_tags_144 },
    { 0xed9ad4ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661ad4f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefad4f7, SUC_lexicon_tags_60 },
    { 0xc45ad4f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x69bad4fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171ad50a, SUC_lexicon_tags_26 },
    { 0xee9ad50a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23fad50e, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107ad512, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e9ad514, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5ad519, SUC_lexicon_tags_60 },
    { 0x5a9ad519, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ad51e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x86bad520, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119ad525, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f3ad527, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xef5ad529, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8bbad52b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x771ad52d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ffad52f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dad536, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x503ad539, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799ad53d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1ad540, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeafad543, SUC_lexicon_tags_7 },
    { 0xda5ad543, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9ad548, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72bad554, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fad557, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3ad55b, SUC_lexicon_tags_15 },
    { 0x1adad55c, SUC_lexicon_tags_12 },
    { 0xd3ad55c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35ad560, SUC_lexicon_tags_27 },
    { 0x299ad561, SUC_lexicon_tags_60 },
    { 0x677ad562, SUC_lexicon_tags_66 },
    { 0x9b7ad563, SUC_lexicon_tags_2 },
    { 0xb45ad564, SUC_lexicon_tags_31 },
    { 0xda7ad561, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9ad569, SUC_lexicon_tags_33 },
    { 0x101ad56a, SUC_lexicon_tags_8 },
    { 0x34fad56b, SUC_lexicon_tags_37 },
    { 0x44fad56c, SUC_lexicon_tags_27 },
    { 0x3fad56a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcf3ad56f, SUC_lexicon_tags_79 },
    { 0xb8dad570, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5ad574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345ad579, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221ad57f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ad583, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbd5ad585, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcc5ad587, SUC_lexicon_tags_23 },
    { 0xc91ad588, SUC_lexicon_tags_27 },
    { 0x5efad589, SUC_lexicon_tags_15 },
    { 0xa83ad58a, SUC_lexicon_tags_32 },
    { 0x223ad58b, SUC_lexicon_tags_38 },
    { 0xd11ad58c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5ad595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979ad59b, SUC_lexicon_tags_1 },
    { 0xde9ad59c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efad59f, SUC_lexicon_tags_11 },
    { 0x947ad5a0, SUC_lexicon_tags_37 },
    { 0x287ad5a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7ad5a6, SUC_lexicon_tags_26 },
    { 0xf6dad5a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15ad5aa, SUC_lexicon_tags_33 },
    { 0xbabad5aa, SUC_lexicon_tags_37 },
    { 0xb09ad5ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaafad5ae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf2bad5b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bad5b7, SUC_lexicon_tags_38 },
    { 0xcdfad5b7, SUC_lexicon_tags_116 },
    { 0x42dad5b8, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x537ad5bb, SUC_lexicon_tags_49 },
    { 0x24dad5bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbad5c3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3ad5c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bad5ca, SUC_lexicon_tags_11 },
    { 0xd05ad5cb, SUC_lexicon_tags_37 },
    { 0xd93ad5cc, SUC_lexicon_tags_25 },
    { 0x29fad5cd, SUC_lexicon_tags_37 },
    { 0x293ad5cb, SUC_lexicon_tags_0 },
    { 0xe21ad5cf, SUC_lexicon_tags_8 },
    { 0x3a9ad5d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2efad5d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bad5d6, SUC_lexicon_tags_0 },
    { 0x961ad5d7, SUC_lexicon_tags_11 },
    { 0x37bad5d8, SUC_lexicon_tags_17 },
    { 0x839ad5d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75ad5db, SUC_lexicon_tags_12 },
    { 0xeb1ad5db, SUC_lexicon_tags_15 },
    { 0x929ad5dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3ad5e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513ad5ef, SUC_lexicon_tags_12 },
    { 0xa51ad5ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3ad5f6, SUC_lexicon_tags_28 },
    { 0x1efad5f7, SUC_lexicon_tags_47 },
    { 0x47fad5f8, SUC_lexicon_tags_68 },
    { 0x27bad5f9, SUC_lexicon_tags_10 },
    { 0x29bad5fa, SUC_lexicon_tags_33 },
    { 0x64dad5fb, SUC_lexicon_tags_11 },
    { 0x1b1ad5fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa31ad5fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5ad605, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3ad60f, SUC_lexicon_tags_77 },
    { 0x685ad610, SUC_lexicon_tags_31 },
    { 0xfc3ad611, SUC_lexicon_tags_16 },
    { 0x9e1ad612, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x883ad615, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9ad619, SUC_lexicon_tags_11 },
    { 0x81dad61a, SUC_lexicon_tags_11 },
    { 0x515ad61b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9bdad61d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dad622, SUC_lexicon_tags_17 },
    { 0x635ad623, SUC_lexicon_tags_19 },
    { 0xdf3ad624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34bad626, SUC_lexicon_tags_1 },
    { 0x84bad626, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbad62b, SUC_lexicon_tags_0 },
    { 0x809ad62b, SUC_lexicon_tags_18 },
    { 0x371ad62b, SUC_lexicon_tags_32 },
    { 0xe67ad62e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55dad633, SUC_lexicon_tags_33 },
    { 0xbb7ad634, SUC_lexicon_tags_16 },
    { 0xe8dad635, SUC_lexicon_tags_17 },
    { 0x32dad636, SUC_lexicon_tags_37 },
    { 0xe7fad636, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1ad640, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdad644, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3ad649, SUC_lexicon_tags_11 },
    { 0xd09ad649, SUC_lexicon_tags_7 },
    { 0xe6dad64b, SUC_lexicon_tags_15 },
    { 0xa5bad64b, SUC_lexicon_tags_37 },
    { 0x2f7ad64d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b7ad64f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c5ad651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edad654, SUC_lexicon_tags_5 },
    { 0xbedad654, SUC_lexicon_tags_27 },
    { 0x98dad656, SUC_lexicon_tags_25 },
    { 0x7b7ad657, SUC_lexicon_tags_47 },
    { 0x507ad655, SUC_lexicon_tags_1 },
    { 0xec3ad659, SUC_lexicon_tags_6 },
    { 0x671ad65a, SUC_lexicon_tags_12 },
    { 0xb49ad65b, SUC_lexicon_tags_45 },
    { 0xed1ad656, SUC_lexicon_tags_65 },
    { 0x127ad658, SUC_lexicon_tags_7 },
    { 0x899ad65b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7ad668, SUC_lexicon_tags_14 },
    { 0xc65ad668, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfd9ad66b, SUC_lexicon_tags_48 },
    { 0x9b3ad66c, SUC_lexicon_tags_7 },
    { 0xbf3ad66c, SUC_lexicon_tags_45 },
    { 0x70fad66e, SUC_lexicon_tags_28 },
    { 0x47fad66f, SUC_lexicon_tags_63 },
    { 0x741ad66f, SUC_lexicon_tags_31 },
    { 0x5f7ad66f, SUC_lexicon_tags_15 },
    { 0x58fad672, SUC_lexicon_tags_11 },
    { 0xde7ad672, SUC_lexicon_tags_14 },
    { 0x875ad674, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fad67a, SUC_lexicon_tags_14 },
    { 0x5cbad67a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa81ad67d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbb9ad67f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadad685, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbad688, SUC_lexicon_tags_8 },
    { 0x909ad689, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fbad68b, SUC_lexicon_tags_25 },
    { 0x3a1ad68c, SUC_lexicon_tags_12 },
    { 0x4e5ad68d, SUC_lexicon_tags_1 },
    { 0x1d1ad68e, SUC_lexicon_tags_343 },
    { 0xb1fad68f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bad692, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efad695, SUC_lexicon_tags_7 },
    { 0x8d1ad696, SUC_lexicon_tags_0 },
    { 0x323ad697, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125ad69e, SUC_lexicon_tags_8 },
    { 0x2dfad69e, SUC_lexicon_tags_2 },
    { 0xfadad69f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47ad6a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21ad6a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x489ad6a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x70bad6aa, SUC_lexicon_tags_17 },
    { 0x525ad6ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dad6b0, SUC_lexicon_tags_33 },
    { 0x3bbad6b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9ad6b6, SUC_lexicon_tags_11 },
    { 0x43ad6b7, SUC_lexicon_tags_11 },
    { 0x4fad6b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efad6bc, SUC_lexicon_tags_1 },
    { 0xa2dad6bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdad6c1, SUC_lexicon_tags_27 },
    { 0xe41ad6c2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3ad6c8, SUC_lexicon_tags_19 },
    { 0x115ad6c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fad6cf, SUC_lexicon_tags_1 },
    { 0xf15ad6d0, SUC_lexicon_tags_11 },
    { 0x19bad6d1, SUC_lexicon_tags_65 },
    { 0xe4dad6d2, SUC_lexicon_tags_17 },
    { 0x533ad6cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x89dad6d5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x23fad6d7, SUC_lexicon_tags_35 },
    { 0xe83ad6d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49ad6dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9ad6df, SUC_lexicon_tags_49 },
    { 0x9cfad6e0, SUC_lexicon_tags_66 },
    { 0x84fad6e0, SUC_lexicon_tags_15 },
    { 0x4a9ad6e2, SUC_lexicon_tags_47 },
    { 0x49ad6e1, SUC_lexicon_tags_37 },
    { 0x699ad6e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5ad6e8, SUC_lexicon_tags_29 },
    { 0x7bdad6e9, SUC_lexicon_tags_2 },
    { 0xaa9ad6ea, SUC_lexicon_tags_70 },
    { 0x6b7ad6e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4afad6ed, SUC_lexicon_tags_63 },
    { 0x70bad6ee, SUC_lexicon_tags_25 },
    { 0x4ebad6ed, SUC_lexicon_tags_45 },
    { 0x123ad6f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x935ad6f3, SUC_lexicon_tags_1 },
    { 0xccfad6f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9ad6fa, SUC_lexicon_tags_28 },
    { 0xa4dad6fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fad704, SUC_lexicon_tags_11 },
    { 0xf83ad704, SUC_lexicon_tags_27 },
    { 0xea5ad705, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ad709, SUC_lexicon_tags_10 },
    { 0x243ad70a, SUC_lexicon_tags_43 },
    { 0xfd1ad70b, SUC_lexicon_tags_45 },
    { 0x5bbad70b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921ad716, SUC_lexicon_tags_15 },
    { 0x32dad717, SUC_lexicon_tags_17 },
    { 0x19fad716, SUC_lexicon_tags_2 },
    { 0x899ad719, SUC_lexicon_tags_25 },
    { 0x9ffad71a, SUC_lexicon_tags_47 },
    { 0x3ffad71a, SUC_lexicon_tags_37 },
    { 0x9a3ad71a, SUC_lexicon_tags_7 },
    { 0xb8fad71b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x655ad71f, SUC_lexicon_tags_1 },
    { 0xe5dad720, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bad725, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x429ad728, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17ad72d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63ad731, SUC_lexicon_tags_14 },
    { 0x2b3ad731, SUC_lexicon_tags_11 },
    { 0x85ad733, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddad738, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63ad73d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13ad742, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bbad744, SUC_lexicon_tags_26 },
    { 0x8fad745, SUC_lexicon_tags_58 },
    { 0x465ad746, SUC_lexicon_tags_23 },
    { 0x3d5ad747, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ad74f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7ad753, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfe1ad755, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa43ad757, SUC_lexicon_tags_0 },
    { 0x1a1ad758, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf93ad75a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf61ad75c, SUC_lexicon_tags_37 },
    { 0xc2dad75c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e7ad75f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bad766, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85ad768, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x60fad76a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x717ad76c, SUC_lexicon_tags_315 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5ad775, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175ad77e, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1ad787, SUC_lexicon_tags_61 },
    { 0x867ad787, SUC_lexicon_tags_28 },
    { 0x259ad789, SUC_lexicon_tags_2 },
    { 0x17dad78a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bad78f, SUC_lexicon_tags_15 },
    { 0x341ad78f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bad793, SUC_lexicon_tags_11 },
    { 0x661ad794, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x341ad796, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x29dad799, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebad79d, SUC_lexicon_tags_43 },
    { 0x43ad79e, SUC_lexicon_tags_27 },
    { 0x4d7ad79f, SUC_lexicon_tags_25 },
    { 0xbb3ad79f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdad7a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbad7a6, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0xba1ad7a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x157ad7ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857ad7b4, SUC_lexicon_tags_37 },
    { 0x595ad7b5, SUC_lexicon_tags_37 },
    { 0xa3fad7b4, SUC_lexicon_tags_37 },
    { 0x749ad7b7, SUC_lexicon_tags_15 },
    { 0x58fad7b8, SUC_lexicon_tags_43 },
    { 0x11ad7b4, SUC_lexicon_tags_37 },
    { 0x8afad7ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fad7c0, SUC_lexicon_tags_714 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1ad7c8, SUC_lexicon_tags_1 },
    { 0xf3ad7c9, SUC_lexicon_tags_7 },
    { 0x157ad7ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x643ad7cc, SUC_lexicon_tags_31 },
    { 0xa1ad7cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x26dad7cf, SUC_lexicon_tags_45 },
    { 0x2ffad7d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5ad7d5, SUC_lexicon_tags_9 },
    { 0x1ddad7d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bad7d9, SUC_lexicon_tags_2 },
    { 0x6dbad7d9, SUC_lexicon_tags_26 },
    { 0xfc5ad7db, SUC_lexicon_tags_37 },
    { 0x7efad7db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5ad7df, SUC_lexicon_tags_15 },
    { 0x86bad7e0, SUC_lexicon_tags_28 },
    { 0xe95ad7e0, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43ad7e5, SUC_lexicon_tags_84 },
    { 0x25bad7e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73ad7eb, SUC_lexicon_tags_15 },
    { 0x1afad7ec, SUC_lexicon_tags_26 },
    { 0xc53ad7ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x657ad7ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983ad7f4, SUC_lexicon_tags_2 },
    { 0x149ad7f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdad7f8, SUC_lexicon_tags_11 },
    { 0xf3fad7f9, SUC_lexicon_tags_14 },
    { 0x13dad7f9, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x531ad7fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fad803, SUC_lexicon_tags_43 },
    { 0x671ad804, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ad808, SUC_lexicon_tags_3 },
    { 0xf09ad808, SUC_lexicon_tags_12 },
    { 0x723ad80a, SUC_lexicon_tags_10 },
    { 0xc1dad80b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7ad810, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5ad813, SUC_lexicon_tags_46 },
    { 0x9c1ad814, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9ad817, SUC_lexicon_tags_19 },
    { 0x45ad817, SUC_lexicon_tags_60 },
    { 0xd37ad817, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bad81d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697ad821, SUC_lexicon_tags_47 },
    { 0x1edad822, SUC_lexicon_tags_55 },
    { 0x18dad823, SUC_lexicon_tags_1 },
    { 0x6f3ad824, SUC_lexicon_tags_11 },
    { 0x79fad825, SUC_lexicon_tags_8 },
    { 0xcf5ad822, SUC_lexicon_tags_295 },
    { 0x8d1ad824, SUC_lexicon_tags_37 },
    { 0xb67ad821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb71ad82b, SUC_lexicon_tags_83 },
    { 0x687ad82b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbad82f, SUC_lexicon_tags_25 },
    { 0xffbad830, SUC_lexicon_tags_35 },
    { 0x68bad830, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2c3ad833, SUC_lexicon_tags_39 },
    { 0x45bad834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fad839, SUC_lexicon_tags_12 },
    { 0x81dad83a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdad83f, SUC_lexicon_tags_2 },
    { 0x253ad840, SUC_lexicon_tags_60 },
    { 0x31bad841, SUC_lexicon_tags_60 },
    { 0xd81ad841, SUC_lexicon_tags_15 },
    { 0x791ad843, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb3dad845, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacfad847, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1a3ad849, SUC_lexicon_tags_15 },
    { 0x7afad84a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297ad84e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fad853, SUC_lexicon_tags_2 },
    { 0xbc3ad854, SUC_lexicon_tags_0 },
    { 0x3c3ad854, SUC_lexicon_tags_47 },
    { 0x8d1ad853, SUC_lexicon_tags_33 },
    { 0x621ad853, SUC_lexicon_tags_3 },
    { 0x21dad858, SUC_lexicon_tags_11 },
    { 0x2dad856, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfad85c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ad861, SUC_lexicon_tags_48 },
    { 0x27ad862, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649ad867, SUC_lexicon_tags_65 },
    { 0x549ad868, SUC_lexicon_tags_30 },
    { 0xb19ad868, SUC_lexicon_tags_45 },
    { 0x399ad86a, SUC_lexicon_tags_11 },
    { 0xd49ad869, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x221ad86d, SUC_lexicon_tags_27 },
    { 0x4e5ad86d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfad871, SUC_lexicon_tags_47 },
    { 0xcc7ad872, SUC_lexicon_tags_70 },
    { 0x871ad873, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbad876, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bad87a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe55ad87c, SUC_lexicon_tags_1 },
    { 0x787ad87d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865ad884, SUC_lexicon_tags_8 },
    { 0x719ad884, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc77ad887, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efad88e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x301ad890, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ad895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x449ad897, SUC_lexicon_tags_37 },
    { 0xa4fad897, SUC_lexicon_tags_15 },
    { 0xd45ad897, SUC_lexicon_tags_26 },
    { 0x6adad899, SUC_lexicon_tags_46 },
    { 0xb67ad897, SUC_lexicon_tags_29 },
    { 0x95dad89c, SUC_lexicon_tags_2 },
    { 0x809ad897, SUC_lexicon_tags_33 },
    { 0xeb7ad89e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69dad8a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x291ad8a3, SUC_lexicon_tags_45 },
    { 0xc3ad8a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfad8a7, SUC_lexicon_tags_37 },
    { 0xe25ad8a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x535ad8aa, SUC_lexicon_tags_2 },
    { 0x4f3ad8aa, SUC_lexicon_tags_2 },
    { 0xcb1ad8ab, SUC_lexicon_tags_31 },
    { 0x8a9ad8aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fad8b2, SUC_lexicon_tags_26 },
    { 0xfabad8b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3ad8b5, SUC_lexicon_tags_14 },
    { 0x857ad8b6, SUC_lexicon_tags_33 },
    { 0xfdbad8b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb05ad8b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x205ad8bb, SUC_lexicon_tags_11 },
    { 0x213ad8bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bfad8be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18bad8c1, SUC_lexicon_tags_17 },
    { 0x7afad8c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1ad8c6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27ad8ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf8bad8cc, SUC_lexicon_tags_61 },
    { 0xeddad8cd, SUC_lexicon_tags_17 },
    { 0x417ad8ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffad8d5, SUC_lexicon_tags_28 },
    { 0xb3ad8d6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453ad8dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3ad8e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821ad8e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c1ad8eb, SUC_lexicon_tags_25 },
    { 0xbb7ad8eb, SUC_lexicon_tags_11 },
    { 0x12bad8eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3ffad8ef, SUC_lexicon_tags_14 },
    { 0x595ad8f0, SUC_lexicon_tags_29 },
    { 0xd3ad8f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bad8f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66fad8f8, SUC_lexicon_tags_2 },
    { 0xa45ad8f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca3ad8fb, SUC_lexicon_tags_25 },
    { 0x52fad8fc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ad901, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fad905, SUC_lexicon_tags_2 },
    { 0x29dad906, SUC_lexicon_tags_17 },
    { 0xdf5ad907, SUC_lexicon_tags_29 },
    { 0xeffad906, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x971ad90a, SUC_lexicon_tags_56 },
    { 0xa17ad90b, SUC_lexicon_tags_14 },
    { 0xccdad90c, SUC_lexicon_tags_15 },
    { 0x291ad90d, SUC_lexicon_tags_11 },
    { 0xd5fad90c, SUC_lexicon_tags_17 },
    { 0x5b5ad90b, SUC_lexicon_tags_47 },
    { 0x727ad910, SUC_lexicon_tags_25 },
    { 0xffbad910, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293ad916, SUC_lexicon_tags_68 },
    { 0xf63ad917, SUC_lexicon_tags_23 },
    { 0xa25ad917, SUC_lexicon_tags_39 },
    { 0xe5fad919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfad91f, SUC_lexicon_tags_47 },
    { 0x88dad920, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xeffad922, SUC_lexicon_tags_1 },
    { 0x725ad923, SUC_lexicon_tags_7 },
    { 0x269ad924, SUC_lexicon_tags_23 },
    { 0xf41ad925, SUC_lexicon_tags_50 },
    { 0xdbdad926, SUC_lexicon_tags_0 },
    { 0x351ad926, SUC_lexicon_tags_7 },
    { 0x853ad926, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0x355ad92b, SUC_lexicon_tags_37 },
    { 0x311ad92c, SUC_lexicon_tags_15 },
    { 0xecbad92c, SUC_lexicon_tags_19 },
    { 0x155ad92d, SUC_lexicon_tags_1 },
    { 0x69dad92d, SUC_lexicon_tags_43 },
    { 0x13dad92f, SUC_lexicon_tags_4 },
    { 0x43ad931, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7a5ad933, SUC_lexicon_tags_10 },
    { 0xa99ad934, SUC_lexicon_tags_25 },
    { 0xb3fad933, SUC_lexicon_tags_1 },
    { 0x333ad936, SUC_lexicon_tags_30 },
    { 0x49dad937, SUC_lexicon_tags_2 },
    { 0x1b5ad937, SUC_lexicon_tags_12 },
    { 0x837ad939, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b1ad93b, SUC_lexicon_tags_39 },
    { 0xad1ad93c, SUC_lexicon_tags_30 },
    { 0xef3ad93c, SUC_lexicon_tags_11 },
    { 0x577ad93b, SUC_lexicon_tags_12 },
    { 0x329ad93f, SUC_lexicon_tags_12 },
    { 0x41dad93f, SUC_lexicon_tags_7 },
    { 0xf03ad93f, SUC_lexicon_tags_21 },
    { 0xcadad93b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25ad946, SUC_lexicon_tags_38 },
    { 0x5c5ad947, SUC_lexicon_tags_49 },
    { 0x5efad946, SUC_lexicon_tags_77 },
    { 0x3e3ad949, SUC_lexicon_tags_26 },
    { 0x875ad948, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bad94e, SUC_lexicon_tags_63 },
    { 0xf01ad94f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc33ad951, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x23bad953, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77ad95c, SUC_lexicon_tags_17 },
    { 0x333ad95c, SUC_lexicon_tags_7 },
    { 0x8c9ad95e, SUC_lexicon_tags_7 },
    { 0xb7fad95f, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0x301ad961, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5b9ad963, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753ad967, SUC_lexicon_tags_35 },
    { 0x72dad968, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929ad96c, SUC_lexicon_tags_25 },
    { 0x215ad96d, SUC_lexicon_tags_17 },
    { 0x939ad96e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969ad973, SUC_lexicon_tags_11 },
    { 0x6ddad974, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e1ad976, SUC_lexicon_tags_1 },
    { 0xd0bad976, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x573ad97a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51ad97c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35ad980, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x525ad983, SUC_lexicon_tags_11 },
    { 0xd69ad984, SUC_lexicon_tags_62 },
    { 0xd67ad985, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dad988, SUC_lexicon_tags_27 },
    { 0x955ad989, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89ad98c, SUC_lexicon_tags_43 },
    { 0xf07ad98d, SUC_lexicon_tags_47 },
    { 0xe13ad98d, SUC_lexicon_tags_45 },
    { 0x483ad98c, SUC_lexicon_tags_1 },
    { 0xe15ad990, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8fbad992, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7ad995, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3ad998, SUC_lexicon_tags_17 },
    { 0x4ddad999, SUC_lexicon_tags_2 },
    { 0x9afad99a, SUC_lexicon_tags_16 },
    { 0x791ad99b, SUC_lexicon_tags_1 },
    { 0x2bfad99c, SUC_lexicon_tags_33 },
    { 0xed9ad99c, SUC_lexicon_tags_7 },
    { 0x303ad99d, SUC_lexicon_tags_37 },
    { 0x3dbad998, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dad9a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6dad9a5, SUC_lexicon_tags_55 },
    { 0x9bdad9a5, SUC_lexicon_tags_17 },
    { 0x393ad9a7, SUC_lexicon_tags_25 },
    { 0x32bad9a8, SUC_lexicon_tags_46 },
    { 0x9dbad9a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e1ad9ab, SUC_lexicon_tags_4 },
    { 0x50dad9ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd21ad9ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdffad9b0, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3ad9b4, SUC_lexicon_tags_29 },
    { 0x41dad9b5, SUC_lexicon_tags_37 },
    { 0x7a3ad9b6, SUC_lexicon_tags_13 },
    { 0xae3ad9b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fad9ba, SUC_lexicon_tags_15 },
    { 0x2c1ad9bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855ad9bf, SUC_lexicon_tags_14 },
    { 0xa3dad9c0, SUC_lexicon_tags_11 },
    { 0x1d5ad9bf, SUC_lexicon_tags_37 },
    { 0x9cfad9c1, SUC_lexicon_tags_30 },
    { 0x395ad9c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325ad9ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3ad9cf, SUC_lexicon_tags_17 },
    { 0x9bfad9d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dad9d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e3ad9d6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1f5ad9d8, SUC_lexicon_tags_8 },
    { 0x1e3ad9d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ad9dd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1fdad9df, SUC_lexicon_tags_1 },
    { 0xbf7ad9e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23dad9e3, SUC_lexicon_tags_33 },
    { 0x7d3ad9e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afad9ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3fad9ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1b7ad9ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d7ad9f0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bad9f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bad9f7, SUC_lexicon_tags_39 },
    { 0xa73ad9f8, SUC_lexicon_tags_7 },
    { 0xbe1ad9f9, SUC_lexicon_tags_28 },
    { 0xb7fad9f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dad9fe, SUC_lexicon_tags_9 },
    { 0xaabad9ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3ada08, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ada0e, SUC_lexicon_tags_29 },
    { 0xf23ada0f, SUC_lexicon_tags_5 },
    { 0x5e1ada0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07ada17, SUC_lexicon_tags_15 },
    { 0xb93ada17, SUC_lexicon_tags_2 },
    { 0x50bada19, SUC_lexicon_tags_7 },
    { 0x6adada1a, SUC_lexicon_tags_19 },
    { 0xdcdada19, SUC_lexicon_tags_2 },
    { 0xe55ada1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54fada1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59ada20, SUC_lexicon_tags_25 },
    { 0x613ada21, SUC_lexicon_tags_7 },
    { 0x1e3ada22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dada2c, SUC_lexicon_tags_11 },
    { 0xfedada2d, SUC_lexicon_tags_17 },
    { 0x905ada2d, SUC_lexicon_tags_0 },
    { 0xb5ada2c, SUC_lexicon_tags_1 },
    { 0x5abada2d, SUC_lexicon_tags_37 },
    { 0x14bada31, SUC_lexicon_tags_25 },
    { 0x57bada30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ada36, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fada3d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdada40, SUC_lexicon_tags_27 },
    { 0xd6bada41, SUC_lexicon_tags_4 },
    { 0x149ada41, SUC_lexicon_tags_38 },
    { 0xdb5ada43, SUC_lexicon_tags_9 },
    { 0x105ada43, SUC_lexicon_tags_47 },
    { 0xe17ada45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2dada47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fada51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ffada53, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa83ada55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x759ada57, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x27bada59, SUC_lexicon_tags_28 },
    { 0xc53ada5a, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dada5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3ada61, SUC_lexicon_tags_65 },
    { 0x47ada62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bada6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc29ada6c, SUC_lexicon_tags_11 },
    { 0x56bada6d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcd7ada6f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa0bada71, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15ada77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bada7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bada7f, SUC_lexicon_tags_21 },
    { 0x473ada7f, SUC_lexicon_tags_160 },
    { 0xd75ada80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665ada87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x945ada89, SUC_lexicon_tags_23 },
    { 0xb4bada89, SUC_lexicon_tags_15 },
    { 0x149ada8a, SUC_lexicon_tags_11 },
    { 0xda3ada8a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bada90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7ada94, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8efada97, SUC_lexicon_tags_14 },
    { 0xb47ada98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdada9d, SUC_lexicon_tags_216 },
    { 0x3bdada9e, SUC_lexicon_tags_92 },
    { 0xf3ada9f, SUC_lexicon_tags_15 },
    { 0xf95adaa0, SUC_lexicon_tags_35 },
    { 0x585adaa1, SUC_lexicon_tags_37 },
    { 0xb6badaa1, SUC_lexicon_tags_12 },
    { 0xca9adaa3, SUC_lexicon_tags_8 },
    { 0x347adaa2, SUC_lexicon_tags_15 },
    { 0xfe3adaa2, SUC_lexicon_tags_37 },
    { 0xed5adaa0, SUC_lexicon_tags_64 },
    { 0xcf3ada9f, SUC_lexicon_tags_11 },
    { 0xb63adaa1, SUC_lexicon_tags_15 },
    { 0xa5fadaa9, SUC_lexicon_tags_12 },
    { 0xc31adaaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67adaae, SUC_lexicon_tags_87 },
    { 0x357adaaf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1adab4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1d3adab6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x529adab8, SUC_lexicon_tags_20 },
    { 0xaf9adab9, SUC_lexicon_tags_2 },
    { 0x9a1adaba, SUC_lexicon_tags_14 },
    { 0xc1fadaba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x513adabe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeedadac0, SUC_lexicon_tags_11 },
    { 0x6afadac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa9adac3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9dadac5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3adac8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55adacd, SUC_lexicon_tags_11 },
    { 0x815adacd, SUC_lexicon_tags_27 },
    { 0xd31adacf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa37adad1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x287adad4, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159adada, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07adadd, SUC_lexicon_tags_30 },
    { 0x829adadd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23adae3, SUC_lexicon_tags_17 },
    { 0x37badae4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbdfadae6, SUC_lexicon_tags_4 },
    { 0xe03adae6, SUC_lexicon_tags_11 },
    { 0xdb3adae6, SUC_lexicon_tags_1 },
    { 0x2d5adae9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x13badaec, SUC_lexicon_tags_13 },
    { 0x353adaec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x263adaef, SUC_lexicon_tags_25 },
    { 0xfbdadaef, SUC_lexicon_tags_23 },
    { 0xacdadaf1, SUC_lexicon_tags_63 },
    { 0x17fadaf2, SUC_lexicon_tags_15 },
    { 0xe0fadaf3, SUC_lexicon_tags_160 },
    { 0xe63adaf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7adaf7, SUC_lexicon_tags_43 },
    { 0x8b1adaf8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x991adafb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fadb02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3adb07, SUC_lexicon_tags_124 },
    { 0x27adb08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81dadb0b, SUC_lexicon_tags_11 },
    { 0xc1badb0c, SUC_lexicon_tags_25 },
    { 0xf1fadb0d, SUC_lexicon_tags_15 },
    { 0xcf3adb0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfadb13, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771adb19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9adb1d, SUC_lexicon_tags_38 },
    { 0x3ebadb1d, SUC_lexicon_tags_10 },
    { 0x6c3adb1f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11adb22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993adb2a, SUC_lexicon_tags_21 },
    { 0x98dadb2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647adb2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x443adb31, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47adb3a, SUC_lexicon_tags_27 },
    { 0x847adb3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fadb41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58dadb44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbadb48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23dadb4a, SUC_lexicon_tags_17 },
    { 0x143adb4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29fadb4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fadb50, SUC_lexicon_tags_7 },
    { 0xe3badb50, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fadb54, SUC_lexicon_tags_19 },
    { 0x127adb55, SUC_lexicon_tags_12 },
    { 0x303adb56, SUC_lexicon_tags_141 },
    { 0xf69adb54, SUC_lexicon_tags_47 },
    { 0x5cfadb58, SUC_lexicon_tags_23 },
    { 0xfcdadb55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5adb62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307adb6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5badb6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbadb71, SUC_lexicon_tags_15 },
    { 0x7c7adb72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31adb79, SUC_lexicon_tags_444 },
    { 0x161adb7a, SUC_lexicon_tags_37 },
    { 0xe7fadb7a, SUC_lexicon_tags_2 },
    { 0xe1adb79, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2f9adb7e, SUC_lexicon_tags_84 },
    { 0xf27adb7f, SUC_lexicon_tags_39 },
    { 0x501adb80, SUC_lexicon_tags_48 },
    { 0xc23adb80, SUC_lexicon_tags_506 },
    { 0x1e5adb80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61adb86, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23adb8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4badb90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x91adb93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x237adb95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ebadb97, SUC_lexicon_tags_46 },
    { 0xc5dadb97, SUC_lexicon_tags_66 },
    { 0xb8dadb99, SUC_lexicon_tags_56 },
    { 0x71dadb9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5f5adb9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4badba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29badba6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddadba9, SUC_lexicon_tags_14 },
    { 0xa7badbaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9f5adbac, SUC_lexicon_tags_1 },
    { 0x835adbad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3badbb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fadbb5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25adbb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffadbbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c5adbbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf03adbbf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519adbc4, SUC_lexicon_tags_31 },
    { 0xe4dadbc5, SUC_lexicon_tags_31 },
    { 0x937adbc6, SUC_lexicon_tags_2 },
    { 0x23badbc4, SUC_lexicon_tags_21 },
    { 0x22badbc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9adbcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2fadbce, SUC_lexicon_tags_63 },
    { 0xad7adbcf, SUC_lexicon_tags_60 },
    { 0x48fadbd0, SUC_lexicon_tags_15 },
    { 0x3b3adbcf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x20badbd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe17adbd5, SUC_lexicon_tags_2 },
    { 0xfc3adbd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfadbd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3adbdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd5adbde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295adbe3, SUC_lexicon_tags_65 },
    { 0x5efadbe4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfadbe8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dadbec, SUC_lexicon_tags_0 },
    { 0xac9adbec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9adbf0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x181adbf2, SUC_lexicon_tags_33 },
    { 0x477adbf2, SUC_lexicon_tags_49 },
    { 0x1abadbf2, SUC_lexicon_tags_8 },
    { 0xfd3adbf5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49adbf9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x979adbfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dadc01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x257adc04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b1adc06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffadc0a, SUC_lexicon_tags_7 },
    { 0x5b7adc0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dadc0f, SUC_lexicon_tags_38 },
    { 0xc99adc0f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9badc15, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5adc1d, SUC_lexicon_tags_2 },
    { 0xf2fadc1e, SUC_lexicon_tags_25 },
    { 0x373adc1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddadc24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdadc29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x763adc2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41badc39, SUC_lexicon_tags_15 },
    { 0xafbadc39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbebadc3c, SUC_lexicon_tags_37 },
    { 0x4f1adc3c, SUC_lexicon_tags_11 },
    { 0x861adc3d, SUC_lexicon_tags_37 },
    { 0x641adc3f, SUC_lexicon_tags_61 },
    { 0xf7fadc40, SUC_lexicon_tags_17 },
    { 0xecdadc40, SUC_lexicon_tags_7 },
    { 0xbb1adc42, SUC_lexicon_tags_33 },
    { 0x9e5adc41, SUC_lexicon_tags_0 },
    { 0x5d9adc3f, SUC_lexicon_tags_14 },
    { 0x1edadc3c, SUC_lexicon_tags_9 },
    { 0x8d3adc45, SUC_lexicon_tags_12 },
    { 0xb0dadc47, SUC_lexicon_tags_1 },
    { 0xeffadc47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfadc4d, SUC_lexicon_tags_63 },
    { 0xde1adc4e, SUC_lexicon_tags_46 },
    { 0xae3adc4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x83adc51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9adc59, SUC_lexicon_tags_10 },
    { 0x491adc5a, SUC_lexicon_tags_15 },
    { 0x501adc5b, SUC_lexicon_tags_33 },
    { 0x76fadc5a, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abadc60, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf83adc62, SUC_lexicon_tags_33 },
    { 0xf35adc63, SUC_lexicon_tags_47 },
    { 0x30badc64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd59adc66, SUC_lexicon_tags_25 },
    { 0x339adc67, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4fbadc69, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfefadc6b, SUC_lexicon_tags_27 },
    { 0x86fadc6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165adc70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65adc73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x895adc75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83adc77, SUC_lexicon_tags_7 },
    { 0x61badc77, SUC_lexicon_tags_26 },
    { 0x6e1adc78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca7adc7b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd7dadc7d, SUC_lexicon_tags_65 },
    { 0xba1adc7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x527adc80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4badc82, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61badc87, SUC_lexicon_tags_25 },
    { 0x3cfadc88, SUC_lexicon_tags_63 },
    { 0xf1badc89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1adc8f, SUC_lexicon_tags_45 },
    { 0xdd3adc8f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fadc94, SUC_lexicon_tags_33 },
    { 0x775adc95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x835adc98, SUC_lexicon_tags_17 },
    { 0xde3adc98, SUC_lexicon_tags_7 },
    { 0x8b9adc99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37badca0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf9adca2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe53adca4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8badca9, SUC_lexicon_tags_17 },
    { 0xed9adca9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdadcaf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbadcb3, SUC_lexicon_tags_14 },
    { 0x779adcb4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dadcb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0badcbf, SUC_lexicon_tags_49 },
    { 0xe81adcc0, SUC_lexicon_tags_27 },
    { 0x5c5adcc0, SUC_lexicon_tags_1 },
    { 0x247adcc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfadcc4, SUC_lexicon_tags_37 },
    { 0x4a3adcc5, SUC_lexicon_tags_30 },
    { 0xd55adcc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa9adcc8, SUC_lexicon_tags_17 },
    { 0xbc7adcc9, SUC_lexicon_tags_2 },
    { 0x321adcc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x725adccc, SUC_lexicon_tags_9 },
    { 0x24dadccd, SUC_lexicon_tags_27 },
    { 0xb5adccc, SUC_lexicon_tags_17 },
    { 0x389adcce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1adcd2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509adcd7, SUC_lexicon_tags_14 },
    { 0x889adcd7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf43adcda, SUC_lexicon_tags_27 },
    { 0xf33adcdb, SUC_lexicon_tags_16 },
    { 0x3e5adcdb, SUC_lexicon_tags_65 },
    { 0x6ebadcdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a1adcdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739adce4, SUC_lexicon_tags_11 },
    { 0x29badce5, SUC_lexicon_tags_29 },
    { 0x4d7adce4, SUC_lexicon_tags_17 },
    { 0xc73adce7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x295adcea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797adcef, SUC_lexicon_tags_87 },
    { 0x2a3adcef, SUC_lexicon_tags_7 },
    { 0x88fadcf0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963adcf5, SUC_lexicon_tags_38 },
    { 0x9e5adcf5, SUC_lexicon_tags_37 },
    { 0x7d3adcf7, SUC_lexicon_tags_23 },
    { 0xfdfadcf6, SUC_lexicon_tags_151 },
    { 0x293adcf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193adcfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdadd00, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcc1add02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15badd08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88fadd0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15add11, SUC_lexicon_tags_63 },
    { 0xc55add12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839add17, SUC_lexicon_tags_7 },
    { 0xf61add17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45add1c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71add23, SUC_lexicon_tags_26 },
    { 0x6cdadd23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x977add26, SUC_lexicon_tags_11 },
    { 0xb47add27, SUC_lexicon_tags_7 },
    { 0xa27add27, SUC_lexicon_tags_37 },
    { 0x66badd29, SUC_lexicon_tags_25 },
    { 0xcd5add2a, SUC_lexicon_tags_7 },
    { 0xb35add2b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701add35, SUC_lexicon_tags_28 },
    { 0xcfadd36, SUC_lexicon_tags_15 },
    { 0x5e3add35, SUC_lexicon_tags_49 },
    { 0xbf1add38, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fadd3d, SUC_lexicon_tags_12 },
    { 0xb11add3d, SUC_lexicon_tags_1 },
    { 0x1bbadd3f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x919add41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5badd43, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12badd49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7add4c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x111add4f, SUC_lexicon_tags_34 },
    { 0x513add50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fadd58, SUC_lexicon_tags_21 },
    { 0x6edadd58, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395add60, SUC_lexicon_tags_11 },
    { 0x9fbadd61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x807add63, SUC_lexicon_tags_1 },
    { 0x47dadd64, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5add6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59add6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adadd72, SUC_lexicon_tags_15 },
    { 0x975add72, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4d5add75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1add7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11add83, SUC_lexicon_tags_46 },
    { 0x251add84, SUC_lexicon_tags_7 },
    { 0xe4dadd85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6badd88, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711add8d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x83badd90, SUC_lexicon_tags_37 },
    { 0x25badd91, SUC_lexicon_tags_0 },
    { 0x52fadd91, SUC_lexicon_tags_27 },
    { 0xc8dadd93, SUC_lexicon_tags_11 },
    { 0xd0dadd94, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4fadd96, SUC_lexicon_tags_37 },
    { 0xe8dadd97, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x487add99, SUC_lexicon_tags_63 },
    { 0x6f3add99, SUC_lexicon_tags_17 },
    { 0x619add9a, SUC_lexicon_tags_49 },
    { 0xab3add9c, SUC_lexicon_tags_43 },
    { 0xb4dadd9a, SUC_lexicon_tags_95 },
    { 0x447add9e, SUC_lexicon_tags_17 },
    { 0xd4dadd9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7adda5, SUC_lexicon_tags_2 },
    { 0x501adda5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205addaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3addad, SUC_lexicon_tags_2 },
    { 0x69baddae, SUC_lexicon_tags_17 },
    { 0xeefaddaf, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x907addb1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9addb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47baddba, SUC_lexicon_tags_12 },
    { 0x739addbb, SUC_lexicon_tags_37 },
    { 0x7cdaddbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x669addbe, SUC_lexicon_tags_65 },
    { 0xf17addbe, SUC_lexicon_tags_1 },
    { 0x351addc0, SUC_lexicon_tags_13 },
    { 0x8f3addc1, SUC_lexicon_tags_4 },
    { 0xe23addc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05addca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x80faddcd, SUC_lexicon_tags_6 },
    { 0x687addce, SUC_lexicon_tags_19 },
    { 0x43baddcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc13addd1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf05addd3, SUC_lexicon_tags_11 },
    { 0x637addd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993adddb, SUC_lexicon_tags_25 },
    { 0xd23adddb, SUC_lexicon_tags_13 },
    { 0xa71adddc, SUC_lexicon_tags_45 },
    { 0x315addde, SUC_lexicon_tags_45 },
    { 0xfa7adddf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc03adde1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49adde9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc9addeb, SUC_lexicon_tags_85 },
    { 0x17faddec, SUC_lexicon_tags_30 },
    { 0xfdfaddec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x933addf0, SUC_lexicon_tags_8 },
    { 0x43baddf1, SUC_lexicon_tags_11 },
    { 0x8daddf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5addf6, SUC_lexicon_tags_12 },
    { 0x267addf7, SUC_lexicon_tags_6 },
    { 0xc4baddf8, SUC_lexicon_tags_37 },
    { 0x619addf7, SUC_lexicon_tags_15 },
    { 0x45daddfa, SUC_lexicon_tags_83 },
    { 0x7f5addfb, SUC_lexicon_tags_25 },
    { 0x78daddf9, SUC_lexicon_tags_37 },
    { 0x427addfd, SUC_lexicon_tags_17 },
    { 0xbffaddfc, SUC_lexicon_tags_45 },
    { 0xeb1addfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3ade04, SUC_lexicon_tags_87 },
    { 0xa1fade05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7ade09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25ade0d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x97fade0f, SUC_lexicon_tags_27 },
    { 0x461ade10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7ade16, SUC_lexicon_tags_1 },
    { 0xd3dade16, SUC_lexicon_tags_2 },
    { 0xc6bade18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x363ade1b, SUC_lexicon_tags_8 },
    { 0x641ade1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339ade21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf99ade23, SUC_lexicon_tags_33 },
    { 0xaf9ade24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf8bade26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ade2d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x67dade30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dade34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1ade37, SUC_lexicon_tags_12 },
    { 0xc59ade37, SUC_lexicon_tags_66 },
    { 0x795ade39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1ade3f, SUC_lexicon_tags_23 },
    { 0x177ade40, SUC_lexicon_tags_17 },
    { 0xa3bade3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135ade46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271ade4a, SUC_lexicon_tags_39 },
    { 0x1d7ade4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcedade4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1ade50, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bade54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bade57, SUC_lexicon_tags_13 },
    { 0x72dade58, SUC_lexicon_tags_1 },
    { 0xaedade58, SUC_lexicon_tags_48 },
    { 0x89ade5a, SUC_lexicon_tags_33 },
    { 0x9dfade5b, SUC_lexicon_tags_33 },
    { 0x379ade5c, SUC_lexicon_tags_15 },
    { 0x681ade5d, SUC_lexicon_tags_30 },
    { 0x6ffade5e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2e1ade60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95ade65, SUC_lexicon_tags_69 },
    { 0x207ade66, SUC_lexicon_tags_33 },
    { 0x89ade66, SUC_lexicon_tags_0 },
    { 0x873ade66, SUC_lexicon_tags_17 },
    { 0x5bdade69, SUC_lexicon_tags_37 },
    { 0x2dade6a, SUC_lexicon_tags_46 },
    { 0x1cfade6b, SUC_lexicon_tags_43 },
    { 0x1abade6c, SUC_lexicon_tags_30 },
    { 0x5c1ade6c, SUC_lexicon_tags_9 },
    { 0x6b9ade67, SUC_lexicon_tags_2 },
    { 0xf61ade6f, SUC_lexicon_tags_65 },
    { 0xc2bade70, SUC_lexicon_tags_2 },
    { 0x817ade6d, SUC_lexicon_tags_63 },
    { 0xffade72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fade78, SUC_lexicon_tags_46 },
    { 0xaf7ade79, SUC_lexicon_tags_35 },
    { 0x2fade7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679ade7e, SUC_lexicon_tags_2 },
    { 0xabdade7f, SUC_lexicon_tags_2 },
    { 0x21ade7f, SUC_lexicon_tags_48 },
    { 0xeb9ade81, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa31ade83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28dade8a, SUC_lexicon_tags_49 },
    { 0x1c5ade8b, SUC_lexicon_tags_1 },
    { 0x32fade8b, SUC_lexicon_tags_64 },
    { 0xebdade8b, SUC_lexicon_tags_14 },
    { 0x9ebade8c, SUC_lexicon_tags_37 },
    { 0x829ade8f, SUC_lexicon_tags_47 },
    { 0x59ade90, SUC_lexicon_tags_33 },
    { 0x9f3ade8f, SUC_lexicon_tags_1 },
    { 0x489ade90, SUC_lexicon_tags_9 },
    { 0x689ade93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ade96, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ade9b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadadea3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x617adea5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39adeab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37dadead, SUC_lexicon_tags_33 },
    { 0xee5adeae, SUC_lexicon_tags_25 },
    { 0x939adeaf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f5adeb1, SUC_lexicon_tags_29 },
    { 0x615adeb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddadeb6, SUC_lexicon_tags_60 },
    { 0xd53adeb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x90badeba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfadebe, SUC_lexicon_tags_17 },
    { 0x8f7adebf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa9adec1, SUC_lexicon_tags_23 },
    { 0x257adec2, SUC_lexicon_tags_29 },
    { 0x61adec2, SUC_lexicon_tags_63 },
    { 0xe3fadec4, SUC_lexicon_tags_12 },
    { 0xd15adec5, SUC_lexicon_tags_17 },
    { 0x4c5adec6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927adecc, SUC_lexicon_tags_16 },
    { 0xa39adecd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37aded0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3aded6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3aded9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465adedd, SUC_lexicon_tags_25 },
    { 0x373adedd, SUC_lexicon_tags_25 },
    { 0xa01adede, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dadeed, SUC_lexicon_tags_14 },
    { 0x1d1adeed, SUC_lexicon_tags_12 },
    { 0xc93adeef, SUC_lexicon_tags_7 },
    { 0x9ddadef0, SUC_lexicon_tags_25 },
    { 0xb33adeef, SUC_lexicon_tags_44 },
    { 0x6cbadeef, SUC_lexicon_tags_11 },
    { 0x997adef3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x569adef6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afadefb, SUC_lexicon_tags_17 },
    { 0x18badefc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69adeff, SUC_lexicon_tags_7 },
    { 0x28fadeff, SUC_lexicon_tags_174 },
    { 0xfbbadf01, SUC_lexicon_tags_19 },
    { 0x2b9adf02, SUC_lexicon_tags_0 },
    { 0x7adf03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573adf0e, SUC_lexicon_tags_10 },
    { 0x531adf0f, SUC_lexicon_tags_38 },
    { 0x25adf10, SUC_lexicon_tags_66 },
    { 0xc37adf11, SUC_lexicon_tags_25 },
    { 0x769adf12, SUC_lexicon_tags_17 },
    { 0xbbbadf10, SUC_lexicon_tags_7 },
    { 0x51adf14, SUC_lexicon_tags_38 },
    { 0xc79adf11, SUC_lexicon_tags_11 },
    { 0x55fadf16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3adf1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x585adf1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x197adf20, SUC_lexicon_tags_33 },
    { 0xb3adf21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x571adf23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9adf29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9adf2c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39adf30, SUC_lexicon_tags_63 },
    { 0x9c5adf30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937adf3a, SUC_lexicon_tags_7 },
    { 0x89badf3b, SUC_lexicon_tags_63 },
    { 0x54dadf3c, SUC_lexicon_tags_13 },
    { 0xd07adf3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94dadf41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdadf46, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9adf49, SUC_lexicon_tags_7 },
    { 0xed9adf4a, SUC_lexicon_tags_14 },
    { 0x11adf4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7ffadf4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x847adf4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fadf52, SUC_lexicon_tags_2 },
    { 0x919adf53, SUC_lexicon_tags_12 },
    { 0x9efadf52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63adf58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31adf5e, SUC_lexicon_tags_6 },
    { 0xf83adf5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9bfadf61, SUC_lexicon_tags_17 },
    { 0x88fadf61, SUC_lexicon_tags_49 },
    { 0xdddadf61, SUC_lexicon_tags_6 },
    { 0x3e9adf64, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebadf6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9adf6f, SUC_lexicon_tags_25 },
    { 0xf11adf70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4e5adf72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87adf75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffadf7c, SUC_lexicon_tags_85 },
    { 0xeddadf7c, SUC_lexicon_tags_31 },
    { 0x239adf7d, SUC_lexicon_tags_27 },
    { 0xed5adf7e, SUC_lexicon_tags_29 },
    { 0x4b3adf80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82badf86, SUC_lexicon_tags_38 },
    { 0x9ebadf86, SUC_lexicon_tags_17 },
    { 0x15fadf88, SUC_lexicon_tags_55 },
    { 0x711adf89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd21adf8b, SUC_lexicon_tags_37 },
    { 0x871adf8c, SUC_lexicon_tags_21 },
    { 0x193adf8c, SUC_lexicon_tags_39 },
    { 0xf8badf8e, SUC_lexicon_tags_1 },
    { 0x2dadf8c, SUC_lexicon_tags_43 },
    { 0xfa1adf8e, SUC_lexicon_tags_29 },
    { 0xfebadf8c, SUC_lexicon_tags_14 },
    { 0x98fadf92, SUC_lexicon_tags_49 },
    { 0xb6dadf8f, SUC_lexicon_tags_23 },
    { 0x61badf91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7adf98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d7adf9a, SUC_lexicon_tags_12 },
    { 0x515adf9b, SUC_lexicon_tags_25 },
    { 0x10fadf9a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953adfa4, SUC_lexicon_tags_12 },
    { 0x2bfadfa5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07adfad, SUC_lexicon_tags_13 },
    { 0x4dbadfae, SUC_lexicon_tags_14 },
    { 0xd65adfaf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fadfb8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efadfbe, SUC_lexicon_tags_84 },
    { 0xeb9adfbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa41adfc1, SUC_lexicon_tags_46 },
    { 0x615adfc1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe01adfc4, SUC_lexicon_tags_37 },
    { 0xf5badfc5, SUC_lexicon_tags_7 },
    { 0x33fadfc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81fadfc9, SUC_lexicon_tags_38 },
    { 0x60dadfc9, SUC_lexicon_tags_37 },
    { 0x9a1adfcb, SUC_lexicon_tags_14 },
    { 0x9cbadfcc, SUC_lexicon_tags_14 },
    { 0x601adfcc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3f7adfcf, SUC_lexicon_tags_17 },
    { 0xeafadfcf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x747adfd2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fadfe1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1badfe5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xaf9adfe7, SUC_lexicon_tags_45 },
    { 0xb7fadfe7, SUC_lexicon_tags_25 },
    { 0xcddadfe9, SUC_lexicon_tags_47 },
    { 0xf4badfea, SUC_lexicon_tags_28 },
    { 0xd7adfe7, SUC_lexicon_tags_33 },
    { 0x4c9adfe8, SUC_lexicon_tags_82 },
    { 0xc23adfed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x573adff0, SUC_lexicon_tags_17 },
    { 0xc9fadff1, SUC_lexicon_tags_37 },
    { 0xc8badff2, SUC_lexicon_tags_101 },
    { 0xfb3adff3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1adff7, SUC_lexicon_tags_47 },
    { 0xba5adff7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b9adffa, SUC_lexicon_tags_11 },
    { 0x7c7adffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x293adffe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x221ae001, SUC_lexicon_tags_84 },
    { 0x22fae002, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fae008, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fae00c, SUC_lexicon_tags_15 },
    { 0x635ae00c, SUC_lexicon_tags_46 },
    { 0xd2fae00e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x967ae010, SUC_lexicon_tags_37 },
    { 0x3c3ae011, SUC_lexicon_tags_49 },
    { 0x161ae011, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05ae015, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x639ae017, SUC_lexicon_tags_12 },
    { 0xba9ae017, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedae01c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x611ae01e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbae022, SUC_lexicon_tags_17 },
    { 0xccbae023, SUC_lexicon_tags_1 },
    { 0x9dae023, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x161ae027, SUC_lexicon_tags_30 },
    { 0xfbbae028, SUC_lexicon_tags_2 },
    { 0xb33ae029, SUC_lexicon_tags_25 },
    { 0x69ae02a, SUC_lexicon_tags_0 },
    { 0x20bae02b, SUC_lexicon_tags_37 },
    { 0x6ddae028, SUC_lexicon_tags_17 },
    { 0x607ae02b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9ae030, SUC_lexicon_tags_37 },
    { 0xc81ae031, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x98dae034, SUC_lexicon_tags_25 },
    { 0xd75ae035, SUC_lexicon_tags_12 },
    { 0xb89ae034, SUC_lexicon_tags_38 },
    { 0x323ae037, SUC_lexicon_tags_2 },
    { 0x1ebae037, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6dfae03a, SUC_lexicon_tags_38 },
    { 0xf87ae03b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61ae042, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x721ae044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7ae048, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3ae04d, SUC_lexicon_tags_4 },
    { 0xe9fae04e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xac3ae050, SUC_lexicon_tags_2 },
    { 0xaa7ae051, SUC_lexicon_tags_35 },
    { 0x853ae050, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7ae056, SUC_lexicon_tags_2 },
    { 0xd23ae056, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707ae05b, SUC_lexicon_tags_13 },
    { 0xdffae05b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9ae05f, SUC_lexicon_tags_6 },
    { 0xf0dae060, SUC_lexicon_tags_60 },
    { 0x88dae060, SUC_lexicon_tags_39 },
    { 0x1a1ae060, SUC_lexicon_tags_63 },
    { 0x811ae061, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3ae067, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ae06a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ae06f, SUC_lexicon_tags_16 },
    { 0xf55ae06f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x587ae072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba1ae074, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823ae079, SUC_lexicon_tags_22 },
    { 0xcd7ae07a, SUC_lexicon_tags_138 },
    { 0xb01ae07b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791ae080, SUC_lexicon_tags_17 },
    { 0xf6fae080, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdae086, SUC_lexicon_tags_2 },
    { 0x453ae086, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe71ae089, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x713ae08b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad1ae08d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf77ae08f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x653ae091, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97ae095, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299ae09a, SUC_lexicon_tags_27 },
    { 0x863ae09a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747ae0a4, SUC_lexicon_tags_10 },
    { 0xb9ae0a4, SUC_lexicon_tags_6 },
    { 0xdbfae0a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761ae0af, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fae0b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5ae0b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafbae0bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551ae0c3, SUC_lexicon_tags_1 },
    { 0xea3ae0c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75dae0c8, SUC_lexicon_tags_12 },
    { 0xe07ae0c9, SUC_lexicon_tags_11 },
    { 0x33dae0ca, SUC_lexicon_tags_33 },
    { 0x14fae0cb, SUC_lexicon_tags_37 },
    { 0xc85ae0cb, SUC_lexicon_tags_23 },
    { 0x6c5ae0cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf0dae0cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe3ae0d1, SUC_lexicon_tags_23 },
    { 0xb01ae0d2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dae0d5, SUC_lexicon_tags_33 },
    { 0x1b9ae0d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbb3ae0d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29ae0de, SUC_lexicon_tags_37 },
    { 0xc61ae0de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x21bae0e1, SUC_lexicon_tags_12 },
    { 0x1a3ae0e2, SUC_lexicon_tags_63 },
    { 0xe2dae0e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fae0e6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ae0ea, SUC_lexicon_tags_4 },
    { 0x5f3ae0eb, SUC_lexicon_tags_16 },
    { 0x147ae0ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3dae0ee, SUC_lexicon_tags_2 },
    { 0x1ddae0ef, SUC_lexicon_tags_2 },
    { 0xce5ae0f0, SUC_lexicon_tags_2 },
    { 0x753ae0f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x431ae0f3, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787ae0f7, SUC_lexicon_tags_37 },
    { 0x21ae0f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x91fae0fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59ae0fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa3ae0ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5c1ae101, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbae10a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dae10d, SUC_lexicon_tags_2 },
    { 0x9cfae10e, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831ae118, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddae11c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2adae11e, SUC_lexicon_tags_11 },
    { 0xf41ae11f, SUC_lexicon_tags_15 },
    { 0xab7ae120, SUC_lexicon_tags_15 },
    { 0xad7ae11f, SUC_lexicon_tags_2 },
    { 0x55ae120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca7ae124, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xe71ae126, SUC_lexicon_tags_11 },
    { 0x389ae127, SUC_lexicon_tags_23 },
    { 0x757ae126, SUC_lexicon_tags_43 },
    { 0x2a3ae129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc7ae12b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7ae135, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adae13a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3ae143, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8efae145, SUC_lexicon_tags_33 },
    { 0xf25ae145, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x797ae148, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3f9ae14a, SUC_lexicon_tags_4 },
    { 0xa8dae14a, SUC_lexicon_tags_12 },
    { 0xf91ae14a, SUC_lexicon_tags_28 },
    { 0x5afae14c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x785ae150, SUC_lexicon_tags_33 },
    { 0x417ae151, SUC_lexicon_tags_39 },
    { 0xc4bae152, SUC_lexicon_tags_212 },
    { 0x805ae153, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65ae157, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ae15a, SUC_lexicon_tags_1 },
    { 0x5d1ae15b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bae15f, SUC_lexicon_tags_15 },
    { 0xa03ae160, SUC_lexicon_tags_11 },
    { 0x9efae161, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ae16d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9ae170, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd01ae172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d9ae174, SUC_lexicon_tags_68 },
    { 0x5bbae175, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa17ae177, SUC_lexicon_tags_7 },
    { 0x3d3ae177, SUC_lexicon_tags_29 },
    { 0x521ae179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x801ae17b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9ae182, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dae187, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5ae18a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdae190, SUC_lexicon_tags_14 },
    { 0x58dae191, SUC_lexicon_tags_16 },
    { 0xe43ae192, SUC_lexicon_tags_15 },
    { 0x39fae190, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabae19f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bae1a6, SUC_lexicon_tags_9 },
    { 0x4b3ae1a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3ae1aa, SUC_lexicon_tags_1 },
    { 0x37bae1aa, SUC_lexicon_tags_23 },
    { 0x3cfae1ac, SUC_lexicon_tags_29 },
    { 0x669ae1ac, SUC_lexicon_tags_1 },
    { 0xec1ae1ae, SUC_lexicon_tags_25 },
    { 0xdf3ae1af, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39ae1b5, SUC_lexicon_tags_11 },
    { 0x41ae1b6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885ae1ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7ae1c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x541ae1c4, SUC_lexicon_tags_23 },
    { 0x47fae1c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ae1ca, SUC_lexicon_tags_17 },
    { 0x565ae1ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5ae1d2, SUC_lexicon_tags_920 },
    { 0, NULL },
    { 0xa2bae1d4, SUC_lexicon_tags_2 },
    { 0x32bae1d5, SUC_lexicon_tags_0 },
    { 0xe0fae1d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf7fae1d8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fae1db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9ae1de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x683ae1e0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15ae1e3, SUC_lexicon_tags_45 },
    { 0xb0dae1e4, SUC_lexicon_tags_28 },
    { 0xa49ae1e5, SUC_lexicon_tags_15 },
    { 0x447ae1e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dae1ed, SUC_lexicon_tags_25 },
    { 0xdfbae1ee, SUC_lexicon_tags_11 },
    { 0x71fae1ee, SUC_lexicon_tags_37 },
    { 0x6bae1f0, SUC_lexicon_tags_15 },
    { 0x431ae1f1, SUC_lexicon_tags_6 },
    { 0x389ae1f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x53bae1f5, SUC_lexicon_tags_27 },
    { 0x7d5ae1f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cbae1f8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dae204, SUC_lexicon_tags_16 },
    { 0xb3ae205, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fae208, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fae20b, SUC_lexicon_tags_45 },
    { 0x133ae20b, SUC_lexicon_tags_27 },
    { 0xff7ae20b, SUC_lexicon_tags_27 },
    { 0x2c7ae20c, SUC_lexicon_tags_14 },
    { 0x4f3ae20f, SUC_lexicon_tags_7 },
    { 0xc03ae210, SUC_lexicon_tags_49 },
    { 0xba9ae210, SUC_lexicon_tags_25 },
    { 0x5bbae211, SUC_lexicon_tags_16 },
    { 0x56bae213, SUC_lexicon_tags_27 },
    { 0x719ae210, SUC_lexicon_tags_37 },
    { 0x4d5ae215, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc6fae217, SUC_lexicon_tags_11 },
    { 0x4e1ae218, SUC_lexicon_tags_9 },
    { 0xd4fae219, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51ae21c, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfae21f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bae224, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa35ae226, SUC_lexicon_tags_60 },
    { 0x567ae227, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bae22c, SUC_lexicon_tags_12 },
    { 0x80dae22c, SUC_lexicon_tags_14 },
    { 0x2e1ae22e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1ae231, SUC_lexicon_tags_1 },
    { 0xaadae232, SUC_lexicon_tags_37 },
    { 0x501ae231, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331ae23d, SUC_lexicon_tags_17 },
    { 0xc43ae23d, SUC_lexicon_tags_37 },
    { 0xb19ae23f, SUC_lexicon_tags_23 },
    { 0x1b7ae240, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc91ae242, SUC_lexicon_tags_33 },
    { 0xdbae243, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533ae255, SUC_lexicon_tags_15 },
    { 0x785ae256, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x387ae258, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x661ae25a, SUC_lexicon_tags_17 },
    { 0xdae25b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e7ae25d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc7ae25f, SUC_lexicon_tags_63 },
    { 0xca7ae260, SUC_lexicon_tags_40 },
    { 0xadfae260, SUC_lexicon_tags_59 },
    { 0x627ae260, SUC_lexicon_tags_9 },
    { 0xadbae261, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bae267, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7ae26a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa79ae26c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa35ae26e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ae277, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1ae27c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdae282, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bae287, SUC_lexicon_tags_38 },
    { 0xc87ae288, SUC_lexicon_tags_30 },
    { 0x7c7ae288, SUC_lexicon_tags_11 },
    { 0x253ae289, SUC_lexicon_tags_23 },
    { 0x181ae28a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebae28f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x45ae291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd31ae293, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab1ae295, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27ae299, SUC_lexicon_tags_60 },
    { 0xd79ae29a, SUC_lexicon_tags_33 },
    { 0x141ae29b, SUC_lexicon_tags_45 },
    { 0xaa5ae29a, SUC_lexicon_tags_11 },
    { 0x86fae29d, SUC_lexicon_tags_87 },
    { 0xd23ae29d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa5fae2a0, SUC_lexicon_tags_8 },
    { 0x13fae2a1, SUC_lexicon_tags_17 },
    { 0x1b9ae2a1, SUC_lexicon_tags_2 },
    { 0xdedae2a3, SUC_lexicon_tags_2 },
    { 0x93dae2a3, SUC_lexicon_tags_0 },
    { 0xefae2a5, SUC_lexicon_tags_11 },
    { 0xb07ae2a3, SUC_lexicon_tags_47 },
    { 0x6f9ae2a6, SUC_lexicon_tags_17 },
    { 0x483ae2a4, SUC_lexicon_tags_15 },
    { 0x907ae2a6, SUC_lexicon_tags_12 },
    { 0xfadae2a3, SUC_lexicon_tags_27 },
    { 0x2fdae2a8, SUC_lexicon_tags_12 },
    { 0x727ae2ac, SUC_lexicon_tags_2 },
    { 0x3a7ae2ad, SUC_lexicon_tags_6 },
    { 0x485ae2aa, SUC_lexicon_tags_14 },
    { 0x9f1ae2a5, SUC_lexicon_tags_25 },
    { 0x2fbae2ab, SUC_lexicon_tags_7 },
    { 0x33dae2af, SUC_lexicon_tags_0 },
    { 0xbe9ae2b2, SUC_lexicon_tags_15 },
    { 0x36fae2b2, SUC_lexicon_tags_29 },
    { 0xecfae2b4, SUC_lexicon_tags_15 },
    { 0x82bae2b5, SUC_lexicon_tags_17 },
    { 0xbd9ae2a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebae2b9, SUC_lexicon_tags_37 },
    { 0x88dae2ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4a3ae2bc, SUC_lexicon_tags_43 },
    { 0x297ae2bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x481ae2bf, SUC_lexicon_tags_19 },
    { 0x82bae2c0, SUC_lexicon_tags_37 },
    { 0x6cfae2c0, SUC_lexicon_tags_27 },
    { 0x991ae2c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf7ae2c4, SUC_lexicon_tags_47 },
    { 0x653ae2c5, SUC_lexicon_tags_17 },
    { 0xaedae2c4, SUC_lexicon_tags_23 },
    { 0xe03ae2c7, SUC_lexicon_tags_1 },
    { 0xe59ae2c7, SUC_lexicon_tags_12 },
    { 0xdd1ae2c9, SUC_lexicon_tags_1 },
    { 0x90bae2c7, SUC_lexicon_tags_55 },
    { 0x5b9ae2cb, SUC_lexicon_tags_37 },
    { 0xc25ae2cc, SUC_lexicon_tags_25 },
    { 0x215ae2ca, SUC_lexicon_tags_37 },
    { 0x2e1ae2ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7ae2d1, SUC_lexicon_tags_91 },
    { 0x2bfae2d2, SUC_lexicon_tags_99 },
    { 0xe2bae2d3, SUC_lexicon_tags_38 },
    { 0x489ae2d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5ae2db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ae2e1, SUC_lexicon_tags_17 },
    { 0x991ae2e2, SUC_lexicon_tags_11 },
    { 0xf19ae2e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1ae2e6, SUC_lexicon_tags_48 },
    { 0xa1fae2e6, SUC_lexicon_tags_3 },
    { 0x2c9ae2e8, SUC_lexicon_tags_40 },
    { 0xdefae2e9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x86dae2ec, SUC_lexicon_tags_60 },
    { 0x96dae2ed, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0x63fae2f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x35ae2f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dae2f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01ae2f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23ae2fd, SUC_lexicon_tags_12 },
    { 0x7f3ae2fd, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ae305, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193ae30a, SUC_lexicon_tags_37 },
    { 0x547ae30b, SUC_lexicon_tags_12 },
    { 0x401ae30c, SUC_lexicon_tags_6 },
    { 0x229ae30b, SUC_lexicon_tags_6 },
    { 0x97fae30e, SUC_lexicon_tags_30 },
    { 0x953ae30b, SUC_lexicon_tags_11 },
    { 0x7f5ae30d, SUC_lexicon_tags_15 },
    { 0x1d5ae311, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x151ae313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1ae316, SUC_lexicon_tags_32 },
    { 0x1afae317, SUC_lexicon_tags_49 },
    { 0x39dae318, SUC_lexicon_tags_30 },
    { 0xbb1ae318, SUC_lexicon_tags_23 },
    { 0xa75ae31a, SUC_lexicon_tags_13 },
    { 0x965ae31a, SUC_lexicon_tags_17 },
    { 0xcbae316, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c5ae31e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5d5ae320, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9ae322, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309ae327, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x59dae329, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03ae32e, SUC_lexicon_tags_37 },
    { 0x329ae32f, SUC_lexicon_tags_19 },
    { 0xe45ae330, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5ae333, SUC_lexicon_tags_6 },
    { 0x119ae333, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dae33b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdae33e, SUC_lexicon_tags_0 },
    { 0x31ae33e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bae346, SUC_lexicon_tags_23 },
    { 0x571ae346, SUC_lexicon_tags_9 },
    { 0x90fae348, SUC_lexicon_tags_1 },
    { 0xa73ae347, SUC_lexicon_tags_38 },
    { 0xd15ae34a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217ae34f, SUC_lexicon_tags_11 },
    { 0xcbbae350, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdfae352, SUC_lexicon_tags_34 },
    { 0xb5bae353, SUC_lexicon_tags_44 },
    { 0x663ae353, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3ae357, SUC_lexicon_tags_2 },
    { 0xdddae357, SUC_lexicon_tags_1 },
    { 0x899ae359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dae35d, SUC_lexicon_tags_27 },
    { 0x7c1ae35e, SUC_lexicon_tags_15 },
    { 0x867ae35e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ae363, SUC_lexicon_tags_8 },
    { 0x3bae364, SUC_lexicon_tags_45 },
    { 0x899ae363, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffae36a, SUC_lexicon_tags_77 },
    { 0xbe7ae36b, SUC_lexicon_tags_43 },
    { 0xd53ae36b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ae373, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe61ae375, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef7ae377, SUC_lexicon_tags_15 },
    { 0x86dae377, SUC_lexicon_tags_45 },
    { 0xd4fae379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91ae37d, SUC_lexicon_tags_12 },
    { 0x47bae37d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1ae382, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757ae392, SUC_lexicon_tags_10 },
    { 0xf33ae392, SUC_lexicon_tags_30 },
    { 0x137ae394, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253ae398, SUC_lexicon_tags_11 },
    { 0xb61ae399, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bae39e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fae3a2, SUC_lexicon_tags_29 },
    { 0x825ae3a3, SUC_lexicon_tags_28 },
    { 0xca9ae3a2, SUC_lexicon_tags_28 },
    { 0xebbae3a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x453ae3a8, SUC_lexicon_tags_43 },
    { 0xecfae3a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567ae3ad, SUC_lexicon_tags_1 },
    { 0xa19ae3ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3ae3b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dae3b5, SUC_lexicon_tags_0 },
    { 0x1b1ae3b6, SUC_lexicon_tags_11 },
    { 0x2afae3b7, SUC_lexicon_tags_12 },
    { 0x697ae3b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdae3c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfae3cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfae3ce, SUC_lexicon_tags_4 },
    { 0x943ae3cf, SUC_lexicon_tags_17 },
    { 0xcc9ae3d0, SUC_lexicon_tags_12 },
    { 0xdcbae3ce, SUC_lexicon_tags_63 },
    { 0x8c7ae3d2, SUC_lexicon_tags_26 },
    { 0xcf5ae3d3, SUC_lexicon_tags_60 },
    { 0x7dbae3ce, SUC_lexicon_tags_82 },
    { 0x55ae3ce, SUC_lexicon_tags_17 },
    { 0x639ae3d4, SUC_lexicon_tags_35 },
    { 0x7bbae3d7, SUC_lexicon_tags_63 },
    { 0x679ae3d7, SUC_lexicon_tags_27 },
    { 0x979ae3d1, SUC_lexicon_tags_35 },
    { 0xd79ae3d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747ae3de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bae3e5, SUC_lexicon_tags_25 },
    { 0xe1ae3e6, SUC_lexicon_tags_1 },
    { 0x11ae3e7, SUC_lexicon_tags_83 },
    { 0x6cfae3e8, SUC_lexicon_tags_24 },
    { 0x5ae3e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edae3ed, SUC_lexicon_tags_12 },
    { 0xcd1ae3ed, SUC_lexicon_tags_2 },
    { 0x34fae3ef, SUC_lexicon_tags_38 },
    { 0xdfbae3ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f9ae3f2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fae3f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebae3f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddae401, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x511ae404, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ae408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fae40b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abae411, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x98bae413, SUC_lexicon_tags_0 },
    { 0x5e9ae414, SUC_lexicon_tags_7 },
    { 0x5c1ae414, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4dfae417, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x249ae419, SUC_lexicon_tags_77 },
    { 0xb87ae419, SUC_lexicon_tags_15 },
    { 0xda5ae41b, SUC_lexicon_tags_13 },
    { 0x279ae41c, SUC_lexicon_tags_17 },
    { 0x567ae41b, SUC_lexicon_tags_70 },
    { 0xc93ae41e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bae423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dae428, SUC_lexicon_tags_37 },
    { 0xe7fae428, SUC_lexicon_tags_37 },
    { 0x5f9ae42a, SUC_lexicon_tags_25 },
    { 0xc9ae42b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5ae430, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3dae432, SUC_lexicon_tags_29 },
    { 0xd8bae432, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4f9ae435, SUC_lexicon_tags_96 },
    { 0xcd7ae436, SUC_lexicon_tags_37 },
    { 0xb15ae436, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x251ae43a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x539ae43c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ae440, SUC_lexicon_tags_15 },
    { 0xce3ae441, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ae446, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xeabae449, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11dae44c, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x8bae44e, SUC_lexicon_tags_39 },
    { 0xdd7ae44f, SUC_lexicon_tags_1479 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dae452, SUC_lexicon_tags_11 },
    { 0xe59ae453, SUC_lexicon_tags_41 },
    { 0x527ae454, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebae458, SUC_lexicon_tags_12 },
    { 0x5ae458, SUC_lexicon_tags_1 },
    { 0xa0bae458, SUC_lexicon_tags_27 },
    { 0x1ebae45b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7ae45e, SUC_lexicon_tags_206 },
    { 0x13ae45f, SUC_lexicon_tags_11 },
    { 0x389ae460, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadae466, SUC_lexicon_tags_25 },
    { 0xbf3ae467, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971ae46c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2edae46e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x49bae470, SUC_lexicon_tags_25 },
    { 0x1ddae471, SUC_lexicon_tags_8 },
    { 0x661ae472, SUC_lexicon_tags_15 },
    { 0x195ae471, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x143ae475, SUC_lexicon_tags_82 },
    { 0x99fae476, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb93ae478, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287ae481, SUC_lexicon_tags_25 },
    { 0x635ae482, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bae487, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c1ae489, SUC_lexicon_tags_0 },
    { 0xf1dae489, SUC_lexicon_tags_13 },
    { 0xe53ae48b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323ae48f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e9ae491, SUC_lexicon_tags_27 },
    { 0xbf9ae491, SUC_lexicon_tags_15 },
    { 0xf4bae493, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x165ae496, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661ae49a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95ae49d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d3ae49f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dae4a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc49ae4a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb17ae4a7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc53ae4a9, SUC_lexicon_tags_15 },
    { 0x30dae4aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x149ae4ac, SUC_lexicon_tags_33 },
    { 0x21ae4ad, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3ae4b3, SUC_lexicon_tags_70 },
    { 0xee9ae4b4, SUC_lexicon_tags_37 },
    { 0xae1ae4b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefae4b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ae4bd, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedae4c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5b5ae4c3, SUC_lexicon_tags_3 },
    { 0xdffae4c4, SUC_lexicon_tags_33 },
    { 0x703ae4c5, SUC_lexicon_tags_14 },
    { 0x6cdae4c6, SUC_lexicon_tags_37 },
    { 0x8a1ae4c7, SUC_lexicon_tags_11 },
    { 0x5ebae4c7, SUC_lexicon_tags_37 },
    { 0x819ae4c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x21dae4cc, SUC_lexicon_tags_1 },
    { 0xf57ae4cc, SUC_lexicon_tags_1 },
    { 0x105ae4cc, SUC_lexicon_tags_33 },
    { 0x48dae4cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd43ae4d1, SUC_lexicon_tags_12 },
    { 0x523ae4d2, SUC_lexicon_tags_3 },
    { 0x1c9ae4d3, SUC_lexicon_tags_34 },
    { 0xaedae4d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1ae4db, SUC_lexicon_tags_95 },
    { 0x4b3ae4db, SUC_lexicon_tags_39 },
    { 0xfb1ae4dd, SUC_lexicon_tags_11 },
    { 0x977ae4dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81ae4e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1ae4ea, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919ae4ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x273ae4f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc3ae4f2, SUC_lexicon_tags_2 },
    { 0x28dae4f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fae4f8, SUC_lexicon_tags_87 },
    { 0xdf7ae4f8, SUC_lexicon_tags_63 },
    { 0xb3fae4f8, SUC_lexicon_tags_47 },
    { 0x2c7ae4fb, SUC_lexicon_tags_26 },
    { 0xe8dae4fb, SUC_lexicon_tags_32 },
    { 0xe7ae4fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x419ae4ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23ae503, SUC_lexicon_tags_69 },
    { 0xa5ae504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf8bae506, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bae50f, SUC_lexicon_tags_11 },
    { 0x717ae510, SUC_lexicon_tags_31 },
    { 0xa43ae510, SUC_lexicon_tags_31 },
    { 0x1dbae512, SUC_lexicon_tags_17 },
    { 0xee7ae511, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39ae519, SUC_lexicon_tags_28 },
    { 0x815ae51a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3ae51f, SUC_lexicon_tags_68 },
    { 0x333ae520, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5a3ae522, SUC_lexicon_tags_10 },
    { 0x44bae523, SUC_lexicon_tags_11 },
    { 0x84fae524, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933ae530, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44bae532, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc41ae534, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9dae536, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x173ae539, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fae53e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1ae544, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae7ae546, SUC_lexicon_tags_15 },
    { 0x2cdae547, SUC_lexicon_tags_7 },
    { 0x141ae548, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449ae54c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8fae54e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x8bae550, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bae555, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69ae55b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dae561, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bae565, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edae56a, SUC_lexicon_tags_65 },
    { 0x7e9ae56b, SUC_lexicon_tags_1 },
    { 0xf3bae56b, SUC_lexicon_tags_2 },
    { 0x675ae56d, SUC_lexicon_tags_37 },
    { 0xcbdae56e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefae574, SUC_lexicon_tags_7 },
    { 0xe73ae575, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ae578, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xabae57a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1dae57c, SUC_lexicon_tags_7 },
    { 0xa8fae57c, SUC_lexicon_tags_124 },
    { 0xd2fae57c, SUC_lexicon_tags_144 },
    { 0x51bae57f, SUC_lexicon_tags_7 },
    { 0xc3bae580, SUC_lexicon_tags_13 },
    { 0x7ebae580, SUC_lexicon_tags_170 },
    { 0x2f5ae582, SUC_lexicon_tags_7 },
    { 0xadbae583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecbae585, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bae58c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595ae591, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbae594, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa37ae596, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x99ae598, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x153ae59b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfae59f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ae5a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1ae5a8, SUC_lexicon_tags_17 },
    { 0x3b7ae5a9, SUC_lexicon_tags_56 },
    { 0xc5ae5aa, SUC_lexicon_tags_359 },
    { 0x1c5ae5aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a7ae5ad, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25ae5b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x255ae5b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fae5ba, SUC_lexicon_tags_29 },
    { 0xb55ae5bb, SUC_lexicon_tags_65 },
    { 0xae5ae5bb, SUC_lexicon_tags_23 },
    { 0x6e5ae5bd, SUC_lexicon_tags_45 },
    { 0x291ae5be, SUC_lexicon_tags_17 },
    { 0xce7ae5be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fae5c4, SUC_lexicon_tags_56 },
    { 0x20bae5c5, SUC_lexicon_tags_55 },
    { 0xd81ae5c4, SUC_lexicon_tags_37 },
    { 0x29bae5c4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcb7ae5c9, SUC_lexicon_tags_30 },
    { 0xd51ae5ca, SUC_lexicon_tags_37 },
    { 0x147ae5cb, SUC_lexicon_tags_37 },
    { 0x5d3ae5c9, SUC_lexicon_tags_21 },
    { 0x61bae5cd, SUC_lexicon_tags_37 },
    { 0x4a1ae5c9, SUC_lexicon_tags_25 },
    { 0x105ae5cf, SUC_lexicon_tags_65 },
    { 0x6a3ae5d0, SUC_lexicon_tags_7 },
    { 0xcbae5d0, SUC_lexicon_tags_39 },
    { 0x807ae5ce, SUC_lexicon_tags_37 },
    { 0xb7fae5cd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xedae5d5, SUC_lexicon_tags_2 },
    { 0x13ae5d5, SUC_lexicon_tags_37 },
    { 0x6cdae5d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x599ae5da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x16dae5dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bae5e5, SUC_lexicon_tags_15 },
    { 0x19ae5e6, SUC_lexicon_tags_82 },
    { 0x2dae5e7, SUC_lexicon_tags_61 },
    { 0x7ae5e8, SUC_lexicon_tags_14 },
    { 0x409ae5e5, SUC_lexicon_tags_37 },
    { 0xb4fae5ea, SUC_lexicon_tags_47 },
    { 0xa6fae5ea, SUC_lexicon_tags_46 },
    { 0xd21ae5ec, SUC_lexicon_tags_43 },
    { 0x94dae5ed, SUC_lexicon_tags_31 },
    { 0x789ae5ee, SUC_lexicon_tags_37 },
    { 0xeb3ae5ea, SUC_lexicon_tags_19 },
    { 0xa97ae5eb, SUC_lexicon_tags_31 },
    { 0x741ae5f1, SUC_lexicon_tags_15 },
    { 0xc3fae5ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x709ae5f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadae5f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdae5ff, SUC_lexicon_tags_3 },
    { 0xc9bae600, SUC_lexicon_tags_61 },
    { 0xc7ae600, SUC_lexicon_tags_9 },
    { 0x81dae5ff, SUC_lexicon_tags_46 },
    { 0x2f5ae603, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dae606, SUC_lexicon_tags_25 },
    { 0x6adae607, SUC_lexicon_tags_3 },
    { 0x2e3ae608, SUC_lexicon_tags_25 },
    { 0x8fdae609, SUC_lexicon_tags_141 },
    { 0xfe3ae609, SUC_lexicon_tags_7 },
    { 0x5c7ae60b, SUC_lexicon_tags_51 },
    { 0xd09ae60b, SUC_lexicon_tags_77 },
    { 0x6c7ae60c, SUC_lexicon_tags_37 },
    { 0xb19ae60e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x507ae610, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa7bae612, SUC_lexicon_tags_15 },
    { 0x35dae613, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511ae618, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x849ae61a, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xaedae61d, SUC_lexicon_tags_1 },
    { 0x9efae61e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4f3ae620, SUC_lexicon_tags_11 },
    { 0x9ebae621, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc07ae623, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x72fae625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957ae629, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9efae62b, SUC_lexicon_tags_77 },
    { 0x165ae62c, SUC_lexicon_tags_39 },
    { 0x79dae62b, SUC_lexicon_tags_25 },
    { 0xff5ae62e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8bae630, SUC_lexicon_tags_33 },
    { 0x40bae631, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccbae633, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbae637, SUC_lexicon_tags_17 },
    { 0xe5ae638, SUC_lexicon_tags_15 },
    { 0xbd9ae639, SUC_lexicon_tags_7 },
    { 0xc69ae63a, SUC_lexicon_tags_12 },
    { 0x3d3ae63b, SUC_lexicon_tags_7 },
    { 0xb9ae63c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edae640, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4a1ae642, SUC_lexicon_tags_85 },
    { 0xdbbae642, SUC_lexicon_tags_37 },
    { 0xb1fae643, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adae64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1ae650, SUC_lexicon_tags_0 },
    { 0xabdae651, SUC_lexicon_tags_15 },
    { 0x501ae650, SUC_lexicon_tags_738 },
    { 0x997ae653, SUC_lexicon_tags_38 },
    { 0xf0fae653, SUC_lexicon_tags_216 },
    { 0x473ae655, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ae658, SUC_lexicon_tags_68 },
    { 0xa21ae659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55ae65e, SUC_lexicon_tags_10 },
    { 0xbddae65f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2d3ae661, SUC_lexicon_tags_7 },
    { 0x947ae661, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5ae666, SUC_lexicon_tags_11 },
    { 0x9edae667, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeb7ae669, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x93bae66b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x347ae66d, SUC_lexicon_tags_32 },
    { 0x485ae66d, SUC_lexicon_tags_45 },
    { 0x8f9ae66d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xde5ae671, SUC_lexicon_tags_7 },
    { 0xb77ae672, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x751ae674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdae678, SUC_lexicon_tags_49 },
    { 0x26bae679, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x517ae67c, SUC_lexicon_tags_26 },
    { 0x759ae67c, SUC_lexicon_tags_11 },
    { 0xf07ae67e, SUC_lexicon_tags_11 },
    { 0xe99ae67c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bae682, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x31bae684, SUC_lexicon_tags_2 },
    { 0x21fae685, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55ae68e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x665ae690, SUC_lexicon_tags_1 },
    { 0x851ae691, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1ae695, SUC_lexicon_tags_26 },
    { 0x3e9ae695, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dae699, SUC_lexicon_tags_25 },
    { 0xedbae69a, SUC_lexicon_tags_7 },
    { 0xd3fae699, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd13ae69d, SUC_lexicon_tags_37 },
    { 0x9afae69e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fae6a3, SUC_lexicon_tags_19 },
    { 0xecfae6a4, SUC_lexicon_tags_43 },
    { 0x861ae6a4, SUC_lexicon_tags_37 },
    { 0x3d7ae6a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x679ae6a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25bae6aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafae6ad, SUC_lexicon_tags_92 },
    { 0xe7bae6ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x699ae6b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdae6b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419ae6b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bae6c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01ae6c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd9fae6c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef9ae6c7, SUC_lexicon_tags_25 },
    { 0xcf3ae6c8, SUC_lexicon_tags_4 },
    { 0x75bae6c9, SUC_lexicon_tags_51 },
    { 0xaa5ae6ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbae6cd, SUC_lexicon_tags_21 },
    { 0xca3ae6ce, SUC_lexicon_tags_1 },
    { 0x31ae6cf, SUC_lexicon_tags_11 },
    { 0x239ae6cf, SUC_lexicon_tags_2 },
    { 0x721ae6d1, SUC_lexicon_tags_43 },
    { 0xfbdae6cd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa43ae6d4, SUC_lexicon_tags_37 },
    { 0xae1ae6d5, SUC_lexicon_tags_1 },
    { 0x19dae6d6, SUC_lexicon_tags_58 },
    { 0x275ae6d7, SUC_lexicon_tags_19 },
    { 0x71fae6d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff3ae6da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07ae6df, SUC_lexicon_tags_33 },
    { 0x4e5ae6df, SUC_lexicon_tags_1 },
    { 0x16dae6df, SUC_lexicon_tags_15 },
    { 0x3d3ae6df, SUC_lexicon_tags_7 },
    { 0xd3bae6e0, SUC_lexicon_tags_11 },
    { 0x1b3ae6e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb19ae6e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefae6ea, SUC_lexicon_tags_26 },
    { 0x895ae6ea, SUC_lexicon_tags_43 },
    { 0x403ae6ec, SUC_lexicon_tags_14 },
    { 0x763ae6ed, SUC_lexicon_tags_17 },
    { 0x72dae6ea, SUC_lexicon_tags_27 },
    { 0x761ae6ef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9ae6f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfae6f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157ae70c, SUC_lexicon_tags_37 },
    { 0xd5bae70d, SUC_lexicon_tags_6 },
    { 0x837ae70e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaffae710, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xd69ae712, SUC_lexicon_tags_667 },
    { 0xcefae713, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9bfae715, SUC_lexicon_tags_17 },
    { 0xd09ae716, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ae71e, SUC_lexicon_tags_16 },
    { 0xe6fae71e, SUC_lexicon_tags_16 },
    { 0x9b5ae71f, SUC_lexicon_tags_27 },
    { 0xa8dae721, SUC_lexicon_tags_35 },
    { 0x327ae720, SUC_lexicon_tags_25 },
    { 0xe79ae723, SUC_lexicon_tags_25 },
    { 0x59fae723, SUC_lexicon_tags_15 },
    { 0xdefae725, SUC_lexicon_tags_531 },
    { 0x765ae726, SUC_lexicon_tags_49 },
    { 0xb5ae727, SUC_lexicon_tags_25 },
    { 0xbddae723, SUC_lexicon_tags_15 },
    { 0xa8fae729, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x903ae72b, SUC_lexicon_tags_49 },
    { 0xac3ae72c, SUC_lexicon_tags_2 },
    { 0x345ae72d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bae735, SUC_lexicon_tags_11 },
    { 0x261ae736, SUC_lexicon_tags_14 },
    { 0xf6dae735, SUC_lexicon_tags_11 },
    { 0xd4dae737, SUC_lexicon_tags_63 },
    { 0x2f7ae739, SUC_lexicon_tags_12 },
    { 0x221ae739, SUC_lexicon_tags_2 },
    { 0x85dae73b, SUC_lexicon_tags_1 },
    { 0xac7ae73c, SUC_lexicon_tags_12 },
    { 0x459ae73d, SUC_lexicon_tags_23 },
    { 0x633ae73e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x363ae740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269ae744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35fae746, SUC_lexicon_tags_10 },
    { 0x2a5ae746, SUC_lexicon_tags_10 },
    { 0x82fae748, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dae74b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafae754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63ae758, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193ae75d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc7fae75f, SUC_lexicon_tags_7 },
    { 0xfa9ae760, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7ae764, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x795ae766, SUC_lexicon_tags_8 },
    { 0x889ae767, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xadfae769, SUC_lexicon_tags_79 },
    { 0x99ae769, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3ae76e, SUC_lexicon_tags_0 },
    { 0xadae76e, SUC_lexicon_tags_46 },
    { 0x4bae770, SUC_lexicon_tags_11 },
    { 0xabdae771, SUC_lexicon_tags_7 },
    { 0x41fae772, SUC_lexicon_tags_15 },
    { 0x207ae76e, SUC_lexicon_tags_25 },
    { 0xda9ae771, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1ae778, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5ae77c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x229ae77e, SUC_lexicon_tags_47 },
    { 0xaedae77f, SUC_lexicon_tags_12 },
    { 0x5f3ae780, SUC_lexicon_tags_17 },
    { 0xe47ae781, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c3ae783, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x903ae785, SUC_lexicon_tags_66 },
    { 0x1c1ae785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fae789, SUC_lexicon_tags_37 },
    { 0xd85ae78a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ae793, SUC_lexicon_tags_28 },
    { 0x419ae793, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849ae79a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa25ae79c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5ae79f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5ae7a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23ae7ab, SUC_lexicon_tags_15 },
    { 0x2ebae7ac, SUC_lexicon_tags_1 },
    { 0xcb7ae7ac, SUC_lexicon_tags_27 },
    { 0xb01ae7ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25fae7b0, SUC_lexicon_tags_37 },
    { 0x2c5ae7b1, SUC_lexicon_tags_12 },
    { 0x277ae7b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33fae7b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2efae7b7, SUC_lexicon_tags_46 },
    { 0x4bfae7b8, SUC_lexicon_tags_25 },
    { 0x5f5ae7b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9ae7bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61ae7bf, SUC_lexicon_tags_17 },
    { 0x9bae7bf, SUC_lexicon_tags_7 },
    { 0x4a7ae7c1, SUC_lexicon_tags_87 },
    { 0x437ae7c1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347ae7c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x21bae7c9, SUC_lexicon_tags_27 },
    { 0x66bae7ca, SUC_lexicon_tags_14 },
    { 0x1cdae7cb, SUC_lexicon_tags_84 },
    { 0x897ae7cb, SUC_lexicon_tags_17 },
    { 0x671ae7c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1ae7d3, SUC_lexicon_tags_6 },
    { 0x185ae7d4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x115ae7d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dae7d9, SUC_lexicon_tags_14 },
    { 0x93ae7d9, SUC_lexicon_tags_25 },
    { 0xc1bae7db, SUC_lexicon_tags_23 },
    { 0x7b3ae7dc, SUC_lexicon_tags_23 },
    { 0xdcfae7db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fae7e4, SUC_lexicon_tags_2 },
    { 0x1bfae7e5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bae7e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xda3ae7eb, SUC_lexicon_tags_15 },
    { 0x94bae7ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65ae7f2, SUC_lexicon_tags_19 },
    { 0xbc1ae7f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x341ae7f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fae7f9, SUC_lexicon_tags_17 },
    { 0x845ae7f9, SUC_lexicon_tags_2 },
    { 0xa11ae7fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8fae7fd, SUC_lexicon_tags_15 },
    { 0x37ae7fe, SUC_lexicon_tags_30 },
    { 0x8dfae7ff, SUC_lexicon_tags_49 },
    { 0x839ae7ff, SUC_lexicon_tags_60 },
    { 0x77fae801, SUC_lexicon_tags_95 },
    { 0xc3dae802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709ae807, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45ae80c, SUC_lexicon_tags_7 },
    { 0xce5ae80c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431ae812, SUC_lexicon_tags_25 },
    { 0xd97ae813, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187ae817, SUC_lexicon_tags_1 },
    { 0xb7bae817, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x863ae81a, SUC_lexicon_tags_25 },
    { 0xac7ae81b, SUC_lexicon_tags_15 },
    { 0xb9fae81c, SUC_lexicon_tags_14 },
    { 0x959ae81c, SUC_lexicon_tags_37 },
    { 0x93bae81e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73ae820, SUC_lexicon_tags_43 },
    { 0xf9bae821, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135ae827, SUC_lexicon_tags_37 },
    { 0x545ae827, SUC_lexicon_tags_18 },
    { 0x7c5ae829, SUC_lexicon_tags_21 },
    { 0x2afae82a, SUC_lexicon_tags_30 },
    { 0x687ae829, SUC_lexicon_tags_19 },
    { 0x60fae82c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5ae82f, SUC_lexicon_tags_25 },
    { 0xb0bae830, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9ae833, SUC_lexicon_tags_91 },
    { 0x56fae834, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fae837, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc2dae839, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bae83c, SUC_lexicon_tags_15 },
    { 0x6f1ae83d, SUC_lexicon_tags_0 },
    { 0x803ae83e, SUC_lexicon_tags_1 },
    { 0x9f5ae83f, SUC_lexicon_tags_11 },
    { 0x987ae840, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x607ae842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x419ae844, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xb5ae846, SUC_lexicon_tags_23 },
    { 0x937ae847, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57ae84e, SUC_lexicon_tags_25 },
    { 0xd11ae84e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73ae852, SUC_lexicon_tags_12 },
    { 0xbd7ae853, SUC_lexicon_tags_40 },
    { 0x9fae854, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47ae858, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a3ae85a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbae85d, SUC_lexicon_tags_12 },
    { 0xd0fae85e, SUC_lexicon_tags_37 },
    { 0x761ae85f, SUC_lexicon_tags_15 },
    { 0xd83ae860, SUC_lexicon_tags_13 },
    { 0x1d7ae860, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5dae863, SUC_lexicon_tags_25 },
    { 0x177ae864, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25ae86a, SUC_lexicon_tags_21 },
    { 0xc39ae86b, SUC_lexicon_tags_15 },
    { 0xfd1ae86b, SUC_lexicon_tags_49 },
    { 0x4d5ae86b, SUC_lexicon_tags_37 },
    { 0x19bae86c, SUC_lexicon_tags_6 },
    { 0x625ae86f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7ae873, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dae876, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x34dae879, SUC_lexicon_tags_1 },
    { 0xc07ae879, SUC_lexicon_tags_25 },
    { 0x3b3ae87a, SUC_lexicon_tags_10 },
    { 0x38fae87c, SUC_lexicon_tags_9 },
    { 0x615ae87d, SUC_lexicon_tags_43 },
    { 0x87ae879, SUC_lexicon_tags_8 },
    { 0x215ae87f, SUC_lexicon_tags_87 },
    { 0x6c9ae87b, SUC_lexicon_tags_27 },
    { 0xba5ae87f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5ae888, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x28fae88a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b7ae88c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3fae88e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dae891, SUC_lexicon_tags_29 },
    { 0xd0fae892, SUC_lexicon_tags_14 },
    { 0xf93ae893, SUC_lexicon_tags_4 },
    { 0x17ae894, SUC_lexicon_tags_87 },
    { 0x2a1ae891, SUC_lexicon_tags_25 },
    { 0x89ae894, SUC_lexicon_tags_355 },
    { 0x6c1ae895, SUC_lexicon_tags_43 },
    { 0xf73ae898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9ae89b, SUC_lexicon_tags_0 },
    { 0x337ae89c, SUC_lexicon_tags_10 },
    { 0xf31ae89d, SUC_lexicon_tags_65 },
    { 0x26bae89b, SUC_lexicon_tags_129 },
    { 0x18dae89e, SUC_lexicon_tags_70 },
    { 0xc51ae89e, SUC_lexicon_tags_15 },
    { 0x6edae8a0, SUC_lexicon_tags_244 },
    { 0x547ae8a2, SUC_lexicon_tags_87 },
    { 0xe4bae8a2, SUC_lexicon_tags_26 },
    { 0x34bae8a4, SUC_lexicon_tags_17 },
    { 0x913ae8a3, SUC_lexicon_tags_28 },
    { 0xdcbae8a6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dae8aa, SUC_lexicon_tags_39 },
    { 0x6c7ae8ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x763ae8ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x79fae8b1, SUC_lexicon_tags_33 },
    { 0xa69ae8b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x145ae8b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc41ae8b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa03ae8b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabae8c0, SUC_lexicon_tags_11 },
    { 0xf47ae8c0, SUC_lexicon_tags_55 },
    { 0x5b5ae8c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9ae8c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x959ae8c7, SUC_lexicon_tags_29 },
    { 0x703ae8c8, SUC_lexicon_tags_29 },
    { 0xca1ae8c9, SUC_lexicon_tags_65 },
    { 0x367ae8ca, SUC_lexicon_tags_25 },
    { 0xa29ae8ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fae8d0, SUC_lexicon_tags_11 },
    { 0x989ae8d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x525ae8d3, SUC_lexicon_tags_29 },
    { 0xdb1ae8d4, SUC_lexicon_tags_12 },
    { 0x60bae8d5, SUC_lexicon_tags_17 },
    { 0xd29ae8d5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dae8da, SUC_lexicon_tags_27 },
    { 0xebae8da, SUC_lexicon_tags_60 },
    { 0xa65ae8da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf23ae8de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7ae8e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241ae8ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9ae8ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdae8f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbae8f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03ae8fc, SUC_lexicon_tags_11 },
    { 0x375ae8fd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7ae900, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7ae902, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfae908, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4c1ae90a, SUC_lexicon_tags_16 },
    { 0xeb3ae90a, SUC_lexicon_tags_1 },
    { 0x56fae90c, SUC_lexicon_tags_47 },
    { 0xc2fae90a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997ae913, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x13fae915, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bae91d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcc9ae91f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49ae923, SUC_lexicon_tags_17 },
    { 0xd4dae923, SUC_lexicon_tags_60 },
    { 0x78dae924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeabae927, SUC_lexicon_tags_37 },
    { 0x1e3ae928, SUC_lexicon_tags_1 },
    { 0x91dae927, SUC_lexicon_tags_4 },
    { 0x345ae92a, SUC_lexicon_tags_23 },
    { 0xe91ae92a, SUC_lexicon_tags_28 },
    { 0xc63ae92c, SUC_lexicon_tags_15 },
    { 0xe1fae92d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bae937, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdae93b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bae93f, SUC_lexicon_tags_2 },
    { 0xc3bae940, SUC_lexicon_tags_11 },
    { 0x8ddae941, SUC_lexicon_tags_17 },
    { 0x86fae941, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97ae945, SUC_lexicon_tags_25 },
    { 0xd79ae946, SUC_lexicon_tags_23 },
    { 0x2a3ae947, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9ae94e, SUC_lexicon_tags_15 },
    { 0xffbae94e, SUC_lexicon_tags_12 },
    { 0x39fae94e, SUC_lexicon_tags_3 },
    { 0xbf9ae951, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xe8fae953, SUC_lexicon_tags_1 },
    { 0xc21ae954, SUC_lexicon_tags_17 },
    { 0x22fae955, SUC_lexicon_tags_3 },
    { 0x6d5ae956, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x81dae958, SUC_lexicon_tags_23 },
    { 0x479ae959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe85ae95b, SUC_lexicon_tags_56 },
    { 0xdf7ae95c, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fae95f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afae964, SUC_lexicon_tags_30 },
    { 0x56dae965, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63ae968, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x673ae96a, SUC_lexicon_tags_7 },
    { 0xa33ae96b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dae96f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19ae979, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ae982, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dae987, SUC_lexicon_tags_25 },
    { 0xfc9ae988, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3ae98f, SUC_lexicon_tags_8 },
    { 0x3f3ae990, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbafae992, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dae9a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fae9ab, SUC_lexicon_tags_4 },
    { 0x8c1ae9ac, SUC_lexicon_tags_16 },
    { 0xfd7ae9ab, SUC_lexicon_tags_0 },
    { 0xdd1ae9ae, SUC_lexicon_tags_15 },
    { 0x941ae9af, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ae9b5, SUC_lexicon_tags_8 },
    { 0x723ae9b6, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfae9bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafdae9c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83ae9c5, SUC_lexicon_tags_37 },
    { 0xa6fae9c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf05ae9c8, SUC_lexicon_tags_26 },
    { 0x9ae9c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x22dae9cc, SUC_lexicon_tags_12 },
    { 0x8e3ae9cd, SUC_lexicon_tags_15 },
    { 0x913ae9ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f5ae9d0, SUC_lexicon_tags_1 },
    { 0x577ae9d1, SUC_lexicon_tags_37 },
    { 0x3d1ae9d2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341ae9d6, SUC_lexicon_tags_25 },
    { 0x47dae9d7, SUC_lexicon_tags_8 },
    { 0x753ae9d8, SUC_lexicon_tags_33 },
    { 0xe5ae9d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373ae9dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x545ae9df, SUC_lexicon_tags_25 },
    { 0xb79ae9df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebae9e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edae9e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3ae9e9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb29ae9eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47ae9ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdae9f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31ae9fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bae9fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66daea02, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43aea09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x635aea0b, SUC_lexicon_tags_12 },
    { 0x42faea0c, SUC_lexicon_tags_37 },
    { 0xa0baea0d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7faea13, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa03aea15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9daea18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801aea1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afaea25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x377aea27, SUC_lexicon_tags_21 },
    { 0x6e9aea28, SUC_lexicon_tags_11 },
    { 0x53baea27, SUC_lexicon_tags_32 },
    { 0xf7aea28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3aea2e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x53baea31, SUC_lexicon_tags_41 },
    { 0x7abaea31, SUC_lexicon_tags_12 },
    { 0xcb9aea33, SUC_lexicon_tags_70 },
    { 0x215aea33, SUC_lexicon_tags_15 },
    { 0x11baea33, SUC_lexicon_tags_25 },
    { 0x8cbaea36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01aea3e, SUC_lexicon_tags_17 },
    { 0xd5baea3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0baea44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5aea4d, SUC_lexicon_tags_17 },
    { 0x41daea4e, SUC_lexicon_tags_11 },
    { 0x5c3aea4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903aea53, SUC_lexicon_tags_37 },
    { 0x655aea54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbaea5a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x7daea5c, SUC_lexicon_tags_17 },
    { 0xccfaea5d, SUC_lexicon_tags_13 },
    { 0xb75aea5e, SUC_lexicon_tags_17 },
    { 0x4f3aea5f, SUC_lexicon_tags_17 },
    { 0x841aea5f, SUC_lexicon_tags_17 },
    { 0x541aea5c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5aea66, SUC_lexicon_tags_14 },
    { 0x163aea67, SUC_lexicon_tags_45 },
    { 0x627aea68, SUC_lexicon_tags_47 },
    { 0xb43aea69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72faea6d, SUC_lexicon_tags_22 },
    { 0x463aea6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577aea74, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x13aea76, SUC_lexicon_tags_57 },
    { 0x75aea76, SUC_lexicon_tags_37 },
    { 0xb05aea77, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479aea7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967aea81, SUC_lexicon_tags_25 },
    { 0x18daea82, SUC_lexicon_tags_30 },
    { 0x40daea82, SUC_lexicon_tags_49 },
    { 0xce1aea82, SUC_lexicon_tags_14 },
    { 0x669aea85, SUC_lexicon_tags_19 },
    { 0x31faea84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51faea8b, SUC_lexicon_tags_25 },
    { 0x3d5aea8b, SUC_lexicon_tags_26 },
    { 0x537aea8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfaea91, SUC_lexicon_tags_15 },
    { 0xbbfaea92, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdaea98, SUC_lexicon_tags_46 },
    { 0xfbaea99, SUC_lexicon_tags_43 },
    { 0x341aea9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x61aea9d, SUC_lexicon_tags_11 },
    { 0x153aea9e, SUC_lexicon_tags_55 },
    { 0xf4daea9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9aeaa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4baeaa4, SUC_lexicon_tags_37 },
    { 0x297aeaa5, SUC_lexicon_tags_66 },
    { 0xe6faeaa6, SUC_lexicon_tags_19 },
    { 0xddaeaa6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdaeaaf, SUC_lexicon_tags_15 },
    { 0x205aeaaf, SUC_lexicon_tags_40 },
    { 0xfb5aeab1, SUC_lexicon_tags_29 },
    { 0x57faeaaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7daeab8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9aeabc, SUC_lexicon_tags_1 },
    { 0xde3aeabd, SUC_lexicon_tags_38 },
    { 0x919aeabc, SUC_lexicon_tags_2 },
    { 0xb73aeabf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24daeac1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35aeac4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1faeac6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbaeaca, SUC_lexicon_tags_9 },
    { 0xfedaeacb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9aeace, SUC_lexicon_tags_11 },
    { 0x4a3aeacf, SUC_lexicon_tags_118 },
    { 0xcd3aeacf, SUC_lexicon_tags_0 },
    { 0xcfaeacf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x777aead3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89aeadb, SUC_lexicon_tags_6 },
    { 0x829aeadb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfaeae0, SUC_lexicon_tags_17 },
    { 0x54daeae1, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7aeaeb, SUC_lexicon_tags_48 },
    { 0xda1aeaec, SUC_lexicon_tags_11 },
    { 0x20baeaec, SUC_lexicon_tags_30 },
    { 0x641aeaee, SUC_lexicon_tags_25 },
    { 0x2abaeaef, SUC_lexicon_tags_37 },
    { 0xf91aeaef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9aeaf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7aeaf8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7aeafe, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adaeb03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9adaeb06, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71aeb0f, SUC_lexicon_tags_8 },
    { 0xb51aeb0f, SUC_lexicon_tags_7 },
    { 0xa9daeb11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7daeb13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291aeb18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1aeb1d, SUC_lexicon_tags_30 },
    { 0xdedaeb1d, SUC_lexicon_tags_1 },
    { 0xd33aeb1f, SUC_lexicon_tags_45 },
    { 0xdbdaeb1e, SUC_lexicon_tags_58 },
    { 0xe4daeb21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x779aeb23, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229aeb28, SUC_lexicon_tags_1 },
    { 0x997aeb29, SUC_lexicon_tags_87 },
    { 0x89aeb2a, SUC_lexicon_tags_27 },
    { 0xae3aeb2b, SUC_lexicon_tags_13 },
    { 0x2bdaeb2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51aeb35, SUC_lexicon_tags_15 },
    { 0xe5daeb35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69aeb3a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3aeb3f, SUC_lexicon_tags_47 },
    { 0xab1aeb40, SUC_lexicon_tags_1 },
    { 0x54daeb41, SUC_lexicon_tags_12 },
    { 0xc15aeb3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3aeb46, SUC_lexicon_tags_49 },
    { 0x6ffaeb47, SUC_lexicon_tags_1 },
    { 0xe1daeb48, SUC_lexicon_tags_55 },
    { 0xd8daeb49, SUC_lexicon_tags_1 },
    { 0x6b9aeb49, SUC_lexicon_tags_12 },
    { 0xf09aeb4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49aeb4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbcbaeb50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abaeb54, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb11aeb56, SUC_lexicon_tags_0 },
    { 0x3ffaeb56, SUC_lexicon_tags_38 },
    { 0x705aeb56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96daeb5c, SUC_lexicon_tags_60 },
    { 0x6daeb5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x691aeb5f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x381aeb61, SUC_lexicon_tags_10 },
    { 0x2a1aeb62, SUC_lexicon_tags_23 },
    { 0xfdbaeb62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbaeb67, SUC_lexicon_tags_1 },
    { 0xa6baeb67, SUC_lexicon_tags_105 },
    { 0x743aeb69, SUC_lexicon_tags_0 },
    { 0x623aeb6a, SUC_lexicon_tags_26 },
    { 0x1a1aeb6b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1aeb6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3aeb72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05aeb76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x803aeb79, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41aeb7d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1aeb83, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5aeb87, SUC_lexicon_tags_15 },
    { 0xe55aeb88, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4e1aeb8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2daeb8d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x45baeb90, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa5aeb92, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9aeb95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347aeb9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea9aeb9d, SUC_lexicon_tags_16 },
    { 0xea1aeb9e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd17aeba0, SUC_lexicon_tags_49 },
    { 0x84baeba1, SUC_lexicon_tags_0 },
    { 0xbadaeba2, SUC_lexicon_tags_29 },
    { 0xc9baeba3, SUC_lexicon_tags_65 },
    { 0xf31aeba0, SUC_lexicon_tags_11 },
    { 0xed9aeba0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7faeba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbfaebaa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3baebad, SUC_lexicon_tags_65 },
    { 0xa05aebae, SUC_lexicon_tags_17 },
    { 0x117aebaf, SUC_lexicon_tags_2 },
    { 0x24baebb0, SUC_lexicon_tags_19 },
    { 0xb69aebb1, SUC_lexicon_tags_68 },
    { 0x383aebb2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7aebb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x735aebbb, SUC_lexicon_tags_40 },
    { 0x1d9aebbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x61baebbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13aebc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43daebc8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9adaebca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3aebd1, SUC_lexicon_tags_39 },
    { 0x8b5aebd2, SUC_lexicon_tags_29 },
    { 0x8cdaebd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7aebd7, SUC_lexicon_tags_26 },
    { 0x8a5aebd8, SUC_lexicon_tags_7 },
    { 0x581aebd9, SUC_lexicon_tags_25 },
    { 0x46daebda, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf2baebdc, SUC_lexicon_tags_37 },
    { 0x67faebdd, SUC_lexicon_tags_2 },
    { 0x4cdaebde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9aebe2, SUC_lexicon_tags_25 },
    { 0x135aebe2, SUC_lexicon_tags_23 },
    { 0xf17aebe2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27daebeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7faebed, SUC_lexicon_tags_0 },
    { 0x4e9aebee, SUC_lexicon_tags_48 },
    { 0x701aebef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7aebfc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x21daebfe, SUC_lexicon_tags_104 },
    { 0x56daebfe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x305aec01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1aec04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7aec0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3aec0e, SUC_lexicon_tags_25 },
    { 0xa63aec0f, SUC_lexicon_tags_10 },
    { 0x75aec0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553aec15, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x215aec18, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xb85aec1a, SUC_lexicon_tags_874 },
    { 0, NULL },
    { 0xe1aec1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x743aec1e, SUC_lexicon_tags_1 },
    { 0x595aec1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x217aec22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x157aec24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd31aec26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efaec29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19aec2c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5aec2f, SUC_lexicon_tags_8 },
    { 0x5b1aec30, SUC_lexicon_tags_2 },
    { 0xb83aec31, SUC_lexicon_tags_2 },
    { 0x93baec2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97aec35, SUC_lexicon_tags_43 },
    { 0x8dfaec36, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x497aec38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953aec3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99daec45, SUC_lexicon_tags_2 },
    { 0xb5faec45, SUC_lexicon_tags_9 },
    { 0xbbaec47, SUC_lexicon_tags_14 },
    { 0xec5aec48, SUC_lexicon_tags_11 },
    { 0xc97aec48, SUC_lexicon_tags_15 },
    { 0x205aec4a, SUC_lexicon_tags_14 },
    { 0x429aec4a, SUC_lexicon_tags_37 },
    { 0x249aec4c, SUC_lexicon_tags_60 },
    { 0x725aec4b, SUC_lexicon_tags_144 },
    { 0xf1aec4e, SUC_lexicon_tags_37 },
    { 0x715aec4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd47aec51, SUC_lexicon_tags_15 },
    { 0xa77aec52, SUC_lexicon_tags_2 },
    { 0x83faec51, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89aec57, SUC_lexicon_tags_127 },
    { 0xbcbaec58, SUC_lexicon_tags_38 },
    { 0xf65aec59, SUC_lexicon_tags_15 },
    { 0x61aec59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x721aec5c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffaec62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x727aec65, SUC_lexicon_tags_2 },
    { 0xf93aec65, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517aec6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37aec6d, SUC_lexicon_tags_12 },
    { 0x5ddaec6d, SUC_lexicon_tags_9 },
    { 0xe91aec6f, SUC_lexicon_tags_26 },
    { 0xb29aec70, SUC_lexicon_tags_11 },
    { 0xac9aec71, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x235aec74, SUC_lexicon_tags_29 },
    { 0x2a1aec75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3faec77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687aec7f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7aec88, SUC_lexicon_tags_14 },
    { 0x2bdaec89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe7aec8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x721aec8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3aec94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8baec9b, SUC_lexicon_tags_7 },
    { 0xa9faec9c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1a3aec9e, SUC_lexicon_tags_7 },
    { 0x643aec9e, SUC_lexicon_tags_38 },
    { 0xbb5aeca0, SUC_lexicon_tags_14 },
    { 0xb0baeca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8daeca7, SUC_lexicon_tags_24 },
    { 0x53aeca8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23aecac, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803aecb0, SUC_lexicon_tags_1 },
    { 0x55faecb1, SUC_lexicon_tags_15 },
    { 0x6f7aecb2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5aecb6, SUC_lexicon_tags_61 },
    { 0xe0faecb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115aecbe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbaecc1, SUC_lexicon_tags_26 },
    { 0x443aecc1, SUC_lexicon_tags_7 },
    { 0xf1daecc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x419aecc6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34daecc8, SUC_lexicon_tags_28 },
    { 0x3ebaecc9, SUC_lexicon_tags_33 },
    { 0x6c3aecca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9efaeccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7aecd3, SUC_lexicon_tags_0 },
    { 0x261aecd3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x831aecd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1baecdd, SUC_lexicon_tags_88 },
    { 0xb03aecde, SUC_lexicon_tags_92 },
    { 0x941aecdd, SUC_lexicon_tags_2 },
    { 0xd07aecdf, SUC_lexicon_tags_39 },
    { 0xd25aece1, SUC_lexicon_tags_1 },
    { 0xf5daece2, SUC_lexicon_tags_17 },
    { 0xb69aece2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x807aece5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3aece9, SUC_lexicon_tags_15 },
    { 0xe7baecea, SUC_lexicon_tags_47 },
    { 0x8bbaece9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa9aeced, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62daecf1, SUC_lexicon_tags_15 },
    { 0x483aecf2, SUC_lexicon_tags_7 },
    { 0x8f7aecf3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7baecf5, SUC_lexicon_tags_11 },
    { 0x13aecf6, SUC_lexicon_tags_7 },
    { 0xea9aecf6, SUC_lexicon_tags_27 },
    { 0xd6daecf8, SUC_lexicon_tags_65 },
    { 0x345aecf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3daecfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3aed04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x475aed06, SUC_lexicon_tags_17 },
    { 0x33daed06, SUC_lexicon_tags_11 },
    { 0x185aed06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x981aed0a, SUC_lexicon_tags_19 },
    { 0xb4faed0b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5aed11, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27aed15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfaed19, SUC_lexicon_tags_27 },
    { 0x229aed1a, SUC_lexicon_tags_19 },
    { 0xc83aed1b, SUC_lexicon_tags_7 },
    { 0xbefaed1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5aed21, SUC_lexicon_tags_17 },
    { 0x5f5aed21, SUC_lexicon_tags_16 },
    { 0x26daed21, SUC_lexicon_tags_37 },
    { 0xbd9aed24, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcdaed26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1aed29, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebaed2d, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x99aed2f, SUC_lexicon_tags_37 },
    { 0x5fdaed2f, SUC_lexicon_tags_77 },
    { 0xe25aed31, SUC_lexicon_tags_15 },
    { 0x3a1aed32, SUC_lexicon_tags_19 },
    { 0x7bbaed33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e7aed35, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60daed3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0daed3c, SUC_lexicon_tags_37 },
    { 0x163aed3d, SUC_lexicon_tags_25 },
    { 0x4f9aed3e, SUC_lexicon_tags_25 },
    { 0x161aed3f, SUC_lexicon_tags_60 },
    { 0x3a1aed40, SUC_lexicon_tags_12 },
    { 0x275aed41, SUC_lexicon_tags_19 },
    { 0x131aed41, SUC_lexicon_tags_37 },
    { 0x8ddaed43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92baed45, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x58faed47, SUC_lexicon_tags_19 },
    { 0xd89aed47, SUC_lexicon_tags_37 },
    { 0xc2faed49, SUC_lexicon_tags_43 },
    { 0xa5daed49, SUC_lexicon_tags_61 },
    { 0x199aed4b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229aed51, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x785aed54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94faed58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439aed5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39baed65, SUC_lexicon_tags_33 },
    { 0x8b1aed65, SUC_lexicon_tags_37 },
    { 0xdbdaed67, SUC_lexicon_tags_12 },
    { 0x7ebaed67, SUC_lexicon_tags_37 },
    { 0x53daed69, SUC_lexicon_tags_26 },
    { 0xb57aed69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62baed6c, SUC_lexicon_tags_37 },
    { 0xfebaed6c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805aed74, SUC_lexicon_tags_18 },
    { 0x115aed75, SUC_lexicon_tags_15 },
    { 0x7daed76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89aed7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc1aed7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x709aed7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8baed83, SUC_lexicon_tags_29 },
    { 0x4b7aed83, SUC_lexicon_tags_7 },
    { 0x4f9aed85, SUC_lexicon_tags_38 },
    { 0x107aed84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddaed8b, SUC_lexicon_tags_23 },
    { 0x88baed8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5aed98, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x44baed9a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa3aed9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bbaed9e, SUC_lexicon_tags_63 },
    { 0x525aed9f, SUC_lexicon_tags_84 },
    { 0xdbfaeda0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc61aeda2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7aeda5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0baeda8, SUC_lexicon_tags_12 },
    { 0x37baeda8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53daedae, SUC_lexicon_tags_14 },
    { 0x43daedaf, SUC_lexicon_tags_14 },
    { 0x3fdaedaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7faedb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01aedb9, SUC_lexicon_tags_29 },
    { 0xd75aedba, SUC_lexicon_tags_25 },
    { 0x28daedbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf95aedbd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459aedc2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31aedc5, SUC_lexicon_tags_24 },
    { 0x439aedc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x771aedc9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x189aedcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed9aedce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fbaedd0, SUC_lexicon_tags_748 },
    { 0x9fbaedd0, SUC_lexicon_tags_54 },
    { 0x7afaedd2, SUC_lexicon_tags_2 },
    { 0x45aedd3, SUC_lexicon_tags_11 },
    { 0xb6daedd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14baeddb, SUC_lexicon_tags_13 },
    { 0xf19aeddb, SUC_lexicon_tags_9 },
    { 0x593aeddd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x677aeddf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1aede2, SUC_lexicon_tags_39 },
    { 0x26faede3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48faede5, SUC_lexicon_tags_30 },
    { 0xa47aede6, SUC_lexicon_tags_15 },
    { 0xc81aede7, SUC_lexicon_tags_15 },
    { 0x83aede7, SUC_lexicon_tags_68 },
    { 0x723aede6, SUC_lexicon_tags_26 },
    { 0xf15aedea, SUC_lexicon_tags_63 },
    { 0xf73aede5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x901aedee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9aedf1, SUC_lexicon_tags_62 },
    { 0x585aedf2, SUC_lexicon_tags_77 },
    { 0x8ebaedf1, SUC_lexicon_tags_30 },
    { 0xe43aedf4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd7aedf6, SUC_lexicon_tags_15 },
    { 0x9b7aedf6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7b1aedf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x257aedfb, SUC_lexicon_tags_2 },
    { 0x33faedfc, SUC_lexicon_tags_48 },
    { 0x555aedfb, SUC_lexicon_tags_30 },
    { 0x845aedfe, SUC_lexicon_tags_49 },
    { 0x10faedfe, SUC_lexicon_tags_15 },
    { 0x53aee00, SUC_lexicon_tags_37 },
    { 0xa51aedfb, SUC_lexicon_tags_8 },
    { 0x645aee02, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59aee08, SUC_lexicon_tags_12 },
    { 0x825aee08, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x15daee0c, SUC_lexicon_tags_44 },
    { 0x64baee0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb9daee0f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853aee13, SUC_lexicon_tags_105 },
    { 0x8faee14, SUC_lexicon_tags_33 },
    { 0xe9daee13, SUC_lexicon_tags_1 },
    { 0xaedaee16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e5aee18, SUC_lexicon_tags_17 },
    { 0x855aee19, SUC_lexicon_tags_25 },
    { 0x773aee1a, SUC_lexicon_tags_37 },
    { 0xbefaee1b, SUC_lexicon_tags_33 },
    { 0x2f5aee19, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6baee26, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1aee2a, SUC_lexicon_tags_0 },
    { 0xb65aee2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x89baee2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39aee33, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5faee36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9daee3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbaee3e, SUC_lexicon_tags_38 },
    { 0xa95aee3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39aee42, SUC_lexicon_tags_25 },
    { 0xdb7aee43, SUC_lexicon_tags_15 },
    { 0x473aee44, SUC_lexicon_tags_37 },
    { 0x4f5aee44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7aee49, SUC_lexicon_tags_31 },
    { 0x64faee49, SUC_lexicon_tags_124 },
    { 0x6cfaee49, SUC_lexicon_tags_1 },
    { 0xa2faee4c, SUC_lexicon_tags_19 },
    { 0xc59aee4d, SUC_lexicon_tags_33 },
    { 0x505aee4e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5faee51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc25aee53, SUC_lexicon_tags_2 },
    { 0xc0daee53, SUC_lexicon_tags_30 },
    { 0x563aee55, SUC_lexicon_tags_28 },
    { 0x1c9aee55, SUC_lexicon_tags_19 },
    { 0x8c1aee55, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3aee5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5aee5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafaee60, SUC_lexicon_tags_17 },
    { 0xbc5aee61, SUC_lexicon_tags_16 },
    { 0x8daee62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5aee67, SUC_lexicon_tags_60 },
    { 0x941aee68, SUC_lexicon_tags_46 },
    { 0x5bdaee69, SUC_lexicon_tags_49 },
    { 0xc13aee6a, SUC_lexicon_tags_7 },
    { 0xbdbaee69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x623aee6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x885aee70, SUC_lexicon_tags_7 },
    { 0xbd5aee71, SUC_lexicon_tags_2 },
    { 0x8cfaee72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb2daee74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415aee78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17faee7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99aee80, SUC_lexicon_tags_2 },
    { 0xcfdaee81, SUC_lexicon_tags_33 },
    { 0x819aee82, SUC_lexicon_tags_1 },
    { 0x5c3aee81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a5aee85, SUC_lexicon_tags_12 },
    { 0xe9aee86, SUC_lexicon_tags_17 },
    { 0xc9daee87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfaee8a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x193aee8d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91aee93, SUC_lexicon_tags_2 },
    { 0x4cdaee93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42faee96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d7aee98, SUC_lexicon_tags_0 },
    { 0x5bbaee99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc79aee9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53aeea1, SUC_lexicon_tags_45 },
    { 0xaefaeea2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67daeea4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x129aeea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbaeeac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747aeeb2, SUC_lexicon_tags_33 },
    { 0x4daeeb2, SUC_lexicon_tags_37 },
    { 0x973aeeb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57aeeb7, SUC_lexicon_tags_18 },
    { 0xdd9aeeb8, SUC_lexicon_tags_2 },
    { 0x1c1aeeb9, SUC_lexicon_tags_2 },
    { 0x99aeeba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43aeebd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfaeec1, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x263aeec3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x195aeec6, SUC_lexicon_tags_25 },
    { 0x695aeec6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb09aeec9, SUC_lexicon_tags_65 },
    { 0x291aeeca, SUC_lexicon_tags_7 },
    { 0xcf9aeecb, SUC_lexicon_tags_17 },
    { 0x7f7aeecb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x19baeece, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551aeed4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc1aeed6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x339aeed8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xa1daeeda, SUC_lexicon_tags_14 },
    { 0xd0daeedb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90daeee0, SUC_lexicon_tags_25 },
    { 0x641aeee1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x191aeee3, SUC_lexicon_tags_7 },
    { 0x2d7aeee3, SUC_lexicon_tags_46 },
    { 0x973aeee4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x415aeee7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3daeee9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8a3aeeeb, SUC_lexicon_tags_21 },
    { 0xb41aeeec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8edaeeee, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5aeef4, SUC_lexicon_tags_12 },
    { 0xeb3aeef5, SUC_lexicon_tags_12 },
    { 0xb27aeef6, SUC_lexicon_tags_2 },
    { 0xe13aeef7, SUC_lexicon_tags_12 },
    { 0x75daeef6, SUC_lexicon_tags_2 },
    { 0x817aeef5, SUC_lexicon_tags_17 },
    { 0xd99aeef6, SUC_lexicon_tags_48 },
    { 0x24daeefb, SUC_lexicon_tags_37 },
    { 0xc59aeefc, SUC_lexicon_tags_27 },
    { 0xa67aeefd, SUC_lexicon_tags_6 },
    { 0x9abaeefe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7aef02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7aef04, SUC_lexicon_tags_17 },
    { 0xc63aef05, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x907aef07, SUC_lexicon_tags_35 },
    { 0xdbaef08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfaef0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7faef15, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x341aef18, SUC_lexicon_tags_11 },
    { 0x9b9aef19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86daef20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x613aef22, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63aef25, SUC_lexicon_tags_27 },
    { 0x53baef25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2baef29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1aef2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdffaef2f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54daef3a, SUC_lexicon_tags_17 },
    { 0xd8daef3b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x327aef3e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85aef41, SUC_lexicon_tags_15 },
    { 0x2c9aef42, SUC_lexicon_tags_38 },
    { 0xb8faef43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a9aef45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1aef48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4baef4a, SUC_lexicon_tags_47 },
    { 0x645aef4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37aef50, SUC_lexicon_tags_11 },
    { 0x3aef51, SUC_lexicon_tags_38 },
    { 0x795aef51, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x673aef54, SUC_lexicon_tags_8 },
    { 0x271aef54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79aef5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc81aef5e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931aef66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b5aef68, SUC_lexicon_tags_1 },
    { 0x9fdaef69, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441aef6d, SUC_lexicon_tags_60 },
    { 0xc0faef6e, SUC_lexicon_tags_47 },
    { 0xdc5aef6e, SUC_lexicon_tags_17 },
    { 0x67faef6e, SUC_lexicon_tags_25 },
    { 0xf5faef70, SUC_lexicon_tags_12 },
    { 0x78daef72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x429aef74, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7aef78, SUC_lexicon_tags_17 },
    { 0xbc1aef79, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x797aef7b, SUC_lexicon_tags_87 },
    { 0x11baef7c, SUC_lexicon_tags_38 },
    { 0x805aef7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x641aef80, SUC_lexicon_tags_7 },
    { 0x47aef80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01aef86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9aef89, SUC_lexicon_tags_15 },
    { 0x991aef8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe99aef8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83aef96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd77aef98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947aef9c, SUC_lexicon_tags_2 },
    { 0xaadaef9d, SUC_lexicon_tags_15 },
    { 0x651aef9e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87aefa1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x245aefa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7aefa7, SUC_lexicon_tags_17 },
    { 0xff3aefa7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x985aefaa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x477aefac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7aefb0, SUC_lexicon_tags_2 },
    { 0xc25aefb1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x47baefb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x26faefb7, SUC_lexicon_tags_11 },
    { 0x33baefb8, SUC_lexicon_tags_14 },
    { 0x319aefb7, SUC_lexicon_tags_14 },
    { 0x7edaefba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93aefbf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x891aefc1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfcbaefc3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x18baefc6, SUC_lexicon_tags_2 },
    { 0x993aefc7, SUC_lexicon_tags_37 },
    { 0xf71aefc6, SUC_lexicon_tags_37 },
    { 0xb7baefc9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xef9aefcb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7dbaefcd, SUC_lexicon_tags_85 },
    { 0xbd3aefcd, SUC_lexicon_tags_35 },
    { 0x1edaefce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x561aefd1, SUC_lexicon_tags_12 },
    { 0x8cbaefd2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5aefd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a1aefd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71daefdd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5aefe0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x46faefe3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe83aefe5, SUC_lexicon_tags_1 },
    { 0xef9aefe6, SUC_lexicon_tags_11 },
    { 0xf25aefe7, SUC_lexicon_tags_87 },
    { 0xef1aefe8, SUC_lexicon_tags_33 },
    { 0x445aefe7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9aefed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7aeff0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7aeff7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x12baeff9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x43baeffb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805aefff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b1af001, SUC_lexicon_tags_17 },
    { 0xf3baf002, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x923af005, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c9af007, SUC_lexicon_tags_29 },
    { 0x7edaf007, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5cbaf00a, SUC_lexicon_tags_25 },
    { 0xf89af00b, SUC_lexicon_tags_30 },
    { 0xf51af00a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edaf013, SUC_lexicon_tags_10 },
    { 0x631af013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7daf016, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x58faf018, SUC_lexicon_tags_37 },
    { 0x8c3af019, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x143af01b, SUC_lexicon_tags_23 },
    { 0x99faf01c, SUC_lexicon_tags_47 },
    { 0xd5daf01d, SUC_lexicon_tags_29 },
    { 0xca5af01e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffdaf020, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7baf022, SUC_lexicon_tags_43 },
    { 0x145af023, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa07af025, SUC_lexicon_tags_35 },
    { 0x711af026, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x749af028, SUC_lexicon_tags_37 },
    { 0xb0daf029, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317af034, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5baf036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265af03b, SUC_lexicon_tags_25 },
    { 0x2fdaf03c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91baf041, SUC_lexicon_tags_75 },
    { 0xa9faf041, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305af04b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83faf053, SUC_lexicon_tags_1 },
    { 0x4f3af054, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6baf056, SUC_lexicon_tags_7 },
    { 0xbdfaf057, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3af05a, SUC_lexicon_tags_201 },
    { 0x395af05a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfaf05e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3baf062, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3af066, SUC_lexicon_tags_62 },
    { 0x4e7af066, SUC_lexicon_tags_11 },
    { 0x2daf068, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x24faf06b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x581af06d, SUC_lexicon_tags_12 },
    { 0x627af06e, SUC_lexicon_tags_29 },
    { 0x8a7af06f, SUC_lexicon_tags_25 },
    { 0xc25af070, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefaf076, SUC_lexicon_tags_23 },
    { 0x5bfaf077, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9baf07a, SUC_lexicon_tags_16 },
    { 0x17faf07a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6daf07f, SUC_lexicon_tags_11 },
    { 0xc4faf07f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ffaf082, SUC_lexicon_tags_43 },
    { 0x527af083, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96baf088, SUC_lexicon_tags_25 },
    { 0x465af089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec5af08b, SUC_lexicon_tags_17 },
    { 0x7cfaf08c, SUC_lexicon_tags_44 },
    { 0xdc9af08c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5af091, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5af098, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfdaf09a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83baf09e, SUC_lexicon_tags_17 },
    { 0xb0faf09f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89baf0a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x273af0a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbaf0a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbaf0ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03af0b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449af0bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5af0be, SUC_lexicon_tags_541 },
    { 0x2a1af0bf, SUC_lexicon_tags_15 },
    { 0x571af0c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x259af0c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebaf0c5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15af0c8, SUC_lexicon_tags_63 },
    { 0xda7af0c8, SUC_lexicon_tags_12 },
    { 0xe8baf0c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7af0cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9af0d3, SUC_lexicon_tags_35 },
    { 0x5d3af0d3, SUC_lexicon_tags_14 },
    { 0x6afaf0d4, SUC_lexicon_tags_27 },
    { 0xd93af0d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3af0d9, SUC_lexicon_tags_11 },
    { 0x351af0da, SUC_lexicon_tags_79 },
    { 0x9faf0db, SUC_lexicon_tags_19 },
    { 0x207af0d9, SUC_lexicon_tags_39 },
    { 0x835af0da, SUC_lexicon_tags_2 },
    { 0x84faf0de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85daf0e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbaf0e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1af0e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77faf0ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15af0ed, SUC_lexicon_tags_15 },
    { 0x903af0ed, SUC_lexicon_tags_31 },
    { 0xc97af0ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfaf0f4, SUC_lexicon_tags_1 },
    { 0x9fdaf0f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367af0fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x701af0ff, SUC_lexicon_tags_37 },
    { 0x5dfaf100, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74faf117, SUC_lexicon_tags_27 },
    { 0x365af118, SUC_lexicon_tags_60 },
    { 0x695af118, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x513af11c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeffaf11e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d3af120, SUC_lexicon_tags_38 },
    { 0x185af121, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63faf123, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xa4baf125, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x581af127, SUC_lexicon_tags_4 },
    { 0x95baf128, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1af131, SUC_lexicon_tags_61 },
    { 0xc1baf132, SUC_lexicon_tags_14 },
    { 0xb03af133, SUC_lexicon_tags_25 },
    { 0x99daf134, SUC_lexicon_tags_27 },
    { 0x87faf134, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82baf138, SUC_lexicon_tags_17 },
    { 0xfe3af138, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa27af13b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x32daf13d, SUC_lexicon_tags_31 },
    { 0x761af13e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1af142, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89af145, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9faf148, SUC_lexicon_tags_14 },
    { 0xc49af149, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36baf154, SUC_lexicon_tags_15 },
    { 0xc83af154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5af158, SUC_lexicon_tags_0 },
    { 0xd3faf159, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2baf161, SUC_lexicon_tags_47 },
    { 0xe21af162, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35af166, SUC_lexicon_tags_37 },
    { 0x495af167, SUC_lexicon_tags_1 },
    { 0xe17af168, SUC_lexicon_tags_1 },
    { 0xba9af169, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb35af16b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7af16f, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x6efaf171, SUC_lexicon_tags_7 },
    { 0xdb5af171, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce7af174, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7af177, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x117af17a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5af17d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93af181, SUC_lexicon_tags_14 },
    { 0x203af182, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x193af185, SUC_lexicon_tags_45 },
    { 0x4d3af186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdaf18a, SUC_lexicon_tags_37 },
    { 0x733af18b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x775af18e, SUC_lexicon_tags_25 },
    { 0x505af18f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffaf194, SUC_lexicon_tags_25 },
    { 0x27faf194, SUC_lexicon_tags_17 },
    { 0x67af196, SUC_lexicon_tags_2 },
    { 0x193af196, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3daf19e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x415af1a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393af1a9, SUC_lexicon_tags_7 },
    { 0xe8baf1aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabaf1b0, SUC_lexicon_tags_178 },
    { 0xb4baf1b0, SUC_lexicon_tags_7 },
    { 0x64baf1b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0baf1b5, SUC_lexicon_tags_10 },
    { 0xe2daf1b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3af1bb, SUC_lexicon_tags_37 },
    { 0xd03af1bc, SUC_lexicon_tags_17 },
    { 0x3a1af1bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa69af1bf, SUC_lexicon_tags_2 },
    { 0xebfaf1c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb0faf1c2, SUC_lexicon_tags_37 },
    { 0x9d3af1c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513af1c7, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xcbfaf1c9, SUC_lexicon_tags_27 },
    { 0xbe3af1ca, SUC_lexicon_tags_37 },
    { 0xa4faf1cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621af1cf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7af1d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb07af1d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbaf1d7, SUC_lexicon_tags_26 },
    { 0xef7af1d8, SUC_lexicon_tags_37 },
    { 0x743af1d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adaf1dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2bbaf1de, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x969af1e1, SUC_lexicon_tags_0 },
    { 0x85faf1e1, SUC_lexicon_tags_43 },
    { 0xd6faf1e3, SUC_lexicon_tags_11 },
    { 0xa99af1e3, SUC_lexicon_tags_27 },
    { 0xb71af1e5, SUC_lexicon_tags_27 },
    { 0x8f5af1e5, SUC_lexicon_tags_3 },
    { 0x7e5af1e7, SUC_lexicon_tags_3 },
    { 0x53daf1e8, SUC_lexicon_tags_1 },
    { 0x523af1e9, SUC_lexicon_tags_37 },
    { 0xab5af1e9, SUC_lexicon_tags_8 },
    { 0xc83af1eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc47af1ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5daf1f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1af1f4, SUC_lexicon_tags_8 },
    { 0xdedaf1f4, SUC_lexicon_tags_39 },
    { 0x437af1f6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfaf1f9, SUC_lexicon_tags_2 },
    { 0x98baf1fa, SUC_lexicon_tags_43 },
    { 0x3b7af1fb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x453af1fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfaf205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431af209, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7af20e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781af214, SUC_lexicon_tags_15 },
    { 0x389af215, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705af219, SUC_lexicon_tags_7 },
    { 0x505af21a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47af221, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x173af223, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd69af225, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75af228, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931af230, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x159af232, SUC_lexicon_tags_49 },
    { 0xd8baf233, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7af236, SUC_lexicon_tags_47 },
    { 0xe9daf236, SUC_lexicon_tags_12 },
    { 0x8c1af237, SUC_lexicon_tags_29 },
    { 0xefaf239, SUC_lexicon_tags_26 },
    { 0x969af23a, SUC_lexicon_tags_37 },
    { 0x55faf23a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe59af23d, SUC_lexicon_tags_37 },
    { 0x263af23e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x363af240, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x73faf242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94baf244, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94baf248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3af24d, SUC_lexicon_tags_17 },
    { 0x395af24e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfaf251, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1af256, SUC_lexicon_tags_43 },
    { 0x889af257, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffaf25a, SUC_lexicon_tags_11 },
    { 0x959af25b, SUC_lexicon_tags_2 },
    { 0xcc5af25b, SUC_lexicon_tags_16 },
    { 0x16baf25c, SUC_lexicon_tags_8 },
    { 0x1afaf25c, SUC_lexicon_tags_229 },
    { 0x361af25e, SUC_lexicon_tags_31 },
    { 0xfbaf25d, SUC_lexicon_tags_11 },
    { 0x9c1af261, SUC_lexicon_tags_25 },
    { 0x49faf25d, SUC_lexicon_tags_14 },
    { 0x14faf262, SUC_lexicon_tags_82 },
    { 0x33faf25a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5af266, SUC_lexicon_tags_47 },
    { 0x5f9af266, SUC_lexicon_tags_42 },
    { 0xfcfaf268, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5baf26b, SUC_lexicon_tags_40 },
    { 0x4efaf26c, SUC_lexicon_tags_23 },
    { 0x177af26c, SUC_lexicon_tags_1 },
    { 0xf9faf26d, SUC_lexicon_tags_10 },
    { 0x9e5af26c, SUC_lexicon_tags_7 },
    { 0x167af270, SUC_lexicon_tags_49 },
    { 0xe89af271, SUC_lexicon_tags_37 },
    { 0x48faf272, SUC_lexicon_tags_14 },
    { 0xdbaf273, SUC_lexicon_tags_8 },
    { 0xabfaf274, SUC_lexicon_tags_37 },
    { 0x223af273, SUC_lexicon_tags_43 },
    { 0xc7baf26c, SUC_lexicon_tags_29 },
    { 0x335af26e, SUC_lexicon_tags_12 },
    { 0x57faf26b, SUC_lexicon_tags_45 },
    { 0xb67af270, SUC_lexicon_tags_6 },
    { 0xc99af27a, SUC_lexicon_tags_144 },
    { 0x93baf27a, SUC_lexicon_tags_11 },
    { 0x457af27c, SUC_lexicon_tags_7 },
    { 0x923af277, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1af283, SUC_lexicon_tags_17 },
    { 0x56daf284, SUC_lexicon_tags_11 },
    { 0x8fdaf284, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455af289, SUC_lexicon_tags_87 },
    { 0xba9af289, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5af297, SUC_lexicon_tags_17 },
    { 0xe2baf297, SUC_lexicon_tags_1 },
    { 0x787af298, SUC_lexicon_tags_37 },
    { 0x273af29a, SUC_lexicon_tags_38 },
    { 0x71baf29a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72baf29d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x9d5af29f, SUC_lexicon_tags_26 },
    { 0xdefaf29f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7faf2a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30faf2ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9af2b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62faf2b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211af2b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1af2bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbaf2c2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76daf2c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99af2c8, SUC_lexicon_tags_11 },
    { 0x397af2c9, SUC_lexicon_tags_2 },
    { 0xdc5af2c9, SUC_lexicon_tags_1 },
    { 0x107af2cb, SUC_lexicon_tags_1 },
    { 0x25af2c8, SUC_lexicon_tags_7 },
    { 0x3f1af2c9, SUC_lexicon_tags_23 },
    { 0xadaf2ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x289af2d0, SUC_lexicon_tags_114 },
    { 0x6e5af2d0, SUC_lexicon_tags_17 },
    { 0x69af2d1, SUC_lexicon_tags_15 },
    { 0x98baf2d3, SUC_lexicon_tags_45 },
    { 0x7b7af2d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4baf2d7, SUC_lexicon_tags_23 },
    { 0xdc9af2d8, SUC_lexicon_tags_11 },
    { 0x283af2d9, SUC_lexicon_tags_25 },
    { 0x913af2da, SUC_lexicon_tags_15 },
    { 0x477af2d9, SUC_lexicon_tags_12 },
    { 0xdd1af2dc, SUC_lexicon_tags_0 },
    { 0xe01af2dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5af2e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x323af2e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdaf2e4, SUC_lexicon_tags_17 },
    { 0x7bbaf2e4, SUC_lexicon_tags_11 },
    { 0x9cfaf2e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf85af2e8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451af2ef, SUC_lexicon_tags_295 },
    { 0xcbbaf2ef, SUC_lexicon_tags_37 },
    { 0x1f7af2f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5c5af2f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7baf2f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf89af2fb, SUC_lexicon_tags_118 },
    { 0xff1af2fb, SUC_lexicon_tags_47 },
    { 0x7cfaf2fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x451af2ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5af304, SUC_lexicon_tags_27 },
    { 0xcb1af305, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317af309, SUC_lexicon_tags_138 },
    { 0x935af30a, SUC_lexicon_tags_15 },
    { 0x6e5af309, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1baf30f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4adaf312, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29af318, SUC_lexicon_tags_37 },
    { 0x7afaf319, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27faf322, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x789af324, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe35af326, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffaf32a, SUC_lexicon_tags_11 },
    { 0xe27af32b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbcfaf32d, SUC_lexicon_tags_8 },
    { 0x2cfaf32e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x865af330, SUC_lexicon_tags_23 },
    { 0x897af331, SUC_lexicon_tags_25 },
    { 0x125af332, SUC_lexicon_tags_17 },
    { 0x8ffaf333, SUC_lexicon_tags_11 },
    { 0x4f1af334, SUC_lexicon_tags_37 },
    { 0x493af334, SUC_lexicon_tags_2 },
    { 0xd3af336, SUC_lexicon_tags_43 },
    { 0xbcfaf333, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x98daf339, SUC_lexicon_tags_33 },
    { 0xd47af33a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47baf33c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41af349, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81baf34d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbaf351, SUC_lexicon_tags_7 },
    { 0x445af351, SUC_lexicon_tags_49 },
    { 0x445af353, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe4baf355, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x873af358, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xc6daf35a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d1af35c, SUC_lexicon_tags_7 },
    { 0xf5faf35d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18daf367, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6daf36b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x11daf36e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x975af370, SUC_lexicon_tags_28 },
    { 0xba7af371, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f9af373, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27af378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe25af37a, SUC_lexicon_tags_7 },
    { 0xce9af37b, SUC_lexicon_tags_15 },
    { 0x43faf37c, SUC_lexicon_tags_4 },
    { 0xfc1af37d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc0faf37f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38faf389, SUC_lexicon_tags_7 },
    { 0xcd7af38a, SUC_lexicon_tags_28 },
    { 0xf3baf38a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65af38f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x353af391, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbaf396, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323af39a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1baf39e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x957af3a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc03af3a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69af3a5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9af3a8, SUC_lexicon_tags_60 },
    { 0x439af3a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afaf3ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeedaf3b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x199af3b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x645af3b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x543af3b9, SUC_lexicon_tags_25 },
    { 0xb23af3ba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68baf3c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52faf3c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b5af3ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfaf3cd, SUC_lexicon_tags_11 },
    { 0xa1faf3cd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbe5af3d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6baf3d4, SUC_lexicon_tags_11 },
    { 0xefaf3d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe5baf3d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3af3dd, SUC_lexicon_tags_60 },
    { 0x107af3de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7af3e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4b9af3e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd37af3e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801af3ef, SUC_lexicon_tags_25 },
    { 0x69daf3f0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161af3f4, SUC_lexicon_tags_0 },
    { 0x907af3f5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x53daf3f7, SUC_lexicon_tags_15 },
    { 0xb91af3f8, SUC_lexicon_tags_23 },
    { 0x8ffaf3f9, SUC_lexicon_tags_28 },
    { 0x5cbaf3f8, SUC_lexicon_tags_37 },
    { 0xa61af3fb, SUC_lexicon_tags_53 },
    { 0x453af3f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3af400, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf5faf402, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915af406, SUC_lexicon_tags_3 },
    { 0xbdaf407, SUC_lexicon_tags_15 },
    { 0xcd5af408, SUC_lexicon_tags_2 },
    { 0x491af409, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93af40c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7af410, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x645af412, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf71af414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf57af416, SUC_lexicon_tags_37 },
    { 0x159af416, SUC_lexicon_tags_25 },
    { 0xb07af418, SUC_lexicon_tags_11 },
    { 0x4cbaf419, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x79af41b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6dbaf41d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49faf41f, SUC_lexicon_tags_84 },
    { 0x6b7af41f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb83af422, SUC_lexicon_tags_12 },
    { 0xf91af423, SUC_lexicon_tags_7 },
    { 0x2c1af423, SUC_lexicon_tags_26 },
    { 0x6dfaf423, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68faf428, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x265af42a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1af42f, SUC_lexicon_tags_60 },
    { 0x933af42f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x973af432, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x529af434, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13af43b, SUC_lexicon_tags_33 },
    { 0x5e9af43c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0faf43f, SUC_lexicon_tags_6 },
    { 0x8b7af440, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x799af443, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd31af445, SUC_lexicon_tags_17 },
    { 0x8f3af446, SUC_lexicon_tags_2 },
    { 0x895af447, SUC_lexicon_tags_62 },
    { 0x31af447, SUC_lexicon_tags_9 },
    { 0xe4baf448, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29af44d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x169af44f, SUC_lexicon_tags_37 },
    { 0xc9af450, SUC_lexicon_tags_31 },
    { 0xe37af451, SUC_lexicon_tags_37 },
    { 0xddfaf452, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbaf455, SUC_lexicon_tags_25 },
    { 0x29faf456, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1af45a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xefbaf45c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71daf460, SUC_lexicon_tags_1 },
    { 0x1dbaf460, SUC_lexicon_tags_60 },
    { 0xf3baf462, SUC_lexicon_tags_19 },
    { 0x3c9af461, SUC_lexicon_tags_12 },
    { 0xe75af464, SUC_lexicon_tags_23 },
    { 0x665af465, SUC_lexicon_tags_68 },
    { 0x4efaf466, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb05af468, SUC_lexicon_tags_188 },
    { 0xfb5af468, SUC_lexicon_tags_60 },
    { 0xae9af469, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1af46f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35af474, SUC_lexicon_tags_37 },
    { 0x9c9af475, SUC_lexicon_tags_31 },
    { 0x40faf475, SUC_lexicon_tags_2 },
    { 0xe89af475, SUC_lexicon_tags_23 },
    { 0x2fbaf477, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559af47e, SUC_lexicon_tags_7 },
    { 0x2a9af47e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1af483, SUC_lexicon_tags_37 },
    { 0x301af484, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb1af486, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4faf488, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7a5af48a, SUC_lexicon_tags_45 },
    { 0x67daf48b, SUC_lexicon_tags_17 },
    { 0x9c1af48c, SUC_lexicon_tags_25 },
    { 0x489af48c, SUC_lexicon_tags_68 },
    { 0x9e7af48e, SUC_lexicon_tags_60 },
    { 0xc01af48f, SUC_lexicon_tags_85 },
    { 0xfb1af490, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1af494, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xcffaf496, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97af49c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39faf49e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x64daf4a1, SUC_lexicon_tags_25 },
    { 0x689af4a1, SUC_lexicon_tags_26 },
    { 0x13af4a3, SUC_lexicon_tags_1 },
    { 0x9e5af4a4, SUC_lexicon_tags_26 },
    { 0xe83af4a5, SUC_lexicon_tags_12 },
    { 0x72daf4a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27daf4ab, SUC_lexicon_tags_17 },
    { 0x7f1af4ac, SUC_lexicon_tags_38 },
    { 0xc1daf4ac, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x849af4af, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6baf4b3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd43af4b5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643af4b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddaf4bd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881af4c3, SUC_lexicon_tags_1 },
    { 0xd23af4c4, SUC_lexicon_tags_12 },
    { 0xd4baf4c3, SUC_lexicon_tags_12 },
    { 0x5d1af4c3, SUC_lexicon_tags_2 },
    { 0x159af4c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303af4cb, SUC_lexicon_tags_38 },
    { 0xc71af4cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3af4d5, SUC_lexicon_tags_63 },
    { 0xeabaf4d5, SUC_lexicon_tags_14 },
    { 0x705af4d7, SUC_lexicon_tags_15 },
    { 0x70baf4d8, SUC_lexicon_tags_3 },
    { 0x5b3af4d9, SUC_lexicon_tags_17 },
    { 0xaa9af4da, SUC_lexicon_tags_25 },
    { 0x3af4d9, SUC_lexicon_tags_1 },
    { 0x4f5af4d9, SUC_lexicon_tags_82 },
    { 0x29af4dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2bbaf4df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f9af4e1, SUC_lexicon_tags_0 },
    { 0xa2faf4e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x651af4e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431af4e9, SUC_lexicon_tags_33 },
    { 0x1f3af4ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x79faf4ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x341af4ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b7af4f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf4f9, SUC_lexicon_tags_63 },
    { 0x49af4fa, SUC_lexicon_tags_1 },
    { 0x311af4fb, SUC_lexicon_tags_12 },
    { 0xe5daf4f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe17af4fe, SUC_lexicon_tags_38 },
    { 0x9a9af4ff, SUC_lexicon_tags_15 },
    { 0xe51af4ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x353af503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9af509, SUC_lexicon_tags_6 },
    { 0x259af50a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9faf50d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7af510, SUC_lexicon_tags_54 },
    { 0xf13af511, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5faf515, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45af518, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07af51e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x663af520, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877af526, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139af52a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb45af52c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855af532, SUC_lexicon_tags_12 },
    { 0xd75af532, SUC_lexicon_tags_9 },
    { 0x16daf533, SUC_lexicon_tags_7 },
    { 0x797af532, SUC_lexicon_tags_15 },
    { 0x3c7af536, SUC_lexicon_tags_8 },
    { 0x427af537, SUC_lexicon_tags_46 },
    { 0x9b1af533, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3af53b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb3af53d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xee1af53f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x283af542, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1baf548, SUC_lexicon_tags_48 },
    { 0x4c1af549, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x659af54c, SUC_lexicon_tags_17 },
    { 0xba9af54d, SUC_lexicon_tags_40 },
    { 0x309af54d, SUC_lexicon_tags_28 },
    { 0x971af54d, SUC_lexicon_tags_2 },
    { 0xa1faf54f, SUC_lexicon_tags_23 },
    { 0xdffaf54c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb67af553, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x933af555, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xde1af557, SUC_lexicon_tags_25 },
    { 0xf11af558, SUC_lexicon_tags_37 },
    { 0xe35af557, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05af55f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdaf56a, SUC_lexicon_tags_17 },
    { 0x473af56b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x397af56d, SUC_lexicon_tags_11 },
    { 0xa09af56d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x8daf571, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x205af574, SUC_lexicon_tags_17 },
    { 0x2ffaf575, SUC_lexicon_tags_1 },
    { 0x7adaf575, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfaf57b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1af581, SUC_lexicon_tags_1 },
    { 0x5f7af582, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x685af585, SUC_lexicon_tags_5 },
    { 0x6adaf585, SUC_lexicon_tags_0 },
    { 0x815af585, SUC_lexicon_tags_21 },
    { 0xfb3af588, SUC_lexicon_tags_17 },
    { 0x7ddaf589, SUC_lexicon_tags_43 },
    { 0xbefaf589, SUC_lexicon_tags_21 },
    { 0xe49af588, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3af58e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x31af590, SUC_lexicon_tags_12 },
    { 0xbe3af591, SUC_lexicon_tags_27 },
    { 0xf83af592, SUC_lexicon_tags_17 },
    { 0xba9af592, SUC_lexicon_tags_995 },
    { 0x4cbaf594, SUC_lexicon_tags_2 },
    { 0xd8daf590, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x99af598, SUC_lexicon_tags_12 },
    { 0x253af599, SUC_lexicon_tags_17 },
    { 0x73faf59a, SUC_lexicon_tags_45 },
    { 0x4a1af59a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c1af59d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5faf5a0, SUC_lexicon_tags_37 },
    { 0xd41af5a1, SUC_lexicon_tags_40 },
    { 0x6ebaf5a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbaf5a5, SUC_lexicon_tags_4 },
    { 0x6dfaf5a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49baf5a9, SUC_lexicon_tags_33 },
    { 0x249af5aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf5af5ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x49daf5ae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571af5b4, SUC_lexicon_tags_2 },
    { 0xfbbaf5b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363af5b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689af5be, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x929af5c0, SUC_lexicon_tags_21 },
    { 0x959af5c1, SUC_lexicon_tags_26 },
    { 0x3bbaf5c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77af5c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67af5c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd5af5cb, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x81faf5cd, SUC_lexicon_tags_14 },
    { 0xa43af5ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x699af5d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x691af5d4, SUC_lexicon_tags_31 },
    { 0x68faf5d5, SUC_lexicon_tags_37 },
    { 0xbddaf5d6, SUC_lexicon_tags_35 },
    { 0xabfaf5d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295af5dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1af5e0, SUC_lexicon_tags_15 },
    { 0x977af5e0, SUC_lexicon_tags_45 },
    { 0xa69af5e1, SUC_lexicon_tags_1 },
    { 0x97baf5e3, SUC_lexicon_tags_63 },
    { 0x167af5e4, SUC_lexicon_tags_7 },
    { 0xea9af5e5, SUC_lexicon_tags_7 },
    { 0x143af5e2, SUC_lexicon_tags_17 },
    { 0x2faf5e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315af5eb, SUC_lexicon_tags_16 },
    { 0xe91af5eb, SUC_lexicon_tags_47 },
    { 0x6a9af5eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcadaf5ef, SUC_lexicon_tags_14 },
    { 0x713af5ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9af5f4, SUC_lexicon_tags_37 },
    { 0xa45af5f5, SUC_lexicon_tags_12 },
    { 0xc63af5f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53faf5fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x575af5fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaf3af5ff, SUC_lexicon_tags_66 },
    { 0x579af600, SUC_lexicon_tags_2 },
    { 0x5b7af601, SUC_lexicon_tags_37 },
    { 0x3b7af602, SUC_lexicon_tags_43 },
    { 0x1bdaf600, SUC_lexicon_tags_31 },
    { 0xcb7af604, SUC_lexicon_tags_37 },
    { 0x4abaf605, SUC_lexicon_tags_25 },
    { 0xb71af603, SUC_lexicon_tags_26 },
    { 0x601af602, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3af60c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9af610, SUC_lexicon_tags_65 },
    { 0x2cfaf611, SUC_lexicon_tags_45 },
    { 0xdc9af612, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e5af614, SUC_lexicon_tags_46 },
    { 0x6efaf615, SUC_lexicon_tags_26 },
    { 0x193af614, SUC_lexicon_tags_0 },
    { 0x2c7af617, SUC_lexicon_tags_30 },
    { 0xd99af618, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfaf61b, SUC_lexicon_tags_11 },
    { 0x129af61b, SUC_lexicon_tags_26 },
    { 0xeebaf61b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9baf622, SUC_lexicon_tags_68 },
    { 0xacdaf623, SUC_lexicon_tags_13 },
    { 0x1c1af624, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x311af626, SUC_lexicon_tags_84 },
    { 0x79faf626, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xafaf62a, SUC_lexicon_tags_29 },
    { 0x1d7af62b, SUC_lexicon_tags_25 },
    { 0xc99af62c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbf9af62e, SUC_lexicon_tags_74 },
    { 0x89faf62f, SUC_lexicon_tags_12 },
    { 0x733af62f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c9af632, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbc1af634, SUC_lexicon_tags_31 },
    { 0x945af635, SUC_lexicon_tags_60 },
    { 0x503af636, SUC_lexicon_tags_37 },
    { 0x943af637, SUC_lexicon_tags_68 },
    { 0x1cbaf638, SUC_lexicon_tags_32 },
    { 0x9b9af637, SUC_lexicon_tags_12 },
    { 0xf91af63a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753af63e, SUC_lexicon_tags_387 },
    { 0x66baf63f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6f3af641, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adaf646, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa8daf648, SUC_lexicon_tags_15 },
    { 0xdb7af649, SUC_lexicon_tags_39 },
    { 0x3baf64a, SUC_lexicon_tags_12 },
    { 0x607af649, SUC_lexicon_tags_2 },
    { 0xaadaf64c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x47faf64e, SUC_lexicon_tags_11 },
    { 0x21daf64e, SUC_lexicon_tags_12 },
    { 0xf3baf64f, SUC_lexicon_tags_47 },
    { 0x79daf651, SUC_lexicon_tags_25 },
    { 0xf89af650, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25af654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fbaf656, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebaf65a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfb5af65c, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5af662, SUC_lexicon_tags_19 },
    { 0x6b9af662, SUC_lexicon_tags_28 },
    { 0x49af664, SUC_lexicon_tags_15 },
    { 0xb03af664, SUC_lexicon_tags_38 },
    { 0x9d7af666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x885af669, SUC_lexicon_tags_30 },
    { 0xc7baf669, SUC_lexicon_tags_14 },
    { 0xdc1af669, SUC_lexicon_tags_25 },
    { 0xb9daf66c, SUC_lexicon_tags_59 },
    { 0xee3af669, SUC_lexicon_tags_47 },
    { 0x6e3af66e, SUC_lexicon_tags_15 },
    { 0x2ffaf66f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdaf679, SUC_lexicon_tags_1 },
    { 0x5f3af67a, SUC_lexicon_tags_47 },
    { 0x7d5af67b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845af67f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdadaf682, SUC_lexicon_tags_9 },
    { 0x10daf682, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x221af685, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc3af687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89af68a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595af68f, SUC_lexicon_tags_2 },
    { 0x83af690, SUC_lexicon_tags_12 },
    { 0x4dfaf690, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x143af694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9af696, SUC_lexicon_tags_29 },
    { 0xed3af696, SUC_lexicon_tags_46 },
    { 0xb4faf696, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07af69b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25baf6a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x865af6a2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x283af6a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc3af6a6, SUC_lexicon_tags_29 },
    { 0xe33af6a7, SUC_lexicon_tags_25 },
    { 0xf13af6a8, SUC_lexicon_tags_63 },
    { 0x2daf6a9, SUC_lexicon_tags_0 },
    { 0x389af6a9, SUC_lexicon_tags_48 },
    { 0xc7daf6ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8adaf6ad, SUC_lexicon_tags_1 },
    { 0x5cbaf6ae, SUC_lexicon_tags_38 },
    { 0x2a7af6ad, SUC_lexicon_tags_11 },
    { 0xf0baf6b0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x195af6b2, SUC_lexicon_tags_70 },
    { 0xd3baf6b3, SUC_lexicon_tags_15 },
    { 0xe5baf6b4, SUC_lexicon_tags_29 },
    { 0x8d1af6b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x291af6b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3daf6ba, SUC_lexicon_tags_7 },
    { 0x6e1af6ba, SUC_lexicon_tags_68 },
    { 0x1dbaf6ba, SUC_lexicon_tags_15 },
    { 0x43af6ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09af6c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9af6c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39af6cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61af6d0, SUC_lexicon_tags_33 },
    { 0x89daf6d1, SUC_lexicon_tags_29 },
    { 0x765af6d1, SUC_lexicon_tags_33 },
    { 0x55baf6d1, SUC_lexicon_tags_7 },
    { 0xdddaf6d4, SUC_lexicon_tags_47 },
    { 0xd8faf6d5, SUC_lexicon_tags_0 },
    { 0x3c1af6d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993af6da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x877af6dc, SUC_lexicon_tags_37 },
    { 0xa97af6dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91af6e0, SUC_lexicon_tags_11 },
    { 0xb7baf6e1, SUC_lexicon_tags_55 },
    { 0x18baf6e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x989af6e5, SUC_lexicon_tags_0 },
    { 0x8c5af6e5, SUC_lexicon_tags_1 },
    { 0x761af6e5, SUC_lexicon_tags_6 },
    { 0xf09af6e8, SUC_lexicon_tags_4 },
    { 0x6c9af6e6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x43af6eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0faf6ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x23baf6f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x395af6f2, SUC_lexicon_tags_37 },
    { 0xf2faf6f3, SUC_lexicon_tags_65 },
    { 0xd8baf6f4, SUC_lexicon_tags_37 },
    { 0xd15af6f2, SUC_lexicon_tags_37 },
    { 0x7c5af6f6, SUC_lexicon_tags_7 },
    { 0xbfdaf6f5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4fdaf6f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55faf6fc, SUC_lexicon_tags_30 },
    { 0x653af6fd, SUC_lexicon_tags_1 },
    { 0xd33af6fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1af703, SUC_lexicon_tags_25 },
    { 0x92baf704, SUC_lexicon_tags_15 },
    { 0x3f9af705, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebaf709, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x28faf70c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x991af70e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x369af710, SUC_lexicon_tags_12 },
    { 0x64daf711, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6faf713, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94baf71d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63daf71f, SUC_lexicon_tags_15 },
    { 0xcc1af720, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1af723, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x165af725, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x983af728, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x14faf72b, SUC_lexicon_tags_48 },
    { 0x71faf72c, SUC_lexicon_tags_29 },
    { 0x9dfaf72d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf01af72f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93daf731, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8baf733, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51af739, SUC_lexicon_tags_37 },
    { 0xaafaf739, SUC_lexicon_tags_11 },
    { 0x9daf739, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7af744, SUC_lexicon_tags_40 },
    { 0x75af744, SUC_lexicon_tags_7 },
    { 0xcedaf746, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xb5baf748, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x255af74a, SUC_lexicon_tags_12 },
    { 0xff7af74b, SUC_lexicon_tags_26 },
    { 0x501af74c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7af74f, SUC_lexicon_tags_63 },
    { 0x94faf750, SUC_lexicon_tags_15 },
    { 0x2bfaf750, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367af758, SUC_lexicon_tags_17 },
    { 0x257af758, SUC_lexicon_tags_12 },
    { 0x49daf75a, SUC_lexicon_tags_30 },
    { 0x3e9af75b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89af75e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1af761, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xe11af763, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5af769, SUC_lexicon_tags_31 },
    { 0x543af76a, SUC_lexicon_tags_21 },
    { 0x305af76b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9af76f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x481af771, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x245af774, SUC_lexicon_tags_10 },
    { 0xb6faf774, SUC_lexicon_tags_15 },
    { 0x9ddaf776, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbaf77a, SUC_lexicon_tags_21 },
    { 0x67daf77b, SUC_lexicon_tags_49 },
    { 0x11baf77b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37daf77f, SUC_lexicon_tags_11 },
    { 0xe53af780, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x483af782, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x43baf784, SUC_lexicon_tags_25 },
    { 0x12faf785, SUC_lexicon_tags_45 },
    { 0x2e5af785, SUC_lexicon_tags_17 },
    { 0xedbaf784, SUC_lexicon_tags_37 },
    { 0x629af788, SUC_lexicon_tags_40 },
    { 0x22daf786, SUC_lexicon_tags_12 },
    { 0x18daf78a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x58baf78c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853af791, SUC_lexicon_tags_15 },
    { 0x427af792, SUC_lexicon_tags_47 },
    { 0x7cdaf793, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1af798, SUC_lexicon_tags_13 },
    { 0xa13af798, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375af79f, SUC_lexicon_tags_1 },
    { 0xeffaf7a0, SUC_lexicon_tags_1 },
    { 0xcdbaf7a1, SUC_lexicon_tags_25 },
    { 0xd5af7a1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1af7a5, SUC_lexicon_tags_1 },
    { 0x83daf7a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60daf7a8, SUC_lexicon_tags_58 },
    { 0x965af7a9, SUC_lexicon_tags_12 },
    { 0xbc7af7aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5daf7ac, SUC_lexicon_tags_11 },
    { 0x9ebaf7ad, SUC_lexicon_tags_33 },
    { 0xb15af7ae, SUC_lexicon_tags_17 },
    { 0x4f5af7ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2faf7b4, SUC_lexicon_tags_0 },
    { 0x7cdaf7b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x557af7b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea3af7ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58daf7bf, SUC_lexicon_tags_37 },
    { 0x621af7c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0daf7c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x863af7c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa03af7c6, SUC_lexicon_tags_11 },
    { 0xafaf7c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x591af7c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7af7cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1af7d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17af7d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8faf7dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17af7e2, SUC_lexicon_tags_8 },
    { 0x949af7e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ebaf7e5, SUC_lexicon_tags_11 },
    { 0x279af7e5, SUC_lexicon_tags_33 },
    { 0xf03af7e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77af7eb, SUC_lexicon_tags_25 },
    { 0xf07af7eb, SUC_lexicon_tags_10 },
    { 0xb2daf7ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0baf7f0, SUC_lexicon_tags_17 },
    { 0xb53af7f1, SUC_lexicon_tags_165 },
    { 0x415af7f1, SUC_lexicon_tags_17 },
    { 0xc29af7f3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xec7af7f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x543af7f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbaf7fc, SUC_lexicon_tags_34 },
    { 0x9dbaf7fc, SUC_lexicon_tags_11 },
    { 0x15daf7fe, SUC_lexicon_tags_27 },
    { 0x165af7fe, SUC_lexicon_tags_63 },
    { 0x3e7af800, SUC_lexicon_tags_49 },
    { 0xeffaf800, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x30faf804, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc83af806, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73faf80b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337af813, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc59af815, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x539af817, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa71af819, SUC_lexicon_tags_17 },
    { 0xc79af819, SUC_lexicon_tags_49 },
    { 0x1d3af81b, SUC_lexicon_tags_8 },
    { 0x9bdaf81b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf45af81e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873af823, SUC_lexicon_tags_66 },
    { 0x59faf823, SUC_lexicon_tags_11 },
    { 0x6abaf825, SUC_lexicon_tags_17 },
    { 0x9b7af825, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67af82f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x571af832, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9af835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61af839, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22faf83b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7af83e, SUC_lexicon_tags_25 },
    { 0xb1faf83f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3af842, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1af846, SUC_lexicon_tags_4 },
    { 0xec1af847, SUC_lexicon_tags_11 },
    { 0xbebaf848, SUC_lexicon_tags_45 },
    { 0x2efaf849, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc97af84b, SUC_lexicon_tags_60 },
    { 0xbe7af84b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3af851, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x83faf853, SUC_lexicon_tags_15 },
    { 0x5cdaf854, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4faf857, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb59af859, SUC_lexicon_tags_7 },
    { 0x15faf85a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463af863, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13af866, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669af86b, SUC_lexicon_tags_63 },
    { 0xed1af86b, SUC_lexicon_tags_11 },
    { 0xa25af86d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdaf870, SUC_lexicon_tags_14 },
    { 0xf97af871, SUC_lexicon_tags_70 },
    { 0xca1af870, SUC_lexicon_tags_11 },
    { 0x855af870, SUC_lexicon_tags_10 },
    { 0xcdaf874, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52faf876, SUC_lexicon_tags_2 },
    { 0xc29af876, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7af87c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x471af87e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb21af880, SUC_lexicon_tags_26 },
    { 0x523af881, SUC_lexicon_tags_29 },
    { 0x2c7af880, SUC_lexicon_tags_48 },
    { 0x9e1af883, SUC_lexicon_tags_12 },
    { 0xeedaf883, SUC_lexicon_tags_17 },
    { 0xdf1af884, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5af888, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b7af88a, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xdd5af88c, SUC_lexicon_tags_12 },
    { 0xa73af88d, SUC_lexicon_tags_304 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1af892, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5af896, SUC_lexicon_tags_1 },
    { 0xa49af897, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecdaf899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917af89d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x833af8a0, SUC_lexicon_tags_28 },
    { 0xeb9af8a1, SUC_lexicon_tags_49 },
    { 0x85baf8a2, SUC_lexicon_tags_17 },
    { 0x2d1af8a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5af8a9, SUC_lexicon_tags_53 },
    { 0x5a1af8aa, SUC_lexicon_tags_9 },
    { 0xd3af8a9, SUC_lexicon_tags_25 },
    { 0x725af8a9, SUC_lexicon_tags_47 },
    { 0xaa5af8a9, SUC_lexicon_tags_12 },
    { 0x74daf8ae, SUC_lexicon_tags_55 },
    { 0x2e5af8ab, SUC_lexicon_tags_11 },
    { 0x6abaf8ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x427af8b3, SUC_lexicon_tags_2 },
    { 0x431af8b3, SUC_lexicon_tags_1 },
    { 0x279af8b5, SUC_lexicon_tags_12 },
    { 0x877af8b6, SUC_lexicon_tags_65 },
    { 0x947af8b3, SUC_lexicon_tags_23 },
    { 0xa29af8b8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9af8c4, SUC_lexicon_tags_25 },
    { 0x735af8c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495af8ce, SUC_lexicon_tags_5 },
    { 0x559af8cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x897af8d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213af8dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5af8e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7af8e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105af8ec, SUC_lexicon_tags_14 },
    { 0xe6daf8ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60baf8f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31daf8f7, SUC_lexicon_tags_16 },
    { 0xfaf8f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xac5af8fa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf8faf8fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7af8ff, SUC_lexicon_tags_72 },
    { 0x3af900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee9af902, SUC_lexicon_tags_11 },
    { 0xeadaf903, SUC_lexicon_tags_7 },
    { 0x145af902, SUC_lexicon_tags_27 },
    { 0x1fdaf905, SUC_lexicon_tags_49 },
    { 0x5dbaf905, SUC_lexicon_tags_23 },
    { 0xda7af905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x215af90a, SUC_lexicon_tags_12 },
    { 0xb51af90b, SUC_lexicon_tags_46 },
    { 0xecdaf90c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c5af90e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1baf91c, SUC_lexicon_tags_17 },
    { 0xedbaf91d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d7af91f, SUC_lexicon_tags_63 },
    { 0xab3af920, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3af923, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9faf92b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddaf930, SUC_lexicon_tags_7 },
    { 0xe23af931, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abaf937, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667af93f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5af942, SUC_lexicon_tags_6 },
    { 0xd6baf942, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ffaf945, SUC_lexicon_tags_4 },
    { 0xadbaf945, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcddaf948, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115af94d, SUC_lexicon_tags_55 },
    { 0x16baf94e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebaf951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x469af953, SUC_lexicon_tags_27 },
    { 0xdbfaf953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x801af956, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3af959, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x573af95c, SUC_lexicon_tags_14 },
    { 0x539af95d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefaf965, SUC_lexicon_tags_29 },
    { 0xa1daf966, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xde3af968, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3baf96a, SUC_lexicon_tags_30 },
    { 0x215af96a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa9af96d, SUC_lexicon_tags_29 },
    { 0x1c5af96e, SUC_lexicon_tags_11 },
    { 0xe11af96f, SUC_lexicon_tags_48 },
    { 0x463af970, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x311af972, SUC_lexicon_tags_63 },
    { 0x25faf972, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13af976, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x579af979, SUC_lexicon_tags_0 },
    { 0x51baf97a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31faf97d, SUC_lexicon_tags_84 },
    { 0x21daf97e, SUC_lexicon_tags_55 },
    { 0x41daf97e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1af983, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdaf989, SUC_lexicon_tags_8 },
    { 0xb5af989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefaf98c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15af991, SUC_lexicon_tags_37 },
    { 0x837af992, SUC_lexicon_tags_68 },
    { 0x20baf991, SUC_lexicon_tags_23 },
    { 0x799af994, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9af997, SUC_lexicon_tags_38 },
    { 0xe7faf998, SUC_lexicon_tags_1 },
    { 0xac7af998, SUC_lexicon_tags_15 },
    { 0x409af99a, SUC_lexicon_tags_25 },
    { 0xaafaf99a, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0x351af99e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xefaf9a0, SUC_lexicon_tags_7 },
    { 0x677af9a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6baf9a6, SUC_lexicon_tags_1 },
    { 0xe0faf9a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5cbaf9a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99daf9af, SUC_lexicon_tags_7 },
    { 0xed1af9b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86faf9b4, SUC_lexicon_tags_2 },
    { 0xfadaf9b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8faf9b9, SUC_lexicon_tags_25 },
    { 0x243af9b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93baf9be, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe2daf9c0, SUC_lexicon_tags_57 },
    { 0x6b3af9c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3af9c6, SUC_lexicon_tags_30 },
    { 0x5ebaf9c6, SUC_lexicon_tags_17 },
    { 0xb07af9c7, SUC_lexicon_tags_37 },
    { 0xd81af9c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe07af9cb, SUC_lexicon_tags_7 },
    { 0xfb7af9cb, SUC_lexicon_tags_37 },
    { 0xd7af9cd, SUC_lexicon_tags_30 },
    { 0x201af9ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65af9d2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x861af9d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99af9dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7af9e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafaf9ea, SUC_lexicon_tags_37 },
    { 0x2d5af9eb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81af9f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x311af9f3, SUC_lexicon_tags_60 },
    { 0xc11af9f3, SUC_lexicon_tags_17 },
    { 0xda9af9f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82baf9fa, SUC_lexicon_tags_2 },
    { 0xadaf9fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x95faf9fe, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897afa02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe57afa04, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdafa0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a5afa0d, SUC_lexicon_tags_12 },
    { 0x397afa0e, SUC_lexicon_tags_2 },
    { 0xe85afa0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883afa1a, SUC_lexicon_tags_1 },
    { 0xa99afa1b, SUC_lexicon_tags_33 },
    { 0x801afa1c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x217afa1e, SUC_lexicon_tags_7 },
    { 0x4abafa1f, SUC_lexicon_tags_65 },
    { 0x59dafa1f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd83afa22, SUC_lexicon_tags_25 },
    { 0x217afa23, SUC_lexicon_tags_1 },
    { 0x60bafa24, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5afa27, SUC_lexicon_tags_12 },
    { 0xd7dafa27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x333afa2b, SUC_lexicon_tags_25 },
    { 0x72bafa2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfafa31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bafa36, SUC_lexicon_tags_1 },
    { 0x7adafa37, SUC_lexicon_tags_0 },
    { 0x3b1afa38, SUC_lexicon_tags_45 },
    { 0xb81afa39, SUC_lexicon_tags_2 },
    { 0x6b1afa3a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bafa3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3afa40, SUC_lexicon_tags_37 },
    { 0xc75afa41, SUC_lexicon_tags_32 },
    { 0x7bbafa42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x101afa45, SUC_lexicon_tags_14 },
    { 0xc25afa46, SUC_lexicon_tags_15 },
    { 0xf41afa45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557afa4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9afa56, SUC_lexicon_tags_508 },
    { 0, NULL },
    { 0xedafa58, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1afa5e, SUC_lexicon_tags_36 },
    { 0xc65afa5f, SUC_lexicon_tags_106 },
    { 0x12fafa60, SUC_lexicon_tags_15 },
    { 0xacbafa61, SUC_lexicon_tags_26 },
    { 0x493afa5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xed1afa64, SUC_lexicon_tags_14 },
    { 0xc43afa65, SUC_lexicon_tags_12 },
    { 0xae1afa65, SUC_lexicon_tags_65 },
    { 0x487afa65, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xea9afa69, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1afa6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75afa6f, SUC_lexicon_tags_17 },
    { 0x1adafa6f, SUC_lexicon_tags_2 },
    { 0x23fafa6f, SUC_lexicon_tags_11 },
    { 0x845afa6f, SUC_lexicon_tags_87 },
    { 0x111afa71, SUC_lexicon_tags_25 },
    { 0xdb5afa74, SUC_lexicon_tags_37 },
    { 0x891afa72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x295afa78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3afa80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2bbafa82, SUC_lexicon_tags_65 },
    { 0xeefafa82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bafa85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7abafa87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bafa8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987afa8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297afa95, SUC_lexicon_tags_47 },
    { 0x8c1afa96, SUC_lexicon_tags_17 },
    { 0xa0fafa97, SUC_lexicon_tags_1 },
    { 0x479afa98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x39bafa9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x913afa9d, SUC_lexicon_tags_12 },
    { 0xb41afa9e, SUC_lexicon_tags_47 },
    { 0x2ebafa9d, SUC_lexicon_tags_15 },
    { 0x14bafa9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x325afaa3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdafaa7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc69afaa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9afab0, SUC_lexicon_tags_37 },
    { 0xb65afab1, SUC_lexicon_tags_9 },
    { 0x67fafab2, SUC_lexicon_tags_116 },
    { 0x7ddafab3, SUC_lexicon_tags_60 },
    { 0x227afab4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197afab8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1afabd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x97fafabf, SUC_lexicon_tags_19 },
    { 0xb71afac0, SUC_lexicon_tags_15 },
    { 0x291afac1, SUC_lexicon_tags_11 },
    { 0x445afac2, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63dafac7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecfafac9, SUC_lexicon_tags_25 },
    { 0xde7afac9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x203afacc, SUC_lexicon_tags_7 },
    { 0xeefafacd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x48dafad0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77afad5, SUC_lexicon_tags_2 },
    { 0x203afad5, SUC_lexicon_tags_2 },
    { 0x3e7afad7, SUC_lexicon_tags_25 },
    { 0x459afad7, SUC_lexicon_tags_14 },
    { 0x2f1afad8, SUC_lexicon_tags_6 },
    { 0xf37afada, SUC_lexicon_tags_7 },
    { 0x701afad9, SUC_lexicon_tags_25 },
    { 0xf2bafadc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1dafade, SUC_lexicon_tags_25 },
    { 0x897afade, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x575afae1, SUC_lexicon_tags_2 },
    { 0x979afae2, SUC_lexicon_tags_17 },
    { 0xf3afae2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x911afae5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63afaec, SUC_lexicon_tags_37 },
    { 0x925afaec, SUC_lexicon_tags_10 },
    { 0x711afaee, SUC_lexicon_tags_49 },
    { 0x3b5afaee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd95afaf1, SUC_lexicon_tags_16 },
    { 0x401afaf1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa47afaf4, SUC_lexicon_tags_1 },
    { 0x82bafaf5, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01afafb, SUC_lexicon_tags_21 },
    { 0x79dafafc, SUC_lexicon_tags_31 },
    { 0xf7dafafd, SUC_lexicon_tags_66 },
    { 0x12dafafe, SUC_lexicon_tags_65 },
    { 0xd9bafaff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf9afb01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfafb05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939afb09, SUC_lexicon_tags_101 },
    { 0x621afb0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89dafb0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afb13, SUC_lexicon_tags_25 },
    { 0x981afb13, SUC_lexicon_tags_6 },
    { 0xfdbafb15, SUC_lexicon_tags_17 },
    { 0xd49afb16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1afb19, SUC_lexicon_tags_65 },
    { 0xcbdafb1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127afb20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3afb23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39afb27, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc0fafb29, SUC_lexicon_tags_11 },
    { 0x629afb2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05afb2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe49afb30, SUC_lexicon_tags_12 },
    { 0x12dafb31, SUC_lexicon_tags_23 },
    { 0xa27afb31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xecfafb34, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0x937afb36, SUC_lexicon_tags_687 },
    { 0x1e7afb37, SUC_lexicon_tags_49 },
    { 0xab7afb38, SUC_lexicon_tags_39 },
    { 0xe65afb39, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x84fafb3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfffafb3d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x82dafb3f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x82dafb41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x913afb44, SUC_lexicon_tags_38 },
    { 0xc11afb45, SUC_lexicon_tags_12 },
    { 0xa77afb45, SUC_lexicon_tags_31 },
    { 0xd4dafb45, SUC_lexicon_tags_4 },
    { 0x4b9afb48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711afb4d, SUC_lexicon_tags_11 },
    { 0x525afb4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56dafb50, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x16dafb52, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebafb56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715afb5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x10dafb5e, SUC_lexicon_tags_17 },
    { 0x635afb5e, SUC_lexicon_tags_15 },
    { 0x1a1afb60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99afb65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x109afb67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdafb6d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x67dafb70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69bafb72, SUC_lexicon_tags_37 },
    { 0x3d9afb73, SUC_lexicon_tags_4 },
    { 0x615afb73, SUC_lexicon_tags_15 },
    { 0x1c9afb75, SUC_lexicon_tags_74 },
    { 0xeffafb76, SUC_lexicon_tags_23 },
    { 0xfebafb77, SUC_lexicon_tags_30 },
    { 0x1cfafb78, SUC_lexicon_tags_37 },
    { 0xab5afb73, SUC_lexicon_tags_1 },
    { 0x9efafb77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15bafb7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77afb82, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x61dafb84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dafb87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b5afb89, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x877afb8c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fafb94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafafb9f, SUC_lexicon_tags_9 },
    { 0x9edafba0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65afba4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dafba8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa6fafbaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80fafbac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85afbb7, SUC_lexicon_tags_25 },
    { 0xd45afbb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67afbba, SUC_lexicon_tags_43 },
    { 0x3d3afbba, SUC_lexicon_tags_37 },
    { 0xc4dafbbc, SUC_lexicon_tags_3 },
    { 0xedfafbba, SUC_lexicon_tags_12 },
    { 0x68dafbba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb65afbc0, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dafbc5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x211afbc7, SUC_lexicon_tags_37 },
    { 0x57bafbc8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1afbcd, SUC_lexicon_tags_7 },
    { 0xbabafbce, SUC_lexicon_tags_7 },
    { 0x949afbcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bafbd5, SUC_lexicon_tags_36 },
    { 0xa59afbd6, SUC_lexicon_tags_2 },
    { 0xf0fafbd7, SUC_lexicon_tags_165 },
    { 0x977afbd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e5afbda, SUC_lexicon_tags_1 },
    { 0xdebafbda, SUC_lexicon_tags_1 },
    { 0x38fafbdc, SUC_lexicon_tags_25 },
    { 0xa09afbdb, SUC_lexicon_tags_23 },
    { 0x8b5afbdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15afbe1, SUC_lexicon_tags_11 },
    { 0x343afbe2, SUC_lexicon_tags_7 },
    { 0x951afbe2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239afbeb, SUC_lexicon_tags_2 },
    { 0x4cdafbec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x999afbee, SUC_lexicon_tags_82 },
    { 0x217afbef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dafbf5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49afc01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdafc03, SUC_lexicon_tags_106 },
    { 0x63fafc04, SUC_lexicon_tags_25 },
    { 0xcc5afc05, SUC_lexicon_tags_11 },
    { 0x115afc06, SUC_lexicon_tags_2 },
    { 0xf17afc04, SUC_lexicon_tags_11 },
    { 0x841afc05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x99afc0b, SUC_lexicon_tags_11 },
    { 0x1c7afc0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1afc0f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bafc13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1afc16, SUC_lexicon_tags_28 },
    { 0xd1bafc17, SUC_lexicon_tags_12 },
    { 0x39afc17, SUC_lexicon_tags_31 },
    { 0x5ffafc19, SUC_lexicon_tags_17 },
    { 0x973afc19, SUC_lexicon_tags_6 },
    { 0xfe3afc1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7afc1e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfafc24, SUC_lexicon_tags_63 },
    { 0x847afc25, SUC_lexicon_tags_17 },
    { 0xb45afc26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x827afc28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dafc2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557afc34, SUC_lexicon_tags_11 },
    { 0x71afc35, SUC_lexicon_tags_33 },
    { 0x2c3afc34, SUC_lexicon_tags_12 },
    { 0xf5dafc35, SUC_lexicon_tags_23 },
    { 0x7c5afc38, SUC_lexicon_tags_31 },
    { 0xb03afc39, SUC_lexicon_tags_19 },
    { 0x589afc35, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7afc3e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd4bafc40, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d7afc42, SUC_lexicon_tags_29 },
    { 0xedbafc43, SUC_lexicon_tags_68 },
    { 0xfa5afc42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16fafc46, SUC_lexicon_tags_2 },
    { 0x45bafc47, SUC_lexicon_tags_151 },
    { 0xf31afc48, SUC_lexicon_tags_1 },
    { 0x601afc48, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fafc4e, SUC_lexicon_tags_35 },
    { 0xf23afc4e, SUC_lexicon_tags_49 },
    { 0x421afc50, SUC_lexicon_tags_82 },
    { 0xaf1afc51, SUC_lexicon_tags_7 },
    { 0xc5afc52, SUC_lexicon_tags_7 },
    { 0xa3fafc52, SUC_lexicon_tags_25 },
    { 0x613afc50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x32fafc56, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1f7afc58, SUC_lexicon_tags_6 },
    { 0x885afc59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbafc5c, SUC_lexicon_tags_27 },
    { 0x5c9afc5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7afc66, SUC_lexicon_tags_25 },
    { 0x139afc66, SUC_lexicon_tags_66 },
    { 0xbdfafc68, SUC_lexicon_tags_37 },
    { 0x391afc68, SUC_lexicon_tags_2 },
    { 0xb0dafc68, SUC_lexicon_tags_7 },
    { 0x7edafc6a, SUC_lexicon_tags_19 },
    { 0xf1dafc67, SUC_lexicon_tags_30 },
    { 0xa6dafc6d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbafc6f, SUC_lexicon_tags_6 },
    { 0x655afc6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88bafc73, SUC_lexicon_tags_15 },
    { 0x86fafc74, SUC_lexicon_tags_17 },
    { 0x255afc75, SUC_lexicon_tags_7 },
    { 0xcffafc76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315afc7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadafc7f, SUC_lexicon_tags_7 },
    { 0x66fafc80, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dafc83, SUC_lexicon_tags_37 },
    { 0xb73afc83, SUC_lexicon_tags_17 },
    { 0x5fbafc84, SUC_lexicon_tags_7 },
    { 0xe93afc85, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935afc8c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9afc92, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0xfa3afc94, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dafc9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22dafc9d, SUC_lexicon_tags_5 },
    { 0x4a7afc9e, SUC_lexicon_tags_16 },
    { 0x387afc9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fafca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbafca6, SUC_lexicon_tags_1 },
    { 0x53dafca7, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581afcab, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x165afcad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbafcb4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffafcb7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9afcba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7afcbe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319afcc3, SUC_lexicon_tags_17 },
    { 0x6d7afcc4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5afccb, SUC_lexicon_tags_77 },
    { 0xe75afccc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebafcd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x155afcd2, SUC_lexicon_tags_32 },
    { 0x695afcd3, SUC_lexicon_tags_7 },
    { 0x5afcd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x997afcd6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9f3afcd8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efafce0, SUC_lexicon_tags_7 },
    { 0xadbafce1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5a3afce3, SUC_lexicon_tags_37 },
    { 0x56dafce4, SUC_lexicon_tags_45 },
    { 0x9bfafce5, SUC_lexicon_tags_28 },
    { 0xa01afce5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb03afce8, SUC_lexicon_tags_23 },
    { 0xda9afce8, SUC_lexicon_tags_10 },
    { 0x16dafce9, SUC_lexicon_tags_26 },
    { 0x101afceb, SUC_lexicon_tags_4 },
    { 0xf69afce8, SUC_lexicon_tags_68 },
    { 0x75bafced, SUC_lexicon_tags_7 },
    { 0xcc3afcee, SUC_lexicon_tags_6 },
    { 0x2c5afcef, SUC_lexicon_tags_11 },
    { 0xf1afcf0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bafcf3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70bafcf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71afcfa, SUC_lexicon_tags_35 },
    { 0x10dafcfa, SUC_lexicon_tags_45 },
    { 0xd87afcfc, SUC_lexicon_tags_38 },
    { 0x8ebafcfd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dafd01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7afd05, SUC_lexicon_tags_12 },
    { 0xaf7afd06, SUC_lexicon_tags_27 },
    { 0x7c3afd07, SUC_lexicon_tags_31 },
    { 0x6e1afd06, SUC_lexicon_tags_49 },
    { 0x605afd09, SUC_lexicon_tags_15 },
    { 0xb35afd0a, SUC_lexicon_tags_33 },
    { 0xcefafd0b, SUC_lexicon_tags_31 },
    { 0x207afd0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1afd12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5fafd14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d5afd16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63afd19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x995afd1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd97afd1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa49afd1f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb83afd21, SUC_lexicon_tags_1 },
    { 0xc75afd22, SUC_lexicon_tags_38 },
    { 0x4dfafd23, SUC_lexicon_tags_11 },
    { 0x8a1afd21, SUC_lexicon_tags_171 },
    { 0x889afd25, SUC_lexicon_tags_7 },
    { 0x54dafd25, SUC_lexicon_tags_64 },
    { 0xe49afd27, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x519afd2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19afd2d, SUC_lexicon_tags_14 },
    { 0xa5dafd2e, SUC_lexicon_tags_17 },
    { 0x2e3afd2f, SUC_lexicon_tags_27 },
    { 0xd19afd30, SUC_lexicon_tags_27 },
    { 0xfc1afd2e, SUC_lexicon_tags_8 },
    { 0x605afd32, SUC_lexicon_tags_0 },
    { 0xf57afd33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fafd3c, SUC_lexicon_tags_27 },
    { 0x187afd3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcbafd3f, SUC_lexicon_tags_14 },
    { 0xff5afd3f, SUC_lexicon_tags_1 },
    { 0x353afd41, SUC_lexicon_tags_25 },
    { 0xfd7afd41, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf99afd44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14fafd46, SUC_lexicon_tags_15 },
    { 0xb05afd47, SUC_lexicon_tags_17 },
    { 0xd63afd47, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf7bafd4a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedafd52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xadfafd54, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9afd57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8dafd59, SUC_lexicon_tags_33 },
    { 0xc87afd59, SUC_lexicon_tags_7 },
    { 0x829afd5b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9afd62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefafd65, SUC_lexicon_tags_60 },
    { 0xc89afd66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1afd6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x381afd71, SUC_lexicon_tags_43 },
    { 0x5fbafd72, SUC_lexicon_tags_33 },
    { 0x98dafd72, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613afd78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ebafd7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1afd7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x95dafd7f, SUC_lexicon_tags_1 },
    { 0xf99afd80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b9afd82, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x877afd84, SUC_lexicon_tags_104 },
    { 0x345afd84, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7afd8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb83afd8e, SUC_lexicon_tags_14 },
    { 0x289afd8f, SUC_lexicon_tags_11 },
    { 0xfd5afd8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a5afd92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685afd99, SUC_lexicon_tags_37 },
    { 0xebdafd9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257afda0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97afda2, SUC_lexicon_tags_33 },
    { 0x5d7afda2, SUC_lexicon_tags_61 },
    { 0xd7fafda3, SUC_lexicon_tags_37 },
    { 0x44dafda4, SUC_lexicon_tags_63 },
    { 0x1fdafda6, SUC_lexicon_tags_37 },
    { 0xbf5afda3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffafdab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbe9afdad, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3afdb2, SUC_lexicon_tags_103 },
    { 0x999afdb3, SUC_lexicon_tags_14 },
    { 0x86fafdb4, SUC_lexicon_tags_17 },
    { 0xeabafdb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x633afdb8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x375afdba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13afdbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x471afdbf, SUC_lexicon_tags_40 },
    { 0x63afdc0, SUC_lexicon_tags_7 },
    { 0x355afdc1, SUC_lexicon_tags_65 },
    { 0xaa7afdc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x325afdc4, SUC_lexicon_tags_38 },
    { 0x8a3afdc5, SUC_lexicon_tags_16 },
    { 0x4e7afdc6, SUC_lexicon_tags_7 },
    { 0x91bafdc6, SUC_lexicon_tags_23 },
    { 0xb8fafdc7, SUC_lexicon_tags_38 },
    { 0xc43afdc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9afdcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde1afdce, SUC_lexicon_tags_23 },
    { 0x2b7afdcf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc8dafdd1, SUC_lexicon_tags_12 },
    { 0xffafdd2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfafdd7, SUC_lexicon_tags_37 },
    { 0xc27afdd7, SUC_lexicon_tags_17 },
    { 0x393afdd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdafddd, SUC_lexicon_tags_49 },
    { 0x5cdafddd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9afde4, SUC_lexicon_tags_27 },
    { 0x72dafde5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x481afde7, SUC_lexicon_tags_31 },
    { 0x621afde8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x567afdea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fafdee, SUC_lexicon_tags_55 },
    { 0xc73afdee, SUC_lexicon_tags_14 },
    { 0x1bfafdf0, SUC_lexicon_tags_12 },
    { 0x529afdf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bafdf5, SUC_lexicon_tags_19 },
    { 0x5e7afdf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x997afdf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdafdfa, SUC_lexicon_tags_21 },
    { 0xe41afdfa, SUC_lexicon_tags_96 },
    { 0x80bafdfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x46bafdfe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfafe04, SUC_lexicon_tags_39 },
    { 0x3abafe05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffafe09, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11afe0c, SUC_lexicon_tags_65 },
    { 0xa93afe0d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4c9afe0f, SUC_lexicon_tags_92 },
    { 0x4d9afe10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdefafe12, SUC_lexicon_tags_30 },
    { 0x3c3afe13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fafe17, SUC_lexicon_tags_25 },
    { 0x79fafe18, SUC_lexicon_tags_17 },
    { 0x643afe17, SUC_lexicon_tags_6 },
    { 0x2d3afe17, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa53afe1c, SUC_lexicon_tags_12 },
    { 0x8c5afe1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99bafe1f, SUC_lexicon_tags_47 },
    { 0xab5afe1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61afe26, SUC_lexicon_tags_37 },
    { 0xa1afe27, SUC_lexicon_tags_11 },
    { 0xdd7afe28, SUC_lexicon_tags_2 },
    { 0x8ddafe29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf61afe2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa23afe2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfbafe2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d7afe31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72dafe33, SUC_lexicon_tags_66 },
    { 0x701afe34, SUC_lexicon_tags_11 },
    { 0x11dafe35, SUC_lexicon_tags_8 },
    { 0xc71afe36, SUC_lexicon_tags_27 },
    { 0xfd1afe37, SUC_lexicon_tags_1 },
    { 0xddafe34, SUC_lexicon_tags_12 },
    { 0xaf3afe38, SUC_lexicon_tags_2 },
    { 0xda9afe36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09afe40, SUC_lexicon_tags_63 },
    { 0x9d7afe40, SUC_lexicon_tags_17 },
    { 0xdc7afe42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf2bafe44, SUC_lexicon_tags_15 },
    { 0xfc7afe45, SUC_lexicon_tags_45 },
    { 0x3fdafe46, SUC_lexicon_tags_85 },
    { 0x13dafe45, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49afe4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x435afe4c, SUC_lexicon_tags_25 },
    { 0x5d7afe4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443afe51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x749afe54, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x355afe56, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bafe59, SUC_lexicon_tags_22 },
    { 0x2cfafe5a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x79fafe5d, SUC_lexicon_tags_37 },
    { 0xbd5afe5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bafe61, SUC_lexicon_tags_65 },
    { 0x27bafe62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fafe66, SUC_lexicon_tags_37 },
    { 0x97dafe67, SUC_lexicon_tags_25 },
    { 0x42bafe66, SUC_lexicon_tags_87 },
    { 0x811afe69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dafe70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bafe73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x577afe75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x345afe78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x159afe7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dafe7d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9afe82, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdefafe84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4afafe86, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdafe89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63afe8e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9afe91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fafe97, SUC_lexicon_tags_13 },
    { 0x573afe97, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbafe9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaadafe9d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33afea2, SUC_lexicon_tags_0 },
    { 0xfadafea3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dafea9, SUC_lexicon_tags_66 },
    { 0x9abafeaa, SUC_lexicon_tags_31 },
    { 0xc39afeaa, SUC_lexicon_tags_35 },
    { 0x11dafeac, SUC_lexicon_tags_23 },
    { 0xd8dafead, SUC_lexicon_tags_23 },
    { 0x9bdafead, SUC_lexicon_tags_11 },
    { 0x3c5afeae, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bafeb5, SUC_lexicon_tags_39 },
    { 0x7e1afeb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbafebb, SUC_lexicon_tags_33 },
    { 0x177afebc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bafebf, SUC_lexicon_tags_60 },
    { 0xbb9afec0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bafec4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8fafec6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x511afec8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc15afeca, SUC_lexicon_tags_11 },
    { 0xe65afeca, SUC_lexicon_tags_25 },
    { 0x693afecb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x153afecf, SUC_lexicon_tags_12 },
    { 0xcb3afed0, SUC_lexicon_tags_19 },
    { 0x977afed1, SUC_lexicon_tags_25 },
    { 0xb93afed2, SUC_lexicon_tags_23 },
    { 0x69afed2, SUC_lexicon_tags_64 },
    { 0xf59afed3, SUC_lexicon_tags_11 },
    { 0xd83afed5, SUC_lexicon_tags_31 },
    { 0x62fafed6, SUC_lexicon_tags_27 },
    { 0xd3dafed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edafedd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9afee0, SUC_lexicon_tags_27 },
    { 0xff3afee1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd01afee3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5afee8, SUC_lexicon_tags_39 },
    { 0x949afee9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbafeec, SUC_lexicon_tags_15 },
    { 0x275afeed, SUC_lexicon_tags_17 },
    { 0xeb1afeee, SUC_lexicon_tags_15 },
    { 0x805afeed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x585afef2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3dbafef4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3afefa, SUC_lexicon_tags_25 },
    { 0xc2dafefa, SUC_lexicon_tags_45 },
    { 0x7bbafefc, SUC_lexicon_tags_33 },
    { 0xbd9afefb, SUC_lexicon_tags_17 },
    { 0x29fafefb, SUC_lexicon_tags_7 },
    { 0xf27afeff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7baff01, SUC_lexicon_tags_15 },
    { 0x439aff02, SUC_lexicon_tags_8 },
    { 0xcd9aff02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x559aff05, SUC_lexicon_tags_234 },
    { 0xc37aff05, SUC_lexicon_tags_7 },
    { 0x663aff07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7adaff09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfaff0c, SUC_lexicon_tags_11 },
    { 0x47baff0c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x787aff0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0daff13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x18daff15, SUC_lexicon_tags_9 },
    { 0x69faff16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf69aff18, SUC_lexicon_tags_7 },
    { 0xd3daff19, SUC_lexicon_tags_68 },
    { 0xc83aff1a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x4d1aff1c, SUC_lexicon_tags_10 },
    { 0x423aff1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd17aff1f, SUC_lexicon_tags_25 },
    { 0x127aff20, SUC_lexicon_tags_11 },
    { 0xa7aff1f, SUC_lexicon_tags_37 },
    { 0x7aff22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4daff25, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdaff28, SUC_lexicon_tags_17 },
    { 0xea5aff29, SUC_lexicon_tags_7 },
    { 0x6afaff2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e3aff2c, SUC_lexicon_tags_25 },
    { 0x669aff2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfaff30, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22daff39, SUC_lexicon_tags_7 },
    { 0xb1baff3a, SUC_lexicon_tags_14 },
    { 0xdbbaff3b, SUC_lexicon_tags_49 },
    { 0x87aff3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afaff41, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xeabaff44, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7aff4b, SUC_lexicon_tags_17 },
    { 0xdb7aff4c, SUC_lexicon_tags_12 },
    { 0x49daff4d, SUC_lexicon_tags_10 },
    { 0x933aff4e, SUC_lexicon_tags_31 },
    { 0x435aff4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x169aff51, SUC_lexicon_tags_47 },
    { 0xfe9aff52, SUC_lexicon_tags_27 },
    { 0x27baff51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8daff59, SUC_lexicon_tags_31 },
    { 0x7ddaff5a, SUC_lexicon_tags_30 },
    { 0x5a9aff5a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x577aff5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a7aff60, SUC_lexicon_tags_21 },
    { 0x8fbaff61, SUC_lexicon_tags_33 },
    { 0xee5aff62, SUC_lexicon_tags_12 },
    { 0x9ddaff61, SUC_lexicon_tags_45 },
    { 0x46baff64, SUC_lexicon_tags_35 },
    { 0x825aff60, SUC_lexicon_tags_11 },
    { 0x799aff62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29aff6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51aff71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7aff78, SUC_lexicon_tags_30 },
    { 0xc73aff78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4faff7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c1aff7e, SUC_lexicon_tags_38 },
    { 0x3c9aff7f, SUC_lexicon_tags_5 },
    { 0x29faff7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8b1aff82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5aff87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2daff89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9aff8f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337aff96, SUC_lexicon_tags_15 },
    { 0x685aff96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119aff9b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3aff9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3a5affa0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0baffa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c1affaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebfaffac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57affaf, SUC_lexicon_tags_27 },
    { 0xabdaffb0, SUC_lexicon_tags_39 },
    { 0x1e7affb1, SUC_lexicon_tags_0 },
    { 0x4b3affb2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9bbaffb4, SUC_lexicon_tags_28 },
    { 0xfbaffb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x337affb7, SUC_lexicon_tags_21 },
    { 0xadfaffb8, SUC_lexicon_tags_14 },
    { 0x57affb7, SUC_lexicon_tags_60 },
    { 0x83daffba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157affbe, SUC_lexicon_tags_12 },
    { 0xd01affbe, SUC_lexicon_tags_185 },
    { 0x5d5affc0, SUC_lexicon_tags_33 },
    { 0xc45affc1, SUC_lexicon_tags_25 },
    { 0x8ffaffc2, SUC_lexicon_tags_33 },
    { 0x411affbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafaffc6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x715affc9, SUC_lexicon_tags_60 },
    { 0x183affca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2e9affcc, SUC_lexicon_tags_11 },
    { 0xbb3affcd, SUC_lexicon_tags_37 },
    { 0x40faffcd, SUC_lexicon_tags_37 },
    { 0xb8faffce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x407affd1, SUC_lexicon_tags_60 },
    { 0xa31affd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x321affd4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1affd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10faffdc, SUC_lexicon_tags_7 },
    { 0xd89affdd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0daffe6, SUC_lexicon_tags_17 },
    { 0x4cbaffe7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fafff3, SUC_lexicon_tags_43 },
    { 0xe2fafff4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131afffd, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0x48db0000, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9b0004, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09b0008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525b000d, SUC_lexicon_tags_15 },
    { 0x77db000d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05b0014, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387b0018, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7b0022, SUC_lexicon_tags_2 },
    { 0xe6db0023, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b002b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42db002e, SUC_lexicon_tags_21 },
    { 0x921b002f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeefb0031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdb0035, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc97b0037, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d9b0039, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5b003b, SUC_lexicon_tags_48 },
    { 0xe8db003c, SUC_lexicon_tags_16 },
    { 0xfd5b003b, SUC_lexicon_tags_26 },
    { 0x897b003e, SUC_lexicon_tags_15 },
    { 0xef1b003f, SUC_lexicon_tags_31 },
    { 0x16bb003f, SUC_lexicon_tags_273 },
    { 0x9b0041, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4afb0043, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1f5b0045, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75b0048, SUC_lexicon_tags_3 },
    { 0xcc3b0049, SUC_lexicon_tags_7 },
    { 0x2b7b004a, SUC_lexicon_tags_15 },
    { 0xe91b004b, SUC_lexicon_tags_17 },
    { 0x5d5b0049, SUC_lexicon_tags_7 },
    { 0x20db004d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0fb004f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b0056, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd2db0058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fb005c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb19b005e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5db0060, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81b0064, SUC_lexicon_tags_43 },
    { 0x903b0064, SUC_lexicon_tags_45 },
    { 0xf9db0064, SUC_lexicon_tags_17 },
    { 0xb73b0067, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdfb0069, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x691b006b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x929b006d, SUC_lexicon_tags_27 },
    { 0x173b006d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x29bb0070, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19b0079, SUC_lexicon_tags_17 },
    { 0x5a3b007a, SUC_lexicon_tags_15 },
    { 0x84db007b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xffb007e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x327b0081, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fb0088, SUC_lexicon_tags_62 },
    { 0x6abb0088, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9bb008b, SUC_lexicon_tags_1 },
    { 0x569b008c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9b0092, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x873b0094, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21b0097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfcdb0099, SUC_lexicon_tags_45 },
    { 0xdcdb009a, SUC_lexicon_tags_15 },
    { 0x215b009b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78db00a4, SUC_lexicon_tags_17 },
    { 0x7d7b00a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe19b00a7, SUC_lexicon_tags_29 },
    { 0x63bb00a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3db00ab, SUC_lexicon_tags_28 },
    { 0x87db00ab, SUC_lexicon_tags_11 },
    { 0xf19b00ad, SUC_lexicon_tags_23 },
    { 0x5bfb00ab, SUC_lexicon_tags_47 },
    { 0xb33b00af, SUC_lexicon_tags_34 },
    { 0xa0fb00af, SUC_lexicon_tags_37 },
    { 0x731b00af, SUC_lexicon_tags_48 },
    { 0x1b00b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85b00b5, SUC_lexicon_tags_25 },
    { 0x917b00b6, SUC_lexicon_tags_45 },
    { 0xed5b00b5, SUC_lexicon_tags_7 },
    { 0xed9b00b5, SUC_lexicon_tags_17 },
    { 0x65b00b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157b00bf, SUC_lexicon_tags_9 },
    { 0x55db00c0, SUC_lexicon_tags_200 },
    { 0x1f9b00c1, SUC_lexicon_tags_22 },
    { 0x94bb00c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d1b00c4, SUC_lexicon_tags_61 },
    { 0x967b00c4, SUC_lexicon_tags_23 },
    { 0x153b00c5, SUC_lexicon_tags_53 },
    { 0xd47b00c4, SUC_lexicon_tags_17 },
    { 0x399b00c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdb00ca, SUC_lexicon_tags_19 },
    { 0xf3db00cb, SUC_lexicon_tags_7 },
    { 0x583b00cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791b00d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3b00d7, SUC_lexicon_tags_6 },
    { 0x45b00d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfb00db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b00e1, SUC_lexicon_tags_48 },
    { 0x5f1b00e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3b00e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b00ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103b00f4, SUC_lexicon_tags_14 },
    { 0xee5b00f5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc01b00f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf29b00f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd2db00fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9b0100, SUC_lexicon_tags_11 },
    { 0x39b0101, SUC_lexicon_tags_1 },
    { 0xb75b0102, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x529b0104, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1fb0106, SUC_lexicon_tags_27 },
    { 0xb79b0106, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3bb0109, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a3b010b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299b0110, SUC_lexicon_tags_146 },
    { 0xeb5b0111, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x223b0113, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b0116, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b011b, SUC_lexicon_tags_11 },
    { 0x3f3b011c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdb0121, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeadb0123, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fb0126, SUC_lexicon_tags_25 },
    { 0xf9db0127, SUC_lexicon_tags_11 },
    { 0x7ebb0128, SUC_lexicon_tags_11 },
    { 0xce5b0128, SUC_lexicon_tags_182 },
    { 0xbb7b012a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x571b012d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x953b012f, SUC_lexicon_tags_17 },
    { 0xf11b0130, SUC_lexicon_tags_37 },
    { 0x239b0131, SUC_lexicon_tags_1 },
    { 0xf07b0132, SUC_lexicon_tags_19 },
    { 0xd5b0133, SUC_lexicon_tags_17 },
    { 0xc2bb0134, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25b0138, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x89fb013a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bb013d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x927b0140, SUC_lexicon_tags_34 },
    { 0x831b0140, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe37b0143, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x305b0145, SUC_lexicon_tags_17 },
    { 0x2b1b0145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799b014b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9fbb014d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db0153, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37b0156, SUC_lexicon_tags_14 },
    { 0xcf7b0157, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3fb0159, SUC_lexicon_tags_27 },
    { 0xd21b015a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bb015d, SUC_lexicon_tags_37 },
    { 0xc01b015e, SUC_lexicon_tags_31 },
    { 0xa55b015d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fb0162, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x37db0165, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1b0174, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827b0178, SUC_lexicon_tags_23 },
    { 0x603b0178, SUC_lexicon_tags_91 },
    { 0xa37b017a, SUC_lexicon_tags_1 },
    { 0x80db017b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bb0186, SUC_lexicon_tags_23 },
    { 0xb63b0187, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x455b0189, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe0db018b, SUC_lexicon_tags_7 },
    { 0xe4db018b, SUC_lexicon_tags_8 },
    { 0xe7db018c, SUC_lexicon_tags_0 },
    { 0x19fb018e, SUC_lexicon_tags_65 },
    { 0x7cfb018e, SUC_lexicon_tags_25 },
    { 0xd8fb018d, SUC_lexicon_tags_13 },
    { 0x92bb0191, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x261b0193, SUC_lexicon_tags_37 },
    { 0x10bb0193, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5b019a, SUC_lexicon_tags_12 },
    { 0x6e3b019b, SUC_lexicon_tags_27 },
    { 0xef3b019c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb019f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9b01a6, SUC_lexicon_tags_37 },
    { 0x74bb01a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37b01ac, SUC_lexicon_tags_8 },
    { 0xd9fb01ad, SUC_lexicon_tags_47 },
    { 0xcdfb01ac, SUC_lexicon_tags_11 },
    { 0x215b01af, SUC_lexicon_tags_87 },
    { 0x4d7b01ac, SUC_lexicon_tags_39 },
    { 0x511b01ac, SUC_lexicon_tags_13 },
    { 0xc1fb01b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ddb01b4, SUC_lexicon_tags_25 },
    { 0x927b01b5, SUC_lexicon_tags_12 },
    { 0x99b01b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x777b01b9, SUC_lexicon_tags_25 },
    { 0x9db01ba, SUC_lexicon_tags_53 },
    { 0xfbfb01bb, SUC_lexicon_tags_16 },
    { 0xbe1b01bc, SUC_lexicon_tags_37 },
    { 0x8d1b01bd, SUC_lexicon_tags_25 },
    { 0x137b01bd, SUC_lexicon_tags_12 },
    { 0x26db01bf, SUC_lexicon_tags_19 },
    { 0x8e5b01bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7db01c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fb01c8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe61b01ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fb01ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1b01d1, SUC_lexicon_tags_38 },
    { 0x42db01d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0db01d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9b01dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3db01df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d1b01e1, SUC_lexicon_tags_85 },
    { 0x30bb01e2, SUC_lexicon_tags_71 },
    { 0x92fb01e2, SUC_lexicon_tags_23 },
    { 0x48db01e4, SUC_lexicon_tags_12 },
    { 0x351b01e5, SUC_lexicon_tags_56 },
    { 0x525b01e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777b01ea, SUC_lexicon_tags_0 },
    { 0x323b01ea, SUC_lexicon_tags_49 },
    { 0xf25b01ec, SUC_lexicon_tags_25 },
    { 0x5ebb01ed, SUC_lexicon_tags_17 },
    { 0xcfbb01ea, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0xf9db01f0, SUC_lexicon_tags_32 },
    { 0xe69b01f1, SUC_lexicon_tags_60 },
    { 0x8bbb01f2, SUC_lexicon_tags_63 },
    { 0xa7fb01f3, SUC_lexicon_tags_0 },
    { 0xd4bb01f0, SUC_lexicon_tags_25 },
    { 0x135b01f4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67db01fe, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3b0209, SUC_lexicon_tags_33 },
    { 0x6f3b020a, SUC_lexicon_tags_37 },
    { 0xecbb020b, SUC_lexicon_tags_25 },
    { 0xbfdb020b, SUC_lexicon_tags_2 },
    { 0xe09b020d, SUC_lexicon_tags_9 },
    { 0x8afb0209, SUC_lexicon_tags_25 },
    { 0x17db020f, SUC_lexicon_tags_27 },
    { 0x54fb020e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7b0216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa85b0218, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69b021c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x289b021e, SUC_lexicon_tags_17 },
    { 0xe9db021f, SUC_lexicon_tags_47 },
    { 0xe01b021f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbb0227, SUC_lexicon_tags_12 },
    { 0x753b0228, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6c7b022a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdb022e, SUC_lexicon_tags_60 },
    { 0xd89b022f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b0235, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb023b, SUC_lexicon_tags_141 },
    { 0x31db023c, SUC_lexicon_tags_45 },
    { 0xf89b023b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebb0240, SUC_lexicon_tags_2 },
    { 0x4c3b0241, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93b0245, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe2bb0247, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x855b0249, SUC_lexicon_tags_25 },
    { 0x5ffb024a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3b024d, SUC_lexicon_tags_84 },
    { 0xe3db024e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa1b0250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x311b0253, SUC_lexicon_tags_17 },
    { 0xd55b0253, SUC_lexicon_tags_13 },
    { 0x4f1b0255, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4db0257, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd55b0259, SUC_lexicon_tags_27 },
    { 0x70bb0259, SUC_lexicon_tags_40 },
    { 0xcebb025b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ffb025d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb09b025f, SUC_lexicon_tags_15 },
    { 0x41fb0260, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3b0265, SUC_lexicon_tags_96 },
    { 0xa5db0266, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77bb0268, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3b026b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdb0271, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdb0276, SUC_lexicon_tags_1 },
    { 0x923b0277, SUC_lexicon_tags_1 },
    { 0x28bb0277, SUC_lexicon_tags_92 },
    { 0x70db0277, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x863b027c, SUC_lexicon_tags_37 },
    { 0x18db027c, SUC_lexicon_tags_11 },
    { 0xb23b027e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afb0282, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41b0286, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf6db0288, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x42bb028a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b028e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227b0292, SUC_lexicon_tags_15 },
    { 0x663b0293, SUC_lexicon_tags_2 },
    { 0xabfb0293, SUC_lexicon_tags_7 },
    { 0x19db0293, SUC_lexicon_tags_49 },
    { 0x8c3b0296, SUC_lexicon_tags_25 },
    { 0x7ddb0297, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9c9b0299, SUC_lexicon_tags_302 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7b029c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b029f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb02a3, SUC_lexicon_tags_20 },
    { 0xbb3b02a4, SUC_lexicon_tags_23 },
    { 0x47bb02a5, SUC_lexicon_tags_1 },
    { 0x135b02a5, SUC_lexicon_tags_15 },
    { 0x9f5b02a7, SUC_lexicon_tags_25 },
    { 0x211b02a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b7b02aa, SUC_lexicon_tags_11 },
    { 0xb89b02ab, SUC_lexicon_tags_45 },
    { 0x43bb02ab, SUC_lexicon_tags_17 },
    { 0xa39b02ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51b02b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadb02b8, SUC_lexicon_tags_9 },
    { 0x41db02b9, SUC_lexicon_tags_29 },
    { 0xfefb02b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ddb02bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b02c0, SUC_lexicon_tags_12 },
    { 0xa3bb02c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27b02c5, SUC_lexicon_tags_17 },
    { 0x38fb02c6, SUC_lexicon_tags_77 },
    { 0x387b02c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bb02ca, SUC_lexicon_tags_25 },
    { 0x9f3b02cb, SUC_lexicon_tags_37 },
    { 0x153b02ca, SUC_lexicon_tags_43 },
    { 0x1dfb02cd, SUC_lexicon_tags_2 },
    { 0xe71b02ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47bb02d0, SUC_lexicon_tags_11 },
    { 0x6c7b02d1, SUC_lexicon_tags_12 },
    { 0xb79b02d2, SUC_lexicon_tags_79 },
    { 0xacfb02d3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x97bb02d5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xeadb02d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225b02dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fb02e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77b02ed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35b02f5, SUC_lexicon_tags_0 },
    { 0x173b02f6, SUC_lexicon_tags_33 },
    { 0xd3db02f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x633b02fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b0303, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afb0306, SUC_lexicon_tags_47 },
    { 0xb93b0307, SUC_lexicon_tags_0 },
    { 0x957b0308, SUC_lexicon_tags_37 },
    { 0xdd9b0306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafb030e, SUC_lexicon_tags_37 },
    { 0x2e1b030e, SUC_lexicon_tags_37 },
    { 0x3db0310, SUC_lexicon_tags_87 },
    { 0x689b0311, SUC_lexicon_tags_7 },
    { 0xf9b0312, SUC_lexicon_tags_60 },
    { 0x7e1b0313, SUC_lexicon_tags_0 },
    { 0x293b0313, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859b0318, SUC_lexicon_tags_1 },
    { 0x5d1b0319, SUC_lexicon_tags_2 },
    { 0x44db031a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x735b031c, SUC_lexicon_tags_59 },
    { 0xcc1b031d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedb0321, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x829b0324, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3b0328, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7efb032b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83b0332, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599b0338, SUC_lexicon_tags_27 },
    { 0xcc7b0338, SUC_lexicon_tags_45 },
    { 0x16db0339, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfcbb033c, SUC_lexicon_tags_47 },
    { 0x3c7b033d, SUC_lexicon_tags_19 },
    { 0x849b033e, SUC_lexicon_tags_34 },
    { 0x135b033e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb61b0341, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x671b0343, SUC_lexicon_tags_11 },
    { 0x97fb0344, SUC_lexicon_tags_6 },
    { 0x305b0345, SUC_lexicon_tags_26 },
    { 0x4e5b0346, SUC_lexicon_tags_37 },
    { 0xb65b0345, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fb034e, SUC_lexicon_tags_48 },
    { 0xedb034f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd97b0351, SUC_lexicon_tags_19 },
    { 0xc27b0352, SUC_lexicon_tags_25 },
    { 0xa3fb0353, SUC_lexicon_tags_28 },
    { 0xbe9b0351, SUC_lexicon_tags_63 },
    { 0x795b0355, SUC_lexicon_tags_26 },
    { 0x149b0354, SUC_lexicon_tags_3 },
    { 0xf3fb0355, SUC_lexicon_tags_7 },
    { 0xd51b0358, SUC_lexicon_tags_7 },
    { 0x855b0359, SUC_lexicon_tags_7 },
    { 0xd6db0359, SUC_lexicon_tags_47 },
    { 0xc95b035b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fb035e, SUC_lexicon_tags_17 },
    { 0x937b035e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4db0361, SUC_lexicon_tags_15 },
    { 0x97fb0362, SUC_lexicon_tags_10 },
    { 0xe51b0363, SUC_lexicon_tags_27 },
    { 0xcf3b0364, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x919b0367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa15b0369, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31b036e, SUC_lexicon_tags_17 },
    { 0x6a7b036f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdb0371, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405b0375, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x167b0377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbb037a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5b037e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89b0382, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733b0387, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x523b0389, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x985b038b, SUC_lexicon_tags_37 },
    { 0x99bb038c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x607b038f, SUC_lexicon_tags_218 },
    { 0xf01b038f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453b0399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb03a1, SUC_lexicon_tags_37 },
    { 0xa1bb03a2, SUC_lexicon_tags_11 },
    { 0x641b03a3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1b03af, SUC_lexicon_tags_3 },
    { 0x853b03b0, SUC_lexicon_tags_17 },
    { 0xc33b03b1, SUC_lexicon_tags_12 },
    { 0x47b03b0, SUC_lexicon_tags_70 },
    { 0x469b03b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153b03b8, SUC_lexicon_tags_25 },
    { 0x69db03b8, SUC_lexicon_tags_17 },
    { 0x6fdb03ba, SUC_lexicon_tags_63 },
    { 0x577b03b9, SUC_lexicon_tags_11 },
    { 0x5ddb03bc, SUC_lexicon_tags_1 },
    { 0x131b03bd, SUC_lexicon_tags_85 },
    { 0xf5bb03bd, SUC_lexicon_tags_0 },
    { 0x35b03bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2bb03c1, SUC_lexicon_tags_11 },
    { 0x4f9b03c2, SUC_lexicon_tags_37 },
    { 0xe29b03c2, SUC_lexicon_tags_31 },
    { 0x1c1b03c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611b03ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9b03ce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5b03d1, SUC_lexicon_tags_1 },
    { 0xab1b03d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87b03d5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91b03d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13b03e4, SUC_lexicon_tags_7 },
    { 0xb49b03e5, SUC_lexicon_tags_19 },
    { 0xf23b03e6, SUC_lexicon_tags_69 },
    { 0x299b03e7, SUC_lexicon_tags_25 },
    { 0x37bb03e7, SUC_lexicon_tags_45 },
    { 0x711b03e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd3b03eb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x64fb03ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21b03f1, SUC_lexicon_tags_2 },
    { 0x747b03f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b1b03f4, SUC_lexicon_tags_12 },
    { 0x44bb03f5, SUC_lexicon_tags_7 },
    { 0xc1fb03f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c5b03f8, SUC_lexicon_tags_130 },
    { 0x411b03f8, SUC_lexicon_tags_26 },
    { 0xd8fb03f9, SUC_lexicon_tags_7 },
    { 0x807b03f9, SUC_lexicon_tags_37 },
    { 0x963b03fb, SUC_lexicon_tags_63 },
    { 0x4c5b03fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x255b03ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x529b0402, SUC_lexicon_tags_12 },
    { 0x861b0403, SUC_lexicon_tags_27 },
    { 0x4e1b0403, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5b0409, SUC_lexicon_tags_33 },
    { 0x7efb040a, SUC_lexicon_tags_34 },
    { 0x75b040b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bb040e, SUC_lexicon_tags_1 },
    { 0xf57b040f, SUC_lexicon_tags_37 },
    { 0x27b0410, SUC_lexicon_tags_12 },
    { 0xee3b0411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x945b0413, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bb0416, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483b0420, SUC_lexicon_tags_15 },
    { 0x58fb0421, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x153b0423, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1b0426, SUC_lexicon_tags_49 },
    { 0xe27b0427, SUC_lexicon_tags_1 },
    { 0xed9b0427, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1b5b042a, SUC_lexicon_tags_27 },
    { 0xa91b042b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db0435, SUC_lexicon_tags_38 },
    { 0x10db0436, SUC_lexicon_tags_69 },
    { 0xd7bb0436, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1b043b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65db0443, SUC_lexicon_tags_6 },
    { 0xeb5b0444, SUC_lexicon_tags_12 },
    { 0x77fb0443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab3b0447, SUC_lexicon_tags_25 },
    { 0x70bb0448, SUC_lexicon_tags_11 },
    { 0xa3db0449, SUC_lexicon_tags_87 },
    { 0xe5b044a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbb044d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297b0455, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5b0458, SUC_lexicon_tags_29 },
    { 0x2db0459, SUC_lexicon_tags_43 },
    { 0x82fb0459, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9b0462, SUC_lexicon_tags_37 },
    { 0xccfb0463, SUC_lexicon_tags_30 },
    { 0x911b0464, SUC_lexicon_tags_11 },
    { 0x727b0465, SUC_lexicon_tags_12 },
    { 0xb0fb0466, SUC_lexicon_tags_7 },
    { 0x1ebb0466, SUC_lexicon_tags_11 },
    { 0x9bbb0468, SUC_lexicon_tags_0 },
    { 0xf7db0469, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd01b046b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61b0471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7b0478, SUC_lexicon_tags_11 },
    { 0x621b0479, SUC_lexicon_tags_15 },
    { 0xd83b0479, SUC_lexicon_tags_38 },
    { 0xeebb047a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95b0480, SUC_lexicon_tags_17 },
    { 0x381b0481, SUC_lexicon_tags_12 },
    { 0x15bb0482, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7efb0485, SUC_lexicon_tags_17 },
    { 0x2c7b0486, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bb048a, SUC_lexicon_tags_25 },
    { 0x947b048a, SUC_lexicon_tags_17 },
    { 0xaf1b048b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5b0490, SUC_lexicon_tags_45 },
    { 0xc4fb0491, SUC_lexicon_tags_7 },
    { 0x8a5b0492, SUC_lexicon_tags_61 },
    { 0xc3bb0490, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc07b0495, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fb0499, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439b04a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1b04a4, SUC_lexicon_tags_17 },
    { 0xc91b04a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f3b04a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35b04ab, SUC_lexicon_tags_7 },
    { 0xd8db04ac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x69db04ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fb04b2, SUC_lexicon_tags_11 },
    { 0x1e3b04b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5b04c1, SUC_lexicon_tags_2 },
    { 0xbfb04c1, SUC_lexicon_tags_15 },
    { 0x559b04c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2edb04c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9bb04c8, SUC_lexicon_tags_25 },
    { 0x291b04c9, SUC_lexicon_tags_47 },
    { 0x587b04ca, SUC_lexicon_tags_61 },
    { 0xa4bb04c8, SUC_lexicon_tags_0 },
    { 0xb73b04c8, SUC_lexicon_tags_17 },
    { 0x15bb04cd, SUC_lexicon_tags_25 },
    { 0x177b04ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa01b04d0, SUC_lexicon_tags_31 },
    { 0xa1db04d1, SUC_lexicon_tags_2 },
    { 0xdd1b04d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x265b04d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30db04dc, SUC_lexicon_tags_32 },
    { 0x60db04dc, SUC_lexicon_tags_14 },
    { 0x1e9b04de, SUC_lexicon_tags_37 },
    { 0xd3b04de, SUC_lexicon_tags_45 },
    { 0xba3b04dd, SUC_lexicon_tags_37 },
    { 0xae1b04dc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b04e5, SUC_lexicon_tags_12 },
    { 0xd6fb04e6, SUC_lexicon_tags_23 },
    { 0x2a7b04e5, SUC_lexicon_tags_7 },
    { 0xf1fb04e8, SUC_lexicon_tags_25 },
    { 0x481b04e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fb04ed, SUC_lexicon_tags_15 },
    { 0xb89b04ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bb04f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01b04f7, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xb35b04f9, SUC_lexicon_tags_37 },
    { 0xa43b04fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9b04fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efb0503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215b050a, SUC_lexicon_tags_17 },
    { 0x497b050b, SUC_lexicon_tags_11 },
    { 0xc3fb050c, SUC_lexicon_tags_18 },
    { 0xbdbb050d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x763b050f, SUC_lexicon_tags_31 },
    { 0x695b0510, SUC_lexicon_tags_14 },
    { 0xdf3b0511, SUC_lexicon_tags_12 },
    { 0x85bb050f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5edb0514, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3b0517, SUC_lexicon_tags_37 },
    { 0xd25b0518, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1b051d, SUC_lexicon_tags_7 },
    { 0xd59b051e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787b0523, SUC_lexicon_tags_63 },
    { 0xa43b0524, SUC_lexicon_tags_11 },
    { 0x503b0525, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b052a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7b052e, SUC_lexicon_tags_17 },
    { 0x3f9b052f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9b0532, SUC_lexicon_tags_27 },
    { 0xd95b0533, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3b0537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75b053b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60db053d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x93b053f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625b0544, SUC_lexicon_tags_32 },
    { 0xa75b0545, SUC_lexicon_tags_7 },
    { 0xf11b0545, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9b0550, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95b0553, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdb0558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb3b055a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bb055f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7b0564, SUC_lexicon_tags_49 },
    { 0x4c9b0565, SUC_lexicon_tags_1 },
    { 0x1e5b0566, SUC_lexicon_tags_25 },
    { 0xf51b0567, SUC_lexicon_tags_22 },
    { 0x161b0568, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2a1b056a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151b056e, SUC_lexicon_tags_19 },
    { 0xa51b056f, SUC_lexicon_tags_11 },
    { 0xef9b056e, SUC_lexicon_tags_2 },
    { 0xf07b056e, SUC_lexicon_tags_27 },
    { 0x7a5b056e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x759b0574, SUC_lexicon_tags_12 },
    { 0x8b5b0575, SUC_lexicon_tags_47 },
    { 0x7e9b0575, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbb0579, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fb057e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd61b0580, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adb0586, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x929b0588, SUC_lexicon_tags_1 },
    { 0x713b0588, SUC_lexicon_tags_7 },
    { 0xc9bb058a, SUC_lexicon_tags_1 },
    { 0x6a9b058a, SUC_lexicon_tags_13 },
    { 0x87bb058c, SUC_lexicon_tags_12 },
    { 0xc91b0588, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f5b058f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x83b0591, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7b0594, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb059b, SUC_lexicon_tags_33 },
    { 0x691b059b, SUC_lexicon_tags_1 },
    { 0x751b059d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x491b05a0, SUC_lexicon_tags_4 },
    { 0x387b05a1, SUC_lexicon_tags_19 },
    { 0x945b05a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7b05ad, SUC_lexicon_tags_11 },
    { 0xce5b05ae, SUC_lexicon_tags_15 },
    { 0x3afb05af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x699b05b1, SUC_lexicon_tags_63 },
    { 0x30db05b2, SUC_lexicon_tags_7 },
    { 0xbb7b05b2, SUC_lexicon_tags_40 },
    { 0x44db05b1, SUC_lexicon_tags_37 },
    { 0x671b05b5, SUC_lexicon_tags_15 },
    { 0x6f1b05b6, SUC_lexicon_tags_25 },
    { 0xc29b05b4, SUC_lexicon_tags_63 },
    { 0xaabb05b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07b05c0, SUC_lexicon_tags_27 },
    { 0x82fb05c1, SUC_lexicon_tags_46 },
    { 0x803b05c0, SUC_lexicon_tags_2 },
    { 0xfa7b05c3, SUC_lexicon_tags_6 },
    { 0xd1db05c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cdb05c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5b05c9, SUC_lexicon_tags_19 },
    { 0x331b05ca, SUC_lexicon_tags_60 },
    { 0xf2fb05cb, SUC_lexicon_tags_1 },
    { 0xa93b05ca, SUC_lexicon_tags_7 },
    { 0x25db05cd, SUC_lexicon_tags_47 },
    { 0xf3fb05ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61bb05d0, SUC_lexicon_tags_29 },
    { 0xb15b05d0, SUC_lexicon_tags_65 },
    { 0x32fb05d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x735b05d5, SUC_lexicon_tags_37 },
    { 0x36fb05d6, SUC_lexicon_tags_21 },
    { 0xf59b05d6, SUC_lexicon_tags_37 },
    { 0x99fb05d5, SUC_lexicon_tags_43 },
    { 0x69b05d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc75b05db, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443b05df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bb05e2, SUC_lexicon_tags_17 },
    { 0x9db05e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc7b05e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd49b05e7, SUC_lexicon_tags_17 },
    { 0x597b05e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bb05ec, SUC_lexicon_tags_37 },
    { 0x2bb05ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27db05f1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9b05f4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x7bdb05f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335b05fc, SUC_lexicon_tags_1 },
    { 0xba9b05fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x549b0600, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575b0604, SUC_lexicon_tags_12 },
    { 0x6b5b0605, SUC_lexicon_tags_17 },
    { 0x585b0606, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91b0609, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xadb060b, SUC_lexicon_tags_17 },
    { 0x6bb060c, SUC_lexicon_tags_63 },
    { 0x55bb060d, SUC_lexicon_tags_39 },
    { 0xa7fb060e, SUC_lexicon_tags_45 },
    { 0xad3b060e, SUC_lexicon_tags_47 },
    { 0x72bb0610, SUC_lexicon_tags_38 },
    { 0xb37b0611, SUC_lexicon_tags_60 },
    { 0x209b060d, SUC_lexicon_tags_37 },
    { 0x677b0613, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e5b0615, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92db0619, SUC_lexicon_tags_60 },
    { 0x2f7b061a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcc1b061c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbb0621, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbb0624, SUC_lexicon_tags_11 },
    { 0xa17b0625, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445b0629, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fb062c, SUC_lexicon_tags_14 },
    { 0x91bb062d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x615b062f, SUC_lexicon_tags_37 },
    { 0x2f9b0630, SUC_lexicon_tags_25 },
    { 0x365b0631, SUC_lexicon_tags_37 },
    { 0x2abb0631, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdfdb0634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149b063a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fb063e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4edb0641, SUC_lexicon_tags_7 },
    { 0xa35b0642, SUC_lexicon_tags_8 },
    { 0x695b0641, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x637b0646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7b064a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1b064e, SUC_lexicon_tags_53 },
    { 0xadbb064e, SUC_lexicon_tags_37 },
    { 0x913b0650, SUC_lexicon_tags_14 },
    { 0xafb064f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf47b0653, SUC_lexicon_tags_63 },
    { 0x2b5b0654, SUC_lexicon_tags_11 },
    { 0x601b0654, SUC_lexicon_tags_37 },
    { 0xf5fb0653, SUC_lexicon_tags_15 },
    { 0xacfb0654, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03b065b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fb065e, SUC_lexicon_tags_25 },
    { 0x64fb065e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x319b0662, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23db0665, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6a1b0667, SUC_lexicon_tags_15 },
    { 0xc91b0668, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97db066e, SUC_lexicon_tags_15 },
    { 0x915b066f, SUC_lexicon_tags_17 },
    { 0xdabb0670, SUC_lexicon_tags_49 },
    { 0x691b0670, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbb0674, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5b0677, SUC_lexicon_tags_43 },
    { 0xdc3b0678, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x781b067a, SUC_lexicon_tags_37 },
    { 0x8efb067b, SUC_lexicon_tags_46 },
    { 0xba3b067c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1afb067e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b0686, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x95db0689, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b068f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x335b0691, SUC_lexicon_tags_12 },
    { 0x16fb0691, SUC_lexicon_tags_16 },
    { 0xe7db0693, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495b0697, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xafbb0699, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x689b069c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe49b069e, SUC_lexicon_tags_7 },
    { 0x49b069f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49db06a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b06ad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9b06b4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2fbb06b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbb06ba, SUC_lexicon_tags_57 },
    { 0xe91b06bb, SUC_lexicon_tags_12 },
    { 0x10db06bc, SUC_lexicon_tags_12 },
    { 0x3b06bb, SUC_lexicon_tags_14 },
    { 0x91b06bd, SUC_lexicon_tags_30 },
    { 0x171b06bf, SUC_lexicon_tags_8 },
    { 0x7c5b06c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x115b06c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49b06c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f5b06c9, SUC_lexicon_tags_55 },
    { 0xeadb06c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x187b06cc, SUC_lexicon_tags_15 },
    { 0xf69b06cd, SUC_lexicon_tags_44 },
    { 0x797b06ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe67b06d0, SUC_lexicon_tags_17 },
    { 0x163b06d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc7db06d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7b06d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x255b06d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe8bb06da, SUC_lexicon_tags_0 },
    { 0xa01b06db, SUC_lexicon_tags_0 },
    { 0xbdb06dc, SUC_lexicon_tags_2 },
    { 0xd09b06dd, SUC_lexicon_tags_15 },
    { 0xe07b06db, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x959b06e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xed9b06e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x555b06e4, SUC_lexicon_tags_25 },
    { 0xfa9b06e5, SUC_lexicon_tags_26 },
    { 0xdf5b06e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3b06eb, SUC_lexicon_tags_36 },
    { 0x255b06eb, SUC_lexicon_tags_19 },
    { 0x207b06eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b5b06ef, SUC_lexicon_tags_82 },
    { 0x105b06f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39b06f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fb06f8, SUC_lexicon_tags_7 },
    { 0x285b06f9, SUC_lexicon_tags_15 },
    { 0xcd7b06fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdb06ff, SUC_lexicon_tags_15 },
    { 0xbbdb0700, SUC_lexicon_tags_2 },
    { 0x749b0700, SUC_lexicon_tags_25 },
    { 0x1b1b0700, SUC_lexicon_tags_31 },
    { 0x9b06ff, SUC_lexicon_tags_8 },
    { 0xa4db0702, SUC_lexicon_tags_60 },
    { 0xd75b0705, SUC_lexicon_tags_13 },
    { 0x33db0700, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3b0711, SUC_lexicon_tags_25 },
    { 0xf33b0712, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19bb0716, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x2d7b0718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x947b071a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fb0721, SUC_lexicon_tags_25 },
    { 0x2c3b0722, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db0726, SUC_lexicon_tags_32 },
    { 0x7db0726, SUC_lexicon_tags_8 },
    { 0x341b0728, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fb072b, SUC_lexicon_tags_295 },
    { 0x24fb072c, SUC_lexicon_tags_11 },
    { 0xf17b072d, SUC_lexicon_tags_17 },
    { 0x2f7b072d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7b0734, SUC_lexicon_tags_1 },
    { 0xf6fb0735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7b073b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95b0740, SUC_lexicon_tags_7 },
    { 0x5edb0740, SUC_lexicon_tags_4 },
    { 0xf1fb0742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87b0744, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19db0752, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86db0754, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5f1b0756, SUC_lexicon_tags_7 },
    { 0x1ddb0756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9b0760, SUC_lexicon_tags_43 },
    { 0xa15b0761, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x569b0763, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x147b0766, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x321b0769, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bb076e, SUC_lexicon_tags_29 },
    { 0x59db076f, SUC_lexicon_tags_15 },
    { 0xa23b076e, SUC_lexicon_tags_8 },
    { 0x1c3b0771, SUC_lexicon_tags_37 },
    { 0x6d1b0770, SUC_lexicon_tags_14 },
    { 0xc0bb0773, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bb0779, SUC_lexicon_tags_37 },
    { 0x387b077a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55b0780, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfadb0782, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71b078a, SUC_lexicon_tags_10 },
    { 0x7c7b078a, SUC_lexicon_tags_7 },
    { 0xc67b078c, SUC_lexicon_tags_25 },
    { 0x319b078d, SUC_lexicon_tags_12 },
    { 0x7d9b078e, SUC_lexicon_tags_1 },
    { 0x535b078e, SUC_lexicon_tags_11 },
    { 0xdedb0790, SUC_lexicon_tags_17 },
    { 0xcc9b0790, SUC_lexicon_tags_11 },
    { 0x9a7b0792, SUC_lexicon_tags_42 },
    { 0x601b0793, SUC_lexicon_tags_15 },
    { 0x411b0794, SUC_lexicon_tags_37 },
    { 0x88db0794, SUC_lexicon_tags_37 },
    { 0xfa5b0796, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643b079b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab5b079d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59b07a0, SUC_lexicon_tags_15 },
    { 0xc7bb07a1, SUC_lexicon_tags_3 },
    { 0x6d5b07a2, SUC_lexicon_tags_14 },
    { 0xf61b07a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59b07a9, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97b07ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabb07b7, SUC_lexicon_tags_37 },
    { 0xe5fb07b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113b07c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93b07c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa5b07ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2db07cc, SUC_lexicon_tags_19 },
    { 0x73db07cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941b07d2, SUC_lexicon_tags_37 },
    { 0x199b07d3, SUC_lexicon_tags_1 },
    { 0x829b07d4, SUC_lexicon_tags_37 },
    { 0xf09b07d5, SUC_lexicon_tags_15 },
    { 0xc1fb07d6, SUC_lexicon_tags_6 },
    { 0x561b07d6, SUC_lexicon_tags_9 },
    { 0xc05b07d6, SUC_lexicon_tags_38 },
    { 0x1b07d9, SUC_lexicon_tags_37 },
    { 0x545b07da, SUC_lexicon_tags_17 },
    { 0xc01b07d2, SUC_lexicon_tags_19 },
    { 0x94fb07d7, SUC_lexicon_tags_14 },
    { 0x2f9b07da, SUC_lexicon_tags_12 },
    { 0x1a5b07de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x831b07e1, SUC_lexicon_tags_74 },
    { 0xa81b07e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27b07e5, SUC_lexicon_tags_25 },
    { 0x885b07e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa5b07e8, SUC_lexicon_tags_37 },
    { 0xb2fb07e9, SUC_lexicon_tags_17 },
    { 0x49db07ea, SUC_lexicon_tags_65 },
    { 0x5db07e9, SUC_lexicon_tags_38 },
    { 0x82fb07e9, SUC_lexicon_tags_262 },
    { 0xfa5b07ed, SUC_lexicon_tags_10 },
    { 0x711b07ee, SUC_lexicon_tags_35 },
    { 0x3bb07ee, SUC_lexicon_tags_9 },
    { 0xa9db07ef, SUC_lexicon_tags_2 },
    { 0xb1db07ee, SUC_lexicon_tags_1 },
    { 0x5e1b07f2, SUC_lexicon_tags_0 },
    { 0xe35b07f3, SUC_lexicon_tags_15 },
    { 0x2f3b07ed, SUC_lexicon_tags_49 },
    { 0xb2bb07e9, SUC_lexicon_tags_118 },
    { 0x61fb07f6, SUC_lexicon_tags_45 },
    { 0x179b07f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b07fe, SUC_lexicon_tags_12 },
    { 0xc2bb07fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5b0804, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb9db0806, SUC_lexicon_tags_12 },
    { 0x479b0806, SUC_lexicon_tags_59 },
    { 0xdebb0808, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdcbb080a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3b080f, SUC_lexicon_tags_1 },
    { 0x481b0810, SUC_lexicon_tags_26 },
    { 0xe63b0810, SUC_lexicon_tags_39 },
    { 0x9a5b0811, SUC_lexicon_tags_1 },
    { 0xabfb0810, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bb0819, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddb0820, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfb0823, SUC_lexicon_tags_23 },
    { 0xd0db0824, SUC_lexicon_tags_7 },
    { 0x379b0825, SUC_lexicon_tags_25 },
    { 0x331b0826, SUC_lexicon_tags_31 },
    { 0x1d3b0825, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb082a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadb082d, SUC_lexicon_tags_13 },
    { 0xfabb082e, SUC_lexicon_tags_2 },
    { 0xcbdb082f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45b0831, SUC_lexicon_tags_2 },
    { 0xa3fb0832, SUC_lexicon_tags_37 },
    { 0x76db0831, SUC_lexicon_tags_38 },
    { 0x76fb0833, SUC_lexicon_tags_66 },
    { 0xcf1b0835, SUC_lexicon_tags_29 },
    { 0x7ffb0835, SUC_lexicon_tags_46 },
    { 0x7bdb0837, SUC_lexicon_tags_18 },
    { 0x7fb0837, SUC_lexicon_tags_15 },
    { 0x67b0838, SUC_lexicon_tags_17 },
    { 0x3b0838, SUC_lexicon_tags_37 },
    { 0x201b083b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719b0846, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fb084b, SUC_lexicon_tags_1 },
    { 0xcabb084c, SUC_lexicon_tags_29 },
    { 0xd27b084d, SUC_lexicon_tags_45 },
    { 0x1d7b084d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x23bb0850, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdb0856, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x52fb0858, SUC_lexicon_tags_15 },
    { 0xd29b0858, SUC_lexicon_tags_0 },
    { 0x189b0859, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xed3b085c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fb085f, SUC_lexicon_tags_56 },
    { 0x427b0860, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebb0864, SUC_lexicon_tags_30 },
    { 0xd25b0865, SUC_lexicon_tags_12 },
    { 0x933b0865, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3b0869, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5db0874, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddb0877, SUC_lexicon_tags_17 },
    { 0xbabb0878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b087d, SUC_lexicon_tags_43 },
    { 0x4a1b087e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa63b0880, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x46bb0882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x493b0885, SUC_lexicon_tags_2 },
    { 0x315b0885, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x86db0889, SUC_lexicon_tags_15 },
    { 0xfcfb088a, SUC_lexicon_tags_6 },
    { 0x8ffb088b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331b088f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f9b0891, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c5b0893, SUC_lexicon_tags_30 },
    { 0xf39b0894, SUC_lexicon_tags_43 },
    { 0x6db0894, SUC_lexicon_tags_25 },
    { 0xaebb0896, SUC_lexicon_tags_62 },
    { 0x951b0897, SUC_lexicon_tags_15 },
    { 0x5a1b0896, SUC_lexicon_tags_9 },
    { 0xc85b0899, SUC_lexicon_tags_29 },
    { 0xac5b0897, SUC_lexicon_tags_7 },
    { 0xf51b0897, SUC_lexicon_tags_12 },
    { 0xb01b089c, SUC_lexicon_tags_6 },
    { 0x1adb089d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x355b08a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b08a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3b08ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67b08b2, SUC_lexicon_tags_61 },
    { 0xf47b08b2, SUC_lexicon_tags_69 },
    { 0x367b08b4, SUC_lexicon_tags_37 },
    { 0x93bb08b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbb08c1, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdb08c7, SUC_lexicon_tags_29 },
    { 0xbd5b08c8, SUC_lexicon_tags_30 },
    { 0x249b08c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9b08cc, SUC_lexicon_tags_37 },
    { 0xfc7b08cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdb08db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469b08df, SUC_lexicon_tags_129 },
    { 0x291b08df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bb08e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9b08e7, SUC_lexicon_tags_37 },
    { 0xec3b08e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe85b08ea, SUC_lexicon_tags_7 },
    { 0x963b08eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb08f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b08f4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x499b08f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bb08fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0db08fd, SUC_lexicon_tags_17 },
    { 0x563b08fe, SUC_lexicon_tags_17 },
    { 0x99db08ff, SUC_lexicon_tags_38 },
    { 0xd03b0900, SUC_lexicon_tags_1 },
    { 0xc6db0900, SUC_lexicon_tags_38 },
    { 0xc49b0900, SUC_lexicon_tags_27 },
    { 0xc8fb0903, SUC_lexicon_tags_37 },
    { 0x3fbb0901, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65b0907, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf93b0909, SUC_lexicon_tags_235 },
    { 0xefbb090a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xcd3b090c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3b0910, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x95bb0913, SUC_lexicon_tags_2 },
    { 0xbbdb0913, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1f9b0916, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e5b0918, SUC_lexicon_tags_66 },
    { 0x4bdb0918, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3b091d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x173b091f, SUC_lexicon_tags_2 },
    { 0x2e7b0920, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4e7b0922, SUC_lexicon_tags_37 },
    { 0x48db0923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b0927, SUC_lexicon_tags_87 },
    { 0x72bb0928, SUC_lexicon_tags_60 },
    { 0xe5db0929, SUC_lexicon_tags_30 },
    { 0x299b092a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7b092d, SUC_lexicon_tags_40 },
    { 0x911b092d, SUC_lexicon_tags_37 },
    { 0xa5fb092f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bb0934, SUC_lexicon_tags_12 },
    { 0x90db0935, SUC_lexicon_tags_23 },
    { 0xf05b0936, SUC_lexicon_tags_0 },
    { 0xa27b0934, SUC_lexicon_tags_37 },
    { 0x873b0935, SUC_lexicon_tags_1 },
    { 0xc39b0939, SUC_lexicon_tags_25 },
    { 0x34bb0939, SUC_lexicon_tags_37 },
    { 0x3a5b093a, SUC_lexicon_tags_17 },
    { 0xe3fb0935, SUC_lexicon_tags_63 },
    { 0xf7bb093b, SUC_lexicon_tags_1658 },
    { 0, NULL },
    { 0x3f7b093f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9c5b0941, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a7b0943, SUC_lexicon_tags_294 },
    { 0x6a7b0943, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x571b0947, SUC_lexicon_tags_37 },
    { 0xb81b0948, SUC_lexicon_tags_18 },
    { 0xe03b0948, SUC_lexicon_tags_11 },
    { 0xedbb0947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b094d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489b0953, SUC_lexicon_tags_1 },
    { 0x4e9b0954, SUC_lexicon_tags_66 },
    { 0x24fb0955, SUC_lexicon_tags_15 },
    { 0xac7b0955, SUC_lexicon_tags_27 },
    { 0xbe1b0957, SUC_lexicon_tags_33 },
    { 0x235b0955, SUC_lexicon_tags_7 },
    { 0xadb0959, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5b095c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x137b095e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97b0962, SUC_lexicon_tags_11 },
    { 0xa35b0962, SUC_lexicon_tags_15 },
    { 0xba3b0962, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7b096c, SUC_lexicon_tags_11 },
    { 0x1db096d, SUC_lexicon_tags_1 },
    { 0x5edb096e, SUC_lexicon_tags_27 },
    { 0xc4bb096f, SUC_lexicon_tags_106 },
    { 0x35bb0970, SUC_lexicon_tags_11 },
    { 0x57fb096e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccbb0973, SUC_lexicon_tags_39 },
    { 0xcf3b0973, SUC_lexicon_tags_14 },
    { 0x527b0975, SUC_lexicon_tags_33 },
    { 0x1fb0975, SUC_lexicon_tags_81 },
    { 0xe87b0977, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xcabb0979, SUC_lexicon_tags_39 },
    { 0x4dfb097a, SUC_lexicon_tags_1 },
    { 0xd31b097a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa2fb097d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03b0980, SUC_lexicon_tags_4 },
    { 0x409b0981, SUC_lexicon_tags_9 },
    { 0x81b0981, SUC_lexicon_tags_65 },
    { 0xebfb0983, SUC_lexicon_tags_43 },
    { 0x941b0984, SUC_lexicon_tags_15 },
    { 0xda5b0985, SUC_lexicon_tags_6 },
    { 0x7c9b0986, SUC_lexicon_tags_14 },
    { 0xc71b0986, SUC_lexicon_tags_15 },
    { 0xfcbb0988, SUC_lexicon_tags_0 },
    { 0xbcdb0989, SUC_lexicon_tags_33 },
    { 0x663b0986, SUC_lexicon_tags_11 },
    { 0x4e7b0982, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fb098e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x963b0991, SUC_lexicon_tags_17 },
    { 0xda1b0992, SUC_lexicon_tags_106 },
    { 0x4f3b0993, SUC_lexicon_tags_37 },
    { 0xb23b0994, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5b0997, SUC_lexicon_tags_37 },
    { 0x301b0997, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x84db099a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb17b099c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9b09a0, SUC_lexicon_tags_84 },
    { 0x79db09a1, SUC_lexicon_tags_1 },
    { 0xe9db09a0, SUC_lexicon_tags_15 },
    { 0xacdb09a3, SUC_lexicon_tags_43 },
    { 0xd59b09a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbb09a7, SUC_lexicon_tags_5 },
    { 0xa77b09a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76db09b4, SUC_lexicon_tags_48 },
    { 0x8d5b09b5, SUC_lexicon_tags_198 },
    { 0xc59b09b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb7b09b8, SUC_lexicon_tags_33 },
    { 0x699b09b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x327b09bb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbb09be, SUC_lexicon_tags_11 },
    { 0xe7fb09bf, SUC_lexicon_tags_7 },
    { 0x25fb09c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751b09c8, SUC_lexicon_tags_12 },
    { 0x29fb09c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb09ce, SUC_lexicon_tags_12 },
    { 0xad7b09ce, SUC_lexicon_tags_113 },
    { 0xd67b09d0, SUC_lexicon_tags_29 },
    { 0x909b09d0, SUC_lexicon_tags_23 },
    { 0x5b3b09d1, SUC_lexicon_tags_43 },
    { 0xa23b09d3, SUC_lexicon_tags_60 },
    { 0xaa1b09d4, SUC_lexicon_tags_60 },
    { 0x949b09d5, SUC_lexicon_tags_37 },
    { 0x65db09d5, SUC_lexicon_tags_32 },
    { 0x17bb09ce, SUC_lexicon_tags_11 },
    { 0xf71b09ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43db09da, SUC_lexicon_tags_23 },
    { 0x779b09db, SUC_lexicon_tags_712 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb09df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a3b09e1, SUC_lexicon_tags_10 },
    { 0xaf9b09e1, SUC_lexicon_tags_25 },
    { 0x63fb09e3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5b09e7, SUC_lexicon_tags_13 },
    { 0x9adb09e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5e1b09ea, SUC_lexicon_tags_17 },
    { 0xf67b09ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6d9b09ed, SUC_lexicon_tags_23 },
    { 0x859b09ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x255b09f0, SUC_lexicon_tags_47 },
    { 0x6cfb09f0, SUC_lexicon_tags_11 },
    { 0x78fb09f0, SUC_lexicon_tags_37 },
    { 0xf4bb09f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24bb09f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447b09fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9db0a03, SUC_lexicon_tags_6 },
    { 0x5f9b0a04, SUC_lexicon_tags_12 },
    { 0x53db0a04, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x811b0a07, SUC_lexicon_tags_2 },
    { 0x9edb0a07, SUC_lexicon_tags_47 },
    { 0xca5b0a09, SUC_lexicon_tags_60 },
    { 0xeb7b0a09, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x615b0a0c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x531b0a0e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7b0a14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bb0a19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x433b0a1b, SUC_lexicon_tags_45 },
    { 0x8a5b0a1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15b0a22, SUC_lexicon_tags_19 },
    { 0xc77b0a23, SUC_lexicon_tags_25 },
    { 0xf97b0a24, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bb0a28, SUC_lexicon_tags_37 },
    { 0x6d1b0a29, SUC_lexicon_tags_33 },
    { 0x2c5b0a2a, SUC_lexicon_tags_17 },
    { 0xabb0a2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03b0a30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1b0a36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6db0a39, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2db0a3e, SUC_lexicon_tags_2 },
    { 0x5c1b0a3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x99fb0a41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9b0a46, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347b0a4b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x53db0a4d, SUC_lexicon_tags_63 },
    { 0xbbbb0a4e, SUC_lexicon_tags_47 },
    { 0x177b0a4f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7db0a52, SUC_lexicon_tags_106 },
    { 0xde5b0a53, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x847b0a56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb0a61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adb0a6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x287b0a6e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x491b0a70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5b0a74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4db0a77, SUC_lexicon_tags_50 },
    { 0x867b0a77, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x64fb0a7a, SUC_lexicon_tags_17 },
    { 0x21b0a7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873b0a84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x567b0a87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33b0a8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f3b0a8e, SUC_lexicon_tags_6 },
    { 0x87db0a8f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fb0a95, SUC_lexicon_tags_174 },
    { 0x2e7b0a96, SUC_lexicon_tags_38 },
    { 0x37bb0a97, SUC_lexicon_tags_37 },
    { 0x889b0a98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc63b0a9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcadb0a9c, SUC_lexicon_tags_12 },
    { 0x2f7b0a9d, SUC_lexicon_tags_7 },
    { 0xd25b0a9e, SUC_lexicon_tags_15 },
    { 0x5bb0a9f, SUC_lexicon_tags_32 },
    { 0x257b0aa0, SUC_lexicon_tags_30 },
    { 0x3f3b0aa1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe37b0aa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09b0aa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435b0aac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88db0ab2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x697b0ab5, SUC_lexicon_tags_12 },
    { 0x5cbb0ab6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27b0ab8, SUC_lexicon_tags_37 },
    { 0xdc5b0ab8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b0abc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x525b0abf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787b0ac3, SUC_lexicon_tags_104 },
    { 0x12fb0ac4, SUC_lexicon_tags_25 },
    { 0xe43b0ac5, SUC_lexicon_tags_17 },
    { 0xcddb0ac5, SUC_lexicon_tags_37 },
    { 0x8dbb0ac6, SUC_lexicon_tags_5 },
    { 0x125b0ac4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3b0ad3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4cbb0ad5, SUC_lexicon_tags_47 },
    { 0xfafb0ad6, SUC_lexicon_tags_17 },
    { 0x91fb0ad5, SUC_lexicon_tags_26 },
    { 0x295b0ad8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e1b0ada, SUC_lexicon_tags_45 },
    { 0xff1b0adb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3bb0add, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3bb0adf, SUC_lexicon_tags_15 },
    { 0xdafb0adf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d3b0ae2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5b0ae7, SUC_lexicon_tags_61 },
    { 0x7a9b0ae8, SUC_lexicon_tags_2 },
    { 0xd5bb0ae9, SUC_lexicon_tags_66 },
    { 0x771b0aea, SUC_lexicon_tags_43 },
    { 0xe39b0ae7, SUC_lexicon_tags_37 },
    { 0xa33b0aec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdb0af1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x28db0af4, SUC_lexicon_tags_1 },
    { 0xb79b0af5, SUC_lexicon_tags_30 },
    { 0xc49b0af6, SUC_lexicon_tags_12 },
    { 0xacbb0af5, SUC_lexicon_tags_47 },
    { 0x59db0af8, SUC_lexicon_tags_17 },
    { 0xc8db0af9, SUC_lexicon_tags_11 },
    { 0x64fb0af9, SUC_lexicon_tags_66 },
    { 0xf93b0afb, SUC_lexicon_tags_63 },
    { 0x7f9b0afb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe79b0afe, SUC_lexicon_tags_546 },
    { 0x651b0aff, SUC_lexicon_tags_25 },
    { 0xdf3b0aff, SUC_lexicon_tags_25 },
    { 0x75bb0b01, SUC_lexicon_tags_3 },
    { 0xd33b0b01, SUC_lexicon_tags_47 },
    { 0xba1b0b03, SUC_lexicon_tags_43 },
    { 0x941b0b04, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b0b0d, SUC_lexicon_tags_16 },
    { 0x4d7b0b0e, SUC_lexicon_tags_60 },
    { 0x2dfb0b0f, SUC_lexicon_tags_37 },
    { 0x113b0b0e, SUC_lexicon_tags_27 },
    { 0x327b0b11, SUC_lexicon_tags_8 },
    { 0xd1db0b10, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd43b0b14, SUC_lexicon_tags_26 },
    { 0x53db0b15, SUC_lexicon_tags_1 },
    { 0x5bb0b16, SUC_lexicon_tags_17 },
    { 0x1b3b0b17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733b0b1b, SUC_lexicon_tags_1 },
    { 0x72fb0b1c, SUC_lexicon_tags_2 },
    { 0xd13b0b1c, SUC_lexicon_tags_49 },
    { 0x599b0b1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43db0b25, SUC_lexicon_tags_15 },
    { 0x86db0b26, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5b0b2b, SUC_lexicon_tags_26 },
    { 0xd59b0b2b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7b0b2f, SUC_lexicon_tags_25 },
    { 0xd89b0b30, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x6c9b0b32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5b0b36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xee7b0b38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc33b0b3a, SUC_lexicon_tags_11 },
    { 0xfc5b0b3a, SUC_lexicon_tags_65 },
    { 0x8fdb0b3a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5b0b40, SUC_lexicon_tags_17 },
    { 0x2a1b0b41, SUC_lexicon_tags_35 },
    { 0x381b0b42, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x29bb0b44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b5b0b46, SUC_lexicon_tags_49 },
    { 0xf0db0b47, SUC_lexicon_tags_12 },
    { 0xa05b0b46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x351b0b4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39db0b4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3b0b54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x211b0b56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4d3b0b58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313b0b64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7b0b67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7b0b6c, SUC_lexicon_tags_32 },
    { 0x257b0b6c, SUC_lexicon_tags_60 },
    { 0x3a7b0b6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a5b0b70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfb0b74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x759b0b76, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923b0b7c, SUC_lexicon_tags_82 },
    { 0xe3fb0b7c, SUC_lexicon_tags_49 },
    { 0x69fb0b7e, SUC_lexicon_tags_11 },
    { 0x235b0b7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257b0b85, SUC_lexicon_tags_63 },
    { 0x17fb0b85, SUC_lexicon_tags_37 },
    { 0x26db0b86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667b0b8d, SUC_lexicon_tags_12 },
    { 0x8db0b8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7b0ba2, SUC_lexicon_tags_108 },
    { 0x721b0ba2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91b0ba8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfb0bad, SUC_lexicon_tags_32 },
    { 0x8fb0bae, SUC_lexicon_tags_65 },
    { 0xe03b0baf, SUC_lexicon_tags_14 },
    { 0xd0bb0bb0, SUC_lexicon_tags_23 },
    { 0xd51b0bb0, SUC_lexicon_tags_33 },
    { 0x97fb0bb2, SUC_lexicon_tags_16 },
    { 0xe5bb0bb3, SUC_lexicon_tags_15 },
    { 0xab7b0bb4, SUC_lexicon_tags_8 },
    { 0x2b7b0bb4, SUC_lexicon_tags_23 },
    { 0x2d5b0bb3, SUC_lexicon_tags_13 },
    { 0xeb9b0bb7, SUC_lexicon_tags_25 },
    { 0xe2db0bae, SUC_lexicon_tags_55 },
    { 0x443b0bb5, SUC_lexicon_tags_70 },
    { 0x635b0bba, SUC_lexicon_tags_28 },
    { 0xfd1b0bb5, SUC_lexicon_tags_82 },
    { 0x9d1b0bbc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc67b0bbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x261b0bc1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x541b0bc4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121b0bcb, SUC_lexicon_tags_2 },
    { 0xc59b0bcc, SUC_lexicon_tags_15 },
    { 0xe7fb0bcc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xac7b0bcf, SUC_lexicon_tags_17 },
    { 0xc13b0bd0, SUC_lexicon_tags_84 },
    { 0xab9b0bd1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b0bd4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49b0bd8, SUC_lexicon_tags_17 },
    { 0x865b0bd9, SUC_lexicon_tags_17 },
    { 0x3f5b0bd9, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xe4fb0bdc, SUC_lexicon_tags_15 },
    { 0x5f5b0bdd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b0be0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65b0be4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1b0bed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdedb0bef, SUC_lexicon_tags_19 },
    { 0x9b7b0bf0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x777b0bf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12db0bf4, SUC_lexicon_tags_37 },
    { 0xe43b0bf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2db0bf7, SUC_lexicon_tags_32 },
    { 0x463b0bf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bdb0bfa, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc1fb0bfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb69b0bfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73b0c04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71b0c08, SUC_lexicon_tags_45 },
    { 0xe1b0c09, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9b0c0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x95fb0c0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x523b0c11, SUC_lexicon_tags_25 },
    { 0x9f9b0c12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05b0c17, SUC_lexicon_tags_29 },
    { 0x42bb0c18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611b0c1c, SUC_lexicon_tags_37 },
    { 0xa85b0c1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd3b0c1f, SUC_lexicon_tags_10 },
    { 0x29fb0c20, SUC_lexicon_tags_19 },
    { 0x6ffb0c1f, SUC_lexicon_tags_65 },
    { 0x13db0c22, SUC_lexicon_tags_8 },
    { 0xfb5b0c23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2fb0c25, SUC_lexicon_tags_29 },
    { 0x745b0c26, SUC_lexicon_tags_47 },
    { 0xae3b0c27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb0c29, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0xf75b0c2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225b0c2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfb0c34, SUC_lexicon_tags_23 },
    { 0x513b0c35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27b0c3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x22db0c3c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fb0c42, SUC_lexicon_tags_17 },
    { 0x769b0c42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x979b0c45, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85b0c4a, SUC_lexicon_tags_7 },
    { 0xb55b0c4b, SUC_lexicon_tags_23 },
    { 0xabbb0c4b, SUC_lexicon_tags_28 },
    { 0xf95b0c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fb0c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57b0c59, SUC_lexicon_tags_37 },
    { 0xe95b0c59, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe59b0c5c, SUC_lexicon_tags_39 },
    { 0xa05b0c5c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bb0c60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde3b0c62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf25b0c64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bb0c69, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddb0c6e, SUC_lexicon_tags_12 },
    { 0x805b0c6f, SUC_lexicon_tags_60 },
    { 0xb8fb0c6f, SUC_lexicon_tags_11 },
    { 0x2e5b0c71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb0c75, SUC_lexicon_tags_37 },
    { 0x815b0c76, SUC_lexicon_tags_15 },
    { 0x201b0c77, SUC_lexicon_tags_6 },
    { 0xd31b0c77, SUC_lexicon_tags_2 },
    { 0x159b0c76, SUC_lexicon_tags_48 },
    { 0xaabb0c7a, SUC_lexicon_tags_3 },
    { 0x573b0c7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bb0c83, SUC_lexicon_tags_74 },
    { 0xd85b0c84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x379b0c86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7b0c8a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebb0c92, SUC_lexicon_tags_19 },
    { 0xee7b0c93, SUC_lexicon_tags_23 },
    { 0xe55b0c94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547b0c9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fb0ca0, SUC_lexicon_tags_0 },
    { 0x8f5b0ca1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7adb0ca3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x379b0ca5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x863b0ca8, SUC_lexicon_tags_11 },
    { 0x80db0ca9, SUC_lexicon_tags_12 },
    { 0x9d3b0caa, SUC_lexicon_tags_12 },
    { 0x21b0caa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03b0cae, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6e3b0cb0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdb0cb5, SUC_lexicon_tags_38 },
    { 0x709b0cb5, SUC_lexicon_tags_11 },
    { 0xe15b0cb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea9b0cb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9cdb0cbb, SUC_lexicon_tags_37 },
    { 0xbbb0cbb, SUC_lexicon_tags_47 },
    { 0x889b0cbb, SUC_lexicon_tags_7 },
    { 0xb83b0cbb, SUC_lexicon_tags_25 },
    { 0x98db0cbe, SUC_lexicon_tags_15 },
    { 0xa9b0cc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x889b0cc2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x603b0cc4, SUC_lexicon_tags_25 },
    { 0xe77b0cc4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3db0ccb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fb0cce, SUC_lexicon_tags_17 },
    { 0x819b0ccf, SUC_lexicon_tags_23 },
    { 0xb5bb0cd0, SUC_lexicon_tags_37 },
    { 0x7b3b0cd1, SUC_lexicon_tags_48 },
    { 0xcfdb0cd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25b0cd5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b1b0cd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x351b0cd9, SUC_lexicon_tags_0 },
    { 0xa5db0cda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bb0cdd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddb0ce1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bb0ce4, SUC_lexicon_tags_38 },
    { 0x44bb0ce4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x40bb0ce7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fb0ceb, SUC_lexicon_tags_11 },
    { 0x8a9b0ceb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b0cf0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1b0cf4, SUC_lexicon_tags_21 },
    { 0x76fb0cf4, SUC_lexicon_tags_31 },
    { 0xc05b0cf6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55b0cf9, SUC_lexicon_tags_37 },
    { 0x9b5b0cfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b9b0cfc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfb0cff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfb0d05, SUC_lexicon_tags_47 },
    { 0x639b0d06, SUC_lexicon_tags_12 },
    { 0x403b0d07, SUC_lexicon_tags_11 },
    { 0x76bb0d06, SUC_lexicon_tags_82 },
    { 0x6a1b0d05, SUC_lexicon_tags_17 },
    { 0x807b0d06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa1bb0d0c, SUC_lexicon_tags_14 },
    { 0x56db0d0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fb0d15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15bb0d17, SUC_lexicon_tags_5 },
    { 0x7fdb0d18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x88fb0d1b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fb0d20, SUC_lexicon_tags_26 },
    { 0x4f1b0d20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7b0d27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72fb0d2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3b0d2e, SUC_lexicon_tags_26 },
    { 0xbdfb0d2e, SUC_lexicon_tags_117 },
    { 0x655b0d30, SUC_lexicon_tags_18 },
    { 0x99db0d31, SUC_lexicon_tags_26 },
    { 0x85b0d32, SUC_lexicon_tags_47 },
    { 0xa3b0d33, SUC_lexicon_tags_47 },
    { 0x5cfb0d34, SUC_lexicon_tags_25 },
    { 0xbb3b0d35, SUC_lexicon_tags_26 },
    { 0x4e7b0d31, SUC_lexicon_tags_13 },
    { 0xae3b0d31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebb0d3c, SUC_lexicon_tags_0 },
    { 0x3abb0d3d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3b0d40, SUC_lexicon_tags_9 },
    { 0x1bb0d41, SUC_lexicon_tags_527 },
    { 0x145b0d40, SUC_lexicon_tags_31 },
    { 0xb41b0d40, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fb0d47, SUC_lexicon_tags_37 },
    { 0xe0bb0d47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4abb0d4a, SUC_lexicon_tags_38 },
    { 0xa7db0d4b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3b0d4e, SUC_lexicon_tags_15 },
    { 0x86bb0d4f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6a1b0d51, SUC_lexicon_tags_534 },
    { 0xa01b0d51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7b0d57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d5b0d59, SUC_lexicon_tags_30 },
    { 0xc5b0d5a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x779b0d5c, SUC_lexicon_tags_46 },
    { 0x85db0d5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83b0d60, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9b0d63, SUC_lexicon_tags_27 },
    { 0xc6bb0d64, SUC_lexicon_tags_37 },
    { 0x4a1b0d65, SUC_lexicon_tags_29 },
    { 0x9e3b0d63, SUC_lexicon_tags_12 },
    { 0x5c1b0d65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9b0d6b, SUC_lexicon_tags_6 },
    { 0xf0fb0d6c, SUC_lexicon_tags_15 },
    { 0xe4bb0d6c, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bb0d73, SUC_lexicon_tags_17 },
    { 0x135b0d74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15bb0d76, SUC_lexicon_tags_15 },
    { 0x853b0d76, SUC_lexicon_tags_56 },
    { 0xe5b0d76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1b0d7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddb0d80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xae7b0d82, SUC_lexicon_tags_37 },
    { 0x73db0d83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x983b0d85, SUC_lexicon_tags_66 },
    { 0xe4fb0d86, SUC_lexicon_tags_43 },
    { 0x2fdb0d87, SUC_lexicon_tags_23 },
    { 0x23db0d88, SUC_lexicon_tags_0 },
    { 0xa6bb0d87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85b0d8b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcc7b0d8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbb0d95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82db0d9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5fb0d9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485b0da3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdb0dab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x875b0dae, SUC_lexicon_tags_31 },
    { 0x293b0dae, SUC_lexicon_tags_17 },
    { 0x415b0db0, SUC_lexicon_tags_37 },
    { 0xd69b0dae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xec3b0db3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653b0db7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x671b0db9, SUC_lexicon_tags_10 },
    { 0xbcbb0db9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949b0dbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3b0dc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa13b0dc4, SUC_lexicon_tags_11 },
    { 0xb91b0dc5, SUC_lexicon_tags_7 },
    { 0xa39b0dc4, SUC_lexicon_tags_19 },
    { 0xce3b0dc4, SUC_lexicon_tags_6 },
    { 0xf5db0dc7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1b0dcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305b0dd0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131b0dd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6abb0dd9, SUC_lexicon_tags_140 },
    { 0xd83b0dda, SUC_lexicon_tags_12 },
    { 0xe45b0ddb, SUC_lexicon_tags_8 },
    { 0xb01b0dd9, SUC_lexicon_tags_39 },
    { 0xb03b0dd9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bb0de0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1b0de3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17b0de6, SUC_lexicon_tags_7 },
    { 0x111b0de7, SUC_lexicon_tags_10 },
    { 0xc21b0de8, SUC_lexicon_tags_29 },
    { 0x19bb0de9, SUC_lexicon_tags_19 },
    { 0x22db0de9, SUC_lexicon_tags_7 },
    { 0x405b0de8, SUC_lexicon_tags_30 },
    { 0xdf3b0de8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfadb0dee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x899b0df1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa37b0df3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9b7b0df5, SUC_lexicon_tags_7 },
    { 0x963b0df5, SUC_lexicon_tags_21 },
    { 0x61fb0df7, SUC_lexicon_tags_17 },
    { 0x4edb0df8, SUC_lexicon_tags_15 },
    { 0xad9b0df7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09b0e00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89b0e04, SUC_lexicon_tags_15 },
    { 0x94fb0e05, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fb0e0f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x7b9b0e11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb0e16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb95b0e18, SUC_lexicon_tags_2 },
    { 0xecdb0e18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07b0e1c, SUC_lexicon_tags_11 },
    { 0x531b0e1d, SUC_lexicon_tags_25 },
    { 0x523b0e1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6ddb0e20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7b0e23, SUC_lexicon_tags_37 },
    { 0x821b0e24, SUC_lexicon_tags_43 },
    { 0x8bfb0e23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197b0e2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab3b0e2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bb0e30, SUC_lexicon_tags_15 },
    { 0x859b0e30, SUC_lexicon_tags_66 },
    { 0xf69b0e31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b0e3c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x84bb0e3e, SUC_lexicon_tags_32 },
    { 0x973b0e3f, SUC_lexicon_tags_19 },
    { 0x42fb0e3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d5b0e42, SUC_lexicon_tags_92 },
    { 0x39b0e43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fb0e49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c9b0e4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7b0e4f, SUC_lexicon_tags_2 },
    { 0x1cdb0e50, SUC_lexicon_tags_16 },
    { 0x971b0e50, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6db0e54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x979b0e57, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fb0e5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5b0e5e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x719b0e61, SUC_lexicon_tags_11 },
    { 0x445b0e61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x221b0e64, SUC_lexicon_tags_37 },
    { 0xff1b0e64, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x88bb0e67, SUC_lexicon_tags_1 },
    { 0x473b0e68, SUC_lexicon_tags_11 },
    { 0xbc5b0e68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddb0e6e, SUC_lexicon_tags_7 },
    { 0xe17b0e6f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99db0e74, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x893b0e76, SUC_lexicon_tags_59 },
    { 0x609b0e77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313b0e7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb0e7f, SUC_lexicon_tags_7 },
    { 0x25db0e80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x615b0e82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53db0e8d, SUC_lexicon_tags_0 },
    { 0xe73b0e8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb0e92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bb0e95, SUC_lexicon_tags_27 },
    { 0xa37b0e96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x395b0e98, SUC_lexicon_tags_9 },
    { 0x18bb0e98, SUC_lexicon_tags_32 },
    { 0xd43b0e9a, SUC_lexicon_tags_49 },
    { 0xc15b0e98, SUC_lexicon_tags_17 },
    { 0xb45b0e9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x493b0e9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681b0ea2, SUC_lexicon_tags_42 },
    { 0xaf7b0ea2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bb0ead, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xadfb0eaf, SUC_lexicon_tags_0 },
    { 0xebb0eaf, SUC_lexicon_tags_33 },
    { 0x463b0eb0, SUC_lexicon_tags_1 },
    { 0xcbfb0eb2, SUC_lexicon_tags_21 },
    { 0x7c1b0eb3, SUC_lexicon_tags_68 },
    { 0xb69b0eb4, SUC_lexicon_tags_7 },
    { 0x439b0eb5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4db0eb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565b0ebd, SUC_lexicon_tags_7 },
    { 0x8b5b0ebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bb0ec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31b0ec5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45b0ed2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaebb0ed4, SUC_lexicon_tags_8 },
    { 0x7ebb0ed5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659b0eda, SUC_lexicon_tags_18 },
    { 0xb2bb0edb, SUC_lexicon_tags_15 },
    { 0x3adb0edb, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fb0edf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x903b0ee1, SUC_lexicon_tags_27 },
    { 0xe25b0ee2, SUC_lexicon_tags_25 },
    { 0x3bbb0ee3, SUC_lexicon_tags_70 },
    { 0xb8db0ee4, SUC_lexicon_tags_37 },
    { 0x1fdb0ee5, SUC_lexicon_tags_15 },
    { 0x989b0ee5, SUC_lexicon_tags_25 },
    { 0x861b0ee7, SUC_lexicon_tags_49 },
    { 0xeddb0ee5, SUC_lexicon_tags_47 },
    { 0x95b0ee9, SUC_lexicon_tags_17 },
    { 0xe61b0eea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975b0eee, SUC_lexicon_tags_1 },
    { 0xfbdb0eee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bb0ef2, SUC_lexicon_tags_2 },
    { 0xf31b0ef3, SUC_lexicon_tags_48 },
    { 0xb07b0ef4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b0ef7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8a7b0ef9, SUC_lexicon_tags_7 },
    { 0x45b0efa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53db0eff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b0f04, SUC_lexicon_tags_25 },
    { 0x703b0f05, SUC_lexicon_tags_15 },
    { 0xf73b0f05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12fb0f08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3b0f0b, SUC_lexicon_tags_26 },
    { 0xf1fb0f0c, SUC_lexicon_tags_25 },
    { 0xba7b0f0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbebb0f0f, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xcf3b0f11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fb0f16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605b0f1a, SUC_lexicon_tags_1 },
    { 0x8bfb0f1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7e1b0f1d, SUC_lexicon_tags_64 },
    { 0x8cbb0f1e, SUC_lexicon_tags_35 },
    { 0x701b0f1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf77b0f21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf69b0f23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efb0f28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3b0f2c, SUC_lexicon_tags_12 },
    { 0x4e7b0f2c, SUC_lexicon_tags_25 },
    { 0x9cdb0f2c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a9b0f30, SUC_lexicon_tags_15 },
    { 0x9e5b0f31, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x313b0f34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fb0f37, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x66bb0f39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x21db0f3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3b0f40, SUC_lexicon_tags_8 },
    { 0x9e3b0f41, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x44db0f43, SUC_lexicon_tags_17 },
    { 0x7f1b0f44, SUC_lexicon_tags_19 },
    { 0xb8bb0f45, SUC_lexicon_tags_37 },
    { 0xa67b0f46, SUC_lexicon_tags_2 },
    { 0xda9b0f46, SUC_lexicon_tags_11 },
    { 0x569b0f48, SUC_lexicon_tags_7 },
    { 0x52bb0f45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fb0f4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5b0f55, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x521b0f58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdbfb0f5a, SUC_lexicon_tags_7 },
    { 0xd39b0f5b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb61b0f5d, SUC_lexicon_tags_25 },
    { 0xa31b0f5e, SUC_lexicon_tags_1 },
    { 0x63b0f5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509b0f68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819b0f6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b0f72, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xdadb0f74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147b0f7a, SUC_lexicon_tags_43 },
    { 0x677b0f7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80db0f81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bb0f85, SUC_lexicon_tags_25 },
    { 0x73b0f86, SUC_lexicon_tags_33 },
    { 0xee5b0f87, SUC_lexicon_tags_2 },
    { 0x76bb0f88, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebb0f8d, SUC_lexicon_tags_43 },
    { 0x51bb0f8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x499b0f90, SUC_lexicon_tags_48 },
    { 0x6a1b0f91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfb0f94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x16fb0f96, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abb0f9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbb0f9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b5b0fa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x815b0fa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31b0fa6, SUC_lexicon_tags_51 },
    { 0x363b0fa7, SUC_lexicon_tags_16 },
    { 0x7ddb0fa8, SUC_lexicon_tags_30 },
    { 0xb55b0fa6, SUC_lexicon_tags_38 },
    { 0xe7bb0faa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x749b0fac, SUC_lexicon_tags_11 },
    { 0xca7b0fad, SUC_lexicon_tags_33 },
    { 0x1c7b0fac, SUC_lexicon_tags_9 },
    { 0x23db0fae, SUC_lexicon_tags_25 },
    { 0x619b0fb0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x53fb0fb3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbbb0fb5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57bb0fb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92db0fba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfdb0fbc, SUC_lexicon_tags_25 },
    { 0xb4bb0fbd, SUC_lexicon_tags_32 },
    { 0xd0fb0fbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627b0fc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfb0fcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x577b0fcf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb0fd5, SUC_lexicon_tags_11 },
    { 0x1b5b0fd5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723b0fda, SUC_lexicon_tags_27 },
    { 0x885b0fdb, SUC_lexicon_tags_66 },
    { 0x761b0fdc, SUC_lexicon_tags_18 },
    { 0x7afb0fdd, SUC_lexicon_tags_4 },
    { 0x8e9b0fde, SUC_lexicon_tags_23 },
    { 0xd5db0fdf, SUC_lexicon_tags_12 },
    { 0xda3b0fdd, SUC_lexicon_tags_25 },
    { 0xdb3b0fe0, SUC_lexicon_tags_46 },
    { 0xd9db0fda, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbb0fe5, SUC_lexicon_tags_21 },
    { 0x7ddb0fe6, SUC_lexicon_tags_2 },
    { 0xecfb0fe6, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1b0fec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fb0fef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779b0ff3, SUC_lexicon_tags_17 },
    { 0x3e3b0ff3, SUC_lexicon_tags_7 },
    { 0x47db0ff3, SUC_lexicon_tags_35 },
    { 0xf93b0ff6, SUC_lexicon_tags_2 },
    { 0x707b0ff7, SUC_lexicon_tags_15 },
    { 0x729b0ff8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b0ffc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db1004, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251b1008, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7db100b, SUC_lexicon_tags_0 },
    { 0xc6bb100c, SUC_lexicon_tags_14 },
    { 0x4db100c, SUC_lexicon_tags_0 },
    { 0x7b5b100c, SUC_lexicon_tags_12 },
    { 0x579b100c, SUC_lexicon_tags_7 },
    { 0x1f9b1010, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567b1015, SUC_lexicon_tags_30 },
    { 0xaddb1016, SUC_lexicon_tags_37 },
    { 0x549b1015, SUC_lexicon_tags_12 },
    { 0x673b1018, SUC_lexicon_tags_31 },
    { 0xd21b1018, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdb101d, SUC_lexicon_tags_136 },
    { 0xe4bb101d, SUC_lexicon_tags_37 },
    { 0xc89b101f, SUC_lexicon_tags_14 },
    { 0x8c1b101e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfb1024, SUC_lexicon_tags_38 },
    { 0xe9bb1025, SUC_lexicon_tags_62 },
    { 0x5cdb1026, SUC_lexicon_tags_26 },
    { 0x41db1027, SUC_lexicon_tags_8 },
    { 0x7a1b1026, SUC_lexicon_tags_1 },
    { 0xabbb1028, SUC_lexicon_tags_17 },
    { 0xfd7b102a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5b102d, SUC_lexicon_tags_25 },
    { 0x1a7b102d, SUC_lexicon_tags_23 },
    { 0xdcdb102f, SUC_lexicon_tags_37 },
    { 0x2dfb102f, SUC_lexicon_tags_21 },
    { 0x8b7b1031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203b1040, SUC_lexicon_tags_84 },
    { 0xe3db1041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bb1044, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429b1048, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3bb104a, SUC_lexicon_tags_1 },
    { 0x8bb104b, SUC_lexicon_tags_48 },
    { 0xba1b104c, SUC_lexicon_tags_15 },
    { 0x35fb104a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbb1057, SUC_lexicon_tags_33 },
    { 0x199b1058, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x113b105b, SUC_lexicon_tags_34 },
    { 0x7a5b105b, SUC_lexicon_tags_12 },
    { 0x525b105c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835b1067, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x51fb106a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143b106e, SUC_lexicon_tags_11 },
    { 0x9cdb106e, SUC_lexicon_tags_57 },
    { 0xa49b1070, SUC_lexicon_tags_7 },
    { 0xf53b1071, SUC_lexicon_tags_27 },
    { 0xd5b1072, SUC_lexicon_tags_104 },
    { 0x309b1073, SUC_lexicon_tags_0 },
    { 0xa7db1070, SUC_lexicon_tags_37 },
    { 0x281b1075, SUC_lexicon_tags_5 },
    { 0x235b1076, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x449b1078, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfb107e, SUC_lexicon_tags_37 },
    { 0xf41b107e, SUC_lexicon_tags_11 },
    { 0xadfb107f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fb1084, SUC_lexicon_tags_12 },
    { 0x1f3b1085, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x13bb1087, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xe73b1089, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4bfb108b, SUC_lexicon_tags_23 },
    { 0xfb108c, SUC_lexicon_tags_17 },
    { 0x4b5b108c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947b1095, SUC_lexicon_tags_15 },
    { 0x2c3b1096, SUC_lexicon_tags_16 },
    { 0x695b1096, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb109b, SUC_lexicon_tags_21 },
    { 0x1b3b109c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x86bb109f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fb10a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3b10a8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473b10ac, SUC_lexicon_tags_37 },
    { 0x441b10ad, SUC_lexicon_tags_17 },
    { 0x859b10ae, SUC_lexicon_tags_7 },
    { 0xdefb10ae, SUC_lexicon_tags_6 },
    { 0x4fb10ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7b10ba, SUC_lexicon_tags_12 },
    { 0x2d9b10bb, SUC_lexicon_tags_11 },
    { 0x58bb10bb, SUC_lexicon_tags_15 },
    { 0x5b9b10ba, SUC_lexicon_tags_6 },
    { 0xd53b10bd, SUC_lexicon_tags_92 },
    { 0x747b10bf, SUC_lexicon_tags_30 },
    { 0x81db10c0, SUC_lexicon_tags_37 },
    { 0xacdb10c1, SUC_lexicon_tags_37 },
    { 0xea3b10c2, SUC_lexicon_tags_45 },
    { 0x391b10c0, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0xe5fb10c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bb10c8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7b10cc, SUC_lexicon_tags_55 },
    { 0xde3b10cd, SUC_lexicon_tags_25 },
    { 0x557b10ce, SUC_lexicon_tags_1 },
    { 0x8cbb10cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55b10d8, SUC_lexicon_tags_7 },
    { 0xd07b10d9, SUC_lexicon_tags_25 },
    { 0x44db10da, SUC_lexicon_tags_47 },
    { 0x5bb10da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb05b10dd, SUC_lexicon_tags_43 },
    { 0x2d3b10de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fb10e3, SUC_lexicon_tags_25 },
    { 0x24bb10e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x65fb10e6, SUC_lexicon_tags_7 },
    { 0x245b10e6, SUC_lexicon_tags_15 },
    { 0x2abb10e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb10eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01b10f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27db10f3, SUC_lexicon_tags_63 },
    { 0x863b10f4, SUC_lexicon_tags_17 },
    { 0x51db10f4, SUC_lexicon_tags_14 },
    { 0x563b10f6, SUC_lexicon_tags_1 },
    { 0xcf3b10f5, SUC_lexicon_tags_7 },
    { 0x887b10f8, SUC_lexicon_tags_25 },
    { 0xf2bb10f6, SUC_lexicon_tags_15 },
    { 0xebb10fa, SUC_lexicon_tags_39 },
    { 0xdb5b10f9, SUC_lexicon_tags_4 },
    { 0xe41b10fc, SUC_lexicon_tags_26 },
    { 0xb4db10fd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b1100, SUC_lexicon_tags_83 },
    { 0xe8fb1101, SUC_lexicon_tags_25 },
    { 0x867b1102, SUC_lexicon_tags_48 },
    { 0x305b1103, SUC_lexicon_tags_32 },
    { 0x405b1104, SUC_lexicon_tags_12 },
    { 0x457b1104, SUC_lexicon_tags_19 },
    { 0x581b1106, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x371b1108, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95b110b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x249b110e, SUC_lexicon_tags_33 },
    { 0x7c9b110e, SUC_lexicon_tags_12 },
    { 0xf37b1110, SUC_lexicon_tags_2 },
    { 0x39b1110, SUC_lexicon_tags_46 },
    { 0xfe5b1112, SUC_lexicon_tags_31 },
    { 0x43b1111, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bb1116, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x993b1118, SUC_lexicon_tags_23 },
    { 0xa9b1119, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83fb111b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479b111f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb1123, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xacfb1125, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551b112b, SUC_lexicon_tags_37 },
    { 0xa19b112c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x749b112e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5b1132, SUC_lexicon_tags_15 },
    { 0x8e1b1132, SUC_lexicon_tags_107 },
    { 0x7a1b1134, SUC_lexicon_tags_43 },
    { 0xca3b1134, SUC_lexicon_tags_63 },
    { 0x26bb1136, SUC_lexicon_tags_25 },
    { 0x74db1137, SUC_lexicon_tags_37 },
    { 0xc9b1137, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdb113b, SUC_lexicon_tags_53 },
    { 0x7bb113c, SUC_lexicon_tags_11 },
    { 0xcc3b113d, SUC_lexicon_tags_10 },
    { 0x9adb113d, SUC_lexicon_tags_12 },
    { 0xee1b113c, SUC_lexicon_tags_60 },
    { 0xdcfb1140, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7bb1142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11b1146, SUC_lexicon_tags_17 },
    { 0x72db1147, SUC_lexicon_tags_47 },
    { 0x71b1148, SUC_lexicon_tags_12 },
    { 0xa11b1149, SUC_lexicon_tags_48 },
    { 0xdc7b1147, SUC_lexicon_tags_28 },
    { 0x9f1b114a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75b114f, SUC_lexicon_tags_32 },
    { 0x263b1150, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867b1155, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x431b1158, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd43b115a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a3b115c, SUC_lexicon_tags_11 },
    { 0xf17b115d, SUC_lexicon_tags_37 },
    { 0xb49b115d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c5b1160, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1b1164, SUC_lexicon_tags_15 },
    { 0xa7db1164, SUC_lexicon_tags_39 },
    { 0x8abb1166, SUC_lexicon_tags_57 },
    { 0xcb1b1166, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ffb1169, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fb116f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983b1177, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275b117c, SUC_lexicon_tags_58 },
    { 0xb07b117d, SUC_lexicon_tags_9 },
    { 0xb51b117e, SUC_lexicon_tags_16 },
    { 0x133b117f, SUC_lexicon_tags_15 },
    { 0x5e3b117f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97b1183, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fb1187, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fb118a, SUC_lexicon_tags_15 },
    { 0x849b118b, SUC_lexicon_tags_14 },
    { 0x8cfb118c, SUC_lexicon_tags_48 },
    { 0x18fb118c, SUC_lexicon_tags_12 },
    { 0x5afb118e, SUC_lexicon_tags_87 },
    { 0xa3b118d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efb1194, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afb119a, SUC_lexicon_tags_9 },
    { 0x2cbb119b, SUC_lexicon_tags_8 },
    { 0x45bb119b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7db119e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5b11a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31b11a4, SUC_lexicon_tags_49 },
    { 0x2a7b11a5, SUC_lexicon_tags_14 },
    { 0x2f9b11a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fb11ab, SUC_lexicon_tags_7 },
    { 0x12fb11ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x989b11ae, SUC_lexicon_tags_38 },
    { 0x7a5b11ae, SUC_lexicon_tags_19 },
    { 0x779b11b0, SUC_lexicon_tags_17 },
    { 0xce1b11b1, SUC_lexicon_tags_19 },
    { 0x7f5b11b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647b11b6, SUC_lexicon_tags_11 },
    { 0xb5fb11b6, SUC_lexicon_tags_1 },
    { 0x59b11b8, SUC_lexicon_tags_15 },
    { 0x1ebb11b9, SUC_lexicon_tags_146 },
    { 0xf9fb11b8, SUC_lexicon_tags_26 },
    { 0xe97b11b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6db11c0, SUC_lexicon_tags_2 },
    { 0xe1bb11c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bb11c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fb11ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7b11cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x541b11ce, SUC_lexicon_tags_60 },
    { 0x9e3b11cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfdb11d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfb11d7, SUC_lexicon_tags_37 },
    { 0x92fb11d7, SUC_lexicon_tags_14 },
    { 0x63db11d9, SUC_lexicon_tags_129 },
    { 0xd2bb11da, SUC_lexicon_tags_17 },
    { 0x485b11db, SUC_lexicon_tags_25 },
    { 0x83db11dc, SUC_lexicon_tags_1 },
    { 0xa45b11d8, SUC_lexicon_tags_66 },
    { 0x1d5b11dc, SUC_lexicon_tags_1 },
    { 0xadfb11df, SUC_lexicon_tags_17 },
    { 0x9edb11e0, SUC_lexicon_tags_46 },
    { 0x84bb11df, SUC_lexicon_tags_66 },
    { 0x7fdb11e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4bb11e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9b11ee, SUC_lexicon_tags_2 },
    { 0x7d1b11ee, SUC_lexicon_tags_31 },
    { 0x2bdb11f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18db11f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6db11f6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf11b11f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915b11fc, SUC_lexicon_tags_82 },
    { 0x23fb11fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3b1203, SUC_lexicon_tags_12 },
    { 0x2dbb1204, SUC_lexicon_tags_9 },
    { 0x1a9b1205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb120b, SUC_lexicon_tags_22 },
    { 0x6a9b120c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22db1210, SUC_lexicon_tags_37 },
    { 0xbb1211, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x69db1214, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed9b1216, SUC_lexicon_tags_17 },
    { 0xae9b1217, SUC_lexicon_tags_65 },
    { 0xd5b1218, SUC_lexicon_tags_15 },
    { 0x2e5b1219, SUC_lexicon_tags_1588 },
    { 0, NULL },
    { 0xbdbb121b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33b1220, SUC_lexicon_tags_11 },
    { 0x5cbb1221, SUC_lexicon_tags_103 },
    { 0x9fb1221, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x799b1224, SUC_lexicon_tags_0 },
    { 0x31b1225, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1b122b, SUC_lexicon_tags_37 },
    { 0x9f9b122c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7b122f, SUC_lexicon_tags_47 },
    { 0x54db1230, SUC_lexicon_tags_23 },
    { 0x2bbb1231, SUC_lexicon_tags_25 },
    { 0x839b1232, SUC_lexicon_tags_4 },
    { 0x8c3b1233, SUC_lexicon_tags_16 },
    { 0xc1b1233, SUC_lexicon_tags_35 },
    { 0x889b1234, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5db1239, SUC_lexicon_tags_33 },
    { 0xa4db1239, SUC_lexicon_tags_17 },
    { 0x809b1239, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19b1243, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7b1248, SUC_lexicon_tags_12 },
    { 0x61bb1248, SUC_lexicon_tags_224 },
    { 0xea9b124a, SUC_lexicon_tags_27 },
    { 0x7b5b124a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x391b124d, SUC_lexicon_tags_3 },
    { 0x63db124d, SUC_lexicon_tags_11 },
    { 0xc61b124d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcbdb1251, SUC_lexicon_tags_23 },
    { 0xa93b1252, SUC_lexicon_tags_68 },
    { 0x743b1253, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1cdb1255, SUC_lexicon_tags_19 },
    { 0x861b1256, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfb125a, SUC_lexicon_tags_6 },
    { 0xe11b125b, SUC_lexicon_tags_17 },
    { 0x79fb125a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xeffb125e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xae1b1261, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab3b1263, SUC_lexicon_tags_37 },
    { 0x913b1264, SUC_lexicon_tags_23 },
    { 0x489b1265, SUC_lexicon_tags_19 },
    { 0x2fbb1264, SUC_lexicon_tags_25 },
    { 0xc0fb1265, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb126a, SUC_lexicon_tags_25 },
    { 0x8ddb126a, SUC_lexicon_tags_15 },
    { 0x7d1b126c, SUC_lexicon_tags_103 },
    { 0xd19b126c, SUC_lexicon_tags_1 },
    { 0x8c7b126c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe55b1270, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45b1277, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129b127d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b1282, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1b1287, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x459b128a, SUC_lexicon_tags_38 },
    { 0x183b128b, SUC_lexicon_tags_2 },
    { 0xc0db128b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdb1291, SUC_lexicon_tags_6 },
    { 0xecdb1292, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295b1296, SUC_lexicon_tags_112 },
    { 0xff9b1297, SUC_lexicon_tags_37 },
    { 0x661b1298, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b12a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdb12a8, SUC_lexicon_tags_12 },
    { 0xc1db12a9, SUC_lexicon_tags_65 },
    { 0x181b12aa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bb12b0, SUC_lexicon_tags_28 },
    { 0x45b12b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b12b7, SUC_lexicon_tags_19 },
    { 0xa01b12b8, SUC_lexicon_tags_2 },
    { 0xd1bb12b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425b12c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5bb12c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9b12c6, SUC_lexicon_tags_65 },
    { 0x85db12c7, SUC_lexicon_tags_43 },
    { 0x369b12c8, SUC_lexicon_tags_25 },
    { 0xf49b12c7, SUC_lexicon_tags_0 },
    { 0xae7b12c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5b12cd, SUC_lexicon_tags_37 },
    { 0xd59b12ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b3b12d0, SUC_lexicon_tags_2 },
    { 0x2abb12d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x397b12d4, SUC_lexicon_tags_55 },
    { 0x587b12d4, SUC_lexicon_tags_11 },
    { 0x5fbb12d6, SUC_lexicon_tags_2 },
    { 0xcdb12d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x335b12da, SUC_lexicon_tags_37 },
    { 0x5e7b12db, SUC_lexicon_tags_29 },
    { 0x64db12dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525b12e0, SUC_lexicon_tags_33 },
    { 0x617b12e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fb12e4, SUC_lexicon_tags_8 },
    { 0x2c3b12e5, SUC_lexicon_tags_23 },
    { 0xd65b12e5, SUC_lexicon_tags_17 },
    { 0x317b12e7, SUC_lexicon_tags_29 },
    { 0xf79b12e8, SUC_lexicon_tags_277 },
    { 0xda7b12e7, SUC_lexicon_tags_47 },
    { 0x929b12e6, SUC_lexicon_tags_32 },
    { 0xfe9b12e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58bb12ed, SUC_lexicon_tags_55 },
    { 0x557b12ed, SUC_lexicon_tags_17 },
    { 0xebbb12ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3b12f6, SUC_lexicon_tags_46 },
    { 0x5cdb12f6, SUC_lexicon_tags_2 },
    { 0xf1bb12f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175b12fe, SUC_lexicon_tags_7 },
    { 0xbcdb12ff, SUC_lexicon_tags_68 },
    { 0x505b1300, SUC_lexicon_tags_63 },
    { 0x66fb1301, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc3db1303, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x885b1305, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfb3b1307, SUC_lexicon_tags_25 },
    { 0xf2db1308, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd7fb130a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45b130f, SUC_lexicon_tags_15 },
    { 0x5ddb1310, SUC_lexicon_tags_26 },
    { 0xd41b130f, SUC_lexicon_tags_1 },
    { 0x93db1310, SUC_lexicon_tags_17 },
    { 0xaf3b1312, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabb1317, SUC_lexicon_tags_37 },
    { 0xb97b1318, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd91b131a, SUC_lexicon_tags_47 },
    { 0x89bb131b, SUC_lexicon_tags_11 },
    { 0x811b131b, SUC_lexicon_tags_10 },
    { 0xe7bb131d, SUC_lexicon_tags_25 },
    { 0x68fb131d, SUC_lexicon_tags_0 },
    { 0x56fb131b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x503b1321, SUC_lexicon_tags_17 },
    { 0x5f9b1322, SUC_lexicon_tags_27 },
    { 0xc17b1323, SUC_lexicon_tags_49 },
    { 0xf39b1323, SUC_lexicon_tags_49 },
    { 0x469b1323, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5b1328, SUC_lexicon_tags_25 },
    { 0xabb1329, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdb132c, SUC_lexicon_tags_7 },
    { 0x935b132d, SUC_lexicon_tags_37 },
    { 0x47bb132d, SUC_lexicon_tags_2 },
    { 0xc63b132f, SUC_lexicon_tags_17 },
    { 0xbddb132f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73b1334, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37b133c, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639b1341, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x321b1343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9b134b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49b1356, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa69b1358, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5b135e, SUC_lexicon_tags_65 },
    { 0x5b7b135f, SUC_lexicon_tags_7 },
    { 0x617b135f, SUC_lexicon_tags_41 },
    { 0x7f5b1360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363b1366, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbb136a, SUC_lexicon_tags_12 },
    { 0xf6fb136b, SUC_lexicon_tags_45 },
    { 0xcfb136c, SUC_lexicon_tags_46 },
    { 0xff7b136d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd27b136f, SUC_lexicon_tags_23 },
    { 0x49b1370, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1b1373, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf3b1375, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95b1379, SUC_lexicon_tags_2 },
    { 0xd0db137a, SUC_lexicon_tags_11 },
    { 0x35fb137b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3b137f, SUC_lexicon_tags_87 },
    { 0x4b1b1380, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x651b1383, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9db1386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491b138c, SUC_lexicon_tags_49 },
    { 0xd59b138c, SUC_lexicon_tags_11 },
    { 0x81bb138c, SUC_lexicon_tags_11 },
    { 0x951b138f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52db1395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37b139c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x449b139f, SUC_lexicon_tags_37 },
    { 0x9fbb13a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9b13a3, SUC_lexicon_tags_47 },
    { 0x23b13a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf07b13a6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fb13ac, SUC_lexicon_tags_124 },
    { 0xa0bb13ad, SUC_lexicon_tags_7 },
    { 0x9db13ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77b13b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7b13b4, SUC_lexicon_tags_21 },
    { 0x9d9b13b5, SUC_lexicon_tags_39 },
    { 0x121b13b6, SUC_lexicon_tags_38 },
    { 0xc5db13b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x483b13b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb13bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf61b13bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b7b13c1, SUC_lexicon_tags_30 },
    { 0x471b13c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fb13c5, SUC_lexicon_tags_37 },
    { 0x7c1b13c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x933b13c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3b13ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363b13d2, SUC_lexicon_tags_23 },
    { 0xc97b13d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115b13d7, SUC_lexicon_tags_1 },
    { 0xaa9b13d8, SUC_lexicon_tags_13 },
    { 0xb85b13d8, SUC_lexicon_tags_23 },
    { 0xfbdb13d9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5b13de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5b13e1, SUC_lexicon_tags_60 },
    { 0xff9b13e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fb13e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1bb13e7, SUC_lexicon_tags_37 },
    { 0x995b13e7, SUC_lexicon_tags_32 },
    { 0xf9b13e9, SUC_lexicon_tags_35 },
    { 0x35b13ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fb13ed, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x907b13ef, SUC_lexicon_tags_43 },
    { 0xb97b13f0, SUC_lexicon_tags_17 },
    { 0x295b13f0, SUC_lexicon_tags_10 },
    { 0xc49b13f1, SUC_lexicon_tags_43 },
    { 0x3d3b13f3, SUC_lexicon_tags_43 },
    { 0x671b13f4, SUC_lexicon_tags_1 },
    { 0x3b3b13f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09b13f8, SUC_lexicon_tags_17 },
    { 0x2bbb13f9, SUC_lexicon_tags_50 },
    { 0xe59b13fa, SUC_lexicon_tags_63 },
    { 0x55fb13fb, SUC_lexicon_tags_63 },
    { 0xc7db13fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7b13ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9b1407, SUC_lexicon_tags_37 },
    { 0xab3b1408, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfb140c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979b1417, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949b141e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf01b1420, SUC_lexicon_tags_47 },
    { 0x7f9b1421, SUC_lexicon_tags_15 },
    { 0x83b1422, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fb1425, SUC_lexicon_tags_25 },
    { 0x37fb1426, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb97b1428, SUC_lexicon_tags_23 },
    { 0x815b1429, SUC_lexicon_tags_11 },
    { 0x5db1429, SUC_lexicon_tags_3 },
    { 0x271b142b, SUC_lexicon_tags_25 },
    { 0x3c7b142c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4bb142e, SUC_lexicon_tags_1 },
    { 0xee9b142e, SUC_lexicon_tags_15 },
    { 0x615b142f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeebb1432, SUC_lexicon_tags_11 },
    { 0x963b1432, SUC_lexicon_tags_11 },
    { 0x579b1434, SUC_lexicon_tags_6 },
    { 0xa21b1432, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x919b1437, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xf6fb1439, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x523b143c, SUC_lexicon_tags_14 },
    { 0x5c7b143d, SUC_lexicon_tags_25 },
    { 0x5d1b143e, SUC_lexicon_tags_31 },
    { 0x83b143e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8b9b1441, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43db1443, SUC_lexicon_tags_4 },
    { 0x5ddb1444, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8cbb1446, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9b144a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x431b144c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabb1455, SUC_lexicon_tags_37 },
    { 0xa3bb1456, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x35db1459, SUC_lexicon_tags_15 },
    { 0xc05b145a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe31b145c, SUC_lexicon_tags_39 },
    { 0xc85b145d, SUC_lexicon_tags_28 },
    { 0xbe3b145d, SUC_lexicon_tags_12 },
    { 0x6f9b145e, SUC_lexicon_tags_11 },
    { 0x6f3b145f, SUC_lexicon_tags_4 },
    { 0x80fb1461, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x657b1464, SUC_lexicon_tags_37 },
    { 0x465b1464, SUC_lexicon_tags_25 },
    { 0x5afb1466, SUC_lexicon_tags_14 },
    { 0x63bb1466, SUC_lexicon_tags_17 },
    { 0x393b1466, SUC_lexicon_tags_37 },
    { 0x4f5b1467, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fb146d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7fdb146f, SUC_lexicon_tags_17 },
    { 0x76db1470, SUC_lexicon_tags_17 },
    { 0x40db1471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8edb1474, SUC_lexicon_tags_11 },
    { 0x6d1b1474, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9b147a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb0bb147c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x905b147e, SUC_lexicon_tags_38 },
    { 0xdf5b147f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefb1485, SUC_lexicon_tags_12 },
    { 0x467b1486, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x501b1488, SUC_lexicon_tags_9 },
    { 0x599b1488, SUC_lexicon_tags_60 },
    { 0x189b148a, SUC_lexicon_tags_7 },
    { 0x965b148b, SUC_lexicon_tags_14 },
    { 0xad5b1488, SUC_lexicon_tags_45 },
    { 0x285b148d, SUC_lexicon_tags_26 },
    { 0xcd3b1489, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835b1494, SUC_lexicon_tags_15 },
    { 0x157b1495, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133b1499, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443b149d, SUC_lexicon_tags_11 },
    { 0xb45b149e, SUC_lexicon_tags_79 },
    { 0x2c9b149f, SUC_lexicon_tags_104 },
    { 0xabb149e, SUC_lexicon_tags_2 },
    { 0x92db149f, SUC_lexicon_tags_4 },
    { 0xf91b14a2, SUC_lexicon_tags_5 },
    { 0x791b14a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb14a6, SUC_lexicon_tags_69 },
    { 0x779b14a6, SUC_lexicon_tags_27 },
    { 0xec3b14a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc93b14aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5b14ad, SUC_lexicon_tags_12 },
    { 0x8bdb14ae, SUC_lexicon_tags_0 },
    { 0x96fb14ae, SUC_lexicon_tags_7 },
    { 0xaafb14b0, SUC_lexicon_tags_2 },
    { 0xbbbb14b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3b14b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd37b14b6, SUC_lexicon_tags_85 },
    { 0x9e5b14b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc5b14b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db14be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39b14c9, SUC_lexicon_tags_82 },
    { 0x645b14ca, SUC_lexicon_tags_0 },
    { 0x521b14c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb14cf, SUC_lexicon_tags_30 },
    { 0x52bb14d0, SUC_lexicon_tags_43 },
    { 0x743b14d0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89b14d5, SUC_lexicon_tags_29 },
    { 0x503b14d5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa5db14d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7b14dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5b14eb, SUC_lexicon_tags_65 },
    { 0x601b14eb, SUC_lexicon_tags_25 },
    { 0x7afb14eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63b14f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdb14f5, SUC_lexicon_tags_57 },
    { 0xf61b14f6, SUC_lexicon_tags_63 },
    { 0x1ebb14f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771b14fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x219b14fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245b1502, SUC_lexicon_tags_65 },
    { 0x5f3b1503, SUC_lexicon_tags_31 },
    { 0xaf5b1504, SUC_lexicon_tags_47 },
    { 0xbd7b1505, SUC_lexicon_tags_27 },
    { 0xfdb1503, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11b1508, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1bbb150a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81b150d, SUC_lexicon_tags_38 },
    { 0x2b3b150e, SUC_lexicon_tags_15 },
    { 0xc85b150f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfb1516, SUC_lexicon_tags_40 },
    { 0x791b1517, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87b151b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8bb151d, SUC_lexicon_tags_26 },
    { 0x9f9b151e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05b152d, SUC_lexicon_tags_30 },
    { 0x5dfb152e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xedb1530, SUC_lexicon_tags_8 },
    { 0xde3b1531, SUC_lexicon_tags_30 },
    { 0x91db1530, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbb1537, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x793b153a, SUC_lexicon_tags_9 },
    { 0xdb5b153a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdb153f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e3b1541, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc3b1543, SUC_lexicon_tags_23 },
    { 0x705b1543, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275b1548, SUC_lexicon_tags_13 },
    { 0x111b1549, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bb154f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfb1555, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa3bb1557, SUC_lexicon_tags_9 },
    { 0xe9bb1557, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x95bb155b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x519b155d, SUC_lexicon_tags_17 },
    { 0x2d3b155d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbb1567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x709b156a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b156f, SUC_lexicon_tags_27 },
    { 0x3d3b156f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3b1572, SUC_lexicon_tags_27 },
    { 0x4c1b1573, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71b1577, SUC_lexicon_tags_60 },
    { 0x2e9b1578, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bdb157a, SUC_lexicon_tags_37 },
    { 0x1edb157b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x667b157e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5b1582, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7fb1584, SUC_lexicon_tags_6 },
    { 0x1fbb1585, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x251b1588, SUC_lexicon_tags_35 },
    { 0x4bdb1588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fb1599, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa77b159b, SUC_lexicon_tags_66 },
    { 0x9a7b159b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bb15a7, SUC_lexicon_tags_17 },
    { 0xd7b15a8, SUC_lexicon_tags_17 },
    { 0x8a5b15a7, SUC_lexicon_tags_23 },
    { 0x2b5b15a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x261b15ac, SUC_lexicon_tags_70 },
    { 0x14bb15ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93db15b1, SUC_lexicon_tags_17 },
    { 0x99db15b2, SUC_lexicon_tags_14 },
    { 0xbdfb15b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb15b9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913b15be, SUC_lexicon_tags_1 },
    { 0xa91b15bf, SUC_lexicon_tags_37 },
    { 0x8c1b15c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x47db15c2, SUC_lexicon_tags_37 },
    { 0xac3b15c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b15cf, SUC_lexicon_tags_45 },
    { 0xc2fb15cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fb15d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9b15dc, SUC_lexicon_tags_77 },
    { 0x721b15dc, SUC_lexicon_tags_43 },
    { 0xb01b15dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdb15e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1b15f0, SUC_lexicon_tags_27 },
    { 0xff7b15f1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbb15f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe3bb15f7, SUC_lexicon_tags_2 },
    { 0x57fb15f8, SUC_lexicon_tags_31 },
    { 0x261b15f9, SUC_lexicon_tags_178 },
    { 0xef3b15fa, SUC_lexicon_tags_49 },
    { 0x3b9b15fb, SUC_lexicon_tags_7 },
    { 0x9efb15fb, SUC_lexicon_tags_95 },
    { 0xe35b15fb, SUC_lexicon_tags_50 },
    { 0xae1b15f8, SUC_lexicon_tags_1 },
    { 0x927b15ff, SUC_lexicon_tags_9 },
    { 0x469b1600, SUC_lexicon_tags_25 },
    { 0x9cdb1601, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x94fb1603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05b1609, SUC_lexicon_tags_1 },
    { 0x30bb160a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3b1610, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fb1613, SUC_lexicon_tags_5 },
    { 0x6edb1614, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea7b1616, SUC_lexicon_tags_31 },
    { 0x6d9b1617, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x785b1619, SUC_lexicon_tags_12 },
    { 0xf23b1619, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbafb161c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xec7b161e, SUC_lexicon_tags_1 },
    { 0x5dbb161e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92fb1621, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfb1624, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761b1628, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe07b162a, SUC_lexicon_tags_1 },
    { 0x6f3b162b, SUC_lexicon_tags_36 },
    { 0xc09b162c, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x67bb162e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bb1632, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bb1635, SUC_lexicon_tags_2 },
    { 0x2ddb1636, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2db163a, SUC_lexicon_tags_42 },
    { 0x1bbb163b, SUC_lexicon_tags_12 },
    { 0x3b3b163b, SUC_lexicon_tags_7 },
    { 0xf51b163d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb1645, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x699b1647, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x169b164a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa5bb164c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc1b164e, SUC_lexicon_tags_39 },
    { 0x97fb164e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761b1653, SUC_lexicon_tags_23 },
    { 0x759b1654, SUC_lexicon_tags_7 },
    { 0x41fb1654, SUC_lexicon_tags_10 },
    { 0x3cfb1654, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb4db1658, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7b165e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73b1661, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62bb1663, SUC_lexicon_tags_47 },
    { 0xfa7b1664, SUC_lexicon_tags_2 },
    { 0x387b1664, SUC_lexicon_tags_116 },
    { 0xc27b1666, SUC_lexicon_tags_43 },
    { 0xcb9b1667, SUC_lexicon_tags_35 },
    { 0xeafb1664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x521b166b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b1670, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x489b1673, SUC_lexicon_tags_37 },
    { 0x881b1673, SUC_lexicon_tags_17 },
    { 0x2a5b1673, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15db167d, SUC_lexicon_tags_30 },
    { 0x2edb167e, SUC_lexicon_tags_45 },
    { 0xf3db167d, SUC_lexicon_tags_37 },
    { 0xba1b1680, SUC_lexicon_tags_31 },
    { 0x2e5b167f, SUC_lexicon_tags_15 },
    { 0x3fbb1682, SUC_lexicon_tags_12 },
    { 0x791b1683, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf43b1685, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf37b1687, SUC_lexicon_tags_15 },
    { 0x441b1687, SUC_lexicon_tags_26 },
    { 0x999b1687, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfb1691, SUC_lexicon_tags_29 },
    { 0xa81b1691, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155b1696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1b169d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bdb169f, SUC_lexicon_tags_11 },
    { 0xfabb16a0, SUC_lexicon_tags_17 },
    { 0xc05b16a1, SUC_lexicon_tags_37 },
    { 0x5bdb169f, SUC_lexicon_tags_15 },
    { 0x2e1b16a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3b16a6, SUC_lexicon_tags_27 },
    { 0x1f7b16a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8d7b16a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541b16b1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5b16b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee1b16b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bb16ba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6abb16bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeddb16be, SUC_lexicon_tags_23 },
    { 0x6adb16bf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36db16c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3b16c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25b16cb, SUC_lexicon_tags_14 },
    { 0x15bb16cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xabfb16cf, SUC_lexicon_tags_30 },
    { 0xabfb16cf, SUC_lexicon_tags_13 },
    { 0x531b16d1, SUC_lexicon_tags_12 },
    { 0xef9b16d2, SUC_lexicon_tags_2 },
    { 0xe89b16d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919b16d9, SUC_lexicon_tags_23 },
    { 0xac9b16da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x305b16dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13b16e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fb16e6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x715b16e8, SUC_lexicon_tags_23 },
    { 0x763b16e9, SUC_lexicon_tags_19 },
    { 0x59bb16e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb16ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x98db16ef, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x527b16f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3bb16f4, SUC_lexicon_tags_17 },
    { 0x921b16f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3b16fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaddb16fd, SUC_lexicon_tags_1 },
    { 0x18bb16fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8db1700, SUC_lexicon_tags_28 },
    { 0x575b1700, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98bb1703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7b1706, SUC_lexicon_tags_32 },
    { 0x57bb1707, SUC_lexicon_tags_11 },
    { 0x889b1707, SUC_lexicon_tags_2 },
    { 0xd03b1709, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf6db170b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7db1712, SUC_lexicon_tags_60 },
    { 0x749b1713, SUC_lexicon_tags_47 },
    { 0x227b1714, SUC_lexicon_tags_60 },
    { 0x881b1715, SUC_lexicon_tags_25 },
    { 0xea3b1716, SUC_lexicon_tags_19 },
    { 0xdedb1716, SUC_lexicon_tags_30 },
    { 0x7b7b1717, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fb171b, SUC_lexicon_tags_32 },
    { 0x309b171b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3b1720, SUC_lexicon_tags_84 },
    { 0xfb1b1721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x413b1724, SUC_lexicon_tags_1 },
    { 0x965b1725, SUC_lexicon_tags_26 },
    { 0x397b1726, SUC_lexicon_tags_37 },
    { 0x199b1726, SUC_lexicon_tags_25 },
    { 0x989b1728, SUC_lexicon_tags_11 },
    { 0x6f7b1729, SUC_lexicon_tags_15 },
    { 0xf57b172a, SUC_lexicon_tags_37 },
    { 0xdfdb1729, SUC_lexicon_tags_65 },
    { 0xfd3b172c, SUC_lexicon_tags_8 },
    { 0x93fb172b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdb1731, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97fb1739, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3b173c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b1741, SUC_lexicon_tags_12 },
    { 0x761b1742, SUC_lexicon_tags_19 },
    { 0xaf5b1741, SUC_lexicon_tags_17 },
    { 0xbbdb1744, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7b174a, SUC_lexicon_tags_2 },
    { 0xe15b174b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fb1751, SUC_lexicon_tags_17 },
    { 0xcfdb1752, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x819b1754, SUC_lexicon_tags_17 },
    { 0x74bb1755, SUC_lexicon_tags_0 },
    { 0xe2fb1756, SUC_lexicon_tags_17 },
    { 0xd6db1757, SUC_lexicon_tags_27 },
    { 0x12fb1758, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7b175c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47b1760, SUC_lexicon_tags_27 },
    { 0x61db1761, SUC_lexicon_tags_37 },
    { 0x9c7b1761, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46fb1764, SUC_lexicon_tags_65 },
    { 0xb9bb1765, SUC_lexicon_tags_38 },
    { 0xe41b1765, SUC_lexicon_tags_27 },
    { 0x2afb1767, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369b176b, SUC_lexicon_tags_39 },
    { 0x289b176c, SUC_lexicon_tags_0 },
    { 0xaf1b176d, SUC_lexicon_tags_103 },
    { 0xd37b176d, SUC_lexicon_tags_12 },
    { 0xb53b176d, SUC_lexicon_tags_87 },
    { 0xa7b1770, SUC_lexicon_tags_7 },
    { 0xa1bb1771, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03b1775, SUC_lexicon_tags_14 },
    { 0x633b1775, SUC_lexicon_tags_17 },
    { 0x341b1775, SUC_lexicon_tags_55 },
    { 0x33bb1778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64fb177a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85b1781, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x437b1783, SUC_lexicon_tags_10 },
    { 0xab7b1784, SUC_lexicon_tags_7 },
    { 0xcedb1784, SUC_lexicon_tags_34 },
    { 0xd47b1786, SUC_lexicon_tags_16 },
    { 0xddb1784, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x97db178a, SUC_lexicon_tags_61 },
    { 0x669b178a, SUC_lexicon_tags_7 },
    { 0xfbb178a, SUC_lexicon_tags_55 },
    { 0x5d3b178d, SUC_lexicon_tags_37 },
    { 0x17db178e, SUC_lexicon_tags_9 },
    { 0xe73b178b, SUC_lexicon_tags_45 },
    { 0x2c7b178f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91b1793, SUC_lexicon_tags_19 },
    { 0xfbdb1793, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9b1797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3b179e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bb17a1, SUC_lexicon_tags_14 },
    { 0x6abb17a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa17b17a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57b17a7, SUC_lexicon_tags_48 },
    { 0xccfb17a8, SUC_lexicon_tags_4 },
    { 0x36bb17a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43b17ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bb17b0, SUC_lexicon_tags_33 },
    { 0x5dfb17b1, SUC_lexicon_tags_23 },
    { 0x33bb17b2, SUC_lexicon_tags_0 },
    { 0xffbb17b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65db17b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x117b17bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7b17bf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9b17c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99b17cc, SUC_lexicon_tags_14 },
    { 0xaebb17cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe15b17cf, SUC_lexicon_tags_38 },
    { 0x98bb17d0, SUC_lexicon_tags_2 },
    { 0x505b17d1, SUC_lexicon_tags_3 },
    { 0x1d3b17d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7b17d6, SUC_lexicon_tags_15 },
    { 0x2e7b17d7, SUC_lexicon_tags_713 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b17dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7b17df, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bb17e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f9b17e4, SUC_lexicon_tags_29 },
    { 0xe3bb17e4, SUC_lexicon_tags_11 },
    { 0x23b17e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b17ec, SUC_lexicon_tags_19 },
    { 0x427b17ed, SUC_lexicon_tags_44 },
    { 0x18bb17ee, SUC_lexicon_tags_42 },
    { 0xa5db17ee, SUC_lexicon_tags_37 },
    { 0x389b17f0, SUC_lexicon_tags_7 },
    { 0xfe5b17f0, SUC_lexicon_tags_1 },
    { 0x6adb17f1, SUC_lexicon_tags_84 },
    { 0x7efb17ee, SUC_lexicon_tags_43 },
    { 0xc7bb17ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdbdb17f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01b17f9, SUC_lexicon_tags_31 },
    { 0xb5b17f9, SUC_lexicon_tags_6 },
    { 0x83fb17fb, SUC_lexicon_tags_38 },
    { 0x2db17fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1db17fe, SUC_lexicon_tags_12 },
    { 0xf8bb17ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1db1801, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x71fb1803, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x809b1806, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bb180b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x487b180d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bb1814, SUC_lexicon_tags_12 },
    { 0xf4db1814, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x35b1817, SUC_lexicon_tags_9 },
    { 0x9a1b1818, SUC_lexicon_tags_9 },
    { 0x2a3b1819, SUC_lexicon_tags_0 },
    { 0x1b3b1819, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1b1823, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x187b1825, SUC_lexicon_tags_37 },
    { 0xe5b1826, SUC_lexicon_tags_32 },
    { 0xa2bb1825, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91b182a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bb1830, SUC_lexicon_tags_43 },
    { 0xc7bb1830, SUC_lexicon_tags_12 },
    { 0x459b1831, SUC_lexicon_tags_37 },
    { 0xbc3b1831, SUC_lexicon_tags_12 },
    { 0x6d9b1834, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6db1838, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adb183f, SUC_lexicon_tags_32 },
    { 0x3c9b1840, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b5b1842, SUC_lexicon_tags_2 },
    { 0xeffb1843, SUC_lexicon_tags_65 },
    { 0x2d5b1844, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfabb1846, SUC_lexicon_tags_84 },
    { 0xe1b1847, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xefb1849, SUC_lexicon_tags_60 },
    { 0xd71b184a, SUC_lexicon_tags_61 },
    { 0x9f7b184b, SUC_lexicon_tags_25 },
    { 0x53fb184c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x743b184f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87db1851, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fb1855, SUC_lexicon_tags_43 },
    { 0x287b1856, SUC_lexicon_tags_7 },
    { 0xb15b1857, SUC_lexicon_tags_29 },
    { 0x9b1858, SUC_lexicon_tags_1 },
    { 0x679b1858, SUC_lexicon_tags_1 },
    { 0x121b1857, SUC_lexicon_tags_1 },
    { 0xd9b185b, SUC_lexicon_tags_47 },
    { 0x975b185b, SUC_lexicon_tags_114 },
    { 0xe5bb185b, SUC_lexicon_tags_37 },
    { 0xf7fb185e, SUC_lexicon_tags_1 },
    { 0xdb5b185e, SUC_lexicon_tags_12 },
    { 0xcedb1860, SUC_lexicon_tags_11 },
    { 0x55bb185d, SUC_lexicon_tags_17 },
    { 0x219b1859, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9fb1864, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563b1869, SUC_lexicon_tags_25 },
    { 0x18fb1869, SUC_lexicon_tags_29 },
    { 0xc85b186b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa39b186d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625b1873, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa25b1875, SUC_lexicon_tags_40 },
    { 0xac9b1876, SUC_lexicon_tags_21 },
    { 0x28bb1876, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3b187b, SUC_lexicon_tags_19 },
    { 0x13db187b, SUC_lexicon_tags_25 },
    { 0x7ebb187b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd1db187f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1b1882, SUC_lexicon_tags_1 },
    { 0x915b1883, SUC_lexicon_tags_11 },
    { 0xf7bb1882, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x183b1887, SUC_lexicon_tags_43 },
    { 0x86db1888, SUC_lexicon_tags_26 },
    { 0xbc9b1887, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5b188c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619b1891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09b1897, SUC_lexicon_tags_30 },
    { 0x2dfb1898, SUC_lexicon_tags_2 },
    { 0xfcbb1897, SUC_lexicon_tags_2 },
    { 0xd21b189a, SUC_lexicon_tags_37 },
    { 0xc1fb189b, SUC_lexicon_tags_25 },
    { 0x51b1898, SUC_lexicon_tags_2 },
    { 0xef9b189b, SUC_lexicon_tags_11 },
    { 0x637b189e, SUC_lexicon_tags_38 },
    { 0xf99b189e, SUC_lexicon_tags_39 },
    { 0x3fb18a0, SUC_lexicon_tags_26 },
    { 0xa11b18a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfc3b18a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d1b18a5, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xd63b18a7, SUC_lexicon_tags_11 },
    { 0x26fb18a7, SUC_lexicon_tags_27 },
    { 0xa21b18a9, SUC_lexicon_tags_23 },
    { 0x753b18a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x61db18ac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691b18b5, SUC_lexicon_tags_25 },
    { 0x51bb18b6, SUC_lexicon_tags_2 },
    { 0x3c1b18b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdefb18b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x483b18bb, SUC_lexicon_tags_63 },
    { 0x36db18bc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69b18c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x679b18c6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63b18cf, SUC_lexicon_tags_4 },
    { 0xbadb18cf, SUC_lexicon_tags_17 },
    { 0xf05b18d1, SUC_lexicon_tags_39 },
    { 0xfdfb18d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adb18d6, SUC_lexicon_tags_1 },
    { 0xbe5b18d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23b18d9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x82bb18db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5b18de, SUC_lexicon_tags_45 },
    { 0xb89b18de, SUC_lexicon_tags_12 },
    { 0x8b1b18e0, SUC_lexicon_tags_31 },
    { 0x135b18e1, SUC_lexicon_tags_31 },
    { 0x6edb18de, SUC_lexicon_tags_49 },
    { 0x565b18e3, SUC_lexicon_tags_37 },
    { 0x1a5b18e2, SUC_lexicon_tags_61 },
    { 0x4f1b18e5, SUC_lexicon_tags_17 },
    { 0x371b18e5, SUC_lexicon_tags_1 },
    { 0x8e5b18e6, SUC_lexicon_tags_15 },
    { 0xa09b18e8, SUC_lexicon_tags_15 },
    { 0xbd3b18e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x683b18ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9b18f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xebbb18f3, SUC_lexicon_tags_14 },
    { 0x489b18f4, SUC_lexicon_tags_37 },
    { 0x235b18f5, SUC_lexicon_tags_1 },
    { 0xe83b18f6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fb18fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x533b18fc, SUC_lexicon_tags_11 },
    { 0xbafb18fd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x11fb18ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x319b1902, SUC_lexicon_tags_35 },
    { 0x277b1903, SUC_lexicon_tags_12 },
    { 0x1f1b1904, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b1908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9b190c, SUC_lexicon_tags_29 },
    { 0x61b190c, SUC_lexicon_tags_64 },
    { 0xe3db190e, SUC_lexicon_tags_12 },
    { 0xa8bb190f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5b1912, SUC_lexicon_tags_12 },
    { 0x19bb1913, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x983b1916, SUC_lexicon_tags_15 },
    { 0xc0db1916, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7b191a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8edb191c, SUC_lexicon_tags_29 },
    { 0x2b3b191d, SUC_lexicon_tags_44 },
    { 0x68fb191e, SUC_lexicon_tags_11 },
    { 0xcbfb191f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fb1922, SUC_lexicon_tags_17 },
    { 0x9a1b1922, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4db192a, SUC_lexicon_tags_200 },
    { 0, NULL },
    { 0xe27b192c, SUC_lexicon_tags_1 },
    { 0xbdbb192d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7b1931, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x79db1934, SUC_lexicon_tags_21 },
    { 0x68db1935, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99b1937, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebb193c, SUC_lexicon_tags_6 },
    { 0xb4fb193c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x169b1940, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9b194a, SUC_lexicon_tags_43 },
    { 0xd51b194a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bb194f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe4fb1951, SUC_lexicon_tags_13 },
    { 0xbcfb1952, SUC_lexicon_tags_62 },
    { 0xcf5b1953, SUC_lexicon_tags_19 },
    { 0xc7b1954, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33db195a, SUC_lexicon_tags_37 },
    { 0xed7b195b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11b195e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x113b1960, SUC_lexicon_tags_6 },
    { 0xa9b1960, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1b1969, SUC_lexicon_tags_28 },
    { 0x657b196a, SUC_lexicon_tags_6 },
    { 0xd5bb196b, SUC_lexicon_tags_2 },
    { 0x4cdb196a, SUC_lexicon_tags_65 },
    { 0xe3db196d, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9b1972, SUC_lexicon_tags_12 },
    { 0xb4bb1973, SUC_lexicon_tags_25 },
    { 0x2abb1973, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95bb1976, SUC_lexicon_tags_151 },
    { 0x1efb1977, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63b197a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc7fb197c, SUC_lexicon_tags_26 },
    { 0xc19b197d, SUC_lexicon_tags_13 },
    { 0xe21b197e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37bb1981, SUC_lexicon_tags_55 },
    { 0x6cfb1982, SUC_lexicon_tags_17 },
    { 0x887b1981, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b1987, SUC_lexicon_tags_2 },
    { 0x439b1988, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bb198d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545b1992, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd1b1994, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31b1997, SUC_lexicon_tags_63 },
    { 0x35fb1998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99b19a0, SUC_lexicon_tags_82 },
    { 0x439b19a1, SUC_lexicon_tags_17 },
    { 0x409b19a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb19a9, SUC_lexicon_tags_25 },
    { 0xcefb19aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5b19ae, SUC_lexicon_tags_14 },
    { 0x217b19af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37db19b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb19bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fb19c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb19c6, SUC_lexicon_tags_11 },
    { 0xda3b19c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56db19c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9b19ce, SUC_lexicon_tags_25 },
    { 0xebfb19cf, SUC_lexicon_tags_14 },
    { 0x6cfb19cf, SUC_lexicon_tags_28 },
    { 0x1d3b19d0, SUC_lexicon_tags_60 },
    { 0xc9b19d2, SUC_lexicon_tags_11 },
    { 0x21b19d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fb19d6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x679b19d9, SUC_lexicon_tags_7 },
    { 0x277b19da, SUC_lexicon_tags_4 },
    { 0x2a9b19da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1b19e0, SUC_lexicon_tags_63 },
    { 0x37bb19e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5b19e4, SUC_lexicon_tags_19 },
    { 0x32bb19e4, SUC_lexicon_tags_5 },
    { 0xaf7b19e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fb19ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1b19ee, SUC_lexicon_tags_47 },
    { 0xb9bb19ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7b19f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x621b19f5, SUC_lexicon_tags_25 },
    { 0xa33b19f5, SUC_lexicon_tags_38 },
    { 0x523b19f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5fb19f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95b1a04, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x101b1a06, SUC_lexicon_tags_19 },
    { 0x37b1a07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5b1a0a, SUC_lexicon_tags_23 },
    { 0x87bb1a0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299b1a15, SUC_lexicon_tags_29 },
    { 0xb37b1a15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc5b1a18, SUC_lexicon_tags_12 },
    { 0xc8db1a18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfb1a20, SUC_lexicon_tags_25 },
    { 0x1e1b1a21, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xecbb1a23, SUC_lexicon_tags_12 },
    { 0x123b1a24, SUC_lexicon_tags_23 },
    { 0xdfdb1a23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x225b1a27, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc3fb1a29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5b1a2d, SUC_lexicon_tags_25 },
    { 0xf43b1a2e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5b1a3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x497b1a3c, SUC_lexicon_tags_31 },
    { 0x74fb1a3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x331b1a40, SUC_lexicon_tags_11 },
    { 0x71fb1a41, SUC_lexicon_tags_12 },
    { 0x59db1a42, SUC_lexicon_tags_14 },
    { 0xd6db1a40, SUC_lexicon_tags_28 },
    { 0xd31b1a44, SUC_lexicon_tags_4 },
    { 0xa4fb1a42, SUC_lexicon_tags_33 },
    { 0x597b1a46, SUC_lexicon_tags_84 },
    { 0xe01b1a47, SUC_lexicon_tags_114 },
    { 0xefdb1a40, SUC_lexicon_tags_14 },
    { 0xffbb1a49, SUC_lexicon_tags_19 },
    { 0xe97b1a4a, SUC_lexicon_tags_37 },
    { 0x4cfb1a46, SUC_lexicon_tags_17 },
    { 0xd0bb1a4c, SUC_lexicon_tags_29 },
    { 0x607b1a44, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43b1a53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4db1a55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf43b1a57, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35b1a5b, SUC_lexicon_tags_15 },
    { 0xc07b1a5c, SUC_lexicon_tags_15 },
    { 0x1e1b1a5d, SUC_lexicon_tags_63 },
    { 0x209b1a5b, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0xb81b1a60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddb1a63, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x19fb1a65, SUC_lexicon_tags_37 },
    { 0x11b1a66, SUC_lexicon_tags_57 },
    { 0x633b1a65, SUC_lexicon_tags_43 },
    { 0x76bb1a68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x337b1a6a, SUC_lexicon_tags_55 },
    { 0x5b1b1a6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3b1a6e, SUC_lexicon_tags_31 },
    { 0xf1db1a6f, SUC_lexicon_tags_27 },
    { 0x521b1a70, SUC_lexicon_tags_26 },
    { 0x6d7b1a71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cdb1a73, SUC_lexicon_tags_43 },
    { 0x265b1a73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35b1a79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945b1a7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x207b1a7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x759b1a82, SUC_lexicon_tags_11 },
    { 0x40bb1a83, SUC_lexicon_tags_32 },
    { 0x3e1b1a82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57b1a89, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53b1a8e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fb1a91, SUC_lexicon_tags_33 },
    { 0x507b1a92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1b1a96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdb1a9b, SUC_lexicon_tags_6 },
    { 0xadbb1a9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527b1aa0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb81b1aa2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89b1aa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561b1aaa, SUC_lexicon_tags_9 },
    { 0x995b1aab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8fb1aad, SUC_lexicon_tags_33 },
    { 0x53bb1aae, SUC_lexicon_tags_42 },
    { 0xe47b1aaf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6d3b1ab1, SUC_lexicon_tags_7 },
    { 0x6d1b1ab2, SUC_lexicon_tags_12 },
    { 0x937b1ab3, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09b1ab6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7b1aba, SUC_lexicon_tags_84 },
    { 0x221b1abb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07b1ac1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431b1acb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1b1ace, SUC_lexicon_tags_1 },
    { 0x601b1acf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x151b1ad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fb1ad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x367b1ad7, SUC_lexicon_tags_15 },
    { 0x813b1ad8, SUC_lexicon_tags_39 },
    { 0xd1bb1ad9, SUC_lexicon_tags_12 },
    { 0x9a7b1ad9, SUC_lexicon_tags_31 },
    { 0x885b1ad8, SUC_lexicon_tags_17 },
    { 0x789b1ada, SUC_lexicon_tags_30 },
    { 0x293b1adc, SUC_lexicon_tags_271 },
    { 0xb43b1ade, SUC_lexicon_tags_38 },
    { 0xfafb1ade, SUC_lexicon_tags_17 },
    { 0xc4db1ad9, SUC_lexicon_tags_27 },
    { 0x9d5b1ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x497b1ae3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x39b1ae5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe57b1ae7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59b1af5, SUC_lexicon_tags_2 },
    { 0x769b1af6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25b1afa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1b1b00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x259b1b03, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9b1b09, SUC_lexicon_tags_80 },
    { 0x1abb1b0a, SUC_lexicon_tags_60 },
    { 0x517b1b0a, SUC_lexicon_tags_15 },
    { 0xe7b1b0b, SUC_lexicon_tags_7 },
    { 0x485b1b0d, SUC_lexicon_tags_39 },
    { 0x7efb1b0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1db1b10, SUC_lexicon_tags_12 },
    { 0x145b1b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb1b14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdb1b1c, SUC_lexicon_tags_11 },
    { 0x1cdb1b1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9b1b22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebb1b2a, SUC_lexicon_tags_92 },
    { 0x1c1b1b2a, SUC_lexicon_tags_1552 },
    { 0, NULL },
    { 0xa0bb1b2d, SUC_lexicon_tags_35 },
    { 0x9f5b1b2e, SUC_lexicon_tags_87 },
    { 0xccdb1b2e, SUC_lexicon_tags_28 },
    { 0x8a1b1b2f, SUC_lexicon_tags_2 },
    { 0xb5bb1b2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x751b1b34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x735b1b37, SUC_lexicon_tags_27 },
    { 0x239b1b37, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5b1b3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcedb1b3d, SUC_lexicon_tags_45 },
    { 0xc8db1b3e, SUC_lexicon_tags_17 },
    { 0x887b1b3f, SUC_lexicon_tags_37 },
    { 0x173b1b3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x509b1b42, SUC_lexicon_tags_35 },
    { 0xa63b1b43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bb1b47, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x353b1b4a, SUC_lexicon_tags_25 },
    { 0x583b1b4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x477b1b4d, SUC_lexicon_tags_7 },
    { 0xf8db1b4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1b1b51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6db1b56, SUC_lexicon_tags_17 },
    { 0x70fb1b57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd49b1b59, SUC_lexicon_tags_37 },
    { 0x849b1b5a, SUC_lexicon_tags_7 },
    { 0x4f9b1b59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedb1b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5b1b66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57b1b6a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545b1b70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf9bb1b72, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x48db1b75, SUC_lexicon_tags_26 },
    { 0x675b1b75, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3b1b7a, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9b1b7d, SUC_lexicon_tags_17 },
    { 0xb11b1b7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabb1b86, SUC_lexicon_tags_30 },
    { 0xdc7b1b87, SUC_lexicon_tags_38 },
    { 0x4edb1b86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7b1b8b, SUC_lexicon_tags_61 },
    { 0xce5b1b8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1fb1b8e, SUC_lexicon_tags_29 },
    { 0xf1bb1b8f, SUC_lexicon_tags_8 },
    { 0xdafb1b90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941b1b96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd93b1b98, SUC_lexicon_tags_7 },
    { 0xdcdb1b99, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x181b1b9c, SUC_lexicon_tags_17 },
    { 0x7b7b1b9c, SUC_lexicon_tags_63 },
    { 0xe05b1b9d, SUC_lexicon_tags_31 },
    { 0x4e7b1b9d, SUC_lexicon_tags_11 },
    { 0x1c9b1ba0, SUC_lexicon_tags_39 },
    { 0x403b1ba1, SUC_lexicon_tags_2 },
    { 0xee3b1ba0, SUC_lexicon_tags_11 },
    { 0x40db1ba3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9b1ba9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x175b1bab, SUC_lexicon_tags_15 },
    { 0x9a3b1bac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x405b1bae, SUC_lexicon_tags_15 },
    { 0x4adb1baf, SUC_lexicon_tags_7 },
    { 0x7dfb1baf, SUC_lexicon_tags_7 },
    { 0x8cfb1baf, SUC_lexicon_tags_60 },
    { 0xa27b1bb2, SUC_lexicon_tags_68 },
    { 0xd95b1bb1, SUC_lexicon_tags_27 },
    { 0x1c1b1bb4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x477b1bb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95fb1bb8, SUC_lexicon_tags_11 },
    { 0x3f1b1bb8, SUC_lexicon_tags_37 },
    { 0xa4db1bb9, SUC_lexicon_tags_23 },
    { 0xc11b1bb9, SUC_lexicon_tags_10 },
    { 0x57bb1bbc, SUC_lexicon_tags_29 },
    { 0xe4fb1bba, SUC_lexicon_tags_11 },
    { 0x9c7b1bbd, SUC_lexicon_tags_0 },
    { 0xcefb1bbc, SUC_lexicon_tags_43 },
    { 0xd5fb1bc0, SUC_lexicon_tags_38 },
    { 0xf3b1bc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3b1bc7, SUC_lexicon_tags_39 },
    { 0x6ffb1bc8, SUC_lexicon_tags_7 },
    { 0xbd5b1bc8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3b1bcf, SUC_lexicon_tags_14 },
    { 0xa85b1bd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x285b1bd2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe3bb1bd4, SUC_lexicon_tags_28 },
    { 0x1ebb1bd4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf73b1bd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54db1bdc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57db1bde, SUC_lexicon_tags_11 },
    { 0xd1fb1bde, SUC_lexicon_tags_23 },
    { 0x8d9b1bdf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2a9b1be2, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7b1be6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf23b1be8, SUC_lexicon_tags_25 },
    { 0xa3b1be8, SUC_lexicon_tags_31 },
    { 0xa03b1bea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x93db1bed, SUC_lexicon_tags_1 },
    { 0xb17b1bee, SUC_lexicon_tags_23 },
    { 0x291b1bef, SUC_lexicon_tags_43 },
    { 0x4bdb1bf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25b1bf5, SUC_lexicon_tags_15 },
    { 0x3c5b1bf5, SUC_lexicon_tags_182 },
    { 0x22db1bf6, SUC_lexicon_tags_25 },
    { 0x101b1bf8, SUC_lexicon_tags_15 },
    { 0xa13b1bf8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x923b1bfc, SUC_lexicon_tags_49 },
    { 0xc4bb1bfc, SUC_lexicon_tags_53 },
    { 0x91b1bfe, SUC_lexicon_tags_11 },
    { 0xe55b1bff, SUC_lexicon_tags_27 },
    { 0xdb5b1bfc, SUC_lexicon_tags_2 },
    { 0xa25b1bfd, SUC_lexicon_tags_7 },
    { 0x597b1c02, SUC_lexicon_tags_25 },
    { 0xac5b1c03, SUC_lexicon_tags_12 },
    { 0xee9b1c04, SUC_lexicon_tags_11 },
    { 0x927b1c00, SUC_lexicon_tags_1 },
    { 0x723b1c04, SUC_lexicon_tags_25 },
    { 0xcfb1c00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769b1c0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x461b1c0f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x713b1c11, SUC_lexicon_tags_15 },
    { 0x3ebb1c12, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x4afb1c14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41b1c18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x49fb1c1a, SUC_lexicon_tags_31 },
    { 0x52db1c1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85b1c23, SUC_lexicon_tags_62 },
    { 0x421b1c24, SUC_lexicon_tags_32 },
    { 0xea7b1c23, SUC_lexicon_tags_37 },
    { 0x6fbb1c26, SUC_lexicon_tags_1 },
    { 0x4a1b1c27, SUC_lexicon_tags_0 },
    { 0xcedb1c24, SUC_lexicon_tags_13 },
    { 0x5c1b1c29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafb1c34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a1b1c36, SUC_lexicon_tags_1 },
    { 0x5edb1c37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c9b1c39, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35b1c3d, SUC_lexicon_tags_15 },
    { 0xb01b1c3d, SUC_lexicon_tags_7 },
    { 0x51bb1c3f, SUC_lexicon_tags_11 },
    { 0x993b1c40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x901b1c42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x997b1c45, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdd3b1c47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x10fb1c49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bb1c4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x793b1c4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1b1c53, SUC_lexicon_tags_17 },
    { 0x947b1c54, SUC_lexicon_tags_15 },
    { 0xd77b1c54, SUC_lexicon_tags_37 },
    { 0x965b1c54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99b1c5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1b1c64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b1c6b, SUC_lexicon_tags_1 },
    { 0x837b1c6c, SUC_lexicon_tags_14 },
    { 0x759b1c6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25b1c73, SUC_lexicon_tags_1 },
    { 0x64db1c74, SUC_lexicon_tags_47 },
    { 0x6f1b1c75, SUC_lexicon_tags_16 },
    { 0x945b1c76, SUC_lexicon_tags_15 },
    { 0xf63b1c77, SUC_lexicon_tags_26 },
    { 0x7cbb1c77, SUC_lexicon_tags_29 },
    { 0x9f3b1c78, SUC_lexicon_tags_46 },
    { 0xbadb1c75, SUC_lexicon_tags_22 },
    { 0x49bb1c78, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bb1c82, SUC_lexicon_tags_1 },
    { 0xc9b1c83, SUC_lexicon_tags_14 },
    { 0x801b1c83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1b1c8b, SUC_lexicon_tags_12 },
    { 0xc71b1c8c, SUC_lexicon_tags_37 },
    { 0xde5b1c8d, SUC_lexicon_tags_30 },
    { 0xbfb1c8e, SUC_lexicon_tags_25 },
    { 0xeebb1c8d, SUC_lexicon_tags_66 },
    { 0x511b1c90, SUC_lexicon_tags_6 },
    { 0xdd5b1c91, SUC_lexicon_tags_42 },
    { 0x399b1c8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bb1c9d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5b1ca0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcc5b1ca2, SUC_lexicon_tags_1 },
    { 0xc6db1ca3, SUC_lexicon_tags_49 },
    { 0xfafb1ca3, SUC_lexicon_tags_30 },
    { 0x95fb1ca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x52bb1ca7, SUC_lexicon_tags_17 },
    { 0xd3b1ca7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ffb1caa, SUC_lexicon_tags_49 },
    { 0x103b1caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc9b1cad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fb1cb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303b1cb4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799b1cba, SUC_lexicon_tags_10 },
    { 0xa03b1cbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1b1cc1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x13bb1cc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8abb1cc5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x311b1cc8, SUC_lexicon_tags_15 },
    { 0x9c9b1cc9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x619b1ccb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdb1cd0, SUC_lexicon_tags_15 },
    { 0xcbdb1cd1, SUC_lexicon_tags_7 },
    { 0x8a9b1cd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x975b1cd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447b1cd8, SUC_lexicon_tags_11 },
    { 0x785b1cd9, SUC_lexicon_tags_35 },
    { 0xca3b1cda, SUC_lexicon_tags_12 },
    { 0xd3db1cd8, SUC_lexicon_tags_32 },
    { 0x391b1cdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x953b1cde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1b1ce2, SUC_lexicon_tags_22 },
    { 0xafb1ce2, SUC_lexicon_tags_49 },
    { 0x1b9b1ce4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52db1ce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1b1ce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fb1cec, SUC_lexicon_tags_25 },
    { 0x271b1ced, SUC_lexicon_tags_15 },
    { 0xc49b1cee, SUC_lexicon_tags_87 },
    { 0x7adb1cef, SUC_lexicon_tags_27 },
    { 0xebfb1cef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bb1cf3, SUC_lexicon_tags_19 },
    { 0x83b1cf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfb1cfe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105b1d05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73db1d07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb1d0c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83fb1d0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c9b1d10, SUC_lexicon_tags_49 },
    { 0x665b1d11, SUC_lexicon_tags_66 },
    { 0xa7fb1d11, SUC_lexicon_tags_0 },
    { 0x5fdb1d13, SUC_lexicon_tags_87 },
    { 0x125b1d13, SUC_lexicon_tags_15 },
    { 0xd81b1d14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b1d1a, SUC_lexicon_tags_28 },
    { 0x93fb1d1b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x73bb1d1d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1b1d23, SUC_lexicon_tags_17 },
    { 0x1d1b1d23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbadb1d26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xadfb1d29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x515b1d2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x831b1d2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57b1d37, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09b1d3a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbedb1d3c, SUC_lexicon_tags_151 },
    { 0x401b1d3d, SUC_lexicon_tags_37 },
    { 0xc99b1d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a5b1d40, SUC_lexicon_tags_27 },
    { 0x807b1d41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5d7b1d43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb1d4a, SUC_lexicon_tags_9 },
    { 0x39bb1d4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fb1d4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3b1d53, SUC_lexicon_tags_225 },
    { 0x4e7b1d54, SUC_lexicon_tags_25 },
    { 0xb51b1d54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835b1d59, SUC_lexicon_tags_112 },
    { 0x2e1b1d5a, SUC_lexicon_tags_25 },
    { 0x29db1d5b, SUC_lexicon_tags_15 },
    { 0x559b1d5c, SUC_lexicon_tags_7 },
    { 0x7d1b1d5d, SUC_lexicon_tags_29 },
    { 0xf67b1d5b, SUC_lexicon_tags_56 },
    { 0xeafb1d5f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7db1d62, SUC_lexicon_tags_0 },
    { 0x5c9b1d63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9b1d69, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe7fb1d6b, SUC_lexicon_tags_12 },
    { 0xbd5b1d6b, SUC_lexicon_tags_38 },
    { 0xb5bb1d6c, SUC_lexicon_tags_2 },
    { 0x5c3b1d6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7cfb1d70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb1d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x327b1d76, SUC_lexicon_tags_47 },
    { 0x26fb1d77, SUC_lexicon_tags_7 },
    { 0x4bbb1d78, SUC_lexicon_tags_6 },
    { 0x3dbb1d79, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbbb1d7b, SUC_lexicon_tags_47 },
    { 0x1afb1d7b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821b1d84, SUC_lexicon_tags_15 },
    { 0x469b1d85, SUC_lexicon_tags_15 },
    { 0xe45b1d85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7d3b1d88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb5b1d8a, SUC_lexicon_tags_25 },
    { 0xfd9b1d8b, SUC_lexicon_tags_46 },
    { 0x163b1d8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459b1d95, SUC_lexicon_tags_23 },
    { 0x34fb1d95, SUC_lexicon_tags_55 },
    { 0x2b9b1d97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743b1d9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fb1d9f, SUC_lexicon_tags_11 },
    { 0x1dbb1da0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb37b1da2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa2fb1da4, SUC_lexicon_tags_7 },
    { 0x445b1da4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249b1daa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcdb1dac, SUC_lexicon_tags_17 },
    { 0xa1db1dad, SUC_lexicon_tags_29 },
    { 0xc29b1dac, SUC_lexicon_tags_63 },
    { 0x9b1b1daf, SUC_lexicon_tags_15 },
    { 0x915b1db0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bb1db3, SUC_lexicon_tags_14 },
    { 0xcd1b1db3, SUC_lexicon_tags_47 },
    { 0xaedb1db3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6bb1db7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x235b1dba, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fb1dbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd45b1dc0, SUC_lexicon_tags_61 },
    { 0xcc5b1dc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37b1dc6, SUC_lexicon_tags_56 },
    { 0x6c1b1dc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17db1dc9, SUC_lexicon_tags_2 },
    { 0x5a9b1dc9, SUC_lexicon_tags_9 },
    { 0x6d3b1dcb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabb1dd0, SUC_lexicon_tags_37 },
    { 0xd07b1dd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7b1dd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfc1b1dd8, SUC_lexicon_tags_1 },
    { 0xdedb1dd9, SUC_lexicon_tags_14 },
    { 0x70fb1dda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295b1dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685b1de3, SUC_lexicon_tags_86 },
    { 0x425b1de4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7b1de8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa83b1dea, SUC_lexicon_tags_25 },
    { 0x3ddb1deb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73b1def, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abb1df2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577b1df6, SUC_lexicon_tags_7 },
    { 0xcb7b1df7, SUC_lexicon_tags_37 },
    { 0x553b1df8, SUC_lexicon_tags_9 },
    { 0x825b1df9, SUC_lexicon_tags_47 },
    { 0xc13b1dfa, SUC_lexicon_tags_5 },
    { 0xa3db1dfb, SUC_lexicon_tags_16 },
    { 0xc67b1dfb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x81fb1dfe, SUC_lexicon_tags_15 },
    { 0xe2db1dff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8c7b1e01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xafdb1e04, SUC_lexicon_tags_38 },
    { 0x1b7b1e04, SUC_lexicon_tags_25 },
    { 0x2ddb1e05, SUC_lexicon_tags_33 },
    { 0xf17b1e04, SUC_lexicon_tags_25 },
    { 0x9fb1e08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5b1e0b, SUC_lexicon_tags_17 },
    { 0x623b1e0c, SUC_lexicon_tags_17 },
    { 0xd2db1e0c, SUC_lexicon_tags_49 },
    { 0x3d3b1e0b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73b1e10, SUC_lexicon_tags_37 },
    { 0x9b1b1e11, SUC_lexicon_tags_11 },
    { 0x7afb1e12, SUC_lexicon_tags_49 },
    { 0xdb1b1e13, SUC_lexicon_tags_52 },
    { 0x647b1e14, SUC_lexicon_tags_2 },
    { 0x71fb1e15, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd55b1e17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501b1e21, SUC_lexicon_tags_47 },
    { 0xbddb1e22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8bb1e24, SUC_lexicon_tags_63 },
    { 0x695b1e24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x617b1e27, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x953b1e29, SUC_lexicon_tags_8 },
    { 0xb5b1e2a, SUC_lexicon_tags_27 },
    { 0x7bdb1e2b, SUC_lexicon_tags_27 },
    { 0x2c7b1e29, SUC_lexicon_tags_11 },
    { 0xbafb1e29, SUC_lexicon_tags_37 },
    { 0xa27b1e2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43b1e31, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7e3b1e33, SUC_lexicon_tags_15 },
    { 0xa21b1e33, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bb1e37, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb6bb1e39, SUC_lexicon_tags_1 },
    { 0x851b1e39, SUC_lexicon_tags_28 },
    { 0x88db1e3b, SUC_lexicon_tags_9 },
    { 0x3b3b1e3c, SUC_lexicon_tags_12 },
    { 0x48fb1e3c, SUC_lexicon_tags_61 },
    { 0x843b1e3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fb1e42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x281b1e44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fb1e48, SUC_lexicon_tags_55 },
    { 0xc8db1e48, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5b1e4c, SUC_lexicon_tags_11 },
    { 0x1ddb1e4d, SUC_lexicon_tags_2 },
    { 0x8a9b1e4e, SUC_lexicon_tags_31 },
    { 0xa4bb1e4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3b1e52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1b1e55, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd3fb1e57, SUC_lexicon_tags_0 },
    { 0x553b1e58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdb1e5b, SUC_lexicon_tags_15 },
    { 0xf03b1e5c, SUC_lexicon_tags_12 },
    { 0x523b1e5d, SUC_lexicon_tags_27 },
    { 0xcc9b1e5d, SUC_lexicon_tags_25 },
    { 0xf29b1e5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffb1e63, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fb1e67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9b1e6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35b1e74, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x809b1e76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edb1e7a, SUC_lexicon_tags_48 },
    { 0xc25b1e7b, SUC_lexicon_tags_23 },
    { 0x499b1e7c, SUC_lexicon_tags_37 },
    { 0x2e5b1e7b, SUC_lexicon_tags_27 },
    { 0x711b1e7d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x993b1e81, SUC_lexicon_tags_37 },
    { 0x3f5b1e81, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbb1e88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7b1e93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7db1e95, SUC_lexicon_tags_21 },
    { 0xa5b1e95, SUC_lexicon_tags_17 },
    { 0x33b1e96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77bb1e99, SUC_lexicon_tags_47 },
    { 0x669b1e9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fb1ea0, SUC_lexicon_tags_12 },
    { 0x93b1ea1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbddb1ea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x833b1ea5, SUC_lexicon_tags_37 },
    { 0x487b1ea5, SUC_lexicon_tags_38 },
    { 0x39b1ea6, SUC_lexicon_tags_12 },
    { 0xdfb1ea5, SUC_lexicon_tags_25 },
    { 0xa77b1ea5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda1b1eab, SUC_lexicon_tags_47 },
    { 0x7dbb1eac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x615b1eae, SUC_lexicon_tags_87 },
    { 0x803b1eaf, SUC_lexicon_tags_14 },
    { 0x597b1eaf, SUC_lexicon_tags_506 },
    { 0x9dfb1eb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdb1eb6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347b1eba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1b1ec1, SUC_lexicon_tags_21 },
    { 0x791b1ec2, SUC_lexicon_tags_13 },
    { 0x16bb1ec2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x76db1ec5, SUC_lexicon_tags_60 },
    { 0xb3b1ec6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03b1ec9, SUC_lexicon_tags_10 },
    { 0x493b1ec9, SUC_lexicon_tags_65 },
    { 0x931b1ecb, SUC_lexicon_tags_12 },
    { 0x21bb1ecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fb1ed0, SUC_lexicon_tags_0 },
    { 0x28fb1ed1, SUC_lexicon_tags_38 },
    { 0xcdb1ed1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795b1ed8, SUC_lexicon_tags_14 },
    { 0x3c9b1ed9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53db1edd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x53b1edf, SUC_lexicon_tags_10 },
    { 0x2b7b1ee0, SUC_lexicon_tags_15 },
    { 0x24db1ee0, SUC_lexicon_tags_37 },
    { 0x237b1ee1, SUC_lexicon_tags_22 },
    { 0x6e1b1edf, SUC_lexicon_tags_12 },
    { 0xb5b1ee4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fb1ee9, SUC_lexicon_tags_32 },
    { 0x889b1eea, SUC_lexicon_tags_69 },
    { 0x98db1eeb, SUC_lexicon_tags_31 },
    { 0x519b1ee9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x90fb1eee, SUC_lexicon_tags_9 },
    { 0x94db1eee, SUC_lexicon_tags_94 },
    { 0xa87b1ef0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x255b1ef3, SUC_lexicon_tags_4 },
    { 0x1f3b1ef4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffb1ef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7b1efd, SUC_lexicon_tags_47 },
    { 0x135b1efe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x525b1f00, SUC_lexicon_tags_12 },
    { 0xeffb1f00, SUC_lexicon_tags_34 },
    { 0x897b1f00, SUC_lexicon_tags_23 },
    { 0xe8bb1f03, SUC_lexicon_tags_46 },
    { 0x79bb1f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xac7b1f06, SUC_lexicon_tags_68 },
    { 0x45db1f07, SUC_lexicon_tags_25 },
    { 0xe1b1f06, SUC_lexicon_tags_29 },
    { 0x1a7b1f06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x521b1f0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bb1f10, SUC_lexicon_tags_60 },
    { 0x857b1f11, SUC_lexicon_tags_37 },
    { 0x9e1b1f10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f7b1f14, SUC_lexicon_tags_68 },
    { 0xe4db1f15, SUC_lexicon_tags_46 },
    { 0x55db1f16, SUC_lexicon_tags_17 },
    { 0xcc9b1f17, SUC_lexicon_tags_1 },
    { 0x26db1f18, SUC_lexicon_tags_37 },
    { 0xf81b1f18, SUC_lexicon_tags_16 },
    { 0xe67b1f19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbb1f1d, SUC_lexicon_tags_230 },
    { 0xb5fb1f1e, SUC_lexicon_tags_15 },
    { 0x10db1f1f, SUC_lexicon_tags_33 },
    { 0x203b1f1f, SUC_lexicon_tags_12 },
    { 0xb41b1f1d, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x9b7b1f23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45db1f29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x131b1f2b, SUC_lexicon_tags_29 },
    { 0x78bb1f2b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x533b1f2e, SUC_lexicon_tags_37 },
    { 0xa05b1f2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2db1f32, SUC_lexicon_tags_29 },
    { 0xde3b1f33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bb1f36, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x813b1f39, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bb1f3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdb1f40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bb1f43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa83b1f45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fb1f49, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bb1f4e, SUC_lexicon_tags_12 },
    { 0xd21b1f4e, SUC_lexicon_tags_194 },
    { 0xb15b1f50, SUC_lexicon_tags_8 },
    { 0xb3b1f51, SUC_lexicon_tags_23 },
    { 0x885b1f51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753b1f57, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa0db1f59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe61b1f5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3b1f5e, SUC_lexicon_tags_49 },
    { 0x64db1f5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9b1f64, SUC_lexicon_tags_8 },
    { 0x8b5b1f65, SUC_lexicon_tags_26 },
    { 0x9ffb1f66, SUC_lexicon_tags_11 },
    { 0xf37b1f67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x755b1f6a, SUC_lexicon_tags_11 },
    { 0xb67b1f6a, SUC_lexicon_tags_21 },
    { 0xb53b1f6a, SUC_lexicon_tags_17 },
    { 0x57bb1f6d, SUC_lexicon_tags_12 },
    { 0xfb1b1f6b, SUC_lexicon_tags_37 },
    { 0x89b1f6f, SUC_lexicon_tags_15 },
    { 0x58db1f70, SUC_lexicon_tags_11 },
    { 0x26bb1f71, SUC_lexicon_tags_25 },
    { 0x339b1f70, SUC_lexicon_tags_12 },
    { 0xfc9b1f70, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x41fb1f75, SUC_lexicon_tags_7 },
    { 0xa9db1f75, SUC_lexicon_tags_37 },
    { 0x429b1f77, SUC_lexicon_tags_25 },
    { 0x5a7b1f78, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1b1f7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53b1f7e, SUC_lexicon_tags_47 },
    { 0xf9fb1f7f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3b1f85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bb1f8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf1db1f8c, SUC_lexicon_tags_37 },
    { 0xde1b1f8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5db1f93, SUC_lexicon_tags_19 },
    { 0xc9fb1f93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x451b1f96, SUC_lexicon_tags_25 },
    { 0xdd9b1f97, SUC_lexicon_tags_45 },
    { 0x223b1f97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47db1f9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb1fa0, SUC_lexicon_tags_1 },
    { 0xe15b1fa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfb1fa6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x91b1fa8, SUC_lexicon_tags_37 },
    { 0x5fdb1fa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x643b1fab, SUC_lexicon_tags_95 },
    { 0x3db1fac, SUC_lexicon_tags_12 },
    { 0x53b1fab, SUC_lexicon_tags_2 },
    { 0x477b1fae, SUC_lexicon_tags_60 },
    { 0x5abb1fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cfb1fb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x369b1fb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x837b1fb5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91b1fbb, SUC_lexicon_tags_17 },
    { 0x4bb1fbc, SUC_lexicon_tags_6 },
    { 0xa19b1fbc, SUC_lexicon_tags_7 },
    { 0x72bb1fbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1b1fc2, SUC_lexicon_tags_11 },
    { 0xe35b1fc3, SUC_lexicon_tags_27 },
    { 0x5f3b1fc4, SUC_lexicon_tags_7 },
    { 0x7f9b1fc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x617b1fc8, SUC_lexicon_tags_12 },
    { 0x83b1fc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfb1fcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc4bb1fce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffb1fd3, SUC_lexicon_tags_10 },
    { 0x607b1fd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x975b1fd7, SUC_lexicon_tags_9 },
    { 0x1dfb1fd8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbd9b1fda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91db1fdc, SUC_lexicon_tags_31 },
    { 0x6d9b1fdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39db1fe1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb13b1fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x477b1fe6, SUC_lexicon_tags_27 },
    { 0x527b1fe7, SUC_lexicon_tags_37 },
    { 0x539b1fe6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385b1fee, SUC_lexicon_tags_0 },
    { 0xb2bb1fef, SUC_lexicon_tags_47 },
    { 0xfbfb1fee, SUC_lexicon_tags_207 },
    { 0x1f1b1ff1, SUC_lexicon_tags_2 },
    { 0x9a3b1ff2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1b1ff5, SUC_lexicon_tags_2 },
    { 0x65fb1ff6, SUC_lexicon_tags_19 },
    { 0x43db1ff6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x427b1ff9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813b1ffe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39b2001, SUC_lexicon_tags_108 },
    { 0xecbb2001, SUC_lexicon_tags_1 },
    { 0xefdb2003, SUC_lexicon_tags_11 },
    { 0x64db2001, SUC_lexicon_tags_35 },
    { 0x1bfb2001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47b2008, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21b200b, SUC_lexicon_tags_12 },
    { 0x5a9b200c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb2010, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51b2015, SUC_lexicon_tags_39 },
    { 0x2f3b2015, SUC_lexicon_tags_16 },
    { 0x28fb2015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb201a, SUC_lexicon_tags_3 },
    { 0xd67b201b, SUC_lexicon_tags_37 },
    { 0x859b201c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f5b201e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9b2022, SUC_lexicon_tags_37 },
    { 0x103b2022, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27b2026, SUC_lexicon_tags_26 },
    { 0x25b2026, SUC_lexicon_tags_23 },
    { 0x26bb2028, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x855b202b, SUC_lexicon_tags_112 },
    { 0xa53b202c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a3b202e, SUC_lexicon_tags_4 },
    { 0x1dbb202f, SUC_lexicon_tags_19 },
    { 0xee5b2030, SUC_lexicon_tags_7 },
    { 0x1a9b2030, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x359b2033, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x13db2035, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c9b2037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8db2039, SUC_lexicon_tags_17 },
    { 0x5c7b203a, SUC_lexicon_tags_23 },
    { 0x203b203b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4efb203e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545b2044, SUC_lexicon_tags_17 },
    { 0x257b2045, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedb204d, SUC_lexicon_tags_419 },
    { 0, NULL },
    { 0, NULL },
    { 0x73b2050, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32db2054, SUC_lexicon_tags_33 },
    { 0xe1fb2054, SUC_lexicon_tags_17 },
    { 0x835b2054, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb2062, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481b2066, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3b2069, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe85b206b, SUC_lexicon_tags_32 },
    { 0xd75b206b, SUC_lexicon_tags_30 },
    { 0xeb3b206c, SUC_lexicon_tags_65 },
    { 0xb63b206b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737b207c, SUC_lexicon_tags_30 },
    { 0xc09b207d, SUC_lexicon_tags_11 },
    { 0x6e7b207e, SUC_lexicon_tags_15 },
    { 0xef9b207d, SUC_lexicon_tags_1 },
    { 0xed1b2080, SUC_lexicon_tags_27 },
    { 0xc47b2080, SUC_lexicon_tags_11 },
    { 0x43db2082, SUC_lexicon_tags_49 },
    { 0xdc5b2083, SUC_lexicon_tags_38 },
    { 0xabfb2084, SUC_lexicon_tags_14 },
    { 0xfb9b2085, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3b208a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b2091, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851b2096, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1b209c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffb20a0, SUC_lexicon_tags_7 },
    { 0x861b20a1, SUC_lexicon_tags_45 },
    { 0x719b20a1, SUC_lexicon_tags_25 },
    { 0x4fdb20a2, SUC_lexicon_tags_17 },
    { 0x1e5b20a4, SUC_lexicon_tags_45 },
    { 0x871b20a4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xae9b20a7, SUC_lexicon_tags_2 },
    { 0x72bb20a8, SUC_lexicon_tags_2 },
    { 0xc3fb20a8, SUC_lexicon_tags_17 },
    { 0x1e3b20a9, SUC_lexicon_tags_7 },
    { 0xd8bb20a9, SUC_lexicon_tags_27 },
    { 0x955b20ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225b20b2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9fb20b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3b20b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2db20ba, SUC_lexicon_tags_11 },
    { 0x411b20bb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddb20be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6db20c7, SUC_lexicon_tags_37 },
    { 0x96fb20c8, SUC_lexicon_tags_23 },
    { 0x47fb20c8, SUC_lexicon_tags_63 },
    { 0x901b20c8, SUC_lexicon_tags_14 },
    { 0x391b20cb, SUC_lexicon_tags_15 },
    { 0xd5b20c7, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427b20d2, SUC_lexicon_tags_1 },
    { 0xfd3b20d3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdb20da, SUC_lexicon_tags_17 },
    { 0xf4fb20db, SUC_lexicon_tags_1 },
    { 0x64fb20db, SUC_lexicon_tags_49 },
    { 0xba9b20dd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859b20e3, SUC_lexicon_tags_33 },
    { 0x7cdb20e4, SUC_lexicon_tags_37 },
    { 0x793b20e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb39b20e7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7f9b20e9, SUC_lexicon_tags_11 },
    { 0x95fb20ea, SUC_lexicon_tags_28 },
    { 0xba7b20eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x653b20ed, SUC_lexicon_tags_26 },
    { 0x6e9b20ee, SUC_lexicon_tags_17 },
    { 0xb15b20ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfb20f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1cdb20f4, SUC_lexicon_tags_28 },
    { 0xf7b20f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92fb20f7, SUC_lexicon_tags_27 },
    { 0x5dbb20f7, SUC_lexicon_tags_1 },
    { 0xb01b20f9, SUC_lexicon_tags_2 },
    { 0x307b20fa, SUC_lexicon_tags_1 },
    { 0xa29b20f9, SUC_lexicon_tags_60 },
    { 0xb49b20fa, SUC_lexicon_tags_34 },
    { 0x1ebb20fd, SUC_lexicon_tags_61 },
    { 0x26fb20fe, SUC_lexicon_tags_39 },
    { 0x953b20ff, SUC_lexicon_tags_12 },
    { 0x75b20f9, SUC_lexicon_tags_7 },
    { 0xab7b2101, SUC_lexicon_tags_30 },
    { 0x4e5b2102, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9b2106, SUC_lexicon_tags_16 },
    { 0x765b2107, SUC_lexicon_tags_23 },
    { 0x55b2108, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x167b210a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9b210d, SUC_lexicon_tags_13 },
    { 0x1d1b210e, SUC_lexicon_tags_12 },
    { 0x89b210d, SUC_lexicon_tags_37 },
    { 0xbddb2110, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4db2119, SUC_lexicon_tags_0 },
    { 0x389b211a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa25b211c, SUC_lexicon_tags_1 },
    { 0x35fb211c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5b2121, SUC_lexicon_tags_37 },
    { 0xcbbb2121, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23b2129, SUC_lexicon_tags_7 },
    { 0xc17b212a, SUC_lexicon_tags_2 },
    { 0x465b212b, SUC_lexicon_tags_15 },
    { 0x243b212b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3b2133, SUC_lexicon_tags_10 },
    { 0x819b2133, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9b2139, SUC_lexicon_tags_11 },
    { 0xaebb2139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71fb213c, SUC_lexicon_tags_2 },
    { 0x815b213d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32fb213f, SUC_lexicon_tags_0 },
    { 0xbe7b2140, SUC_lexicon_tags_25 },
    { 0x1a9b2140, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793b2146, SUC_lexicon_tags_15 },
    { 0x60bb2146, SUC_lexicon_tags_17 },
    { 0x7f3b2146, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bb214e, SUC_lexicon_tags_32 },
    { 0xc89b214f, SUC_lexicon_tags_7 },
    { 0x5cbb214f, SUC_lexicon_tags_4 },
    { 0xb2bb2151, SUC_lexicon_tags_14 },
    { 0x31b2152, SUC_lexicon_tags_9 },
    { 0x569b2153, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911b2157, SUC_lexicon_tags_60 },
    { 0xe53b2158, SUC_lexicon_tags_60 },
    { 0xdafb2159, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37b215d, SUC_lexicon_tags_13 },
    { 0xa99b215e, SUC_lexicon_tags_1 },
    { 0x859b215f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1b2162, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdb2168, SUC_lexicon_tags_32 },
    { 0x43bb2168, SUC_lexicon_tags_500 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3b216e, SUC_lexicon_tags_0 },
    { 0x55fb216f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21b2174, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x75db2177, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5b217c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afb2185, SUC_lexicon_tags_7 },
    { 0xcabb2186, SUC_lexicon_tags_45 },
    { 0x45fb2187, SUC_lexicon_tags_30 },
    { 0x427b2187, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b5b218a, SUC_lexicon_tags_10 },
    { 0xcafb218b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bb2190, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb2193, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcdfb2195, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251b2199, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3b219d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xc91b219f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bb21a4, SUC_lexicon_tags_18 },
    { 0x4efb21a5, SUC_lexicon_tags_1 },
    { 0xe71b21a6, SUC_lexicon_tags_1108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebb21ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78db21ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137b21b3, SUC_lexicon_tags_37 },
    { 0xd3bb21b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7b21b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87b21c0, SUC_lexicon_tags_11 },
    { 0xc75b21c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf7b21c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4dbb21c5, SUC_lexicon_tags_11 },
    { 0x8bfb21c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6e9b21c8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611b21cc, SUC_lexicon_tags_17 },
    { 0x719b21cd, SUC_lexicon_tags_19 },
    { 0x3a7b21cc, SUC_lexicon_tags_37 },
    { 0xff5b21cf, SUC_lexicon_tags_11 },
    { 0xb05b21cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5b21d5, SUC_lexicon_tags_4 },
    { 0xc95b21d5, SUC_lexicon_tags_37 },
    { 0xfadb21d6, SUC_lexicon_tags_37 },
    { 0x917b21d6, SUC_lexicon_tags_32 },
    { 0x9b7b21d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21b21dc, SUC_lexicon_tags_7 },
    { 0xf25b21dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01b21e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1b21e3, SUC_lexicon_tags_25 },
    { 0x85db21e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bb21e6, SUC_lexicon_tags_27 },
    { 0x745b21e6, SUC_lexicon_tags_10 },
    { 0xb7b21e7, SUC_lexicon_tags_27 },
    { 0x409b21e9, SUC_lexicon_tags_45 },
    { 0xeb7b21ea, SUC_lexicon_tags_27 },
    { 0x413b21eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd01b21ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bb21f0, SUC_lexicon_tags_25 },
    { 0x2b7b21f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b21f5, SUC_lexicon_tags_60 },
    { 0x8dfb21f6, SUC_lexicon_tags_28 },
    { 0x90db21f7, SUC_lexicon_tags_45 },
    { 0xa11b21f8, SUC_lexicon_tags_2 },
    { 0x5f3b21f7, SUC_lexicon_tags_12 },
    { 0x827b21fa, SUC_lexicon_tags_2 },
    { 0x8abb21f8, SUC_lexicon_tags_11 },
    { 0xb63b21fc, SUC_lexicon_tags_12 },
    { 0xdadb21fd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfc5b21ff, SUC_lexicon_tags_47 },
    { 0x2ebb21ff, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfb2205, SUC_lexicon_tags_17 },
    { 0xe75b2205, SUC_lexicon_tags_6 },
    { 0x4b5b2207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x877b2209, SUC_lexicon_tags_61 },
    { 0xf91b2209, SUC_lexicon_tags_1 },
    { 0xdf3b220b, SUC_lexicon_tags_0 },
    { 0x3b3b220c, SUC_lexicon_tags_179 },
    { 0xc97b220c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23b2210, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21b2213, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119b221d, SUC_lexicon_tags_2 },
    { 0x22bb221e, SUC_lexicon_tags_47 },
    { 0x937b221f, SUC_lexicon_tags_1 },
    { 0xf6fb2220, SUC_lexicon_tags_46 },
    { 0xc7db221f, SUC_lexicon_tags_2 },
    { 0x775b2222, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x589b2224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf7b2226, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fb222b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7b222f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x371b2231, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x249b2233, SUC_lexicon_tags_23 },
    { 0x49bb2234, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fb223b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297b223f, SUC_lexicon_tags_1295 },
    { 0x7fbb2240, SUC_lexicon_tags_27 },
    { 0x541b223f, SUC_lexicon_tags_12 },
    { 0x7b5b2242, SUC_lexicon_tags_7 },
    { 0x2f1b2243, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9db2248, SUC_lexicon_tags_0 },
    { 0xdcdb2249, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b3b224b, SUC_lexicon_tags_11 },
    { 0xa93b224c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5b2253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07b2256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7cfb2258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fb225b, SUC_lexicon_tags_66 },
    { 0x1b1b225c, SUC_lexicon_tags_10 },
    { 0x691b225d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x599b2260, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edb226a, SUC_lexicon_tags_37 },
    { 0xee5b226b, SUC_lexicon_tags_47 },
    { 0x2db226c, SUC_lexicon_tags_28 },
    { 0xba1b226d, SUC_lexicon_tags_23 },
    { 0xb43b226d, SUC_lexicon_tags_15 },
    { 0xffb226f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fb2272, SUC_lexicon_tags_63 },
    { 0x57b2272, SUC_lexicon_tags_1 },
    { 0x74bb2274, SUC_lexicon_tags_11 },
    { 0x531b2274, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0db2279, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56fb227b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x995b227e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe23b2280, SUC_lexicon_tags_7 },
    { 0x429b2281, SUC_lexicon_tags_34 },
    { 0x3f9b2280, SUC_lexicon_tags_25 },
    { 0xf81b2280, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa27b2285, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bb228b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fb2292, SUC_lexicon_tags_246 },
    { 0x41bb2292, SUC_lexicon_tags_27 },
    { 0xf37b2294, SUC_lexicon_tags_12 },
    { 0x905b2292, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2bfb2297, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x159b2299, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb229d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d7b229f, SUC_lexicon_tags_17 },
    { 0x519b22a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29b22a4, SUC_lexicon_tags_37 },
    { 0x821b22a5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9b22a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6db22ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3db22b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153b22b6, SUC_lexicon_tags_23 },
    { 0x4a5b22b6, SUC_lexicon_tags_17 },
    { 0x2cdb22b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf59b22ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe01b22bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bb22bf, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdb22c5, SUC_lexicon_tags_35 },
    { 0x4e5b22c5, SUC_lexicon_tags_11 },
    { 0x3b9b22c7, SUC_lexicon_tags_25 },
    { 0xaebb22c8, SUC_lexicon_tags_7 },
    { 0xe35b22c6, SUC_lexicon_tags_7 },
    { 0xfe9b22ca, SUC_lexicon_tags_31 },
    { 0xbc1b22ca, SUC_lexicon_tags_45 },
    { 0xd9bb22ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5b22d1, SUC_lexicon_tags_26 },
    { 0xa17b22d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37b22d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903b22e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7b22e3, SUC_lexicon_tags_25 },
    { 0x20db22e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa2db22e6, SUC_lexicon_tags_14 },
    { 0xaa1b22e7, SUC_lexicon_tags_25 },
    { 0xc5b22e7, SUC_lexicon_tags_1 },
    { 0xf3bb22e9, SUC_lexicon_tags_66 },
    { 0xb5bb22ea, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffb22f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b22f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81b22fe, SUC_lexicon_tags_17 },
    { 0x1bb22ff, SUC_lexicon_tags_2 },
    { 0x1e9b22fe, SUC_lexicon_tags_7 },
    { 0xa55b22fe, SUC_lexicon_tags_25 },
    { 0x573b2302, SUC_lexicon_tags_3 },
    { 0xb81b22fe, SUC_lexicon_tags_13 },
    { 0xbf1b2304, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81b2309, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa0bb230b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6fb230d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7b2310, SUC_lexicon_tags_12 },
    { 0x639b2311, SUC_lexicon_tags_37 },
    { 0x589b2310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77b2316, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3b231c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x229b231e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef5b2320, SUC_lexicon_tags_17 },
    { 0x7fdb2321, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d3b2323, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa29b2325, SUC_lexicon_tags_12 },
    { 0xff9b2326, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c9b2328, SUC_lexicon_tags_60 },
    { 0xd87b2329, SUC_lexicon_tags_37 },
    { 0x4a3b232a, SUC_lexicon_tags_38 },
    { 0x7a7b232b, SUC_lexicon_tags_27 },
    { 0x4a5b232c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc7b232e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x18fb2330, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83b2332, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4db2336, SUC_lexicon_tags_17 },
    { 0xb87b2337, SUC_lexicon_tags_60 },
    { 0xb75b2338, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb45b233b, SUC_lexicon_tags_27 },
    { 0x93fb233c, SUC_lexicon_tags_7 },
    { 0xfd3b233c, SUC_lexicon_tags_12 },
    { 0xfdbb233e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b2342, SUC_lexicon_tags_12 },
    { 0x5e9b2343, SUC_lexicon_tags_17 },
    { 0x11fb2344, SUC_lexicon_tags_28 },
    { 0x867b2345, SUC_lexicon_tags_34 },
    { 0x7ffb2344, SUC_lexicon_tags_25 },
    { 0xf43b2347, SUC_lexicon_tags_66 },
    { 0x151b2348, SUC_lexicon_tags_17 },
    { 0xd23b2345, SUC_lexicon_tags_25 },
    { 0xb5bb2342, SUC_lexicon_tags_84 },
    { 0xd25b234a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03b2350, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9b2354, SUC_lexicon_tags_15 },
    { 0x46bb2355, SUC_lexicon_tags_15 },
    { 0xc35b2356, SUC_lexicon_tags_2 },
    { 0x359b2355, SUC_lexicon_tags_65 },
    { 0x557b2358, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeafb235b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15b235f, SUC_lexicon_tags_15 },
    { 0x929b235f, SUC_lexicon_tags_37 },
    { 0xc4bb2360, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1b2364, SUC_lexicon_tags_37 },
    { 0x485b2365, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x607b2367, SUC_lexicon_tags_37 },
    { 0xc35b2367, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1db236d, SUC_lexicon_tags_37 },
    { 0xa15b236d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9b2373, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x657b2376, SUC_lexicon_tags_17 },
    { 0x651b2377, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa27b237a, SUC_lexicon_tags_15 },
    { 0x885b237b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc79b237d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5bb237f, SUC_lexicon_tags_2 },
    { 0xfc3b237f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x34fb2382, SUC_lexicon_tags_43 },
    { 0xaf7b2383, SUC_lexicon_tags_14 },
    { 0x90bb2384, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdebb2386, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911b238c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdb2392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975b2396, SUC_lexicon_tags_15 },
    { 0x5afb2396, SUC_lexicon_tags_65 },
    { 0x3f9b2397, SUC_lexicon_tags_37 },
    { 0xdfdb2399, SUC_lexicon_tags_7 },
    { 0xfbdb239a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45b239d, SUC_lexicon_tags_49 },
    { 0x8fbb239e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fb23a0, SUC_lexicon_tags_3 },
    { 0x73bb23a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3b23ac, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x935b23af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bb23b5, SUC_lexicon_tags_17 },
    { 0x5efb23b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a1b23b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bb23bb, SUC_lexicon_tags_15 },
    { 0x405b23bb, SUC_lexicon_tags_33 },
    { 0x2a1b23bd, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x6d9b23bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fb23c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7b23c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x633b23c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5fb23cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adb23d0, SUC_lexicon_tags_11 },
    { 0x9f9b23d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b23d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf89b23d6, SUC_lexicon_tags_53 },
    { 0x529b23d7, SUC_lexicon_tags_25 },
    { 0x469b23d6, SUC_lexicon_tags_56 },
    { 0xed7b23d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917b23df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bb23e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x819b23e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63b23ea, SUC_lexicon_tags_11 },
    { 0x89b23eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3b23ee, SUC_lexicon_tags_2 },
    { 0x1fdb23ef, SUC_lexicon_tags_23 },
    { 0x93db23f0, SUC_lexicon_tags_38 },
    { 0x1cfb23f1, SUC_lexicon_tags_112 },
    { 0x9d5b23f2, SUC_lexicon_tags_17 },
    { 0x19b23f3, SUC_lexicon_tags_63 },
    { 0xc81b23f0, SUC_lexicon_tags_15 },
    { 0x8e7b23ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x63db23f8, SUC_lexicon_tags_17 },
    { 0x471b23f9, SUC_lexicon_tags_7 },
    { 0x6d1b23fa, SUC_lexicon_tags_14 },
    { 0x7a1b23fb, SUC_lexicon_tags_19 },
    { 0x3cfb23fb, SUC_lexicon_tags_23 },
    { 0x6c1b23fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x271b2400, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95b2407, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41b240e, SUC_lexicon_tags_15 },
    { 0xb99b240e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09b2414, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15b2417, SUC_lexicon_tags_28 },
    { 0x275b2418, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x8e3b241a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1b2426, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb242a, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x6efb242c, SUC_lexicon_tags_28 },
    { 0x66db242d, SUC_lexicon_tags_45 },
    { 0x8dbb242c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x301b2430, SUC_lexicon_tags_16 },
    { 0x8f3b2430, SUC_lexicon_tags_4 },
    { 0xf5fb2432, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53b2435, SUC_lexicon_tags_2 },
    { 0x5d5b2436, SUC_lexicon_tags_25 },
    { 0xb71b2437, SUC_lexicon_tags_6 },
    { 0x1a3b2437, SUC_lexicon_tags_43 },
    { 0x5c9b2439, SUC_lexicon_tags_25 },
    { 0x16db2439, SUC_lexicon_tags_17 },
    { 0xd2fb2439, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbb243e, SUC_lexicon_tags_8 },
    { 0x799b243f, SUC_lexicon_tags_37 },
    { 0x1e7b243f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbb2445, SUC_lexicon_tags_8 },
    { 0xa6db2446, SUC_lexicon_tags_23 },
    { 0x3edb2447, SUC_lexicon_tags_3 },
    { 0xcf5b2448, SUC_lexicon_tags_87 },
    { 0xfd9b2447, SUC_lexicon_tags_29 },
    { 0x723b244a, SUC_lexicon_tags_26 },
    { 0xd81b2449, SUC_lexicon_tags_17 },
    { 0xc7db2446, SUC_lexicon_tags_37 },
    { 0x445b2445, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b2452, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1b2455, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113b245f, SUC_lexicon_tags_17 },
    { 0x2ddb2460, SUC_lexicon_tags_4 },
    { 0xc67b2461, SUC_lexicon_tags_11 },
    { 0x8adb2462, SUC_lexicon_tags_15 },
    { 0xb33b2462, SUC_lexicon_tags_53 },
    { 0x1efb2462, SUC_lexicon_tags_37 },
    { 0xd15b2465, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffb246a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89b246d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaffb246f, SUC_lexicon_tags_2 },
    { 0x9e1b2470, SUC_lexicon_tags_27 },
    { 0xd8db2471, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x533b2474, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x171b2476, SUC_lexicon_tags_45 },
    { 0x947b2476, SUC_lexicon_tags_37 },
    { 0xac9b2477, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x56fb247a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bb2480, SUC_lexicon_tags_16 },
    { 0x4efb2481, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a3b2483, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7c1b2485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae1b2487, SUC_lexicon_tags_29 },
    { 0xcddb2488, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x29fb248a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1b248e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd27b2490, SUC_lexicon_tags_12 },
    { 0xba7b2491, SUC_lexicon_tags_30 },
    { 0x19db2492, SUC_lexicon_tags_37 },
    { 0xf67b2491, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f1b2495, SUC_lexicon_tags_44 },
    { 0xeadb2495, SUC_lexicon_tags_17 },
    { 0xa13b2497, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9b249a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x91fb249d, SUC_lexicon_tags_11 },
    { 0xca9b249e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3b24a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62db24a3, SUC_lexicon_tags_28 },
    { 0xef1b24a4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x59fb24a7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x875b24a9, SUC_lexicon_tags_29 },
    { 0xd3db24aa, SUC_lexicon_tags_21 },
    { 0xd17b24aa, SUC_lexicon_tags_0 },
    { 0xd91b24ab, SUC_lexicon_tags_11 },
    { 0x56db24ac, SUC_lexicon_tags_27 },
    { 0x327b24ae, SUC_lexicon_tags_38 },
    { 0x4db24af, SUC_lexicon_tags_0 },
    { 0xb2db24b0, SUC_lexicon_tags_17 },
    { 0xedfb24b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5b24b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45b24ba, SUC_lexicon_tags_49 },
    { 0xcd5b24bb, SUC_lexicon_tags_1 },
    { 0xa39b24bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fb24c0, SUC_lexicon_tags_12 },
    { 0x125b24c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa6db24c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2efb24c5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdb24cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65b24cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x459b24d1, SUC_lexicon_tags_10 },
    { 0x717b24d1, SUC_lexicon_tags_1 },
    { 0x15b24d3, SUC_lexicon_tags_12 },
    { 0x96bb24d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1b24d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7b24df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x493b24e1, SUC_lexicon_tags_43 },
    { 0x821b24e2, SUC_lexicon_tags_27 },
    { 0xf4fb24e2, SUC_lexicon_tags_26 },
    { 0x81bb24e4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa51b24e6, SUC_lexicon_tags_25 },
    { 0x6e7b24e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65b24eb, SUC_lexicon_tags_15 },
    { 0x795b24eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127b24f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31fb24f4, SUC_lexicon_tags_91 },
    { 0x971b24f5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x675b24f8, SUC_lexicon_tags_0 },
    { 0x703b24f9, SUC_lexicon_tags_6 },
    { 0x2a5b24fa, SUC_lexicon_tags_2 },
    { 0x45fb24fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85bb24fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645b2503, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebb2507, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa8db2509, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbfdb250b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5b250f, SUC_lexicon_tags_1 },
    { 0x93bb250f, SUC_lexicon_tags_6 },
    { 0xa89b2511, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51b2514, SUC_lexicon_tags_37 },
    { 0x357b2515, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fb2518, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fb251f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7b2522, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefb2528, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9b252d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5db252f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fb2534, SUC_lexicon_tags_7 },
    { 0xfe3b2535, SUC_lexicon_tags_29 },
    { 0x1d3b2536, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fb2539, SUC_lexicon_tags_24 },
    { 0xa3bb2539, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29b253e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaddb2541, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791b2545, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01b254d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a1b254f, SUC_lexicon_tags_1135 },
    { 0xc55b254f, SUC_lexicon_tags_14 },
    { 0x11b2551, SUC_lexicon_tags_7 },
    { 0x509b2551, SUC_lexicon_tags_17 },
    { 0xa1db2553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9b2558, SUC_lexicon_tags_15 },
    { 0xa0bb2559, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bb255e, SUC_lexicon_tags_25 },
    { 0x32bb255f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6db2563, SUC_lexicon_tags_29 },
    { 0xb5fb2564, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xee9b2566, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db2569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x147b256c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1db2570, SUC_lexicon_tags_25 },
    { 0xbe5b2571, SUC_lexicon_tags_12 },
    { 0x533b2572, SUC_lexicon_tags_1 },
    { 0x42bb2573, SUC_lexicon_tags_25 },
    { 0x36db2574, SUC_lexicon_tags_30 },
    { 0xc33b2574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989b257a, SUC_lexicon_tags_15 },
    { 0x9bb257b, SUC_lexicon_tags_25 },
    { 0x61bb257b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1b257f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6abb2581, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7b2586, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bb2589, SUC_lexicon_tags_23 },
    { 0x1afb258a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x18db258d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5b2594, SUC_lexicon_tags_17 },
    { 0x237b2595, SUC_lexicon_tags_33 },
    { 0xa2fb2596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4adb2598, SUC_lexicon_tags_2 },
    { 0xbd7b2599, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1b259c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffb259f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2fb25a1, SUC_lexicon_tags_49 },
    { 0x9b25a2, SUC_lexicon_tags_21 },
    { 0x513b25a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd05b25a5, SUC_lexicon_tags_37 },
    { 0xfcdb25a5, SUC_lexicon_tags_127 },
    { 0xf5b25a7, SUC_lexicon_tags_27 },
    { 0xdbfb25a6, SUC_lexicon_tags_47 },
    { 0xb69b25a9, SUC_lexicon_tags_14 },
    { 0xaebb25aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3b25ae, SUC_lexicon_tags_1 },
    { 0x5a7b25af, SUC_lexicon_tags_2 },
    { 0xfb1b25ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfb25b4, SUC_lexicon_tags_11 },
    { 0xfdb25b5, SUC_lexicon_tags_25 },
    { 0x11fb25b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0db25be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa67b25c0, SUC_lexicon_tags_14 },
    { 0x7ebb25c1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1b25c4, SUC_lexicon_tags_15 },
    { 0x3b5b25c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeebb25c7, SUC_lexicon_tags_33 },
    { 0xa6fb25c8, SUC_lexicon_tags_11 },
    { 0xb8bb25c7, SUC_lexicon_tags_23 },
    { 0x83b25ca, SUC_lexicon_tags_23 },
    { 0xf37b25ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3b25ce, SUC_lexicon_tags_32 },
    { 0xe87b25ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x565b25d1, SUC_lexicon_tags_22 },
    { 0x565b25d2, SUC_lexicon_tags_25 },
    { 0xa83b25d3, SUC_lexicon_tags_40 },
    { 0xb9db25d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf75b25d6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79b25db, SUC_lexicon_tags_45 },
    { 0xee5b25db, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bb25e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x429b25e2, SUC_lexicon_tags_1 },
    { 0x481b25e3, SUC_lexicon_tags_25 },
    { 0xb6bb25e4, SUC_lexicon_tags_15 },
    { 0x655b25e5, SUC_lexicon_tags_29 },
    { 0x97b25e6, SUC_lexicon_tags_2 },
    { 0xcadb25e7, SUC_lexicon_tags_17 },
    { 0x835b25e7, SUC_lexicon_tags_60 },
    { 0x3cdb25e8, SUC_lexicon_tags_7 },
    { 0xfa7b25ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69b25ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7b25f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x395b25f4, SUC_lexicon_tags_11 },
    { 0x68fb25f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b25fa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195b25fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f1b2600, SUC_lexicon_tags_11 },
    { 0xf47b2601, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaadb2603, SUC_lexicon_tags_25 },
    { 0x6fbb2604, SUC_lexicon_tags_49 },
    { 0xeb1b2605, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x335b2608, SUC_lexicon_tags_37 },
    { 0xa45b2608, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60db2611, SUC_lexicon_tags_7 },
    { 0xa8bb2611, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x99b2614, SUC_lexicon_tags_2 },
    { 0x367b2615, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1b261d, SUC_lexicon_tags_37 },
    { 0x6e7b261e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47b2622, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309b262b, SUC_lexicon_tags_42 },
    { 0xea1b262c, SUC_lexicon_tags_11 },
    { 0x97bb262c, SUC_lexicon_tags_10 },
    { 0xd41b262c, SUC_lexicon_tags_60 },
    { 0xa35b262f, SUC_lexicon_tags_25 },
    { 0x4a1b262f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55b2637, SUC_lexicon_tags_12 },
    { 0xcdb2638, SUC_lexicon_tags_17 },
    { 0x6ebb2639, SUC_lexicon_tags_46 },
    { 0x47bb2638, SUC_lexicon_tags_14 },
    { 0xfa9b263a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b2640, SUC_lexicon_tags_61 },
    { 0x94fb2641, SUC_lexicon_tags_17 },
    { 0xf1fb2642, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91b2646, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb264a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb264f, SUC_lexicon_tags_38 },
    { 0xb29b264f, SUC_lexicon_tags_167 },
    { 0x2efb2651, SUC_lexicon_tags_5 },
    { 0x501b264f, SUC_lexicon_tags_6 },
    { 0x87db2653, SUC_lexicon_tags_17 },
    { 0x4fdb2654, SUC_lexicon_tags_48 },
    { 0x77fb2655, SUC_lexicon_tags_37 },
    { 0xe2fb264f, SUC_lexicon_tags_26 },
    { 0xb5b2654, SUC_lexicon_tags_37 },
    { 0x585b2656, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59bb265a, SUC_lexicon_tags_26 },
    { 0x3bbb265a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1b265e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbb2661, SUC_lexicon_tags_1 },
    { 0x579b2661, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7b2668, SUC_lexicon_tags_7 },
    { 0x687b2669, SUC_lexicon_tags_17 },
    { 0xcd1b2669, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bb2673, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x383b2675, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9b2679, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adb267d, SUC_lexicon_tags_1 },
    { 0xa8db267e, SUC_lexicon_tags_14 },
    { 0x9a7b267d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e5b2681, SUC_lexicon_tags_17 },
    { 0xe9fb2682, SUC_lexicon_tags_33 },
    { 0xb95b2683, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443b2687, SUC_lexicon_tags_14 },
    { 0xaadb2688, SUC_lexicon_tags_1 },
    { 0xc17b2689, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703b268e, SUC_lexicon_tags_37 },
    { 0x979b268e, SUC_lexicon_tags_15 },
    { 0xb8db2690, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62bb2692, SUC_lexicon_tags_15 },
    { 0x89bb2693, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x17b2695, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfb2699, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x237b269b, SUC_lexicon_tags_37 },
    { 0x2cbb269c, SUC_lexicon_tags_49 },
    { 0x609b269b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa85b269f, SUC_lexicon_tags_7 },
    { 0x997b26a0, SUC_lexicon_tags_11 },
    { 0x15db26a0, SUC_lexicon_tags_49 },
    { 0xe2db26a2, SUC_lexicon_tags_37 },
    { 0x785b269f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb05b26a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x725b26a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x58fb26a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x189b26ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x905b26ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x111b26b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1b26b3, SUC_lexicon_tags_11 },
    { 0xbd9b26b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x961b26b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fb26bc, SUC_lexicon_tags_15 },
    { 0xd55b26bd, SUC_lexicon_tags_21 },
    { 0x373b26bc, SUC_lexicon_tags_45 },
    { 0x619b26bc, SUC_lexicon_tags_31 },
    { 0xb0bb26bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27fb26c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13db26c6, SUC_lexicon_tags_26 },
    { 0x1d1b26c7, SUC_lexicon_tags_11 },
    { 0xbfb26c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x601b26ca, SUC_lexicon_tags_40 },
    { 0x5c9b26cb, SUC_lexicon_tags_55 },
    { 0x93db26cc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325b26d3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xcdb26d5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5b26d8, SUC_lexicon_tags_39 },
    { 0x73b26d9, SUC_lexicon_tags_42 },
    { 0x5efb26da, SUC_lexicon_tags_60 },
    { 0xb2bb26da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffb26de, SUC_lexicon_tags_9 },
    { 0x3d7b26df, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909b26e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3b26e9, SUC_lexicon_tags_11 },
    { 0xee5b26ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbb26ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71bb26f0, SUC_lexicon_tags_35 },
    { 0xda5b26f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc45b26f3, SUC_lexicon_tags_27 },
    { 0x4dfb26f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bb26f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7b26ff, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf09b2701, SUC_lexicon_tags_37 },
    { 0xef9b2702, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587b2709, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd15b270b, SUC_lexicon_tags_47 },
    { 0x319b270c, SUC_lexicon_tags_30 },
    { 0x73bb270d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89db270f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bb2713, SUC_lexicon_tags_14 },
    { 0x93fb2714, SUC_lexicon_tags_2 },
    { 0x139b2715, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x6f7b2717, SUC_lexicon_tags_106 },
    { 0xf97b2718, SUC_lexicon_tags_7 },
    { 0xabdb2718, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfb271c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddb2720, SUC_lexicon_tags_11 },
    { 0x809b2721, SUC_lexicon_tags_43 },
    { 0xe69b2722, SUC_lexicon_tags_25 },
    { 0xccbb2723, SUC_lexicon_tags_8 },
    { 0xc6db2720, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bb2727, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39b272a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x647b272c, SUC_lexicon_tags_11 },
    { 0xe51b272d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1b2731, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb2735, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdabb2737, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb67b2739, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3b273d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x665b273f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x605b2742, SUC_lexicon_tags_19 },
    { 0xf4db2743, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591b2748, SUC_lexicon_tags_70 },
    { 0xa6fb2749, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf0bb274b, SUC_lexicon_tags_14 },
    { 0x5bb274b, SUC_lexicon_tags_46 },
    { 0xf27b274d, SUC_lexicon_tags_12 },
    { 0x96bb274d, SUC_lexicon_tags_17 },
    { 0x647b274f, SUC_lexicon_tags_37 },
    { 0x8cbb274d, SUC_lexicon_tags_37 },
    { 0x603b2751, SUC_lexicon_tags_39 },
    { 0x4d3b2752, SUC_lexicon_tags_124 },
    { 0x21fb2753, SUC_lexicon_tags_17 },
    { 0x4d3b2754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35b2757, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987b275e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7b2761, SUC_lexicon_tags_37 },
    { 0x425b2762, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773b2766, SUC_lexicon_tags_48 },
    { 0x861b2767, SUC_lexicon_tags_25 },
    { 0x69db2768, SUC_lexicon_tags_1 },
    { 0x8a9b2768, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfb276d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5b2771, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5b1b2773, SUC_lexicon_tags_652 },
    { 0, NULL },
    { 0x1ddb2775, SUC_lexicon_tags_0 },
    { 0xd11b2776, SUC_lexicon_tags_45 },
    { 0xe0bb2777, SUC_lexicon_tags_7 },
    { 0x58fb2776, SUC_lexicon_tags_37 },
    { 0x2cdb2779, SUC_lexicon_tags_38 },
    { 0x8e9b277a, SUC_lexicon_tags_7 },
    { 0xbe7b277b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b2783, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedb2787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x785b2789, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867b278e, SUC_lexicon_tags_7 },
    { 0x7c7b278f, SUC_lexicon_tags_17 },
    { 0x25bb2790, SUC_lexicon_tags_2 },
    { 0x59b2790, SUC_lexicon_tags_37 },
    { 0xab9b2792, SUC_lexicon_tags_29 },
    { 0xe65b2790, SUC_lexicon_tags_1 },
    { 0xeb3b2791, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x18fb2797, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bb279b, SUC_lexicon_tags_7 },
    { 0x557b279b, SUC_lexicon_tags_21 },
    { 0x9cdb279d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x219b27a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b27a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5b27aa, SUC_lexicon_tags_41 },
    { 0xa5b27ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3b27ae, SUC_lexicon_tags_33 },
    { 0x149b27ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19db27b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x521b27b4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x311b27b6, SUC_lexicon_tags_45 },
    { 0x9a5b27b6, SUC_lexicon_tags_47 },
    { 0x207b27b8, SUC_lexicon_tags_17 },
    { 0xb77b27b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x59db27bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5b27c3, SUC_lexicon_tags_31 },
    { 0xe1bb27c4, SUC_lexicon_tags_1 },
    { 0xe81b27c5, SUC_lexicon_tags_11 },
    { 0x425b27c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5b27c9, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x5bfb27cb, SUC_lexicon_tags_25 },
    { 0xe15b27cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267b27d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1b27d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x27bb27d5, SUC_lexicon_tags_14 },
    { 0x7c1b27d6, SUC_lexicon_tags_672 },
    { 0x757b27d7, SUC_lexicon_tags_81 },
    { 0x25fb27d8, SUC_lexicon_tags_14 },
    { 0x61b27d9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3b27dd, SUC_lexicon_tags_11 },
    { 0x98bb27dd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffb27e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b27e4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x223b27e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd5b27e8, SUC_lexicon_tags_45 },
    { 0xb1db27e8, SUC_lexicon_tags_19 },
    { 0x2f1b27e9, SUC_lexicon_tags_12 },
    { 0x83b27eb, SUC_lexicon_tags_32 },
    { 0xab3b27eb, SUC_lexicon_tags_33 },
    { 0x45db27eb, SUC_lexicon_tags_20 },
    { 0xd2db27ec, SUC_lexicon_tags_37 },
    { 0xd1bb27ef, SUC_lexicon_tags_17 },
    { 0xf8bb27f0, SUC_lexicon_tags_23 },
    { 0x233b27f1, SUC_lexicon_tags_12 },
    { 0x273b27ef, SUC_lexicon_tags_27 },
    { 0xfe7b27f3, SUC_lexicon_tags_6 },
    { 0xc71b27f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43b27fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bb27fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafb2801, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fb2806, SUC_lexicon_tags_0 },
    { 0x3afb2807, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b2811, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1b2814, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x299b2817, SUC_lexicon_tags_7 },
    { 0x23fb2818, SUC_lexicon_tags_39 },
    { 0xd29b2819, SUC_lexicon_tags_37 },
    { 0x83b2818, SUC_lexicon_tags_23 },
    { 0x781b281b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1b281e, SUC_lexicon_tags_15 },
    { 0x8afb281f, SUC_lexicon_tags_2 },
    { 0x7f9b2820, SUC_lexicon_tags_36 },
    { 0xd59b2820, SUC_lexicon_tags_17 },
    { 0x6c1b2821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53b282a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfe9b282c, SUC_lexicon_tags_7 },
    { 0xabfb282d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbb2832, SUC_lexicon_tags_1 },
    { 0xe45b2833, SUC_lexicon_tags_25 },
    { 0x9b5b2832, SUC_lexicon_tags_28 },
    { 0xb59b2835, SUC_lexicon_tags_11 },
    { 0x987b2835, SUC_lexicon_tags_15 },
    { 0x105b2837, SUC_lexicon_tags_34 },
    { 0xbe1b2837, SUC_lexicon_tags_25 },
    { 0x3dfb2839, SUC_lexicon_tags_30 },
    { 0xcedb283a, SUC_lexicon_tags_17 },
    { 0xf19b2834, SUC_lexicon_tags_27 },
    { 0xa35b283c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbb2840, SUC_lexicon_tags_15 },
    { 0x7ddb2841, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833b2845, SUC_lexicon_tags_26 },
    { 0x5c9b2846, SUC_lexicon_tags_92 },
    { 0x5c3b2846, SUC_lexicon_tags_27 },
    { 0x883b2848, SUC_lexicon_tags_11 },
    { 0x741b2849, SUC_lexicon_tags_26 },
    { 0x727b284a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcfb284c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x417b284e, SUC_lexicon_tags_47 },
    { 0x2bb284f, SUC_lexicon_tags_49 },
    { 0x5f1b2850, SUC_lexicon_tags_15 },
    { 0x6ddb2851, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x201b2853, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x245b2855, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329b285a, SUC_lexicon_tags_118 },
    { 0x915b285b, SUC_lexicon_tags_17 },
    { 0x399b285c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdb2860, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61b2863, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfb286a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfb286d, SUC_lexicon_tags_63 },
    { 0x5c7b286d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15b2871, SUC_lexicon_tags_25 },
    { 0x3f3b2871, SUC_lexicon_tags_14 },
    { 0x6b1b2873, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23b287c, SUC_lexicon_tags_19 },
    { 0x8b5b287d, SUC_lexicon_tags_49 },
    { 0xe4db287d, SUC_lexicon_tags_13 },
    { 0xcd1b287f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5b288b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53b288f, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fb2893, SUC_lexicon_tags_1 },
    { 0xe13b2893, SUC_lexicon_tags_87 },
    { 0xea7b2894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9fb2897, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5cdb2899, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x845b289b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x305b289d, SUC_lexicon_tags_11 },
    { 0xc75b289d, SUC_lexicon_tags_25 },
    { 0x707b289e, SUC_lexicon_tags_11 },
    { 0x4fb289e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa13b28a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89b28a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49fb28a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x823b28a9, SUC_lexicon_tags_33 },
    { 0x115b28aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x859b28ac, SUC_lexicon_tags_27 },
    { 0x9ddb28ad, SUC_lexicon_tags_193 },
    { 0x909b28ae, SUC_lexicon_tags_63 },
    { 0x313b28ae, SUC_lexicon_tags_17 },
    { 0x81b28ac, SUC_lexicon_tags_25 },
    { 0x6d5b28ad, SUC_lexicon_tags_29 },
    { 0x385b28b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77b28be, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551b28ca, SUC_lexicon_tags_15 },
    { 0x93db28cb, SUC_lexicon_tags_74 },
    { 0x17db28cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x211b28ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971b28d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89b28d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab5b28d8, SUC_lexicon_tags_9 },
    { 0x197b28d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x819b28db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c7b28dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc29b28df, SUC_lexicon_tags_1 },
    { 0x5a1b28e0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x205b28e2, SUC_lexicon_tags_2 },
    { 0x755b28e2, SUC_lexicon_tags_15 },
    { 0x65db28e4, SUC_lexicon_tags_11 },
    { 0x80fb28e5, SUC_lexicon_tags_23 },
    { 0x17db28e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5b28e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bdb28eb, SUC_lexicon_tags_2 },
    { 0x81bb28ec, SUC_lexicon_tags_1 },
    { 0xcf7b28ec, SUC_lexicon_tags_19 },
    { 0xa41b28eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b28f4, SUC_lexicon_tags_79 },
    { 0xf05b28f5, SUC_lexicon_tags_26 },
    { 0x2b9b28f4, SUC_lexicon_tags_37 },
    { 0xd17b28f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17b28fc, SUC_lexicon_tags_17 },
    { 0x305b28fd, SUC_lexicon_tags_26 },
    { 0xb35b28fe, SUC_lexicon_tags_38 },
    { 0xcc7b28fd, SUC_lexicon_tags_17 },
    { 0x7fdb2900, SUC_lexicon_tags_2 },
    { 0x531b2901, SUC_lexicon_tags_37 },
    { 0xa3fb28fd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783b2908, SUC_lexicon_tags_33 },
    { 0x9db2909, SUC_lexicon_tags_27 },
    { 0xe6fb2908, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfb2910, SUC_lexicon_tags_7 },
    { 0x79b2911, SUC_lexicon_tags_37 },
    { 0x849b2912, SUC_lexicon_tags_82 },
    { 0x5cdb2913, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bb291b, SUC_lexicon_tags_30 },
    { 0x4bfb291c, SUC_lexicon_tags_61 },
    { 0x59fb291c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5b2921, SUC_lexicon_tags_4 },
    { 0x6b5b2922, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x58db2925, SUC_lexicon_tags_60 },
    { 0x7d5b2926, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82db2929, SUC_lexicon_tags_7 },
    { 0x587b292a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adb2931, SUC_lexicon_tags_37 },
    { 0x14fb2932, SUC_lexicon_tags_12 },
    { 0xc45b2933, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309b293a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efb2942, SUC_lexicon_tags_12 },
    { 0x3f7b2943, SUC_lexicon_tags_57 },
    { 0xf73b2942, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759b294b, SUC_lexicon_tags_30 },
    { 0x5b1b294c, SUC_lexicon_tags_29 },
    { 0xa8fb294c, SUC_lexicon_tags_12 },
    { 0xb63b294e, SUC_lexicon_tags_37 },
    { 0x893b294c, SUC_lexicon_tags_25 },
    { 0x4e5b2950, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x93db2953, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x213b2955, SUC_lexicon_tags_38 },
    { 0x5b3b2955, SUC_lexicon_tags_28 },
    { 0xd59b2956, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bb295c, SUC_lexicon_tags_39 },
    { 0x7efb295c, SUC_lexicon_tags_84 },
    { 0x9b5b295e, SUC_lexicon_tags_7 },
    { 0x2e9b295f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5b2964, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa27b2966, SUC_lexicon_tags_25 },
    { 0xc65b2966, SUC_lexicon_tags_3 },
    { 0xf29b2967, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b296c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bb2972, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99b2975, SUC_lexicon_tags_38 },
    { 0xb7b2975, SUC_lexicon_tags_31 },
    { 0x867b2976, SUC_lexicon_tags_6 },
    { 0x19b2978, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1e3b297a, SUC_lexicon_tags_27 },
    { 0x6d1b297a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82db2981, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283b2985, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe4db2987, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2cdb2989, SUC_lexicon_tags_12 },
    { 0x96fb298a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791b298f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5b2993, SUC_lexicon_tags_16 },
    { 0xdedb2993, SUC_lexicon_tags_33 },
    { 0xdb3b2995, SUC_lexicon_tags_4 },
    { 0xe29b2995, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb81b2998, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61b299d, SUC_lexicon_tags_3 },
    { 0xfb299e, SUC_lexicon_tags_9 },
    { 0xb0db299e, SUC_lexicon_tags_1 },
    { 0x959b29a0, SUC_lexicon_tags_6 },
    { 0xa9bb29a1, SUC_lexicon_tags_32 },
    { 0xabb29a2, SUC_lexicon_tags_61 },
    { 0xe97b299d, SUC_lexicon_tags_6 },
    { 0x9e1b29a4, SUC_lexicon_tags_25 },
    { 0x631b29a5, SUC_lexicon_tags_19 },
    { 0xf9b29a6, SUC_lexicon_tags_14 },
    { 0x30db29a7, SUC_lexicon_tags_70 },
    { 0x81b29a8, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x209b29aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3b29af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd69b29b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x44db29b3, SUC_lexicon_tags_38 },
    { 0x52db29b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x475b29b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb43b29b8, SUC_lexicon_tags_16 },
    { 0xb7b29b9, SUC_lexicon_tags_14 },
    { 0x7fbb29ba, SUC_lexicon_tags_9 },
    { 0x745b29bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x471b29be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x407b29c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfb29c6, SUC_lexicon_tags_1 },
    { 0xa7db29c7, SUC_lexicon_tags_27 },
    { 0x33db29c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8adb29ca, SUC_lexicon_tags_35 },
    { 0x5fdb29cb, SUC_lexicon_tags_23 },
    { 0x59db29cc, SUC_lexicon_tags_9 },
    { 0x63db29cd, SUC_lexicon_tags_14 },
    { 0x43bb29cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d1b29d0, SUC_lexicon_tags_11 },
    { 0x629b29d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fb29d5, SUC_lexicon_tags_0 },
    { 0xa39b29d5, SUC_lexicon_tags_7 },
    { 0x313b29d5, SUC_lexicon_tags_17 },
    { 0xbb29d8, SUC_lexicon_tags_29 },
    { 0x653b29d9, SUC_lexicon_tags_38 },
    { 0x6efb29da, SUC_lexicon_tags_27 },
    { 0xb17b29da, SUC_lexicon_tags_92 },
    { 0x595b29db, SUC_lexicon_tags_60 },
    { 0xd6bb29dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831b29e5, SUC_lexicon_tags_106 },
    { 0xc8fb29e6, SUC_lexicon_tags_82 },
    { 0x247b29e7, SUC_lexicon_tags_37 },
    { 0xcf9b29e6, SUC_lexicon_tags_65 },
    { 0x705b29e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289b29ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4fb29f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a9b29f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643b29f8, SUC_lexicon_tags_38 },
    { 0xa31b29f9, SUC_lexicon_tags_60 },
    { 0x6e5b29fa, SUC_lexicon_tags_8 },
    { 0x1f3b29fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb2a00, SUC_lexicon_tags_17 },
    { 0x9cbb2a01, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5b2a05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x51b2a08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43db2a0e, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bb2a11, SUC_lexicon_tags_11 },
    { 0x2fb2a12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b2a17, SUC_lexicon_tags_7 },
    { 0xd59b2a18, SUC_lexicon_tags_19 },
    { 0x7b7b2a18, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2c5b2a1b, SUC_lexicon_tags_114 },
    { 0x62bb2a1c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03b2a20, SUC_lexicon_tags_21 },
    { 0xc47b2a21, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x213b2a23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39bb2a25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51b2a28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebb2a31, SUC_lexicon_tags_17 },
    { 0x1cdb2a32, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x10fb2a34, SUC_lexicon_tags_38 },
    { 0xc9fb2a34, SUC_lexicon_tags_63 },
    { 0x205b2a35, SUC_lexicon_tags_37 },
    { 0x4a3b2a35, SUC_lexicon_tags_15 },
    { 0x3ffb2a38, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b2a3b, SUC_lexicon_tags_33 },
    { 0xa8bb2a3b, SUC_lexicon_tags_11 },
    { 0x7d1b2a3d, SUC_lexicon_tags_15 },
    { 0x4fb2a3e, SUC_lexicon_tags_19 },
    { 0x391b2a3c, SUC_lexicon_tags_45 },
    { 0x6d3b2a3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83b2a43, SUC_lexicon_tags_26 },
    { 0x539b2a44, SUC_lexicon_tags_37 },
    { 0xcd1b2a45, SUC_lexicon_tags_15 },
    { 0x8adb2a46, SUC_lexicon_tags_12 },
    { 0x561b2a46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5afb2a49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3b2a4d, SUC_lexicon_tags_14 },
    { 0xedbb2a4e, SUC_lexicon_tags_11 },
    { 0x5f1b2a4f, SUC_lexicon_tags_55 },
    { 0x885b2a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf55b2a52, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x203b2a55, SUC_lexicon_tags_38 },
    { 0xb73b2a56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5a3b2a58, SUC_lexicon_tags_33 },
    { 0xe1b2a58, SUC_lexicon_tags_27 },
    { 0xcc5b2a5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9b2a5e, SUC_lexicon_tags_33 },
    { 0xb03b2a5e, SUC_lexicon_tags_37 },
    { 0x323b2a60, SUC_lexicon_tags_17 },
    { 0x67db2a60, SUC_lexicon_tags_69 },
    { 0x111b2a62, SUC_lexicon_tags_11 },
    { 0x537b2a63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x195b2a65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x199b2a68, SUC_lexicon_tags_5 },
    { 0x75bb2a69, SUC_lexicon_tags_61 },
    { 0x161b2a68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83db2a6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9b2a75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fb2a78, SUC_lexicon_tags_30 },
    { 0xfcdb2a78, SUC_lexicon_tags_17 },
    { 0xcadb2a7a, SUC_lexicon_tags_47 },
    { 0x941b2a7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb2a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955b2a82, SUC_lexicon_tags_12 },
    { 0x401b2a83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789b2a88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xea9b2a8a, SUC_lexicon_tags_11 },
    { 0xd0fb2a8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa7bb2a8d, SUC_lexicon_tags_37 },
    { 0x921b2a8e, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x963b2a90, SUC_lexicon_tags_33 },
    { 0xc83b2a90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd0bb2a93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x465b2a95, SUC_lexicon_tags_11 },
    { 0xf09b2a95, SUC_lexicon_tags_29 },
    { 0x7db2a95, SUC_lexicon_tags_25 },
    { 0x98fb2a95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5bb2a9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917b2aa7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491b2ab0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91b2ab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbb2ac3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229b2ac7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xddfb2ac9, SUC_lexicon_tags_7 },
    { 0x6fdb2ac9, SUC_lexicon_tags_14 },
    { 0x817b2acb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f7b2acd, SUC_lexicon_tags_39 },
    { 0x67b2ace, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d1b2ad0, SUC_lexicon_tags_29 },
    { 0xe09b2ad1, SUC_lexicon_tags_1 },
    { 0x631b2ad2, SUC_lexicon_tags_1 },
    { 0x793b2ad2, SUC_lexicon_tags_6 },
    { 0xb85b2ad0, SUC_lexicon_tags_1 },
    { 0x1d7b2ad5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9b2ad8, SUC_lexicon_tags_68 },
    { 0x90fb2ad9, SUC_lexicon_tags_15 },
    { 0x66db2ad9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adb2add, SUC_lexicon_tags_28 },
    { 0x751b2ade, SUC_lexicon_tags_63 },
    { 0xf0fb2adf, SUC_lexicon_tags_17 },
    { 0x901b2ae0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x92bb2ae2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3db2ae8, SUC_lexicon_tags_15 },
    { 0x31b2ae9, SUC_lexicon_tags_25 },
    { 0xd95b2ae8, SUC_lexicon_tags_10 },
    { 0x671b2aeb, SUC_lexicon_tags_65 },
    { 0x3fb2aec, SUC_lexicon_tags_2 },
    { 0xbf9b2aec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5b2af0, SUC_lexicon_tags_29 },
    { 0x8d7b2af0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x435b2af3, SUC_lexicon_tags_60 },
    { 0x8c9b2af4, SUC_lexicon_tags_60 },
    { 0x333b2af4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebb2af8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95b2aff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633b2b07, SUC_lexicon_tags_15 },
    { 0x20db2b08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809b2b0d, SUC_lexicon_tags_63 },
    { 0x785b2b0e, SUC_lexicon_tags_12 },
    { 0xdb9b2b0d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73b2b12, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03b2b19, SUC_lexicon_tags_4 },
    { 0xb1fb2b1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffb2b26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9b2b35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b7b2b37, SUC_lexicon_tags_11 },
    { 0xbb3b2b38, SUC_lexicon_tags_2 },
    { 0xecbb2b39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb09b2b3b, SUC_lexicon_tags_12 },
    { 0x149b2b3b, SUC_lexicon_tags_43 },
    { 0x883b2b3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbb2b43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bb2b47, SUC_lexicon_tags_8 },
    { 0x905b2b47, SUC_lexicon_tags_37 },
    { 0x709b2b49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5cfb2b4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabb2b4d, SUC_lexicon_tags_63 },
    { 0x481b2b4e, SUC_lexicon_tags_16 },
    { 0x7d5b2b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7b2b52, SUC_lexicon_tags_7 },
    { 0x355b2b53, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x425b2b55, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60db2b59, SUC_lexicon_tags_45 },
    { 0xf41b2b59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd47b2b5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59b2b5f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7b2b63, SUC_lexicon_tags_29 },
    { 0x465b2b63, SUC_lexicon_tags_37 },
    { 0x5afb2b63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89b2b68, SUC_lexicon_tags_37 },
    { 0x6c5b2b68, SUC_lexicon_tags_31 },
    { 0xf59b2b68, SUC_lexicon_tags_25 },
    { 0x4f7b2b6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3b2b6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x269b2b70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fb2b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5b2b79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbb2b7c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bb2b83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23b2b87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3b2b8a, SUC_lexicon_tags_7 },
    { 0x84bb2b8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fb2b8e, SUC_lexicon_tags_0 },
    { 0x257b2b8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1b2b96, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bb2b99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe19b2b9b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741b2ba0, SUC_lexicon_tags_45 },
    { 0x589b2ba1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f5b2ba3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcc1b2ba5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x609b2ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x967b2ba9, SUC_lexicon_tags_19 },
    { 0x4b5b2baa, SUC_lexicon_tags_11 },
    { 0xedbb2bab, SUC_lexicon_tags_2 },
    { 0x471b2bac, SUC_lexicon_tags_23 },
    { 0xa2db2bad, SUC_lexicon_tags_17 },
    { 0xb47b2bac, SUC_lexicon_tags_17 },
    { 0x8b7b2bab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bb2bb2, SUC_lexicon_tags_37 },
    { 0x267b2bb3, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb2bba, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9b2bc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5b2bc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9b2bd3, SUC_lexicon_tags_37 },
    { 0x63db2bd4, SUC_lexicon_tags_60 },
    { 0xcadb2bd5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bb2bd8, SUC_lexicon_tags_31 },
    { 0xdb3b2bd8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7b2be2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1b2be5, SUC_lexicon_tags_37 },
    { 0x1c9b2be5, SUC_lexicon_tags_15 },
    { 0x70db2be6, SUC_lexicon_tags_31 },
    { 0xebbb2be8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb97b2bea, SUC_lexicon_tags_45 },
    { 0x5b2bea, SUC_lexicon_tags_42 },
    { 0x9afb2bec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1b2bef, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bb2bf4, SUC_lexicon_tags_0 },
    { 0x8e5b2bf4, SUC_lexicon_tags_17 },
    { 0x151b2bf6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfcbb2bf8, SUC_lexicon_tags_8 },
    { 0xb53b2bf8, SUC_lexicon_tags_11 },
    { 0xac9b2bf9, SUC_lexicon_tags_7 },
    { 0xcfdb2bfb, SUC_lexicon_tags_46 },
    { 0x6abb2bfa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b1b2bfe, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abb2c02, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x72bb2c04, SUC_lexicon_tags_37 },
    { 0xa69b2c05, SUC_lexicon_tags_17 },
    { 0x3f5b2c06, SUC_lexicon_tags_118 },
    { 0xe5b2c06, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x763b2c09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddb2c0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66db2c13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfb2c16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fdb2c18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c5b2c1a, SUC_lexicon_tags_7 },
    { 0x565b2c1a, SUC_lexicon_tags_47 },
    { 0x70db2c1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9b2c2b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe7fb2c2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5b2c30, SUC_lexicon_tags_37 },
    { 0xc51b2c30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3f3b2c33, SUC_lexicon_tags_15 },
    { 0x6bb2c34, SUC_lexicon_tags_49 },
    { 0x4ebb2c33, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bb2c38, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57b2c3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf57b2c3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241b2c44, SUC_lexicon_tags_25 },
    { 0xee3b2c45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c7b2c47, SUC_lexicon_tags_37 },
    { 0x9ffb2c48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3c9b2c4a, SUC_lexicon_tags_30 },
    { 0xe69b2c4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x327b2c4e, SUC_lexicon_tags_8 },
    { 0x65db2c4f, SUC_lexicon_tags_27 },
    { 0x723b2c4f, SUC_lexicon_tags_33 },
    { 0x47b2c51, SUC_lexicon_tags_26 },
    { 0x997b2c4e, SUC_lexicon_tags_12 },
    { 0xbb3b2c53, SUC_lexicon_tags_25 },
    { 0x84db2c51, SUC_lexicon_tags_29 },
    { 0xb55b2c55, SUC_lexicon_tags_11 },
    { 0x217b2c52, SUC_lexicon_tags_17 },
    { 0xab1b2c57, SUC_lexicon_tags_25 },
    { 0x995b2c53, SUC_lexicon_tags_38 },
    { 0xef3b2c59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81db2c5b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333b2c69, SUC_lexicon_tags_7 },
    { 0x9cbb2c6a, SUC_lexicon_tags_17 },
    { 0x8fbb2c6b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x167b2c6d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1b2c70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x331b2c73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3db2c75, SUC_lexicon_tags_27 },
    { 0xb1b2c76, SUC_lexicon_tags_8 },
    { 0x5f3b2c77, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xddfb2c7a, SUC_lexicon_tags_14 },
    { 0x421b2c7a, SUC_lexicon_tags_15 },
    { 0xb1db2c7c, SUC_lexicon_tags_37 },
    { 0x599b2c7b, SUC_lexicon_tags_12 },
    { 0x73b2c7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1b2c80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb2c83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1b2c88, SUC_lexicon_tags_29 },
    { 0x653b2c89, SUC_lexicon_tags_25 },
    { 0xa29b2c8a, SUC_lexicon_tags_11 },
    { 0xefb2c89, SUC_lexicon_tags_9 },
    { 0xa6bb2c8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdb2c90, SUC_lexicon_tags_7 },
    { 0x185b2c91, SUC_lexicon_tags_15 },
    { 0x597b2c92, SUC_lexicon_tags_19 },
    { 0xc39b2c92, SUC_lexicon_tags_25 },
    { 0x933b2c94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x45b2c96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fb2c9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87db2ca8, SUC_lexicon_tags_39 },
    { 0x58db2ca9, SUC_lexicon_tags_1 },
    { 0x91fb2ca8, SUC_lexicon_tags_55 },
    { 0x7a1b2cab, SUC_lexicon_tags_23 },
    { 0xa2bb2cac, SUC_lexicon_tags_14 },
    { 0x633b2cac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcabb2caf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29b2cb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa27b2cb5, SUC_lexicon_tags_63 },
    { 0xe73b2cb6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b2cb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7fbb2cbb, SUC_lexicon_tags_15 },
    { 0x7e3b2cbc, SUC_lexicon_tags_17 },
    { 0xfd7b2cbc, SUC_lexicon_tags_37 },
    { 0x145b2cbe, SUC_lexicon_tags_351 },
    { 0x8b5b2cbf, SUC_lexicon_tags_6 },
    { 0x723b2cbc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7b2cc5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5b2ccb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95b2ccf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bb2cd2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x63bb2cd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc7b2cd6, SUC_lexicon_tags_12 },
    { 0xccfb2cd7, SUC_lexicon_tags_1 },
    { 0xeffb2cd8, SUC_lexicon_tags_10 },
    { 0x831b2cd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x453b2cdb, SUC_lexicon_tags_17 },
    { 0x8c7b2cdc, SUC_lexicon_tags_91 },
    { 0x2ebb2cdd, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x871b2cdf, SUC_lexicon_tags_23 },
    { 0x4bfb2ce0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x19b2ce3, SUC_lexicon_tags_17 },
    { 0x23fb2ce4, SUC_lexicon_tags_4 },
    { 0x415b2ce4, SUC_lexicon_tags_11 },
    { 0x153b2ce6, SUC_lexicon_tags_104 },
    { 0x4e7b2ce4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8adb2ce9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49b2ced, SUC_lexicon_tags_45 },
    { 0x3e1b2cee, SUC_lexicon_tags_9 },
    { 0xc05b2cef, SUC_lexicon_tags_30 },
    { 0xe9fb2ced, SUC_lexicon_tags_45 },
    { 0x715b2cf1, SUC_lexicon_tags_11 },
    { 0xbabb2cf1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01b2cf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8fdb2cf7, SUC_lexicon_tags_4 },
    { 0x19db2cf8, SUC_lexicon_tags_11 },
    { 0xe9bb2cf9, SUC_lexicon_tags_11 },
    { 0x6dfb2cfa, SUC_lexicon_tags_19 },
    { 0xce3b2cf9, SUC_lexicon_tags_22 },
    { 0x21db2cfc, SUC_lexicon_tags_47 },
    { 0xa75b2cfa, SUC_lexicon_tags_60 },
    { 0xa2db2cfe, SUC_lexicon_tags_1 },
    { 0x971b2cfb, SUC_lexicon_tags_60 },
    { 0x29db2d00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69b2d06, SUC_lexicon_tags_15 },
    { 0x1cdb2d07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5b2d0a, SUC_lexicon_tags_25 },
    { 0xd5fb2d0b, SUC_lexicon_tags_33 },
    { 0xef3b2d0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54bb2d0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5b2d12, SUC_lexicon_tags_28 },
    { 0xda5b2d12, SUC_lexicon_tags_47 },
    { 0xf3bb2d13, SUC_lexicon_tags_48 },
    { 0x3edb2d15, SUC_lexicon_tags_37 },
    { 0x9bdb2d15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcffb2d18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0fb2d1a, SUC_lexicon_tags_6 },
    { 0x17bb2d1a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677b2d1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95b2d22, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xffbb2d24, SUC_lexicon_tags_35 },
    { 0xa2bb2d25, SUC_lexicon_tags_0 },
    { 0xcc7b2d26, SUC_lexicon_tags_15 },
    { 0x2adb2d27, SUC_lexicon_tags_57 },
    { 0xc0db2d25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9b2d2a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x22fb2d2c, SUC_lexicon_tags_9 },
    { 0xca7b2d2c, SUC_lexicon_tags_21 },
    { 0xf57b2d2e, SUC_lexicon_tags_45 },
    { 0x31b2d2f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa5fb2d31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x181b2d33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64db2d36, SUC_lexicon_tags_17 },
    { 0x94bb2d37, SUC_lexicon_tags_12 },
    { 0x459b2d36, SUC_lexicon_tags_29 },
    { 0xe55b2d37, SUC_lexicon_tags_4 },
    { 0x133b2d39, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x97db2d3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6f9b2d3e, SUC_lexicon_tags_15 },
    { 0xc75b2d3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb2d42, SUC_lexicon_tags_12 },
    { 0x1b1b2d43, SUC_lexicon_tags_25 },
    { 0x8ddb2d44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911b2d4d, SUC_lexicon_tags_235 },
    { 0x92db2d4d, SUC_lexicon_tags_39 },
    { 0x5c1b2d4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd17b2d51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7b2d54, SUC_lexicon_tags_7 },
    { 0x3cfb2d55, SUC_lexicon_tags_25 },
    { 0x40db2d56, SUC_lexicon_tags_42 },
    { 0x2dfb2d55, SUC_lexicon_tags_6 },
    { 0x5db2d56, SUC_lexicon_tags_65 },
    { 0x15b2d59, SUC_lexicon_tags_19 },
    { 0xbf9b2d59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x84bb2d5c, SUC_lexicon_tags_38 },
    { 0x287b2d5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x381b2d5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb95b2d61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x25db2d64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45b2d68, SUC_lexicon_tags_7 },
    { 0x81bb2d69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147b2d70, SUC_lexicon_tags_25 },
    { 0xc31b2d71, SUC_lexicon_tags_10 },
    { 0x2a5b2d72, SUC_lexicon_tags_1 },
    { 0x9abb2d70, SUC_lexicon_tags_12 },
    { 0x611b2d73, SUC_lexicon_tags_37 },
    { 0xfffb2d75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafb2d78, SUC_lexicon_tags_79 },
    { 0xa05b2d79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a9b2d7b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9d7b2d7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313b2d82, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3b2d89, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adb2d8e, SUC_lexicon_tags_15 },
    { 0x8f5b2d8f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfb2d92, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebb2d95, SUC_lexicon_tags_39 },
    { 0x273b2d95, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1b2d99, SUC_lexicon_tags_25 },
    { 0x7f3b2d9a, SUC_lexicon_tags_1 },
    { 0xd17b2d9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x94bb2d9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5b2da0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283b2da7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x44fb2da9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x56fb2dab, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x8bdb2dad, SUC_lexicon_tags_25 },
    { 0x373b2dae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddb2db5, SUC_lexicon_tags_10 },
    { 0x6a5b2db6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x997b2db8, SUC_lexicon_tags_30 },
    { 0xc97b2db9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bb2dbc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9b2dbe, SUC_lexicon_tags_37 },
    { 0xd1db2dbf, SUC_lexicon_tags_1 },
    { 0x75bb2dbe, SUC_lexicon_tags_0 },
    { 0x535b2dc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189b2dc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23b2dcd, SUC_lexicon_tags_46 },
    { 0xe41b2dce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37b2dd5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb4db2dd7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b2ddb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbb2de3, SUC_lexicon_tags_15 },
    { 0x831b2de4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb01b2de6, SUC_lexicon_tags_39 },
    { 0x979b2de6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3b2deb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefb2df3, SUC_lexicon_tags_49 },
    { 0x471b2df4, SUC_lexicon_tags_11 },
    { 0x2f7b2df5, SUC_lexicon_tags_25 },
    { 0xdfbb2df6, SUC_lexicon_tags_47 },
    { 0x31b2df6, SUC_lexicon_tags_25 },
    { 0xc11b2df6, SUC_lexicon_tags_0 },
    { 0x939b2df9, SUC_lexicon_tags_43 },
    { 0x5afb2df7, SUC_lexicon_tags_7 },
    { 0xf7bb2df7, SUC_lexicon_tags_0 },
    { 0x3afb2df8, SUC_lexicon_tags_17 },
    { 0xf3b2df4, SUC_lexicon_tags_6 },
    { 0x137b2df9, SUC_lexicon_tags_14 },
    { 0xeb1b2dff, SUC_lexicon_tags_12 },
    { 0xdedb2dfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3b2e07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fb2e0e, SUC_lexicon_tags_17 },
    { 0xadfb2e0e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61b2e16, SUC_lexicon_tags_29 },
    { 0xd97b2e17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b2e1a, SUC_lexicon_tags_15 },
    { 0x7e5b2e1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99b2e1d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebb2e20, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57b2e27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x19bb2e29, SUC_lexicon_tags_14 },
    { 0x701b2e2a, SUC_lexicon_tags_64 },
    { 0x209b2e29, SUC_lexicon_tags_29 },
    { 0xe4db2e2c, SUC_lexicon_tags_44 },
    { 0x9e7b2e2c, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11b2e31, SUC_lexicon_tags_1 },
    { 0x9edb2e32, SUC_lexicon_tags_27 },
    { 0x3cbb2e33, SUC_lexicon_tags_39 },
    { 0xac9b2e32, SUC_lexicon_tags_92 },
    { 0x107b2e35, SUC_lexicon_tags_7 },
    { 0x9d3b2e35, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47b2e40, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6cdb2e42, SUC_lexicon_tags_7 },
    { 0x9b3b2e42, SUC_lexicon_tags_17 },
    { 0x429b2e44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfc3b2e46, SUC_lexicon_tags_1 },
    { 0xc4db2e47, SUC_lexicon_tags_68 },
    { 0x2a9b2e48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907b2e4d, SUC_lexicon_tags_26 },
    { 0x415b2e4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b2e51, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7b2e5c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x357b2e5e, SUC_lexicon_tags_60 },
    { 0xda9b2e5f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b5b2e61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fb2e64, SUC_lexicon_tags_17 },
    { 0x663b2e64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x593b2e67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x437b2e6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6db2e70, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efb2e76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edb2e81, SUC_lexicon_tags_29 },
    { 0xb3fb2e82, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x24db2e85, SUC_lexicon_tags_17 },
    { 0xb39b2e86, SUC_lexicon_tags_0 },
    { 0xa4db2e85, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcf3b2e89, SUC_lexicon_tags_37 },
    { 0x17fb2e8a, SUC_lexicon_tags_15 },
    { 0xe47b2e8b, SUC_lexicon_tags_6 },
    { 0x135b2e8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd27b2e8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59b2e93, SUC_lexicon_tags_25 },
    { 0xe2db2e94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9b2e97, SUC_lexicon_tags_17 },
    { 0xfe3b2e98, SUC_lexicon_tags_11 },
    { 0xf33b2e98, SUC_lexicon_tags_12 },
    { 0xfc3b2e9a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01b2e9e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x7dfb2ea0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b2ea3, SUC_lexicon_tags_12 },
    { 0x4dbb2ea4, SUC_lexicon_tags_63 },
    { 0xfebb2ea5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1b2eac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81b2eb0, SUC_lexicon_tags_0 },
    { 0xf15b2eb1, SUC_lexicon_tags_12 },
    { 0xde9b2eb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb7b2eb4, SUC_lexicon_tags_7 },
    { 0x8cbb2eb5, SUC_lexicon_tags_2 },
    { 0x939b2eb6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efb2ec3, SUC_lexicon_tags_12 },
    { 0x48fb2ec4, SUC_lexicon_tags_43 },
    { 0x3bfb2ec5, SUC_lexicon_tags_11 },
    { 0xbf7b2ec4, SUC_lexicon_tags_11 },
    { 0x489b2ec7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d9b2ec9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57fb2ecb, SUC_lexicon_tags_7 },
    { 0x16fb2ecb, SUC_lexicon_tags_77 },
    { 0x813b2ecc, SUC_lexicon_tags_30 },
    { 0xbbb2ecc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xff5b2ed0, SUC_lexicon_tags_40 },
    { 0xcd1b2ed1, SUC_lexicon_tags_60 },
    { 0x7d9b2ed1, SUC_lexicon_tags_43 },
    { 0x84db2ed3, SUC_lexicon_tags_15 },
    { 0x1ffb2ed4, SUC_lexicon_tags_37 },
    { 0x767b2ed5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9b2ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b2edb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbb2ede, SUC_lexicon_tags_16 },
    { 0x6ddb2ede, SUC_lexicon_tags_7 },
    { 0xc51b2edf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb2ee4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdb2ee8, SUC_lexicon_tags_11 },
    { 0xda1b2ee8, SUC_lexicon_tags_31 },
    { 0xeb1b2eea, SUC_lexicon_tags_82 },
    { 0x6a3b2eea, SUC_lexicon_tags_229 },
    { 0xed1b2eec, SUC_lexicon_tags_39 },
    { 0xcf7b2ee9, SUC_lexicon_tags_38 },
    { 0x36bb2eed, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805b2ef2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bb2ef5, SUC_lexicon_tags_49 },
    { 0x717b2ef6, SUC_lexicon_tags_11 },
    { 0x66fb2ef5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd79b2ef9, SUC_lexicon_tags_56 },
    { 0x88db2efa, SUC_lexicon_tags_77 },
    { 0x8d9b2ef9, SUC_lexicon_tags_45 },
    { 0xfcdb2efb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe05b2efe, SUC_lexicon_tags_60 },
    { 0xcc9b2eff, SUC_lexicon_tags_31 },
    { 0x9cbb2f00, SUC_lexicon_tags_33 },
    { 0x1bfb2f00, SUC_lexicon_tags_11 },
    { 0xeddb2f02, SUC_lexicon_tags_68 },
    { 0x907b2f03, SUC_lexicon_tags_17 },
    { 0x729b2f03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5b2f09, SUC_lexicon_tags_31 },
    { 0x17fb2f0a, SUC_lexicon_tags_38 },
    { 0xdd5b2f0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93b2f0f, SUC_lexicon_tags_82 },
    { 0x22fb2f0f, SUC_lexicon_tags_45 },
    { 0x63b2f0f, SUC_lexicon_tags_33 },
    { 0xa63b2f12, SUC_lexicon_tags_253 },
    { 0x271b2f13, SUC_lexicon_tags_11 },
    { 0x7a7b2f14, SUC_lexicon_tags_96 },
    { 0x829b2f15, SUC_lexicon_tags_33 },
    { 0xf0fb2f15, SUC_lexicon_tags_31 },
    { 0x881b2f15, SUC_lexicon_tags_9 },
    { 0x9d3b2f15, SUC_lexicon_tags_43 },
    { 0xc2fb2f15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9b2f20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abb2f23, SUC_lexicon_tags_2 },
    { 0xd4db2f23, SUC_lexicon_tags_35 },
    { 0x761b2f23, SUC_lexicon_tags_17 },
    { 0xa2bb2f25, SUC_lexicon_tags_37 },
    { 0x2cfb2f23, SUC_lexicon_tags_37 },
    { 0xc37b2f25, SUC_lexicon_tags_10 },
    { 0x961b2f24, SUC_lexicon_tags_10 },
    { 0x3b2f2a, SUC_lexicon_tags_30 },
    { 0xcf9b2f2b, SUC_lexicon_tags_3 },
    { 0xa5b2f2c, SUC_lexicon_tags_2 },
    { 0x257b2f24, SUC_lexicon_tags_47 },
    { 0x5cfb2f26, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfb2f31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcd7b2f33, SUC_lexicon_tags_1 },
    { 0x71b2f33, SUC_lexicon_tags_19 },
    { 0x649b2f35, SUC_lexicon_tags_11 },
    { 0x9b9b2f36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63b2f39, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b2f3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59b2f40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdb2f43, SUC_lexicon_tags_255 },
    { 0x54bb2f44, SUC_lexicon_tags_12 },
    { 0x189b2f45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabb2f51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dbb2f53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x985b2f56, SUC_lexicon_tags_15 },
    { 0x999b2f56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12db2f5b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7b2f5e, SUC_lexicon_tags_15 },
    { 0x213b2f5e, SUC_lexicon_tags_37 },
    { 0xddfb2f60, SUC_lexicon_tags_7 },
    { 0xff1b2f5e, SUC_lexicon_tags_7 },
    { 0x799b2f62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347b2f6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb2f73, SUC_lexicon_tags_238 },
    { 0x461b2f74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x707b2f77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bb2f81, SUC_lexicon_tags_33 },
    { 0xa6db2f81, SUC_lexicon_tags_0 },
    { 0x8e3b2f83, SUC_lexicon_tags_26 },
    { 0xd1b2f84, SUC_lexicon_tags_55 },
    { 0xd0bb2f83, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65b2f8d, SUC_lexicon_tags_17 },
    { 0x257b2f8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffb2f93, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbb2f98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b9b2f9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edb2f9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd77b2f9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5b2fa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191b2fae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bb2fb2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3bb2fb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fb2fbc, SUC_lexicon_tags_7 },
    { 0xc33b2fbd, SUC_lexicon_tags_15 },
    { 0x8b7b2fbd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9db2fc2, SUC_lexicon_tags_17 },
    { 0x121b2fc3, SUC_lexicon_tags_63 },
    { 0xe73b2fc4, SUC_lexicon_tags_11 },
    { 0x40db2fc5, SUC_lexicon_tags_37 },
    { 0x4b3b2fc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bb2fcb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35b2fd2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd19b2fd4, SUC_lexicon_tags_17 },
    { 0x5d9b2fd5, SUC_lexicon_tags_25 },
    { 0xf3fb2fd4, SUC_lexicon_tags_34 },
    { 0xbc9b2fd6, SUC_lexicon_tags_56 },
    { 0x8a3b2fd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ffb2fda, SUC_lexicon_tags_1 },
    { 0xafdb2fdb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdb2fde, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xe1bb2fe0, SUC_lexicon_tags_14 },
    { 0x76bb2fe0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1b2fe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5b2feb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x23db2fee, SUC_lexicon_tags_7 },
    { 0xbe5b2fee, SUC_lexicon_tags_26 },
    { 0x2b3b2ff0, SUC_lexicon_tags_1 },
    { 0x319b2ff1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x473b2ff4, SUC_lexicon_tags_11 },
    { 0x9a9b2ff4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfd3b2ff7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1b2ffa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2bfb2ffc, SUC_lexicon_tags_8 },
    { 0x503b2ffd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a7b2fff, SUC_lexicon_tags_14 },
    { 0x86db2fff, SUC_lexicon_tags_6 },
    { 0xa9fb3001, SUC_lexicon_tags_7 },
    { 0xc5b3002, SUC_lexicon_tags_43 },
    { 0xd13b3003, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x239b3005, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83b3010, SUC_lexicon_tags_11 },
    { 0x8b9b3011, SUC_lexicon_tags_1 },
    { 0xebdb3010, SUC_lexicon_tags_4 },
    { 0xc03b3012, SUC_lexicon_tags_117 },
    { 0xf4bb3014, SUC_lexicon_tags_30 },
    { 0xbe5b3015, SUC_lexicon_tags_26 },
    { 0x2e3b3016, SUC_lexicon_tags_14 },
    { 0x84fb3016, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5a1b3019, SUC_lexicon_tags_29 },
    { 0x7ddb301a, SUC_lexicon_tags_12 },
    { 0x2dbb301b, SUC_lexicon_tags_23 },
    { 0x9efb301c, SUC_lexicon_tags_3 },
    { 0x24db301b, SUC_lexicon_tags_47 },
    { 0x677b301d, SUC_lexicon_tags_37 },
    { 0x227b301a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6c9b3021, SUC_lexicon_tags_19 },
    { 0x8e5b3022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb47b3024, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27b3027, SUC_lexicon_tags_88 },
    { 0x55b3028, SUC_lexicon_tags_7 },
    { 0xb7bb3029, SUC_lexicon_tags_1 },
    { 0xe9b3028, SUC_lexicon_tags_25 },
    { 0x55bb302b, SUC_lexicon_tags_25 },
    { 0x815b302c, SUC_lexicon_tags_68 },
    { 0x331b302d, SUC_lexicon_tags_11 },
    { 0xc77b302e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b3031, SUC_lexicon_tags_35 },
    { 0xa9bb3031, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb0bb3034, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb8fb3036, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x585b3038, SUC_lexicon_tags_100 },
    { 0x83db3038, SUC_lexicon_tags_17 },
    { 0x4d1b3039, SUC_lexicon_tags_63 },
    { 0xa83b303b, SUC_lexicon_tags_48 },
    { 0xbf9b303b, SUC_lexicon_tags_104 },
    { 0xbbb303d, SUC_lexicon_tags_30 },
    { 0xb75b303d, SUC_lexicon_tags_27 },
    { 0x6f5b303f, SUC_lexicon_tags_14 },
    { 0x655b3040, SUC_lexicon_tags_37 },
    { 0xd0bb3038, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1b3045, SUC_lexicon_tags_25 },
    { 0xe09b3046, SUC_lexicon_tags_59 },
    { 0xd49b3047, SUC_lexicon_tags_11 },
    { 0x4d5b3048, SUC_lexicon_tags_38 },
    { 0x9bfb3047, SUC_lexicon_tags_15 },
    { 0x3b7b304a, SUC_lexicon_tags_49 },
    { 0x3a1b304a, SUC_lexicon_tags_25 },
    { 0x4adb304b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927b3050, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x937b3052, SUC_lexicon_tags_25 },
    { 0x2a7b3053, SUC_lexicon_tags_23 },
    { 0xdb7b3053, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613b3059, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5b305c, SUC_lexicon_tags_47 },
    { 0x19bb305d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495b3061, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x295b3063, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247b3069, SUC_lexicon_tags_3 },
    { 0xc39b306a, SUC_lexicon_tags_39 },
    { 0x651b306b, SUC_lexicon_tags_12 },
    { 0x495b306c, SUC_lexicon_tags_63 },
    { 0x697b306c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x267b306f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601b3073, SUC_lexicon_tags_31 },
    { 0xf35b3073, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10db307b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x589b307e, SUC_lexicon_tags_10 },
    { 0xbe5b307f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd3b3081, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x33b3083, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5b3086, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7b308f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627b3093, SUC_lexicon_tags_47 },
    { 0x9b3b3094, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcd3b3096, SUC_lexicon_tags_62 },
    { 0xb6fb3097, SUC_lexicon_tags_4 },
    { 0xe83b3098, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7b309d, SUC_lexicon_tags_0 },
    { 0x4edb309e, SUC_lexicon_tags_2 },
    { 0xa89b309e, SUC_lexicon_tags_25 },
    { 0x2cdb30a0, SUC_lexicon_tags_23 },
    { 0xa59b30a1, SUC_lexicon_tags_27 },
    { 0xd13b30a1, SUC_lexicon_tags_37 },
    { 0x6a1b309e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf9bb30a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb3b30a7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641b30ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1db30ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791b30b5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67b30b8, SUC_lexicon_tags_26 },
    { 0xdc5b30b9, SUC_lexicon_tags_15 },
    { 0x9c1b30b9, SUC_lexicon_tags_1 },
    { 0x721b30ba, SUC_lexicon_tags_37 },
    { 0x86bb30b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ddb30be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7b30c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa65b30c3, SUC_lexicon_tags_37 },
    { 0x609b30c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5b30cb, SUC_lexicon_tags_37 },
    { 0xccfb30cc, SUC_lexicon_tags_9 },
    { 0xa75b30cd, SUC_lexicon_tags_37 },
    { 0x147b30cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fb30d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x203b30d5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679b30dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9db30e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc93b30e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839b30e8, SUC_lexicon_tags_2 },
    { 0x9d5b30e8, SUC_lexicon_tags_26 },
    { 0xfa5b30e9, SUC_lexicon_tags_7 },
    { 0xa37b30eb, SUC_lexicon_tags_47 },
    { 0x2cdb30ec, SUC_lexicon_tags_30 },
    { 0x301b30ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89b30f0, SUC_lexicon_tags_60 },
    { 0xd63b30f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x695b30f3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x233b30f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bb30f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x265b30fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569b30fe, SUC_lexicon_tags_47 },
    { 0x7bbb30fe, SUC_lexicon_tags_21 },
    { 0xf63b3100, SUC_lexicon_tags_15 },
    { 0xba3b3101, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddb3105, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbb310b, SUC_lexicon_tags_19 },
    { 0xafdb310c, SUC_lexicon_tags_15 },
    { 0x309b310d, SUC_lexicon_tags_13 },
    { 0x26db310d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebb3112, SUC_lexicon_tags_45 },
    { 0x731b3113, SUC_lexicon_tags_9 },
    { 0xbd9b3114, SUC_lexicon_tags_60 },
    { 0x985b3113, SUC_lexicon_tags_25 },
    { 0xcddb3116, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8fb3118, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899b3122, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb7b3124, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1b3127, SUC_lexicon_tags_45 },
    { 0x8efb3127, SUC_lexicon_tags_7 },
    { 0xe6fb3128, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x147b312b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5b3137, SUC_lexicon_tags_1 },
    { 0x515b3138, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9b313c, SUC_lexicon_tags_1 },
    { 0x1d5b313d, SUC_lexicon_tags_12 },
    { 0xa21b313c, SUC_lexicon_tags_15 },
    { 0x669b313e, SUC_lexicon_tags_25 },
    { 0xd1db3140, SUC_lexicon_tags_17 },
    { 0xebbb313f, SUC_lexicon_tags_7 },
    { 0x483b3142, SUC_lexicon_tags_72 },
    { 0xbe3b3143, SUC_lexicon_tags_87 },
    { 0x7e3b3142, SUC_lexicon_tags_19 },
    { 0x5d5b3141, SUC_lexicon_tags_12 },
    { 0x6e7b313e, SUC_lexicon_tags_55 },
    { 0x423b3143, SUC_lexicon_tags_37 },
    { 0x1abb3148, SUC_lexicon_tags_38 },
    { 0x4f9b3149, SUC_lexicon_tags_12 },
    { 0xc45b314a, SUC_lexicon_tags_31 },
    { 0x6e1b3147, SUC_lexicon_tags_17 },
    { 0x6c3b314c, SUC_lexicon_tags_109 },
    { 0x761b314c, SUC_lexicon_tags_1 },
    { 0x28bb3148, SUC_lexicon_tags_7 },
    { 0x3a9b3141, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x81fb3151, SUC_lexicon_tags_4 },
    { 0x2f7b3151, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfb315b, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15b315f, SUC_lexicon_tags_63 },
    { 0x159b3160, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91b3163, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x727b3165, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xec9b3167, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x34bb316a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12db316f, SUC_lexicon_tags_11 },
    { 0xea5b316f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bb3173, SUC_lexicon_tags_17 },
    { 0xd95b3174, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x325b3176, SUC_lexicon_tags_114 },
    { 0xa7db3177, SUC_lexicon_tags_12 },
    { 0x867b3177, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97b317e, SUC_lexicon_tags_32 },
    { 0xed3b317f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9b3182, SUC_lexicon_tags_0 },
    { 0xdbbb3182, SUC_lexicon_tags_25 },
    { 0xf39b3182, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5b3187, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bb318f, SUC_lexicon_tags_15 },
    { 0x7edb3190, SUC_lexicon_tags_15 },
    { 0x859b3190, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99b3195, SUC_lexicon_tags_8 },
    { 0x4b1b3196, SUC_lexicon_tags_19 },
    { 0x8d9b3196, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abb319e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe89b31a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9afb31a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e5b31a4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b31b1, SUC_lexicon_tags_43 },
    { 0x549b31b2, SUC_lexicon_tags_17 },
    { 0x48db31b3, SUC_lexicon_tags_41 },
    { 0x2cdb31b2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdbfb31b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1b31bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa65b31bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b31c5, SUC_lexicon_tags_17 },
    { 0x809b31c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29b31c9, SUC_lexicon_tags_37 },
    { 0x593b31ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x795b31cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbe1b31ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91b31d5, SUC_lexicon_tags_11 },
    { 0xdfb31d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bb31da, SUC_lexicon_tags_37 },
    { 0x6ebb31db, SUC_lexicon_tags_33 },
    { 0x2cdb31db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7b31df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x501b31e1, SUC_lexicon_tags_30 },
    { 0xc83b31e2, SUC_lexicon_tags_2 },
    { 0xf4db31e3, SUC_lexicon_tags_35 },
    { 0x941b31e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc87b31e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3b31e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83fb31ec, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x581b31ef, SUC_lexicon_tags_47 },
    { 0x271b31f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11b31f4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5b31f9, SUC_lexicon_tags_43 },
    { 0x531b31fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbbb31fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffb31fe, SUC_lexicon_tags_17 },
    { 0x59b31fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8db3201, SUC_lexicon_tags_37 },
    { 0xb81b3202, SUC_lexicon_tags_60 },
    { 0x841b3203, SUC_lexicon_tags_23 },
    { 0x829b3204, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbb320d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x26fb320f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497b3213, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x90bb3215, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7b321a, SUC_lexicon_tags_33 },
    { 0x155b321a, SUC_lexicon_tags_11 },
    { 0x25fb321a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5b3221, SUC_lexicon_tags_48 },
    { 0x659b3221, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fb3227, SUC_lexicon_tags_17 },
    { 0x43b3227, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x513b322b, SUC_lexicon_tags_16 },
    { 0x63b322c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5b3230, SUC_lexicon_tags_39 },
    { 0x9f5b3231, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x759b3233, SUC_lexicon_tags_2 },
    { 0x109b3234, SUC_lexicon_tags_17 },
    { 0xadb3235, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x953b3238, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcddb323a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x711b323c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bb3240, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3b3245, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05b3252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1b3255, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b3257, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x515b3259, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3b3261, SUC_lexicon_tags_30 },
    { 0x281b3261, SUC_lexicon_tags_1 },
    { 0xd8bb3262, SUC_lexicon_tags_11 },
    { 0x4f3b3264, SUC_lexicon_tags_37 },
    { 0x44fb3264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89b3267, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca9b3269, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a7b326b, SUC_lexicon_tags_16 },
    { 0xf01b326c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb33b326e, SUC_lexicon_tags_33 },
    { 0xf6bb326f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbedb3271, SUC_lexicon_tags_47 },
    { 0x383b3272, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x291b3274, SUC_lexicon_tags_28 },
    { 0xb19b3275, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173b327e, SUC_lexicon_tags_63 },
    { 0xe6db327e, SUC_lexicon_tags_79 },
    { 0x497b3280, SUC_lexicon_tags_7 },
    { 0x265b3281, SUC_lexicon_tags_12 },
    { 0xdcbb327f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x765b3284, SUC_lexicon_tags_11 },
    { 0x63bb3284, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7d1b3287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86fb328a, SUC_lexicon_tags_16 },
    { 0xd7b328a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9efb328e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b3298, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551b329e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7b32a1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedb32a5, SUC_lexicon_tags_0 },
    { 0xf49b32a6, SUC_lexicon_tags_38 },
    { 0x50fb32a6, SUC_lexicon_tags_7 },
    { 0x13fb32a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927b32ac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b32b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e3b32b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9b32bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbb32c3, SUC_lexicon_tags_17 },
    { 0x303b32c3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x327b32c7, SUC_lexicon_tags_8 },
    { 0xd33b32c7, SUC_lexicon_tags_27 },
    { 0x1fbb32c9, SUC_lexicon_tags_25 },
    { 0xf8fb32ca, SUC_lexicon_tags_37 },
    { 0xd67b32c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x441b32ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247b32d2, SUC_lexicon_tags_60 },
    { 0x3ffb32d2, SUC_lexicon_tags_14 },
    { 0x5bfb32d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e9b32d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x809b32d9, SUC_lexicon_tags_19 },
    { 0x487b32d9, SUC_lexicon_tags_30 },
    { 0xd39b32db, SUC_lexicon_tags_3 },
    { 0xcefb32dc, SUC_lexicon_tags_25 },
    { 0xc1b32dd, SUC_lexicon_tags_8 },
    { 0x5ffb32de, SUC_lexicon_tags_60 },
    { 0xc9bb32da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5b32e3, SUC_lexicon_tags_62 },
    { 0x69fb32e3, SUC_lexicon_tags_7 },
    { 0x265b32e5, SUC_lexicon_tags_56 },
    { 0xab1b32e5, SUC_lexicon_tags_46 },
    { 0xb17b32e6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7b32ea, SUC_lexicon_tags_2 },
    { 0x9efb32eb, SUC_lexicon_tags_14 },
    { 0xf55b32ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5b32ef, SUC_lexicon_tags_14 },
    { 0x531b32f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637b32f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb1db32f7, SUC_lexicon_tags_2 },
    { 0x13bb32f8, SUC_lexicon_tags_37 },
    { 0xce7b32f8, SUC_lexicon_tags_47 },
    { 0x19b32f8, SUC_lexicon_tags_31 },
    { 0x541b32fb, SUC_lexicon_tags_12 },
    { 0xec9b32fc, SUC_lexicon_tags_30 },
    { 0x63bb32fb, SUC_lexicon_tags_25 },
    { 0x97fb32fc, SUC_lexicon_tags_2 },
    { 0x88db32ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fb3303, SUC_lexicon_tags_25 },
    { 0x23b3304, SUC_lexicon_tags_4 },
    { 0x74db3305, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x423b3308, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xfe5b330a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3b330e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59b3316, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefb331e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb49b3320, SUC_lexicon_tags_11 },
    { 0xdebb3321, SUC_lexicon_tags_47 },
    { 0xd93b3322, SUC_lexicon_tags_8 },
    { 0x7db3321, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4fb3325, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bb3328, SUC_lexicon_tags_11 },
    { 0x91b3328, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fb332c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x965b332e, SUC_lexicon_tags_37 },
    { 0x1afb332f, SUC_lexicon_tags_26 },
    { 0x6efb3330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c5b3332, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167b3338, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebb333b, SUC_lexicon_tags_60 },
    { 0xbdbb333c, SUC_lexicon_tags_25 },
    { 0xfc7b333d, SUC_lexicon_tags_8 },
    { 0xde7b333e, SUC_lexicon_tags_14 },
    { 0xbf7b333e, SUC_lexicon_tags_17 },
    { 0x34bb333e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x653b3342, SUC_lexicon_tags_17 },
    { 0x487b3343, SUC_lexicon_tags_11 },
    { 0x9f5b3344, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c7b3346, SUC_lexicon_tags_25 },
    { 0xa9b3347, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9db3349, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0xad1b334b, SUC_lexicon_tags_19 },
    { 0xfd5b334c, SUC_lexicon_tags_6 },
    { 0x305b334b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7b3353, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3b335c, SUC_lexicon_tags_23 },
    { 0x893b335d, SUC_lexicon_tags_7 },
    { 0x6a9b335e, SUC_lexicon_tags_25 },
    { 0x237b335d, SUC_lexicon_tags_37 },
    { 0xcc1b3360, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb3364, SUC_lexicon_tags_66 },
    { 0xe91b3365, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739b336a, SUC_lexicon_tags_1 },
    { 0x5afb336b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5b336f, SUC_lexicon_tags_0 },
    { 0x17b3370, SUC_lexicon_tags_25 },
    { 0xd23b3370, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x957b3374, SUC_lexicon_tags_43 },
    { 0x957b3375, SUC_lexicon_tags_65 },
    { 0xab7b3374, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5b337b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeddb337d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbb3381, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x715b3383, SUC_lexicon_tags_14 },
    { 0xc3bb3384, SUC_lexicon_tags_26 },
    { 0x1e3b3385, SUC_lexicon_tags_38 },
    { 0xf85b3386, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb71b3388, SUC_lexicon_tags_14 },
    { 0xe01b3388, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bb338d, SUC_lexicon_tags_37 },
    { 0x6bdb338d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbb7b3390, SUC_lexicon_tags_27 },
    { 0xf33b3390, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69b3395, SUC_lexicon_tags_37 },
    { 0x7c9b3396, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x543b3398, SUC_lexicon_tags_39 },
    { 0xcafb3398, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83db339d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x263b339f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x757b33a1, SUC_lexicon_tags_2 },
    { 0xdc9b33a2, SUC_lexicon_tags_12 },
    { 0xf4bb33a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3edb33a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x233b33a7, SUC_lexicon_tags_37 },
    { 0x5db33a8, SUC_lexicon_tags_37 },
    { 0xfc7b33a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b33ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bb33b1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3c3b33b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bb33b8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x957b33ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bb33c7, SUC_lexicon_tags_0 },
    { 0x88db33c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321b33cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42fb33cf, SUC_lexicon_tags_37 },
    { 0xe7bb33cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa69b33d2, SUC_lexicon_tags_1 },
    { 0x961b33d2, SUC_lexicon_tags_15 },
    { 0x82db33d4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9b33d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bb33e1, SUC_lexicon_tags_38 },
    { 0x155b33e2, SUC_lexicon_tags_17 },
    { 0x8a9b33e1, SUC_lexicon_tags_0 },
    { 0xf89b33e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7b33e9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257b33ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd47b33f0, SUC_lexicon_tags_12 },
    { 0xe6bb33f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1b33f4, SUC_lexicon_tags_19 },
    { 0xb37b33f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb33f9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x509b33fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33b33ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5db3401, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdefb3403, SUC_lexicon_tags_27 },
    { 0x5bdb3404, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55b3408, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b5b340a, SUC_lexicon_tags_55 },
    { 0xf25b340a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3b340d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26db3412, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11db3418, SUC_lexicon_tags_0 },
    { 0x7e1b3419, SUC_lexicon_tags_30 },
    { 0xbf9b341a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x1b3b341c, SUC_lexicon_tags_396 },
    { 0xb2fb341d, SUC_lexicon_tags_17 },
    { 0x3e5b341e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96db3426, SUC_lexicon_tags_4 },
    { 0xbfbb3426, SUC_lexicon_tags_15 },
    { 0xf7b3427, SUC_lexicon_tags_29 },
    { 0x92fb3428, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abb343e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf03b3440, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x399b3442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419b3447, SUC_lexicon_tags_43 },
    { 0x3bfb3447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabb344c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x199b344e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x95bb3450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1db3455, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edb3458, SUC_lexicon_tags_37 },
    { 0x4c9b3459, SUC_lexicon_tags_44 },
    { 0xb03b345a, SUC_lexicon_tags_15 },
    { 0x7d7b3459, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5b3462, SUC_lexicon_tags_37 },
    { 0x14fb3463, SUC_lexicon_tags_11 },
    { 0x507b3464, SUC_lexicon_tags_0 },
    { 0xee1b3465, SUC_lexicon_tags_37 },
    { 0x4fb3466, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307b346c, SUC_lexicon_tags_25 },
    { 0xcfbb346c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb346f, SUC_lexicon_tags_55 },
    { 0x275b3470, SUC_lexicon_tags_26 },
    { 0x543b3471, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x637b3473, SUC_lexicon_tags_12 },
    { 0xa3b3473, SUC_lexicon_tags_1 },
    { 0x673b3475, SUC_lexicon_tags_37 },
    { 0xfd5b3475, SUC_lexicon_tags_11 },
    { 0x503b3477, SUC_lexicon_tags_47 },
    { 0xf31b3478, SUC_lexicon_tags_10 },
    { 0xae9b3478, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137b347d, SUC_lexicon_tags_37 },
    { 0x9e7b347e, SUC_lexicon_tags_37 },
    { 0xf5b347e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdb3481, SUC_lexicon_tags_17 },
    { 0x76db3482, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd3b3484, SUC_lexicon_tags_23 },
    { 0xc85b3485, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x41db3488, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1b9b348a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdbb348c, SUC_lexicon_tags_47 },
    { 0xdd7b348c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xabbb348f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdd1b3491, SUC_lexicon_tags_12 },
    { 0xa63b3492, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb3b3494, SUC_lexicon_tags_37 },
    { 0x6e3b3494, SUC_lexicon_tags_15 },
    { 0x8dfb3494, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbb3499, SUC_lexicon_tags_236 },
    { 0x6c7b349a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xccbb349c, SUC_lexicon_tags_11 },
    { 0x60bb349c, SUC_lexicon_tags_28 },
    { 0x3efb349e, SUC_lexicon_tags_29 },
    { 0x621b349f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83b34a3, SUC_lexicon_tags_16 },
    { 0x4c5b34a4, SUC_lexicon_tags_11 },
    { 0x5e1b34a4, SUC_lexicon_tags_12 },
    { 0x14db34a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x193b34a9, SUC_lexicon_tags_19 },
    { 0x285b34aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3b34ad, SUC_lexicon_tags_11 },
    { 0xb53b34ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x153b34b0, SUC_lexicon_tags_38 },
    { 0xfa3b34b1, SUC_lexicon_tags_15 },
    { 0x3afb34b2, SUC_lexicon_tags_11 },
    { 0xb11b34b3, SUC_lexicon_tags_26 },
    { 0x3ebb34b4, SUC_lexicon_tags_25 },
    { 0x769b34b4, SUC_lexicon_tags_23 },
    { 0x2b3b34b2, SUC_lexicon_tags_12 },
    { 0xc97b34b7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5b34be, SUC_lexicon_tags_14 },
    { 0x1efb34bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16fb34c1, SUC_lexicon_tags_30 },
    { 0xfafb34c2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fb34c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb3b34c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c3b34c9, SUC_lexicon_tags_37 },
    { 0xae3b34ca, SUC_lexicon_tags_0 },
    { 0x8c1b34cb, SUC_lexicon_tags_19 },
    { 0x9bdb34c9, SUC_lexicon_tags_7 },
    { 0xef7b34cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb1db34cf, SUC_lexicon_tags_60 },
    { 0xc21b34d0, SUC_lexicon_tags_35 },
    { 0x231b34d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf39b34d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31b34df, SUC_lexicon_tags_28 },
    { 0x57db34df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe45b34e2, SUC_lexicon_tags_43 },
    { 0x8a7b34e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3b34e7, SUC_lexicon_tags_38 },
    { 0x74fb34e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471b34f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x235b34f6, SUC_lexicon_tags_60 },
    { 0xf27b34f7, SUC_lexicon_tags_87 },
    { 0x81b34f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b34fc, SUC_lexicon_tags_23 },
    { 0x2d5b34fd, SUC_lexicon_tags_65 },
    { 0x3cfb34fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0db3502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325b3508, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc45b350a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24fb350c, SUC_lexicon_tags_30 },
    { 0x2ebb350d, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x833b350f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9b3513, SUC_lexicon_tags_6 },
    { 0x641b3514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fb351f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3b3522, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58db352a, SUC_lexicon_tags_38 },
    { 0x9b3b352b, SUC_lexicon_tags_77 },
    { 0x781b352c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edb3530, SUC_lexicon_tags_35 },
    { 0xabdb3531, SUC_lexicon_tags_12 },
    { 0x595b3531, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdb3536, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0db353a, SUC_lexicon_tags_25 },
    { 0x1b353b, SUC_lexicon_tags_15 },
    { 0x527b353a, SUC_lexicon_tags_1 },
    { 0xfbbb353c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37b3541, SUC_lexicon_tags_19 },
    { 0x3fdb3542, SUC_lexicon_tags_25 },
    { 0x675b3543, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715b3547, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1b354e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afb3552, SUC_lexicon_tags_39 },
    { 0xf9bb3552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5b3557, SUC_lexicon_tags_12 },
    { 0xe05b3558, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x11b355a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1b355e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b3561, SUC_lexicon_tags_2 },
    { 0x111b3562, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875b3566, SUC_lexicon_tags_60 },
    { 0xc77b3567, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75b356a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x831b356c, SUC_lexicon_tags_11 },
    { 0xa43b356d, SUC_lexicon_tags_63 },
    { 0x40db356e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5b3572, SUC_lexicon_tags_33 },
    { 0xd43b3573, SUC_lexicon_tags_17 },
    { 0xce1b3574, SUC_lexicon_tags_1 },
    { 0xbabb3574, SUC_lexicon_tags_6 },
    { 0xdc7b3576, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269b357b, SUC_lexicon_tags_21 },
    { 0x1d1b357c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x661b357e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x679b3580, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefb3583, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb3589, SUC_lexicon_tags_17 },
    { 0x2d3b358a, SUC_lexicon_tags_0 },
    { 0xfddb358b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x38bb358d, SUC_lexicon_tags_1 },
    { 0x495b358e, SUC_lexicon_tags_23 },
    { 0x25bb358e, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0x84db3591, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x479b3594, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbb3b3596, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01b359b, SUC_lexicon_tags_37 },
    { 0xe3bb359c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fb35a2, SUC_lexicon_tags_0 },
    { 0x25fb35a3, SUC_lexicon_tags_15 },
    { 0x321b35a3, SUC_lexicon_tags_2 },
    { 0x73db35a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7b35aa, SUC_lexicon_tags_11 },
    { 0x855b35ab, SUC_lexicon_tags_6 },
    { 0xbd7b35aa, SUC_lexicon_tags_43 },
    { 0xa4bb35ad, SUC_lexicon_tags_87 },
    { 0x87db35ad, SUC_lexicon_tags_60 },
    { 0x81bb35af, SUC_lexicon_tags_49 },
    { 0x659b35b0, SUC_lexicon_tags_25 },
    { 0x70db35b1, SUC_lexicon_tags_11 },
    { 0x55b35b1, SUC_lexicon_tags_38 },
    { 0x6b3b35b3, SUC_lexicon_tags_61 },
    { 0xe89b35b3, SUC_lexicon_tags_12 },
    { 0xc95b35b3, SUC_lexicon_tags_27 },
    { 0xea1b35b3, SUC_lexicon_tags_100 },
    { 0xb43b35b1, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243b35bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe5b35bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92fb35bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595b35c4, SUC_lexicon_tags_17 },
    { 0xfe7b35c5, SUC_lexicon_tags_8 },
    { 0x4e1b35c6, SUC_lexicon_tags_12 },
    { 0x1f5b35c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5b35cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd49b35cf, SUC_lexicon_tags_7 },
    { 0x1b9b35d0, SUC_lexicon_tags_7 },
    { 0x925b35d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9b35d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abb35d8, SUC_lexicon_tags_25 },
    { 0xeadb35d9, SUC_lexicon_tags_11 },
    { 0x887b35da, SUC_lexicon_tags_38 },
    { 0x99bb35d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7dbb35dd, SUC_lexicon_tags_14 },
    { 0x2d5b35de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf17b35e0, SUC_lexicon_tags_93 },
    { 0x201b35e1, SUC_lexicon_tags_19 },
    { 0xc13b35e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x117b35e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfedb35e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27b35e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79b35ed, SUC_lexicon_tags_75 },
    { 0x63bb35ee, SUC_lexicon_tags_56 },
    { 0x609b35ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3b35f6, SUC_lexicon_tags_61 },
    { 0x2e9b35f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15b35fc, SUC_lexicon_tags_2 },
    { 0x49db35fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x789b35ff, SUC_lexicon_tags_60 },
    { 0x28fb35ff, SUC_lexicon_tags_2 },
    { 0xa1db3601, SUC_lexicon_tags_91 },
    { 0xa11b3600, SUC_lexicon_tags_7 },
    { 0xbdfb3603, SUC_lexicon_tags_17 },
    { 0xf1b3602, SUC_lexicon_tags_25 },
    { 0xdb9b3605, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x283b3607, SUC_lexicon_tags_15 },
    { 0xffdb3608, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a1b360a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x589b360d, SUC_lexicon_tags_15 },
    { 0xb8db360e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x949b3611, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fb3619, SUC_lexicon_tags_60 },
    { 0xc4fb361a, SUC_lexicon_tags_37 },
    { 0xde7b361b, SUC_lexicon_tags_26 },
    { 0x93bb361b, SUC_lexicon_tags_12 },
    { 0x68db361a, SUC_lexicon_tags_65 },
    { 0xef7b361b, SUC_lexicon_tags_77 },
    { 0xf7db361f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x133b3621, SUC_lexicon_tags_11 },
    { 0x161b3621, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37b3625, SUC_lexicon_tags_15 },
    { 0x9c5b3626, SUC_lexicon_tags_43 },
    { 0x117b3627, SUC_lexicon_tags_17 },
    { 0xf97b3625, SUC_lexicon_tags_29 },
    { 0xc4bb3627, SUC_lexicon_tags_23 },
    { 0xe81b362a, SUC_lexicon_tags_60 },
    { 0x523b362a, SUC_lexicon_tags_81 },
    { 0x6b5b362c, SUC_lexicon_tags_33 },
    { 0x2d3b362c, SUC_lexicon_tags_7 },
    { 0x25db362e, SUC_lexicon_tags_11 },
    { 0xea1b362e, SUC_lexicon_tags_7 },
    { 0x607b3630, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291b3635, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x21fb3637, SUC_lexicon_tags_25 },
    { 0xf59b3638, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3b363c, SUC_lexicon_tags_33 },
    { 0x551b363d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdb3643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871b364a, SUC_lexicon_tags_12 },
    { 0xd07b364a, SUC_lexicon_tags_11 },
    { 0x5c5b364b, SUC_lexicon_tags_0 },
    { 0xf85b364c, SUC_lexicon_tags_27 },
    { 0xcc9b364e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7b3652, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x11b3654, SUC_lexicon_tags_82 },
    { 0x607b3654, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa73b3657, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15b365b, SUC_lexicon_tags_45 },
    { 0xebdb365c, SUC_lexicon_tags_30 },
    { 0xf5bb365d, SUC_lexicon_tags_37 },
    { 0xef9b365c, SUC_lexicon_tags_14 },
    { 0x92db365c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fb3663, SUC_lexicon_tags_45 },
    { 0x98db3663, SUC_lexicon_tags_60 },
    { 0x81bb3665, SUC_lexicon_tags_187 },
    { 0x58bb3666, SUC_lexicon_tags_37 },
    { 0x369b3667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41b366a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefb3672, SUC_lexicon_tags_9 },
    { 0x589b3672, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab9b3675, SUC_lexicon_tags_12 },
    { 0xe5fb3676, SUC_lexicon_tags_15 },
    { 0x8c5b3676, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x60fb3679, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67b367c, SUC_lexicon_tags_11 },
    { 0x669b367d, SUC_lexicon_tags_37 },
    { 0xcfbb367d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8bfb3680, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x493b3682, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fb3687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x137b3689, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xc47b368b, SUC_lexicon_tags_6 },
    { 0x307b368c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x16bb368e, SUC_lexicon_tags_20 },
    { 0x285b368f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbb3692, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x35db3695, SUC_lexicon_tags_61 },
    { 0x9cfb3696, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819b369d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7b36a0, SUC_lexicon_tags_16 },
    { 0xfb1b36a1, SUC_lexicon_tags_28 },
    { 0x357b36a2, SUC_lexicon_tags_2 },
    { 0xd97b36a3, SUC_lexicon_tags_23 },
    { 0x185b36a3, SUC_lexicon_tags_28 },
    { 0x4bdb36a5, SUC_lexicon_tags_25 },
    { 0xd95b36a4, SUC_lexicon_tags_29 },
    { 0x8d7b36a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x225b36a9, SUC_lexicon_tags_46 },
    { 0x769b36a9, SUC_lexicon_tags_49 },
    { 0xc9b36ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fb36ae, SUC_lexicon_tags_49 },
    { 0xfcdb36af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1b36b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab5b36b4, SUC_lexicon_tags_15 },
    { 0xfa1b36b4, SUC_lexicon_tags_19 },
    { 0xa91b36b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbb36bb, SUC_lexicon_tags_10 },
    { 0x4a3b36bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e3b36be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adb36c2, SUC_lexicon_tags_15 },
    { 0x4abb36c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47b36c8, SUC_lexicon_tags_45 },
    { 0xb33b36c9, SUC_lexicon_tags_7 },
    { 0xeadb36ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb36ce, SUC_lexicon_tags_39 },
    { 0x887b36cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bb36d6, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59b36e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411b36e4, SUC_lexicon_tags_1 },
    { 0xb9b36e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3b36e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605b36ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bb36f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b7b36f3, SUC_lexicon_tags_7 },
    { 0x39db36f4, SUC_lexicon_tags_16 },
    { 0x333b36f5, SUC_lexicon_tags_647 },
    { 0x95fb36f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447b36fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc65b36fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x505b36fe, SUC_lexicon_tags_39 },
    { 0xe35b36ff, SUC_lexicon_tags_17 },
    { 0x135b3700, SUC_lexicon_tags_48 },
    { 0x629b3701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db3704, SUC_lexicon_tags_47 },
    { 0xd87b3704, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203b3715, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67b3719, SUC_lexicon_tags_11 },
    { 0xb01b371a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88bb3724, SUC_lexicon_tags_27 },
    { 0x6cdb3725, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdb3728, SUC_lexicon_tags_30 },
    { 0x84fb3729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1db372e, SUC_lexicon_tags_23 },
    { 0xdffb372e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x333b3731, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x123b3734, SUC_lexicon_tags_7 },
    { 0x2ebb3735, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x87db3738, SUC_lexicon_tags_2 },
    { 0x807b3738, SUC_lexicon_tags_17 },
    { 0x34fb373a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x19db373d, SUC_lexicon_tags_15 },
    { 0x5dbb373e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3b3742, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1b3745, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xf0fb3747, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf87b3749, SUC_lexicon_tags_15 },
    { 0x457b374a, SUC_lexicon_tags_36 },
    { 0x14fb374b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffb374e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bb3755, SUC_lexicon_tags_30 },
    { 0x8e9b3756, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x911b3759, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fb375d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x629b3760, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fb3765, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0db3769, SUC_lexicon_tags_33 },
    { 0x6f3b376a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x963b376d, SUC_lexicon_tags_2 },
    { 0x339b376d, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491b3774, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd7b3776, SUC_lexicon_tags_14 },
    { 0x285b3777, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x67bb3779, SUC_lexicon_tags_2 },
    { 0xd3fb377a, SUC_lexicon_tags_0 },
    { 0xccbb377b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89b3783, SUC_lexicon_tags_25 },
    { 0x359b3784, SUC_lexicon_tags_7 },
    { 0x809b3785, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3bbb3787, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa29b3789, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x229b378b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca1b378d, SUC_lexicon_tags_1 },
    { 0x87fb378e, SUC_lexicon_tags_2 },
    { 0x30bb378f, SUC_lexicon_tags_87 },
    { 0x931b3790, SUC_lexicon_tags_34 },
    { 0xc6bb3791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb3794, SUC_lexicon_tags_29 },
    { 0x4cfb3795, SUC_lexicon_tags_12 },
    { 0x6dbb3794, SUC_lexicon_tags_37 },
    { 0xc7fb3796, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3fb3799, SUC_lexicon_tags_2 },
    { 0x373b379a, SUC_lexicon_tags_21 },
    { 0xdf1b3799, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15b379f, SUC_lexicon_tags_16 },
    { 0xd99b37a0, SUC_lexicon_tags_25 },
    { 0xd25b37a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x263b37a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdb37ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fb37b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e9b37b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc21b37b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fb37c0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xae1b37c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b1b37c4, SUC_lexicon_tags_25 },
    { 0xa5b37c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b37c8, SUC_lexicon_tags_37 },
    { 0xdefb37c9, SUC_lexicon_tags_30 },
    { 0xfa9b37ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x829b37cc, SUC_lexicon_tags_1 },
    { 0xba9b37cd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7b37d2, SUC_lexicon_tags_14 },
    { 0xbadb37d3, SUC_lexicon_tags_15 },
    { 0x649b37d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbb37d7, SUC_lexicon_tags_42 },
    { 0x3ddb37d8, SUC_lexicon_tags_25 },
    { 0x587b37d7, SUC_lexicon_tags_46 },
    { 0xa05b37d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd7b37dc, SUC_lexicon_tags_108 },
    { 0x8c5b37dc, SUC_lexicon_tags_45 },
    { 0x1f1b37de, SUC_lexicon_tags_19 },
    { 0x21db37dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc85b37e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe01b37e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xedfb37e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x17b37e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x957b37e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd7b37eb, SUC_lexicon_tags_87 },
    { 0x88db37ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x603b37ee, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x88fb37f0, SUC_lexicon_tags_66 },
    { 0x367b37f1, SUC_lexicon_tags_2 },
    { 0x741b37f0, SUC_lexicon_tags_25 },
    { 0x2db37f3, SUC_lexicon_tags_12 },
    { 0x5fbb37f4, SUC_lexicon_tags_17 },
    { 0x1ebb37f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdb37f9, SUC_lexicon_tags_38 },
    { 0xa67b37fa, SUC_lexicon_tags_1 },
    { 0x93bb37fb, SUC_lexicon_tags_2 },
    { 0x9e5b37fc, SUC_lexicon_tags_11 },
    { 0x375b37fb, SUC_lexicon_tags_14 },
    { 0xbbb37fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fb3801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399b3807, SUC_lexicon_tags_38 },
    { 0xf03b3808, SUC_lexicon_tags_19 },
    { 0xc41b3809, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87db380e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3b3812, SUC_lexicon_tags_66 },
    { 0x589b3813, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b381e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469b3825, SUC_lexicon_tags_19 },
    { 0xca7b3825, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1bb3828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7b382f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x211b3831, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bb383a, SUC_lexicon_tags_19 },
    { 0xc29b383a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61b383e, SUC_lexicon_tags_39 },
    { 0xd2db383e, SUC_lexicon_tags_37 },
    { 0xde1b3840, SUC_lexicon_tags_6 },
    { 0x879b3840, SUC_lexicon_tags_17 },
    { 0x5e9b3842, SUC_lexicon_tags_17 },
    { 0x62fb3843, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527b3847, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fb384c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3b384f, SUC_lexicon_tags_11 },
    { 0xa5db3850, SUC_lexicon_tags_11 },
    { 0x20bb384f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x335b3854, SUC_lexicon_tags_85 },
    { 0x129b3855, SUC_lexicon_tags_11 },
    { 0x48fb3855, SUC_lexicon_tags_12 },
    { 0xdbb3857, SUC_lexicon_tags_17 },
    { 0x25bb3857, SUC_lexicon_tags_12 },
    { 0x127b3859, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x83b385c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83b385f, SUC_lexicon_tags_80 },
    { 0x30fb3860, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fb3864, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33fb3866, SUC_lexicon_tags_19 },
    { 0x1edb3867, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bb386a, SUC_lexicon_tags_17 },
    { 0xdc1b386b, SUC_lexicon_tags_25 },
    { 0x8dbb386c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1db386e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x851b3871, SUC_lexicon_tags_2 },
    { 0x74db3872, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xccfb3874, SUC_lexicon_tags_19 },
    { 0xe37b3874, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7b387b, SUC_lexicon_tags_14 },
    { 0xb1bb387b, SUC_lexicon_tags_72 },
    { 0x83fb387d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x535b3880, SUC_lexicon_tags_1 },
    { 0xde7b3880, SUC_lexicon_tags_15 },
    { 0xf97b3881, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xa35b3884, SUC_lexicon_tags_1 },
    { 0xd83b3884, SUC_lexicon_tags_49 },
    { 0x4e3b3886, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x5edb3888, SUC_lexicon_tags_15 },
    { 0x311b3889, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5b388d, SUC_lexicon_tags_3 },
    { 0xf7fb388e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80bb3890, SUC_lexicon_tags_47 },
    { 0x9a3b3890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdb3896, SUC_lexicon_tags_15 },
    { 0x2ddb3897, SUC_lexicon_tags_38 },
    { 0xeb1b3897, SUC_lexicon_tags_2 },
    { 0x5b3899, SUC_lexicon_tags_68 },
    { 0x15b389a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadb389d, SUC_lexicon_tags_37 },
    { 0xdbbb389d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x109b38a1, SUC_lexicon_tags_4 },
    { 0xd23b38a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd37b38a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x891b38a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bb38a9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x285b38ab, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0x785b38ae, SUC_lexicon_tags_61 },
    { 0x2c5b38af, SUC_lexicon_tags_2 },
    { 0x4a1b38ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edb38b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x739b38b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9b38bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93b38c7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa93b38c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6db38cc, SUC_lexicon_tags_27 },
    { 0xbadb38cc, SUC_lexicon_tags_27 },
    { 0x8a1b38ce, SUC_lexicon_tags_1 },
    { 0x8d9b38cf, SUC_lexicon_tags_4 },
    { 0x143b38cd, SUC_lexicon_tags_17 },
    { 0xa87b38d1, SUC_lexicon_tags_14 },
    { 0x891b38d2, SUC_lexicon_tags_43 },
    { 0x42bb38d2, SUC_lexicon_tags_1 },
    { 0x98db38d3, SUC_lexicon_tags_56 },
    { 0x5e7b38d5, SUC_lexicon_tags_0 },
    { 0x617b38d1, SUC_lexicon_tags_25 },
    { 0xaa7b38d7, SUC_lexicon_tags_3 },
    { 0x7b3b38d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adb38db, SUC_lexicon_tags_12 },
    { 0x125b38dc, SUC_lexicon_tags_32 },
    { 0x8bfb38dd, SUC_lexicon_tags_637 },
    { 0, NULL },
    { 0x7b1b38df, SUC_lexicon_tags_1 },
    { 0xbbb38e0, SUC_lexicon_tags_4 },
    { 0x83db38e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11b38e4, SUC_lexicon_tags_45 },
    { 0x141b38e5, SUC_lexicon_tags_23 },
    { 0xc39b38e6, SUC_lexicon_tags_12 },
    { 0x269b38e7, SUC_lexicon_tags_37 },
    { 0xccbb38e8, SUC_lexicon_tags_82 },
    { 0x555b38e7, SUC_lexicon_tags_26 },
    { 0xe2bb38e7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fb38ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x847b38ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x63db38f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183b38f5, SUC_lexicon_tags_1 },
    { 0x239b38f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8db38f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1b38fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85b3900, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b3903, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c1b3905, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe51b3907, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfb390f, SUC_lexicon_tags_38 },
    { 0x36fb3910, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf1fb3912, SUC_lexicon_tags_14 },
    { 0xe2fb3913, SUC_lexicon_tags_12 },
    { 0xeb1b3913, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bb3918, SUC_lexicon_tags_29 },
    { 0xf83b3918, SUC_lexicon_tags_25 },
    { 0x913b391a, SUC_lexicon_tags_60 },
    { 0xb1b391b, SUC_lexicon_tags_11 },
    { 0xbf9b391c, SUC_lexicon_tags_37 },
    { 0x13bb391d, SUC_lexicon_tags_60 },
    { 0xb19b391e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x82db3921, SUC_lexicon_tags_8 },
    { 0x385b3922, SUC_lexicon_tags_0 },
    { 0xdf5b3923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x415b3926, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107b392f, SUC_lexicon_tags_47 },
    { 0xe8db3930, SUC_lexicon_tags_30 },
    { 0x337b3930, SUC_lexicon_tags_8 },
    { 0x669b3930, SUC_lexicon_tags_37 },
    { 0x2e9b3933, SUC_lexicon_tags_27 },
    { 0xac5b3931, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5b393a, SUC_lexicon_tags_56 },
    { 0xd8db393b, SUC_lexicon_tags_25 },
    { 0x873b393c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb47b393e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619b3944, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1c7b3946, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1b394b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b7b394d, SUC_lexicon_tags_49 },
    { 0x737b394e, SUC_lexicon_tags_23 },
    { 0x501b394e, SUC_lexicon_tags_1 },
    { 0x3f7b394d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9b3955, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bb3958, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07b395c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1b3963, SUC_lexicon_tags_65 },
    { 0x50bb3963, SUC_lexicon_tags_37 },
    { 0x44bb3965, SUC_lexicon_tags_39 },
    { 0xc85b3965, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9b396b, SUC_lexicon_tags_44 },
    { 0x88db396c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83bb396e, SUC_lexicon_tags_45 },
    { 0xdd5b396f, SUC_lexicon_tags_12 },
    { 0xd2db3970, SUC_lexicon_tags_15 },
    { 0x7f5b396f, SUC_lexicon_tags_14 },
    { 0x2f1b3972, SUC_lexicon_tags_14 },
    { 0x493b3972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9f7b3975, SUC_lexicon_tags_31 },
    { 0x73db3976, SUC_lexicon_tags_17 },
    { 0xe7bb3977, SUC_lexicon_tags_15 },
    { 0xf0bb3978, SUC_lexicon_tags_11 },
    { 0xb09b3976, SUC_lexicon_tags_25 },
    { 0x347b3976, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd5b397c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x36db397f, SUC_lexicon_tags_12 },
    { 0x6bdb3980, SUC_lexicon_tags_37 },
    { 0x533b397f, SUC_lexicon_tags_11 },
    { 0x7dfb3981, SUC_lexicon_tags_9 },
    { 0x559b3980, SUC_lexicon_tags_12 },
    { 0xe1fb3982, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa23b3986, SUC_lexicon_tags_1 },
    { 0x44db3987, SUC_lexicon_tags_0 },
    { 0xaffb3988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x87b398b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fb398f, SUC_lexicon_tags_56 },
    { 0xb65b3990, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdabb3992, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85b3996, SUC_lexicon_tags_8 },
    { 0x8f3b3997, SUC_lexicon_tags_15 },
    { 0xa45b3997, SUC_lexicon_tags_412 },
    { 0xd57b3997, SUC_lexicon_tags_15 },
    { 0xfb1b399a, SUC_lexicon_tags_11 },
    { 0x2d1b3999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9b39a7, SUC_lexicon_tags_28 },
    { 0x1a5b39a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fb39ab, SUC_lexicon_tags_84 },
    { 0x19bb39ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abb39af, SUC_lexicon_tags_11 },
    { 0x475b39b0, SUC_lexicon_tags_1 },
    { 0x817b39af, SUC_lexicon_tags_33 },
    { 0xf57b39b2, SUC_lexicon_tags_64 },
    { 0x9f5b39b3, SUC_lexicon_tags_45 },
    { 0x563b39b3, SUC_lexicon_tags_15 },
    { 0x3efb39b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fb39b9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133b39bf, SUC_lexicon_tags_0 },
    { 0x33b39c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe95b39c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3b39c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf5b39c9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69b39ce, SUC_lexicon_tags_11 },
    { 0x783b39cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67b39d6, SUC_lexicon_tags_16 },
    { 0xd01b39d7, SUC_lexicon_tags_63 },
    { 0x557b39d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db39dc, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9b39e0, SUC_lexicon_tags_25 },
    { 0x501b39e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x773b39e3, SUC_lexicon_tags_15 },
    { 0xfdfb39e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x975b39e7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abb39ea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf25b39ec, SUC_lexicon_tags_33 },
    { 0xc03b39ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79b39f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3b39f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbb39f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125b39fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bb39fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x855b3a00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681b3a05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x36bb3a08, SUC_lexicon_tags_101 },
    { 0x9e5b3a09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7efb3a0b, SUC_lexicon_tags_49 },
    { 0xb39b3a0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd55b3a0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bb3a12, SUC_lexicon_tags_2 },
    { 0xa1b3a12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bb3a16, SUC_lexicon_tags_12 },
    { 0x71db3a17, SUC_lexicon_tags_38 },
    { 0x8bdb3a18, SUC_lexicon_tags_15 },
    { 0xb11b3a19, SUC_lexicon_tags_0 },
    { 0x7d3b3a19, SUC_lexicon_tags_14 },
    { 0xa7fb3a1a, SUC_lexicon_tags_28 },
    { 0x1b5b3a1c, SUC_lexicon_tags_19 },
    { 0xf7fb3a1d, SUC_lexicon_tags_12 },
    { 0xc53b3a1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc79b3a20, SUC_lexicon_tags_19 },
    { 0xd2db3a21, SUC_lexicon_tags_25 },
    { 0xa11b3a22, SUC_lexicon_tags_16 },
    { 0x81b3a20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7b3a26, SUC_lexicon_tags_55 },
    { 0x657b3a26, SUC_lexicon_tags_25 },
    { 0xce5b3a28, SUC_lexicon_tags_12 },
    { 0x1f9b3a29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41b3a2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fb3a36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9b3a3c, SUC_lexicon_tags_30 },
    { 0xb75b3a3d, SUC_lexicon_tags_0 },
    { 0xd7bb3a3e, SUC_lexicon_tags_9 },
    { 0xa5bb3a3e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x16bb3a41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1b3a52, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789b3a57, SUC_lexicon_tags_15 },
    { 0x24fb3a58, SUC_lexicon_tags_25 },
    { 0xeb3b3a57, SUC_lexicon_tags_2 },
    { 0xff1b3a58, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bb3a60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5db3a65, SUC_lexicon_tags_43 },
    { 0x9e9b3a66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb3b3a68, SUC_lexicon_tags_15 },
    { 0x6bfb3a68, SUC_lexicon_tags_15 },
    { 0x581b3a6a, SUC_lexicon_tags_11 },
    { 0xd65b3a68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x367b3a6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b3a73, SUC_lexicon_tags_9 },
    { 0x87fb3a73, SUC_lexicon_tags_13 },
    { 0x19bb3a73, SUC_lexicon_tags_25 },
    { 0xf21b3a76, SUC_lexicon_tags_6 },
    { 0x7c5b3a77, SUC_lexicon_tags_40 },
    { 0x66fb3a75, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd0fb3a7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff7b3a7c, SUC_lexicon_tags_2 },
    { 0x2c7b3a7c, SUC_lexicon_tags_25 },
    { 0x4cfb3a7e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385b3a84, SUC_lexicon_tags_32 },
    { 0x6f5b3a84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5b3a8b, SUC_lexicon_tags_2 },
    { 0xa09b3a8c, SUC_lexicon_tags_28 },
    { 0x90fb3a8c, SUC_lexicon_tags_25 },
    { 0x97db3a8d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x255b3a91, SUC_lexicon_tags_33 },
    { 0x347b3a91, SUC_lexicon_tags_23 },
    { 0xd3bb3a93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3b3a9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x225b3a9f, SUC_lexicon_tags_35 },
    { 0x2a9b3aa0, SUC_lexicon_tags_15 },
    { 0x30db3aa1, SUC_lexicon_tags_81 },
    { 0xdfbb3aa0, SUC_lexicon_tags_45 },
    { 0x23bb3a9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe89b3aa5, SUC_lexicon_tags_15 },
    { 0x47b3aa5, SUC_lexicon_tags_12 },
    { 0xb73b3aa7, SUC_lexicon_tags_17 },
    { 0xf23b3aa8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9d3b3aaa, SUC_lexicon_tags_32 },
    { 0xf21b3aab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb3aaf, SUC_lexicon_tags_11 },
    { 0xcfb3ab0, SUC_lexicon_tags_2 },
    { 0x3c3b3aaf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x10db3ab4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb3ab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9fdb3aba, SUC_lexicon_tags_7 },
    { 0x1edb3aba, SUC_lexicon_tags_26 },
    { 0x375b3abc, SUC_lexicon_tags_19 },
    { 0x3f9b3abc, SUC_lexicon_tags_25 },
    { 0x97b3abd, SUC_lexicon_tags_25 },
    { 0x353b3abd, SUC_lexicon_tags_25 },
    { 0xc87b3abf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x381b3ac2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9b3ac5, SUC_lexicon_tags_11 },
    { 0x537b3ac5, SUC_lexicon_tags_14 },
    { 0x14bb3ac5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefbb3ac9, SUC_lexicon_tags_11 },
    { 0x63b3ac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x249b3acc, SUC_lexicon_tags_12 },
    { 0xf41b3acd, SUC_lexicon_tags_4 },
    { 0xf21b3acc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbb3ada, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5a5b3adc, SUC_lexicon_tags_37 },
    { 0x34bb3adc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43b3ae2, SUC_lexicon_tags_1 },
    { 0xcdfb3ae3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdb3ae7, SUC_lexicon_tags_33 },
    { 0xdfb3ae8, SUC_lexicon_tags_7 },
    { 0x86db3ae9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3b9b3aeb, SUC_lexicon_tags_0 },
    { 0x9f5b3aeb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757b3af0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x113b3af3, SUC_lexicon_tags_12 },
    { 0xe1b3af4, SUC_lexicon_tags_23 },
    { 0xe39b3af3, SUC_lexicon_tags_2 },
    { 0x647b3af4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5f1b3af8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9b3afc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x187b3aff, SUC_lexicon_tags_48 },
    { 0xf99b3aff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367b3b06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37b3b09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x877b3b0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bb3b10, SUC_lexicon_tags_60 },
    { 0x4bdb3b11, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1a7b3b13, SUC_lexicon_tags_37 },
    { 0xd73b3b14, SUC_lexicon_tags_38 },
    { 0x2e1b3b15, SUC_lexicon_tags_66 },
    { 0x6ebb3b16, SUC_lexicon_tags_11 },
    { 0x53bb3b16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7b3b1b, SUC_lexicon_tags_6 },
    { 0x14db3b1c, SUC_lexicon_tags_26 },
    { 0x197b3b1d, SUC_lexicon_tags_33 },
    { 0x2c3b3b1d, SUC_lexicon_tags_25 },
    { 0xf1db3b1b, SUC_lexicon_tags_15 },
    { 0xe81b3b20, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3b3b27, SUC_lexicon_tags_60 },
    { 0xe6bb3b28, SUC_lexicon_tags_48 },
    { 0xa55b3b28, SUC_lexicon_tags_17 },
    { 0xbe1b3b2a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x995b3b2c, SUC_lexicon_tags_1 },
    { 0x873b3b2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bb3b33, SUC_lexicon_tags_26 },
    { 0xcbbb3b34, SUC_lexicon_tags_45 },
    { 0xba5b3b35, SUC_lexicon_tags_7 },
    { 0x2c3b3b34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x45fb3b39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9b3b3b, SUC_lexicon_tags_45 },
    { 0x815b3b3b, SUC_lexicon_tags_30 },
    { 0x483b3b3d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x351b3b3f, SUC_lexicon_tags_29 },
    { 0x1a7b3b40, SUC_lexicon_tags_0 },
    { 0xeedb3b41, SUC_lexicon_tags_8 },
    { 0x261b3b41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667b3b46, SUC_lexicon_tags_0 },
    { 0xfbb3b47, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75db3b4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bb3b4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf37b3b50, SUC_lexicon_tags_2 },
    { 0xe61b3b50, SUC_lexicon_tags_21 },
    { 0x663b3b52, SUC_lexicon_tags_47 },
    { 0x8bbb3b52, SUC_lexicon_tags_37 },
    { 0xb99b3b54, SUC_lexicon_tags_14 },
    { 0x36db3b55, SUC_lexicon_tags_160 },
    { 0x997b3b56, SUC_lexicon_tags_1 },
    { 0xfd5b3b56, SUC_lexicon_tags_4 },
    { 0x64db3b58, SUC_lexicon_tags_19 },
    { 0x26fb3b54, SUC_lexicon_tags_25 },
    { 0x26bb3b59, SUC_lexicon_tags_23 },
    { 0x953b3b5a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x29db3b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1b3b62, SUC_lexicon_tags_23 },
    { 0x3bdb3b63, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcc9b3b65, SUC_lexicon_tags_38 },
    { 0x59db3b66, SUC_lexicon_tags_136 },
    { 0x233b3b67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb3b71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9b3b7a, SUC_lexicon_tags_1 },
    { 0x851b3b7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x263b3b7d, SUC_lexicon_tags_38 },
    { 0xfdb3b7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53db3b80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa27b3b82, SUC_lexicon_tags_44 },
    { 0x903b3b82, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fb3b89, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x957b3b8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79b3b94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bb3b9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe7db3b9e, SUC_lexicon_tags_0 },
    { 0x48fb3b9f, SUC_lexicon_tags_37 },
    { 0x5b3b3b9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5b3ba3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeafb3ba5, SUC_lexicon_tags_114 },
    { 0xb29b3ba6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x987b3ba8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd39b3baa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1b3bbb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf53b3bbd, SUC_lexicon_tags_1 },
    { 0x5dfb3bbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed9b3bc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1b3bc5, SUC_lexicon_tags_17 },
    { 0x825b3bc6, SUC_lexicon_tags_1 },
    { 0xe13b3bc7, SUC_lexicon_tags_30 },
    { 0x797b3bc7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7bbb3bca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6db3bcc, SUC_lexicon_tags_11 },
    { 0xec3b3bcd, SUC_lexicon_tags_1273 },
    { 0x98fb3bce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355b3bd2, SUC_lexicon_tags_45 },
    { 0x8dfb3bd3, SUC_lexicon_tags_6 },
    { 0x647b3bd3, SUC_lexicon_tags_47 },
    { 0x755b3bd5, SUC_lexicon_tags_17 },
    { 0xa47b3bd6, SUC_lexicon_tags_11 },
    { 0xca1b3bd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbb3bdc, SUC_lexicon_tags_9 },
    { 0x51fb3bdc, SUC_lexicon_tags_12 },
    { 0xd9db3bdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdb3be4, SUC_lexicon_tags_1354 },
    { 0x141b3be5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775b3be9, SUC_lexicon_tags_15 },
    { 0x259b3be9, SUC_lexicon_tags_8 },
    { 0xd95b3beb, SUC_lexicon_tags_182 },
    { 0xae7b3bea, SUC_lexicon_tags_7 },
    { 0xec3b3bed, SUC_lexicon_tags_29 },
    { 0xe61b3bee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x277b3bf0, SUC_lexicon_tags_29 },
    { 0x33b3bf0, SUC_lexicon_tags_9 },
    { 0x523b3bf1, SUC_lexicon_tags_476 },
    { 0, NULL },
    { 0, NULL },
    { 0x49db3bf5, SUC_lexicon_tags_37 },
    { 0x1ffb3bf6, SUC_lexicon_tags_17 },
    { 0xa45b3bf7, SUC_lexicon_tags_10 },
    { 0x1a1b3bf8, SUC_lexicon_tags_17 },
    { 0x4abb3bf9, SUC_lexicon_tags_47 },
    { 0xcd9b3bfa, SUC_lexicon_tags_17 },
    { 0xa0bb3bfb, SUC_lexicon_tags_37 },
    { 0xf57b3bfb, SUC_lexicon_tags_56 },
    { 0xa93b3bfd, SUC_lexicon_tags_16 },
    { 0x533b3bfa, SUC_lexicon_tags_61 },
    { 0x819b3bff, SUC_lexicon_tags_46 },
    { 0x43db3c00, SUC_lexicon_tags_84 },
    { 0x859b3bff, SUC_lexicon_tags_12 },
    { 0xe33b3c02, SUC_lexicon_tags_1 },
    { 0x4f1b3c03, SUC_lexicon_tags_2 },
    { 0xd8fb3c03, SUC_lexicon_tags_7 },
    { 0x75db3c05, SUC_lexicon_tags_13 },
    { 0xd7db3c05, SUC_lexicon_tags_17 },
    { 0x887b3c05, SUC_lexicon_tags_15 },
    { 0x71db3c04, SUC_lexicon_tags_23 },
    { 0xf5fb3c02, SUC_lexicon_tags_15 },
    { 0x4cfb3bf8, SUC_lexicon_tags_25 },
    { 0xadb3c00, SUC_lexicon_tags_17 },
    { 0x111b3c00, SUC_lexicon_tags_65 },
    { 0x4bbb3bfa, SUC_lexicon_tags_45 },
    { 0xb2fb3c02, SUC_lexicon_tags_25 },
    { 0x141b3c0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ebb3c11, SUC_lexicon_tags_23 },
    { 0x7ddb3c12, SUC_lexicon_tags_7 },
    { 0x3f9b3c12, SUC_lexicon_tags_12 },
    { 0x687b3c11, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edb3c18, SUC_lexicon_tags_46 },
    { 0x4c7b3c19, SUC_lexicon_tags_17 },
    { 0x5c3b3c1a, SUC_lexicon_tags_17 },
    { 0xb15b3c1b, SUC_lexicon_tags_43 },
    { 0x997b3c1c, SUC_lexicon_tags_6 },
    { 0xa81b3c1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bb3c24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x831b3c26, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x42bb3c28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bb3c2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5b3c31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfb3c34, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x863b3c36, SUC_lexicon_tags_37 },
    { 0x15fb3c37, SUC_lexicon_tags_25 },
    { 0x99bb3c37, SUC_lexicon_tags_7 },
    { 0x3a3b3c39, SUC_lexicon_tags_43 },
    { 0x1edb3c36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3b3c3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57b3c41, SUC_lexicon_tags_138 },
    { 0xe4db3c41, SUC_lexicon_tags_37 },
    { 0x43fb3c43, SUC_lexicon_tags_47 },
    { 0x195b3c43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd71b3c46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1b3c4b, SUC_lexicon_tags_25 },
    { 0x24db3c4b, SUC_lexicon_tags_15 },
    { 0x8a1b3c4b, SUC_lexicon_tags_19 },
    { 0xd29b3c4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611b3c52, SUC_lexicon_tags_45 },
    { 0xff9b3c52, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xde1b3c55, SUC_lexicon_tags_37 },
    { 0xafdb3c56, SUC_lexicon_tags_82 },
    { 0x2bb3c57, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59b3c5c, SUC_lexicon_tags_8 },
    { 0x93b3c5c, SUC_lexicon_tags_34 },
    { 0x9e5b3c5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17b3c62, SUC_lexicon_tags_14 },
    { 0xc5fb3c63, SUC_lexicon_tags_47 },
    { 0x259b3c62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77b3c6c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b3c71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849b3c75, SUC_lexicon_tags_48 },
    { 0xf55b3c76, SUC_lexicon_tags_39 },
    { 0xa1bb3c77, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691b3c7b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c7b3c7d, SUC_lexicon_tags_61 },
    { 0x533b3c7d, SUC_lexicon_tags_0 },
    { 0xf25b3c7d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621b3c83, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605b3c8b, SUC_lexicon_tags_15 },
    { 0x6c5b3c8b, SUC_lexicon_tags_27 },
    { 0xe91b3c8d, SUC_lexicon_tags_8 },
    { 0xca1b3c8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4bb3c90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bb3c94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75b3c97, SUC_lexicon_tags_25 },
    { 0x995b3c98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd97b3c9a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263b3c9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe3b3ca0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebb3ca5, SUC_lexicon_tags_18 },
    { 0xdc9b3ca5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29b3cab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xda5b3cad, SUC_lexicon_tags_8 },
    { 0x907b3cae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781b3cb6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fb3cba, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x375b3cbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83b3cc0, SUC_lexicon_tags_1 },
    { 0x6f7b3cc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523b3ccd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xedb3cd0, SUC_lexicon_tags_15 },
    { 0x7ebb3cd1, SUC_lexicon_tags_33 },
    { 0xd27b3cd1, SUC_lexicon_tags_25 },
    { 0xa2fb3cd3, SUC_lexicon_tags_9 },
    { 0x2b3b3cd2, SUC_lexicon_tags_27 },
    { 0xebbb3cd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553b3cda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3b3cde, SUC_lexicon_tags_257 },
    { 0x33bb3cde, SUC_lexicon_tags_56 },
    { 0x469b3cde, SUC_lexicon_tags_17 },
    { 0x359b3cdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x761b3ce3, SUC_lexicon_tags_39 },
    { 0x2fdb3ce4, SUC_lexicon_tags_29 },
    { 0xac9b3ce5, SUC_lexicon_tags_38 },
    { 0x91db3ce4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x305b3ce8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5fbb3cea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83b3ced, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1b3cf1, SUC_lexicon_tags_43 },
    { 0xd1db3cf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61b3cfa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x893b3cfd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1b3d01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905b3d06, SUC_lexicon_tags_33 },
    { 0x63b3d06, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425b3d0f, SUC_lexicon_tags_12 },
    { 0x2e7b3d10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6ffb3d12, SUC_lexicon_tags_0 },
    { 0x171b3d12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x393b3d16, SUC_lexicon_tags_49 },
    { 0x27b3d17, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x559b3d1a, SUC_lexicon_tags_25 },
    { 0xd5db3d1b, SUC_lexicon_tags_9 },
    { 0x9abb3d1c, SUC_lexicon_tags_27 },
    { 0x18bb3d1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687b3d26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffb3d29, SUC_lexicon_tags_12 },
    { 0xaf3b3d2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3bb3d2c, SUC_lexicon_tags_38 },
    { 0x379b3d2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4abb3d2f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bb3d33, SUC_lexicon_tags_10 },
    { 0x7d9b3d33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf99b3d36, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63b3d39, SUC_lexicon_tags_12 },
    { 0x857b3d3a, SUC_lexicon_tags_43 },
    { 0x1e9b3d3b, SUC_lexicon_tags_47 },
    { 0xdb7b3d3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84bb3d3e, SUC_lexicon_tags_12 },
    { 0x75bb3d3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295b3d46, SUC_lexicon_tags_15 },
    { 0x737b3d46, SUC_lexicon_tags_14 },
    { 0x2edb3d47, SUC_lexicon_tags_38 },
    { 0xe99b3d47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe67b3d4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee5b3d4d, SUC_lexicon_tags_7 },
    { 0xefdb3d4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf3b3d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419b3d56, SUC_lexicon_tags_2 },
    { 0xf2fb3d57, SUC_lexicon_tags_17 },
    { 0x783b3d58, SUC_lexicon_tags_15 },
    { 0x19bb3d56, SUC_lexicon_tags_48 },
    { 0xf09b3d5a, SUC_lexicon_tags_33 },
    { 0xf9db3d5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51b3d5e, SUC_lexicon_tags_12 },
    { 0xeb1b3d5e, SUC_lexicon_tags_30 },
    { 0x827b3d60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f1b3d62, SUC_lexicon_tags_55 },
    { 0xafb3d62, SUC_lexicon_tags_7 },
    { 0x1f5b3d62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9b3d69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb5b3d6b, SUC_lexicon_tags_44 },
    { 0x10db3d6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bb3d6f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35b3d71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fb3d75, SUC_lexicon_tags_43 },
    { 0xa21b3d75, SUC_lexicon_tags_14 },
    { 0xe21b3d77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abb3d7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x81db3d7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x119b3d80, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965b3d84, SUC_lexicon_tags_23 },
    { 0xdf5b3d85, SUC_lexicon_tags_12 },
    { 0xc0bb3d86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355b3d8a, SUC_lexicon_tags_15 },
    { 0xf87b3d8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb97b3d8d, SUC_lexicon_tags_12 },
    { 0x361b3d8e, SUC_lexicon_tags_63 },
    { 0x6abb3d8d, SUC_lexicon_tags_14 },
    { 0xa17b3d90, SUC_lexicon_tags_23 },
    { 0x4c1b3d91, SUC_lexicon_tags_17 },
    { 0x479b3d92, SUC_lexicon_tags_11 },
    { 0xfadb3d92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbb3d96, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x365b3d98, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7b3d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb05b3d9f, SUC_lexicon_tags_63 },
    { 0x6bfb3da0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705b3da7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3b3daa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267b3db2, SUC_lexicon_tags_37 },
    { 0x40fb3db2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7bbb3db5, SUC_lexicon_tags_33 },
    { 0x3a3b3db6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc5b3db8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe5fb3dba, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1b3dbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1b3dc2, SUC_lexicon_tags_37 },
    { 0x401b3dc3, SUC_lexicon_tags_35 },
    { 0x599b3dc2, SUC_lexicon_tags_11 },
    { 0xda3b3dc5, SUC_lexicon_tags_29 },
    { 0xebb3dc5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1b3dcc, SUC_lexicon_tags_15 },
    { 0x71fb3dcc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3dd3, SUC_lexicon_tags_7 },
    { 0x797b3dd4, SUC_lexicon_tags_43 },
    { 0x36bb3dd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5b3dda, SUC_lexicon_tags_17 },
    { 0xaa9b3ddb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8bb3ddd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf7b3ddf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785b3de3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x11fb3de5, SUC_lexicon_tags_7 },
    { 0xaffb3de6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913b3deb, SUC_lexicon_tags_2 },
    { 0xddfb3deb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69db3def, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5b3df2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1b3df6, SUC_lexicon_tags_37 },
    { 0x41bb3df7, SUC_lexicon_tags_63 },
    { 0xf85b3df8, SUC_lexicon_tags_37 },
    { 0xa75b3df9, SUC_lexicon_tags_38 },
    { 0xc0fb3dfa, SUC_lexicon_tags_11 },
    { 0x95fb3dfb, SUC_lexicon_tags_66 },
    { 0xd5bb3dfc, SUC_lexicon_tags_165 },
    { 0x707b3dfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cfb3dff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55db3e0a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b3e0d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3e10, SUC_lexicon_tags_8 },
    { 0x213b3e10, SUC_lexicon_tags_37 },
    { 0xf25b3e12, SUC_lexicon_tags_32 },
    { 0x7dbb3e11, SUC_lexicon_tags_17 },
    { 0xfd7b3e14, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487b3e1f, SUC_lexicon_tags_60 },
    { 0x3f3b3e20, SUC_lexicon_tags_6 },
    { 0xe87b3e21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1afb3e23, SUC_lexicon_tags_12 },
    { 0xd7bb3e24, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x703b3e27, SUC_lexicon_tags_63 },
    { 0x3cfb3e28, SUC_lexicon_tags_9 },
    { 0x2f9b3e27, SUC_lexicon_tags_37 },
    { 0xdd5b3e29, SUC_lexicon_tags_38 },
    { 0xb65b3e2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fb3e2f, SUC_lexicon_tags_2 },
    { 0x6fb3e30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30fb3e32, SUC_lexicon_tags_11 },
    { 0x431b3e33, SUC_lexicon_tags_10 },
    { 0x7f5b3e34, SUC_lexicon_tags_38 },
    { 0x251b3e35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe29b3e37, SUC_lexicon_tags_17 },
    { 0xaedb3e38, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587b3e3d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x503b3e3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbd7b3e41, SUC_lexicon_tags_25 },
    { 0xd15b3e42, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabb3e45, SUC_lexicon_tags_1 },
    { 0xd03b3e45, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x399b3e48, SUC_lexicon_tags_7 },
    { 0x92fb3e49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3b3e4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7b3e53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fb3e55, SUC_lexicon_tags_84 },
    { 0x1c3b3e55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x573b3e58, SUC_lexicon_tags_47 },
    { 0xa6fb3e58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x45bb3e5c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5b3e60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9b3e65, SUC_lexicon_tags_47 },
    { 0x54fb3e66, SUC_lexicon_tags_49 },
    { 0x4dbb3e66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3b3e6a, SUC_lexicon_tags_2 },
    { 0xb1db3e6b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x21fb3e6d, SUC_lexicon_tags_30 },
    { 0x6a3b3e6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515b3e73, SUC_lexicon_tags_5 },
    { 0x45bb3e74, SUC_lexicon_tags_25 },
    { 0x811b3e75, SUC_lexicon_tags_8 },
    { 0x14fb3e75, SUC_lexicon_tags_65 },
    { 0x977b3e76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x507b3e79, SUC_lexicon_tags_23 },
    { 0xb43b3e7a, SUC_lexicon_tags_82 },
    { 0x187b3e79, SUC_lexicon_tags_37 },
    { 0xec7b3e7c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85b3e7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x875b3e82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9b3e8a, SUC_lexicon_tags_17 },
    { 0x3f7b3e8a, SUC_lexicon_tags_37 },
    { 0x39db3e8b, SUC_lexicon_tags_32 },
    { 0x7abb3e8d, SUC_lexicon_tags_61 },
    { 0x647b3e8e, SUC_lexicon_tags_1 },
    { 0x835b3e8e, SUC_lexicon_tags_21 },
    { 0xe25b3e8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553b3e98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf11b3e9a, SUC_lexicon_tags_17 },
    { 0xf61b3e9b, SUC_lexicon_tags_0 },
    { 0xa23b3e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65b3ea3, SUC_lexicon_tags_77 },
    { 0x471b3ea4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d7b3ea6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47b3ea9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bb3eac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x495b3eae, SUC_lexicon_tags_25 },
    { 0x971b3eaf, SUC_lexicon_tags_25 },
    { 0xfb7b3eaf, SUC_lexicon_tags_43 },
    { 0xedb3eb1, SUC_lexicon_tags_25 },
    { 0x2f9b3eb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf67b3eb4, SUC_lexicon_tags_30 },
    { 0xf87b3eb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x441b3eb8, SUC_lexicon_tags_63 },
    { 0xd71b3eb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9b3ebd, SUC_lexicon_tags_19 },
    { 0x859b3ebe, SUC_lexicon_tags_17 },
    { 0xb81b3ebf, SUC_lexicon_tags_39 },
    { 0xe79b3ebe, SUC_lexicon_tags_220 },
    { 0xb61b3ebe, SUC_lexicon_tags_17 },
    { 0xc4bb3ec2, SUC_lexicon_tags_37 },
    { 0xcdb3ec3, SUC_lexicon_tags_43 },
    { 0xa8fb3ebd, SUC_lexicon_tags_21 },
    { 0x935b3ec5, SUC_lexicon_tags_65 },
    { 0x767b3ec6, SUC_lexicon_tags_46 },
    { 0x9db3ec7, SUC_lexicon_tags_28 },
    { 0x609b3ec7, SUC_lexicon_tags_30 },
    { 0x441b3ec5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10fb3ecb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf21b3ecd, SUC_lexicon_tags_37 },
    { 0xdfdb3ece, SUC_lexicon_tags_12 },
    { 0x983b3ecf, SUC_lexicon_tags_14 },
    { 0xbefb3ed0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc45b3ed2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fb3edc, SUC_lexicon_tags_65 },
    { 0xaf5b3edd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3db3ee5, SUC_lexicon_tags_17 },
    { 0x981b3ee6, SUC_lexicon_tags_0 },
    { 0x9d9b3ee7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957b3ef0, SUC_lexicon_tags_106 },
    { 0x387b3ef1, SUC_lexicon_tags_12 },
    { 0x90db3ef1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x219b3ef4, SUC_lexicon_tags_26 },
    { 0xa4bb3ef5, SUC_lexicon_tags_2 },
    { 0x533b3ef5, SUC_lexicon_tags_11 },
    { 0x34db3ef7, SUC_lexicon_tags_17 },
    { 0x429b3ef8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21b3f00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5b3f05, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717b3f0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4fb3f0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfb3f12, SUC_lexicon_tags_11 },
    { 0xc2fb3f13, SUC_lexicon_tags_17 },
    { 0xbe5b3f13, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247b3f1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc5bb3f1c, SUC_lexicon_tags_7 },
    { 0xd01b3f1d, SUC_lexicon_tags_25 },
    { 0x659b3f1e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x669b3f20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x969b3f22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7b3f26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfb3f29, SUC_lexicon_tags_0 },
    { 0x1b9b3f2a, SUC_lexicon_tags_23 },
    { 0x1d3b3f2a, SUC_lexicon_tags_17 },
    { 0x8ebb3f29, SUC_lexicon_tags_47 },
    { 0x605b3f2d, SUC_lexicon_tags_7 },
    { 0xabbb3f2e, SUC_lexicon_tags_32 },
    { 0x505b3f2d, SUC_lexicon_tags_30 },
    { 0xc7bb3f30, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357b3f37, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fb3f3c, SUC_lexicon_tags_66 },
    { 0x5adb3f3d, SUC_lexicon_tags_84 },
    { 0x2a7b3f3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbb3f48, SUC_lexicon_tags_37 },
    { 0x2ddb3f48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x947b3f4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09b3f4e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949b3f53, SUC_lexicon_tags_68 },
    { 0xed1b3f53, SUC_lexicon_tags_21 },
    { 0xf35b3f55, SUC_lexicon_tags_11 },
    { 0xc49b3f56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x69bb3f58, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7b3f5b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbb3f60, SUC_lexicon_tags_52 },
    { 0x531b3f61, SUC_lexicon_tags_11 },
    { 0xa0bb3f61, SUC_lexicon_tags_31 },
    { 0x731b3f63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1b3f66, SUC_lexicon_tags_25 },
    { 0x967b3f67, SUC_lexicon_tags_17 },
    { 0x8e5b3f66, SUC_lexicon_tags_11 },
    { 0xc03b3f69, SUC_lexicon_tags_37 },
    { 0x30db3f6a, SUC_lexicon_tags_7 },
    { 0xb3b3f6b, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xa43b3f6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b7b3f6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65b3f76, SUC_lexicon_tags_79 },
    { 0xb57b3f76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf27b3f79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xedfb3f7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa9b3f7d, SUC_lexicon_tags_17 },
    { 0x5ddb3f7e, SUC_lexicon_tags_43 },
    { 0x4fbb3f7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93b3f82, SUC_lexicon_tags_7 },
    { 0x3b1b3f83, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb0db3f85, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149b3f8a, SUC_lexicon_tags_60 },
    { 0xf6fb3f8b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41b3f8e, SUC_lexicon_tags_37 },
    { 0xad7b3f8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc3b3f91, SUC_lexicon_tags_0 },
    { 0xb01b3f92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9b3f96, SUC_lexicon_tags_1 },
    { 0xc7db3f96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1b3f9a, SUC_lexicon_tags_15 },
    { 0xb85b3f9a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187b3fa1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865b3fa6, SUC_lexicon_tags_7 },
    { 0xc41b3fa7, SUC_lexicon_tags_129 },
    { 0x89bb3fa7, SUC_lexicon_tags_27 },
    { 0xee9b3fa6, SUC_lexicon_tags_2 },
    { 0x67fb3faa, SUC_lexicon_tags_25 },
    { 0x61b3fa6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95b3fae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7b3fb1, SUC_lexicon_tags_2 },
    { 0x7edb3fb2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99b3fb5, SUC_lexicon_tags_19 },
    { 0xf89b3fb6, SUC_lexicon_tags_0 },
    { 0x633b3fb7, SUC_lexicon_tags_1 },
    { 0x7d9b3fb8, SUC_lexicon_tags_47 },
    { 0x589b3fb9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x521b3fbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3b3fc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd03b3fc3, SUC_lexicon_tags_25 },
    { 0xe67b3fc3, SUC_lexicon_tags_33 },
    { 0x6a1b3fc4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xcc3b3fc7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fb3fce, SUC_lexicon_tags_11 },
    { 0x2dbb3fcf, SUC_lexicon_tags_1 },
    { 0xa6bb3fce, SUC_lexicon_tags_17 },
    { 0x3dfb3fce, SUC_lexicon_tags_29 },
    { 0xc9b3fd2, SUC_lexicon_tags_35 },
    { 0x109b3fce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bb3fd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591b3fe0, SUC_lexicon_tags_56 },
    { 0x799b3fe1, SUC_lexicon_tags_17 },
    { 0x3a1b3fe0, SUC_lexicon_tags_33 },
    { 0xe9bb3fe3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb9bb3fe5, SUC_lexicon_tags_17 },
    { 0xad3b3fe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7b3fee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20fb3ff0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3b3ff5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x92bb3ff7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xae3b3ff9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcc7b3ffb, SUC_lexicon_tags_25 },
    { 0x5fbb3ffc, SUC_lexicon_tags_47 },
    { 0xc81b3ffd, SUC_lexicon_tags_37 },
    { 0xee7b3ffd, SUC_lexicon_tags_11 },
    { 0x87fb3fff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xffdb4001, SUC_lexicon_tags_25 },
    { 0x7b4002, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7b4006, SUC_lexicon_tags_269 },
    { 0x4b7b4007, SUC_lexicon_tags_7 },
    { 0x2d5b4008, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283b4013, SUC_lexicon_tags_1 },
    { 0xf2bb4014, SUC_lexicon_tags_82 },
    { 0x849b4015, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x209b4018, SUC_lexicon_tags_12 },
    { 0x577b4018, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63b401d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bb4020, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xa6bb4022, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5b4025, SUC_lexicon_tags_63 },
    { 0xa49b4026, SUC_lexicon_tags_35 },
    { 0xe1bb4027, SUC_lexicon_tags_21 },
    { 0x94bb4025, SUC_lexicon_tags_37 },
    { 0x3c7b4029, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x135b402b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x379b402d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09b4031, SUC_lexicon_tags_0 },
    { 0x6c1b4031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb4037, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25b403a, SUC_lexicon_tags_7 },
    { 0xca9b403b, SUC_lexicon_tags_27 },
    { 0x4bbb403a, SUC_lexicon_tags_21 },
    { 0xedbb403c, SUC_lexicon_tags_25 },
    { 0xed9b403e, SUC_lexicon_tags_7 },
    { 0xe77b403f, SUC_lexicon_tags_25 },
    { 0x681b403b, SUC_lexicon_tags_63 },
    { 0xcf5b4041, SUC_lexicon_tags_25 },
    { 0xd2fb4042, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e3b4044, SUC_lexicon_tags_2 },
    { 0x441b4045, SUC_lexicon_tags_9 },
    { 0xc0bb4046, SUC_lexicon_tags_19 },
    { 0xe83b4047, SUC_lexicon_tags_47 },
    { 0x187b4048, SUC_lexicon_tags_16 },
    { 0x1f3b4048, SUC_lexicon_tags_15 },
    { 0x6f1b404a, SUC_lexicon_tags_25 },
    { 0xf43b404a, SUC_lexicon_tags_23 },
    { 0x105b404c, SUC_lexicon_tags_23 },
    { 0xf95b404d, SUC_lexicon_tags_25 },
    { 0xb35b404e, SUC_lexicon_tags_191 },
    { 0x47b404c, SUC_lexicon_tags_27 },
    { 0x473b4049, SUC_lexicon_tags_17 },
    { 0x9b4051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x125b4053, SUC_lexicon_tags_31 },
    { 0xc31b4054, SUC_lexicon_tags_7 },
    { 0x1cfb4054, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49db4057, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11b405e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9b4061, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1b4065, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b4069, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39b406e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07b4073, SUC_lexicon_tags_23 },
    { 0x25b4073, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fb407c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x15db407f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x781b4081, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a1b4083, SUC_lexicon_tags_25 },
    { 0xa99b4083, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xab7b4086, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11b4089, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19b4090, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf65b4092, SUC_lexicon_tags_37 },
    { 0x5edb4092, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efb4098, SUC_lexicon_tags_26 },
    { 0xc9b4099, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39b409d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b40a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b40a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x627b40ac, SUC_lexicon_tags_43 },
    { 0xfd9b40ad, SUC_lexicon_tags_1 },
    { 0x6c1b40ac, SUC_lexicon_tags_28 },
    { 0xfa3b40ac, SUC_lexicon_tags_91 },
    { 0x1b3b40b0, SUC_lexicon_tags_31 },
    { 0xaebb40af, SUC_lexicon_tags_15 },
    { 0xe85b40b2, SUC_lexicon_tags_224 },
    { 0x16db40b3, SUC_lexicon_tags_38 },
    { 0x993b40b4, SUC_lexicon_tags_30 },
    { 0xe1bb40ad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b40ba, SUC_lexicon_tags_15 },
    { 0xf2fb40bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x901b40bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdb40c5, SUC_lexicon_tags_37 },
    { 0xb2db40c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x63db40c8, SUC_lexicon_tags_2 },
    { 0x16db40c9, SUC_lexicon_tags_19 },
    { 0x89b40ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87b40cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1b40d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9b40d5, SUC_lexicon_tags_11 },
    { 0x70bb40d6, SUC_lexicon_tags_17 },
    { 0x1a9b40d7, SUC_lexicon_tags_25 },
    { 0xf9db40d8, SUC_lexicon_tags_95 },
    { 0xf95b40d9, SUC_lexicon_tags_25 },
    { 0xc69b40d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219b40de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x203b40e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff5b40e2, SUC_lexicon_tags_55 },
    { 0xcfdb40e3, SUC_lexicon_tags_28 },
    { 0xc59b40e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9db40e6, SUC_lexicon_tags_60 },
    { 0x967b40e7, SUC_lexicon_tags_14 },
    { 0x875b40e6, SUC_lexicon_tags_25 },
    { 0x873b40e9, SUC_lexicon_tags_12 },
    { 0x4d5b40ea, SUC_lexicon_tags_14 },
    { 0xef7b40eb, SUC_lexicon_tags_27 },
    { 0xd2fb40ea, SUC_lexicon_tags_1 },
    { 0x9e9b40ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7b40f3, SUC_lexicon_tags_2 },
    { 0xf25b40f4, SUC_lexicon_tags_25 },
    { 0x905b40f5, SUC_lexicon_tags_37 },
    { 0xa59b40f6, SUC_lexicon_tags_27 },
    { 0xd1bb40f7, SUC_lexicon_tags_0 },
    { 0xd39b40f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3b40fc, SUC_lexicon_tags_55 },
    { 0x2fb40fd, SUC_lexicon_tags_7 },
    { 0x26bb40fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65b4101, SUC_lexicon_tags_28 },
    { 0xae7b4101, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891b4107, SUC_lexicon_tags_25 },
    { 0x141b4108, SUC_lexicon_tags_14 },
    { 0x4f1b4107, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1b4114, SUC_lexicon_tags_60 },
    { 0xfedb4115, SUC_lexicon_tags_11 },
    { 0x57b4114, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbb4119, SUC_lexicon_tags_60 },
    { 0xd23b411a, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19b4120, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1db4125, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549b4129, SUC_lexicon_tags_1 },
    { 0xcbbb412a, SUC_lexicon_tags_17 },
    { 0xcd3b412b, SUC_lexicon_tags_410 },
    { 0xa71b412c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137b4131, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9dfb4133, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37b413e, SUC_lexicon_tags_160 },
    { 0x8d3b413e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe6fb4141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9b414b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749b4152, SUC_lexicon_tags_0 },
    { 0xafbb4153, SUC_lexicon_tags_28 },
    { 0x22db4154, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b4158, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdb415d, SUC_lexicon_tags_4 },
    { 0xb07b415d, SUC_lexicon_tags_26 },
    { 0x40fb415f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bb4166, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b4169, SUC_lexicon_tags_1 },
    { 0xb5fb416a, SUC_lexicon_tags_37 },
    { 0x749b416b, SUC_lexicon_tags_9 },
    { 0xc37b416b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bb4170, SUC_lexicon_tags_10 },
    { 0xce1b4171, SUC_lexicon_tags_17 },
    { 0xdb5b4172, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x679b4175, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd49b4177, SUC_lexicon_tags_14 },
    { 0x271b4178, SUC_lexicon_tags_4 },
    { 0xf85b4179, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7b417c, SUC_lexicon_tags_45 },
    { 0xd27b417d, SUC_lexicon_tags_17 },
    { 0xc7bb417d, SUC_lexicon_tags_56 },
    { 0x1dbb417e, SUC_lexicon_tags_37 },
    { 0xe3b4180, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fb4186, SUC_lexicon_tags_14 },
    { 0xba5b4186, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3b418c, SUC_lexicon_tags_37 },
    { 0xd6bb418d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8cdb418f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285b4194, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63bb4196, SUC_lexicon_tags_82 },
    { 0xc63b4197, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x87b4199, SUC_lexicon_tags_11 },
    { 0x169b4199, SUC_lexicon_tags_28 },
    { 0xb3db419a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae5b419d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3b41a0, SUC_lexicon_tags_6 },
    { 0x94db41a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847b41aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bb41b1, SUC_lexicon_tags_37 },
    { 0x2d7b41b2, SUC_lexicon_tags_14 },
    { 0xcefb41b3, SUC_lexicon_tags_47 },
    { 0xa41b41b1, SUC_lexicon_tags_2 },
    { 0x679b41b5, SUC_lexicon_tags_79 },
    { 0xb0fb41b5, SUC_lexicon_tags_16 },
    { 0xac3b41b7, SUC_lexicon_tags_37 },
    { 0x457b41b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9db41bc, SUC_lexicon_tags_30 },
    { 0x63fb41bd, SUC_lexicon_tags_15 },
    { 0xefb41bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9e5b41c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fb41c3, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fb41cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x743b41d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebb41d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87b41d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8cfb41d9, SUC_lexicon_tags_7 },
    { 0x40bb41da, SUC_lexicon_tags_17 },
    { 0x60db41d9, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xe65b41dd, SUC_lexicon_tags_12 },
    { 0x9c1b41dd, SUC_lexicon_tags_25 },
    { 0x991b41df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b41e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ddb41e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x199b41eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x451b41ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1db41f4, SUC_lexicon_tags_25 },
    { 0x39bb41f5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x62db41f7, SUC_lexicon_tags_15 },
    { 0x67b41f8, SUC_lexicon_tags_42 },
    { 0xc11b41f9, SUC_lexicon_tags_1 },
    { 0xf11b41f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb41fe, SUC_lexicon_tags_17 },
    { 0x68bb41ff, SUC_lexicon_tags_45 },
    { 0xfdbb4200, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9b4206, SUC_lexicon_tags_11 },
    { 0x40bb4207, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fb420c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1b4210, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca3b4212, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x205b4214, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fb4218, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295b421c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399b4223, SUC_lexicon_tags_549 },
    { 0x87db4224, SUC_lexicon_tags_14 },
    { 0x8cbb4225, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf5db4227, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b422b, SUC_lexicon_tags_12 },
    { 0xa1bb422b, SUC_lexicon_tags_12 },
    { 0x11bb422d, SUC_lexicon_tags_26 },
    { 0xa9b422d, SUC_lexicon_tags_15 },
    { 0x511b422b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61b4233, SUC_lexicon_tags_19 },
    { 0xd13b4234, SUC_lexicon_tags_39 },
    { 0x801b4235, SUC_lexicon_tags_35 },
    { 0x35db4236, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafb4239, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x49b423b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bb4245, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfb424a, SUC_lexicon_tags_7 },
    { 0x73bb424b, SUC_lexicon_tags_27 },
    { 0xae7b424c, SUC_lexicon_tags_15 },
    { 0x867b424b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7b4252, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1b4256, SUC_lexicon_tags_14 },
    { 0x919b4256, SUC_lexicon_tags_30 },
    { 0xbafb4258, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21db425b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xef1b425d, SUC_lexicon_tags_26 },
    { 0xdcfb425e, SUC_lexicon_tags_37 },
    { 0x233b425e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1b4262, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x461b4264, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb4268, SUC_lexicon_tags_0 },
    { 0xe59b4269, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689b4270, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bb4278, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33b427a, SUC_lexicon_tags_13 },
    { 0x5b427a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x311b427d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69db427f, SUC_lexicon_tags_39 },
    { 0xf3db427f, SUC_lexicon_tags_9 },
    { 0xa0bb4280, SUC_lexicon_tags_25 },
    { 0x2c3b4280, SUC_lexicon_tags_39 },
    { 0x679b4280, SUC_lexicon_tags_7 },
    { 0x49bb4284, SUC_lexicon_tags_1 },
    { 0xdbdb4285, SUC_lexicon_tags_61 },
    { 0xd77b4286, SUC_lexicon_tags_37 },
    { 0x5bb4286, SUC_lexicon_tags_70 },
    { 0x507b4287, SUC_lexicon_tags_14 },
    { 0xfc5b4285, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7db428c, SUC_lexicon_tags_45 },
    { 0x2abb428d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xddb428f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b4292, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb51b4294, SUC_lexicon_tags_15 },
    { 0xabbb4295, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d5b4297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503b429f, SUC_lexicon_tags_7 },
    { 0x50db42a0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bb42a9, SUC_lexicon_tags_0 },
    { 0xd55b42aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa07b42ac, SUC_lexicon_tags_17 },
    { 0xc93b42ad, SUC_lexicon_tags_43 },
    { 0x893b42ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b42b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb42b7, SUC_lexicon_tags_15 },
    { 0xa0db42b8, SUC_lexicon_tags_68 },
    { 0xfa9b42b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53db42bb, SUC_lexicon_tags_65 },
    { 0x435b42bc, SUC_lexicon_tags_162 },
    { 0x8b5b42bc, SUC_lexicon_tags_8 },
    { 0x8e1b42bd, SUC_lexicon_tags_27 },
    { 0x675b42bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5b42c2, SUC_lexicon_tags_63 },
    { 0xa8bb42c3, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffb42ca, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1b42cd, SUC_lexicon_tags_7 },
    { 0x3ddb42ce, SUC_lexicon_tags_1 },
    { 0x377b42cf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587b42d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b42d6, SUC_lexicon_tags_2 },
    { 0xb79b42d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9b42db, SUC_lexicon_tags_47 },
    { 0x61bb42dc, SUC_lexicon_tags_2 },
    { 0x929b42dd, SUC_lexicon_tags_27 },
    { 0xc71b42de, SUC_lexicon_tags_1 },
    { 0x3d3b42dd, SUC_lexicon_tags_17 },
    { 0x8dbb42dd, SUC_lexicon_tags_63 },
    { 0x719b42dd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9db42e4, SUC_lexicon_tags_55 },
    { 0xd7db42e4, SUC_lexicon_tags_5 },
    { 0x883b42e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b42e9, SUC_lexicon_tags_27 },
    { 0x845b42ea, SUC_lexicon_tags_23 },
    { 0x779b42eb, SUC_lexicon_tags_11 },
    { 0x551b42eb, SUC_lexicon_tags_39 },
    { 0x513b42eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x815b42f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb42f4, SUC_lexicon_tags_2 },
    { 0xaf9b42f4, SUC_lexicon_tags_8 },
    { 0x723b42f5, SUC_lexicon_tags_56 },
    { 0x5fdb42f4, SUC_lexicon_tags_15 },
    { 0xd1b42f6, SUC_lexicon_tags_27 },
    { 0x6ffb42f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60db42ff, SUC_lexicon_tags_11 },
    { 0xb67b4300, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431b4304, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fb4307, SUC_lexicon_tags_37 },
    { 0x7dbb4308, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfb430c, SUC_lexicon_tags_28 },
    { 0x555b430d, SUC_lexicon_tags_63 },
    { 0xddfb430d, SUC_lexicon_tags_7 },
    { 0x50bb430e, SUC_lexicon_tags_32 },
    { 0x4f1b430c, SUC_lexicon_tags_7 },
    { 0xf0db4311, SUC_lexicon_tags_16 },
    { 0x817b430e, SUC_lexicon_tags_31 },
    { 0x44bb430d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x505b4315, SUC_lexicon_tags_31 },
    { 0xd13b4316, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fb4319, SUC_lexicon_tags_7 },
    { 0x1ddb431a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x431b431c, SUC_lexicon_tags_25 },
    { 0x95db431d, SUC_lexicon_tags_79 },
    { 0x143b431e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1b4322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdb432b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbb4336, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb8bb4338, SUC_lexicon_tags_8 },
    { 0xb4fb4339, SUC_lexicon_tags_30 },
    { 0x18bb433a, SUC_lexicon_tags_17 },
    { 0x343b433b, SUC_lexicon_tags_14 },
    { 0x599b433c, SUC_lexicon_tags_10 },
    { 0x14db433a, SUC_lexicon_tags_6 },
    { 0x5e5b433e, SUC_lexicon_tags_43 },
    { 0x3d9b433d, SUC_lexicon_tags_29 },
    { 0xb1bb4340, SUC_lexicon_tags_21 },
    { 0x9b7b4340, SUC_lexicon_tags_1 },
    { 0x983b4341, SUC_lexicon_tags_26 },
    { 0xe7bb4343, SUC_lexicon_tags_25 },
    { 0x69fb4344, SUC_lexicon_tags_17 },
    { 0x459b4345, SUC_lexicon_tags_17 },
    { 0x95db4345, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x181b4348, SUC_lexicon_tags_45 },
    { 0xe15b4348, SUC_lexicon_tags_15 },
    { 0x7c7b434a, SUC_lexicon_tags_14 },
    { 0x87db434b, SUC_lexicon_tags_2 },
    { 0x483b434b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x579b434e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09b4352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cbb4354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1b4358, SUC_lexicon_tags_17 },
    { 0x113b4359, SUC_lexicon_tags_66 },
    { 0x26bb4359, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3b435d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447b4367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x495b436a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a5b436c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeadb436f, SUC_lexicon_tags_37 },
    { 0x28bb4370, SUC_lexicon_tags_37 },
    { 0x501b4371, SUC_lexicon_tags_37 },
    { 0x883b4372, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddb4376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661b4383, SUC_lexicon_tags_1 },
    { 0xc5b4383, SUC_lexicon_tags_0 },
    { 0xc57b4385, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1b4388, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e3b438a, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9b4390, SUC_lexicon_tags_2 },
    { 0x415b4391, SUC_lexicon_tags_8 },
    { 0xc93b4391, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfb4395, SUC_lexicon_tags_28 },
    { 0xc09b4395, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc1fb4398, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5b439f, SUC_lexicon_tags_37 },
    { 0x49fb43a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4fdb43a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x109b43a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfb43a7, SUC_lexicon_tags_37 },
    { 0x273b43a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a1b43aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f1b43ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfb43b4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9b43b7, SUC_lexicon_tags_1 },
    { 0x765b43b8, SUC_lexicon_tags_99 },
    { 0x479b43b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x323b43bb, SUC_lexicon_tags_0 },
    { 0xe3db43bb, SUC_lexicon_tags_3 },
    { 0xe93b43bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1b43c4, SUC_lexicon_tags_19 },
    { 0xc51b43c5, SUC_lexicon_tags_11 },
    { 0xc89b43c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9d9b43c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5bb43ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645b43d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13b43d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79b43dc, SUC_lexicon_tags_47 },
    { 0xe6bb43dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425b43e4, SUC_lexicon_tags_33 },
    { 0x32db43e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a1b43e7, SUC_lexicon_tags_27 },
    { 0x773b43e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69b43eb, SUC_lexicon_tags_49 },
    { 0x5f7b43ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5b43f2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8edb43f4, SUC_lexicon_tags_201 },
    { 0x58bb43f4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5b43fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x49db43fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bb4404, SUC_lexicon_tags_64 },
    { 0x22db4405, SUC_lexicon_tags_418 },
    { 0x185b4405, SUC_lexicon_tags_1 },
    { 0x2bbb4407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcffb4409, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfabb440b, SUC_lexicon_tags_37 },
    { 0x113b440c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x523b440e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fb441a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fb4424, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173b4429, SUC_lexicon_tags_12 },
    { 0x425b442a, SUC_lexicon_tags_26 },
    { 0x579b442a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30db442f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5b4432, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbb4438, SUC_lexicon_tags_0 },
    { 0x819b4438, SUC_lexicon_tags_0 },
    { 0x6f1b443a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x107b443c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb7b443e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645b4442, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9b444a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa55b444c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fb444f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebb445a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x991b445c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9b445f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bb4464, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fb446a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdb446d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99db4474, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afb4477, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e9b4479, SUC_lexicon_tags_4 },
    { 0x101b447a, SUC_lexicon_tags_46 },
    { 0x8b1b4479, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe8fb447d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33b4481, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x425b4484, SUC_lexicon_tags_29 },
    { 0xf83b4484, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1b448b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59b448f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a3b4491, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf89b4493, SUC_lexicon_tags_14 },
    { 0x1a7b4493, SUC_lexicon_tags_71 },
    { 0xe05b4495, SUC_lexicon_tags_37 },
    { 0x1edb4494, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1b449b, SUC_lexicon_tags_31 },
    { 0xcebb449c, SUC_lexicon_tags_55 },
    { 0x76fb449d, SUC_lexicon_tags_15 },
    { 0xcc9b449e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703b44a2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x139b44a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987b44aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb3b44ac, SUC_lexicon_tags_27 },
    { 0xd07b44ac, SUC_lexicon_tags_19 },
    { 0xc25b44ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf69b44b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0db44b3, SUC_lexicon_tags_28 },
    { 0xab7b44b4, SUC_lexicon_tags_37 },
    { 0xcd7b44b5, SUC_lexicon_tags_17 },
    { 0xb5bb44b6, SUC_lexicon_tags_15 },
    { 0x6c1b44b4, SUC_lexicon_tags_10 },
    { 0x735b44b7, SUC_lexicon_tags_1463 },
    { 0xc93b44b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe7b44bb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1fb44bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813b44c3, SUC_lexicon_tags_61 },
    { 0x77fb44c4, SUC_lexicon_tags_11 },
    { 0x367b44c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70db44c7, SUC_lexicon_tags_15 },
    { 0x335b44c8, SUC_lexicon_tags_27 },
    { 0x5fbb44c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3b44cc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x97bb44ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3b44d2, SUC_lexicon_tags_18 },
    { 0xa35b44d3, SUC_lexicon_tags_13 },
    { 0x2ffb44d4, SUC_lexicon_tags_8 },
    { 0xb8db44d3, SUC_lexicon_tags_9 },
    { 0x851b44d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b44da, SUC_lexicon_tags_37 },
    { 0xa3db44db, SUC_lexicon_tags_26 },
    { 0xa01b44dc, SUC_lexicon_tags_66 },
    { 0x291b44da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25b44e1, SUC_lexicon_tags_29 },
    { 0xf1bb44e2, SUC_lexicon_tags_17 },
    { 0xa85b44e3, SUC_lexicon_tags_18 },
    { 0xe7fb44e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xac5b44e6, SUC_lexicon_tags_14 },
    { 0x741b44e6, SUC_lexicon_tags_60 },
    { 0x75b44e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfb44f0, SUC_lexicon_tags_82 },
    { 0x6c7b44f1, SUC_lexicon_tags_27 },
    { 0x815b44f1, SUC_lexicon_tags_17 },
    { 0x40db44f3, SUC_lexicon_tags_25 },
    { 0x6c1b44f2, SUC_lexicon_tags_1 },
    { 0x43fb44f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdb44fa, SUC_lexicon_tags_6 },
    { 0x7e1b44fa, SUC_lexicon_tags_12 },
    { 0x159b44fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbb4500, SUC_lexicon_tags_15 },
    { 0x297b4501, SUC_lexicon_tags_60 },
    { 0x94fb4502, SUC_lexicon_tags_236 },
    { 0x53bb4503, SUC_lexicon_tags_83 },
    { 0xc1b4504, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x65b4507, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfb450f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95b4512, SUC_lexicon_tags_37 },
    { 0x3fdb4513, SUC_lexicon_tags_25 },
    { 0x3f5b4514, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x109b4517, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57b451c, SUC_lexicon_tags_15 },
    { 0xe21b451c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x303b451f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499b4523, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ddb4525, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x281b4528, SUC_lexicon_tags_87 },
    { 0x37fb4529, SUC_lexicon_tags_7 },
    { 0x5b5b452a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115b452f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555b4533, SUC_lexicon_tags_23 },
    { 0xa21b4534, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665b453a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5b4540, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e1b4542, SUC_lexicon_tags_64 },
    { 0x58fb4543, SUC_lexicon_tags_92 },
    { 0x201b4542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x267b4547, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fb454a, SUC_lexicon_tags_207 },
    { 0x637b454b, SUC_lexicon_tags_65 },
    { 0xff3b454c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x68bb454e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bb4558, SUC_lexicon_tags_9 },
    { 0x47fb4558, SUC_lexicon_tags_8 },
    { 0xd27b455a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d1b455c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x785b455e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbb4565, SUC_lexicon_tags_27 },
    { 0x761b4566, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999b456a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa65b456c, SUC_lexicon_tags_37 },
    { 0x26bb456d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x407b4570, SUC_lexicon_tags_23 },
    { 0x709b4571, SUC_lexicon_tags_31 },
    { 0xcbdb4570, SUC_lexicon_tags_31 },
    { 0xb15b4571, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15b4576, SUC_lexicon_tags_17 },
    { 0xac7b4576, SUC_lexicon_tags_1 },
    { 0x915b4576, SUC_lexicon_tags_33 },
    { 0x1b5b4579, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x227b457c, SUC_lexicon_tags_1 },
    { 0x3f5b457c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69bb457f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889b4585, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6db4588, SUC_lexicon_tags_27 },
    { 0xdfdb4589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bb458d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb4590, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fb4593, SUC_lexicon_tags_28 },
    { 0xce3b4594, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3b4599, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28db459d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1b45a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x101b45a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519b45aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7dfb45ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3b45b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfb45b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715b45ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5b45bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x37bb45c0, SUC_lexicon_tags_17 },
    { 0x223b45c1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fb45c5, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bb45cf, SUC_lexicon_tags_37 },
    { 0xb81b45d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abb45d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39b45d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdb5b45db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93b45df, SUC_lexicon_tags_12 },
    { 0xfe5b45e0, SUC_lexicon_tags_25 },
    { 0x6efb45e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa27b45e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x83bb45e6, SUC_lexicon_tags_17 },
    { 0xa77b45e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40bb45e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fb45ef, SUC_lexicon_tags_37 },
    { 0xff7b45f0, SUC_lexicon_tags_57 },
    { 0x87db45f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721b45f6, SUC_lexicon_tags_31 },
    { 0x151b45f6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b45fa, SUC_lexicon_tags_68 },
    { 0xd95b45fb, SUC_lexicon_tags_29 },
    { 0x239b45fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb4600, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfb4604, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x209b4606, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25b4608, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x147b460b, SUC_lexicon_tags_43 },
    { 0xc1b460c, SUC_lexicon_tags_2 },
    { 0x28db460d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf87b460f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa81b4611, SUC_lexicon_tags_15 },
    { 0x28fb4612, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x749b4614, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x38db4617, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x135b461a, SUC_lexicon_tags_65 },
    { 0x44fb461b, SUC_lexicon_tags_66 },
    { 0x8b3b461c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605b4620, SUC_lexicon_tags_11 },
    { 0x82fb4621, SUC_lexicon_tags_11 },
    { 0xaefb4620, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05b4627, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbb4629, SUC_lexicon_tags_33 },
    { 0xf9b462a, SUC_lexicon_tags_62 },
    { 0x233b462b, SUC_lexicon_tags_19 },
    { 0x195b462b, SUC_lexicon_tags_28 },
    { 0xd49b462d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6e1b462f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9b4632, SUC_lexicon_tags_56 },
    { 0xcffb4633, SUC_lexicon_tags_37 },
    { 0xe6bb4632, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35db463d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc7b463f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbb4643, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb4646, SUC_lexicon_tags_28 },
    { 0x4afb4647, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403b464e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709b4662, SUC_lexicon_tags_18 },
    { 0x973b4663, SUC_lexicon_tags_6 },
    { 0x505b4664, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5b466a, SUC_lexicon_tags_104 },
    { 0xc69b466b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe99b466d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15b4670, SUC_lexicon_tags_3 },
    { 0x921b4671, SUC_lexicon_tags_27 },
    { 0xb87b4670, SUC_lexicon_tags_47 },
    { 0x15bb4670, SUC_lexicon_tags_27 },
    { 0xebdb4671, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9b4677, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d3b4679, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x635b467b, SUC_lexicon_tags_27 },
    { 0xb2db467c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x247b467f, SUC_lexicon_tags_2 },
    { 0x77b467f, SUC_lexicon_tags_15 },
    { 0xa23b4681, SUC_lexicon_tags_17 },
    { 0xc93b4681, SUC_lexicon_tags_37 },
    { 0x3f1b4683, SUC_lexicon_tags_57 },
    { 0x5fdb4684, SUC_lexicon_tags_12 },
    { 0x11b4684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95b468a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447b468e, SUC_lexicon_tags_11 },
    { 0x4dfb468e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2db4692, SUC_lexicon_tags_38 },
    { 0x7fbb4692, SUC_lexicon_tags_23 },
    { 0xc8bb4694, SUC_lexicon_tags_37 },
    { 0x9cfb4695, SUC_lexicon_tags_3 },
    { 0x9a3b4696, SUC_lexicon_tags_15 },
    { 0x467b4697, SUC_lexicon_tags_38 },
    { 0xabb4697, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6a3b469a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3db46a0, SUC_lexicon_tags_43 },
    { 0x20bb46a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x917b46a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeddb46a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fb46a8, SUC_lexicon_tags_25 },
    { 0x509b46a8, SUC_lexicon_tags_30 },
    { 0xa5b46aa, SUC_lexicon_tags_27 },
    { 0x21db46a8, SUC_lexicon_tags_23 },
    { 0x9b46ac, SUC_lexicon_tags_74 },
    { 0xc93b46ad, SUC_lexicon_tags_80 },
    { 0x1bb46a8, SUC_lexicon_tags_37 },
    { 0xa45b46af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdb46b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fb46b6, SUC_lexicon_tags_29 },
    { 0x2efb46b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x617b46b9, SUC_lexicon_tags_21 },
    { 0xe17b46b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73fb46bc, SUC_lexicon_tags_56 },
    { 0x6edb46bc, SUC_lexicon_tags_37 },
    { 0xc07b46bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7bb46c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b46c3, SUC_lexicon_tags_37 },
    { 0xf0fb46c3, SUC_lexicon_tags_79 },
    { 0xe41b46c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddb46c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x351b46ca, SUC_lexicon_tags_2 },
    { 0x9a7b46ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x695b46cd, SUC_lexicon_tags_6 },
    { 0x239b46cd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6db46d2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbb46d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6db46d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5b46db, SUC_lexicon_tags_12 },
    { 0x9c7b46dc, SUC_lexicon_tags_37 },
    { 0x113b46dd, SUC_lexicon_tags_254 },
    { 0x5a9b46de, SUC_lexicon_tags_19 },
    { 0x64bb46df, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5b46e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa9b46e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7b46e7, SUC_lexicon_tags_85 },
    { 0x89bb46e8, SUC_lexicon_tags_19 },
    { 0xe7fb46e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5b46ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933b46f1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3f7b46f3, SUC_lexicon_tags_25 },
    { 0x527b46f4, SUC_lexicon_tags_27 },
    { 0x911b46f5, SUC_lexicon_tags_25 },
    { 0xbb3b46f3, SUC_lexicon_tags_11 },
    { 0xe67b46f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x713b46fa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8e3b46fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe99b46fe, SUC_lexicon_tags_25 },
    { 0xabdb46ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd51b4701, SUC_lexicon_tags_3 },
    { 0x25db4702, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25b4706, SUC_lexicon_tags_17 },
    { 0x893b4706, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bb470a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39b470d, SUC_lexicon_tags_29 },
    { 0x75fb470d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbb4713, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8dfb4715, SUC_lexicon_tags_27 },
    { 0xbc7b4715, SUC_lexicon_tags_37 },
    { 0x16bb4717, SUC_lexicon_tags_55 },
    { 0x1efb4718, SUC_lexicon_tags_37 },
    { 0xbc7b4716, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9db471f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x19db4721, SUC_lexicon_tags_27 },
    { 0x99fb4721, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21b4725, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3efb4727, SUC_lexicon_tags_26 },
    { 0x1d3b4728, SUC_lexicon_tags_22 },
    { 0xeffb4728, SUC_lexicon_tags_11 },
    { 0xddb4727, SUC_lexicon_tags_25 },
    { 0xc4bb472b, SUC_lexicon_tags_28 },
    { 0x153b472b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bb4736, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6f5b4738, SUC_lexicon_tags_15 },
    { 0x2b1b4739, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10db473d, SUC_lexicon_tags_60 },
    { 0xd83b473e, SUC_lexicon_tags_33 },
    { 0x7fbb473e, SUC_lexicon_tags_14 },
    { 0x69db4740, SUC_lexicon_tags_11 },
    { 0xf3bb4741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1b4744, SUC_lexicon_tags_26 },
    { 0xd3db4745, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0fb4747, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb71b4749, SUC_lexicon_tags_68 },
    { 0x2e1b474a, SUC_lexicon_tags_19 },
    { 0xca1b474b, SUC_lexicon_tags_15 },
    { 0x6d1b474b, SUC_lexicon_tags_25 },
    { 0x785b474c, SUC_lexicon_tags_17 },
    { 0x471b474e, SUC_lexicon_tags_1 },
    { 0x7d5b474f, SUC_lexicon_tags_2 },
    { 0xbbfb4750, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7b4754, SUC_lexicon_tags_25 },
    { 0x71fb4755, SUC_lexicon_tags_37 },
    { 0x539b4754, SUC_lexicon_tags_49 },
    { 0x293b4757, SUC_lexicon_tags_17 },
    { 0xe6bb4755, SUC_lexicon_tags_43 },
    { 0xcddb4759, SUC_lexicon_tags_12 },
    { 0x595b475a, SUC_lexicon_tags_1 },
    { 0x9b7b475b, SUC_lexicon_tags_23 },
    { 0x70fb4758, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7b4760, SUC_lexicon_tags_70 },
    { 0xf8fb4761, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645b4766, SUC_lexicon_tags_37 },
    { 0xfcbb4766, SUC_lexicon_tags_61 },
    { 0xfc7b4768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbb476b, SUC_lexicon_tags_17 },
    { 0xcddb476c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe3db476e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b4776, SUC_lexicon_tags_37 },
    { 0x87fb4776, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebb477b, SUC_lexicon_tags_17 },
    { 0x64bb477c, SUC_lexicon_tags_27 },
    { 0x835b477d, SUC_lexicon_tags_15 },
    { 0x2fbb477b, SUC_lexicon_tags_2 },
    { 0x407b477e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781b4783, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13b4789, SUC_lexicon_tags_28 },
    { 0x75db478a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1b478d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71b4790, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679b4795, SUC_lexicon_tags_0 },
    { 0xc6fb4795, SUC_lexicon_tags_7 },
    { 0xe59b4797, SUC_lexicon_tags_6 },
    { 0xce5b4798, SUC_lexicon_tags_15 },
    { 0xef3b4796, SUC_lexicon_tags_13 },
    { 0xc8db4797, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x69db479c, SUC_lexicon_tags_11 },
    { 0x65fb479d, SUC_lexicon_tags_65 },
    { 0x8b1b479e, SUC_lexicon_tags_49 },
    { 0x33bb479f, SUC_lexicon_tags_27 },
    { 0xe4fb47a0, SUC_lexicon_tags_39 },
    { 0x5abb47a1, SUC_lexicon_tags_4 },
    { 0x615b47a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209b47b6, SUC_lexicon_tags_2 },
    { 0xeb5b47b6, SUC_lexicon_tags_27 },
    { 0x7b5b47b6, SUC_lexicon_tags_27 },
    { 0xe7fb47b9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x935b47bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x50fb47be, SUC_lexicon_tags_7 },
    { 0xf4bb47bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47fb47c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xca1b47c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bb47c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85b47ce, SUC_lexicon_tags_37 },
    { 0x51bb47cf, SUC_lexicon_tags_1 },
    { 0x4f7b47cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x591b47d2, SUC_lexicon_tags_13 },
    { 0x22fb47d2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15b47db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70bb47dd, SUC_lexicon_tags_37 },
    { 0xa57b47dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853b47e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297b47e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3db47ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcbdb47ef, SUC_lexicon_tags_25 },
    { 0x2cbb47f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83b47f3, SUC_lexicon_tags_15 },
    { 0xadfb47f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75b47f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7b47fa, SUC_lexicon_tags_39 },
    { 0xce7b47fb, SUC_lexicon_tags_7 },
    { 0x38db47fa, SUC_lexicon_tags_27 },
    { 0xdcbb47fd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99db4804, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3b480d, SUC_lexicon_tags_9 },
    { 0xb67b480d, SUC_lexicon_tags_12 },
    { 0x527b480f, SUC_lexicon_tags_1 },
    { 0xe21b480d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde1b4812, SUC_lexicon_tags_38 },
    { 0x3d1b4812, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbb481a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x695b481c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1b481f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29b4823, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb4827, SUC_lexicon_tags_19 },
    { 0xef3b4828, SUC_lexicon_tags_33 },
    { 0x16bb4829, SUC_lexicon_tags_14 },
    { 0xe63b4829, SUC_lexicon_tags_28 },
    { 0xd67b482b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66fb482d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdefb482f, SUC_lexicon_tags_15 },
    { 0xea9b4830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3cbb4832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bb483c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x385b483e, SUC_lexicon_tags_2 },
    { 0x7f5b483f, SUC_lexicon_tags_68 },
    { 0xc57b483e, SUC_lexicon_tags_27 },
    { 0x1b4841, SUC_lexicon_tags_47 },
    { 0x443b4842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdb4845, SUC_lexicon_tags_37 },
    { 0x5a5b4846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29b4849, SUC_lexicon_tags_6 },
    { 0x40db484a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x647b484c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45b4853, SUC_lexicon_tags_84 },
    { 0x6db4853, SUC_lexicon_tags_48 },
    { 0x6c5b4855, SUC_lexicon_tags_0 },
    { 0xda9b4856, SUC_lexicon_tags_1 },
    { 0x7d5b4857, SUC_lexicon_tags_28 },
    { 0x7bfb4853, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611b4869, SUC_lexicon_tags_12 },
    { 0x5b9b4869, SUC_lexicon_tags_37 },
    { 0xc29b486b, SUC_lexicon_tags_33 },
    { 0x1d7b486c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdb486f, SUC_lexicon_tags_63 },
    { 0x843b486f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1b4874, SUC_lexicon_tags_11 },
    { 0xc3fb4874, SUC_lexicon_tags_66 },
    { 0xcb9b4876, SUC_lexicon_tags_9 },
    { 0x38fb4874, SUC_lexicon_tags_15 },
    { 0xd8fb4878, SUC_lexicon_tags_37 },
    { 0x301b4875, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1b487e, SUC_lexicon_tags_141 },
    { 0x35bb487e, SUC_lexicon_tags_56 },
    { 0x43db4880, SUC_lexicon_tags_15 },
    { 0x33bb4880, SUC_lexicon_tags_14 },
    { 0x24fb487e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdb4885, SUC_lexicon_tags_12 },
    { 0xea9b4886, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd05b4888, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0xd5b488a, SUC_lexicon_tags_25 },
    { 0x41bb488b, SUC_lexicon_tags_2 },
    { 0x3d1b488b, SUC_lexicon_tags_7 },
    { 0x7cfb488d, SUC_lexicon_tags_37 },
    { 0x3b1b488e, SUC_lexicon_tags_25 },
    { 0x8d1b488f, SUC_lexicon_tags_40 },
    { 0x83bb4890, SUC_lexicon_tags_49 },
    { 0x855b4890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97fb4893, SUC_lexicon_tags_29 },
    { 0x2adb4894, SUC_lexicon_tags_8 },
    { 0xaabb4895, SUC_lexicon_tags_17 },
    { 0x16fb4895, SUC_lexicon_tags_45 },
    { 0xd1fb4897, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4f3b4899, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bb48a0, SUC_lexicon_tags_114 },
    { 0xaddb48a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x971b48a3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1b48a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13b48a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x937b48aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcffb48ac, SUC_lexicon_tags_7 },
    { 0xd19b48ad, SUC_lexicon_tags_14 },
    { 0xf43b48ad, SUC_lexicon_tags_15 },
    { 0x91bb48af, SUC_lexicon_tags_17 },
    { 0x30fb48b0, SUC_lexicon_tags_17 },
    { 0xb2bb48b0, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361b48b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x729b48b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11db48be, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9b48c1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x599b48c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7db48c9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x30fb48cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9b48cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57db48d1, SUC_lexicon_tags_10 },
    { 0x3e9b48d1, SUC_lexicon_tags_11 },
    { 0x199b48d3, SUC_lexicon_tags_11 },
    { 0xd95b48d3, SUC_lexicon_tags_62 },
    { 0xdbbb48d5, SUC_lexicon_tags_6 },
    { 0xd9bb48d6, SUC_lexicon_tags_26 },
    { 0xd7db48d4, SUC_lexicon_tags_10 },
    { 0x5d7b48d5, SUC_lexicon_tags_8 },
    { 0x765b48d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bb48dd, SUC_lexicon_tags_37 },
    { 0xaffb48de, SUC_lexicon_tags_43 },
    { 0xd41b48df, SUC_lexicon_tags_60 },
    { 0xf97b48df, SUC_lexicon_tags_13 },
    { 0x167b48e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x45b48e3, SUC_lexicon_tags_19 },
    { 0x4e1b48e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4bb48e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5b48ed, SUC_lexicon_tags_17 },
    { 0x64fb48ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x829b48f0, SUC_lexicon_tags_19 },
    { 0xe01b48f1, SUC_lexicon_tags_11 },
    { 0x389b48f1, SUC_lexicon_tags_32 },
    { 0xea3b48f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb48f8, SUC_lexicon_tags_1 },
    { 0x457b48f9, SUC_lexicon_tags_6 },
    { 0xc99b48f9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993b4901, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3db4904, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfb4909, SUC_lexicon_tags_39 },
    { 0xabb490a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cdb490c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9b490f, SUC_lexicon_tags_19 },
    { 0x529b490f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x649b4912, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07b4919, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21b4922, SUC_lexicon_tags_37 },
    { 0x44db4923, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301b492a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14bb492c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb492f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73b4933, SUC_lexicon_tags_14 },
    { 0xba1b4933, SUC_lexicon_tags_11 },
    { 0x667b4935, SUC_lexicon_tags_28 },
    { 0x6db4936, SUC_lexicon_tags_74 },
    { 0xcedb4937, SUC_lexicon_tags_36 },
    { 0x2a7b4937, SUC_lexicon_tags_1 },
    { 0x37b4937, SUC_lexicon_tags_25 },
    { 0x483b493a, SUC_lexicon_tags_38 },
    { 0xe17b493b, SUC_lexicon_tags_55 },
    { 0x723b4937, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb4942, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44fb4944, SUC_lexicon_tags_33 },
    { 0xb6fb4945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8db4947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7dfb4949, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63b494d, SUC_lexicon_tags_38 },
    { 0x41db494e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41db4951, SUC_lexicon_tags_29 },
    { 0x961b4952, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1b4955, SUC_lexicon_tags_14 },
    { 0x495b4956, SUC_lexicon_tags_19 },
    { 0x47db4955, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48fb495c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36db4962, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b4965, SUC_lexicon_tags_87 },
    { 0x317b4966, SUC_lexicon_tags_27 },
    { 0x725b4965, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b496c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811b4976, SUC_lexicon_tags_38 },
    { 0xb87b4977, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4cfb4979, SUC_lexicon_tags_18 },
    { 0x729b497a, SUC_lexicon_tags_42 },
    { 0xb65b497b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bb497e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869b4987, SUC_lexicon_tags_5 },
    { 0xbedb4987, SUC_lexicon_tags_68 },
    { 0xeb1b4989, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xa9db498b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989b4990, SUC_lexicon_tags_25 },
    { 0x35bb4991, SUC_lexicon_tags_7 },
    { 0x84fb4992, SUC_lexicon_tags_1 },
    { 0x1db4993, SUC_lexicon_tags_17 },
    { 0x379b4993, SUC_lexicon_tags_25 },
    { 0x23b4994, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabb4999, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfb49a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69b49a5, SUC_lexicon_tags_25 },
    { 0xf79b49a6, SUC_lexicon_tags_1 },
    { 0x8f9b49a7, SUC_lexicon_tags_8 },
    { 0x635b49a7, SUC_lexicon_tags_92 },
    { 0x479b49a7, SUC_lexicon_tags_10 },
    { 0x155b49a9, SUC_lexicon_tags_25 },
    { 0x869b49a7, SUC_lexicon_tags_1 },
    { 0xe9fb49ac, SUC_lexicon_tags_15 },
    { 0x53db49a7, SUC_lexicon_tags_19 },
    { 0x1e3b49a9, SUC_lexicon_tags_17 },
    { 0x3e9b49af, SUC_lexicon_tags_48 },
    { 0x427b49ae, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x145b49b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fb49b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5b49ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb49be, SUC_lexicon_tags_15 },
    { 0x4f9b49bf, SUC_lexicon_tags_0 },
    { 0x363b49bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64db49c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81b49c7, SUC_lexicon_tags_12 },
    { 0x9b7b49c8, SUC_lexicon_tags_13 },
    { 0x893b49c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cfb49cb, SUC_lexicon_tags_49 },
    { 0x925b49cb, SUC_lexicon_tags_15 },
    { 0x3f1b49cb, SUC_lexicon_tags_66 },
    { 0x8ffb49ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bb49d3, SUC_lexicon_tags_17 },
    { 0x3f1b49d4, SUC_lexicon_tags_27 },
    { 0xd41b49d4, SUC_lexicon_tags_27 },
    { 0x65bb49d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fb49da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237b49e2, SUC_lexicon_tags_2 },
    { 0x27b49e2, SUC_lexicon_tags_45 },
    { 0x37bb49e3, SUC_lexicon_tags_13 },
    { 0x35bb49e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7b49f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbb49f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7b49ff, SUC_lexicon_tags_7 },
    { 0xe65b4a00, SUC_lexicon_tags_7 },
    { 0xa17b4a00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x63db4a04, SUC_lexicon_tags_15 },
    { 0xe6bb4a05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b4a09, SUC_lexicon_tags_2 },
    { 0xe8bb4a09, SUC_lexicon_tags_17 },
    { 0x409b4a0a, SUC_lexicon_tags_31 },
    { 0x73db4a0c, SUC_lexicon_tags_7 },
    { 0x263b4a0d, SUC_lexicon_tags_21 },
    { 0xa7fb4a0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fb4a12, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1db4a14, SUC_lexicon_tags_23 },
    { 0x3a5b4a14, SUC_lexicon_tags_12 },
    { 0x73bb4a16, SUC_lexicon_tags_2 },
    { 0x7d1b4a17, SUC_lexicon_tags_0 },
    { 0xfd3b4a17, SUC_lexicon_tags_38 },
    { 0xdb5b4a17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70db4a1f, SUC_lexicon_tags_16 },
    { 0xfb3b4a1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39b4a2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfb4a2f, SUC_lexicon_tags_47 },
    { 0xc21b4a2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909b4a40, SUC_lexicon_tags_63 },
    { 0x675b4a41, SUC_lexicon_tags_49 },
    { 0x5d3b4a42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x691b4a45, SUC_lexicon_tags_27 },
    { 0x431b4a45, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b4a50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4fbb4a52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fb4a56, SUC_lexicon_tags_1 },
    { 0x235b4a56, SUC_lexicon_tags_11 },
    { 0xcc5b4a58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2db4a5a, SUC_lexicon_tags_27 },
    { 0x565b4a5b, SUC_lexicon_tags_11 },
    { 0xbcdb4a5b, SUC_lexicon_tags_15 },
    { 0xe07b4a5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64fb4a60, SUC_lexicon_tags_10 },
    { 0x6f7b4a61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfdb4a63, SUC_lexicon_tags_31 },
    { 0x359b4a63, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0db4a68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbb4a6f, SUC_lexicon_tags_26 },
    { 0x7b9b4a6f, SUC_lexicon_tags_151 },
    { 0x6afb4a71, SUC_lexicon_tags_27 },
    { 0xc95b4a72, SUC_lexicon_tags_37 },
    { 0xdd3b4a73, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x543b4a76, SUC_lexicon_tags_29 },
    { 0x69b4a77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3b4a7c, SUC_lexicon_tags_2 },
    { 0xfb1b4a7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x251b4a7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x133b4a81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afb4a8a, SUC_lexicon_tags_25 },
    { 0x929b4a8b, SUC_lexicon_tags_70 },
    { 0x635b4a8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf41b4a8e, SUC_lexicon_tags_37 },
    { 0xff7b4a8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bb4a93, SUC_lexicon_tags_45 },
    { 0x323b4a93, SUC_lexicon_tags_11 },
    { 0x93fb4a95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x155b4a97, SUC_lexicon_tags_23 },
    { 0x9adb4a98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589b4a9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabb4aa2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7b4aa5, SUC_lexicon_tags_17 },
    { 0xe1db4aa6, SUC_lexicon_tags_25 },
    { 0x165b4aa7, SUC_lexicon_tags_36 },
    { 0x93b4aa7, SUC_lexicon_tags_17 },
    { 0xc87b4aa9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x377b4aac, SUC_lexicon_tags_33 },
    { 0xb5bb4aac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd07b4aaf, SUC_lexicon_tags_25 },
    { 0xb69b4ab0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93b4ab5, SUC_lexicon_tags_48 },
    { 0x567b4ab6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fb4ab9, SUC_lexicon_tags_11 },
    { 0xcd7b4ab9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x397b4abd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45b4ac2, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x825b4ac5, SUC_lexicon_tags_25 },
    { 0x401b4ac6, SUC_lexicon_tags_96 },
    { 0xf67b4ac6, SUC_lexicon_tags_7 },
    { 0x5bb4ac8, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x753b4aca, SUC_lexicon_tags_10 },
    { 0x697b4aca, SUC_lexicon_tags_17 },
    { 0x66fb4acc, SUC_lexicon_tags_29 },
    { 0xb9db4acb, SUC_lexicon_tags_18 },
    { 0x9b3b4acb, SUC_lexicon_tags_37 },
    { 0xd8db4aca, SUC_lexicon_tags_25 },
    { 0x117b4acb, SUC_lexicon_tags_42 },
    { 0xaffb4ad1, SUC_lexicon_tags_11 },
    { 0xea5b4ad2, SUC_lexicon_tags_34 },
    { 0xa51b4ad3, SUC_lexicon_tags_47 },
    { 0x851b4ad4, SUC_lexicon_tags_25 },
    { 0x403b4acd, SUC_lexicon_tags_21 },
    { 0x5cfb4acd, SUC_lexicon_tags_84 },
    { 0x855b4ad7, SUC_lexicon_tags_37 },
    { 0xef9b4ad7, SUC_lexicon_tags_35 },
    { 0xebb4ad1, SUC_lexicon_tags_11 },
    { 0xa25b4ad4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7c7b4adc, SUC_lexicon_tags_17 },
    { 0x369b4add, SUC_lexicon_tags_32 },
    { 0x2dfb4ade, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69db4ae2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fb4ae6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19b4ae9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239b4aee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb4af3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03b4af6, SUC_lexicon_tags_27 },
    { 0xae7b4af7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x401b4af9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5b4aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791b4b03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbb4b08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139b4b0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x989b4b0e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fb4b12, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107b4b18, SUC_lexicon_tags_2 },
    { 0xdbdb4b19, SUC_lexicon_tags_14 },
    { 0xbb4b1a, SUC_lexicon_tags_66 },
    { 0x3dbb4b18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfb4b1f, SUC_lexicon_tags_45 },
    { 0xee7b4b20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683b4b25, SUC_lexicon_tags_37 },
    { 0xd03b4b25, SUC_lexicon_tags_7 },
    { 0x3e3b4b27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x155b4b2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x527b4b2c, SUC_lexicon_tags_46 },
    { 0x1b7b4b2d, SUC_lexicon_tags_252 },
    { 0xc13b4b2e, SUC_lexicon_tags_48 },
    { 0xcc1b4b2e, SUC_lexicon_tags_66 },
    { 0x6f3b4b30, SUC_lexicon_tags_12 },
    { 0x6e5b4b30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1b4b34, SUC_lexicon_tags_1 },
    { 0xc93b4b35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f1b4b37, SUC_lexicon_tags_3 },
    { 0xca7b4b37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1b4b3c, SUC_lexicon_tags_282 },
    { 0xa37b4b3d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657b4b42, SUC_lexicon_tags_68 },
    { 0x1d5b4b42, SUC_lexicon_tags_43 },
    { 0xf11b4b42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x191b4b46, SUC_lexicon_tags_1 },
    { 0xdc1b4b47, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x887b4b49, SUC_lexicon_tags_25 },
    { 0x7b4b4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7b4b4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66bb4b4f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x127b4b51, SUC_lexicon_tags_2 },
    { 0x9f5b4b52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebb4b55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bb4b5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fb4b64, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fb4b67, SUC_lexicon_tags_7 },
    { 0x3d3b4b68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9db4b6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x837b4b6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fb4b72, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fb4b77, SUC_lexicon_tags_45 },
    { 0xcfbb4b77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae7b4b7a, SUC_lexicon_tags_77 },
    { 0xde9b4b7a, SUC_lexicon_tags_0 },
    { 0x2fbb4b7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bb4b80, SUC_lexicon_tags_83 },
    { 0x369b4b81, SUC_lexicon_tags_1 },
    { 0xf2db4b82, SUC_lexicon_tags_65 },
    { 0x33db4b80, SUC_lexicon_tags_7 },
    { 0x69b4b84, SUC_lexicon_tags_28 },
    { 0xeb1b4b81, SUC_lexicon_tags_26 },
    { 0x5d1b4b86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fb4b89, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdb4b8d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247b4b96, SUC_lexicon_tags_33 },
    { 0xc81b4b97, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xef7b4b99, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fb4ba0, SUC_lexicon_tags_26 },
    { 0xd71b4ba0, SUC_lexicon_tags_14 },
    { 0x5f9b4ba1, SUC_lexicon_tags_37 },
    { 0x2c1b4ba0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51b4ba6, SUC_lexicon_tags_37 },
    { 0x461b4ba7, SUC_lexicon_tags_25 },
    { 0xddb4ba8, SUC_lexicon_tags_27 },
    { 0xf4bb4ba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39b4bad, SUC_lexicon_tags_30 },
    { 0x5bbb4bae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89b4bb8, SUC_lexicon_tags_23 },
    { 0x71fb4bb8, SUC_lexicon_tags_2 },
    { 0x105b4bba, SUC_lexicon_tags_2 },
    { 0x505b4bba, SUC_lexicon_tags_2 },
    { 0xdc3b4bbc, SUC_lexicon_tags_65 },
    { 0x5e3b4bba, SUC_lexicon_tags_43 },
    { 0xc27b4bbe, SUC_lexicon_tags_11 },
    { 0x4b3b4bbf, SUC_lexicon_tags_30 },
    { 0x2e1b4bc0, SUC_lexicon_tags_27 },
    { 0xaffb4bbc, SUC_lexicon_tags_23 },
    { 0x5c1b4bc2, SUC_lexicon_tags_107 },
    { 0xc3b4bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1db4bc6, SUC_lexicon_tags_25 },
    { 0x985b4bc6, SUC_lexicon_tags_15 },
    { 0x38fb4bc8, SUC_lexicon_tags_12 },
    { 0xf21b4bc9, SUC_lexicon_tags_38 },
    { 0xfd1b4bca, SUC_lexicon_tags_60 },
    { 0x7a3b4bcb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7b4bd0, SUC_lexicon_tags_61 },
    { 0x537b4bd0, SUC_lexicon_tags_45 },
    { 0x10fb4bd2, SUC_lexicon_tags_35 },
    { 0xc85b4bd3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173b4bdc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675b4be3, SUC_lexicon_tags_15 },
    { 0x21bb4be4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3b4beb, SUC_lexicon_tags_38 },
    { 0xa9b4bec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbb4bf1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb4bf6, SUC_lexicon_tags_7 },
    { 0xba1b4bf7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9a5b4bf9, SUC_lexicon_tags_47 },
    { 0x933b4bfa, SUC_lexicon_tags_37 },
    { 0x79b4bf9, SUC_lexicon_tags_1 },
    { 0xeb9b4bfc, SUC_lexicon_tags_141 },
    { 0x161b4bfd, SUC_lexicon_tags_56 },
    { 0x71db4bfc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6c3b4c00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4db4c02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5b4c07, SUC_lexicon_tags_15 },
    { 0xf3db4c08, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfb4c0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9c1b4c10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53fb4c12, SUC_lexicon_tags_25 },
    { 0xe55b4c13, SUC_lexicon_tags_21 },
    { 0x8c9b4c13, SUC_lexicon_tags_11 },
    { 0x659b4c15, SUC_lexicon_tags_63 },
    { 0xd7fb4c15, SUC_lexicon_tags_19 },
    { 0xf0fb4c17, SUC_lexicon_tags_7 },
    { 0x47b4c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3b4c22, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x517b4c24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95b4c26, SUC_lexicon_tags_7 },
    { 0x60bb4c26, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x971b4c29, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdb4c2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc89b4c2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x793b4c30, SUC_lexicon_tags_45 },
    { 0x797b4c31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2f7b4c33, SUC_lexicon_tags_17 },
    { 0x98fb4c33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x17bb4c37, SUC_lexicon_tags_37 },
    { 0xe0fb4c38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7b4c3b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x86fb4c3e, SUC_lexicon_tags_25 },
    { 0x28bb4c3f, SUC_lexicon_tags_14 },
    { 0x347b4c40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71b4c43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f3b4c45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97b4c48, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fb4c4f, SUC_lexicon_tags_0 },
    { 0xcd9b4c4f, SUC_lexicon_tags_25 },
    { 0x459b4c51, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa2bb4c53, SUC_lexicon_tags_12 },
    { 0xaefb4c54, SUC_lexicon_tags_37 },
    { 0x9bfb4c53, SUC_lexicon_tags_25 },
    { 0x61fb4c55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x43db4c59, SUC_lexicon_tags_15 },
    { 0xfc5b4c5a, SUC_lexicon_tags_12 },
    { 0xd9fb4c5b, SUC_lexicon_tags_37 },
    { 0xd4db4c5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1b4c5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xac3b4c61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x629b4c63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45b4c67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1b4c6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1b4c72, SUC_lexicon_tags_49 },
    { 0xc55b4c72, SUC_lexicon_tags_1 },
    { 0xc87b4c72, SUC_lexicon_tags_33 },
    { 0xc2db4c75, SUC_lexicon_tags_12 },
    { 0x56db4c73, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdcbb4c78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e5b4c7a, SUC_lexicon_tags_38 },
    { 0xa4bb4c7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bb4c7f, SUC_lexicon_tags_11 },
    { 0xc2fb4c80, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7b4c86, SUC_lexicon_tags_25 },
    { 0x1c7b4c87, SUC_lexicon_tags_37 },
    { 0xc4fb4c88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13b4c8c, SUC_lexicon_tags_0 },
    { 0x9bdb4c8d, SUC_lexicon_tags_37 },
    { 0x187b4c8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x631b4c90, SUC_lexicon_tags_49 },
    { 0x52db4c90, SUC_lexicon_tags_60 },
    { 0x95bb4c92, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdb4c96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1b4c99, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65b4ca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7b4ca4, SUC_lexicon_tags_17 },
    { 0xa5db4ca4, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x991b4ca7, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151b4cac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88db4cb1, SUC_lexicon_tags_17 },
    { 0x8b3b4cb2, SUC_lexicon_tags_11 },
    { 0x8efb4cb3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30db4cb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4db4cbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617b4cc0, SUC_lexicon_tags_33 },
    { 0x43db4cc1, SUC_lexicon_tags_82 },
    { 0x4c7b4cc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc2bb4cc4, SUC_lexicon_tags_6 },
    { 0x533b4cc5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635b4cc9, SUC_lexicon_tags_15 },
    { 0x2d1b4cca, SUC_lexicon_tags_63 },
    { 0x9f9b4ccb, SUC_lexicon_tags_55 },
    { 0x8abb4ccc, SUC_lexicon_tags_26 },
    { 0x863b4ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc5b4ccf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x543b4cd2, SUC_lexicon_tags_1 },
    { 0xa95b4cd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bb4cd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23b4cdb, SUC_lexicon_tags_40 },
    { 0x449b4cdb, SUC_lexicon_tags_7 },
    { 0x7adb4cdd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x47bb4cdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41db4ce5, SUC_lexicon_tags_2 },
    { 0xedfb4ce6, SUC_lexicon_tags_11 },
    { 0x2f5b4ce7, SUC_lexicon_tags_33 },
    { 0xfd5b4ce5, SUC_lexicon_tags_9 },
    { 0xc89b4ce9, SUC_lexicon_tags_33 },
    { 0xc1db4ce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167b4cef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8a1b4cf1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5b4cf8, SUC_lexicon_tags_87 },
    { 0xa09b4cf9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xecdb4cfc, SUC_lexicon_tags_38 },
    { 0xd87b4cfc, SUC_lexicon_tags_28 },
    { 0x96db4cfe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x435b4d00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fb4d04, SUC_lexicon_tags_11 },
    { 0x851b4d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe45b4d07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1efb4d09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5b4d0c, SUC_lexicon_tags_3 },
    { 0xe81b4d0d, SUC_lexicon_tags_12 },
    { 0xb8fb4d0e, SUC_lexicon_tags_15 },
    { 0xce5b4d0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbedb4d11, SUC_lexicon_tags_23 },
    { 0xad7b4d11, SUC_lexicon_tags_57 },
    { 0x537b4d13, SUC_lexicon_tags_23 },
    { 0x1a9b4d11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x115b4d17, SUC_lexicon_tags_1 },
    { 0x683b4d18, SUC_lexicon_tags_19 },
    { 0xb83b4d17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7db4d1c, SUC_lexicon_tags_15 },
    { 0xfb4d1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x591b4d1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa51b4d22, SUC_lexicon_tags_79 },
    { 0xea7b4d23, SUC_lexicon_tags_15 },
    { 0x6fbb4d24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f1b4d26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5b4d29, SUC_lexicon_tags_47 },
    { 0x559b4d2a, SUC_lexicon_tags_1 },
    { 0xfddb4d2a, SUC_lexicon_tags_3 },
    { 0xd71b4d2c, SUC_lexicon_tags_0 },
    { 0x2cdb4d2a, SUC_lexicon_tags_97 },
    { 0x279b4d2e, SUC_lexicon_tags_12 },
    { 0x141b4d2f, SUC_lexicon_tags_25 },
    { 0x5d1b4d2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbb4d34, SUC_lexicon_tags_11 },
    { 0x243b4d34, SUC_lexicon_tags_2 },
    { 0xc7db4d36, SUC_lexicon_tags_0 },
    { 0xee9b4d36, SUC_lexicon_tags_60 },
    { 0x55db4d35, SUC_lexicon_tags_16 },
    { 0x1d3b4d37, SUC_lexicon_tags_11 },
    { 0x2a1b4d34, SUC_lexicon_tags_19 },
    { 0x159b4d3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73b4d41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x567b4d43, SUC_lexicon_tags_38 },
    { 0x9e7b4d44, SUC_lexicon_tags_15 },
    { 0xcdb4d43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19db4d47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5db4d4d, SUC_lexicon_tags_11 },
    { 0xf09b4d4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc27b4d50, SUC_lexicon_tags_1 },
    { 0xd43b4d51, SUC_lexicon_tags_11 },
    { 0xc8bb4d51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f3b4d54, SUC_lexicon_tags_63 },
    { 0xfb1b4d54, SUC_lexicon_tags_12 },
    { 0x61fb4d56, SUC_lexicon_tags_8 },
    { 0xe9b4d57, SUC_lexicon_tags_7 },
    { 0x30db4d58, SUC_lexicon_tags_17 },
    { 0xdc5b4d57, SUC_lexicon_tags_19 },
    { 0x3f9b4d57, SUC_lexicon_tags_7 },
    { 0xd4bb4d5b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fb4d5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9b4d63, SUC_lexicon_tags_14 },
    { 0xe67b4d63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x481b4d67, SUC_lexicon_tags_43 },
    { 0xf23b4d68, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33b4d6c, SUC_lexicon_tags_33 },
    { 0x4a1b4d6d, SUC_lexicon_tags_33 },
    { 0xb4fb4d6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbb4d76, SUC_lexicon_tags_1 },
    { 0xaedb4d77, SUC_lexicon_tags_17 },
    { 0x8f1b4d78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35b4d7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9b4d81, SUC_lexicon_tags_25 },
    { 0x2a7b4d82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731b4d8c, SUC_lexicon_tags_29 },
    { 0xbb3b4d8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1fb4d8f, SUC_lexicon_tags_43 },
    { 0xd07b4d8f, SUC_lexicon_tags_25 },
    { 0x2d5b4d91, SUC_lexicon_tags_8 },
    { 0xcd9b4d92, SUC_lexicon_tags_43 },
    { 0xcabb4d93, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47b4d9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95db4d9f, SUC_lexicon_tags_49 },
    { 0x831b4da0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37db4da4, SUC_lexicon_tags_27 },
    { 0x725b4da5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdb4da9, SUC_lexicon_tags_25 },
    { 0x555b4da9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bb4dad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbebb4daf, SUC_lexicon_tags_28 },
    { 0xc29b4daf, SUC_lexicon_tags_17 },
    { 0x329b4db1, SUC_lexicon_tags_1 },
    { 0x4c9b4db1, SUC_lexicon_tags_15 },
    { 0x6bbb4db2, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xf99b4db6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13bb4db9, SUC_lexicon_tags_39 },
    { 0xc79b4db9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb4dbe, SUC_lexicon_tags_2 },
    { 0x90db4dbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7b4dc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3b4dcd, SUC_lexicon_tags_17 },
    { 0x221b4dce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07b4dd1, SUC_lexicon_tags_38 },
    { 0xa1db4dd2, SUC_lexicon_tags_1 },
    { 0xa81b4dd3, SUC_lexicon_tags_25 },
    { 0xa05b4dd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdb4dd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x633b4dda, SUC_lexicon_tags_201 },
    { 0xc55b4dda, SUC_lexicon_tags_7 },
    { 0xcdbb4ddc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xabfb4ddf, SUC_lexicon_tags_43 },
    { 0xb73b4de0, SUC_lexicon_tags_37 },
    { 0x5a3b4de1, SUC_lexicon_tags_10 },
    { 0x6a1b4de0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fb4deb, SUC_lexicon_tags_11 },
    { 0x4a1b4dec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bb4df8, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x33b4dfb, SUC_lexicon_tags_33 },
    { 0x37bb4dfb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x821b4dff, SUC_lexicon_tags_3 },
    { 0x439b4e00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84db4e06, SUC_lexicon_tags_47 },
    { 0x101b4e07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b4e0d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581b4e11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31b4e14, SUC_lexicon_tags_15 },
    { 0x12db4e15, SUC_lexicon_tags_31 },
    { 0x9ebb4e16, SUC_lexicon_tags_66 },
    { 0x427b4e15, SUC_lexicon_tags_27 },
    { 0xf1fb4e18, SUC_lexicon_tags_27 },
    { 0x931b4e19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x841b4e1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bb4e22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05b4e25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x341b4e28, SUC_lexicon_tags_0 },
    { 0x1abb4e28, SUC_lexicon_tags_15 },
    { 0xcb9b4e2a, SUC_lexicon_tags_12 },
    { 0xcf7b4e2b, SUC_lexicon_tags_3 },
    { 0xed7b4e2b, SUC_lexicon_tags_37 },
    { 0x419b4e29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf39b4e2f, SUC_lexicon_tags_17 },
    { 0xc6bb4e30, SUC_lexicon_tags_23 },
    { 0x945b4e31, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8bfb4e33, SUC_lexicon_tags_38 },
    { 0xc07b4e34, SUC_lexicon_tags_65 },
    { 0xf15b4e35, SUC_lexicon_tags_11 },
    { 0xfbb4e33, SUC_lexicon_tags_1 },
    { 0xd21b4e37, SUC_lexicon_tags_8 },
    { 0x39b4e38, SUC_lexicon_tags_136 },
    { 0x421b4e35, SUC_lexicon_tags_2 },
    { 0x765b4e3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdb4e3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9b4e40, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19b4e43, SUC_lexicon_tags_52 },
    { 0x729b4e43, SUC_lexicon_tags_15 },
    { 0xce5b4e45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81fb4e48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6db4e4c, SUC_lexicon_tags_12 },
    { 0xf55b4e4d, SUC_lexicon_tags_1119 },
    { 0x2bb4e4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cbb4e50, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x285b4e53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3b4e5a, SUC_lexicon_tags_25 },
    { 0x439b4e5b, SUC_lexicon_tags_23 },
    { 0x7b3b4e5b, SUC_lexicon_tags_10 },
    { 0x8a7b4e5d, SUC_lexicon_tags_98 },
    { 0x829b4e5c, SUC_lexicon_tags_4 },
    { 0x217b4e5f, SUC_lexicon_tags_1 },
    { 0x9b5b4e5f, SUC_lexicon_tags_17 },
    { 0xc55b4e5b, SUC_lexicon_tags_39 },
    { 0xd95b4e62, SUC_lexicon_tags_25 },
    { 0x98bb4e63, SUC_lexicon_tags_9 },
    { 0x4f1b4e64, SUC_lexicon_tags_56 },
    { 0xee3b4e65, SUC_lexicon_tags_17 },
    { 0xeefb4e64, SUC_lexicon_tags_12 },
    { 0xd49b4e5a, SUC_lexicon_tags_25 },
    { 0xcbfb4e68, SUC_lexicon_tags_2 },
    { 0xeffb4e69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b4e6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b9b4e70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71b4e74, SUC_lexicon_tags_61 },
    { 0x1a3b4e75, SUC_lexicon_tags_38 },
    { 0x215b4e75, SUC_lexicon_tags_12 },
    { 0x68bb4e75, SUC_lexicon_tags_3 },
    { 0x975b4e75, SUC_lexicon_tags_37 },
    { 0x9cfb4e79, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631b4e81, SUC_lexicon_tags_38 },
    { 0x395b4e82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3b4e85, SUC_lexicon_tags_0 },
    { 0x63bb4e86, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdb4e89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bb4e8e, SUC_lexicon_tags_26 },
    { 0x649b4e8f, SUC_lexicon_tags_74 },
    { 0xac9b4e8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdb4e93, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fb4e96, SUC_lexicon_tags_55 },
    { 0x713b4e97, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43b4e9d, SUC_lexicon_tags_17 },
    { 0xc85b4e9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65b4ea2, SUC_lexicon_tags_1 },
    { 0xf19b4ea2, SUC_lexicon_tags_39 },
    { 0xf79b4ea4, SUC_lexicon_tags_70 },
    { 0xec7b4ea5, SUC_lexicon_tags_17 },
    { 0xa5fb4ea6, SUC_lexicon_tags_23 },
    { 0x265b4ea7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb01b4ea9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81b4eae, SUC_lexicon_tags_5 },
    { 0x267b4eaf, SUC_lexicon_tags_37 },
    { 0x3f5b4eb0, SUC_lexicon_tags_4 },
    { 0xe73b4eb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x769b4eb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33b4ebd, SUC_lexicon_tags_216 },
    { 0x79db4ebd, SUC_lexicon_tags_2 },
    { 0xdafb4ebd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94db4ecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fb4ecf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x799b4ed1, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abb4ed6, SUC_lexicon_tags_36 },
    { 0x657b4ed6, SUC_lexicon_tags_656 },
    { 0x421b4ed8, SUC_lexicon_tags_63 },
    { 0x715b4ed8, SUC_lexicon_tags_37 },
    { 0xd1fb4ed9, SUC_lexicon_tags_34 },
    { 0xa7bb4eda, SUC_lexicon_tags_12 },
    { 0xd63b4ed6, SUC_lexicon_tags_75 },
    { 0x53db4eda, SUC_lexicon_tags_18 },
    { 0xe25b4ed9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fb4ee2, SUC_lexicon_tags_19 },
    { 0xa9db4ee3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29b4ee7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7b4eee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3db4ef0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdb4ef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddb4ef9, SUC_lexicon_tags_63 },
    { 0xa09b4ef9, SUC_lexicon_tags_478 },
    { 0x27db4efb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2b1b4efd, SUC_lexicon_tags_30 },
    { 0xd6bb4efe, SUC_lexicon_tags_31 },
    { 0xc95b4eff, SUC_lexicon_tags_9 },
    { 0xa2fb4efe, SUC_lexicon_tags_58 },
    { 0xaf3b4efd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x78bb4f03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43b4f07, SUC_lexicon_tags_28 },
    { 0x829b4f07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5b4f0b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x16fb4f0d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x20fb4f0f, SUC_lexicon_tags_121 },
    { 0x43b4f0f, SUC_lexicon_tags_7 },
    { 0x581b4f11, SUC_lexicon_tags_12 },
    { 0x6fb4f12, SUC_lexicon_tags_12 },
    { 0x93bb4f13, SUC_lexicon_tags_56 },
    { 0xdb7b4f14, SUC_lexicon_tags_11 },
    { 0x829b4f15, SUC_lexicon_tags_60 },
    { 0x2fbb4f16, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbb4f1a, SUC_lexicon_tags_19 },
    { 0x4f9b4f1b, SUC_lexicon_tags_14 },
    { 0x487b4f1a, SUC_lexicon_tags_25 },
    { 0xe71b4f1d, SUC_lexicon_tags_32 },
    { 0x699b4f1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb4bb4f20, SUC_lexicon_tags_17 },
    { 0xca5b4f21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17b4f25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbb4f29, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebb4f2f, SUC_lexicon_tags_37 },
    { 0xcffb4f30, SUC_lexicon_tags_17 },
    { 0x8f7b4f30, SUC_lexicon_tags_17 },
    { 0x7dfb4f32, SUC_lexicon_tags_14 },
    { 0xa9bb4f33, SUC_lexicon_tags_11 },
    { 0x1c9b4f33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x753b4f37, SUC_lexicon_tags_37 },
    { 0x637b4f38, SUC_lexicon_tags_25 },
    { 0xe9fb4f39, SUC_lexicon_tags_2 },
    { 0x3c9b4f38, SUC_lexicon_tags_2 },
    { 0xaa3b4f3b, SUC_lexicon_tags_28 },
    { 0x395b4f3c, SUC_lexicon_tags_25 },
    { 0x1fbb4f37, SUC_lexicon_tags_65 },
    { 0x719b4f39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abb4f41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x78db4f43, SUC_lexicon_tags_11 },
    { 0xf1b4f44, SUC_lexicon_tags_45 },
    { 0x66bb4f44, SUC_lexicon_tags_11 },
    { 0x467b4f44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbb4f4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fb4f51, SUC_lexicon_tags_17 },
    { 0x87bb4f52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db4f56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa5bb4f58, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95b4f5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3b4f62, SUC_lexicon_tags_2 },
    { 0x8efb4f62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x561b4f66, SUC_lexicon_tags_7 },
    { 0xc11b4f67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x241b4f6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305b4f6e, SUC_lexicon_tags_19 },
    { 0x9abb4f6e, SUC_lexicon_tags_15 },
    { 0x3e9b4f6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb31b4f72, SUC_lexicon_tags_1 },
    { 0x39db4f72, SUC_lexicon_tags_10 },
    { 0x7c1b4f73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5b4f78, SUC_lexicon_tags_37 },
    { 0xfe3b4f79, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fb4f7c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x807b4f7e, SUC_lexicon_tags_12 },
    { 0x4b7b4f7e, SUC_lexicon_tags_12 },
    { 0x21fb4f80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbb4f86, SUC_lexicon_tags_12 },
    { 0x293b4f86, SUC_lexicon_tags_16 },
    { 0x4b9b4f88, SUC_lexicon_tags_56 },
    { 0xed3b4f88, SUC_lexicon_tags_15 },
    { 0xc8fb4f88, SUC_lexicon_tags_138 },
    { 0x79db4f8b, SUC_lexicon_tags_25 },
    { 0x8b3b4f8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bb4f8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5b4f97, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb6db4f99, SUC_lexicon_tags_3 },
    { 0xcdb4f9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cfb4f9c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x803b4f9e, SUC_lexicon_tags_38 },
    { 0xeb1b4f9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbb4fa7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7b4fab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50db4fad, SUC_lexicon_tags_31 },
    { 0xb89b4fae, SUC_lexicon_tags_82 },
    { 0x6f9b4faf, SUC_lexicon_tags_1 },
    { 0xb3fb4fae, SUC_lexicon_tags_28 },
    { 0x54fb4fb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbb4fba, SUC_lexicon_tags_47 },
    { 0xe6db4fbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7b4fbf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bb4fc3, SUC_lexicon_tags_132 },
    { 0x3ddb4fc4, SUC_lexicon_tags_37 },
    { 0x71bb4fc3, SUC_lexicon_tags_103 },
    { 0x7b7b4fc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25b4fcb, SUC_lexicon_tags_65 },
    { 0xaf7b4fcc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x451b4fcf, SUC_lexicon_tags_47 },
    { 0xb3b4fd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1db4fd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91db4fd6, SUC_lexicon_tags_15 },
    { 0xbfdb4fd7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9b4fda, SUC_lexicon_tags_6 },
    { 0xa9fb4fda, SUC_lexicon_tags_17 },
    { 0x963b4fdc, SUC_lexicon_tags_21 },
    { 0x34bb4fdc, SUC_lexicon_tags_79 },
    { 0x11fb4fde, SUC_lexicon_tags_12 },
    { 0xd3db4fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe95b4fe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33db4fe4, SUC_lexicon_tags_7 },
    { 0x6adb4fe4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9efb4fe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfb4fec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5b4ff0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1edb4ff2, SUC_lexicon_tags_6 },
    { 0x3cfb4ff3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a3b4ff5, SUC_lexicon_tags_7 },
    { 0x85b4ff5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fb4ffa, SUC_lexicon_tags_17 },
    { 0x97fb4ffa, SUC_lexicon_tags_33 },
    { 0x20fb4ffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2bb4ffe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x467b5001, SUC_lexicon_tags_26 },
    { 0xce3b5002, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97bb5004, SUC_lexicon_tags_23 },
    { 0xf5bb5005, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1b5008, SUC_lexicon_tags_17 },
    { 0x28bb5008, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3fb500b, SUC_lexicon_tags_37 },
    { 0x66bb500b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3b500f, SUC_lexicon_tags_49 },
    { 0x873b500f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9b5015, SUC_lexicon_tags_83 },
    { 0x853b5016, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5b5019, SUC_lexicon_tags_8 },
    { 0x3a1b501a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efb501e, SUC_lexicon_tags_16 },
    { 0xcd7b501f, SUC_lexicon_tags_1 },
    { 0x2e5b501f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5fb5022, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1b5025, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607b502a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7b502d, SUC_lexicon_tags_1 },
    { 0x369b502d, SUC_lexicon_tags_14 },
    { 0xfefb502f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ebb5031, SUC_lexicon_tags_47 },
    { 0x6fbb5031, SUC_lexicon_tags_4 },
    { 0x205b5031, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bb5039, SUC_lexicon_tags_57 },
    { 0xf33b5039, SUC_lexicon_tags_4 },
    { 0x4bbb503b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2fb503d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd5fb503f, SUC_lexicon_tags_68 },
    { 0x569b5040, SUC_lexicon_tags_17 },
    { 0x497b5041, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f5b5043, SUC_lexicon_tags_37 },
    { 0x8a9b5044, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf61b5046, SUC_lexicon_tags_19 },
    { 0xebb5046, SUC_lexicon_tags_82 },
    { 0x12fb5048, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbb504b, SUC_lexicon_tags_25 },
    { 0x899b504c, SUC_lexicon_tags_7 },
    { 0xdf7b504d, SUC_lexicon_tags_1441 },
    { 0, NULL },
    { 0xbedb504f, SUC_lexicon_tags_34 },
    { 0x5fbb5050, SUC_lexicon_tags_33 },
    { 0xbf3b5050, SUC_lexicon_tags_6 },
    { 0xe8fb5052, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1b5056, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1f5b5058, SUC_lexicon_tags_25 },
    { 0xf53b5059, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x145b505c, SUC_lexicon_tags_6 },
    { 0x8ddb505d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3db5060, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x873b5062, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125b5067, SUC_lexicon_tags_45 },
    { 0x8e1b5068, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb506b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777b506f, SUC_lexicon_tags_38 },
    { 0xcedb506f, SUC_lexicon_tags_16 },
    { 0x7d7b5070, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3b5074, SUC_lexicon_tags_28 },
    { 0x40bb5075, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973b5079, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953b507e, SUC_lexicon_tags_114 },
    { 0x327b507f, SUC_lexicon_tags_7 },
    { 0x2a7b5080, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465b5087, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859b508c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05b508f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697b5097, SUC_lexicon_tags_7 },
    { 0xfb1b5098, SUC_lexicon_tags_11 },
    { 0x30fb5099, SUC_lexicon_tags_15 },
    { 0x5ddb509a, SUC_lexicon_tags_15 },
    { 0xbbdb5098, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb7b509d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a3b509f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52db50a5, SUC_lexicon_tags_11 },
    { 0x251b50a6, SUC_lexicon_tags_11 },
    { 0xa65b50a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105b50ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491b50b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fb50b5, SUC_lexicon_tags_37 },
    { 0x265b50b6, SUC_lexicon_tags_17 },
    { 0xcabb50b7, SUC_lexicon_tags_60 },
    { 0x345b50b8, SUC_lexicon_tags_17 },
    { 0x2d5b50b9, SUC_lexicon_tags_21 },
    { 0xd77b50ba, SUC_lexicon_tags_2 },
    { 0x3e7b50ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fb50be, SUC_lexicon_tags_17 },
    { 0x4bfb50bf, SUC_lexicon_tags_11 },
    { 0xb4db50bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5b50c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61b50c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d7b50c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a9b50cb, SUC_lexicon_tags_4 },
    { 0x483b50cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbffb50ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37b50d7, SUC_lexicon_tags_1 },
    { 0xde7b50d8, SUC_lexicon_tags_11 },
    { 0x775b50d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09b50dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9b50e3, SUC_lexicon_tags_14 },
    { 0x3c3b50e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb50e8, SUC_lexicon_tags_10 },
    { 0x6bdb50e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21db50ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1b50f9, SUC_lexicon_tags_0 },
    { 0x4d1b50fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1b50fd, SUC_lexicon_tags_17 },
    { 0x97b50fe, SUC_lexicon_tags_60 },
    { 0xc87b50ff, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xc03b5101, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa7b5103, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddb510c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97b510f, SUC_lexicon_tags_49 },
    { 0xb6fb5110, SUC_lexicon_tags_49 },
    { 0xb35b510f, SUC_lexicon_tags_25 },
    { 0x591b5112, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fb5116, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305b511a, SUC_lexicon_tags_17 },
    { 0x29bb511b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983b5123, SUC_lexicon_tags_19 },
    { 0xa17b5123, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xacbb5126, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x53fb5128, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bb5131, SUC_lexicon_tags_15 },
    { 0xa57b5132, SUC_lexicon_tags_127 },
    { 0x95fb5132, SUC_lexicon_tags_15 },
    { 0x67bb5133, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fb5137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3db513e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97b5142, SUC_lexicon_tags_39 },
    { 0xff9b5143, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf25b5145, SUC_lexicon_tags_14 },
    { 0x367b5146, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9b514b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5b5152, SUC_lexicon_tags_38 },
    { 0x9a3b5153, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x717b5156, SUC_lexicon_tags_64 },
    { 0x4edb5157, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61b515a, SUC_lexicon_tags_61 },
    { 0x63fb515a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd65b515d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225b5161, SUC_lexicon_tags_79 },
    { 0x493b5162, SUC_lexicon_tags_46 },
    { 0xe13b5162, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbb516c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb516f, SUC_lexicon_tags_11 },
    { 0xd9db516f, SUC_lexicon_tags_7 },
    { 0x361b5171, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x30bb5174, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93b517a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e9b517c, SUC_lexicon_tags_17 },
    { 0x2bdb517c, SUC_lexicon_tags_1 },
    { 0xa13b517e, SUC_lexicon_tags_48 },
    { 0xc3db517c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x775b5182, SUC_lexicon_tags_2 },
    { 0x937b5182, SUC_lexicon_tags_3 },
    { 0xba1b5183, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedb5187, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x30db5189, SUC_lexicon_tags_17 },
    { 0xf3bb518a, SUC_lexicon_tags_2 },
    { 0x8d3b518b, SUC_lexicon_tags_12 },
    { 0x7e5b518b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223b5190, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bb5196, SUC_lexicon_tags_49 },
    { 0xd11b5197, SUC_lexicon_tags_15 },
    { 0x129b5198, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5b519c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5b51a3, SUC_lexicon_tags_3 },
    { 0xbe1b51a4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667b51b2, SUC_lexicon_tags_28 },
    { 0x6cfb51b3, SUC_lexicon_tags_6 },
    { 0x161b51b4, SUC_lexicon_tags_68 },
    { 0xad1b51b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7b51b9, SUC_lexicon_tags_12 },
    { 0xfb51ba, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5b51bf, SUC_lexicon_tags_21 },
    { 0x9d1b51bf, SUC_lexicon_tags_1 },
    { 0xff7b51c1, SUC_lexicon_tags_37 },
    { 0x655b51c2, SUC_lexicon_tags_33 },
    { 0xc11b51c0, SUC_lexicon_tags_19 },
    { 0x44fb51c4, SUC_lexicon_tags_11 },
    { 0xd3bb51c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc61b51c7, SUC_lexicon_tags_15 },
    { 0xda1b51c7, SUC_lexicon_tags_64 },
    { 0x5e3b51c7, SUC_lexicon_tags_66 },
    { 0x16bb51c8, SUC_lexicon_tags_45 },
    { 0x6f1b51c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7b51ce, SUC_lexicon_tags_82 },
    { 0xe89b51cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd85b51d1, SUC_lexicon_tags_85 },
    { 0xc1fb51d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xddbb51d4, SUC_lexicon_tags_1 },
    { 0x235b51d5, SUC_lexicon_tags_47 },
    { 0x4adb51d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf6db51d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x939b51db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbfb51dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdb51e1, SUC_lexicon_tags_15 },
    { 0x48fb51e2, SUC_lexicon_tags_17 },
    { 0x543b51e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x95bb51e6, SUC_lexicon_tags_116 },
    { 0x7ddb51e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5b51ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa51b51ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fb51f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5d5b51f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x90db51f5, SUC_lexicon_tags_47 },
    { 0x5b51f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edb51f9, SUC_lexicon_tags_23 },
    { 0xc13b51fa, SUC_lexicon_tags_37 },
    { 0xb5bb51fb, SUC_lexicon_tags_11 },
    { 0x679b51fc, SUC_lexicon_tags_25 },
    { 0x22bb51fd, SUC_lexicon_tags_25 },
    { 0xc81b51f9, SUC_lexicon_tags_17 },
    { 0xac5b51ff, SUC_lexicon_tags_29 },
    { 0x5cfb51fb, SUC_lexicon_tags_25 },
    { 0xf6fb5201, SUC_lexicon_tags_6 },
    { 0x5fbb5202, SUC_lexicon_tags_33 },
    { 0xb45b5203, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8bb5205, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x381b5207, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39b520a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x27bb520c, SUC_lexicon_tags_15 },
    { 0x2cfb520d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x947b520f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9b5212, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599b5217, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351b521b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x189b521d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd17b521f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7b5222, SUC_lexicon_tags_17 },
    { 0xf3bb5223, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4db5225, SUC_lexicon_tags_11 },
    { 0x8cfb5225, SUC_lexicon_tags_25 },
    { 0x961b5226, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9db522c, SUC_lexicon_tags_32 },
    { 0x7ebb522c, SUC_lexicon_tags_47 },
    { 0xfc3b522e, SUC_lexicon_tags_37 },
    { 0xbb3b522f, SUC_lexicon_tags_28 },
    { 0x61bb5230, SUC_lexicon_tags_26 },
    { 0x2f1b5230, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5b5235, SUC_lexicon_tags_1 },
    { 0x267b5235, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x9f9b5238, SUC_lexicon_tags_87 },
    { 0x8bbb5239, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x667b523c, SUC_lexicon_tags_11 },
    { 0xb59b523c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9b5240, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x703b5242, SUC_lexicon_tags_31 },
    { 0xe61b5242, SUC_lexicon_tags_82 },
    { 0x4a1b5244, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499b524a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbb524d, SUC_lexicon_tags_26 },
    { 0x29fb524e, SUC_lexicon_tags_31 },
    { 0x9cdb524e, SUC_lexicon_tags_15 },
    { 0x379b5250, SUC_lexicon_tags_38 },
    { 0xf21b5250, SUC_lexicon_tags_29 },
    { 0xe9bb5251, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa65b5254, SUC_lexicon_tags_27 },
    { 0x1d5b5254, SUC_lexicon_tags_1 },
    { 0x555b5255, SUC_lexicon_tags_17 },
    { 0xbfb5256, SUC_lexicon_tags_37 },
    { 0x647b5258, SUC_lexicon_tags_61 },
    { 0xd13b5254, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b525c, SUC_lexicon_tags_2 },
    { 0x4afb525c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x521b525f, SUC_lexicon_tags_11 },
    { 0x25b525f, SUC_lexicon_tags_63 },
    { 0x417b5261, SUC_lexicon_tags_10 },
    { 0x98fb5262, SUC_lexicon_tags_1 },
    { 0xa09b5261, SUC_lexicon_tags_65 },
    { 0x7afb5262, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bb526a, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0xa87b526d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61b5274, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213b5281, SUC_lexicon_tags_1 },
    { 0xb0bb5282, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb43b5284, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05b5287, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcfdb5289, SUC_lexicon_tags_57 },
    { 0xb8db528a, SUC_lexicon_tags_38 },
    { 0x253b528b, SUC_lexicon_tags_23 },
    { 0x44bb528c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5b528f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffb5292, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bb5296, SUC_lexicon_tags_37 },
    { 0x5c7b5297, SUC_lexicon_tags_17 },
    { 0xacfb5298, SUC_lexicon_tags_33 },
    { 0xea7b5297, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x139b529b, SUC_lexicon_tags_25 },
    { 0x7c9b529b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a7b529e, SUC_lexicon_tags_19 },
    { 0xbc7b529e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadb52a9, SUC_lexicon_tags_27 },
    { 0xa8bb52a9, SUC_lexicon_tags_7 },
    { 0x2a9b52ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1db52b2, SUC_lexicon_tags_43 },
    { 0x2fb52b3, SUC_lexicon_tags_11 },
    { 0x43bb52b4, SUC_lexicon_tags_12 },
    { 0xeb3b52b3, SUC_lexicon_tags_29 },
    { 0x621b52b4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fb52b9, SUC_lexicon_tags_154 },
    { 0x947b52ba, SUC_lexicon_tags_12 },
    { 0x83bb52bb, SUC_lexicon_tags_11 },
    { 0xac7b52bb, SUC_lexicon_tags_85 },
    { 0xbe7b52bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbb52c9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdbdb52cb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9b52d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfb52d5, SUC_lexicon_tags_7 },
    { 0x39b52d6, SUC_lexicon_tags_6 },
    { 0x47bb52d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adb52db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x80fb52de, SUC_lexicon_tags_38 },
    { 0x4c9b52de, SUC_lexicon_tags_27 },
    { 0x39fb52e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b52e4, SUC_lexicon_tags_12 },
    { 0x705b52e5, SUC_lexicon_tags_25 },
    { 0x3e1b52e6, SUC_lexicon_tags_1 },
    { 0xb19b52e7, SUC_lexicon_tags_15 },
    { 0x51bb52e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x75b52eb, SUC_lexicon_tags_1 },
    { 0x1e5b52eb, SUC_lexicon_tags_77 },
    { 0xb0bb52ed, SUC_lexicon_tags_115 },
    { 0xf31b52eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbb52f5, SUC_lexicon_tags_7 },
    { 0x423b52f6, SUC_lexicon_tags_34 },
    { 0x89fb52f6, SUC_lexicon_tags_2 },
    { 0xab7b52f6, SUC_lexicon_tags_106 },
    { 0x4cdb52f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x611b52fb, SUC_lexicon_tags_21 },
    { 0x7f7b52fc, SUC_lexicon_tags_17 },
    { 0x3ebb52fd, SUC_lexicon_tags_15 },
    { 0x521b52fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82db5303, SUC_lexicon_tags_35 },
    { 0x9ddb5304, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x139b5306, SUC_lexicon_tags_37 },
    { 0xf7b5306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115b530b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbb530f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x111b5312, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xb45b5315, SUC_lexicon_tags_72 },
    { 0x8d1b5315, SUC_lexicon_tags_37 },
    { 0x1a7b5317, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1b531b, SUC_lexicon_tags_15 },
    { 0x1e1b531c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77b531f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdb5322, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1b5327, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2f7b5329, SUC_lexicon_tags_15 },
    { 0x43b5329, SUC_lexicon_tags_33 },
    { 0xce5b532b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b532d, SUC_lexicon_tags_45 },
    { 0x27b532e, SUC_lexicon_tags_9 },
    { 0xd1fb532f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2a1b5331, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1b5334, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269b5339, SUC_lexicon_tags_25 },
    { 0x25b533a, SUC_lexicon_tags_27 },
    { 0xeafb533b, SUC_lexicon_tags_15 },
    { 0xa49b5339, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1db533f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221b5343, SUC_lexicon_tags_2 },
    { 0x35fb5344, SUC_lexicon_tags_14 },
    { 0xaa7b5344, SUC_lexicon_tags_7 },
    { 0x7b5b5346, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbfb5348, SUC_lexicon_tags_7 },
    { 0xac9b5349, SUC_lexicon_tags_17 },
    { 0xe27b5349, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7db5350, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x701b5353, SUC_lexicon_tags_33 },
    { 0x7a9b5353, SUC_lexicon_tags_0 },
    { 0x75db5354, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73b535c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83b535f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05b5364, SUC_lexicon_tags_14 },
    { 0x5a1b5364, SUC_lexicon_tags_26 },
    { 0x8f3b5366, SUC_lexicon_tags_25 },
    { 0xcbdb5366, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839b536b, SUC_lexicon_tags_19 },
    { 0xfdb536c, SUC_lexicon_tags_47 },
    { 0xa85b536d, SUC_lexicon_tags_84 },
    { 0x89db536c, SUC_lexicon_tags_7 },
    { 0xa6fb536f, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fb5378, SUC_lexicon_tags_26 },
    { 0xdebb5378, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23db537b, SUC_lexicon_tags_37 },
    { 0x45fb537c, SUC_lexicon_tags_29 },
    { 0x2b9b537d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bb5381, SUC_lexicon_tags_224 },
    { 0x87b5382, SUC_lexicon_tags_1 },
    { 0xb7b5382, SUC_lexicon_tags_25 },
    { 0x6b3b5384, SUC_lexicon_tags_11 },
    { 0xdb9b5385, SUC_lexicon_tags_1 },
    { 0x439b5386, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6f9b5388, SUC_lexicon_tags_17 },
    { 0xc8db5389, SUC_lexicon_tags_65 },
    { 0x7bfb538a, SUC_lexicon_tags_45 },
    { 0x17db5389, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac1b538d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4d7b538f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb5395, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39b539d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245b53a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x743b53a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x13b53a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3b53ad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeebb53af, SUC_lexicon_tags_27 },
    { 0xa5db53b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x84db53b3, SUC_lexicon_tags_43 },
    { 0xcf3b53b4, SUC_lexicon_tags_43 },
    { 0xedfb53b5, SUC_lexicon_tags_42 },
    { 0xf07b53b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x84bb53b8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa61b53ba, SUC_lexicon_tags_7 },
    { 0xa83b53bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x385b53be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81db53c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35b53c5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x201b53c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3b53cb, SUC_lexicon_tags_8 },
    { 0x997b53cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x575b53ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddb53d2, SUC_lexicon_tags_82 },
    { 0xf19b53d3, SUC_lexicon_tags_12 },
    { 0xb57b53d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1b53dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x649b53e0, SUC_lexicon_tags_26 },
    { 0xe0bb53e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa0bb53e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x811b53e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x367b53e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07b53eb, SUC_lexicon_tags_12 },
    { 0xb5db53eb, SUC_lexicon_tags_14 },
    { 0x7db53ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17b53f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fb53fa, SUC_lexicon_tags_17 },
    { 0xc85b53fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fb53ff, SUC_lexicon_tags_25 },
    { 0x995b53ff, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619b5405, SUC_lexicon_tags_37 },
    { 0x97b5406, SUC_lexicon_tags_12 },
    { 0xae7b5406, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3b540e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3b5411, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x337b5413, SUC_lexicon_tags_43 },
    { 0xe63b5413, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb5419, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7b541d, SUC_lexicon_tags_11 },
    { 0xe9bb541d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x38fb5421, SUC_lexicon_tags_28 },
    { 0x591b5422, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95b5425, SUC_lexicon_tags_11 },
    { 0x91fb5426, SUC_lexicon_tags_87 },
    { 0x733b5427, SUC_lexicon_tags_38 },
    { 0xbd1b5428, SUC_lexicon_tags_18 },
    { 0x877b5429, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfb542f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc67b5431, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf2fb5433, SUC_lexicon_tags_63 },
    { 0x18fb5433, SUC_lexicon_tags_0 },
    { 0x6f9b5433, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3db5442, SUC_lexicon_tags_12 },
    { 0xfb5443, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21b5448, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b544b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1b544e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16fb5450, SUC_lexicon_tags_11 },
    { 0x28fb5450, SUC_lexicon_tags_25 },
    { 0xa9fb5451, SUC_lexicon_tags_23 },
    { 0xb15b5450, SUC_lexicon_tags_1 },
    { 0x929b5454, SUC_lexicon_tags_12 },
    { 0x303b5455, SUC_lexicon_tags_37 },
    { 0x383b5455, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69b545b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbb545e, SUC_lexicon_tags_46 },
    { 0xa25b545e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac5b5461, SUC_lexicon_tags_87 },
    { 0x8e5b5461, SUC_lexicon_tags_16 },
    { 0xfdb5463, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b1b5465, SUC_lexicon_tags_33 },
    { 0xe6fb5466, SUC_lexicon_tags_32 },
    { 0x381b5466, SUC_lexicon_tags_7 },
    { 0xe29b5467, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cdb546a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb546d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9bdb546f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x285b5471, SUC_lexicon_tags_11 },
    { 0x11bb5471, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0db5475, SUC_lexicon_tags_1 },
    { 0x20fb5476, SUC_lexicon_tags_15 },
    { 0x839b5476, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9b547f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7b5482, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7b5485, SUC_lexicon_tags_25 },
    { 0xbe9b5486, SUC_lexicon_tags_63 },
    { 0x92bb5487, SUC_lexicon_tags_30 },
    { 0xe01b5487, SUC_lexicon_tags_17 },
    { 0x467b5489, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61b5490, SUC_lexicon_tags_12 },
    { 0x6d7b5490, SUC_lexicon_tags_15 },
    { 0x91fb5491, SUC_lexicon_tags_49 },
    { 0x51fb5493, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb5499, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58db549b, SUC_lexicon_tags_37 },
    { 0xcadb549c, SUC_lexicon_tags_47 },
    { 0x8a7b549d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99db54a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fb54a5, SUC_lexicon_tags_6 },
    { 0x131b54a5, SUC_lexicon_tags_37 },
    { 0x50bb54a7, SUC_lexicon_tags_2 },
    { 0xd17b54a6, SUC_lexicon_tags_29 },
    { 0x27bb54a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89b54ad, SUC_lexicon_tags_2 },
    { 0x307b54ae, SUC_lexicon_tags_2 },
    { 0xe6db54ae, SUC_lexicon_tags_1033 },
    { 0, NULL },
    { 0x5e3b54b1, SUC_lexicon_tags_12 },
    { 0xc3db54b1, SUC_lexicon_tags_27 },
    { 0xaebb54b3, SUC_lexicon_tags_17 },
    { 0x309b54b4, SUC_lexicon_tags_12 },
    { 0xd37b54b3, SUC_lexicon_tags_1 },
    { 0x6ffb54b6, SUC_lexicon_tags_7 },
    { 0x4b5b54b1, SUC_lexicon_tags_252 },
    { 0x1dfb54b4, SUC_lexicon_tags_18 },
    { 0x78db54b9, SUC_lexicon_tags_11 },
    { 0x10db54b7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bb54bd, SUC_lexicon_tags_3 },
    { 0xdc3b54be, SUC_lexicon_tags_12 },
    { 0x23fb54be, SUC_lexicon_tags_19 },
    { 0xf13b54c0, SUC_lexicon_tags_87 },
    { 0x371b54bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc21b54c3, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401b54c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9b54cc, SUC_lexicon_tags_35 },
    { 0xd4db54cd, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05b54d2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603b54d6, SUC_lexicon_tags_48 },
    { 0x439b54d7, SUC_lexicon_tags_27 },
    { 0x3b7b54d8, SUC_lexicon_tags_11 },
    { 0x43db54d9, SUC_lexicon_tags_17 },
    { 0xa75b54d9, SUC_lexicon_tags_37 },
    { 0xc05b54db, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571b54df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbb54e3, SUC_lexicon_tags_48 },
    { 0xd6bb54e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x497b54e7, SUC_lexicon_tags_11 },
    { 0xe11b54e7, SUC_lexicon_tags_35 },
    { 0xd5fb54e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cbb54eb, SUC_lexicon_tags_1 },
    { 0x259b54ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3b54f0, SUC_lexicon_tags_25 },
    { 0xb87b54f1, SUC_lexicon_tags_0 },
    { 0xfb5b54f1, SUC_lexicon_tags_25 },
    { 0xdb3b54f3, SUC_lexicon_tags_55 },
    { 0xd9bb54f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827b54f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefb54fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfb5501, SUC_lexicon_tags_45 },
    { 0xb55b5502, SUC_lexicon_tags_21 },
    { 0x77bb5503, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187b5507, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9b550c, SUC_lexicon_tags_17 },
    { 0xb99b550d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdb5511, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd39b5513, SUC_lexicon_tags_17 },
    { 0x43db5513, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9c9b5516, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6dfb5518, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfb551c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b9b551e, SUC_lexicon_tags_11 },
    { 0x65fb551f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8a3b5521, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd7fb5523, SUC_lexicon_tags_25 },
    { 0x633b5523, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7b5527, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xd91b5529, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbbb552b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xcafb552d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbb5530, SUC_lexicon_tags_8 },
    { 0xfdb5531, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465b5538, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55b553b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53bb553d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711b5543, SUC_lexicon_tags_2 },
    { 0x639b5544, SUC_lexicon_tags_1 },
    { 0xeb7b5544, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99b5551, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bb5556, SUC_lexicon_tags_37 },
    { 0xb3bb5557, SUC_lexicon_tags_12 },
    { 0xe47b5558, SUC_lexicon_tags_12 },
    { 0x8c1b5556, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89b555d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadb5560, SUC_lexicon_tags_11 },
    { 0x1d5b5561, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5b5568, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7adb556a, SUC_lexicon_tags_79 },
    { 0xe59b556b, SUC_lexicon_tags_15 },
    { 0x411b556c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43db5572, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdb5578, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdb557c, SUC_lexicon_tags_47 },
    { 0x92fb557c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ebb557f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x569b5581, SUC_lexicon_tags_29 },
    { 0x9b3b5582, SUC_lexicon_tags_47 },
    { 0x979b5583, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa71b5585, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e1b5587, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3b558b, SUC_lexicon_tags_1 },
    { 0x47b558c, SUC_lexicon_tags_6 },
    { 0x7ddb558d, SUC_lexicon_tags_84 },
    { 0x857b558b, SUC_lexicon_tags_45 },
    { 0xf69b558f, SUC_lexicon_tags_68 },
    { 0xee9b558d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x447b5592, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7b559a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bb55a0, SUC_lexicon_tags_12 },
    { 0x5e7b55a1, SUC_lexicon_tags_0 },
    { 0x857b55a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51bb55a4, SUC_lexicon_tags_43 },
    { 0xaa7b55a5, SUC_lexicon_tags_12 },
    { 0xd23b55a5, SUC_lexicon_tags_15 },
    { 0x259b55a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99bb55a9, SUC_lexicon_tags_15 },
    { 0x7bb55aa, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0xe6db55ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3b9b55ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62db55b1, SUC_lexicon_tags_47 },
    { 0xdf7b55b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x257b55b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc81b55b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53b55bc, SUC_lexicon_tags_37 },
    { 0xd79b55bd, SUC_lexicon_tags_8 },
    { 0x1ffb55bd, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137b55c6, SUC_lexicon_tags_29 },
    { 0xadbb55c7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x737b55ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bb55cf, SUC_lexicon_tags_15 },
    { 0xfb5b55d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abb55d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaebb55d5, SUC_lexicon_tags_30 },
    { 0xed1b55d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93b55de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847b55e5, SUC_lexicon_tags_25 },
    { 0x3c9b55e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595b55ea, SUC_lexicon_tags_2 },
    { 0x2dbb55eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bb55f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15b55f6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531b55fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddb55ff, SUC_lexicon_tags_28 },
    { 0xaf5b55ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb5606, SUC_lexicon_tags_77 },
    { 0x4cfb5606, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afb560c, SUC_lexicon_tags_151 },
    { 0x7d3b560d, SUC_lexicon_tags_11 },
    { 0xef5b560e, SUC_lexicon_tags_4 },
    { 0x1efb560e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffb5613, SUC_lexicon_tags_37 },
    { 0xa17b5613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535b5623, SUC_lexicon_tags_27 },
    { 0x7fb5624, SUC_lexicon_tags_49 },
    { 0xa79b5624, SUC_lexicon_tags_55 },
    { 0xe31b5626, SUC_lexicon_tags_87 },
    { 0x55b5627, SUC_lexicon_tags_49 },
    { 0x93db5623, SUC_lexicon_tags_63 },
    { 0xd7fb5629, SUC_lexicon_tags_23 },
    { 0xc1fb562a, SUC_lexicon_tags_98 },
    { 0x45b562a, SUC_lexicon_tags_14 },
    { 0xd4bb562c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde3b562e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5b5631, SUC_lexicon_tags_15 },
    { 0xc1bb5632, SUC_lexicon_tags_11 },
    { 0x2c5b5633, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb5b5635, SUC_lexicon_tags_28 },
    { 0xccfb5636, SUC_lexicon_tags_77 },
    { 0x561b5637, SUC_lexicon_tags_17 },
    { 0x845b5637, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fb563c, SUC_lexicon_tags_14 },
    { 0x7edb563d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdfb563f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedb5646, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdb5649, SUC_lexicon_tags_40 },
    { 0xa1bb564a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x487b564d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707b5653, SUC_lexicon_tags_15 },
    { 0xcbb5653, SUC_lexicon_tags_27 },
    { 0x213b5653, SUC_lexicon_tags_30 },
    { 0x419b5655, SUC_lexicon_tags_11 },
    { 0x11db5657, SUC_lexicon_tags_11 },
    { 0x93bb5653, SUC_lexicon_tags_2 },
    { 0x77fb5659, SUC_lexicon_tags_39 },
    { 0x1c9b5659, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5a3b565c, SUC_lexicon_tags_45 },
    { 0x6d1b565d, SUC_lexicon_tags_11 },
    { 0xd19b565c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x217b5660, SUC_lexicon_tags_61 },
    { 0xce7b5661, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311b5665, SUC_lexicon_tags_14 },
    { 0x3b5665, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77db5669, SUC_lexicon_tags_1 },
    { 0x775b566a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x993b566d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c3b566f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5b5672, SUC_lexicon_tags_11 },
    { 0xf31b5673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf33b5675, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x947b5678, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47b567b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdabb567d, SUC_lexicon_tags_28 },
    { 0x401b567e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fb5682, SUC_lexicon_tags_29 },
    { 0x2cbb5683, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x84fb5685, SUC_lexicon_tags_17 },
    { 0xbf9b5685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bb5689, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x401b568b, SUC_lexicon_tags_60 },
    { 0x78fb568b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdb5692, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1b5699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edb569f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61b56a2, SUC_lexicon_tags_25 },
    { 0x6fb56a3, SUC_lexicon_tags_37 },
    { 0xf8db56a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe11b56a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x367b56a8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6db56ab, SUC_lexicon_tags_57 },
    { 0xef1b56ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39b56b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457b56bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fb56c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x80db56c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893b56c9, SUC_lexicon_tags_7 },
    { 0x8b3b56ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955b56d0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x6a5b56d2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581b56d9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5b56de, SUC_lexicon_tags_17 },
    { 0x483b56de, SUC_lexicon_tags_0 },
    { 0xba1b56e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb56e5, SUC_lexicon_tags_19 },
    { 0x841b56e5, SUC_lexicon_tags_29 },
    { 0x8a7b56e7, SUC_lexicon_tags_17 },
    { 0x815b56e5, SUC_lexicon_tags_118 },
    { 0x2ebb56e5, SUC_lexicon_tags_33 },
    { 0x7b9b56ea, SUC_lexicon_tags_2 },
    { 0x211b56e5, SUC_lexicon_tags_37 },
    { 0x817b56ec, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77b56f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xda5b56f5, SUC_lexicon_tags_8 },
    { 0x571b56f6, SUC_lexicon_tags_1 },
    { 0xe65b56f7, SUC_lexicon_tags_37 },
    { 0xaa5b56f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbb56fd, SUC_lexicon_tags_17 },
    { 0xe11b56fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1a3b5700, SUC_lexicon_tags_11 },
    { 0x419b5701, SUC_lexicon_tags_17 },
    { 0xa6db5702, SUC_lexicon_tags_17 },
    { 0x6cdb5703, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe15b5705, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x91db5707, SUC_lexicon_tags_11 },
    { 0x23fb5708, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1b570b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f7b570d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x581b570f, SUC_lexicon_tags_83 },
    { 0xbcdb5710, SUC_lexicon_tags_7 },
    { 0xe1db5711, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x85bb5713, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bb5716, SUC_lexicon_tags_4 },
    { 0xba5b5717, SUC_lexicon_tags_25 },
    { 0xc63b5717, SUC_lexicon_tags_31 },
    { 0x715b5719, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x517b571b, SUC_lexicon_tags_1 },
    { 0x71fb571b, SUC_lexicon_tags_2 },
    { 0x1b9b571d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbb5720, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5b572a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4ebb572c, SUC_lexicon_tags_3 },
    { 0xe0bb572d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x571b572f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efb5734, SUC_lexicon_tags_2 },
    { 0xd91b5735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c7b5737, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49b573a, SUC_lexicon_tags_2 },
    { 0xd03b573a, SUC_lexicon_tags_37 },
    { 0x66bb573c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x407b573e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc81b5740, SUC_lexicon_tags_17 },
    { 0xa5b5741, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x89fb5743, SUC_lexicon_tags_37 },
    { 0xc41b5744, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x279b5746, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddb574d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa95b574f, SUC_lexicon_tags_15 },
    { 0x387b5750, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x951b5753, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3b575a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b575e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481b5762, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37fb5764, SUC_lexicon_tags_11 },
    { 0x58fb5764, SUC_lexicon_tags_65 },
    { 0x343b5764, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb49b5768, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5b576e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x847b5770, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5b5778, SUC_lexicon_tags_16 },
    { 0x4d1b5779, SUC_lexicon_tags_11 },
    { 0x5dfb5779, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x551b577d, SUC_lexicon_tags_37 },
    { 0xfb3b577e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6db5785, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5b5788, SUC_lexicon_tags_11 },
    { 0x3a1b5788, SUC_lexicon_tags_26 },
    { 0x75b578a, SUC_lexicon_tags_9 },
    { 0x9afb578b, SUC_lexicon_tags_12 },
    { 0x9c9b5788, SUC_lexicon_tags_38 },
    { 0x715b578d, SUC_lexicon_tags_11 },
    { 0x163b578e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x401b5790, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7b5793, SUC_lexicon_tags_37 },
    { 0x591b5794, SUC_lexicon_tags_410 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837b579d, SUC_lexicon_tags_97 },
    { 0x705b579e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45b57a2, SUC_lexicon_tags_430 },
    { 0xb63b57a3, SUC_lexicon_tags_27 },
    { 0x109b57a4, SUC_lexicon_tags_31 },
    { 0x473b57a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65db57a7, SUC_lexicon_tags_21 },
    { 0x68bb57a7, SUC_lexicon_tags_37 },
    { 0x76fb57a9, SUC_lexicon_tags_6 },
    { 0xfffb57a8, SUC_lexicon_tags_0 },
    { 0x46fb57ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451b57af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b57b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895b57ba, SUC_lexicon_tags_43 },
    { 0x799b57bb, SUC_lexicon_tags_19 },
    { 0x367b57bb, SUC_lexicon_tags_5 },
    { 0x265b57bd, SUC_lexicon_tags_11 },
    { 0xe3b57be, SUC_lexicon_tags_26 },
    { 0x5e9b57bf, SUC_lexicon_tags_1 },
    { 0xb99b57c0, SUC_lexicon_tags_2 },
    { 0x38db57c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8edb57c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3b57cc, SUC_lexicon_tags_83 },
    { 0xb5fb57cd, SUC_lexicon_tags_87 },
    { 0x87b57cd, SUC_lexicon_tags_15 },
    { 0xbffb57cf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9b57d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5b57d8, SUC_lexicon_tags_17 },
    { 0xfb3b57d9, SUC_lexicon_tags_2 },
    { 0x6c7b57da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bb57df, SUC_lexicon_tags_37 },
    { 0x8d1b57df, SUC_lexicon_tags_27 },
    { 0xab9b57e1, SUC_lexicon_tags_17 },
    { 0xba7b57e2, SUC_lexicon_tags_15 },
    { 0x819b57e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12db57e7, SUC_lexicon_tags_63 },
    { 0x925b57e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfb57ed, SUC_lexicon_tags_293 },
    { 0x715b57ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x659b57f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x60db57f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33b57f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161b5800, SUC_lexicon_tags_25 },
    { 0x163b5800, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5b5806, SUC_lexicon_tags_17 },
    { 0x385b5807, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fdb5809, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b580c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9b580f, SUC_lexicon_tags_23 },
    { 0x3f1b5810, SUC_lexicon_tags_38 },
    { 0xce1b5811, SUC_lexicon_tags_27 },
    { 0xd2db5812, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x47db5814, SUC_lexicon_tags_7 },
    { 0x717b5815, SUC_lexicon_tags_37 },
    { 0xf51b5816, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x48db5818, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91fb581a, SUC_lexicon_tags_43 },
    { 0xbd3b581a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5b581f, SUC_lexicon_tags_38 },
    { 0x63bb581f, SUC_lexicon_tags_15 },
    { 0x8bdb5821, SUC_lexicon_tags_7 },
    { 0xf6db5822, SUC_lexicon_tags_47 },
    { 0x4c1b5822, SUC_lexicon_tags_11 },
    { 0x323b5824, SUC_lexicon_tags_15 },
    { 0x969b5825, SUC_lexicon_tags_0 },
    { 0xa17b5822, SUC_lexicon_tags_47 },
    { 0x89bb5824, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bb5836, SUC_lexicon_tags_421 },
    { 0xb6db5837, SUC_lexicon_tags_8 },
    { 0xcbfb5837, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbbdb583a, SUC_lexicon_tags_15 },
    { 0x72fb583b, SUC_lexicon_tags_7 },
    { 0x521b583c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd17b583e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefb5845, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc23b5847, SUC_lexicon_tags_37 },
    { 0x44db5847, SUC_lexicon_tags_33 },
    { 0x947b5849, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc7b584b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693b5859, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27b585f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3b5862, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59b586c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e9b586e, SUC_lexicon_tags_7 },
    { 0x83bb586f, SUC_lexicon_tags_12 },
    { 0x8e7b586f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x84bb5872, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa23b5874, SUC_lexicon_tags_0 },
    { 0xf17b5875, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0x8b9b5877, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x38db5879, SUC_lexicon_tags_17 },
    { 0xd8bb587a, SUC_lexicon_tags_28 },
    { 0x4e3b587b, SUC_lexicon_tags_23 },
    { 0x7f9b5879, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9b5883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11b5886, SUC_lexicon_tags_79 },
    { 0x90db5887, SUC_lexicon_tags_0 },
    { 0xab7b5888, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21b588b, SUC_lexicon_tags_37 },
    { 0x3d3b588b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113b5890, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281b5897, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349b589c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1b589f, SUC_lexicon_tags_2 },
    { 0x19b589f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x69b58a2, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3b58a5, SUC_lexicon_tags_37 },
    { 0x5d3b58a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x277b58a9, SUC_lexicon_tags_27 },
    { 0x8dbb58aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x863b58ac, SUC_lexicon_tags_43 },
    { 0x8fb58ad, SUC_lexicon_tags_19 },
    { 0x4c1b58ae, SUC_lexicon_tags_47 },
    { 0xe05b58ac, SUC_lexicon_tags_31 },
    { 0xdefb58b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb61b58b2, SUC_lexicon_tags_26 },
    { 0xf4fb58b3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x857b58b5, SUC_lexicon_tags_25 },
    { 0xc1fb58b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667b58ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0bb58bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe59b58be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb58c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17b58c6, SUC_lexicon_tags_25 },
    { 0x72bb58c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x505b58c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fb58cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xca7b58cf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51b58d3, SUC_lexicon_tags_9 },
    { 0xa71b58d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37b58d8, SUC_lexicon_tags_2 },
    { 0x2c5b58d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xccbb58db, SUC_lexicon_tags_56 },
    { 0xb53b58dc, SUC_lexicon_tags_66 },
    { 0x9dbb58db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x849b58e0, SUC_lexicon_tags_1 },
    { 0x7b9b58e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x235b58e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253b58eb, SUC_lexicon_tags_243 },
    { 0x3d5b58ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x871b58ee, SUC_lexicon_tags_12 },
    { 0xa2fb58ef, SUC_lexicon_tags_1 },
    { 0xf0bb58f0, SUC_lexicon_tags_61 },
    { 0x9bbb58f1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa23b58f3, SUC_lexicon_tags_17 },
    { 0x853b58f4, SUC_lexicon_tags_11 },
    { 0x5c7b58f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8db58f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db58fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x899b5900, SUC_lexicon_tags_27 },
    { 0x57b5901, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbb5904, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85db5906, SUC_lexicon_tags_17 },
    { 0x43bb5907, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1b590b, SUC_lexicon_tags_6 },
    { 0xef9b590b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bb590f, SUC_lexicon_tags_47 },
    { 0x701b5910, SUC_lexicon_tags_12 },
    { 0x31b5911, SUC_lexicon_tags_30 },
    { 0x3d5b5911, SUC_lexicon_tags_17 },
    { 0x56db5913, SUC_lexicon_tags_29 },
    { 0x791b5914, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bb5917, SUC_lexicon_tags_1 },
    { 0x9e3b5917, SUC_lexicon_tags_1 },
    { 0x35bb5919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3b591f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x273b5922, SUC_lexicon_tags_47 },
    { 0xfc1b5923, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfb5926, SUC_lexicon_tags_7 },
    { 0x4efb5927, SUC_lexicon_tags_9 },
    { 0xdbbb5927, SUC_lexicon_tags_8 },
    { 0xbfb5929, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbe1b592b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d3b592d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133b5931, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afb5934, SUC_lexicon_tags_2 },
    { 0x84db5935, SUC_lexicon_tags_8 },
    { 0xb5fb5936, SUC_lexicon_tags_46 },
    { 0xeb7b5936, SUC_lexicon_tags_21 },
    { 0xbd9b5938, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb593d, SUC_lexicon_tags_87 },
    { 0x3f9b593e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edb5944, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71b594b, SUC_lexicon_tags_12 },
    { 0xa7db594c, SUC_lexicon_tags_37 },
    { 0x8c5b594d, SUC_lexicon_tags_14 },
    { 0xa4bb594e, SUC_lexicon_tags_37 },
    { 0x61bb594f, SUC_lexicon_tags_21 },
    { 0xc6bb594c, SUC_lexicon_tags_15 },
    { 0x7b9b5951, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac9b5953, SUC_lexicon_tags_33 },
    { 0x4dfb5954, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2bdb5956, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d1b5958, SUC_lexicon_tags_48 },
    { 0x521b5959, SUC_lexicon_tags_60 },
    { 0x115b5959, SUC_lexicon_tags_48 },
    { 0x3edb595b, SUC_lexicon_tags_11 },
    { 0xe65b595c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1d5b595e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bb5961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbdb5963, SUC_lexicon_tags_25 },
    { 0xe97b5963, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25b5967, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x30db596a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3efb596c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9b5972, SUC_lexicon_tags_29 },
    { 0xc4db5973, SUC_lexicon_tags_17 },
    { 0xd9db5973, SUC_lexicon_tags_28 },
    { 0x4cdb5975, SUC_lexicon_tags_34 },
    { 0xb9bb5973, SUC_lexicon_tags_14 },
    { 0xb8bb5976, SUC_lexicon_tags_3 },
    { 0xf4db5975, SUC_lexicon_tags_47 },
    { 0x383b5979, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fb597d, SUC_lexicon_tags_6 },
    { 0x1efb597e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51b5984, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d5b5986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac1b5988, SUC_lexicon_tags_37 },
    { 0x61b5989, SUC_lexicon_tags_48 },
    { 0xed1b598a, SUC_lexicon_tags_23 },
    { 0x637b5989, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191b5990, SUC_lexicon_tags_28 },
    { 0xd7db5991, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb5994, SUC_lexicon_tags_23 },
    { 0x5adb5994, SUC_lexicon_tags_34 },
    { 0x591b5996, SUC_lexicon_tags_17 },
    { 0xb57b5994, SUC_lexicon_tags_11 },
    { 0xc01b5997, SUC_lexicon_tags_11 },
    { 0x48db5999, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331b599e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6db59a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebb59a6, SUC_lexicon_tags_8 },
    { 0x6b9b59a7, SUC_lexicon_tags_1 },
    { 0xa93b59a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2fb59aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38fb59ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d5b59ae, SUC_lexicon_tags_15 },
    { 0x6afb59af, SUC_lexicon_tags_40 },
    { 0x40db59ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21b59b3, SUC_lexicon_tags_9 },
    { 0x989b59b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1b59ba, SUC_lexicon_tags_19 },
    { 0x54fb59bb, SUC_lexicon_tags_11 },
    { 0x6e1b59bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64bb59be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311b59c2, SUC_lexicon_tags_26 },
    { 0xb3db59c2, SUC_lexicon_tags_2 },
    { 0x61db59c4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bb59c7, SUC_lexicon_tags_37 },
    { 0x261b59c7, SUC_lexicon_tags_124 },
    { 0x239b59c9, SUC_lexicon_tags_17 },
    { 0x749b59ca, SUC_lexicon_tags_46 },
    { 0x9f5b59ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b7b59cd, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fb59d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x895b59d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235b59d9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc69b59db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9b59e0, SUC_lexicon_tags_15 },
    { 0xe3db59e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x173b59e3, SUC_lexicon_tags_25 },
    { 0xddbb59e4, SUC_lexicon_tags_25 },
    { 0xebfb59e4, SUC_lexicon_tags_26 },
    { 0x86bb59e6, SUC_lexicon_tags_6 },
    { 0x203b59e7, SUC_lexicon_tags_29 },
    { 0x281b59e8, SUC_lexicon_tags_31 },
    { 0x5adb59e4, SUC_lexicon_tags_23 },
    { 0x833b59e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fb59ed, SUC_lexicon_tags_12 },
    { 0xae9b59ee, SUC_lexicon_tags_15 },
    { 0x8e1b59ee, SUC_lexicon_tags_85 },
    { 0x63fb59ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bb59f6, SUC_lexicon_tags_15 },
    { 0xb91b59f7, SUC_lexicon_tags_19 },
    { 0x2c9b59f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffb5a02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebb5a09, SUC_lexicon_tags_31 },
    { 0x1a9b5a0a, SUC_lexicon_tags_25 },
    { 0x269b5a09, SUC_lexicon_tags_2 },
    { 0x391b5a0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbb5a10, SUC_lexicon_tags_53 },
    { 0x17db5a11, SUC_lexicon_tags_7 },
    { 0xd37b5a12, SUC_lexicon_tags_23 },
    { 0xf67b5a13, SUC_lexicon_tags_9 },
    { 0x865b5a12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x91bb5a16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffb5a1a, SUC_lexicon_tags_9 },
    { 0x15b5a1a, SUC_lexicon_tags_2 },
    { 0xab5b5a1c, SUC_lexicon_tags_23 },
    { 0x69bb5a1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9afb5a1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6db5a21, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fb5a28, SUC_lexicon_tags_17 },
    { 0x419b5a28, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6db5a2c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfb5a30, SUC_lexicon_tags_19 },
    { 0xc1fb5a30, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabb5a38, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x859b5a3a, SUC_lexicon_tags_77 },
    { 0x457b5a3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903b5a40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x325b5a42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7b5a45, SUC_lexicon_tags_17 },
    { 0x40db5a46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddb5a4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5edb5a4e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xbafb5a50, SUC_lexicon_tags_27 },
    { 0xb81b5a51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3db5a56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfb5a5a, SUC_lexicon_tags_11 },
    { 0xfe7b5a5b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x47db5a5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x795b5a5f, SUC_lexicon_tags_1 },
    { 0x2b1b5a60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55b5a69, SUC_lexicon_tags_19 },
    { 0x42fb5a6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x155b5a6c, SUC_lexicon_tags_12 },
    { 0xcf9b5a6d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x709b5a70, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585b5a75, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2adb5a79, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7b5a7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x773b5a81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadfb5a84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83b5a87, SUC_lexicon_tags_47 },
    { 0x4c9b5a88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97b5a8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3b5a91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725b5a95, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4a5b5a97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b5aa3, SUC_lexicon_tags_37 },
    { 0xeb7b5aa4, SUC_lexicon_tags_11 },
    { 0xbefb5aa3, SUC_lexicon_tags_37 },
    { 0x643b5aa5, SUC_lexicon_tags_38 },
    { 0xdc3b5aa7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1b5aaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63b5aaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bb5ab3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b5ab8, SUC_lexicon_tags_50 },
    { 0xc2bb5ab8, SUC_lexicon_tags_48 },
    { 0x367b5aba, SUC_lexicon_tags_26 },
    { 0x709b5abb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x401b5abe, SUC_lexicon_tags_57 },
    { 0x5f9b5abf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x933b5ac1, SUC_lexicon_tags_12 },
    { 0x935b5ac2, SUC_lexicon_tags_19 },
    { 0xe73b5ac2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x991b5ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37b5ac9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85b5ace, SUC_lexicon_tags_1 },
    { 0xd75b5ace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf41b5ad1, SUC_lexicon_tags_8 },
    { 0xb47b5ad2, SUC_lexicon_tags_14 },
    { 0xfd7b5ad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdb5ad6, SUC_lexicon_tags_38 },
    { 0x9efb5ad7, SUC_lexicon_tags_1 },
    { 0xabb5ad8, SUC_lexicon_tags_15 },
    { 0x8abb5ad8, SUC_lexicon_tags_1 },
    { 0x297b5ada, SUC_lexicon_tags_2 },
    { 0x34db5ad7, SUC_lexicon_tags_45 },
    { 0x35fb5ad7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x853b5adf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1b5aea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07b5af6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbb5af9, SUC_lexicon_tags_25 },
    { 0x9c5b5afa, SUC_lexicon_tags_17 },
    { 0xc9b5afa, SUC_lexicon_tags_28 },
    { 0xf3bb5afa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1db5b00, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x597b5b02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bb5b05, SUC_lexicon_tags_63 },
    { 0xa2db5b06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddb5b0b, SUC_lexicon_tags_26 },
    { 0xd7bb5b0c, SUC_lexicon_tags_70 },
    { 0xe3b5b0d, SUC_lexicon_tags_19 },
    { 0x29db5b0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfb5b17, SUC_lexicon_tags_0 },
    { 0x8dbb5b17, SUC_lexicon_tags_30 },
    { 0x5efb5b19, SUC_lexicon_tags_2 },
    { 0x94bb5b1a, SUC_lexicon_tags_37 },
    { 0x95b5b19, SUC_lexicon_tags_30 },
    { 0xb3db5b18, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xd47b5b1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fb5b2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbb5b2e, SUC_lexicon_tags_46 },
    { 0xdfdb5b2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36bb5b31, SUC_lexicon_tags_11 },
    { 0x13fb5b32, SUC_lexicon_tags_1 },
    { 0xef1b5b32, SUC_lexicon_tags_33 },
    { 0x50fb5b32, SUC_lexicon_tags_37 },
    { 0x635b5b32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295b5b3c, SUC_lexicon_tags_14 },
    { 0xf0fb5b3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bb5b42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x225b5b45, SUC_lexicon_tags_30 },
    { 0x40fb5b46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc75b5b48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9b5b4b, SUC_lexicon_tags_38 },
    { 0x653b5b4b, SUC_lexicon_tags_48 },
    { 0xca3b5b4c, SUC_lexicon_tags_55 },
    { 0x3abb5b4e, SUC_lexicon_tags_17 },
    { 0xadbb5b4f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5b5b54, SUC_lexicon_tags_23 },
    { 0xbe1b5b54, SUC_lexicon_tags_7 },
    { 0x77bb5b56, SUC_lexicon_tags_84 },
    { 0xb69b5b57, SUC_lexicon_tags_47 },
    { 0x1a5b5b58, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xabb5b5a, SUC_lexicon_tags_26 },
    { 0x18fb5b5b, SUC_lexicon_tags_17 },
    { 0x4dbb5b5a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afb5b5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5b5b63, SUC_lexicon_tags_46 },
    { 0xee1b5b64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55b5b66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abb5b69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdb5b74, SUC_lexicon_tags_17 },
    { 0x8adb5b75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277b5b79, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1b5b7c, SUC_lexicon_tags_37 },
    { 0xe3bb5b7d, SUC_lexicon_tags_37 },
    { 0x1bbb5b7c, SUC_lexicon_tags_1 },
    { 0x419b5b7e, SUC_lexicon_tags_68 },
    { 0xb43b5b80, SUC_lexicon_tags_23 },
    { 0x1b5b5b81, SUC_lexicon_tags_12 },
    { 0x4bdb5b82, SUC_lexicon_tags_37 },
    { 0x7dfb5b81, SUC_lexicon_tags_11 },
    { 0x315b5b84, SUC_lexicon_tags_1 },
    { 0x247b5b85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7b5b88, SUC_lexicon_tags_19 },
    { 0xc2bb5b89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7fb5b8b, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7b5b8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb5b92, SUC_lexicon_tags_30 },
    { 0xa45b5b93, SUC_lexicon_tags_37 },
    { 0xaf5b5b94, SUC_lexicon_tags_17 },
    { 0xb19b5b93, SUC_lexicon_tags_27 },
    { 0x31b5b92, SUC_lexicon_tags_66 },
    { 0x241b5b97, SUC_lexicon_tags_11 },
    { 0xdbfb5b97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31fb5b9b, SUC_lexicon_tags_15 },
    { 0x497b5b9c, SUC_lexicon_tags_23 },
    { 0x8b3b5b9d, SUC_lexicon_tags_30 },
    { 0x7c5b5b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff7b5ba0, SUC_lexicon_tags_37 },
    { 0x117b5ba0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8db5ba4, SUC_lexicon_tags_1 },
    { 0x229b5ba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fb5ba8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9fb5baa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc09b5bac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289b5bb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fb5bb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fb5bb9, SUC_lexicon_tags_11 },
    { 0x9f7b5bba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327b5bbe, SUC_lexicon_tags_19 },
    { 0xf45b5bbe, SUC_lexicon_tags_42 },
    { 0xef7b5bc0, SUC_lexicon_tags_25 },
    { 0x629b5bc1, SUC_lexicon_tags_29 },
    { 0xf6bb5bc2, SUC_lexicon_tags_77 },
    { 0xcf9b5bc3, SUC_lexicon_tags_17 },
    { 0xa27b5bc3, SUC_lexicon_tags_33 },
    { 0xda5b5bc5, SUC_lexicon_tags_32 },
    { 0xb29b5bc6, SUC_lexicon_tags_45 },
    { 0x609b5bc7, SUC_lexicon_tags_21 },
    { 0x2b3b5bc5, SUC_lexicon_tags_118 },
    { 0xb61b5bc3, SUC_lexicon_tags_30 },
    { 0x781b5bca, SUC_lexicon_tags_25 },
    { 0xfb5b5bc4, SUC_lexicon_tags_11 },
    { 0x8fb5bc5, SUC_lexicon_tags_33 },
    { 0x5afb5bca, SUC_lexicon_tags_2 },
    { 0xc7bb5bce, SUC_lexicon_tags_23 },
    { 0x9c9b5bcf, SUC_lexicon_tags_14 },
    { 0x47db5bcf, SUC_lexicon_tags_37 },
    { 0xf73b5bc5, SUC_lexicon_tags_43 },
    { 0xb01b5bd2, SUC_lexicon_tags_65 },
    { 0xb47b5bd3, SUC_lexicon_tags_29 },
    { 0xa9db5bd4, SUC_lexicon_tags_21 },
    { 0x3db5bd0, SUC_lexicon_tags_7 },
    { 0xe11b5bd6, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96db5bdb, SUC_lexicon_tags_15 },
    { 0x1a9b5bdc, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7db5be6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73b5bef, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fb5bf2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdb5bf5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3b5bf8, SUC_lexicon_tags_31 },
    { 0x3edb5bf8, SUC_lexicon_tags_2 },
    { 0xc77b5bf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fb5c00, SUC_lexicon_tags_1243 },
    { 0x717b5c00, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42db5c07, SUC_lexicon_tags_2 },
    { 0x3d7b5c07, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1b5c0b, SUC_lexicon_tags_28 },
    { 0x975b5c0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fb5c12, SUC_lexicon_tags_7 },
    { 0x49db5c12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80bb5c15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x96bb5c17, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35b5c1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19b5c21, SUC_lexicon_tags_11 },
    { 0xf79b5c22, SUC_lexicon_tags_33 },
    { 0x917b5c23, SUC_lexicon_tags_11 },
    { 0xa25b5c24, SUC_lexicon_tags_60 },
    { 0xb05b5c25, SUC_lexicon_tags_29 },
    { 0x9ebb5c26, SUC_lexicon_tags_1 },
    { 0xeb7b5c27, SUC_lexicon_tags_1 },
    { 0x36bb5c26, SUC_lexicon_tags_48 },
    { 0xec9b5c29, SUC_lexicon_tags_49 },
    { 0x56fb5c23, SUC_lexicon_tags_4 },
    { 0x7e9b5c24, SUC_lexicon_tags_187 },
    { 0x495b5c2c, SUC_lexicon_tags_15 },
    { 0x2dfb5c2b, SUC_lexicon_tags_15 },
    { 0x107b5c26, SUC_lexicon_tags_39 },
    { 0xedb5c24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfdbb5c31, SUC_lexicon_tags_17 },
    { 0xe1bb5c32, SUC_lexicon_tags_37 },
    { 0x1db5c33, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afb5c38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbb5c3b, SUC_lexicon_tags_28 },
    { 0xe5bb5c3c, SUC_lexicon_tags_107 },
    { 0xa75b5c3c, SUC_lexicon_tags_45 },
    { 0xb95b5c3c, SUC_lexicon_tags_30 },
    { 0xa69b5c3f, SUC_lexicon_tags_0 },
    { 0x3f1b5c40, SUC_lexicon_tags_68 },
    { 0xc3b5c3e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83b5c45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebb5c48, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb5c4c, SUC_lexicon_tags_23 },
    { 0x9db5c4d, SUC_lexicon_tags_37 },
    { 0xb35b5c4c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfb5c54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x141b5c57, SUC_lexicon_tags_39 },
    { 0x6abb5c58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabb5c5d, SUC_lexicon_tags_27 },
    { 0xa57b5c5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bb5c63, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfb5c68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x941b5c6a, SUC_lexicon_tags_14 },
    { 0x19bb5c6b, SUC_lexicon_tags_12 },
    { 0xcd9b5c6b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x961b5c6e, SUC_lexicon_tags_38 },
    { 0xa3fb5c6f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55db5c75, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749b5c7a, SUC_lexicon_tags_47 },
    { 0xf33b5c7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49b5c80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655b5c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249b5c89, SUC_lexicon_tags_15 },
    { 0xf3db5c8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9b5c92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7b5c95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467b5c9b, SUC_lexicon_tags_2 },
    { 0x85db5c9c, SUC_lexicon_tags_3 },
    { 0x389b5c9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7fdb5c9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40fb5ca1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35b5ca6, SUC_lexicon_tags_37 },
    { 0x177b5ca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x311b5ca9, SUC_lexicon_tags_47 },
    { 0xe57b5caa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x375b5cad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84db5cb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7b5cb5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55b5cba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8db5cbc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x55fb5cbe, SUC_lexicon_tags_25 },
    { 0x1db5cbe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32db5cca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adb5ccd, SUC_lexicon_tags_15 },
    { 0x8adb5cce, SUC_lexicon_tags_29 },
    { 0x8a7b5ccf, SUC_lexicon_tags_14 },
    { 0xfb1b5cd0, SUC_lexicon_tags_28 },
    { 0x3bbb5ccf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xabfb5cd4, SUC_lexicon_tags_60 },
    { 0x5bb5cd5, SUC_lexicon_tags_43 },
    { 0xcd7b5cd4, SUC_lexicon_tags_81 },
    { 0x811b5cd7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29b5cdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91b5ce1, SUC_lexicon_tags_37 },
    { 0xb7bb5ce1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5db5ce6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2c5b5ce8, SUC_lexicon_tags_11 },
    { 0xd2bb5ce8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609b5ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967b5cf1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3b5cf5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6db5cfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb5d04, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50db5d09, SUC_lexicon_tags_14 },
    { 0xf1fb5d0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbb5d0e, SUC_lexicon_tags_27 },
    { 0x47fb5d0e, SUC_lexicon_tags_4 },
    { 0xf5db5d10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bb5d18, SUC_lexicon_tags_7 },
    { 0x517b5d19, SUC_lexicon_tags_25 },
    { 0x243b5d1a, SUC_lexicon_tags_7 },
    { 0xa03b5d1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93b5d20, SUC_lexicon_tags_11 },
    { 0xccfb5d21, SUC_lexicon_tags_60 },
    { 0x5b5b5d22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x419b5d24, SUC_lexicon_tags_1 },
    { 0xd57b5d25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfb5d29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x76db5d2c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbb5d30, SUC_lexicon_tags_65 },
    { 0x2ffb5d31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167b5d35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fb5d3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x643b5d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147b5d41, SUC_lexicon_tags_37 },
    { 0xa11b5d42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb65b5d44, SUC_lexicon_tags_1 },
    { 0xa45b5d45, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5b5d48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283b5d4e, SUC_lexicon_tags_14 },
    { 0xa67b5d4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bb5d53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fb5d56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5b5d58, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x947b5d5a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fb5d5d, SUC_lexicon_tags_17 },
    { 0xf09b5d5e, SUC_lexicon_tags_27 },
    { 0xc0db5d5e, SUC_lexicon_tags_11 },
    { 0x595b5d5f, SUC_lexicon_tags_98 },
    { 0xca5b5d5e, SUC_lexicon_tags_7 },
    { 0xc27b5d62, SUC_lexicon_tags_1 },
    { 0x349b5d62, SUC_lexicon_tags_82 },
    { 0x453b5d64, SUC_lexicon_tags_37 },
    { 0xf39b5d65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51b5d69, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b5d70, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafb5d79, SUC_lexicon_tags_57 },
    { 0x4b1b5d7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9b5d7e, SUC_lexicon_tags_14 },
    { 0xdb5b5d7f, SUC_lexicon_tags_25 },
    { 0x1f5b5d7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649b5d85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x871b5d87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x443b5d89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x527b5d8b, SUC_lexicon_tags_45 },
    { 0xdf5b5d8b, SUC_lexicon_tags_33 },
    { 0xc4fb5d8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x33fb5d8f, SUC_lexicon_tags_46 },
    { 0x70bb5d8f, SUC_lexicon_tags_43 },
    { 0x687b5d91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669b5d95, SUC_lexicon_tags_2 },
    { 0xe0db5d96, SUC_lexicon_tags_68 },
    { 0x909b5d97, SUC_lexicon_tags_30 },
    { 0x715b5d98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb5d9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdb5d9f, SUC_lexicon_tags_10 },
    { 0x39db5da0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99b5da7, SUC_lexicon_tags_1 },
    { 0xf41b5da7, SUC_lexicon_tags_46 },
    { 0x923b5da9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafb5dac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1b5db1, SUC_lexicon_tags_15 },
    { 0x7a7b5db2, SUC_lexicon_tags_55 },
    { 0xc7fb5db1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551b5dbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49db5dbf, SUC_lexicon_tags_37 },
    { 0xdb1b5dc0, SUC_lexicon_tags_15 },
    { 0x5bdb5dc0, SUC_lexicon_tags_16 },
    { 0xce9b5dc2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x12db5dc5, SUC_lexicon_tags_2 },
    { 0xc0bb5dc6, SUC_lexicon_tags_40 },
    { 0x233b5dc7, SUC_lexicon_tags_15 },
    { 0xc95b5dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd1b5dca, SUC_lexicon_tags_85 },
    { 0x165b5dcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191b5dd0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1c3b5dd2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa05b5dd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafb5dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4fbb5dd9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fb5ddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47b5de2, SUC_lexicon_tags_1 },
    { 0x319b5de3, SUC_lexicon_tags_17 },
    { 0x1edb5de4, SUC_lexicon_tags_45 },
    { 0x7cfb5de2, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325b5dea, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x90bb5dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bb5df0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bb5df7, SUC_lexicon_tags_6 },
    { 0x137b5df8, SUC_lexicon_tags_29 },
    { 0x811b5df8, SUC_lexicon_tags_64 },
    { 0x361b5dfa, SUC_lexicon_tags_92 },
    { 0x55fb5dfb, SUC_lexicon_tags_2 },
    { 0x59bb5dfc, SUC_lexicon_tags_2 },
    { 0x337b5df9, SUC_lexicon_tags_37 },
    { 0x27fb5dfe, SUC_lexicon_tags_28 },
    { 0xe43b5dff, SUC_lexicon_tags_15 },
    { 0x89db5e00, SUC_lexicon_tags_35 },
    { 0x445b5e01, SUC_lexicon_tags_11 },
    { 0xfc7b5dfc, SUC_lexicon_tags_17 },
    { 0xee9b5dfa, SUC_lexicon_tags_151 },
    { 0x149b5e04, SUC_lexicon_tags_14 },
    { 0x9f9b5e03, SUC_lexicon_tags_87 },
    { 0xb11b5dfb, SUC_lexicon_tags_1 },
    { 0xe0fb5e07, SUC_lexicon_tags_7 },
    { 0x681b5e07, SUC_lexicon_tags_37 },
    { 0xd8db5e00, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efb5e10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x63fb5e12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831b5e18, SUC_lexicon_tags_12 },
    { 0x8f5b5e19, SUC_lexicon_tags_71 },
    { 0x8db5e18, SUC_lexicon_tags_45 },
    { 0x5c7b5e1b, SUC_lexicon_tags_7 },
    { 0x1a5b5e18, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x403b5e1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4bbb5e20, SUC_lexicon_tags_47 },
    { 0x9c5b5e21, SUC_lexicon_tags_23 },
    { 0x7bdb5e22, SUC_lexicon_tags_1 },
    { 0x491b5e21, SUC_lexicon_tags_23 },
    { 0x7ffb5e20, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03b5e28, SUC_lexicon_tags_15 },
    { 0xfa3b5e28, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803b5e34, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1b5e3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa93b5e3c, SUC_lexicon_tags_12 },
    { 0xd71b5e3d, SUC_lexicon_tags_43 },
    { 0xf03b5e3d, SUC_lexicon_tags_13 },
    { 0xaf1b5e3f, SUC_lexicon_tags_14 },
    { 0xa25b5e3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd85b5e42, SUC_lexicon_tags_7 },
    { 0x3dbb5e43, SUC_lexicon_tags_15 },
    { 0x399b5e44, SUC_lexicon_tags_77 },
    { 0xcd7b5e44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7b5e4d, SUC_lexicon_tags_1 },
    { 0x3fb5e4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b9b5e50, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bb5e53, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x791b5e55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307b5e59, SUC_lexicon_tags_7 },
    { 0x57bb5e5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9db5e5f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5b5e66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1b5e6a, SUC_lexicon_tags_12 },
    { 0x219b5e6a, SUC_lexicon_tags_36 },
    { 0x6cdb5e6c, SUC_lexicon_tags_47 },
    { 0xc85b5e6a, SUC_lexicon_tags_25 },
    { 0x27fb5e6a, SUC_lexicon_tags_29 },
    { 0x1c1b5e6f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x83fb5e71, SUC_lexicon_tags_54 },
    { 0xca9b5e71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bb5e76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x869b5e78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db5e7f, SUC_lexicon_tags_68 },
    { 0xdfdb5e80, SUC_lexicon_tags_7 },
    { 0x16fb5e81, SUC_lexicon_tags_25 },
    { 0xb33b5e82, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdb5e8a, SUC_lexicon_tags_23 },
    { 0x8c3b5e8b, SUC_lexicon_tags_57 },
    { 0x931b5e8c, SUC_lexicon_tags_37 },
    { 0x7f3b5e8d, SUC_lexicon_tags_7 },
    { 0x85fb5e8e, SUC_lexicon_tags_37 },
    { 0x3c1b5e8f, SUC_lexicon_tags_87 },
    { 0x9b1b5e8e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x97bb5e92, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbb5e95, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fb5e98, SUC_lexicon_tags_17 },
    { 0x8efb5e98, SUC_lexicon_tags_2 },
    { 0x8fbb5e99, SUC_lexicon_tags_28 },
    { 0x12fb5e98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82db5e9f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913b5ea3, SUC_lexicon_tags_17 },
    { 0x471b5ea4, SUC_lexicon_tags_37 },
    { 0xb87b5ea5, SUC_lexicon_tags_1 },
    { 0xa9bb5ea6, SUC_lexicon_tags_17 },
    { 0x769b5ea4, SUC_lexicon_tags_25 },
    { 0xc1bb5ea8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1b5eac, SUC_lexicon_tags_41 },
    { 0x8b7b5eac, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfb5eb0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x21bb5eb2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddb5eb6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989b5eba, SUC_lexicon_tags_37 },
    { 0x7dbb5eba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30db5ec1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fb5ec4, SUC_lexicon_tags_72 },
    { 0xca3b5ec5, SUC_lexicon_tags_8 },
    { 0x727b5ec5, SUC_lexicon_tags_6 },
    { 0xcbb5ec5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x80bb5ec9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fb5ecc, SUC_lexicon_tags_37 },
    { 0x8d1b5ecc, SUC_lexicon_tags_2 },
    { 0x923b5ecd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x375b5ed0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71b5ed4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63b5ed7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bb5eda, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x33fb5edc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5a9b5ede, SUC_lexicon_tags_40 },
    { 0x92db5edf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94db5ee2, SUC_lexicon_tags_66 },
    { 0xb41b5ee3, SUC_lexicon_tags_30 },
    { 0x50db5ee2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7b5ee9, SUC_lexicon_tags_30 },
    { 0xe95b5ee9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfb5ef0, SUC_lexicon_tags_49 },
    { 0xff1b5ef1, SUC_lexicon_tags_11 },
    { 0xfb3b5ef1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7db5ef5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7b5efa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37b5eff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa79b5f01, SUC_lexicon_tags_27 },
    { 0xf35b5f02, SUC_lexicon_tags_229 },
    { 0x873b5f03, SUC_lexicon_tags_30 },
    { 0xaf5b5f04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb5f08, SUC_lexicon_tags_144 },
    { 0x8d1b5f09, SUC_lexicon_tags_25 },
    { 0x4b5b5f09, SUC_lexicon_tags_55 },
    { 0x543b5f0b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71b5f11, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fb5f16, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x29fb5f18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bb5f21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5b5f25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x443b5f28, SUC_lexicon_tags_37 },
    { 0x997b5f29, SUC_lexicon_tags_4 },
    { 0x46db5f28, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fb5f2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209b5f33, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89b5f36, SUC_lexicon_tags_0 },
    { 0x65db5f37, SUC_lexicon_tags_38 },
    { 0xb8db5f37, SUC_lexicon_tags_7 },
    { 0xd5bb5f39, SUC_lexicon_tags_17 },
    { 0x71db5f39, SUC_lexicon_tags_79 },
    { 0x15bb5f39, SUC_lexicon_tags_17 },
    { 0x54db5f3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdbdb5f3e, SUC_lexicon_tags_17 },
    { 0x27fb5f3f, SUC_lexicon_tags_30 },
    { 0xe4bb5f3f, SUC_lexicon_tags_28 },
    { 0x25fb5f41, SUC_lexicon_tags_2 },
    { 0x311b5f42, SUC_lexicon_tags_12 },
    { 0x48db5f42, SUC_lexicon_tags_14 },
    { 0xcabb5f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5fbb5f46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95b5f4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x79db5f4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bb5f51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7f9b5f53, SUC_lexicon_tags_25 },
    { 0xc8bb5f54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921b5f59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe71b5f5b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1b5f5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbb5f63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x83b5f65, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x889b5f68, SUC_lexicon_tags_17 },
    { 0x885b5f69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59b5f70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x579b5f72, SUC_lexicon_tags_38 },
    { 0x549b5f72, SUC_lexicon_tags_7 },
    { 0x5d3b5f74, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbb5f7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc0db5f80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3b5f84, SUC_lexicon_tags_26 },
    { 0xbe3b5f85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x72db5f87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x639b5f89, SUC_lexicon_tags_6 },
    { 0x855b5f8a, SUC_lexicon_tags_35 },
    { 0xa23b5f8a, SUC_lexicon_tags_14 },
    { 0xeb9b5f8a, SUC_lexicon_tags_49 },
    { 0x77bb5f89, SUC_lexicon_tags_80 },
    { 0xaa9b5f8c, SUC_lexicon_tags_1 },
    { 0x1db5f8f, SUC_lexicon_tags_12 },
    { 0x873b5f8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5db5f92, SUC_lexicon_tags_15 },
    { 0x3a9b5f93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9b5f9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7c9b5f9c, SUC_lexicon_tags_23 },
    { 0x167b5f9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1b5fa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1db5fa8, SUC_lexicon_tags_25 },
    { 0x99b5fa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629b5fae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffb5fb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e5b5fb7, SUC_lexicon_tags_2 },
    { 0x47b5fb7, SUC_lexicon_tags_17 },
    { 0xb59b5fb9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99b5fbf, SUC_lexicon_tags_37 },
    { 0x629b5fc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8d7b5fc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcfb5fc4, SUC_lexicon_tags_43 },
    { 0xc99b5fc5, SUC_lexicon_tags_31 },
    { 0x6e1b5fc6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf5b5fc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f7b5fca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9b5fcf, SUC_lexicon_tags_45 },
    { 0xfcfb5fd0, SUC_lexicon_tags_30 },
    { 0xb71b5fd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bb5fd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55b5fd7, SUC_lexicon_tags_27 },
    { 0x5edb5fd7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89b5fdd, SUC_lexicon_tags_17 },
    { 0x875b5fde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbadb5fe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x557b5fe3, SUC_lexicon_tags_17 },
    { 0xb6fb5fe4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fb5fe8, SUC_lexicon_tags_44 },
    { 0x9c9b5fe9, SUC_lexicon_tags_25 },
    { 0x40bb5fe9, SUC_lexicon_tags_12 },
    { 0xdbfb5feb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x55bb5fed, SUC_lexicon_tags_323 },
    { 0x6db5fee, SUC_lexicon_tags_10 },
    { 0x143b5fed, SUC_lexicon_tags_15 },
    { 0x89b5ff0, SUC_lexicon_tags_60 },
    { 0xc47b5ff0, SUC_lexicon_tags_23 },
    { 0x13fb5ff2, SUC_lexicon_tags_25 },
    { 0xd01b5ff3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xd35b5ff5, SUC_lexicon_tags_0 },
    { 0x1d9b5ff6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38db5ff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7db6006, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bb600a, SUC_lexicon_tags_11 },
    { 0x903b600b, SUC_lexicon_tags_56 },
    { 0xd71b600c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9b6016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b601a, SUC_lexicon_tags_23 },
    { 0x2adb601b, SUC_lexicon_tags_26 },
    { 0x719b601c, SUC_lexicon_tags_33 },
    { 0xf41b601d, SUC_lexicon_tags_32 },
    { 0x49bb601d, SUC_lexicon_tags_32 },
    { 0xbc1b601f, SUC_lexicon_tags_8 },
    { 0x2b3b601a, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x8c3b6022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955b6027, SUC_lexicon_tags_23 },
    { 0x375b6028, SUC_lexicon_tags_19 },
    { 0xc8fb6029, SUC_lexicon_tags_118 },
    { 0x757b6027, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc7b602c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac7b602e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfb6033, SUC_lexicon_tags_45 },
    { 0x919b6034, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbfb6036, SUC_lexicon_tags_15 },
    { 0x333b6037, SUC_lexicon_tags_47 },
    { 0xc49b6036, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa3b603a, SUC_lexicon_tags_12 },
    { 0xc75b603b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x67db603e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfb6041, SUC_lexicon_tags_15 },
    { 0xabfb6042, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49b6047, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bb604a, SUC_lexicon_tags_7 },
    { 0xf8bb604b, SUC_lexicon_tags_34 },
    { 0xf25b604a, SUC_lexicon_tags_37 },
    { 0x35fb604a, SUC_lexicon_tags_84 },
    { 0xc7b604d, SUC_lexicon_tags_25 },
    { 0x7e5b604f, SUC_lexicon_tags_12 },
    { 0x38bb604c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47b6054, SUC_lexicon_tags_12 },
    { 0x953b6054, SUC_lexicon_tags_25 },
    { 0x59db6056, SUC_lexicon_tags_6 },
    { 0xd1fb6057, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41b605b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41db605d, SUC_lexicon_tags_23 },
    { 0x7f7b605e, SUC_lexicon_tags_9 },
    { 0xd0bb605f, SUC_lexicon_tags_1 },
    { 0x3c5b6060, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x443b6063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577b6067, SUC_lexicon_tags_16 },
    { 0x4f1b6068, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3b606c, SUC_lexicon_tags_3 },
    { 0x947b606d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x527b6070, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35b607a, SUC_lexicon_tags_6 },
    { 0x9e1b607b, SUC_lexicon_tags_5 },
    { 0x97bb607b, SUC_lexicon_tags_23 },
    { 0x361b607a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe6fb607f, SUC_lexicon_tags_97 },
    { 0x639b6080, SUC_lexicon_tags_14 },
    { 0x847b6081, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3b608e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7b6092, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x283b6094, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3b609d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09b60a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7b60a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63b60a8, SUC_lexicon_tags_17 },
    { 0xd8db60a8, SUC_lexicon_tags_12 },
    { 0x7cbb60aa, SUC_lexicon_tags_25 },
    { 0x475b60ab, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b60b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129b60b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88bb60ba, SUC_lexicon_tags_35 },
    { 0x997b60ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x74db60bd, SUC_lexicon_tags_25 },
    { 0xc37b60bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19b60c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd99b60c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x53fb60c5, SUC_lexicon_tags_6 },
    { 0xb41b60c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3b60ca, SUC_lexicon_tags_12 },
    { 0x607b60cb, SUC_lexicon_tags_1 },
    { 0xc47b60cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1b1b60ce, SUC_lexicon_tags_0 },
    { 0xd9db60ce, SUC_lexicon_tags_37 },
    { 0xe33b60d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5b60d4, SUC_lexicon_tags_21 },
    { 0xe11b60d5, SUC_lexicon_tags_17 },
    { 0x45bb60d4, SUC_lexicon_tags_33 },
    { 0xacfb60d6, SUC_lexicon_tags_19 },
    { 0x791b60d8, SUC_lexicon_tags_16 },
    { 0xfc9b60d8, SUC_lexicon_tags_23 },
    { 0x823b60d5, SUC_lexicon_tags_17 },
    { 0x617b60d6, SUC_lexicon_tags_23 },
    { 0x721b60dc, SUC_lexicon_tags_7 },
    { 0xf8fb60dd, SUC_lexicon_tags_2 },
    { 0xa8fb60de, SUC_lexicon_tags_1 },
    { 0x495b60dc, SUC_lexicon_tags_29 },
    { 0xa2fb60d5, SUC_lexicon_tags_55 },
    { 0x6c7b60e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1b60e5, SUC_lexicon_tags_29 },
    { 0x1b5b60e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331b60ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09b60ee, SUC_lexicon_tags_65 },
    { 0xa63b60ef, SUC_lexicon_tags_0 },
    { 0xb15b60ef, SUC_lexicon_tags_32 },
    { 0xb61b60f0, SUC_lexicon_tags_17 },
    { 0x4bfb60ee, SUC_lexicon_tags_32 },
    { 0x627b60ee, SUC_lexicon_tags_14 },
    { 0xb07b60f4, SUC_lexicon_tags_7 },
    { 0x5ffb60f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa63b60f7, SUC_lexicon_tags_1 },
    { 0x91b60f8, SUC_lexicon_tags_63 },
    { 0xfbdb60f9, SUC_lexicon_tags_12 },
    { 0x37db60fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x32db60fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdffb60ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fb6106, SUC_lexicon_tags_29 },
    { 0x831b6107, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bb610a, SUC_lexicon_tags_7 },
    { 0xadb610b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdb610e, SUC_lexicon_tags_15 },
    { 0xcf9b610e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb2bb6111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17b6114, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95b611a, SUC_lexicon_tags_3 },
    { 0xd7b611a, SUC_lexicon_tags_37 },
    { 0xfb1b611a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9b611f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1b6123, SUC_lexicon_tags_55 },
    { 0x39b6123, SUC_lexicon_tags_63 },
    { 0xc8db6125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8db6127, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621b612e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fb6132, SUC_lexicon_tags_45 },
    { 0xaadb6133, SUC_lexicon_tags_47 },
    { 0xdcdb6132, SUC_lexicon_tags_43 },
    { 0x751b6135, SUC_lexicon_tags_43 },
    { 0xa71b6133, SUC_lexicon_tags_37 },
    { 0x15db6134, SUC_lexicon_tags_14 },
    { 0x22bb6138, SUC_lexicon_tags_43 },
    { 0xa5bb6136, SUC_lexicon_tags_178 },
    { 0x6d3b613a, SUC_lexicon_tags_47 },
    { 0x955b613b, SUC_lexicon_tags_31 },
    { 0x7d7b613b, SUC_lexicon_tags_0 },
    { 0xd6db6137, SUC_lexicon_tags_7 },
    { 0x4cfb613e, SUC_lexicon_tags_31 },
    { 0x7c1b6139, SUC_lexicon_tags_47 },
    { 0xca5b613b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7b6144, SUC_lexicon_tags_19 },
    { 0xad5b6145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bb6149, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3db614b, SUC_lexicon_tags_19 },
    { 0xc8bb614c, SUC_lexicon_tags_7 },
    { 0x5c9b614c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x817b6150, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705b6156, SUC_lexicon_tags_30 },
    { 0xe91b6157, SUC_lexicon_tags_27 },
    { 0x5f5b6156, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bb6161, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbb7b6163, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43b6166, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x279b6169, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63b616c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5dfb616e, SUC_lexicon_tags_172 },
    { 0x33db616f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85b6174, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x90fb6176, SUC_lexicon_tags_15 },
    { 0x76fb6177, SUC_lexicon_tags_1 },
    { 0xad3b6178, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe69b617a, SUC_lexicon_tags_17 },
    { 0x517b617b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xea3b617d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x409b6180, SUC_lexicon_tags_11 },
    { 0x933b6180, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abb6187, SUC_lexicon_tags_66 },
    { 0xa5db6188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbb618c, SUC_lexicon_tags_7 },
    { 0x2dfb618c, SUC_lexicon_tags_46 },
    { 0x2e7b618e, SUC_lexicon_tags_11 },
    { 0x391b618d, SUC_lexicon_tags_33 },
    { 0x347b6190, SUC_lexicon_tags_11 },
    { 0xafb6191, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x353b6193, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79b6199, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x67db619c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17db61a0, SUC_lexicon_tags_7 },
    { 0x133b61a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7b61a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x147b61a8, SUC_lexicon_tags_14 },
    { 0xf51b61a9, SUC_lexicon_tags_19 },
    { 0xbcbb61aa, SUC_lexicon_tags_10 },
    { 0xe35b61ab, SUC_lexicon_tags_69 },
    { 0x42fb61ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271b61b0, SUC_lexicon_tags_16 },
    { 0x97b61b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fb61b5, SUC_lexicon_tags_17 },
    { 0x70fb61b5, SUC_lexicon_tags_19 },
    { 0xe4fb61b7, SUC_lexicon_tags_17 },
    { 0xa35b61b7, SUC_lexicon_tags_8 },
    { 0xf4db61b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1b9b61bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fb61be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695b61c2, SUC_lexicon_tags_2 },
    { 0xd9bb61c3, SUC_lexicon_tags_8 },
    { 0x1edb61c3, SUC_lexicon_tags_32 },
    { 0xfd5b61c4, SUC_lexicon_tags_23 },
    { 0x21fb61c6, SUC_lexicon_tags_23 },
    { 0xe59b61c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1db61cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253b61d5, SUC_lexicon_tags_30 },
    { 0x6cbb61d6, SUC_lexicon_tags_11 },
    { 0xa9bb61d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5b61dc, SUC_lexicon_tags_2 },
    { 0xd29b61dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x53db61df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807b61e3, SUC_lexicon_tags_7 },
    { 0xe03b61e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fb61ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb61f2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9b61ff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x557b6201, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bb6207, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fb620a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889b6213, SUC_lexicon_tags_25 },
    { 0x4d5b6214, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x409b6216, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fb621b, SUC_lexicon_tags_11 },
    { 0x60fb621c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x151b621e, SUC_lexicon_tags_174 },
    { 0xce3b621f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe79b6221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x155b6224, SUC_lexicon_tags_38 },
    { 0xbb1b6225, SUC_lexicon_tags_49 },
    { 0x7f1b6226, SUC_lexicon_tags_11 },
    { 0xb1fb6226, SUC_lexicon_tags_35 },
    { 0xd49b6228, SUC_lexicon_tags_19 },
    { 0xf6fb6226, SUC_lexicon_tags_13 },
    { 0xa1fb6229, SUC_lexicon_tags_19 },
    { 0x2d5b6229, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb6235, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbb6238, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fb623c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8cfb623e, SUC_lexicon_tags_17 },
    { 0xb51b623e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71b6244, SUC_lexicon_tags_37 },
    { 0x849b6245, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17b624d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb6251, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29db6253, SUC_lexicon_tags_27 },
    { 0x6e3b6254, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebb625a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x343b625c, SUC_lexicon_tags_6 },
    { 0x6e7b625d, SUC_lexicon_tags_235 },
    { 0x5d5b625d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5b6261, SUC_lexicon_tags_46 },
    { 0xe7bb6262, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x29b6265, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafb6268, SUC_lexicon_tags_48 },
    { 0x953b6268, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6bbb626b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755b6270, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0x6cdb6272, SUC_lexicon_tags_23 },
    { 0xee3b6273, SUC_lexicon_tags_7 },
    { 0xfc1b6274, SUC_lexicon_tags_25 },
    { 0x5e3b6273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367b627b, SUC_lexicon_tags_70 },
    { 0x227b627b, SUC_lexicon_tags_46 },
    { 0xc7db627d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3b6281, SUC_lexicon_tags_11 },
    { 0xaebb6282, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd85b6284, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9b6288, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbb6292, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33b6298, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6fdb629a, SUC_lexicon_tags_3 },
    { 0xf8db629b, SUC_lexicon_tags_45 },
    { 0x9e3b629a, SUC_lexicon_tags_21 },
    { 0xfa7b629a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5b62a0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fb62a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81db62a8, SUC_lexicon_tags_27 },
    { 0x519b62a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4bb62ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abb62ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bb62b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903b62ba, SUC_lexicon_tags_11 },
    { 0xae3b62bb, SUC_lexicon_tags_11 },
    { 0x961b62bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x793b62be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b62c2, SUC_lexicon_tags_7 },
    { 0x7bfb62c2, SUC_lexicon_tags_35 },
    { 0x433b62c4, SUC_lexicon_tags_49 },
    { 0xae5b62c2, SUC_lexicon_tags_12 },
    { 0xde9b62c5, SUC_lexicon_tags_8 },
    { 0x42bb62c7, SUC_lexicon_tags_47 },
    { 0x8bbb62c2, SUC_lexicon_tags_15 },
    { 0xc49b62c4, SUC_lexicon_tags_2 },
    { 0x341b62ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7b62ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07b62d1, SUC_lexicon_tags_17 },
    { 0x3a5b62d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bb62d8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fb62dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7b62df, SUC_lexicon_tags_49 },
    { 0x3ebb62e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3b62e4, SUC_lexicon_tags_79 },
    { 0x7b3b62e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bb62ec, SUC_lexicon_tags_12 },
    { 0x82db62ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327b62f1, SUC_lexicon_tags_49 },
    { 0xbbbb62f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b62f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55b62fc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3b6303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39db6308, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42db6311, SUC_lexicon_tags_12 },
    { 0xd79b6311, SUC_lexicon_tags_87 },
    { 0x17b6312, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07b6316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01b631a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x691b631d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b6321, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1b6324, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7b6330, SUC_lexicon_tags_37 },
    { 0x899b6330, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x38db6334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471b633d, SUC_lexicon_tags_9 },
    { 0xeb9b633e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20bb6340, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x9b6342, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x56bb6345, SUC_lexicon_tags_15 },
    { 0x7e1b6346, SUC_lexicon_tags_19 },
    { 0x329b6346, SUC_lexicon_tags_2 },
    { 0xbb6347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b634d, SUC_lexicon_tags_2 },
    { 0x51db634d, SUC_lexicon_tags_14 },
    { 0xdf1b634f, SUC_lexicon_tags_37 },
    { 0x541b6350, SUC_lexicon_tags_47 },
    { 0xc5fb6350, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7bb6353, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcabb6355, SUC_lexicon_tags_7 },
    { 0xec3b6355, SUC_lexicon_tags_19 },
    { 0xd41b6355, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x165b635a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbb635e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa07b6360, SUC_lexicon_tags_0 },
    { 0x229b6361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebb6367, SUC_lexicon_tags_1 },
    { 0xdf3b6368, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9c3b636a, SUC_lexicon_tags_61 },
    { 0x329b636a, SUC_lexicon_tags_17 },
    { 0x5a3b636a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf65b636f, SUC_lexicon_tags_25 },
    { 0x579b636f, SUC_lexicon_tags_9 },
    { 0xe17b6371, SUC_lexicon_tags_15 },
    { 0xdedb6372, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed5b6374, SUC_lexicon_tags_25 },
    { 0x97fb6375, SUC_lexicon_tags_8 },
    { 0xaa1b6376, SUC_lexicon_tags_7 },
    { 0x3e1b6377, SUC_lexicon_tags_19 },
    { 0x7c5b6378, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabdb637a, SUC_lexicon_tags_25 },
    { 0x93bb637b, SUC_lexicon_tags_31 },
    { 0x7b3b637a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1b6380, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x73db6382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x145b6385, SUC_lexicon_tags_16 },
    { 0x87db6386, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193b638f, SUC_lexicon_tags_3 },
    { 0xb43b6390, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6efb6392, SUC_lexicon_tags_17 },
    { 0x89db6393, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71b6398, SUC_lexicon_tags_151 },
    { 0x98fb6398, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3b639d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x339b63a0, SUC_lexicon_tags_11 },
    { 0x227b63a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaedb63a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97fb63a9, SUC_lexicon_tags_46 },
    { 0x8bfb63a9, SUC_lexicon_tags_12 },
    { 0x53b63ab, SUC_lexicon_tags_33 },
    { 0x6b3b63ac, SUC_lexicon_tags_6 },
    { 0x989b63ad, SUC_lexicon_tags_23 },
    { 0xbffb63ae, SUC_lexicon_tags_37 },
    { 0xff3b63ad, SUC_lexicon_tags_0 },
    { 0x445b63ab, SUC_lexicon_tags_12 },
    { 0xaf9b63ab, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x16fb63b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87b63be, SUC_lexicon_tags_11 },
    { 0x333b63bf, SUC_lexicon_tags_25 },
    { 0xba3b63bf, SUC_lexicon_tags_33 },
    { 0xbcfb63bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafb63c4, SUC_lexicon_tags_38 },
    { 0x8f5b63c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3b63c8, SUC_lexicon_tags_25 },
    { 0x12fb63c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x387b63cb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa35b63cd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafb63d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6e3b63d2, SUC_lexicon_tags_60 },
    { 0xa93b63d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9b63d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1b63da, SUC_lexicon_tags_15 },
    { 0x2e5b63db, SUC_lexicon_tags_12 },
    { 0xf6db63dc, SUC_lexicon_tags_15 },
    { 0xed7b63dd, SUC_lexicon_tags_60 },
    { 0x3ffb63dd, SUC_lexicon_tags_1 },
    { 0x3edb63df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffb63e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5b63e7, SUC_lexicon_tags_1 },
    { 0x321b63e8, SUC_lexicon_tags_12 },
    { 0x35bb63e9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x191b63eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf77b63ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbb63f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89b63f4, SUC_lexicon_tags_11 },
    { 0x557b63f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb63fa, SUC_lexicon_tags_9 },
    { 0x7a7b63fb, SUC_lexicon_tags_33 },
    { 0xcb1b63fc, SUC_lexicon_tags_6 },
    { 0x239b63fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xefb63ff, SUC_lexicon_tags_7 },
    { 0x30fb6400, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3b6404, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc05b6406, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73b640b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x181b640e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497b641b, SUC_lexicon_tags_28 },
    { 0x2efb641c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289b6420, SUC_lexicon_tags_9 },
    { 0x62bb6420, SUC_lexicon_tags_60 },
    { 0xb37b6422, SUC_lexicon_tags_37 },
    { 0xcc9b6423, SUC_lexicon_tags_23 },
    { 0xf7db6421, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3bdb6426, SUC_lexicon_tags_68 },
    { 0xcffb6426, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x841b6429, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfb642d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x615b6430, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddb6435, SUC_lexicon_tags_57 },
    { 0xb15b6435, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb643a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x601b643d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689b6444, SUC_lexicon_tags_47 },
    { 0x37b6445, SUC_lexicon_tags_11 },
    { 0x587b6445, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435b644b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975b644f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x499b6451, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179b6455, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1b6459, SUC_lexicon_tags_80 },
    { 0xfa1b6459, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x425b645d, SUC_lexicon_tags_1 },
    { 0x429b645e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebb6463, SUC_lexicon_tags_29 },
    { 0xd3db6463, SUC_lexicon_tags_7 },
    { 0x25b6464, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5b6469, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69b646e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xec9b6470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bb6473, SUC_lexicon_tags_29 },
    { 0xf85b6474, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3c5b6476, SUC_lexicon_tags_37 },
    { 0xbefb6477, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bb647b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7b6484, SUC_lexicon_tags_39 },
    { 0x2b3b6485, SUC_lexicon_tags_6 },
    { 0x7e7b6484, SUC_lexicon_tags_55 },
    { 0xabfb6485, SUC_lexicon_tags_37 },
    { 0x9adb6488, SUC_lexicon_tags_63 },
    { 0x415b6489, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa55b648b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bb648f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7b6492, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73db6498, SUC_lexicon_tags_37 },
    { 0xfabb6499, SUC_lexicon_tags_1 },
    { 0xfc1b6499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65b64a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b5b64a2, SUC_lexicon_tags_46 },
    { 0xe7b64a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9b64a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bb64ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fb64b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bb64b5, SUC_lexicon_tags_11 },
    { 0x673b64b6, SUC_lexicon_tags_27 },
    { 0x797b64b6, SUC_lexicon_tags_11 },
    { 0x73fb64b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb53b64ba, SUC_lexicon_tags_2 },
    { 0xa13b64ba, SUC_lexicon_tags_37 },
    { 0x37db64bc, SUC_lexicon_tags_12 },
    { 0x91fb64ba, SUC_lexicon_tags_13 },
    { 0x31fb64be, SUC_lexicon_tags_17 },
    { 0xe4db64bb, SUC_lexicon_tags_35 },
    { 0x827b64c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb0db64c2, SUC_lexicon_tags_31 },
    { 0xb25b64c3, SUC_lexicon_tags_38 },
    { 0x2db64c3, SUC_lexicon_tags_37 },
    { 0xfafb64c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbb64cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfb64d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3b64d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bb64d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41b64d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fb64e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687b64e5, SUC_lexicon_tags_31 },
    { 0x651b64e6, SUC_lexicon_tags_25 },
    { 0x6abb64e7, SUC_lexicon_tags_27 },
    { 0xe13b64e8, SUC_lexicon_tags_2 },
    { 0xf73b64e9, SUC_lexicon_tags_9 },
    { 0x835b64e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1b64ed, SUC_lexicon_tags_6 },
    { 0x1cbb64ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8e1b64f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x11fb64f2, SUC_lexicon_tags_9 },
    { 0xb5b64f2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7b64fd, SUC_lexicon_tags_34 },
    { 0xa8bb64fe, SUC_lexicon_tags_22 },
    { 0x9c5b64fe, SUC_lexicon_tags_25 },
    { 0xee7b6500, SUC_lexicon_tags_19 },
    { 0x9bb6501, SUC_lexicon_tags_11 },
    { 0xd07b6500, SUC_lexicon_tags_37 },
    { 0x80fb64ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x33db6506, SUC_lexicon_tags_63 },
    { 0xcf9b6507, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9b650b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1b6510, SUC_lexicon_tags_49 },
    { 0x439b6510, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x42db6514, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc61b6516, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf81b6518, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x501b651b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b651e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc07b6520, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867b6524, SUC_lexicon_tags_17 },
    { 0x383b6525, SUC_lexicon_tags_69 },
    { 0x419b6524, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fb652a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96bb652d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xedbb652f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcedb6531, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fb6536, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63b653d, SUC_lexicon_tags_1 },
    { 0x549b653d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155b6544, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2edb6546, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133b654d, SUC_lexicon_tags_38 },
    { 0xca1b654e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd67b6550, SUC_lexicon_tags_55 },
    { 0xfe3b6550, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963b6556, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203b655c, SUC_lexicon_tags_9 },
    { 0x381b655c, SUC_lexicon_tags_0 },
    { 0x547b655e, SUC_lexicon_tags_9 },
    { 0xb7b655f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8bbb6561, SUC_lexicon_tags_44 },
    { 0xa19b6562, SUC_lexicon_tags_30 },
    { 0xfb3b6563, SUC_lexicon_tags_17 },
    { 0x6a1b6564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29db6566, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311b656a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9b656d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31b6572, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x283b6574, SUC_lexicon_tags_13 },
    { 0x3afb6575, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e1b6577, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39b657b, SUC_lexicon_tags_11 },
    { 0xc53b657c, SUC_lexicon_tags_38 },
    { 0xbcbb657c, SUC_lexicon_tags_25 },
    { 0xc3b657e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bb6583, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5db6586, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9d7b6588, SUC_lexicon_tags_351 },
    { 0x277b6589, SUC_lexicon_tags_37 },
    { 0xb53b658a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fb658d, SUC_lexicon_tags_31 },
    { 0x7d5b658d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941b6599, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573b659d, SUC_lexicon_tags_1 },
    { 0xcd1b659e, SUC_lexicon_tags_2 },
    { 0xf01b659f, SUC_lexicon_tags_11 },
    { 0xd67b65a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bb65a3, SUC_lexicon_tags_1 },
    { 0xd09b65a4, SUC_lexicon_tags_37 },
    { 0x8cdb65a4, SUC_lexicon_tags_7 },
    { 0xac9b65a6, SUC_lexicon_tags_19 },
    { 0xb4bb65a7, SUC_lexicon_tags_30 },
    { 0x1efb65a8, SUC_lexicon_tags_14 },
    { 0xf3b65a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3b65af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87b65b2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1b65b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbb65b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7b65ba, SUC_lexicon_tags_19 },
    { 0xacbb65bb, SUC_lexicon_tags_20 },
    { 0x2fdb65bc, SUC_lexicon_tags_23 },
    { 0xbedb65ba, SUC_lexicon_tags_25 },
    { 0x5f3b65be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357b65c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf6bb65c7, SUC_lexicon_tags_28 },
    { 0x271b65c8, SUC_lexicon_tags_23 },
    { 0x85b65c9, SUC_lexicon_tags_85 },
    { 0x275b65ca, SUC_lexicon_tags_1 },
    { 0xeb1b65ca, SUC_lexicon_tags_84 },
    { 0xdbdb65cc, SUC_lexicon_tags_37 },
    { 0x8c1b65cd, SUC_lexicon_tags_9 },
    { 0xf03b65ce, SUC_lexicon_tags_25 },
    { 0xa33b65cf, SUC_lexicon_tags_27 },
    { 0x4d1b65c8, SUC_lexicon_tags_1669 },
    { 0, NULL },
    { 0x899b65d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31b65dc, SUC_lexicon_tags_11 },
    { 0x955b65dd, SUC_lexicon_tags_7 },
    { 0xa9db65dd, SUC_lexicon_tags_11 },
    { 0xd17b65dd, SUC_lexicon_tags_1 },
    { 0xf63b65de, SUC_lexicon_tags_60 },
    { 0x7b7b65e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2d7b65e3, SUC_lexicon_tags_77 },
    { 0xdadb65e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463b65e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163b65ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabb65f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7b65f7, SUC_lexicon_tags_28 },
    { 0x661b65f8, SUC_lexicon_tags_56 },
    { 0x2a7b65f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x279b65fc, SUC_lexicon_tags_13 },
    { 0x80fb65fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893b6601, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8e3b6603, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91b6605, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3b6609, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x351b660b, SUC_lexicon_tags_17 },
    { 0x905b660b, SUC_lexicon_tags_25 },
    { 0x731b660d, SUC_lexicon_tags_34 },
    { 0x629b660e, SUC_lexicon_tags_12 },
    { 0x739b660e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f1b6611, SUC_lexicon_tags_27 },
    { 0xdfbb6612, SUC_lexicon_tags_45 },
    { 0xaefb6613, SUC_lexicon_tags_0 },
    { 0x14db6613, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfb6618, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb5b661a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbb661e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x75fb6620, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadb6623, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfb6627, SUC_lexicon_tags_37 },
    { 0xa91b6628, SUC_lexicon_tags_17 },
    { 0xaa1b6629, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5b662d, SUC_lexicon_tags_2 },
    { 0xbe3b662d, SUC_lexicon_tags_2 },
    { 0x30db662d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49b6631, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253b6638, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53b6641, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x187b6643, SUC_lexicon_tags_34 },
    { 0xf67b6644, SUC_lexicon_tags_6 },
    { 0xfe5b6643, SUC_lexicon_tags_17 },
    { 0x519b6645, SUC_lexicon_tags_37 },
    { 0xcf1b6647, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1b664a, SUC_lexicon_tags_9 },
    { 0x7b1b664a, SUC_lexicon_tags_65 },
    { 0xac9b664c, SUC_lexicon_tags_10 },
    { 0xdc7b664c, SUC_lexicon_tags_34 },
    { 0x8edb664e, SUC_lexicon_tags_45 },
    { 0xf9fb664e, SUC_lexicon_tags_148 },
    { 0xe4fb6650, SUC_lexicon_tags_23 },
    { 0xca9b6651, SUC_lexicon_tags_47 },
    { 0x5b3b6652, SUC_lexicon_tags_39 },
    { 0xe9fb6651, SUC_lexicon_tags_79 },
    { 0x3bbb6651, SUC_lexicon_tags_8 },
    { 0x333b6654, SUC_lexicon_tags_25 },
    { 0xa9db664e, SUC_lexicon_tags_23 },
    { 0x9d1b6657, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x627b665a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bb665e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2e9b6660, SUC_lexicon_tags_37 },
    { 0xd55b6661, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7b6664, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3b6667, SUC_lexicon_tags_178 },
    { 0x973b6668, SUC_lexicon_tags_37 },
    { 0x8a5b6669, SUC_lexicon_tags_33 },
    { 0xa0db6668, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddb666f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x915b6671, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41b6677, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xde9b6679, SUC_lexicon_tags_29 },
    { 0x83bb667a, SUC_lexicon_tags_60 },
    { 0xbc9b667b, SUC_lexicon_tags_47 },
    { 0x40fb667a, SUC_lexicon_tags_12 },
    { 0x613b667d, SUC_lexicon_tags_17 },
    { 0x687b667d, SUC_lexicon_tags_12 },
    { 0x75b667f, SUC_lexicon_tags_28 },
    { 0x137b6680, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3b6687, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09b668c, SUC_lexicon_tags_27 },
    { 0x109b668d, SUC_lexicon_tags_40 },
    { 0xd89b668c, SUC_lexicon_tags_57 },
    { 0xcd5b668f, SUC_lexicon_tags_1 },
    { 0x733b668c, SUC_lexicon_tags_47 },
    { 0x7dfb6691, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fb6694, SUC_lexicon_tags_63 },
    { 0x2bbb6694, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fb6698, SUC_lexicon_tags_37 },
    { 0x205b6699, SUC_lexicon_tags_11 },
    { 0x44bb6698, SUC_lexicon_tags_15 },
    { 0xc59b669b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fb66a0, SUC_lexicon_tags_37 },
    { 0x61b66a1, SUC_lexicon_tags_49 },
    { 0x5d3b66a2, SUC_lexicon_tags_6 },
    { 0x7e1b66a2, SUC_lexicon_tags_63 },
    { 0x309b66a4, SUC_lexicon_tags_25 },
    { 0x84db66a4, SUC_lexicon_tags_27 },
    { 0xd41b66a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fb66a9, SUC_lexicon_tags_15 },
    { 0x6f9b66aa, SUC_lexicon_tags_33 },
    { 0xc91b66aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd59b66ad, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3b66b1, SUC_lexicon_tags_29 },
    { 0xe7bb66b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb66b5, SUC_lexicon_tags_8 },
    { 0xc17b66b5, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51b66ba, SUC_lexicon_tags_116 },
    { 0xf3bb66bb, SUC_lexicon_tags_8 },
    { 0xddb66bc, SUC_lexicon_tags_16 },
    { 0xc63b66bd, SUC_lexicon_tags_21 },
    { 0x477b66ba, SUC_lexicon_tags_158 },
    { 0x881b66bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291b66c6, SUC_lexicon_tags_27 },
    { 0xd3bb66c7, SUC_lexicon_tags_11 },
    { 0x58db66c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b5b66ca, SUC_lexicon_tags_32 },
    { 0xddb66ca, SUC_lexicon_tags_19 },
    { 0x531b66ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab9b66ce, SUC_lexicon_tags_15 },
    { 0xc7bb66cf, SUC_lexicon_tags_60 },
    { 0x545b66ce, SUC_lexicon_tags_12 },
    { 0xfbdb66d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7db66d6, SUC_lexicon_tags_25 },
    { 0x739b66d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddb66e1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x82fb66e3, SUC_lexicon_tags_17 },
    { 0xd6fb66e3, SUC_lexicon_tags_4 },
    { 0x5d1b66e5, SUC_lexicon_tags_7 },
    { 0xa11b66e6, SUC_lexicon_tags_11 },
    { 0x721b66e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33b66e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219b66ed, SUC_lexicon_tags_1 },
    { 0xcf9b66ee, SUC_lexicon_tags_60 },
    { 0xf0bb66ed, SUC_lexicon_tags_37 },
    { 0xb7b66f0, SUC_lexicon_tags_33 },
    { 0xbbbb66f1, SUC_lexicon_tags_37 },
    { 0xbffb66f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95b66f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6c1b66f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5b66ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1b6704, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0x611b6706, SUC_lexicon_tags_28 },
    { 0x6b9b6707, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8db670b, SUC_lexicon_tags_15 },
    { 0x9f5b670b, SUC_lexicon_tags_11 },
    { 0x8ffb670b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e9b670f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x305b6712, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725b6717, SUC_lexicon_tags_2 },
    { 0xc15b6718, SUC_lexicon_tags_37 },
    { 0x79bb6717, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4db671c, SUC_lexicon_tags_15 },
    { 0xe65b671d, SUC_lexicon_tags_7 },
    { 0x40fb671e, SUC_lexicon_tags_61 },
    { 0xf89b671f, SUC_lexicon_tags_82 },
    { 0x1c5b6720, SUC_lexicon_tags_23 },
    { 0x87bb671f, SUC_lexicon_tags_25 },
    { 0xcb9b671d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7b672c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91fb672e, SUC_lexicon_tags_37 },
    { 0x5fdb672f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x377b6732, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3b6735, SUC_lexicon_tags_17 },
    { 0x401b6736, SUC_lexicon_tags_25 },
    { 0x86fb6736, SUC_lexicon_tags_25 },
    { 0x9a5b6736, SUC_lexicon_tags_15 },
    { 0x28fb6736, SUC_lexicon_tags_32 },
    { 0x2edb673a, SUC_lexicon_tags_15 },
    { 0x5fdb673b, SUC_lexicon_tags_1 },
    { 0xe47b673c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fb6740, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305b6745, SUC_lexicon_tags_55 },
    { 0x1d1b6745, SUC_lexicon_tags_4 },
    { 0x515b6746, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedb674c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x229b674e, SUC_lexicon_tags_37 },
    { 0x673b674f, SUC_lexicon_tags_17 },
    { 0x277b6750, SUC_lexicon_tags_25 },
    { 0x93fb6750, SUC_lexicon_tags_38 },
    { 0x85b6752, SUC_lexicon_tags_37 },
    { 0xd27b6752, SUC_lexicon_tags_29 },
    { 0x1e1b6753, SUC_lexicon_tags_1 },
    { 0xd2bb6755, SUC_lexicon_tags_25 },
    { 0x399b6756, SUC_lexicon_tags_6 },
    { 0x709b6755, SUC_lexicon_tags_37 },
    { 0x1fdb6758, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x67bb675a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc7fb675c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x475b675f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3efb6761, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35b6765, SUC_lexicon_tags_43 },
    { 0x70bb6766, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x32fb6769, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5b6773, SUC_lexicon_tags_15 },
    { 0x3b3b6774, SUC_lexicon_tags_11 },
    { 0x9a1b6775, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169b677d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3b6781, SUC_lexicon_tags_47 },
    { 0x3bb6782, SUC_lexicon_tags_61 },
    { 0xba9b6782, SUC_lexicon_tags_10 },
    { 0x18bb6782, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x86fb6787, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb678c, SUC_lexicon_tags_82 },
    { 0xeb7b678d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd9db678f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fb6794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe95b6796, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447b679a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x18db679d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb67a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2db67a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafb67ab, SUC_lexicon_tags_26 },
    { 0x525b67ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf27b67ae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9a5b67b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03b67b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf7fb67b7, SUC_lexicon_tags_15 },
    { 0x9b1b67b8, SUC_lexicon_tags_15 },
    { 0xaedb67b8, SUC_lexicon_tags_55 },
    { 0xc63b67ba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc7db67bc, SUC_lexicon_tags_1 },
    { 0x957b67bc, SUC_lexicon_tags_16 },
    { 0x1b67be, SUC_lexicon_tags_65 },
    { 0x4dbb67be, SUC_lexicon_tags_23 },
    { 0xc63b67bf, SUC_lexicon_tags_1 },
    { 0x3c1b67c1, SUC_lexicon_tags_63 },
    { 0x38fb67c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebb67c5, SUC_lexicon_tags_37 },
    { 0xa9b67c6, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3b67ca, SUC_lexicon_tags_15 },
    { 0xd8fb67ca, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x291b67ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1b67d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd3fb67d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x95b67d5, SUC_lexicon_tags_16 },
    { 0xce7b67d6, SUC_lexicon_tags_17 },
    { 0x633b67d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x937b67da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e1b67dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3b67e0, SUC_lexicon_tags_25 },
    { 0xc79b67e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b67e8, SUC_lexicon_tags_63 },
    { 0x15bb67e9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7db67ed, SUC_lexicon_tags_14 },
    { 0xe3db67ee, SUC_lexicon_tags_84 },
    { 0xe83b67ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9b67f7, SUC_lexicon_tags_104 },
    { 0xc17b67f8, SUC_lexicon_tags_15 },
    { 0x90bb67f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8db67fc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1d3b67fe, SUC_lexicon_tags_7 },
    { 0x2fbb67ff, SUC_lexicon_tags_46 },
    { 0x379b6800, SUC_lexicon_tags_14 },
    { 0x855b6801, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3b6806, SUC_lexicon_tags_2 },
    { 0x5ebb6806, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5bb6809, SUC_lexicon_tags_15 },
    { 0xdcdb680a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1b6810, SUC_lexicon_tags_26 },
    { 0xf93b6811, SUC_lexicon_tags_25 },
    { 0xb53b6812, SUC_lexicon_tags_15 },
    { 0x57bb6813, SUC_lexicon_tags_34 },
    { 0x82db6814, SUC_lexicon_tags_57 },
    { 0x1efb6811, SUC_lexicon_tags_47 },
    { 0x1c1b6813, SUC_lexicon_tags_7 },
    { 0x7a3b6817, SUC_lexicon_tags_1 },
    { 0xd21b6814, SUC_lexicon_tags_33 },
    { 0x34fb6819, SUC_lexicon_tags_40 },
    { 0x6edb6812, SUC_lexicon_tags_47 },
    { 0x715b681b, SUC_lexicon_tags_58 },
    { 0xe65b681c, SUC_lexicon_tags_21 },
    { 0xa51b681b, SUC_lexicon_tags_37 },
    { 0x1cbb681e, SUC_lexicon_tags_82 },
    { 0xc81b681e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673b6823, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fb6826, SUC_lexicon_tags_5 },
    { 0xf65b6826, SUC_lexicon_tags_26 },
    { 0x721b6828, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bb682b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7b6832, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bb6835, SUC_lexicon_tags_172 },
    { 0x339b6836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f5b6838, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11db6842, SUC_lexicon_tags_37 },
    { 0x327b6842, SUC_lexicon_tags_7 },
    { 0x6a9b6844, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939b684a, SUC_lexicon_tags_63 },
    { 0x7b5b684b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83bb684d, SUC_lexicon_tags_37 },
    { 0xdafb684e, SUC_lexicon_tags_37 },
    { 0xff3b684e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf53b6851, SUC_lexicon_tags_26 },
    { 0x3a9b6852, SUC_lexicon_tags_8 },
    { 0x809b6853, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399b685a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbb685f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x93db6862, SUC_lexicon_tags_178 },
    { 0xd47b6863, SUC_lexicon_tags_11 },
    { 0x6efb6864, SUC_lexicon_tags_45 },
    { 0x225b6863, SUC_lexicon_tags_17 },
    { 0x1adb6866, SUC_lexicon_tags_3 },
    { 0xae7b6867, SUC_lexicon_tags_33 },
    { 0xcffb6862, SUC_lexicon_tags_3 },
    { 0x1d5b6863, SUC_lexicon_tags_2 },
    { 0xd2fb6869, SUC_lexicon_tags_2 },
    { 0x3fb6866, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afb686f, SUC_lexicon_tags_11 },
    { 0x89db686f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x9e3b6872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227b6877, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bbb6879, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x225b687c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39b687f, SUC_lexicon_tags_15 },
    { 0xbd3b6880, SUC_lexicon_tags_37 },
    { 0x797b6881, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84db6887, SUC_lexicon_tags_15 },
    { 0x591b6888, SUC_lexicon_tags_18 },
    { 0x5e7b6888, SUC_lexicon_tags_7 },
    { 0x985b6888, SUC_lexicon_tags_16 },
    { 0x301b6887, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb9b688d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85db6894, SUC_lexicon_tags_14 },
    { 0x301b6895, SUC_lexicon_tags_11 },
    { 0x111b6896, SUC_lexicon_tags_70 },
    { 0xb49b6897, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49b68a0, SUC_lexicon_tags_11 },
    { 0xa6fb68a1, SUC_lexicon_tags_1 },
    { 0x249b68a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9b68a5, SUC_lexicon_tags_30 },
    { 0xe2db68a6, SUC_lexicon_tags_12 },
    { 0x77b68a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fb68ab, SUC_lexicon_tags_47 },
    { 0xa35b68ac, SUC_lexicon_tags_11 },
    { 0xf25b68ad, SUC_lexicon_tags_32 },
    { 0x81bb68ab, SUC_lexicon_tags_2 },
    { 0x9adb68af, SUC_lexicon_tags_150 },
    { 0xa55b68b0, SUC_lexicon_tags_14 },
    { 0x2ddb68b0, SUC_lexicon_tags_63 },
    { 0x581b68b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d7b68b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9ebb68b6, SUC_lexicon_tags_25 },
    { 0xb21b68b7, SUC_lexicon_tags_198 },
    { 0x19bb68b8, SUC_lexicon_tags_28 },
    { 0x481b68b8, SUC_lexicon_tags_25 },
    { 0xf0fb68b8, SUC_lexicon_tags_1 },
    { 0x3d5b68b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79b68c2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x16fb68c4, SUC_lexicon_tags_32 },
    { 0xf3db68c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeddb68c8, SUC_lexicon_tags_1 },
    { 0xde7b68c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727b68cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x293b68d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99b68d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fb68de, SUC_lexicon_tags_106 },
    { 0xdebb68df, SUC_lexicon_tags_10 },
    { 0xfc1b68df, SUC_lexicon_tags_17 },
    { 0xbd3b68e1, SUC_lexicon_tags_17 },
    { 0x3afb68e2, SUC_lexicon_tags_146 },
    { 0x429b68e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbb68e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x969b68e8, SUC_lexicon_tags_63 },
    { 0x705b68e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x373b68ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53b68ef, SUC_lexicon_tags_15 },
    { 0xb51b68f0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd7db68f2, SUC_lexicon_tags_27 },
    { 0xf4fb68f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ddb68f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x835b68f7, SUC_lexicon_tags_7 },
    { 0x51b68f8, SUC_lexicon_tags_15 },
    { 0xd1bb68f9, SUC_lexicon_tags_15 },
    { 0xbe1b68f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14bb68fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x837b68ff, SUC_lexicon_tags_12 },
    { 0xcb7b68ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3f1b6902, SUC_lexicon_tags_391 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9b6905, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc81b6907, SUC_lexicon_tags_15 },
    { 0xd1db6907, SUC_lexicon_tags_25 },
    { 0xcc9b6908, SUC_lexicon_tags_21 },
    { 0x4bfb6907, SUC_lexicon_tags_10 },
    { 0xb6fb690b, SUC_lexicon_tags_23 },
    { 0x4e3b690b, SUC_lexicon_tags_16 },
    { 0x3cfb690d, SUC_lexicon_tags_11 },
    { 0xcb9b690e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5b6911, SUC_lexicon_tags_60 },
    { 0xabfb6912, SUC_lexicon_tags_14 },
    { 0x4ffb6913, SUC_lexicon_tags_255 },
    { 0xf55b6911, SUC_lexicon_tags_1 },
    { 0x28db6915, SUC_lexicon_tags_17 },
    { 0x8bfb6916, SUC_lexicon_tags_16 },
    { 0xf9db6915, SUC_lexicon_tags_87 },
    { 0x2c5b6917, SUC_lexicon_tags_12 },
    { 0xfd9b6919, SUC_lexicon_tags_11 },
    { 0xb53b691a, SUC_lexicon_tags_68 },
    { 0x685b691a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5b691e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9b6921, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69b6925, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83bb6927, SUC_lexicon_tags_26 },
    { 0xdc5b6928, SUC_lexicon_tags_37 },
    { 0x6d3b6928, SUC_lexicon_tags_30 },
    { 0x24fb692a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fb692d, SUC_lexicon_tags_15 },
    { 0x9bbb692d, SUC_lexicon_tags_63 },
    { 0xee5b692f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fb6934, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x56bb6937, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2db6939, SUC_lexicon_tags_1 },
    { 0xa1bb693a, SUC_lexicon_tags_0 },
    { 0x4db693b, SUC_lexicon_tags_7 },
    { 0x361b6939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59b693f, SUC_lexicon_tags_61 },
    { 0x7a7b693f, SUC_lexicon_tags_37 },
    { 0xed3b6941, SUC_lexicon_tags_39 },
    { 0xc15b6941, SUC_lexicon_tags_10 },
    { 0x7bb6942, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5b6947, SUC_lexicon_tags_29 },
    { 0x35db6948, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1b694c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9b694f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9b6954, SUC_lexicon_tags_12 },
    { 0x34fb6955, SUC_lexicon_tags_48 },
    { 0x9c5b6956, SUC_lexicon_tags_15 },
    { 0x8e5b6955, SUC_lexicon_tags_14 },
    { 0x169b6958, SUC_lexicon_tags_7 },
    { 0x7a7b6959, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423b695d, SUC_lexicon_tags_38 },
    { 0x57db695e, SUC_lexicon_tags_4 },
    { 0xecbb695f, SUC_lexicon_tags_21 },
    { 0xc27b6960, SUC_lexicon_tags_11 },
    { 0x2bfb6960, SUC_lexicon_tags_27 },
    { 0xccbb6962, SUC_lexicon_tags_0 },
    { 0xdc5b6961, SUC_lexicon_tags_21 },
    { 0xa65b6964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01b6969, SUC_lexicon_tags_60 },
    { 0x841b696a, SUC_lexicon_tags_65 },
    { 0x9bfb696b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc45b696d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871b6972, SUC_lexicon_tags_37 },
    { 0xa2fb6972, SUC_lexicon_tags_98 },
    { 0xf65b6974, SUC_lexicon_tags_25 },
    { 0xf7fb6974, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbb6978, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b697d, SUC_lexicon_tags_26 },
    { 0x75bb697d, SUC_lexicon_tags_16 },
    { 0x907b697f, SUC_lexicon_tags_47 },
    { 0xf25b697d, SUC_lexicon_tags_45 },
    { 0x2a1b6981, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76fb6983, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36db698c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdf5b698e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x749b6990, SUC_lexicon_tags_37 },
    { 0xb8bb6991, SUC_lexicon_tags_47 },
    { 0x97b6992, SUC_lexicon_tags_47 },
    { 0xca7b6992, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xe99b6995, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705b6999, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bb699c, SUC_lexicon_tags_43 },
    { 0x10db699d, SUC_lexicon_tags_46 },
    { 0x3d5b699e, SUC_lexicon_tags_16 },
    { 0xe2db699e, SUC_lexicon_tags_12 },
    { 0x89db699d, SUC_lexicon_tags_39 },
    { 0x489b69a0, SUC_lexicon_tags_40 },
    { 0xdddb699d, SUC_lexicon_tags_2 },
    { 0x97fb69a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x709b69a5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3b69a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7b69ae, SUC_lexicon_tags_21 },
    { 0x587b69ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bb69b2, SUC_lexicon_tags_25 },
    { 0x845b69b2, SUC_lexicon_tags_63 },
    { 0x82bb69b3, SUC_lexicon_tags_87 },
    { 0x301b69b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807b69bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bb69c1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfb69cb, SUC_lexicon_tags_1 },
    { 0x449b69cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x40db69cf, SUC_lexicon_tags_8 },
    { 0x89fb69cf, SUC_lexicon_tags_4 },
    { 0x959b69cf, SUC_lexicon_tags_25 },
    { 0xba7b69d2, SUC_lexicon_tags_45 },
    { 0x6edb69d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9b69d9, SUC_lexicon_tags_35 },
    { 0x27fb69d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5b69de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe27b69e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edb69e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7db69e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x865b69eb, SUC_lexicon_tags_47 },
    { 0x381b69eb, SUC_lexicon_tags_11 },
    { 0x129b69eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725b69f4, SUC_lexicon_tags_66 },
    { 0xe3bb69f5, SUC_lexicon_tags_74 },
    { 0xcdb69f6, SUC_lexicon_tags_47 },
    { 0xe75b69f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3b6a00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7f7b6a02, SUC_lexicon_tags_13 },
    { 0x8b1b6a03, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x759b6a05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fb6a0c, SUC_lexicon_tags_37 },
    { 0x2ffb6a0d, SUC_lexicon_tags_15 },
    { 0xb3b6a0e, SUC_lexicon_tags_12 },
    { 0xee1b6a0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3b6a15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x779b6a18, SUC_lexicon_tags_62 },
    { 0xd09b6a19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc5b6a1b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9b6a1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e5b6a20, SUC_lexicon_tags_31 },
    { 0x531b6a21, SUC_lexicon_tags_2 },
    { 0xf6fb6a22, SUC_lexicon_tags_37 },
    { 0x225b6a23, SUC_lexicon_tags_1 },
    { 0x8bbb6a23, SUC_lexicon_tags_15 },
    { 0x77fb6a25, SUC_lexicon_tags_11 },
    { 0x489b6a26, SUC_lexicon_tags_1 },
    { 0xc0fb6a23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397b6a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99b6a33, SUC_lexicon_tags_45 },
    { 0xfa1b6a34, SUC_lexicon_tags_2 },
    { 0x785b6a34, SUC_lexicon_tags_49 },
    { 0xf45b6a36, SUC_lexicon_tags_2 },
    { 0xf85b6a34, SUC_lexicon_tags_15 },
    { 0xd9db6a38, SUC_lexicon_tags_17 },
    { 0x62db6a39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4a5b6a3b, SUC_lexicon_tags_60 },
    { 0x713b6a3c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x919b6a3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21b6a43, SUC_lexicon_tags_60 },
    { 0xe43b6a44, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bb6a47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1b6a4d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fb6a50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1b6a53, SUC_lexicon_tags_21 },
    { 0x611b6a54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bb6a57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x691b6a59, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x197b6a5b, SUC_lexicon_tags_1 },
    { 0x82db6a5c, SUC_lexicon_tags_37 },
    { 0x239b6a5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5b6a61, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x175b6a63, SUC_lexicon_tags_17 },
    { 0xc5fb6a63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1b6a69, SUC_lexicon_tags_38 },
    { 0x763b6a6a, SUC_lexicon_tags_7 },
    { 0xd23b6a69, SUC_lexicon_tags_11 },
    { 0x689b6a6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffb6a73, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331b6a79, SUC_lexicon_tags_23 },
    { 0x69b6a7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bfb6a7c, SUC_lexicon_tags_56 },
    { 0x1edb6a7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeffb6a7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5b6a83, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcadb6a85, SUC_lexicon_tags_17 },
    { 0x9ddb6a86, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5adb6a88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673b6a8d, SUC_lexicon_tags_21 },
    { 0x6dbb6a8e, SUC_lexicon_tags_2 },
    { 0xb13b6a8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8db6a92, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x2e1b6a94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fb6a98, SUC_lexicon_tags_11 },
    { 0x59fb6a98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1b6a9f, SUC_lexicon_tags_0 },
    { 0xefb6aa0, SUC_lexicon_tags_45 },
    { 0xc97b6aa0, SUC_lexicon_tags_47 },
    { 0x40bb6a9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb6aa8, SUC_lexicon_tags_28 },
    { 0x63db6aa9, SUC_lexicon_tags_47 },
    { 0x519b6aaa, SUC_lexicon_tags_24 },
    { 0xd3db6aab, SUC_lexicon_tags_1 },
    { 0xd35b6aac, SUC_lexicon_tags_29 },
    { 0xfbdb6aab, SUC_lexicon_tags_37 },
    { 0x893b6aad, SUC_lexicon_tags_15 },
    { 0xac9b6aae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb6ab8, SUC_lexicon_tags_5 },
    { 0xdabb6ab8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86bb6abb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x549b6abe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x251b6ac0, SUC_lexicon_tags_14 },
    { 0x87fb6ac0, SUC_lexicon_tags_17 },
    { 0x7d9b6ac0, SUC_lexicon_tags_11 },
    { 0xa15b6ac3, SUC_lexicon_tags_56 },
    { 0xd97b6ac3, SUC_lexicon_tags_47 },
    { 0x407b6ac2, SUC_lexicon_tags_14 },
    { 0x81b6ac6, SUC_lexicon_tags_172 },
    { 0x3cdb6ac7, SUC_lexicon_tags_12 },
    { 0x3d9b6ac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391b6ace, SUC_lexicon_tags_37 },
    { 0x81b6acf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x281b6ad1, SUC_lexicon_tags_74 },
    { 0x2f5b6ad1, SUC_lexicon_tags_7 },
    { 0x6abb6ad3, SUC_lexicon_tags_26 },
    { 0xe2bb6ad4, SUC_lexicon_tags_25 },
    { 0x153b6ad5, SUC_lexicon_tags_37 },
    { 0x663b6ad1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf89b6ad9, SUC_lexicon_tags_63 },
    { 0xf39b6ada, SUC_lexicon_tags_2 },
    { 0x515b6adb, SUC_lexicon_tags_16 },
    { 0x199b6adc, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fb6ae1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bb6ae7, SUC_lexicon_tags_38 },
    { 0xb57b6ae8, SUC_lexicon_tags_12 },
    { 0x4dfb6ae9, SUC_lexicon_tags_38 },
    { 0x7dfb6aea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xce5b6aec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdb6aef, SUC_lexicon_tags_1 },
    { 0xb6bb6aef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x68fb6af3, SUC_lexicon_tags_15 },
    { 0x9adb6af4, SUC_lexicon_tags_82 },
    { 0x90db6af5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfb6af9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31b6afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x391b6b01, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56db6b06, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33b6b09, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffb6b12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39b6b18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe19b6b1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3b6b1d, SUC_lexicon_tags_47 },
    { 0xffb6b1e, SUC_lexicon_tags_60 },
    { 0x997b6b1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bb6b25, SUC_lexicon_tags_15 },
    { 0x361b6b25, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47b6b2e, SUC_lexicon_tags_45 },
    { 0x281b6b2f, SUC_lexicon_tags_0 },
    { 0x7adb6b2e, SUC_lexicon_tags_19 },
    { 0x8bb6b31, SUC_lexicon_tags_21 },
    { 0x5e9b6b31, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4c1b6b34, SUC_lexicon_tags_12 },
    { 0x4d9b6b34, SUC_lexicon_tags_27 },
    { 0x3abb6b36, SUC_lexicon_tags_1 },
    { 0x21db6b37, SUC_lexicon_tags_12 },
    { 0x887b6b38, SUC_lexicon_tags_12 },
    { 0x971b6b39, SUC_lexicon_tags_17 },
    { 0xcd1b6b3a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaf7b6b3c, SUC_lexicon_tags_1 },
    { 0xf9bb6b3d, SUC_lexicon_tags_37 },
    { 0x733b6b3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bb6b41, SUC_lexicon_tags_38 },
    { 0x8f3b6b42, SUC_lexicon_tags_12 },
    { 0x5f1b6b43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105b6b47, SUC_lexicon_tags_0 },
    { 0x865b6b47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x421b6b4a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07b6b4d, SUC_lexicon_tags_8 },
    { 0xe55b6b4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x701b6b50, SUC_lexicon_tags_49 },
    { 0x847b6b50, SUC_lexicon_tags_19 },
    { 0xdd3b6b51, SUC_lexicon_tags_33 },
    { 0xab9b6b52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11bb6b55, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x127b6b57, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9b6b5b, SUC_lexicon_tags_37 },
    { 0x421b6b5c, SUC_lexicon_tags_66 },
    { 0x907b6b5d, SUC_lexicon_tags_39 },
    { 0x6f7b6b5d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fb6b61, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xc03b6b63, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdb6b66, SUC_lexicon_tags_0 },
    { 0x971b6b67, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfb6b6a, SUC_lexicon_tags_61 },
    { 0x279b6b6a, SUC_lexicon_tags_48 },
    { 0xe4db6b6a, SUC_lexicon_tags_69 },
    { 0xae7b6b6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x877b6b70, SUC_lexicon_tags_15 },
    { 0xca9b6b70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbb6b75, SUC_lexicon_tags_37 },
    { 0xb7b6b75, SUC_lexicon_tags_37 },
    { 0xac9b6b77, SUC_lexicon_tags_25 },
    { 0x231b6b77, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x669b6b7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fb6b7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x143b6b81, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67db6b88, SUC_lexicon_tags_35 },
    { 0x89fb6b89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15bb6b8b, SUC_lexicon_tags_17 },
    { 0x67bb6b8c, SUC_lexicon_tags_31 },
    { 0xd53b6b8d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x467b6b8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abb6b93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3b6b97, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x11b6b9a, SUC_lexicon_tags_25 },
    { 0xf93b6b9a, SUC_lexicon_tags_3 },
    { 0x9afb6b9c, SUC_lexicon_tags_31 },
    { 0x5f9b6b9d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x371b6b9f, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3b6ba5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe17b6ba7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95b6baa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x875b6bac, SUC_lexicon_tags_12 },
    { 0xa6bb6bac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3abb6baf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x129b6bb2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x247b6bb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45b6bb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61b6bbb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381b6bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21fb6bc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173b6bc7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x48bb6bc9, SUC_lexicon_tags_8 },
    { 0xbd1b6bca, SUC_lexicon_tags_39 },
    { 0xaf1b6bcb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29b6bd4, SUC_lexicon_tags_45 },
    { 0x319b6bd5, SUC_lexicon_tags_16 },
    { 0x82db6bd6, SUC_lexicon_tags_17 },
    { 0x7c1b6bd7, SUC_lexicon_tags_43 },
    { 0xf71b6bd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c9b6bda, SUC_lexicon_tags_120 },
    { 0xb19b6bda, SUC_lexicon_tags_7 },
    { 0x191b6bdc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a9b6bde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67b6be2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db6be5, SUC_lexicon_tags_4 },
    { 0x753b6be5, SUC_lexicon_tags_11 },
    { 0xe17b6be5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7b6bea, SUC_lexicon_tags_48 },
    { 0x6bbb6beb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x725b6bed, SUC_lexicon_tags_49 },
    { 0x8cfb6bee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79b6bf3, SUC_lexicon_tags_11 },
    { 0xb13b6bf4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85b6bfc, SUC_lexicon_tags_29 },
    { 0x9a7b6bfd, SUC_lexicon_tags_17 },
    { 0xb4bb6bfe, SUC_lexicon_tags_45 },
    { 0xfb6bfd, SUC_lexicon_tags_1 },
    { 0xf0bb6bfe, SUC_lexicon_tags_19 },
    { 0xe1db6bfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9fb6c03, SUC_lexicon_tags_1 },
    { 0xa27b6c03, SUC_lexicon_tags_11 },
    { 0x34db6c05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f9b6c07, SUC_lexicon_tags_25 },
    { 0xc2db6c08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3b6c0d, SUC_lexicon_tags_47 },
    { 0xcd7b6c0e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5b6c12, SUC_lexicon_tags_19 },
    { 0xf9bb6c13, SUC_lexicon_tags_25 },
    { 0x8ffb6c14, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0x87bb6c16, SUC_lexicon_tags_37 },
    { 0x3f3b6c17, SUC_lexicon_tags_31 },
    { 0x207b6c18, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bb6c25, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcddb6c27, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553b6c2d, SUC_lexicon_tags_45 },
    { 0xdfdb6c2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3b6c31, SUC_lexicon_tags_264 },
    { 0xfdfb6c31, SUC_lexicon_tags_33 },
    { 0x1adb6c33, SUC_lexicon_tags_8 },
    { 0x143b6c31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79b6c3e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x47b6c40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd3b6c42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5db6c46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09b6c4c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe7fb6c4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61db6c53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7db6c55, SUC_lexicon_tags_23 },
    { 0xfddb6c56, SUC_lexicon_tags_21 },
    { 0xcd3b6c57, SUC_lexicon_tags_10 },
    { 0xcf3b6c58, SUC_lexicon_tags_3 },
    { 0xb81b6c59, SUC_lexicon_tags_11 },
    { 0x25db6c59, SUC_lexicon_tags_17 },
    { 0x1b1b6c5b, SUC_lexicon_tags_26 },
    { 0x5e9b6c59, SUC_lexicon_tags_77 },
    { 0xb3b6c5d, SUC_lexicon_tags_47 },
    { 0x9efb6c5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b5b6c60, SUC_lexicon_tags_12 },
    { 0x571b6c61, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5b6c68, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x78db6c6a, SUC_lexicon_tags_2 },
    { 0x211b6c6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bb6c6e, SUC_lexicon_tags_19 },
    { 0x377b6c6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9bb6c71, SUC_lexicon_tags_14 },
    { 0xffb6c71, SUC_lexicon_tags_7 },
    { 0xe25b6c71, SUC_lexicon_tags_7 },
    { 0xbbfb6c73, SUC_lexicon_tags_25 },
    { 0xcc5b6c75, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x415b6c78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1b6c7d, SUC_lexicon_tags_37 },
    { 0xb39b6c7e, SUC_lexicon_tags_37 },
    { 0x2f3b6c7e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bb6c87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcfbb6c89, SUC_lexicon_tags_17 },
    { 0x2e9b6c8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a1b6c8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb6c91, SUC_lexicon_tags_45 },
    { 0x385b6c92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb6c95, SUC_lexicon_tags_25 },
    { 0x457b6c96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x885b6c98, SUC_lexicon_tags_1 },
    { 0x915b6c99, SUC_lexicon_tags_2 },
    { 0x8cbb6c9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fb6c9d, SUC_lexicon_tags_30 },
    { 0x85b6c9e, SUC_lexicon_tags_32 },
    { 0xaabb6c9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9b6ca5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9b6ca8, SUC_lexicon_tags_17 },
    { 0xfbdb6ca9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423b6cb0, SUC_lexicon_tags_7 },
    { 0x1d3b6cb1, SUC_lexicon_tags_49 },
    { 0x8c1b6cb2, SUC_lexicon_tags_33 },
    { 0x923b6cb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x89bb6cb5, SUC_lexicon_tags_11 },
    { 0xd75b6cb6, SUC_lexicon_tags_87 },
    { 0x8a9b6cb6, SUC_lexicon_tags_37 },
    { 0xf29b6cb8, SUC_lexicon_tags_45 },
    { 0xb89b6cb9, SUC_lexicon_tags_25 },
    { 0x1b6cb9, SUC_lexicon_tags_15 },
    { 0x3afb6cb7, SUC_lexicon_tags_2 },
    { 0xf27b6cbc, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fb6cc0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf11b6cc2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3b6cc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63b6cc9, SUC_lexicon_tags_1 },
    { 0x5f5b6cca, SUC_lexicon_tags_37 },
    { 0x525b6cc9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xec5b6ccd, SUC_lexicon_tags_10 },
    { 0x62bb6cce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9b6cd1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bb6cdb, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237b6ce3, SUC_lexicon_tags_14 },
    { 0xf11b6ce3, SUC_lexicon_tags_19 },
    { 0x46db6ce5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x223b6ce7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd47b6ce9, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb6cec, SUC_lexicon_tags_0 },
    { 0x641b6ced, SUC_lexicon_tags_37 },
    { 0xfdb6ced, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edb6cf8, SUC_lexicon_tags_1 },
    { 0x7a5b6cf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabb6cfc, SUC_lexicon_tags_23 },
    { 0xa1db6cfd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9b6d00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x641b6d02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x491b6d05, SUC_lexicon_tags_47 },
    { 0xa3fb6d06, SUC_lexicon_tags_11 },
    { 0x7e1b6d07, SUC_lexicon_tags_14 },
    { 0x573b6d07, SUC_lexicon_tags_15 },
    { 0x339b6d09, SUC_lexicon_tags_38 },
    { 0x15fb6d0a, SUC_lexicon_tags_11 },
    { 0xc97b6d0b, SUC_lexicon_tags_14 },
    { 0x2c1b6d0c, SUC_lexicon_tags_15 },
    { 0xf4bb6d08, SUC_lexicon_tags_0 },
    { 0xd31b6d0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3b6d12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645b6d16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x333b6d18, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3b6d1b, SUC_lexicon_tags_34 },
    { 0x8b5b6d1c, SUC_lexicon_tags_14 },
    { 0x5b6d1c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b6d22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709b6d26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfb6d29, SUC_lexicon_tags_12 },
    { 0xb17b6d2a, SUC_lexicon_tags_8 },
    { 0x4c5b6d2a, SUC_lexicon_tags_40 },
    { 0x17fb6d2c, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0x55db6d2f, SUC_lexicon_tags_82 },
    { 0x59db6d30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdb6d35, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5b6d3b, SUC_lexicon_tags_17 },
    { 0x2bfb6d3c, SUC_lexicon_tags_15 },
    { 0x4f1b6d3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8db6d40, SUC_lexicon_tags_5 },
    { 0x82db6d41, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91b6d45, SUC_lexicon_tags_25 },
    { 0xf1bb6d45, SUC_lexicon_tags_37 },
    { 0x9d9b6d47, SUC_lexicon_tags_29 },
    { 0x7dfb6d47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb9db6d4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1b6d4d, SUC_lexicon_tags_43 },
    { 0x549b6d4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5e7b6d50, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43b6d58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1b6d5e, SUC_lexicon_tags_11 },
    { 0x321b6d5f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x705b6d61, SUC_lexicon_tags_49 },
    { 0xae5b6d62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47b6d68, SUC_lexicon_tags_27 },
    { 0x72fb6d69, SUC_lexicon_tags_37 },
    { 0x53b6d6a, SUC_lexicon_tags_25 },
    { 0x53fb6d6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69fb6d6d, SUC_lexicon_tags_2 },
    { 0xedbb6d6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963b6d75, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdafb6d77, SUC_lexicon_tags_6 },
    { 0x5bbb6d77, SUC_lexicon_tags_60 },
    { 0x4f7b6d77, SUC_lexicon_tags_2 },
    { 0x7b3b6d77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f1b6d7c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5db6d80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1b6d87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc39b6d89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1cbb6d8b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1b6d90, SUC_lexicon_tags_63 },
    { 0x987b6d91, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44db6d95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0fb6d97, SUC_lexicon_tags_17 },
    { 0x85bb6d97, SUC_lexicon_tags_8 },
    { 0x53fb6d97, SUC_lexicon_tags_45 },
    { 0xac7b6d99, SUC_lexicon_tags_7 },
    { 0x2ddb6d9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173b6da1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bb6dab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939b6daf, SUC_lexicon_tags_30 },
    { 0x359b6daf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcb9b6db2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9fbb6db4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51db6db8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3bb6dba, SUC_lexicon_tags_15 },
    { 0xda3b6dbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dfb6dbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5b6dc2, SUC_lexicon_tags_45 },
    { 0xde1b6dc3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x321b6dc6, SUC_lexicon_tags_7 },
    { 0xe99b6dc7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fb6dcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffb6dce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605b6dd3, SUC_lexicon_tags_25 },
    { 0x573b6dd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe29b6dd6, SUC_lexicon_tags_29 },
    { 0x8b7b6dd6, SUC_lexicon_tags_25 },
    { 0xb13b6dd6, SUC_lexicon_tags_1 },
    { 0x865b6dd9, SUC_lexicon_tags_37 },
    { 0x9d7b6dd9, SUC_lexicon_tags_46 },
    { 0x7ddb6ddb, SUC_lexicon_tags_4 },
    { 0xb33b6dd8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189b6de1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc6bb6de3, SUC_lexicon_tags_160 },
    { 0x903b6de4, SUC_lexicon_tags_87 },
    { 0xf49b6de4, SUC_lexicon_tags_7 },
    { 0x2f3b6de6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fb6dea, SUC_lexicon_tags_11 },
    { 0x229b6deb, SUC_lexicon_tags_11 },
    { 0x7efb6deb, SUC_lexicon_tags_2 },
    { 0x47db6dea, SUC_lexicon_tags_25 },
    { 0x6a3b6deb, SUC_lexicon_tags_7 },
    { 0x5bbb6ded, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9b6df3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdb6dfa, SUC_lexicon_tags_760 },
    { 0, NULL },
    { 0xfc5b6dfc, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bb6e00, SUC_lexicon_tags_15 },
    { 0xb53b6e01, SUC_lexicon_tags_92 },
    { 0x2ddb6e02, SUC_lexicon_tags_1 },
    { 0x185b6e02, SUC_lexicon_tags_61 },
    { 0x4c7b6e04, SUC_lexicon_tags_15 },
    { 0xacdb6e05, SUC_lexicon_tags_17 },
    { 0x3d7b6e06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdb6e09, SUC_lexicon_tags_30 },
    { 0x295b6e0a, SUC_lexicon_tags_14 },
    { 0xdd3b6e0a, SUC_lexicon_tags_25 },
    { 0x2f9b6e09, SUC_lexicon_tags_12 },
    { 0xe85b6e0d, SUC_lexicon_tags_11 },
    { 0xc9db6e0e, SUC_lexicon_tags_42 },
    { 0xffbb6e0d, SUC_lexicon_tags_87 },
    { 0x183b6e10, SUC_lexicon_tags_12 },
    { 0x7efb6e0d, SUC_lexicon_tags_15 },
    { 0x263b6e11, SUC_lexicon_tags_17 },
    { 0xd49b6e13, SUC_lexicon_tags_11 },
    { 0xd2db6e0e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x645b6e17, SUC_lexicon_tags_43 },
    { 0x87db6e17, SUC_lexicon_tags_1 },
    { 0x71b6e19, SUC_lexicon_tags_11 },
    { 0x1f5b6e1a, SUC_lexicon_tags_48 },
    { 0x6a1b6e1a, SUC_lexicon_tags_2 },
    { 0x865b6e1c, SUC_lexicon_tags_11 },
    { 0x5bfb6e1b, SUC_lexicon_tags_3 },
    { 0x397b6e1e, SUC_lexicon_tags_29 },
    { 0x895b6e1f, SUC_lexicon_tags_12 },
    { 0xc21b6e1b, SUC_lexicon_tags_43 },
    { 0x15b6e1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x887b6e23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773b6e29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9b6e2c, SUC_lexicon_tags_25 },
    { 0xfdfb6e2d, SUC_lexicon_tags_0 },
    { 0xc2bb6e2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb49b6e30, SUC_lexicon_tags_15 },
    { 0x5e7b6e31, SUC_lexicon_tags_26 },
    { 0xb59b6e31, SUC_lexicon_tags_66 },
    { 0x73fb6e33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fb6e36, SUC_lexicon_tags_1 },
    { 0x68bb6e37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d3b6e39, SUC_lexicon_tags_68 },
    { 0x875b6e39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7db6e3c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xac1b6e3f, SUC_lexicon_tags_7 },
    { 0x3dfb6e3f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0db6e43, SUC_lexicon_tags_1215 },
    { 0x5ebb6e44, SUC_lexicon_tags_15 },
    { 0xdfbb6e44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3b6e4a, SUC_lexicon_tags_47 },
    { 0x303b6e4a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fb6e53, SUC_lexicon_tags_37 },
    { 0x1ddb6e53, SUC_lexicon_tags_92 },
    { 0x32fb6e55, SUC_lexicon_tags_2 },
    { 0x8cdb6e55, SUC_lexicon_tags_37 },
    { 0x7a7b6e55, SUC_lexicon_tags_15 },
    { 0x223b6e58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11b6e5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3b6e61, SUC_lexicon_tags_25 },
    { 0x2f7b6e62, SUC_lexicon_tags_27 },
    { 0x761b6e62, SUC_lexicon_tags_34 },
    { 0xbe7b6e63, SUC_lexicon_tags_11 },
    { 0xc85b6e65, SUC_lexicon_tags_1 },
    { 0xcebb6e62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdbb6e68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81b6e6b, SUC_lexicon_tags_30 },
    { 0xc9fb6e6b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fb6e71, SUC_lexicon_tags_60 },
    { 0x335b6e72, SUC_lexicon_tags_101 },
    { 0x5b1b6e73, SUC_lexicon_tags_63 },
    { 0x3a9b6e72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52db6e77, SUC_lexicon_tags_25 },
    { 0x19db6e78, SUC_lexicon_tags_17 },
    { 0xddbb6e79, SUC_lexicon_tags_23 },
    { 0x915b6e7a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5b6e85, SUC_lexicon_tags_82 },
    { 0xfe3b6e86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29db6e8a, SUC_lexicon_tags_10 },
    { 0x613b6e8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b5b6e8d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5c1b6e8f, SUC_lexicon_tags_38 },
    { 0xf9fb6e90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2fb6e92, SUC_lexicon_tags_1 },
    { 0x421b6e93, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x53fb6e95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbb6e9d, SUC_lexicon_tags_77 },
    { 0xa77b6e9e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27b6ea1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb6ea6, SUC_lexicon_tags_43 },
    { 0x8e3b6ea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7b6ead, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb73b6eaf, SUC_lexicon_tags_8 },
    { 0xfc7b6eaf, SUC_lexicon_tags_48 },
    { 0x217b6eaf, SUC_lexicon_tags_12 },
    { 0xa7fb6eb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa43b6eb4, SUC_lexicon_tags_7 },
    { 0x37bb6eb5, SUC_lexicon_tags_29 },
    { 0x8bfb6eb4, SUC_lexicon_tags_15 },
    { 0x88db6eb7, SUC_lexicon_tags_96 },
    { 0xe47b6eb8, SUC_lexicon_tags_43 },
    { 0x273b6eb8, SUC_lexicon_tags_107 },
    { 0x419b6eb9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x13bb6ebc, SUC_lexicon_tags_27 },
    { 0xa67b6ebc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119b6ec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5b6ec9, SUC_lexicon_tags_25 },
    { 0x405b6eca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb41b6ecc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409b6ed0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x869b6ed2, SUC_lexicon_tags_47 },
    { 0xc13b6ed3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa59b6ed5, SUC_lexicon_tags_37 },
    { 0xdbfb6ed5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5db6ed8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x817b6eda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9db6ee1, SUC_lexicon_tags_1 },
    { 0x95fb6ee2, SUC_lexicon_tags_15 },
    { 0xefb6ee3, SUC_lexicon_tags_31 },
    { 0xcebb6ee3, SUC_lexicon_tags_16 },
    { 0x613b6ee3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x60db6ee8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41db6ef1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb25b6ef3, SUC_lexicon_tags_34 },
    { 0xb63b6ef4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573b6ef9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bb6efd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2f7b6eff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7b6f03, SUC_lexicon_tags_37 },
    { 0x7a1b6f03, SUC_lexicon_tags_61 },
    { 0x5a5b6f04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf1fb6f07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa7b6f09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81bb6f0b, SUC_lexicon_tags_12 },
    { 0x27fb6f0b, SUC_lexicon_tags_11 },
    { 0x753b6f0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91b6f10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7efb6f12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbb6f15, SUC_lexicon_tags_0 },
    { 0x7e5b6f16, SUC_lexicon_tags_11 },
    { 0xb97b6f17, SUC_lexicon_tags_47 },
    { 0x68fb6f18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7b6f1c, SUC_lexicon_tags_27 },
    { 0xfe5b6f1d, SUC_lexicon_tags_63 },
    { 0x8ffb6f1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1b6f22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x667b6f24, SUC_lexicon_tags_60 },
    { 0x6c1b6f25, SUC_lexicon_tags_60 },
    { 0x283b6f25, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdb6f29, SUC_lexicon_tags_60 },
    { 0x71bb6f29, SUC_lexicon_tags_16 },
    { 0xd0db6f2b, SUC_lexicon_tags_43 },
    { 0xd3db6f2c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa87b6f2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7b6f32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x213b6f34, SUC_lexicon_tags_20 },
    { 0x33bb6f35, SUC_lexicon_tags_25 },
    { 0x85bb6f36, SUC_lexicon_tags_1 },
    { 0x7afb6f36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671b6f3c, SUC_lexicon_tags_1 },
    { 0x83db6f3d, SUC_lexicon_tags_7 },
    { 0xe25b6f3d, SUC_lexicon_tags_63 },
    { 0xff7b6f3f, SUC_lexicon_tags_41 },
    { 0x617b6f40, SUC_lexicon_tags_8 },
    { 0x403b6f41, SUC_lexicon_tags_82 },
    { 0x52db6f41, SUC_lexicon_tags_47 },
    { 0x9a1b6f43, SUC_lexicon_tags_45 },
    { 0x53fb6f3f, SUC_lexicon_tags_2 },
    { 0x71bb6f45, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x571b6f47, SUC_lexicon_tags_57 },
    { 0xf41b6f47, SUC_lexicon_tags_32 },
    { 0x8b9b6f49, SUC_lexicon_tags_6 },
    { 0x233b6f49, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabb6f51, SUC_lexicon_tags_37 },
    { 0x8c3b6f52, SUC_lexicon_tags_32 },
    { 0xb2db6f51, SUC_lexicon_tags_0 },
    { 0xc1bb6f54, SUC_lexicon_tags_7 },
    { 0x591b6f51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3f7b6f57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5b6f5b, SUC_lexicon_tags_17 },
    { 0xda3b6f5b, SUC_lexicon_tags_8 },
    { 0xadbb6f5b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8f7b6f5f, SUC_lexicon_tags_31 },
    { 0x91db6f60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59b6f66, SUC_lexicon_tags_15 },
    { 0x865b6f66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641b6f70, SUC_lexicon_tags_35 },
    { 0x25db6f71, SUC_lexicon_tags_15 },
    { 0x2b7b6f72, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x46db6f74, SUC_lexicon_tags_2 },
    { 0x44bb6f75, SUC_lexicon_tags_11 },
    { 0x2e9b6f76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdb6f7c, SUC_lexicon_tags_60 },
    { 0xa85b6f7d, SUC_lexicon_tags_30 },
    { 0xfc5b6f7e, SUC_lexicon_tags_12 },
    { 0x235b6f7f, SUC_lexicon_tags_68 },
    { 0x8c5b6f7c, SUC_lexicon_tags_12 },
    { 0x6c9b6f81, SUC_lexicon_tags_19 },
    { 0x87b6f7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7db6f85, SUC_lexicon_tags_12 },
    { 0x9bbb6f86, SUC_lexicon_tags_12 },
    { 0xdbdb6f85, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fb6f8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26db6f8d, SUC_lexicon_tags_11 },
    { 0x6c1b6f8e, SUC_lexicon_tags_26 },
    { 0xd63b6f8e, SUC_lexicon_tags_3 },
    { 0x5f1b6f90, SUC_lexicon_tags_7 },
    { 0x56bb6f91, SUC_lexicon_tags_55 },
    { 0xdc7b6f92, SUC_lexicon_tags_2 },
    { 0x2bdb6f93, SUC_lexicon_tags_12 },
    { 0x19bb6f93, SUC_lexicon_tags_43 },
    { 0x16bb6f95, SUC_lexicon_tags_1 },
    { 0x109b6f94, SUC_lexicon_tags_25 },
    { 0x227b6f97, SUC_lexicon_tags_15 },
    { 0x10fb6f97, SUC_lexicon_tags_12 },
    { 0xd59b6f94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8db6f9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8fb6f9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fb6fa5, SUC_lexicon_tags_23 },
    { 0x7fbb6fa6, SUC_lexicon_tags_4 },
    { 0xad3b6fa5, SUC_lexicon_tags_7 },
    { 0x7d9b6fa7, SUC_lexicon_tags_31 },
    { 0x801b6fa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9b6fad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451b6fb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93bb6fb3, SUC_lexicon_tags_4 },
    { 0x6b7b6fb3, SUC_lexicon_tags_13 },
    { 0xad9b6fb5, SUC_lexicon_tags_43 },
    { 0x8dfb6fb5, SUC_lexicon_tags_11 },
    { 0x7bfb6fb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x465b6fb9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x573b6fbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x611b6fbd, SUC_lexicon_tags_43 },
    { 0x4cbb6fbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9b6fc1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47b6fc5, SUC_lexicon_tags_12 },
    { 0xaa1b6fc5, SUC_lexicon_tags_25 },
    { 0x6cdb6fc7, SUC_lexicon_tags_63 },
    { 0xc09b6fc5, SUC_lexicon_tags_1 },
    { 0x7db6fc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c7b6fcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x205b6fcd, SUC_lexicon_tags_15 },
    { 0xee9b6fce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2cdb6fd0, SUC_lexicon_tags_17 },
    { 0xf8db6fd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bb6fd7, SUC_lexicon_tags_38 },
    { 0xcb3b6fd8, SUC_lexicon_tags_39 },
    { 0x42db6fd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf41b6fdb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5b6fde, SUC_lexicon_tags_63 },
    { 0x855b6fdf, SUC_lexicon_tags_34 },
    { 0x6c5b6fdf, SUC_lexicon_tags_1 },
    { 0x109b6fe1, SUC_lexicon_tags_30 },
    { 0x1a1b6fe1, SUC_lexicon_tags_15 },
    { 0x5edb6fe3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9b6ff0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb6ff6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bb6ffb, SUC_lexicon_tags_40 },
    { 0x763b6ffc, SUC_lexicon_tags_12 },
    { 0x339b6ffd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d3b6fff, SUC_lexicon_tags_31 },
    { 0x56db7000, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1b7004, SUC_lexicon_tags_12 },
    { 0xf0db7005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7afb7007, SUC_lexicon_tags_11 },
    { 0x963b7008, SUC_lexicon_tags_25 },
    { 0xd15b7009, SUC_lexicon_tags_14 },
    { 0x203b700a, SUC_lexicon_tags_54 },
    { 0x689b7008, SUC_lexicon_tags_17 },
    { 0x1d1b700c, SUC_lexicon_tags_7 },
    { 0x971b7008, SUC_lexicon_tags_37 },
    { 0xe45b700a, SUC_lexicon_tags_37 },
    { 0x467b700f, SUC_lexicon_tags_53 },
    { 0xc07b7010, SUC_lexicon_tags_22 },
    { 0x473b7011, SUC_lexicon_tags_44 },
    { 0x87bb7012, SUC_lexicon_tags_62 },
    { 0x3a7b7013, SUC_lexicon_tags_11 },
    { 0xebdb700d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51b7019, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbb701d, SUC_lexicon_tags_17 },
    { 0x589b701e, SUC_lexicon_tags_9 },
    { 0x811b701d, SUC_lexicon_tags_1 },
    { 0xa7db7020, SUC_lexicon_tags_11 },
    { 0xac9b7020, SUC_lexicon_tags_1 },
    { 0xc0db7021, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x181b7024, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x925b7027, SUC_lexicon_tags_60 },
    { 0x23db7028, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb8bb702a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x103b702d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1b7031, SUC_lexicon_tags_11 },
    { 0x453b7032, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebb7037, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x489b7039, SUC_lexicon_tags_17 },
    { 0x1b703a, SUC_lexicon_tags_38 },
    { 0xdcfb703b, SUC_lexicon_tags_14 },
    { 0x4d1b703c, SUC_lexicon_tags_2 },
    { 0x811b703d, SUC_lexicon_tags_11 },
    { 0x6dfb703e, SUC_lexicon_tags_30 },
    { 0x4dbb703d, SUC_lexicon_tags_3 },
    { 0x19fb703c, SUC_lexicon_tags_34 },
    { 0x7b7b7041, SUC_lexicon_tags_11 },
    { 0xd7b7039, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bb7048, SUC_lexicon_tags_34 },
    { 0x5cdb7049, SUC_lexicon_tags_37 },
    { 0x5d5b704a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x445b704c, SUC_lexicon_tags_441 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fb704f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159b7058, SUC_lexicon_tags_23 },
    { 0xee9b7058, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b705c, SUC_lexicon_tags_46 },
    { 0x3d5b705c, SUC_lexicon_tags_14 },
    { 0x647b705c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31b7066, SUC_lexicon_tags_12 },
    { 0xba1b7067, SUC_lexicon_tags_17 },
    { 0x43fb7067, SUC_lexicon_tags_49 },
    { 0xe9db7069, SUC_lexicon_tags_37 },
    { 0x279b706a, SUC_lexicon_tags_37 },
    { 0x3afb706b, SUC_lexicon_tags_56 },
    { 0xb23b706c, SUC_lexicon_tags_1 },
    { 0x6fb706a, SUC_lexicon_tags_7 },
    { 0xdb1b706e, SUC_lexicon_tags_68 },
    { 0xe4db706e, SUC_lexicon_tags_6 },
    { 0x84fb706d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb25b7072, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x571b7075, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x483b7078, SUC_lexicon_tags_33 },
    { 0x601b7079, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa09b707b, SUC_lexicon_tags_31 },
    { 0x94fb707b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15b7081, SUC_lexicon_tags_12 },
    { 0x4cbb7082, SUC_lexicon_tags_47 },
    { 0xc51b7081, SUC_lexicon_tags_25 },
    { 0x301b7084, SUC_lexicon_tags_2 },
    { 0x891b7085, SUC_lexicon_tags_53 },
    { 0x27b7086, SUC_lexicon_tags_7 },
    { 0xf5fb7085, SUC_lexicon_tags_45 },
    { 0xf6fb7088, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11db708a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3db708c, SUC_lexicon_tags_19 },
    { 0x4f5b708d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xdbdb708f, SUC_lexicon_tags_17 },
    { 0x6d3b708f, SUC_lexicon_tags_2 },
    { 0x13db708f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53db7096, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3b709b, SUC_lexicon_tags_12 },
    { 0x827b709c, SUC_lexicon_tags_9 },
    { 0xf71b709d, SUC_lexicon_tags_7 },
    { 0x391b709d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5d7b70a0, SUC_lexicon_tags_39 },
    { 0x6c1b70a0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa9db70a3, SUC_lexicon_tags_0 },
    { 0xa93b70a4, SUC_lexicon_tags_7 },
    { 0x98db70a5, SUC_lexicon_tags_1 },
    { 0x197b70a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd5db70a8, SUC_lexicon_tags_19 },
    { 0x1bbb70a9, SUC_lexicon_tags_36 },
    { 0x9ddb70aa, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x62bb70ac, SUC_lexicon_tags_16 },
    { 0x5cbb70ad, SUC_lexicon_tags_25 },
    { 0x127b70ae, SUC_lexicon_tags_47 },
    { 0xbd5b70ae, SUC_lexicon_tags_53 },
    { 0x4a1b70b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfb70b5, SUC_lexicon_tags_63 },
    { 0xd0fb70b6, SUC_lexicon_tags_17 },
    { 0x8e3b70b7, SUC_lexicon_tags_38 },
    { 0x523b70b6, SUC_lexicon_tags_26 },
    { 0x3afb70b9, SUC_lexicon_tags_8 },
    { 0x5cfb70ba, SUC_lexicon_tags_28 },
    { 0xaedb70ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7b70be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbb70c4, SUC_lexicon_tags_25 },
    { 0x80fb70c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57b70c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x729b70cb, SUC_lexicon_tags_61 },
    { 0xc1fb70cc, SUC_lexicon_tags_82 },
    { 0x6d3b70cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45fb70cf, SUC_lexicon_tags_8 },
    { 0xe8bb70d0, SUC_lexicon_tags_37 },
    { 0x3ddb70d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9b70d9, SUC_lexicon_tags_0 },
    { 0x631b70d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x345b70dd, SUC_lexicon_tags_27 },
    { 0x347b70dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xff3b70e0, SUC_lexicon_tags_11 },
    { 0x91fb70e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afb70e5, SUC_lexicon_tags_37 },
    { 0x1d1b70e6, SUC_lexicon_tags_19 },
    { 0x5adb70e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87fb70e9, SUC_lexicon_tags_56 },
    { 0xeb1b70ea, SUC_lexicon_tags_1 },
    { 0xb43b70ea, SUC_lexicon_tags_1 },
    { 0x5c3b70ec, SUC_lexicon_tags_23 },
    { 0xf2db70e9, SUC_lexicon_tags_0 },
    { 0x673b70ee, SUC_lexicon_tags_10 },
    { 0x617b70ee, SUC_lexicon_tags_25 },
    { 0xfb70ef, SUC_lexicon_tags_26 },
    { 0x82bb70ea, SUC_lexicon_tags_37 },
    { 0x777b70f2, SUC_lexicon_tags_37 },
    { 0x529b70f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31b70f7, SUC_lexicon_tags_17 },
    { 0x8ebb70f7, SUC_lexicon_tags_14 },
    { 0x3c3b70f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x42fb70fc, SUC_lexicon_tags_17 },
    { 0x381b70fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fb7100, SUC_lexicon_tags_0 },
    { 0xbd5b7100, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893b7106, SUC_lexicon_tags_8 },
    { 0x851b7107, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345b710c, SUC_lexicon_tags_37 },
    { 0x43fb710d, SUC_lexicon_tags_295 },
    { 0xe71b710e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fb7112, SUC_lexicon_tags_31 },
    { 0xa81b7113, SUC_lexicon_tags_2 },
    { 0xd7bb7114, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x85b7116, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75b711a, SUC_lexicon_tags_45 },
    { 0xc47b711b, SUC_lexicon_tags_57 },
    { 0x7dbb711a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc7b711e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9b7124, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bb712e, SUC_lexicon_tags_26 },
    { 0xbedb712f, SUC_lexicon_tags_37 },
    { 0x2e1b7130, SUC_lexicon_tags_12 },
    { 0xde7b7131, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6db7133, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95b7137, SUC_lexicon_tags_30 },
    { 0xf41b7137, SUC_lexicon_tags_33 },
    { 0x9c9b7139, SUC_lexicon_tags_0 },
    { 0xff9b713a, SUC_lexicon_tags_9 },
    { 0xa2fb713a, SUC_lexicon_tags_39 },
    { 0xfd1b713a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fb713f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49b7143, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157b7147, SUC_lexicon_tags_7 },
    { 0x863b7148, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9b7151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515b7157, SUC_lexicon_tags_37 },
    { 0x197b7158, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983b715d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x343b7160, SUC_lexicon_tags_17 },
    { 0x87b7160, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bb7168, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3b716c, SUC_lexicon_tags_45 },
    { 0xee5b716d, SUC_lexicon_tags_6 },
    { 0x109b716e, SUC_lexicon_tags_13 },
    { 0xe5b716f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdb7172, SUC_lexicon_tags_56 },
    { 0xb21b7172, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453b7177, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27b717b, SUC_lexicon_tags_2 },
    { 0xeb3b717c, SUC_lexicon_tags_17 },
    { 0xa25b717c, SUC_lexicon_tags_37 },
    { 0x7ffb717b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x113b7181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe87b7183, SUC_lexicon_tags_7 },
    { 0x593b7184, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9b7188, SUC_lexicon_tags_7 },
    { 0x641b7189, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfb718d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3b5b718f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7b7191, SUC_lexicon_tags_30 },
    { 0x149b7192, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x985b7195, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397b719b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79b719e, SUC_lexicon_tags_2 },
    { 0xb0bb719f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839b71a9, SUC_lexicon_tags_6 },
    { 0x9a1b71aa, SUC_lexicon_tags_27 },
    { 0x179b71aa, SUC_lexicon_tags_38 },
    { 0x92db71aa, SUC_lexicon_tags_27 },
    { 0x631b71a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7db71b6, SUC_lexicon_tags_27 },
    { 0x1cfb71b7, SUC_lexicon_tags_98 },
    { 0xc5db71b6, SUC_lexicon_tags_40 },
    { 0x3cfb71b6, SUC_lexicon_tags_66 },
    { 0xa89b71b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b71bd, SUC_lexicon_tags_46 },
    { 0x92db71be, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b71c1, SUC_lexicon_tags_15 },
    { 0x82db71c2, SUC_lexicon_tags_1 },
    { 0x459b71c3, SUC_lexicon_tags_29 },
    { 0x815b71c2, SUC_lexicon_tags_18 },
    { 0x9f3b71c5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05b71cb, SUC_lexicon_tags_25 },
    { 0x1dfb71cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9b71d1, SUC_lexicon_tags_32 },
    { 0xbd7b71d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99b71d5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xff1b71d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bb71db, SUC_lexicon_tags_37 },
    { 0x329b71dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfb71e0, SUC_lexicon_tags_17 },
    { 0x527b71e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15b71e6, SUC_lexicon_tags_2 },
    { 0xe75b71e7, SUC_lexicon_tags_17 },
    { 0x129b71e8, SUC_lexicon_tags_7 },
    { 0x37bb71e8, SUC_lexicon_tags_32 },
    { 0xd6fb71e8, SUC_lexicon_tags_12 },
    { 0xaddb71eb, SUC_lexicon_tags_11 },
    { 0xb07b71ec, SUC_lexicon_tags_27 },
    { 0x18bb71ed, SUC_lexicon_tags_49 },
    { 0x649b71ee, SUC_lexicon_tags_56 },
    { 0xad7b71ee, SUC_lexicon_tags_39 },
    { 0x123b71f0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x877b71f2, SUC_lexicon_tags_1 },
    { 0xb2bb71f3, SUC_lexicon_tags_55 },
    { 0xba3b71f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f1b71f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfb71fb, SUC_lexicon_tags_19 },
    { 0xefdb71fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921b7200, SUC_lexicon_tags_15 },
    { 0x713b7201, SUC_lexicon_tags_12 },
    { 0xe91b7202, SUC_lexicon_tags_11 },
    { 0xc7b7203, SUC_lexicon_tags_29 },
    { 0xf59b7203, SUC_lexicon_tags_17 },
    { 0xc99b7205, SUC_lexicon_tags_7 },
    { 0xe79b7205, SUC_lexicon_tags_37 },
    { 0xfb5b7200, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7b720a, SUC_lexicon_tags_12 },
    { 0x841b720a, SUC_lexicon_tags_60 },
    { 0xb39b720c, SUC_lexicon_tags_6 },
    { 0x595b720d, SUC_lexicon_tags_49 },
    { 0x25b720d, SUC_lexicon_tags_11 },
    { 0x3c3b720f, SUC_lexicon_tags_15 },
    { 0x53b7210, SUC_lexicon_tags_37 },
    { 0xa2fb7210, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7b7216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bb721a, SUC_lexicon_tags_43 },
    { 0x8fb721b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3b7222, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e3b7224, SUC_lexicon_tags_14 },
    { 0xe4bb7224, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bb722b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229b7230, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3b7234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81db723a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x86db723c, SUC_lexicon_tags_29 },
    { 0xd55b723d, SUC_lexicon_tags_95 },
    { 0x1e5b723c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56db7240, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x991b7242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31b7245, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5b7248, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffb724b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07b724f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fb7252, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35b7256, SUC_lexicon_tags_87 },
    { 0xab7b7257, SUC_lexicon_tags_82 },
    { 0xe03b7258, SUC_lexicon_tags_38 },
    { 0x9b7b7259, SUC_lexicon_tags_38 },
    { 0xbc5b7258, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afb725f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1b7264, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x285b7267, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbb726f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efb7274, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdb7277, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bb727a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35b727e, SUC_lexicon_tags_17 },
    { 0xbf7b727f, SUC_lexicon_tags_46 },
    { 0x85fb7280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09b7285, SUC_lexicon_tags_2 },
    { 0x75fb7286, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9b7289, SUC_lexicon_tags_2 },
    { 0xfbbb728a, SUC_lexicon_tags_3 },
    { 0xbb3b728b, SUC_lexicon_tags_26 },
    { 0xb9db728b, SUC_lexicon_tags_1 },
    { 0x69db728a, SUC_lexicon_tags_29 },
    { 0xfb7289, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47bb7290, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7b7295, SUC_lexicon_tags_26 },
    { 0x863b7296, SUC_lexicon_tags_21 },
    { 0x6d7b7297, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edb729b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5b729e, SUC_lexicon_tags_64 },
    { 0x52fb729f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cfb72a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333b72a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3b72a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38db72b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x977b72b3, SUC_lexicon_tags_12 },
    { 0xb15b72b3, SUC_lexicon_tags_43 },
    { 0xd73b72b4, SUC_lexicon_tags_47 },
    { 0x359b72b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fb72be, SUC_lexicon_tags_1 },
    { 0x1ebb72bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57b72c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd59b72c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61b72c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651b72cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d5b72cd, SUC_lexicon_tags_43 },
    { 0xb1db72ce, SUC_lexicon_tags_15 },
    { 0x9cfb72cf, SUC_lexicon_tags_5 },
    { 0x233b72d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x677b72d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15b72da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x637b72dc, SUC_lexicon_tags_45 },
    { 0xbffb72dd, SUC_lexicon_tags_47 },
    { 0x99bb72dd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91b72e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8fb72e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xabbb72e7, SUC_lexicon_tags_7 },
    { 0x255b72e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8db72ec, SUC_lexicon_tags_7 },
    { 0x66bb72ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x709b72ef, SUC_lexicon_tags_7 },
    { 0xaefb72f0, SUC_lexicon_tags_38 },
    { 0x42bb72ef, SUC_lexicon_tags_17 },
    { 0xe49b72f2, SUC_lexicon_tags_2 },
    { 0xefbb72f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773b72f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415b7300, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edb7305, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3b730a, SUC_lexicon_tags_11 },
    { 0x6cbb730b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe0fb730d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bb730f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebb7315, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa81b7317, SUC_lexicon_tags_13 },
    { 0x72fb7317, SUC_lexicon_tags_41 },
    { 0xbfdb7319, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x67db731c, SUC_lexicon_tags_0 },
    { 0x2b5b731c, SUC_lexicon_tags_44 },
    { 0x107b731e, SUC_lexicon_tags_14 },
    { 0xe07b731e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec1b7321, SUC_lexicon_tags_0 },
    { 0x29b7322, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523b732e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3b7332, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x255b7334, SUC_lexicon_tags_38 },
    { 0x629b7335, SUC_lexicon_tags_45 },
    { 0x40fb7336, SUC_lexicon_tags_15 },
    { 0x951b7336, SUC_lexicon_tags_8 },
    { 0xaddb7338, SUC_lexicon_tags_37 },
    { 0x6db7338, SUC_lexicon_tags_30 },
    { 0xaddb7338, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21db7344, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3b7346, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x645b7348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabb734d, SUC_lexicon_tags_29 },
    { 0xabb734e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7b7357, SUC_lexicon_tags_7 },
    { 0xde3b7357, SUC_lexicon_tags_57 },
    { 0xd53b7358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x781b735b, SUC_lexicon_tags_136 },
    { 0xe45b735c, SUC_lexicon_tags_17 },
    { 0x787b735d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03b7368, SUC_lexicon_tags_92 },
    { 0x2edb7369, SUC_lexicon_tags_37 },
    { 0x4f7b7368, SUC_lexicon_tags_47 },
    { 0x857b736b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5b7370, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461b7375, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49b7378, SUC_lexicon_tags_46 },
    { 0x4c3b7379, SUC_lexicon_tags_28 },
    { 0x6fb737a, SUC_lexicon_tags_17 },
    { 0xb97b737a, SUC_lexicon_tags_28 },
    { 0x46bb737c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18fb737f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbb7383, SUC_lexicon_tags_46 },
    { 0x65fb7383, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x347b7386, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05b738a, SUC_lexicon_tags_61 },
    { 0x251b738b, SUC_lexicon_tags_27 },
    { 0xe3fb738b, SUC_lexicon_tags_7 },
    { 0x2abb738b, SUC_lexicon_tags_18 },
    { 0x75bb738d, SUC_lexicon_tags_60 },
    { 0x243b738b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2adb7394, SUC_lexicon_tags_1 },
    { 0x1ebb7394, SUC_lexicon_tags_11 },
    { 0xfe1b7395, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9b7399, SUC_lexicon_tags_45 },
    { 0xb45b7399, SUC_lexicon_tags_43 },
    { 0xfcbb739b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33b739f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1b73ab, SUC_lexicon_tags_31 },
    { 0x6ebb73ab, SUC_lexicon_tags_17 },
    { 0xc75b73ad, SUC_lexicon_tags_17 },
    { 0xaf1b73ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa43b73b0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdc1b73b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2db73b4, SUC_lexicon_tags_12 },
    { 0xa5b73b4, SUC_lexicon_tags_6 },
    { 0x963b73b4, SUC_lexicon_tags_1 },
    { 0x129b73b7, SUC_lexicon_tags_60 },
    { 0x9a7b73b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5db73ba, SUC_lexicon_tags_178 },
    { 0xdb7b73bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bb73c0, SUC_lexicon_tags_60 },
    { 0x20bb73c0, SUC_lexicon_tags_11 },
    { 0x8cbb73c1, SUC_lexicon_tags_15 },
    { 0x9b3b73c3, SUC_lexicon_tags_17 },
    { 0x109b73c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1b73cd, SUC_lexicon_tags_7 },
    { 0xfd3b73ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb1b73d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf83b73d2, SUC_lexicon_tags_4 },
    { 0x1cbb73d3, SUC_lexicon_tags_16 },
    { 0x31db73d2, SUC_lexicon_tags_1 },
    { 0x7f9b73d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdb73d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x913b73db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc47b73dd, SUC_lexicon_tags_17 },
    { 0xdddb73de, SUC_lexicon_tags_27 },
    { 0x6edb73de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9b73e2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7b73e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667b73e9, SUC_lexicon_tags_25 },
    { 0x2abb73ea, SUC_lexicon_tags_108 },
    { 0xebb73ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8db73f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73b73f7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffb7402, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fb7405, SUC_lexicon_tags_47 },
    { 0xab9b7405, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fb740b, SUC_lexicon_tags_65 },
    { 0xbbfb740c, SUC_lexicon_tags_2 },
    { 0x4fbb740d, SUC_lexicon_tags_25 },
    { 0x579b740d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e3b7410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29b7414, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbb7419, SUC_lexicon_tags_19 },
    { 0xe79b741a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc15b741c, SUC_lexicon_tags_9 },
    { 0x551b741d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb7421, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x92bb7423, SUC_lexicon_tags_63 },
    { 0x9bfb7424, SUC_lexicon_tags_12 },
    { 0xf8fb7424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17db7427, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7b742a, SUC_lexicon_tags_124 },
    { 0xa6bb742a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44db742e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa65b7430, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69b7434, SUC_lexicon_tags_60 },
    { 0x227b7434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe85b7437, SUC_lexicon_tags_31 },
    { 0x675b7438, SUC_lexicon_tags_21 },
    { 0x9db7439, SUC_lexicon_tags_28 },
    { 0xb07b743a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7db743c, SUC_lexicon_tags_38 },
    { 0x29bb743c, SUC_lexicon_tags_8 },
    { 0x649b743e, SUC_lexicon_tags_37 },
    { 0x531b743d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xd53b7441, SUC_lexicon_tags_2 },
    { 0x8fdb7442, SUC_lexicon_tags_15 },
    { 0xd3b7443, SUC_lexicon_tags_27 },
    { 0xdf9b7444, SUC_lexicon_tags_7 },
    { 0xa39b7441, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x765b7447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75b7449, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x321b744c, SUC_lexicon_tags_23 },
    { 0x8d1b744d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49b7450, SUC_lexicon_tags_8 },
    { 0xa09b7450, SUC_lexicon_tags_1 },
    { 0xe0bb7451, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72db7454, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1b745c, SUC_lexicon_tags_37 },
    { 0x129b745d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853b7462, SUC_lexicon_tags_123 },
    { 0x711b7463, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb7466, SUC_lexicon_tags_12 },
    { 0xb9fb7466, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x737b746a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efb746e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc55b7470, SUC_lexicon_tags_28 },
    { 0x92db7471, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x275b7474, SUC_lexicon_tags_0 },
    { 0x79db7475, SUC_lexicon_tags_27 },
    { 0x241b7475, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x203b7478, SUC_lexicon_tags_276 },
    { 0, NULL },
    { 0xce5b747a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3b747f, SUC_lexicon_tags_8 },
    { 0xed1b7480, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1b7486, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183b748c, SUC_lexicon_tags_121 },
    { 0x2e7b748c, SUC_lexicon_tags_12 },
    { 0xd0db748d, SUC_lexicon_tags_12 },
    { 0x17b748f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x21bb7491, SUC_lexicon_tags_12 },
    { 0x5d5b7492, SUC_lexicon_tags_45 },
    { 0x35db7492, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39b7497, SUC_lexicon_tags_0 },
    { 0x589b7498, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbb749c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851b74a9, SUC_lexicon_tags_23 },
    { 0xed7b74aa, SUC_lexicon_tags_12 },
    { 0x28db74aa, SUC_lexicon_tags_0 },
    { 0xd73b74ab, SUC_lexicon_tags_37 },
    { 0xe81b74ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9db74af, SUC_lexicon_tags_26 },
    { 0x1b7b74b0, SUC_lexicon_tags_15 },
    { 0xa1bb74af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57db74b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x695b74b7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5b74ba, SUC_lexicon_tags_43 },
    { 0x7abb74ba, SUC_lexicon_tags_27 },
    { 0xcefb74ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcdb74be, SUC_lexicon_tags_12 },
    { 0x799b74bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1b74c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59fb74c7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6c3b74c9, SUC_lexicon_tags_17 },
    { 0xc13b74ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53db74ce, SUC_lexicon_tags_37 },
    { 0x2ffb74cf, SUC_lexicon_tags_16 },
    { 0x5bb74cf, SUC_lexicon_tags_30 },
    { 0xf1fb74d1, SUC_lexicon_tags_48 },
    { 0x289b74d2, SUC_lexicon_tags_43 },
    { 0xb59b74d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5b74d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51fb74d9, SUC_lexicon_tags_2 },
    { 0x1db74da, SUC_lexicon_tags_84 },
    { 0x4abb74d9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1b74de, SUC_lexicon_tags_1 },
    { 0x6bdb74de, SUC_lexicon_tags_7 },
    { 0x48db74e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42bb74e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc07b74e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x755b74e6, SUC_lexicon_tags_7 },
    { 0x6d9b74e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b3b74e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xddfb74ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05b74f5, SUC_lexicon_tags_47 },
    { 0x603b74f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x485b74f8, SUC_lexicon_tags_9 },
    { 0x51fb74f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3b74fc, SUC_lexicon_tags_25 },
    { 0xca1b74fd, SUC_lexicon_tags_26 },
    { 0x167b74fe, SUC_lexicon_tags_0 },
    { 0x531b74ff, SUC_lexicon_tags_7 },
    { 0x3cdb74fd, SUC_lexicon_tags_37 },
    { 0x807b74ff, SUC_lexicon_tags_60 },
    { 0x967b7502, SUC_lexicon_tags_27 },
    { 0xacbb74fd, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbb7506, SUC_lexicon_tags_15 },
    { 0x81db7506, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229b750b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x685b750e, SUC_lexicon_tags_12 },
    { 0x739b750f, SUC_lexicon_tags_6 },
    { 0xb8fb750f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e9b7512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bb7515, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bb7518, SUC_lexicon_tags_2 },
    { 0x1f9b7518, SUC_lexicon_tags_54 },
    { 0x34bb7519, SUC_lexicon_tags_14 },
    { 0xb1fb7519, SUC_lexicon_tags_17 },
    { 0x2e3b7518, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7b751f, SUC_lexicon_tags_8 },
    { 0x719b7520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3b7528, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229b752d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1bb752f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x879b7532, SUC_lexicon_tags_113 },
    { 0x23b7533, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bb753d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadfb753f, SUC_lexicon_tags_25 },
    { 0x955b7540, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99b7545, SUC_lexicon_tags_31 },
    { 0x9cfb7545, SUC_lexicon_tags_10 },
    { 0xa5db7546, SUC_lexicon_tags_49 },
    { 0x2bbb7548, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afb754e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfddb7550, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5f9b7552, SUC_lexicon_tags_12 },
    { 0x56db7553, SUC_lexicon_tags_43 },
    { 0x4f7b7552, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddb7559, SUC_lexicon_tags_8 },
    { 0xae3b755a, SUC_lexicon_tags_4 },
    { 0x33db7559, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bb755f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863b7568, SUC_lexicon_tags_16 },
    { 0xa1b7568, SUC_lexicon_tags_3 },
    { 0xfc9b756a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3b756d, SUC_lexicon_tags_70 },
    { 0xa79b756d, SUC_lexicon_tags_40 },
    { 0x91fb756e, SUC_lexicon_tags_63 },
    { 0xe13b756d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd21b7572, SUC_lexicon_tags_63 },
    { 0xe5fb7572, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bb7577, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x723b7579, SUC_lexicon_tags_60 },
    { 0xf1fb7579, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3b757d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27b7580, SUC_lexicon_tags_3 },
    { 0xe01b7581, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe8bb7583, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1b7587, SUC_lexicon_tags_36 },
    { 0x21bb7588, SUC_lexicon_tags_12 },
    { 0x83fb7589, SUC_lexicon_tags_37 },
    { 0x213b7588, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9b758e, SUC_lexicon_tags_27 },
    { 0xa05b758f, SUC_lexicon_tags_4 },
    { 0xeffb758f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2a9b7592, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3b7598, SUC_lexicon_tags_34 },
    { 0x8cfb7598, SUC_lexicon_tags_27 },
    { 0x491b759a, SUC_lexicon_tags_25 },
    { 0xdbfb759b, SUC_lexicon_tags_11 },
    { 0x5b9b759a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bb75a1, SUC_lexicon_tags_2 },
    { 0x23b75a2, SUC_lexicon_tags_0 },
    { 0xa0bb75a1, SUC_lexicon_tags_25 },
    { 0x5efb75a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfb75a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8db75ae, SUC_lexicon_tags_25 },
    { 0x939b75af, SUC_lexicon_tags_19 },
    { 0x169b75b0, SUC_lexicon_tags_15 },
    { 0x3f1b75af, SUC_lexicon_tags_47 },
    { 0x353b75b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8fb75b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6a9b75b6, SUC_lexicon_tags_63 },
    { 0xa7bb75b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x593b75ba, SUC_lexicon_tags_37 },
    { 0xebbb75bb, SUC_lexicon_tags_37 },
    { 0xbafb75bb, SUC_lexicon_tags_2 },
    { 0x3cdb75bd, SUC_lexicon_tags_49 },
    { 0xc71b75bb, SUC_lexicon_tags_25 },
    { 0x2b9b75bf, SUC_lexicon_tags_52 },
    { 0xf9db75bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57b75c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fb75c8, SUC_lexicon_tags_2 },
    { 0xa47b75c9, SUC_lexicon_tags_38 },
    { 0xb75b75c8, SUC_lexicon_tags_65 },
    { 0x977b75cb, SUC_lexicon_tags_17 },
    { 0xf3db75cc, SUC_lexicon_tags_21 },
    { 0x9dfb75c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff9b75cf, SUC_lexicon_tags_17 },
    { 0x53b75d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60bb75d2, SUC_lexicon_tags_60 },
    { 0xab5b75d2, SUC_lexicon_tags_43 },
    { 0xa4db75d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595b75da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebb75de, SUC_lexicon_tags_17 },
    { 0xd25b75df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88db75e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5c9b75e7, SUC_lexicon_tags_7 },
    { 0x787b75e8, SUC_lexicon_tags_37 },
    { 0xd0bb75e9, SUC_lexicon_tags_40 },
    { 0xbc3b75e7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bb75ee, SUC_lexicon_tags_11 },
    { 0x193b75ef, SUC_lexicon_tags_65 },
    { 0xa43b75ef, SUC_lexicon_tags_1 },
    { 0x6f5b75f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527b75f6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bb75f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8db75ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddb7603, SUC_lexicon_tags_1 },
    { 0xb27b7603, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5b7611, SUC_lexicon_tags_15 },
    { 0xf13b7611, SUC_lexicon_tags_46 },
    { 0xddb7611, SUC_lexicon_tags_0 },
    { 0xf05b7612, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffb7619, SUC_lexicon_tags_7 },
    { 0x1a7b7619, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x88fb761d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb1b761f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x237b7621, SUC_lexicon_tags_27 },
    { 0xd03b7622, SUC_lexicon_tags_14 },
    { 0x9a1b7622, SUC_lexicon_tags_61 },
    { 0xb9bb7623, SUC_lexicon_tags_37 },
    { 0xfddb7625, SUC_lexicon_tags_2 },
    { 0x5bdb7625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd75b7628, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645b762c, SUC_lexicon_tags_7 },
    { 0x26fb762c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61b7631, SUC_lexicon_tags_37 },
    { 0x9e7b7632, SUC_lexicon_tags_38 },
    { 0x32db7633, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffb7639, SUC_lexicon_tags_12 },
    { 0xe11b763a, SUC_lexicon_tags_45 },
    { 0x91bb763b, SUC_lexicon_tags_27 },
    { 0xb07b763a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63b7645, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x829b7647, SUC_lexicon_tags_33 },
    { 0xaafb7648, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82db764a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05b7653, SUC_lexicon_tags_37 },
    { 0xf7bb7653, SUC_lexicon_tags_17 },
    { 0xb13b7655, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fb765d, SUC_lexicon_tags_12 },
    { 0x675b765e, SUC_lexicon_tags_12 },
    { 0xebb765f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3b7662, SUC_lexicon_tags_65 },
    { 0x38fb7662, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x511b7665, SUC_lexicon_tags_32 },
    { 0x333b7665, SUC_lexicon_tags_35 },
    { 0x8c7b7667, SUC_lexicon_tags_39 },
    { 0xe17b7667, SUC_lexicon_tags_25 },
    { 0x943b7667, SUC_lexicon_tags_12 },
    { 0x8edb7667, SUC_lexicon_tags_1 },
    { 0x80bb766b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7b766e, SUC_lexicon_tags_1351 },
    { 0x2fbb766f, SUC_lexicon_tags_68 },
    { 0xebb7670, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bb7676, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbbb7678, SUC_lexicon_tags_11 },
    { 0xdd9b7678, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f1b767b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbb767e, SUC_lexicon_tags_12 },
    { 0x72bb767e, SUC_lexicon_tags_15 },
    { 0xb27b7680, SUC_lexicon_tags_47 },
    { 0x60bb767f, SUC_lexicon_tags_4 },
    { 0x22bb767f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x989b7684, SUC_lexicon_tags_7 },
    { 0x50bb7685, SUC_lexicon_tags_37 },
    { 0x9fbb7685, SUC_lexicon_tags_55 },
    { 0x33db7685, SUC_lexicon_tags_37 },
    { 0x1a7b7688, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x953b768a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x46db768c, SUC_lexicon_tags_1 },
    { 0x84bb768d, SUC_lexicon_tags_49 },
    { 0x3c7b768e, SUC_lexicon_tags_66 },
    { 0x8f9b768c, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0xe05b7691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce1b7693, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca7b7695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bb7699, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x225b769b, SUC_lexicon_tags_60 },
    { 0xa53b769b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7b76a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb61b76a3, SUC_lexicon_tags_32 },
    { 0x507b76a4, SUC_lexicon_tags_49 },
    { 0x2cdb76a5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37b76aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651b76b0, SUC_lexicon_tags_14 },
    { 0x635b76b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xadfb76b3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3b76ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7db76bd, SUC_lexicon_tags_57 },
    { 0x35fb76bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbb76c1, SUC_lexicon_tags_15 },
    { 0xa6db76c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x627b76c5, SUC_lexicon_tags_7 },
    { 0x9c7b76c6, SUC_lexicon_tags_15 },
    { 0x63bb76c7, SUC_lexicon_tags_15 },
    { 0x2b7b76c6, SUC_lexicon_tags_30 },
    { 0xce9b76c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffb76ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bb76d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x741b76d6, SUC_lexicon_tags_15 },
    { 0x95bb76d7, SUC_lexicon_tags_23 },
    { 0xe73b76d8, SUC_lexicon_tags_1 },
    { 0x91bb76d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3b76dc, SUC_lexicon_tags_15 },
    { 0xcd5b76dd, SUC_lexicon_tags_11 },
    { 0x2f3b76de, SUC_lexicon_tags_11 },
    { 0xe63b76dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7b76e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5adb76e4, SUC_lexicon_tags_25 },
    { 0x5c3b76e4, SUC_lexicon_tags_8 },
    { 0x60bb76e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xba1b76e8, SUC_lexicon_tags_59 },
    { 0x843b76e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb1db76eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75b76ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe37b76f1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3b76fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x47bb76fe, SUC_lexicon_tags_22 },
    { 0x5b76fe, SUC_lexicon_tags_26 },
    { 0x25db76fe, SUC_lexicon_tags_1 },
    { 0xebdb7701, SUC_lexicon_tags_25 },
    { 0xcefb7702, SUC_lexicon_tags_21 },
    { 0x6bfb7701, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcdb7705, SUC_lexicon_tags_11 },
    { 0x1d5b7706, SUC_lexicon_tags_10 },
    { 0x191b7705, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x589b770a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9b770f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbb7711, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47b7717, SUC_lexicon_tags_13 },
    { 0x987b7717, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3b771b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167b7720, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bb7723, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34db7728, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89b772c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1b772f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bb7732, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3b7736, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcc7b7738, SUC_lexicon_tags_17 },
    { 0x90bb7738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49bb773b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fb773f, SUC_lexicon_tags_7 },
    { 0xcb3b773f, SUC_lexicon_tags_40 },
    { 0x9d1b773f, SUC_lexicon_tags_46 },
    { 0xd21b7740, SUC_lexicon_tags_37 },
    { 0x691b7740, SUC_lexicon_tags_38 },
    { 0x345b773f, SUC_lexicon_tags_14 },
    { 0x81b7745, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37b774a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bb7750, SUC_lexicon_tags_2 },
    { 0xea5b7751, SUC_lexicon_tags_68 },
    { 0xdedb7750, SUC_lexicon_tags_12 },
    { 0x86db7752, SUC_lexicon_tags_26 },
    { 0x3d1b7751, SUC_lexicon_tags_68 },
    { 0x241b7751, SUC_lexicon_tags_37 },
    { 0x37bb7756, SUC_lexicon_tags_0 },
    { 0x8efb7757, SUC_lexicon_tags_17 },
    { 0x319b7758, SUC_lexicon_tags_17 },
    { 0x1abb7759, SUC_lexicon_tags_43 },
    { 0x69bb775a, SUC_lexicon_tags_11 },
    { 0x527b7751, SUC_lexicon_tags_1 },
    { 0x867b775c, SUC_lexicon_tags_11 },
    { 0xe93b775c, SUC_lexicon_tags_0 },
    { 0xd99b7755, SUC_lexicon_tags_83 },
    { 0x9c1b775f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5b7765, SUC_lexicon_tags_17 },
    { 0xe5bb7765, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbb7769, SUC_lexicon_tags_48 },
    { 0xfa9b776a, SUC_lexicon_tags_6 },
    { 0x3d9b776b, SUC_lexicon_tags_25 },
    { 0x4adb776c, SUC_lexicon_tags_23 },
    { 0xc9b776d, SUC_lexicon_tags_15 },
    { 0xc7b776e, SUC_lexicon_tags_29 },
    { 0xbffb776f, SUC_lexicon_tags_28 },
    { 0x8f9b776e, SUC_lexicon_tags_16 },
    { 0x177b7770, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfb7776, SUC_lexicon_tags_32 },
    { 0x797b7777, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x581b777a, SUC_lexicon_tags_9 },
    { 0xbebb777b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b7780, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5b7783, SUC_lexicon_tags_25 },
    { 0x36db7784, SUC_lexicon_tags_17 },
    { 0x7b1b7785, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x51fb7787, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50db7789, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23b778c, SUC_lexicon_tags_15 },
    { 0x4fbb778c, SUC_lexicon_tags_27 },
    { 0xf9db778e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b7793, SUC_lexicon_tags_47 },
    { 0x8bbb7794, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31bb7796, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x373b7798, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x425b779a, SUC_lexicon_tags_14 },
    { 0xa75b779a, SUC_lexicon_tags_37 },
    { 0x449b779c, SUC_lexicon_tags_37 },
    { 0x1d3b779d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48fb77a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19b77a7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b77aa, SUC_lexicon_tags_21 },
    { 0x8bfb77ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf41b77ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd7b77af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5b77b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fb77bf, SUC_lexicon_tags_60 },
    { 0x355b77c0, SUC_lexicon_tags_12 },
    { 0x7b3b77c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf57b77c3, SUC_lexicon_tags_43 },
    { 0x195b77c3, SUC_lexicon_tags_49 },
    { 0x49fb77c5, SUC_lexicon_tags_160 },
    { 0xe19b77c6, SUC_lexicon_tags_47 },
    { 0x20fb77c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97b77cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fb77d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629b77d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x191b77db, SUC_lexicon_tags_17 },
    { 0x22fb77db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07b77e2, SUC_lexicon_tags_31 },
    { 0x501b77e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29db77ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9b77ed, SUC_lexicon_tags_63 },
    { 0x6c9b77ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51db77f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4fb77f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905b77fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831b7802, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x845b7804, SUC_lexicon_tags_1 },
    { 0x4bb7804, SUC_lexicon_tags_37 },
    { 0xa91b7805, SUC_lexicon_tags_12 },
    { 0x83fb7804, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1b7811, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5b7816, SUC_lexicon_tags_5 },
    { 0xe79b7816, SUC_lexicon_tags_16 },
    { 0xed1b7818, SUC_lexicon_tags_10 },
    { 0x3cdb7817, SUC_lexicon_tags_42 },
    { 0x7c1b7817, SUC_lexicon_tags_17 },
    { 0xcc7b7816, SUC_lexicon_tags_116 },
    { 0xe6fb7819, SUC_lexicon_tags_9 },
    { 0xf83b781a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbb7822, SUC_lexicon_tags_70 },
    { 0x3bbb7822, SUC_lexicon_tags_2 },
    { 0x569b7823, SUC_lexicon_tags_29 },
    { 0xca3b7825, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3d9b7827, SUC_lexicon_tags_12 },
    { 0xe91b7828, SUC_lexicon_tags_25 },
    { 0xb31b7829, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x923b782c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23b782f, SUC_lexicon_tags_1 },
    { 0x349b7830, SUC_lexicon_tags_33 },
    { 0x6a1b7830, SUC_lexicon_tags_19 },
    { 0x75db7832, SUC_lexicon_tags_43 },
    { 0x14db7833, SUC_lexicon_tags_30 },
    { 0x7bb7833, SUC_lexicon_tags_37 },
    { 0x747b7831, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x993b7838, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77b783e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x61fb7840, SUC_lexicon_tags_43 },
    { 0xc97b7841, SUC_lexicon_tags_33 },
    { 0x1b1b7842, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19b7847, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bb784c, SUC_lexicon_tags_26 },
    { 0x285b784d, SUC_lexicon_tags_2 },
    { 0x643b784e, SUC_lexicon_tags_10 },
    { 0x71fb784f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bb785d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27bb785f, SUC_lexicon_tags_7 },
    { 0x749b7860, SUC_lexicon_tags_32 },
    { 0xdffb7861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7db7863, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x989b7866, SUC_lexicon_tags_14 },
    { 0xba3b7866, SUC_lexicon_tags_17 },
    { 0x365b7868, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bb786d, SUC_lexicon_tags_16 },
    { 0xef9b786e, SUC_lexicon_tags_7 },
    { 0xaa1b786e, SUC_lexicon_tags_37 },
    { 0xf1db786d, SUC_lexicon_tags_11 },
    { 0x5f7b7870, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27db7879, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x89db787c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371b7883, SUC_lexicon_tags_0 },
    { 0xb71b7884, SUC_lexicon_tags_11 },
    { 0xffdb7885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93b788b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdb788d, SUC_lexicon_tags_7 },
    { 0x411b788e, SUC_lexicon_tags_6 },
    { 0xc75b788f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b7892, SUC_lexicon_tags_17 },
    { 0x475b7892, SUC_lexicon_tags_45 },
    { 0x1b7893, SUC_lexicon_tags_45 },
    { 0x43fb7895, SUC_lexicon_tags_49 },
    { 0xc8db7896, SUC_lexicon_tags_12 },
    { 0x265b7895, SUC_lexicon_tags_30 },
    { 0x7b5b7892, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9b789b, SUC_lexicon_tags_3 },
    { 0x353b789c, SUC_lexicon_tags_7 },
    { 0xe57b789d, SUC_lexicon_tags_87 },
    { 0xbdfb789c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedb78a3, SUC_lexicon_tags_21 },
    { 0xe5db78a4, SUC_lexicon_tags_17 },
    { 0x749b78a5, SUC_lexicon_tags_7 },
    { 0xa6db78a3, SUC_lexicon_tags_7 },
    { 0x7cfb78a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bb78aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9b78ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x487b78af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7b78b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6afb78b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb78ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc9b78bc, SUC_lexicon_tags_37 },
    { 0xec7b78bd, SUC_lexicon_tags_25 },
    { 0x261b78bc, SUC_lexicon_tags_47 },
    { 0x10fb78be, SUC_lexicon_tags_47 },
    { 0xd73b78bf, SUC_lexicon_tags_19 },
    { 0x3bb78c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x335b78c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809b78ca, SUC_lexicon_tags_7 },
    { 0xfdbb78cb, SUC_lexicon_tags_68 },
    { 0xcc5b78cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb93b78ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd5fb78d0, SUC_lexicon_tags_33 },
    { 0xbbbb78d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51b78d4, SUC_lexicon_tags_6 },
    { 0xdfdb78d4, SUC_lexicon_tags_3 },
    { 0x7d1b78d6, SUC_lexicon_tags_37 },
    { 0x1a5b78d7, SUC_lexicon_tags_43 },
    { 0xc43b78d4, SUC_lexicon_tags_30 },
    { 0xe09b78d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7b78dd, SUC_lexicon_tags_13 },
    { 0xbf5b78dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5b78e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x875b78e5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887b78eb, SUC_lexicon_tags_2 },
    { 0xbcbb78ec, SUC_lexicon_tags_17 },
    { 0xe29b78eb, SUC_lexicon_tags_16 },
    { 0x2bbb78ed, SUC_lexicon_tags_37 },
    { 0xe9b78ef, SUC_lexicon_tags_39 },
    { 0x925b78ee, SUC_lexicon_tags_17 },
    { 0xcfbb78eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829b78fa, SUC_lexicon_tags_0 },
    { 0x517b78fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129b7901, SUC_lexicon_tags_6 },
    { 0xfa1b7901, SUC_lexicon_tags_25 },
    { 0x52db7901, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21b790a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbb790d, SUC_lexicon_tags_8 },
    { 0xca5b790d, SUC_lexicon_tags_27 },
    { 0x53db790f, SUC_lexicon_tags_23 },
    { 0x811b7910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75b7914, SUC_lexicon_tags_15 },
    { 0xcf7b7915, SUC_lexicon_tags_19 },
    { 0xc1fb7916, SUC_lexicon_tags_12 },
    { 0x763b7916, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161b791b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf2fb791d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbb7920, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9b7928, SUC_lexicon_tags_32 },
    { 0xa9b7929, SUC_lexicon_tags_17 },
    { 0xc4fb792a, SUC_lexicon_tags_4 },
    { 0x8d5b792b, SUC_lexicon_tags_7 },
    { 0xab5b792a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9b792f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbb7934, SUC_lexicon_tags_12 },
    { 0xf69b7935, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6bb7937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7dbb7939, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7b793c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd21b793e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99b7949, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29b7954, SUC_lexicon_tags_4 },
    { 0x617b7954, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc39b7957, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bb795c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7abb795e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2e3b7960, SUC_lexicon_tags_8 },
    { 0x1e7b7961, SUC_lexicon_tags_1 },
    { 0x6db7960, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x715b7965, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7969, SUC_lexicon_tags_49 },
    { 0x5fdb7969, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfb796e, SUC_lexicon_tags_12 },
    { 0x8dbb796f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe6db7971, SUC_lexicon_tags_14 },
    { 0x5b3b7972, SUC_lexicon_tags_66 },
    { 0x8b3b7971, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbcdb7975, SUC_lexicon_tags_47 },
    { 0x827b7975, SUC_lexicon_tags_77 },
    { 0x3bdb7977, SUC_lexicon_tags_19 },
    { 0x679b7978, SUC_lexicon_tags_6 },
    { 0x4a7b7977, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47b797d, SUC_lexicon_tags_38 },
    { 0x223b797d, SUC_lexicon_tags_30 },
    { 0x525b797f, SUC_lexicon_tags_70 },
    { 0x9d9b7980, SUC_lexicon_tags_25 },
    { 0x2e1b797d, SUC_lexicon_tags_15 },
    { 0x1c5b7981, SUC_lexicon_tags_2 },
    { 0xac9b7983, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xce9b7985, SUC_lexicon_tags_11 },
    { 0x571b7986, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x961b7988, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdb7990, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x471b7992, SUC_lexicon_tags_6 },
    { 0x17db7992, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43db7999, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3b799c, SUC_lexicon_tags_63 },
    { 0x4f7b799c, SUC_lexicon_tags_15 },
    { 0x517b799e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x603b79a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357b79a7, SUC_lexicon_tags_11 },
    { 0xa6db79a8, SUC_lexicon_tags_1 },
    { 0x237b79a9, SUC_lexicon_tags_56 },
    { 0x9cdb79a9, SUC_lexicon_tags_46 },
    { 0xddbb79ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3db79ae, SUC_lexicon_tags_17 },
    { 0xc5bb79af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45bb79b1, SUC_lexicon_tags_65 },
    { 0xd2db79b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fb79b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbbfb79b9, SUC_lexicon_tags_79 },
    { 0xd03b79b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99b79be, SUC_lexicon_tags_25 },
    { 0x853b79bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9b79c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa7b79c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfb79cc, SUC_lexicon_tags_82 },
    { 0x7c5b79cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797b79d1, SUC_lexicon_tags_15 },
    { 0x365b79d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19fb79d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xad7b79d6, SUC_lexicon_tags_12 },
    { 0x2fb79d7, SUC_lexicon_tags_28 },
    { 0x7c9b79d6, SUC_lexicon_tags_47 },
    { 0xa9bb79d9, SUC_lexicon_tags_5 },
    { 0xe0bb79da, SUC_lexicon_tags_490 },
    { 0xb4db79da, SUC_lexicon_tags_30 },
    { 0x22db79dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fb79df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x65b79e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6edb79e4, SUC_lexicon_tags_11 },
    { 0x3d9b79e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3b79e8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xba1b79ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667b79ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59b79f1, SUC_lexicon_tags_11 },
    { 0xf07b79f1, SUC_lexicon_tags_37 },
    { 0xd95b79f3, SUC_lexicon_tags_47 },
    { 0x62bb79f4, SUC_lexicon_tags_5 },
    { 0x79db79f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x841b79f8, SUC_lexicon_tags_17 },
    { 0x1c9b79f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x297b79fb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5b7a03, SUC_lexicon_tags_47 },
    { 0xe53b7a03, SUC_lexicon_tags_9 },
    { 0x95bb7a05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bb7a0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9b7a0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bb7a12, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1fbb7a14, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x515b7a16, SUC_lexicon_tags_65 },
    { 0x3d5b7a17, SUC_lexicon_tags_0 },
    { 0x35db7a18, SUC_lexicon_tags_15 },
    { 0x1cfb7a19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93b7a1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5b7a20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fb7a25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x83b7a27, SUC_lexicon_tags_7 },
    { 0x985b7a28, SUC_lexicon_tags_1 },
    { 0xc1fb7a29, SUC_lexicon_tags_27 },
    { 0x2ebb7a2a, SUC_lexicon_tags_61 },
    { 0xe77b7a2a, SUC_lexicon_tags_4 },
    { 0x687b7a29, SUC_lexicon_tags_7 },
    { 0xdc9b7a2d, SUC_lexicon_tags_33 },
    { 0xe8bb7a27, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x945b7a30, SUC_lexicon_tags_53 },
    { 0x91bb7a31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fb7a36, SUC_lexicon_tags_26 },
    { 0x813b7a37, SUC_lexicon_tags_27 },
    { 0x577b7a36, SUC_lexicon_tags_14 },
    { 0xfc3b7a36, SUC_lexicon_tags_57 },
    { 0x935b7a38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fb7a3e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5b9b7a40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597b7a45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3b7a49, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79b7a4c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1b7a4f, SUC_lexicon_tags_38 },
    { 0x501b7a4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b7a5b, SUC_lexicon_tags_7 },
    { 0x125b7a5c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x113b7a5e, SUC_lexicon_tags_25 },
    { 0xcb9b7a5f, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77b7a63, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7e3b7a65, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x873b7a68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x307b7a6a, SUC_lexicon_tags_12 },
    { 0x3f7b7a6b, SUC_lexicon_tags_0 },
    { 0x23fb7a6b, SUC_lexicon_tags_37 },
    { 0x1e7b7a6d, SUC_lexicon_tags_11 },
    { 0x627b7a6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5b7a72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3b7a76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee5b7a78, SUC_lexicon_tags_37 },
    { 0x87fb7a78, SUC_lexicon_tags_8 },
    { 0xad9b7a79, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5b7a80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b5b7a82, SUC_lexicon_tags_17 },
    { 0x77db7a83, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6ebb7a85, SUC_lexicon_tags_30 },
    { 0x865b7a85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71b7a8a, SUC_lexicon_tags_46 },
    { 0x777b7a8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97b7a91, SUC_lexicon_tags_0 },
    { 0xbf7b7a92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83b7a98, SUC_lexicon_tags_25 },
    { 0xb61b7a98, SUC_lexicon_tags_28 },
    { 0x651b7a9a, SUC_lexicon_tags_15 },
    { 0x405b7a99, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3c9b7a9d, SUC_lexicon_tags_60 },
    { 0xb7fb7a9d, SUC_lexicon_tags_2 },
    { 0x717b7a9f, SUC_lexicon_tags_4 },
    { 0x179b7aa0, SUC_lexicon_tags_12 },
    { 0xf01b7aa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd2fb7aa3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361b7aa9, SUC_lexicon_tags_11 },
    { 0xf4db7aaa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19b7aaf, SUC_lexicon_tags_7 },
    { 0x23b7ab0, SUC_lexicon_tags_160 },
    { 0x51bb7ab1, SUC_lexicon_tags_11 },
    { 0x5a7b7ab2, SUC_lexicon_tags_65 },
    { 0x3ffb7ab2, SUC_lexicon_tags_10 },
    { 0x4ebb7ab4, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0x6a3b7ab6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x505b7ab8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7abc, SUC_lexicon_tags_10 },
    { 0x7b3b7abd, SUC_lexicon_tags_37 },
    { 0x9d3b7abc, SUC_lexicon_tags_14 },
    { 0xe7b7abd, SUC_lexicon_tags_43 },
    { 0x491b7abf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9f5b7ac2, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x851b7ac4, SUC_lexicon_tags_37 },
    { 0xa2db7ac4, SUC_lexicon_tags_14 },
    { 0xfddb7ac6, SUC_lexicon_tags_28 },
    { 0x797b7ac6, SUC_lexicon_tags_57 },
    { 0xea9b7ac5, SUC_lexicon_tags_53 },
    { 0xfabb7ac9, SUC_lexicon_tags_31 },
    { 0x24fb7ac9, SUC_lexicon_tags_10 },
    { 0x865b7ac9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc71b7acd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x953b7acf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x351b7ad1, SUC_lexicon_tags_38 },
    { 0xdf1b7ad2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbb7ad5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bb7ad8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfbb7ada, SUC_lexicon_tags_1 },
    { 0xab1b7ada, SUC_lexicon_tags_8 },
    { 0xc23b7adc, SUC_lexicon_tags_1 },
    { 0xbb3b7add, SUC_lexicon_tags_9 },
    { 0x421b7adc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9b7ae2, SUC_lexicon_tags_11 },
    { 0xe0fb7ae2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9b7aea, SUC_lexicon_tags_17 },
    { 0xe7db7aeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e5b7aed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bb7af2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efb7af7, SUC_lexicon_tags_28 },
    { 0xd73b7af7, SUC_lexicon_tags_25 },
    { 0xc29b7af9, SUC_lexicon_tags_27 },
    { 0xd33b7afa, SUC_lexicon_tags_136 },
    { 0x4d5b7af9, SUC_lexicon_tags_51 },
    { 0xb7b7af8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x613b7afe, SUC_lexicon_tags_4 },
    { 0x15b7aff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6db7b02, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd73b7b04, SUC_lexicon_tags_26 },
    { 0x1db7b05, SUC_lexicon_tags_6 },
    { 0x46bb7b06, SUC_lexicon_tags_11 },
    { 0x1dbb7b05, SUC_lexicon_tags_11 },
    { 0x213b7b05, SUC_lexicon_tags_11 },
    { 0x3bbb7b09, SUC_lexicon_tags_8 },
    { 0x533b7b0a, SUC_lexicon_tags_25 },
    { 0x983b7b0b, SUC_lexicon_tags_14 },
    { 0x561b7b09, SUC_lexicon_tags_14 },
    { 0x65b7b0b, SUC_lexicon_tags_33 },
    { 0x21bb7b0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25b7b11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25fb7b13, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xa57b7b15, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63b7b19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141b7b1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x473b7b21, SUC_lexicon_tags_11 },
    { 0xb8db7b21, SUC_lexicon_tags_2 },
    { 0x2c3b7b23, SUC_lexicon_tags_39 },
    { 0xc8fb7b21, SUC_lexicon_tags_44 },
    { 0x165b7b21, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb9b7b27, SUC_lexicon_tags_1 },
    { 0xe3bb7b27, SUC_lexicon_tags_26 },
    { 0xa23b7b29, SUC_lexicon_tags_48 },
    { 0x82fb7b27, SUC_lexicon_tags_25 },
    { 0x2e5b7b27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1db7b2e, SUC_lexicon_tags_15 },
    { 0x855b7b2e, SUC_lexicon_tags_33 },
    { 0xe33b7b2f, SUC_lexicon_tags_14 },
    { 0xa27b7b2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x819b7b34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443b7b3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x851b7b40, SUC_lexicon_tags_11 },
    { 0x81bb7b41, SUC_lexicon_tags_79 },
    { 0x661b7b41, SUC_lexicon_tags_23 },
    { 0xb09b7b41, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdb7b47, SUC_lexicon_tags_37 },
    { 0x67b7b48, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbb7b4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87b7b4f, SUC_lexicon_tags_91 },
    { 0x1b3b7b50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x70db7b53, SUC_lexicon_tags_1 },
    { 0xeb7b7b54, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5e7b7b56, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699b7b5a, SUC_lexicon_tags_37 },
    { 0xae1b7b5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x747b7b5d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4bbb7b5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797b7b67, SUC_lexicon_tags_15 },
    { 0xf73b7b67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x989b7b6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdb7b6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14fb7b70, SUC_lexicon_tags_12 },
    { 0xf71b7b70, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc7bb7b73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9b7b75, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x163b7b77, SUC_lexicon_tags_1 },
    { 0x533b7b77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95b7b80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58bb7b82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x597b7b84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7b7b8a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673b7b8e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbb7b92, SUC_lexicon_tags_8 },
    { 0x201b7b93, SUC_lexicon_tags_15 },
    { 0x34db7b94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29b7b98, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf2db7b9a, SUC_lexicon_tags_16 },
    { 0x365b7b9b, SUC_lexicon_tags_27 },
    { 0x9d3b7b9c, SUC_lexicon_tags_31 },
    { 0x4f9b7b9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335b7ba2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3b7ba5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9b7bb0, SUC_lexicon_tags_19 },
    { 0xd21b7bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddb7bb9, SUC_lexicon_tags_25 },
    { 0xfd7b7bba, SUC_lexicon_tags_55 },
    { 0xa83b7bbb, SUC_lexicon_tags_7 },
    { 0x37bb7bbc, SUC_lexicon_tags_19 },
    { 0x967b7bbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe77b7bbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee9b7bc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x783b7bc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc77b7bc5, SUC_lexicon_tags_29 },
    { 0xfe3b7bc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bb7bcb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x601b7bcd, SUC_lexicon_tags_37 },
    { 0x1f9b7bcd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51b7bd3, SUC_lexicon_tags_32 },
    { 0xce1b7bd4, SUC_lexicon_tags_58 },
    { 0x59db7bd5, SUC_lexicon_tags_70 },
    { 0x4a3b7bd6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3b7bd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663b7bdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7b7be0, SUC_lexicon_tags_1 },
    { 0xe51b7be0, SUC_lexicon_tags_39 },
    { 0x84db7be1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d7b7be4, SUC_lexicon_tags_11 },
    { 0xc4bb7be5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45b7beb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44db7bed, SUC_lexicon_tags_63 },
    { 0xd85b7bee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdb7bf1, SUC_lexicon_tags_26 },
    { 0x7fb7bf1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa6fb7bf4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9b7bf7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bb7bfe, SUC_lexicon_tags_14 },
    { 0xebdb7bff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fb7c02, SUC_lexicon_tags_15 },
    { 0x9ddb7c03, SUC_lexicon_tags_16 },
    { 0x8bdb7c03, SUC_lexicon_tags_43 },
    { 0xae1b7c04, SUC_lexicon_tags_38 },
    { 0x52fb7c06, SUC_lexicon_tags_49 },
    { 0xefdb7c05, SUC_lexicon_tags_23 },
    { 0xc6bb7c08, SUC_lexicon_tags_43 },
    { 0x477b7c09, SUC_lexicon_tags_7 },
    { 0x59db7c03, SUC_lexicon_tags_23 },
    { 0x6a9b7c0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfb7c12, SUC_lexicon_tags_18 },
    { 0x8a9b7c12, SUC_lexicon_tags_17 },
    { 0x9f1b7c14, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19b7c17, SUC_lexicon_tags_12 },
    { 0xcfb7c17, SUC_lexicon_tags_27 },
    { 0x539b7c18, SUC_lexicon_tags_12 },
    { 0x9abb7c1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13b7c1d, SUC_lexicon_tags_280 },
    { 0x3dfb7c1e, SUC_lexicon_tags_28 },
    { 0x6e1b7c1e, SUC_lexicon_tags_15 },
    { 0xf11b7c1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x225b7c22, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9b7c28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1b7c33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fb7c38, SUC_lexicon_tags_23 },
    { 0x719b7c38, SUC_lexicon_tags_49 },
    { 0x7fbb7c38, SUC_lexicon_tags_27 },
    { 0xe0bb7c3b, SUC_lexicon_tags_17 },
    { 0x607b7c3c, SUC_lexicon_tags_33 },
    { 0xfb5b7c3d, SUC_lexicon_tags_19 },
    { 0xc55b7c3c, SUC_lexicon_tags_104 },
    { 0x457b7c3d, SUC_lexicon_tags_594 },
    { 0x6b9b7c40, SUC_lexicon_tags_25 },
    { 0xd2db7c3c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb7c45, SUC_lexicon_tags_46 },
    { 0x43b7c46, SUC_lexicon_tags_10 },
    { 0xd9db7c47, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fb7c4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa83b7c4c, SUC_lexicon_tags_7 },
    { 0x25db7c4d, SUC_lexicon_tags_2 },
    { 0x8a5b7c4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27b7c52, SUC_lexicon_tags_2 },
    { 0xcb3b7c53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fb7c56, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b7c59, SUC_lexicon_tags_7 },
    { 0xd2bb7c5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803b7c5e, SUC_lexicon_tags_60 },
    { 0x209b7c5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7bbb7c61, SUC_lexicon_tags_2 },
    { 0xf89b7c62, SUC_lexicon_tags_23 },
    { 0x76bb7c63, SUC_lexicon_tags_17 },
    { 0x129b7c62, SUC_lexicon_tags_29 },
    { 0xcb9b7c64, SUC_lexicon_tags_118 },
    { 0x991b7c66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d5b7c68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x935b7c6a, SUC_lexicon_tags_15 },
    { 0x683b7c6b, SUC_lexicon_tags_6 },
    { 0xab9b7c6c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5b7c72, SUC_lexicon_tags_25 },
    { 0x2abb7c72, SUC_lexicon_tags_25 },
    { 0xff7b7c74, SUC_lexicon_tags_48 },
    { 0xc71b7c75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fb7c84, SUC_lexicon_tags_17 },
    { 0xbf3b7c85, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x985b7c88, SUC_lexicon_tags_11 },
    { 0x44fb7c89, SUC_lexicon_tags_37 },
    { 0x10db7c8a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x74db7c8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd99b7c8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbb7c94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfb7c99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5b7ca2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23b7ca6, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9b7ca9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb4db7cab, SUC_lexicon_tags_28 },
    { 0x9b7b7cac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88db7caf, SUC_lexicon_tags_595 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649b7cb4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1a7b7cb6, SUC_lexicon_tags_38 },
    { 0x6d9b7cb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d7b7cb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbb7cbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x97fb7cc1, SUC_lexicon_tags_48 },
    { 0xb1fb7cc1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x481b7cc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefb7cc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1abb7ccb, SUC_lexicon_tags_38 },
    { 0xb79b7ccc, SUC_lexicon_tags_12 },
    { 0x3edb7ccd, SUC_lexicon_tags_26 },
    { 0xb8db7ccd, SUC_lexicon_tags_68 },
    { 0xbc7b7cce, SUC_lexicon_tags_37 },
    { 0x6b3b7cce, SUC_lexicon_tags_7 },
    { 0x59db7cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267b7cd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5b7cde, SUC_lexicon_tags_43 },
    { 0x19fb7cdf, SUC_lexicon_tags_17 },
    { 0xdebb7ce0, SUC_lexicon_tags_66 },
    { 0x3f1b7ce1, SUC_lexicon_tags_47 },
    { 0xffb7ce0, SUC_lexicon_tags_49 },
    { 0x525b7cdf, SUC_lexicon_tags_207 },
    { 0xec7b7cdf, SUC_lexicon_tags_17 },
    { 0xe09b7ce5, SUC_lexicon_tags_27 },
    { 0x393b7ce5, SUC_lexicon_tags_37 },
    { 0xb0db7ce7, SUC_lexicon_tags_2 },
    { 0xf65b7ce8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3b7cea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839b7cef, SUC_lexicon_tags_27 },
    { 0x4f7b7cef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa25b7cf2, SUC_lexicon_tags_8 },
    { 0x75db7cf3, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0xe7b7cf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43db7cf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x34db7cfa, SUC_lexicon_tags_0 },
    { 0xaf3b7cfb, SUC_lexicon_tags_37 },
    { 0x64bb7cfc, SUC_lexicon_tags_1 },
    { 0xd41b7cfd, SUC_lexicon_tags_37 },
    { 0x5e5b7cfc, SUC_lexicon_tags_1 },
    { 0x42db7cfe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe11b7d01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765b7d07, SUC_lexicon_tags_1 },
    { 0xeb3b7d07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8bb7d0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66bb7d0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bb7d10, SUC_lexicon_tags_11 },
    { 0x9c5b7d10, SUC_lexicon_tags_12 },
    { 0x4a3b7d10, SUC_lexicon_tags_11 },
    { 0x9f9b7d13, SUC_lexicon_tags_27 },
    { 0x535b7d14, SUC_lexicon_tags_37 },
    { 0xf0db7d14, SUC_lexicon_tags_37 },
    { 0xed3b7d13, SUC_lexicon_tags_1 },
    { 0x56db7d16, SUC_lexicon_tags_31 },
    { 0x475b7d16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95b7d1e, SUC_lexicon_tags_45 },
    { 0x577b7d1f, SUC_lexicon_tags_49 },
    { 0xc4bb7d20, SUC_lexicon_tags_55 },
    { 0x97db7d21, SUC_lexicon_tags_0 },
    { 0xcf1b7d22, SUC_lexicon_tags_1 },
    { 0xb09b7d22, SUC_lexicon_tags_14 },
    { 0x853b7d24, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x719b7d26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec1b7d28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955b7d2e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x88bb7d31, SUC_lexicon_tags_0 },
    { 0xfdbb7d31, SUC_lexicon_tags_19 },
    { 0x511b7d32, SUC_lexicon_tags_46 },
    { 0xbfbb7d34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe1db7d36, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1b7d39, SUC_lexicon_tags_2 },
    { 0x31fb7d39, SUC_lexicon_tags_19 },
    { 0x98db7d3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44bb7d3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27fb7d3f, SUC_lexicon_tags_37 },
    { 0x51bb7d40, SUC_lexicon_tags_31 },
    { 0x369b7d41, SUC_lexicon_tags_12 },
    { 0xf89b7d42, SUC_lexicon_tags_25 },
    { 0x243b7d43, SUC_lexicon_tags_21 },
    { 0xfd9b7d44, SUC_lexicon_tags_1 },
    { 0x4a7b7d45, SUC_lexicon_tags_38 },
    { 0xb7fb7d42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615b7d51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b7d54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x747b7d56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db7d5a, SUC_lexicon_tags_25 },
    { 0x6fbb7d5b, SUC_lexicon_tags_17 },
    { 0xcf3b7d5c, SUC_lexicon_tags_21 },
    { 0x5cbb7d5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1b7d61, SUC_lexicon_tags_37 },
    { 0xa99b7d62, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfb7d66, SUC_lexicon_tags_30 },
    { 0x317b7d67, SUC_lexicon_tags_2 },
    { 0xa25b7d68, SUC_lexicon_tags_13 },
    { 0xbd7b7d67, SUC_lexicon_tags_17 },
    { 0x157b7d66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x459b7d6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6db7d6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859b7d75, SUC_lexicon_tags_32 },
    { 0xf09b7d76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedb7d7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x829b7d7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1cbb7d80, SUC_lexicon_tags_45 },
    { 0xe75b7d81, SUC_lexicon_tags_7 },
    { 0x1afb7d82, SUC_lexicon_tags_7 },
    { 0x907b7d83, SUC_lexicon_tags_25 },
    { 0x901b7d84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51b7d87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x741b7d8a, SUC_lexicon_tags_3 },
    { 0x107b7d8a, SUC_lexicon_tags_7 },
    { 0xc7db7d8c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4db7d90, SUC_lexicon_tags_37 },
    { 0xf3b7d90, SUC_lexicon_tags_82 },
    { 0x87fb7d92, SUC_lexicon_tags_19 },
    { 0xc55b7d90, SUC_lexicon_tags_10 },
    { 0xfbb7d94, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781b7d98, SUC_lexicon_tags_17 },
    { 0xce1b7d99, SUC_lexicon_tags_17 },
    { 0x53b7d9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83b7da2, SUC_lexicon_tags_66 },
    { 0x755b7da3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809b7dad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xda9b7daf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3b7db5, SUC_lexicon_tags_10 },
    { 0xb65b7db6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2db7dbb, SUC_lexicon_tags_31 },
    { 0x661b7dbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317b7dc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bb7dc3, SUC_lexicon_tags_17 },
    { 0xa27b7dc4, SUC_lexicon_tags_27 },
    { 0xc9b7dc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebb7dca, SUC_lexicon_tags_5 },
    { 0xbafb7dca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbdb7dcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x769b7dcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93b7dd1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0db7dd4, SUC_lexicon_tags_37 },
    { 0x677b7dd5, SUC_lexicon_tags_16 },
    { 0x59fb7dd6, SUC_lexicon_tags_6 },
    { 0x52fb7dd7, SUC_lexicon_tags_47 },
    { 0x36fb7dd7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x993b7ddb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdb7de0, SUC_lexicon_tags_26 },
    { 0x2fb7de1, SUC_lexicon_tags_56 },
    { 0x23b7de0, SUC_lexicon_tags_4 },
    { 0x859b7de3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6db7de7, SUC_lexicon_tags_8 },
    { 0x92bb7de8, SUC_lexicon_tags_1 },
    { 0x88fb7de7, SUC_lexicon_tags_9 },
    { 0x5bbb7dea, SUC_lexicon_tags_14 },
    { 0x84fb7deb, SUC_lexicon_tags_11 },
    { 0xb39b7deb, SUC_lexicon_tags_17 },
    { 0x793b7de7, SUC_lexicon_tags_14 },
    { 0x581b7ded, SUC_lexicon_tags_15 },
    { 0x55b7de8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9b7df4, SUC_lexicon_tags_17 },
    { 0x677b7df5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc57b7df7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517b7dfb, SUC_lexicon_tags_9 },
    { 0x62bb7dfc, SUC_lexicon_tags_1 },
    { 0x847b7dfd, SUC_lexicon_tags_25 },
    { 0x1abb7dfd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9b7e01, SUC_lexicon_tags_34 },
    { 0x33b7e02, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1b7e06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13b7e0e, SUC_lexicon_tags_63 },
    { 0xebfb7e0e, SUC_lexicon_tags_12 },
    { 0xca5b7e10, SUC_lexicon_tags_4 },
    { 0xc67b7e0f, SUC_lexicon_tags_1 },
    { 0x165b7e12, SUC_lexicon_tags_27 },
    { 0x273b7e12, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1b7e1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8bfb7e1e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd33b7e20, SUC_lexicon_tags_11 },
    { 0x975b7e21, SUC_lexicon_tags_3 },
    { 0xfd3b7e22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33b7e26, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x74fb7e28, SUC_lexicon_tags_11 },
    { 0x2e9b7e29, SUC_lexicon_tags_25 },
    { 0xb3bb7e2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x461b7e2c, SUC_lexicon_tags_43 },
    { 0xa87b7e2c, SUC_lexicon_tags_28 },
    { 0x53db7e2d, SUC_lexicon_tags_10 },
    { 0x7e9b7e2f, SUC_lexicon_tags_92 },
    { 0x4c5b7e2f, SUC_lexicon_tags_25 },
    { 0xf3fb7e2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadb7e37, SUC_lexicon_tags_25 },
    { 0x791b7e38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf45b7e3a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47b7e40, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc39b7e42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95b7e45, SUC_lexicon_tags_30 },
    { 0x5d9b7e45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9b7e4a, SUC_lexicon_tags_66 },
    { 0xaefb7e4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475b7e50, SUC_lexicon_tags_37 },
    { 0x9b7e51, SUC_lexicon_tags_28 },
    { 0xcfbb7e52, SUC_lexicon_tags_33 },
    { 0x8f3b7e51, SUC_lexicon_tags_11 },
    { 0x937b7e52, SUC_lexicon_tags_37 },
    { 0x64db7e51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31b7e5a, SUC_lexicon_tags_25 },
    { 0xe93b7e5b, SUC_lexicon_tags_11 },
    { 0xfcbb7e5c, SUC_lexicon_tags_45 },
    { 0xc85b7e5d, SUC_lexicon_tags_1 },
    { 0x30bb7e5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfb7e61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfb7e64, SUC_lexicon_tags_3 },
    { 0xe8bb7e64, SUC_lexicon_tags_25 },
    { 0x3b1b7e66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x15b7e69, SUC_lexicon_tags_7 },
    { 0xbc9b7e6a, SUC_lexicon_tags_1 },
    { 0xebdb7e6a, SUC_lexicon_tags_17 },
    { 0xb65b7e6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bb7e70, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xafdb7e72, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27db7e78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2db7e7b, SUC_lexicon_tags_60 },
    { 0xeb3b7e7c, SUC_lexicon_tags_1 },
    { 0x209b7e7b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933b7e83, SUC_lexicon_tags_25 },
    { 0x939b7e84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54db7e86, SUC_lexicon_tags_35 },
    { 0xa97b7e87, SUC_lexicon_tags_15 },
    { 0x25fb7e87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9b7e8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53db7e9b, SUC_lexicon_tags_17 },
    { 0x2bbb7e9b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afb7ea7, SUC_lexicon_tags_37 },
    { 0xa2bb7ea7, SUC_lexicon_tags_1 },
    { 0x267b7ea8, SUC_lexicon_tags_2 },
    { 0x7abb7ea8, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x4db7eac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179b7eb2, SUC_lexicon_tags_43 },
    { 0x913b7eb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9b7ebc, SUC_lexicon_tags_0 },
    { 0xda9b7ebc, SUC_lexicon_tags_25 },
    { 0xb85b7ebd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf5b7ec0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x957b7ec2, SUC_lexicon_tags_2 },
    { 0xdcbb7ec3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9b7ec6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b7ec8, SUC_lexicon_tags_7 },
    { 0x127b7ec9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1b7ece, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbf5b7ed0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1b7ed6, SUC_lexicon_tags_9 },
    { 0x729b7ed6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b7edb, SUC_lexicon_tags_60 },
    { 0x57db7edc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1b7ee0, SUC_lexicon_tags_17 },
    { 0x877b7ee1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31b7ee5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db7eeb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x29bb7eed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49b7ef0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7b7ef4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3b7ef9, SUC_lexicon_tags_4 },
    { 0xffbb7efa, SUC_lexicon_tags_25 },
    { 0x141b7efb, SUC_lexicon_tags_27 },
    { 0x35db7ef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdb7eff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd29b7f01, SUC_lexicon_tags_30 },
    { 0xc9fb7f02, SUC_lexicon_tags_25 },
    { 0xa6fb7f02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bdb7f05, SUC_lexicon_tags_21 },
    { 0x3ebb7f05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03b7f0a, SUC_lexicon_tags_64 },
    { 0xecfb7f0b, SUC_lexicon_tags_68 },
    { 0x107b7f0b, SUC_lexicon_tags_1 },
    { 0x739b7f0b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x47b7f0f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99b7f19, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x913b7f1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4bb7f1e, SUC_lexicon_tags_10 },
    { 0x2a1b7f1f, SUC_lexicon_tags_19 },
    { 0x18db7f1f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3b7f23, SUC_lexicon_tags_22 },
    { 0xb59b7f23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fb7f28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7b7f2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fb7f2f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9db7f37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x71bb7f3a, SUC_lexicon_tags_31 },
    { 0x5abb7f3b, SUC_lexicon_tags_37 },
    { 0x1c9b7f3a, SUC_lexicon_tags_33 },
    { 0x8d9b7f3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadfb7f3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6dfb7f41, SUC_lexicon_tags_37 },
    { 0xb87b7f41, SUC_lexicon_tags_33 },
    { 0xbf5b7f43, SUC_lexicon_tags_26 },
    { 0xef9b7f43, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf27b7f46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38db7f4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdb7f4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe29b7f51, SUC_lexicon_tags_11 },
    { 0x7f3b7f52, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc13b7f54, SUC_lexicon_tags_11 },
    { 0xa19b7f55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfb7f59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x789b7f5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdb7f5f, SUC_lexicon_tags_18 },
    { 0x87b7f60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b7f65, SUC_lexicon_tags_37 },
    { 0x399b7f66, SUC_lexicon_tags_7 },
    { 0x391b7f67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x757b7f69, SUC_lexicon_tags_43 },
    { 0xc45b7f6a, SUC_lexicon_tags_11 },
    { 0x1cbb7f6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11b7f6e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309b7f74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fb7f78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd1b7f7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5b7f7e, SUC_lexicon_tags_17 },
    { 0xb3fb7f7f, SUC_lexicon_tags_11 },
    { 0x47fb7f80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fb7f86, SUC_lexicon_tags_17 },
    { 0x881b7f87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce5b7f89, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x93b7f8c, SUC_lexicon_tags_141 },
    { 0xe95b7f8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771b7f95, SUC_lexicon_tags_11 },
    { 0xe47b7f96, SUC_lexicon_tags_92 },
    { 0x749b7f97, SUC_lexicon_tags_49 },
    { 0xecdb7f97, SUC_lexicon_tags_66 },
    { 0xad9b7f98, SUC_lexicon_tags_1 },
    { 0x54fb7f99, SUC_lexicon_tags_2 },
    { 0xf99b7f9a, SUC_lexicon_tags_33 },
    { 0x37b7f98, SUC_lexicon_tags_33 },
    { 0x7fbb7f9d, SUC_lexicon_tags_25 },
    { 0xbc3b7f96, SUC_lexicon_tags_15 },
    { 0x313b7f9f, SUC_lexicon_tags_20 },
    { 0xf63b7f9f, SUC_lexicon_tags_87 },
    { 0xd5fb7fa1, SUC_lexicon_tags_25 },
    { 0xd8bb7fa2, SUC_lexicon_tags_14 },
    { 0x317b7fa3, SUC_lexicon_tags_18 },
    { 0x597b7fa2, SUC_lexicon_tags_47 },
    { 0xc4bb7fa2, SUC_lexicon_tags_11 },
    { 0x1a7b7f96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbb7faa, SUC_lexicon_tags_8 },
    { 0x509b7faa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3b7fba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939b7fbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3b7fc3, SUC_lexicon_tags_7 },
    { 0x2a7b7fc3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bb7fc8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc4fb7fca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7ddb7fcc, SUC_lexicon_tags_37 },
    { 0x5b5b7fcc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847b7fd6, SUC_lexicon_tags_11 },
    { 0x2e3b7fd6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x949b7fd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207b7fde, SUC_lexicon_tags_2 },
    { 0x4bb7fdf, SUC_lexicon_tags_17 },
    { 0x239b7fe0, SUC_lexicon_tags_23 },
    { 0xfe9b7fe1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2db7fe6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21b7fec, SUC_lexicon_tags_38 },
    { 0x627b7fed, SUC_lexicon_tags_37 },
    { 0xe15b7fee, SUC_lexicon_tags_15 },
    { 0x679b7fef, SUC_lexicon_tags_47 },
    { 0x1e7b7ff0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51db7ff2, SUC_lexicon_tags_35 },
    { 0x3f9b7ff3, SUC_lexicon_tags_11 },
    { 0x2adb7ff3, SUC_lexicon_tags_25 },
    { 0xed7b7ff3, SUC_lexicon_tags_2 },
    { 0xcd3b7ff6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47b7ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fb7ffd, SUC_lexicon_tags_39 },
    { 0x92bb7ffd, SUC_lexicon_tags_37 },
    { 0xf93b7ffe, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2c3b8001, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f3b8003, SUC_lexicon_tags_21 },
    { 0x99db8004, SUC_lexicon_tags_28 },
    { 0x531b8004, SUC_lexicon_tags_1 },
    { 0xabfb8004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x627b8009, SUC_lexicon_tags_33 },
    { 0xdf7b8009, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5b8011, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b1b8013, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bb8016, SUC_lexicon_tags_16 },
    { 0xcefb8016, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491b801b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x301b801d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x76fb801f, SUC_lexicon_tags_3 },
    { 0x5a3b801f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfcdb8022, SUC_lexicon_tags_58 },
    { 0xeb1b8023, SUC_lexicon_tags_26 },
    { 0xe8db8022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79b8027, SUC_lexicon_tags_8 },
    { 0x5a7b8028, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6b9b802a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133b8030, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffb8038, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15b8041, SUC_lexicon_tags_81 },
    { 0xdabb8042, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed1b8044, SUC_lexicon_tags_1 },
    { 0xd79b8045, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadb8048, SUC_lexicon_tags_12 },
    { 0xe8db8049, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb1b804b, SUC_lexicon_tags_7 },
    { 0xf27b804c, SUC_lexicon_tags_25 },
    { 0x81bb804d, SUC_lexicon_tags_38 },
    { 0xab3b804c, SUC_lexicon_tags_27 },
    { 0x927b804f, SUC_lexicon_tags_56 },
    { 0x107b8050, SUC_lexicon_tags_37 },
    { 0x145b8051, SUC_lexicon_tags_19 },
    { 0x323b8050, SUC_lexicon_tags_17 },
    { 0xfffb8053, SUC_lexicon_tags_14 },
    { 0xa65b8054, SUC_lexicon_tags_12 },
    { 0xaefb8055, SUC_lexicon_tags_19 },
    { 0xef9b8054, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785b805e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5b8062, SUC_lexicon_tags_1 },
    { 0xe4fb8063, SUC_lexicon_tags_11 },
    { 0xf7b8062, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79b8068, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55b806b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bb806e, SUC_lexicon_tags_15 },
    { 0xa05b806e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23db8071, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7b8076, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdb807a, SUC_lexicon_tags_62 },
    { 0xed3b807b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b807e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9b8081, SUC_lexicon_tags_37 },
    { 0xcd7b8082, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdb8085, SUC_lexicon_tags_17 },
    { 0x699b8085, SUC_lexicon_tags_37 },
    { 0xee7b8087, SUC_lexicon_tags_14 },
    { 0xfa5b8086, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7b808c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ddb808e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x201b8090, SUC_lexicon_tags_49 },
    { 0x1d1b8091, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdb8097, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x891b8099, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797b80a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf71b80a3, SUC_lexicon_tags_37 },
    { 0xb97b80a3, SUC_lexicon_tags_46 },
    { 0x335b80a5, SUC_lexicon_tags_25 },
    { 0xd1bb80a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x837b80a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2edb80ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x665b80ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb5b80b0, SUC_lexicon_tags_45 },
    { 0x355b80b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbb80b6, SUC_lexicon_tags_11 },
    { 0x63fb80b7, SUC_lexicon_tags_7 },
    { 0x153b80b8, SUC_lexicon_tags_9 },
    { 0xde5b80b6, SUC_lexicon_tags_7 },
    { 0x8b3b80ba, SUC_lexicon_tags_25 },
    { 0xdb7b80bb, SUC_lexicon_tags_0 },
    { 0x43fb80bc, SUC_lexicon_tags_1 },
    { 0x2cbb80bd, SUC_lexicon_tags_13 },
    { 0x1abb80bc, SUC_lexicon_tags_6 },
    { 0xdcdb80be, SUC_lexicon_tags_45 },
    { 0xf09b80be, SUC_lexicon_tags_15 },
    { 0x719b80c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbb80c4, SUC_lexicon_tags_107 },
    { 0xfb1b80c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb95b80c7, SUC_lexicon_tags_7 },
    { 0x6db80c8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5b80cd, SUC_lexicon_tags_39 },
    { 0x49db80ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe45b80d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825b80d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fb80db, SUC_lexicon_tags_28 },
    { 0x1e5b80db, SUC_lexicon_tags_37 },
    { 0x967b80db, SUC_lexicon_tags_49 },
    { 0xcc3b80db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x585b80e1, SUC_lexicon_tags_15 },
    { 0x341b80e2, SUC_lexicon_tags_96 },
    { 0x6b7b80e3, SUC_lexicon_tags_12 },
    { 0x8dbb80e4, SUC_lexicon_tags_15 },
    { 0xbc7b80e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1db80e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16db80ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97b80ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813b80f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bb80f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdb80fe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51b8107, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbb810d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877b8111, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1b8115, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x313b8117, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x859b8119, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x33b811b, SUC_lexicon_tags_37 },
    { 0xd17b811c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x539b811e, SUC_lexicon_tags_0 },
    { 0x2bdb811f, SUC_lexicon_tags_23 },
    { 0xfdfb8120, SUC_lexicon_tags_37 },
    { 0xab7b811f, SUC_lexicon_tags_29 },
    { 0x779b8122, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7b8125, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827b812a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b812f, SUC_lexicon_tags_31 },
    { 0x80bb8130, SUC_lexicon_tags_23 },
    { 0xb29b8131, SUC_lexicon_tags_2 },
    { 0x839b8132, SUC_lexicon_tags_11 },
    { 0xc9fb8133, SUC_lexicon_tags_65 },
    { 0x379b8133, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5b813d, SUC_lexicon_tags_27 },
    { 0x137b813d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53db8142, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabb8145, SUC_lexicon_tags_30 },
    { 0x12db8145, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddb8149, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535b814d, SUC_lexicon_tags_2 },
    { 0xc9db814e, SUC_lexicon_tags_31 },
    { 0x47fb814f, SUC_lexicon_tags_1 },
    { 0x2cbb8150, SUC_lexicon_tags_30 },
    { 0xad9b8151, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x949b8154, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879b815a, SUC_lexicon_tags_17 },
    { 0xcc3b815b, SUC_lexicon_tags_49 },
    { 0xd69b815b, SUC_lexicon_tags_2 },
    { 0x827b815d, SUC_lexicon_tags_77 },
    { 0x865b815e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fb8164, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1b816a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x33db816d, SUC_lexicon_tags_37 },
    { 0x91fb816d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999b8173, SUC_lexicon_tags_37 },
    { 0x72db8174, SUC_lexicon_tags_31 },
    { 0x283b8174, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc73b8177, SUC_lexicon_tags_57 },
    { 0xfcdb8177, SUC_lexicon_tags_7 },
    { 0x38fb8178, SUC_lexicon_tags_16 },
    { 0x101b817a, SUC_lexicon_tags_15 },
    { 0xf0fb8178, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15db817d, SUC_lexicon_tags_11 },
    { 0xa55b817e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85b8185, SUC_lexicon_tags_25 },
    { 0xcebb8186, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb07b8188, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfb818b, SUC_lexicon_tags_30 },
    { 0xa1fb818c, SUC_lexicon_tags_68 },
    { 0x2d3b818d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b7b818f, SUC_lexicon_tags_29 },
    { 0x76fb818f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1b8193, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5b8199, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb35b819b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fb81a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3db81a6, SUC_lexicon_tags_17 },
    { 0xb1fb81a6, SUC_lexicon_tags_47 },
    { 0x7b81a7, SUC_lexicon_tags_14 },
    { 0xa3b81a7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1b81b0, SUC_lexicon_tags_1 },
    { 0xbf7b81b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9b81b5, SUC_lexicon_tags_29 },
    { 0x6b7b81b6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x507b81b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bb81c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x431b81c4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29b81c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9bb81ca, SUC_lexicon_tags_2 },
    { 0x427b81ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699b81cf, SUC_lexicon_tags_11 },
    { 0xd71b81cf, SUC_lexicon_tags_170 },
    { 0x781b81d1, SUC_lexicon_tags_37 },
    { 0x4b9b81d2, SUC_lexicon_tags_85 },
    { 0x2ebb81d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5b81d6, SUC_lexicon_tags_21 },
    { 0x857b81d7, SUC_lexicon_tags_26 },
    { 0xcedb81d8, SUC_lexicon_tags_32 },
    { 0xbcdb81d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fb81dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5b81e0, SUC_lexicon_tags_17 },
    { 0x115b81e1, SUC_lexicon_tags_45 },
    { 0xdbfb81e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbb81ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fb81f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967b81fc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1db8200, SUC_lexicon_tags_43 },
    { 0x46fb8200, SUC_lexicon_tags_56 },
    { 0x5ddb8202, SUC_lexicon_tags_11 },
    { 0x3b8201, SUC_lexicon_tags_0 },
    { 0x55fb8201, SUC_lexicon_tags_39 },
    { 0xd81b8205, SUC_lexicon_tags_51 },
    { 0x687b8203, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfb8209, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fb8210, SUC_lexicon_tags_10 },
    { 0x1f7b8211, SUC_lexicon_tags_14 },
    { 0xd03b8212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbb8216, SUC_lexicon_tags_38 },
    { 0x815b8216, SUC_lexicon_tags_39 },
    { 0x267b8217, SUC_lexicon_tags_37 },
    { 0x50bb8219, SUC_lexicon_tags_1186 },
    { 0, NULL },
    { 0x22bb821b, SUC_lexicon_tags_2 },
    { 0x42db821c, SUC_lexicon_tags_43 },
    { 0xe61b821b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bb8223, SUC_lexicon_tags_37 },
    { 0x869b8224, SUC_lexicon_tags_6 },
    { 0x977b8225, SUC_lexicon_tags_14 },
    { 0x67db8225, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x425b8228, SUC_lexicon_tags_25 },
    { 0xc4db8229, SUC_lexicon_tags_17 },
    { 0xfdb8229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1b822e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa9b8230, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fb8233, SUC_lexicon_tags_23 },
    { 0x2ddb8233, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x387b8236, SUC_lexicon_tags_27 },
    { 0xb8bb8236, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdb823d, SUC_lexicon_tags_12 },
    { 0xbc1b823e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bb8241, SUC_lexicon_tags_14 },
    { 0xc13b8241, SUC_lexicon_tags_114 },
    { 0xbddb8243, SUC_lexicon_tags_37 },
    { 0x7cfb8241, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5b8247, SUC_lexicon_tags_14 },
    { 0x6b1b8248, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x78db824b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x487b824d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa33b824f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x965b8251, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x761b8253, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59b8258, SUC_lexicon_tags_37 },
    { 0xe85b8259, SUC_lexicon_tags_25 },
    { 0x591b825a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7b8262, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36db8266, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x395b8268, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1b826b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc2bb826d, SUC_lexicon_tags_7 },
    { 0xbbdb826d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221b8272, SUC_lexicon_tags_15 },
    { 0xdbfb8272, SUC_lexicon_tags_18 },
    { 0xc19b8273, SUC_lexicon_tags_28 },
    { 0x49bb8274, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6c7b8277, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329b827e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15b8284, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x51db8286, SUC_lexicon_tags_45 },
    { 0xf09b8286, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bb828b, SUC_lexicon_tags_39 },
    { 0xd23b828c, SUC_lexicon_tags_35 },
    { 0xcb1b828d, SUC_lexicon_tags_84 },
    { 0x57b828c, SUC_lexicon_tags_56 },
    { 0xc23b828f, SUC_lexicon_tags_2 },
    { 0x55bb828f, SUC_lexicon_tags_11 },
    { 0xe71b828b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3b8295, SUC_lexicon_tags_1 },
    { 0xea1b8295, SUC_lexicon_tags_11 },
    { 0x77db8297, SUC_lexicon_tags_25 },
    { 0x81bb8295, SUC_lexicon_tags_47 },
    { 0x7efb8299, SUC_lexicon_tags_37 },
    { 0x765b829a, SUC_lexicon_tags_1 },
    { 0xffb8295, SUC_lexicon_tags_37 },
    { 0xdafb8296, SUC_lexicon_tags_6 },
    { 0xb51b829d, SUC_lexicon_tags_17 },
    { 0x721b829e, SUC_lexicon_tags_25 },
    { 0xdc9b829f, SUC_lexicon_tags_12 },
    { 0x39b829e, SUC_lexicon_tags_31 },
    { 0xc73b829e, SUC_lexicon_tags_43 },
    { 0xf9bb82a2, SUC_lexicon_tags_12 },
    { 0x87bb82a3, SUC_lexicon_tags_37 },
    { 0x6ebb82a4, SUC_lexicon_tags_17 },
    { 0x275b82a5, SUC_lexicon_tags_15 },
    { 0xb6fb82a4, SUC_lexicon_tags_63 },
    { 0xd8bb829b, SUC_lexicon_tags_96 },
    { 0x6cfb82a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fb82ab, SUC_lexicon_tags_12 },
    { 0x353b82ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xde1b82ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5b82b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b82b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4c5b82b7, SUC_lexicon_tags_439 },
    { 0x861b82b7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9b82bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x39db82bf, SUC_lexicon_tags_87 },
    { 0xd53b82bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75b82c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bb82c8, SUC_lexicon_tags_211 },
    { 0xfe9b82c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xce9b82cb, SUC_lexicon_tags_65 },
    { 0xdfb82cb, SUC_lexicon_tags_34 },
    { 0xefdb82cd, SUC_lexicon_tags_7 },
    { 0xb17b82ce, SUC_lexicon_tags_201 },
    { 0x10db82cf, SUC_lexicon_tags_17 },
    { 0x7d9b82cf, SUC_lexicon_tags_37 },
    { 0x647b82cf, SUC_lexicon_tags_0 },
    { 0x529b82d1, SUC_lexicon_tags_37 },
    { 0x6b9b82d0, SUC_lexicon_tags_17 },
    { 0x1a1b82d4, SUC_lexicon_tags_45 },
    { 0x75db82d5, SUC_lexicon_tags_87 },
    { 0x595b82d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37b82d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293b82de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x721b82e1, SUC_lexicon_tags_0 },
    { 0xd7db82e1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfb82e7, SUC_lexicon_tags_19 },
    { 0x67bb82e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9db82eb, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bb82f1, SUC_lexicon_tags_11 },
    { 0xa6bb82f2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xaedb82f5, SUC_lexicon_tags_25 },
    { 0x229b82f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x101b82f8, SUC_lexicon_tags_63 },
    { 0xea5b82f9, SUC_lexicon_tags_11 },
    { 0x425b82f9, SUC_lexicon_tags_11 },
    { 0x209b82fb, SUC_lexicon_tags_11 },
    { 0x7f9b82f8, SUC_lexicon_tags_37 },
    { 0xa9bb82f9, SUC_lexicon_tags_28 },
    { 0x2ddb82fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x917b8300, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6db8309, SUC_lexicon_tags_65 },
    { 0xd91b830a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95b830e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31db8313, SUC_lexicon_tags_14 },
    { 0xd03b8314, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf33b8316, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5b8319, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfb831d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee7b831f, SUC_lexicon_tags_11 },
    { 0x2cbb831f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07b8324, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb9bb8326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bb832c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fb8333, SUC_lexicon_tags_62 },
    { 0x4ffb8333, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bb8338, SUC_lexicon_tags_49 },
    { 0x657b8338, SUC_lexicon_tags_25 },
    { 0xe1fb833a, SUC_lexicon_tags_1 },
    { 0x1edb833b, SUC_lexicon_tags_43 },
    { 0xca7b833c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x869b833e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fb8344, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1b8348, SUC_lexicon_tags_63 },
    { 0x96db8349, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7dbb834b, SUC_lexicon_tags_48 },
    { 0xc9bb834b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551b8351, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db8355, SUC_lexicon_tags_42 },
    { 0xf49b8355, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b8359, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbb835c, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bb8362, SUC_lexicon_tags_549 },
    { 0x75b8363, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbb8369, SUC_lexicon_tags_27 },
    { 0x70fb836a, SUC_lexicon_tags_27 },
    { 0x4d9b836a, SUC_lexicon_tags_17 },
    { 0xe1db8369, SUC_lexicon_tags_9 },
    { 0x6a7b836c, SUC_lexicon_tags_37 },
    { 0xcc1b836e, SUC_lexicon_tags_1 },
    { 0xbf5b836f, SUC_lexicon_tags_47 },
    { 0xbe3b8370, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5b8374, SUC_lexicon_tags_26 },
    { 0xf51b8375, SUC_lexicon_tags_77 },
    { 0x42bb8376, SUC_lexicon_tags_45 },
    { 0xab9b8377, SUC_lexicon_tags_7 },
    { 0xd5db8377, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233b837c, SUC_lexicon_tags_33 },
    { 0xb87b837d, SUC_lexicon_tags_1 },
    { 0xad9b837d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2bb8380, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37b8383, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c9b8385, SUC_lexicon_tags_25 },
    { 0x9d1b8385, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29db838d, SUC_lexicon_tags_9 },
    { 0x723b838e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17b8393, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x359b8395, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95b8399, SUC_lexicon_tags_60 },
    { 0xa09b8399, SUC_lexicon_tags_47 },
    { 0xabfb8399, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xecfb839d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x259b83a0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bb83a6, SUC_lexicon_tags_0 },
    { 0x4afb83a6, SUC_lexicon_tags_11 },
    { 0xd75b83a8, SUC_lexicon_tags_19 },
    { 0x2dfb83a6, SUC_lexicon_tags_27 },
    { 0xb3b83aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x881b83ac, SUC_lexicon_tags_33 },
    { 0x4cdb83ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5b83af, SUC_lexicon_tags_8 },
    { 0x3fbb83b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd71b83b2, SUC_lexicon_tags_46 },
    { 0x7e1b83b3, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x255b83b5, SUC_lexicon_tags_65 },
    { 0x815b83b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae5b83b8, SUC_lexicon_tags_0 },
    { 0x55db83b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5fb83bb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7b83be, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5afb83c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9bb83c2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd1db83c4, SUC_lexicon_tags_45 },
    { 0x8a5b83c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x461b83c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b7b83ca, SUC_lexicon_tags_11 },
    { 0x833b83cb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237b83d0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39b83d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x175b83d8, SUC_lexicon_tags_37 },
    { 0xefbb83d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3b83e0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfb83e8, SUC_lexicon_tags_7 },
    { 0xf45b83e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x407b83ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbdb83ee, SUC_lexicon_tags_1 },
    { 0xd51b83ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67b83f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3b83f9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7e9b83fb, SUC_lexicon_tags_65 },
    { 0x673b83fc, SUC_lexicon_tags_6 },
    { 0x17fb83fd, SUC_lexicon_tags_49 },
    { 0xfcbb83fd, SUC_lexicon_tags_27 },
    { 0xa59b83ff, SUC_lexicon_tags_47 },
    { 0x1db83fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09b8403, SUC_lexicon_tags_66 },
    { 0x91fb8404, SUC_lexicon_tags_16 },
    { 0x715b8405, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfedb8407, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351b840d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13b8410, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x771b8412, SUC_lexicon_tags_12 },
    { 0xd55b8412, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999b8417, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bb841a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafb8420, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131b8424, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68db842a, SUC_lexicon_tags_45 },
    { 0x50fb842b, SUC_lexicon_tags_25 },
    { 0x27fb842c, SUC_lexicon_tags_45 },
    { 0xaf1b842d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe25b842f, SUC_lexicon_tags_2 },
    { 0x635b8430, SUC_lexicon_tags_49 },
    { 0x9d9b8431, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b5b8433, SUC_lexicon_tags_15 },
    { 0x519b8433, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bb8437, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5b843a, SUC_lexicon_tags_33 },
    { 0x2d1b843a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fb8441, SUC_lexicon_tags_26 },
    { 0x98db8442, SUC_lexicon_tags_0 },
    { 0x933b8442, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47db844a, SUC_lexicon_tags_14 },
    { 0x797b844a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1b844e, SUC_lexicon_tags_25 },
    { 0x987b844e, SUC_lexicon_tags_31 },
    { 0xbfbb8450, SUC_lexicon_tags_80 },
    { 0x35fb8451, SUC_lexicon_tags_29 },
    { 0x639b8452, SUC_lexicon_tags_77 },
    { 0x48db8452, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x933b8455, SUC_lexicon_tags_15 },
    { 0xcd1b8456, SUC_lexicon_tags_8 },
    { 0x1b3b8455, SUC_lexicon_tags_55 },
    { 0x1afb8456, SUC_lexicon_tags_43 },
    { 0xeddb8459, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf2bb845b, SUC_lexicon_tags_37 },
    { 0x5a7b845b, SUC_lexicon_tags_27 },
    { 0xcbbb845c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5db8460, SUC_lexicon_tags_38 },
    { 0x851b8461, SUC_lexicon_tags_1 },
    { 0x699b8461, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7b8466, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x613b8468, SUC_lexicon_tags_28 },
    { 0x3f5b8468, SUC_lexicon_tags_26 },
    { 0xd91b8469, SUC_lexicon_tags_26 },
    { 0x367b846b, SUC_lexicon_tags_11 },
    { 0x21b846c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedb8471, SUC_lexicon_tags_56 },
    { 0xf2bb8471, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1b8477, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81bb8479, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc11b847b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5b8480, SUC_lexicon_tags_11 },
    { 0x85fb8480, SUC_lexicon_tags_11 },
    { 0x22bb8480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37b8486, SUC_lexicon_tags_25 },
    { 0x361b8487, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfb848c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x775b848e, SUC_lexicon_tags_49 },
    { 0xaedb848f, SUC_lexicon_tags_7 },
    { 0x165b848f, SUC_lexicon_tags_19 },
    { 0xa57b8491, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x845b8493, SUC_lexicon_tags_45 },
    { 0x3ebb8494, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa3b8496, SUC_lexicon_tags_14 },
    { 0x6b9b8496, SUC_lexicon_tags_8 },
    { 0x705b8498, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb849e, SUC_lexicon_tags_25 },
    { 0x8b1b849f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x477b84a1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01b84a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fb84aa, SUC_lexicon_tags_11 },
    { 0xa01b84aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51b84af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69b84b2, SUC_lexicon_tags_3 },
    { 0x967b84b3, SUC_lexicon_tags_15 },
    { 0x469b84b2, SUC_lexicon_tags_12 },
    { 0x589b84b5, SUC_lexicon_tags_35 },
    { 0x1dfb84b5, SUC_lexicon_tags_17 },
    { 0xe5db84b7, SUC_lexicon_tags_14 },
    { 0xc2db84b8, SUC_lexicon_tags_15 },
    { 0xc05b84b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbb84c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf0bb84c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3b84c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b84ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3ffb84cc, SUC_lexicon_tags_43 },
    { 0x17b84cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5efb84cf, SUC_lexicon_tags_29 },
    { 0xef7b84d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfb84d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fb84db, SUC_lexicon_tags_1 },
    { 0x2a9b84db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebb84df, SUC_lexicon_tags_15 },
    { 0xf9bb84e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5b84e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x379b84e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe6fb84ea, SUC_lexicon_tags_37 },
    { 0xe7db84eb, SUC_lexicon_tags_45 },
    { 0xd01b84ec, SUC_lexicon_tags_17 },
    { 0xc7bb84ed, SUC_lexicon_tags_9 },
    { 0x19b84ee, SUC_lexicon_tags_17 },
    { 0xad5b84eb, SUC_lexicon_tags_21 },
    { 0xf1bb84f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bb84f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfb84f9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffb84fc, SUC_lexicon_tags_14 },
    { 0x9d3b84fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bb8500, SUC_lexicon_tags_18 },
    { 0xd49b8501, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a9b8503, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fb850b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7b850e, SUC_lexicon_tags_9 },
    { 0x809b850e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5b8515, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519b8519, SUC_lexicon_tags_29 },
    { 0xd51b8519, SUC_lexicon_tags_32 },
    { 0xdadb851b, SUC_lexicon_tags_37 },
    { 0x2f3b851b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x777b851e, SUC_lexicon_tags_10 },
    { 0x553b851f, SUC_lexicon_tags_37 },
    { 0xb63b8520, SUC_lexicon_tags_15 },
    { 0x219b8521, SUC_lexicon_tags_39 },
    { 0xd23b8520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x861b8524, SUC_lexicon_tags_208 },
    { 0xa69b8525, SUC_lexicon_tags_11 },
    { 0xab9b8524, SUC_lexicon_tags_15 },
    { 0xfbb8527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x115b8529, SUC_lexicon_tags_0 },
    { 0x831b852a, SUC_lexicon_tags_12 },
    { 0xf0db852b, SUC_lexicon_tags_28 },
    { 0x7f5b852c, SUC_lexicon_tags_32 },
    { 0xee3b852c, SUC_lexicon_tags_39 },
    { 0x9fb852d, SUC_lexicon_tags_25 },
    { 0x8e1b852f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97b853a, SUC_lexicon_tags_55 },
    { 0xc81b853b, SUC_lexicon_tags_15 },
    { 0x1d1b853b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77b853f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1b8542, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdb8547, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffb854a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9b854e, SUC_lexicon_tags_36 },
    { 0x201b854e, SUC_lexicon_tags_37 },
    { 0x741b8550, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbb8554, SUC_lexicon_tags_2 },
    { 0x88fb8554, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91b8559, SUC_lexicon_tags_11 },
    { 0xeb9b855a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89db855e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53b8561, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebb8565, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fb8569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bb8576, SUC_lexicon_tags_17 },
    { 0xfd5b8576, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edb857b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x763b857d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7b8581, SUC_lexicon_tags_53 },
    { 0x857b8582, SUC_lexicon_tags_43 },
    { 0x271b8582, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433b8587, SUC_lexicon_tags_12 },
    { 0xb1fb8588, SUC_lexicon_tags_63 },
    { 0x565b8587, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x527b858b, SUC_lexicon_tags_38 },
    { 0x399b858c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcd1b858e, SUC_lexicon_tags_15 },
    { 0x997b858e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b8595, SUC_lexicon_tags_31 },
    { 0x985b8596, SUC_lexicon_tags_29 },
    { 0xf03b8597, SUC_lexicon_tags_64 },
    { 0x1c7b8598, SUC_lexicon_tags_47 },
    { 0x687b8596, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bb859d, SUC_lexicon_tags_11 },
    { 0x10fb859e, SUC_lexicon_tags_15 },
    { 0xcb3b859d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5b85a3, SUC_lexicon_tags_41 },
    { 0x303b85a4, SUC_lexicon_tags_17 },
    { 0x349b85a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffb85a9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445b85b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109b85b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdb85bb, SUC_lexicon_tags_19 },
    { 0x1f1b85bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5b85c3, SUC_lexicon_tags_33 },
    { 0xea9b85c4, SUC_lexicon_tags_60 },
    { 0x24db85c5, SUC_lexicon_tags_37 },
    { 0x6fb85c4, SUC_lexicon_tags_7 },
    { 0xf69b85c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x943b85ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd01b85cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7b85d5, SUC_lexicon_tags_6 },
    { 0xa99b85d6, SUC_lexicon_tags_2 },
    { 0x971b85d6, SUC_lexicon_tags_15 },
    { 0xad9b85d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7db85db, SUC_lexicon_tags_47 },
    { 0x653b85dc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4dfb85de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb21b85e0, SUC_lexicon_tags_13 },
    { 0xd91b85e1, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbb85e4, SUC_lexicon_tags_15 },
    { 0xe41b85e4, SUC_lexicon_tags_87 },
    { 0x1fbb85e6, SUC_lexicon_tags_60 },
    { 0xf47b85e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75db85ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5b85ed, SUC_lexicon_tags_15 },
    { 0x1e5b85ed, SUC_lexicon_tags_43 },
    { 0xac9b85ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4b7b85f1, SUC_lexicon_tags_6 },
    { 0xa8fb85f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fb85f6, SUC_lexicon_tags_47 },
    { 0x651b85f6, SUC_lexicon_tags_19 },
    { 0xa51b85f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb85fb, SUC_lexicon_tags_25 },
    { 0x1a9b85fb, SUC_lexicon_tags_17 },
    { 0xb47b85fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe47b85ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fb8602, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x323b8605, SUC_lexicon_tags_37 },
    { 0xac1b8606, SUC_lexicon_tags_0 },
    { 0xe1fb8607, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7b860a, SUC_lexicon_tags_11 },
    { 0xf71b860a, SUC_lexicon_tags_19 },
    { 0x175b860c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xebb860e, SUC_lexicon_tags_27 },
    { 0xdb5b860f, SUC_lexicon_tags_25 },
    { 0x68fb8610, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbb8615, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6abb8617, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9b7b8619, SUC_lexicon_tags_14 },
    { 0x78db8619, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35b861e, SUC_lexicon_tags_31 },
    { 0x33b861f, SUC_lexicon_tags_179 },
    { 0x273b861e, SUC_lexicon_tags_25 },
    { 0xeffb861f, SUC_lexicon_tags_17 },
    { 0xd07b8622, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbb8628, SUC_lexicon_tags_43 },
    { 0xfebb8629, SUC_lexicon_tags_8 },
    { 0x429b862a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc3fb862c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53b862e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x249b8631, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x797b8633, SUC_lexicon_tags_39 },
    { 0x35bb8634, SUC_lexicon_tags_12 },
    { 0xcedb8634, SUC_lexicon_tags_14 },
    { 0x8fb8636, SUC_lexicon_tags_43 },
    { 0x9fb8636, SUC_lexicon_tags_15 },
    { 0xc15b8638, SUC_lexicon_tags_11 },
    { 0x5e7b8633, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x37db863c, SUC_lexicon_tags_17 },
    { 0x9b9b863d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5b8642, SUC_lexicon_tags_12 },
    { 0xb85b8642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db864f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa53b8651, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fb8659, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf9bb865b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc5b865d, SUC_lexicon_tags_0 },
    { 0xfdbb865e, SUC_lexicon_tags_4 },
    { 0x1cfb865f, SUC_lexicon_tags_37 },
    { 0xadfb8660, SUC_lexicon_tags_17 },
    { 0xaa1b8661, SUC_lexicon_tags_11 },
    { 0x4cbb8662, SUC_lexicon_tags_1 },
    { 0xe39b8662, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fbb8665, SUC_lexicon_tags_25 },
    { 0x783b8666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15b8669, SUC_lexicon_tags_8 },
    { 0x27b866a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19b866e, SUC_lexicon_tags_3 },
    { 0xb3fb866f, SUC_lexicon_tags_48 },
    { 0x741b8670, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7ffb8672, SUC_lexicon_tags_10 },
    { 0xc5bb8673, SUC_lexicon_tags_17 },
    { 0x45db8674, SUC_lexicon_tags_2 },
    { 0xddfb8674, SUC_lexicon_tags_2 },
    { 0x549b8675, SUC_lexicon_tags_1 },
    { 0x17fb8672, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137b867f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279b8683, SUC_lexicon_tags_1 },
    { 0x995b8684, SUC_lexicon_tags_59 },
    { 0xcb9b8684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fb8689, SUC_lexicon_tags_49 },
    { 0xd3bb868a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db868f, SUC_lexicon_tags_15 },
    { 0x407b8690, SUC_lexicon_tags_62 },
    { 0x6fbb868f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5b8694, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e1b8696, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bb8699, SUC_lexicon_tags_17 },
    { 0xa1b869a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb86a0, SUC_lexicon_tags_45 },
    { 0x4dfb86a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb61b86a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bb86a6, SUC_lexicon_tags_27 },
    { 0xb57b86a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa05b86a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7b86ad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xac7b86af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bb86b5, SUC_lexicon_tags_1 },
    { 0x24fb86b6, SUC_lexicon_tags_48 },
    { 0x82fb86b7, SUC_lexicon_tags_27 },
    { 0x497b86b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdb86bb, SUC_lexicon_tags_12 },
    { 0xa5bb86bc, SUC_lexicon_tags_2 },
    { 0xafb86bd, SUC_lexicon_tags_47 },
    { 0x7ebb86bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db86c5, SUC_lexicon_tags_27 },
    { 0xe75b86c5, SUC_lexicon_tags_27 },
    { 0x67db86c7, SUC_lexicon_tags_17 },
    { 0xbebb86c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39b86cf, SUC_lexicon_tags_17 },
    { 0x537b86d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x103b86d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759b86da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77b86e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x901b86e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfb86e8, SUC_lexicon_tags_10 },
    { 0x9d7b86e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1b86ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6db86f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe43b86f3, SUC_lexicon_tags_69 },
    { 0x815b86f4, SUC_lexicon_tags_11 },
    { 0xc71b86f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531b86f9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fb8703, SUC_lexicon_tags_23 },
    { 0x5bb8704, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x54fb8706, SUC_lexicon_tags_31 },
    { 0xd45b8707, SUC_lexicon_tags_15 },
    { 0xa27b8708, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xff7b870a, SUC_lexicon_tags_27 },
    { 0x8dbb870b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x579b870e, SUC_lexicon_tags_55 },
    { 0x91fb870e, SUC_lexicon_tags_15 },
    { 0x811b8710, SUC_lexicon_tags_1 },
    { 0xaa3b8711, SUC_lexicon_tags_15 },
    { 0x11fb8711, SUC_lexicon_tags_15 },
    { 0xcb5b8713, SUC_lexicon_tags_37 },
    { 0x88db8710, SUC_lexicon_tags_37 },
    { 0xad9b8715, SUC_lexicon_tags_17 },
    { 0x75db8715, SUC_lexicon_tags_1 },
    { 0x49bb8717, SUC_lexicon_tags_55 },
    { 0x39db8718, SUC_lexicon_tags_25 },
    { 0x273b8719, SUC_lexicon_tags_17 },
    { 0x7d7b871a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf07b871c, SUC_lexicon_tags_11 },
    { 0x8f3b871c, SUC_lexicon_tags_28 },
    { 0x9e3b871d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeb9b8720, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bb8723, SUC_lexicon_tags_0 },
    { 0xafbb8723, SUC_lexicon_tags_1 },
    { 0x7bdb8725, SUC_lexicon_tags_59 },
    { 0x54bb8725, SUC_lexicon_tags_38 },
    { 0x917b8727, SUC_lexicon_tags_1 },
    { 0x305b8727, SUC_lexicon_tags_23 },
    { 0xa23b8726, SUC_lexicon_tags_37 },
    { 0x603b8728, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743b872f, SUC_lexicon_tags_25 },
    { 0x917b872f, SUC_lexicon_tags_4 },
    { 0x3db872f, SUC_lexicon_tags_11 },
    { 0x9cfb8732, SUC_lexicon_tags_12 },
    { 0x9d5b8732, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bb8736, SUC_lexicon_tags_1 },
    { 0x91bb8737, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x93fb8739, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x137b873b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bb873e, SUC_lexicon_tags_31 },
    { 0x4bbb873f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x129b8741, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465b8746, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855b874a, SUC_lexicon_tags_2 },
    { 0xe4db874a, SUC_lexicon_tags_12 },
    { 0x3d3b874c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x473b874e, SUC_lexicon_tags_45 },
    { 0x7c5b874f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789b8754, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19b8758, SUC_lexicon_tags_37 },
    { 0x351b8758, SUC_lexicon_tags_38 },
    { 0x717b875a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfb875e, SUC_lexicon_tags_1 },
    { 0x9e3b875f, SUC_lexicon_tags_29 },
    { 0xe11b875f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfb8768, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05b876f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fb8773, SUC_lexicon_tags_6 },
    { 0xec7b8774, SUC_lexicon_tags_17 },
    { 0x42fb8775, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x68db8777, SUC_lexicon_tags_63 },
    { 0xe4bb8777, SUC_lexicon_tags_49 },
    { 0x1f1b8778, SUC_lexicon_tags_13 },
    { 0xdc3b8778, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bb877e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad9b8780, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x25b8782, SUC_lexicon_tags_43 },
    { 0x795b8783, SUC_lexicon_tags_11 },
    { 0x4e1b8784, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89b8788, SUC_lexicon_tags_12 },
    { 0xaabb8789, SUC_lexicon_tags_11 },
    { 0x2e9b878a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x85fb878c, SUC_lexicon_tags_15 },
    { 0x89bb878d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x709b8790, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5b8798, SUC_lexicon_tags_12 },
    { 0xbdfb8799, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619b879e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x745b87a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52bb87a3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3b87a8, SUC_lexicon_tags_37 },
    { 0x10fb87a9, SUC_lexicon_tags_15 },
    { 0xc67b87a9, SUC_lexicon_tags_66 },
    { 0xb89b87ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9b87af, SUC_lexicon_tags_60 },
    { 0x581b87b0, SUC_lexicon_tags_10 },
    { 0xc25b87b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x567b87b3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1b87b6, SUC_lexicon_tags_7 },
    { 0xad7b87b7, SUC_lexicon_tags_19 },
    { 0xba7b87b8, SUC_lexicon_tags_15 },
    { 0xf7b87b9, SUC_lexicon_tags_37 },
    { 0xb4db87ba, SUC_lexicon_tags_124 },
    { 0x5fbb87bb, SUC_lexicon_tags_11 },
    { 0x8d1b87b9, SUC_lexicon_tags_13 },
    { 0xa83b87bc, SUC_lexicon_tags_36 },
    { 0x6db87bd, SUC_lexicon_tags_26 },
    { 0x6bbb87bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfb87c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897b87ce, SUC_lexicon_tags_1 },
    { 0xed9b87cf, SUC_lexicon_tags_28 },
    { 0xd53b87cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47b87d3, SUC_lexicon_tags_11 },
    { 0x72fb87d4, SUC_lexicon_tags_25 },
    { 0x4d7b87d4, SUC_lexicon_tags_28 },
    { 0xad9b87d6, SUC_lexicon_tags_68 },
    { 0x985b87d7, SUC_lexicon_tags_11 },
    { 0x69db87d4, SUC_lexicon_tags_23 },
    { 0x45bb87d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bb87de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48bb87e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841b87e7, SUC_lexicon_tags_49 },
    { 0x8abb87e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74fb87ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15b87f7, SUC_lexicon_tags_13 },
    { 0xd9bb87f8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35b8800, SUC_lexicon_tags_25 },
    { 0xa8bb8801, SUC_lexicon_tags_37 },
    { 0x6d5b8802, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcbb8804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b8808, SUC_lexicon_tags_23 },
    { 0xa29b8809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77b880e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x679b8810, SUC_lexicon_tags_37 },
    { 0x6d3b8810, SUC_lexicon_tags_7 },
    { 0x3edb8812, SUC_lexicon_tags_34 },
    { 0x3c5b8813, SUC_lexicon_tags_28 },
    { 0x987b8812, SUC_lexicon_tags_45 },
    { 0xd89b8815, SUC_lexicon_tags_25 },
    { 0x891b8816, SUC_lexicon_tags_70 },
    { 0x9cbb8815, SUC_lexicon_tags_43 },
    { 0x27db8814, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfb881f, SUC_lexicon_tags_47 },
    { 0x399b8820, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa83b8822, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x35fb8824, SUC_lexicon_tags_2 },
    { 0xd83b8824, SUC_lexicon_tags_37 },
    { 0xcdb8826, SUC_lexicon_tags_11 },
    { 0x315b8827, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xa37b8829, SUC_lexicon_tags_761 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85b882e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bb8832, SUC_lexicon_tags_35 },
    { 0x3f5b8833, SUC_lexicon_tags_424 },
    { 0x32fb8832, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09b8837, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfefb8839, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39b883d, SUC_lexicon_tags_4 },
    { 0x24fb883e, SUC_lexicon_tags_7 },
    { 0x13db883f, SUC_lexicon_tags_49 },
    { 0x71fb8840, SUC_lexicon_tags_11 },
    { 0x449b8841, SUC_lexicon_tags_43 },
    { 0x877b8840, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x875b8844, SUC_lexicon_tags_68 },
    { 0xb1db8844, SUC_lexicon_tags_37 },
    { 0xb77b8846, SUC_lexicon_tags_0 },
    { 0x981b8847, SUC_lexicon_tags_43 },
    { 0xa85b8844, SUC_lexicon_tags_0 },
    { 0xb7bb8849, SUC_lexicon_tags_82 },
    { 0x9d7b8849, SUC_lexicon_tags_86 },
    { 0x393b884b, SUC_lexicon_tags_0 },
    { 0xa21b8849, SUC_lexicon_tags_12 },
    { 0xf89b884d, SUC_lexicon_tags_33 },
    { 0x27fb8844, SUC_lexicon_tags_0 },
    { 0x553b884f, SUC_lexicon_tags_2 },
    { 0xc9b884a, SUC_lexicon_tags_507 },
    { 0xa47b8851, SUC_lexicon_tags_28 },
    { 0x309b8852, SUC_lexicon_tags_14 },
    { 0x28bb8853, SUC_lexicon_tags_25 },
    { 0xb1bb8853, SUC_lexicon_tags_12 },
    { 0x5a1b884f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87b8859, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x43db885b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf33b885d, SUC_lexicon_tags_15 },
    { 0x517b885e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x54db8860, SUC_lexicon_tags_23 },
    { 0x2a9b8861, SUC_lexicon_tags_19 },
    { 0x2edb8861, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363b8866, SUC_lexicon_tags_27 },
    { 0x419b8866, SUC_lexicon_tags_37 },
    { 0xf3fb8867, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe97b886a, SUC_lexicon_tags_63 },
    { 0xeb9b886b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7db886f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a9b8871, SUC_lexicon_tags_92 },
    { 0x337b8872, SUC_lexicon_tags_47 },
    { 0xeb5b8873, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1b887a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83b887f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09b8882, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcfb8884, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x319b8886, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x919b8888, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329b888e, SUC_lexicon_tags_2 },
    { 0xdc5b888f, SUC_lexicon_tags_63 },
    { 0x337b888f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1b8893, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efb8897, SUC_lexicon_tags_11 },
    { 0x6c9b8898, SUC_lexicon_tags_7 },
    { 0xa0fb8899, SUC_lexicon_tags_0 },
    { 0xd39b889a, SUC_lexicon_tags_17 },
    { 0xe19b889b, SUC_lexicon_tags_68 },
    { 0xa6fb889c, SUC_lexicon_tags_11 },
    { 0x6ddb889c, SUC_lexicon_tags_82 },
    { 0x7b3b889b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9b88a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x191b88a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x965b88a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41db88ab, SUC_lexicon_tags_4 },
    { 0x795b88ab, SUC_lexicon_tags_45 },
    { 0x31bb88ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2ffb88af, SUC_lexicon_tags_8 },
    { 0x47fb88af, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85b88b3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9b88b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbb88ba, SUC_lexicon_tags_15 },
    { 0x98bb88bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x471b88be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bb88c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7b88ca, SUC_lexicon_tags_12 },
    { 0xf65b88ca, SUC_lexicon_tags_26 },
    { 0xe7db88cc, SUC_lexicon_tags_18 },
    { 0x295b88cd, SUC_lexicon_tags_15 },
    { 0x813b88ce, SUC_lexicon_tags_15 },
    { 0x42fb88cc, SUC_lexicon_tags_318 },
    { 0xe0fb88d0, SUC_lexicon_tags_11 },
    { 0x253b88cc, SUC_lexicon_tags_38 },
    { 0x455b88cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3b88d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x277b88d9, SUC_lexicon_tags_4 },
    { 0x25b88d9, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3b88dd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403b88e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadbb88e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x597b88e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3db88ed, SUC_lexicon_tags_25 },
    { 0x34bb88ee, SUC_lexicon_tags_11 },
    { 0xe8fb88ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x195b88f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x26fb88f3, SUC_lexicon_tags_15 },
    { 0xe43b88f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb79b88f6, SUC_lexicon_tags_29 },
    { 0xd15b88f6, SUC_lexicon_tags_61 },
    { 0x9f1b88f8, SUC_lexicon_tags_37 },
    { 0x44db88f9, SUC_lexicon_tags_17 },
    { 0x2b5b88fa, SUC_lexicon_tags_25 },
    { 0x4c5b88fa, SUC_lexicon_tags_12 },
    { 0x6fdb88fc, SUC_lexicon_tags_17 },
    { 0x419b88fd, SUC_lexicon_tags_334 },
    { 0xcedb88fe, SUC_lexicon_tags_12 },
    { 0x603b88f9, SUC_lexicon_tags_29 },
    { 0x949b88f6, SUC_lexicon_tags_7 },
    { 0x7c5b8901, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9b8907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xea3b8909, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47db8912, SUC_lexicon_tags_37 },
    { 0x735b8913, SUC_lexicon_tags_88 },
    { 0x32bb8913, SUC_lexicon_tags_17 },
    { 0xce9b8914, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc07b8917, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5b891b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe5fb891d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86fb891f, SUC_lexicon_tags_29 },
    { 0x711b8920, SUC_lexicon_tags_13 },
    { 0xcefb891f, SUC_lexicon_tags_12 },
    { 0xd41b8922, SUC_lexicon_tags_27 },
    { 0x85db8923, SUC_lexicon_tags_14 },
    { 0xe5db8923, SUC_lexicon_tags_17 },
    { 0xf85b8924, SUC_lexicon_tags_37 },
    { 0x369b8926, SUC_lexicon_tags_25 },
    { 0x7b5b8926, SUC_lexicon_tags_69 },
    { 0xefb8928, SUC_lexicon_tags_25 },
    { 0xc6fb8928, SUC_lexicon_tags_14 },
    { 0x993b8925, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731b892f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4bb8931, SUC_lexicon_tags_53 },
    { 0xe65b8932, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbb8935, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37b893b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x83bb893e, SUC_lexicon_tags_37 },
    { 0x569b893f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x78db8942, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x163b8944, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31db8948, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31bb894a, SUC_lexicon_tags_28 },
    { 0x9bbb894b, SUC_lexicon_tags_175 },
    { 0x823b894c, SUC_lexicon_tags_1 },
    { 0xe0db894d, SUC_lexicon_tags_2 },
    { 0xbf1b894e, SUC_lexicon_tags_2 },
    { 0xb19b894f, SUC_lexicon_tags_11 },
    { 0x29bb894e, SUC_lexicon_tags_11 },
    { 0x9b1b8951, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba7b8954, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5b8958, SUC_lexicon_tags_43 },
    { 0x4a3b8959, SUC_lexicon_tags_27 },
    { 0xd53b895a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5b895d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5b8960, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bb8964, SUC_lexicon_tags_7 },
    { 0x7e5b8964, SUC_lexicon_tags_17 },
    { 0x1c7b8966, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75b896a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bb896e, SUC_lexicon_tags_19 },
    { 0x33b896e, SUC_lexicon_tags_38 },
    { 0x62bb896e, SUC_lexicon_tags_29 },
    { 0xa03b8971, SUC_lexicon_tags_66 },
    { 0xb79b8970, SUC_lexicon_tags_25 },
    { 0xc15b8973, SUC_lexicon_tags_77 },
    { 0xd4db8970, SUC_lexicon_tags_30 },
    { 0x34db8973, SUC_lexicon_tags_126 },
    { 0x153b8972, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bb897e, SUC_lexicon_tags_23 },
    { 0x385b897f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1b8985, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7b8988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x115b898a, SUC_lexicon_tags_47 },
    { 0x95db898a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7b8991, SUC_lexicon_tags_17 },
    { 0xbcdb8992, SUC_lexicon_tags_6 },
    { 0x16db8993, SUC_lexicon_tags_70 },
    { 0xbfdb8991, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4db8996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5b8999, SUC_lexicon_tags_38 },
    { 0x4fb899a, SUC_lexicon_tags_23 },
    { 0x78fb899b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309b89a0, SUC_lexicon_tags_2 },
    { 0x39fb89a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x62fb89a3, SUC_lexicon_tags_13 },
    { 0xf6bb89a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1b89a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fb89ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x277b89af, SUC_lexicon_tags_11 },
    { 0xfe9b89af, SUC_lexicon_tags_3 },
    { 0x73bb89b1, SUC_lexicon_tags_23 },
    { 0xbd5b89b2, SUC_lexicon_tags_7 },
    { 0xdb7b89b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x669b89b5, SUC_lexicon_tags_39 },
    { 0xcc5b89b6, SUC_lexicon_tags_2 },
    { 0xddb89b7, SUC_lexicon_tags_45 },
    { 0xf2db89b7, SUC_lexicon_tags_1 },
    { 0xecdb89b9, SUC_lexicon_tags_26 },
    { 0xf69b89ba, SUC_lexicon_tags_15 },
    { 0xbe9b89b5, SUC_lexicon_tags_17 },
    { 0x857b89bc, SUC_lexicon_tags_39 },
    { 0xd7b89bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x583b89c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bb89c9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7dbb89cb, SUC_lexicon_tags_11 },
    { 0x61fb89cc, SUC_lexicon_tags_25 },
    { 0xb41b89cb, SUC_lexicon_tags_1 },
    { 0x2f7b89ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbc9b89d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32db89d8, SUC_lexicon_tags_6 },
    { 0x195b89d9, SUC_lexicon_tags_60 },
    { 0x2ffb89d9, SUC_lexicon_tags_10 },
    { 0x219b89d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e7b89dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89b89e0, SUC_lexicon_tags_33 },
    { 0x7afb89e0, SUC_lexicon_tags_23 },
    { 0x8a1b89e1, SUC_lexicon_tags_82 },
    { 0x9b1b89e3, SUC_lexicon_tags_1 },
    { 0xa8db89e1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851b89e8, SUC_lexicon_tags_11 },
    { 0x5e9b89e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bb89fb, SUC_lexicon_tags_63 },
    { 0x5bb89fc, SUC_lexicon_tags_9 },
    { 0x6db89fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7b8a09, SUC_lexicon_tags_8 },
    { 0x9c7b8a0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1b8a0e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5b8a1b, SUC_lexicon_tags_9 },
    { 0x43db8a1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c9b8a1e, SUC_lexicon_tags_27 },
    { 0xb9bb8a1e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7c3b8a21, SUC_lexicon_tags_39 },
    { 0x6d3b8a22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x91db8a24, SUC_lexicon_tags_47 },
    { 0xc11b8a24, SUC_lexicon_tags_2 },
    { 0x721b8a26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x307b8a29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x71bb8a2b, SUC_lexicon_tags_5 },
    { 0xfa3b8a2c, SUC_lexicon_tags_1 },
    { 0x963b8a2c, SUC_lexicon_tags_17 },
    { 0x95fb8a2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75b8a34, SUC_lexicon_tags_25 },
    { 0xb53b8a35, SUC_lexicon_tags_43 },
    { 0x6f9b8a36, SUC_lexicon_tags_15 },
    { 0xb2fb8a37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bb8a3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abb8a41, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x191b8a43, SUC_lexicon_tags_17 },
    { 0x3b8a43, SUC_lexicon_tags_62 },
    { 0xdc5b8a45, SUC_lexicon_tags_12 },
    { 0x251b8a46, SUC_lexicon_tags_8 },
    { 0x551b8a47, SUC_lexicon_tags_17 },
    { 0x427b8a48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc5db8a4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84db8a4d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbb8a55, SUC_lexicon_tags_7 },
    { 0x31bb8a55, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fb8a5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abb8a60, SUC_lexicon_tags_30 },
    { 0x20bb8a61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55b8a67, SUC_lexicon_tags_15 },
    { 0x659b8a67, SUC_lexicon_tags_7 },
    { 0xfb9b8a67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x279b8a6b, SUC_lexicon_tags_13 },
    { 0xdddb8a6c, SUC_lexicon_tags_7 },
    { 0xf55b8a6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f7b8a6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15b8a71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71b8a75, SUC_lexicon_tags_36 },
    { 0x4e9b8a75, SUC_lexicon_tags_37 },
    { 0xe9bb8a76, SUC_lexicon_tags_7 },
    { 0x9fb8a78, SUC_lexicon_tags_68 },
    { 0x8abb8a76, SUC_lexicon_tags_1 },
    { 0x12fb8a7a, SUC_lexicon_tags_15 },
    { 0x5d5b8a7b, SUC_lexicon_tags_15 },
    { 0x231b8a75, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x1edb8a7e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5b8a82, SUC_lexicon_tags_646 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831b8a8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc89b8a8f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bb8a98, SUC_lexicon_tags_45 },
    { 0x177b8a99, SUC_lexicon_tags_17 },
    { 0x141b8a99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4db8a9c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03b8aa0, SUC_lexicon_tags_15 },
    { 0xa2fb8aa1, SUC_lexicon_tags_21 },
    { 0x221b8aa2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7b8aa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431b8aa9, SUC_lexicon_tags_226 },
    { 0x40bb8aaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7b8ab0, SUC_lexicon_tags_7 },
    { 0x8afb8ab0, SUC_lexicon_tags_37 },
    { 0x39bb8ab2, SUC_lexicon_tags_19 },
    { 0x175b8ab3, SUC_lexicon_tags_33 },
    { 0x197b8ab4, SUC_lexicon_tags_9 },
    { 0x18db8ab4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87b8abb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695b8ac5, SUC_lexicon_tags_27 },
    { 0x985b8ac6, SUC_lexicon_tags_1010 },
    { 0x163b8ac7, SUC_lexicon_tags_56 },
    { 0x175b8ac8, SUC_lexicon_tags_47 },
    { 0x277b8ac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1db8ad0, SUC_lexicon_tags_28 },
    { 0x4a3b8ad1, SUC_lexicon_tags_28 },
    { 0xadb8ad2, SUC_lexicon_tags_1 },
    { 0x581b8ad2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc5b8ad5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9b8ad9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd63b8adb, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7b8ae1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd61b8ae3, SUC_lexicon_tags_7 },
    { 0x907b8ae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5b8ae8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x487b8aea, SUC_lexicon_tags_7 },
    { 0x5c5b8aeb, SUC_lexicon_tags_11 },
    { 0x1d1b8aec, SUC_lexicon_tags_127 },
    { 0x4cbb8aeb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x94db8aef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x445b8af2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb8af5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5db8af9, SUC_lexicon_tags_12 },
    { 0x3f9b8afa, SUC_lexicon_tags_37 },
    { 0xc65b8afa, SUC_lexicon_tags_17 },
    { 0xcd1b8afa, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435b8b00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35b8b05, SUC_lexicon_tags_60 },
    { 0x6f3b8b06, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6db8b0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdabb8b0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7b8b12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefb8b15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5b8b18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x827b8b1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1b8b1d, SUC_lexicon_tags_10 },
    { 0xaf1b8b1e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3b8b27, SUC_lexicon_tags_15 },
    { 0xbfbb8b28, SUC_lexicon_tags_17 },
    { 0xbb8b28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03b8b2c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467b8b30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503b8b36, SUC_lexicon_tags_14 },
    { 0xa75b8b36, SUC_lexicon_tags_11 },
    { 0x5ffb8b38, SUC_lexicon_tags_0 },
    { 0xf3b8b37, SUC_lexicon_tags_26 },
    { 0xbcdb8b38, SUC_lexicon_tags_15 },
    { 0x631b8b3b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77b8b42, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931b8b4d, SUC_lexicon_tags_1 },
    { 0x1f5b8b4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9b8b53, SUC_lexicon_tags_25 },
    { 0xeb9b8b53, SUC_lexicon_tags_25 },
    { 0xed7b8b53, SUC_lexicon_tags_55 },
    { 0x3f5b8b55, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xce7b8b58, SUC_lexicon_tags_37 },
    { 0x42db8b59, SUC_lexicon_tags_37 },
    { 0xb9b8b5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x669b8b5c, SUC_lexicon_tags_8 },
    { 0x97bb8b5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfb8b5f, SUC_lexicon_tags_0 },
    { 0xe9fb8b5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad7b8b62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329b8b69, SUC_lexicon_tags_25 },
    { 0x9a7b8b6a, SUC_lexicon_tags_8 },
    { 0xda7b8b6b, SUC_lexicon_tags_2 },
    { 0x2d9b8b6c, SUC_lexicon_tags_16 },
    { 0x745b8b6d, SUC_lexicon_tags_2 },
    { 0x511b8b6a, SUC_lexicon_tags_13 },
    { 0xe3bb8b6d, SUC_lexicon_tags_25 },
    { 0x33b8b70, SUC_lexicon_tags_55 },
    { 0x9e3b8b6a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb71b8b73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5b8b77, SUC_lexicon_tags_8 },
    { 0x2f5b8b78, SUC_lexicon_tags_3 },
    { 0x833b8b78, SUC_lexicon_tags_17 },
    { 0x6f7b8b7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97b8b80, SUC_lexicon_tags_11 },
    { 0x39b8b81, SUC_lexicon_tags_113 },
    { 0x2edb8b81, SUC_lexicon_tags_0 },
    { 0xa7bb8b82, SUC_lexicon_tags_64 },
    { 0x5efb8b80, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x793b8b86, SUC_lexicon_tags_2 },
    { 0xd2db8b87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9b8b90, SUC_lexicon_tags_25 },
    { 0xa03b8b91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71b8b95, SUC_lexicon_tags_39 },
    { 0x6c9b8b95, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bb8b9a, SUC_lexicon_tags_17 },
    { 0x5e1b8b9b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xad1b8b9d, SUC_lexicon_tags_23 },
    { 0x691b8b9d, SUC_lexicon_tags_60 },
    { 0xac3b8b9f, SUC_lexicon_tags_12 },
    { 0xb65b8ba0, SUC_lexicon_tags_10 },
    { 0xc93b8b9e, SUC_lexicon_tags_60 },
    { 0xe63b8ba2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49b8ba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x127b8ba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63db8bad, SUC_lexicon_tags_48 },
    { 0x727b8bae, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x267b8bb0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7b8bb4, SUC_lexicon_tags_2 },
    { 0x141b8bb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa11b8bb7, SUC_lexicon_tags_9 },
    { 0x763b8bb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23b8bbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x357b8bbe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793b8bc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fb8bc8, SUC_lexicon_tags_15 },
    { 0xc25b8bc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a7b8bcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbb8bd3, SUC_lexicon_tags_37 },
    { 0x8a1b8bd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709b8be1, SUC_lexicon_tags_38 },
    { 0x249b8be2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x747b8be5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271b8beb, SUC_lexicon_tags_15 },
    { 0xf03b8bec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x171b8bee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb61b8bf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d9b8bf2, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x8afb8bf5, SUC_lexicon_tags_11 },
    { 0x205b8bf6, SUC_lexicon_tags_12 },
    { 0xda7b8bf7, SUC_lexicon_tags_25 },
    { 0xba9b8bf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb7b8bfa, SUC_lexicon_tags_37 },
    { 0x9e3b8bfa, SUC_lexicon_tags_65 },
    { 0xcafb8bfc, SUC_lexicon_tags_27 },
    { 0x9a1b8bfb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x9f7b8bff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25b8c05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x247b8c07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45bb8c0a, SUC_lexicon_tags_11 },
    { 0xdbdb8c0b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5b8c12, SUC_lexicon_tags_0 },
    { 0x7c7b8c13, SUC_lexicon_tags_25 },
    { 0x903b8c14, SUC_lexicon_tags_113 },
    { 0x91b8c15, SUC_lexicon_tags_13 },
    { 0xbe7b8c15, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3b8c1b, SUC_lexicon_tags_74 },
    { 0x381b8c1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a5b8c1e, SUC_lexicon_tags_37 },
    { 0x84bb8c1f, SUC_lexicon_tags_61 },
    { 0x68bb8c20, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x883b8c22, SUC_lexicon_tags_12 },
    { 0xbcbb8c22, SUC_lexicon_tags_29 },
    { 0x2f9b8c24, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2c9b8c26, SUC_lexicon_tags_25 },
    { 0xc09b8c27, SUC_lexicon_tags_15 },
    { 0x92bb8c26, SUC_lexicon_tags_28 },
    { 0xfedb8c27, SUC_lexicon_tags_12 },
    { 0x8b9b8c26, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fb8c2f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33b8c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa11b8c35, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50db8c3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56db8c3c, SUC_lexicon_tags_11 },
    { 0x965b8c3d, SUC_lexicon_tags_7 },
    { 0xd1b8c3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251b8c48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71db8c4a, SUC_lexicon_tags_25 },
    { 0x3afb8c4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbb8c4e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf33b8c50, SUC_lexicon_tags_30 },
    { 0xbedb8c51, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65b8c57, SUC_lexicon_tags_11 },
    { 0x441b8c57, SUC_lexicon_tags_15 },
    { 0x54fb8c59, SUC_lexicon_tags_25 },
    { 0xd61b8c5a, SUC_lexicon_tags_2 },
    { 0x4a7b8c5b, SUC_lexicon_tags_25 },
    { 0x673b8c5b, SUC_lexicon_tags_68 },
    { 0xd07b8c59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x39db8c5f, SUC_lexicon_tags_47 },
    { 0x809b8c60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x611b8c62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13db8c67, SUC_lexicon_tags_43 },
    { 0xa6db8c68, SUC_lexicon_tags_12 },
    { 0x883b8c69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3b8c6d, SUC_lexicon_tags_14 },
    { 0x447b8c6e, SUC_lexicon_tags_11 },
    { 0x903b8c6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x223b8c71, SUC_lexicon_tags_26 },
    { 0x3a5b8c71, SUC_lexicon_tags_38 },
    { 0xf79b8c73, SUC_lexicon_tags_17 },
    { 0xfc3b8c72, SUC_lexicon_tags_10 },
    { 0x8f5b8c75, SUC_lexicon_tags_23 },
    { 0x149b8c76, SUC_lexicon_tags_37 },
    { 0x60bb8c75, SUC_lexicon_tags_1 },
    { 0xf0fb8c75, SUC_lexicon_tags_15 },
    { 0xe33b8c71, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5e3b8c7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fb8c7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb8c83, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x915b8c86, SUC_lexicon_tags_7 },
    { 0x349b8c87, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bb8c8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x637b8c8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407b8c91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3b8c95, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93b8c99, SUC_lexicon_tags_29 },
    { 0xd49b8c99, SUC_lexicon_tags_1 },
    { 0x339b8c9b, SUC_lexicon_tags_29 },
    { 0xccdb8c99, SUC_lexicon_tags_14 },
    { 0x9b7b8c9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76bb8ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x479b8ca2, SUC_lexicon_tags_1 },
    { 0x1e3b8ca3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b8ca9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a7b8cab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x40db8cad, SUC_lexicon_tags_30 },
    { 0x4c3b8cad, SUC_lexicon_tags_38 },
    { 0xdf7b8cad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fb8cb3, SUC_lexicon_tags_60 },
    { 0x14bb8cb3, SUC_lexicon_tags_19 },
    { 0xa69b8cb5, SUC_lexicon_tags_12 },
    { 0x96bb8cb4, SUC_lexicon_tags_27 },
    { 0x8efb8cb5, SUC_lexicon_tags_7 },
    { 0xc8fb8cb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x619b8cba, SUC_lexicon_tags_17 },
    { 0xbcbb8cbb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1b8cbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bb8cc3, SUC_lexicon_tags_2 },
    { 0xaafb8cc3, SUC_lexicon_tags_60 },
    { 0xf45b8cc5, SUC_lexicon_tags_32 },
    { 0xaf1b8cc6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x53fb8cc9, SUC_lexicon_tags_37 },
    { 0xed1b8cc9, SUC_lexicon_tags_15 },
    { 0xd01b8cc9, SUC_lexicon_tags_7 },
    { 0x845b8cca, SUC_lexicon_tags_2 },
    { 0x42bb8ccd, SUC_lexicon_tags_35 },
    { 0x347b8ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7b8cd3, SUC_lexicon_tags_36 },
    { 0xdefb8cd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8db8cd8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x45b8cdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1b8cde, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfb8ce1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9b8ce7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf6bb8ce9, SUC_lexicon_tags_19 },
    { 0x83bb8cea, SUC_lexicon_tags_25 },
    { 0xf91b8cea, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x3c1b8ced, SUC_lexicon_tags_10 },
    { 0x3c5b8cee, SUC_lexicon_tags_23 },
    { 0x9e9b8cef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63db8cf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bb8cf4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x31fb8cf6, SUC_lexicon_tags_38 },
    { 0x23b8cf7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x18bb8cfa, SUC_lexicon_tags_37 },
    { 0x969b8cfa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85b8d03, SUC_lexicon_tags_60 },
    { 0x505b8d04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bb8d0a, SUC_lexicon_tags_46 },
    { 0xf0db8d0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1b8d0e, SUC_lexicon_tags_47 },
    { 0xf19b8d0f, SUC_lexicon_tags_9 },
    { 0x89b8d10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5b8d13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dfb8d15, SUC_lexicon_tags_15 },
    { 0x7adb8d15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bb8d1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa43b8d1c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbb8d1f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebb8d22, SUC_lexicon_tags_4 },
    { 0x551b8d22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7b8d26, SUC_lexicon_tags_8 },
    { 0xe9db8d27, SUC_lexicon_tags_31 },
    { 0x129b8d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbb8d2c, SUC_lexicon_tags_14 },
    { 0xf85b8d2d, SUC_lexicon_tags_45 },
    { 0xbadb8d2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57b8d33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd25b8d35, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453b8d3d, SUC_lexicon_tags_17 },
    { 0x8f7b8d3d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fb8d41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x523b8d43, SUC_lexicon_tags_17 },
    { 0x7a3b8d44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd9fb8d46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddb8d4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x32fb8d4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3ebb8d50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659b8d54, SUC_lexicon_tags_13 },
    { 0xa37b8d55, SUC_lexicon_tags_14 },
    { 0x483b8d56, SUC_lexicon_tags_1 },
    { 0xe29b8d56, SUC_lexicon_tags_16 },
    { 0xfe3b8d56, SUC_lexicon_tags_35 },
    { 0xd93b8d59, SUC_lexicon_tags_65 },
    { 0xb75b8d55, SUC_lexicon_tags_0 },
    { 0x33b8d58, SUC_lexicon_tags_14 },
    { 0xd57b8d5c, SUC_lexicon_tags_47 },
    { 0xd13b8d5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x321b8d5f, SUC_lexicon_tags_18 },
    { 0xccdb8d5f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bb8d67, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa49b8d69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bb8d6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x525b8d6f, SUC_lexicon_tags_82 },
    { 0x3bbb8d70, SUC_lexicon_tags_34 },
    { 0xd69b8d70, SUC_lexicon_tags_25 },
    { 0x93fb8d71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf5b8d74, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc2bb8d76, SUC_lexicon_tags_43 },
    { 0xdc9b8d77, SUC_lexicon_tags_11 },
    { 0xa5db8d77, SUC_lexicon_tags_48 },
    { 0x6d9b8d76, SUC_lexicon_tags_1 },
    { 0x5adb8d7a, SUC_lexicon_tags_8 },
    { 0x23b8d7b, SUC_lexicon_tags_38 },
    { 0x547b8d7c, SUC_lexicon_tags_45 },
    { 0x309b8d7d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf97b8d7f, SUC_lexicon_tags_16 },
    { 0x225b8d80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76db8d88, SUC_lexicon_tags_1 },
    { 0x837b8d88, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xad1b8d8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x52bb8d8d, SUC_lexicon_tags_15 },
    { 0x9fbb8d8e, SUC_lexicon_tags_29 },
    { 0x373b8d8f, SUC_lexicon_tags_9 },
    { 0x7fdb8d8e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1b8d93, SUC_lexicon_tags_551 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffb8d96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb7fb8d98, SUC_lexicon_tags_31 },
    { 0xf19b8d99, SUC_lexicon_tags_2 },
    { 0xfe7b8d98, SUC_lexicon_tags_10 },
    { 0xdb9b8d9a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7b8d9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97b8da5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x421b8da7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bb8db1, SUC_lexicon_tags_44 },
    { 0x7adb8db2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7b8db5, SUC_lexicon_tags_68 },
    { 0xfcbb8db6, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x947b8db8, SUC_lexicon_tags_31 },
    { 0x6fb8db8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55b8dbd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99b8dc0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf97b8dc2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3b8dc6, SUC_lexicon_tags_8 },
    { 0x12bb8dc7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbb8dca, SUC_lexicon_tags_49 },
    { 0x33fb8dca, SUC_lexicon_tags_28 },
    { 0xe53b8dcc, SUC_lexicon_tags_25 },
    { 0xa6db8dcd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5b8dd2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6c3b8dd4, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0xb87b8dd6, SUC_lexicon_tags_15 },
    { 0x613b8dd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53b8ddf, SUC_lexicon_tags_60 },
    { 0xc4db8ddf, SUC_lexicon_tags_63 },
    { 0x743b8de1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35b8de3, SUC_lexicon_tags_1 },
    { 0x673b8de4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bb8de8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fb8ded, SUC_lexicon_tags_19 },
    { 0xab9b8dee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5b8df3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9b8df7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebb8dfa, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123b8dfe, SUC_lexicon_tags_25 },
    { 0x99db8dff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x533b8e01, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5b8e06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x17fb8e08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc29b8e0a, SUC_lexicon_tags_9 },
    { 0xac1b8e0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889b8e13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x56bb8e15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149b8e1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x455b8e1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39b8e20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x973b8e23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53b8e26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79b8e2b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x383b8e2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743b8e32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43fb8e34, SUC_lexicon_tags_11 },
    { 0x26db8e34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e7b8e37, SUC_lexicon_tags_15 },
    { 0x4bdb8e37, SUC_lexicon_tags_38 },
    { 0x9adb8e39, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x885b8e3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc25b8e3d, SUC_lexicon_tags_7 },
    { 0x275b8e3e, SUC_lexicon_tags_1 },
    { 0x87db8e3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x577b8e42, SUC_lexicon_tags_12 },
    { 0x44db8e43, SUC_lexicon_tags_1 },
    { 0x587b8e44, SUC_lexicon_tags_12 },
    { 0xe1bb8e45, SUC_lexicon_tags_40 },
    { 0x405b8e46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9fbb8e48, SUC_lexicon_tags_47 },
    { 0x677b8e49, SUC_lexicon_tags_43 },
    { 0xc8fb8e49, SUC_lexicon_tags_6 },
    { 0x62bb8e49, SUC_lexicon_tags_2 },
    { 0x34db8e4c, SUC_lexicon_tags_38 },
    { 0x531b8e4c, SUC_lexicon_tags_8 },
    { 0x36bb8e4e, SUC_lexicon_tags_1 },
    { 0xa59b8e4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9b8e56, SUC_lexicon_tags_30 },
    { 0x461b8e57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x171b8e5a, SUC_lexicon_tags_2 },
    { 0x4b1b8e5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf93b8e5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7b8e60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x343b8e63, SUC_lexicon_tags_1 },
    { 0xf69b8e64, SUC_lexicon_tags_15 },
    { 0x50fb8e65, SUC_lexicon_tags_17 },
    { 0x765b8e65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75b8e69, SUC_lexicon_tags_47 },
    { 0xaedb8e6a, SUC_lexicon_tags_7 },
    { 0x75fb8e6a, SUC_lexicon_tags_11 },
    { 0xfa5b8e6c, SUC_lexicon_tags_34 },
    { 0xf05b8e6c, SUC_lexicon_tags_19 },
    { 0x2cfb8e6c, SUC_lexicon_tags_60 },
    { 0xca1b8e6a, SUC_lexicon_tags_13 },
    { 0x729b8e70, SUC_lexicon_tags_2 },
    { 0x821b8e70, SUC_lexicon_tags_30 },
    { 0x7fbb8e72, SUC_lexicon_tags_46 },
    { 0x6a5b8e72, SUC_lexicon_tags_29 },
    { 0xe39b8e70, SUC_lexicon_tags_46 },
    { 0x94db8e70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x31db8e77, SUC_lexicon_tags_6 },
    { 0x6dfb8e78, SUC_lexicon_tags_17 },
    { 0xd49b8e79, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x677b8e7b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xee3b8e7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3b8e80, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673b8e91, SUC_lexicon_tags_27 },
    { 0x933b8e92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95b8e95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3b8e98, SUC_lexicon_tags_620 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5b8e9b, SUC_lexicon_tags_65 },
    { 0xa55b8e9b, SUC_lexicon_tags_5 },
    { 0x433b8e9d, SUC_lexicon_tags_0 },
    { 0xa11b8e9b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3b7b8ea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb11b8ea2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x893b8ea5, SUC_lexicon_tags_66 },
    { 0xf67b8ea6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5db8eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bb8eaf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xecbb8eb1, SUC_lexicon_tags_4 },
    { 0x559b8eb2, SUC_lexicon_tags_17 },
    { 0xbfbb8eb3, SUC_lexicon_tags_554 },
    { 0xed7b8eb3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325b8eb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b1b8eba, SUC_lexicon_tags_16 },
    { 0x2e5b8ebb, SUC_lexicon_tags_0 },
    { 0xe05b8ebc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3db8ebe, SUC_lexicon_tags_11 },
    { 0x209b8ebf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975b8ecc, SUC_lexicon_tags_62 },
    { 0x5fb8ecd, SUC_lexicon_tags_7 },
    { 0x5e7b8ecd, SUC_lexicon_tags_33 },
    { 0x565b8ecf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb9db8ed1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2b9b8ed3, SUC_lexicon_tags_58 },
    { 0x193b8ed4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e3b8ed6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bb8eda, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3b8ede, SUC_lexicon_tags_56 },
    { 0x611b8ede, SUC_lexicon_tags_1 },
    { 0xf89b8ede, SUC_lexicon_tags_47 },
    { 0x67bb8edf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67b8ee4, SUC_lexicon_tags_7 },
    { 0x23bb8ee5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc85b8ee7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd21b8ee9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3b8eed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x725b8ef0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x993b8ef3, SUC_lexicon_tags_13 },
    { 0xb8bb8ef3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x31bb8ef7, SUC_lexicon_tags_43 },
    { 0xca7b8ef8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313b8eff, SUC_lexicon_tags_664 },
    { 0x237b8eff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29b8f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189b8f07, SUC_lexicon_tags_1 },
    { 0x4cdb8f07, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfb7b8f0a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xea3b8f0c, SUC_lexicon_tags_37 },
    { 0x5c3b8f0d, SUC_lexicon_tags_25 },
    { 0x981b8f0e, SUC_lexicon_tags_19 },
    { 0xc03b8f0c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x217b8f11, SUC_lexicon_tags_34 },
    { 0x88db8f11, SUC_lexicon_tags_27 },
    { 0x24bb8f13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81b8f19, SUC_lexicon_tags_0 },
    { 0x631b8f19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3b8f20, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x74bb8f22, SUC_lexicon_tags_46 },
    { 0xde7b8f22, SUC_lexicon_tags_11 },
    { 0xde5b8f24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01b8f29, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa4fb8f2b, SUC_lexicon_tags_29 },
    { 0x171b8f2c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8bbb8f2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbb8f31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f1b8f33, SUC_lexicon_tags_17 },
    { 0xa41b8f34, SUC_lexicon_tags_47 },
    { 0x3a5b8f33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579b8f3f, SUC_lexicon_tags_25 },
    { 0x723b8f40, SUC_lexicon_tags_37 },
    { 0xac3b8f40, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x3efb8f43, SUC_lexicon_tags_574 },
    { 0, NULL },
    { 0x5f3b8f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6db8f47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9db8f4a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edb8f4e, SUC_lexicon_tags_9 },
    { 0x76fb8f4e, SUC_lexicon_tags_11 },
    { 0x39db8f50, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fb8f53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x731b8f55, SUC_lexicon_tags_26 },
    { 0xa6fb8f56, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x64bb8f58, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bb8f5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65b8f61, SUC_lexicon_tags_37 },
    { 0x381b8f62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd03b8f64, SUC_lexicon_tags_1 },
    { 0x899b8f64, SUC_lexicon_tags_25 },
    { 0x789b8f66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239b8f6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd47b8f6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bb8f70, SUC_lexicon_tags_8 },
    { 0xac1b8f70, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91b8f78, SUC_lexicon_tags_379 },
    { 0x4b5b8f79, SUC_lexicon_tags_14 },
    { 0xe43b8f7a, SUC_lexicon_tags_34 },
    { 0x1c7b8f7b, SUC_lexicon_tags_30 },
    { 0x513b8f7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc57b8f7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x593b8f80, SUC_lexicon_tags_7 },
    { 0x509b8f81, SUC_lexicon_tags_33 },
    { 0x38fb8f82, SUC_lexicon_tags_75 },
    { 0x3fbb8f82, SUC_lexicon_tags_25 },
    { 0xdfbb8f83, SUC_lexicon_tags_35 },
    { 0xd67b8f85, SUC_lexicon_tags_68 },
    { 0x331b8f82, SUC_lexicon_tags_84 },
    { 0x55b8f83, SUC_lexicon_tags_11 },
    { 0xe57b8f88, SUC_lexicon_tags_10 },
    { 0x14bb8f89, SUC_lexicon_tags_65 },
    { 0x23fb8f85, SUC_lexicon_tags_37 },
    { 0x7c3b8f88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3b8f8f, SUC_lexicon_tags_37 },
    { 0x38fb8f90, SUC_lexicon_tags_37 },
    { 0xde3b8f91, SUC_lexicon_tags_43 },
    { 0xa53b8f92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1b8f97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81b8fa0, SUC_lexicon_tags_47 },
    { 0xef9b8fa1, SUC_lexicon_tags_35 },
    { 0xa9db8fa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e7b8fa4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3b8fa7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5f7b8fa9, SUC_lexicon_tags_17 },
    { 0x3ddb8faa, SUC_lexicon_tags_23 },
    { 0xadbb8fab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83b8fae, SUC_lexicon_tags_27 },
    { 0x545b8faf, SUC_lexicon_tags_15 },
    { 0x435b8fae, SUC_lexicon_tags_0 },
    { 0xbabb8fb1, SUC_lexicon_tags_10 },
    { 0xa89b8fb1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3b8fbb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bb8fc4, SUC_lexicon_tags_11 },
    { 0x273b8fc4, SUC_lexicon_tags_2 },
    { 0xa1bb8fc6, SUC_lexicon_tags_61 },
    { 0x309b8fc6, SUC_lexicon_tags_11 },
    { 0x51b8fc8, SUC_lexicon_tags_17 },
    { 0x157b8fc9, SUC_lexicon_tags_37 },
    { 0x22bb8fca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cfb8fcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9b8fcf, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xf53b8fd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9b8fd5, SUC_lexicon_tags_0 },
    { 0xf6fb8fd6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3b8fda, SUC_lexicon_tags_55 },
    { 0xf0fb8fda, SUC_lexicon_tags_9 },
    { 0xc03b8fdc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951b8fe1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfb8fe4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb8fe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efb8fec, SUC_lexicon_tags_14 },
    { 0xe71b8fec, SUC_lexicon_tags_55 },
    { 0x76bb8fed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x651b8ff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3b8ff6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9b8ffa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1efb8ffc, SUC_lexicon_tags_8 },
    { 0xefbb8ffc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1b9000, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697b9004, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff7b9006, SUC_lexicon_tags_727 },
    { 0x229b9007, SUC_lexicon_tags_1 },
    { 0x901b9006, SUC_lexicon_tags_9 },
    { 0xba1b9008, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5edb900b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabb900f, SUC_lexicon_tags_30 },
    { 0xab7b9010, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223b9016, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddb901c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfb9024, SUC_lexicon_tags_51 },
    { 0xda3b9025, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3b9029, SUC_lexicon_tags_15 },
    { 0x277b902a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb7bb902c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x787b902e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3b9032, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4f9b9034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfb9038, SUC_lexicon_tags_107 },
    { 0xf87b9039, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48fb903b, SUC_lexicon_tags_250 },
    { 0x131b903c, SUC_lexicon_tags_37 },
    { 0x2e7b903d, SUC_lexicon_tags_7 },
    { 0x1b5b903b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7b9042, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77b9047, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f9b9049, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1b904d, SUC_lexicon_tags_1 },
    { 0x62bb904e, SUC_lexicon_tags_21 },
    { 0x537b904f, SUC_lexicon_tags_37 },
    { 0x7cdb9050, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde1b9052, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1b9056, SUC_lexicon_tags_31 },
    { 0xa73b9056, SUC_lexicon_tags_2 },
    { 0xbabb9058, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675b905e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa19b9060, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd55b9062, SUC_lexicon_tags_47 },
    { 0x6b3b9062, SUC_lexicon_tags_30 },
    { 0x8b5b9063, SUC_lexicon_tags_43 },
    { 0xebb9064, SUC_lexicon_tags_46 },
    { 0x167b9064, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191b906e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fb9073, SUC_lexicon_tags_23 },
    { 0x469b9074, SUC_lexicon_tags_25 },
    { 0x991b9075, SUC_lexicon_tags_114 },
    { 0xd75b9075, SUC_lexicon_tags_2 },
    { 0x113b9073, SUC_lexicon_tags_33 },
    { 0x15b9074, SUC_lexicon_tags_34 },
    { 0xe31b9073, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9b907e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bb908e, SUC_lexicon_tags_17 },
    { 0xe39b908e, SUC_lexicon_tags_25 },
    { 0xb23b908e, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86db9095, SUC_lexicon_tags_23 },
    { 0xd3b9096, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afb9099, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fb909d, SUC_lexicon_tags_19 },
    { 0x6ffb909d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bdb90a0, SUC_lexicon_tags_8 },
    { 0x6c3b90a1, SUC_lexicon_tags_17 },
    { 0x5b7b90a2, SUC_lexicon_tags_7 },
    { 0xa49b90a3, SUC_lexicon_tags_34 },
    { 0xd0db90a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7b90a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5b90ae, SUC_lexicon_tags_1 },
    { 0xe9b90ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81b90b1, SUC_lexicon_tags_33 },
    { 0x35fb90b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afb90b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bb90bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fb90c4, SUC_lexicon_tags_30 },
    { 0xd1bb90c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1b90c8, SUC_lexicon_tags_33 },
    { 0x91fb90c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x283b90cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55b90ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2db90d1, SUC_lexicon_tags_32 },
    { 0xe6db90d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fb90d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafb90d9, SUC_lexicon_tags_3 },
    { 0xc85b90da, SUC_lexicon_tags_12 },
    { 0x921b90da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47db90e2, SUC_lexicon_tags_13 },
    { 0xd6db90e3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8e7b90e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x655b90e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e5b90e9, SUC_lexicon_tags_2 },
    { 0x581b90e9, SUC_lexicon_tags_37 },
    { 0xd27b90ea, SUC_lexicon_tags_8 },
    { 0x183b90e9, SUC_lexicon_tags_0 },
    { 0xfb90ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x553b90ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbb90f2, SUC_lexicon_tags_2 },
    { 0xbe7b90f2, SUC_lexicon_tags_14 },
    { 0xb9bb90f4, SUC_lexicon_tags_27 },
    { 0x925b90f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x767b90f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x27b90fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05b90fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x719b9100, SUC_lexicon_tags_15 },
    { 0xedbb9100, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x751b9103, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x74db9105, SUC_lexicon_tags_12 },
    { 0xd8fb9105, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9e7b9108, SUC_lexicon_tags_29 },
    { 0xcf9b9109, SUC_lexicon_tags_39 },
    { 0x6c7b910a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273b910e, SUC_lexicon_tags_7 },
    { 0xfd9b910f, SUC_lexicon_tags_31 },
    { 0x7ffb9110, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x713b9113, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70bb9115, SUC_lexicon_tags_25 },
    { 0x783b9116, SUC_lexicon_tags_17 },
    { 0xe39b9117, SUC_lexicon_tags_37 },
    { 0xda1b9118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fb911b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31b911f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdb9126, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127b912a, SUC_lexicon_tags_14 },
    { 0xbe5b912b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bb912e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29fb9130, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fb9133, SUC_lexicon_tags_15 },
    { 0xb43b9134, SUC_lexicon_tags_17 },
    { 0x97bb9135, SUC_lexicon_tags_17 },
    { 0xbf5b9136, SUC_lexicon_tags_33 },
    { 0xe57b9135, SUC_lexicon_tags_30 },
    { 0x1ffb9137, SUC_lexicon_tags_7 },
    { 0x22fb9139, SUC_lexicon_tags_55 },
    { 0x1adb9139, SUC_lexicon_tags_25 },
    { 0xcf9b9133, SUC_lexicon_tags_1 },
    { 0xf0bb913c, SUC_lexicon_tags_14 },
    { 0xbedb9138, SUC_lexicon_tags_28 },
    { 0x96db913e, SUC_lexicon_tags_2 },
    { 0xc6bb913f, SUC_lexicon_tags_30 },
    { 0xa39b9139, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fb9143, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x31bb9146, SUC_lexicon_tags_47 },
    { 0xdb1b9147, SUC_lexicon_tags_31 },
    { 0xd15b9148, SUC_lexicon_tags_2 },
    { 0xc9db9149, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf9b914b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf87b914d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907b9151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x417b9154, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db9158, SUC_lexicon_tags_14 },
    { 0x777b9159, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bb915d, SUC_lexicon_tags_27 },
    { 0x637b915e, SUC_lexicon_tags_28 },
    { 0x77fb915e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1b9165, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfb916a, SUC_lexicon_tags_8 },
    { 0x79db916b, SUC_lexicon_tags_17 },
    { 0xa9bb916c, SUC_lexicon_tags_17 },
    { 0xe2db916b, SUC_lexicon_tags_7 },
    { 0x45db916e, SUC_lexicon_tags_64 },
    { 0x53fb916f, SUC_lexicon_tags_19 },
    { 0xd5bb916e, SUC_lexicon_tags_14 },
    { 0x41bb916f, SUC_lexicon_tags_25 },
    { 0x157b9172, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc2db9174, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xb4fb9176, SUC_lexicon_tags_37 },
    { 0xa55b9176, SUC_lexicon_tags_43 },
    { 0x4adb9178, SUC_lexicon_tags_45 },
    { 0xcd5b9179, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93b9181, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bb918c, SUC_lexicon_tags_45 },
    { 0xb87b918d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79b9190, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31bb9192, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bb9199, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x27bb919b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fb919f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31b91a4, SUC_lexicon_tags_12 },
    { 0xb07b91a5, SUC_lexicon_tags_49 },
    { 0x937b91a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625b91aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adb91ad, SUC_lexicon_tags_56 },
    { 0x75fb91ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bb91b1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485b91b8, SUC_lexicon_tags_4 },
    { 0x475b91b9, SUC_lexicon_tags_32 },
    { 0x7cbb91b9, SUC_lexicon_tags_9 },
    { 0x75b91b9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1b91bf, SUC_lexicon_tags_17 },
    { 0x20fb91c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdb91c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ffb91c6, SUC_lexicon_tags_55 },
    { 0x4dfb91c7, SUC_lexicon_tags_37 },
    { 0x525b91c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfc7b91ca, SUC_lexicon_tags_10 },
    { 0x7ffb91cb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9b91cf, SUC_lexicon_tags_56 },
    { 0x30fb91d0, SUC_lexicon_tags_2 },
    { 0x19bb91d1, SUC_lexicon_tags_11 },
    { 0xdfbb91d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc49b91d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x353b91d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149b91de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9b91e2, SUC_lexicon_tags_1 },
    { 0x90bb91e3, SUC_lexicon_tags_15 },
    { 0x18db91e4, SUC_lexicon_tags_23 },
    { 0x371b91e5, SUC_lexicon_tags_65 },
    { 0x8a5b91e6, SUC_lexicon_tags_25 },
    { 0xe0bb91e3, SUC_lexicon_tags_0 },
    { 0x341b91e7, SUC_lexicon_tags_11 },
    { 0xbfbb91e8, SUC_lexicon_tags_15 },
    { 0xfabb91e3, SUC_lexicon_tags_29 },
    { 0xfafb91eb, SUC_lexicon_tags_33 },
    { 0x5d9b91ec, SUC_lexicon_tags_220 },
    { 0xe5bb91e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efb91f3, SUC_lexicon_tags_48 },
    { 0x59b91f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x347b91f6, SUC_lexicon_tags_17 },
    { 0x213b91f7, SUC_lexicon_tags_47 },
    { 0x509b91f8, SUC_lexicon_tags_12 },
    { 0x8abb91f6, SUC_lexicon_tags_84 },
    { 0x925b91fa, SUC_lexicon_tags_17 },
    { 0xfc1b91fb, SUC_lexicon_tags_192 },
    { 0x2e7b91fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75b91fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33b9201, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39b920e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5b9211, SUC_lexicon_tags_8 },
    { 0xa75b9212, SUC_lexicon_tags_30 },
    { 0x56db9212, SUC_lexicon_tags_4 },
    { 0xd63b9214, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf61b9216, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x34db9218, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9db921c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67b9220, SUC_lexicon_tags_37 },
    { 0xb71b9220, SUC_lexicon_tags_1 },
    { 0x79bb9222, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf3b9224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26db9227, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651b922d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadb9231, SUC_lexicon_tags_65 },
    { 0x375b9232, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1f9b9234, SUC_lexicon_tags_34 },
    { 0x29db9235, SUC_lexicon_tags_15 },
    { 0x8a1b9236, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fb9239, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffb923b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459b923f, SUC_lexicon_tags_33 },
    { 0x51bb9240, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf3b9242, SUC_lexicon_tags_25 },
    { 0x835b9242, SUC_lexicon_tags_29 },
    { 0x997b9244, SUC_lexicon_tags_12 },
    { 0xb57b9242, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbb924b, SUC_lexicon_tags_15 },
    { 0xecdb924b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3b924f, SUC_lexicon_tags_65 },
    { 0x9b9b9250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbb9258, SUC_lexicon_tags_31 },
    { 0xb9db9259, SUC_lexicon_tags_2 },
    { 0xb7bb925a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x767b925c, SUC_lexicon_tags_28 },
    { 0x381b925c, SUC_lexicon_tags_60 },
    { 0xa93b925e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239b926a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c3b926c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf25b926e, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1b9271, SUC_lexicon_tags_25 },
    { 0x53db9272, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23db9276, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3b927c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7b9284, SUC_lexicon_tags_82 },
    { 0x991b9285, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33b9287, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb928e, SUC_lexicon_tags_15 },
    { 0x749b928f, SUC_lexicon_tags_25 },
    { 0x7d5b928f, SUC_lexicon_tags_17 },
    { 0xc21b9291, SUC_lexicon_tags_41 },
    { 0x5a1b9292, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x275b9294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75b92a2, SUC_lexicon_tags_11 },
    { 0xec1b92a3, SUC_lexicon_tags_34 },
    { 0x6e7b92a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b9b92a6, SUC_lexicon_tags_32 },
    { 0x467b92a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9db92a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1b92ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fb92b0, SUC_lexicon_tags_25 },
    { 0x501b92b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fdb92b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1b92b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x383b92b8, SUC_lexicon_tags_23 },
    { 0xcb5b92b9, SUC_lexicon_tags_63 },
    { 0xb35b92b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdb92c3, SUC_lexicon_tags_17 },
    { 0xb8db92c4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x96bb92c7, SUC_lexicon_tags_8 },
    { 0xd6bb92c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25db92cc, SUC_lexicon_tags_19 },
    { 0x3ebb92cd, SUC_lexicon_tags_37 },
    { 0x2e5b92ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237b92d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbb92d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93b92db, SUC_lexicon_tags_33 },
    { 0x20bb92dc, SUC_lexicon_tags_23 },
    { 0x8e7b92dd, SUC_lexicon_tags_48 },
    { 0xec9b92db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdb92e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5b92e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1b92ea, SUC_lexicon_tags_25 },
    { 0x1cdb92eb, SUC_lexicon_tags_63 },
    { 0xc0fb92ec, SUC_lexicon_tags_1 },
    { 0xc8fb92ec, SUC_lexicon_tags_27 },
    { 0xb7db92ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349b92fa, SUC_lexicon_tags_37 },
    { 0xef7b92fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991b92ff, SUC_lexicon_tags_6 },
    { 0x8b1b9300, SUC_lexicon_tags_103 },
    { 0xd07b9301, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3e1b9303, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5c7b9305, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bb9309, SUC_lexicon_tags_60 },
    { 0xfc7b930a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7b930d, SUC_lexicon_tags_15 },
    { 0xfbbb930e, SUC_lexicon_tags_46 },
    { 0x21fb930f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefb9314, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc61b9316, SUC_lexicon_tags_17 },
    { 0xc07b9317, SUC_lexicon_tags_15 },
    { 0x6fbb9317, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961b931e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efb9321, SUC_lexicon_tags_2 },
    { 0xa4fb9322, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3b9325, SUC_lexicon_tags_17 },
    { 0x919b9325, SUC_lexicon_tags_9 },
    { 0x33db9325, SUC_lexicon_tags_33 },
    { 0xc0bb9328, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507b932d, SUC_lexicon_tags_11 },
    { 0xe1bb932d, SUC_lexicon_tags_11 },
    { 0xf93b932f, SUC_lexicon_tags_8 },
    { 0x627b932d, SUC_lexicon_tags_12 },
    { 0xac3b9331, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa95b9333, SUC_lexicon_tags_14 },
    { 0xa43b9333, SUC_lexicon_tags_17 },
    { 0x711b9335, SUC_lexicon_tags_253 },
    { 0x881b9333, SUC_lexicon_tags_14 },
    { 0xe4fb9337, SUC_lexicon_tags_2 },
    { 0x8ebb9335, SUC_lexicon_tags_60 },
    { 0x855b9333, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fb933e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xe57b9340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25fb9343, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29b9347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c5b9349, SUC_lexicon_tags_1 },
    { 0xfcbb9349, SUC_lexicon_tags_71 },
    { 0x51fb934b, SUC_lexicon_tags_4 },
    { 0xaf1b934b, SUC_lexicon_tags_12 },
    { 0x67bb934d, SUC_lexicon_tags_47 },
    { 0x259b934e, SUC_lexicon_tags_4 },
    { 0xa2fb934f, SUC_lexicon_tags_7 },
    { 0x7c7b9350, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319b935a, SUC_lexicon_tags_31 },
    { 0x4e9b935a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fb9360, SUC_lexicon_tags_2 },
    { 0xf87b9361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdb9364, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x21bb9366, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215b936b, SUC_lexicon_tags_27 },
    { 0xb4db936b, SUC_lexicon_tags_30 },
    { 0x52fb936c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa11b936f, SUC_lexicon_tags_17 },
    { 0xa41b9370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89b9374, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x24db9376, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69b9379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x755b937b, SUC_lexicon_tags_1 },
    { 0xe13b937b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1b937e, SUC_lexicon_tags_37 },
    { 0x8ffb937f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fb9383, SUC_lexicon_tags_25 },
    { 0xa65b9383, SUC_lexicon_tags_15 },
    { 0x7b9b9385, SUC_lexicon_tags_19 },
    { 0xffbb9383, SUC_lexicon_tags_11 },
    { 0x81b9387, SUC_lexicon_tags_33 },
    { 0x3cbb9384, SUC_lexicon_tags_37 },
    { 0xd01b9389, SUC_lexicon_tags_6 },
    { 0x80bb938a, SUC_lexicon_tags_11 },
    { 0x8c1b938b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485b9391, SUC_lexicon_tags_49 },
    { 0x111b9392, SUC_lexicon_tags_14 },
    { 0x43bb9391, SUC_lexicon_tags_27 },
    { 0x807b9394, SUC_lexicon_tags_24 },
    { 0x8a5b9392, SUC_lexicon_tags_15 },
    { 0x33bb9396, SUC_lexicon_tags_25 },
    { 0x31b9391, SUC_lexicon_tags_26 },
    { 0x815b9398, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875b939c, SUC_lexicon_tags_79 },
    { 0xb81b939c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe5b939f, SUC_lexicon_tags_25 },
    { 0x1c9b93a0, SUC_lexicon_tags_87 },
    { 0xc7db939f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7db93aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03b93b3, SUC_lexicon_tags_47 },
    { 0x12db93b4, SUC_lexicon_tags_29 },
    { 0x9cfb93b5, SUC_lexicon_tags_14 },
    { 0x7f3b93b6, SUC_lexicon_tags_31 },
    { 0x2ebb93b7, SUC_lexicon_tags_106 },
    { 0x40db93b8, SUC_lexicon_tags_25 },
    { 0xefb93b9, SUC_lexicon_tags_4 },
    { 0x77b93ba, SUC_lexicon_tags_26 },
    { 0x4ebb93bb, SUC_lexicon_tags_1 },
    { 0x4dbb93b6, SUC_lexicon_tags_3 },
    { 0xea1b93b4, SUC_lexicon_tags_17 },
    { 0xd7fb93ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fb93c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ddb93c6, SUC_lexicon_tags_21 },
    { 0xeadb93c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521b93cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81b93d2, SUC_lexicon_tags_17 },
    { 0x6bfb93d3, SUC_lexicon_tags_38 },
    { 0xa39b93d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x749b93d6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6bfb93d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47b93db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39db93dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5db93e0, SUC_lexicon_tags_87 },
    { 0x373b93e1, SUC_lexicon_tags_23 },
    { 0x465b93e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddb93ea, SUC_lexicon_tags_7 },
    { 0x839b93eb, SUC_lexicon_tags_63 },
    { 0xfb93ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bb93f0, SUC_lexicon_tags_47 },
    { 0xf31b93f0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89b93f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14bb93fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4db93fc, SUC_lexicon_tags_1 },
    { 0x703b93fd, SUC_lexicon_tags_37 },
    { 0x8ebb93fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbb5b9400, SUC_lexicon_tags_49 },
    { 0x2f7b9401, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb53b9403, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbffb9405, SUC_lexicon_tags_0 },
    { 0x545b9405, SUC_lexicon_tags_14 },
    { 0xb8bb9407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd5b9409, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfb940c, SUC_lexicon_tags_16 },
    { 0xd6fb940d, SUC_lexicon_tags_12 },
    { 0xafdb940e, SUC_lexicon_tags_17 },
    { 0x64db940e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9b9418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c5b941a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77b9420, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x877b9423, SUC_lexicon_tags_21 },
    { 0x40bb9424, SUC_lexicon_tags_9 },
    { 0xaa9b9425, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebb9428, SUC_lexicon_tags_12 },
    { 0xd45b9428, SUC_lexicon_tags_11 },
    { 0x925b9429, SUC_lexicon_tags_25 },
    { 0x171b942b, SUC_lexicon_tags_37 },
    { 0xeabb9429, SUC_lexicon_tags_39 },
    { 0x45b942d, SUC_lexicon_tags_66 },
    { 0x809b942b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6db9431, SUC_lexicon_tags_9 },
    { 0x47b9432, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8db9435, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc3b9437, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa35b9439, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb3db943b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdb943e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b5b9440, SUC_lexicon_tags_31 },
    { 0x629b9441, SUC_lexicon_tags_19 },
    { 0xa5fb9442, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x791b9445, SUC_lexicon_tags_25 },
    { 0x4b9b9446, SUC_lexicon_tags_7 },
    { 0xd6db9446, SUC_lexicon_tags_25 },
    { 0xc79b9448, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9b944e, SUC_lexicon_tags_82 },
    { 0xe7b944f, SUC_lexicon_tags_11 },
    { 0x9e3b9450, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeefb9453, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68fb9455, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bb9458, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdb945c, SUC_lexicon_tags_12 },
    { 0xe7db945d, SUC_lexicon_tags_0 },
    { 0xc27b945c, SUC_lexicon_tags_14 },
    { 0xa4fb945f, SUC_lexicon_tags_15 },
    { 0x5e5b945f, SUC_lexicon_tags_48 },
    { 0x847b9461, SUC_lexicon_tags_45 },
    { 0x963b9462, SUC_lexicon_tags_37 },
    { 0x69fb9463, SUC_lexicon_tags_32 },
    { 0x3f3b9463, SUC_lexicon_tags_47 },
    { 0xd3b945d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619b946a, SUC_lexicon_tags_61 },
    { 0xf25b946b, SUC_lexicon_tags_17 },
    { 0x8cfb946c, SUC_lexicon_tags_13 },
    { 0x9d9b946d, SUC_lexicon_tags_9 },
    { 0x635b946e, SUC_lexicon_tags_45 },
    { 0x26bb946f, SUC_lexicon_tags_7 },
    { 0x5ddb946c, SUC_lexicon_tags_45 },
    { 0x6cfb9471, SUC_lexicon_tags_47 },
    { 0xf4db946c, SUC_lexicon_tags_12 },
    { 0x791b9473, SUC_lexicon_tags_77 },
    { 0x65fb946e, SUC_lexicon_tags_25 },
    { 0x37fb9475, SUC_lexicon_tags_45 },
    { 0x931b9476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19b947a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41b947e, SUC_lexicon_tags_31 },
    { 0x301b947f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3b9484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfadb9486, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcefb9488, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcdb948a, SUC_lexicon_tags_29 },
    { 0xc5fb948b, SUC_lexicon_tags_49 },
    { 0x571b948b, SUC_lexicon_tags_84 },
    { 0x851b948c, SUC_lexicon_tags_63 },
    { 0x8f3b948c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf65b9491, SUC_lexicon_tags_25 },
    { 0xed5b9491, SUC_lexicon_tags_7 },
    { 0x17bb9493, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4ddb9495, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x813b9497, SUC_lexicon_tags_218 },
    { 0xe57b9498, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749b949f, SUC_lexicon_tags_37 },
    { 0xe6fb94a0, SUC_lexicon_tags_11 },
    { 0xc69b94a1, SUC_lexicon_tags_12 },
    { 0xd6db94a2, SUC_lexicon_tags_194 },
    { 0xa13b94a1, SUC_lexicon_tags_45 },
    { 0x6d7b94a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8edb94a6, SUC_lexicon_tags_1 },
    { 0x61b94a6, SUC_lexicon_tags_11 },
    { 0x775b94a8, SUC_lexicon_tags_13 },
    { 0xdf1b94a8, SUC_lexicon_tags_1 },
    { 0x3a1b94aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd75b94ac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdb94b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13b94b5, SUC_lexicon_tags_43 },
    { 0xa59b94b6, SUC_lexicon_tags_39 },
    { 0xd7fb94b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x961b94ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x307b94bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85b94c4, SUC_lexicon_tags_15 },
    { 0x2b7b94c5, SUC_lexicon_tags_6 },
    { 0xd07b94c4, SUC_lexicon_tags_1 },
    { 0x311b94c7, SUC_lexicon_tags_1 },
    { 0x517b94c8, SUC_lexicon_tags_11 },
    { 0xff5b94c9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1b94cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51b94d0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fb94d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9b94d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36db94de, SUC_lexicon_tags_10 },
    { 0xb49b94df, SUC_lexicon_tags_7 },
    { 0xdd3b94df, SUC_lexicon_tags_47 },
    { 0x6dbb94e1, SUC_lexicon_tags_11 },
    { 0x2bbb94df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55b94e5, SUC_lexicon_tags_12 },
    { 0x7afb94e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x50db94e8, SUC_lexicon_tags_61 },
    { 0x387b94e8, SUC_lexicon_tags_27 },
    { 0x47fb94ea, SUC_lexicon_tags_49 },
    { 0x925b94e9, SUC_lexicon_tags_14 },
    { 0x38fb94eb, SUC_lexicon_tags_37 },
    { 0x125b94ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9b94f1, SUC_lexicon_tags_77 },
    { 0x16db94f2, SUC_lexicon_tags_10 },
    { 0x265b94f2, SUC_lexicon_tags_104 },
    { 0x9cdb94f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6bdb94f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97b94fa, SUC_lexicon_tags_37 },
    { 0xe47b94fb, SUC_lexicon_tags_23 },
    { 0xacdb94fc, SUC_lexicon_tags_14 },
    { 0x921b94fc, SUC_lexicon_tags_15 },
    { 0x987b94fc, SUC_lexicon_tags_7 },
    { 0x683b94fa, SUC_lexicon_tags_1 },
    { 0x237b9500, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7b9509, SUC_lexicon_tags_7 },
    { 0xb65b950a, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x941b950c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfb9510, SUC_lexicon_tags_7 },
    { 0xed9b9511, SUC_lexicon_tags_21 },
    { 0xc65b9512, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7b951a, SUC_lexicon_tags_17 },
    { 0x573b951b, SUC_lexicon_tags_14 },
    { 0x19bb951c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebb951f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635b9527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5b952b, SUC_lexicon_tags_53 },
    { 0xab7b952c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5b9536, SUC_lexicon_tags_10 },
    { 0x861b9536, SUC_lexicon_tags_38 },
    { 0x2cbb9537, SUC_lexicon_tags_11 },
    { 0x9b9539, SUC_lexicon_tags_1 },
    { 0x8b1b953a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd7b953c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe29b953e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa5b9540, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301b9544, SUC_lexicon_tags_47 },
    { 0x873b9544, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7b9548, SUC_lexicon_tags_1 },
    { 0x495b9549, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x997b954b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fb954e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4abb9550, SUC_lexicon_tags_1 },
    { 0x607b9551, SUC_lexicon_tags_11 },
    { 0x1a3b9550, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5adb9554, SUC_lexicon_tags_17 },
    { 0x497b9555, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdb9559, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc13b955b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697b955f, SUC_lexicon_tags_37 },
    { 0xb5fb955f, SUC_lexicon_tags_15 },
    { 0xf23b9561, SUC_lexicon_tags_11 },
    { 0x671b9561, SUC_lexicon_tags_43 },
    { 0x127b9560, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e9b9565, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf73b9567, SUC_lexicon_tags_45 },
    { 0x7f9b9568, SUC_lexicon_tags_25 },
    { 0x3e5b9568, SUC_lexicon_tags_37 },
    { 0xf2bb956a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4b5b956c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xff3b956e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fb9571, SUC_lexicon_tags_23 },
    { 0x57b9571, SUC_lexicon_tags_15 },
    { 0xa1db9573, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435b957b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25b9581, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfb9584, SUC_lexicon_tags_17 },
    { 0xefbb9585, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x937b9588, SUC_lexicon_tags_0 },
    { 0xb8bb9588, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5db958b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x211b958e, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835b9592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bb9596, SUC_lexicon_tags_37 },
    { 0xfb9b9597, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x737b9599, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13b959e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01b95a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395b95a9, SUC_lexicon_tags_15 },
    { 0x7dbb95aa, SUC_lexicon_tags_63 },
    { 0xab1b95aa, SUC_lexicon_tags_14 },
    { 0x50fb95a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fb95b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x969b95b6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7b95ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5b95be, SUC_lexicon_tags_37 },
    { 0xe25b95bf, SUC_lexicon_tags_17 },
    { 0xb1db95bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1b95c3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x6ebb95c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a3b95c7, SUC_lexicon_tags_6 },
    { 0xfbdb95c7, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759b95cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75b95d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77b95d6, SUC_lexicon_tags_33 },
    { 0x63b95d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543b95df, SUC_lexicon_tags_18 },
    { 0x579b95df, SUC_lexicon_tags_33 },
    { 0x3a5b95e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fb95e6, SUC_lexicon_tags_37 },
    { 0x5b5b95e7, SUC_lexicon_tags_12 },
    { 0x6fbb95e8, SUC_lexicon_tags_25 },
    { 0x93db95e9, SUC_lexicon_tags_25 },
    { 0xa21b95ea, SUC_lexicon_tags_7 },
    { 0x6ffb95e8, SUC_lexicon_tags_14 },
    { 0x1d5b95ea, SUC_lexicon_tags_15 },
    { 0x439b95ed, SUC_lexicon_tags_15 },
    { 0xe99b95ee, SUC_lexicon_tags_37 },
    { 0x681b95ef, SUC_lexicon_tags_29 },
    { 0xae9b95f0, SUC_lexicon_tags_63 },
    { 0xb63b95ed, SUC_lexicon_tags_11 },
    { 0x6d3b95f2, SUC_lexicon_tags_11 },
    { 0xf57b95f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x11fb95f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc9fb95f7, SUC_lexicon_tags_14 },
    { 0x5cdb95f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69db95fa, SUC_lexicon_tags_16 },
    { 0xc7db95fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9b95fe, SUC_lexicon_tags_46 },
    { 0x61db95ff, SUC_lexicon_tags_1 },
    { 0x6f3b9600, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x30db9602, SUC_lexicon_tags_87 },
    { 0xb53b9603, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abb9607, SUC_lexicon_tags_37 },
    { 0xc21b9607, SUC_lexicon_tags_85 },
    { 0x667b9609, SUC_lexicon_tags_25 },
    { 0x229b9607, SUC_lexicon_tags_25 },
    { 0xd5db960b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x531b960d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fb9610, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x551b9612, SUC_lexicon_tags_60 },
    { 0x913b9612, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4d3b9615, SUC_lexicon_tags_22 },
    { 0xb99b9616, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b961e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6fb9620, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd19b9622, SUC_lexicon_tags_27 },
    { 0x595b9623, SUC_lexicon_tags_7 },
    { 0x735b9624, SUC_lexicon_tags_43 },
    { 0x3b5b9624, SUC_lexicon_tags_25 },
    { 0xa6db9626, SUC_lexicon_tags_2 },
    { 0xde5b9627, SUC_lexicon_tags_11 },
    { 0xbb1b9626, SUC_lexicon_tags_43 },
    { 0x591b9629, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa45b962b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x8b3b962d, SUC_lexicon_tags_16 },
    { 0x9bdb962e, SUC_lexicon_tags_25 },
    { 0xbcfb962d, SUC_lexicon_tags_25 },
    { 0xb8db9630, SUC_lexicon_tags_8 },
    { 0xd91b962f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bb9634, SUC_lexicon_tags_4 },
    { 0x871b9635, SUC_lexicon_tags_47 },
    { 0xf5db9634, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357b963d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfb9642, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71b9645, SUC_lexicon_tags_1 },
    { 0x31db9646, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbb964c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd5fb964e, SUC_lexicon_tags_61 },
    { 0x8afb964e, SUC_lexicon_tags_9 },
    { 0x233b9650, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfedb9652, SUC_lexicon_tags_7 },
    { 0x127b9653, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57b9658, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17b9662, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563b9668, SUC_lexicon_tags_12 },
    { 0xc77b9669, SUC_lexicon_tags_55 },
    { 0x42bb9668, SUC_lexicon_tags_27 },
    { 0x74fb966a, SUC_lexicon_tags_33 },
    { 0xd81b966a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbb9671, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x377b9673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ebb9675, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x515b9677, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db967c, SUC_lexicon_tags_12 },
    { 0x373b967d, SUC_lexicon_tags_1 },
    { 0xb9db967c, SUC_lexicon_tags_2 },
    { 0x37bb967c, SUC_lexicon_tags_47 },
    { 0x965b967c, SUC_lexicon_tags_17 },
    { 0x3edb9681, SUC_lexicon_tags_25 },
    { 0xe3db9680, SUC_lexicon_tags_11 },
    { 0x98db9683, SUC_lexicon_tags_14 },
    { 0xe37b9684, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33b9688, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97b968b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f9b968d, SUC_lexicon_tags_202 },
    { 0, NULL },
    { 0x50db968f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2dfb9691, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe71b9693, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5fb9695, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77b9698, SUC_lexicon_tags_33 },
    { 0x3cbb9699, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35b969e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd41b96a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea3b96a2, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xc71b96a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e9b96a6, SUC_lexicon_tags_45 },
    { 0x4edb96a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8db96ae, SUC_lexicon_tags_37 },
    { 0x2f1b96af, SUC_lexicon_tags_10 },
    { 0xae3b96b0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf2bb96b2, SUC_lexicon_tags_12 },
    { 0xf5db96b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2efb96b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29b96bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x305b96bd, SUC_lexicon_tags_2 },
    { 0xe15b96be, SUC_lexicon_tags_14 },
    { 0xe15b96be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3b96c6, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x1ddb96c8, SUC_lexicon_tags_36 },
    { 0x523b96c8, SUC_lexicon_tags_2 },
    { 0x521b96c8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x563b96cd, SUC_lexicon_tags_615 },
    { 0x4e1b96ce, SUC_lexicon_tags_19 },
    { 0xb3bb96cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7abb96d1, SUC_lexicon_tags_7 },
    { 0xdf1b96d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7b96d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3b96d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x18fb96dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdb96e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe5b96e6, SUC_lexicon_tags_47 },
    { 0x3a5b96e6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7db96ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde9b96ef, SUC_lexicon_tags_30 },
    { 0x7bb96ef, SUC_lexicon_tags_44 },
    { 0x329b96ef, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25db96f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adb96fc, SUC_lexicon_tags_43 },
    { 0x41db96fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc23b96ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3b9707, SUC_lexicon_tags_10 },
    { 0xa0bb9707, SUC_lexicon_tags_15 },
    { 0x53db9709, SUC_lexicon_tags_16 },
    { 0x551b970a, SUC_lexicon_tags_23 },
    { 0xcbfb970a, SUC_lexicon_tags_8 },
    { 0x339b970a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9b9710, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339b9714, SUC_lexicon_tags_25 },
    { 0x9dfb9715, SUC_lexicon_tags_9 },
    { 0x653b9715, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfb971b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31fb971d, SUC_lexicon_tags_21 },
    { 0xcc7b971d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423b9722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa27b9724, SUC_lexicon_tags_2 },
    { 0xf7fb9724, SUC_lexicon_tags_17 },
    { 0x9cfb9724, SUC_lexicon_tags_806 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5b972b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefb972f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143b9736, SUC_lexicon_tags_549 },
    { 0xcb7b9737, SUC_lexicon_tags_46 },
    { 0xb7bb9738, SUC_lexicon_tags_84 },
    { 0x45db9738, SUC_lexicon_tags_27 },
    { 0x30bb973a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xda3b973c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc59b973e, SUC_lexicon_tags_15 },
    { 0x2f3b973e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x391b9741, SUC_lexicon_tags_25 },
    { 0xab1b9742, SUC_lexicon_tags_68 },
    { 0x47db9742, SUC_lexicon_tags_2 },
    { 0x8afb9742, SUC_lexicon_tags_14 },
    { 0x1efb9745, SUC_lexicon_tags_37 },
    { 0x5bbb9745, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23b974a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21db974c, SUC_lexicon_tags_2 },
    { 0xe33b974d, SUC_lexicon_tags_37 },
    { 0x9e3b974e, SUC_lexicon_tags_23 },
    { 0x405b974f, SUC_lexicon_tags_17 },
    { 0x991b974f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307b9755, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fb975b, SUC_lexicon_tags_60 },
    { 0x58bb975c, SUC_lexicon_tags_15 },
    { 0xdffb975c, SUC_lexicon_tags_43 },
    { 0xff7b975e, SUC_lexicon_tags_0 },
    { 0x347b975e, SUC_lexicon_tags_12 },
    { 0x463b975f, SUC_lexicon_tags_37 },
    { 0xc97b9761, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8db9765, SUC_lexicon_tags_7 },
    { 0xf65b9766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc59b9768, SUC_lexicon_tags_7 },
    { 0x833b9769, SUC_lexicon_tags_49 },
    { 0x179b9769, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1b976d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7b9774, SUC_lexicon_tags_63 },
    { 0xcbb9775, SUC_lexicon_tags_8 },
    { 0x1dbb9775, SUC_lexicon_tags_29 },
    { 0x2bdb9777, SUC_lexicon_tags_65 },
    { 0x63db9774, SUC_lexicon_tags_35 },
    { 0xf07b9779, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc5db977b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbb9781, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263b9786, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x931b9788, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5fb978a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655b9797, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849b979b, SUC_lexicon_tags_25 },
    { 0x513b979c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3db979e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95b97a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1b97a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47b97ae, SUC_lexicon_tags_393 },
    { 0x419b97af, SUC_lexicon_tags_37 },
    { 0x3e7b97ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9b97b3, SUC_lexicon_tags_16 },
    { 0xd9bb97b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d9b97b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59b97b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1db97bb, SUC_lexicon_tags_10 },
    { 0x6a9b97bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3dfb97be, SUC_lexicon_tags_32 },
    { 0x9ffb97bf, SUC_lexicon_tags_22 },
    { 0xc07b97bf, SUC_lexicon_tags_33 },
    { 0x463b97c0, SUC_lexicon_tags_55 },
    { 0x735b97bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e9b97c4, SUC_lexicon_tags_9 },
    { 0x431b97c5, SUC_lexicon_tags_47 },
    { 0xc67b97c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3adb97c8, SUC_lexicon_tags_7 },
    { 0x1d7b97c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5b97cc, SUC_lexicon_tags_30 },
    { 0xcfdb97cd, SUC_lexicon_tags_7 },
    { 0x7b5b97ce, SUC_lexicon_tags_2 },
    { 0x3dbb97cf, SUC_lexicon_tags_26 },
    { 0xf2bb97cc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa6fb97d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x813b97d4, SUC_lexicon_tags_37 },
    { 0x215b97d5, SUC_lexicon_tags_45 },
    { 0xf9db97d6, SUC_lexicon_tags_8 },
    { 0xc41b97d7, SUC_lexicon_tags_2 },
    { 0x5d7b97d6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafb97df, SUC_lexicon_tags_67 },
    { 0x83b97df, SUC_lexicon_tags_45 },
    { 0x869b97e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed3b97e3, SUC_lexicon_tags_11 },
    { 0xd8db97e4, SUC_lexicon_tags_17 },
    { 0xbdb97e5, SUC_lexicon_tags_37 },
    { 0x513b97e5, SUC_lexicon_tags_45 },
    { 0x121b97e6, SUC_lexicon_tags_16 },
    { 0xf13b97e8, SUC_lexicon_tags_38 },
    { 0x6a7b97e8, SUC_lexicon_tags_28 },
    { 0x13fb97e9, SUC_lexicon_tags_6 },
    { 0x7db97e5, SUC_lexicon_tags_14 },
    { 0xcfb97ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7b97f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89bb97f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269b97f7, SUC_lexicon_tags_17 },
    { 0x42db97f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfb97fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd85b97fd, SUC_lexicon_tags_29 },
    { 0x8b7b97fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5cdb9800, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7a3b9802, SUC_lexicon_tags_12 },
    { 0x6fbb9803, SUC_lexicon_tags_17 },
    { 0x2e9b9802, SUC_lexicon_tags_11 },
    { 0xe69b9805, SUC_lexicon_tags_38 },
    { 0x393b9802, SUC_lexicon_tags_46 },
    { 0x6d9b9807, SUC_lexicon_tags_1 },
    { 0xefdb9808, SUC_lexicon_tags_15 },
    { 0xbefb9807, SUC_lexicon_tags_43 },
    { 0xeabb980a, SUC_lexicon_tags_25 },
    { 0xaf5b980b, SUC_lexicon_tags_14 },
    { 0xc15b980c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fb9813, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149b9817, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbb981b, SUC_lexicon_tags_65 },
    { 0xdb9b981c, SUC_lexicon_tags_12 },
    { 0xe11b981d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37b9820, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56fb9822, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bb9826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42db9828, SUC_lexicon_tags_17 },
    { 0x9dbb9829, SUC_lexicon_tags_25 },
    { 0x24fb9829, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xdf9b982c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x49b982f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21b9837, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdb983d, SUC_lexicon_tags_15 },
    { 0x959b983e, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfb9845, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afb984b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x99db984e, SUC_lexicon_tags_17 },
    { 0xb83b984e, SUC_lexicon_tags_9 },
    { 0x43bb9850, SUC_lexicon_tags_15 },
    { 0xebbb984f, SUC_lexicon_tags_1 },
    { 0xa59b9852, SUC_lexicon_tags_17 },
    { 0xaf9b9853, SUC_lexicon_tags_14 },
    { 0x1a3b9852, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481b9859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x277b985c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfb986c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7b9871, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a7b9873, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17b9877, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3b1b9879, SUC_lexicon_tags_1023 },
    { 0x8bb987a, SUC_lexicon_tags_25 },
    { 0xda9b987b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b987e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1b9886, SUC_lexicon_tags_17 },
    { 0xdcbb9887, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x407b9889, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb87b988b, SUC_lexicon_tags_17 },
    { 0x165b988c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39b988f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedb9894, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ffb9896, SUC_lexicon_tags_5 },
    { 0xfd3b9896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3b989a, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x80db989c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc93b989e, SUC_lexicon_tags_47 },
    { 0x371b989f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387b98a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5db98a7, SUC_lexicon_tags_0 },
    { 0xc89b98a7, SUC_lexicon_tags_7 },
    { 0xb9fb98a8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fb98b2, SUC_lexicon_tags_39 },
    { 0x36fb98b3, SUC_lexicon_tags_22 },
    { 0x537b98b4, SUC_lexicon_tags_37 },
    { 0xe67b98b5, SUC_lexicon_tags_31 },
    { 0x9bfb98b6, SUC_lexicon_tags_37 },
    { 0x4edb98b3, SUC_lexicon_tags_32 },
    { 0xc1db98b8, SUC_lexicon_tags_37 },
    { 0xd33b98b8, SUC_lexicon_tags_10 },
    { 0x39bb98ba, SUC_lexicon_tags_49 },
    { 0x21bb98ba, SUC_lexicon_tags_25 },
    { 0x5b3b98bc, SUC_lexicon_tags_15 },
    { 0xbd5b98bd, SUC_lexicon_tags_25 },
    { 0xcdbb98bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x897b98c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7db98c2, SUC_lexicon_tags_23 },
    { 0x6efb98c3, SUC_lexicon_tags_66 },
    { 0x517b98c3, SUC_lexicon_tags_6 },
    { 0x35b98c4, SUC_lexicon_tags_15 },
    { 0x599b98c3, SUC_lexicon_tags_17 },
    { 0x589b98c5, SUC_lexicon_tags_17 },
    { 0x97db98c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc5db98ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x303b98cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87b98d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x981b98d4, SUC_lexicon_tags_42 },
    { 0xbd1b98d4, SUC_lexicon_tags_37 },
    { 0xa85b98d6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa15b98d8, SUC_lexicon_tags_43 },
    { 0x3f1b98d9, SUC_lexicon_tags_3 },
    { 0x607b98da, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa5bb98dc, SUC_lexicon_tags_6 },
    { 0xeb5b98dd, SUC_lexicon_tags_19 },
    { 0xcddb98de, SUC_lexicon_tags_37 },
    { 0xe45b98de, SUC_lexicon_tags_12 },
    { 0x7c1b98e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdb98e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fb98e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafb98ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5db98f1, SUC_lexicon_tags_33 },
    { 0x15fb98f2, SUC_lexicon_tags_29 },
    { 0x9b3b98f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x939b98f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457b98fa, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x593b98fc, SUC_lexicon_tags_15 },
    { 0xaf5b98fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x285b9900, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675b9905, SUC_lexicon_tags_11 },
    { 0x5b3b9905, SUC_lexicon_tags_8 },
    { 0xdb7b9905, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc93b9909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87bb990b, SUC_lexicon_tags_24 },
    { 0x69db990c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5b9910, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909b9916, SUC_lexicon_tags_206 },
    { 0xfb5b9916, SUC_lexicon_tags_136 },
    { 0x99db9918, SUC_lexicon_tags_11 },
    { 0xd65b9917, SUC_lexicon_tags_6 },
    { 0xfabb9919, SUC_lexicon_tags_87 },
    { 0x9adb991b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425b9921, SUC_lexicon_tags_49 },
    { 0x1cbb9922, SUC_lexicon_tags_12 },
    { 0x95fb9923, SUC_lexicon_tags_2 },
    { 0xf45b9924, SUC_lexicon_tags_25 },
    { 0x579b9924, SUC_lexicon_tags_26 },
    { 0xdc7b9926, SUC_lexicon_tags_8 },
    { 0x32bb9927, SUC_lexicon_tags_56 },
    { 0x7cbb9928, SUC_lexicon_tags_12 },
    { 0xbb9929, SUC_lexicon_tags_1 },
    { 0x7a7b9923, SUC_lexicon_tags_22 },
    { 0xcffb9929, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ffb992d, SUC_lexicon_tags_38 },
    { 0x8d9b992d, SUC_lexicon_tags_44 },
    { 0xf7b992f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xccbb9931, SUC_lexicon_tags_17 },
    { 0x577b9932, SUC_lexicon_tags_10 },
    { 0xb45b9933, SUC_lexicon_tags_6 },
    { 0x677b9934, SUC_lexicon_tags_19 },
    { 0xd97b9933, SUC_lexicon_tags_25 },
    { 0xa15b9931, SUC_lexicon_tags_47 },
    { 0xe5db9937, SUC_lexicon_tags_49 },
    { 0x71bb9935, SUC_lexicon_tags_37 },
    { 0x39fb9939, SUC_lexicon_tags_47 },
    { 0xdedb993a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddb9943, SUC_lexicon_tags_15 },
    { 0xd1db9944, SUC_lexicon_tags_25 },
    { 0x94db9945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb71b9947, SUC_lexicon_tags_1 },
    { 0x203b9948, SUC_lexicon_tags_37 },
    { 0xf01b9948, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19b994f, SUC_lexicon_tags_45 },
    { 0xecfb994f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b7b9952, SUC_lexicon_tags_37 },
    { 0x4cdb9953, SUC_lexicon_tags_4 },
    { 0xc01b9954, SUC_lexicon_tags_15 },
    { 0x69bb9954, SUC_lexicon_tags_87 },
    { 0x437b9956, SUC_lexicon_tags_10 },
    { 0x96db9952, SUC_lexicon_tags_8 },
    { 0xda3b9958, SUC_lexicon_tags_15 },
    { 0xee5b9959, SUC_lexicon_tags_12 },
    { 0x29b9959, SUC_lexicon_tags_5 },
    { 0x157b995b, SUC_lexicon_tags_17 },
    { 0x955b9953, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43b9960, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bb9963, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bb9967, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5b996d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31db9971, SUC_lexicon_tags_66 },
    { 0x7dbb9971, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabb9976, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1b997c, SUC_lexicon_tags_17 },
    { 0xf07b997c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fb9980, SUC_lexicon_tags_37 },
    { 0xa3b9981, SUC_lexicon_tags_21 },
    { 0x885b9982, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffb9987, SUC_lexicon_tags_19 },
    { 0x3e3b9987, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109b9990, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303b9995, SUC_lexicon_tags_7 },
    { 0xc9db9996, SUC_lexicon_tags_1 },
    { 0x40db9997, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59b999d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949b99a5, SUC_lexicon_tags_92 },
    { 0x727b99a6, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3b99ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd61b99ae, SUC_lexicon_tags_8 },
    { 0xb41b99ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfb99b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9b99b6, SUC_lexicon_tags_12 },
    { 0xffdb99b7, SUC_lexicon_tags_33 },
    { 0x989b99b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3b99bc, SUC_lexicon_tags_64 },
    { 0x589b99bd, SUC_lexicon_tags_6 },
    { 0x659b99bd, SUC_lexicon_tags_49 },
    { 0xac7b99bc, SUC_lexicon_tags_1 },
    { 0xa1db99c0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863b99c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x305b99cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xecfb99cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb3db99cf, SUC_lexicon_tags_37 },
    { 0x35db99cf, SUC_lexicon_tags_47 },
    { 0x2f3b99d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5b99d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7fdb99d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bb99da, SUC_lexicon_tags_38 },
    { 0x8db99da, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x735b99dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef3b99df, SUC_lexicon_tags_22 },
    { 0xc17b99e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fb99e6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x78bb99e8, SUC_lexicon_tags_43 },
    { 0xd9bb99e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39bb99eb, SUC_lexicon_tags_69 },
    { 0xbddb99ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbb99f4, SUC_lexicon_tags_9 },
    { 0x23db99f5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x98bb99f7, SUC_lexicon_tags_29 },
    { 0x7cbb99f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fb99fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x719b99ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x895b9a02, SUC_lexicon_tags_10 },
    { 0xf05b9a03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98bb9a05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1b9a09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1b9a10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdb9a15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x511b9a17, SUC_lexicon_tags_23 },
    { 0x52fb9a17, SUC_lexicon_tags_7 },
    { 0xc3b9a18, SUC_lexicon_tags_29 },
    { 0xdedb9a18, SUC_lexicon_tags_7 },
    { 0x77fb9a1b, SUC_lexicon_tags_188 },
    { 0x3c3b9a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fb9a1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5b9a23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9a26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775b9a2e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x73bb9a31, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x359b9a33, SUC_lexicon_tags_17 },
    { 0xdc3b9a34, SUC_lexicon_tags_55 },
    { 0x4d7b9a35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1b9a39, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44db9a40, SUC_lexicon_tags_37 },
    { 0xae1b9a40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x30db9a44, SUC_lexicon_tags_25 },
    { 0xb41b9a45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x769b9a48, SUC_lexicon_tags_28 },
    { 0xda5b9a49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18fb9a4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbb9a55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fb9a59, SUC_lexicon_tags_7 },
    { 0x925b9a5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41fb9a5c, SUC_lexicon_tags_27 },
    { 0x40fb9a5c, SUC_lexicon_tags_19 },
    { 0xc4fb9a5e, SUC_lexicon_tags_25 },
    { 0xd3b9a5f, SUC_lexicon_tags_17 },
    { 0xfc7b9a5d, SUC_lexicon_tags_13 },
    { 0x71b9a5e, SUC_lexicon_tags_11 },
    { 0xf27b9a62, SUC_lexicon_tags_47 },
    { 0xe0bb9a61, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1b9a67, SUC_lexicon_tags_1 },
    { 0xadfb9a68, SUC_lexicon_tags_37 },
    { 0xb69b9a68, SUC_lexicon_tags_48 },
    { 0xe29b9a6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdb9a6e, SUC_lexicon_tags_13 },
    { 0x5c7b9a6f, SUC_lexicon_tags_53 },
    { 0xfe3b9a6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26bb9a73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x557b9a76, SUC_lexicon_tags_45 },
    { 0x107b9a77, SUC_lexicon_tags_15 },
    { 0xfb3b9a77, SUC_lexicon_tags_11 },
    { 0xc2bb9a78, SUC_lexicon_tags_1 },
    { 0x819b9a7a, SUC_lexicon_tags_19 },
    { 0x5b9a79, SUC_lexicon_tags_37 },
    { 0x22db9a7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x505b9a7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439b9a83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527b9a87, SUC_lexicon_tags_19 },
    { 0xf2fb9a88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9b9a8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bb9a8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09b9a95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x97bb9a97, SUC_lexicon_tags_17 },
    { 0x901b9a98, SUC_lexicon_tags_25 },
    { 0x79fb9a98, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afb9a9e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f9b9aa0, SUC_lexicon_tags_2 },
    { 0xb31b9aa0, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3b9aa5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbb3b9aa7, SUC_lexicon_tags_59 },
    { 0x9abb9aa8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x883b9aab, SUC_lexicon_tags_31 },
    { 0x103b9aab, SUC_lexicon_tags_29 },
    { 0xd7fb9aad, SUC_lexicon_tags_14 },
    { 0xd81b9aab, SUC_lexicon_tags_27 },
    { 0xd21b9aaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x98bb9ab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65b9ab5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bb9aba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x81b9abc, SUC_lexicon_tags_15 },
    { 0x193b9abd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x691b9ac0, SUC_lexicon_tags_39 },
    { 0xe61b9ac1, SUC_lexicon_tags_0 },
    { 0x515b9ac2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x167b9ac5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cbb9ac7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x383b9ac9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9edb9acb, SUC_lexicon_tags_25 },
    { 0x71b9acc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327b9ada, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb29b9adc, SUC_lexicon_tags_43 },
    { 0x47bb9add, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfadb9adf, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103b9ae3, SUC_lexicon_tags_178 },
    { 0x231b9ae4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72fb9ae7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fb9aeb, SUC_lexicon_tags_38 },
    { 0xcc3b9aec, SUC_lexicon_tags_14 },
    { 0xaf1b9aed, SUC_lexicon_tags_16 },
    { 0x319b9aee, SUC_lexicon_tags_1 },
    { 0xa8db9aef, SUC_lexicon_tags_17 },
    { 0x7fbb9aef, SUC_lexicon_tags_23 },
    { 0x2ddb9aed, SUC_lexicon_tags_15 },
    { 0x2adb9af0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed3b9af4, SUC_lexicon_tags_8 },
    { 0x481b9af5, SUC_lexicon_tags_10 },
    { 0xcd3b9af6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcabb9af8, SUC_lexicon_tags_15 },
    { 0x17b9af8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6edb9afb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409b9b01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13b9b04, SUC_lexicon_tags_3 },
    { 0x14db9b05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17b9b09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69b9b0e, SUC_lexicon_tags_19 },
    { 0xfb9b0e, SUC_lexicon_tags_9 },
    { 0xa2db9b0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adb9b15, SUC_lexicon_tags_19 },
    { 0x9fbb9b16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b5b9b18, SUC_lexicon_tags_11 },
    { 0x5efb9b18, SUC_lexicon_tags_33 },
    { 0x531b9b1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x431b9b1c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x613b9b1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349b9b22, SUC_lexicon_tags_9 },
    { 0xc8fb9b23, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9adb9b25, SUC_lexicon_tags_37 },
    { 0xb7bb9b25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x44db9b29, SUC_lexicon_tags_17 },
    { 0x4bfb9b2a, SUC_lexicon_tags_49 },
    { 0x461b9b2b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6db9b2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3b9b31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x685b9b34, SUC_lexicon_tags_37 },
    { 0xb1db9b35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45bb9b38, SUC_lexicon_tags_57 },
    { 0x82fb9b38, SUC_lexicon_tags_32 },
    { 0x437b9b38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbebb9b3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1b9b40, SUC_lexicon_tags_30 },
    { 0x321b9b41, SUC_lexicon_tags_11 },
    { 0x893b9b40, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xae3b9b44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf55b9b46, SUC_lexicon_tags_23 },
    { 0xd07b9b46, SUC_lexicon_tags_29 },
    { 0x195b9b48, SUC_lexicon_tags_37 },
    { 0x453b9b46, SUC_lexicon_tags_55 },
    { 0x797b9b4a, SUC_lexicon_tags_29 },
    { 0xca3b9b4a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x383b9b4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf5db9b4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x669b9b51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fb9b54, SUC_lexicon_tags_13 },
    { 0xa69b9b55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x835b9b57, SUC_lexicon_tags_11 },
    { 0x31b9b57, SUC_lexicon_tags_11 },
    { 0x635b9b58, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x575b9b5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3b9b63, SUC_lexicon_tags_25 },
    { 0xe79b9b63, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b9b69, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x487b9b6b, SUC_lexicon_tags_35 },
    { 0x203b9b6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7b9b71, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x337b9b73, SUC_lexicon_tags_27 },
    { 0xbcfb9b73, SUC_lexicon_tags_2 },
    { 0x4d1b9b75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x82bb9b78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x32bb9b7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9b9b81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5b9b84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x537b9b86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975b9b90, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72db9b96, SUC_lexicon_tags_7 },
    { 0xb13b9b97, SUC_lexicon_tags_0 },
    { 0x59db9b98, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4efb9b9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21b9b9d, SUC_lexicon_tags_0 },
    { 0x7e7b9b9e, SUC_lexicon_tags_1 },
    { 0xb53b9b9f, SUC_lexicon_tags_23 },
    { 0xb53b9b9f, SUC_lexicon_tags_35 },
    { 0x7d7b9ba1, SUC_lexicon_tags_65 },
    { 0xdebb9b9f, SUC_lexicon_tags_26 },
    { 0x11bb9ba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99b9ba6, SUC_lexicon_tags_8 },
    { 0x455b9ba7, SUC_lexicon_tags_37 },
    { 0xa71b9ba8, SUC_lexicon_tags_4 },
    { 0xcffb9ba8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1b9bae, SUC_lexicon_tags_0 },
    { 0xe83b9baf, SUC_lexicon_tags_15 },
    { 0xd11b9bb0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81db9bb4, SUC_lexicon_tags_2 },
    { 0x3f1b9bb5, SUC_lexicon_tags_37 },
    { 0x401b9bb5, SUC_lexicon_tags_38 },
    { 0xeb5b9bb7, SUC_lexicon_tags_34 },
    { 0xe91b9bb7, SUC_lexicon_tags_82 },
    { 0x787b9bb9, SUC_lexicon_tags_21 },
    { 0x5f7b9bb7, SUC_lexicon_tags_25 },
    { 0x69fb9bb5, SUC_lexicon_tags_11 },
    { 0xa41b9bb9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5b9bc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3b9bc8, SUC_lexicon_tags_60 },
    { 0x35fb9bc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeebb9bcb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x389b9bce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0db9bd1, SUC_lexicon_tags_30 },
    { 0x4f9b9bd2, SUC_lexicon_tags_8 },
    { 0xe3fb9bd3, SUC_lexicon_tags_11 },
    { 0x875b9bd4, SUC_lexicon_tags_12 },
    { 0x705b9bd5, SUC_lexicon_tags_2 },
    { 0xaefb9bd6, SUC_lexicon_tags_26 },
    { 0xee3b9bd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca5b9bd9, SUC_lexicon_tags_104 },
    { 0x6abb9bd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bb9bdd, SUC_lexicon_tags_45 },
    { 0x117b9bdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fb9be2, SUC_lexicon_tags_8 },
    { 0x763b9be3, SUC_lexicon_tags_17 },
    { 0xedbb9be2, SUC_lexicon_tags_39 },
    { 0x635b9be4, SUC_lexicon_tags_9 },
    { 0xe3b9be6, SUC_lexicon_tags_19 },
    { 0x6b3b9be6, SUC_lexicon_tags_151 },
    { 0x897b9be7, SUC_lexicon_tags_1 },
    { 0xbdb9be7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fb9bed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7fbb9bef, SUC_lexicon_tags_4 },
    { 0x5f9b9bef, SUC_lexicon_tags_0 },
    { 0x951b9bf1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x223b9bf3, SUC_lexicon_tags_26 },
    { 0x9adb9bf4, SUC_lexicon_tags_46 },
    { 0x749b9bf5, SUC_lexicon_tags_17 },
    { 0x40bb9bf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13b9bf8, SUC_lexicon_tags_29 },
    { 0xd03b9bf9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fb9bfc, SUC_lexicon_tags_38 },
    { 0x943b9bfd, SUC_lexicon_tags_6 },
    { 0x455b9bfd, SUC_lexicon_tags_12 },
    { 0x223b9bff, SUC_lexicon_tags_30 },
    { 0xe45b9c00, SUC_lexicon_tags_12 },
    { 0xe57b9c01, SUC_lexicon_tags_37 },
    { 0xed7b9bfc, SUC_lexicon_tags_9 },
    { 0x8a1b9c03, SUC_lexicon_tags_27 },
    { 0xfcdb9bfc, SUC_lexicon_tags_14 },
    { 0xca3b9c05, SUC_lexicon_tags_19 },
    { 0x429b9bfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbb9c0c, SUC_lexicon_tags_87 },
    { 0x4ffb9c0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101b9c11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c9b9c13, SUC_lexicon_tags_23 },
    { 0xbb3b9c14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5b9c1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfb9c20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa6db9c22, SUC_lexicon_tags_16 },
    { 0xf99b9c22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3db9c28, SUC_lexicon_tags_17 },
    { 0xc71b9c29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b3b9c2b, SUC_lexicon_tags_33 },
    { 0x6adb9c2c, SUC_lexicon_tags_23 },
    { 0x4c7b9c2d, SUC_lexicon_tags_1 },
    { 0xf7b9c2d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adb9c33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f9b9c35, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb7b9c37, SUC_lexicon_tags_165 },
    { 0x9c3b9c38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5b9c3d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf5fb9c3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295b9c44, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7b9c4b, SUC_lexicon_tags_45 },
    { 0x121b9c4c, SUC_lexicon_tags_7 },
    { 0x65fb9c4c, SUC_lexicon_tags_27 },
    { 0xd0bb9c4e, SUC_lexicon_tags_15 },
    { 0x18bb9c4c, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0xb89b9c51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71b9c56, SUC_lexicon_tags_25 },
    { 0xbb9b9c57, SUC_lexicon_tags_15 },
    { 0xfc5b9c58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdb9c5e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553b9c66, SUC_lexicon_tags_37 },
    { 0x11bb9c67, SUC_lexicon_tags_25 },
    { 0xb01b9c68, SUC_lexicon_tags_6 },
    { 0x9abb9c68, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7b9c6c, SUC_lexicon_tags_37 },
    { 0x511b9c6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1db9c6f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3b9c74, SUC_lexicon_tags_28 },
    { 0x639b9c74, SUC_lexicon_tags_37 },
    { 0xa0db9c76, SUC_lexicon_tags_28 },
    { 0xefdb9c76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35b9c7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x981b9c7e, SUC_lexicon_tags_11 },
    { 0xac9b9c7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71b9c89, SUC_lexicon_tags_17 },
    { 0xad3b9c8a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd3fb9c8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd09b9c8e, SUC_lexicon_tags_8 },
    { 0xf91b9c8e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9b9c95, SUC_lexicon_tags_0 },
    { 0x639b9c95, SUC_lexicon_tags_65 },
    { 0xacdb9c97, SUC_lexicon_tags_25 },
    { 0x645b9c97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec7b9c9a, SUC_lexicon_tags_5 },
    { 0x40bb9c9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a9b9c9d, SUC_lexicon_tags_1 },
    { 0xc6fb9c9e, SUC_lexicon_tags_17 },
    { 0x913b9c9f, SUC_lexicon_tags_28 },
    { 0x3cfb9ca0, SUC_lexicon_tags_0 },
    { 0xb17b9ca1, SUC_lexicon_tags_68 },
    { 0x291b9ca2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935b9ca8, SUC_lexicon_tags_9 },
    { 0xbf7b9ca8, SUC_lexicon_tags_2 },
    { 0xf69b9ca8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6cfb9cac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1b9caf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf3b9cb1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9fb9cb3, SUC_lexicon_tags_26 },
    { 0x357b9cb3, SUC_lexicon_tags_13 },
    { 0x423b9cb5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5b9cbb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27db9cbf, SUC_lexicon_tags_25 },
    { 0x869b9cc0, SUC_lexicon_tags_14 },
    { 0x2f7b9cc1, SUC_lexicon_tags_5 },
    { 0x313b9cc0, SUC_lexicon_tags_37 },
    { 0x19bb9cc3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01b9cc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0fb9ccb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0db9ccd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9b9cd3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1b9cd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5b9cdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3bb9cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x855b9ce1, SUC_lexicon_tags_38 },
    { 0x2ffb9ce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60db9ceb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5b9cf0, SUC_lexicon_tags_37 },
    { 0x427b9cf1, SUC_lexicon_tags_1086 },
    { 0x3d3b9cf2, SUC_lexicon_tags_15 },
    { 0xe31b9cf2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9c9b9cf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9b9cfa, SUC_lexicon_tags_47 },
    { 0x7ddb9cfa, SUC_lexicon_tags_87 },
    { 0xc81b9cfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453b9d01, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x25b9d03, SUC_lexicon_tags_11 },
    { 0xd7db9d03, SUC_lexicon_tags_27 },
    { 0x149b9d05, SUC_lexicon_tags_11 },
    { 0x483b9d06, SUC_lexicon_tags_37 },
    { 0x149b9d06, SUC_lexicon_tags_7 },
    { 0xd05b9d04, SUC_lexicon_tags_14 },
    { 0x1b3b9d04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae9b9d0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x865b9d0e, SUC_lexicon_tags_65 },
    { 0xb3db9d0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afb9d13, SUC_lexicon_tags_26 },
    { 0xb95b9d14, SUC_lexicon_tags_7 },
    { 0x251b9d15, SUC_lexicon_tags_12 },
    { 0x799b9d16, SUC_lexicon_tags_37 },
    { 0x4f7b9d15, SUC_lexicon_tags_37 },
    { 0x4d5b9d15, SUC_lexicon_tags_77 },
    { 0x179b9d16, SUC_lexicon_tags_11 },
    { 0x229b9d17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb9b9d1c, SUC_lexicon_tags_1 },
    { 0x161b9d1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb9b9d1f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6c7b9d21, SUC_lexicon_tags_1 },
    { 0xd95b9d22, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x369b9d25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9b9d2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fb9d34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x927b9d36, SUC_lexicon_tags_27 },
    { 0x971b9d37, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3b9d3a, SUC_lexicon_tags_12 },
    { 0xc33b9d3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x547b9d3d, SUC_lexicon_tags_26 },
    { 0xbbfb9d3d, SUC_lexicon_tags_43 },
    { 0x545b9d3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbb9d43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1db9d46, SUC_lexicon_tags_20 },
    { 0xedb9d47, SUC_lexicon_tags_23 },
    { 0x639b9d48, SUC_lexicon_tags_37 },
    { 0xedb9d46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83db9d4d, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fb9d51, SUC_lexicon_tags_7 },
    { 0x6a3b9d52, SUC_lexicon_tags_17 },
    { 0x53db9d53, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283b9d5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xea9b9d60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715b9d65, SUC_lexicon_tags_43 },
    { 0xf83b9d66, SUC_lexicon_tags_12 },
    { 0xe2fb9d66, SUC_lexicon_tags_25 },
    { 0x119b9d68, SUC_lexicon_tags_1 },
    { 0x15bb9d66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd35b9d6b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abb9d6f, SUC_lexicon_tags_17 },
    { 0xf69b9d70, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1afb9d72, SUC_lexicon_tags_7 },
    { 0x485b9d73, SUC_lexicon_tags_39 },
    { 0x86db9d74, SUC_lexicon_tags_8 },
    { 0x5a9b9d74, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209b9d79, SUC_lexicon_tags_27 },
    { 0xed3b9d7a, SUC_lexicon_tags_2 },
    { 0x791b9d7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfadb9d7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6bb9d7f, SUC_lexicon_tags_63 },
    { 0x57db9d80, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fb9d84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547b9d89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7b9d90, SUC_lexicon_tags_11 },
    { 0xefdb9d90, SUC_lexicon_tags_18 },
    { 0x3e1b9d91, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa3bb9d94, SUC_lexicon_tags_17 },
    { 0x875b9d95, SUC_lexicon_tags_33 },
    { 0xf1fb9d96, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x18bb9d98, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x56bb9d9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bb9d9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9b9daa, SUC_lexicon_tags_47 },
    { 0xa51b9daa, SUC_lexicon_tags_8 },
    { 0xe5db9dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373b9db1, SUC_lexicon_tags_6 },
    { 0x441b9db1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49b9db6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc9b9db8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4ebb9dba, SUC_lexicon_tags_14 },
    { 0x553b9dbb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94db9dbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71b9dc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xafb9dc5, SUC_lexicon_tags_27 },
    { 0x1f1b9dc6, SUC_lexicon_tags_25 },
    { 0xcf5b9dc5, SUC_lexicon_tags_33 },
    { 0x5b5b9dc8, SUC_lexicon_tags_81 },
    { 0x949b9dc5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3d1b9dcb, SUC_lexicon_tags_12 },
    { 0x777b9dcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29db9dd1, SUC_lexicon_tags_25 },
    { 0x9b5b9dd1, SUC_lexicon_tags_25 },
    { 0x8b1b9dd2, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117b9dd8, SUC_lexicon_tags_48 },
    { 0x391b9dd9, SUC_lexicon_tags_39 },
    { 0x87bb9dda, SUC_lexicon_tags_3 },
    { 0xab7b9dd9, SUC_lexicon_tags_48 },
    { 0xe0fb9ddc, SUC_lexicon_tags_7 },
    { 0x713b9ddc, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x921b9ddf, SUC_lexicon_tags_23 },
    { 0x6a3b9de0, SUC_lexicon_tags_47 },
    { 0x4f9b9de1, SUC_lexicon_tags_17 },
    { 0x8c3b9ddf, SUC_lexicon_tags_11 },
    { 0x2cdb9ddf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x881b9de5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82db9de9, SUC_lexicon_tags_17 },
    { 0xa53b9dea, SUC_lexicon_tags_37 },
    { 0xfb9dea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987b9df2, SUC_lexicon_tags_92 },
    { 0x357b9df2, SUC_lexicon_tags_37 },
    { 0xb2fb9df4, SUC_lexicon_tags_60 },
    { 0xf91b9df2, SUC_lexicon_tags_2 },
    { 0x605b9df6, SUC_lexicon_tags_27 },
    { 0x9c9b9df7, SUC_lexicon_tags_21 },
    { 0xcd9b9df8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3db9dfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbabb9dff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbb9e02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3db9e04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa01b9e06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fb9e08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933b9e0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x39b9e0e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5b9e12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53b9e16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9b9e26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd29b9e28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b7b9e2a, SUC_lexicon_tags_49 },
    { 0x947b9e2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9bbb9e2d, SUC_lexicon_tags_80 },
    { 0x85fb9e2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fb9e32, SUC_lexicon_tags_81 },
    { 0xb2bb9e33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5db9e35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdb9e38, SUC_lexicon_tags_37 },
    { 0xccbb9e38, SUC_lexicon_tags_25 },
    { 0x375b9e3a, SUC_lexicon_tags_7 },
    { 0x603b9e3b, SUC_lexicon_tags_24 },
    { 0x349b9e3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201b9e43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7b9e4c, SUC_lexicon_tags_17 },
    { 0x5abb9e4d, SUC_lexicon_tags_12 },
    { 0x37b9e4e, SUC_lexicon_tags_7 },
    { 0x6ffb9e4f, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0xf47b9e51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x373b9e54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8db9e57, SUC_lexicon_tags_35 },
    { 0x307b9e58, SUC_lexicon_tags_2 },
    { 0x1e1b9e58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc9b9e5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe91b9e5d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37b9e64, SUC_lexicon_tags_47 },
    { 0x8dfb9e65, SUC_lexicon_tags_37 },
    { 0xcebb9e64, SUC_lexicon_tags_11 },
    { 0x6afb9e66, SUC_lexicon_tags_25 },
    { 0x80bb9e68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37b9e6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01b9e6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbb9e75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59b9e7e, SUC_lexicon_tags_17 },
    { 0xd95b9e7f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697b9e84, SUC_lexicon_tags_63 },
    { 0xc65b9e84, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xb3db9e87, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfbfb9e89, SUC_lexicon_tags_17 },
    { 0xa51b9e8a, SUC_lexicon_tags_11 },
    { 0x9efb9e8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x501b9e8e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdd3b9e90, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3b9e93, SUC_lexicon_tags_45 },
    { 0x37bb9e94, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fb9e9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10db9ea2, SUC_lexicon_tags_16 },
    { 0xa05b9ea2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x309b9ea5, SUC_lexicon_tags_25 },
    { 0xe0db9ea6, SUC_lexicon_tags_9 },
    { 0xfebb9ea7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedb9eaa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48db9eae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5b9eb7, SUC_lexicon_tags_1 },
    { 0x7d3b9eb8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9b9ebb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53b9ec1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadb9ec5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x819b9ec8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbb9ece, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fb9ed4, SUC_lexicon_tags_17 },
    { 0xf1db9ed5, SUC_lexicon_tags_11 },
    { 0xf99b9ed4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x24bb9ed8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89b9ee0, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x76db9ee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101b9ee6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbb9ef0, SUC_lexicon_tags_37 },
    { 0x24fb9ef0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7b9ef4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3b9efb, SUC_lexicon_tags_56 },
    { 0x757b9efc, SUC_lexicon_tags_60 },
    { 0x279b9efd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9b9f00, SUC_lexicon_tags_66 },
    { 0x3b1b9f00, SUC_lexicon_tags_43 },
    { 0x7bfb9f02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211b9f06, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x4db9f08, SUC_lexicon_tags_9 },
    { 0x23bb9f08, SUC_lexicon_tags_11 },
    { 0x28db9f0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2db9f10, SUC_lexicon_tags_30 },
    { 0x9e5b9f11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11b9f16, SUC_lexicon_tags_38 },
    { 0x8fb9f17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe3b9f19, SUC_lexicon_tags_2 },
    { 0x887b9f19, SUC_lexicon_tags_17 },
    { 0x38db9f1b, SUC_lexicon_tags_17 },
    { 0x335b9f1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13b9f20, SUC_lexicon_tags_42 },
    { 0x66db9f21, SUC_lexicon_tags_69 },
    { 0x2dbb9f20, SUC_lexicon_tags_30 },
    { 0x541b9f23, SUC_lexicon_tags_34 },
    { 0xb51b9f24, SUC_lexicon_tags_15 },
    { 0x281b9f25, SUC_lexicon_tags_45 },
    { 0x46fb9f25, SUC_lexicon_tags_4 },
    { 0x201b9f27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x443b9f2a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc39b9f2c, SUC_lexicon_tags_12 },
    { 0xa63b9f2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bb9f31, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaedb9f33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfb9f36, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405b9f3b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb2fb9f3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bb9f41, SUC_lexicon_tags_10 },
    { 0x597b9f42, SUC_lexicon_tags_17 },
    { 0x169b9f42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb87b9f45, SUC_lexicon_tags_63 },
    { 0xd75b9f45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bb9f49, SUC_lexicon_tags_17 },
    { 0x9afb9f4a, SUC_lexicon_tags_10 },
    { 0xbd9b9f4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae1b9f4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x39db9f4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5b9f53, SUC_lexicon_tags_39 },
    { 0x6bdb9f54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb87b9f56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7abb9f58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49b9f5d, SUC_lexicon_tags_37 },
    { 0xd07b9f5e, SUC_lexicon_tags_48 },
    { 0xf9fb9f5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x253b9f61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bb9f65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91b9f69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc0bb9f6b, SUC_lexicon_tags_7 },
    { 0xfefb9f6b, SUC_lexicon_tags_2 },
    { 0x81b9f6d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xa57b9f6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5b9f73, SUC_lexicon_tags_38 },
    { 0x53b9f73, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x21db9f77, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdb9f7a, SUC_lexicon_tags_17 },
    { 0x38bb9f7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x513b9f7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x205b9f81, SUC_lexicon_tags_4 },
    { 0xa45b9f82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafb9f87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9b9f8a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1b9f8d, SUC_lexicon_tags_0 },
    { 0x697b9f8d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xab9b9f90, SUC_lexicon_tags_25 },
    { 0xcfb9f90, SUC_lexicon_tags_29 },
    { 0x847b9f90, SUC_lexicon_tags_65 },
    { 0xdd7b9f93, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3ebb9f95, SUC_lexicon_tags_14 },
    { 0x237b9f95, SUC_lexicon_tags_9 },
    { 0xe3bb9f97, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad5b9f99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x427b9f9c, SUC_lexicon_tags_0 },
    { 0xec1b9f9d, SUC_lexicon_tags_11 },
    { 0x25fb9f9e, SUC_lexicon_tags_27 },
    { 0xd05b9f9c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1b9fa6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bb9faa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbb9faf, SUC_lexicon_tags_25 },
    { 0x65db9fb0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x171b9fb2, SUC_lexicon_tags_6 },
    { 0x531b9fb3, SUC_lexicon_tags_14 },
    { 0x607b9fb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413b9fb9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25b9fbc, SUC_lexicon_tags_17 },
    { 0x9f9b9fbd, SUC_lexicon_tags_152 },
    { 0x803b9fbe, SUC_lexicon_tags_6 },
    { 0x619b9fbc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5e7b9fc1, SUC_lexicon_tags_25 },
    { 0x80fb9fc2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x93fb9fc4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x527b9fc6, SUC_lexicon_tags_805 },
    { 0x9a5b9fc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9fca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17b9fcd, SUC_lexicon_tags_15 },
    { 0x707b9fce, SUC_lexicon_tags_2 },
    { 0x95fb9fcf, SUC_lexicon_tags_82 },
    { 0xc25b9fcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab1b9fd2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56db9fd6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bb9fda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5bb9fdc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9b9fe1, SUC_lexicon_tags_11 },
    { 0xf13b9fe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa63b9fe4, SUC_lexicon_tags_46 },
    { 0x1d1b9fe5, SUC_lexicon_tags_9 },
    { 0xca9b9fe6, SUC_lexicon_tags_7 },
    { 0x1f9b9fe4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7db9ff0, SUC_lexicon_tags_17 },
    { 0x47b9ff1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x201b9ff4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981b9ffc, SUC_lexicon_tags_23 },
    { 0x613b9ffd, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3ba000, SUC_lexicon_tags_2 },
    { 0x939ba001, SUC_lexicon_tags_9 },
    { 0xe8bba002, SUC_lexicon_tags_23 },
    { 0x9d3ba003, SUC_lexicon_tags_60 },
    { 0xc69ba003, SUC_lexicon_tags_15 },
    { 0xd09ba005, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x629ba007, SUC_lexicon_tags_11 },
    { 0x869ba008, SUC_lexicon_tags_17 },
    { 0xb45ba009, SUC_lexicon_tags_47 },
    { 0xd9ba00a, SUC_lexicon_tags_6 },
    { 0x7a3ba00b, SUC_lexicon_tags_12 },
    { 0xedba00b, SUC_lexicon_tags_1 },
    { 0x2dfba00d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x78fba00f, SUC_lexicon_tags_46 },
    { 0x749ba010, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dba013, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fba016, SUC_lexicon_tags_1 },
    { 0xc4fba017, SUC_lexicon_tags_1 },
    { 0xb2dba016, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ba01c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbba021, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11ba028, SUC_lexicon_tags_46 },
    { 0x68dba029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x727ba02b, SUC_lexicon_tags_2 },
    { 0x175ba02c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfba038, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bba041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ba046, SUC_lexicon_tags_27 },
    { 0x3dbba047, SUC_lexicon_tags_33 },
    { 0x6d9ba047, SUC_lexicon_tags_80 },
    { 0xc21ba048, SUC_lexicon_tags_25 },
    { 0xae5ba048, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533ba04f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159ba054, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fba057, SUC_lexicon_tags_17 },
    { 0xa81ba057, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbb3ba05a, SUC_lexicon_tags_31 },
    { 0xcd1ba05a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8adba05d, SUC_lexicon_tags_19 },
    { 0x777ba05d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63dba060, SUC_lexicon_tags_2 },
    { 0x65fba061, SUC_lexicon_tags_31 },
    { 0xe31ba062, SUC_lexicon_tags_29 },
    { 0xa05ba063, SUC_lexicon_tags_1 },
    { 0x117ba063, SUC_lexicon_tags_35 },
    { 0x81bba060, SUC_lexicon_tags_28 },
    { 0xf9ba060, SUC_lexicon_tags_8 },
    { 0x3c9ba067, SUC_lexicon_tags_1 },
    { 0xdbba068, SUC_lexicon_tags_11 },
    { 0xabba069, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x2c1ba06b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x103ba06d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dba073, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1ba077, SUC_lexicon_tags_23 },
    { 0xe49ba078, SUC_lexicon_tags_28 },
    { 0xae3ba079, SUC_lexicon_tags_15 },
    { 0x609ba07a, SUC_lexicon_tags_53 },
    { 0xfe7ba079, SUC_lexicon_tags_27 },
    { 0x3f7ba07c, SUC_lexicon_tags_23 },
    { 0x593ba07d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfba080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3cdba082, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28dba087, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb3ba089, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x419ba08b, SUC_lexicon_tags_55 },
    { 0x6b5ba08c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9ba09a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf93ba09c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbba0a0, SUC_lexicon_tags_28 },
    { 0xddbba0a1, SUC_lexicon_tags_17 },
    { 0x66fba0a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1ba0a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bfba0a7, SUC_lexicon_tags_7 },
    { 0x147ba0a7, SUC_lexicon_tags_11 },
    { 0x3f5ba0a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x519ba0ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89ba0ae, SUC_lexicon_tags_4 },
    { 0x261ba0af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3ba0b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98fba0b4, SUC_lexicon_tags_1 },
    { 0xaf1ba0b5, SUC_lexicon_tags_11 },
    { 0xa35ba0b4, SUC_lexicon_tags_37 },
    { 0x63bba0b7, SUC_lexicon_tags_23 },
    { 0xb37ba0b5, SUC_lexicon_tags_15 },
    { 0xf01ba0b5, SUC_lexicon_tags_37 },
    { 0x4fdba0b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985ba0bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x737ba0c2, SUC_lexicon_tags_17 },
    { 0xf79ba0c2, SUC_lexicon_tags_60 },
    { 0xd8bba0c3, SUC_lexicon_tags_70 },
    { 0xb93ba0c4, SUC_lexicon_tags_17 },
    { 0xca1ba0c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54bba0c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15ba0cc, SUC_lexicon_tags_82 },
    { 0x989ba0cd, SUC_lexicon_tags_3 },
    { 0xeffba0ce, SUC_lexicon_tags_28 },
    { 0x9fdba0ce, SUC_lexicon_tags_6 },
    { 0x39ba0cc, SUC_lexicon_tags_11 },
    { 0x951ba0d1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfba0d4, SUC_lexicon_tags_15 },
    { 0xf9dba0d4, SUC_lexicon_tags_1 },
    { 0x735ba0d6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73ba0db, SUC_lexicon_tags_33 },
    { 0x8d9ba0dc, SUC_lexicon_tags_33 },
    { 0xa91ba0dd, SUC_lexicon_tags_39 },
    { 0x8b3ba0de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85ba0e2, SUC_lexicon_tags_47 },
    { 0x917ba0e2, SUC_lexicon_tags_65 },
    { 0x3d3ba0e4, SUC_lexicon_tags_7 },
    { 0x103ba0e5, SUC_lexicon_tags_47 },
    { 0x25bba0e5, SUC_lexicon_tags_47 },
    { 0x821ba0e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bba0ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xef9ba0ef, SUC_lexicon_tags_201 },
    { 0xdf5ba0ef, SUC_lexicon_tags_37 },
    { 0x2e9ba0f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x541ba0f3, SUC_lexicon_tags_37 },
    { 0x3e3ba0f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb8bba0f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcadba0f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fba0fd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731ba101, SUC_lexicon_tags_53 },
    { 0x87dba101, SUC_lexicon_tags_19 },
    { 0xa5dba103, SUC_lexicon_tags_0 },
    { 0xe25ba102, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x711ba106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d3ba108, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dba10c, SUC_lexicon_tags_11 },
    { 0x163ba10d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83dba110, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5ba115, SUC_lexicon_tags_37 },
    { 0x98dba116, SUC_lexicon_tags_33 },
    { 0x473ba117, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x443ba119, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6d9ba11b, SUC_lexicon_tags_30 },
    { 0x445ba11c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55ba12a, SUC_lexicon_tags_63 },
    { 0x179ba12b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bba12f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x257ba131, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbba135, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bba137, SUC_lexicon_tags_12 },
    { 0x9f1ba138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1ba13e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441ba143, SUC_lexicon_tags_60 },
    { 0x889ba144, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x789ba147, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1ba14d, SUC_lexicon_tags_27 },
    { 0x19bba14e, SUC_lexicon_tags_55 },
    { 0x239ba14f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9ba154, SUC_lexicon_tags_45 },
    { 0x7f5ba155, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73ba158, SUC_lexicon_tags_45 },
    { 0x1a3ba159, SUC_lexicon_tags_40 },
    { 0xf7dba159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x383ba15c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ba15f, SUC_lexicon_tags_65 },
    { 0xc2fba160, SUC_lexicon_tags_12 },
    { 0xf07ba161, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfba167, SUC_lexicon_tags_31 },
    { 0xcafba168, SUC_lexicon_tags_2 },
    { 0x969ba169, SUC_lexicon_tags_27 },
    { 0xc2bba169, SUC_lexicon_tags_27 },
    { 0xdf5ba169, SUC_lexicon_tags_45 },
    { 0x9dfba16c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6fba16e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dba173, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bba17a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc35ba17c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x53bba17e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bba184, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x98dba186, SUC_lexicon_tags_12 },
    { 0x39dba187, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc17ba189, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x387ba18b, SUC_lexicon_tags_37 },
    { 0x3a7ba18b, SUC_lexicon_tags_23 },
    { 0x559ba18d, SUC_lexicon_tags_15 },
    { 0x3a5ba18e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x473ba191, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493ba195, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fba199, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x693ba19b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a7ba19d, SUC_lexicon_tags_37 },
    { 0x2a7ba19d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515ba1a5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x17ba1a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565ba1ac, SUC_lexicon_tags_8 },
    { 0x4d3ba1ac, SUC_lexicon_tags_45 },
    { 0x6efba1ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93ba1b3, SUC_lexicon_tags_9 },
    { 0x709ba1b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x453ba1b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3fba1b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ba1bd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89ba1c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77ba1c3, SUC_lexicon_tags_17 },
    { 0xf67ba1c4, SUC_lexicon_tags_33 },
    { 0xe97ba1c4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa7fba1c7, SUC_lexicon_tags_17 },
    { 0x5c5ba1c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17ba1ce, SUC_lexicon_tags_183 },
    { 0, NULL },
    { 0x7a7ba1d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ba1d3, SUC_lexicon_tags_49 },
    { 0x3e5ba1d3, SUC_lexicon_tags_2 },
    { 0x913ba1d5, SUC_lexicon_tags_23 },
    { 0x9f1ba1d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dba1da, SUC_lexicon_tags_35 },
    { 0xa3ba1db, SUC_lexicon_tags_14 },
    { 0xa63ba1dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dba1df, SUC_lexicon_tags_17 },
    { 0x87dba1df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x28dba1e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dba1e5, SUC_lexicon_tags_42 },
    { 0x6a5ba1e6, SUC_lexicon_tags_29 },
    { 0x13fba1e6, SUC_lexicon_tags_15 },
    { 0x773ba1e7, SUC_lexicon_tags_15 },
    { 0x807ba1e9, SUC_lexicon_tags_17 },
    { 0x7bdba1e9, SUC_lexicon_tags_75 },
    { 0xe37ba1e6, SUC_lexicon_tags_49 },
    { 0xed1ba1e5, SUC_lexicon_tags_37 },
    { 0x68bba1ed, SUC_lexicon_tags_15 },
    { 0xcb5ba1ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbba1f7, SUC_lexicon_tags_4 },
    { 0x51ba1f8, SUC_lexicon_tags_1 },
    { 0x7e3ba1f9, SUC_lexicon_tags_33 },
    { 0x3bfba1f9, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x1dba1fc, SUC_lexicon_tags_15 },
    { 0x8e3ba1fd, SUC_lexicon_tags_17 },
    { 0x29fba1fd, SUC_lexicon_tags_56 },
    { 0x613ba1fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x92dba201, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbba214, SUC_lexicon_tags_12 },
    { 0x9e5ba214, SUC_lexicon_tags_1 },
    { 0xb57ba216, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7ba21c, SUC_lexicon_tags_37 },
    { 0xca1ba21c, SUC_lexicon_tags_63 },
    { 0x631ba21c, SUC_lexicon_tags_19 },
    { 0x395ba21f, SUC_lexicon_tags_2 },
    { 0xc73ba220, SUC_lexicon_tags_15 },
    { 0x7a5ba21c, SUC_lexicon_tags_7 },
    { 0xd89ba222, SUC_lexicon_tags_1 },
    { 0x1dbba222, SUC_lexicon_tags_2 },
    { 0xa19ba223, SUC_lexicon_tags_8 },
    { 0xe2bba221, SUC_lexicon_tags_47 },
    { 0x973ba21d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39ba229, SUC_lexicon_tags_1 },
    { 0xb5bba22a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x891ba22c, SUC_lexicon_tags_11 },
    { 0x5e5ba22d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x40dba22f, SUC_lexicon_tags_25 },
    { 0xe3fba230, SUC_lexicon_tags_121 },
    { 0x86bba231, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3ba235, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fba23a, SUC_lexicon_tags_11 },
    { 0x5d5ba23b, SUC_lexicon_tags_14 },
    { 0x9e9ba23c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfba23e, SUC_lexicon_tags_1 },
    { 0x3c5ba23e, SUC_lexicon_tags_47 },
    { 0xaa3ba23e, SUC_lexicon_tags_17 },
    { 0x12bba241, SUC_lexicon_tags_12 },
    { 0xfa9ba242, SUC_lexicon_tags_17 },
    { 0x7edba243, SUC_lexicon_tags_37 },
    { 0x28dba244, SUC_lexicon_tags_5 },
    { 0xcb7ba242, SUC_lexicon_tags_37 },
    { 0xdc9ba243, SUC_lexicon_tags_17 },
    { 0xe3bba23f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3ba24a, SUC_lexicon_tags_23 },
    { 0xc99ba24b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbba250, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3ba254, SUC_lexicon_tags_4 },
    { 0xdebba254, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99ba258, SUC_lexicon_tags_17 },
    { 0xd99ba259, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfba25d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x647ba260, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5ba263, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1ba266, SUC_lexicon_tags_7 },
    { 0xf31ba267, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd1ba269, SUC_lexicon_tags_15 },
    { 0x5fba269, SUC_lexicon_tags_17 },
    { 0xf67ba26b, SUC_lexicon_tags_7 },
    { 0x4fdba26b, SUC_lexicon_tags_29 },
    { 0xa35ba26d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xadbba26f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72fba271, SUC_lexicon_tags_17 },
    { 0x357ba272, SUC_lexicon_tags_11 },
    { 0xaf9ba272, SUC_lexicon_tags_31 },
    { 0x8e9ba274, SUC_lexicon_tags_23 },
    { 0x8f3ba275, SUC_lexicon_tags_25 },
    { 0x473ba275, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x53bba279, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x60fba27b, SUC_lexicon_tags_29 },
    { 0xc87ba27b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307ba280, SUC_lexicon_tags_1 },
    { 0x9adba281, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x151ba283, SUC_lexicon_tags_21 },
    { 0x51dba284, SUC_lexicon_tags_63 },
    { 0xde5ba284, SUC_lexicon_tags_27 },
    { 0x4e1ba286, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddba289, SUC_lexicon_tags_43 },
    { 0xdefba289, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ba28f, SUC_lexicon_tags_1 },
    { 0x6d5ba28f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab5ba292, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdcfba294, SUC_lexicon_tags_17 },
    { 0xe31ba295, SUC_lexicon_tags_12 },
    { 0xec3ba296, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1ba29d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5ba2a3, SUC_lexicon_tags_1 },
    { 0x8d9ba2a4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x99bba2a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb77ba2a8, SUC_lexicon_tags_30 },
    { 0x623ba2a9, SUC_lexicon_tags_35 },
    { 0xb25ba2aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711ba2ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187ba2b2, SUC_lexicon_tags_1 },
    { 0x7f5ba2b3, SUC_lexicon_tags_25 },
    { 0x137ba2b2, SUC_lexicon_tags_62 },
    { 0xf8bba2b3, SUC_lexicon_tags_10 },
    { 0xae1ba2b6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xd75ba2b8, SUC_lexicon_tags_38 },
    { 0x3d1ba2b8, SUC_lexicon_tags_27 },
    { 0x3a7ba2b8, SUC_lexicon_tags_25 },
    { 0x7cfba2bb, SUC_lexicon_tags_15 },
    { 0x67bba2bc, SUC_lexicon_tags_12 },
    { 0xfbba2bd, SUC_lexicon_tags_25 },
    { 0xf75ba2bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9ba2c2, SUC_lexicon_tags_63 },
    { 0x197ba2c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf7ba2c5, SUC_lexicon_tags_11 },
    { 0xbdba2c6, SUC_lexicon_tags_33 },
    { 0xb1fba2c7, SUC_lexicon_tags_25 },
    { 0x6cfba2c7, SUC_lexicon_tags_15 },
    { 0x129ba2c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x243ba2cb, SUC_lexicon_tags_31 },
    { 0xce9ba2cb, SUC_lexicon_tags_30 },
    { 0x805ba2cb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x82fba2cf, SUC_lexicon_tags_34 },
    { 0xff7ba2d0, SUC_lexicon_tags_17 },
    { 0xa87ba2d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bba2d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe67ba2d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc09ba2d8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xadfba2db, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afba2e1, SUC_lexicon_tags_1 },
    { 0x371ba2e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bba2e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x61bba2ea, SUC_lexicon_tags_25 },
    { 0x43dba2eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbd9ba2ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7e9ba2ef, SUC_lexicon_tags_1 },
    { 0x6dfba2f0, SUC_lexicon_tags_0 },
    { 0xa31ba2f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x831ba2f4, SUC_lexicon_tags_14 },
    { 0xdcdba2f5, SUC_lexicon_tags_17 },
    { 0x8d7ba2f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb7ba2f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7ba2fc, SUC_lexicon_tags_17 },
    { 0x6a1ba2fd, SUC_lexicon_tags_25 },
    { 0xabdba2fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727ba304, SUC_lexicon_tags_7 },
    { 0xa67ba304, SUC_lexicon_tags_25 },
    { 0x109ba306, SUC_lexicon_tags_3 },
    { 0x5ffba305, SUC_lexicon_tags_47 },
    { 0xccfba308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x127ba30a, SUC_lexicon_tags_11 },
    { 0x5bba30b, SUC_lexicon_tags_60 },
    { 0x9fba30b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6ebba30e, SUC_lexicon_tags_21 },
    { 0xdabba30e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ba315, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea5ba317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bba31c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x813ba31f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5ba323, SUC_lexicon_tags_14 },
    { 0xba9ba324, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afba328, SUC_lexicon_tags_4 },
    { 0xd95ba328, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591ba32e, SUC_lexicon_tags_49 },
    { 0x98fba32f, SUC_lexicon_tags_29 },
    { 0x93ba32e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbba338, SUC_lexicon_tags_60 },
    { 0x81ba339, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbd9ba33b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7ba33e, SUC_lexicon_tags_19 },
    { 0xe57ba33f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebba342, SUC_lexicon_tags_4 },
    { 0x8d3ba343, SUC_lexicon_tags_17 },
    { 0xf97ba344, SUC_lexicon_tags_63 },
    { 0xb79ba344, SUC_lexicon_tags_15 },
    { 0xa4dba346, SUC_lexicon_tags_15 },
    { 0x78fba347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd05ba349, SUC_lexicon_tags_481 },
    { 0x8edba34a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x28bba34c, SUC_lexicon_tags_49 },
    { 0xa99ba34c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12bba34f, SUC_lexicon_tags_39 },
    { 0x8bfba350, SUC_lexicon_tags_10 },
    { 0xed9ba351, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97ba354, SUC_lexicon_tags_63 },
    { 0xc63ba354, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9ba35e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5ba361, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77ba36c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xddfba36e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2fba370, SUC_lexicon_tags_37 },
    { 0xe17ba370, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bba37d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dba37f, SUC_lexicon_tags_49 },
    { 0xc07ba380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc71ba382, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e9ba384, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x667ba387, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb31ba389, SUC_lexicon_tags_12 },
    { 0x9ebba389, SUC_lexicon_tags_11 },
    { 0x43fba38b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddba38e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70dba390, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167ba394, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afba397, SUC_lexicon_tags_7 },
    { 0x9e9ba398, SUC_lexicon_tags_45 },
    { 0x1a9ba398, SUC_lexicon_tags_65 },
    { 0x6bfba397, SUC_lexicon_tags_33 },
    { 0x15fba397, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x235ba39d, SUC_lexicon_tags_45 },
    { 0xd97ba39d, SUC_lexicon_tags_6 },
    { 0xb9ba39f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2bfba3a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x321ba3a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1ba3a6, SUC_lexicon_tags_47 },
    { 0x67bba3a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x971ba3aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15ba3ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa69ba3b0, SUC_lexicon_tags_11 },
    { 0xb91ba3b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x15dba3b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d7ba3b5, SUC_lexicon_tags_17 },
    { 0x20dba3b6, SUC_lexicon_tags_37 },
    { 0x389ba3b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdfba3b9, SUC_lexicon_tags_1 },
    { 0xe71ba3ba, SUC_lexicon_tags_27 },
    { 0xf9dba3bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485ba3c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87ba3c7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865ba3cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7ba3ce, SUC_lexicon_tags_37 },
    { 0x2bbba3cf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335ba3d4, SUC_lexicon_tags_1 },
    { 0x3b5ba3d5, SUC_lexicon_tags_39 },
    { 0xf7bba3d6, SUC_lexicon_tags_29 },
    { 0xbc5ba3d7, SUC_lexicon_tags_15 },
    { 0xabba3d6, SUC_lexicon_tags_27 },
    { 0x8bbba3d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1ba3e1, SUC_lexicon_tags_25 },
    { 0xed1ba3e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69ba3e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7ba3ed, SUC_lexicon_tags_10 },
    { 0x63dba3ee, SUC_lexicon_tags_0 },
    { 0xf05ba3ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5ba3f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfba3fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdba3fe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339ba402, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fba405, SUC_lexicon_tags_17 },
    { 0x285ba406, SUC_lexicon_tags_7 },
    { 0x93bba407, SUC_lexicon_tags_33 },
    { 0x76bba408, SUC_lexicon_tags_12 },
    { 0xcc9ba408, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f7ba40b, SUC_lexicon_tags_30 },
    { 0xbba40c, SUC_lexicon_tags_27 },
    { 0x1a1ba40d, SUC_lexicon_tags_33 },
    { 0x547ba40e, SUC_lexicon_tags_15 },
    { 0x1a1ba40f, SUC_lexicon_tags_15 },
    { 0x30bba40e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493ba414, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x149ba416, SUC_lexicon_tags_11 },
    { 0x767ba416, SUC_lexicon_tags_7 },
    { 0x49dba417, SUC_lexicon_tags_7 },
    { 0xe83ba417, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1ba41d, SUC_lexicon_tags_17 },
    { 0xb61ba41d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x833ba420, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x601ba422, SUC_lexicon_tags_47 },
    { 0x121ba423, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997ba427, SUC_lexicon_tags_151 },
    { 0x925ba428, SUC_lexicon_tags_46 },
    { 0xeb9ba427, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbba42f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335ba435, SUC_lexicon_tags_63 },
    { 0xbefba436, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddba439, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121ba43e, SUC_lexicon_tags_6 },
    { 0xf05ba43f, SUC_lexicon_tags_12 },
    { 0xad1ba43f, SUC_lexicon_tags_28 },
    { 0xcd3ba441, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ba446, SUC_lexicon_tags_143 },
    { 0, NULL },
    { 0, NULL },
    { 0x229ba449, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x53bba44b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27ba452, SUC_lexicon_tags_7 },
    { 0xe65ba453, SUC_lexicon_tags_1 },
    { 0xdc3ba454, SUC_lexicon_tags_14 },
    { 0xb73ba455, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xa65ba457, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x133ba459, SUC_lexicon_tags_37 },
    { 0x903ba459, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5ba45e, SUC_lexicon_tags_47 },
    { 0x197ba45f, SUC_lexicon_tags_42 },
    { 0xe17ba45f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x765ba463, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101ba469, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbb9ba46b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3ba46e, SUC_lexicon_tags_23 },
    { 0x5f5ba46f, SUC_lexicon_tags_31 },
    { 0xcd3ba470, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ba477, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb53ba47a, SUC_lexicon_tags_0 },
    { 0xb3fba47b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x703ba47e, SUC_lexicon_tags_11 },
    { 0xdcfba47f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x363ba481, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9ba489, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbfbba48b, SUC_lexicon_tags_68 },
    { 0x2f9ba48c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3ba48f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47ba492, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe87ba494, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdd7ba496, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79ba49a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9ba49e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x319ba4a0, SUC_lexicon_tags_42 },
    { 0xbcdba4a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6afba4a3, SUC_lexicon_tags_11 },
    { 0xc81ba4a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x959ba4a7, SUC_lexicon_tags_10 },
    { 0xaa1ba4a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47fba4aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5ba4ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809ba4b2, SUC_lexicon_tags_17 },
    { 0x65dba4b3, SUC_lexicon_tags_1 },
    { 0x48dba4b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1ba4b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c7ba4ba, SUC_lexicon_tags_37 },
    { 0xe17ba4ba, SUC_lexicon_tags_25 },
    { 0xb15ba4ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51ba4c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7ba4c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bba4c8, SUC_lexicon_tags_25 },
    { 0xee3ba4c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf3ba4cb, SUC_lexicon_tags_15 },
    { 0xb53ba4cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf09ba4ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65dba4d0, SUC_lexicon_tags_17 },
    { 0x77fba4d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bba4db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e3ba4dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43ba4e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fba4e7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x707ba4e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed9ba4eb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9ba4f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfba4f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbafba4f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x555ba4f8, SUC_lexicon_tags_47 },
    { 0xe05ba4f9, SUC_lexicon_tags_25 },
    { 0xbe9ba4f8, SUC_lexicon_tags_47 },
    { 0x65bba4f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69ba503, SUC_lexicon_tags_2 },
    { 0x7efba504, SUC_lexicon_tags_4 },
    { 0x53bba505, SUC_lexicon_tags_29 },
    { 0x5dbba506, SUC_lexicon_tags_25 },
    { 0x9b1ba507, SUC_lexicon_tags_13 },
    { 0x8bfba508, SUC_lexicon_tags_151 },
    { 0xa35ba507, SUC_lexicon_tags_11 },
    { 0xa31ba50a, SUC_lexicon_tags_42 },
    { 0x463ba50a, SUC_lexicon_tags_82 },
    { 0xac1ba508, SUC_lexicon_tags_31 },
    { 0x7ddba50d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xea5ba50f, SUC_lexicon_tags_21 },
    { 0x557ba510, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbba517, SUC_lexicon_tags_2 },
    { 0x1adba517, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437ba51c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1ba522, SUC_lexicon_tags_19 },
    { 0xc29ba523, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53ba528, SUC_lexicon_tags_2 },
    { 0x183ba528, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5ba539, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3ba53c, SUC_lexicon_tags_25 },
    { 0x8abba53c, SUC_lexicon_tags_29 },
    { 0x3d3ba53d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03ba541, SUC_lexicon_tags_16 },
    { 0x73fba542, SUC_lexicon_tags_30 },
    { 0xa4bba542, SUC_lexicon_tags_14 },
    { 0x52bba542, SUC_lexicon_tags_7 },
    { 0x753ba544, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fba54b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xddba54d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29dba54f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bba556, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73fba558, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x411ba55a, SUC_lexicon_tags_1 },
    { 0x28dba55a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53ba55f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x723ba561, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x181ba563, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7ba567, SUC_lexicon_tags_11 },
    { 0x8ffba568, SUC_lexicon_tags_12 },
    { 0x191ba568, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bba56d, SUC_lexicon_tags_6 },
    { 0x853ba56e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x52fba571, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47bba575, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x53fba577, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39ba57a, SUC_lexicon_tags_2 },
    { 0x7b9ba57b, SUC_lexicon_tags_33 },
    { 0x12dba57c, SUC_lexicon_tags_28 },
    { 0x93ba57d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb43ba57f, SUC_lexicon_tags_46 },
    { 0xbf7ba580, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97fba585, SUC_lexicon_tags_7 },
    { 0xcafba586, SUC_lexicon_tags_18 },
    { 0x6e9ba587, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xf29ba589, SUC_lexicon_tags_13 },
    { 0x18fba58a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdba58e, SUC_lexicon_tags_42 },
    { 0x29bba58f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dba592, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb4fba594, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69bba596, SUC_lexicon_tags_28 },
    { 0x869ba597, SUC_lexicon_tags_26 },
    { 0x399ba598, SUC_lexicon_tags_17 },
    { 0x683ba599, SUC_lexicon_tags_2 },
    { 0x9f5ba59a, SUC_lexicon_tags_10 },
    { 0x627ba59b, SUC_lexicon_tags_37 },
    { 0x3fba59b, SUC_lexicon_tags_25 },
    { 0x8c7ba59c, SUC_lexicon_tags_17 },
    { 0x6ffba59b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe81ba5a0, SUC_lexicon_tags_9 },
    { 0x917ba5a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51ba5a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa2bba5a6, SUC_lexicon_tags_63 },
    { 0x897ba5a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1b1ba5a9, SUC_lexicon_tags_28 },
    { 0xb11ba5aa, SUC_lexicon_tags_7 },
    { 0x1b3ba5ab, SUC_lexicon_tags_10 },
    { 0x87dba5ac, SUC_lexicon_tags_30 },
    { 0x439ba5ab, SUC_lexicon_tags_25 },
    { 0xb7bba5ae, SUC_lexicon_tags_1 },
    { 0x581ba5aa, SUC_lexicon_tags_35 },
    { 0x88fba5ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7ba5b4, SUC_lexicon_tags_17 },
    { 0xccbba5b5, SUC_lexicon_tags_11 },
    { 0x6bbba5b5, SUC_lexicon_tags_7 },
    { 0x5d1ba5b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bba5bb, SUC_lexicon_tags_47 },
    { 0x44dba5bc, SUC_lexicon_tags_9 },
    { 0xf11ba5bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x667ba5c0, SUC_lexicon_tags_33 },
    { 0x40dba5c1, SUC_lexicon_tags_25 },
    { 0x4bdba5c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9ba5c5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9ba5c8, SUC_lexicon_tags_38 },
    { 0xe2bba5c9, SUC_lexicon_tags_14 },
    { 0x811ba5c8, SUC_lexicon_tags_14 },
    { 0xcabba5cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfba5cf, SUC_lexicon_tags_7 },
    { 0xa4fba5cf, SUC_lexicon_tags_13 },
    { 0x75ba5d1, SUC_lexicon_tags_49 },
    { 0xe21ba5d2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291ba5de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x57dba5e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x313ba5e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137ba5e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17ba5ec, SUC_lexicon_tags_2 },
    { 0xc89ba5ed, SUC_lexicon_tags_43 },
    { 0x59dba5ee, SUC_lexicon_tags_11 },
    { 0x451ba5ef, SUC_lexicon_tags_47 },
    { 0x4d7ba5ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5dba5f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdebba5f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dba5f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedba5fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdedba5fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xafba600, SUC_lexicon_tags_27 },
    { 0x439ba600, SUC_lexicon_tags_2 },
    { 0x907ba602, SUC_lexicon_tags_0 },
    { 0x595ba603, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bba60a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63ba60e, SUC_lexicon_tags_7 },
    { 0xf3ba60f, SUC_lexicon_tags_54 },
    { 0x6c1ba60f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x801ba613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa73ba615, SUC_lexicon_tags_47 },
    { 0x22fba616, SUC_lexicon_tags_2 },
    { 0xe5fba617, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x34bba61a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15dba61c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3ba621, SUC_lexicon_tags_46 },
    { 0xc67ba622, SUC_lexicon_tags_37 },
    { 0xaadba623, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x35dba625, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfba62a, SUC_lexicon_tags_17 },
    { 0x119ba62a, SUC_lexicon_tags_714 },
    { 0x7e9ba62a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfba62f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf37ba631, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3ba638, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5ba63f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51ba642, SUC_lexicon_tags_8 },
    { 0xf29ba643, SUC_lexicon_tags_49 },
    { 0xc81ba644, SUC_lexicon_tags_2 },
    { 0x373ba642, SUC_lexicon_tags_17 },
    { 0x629ba643, SUC_lexicon_tags_7 },
    { 0x99fba644, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dba64b, SUC_lexicon_tags_37 },
    { 0xf99ba64c, SUC_lexicon_tags_33 },
    { 0x36fba64d, SUC_lexicon_tags_25 },
    { 0x949ba64e, SUC_lexicon_tags_32 },
    { 0xf67ba64b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965ba653, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5ba656, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x19bba658, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405ba65d, SUC_lexicon_tags_15 },
    { 0x925ba65d, SUC_lexicon_tags_11 },
    { 0x17dba65f, SUC_lexicon_tags_17 },
    { 0xbdba65f, SUC_lexicon_tags_15 },
    { 0xfb3ba660, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x723ba663, SUC_lexicon_tags_45 },
    { 0xcbba664, SUC_lexicon_tags_14 },
    { 0x999ba664, SUC_lexicon_tags_55 },
    { 0x6d7ba666, SUC_lexicon_tags_57 },
    { 0x8d1ba667, SUC_lexicon_tags_43 },
    { 0xdb9ba667, SUC_lexicon_tags_65 },
    { 0x413ba664, SUC_lexicon_tags_17 },
    { 0xc19ba66a, SUC_lexicon_tags_17 },
    { 0x3b9ba667, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x59dba66e, SUC_lexicon_tags_11 },
    { 0x60bba66f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85ba677, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5ba67b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9ba67f, SUC_lexicon_tags_25 },
    { 0xfedba680, SUC_lexicon_tags_7 },
    { 0x2efba681, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d9ba683, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x999ba685, SUC_lexicon_tags_9 },
    { 0x5efba686, SUC_lexicon_tags_6 },
    { 0xe59ba686, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265ba68b, SUC_lexicon_tags_25 },
    { 0xfdba68b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9ba691, SUC_lexicon_tags_26 },
    { 0x161ba692, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7ba695, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fba699, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfb3ba69b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61ba69e, SUC_lexicon_tags_7 },
    { 0xbd1ba69f, SUC_lexicon_tags_13 },
    { 0x5a5ba69e, SUC_lexicon_tags_12 },
    { 0x107ba6a0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x901ba6a4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x951ba6a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303ba6ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79ba6af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb5dba6b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x125ba6b4, SUC_lexicon_tags_25 },
    { 0xdc5ba6b4, SUC_lexicon_tags_14 },
    { 0x74bba6b6, SUC_lexicon_tags_27 },
    { 0x471ba6b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3ba6bb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa3fba6bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fba6c1, SUC_lexicon_tags_25 },
    { 0x233ba6c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727ba6c9, SUC_lexicon_tags_17 },
    { 0x8c5ba6ca, SUC_lexicon_tags_11 },
    { 0x37dba6ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x975ba6cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8bba6cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0dba6d1, SUC_lexicon_tags_113 },
    { 0x8e7ba6d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107ba6d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x883ba6d9, SUC_lexicon_tags_26 },
    { 0xdcdba6da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8fba6dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf19ba6de, SUC_lexicon_tags_27 },
    { 0x733ba6df, SUC_lexicon_tags_28 },
    { 0x36bba6de, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5ebba6e2, SUC_lexicon_tags_21 },
    { 0xc55ba6e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x931ba6e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x859ba6e7, SUC_lexicon_tags_33 },
    { 0xc95ba6e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afba6ee, SUC_lexicon_tags_25 },
    { 0x185ba6ef, SUC_lexicon_tags_49 },
    { 0x623ba6f0, SUC_lexicon_tags_4 },
    { 0x65bba6f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd2dba6f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fba6f7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fba6fa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x927ba6fd, SUC_lexicon_tags_11 },
    { 0x239ba6fd, SUC_lexicon_tags_28 },
    { 0xcd3ba6ff, SUC_lexicon_tags_66 },
    { 0x4f3ba6ff, SUC_lexicon_tags_3 },
    { 0x7f7ba700, SUC_lexicon_tags_11 },
    { 0x8f1ba702, SUC_lexicon_tags_11 },
    { 0x927ba703, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3ba706, SUC_lexicon_tags_13 },
    { 0xc1bba707, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23ba70e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543ba715, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x881ba717, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37ba71a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ba71e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafba724, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73ba727, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8efba729, SUC_lexicon_tags_11 },
    { 0x7a1ba729, SUC_lexicon_tags_7 },
    { 0xa83ba72b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1ba72e, SUC_lexicon_tags_8 },
    { 0x487ba72f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ba732, SUC_lexicon_tags_33 },
    { 0x32bba733, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x219ba735, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5ba738, SUC_lexicon_tags_19 },
    { 0x47ba739, SUC_lexicon_tags_65 },
    { 0x559ba73a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x895ba73c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x269ba73e, SUC_lexicon_tags_1 },
    { 0xe6bba73e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597ba746, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcfba748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ba74c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x717ba74e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783ba757, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7ba75a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa0dba75c, SUC_lexicon_tags_15 },
    { 0x1afba75c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99ba760, SUC_lexicon_tags_7 },
    { 0x2dba761, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d3ba763, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x16bba765, SUC_lexicon_tags_23 },
    { 0xc35ba766, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681ba76f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601ba779, SUC_lexicon_tags_6 },
    { 0x547ba77a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x441ba77d, SUC_lexicon_tags_6 },
    { 0x2a7ba77e, SUC_lexicon_tags_63 },
    { 0x3dbba77d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbb7ba781, SUC_lexicon_tags_108 },
    { 0x6c9ba782, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243ba786, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdba78d, SUC_lexicon_tags_37 },
    { 0x17ba78e, SUC_lexicon_tags_26 },
    { 0x7fba78d, SUC_lexicon_tags_25 },
    { 0x753ba790, SUC_lexicon_tags_43 },
    { 0x4f3ba791, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1d9ba793, SUC_lexicon_tags_151 },
    { 0xd3bba794, SUC_lexicon_tags_63 },
    { 0xc73ba795, SUC_lexicon_tags_2 },
    { 0xebdba796, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1ba79c, SUC_lexicon_tags_10 },
    { 0x927ba79d, SUC_lexicon_tags_2 },
    { 0xce5ba79e, SUC_lexicon_tags_17 },
    { 0xd59ba79f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbba7a6, SUC_lexicon_tags_25 },
    { 0x95bba7a6, SUC_lexicon_tags_19 },
    { 0x8adba7a8, SUC_lexicon_tags_25 },
    { 0x18dba7a9, SUC_lexicon_tags_14 },
    { 0x1e7ba7aa, SUC_lexicon_tags_4 },
    { 0x899ba7aa, SUC_lexicon_tags_37 },
    { 0x1f3ba7a9, SUC_lexicon_tags_37 },
    { 0x451ba7aa, SUC_lexicon_tags_27 },
    { 0xadba7ab, SUC_lexicon_tags_63 },
    { 0x939ba7ae, SUC_lexicon_tags_37 },
    { 0x6d1ba7a6, SUC_lexicon_tags_63 },
    { 0x297ba7a8, SUC_lexicon_tags_61 },
    { 0xb0dba7b2, SUC_lexicon_tags_1 },
    { 0xcffba7b2, SUC_lexicon_tags_47 },
    { 0xb31ba7b4, SUC_lexicon_tags_17 },
    { 0xd91ba7af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbba7ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ba7bd, SUC_lexicon_tags_38 },
    { 0x4e9ba7be, SUC_lexicon_tags_8 },
    { 0xb75ba7bf, SUC_lexicon_tags_49 },
    { 0xe95ba7c0, SUC_lexicon_tags_19 },
    { 0xbf9ba7c0, SUC_lexicon_tags_11 },
    { 0x127ba7bf, SUC_lexicon_tags_14 },
    { 0xf61ba7c2, SUC_lexicon_tags_32 },
    { 0x2a3ba7c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7ba7ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaedba7cc, SUC_lexicon_tags_47 },
    { 0x253ba7cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107ba7d1, SUC_lexicon_tags_31 },
    { 0xf53ba7d2, SUC_lexicon_tags_47 },
    { 0x1d7ba7d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83ba7d7, SUC_lexicon_tags_63 },
    { 0x4b1ba7d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa29ba7da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2adba7dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bba7df, SUC_lexicon_tags_60 },
    { 0xdc9ba7df, SUC_lexicon_tags_2 },
    { 0xdbfba7e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4a5ba7e3, SUC_lexicon_tags_33 },
    { 0x291ba7e4, SUC_lexicon_tags_3 },
    { 0xd35ba7e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fdba7e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfbba7e9, SUC_lexicon_tags_47 },
    { 0x8efba7e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73ba7f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7fdba7f3, SUC_lexicon_tags_17 },
    { 0x439ba7f4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7ba7f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc13ba7fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3ba7fe, SUC_lexicon_tags_15 },
    { 0xca9ba7ff, SUC_lexicon_tags_47 },
    { 0xa05ba7ff, SUC_lexicon_tags_33 },
    { 0xc59ba801, SUC_lexicon_tags_6 },
    { 0xc7bba802, SUC_lexicon_tags_11 },
    { 0xfbdba803, SUC_lexicon_tags_4 },
    { 0xa23ba803, SUC_lexicon_tags_60 },
    { 0x48bba804, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x439ba807, SUC_lexicon_tags_10 },
    { 0xf29ba808, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9ba80b, SUC_lexicon_tags_82 },
    { 0x8bbba80c, SUC_lexicon_tags_30 },
    { 0xc7dba80c, SUC_lexicon_tags_37 },
    { 0x331ba80e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb0fba810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a1ba812, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39ba817, SUC_lexicon_tags_38 },
    { 0xfadba818, SUC_lexicon_tags_38 },
    { 0x54fba819, SUC_lexicon_tags_2 },
    { 0xd2bba81a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x579ba81d, SUC_lexicon_tags_65 },
    { 0x47ba81e, SUC_lexicon_tags_17 },
    { 0xc27ba81e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fba824, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbafba826, SUC_lexicon_tags_17 },
    { 0xbe3ba827, SUC_lexicon_tags_25 },
    { 0xa1bba826, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x661ba82a, SUC_lexicon_tags_10 },
    { 0xa1fba82b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdba830, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdbba832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x789ba834, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xecdba837, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95ba83a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf1ba83c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905ba840, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79ba844, SUC_lexicon_tags_19 },
    { 0xe03ba845, SUC_lexicon_tags_16 },
    { 0x64bba844, SUC_lexicon_tags_4 },
    { 0xd3ba847, SUC_lexicon_tags_10 },
    { 0xfdba847, SUC_lexicon_tags_40 },
    { 0x3f3ba849, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e3ba84b, SUC_lexicon_tags_12 },
    { 0xd5dba84c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x579ba84f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbba855, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x36dba857, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x98dba85a, SUC_lexicon_tags_38 },
    { 0xec3ba85b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7ba85e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x643ba861, SUC_lexicon_tags_27 },
    { 0x9afba862, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbba865, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c5ba867, SUC_lexicon_tags_31 },
    { 0xa0dba868, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7efba86a, SUC_lexicon_tags_2 },
    { 0x5ba86b, SUC_lexicon_tags_414 },
    { 0xbdfba86b, SUC_lexicon_tags_0 },
    { 0xb85ba86b, SUC_lexicon_tags_68 },
    { 0x8a1ba86a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bba872, SUC_lexicon_tags_60 },
    { 0xfa1ba873, SUC_lexicon_tags_7 },
    { 0x33bba874, SUC_lexicon_tags_12 },
    { 0xd85ba875, SUC_lexicon_tags_23 },
    { 0xc33ba876, SUC_lexicon_tags_63 },
    { 0xc95ba874, SUC_lexicon_tags_49 },
    { 0x5b5ba874, SUC_lexicon_tags_25 },
    { 0x37bba873, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955ba881, SUC_lexicon_tags_17 },
    { 0x92fba881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc7ba884, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd5ba886, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x13fba889, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbba88c, SUC_lexicon_tags_15 },
    { 0x243ba88d, SUC_lexicon_tags_12 },
    { 0xea3ba88d, SUC_lexicon_tags_17 },
    { 0xb3bba88f, SUC_lexicon_tags_19 },
    { 0xbafba88f, SUC_lexicon_tags_12 },
    { 0x547ba890, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bba895, SUC_lexicon_tags_37 },
    { 0xccdba895, SUC_lexicon_tags_63 },
    { 0xa91ba895, SUC_lexicon_tags_37 },
    { 0x753ba897, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7ba89d, SUC_lexicon_tags_37 },
    { 0xbfdba89e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbba8a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfba8b1, SUC_lexicon_tags_14 },
    { 0x1cdba8b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bba8b5, SUC_lexicon_tags_9 },
    { 0x93ba8b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7ba8c1, SUC_lexicon_tags_917 },
    { 0, NULL },
    { 0, NULL },
    { 0x22dba8c4, SUC_lexicon_tags_2 },
    { 0x2d1ba8c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bba8c8, SUC_lexicon_tags_12 },
    { 0x93ba8c9, SUC_lexicon_tags_21 },
    { 0x7a1ba8ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5ba8cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x50bba8d1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1ba8d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x513ba8d7, SUC_lexicon_tags_7 },
    { 0x3f3ba8d7, SUC_lexicon_tags_44 },
    { 0x815ba8d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3ba8dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27ba8df, SUC_lexicon_tags_2 },
    { 0xbabba8e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbdfba8e2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abba8e8, SUC_lexicon_tags_30 },
    { 0x35bba8e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x90bba8eb, SUC_lexicon_tags_55 },
    { 0xec9ba8eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x511ba8ee, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x331ba8f0, SUC_lexicon_tags_9 },
    { 0x93dba8f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efba8f4, SUC_lexicon_tags_15 },
    { 0xab1ba8f5, SUC_lexicon_tags_15 },
    { 0xce5ba8f5, SUC_lexicon_tags_43 },
    { 0xf4fba8f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9ba902, SUC_lexicon_tags_66 },
    { 0x6cdba903, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1b3ba905, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x773ba907, SUC_lexicon_tags_11 },
    { 0x187ba908, SUC_lexicon_tags_31 },
    { 0x7f3ba907, SUC_lexicon_tags_66 },
    { 0xac7ba908, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2bbba90c, SUC_lexicon_tags_45 },
    { 0x26bba90d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f3ba90f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d1ba911, SUC_lexicon_tags_1 },
    { 0x18bba911, SUC_lexicon_tags_14 },
    { 0xfba913, SUC_lexicon_tags_52 },
    { 0xb63ba911, SUC_lexicon_tags_5 },
    { 0x72bba913, SUC_lexicon_tags_29 },
    { 0xf1bba913, SUC_lexicon_tags_77 },
    { 0x239ba917, SUC_lexicon_tags_12 },
    { 0x949ba912, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0dba91a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x171ba91c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7ba920, SUC_lexicon_tags_27 },
    { 0x413ba921, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79ba924, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335ba928, SUC_lexicon_tags_7 },
    { 0xff7ba928, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fba92f, SUC_lexicon_tags_14 },
    { 0x977ba930, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9ba934, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x11bba936, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x895ba939, SUC_lexicon_tags_65 },
    { 0x137ba939, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41bba93c, SUC_lexicon_tags_33 },
    { 0x283ba93d, SUC_lexicon_tags_15 },
    { 0x697ba93c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3ba943, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7ba949, SUC_lexicon_tags_25 },
    { 0x5c5ba949, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe85ba94c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271ba952, SUC_lexicon_tags_37 },
    { 0xbd5ba952, SUC_lexicon_tags_1 },
    { 0xdddba954, SUC_lexicon_tags_0 },
    { 0xc75ba955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x329ba957, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4adba959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b3ba95b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x225ba95e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x233ba960, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473ba964, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdba967, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dba96e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x841ba970, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7ba975, SUC_lexicon_tags_69 },
    { 0x64fba976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf51ba978, SUC_lexicon_tags_0 },
    { 0x35ba979, SUC_lexicon_tags_7 },
    { 0x5a1ba979, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x92fba97c, SUC_lexicon_tags_48 },
    { 0xd37ba97d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x281ba97f, SUC_lexicon_tags_652 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bba982, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x56fba984, SUC_lexicon_tags_15 },
    { 0x305ba985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f9ba987, SUC_lexicon_tags_37 },
    { 0x581ba988, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x43dba98a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679ba997, SUC_lexicon_tags_37 },
    { 0x6ffba998, SUC_lexicon_tags_28 },
    { 0xdefba999, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd93ba99c, SUC_lexicon_tags_12 },
    { 0xfe3ba99d, SUC_lexicon_tags_35 },
    { 0xb91ba99e, SUC_lexicon_tags_66 },
    { 0xfc9ba99e, SUC_lexicon_tags_37 },
    { 0x8bfba9a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9ba9a7, SUC_lexicon_tags_23 },
    { 0xeddba9a8, SUC_lexicon_tags_3 },
    { 0xe5fba9a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x367ba9ac, SUC_lexicon_tags_30 },
    { 0x907ba9ac, SUC_lexicon_tags_11 },
    { 0x65ba9ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c1ba9b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83ba9b7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bba9bb, SUC_lexicon_tags_82 },
    { 0x48bba9bb, SUC_lexicon_tags_7 },
    { 0xbefba9bd, SUC_lexicon_tags_21 },
    { 0xba3ba9bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x827ba9c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bba9c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fdba9ca, SUC_lexicon_tags_3 },
    { 0x44bba9cb, SUC_lexicon_tags_26 },
    { 0xf61ba9cc, SUC_lexicon_tags_14 },
    { 0xffba9cc, SUC_lexicon_tags_8 },
    { 0x3dba9cc, SUC_lexicon_tags_32 },
    { 0xe61ba9cf, SUC_lexicon_tags_14 },
    { 0x95ba9d0, SUC_lexicon_tags_29 },
    { 0x87bba9cb, SUC_lexicon_tags_37 },
    { 0x2e7ba9d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731ba9d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7ba9d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3ba9dd, SUC_lexicon_tags_6 },
    { 0x66fba9de, SUC_lexicon_tags_0 },
    { 0x529ba9dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817ba9e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebfba9e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ba9ea, SUC_lexicon_tags_42 },
    { 0x43dba9eb, SUC_lexicon_tags_7 },
    { 0xe3fba9eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d7ba9ee, SUC_lexicon_tags_11 },
    { 0xde5ba9ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43dba9f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469ba9f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bba9fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddba9fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbbaa04, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09baa07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfbaa0c, SUC_lexicon_tags_2 },
    { 0x55fbaa0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x413baa10, SUC_lexicon_tags_7 },
    { 0x57baa11, SUC_lexicon_tags_33 },
    { 0x47baa12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03baa18, SUC_lexicon_tags_25 },
    { 0x1bbaa19, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x1fbbaa1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297baa22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa19baa24, SUC_lexicon_tags_12 },
    { 0xd93baa24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5baa2a, SUC_lexicon_tags_1 },
    { 0x21dbaa2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5baa39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21baa3c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x697baa3e, SUC_lexicon_tags_60 },
    { 0xb37baa3f, SUC_lexicon_tags_10 },
    { 0xe8fbaa40, SUC_lexicon_tags_45 },
    { 0x67bbaa41, SUC_lexicon_tags_37 },
    { 0x671baa40, SUC_lexicon_tags_37 },
    { 0xf43baa43, SUC_lexicon_tags_21 },
    { 0x51baa42, SUC_lexicon_tags_12 },
    { 0xa29baa41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67baa48, SUC_lexicon_tags_37 },
    { 0xc01baa49, SUC_lexicon_tags_39 },
    { 0xb79baa4a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb57baa4c, SUC_lexicon_tags_37 },
    { 0x1ddbaa4d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe79baa4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edbaa56, SUC_lexicon_tags_45 },
    { 0x843baa57, SUC_lexicon_tags_11 },
    { 0x499baa57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfffbaa5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf4bbaa5c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2f7baa5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ddbaa60, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3dbbaa62, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249baa67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21baa6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x515baa6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1baa75, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fbaa78, SUC_lexicon_tags_1 },
    { 0xc3bbaa79, SUC_lexicon_tags_11 },
    { 0x49baa78, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x29dbaa7d, SUC_lexicon_tags_12 },
    { 0x20fbaa7e, SUC_lexicon_tags_19 },
    { 0xe6fbaa7f, SUC_lexicon_tags_17 },
    { 0xec5baa80, SUC_lexicon_tags_37 },
    { 0x4a5baa7f, SUC_lexicon_tags_1 },
    { 0xb97baa7d, SUC_lexicon_tags_37 },
    { 0xd0bbaa7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6dbaa85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc9baa87, SUC_lexicon_tags_29 },
    { 0x53fbaa88, SUC_lexicon_tags_25 },
    { 0x999baa87, SUC_lexicon_tags_0 },
    { 0x521baa89, SUC_lexicon_tags_43 },
    { 0x959baa8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x671baa8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7baa93, SUC_lexicon_tags_2 },
    { 0x8abbaa94, SUC_lexicon_tags_25 },
    { 0x9dbbaa95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x22fbaa98, SUC_lexicon_tags_14 },
    { 0x60bbaa99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23baa9d, SUC_lexicon_tags_1 },
    { 0x8c7baa9e, SUC_lexicon_tags_37 },
    { 0x5d5baa9f, SUC_lexicon_tags_13 },
    { 0x55dbaaa0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3baaa3, SUC_lexicon_tags_612 },
    { 0xb1fbaaa3, SUC_lexicon_tags_14 },
    { 0x8a3baaa5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1baaa8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x207baaaa, SUC_lexicon_tags_17 },
    { 0xe91baaab, SUC_lexicon_tags_15 },
    { 0xf6dbaaac, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3baab2, SUC_lexicon_tags_38 },
    { 0x689baab2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9baab8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x8e5baaba, SUC_lexicon_tags_65 },
    { 0x1e3baabb, SUC_lexicon_tags_14 },
    { 0x745baaba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fbaac3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x827baac5, SUC_lexicon_tags_15 },
    { 0xe19baac6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dbaac9, SUC_lexicon_tags_0 },
    { 0xbdbaaca, SUC_lexicon_tags_38 },
    { 0xb39baacb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7baad3, SUC_lexicon_tags_18 },
    { 0x789baad4, SUC_lexicon_tags_7 },
    { 0xcbbaad5, SUC_lexicon_tags_6 },
    { 0x86dbaad6, SUC_lexicon_tags_25 },
    { 0x773baad7, SUC_lexicon_tags_424 },
    { 0xc95baad7, SUC_lexicon_tags_55 },
    { 0x145baad6, SUC_lexicon_tags_1 },
    { 0x809baada, SUC_lexicon_tags_12 },
    { 0xefbbaad8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfbaadf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fbaae2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x119baae4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x653baae6, SUC_lexicon_tags_37 },
    { 0x7fbaae6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9baaeb, SUC_lexicon_tags_42 },
    { 0xa6bbaaec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bdbaaee, SUC_lexicon_tags_11 },
    { 0x637baaef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bbaaf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1bbaaf6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x811baaf8, SUC_lexicon_tags_84 },
    { 0x911baaf9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3baafc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c7baafe, SUC_lexicon_tags_30 },
    { 0xd55baafe, SUC_lexicon_tags_8 },
    { 0xff9baafe, SUC_lexicon_tags_47 },
    { 0x31bab01, SUC_lexicon_tags_28 },
    { 0xdc3bab01, SUC_lexicon_tags_1 },
    { 0x369bab03, SUC_lexicon_tags_151 },
    { 0x3dfbab02, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bab07, SUC_lexicon_tags_14 },
    { 0x175bab08, SUC_lexicon_tags_17 },
    { 0xd75bab09, SUC_lexicon_tags_27 },
    { 0xac5bab08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2dbab0c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fbab0f, SUC_lexicon_tags_27 },
    { 0x1e5bab10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc31bab12, SUC_lexicon_tags_14 },
    { 0x793bab13, SUC_lexicon_tags_44 },
    { 0x4fdbab14, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9d5bab16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35bab1d, SUC_lexicon_tags_14 },
    { 0x743bab1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b7bab20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x579bab23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8bbab25, SUC_lexicon_tags_1 },
    { 0xe1bab26, SUC_lexicon_tags_65 },
    { 0x837bab27, SUC_lexicon_tags_15 },
    { 0xf41bab28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x887bab2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x681bab2e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05bab34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9bab39, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x16dbab3b, SUC_lexicon_tags_45 },
    { 0x97bbab3b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6a1bab3e, SUC_lexicon_tags_45 },
    { 0x8f9bab3e, SUC_lexicon_tags_11 },
    { 0x955bab40, SUC_lexicon_tags_43 },
    { 0x1a9bab41, SUC_lexicon_tags_49 },
    { 0x5f1bab3f, SUC_lexicon_tags_32 },
    { 0x651bab41, SUC_lexicon_tags_23 },
    { 0xacbbab3f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x76bbab47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fbab4e, SUC_lexicon_tags_9 },
    { 0x72fbab4f, SUC_lexicon_tags_15 },
    { 0x971bab4f, SUC_lexicon_tags_25 },
    { 0xc91bab51, SUC_lexicon_tags_23 },
    { 0xca1bab4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1bab5a, SUC_lexicon_tags_14 },
    { 0x28dbab5b, SUC_lexicon_tags_0 },
    { 0x4e3bab5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x24bbab5f, SUC_lexicon_tags_66 },
    { 0xacfbab5f, SUC_lexicon_tags_37 },
    { 0xb77bab61, SUC_lexicon_tags_3 },
    { 0x6fbab61, SUC_lexicon_tags_12 },
    { 0x819bab61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd7bab65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45dbab6c, SUC_lexicon_tags_23 },
    { 0xe69bab6c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x757bab6f, SUC_lexicon_tags_11 },
    { 0xd95bab70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8fbab72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a9bab74, SUC_lexicon_tags_12 },
    { 0xbe5bab74, SUC_lexicon_tags_2 },
    { 0xec7bab76, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x971bab78, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4fbab7a, SUC_lexicon_tags_7 },
    { 0xc31bab7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469bab85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71fbab87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99bab8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3bab8e, SUC_lexicon_tags_27 },
    { 0xfb3bab8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bbab92, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7bab99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xecdbab9c, SUC_lexicon_tags_7 },
    { 0xdc1bab9d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9baba0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783baba4, SUC_lexicon_tags_14 },
    { 0x503baba5, SUC_lexicon_tags_7 },
    { 0x2e9baba6, SUC_lexicon_tags_25 },
    { 0x117baba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7babaa, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5babae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xce5babb0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69babb4, SUC_lexicon_tags_12 },
    { 0xeabbabb5, SUC_lexicon_tags_45 },
    { 0x46dbabb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983babbc, SUC_lexicon_tags_17 },
    { 0x7bfbabbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5babc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5babc5, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bbabc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa4bbabca, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x78dbabcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937babd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efbabd7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x27fbabd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349babdd, SUC_lexicon_tags_49 },
    { 0x491babde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x513babe0, SUC_lexicon_tags_8 },
    { 0xe75babe1, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dbabe5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b5babe7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cfbabe9, SUC_lexicon_tags_55 },
    { 0xfefbabea, SUC_lexicon_tags_17 },
    { 0x837babe9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x31fbabed, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x337babf0, SUC_lexicon_tags_26 },
    { 0x357babf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1babf4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7babf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3bac08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdbac0d, SUC_lexicon_tags_2 },
    { 0xf9bbac0e, SUC_lexicon_tags_25 },
    { 0x731bac0f, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dbac14, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9a9bac16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bac1a, SUC_lexicon_tags_39 },
    { 0x2c1bac1b, SUC_lexicon_tags_0 },
    { 0x961bac1c, SUC_lexicon_tags_9 },
    { 0xbdbbac1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0dbac1f, SUC_lexicon_tags_17 },
    { 0xb95bac1f, SUC_lexicon_tags_29 },
    { 0x5dfbac21, SUC_lexicon_tags_48 },
    { 0x461bac20, SUC_lexicon_tags_47 },
    { 0xcffbac1f, SUC_lexicon_tags_26 },
    { 0x6b3bac21, SUC_lexicon_tags_11 },
    { 0xa67bac25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf63bac27, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31bac2c, SUC_lexicon_tags_17 },
    { 0x2cdbac2c, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1bac34, SUC_lexicon_tags_26 },
    { 0xf3bac35, SUC_lexicon_tags_6 },
    { 0x529bac36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dbac3a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1bac3e, SUC_lexicon_tags_37 },
    { 0x6f1bac3f, SUC_lexicon_tags_7 },
    { 0x31bac40, SUC_lexicon_tags_38 },
    { 0x907bac40, SUC_lexicon_tags_54 },
    { 0xe2bbac41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23bac45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dbac49, SUC_lexicon_tags_33 },
    { 0xc15bac49, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dbac50, SUC_lexicon_tags_15 },
    { 0x601bac50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x473bac53, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x69dbac56, SUC_lexicon_tags_2 },
    { 0xcddbac57, SUC_lexicon_tags_62 },
    { 0x911bac57, SUC_lexicon_tags_8 },
    { 0xe1dbac59, SUC_lexicon_tags_49 },
    { 0x8c9bac59, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fbac62, SUC_lexicon_tags_27 },
    { 0xb87bac63, SUC_lexicon_tags_37 },
    { 0x7f9bac64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x313bac67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fbac6a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3bac6d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fbac72, SUC_lexicon_tags_11 },
    { 0xd0bbac73, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1bac79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x92dbac7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7bac81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebbac85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b3bac87, SUC_lexicon_tags_26 },
    { 0x87bbac88, SUC_lexicon_tags_13 },
    { 0x46fbac89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11fbac8e, SUC_lexicon_tags_37 },
    { 0xaddbac8e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x60dbac91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbbac97, SUC_lexicon_tags_17 },
    { 0x41bac97, SUC_lexicon_tags_2 },
    { 0xeabbac99, SUC_lexicon_tags_23 },
    { 0x2dbac9a, SUC_lexicon_tags_1 },
    { 0x303bac97, SUC_lexicon_tags_46 },
    { 0x637bac9c, SUC_lexicon_tags_25 },
    { 0x245bac9d, SUC_lexicon_tags_37 },
    { 0x1d3bac97, SUC_lexicon_tags_15 },
    { 0xa75bac9f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x563baca1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ddbaca3, SUC_lexicon_tags_49 },
    { 0x18bbaca3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x323baca6, SUC_lexicon_tags_47 },
    { 0xa69baca7, SUC_lexicon_tags_27 },
    { 0x1f3baca8, SUC_lexicon_tags_79 },
    { 0x2f5baca6, SUC_lexicon_tags_12 },
    { 0x527bacaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x375bacad, SUC_lexicon_tags_21 },
    { 0x539bacae, SUC_lexicon_tags_12 },
    { 0x4f3bacaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffbacb4, SUC_lexicon_tags_47 },
    { 0x67fbacb4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x28dbacb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5bacba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x32fbacbd, SUC_lexicon_tags_1 },
    { 0x811bacbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7bacc2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75bacc5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbacca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8bfbaccc, SUC_lexicon_tags_26 },
    { 0x3e9baccd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xadfbaccf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcefbacd1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb59bacd3, SUC_lexicon_tags_7 },
    { 0x97bbacd4, SUC_lexicon_tags_2 },
    { 0x83bbacd5, SUC_lexicon_tags_19 },
    { 0x18bbacd6, SUC_lexicon_tags_49 },
    { 0x1adbacd5, SUC_lexicon_tags_38 },
    { 0xc95bacd8, SUC_lexicon_tags_7 },
    { 0xc79bacd5, SUC_lexicon_tags_46 },
    { 0x5b7bacd7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bbace0, SUC_lexicon_tags_11 },
    { 0x653bace1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dbace6, SUC_lexicon_tags_17 },
    { 0xf49bace7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdebbace9, SUC_lexicon_tags_1 },
    { 0x71fbacea, SUC_lexicon_tags_30 },
    { 0x2d3baceb, SUC_lexicon_tags_1 },
    { 0x505bacec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc77bacee, SUC_lexicon_tags_7 },
    { 0xea9bacee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bbacf3, SUC_lexicon_tags_17 },
    { 0x97dbacf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3bacf7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9bacfc, SUC_lexicon_tags_45 },
    { 0x95fbacfd, SUC_lexicon_tags_63 },
    { 0x123bacfe, SUC_lexicon_tags_47 },
    { 0x1c3bacff, SUC_lexicon_tags_49 },
    { 0xd77bad00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x357bad02, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x705bad05, SUC_lexicon_tags_25 },
    { 0x3f1bad06, SUC_lexicon_tags_43 },
    { 0x9c1bad07, SUC_lexicon_tags_25 },
    { 0x3cbbad06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x349bad0b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfbad0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x827bad12, SUC_lexicon_tags_43 },
    { 0xb5bad13, SUC_lexicon_tags_1 },
    { 0xdc3bad14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269bad18, SUC_lexicon_tags_12 },
    { 0x57fbad18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x36fbad1c, SUC_lexicon_tags_2 },
    { 0xa27bad1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dbad21, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605bad25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dbad2a, SUC_lexicon_tags_53 },
    { 0x4c1bad2b, SUC_lexicon_tags_37 },
    { 0xbbfbad2c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8afbad2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc03bad31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90dbad34, SUC_lexicon_tags_49 },
    { 0x657bad35, SUC_lexicon_tags_26 },
    { 0xb2fbad36, SUC_lexicon_tags_2 },
    { 0x5ddbad37, SUC_lexicon_tags_39 },
    { 0x3bad37, SUC_lexicon_tags_11 },
    { 0xa8dbad36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddbad42, SUC_lexicon_tags_47 },
    { 0x511bad43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x235bad45, SUC_lexicon_tags_17 },
    { 0x503bad46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41bad4a, SUC_lexicon_tags_34 },
    { 0x961bad4b, SUC_lexicon_tags_2 },
    { 0x7d3bad4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fbad50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x701bad52, SUC_lexicon_tags_63 },
    { 0x61bbad53, SUC_lexicon_tags_15 },
    { 0xc5bad53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x991bad57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x159bad59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1bad5c, SUC_lexicon_tags_17 },
    { 0xb91bad5c, SUC_lexicon_tags_37 },
    { 0x607bad5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfbad62, SUC_lexicon_tags_12 },
    { 0x273bad63, SUC_lexicon_tags_6 },
    { 0xd9bbad64, SUC_lexicon_tags_12 },
    { 0xc7bad65, SUC_lexicon_tags_7 },
    { 0xda9bad65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bbad6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc63bad71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967bad77, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x809bad79, SUC_lexicon_tags_12 },
    { 0x65bad7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa5bad7c, SUC_lexicon_tags_0 },
    { 0x6e1bad7d, SUC_lexicon_tags_27 },
    { 0x647bad7c, SUC_lexicon_tags_3 },
    { 0xff7bad7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bad82, SUC_lexicon_tags_65 },
    { 0x4c1bad83, SUC_lexicon_tags_1 },
    { 0xbbbad82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb5bad86, SUC_lexicon_tags_65 },
    { 0x631bad86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bbad8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefbad93, SUC_lexicon_tags_28 },
    { 0x3ddbad93, SUC_lexicon_tags_14 },
    { 0x85fbad95, SUC_lexicon_tags_37 },
    { 0x2c9bad96, SUC_lexicon_tags_22 },
    { 0x5fbad94, SUC_lexicon_tags_63 },
    { 0x8f7bad96, SUC_lexicon_tags_32 },
    { 0x5cbbad99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bad9d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x4bdbad9f, SUC_lexicon_tags_57 },
    { 0x63bbad9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91bada3, SUC_lexicon_tags_17 },
    { 0x787bada4, SUC_lexicon_tags_2 },
    { 0xdd7bada5, SUC_lexicon_tags_25 },
    { 0xb63bada5, SUC_lexicon_tags_29 },
    { 0x56bbada7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149badab, SUC_lexicon_tags_47 },
    { 0x735badac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dbadaf, SUC_lexicon_tags_12 },
    { 0x8b7badaf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1badb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe1badb5, SUC_lexicon_tags_3 },
    { 0x1a3badb6, SUC_lexicon_tags_45 },
    { 0x769badb6, SUC_lexicon_tags_37 },
    { 0x199badb8, SUC_lexicon_tags_95 },
    { 0x8a7badb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fbadbc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb33badbe, SUC_lexicon_tags_1 },
    { 0x87dbadbf, SUC_lexicon_tags_23 },
    { 0xc59badbe, SUC_lexicon_tags_11 },
    { 0x201badc1, SUC_lexicon_tags_17 },
    { 0x557badbf, SUC_lexicon_tags_23 },
    { 0xf37badc3, SUC_lexicon_tags_37 },
    { 0x4ffbadc4, SUC_lexicon_tags_17 },
    { 0xb35badc4, SUC_lexicon_tags_15 },
    { 0xa79badc2, SUC_lexicon_tags_13 },
    { 0x373badc6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bbadce, SUC_lexicon_tags_17 },
    { 0x527badcf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x319badd2, SUC_lexicon_tags_29 },
    { 0xb53badd3, SUC_lexicon_tags_37 },
    { 0x13dbadd3, SUC_lexicon_tags_9 },
    { 0xb15badd3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x88dbadd7, SUC_lexicon_tags_39 },
    { 0x5f1badd8, SUC_lexicon_tags_12 },
    { 0x64bbadd8, SUC_lexicon_tags_47 },
    { 0xe49badda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x499baddc, SUC_lexicon_tags_37 },
    { 0x403baddc, SUC_lexicon_tags_60 },
    { 0x733baddc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc63bade0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dbade3, SUC_lexicon_tags_23 },
    { 0x451bade3, SUC_lexicon_tags_17 },
    { 0x507bade5, SUC_lexicon_tags_1 },
    { 0x521bade5, SUC_lexicon_tags_62 },
    { 0x369bade7, SUC_lexicon_tags_25 },
    { 0xaf5bade8, SUC_lexicon_tags_11 },
    { 0x1adbade8, SUC_lexicon_tags_32 },
    { 0xab5badea, SUC_lexicon_tags_2 },
    { 0x80dbadea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x233baded, SUC_lexicon_tags_11 },
    { 0x515badee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7badf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1badf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffbadf9, SUC_lexicon_tags_17 },
    { 0x65bbadf9, SUC_lexicon_tags_25 },
    { 0x7b3badfb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9edbadfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fbbadff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abbae04, SUC_lexicon_tags_14 },
    { 0xc41bae04, SUC_lexicon_tags_14 },
    { 0xbc5bae05, SUC_lexicon_tags_47 },
    { 0x55dbae07, SUC_lexicon_tags_19 },
    { 0x4b3bae04, SUC_lexicon_tags_32 },
    { 0xd05bae09, SUC_lexicon_tags_1 },
    { 0x4d9bae0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9bae0e, SUC_lexicon_tags_1 },
    { 0x909bae0f, SUC_lexicon_tags_47 },
    { 0xb8dbae0f, SUC_lexicon_tags_11 },
    { 0xf01bae10, SUC_lexicon_tags_8 },
    { 0xdf5bae12, SUC_lexicon_tags_37 },
    { 0x695bae13, SUC_lexicon_tags_21 },
    { 0xe9bbae14, SUC_lexicon_tags_15 },
    { 0xbcdbae0e, SUC_lexicon_tags_1 },
    { 0x24dbae11, SUC_lexicon_tags_85 },
    { 0xbc3bae11, SUC_lexicon_tags_35 },
    { 0x1cbbae18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadbae1b, SUC_lexicon_tags_17 },
    { 0xb3bae1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bbae21, SUC_lexicon_tags_9 },
    { 0x56dbae22, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf0bbae24, SUC_lexicon_tags_17 },
    { 0x6ffbae24, SUC_lexicon_tags_41 },
    { 0xbb3bae26, SUC_lexicon_tags_7 },
    { 0x59bbae25, SUC_lexicon_tags_37 },
    { 0x731bae26, SUC_lexicon_tags_0 },
    { 0xa1bae29, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adbae2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47bae30, SUC_lexicon_tags_8 },
    { 0xd7dbae31, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x72bbae33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57bae37, SUC_lexicon_tags_9 },
    { 0xd4dbae38, SUC_lexicon_tags_1 },
    { 0x283bae38, SUC_lexicon_tags_37 },
    { 0x9e5bae39, SUC_lexicon_tags_0 },
    { 0xd45bae3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9bae3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x935bae42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x41dbae44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809bae48, SUC_lexicon_tags_2 },
    { 0x863bae48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51fbae4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb99bae4d, SUC_lexicon_tags_14 },
    { 0x943bae4d, SUC_lexicon_tags_58 },
    { 0x35dbae4f, SUC_lexicon_tags_43 },
    { 0xd39bae4f, SUC_lexicon_tags_31 },
    { 0x235bae51, SUC_lexicon_tags_25 },
    { 0xbd7bae51, SUC_lexicon_tags_2 },
    { 0x20dbae53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x673bae55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9fbae57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17bae5a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7c1bae5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x823bae5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b5bae60, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdbae63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe99bae65, SUC_lexicon_tags_17 },
    { 0xf9fbae66, SUC_lexicon_tags_11 },
    { 0x8a7bae67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301bae71, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491bae7c, SUC_lexicon_tags_15 },
    { 0xa2dbae7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dbae82, SUC_lexicon_tags_25 },
    { 0x21bae83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5bae8d, SUC_lexicon_tags_34 },
    { 0xaf3bae8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5bae91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x507bae93, SUC_lexicon_tags_11 },
    { 0x621bae93, SUC_lexicon_tags_2 },
    { 0x897bae95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bae99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x565bae9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169baea2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97baea6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9baea9, SUC_lexicon_tags_1 },
    { 0x225baeaa, SUC_lexicon_tags_38 },
    { 0xc29baeab, SUC_lexicon_tags_0 },
    { 0x17baeaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9baeaf, SUC_lexicon_tags_12 },
    { 0xa0dbaeb0, SUC_lexicon_tags_0 },
    { 0x661baeb1, SUC_lexicon_tags_15 },
    { 0x725baeb1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebbaeb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fbaeb9, SUC_lexicon_tags_0 },
    { 0xe1baeba, SUC_lexicon_tags_25 },
    { 0xa3dbaeba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217baebf, SUC_lexicon_tags_8 },
    { 0x53fbaec0, SUC_lexicon_tags_17 },
    { 0xa7bbaebf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327baec5, SUC_lexicon_tags_32 },
    { 0x771baec6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dbaec9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bbaecc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795baed3, SUC_lexicon_tags_0 },
    { 0x9d5baed3, SUC_lexicon_tags_5 },
    { 0x8d5baed5, SUC_lexicon_tags_19 },
    { 0xdd3baed4, SUC_lexicon_tags_34 },
    { 0xc3bbaed5, SUC_lexicon_tags_12 },
    { 0xcebbaed4, SUC_lexicon_tags_22 },
    { 0xc17baed4, SUC_lexicon_tags_33 },
    { 0xf8bbaeda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34fbaedc, SUC_lexicon_tags_63 },
    { 0x2c5baedd, SUC_lexicon_tags_12 },
    { 0xf3dbaede, SUC_lexicon_tags_2 },
    { 0x60bbaedf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5baee3, SUC_lexicon_tags_14 },
    { 0x31bbaee4, SUC_lexicon_tags_17 },
    { 0xc5baee5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7baeea, SUC_lexicon_tags_25 },
    { 0xb5dbaeeb, SUC_lexicon_tags_16 },
    { 0x12dbaeec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e3baeee, SUC_lexicon_tags_28 },
    { 0x70bbaeef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579baef3, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x27fbaef5, SUC_lexicon_tags_30 },
    { 0x88fbaef6, SUC_lexicon_tags_63 },
    { 0xe61baef7, SUC_lexicon_tags_45 },
    { 0xb8bbaef5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c3baefa, SUC_lexicon_tags_31 },
    { 0x541baefa, SUC_lexicon_tags_25 },
    { 0x9fbbaefb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x885baefe, SUC_lexicon_tags_15 },
    { 0x819baeff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5baf04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9b5baf06, SUC_lexicon_tags_12 },
    { 0xbc5baf07, SUC_lexicon_tags_0 },
    { 0x697baf08, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8b1baf0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc0bbaf0c, SUC_lexicon_tags_6 },
    { 0xbdbbaf0c, SUC_lexicon_tags_8 },
    { 0x6cfbaf0e, SUC_lexicon_tags_45 },
    { 0xb5bbaf0e, SUC_lexicon_tags_11 },
    { 0xa8dbaf10, SUC_lexicon_tags_1 },
    { 0xf9bbaf0d, SUC_lexicon_tags_125 },
    { 0xf1fbaf12, SUC_lexicon_tags_21 },
    { 0xf5fbaf12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdbaf18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x611baf1a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193baf1e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x519baf20, SUC_lexicon_tags_14 },
    { 0x561baf21, SUC_lexicon_tags_37 },
    { 0x231baf22, SUC_lexicon_tags_23 },
    { 0x5e3baf23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x391baf25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dbaf28, SUC_lexicon_tags_1 },
    { 0xf19baf28, SUC_lexicon_tags_15 },
    { 0x57bbaf29, SUC_lexicon_tags_0 },
    { 0xe73baf2b, SUC_lexicon_tags_37 },
    { 0x1fbaf2c, SUC_lexicon_tags_7 },
    { 0x2ebbaf2a, SUC_lexicon_tags_7 },
    { 0x815baf28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61baf33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x305baf36, SUC_lexicon_tags_14 },
    { 0xff7baf37, SUC_lexicon_tags_2 },
    { 0x933baf38, SUC_lexicon_tags_25 },
    { 0xa2fbaf36, SUC_lexicon_tags_0 },
    { 0x675baf3a, SUC_lexicon_tags_12 },
    { 0xde1baf3a, SUC_lexicon_tags_37 },
    { 0xd6dbaf36, SUC_lexicon_tags_44 },
    { 0x259baf3d, SUC_lexicon_tags_7 },
    { 0xcb3baf3e, SUC_lexicon_tags_13 },
    { 0x953baf3e, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97baf43, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x825baf45, SUC_lexicon_tags_28 },
    { 0xd85baf46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13dbaf48, SUC_lexicon_tags_17 },
    { 0x3a1baf48, SUC_lexicon_tags_15 },
    { 0x299baf49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5e7baf4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddbaf51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bbaf56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fbaf5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb4dbaf5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9baf5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x985baf61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc05baf63, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe95baf65, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5baf6a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfbaf6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9baf71, SUC_lexicon_tags_25 },
    { 0x55dbaf72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x731baf74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213baf78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22dbaf7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85baf7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1fbaf80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3fbaf82, SUC_lexicon_tags_47 },
    { 0xfbaf82, SUC_lexicon_tags_10 },
    { 0xffbbaf83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x881baf86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bbaf89, SUC_lexicon_tags_37 },
    { 0xc13baf89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdadbaf8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29baf92, SUC_lexicon_tags_2 },
    { 0xcb5baf93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcffbaf95, SUC_lexicon_tags_17 },
    { 0x939baf95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1baf99, SUC_lexicon_tags_60 },
    { 0xdc3baf9a, SUC_lexicon_tags_11 },
    { 0x53bbaf9b, SUC_lexicon_tags_8 },
    { 0x5edbaf9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bbafa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7bafa2, SUC_lexicon_tags_65 },
    { 0xc3bbafa2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaf3bafa5, SUC_lexicon_tags_277 },
    { 0, NULL },
    { 0, NULL },
    { 0x65bbafa8, SUC_lexicon_tags_11 },
    { 0x4bfbafa9, SUC_lexicon_tags_43 },
    { 0x6d5bafaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dbafaf, SUC_lexicon_tags_78 },
    { 0xf6dbafb0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bbafb4, SUC_lexicon_tags_33 },
    { 0x8fdbafb5, SUC_lexicon_tags_23 },
    { 0xc21bafb4, SUC_lexicon_tags_77 },
    { 0xa61bafb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5bafc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63fbafc4, SUC_lexicon_tags_17 },
    { 0x9a5bafc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7efbafc7, SUC_lexicon_tags_90 },
    { 0xd09bafc8, SUC_lexicon_tags_38 },
    { 0x753bafc8, SUC_lexicon_tags_16 },
    { 0xb27bafc8, SUC_lexicon_tags_11 },
    { 0x7abbafc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4bbafcd, SUC_lexicon_tags_77 },
    { 0x757bafce, SUC_lexicon_tags_37 },
    { 0x85bbafcf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fbafd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299bafd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3bafd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183bafdd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x73bbafdf, SUC_lexicon_tags_42 },
    { 0xe11bafdf, SUC_lexicon_tags_45 },
    { 0x755bafe1, SUC_lexicon_tags_27 },
    { 0xf57bafdf, SUC_lexicon_tags_19 },
    { 0x413bafdf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915bafe7, SUC_lexicon_tags_26 },
    { 0xf39bafe8, SUC_lexicon_tags_12 },
    { 0x203bafe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347baff3, SUC_lexicon_tags_21 },
    { 0x777baff4, SUC_lexicon_tags_17 },
    { 0x645baff5, SUC_lexicon_tags_11 },
    { 0xf51baff6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5baffd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x869bb000, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x769bb003, SUC_lexicon_tags_12 },
    { 0xf47bb004, SUC_lexicon_tags_14 },
    { 0x7d9bb003, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59bb00c, SUC_lexicon_tags_10 },
    { 0x205bb00c, SUC_lexicon_tags_7 },
    { 0x9a9bb00e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89bb016, SUC_lexicon_tags_37 },
    { 0x63fbb017, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf4fbb019, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fbb01e, SUC_lexicon_tags_7 },
    { 0xf77bb01f, SUC_lexicon_tags_23 },
    { 0x8fdbb020, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x749bb022, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fbb02b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfbb02e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889bb032, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97bbb034, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7bb038, SUC_lexicon_tags_49 },
    { 0x7ddbb038, SUC_lexicon_tags_2 },
    { 0x3bdbb03a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebbb03e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x489bb041, SUC_lexicon_tags_1 },
    { 0x15bb041, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bbb048, SUC_lexicon_tags_17 },
    { 0x8b3bb049, SUC_lexicon_tags_31 },
    { 0xfdbbb04a, SUC_lexicon_tags_7 },
    { 0x1c1bb04a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x55fbb04d, SUC_lexicon_tags_13 },
    { 0x5dfbb04e, SUC_lexicon_tags_1 },
    { 0xcbbbb04f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3dbb051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa65bb053, SUC_lexicon_tags_29 },
    { 0x383bb053, SUC_lexicon_tags_11 },
    { 0x219bb054, SUC_lexicon_tags_0 },
    { 0x4b1bb056, SUC_lexicon_tags_37 },
    { 0xa8dbb057, SUC_lexicon_tags_11 },
    { 0xdb3bb058, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41bb05d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdbb062, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xaf3bb064, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493bb068, SUC_lexicon_tags_40 },
    { 0xaa1bb069, SUC_lexicon_tags_27 },
    { 0xd35bb06a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3bb070, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25bb076, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613bb07a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7bb080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fbb087, SUC_lexicon_tags_16 },
    { 0xb31bb088, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebbb08d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bb090, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8a5bb092, SUC_lexicon_tags_15 },
    { 0xd73bb093, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dbb097, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x851bb099, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bbb0a1, SUC_lexicon_tags_17 },
    { 0xcdbbb0a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9bb0a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201bb0a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xddbb0ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fbb0af, SUC_lexicon_tags_1 },
    { 0xdd7bb0b0, SUC_lexicon_tags_25 },
    { 0xd97bb0b0, SUC_lexicon_tags_47 },
    { 0x117bb0af, SUC_lexicon_tags_25 },
    { 0xf47bb0b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaafbb0b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddbb0b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39bb0bc, SUC_lexicon_tags_25 },
    { 0x7bbb0bc, SUC_lexicon_tags_23 },
    { 0x885bb0bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5bb0c3, SUC_lexicon_tags_11 },
    { 0xb5dbb0c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e7bb0c6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67bb0cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1a1bb0cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85dbb0cf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dbb0d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x24fbb0d5, SUC_lexicon_tags_25 },
    { 0x4e1bb0d5, SUC_lexicon_tags_56 },
    { 0x5dfbb0d7, SUC_lexicon_tags_23 },
    { 0xefbb0d5, SUC_lexicon_tags_27 },
    { 0x5dbb0d6, SUC_lexicon_tags_11 },
    { 0xb57bb0da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9bb0de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23bb0e4, SUC_lexicon_tags_33 },
    { 0x5e1bb0e5, SUC_lexicon_tags_2 },
    { 0xad9bb0e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1bb0ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171bb0f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3bb0f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5bb0f9, SUC_lexicon_tags_0 },
    { 0xbadbb0fa, SUC_lexicon_tags_37 },
    { 0x7a1bb0fb, SUC_lexicon_tags_29 },
    { 0x8d1bb0fb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x5c9bb0fe, SUC_lexicon_tags_1 },
    { 0x1fdbb0fe, SUC_lexicon_tags_67 },
    { 0xd79bb0ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35bb103, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9bb106, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebbb10b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3bb10e, SUC_lexicon_tags_1 },
    { 0x6b3bb10f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefbb116, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463bb11c, SUC_lexicon_tags_39 },
    { 0x1a5bb11c, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5bb122, SUC_lexicon_tags_37 },
    { 0x6e9bb122, SUC_lexicon_tags_37 },
    { 0x7bdbb124, SUC_lexicon_tags_25 },
    { 0x4d1bb124, SUC_lexicon_tags_63 },
    { 0x53bb126, SUC_lexicon_tags_2 },
    { 0xacdbb127, SUC_lexicon_tags_15 },
    { 0x3d9bb128, SUC_lexicon_tags_49 },
    { 0xaadbb122, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x191bb12c, SUC_lexicon_tags_17 },
    { 0x239bb12d, SUC_lexicon_tags_2 },
    { 0xf81bb12c, SUC_lexicon_tags_11 },
    { 0x17bbb12f, SUC_lexicon_tags_0 },
    { 0xeefbb12f, SUC_lexicon_tags_37 },
    { 0x6d7bb131, SUC_lexicon_tags_24 },
    { 0x90bbb132, SUC_lexicon_tags_33 },
    { 0xe1dbb130, SUC_lexicon_tags_11 },
    { 0x28bbb132, SUC_lexicon_tags_22 },
    { 0x65fbb12e, SUC_lexicon_tags_31 },
    { 0x4e1bb136, SUC_lexicon_tags_7 },
    { 0x9d9bb133, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89fbb139, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edbb13d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7bb141, SUC_lexicon_tags_45 },
    { 0xa3bbb141, SUC_lexicon_tags_10 },
    { 0x8d9bb143, SUC_lexicon_tags_7 },
    { 0x64dbb142, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bb148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7bb14d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbbb150, SUC_lexicon_tags_6 },
    { 0xc69bb151, SUC_lexicon_tags_32 },
    { 0x309bb151, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26dbb154, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dbb159, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5dbbb15b, SUC_lexicon_tags_1 },
    { 0xf03bb15c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbbb15f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b7bb161, SUC_lexicon_tags_25 },
    { 0xfffbb162, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e1bb164, SUC_lexicon_tags_11 },
    { 0x80fbb165, SUC_lexicon_tags_37 },
    { 0xa8bbb166, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x97dbb169, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5bb16c, SUC_lexicon_tags_23 },
    { 0x9e7bb16d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67bb170, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x755bb173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdbb177, SUC_lexicon_tags_7 },
    { 0x2bdbb177, SUC_lexicon_tags_12 },
    { 0xf43bb178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97bb17c, SUC_lexicon_tags_2 },
    { 0x9cbbb17c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x11fbb17f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x51dbb181, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebbb187, SUC_lexicon_tags_37 },
    { 0xddbb188, SUC_lexicon_tags_87 },
    { 0xd05bb189, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x671bb18b, SUC_lexicon_tags_31 },
    { 0xa15bb18b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bbb190, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413bb195, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dbb198, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d7bb19a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x951bb19d, SUC_lexicon_tags_15 },
    { 0xe31bb19e, SUC_lexicon_tags_14 },
    { 0x271bb19f, SUC_lexicon_tags_37 },
    { 0x459bb1a0, SUC_lexicon_tags_28 },
    { 0xf85bb1a0, SUC_lexicon_tags_1 },
    { 0xc6dbb1a2, SUC_lexicon_tags_6 },
    { 0xa5dbb1a2, SUC_lexicon_tags_15 },
    { 0x9dfbb19f, SUC_lexicon_tags_62 },
    { 0x6a9bb1a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fbb1ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x375bb1ae, SUC_lexicon_tags_25 },
    { 0xc9bb1ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x527bb1b2, SUC_lexicon_tags_48 },
    { 0xeadbb1b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865bb1b7, SUC_lexicon_tags_1 },
    { 0x2c7bb1b8, SUC_lexicon_tags_63 },
    { 0xa1bb1b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x519bb1bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bbb1c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bbb1cb, SUC_lexicon_tags_25 },
    { 0x813bb1cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fbb1d0, SUC_lexicon_tags_65 },
    { 0x7ffbb1d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1fbb1d3, SUC_lexicon_tags_31 },
    { 0x821bb1d3, SUC_lexicon_tags_19 },
    { 0xb25bb1d5, SUC_lexicon_tags_33 },
    { 0x9dbbb1d6, SUC_lexicon_tags_31 },
    { 0x9abbb1d6, SUC_lexicon_tags_1 },
    { 0xe67bb1d8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825bb1dc, SUC_lexicon_tags_8 },
    { 0xeabbb1dc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x445bb1df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe9bb1e1, SUC_lexicon_tags_15 },
    { 0x7d9bb1e1, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x52fbb1e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fbb1e7, SUC_lexicon_tags_14 },
    { 0xfcfbb1e7, SUC_lexicon_tags_11 },
    { 0x16bbb1e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad9bb1eb, SUC_lexicon_tags_37 },
    { 0xd99bb1ec, SUC_lexicon_tags_12 },
    { 0xf33bb1ec, SUC_lexicon_tags_15 },
    { 0x28fbb1ee, SUC_lexicon_tags_23 },
    { 0xf23bb1ef, SUC_lexicon_tags_12 },
    { 0x2bfbb1ee, SUC_lexicon_tags_84 },
    { 0x679bb1f1, SUC_lexicon_tags_32 },
    { 0x193bb1f2, SUC_lexicon_tags_28 },
    { 0x69fbb1ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9bb1f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9bb1fc, SUC_lexicon_tags_6 },
    { 0x8f5bb1fd, SUC_lexicon_tags_37 },
    { 0xdebbb1fe, SUC_lexicon_tags_0 },
    { 0xd5fbb1fc, SUC_lexicon_tags_26 },
    { 0x7fbb200, SUC_lexicon_tags_31 },
    { 0x54dbb201, SUC_lexicon_tags_13 },
    { 0x3dbbb1fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edbb208, SUC_lexicon_tags_37 },
    { 0x1c9bb209, SUC_lexicon_tags_2 },
    { 0xf37bb20a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403bb20e, SUC_lexicon_tags_38 },
    { 0xc89bb20f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bbb218, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe7bbb21a, SUC_lexicon_tags_1 },
    { 0xc4fbb21b, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dbb21f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb87bb221, SUC_lexicon_tags_15 },
    { 0xe67bb222, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fbb225, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83bb227, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dbb22c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dbb230, SUC_lexicon_tags_27 },
    { 0x45fbb230, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bb239, SUC_lexicon_tags_15 },
    { 0xe05bb239, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35bb23f, SUC_lexicon_tags_7 },
    { 0x43fbb23f, SUC_lexicon_tags_11 },
    { 0x215bb241, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbffbb243, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3bb24c, SUC_lexicon_tags_25 },
    { 0xf6fbb24d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x45bb24f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5dbb251, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61bb256, SUC_lexicon_tags_15 },
    { 0x151bb256, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc5bb259, SUC_lexicon_tags_2 },
    { 0xac5bb259, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9bb25e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x567bb261, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5bb264, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47bb269, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1bb26f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bbb274, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x143bb276, SUC_lexicon_tags_43 },
    { 0xb5bbb276, SUC_lexicon_tags_32 },
    { 0x4b1bb278, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bbb27d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x731bb280, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb07bb282, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7bb288, SUC_lexicon_tags_60 },
    { 0x693bb289, SUC_lexicon_tags_49 },
    { 0x223bb288, SUC_lexicon_tags_56 },
    { 0x341bb28b, SUC_lexicon_tags_37 },
    { 0x73dbb288, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31bb28f, SUC_lexicon_tags_1 },
    { 0x3f5bb290, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbbb298, SUC_lexicon_tags_47 },
    { 0x35bbb299, SUC_lexicon_tags_25 },
    { 0x875bb299, SUC_lexicon_tags_33 },
    { 0xfc5bb29b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1bb29f, SUC_lexicon_tags_65 },
    { 0x4e3bb29f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabbb2a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1bb2a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03bb2ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbb2b0, SUC_lexicon_tags_31 },
    { 0x26bbb2b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adbb2b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xddbb2b8, SUC_lexicon_tags_43 },
    { 0x9e7bb2b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x795bb2bb, SUC_lexicon_tags_16 },
    { 0xf2bbb2bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5bb2c0, SUC_lexicon_tags_12 },
    { 0x31fbb2c1, SUC_lexicon_tags_45 },
    { 0xdf3bb2c1, SUC_lexicon_tags_68 },
    { 0x1a1bb2c3, SUC_lexicon_tags_7 },
    { 0x909bb2c4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x93bb2c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57bb2c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae5bb2cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf09bb2cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebbb2d0, SUC_lexicon_tags_70 },
    { 0x371bb2d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7bb2d4, SUC_lexicon_tags_48 },
    { 0x1d7bb2d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe49bb2d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37dbb2d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x503bb2dc, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fbb2df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827bb2e4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x685bb2e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x80fbb2e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9bb2eb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fbb2ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc87bb2f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x857bb2f3, SUC_lexicon_tags_37 },
    { 0x55bb2f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3bb2fe, SUC_lexicon_tags_46 },
    { 0x1f9bb2ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75bb302, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x905bb304, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3bb308, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdbb30d, SUC_lexicon_tags_37 },
    { 0x18dbb30e, SUC_lexicon_tags_9 },
    { 0x1f7bb30f, SUC_lexicon_tags_17 },
    { 0x90fbb310, SUC_lexicon_tags_33 },
    { 0x443bb311, SUC_lexicon_tags_48 },
    { 0x243bb311, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115bb31c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5bb321, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79bb326, SUC_lexicon_tags_23 },
    { 0x57fbb327, SUC_lexicon_tags_11 },
    { 0xca5bb328, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8c9bb32a, SUC_lexicon_tags_16 },
    { 0xa5bbb32b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03bb332, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bb335, SUC_lexicon_tags_37 },
    { 0xd69bb336, SUC_lexicon_tags_11 },
    { 0xc61bb336, SUC_lexicon_tags_19 },
    { 0x2c1bb338, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x259bb33b, SUC_lexicon_tags_11 },
    { 0xc1fbb33b, SUC_lexicon_tags_25 },
    { 0xd5bbb33d, SUC_lexicon_tags_31 },
    { 0x691bb33c, SUC_lexicon_tags_12 },
    { 0x62dbb33f, SUC_lexicon_tags_19 },
    { 0x685bb340, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5fbb342, SUC_lexicon_tags_2 },
    { 0xebdbb343, SUC_lexicon_tags_25 },
    { 0x125bb344, SUC_lexicon_tags_33 },
    { 0x455bb345, SUC_lexicon_tags_1 },
    { 0xc8dbb346, SUC_lexicon_tags_1 },
    { 0xf4bbb345, SUC_lexicon_tags_39 },
    { 0x17bbb348, SUC_lexicon_tags_49 },
    { 0xf5bbb346, SUC_lexicon_tags_37 },
    { 0x751bb347, SUC_lexicon_tags_57 },
    { 0x4d9bb34a, SUC_lexicon_tags_45 },
    { 0xa57bb347, SUC_lexicon_tags_29 },
    { 0x1f7bb34d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbbb350, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x255bb353, SUC_lexicon_tags_39 },
    { 0x7d1bb354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97bb356, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x58fbb358, SUC_lexicon_tags_45 },
    { 0x75bb359, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebbb35e, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x2edbb360, SUC_lexicon_tags_15 },
    { 0xa3bbb360, SUC_lexicon_tags_87 },
    { 0x50bbb362, SUC_lexicon_tags_1 },
    { 0x8d5bb362, SUC_lexicon_tags_15 },
    { 0xfebbb363, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fbb369, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x54dbb36b, SUC_lexicon_tags_14 },
    { 0x71bbb36c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcc5bb36e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b9bb370, SUC_lexicon_tags_0 },
    { 0x83dbb371, SUC_lexicon_tags_612 },
    { 0x957bb370, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1bb378, SUC_lexicon_tags_165 },
    { 0x30bbb379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa5bb37b, SUC_lexicon_tags_43 },
    { 0xbbdbb37c, SUC_lexicon_tags_1 },
    { 0xadbbb37b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdf7bb37f, SUC_lexicon_tags_2 },
    { 0x5cdbb380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdefbb382, SUC_lexicon_tags_7 },
    { 0x125bb383, SUC_lexicon_tags_49 },
    { 0x85bb384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1bb388, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd17bb38a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7bb38d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93bb392, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9bb395, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5bb39f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7bb3a5, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edbb3ab, SUC_lexicon_tags_37 },
    { 0xf55bb3ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fbb3b0, SUC_lexicon_tags_26 },
    { 0x7efbb3b0, SUC_lexicon_tags_2 },
    { 0x853bb3b2, SUC_lexicon_tags_4 },
    { 0x3e1bb3b3, SUC_lexicon_tags_65 },
    { 0xf43bb3b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adbb3b9, SUC_lexicon_tags_28 },
    { 0x47dbb3ba, SUC_lexicon_tags_23 },
    { 0xc77bb3bb, SUC_lexicon_tags_27 },
    { 0x2dbbb3bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf05bb3be, SUC_lexicon_tags_11 },
    { 0xd5bb3be, SUC_lexicon_tags_27 },
    { 0xd15bb3c0, SUC_lexicon_tags_124 },
    { 0xcfbb3bf, SUC_lexicon_tags_1 },
    { 0x785bb3bf, SUC_lexicon_tags_32 },
    { 0x9a3bb3c3, SUC_lexicon_tags_18 },
    { 0xbcdbb3c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea1bb3c6, SUC_lexicon_tags_34 },
    { 0xc4fbb3c6, SUC_lexicon_tags_21 },
    { 0x5bdbb3c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1bb3d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1bb3d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599bb3da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9bb3e1, SUC_lexicon_tags_1 },
    { 0x789bb3e1, SUC_lexicon_tags_17 },
    { 0x39fbb3e3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121bb3e8, SUC_lexicon_tags_19 },
    { 0x903bb3e8, SUC_lexicon_tags_25 },
    { 0x383bb3ea, SUC_lexicon_tags_17 },
    { 0x5c5bb3eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdbb3f0, SUC_lexicon_tags_17 },
    { 0x25bbb3f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdbb3f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadbb3f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769bb400, SUC_lexicon_tags_4 },
    { 0xf45bb401, SUC_lexicon_tags_12 },
    { 0x7dbb401, SUC_lexicon_tags_37 },
    { 0x207bb403, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fbb409, SUC_lexicon_tags_30 },
    { 0x12fbb40a, SUC_lexicon_tags_2 },
    { 0x30dbb409, SUC_lexicon_tags_7 },
    { 0x653bb40c, SUC_lexicon_tags_21 },
    { 0xf37bb40d, SUC_lexicon_tags_19 },
    { 0xa9bb40a, SUC_lexicon_tags_1 },
    { 0xc11bb40f, SUC_lexicon_tags_12 },
    { 0xe9fbb40f, SUC_lexicon_tags_37 },
    { 0xb9dbb411, SUC_lexicon_tags_12 },
    { 0xa41bb412, SUC_lexicon_tags_0 },
    { 0xc0bbb410, SUC_lexicon_tags_1 },
    { 0x5d5bb414, SUC_lexicon_tags_8 },
    { 0xeddbb40a, SUC_lexicon_tags_23 },
    { 0xaddbb416, SUC_lexicon_tags_12 },
    { 0xf7bbb40a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd57bb419, SUC_lexicon_tags_6 },
    { 0x8d1bb41a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4d5bb41c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x461bb41f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa99bb421, SUC_lexicon_tags_37 },
    { 0x99bbb422, SUC_lexicon_tags_7 },
    { 0xef7bb423, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dbb429, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc15bb42b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1bb42e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dbb432, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331bb43a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261bb440, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x47bb442, SUC_lexicon_tags_14 },
    { 0x755bb443, SUC_lexicon_tags_9 },
    { 0xb47bb442, SUC_lexicon_tags_55 },
    { 0x943bb445, SUC_lexicon_tags_17 },
    { 0x211bb446, SUC_lexicon_tags_37 },
    { 0x577bb447, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73bbb449, SUC_lexicon_tags_19 },
    { 0xc5bbb44a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdbb44e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5bb459, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9bbb45b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x2ebbb45d, SUC_lexicon_tags_29 },
    { 0xebbb45d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfbb461, SUC_lexicon_tags_12 },
    { 0x669bb461, SUC_lexicon_tags_57 },
    { 0x30bbb463, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1bb468, SUC_lexicon_tags_11 },
    { 0x281bb468, SUC_lexicon_tags_17 },
    { 0xab5bb468, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381bb46e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bbb47a, SUC_lexicon_tags_38 },
    { 0xf07bb47a, SUC_lexicon_tags_17 },
    { 0xabdbb47c, SUC_lexicon_tags_43 },
    { 0xaf1bb47a, SUC_lexicon_tags_17 },
    { 0x3e3bb47c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bbb483, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedbb487, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbbb493, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcffbb495, SUC_lexicon_tags_1 },
    { 0xca3bb496, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x689bb498, SUC_lexicon_tags_38 },
    { 0x999bb499, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9bb49c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x505bb49e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3bb4a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93bb4ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fbb4b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7bb4b9, SUC_lexicon_tags_57 },
    { 0x6afbb4b9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5d1bb4bc, SUC_lexicon_tags_1 },
    { 0x7bfbb4bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297bb4c3, SUC_lexicon_tags_1 },
    { 0x869bb4c4, SUC_lexicon_tags_8 },
    { 0x3cbbb4c3, SUC_lexicon_tags_66 },
    { 0xcddbb4c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52dbb4c9, SUC_lexicon_tags_37 },
    { 0x5c5bb4ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x667bb4cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1bb4d0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xde5bb4d2, SUC_lexicon_tags_32 },
    { 0xec3bb4d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd37bb4d5, SUC_lexicon_tags_82 },
    { 0xe67bb4d5, SUC_lexicon_tags_38 },
    { 0xba7bb4d7, SUC_lexicon_tags_30 },
    { 0x811bb4d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845bb4e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1ebbb4e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddbb4e7, SUC_lexicon_tags_10 },
    { 0x2a1bb4e8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xcddbb4ea, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x66fbb4ec, SUC_lexicon_tags_14 },
    { 0x6d3bb4ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f1bb4ef, SUC_lexicon_tags_38 },
    { 0x6f3bb4f0, SUC_lexicon_tags_15 },
    { 0xd61bb4f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645bb4f9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x9e1bb4fb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf9bbb4fd, SUC_lexicon_tags_37 },
    { 0x335bb4fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5bb502, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x83fbb504, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efbb50c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x863bb50e, SUC_lexicon_tags_23 },
    { 0x5fdbb50e, SUC_lexicon_tags_26 },
    { 0x9bb510, SUC_lexicon_tags_6 },
    { 0xe77bb511, SUC_lexicon_tags_4 },
    { 0x863bb510, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1bb516, SUC_lexicon_tags_66 },
    { 0xe01bb516, SUC_lexicon_tags_12 },
    { 0x747bb517, SUC_lexicon_tags_33 },
    { 0x407bb519, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01bb522, SUC_lexicon_tags_30 },
    { 0x797bb523, SUC_lexicon_tags_28 },
    { 0x55bbb522, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdbb52a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfbb52e, SUC_lexicon_tags_21 },
    { 0x449bb52f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771bb536, SUC_lexicon_tags_12 },
    { 0xf53bb537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e7bb539, SUC_lexicon_tags_47 },
    { 0xd09bb539, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5bb53f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc9bb541, SUC_lexicon_tags_55 },
    { 0x69bb542, SUC_lexicon_tags_21 },
    { 0x5d5bb543, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21bb546, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xafdbb548, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d1bb54a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381bb54e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99fbb551, SUC_lexicon_tags_0 },
    { 0x233bb552, SUC_lexicon_tags_27 },
    { 0xc31bb552, SUC_lexicon_tags_12 },
    { 0x92bbb554, SUC_lexicon_tags_12 },
    { 0x32dbb553, SUC_lexicon_tags_1 },
    { 0xacdbb556, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefbb558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbbb55b, SUC_lexicon_tags_118 },
    { 0x351bb55b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbbb564, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bbb567, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xec3bb56a, SUC_lexicon_tags_14 },
    { 0xe63bb56b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7bb570, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x87fbb573, SUC_lexicon_tags_38 },
    { 0xe0dbb573, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33bb578, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x403bb57a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdc9bb57c, SUC_lexicon_tags_27 },
    { 0xe87bb57d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7bb584, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fbb588, SUC_lexicon_tags_7 },
    { 0x703bb589, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2bbb58b, SUC_lexicon_tags_25 },
    { 0xd51bb58b, SUC_lexicon_tags_15 },
    { 0x96fbb58d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfbb590, SUC_lexicon_tags_11 },
    { 0x4cfbb591, SUC_lexicon_tags_27 },
    { 0x339bb592, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5bb595, SUC_lexicon_tags_46 },
    { 0x4b9bb596, SUC_lexicon_tags_33 },
    { 0xd5bbb597, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45bb59a, SUC_lexicon_tags_8 },
    { 0x10fbb59a, SUC_lexicon_tags_0 },
    { 0x285bb59a, SUC_lexicon_tags_63 },
    { 0x65fbb59d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5bb5a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fbb5b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x981bb5b4, SUC_lexicon_tags_17 },
    { 0x763bb5b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbbb5ba, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x823bb5bd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdbb5c0, SUC_lexicon_tags_15 },
    { 0xc8fbb5c1, SUC_lexicon_tags_38 },
    { 0x56dbb5c0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x301bb5c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x771bb5c7, SUC_lexicon_tags_9 },
    { 0x7d5bb5c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61bb5cf, SUC_lexicon_tags_198 },
    { 0x9f5bb5d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xba7bb5d3, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbb5d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bbb5df, SUC_lexicon_tags_25 },
    { 0xd31bb5df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x55dbb5e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fbb5ee, SUC_lexicon_tags_58 },
    { 0xca3bb5ef, SUC_lexicon_tags_38 },
    { 0xd0bbb5f0, SUC_lexicon_tags_1 },
    { 0xdfdbb5f1, SUC_lexicon_tags_45 },
    { 0xbe5bb5f1, SUC_lexicon_tags_68 },
    { 0x1ebbb5f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfbb5f9, SUC_lexicon_tags_17 },
    { 0x80fbb5fa, SUC_lexicon_tags_47 },
    { 0x437bb5fb, SUC_lexicon_tags_11 },
    { 0xf59bb5f9, SUC_lexicon_tags_46 },
    { 0xfdbb5fb, SUC_lexicon_tags_0 },
    { 0x75dbb5fc, SUC_lexicon_tags_49 },
    { 0x6a1bb5fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x305bb601, SUC_lexicon_tags_23 },
    { 0x85bb602, SUC_lexicon_tags_0 },
    { 0x36bbb602, SUC_lexicon_tags_32 },
    { 0xbbdbb604, SUC_lexicon_tags_57 },
    { 0xa0fbb605, SUC_lexicon_tags_25 },
    { 0x2a7bb603, SUC_lexicon_tags_10 },
    { 0x4f7bb605, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x96dbb609, SUC_lexicon_tags_25 },
    { 0xd87bb60a, SUC_lexicon_tags_33 },
    { 0xa93bb60b, SUC_lexicon_tags_25 },
    { 0x1c5bb60c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfb9bb60e, SUC_lexicon_tags_12 },
    { 0x183bb60f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13bb616, SUC_lexicon_tags_2 },
    { 0xc5bbb617, SUC_lexicon_tags_37 },
    { 0xfb5bb618, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x51bb61a, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135bb61f, SUC_lexicon_tags_11 },
    { 0x1b3bb61f, SUC_lexicon_tags_60 },
    { 0xfc1bb621, SUC_lexicon_tags_37 },
    { 0xb3dbb620, SUC_lexicon_tags_57 },
    { 0x543bb621, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bbb627, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6cdbb629, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adbb62d, SUC_lexicon_tags_11 },
    { 0xe27bb62d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4bbb630, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c9bb632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xffbb634, SUC_lexicon_tags_17 },
    { 0x129bb635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddbb63b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bbb63f, SUC_lexicon_tags_12 },
    { 0x163bb640, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3bb647, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x17fbb649, SUC_lexicon_tags_19 },
    { 0x571bb64a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x703bb64d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271bb652, SUC_lexicon_tags_7 },
    { 0xa33bb653, SUC_lexicon_tags_45 },
    { 0x3f1bb654, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x75bb656, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fbb65a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bbb65d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3bb663, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fbb666, SUC_lexicon_tags_2 },
    { 0x147bb667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x41dbb669, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dbb66b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21bb66e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb6dbb670, SUC_lexicon_tags_12 },
    { 0xbd1bb671, SUC_lexicon_tags_37 },
    { 0x767bb672, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x517bb675, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64dbb67b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bbb67e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9bb680, SUC_lexicon_tags_7 },
    { 0xd37bb681, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bb684, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bbb687, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x725bb689, SUC_lexicon_tags_60 },
    { 0xcf5bb68a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75bb68d, SUC_lexicon_tags_46 },
    { 0x62dbb68d, SUC_lexicon_tags_7 },
    { 0x789bb68f, SUC_lexicon_tags_37 },
    { 0x709bb68f, SUC_lexicon_tags_37 },
    { 0x1b3bb691, SUC_lexicon_tags_15 },
    { 0x1e7bb691, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05bb696, SUC_lexicon_tags_32 },
    { 0x62bbb697, SUC_lexicon_tags_7 },
    { 0xff3bb698, SUC_lexicon_tags_1 },
    { 0x817bb698, SUC_lexicon_tags_2 },
    { 0x835bb69a, SUC_lexicon_tags_15 },
    { 0xedfbb69a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39bb6a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x863bb6a3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1bb6a7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x917bb6aa, SUC_lexicon_tags_43 },
    { 0x62fbb6ab, SUC_lexicon_tags_12 },
    { 0x1c5bb6ab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afbb6af, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163bb6b3, SUC_lexicon_tags_11 },
    { 0x947bb6b4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2a3bb6b6, SUC_lexicon_tags_14 },
    { 0xc2fbb6b7, SUC_lexicon_tags_12 },
    { 0xf27bb6b6, SUC_lexicon_tags_11 },
    { 0xd95bb6b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbbb6bc, SUC_lexicon_tags_27 },
    { 0x5b9bb6bd, SUC_lexicon_tags_47 },
    { 0xb59bb6bd, SUC_lexicon_tags_68 },
    { 0xdf7bb6bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc57bb6c1, SUC_lexicon_tags_1 },
    { 0x667bb6c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3bb6c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afbb6c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2edbb6cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdbb6d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3bb6d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bb6d7, SUC_lexicon_tags_15 },
    { 0xc59bb6d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83bb6dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x387bb6df, SUC_lexicon_tags_9 },
    { 0x719bb6e0, SUC_lexicon_tags_45 },
    { 0xd73bb6e0, SUC_lexicon_tags_17 },
    { 0x661bb6e2, SUC_lexicon_tags_0 },
    { 0xb75bb6e3, SUC_lexicon_tags_9 },
    { 0xf4bbb6e1, SUC_lexicon_tags_12 },
    { 0x28fbb6e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dbb6f2, SUC_lexicon_tags_19 },
    { 0xcf1bb6f3, SUC_lexicon_tags_16 },
    { 0xe3bb6f4, SUC_lexicon_tags_27 },
    { 0x6abbb6f4, SUC_lexicon_tags_54 },
    { 0xcdbb6f6, SUC_lexicon_tags_17 },
    { 0x2bbb6f7, SUC_lexicon_tags_2 },
    { 0xecfbb6f8, SUC_lexicon_tags_17 },
    { 0xab9bb6f8, SUC_lexicon_tags_55 },
    { 0xa3dbb6fa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921bb700, SUC_lexicon_tags_8 },
    { 0xf9bbb700, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39bb704, SUC_lexicon_tags_17 },
    { 0xcb5bb705, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c9bb707, SUC_lexicon_tags_16 },
    { 0x98dbb707, SUC_lexicon_tags_25 },
    { 0xd7bb707, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fbb70c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3bb710, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51bb717, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49bb71d, SUC_lexicon_tags_17 },
    { 0x6dfbb71e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fbb725, SUC_lexicon_tags_79 },
    { 0x85dbb726, SUC_lexicon_tags_87 },
    { 0x74dbb727, SUC_lexicon_tags_63 },
    { 0x531bb726, SUC_lexicon_tags_47 },
    { 0xb31bb729, SUC_lexicon_tags_33 },
    { 0xa8bbb725, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7bb72e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x269bb730, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5bb733, SUC_lexicon_tags_30 },
    { 0x853bb733, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdbb73b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7bb73e, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fbb744, SUC_lexicon_tags_87 },
    { 0xfebbb744, SUC_lexicon_tags_7 },
    { 0x3a1bb745, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xeadbb749, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7bb752, SUC_lexicon_tags_8 },
    { 0x9cdbb753, SUC_lexicon_tags_4 },
    { 0xac9bb753, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bbb757, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb65bb759, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdbb75e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751bb765, SUC_lexicon_tags_45 },
    { 0x601bb766, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x76fbb768, SUC_lexicon_tags_25 },
    { 0x2d3bb768, SUC_lexicon_tags_12 },
    { 0x1a9bb76a, SUC_lexicon_tags_63 },
    { 0xc4bbb76a, SUC_lexicon_tags_55 },
    { 0xfcdbb76c, SUC_lexicon_tags_1 },
    { 0x40fbb76d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e1bb76f, SUC_lexicon_tags_15 },
    { 0x84fbb76f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81dbb772, SUC_lexicon_tags_40 },
    { 0xebbbb773, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1bbb775, SUC_lexicon_tags_1 },
    { 0x3afbb776, SUC_lexicon_tags_33 },
    { 0x829bb776, SUC_lexicon_tags_63 },
    { 0xff1bb775, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x137bb77a, SUC_lexicon_tags_17 },
    { 0x623bb77b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51bb786, SUC_lexicon_tags_1 },
    { 0x5c3bb786, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc31bb789, SUC_lexicon_tags_26 },
    { 0x8b5bb789, SUC_lexicon_tags_2 },
    { 0x6bbbb78b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fbb792, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddbb795, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f9bb797, SUC_lexicon_tags_25 },
    { 0xbbb797, SUC_lexicon_tags_63 },
    { 0xb77bb799, SUC_lexicon_tags_11 },
    { 0x58fbb799, SUC_lexicon_tags_23 },
    { 0x95bbb799, SUC_lexicon_tags_1 },
    { 0x271bb79a, SUC_lexicon_tags_66 },
    { 0x8c9bb79d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc97bb79f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7bb7a2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421bb7a9, SUC_lexicon_tags_68 },
    { 0x5d7bb7aa, SUC_lexicon_tags_15 },
    { 0xfd3bb7aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9d3bb7ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfbb7c0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bb7c3, SUC_lexicon_tags_14 },
    { 0x48bbb7c4, SUC_lexicon_tags_8 },
    { 0xfa9bb7c4, SUC_lexicon_tags_12 },
    { 0xf41bb7c3, SUC_lexicon_tags_37 },
    { 0x695bb7c7, SUC_lexicon_tags_37 },
    { 0x48fbb7c8, SUC_lexicon_tags_49 },
    { 0x959bb7c7, SUC_lexicon_tags_47 },
    { 0xa99bb7ca, SUC_lexicon_tags_17 },
    { 0xa89bb7cb, SUC_lexicon_tags_17 },
    { 0x78dbb7cc, SUC_lexicon_tags_46 },
    { 0xddbb7cc, SUC_lexicon_tags_28 },
    { 0xcf5bb7ce, SUC_lexicon_tags_25 },
    { 0xaf9bb7cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fbb7d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bbb7d8, SUC_lexicon_tags_25 },
    { 0xe05bb7d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129bb7df, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5bb7e8, SUC_lexicon_tags_60 },
    { 0xb57bb7e9, SUC_lexicon_tags_34 },
    { 0x443bb7e9, SUC_lexicon_tags_2 },
    { 0xc3fbb7e9, SUC_lexicon_tags_11 },
    { 0xe9dbb7e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9bb7ef, SUC_lexicon_tags_17 },
    { 0x3dbb7f0, SUC_lexicon_tags_7 },
    { 0x75dbb7f0, SUC_lexicon_tags_19 },
    { 0xce5bb7f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b9bb7f4, SUC_lexicon_tags_37 },
    { 0x387bb7f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dbb7f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35bb7fc, SUC_lexicon_tags_11 },
    { 0x5e5bb7fc, SUC_lexicon_tags_27 },
    { 0xcf5bb7fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211bb809, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefbb80d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60bbb80f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad3bb811, SUC_lexicon_tags_1 },
    { 0xb19bb812, SUC_lexicon_tags_2 },
    { 0x6d5bb812, SUC_lexicon_tags_39 },
    { 0x835bb814, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467bb819, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfabbb81b, SUC_lexicon_tags_7 },
    { 0x3f9bb81c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79bb820, SUC_lexicon_tags_37 },
    { 0xda3bb820, SUC_lexicon_tags_39 },
    { 0x4c9bb820, SUC_lexicon_tags_26 },
    { 0xb09bb823, SUC_lexicon_tags_10 },
    { 0xe45bb824, SUC_lexicon_tags_23 },
    { 0x1dfbb824, SUC_lexicon_tags_37 },
    { 0xbbdbb825, SUC_lexicon_tags_21 },
    { 0xd5fbb827, SUC_lexicon_tags_17 },
    { 0x8e5bb826, SUC_lexicon_tags_9 },
    { 0x589bb829, SUC_lexicon_tags_11 },
    { 0x67bbb82a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73bb833, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe35bb835, SUC_lexicon_tags_2 },
    { 0x49bbb836, SUC_lexicon_tags_12 },
    { 0x967bb836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefbb83b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x683bb83d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc17bb83f, SUC_lexicon_tags_8 },
    { 0xbc1bb840, SUC_lexicon_tags_2 },
    { 0x797bb841, SUC_lexicon_tags_56 },
    { 0x20dbb841, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bbb845, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1bbb847, SUC_lexicon_tags_25 },
    { 0xaf3bb847, SUC_lexicon_tags_17 },
    { 0x3bbbb849, SUC_lexicon_tags_65 },
    { 0x44dbb84a, SUC_lexicon_tags_68 },
    { 0x95bb849, SUC_lexicon_tags_25 },
    { 0xc9bb847, SUC_lexicon_tags_2 },
    { 0x2f3bb849, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79bb857, SUC_lexicon_tags_0 },
    { 0x53dbb858, SUC_lexicon_tags_23 },
    { 0xf39bb857, SUC_lexicon_tags_37 },
    { 0x123bb858, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x359bb85d, SUC_lexicon_tags_11 },
    { 0xf45bb85e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315bb863, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa53bb865, SUC_lexicon_tags_160 },
    { 0xc8bbb865, SUC_lexicon_tags_63 },
    { 0x49bb867, SUC_lexicon_tags_2 },
    { 0xe95bb868, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23bb86d, SUC_lexicon_tags_37 },
    { 0x7c3bb86e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8cfbb870, SUC_lexicon_tags_25 },
    { 0x7ddbb871, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x21bbb873, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcbbbb875, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1bb878, SUC_lexicon_tags_66 },
    { 0x16dbb878, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe5bb87b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791bb87f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93bb885, SUC_lexicon_tags_47 },
    { 0x307bb886, SUC_lexicon_tags_28 },
    { 0x981bb887, SUC_lexicon_tags_32 },
    { 0xc9dbb888, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b7bb88a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9bb88d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65bb890, SUC_lexicon_tags_11 },
    { 0xd49bb891, SUC_lexicon_tags_15 },
    { 0xe19bb890, SUC_lexicon_tags_23 },
    { 0xe99bb893, SUC_lexicon_tags_37 },
    { 0xd99bb894, SUC_lexicon_tags_776 },
    { 0x73bbb893, SUC_lexicon_tags_37 },
    { 0x12dbb896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bbb89c, SUC_lexicon_tags_15 },
    { 0xd89bb89d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x47fbb89f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dbb8a4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x507bb8a6, SUC_lexicon_tags_65 },
    { 0x3bfbb8a7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09bb8aa, SUC_lexicon_tags_0 },
    { 0xebbb8ab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5d1bb8ad, SUC_lexicon_tags_9 },
    { 0x9b3bb8ae, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x745bb8b1, SUC_lexicon_tags_57 },
    { 0x1e5bb8b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11fbb8b4, SUC_lexicon_tags_140 },
    { 0x7f5bb8b5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7bb8b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddbb8c3, SUC_lexicon_tags_25 },
    { 0x689bb8c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bbb8c8, SUC_lexicon_tags_0 },
    { 0x44bbb8c9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fbb8ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bbb8d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x375bb8d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11bb8d7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291bb8db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43bb8de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedbb8e4, SUC_lexicon_tags_4 },
    { 0xa1dbb8e5, SUC_lexicon_tags_63 },
    { 0x35bb8e6, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0, NULL },
    { 0x165bb8e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729bb8ef, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x40fbb8f1, SUC_lexicon_tags_14 },
    { 0x67fbb8f2, SUC_lexicon_tags_11 },
    { 0x6f9bb8f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4e7bb8f5, SUC_lexicon_tags_15 },
    { 0x5c3bb8f6, SUC_lexicon_tags_60 },
    { 0x49dbb8f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175bb8fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351bb901, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3bb904, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fbb909, SUC_lexicon_tags_7 },
    { 0x681bb90a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bb90d, SUC_lexicon_tags_37 },
    { 0xdcdbb90e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa65bb910, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5bb914, SUC_lexicon_tags_11 },
    { 0x4cfbb915, SUC_lexicon_tags_11 },
    { 0x637bb914, SUC_lexicon_tags_5 },
    { 0x461bb915, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x555bb91a, SUC_lexicon_tags_2 },
    { 0xb2bbb91b, SUC_lexicon_tags_17 },
    { 0xe5bb91b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12bbb91e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbb922, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fbb925, SUC_lexicon_tags_37 },
    { 0xfdbb926, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x957bb929, SUC_lexicon_tags_29 },
    { 0xf7bbb92a, SUC_lexicon_tags_47 },
    { 0x3bfbb92b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaabbb92d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdbb930, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x10dbb933, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dbb937, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x51bb939, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93bb93e, SUC_lexicon_tags_23 },
    { 0x8edbb93f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x94bbb941, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x279bb944, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5bb94d, SUC_lexicon_tags_11 },
    { 0xddbb94e, SUC_lexicon_tags_7 },
    { 0x78bbb94e, SUC_lexicon_tags_61 },
    { 0xe33bb94d, SUC_lexicon_tags_2 },
    { 0x45dbb951, SUC_lexicon_tags_11 },
    { 0x169bb94d, SUC_lexicon_tags_2 },
    { 0x1fbbb953, SUC_lexicon_tags_32 },
    { 0x79bbb952, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99bb957, SUC_lexicon_tags_21 },
    { 0xc29bb957, SUC_lexicon_tags_2 },
    { 0xfebbb959, SUC_lexicon_tags_17 },
    { 0xb97bb958, SUC_lexicon_tags_17 },
    { 0x425bb95a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf49bb95d, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5bb960, SUC_lexicon_tags_15 },
    { 0x579bb961, SUC_lexicon_tags_1 },
    { 0x57bb961, SUC_lexicon_tags_43 },
    { 0xe65bb960, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5bbb965, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x415bb967, SUC_lexicon_tags_12 },
    { 0x15bbb968, SUC_lexicon_tags_12 },
    { 0xeedbb969, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x755bb96c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x921bb96e, SUC_lexicon_tags_15 },
    { 0x8cfbb96f, SUC_lexicon_tags_39 },
    { 0x59dbb970, SUC_lexicon_tags_77 },
    { 0x71fbb970, SUC_lexicon_tags_11 },
    { 0x32dbb971, SUC_lexicon_tags_12 },
    { 0x51bbb973, SUC_lexicon_tags_16 },
    { 0x937bb96e, SUC_lexicon_tags_43 },
    { 0x201bb975, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5bb979, SUC_lexicon_tags_17 },
    { 0x557bb97a, SUC_lexicon_tags_12 },
    { 0x125bb97b, SUC_lexicon_tags_43 },
    { 0x465bb97a, SUC_lexicon_tags_47 },
    { 0x21fbb97a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181bb983, SUC_lexicon_tags_47 },
    { 0x7afbb984, SUC_lexicon_tags_11 },
    { 0x333bb985, SUC_lexicon_tags_9 },
    { 0x1b1bb986, SUC_lexicon_tags_2 },
    { 0x2a5bb985, SUC_lexicon_tags_11 },
    { 0xeddbb988, SUC_lexicon_tags_14 },
    { 0x6dbbb987, SUC_lexicon_tags_1 },
    { 0x85bbb984, SUC_lexicon_tags_7 },
    { 0x997bb984, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713bb98f, SUC_lexicon_tags_37 },
    { 0xdadbb98f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fbb998, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x813bb99a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325bb99e, SUC_lexicon_tags_2 },
    { 0x1b9bb99f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9bbb9a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfbb9a5, SUC_lexicon_tags_11 },
    { 0xd3fbb9a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x217bb9a8, SUC_lexicon_tags_9 },
    { 0x1c3bb9a9, SUC_lexicon_tags_8 },
    { 0x789bb9aa, SUC_lexicon_tags_206 },
    { 0xfadbb9ab, SUC_lexicon_tags_15 },
    { 0x8a1bb9a9, SUC_lexicon_tags_16 },
    { 0xf17bb9ab, SUC_lexicon_tags_25 },
    { 0xb85bb9ae, SUC_lexicon_tags_27 },
    { 0xefbb9af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dbb9b3, SUC_lexicon_tags_11 },
    { 0x833bb9b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x601bb9b6, SUC_lexicon_tags_32 },
    { 0x25fbb9b6, SUC_lexicon_tags_11 },
    { 0x263bb9b8, SUC_lexicon_tags_19 },
    { 0x92dbb9b9, SUC_lexicon_tags_55 },
    { 0x7c5bb9b7, SUC_lexicon_tags_0 },
    { 0xf2fbb9bb, SUC_lexicon_tags_3 },
    { 0x3cbbb9bc, SUC_lexicon_tags_62 },
    { 0x85bbb9bd, SUC_lexicon_tags_43 },
    { 0x1b9bb9be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54bbb9c0, SUC_lexicon_tags_20 },
    { 0x877bb9c0, SUC_lexicon_tags_8 },
    { 0x20fbb9c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201bb9c6, SUC_lexicon_tags_28 },
    { 0xd53bb9c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01bb9ca, SUC_lexicon_tags_47 },
    { 0x493bb9ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x379bb9cd, SUC_lexicon_tags_38 },
    { 0xbd1bb9ce, SUC_lexicon_tags_66 },
    { 0xf1dbb9cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffbb9d2, SUC_lexicon_tags_3 },
    { 0x1e7bb9d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221bb9d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9bb9de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe01bb9e0, SUC_lexicon_tags_6 },
    { 0x79fbb9e1, SUC_lexicon_tags_6 },
    { 0x55dbb9e1, SUC_lexicon_tags_38 },
    { 0x179bb9e3, SUC_lexicon_tags_17 },
    { 0x5d5bb9e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d5bb9e6, SUC_lexicon_tags_7 },
    { 0x915bb9e7, SUC_lexicon_tags_8 },
    { 0x333bb9e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x461bb9eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x809bb9ed, SUC_lexicon_tags_25 },
    { 0x8f1bb9ed, SUC_lexicon_tags_11 },
    { 0x7bdbb9ee, SUC_lexicon_tags_11 },
    { 0x9abbb9f0, SUC_lexicon_tags_33 },
    { 0x699bb9f1, SUC_lexicon_tags_31 },
    { 0xb41bb9f2, SUC_lexicon_tags_4 },
    { 0x66dbb9ef, SUC_lexicon_tags_65 },
    { 0xc5bbb9f4, SUC_lexicon_tags_12 },
    { 0x57dbb9ee, SUC_lexicon_tags_12 },
    { 0x45bbb9ed, SUC_lexicon_tags_39 },
    { 0xd57bb9f7, SUC_lexicon_tags_17 },
    { 0xe31bb9f8, SUC_lexicon_tags_31 },
    { 0x483bb9f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x157bb9fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2bbb9fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bbba02, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b1bba04, SUC_lexicon_tags_21 },
    { 0x9b9bba05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439bba0d, SUC_lexicon_tags_26 },
    { 0x41bba0e, SUC_lexicon_tags_37 },
    { 0xe9fbba0e, SUC_lexicon_tags_21 },
    { 0xd3fbba0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd95bba12, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bbba17, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fbba1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x863bba1d, SUC_lexicon_tags_11 },
    { 0x9a7bba1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffbba24, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xba5bba26, SUC_lexicon_tags_15 },
    { 0x597bba26, SUC_lexicon_tags_7 },
    { 0x44bbba28, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dbba2c, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bbba30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x229bba33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35bba36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9bbba38, SUC_lexicon_tags_15 },
    { 0x33dbba39, SUC_lexicon_tags_38 },
    { 0x61fbba3a, SUC_lexicon_tags_65 },
    { 0x9d3bba3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55bba3f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231bba43, SUC_lexicon_tags_33 },
    { 0xe11bba44, SUC_lexicon_tags_14 },
    { 0xa67bba43, SUC_lexicon_tags_1 },
    { 0x6edbba43, SUC_lexicon_tags_25 },
    { 0x705bba47, SUC_lexicon_tags_17 },
    { 0x919bba48, SUC_lexicon_tags_25 },
    { 0x2e9bba47, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891bba4f, SUC_lexicon_tags_43 },
    { 0x9e3bba50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcafbba52, SUC_lexicon_tags_21 },
    { 0x7d3bba53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abbba5a, SUC_lexicon_tags_14 },
    { 0x6cdbba5b, SUC_lexicon_tags_47 },
    { 0x5a5bba5b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bba5f, SUC_lexicon_tags_37 },
    { 0xf75bba60, SUC_lexicon_tags_37 },
    { 0x11dbba61, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabbba64, SUC_lexicon_tags_9 },
    { 0xb3fbba65, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf3fbba67, SUC_lexicon_tags_25 },
    { 0xf69bba68, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77bba6b, SUC_lexicon_tags_11 },
    { 0x547bba6c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dbba74, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bba7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705bba7f, SUC_lexicon_tags_8 },
    { 0x509bba80, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xa35bba82, SUC_lexicon_tags_11 },
    { 0xdfdbba83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a9bba85, SUC_lexicon_tags_9 },
    { 0x759bba86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x16bbba88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbbba8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349bba91, SUC_lexicon_tags_11 },
    { 0x335bba92, SUC_lexicon_tags_14 },
    { 0x677bba91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c9bba95, SUC_lexicon_tags_434 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bba9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9bba9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdbbaa1, SUC_lexicon_tags_40 },
    { 0x4fdbbaa2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7bbaa5, SUC_lexicon_tags_15 },
    { 0xf11bbaa6, SUC_lexicon_tags_14 },
    { 0xb53bbaa5, SUC_lexicon_tags_37 },
    { 0x5e1bbaa5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7bbaad, SUC_lexicon_tags_1 },
    { 0x9bdbbaae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bbbab1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf89bbab3, SUC_lexicon_tags_14 },
    { 0xa6dbbab4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba9bbab6, SUC_lexicon_tags_25 },
    { 0xa99bbab6, SUC_lexicon_tags_14 },
    { 0xe99bbab8, SUC_lexicon_tags_17 },
    { 0x58fbbab8, SUC_lexicon_tags_29 },
    { 0xffdbbab9, SUC_lexicon_tags_12 },
    { 0x77bbbabb, SUC_lexicon_tags_15 },
    { 0x7d5bbabb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5bbac0, SUC_lexicon_tags_47 },
    { 0x1f1bbac1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37bbac5, SUC_lexicon_tags_11 },
    { 0x4ebbbac5, SUC_lexicon_tags_0 },
    { 0xc6dbbac7, SUC_lexicon_tags_17 },
    { 0xaa5bbac8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6c3bbaca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb4bbbacc, SUC_lexicon_tags_47 },
    { 0x831bbacd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x965bbacf, SUC_lexicon_tags_15 },
    { 0xda3bbacf, SUC_lexicon_tags_38 },
    { 0x193bbad1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63bbad4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3bbad7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa99bbad9, SUC_lexicon_tags_38 },
    { 0xdc7bbada, SUC_lexicon_tags_14 },
    { 0x455bbadb, SUC_lexicon_tags_10 },
    { 0xbffbbadc, SUC_lexicon_tags_2 },
    { 0x645bbada, SUC_lexicon_tags_2 },
    { 0xacbbbade, SUC_lexicon_tags_25 },
    { 0x5ddbbade, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517bbae4, SUC_lexicon_tags_20 },
    { 0x187bbae5, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1bbae8, SUC_lexicon_tags_12 },
    { 0x4c1bbae9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7bbaed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217bbaf3, SUC_lexicon_tags_37 },
    { 0x7b7bbaf4, SUC_lexicon_tags_1 },
    { 0x5a1bbaf5, SUC_lexicon_tags_25 },
    { 0xdb7bbaf3, SUC_lexicon_tags_7 },
    { 0x57fbbaf7, SUC_lexicon_tags_6 },
    { 0x585bbaf8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd9dbbafa, SUC_lexicon_tags_31 },
    { 0x285bbafa, SUC_lexicon_tags_17 },
    { 0x51fbbafc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4bbbafe, SUC_lexicon_tags_7 },
    { 0x9b9bbaff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefbbb08, SUC_lexicon_tags_37 },
    { 0xd3fbbb09, SUC_lexicon_tags_23 },
    { 0x39bbb0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbbbb0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dbbb14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6d9bbb16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755bbb1f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x95dbbb22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ddbbb24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffbbb2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271bbb2f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x81dbbb31, SUC_lexicon_tags_11 },
    { 0xc29bbb31, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5bbb35, SUC_lexicon_tags_6 },
    { 0xfa5bbb35, SUC_lexicon_tags_48 },
    { 0x3e5bbb37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbbbb3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05bbb3f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bbbb42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x983bbb45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7bbb49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x493bbb4c, SUC_lexicon_tags_7 },
    { 0x771bbb4d, SUC_lexicon_tags_45 },
    { 0x735bbb4e, SUC_lexicon_tags_37 },
    { 0xcc7bbb4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b7bbb51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bbbb57, SUC_lexicon_tags_7 },
    { 0x4bbbb57, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7bbb5b, SUC_lexicon_tags_62 },
    { 0x62dbbb5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x40dbbb5f, SUC_lexicon_tags_30 },
    { 0xc77bbb60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1bbb66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x83dbbb69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c1bbb6b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19bbb6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd67bbb71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9bbb77, SUC_lexicon_tags_82 },
    { 0x58fbbb77, SUC_lexicon_tags_79 },
    { 0x273bbb78, SUC_lexicon_tags_43 },
    { 0xebbbbb7a, SUC_lexicon_tags_15 },
    { 0xc19bbb7a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0fbbb7d, SUC_lexicon_tags_25 },
    { 0x80fbbb7e, SUC_lexicon_tags_17 },
    { 0x773bbb7f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x961bbb82, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fbbb89, SUC_lexicon_tags_7 },
    { 0x71dbbb8a, SUC_lexicon_tags_25 },
    { 0xbcfbbb8a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735bbb8f, SUC_lexicon_tags_12 },
    { 0xcf9bbb90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f3bbb92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981bbb97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7bbb9b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3dbbb9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbbba0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x83dbbba2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58dbbba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1bbbac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc1bbbae, SUC_lexicon_tags_47 },
    { 0x269bbbae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353bbbb3, SUC_lexicon_tags_82 },
    { 0x41bbbbb4, SUC_lexicon_tags_31 },
    { 0x247bbbb3, SUC_lexicon_tags_25 },
    { 0x7b7bbbb4, SUC_lexicon_tags_56 },
    { 0x143bbbb7, SUC_lexicon_tags_37 },
    { 0xa5bbbbb8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x767bbbba, SUC_lexicon_tags_6 },
    { 0x2cdbbbbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85dbbbc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fbbbc5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x81bbbbc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x225bbbca, SUC_lexicon_tags_1482 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbbbbd1, SUC_lexicon_tags_25 },
    { 0xd4dbbbd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7bbbd8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85bbbdb, SUC_lexicon_tags_7 },
    { 0x119bbbdc, SUC_lexicon_tags_25 },
    { 0x919bbbdd, SUC_lexicon_tags_43 },
    { 0x34bbbbdd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7bbbe8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9bbbee, SUC_lexicon_tags_46 },
    { 0x7f5bbbef, SUC_lexicon_tags_37 },
    { 0xc13bbbf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dbbbf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bfbbbfa, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x271bbbfd, SUC_lexicon_tags_7 },
    { 0xbd1bbbfe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bbbc01, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81bbc05, SUC_lexicon_tags_15 },
    { 0xae1bbc05, SUC_lexicon_tags_9 },
    { 0xbcbbbc06, SUC_lexicon_tags_216 },
    { 0xfcdbbc08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbbbc0b, SUC_lexicon_tags_29 },
    { 0xadfbbc0c, SUC_lexicon_tags_28 },
    { 0xd09bbc0d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xea9bbc0f, SUC_lexicon_tags_257 },
    { 0x4edbbc10, SUC_lexicon_tags_45 },
    { 0xb31bbc10, SUC_lexicon_tags_17 },
    { 0xfc7bbc12, SUC_lexicon_tags_11 },
    { 0xd9fbbc13, SUC_lexicon_tags_39 },
    { 0xdc9bbc11, SUC_lexicon_tags_11 },
    { 0xaf3bbc13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7bbc18, SUC_lexicon_tags_11 },
    { 0x117bbc19, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ddbbc1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bbbc1f, SUC_lexicon_tags_37 },
    { 0x349bbc20, SUC_lexicon_tags_47 },
    { 0x755bbc20, SUC_lexicon_tags_60 },
    { 0x583bbc21, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0xb8dbbc24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefbbc28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bbc2d, SUC_lexicon_tags_11 },
    { 0x869bbc2d, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfbbc32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bbc37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x889bbc39, SUC_lexicon_tags_26 },
    { 0xa91bbc3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x861bbc3d, SUC_lexicon_tags_49 },
    { 0xbd3bbc3e, SUC_lexicon_tags_14 },
    { 0xf0dbbc3f, SUC_lexicon_tags_44 },
    { 0x1a7bbc40, SUC_lexicon_tags_19 },
    { 0xb79bbc3e, SUC_lexicon_tags_23 },
    { 0x4fbbc42, SUC_lexicon_tags_167 },
    { 0xacfbbc41, SUC_lexicon_tags_49 },
    { 0xc17bbc44, SUC_lexicon_tags_25 },
    { 0x831bbc44, SUC_lexicon_tags_43 },
    { 0xca9bbc45, SUC_lexicon_tags_25 },
    { 0xe2bbbc46, SUC_lexicon_tags_39 },
    { 0xfefbbc48, SUC_lexicon_tags_15 },
    { 0xc21bbc44, SUC_lexicon_tags_54 },
    { 0xf75bbc45, SUC_lexicon_tags_26 },
    { 0xfa9bbc42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bbbc53, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x491bbc55, SUC_lexicon_tags_1 },
    { 0x221bbc56, SUC_lexicon_tags_0 },
    { 0x555bbc57, SUC_lexicon_tags_43 },
    { 0xc35bbc57, SUC_lexicon_tags_45 },
    { 0x8e5bbc57, SUC_lexicon_tags_4 },
    { 0x3c5bbc59, SUC_lexicon_tags_38 },
    { 0xb6dbbc58, SUC_lexicon_tags_17 },
    { 0x51fbbc5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7bbc5f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3bbc63, SUC_lexicon_tags_65 },
    { 0x473bbc63, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26fbbc66, SUC_lexicon_tags_41 },
    { 0x7f7bbc66, SUC_lexicon_tags_124 },
    { 0x1d5bbc68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291bbc6c, SUC_lexicon_tags_11 },
    { 0x8bbbc6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5bbc70, SUC_lexicon_tags_14 },
    { 0xd9dbbc70, SUC_lexicon_tags_43 },
    { 0xa03bbc72, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543bbc77, SUC_lexicon_tags_16 },
    { 0xd69bbc78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x417bbc7b, SUC_lexicon_tags_8 },
    { 0x661bbc7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x759bbc7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x337bbc82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbbbc85, SUC_lexicon_tags_2 },
    { 0x97dbbc85, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0xda5bbc88, SUC_lexicon_tags_17 },
    { 0x623bbc88, SUC_lexicon_tags_15 },
    { 0x187bbc8a, SUC_lexicon_tags_6 },
    { 0x1a3bbc89, SUC_lexicon_tags_3 },
    { 0xf9dbbc8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fbbc92, SUC_lexicon_tags_11 },
    { 0x7edbbc92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bbbc96, SUC_lexicon_tags_25 },
    { 0x8fbbbc96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b9bbc99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bbbc9c, SUC_lexicon_tags_23 },
    { 0x37bbbc9d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7bbca2, SUC_lexicon_tags_12 },
    { 0x4fbbca3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61bbca7, SUC_lexicon_tags_27 },
    { 0x3bbca8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc43bbcaa, SUC_lexicon_tags_25 },
    { 0x8a7bbcab, SUC_lexicon_tags_12 },
    { 0xf3dbbcac, SUC_lexicon_tags_60 },
    { 0x2abbbcad, SUC_lexicon_tags_39 },
    { 0x98fbbcac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b7bbcb0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffbbcb6, SUC_lexicon_tags_13 },
    { 0xd7bbbcb7, SUC_lexicon_tags_2 },
    { 0xa2bbbcb8, SUC_lexicon_tags_11 },
    { 0x3c5bbcb9, SUC_lexicon_tags_2 },
    { 0x513bbcba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493bbcbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1bbcc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193bbcd2, SUC_lexicon_tags_2 },
    { 0xed7bbcd3, SUC_lexicon_tags_45 },
    { 0x7f7bbcd4, SUC_lexicon_tags_37 },
    { 0x445bbcd3, SUC_lexicon_tags_10 },
    { 0x5bdbbcd6, SUC_lexicon_tags_15 },
    { 0xfadbbcd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281bbcdd, SUC_lexicon_tags_21 },
    { 0x85bbbcde, SUC_lexicon_tags_27 },
    { 0x27fbbcdf, SUC_lexicon_tags_113 },
    { 0xb37bbce0, SUC_lexicon_tags_6 },
    { 0x32bbbce1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x675bbce3, SUC_lexicon_tags_0 },
    { 0x8ebbbce4, SUC_lexicon_tags_15 },
    { 0x4b9bbce5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dbbcea, SUC_lexicon_tags_27 },
    { 0xabdbbcea, SUC_lexicon_tags_1 },
    { 0x3c7bbcec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x43bbbcef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31bbcf2, SUC_lexicon_tags_16 },
    { 0x69bbcf2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf9bbbcf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dbbcfd, SUC_lexicon_tags_37 },
    { 0xf4bbbcfe, SUC_lexicon_tags_37 },
    { 0x8ddbbcfe, SUC_lexicon_tags_25 },
    { 0xde1bbcfe, SUC_lexicon_tags_33 },
    { 0xce9bbd01, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fbbd06, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845bbd0c, SUC_lexicon_tags_7 },
    { 0x42fbbd0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa13bbd0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xadbbd11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7bbd15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xddbbbd17, SUC_lexicon_tags_17 },
    { 0x341bbd17, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33bbd1f, SUC_lexicon_tags_45 },
    { 0xb69bbd20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493bbd25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383bbd2a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837bbd2f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x18dbbd32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x83dbbd34, SUC_lexicon_tags_25 },
    { 0x749bbd35, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25bbd3d, SUC_lexicon_tags_2 },
    { 0x655bbd3e, SUC_lexicon_tags_15 },
    { 0x597bbd3e, SUC_lexicon_tags_25 },
    { 0x993bbd3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfbbd49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3bbd4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x74fbbd50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb9bbd52, SUC_lexicon_tags_7 },
    { 0x46dbbd53, SUC_lexicon_tags_54 },
    { 0x9adbbd54, SUC_lexicon_tags_23 },
    { 0x429bbd53, SUC_lexicon_tags_15 },
    { 0xe77bbd56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dbbd5a, SUC_lexicon_tags_17 },
    { 0xeadbbd5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1bbd5f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1bbd64, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbbd6b, SUC_lexicon_tags_49 },
    { 0xfe7bbd6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bbbd74, SUC_lexicon_tags_57 },
    { 0x72fbbd75, SUC_lexicon_tags_1 },
    { 0x5adbbd76, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb73bbd78, SUC_lexicon_tags_1 },
    { 0x20fbbd79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x37fbbd7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x889bbd7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbbd82, SUC_lexicon_tags_12 },
    { 0x267bbd83, SUC_lexicon_tags_93 },
    { 0xa97bbd83, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9bbd87, SUC_lexicon_tags_37 },
    { 0x4fbbd88, SUC_lexicon_tags_11 },
    { 0x725bbd89, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x507bbd8b, SUC_lexicon_tags_37 },
    { 0x223bbd8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77bbd8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x445bbd91, SUC_lexicon_tags_24 },
    { 0xdc3bbd92, SUC_lexicon_tags_25 },
    { 0x4dbbbd91, SUC_lexicon_tags_170 },
    { 0x309bbd94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119bbd98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbbbd9e, SUC_lexicon_tags_47 },
    { 0x5fdbbd9e, SUC_lexicon_tags_6 },
    { 0x3fbbbda0, SUC_lexicon_tags_7 },
    { 0x2ddbbda1, SUC_lexicon_tags_6 },
    { 0x9f7bbd9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953bbdad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1bbdb5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa61bbdb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bbdba, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0, NULL },
    { 0x13bbbdbd, SUC_lexicon_tags_47 },
    { 0xb03bbdbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fbbdc3, SUC_lexicon_tags_12 },
    { 0x84bbbdc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5bbdc7, SUC_lexicon_tags_12 },
    { 0x255bbdc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fbbdcc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb43bbdce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91bbdd5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1f7bbdd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27bbdda, SUC_lexicon_tags_6 },
    { 0x417bbddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ffbbddd, SUC_lexicon_tags_8 },
    { 0x1fbbddd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x229bbde0, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x37bbbde2, SUC_lexicon_tags_57 },
    { 0x1adbbde3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3cfbbde5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9bbde8, SUC_lexicon_tags_17 },
    { 0x61bbbde8, SUC_lexicon_tags_14 },
    { 0xf77bbdea, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bbded, SUC_lexicon_tags_16 },
    { 0xcd1bbdee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd53bbdf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d1bbdf2, SUC_lexicon_tags_29 },
    { 0xe21bbdf3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bbbdf6, SUC_lexicon_tags_7 },
    { 0x3f3bbdf7, SUC_lexicon_tags_77 },
    { 0x55bbdf6, SUC_lexicon_tags_14 },
    { 0x109bbdf9, SUC_lexicon_tags_17 },
    { 0xa2bbbdf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfbbdff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfbbe06, SUC_lexicon_tags_11 },
    { 0x989bbe07, SUC_lexicon_tags_16 },
    { 0xa55bbe08, SUC_lexicon_tags_87 },
    { 0xb55bbe09, SUC_lexicon_tags_47 },
    { 0xa3bbe09, SUC_lexicon_tags_37 },
    { 0x6abbbe0b, SUC_lexicon_tags_11 },
    { 0x757bbe07, SUC_lexicon_tags_68 },
    { 0xddfbbe0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635bbe13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1bbe18, SUC_lexicon_tags_42 },
    { 0x5e3bbe19, SUC_lexicon_tags_11 },
    { 0x4a5bbe19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907bbe1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23bbe25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157bbe29, SUC_lexicon_tags_6 },
    { 0x2b1bbe2a, SUC_lexicon_tags_1 },
    { 0xc75bbe2a, SUC_lexicon_tags_15 },
    { 0x401bbe2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbbbe2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x863bbe31, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x6f3bbe33, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bbbe36, SUC_lexicon_tags_1 },
    { 0xd5dbbe37, SUC_lexicon_tags_15 },
    { 0x76dbbe36, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x69bbbe3a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x789bbe3d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x84dbbe3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62bbbe41, SUC_lexicon_tags_4 },
    { 0xf1bbbe42, SUC_lexicon_tags_15 },
    { 0x291bbe43, SUC_lexicon_tags_43 },
    { 0x957bbe44, SUC_lexicon_tags_12 },
    { 0xb93bbe45, SUC_lexicon_tags_14 },
    { 0xc4bbbe46, SUC_lexicon_tags_17 },
    { 0x825bbe43, SUC_lexicon_tags_14 },
    { 0x591bbe48, SUC_lexicon_tags_17 },
    { 0xdbfbbe49, SUC_lexicon_tags_34 },
    { 0xcedbbe4a, SUC_lexicon_tags_14 },
    { 0x39bbbe49, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3bbe4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xff7bbe50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dbbe54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657bbe5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9bbe61, SUC_lexicon_tags_16 },
    { 0x465bbe62, SUC_lexicon_tags_2 },
    { 0x467bbe61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bbbe66, SUC_lexicon_tags_27 },
    { 0x59fbbe67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73bbe6a, SUC_lexicon_tags_66 },
    { 0xfbbe6a, SUC_lexicon_tags_40 },
    { 0xcd9bbe6c, SUC_lexicon_tags_11 },
    { 0x7b5bbe6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbbbe70, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x47dbbe72, SUC_lexicon_tags_37 },
    { 0x15bbe73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97bbe7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefbbe80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745bbe88, SUC_lexicon_tags_33 },
    { 0x9b7bbe89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7bbe8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bbbe90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bbbe95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21bbe98, SUC_lexicon_tags_31 },
    { 0x539bbe98, SUC_lexicon_tags_1 },
    { 0x767bbe99, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bbe9d, SUC_lexicon_tags_11 },
    { 0x4d3bbe9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8bbbea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x53bbbea3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd9bbea5, SUC_lexicon_tags_64 },
    { 0xd11bbea5, SUC_lexicon_tags_25 },
    { 0xdbbea6, SUC_lexicon_tags_57 },
    { 0xb73bbea5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93bbeac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x19dbbeae, SUC_lexicon_tags_21 },
    { 0xfb9bbeaf, SUC_lexicon_tags_19 },
    { 0xca1bbeb0, SUC_lexicon_tags_63 },
    { 0x559bbeb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd91bbeb3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x249bbeb5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x471bbeb8, SUC_lexicon_tags_92 },
    { 0xffdbbeb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37bbebd, SUC_lexicon_tags_37 },
    { 0x67bbebe, SUC_lexicon_tags_11 },
    { 0x7a9bbebe, SUC_lexicon_tags_63 },
    { 0x81bbbebd, SUC_lexicon_tags_7 },
    { 0xb85bbebe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fbbec4, SUC_lexicon_tags_10 },
    { 0x19bbbec4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dbbec8, SUC_lexicon_tags_55 },
    { 0xaabbbec9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7d3bbecb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3bbece, SUC_lexicon_tags_1 },
    { 0xdcbbbecf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x201bbed1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x515bbed3, SUC_lexicon_tags_27 },
    { 0x98dbbed4, SUC_lexicon_tags_1 },
    { 0x375bbed5, SUC_lexicon_tags_1 },
    { 0xf39bbed6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7bbeda, SUC_lexicon_tags_3 },
    { 0xcc9bbedb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x161bbedd, SUC_lexicon_tags_11 },
    { 0x4a7bbede, SUC_lexicon_tags_59 },
    { 0xb05bbede, SUC_lexicon_tags_37 },
    { 0x6e9bbedd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2b9bbee2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abbbee5, SUC_lexicon_tags_11 },
    { 0x3abbbee5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fbbeec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dbbef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bbefb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x811bbefe, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x611bbf00, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fbbf04, SUC_lexicon_tags_65 },
    { 0x421bbf04, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdbbf08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedbbf0a, SUC_lexicon_tags_8 },
    { 0x629bbf0a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7bbf0e, SUC_lexicon_tags_11 },
    { 0xc4fbbf0f, SUC_lexicon_tags_37 },
    { 0x7e9bbf0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc55bbf12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91bbbf14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9bbf1a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb51bbf1c, SUC_lexicon_tags_69 },
    { 0x5b5bbf1d, SUC_lexicon_tags_1 },
    { 0x4e1bbf1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x723bbf20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbbbf27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627bbf2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5bbf2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3dbbf31, SUC_lexicon_tags_7 },
    { 0x17bbbf32, SUC_lexicon_tags_15 },
    { 0xb61bbf32, SUC_lexicon_tags_329 },
    { 0, NULL },
    { 0x3d1bbf35, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x433bbf37, SUC_lexicon_tags_10 },
    { 0xad5bbf37, SUC_lexicon_tags_45 },
    { 0xb93bbf38, SUC_lexicon_tags_38 },
    { 0xbdfbbf3a, SUC_lexicon_tags_25 },
    { 0xab7bbf38, SUC_lexicon_tags_17 },
    { 0x305bbf38, SUC_lexicon_tags_63 },
    { 0x34bbbf3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabbbf40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1bbf47, SUC_lexicon_tags_91 },
    { 0xf05bbf48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x481bbf4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc45bbf4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x80fbbf50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfbbf53, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161bbf57, SUC_lexicon_tags_7 },
    { 0xdb7bbf57, SUC_lexicon_tags_17 },
    { 0x4f3bbf59, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x8e3bbf5b, SUC_lexicon_tags_87 },
    { 0x715bbf5c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfbbf5f, SUC_lexicon_tags_6 },
    { 0x61fbbf60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xff3bbf62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x363bbf65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975bbf6a, SUC_lexicon_tags_15 },
    { 0x3d5bbf6b, SUC_lexicon_tags_172 },
    { 0x12fbbf6c, SUC_lexicon_tags_40 },
    { 0x3efbbf6a, SUC_lexicon_tags_17 },
    { 0x17dbbf6d, SUC_lexicon_tags_11 },
    { 0x299bbf6f, SUC_lexicon_tags_4 },
    { 0xb29bbf6c, SUC_lexicon_tags_17 },
    { 0x305bbf71, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1bbf74, SUC_lexicon_tags_29 },
    { 0x819bbf75, SUC_lexicon_tags_6 },
    { 0xd65bbf76, SUC_lexicon_tags_25 },
    { 0x281bbf76, SUC_lexicon_tags_1 },
    { 0xefdbbf78, SUC_lexicon_tags_7 },
    { 0xd47bbf79, SUC_lexicon_tags_33 },
    { 0x10bbbf79, SUC_lexicon_tags_62 },
    { 0xa91bbf76, SUC_lexicon_tags_49 },
    { 0x39dbbf7c, SUC_lexicon_tags_17 },
    { 0x44bbbf7c, SUC_lexicon_tags_17 },
    { 0x9c5bbf7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5edbbf80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1bbf84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27bbf89, SUC_lexicon_tags_47 },
    { 0x3bbf8a, SUC_lexicon_tags_15 },
    { 0xe25bbf8b, SUC_lexicon_tags_23 },
    { 0x187bbf89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb93bbf8f, SUC_lexicon_tags_28 },
    { 0xa67bbf8f, SUC_lexicon_tags_23 },
    { 0xef9bbf91, SUC_lexicon_tags_1 },
    { 0x55fbbf90, SUC_lexicon_tags_60 },
    { 0x823bbf93, SUC_lexicon_tags_37 },
    { 0xac7bbf93, SUC_lexicon_tags_15 },
    { 0x813bbf93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59bbf9e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1bbfa1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x375bbfa3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47bbfa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41bbfab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fbbfae, SUC_lexicon_tags_15 },
    { 0xcc3bbfae, SUC_lexicon_tags_29 },
    { 0x8a5bbfb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdbbfb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaebbbfb6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3bbfb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7bbfc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x415bbfc6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x849bbfc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbfcc, SUC_lexicon_tags_2 },
    { 0x8dbbbfcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbbbfd0, SUC_lexicon_tags_46 },
    { 0x307bbfd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727bbfd7, SUC_lexicon_tags_47 },
    { 0x3a1bbfd8, SUC_lexicon_tags_1 },
    { 0x7abbbfd7, SUC_lexicon_tags_1 },
    { 0x7afbbfd8, SUC_lexicon_tags_68 },
    { 0xb01bbfdb, SUC_lexicon_tags_25 },
    { 0x683bbfdc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05bbfe0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x92fbbfe2, SUC_lexicon_tags_55 },
    { 0x70bbbfe3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7dbbfe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x491bbfe7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfbbfea, SUC_lexicon_tags_29 },
    { 0xc1fbbfeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77dbbfee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73bbff1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeddbbff3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849bbffa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bbc003, SUC_lexicon_tags_37 },
    { 0x3abbc003, SUC_lexicon_tags_1 },
    { 0xcf9bc005, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x707bc008, SUC_lexicon_tags_43 },
    { 0xe71bc009, SUC_lexicon_tags_37 },
    { 0x429bc00a, SUC_lexicon_tags_46 },
    { 0xcc9bc00b, SUC_lexicon_tags_1 },
    { 0x423bc00b, SUC_lexicon_tags_38 },
    { 0x425bc00d, SUC_lexicon_tags_12 },
    { 0xe3dbc00e, SUC_lexicon_tags_12 },
    { 0x1f1bc00b, SUC_lexicon_tags_37 },
    { 0x3f1bc00b, SUC_lexicon_tags_0 },
    { 0xdd3bc011, SUC_lexicon_tags_0 },
    { 0xb57bc012, SUC_lexicon_tags_21 },
    { 0x853bc00c, SUC_lexicon_tags_712 },
    { 0x9fbbc014, SUC_lexicon_tags_1 },
    { 0xd27bc015, SUC_lexicon_tags_11 },
    { 0xcefbc011, SUC_lexicon_tags_33 },
    { 0xfb1bc017, SUC_lexicon_tags_17 },
    { 0xd97bc00b, SUC_lexicon_tags_12 },
    { 0x63fbc019, SUC_lexicon_tags_47 },
    { 0xa39bc017, SUC_lexicon_tags_10 },
    { 0x93bbc01b, SUC_lexicon_tags_12 },
    { 0xa4bbc01c, SUC_lexicon_tags_1 },
    { 0x681bc01c, SUC_lexicon_tags_28 },
    { 0x2adbc01c, SUC_lexicon_tags_15 },
    { 0x759bc012, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd15bc021, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafbc027, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ffbc029, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x375bc02b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbc031, SUC_lexicon_tags_57 },
    { 0xac7bc031, SUC_lexicon_tags_19 },
    { 0x45fbc033, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7bc036, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3bc03d, SUC_lexicon_tags_31 },
    { 0xd7fbc03d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345bc043, SUC_lexicon_tags_26 },
    { 0x2e7bc044, SUC_lexicon_tags_23 },
    { 0xc29bc045, SUC_lexicon_tags_49 },
    { 0x1edbc046, SUC_lexicon_tags_15 },
    { 0xb0bbc047, SUC_lexicon_tags_17 },
    { 0x9ffbc044, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec5bc04a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1bc04d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dbc056, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767bc05a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x215bc05c, SUC_lexicon_tags_11 },
    { 0xd5bbc05d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bbc05f, SUC_lexicon_tags_11 },
    { 0x16bbc060, SUC_lexicon_tags_2 },
    { 0x32bbc060, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47bc065, SUC_lexicon_tags_4 },
    { 0x68bbc066, SUC_lexicon_tags_37 },
    { 0x847bc066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9bc071, SUC_lexicon_tags_11 },
    { 0x17fbc072, SUC_lexicon_tags_9 },
    { 0xa5bc073, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9fbc075, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbbc07b, SUC_lexicon_tags_27 },
    { 0x43bbc07c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67bc07f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efbc082, SUC_lexicon_tags_46 },
    { 0x9fbc082, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xee7bc085, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3bc088, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbdbc08a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe41bc08c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x51bbc08e, SUC_lexicon_tags_33 },
    { 0x2fbc08f, SUC_lexicon_tags_27 },
    { 0x9c3bc090, SUC_lexicon_tags_49 },
    { 0xf99bc091, SUC_lexicon_tags_104 },
    { 0xdfbbc092, SUC_lexicon_tags_6 },
    { 0x6dbc093, SUC_lexicon_tags_8 },
    { 0x673bc094, SUC_lexicon_tags_67 },
    { 0x5cfbc090, SUC_lexicon_tags_12 },
    { 0x88fbc08f, SUC_lexicon_tags_37 },
    { 0x6f9bc093, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdbc09b, SUC_lexicon_tags_3 },
    { 0x97bbc09c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37dbc09e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81bc0a2, SUC_lexicon_tags_17 },
    { 0xa75bc0a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x74dbc0a6, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xabbbc0a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce3bc0aa, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3abbc0ac, SUC_lexicon_tags_4 },
    { 0x42bbc0ad, SUC_lexicon_tags_19 },
    { 0x717bc0ad, SUC_lexicon_tags_4 },
    { 0xfebbc0ae, SUC_lexicon_tags_12 },
    { 0x88fbc0ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7bc0b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbbc0b8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65bc0bb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfbc0bf, SUC_lexicon_tags_49 },
    { 0x8e1bc0c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfbc0c3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5bc0c8, SUC_lexicon_tags_43 },
    { 0x411bc0c9, SUC_lexicon_tags_31 },
    { 0x1f7bc0ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6dbbc0cc, SUC_lexicon_tags_14 },
    { 0x1bfbc0cc, SUC_lexicon_tags_0 },
    { 0x637bc0ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dbc0d1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3bc0d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfbc0dc, SUC_lexicon_tags_60 },
    { 0x6dbbc0dd, SUC_lexicon_tags_68 },
    { 0xf8fbc0de, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5bc0e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dbc0e3, SUC_lexicon_tags_63 },
    { 0x6adbc0e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961bc0e8, SUC_lexicon_tags_7 },
    { 0xa6dbc0e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x449bc0eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fbc0f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x829bc0f5, SUC_lexicon_tags_63 },
    { 0xdd9bc0f6, SUC_lexicon_tags_38 },
    { 0x289bc0f7, SUC_lexicon_tags_37 },
    { 0x767bc0f8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x1c3bc0fa, SUC_lexicon_tags_67 },
    { 0x9a9bc0fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dbc100, SUC_lexicon_tags_17 },
    { 0x855bc101, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35bc10d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3bc110, SUC_lexicon_tags_10 },
    { 0x253bc110, SUC_lexicon_tags_25 },
    { 0x2a1bc112, SUC_lexicon_tags_1 },
    { 0x527bc113, SUC_lexicon_tags_6 },
    { 0x8f9bc114, SUC_lexicon_tags_6 },
    { 0x4a9bc115, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaa3bc117, SUC_lexicon_tags_7 },
    { 0xe51bc118, SUC_lexicon_tags_49 },
    { 0x23dbc119, SUC_lexicon_tags_13 },
    { 0x769bc11a, SUC_lexicon_tags_1 },
    { 0xb5fbc11b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efbc125, SUC_lexicon_tags_17 },
    { 0x435bc126, SUC_lexicon_tags_37 },
    { 0x905bc126, SUC_lexicon_tags_28 },
    { 0x8a7bc128, SUC_lexicon_tags_14 },
    { 0xd93bc126, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3bc12d, SUC_lexicon_tags_84 },
    { 0x285bc12e, SUC_lexicon_tags_12 },
    { 0x233bc12f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5bc133, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xefdbc136, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425bc13b, SUC_lexicon_tags_12 },
    { 0xdc3bc13b, SUC_lexicon_tags_14 },
    { 0x49dbc13d, SUC_lexicon_tags_32 },
    { 0x8c5bc13c, SUC_lexicon_tags_17 },
    { 0xa7bbc13b, SUC_lexicon_tags_25 },
    { 0x233bc13d, SUC_lexicon_tags_26 },
    { 0x41dbc141, SUC_lexicon_tags_37 },
    { 0x75dbc142, SUC_lexicon_tags_12 },
    { 0x7d5bc13b, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0xf3bc145, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913bc14e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadbc154, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc99bc156, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x665bc158, SUC_lexicon_tags_44 },
    { 0xa3bc159, SUC_lexicon_tags_11 },
    { 0xdb5bc158, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x197bc15d, SUC_lexicon_tags_21 },
    { 0xa91bc15d, SUC_lexicon_tags_17 },
    { 0xb63bc15f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37bc164, SUC_lexicon_tags_28 },
    { 0x13bc165, SUC_lexicon_tags_17 },
    { 0x8f7bc166, SUC_lexicon_tags_38 },
    { 0x227bc167, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54dbc169, SUC_lexicon_tags_27 },
    { 0x4b7bc16a, SUC_lexicon_tags_19 },
    { 0x799bc16b, SUC_lexicon_tags_10 },
    { 0x559bc16c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x419bc16f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05bc172, SUC_lexicon_tags_11 },
    { 0x665bc173, SUC_lexicon_tags_54 },
    { 0xa7bc174, SUC_lexicon_tags_26 },
    { 0x3a9bc175, SUC_lexicon_tags_10 },
    { 0xba9bc175, SUC_lexicon_tags_11 },
    { 0x309bc173, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85bc17c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe1bc17e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd13bc180, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3bc187, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41bc18f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x673bc191, SUC_lexicon_tags_29 },
    { 0x6b1bc191, SUC_lexicon_tags_82 },
    { 0xf8bbc192, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841bc19f, SUC_lexicon_tags_11 },
    { 0x51fbc1a0, SUC_lexicon_tags_17 },
    { 0x677bc1a0, SUC_lexicon_tags_16 },
    { 0x7c9bc1a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ddbc1a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9bc1a9, SUC_lexicon_tags_63 },
    { 0x6a7bc1aa, SUC_lexicon_tags_27 },
    { 0x10fbc1ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531bc1af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfbbc1b1, SUC_lexicon_tags_63 },
    { 0xc89bc1b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc97bc1b4, SUC_lexicon_tags_25 },
    { 0x511bc1b5, SUC_lexicon_tags_65 },
    { 0x41dbc1b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efbc1ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2f5bc1bc, SUC_lexicon_tags_11 },
    { 0xd19bc1bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5f5bc1bf, SUC_lexicon_tags_107 },
    { 0x903bc1c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x46dbc1c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x335bc1c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77bc1c8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x793bc1ca, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0x58fbc1cd, SUC_lexicon_tags_43 },
    { 0x54fbc1ce, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9bc1d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dbc1d3, SUC_lexicon_tags_25 },
    { 0xdbdbc1d4, SUC_lexicon_tags_14 },
    { 0x5e9bc1d4, SUC_lexicon_tags_55 },
    { 0xe13bc1d6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425bc1de, SUC_lexicon_tags_6 },
    { 0x6e9bc1de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80fbc1e1, SUC_lexicon_tags_37 },
    { 0x831bc1e2, SUC_lexicon_tags_19 },
    { 0x965bc1e2, SUC_lexicon_tags_8 },
    { 0x88bbc1e3, SUC_lexicon_tags_25 },
    { 0xfa7bc1e5, SUC_lexicon_tags_23 },
    { 0xd07bc1e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde9bc1e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b1bc1ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe81bc1ec, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb6fbc1ee, SUC_lexicon_tags_0 },
    { 0x1b7bc1ef, SUC_lexicon_tags_14 },
    { 0x4e3bc1f0, SUC_lexicon_tags_23 },
    { 0x64dbc1f0, SUC_lexicon_tags_28 },
    { 0xd9bbc1f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3bc1f8, SUC_lexicon_tags_7 },
    { 0xa51bc1f9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1bc1fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dbc202, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1bc206, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbbc209, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5bc20d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65bc211, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7bc214, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x753bc216, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5bc21a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5bc221, SUC_lexicon_tags_26 },
    { 0x9c1bc221, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91bc226, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81bc22a, SUC_lexicon_tags_63 },
    { 0xca7bc22a, SUC_lexicon_tags_16 },
    { 0x45bc22c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5bc232, SUC_lexicon_tags_1 },
    { 0xeebbc232, SUC_lexicon_tags_12 },
    { 0xbedbc233, SUC_lexicon_tags_15 },
    { 0xf3bbc235, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bc238, SUC_lexicon_tags_83 },
    { 0xe43bc238, SUC_lexicon_tags_37 },
    { 0x39bbc23a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bbc23d, SUC_lexicon_tags_26 },
    { 0xafdbc23e, SUC_lexicon_tags_201 },
    { 0x1f5bc23d, SUC_lexicon_tags_56 },
    { 0xdcfbc240, SUC_lexicon_tags_10 },
    { 0x1cfbc241, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f3bc243, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3b7bc245, SUC_lexicon_tags_19 },
    { 0xb2bbc245, SUC_lexicon_tags_37 },
    { 0x85fbc246, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abbc24d, SUC_lexicon_tags_47 },
    { 0xd47bc24e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfd3bc250, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24bbc253, SUC_lexicon_tags_14 },
    { 0xeb5bc254, SUC_lexicon_tags_27 },
    { 0x7abbc255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33bbc257, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17bc261, SUC_lexicon_tags_25 },
    { 0x2cfbc262, SUC_lexicon_tags_15 },
    { 0x203bc263, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x461bc265, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abbc269, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7bc26e, SUC_lexicon_tags_17 },
    { 0x7d5bc26f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9bc273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbbc275, SUC_lexicon_tags_14 },
    { 0x959bc275, SUC_lexicon_tags_66 },
    { 0xda3bc277, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9bc27b, SUC_lexicon_tags_37 },
    { 0xdf3bc27b, SUC_lexicon_tags_19 },
    { 0x20dbc27b, SUC_lexicon_tags_27 },
    { 0x2f7bc27e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ffbc280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x719bc282, SUC_lexicon_tags_7 },
    { 0x6fdbc282, SUC_lexicon_tags_10 },
    { 0xc01bc282, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb9bc286, SUC_lexicon_tags_7 },
    { 0x73bc287, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71dbc289, SUC_lexicon_tags_72 },
    { 0xe8dbc28a, SUC_lexicon_tags_28 },
    { 0x9f7bc28b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fbc28f, SUC_lexicon_tags_11 },
    { 0x1a9bc290, SUC_lexicon_tags_1 },
    { 0xbffbc290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7bc294, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe61bc296, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71dbc298, SUC_lexicon_tags_1 },
    { 0xd97bc298, SUC_lexicon_tags_2 },
    { 0x503bc299, SUC_lexicon_tags_7 },
    { 0xff3bc29b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb1bbc29d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfcfbc29f, SUC_lexicon_tags_29 },
    { 0xa11bc2a0, SUC_lexicon_tags_68 },
    { 0x775bc2a0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71bc2a8, SUC_lexicon_tags_37 },
    { 0x481bc2a9, SUC_lexicon_tags_13 },
    { 0xa29bc2aa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bc2b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddbc2b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7fdbc2b6, SUC_lexicon_tags_68 },
    { 0x9f3bc2b7, SUC_lexicon_tags_6 },
    { 0x921bc2b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadbc2bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525bc2c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5bc2c6, SUC_lexicon_tags_15 },
    { 0x279bc2c6, SUC_lexicon_tags_27 },
    { 0xb67bc2c6, SUC_lexicon_tags_10 },
    { 0xc75bc2c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123bc2cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbbc2d0, SUC_lexicon_tags_37 },
    { 0xc83bc2d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dbc2d8, SUC_lexicon_tags_7 },
    { 0x379bc2d8, SUC_lexicon_tags_1 },
    { 0x455bc2da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x315bc2dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bbc2e1, SUC_lexicon_tags_7 },
    { 0xc03bc2e2, SUC_lexicon_tags_12 },
    { 0x789bc2e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2f7bc2e5, SUC_lexicon_tags_38 },
    { 0xb05bc2e6, SUC_lexicon_tags_23 },
    { 0x45bc2e7, SUC_lexicon_tags_65 },
    { 0x877bc2e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc37bc2ea, SUC_lexicon_tags_25 },
    { 0xa93bc2ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fbc2f0, SUC_lexicon_tags_40 },
    { 0x50dbc2f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb1bc2f3, SUC_lexicon_tags_11 },
    { 0xbc1bc2f4, SUC_lexicon_tags_37 },
    { 0x8a1bc2f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x125bc2f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dbc2fa, SUC_lexicon_tags_3 },
    { 0xe8fbc2fa, SUC_lexicon_tags_25 },
    { 0x9ffbc2fb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc45bc2fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x66bbc301, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddbc305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa03bc307, SUC_lexicon_tags_19 },
    { 0xc37bc308, SUC_lexicon_tags_7 },
    { 0xc23bc309, SUC_lexicon_tags_30 },
    { 0x3d1bc309, SUC_lexicon_tags_1 },
    { 0xdadbc30b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f1bc30d, SUC_lexicon_tags_21 },
    { 0x781bc30d, SUC_lexicon_tags_19 },
    { 0xd51bc30f, SUC_lexicon_tags_12 },
    { 0x50dbc30f, SUC_lexicon_tags_7 },
    { 0xb5bbc310, SUC_lexicon_tags_25 },
    { 0x2d9bc311, SUC_lexicon_tags_9 },
    { 0xfb1bc313, SUC_lexicon_tags_32 },
    { 0x781bc30d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfbc317, SUC_lexicon_tags_11 },
    { 0x255bc318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x775bc31a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913bc320, SUC_lexicon_tags_60 },
    { 0xeefbc321, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2dbc323, SUC_lexicon_tags_17 },
    { 0x243bc323, SUC_lexicon_tags_30 },
    { 0x7f7bc325, SUC_lexicon_tags_1 },
    { 0x4f9bc326, SUC_lexicon_tags_17 },
    { 0x455bc327, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x385bc329, SUC_lexicon_tags_25 },
    { 0xf77bc329, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7bc33b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe91bc33d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fbc340, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd89bc342, SUC_lexicon_tags_7 },
    { 0xcffbc343, SUC_lexicon_tags_0 },
    { 0xb0bbc342, SUC_lexicon_tags_37 },
    { 0xe6dbc345, SUC_lexicon_tags_29 },
    { 0x9abbc346, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1bc34a, SUC_lexicon_tags_15 },
    { 0xb43bc34b, SUC_lexicon_tags_124 },
    { 0xa47bc34b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933bc353, SUC_lexicon_tags_6 },
    { 0xfc9bc353, SUC_lexicon_tags_2 },
    { 0x6bdbc355, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dbc358, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2e1bc35a, SUC_lexicon_tags_11 },
    { 0xe1fbc35a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27bbc35d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd59bc35f, SUC_lexicon_tags_8 },
    { 0xe93bc360, SUC_lexicon_tags_49 },
    { 0x913bc361, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x127bc363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7fbc365, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5afbc368, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fbc36e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527bc374, SUC_lexicon_tags_47 },
    { 0x961bc375, SUC_lexicon_tags_31 },
    { 0x2a3bc375, SUC_lexicon_tags_11 },
    { 0x2afbc377, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x71fbc379, SUC_lexicon_tags_17 },
    { 0xcc5bc379, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18dbc37d, SUC_lexicon_tags_12 },
    { 0xab1bc37e, SUC_lexicon_tags_8 },
    { 0x29fbc37d, SUC_lexicon_tags_12 },
    { 0x8f9bc380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffbc384, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49bc387, SUC_lexicon_tags_19 },
    { 0x55bbc388, SUC_lexicon_tags_7 },
    { 0x269bc389, SUC_lexicon_tags_1 },
    { 0x8d3bc38a, SUC_lexicon_tags_2 },
    { 0x859bc38b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5bc390, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xdddbc392, SUC_lexicon_tags_17 },
    { 0x41bc393, SUC_lexicon_tags_30 },
    { 0x385bc392, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bbc399, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e5bc39b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1bc3a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bc3a4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xcffbc3a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1bc3ab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3f3bc3ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x867bc3af, SUC_lexicon_tags_49 },
    { 0x489bc3b0, SUC_lexicon_tags_2 },
    { 0x86bbc3b1, SUC_lexicon_tags_27 },
    { 0xb9dbc3b0, SUC_lexicon_tags_30 },
    { 0xf9fbc3af, SUC_lexicon_tags_37 },
    { 0x739bc3b4, SUC_lexicon_tags_13 },
    { 0x19bbc3b5, SUC_lexicon_tags_25 },
    { 0x579bc3b3, SUC_lexicon_tags_25 },
    { 0x91fbc3b0, SUC_lexicon_tags_60 },
    { 0x81fbc3b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bbc3bc, SUC_lexicon_tags_46 },
    { 0x9afbc3bd, SUC_lexicon_tags_32 },
    { 0x1e3bc3be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a5bc3c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7bc3cb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x64dbc3cd, SUC_lexicon_tags_7 },
    { 0x5d9bc3ce, SUC_lexicon_tags_11 },
    { 0x1cfbc3cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9bc3d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x593bc3d4, SUC_lexicon_tags_11 },
    { 0x3dfbc3d5, SUC_lexicon_tags_7 },
    { 0x87bc3d4, SUC_lexicon_tags_25 },
    { 0xe4fbc3d7, SUC_lexicon_tags_12 },
    { 0xf89bc3d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37fbc3da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf3bc3dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaa5bc3de, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbbc3e2, SUC_lexicon_tags_37 },
    { 0x19fbc3e3, SUC_lexicon_tags_63 },
    { 0x4b3bc3e4, SUC_lexicon_tags_44 },
    { 0x671bc3e4, SUC_lexicon_tags_25 },
    { 0xbddbc3e4, SUC_lexicon_tags_28 },
    { 0x8cbbc3e6, SUC_lexicon_tags_20 },
    { 0x6c5bc3e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1bc3eb, SUC_lexicon_tags_15 },
    { 0x4a3bc3ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bbc3f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67bc3f3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x75fbc3f5, SUC_lexicon_tags_43 },
    { 0xe6dbc3f6, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fbc3fc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dbc404, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0fbc406, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd5bc408, SUC_lexicon_tags_31 },
    { 0xfe9bc409, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x91dbc40b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x88dbc40e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dbc412, SUC_lexicon_tags_56 },
    { 0x949bc413, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebbc41c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd93bc41f, SUC_lexicon_tags_45 },
    { 0x3efbc420, SUC_lexicon_tags_17 },
    { 0x71fbc421, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf61bc423, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7bc426, SUC_lexicon_tags_11 },
    { 0x71bc427, SUC_lexicon_tags_25 },
    { 0x2cfbc426, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25bc42b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5bc42e, SUC_lexicon_tags_9 },
    { 0x8dbbc42e, SUC_lexicon_tags_63 },
    { 0xef9bc42e, SUC_lexicon_tags_15 },
    { 0xc25bc430, SUC_lexicon_tags_17 },
    { 0xbafbc432, SUC_lexicon_tags_27 },
    { 0x735bc433, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7bc435, SUC_lexicon_tags_17 },
    { 0xd33bc436, SUC_lexicon_tags_1 },
    { 0x317bc437, SUC_lexicon_tags_82 },
    { 0x17dbc437, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bbc43e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa51bc440, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbc444, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fbc448, SUC_lexicon_tags_42 },
    { 0x127bc449, SUC_lexicon_tags_25 },
    { 0x7f3bc449, SUC_lexicon_tags_48 },
    { 0xfd5bc44b, SUC_lexicon_tags_0 },
    { 0xb75bc44c, SUC_lexicon_tags_39 },
    { 0xbb9bc44d, SUC_lexicon_tags_28 },
    { 0x8bbc44e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x479bc450, SUC_lexicon_tags_17 },
    { 0x187bc451, SUC_lexicon_tags_27 },
    { 0x30fbc452, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dbc45b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1c1bc45d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bc460, SUC_lexicon_tags_46 },
    { 0x279bc461, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18fbc463, SUC_lexicon_tags_63 },
    { 0xf75bc464, SUC_lexicon_tags_37 },
    { 0x827bc464, SUC_lexicon_tags_40 },
    { 0x3d7bc466, SUC_lexicon_tags_15 },
    { 0x8d5bc467, SUC_lexicon_tags_39 },
    { 0x5a7bc465, SUC_lexicon_tags_23 },
    { 0xd2fbc467, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfbc46d, SUC_lexicon_tags_87 },
    { 0xe77bc46d, SUC_lexicon_tags_37 },
    { 0x5f9bc46e, SUC_lexicon_tags_17 },
    { 0x9f1bc470, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9bc474, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5bc478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf9bc47a, SUC_lexicon_tags_14 },
    { 0xa65bc47a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1bc480, SUC_lexicon_tags_6 },
    { 0xf45bc481, SUC_lexicon_tags_27 },
    { 0xa83bc482, SUC_lexicon_tags_38 },
    { 0x77dbc483, SUC_lexicon_tags_68 },
    { 0xf9dbc482, SUC_lexicon_tags_336 },
    { 0xa57bc485, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263bc48b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fbc48f, SUC_lexicon_tags_33 },
    { 0x3adbc48f, SUC_lexicon_tags_17 },
    { 0x7e7bc491, SUC_lexicon_tags_25 },
    { 0x873bc492, SUC_lexicon_tags_47 },
    { 0x1d5bc492, SUC_lexicon_tags_11 },
    { 0x77bc494, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47bbc496, SUC_lexicon_tags_7 },
    { 0x3f7bc496, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x295bc499, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbbc49c, SUC_lexicon_tags_33 },
    { 0xed5bc49d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1bc4a0, SUC_lexicon_tags_1663 },
    { 0x60bbc4a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c5bc4a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dbc4a6, SUC_lexicon_tags_27 },
    { 0x98bbc4a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbbc4aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5bc4af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8e9bc4b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xad5bc4b3, SUC_lexicon_tags_25 },
    { 0xa57bc4b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbbc4b7, SUC_lexicon_tags_1 },
    { 0x64dbc4b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0fbc4ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fbc4bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3bc4c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bbc4c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbbc4cb, SUC_lexicon_tags_26 },
    { 0x663bc4cb, SUC_lexicon_tags_14 },
    { 0x621bc4cb, SUC_lexicon_tags_14 },
    { 0x5fbbc4cd, SUC_lexicon_tags_7 },
    { 0xa93bc4cf, SUC_lexicon_tags_8 },
    { 0x4cbbc4d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92fbc4d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdbc4d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b9bc4d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9bc4db, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9bc4de, SUC_lexicon_tags_7 },
    { 0xa9bc4df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1bc4e2, SUC_lexicon_tags_6 },
    { 0xca5bc4e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d3bc4e5, SUC_lexicon_tags_7 },
    { 0x10dbc4e6, SUC_lexicon_tags_43 },
    { 0x8f3bc4e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x305bc4e9, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x335bc4ec, SUC_lexicon_tags_16 },
    { 0xb2bbc4ed, SUC_lexicon_tags_38 },
    { 0x9cdbc4ed, SUC_lexicon_tags_25 },
    { 0x20bbc4ee, SUC_lexicon_tags_37 },
    { 0xee7bc4f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1bc4f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175bc4f7, SUC_lexicon_tags_15 },
    { 0xd19bc4f8, SUC_lexicon_tags_27 },
    { 0xae1bc4f7, SUC_lexicon_tags_37 },
    { 0xcbbc4fa, SUC_lexicon_tags_32 },
    { 0xd7dbc4fb, SUC_lexicon_tags_7 },
    { 0x8cdbc4fc, SUC_lexicon_tags_7 },
    { 0xf4fbc4fb, SUC_lexicon_tags_82 },
    { 0xbb7bc4fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x93bbc500, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x111bc502, SUC_lexicon_tags_37 },
    { 0x46fbc503, SUC_lexicon_tags_11 },
    { 0x805bc502, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dbc509, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9bc516, SUC_lexicon_tags_46 },
    { 0xe1bc516, SUC_lexicon_tags_0 },
    { 0xbc5bc518, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdbc51b, SUC_lexicon_tags_11 },
    { 0x861bc51c, SUC_lexicon_tags_6 },
    { 0xd67bc51d, SUC_lexicon_tags_61 },
    { 0x67fbc51e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x27dbc521, SUC_lexicon_tags_17 },
    { 0x5e7bc521, SUC_lexicon_tags_65 },
    { 0xb49bc523, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3bc52b, SUC_lexicon_tags_46 },
    { 0xc45bc52c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bc531, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf7bc533, SUC_lexicon_tags_2 },
    { 0x441bc534, SUC_lexicon_tags_5 },
    { 0xad9bc533, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x469bc538, SUC_lexicon_tags_108 },
    { 0x9efbc539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bbc53c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13bc53e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xabbc540, SUC_lexicon_tags_14 },
    { 0x479bc540, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97dbc543, SUC_lexicon_tags_14 },
    { 0x825bc543, SUC_lexicon_tags_40 },
    { 0xff9bc545, SUC_lexicon_tags_25 },
    { 0x88bbc546, SUC_lexicon_tags_97 },
    { 0x175bc547, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x987bc54a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffbc54f, SUC_lexicon_tags_30 },
    { 0x3dbc550, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbdbc552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9dfbc554, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3bc557, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fbc55b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x495bc55e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x551bc560, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9bc563, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1bc567, SUC_lexicon_tags_1 },
    { 0xf03bc567, SUC_lexicon_tags_37 },
    { 0xdcbbc569, SUC_lexicon_tags_0 },
    { 0xf3dbc56a, SUC_lexicon_tags_11 },
    { 0x543bc567, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd95bc56d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x63bbc570, SUC_lexicon_tags_7 },
    { 0x791bc570, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x319bc573, SUC_lexicon_tags_39 },
    { 0xf35bc573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb4bbc576, SUC_lexicon_tags_29 },
    { 0x3e5bc576, SUC_lexicon_tags_25 },
    { 0xbefbc578, SUC_lexicon_tags_8 },
    { 0xad9bc577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbbc57e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xeb7bc580, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07bc583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x619bc586, SUC_lexicon_tags_37 },
    { 0x6c9bc587, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227bc58c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4c1bc58e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdbc598, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7bc59b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x887bc59d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749bc5a2, SUC_lexicon_tags_11 },
    { 0x2d5bc5a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbc5a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x34bbc5a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bc5ac, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa87bc5af, SUC_lexicon_tags_91 },
    { 0xcafbc5b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56fbc5b2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9bc5b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bbc5b9, SUC_lexicon_tags_1 },
    { 0x661bc5ba, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bbc5bd, SUC_lexicon_tags_33 },
    { 0x27bc5bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xedbbc5c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935bc5c4, SUC_lexicon_tags_33 },
    { 0x7ebbc5c5, SUC_lexicon_tags_0 },
    { 0x9e9bc5c6, SUC_lexicon_tags_27 },
    { 0x97bc5c7, SUC_lexicon_tags_7 },
    { 0x21bc5c4, SUC_lexicon_tags_39 },
    { 0x5a9bc5c9, SUC_lexicon_tags_179 },
    { 0xbf9bc5ca, SUC_lexicon_tags_38 },
    { 0xf7dbc5cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9b9bc5cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bc5d0, SUC_lexicon_tags_63 },
    { 0xd2dbc5d1, SUC_lexicon_tags_11 },
    { 0xf5bbc5d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x751bc5d4, SUC_lexicon_tags_0 },
    { 0x77bbc5d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64fbc5d7, SUC_lexicon_tags_2 },
    { 0xb11bc5d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3bc5dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5bc5e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647bc5e5, SUC_lexicon_tags_11 },
    { 0x173bc5e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1bc5eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3bc5ee, SUC_lexicon_tags_25 },
    { 0x81bc5ee, SUC_lexicon_tags_31 },
    { 0xfb5bc5f0, SUC_lexicon_tags_60 },
    { 0xbfbbc5f1, SUC_lexicon_tags_6 },
    { 0x625bc5f2, SUC_lexicon_tags_60 },
    { 0xacdbc5f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dbc5f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3bc5fb, SUC_lexicon_tags_12 },
    { 0x163bc5fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeedbc5fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbbc601, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57bc607, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659bc60f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bbc616, SUC_lexicon_tags_19 },
    { 0x1f7bc617, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05bc61d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7cdbc61f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xd3bbc621, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9bc626, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x647bc629, SUC_lexicon_tags_12 },
    { 0x9a7bc62a, SUC_lexicon_tags_91 },
    { 0x54dbc62b, SUC_lexicon_tags_27 },
    { 0x437bc629, SUC_lexicon_tags_32 },
    { 0x95bbc62d, SUC_lexicon_tags_27 },
    { 0xd15bc62b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x83bbc630, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x867bc633, SUC_lexicon_tags_30 },
    { 0x7a3bc634, SUC_lexicon_tags_0 },
    { 0x357bc633, SUC_lexicon_tags_30 },
    { 0x49dbc636, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473bc63a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddbc63d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349bc641, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99bbc643, SUC_lexicon_tags_19 },
    { 0xb5dbc644, SUC_lexicon_tags_23 },
    { 0xd75bc644, SUC_lexicon_tags_11 },
    { 0xed9bc645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7bc64e, SUC_lexicon_tags_47 },
    { 0x17bc64f, SUC_lexicon_tags_2 },
    { 0x55bc64e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bbc655, SUC_lexicon_tags_4 },
    { 0xb1fbc655, SUC_lexicon_tags_10 },
    { 0x797bc657, SUC_lexicon_tags_299 },
    { 0x90dbc658, SUC_lexicon_tags_11 },
    { 0x85bc659, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17bc661, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8c7bc663, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfbc666, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93bc670, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bc674, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x237bc676, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb09bc678, SUC_lexicon_tags_2 },
    { 0xf19bc679, SUC_lexicon_tags_17 },
    { 0xb7bc67a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x527bc67d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7dbc67f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fbc682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x119bc685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x897bc687, SUC_lexicon_tags_23 },
    { 0x1e9bc688, SUC_lexicon_tags_46 },
    { 0xda7bc689, SUC_lexicon_tags_37 },
    { 0x545bc68a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bbc68d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdedbc690, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfbc693, SUC_lexicon_tags_47 },
    { 0xfddbc694, SUC_lexicon_tags_46 },
    { 0x55dbc694, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07bc699, SUC_lexicon_tags_56 },
    { 0x359bc69a, SUC_lexicon_tags_15 },
    { 0x463bc69b, SUC_lexicon_tags_14 },
    { 0x7d1bc69b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479bc6a0, SUC_lexicon_tags_2 },
    { 0x2e3bc6a1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xbbdbc6a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fbc6a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x153bc6a8, SUC_lexicon_tags_27 },
    { 0x629bc6a9, SUC_lexicon_tags_43 },
    { 0xe9dbc6aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x961bc6ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b9bc6af, SUC_lexicon_tags_23 },
    { 0x7b1bc6af, SUC_lexicon_tags_20 },
    { 0x5a5bc6b1, SUC_lexicon_tags_33 },
    { 0x341bc6af, SUC_lexicon_tags_28 },
    { 0xedbbc6b3, SUC_lexicon_tags_46 },
    { 0xd85bc6b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x959bc6b7, SUC_lexicon_tags_1 },
    { 0xd0dbc6b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655bc6be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x595bc6c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9bc6c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78bbc6c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3edbc6ca, SUC_lexicon_tags_47 },
    { 0x963bc6ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc5bc6cd, SUC_lexicon_tags_2 },
    { 0xecbbc6cd, SUC_lexicon_tags_2 },
    { 0x195bc6cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747bc6d4, SUC_lexicon_tags_7 },
    { 0x48fbc6d5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x189bc6d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbbc6dc, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x39bbc6de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dbc6e4, SUC_lexicon_tags_19 },
    { 0x9b9bc6e5, SUC_lexicon_tags_12 },
    { 0x653bc6e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1dfbc6e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57bc6ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7bbbc6ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9bc6f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x541bc6f6, SUC_lexicon_tags_17 },
    { 0x3cbbc6f7, SUC_lexicon_tags_1 },
    { 0x725bc6f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81fbc6fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dbc701, SUC_lexicon_tags_37 },
    { 0xdbc701, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf1bc704, SUC_lexicon_tags_30 },
    { 0x629bc705, SUC_lexicon_tags_12 },
    { 0xb63bc706, SUC_lexicon_tags_27 },
    { 0x21fbc704, SUC_lexicon_tags_28 },
    { 0x5d1bc708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c7bc70a, SUC_lexicon_tags_25 },
    { 0xea1bc70a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3bc710, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf69bc712, SUC_lexicon_tags_25 },
    { 0xf0bbc713, SUC_lexicon_tags_17 },
    { 0x7c3bc713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa5bc716, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bbc71d, SUC_lexicon_tags_10 },
    { 0xa1dbc71d, SUC_lexicon_tags_2 },
    { 0xb1dbc71f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x25dbc722, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511bc727, SUC_lexicon_tags_17 },
    { 0x37bbc728, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x553bc72b, SUC_lexicon_tags_45 },
    { 0x59fbc72c, SUC_lexicon_tags_30 },
    { 0xc13bc72d, SUC_lexicon_tags_38 },
    { 0x17fbc72e, SUC_lexicon_tags_30 },
    { 0xb8fbc72f, SUC_lexicon_tags_7 },
    { 0xb11bc730, SUC_lexicon_tags_1 },
    { 0xe99bc731, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43bc734, SUC_lexicon_tags_26 },
    { 0xa17bc735, SUC_lexicon_tags_1 },
    { 0x15bbc736, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47bc73a, SUC_lexicon_tags_29 },
    { 0xeb5bc73b, SUC_lexicon_tags_33 },
    { 0x3c1bc73b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855bc743, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2edbc746, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969bc74b, SUC_lexicon_tags_7 },
    { 0xda5bc74b, SUC_lexicon_tags_68 },
    { 0x21bbc74d, SUC_lexicon_tags_0 },
    { 0x8fdbc74b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbbc756, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75bc759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c9bc75b, SUC_lexicon_tags_91 },
    { 0xe5bbc75b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbbc760, SUC_lexicon_tags_1 },
    { 0x8bdbc761, SUC_lexicon_tags_12 },
    { 0x5afbc762, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1bc767, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fbc76a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x429bc76c, SUC_lexicon_tags_1 },
    { 0xecdbc76d, SUC_lexicon_tags_38 },
    { 0x905bc76e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x791bc770, SUC_lexicon_tags_17 },
    { 0xa53bc771, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7bc776, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f5bc778, SUC_lexicon_tags_0 },
    { 0xefbbc778, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ddbc77b, SUC_lexicon_tags_74 },
    { 0x8bbc77c, SUC_lexicon_tags_34 },
    { 0x671bc77d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3bc785, SUC_lexicon_tags_37 },
    { 0x825bc786, SUC_lexicon_tags_63 },
    { 0xa6fbc787, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeabbc789, SUC_lexicon_tags_36 },
    { 0xba1bc78a, SUC_lexicon_tags_8 },
    { 0x37fbc78b, SUC_lexicon_tags_1 },
    { 0x949bc78b, SUC_lexicon_tags_12 },
    { 0xc5fbc78a, SUC_lexicon_tags_1 },
    { 0x469bc78a, SUC_lexicon_tags_12 },
    { 0x5b1bc78f, SUC_lexicon_tags_28 },
    { 0x993bc78f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75bc796, SUC_lexicon_tags_8 },
    { 0x625bc797, SUC_lexicon_tags_36 },
    { 0xf01bc798, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x453bc79a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb9bc79c, SUC_lexicon_tags_31 },
    { 0xc89bc79d, SUC_lexicon_tags_11 },
    { 0xe63bc79c, SUC_lexicon_tags_26 },
    { 0x8fbbc79d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x773bc7a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421bc7a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efbc7ab, SUC_lexicon_tags_60 },
    { 0xa1dbc7ac, SUC_lexicon_tags_71 },
    { 0x8bfbc7ac, SUC_lexicon_tags_4 },
    { 0x8c3bc7ae, SUC_lexicon_tags_49 },
    { 0x693bc7af, SUC_lexicon_tags_45 },
    { 0xc8bbc7b0, SUC_lexicon_tags_55 },
    { 0xefbbc7b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddbc7b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39bc7b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x583bc7ba, SUC_lexicon_tags_12 },
    { 0x48dbc7bb, SUC_lexicon_tags_45 },
    { 0x219bc7ba, SUC_lexicon_tags_7 },
    { 0xad1bc7bd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x383bc7bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51bc7c2, SUC_lexicon_tags_1 },
    { 0x95fbc7c2, SUC_lexicon_tags_17 },
    { 0x36dbc7c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x391bc7c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddbc7c9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xff1bc7cb, SUC_lexicon_tags_1 },
    { 0x811bc7cb, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5bc7d2, SUC_lexicon_tags_4 },
    { 0xbdfbc7d3, SUC_lexicon_tags_0 },
    { 0x5c3bc7d4, SUC_lexicon_tags_63 },
    { 0x2d9bc7d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc1bc7d7, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43bc7da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edbc7dd, SUC_lexicon_tags_32 },
    { 0x77bbc7dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67bc7e1, SUC_lexicon_tags_1 },
    { 0x6bfbc7e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1bc7ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5bc7ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89bc7f1, SUC_lexicon_tags_47 },
    { 0xc33bc7f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x83dbc7f5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279bc7fd, SUC_lexicon_tags_77 },
    { 0x357bc7fd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dbc804, SUC_lexicon_tags_37 },
    { 0x52dbc805, SUC_lexicon_tags_16 },
    { 0xf43bc806, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fbc80b, SUC_lexicon_tags_0 },
    { 0x9dbbc80c, SUC_lexicon_tags_30 },
    { 0xb05bc80d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x187bc810, SUC_lexicon_tags_31 },
    { 0x16dbc810, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb77bc813, SUC_lexicon_tags_35 },
    { 0xfa1bc814, SUC_lexicon_tags_37 },
    { 0xe0fbc815, SUC_lexicon_tags_7 },
    { 0xfe3bc815, SUC_lexicon_tags_49 },
    { 0x1dfbc816, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5bc81a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x549bc81d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xabbbc81f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fbc824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x71fbc826, SUC_lexicon_tags_5 },
    { 0xbdbc826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bbc82b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39fbc82d, SUC_lexicon_tags_47 },
    { 0x981bc82e, SUC_lexicon_tags_11 },
    { 0xa21bc82d, SUC_lexicon_tags_39 },
    { 0x27dbc830, SUC_lexicon_tags_3 },
    { 0x78dbc831, SUC_lexicon_tags_15 },
    { 0x57fbc82d, SUC_lexicon_tags_63 },
    { 0xc05bc830, SUC_lexicon_tags_37 },
    { 0xb8fbc834, SUC_lexicon_tags_17 },
    { 0xa27bc834, SUC_lexicon_tags_9 },
    { 0x945bc82f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb03bc838, SUC_lexicon_tags_69 },
    { 0x839bc839, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x121bc83c, SUC_lexicon_tags_2 },
    { 0xe11bc83d, SUC_lexicon_tags_37 },
    { 0x8bbbc83e, SUC_lexicon_tags_37 },
    { 0x6e7bc83c, SUC_lexicon_tags_47 },
    { 0xdbfbc840, SUC_lexicon_tags_49 },
    { 0xb3bbc840, SUC_lexicon_tags_7 },
    { 0x521bc83c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdbc84c, SUC_lexicon_tags_23 },
    { 0x427bc84d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a7bc84f, SUC_lexicon_tags_37 },
    { 0x641bc850, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bbc855, SUC_lexicon_tags_26 },
    { 0x59dbc856, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20dbc85b, SUC_lexicon_tags_46 },
    { 0x655bc85c, SUC_lexicon_tags_25 },
    { 0x9dfbc85c, SUC_lexicon_tags_63 },
    { 0x115bc85d, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffbc863, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x159bc865, SUC_lexicon_tags_26 },
    { 0xe51bc866, SUC_lexicon_tags_2 },
    { 0xcb5bc866, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287bc86c, SUC_lexicon_tags_12 },
    { 0x903bc86c, SUC_lexicon_tags_27 },
    { 0x7b3bc86d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7bc873, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebbc878, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf81bc87a, SUC_lexicon_tags_61 },
    { 0xcd5bc87b, SUC_lexicon_tags_7 },
    { 0xc31bc87c, SUC_lexicon_tags_7 },
    { 0x877bc87a, SUC_lexicon_tags_25 },
    { 0x9b1bc87c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e1bc880, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fbc883, SUC_lexicon_tags_15 },
    { 0xd57bc884, SUC_lexicon_tags_45 },
    { 0x9cfbc885, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x451bc888, SUC_lexicon_tags_25 },
    { 0xa53bc888, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bbc88c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac5bc88e, SUC_lexicon_tags_185 },
    { 0x29dbc88f, SUC_lexicon_tags_15 },
    { 0xf0dbc890, SUC_lexicon_tags_7 },
    { 0xb71bc891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33bc894, SUC_lexicon_tags_17 },
    { 0x83dbc895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fbc899, SUC_lexicon_tags_39 },
    { 0xc23bc89a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb57bc89c, SUC_lexicon_tags_43 },
    { 0xfc5bc89d, SUC_lexicon_tags_48 },
    { 0xc2dbc89c, SUC_lexicon_tags_26 },
    { 0x831bc89f, SUC_lexicon_tags_11 },
    { 0x153bc8a0, SUC_lexicon_tags_35 },
    { 0xc11bc8a0, SUC_lexicon_tags_17 },
    { 0xf97bc8a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7bc8a6, SUC_lexicon_tags_37 },
    { 0x365bc8a7, SUC_lexicon_tags_35 },
    { 0x1b5bc8a8, SUC_lexicon_tags_12 },
    { 0xeb5bc8a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe71bc8ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fbc8ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x793bc8b1, SUC_lexicon_tags_19 },
    { 0xe2bbc8b2, SUC_lexicon_tags_25 },
    { 0x9bfbc8b2, SUC_lexicon_tags_66 },
    { 0x173bc8b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dbc8c1, SUC_lexicon_tags_49 },
    { 0x725bc8c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x563bc8c4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x24fbc8c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13bc8c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41bc8cb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561bc8d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x533bc8d6, SUC_lexicon_tags_33 },
    { 0x9f9bc8d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48fbc8e0, SUC_lexicon_tags_10 },
    { 0xaf5bc8e1, SUC_lexicon_tags_38 },
    { 0x38bbc8e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1ffbc8e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fbc8ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x893bc8f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23bc8f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf07bc8fa, SUC_lexicon_tags_10 },
    { 0x84bbc8fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x15dbc8fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cdbc900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3bc903, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9bc90a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909bc911, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xd33bc913, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x58dbc915, SUC_lexicon_tags_46 },
    { 0xcafbc916, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91bc91f, SUC_lexicon_tags_1 },
    { 0x7dbbc920, SUC_lexicon_tags_70 },
    { 0x4ddbc921, SUC_lexicon_tags_1 },
    { 0xa6bbc922, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1bc92b, SUC_lexicon_tags_12 },
    { 0xe7bc92c, SUC_lexicon_tags_12 },
    { 0x11bc92c, SUC_lexicon_tags_19 },
    { 0x8adbc92e, SUC_lexicon_tags_11 },
    { 0xe11bc92f, SUC_lexicon_tags_57 },
    { 0x7c1bc92f, SUC_lexicon_tags_63 },
    { 0xbafbc931, SUC_lexicon_tags_37 },
    { 0x469bc931, SUC_lexicon_tags_37 },
    { 0x1c3bc933, SUC_lexicon_tags_14 },
    { 0xb85bc934, SUC_lexicon_tags_408 },
    { 0xc0fbc935, SUC_lexicon_tags_7 },
    { 0x7f9bc931, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fbc93a, SUC_lexicon_tags_77 },
    { 0x687bc93b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc35bc93d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x343bc940, SUC_lexicon_tags_87 },
    { 0x7bbbc941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0fbc943, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bbc94b, SUC_lexicon_tags_65 },
    { 0xa7dbc94c, SUC_lexicon_tags_103 },
    { 0xdc9bc94d, SUC_lexicon_tags_38 },
    { 0x92dbc94c, SUC_lexicon_tags_17 },
    { 0xb5bc94f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x15dbc952, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4c1bc954, SUC_lexicon_tags_43 },
    { 0x9c9bc955, SUC_lexicon_tags_47 },
    { 0x4b3bc956, SUC_lexicon_tags_10 },
    { 0x939bc956, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d3bc959, SUC_lexicon_tags_15 },
    { 0xc41bc95a, SUC_lexicon_tags_26 },
    { 0xa89bc95a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x649bc95d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58dbc964, SUC_lexicon_tags_6 },
    { 0x6b9bc965, SUC_lexicon_tags_12 },
    { 0xa1bc966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d5bc968, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf61bc96a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fbc96f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda3bc971, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bbc974, SUC_lexicon_tags_48 },
    { 0x311bc975, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5adbc977, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bbc97b, SUC_lexicon_tags_27 },
    { 0xd0fbc97c, SUC_lexicon_tags_631 },
    { 0xef1bc97c, SUC_lexicon_tags_27 },
    { 0x35bc97e, SUC_lexicon_tags_0 },
    { 0x967bc97d, SUC_lexicon_tags_11 },
    { 0x889bc97c, SUC_lexicon_tags_37 },
    { 0xb45bc981, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfe1bc983, SUC_lexicon_tags_18 },
    { 0x10dbc984, SUC_lexicon_tags_17 },
    { 0xb6dbc985, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfbc98c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x633bc98e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x409bc991, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9dbc993, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfbc996, SUC_lexicon_tags_11 },
    { 0xb63bc996, SUC_lexicon_tags_15 },
    { 0x341bc998, SUC_lexicon_tags_17 },
    { 0xe2fbc998, SUC_lexicon_tags_26 },
    { 0xdf3bc99a, SUC_lexicon_tags_1 },
    { 0xf41bc99a, SUC_lexicon_tags_25 },
    { 0xd6bbc99a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebbc99f, SUC_lexicon_tags_15 },
    { 0x97dbc9a0, SUC_lexicon_tags_16 },
    { 0x851bc9a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x825bc9a3, SUC_lexicon_tags_17 },
    { 0x13bc9a4, SUC_lexicon_tags_1 },
    { 0xf4fbc9a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x813bc9a7, SUC_lexicon_tags_37 },
    { 0x9b3bc9a8, SUC_lexicon_tags_12 },
    { 0xf53bc9a9, SUC_lexicon_tags_17 },
    { 0xe75bc9a9, SUC_lexicon_tags_7 },
    { 0xbc1bc9a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7bc9ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03bc9b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441bc9b4, SUC_lexicon_tags_2 },
    { 0x629bc9b5, SUC_lexicon_tags_27 },
    { 0x917bc9b6, SUC_lexicon_tags_31 },
    { 0x29bbc9b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafbc9ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x70bbc9bc, SUC_lexicon_tags_4 },
    { 0xe29bc9bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbbc9c4, SUC_lexicon_tags_12 },
    { 0xcd7bc9c5, SUC_lexicon_tags_25 },
    { 0x7d7bc9c6, SUC_lexicon_tags_16 },
    { 0x421bc9c7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x25bbc9c9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1bc9cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb1bbc9d1, SUC_lexicon_tags_14 },
    { 0x743bc9d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdbc9d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69bc9db, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xebbc9dd, SUC_lexicon_tags_17 },
    { 0xfa9bc9de, SUC_lexicon_tags_25 },
    { 0xba1bc9df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167bc9e3, SUC_lexicon_tags_27 },
    { 0xc45bc9e4, SUC_lexicon_tags_23 },
    { 0x897bc9e5, SUC_lexicon_tags_127 },
    { 0x1ebbc9e6, SUC_lexicon_tags_37 },
    { 0x8b9bc9e7, SUC_lexicon_tags_23 },
    { 0xf0fbc9e8, SUC_lexicon_tags_37 },
    { 0x801bc9e9, SUC_lexicon_tags_0 },
    { 0x93bbc9ea, SUC_lexicon_tags_11 },
    { 0x9bdbc9eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5bdbc9ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x563bc9f0, SUC_lexicon_tags_66 },
    { 0xc9dbc9f1, SUC_lexicon_tags_17 },
    { 0x479bc9f2, SUC_lexicon_tags_8 },
    { 0x3d7bc9f3, SUC_lexicon_tags_85 },
    { 0xcfbc9f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebbc9f7, SUC_lexicon_tags_12 },
    { 0xe1fbc9f8, SUC_lexicon_tags_9 },
    { 0xf33bc9f9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1bc9fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429bca01, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebbca05, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbabbca07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x157bca0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x671bca0c, SUC_lexicon_tags_55 },
    { 0x9d3bca0d, SUC_lexicon_tags_13 },
    { 0x833bca0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ebbca10, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x711bca12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfbca17, SUC_lexicon_tags_1 },
    { 0x5e1bca18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x469bca1a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfbca1f, SUC_lexicon_tags_27 },
    { 0xde1bca20, SUC_lexicon_tags_26 },
    { 0xe15bca1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbe7bca23, SUC_lexicon_tags_25 },
    { 0x899bca23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703bca2e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19bca39, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xe71bca3b, SUC_lexicon_tags_11 },
    { 0xd45bca3c, SUC_lexicon_tags_25 },
    { 0xd01bca3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bca42, SUC_lexicon_tags_30 },
    { 0x23bbca42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb19bca45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bbca49, SUC_lexicon_tags_25 },
    { 0xecbbca49, SUC_lexicon_tags_14 },
    { 0x44dbca4b, SUC_lexicon_tags_14 },
    { 0xfc9bca49, SUC_lexicon_tags_17 },
    { 0xa2dbca4a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639bca54, SUC_lexicon_tags_25 },
    { 0xfbbca54, SUC_lexicon_tags_25 },
    { 0xc6fbca56, SUC_lexicon_tags_17 },
    { 0x8d7bca57, SUC_lexicon_tags_33 },
    { 0xed9bca58, SUC_lexicon_tags_45 },
    { 0x6a9bca59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bbca5e, SUC_lexicon_tags_15 },
    { 0x3d5bca5f, SUC_lexicon_tags_0 },
    { 0xaabbca60, SUC_lexicon_tags_17 },
    { 0x98bbca61, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b7bca63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcc9bca65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbbca6a, SUC_lexicon_tags_15 },
    { 0x30dbca6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dbca70, SUC_lexicon_tags_44 },
    { 0x531bca70, SUC_lexicon_tags_47 },
    { 0x55dbca72, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15bca77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1bca7a, SUC_lexicon_tags_31 },
    { 0x21bca7b, SUC_lexicon_tags_8 },
    { 0x72dbca7c, SUC_lexicon_tags_28 },
    { 0x559bca7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417bca82, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x837bca84, SUC_lexicon_tags_11 },
    { 0x64dbca84, SUC_lexicon_tags_11 },
    { 0x849bca85, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x41bbca88, SUC_lexicon_tags_28 },
    { 0xa63bca89, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47bca8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0dbca8e, SUC_lexicon_tags_17 },
    { 0x9e9bca8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbffbca91, SUC_lexicon_tags_0 },
    { 0xde1bca91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9a1bca94, SUC_lexicon_tags_1 },
    { 0xadbca94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1bca99, SUC_lexicon_tags_1 },
    { 0x92dbca9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x845bca9d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcbfbca9f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x527bcaa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19bcaae, SUC_lexicon_tags_2 },
    { 0xdefbcaaf, SUC_lexicon_tags_11 },
    { 0xfbbbcaaf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153bcab6, SUC_lexicon_tags_12 },
    { 0xc59bcab6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3bcabb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5bcac0, SUC_lexicon_tags_15 },
    { 0x1cdbcac1, SUC_lexicon_tags_37 },
    { 0xfd9bcac1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37bcacb, SUC_lexicon_tags_4 },
    { 0x58fbcacb, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fbcad1, SUC_lexicon_tags_47 },
    { 0xec9bcad2, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0x175bcad5, SUC_lexicon_tags_21 },
    { 0x6afbcad6, SUC_lexicon_tags_31 },
    { 0x553bcad5, SUC_lexicon_tags_3 },
    { 0xe1dbcad8, SUC_lexicon_tags_25 },
    { 0xe3bbcad9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x831bcadb, SUC_lexicon_tags_49 },
    { 0xe05bcadb, SUC_lexicon_tags_42 },
    { 0x1a9bcadd, SUC_lexicon_tags_14 },
    { 0x8cbbcadd, SUC_lexicon_tags_1 },
    { 0xe3fbcadf, SUC_lexicon_tags_30 },
    { 0xad7bcae0, SUC_lexicon_tags_17 },
    { 0x4fdbcae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cdbcae3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bcae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddbcaea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3bcaee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd33bcaf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9d9bcaf2, SUC_lexicon_tags_12 },
    { 0xd85bcaf2, SUC_lexicon_tags_27 },
    { 0xbf7bcaf4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855bcafe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bbcb01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789bcb09, SUC_lexicon_tags_17 },
    { 0x68bbcb09, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x681bcb0d, SUC_lexicon_tags_4 },
    { 0x7d3bcb0e, SUC_lexicon_tags_29 },
    { 0x9f3bcb0f, SUC_lexicon_tags_37 },
    { 0xd81bcb0e, SUC_lexicon_tags_37 },
    { 0x969bcb0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05bcb19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7bcb1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x803bcb1f, SUC_lexicon_tags_47 },
    { 0xe9bcb1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421bcb24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x327bcb27, SUC_lexicon_tags_1 },
    { 0x33fbcb28, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x957bcb2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369bcb2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fbcb32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9bcb36, SUC_lexicon_tags_11 },
    { 0xe63bcb37, SUC_lexicon_tags_23 },
    { 0xdbdbcb37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35bcb3c, SUC_lexicon_tags_16 },
    { 0x6bdbcb3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x739bcb40, SUC_lexicon_tags_47 },
    { 0xaf3bcb41, SUC_lexicon_tags_2 },
    { 0xbb3bcb41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789bcb46, SUC_lexicon_tags_37 },
    { 0x7ebbcb46, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5bcb4a, SUC_lexicon_tags_1 },
    { 0x367bcb4b, SUC_lexicon_tags_45 },
    { 0x683bcb4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9bcb53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x67dbcb55, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2e7bcb57, SUC_lexicon_tags_37 },
    { 0xd1fbcb57, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1bbcb5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9bcb5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1bcb5f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbbcb62, SUC_lexicon_tags_46 },
    { 0xa61bcb63, SUC_lexicon_tags_2 },
    { 0x507bcb62, SUC_lexicon_tags_12 },
    { 0xd69bcb63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1bcb6a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9bcb73, SUC_lexicon_tags_11 },
    { 0xbfdbcb74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf2fbcb76, SUC_lexicon_tags_17 },
    { 0xacfbcb77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fbcb7b, SUC_lexicon_tags_23 },
    { 0x351bcb7b, SUC_lexicon_tags_63 },
    { 0xfcbbcb7c, SUC_lexicon_tags_31 },
    { 0x8e3bcb7e, SUC_lexicon_tags_39 },
    { 0xfa5bcb7b, SUC_lexicon_tags_17 },
    { 0xa35bcb80, SUC_lexicon_tags_21 },
    { 0x907bcb80, SUC_lexicon_tags_14 },
    { 0xf5bbcb82, SUC_lexicon_tags_48 },
    { 0xd43bcb83, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xee1bcb85, SUC_lexicon_tags_53 },
    { 0x413bcb86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9bcb89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8c5bcb8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11bcb8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc9bcb90, SUC_lexicon_tags_49 },
    { 0x1edbcb91, SUC_lexicon_tags_8 },
    { 0x495bcb92, SUC_lexicon_tags_37 },
    { 0x59bbcb92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5bcb98, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc51bcb9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdbcb9d, SUC_lexicon_tags_0 },
    { 0xb1bbcb9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1bcba3, SUC_lexicon_tags_63 },
    { 0x541bcba4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5f3bcba6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd95bcba8, SUC_lexicon_tags_10 },
    { 0x90fbcba9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35bcbac, SUC_lexicon_tags_23 },
    { 0x45fbcbac, SUC_lexicon_tags_15 },
    { 0x83bcbae, SUC_lexicon_tags_17 },
    { 0x42bbcbaf, SUC_lexicon_tags_8 },
    { 0xc5dbcbb0, SUC_lexicon_tags_14 },
    { 0xb5bbcbad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bcbb4, SUC_lexicon_tags_87 },
    { 0x10dbcbb5, SUC_lexicon_tags_25 },
    { 0xba3bcbb5, SUC_lexicon_tags_29 },
    { 0xcc3bcbb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x961bcbb9, SUC_lexicon_tags_25 },
    { 0x46bbcbba, SUC_lexicon_tags_4 },
    { 0xd9dbcbbb, SUC_lexicon_tags_25 },
    { 0x449bcbba, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x52dbcbbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x607bcbc1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7bcbc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x485bcbc7, SUC_lexicon_tags_37 },
    { 0x683bcbc8, SUC_lexicon_tags_37 },
    { 0xce7bcbc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9bcbd0, SUC_lexicon_tags_1 },
    { 0x7edbcbd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1b9bcbd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bbcbda, SUC_lexicon_tags_11 },
    { 0xc41bcbdb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x651bcbdd, SUC_lexicon_tags_49 },
    { 0x8b9bcbdd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bbcbe2, SUC_lexicon_tags_10 },
    { 0x6d7bcbe3, SUC_lexicon_tags_39 },
    { 0x5cbbcbe4, SUC_lexicon_tags_26 },
    { 0xf55bcbe5, SUC_lexicon_tags_17 },
    { 0x381bcbe3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x16dbcbe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3bcbeb, SUC_lexicon_tags_15 },
    { 0xafbcbec, SUC_lexicon_tags_11 },
    { 0xd87bcbed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9bcbef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x697bcbf1, SUC_lexicon_tags_15 },
    { 0xba7bcbf1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adbcbf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe91bcbf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x789bcbfc, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9bcc01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfe5bcc03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299bcc08, SUC_lexicon_tags_49 },
    { 0xbb7bcc08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd6bbcc0b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5bcc0e, SUC_lexicon_tags_11 },
    { 0x845bcc0f, SUC_lexicon_tags_77 },
    { 0xcadbcc0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbbcc15, SUC_lexicon_tags_33 },
    { 0x35bbcc15, SUC_lexicon_tags_17 },
    { 0x3a5bcc17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9a5bcc19, SUC_lexicon_tags_16 },
    { 0x321bcc1a, SUC_lexicon_tags_2 },
    { 0x251bcc1b, SUC_lexicon_tags_25 },
    { 0xee9bcc1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x639bcc1e, SUC_lexicon_tags_23 },
    { 0x3e3bcc1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbcc22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3bcc2b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fbcc31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb81bcc33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331bcc3b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adbcc40, SUC_lexicon_tags_25 },
    { 0xc19bcc41, SUC_lexicon_tags_82 },
    { 0xab5bcc41, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc0fbcc44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9bcc48, SUC_lexicon_tags_11 },
    { 0xb45bcc48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475bcc4d, SUC_lexicon_tags_23 },
    { 0xa09bcc4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x61dbcc50, SUC_lexicon_tags_25 },
    { 0x495bcc50, SUC_lexicon_tags_23 },
    { 0x583bcc52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb17bcc54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69bcc57, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebbcc5d, SUC_lexicon_tags_16 },
    { 0x6b3bcc5d, SUC_lexicon_tags_33 },
    { 0xae9bcc5f, SUC_lexicon_tags_14 },
    { 0xa71bcc60, SUC_lexicon_tags_32 },
    { 0xf35bcc61, SUC_lexicon_tags_15 },
    { 0x931bcc5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41bcc65, SUC_lexicon_tags_11 },
    { 0x887bcc66, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27bcc6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fbcc70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe99bcc72, SUC_lexicon_tags_60 },
    { 0xd19bcc73, SUC_lexicon_tags_68 },
    { 0xd51bcc73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169bcc78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9bcc7f, SUC_lexicon_tags_47 },
    { 0x1b5bcc80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403bcc87, SUC_lexicon_tags_14 },
    { 0x90bbcc87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa9bcc8a, SUC_lexicon_tags_25 },
    { 0xbbfbcc8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29dbcc8d, SUC_lexicon_tags_37 },
    { 0x52fbcc8e, SUC_lexicon_tags_109 },
    { 0x899bcc8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x751bcc92, SUC_lexicon_tags_37 },
    { 0xb07bcc93, SUC_lexicon_tags_37 },
    { 0x6ffbcc92, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fbcc9a, SUC_lexicon_tags_27 },
    { 0x441bcc9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe81bcc9d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffbcca0, SUC_lexicon_tags_17 },
    { 0xbb3bcca1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9bcca6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7edbcca8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bbccae, SUC_lexicon_tags_12 },
    { 0x3ebbccaf, SUC_lexicon_tags_26 },
    { 0xe17bccb0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x971bccb2, SUC_lexicon_tags_2 },
    { 0x56fbccb3, SUC_lexicon_tags_25 },
    { 0xc11bccb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1bccb9, SUC_lexicon_tags_60 },
    { 0xc73bccba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709bccbe, SUC_lexicon_tags_45 },
    { 0x6fdbccbf, SUC_lexicon_tags_27 },
    { 0x843bccc0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fbccc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdadbccc5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xacdbccc7, SUC_lexicon_tags_25 },
    { 0xba9bccc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267bcccd, SUC_lexicon_tags_38 },
    { 0x499bccce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x513bccd0, SUC_lexicon_tags_31 },
    { 0x943bccd1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x38dbccd4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9bccd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbccdd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x61bbcce0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x385bcce3, SUC_lexicon_tags_37 },
    { 0xf49bcce4, SUC_lexicon_tags_37 },
    { 0x757bcce5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x92fbcce7, SUC_lexicon_tags_37 },
    { 0x773bcce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfbccec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbbccef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79bccf2, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xf6bbccf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7bccf8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x611bccfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa43bccfc, SUC_lexicon_tags_30 },
    { 0x56bbccfd, SUC_lexicon_tags_83 },
    { 0x409bccfe, SUC_lexicon_tags_11 },
    { 0x367bccff, SUC_lexicon_tags_25 },
    { 0x529bccff, SUC_lexicon_tags_39 },
    { 0x8cbbcd01, SUC_lexicon_tags_28 },
    { 0xaf7bccfe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773bcd06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1bcd0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ebbcd0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfbcd0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x875bcd11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dbcd15, SUC_lexicon_tags_35 },
    { 0xbefbcd15, SUC_lexicon_tags_17 },
    { 0xd49bcd17, SUC_lexicon_tags_17 },
    { 0x11bbcd17, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fbcd1d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x68bbcd1f, SUC_lexicon_tags_17 },
    { 0xa6dbcd1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x303bcd23, SUC_lexicon_tags_146 },
    { 0x9e5bcd24, SUC_lexicon_tags_7 },
    { 0xfbcd24, SUC_lexicon_tags_2 },
    { 0xf15bcd24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3bcd2d, SUC_lexicon_tags_33 },
    { 0x497bcd2d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3bcd3d, SUC_lexicon_tags_17 },
    { 0x467bcd3e, SUC_lexicon_tags_14 },
    { 0x1c3bcd3f, SUC_lexicon_tags_46 },
    { 0x369bcd3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29bcd46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x991bcd48, SUC_lexicon_tags_43 },
    { 0xc67bcd48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69bcd4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x969bcd4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27bcd51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb89bcd53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x235bcd56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fbcd5b, SUC_lexicon_tags_66 },
    { 0x537bcd5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe63bcd5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1bcd66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x925bcd68, SUC_lexicon_tags_37 },
    { 0x579bcd69, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x235bcd6b, SUC_lexicon_tags_63 },
    { 0x807bcd6c, SUC_lexicon_tags_12 },
    { 0xf83bcd6d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63bcd70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x73bcd72, SUC_lexicon_tags_30 },
    { 0x393bcd73, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe43bcd75, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbbcd7d, SUC_lexicon_tags_17 },
    { 0x425bcd7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245bcd8a, SUC_lexicon_tags_87 },
    { 0xbe1bcd8b, SUC_lexicon_tags_0 },
    { 0xc45bcd8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x143bcd8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bbcd92, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bbcd96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fbcd9a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3bcd9c, SUC_lexicon_tags_47 },
    { 0xfebbcd9c, SUC_lexicon_tags_23 },
    { 0xc17bcd9e, SUC_lexicon_tags_31 },
    { 0x119bcd9e, SUC_lexicon_tags_30 },
    { 0x45bbcd9c, SUC_lexicon_tags_296 },
    { 0xd8fbcda1, SUC_lexicon_tags_16 },
    { 0xe71bcd9f, SUC_lexicon_tags_15 },
    { 0xfd7bcda3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdfbcda5, SUC_lexicon_tags_17 },
    { 0xeafbcda6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5bcda9, SUC_lexicon_tags_138 },
    { 0x20fbcdaa, SUC_lexicon_tags_27 },
    { 0x70bbcda9, SUC_lexicon_tags_35 },
    { 0x57bcdac, SUC_lexicon_tags_33 },
    { 0x841bcdaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7bcdb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x85dbcdb5, SUC_lexicon_tags_15 },
    { 0x50fbcdb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe13bcdb8, SUC_lexicon_tags_17 },
    { 0x64dbcdb9, SUC_lexicon_tags_19 },
    { 0xee7bcdb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835bcdc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x873bcdc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbbcdc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x819bcdcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5bcdd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x793bcdd7, SUC_lexicon_tags_19 },
    { 0x12fbcdd8, SUC_lexicon_tags_7 },
    { 0xa6fbcdd9, SUC_lexicon_tags_61 },
    { 0x47fbcdda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3bcde7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9dbcde9, SUC_lexicon_tags_33 },
    { 0x51bbcdea, SUC_lexicon_tags_30 },
    { 0x2d3bcde9, SUC_lexicon_tags_11 },
    { 0x5dbcdec, SUC_lexicon_tags_0 },
    { 0xe85bcdec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88fbcdef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877bcdf3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x553bcdf5, SUC_lexicon_tags_14 },
    { 0x4b1bcdf6, SUC_lexicon_tags_12 },
    { 0xbb7bcdf6, SUC_lexicon_tags_2 },
    { 0x983bcdf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fbcdfb, SUC_lexicon_tags_17 },
    { 0x47bcdfc, SUC_lexicon_tags_25 },
    { 0x4fbcdfd, SUC_lexicon_tags_2 },
    { 0x509bcdfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1bce02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9bce05, SUC_lexicon_tags_49 },
    { 0x2bbce06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47fbce08, SUC_lexicon_tags_15 },
    { 0x1bdbce09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xadfbce0c, SUC_lexicon_tags_28 },
    { 0x621bce0d, SUC_lexicon_tags_2 },
    { 0x48fbce0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x859bce10, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345bce15, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xf2bbce17, SUC_lexicon_tags_15 },
    { 0x5fbbce18, SUC_lexicon_tags_2 },
    { 0x69dbce17, SUC_lexicon_tags_2 },
    { 0x585bce1a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe9dbce1c, SUC_lexicon_tags_47 },
    { 0xad9bce1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995bce29, SUC_lexicon_tags_11 },
    { 0x1dbbce29, SUC_lexicon_tags_1 },
    { 0xd71bce2b, SUC_lexicon_tags_27 },
    { 0xeb5bce29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb0dbce2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbbce33, SUC_lexicon_tags_1 },
    { 0x861bce34, SUC_lexicon_tags_63 },
    { 0xf57bce34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19bce3b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x715bce3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fbce41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xccbbce43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bce51, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1bce54, SUC_lexicon_tags_60 },
    { 0x4dbbce55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fbce58, SUC_lexicon_tags_505 },
    { 0xbbbce59, SUC_lexicon_tags_12 },
    { 0x93dbce58, SUC_lexicon_tags_14 },
    { 0xb1bbce59, SUC_lexicon_tags_45 },
    { 0x7e1bce5c, SUC_lexicon_tags_15 },
    { 0x87fbce5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedbbce5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3bce65, SUC_lexicon_tags_40 },
    { 0xd71bce66, SUC_lexicon_tags_25 },
    { 0x4dbce67, SUC_lexicon_tags_32 },
    { 0xa25bce67, SUC_lexicon_tags_71 },
    { 0xc7bce69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x727bce6b, SUC_lexicon_tags_68 },
    { 0xb51bce6c, SUC_lexicon_tags_9 },
    { 0x15fbce6b, SUC_lexicon_tags_12 },
    { 0xd0fbce6c, SUC_lexicon_tags_8 },
    { 0x1c9bce6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90bbce72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861bce77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41bce7c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x227bce7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5bce83, SUC_lexicon_tags_17 },
    { 0x4b7bce84, SUC_lexicon_tags_60 },
    { 0x36dbce85, SUC_lexicon_tags_27 },
    { 0x475bce85, SUC_lexicon_tags_25 },
    { 0xb61bce84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x245bce8a, SUC_lexicon_tags_27 },
    { 0x373bce8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923bce91, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33bce95, SUC_lexicon_tags_66 },
    { 0x573bce95, SUC_lexicon_tags_42 },
    { 0x9edbce96, SUC_lexicon_tags_21 },
    { 0x46dbce95, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255bce9c, SUC_lexicon_tags_11 },
    { 0x20fbce9d, SUC_lexicon_tags_38 },
    { 0x1fdbce9c, SUC_lexicon_tags_12 },
    { 0xf41bce9c, SUC_lexicon_tags_31 },
    { 0x207bcea0, SUC_lexicon_tags_23 },
    { 0x519bcea1, SUC_lexicon_tags_2 },
    { 0xc67bce9d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fbcea5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfc7bcea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1c9bcea9, SUC_lexicon_tags_12 },
    { 0xe1fbceaa, SUC_lexicon_tags_0 },
    { 0x4b5bceab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf3bcead, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bceb1, SUC_lexicon_tags_41 },
    { 0xedfbceb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x253bceb4, SUC_lexicon_tags_12 },
    { 0x151bceb5, SUC_lexicon_tags_55 },
    { 0xf85bceb6, SUC_lexicon_tags_1 },
    { 0x3d9bceb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c3bceb9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x409bcebb, SUC_lexicon_tags_14 },
    { 0x9d7bcebb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x309bcebf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbcec3, SUC_lexicon_tags_37 },
    { 0x523bcec4, SUC_lexicon_tags_25 },
    { 0xbabbcec4, SUC_lexicon_tags_32 },
    { 0xb8dbcec6, SUC_lexicon_tags_25 },
    { 0x787bcec6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97dbcec9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x603bcecb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x79bcecd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5bced6, SUC_lexicon_tags_47 },
    { 0xb7bbced6, SUC_lexicon_tags_2 },
    { 0x5f5bced7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfffbceda, SUC_lexicon_tags_85 },
    { 0x8b9bceda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17bcede, SUC_lexicon_tags_12 },
    { 0x12dbcedf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x777bcee2, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xc1dbcee4, SUC_lexicon_tags_1 },
    { 0xd5dbcee5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x337bcee7, SUC_lexicon_tags_16 },
    { 0x6c3bcee8, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xec3bceea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bbcef0, SUC_lexicon_tags_31 },
    { 0xc45bcef0, SUC_lexicon_tags_66 },
    { 0x7b7bcef2, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xa69bcef4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeadbcef6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1bcefb, SUC_lexicon_tags_2 },
    { 0xdb1bcefc, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0x847bceff, SUC_lexicon_tags_27 },
    { 0x9c5bceff, SUC_lexicon_tags_27 },
    { 0x3fdbcf01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e3bcf03, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795bcf07, SUC_lexicon_tags_123 },
    { 0x9f3bcf07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x837bcf0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efbcf0e, SUC_lexicon_tags_1 },
    { 0xeddbcf0e, SUC_lexicon_tags_12 },
    { 0x89bcf0f, SUC_lexicon_tags_3 },
    { 0x27bcf10, SUC_lexicon_tags_19 },
    { 0x429bcf0f, SUC_lexicon_tags_16 },
    { 0x6f9bcf12, SUC_lexicon_tags_28 },
    { 0x7dbcf14, SUC_lexicon_tags_62 },
    { 0x5bfbcf10, SUC_lexicon_tags_14 },
    { 0xc95bcf14, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87bcf20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfbcf23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9bcf30, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3bcf3a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbcf3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17bcf41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f1bcf43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7fbcf45, SUC_lexicon_tags_41 },
    { 0xfefbcf46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fbcf4b, SUC_lexicon_tags_11 },
    { 0xf57bcf4c, SUC_lexicon_tags_1 },
    { 0xedbcf4d, SUC_lexicon_tags_0 },
    { 0xfd7bcf4e, SUC_lexicon_tags_31 },
    { 0x31bcf4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8fbcf51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f1bcf53, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x88bbcf55, SUC_lexicon_tags_53 },
    { 0x575bcf56, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x409bcf58, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x31dbcf5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbbcf5d, SUC_lexicon_tags_17 },
    { 0x4afbcf5d, SUC_lexicon_tags_0 },
    { 0x231bcf5f, SUC_lexicon_tags_1 },
    { 0xa65bcf5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fbcf64, SUC_lexicon_tags_2 },
    { 0x771bcf65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x479bcf67, SUC_lexicon_tags_48 },
    { 0x6afbcf68, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07bcf6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x827bcf6e, SUC_lexicon_tags_38 },
    { 0xe5fbcf6f, SUC_lexicon_tags_11 },
    { 0xfdbcf70, SUC_lexicon_tags_92 },
    { 0x20bbcf71, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71bcf75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbbcf78, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bbcf7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbbcf82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x55dbcf84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5bcf8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7bcf92, SUC_lexicon_tags_17 },
    { 0x99bbcf93, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1bcf98, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x609bcf9a, SUC_lexicon_tags_63 },
    { 0x999bcf9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x231bcf9d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xa49bcf9f, SUC_lexicon_tags_2 },
    { 0x809bcfa0, SUC_lexicon_tags_27 },
    { 0xf37bcfa0, SUC_lexicon_tags_1 },
    { 0xd13bcfa2, SUC_lexicon_tags_82 },
    { 0xfd9bcfa2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dbcfa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x519bcfa8, SUC_lexicon_tags_1 },
    { 0xff5bcfa9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x127bcfab, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bbcfaf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29bcfb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd59bcfb4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x491bcfb7, SUC_lexicon_tags_77 },
    { 0xe3dbcfb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fbcfbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dbcfc0, SUC_lexicon_tags_63 },
    { 0xcfbcfc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5bcfc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe2bbcfc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79bcfce, SUC_lexicon_tags_38 },
    { 0xd4dbcfcf, SUC_lexicon_tags_17 },
    { 0xfc3bcfd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2fbcfd2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x10dbcfd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dbcfd8, SUC_lexicon_tags_33 },
    { 0xd2dbcfd9, SUC_lexicon_tags_15 },
    { 0x2a9bcfda, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x845bcfdc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7bcfe0, SUC_lexicon_tags_14 },
    { 0x7b1bcfe0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf75bcfe3, SUC_lexicon_tags_12 },
    { 0xf63bcfe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeffbcfe6, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0x511bcfe8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30fbcfea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfbcfee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5bcff2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91bcff5, SUC_lexicon_tags_40 },
    { 0x7c3bcff5, SUC_lexicon_tags_47 },
    { 0x3fbcff7, SUC_lexicon_tags_11 },
    { 0xcabbcff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x441bcffb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd002, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x825bd005, SUC_lexicon_tags_55 },
    { 0x103bd006, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb65bd008, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x483bd00a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dbd015, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb71bd017, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19bd01a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03bd01d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fbd025, SUC_lexicon_tags_25 },
    { 0xda1bd026, SUC_lexicon_tags_0 },
    { 0xc71bd025, SUC_lexicon_tags_12 },
    { 0x591bd027, SUC_lexicon_tags_55 },
    { 0x529bd027, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdf9bd02b, SUC_lexicon_tags_23 },
    { 0x6c9bd02c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dbd030, SUC_lexicon_tags_43 },
    { 0x9f7bd031, SUC_lexicon_tags_3 },
    { 0xe83bd032, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x591bd035, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x695bd037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdbd03d, SUC_lexicon_tags_56 },
    { 0x585bd03e, SUC_lexicon_tags_17 },
    { 0x891bd03f, SUC_lexicon_tags_16 },
    { 0xbafbd03e, SUC_lexicon_tags_1 },
    { 0x73fbd041, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xccfbd043, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x87dbd045, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bd048, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bd04b, SUC_lexicon_tags_18 },
    { 0xef5bd04b, SUC_lexicon_tags_65 },
    { 0x333bd04d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85dbd050, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xac1bd052, SUC_lexicon_tags_27 },
    { 0xedbbd053, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3bd057, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bbd05b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfbd05e, SUC_lexicon_tags_87 },
    { 0x1b3bd05e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fbd067, SUC_lexicon_tags_7 },
    { 0xf89bd068, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857bd06c, SUC_lexicon_tags_7 },
    { 0xebbbd06d, SUC_lexicon_tags_16 },
    { 0xe3dbd06d, SUC_lexicon_tags_47 },
    { 0x77bbd06c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1bd072, SUC_lexicon_tags_27 },
    { 0x379bd073, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7bd076, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6fbd078, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x217bd07a, SUC_lexicon_tags_101 },
    { 0x6dbd07b, SUC_lexicon_tags_38 },
    { 0x5fbbd07b, SUC_lexicon_tags_1 },
    { 0xfd5bd07d, SUC_lexicon_tags_37 },
    { 0x89fbd07c, SUC_lexicon_tags_172 },
    { 0x16fbd07f, SUC_lexicon_tags_17 },
    { 0xb37bd080, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8ebbd082, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bbd085, SUC_lexicon_tags_79 },
    { 0x833bd086, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435bd08c, SUC_lexicon_tags_11 },
    { 0xda1bd08c, SUC_lexicon_tags_0 },
    { 0x1a7bd08e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9bd091, SUC_lexicon_tags_63 },
    { 0x5c9bd092, SUC_lexicon_tags_48 },
    { 0x56bbd092, SUC_lexicon_tags_45 },
    { 0x1e1bd094, SUC_lexicon_tags_23 },
    { 0x5ebbd091, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d9bd097, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419bd09c, SUC_lexicon_tags_33 },
    { 0x41bd09c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1bd0a0, SUC_lexicon_tags_29 },
    { 0x431bd0a1, SUC_lexicon_tags_15 },
    { 0xd83bd0a1, SUC_lexicon_tags_179 },
    { 0x215bd0a3, SUC_lexicon_tags_37 },
    { 0x17bd0a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6f7bd0a6, SUC_lexicon_tags_17 },
    { 0xa73bd0a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fbd0ab, SUC_lexicon_tags_1 },
    { 0xd9bd0ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb1bd0ae, SUC_lexicon_tags_2 },
    { 0xeafbd0af, SUC_lexicon_tags_63 },
    { 0x613bd0b0, SUC_lexicon_tags_71 },
    { 0x62bbd0b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01bd0b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9bd0b7, SUC_lexicon_tags_47 },
    { 0xd7bbd0b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe53bd0ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebbd0bd, SUC_lexicon_tags_27 },
    { 0xf6dbd0bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fbd0c2, SUC_lexicon_tags_14 },
    { 0x22fbd0c3, SUC_lexicon_tags_37 },
    { 0x38dbd0c2, SUC_lexicon_tags_17 },
    { 0x9c1bd0c2, SUC_lexicon_tags_25 },
    { 0xd31bd0c3, SUC_lexicon_tags_25 },
    { 0x37fbd0c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9bd0ce, SUC_lexicon_tags_47 },
    { 0x77dbd0cf, SUC_lexicon_tags_38 },
    { 0x43bd0ce, SUC_lexicon_tags_31 },
    { 0xb29bd0d0, SUC_lexicon_tags_0 },
    { 0x2a3bd0cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bbd0d8, SUC_lexicon_tags_45 },
    { 0x24fbd0d9, SUC_lexicon_tags_25 },
    { 0x7dbbd0da, SUC_lexicon_tags_0 },
    { 0x3b7bd0db, SUC_lexicon_tags_7 },
    { 0x15fbd0dc, SUC_lexicon_tags_14 },
    { 0x563bd0dd, SUC_lexicon_tags_12 },
    { 0xaf9bd0de, SUC_lexicon_tags_33 },
    { 0xafbbd0d9, SUC_lexicon_tags_25 },
    { 0xf2fbd0e0, SUC_lexicon_tags_9 },
    { 0xec9bd0e0, SUC_lexicon_tags_30 },
    { 0x8a5bd0e2, SUC_lexicon_tags_15 },
    { 0x223bd0da, SUC_lexicon_tags_84 },
    { 0x385bd0df, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xdc9bd0e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bd0ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda3bd0ef, SUC_lexicon_tags_7 },
    { 0x701bd0ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x525bd0f2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65bd0f9, SUC_lexicon_tags_31 },
    { 0x35bbd0fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49bd0ff, SUC_lexicon_tags_49 },
    { 0x91bd0ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94dbd108, SUC_lexicon_tags_79 },
    { 0xc51bd108, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2cfbd10b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x275bd10e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11bd110, SUC_lexicon_tags_10 },
    { 0xa35bd110, SUC_lexicon_tags_31 },
    { 0x17bd112, SUC_lexicon_tags_17 },
    { 0x8f5bd113, SUC_lexicon_tags_7 },
    { 0xa55bd112, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4edbd116, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371bd11d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb31bd11f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bd124, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9bd12a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7b1bd12c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85dbd12e, SUC_lexicon_tags_1 },
    { 0xe29bd12e, SUC_lexicon_tags_27 },
    { 0xa57bd130, SUC_lexicon_tags_1 },
    { 0xbdbd131, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03bd138, SUC_lexicon_tags_1 },
    { 0xe03bd138, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9bd13c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf43bd13e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f7bd140, SUC_lexicon_tags_2 },
    { 0x385bd141, SUC_lexicon_tags_9 },
    { 0x77bd142, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afbd146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b9bd148, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3bd14b, SUC_lexicon_tags_8 },
    { 0xe1bd14c, SUC_lexicon_tags_1002 },
    { 0xd31bd14c, SUC_lexicon_tags_19 },
    { 0x5dbd14d, SUC_lexicon_tags_12 },
    { 0xe99bd14f, SUC_lexicon_tags_7 },
    { 0xbe9bd14f, SUC_lexicon_tags_30 },
    { 0x549bd14f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483bd156, SUC_lexicon_tags_98 },
    { 0x3d3bd156, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2afbd159, SUC_lexicon_tags_2 },
    { 0x539bd159, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71bd15d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fbd163, SUC_lexicon_tags_25 },
    { 0xa03bd164, SUC_lexicon_tags_30 },
    { 0x47dbd165, SUC_lexicon_tags_19 },
    { 0xd53bd166, SUC_lexicon_tags_37 },
    { 0xb1fbd166, SUC_lexicon_tags_1155 },
    { 0x85dbd168, SUC_lexicon_tags_7 },
    { 0x751bd169, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x943bd16c, SUC_lexicon_tags_15 },
    { 0x165bd16d, SUC_lexicon_tags_17 },
    { 0xfbdbd16e, SUC_lexicon_tags_11 },
    { 0x885bd16d, SUC_lexicon_tags_37 },
    { 0x851bd170, SUC_lexicon_tags_63 },
    { 0x31dbd170, SUC_lexicon_tags_42 },
    { 0x26fbd16f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdbd176, SUC_lexicon_tags_1 },
    { 0x4c3bd177, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7bd17a, SUC_lexicon_tags_15 },
    { 0x9e3bd17b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x413bd17d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e5bd17f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5bd183, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6bdbd185, SUC_lexicon_tags_37 },
    { 0x6efbd186, SUC_lexicon_tags_15 },
    { 0x7c9bd186, SUC_lexicon_tags_17 },
    { 0x347bd185, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadbd192, SUC_lexicon_tags_0 },
    { 0x51bbd193, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47bd197, SUC_lexicon_tags_40 },
    { 0xb5fbd198, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc05bd19a, SUC_lexicon_tags_12 },
    { 0x157bd19b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdbd19f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dbd1a3, SUC_lexicon_tags_1 },
    { 0xecbbd1a4, SUC_lexicon_tags_38 },
    { 0xc9bbd1a5, SUC_lexicon_tags_6 },
    { 0x24dbd1a5, SUC_lexicon_tags_25 },
    { 0x681bd1a7, SUC_lexicon_tags_1 },
    { 0xd0bbd1a7, SUC_lexicon_tags_25 },
    { 0x983bd1a8, SUC_lexicon_tags_29 },
    { 0x79fbd1a7, SUC_lexicon_tags_15 },
    { 0x32fbd1ab, SUC_lexicon_tags_11 },
    { 0x865bd1ac, SUC_lexicon_tags_39 },
    { 0x7dfbd1a7, SUC_lexicon_tags_1 },
    { 0xd29bd1a9, SUC_lexicon_tags_11 },
    { 0xd4dbd1af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7bd1b4, SUC_lexicon_tags_15 },
    { 0x185bd1b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefbd1b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x247bd1ba, SUC_lexicon_tags_12 },
    { 0x2a1bd1bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae3bd1bd, SUC_lexicon_tags_21 },
    { 0x137bd1be, SUC_lexicon_tags_15 },
    { 0xb97bd1bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a3bd1c1, SUC_lexicon_tags_49 },
    { 0x965bd1c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8fbd1c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77bd1c9, SUC_lexicon_tags_25 },
    { 0xfbfbd1ca, SUC_lexicon_tags_3 },
    { 0x2bbd1cb, SUC_lexicon_tags_37 },
    { 0xb47bd1cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbd1cf, SUC_lexicon_tags_17 },
    { 0x603bd1cf, SUC_lexicon_tags_11 },
    { 0x4f9bd1cf, SUC_lexicon_tags_12 },
    { 0x763bd1d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2cfbd1d4, SUC_lexicon_tags_47 },
    { 0x493bd1d4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x417bd1d7, SUC_lexicon_tags_45 },
    { 0x71bd1d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f5bd1da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4d7bd1dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5bd1ec, SUC_lexicon_tags_37 },
    { 0x97bbd1ec, SUC_lexicon_tags_47 },
    { 0x143bd1ee, SUC_lexicon_tags_54 },
    { 0xde7bd1ef, SUC_lexicon_tags_87 },
    { 0x19bbd1f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x319bd1f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x689bd1f4, SUC_lexicon_tags_32 },
    { 0x813bd1f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1bd1f8, SUC_lexicon_tags_7 },
    { 0xe01bd1f9, SUC_lexicon_tags_9 },
    { 0x1bfbd1f8, SUC_lexicon_tags_7 },
    { 0xdf3bd1fb, SUC_lexicon_tags_49 },
    { 0x22fbd1fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfbd200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfbd208, SUC_lexicon_tags_47 },
    { 0x4e3bd208, SUC_lexicon_tags_25 },
    { 0x873bd20a, SUC_lexicon_tags_60 },
    { 0x73bbd20b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3bd212, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd29bd214, SUC_lexicon_tags_14 },
    { 0x55bd215, SUC_lexicon_tags_30 },
    { 0x7bd215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x61bd218, SUC_lexicon_tags_14 },
    { 0xd57bd219, SUC_lexicon_tags_36 },
    { 0xaddbd218, SUC_lexicon_tags_17 },
    { 0xd55bd21b, SUC_lexicon_tags_0 },
    { 0x173bd218, SUC_lexicon_tags_9 },
    { 0x58fbd21d, SUC_lexicon_tags_47 },
    { 0x66dbd21e, SUC_lexicon_tags_37 },
    { 0x17fbd21f, SUC_lexicon_tags_11 },
    { 0x811bd21f, SUC_lexicon_tags_33 },
    { 0xeb1bd21c, SUC_lexicon_tags_0 },
    { 0xfd9bd219, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56fbd224, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61bd226, SUC_lexicon_tags_14 },
    { 0xee9bd227, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0xe6dbd229, SUC_lexicon_tags_26 },
    { 0xc87bd229, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdedbd22c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21bd235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7bd23a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3bd240, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23bd243, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdbd24c, SUC_lexicon_tags_34 },
    { 0x391bd24c, SUC_lexicon_tags_37 },
    { 0xc53bd24e, SUC_lexicon_tags_0 },
    { 0x8fdbd24f, SUC_lexicon_tags_16 },
    { 0x46fbd250, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621bd25a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fbd261, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbbd266, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9f3bd268, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3bd26b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb19bd26d, SUC_lexicon_tags_37 },
    { 0xa15bd26e, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889bd272, SUC_lexicon_tags_25 },
    { 0x887bd273, SUC_lexicon_tags_12 },
    { 0xa65bd273, SUC_lexicon_tags_25 },
    { 0x717bd275, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243bd27b, SUC_lexicon_tags_25 },
    { 0x3fbbd27c, SUC_lexicon_tags_28 },
    { 0x715bd27d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85dbd27f, SUC_lexicon_tags_7 },
    { 0x63bbd27f, SUC_lexicon_tags_16 },
    { 0xf89bd281, SUC_lexicon_tags_11 },
    { 0x51bbd282, SUC_lexicon_tags_10 },
    { 0x7e1bd283, SUC_lexicon_tags_15 },
    { 0x2cfbd280, SUC_lexicon_tags_7 },
    { 0x4c7bd285, SUC_lexicon_tags_39 },
    { 0xa6bbd284, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd49bd288, SUC_lexicon_tags_33 },
    { 0xe07bd288, SUC_lexicon_tags_23 },
    { 0x705bd288, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dbd28d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11bd293, SUC_lexicon_tags_0 },
    { 0xcadbd294, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93dbd296, SUC_lexicon_tags_23 },
    { 0xcf9bd297, SUC_lexicon_tags_39 },
    { 0xf79bd298, SUC_lexicon_tags_12 },
    { 0xb4fbd299, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe7bd29b, SUC_lexicon_tags_37 },
    { 0x69bd29b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbd2a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfbd2a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fbd2aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49bd2ad, SUC_lexicon_tags_87 },
    { 0xa83bd2ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75bd2b2, SUC_lexicon_tags_11 },
    { 0x73bd2b2, SUC_lexicon_tags_92 },
    { 0xd79bd2b4, SUC_lexicon_tags_17 },
    { 0x337bd2b5, SUC_lexicon_tags_58 },
    { 0x40bbd2b4, SUC_lexicon_tags_12 },
    { 0xbfbbd2b3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297bd2bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839bd2c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371bd2c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffbd2ca, SUC_lexicon_tags_48 },
    { 0xfd5bd2cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x507bd2cd, SUC_lexicon_tags_4 },
    { 0x697bd2ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1bd2d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x737bd2d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe41bd2d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21bd2e2, SUC_lexicon_tags_17 },
    { 0x7ddbd2e2, SUC_lexicon_tags_23 },
    { 0xb81bd2e4, SUC_lexicon_tags_15 },
    { 0x331bd2e3, SUC_lexicon_tags_63 },
    { 0x759bd2e6, SUC_lexicon_tags_25 },
    { 0x78fbd2e7, SUC_lexicon_tags_1 },
    { 0xa2dbd2e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99bd2ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x417bd2f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61bd2f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dbd2f7, SUC_lexicon_tags_21 },
    { 0x71dbd2f8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4dfbd2fa, SUC_lexicon_tags_1 },
    { 0x6c9bd2fa, SUC_lexicon_tags_14 },
    { 0x773bd2fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251bd306, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5bd308, SUC_lexicon_tags_12 },
    { 0x845bd309, SUC_lexicon_tags_63 },
    { 0xcadbd309, SUC_lexicon_tags_47 },
    { 0xe21bd309, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfbbbd30d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x61fbd30f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fbd313, SUC_lexicon_tags_88 },
    { 0x9afbd314, SUC_lexicon_tags_12 },
    { 0x57bbd313, SUC_lexicon_tags_11 },
    { 0x581bd315, SUC_lexicon_tags_22 },
    { 0x9b5bd317, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dbd31a, SUC_lexicon_tags_12 },
    { 0x459bd31b, SUC_lexicon_tags_18 },
    { 0xdcbbd31c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe59bd31e, SUC_lexicon_tags_39 },
    { 0x32fbd31f, SUC_lexicon_tags_3 },
    { 0x383bd31e, SUC_lexicon_tags_12 },
    { 0xc45bd31f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977bd326, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abbd329, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fbd32d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fbd330, SUC_lexicon_tags_37 },
    { 0x3abbd331, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x103bd333, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddbd336, SUC_lexicon_tags_4 },
    { 0x809bd337, SUC_lexicon_tags_1 },
    { 0x24bbd338, SUC_lexicon_tags_1 },
    { 0xba5bd339, SUC_lexicon_tags_9 },
    { 0x453bd337, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fbd33d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561bd342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27bd346, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5bd34a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bd34d, SUC_lexicon_tags_2 },
    { 0x6dbd34e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9bd358, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bd35b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fbd35f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x949bd361, SUC_lexicon_tags_38 },
    { 0x1e3bd362, SUC_lexicon_tags_23 },
    { 0x9fdbd363, SUC_lexicon_tags_2 },
    { 0x77bbd361, SUC_lexicon_tags_4 },
    { 0x91bd362, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x505bd367, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bbd36c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23bd371, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf75bd373, SUC_lexicon_tags_19 },
    { 0x55bbd374, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3dbd376, SUC_lexicon_tags_124 },
    { 0x127bd377, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39bd37a, SUC_lexicon_tags_148 },
    { 0xb35bd37a, SUC_lexicon_tags_47 },
    { 0x861bd37c, SUC_lexicon_tags_37 },
    { 0x43fbd37b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dbd380, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x921bd382, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa55bd384, SUC_lexicon_tags_28 },
    { 0xc23bd385, SUC_lexicon_tags_25 },
    { 0xb1bbd384, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9bd38b, SUC_lexicon_tags_6 },
    { 0x979bd38c, SUC_lexicon_tags_25 },
    { 0xad9bd38c, SUC_lexicon_tags_70 },
    { 0x1ddbd38b, SUC_lexicon_tags_46 },
    { 0xcf3bd38e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3bd393, SUC_lexicon_tags_60 },
    { 0x977bd393, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93bd39a, SUC_lexicon_tags_39 },
    { 0xc8dbd39b, SUC_lexicon_tags_37 },
    { 0x881bd39c, SUC_lexicon_tags_23 },
    { 0x161bd39d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdbbbd39f, SUC_lexicon_tags_15 },
    { 0x187bd3a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3bd3a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe7dbd3a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75dbd3aa, SUC_lexicon_tags_25 },
    { 0x3afbd3ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fbd3b0, SUC_lexicon_tags_1 },
    { 0x7f3bd3b1, SUC_lexicon_tags_28 },
    { 0x49bbd3b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fbd3b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x477bd3b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bbd3be, SUC_lexicon_tags_41 },
    { 0x32fbd3bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbd3c2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fbd3c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3bd3c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491bd3cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfddbd3ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc77bd3d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67dbd3d3, SUC_lexicon_tags_37 },
    { 0x749bd3d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10bbd3d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x74fbd3d9, SUC_lexicon_tags_31 },
    { 0x487bd3da, SUC_lexicon_tags_63 },
    { 0x5cdbd3db, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1bd3df, SUC_lexicon_tags_30 },
    { 0x99bd3e0, SUC_lexicon_tags_33 },
    { 0xd1bbd3e1, SUC_lexicon_tags_10 },
    { 0xb35bd3e2, SUC_lexicon_tags_14 },
    { 0xe7bd3e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf0fbd3e5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x263bd3e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981bd3ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaebbd3f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7bd3f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bd3f7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bd3fa, SUC_lexicon_tags_45 },
    { 0x87fbd3fb, SUC_lexicon_tags_6 },
    { 0xf8fbd3fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd95bd3fe, SUC_lexicon_tags_6 },
    { 0x55fbd3fe, SUC_lexicon_tags_25 },
    { 0xc31bd400, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dbd407, SUC_lexicon_tags_7 },
    { 0x203bd408, SUC_lexicon_tags_30 },
    { 0x191bd409, SUC_lexicon_tags_66 },
    { 0x43bd409, SUC_lexicon_tags_46 },
    { 0x4b3bd40b, SUC_lexicon_tags_29 },
    { 0x15fbd40c, SUC_lexicon_tags_17 },
    { 0xa59bd40d, SUC_lexicon_tags_16 },
    { 0xc8dbd40d, SUC_lexicon_tags_25 },
    { 0xd0dbd40f, SUC_lexicon_tags_3 },
    { 0xb1bbd40c, SUC_lexicon_tags_16 },
    { 0x1bfbd411, SUC_lexicon_tags_7 },
    { 0x529bd411, SUC_lexicon_tags_107 },
    { 0x239bd40e, SUC_lexicon_tags_37 },
    { 0x7a7bd40f, SUC_lexicon_tags_1 },
    { 0x113bd415, SUC_lexicon_tags_49 },
    { 0xe81bd40c, SUC_lexicon_tags_17 },
    { 0xb63bd417, SUC_lexicon_tags_47 },
    { 0x11bbd417, SUC_lexicon_tags_61 },
    { 0xc09bd419, SUC_lexicon_tags_11 },
    { 0xaffbd40e, SUC_lexicon_tags_17 },
    { 0x367bd41b, SUC_lexicon_tags_27 },
    { 0x749bd41c, SUC_lexicon_tags_2 },
    { 0xf07bd41d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a1bd41f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bbd427, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x26dbd42a, SUC_lexicon_tags_14 },
    { 0x169bd42a, SUC_lexicon_tags_11 },
    { 0xf0dbd42c, SUC_lexicon_tags_46 },
    { 0x4d3bd42d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdbd432, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x735bd434, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9bd439, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcc5bd43b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x62fbd43d, SUC_lexicon_tags_60 },
    { 0xf1bd43d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bbd441, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639bd445, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x187bd447, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf55bd449, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbbd44c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bd456, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3bd459, SUC_lexicon_tags_63 },
    { 0x8e1bd45a, SUC_lexicon_tags_4 },
    { 0x679bd45b, SUC_lexicon_tags_23 },
    { 0x2bbbd45b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bbd464, SUC_lexicon_tags_1 },
    { 0x405bd464, SUC_lexicon_tags_11 },
    { 0x1e7bd466, SUC_lexicon_tags_7 },
    { 0xcedbd466, SUC_lexicon_tags_1330 },
    { 0x529bd468, SUC_lexicon_tags_4 },
    { 0xa27bd469, SUC_lexicon_tags_27 },
    { 0x6fbbd46a, SUC_lexicon_tags_37 },
    { 0xb81bd46b, SUC_lexicon_tags_13 },
    { 0xff1bd46c, SUC_lexicon_tags_899 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dbd470, SUC_lexicon_tags_15 },
    { 0xb69bd470, SUC_lexicon_tags_253 },
    { 0xd71bd472, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54dbd476, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbd479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49fbd47b, SUC_lexicon_tags_28 },
    { 0xcc7bd47c, SUC_lexicon_tags_45 },
    { 0x705bd47b, SUC_lexicon_tags_138 },
    { 0xc0bbd47e, SUC_lexicon_tags_2 },
    { 0x919bd47e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9bd485, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6dbbd487, SUC_lexicon_tags_12 },
    { 0x4d5bd488, SUC_lexicon_tags_26 },
    { 0x745bd489, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691bd48d, SUC_lexicon_tags_7 },
    { 0x973bd48e, SUC_lexicon_tags_661 },
    { 0xb9bd48d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbbd492, SUC_lexicon_tags_17 },
    { 0x695bd493, SUC_lexicon_tags_14 },
    { 0xd61bd494, SUC_lexicon_tags_17 },
    { 0xf55bd495, SUC_lexicon_tags_45 },
    { 0x56fbd496, SUC_lexicon_tags_31 },
    { 0xb0bbd495, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac3bd499, SUC_lexicon_tags_37 },
    { 0x9ddbd49a, SUC_lexicon_tags_37 },
    { 0xc89bd49b, SUC_lexicon_tags_25 },
    { 0x89dbd49c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc1bbd49e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x495bd4a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x879bd4a2, SUC_lexicon_tags_32 },
    { 0xa7dbd4a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1bd4aa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abbd4ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x847bd4b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdbd4b3, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fbd4b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5bd4bd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbbd4c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59bbd4c2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x14dbd4c5, SUC_lexicon_tags_28 },
    { 0x53bd4c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bbd4cf, SUC_lexicon_tags_235 },
    { 0xc7dbd4d0, SUC_lexicon_tags_33 },
    { 0x275bd4d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75bd4db, SUC_lexicon_tags_80 },
    { 0xaffbd4dc, SUC_lexicon_tags_14 },
    { 0x901bd4dd, SUC_lexicon_tags_2 },
    { 0x937bd4de, SUC_lexicon_tags_46 },
    { 0x349bd4dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fbd4e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabbd4e5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x39dbd4e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331bd4ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdbd4f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe3bd4f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f1bd4f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63bd4fc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xea5bd4fe, SUC_lexicon_tags_27 },
    { 0x1b3bd4ff, SUC_lexicon_tags_26 },
    { 0x3a5bd500, SUC_lexicon_tags_37 },
    { 0x887bd501, SUC_lexicon_tags_60 },
    { 0xe2fbd4fe, SUC_lexicon_tags_11 },
    { 0x829bd501, SUC_lexicon_tags_66 },
    { 0x485bd504, SUC_lexicon_tags_25 },
    { 0xb2fbd505, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb9bd507, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2d9bd509, SUC_lexicon_tags_7 },
    { 0x93dbd509, SUC_lexicon_tags_11 },
    { 0xfd9bd50a, SUC_lexicon_tags_25 },
    { 0xbf1bd50c, SUC_lexicon_tags_12 },
    { 0xa09bd50d, SUC_lexicon_tags_17 },
    { 0xfb3bd50e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90fbd510, SUC_lexicon_tags_11 },
    { 0xff7bd510, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xf35bd513, SUC_lexicon_tags_43 },
    { 0x595bd513, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcffbd516, SUC_lexicon_tags_15 },
    { 0x9b1bd517, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ddbd519, SUC_lexicon_tags_77 },
    { 0xe4bbd51a, SUC_lexicon_tags_63 },
    { 0x8f7bd51b, SUC_lexicon_tags_64 },
    { 0x3a1bd51a, SUC_lexicon_tags_1498 },
    { 0xf8bbd51d, SUC_lexicon_tags_14 },
    { 0x287bd51e, SUC_lexicon_tags_87 },
    { 0x4f5bd51e, SUC_lexicon_tags_47 },
    { 0xb5bbd520, SUC_lexicon_tags_1 },
    { 0x70fbd51d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379bd526, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749bd52b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667bd530, SUC_lexicon_tags_23 },
    { 0x881bd531, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32dbd534, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fbd537, SUC_lexicon_tags_19 },
    { 0x29bbd537, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99bd53f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7bd547, SUC_lexicon_tags_37 },
    { 0xc75bd548, SUC_lexicon_tags_15 },
    { 0x995bd549, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33bbd54b, SUC_lexicon_tags_25 },
    { 0x517bd54c, SUC_lexicon_tags_37 },
    { 0xe57bd54c, SUC_lexicon_tags_77 },
    { 0xd3bd54e, SUC_lexicon_tags_12 },
    { 0x1cfbd54c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18bbd551, SUC_lexicon_tags_2 },
    { 0x7bdbd552, SUC_lexicon_tags_7 },
    { 0xed3bd553, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x22fbd555, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fbd558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfbd55e, SUC_lexicon_tags_15 },
    { 0xc4fbd55e, SUC_lexicon_tags_4 },
    { 0x8a9bd560, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5bd567, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bbd56b, SUC_lexicon_tags_29 },
    { 0x70dbd56b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bbd570, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf01bd572, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x517bd574, SUC_lexicon_tags_129 },
    { 0x4d7bd575, SUC_lexicon_tags_10 },
    { 0xc3bd576, SUC_lexicon_tags_21 },
    { 0xec5bd576, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843bd57c, SUC_lexicon_tags_12 },
    { 0x9c5bd57d, SUC_lexicon_tags_0 },
    { 0xe97bd57d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x809bd580, SUC_lexicon_tags_12 },
    { 0xd25bd580, SUC_lexicon_tags_37 },
    { 0x353bd581, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697bd587, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1bd58b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8bbd58d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525bd593, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x98bbd595, SUC_lexicon_tags_16 },
    { 0xdddbd595, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7bd59b, SUC_lexicon_tags_32 },
    { 0x6b9bd59b, SUC_lexicon_tags_37 },
    { 0x723bd59c, SUC_lexicon_tags_17 },
    { 0xd01bd59e, SUC_lexicon_tags_26 },
    { 0x387bd59c, SUC_lexicon_tags_8 },
    { 0xfe7bd59e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817bd5a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1bd5aa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xeefbd5ac, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373bd5b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x833bd5b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57bd5b7, SUC_lexicon_tags_11 },
    { 0xd93bd5b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x385bd5bb, SUC_lexicon_tags_63 },
    { 0xb2dbd5bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bd5c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23bd5cc, SUC_lexicon_tags_55 },
    { 0xe89bd5cd, SUC_lexicon_tags_19 },
    { 0xae5bd5ce, SUC_lexicon_tags_66 },
    { 0xfdbbd5cf, SUC_lexicon_tags_1 },
    { 0xabbd5cf, SUC_lexicon_tags_35 },
    { 0x49bd5d0, SUC_lexicon_tags_26 },
    { 0x899bd5d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebbd5d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589bd5da, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833bd5e2, SUC_lexicon_tags_17 },
    { 0x1abbd5e2, SUC_lexicon_tags_25 },
    { 0x5a7bd5e4, SUC_lexicon_tags_15 },
    { 0x287bd5e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7bd5e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5bd5ec, SUC_lexicon_tags_16 },
    { 0xe17bd5ed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bbd5f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325bd5f5, SUC_lexicon_tags_11 },
    { 0x70dbd5f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fbd5f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed9bd5fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafbd5fe, SUC_lexicon_tags_881 },
    { 0, NULL },
    { 0, NULL },
    { 0x843bd601, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x91fbd604, SUC_lexicon_tags_47 },
    { 0xd4fbd605, SUC_lexicon_tags_24 },
    { 0xc63bd604, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4f5bd608, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb3bd60a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3abbd60c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe85bd60e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1bd613, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fbd618, SUC_lexicon_tags_37 },
    { 0xaafbd619, SUC_lexicon_tags_6 },
    { 0xf8dbd61a, SUC_lexicon_tags_7 },
    { 0xa1bbd61b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dbd61f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7bd622, SUC_lexicon_tags_37 },
    { 0x837bd623, SUC_lexicon_tags_3 },
    { 0xec3bd623, SUC_lexicon_tags_47 },
    { 0xe27bd625, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc21bd627, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfbd62e, SUC_lexicon_tags_79 },
    { 0x1bd62f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x515bd632, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883bd636, SUC_lexicon_tags_15 },
    { 0x98bbd637, SUC_lexicon_tags_60 },
    { 0xae5bd638, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fbd63e, SUC_lexicon_tags_25 },
    { 0xf7fbd63e, SUC_lexicon_tags_38 },
    { 0xefbbd640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69bd645, SUC_lexicon_tags_12 },
    { 0xcdfbd646, SUC_lexicon_tags_37 },
    { 0xec5bd647, SUC_lexicon_tags_25 },
    { 0xa89bd648, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5dbbd64a, SUC_lexicon_tags_31 },
    { 0x35fbd64a, SUC_lexicon_tags_17 },
    { 0x69bd64c, SUC_lexicon_tags_151 },
    { 0x7c5bd64d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7bd653, SUC_lexicon_tags_238 },
    { 0x7d1bd653, SUC_lexicon_tags_7 },
    { 0xe09bd654, SUC_lexicon_tags_55 },
    { 0xcffbd654, SUC_lexicon_tags_46 },
    { 0xc8bbd657, SUC_lexicon_tags_1 },
    { 0xc07bd654, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fbd65f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fbd665, SUC_lexicon_tags_60 },
    { 0x5cdbd666, SUC_lexicon_tags_47 },
    { 0x8bdbd667, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1bd66a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35bd66e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb69bd670, SUC_lexicon_tags_121 },
    { 0xf67bd671, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725bd675, SUC_lexicon_tags_9 },
    { 0x49bbd676, SUC_lexicon_tags_23 },
    { 0xfefbd677, SUC_lexicon_tags_11 },
    { 0x2f1bd676, SUC_lexicon_tags_0 },
    { 0xb33bd679, SUC_lexicon_tags_21 },
    { 0xf47bd67a, SUC_lexicon_tags_29 },
    { 0x523bd679, SUC_lexicon_tags_9 },
    { 0x9f7bd67b, SUC_lexicon_tags_12 },
    { 0x2ffbd67a, SUC_lexicon_tags_37 },
    { 0xa03bd67a, SUC_lexicon_tags_11 },
    { 0xb85bd67f, SUC_lexicon_tags_35 },
    { 0x8ffbd680, SUC_lexicon_tags_12 },
    { 0x4e9bd680, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afbd685, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7bd68a, SUC_lexicon_tags_43 },
    { 0xb41bd68a, SUC_lexicon_tags_762 },
    { 0x9d5bd68b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7bd68f, SUC_lexicon_tags_8 },
    { 0xf6bbd690, SUC_lexicon_tags_12 },
    { 0xd3bbd691, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x307bd693, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19bd696, SUC_lexicon_tags_9 },
    { 0x289bd697, SUC_lexicon_tags_27 },
    { 0x7d9bd697, SUC_lexicon_tags_11 },
    { 0x3e5bd696, SUC_lexicon_tags_10 },
    { 0x51bd69a, SUC_lexicon_tags_27 },
    { 0x73bd69b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa0fbd69d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03bd6a1, SUC_lexicon_tags_27 },
    { 0x753bd6a1, SUC_lexicon_tags_246 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19bd6aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7bd6ae, SUC_lexicon_tags_45 },
    { 0x815bd6af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdbd6b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbd7bd6b4, SUC_lexicon_tags_21 },
    { 0x5dbbd6b4, SUC_lexicon_tags_37 },
    { 0x70bbd6b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fbd6be, SUC_lexicon_tags_15 },
    { 0x1f9bd6be, SUC_lexicon_tags_12 },
    { 0x3a7bd6c0, SUC_lexicon_tags_11 },
    { 0xd71bd6c1, SUC_lexicon_tags_15 },
    { 0xd37bd6c2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9bd6c7, SUC_lexicon_tags_1 },
    { 0xf07bd6c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88bbd6cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dbd6d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bbd6db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x953bd6de, SUC_lexicon_tags_2 },
    { 0x399bd6de, SUC_lexicon_tags_38 },
    { 0x6ebbd6e0, SUC_lexicon_tags_18 },
    { 0xaabbd6e0, SUC_lexicon_tags_37 },
    { 0x95bbd6e1, SUC_lexicon_tags_14 },
    { 0x2c7bd6e3, SUC_lexicon_tags_105 },
    { 0x4c3bd6e3, SUC_lexicon_tags_17 },
    { 0xd39bd6e3, SUC_lexicon_tags_13 },
    { 0xc9dbd6e5, SUC_lexicon_tags_27 },
    { 0x141bd6e6, SUC_lexicon_tags_5 },
    { 0x505bd6e8, SUC_lexicon_tags_17 },
    { 0x925bd6e9, SUC_lexicon_tags_45 },
    { 0xd79bd6e8, SUC_lexicon_tags_32 },
    { 0x1e9bd6eb, SUC_lexicon_tags_3 },
    { 0x62fbd6e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb37bd6ee, SUC_lexicon_tags_7 },
    { 0x1cbbd6ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x203bd6f1, SUC_lexicon_tags_47 },
    { 0x78dbd6f2, SUC_lexicon_tags_12 },
    { 0x2c5bd6f2, SUC_lexicon_tags_25 },
    { 0x5e1bd6f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc7bd6f6, SUC_lexicon_tags_11 },
    { 0x657bd6f7, SUC_lexicon_tags_12 },
    { 0x161bd6f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25bd702, SUC_lexicon_tags_47 },
    { 0x107bd702, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3a5bd705, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdbdbd707, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4f5bd709, SUC_lexicon_tags_2 },
    { 0xe11bd70a, SUC_lexicon_tags_27 },
    { 0xed3bd70b, SUC_lexicon_tags_19 },
    { 0x79bbd70c, SUC_lexicon_tags_35 },
    { 0xb71bd70a, SUC_lexicon_tags_1 },
    { 0x78fbd70e, SUC_lexicon_tags_7 },
    { 0x953bd70b, SUC_lexicon_tags_45 },
    { 0xb9bbd70b, SUC_lexicon_tags_38 },
    { 0xcc5bd711, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141bd715, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795bd71a, SUC_lexicon_tags_19 },
    { 0x32fbd71b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9bd71e, SUC_lexicon_tags_26 },
    { 0x851bd71f, SUC_lexicon_tags_11 },
    { 0xa3fbd71f, SUC_lexicon_tags_33 },
    { 0x61bd721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbbd729, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba1bd72b, SUC_lexicon_tags_14 },
    { 0xa55bd72b, SUC_lexicon_tags_12 },
    { 0xb99bd72d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79bd730, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x941bd733, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629bd739, SUC_lexicon_tags_1 },
    { 0xdfdbd739, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bbd74a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bbd74d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x275bd750, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3bd753, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65bd75f, SUC_lexicon_tags_45 },
    { 0xefbbd760, SUC_lexicon_tags_47 },
    { 0x11bbd761, SUC_lexicon_tags_1 },
    { 0x841bd760, SUC_lexicon_tags_36 },
    { 0x9dbd763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695bd76b, SUC_lexicon_tags_29 },
    { 0x469bd76c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253bd773, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5bd77b, SUC_lexicon_tags_2 },
    { 0x217bd77c, SUC_lexicon_tags_47 },
    { 0xb51bd77d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1bd77f, SUC_lexicon_tags_65 },
    { 0x70fbd780, SUC_lexicon_tags_19 },
    { 0x27fbd781, SUC_lexicon_tags_33 },
    { 0x607bd782, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541bd786, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bd789, SUC_lexicon_tags_47 },
    { 0xc89bd78a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29bd78d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfbd792, SUC_lexicon_tags_12 },
    { 0xde3bd793, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bd796, SUC_lexicon_tags_38 },
    { 0x355bd797, SUC_lexicon_tags_12 },
    { 0x59fbd798, SUC_lexicon_tags_33 },
    { 0x3e7bd798, SUC_lexicon_tags_1 },
    { 0x6f5bd79a, SUC_lexicon_tags_10 },
    { 0x1b1bd79b, SUC_lexicon_tags_114 },
    { 0xa97bd79c, SUC_lexicon_tags_14 },
    { 0xa01bd79a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dbd7a1, SUC_lexicon_tags_37 },
    { 0x73bbd7a2, SUC_lexicon_tags_4 },
    { 0x23bd7a3, SUC_lexicon_tags_9 },
    { 0x87dbd7a1, SUC_lexicon_tags_28 },
    { 0x51dbd7a1, SUC_lexicon_tags_15 },
    { 0x6f9bd7a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1dfbd7a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29bd7ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bbd7b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97bd7b5, SUC_lexicon_tags_7 },
    { 0x863bd7b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161bd7ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x343bd7bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317bd7c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1bd7c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe89bd7cb, SUC_lexicon_tags_31 },
    { 0x25dbd7cb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677bd7d1, SUC_lexicon_tags_37 },
    { 0x117bd7d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3b3bd7d4, SUC_lexicon_tags_27 },
    { 0x199bd7d5, SUC_lexicon_tags_19 },
    { 0x203bd7d5, SUC_lexicon_tags_12 },
    { 0xd2dbd7d7, SUC_lexicon_tags_9 },
    { 0x249bd7d7, SUC_lexicon_tags_43 },
    { 0x9dfbd7d7, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1bd7de, SUC_lexicon_tags_7 },
    { 0xe31bd7df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd09bd7e1, SUC_lexicon_tags_25 },
    { 0x449bd7e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f3bd7e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x337bd7e6, SUC_lexicon_tags_7 },
    { 0x611bd7e7, SUC_lexicon_tags_0 },
    { 0xb0fbd7e8, SUC_lexicon_tags_31 },
    { 0xbb5bd7e9, SUC_lexicon_tags_34 },
    { 0xd3dbd7e9, SUC_lexicon_tags_37 },
    { 0xc81bd7e9, SUC_lexicon_tags_34 },
    { 0x957bd7ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bbd7f1, SUC_lexicon_tags_46 },
    { 0x5c3bd7f2, SUC_lexicon_tags_46 },
    { 0x997bd7f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9bd7f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfbd7f9, SUC_lexicon_tags_31 },
    { 0x571bd7fa, SUC_lexicon_tags_17 },
    { 0x707bd7f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x847bd7fe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bbd804, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bbd80b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x859bd80d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdffbd80f, SUC_lexicon_tags_17 },
    { 0x93dbd810, SUC_lexicon_tags_11 },
    { 0x5edbd80f, SUC_lexicon_tags_8 },
    { 0xa95bd812, SUC_lexicon_tags_43 },
    { 0x4e9bd813, SUC_lexicon_tags_38 },
    { 0x3b1bd814, SUC_lexicon_tags_55 },
    { 0x42fbd815, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7bd819, SUC_lexicon_tags_0 },
    { 0xc2dbd819, SUC_lexicon_tags_62 },
    { 0x37fbd819, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x451bd81e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5bd821, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe2dbd823, SUC_lexicon_tags_34 },
    { 0xdcfbd824, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa5bbd826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5bd82c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9fbbd82e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81bd833, SUC_lexicon_tags_14 },
    { 0xfcbbd834, SUC_lexicon_tags_14 },
    { 0xa0dbd835, SUC_lexicon_tags_79 },
    { 0xca9bd836, SUC_lexicon_tags_15 },
    { 0x4ddbd837, SUC_lexicon_tags_34 },
    { 0x4c7bd836, SUC_lexicon_tags_0 },
    { 0x4f3bd839, SUC_lexicon_tags_250 },
    { 0xd93bd839, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x819bd83d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x59dbd83f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5bd842, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x541bd844, SUC_lexicon_tags_15 },
    { 0xb43bd845, SUC_lexicon_tags_39 },
    { 0x9cfbd846, SUC_lexicon_tags_6 },
    { 0x77dbd846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38bbd849, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd11bd84b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x477bd84d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdbd850, SUC_lexicon_tags_45 },
    { 0xd0bbd851, SUC_lexicon_tags_38 },
    { 0xe4fbd852, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x66dbd855, SUC_lexicon_tags_15 },
    { 0x109bd855, SUC_lexicon_tags_11 },
    { 0x473bd857, SUC_lexicon_tags_624 },
    { 0x287bd858, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7bd861, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fbd864, SUC_lexicon_tags_33 },
    { 0xc75bd864, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc37bd867, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bd871, SUC_lexicon_tags_11 },
    { 0x761bd872, SUC_lexicon_tags_1 },
    { 0x61bd873, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdfbd875, SUC_lexicon_tags_26 },
    { 0x335bd876, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375bd87b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7bd889, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x989bd88b, SUC_lexicon_tags_1 },
    { 0x5b9bd88b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485bd897, SUC_lexicon_tags_45 },
    { 0x97bbd897, SUC_lexicon_tags_28 },
    { 0xffdbd899, SUC_lexicon_tags_46 },
    { 0x809bd89a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x39bbd89c, SUC_lexicon_tags_17 },
    { 0x4c7bd89d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbbd8a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bbd8a5, SUC_lexicon_tags_68 },
    { 0x921bd8a5, SUC_lexicon_tags_17 },
    { 0xd8bbd8a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1bd8aa, SUC_lexicon_tags_47 },
    { 0xd9bd8aa, SUC_lexicon_tags_19 },
    { 0x9fbd8ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53bd8ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x56fbd8b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391bd8b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325bd8ba, SUC_lexicon_tags_39 },
    { 0x417bd8bb, SUC_lexicon_tags_4 },
    { 0xad5bd8bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e7bd8be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xba1bd8c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x595bd8c2, SUC_lexicon_tags_37 },
    { 0xd1bbd8c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9bd8c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47bd8c9, SUC_lexicon_tags_30 },
    { 0xc15bd8ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbbd8cd, SUC_lexicon_tags_31 },
    { 0x807bd8ce, SUC_lexicon_tags_11 },
    { 0x4c1bd8cd, SUC_lexicon_tags_15 },
    { 0x3afbd8cd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5bd8d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x727bd8d5, SUC_lexicon_tags_63 },
    { 0xf43bd8d6, SUC_lexicon_tags_26 },
    { 0x2ebbd8d7, SUC_lexicon_tags_39 },
    { 0x1b3bd8d8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xde7bd8da, SUC_lexicon_tags_7 },
    { 0xa15bd8db, SUC_lexicon_tags_33 },
    { 0xdcdbd8dc, SUC_lexicon_tags_37 },
    { 0x1bbbd8db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9d5bd8df, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4dbbd8e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7bd8e4, SUC_lexicon_tags_47 },
    { 0x8a3bd8e4, SUC_lexicon_tags_47 },
    { 0x3a3bd8e6, SUC_lexicon_tags_61 },
    { 0xc89bd8e6, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x4b5bd8e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17bd8f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89bd8f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5bd8f8, SUC_lexicon_tags_12 },
    { 0x7dfbd8f9, SUC_lexicon_tags_37 },
    { 0x4b1bd8f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21bd8fc, SUC_lexicon_tags_2 },
    { 0x4c9bd8fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x169bd8ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2ebbd901, SUC_lexicon_tags_23 },
    { 0xe2dbd902, SUC_lexicon_tags_45 },
    { 0x505bd903, SUC_lexicon_tags_31 },
    { 0x70dbd903, SUC_lexicon_tags_46 },
    { 0x5ebbd905, SUC_lexicon_tags_38 },
    { 0x561bd903, SUC_lexicon_tags_11 },
    { 0xa37bd903, SUC_lexicon_tags_6 },
    { 0x9dfbd908, SUC_lexicon_tags_0 },
    { 0x117bd908, SUC_lexicon_tags_4 },
    { 0xcabbd902, SUC_lexicon_tags_55 },
    { 0xdb7bd908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88dbd911, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x309bd913, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7e7bd915, SUC_lexicon_tags_10 },
    { 0x1e1bd916, SUC_lexicon_tags_77 },
    { 0x329bd917, SUC_lexicon_tags_37 },
    { 0xabbbd918, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3bd91c, SUC_lexicon_tags_46 },
    { 0xe37bd91d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5bd926, SUC_lexicon_tags_26 },
    { 0xa9bd927, SUC_lexicon_tags_29 },
    { 0x5f9bd928, SUC_lexicon_tags_0 },
    { 0x8d3bd928, SUC_lexicon_tags_27 },
    { 0xad3bd928, SUC_lexicon_tags_65 },
    { 0x501bd92b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9bd931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efbd93c, SUC_lexicon_tags_1 },
    { 0xb11bd93d, SUC_lexicon_tags_1 },
    { 0xa91bd93c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x54fbd940, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5cdbd942, SUC_lexicon_tags_87 },
    { 0x677bd943, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9bd946, SUC_lexicon_tags_25 },
    { 0x657bd947, SUC_lexicon_tags_45 },
    { 0xd7bbd948, SUC_lexicon_tags_63 },
    { 0x11fbd948, SUC_lexicon_tags_82 },
    { 0x445bd94a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45bd94d, SUC_lexicon_tags_37 },
    { 0xde1bd94e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc33bd950, SUC_lexicon_tags_17 },
    { 0xa9bbd951, SUC_lexicon_tags_9 },
    { 0xc7bbd951, SUC_lexicon_tags_6 },
    { 0x25bbd952, SUC_lexicon_tags_2 },
    { 0x39dbd954, SUC_lexicon_tags_8 },
    { 0x513bd950, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51bd95a, SUC_lexicon_tags_55 },
    { 0x3f9bd95b, SUC_lexicon_tags_37 },
    { 0xaa1bd95c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf27bd95e, SUC_lexicon_tags_11 },
    { 0x481bd95f, SUC_lexicon_tags_17 },
    { 0xfb7bd95e, SUC_lexicon_tags_84 },
    { 0x7b7bd961, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbd966, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5bd96a, SUC_lexicon_tags_68 },
    { 0x3f9bd96b, SUC_lexicon_tags_43 },
    { 0xf19bd96b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xecfbd96e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x485bd971, SUC_lexicon_tags_12 },
    { 0x1fdbd972, SUC_lexicon_tags_31 },
    { 0x275bd971, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7bd976, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x46fbd978, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5bd97c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffbd981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99dbd983, SUC_lexicon_tags_37 },
    { 0x401bd983, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1bd989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483bd98d, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xa0fbd98f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbbd992, SUC_lexicon_tags_27 },
    { 0x879bd993, SUC_lexicon_tags_37 },
    { 0xca9bd994, SUC_lexicon_tags_61 },
    { 0x593bd994, SUC_lexicon_tags_65 },
    { 0xf9dbd993, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc93bd998, SUC_lexicon_tags_12 },
    { 0x9a7bd999, SUC_lexicon_tags_23 },
    { 0xde5bd999, SUC_lexicon_tags_7 },
    { 0x2a3bd99b, SUC_lexicon_tags_38 },
    { 0xc6dbd99c, SUC_lexicon_tags_60 },
    { 0x27bd999, SUC_lexicon_tags_0 },
    { 0xde3bd99c, SUC_lexicon_tags_17 },
    { 0x611bd99d, SUC_lexicon_tags_37 },
    { 0x23dbd9a0, SUC_lexicon_tags_27 },
    { 0x3bbd9a1, SUC_lexicon_tags_2 },
    { 0x66bbd9a1, SUC_lexicon_tags_11 },
    { 0x757bd9a1, SUC_lexicon_tags_41 },
    { 0xb67bd99a, SUC_lexicon_tags_1 },
    { 0xe73bd9a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x76dbd9a7, SUC_lexicon_tags_26 },
    { 0x5a3bd9a8, SUC_lexicon_tags_37 },
    { 0x7c3bd9a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1bd9ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15bbd9af, SUC_lexicon_tags_17 },
    { 0xde1bd9b0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x85bbd9b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33bd9b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3bd9bb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dbd9c1, SUC_lexicon_tags_28 },
    { 0xa3bd9c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa43bd9c4, SUC_lexicon_tags_47 },
    { 0x9dfbd9c5, SUC_lexicon_tags_7 },
    { 0x8f1bd9c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bd9c9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bbd9cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf3dbd9ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99bd9d1, SUC_lexicon_tags_47 },
    { 0xea9bd9d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbbd9db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc2dbd9dd, SUC_lexicon_tags_11 },
    { 0xd85bd9dd, SUC_lexicon_tags_17 },
    { 0xbc5bd9df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9fbd9e1, SUC_lexicon_tags_2 },
    { 0x317bd9e1, SUC_lexicon_tags_11 },
    { 0x18bbd9e1, SUC_lexicon_tags_17 },
    { 0x6a9bd9e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3bd9e8, SUC_lexicon_tags_15 },
    { 0xf1bbd9e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dbd9ef, SUC_lexicon_tags_7 },
    { 0x253bd9f0, SUC_lexicon_tags_84 },
    { 0x9a5bd9f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x275bd9f3, SUC_lexicon_tags_9 },
    { 0xd5bd9f4, SUC_lexicon_tags_12 },
    { 0x2ffbd9f4, SUC_lexicon_tags_87 },
    { 0xf71bd9f6, SUC_lexicon_tags_15 },
    { 0xa5bbd9f7, SUC_lexicon_tags_37 },
    { 0x865bd9f8, SUC_lexicon_tags_29 },
    { 0x4f7bd9f9, SUC_lexicon_tags_7 },
    { 0x7c9bd9f8, SUC_lexicon_tags_63 },
    { 0x8efbd9f6, SUC_lexicon_tags_14 },
    { 0xcfdbd9fc, SUC_lexicon_tags_17 },
    { 0x9bfbd9fa, SUC_lexicon_tags_45 },
    { 0x4c3bd9fe, SUC_lexicon_tags_37 },
    { 0xbe9bd9f6, SUC_lexicon_tags_11 },
    { 0x8f5bda00, SUC_lexicon_tags_31 },
    { 0x8e9bda01, SUC_lexicon_tags_47 },
    { 0xd5bbd9f4, SUC_lexicon_tags_11 },
    { 0x4d9bda03, SUC_lexicon_tags_16 },
    { 0x9bdbda04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1bda09, SUC_lexicon_tags_23 },
    { 0xb57bda09, SUC_lexicon_tags_1 },
    { 0x64fbda09, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbf5bda0d, SUC_lexicon_tags_47 },
    { 0xc4fbda0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa97bda10, SUC_lexicon_tags_7 },
    { 0xc33bda11, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdbda16, SUC_lexicon_tags_10 },
    { 0xa57bda17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddbda1a, SUC_lexicon_tags_77 },
    { 0xcedbda1b, SUC_lexicon_tags_11 },
    { 0x833bda1a, SUC_lexicon_tags_38 },
    { 0x4fbbda1a, SUC_lexicon_tags_9 },
    { 0xdb7bda1e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11bda24, SUC_lexicon_tags_33 },
    { 0xe35bda25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1a1bda27, SUC_lexicon_tags_11 },
    { 0xaafbda28, SUC_lexicon_tags_23 },
    { 0x7e5bda29, SUC_lexicon_tags_25 },
    { 0x2d3bda2a, SUC_lexicon_tags_38 },
    { 0x9bdbda2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21fbda2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bbda31, SUC_lexicon_tags_0 },
    { 0x8c3bda32, SUC_lexicon_tags_25 },
    { 0xaf5bda32, SUC_lexicon_tags_38 },
    { 0x4d5bda32, SUC_lexicon_tags_0 },
    { 0x427bda31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfbda3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74bbda3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x157bda3e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x1a5bda40, SUC_lexicon_tags_15 },
    { 0xd39bda41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7bda45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe93bda47, SUC_lexicon_tags_11 },
    { 0xbd9bda48, SUC_lexicon_tags_15 },
    { 0x48fbda48, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53bda4c, SUC_lexicon_tags_14 },
    { 0x90bbda4d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bbda52, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401bda56, SUC_lexicon_tags_11 },
    { 0x7a1bda57, SUC_lexicon_tags_66 },
    { 0xcc1bda58, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcbbda5a, SUC_lexicon_tags_17 },
    { 0x497bda5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1bda5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9bda61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabbda65, SUC_lexicon_tags_25 },
    { 0x103bda65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25bda69, SUC_lexicon_tags_65 },
    { 0x20bbda69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfbda6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd1bda70, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x641bda73, SUC_lexicon_tags_11 },
    { 0xa69bda74, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5bda77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffbda7b, SUC_lexicon_tags_1 },
    { 0x3b3bda7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xec5bda7e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9bda85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7bda89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bbda8c, SUC_lexicon_tags_8 },
    { 0x585bda8d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79bda93, SUC_lexicon_tags_2 },
    { 0xfebbda94, SUC_lexicon_tags_28 },
    { 0xfb7bda95, SUC_lexicon_tags_37 },
    { 0x3c3bda96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5bda99, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9bda9c, SUC_lexicon_tags_49 },
    { 0x56fbda9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1bdaa2, SUC_lexicon_tags_17 },
    { 0x5e1bdaa3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x19bdaa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfbdaab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407bdab2, SUC_lexicon_tags_3 },
    { 0xa17bdab3, SUC_lexicon_tags_39 },
    { 0x561bdab3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bbdab7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e7bdab9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1bdabe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fbdac6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x26fbdac8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51bdacc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39bdad6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55bdade, SUC_lexicon_tags_15 },
    { 0x1a1bdadf, SUC_lexicon_tags_31 },
    { 0x9cdbdae0, SUC_lexicon_tags_2 },
    { 0x4bbdae1, SUC_lexicon_tags_25 },
    { 0x413bdae2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3bdaee, SUC_lexicon_tags_8 },
    { 0x64dbdaef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bdaf5, SUC_lexicon_tags_11 },
    { 0x25bdaf5, SUC_lexicon_tags_32 },
    { 0xb45bdaf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5bdafe, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83bdb01, SUC_lexicon_tags_33 },
    { 0x3fdbdb01, SUC_lexicon_tags_1 },
    { 0x125bdb01, SUC_lexicon_tags_9 },
    { 0x32dbdb04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x327bdb06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487bdb0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83dbdb0d, SUC_lexicon_tags_3 },
    { 0x65bbdb0e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x96fbdb10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf71bdb12, SUC_lexicon_tags_11 },
    { 0x5bdbdb13, SUC_lexicon_tags_14 },
    { 0x53dbdb12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93bdb19, SUC_lexicon_tags_11 },
    { 0x87fbdb1a, SUC_lexicon_tags_11 },
    { 0x329bdb1b, SUC_lexicon_tags_34 },
    { 0x431bdb1c, SUC_lexicon_tags_39 },
    { 0x4e1bdb1a, SUC_lexicon_tags_68 },
    { 0xa1bdb1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde7bdb20, SUC_lexicon_tags_1 },
    { 0x74dbdb21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9bdb24, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dbdb31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5bdb35, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369bdb3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfdbdb40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x475bdb43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91bdb4c, SUC_lexicon_tags_11 },
    { 0x4ffbdb4c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9bdb52, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x903bdb55, SUC_lexicon_tags_46 },
    { 0x967bdb56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe9bbdb58, SUC_lexicon_tags_28 },
    { 0x3a7bdb58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabbdb5f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77bdb63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x51dbdb66, SUC_lexicon_tags_12 },
    { 0x901bdb66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1bdb6b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1bdb6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361bdb76, SUC_lexicon_tags_14 },
    { 0x701bdb77, SUC_lexicon_tags_11 },
    { 0xb25bdb78, SUC_lexicon_tags_1 },
    { 0x96fbdb79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9bdb7f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x59fbdb81, SUC_lexicon_tags_21 },
    { 0x151bdb82, SUC_lexicon_tags_2 },
    { 0x1bbdb81, SUC_lexicon_tags_37 },
    { 0x1e3bdb83, SUC_lexicon_tags_11 },
    { 0x95bbdb85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85bdb88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bbdb8b, SUC_lexicon_tags_49 },
    { 0xc01bdb8b, SUC_lexicon_tags_11 },
    { 0x97bbdb8d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5bdb93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bdb98, SUC_lexicon_tags_37 },
    { 0xd87bdb98, SUC_lexicon_tags_43 },
    { 0xda9bdb98, SUC_lexicon_tags_7 },
    { 0xbbbdb9b, SUC_lexicon_tags_11 },
    { 0x149bdb9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x97fbdb9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7bdba3, SUC_lexicon_tags_37 },
    { 0x6b1bdba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x141bdba6, SUC_lexicon_tags_0 },
    { 0xc4bbdba6, SUC_lexicon_tags_12 },
    { 0xc17bdba8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9bbdbaa, SUC_lexicon_tags_45 },
    { 0xf39bdbab, SUC_lexicon_tags_12 },
    { 0x10dbdbab, SUC_lexicon_tags_65 },
    { 0xcd9bdbab, SUC_lexicon_tags_38 },
    { 0xbdfbdbac, SUC_lexicon_tags_12 },
    { 0xb4bbdbaf, SUC_lexicon_tags_5 },
    { 0x849bdbb0, SUC_lexicon_tags_17 },
    { 0x619bdbb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x22fbdbb4, SUC_lexicon_tags_30 },
    { 0x403bdbb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18fbdbb8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc9dbdbba, SUC_lexicon_tags_15 },
    { 0x93dbdbbb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbdbc1, SUC_lexicon_tags_33 },
    { 0xe83bdbc2, SUC_lexicon_tags_11 },
    { 0x7d5bdbc3, SUC_lexicon_tags_1 },
    { 0x565bdbc3, SUC_lexicon_tags_2 },
    { 0x623bdbc1, SUC_lexicon_tags_38 },
    { 0xd41bdbc6, SUC_lexicon_tags_2 },
    { 0x247bdbc7, SUC_lexicon_tags_37 },
    { 0x377bdbc8, SUC_lexicon_tags_37 },
    { 0xfd5bdbc8, SUC_lexicon_tags_2 },
    { 0x48bbdbc9, SUC_lexicon_tags_7 },
    { 0x337bdbc4, SUC_lexicon_tags_2 },
    { 0x58fbdbcc, SUC_lexicon_tags_15 },
    { 0x34dbdbcd, SUC_lexicon_tags_47 },
    { 0x7b9bdbcc, SUC_lexicon_tags_38 },
    { 0xc95bdbcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbbdbd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abbdbda, SUC_lexicon_tags_13 },
    { 0x641bdbda, SUC_lexicon_tags_23 },
    { 0x8f9bdbda, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x209bdbde, SUC_lexicon_tags_1 },
    { 0x855bdbdf, SUC_lexicon_tags_125 },
    { 0xa91bdbe0, SUC_lexicon_tags_114 },
    { 0x61fbdbe0, SUC_lexicon_tags_82 },
    { 0xb49bdbe2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5bdbeb, SUC_lexicon_tags_5 },
    { 0x97fbdbec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69bdbee, SUC_lexicon_tags_12 },
    { 0x6c7bdbef, SUC_lexicon_tags_23 },
    { 0x42dbdbef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dbdbf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43dbdbf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1bdbf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x273bdbfb, SUC_lexicon_tags_30 },
    { 0x779bdbfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27bdc02, SUC_lexicon_tags_17 },
    { 0x3efbdc03, SUC_lexicon_tags_113 },
    { 0xd63bdc04, SUC_lexicon_tags_23 },
    { 0xff9bdc05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x849bdc07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x615bdc0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5bdc15, SUC_lexicon_tags_13 },
    { 0xcddbdc16, SUC_lexicon_tags_53 },
    { 0x799bdc17, SUC_lexicon_tags_29 },
    { 0x1c7bdc17, SUC_lexicon_tags_15 },
    { 0xe5bbdc19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5bdc20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x375bdc23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x601bdc25, SUC_lexicon_tags_29 },
    { 0x303bdc26, SUC_lexicon_tags_25 },
    { 0x493bdc25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x659bdc2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa7bdc2c, SUC_lexicon_tags_38 },
    { 0xa59bdc2c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fbdc31, SUC_lexicon_tags_63 },
    { 0xf27bdc31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bdc37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c1bdc39, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbdc3d, SUC_lexicon_tags_45 },
    { 0xbf9bdc3d, SUC_lexicon_tags_9 },
    { 0x6bbdc3f, SUC_lexicon_tags_32 },
    { 0x82dbdc3d, SUC_lexicon_tags_7 },
    { 0x925bdc41, SUC_lexicon_tags_6 },
    { 0x63bbdc3f, SUC_lexicon_tags_80 },
    { 0xc85bdc43, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9bdc4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edbdc52, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b5bdc54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9bdc59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3bdc60, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xff1bdc62, SUC_lexicon_tags_2 },
    { 0xcbbbdc62, SUC_lexicon_tags_37 },
    { 0xb89bdc63, SUC_lexicon_tags_29 },
    { 0xd3bbdc65, SUC_lexicon_tags_85 },
    { 0x2c7bdc65, SUC_lexicon_tags_15 },
    { 0xadbdc64, SUC_lexicon_tags_60 },
    { 0x57bdc68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3bbbdc6a, SUC_lexicon_tags_2 },
    { 0x30bbdc6a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27bdc6e, SUC_lexicon_tags_9 },
    { 0xd99bdc6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5f3bdc71, SUC_lexicon_tags_25 },
    { 0x8dfbdc72, SUC_lexicon_tags_40 },
    { 0xfb1bdc72, SUC_lexicon_tags_11 },
    { 0x345bdc71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dbdc79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9bdc83, SUC_lexicon_tags_45 },
    { 0x61bbdc84, SUC_lexicon_tags_11 },
    { 0xcbfbdc84, SUC_lexicon_tags_0 },
    { 0x1bfbdc84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ffbdc88, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665bdc8c, SUC_lexicon_tags_11 },
    { 0xdfbbdc8c, SUC_lexicon_tags_37 },
    { 0xa7bbdc8e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2e3bdc90, SUC_lexicon_tags_84 },
    { 0x8a7bdc90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105bdc95, SUC_lexicon_tags_2 },
    { 0x4e9bdc96, SUC_lexicon_tags_15 },
    { 0x84dbdc97, SUC_lexicon_tags_17 },
    { 0x791bdc96, SUC_lexicon_tags_43 },
    { 0x8cbbdc97, SUC_lexicon_tags_27 },
    { 0xb4dbdc9a, SUC_lexicon_tags_6 },
    { 0xb5fbdc9b, SUC_lexicon_tags_0 },
    { 0x26fbdc9c, SUC_lexicon_tags_26 },
    { 0xf2dbdc9b, SUC_lexicon_tags_82 },
    { 0x7fbbdc99, SUC_lexicon_tags_11 },
    { 0xaa9bdc9c, SUC_lexicon_tags_0 },
    { 0x249bdca0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc97bdca2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7cbbdca4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bbdca8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3bdcb0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4efbdcb2, SUC_lexicon_tags_17 },
    { 0x267bdcb3, SUC_lexicon_tags_38 },
    { 0x40fbdcb4, SUC_lexicon_tags_33 },
    { 0x387bdcb5, SUC_lexicon_tags_31 },
    { 0xf2bbdcb6, SUC_lexicon_tags_7 },
    { 0x1dbbdcb3, SUC_lexicon_tags_1711 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91bdcbb, SUC_lexicon_tags_16 },
    { 0x50fbdcbc, SUC_lexicon_tags_15 },
    { 0xa27bdcbd, SUC_lexicon_tags_0 },
    { 0xc6dbdcbe, SUC_lexicon_tags_25 },
    { 0x89dbdcbf, SUC_lexicon_tags_12 },
    { 0x355bdcbd, SUC_lexicon_tags_17 },
    { 0x1f1bdcc1, SUC_lexicon_tags_37 },
    { 0x903bdcbc, SUC_lexicon_tags_15 },
    { 0x183bdcc1, SUC_lexicon_tags_27 },
    { 0x2edbdcbb, SUC_lexicon_tags_82 },
    { 0x77dbdcc4, SUC_lexicon_tags_12 },
    { 0xb9bbdcc6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bbdccb, SUC_lexicon_tags_17 },
    { 0x555bdccc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e3bdcce, SUC_lexicon_tags_15 },
    { 0x41dbdccf, SUC_lexicon_tags_17 },
    { 0x9e1bdcd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401bdcd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bbbdcda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bbdce4, SUC_lexicon_tags_43 },
    { 0x673bdce4, SUC_lexicon_tags_17 },
    { 0x3d3bdce6, SUC_lexicon_tags_35 },
    { 0x9b3bdce6, SUC_lexicon_tags_47 },
    { 0xddbdce8, SUC_lexicon_tags_30 },
    { 0x699bdce9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951bdcef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d3bdcf1, SUC_lexicon_tags_16 },
    { 0x731bdcf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x735bdcf4, SUC_lexicon_tags_75 },
    { 0xdddbdcf4, SUC_lexicon_tags_27 },
    { 0x6cdbdcf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5bdcf9, SUC_lexicon_tags_43 },
    { 0xfcfbdcf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x279bdcfd, SUC_lexicon_tags_11 },
    { 0xe5fbdcfe, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0x1a1bdd00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631bdd05, SUC_lexicon_tags_7 },
    { 0xc9fbdd05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bbdd0b, SUC_lexicon_tags_419 },
    { 0xc79bdd0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaa5bdd0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81bdd12, SUC_lexicon_tags_8 },
    { 0xfffbdd13, SUC_lexicon_tags_28 },
    { 0x781bdd14, SUC_lexicon_tags_68 },
    { 0xba3bdd15, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0xb89bdd17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb91bdd19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fbdd1e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb71bdd20, SUC_lexicon_tags_2 },
    { 0x24fbdd20, SUC_lexicon_tags_46 },
    { 0x3c9bdd20, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x417bdd24, SUC_lexicon_tags_11 },
    { 0x2f7bdd25, SUC_lexicon_tags_1 },
    { 0x977bdd26, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47bdd2d, SUC_lexicon_tags_57 },
    { 0x415bdd2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x869bdd31, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773bdd3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7bdd3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3bdd42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5dbdd44, SUC_lexicon_tags_11 },
    { 0x5c5bdd45, SUC_lexicon_tags_30 },
    { 0x5b1bdd46, SUC_lexicon_tags_29 },
    { 0x577bdd46, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xf1fbdd49, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fbdd4d, SUC_lexicon_tags_62 },
    { 0xc3dbdd4e, SUC_lexicon_tags_2 },
    { 0x97bdd4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449bdd54, SUC_lexicon_tags_11 },
    { 0x861bdd54, SUC_lexicon_tags_7 },
    { 0x72bbdd55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x139bdd58, SUC_lexicon_tags_12 },
    { 0xf03bdd58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dbdd5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1e3bdd5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd81bdd61, SUC_lexicon_tags_45 },
    { 0x707bdd62, SUC_lexicon_tags_1 },
    { 0x6bdbdd63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641bdd6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x181bdd70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x419bdd72, SUC_lexicon_tags_1 },
    { 0x139bdd72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3bdd76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1bdd79, SUC_lexicon_tags_15 },
    { 0x3b1bdd7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x637bdd7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x787bdd80, SUC_lexicon_tags_25 },
    { 0x75dbdd81, SUC_lexicon_tags_27 },
    { 0xfc9bdd82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fbdd88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1bdd8c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd9bbdd8e, SUC_lexicon_tags_7 },
    { 0xbf5bdd8f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffbdd92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd5bdd94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3bdd97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969bdd9e, SUC_lexicon_tags_17 },
    { 0x21fbdd9f, SUC_lexicon_tags_91 },
    { 0x153bdda0, SUC_lexicon_tags_2 },
    { 0x841bdda0, SUC_lexicon_tags_45 },
    { 0x6f5bdda2, SUC_lexicon_tags_70 },
    { 0xe81bdda0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ebbdda5, SUC_lexicon_tags_15 },
    { 0x5d3bdda6, SUC_lexicon_tags_0 },
    { 0xcffbdda7, SUC_lexicon_tags_6 },
    { 0x197bdda7, SUC_lexicon_tags_7 },
    { 0x5b5bdda7, SUC_lexicon_tags_25 },
    { 0xb37bddaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa59bddac, SUC_lexicon_tags_87 },
    { 0x49fbddad, SUC_lexicon_tags_37 },
    { 0x553bddad, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdbddb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb2dbddb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xecbbddb5, SUC_lexicon_tags_17 },
    { 0x7ebbddb6, SUC_lexicon_tags_14 },
    { 0xcbbddb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d5bddb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15bddbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdbddbf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdbddc2, SUC_lexicon_tags_35 },
    { 0x561bddc2, SUC_lexicon_tags_1 },
    { 0xe6bbddc4, SUC_lexicon_tags_7 },
    { 0xccfbddc5, SUC_lexicon_tags_49 },
    { 0x539bddc2, SUC_lexicon_tags_5 },
    { 0x7b9bddc7, SUC_lexicon_tags_2 },
    { 0xde9bddc7, SUC_lexicon_tags_34 },
    { 0x4a1bddc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb7bddcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7bddce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73bddd2, SUC_lexicon_tags_32 },
    { 0xed7bddd3, SUC_lexicon_tags_17 },
    { 0xf39bddd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721bddd8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3bdddc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6fbddde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d7bdde0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbbdde5, SUC_lexicon_tags_17 },
    { 0xd4fbdde6, SUC_lexicon_tags_25 },
    { 0x41dbdde7, SUC_lexicon_tags_25 },
    { 0x197bdde7, SUC_lexicon_tags_4 },
    { 0x485bdde8, SUC_lexicon_tags_4 },
    { 0x549bddea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x887bddec, SUC_lexicon_tags_31 },
    { 0x36bbdded, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3bddf0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7bddf5, SUC_lexicon_tags_11 },
    { 0x1d7bddf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdffbddf8, SUC_lexicon_tags_26 },
    { 0xcb3bddf9, SUC_lexicon_tags_10 },
    { 0x77fbddf9, SUC_lexicon_tags_1 },
    { 0x731bddfb, SUC_lexicon_tags_11 },
    { 0xbadbddf9, SUC_lexicon_tags_32 },
    { 0x55dbddfd, SUC_lexicon_tags_3 },
    { 0xf13bddf8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135bde05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfffbde07, SUC_lexicon_tags_33 },
    { 0xc49bde07, SUC_lexicon_tags_15 },
    { 0x1fbde09, SUC_lexicon_tags_19 },
    { 0x667bde09, SUC_lexicon_tags_47 },
    { 0x30bbde0b, SUC_lexicon_tags_46 },
    { 0x305bde0c, SUC_lexicon_tags_25 },
    { 0x447bde0d, SUC_lexicon_tags_15 },
    { 0x86bbde0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafbde12, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5bde15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x71bde17, SUC_lexicon_tags_25 },
    { 0xa31bde18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961bde1e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x979bde20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdbde23, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323bde28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5bde2c, SUC_lexicon_tags_1 },
    { 0xb4fbde2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11bde2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fbde3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xab5bde3d, SUC_lexicon_tags_2 },
    { 0xadfbde3e, SUC_lexicon_tags_16 },
    { 0x1e7bde3f, SUC_lexicon_tags_45 },
    { 0x4c9bde3f, SUC_lexicon_tags_15 },
    { 0xe9bde3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445bde45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1bde49, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dbde4d, SUC_lexicon_tags_28 },
    { 0x22fbde4d, SUC_lexicon_tags_12 },
    { 0x923bde4f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195bde54, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf49bde56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bbde59, SUC_lexicon_tags_15 },
    { 0x15dbde5a, SUC_lexicon_tags_7 },
    { 0x567bde5b, SUC_lexicon_tags_37 },
    { 0x4dfbde5c, SUC_lexicon_tags_64 },
    { 0x8cbbde5d, SUC_lexicon_tags_15 },
    { 0x679bde5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819bde63, SUC_lexicon_tags_11 },
    { 0x9fbde63, SUC_lexicon_tags_37 },
    { 0x8efbde65, SUC_lexicon_tags_27 },
    { 0x369bde65, SUC_lexicon_tags_18 },
    { 0xa35bde67, SUC_lexicon_tags_28 },
    { 0x26fbde68, SUC_lexicon_tags_17 },
    { 0x6afbde68, SUC_lexicon_tags_17 },
    { 0x499bde6a, SUC_lexicon_tags_25 },
    { 0x1b3bde6b, SUC_lexicon_tags_107 },
    { 0x541bde68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fbde72, SUC_lexicon_tags_7 },
    { 0x6d5bde73, SUC_lexicon_tags_38 },
    { 0x633bde73, SUC_lexicon_tags_25 },
    { 0x69dbde73, SUC_lexicon_tags_0 },
    { 0xe45bde72, SUC_lexicon_tags_15 },
    { 0xe35bde77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbbde7c, SUC_lexicon_tags_7 },
    { 0x791bde7c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fbde88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dfbde8a, SUC_lexicon_tags_6 },
    { 0x115bde8b, SUC_lexicon_tags_11 },
    { 0x207bde8c, SUC_lexicon_tags_37 },
    { 0xa63bde8d, SUC_lexicon_tags_70 },
    { 0xe1bbde8e, SUC_lexicon_tags_65 },
    { 0xfcdbde8e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdfbde91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x529bde93, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbde96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbbde9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2b1bde9e, SUC_lexicon_tags_57 },
    { 0xea1bde9f, SUC_lexicon_tags_25 },
    { 0x293bde9f, SUC_lexicon_tags_10 },
    { 0x1bfbdea1, SUC_lexicon_tags_66 },
    { 0x505bdea1, SUC_lexicon_tags_28 },
    { 0x1a7bdea2, SUC_lexicon_tags_27 },
    { 0x7cdbdea4, SUC_lexicon_tags_19 },
    { 0x49dbdea4, SUC_lexicon_tags_2 },
    { 0x831bde9e, SUC_lexicon_tags_46 },
    { 0xcd1bde9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1bdea9, SUC_lexicon_tags_17 },
    { 0xff5bdea9, SUC_lexicon_tags_7 },
    { 0x9cfbdeab, SUC_lexicon_tags_19 },
    { 0x5d7bdeab, SUC_lexicon_tags_69 },
    { 0xa63bdeac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdbdeb0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3bdeb3, SUC_lexicon_tags_23 },
    { 0xe83bdeb4, SUC_lexicon_tags_27 },
    { 0x78fbdeb5, SUC_lexicon_tags_32 },
    { 0x977bdeb6, SUC_lexicon_tags_87 },
    { 0xe99bdeb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bbdebb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1bdec1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc57bdec3, SUC_lexicon_tags_17 },
    { 0x81bdec4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95bdec8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x89bdeca, SUC_lexicon_tags_17 },
    { 0x529bdeca, SUC_lexicon_tags_37 },
    { 0xdc7bdecb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f5bdece, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bbded2, SUC_lexicon_tags_47 },
    { 0xb55bded3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87bded6, SUC_lexicon_tags_151 },
    { 0xdebbded7, SUC_lexicon_tags_2 },
    { 0xb17bded8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf09bdeda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3bdee1, SUC_lexicon_tags_28 },
    { 0x2c7bdee2, SUC_lexicon_tags_9 },
    { 0xfedbdee3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527bdeea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7afbdeed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffbdef0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x385bdef3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x82bbdef5, SUC_lexicon_tags_23 },
    { 0xf99bdef6, SUC_lexicon_tags_12 },
    { 0x387bdef6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x15bbdef9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bbdefc, SUC_lexicon_tags_7 },
    { 0x81bdefd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3bdf00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86bbdf02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5bdf07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1bdf14, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x191bdf16, SUC_lexicon_tags_17 },
    { 0x5a9bdf16, SUC_lexicon_tags_15 },
    { 0x9ebbdf17, SUC_lexicon_tags_1 },
    { 0xce7bdf17, SUC_lexicon_tags_39 },
    { 0xe77bdf1a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07bdf1f, SUC_lexicon_tags_7 },
    { 0x447bdf1f, SUC_lexicon_tags_7 },
    { 0x341bdf1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963bdf28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751bdf32, SUC_lexicon_tags_25 },
    { 0x493bdf33, SUC_lexicon_tags_2 },
    { 0x3bbbdf33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbbdf39, SUC_lexicon_tags_36 },
    { 0x26dbdf3a, SUC_lexicon_tags_43 },
    { 0xa9dbdf39, SUC_lexicon_tags_38 },
    { 0x6bbbdf3b, SUC_lexicon_tags_17 },
    { 0x62bbdf39, SUC_lexicon_tags_19 },
    { 0x6ddbdf3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1fbdf40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3bdf44, SUC_lexicon_tags_60 },
    { 0x54dbdf45, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99bdf4d, SUC_lexicon_tags_7 },
    { 0x3d7bdf4e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fbdf55, SUC_lexicon_tags_8 },
    { 0x58dbdf56, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5bdf5a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfbdf5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbbdf60, SUC_lexicon_tags_8 },
    { 0xadbdf61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd33bdf63, SUC_lexicon_tags_7 },
    { 0xb47bdf64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7bdf68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba5bdf6a, SUC_lexicon_tags_37 },
    { 0xf65bdf6b, SUC_lexicon_tags_25 },
    { 0x99bdf6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc91bdf6e, SUC_lexicon_tags_12 },
    { 0xaf9bdf6f, SUC_lexicon_tags_38 },
    { 0xf7bbdf6f, SUC_lexicon_tags_25 },
    { 0x24fbdf71, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bbdf75, SUC_lexicon_tags_11 },
    { 0xbdbbdf76, SUC_lexicon_tags_12 },
    { 0x11bbdf75, SUC_lexicon_tags_15 },
    { 0xefdbdf77, SUC_lexicon_tags_33 },
    { 0xfdbbdf79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddbdf7e, SUC_lexicon_tags_33 },
    { 0xb97bdf7f, SUC_lexicon_tags_45 },
    { 0x91bdf80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc63bdf82, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbdf88, SUC_lexicon_tags_15 },
    { 0x237bdf89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe53bdf8b, SUC_lexicon_tags_37 },
    { 0x627bdf8c, SUC_lexicon_tags_45 },
    { 0x1d9bdf8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79bbdf8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76dbdf91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157bdf95, SUC_lexicon_tags_15 },
    { 0x95bdf95, SUC_lexicon_tags_54 },
    { 0x1c5bdf97, SUC_lexicon_tags_15 },
    { 0x45bbdf95, SUC_lexicon_tags_26 },
    { 0x961bdf99, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3dbdf9b, SUC_lexicon_tags_2 },
    { 0xe67bdf9c, SUC_lexicon_tags_25 },
    { 0xab3bdf9b, SUC_lexicon_tags_21 },
    { 0x8adbdf9c, SUC_lexicon_tags_129 },
    { 0x975bdf9f, SUC_lexicon_tags_8 },
    { 0x3e9bdf9f, SUC_lexicon_tags_15 },
    { 0x23fbdf9c, SUC_lexicon_tags_8 },
    { 0x241bdfa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09bdfa5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557bdfa9, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485bdfae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x529bdfb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53dbdfb2, SUC_lexicon_tags_61 },
    { 0x51bdfb2, SUC_lexicon_tags_8 },
    { 0xe97bdfb4, SUC_lexicon_tags_25 },
    { 0xe2fbdfb5, SUC_lexicon_tags_44 },
    { 0xbd3bdfb4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfbdfb9, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x67bdfbb, SUC_lexicon_tags_26 },
    { 0x4bfbdfbc, SUC_lexicon_tags_19 },
    { 0x2efbdfbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9bdfc1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49dbdfc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x207bdfc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3bdfd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9bdfe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17bdfe6, SUC_lexicon_tags_14 },
    { 0xf69bdfe7, SUC_lexicon_tags_4 },
    { 0xcd1bdfe8, SUC_lexicon_tags_14 },
    { 0x9abbdfe7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5bdff3, SUC_lexicon_tags_0 },
    { 0x829bdff4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7bdff7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71bdffa, SUC_lexicon_tags_25 },
    { 0x10bbdffb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07bdfff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abbe004, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc59be006, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe9bbe008, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2dbe00a, SUC_lexicon_tags_30 },
    { 0xcabbe00a, SUC_lexicon_tags_23 },
    { 0xe41be00c, SUC_lexicon_tags_7 },
    { 0x8c1be00b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199be013, SUC_lexicon_tags_31 },
    { 0xd07be014, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47be01b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5be01e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef5be020, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209be025, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6abbe027, SUC_lexicon_tags_23 },
    { 0xefbbe028, SUC_lexicon_tags_2 },
    { 0x1abbe027, SUC_lexicon_tags_1 },
    { 0x6ffbe02a, SUC_lexicon_tags_25 },
    { 0xf4fbe028, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69be02e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde5be030, SUC_lexicon_tags_60 },
    { 0xa97be031, SUC_lexicon_tags_27 },
    { 0x8d9be031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x683be034, SUC_lexicon_tags_26 },
    { 0xbcbbe034, SUC_lexicon_tags_15 },
    { 0x231be035, SUC_lexicon_tags_47 },
    { 0x60fbe037, SUC_lexicon_tags_17 },
    { 0x3c5be038, SUC_lexicon_tags_11 },
    { 0x12fbe035, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe39be03b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fbe040, SUC_lexicon_tags_669 },
    { 0, NULL },
    { 0xf85be042, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d9be044, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911be048, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46bbe04a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bbe050, SUC_lexicon_tags_351 },
    { 0xd4dbe050, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb83be053, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdbe056, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x893be059, SUC_lexicon_tags_17 },
    { 0xa67be059, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2efbe05d, SUC_lexicon_tags_235 },
    { 0xed1be05e, SUC_lexicon_tags_7 },
    { 0xfbe05f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7be064, SUC_lexicon_tags_14 },
    { 0xfcdbe065, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1be06c, SUC_lexicon_tags_37 },
    { 0x997be06d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23be071, SUC_lexicon_tags_2 },
    { 0x1fdbe071, SUC_lexicon_tags_16 },
    { 0x31fbe073, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dbe076, SUC_lexicon_tags_17 },
    { 0x13dbe077, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fbe07a, SUC_lexicon_tags_17 },
    { 0xd85be07b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x911be07d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3be080, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fbe08d, SUC_lexicon_tags_25 },
    { 0x65bbe08e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2bbe090, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9be096, SUC_lexicon_tags_68 },
    { 0x3fdbe097, SUC_lexicon_tags_49 },
    { 0xe3bbe097, SUC_lexicon_tags_47 },
    { 0xcedbe096, SUC_lexicon_tags_37 },
    { 0xd89be09a, SUC_lexicon_tags_37 },
    { 0x453be09b, SUC_lexicon_tags_28 },
    { 0xfadbe09b, SUC_lexicon_tags_49 },
    { 0xd31be09b, SUC_lexicon_tags_7 },
    { 0x631be09b, SUC_lexicon_tags_8 },
    { 0x43bbe09f, SUC_lexicon_tags_87 },
    { 0x651be0a0, SUC_lexicon_tags_55 },
    { 0x9dbe09f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x177be0a3, SUC_lexicon_tags_11 },
    { 0x6a1be0a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x327be0a7, SUC_lexicon_tags_38 },
    { 0xaffbe0a8, SUC_lexicon_tags_1 },
    { 0x145be0a8, SUC_lexicon_tags_47 },
    { 0x6b5be0a8, SUC_lexicon_tags_19 },
    { 0x77be0ab, SUC_lexicon_tags_19 },
    { 0xf19be0ac, SUC_lexicon_tags_37 },
    { 0xf69be0ac, SUC_lexicon_tags_17 },
    { 0x549be0ae, SUC_lexicon_tags_33 },
    { 0x9a9be0ae, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xca9be0b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf77be0b3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xef5be0b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdbe0bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbe0be, SUC_lexicon_tags_22 },
    { 0x8c7be0bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bfbe0c1, SUC_lexicon_tags_28 },
    { 0x505be0c2, SUC_lexicon_tags_1 },
    { 0xb2fbe0c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7be0c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225be0cd, SUC_lexicon_tags_17 },
    { 0x26dbe0ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7be0d2, SUC_lexicon_tags_12 },
    { 0x98bbe0d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x61fbe0d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291be0da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x765be0dc, SUC_lexicon_tags_29 },
    { 0xb83be0dc, SUC_lexicon_tags_44 },
    { 0x5fdbe0de, SUC_lexicon_tags_17 },
    { 0x385be0de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc4dbe0e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf75be0e3, SUC_lexicon_tags_10 },
    { 0x315be0e3, SUC_lexicon_tags_17 },
    { 0x65dbe0e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fbe0e8, SUC_lexicon_tags_68 },
    { 0x221be0e9, SUC_lexicon_tags_25 },
    { 0x339be0ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x369be0ec, SUC_lexicon_tags_35 },
    { 0x375be0ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83be0f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7be0f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x191be0f7, SUC_lexicon_tags_33 },
    { 0x3f5be0f8, SUC_lexicon_tags_27 },
    { 0x217be0f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365be0ff, SUC_lexicon_tags_38 },
    { 0xfe5be0ff, SUC_lexicon_tags_15 },
    { 0xc15be100, SUC_lexicon_tags_1 },
    { 0x73be101, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b7be104, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x151be106, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfefbe109, SUC_lexicon_tags_0 },
    { 0x571be10a, SUC_lexicon_tags_60 },
    { 0xf57be10a, SUC_lexicon_tags_9 },
    { 0xe0bbe10c, SUC_lexicon_tags_49 },
    { 0x67fbe10d, SUC_lexicon_tags_55 },
    { 0xdc9be10e, SUC_lexicon_tags_15 },
    { 0x419be10f, SUC_lexicon_tags_29 },
    { 0x7e9be109, SUC_lexicon_tags_7 },
    { 0xd93be10b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907be118, SUC_lexicon_tags_27 },
    { 0xf5fbe119, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x997be11c, SUC_lexicon_tags_0 },
    { 0xe97be11d, SUC_lexicon_tags_9 },
    { 0xb15be11e, SUC_lexicon_tags_60 },
    { 0xd7dbe11f, SUC_lexicon_tags_58 },
    { 0x5b5be120, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bbe128, SUC_lexicon_tags_17 },
    { 0x9cbbe129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65be12d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x349be130, SUC_lexicon_tags_2 },
    { 0xcf1be131, SUC_lexicon_tags_49 },
    { 0x93fbe132, SUC_lexicon_tags_126 },
    { 0x451be133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3be13a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489be13e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dbe143, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xeddbe146, SUC_lexicon_tags_21 },
    { 0x84dbe147, SUC_lexicon_tags_23 },
    { 0x32bbe146, SUC_lexicon_tags_25 },
    { 0x551be149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc77be14c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3be14f, SUC_lexicon_tags_37 },
    { 0x83bbe150, SUC_lexicon_tags_15 },
    { 0x94bbe151, SUC_lexicon_tags_60 },
    { 0xcb5be150, SUC_lexicon_tags_25 },
    { 0xb47be153, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17be156, SUC_lexicon_tags_27 },
    { 0x9f5be157, SUC_lexicon_tags_7 },
    { 0xb67be157, SUC_lexicon_tags_14 },
    { 0xac1be157, SUC_lexicon_tags_57 },
    { 0xd03be15a, SUC_lexicon_tags_45 },
    { 0x867be15b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795be15f, SUC_lexicon_tags_12 },
    { 0xb35be160, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59be164, SUC_lexicon_tags_37 },
    { 0x793be165, SUC_lexicon_tags_26 },
    { 0xd73be166, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fbe16b, SUC_lexicon_tags_37 },
    { 0xeedbe16c, SUC_lexicon_tags_11 },
    { 0x45be16d, SUC_lexicon_tags_2 },
    { 0xd07be16e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d1be170, SUC_lexicon_tags_16 },
    { 0xb05be170, SUC_lexicon_tags_28 },
    { 0x7f1be172, SUC_lexicon_tags_17 },
    { 0xeb7be173, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x961be175, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x309be177, SUC_lexicon_tags_25 },
    { 0x1c5be178, SUC_lexicon_tags_49 },
    { 0xc49be177, SUC_lexicon_tags_33 },
    { 0x641be179, SUC_lexicon_tags_17 },
    { 0xa0dbe178, SUC_lexicon_tags_15 },
    { 0xf67be179, SUC_lexicon_tags_47 },
    { 0xaf5be179, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x41bbe17f, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xb7be181, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x181be184, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8e1be186, SUC_lexicon_tags_37 },
    { 0xcc9be186, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3be18a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f1be18c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1be195, SUC_lexicon_tags_45 },
    { 0x1d5be195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29be198, SUC_lexicon_tags_25 },
    { 0xd73be199, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c3be19b, SUC_lexicon_tags_14 },
    { 0x9efbe19c, SUC_lexicon_tags_664 },
    { 0xed3be19d, SUC_lexicon_tags_4 },
    { 0xb03be19e, SUC_lexicon_tags_47 },
    { 0xb7be19f, SUC_lexicon_tags_1 },
    { 0x47dbe1a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fbe1a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855be1a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899be1ad, SUC_lexicon_tags_39 },
    { 0x9f9be1ae, SUC_lexicon_tags_65 },
    { 0x8afbe1af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0dbe1b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x453be1b4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fbe1b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31be1bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795be1c9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fbe1cd, SUC_lexicon_tags_5 },
    { 0x8c3be1ce, SUC_lexicon_tags_42 },
    { 0x4d7be1cf, SUC_lexicon_tags_17 },
    { 0x76dbe1d0, SUC_lexicon_tags_16 },
    { 0x6bfbe1d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567be1d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683be1d9, SUC_lexicon_tags_7 },
    { 0x763be1d9, SUC_lexicon_tags_56 },
    { 0x2bfbe1d9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdbe1de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9be1e1, SUC_lexicon_tags_25 },
    { 0x643be1e2, SUC_lexicon_tags_11 },
    { 0x789be1e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261be1e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fbe1eb, SUC_lexicon_tags_491 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddbe1ef, SUC_lexicon_tags_11 },
    { 0x8d9be1f0, SUC_lexicon_tags_6 },
    { 0x879be1ef, SUC_lexicon_tags_13 },
    { 0x3f9be1f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x413be1f4, SUC_lexicon_tags_37 },
    { 0x87bbe1f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1be1fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295be200, SUC_lexicon_tags_31 },
    { 0x961be201, SUC_lexicon_tags_15 },
    { 0x899be202, SUC_lexicon_tags_37 },
    { 0x9e9be202, SUC_lexicon_tags_56 },
    { 0xcedbe203, SUC_lexicon_tags_277 },
    { 0x873be201, SUC_lexicon_tags_16 },
    { 0x819be204, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49be20a, SUC_lexicon_tags_9 },
    { 0xdd7be20a, SUC_lexicon_tags_43 },
    { 0x611be20b, SUC_lexicon_tags_47 },
    { 0xedbbe20d, SUC_lexicon_tags_12 },
    { 0xd1be20e, SUC_lexicon_tags_25 },
    { 0x26dbe20b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81be212, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfbe215, SUC_lexicon_tags_12 },
    { 0x78bbe216, SUC_lexicon_tags_37 },
    { 0x17bbe215, SUC_lexicon_tags_3 },
    { 0xed9be218, SUC_lexicon_tags_11 },
    { 0x717be216, SUC_lexicon_tags_25 },
    { 0x477be21a, SUC_lexicon_tags_7 },
    { 0x37fbe215, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe99be21d, SUC_lexicon_tags_7 },
    { 0xf4dbe21d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x2afbe221, SUC_lexicon_tags_17 },
    { 0x3fbbe222, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafbe225, SUC_lexicon_tags_1 },
    { 0xbcbbe226, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97be22b, SUC_lexicon_tags_1 },
    { 0x3a1be22c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27dbe22f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7be232, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe7be234, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15dbe236, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3be239, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57fbe23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01be23e, SUC_lexicon_tags_66 },
    { 0xbc3be23f, SUC_lexicon_tags_37 },
    { 0xb57be240, SUC_lexicon_tags_30 },
    { 0xd7be240, SUC_lexicon_tags_60 },
    { 0xed9be242, SUC_lexicon_tags_39 },
    { 0xdf7be243, SUC_lexicon_tags_28 },
    { 0xd4bbe244, SUC_lexicon_tags_21 },
    { 0xa8fbe23e, SUC_lexicon_tags_15 },
    { 0x7ffbe242, SUC_lexicon_tags_4 },
    { 0x53fbe247, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3edbe249, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde1be24b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9be250, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93be254, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dbe258, SUC_lexicon_tags_15 },
    { 0xb51be259, SUC_lexicon_tags_1 },
    { 0x13be25a, SUC_lexicon_tags_8 },
    { 0xe39be25b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75be25e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dbe26b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18dbe26e, SUC_lexicon_tags_7 },
    { 0xfcbbe26f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e7be271, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x185be274, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xdf9be276, SUC_lexicon_tags_37 },
    { 0x1a5be276, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9be27c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbbe280, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e7be282, SUC_lexicon_tags_37 },
    { 0x3e5be283, SUC_lexicon_tags_37 },
    { 0x5bdbe284, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683be289, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401be28d, SUC_lexicon_tags_45 },
    { 0x273be28e, SUC_lexicon_tags_45 },
    { 0xb61be28e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4fbe291, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1be296, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45be29a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbbe29d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbe2a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41be2aa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3edbe2ac, SUC_lexicon_tags_84 },
    { 0x665be2ad, SUC_lexicon_tags_37 },
    { 0x661be2ae, SUC_lexicon_tags_61 },
    { 0x8d3be2af, SUC_lexicon_tags_2 },
    { 0xfc9be2ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x285be2b2, SUC_lexicon_tags_1 },
    { 0xbadbe2b3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xdd3be2b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241be2b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x563be2bb, SUC_lexicon_tags_32 },
    { 0x659be2bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dbe2c0, SUC_lexicon_tags_29 },
    { 0xef5be2c1, SUC_lexicon_tags_47 },
    { 0x861be2c2, SUC_lexicon_tags_25 },
    { 0x989be2c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829be2cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555be2d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x82fbe2d2, SUC_lexicon_tags_16 },
    { 0x1abbe2d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x371be2d6, SUC_lexicon_tags_37 },
    { 0xcd1be2d6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1be2e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbbe2ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdbe2ee, SUC_lexicon_tags_7 },
    { 0x441be2ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dbe2f5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375be2ff, SUC_lexicon_tags_25 },
    { 0xf19be300, SUC_lexicon_tags_27 },
    { 0x989be301, SUC_lexicon_tags_17 },
    { 0xb97be300, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcbfbe304, SUC_lexicon_tags_2 },
    { 0x60fbe305, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6b1be307, SUC_lexicon_tags_3 },
    { 0xfd1be307, SUC_lexicon_tags_36 },
    { 0x8f9be309, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947be30e, SUC_lexicon_tags_6 },
    { 0xa11be30f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x247be312, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fbe31b, SUC_lexicon_tags_49 },
    { 0xe5fbe31b, SUC_lexicon_tags_17 },
    { 0x617be31b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x841be320, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe75be322, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fbe326, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd5dbe328, SUC_lexicon_tags_10 },
    { 0x25dbe329, SUC_lexicon_tags_15 },
    { 0x2d9be32a, SUC_lexicon_tags_33 },
    { 0xb27be32a, SUC_lexicon_tags_33 },
    { 0x175be32c, SUC_lexicon_tags_79 },
    { 0xf9be32d, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3be330, SUC_lexicon_tags_12 },
    { 0x475be331, SUC_lexicon_tags_11 },
    { 0xca5be332, SUC_lexicon_tags_7 },
    { 0x9c1be333, SUC_lexicon_tags_25 },
    { 0x6c1be333, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bbe33a, SUC_lexicon_tags_8 },
    { 0x1fbe33b, SUC_lexicon_tags_12 },
    { 0x127be33c, SUC_lexicon_tags_5 },
    { 0x5afbe33d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x797be340, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf65be342, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabbe345, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dbe34b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bbe350, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4fdbe352, SUC_lexicon_tags_10 },
    { 0xb47be352, SUC_lexicon_tags_28 },
    { 0xea7be353, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85be35b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423be35f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xeafbe361, SUC_lexicon_tags_39 },
    { 0x68bbe361, SUC_lexicon_tags_38 },
    { 0x315be363, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dbe367, SUC_lexicon_tags_11 },
    { 0x58fbe368, SUC_lexicon_tags_23 },
    { 0x76fbe368, SUC_lexicon_tags_17 },
    { 0x79be36a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bbe36f, SUC_lexicon_tags_17 },
    { 0x43fbe370, SUC_lexicon_tags_11 },
    { 0x319be370, SUC_lexicon_tags_19 },
    { 0x579be372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65be376, SUC_lexicon_tags_15 },
    { 0x5a7be376, SUC_lexicon_tags_14 },
    { 0xc07be378, SUC_lexicon_tags_4 },
    { 0x6edbe378, SUC_lexicon_tags_37 },
    { 0x2e3be37a, SUC_lexicon_tags_37 },
    { 0x619be37a, SUC_lexicon_tags_28 },
    { 0x5a9be37c, SUC_lexicon_tags_37 },
    { 0x5a5be37b, SUC_lexicon_tags_87 },
    { 0x15fbe37e, SUC_lexicon_tags_17 },
    { 0x82dbe37f, SUC_lexicon_tags_11 },
    { 0xa5dbe37a, SUC_lexicon_tags_87 },
    { 0x345be378, SUC_lexicon_tags_15 },
    { 0xf65be382, SUC_lexicon_tags_25 },
    { 0x1dbbe383, SUC_lexicon_tags_35 },
    { 0x8cdbe381, SUC_lexicon_tags_61 },
    { 0x829be385, SUC_lexicon_tags_2 },
    { 0xf47be386, SUC_lexicon_tags_19 },
    { 0xb13be386, SUC_lexicon_tags_37 },
    { 0x437be386, SUC_lexicon_tags_17 },
    { 0x6a3be389, SUC_lexicon_tags_12 },
    { 0x46dbe38a, SUC_lexicon_tags_7 },
    { 0x77bbe38a, SUC_lexicon_tags_1 },
    { 0x51be38c, SUC_lexicon_tags_47 },
    { 0xc69be38d, SUC_lexicon_tags_27 },
    { 0x3c3be38b, SUC_lexicon_tags_48 },
    { 0x4a7be37c, SUC_lexicon_tags_11 },
    { 0xd69be390, SUC_lexicon_tags_23 },
    { 0xc23be380, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xacdbe393, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xec3be395, SUC_lexicon_tags_15 },
    { 0x2f9be395, SUC_lexicon_tags_11 },
    { 0xef9be395, SUC_lexicon_tags_9 },
    { 0xa6fbe398, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efbe39d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x659be3a0, SUC_lexicon_tags_12 },
    { 0xe09be3a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9be3aa, SUC_lexicon_tags_1 },
    { 0xee3be3aa, SUC_lexicon_tags_29 },
    { 0x87bbe3ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fbe3b0, SUC_lexicon_tags_27 },
    { 0x551be3b1, SUC_lexicon_tags_38 },
    { 0x9be3b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347be3b6, SUC_lexicon_tags_31 },
    { 0xabbbe3b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81be3c1, SUC_lexicon_tags_12 },
    { 0xb1bbe3c2, SUC_lexicon_tags_84 },
    { 0xec1be3c3, SUC_lexicon_tags_25 },
    { 0x379be3c4, SUC_lexicon_tags_2 },
    { 0xdc5be3c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fbe3c8, SUC_lexicon_tags_25 },
    { 0x735be3c8, SUC_lexicon_tags_49 },
    { 0x537be3ca, SUC_lexicon_tags_38 },
    { 0xce5be3cb, SUC_lexicon_tags_39 },
    { 0xc5fbe3cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855be3d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x729be3d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bbe3d6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bbe3da, SUC_lexicon_tags_18 },
    { 0x3f5be3db, SUC_lexicon_tags_6 },
    { 0x353be3dc, SUC_lexicon_tags_12 },
    { 0x753be3dc, SUC_lexicon_tags_38 },
    { 0x61fbe3de, SUC_lexicon_tags_38 },
    { 0x10fbe3db, SUC_lexicon_tags_17 },
    { 0x52fbe3db, SUC_lexicon_tags_37 },
    { 0xb1bbe3e1, SUC_lexicon_tags_47 },
    { 0x99be3e2, SUC_lexicon_tags_2 },
    { 0xbe3be3e2, SUC_lexicon_tags_29 },
    { 0x903be3e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153be3e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b1be3ea, SUC_lexicon_tags_65 },
    { 0x8afbe3eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x903be3ee, SUC_lexicon_tags_17 },
    { 0x2a3be3ef, SUC_lexicon_tags_37 },
    { 0x399be3f0, SUC_lexicon_tags_15 },
    { 0x513be3f0, SUC_lexicon_tags_28 },
    { 0x6d9be3f1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x97fbe3f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x917be3f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e5be3f9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x833be3fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89be3ff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x379be402, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903be407, SUC_lexicon_tags_17 },
    { 0x525be407, SUC_lexicon_tags_10 },
    { 0x957be407, SUC_lexicon_tags_12 },
    { 0x717be408, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc77be40d, SUC_lexicon_tags_114 },
    { 0xb59be40d, SUC_lexicon_tags_14 },
    { 0xc85be40f, SUC_lexicon_tags_23 },
    { 0x4edbe40f, SUC_lexicon_tags_32 },
    { 0xbb5be411, SUC_lexicon_tags_6 },
    { 0x631be411, SUC_lexicon_tags_30 },
    { 0xdd9be413, SUC_lexicon_tags_23 },
    { 0x7fbe414, SUC_lexicon_tags_17 },
    { 0x11dbe415, SUC_lexicon_tags_0 },
    { 0x5a7be414, SUC_lexicon_tags_12 },
    { 0xf43be417, SUC_lexicon_tags_15 },
    { 0x9cdbe418, SUC_lexicon_tags_2 },
    { 0x5ffbe419, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fbe425, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1be429, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfbe42d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xa0fbe42f, SUC_lexicon_tags_2 },
    { 0xe8bbe430, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591be434, SUC_lexicon_tags_47 },
    { 0x50bbe435, SUC_lexicon_tags_45 },
    { 0x563be436, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xce9be438, SUC_lexicon_tags_12 },
    { 0x159be438, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x511be43b, SUC_lexicon_tags_33 },
    { 0xd39be43b, SUC_lexicon_tags_37 },
    { 0xfadbe43d, SUC_lexicon_tags_47 },
    { 0x2c7be43d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9be445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a7be447, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdc5be449, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1be44b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5be44f, SUC_lexicon_tags_2 },
    { 0xcebbe450, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79be453, SUC_lexicon_tags_11 },
    { 0x4fdbe454, SUC_lexicon_tags_15 },
    { 0x5d5be455, SUC_lexicon_tags_44 },
    { 0x21fbe455, SUC_lexicon_tags_11 },
    { 0xf5bbe457, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef9be459, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bbe464, SUC_lexicon_tags_2 },
    { 0xdc3be465, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbbe469, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7be46c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fbe470, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5be473, SUC_lexicon_tags_44 },
    { 0xcf1be474, SUC_lexicon_tags_37 },
    { 0xb9bbe473, SUC_lexicon_tags_42 },
    { 0x191be476, SUC_lexicon_tags_16 },
    { 0xe85be477, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbbe47b, SUC_lexicon_tags_104 },
    { 0xc27be47c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3be47f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991be484, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7fbe486, SUC_lexicon_tags_43 },
    { 0x211be486, SUC_lexicon_tags_17 },
    { 0x7e1be486, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x661be48a, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15be48d, SUC_lexicon_tags_15 },
    { 0xd83be48e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xecfbe490, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafbe492, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dbe497, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x303be499, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899be4a0, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811be4a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1bbe4a8, SUC_lexicon_tags_7 },
    { 0x3c7be4a9, SUC_lexicon_tags_39 },
    { 0x59bbe4aa, SUC_lexicon_tags_15 },
    { 0x453be4a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7be4af, SUC_lexicon_tags_47 },
    { 0xc81be4b0, SUC_lexicon_tags_39 },
    { 0xbbbbe4af, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3be4b4, SUC_lexicon_tags_25 },
    { 0xb37be4b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bbe4bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91be4c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bbe4c5, SUC_lexicon_tags_7 },
    { 0x2dfbe4c6, SUC_lexicon_tags_23 },
    { 0xee1be4c5, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bbe4cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x699be4d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x347be4d3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xd33be4d5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35be4d8, SUC_lexicon_tags_12 },
    { 0x105be4d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491be4dd, SUC_lexicon_tags_66 },
    { 0x6bfbe4dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abbe4e4, SUC_lexicon_tags_37 },
    { 0x9f1be4e5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dbe4ea, SUC_lexicon_tags_49 },
    { 0x995be4eb, SUC_lexicon_tags_1 },
    { 0x2fbe4ec, SUC_lexicon_tags_17 },
    { 0xdd5be4ed, SUC_lexicon_tags_47 },
    { 0x1c9be4ee, SUC_lexicon_tags_23 },
    { 0xf2dbe4ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96dbe4f2, SUC_lexicon_tags_47 },
    { 0xbb1be4f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dbe4f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7afbe4f9, SUC_lexicon_tags_29 },
    { 0xbbfbe4fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fbe4fe, SUC_lexicon_tags_49 },
    { 0xcebbe4ff, SUC_lexicon_tags_11 },
    { 0x5efbe4ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9be503, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105be50a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x217be50c, SUC_lexicon_tags_15 },
    { 0x219be50d, SUC_lexicon_tags_2 },
    { 0x9d9be50c, SUC_lexicon_tags_140 },
    { 0x681be50f, SUC_lexicon_tags_37 },
    { 0x6e3be50c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dbe516, SUC_lexicon_tags_45 },
    { 0xb11be517, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x817be51a, SUC_lexicon_tags_17 },
    { 0x375be51b, SUC_lexicon_tags_17 },
    { 0x55be51a, SUC_lexicon_tags_25 },
    { 0x729be51d, SUC_lexicon_tags_65 },
    { 0x503be51c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7be526, SUC_lexicon_tags_33 },
    { 0x7dfbe526, SUC_lexicon_tags_38 },
    { 0x5bbbe528, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xeffbe52a, SUC_lexicon_tags_37 },
    { 0x165be52b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a5be52d, SUC_lexicon_tags_30 },
    { 0xb5dbe52e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65be539, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x615be53b, SUC_lexicon_tags_47 },
    { 0xfd3be53c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x167be53e, SUC_lexicon_tags_37 },
    { 0x959be53f, SUC_lexicon_tags_45 },
    { 0xb41be53f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x257be543, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dbe549, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x241be54b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x123be54e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x723be551, SUC_lexicon_tags_17 },
    { 0xbd9be551, SUC_lexicon_tags_28 },
    { 0x803be553, SUC_lexicon_tags_2 },
    { 0xf99be554, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5d7be556, SUC_lexicon_tags_12 },
    { 0x54dbe557, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9be55a, SUC_lexicon_tags_13 },
    { 0x561be55b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afbe55f, SUC_lexicon_tags_1 },
    { 0x5d5be560, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x35dbe562, SUC_lexicon_tags_10 },
    { 0x54dbe563, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x18dbe565, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbbe56c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07be576, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x607be579, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fbe57c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbbe582, SUC_lexicon_tags_10 },
    { 0x85bbe583, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9be588, SUC_lexicon_tags_17 },
    { 0x5f7be589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dbe58e, SUC_lexicon_tags_61 },
    { 0xcf7be58e, SUC_lexicon_tags_11 },
    { 0x977be590, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3be593, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3be597, SUC_lexicon_tags_17 },
    { 0xafbbe597, SUC_lexicon_tags_47 },
    { 0x445be597, SUC_lexicon_tags_15 },
    { 0x67dbe598, SUC_lexicon_tags_68 },
    { 0x27bbe59b, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x899be59e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335be5a4, SUC_lexicon_tags_4 },
    { 0x415be5a4, SUC_lexicon_tags_55 },
    { 0xb99be5a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dbe5aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9be5af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe61be5b1, SUC_lexicon_tags_11 },
    { 0x231be5b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe09be5b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1be5b9, SUC_lexicon_tags_7 },
    { 0x667be5ba, SUC_lexicon_tags_38 },
    { 0x859be5b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa87be5bd, SUC_lexicon_tags_26 },
    { 0xc1dbe5be, SUC_lexicon_tags_1 },
    { 0x6ebbe5bd, SUC_lexicon_tags_45 },
    { 0x99be5c0, SUC_lexicon_tags_17 },
    { 0x82fbe5c1, SUC_lexicon_tags_37 },
    { 0x183be5c2, SUC_lexicon_tags_37 },
    { 0x185be5c3, SUC_lexicon_tags_2 },
    { 0xf21be5c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd01be5c6, SUC_lexicon_tags_11 },
    { 0xd7dbe5c6, SUC_lexicon_tags_19 },
    { 0xffbe5c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2bdbe5ca, SUC_lexicon_tags_17 },
    { 0xbbfbe5ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93be5ce, SUC_lexicon_tags_7 },
    { 0xf4fbe5ce, SUC_lexicon_tags_35 },
    { 0xb21be5d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x603be5d2, SUC_lexicon_tags_25 },
    { 0x87dbe5d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x917be5d5, SUC_lexicon_tags_23 },
    { 0x35be5d5, SUC_lexicon_tags_2 },
    { 0xdf1be5d5, SUC_lexicon_tags_35 },
    { 0x18dbe5d8, SUC_lexicon_tags_23 },
    { 0xc39be5d8, SUC_lexicon_tags_46 },
    { 0x5bbe5d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4d3be5dc, SUC_lexicon_tags_17 },
    { 0x39be5dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x255be5df, SUC_lexicon_tags_37 },
    { 0x18fbe5df, SUC_lexicon_tags_2 },
    { 0x455be5df, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x6f1be5e3, SUC_lexicon_tags_1 },
    { 0x7e7be5e4, SUC_lexicon_tags_63 },
    { 0x7abbe5e5, SUC_lexicon_tags_1 },
    { 0x64dbe5e5, SUC_lexicon_tags_9 },
    { 0xb21be5e4, SUC_lexicon_tags_12 },
    { 0x543be5e8, SUC_lexicon_tags_11 },
    { 0xcbfbe5e9, SUC_lexicon_tags_11 },
    { 0x16dbe5e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751be5ee, SUC_lexicon_tags_37 },
    { 0x619be5ef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x809be5f2, SUC_lexicon_tags_33 },
    { 0x3c3be5f3, SUC_lexicon_tags_43 },
    { 0x827be5f4, SUC_lexicon_tags_4 },
    { 0x463be5f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93be5f8, SUC_lexicon_tags_23 },
    { 0xbf7be5f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x36bbe5fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7be601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x135be603, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721be608, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57be60b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139be60f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x393be612, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fbe616, SUC_lexicon_tags_11 },
    { 0xfa7be617, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fbe625, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdfbe627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ebbe629, SUC_lexicon_tags_1 },
    { 0x479be62a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x189be62c, SUC_lexicon_tags_37 },
    { 0xfbfbe62c, SUC_lexicon_tags_25 },
    { 0x499be62d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf7dbe630, SUC_lexicon_tags_4 },
    { 0x7d5be631, SUC_lexicon_tags_8 },
    { 0xbb5be632, SUC_lexicon_tags_17 },
    { 0x15dbe633, SUC_lexicon_tags_74 },
    { 0x5c1be630, SUC_lexicon_tags_48 },
    { 0x74dbe635, SUC_lexicon_tags_32 },
    { 0x667be636, SUC_lexicon_tags_11 },
    { 0x385be631, SUC_lexicon_tags_14 },
    { 0x563be636, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83be63b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bbe641, SUC_lexicon_tags_33 },
    { 0xfffbe641, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x209be644, SUC_lexicon_tags_47 },
    { 0xb33be645, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7be64c, SUC_lexicon_tags_17 },
    { 0x909be64d, SUC_lexicon_tags_85 },
    { 0x4b7be64d, SUC_lexicon_tags_37 },
    { 0xb15be64c, SUC_lexicon_tags_15 },
    { 0xdcbbe64f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebbe654, SUC_lexicon_tags_15 },
    { 0x8cbbe655, SUC_lexicon_tags_11 },
    { 0x2b7be656, SUC_lexicon_tags_66 },
    { 0x941be657, SUC_lexicon_tags_37 },
    { 0xf73be658, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x95dbe65b, SUC_lexicon_tags_79 },
    { 0xfdfbe65c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1be660, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc53be662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdd5be664, SUC_lexicon_tags_37 },
    { 0x977be665, SUC_lexicon_tags_1 },
    { 0x3b5be666, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xba3be668, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129be66e, SUC_lexicon_tags_12 },
    { 0x4fbe66f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6ffbe671, SUC_lexicon_tags_87 },
    { 0x5e3be671, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425be676, SUC_lexicon_tags_2 },
    { 0xcfdbe676, SUC_lexicon_tags_1 },
    { 0x831be678, SUC_lexicon_tags_25 },
    { 0x207be679, SUC_lexicon_tags_25 },
    { 0x385be67a, SUC_lexicon_tags_37 },
    { 0xa3bbe678, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x58bbe67d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535be687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bbe68d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x939be68f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x82bbe691, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b1be693, SUC_lexicon_tags_14 },
    { 0x8b1be693, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9be697, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebbe69b, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0x353be69e, SUC_lexicon_tags_6 },
    { 0xadbe69f, SUC_lexicon_tags_25 },
    { 0x277be6a0, SUC_lexicon_tags_28 },
    { 0x177be6a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5be6a8, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17be6ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dbe6ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x759be6b0, SUC_lexicon_tags_12 },
    { 0xa61be6b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5bbe6b3, SUC_lexicon_tags_3 },
    { 0x6b9be6b4, SUC_lexicon_tags_11 },
    { 0x1a9be6b5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xde9be6b7, SUC_lexicon_tags_34 },
    { 0x53fbe6b7, SUC_lexicon_tags_65 },
    { 0xc85be6b9, SUC_lexicon_tags_33 },
    { 0x7b3be6b9, SUC_lexicon_tags_1 },
    { 0x179be6ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xef5be6bd, SUC_lexicon_tags_15 },
    { 0xd67be6be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e3be6c0, SUC_lexicon_tags_27 },
    { 0x9abbe6c0, SUC_lexicon_tags_11 },
    { 0xf1fbe6c1, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1be6c7, SUC_lexicon_tags_60 },
    { 0x809be6c8, SUC_lexicon_tags_9 },
    { 0x7e1be6c9, SUC_lexicon_tags_37 },
    { 0xd21be6ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dbe6cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x217be6d0, SUC_lexicon_tags_15 },
    { 0x25be6d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bbe6d9, SUC_lexicon_tags_27 },
    { 0xabdbe6da, SUC_lexicon_tags_38 },
    { 0x1a9be6db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa8fbe6dd, SUC_lexicon_tags_7 },
    { 0xc3be6dd, SUC_lexicon_tags_17 },
    { 0xf39be6df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x625be6e1, SUC_lexicon_tags_30 },
    { 0xb47be6e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09be6e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x325be6e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x497be6eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9be6ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edbe6f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521be6f6, SUC_lexicon_tags_19 },
    { 0x1fbbe6f7, SUC_lexicon_tags_16 },
    { 0x25dbe6f7, SUC_lexicon_tags_48 },
    { 0x2fbe6f9, SUC_lexicon_tags_0 },
    { 0xf57be6f7, SUC_lexicon_tags_2 },
    { 0x431be6f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9be702, SUC_lexicon_tags_15 },
    { 0xde5be702, SUC_lexicon_tags_7 },
    { 0xd71be704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3be708, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7be70f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb99be711, SUC_lexicon_tags_12 },
    { 0x495be712, SUC_lexicon_tags_11 },
    { 0xafbe713, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7be717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x143be719, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdbe71e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfbe722, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x283be724, SUC_lexicon_tags_7 },
    { 0xe7dbe724, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x235be728, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x611be72a, SUC_lexicon_tags_1 },
    { 0xdedbe72b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bbe733, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddbe738, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39be73c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x885be73e, SUC_lexicon_tags_32 },
    { 0x413be73e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x42dbe742, SUC_lexicon_tags_22 },
    { 0x3f1be742, SUC_lexicon_tags_31 },
    { 0xc0dbe744, SUC_lexicon_tags_34 },
    { 0x741be745, SUC_lexicon_tags_27 },
    { 0x417be746, SUC_lexicon_tags_25 },
    { 0x313be746, SUC_lexicon_tags_27 },
    { 0x429be748, SUC_lexicon_tags_12 },
    { 0x173be749, SUC_lexicon_tags_16 },
    { 0x7dbe74a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adbe75a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd7bbe75c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35be75f, SUC_lexicon_tags_60 },
    { 0x623be760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fbe764, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x167be766, SUC_lexicon_tags_15 },
    { 0x4dbe767, SUC_lexicon_tags_38 },
    { 0x505be767, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x743be76b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619be76f, SUC_lexicon_tags_29 },
    { 0x65be76f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dbe773, SUC_lexicon_tags_7 },
    { 0x50fbe774, SUC_lexicon_tags_17 },
    { 0x4efbe774, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01be778, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05be77e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbbe786, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5be78a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133be791, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbbe794, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaedbe796, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bbe79a, SUC_lexicon_tags_2 },
    { 0x573be79b, SUC_lexicon_tags_21 },
    { 0x7f5be79c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41be7a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65fbe7a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb25be7a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14fbe7a7, SUC_lexicon_tags_1 },
    { 0x2a7be7a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbbe7ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bbe7af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x109be7b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe97be7b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x83fbe7b6, SUC_lexicon_tags_17 },
    { 0x7cdbe7b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x269be7b9, SUC_lexicon_tags_8 },
    { 0x863be7ba, SUC_lexicon_tags_25 },
    { 0x457be7b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x219be7bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f5be7bf, SUC_lexicon_tags_38 },
    { 0x44fbe7bf, SUC_lexicon_tags_0 },
    { 0xa6bbe7c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4ddbe7c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x821be7c5, SUC_lexicon_tags_87 },
    { 0x1d9be7c6, SUC_lexicon_tags_30 },
    { 0x5fbbe7c6, SUC_lexicon_tags_49 },
    { 0xea7be7c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7be7cb, SUC_lexicon_tags_23 },
    { 0x5dbbe7cc, SUC_lexicon_tags_10 },
    { 0xb9be7cc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x33be7d0, SUC_lexicon_tags_14 },
    { 0xde1be7d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x285be7d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03be7d8, SUC_lexicon_tags_6 },
    { 0xa1be7d8, SUC_lexicon_tags_23 },
    { 0x269be7d9, SUC_lexicon_tags_36 },
    { 0x8f9be7d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x69dbe7de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3be7e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd01be7e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a3be7e7, SUC_lexicon_tags_31 },
    { 0x3f7be7e7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345be7f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x341be7f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5be7f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x363be7fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x9ebbe7fc, SUC_lexicon_tags_63 },
    { 0xac9be7fd, SUC_lexicon_tags_25 },
    { 0xd75be7fc, SUC_lexicon_tags_2 },
    { 0x5d1be7fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9be803, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb73be805, SUC_lexicon_tags_23 },
    { 0x4afbe806, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27be80b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf35be80d, SUC_lexicon_tags_29 },
    { 0x89fbe80d, SUC_lexicon_tags_38 },
    { 0xc41be80f, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x8afbe811, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61be815, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9dbe81b, SUC_lexicon_tags_60 },
    { 0x495be81c, SUC_lexicon_tags_95 },
    { 0xa47be81d, SUC_lexicon_tags_12 },
    { 0x1c5be81d, SUC_lexicon_tags_43 },
    { 0xb6fbe81d, SUC_lexicon_tags_47 },
    { 0x119be820, SUC_lexicon_tags_14 },
    { 0x9dbe821, SUC_lexicon_tags_46 },
    { 0x3edbe822, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbfbe824, SUC_lexicon_tags_12 },
    { 0xcffbe824, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdbe82c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdbe832, SUC_lexicon_tags_11 },
    { 0x34fbe833, SUC_lexicon_tags_42 },
    { 0xcd9be834, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5be839, SUC_lexicon_tags_452 },
    { 0, NULL },
    { 0x983be83b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdbe83e, SUC_lexicon_tags_61 },
    { 0x285be83f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5be845, SUC_lexicon_tags_30 },
    { 0x991be845, SUC_lexicon_tags_12 },
    { 0xab7be846, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdbe84b, SUC_lexicon_tags_47 },
    { 0x797be84c, SUC_lexicon_tags_31 },
    { 0x9b1be84b, SUC_lexicon_tags_31 },
    { 0x6b1be84e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbe851, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3efbe853, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbbe857, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bbe85a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5be85d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xddfbe85f, SUC_lexicon_tags_11 },
    { 0x82dbe860, SUC_lexicon_tags_37 },
    { 0x467be861, SUC_lexicon_tags_66 },
    { 0x205be862, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dbe866, SUC_lexicon_tags_30 },
    { 0xa63be867, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1be86c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7be871, SUC_lexicon_tags_28 },
    { 0xa41be872, SUC_lexicon_tags_127 },
    { 0x3a3be873, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x86dbe875, SUC_lexicon_tags_45 },
    { 0x751be876, SUC_lexicon_tags_37 },
    { 0x225be877, SUC_lexicon_tags_5 },
    { 0x3a5be876, SUC_lexicon_tags_8 },
    { 0x167be876, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd2fbe87b, SUC_lexicon_tags_25 },
    { 0x351be87c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x8efbe87f, SUC_lexicon_tags_12 },
    { 0xbe3be880, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dbe884, SUC_lexicon_tags_7 },
    { 0xd35be885, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf1be887, SUC_lexicon_tags_2 },
    { 0xabbbe888, SUC_lexicon_tags_11 },
    { 0x7ddbe889, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765be88d, SUC_lexicon_tags_37 },
    { 0x611be88e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x82bbe890, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3be894, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbbe899, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc33be89b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbe89f, SUC_lexicon_tags_7 },
    { 0xd79be8a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1dbe8a2, SUC_lexicon_tags_12 },
    { 0x661be8a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7be8ac, SUC_lexicon_tags_10 },
    { 0x347be8ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf89be8af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72dbe8b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9be8b7, SUC_lexicon_tags_17 },
    { 0x145be8b7, SUC_lexicon_tags_2 },
    { 0x7d3be8b7, SUC_lexicon_tags_10 },
    { 0x971be8b9, SUC_lexicon_tags_61 },
    { 0xa5dbe8b7, SUC_lexicon_tags_47 },
    { 0xea9be8b9, SUC_lexicon_tags_98 },
    { 0x455be8b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42fbe8bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73be8c6, SUC_lexicon_tags_17 },
    { 0xdf1be8c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07be8cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x45dbe8cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x973be8cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf85be8d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80dbe8d3, SUC_lexicon_tags_11 },
    { 0x1b5be8d3, SUC_lexicon_tags_23 },
    { 0x3c5be8d5, SUC_lexicon_tags_77 },
    { 0x54fbe8d4, SUC_lexicon_tags_21 },
    { 0xf91be8d7, SUC_lexicon_tags_11 },
    { 0x599be8d7, SUC_lexicon_tags_15 },
    { 0x4e1be8d9, SUC_lexicon_tags_7 },
    { 0x30bbe8da, SUC_lexicon_tags_45 },
    { 0xa01be8d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc55be8dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f7be8df, SUC_lexicon_tags_4 },
    { 0x45dbe8e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa13be8e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3b3be8e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fbe8e7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbe8ef, SUC_lexicon_tags_23 },
    { 0xaa9be8f0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa23be8f2, SUC_lexicon_tags_2 },
    { 0xaebbe8f3, SUC_lexicon_tags_14 },
    { 0x8d3be8f3, SUC_lexicon_tags_11 },
    { 0xf29be8f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9be8f8, SUC_lexicon_tags_1 },
    { 0xe3fbe8f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x935be8fb, SUC_lexicon_tags_17 },
    { 0x97bbe8fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbbe900, SUC_lexicon_tags_0 },
    { 0x4a7be901, SUC_lexicon_tags_38 },
    { 0xa7bbe902, SUC_lexicon_tags_14 },
    { 0xcf7be902, SUC_lexicon_tags_17 },
    { 0xd9bbe901, SUC_lexicon_tags_66 },
    { 0xcc3be905, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x999be907, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe15be909, SUC_lexicon_tags_37 },
    { 0x245be90a, SUC_lexicon_tags_85 },
    { 0xcf7be90a, SUC_lexicon_tags_49 },
    { 0xd01be90c, SUC_lexicon_tags_4 },
    { 0x215be90d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5be911, SUC_lexicon_tags_2 },
    { 0xf79be911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1be915, SUC_lexicon_tags_7 },
    { 0x4bbbe916, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddbe919, SUC_lexicon_tags_9 },
    { 0xa6fbe91a, SUC_lexicon_tags_29 },
    { 0xc57be919, SUC_lexicon_tags_49 },
    { 0xebdbe91c, SUC_lexicon_tags_32 },
    { 0xf43be919, SUC_lexicon_tags_15 },
    { 0xc63be91e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x11fbe920, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29be923, SUC_lexicon_tags_62 },
    { 0x9e1be924, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e5be926, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7be929, SUC_lexicon_tags_33 },
    { 0x73dbe92a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x36fbe92c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9be930, SUC_lexicon_tags_11 },
    { 0xb6bbe931, SUC_lexicon_tags_37 },
    { 0x513be931, SUC_lexicon_tags_68 },
    { 0xc7be933, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375be938, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dbe93c, SUC_lexicon_tags_25 },
    { 0x15be93d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225be941, SUC_lexicon_tags_26 },
    { 0x4bbe941, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbbe947, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27be94a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x80dbe94c, SUC_lexicon_tags_17 },
    { 0x447be94d, SUC_lexicon_tags_37 },
    { 0xcd3be94e, SUC_lexicon_tags_17 },
    { 0xf05be94e, SUC_lexicon_tags_1 },
    { 0x6a5be94f, SUC_lexicon_tags_17 },
    { 0x735be94c, SUC_lexicon_tags_29 },
    { 0x69be952, SUC_lexicon_tags_14 },
    { 0x68dbe953, SUC_lexicon_tags_43 },
    { 0x4b1be952, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7be959, SUC_lexicon_tags_6 },
    { 0x93be95a, SUC_lexicon_tags_63 },
    { 0x623be95b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fbe95e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7fbe960, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9be964, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x827be966, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe972, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfbe976, SUC_lexicon_tags_230 },
    { 0xabbe976, SUC_lexicon_tags_2 },
    { 0x2c5be978, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe97e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdbe984, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbbe988, SUC_lexicon_tags_138 },
    { 0x3efbe989, SUC_lexicon_tags_27 },
    { 0x31fbe988, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x75dbe98c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x263be98f, SUC_lexicon_tags_0 },
    { 0xd7bbe98f, SUC_lexicon_tags_17 },
    { 0x4bdbe991, SUC_lexicon_tags_7 },
    { 0x92dbe992, SUC_lexicon_tags_33 },
    { 0x833be992, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe03be995, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x375be997, SUC_lexicon_tags_25 },
    { 0x16dbe997, SUC_lexicon_tags_37 },
    { 0x9ffbe999, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x65fbe99b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85dbe9a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bbe9a5, SUC_lexicon_tags_68 },
    { 0xfc9be9a5, SUC_lexicon_tags_85 },
    { 0x5bfbe9a7, SUC_lexicon_tags_17 },
    { 0x7f9be9a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabbe9ad, SUC_lexicon_tags_15 },
    { 0xa8dbe9ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x297be9b0, SUC_lexicon_tags_35 },
    { 0x94fbe9b0, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3be9b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdbe9be, SUC_lexicon_tags_45 },
    { 0xbbe9bf, SUC_lexicon_tags_17 },
    { 0xaa1be9c0, SUC_lexicon_tags_15 },
    { 0x365be9c1, SUC_lexicon_tags_32 },
    { 0x567be9c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbbbe9c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263be9c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x885be9cb, SUC_lexicon_tags_65 },
    { 0xd3be9cc, SUC_lexicon_tags_15 },
    { 0x661be9cd, SUC_lexicon_tags_4 },
    { 0x5fbbe9ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc3be9d0, SUC_lexicon_tags_17 },
    { 0x25be9d1, SUC_lexicon_tags_4 },
    { 0xfbbe9d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afbe9d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7be9d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbe9dc, SUC_lexicon_tags_53 },
    { 0xc1fbe9dd, SUC_lexicon_tags_46 },
    { 0x371be9de, SUC_lexicon_tags_11 },
    { 0x83be9df, SUC_lexicon_tags_12 },
    { 0xca7be9e0, SUC_lexicon_tags_25 },
    { 0xe2fbe9de, SUC_lexicon_tags_12 },
    { 0x251be9e2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1be9e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49be9ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc35be9ec, SUC_lexicon_tags_2 },
    { 0xc7bbe9ed, SUC_lexicon_tags_892 },
    { 0x4a5be9ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x915be9f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47be9f9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585be9fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3bea07, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x143bea0a, SUC_lexicon_tags_11 },
    { 0x2f3bea0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fbea0f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x895bea11, SUC_lexicon_tags_34 },
    { 0x813bea11, SUC_lexicon_tags_0 },
    { 0xc25bea11, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x3e5bea15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddbea18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x597bea1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fbea1f, SUC_lexicon_tags_10 },
    { 0x1c3bea1f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bbea28, SUC_lexicon_tags_0 },
    { 0xabdbea29, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ffbea2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb21bea2d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935bea34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7bea38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3bea43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365bea4a, SUC_lexicon_tags_11 },
    { 0x813bea4b, SUC_lexicon_tags_45 },
    { 0x30fbea4b, SUC_lexicon_tags_15 },
    { 0x653bea4a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bbea52, SUC_lexicon_tags_42 },
    { 0xa1bbea53, SUC_lexicon_tags_23 },
    { 0x9b9bea54, SUC_lexicon_tags_31 },
    { 0xab7bea55, SUC_lexicon_tags_14 },
    { 0x91dbea56, SUC_lexicon_tags_25 },
    { 0x379bea57, SUC_lexicon_tags_60 },
    { 0x96bbea57, SUC_lexicon_tags_15 },
    { 0xc4bbea59, SUC_lexicon_tags_37 },
    { 0xc95bea5a, SUC_lexicon_tags_6 },
    { 0x405bea5a, SUC_lexicon_tags_30 },
    { 0x535bea58, SUC_lexicon_tags_12 },
    { 0x3efbea5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcbbea5f, SUC_lexicon_tags_114 },
    { 0x47dbea60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bbea65, SUC_lexicon_tags_25 },
    { 0xeedbea66, SUC_lexicon_tags_25 },
    { 0x9a9bea66, SUC_lexicon_tags_30 },
    { 0x3bdbea67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb85bea6a, SUC_lexicon_tags_44 },
    { 0x651bea6a, SUC_lexicon_tags_49 },
    { 0xb8fbea6c, SUC_lexicon_tags_7 },
    { 0xda7bea6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bbea71, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9cbbea73, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5bea76, SUC_lexicon_tags_15 },
    { 0xc87bea77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbbea7e, SUC_lexicon_tags_25 },
    { 0x9e5bea7e, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x3d7bea81, SUC_lexicon_tags_11 },
    { 0xd23bea82, SUC_lexicon_tags_7 },
    { 0xa73bea82, SUC_lexicon_tags_19 },
    { 0x2dfbea84, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27bea89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5bea8e, SUC_lexicon_tags_49 },
    { 0xdbea8e, SUC_lexicon_tags_2 },
    { 0xc5fbea90, SUC_lexicon_tags_10 },
    { 0x13fbea91, SUC_lexicon_tags_1 },
    { 0xae9bea91, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfc1bea94, SUC_lexicon_tags_11 },
    { 0xa97bea95, SUC_lexicon_tags_7 },
    { 0xc9bea95, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85bea9a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x16fbea9c, SUC_lexicon_tags_0 },
    { 0xefbbea9d, SUC_lexicon_tags_15 },
    { 0x45bbea9e, SUC_lexicon_tags_15 },
    { 0xc7bea9f, SUC_lexicon_tags_11 },
    { 0xeb1beaa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae9beaa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77beaaa, SUC_lexicon_tags_27 },
    { 0xbf9beaab, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3beaba, SUC_lexicon_tags_60 },
    { 0x457beaba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7beabf, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xdbeac1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb7beac3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbead0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x305bead2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5fbead4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc5bbead6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1beadc, SUC_lexicon_tags_31 },
    { 0xd25beadc, SUC_lexicon_tags_45 },
    { 0x69beade, SUC_lexicon_tags_45 },
    { 0xdb7beadf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1beae2, SUC_lexicon_tags_15 },
    { 0x285beae3, SUC_lexicon_tags_7 },
    { 0xb5beae3, SUC_lexicon_tags_55 },
    { 0x77fbeae5, SUC_lexicon_tags_6 },
    { 0x70bbeae3, SUC_lexicon_tags_6 },
    { 0x1f3beae5, SUC_lexicon_tags_25 },
    { 0x153beae8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7beaee, SUC_lexicon_tags_17 },
    { 0xaf7beaef, SUC_lexicon_tags_25 },
    { 0x801beaf0, SUC_lexicon_tags_32 },
    { 0x439beaf1, SUC_lexicon_tags_1 },
    { 0xe8dbeaf2, SUC_lexicon_tags_28 },
    { 0x3d5beaf3, SUC_lexicon_tags_33 },
    { 0x723beaf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfbeaf9, SUC_lexicon_tags_17 },
    { 0x4d9beaf9, SUC_lexicon_tags_17 },
    { 0xd9bbeafa, SUC_lexicon_tags_37 },
    { 0x2d9beaf9, SUC_lexicon_tags_12 },
    { 0xfd7beafc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe9beaff, SUC_lexicon_tags_12 },
    { 0x19dbeb00, SUC_lexicon_tags_7 },
    { 0xcd1beb01, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dbeb04, SUC_lexicon_tags_0 },
    { 0x989beb05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257beb0a, SUC_lexicon_tags_25 },
    { 0x6afbeb0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3bbeb0d, SUC_lexicon_tags_1 },
    { 0x8e5beb0d, SUC_lexicon_tags_0 },
    { 0xbf9beb0f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xabbeb11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dbeb1d, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fbeb21, SUC_lexicon_tags_25 },
    { 0xcefbeb22, SUC_lexicon_tags_38 },
    { 0x557beb23, SUC_lexicon_tags_7 },
    { 0xf0dbeb22, SUC_lexicon_tags_14 },
    { 0xe5dbeb24, SUC_lexicon_tags_39 },
    { 0xfdbbeb23, SUC_lexicon_tags_33 },
    { 0x193beb25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6dbeb29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x73dbeb2c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd43beb2e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x833beb31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dbeb37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x78dbeb3a, SUC_lexicon_tags_2 },
    { 0x493beb3b, SUC_lexicon_tags_11 },
    { 0x26fbeb3b, SUC_lexicon_tags_25 },
    { 0x8fdbeb3d, SUC_lexicon_tags_30 },
    { 0xfa1beb3e, SUC_lexicon_tags_43 },
    { 0x73beb3e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441beb46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x199beb48, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x505beb4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989beb50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb9beb52, SUC_lexicon_tags_37 },
    { 0xfddbeb53, SUC_lexicon_tags_11 },
    { 0x5bdbeb53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fbeb57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dbeb5a, SUC_lexicon_tags_7 },
    { 0xdd7beb5b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485beb5f, SUC_lexicon_tags_17 },
    { 0xbb7beb60, SUC_lexicon_tags_42 },
    { 0x5beb60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551beb66, SUC_lexicon_tags_11 },
    { 0xcf7beb66, SUC_lexicon_tags_37 },
    { 0x88fbeb68, SUC_lexicon_tags_47 },
    { 0x69fbeb68, SUC_lexicon_tags_7 },
    { 0x825beb69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3beb6e, SUC_lexicon_tags_47 },
    { 0xa01beb6f, SUC_lexicon_tags_12 },
    { 0x9cbbeb70, SUC_lexicon_tags_31 },
    { 0xa3bbeb71, SUC_lexicon_tags_15 },
    { 0xe0fbeb71, SUC_lexicon_tags_15 },
    { 0xdd5beb73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3beb77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fbeb7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbadbeb7f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485beb84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfdbbeb86, SUC_lexicon_tags_12 },
    { 0x133beb87, SUC_lexicon_tags_33 },
    { 0xd5dbeb86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5beb8b, SUC_lexicon_tags_38 },
    { 0x3fdbeb8c, SUC_lexicon_tags_11 },
    { 0xc79beb8c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x68dbeb90, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47beb93, SUC_lexicon_tags_12 },
    { 0xe91beb94, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dbeb97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7beb9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dbeb9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117beba5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2c7beba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fbeba9, SUC_lexicon_tags_11 },
    { 0x321beba9, SUC_lexicon_tags_37 },
    { 0x3d9beba9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21bbebae, SUC_lexicon_tags_10 },
    { 0x2cfbebaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bbebb3, SUC_lexicon_tags_43 },
    { 0x857bebb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631bebb9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fbebbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611bebc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb61bebc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbebc8, SUC_lexicon_tags_30 },
    { 0xb69bebc9, SUC_lexicon_tags_7 },
    { 0x67dbebc9, SUC_lexicon_tags_0 },
    { 0x625bebcb, SUC_lexicon_tags_7 },
    { 0x411bebc8, SUC_lexicon_tags_9 },
    { 0xad7bebcd, SUC_lexicon_tags_37 },
    { 0x4efbebca, SUC_lexicon_tags_0 },
    { 0x4e9bebcf, SUC_lexicon_tags_29 },
    { 0x7a7bebcf, SUC_lexicon_tags_25 },
    { 0x113bebd1, SUC_lexicon_tags_7 },
    { 0xa85bebd1, SUC_lexicon_tags_25 },
    { 0xef3bebd3, SUC_lexicon_tags_2 },
    { 0x723bebd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1bebd9, SUC_lexicon_tags_47 },
    { 0xd53bebda, SUC_lexicon_tags_15 },
    { 0xeb3bebdb, SUC_lexicon_tags_6 },
    { 0x7bebdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dbebe0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bbebe3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x8c3bebe5, SUC_lexicon_tags_29 },
    { 0x5cfbebe5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x44bbebe8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x259bebeb, SUC_lexicon_tags_14 },
    { 0x3dfbebec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca1bebee, SUC_lexicon_tags_10 },
    { 0x701bebef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x329bebf1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7bebfc, SUC_lexicon_tags_30 },
    { 0x76bbebfd, SUC_lexicon_tags_31 },
    { 0xc17bebfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4cbbec00, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe23bec02, SUC_lexicon_tags_13 },
    { 0x563bec03, SUC_lexicon_tags_49 },
    { 0x13fbec03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dbec07, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafbec0b, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833bec11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd5bec13, SUC_lexicon_tags_38 },
    { 0x77dbec13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9bec1f, SUC_lexicon_tags_13 },
    { 0xc7fbec20, SUC_lexicon_tags_30 },
    { 0x7b7bec21, SUC_lexicon_tags_81 },
    { 0x2e5bec21, SUC_lexicon_tags_60 },
    { 0x1e1bec1f, SUC_lexicon_tags_8 },
    { 0xbb3bec21, SUC_lexicon_tags_19 },
    { 0x197bec25, SUC_lexicon_tags_37 },
    { 0x6fdbec24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcedbec28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bbec2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bec2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07bec32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bbec35, SUC_lexicon_tags_19 },
    { 0x6f3bec36, SUC_lexicon_tags_17 },
    { 0xe0dbec37, SUC_lexicon_tags_49 },
    { 0xe8bbec35, SUC_lexicon_tags_7 },
    { 0xe0bbec39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x64dbec3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fbec3e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89bec43, SUC_lexicon_tags_32 },
    { 0x2dfbec44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bec49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bbec4d, SUC_lexicon_tags_25 },
    { 0xe5dbec4e, SUC_lexicon_tags_50 },
    { 0x961bec4d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8a3bec51, SUC_lexicon_tags_60 },
    { 0xe9bbec52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5bec59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfdbec5b, SUC_lexicon_tags_170 },
    { 0xca9bec5c, SUC_lexicon_tags_15 },
    { 0x921bec5c, SUC_lexicon_tags_27 },
    { 0xc3bbec5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7a7bec60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1bec64, SUC_lexicon_tags_43 },
    { 0xa19bec65, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x391bec67, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x259bec6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945bec72, SUC_lexicon_tags_1 },
    { 0x6a3bec73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x79dbec75, SUC_lexicon_tags_15 },
    { 0xdbfbec75, SUC_lexicon_tags_87 },
    { 0xed9bec77, SUC_lexicon_tags_88 },
    { 0x203bec77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843bec80, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abbec86, SUC_lexicon_tags_23 },
    { 0x477bec86, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5bec8a, SUC_lexicon_tags_160 },
    { 0x2f1bec8a, SUC_lexicon_tags_37 },
    { 0xfb3bec8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41bec92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3bec95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dfbec97, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341bec9c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dbeca1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdbeca5, SUC_lexicon_tags_133 },
    { 0xf39beca6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17beca9, SUC_lexicon_tags_45 },
    { 0xf2dbecaa, SUC_lexicon_tags_45 },
    { 0x2bfbecaa, SUC_lexicon_tags_9 },
    { 0x277becaa, SUC_lexicon_tags_15 },
    { 0xffbbeca9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x791becaf, SUC_lexicon_tags_63 },
    { 0xbebbecb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3becb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45becb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x927becba, SUC_lexicon_tags_25 },
    { 0xed3becba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dbecc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bbecc9, SUC_lexicon_tags_11 },
    { 0x139becca, SUC_lexicon_tags_0 },
    { 0xfe3beccb, SUC_lexicon_tags_8 },
    { 0x1afbeccb, SUC_lexicon_tags_11 },
    { 0x1efbeccd, SUC_lexicon_tags_37 },
    { 0xa93becce, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bbecd1, SUC_lexicon_tags_47 },
    { 0x31fbecd1, SUC_lexicon_tags_17 },
    { 0x90fbecd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x84bbecd5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fbecdd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8c1becdf, SUC_lexicon_tags_31 },
    { 0x7becdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbece3, SUC_lexicon_tags_7 },
    { 0x6f9bece4, SUC_lexicon_tags_39 },
    { 0x4b1bece4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75bece8, SUC_lexicon_tags_27 },
    { 0xec5bece9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bbecef, SUC_lexicon_tags_26 },
    { 0x787becf0, SUC_lexicon_tags_25 },
    { 0xe65becf1, SUC_lexicon_tags_63 },
    { 0x43fbecf1, SUC_lexicon_tags_17 },
    { 0x57becf3, SUC_lexicon_tags_79 },
    { 0xde3becf3, SUC_lexicon_tags_27 },
    { 0xc51becf5, SUC_lexicon_tags_0 },
    { 0xce3becf1, SUC_lexicon_tags_10 },
    { 0xc0dbecf0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437becfd, SUC_lexicon_tags_14 },
    { 0xdc3becfe, SUC_lexicon_tags_15 },
    { 0xdd1becff, SUC_lexicon_tags_15 },
    { 0x1ebbecff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64dbed02, SUC_lexicon_tags_9 },
    { 0x9f3bed03, SUC_lexicon_tags_33 },
    { 0x187bed04, SUC_lexicon_tags_23 },
    { 0xd47bed04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfedbed07, SUC_lexicon_tags_46 },
    { 0x87dbed08, SUC_lexicon_tags_65 },
    { 0x879bed08, SUC_lexicon_tags_201 },
    { 0x6dbbed0a, SUC_lexicon_tags_6 },
    { 0xe0fbed0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa79bed0d, SUC_lexicon_tags_60 },
    { 0xb8bbed0e, SUC_lexicon_tags_33 },
    { 0x141bed0d, SUC_lexicon_tags_12 },
    { 0xf2bbed10, SUC_lexicon_tags_9 },
    { 0x76fbed11, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfbed15, SUC_lexicon_tags_17 },
    { 0x65fbed16, SUC_lexicon_tags_66 },
    { 0xbd9bed15, SUC_lexicon_tags_11 },
    { 0x677bed17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbbed1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dbed20, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffbed23, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xb25bed25, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dbed2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879bed2f, SUC_lexicon_tags_12 },
    { 0x783bed2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85bed34, SUC_lexicon_tags_14 },
    { 0x725bed35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bbed38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fdbed3a, SUC_lexicon_tags_12 },
    { 0x27bed3a, SUC_lexicon_tags_17 },
    { 0xdbdbed3a, SUC_lexicon_tags_43 },
    { 0x4cfbed3a, SUC_lexicon_tags_25 },
    { 0x1a3bed3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x19dbed40, SUC_lexicon_tags_27 },
    { 0x6d3bed41, SUC_lexicon_tags_31 },
    { 0xfb1bed42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919bed48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bed50, SUC_lexicon_tags_12 },
    { 0x917bed50, SUC_lexicon_tags_98 },
    { 0x627bed50, SUC_lexicon_tags_2 },
    { 0xdcbbed53, SUC_lexicon_tags_12 },
    { 0xbcdbed54, SUC_lexicon_tags_11 },
    { 0xf21bed55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4dfbed57, SUC_lexicon_tags_25 },
    { 0xe61bed58, SUC_lexicon_tags_2 },
    { 0xc6dbed58, SUC_lexicon_tags_19 },
    { 0xee9bed58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365bed5e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x47dbed60, SUC_lexicon_tags_37 },
    { 0xf1bbed60, SUC_lexicon_tags_7 },
    { 0x597bed61, SUC_lexicon_tags_33 },
    { 0x4ddbed60, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83bed66, SUC_lexicon_tags_17 },
    { 0xe1dbed67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4bbed69, SUC_lexicon_tags_47 },
    { 0xc1fbed6a, SUC_lexicon_tags_25 },
    { 0xb49bed6a, SUC_lexicon_tags_27 },
    { 0x62bbed6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1bed6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01bed75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdbed79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa7bed7b, SUC_lexicon_tags_60 },
    { 0x6d7bed7c, SUC_lexicon_tags_63 },
    { 0x239bed7d, SUC_lexicon_tags_15 },
    { 0x8fdbed7e, SUC_lexicon_tags_8 },
    { 0x35fbed7e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7bed8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7bed90, SUC_lexicon_tags_31 },
    { 0x9e9bed91, SUC_lexicon_tags_15 },
    { 0x469bed91, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x81fbed94, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x311bed97, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfbed9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abbeda1, SUC_lexicon_tags_2 },
    { 0x15dbeda1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x18fbeda4, SUC_lexicon_tags_12 },
    { 0x2dbeda5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5beda9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa09bedab, SUC_lexicon_tags_95 },
    { 0x583bedac, SUC_lexicon_tags_17 },
    { 0x44dbedac, SUC_lexicon_tags_224 },
    { 0xb0bbedae, SUC_lexicon_tags_16 },
    { 0x82fbedaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc23bedb1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x91fbedb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdddbedb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799bedbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3bedbf, SUC_lexicon_tags_18 },
    { 0x687bedc0, SUC_lexicon_tags_15 },
    { 0x779bedc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x817bedc3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x33fbedc5, SUC_lexicon_tags_1 },
    { 0xc23bedc5, SUC_lexicon_tags_28 },
    { 0xbdfbedc6, SUC_lexicon_tags_36 },
    { 0x2e1bedc8, SUC_lexicon_tags_14 },
    { 0x447bedc8, SUC_lexicon_tags_32 },
    { 0x8f7bedca, SUC_lexicon_tags_12 },
    { 0x79bedcb, SUC_lexicon_tags_11 },
    { 0xc2bbedcc, SUC_lexicon_tags_37 },
    { 0x19bbedcd, SUC_lexicon_tags_107 },
    { 0xfcdbedce, SUC_lexicon_tags_15 },
    { 0x399bedcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4adbedd1, SUC_lexicon_tags_29 },
    { 0xa3fbedd1, SUC_lexicon_tags_43 },
    { 0x2abbedd1, SUC_lexicon_tags_14 },
    { 0x82bbedd4, SUC_lexicon_tags_37 },
    { 0x97fbedd5, SUC_lexicon_tags_2 },
    { 0xb19bedd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a7bedd8, SUC_lexicon_tags_25 },
    { 0xe89bedd9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20dbedde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x331bede1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfbede5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1bede8, SUC_lexicon_tags_37 },
    { 0x7a1bede9, SUC_lexicon_tags_45 },
    { 0xe97bedea, SUC_lexicon_tags_9 },
    { 0x5d7bede9, SUC_lexicon_tags_33 },
    { 0x87bbedec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bedf0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491bedf5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237bedf9, SUC_lexicon_tags_2 },
    { 0x82fbedfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x995bedfd, SUC_lexicon_tags_1390 },
    { 0x6f5bedfd, SUC_lexicon_tags_27 },
    { 0x561bedff, SUC_lexicon_tags_7 },
    { 0xec9bedfd, SUC_lexicon_tags_25 },
    { 0x431bee01, SUC_lexicon_tags_31 },
    { 0x7ffbee02, SUC_lexicon_tags_37 },
    { 0x25fbee02, SUC_lexicon_tags_20 },
    { 0x5c1bee01, SUC_lexicon_tags_2 },
    { 0xc2bbee04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1bbee07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3bee0b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1bee10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65bee15, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5bee18, SUC_lexicon_tags_87 },
    { 0x34dbee18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4fbee1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9bee20, SUC_lexicon_tags_45 },
    { 0x507bee21, SUC_lexicon_tags_7 },
    { 0xf3fbee21, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbbee25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9bee29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x547bee2b, SUC_lexicon_tags_15 },
    { 0x257bee2c, SUC_lexicon_tags_19 },
    { 0x8f3bee2d, SUC_lexicon_tags_15 },
    { 0x43fbee2d, SUC_lexicon_tags_0 },
    { 0xdd9bee2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5bee32, SUC_lexicon_tags_45 },
    { 0x473bee32, SUC_lexicon_tags_31 },
    { 0xe2bbee34, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3edbee36, SUC_lexicon_tags_19 },
    { 0x925bee36, SUC_lexicon_tags_29 },
    { 0xc5bee37, SUC_lexicon_tags_25 },
    { 0x683bee39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05bee3c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafbee42, SUC_lexicon_tags_7 },
    { 0x44fbee43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39fbee45, SUC_lexicon_tags_11 },
    { 0xb05bee46, SUC_lexicon_tags_2 },
    { 0x10dbee47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dbee4c, SUC_lexicon_tags_12 },
    { 0x561bee4d, SUC_lexicon_tags_30 },
    { 0xd69bee4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a9bee50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x565bee52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293bee57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x597bee59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bbee5e, SUC_lexicon_tags_0 },
    { 0xfffbee5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5bee63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9bee68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85bee6a, SUC_lexicon_tags_49 },
    { 0xc89bee6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2dbee6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5bee70, SUC_lexicon_tags_8 },
    { 0x807bee71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb13bee73, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35bee76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fbee79, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x665bee7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17bbee7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70fbee7f, SUC_lexicon_tags_307 },
    { 0xb8fbee7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181bee84, SUC_lexicon_tags_47 },
    { 0x567bee85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x27dbee88, SUC_lexicon_tags_3 },
    { 0x43bee88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xee9bee8b, SUC_lexicon_tags_23 },
    { 0x43bbee8c, SUC_lexicon_tags_37 },
    { 0x75fbee8c, SUC_lexicon_tags_38 },
    { 0xdcfbee8d, SUC_lexicon_tags_47 },
    { 0x4b1bee8f, SUC_lexicon_tags_15 },
    { 0xbd1bee8f, SUC_lexicon_tags_19 },
    { 0x171bee91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e1bee93, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x76bbee96, SUC_lexicon_tags_17 },
    { 0x8f5bee97, SUC_lexicon_tags_15 },
    { 0x297bee97, SUC_lexicon_tags_19 },
    { 0xf97bee97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fbee9d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x135bee9f, SUC_lexicon_tags_2 },
    { 0x285beea0, SUC_lexicon_tags_15 },
    { 0x973beea1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe7fbeea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43beea6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319beeab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xaddbeead, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1fbeeaf, SUC_lexicon_tags_12 },
    { 0x1a7beeb0, SUC_lexicon_tags_15 },
    { 0xea5beeb0, SUC_lexicon_tags_2 },
    { 0x49dbeeaf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xbb7beeb4, SUC_lexicon_tags_39 },
    { 0x1b3beeb5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95beec1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81beec4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85beec7, SUC_lexicon_tags_17 },
    { 0xcbfbeec8, SUC_lexicon_tags_13 },
    { 0xdcdbeec9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x455beecb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507beed0, SUC_lexicon_tags_12 },
    { 0x4e5beed1, SUC_lexicon_tags_37 },
    { 0x7e5beed1, SUC_lexicon_tags_43 },
    { 0x401beed3, SUC_lexicon_tags_15 },
    { 0x4c1beed4, SUC_lexicon_tags_45 },
    { 0xe03beed5, SUC_lexicon_tags_11 },
    { 0xf39beed6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efbeeda, SUC_lexicon_tags_31 },
    { 0x8e7beedb, SUC_lexicon_tags_47 },
    { 0xbb7beedc, SUC_lexicon_tags_7 },
    { 0x22dbeedb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabbeee3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dbeee8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5beeec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ddbeeee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45beef3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cfbeef5, SUC_lexicon_tags_45 },
    { 0x3beef6, SUC_lexicon_tags_0 },
    { 0x625beef7, SUC_lexicon_tags_60 },
    { 0xa37beef8, SUC_lexicon_tags_7 },
    { 0x36fbeef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09beefd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03bef00, SUC_lexicon_tags_66 },
    { 0x8f9bef01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf83bef03, SUC_lexicon_tags_14 },
    { 0xaf9bef04, SUC_lexicon_tags_19 },
    { 0xf27bef05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1b7bef07, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921bef0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03bef12, SUC_lexicon_tags_14 },
    { 0xd85bef12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1bef1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fbef1f, SUC_lexicon_tags_38 },
    { 0x785bef20, SUC_lexicon_tags_23 },
    { 0xc67bef21, SUC_lexicon_tags_8 },
    { 0x27fbef22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bbef25, SUC_lexicon_tags_11 },
    { 0x2d3bef26, SUC_lexicon_tags_45 },
    { 0xb93bef26, SUC_lexicon_tags_16 },
    { 0x7d9bef28, SUC_lexicon_tags_22 },
    { 0x3bfbef26, SUC_lexicon_tags_38 },
    { 0x2ffbef2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bef2d, SUC_lexicon_tags_2 },
    { 0x641bef2d, SUC_lexicon_tags_38 },
    { 0xa2fbef2f, SUC_lexicon_tags_37 },
    { 0xf21bef2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293bef36, SUC_lexicon_tags_23 },
    { 0x54bbef37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x423bef3a, SUC_lexicon_tags_2 },
    { 0x557bef3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa5fbef3d, SUC_lexicon_tags_27 },
    { 0xf29bef3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd47bef40, SUC_lexicon_tags_37 },
    { 0x313bef40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2efbef44, SUC_lexicon_tags_11 },
    { 0xe7dbef44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49bef48, SUC_lexicon_tags_15 },
    { 0x605bef49, SUC_lexicon_tags_15 },
    { 0xdcbbef4a, SUC_lexicon_tags_2 },
    { 0xad9bef49, SUC_lexicon_tags_37 },
    { 0x499bef48, SUC_lexicon_tags_31 },
    { 0x51bbef4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9bef52, SUC_lexicon_tags_84 },
    { 0x371bef52, SUC_lexicon_tags_38 },
    { 0x9afbef54, SUC_lexicon_tags_2 },
    { 0x4b7bef55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d9bef57, SUC_lexicon_tags_11 },
    { 0xf53bef58, SUC_lexicon_tags_9 },
    { 0xa1fbef59, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x24fbef5b, SUC_lexicon_tags_8 },
    { 0xb3bef5b, SUC_lexicon_tags_1 },
    { 0x57bbef5d, SUC_lexicon_tags_37 },
    { 0xb61bef5e, SUC_lexicon_tags_26 },
    { 0xef1bef5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbbef67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5bef6d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7bef72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea7bef74, SUC_lexicon_tags_8 },
    { 0x3d9bef74, SUC_lexicon_tags_2 },
    { 0xb3fbef75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33bef7b, SUC_lexicon_tags_68 },
    { 0x92dbef7b, SUC_lexicon_tags_48 },
    { 0x9d3bef7d, SUC_lexicon_tags_15 },
    { 0x157bef7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7bef82, SUC_lexicon_tags_15 },
    { 0x315bef83, SUC_lexicon_tags_0 },
    { 0x8d3bef83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fbef88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x21fbef8b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817bef90, SUC_lexicon_tags_19 },
    { 0xa5dbef91, SUC_lexicon_tags_11 },
    { 0x7bef92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f1bef94, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27bef97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe85bef99, SUC_lexicon_tags_37 },
    { 0xaf9bef99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5afbef9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9befa3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fbefa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fbefab, SUC_lexicon_tags_24 },
    { 0x13befac, SUC_lexicon_tags_1 },
    { 0x5fbbefad, SUC_lexicon_tags_45 },
    { 0xb8dbefae, SUC_lexicon_tags_15 },
    { 0xaa9befae, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x685befb1, SUC_lexicon_tags_46 },
    { 0xa29befb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x739befb4, SUC_lexicon_tags_28 },
    { 0x2dfbefb5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bbefb8, SUC_lexicon_tags_56 },
    { 0xf07befb8, SUC_lexicon_tags_15 },
    { 0xddfbefba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x789befbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd2bbefbe, SUC_lexicon_tags_37 },
    { 0x673befbe, SUC_lexicon_tags_46 },
    { 0x291befbe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x145befc2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd7befc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5befc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3befcb, SUC_lexicon_tags_49 },
    { 0xc1fbefcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdbefd2, SUC_lexicon_tags_32 },
    { 0xe83befd2, SUC_lexicon_tags_11 },
    { 0x615befd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29befd7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bbefdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229befe5, SUC_lexicon_tags_11 },
    { 0xe41befe6, SUC_lexicon_tags_17 },
    { 0xa8dbefe7, SUC_lexicon_tags_87 },
    { 0xb93befe8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36fbefea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7beff0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbeff8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47beffd, SUC_lexicon_tags_10 },
    { 0xa8bbeffd, SUC_lexicon_tags_17 },
    { 0x9a7befff, SUC_lexicon_tags_37 },
    { 0xcb5bf000, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17bf005, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaffbf007, SUC_lexicon_tags_26 },
    { 0xab1bf007, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x481bf00a, SUC_lexicon_tags_1 },
    { 0x78dbf00a, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0xffdbf00e, SUC_lexicon_tags_34 },
    { 0xf2fbf00f, SUC_lexicon_tags_45 },
    { 0x5bbf00f, SUC_lexicon_tags_19 },
    { 0x723bf011, SUC_lexicon_tags_33 },
    { 0x685bf012, SUC_lexicon_tags_52 },
    { 0xdddbf011, SUC_lexicon_tags_92 },
    { 0x1bdbf012, SUC_lexicon_tags_11 },
    { 0x25fbf015, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1bf019, SUC_lexicon_tags_49 },
    { 0xd59bf01a, SUC_lexicon_tags_45 },
    { 0x143bf019, SUC_lexicon_tags_7 },
    { 0xf7bbf01c, SUC_lexicon_tags_11 },
    { 0x667bf01d, SUC_lexicon_tags_55 },
    { 0xf9dbf01e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efbf021, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fbf024, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbf029, SUC_lexicon_tags_79 },
    { 0x39bf02a, SUC_lexicon_tags_11 },
    { 0xdbdbf02b, SUC_lexicon_tags_84 },
    { 0xe45bf02c, SUC_lexicon_tags_15 },
    { 0x533bf02b, SUC_lexicon_tags_2 },
    { 0x487bf02b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca3bf030, SUC_lexicon_tags_37 },
    { 0x877bf031, SUC_lexicon_tags_40 },
    { 0xd05bf032, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x60fbf034, SUC_lexicon_tags_11 },
    { 0x1fbf035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3bf039, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fbf040, SUC_lexicon_tags_60 },
    { 0xcdbbf040, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73bbf043, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1bf046, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fbf04e, SUC_lexicon_tags_11 },
    { 0x7bbf04f, SUC_lexicon_tags_7 },
    { 0x971bf04e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9bf054, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1bf058, SUC_lexicon_tags_60 },
    { 0xa95bf059, SUC_lexicon_tags_1452 },
    { 0x209bf059, SUC_lexicon_tags_15 },
    { 0x10fbf05b, SUC_lexicon_tags_88 },
    { 0x52fbf05b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419bf060, SUC_lexicon_tags_43 },
    { 0x4dbbf061, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x471bf063, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x63bf065, SUC_lexicon_tags_33 },
    { 0x66dbf066, SUC_lexicon_tags_14 },
    { 0xc39bf067, SUC_lexicon_tags_63 },
    { 0x1dbbf068, SUC_lexicon_tags_15 },
    { 0x309bf069, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fbf06d, SUC_lexicon_tags_68 },
    { 0x4b7bf06d, SUC_lexicon_tags_37 },
    { 0xebdbf06f, SUC_lexicon_tags_23 },
    { 0x77dbf070, SUC_lexicon_tags_19 },
    { 0xf55bf070, SUC_lexicon_tags_37 },
    { 0xcbfbf072, SUC_lexicon_tags_60 },
    { 0xcbdbf073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa31bf075, SUC_lexicon_tags_224 },
    { 0xc4dbf076, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f5bf078, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x389bf07b, SUC_lexicon_tags_23 },
    { 0x413bf07b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dbf07f, SUC_lexicon_tags_10 },
    { 0xb49bf080, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x743bf082, SUC_lexicon_tags_7 },
    { 0x877bf083, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4f5bf085, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6abbf087, SUC_lexicon_tags_14 },
    { 0x8a9bf088, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x24bbf08b, SUC_lexicon_tags_45 },
    { 0x5e7bf08c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bbf08f, SUC_lexicon_tags_37 },
    { 0x10fbf08f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e1bf092, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19fbf094, SUC_lexicon_tags_37 },
    { 0x693bf095, SUC_lexicon_tags_12 },
    { 0x4e1bf096, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fbf09a, SUC_lexicon_tags_6 },
    { 0xddbbf09b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bf0a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf57bf0a4, SUC_lexicon_tags_11 },
    { 0x511bf0a5, SUC_lexicon_tags_4 },
    { 0xe73bf0a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7bf0a9, SUC_lexicon_tags_63 },
    { 0xe91bf0a9, SUC_lexicon_tags_15 },
    { 0xdd7bf0ab, SUC_lexicon_tags_133 },
    { 0x847bf0ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fbf0b0, SUC_lexicon_tags_0 },
    { 0x291bf0b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adbf0b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2edbf0b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedbf0bb, SUC_lexicon_tags_87 },
    { 0x475bf0bc, SUC_lexicon_tags_2 },
    { 0x3bdbf0bd, SUC_lexicon_tags_2 },
    { 0x8bfbf0bd, SUC_lexicon_tags_34 },
    { 0xc17bf0be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40dbf0c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7bbf0c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79bf0c7, SUC_lexicon_tags_11 },
    { 0x565bf0c8, SUC_lexicon_tags_12 },
    { 0x799bf0c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bbf0cd, SUC_lexicon_tags_27 },
    { 0x389bf0cd, SUC_lexicon_tags_17 },
    { 0x119bf0cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcedbf0d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135bf0d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29fbf0dc, SUC_lexicon_tags_33 },
    { 0xdf5bf0dc, SUC_lexicon_tags_12 },
    { 0x233bf0de, SUC_lexicon_tags_0 },
    { 0x36bbf0de, SUC_lexicon_tags_39 },
    { 0xfe9bf0e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bf0e5, SUC_lexicon_tags_27 },
    { 0xcf7bf0e6, SUC_lexicon_tags_45 },
    { 0x50bbf0e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dbf0ec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51bf0f1, SUC_lexicon_tags_36 },
    { 0x3dbf0f2, SUC_lexicon_tags_17 },
    { 0x149bf0f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x943bf0f6, SUC_lexicon_tags_47 },
    { 0xeedbf0f6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883bf0fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1bf102, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3bf109, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1bf10d, SUC_lexicon_tags_25 },
    { 0xe95bf10e, SUC_lexicon_tags_7 },
    { 0x775bf10e, SUC_lexicon_tags_37 },
    { 0x65fbf110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a7bf112, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa15bf114, SUC_lexicon_tags_11 },
    { 0xc9dbf114, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245bf119, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fbbf11b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259bf123, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x871bf126, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd99bf128, SUC_lexicon_tags_9 },
    { 0xd11bf129, SUC_lexicon_tags_0 },
    { 0xc3dbf12a, SUC_lexicon_tags_15 },
    { 0x1d5bf128, SUC_lexicon_tags_15 },
    { 0xffdbf12b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3bf131, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9bf137, SUC_lexicon_tags_1 },
    { 0x8c1bf138, SUC_lexicon_tags_0 },
    { 0x719bf138, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1bf140, SUC_lexicon_tags_25 },
    { 0xd3bf140, SUC_lexicon_tags_77 },
    { 0x651bf141, SUC_lexicon_tags_14 },
    { 0x92dbf141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x91fbf145, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf7bf147, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31dbf14d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48bbf14f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffbf154, SUC_lexicon_tags_17 },
    { 0x981bf155, SUC_lexicon_tags_31 },
    { 0x197bf155, SUC_lexicon_tags_15 },
    { 0xeb1bf157, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xacdbf159, SUC_lexicon_tags_2 },
    { 0xdb5bf15a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcbbf15c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6fbf15e, SUC_lexicon_tags_26 },
    { 0x53bbf15f, SUC_lexicon_tags_63 },
    { 0xeddbf160, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39bf163, SUC_lexicon_tags_15 },
    { 0xab5bf164, SUC_lexicon_tags_11 },
    { 0xc83bf164, SUC_lexicon_tags_33 },
    { 0x617bf166, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3dbf16c, SUC_lexicon_tags_23 },
    { 0xbd9bf16c, SUC_lexicon_tags_17 },
    { 0x905bf16d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3bf172, SUC_lexicon_tags_30 },
    { 0x1f5bf173, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x375bf175, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbbf17a, SUC_lexicon_tags_25 },
    { 0x533bf17a, SUC_lexicon_tags_47 },
    { 0xa7dbf17a, SUC_lexicon_tags_37 },
    { 0x4ffbf17d, SUC_lexicon_tags_12 },
    { 0xb3dbf17a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9fbf180, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadbf183, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xc01bf185, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5bf18e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x135bf191, SUC_lexicon_tags_17 },
    { 0xa7bbf192, SUC_lexicon_tags_23 },
    { 0x97bf192, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x869bf195, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabbf198, SUC_lexicon_tags_7 },
    { 0xc13bf198, SUC_lexicon_tags_43 },
    { 0x8e3bf19a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7bf19d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbf1a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31fbf1a2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf3bf1a4, SUC_lexicon_tags_2 },
    { 0xa5fbf1a5, SUC_lexicon_tags_45 },
    { 0xce1bf1a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x181bf1a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc3fbf1aa, SUC_lexicon_tags_17 },
    { 0x607bf1aa, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x967bf1ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x949bf1af, SUC_lexicon_tags_11 },
    { 0x739bf1b0, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88bbf1b4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf1b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9bf1bd, SUC_lexicon_tags_15 },
    { 0x1edbf1bd, SUC_lexicon_tags_25 },
    { 0xb5dbf1bf, SUC_lexicon_tags_23 },
    { 0x78fbf1c0, SUC_lexicon_tags_63 },
    { 0x113bf1c1, SUC_lexicon_tags_38 },
    { 0x59fbf1c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x725bf1c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71bf1ca, SUC_lexicon_tags_23 },
    { 0x13bf1cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9bf1d0, SUC_lexicon_tags_55 },
    { 0xc6bbf1d1, SUC_lexicon_tags_29 },
    { 0x81bbf1d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x411bf1d5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x5c3bf1d7, SUC_lexicon_tags_60 },
    { 0xe45bf1d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77bbf1da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dbf1dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3bf1e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05bf1e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbf1e6, SUC_lexicon_tags_7 },
    { 0x59fbf1e7, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xc4dbf1e9, SUC_lexicon_tags_3 },
    { 0x5c9bf1ea, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x83bf1ec, SUC_lexicon_tags_14 },
    { 0xdd7bf1ed, SUC_lexicon_tags_62 },
    { 0x28fbf1ec, SUC_lexicon_tags_46 },
    { 0x553bf1ef, SUC_lexicon_tags_12 },
    { 0x23fbf1f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bbf1f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddbf1f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81bf1fa, SUC_lexicon_tags_11 },
    { 0x15dbf1fb, SUC_lexicon_tags_108 },
    { 0xf6bbf1fa, SUC_lexicon_tags_37 },
    { 0x637bf1fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfbf200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5bf203, SUC_lexicon_tags_33 },
    { 0x5b9bf204, SUC_lexicon_tags_17 },
    { 0xa3bf205, SUC_lexicon_tags_7 },
    { 0xc61bf203, SUC_lexicon_tags_26 },
    { 0x69bf207, SUC_lexicon_tags_17 },
    { 0xf9bf208, SUC_lexicon_tags_17 },
    { 0x8fbbf205, SUC_lexicon_tags_17 },
    { 0x3cfbf20a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x98dbf20c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5bf20f, SUC_lexicon_tags_61 },
    { 0x377bf210, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519bf214, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xee7bf216, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bbf21a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x551bf21d, SUC_lexicon_tags_19 },
    { 0x7b1bf21e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abbf221, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bbf225, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd69bf227, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bbf22b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49bf231, SUC_lexicon_tags_2 },
    { 0x847bf232, SUC_lexicon_tags_25 },
    { 0xc95bf232, SUC_lexicon_tags_37 },
    { 0xaa1bf232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615bf239, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fbf23f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997bf243, SUC_lexicon_tags_19 },
    { 0x58bbf243, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efbf247, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x483bf249, SUC_lexicon_tags_25 },
    { 0xbcbbf249, SUC_lexicon_tags_37 },
    { 0x315bf24b, SUC_lexicon_tags_0 },
    { 0x1dbbf24b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bbf253, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x505bf256, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1bf25f, SUC_lexicon_tags_1653 },
    { 0, NULL },
    { 0x4c3bf261, SUC_lexicon_tags_9 },
    { 0x9efbf262, SUC_lexicon_tags_45 },
    { 0x9b5bf261, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efbf268, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bf26b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedbf271, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dbf275, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38fbf277, SUC_lexicon_tags_45 },
    { 0xdbf278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf83bf27a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3bf27d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1bf281, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41bf284, SUC_lexicon_tags_27 },
    { 0xbf7bf285, SUC_lexicon_tags_11 },
    { 0x715bf286, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fbf28a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751bf292, SUC_lexicon_tags_6 },
    { 0xdedbf293, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x633bf295, SUC_lexicon_tags_12 },
    { 0x235bf296, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x643bf299, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9bf29c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421bf2a0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1ffbf2a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f3bf2a4, SUC_lexicon_tags_60 },
    { 0xa97bf2a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x65bbf2a8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xae5bf2aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6edbf2ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8dfbf2ae, SUC_lexicon_tags_2 },
    { 0x997bf2ae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdbf2b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07bf2b6, SUC_lexicon_tags_15 },
    { 0x24bbf2b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa07bf2b9, SUC_lexicon_tags_14 },
    { 0x9bbbf2ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fbf2bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x723bf2bf, SUC_lexicon_tags_1 },
    { 0x59bbf2bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fbf2c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51bf2c5, SUC_lexicon_tags_27 },
    { 0x151bf2c6, SUC_lexicon_tags_15 },
    { 0x55bbf2c7, SUC_lexicon_tags_12 },
    { 0x125bf2c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15bf2cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5bf2d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb09bf2d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dbf2d5, SUC_lexicon_tags_7 },
    { 0xfbfbf2d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x231bf2d8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fbf2db, SUC_lexicon_tags_37 },
    { 0xa99bf2db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe43bf2de, SUC_lexicon_tags_9 },
    { 0x397bf2df, SUC_lexicon_tags_25 },
    { 0xbfbbf2e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c1bf2e2, SUC_lexicon_tags_64 },
    { 0x1fbf2e3, SUC_lexicon_tags_605 },
    { 0xb01bf2e4, SUC_lexicon_tags_38 },
    { 0xdedbf2e4, SUC_lexicon_tags_15 },
    { 0x33bf2e6, SUC_lexicon_tags_49 },
    { 0xf1fbf2e2, SUC_lexicon_tags_38 },
    { 0xf79bf2e7, SUC_lexicon_tags_7 },
    { 0x7e1bf2e3, SUC_lexicon_tags_17 },
    { 0xaddbf2ea, SUC_lexicon_tags_46 },
    { 0x5afbf2e2, SUC_lexicon_tags_45 },
    { 0x1c1bf2e6, SUC_lexicon_tags_2 },
    { 0xe1fbf2eb, SUC_lexicon_tags_0 },
    { 0xc07bf2ee, SUC_lexicon_tags_22 },
    { 0x751bf2ee, SUC_lexicon_tags_118 },
    { 0xd29bf2f0, SUC_lexicon_tags_38 },
    { 0x42dbf2f1, SUC_lexicon_tags_33 },
    { 0x3e1bf2f2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x513bf2f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3b9bf2f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x953bf2f8, SUC_lexicon_tags_138 },
    { 0xf6dbf2f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5bf2ff, SUC_lexicon_tags_1 },
    { 0x2b1bf2ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bbf303, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd29bf305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5bf309, SUC_lexicon_tags_20 },
    { 0x18bbf30a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9bf30f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467bf315, SUC_lexicon_tags_47 },
    { 0x791bf316, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869bf31d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7bf320, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bbf325, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x88fbf327, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889bf32d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bbf33d, SUC_lexicon_tags_29 },
    { 0x3e9bf33d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd3dbf340, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bf345, SUC_lexicon_tags_29 },
    { 0x5e9bf346, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95bf349, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe5bf34b, SUC_lexicon_tags_25 },
    { 0x259bf34b, SUC_lexicon_tags_26 },
    { 0xbfdbf34b, SUC_lexicon_tags_1 },
    { 0xea9bf34c, SUC_lexicon_tags_12 },
    { 0xc19bf34f, SUC_lexicon_tags_3 },
    { 0xcffbf34f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bbf353, SUC_lexicon_tags_65 },
    { 0x62dbf353, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fbf35a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bbf35f, SUC_lexicon_tags_1147 },
    { 0x5f3bf360, SUC_lexicon_tags_7 },
    { 0x6ddbf360, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385bf369, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5bf36d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x68dbf36f, SUC_lexicon_tags_3 },
    { 0x4e1bf370, SUC_lexicon_tags_12 },
    { 0x3f7bf371, SUC_lexicon_tags_11 },
    { 0xa35bf372, SUC_lexicon_tags_1 },
    { 0x9bbbf373, SUC_lexicon_tags_32 },
    { 0xc6fbf371, SUC_lexicon_tags_82 },
    { 0x381bf375, SUC_lexicon_tags_23 },
    { 0x673bf375, SUC_lexicon_tags_33 },
    { 0x50bbf377, SUC_lexicon_tags_2 },
    { 0xbf9bf378, SUC_lexicon_tags_107 },
    { 0x203bf379, SUC_lexicon_tags_1 },
    { 0x8e7bf36f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dbf37e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2b9bf380, SUC_lexicon_tags_60 },
    { 0x6f9bf381, SUC_lexicon_tags_11 },
    { 0xba5bf382, SUC_lexicon_tags_25 },
    { 0x129bf381, SUC_lexicon_tags_32 },
    { 0xe11bf380, SUC_lexicon_tags_82 },
    { 0x6c5bf381, SUC_lexicon_tags_37 },
    { 0xcf9bf386, SUC_lexicon_tags_45 },
    { 0xfc9bf387, SUC_lexicon_tags_37 },
    { 0x855bf388, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x297bf38a, SUC_lexicon_tags_45 },
    { 0x8c7bf38b, SUC_lexicon_tags_15 },
    { 0x6cdbf38b, SUC_lexicon_tags_60 },
    { 0x187bf38d, SUC_lexicon_tags_39 },
    { 0x7b7bf38e, SUC_lexicon_tags_23 },
    { 0xd57bf38c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9b5bf391, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5bf396, SUC_lexicon_tags_7 },
    { 0x643bf397, SUC_lexicon_tags_10 },
    { 0x49bbf398, SUC_lexicon_tags_33 },
    { 0x6bfbf397, SUC_lexicon_tags_63 },
    { 0x8e7bf39a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe69bf39c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1bf3a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x69dbf3a3, SUC_lexicon_tags_46 },
    { 0xce1bf3a4, SUC_lexicon_tags_14 },
    { 0xe61bf3a5, SUC_lexicon_tags_33 },
    { 0x103bf3a3, SUC_lexicon_tags_23 },
    { 0x385bf3a7, SUC_lexicon_tags_14 },
    { 0x219bf3a8, SUC_lexicon_tags_28 },
    { 0xe1bbf3a7, SUC_lexicon_tags_27 },
    { 0x21bf3aa, SUC_lexicon_tags_12 },
    { 0xbf3bf3a5, SUC_lexicon_tags_11 },
    { 0xd87bf3a4, SUC_lexicon_tags_31 },
    { 0xc91bf3ad, SUC_lexicon_tags_6 },
    { 0x619bf3ae, SUC_lexicon_tags_17 },
    { 0x70bbf3af, SUC_lexicon_tags_3 },
    { 0xb03bf3a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639bf3b8, SUC_lexicon_tags_7 },
    { 0xb0fbf3b8, SUC_lexicon_tags_26 },
    { 0x5dbbf3b8, SUC_lexicon_tags_1 },
    { 0xa8fbf3bb, SUC_lexicon_tags_61 },
    { 0x681bf3bc, SUC_lexicon_tags_46 },
    { 0x451bf3bc, SUC_lexicon_tags_12 },
    { 0x1abbf3be, SUC_lexicon_tags_29 },
    { 0x7d3bf3bf, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dbf3c6, SUC_lexicon_tags_1 },
    { 0x6f9bf3c6, SUC_lexicon_tags_49 },
    { 0xce5bf3c8, SUC_lexicon_tags_19 },
    { 0x951bf3c9, SUC_lexicon_tags_39 },
    { 0xb37bf3c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dbf3d0, SUC_lexicon_tags_15 },
    { 0x8edbf3d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89bf3d3, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfbf3d8, SUC_lexicon_tags_17 },
    { 0x8cdbf3d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fbf3e0, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fbf3e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbbf3e8, SUC_lexicon_tags_38 },
    { 0xea7bf3e9, SUC_lexicon_tags_49 },
    { 0x34dbf3e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05bf3ed, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x1ffbf3ef, SUC_lexicon_tags_71 },
    { 0xfffbf3f0, SUC_lexicon_tags_12 },
    { 0xc2fbf3ef, SUC_lexicon_tags_11 },
    { 0x71bf3f2, SUC_lexicon_tags_0 },
    { 0x7dbbf3f3, SUC_lexicon_tags_47 },
    { 0xfc7bf3f1, SUC_lexicon_tags_11 },
    { 0xfc9bf3f5, SUC_lexicon_tags_15 },
    { 0x3c3bf3f6, SUC_lexicon_tags_12 },
    { 0x383bf3f7, SUC_lexicon_tags_11 },
    { 0x99fbf3f6, SUC_lexicon_tags_11 },
    { 0x699bf3f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dbf3fd, SUC_lexicon_tags_2 },
    { 0x5a1bf3fe, SUC_lexicon_tags_7 },
    { 0xd8fbf3fd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9bf404, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bbf408, SUC_lexicon_tags_60 },
    { 0x111bf409, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99bf411, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249bf416, SUC_lexicon_tags_33 },
    { 0x5a3bf417, SUC_lexicon_tags_31 },
    { 0x4bbbf417, SUC_lexicon_tags_65 },
    { 0xb03bf417, SUC_lexicon_tags_7 },
    { 0xf03bf41a, SUC_lexicon_tags_33 },
    { 0x477bf419, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x527bf41d, SUC_lexicon_tags_37 },
    { 0xad9bf41e, SUC_lexicon_tags_68 },
    { 0xc91bf41f, SUC_lexicon_tags_2 },
    { 0xbe1bf420, SUC_lexicon_tags_37 },
    { 0x9a7bf421, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc3bf423, SUC_lexicon_tags_1 },
    { 0x61bf424, SUC_lexicon_tags_12 },
    { 0x4b1bf425, SUC_lexicon_tags_16 },
    { 0xafbf426, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdbf42a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcf5bf42c, SUC_lexicon_tags_60 },
    { 0x383bf42d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x19dbf42f, SUC_lexicon_tags_2 },
    { 0x8a7bf430, SUC_lexicon_tags_45 },
    { 0x5c7bf430, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97bf43c, SUC_lexicon_tags_47 },
    { 0xd5fbf43d, SUC_lexicon_tags_113 },
    { 0x5e3bf43c, SUC_lexicon_tags_62 },
    { 0x4c3bf43d, SUC_lexicon_tags_37 },
    { 0xf2dbf440, SUC_lexicon_tags_8 },
    { 0xcc5bf440, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51bf445, SUC_lexicon_tags_41 },
    { 0x3cdbf446, SUC_lexicon_tags_15 },
    { 0xfcdbf447, SUC_lexicon_tags_55 },
    { 0x823bf448, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71bf44f, SUC_lexicon_tags_24 },
    { 0xbfbbf44f, SUC_lexicon_tags_47 },
    { 0x163bf450, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7bf454, SUC_lexicon_tags_14 },
    { 0x2e3bf455, SUC_lexicon_tags_1 },
    { 0x679bf454, SUC_lexicon_tags_12 },
    { 0xe3dbf457, SUC_lexicon_tags_17 },
    { 0x2e9bf457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bdbf45a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23bf45d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa71bf45f, SUC_lexicon_tags_15 },
    { 0xb1dbf45f, SUC_lexicon_tags_7 },
    { 0x715bf461, SUC_lexicon_tags_27 },
    { 0x151bf461, SUC_lexicon_tags_33 },
    { 0x5dbbf461, SUC_lexicon_tags_2 },
    { 0xd65bf463, SUC_lexicon_tags_10 },
    { 0xe7fbf465, SUC_lexicon_tags_15 },
    { 0x4d7bf466, SUC_lexicon_tags_47 },
    { 0xe7bf463, SUC_lexicon_tags_17 },
    { 0x34bbf461, SUC_lexicon_tags_7 },
    { 0x8bdbf468, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a1bf46b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfbf46e, SUC_lexicon_tags_17 },
    { 0x97bbf46f, SUC_lexicon_tags_14 },
    { 0x101bf470, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x611bf472, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471bf479, SUC_lexicon_tags_13 },
    { 0xb23bf47a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5bf47e, SUC_lexicon_tags_16 },
    { 0x8f1bf47f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85fbf481, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55bbf483, SUC_lexicon_tags_12 },
    { 0x555bf483, SUC_lexicon_tags_57 },
    { 0xb47bf485, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d1bf487, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9bf48d, SUC_lexicon_tags_30 },
    { 0xf03bf48e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xeebbf490, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88dbf494, SUC_lexicon_tags_46 },
    { 0x203bf495, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x863bf497, SUC_lexicon_tags_12 },
    { 0x947bf498, SUC_lexicon_tags_8 },
    { 0x41bf499, SUC_lexicon_tags_8 },
    { 0xf7dbf497, SUC_lexicon_tags_77 },
    { 0x907bf49b, SUC_lexicon_tags_15 },
    { 0x31bbf49c, SUC_lexicon_tags_39 },
    { 0xb3dbf49c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3bf4a0, SUC_lexicon_tags_91 },
    { 0x41bf4a0, SUC_lexicon_tags_6 },
    { 0x241bf4a2, SUC_lexicon_tags_37 },
    { 0x53dbf4a3, SUC_lexicon_tags_17 },
    { 0xf4bbf4a3, SUC_lexicon_tags_12 },
    { 0x83dbf4a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x219bf4a8, SUC_lexicon_tags_37 },
    { 0x6cfbf4a9, SUC_lexicon_tags_62 },
    { 0x56bbf4aa, SUC_lexicon_tags_2 },
    { 0x269bf4ab, SUC_lexicon_tags_5 },
    { 0x9a5bf4ac, SUC_lexicon_tags_33 },
    { 0x3cbbf4ab, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665bf4b6, SUC_lexicon_tags_18 },
    { 0x891bf4b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bf4bd, SUC_lexicon_tags_17 },
    { 0xff3bf4be, SUC_lexicon_tags_1 },
    { 0x5a1bf4be, SUC_lexicon_tags_16 },
    { 0x871bf4c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5bf4c5, SUC_lexicon_tags_49 },
    { 0x23dbf4c6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc95bf4c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21dbf4d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137bf4d5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x989bf4d8, SUC_lexicon_tags_35 },
    { 0x19fbf4d9, SUC_lexicon_tags_19 },
    { 0x885bf4d9, SUC_lexicon_tags_6 },
    { 0x67dbf4da, SUC_lexicon_tags_25 },
    { 0x149bf4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dbf4e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffdbf4e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7b9bf4e8, SUC_lexicon_tags_37 },
    { 0xe95bf4e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfbf4ed, SUC_lexicon_tags_37 },
    { 0x1a5bf4ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x677bf4f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bbf4f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa81bf4f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9c7bf4f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x19dbf4fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x869bf4fd, SUC_lexicon_tags_28 },
    { 0xf53bf4fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7bf502, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdbf506, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f7bf508, SUC_lexicon_tags_11 },
    { 0xa55bf509, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x45fbf50c, SUC_lexicon_tags_7 },
    { 0xee1bf50c, SUC_lexicon_tags_39 },
    { 0x28bbf50e, SUC_lexicon_tags_40 },
    { 0xbabbf50e, SUC_lexicon_tags_45 },
    { 0xb89bf50e, SUC_lexicon_tags_12 },
    { 0xb6bbf511, SUC_lexicon_tags_2 },
    { 0x435bf512, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bf515, SUC_lexicon_tags_19 },
    { 0x95bbf516, SUC_lexicon_tags_2 },
    { 0x59fbf517, SUC_lexicon_tags_2 },
    { 0xcf7bf516, SUC_lexicon_tags_25 },
    { 0xf8bbf519, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bf51f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9bf522, SUC_lexicon_tags_19 },
    { 0x257bf522, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dbf526, SUC_lexicon_tags_23 },
    { 0x153bf526, SUC_lexicon_tags_8 },
    { 0x1ffbf528, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbf52b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43bf530, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dbf534, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7bf538, SUC_lexicon_tags_34 },
    { 0x959bf539, SUC_lexicon_tags_25 },
    { 0xee3bf53a, SUC_lexicon_tags_12 },
    { 0x6a7bf53b, SUC_lexicon_tags_12 },
    { 0x109bf53c, SUC_lexicon_tags_17 },
    { 0xb0fbf53c, SUC_lexicon_tags_95 },
    { 0x479bf53e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbf542, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x151bf545, SUC_lexicon_tags_23 },
    { 0x413bf546, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x781bf548, SUC_lexicon_tags_25 },
    { 0xa1bbf549, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3bf54c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85bf551, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1bf55b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33dbf561, SUC_lexicon_tags_43 },
    { 0xdbdbf562, SUC_lexicon_tags_26 },
    { 0x443bf563, SUC_lexicon_tags_17 },
    { 0xd5bbf564, SUC_lexicon_tags_12 },
    { 0x8b5bf564, SUC_lexicon_tags_10 },
    { 0xbedbf566, SUC_lexicon_tags_1 },
    { 0x9abbf566, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fbf56c, SUC_lexicon_tags_63 },
    { 0xfbdbf56d, SUC_lexicon_tags_45 },
    { 0xae9bf56e, SUC_lexicon_tags_12 },
    { 0x9b3bf56e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fbf574, SUC_lexicon_tags_11 },
    { 0x5e5bf575, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dbf578, SUC_lexicon_tags_82 },
    { 0xe65bf579, SUC_lexicon_tags_43 },
    { 0xe55bf57a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7dbf57c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bf580, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bbf589, SUC_lexicon_tags_49 },
    { 0x487bf58a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f7bf58c, SUC_lexicon_tags_15 },
    { 0x73fbf58d, SUC_lexicon_tags_1 },
    { 0x909bf58e, SUC_lexicon_tags_19 },
    { 0x2efbf58e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1bf596, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fbf599, SUC_lexicon_tags_17 },
    { 0x26fbf59a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x13fbf59d, SUC_lexicon_tags_42 },
    { 0xce9bf59e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43bf5a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc9fbf5a7, SUC_lexicon_tags_30 },
    { 0x1e3bf5a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cdbf5aa, SUC_lexicon_tags_38 },
    { 0xba9bf5ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17bf5b1, SUC_lexicon_tags_26 },
    { 0xd4bbf5b2, SUC_lexicon_tags_17 },
    { 0x61fbf5b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddbf5b6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5b1bf5b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187bf5bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb57bf5be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbbf5c3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1bf5c6, SUC_lexicon_tags_45 },
    { 0x1d3bf5c7, SUC_lexicon_tags_40 },
    { 0xb1dbf5c8, SUC_lexicon_tags_45 },
    { 0xcc7bf5c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bbf5cd, SUC_lexicon_tags_87 },
    { 0xec7bf5ce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7bf5d1, SUC_lexicon_tags_27 },
    { 0x9d5bf5d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71bf5d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84fbf5d8, SUC_lexicon_tags_1 },
    { 0x775bf5d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bf5df, SUC_lexicon_tags_68 },
    { 0x261bf5df, SUC_lexicon_tags_37 },
    { 0xa05bf5e1, SUC_lexicon_tags_43 },
    { 0x9f7bf5e2, SUC_lexicon_tags_7 },
    { 0xc9bf5df, SUC_lexicon_tags_43 },
    { 0x3e1bf5e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x965bf5e6, SUC_lexicon_tags_17 },
    { 0x499bf5e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbf5ea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbbf5ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efbf5f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323bf5fe, SUC_lexicon_tags_45 },
    { 0x9a5bf5fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x849bf601, SUC_lexicon_tags_11 },
    { 0xfbdbf601, SUC_lexicon_tags_38 },
    { 0xac5bf603, SUC_lexicon_tags_19 },
    { 0x533bf604, SUC_lexicon_tags_11 },
    { 0x7a7bf604, SUC_lexicon_tags_31 },
    { 0x5c3bf601, SUC_lexicon_tags_11 },
    { 0xdc9bf602, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x65bbf609, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1bf60d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf35bf60f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf81bf611, SUC_lexicon_tags_17 },
    { 0xddbf612, SUC_lexicon_tags_216 },
    { 0xe69bf611, SUC_lexicon_tags_25 },
    { 0x903bf613, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x319bf616, SUC_lexicon_tags_46 },
    { 0xef5bf616, SUC_lexicon_tags_11 },
    { 0x175bf617, SUC_lexicon_tags_37 },
    { 0xfcfbf619, SUC_lexicon_tags_12 },
    { 0xfe1bf619, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee1bf61c, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xa6fbf61e, SUC_lexicon_tags_11 },
    { 0x829bf61f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81bf624, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23bbf626, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bbf62a, SUC_lexicon_tags_43 },
    { 0x59bbf62b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadbf630, SUC_lexicon_tags_23 },
    { 0x263bf630, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bf635, SUC_lexicon_tags_21 },
    { 0x93dbf635, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bf63a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891bf640, SUC_lexicon_tags_15 },
    { 0x2e1bf641, SUC_lexicon_tags_37 },
    { 0x4a1bf642, SUC_lexicon_tags_12 },
    { 0x7bfbf643, SUC_lexicon_tags_63 },
    { 0xbdfbf644, SUC_lexicon_tags_12 },
    { 0x151bf641, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5bf649, SUC_lexicon_tags_11 },
    { 0xd89bf64a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433bf64f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4cfbf651, SUC_lexicon_tags_1 },
    { 0x553bf652, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdbf657, SUC_lexicon_tags_11 },
    { 0xc41bf657, SUC_lexicon_tags_16 },
    { 0x297bf659, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325bf65e, SUC_lexicon_tags_1 },
    { 0xc6bbf65f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dfbf661, SUC_lexicon_tags_17 },
    { 0xed5bf662, SUC_lexicon_tags_17 },
    { 0x48bbf662, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdd9bf665, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7bf669, SUC_lexicon_tags_33 },
    { 0x8a7bf669, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69bf66d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x89bf66f, SUC_lexicon_tags_11 },
    { 0x1b7bf66f, SUC_lexicon_tags_25 },
    { 0x89dbf66f, SUC_lexicon_tags_33 },
    { 0xa2dbf66f, SUC_lexicon_tags_23 },
    { 0xe6dbf673, SUC_lexicon_tags_25 },
    { 0x1dbbf670, SUC_lexicon_tags_634 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfbf678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0fbf67a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x105bf67c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3bf680, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1bf685, SUC_lexicon_tags_15 },
    { 0xedbbf685, SUC_lexicon_tags_19 },
    { 0x50fbf685, SUC_lexicon_tags_11 },
    { 0xc5dbf685, SUC_lexicon_tags_47 },
    { 0x747bf689, SUC_lexicon_tags_38 },
    { 0xdabbf689, SUC_lexicon_tags_43 },
    { 0x737bf689, SUC_lexicon_tags_47 },
    { 0x109bf68c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xf79bf68e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bbf691, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fbf695, SUC_lexicon_tags_31 },
    { 0xb09bf695, SUC_lexicon_tags_17 },
    { 0x7b3bf697, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dbf69c, SUC_lexicon_tags_12 },
    { 0xfebbf69d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf8dbf69f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27bf6a3, SUC_lexicon_tags_60 },
    { 0xec5bf6a4, SUC_lexicon_tags_38 },
    { 0x1f1bf6a5, SUC_lexicon_tags_43 },
    { 0xe39bf6a6, SUC_lexicon_tags_37 },
    { 0x191bf6a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab1bf6a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x979bf6ab, SUC_lexicon_tags_6 },
    { 0xbd1bf6ac, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xc2fbf6ae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x753bf6b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadbf6b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dbf6bc, SUC_lexicon_tags_53 },
    { 0xd39bf6bd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdf1bf6bf, SUC_lexicon_tags_26 },
    { 0xaebbf6c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1bf6c3, SUC_lexicon_tags_7 },
    { 0x5d9bf6c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x651bf6c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5bf6ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39bf6ce, SUC_lexicon_tags_7 },
    { 0xe4fbf6cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdbf6d6, SUC_lexicon_tags_2 },
    { 0x705bf6d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39bf6dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5bf6e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bbf6e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55bf6ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbbf6f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5bf6f4, SUC_lexicon_tags_23 },
    { 0x7b5bf6f5, SUC_lexicon_tags_15 },
    { 0xac1bf6f5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21bf6fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcafbf6fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3bf6ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65bf703, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49bf707, SUC_lexicon_tags_27 },
    { 0xc71bf708, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1bf710, SUC_lexicon_tags_37 },
    { 0xa29bf710, SUC_lexicon_tags_8 },
    { 0x30dbf712, SUC_lexicon_tags_17 },
    { 0x371bf713, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567bf71d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99bf722, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81bf725, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x473bf728, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39dbf72a, SUC_lexicon_tags_29 },
    { 0x39dbf72b, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7bf733, SUC_lexicon_tags_11 },
    { 0x30fbf734, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xf21bf736, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623bf741, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x743bf743, SUC_lexicon_tags_69 },
    { 0xbc3bf743, SUC_lexicon_tags_15 },
    { 0x79bbf745, SUC_lexicon_tags_17 },
    { 0x72bbf744, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5bf751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25bf754, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79dbf756, SUC_lexicon_tags_15 },
    { 0xed1bf756, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191bf75c, SUC_lexicon_tags_17 },
    { 0x399bf75d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadbf761, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61bf766, SUC_lexicon_tags_17 },
    { 0x3dfbf767, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfbf76a, SUC_lexicon_tags_2 },
    { 0xd07bf76b, SUC_lexicon_tags_25 },
    { 0xc81bf76c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb9bf76e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdc7bf770, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x115bf773, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c5bf775, SUC_lexicon_tags_29 },
    { 0xdc7bf776, SUC_lexicon_tags_10 },
    { 0x57fbf777, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79bf77a, SUC_lexicon_tags_0 },
    { 0x39bbf77a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5bf780, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35bf782, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141bf786, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbf78c, SUC_lexicon_tags_2 },
    { 0xe65bf78c, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xfe1bf78f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x731bf792, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb1bf794, SUC_lexicon_tags_63 },
    { 0x815bf795, SUC_lexicon_tags_45 },
    { 0x1abbf796, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf09bf798, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55bf79c, SUC_lexicon_tags_112 },
    { 0x1bbbf79d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x491bf79f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1bf7a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87bf7a6, SUC_lexicon_tags_1 },
    { 0xfddbf7a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fbf7ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39bf7af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49bf7b2, SUC_lexicon_tags_66 },
    { 0xfc5bf7b3, SUC_lexicon_tags_13 },
    { 0xeffbf7b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b3bf7b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x85dbf7b9, SUC_lexicon_tags_40 },
    { 0xb01bf7ba, SUC_lexicon_tags_1 },
    { 0x3bfbf7bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x183bf7bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde5bf7bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13bf7c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7bf7c7, SUC_lexicon_tags_9 },
    { 0x55bbf7c8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dbf7cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x211bf7cd, SUC_lexicon_tags_11 },
    { 0xa7bbf7ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7bf7d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bbf7d9, SUC_lexicon_tags_38 },
    { 0x717bf7da, SUC_lexicon_tags_19 },
    { 0x22fbf7da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3bf7df, SUC_lexicon_tags_12 },
    { 0x57bf7df, SUC_lexicon_tags_17 },
    { 0x9b5bf7e1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x23fbf7e3, SUC_lexicon_tags_47 },
    { 0x855bf7e4, SUC_lexicon_tags_37 },
    { 0x7c3bf7e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf61bf7e7, SUC_lexicon_tags_17 },
    { 0x6ddbf7e8, SUC_lexicon_tags_45 },
    { 0x277bf7e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3bf7ed, SUC_lexicon_tags_8 },
    { 0x6a3bf7ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb1fbf7f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa19bf7f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x397bf7f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfbf7f9, SUC_lexicon_tags_25 },
    { 0x76bbf7fa, SUC_lexicon_tags_31 },
    { 0x9dbf7fa, SUC_lexicon_tags_30 },
    { 0xd83bf7fa, SUC_lexicon_tags_112 },
    { 0xb2fbf7fc, SUC_lexicon_tags_47 },
    { 0x32fbf7fe, SUC_lexicon_tags_25 },
    { 0xf51bf7ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd25bf801, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd27bf803, SUC_lexicon_tags_5 },
    { 0xa5fbf804, SUC_lexicon_tags_30 },
    { 0xe93bf804, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfd5bf807, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa5bf809, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe65bf80b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xec9bf80e, SUC_lexicon_tags_12 },
    { 0xdcdbf80f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379bf816, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175bf81a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc11bf81c, SUC_lexicon_tags_25 },
    { 0x1a3bf81c, SUC_lexicon_tags_1 },
    { 0xcfbbf81c, SUC_lexicon_tags_84 },
    { 0x23dbf81c, SUC_lexicon_tags_2 },
    { 0xb8fbf820, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x461bf822, SUC_lexicon_tags_46 },
    { 0xf7dbf823, SUC_lexicon_tags_31 },
    { 0xab3bf823, SUC_lexicon_tags_108 },
    { 0xbb5bf824, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x685bf828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34bbf82a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf77bf82c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633bf831, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95bf83d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x393bf83f, SUC_lexicon_tags_28 },
    { 0xa09bf840, SUC_lexicon_tags_12 },
    { 0x285bf841, SUC_lexicon_tags_37 },
    { 0xcbbf842, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9bf845, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2f7bf847, SUC_lexicon_tags_47 },
    { 0xc7fbf847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbbf853, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f9bf855, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bbf85c, SUC_lexicon_tags_15 },
    { 0x4d9bf85d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1bf860, SUC_lexicon_tags_25 },
    { 0x8c3bf860, SUC_lexicon_tags_80 },
    { 0x3ddbf860, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x969bf865, SUC_lexicon_tags_7 },
    { 0x47bf865, SUC_lexicon_tags_31 },
    { 0xe2dbf867, SUC_lexicon_tags_43 },
    { 0xa7bf868, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5bf871, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x983bf874, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79bf879, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x41fbf87c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dbf881, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17bf885, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bbf889, SUC_lexicon_tags_28 },
    { 0x60dbf88a, SUC_lexicon_tags_27 },
    { 0x2cdbf88b, SUC_lexicon_tags_76 },
    { 0, NULL },
    { 0x51bbf88d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x59dbf890, SUC_lexicon_tags_19 },
    { 0x465bf890, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x651bf894, SUC_lexicon_tags_1 },
    { 0x863bf895, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223bf899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x51bf89c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3bf8a0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65bf8a4, SUC_lexicon_tags_15 },
    { 0x9cdbf8a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59dbf8a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bf8ac, SUC_lexicon_tags_0 },
    { 0xbdfbf8ac, SUC_lexicon_tags_66 },
    { 0x2c5bf8ac, SUC_lexicon_tags_25 },
    { 0x1ebbf8af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x33dbf8b1, SUC_lexicon_tags_2 },
    { 0x5bf8b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1bf8b8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x483bf8bb, SUC_lexicon_tags_19 },
    { 0x6abbf8bb, SUC_lexicon_tags_17 },
    { 0xae1bf8bc, SUC_lexicon_tags_21 },
    { 0x509bf8bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd71bf8c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92fbf8c2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5bf8c7, SUC_lexicon_tags_65 },
    { 0x407bf8c8, SUC_lexicon_tags_2 },
    { 0xc05bf8c9, SUC_lexicon_tags_37 },
    { 0xcd3bf8ca, SUC_lexicon_tags_27 },
    { 0x913bf8ca, SUC_lexicon_tags_31 },
    { 0x3a7bf8cb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x167bf8ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857bf8d4, SUC_lexicon_tags_27 },
    { 0x759bf8d5, SUC_lexicon_tags_7 },
    { 0x5d7bf8d6, SUC_lexicon_tags_14 },
    { 0xc3dbf8d7, SUC_lexicon_tags_47 },
    { 0x555bf8d7, SUC_lexicon_tags_37 },
    { 0xa3bf8d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bbf8e2, SUC_lexicon_tags_0 },
    { 0x227bf8e3, SUC_lexicon_tags_14 },
    { 0x8bfbf8e2, SUC_lexicon_tags_6 },
    { 0x8d9bf8e2, SUC_lexicon_tags_19 },
    { 0x465bf8e6, SUC_lexicon_tags_37 },
    { 0x30fbf8e5, SUC_lexicon_tags_1 },
    { 0xfa3bf8e8, SUC_lexicon_tags_43 },
    { 0xfc7bf8e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x915bf8eb, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0x83bbf8ed, SUC_lexicon_tags_0 },
    { 0xb99bf8ed, SUC_lexicon_tags_30 },
    { 0x8cdbf8ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3d9bf8f1, SUC_lexicon_tags_8 },
    { 0xe99bf8f2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3a1bf8f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3bf8fd, SUC_lexicon_tags_7 },
    { 0x48fbf8fd, SUC_lexicon_tags_6 },
    { 0x1c3bf8ff, SUC_lexicon_tags_7 },
    { 0xc7fbf900, SUC_lexicon_tags_15 },
    { 0xf4bbf8ff, SUC_lexicon_tags_15 },
    { 0x30fbf902, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x49dbf904, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681bf909, SUC_lexicon_tags_38 },
    { 0xe45bf909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53bf90e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbbf912, SUC_lexicon_tags_39 },
    { 0x4c3bf913, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5bf91d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1fbbf91f, SUC_lexicon_tags_47 },
    { 0x201bf920, SUC_lexicon_tags_11 },
    { 0xa2fbf921, SUC_lexicon_tags_1 },
    { 0x46bbf922, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bbf92d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65bf933, SUC_lexicon_tags_2 },
    { 0xb09bf934, SUC_lexicon_tags_11 },
    { 0x5ffbf935, SUC_lexicon_tags_45 },
    { 0x3dfbf935, SUC_lexicon_tags_15 },
    { 0x5c1bf936, SUC_lexicon_tags_48 },
    { 0xc0dbf935, SUC_lexicon_tags_25 },
    { 0xc33bf939, SUC_lexicon_tags_11 },
    { 0x6e9bf93a, SUC_lexicon_tags_14 },
    { 0xe6dbf939, SUC_lexicon_tags_15 },
    { 0xfc1bf93a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2afbf93e, SUC_lexicon_tags_47 },
    { 0x23bbf93e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc19bf941, SUC_lexicon_tags_16 },
    { 0x707bf941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5bf945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459bf949, SUC_lexicon_tags_11 },
    { 0x32dbf949, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbf94d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3bf951, SUC_lexicon_tags_27 },
    { 0xc0fbf952, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x77dbf954, SUC_lexicon_tags_27 },
    { 0x7b5bf955, SUC_lexicon_tags_17 },
    { 0x53fbf955, SUC_lexicon_tags_25 },
    { 0xa45bf956, SUC_lexicon_tags_15 },
    { 0x8fbf958, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5bf95f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5bf964, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1bf968, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bbf96d, SUC_lexicon_tags_8 },
    { 0x137bf96e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddbf972, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bbf976, SUC_lexicon_tags_27 },
    { 0x2b5bf977, SUC_lexicon_tags_37 },
    { 0x177bf977, SUC_lexicon_tags_2 },
    { 0xf7bf977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9dfbf97b, SUC_lexicon_tags_23 },
    { 0x98dbf97b, SUC_lexicon_tags_17 },
    { 0xe4dbf97d, SUC_lexicon_tags_2 },
    { 0x81fbf97d, SUC_lexicon_tags_1 },
    { 0x447bf97f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xde7bf981, SUC_lexicon_tags_11 },
    { 0xd09bf981, SUC_lexicon_tags_0 },
    { 0xe01bf983, SUC_lexicon_tags_46 },
    { 0xfb3bf984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdbf988, SUC_lexicon_tags_173 },
    { 0xa8fbf988, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa1bf98b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe9bf98d, SUC_lexicon_tags_11 },
    { 0x437bf98e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fbf993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19bf99b, SUC_lexicon_tags_11 },
    { 0x35bbf99b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80fbf99e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x23fbf9a1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afbf9a4, SUC_lexicon_tags_29 },
    { 0x5ddbf9a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919bf9ae, SUC_lexicon_tags_25 },
    { 0x6a7bf9ae, SUC_lexicon_tags_33 },
    { 0x277bf9af, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bbf9b3, SUC_lexicon_tags_1 },
    { 0x197bf9b4, SUC_lexicon_tags_12 },
    { 0x28fbf9b3, SUC_lexicon_tags_25 },
    { 0xabbf9b6, SUC_lexicon_tags_37 },
    { 0x405bf9b7, SUC_lexicon_tags_1 },
    { 0xe8bbf9b8, SUC_lexicon_tags_17 },
    { 0xbb5bf9b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501bf9bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25bf9c1, SUC_lexicon_tags_37 },
    { 0x393bf9c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x683bf9c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05bf9ca, SUC_lexicon_tags_17 },
    { 0xfc9bf9cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73bf9d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7bf9d3, SUC_lexicon_tags_1 },
    { 0xd5dbf9d4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x803bf9d7, SUC_lexicon_tags_55 },
    { 0x25bf9d8, SUC_lexicon_tags_60 },
    { 0x8a9bf9d9, SUC_lexicon_tags_49 },
    { 0x557bf9d9, SUC_lexicon_tags_12 },
    { 0xf5bbf9d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa41bf9dd, SUC_lexicon_tags_9 },
    { 0xf2dbf9dd, SUC_lexicon_tags_17 },
    { 0x269bf9dd, SUC_lexicon_tags_15 },
    { 0x6a1bf9dd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17bf9e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x567bf9e7, SUC_lexicon_tags_11 },
    { 0x881bf9e7, SUC_lexicon_tags_63 },
    { 0x8c5bf9e9, SUC_lexicon_tags_29 },
    { 0xee9bf9e9, SUC_lexicon_tags_43 },
    { 0x4ddbf9e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x271bf9ed, SUC_lexicon_tags_47 },
    { 0xe1bf9ed, SUC_lexicon_tags_17 },
    { 0xcefbf9ef, SUC_lexicon_tags_21 },
    { 0x347bf9f0, SUC_lexicon_tags_60 },
    { 0x9d5bf9f1, SUC_lexicon_tags_38 },
    { 0x4fbf9ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcffbf9f4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fbf9fa, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5bf9fd, SUC_lexicon_tags_2 },
    { 0xbd7bf9fe, SUC_lexicon_tags_48 },
    { 0xea5bf9ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967bfa07, SUC_lexicon_tags_31 },
    { 0x3cfbfa07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bbfa0d, SUC_lexicon_tags_60 },
    { 0xbd9bfa0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e7bfa10, SUC_lexicon_tags_43 },
    { 0x733bfa11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2d3bfa13, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa91bfa15, SUC_lexicon_tags_60 },
    { 0x1cbbfa16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403bfa1b, SUC_lexicon_tags_7 },
    { 0x445bfa1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdbfa1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaa1bfa21, SUC_lexicon_tags_7 },
    { 0x203bfa22, SUC_lexicon_tags_37 },
    { 0xca9bfa23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9bfa27, SUC_lexicon_tags_2 },
    { 0x863bfa27, SUC_lexicon_tags_2 },
    { 0xb2bbfa27, SUC_lexicon_tags_65 },
    { 0x127bfa2a, SUC_lexicon_tags_17 },
    { 0x9fbfa2b, SUC_lexicon_tags_7 },
    { 0x19dbfa2c, SUC_lexicon_tags_47 },
    { 0xa7dbfa2d, SUC_lexicon_tags_1 },
    { 0xb13bfa2e, SUC_lexicon_tags_9 },
    { 0x299bfa2b, SUC_lexicon_tags_33 },
    { 0xb13bfa2b, SUC_lexicon_tags_9 },
    { 0xd5bbfa2d, SUC_lexicon_tags_15 },
    { 0xd49bfa2f, SUC_lexicon_tags_0 },
    { 0x615bfa33, SUC_lexicon_tags_24 },
    { 0xf99bfa33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efbfa39, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x305bfa3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdedbfa3e, SUC_lexicon_tags_6 },
    { 0x1cbbfa3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x381bfa41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11bfa47, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9bfa4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923bfa4e, SUC_lexicon_tags_0 },
    { 0x1bdbfa4e, SUC_lexicon_tags_17 },
    { 0xcf1bfa50, SUC_lexicon_tags_15 },
    { 0xad7bfa4e, SUC_lexicon_tags_11 },
    { 0xfbfa52, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x8fdbfa54, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1bfa57, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47bfa5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb31bfa5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927bfa63, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75bfa6b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fbfa71, SUC_lexicon_tags_39 },
    { 0x413bfa72, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcf7bfa74, SUC_lexicon_tags_39 },
    { 0x7d1bfa75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bbfa79, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x9d1bfa7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13bfa7e, SUC_lexicon_tags_1 },
    { 0x7a9bfa7f, SUC_lexicon_tags_2 },
    { 0x271bfa80, SUC_lexicon_tags_17 },
    { 0xd75bfa80, SUC_lexicon_tags_29 },
    { 0x6f5bfa82, SUC_lexicon_tags_29 },
    { 0x621bfa83, SUC_lexicon_tags_17 },
    { 0xbefbfa82, SUC_lexicon_tags_37 },
    { 0xa37bfa82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x53bfa88, SUC_lexicon_tags_7 },
    { 0xa7fbfa89, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x187bfa8c, SUC_lexicon_tags_45 },
    { 0x81bfa8c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfbfa90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11bfa95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41bfa98, SUC_lexicon_tags_23 },
    { 0x649bfa99, SUC_lexicon_tags_37 },
    { 0x9bbfa99, SUC_lexicon_tags_17 },
    { 0x835bfa9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5f3bfa9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x145bfaa0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x223bfaa3, SUC_lexicon_tags_11 },
    { 0x7d9bfaa4, SUC_lexicon_tags_8 },
    { 0x1c9bfaa4, SUC_lexicon_tags_37 },
    { 0x221bfaa5, SUC_lexicon_tags_11 },
    { 0x913bfaa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453bfaae, SUC_lexicon_tags_11 },
    { 0xf37bfaae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431bfab5, SUC_lexicon_tags_60 },
    { 0x8dfbfab6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9bfab9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615bfabd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45bfac1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555bfac5, SUC_lexicon_tags_0 },
    { 0xb9bfac6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafbfacd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677bfad2, SUC_lexicon_tags_15 },
    { 0x667bfad3, SUC_lexicon_tags_28 },
    { 0xe5bfad4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785bfad8, SUC_lexicon_tags_14 },
    { 0x4adbfad9, SUC_lexicon_tags_38 },
    { 0xda9bfada, SUC_lexicon_tags_1 },
    { 0x7dbbfadb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bfadf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319bfae4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fbfaec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1bfaf4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfbfaf7, SUC_lexicon_tags_17 },
    { 0x279bfaf8, SUC_lexicon_tags_11 },
    { 0xb5bbfaf9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695bfb05, SUC_lexicon_tags_84 },
    { 0xd63bfb06, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbbfb13, SUC_lexicon_tags_49 },
    { 0x105bfb13, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xf85bfb16, SUC_lexicon_tags_28 },
    { 0xcb3bfb17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d1bfb19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x45fbfb1c, SUC_lexicon_tags_63 },
    { 0x7afbfb1c, SUC_lexicon_tags_2 },
    { 0xd37bfb1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x413bfb21, SUC_lexicon_tags_12 },
    { 0x385bfb21, SUC_lexicon_tags_1 },
    { 0xdddbfb23, SUC_lexicon_tags_7 },
    { 0x761bfb24, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbfb27, SUC_lexicon_tags_27 },
    { 0x3adbfb27, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bbfb2d, SUC_lexicon_tags_2 },
    { 0xf31bfb2e, SUC_lexicon_tags_1 },
    { 0x1dbbfb2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x357bfb32, SUC_lexicon_tags_18 },
    { 0x9a1bfb33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x327bfb35, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bfb39, SUC_lexicon_tags_11 },
    { 0xc43bfb3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45bfb46, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x641bfb49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1f1bfb4b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683bfb51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7bfb57, SUC_lexicon_tags_17 },
    { 0xe3fbfb57, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x68dbfb5a, SUC_lexicon_tags_2 },
    { 0x8cfbfb5b, SUC_lexicon_tags_14 },
    { 0xe69bfb5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69bfb60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdbfb63, SUC_lexicon_tags_39 },
    { 0xa61bfb63, SUC_lexicon_tags_8 },
    { 0xc01bfb64, SUC_lexicon_tags_17 },
    { 0x393bfb66, SUC_lexicon_tags_11 },
    { 0x957bfb67, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x247bfb6a, SUC_lexicon_tags_2 },
    { 0xbd9bfb6b, SUC_lexicon_tags_11 },
    { 0x3d5bfb6a, SUC_lexicon_tags_25 },
    { 0x693bfb6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x653bfb6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x385bfb71, SUC_lexicon_tags_49 },
    { 0x443bfb72, SUC_lexicon_tags_29 },
    { 0x655bfb73, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc0bbfb75, SUC_lexicon_tags_12 },
    { 0x5ddbfb76, SUC_lexicon_tags_19 },
    { 0x4ffbfb77, SUC_lexicon_tags_7 },
    { 0xfc5bfb77, SUC_lexicon_tags_19 },
    { 0xef9bfb79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109bfb7d, SUC_lexicon_tags_27 },
    { 0x23fbfb7e, SUC_lexicon_tags_7 },
    { 0x82bbfb7d, SUC_lexicon_tags_10 },
    { 0x8afbfb7f, SUC_lexicon_tags_25 },
    { 0x2e3bfb81, SUC_lexicon_tags_45 },
    { 0x767bfb81, SUC_lexicon_tags_7 },
    { 0x5cbbfb7e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bfb8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x899bfb8f, SUC_lexicon_tags_11 },
    { 0x305bfb90, SUC_lexicon_tags_11 },
    { 0x31fbfb90, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbfb99, SUC_lexicon_tags_8 },
    { 0xaf3bfb99, SUC_lexicon_tags_27 },
    { 0xa75bfb9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dbbfb9d, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751bfba2, SUC_lexicon_tags_14 },
    { 0x4f3bfba3, SUC_lexicon_tags_14 },
    { 0x86bbfba3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5fbfba6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x68dbfba8, SUC_lexicon_tags_46 },
    { 0x351bfba9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e5bfbab, SUC_lexicon_tags_11 },
    { 0x85bbfbac, SUC_lexicon_tags_2 },
    { 0xeefbfbab, SUC_lexicon_tags_15 },
    { 0xb2dbfbae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7bbfbb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89bfbb5, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11bfbb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdfbfbbb, SUC_lexicon_tags_1 },
    { 0xbcbbfbbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bbfbc6, SUC_lexicon_tags_19 },
    { 0xfa3bfbc6, SUC_lexicon_tags_15 },
    { 0x197bfbc7, SUC_lexicon_tags_46 },
    { 0x667bfbc9, SUC_lexicon_tags_29 },
    { 0x927bfbca, SUC_lexicon_tags_27 },
    { 0x603bfbc9, SUC_lexicon_tags_418 },
    { 0x845bfbcc, SUC_lexicon_tags_30 },
    { 0x471bfbcd, SUC_lexicon_tags_47 },
    { 0xc1dbfbcd, SUC_lexicon_tags_47 },
    { 0x607bfbcf, SUC_lexicon_tags_25 },
    { 0xca7bfbcc, SUC_lexicon_tags_0 },
    { 0x829bfbd1, SUC_lexicon_tags_17 },
    { 0xf71bfbd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887bfbd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fbfbdc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb87bfbde, SUC_lexicon_tags_55 },
    { 0x665bfbdf, SUC_lexicon_tags_12 },
    { 0x861bfbe0, SUC_lexicon_tags_12 },
    { 0xc43bfbdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fbfbe4, SUC_lexicon_tags_19 },
    { 0x6d9bfbe4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb31bfbe7, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95bfbeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb55bfbed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75bfbf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03bfbf6, SUC_lexicon_tags_25 },
    { 0xe13bfbf7, SUC_lexicon_tags_11 },
    { 0x6afbfbf7, SUC_lexicon_tags_11 },
    { 0xf73bfbf9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe5bfbfb, SUC_lexicon_tags_23 },
    { 0x821bfbfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9bfc00, SUC_lexicon_tags_26 },
    { 0x719bfc01, SUC_lexicon_tags_12 },
    { 0xe29bfc01, SUC_lexicon_tags_87 },
    { 0x3f7bfc03, SUC_lexicon_tags_38 },
    { 0x4cdbfc01, SUC_lexicon_tags_23 },
    { 0x97dbfc05, SUC_lexicon_tags_56 },
    { 0xc57bfc06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47bfc0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dbfc10, SUC_lexicon_tags_7 },
    { 0xe31bfc11, SUC_lexicon_tags_15 },
    { 0x7c5bfc12, SUC_lexicon_tags_11 },
    { 0x495bfc12, SUC_lexicon_tags_17 },
    { 0x233bfc13, SUC_lexicon_tags_56 },
    { 0x375bfc10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2fbfc17, SUC_lexicon_tags_38 },
    { 0xf51bfc18, SUC_lexicon_tags_37 },
    { 0x951bfc18, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x13fbfc1c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbfc22, SUC_lexicon_tags_1 },
    { 0x349bfc22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01bfc27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3bfc2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15bfc33, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x80dbfc36, SUC_lexicon_tags_6 },
    { 0x719bfc37, SUC_lexicon_tags_6 },
    { 0x7ddbfc38, SUC_lexicon_tags_29 },
    { 0x31bfc39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x153bfc3c, SUC_lexicon_tags_30 },
    { 0x6fbbfc3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x785bfc3f, SUC_lexicon_tags_45 },
    { 0x7dbbfc3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9bfc43, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x753bfc45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x761bfc47, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2d7bfc49, SUC_lexicon_tags_243 },
    { 0x507bfc49, SUC_lexicon_tags_47 },
    { 0xc95bfc49, SUC_lexicon_tags_1 },
    { 0xc49bfc4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229bfc50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9bfc59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69bfc60, SUC_lexicon_tags_31 },
    { 0xf59bfc60, SUC_lexicon_tags_15 },
    { 0xe87bfc62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37bfc66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa63bfc68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbcfbfc6a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfc76, SUC_lexicon_tags_28 },
    { 0x35bfc77, SUC_lexicon_tags_53 },
    { 0x559bfc77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ffbfc7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7bfc7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffbfc82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7dbbfc84, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6d1bfc86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c3bfc88, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafbfc8b, SUC_lexicon_tags_124 },
    { 0x571bfc8c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8adbfc8e, SUC_lexicon_tags_79 },
    { 0xc33bfc8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31bfc93, SUC_lexicon_tags_4 },
    { 0xe55bfc94, SUC_lexicon_tags_7 },
    { 0xa49bfc95, SUC_lexicon_tags_37 },
    { 0xbe9bfc96, SUC_lexicon_tags_25 },
    { 0x94fbfc97, SUC_lexicon_tags_6 },
    { 0xff9bfc98, SUC_lexicon_tags_35 },
    { 0x585bfc99, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659bfc9d, SUC_lexicon_tags_25 },
    { 0xb07bfc9d, SUC_lexicon_tags_12 },
    { 0xc55bfc9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd5bfca1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fbfca7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167bfcad, SUC_lexicon_tags_35 },
    { 0xa4bbfcae, SUC_lexicon_tags_15 },
    { 0xe55bfcae, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fbfcb5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dbfcbd, SUC_lexicon_tags_7 },
    { 0xe27bfcbd, SUC_lexicon_tags_522 },
    { 0xd0dbfcbe, SUC_lexicon_tags_12 },
    { 0xd33bfcc0, SUC_lexicon_tags_12 },
    { 0xe29bfcc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x849bfcc3, SUC_lexicon_tags_27 },
    { 0x9e3bfcc4, SUC_lexicon_tags_15 },
    { 0xe7dbfcc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fbfccb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51bfccf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3bfcd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5bfcda, SUC_lexicon_tags_25 },
    { 0xb91bfcdb, SUC_lexicon_tags_38 },
    { 0x415bfcdb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159bfce2, SUC_lexicon_tags_15 },
    { 0x5ebbfce3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7bfce8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155bfcec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769bfcf2, SUC_lexicon_tags_2 },
    { 0x9c1bfcf3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72bbfcf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x767bfcfa, SUC_lexicon_tags_30 },
    { 0x6b9bfcfb, SUC_lexicon_tags_28 },
    { 0xb4bbfcfc, SUC_lexicon_tags_11 },
    { 0xd8bbfcfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33bfd00, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfd03, SUC_lexicon_tags_9 },
    { 0xaf1bfd04, SUC_lexicon_tags_37 },
    { 0xb69bfd05, SUC_lexicon_tags_11 },
    { 0x28fbfd05, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x503bfd08, SUC_lexicon_tags_25 },
    { 0xf79bfd09, SUC_lexicon_tags_60 },
    { 0x691bfd09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x485bfd0c, SUC_lexicon_tags_12 },
    { 0x6bdbfd0c, SUC_lexicon_tags_37 },
    { 0xf1bfd0d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf85bfd10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c9bfd12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffbfd17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58bbfd19, SUC_lexicon_tags_38 },
    { 0x819bfd1a, SUC_lexicon_tags_19 },
    { 0xacbbfd1b, SUC_lexicon_tags_38 },
    { 0x5ebbfd1a, SUC_lexicon_tags_30 },
    { 0xf99bfd1c, SUC_lexicon_tags_58 },
    { 0xf01bfd1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9bfd23, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9bfd2a, SUC_lexicon_tags_12 },
    { 0xf17bfd2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8abbfd2d, SUC_lexicon_tags_38 },
    { 0xfcbbfd2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x70fbfd30, SUC_lexicon_tags_12 },
    { 0xc91bfd31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x47bbfd33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdbfd36, SUC_lexicon_tags_87 },
    { 0x7f5bfd37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd1bfd39, SUC_lexicon_tags_17 },
    { 0x1bfd39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dbfd3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9bfd49, SUC_lexicon_tags_7 },
    { 0x33dbfd4a, SUC_lexicon_tags_0 },
    { 0x65bbfd4b, SUC_lexicon_tags_37 },
    { 0x48bbfd4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf25bfd4e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03bfd51, SUC_lexicon_tags_9 },
    { 0x7dfbfd52, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43bfd57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe55bfd59, SUC_lexicon_tags_11 },
    { 0x487bfd5a, SUC_lexicon_tags_65 },
    { 0xa9fbfd59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbfd5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abbfd64, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63bfd6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bfd6d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddbfd72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267bfd76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bbfd7a, SUC_lexicon_tags_33 },
    { 0x32bbfd7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x745bfd7d, SUC_lexicon_tags_7 },
    { 0x7cdbfd7e, SUC_lexicon_tags_23 },
    { 0x24fbfd7e, SUC_lexicon_tags_17 },
    { 0xf3bbfd80, SUC_lexicon_tags_14 },
    { 0x3afbfd81, SUC_lexicon_tags_21 },
    { 0xa61bfd82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503bfd89, SUC_lexicon_tags_57 },
    { 0x209bfd8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bfd8d, SUC_lexicon_tags_8 },
    { 0x809bfd8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a1bfd90, SUC_lexicon_tags_27 },
    { 0xa1fbfd91, SUC_lexicon_tags_9 },
    { 0x3bbfd92, SUC_lexicon_tags_47 },
    { 0x2a7bfd91, SUC_lexicon_tags_2 },
    { 0x813bfd91, SUC_lexicon_tags_1 },
    { 0xab3bfd95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x745bfd98, SUC_lexicon_tags_15 },
    { 0xdafbfd99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d3bfd9b, SUC_lexicon_tags_37 },
    { 0x6a1bfd9b, SUC_lexicon_tags_39 },
    { 0x103bfd9d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf17bfd9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bbfda3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa07bfda5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x851bfda8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbefbfdaa, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0xfa3bfdac, SUC_lexicon_tags_11 },
    { 0x25fbfdad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5bfdb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19bfdb6, SUC_lexicon_tags_68 },
    { 0x421bfdb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137bfdbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85bfdc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5a7bfdc2, SUC_lexicon_tags_37 },
    { 0xdcbbfdc3, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9bfdcc, SUC_lexicon_tags_7 },
    { 0xd6bbfdcd, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63dbfdd8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73bfde2, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xb6fbfde4, SUC_lexicon_tags_7 },
    { 0x15fbfde5, SUC_lexicon_tags_45 },
    { 0x82bbfde6, SUC_lexicon_tags_18 },
    { 0xd61bfde6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59bfdf1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621bfdf5, SUC_lexicon_tags_9 },
    { 0xd65bfdf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf7bfdf8, SUC_lexicon_tags_15 },
    { 0xf9bfdf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5bfdfc, SUC_lexicon_tags_7 },
    { 0x817bfdfd, SUC_lexicon_tags_12 },
    { 0x2bbbfdfe, SUC_lexicon_tags_9 },
    { 0x87dbfdfe, SUC_lexicon_tags_63 },
    { 0x647bfdfe, SUC_lexicon_tags_29 },
    { 0x70bbfdff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x605bfe04, SUC_lexicon_tags_12 },
    { 0x3e9bfe04, SUC_lexicon_tags_11 },
    { 0xdb9bfe06, SUC_lexicon_tags_12 },
    { 0x927bfe06, SUC_lexicon_tags_7 },
    { 0x9c5bfe06, SUC_lexicon_tags_57 },
    { 0x8b1bfe09, SUC_lexicon_tags_32 },
    { 0xe51bfe0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9bfe0d, SUC_lexicon_tags_11 },
    { 0xd5bfe0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67bfe1f, SUC_lexicon_tags_49 },
    { 0x255bfe1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753bfe24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x531bfe26, SUC_lexicon_tags_45 },
    { 0xb6bbfe26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5bfe2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb99bfe2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89dbfe32, SUC_lexicon_tags_66 },
    { 0xb2bbfe33, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x41fbfe36, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb41bfe38, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65bfe40, SUC_lexicon_tags_47 },
    { 0xdf1bfe41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627bfe47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5dbfe49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bfe4d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x2bbfe4f, SUC_lexicon_tags_37 },
    { 0xfa3bfe50, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11bfe55, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5bfe5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1bfe60, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fbfe68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9bfe6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbfe6f, SUC_lexicon_tags_12 },
    { 0x917bfe70, SUC_lexicon_tags_56 },
    { 0x56bbfe71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59bfe75, SUC_lexicon_tags_25 },
    { 0x3b5bfe76, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67bfe82, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3bfe8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31fbfe8d, SUC_lexicon_tags_13 },
    { 0xa9bfe8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751bfe93, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29bfe98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbfe9b, SUC_lexicon_tags_8 },
    { 0x599bfe9b, SUC_lexicon_tags_37 },
    { 0x64fbfe9c, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5bfea0, SUC_lexicon_tags_7 },
    { 0x135bfea1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebbfea5, SUC_lexicon_tags_47 },
    { 0xb93bfea6, SUC_lexicon_tags_12 },
    { 0x725bfea7, SUC_lexicon_tags_25 },
    { 0xd9bfea8, SUC_lexicon_tags_17 },
    { 0xbbfea5, SUC_lexicon_tags_12 },
    { 0x9a5bfeaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfbfead, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fbfeb0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6fbbfeb2, SUC_lexicon_tags_9 },
    { 0x645bfeb3, SUC_lexicon_tags_1 },
    { 0x521bfeb4, SUC_lexicon_tags_65 },
    { 0xec7bfeb5, SUC_lexicon_tags_19 },
    { 0x19bbfeb3, SUC_lexicon_tags_15 },
    { 0x373bfeb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58dbfeb9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3bfebe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb9dbfec0, SUC_lexicon_tags_10 },
    { 0x1b3bfec0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3bfec5, SUC_lexicon_tags_37 },
    { 0x97bbfec5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x709bfec9, SUC_lexicon_tags_3 },
    { 0x109bfec9, SUC_lexicon_tags_27 },
    { 0xb01bfeca, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639bfecf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945bfed4, SUC_lexicon_tags_61 },
    { 0x8f1bfed5, SUC_lexicon_tags_87 },
    { 0xaefbfed5, SUC_lexicon_tags_12 },
    { 0x12fbfed6, SUC_lexicon_tags_87 },
    { 0x6cdbfed8, SUC_lexicon_tags_47 },
    { 0x9b5bfed8, SUC_lexicon_tags_56 },
    { 0xa5dbfeda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5bfedd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x513bfedf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfbfee5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fbfee9, SUC_lexicon_tags_25 },
    { 0x97dbfeea, SUC_lexicon_tags_43 },
    { 0x80bbfeeb, SUC_lexicon_tags_23 },
    { 0x6f1bfeec, SUC_lexicon_tags_7 },
    { 0xe67bfeed, SUC_lexicon_tags_7 },
    { 0x9ddbfeea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62bbfef0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dbfef4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49bff03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x32dbff05, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x86fbff08, SUC_lexicon_tags_48 },
    { 0xb7dbff09, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657bff0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49bff11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x37bbff14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07bff19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489bff1e, SUC_lexicon_tags_0 },
    { 0xfb5bff1f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dbff23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7bff26, SUC_lexicon_tags_31 },
    { 0x2d7bff27, SUC_lexicon_tags_8 },
    { 0x73dbff26, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dbff2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x771bff31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7bff33, SUC_lexicon_tags_19 },
    { 0xe35bff33, SUC_lexicon_tags_15 },
    { 0xc3bbff35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299bff3f, SUC_lexicon_tags_17 },
    { 0x2c9bff40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7bff44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5bff4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbff4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983bff54, SUC_lexicon_tags_49 },
    { 0x15fbff54, SUC_lexicon_tags_7 },
    { 0x79fbff54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bbff5a, SUC_lexicon_tags_38 },
    { 0xb91bff5b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05bff5e, SUC_lexicon_tags_1 },
    { 0x315bff5f, SUC_lexicon_tags_24 },
    { 0xa4fbff5f, SUC_lexicon_tags_63 },
    { 0x989bff61, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1bff67, SUC_lexicon_tags_83 },
    { 0xc91bff67, SUC_lexicon_tags_37 },
    { 0xd0fbff67, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bff6d, SUC_lexicon_tags_56 },
    { 0x683bff6e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x691bff70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x10fbff72, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bbff75, SUC_lexicon_tags_27 },
    { 0xf83bff76, SUC_lexicon_tags_23 },
    { 0x8b9bff77, SUC_lexicon_tags_11 },
    { 0x635bff77, SUC_lexicon_tags_23 },
    { 0xda5bff77, SUC_lexicon_tags_46 },
    { 0xf77bff75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bbbff7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x273bff7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x935bff81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877bff85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9bff8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9fbff8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5bff92, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x16dbff94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x96dbff97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abbff9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dbff9e, SUC_lexicon_tags_7 },
    { 0xb79bff9f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1bffa2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa63bffa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7bffa7, SUC_lexicon_tags_25 },
    { 0x6b7bffa8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3bffad, SUC_lexicon_tags_1 },
    { 0x9a7bffad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x227bffb0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x60dbffb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07bffb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfbffba, SUC_lexicon_tags_449 },
    { 0x4e1bffbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x511bffbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bffbf, SUC_lexicon_tags_30 },
    { 0xad7bffbf, SUC_lexicon_tags_86 },
    { 0xb71bffc0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adbffc6, SUC_lexicon_tags_16 },
    { 0x435bffc7, SUC_lexicon_tags_27 },
    { 0x1bbbffc7, SUC_lexicon_tags_87 },
    { 0xc35bffc9, SUC_lexicon_tags_37 },
    { 0x90fbffc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3bffcf, SUC_lexicon_tags_66 },
    { 0x4a1bffd0, SUC_lexicon_tags_25 },
    { 0x8bdbffd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6afbffd3, SUC_lexicon_tags_16 },
    { 0x8efbffd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbbffd7, SUC_lexicon_tags_21 },
    { 0xe67bffd8, SUC_lexicon_tags_17 },
    { 0xa25bffd9, SUC_lexicon_tags_15 },
    { 0x913bffd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd45bffdc, SUC_lexicon_tags_33 },
    { 0x2afbffdd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x623bffe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x357bffe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683bffed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97fbffef, SUC_lexicon_tags_11 },
    { 0x67bffef, SUC_lexicon_tags_17 },
    { 0xb41bfff1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8dbfff3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa9bbfff5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fbbfff7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x813bfffa, SUC_lexicon_tags_35 },
    { 0x235bfffb, SUC_lexicon_tags_71 },
    { 0x29bbfffb, SUC_lexicon_tags_63 },
    { 0x63dbfffb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abc0000, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc0003, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x891c0006, SUC_lexicon_tags_61 },
    { 0xbf1c0007, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc1bc0009, SUC_lexicon_tags_2 },
    { 0xfb5c0009, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebc0012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe25c0014, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x563c0016, SUC_lexicon_tags_17 },
    { 0xa1fc0017, SUC_lexicon_tags_27 },
    { 0x9a1c0016, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebc001e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fc0021, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x953c0023, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243c0029, SUC_lexicon_tags_11 },
    { 0xbffc002a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7c002d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dc0032, SUC_lexicon_tags_38 },
    { 0x21bc0032, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567c0038, SUC_lexicon_tags_27 },
    { 0xcbc0039, SUC_lexicon_tags_7 },
    { 0x781c003a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4bfc003c, SUC_lexicon_tags_15 },
    { 0x1dfc003d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee1c003f, SUC_lexicon_tags_46 },
    { 0x749c0040, SUC_lexicon_tags_15 },
    { 0x5ebc003f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xde5c0043, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655c0049, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75c004c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1c0053, SUC_lexicon_tags_28 },
    { 0x161c0054, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x95fc0056, SUC_lexicon_tags_46 },
    { 0xccfc0057, SUC_lexicon_tags_27 },
    { 0x2dfc0058, SUC_lexicon_tags_11 },
    { 0x209c0057, SUC_lexicon_tags_30 },
    { 0x5c7c005a, SUC_lexicon_tags_1 },
    { 0xbe9c005b, SUC_lexicon_tags_63 },
    { 0x35fc005c, SUC_lexicon_tags_15 },
    { 0xbc3c005b, SUC_lexicon_tags_11 },
    { 0xb1bc005d, SUC_lexicon_tags_25 },
    { 0x4cdc005d, SUC_lexicon_tags_82 },
    { 0x7c9c005b, SUC_lexicon_tags_12 },
    { 0x947c0060, SUC_lexicon_tags_2 },
    { 0xa83c005e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d3c0064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc006a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x449c006c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bc0070, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717c007a, SUC_lexicon_tags_25 },
    { 0x553c007b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33c007f, SUC_lexicon_tags_29 },
    { 0x1c7c0080, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f3c0082, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf63c0084, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x23fc0086, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5c008b, SUC_lexicon_tags_28 },
    { 0x4b1c008c, SUC_lexicon_tags_25 },
    { 0x971c008d, SUC_lexicon_tags_37 },
    { 0x811c008e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9c0091, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4b7c0093, SUC_lexicon_tags_2 },
    { 0x42bc0094, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95c009a, SUC_lexicon_tags_16 },
    { 0x147c009b, SUC_lexicon_tags_17 },
    { 0x25c009c, SUC_lexicon_tags_32 },
    { 0x6a5c009d, SUC_lexicon_tags_37 },
    { 0x1d5c009a, SUC_lexicon_tags_34 },
    { 0x7e5c009e, SUC_lexicon_tags_29 },
    { 0x4e1c009b, SUC_lexicon_tags_84 },
    { 0x3dc00a1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317c00a6, SUC_lexicon_tags_19 },
    { 0xd97c00a7, SUC_lexicon_tags_38 },
    { 0x265c00a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5c00ad, SUC_lexicon_tags_38 },
    { 0xaa9c00ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x829c00b0, SUC_lexicon_tags_11 },
    { 0xe21c00b1, SUC_lexicon_tags_66 },
    { 0xee7c00b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9c00b7, SUC_lexicon_tags_38 },
    { 0x949c00b8, SUC_lexicon_tags_25 },
    { 0x84bc00b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfd5c00bb, SUC_lexicon_tags_108 },
    { 0x5c3c00bb, SUC_lexicon_tags_17 },
    { 0x2f5c00bd, SUC_lexicon_tags_17 },
    { 0xbd9c00be, SUC_lexicon_tags_12 },
    { 0x541c00bf, SUC_lexicon_tags_43 },
    { 0x855c00bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bc00c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c1c00c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321c00ca, SUC_lexicon_tags_25 },
    { 0xd13c00cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x941c00ce, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fc00d2, SUC_lexicon_tags_37 },
    { 0x337c00d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x107c00d6, SUC_lexicon_tags_45 },
    { 0x25dc00d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323c00df, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x67dc00e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61c00e5, SUC_lexicon_tags_11 },
    { 0xd53c00e6, SUC_lexicon_tags_14 },
    { 0xa49c00e7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01c00ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1c00ef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971c00fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x165c00fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddc0101, SUC_lexicon_tags_17 },
    { 0x369c0102, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2afc0105, SUC_lexicon_tags_11 },
    { 0x59c0105, SUC_lexicon_tags_17 },
    { 0x1ebc0106, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x437c0109, SUC_lexicon_tags_15 },
    { 0xb75c0109, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa11c010c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x491c010f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7c0112, SUC_lexicon_tags_1 },
    { 0xf11c0113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc13c0115, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57c011d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637c0127, SUC_lexicon_tags_17 },
    { 0xdebc0128, SUC_lexicon_tags_11 },
    { 0xcefc0129, SUC_lexicon_tags_15 },
    { 0xdddc0128, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9c012d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a1c012f, SUC_lexicon_tags_47 },
    { 0x60bc0130, SUC_lexicon_tags_38 },
    { 0x43dc012f, SUC_lexicon_tags_17 },
    { 0xfffc0132, SUC_lexicon_tags_3 },
    { 0x7f5c0133, SUC_lexicon_tags_32 },
    { 0xb95c0134, SUC_lexicon_tags_8 },
    { 0x6dc0135, SUC_lexicon_tags_9 },
    { 0x9f9c0131, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfc013f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd59c0141, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x173c0144, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01c0149, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e3c014b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5c0158, SUC_lexicon_tags_55 },
    { 0x4b1c0159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe99c015b, SUC_lexicon_tags_14 },
    { 0xde7c015b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc0163, SUC_lexicon_tags_56 },
    { 0xf01c0164, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x193c0166, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x277c0168, SUC_lexicon_tags_31 },
    { 0x933c0168, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x945c016b, SUC_lexicon_tags_37 },
    { 0x319c016c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefc0179, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dc0182, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3bfc0184, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc0189, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd07c018b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe8fc018d, SUC_lexicon_tags_47 },
    { 0x3b5c018d, SUC_lexicon_tags_12 },
    { 0x2fdc018f, SUC_lexicon_tags_11 },
    { 0xc67c018d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5c0193, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x45fc0196, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51c019a, SUC_lexicon_tags_37 },
    { 0x387c019b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc01a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47fc01a4, SUC_lexicon_tags_63 },
    { 0xcabc01a4, SUC_lexicon_tags_15 },
    { 0xc31c01a6, SUC_lexicon_tags_33 },
    { 0xc75c01a5, SUC_lexicon_tags_11 },
    { 0xb6dc01a8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7c01ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5c01ae, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5c01b4, SUC_lexicon_tags_1 },
    { 0x3f7c01b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979c01b9, SUC_lexicon_tags_6 },
    { 0x2abc01ba, SUC_lexicon_tags_9 },
    { 0xac1c01bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefc01c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x82dc01c5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751c01cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc01cf, SUC_lexicon_tags_87 },
    { 0x1dbc01d0, SUC_lexicon_tags_2 },
    { 0x6e9c01d1, SUC_lexicon_tags_0 },
    { 0x5a1c01d2, SUC_lexicon_tags_2 },
    { 0xe35c01d2, SUC_lexicon_tags_66 },
    { 0xb19c01d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317c01d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc01db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x319c01de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2dbc01e0, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5c01e4, SUC_lexicon_tags_37 },
    { 0x7dfc01e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9c01e8, SUC_lexicon_tags_4 },
    { 0x671c01e8, SUC_lexicon_tags_30 },
    { 0x703c01e9, SUC_lexicon_tags_11 },
    { 0xe83c01eb, SUC_lexicon_tags_77 },
    { 0xb25c01ec, SUC_lexicon_tags_37 },
    { 0x91bc01e9, SUC_lexicon_tags_4 },
    { 0x6cdc01ed, SUC_lexicon_tags_17 },
    { 0x6dc01ef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01c01f2, SUC_lexicon_tags_48 },
    { 0xc01c01f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe6bc01f5, SUC_lexicon_tags_15 },
    { 0xc33c01f6, SUC_lexicon_tags_21 },
    { 0x807c01f5, SUC_lexicon_tags_47 },
    { 0xa01c01f8, SUC_lexicon_tags_48 },
    { 0xd33c01f9, SUC_lexicon_tags_14 },
    { 0x89bc01f9, SUC_lexicon_tags_69 },
    { 0x4e7c01fa, SUC_lexicon_tags_12 },
    { 0x3efc01fc, SUC_lexicon_tags_2 },
    { 0xca9c01fd, SUC_lexicon_tags_29 },
    { 0x683c01f8, SUC_lexicon_tags_2 },
    { 0x5b9c01ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1c0206, SUC_lexicon_tags_47 },
    { 0xc3fc0207, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283c020d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81c0210, SUC_lexicon_tags_17 },
    { 0x7bfc0210, SUC_lexicon_tags_18 },
    { 0xd9fc0210, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53dc0214, SUC_lexicon_tags_536 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3c0217, SUC_lexicon_tags_26 },
    { 0x877c0218, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edc021d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3c0220, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x589c0223, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x281c0225, SUC_lexicon_tags_7 },
    { 0x22fc0225, SUC_lexicon_tags_37 },
    { 0xfc9c0225, SUC_lexicon_tags_57 },
    { 0x867c0225, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9f1c022a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8bdc022c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x235c022f, SUC_lexicon_tags_30 },
    { 0x409c0230, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1c0233, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x263c0235, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x571c0237, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbc023b, SUC_lexicon_tags_27 },
    { 0x9fdc023c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7fc023e, SUC_lexicon_tags_17 },
    { 0x3efc023f, SUC_lexicon_tags_38 },
    { 0xe15c0240, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3f7c0242, SUC_lexicon_tags_2 },
    { 0x1f9c0243, SUC_lexicon_tags_7 },
    { 0x91bc0243, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7c0249, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b7c024b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9c024e, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xd45c0250, SUC_lexicon_tags_7 },
    { 0x93bc0251, SUC_lexicon_tags_63 },
    { 0x2fdc0250, SUC_lexicon_tags_19 },
    { 0x65fc0251, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dc0259, SUC_lexicon_tags_12 },
    { 0x6ebc025a, SUC_lexicon_tags_62 },
    { 0xb0dc025a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525c026b, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929c026f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fc0274, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x453c0277, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681c027b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfc0284, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bc0287, SUC_lexicon_tags_6 },
    { 0x10bc0288, SUC_lexicon_tags_11 },
    { 0x73bc0289, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7c028c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587c0293, SUC_lexicon_tags_23 },
    { 0x351c0294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bc0299, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbc02a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bc02a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7edc02aa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1dfc02ac, SUC_lexicon_tags_10 },
    { 0x351c02ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe3c02af, SUC_lexicon_tags_11 },
    { 0x127c02b0, SUC_lexicon_tags_17 },
    { 0x81bc02b1, SUC_lexicon_tags_25 },
    { 0xa71c02b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151c02b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2dbc02b9, SUC_lexicon_tags_45 },
    { 0x387c02b9, SUC_lexicon_tags_4 },
    { 0xcb1c02ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x331c02be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf7c02c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59dc02c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9c02c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541c02cd, SUC_lexicon_tags_37 },
    { 0x30fc02cd, SUC_lexicon_tags_61 },
    { 0xb19c02cf, SUC_lexicon_tags_65 },
    { 0x79bc02ce, SUC_lexicon_tags_6 },
    { 0xff9c02d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc39c02d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3c02dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9c02e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c02e7, SUC_lexicon_tags_47 },
    { 0xb8bc02e8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb3fc02ea, SUC_lexicon_tags_7 },
    { 0x193c02eb, SUC_lexicon_tags_10 },
    { 0x783c02ec, SUC_lexicon_tags_27 },
    { 0xe6fc02ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bc02f1, SUC_lexicon_tags_25 },
    { 0xbf9c02f1, SUC_lexicon_tags_30 },
    { 0x475c02f3, SUC_lexicon_tags_17 },
    { 0x153c02f4, SUC_lexicon_tags_2 },
    { 0x8bc02f5, SUC_lexicon_tags_0 },
    { 0xac5c02f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c0300, SUC_lexicon_tags_63 },
    { 0x721c0300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2efc0304, SUC_lexicon_tags_45 },
    { 0x4cdc0305, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5c0308, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1c030c, SUC_lexicon_tags_25 },
    { 0xf2bc030c, SUC_lexicon_tags_3 },
    { 0x2cfc030e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c0311, SUC_lexicon_tags_14 },
    { 0x249c0312, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c0319, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdc9c031b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xdc1c031d, SUC_lexicon_tags_37 },
    { 0xbc7c031d, SUC_lexicon_tags_23 },
    { 0x959c031d, SUC_lexicon_tags_66 },
    { 0x265c0320, SUC_lexicon_tags_25 },
    { 0x61dc031e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd7c0323, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc09c0325, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc9c0327, SUC_lexicon_tags_30 },
    { 0x93bc0327, SUC_lexicon_tags_68 },
    { 0xbf7c0328, SUC_lexicon_tags_17 },
    { 0x92dc0327, SUC_lexicon_tags_43 },
    { 0xd07c032b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffc0333, SUC_lexicon_tags_0 },
    { 0x6e9c0334, SUC_lexicon_tags_25 },
    { 0xd0fc0335, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197c033a, SUC_lexicon_tags_11 },
    { 0xaefc033b, SUC_lexicon_tags_1 },
    { 0x417c033a, SUC_lexicon_tags_11 },
    { 0xdbbc033b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x201c033f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329c0348, SUC_lexicon_tags_1 },
    { 0x3dc0348, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb9c034b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x597c034d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c0352, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x591c0355, SUC_lexicon_tags_65 },
    { 0x763c0355, SUC_lexicon_tags_7 },
    { 0xb6bc0356, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x25dc0359, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdd7c035b, SUC_lexicon_tags_25 },
    { 0x91fc035b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfa3c035e, SUC_lexicon_tags_17 },
    { 0xd29c035e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbc0365, SUC_lexicon_tags_0 },
    { 0xac7c0366, SUC_lexicon_tags_37 },
    { 0xc8bc0365, SUC_lexicon_tags_10 },
    { 0x157c0368, SUC_lexicon_tags_25 },
    { 0xfe3c0369, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dc036d, SUC_lexicon_tags_37 },
    { 0x25fc036e, SUC_lexicon_tags_17 },
    { 0xb8bc036e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49c0372, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd49c0374, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bbc0376, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbc037f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739c0389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dc038c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bc0390, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dc0395, SUC_lexicon_tags_3 },
    { 0x229c0396, SUC_lexicon_tags_28 },
    { 0x2afc0397, SUC_lexicon_tags_1 },
    { 0x661c0398, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57c039b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343c03a7, SUC_lexicon_tags_17 },
    { 0xaddc03a8, SUC_lexicon_tags_47 },
    { 0xa79c03a7, SUC_lexicon_tags_1 },
    { 0x417c03a7, SUC_lexicon_tags_75 },
    { 0x359c03ab, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa63c03ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641c03b1, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0, NULL },
    { 0x929c03b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd15c03b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc03ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fc03c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd45c03c2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xadbc03c4, SUC_lexicon_tags_49 },
    { 0xf6fc03c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23c03cb, SUC_lexicon_tags_63 },
    { 0x371c03cb, SUC_lexicon_tags_5 },
    { 0x48dc03cd, SUC_lexicon_tags_25 },
    { 0x979c03cd, SUC_lexicon_tags_2 },
    { 0xdd7c03cf, SUC_lexicon_tags_37 },
    { 0xf0bc03cf, SUC_lexicon_tags_14 },
    { 0x697c03cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bc03d5, SUC_lexicon_tags_11 },
    { 0x531c03d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03c03db, SUC_lexicon_tags_84 },
    { 0x347c03dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dc03e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x82fc03e2, SUC_lexicon_tags_77 },
    { 0xd45c03e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95c03e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917c03ec, SUC_lexicon_tags_3 },
    { 0xb99c03ed, SUC_lexicon_tags_17 },
    { 0x4c9c03ee, SUC_lexicon_tags_32 },
    { 0x6ebc03ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fc03f3, SUC_lexicon_tags_112 },
    { 0xe0fc03f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3c03f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fc0401, SUC_lexicon_tags_2 },
    { 0x989c0402, SUC_lexicon_tags_34 },
    { 0x9cbc0402, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebc0406, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xba9c0408, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91c040c, SUC_lexicon_tags_7 },
    { 0xa8fc040c, SUC_lexicon_tags_23 },
    { 0x83dc040e, SUC_lexicon_tags_8 },
    { 0x427c040c, SUC_lexicon_tags_2 },
    { 0xba9c0410, SUC_lexicon_tags_7 },
    { 0xd9c0411, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c0418, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c041d, SUC_lexicon_tags_15 },
    { 0xc91c041e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fc0421, SUC_lexicon_tags_17 },
    { 0x231c0422, SUC_lexicon_tags_719 },
    { 0xe05c0423, SUC_lexicon_tags_17 },
    { 0xa37c0424, SUC_lexicon_tags_1 },
    { 0x221c0424, SUC_lexicon_tags_15 },
    { 0xd65c0426, SUC_lexicon_tags_63 },
    { 0xa19c0427, SUC_lexicon_tags_10 },
    { 0x961c0428, SUC_lexicon_tags_16 },
    { 0x8f1c0429, SUC_lexicon_tags_57 },
    { 0xe03c0428, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71c042d, SUC_lexicon_tags_2 },
    { 0xebbc042e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edc0433, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109c0437, SUC_lexicon_tags_31 },
    { 0x3c5c0438, SUC_lexicon_tags_15 },
    { 0xa8bc0437, SUC_lexicon_tags_15 },
    { 0x73c043a, SUC_lexicon_tags_27 },
    { 0x97dc043a, SUC_lexicon_tags_37 },
    { 0x477c043c, SUC_lexicon_tags_45 },
    { 0x5bbc043d, SUC_lexicon_tags_7 },
    { 0x691c043d, SUC_lexicon_tags_15 },
    { 0xac1c0439, SUC_lexicon_tags_46 },
    { 0x2abc0440, SUC_lexicon_tags_17 },
    { 0x4b3c0440, SUC_lexicon_tags_12 },
    { 0x639c0442, SUC_lexicon_tags_11 },
    { 0x4d3c043e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x17bc0446, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3c0449, SUC_lexicon_tags_30 },
    { 0x14fc044a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abc0453, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa7c0455, SUC_lexicon_tags_11 },
    { 0x217c0456, SUC_lexicon_tags_8 },
    { 0x24bc0457, SUC_lexicon_tags_11 },
    { 0x903c0458, SUC_lexicon_tags_25 },
    { 0xdbc0458, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9c045f, SUC_lexicon_tags_55 },
    { 0x98bc0460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd57c0462, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd7dc0464, SUC_lexicon_tags_69 },
    { 0xb4fc0464, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffc0468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x795c046a, SUC_lexicon_tags_12 },
    { 0xa6bc046a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50dc046d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bc0470, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc43c0472, SUC_lexicon_tags_27 },
    { 0x2c3c0473, SUC_lexicon_tags_10 },
    { 0x5bdc0474, SUC_lexicon_tags_15 },
    { 0x40dc0474, SUC_lexicon_tags_21 },
    { 0xbe7c0474, SUC_lexicon_tags_49 },
    { 0xa63c0477, SUC_lexicon_tags_37 },
    { 0xf65c0478, SUC_lexicon_tags_0 },
    { 0x987c0478, SUC_lexicon_tags_2 },
    { 0xf4fc047a, SUC_lexicon_tags_66 },
    { 0xf8bc0477, SUC_lexicon_tags_25 },
    { 0xd87c0475, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1c0482, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x881c0484, SUC_lexicon_tags_25 },
    { 0xb51c0484, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x933c0487, SUC_lexicon_tags_17 },
    { 0x65c0488, SUC_lexicon_tags_40 },
    { 0xf37c0489, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51c0492, SUC_lexicon_tags_11 },
    { 0xc09c0493, SUC_lexicon_tags_63 },
    { 0x90bc0492, SUC_lexicon_tags_0 },
    { 0x825c0495, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3fbc0497, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adc049e, SUC_lexicon_tags_47 },
    { 0x233c049f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbc04a2, SUC_lexicon_tags_25 },
    { 0x337c04a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fc04a8, SUC_lexicon_tags_25 },
    { 0xcdfc04a9, SUC_lexicon_tags_87 },
    { 0x52fc04aa, SUC_lexicon_tags_15 },
    { 0x18fc04aa, SUC_lexicon_tags_11 },
    { 0x7b1c04a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165c04b3, SUC_lexicon_tags_27 },
    { 0xaf7c04b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24dc04b7, SUC_lexicon_tags_25 },
    { 0xe3c04b8, SUC_lexicon_tags_16 },
    { 0x385c04b7, SUC_lexicon_tags_8 },
    { 0xa17c04ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x731c04bc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fc04c6, SUC_lexicon_tags_1 },
    { 0x58dc04c7, SUC_lexicon_tags_7 },
    { 0x8c9c04c8, SUC_lexicon_tags_63 },
    { 0xa31c04c9, SUC_lexicon_tags_47 },
    { 0xac3c04ca, SUC_lexicon_tags_51 },
    { 0x443c04c6, SUC_lexicon_tags_32 },
    { 0xcc9c04cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdc04d4, SUC_lexicon_tags_37 },
    { 0xc0dc04d4, SUC_lexicon_tags_25 },
    { 0xee3c04d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5c04de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb35c04e0, SUC_lexicon_tags_8 },
    { 0x21bc04e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787c04e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb75c04e7, SUC_lexicon_tags_27 },
    { 0x87bc04e8, SUC_lexicon_tags_1200 },
    { 0, NULL },
    { 0x319c04ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fc04ed, SUC_lexicon_tags_32 },
    { 0xc29c04ee, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xe7c04f0, SUC_lexicon_tags_0 },
    { 0x40bc04f0, SUC_lexicon_tags_66 },
    { 0xe67c04f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dc04f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x373c04fb, SUC_lexicon_tags_7 },
    { 0xcf1c04fc, SUC_lexicon_tags_37 },
    { 0x33c04fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xecdc0500, SUC_lexicon_tags_17 },
    { 0x99dc0501, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x919c0503, SUC_lexicon_tags_10 },
    { 0xcb9c0503, SUC_lexicon_tags_82 },
    { 0xd27c0505, SUC_lexicon_tags_23 },
    { 0x217c0505, SUC_lexicon_tags_2 },
    { 0xe1c0507, SUC_lexicon_tags_47 },
    { 0xa49c0508, SUC_lexicon_tags_17 },
    { 0x3efc0509, SUC_lexicon_tags_15 },
    { 0x555c0503, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25c050d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57c0513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8adc0515, SUC_lexicon_tags_39 },
    { 0x371c0516, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc0519, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bc051d, SUC_lexicon_tags_37 },
    { 0x6a1c051d, SUC_lexicon_tags_31 },
    { 0xa1bc051f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf3fc0521, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9c0525, SUC_lexicon_tags_18 },
    { 0x5e5c0525, SUC_lexicon_tags_15 },
    { 0x9abc0526, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fc052a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf49c052c, SUC_lexicon_tags_38 },
    { 0x7a1c052c, SUC_lexicon_tags_9 },
    { 0x265c052e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1c0534, SUC_lexicon_tags_30 },
    { 0xf7bc0534, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253c053a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fc053e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddc0541, SUC_lexicon_tags_87 },
    { 0xee7c0542, SUC_lexicon_tags_14 },
    { 0x279c0541, SUC_lexicon_tags_37 },
    { 0x873c0544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edc0549, SUC_lexicon_tags_39 },
    { 0xbb1c054a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4bc054c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85c0551, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x835c0553, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x737c0555, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dc0559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c055d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dc0561, SUC_lexicon_tags_25 },
    { 0x7a7c0561, SUC_lexicon_tags_62 },
    { 0x45dc0563, SUC_lexicon_tags_7 },
    { 0x3c7c0561, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dc0568, SUC_lexicon_tags_1 },
    { 0x271c0569, SUC_lexicon_tags_12 },
    { 0xf17c0568, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x433c056c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x223c056e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c9c0570, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x287c0573, SUC_lexicon_tags_55 },
    { 0x1fdc0574, SUC_lexicon_tags_52 },
    { 0xf43c0575, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfabc0577, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf75c0579, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bc057b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85c0586, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bc058b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859c0592, SUC_lexicon_tags_1 },
    { 0x4dc0593, SUC_lexicon_tags_27 },
    { 0xd9c0593, SUC_lexicon_tags_37 },
    { 0x8e9c0593, SUC_lexicon_tags_2 },
    { 0x8f1c0596, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x669c0598, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbc059c, SUC_lexicon_tags_1 },
    { 0x56dc059c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd09c059f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431c05a4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9c05a9, SUC_lexicon_tags_7 },
    { 0x275c05a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefbc05ac, SUC_lexicon_tags_63 },
    { 0x825c05ad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x115c05af, SUC_lexicon_tags_11 },
    { 0x65dc05b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda7c05b2, SUC_lexicon_tags_37 },
    { 0x8dc05b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e5c05b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc05b9, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc05bc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5c05c0, SUC_lexicon_tags_19 },
    { 0x9edc05c1, SUC_lexicon_tags_47 },
    { 0xdc9c05c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdc05c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x42fc05cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c05cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1c05d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x141c05d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x335c05d7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49c05dc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2a9c05de, SUC_lexicon_tags_0 },
    { 0x263c05df, SUC_lexicon_tags_0 },
    { 0x5b7c05df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bc05e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dc05ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x861c05ec, SUC_lexicon_tags_68 },
    { 0x491c05ed, SUC_lexicon_tags_2 },
    { 0x9f1c05ee, SUC_lexicon_tags_17 },
    { 0x893c05ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1c05f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xafc05f6, SUC_lexicon_tags_1 },
    { 0x97fc05f6, SUC_lexicon_tags_37 },
    { 0x283c05f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbc05fd, SUC_lexicon_tags_27 },
    { 0x65dc05fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49c0601, SUC_lexicon_tags_21 },
    { 0x5bc0602, SUC_lexicon_tags_43 },
    { 0xfa9c0603, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bc0606, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x821c0609, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x713c060c, SUC_lexicon_tags_15 },
    { 0x627c060d, SUC_lexicon_tags_37 },
    { 0x585c060c, SUC_lexicon_tags_25 },
    { 0xe61c060f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99c0612, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77c0616, SUC_lexicon_tags_38 },
    { 0xb4dc0617, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x445c061a, SUC_lexicon_tags_26 },
    { 0x5c1c061a, SUC_lexicon_tags_6 },
    { 0x6a5c061c, SUC_lexicon_tags_7 },
    { 0xc83c061c, SUC_lexicon_tags_25 },
    { 0xccdc061c, SUC_lexicon_tags_0 },
    { 0xbb9c061e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b7c0621, SUC_lexicon_tags_11 },
    { 0x84fc0621, SUC_lexicon_tags_1 },
    { 0x3cfc0623, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdfc0625, SUC_lexicon_tags_31 },
    { 0x5fc0626, SUC_lexicon_tags_26 },
    { 0x2c7c0627, SUC_lexicon_tags_1 },
    { 0x75dc0628, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7c062d, SUC_lexicon_tags_6 },
    { 0x5c9c062d, SUC_lexicon_tags_12 },
    { 0xa6bc062f, SUC_lexicon_tags_25 },
    { 0x953c062d, SUC_lexicon_tags_17 },
    { 0x5b5c062d, SUC_lexicon_tags_62 },
    { 0x64bc0632, SUC_lexicon_tags_0 },
    { 0x2a5c0633, SUC_lexicon_tags_17 },
    { 0xa4dc0632, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35c0637, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9c063a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5c0645, SUC_lexicon_tags_25 },
    { 0x42dc0646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe7c0648, SUC_lexicon_tags_25 },
    { 0x3c9c0649, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x709c064c, SUC_lexicon_tags_37 },
    { 0x779c064c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fc0652, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9c065b, SUC_lexicon_tags_44 },
    { 0xe83c065c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bc065f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bc0665, SUC_lexicon_tags_25 },
    { 0xdc9c0665, SUC_lexicon_tags_38 },
    { 0x795c0665, SUC_lexicon_tags_12 },
    { 0xc71c0668, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x36fc066b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f9c066d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc0672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdc0679, SUC_lexicon_tags_25 },
    { 0x73bc0679, SUC_lexicon_tags_23 },
    { 0xb57c067b, SUC_lexicon_tags_56 },
    { 0x9f1c067b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7c7c067e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21c0681, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45c0685, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93c0690, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c0696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xafdc0699, SUC_lexicon_tags_68 },
    { 0xd59c069a, SUC_lexicon_tags_12 },
    { 0x3cbc069b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fc069f, SUC_lexicon_tags_9 },
    { 0x833c069f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x83fc06a2, SUC_lexicon_tags_27 },
    { 0x603c06a3, SUC_lexicon_tags_38 },
    { 0xd55c06a4, SUC_lexicon_tags_44 },
    { 0xfdc06a5, SUC_lexicon_tags_66 },
    { 0x48bc06a6, SUC_lexicon_tags_15 },
    { 0x173c06a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe73c06a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabc06ad, SUC_lexicon_tags_37 },
    { 0xa97c06ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbefc06b0, SUC_lexicon_tags_33 },
    { 0xa43c06b1, SUC_lexicon_tags_55 },
    { 0xb21c06b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x721c06b5, SUC_lexicon_tags_37 },
    { 0x441c06b6, SUC_lexicon_tags_49 },
    { 0x65bc06b7, SUC_lexicon_tags_33 },
    { 0xd4fc06b6, SUC_lexicon_tags_17 },
    { 0xe4fc06b9, SUC_lexicon_tags_12 },
    { 0xfddc06b9, SUC_lexicon_tags_49 },
    { 0x17bc06bb, SUC_lexicon_tags_13 },
    { 0x275c06bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c06c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x73dc06c4, SUC_lexicon_tags_14 },
    { 0xd61c06c4, SUC_lexicon_tags_45 },
    { 0x637c06c6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4e9c06c8, SUC_lexicon_tags_204 },
    { 0x293c06c9, SUC_lexicon_tags_1 },
    { 0x1dc06ca, SUC_lexicon_tags_15 },
    { 0xba7c06ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcafc06cd, SUC_lexicon_tags_1 },
    { 0xedbc06cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3c06d1, SUC_lexicon_tags_60 },
    { 0x391c06d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe0fc06d4, SUC_lexicon_tags_275 },
    { 0x2d3c06d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce1c06d7, SUC_lexicon_tags_11 },
    { 0x247c06d8, SUC_lexicon_tags_11 },
    { 0x4afc06d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbc06dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fc06e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bc06e3, SUC_lexicon_tags_46 },
    { 0x3f5c06e4, SUC_lexicon_tags_27 },
    { 0xe8fc06e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405c06eb, SUC_lexicon_tags_38 },
    { 0x81bc06ec, SUC_lexicon_tags_34 },
    { 0x843c06ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedc06f2, SUC_lexicon_tags_17 },
    { 0xa97c06f3, SUC_lexicon_tags_60 },
    { 0x12dc06f4, SUC_lexicon_tags_12 },
    { 0x5abc06f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3c06f9, SUC_lexicon_tags_15 },
    { 0x587c06f9, SUC_lexicon_tags_319 },
    { 0, NULL },
    { 0x259c06fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bc06ff, SUC_lexicon_tags_35 },
    { 0xe55c0700, SUC_lexicon_tags_66 },
    { 0x4f5c0701, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7c0706, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa0dc0708, SUC_lexicon_tags_2 },
    { 0xfdc0708, SUC_lexicon_tags_49 },
    { 0xc07c070a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117c0713, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1c0717, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5c071a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddc0721, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867c0725, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffc0729, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1c072d, SUC_lexicon_tags_2 },
    { 0xa03c072e, SUC_lexicon_tags_6 },
    { 0xa5bc072f, SUC_lexicon_tags_1 },
    { 0x893c0730, SUC_lexicon_tags_17 },
    { 0xdcfc0731, SUC_lexicon_tags_32 },
    { 0x589c0732, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x213c0734, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x28fc0736, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c073a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bc073d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x849c073f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x187c0741, SUC_lexicon_tags_11 },
    { 0x7bdc0742, SUC_lexicon_tags_37 },
    { 0x271c0743, SUC_lexicon_tags_37 },
    { 0x237c0744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c1c0746, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dc074e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1c0754, SUC_lexicon_tags_8 },
    { 0x3abc0755, SUC_lexicon_tags_37 },
    { 0x9efc0756, SUC_lexicon_tags_6 },
    { 0xd87c0757, SUC_lexicon_tags_17 },
    { 0x869c0758, SUC_lexicon_tags_10 },
    { 0x9dbc0759, SUC_lexicon_tags_37 },
    { 0xd1dc0755, SUC_lexicon_tags_37 },
    { 0x81bc075b, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xf3c075d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c0762, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7c0765, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81fc0768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc0773, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73c0778, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1c077c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x81dc077e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5bdc0780, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599c0784, SUC_lexicon_tags_37 },
    { 0xa13c0785, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0x67dc0787, SUC_lexicon_tags_25 },
    { 0x311c0787, SUC_lexicon_tags_43 },
    { 0xa5dc0788, SUC_lexicon_tags_2 },
    { 0x31fc078a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfc078e, SUC_lexicon_tags_27 },
    { 0x249c078e, SUC_lexicon_tags_37 },
    { 0x97fc078f, SUC_lexicon_tags_11 },
    { 0x823c0791, SUC_lexicon_tags_19 },
    { 0x249c0792, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x929c0794, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2e3c0796, SUC_lexicon_tags_12 },
    { 0x71fc0797, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb9c0799, SUC_lexicon_tags_40 },
    { 0xb4fc079a, SUC_lexicon_tags_9 },
    { 0x1d9c079b, SUC_lexicon_tags_0 },
    { 0x111c079c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x683c079e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf09c07a0, SUC_lexicon_tags_11 },
    { 0xe23c07a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89dc07a3, SUC_lexicon_tags_23 },
    { 0x7cdc07a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b7c07a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb03c07a8, SUC_lexicon_tags_11 },
    { 0xc95c07a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x78dc07ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dc07b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe69c07b5, SUC_lexicon_tags_63 },
    { 0x141c07b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1c07b9, SUC_lexicon_tags_17 },
    { 0x5f3c07ba, SUC_lexicon_tags_2 },
    { 0xf07c07bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5c07be, SUC_lexicon_tags_15 },
    { 0xe1c07bf, SUC_lexicon_tags_37 },
    { 0xfc5c07c0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x283c07c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfc07c6, SUC_lexicon_tags_47 },
    { 0x787c07c7, SUC_lexicon_tags_33 },
    { 0x4e9c07c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3c07ce, SUC_lexicon_tags_25 },
    { 0x423c07cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd83c07d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fc07da, SUC_lexicon_tags_11 },
    { 0xd97c07da, SUC_lexicon_tags_6 },
    { 0x967c07dc, SUC_lexicon_tags_26 },
    { 0xa9dc07dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc73c07df, SUC_lexicon_tags_7 },
    { 0x599c07e0, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5c07e4, SUC_lexicon_tags_1 },
    { 0xbf3c07e4, SUC_lexicon_tags_75 },
    { 0x98bc07e6, SUC_lexicon_tags_15 },
    { 0xdc7c07e6, SUC_lexicon_tags_14 },
    { 0x5a1c07e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc07ec, SUC_lexicon_tags_1 },
    { 0x829c07ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x215c07f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bc07f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ffc07f6, SUC_lexicon_tags_0 },
    { 0x37c07f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd69c07f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb27c07fb, SUC_lexicon_tags_7 },
    { 0x5bc07fc, SUC_lexicon_tags_0 },
    { 0xd47c07fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3cfc07ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeedc0801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3c080a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x545c080c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5c0810, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69c0814, SUC_lexicon_tags_37 },
    { 0xf21c0815, SUC_lexicon_tags_49 },
    { 0x1afc0816, SUC_lexicon_tags_12 },
    { 0xc09c0817, SUC_lexicon_tags_0 },
    { 0x293c0816, SUC_lexicon_tags_17 },
    { 0x975c0817, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2fdc081b, SUC_lexicon_tags_48 },
    { 0xe83c081c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3c081f, SUC_lexicon_tags_1 },
    { 0xb15c081f, SUC_lexicon_tags_15 },
    { 0xfc3c0821, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0824, SUC_lexicon_tags_25 },
    { 0x79fc0825, SUC_lexicon_tags_25 },
    { 0x72bc0826, SUC_lexicon_tags_7 },
    { 0x59dc0825, SUC_lexicon_tags_12 },
    { 0x48fc0826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad7c082a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83c082d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5c0830, SUC_lexicon_tags_33 },
    { 0x6c1c0831, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1c083b, SUC_lexicon_tags_2 },
    { 0x93fc083c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923c0842, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd03c0844, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9dfc0846, SUC_lexicon_tags_39 },
    { 0xcdfc0846, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1c084b, SUC_lexicon_tags_31 },
    { 0x55dc084b, SUC_lexicon_tags_6 },
    { 0xf67c084d, SUC_lexicon_tags_55 },
    { 0xde1c084c, SUC_lexicon_tags_2 },
    { 0xb6dc084f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37c0855, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa59c0857, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x7f5c0859, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xb55c085b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf6fc085d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95c086c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbc0871, SUC_lexicon_tags_1 },
    { 0x7adc0872, SUC_lexicon_tags_29 },
    { 0xa47c0873, SUC_lexicon_tags_11 },
    { 0x6dc0874, SUC_lexicon_tags_4 },
    { 0x2b3c0875, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35c0877, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb77c0879, SUC_lexicon_tags_11 },
    { 0x513c0879, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1c0883, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37c0886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3c088e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5c0897, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x47dc089a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xf63c089c, SUC_lexicon_tags_46 },
    { 0x167c089c, SUC_lexicon_tags_40 },
    { 0x577c089d, SUC_lexicon_tags_1 },
    { 0x87c089f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17c08a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7a9c08a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99c08aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbc08b1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633c08b6, SUC_lexicon_tags_15 },
    { 0xecbc08b7, SUC_lexicon_tags_37 },
    { 0x825c08b8, SUC_lexicon_tags_61 },
    { 0xa1dc08b9, SUC_lexicon_tags_14 },
    { 0x601c08ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551c08be, SUC_lexicon_tags_81 },
    { 0x5e5c08be, SUC_lexicon_tags_25 },
    { 0xd5bc08c0, SUC_lexicon_tags_28 },
    { 0xb11c08c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26bc08c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fc08c7, SUC_lexicon_tags_49 },
    { 0xb1dc08c8, SUC_lexicon_tags_12 },
    { 0x13bc08c8, SUC_lexicon_tags_37 },
    { 0xe47c08ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdc08ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdc08d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfc08d9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbc5c08db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb77c08dd, SUC_lexicon_tags_27 },
    { 0x7a7c08de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67c08e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1c08e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6dbc08e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bc08eb, SUC_lexicon_tags_21 },
    { 0x5d9c08ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7c08f1, SUC_lexicon_tags_7 },
    { 0x815c08f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdc08f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9c08fb, SUC_lexicon_tags_12 },
    { 0xa2fc08fc, SUC_lexicon_tags_25 },
    { 0xd61c08fc, SUC_lexicon_tags_2 },
    { 0x3c3c08fc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89c0902, SUC_lexicon_tags_11 },
    { 0xcafc0902, SUC_lexicon_tags_47 },
    { 0x71fc0903, SUC_lexicon_tags_23 },
    { 0x11bc0905, SUC_lexicon_tags_17 },
    { 0x3cdc0904, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfc090c, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fc090f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653c0914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbc0917, SUC_lexicon_tags_17 },
    { 0xaf1c0918, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f5c091a, SUC_lexicon_tags_27 },
    { 0x2c3c091b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939c0923, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x135c0925, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55dc092b, SUC_lexicon_tags_11 },
    { 0xf9dc092c, SUC_lexicon_tags_12 },
    { 0xf5bc092b, SUC_lexicon_tags_15 },
    { 0x655c092e, SUC_lexicon_tags_25 },
    { 0xbcdc092b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bc0935, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44fc0937, SUC_lexicon_tags_18 },
    { 0x12bc0938, SUC_lexicon_tags_9 },
    { 0x7e7c0939, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fc093f, SUC_lexicon_tags_28 },
    { 0x933c093f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abc0943, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bc0948, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7c094c, SUC_lexicon_tags_45 },
    { 0xc35c094d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddc0954, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43c095f, SUC_lexicon_tags_0 },
    { 0xb73c0960, SUC_lexicon_tags_1 },
    { 0x17fc0960, SUC_lexicon_tags_37 },
    { 0x60fc0961, SUC_lexicon_tags_15 },
    { 0xbb1c0963, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa27c0965, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda5c0967, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabc096d, SUC_lexicon_tags_17 },
    { 0x4a7c096e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7c0971, SUC_lexicon_tags_11 },
    { 0x837c0972, SUC_lexicon_tags_37 },
    { 0xdb3c0972, SUC_lexicon_tags_47 },
    { 0x511c0974, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5c0977, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x507c0979, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fc097c, SUC_lexicon_tags_79 },
    { 0x35dc097d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42fc097f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x81fc0981, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x42dc0983, SUC_lexicon_tags_65 },
    { 0xd4fc0983, SUC_lexicon_tags_28 },
    { 0x4fbc0985, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abc0989, SUC_lexicon_tags_37 },
    { 0x473c0989, SUC_lexicon_tags_6 },
    { 0x329c098b, SUC_lexicon_tags_80 },
    { 0x99bc098a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4bc098e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x491c0990, SUC_lexicon_tags_91 },
    { 0x3bfc0991, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xecfc0993, SUC_lexicon_tags_60 },
    { 0x277c0994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x103c0996, SUC_lexicon_tags_17 },
    { 0xd17c0996, SUC_lexicon_tags_12 },
    { 0xdd1c0998, SUC_lexicon_tags_63 },
    { 0x30dc0998, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc099f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x711c09a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83bc09a3, SUC_lexicon_tags_127 },
    { 0x1afc09a4, SUC_lexicon_tags_12 },
    { 0x615c09a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dc09aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59c09ad, SUC_lexicon_tags_49 },
    { 0x79bc09ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x475c09b0, SUC_lexicon_tags_11 },
    { 0x893c09b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78dc09b4, SUC_lexicon_tags_1 },
    { 0x8f9c09b5, SUC_lexicon_tags_12 },
    { 0x7e3c09b6, SUC_lexicon_tags_2 },
    { 0xb13c09b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7c09ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5c09bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf3c09c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x185c09c3, SUC_lexicon_tags_57 },
    { 0x32dc09c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3c09c9, SUC_lexicon_tags_30 },
    { 0x40fc09ca, SUC_lexicon_tags_25 },
    { 0x4cbc09ca, SUC_lexicon_tags_1 },
    { 0x385c09cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x971c09ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9c09d1, SUC_lexicon_tags_7 },
    { 0x5bfc09d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbc09d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7f3c09d7, SUC_lexicon_tags_12 },
    { 0x4c3c09d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x75bc09da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3bc09dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647c09e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x509c09e4, SUC_lexicon_tags_4 },
    { 0xdcfc09e5, SUC_lexicon_tags_2 },
    { 0x285c09e5, SUC_lexicon_tags_25 },
    { 0x45bc09e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81c09e9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x833c09ec, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x727c09ee, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc77c09f0, SUC_lexicon_tags_7 },
    { 0xe6bc09f1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9c09f4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c09f8, SUC_lexicon_tags_12 },
    { 0x551c09f9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x2c9c09fb, SUC_lexicon_tags_14 },
    { 0x39c09fc, SUC_lexicon_tags_63 },
    { 0x727c09fd, SUC_lexicon_tags_65 },
    { 0x45dc09fd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bc0a03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x963c0a05, SUC_lexicon_tags_32 },
    { 0x7c5c0a05, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bc0a0c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x243c0a0e, SUC_lexicon_tags_15 },
    { 0x22fc0a0f, SUC_lexicon_tags_49 },
    { 0x883c0a0f, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337c0a16, SUC_lexicon_tags_1 },
    { 0xa8bc0a17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9c0a1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x693c0a1d, SUC_lexicon_tags_25 },
    { 0xbdc0a1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1c0a23, SUC_lexicon_tags_1 },
    { 0xb85c0a24, SUC_lexicon_tags_16 },
    { 0xd51c0a23, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc0a28, SUC_lexicon_tags_14 },
    { 0xf0bc0a29, SUC_lexicon_tags_53 },
    { 0x831c0a29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x447c0a2d, SUC_lexicon_tags_2 },
    { 0xabbc0a2d, SUC_lexicon_tags_34 },
    { 0xaa1c0a2e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedc0a37, SUC_lexicon_tags_57 },
    { 0x829c0a37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce9c0a3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc0fc0a3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x863c0a3f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1c0a45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5adc0a47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x487c0a49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x215c0a4c, SUC_lexicon_tags_11 },
    { 0xa99c0a4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x13bc0a4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3c0a52, SUC_lexicon_tags_47 },
    { 0xa09c0a53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165c0a5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfc0a5e, SUC_lexicon_tags_7 },
    { 0x703c0a5f, SUC_lexicon_tags_60 },
    { 0x79fc0a60, SUC_lexicon_tags_15 },
    { 0x467c0a60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x427c0a63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaf1c0a65, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2cfc0a67, SUC_lexicon_tags_10 },
    { 0x5c1c0a68, SUC_lexicon_tags_46 },
    { 0x65dc0a69, SUC_lexicon_tags_816 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61c0a6d, SUC_lexicon_tags_48 },
    { 0xb1c0a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x75bc0a70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x253c0a72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x633c0a75, SUC_lexicon_tags_37 },
    { 0x4a5c0a75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc0a7b, SUC_lexicon_tags_37 },
    { 0xb3c0a7c, SUC_lexicon_tags_235 },
    { 0x3edc0a7b, SUC_lexicon_tags_4 },
    { 0x703c0a7e, SUC_lexicon_tags_37 },
    { 0x651c0a7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635c0a8c, SUC_lexicon_tags_1704 },
    { 0x855c0a8d, SUC_lexicon_tags_25 },
    { 0xc5c0a8d, SUC_lexicon_tags_31 },
    { 0x6ffc0a8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96dc0a91, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851c0a98, SUC_lexicon_tags_17 },
    { 0x195c0a98, SUC_lexicon_tags_11 },
    { 0x643c0a9a, SUC_lexicon_tags_15 },
    { 0x62fc0a9a, SUC_lexicon_tags_12 },
    { 0x6c9c0a9a, SUC_lexicon_tags_68 },
    { 0xd49c0a9d, SUC_lexicon_tags_47 },
    { 0x84fc0a9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33c0aa1, SUC_lexicon_tags_25 },
    { 0x82fc0aa2, SUC_lexicon_tags_33 },
    { 0x20bc0aa3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bc0aa9, SUC_lexicon_tags_9 },
    { 0x721c0aaa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa27c0aac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb0bc0aae, SUC_lexicon_tags_13 },
    { 0x4f9c0aaf, SUC_lexicon_tags_402 },
    { 0x5d3c0aae, SUC_lexicon_tags_15 },
    { 0xe15c0ab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x493c0ab3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bc0aba, SUC_lexicon_tags_25 },
    { 0xb3dc0abb, SUC_lexicon_tags_5 },
    { 0xe3fc0abc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x50bc0abe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43c0ac4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37c0ac8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9c0acd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x355c0acf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97c0ad6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9c0ad9, SUC_lexicon_tags_0 },
    { 0xb15c0ada, SUC_lexicon_tags_6 },
    { 0x717c0adb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63fc0add, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401c0aea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9ffc0aec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfc0aef, SUC_lexicon_tags_62 },
    { 0x89dc0af0, SUC_lexicon_tags_60 },
    { 0x811c0af1, SUC_lexicon_tags_29 },
    { 0xe6fc0af2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fc0af8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x325c0afb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77c0b00, SUC_lexicon_tags_47 },
    { 0xc9dc0b00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4fc0b03, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74dc0b07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f5c0b09, SUC_lexicon_tags_63 },
    { 0xf37c0b0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x145c0b0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25c0b13, SUC_lexicon_tags_9 },
    { 0x54dc0b14, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abc0b18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85c0b1b, SUC_lexicon_tags_11 },
    { 0x57c0b1c, SUC_lexicon_tags_12 },
    { 0x24fc0b1b, SUC_lexicon_tags_4 },
    { 0x11dc0b1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bc0b22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4f9c0b24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77dc0b27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dc0b2b, SUC_lexicon_tags_45 },
    { 0xe5fc0b2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58fc0b2f, SUC_lexicon_tags_7 },
    { 0xadbc0b30, SUC_lexicon_tags_12 },
    { 0x2cdc0b31, SUC_lexicon_tags_29 },
    { 0xbc9c0b31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577c0b37, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc4bc0b39, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97c0b3c, SUC_lexicon_tags_25 },
    { 0x5c3c0b3c, SUC_lexicon_tags_60 },
    { 0x393c0b3d, SUC_lexicon_tags_21 },
    { 0x42bc0b3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc0b46, SUC_lexicon_tags_21 },
    { 0x7fdc0b46, SUC_lexicon_tags_49 },
    { 0x8b3c0b48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7c0b4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657c0b50, SUC_lexicon_tags_25 },
    { 0xd43c0b51, SUC_lexicon_tags_37 },
    { 0x443c0b52, SUC_lexicon_tags_0 },
    { 0x73dc0b53, SUC_lexicon_tags_37 },
    { 0x2fbc0b52, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1c0b5a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbc0b5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bc0b61, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61c0b66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x391c0b69, SUC_lexicon_tags_17 },
    { 0xf63c0b6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1bc0b6c, SUC_lexicon_tags_7 },
    { 0xf69c0b6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9efc0b6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffc0b76, SUC_lexicon_tags_39 },
    { 0x209c0b77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x681c0b79, SUC_lexicon_tags_11 },
    { 0x12fc0b7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85c0b7f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83c0b83, SUC_lexicon_tags_7 },
    { 0xccfc0b84, SUC_lexicon_tags_46 },
    { 0x889c0b85, SUC_lexicon_tags_12 },
    { 0x3d3c0b86, SUC_lexicon_tags_44 },
    { 0x6b1c0b87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5fc0b89, SUC_lexicon_tags_11 },
    { 0xbcbc0b8a, SUC_lexicon_tags_60 },
    { 0xa3c0b8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fc0b94, SUC_lexicon_tags_37 },
    { 0x18dc0b95, SUC_lexicon_tags_19 },
    { 0x6edc0b94, SUC_lexicon_tags_11 },
    { 0xca1c0b97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfc0b9e, SUC_lexicon_tags_11 },
    { 0x38dc0b9e, SUC_lexicon_tags_15 },
    { 0x957c0ba0, SUC_lexicon_tags_13 },
    { 0xc4fc0ba1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fc0ba4, SUC_lexicon_tags_92 },
    { 0x22bc0ba5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x645c0ba7, SUC_lexicon_tags_26 },
    { 0xee7c0ba7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x485c0bab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffc0bb3, SUC_lexicon_tags_37 },
    { 0x5cdc0bb3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3c0bb8, SUC_lexicon_tags_23 },
    { 0xa7fc0bb9, SUC_lexicon_tags_15 },
    { 0x85fc0bb8, SUC_lexicon_tags_0 },
    { 0x7b9c0bba, SUC_lexicon_tags_4 },
    { 0x335c0bbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x811c0bbf, SUC_lexicon_tags_3 },
    { 0x30dc0bc0, SUC_lexicon_tags_19 },
    { 0x5c0bc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1c0bc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x367c0bc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191c0bd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6b9c0bd2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dc0bd9, SUC_lexicon_tags_17 },
    { 0x3e1c0bd9, SUC_lexicon_tags_7 },
    { 0x3e7c0bda, SUC_lexicon_tags_23 },
    { 0x901c0bdc, SUC_lexicon_tags_37 },
    { 0x411c0bd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67c0bdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x855c0be2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fc0be6, SUC_lexicon_tags_30 },
    { 0xe83c0be7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9c0bec, SUC_lexicon_tags_14 },
    { 0xc47c0bec, SUC_lexicon_tags_6 },
    { 0xd25c0bec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bc0bf8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83c0bfc, SUC_lexicon_tags_38 },
    { 0x9a5c0bfd, SUC_lexicon_tags_29 },
    { 0xc85c0bfe, SUC_lexicon_tags_6 },
    { 0x9b7c0bfc, SUC_lexicon_tags_10 },
    { 0x54bc0c00, SUC_lexicon_tags_11 },
    { 0xebc0c01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97c0c05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae7c0c07, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3a9c0c09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1c0c0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5c0c12, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfffc0c14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc0c17, SUC_lexicon_tags_38 },
    { 0x6c3c0c18, SUC_lexicon_tags_23 },
    { 0x9b9c0c19, SUC_lexicon_tags_45 },
    { 0x6ebc0c19, SUC_lexicon_tags_11 },
    { 0x995c0c1b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe6fc0c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3c0c21, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xfb1c0c23, SUC_lexicon_tags_29 },
    { 0xa2fc0c23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0fc0c26, SUC_lexicon_tags_15 },
    { 0xfc5c0c27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fc0c2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49c0c2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71c0c32, SUC_lexicon_tags_7 },
    { 0x23dc0c33, SUC_lexicon_tags_0 },
    { 0x3c3c0c34, SUC_lexicon_tags_14 },
    { 0x761c0c34, SUC_lexicon_tags_17 },
    { 0x9dbc0c35, SUC_lexicon_tags_7 },
    { 0x9dbc0c37, SUC_lexicon_tags_11 },
    { 0xf43c0c38, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x519c0c3a, SUC_lexicon_tags_25 },
    { 0xf97c0c3b, SUC_lexicon_tags_9 },
    { 0xb97c0c3c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fc0c3f, SUC_lexicon_tags_11 },
    { 0xa97c0c40, SUC_lexicon_tags_37 },
    { 0xbc7c0c40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x837c0c43, SUC_lexicon_tags_26 },
    { 0xd7fc0c43, SUC_lexicon_tags_87 },
    { 0xe95c0c45, SUC_lexicon_tags_47 },
    { 0xe61c0c46, SUC_lexicon_tags_37 },
    { 0x317c0c47, SUC_lexicon_tags_48 },
    { 0xf87c0c48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75c0c4c, SUC_lexicon_tags_64 },
    { 0x82bc0c4c, SUC_lexicon_tags_39 },
    { 0x553c0c4e, SUC_lexicon_tags_25 },
    { 0x1dfc0c4f, SUC_lexicon_tags_22 },
    { 0xee5c0c4f, SUC_lexicon_tags_11 },
    { 0xb59c0c51, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dc0c55, SUC_lexicon_tags_47 },
    { 0x9b1c0c56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c7c0c58, SUC_lexicon_tags_17 },
    { 0x575c0c58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05c0c5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1c0c61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5c0c64, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x879c0c66, SUC_lexicon_tags_16 },
    { 0xf13c0c67, SUC_lexicon_tags_15 },
    { 0x32fc0c68, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0x823c0c6b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x32fc0c6d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x249c0c70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853c0c75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3c0c7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc5c0c7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x605c0c80, SUC_lexicon_tags_82 },
    { 0x31dc0c81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x643c0c84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x277c0c86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd99c0c88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbc0c8c, SUC_lexicon_tags_1 },
    { 0x765c0c8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfc0c92, SUC_lexicon_tags_49 },
    { 0x67dc0c92, SUC_lexicon_tags_43 },
    { 0x347c0c94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa67c0c96, SUC_lexicon_tags_12 },
    { 0x1a3c0c97, SUC_lexicon_tags_15 },
    { 0x649c0c96, SUC_lexicon_tags_17 },
    { 0x9efc0c99, SUC_lexicon_tags_37 },
    { 0x95dc0c97, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf99c0c9c, SUC_lexicon_tags_47 },
    { 0xbe9c0c9d, SUC_lexicon_tags_17 },
    { 0xc65c0c9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdc0ca2, SUC_lexicon_tags_17 },
    { 0xb9fc0ca2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1c0ca6, SUC_lexicon_tags_7 },
    { 0x96dc0ca7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x405c0caa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfbbc0cac, SUC_lexicon_tags_112 },
    { 0xff9c0cad, SUC_lexicon_tags_27 },
    { 0x5cfc0cae, SUC_lexicon_tags_60 },
    { 0x143c0caf, SUC_lexicon_tags_25 },
    { 0xbe5c0cb0, SUC_lexicon_tags_39 },
    { 0x441c0cb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd45c0cb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x64fc0cb5, SUC_lexicon_tags_11 },
    { 0xc23c0cb6, SUC_lexicon_tags_6 },
    { 0xf21c0cb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81c0cbe, SUC_lexicon_tags_27 },
    { 0x567c0cbf, SUC_lexicon_tags_9 },
    { 0x531c0cc0, SUC_lexicon_tags_0 },
    { 0x17c0cbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x763c0cc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1c0cc6, SUC_lexicon_tags_1 },
    { 0x753c0cc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593c0cce, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dc0cd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3c0cdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bbc0ce1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fc0ce4, SUC_lexicon_tags_12 },
    { 0x581c0ce4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfc0ce9, SUC_lexicon_tags_24 },
    { 0xb1c0ce9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9c0cee, SUC_lexicon_tags_2 },
    { 0x44dc0cee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc0cf4, SUC_lexicon_tags_23 },
    { 0x7c1c0cf4, SUC_lexicon_tags_17 },
    { 0xaafc0cf6, SUC_lexicon_tags_7 },
    { 0x155c0cf6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x655c0cf9, SUC_lexicon_tags_17 },
    { 0xcfdc0cfa, SUC_lexicon_tags_15 },
    { 0x925c0cfb, SUC_lexicon_tags_25 },
    { 0x94fc0cfa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa07c0cfe, SUC_lexicon_tags_28 },
    { 0x425c0cfe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x29bc0d01, SUC_lexicon_tags_2 },
    { 0xc2fc0d02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfc0d0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91c0d0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x53fc0d10, SUC_lexicon_tags_48 },
    { 0xb09c0d10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7c0d19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adc0d1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fc0d23, SUC_lexicon_tags_37 },
    { 0x54bc0d24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ebc0d26, SUC_lexicon_tags_1 },
    { 0xdc1c0d27, SUC_lexicon_tags_7 },
    { 0x76fc0d27, SUC_lexicon_tags_47 },
    { 0x741c0d29, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xde9c0d2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1c0d2e, SUC_lexicon_tags_7 },
    { 0x985c0d2f, SUC_lexicon_tags_2 },
    { 0x915c0d30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efc0d34, SUC_lexicon_tags_0 },
    { 0x1e7c0d35, SUC_lexicon_tags_17 },
    { 0xa3bc0d35, SUC_lexicon_tags_16 },
    { 0x82fc0d36, SUC_lexicon_tags_10 },
    { 0xf69c0d34, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107c0d3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03c0d40, SUC_lexicon_tags_21 },
    { 0xdd9c0d40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x689c0d43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3c0d4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefc0d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fc0d53, SUC_lexicon_tags_17 },
    { 0xb6fc0d54, SUC_lexicon_tags_0 },
    { 0x30fc0d55, SUC_lexicon_tags_46 },
    { 0x12dc0d54, SUC_lexicon_tags_45 },
    { 0x20fc0d57, SUC_lexicon_tags_2 },
    { 0xf9dc0d55, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31dc0d5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc0d62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5c0d66, SUC_lexicon_tags_14 },
    { 0xeffc0d66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fc0d71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bc0d75, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61c0d78, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233c0d7e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdc0d81, SUC_lexicon_tags_60 },
    { 0xeb5c0d82, SUC_lexicon_tags_231 },
    { 0x783c0d82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401c0d88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x597c0d8a, SUC_lexicon_tags_7 },
    { 0xea1c0d8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5c0d90, SUC_lexicon_tags_11 },
    { 0x1f5c0d91, SUC_lexicon_tags_27 },
    { 0xb1dc0d92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81bc0d94, SUC_lexicon_tags_17 },
    { 0x6a5c0d94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bc0d9d, SUC_lexicon_tags_45 },
    { 0x3dbc0d9e, SUC_lexicon_tags_65 },
    { 0xb7c0d9e, SUC_lexicon_tags_1 },
    { 0x29c0d9e, SUC_lexicon_tags_47 },
    { 0xabfc0d9f, SUC_lexicon_tags_11 },
    { 0x2e9c0da0, SUC_lexicon_tags_21 },
    { 0xeabc0d9e, SUC_lexicon_tags_33 },
    { 0x783c0da1, SUC_lexicon_tags_45 },
    { 0x6a3c0da3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1c0dac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823c0db2, SUC_lexicon_tags_15 },
    { 0x441c0db3, SUC_lexicon_tags_6 },
    { 0xfbbc0db3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xf2dc0db6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8afc0db8, SUC_lexicon_tags_3 },
    { 0x153c0db9, SUC_lexicon_tags_496 },
    { 0x5c0dba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647c0dbe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1cfc0dc0, SUC_lexicon_tags_14 },
    { 0x31bc0dc1, SUC_lexicon_tags_1 },
    { 0x1e3c0dc1, SUC_lexicon_tags_38 },
    { 0xd5dc0dc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1c0dc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c0dcd, SUC_lexicon_tags_15 },
    { 0x40dc0dcd, SUC_lexicon_tags_68 },
    { 0x291c0dcf, SUC_lexicon_tags_17 },
    { 0x737c0dd0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x507c0dd2, SUC_lexicon_tags_45 },
    { 0x239c0dd2, SUC_lexicon_tags_60 },
    { 0xb3fc0dd4, SUC_lexicon_tags_52 },
    { 0x6e3c0dd4, SUC_lexicon_tags_7 },
    { 0x133c0dd6, SUC_lexicon_tags_49 },
    { 0x1b7c0dd7, SUC_lexicon_tags_14 },
    { 0x5e7c0dd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25dc0dda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bc0ddd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bc0de1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc0de5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x57c0de8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6dbc0dea, SUC_lexicon_tags_37 },
    { 0x245c0deb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95dc0ded, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433c0df1, SUC_lexicon_tags_11 },
    { 0x1cfc0df1, SUC_lexicon_tags_17 },
    { 0xcebc0df3, SUC_lexicon_tags_23 },
    { 0xa69c0df4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20bc0df6, SUC_lexicon_tags_84 },
    { 0x599c0df7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bc0dfd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3c0e02, SUC_lexicon_tags_1239 },
    { 0x283c0e03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7c0e08, SUC_lexicon_tags_13 },
    { 0x61fc0e08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x137c0e0c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9c0e0f, SUC_lexicon_tags_37 },
    { 0x19fc0e10, SUC_lexicon_tags_60 },
    { 0x81fc0e0f, SUC_lexicon_tags_26 },
    { 0x517c0e11, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fc0e16, SUC_lexicon_tags_131 },
    { 0x551c0e17, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5c0e1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d9c0e20, SUC_lexicon_tags_2 },
    { 0x10bc0e20, SUC_lexicon_tags_60 },
    { 0xdcbc0e21, SUC_lexicon_tags_25 },
    { 0xfe9c0e22, SUC_lexicon_tags_65 },
    { 0x81dc0e20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7c0e2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9c0e2d, SUC_lexicon_tags_11 },
    { 0x567c0e2e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bc0e32, SUC_lexicon_tags_11 },
    { 0x28fc0e33, SUC_lexicon_tags_114 },
    { 0x481c0e32, SUC_lexicon_tags_7 },
    { 0x755c0e35, SUC_lexicon_tags_31 },
    { 0x123c0e36, SUC_lexicon_tags_25 },
    { 0xcf7c0e37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xddbc0e39, SUC_lexicon_tags_47 },
    { 0xfedc0e39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa49c0e3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dc0e40, SUC_lexicon_tags_6 },
    { 0xb6fc0e41, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeffc0e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x565c0e45, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619c0e4a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x409c0e4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdfc0e4f, SUC_lexicon_tags_9 },
    { 0xcd3c0e4f, SUC_lexicon_tags_15 },
    { 0x27dc0e50, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x68dc0e54, SUC_lexicon_tags_29 },
    { 0xb61c0e55, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987c0e5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb9fc0e5d, SUC_lexicon_tags_39 },
    { 0xb59c0e5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579c0e62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c0e66, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa79c0e68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fc0e74, SUC_lexicon_tags_43 },
    { 0xd0dc0e75, SUC_lexicon_tags_15 },
    { 0x5d5c0e75, SUC_lexicon_tags_11 },
    { 0xb77c0e77, SUC_lexicon_tags_11 },
    { 0xefbc0e78, SUC_lexicon_tags_17 },
    { 0xa93c0e79, SUC_lexicon_tags_0 },
    { 0x377c0e7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4adc0e7c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x453c0e7e, SUC_lexicon_tags_43 },
    { 0x4bdc0e7f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455c0e84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839c0e88, SUC_lexicon_tags_14 },
    { 0x813c0e89, SUC_lexicon_tags_27 },
    { 0x5e1c0e8a, SUC_lexicon_tags_9 },
    { 0x349c0e8b, SUC_lexicon_tags_38 },
    { 0xa49c0e8c, SUC_lexicon_tags_15 },
    { 0xd39c0e8d, SUC_lexicon_tags_87 },
    { 0x5afc0e8e, SUC_lexicon_tags_11 },
    { 0x637c0e8f, SUC_lexicon_tags_17 },
    { 0x25c0e90, SUC_lexicon_tags_15 },
    { 0xb6fc0e8f, SUC_lexicon_tags_11 },
    { 0x219c0e8e, SUC_lexicon_tags_2 },
    { 0xcbdc0e8f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999c0ea2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dc0ea5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x719c0ea7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3fc0ea9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19c0eac, SUC_lexicon_tags_29 },
    { 0xfc0ead, SUC_lexicon_tags_45 },
    { 0x871c0ead, SUC_lexicon_tags_65 },
    { 0x4bdc0eae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1c0eb3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283c0ebb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x72bc0ebd, SUC_lexicon_tags_15 },
    { 0x157c0ebe, SUC_lexicon_tags_14 },
    { 0x349c0ebf, SUC_lexicon_tags_2 },
    { 0x343c0ec0, SUC_lexicon_tags_1 },
    { 0x3f7c0ec1, SUC_lexicon_tags_87 },
    { 0xb43c0ec1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adc0ec5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x67dc0ec8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57c0ed2, SUC_lexicon_tags_45 },
    { 0xca1c0ed3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5c0ede, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dc0ee2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61c0ee6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91c0eea, SUC_lexicon_tags_15 },
    { 0x7cfc0eeb, SUC_lexicon_tags_55 },
    { 0x8b3c0eeb, SUC_lexicon_tags_34 },
    { 0x181c0eeb, SUC_lexicon_tags_15 },
    { 0x839c0eed, SUC_lexicon_tags_21 },
    { 0x3fbc0eec, SUC_lexicon_tags_37 },
    { 0xa15c0ef0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3c0ef4, SUC_lexicon_tags_1 },
    { 0xe59c0ef5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6fc0ef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c0efa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97c0eff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36bc0f01, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbc0f04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfc0f0a, SUC_lexicon_tags_47 },
    { 0x5efc0f0b, SUC_lexicon_tags_12 },
    { 0x1bc0f0b, SUC_lexicon_tags_14 },
    { 0x9abc0f0b, SUC_lexicon_tags_27 },
    { 0x169c0f0a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5a1c0f10, SUC_lexicon_tags_15 },
    { 0xfdfc0f10, SUC_lexicon_tags_11 },
    { 0x12dc0f11, SUC_lexicon_tags_80 },
    { 0x757c0f13, SUC_lexicon_tags_33 },
    { 0x4f5c0f14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x823c0f16, SUC_lexicon_tags_29 },
    { 0x731c0f17, SUC_lexicon_tags_9 },
    { 0x441c0f17, SUC_lexicon_tags_27 },
    { 0xaddc0f19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c0f20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707c0f28, SUC_lexicon_tags_63 },
    { 0x77fc0f29, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf0dc0f2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dc0f37, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc2dc0f39, SUC_lexicon_tags_17 },
    { 0xe07c0f39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03c0f3f, SUC_lexicon_tags_10 },
    { 0xcd3c0f40, SUC_lexicon_tags_11 },
    { 0xb61c0f41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc0f44, SUC_lexicon_tags_37 },
    { 0x129c0f45, SUC_lexicon_tags_106 },
    { 0x9d5c0f45, SUC_lexicon_tags_37 },
    { 0x1d1c0f47, SUC_lexicon_tags_63 },
    { 0x6efc0f46, SUC_lexicon_tags_23 },
    { 0xb31c0f44, SUC_lexicon_tags_9 },
    { 0x53dc0f48, SUC_lexicon_tags_12 },
    { 0xfcbc0f4a, SUC_lexicon_tags_1 },
    { 0x647c0f4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc99c0f4e, SUC_lexicon_tags_66 },
    { 0x5bdc0f4f, SUC_lexicon_tags_17 },
    { 0xc9fc0f50, SUC_lexicon_tags_47 },
    { 0xb47c0f51, SUC_lexicon_tags_12 },
    { 0xdffc0f51, SUC_lexicon_tags_14 },
    { 0x2d7c0f4f, SUC_lexicon_tags_17 },
    { 0x5efc0f54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45c0f59, SUC_lexicon_tags_11 },
    { 0xd9bc0f5a, SUC_lexicon_tags_25 },
    { 0xe2bc0f5b, SUC_lexicon_tags_25 },
    { 0x225c0f5c, SUC_lexicon_tags_7 },
    { 0x53bc0f5d, SUC_lexicon_tags_43 },
    { 0x739c0f5c, SUC_lexicon_tags_32 },
    { 0x555c0f5f, SUC_lexicon_tags_15 },
    { 0x89bc0f5f, SUC_lexicon_tags_29 },
    { 0x3cfc0f61, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xbabc0f63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1c0f68, SUC_lexicon_tags_25 },
    { 0x963c0f69, SUC_lexicon_tags_25 },
    { 0x2d5c0f69, SUC_lexicon_tags_68 },
    { 0xda9c0f6b, SUC_lexicon_tags_33 },
    { 0xd77c0f6b, SUC_lexicon_tags_47 },
    { 0xd2dc0f6b, SUC_lexicon_tags_68 },
    { 0x115c0f6e, SUC_lexicon_tags_1 },
    { 0x53bc0f6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2f1c0f71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5c0f74, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xda5c0f76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fc0f7b, SUC_lexicon_tags_44 },
    { 0x84fc0f7c, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xbfbc0f7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfc0f81, SUC_lexicon_tags_49 },
    { 0xe4fc0f82, SUC_lexicon_tags_146 },
    { 0x4bc0f82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd55c0f85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1c0f8f, SUC_lexicon_tags_87 },
    { 0x3a7c0f8f, SUC_lexicon_tags_65 },
    { 0x95bc0f8f, SUC_lexicon_tags_2 },
    { 0x21dc0f92, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9c0f96, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x815c0f98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81fc0f9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb0bc0f9c, SUC_lexicon_tags_2 },
    { 0x3c5c0f9d, SUC_lexicon_tags_12 },
    { 0xf47c0f9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebc0fa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bc0fa6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xccfc0fa8, SUC_lexicon_tags_25 },
    { 0xfa1c0fa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951c0fad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c0fb0, SUC_lexicon_tags_45 },
    { 0xfd9c0fb0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffc0fb4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x86bc0fb6, SUC_lexicon_tags_47 },
    { 0x5bbc0fb7, SUC_lexicon_tags_28 },
    { 0x655c0fb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69c0fbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dc0fc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13c0fc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1c0fcb, SUC_lexicon_tags_37 },
    { 0x8abc0fcc, SUC_lexicon_tags_37 },
    { 0xf0fc0fcd, SUC_lexicon_tags_17 },
    { 0x401c0fce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x609c0fd0, SUC_lexicon_tags_25 },
    { 0xeebc0fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67c0fd3, SUC_lexicon_tags_29 },
    { 0x4d1c0fd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15dc0fd6, SUC_lexicon_tags_55 },
    { 0x479c0fd6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bc0fdb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x357c0fdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c0fe2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7fc0fe4, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x1c7c0fe6, SUC_lexicon_tags_11 },
    { 0xfa5c0fe7, SUC_lexicon_tags_9 },
    { 0x5b9c0fe8, SUC_lexicon_tags_29 },
    { 0x3a1c0fe9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x877c0fec, SUC_lexicon_tags_5 },
    { 0x625c0fec, SUC_lexicon_tags_31 },
    { 0x22dc0fee, SUC_lexicon_tags_25 },
    { 0x9d3c0fed, SUC_lexicon_tags_28 },
    { 0x37fc0ff0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafc0ff3, SUC_lexicon_tags_37 },
    { 0xf7dc0ff4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeffc0ff6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x97dc0ff8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc7fc0ffa, SUC_lexicon_tags_17 },
    { 0x5b7c0ffb, SUC_lexicon_tags_15 },
    { 0x23c0ffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x16bc0ffe, SUC_lexicon_tags_23 },
    { 0x829c0ffe, SUC_lexicon_tags_7 },
    { 0x2a1c1000, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x199c1002, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbcbc1004, SUC_lexicon_tags_7 },
    { 0x985c1005, SUC_lexicon_tags_43 },
    { 0xc11c1005, SUC_lexicon_tags_63 },
    { 0x4dc1006, SUC_lexicon_tags_32 },
    { 0x4d7c1004, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb87c100a, SUC_lexicon_tags_25 },
    { 0x15dc100b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5c100f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f5c1011, SUC_lexicon_tags_47 },
    { 0x21dc1012, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x96dc1015, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc101c, SUC_lexicon_tags_1 },
    { 0xdf5c101d, SUC_lexicon_tags_7 },
    { 0xa1fc101d, SUC_lexicon_tags_4 },
    { 0x3dbc101d, SUC_lexicon_tags_1 },
    { 0xbe5c101e, SUC_lexicon_tags_16 },
    { 0x8dbc1021, SUC_lexicon_tags_62 },
    { 0x88fc101d, SUC_lexicon_tags_12 },
    { 0x823c101f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd69c1025, SUC_lexicon_tags_15 },
    { 0xb69c1025, SUC_lexicon_tags_47 },
    { 0x4f5c1027, SUC_lexicon_tags_17 },
    { 0x265c1026, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847c102c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeafc102e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafc1034, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe15c1036, SUC_lexicon_tags_87 },
    { 0x123c1037, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dc103a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bc103f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9c1045, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2a5c1047, SUC_lexicon_tags_34 },
    { 0x1cbc1047, SUC_lexicon_tags_17 },
    { 0x85c1047, SUC_lexicon_tags_1 },
    { 0xe8dc104a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c104d, SUC_lexicon_tags_11 },
    { 0x68dc104d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xadfc1050, SUC_lexicon_tags_8 },
    { 0x813c1051, SUC_lexicon_tags_8 },
    { 0x4fc1052, SUC_lexicon_tags_7 },
    { 0xbf5c1053, SUC_lexicon_tags_11 },
    { 0xd9dc1054, SUC_lexicon_tags_7 },
    { 0x7abc1055, SUC_lexicon_tags_27 },
    { 0x905c1055, SUC_lexicon_tags_1 },
    { 0x2fdc1057, SUC_lexicon_tags_62 },
    { 0xe7fc1057, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1dc105a, SUC_lexicon_tags_34 },
    { 0x50fc105a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bc105e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29c1061, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8fc1063, SUC_lexicon_tags_31 },
    { 0x5e7c1064, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77c106c, SUC_lexicon_tags_14 },
    { 0x21dc106d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc1073, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309c1079, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427c107d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbc1080, SUC_lexicon_tags_12 },
    { 0x4efc1081, SUC_lexicon_tags_0 },
    { 0x3d5c1080, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa7c1084, SUC_lexicon_tags_17 },
    { 0xd51c1085, SUC_lexicon_tags_11 },
    { 0xda3c1085, SUC_lexicon_tags_17 },
    { 0xd9bc1087, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb0dc1089, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7c1091, SUC_lexicon_tags_19 },
    { 0xc0fc1092, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x97dc1094, SUC_lexicon_tags_13 },
    { 0x8cfc1095, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497c109c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffc10a4, SUC_lexicon_tags_30 },
    { 0xec3c10a4, SUC_lexicon_tags_2 },
    { 0xb97c10a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x217c10a8, SUC_lexicon_tags_14 },
    { 0xf69c10a9, SUC_lexicon_tags_15 },
    { 0x6bbc10aa, SUC_lexicon_tags_37 },
    { 0xc7bc10ab, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc10b0, SUC_lexicon_tags_17 },
    { 0xc5c10b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x999c10b3, SUC_lexicon_tags_83 },
    { 0xa5dc10b4, SUC_lexicon_tags_37 },
    { 0xeb7c10b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x183c10b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x549c10b9, SUC_lexicon_tags_10 },
    { 0x665c10ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60dc10bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc53c10c1, SUC_lexicon_tags_2 },
    { 0x7b3c10c2, SUC_lexicon_tags_25 },
    { 0xca3c10c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bc10c7, SUC_lexicon_tags_30 },
    { 0x63dc10c8, SUC_lexicon_tags_17 },
    { 0xe0bc10c7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88dc10ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x667c10d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95c10d9, SUC_lexicon_tags_63 },
    { 0x819c10d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa21c10dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43c10e0, SUC_lexicon_tags_45 },
    { 0xfdc10e1, SUC_lexicon_tags_47 },
    { 0xcebc10e2, SUC_lexicon_tags_17 },
    { 0x947c10e1, SUC_lexicon_tags_1 },
    { 0xb1bc10e0, SUC_lexicon_tags_23 },
    { 0x38dc10e3, SUC_lexicon_tags_77 },
    { 0x4dbc10e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1bc10e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9c10ee, SUC_lexicon_tags_2 },
    { 0xa81c10ef, SUC_lexicon_tags_60 },
    { 0xb09c10ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fc10f6, SUC_lexicon_tags_16 },
    { 0x8b3c10f6, SUC_lexicon_tags_14 },
    { 0x6d1c10f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbc10fb, SUC_lexicon_tags_92 },
    { 0xd9c10fb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabc1104, SUC_lexicon_tags_2 },
    { 0xbb1c1105, SUC_lexicon_tags_10 },
    { 0x7f3c1106, SUC_lexicon_tags_68 },
    { 0x9cfc1107, SUC_lexicon_tags_2 },
    { 0xb39c1104, SUC_lexicon_tags_47 },
    { 0xf2dc1109, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc110c, SUC_lexicon_tags_16 },
    { 0x977c110d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dc1111, SUC_lexicon_tags_37 },
    { 0x591c1112, SUC_lexicon_tags_19 },
    { 0xb6dc1113, SUC_lexicon_tags_30 },
    { 0xdb1c1111, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323c1118, SUC_lexicon_tags_11 },
    { 0xe1fc1119, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcabc111b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafc1120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13c1124, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x711c1126, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58bc1128, SUC_lexicon_tags_25 },
    { 0xf25c1129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x109c112c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205c1130, SUC_lexicon_tags_1 },
    { 0x18bc1130, SUC_lexicon_tags_34 },
    { 0xc0dc1132, SUC_lexicon_tags_15 },
    { 0x18dc1133, SUC_lexicon_tags_12 },
    { 0xe47c1130, SUC_lexicon_tags_42 },
    { 0x673c1135, SUC_lexicon_tags_37 },
    { 0x755c1130, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7c1139, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fc1141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbc1144, SUC_lexicon_tags_1370 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537c114b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911c114f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fc1152, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c1158, SUC_lexicon_tags_21 },
    { 0x983c1158, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbc115e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x499c1160, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x355c1163, SUC_lexicon_tags_12 },
    { 0x917c1163, SUC_lexicon_tags_37 },
    { 0xcbfc1164, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7c1168, SUC_lexicon_tags_45 },
    { 0x319c1169, SUC_lexicon_tags_47 },
    { 0xa55c116a, SUC_lexicon_tags_11 },
    { 0xb7c1168, SUC_lexicon_tags_55 },
    { 0x3a3c116c, SUC_lexicon_tags_33 },
    { 0x813c116d, SUC_lexicon_tags_14 },
    { 0xafbc116d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x583c1170, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82dc1173, SUC_lexicon_tags_15 },
    { 0x16dc1174, SUC_lexicon_tags_6 },
    { 0x469c1175, SUC_lexicon_tags_13 },
    { 0x5b3c1176, SUC_lexicon_tags_34 },
    { 0x91fc1177, SUC_lexicon_tags_17 },
    { 0xc5dc1173, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x511c117a, SUC_lexicon_tags_23 },
    { 0x5cfc117b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x209c117d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9c1181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fc1184, SUC_lexicon_tags_19 },
    { 0xd23c1185, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dc1189, SUC_lexicon_tags_63 },
    { 0x715c118a, SUC_lexicon_tags_14 },
    { 0xbd5c118b, SUC_lexicon_tags_31 },
    { 0x8e3c118c, SUC_lexicon_tags_11 },
    { 0xdc3c118c, SUC_lexicon_tags_68 },
    { 0xd61c118e, SUC_lexicon_tags_12 },
    { 0xaf9c118e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1c1198, SUC_lexicon_tags_25 },
    { 0x45dc1199, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bc119f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889c11a3, SUC_lexicon_tags_15 },
    { 0xbd9c11a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdffc11a6, SUC_lexicon_tags_15 },
    { 0xc67c11a6, SUC_lexicon_tags_33 },
    { 0xebbc11a8, SUC_lexicon_tags_32 },
    { 0x53dc11a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbc11ac, SUC_lexicon_tags_19 },
    { 0xf3dc11ac, SUC_lexicon_tags_2 },
    { 0x6bdc11ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x807c11b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1c11b3, SUC_lexicon_tags_17 },
    { 0xebdc11b3, SUC_lexicon_tags_2 },
    { 0xf3fc11b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3c11b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbcfc11ba, SUC_lexicon_tags_37 },
    { 0xbcbc11bb, SUC_lexicon_tags_15 },
    { 0xd1bc11bc, SUC_lexicon_tags_0 },
    { 0xb5c11ba, SUC_lexicon_tags_6 },
    { 0x435c11be, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe3c11c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37bc11c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869c11c7, SUC_lexicon_tags_27 },
    { 0x463c11c8, SUC_lexicon_tags_8 },
    { 0x891c11c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3c11cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fc11d0, SUC_lexicon_tags_31 },
    { 0x6edc11d1, SUC_lexicon_tags_1 },
    { 0xcbdc11d1, SUC_lexicon_tags_0 },
    { 0x601c11d1, SUC_lexicon_tags_63 },
    { 0x889c11d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x541c11d6, SUC_lexicon_tags_30 },
    { 0x585c11d7, SUC_lexicon_tags_12 },
    { 0x2dbc11d8, SUC_lexicon_tags_31 },
    { 0x4f9c11d9, SUC_lexicon_tags_31 },
    { 0x5f7c11d9, SUC_lexicon_tags_40 },
    { 0x5efc11db, SUC_lexicon_tags_21 },
    { 0x269c11db, SUC_lexicon_tags_2 },
    { 0xb89c11dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x203c11e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadbc11e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x739c11e4, SUC_lexicon_tags_17 },
    { 0x2bdc11e5, SUC_lexicon_tags_17 },
    { 0xc69c11e5, SUC_lexicon_tags_37 },
    { 0x35dc11e7, SUC_lexicon_tags_26 },
    { 0x285c11e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fc11eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1c11ee, SUC_lexicon_tags_19 },
    { 0x161c11ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dc11f3, SUC_lexicon_tags_1 },
    { 0x4bc11f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc79c11f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dc11fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fc11ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c1204, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xf5bc1206, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c1209, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bc120c, SUC_lexicon_tags_15 },
    { 0x801c120d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bc1211, SUC_lexicon_tags_37 },
    { 0x413c1211, SUC_lexicon_tags_11 },
    { 0xd2dc1213, SUC_lexicon_tags_10 },
    { 0x6f9c1214, SUC_lexicon_tags_7 },
    { 0xf23c1215, SUC_lexicon_tags_19 },
    { 0xa67c1215, SUC_lexicon_tags_7 },
    { 0x5c5c1217, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfc121d, SUC_lexicon_tags_7 },
    { 0x2d5c121e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bc1222, SUC_lexicon_tags_17 },
    { 0xbf5c1222, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5f3c1225, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x89dc1228, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x291c122b, SUC_lexicon_tags_8 },
    { 0x125c122c, SUC_lexicon_tags_45 },
    { 0xb87c122d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6fc122f, SUC_lexicon_tags_0 },
    { 0x197c1230, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe33c1232, SUC_lexicon_tags_7 },
    { 0x7ffc1233, SUC_lexicon_tags_27 },
    { 0xa8fc1233, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c123b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x423c123d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325c1245, SUC_lexicon_tags_7 },
    { 0xcc5c1246, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x221c1249, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf13c124b, SUC_lexicon_tags_7 },
    { 0xc73c124c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149c1251, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227c1257, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd67c1259, SUC_lexicon_tags_37 },
    { 0xfc7c125a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51dc125c, SUC_lexicon_tags_45 },
    { 0xfdfc125c, SUC_lexicon_tags_12 },
    { 0x8d5c125e, SUC_lexicon_tags_37 },
    { 0x385c125f, SUC_lexicon_tags_11 },
    { 0xc21c1260, SUC_lexicon_tags_61 },
    { 0x8b1c125f, SUC_lexicon_tags_15 },
    { 0x56fc1262, SUC_lexicon_tags_2 },
    { 0xebdc1263, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeefc1265, SUC_lexicon_tags_25 },
    { 0x8dbc1266, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc1269, SUC_lexicon_tags_25 },
    { 0x2afc126a, SUC_lexicon_tags_14 },
    { 0xf23c126a, SUC_lexicon_tags_45 },
    { 0xfe7c126c, SUC_lexicon_tags_43 },
    { 0xf79c126d, SUC_lexicon_tags_38 },
    { 0xeb3c126c, SUC_lexicon_tags_23 },
    { 0x941c126c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bc1272, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271c127c, SUC_lexicon_tags_12 },
    { 0xd03c127d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fc1287, SUC_lexicon_tags_11 },
    { 0x6b3c1288, SUC_lexicon_tags_17 },
    { 0xfafc1289, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x36dc128c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x25fc128e, SUC_lexicon_tags_31 },
    { 0x72fc128f, SUC_lexicon_tags_11 },
    { 0x13bc128f, SUC_lexicon_tags_158 },
    { 0x657c1291, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71c1294, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fc1297, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c129b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc3c129d, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffc12a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99c12a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35c12af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfabc12b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487c12b5, SUC_lexicon_tags_27 },
    { 0x405c12b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbbdc12b8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3c12bc, SUC_lexicon_tags_17 },
    { 0x8dbc12bd, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddc12c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fc12c7, SUC_lexicon_tags_25 },
    { 0xf43c12c8, SUC_lexicon_tags_26 },
    { 0x4e9c12c7, SUC_lexicon_tags_60 },
    { 0xad7c12ca, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9c12cf, SUC_lexicon_tags_1 },
    { 0xd59c12d0, SUC_lexicon_tags_25 },
    { 0x455c12d1, SUC_lexicon_tags_17 },
    { 0xdc1c12d1, SUC_lexicon_tags_27 },
    { 0x1f1c12d3, SUC_lexicon_tags_424 },
    { 0xe47c12d4, SUC_lexicon_tags_14 },
    { 0x213c12d5, SUC_lexicon_tags_49 },
    { 0x607c12d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85c12da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc12dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcebc12df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fc12e7, SUC_lexicon_tags_28 },
    { 0x3b7c12e8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9f3c12ea, SUC_lexicon_tags_15 },
    { 0xccdc12ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x753c12ee, SUC_lexicon_tags_38 },
    { 0x1a9c12ef, SUC_lexicon_tags_87 },
    { 0xa4bc12f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc41c12f2, SUC_lexicon_tags_37 },
    { 0x49bc12f2, SUC_lexicon_tags_11 },
    { 0x9e7c12f4, SUC_lexicon_tags_31 },
    { 0x9c7c12f4, SUC_lexicon_tags_106 },
    { 0x29c12f6, SUC_lexicon_tags_138 },
    { 0xb5fc12f7, SUC_lexicon_tags_14 },
    { 0x2e3c12f6, SUC_lexicon_tags_15 },
    { 0x801c12f9, SUC_lexicon_tags_37 },
    { 0x1b5c12f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbc12ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe17c1301, SUC_lexicon_tags_47 },
    { 0xd5c1301, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43c130a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fc130f, SUC_lexicon_tags_29 },
    { 0x741c130f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49c1318, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299c131e, SUC_lexicon_tags_12 },
    { 0xfedc131f, SUC_lexicon_tags_32 },
    { 0x86fc1320, SUC_lexicon_tags_25 },
    { 0x96bc1321, SUC_lexicon_tags_5 },
    { 0x61dc131f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bc132a, SUC_lexicon_tags_47 },
    { 0x87dc132a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fc1331, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x677c1333, SUC_lexicon_tags_14 },
    { 0xddfc1334, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9c133a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fc1341, SUC_lexicon_tags_38 },
    { 0xc91c1342, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25dc1346, SUC_lexicon_tags_25 },
    { 0x325c1346, SUC_lexicon_tags_2 },
    { 0xc0fc1348, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7c134b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7c134e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74dc1350, SUC_lexicon_tags_15 },
    { 0x173c1350, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdd3c1353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845c1358, SUC_lexicon_tags_2 },
    { 0xc55c1359, SUC_lexicon_tags_14 },
    { 0x809c1358, SUC_lexicon_tags_37 },
    { 0xd1c135b, SUC_lexicon_tags_25 },
    { 0x5f9c135c, SUC_lexicon_tags_37 },
    { 0xa0fc135d, SUC_lexicon_tags_171 },
    { 0x40dc135c, SUC_lexicon_tags_14 },
    { 0x175c1358, SUC_lexicon_tags_10 },
    { 0x9b5c1360, SUC_lexicon_tags_56 },
    { 0x8dc1361, SUC_lexicon_tags_37 },
    { 0x2cdc135b, SUC_lexicon_tags_43 },
    { 0x717c1363, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf55c1365, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x889c1368, SUC_lexicon_tags_25 },
    { 0xa1dc1369, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c136c, SUC_lexicon_tags_39 },
    { 0x3f9c136c, SUC_lexicon_tags_7 },
    { 0xe67c136e, SUC_lexicon_tags_2 },
    { 0xadbc136c, SUC_lexicon_tags_10 },
    { 0x4e5c136c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfc1374, SUC_lexicon_tags_15 },
    { 0xb11c1375, SUC_lexicon_tags_2 },
    { 0x39c1374, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fc1379, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x89dc137b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c7c137d, SUC_lexicon_tags_43 },
    { 0x1a9c137d, SUC_lexicon_tags_65 },
    { 0xd2dc137f, SUC_lexicon_tags_27 },
    { 0x83dc137f, SUC_lexicon_tags_1 },
    { 0x20dc1381, SUC_lexicon_tags_0 },
    { 0xdc1382, SUC_lexicon_tags_26 },
    { 0x85c1383, SUC_lexicon_tags_108 },
    { 0xf87c137f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bc138e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x629c1391, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dc1395, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7c1399, SUC_lexicon_tags_23 },
    { 0x259c1399, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2efc139c, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x527c139f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x991c13a1, SUC_lexicon_tags_107 },
    { 0x941c13a2, SUC_lexicon_tags_349 },
    { 0xcf1c13a2, SUC_lexicon_tags_2 },
    { 0xefdc13a3, SUC_lexicon_tags_45 },
    { 0x661c13a5, SUC_lexicon_tags_31 },
    { 0x97c13a1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbc13ab, SUC_lexicon_tags_14 },
    { 0x6fc13ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xacfc13ae, SUC_lexicon_tags_1 },
    { 0x26bc13af, SUC_lexicon_tags_11 },
    { 0xaafc13b0, SUC_lexicon_tags_12 },
    { 0xc75c13b1, SUC_lexicon_tags_65 },
    { 0x45bc13b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb1fc13b4, SUC_lexicon_tags_13 },
    { 0xdb1c13b5, SUC_lexicon_tags_30 },
    { 0x459c13b6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x29dc13b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21c13bc, SUC_lexicon_tags_10 },
    { 0xdc9c13bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdc13c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c13c7, SUC_lexicon_tags_11 },
    { 0x50bc13c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331c13cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae1c13d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bc13d5, SUC_lexicon_tags_31 },
    { 0x89dc13d5, SUC_lexicon_tags_25 },
    { 0x73c13d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bc13e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fc13e4, SUC_lexicon_tags_2 },
    { 0xe5dc13e5, SUC_lexicon_tags_12 },
    { 0xc6fc13e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe8dc13e8, SUC_lexicon_tags_4 },
    { 0x885c13e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dc13ed, SUC_lexicon_tags_25 },
    { 0x643c13ed, SUC_lexicon_tags_66 },
    { 0xe9bc13ee, SUC_lexicon_tags_12 },
    { 0x901c13f0, SUC_lexicon_tags_38 },
    { 0x3c3c13f1, SUC_lexicon_tags_37 },
    { 0xe81c13ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xae3c13f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x605c13f6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x251c13f8, SUC_lexicon_tags_17 },
    { 0x799c13f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bc13fc, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85c1401, SUC_lexicon_tags_46 },
    { 0xdb9c1402, SUC_lexicon_tags_30 },
    { 0x635c1403, SUC_lexicon_tags_37 },
    { 0xdefc1401, SUC_lexicon_tags_14 },
    { 0x6b7c1405, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x993c1408, SUC_lexicon_tags_138 },
    { 0xb9c1408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153c1411, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5dc1413, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb1fc1415, SUC_lexicon_tags_17 },
    { 0x2d1c1416, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f7c1418, SUC_lexicon_tags_25 },
    { 0xefc1418, SUC_lexicon_tags_6 },
    { 0x985c1418, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c9c141c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14dc141e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x809c1420, SUC_lexicon_tags_84 },
    { 0x4d3c1421, SUC_lexicon_tags_6 },
    { 0x95bc1422, SUC_lexicon_tags_68 },
    { 0xa17c1420, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581c1428, SUC_lexicon_tags_19 },
    { 0xc3fc1429, SUC_lexicon_tags_25 },
    { 0xe91c142a, SUC_lexicon_tags_6 },
    { 0x101c142b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5c142e, SUC_lexicon_tags_12 },
    { 0xe8dc142f, SUC_lexicon_tags_66 },
    { 0xc7fc1430, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183c1435, SUC_lexicon_tags_17 },
    { 0x141c1436, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fc1442, SUC_lexicon_tags_9 },
    { 0xf7bc1443, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xec3c1445, SUC_lexicon_tags_4 },
    { 0xa57c1446, SUC_lexicon_tags_28 },
    { 0x35bc1447, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bc144a, SUC_lexicon_tags_68 },
    { 0xf51c144b, SUC_lexicon_tags_15 },
    { 0x3e1c144b, SUC_lexicon_tags_7 },
    { 0x11fc144b, SUC_lexicon_tags_1 },
    { 0x62fc144c, SUC_lexicon_tags_17 },
    { 0x439c144f, SUC_lexicon_tags_15 },
    { 0xa45c144f, SUC_lexicon_tags_410 },
    { 0xd7fc1450, SUC_lexicon_tags_17 },
    { 0xa41c1452, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5c1457, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1c145b, SUC_lexicon_tags_17 },
    { 0x393c145c, SUC_lexicon_tags_25 },
    { 0xaedc145c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc1463, SUC_lexicon_tags_36 },
    { 0xe99c1464, SUC_lexicon_tags_45 },
    { 0xbedc1465, SUC_lexicon_tags_17 },
    { 0x9d9c1466, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddc146a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbc146d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27c1471, SUC_lexicon_tags_17 },
    { 0x77c1472, SUC_lexicon_tags_12 },
    { 0x155c1473, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1c1476, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x623c1479, SUC_lexicon_tags_37 },
    { 0x3f1c147a, SUC_lexicon_tags_57 },
    { 0x2b7c147b, SUC_lexicon_tags_60 },
    { 0x5a5c1479, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5c147f, SUC_lexicon_tags_68 },
    { 0x91fc1480, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249c1486, SUC_lexicon_tags_4 },
    { 0x4efc1487, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afc148b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5bc148d, SUC_lexicon_tags_34 },
    { 0x2ffc148d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fc1494, SUC_lexicon_tags_37 },
    { 0x31fc1495, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x101c1497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9c149b, SUC_lexicon_tags_14 },
    { 0x83dc149c, SUC_lexicon_tags_37 },
    { 0x643c149d, SUC_lexicon_tags_73 },
    { 0xa99c149e, SUC_lexicon_tags_17 },
    { 0xfb5c149f, SUC_lexicon_tags_37 },
    { 0x177c14a0, SUC_lexicon_tags_1 },
    { 0x8f7c149e, SUC_lexicon_tags_25 },
    { 0xbcdc14a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x121c14a5, SUC_lexicon_tags_10 },
    { 0xcffc14a6, SUC_lexicon_tags_2 },
    { 0xc5c14a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x793c14a9, SUC_lexicon_tags_19 },
    { 0xf0fc14aa, SUC_lexicon_tags_25 },
    { 0x3edc14aa, SUC_lexicon_tags_87 },
    { 0x311c14ab, SUC_lexicon_tags_17 },
    { 0x931c14ad, SUC_lexicon_tags_0 },
    { 0x61bc14ad, SUC_lexicon_tags_45 },
    { 0x50fc14ab, SUC_lexicon_tags_17 },
    { 0x8bbc14b0, SUC_lexicon_tags_29 },
    { 0x27dc14aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7b5c14b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c14ba, SUC_lexicon_tags_4 },
    { 0x365c14ba, SUC_lexicon_tags_55 },
    { 0x8d1c14ba, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x497c14be, SUC_lexicon_tags_0 },
    { 0x6b1c14bf, SUC_lexicon_tags_2 },
    { 0x3a3c14c0, SUC_lexicon_tags_15 },
    { 0x427c14c1, SUC_lexicon_tags_129 },
    { 0xc73c14c2, SUC_lexicon_tags_27 },
    { 0x7a7c14c3, SUC_lexicon_tags_77 },
    { 0x6b5c14c4, SUC_lexicon_tags_8 },
    { 0xa83c14c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3c14cc, SUC_lexicon_tags_15 },
    { 0x10fc14cd, SUC_lexicon_tags_10 },
    { 0x847c14cc, SUC_lexicon_tags_47 },
    { 0xd03c14cf, SUC_lexicon_tags_17 },
    { 0x8d5c14d0, SUC_lexicon_tags_9 },
    { 0xaa9c14d0, SUC_lexicon_tags_1 },
    { 0x417c14d2, SUC_lexicon_tags_2 },
    { 0x335c14cc, SUC_lexicon_tags_11 },
    { 0x417c14d0, SUC_lexicon_tags_15 },
    { 0x3efc14ce, SUC_lexicon_tags_2 },
    { 0xaafc14d1, SUC_lexicon_tags_30 },
    { 0x95dc14d7, SUC_lexicon_tags_31 },
    { 0x921c14d6, SUC_lexicon_tags_22 },
    { 0x53fc14d9, SUC_lexicon_tags_1 },
    { 0x62dc14da, SUC_lexicon_tags_0 },
    { 0x63bc14da, SUC_lexicon_tags_49 },
    { 0x171c14dc, SUC_lexicon_tags_22 },
    { 0xa59c14da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad7c14df, SUC_lexicon_tags_26 },
    { 0x727c14df, SUC_lexicon_tags_21 },
    { 0x1fc14e1, SUC_lexicon_tags_7 },
    { 0xd15c14e2, SUC_lexicon_tags_17 },
    { 0xe71c14e1, SUC_lexicon_tags_15 },
    { 0xe53c14e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1c14e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37c14ed, SUC_lexicon_tags_17 },
    { 0xbfc14ee, SUC_lexicon_tags_28 },
    { 0x897c14ef, SUC_lexicon_tags_15 },
    { 0x971c14f0, SUC_lexicon_tags_11 },
    { 0x34dc14f1, SUC_lexicon_tags_9 },
    { 0xac1c14ed, SUC_lexicon_tags_11 },
    { 0x2e7c14f3, SUC_lexicon_tags_63 },
    { 0x8cdc14f3, SUC_lexicon_tags_30 },
    { 0xa8dc14f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33c14f9, SUC_lexicon_tags_31 },
    { 0x517c14f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9c1500, SUC_lexicon_tags_37 },
    { 0x327c1501, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41c1505, SUC_lexicon_tags_9 },
    { 0xd5dc1506, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdb7c1508, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bc150e, SUC_lexicon_tags_25 },
    { 0x863c150f, SUC_lexicon_tags_2 },
    { 0xbf1c150e, SUC_lexicon_tags_27 },
    { 0x9c5c1511, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9c1515, SUC_lexicon_tags_11 },
    { 0x34dc1516, SUC_lexicon_tags_12 },
    { 0x219c1515, SUC_lexicon_tags_10 },
    { 0x719c1518, SUC_lexicon_tags_37 },
    { 0xe7dc1519, SUC_lexicon_tags_9 },
    { 0xf95c1515, SUC_lexicon_tags_17 },
    { 0x8b5c151b, SUC_lexicon_tags_60 },
    { 0xc65c151c, SUC_lexicon_tags_63 },
    { 0xb4fc1515, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc1520, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x879c1523, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97c152a, SUC_lexicon_tags_23 },
    { 0x6cbc152a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdc1535, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa37c1537, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13c153f, SUC_lexicon_tags_8 },
    { 0x33dc1540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c1543, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x303c1546, SUC_lexicon_tags_47 },
    { 0xd89c1547, SUC_lexicon_tags_25 },
    { 0x919c1548, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x545c154a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x525c154c, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19c154f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbc1555, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x977c1557, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81c1559, SUC_lexicon_tags_1 },
    { 0x6a7c1559, SUC_lexicon_tags_63 },
    { 0xc6dc155b, SUC_lexicon_tags_60 },
    { 0xfefc155c, SUC_lexicon_tags_19 },
    { 0xe4dc155d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25c1564, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xadfc1567, SUC_lexicon_tags_2 },
    { 0x4bc1568, SUC_lexicon_tags_77 },
    { 0x525c1567, SUC_lexicon_tags_43 },
    { 0xc93c1567, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dc156e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1c1571, SUC_lexicon_tags_1 },
    { 0x7d3c1572, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7dc1574, SUC_lexicon_tags_27 },
    { 0xdfbc1574, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd25c1577, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e9c1579, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bc157e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1c1582, SUC_lexicon_tags_30 },
    { 0xd15c1583, SUC_lexicon_tags_37 },
    { 0x737c1583, SUC_lexicon_tags_27 },
    { 0x1c1585, SUC_lexicon_tags_25 },
    { 0xa1fc1586, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37c1589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9c158c, SUC_lexicon_tags_19 },
    { 0xe75c158d, SUC_lexicon_tags_33 },
    { 0xf39c158d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88fc1590, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3c159c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51c15a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x609c15a2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611c15a6, SUC_lexicon_tags_37 },
    { 0x9c1c15a7, SUC_lexicon_tags_30 },
    { 0x7afc15a8, SUC_lexicon_tags_7 },
    { 0x13dc15a9, SUC_lexicon_tags_11 },
    { 0xabc15a6, SUC_lexicon_tags_201 },
    { 0xe1fc15ab, SUC_lexicon_tags_17 },
    { 0xbdfc15ac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc15b0, SUC_lexicon_tags_17 },
    { 0xa5dc15b1, SUC_lexicon_tags_2 },
    { 0xe85c15b2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x23c15b5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xfc9c15b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07c15ba, SUC_lexicon_tags_31 },
    { 0x84bc15bb, SUC_lexicon_tags_30 },
    { 0x127c15ba, SUC_lexicon_tags_15 },
    { 0x9fdc15ba, SUC_lexicon_tags_7 },
    { 0xb7dc15be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdc15c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9c15cb, SUC_lexicon_tags_63 },
    { 0x99fc15cb, SUC_lexicon_tags_37 },
    { 0x5f7c15cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dc15d1, SUC_lexicon_tags_6 },
    { 0x72dc15d2, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9c15d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17c15dc, SUC_lexicon_tags_2 },
    { 0xd3dc15dd, SUC_lexicon_tags_25 },
    { 0x4e5c15de, SUC_lexicon_tags_25 },
    { 0x319c15df, SUC_lexicon_tags_47 },
    { 0x2ffc15de, SUC_lexicon_tags_26 },
    { 0x977c15e1, SUC_lexicon_tags_63 },
    { 0x4d5c15e2, SUC_lexicon_tags_2 },
    { 0xcd1c15de, SUC_lexicon_tags_11 },
    { 0xcb1c15e2, SUC_lexicon_tags_43 },
    { 0xb0fc15dc, SUC_lexicon_tags_11 },
    { 0x671c15e0, SUC_lexicon_tags_15 },
    { 0xd1bc15dd, SUC_lexicon_tags_2 },
    { 0xc91c15e0, SUC_lexicon_tags_15 },
    { 0xc6bc15e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdc15ee, SUC_lexicon_tags_35 },
    { 0xd13c15ef, SUC_lexicon_tags_2 },
    { 0x2d7c15f0, SUC_lexicon_tags_38 },
    { 0x903c15f1, SUC_lexicon_tags_62 },
    { 0x2c7c15ef, SUC_lexicon_tags_38 },
    { 0xfdfc15ee, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55c15f6, SUC_lexicon_tags_29 },
    { 0xaf5c15f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fc15fa, SUC_lexicon_tags_23 },
    { 0x9e3c15fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bc1601, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x68bc1603, SUC_lexicon_tags_12 },
    { 0xf5bc1604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfc1608, SUC_lexicon_tags_0 },
    { 0x613c1609, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f5c160b, SUC_lexicon_tags_46 },
    { 0x315c160b, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xb53c160e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x829c1610, SUC_lexicon_tags_14 },
    { 0x9bdc1611, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dc1614, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e3c1616, SUC_lexicon_tags_26 },
    { 0xd7dc1616, SUC_lexicon_tags_19 },
    { 0xd9bc1618, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdc161b, SUC_lexicon_tags_7 },
    { 0x837c161c, SUC_lexicon_tags_37 },
    { 0x737c161c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7c1620, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x29c1622, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677c1628, SUC_lexicon_tags_11 },
    { 0xa47c1628, SUC_lexicon_tags_17 },
    { 0x747c162a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fc1630, SUC_lexicon_tags_30 },
    { 0xf4bc1631, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x493c1634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19c163d, SUC_lexicon_tags_60 },
    { 0x653c163e, SUC_lexicon_tags_45 },
    { 0x3ddc163d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4fc1641, SUC_lexicon_tags_11 },
    { 0x2bdc1641, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbc1646, SUC_lexicon_tags_71 },
    { 0xf05c1647, SUC_lexicon_tags_1 },
    { 0x5e3c1648, SUC_lexicon_tags_7 },
    { 0xc53c1646, SUC_lexicon_tags_37 },
    { 0xc15c164a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc164f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45dc1651, SUC_lexicon_tags_47 },
    { 0x12dc1651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57c1655, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67c165a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fc165f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c1665, SUC_lexicon_tags_28 },
    { 0xd17c1666, SUC_lexicon_tags_47 },
    { 0x4fbc1665, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc166a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffc166e, SUC_lexicon_tags_12 },
    { 0x99fc166e, SUC_lexicon_tags_27 },
    { 0xa69c1670, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687c1678, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23c167c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x841c167e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7c168a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x887c168c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229c1692, SUC_lexicon_tags_19 },
    { 0x8d9c1693, SUC_lexicon_tags_25 },
    { 0x7b1c1694, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdf1c1696, SUC_lexicon_tags_61 },
    { 0x2b5c1697, SUC_lexicon_tags_27 },
    { 0xb1bc1698, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dc169d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebc16a2, SUC_lexicon_tags_31 },
    { 0xa99c16a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe03c16a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x329c16a8, SUC_lexicon_tags_30 },
    { 0xe7dc16a9, SUC_lexicon_tags_9 },
    { 0x303c16aa, SUC_lexicon_tags_38 },
    { 0x1bc16ab, SUC_lexicon_tags_25 },
    { 0x3a5c16aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdc16b3, SUC_lexicon_tags_55 },
    { 0xb6fc16b4, SUC_lexicon_tags_29 },
    { 0x265c16b4, SUC_lexicon_tags_37 },
    { 0xbe3c16b6, SUC_lexicon_tags_63 },
    { 0xf6dc16b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abc16ba, SUC_lexicon_tags_12 },
    { 0x8a5c16bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dc16c3, SUC_lexicon_tags_17 },
    { 0x779c16c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bc16c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1c16cb, SUC_lexicon_tags_12 },
    { 0x30fc16cc, SUC_lexicon_tags_33 },
    { 0xbc9c16cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdcfc16cf, SUC_lexicon_tags_11 },
    { 0x7fc16cf, SUC_lexicon_tags_47 },
    { 0xfd1c16d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c16d5, SUC_lexicon_tags_26 },
    { 0x5f9c16d6, SUC_lexicon_tags_3 },
    { 0x2adc16d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3efc16d9, SUC_lexicon_tags_15 },
    { 0xbf3c16da, SUC_lexicon_tags_34 },
    { 0xfb5c16da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853c16e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x821c16e2, SUC_lexicon_tags_47 },
    { 0xfa5c16e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadc16e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbc16ea, SUC_lexicon_tags_37 },
    { 0x5edc16ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb99c16ed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8cbc16ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5c16f7, SUC_lexicon_tags_12 },
    { 0x4bbc16f8, SUC_lexicon_tags_45 },
    { 0xde7c16f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93c16fb, SUC_lexicon_tags_37 },
    { 0x817c16fc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbc16ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bc1705, SUC_lexicon_tags_0 },
    { 0xb0bc1706, SUC_lexicon_tags_49 },
    { 0x9fdc1706, SUC_lexicon_tags_455 },
    { 0x585c1708, SUC_lexicon_tags_47 },
    { 0x8cdc1709, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5c170d, SUC_lexicon_tags_11 },
    { 0xce1c170e, SUC_lexicon_tags_32 },
    { 0x5bfc170e, SUC_lexicon_tags_37 },
    { 0x91fc1710, SUC_lexicon_tags_27 },
    { 0x26bc1711, SUC_lexicon_tags_15 },
    { 0x6b7c1712, SUC_lexicon_tags_2 },
    { 0x7e1c1713, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111c1719, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc1722, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f7c1724, SUC_lexicon_tags_129 },
    { 0xa83c1725, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dc1729, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x255c172c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dc1730, SUC_lexicon_tags_38 },
    { 0x7afc1730, SUC_lexicon_tags_132 },
    { 0, NULL },
    { 0x83dc1733, SUC_lexicon_tags_11 },
    { 0x143c1734, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07c173c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x57fc173e, SUC_lexicon_tags_25 },
    { 0x20dc173e, SUC_lexicon_tags_62 },
    { 0x67fc1740, SUC_lexicon_tags_11 },
    { 0x3f1c1741, SUC_lexicon_tags_492 },
    { 0x347c1740, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc87c1744, SUC_lexicon_tags_49 },
    { 0xe73c1744, SUC_lexicon_tags_17 },
    { 0xe23c1744, SUC_lexicon_tags_7 },
    { 0x687c1747, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabc1749, SUC_lexicon_tags_8 },
    { 0x9f9c174a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5c174d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dc1752, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x475c1754, SUC_lexicon_tags_25 },
    { 0x531c1755, SUC_lexicon_tags_60 },
    { 0x20bc1754, SUC_lexicon_tags_84 },
    { 0x78dc1757, SUC_lexicon_tags_63 },
    { 0x4edc1757, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973c175c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x873c175e, SUC_lexicon_tags_2 },
    { 0x515c175f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc0bc1761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbfc1763, SUC_lexicon_tags_17 },
    { 0x33dc1764, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe4dc1766, SUC_lexicon_tags_2 },
    { 0x7b5c1766, SUC_lexicon_tags_8 },
    { 0xcdfc1768, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329c176e, SUC_lexicon_tags_25 },
    { 0xf6bc176f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x589c1771, SUC_lexicon_tags_21 },
    { 0x683c1772, SUC_lexicon_tags_26 },
    { 0x8b1c1773, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dc177b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbc177e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x775c1780, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdc179b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997c17a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3c17aa, SUC_lexicon_tags_37 },
    { 0xb8dc17aa, SUC_lexicon_tags_37 },
    { 0x76dc17aa, SUC_lexicon_tags_12 },
    { 0xb6bc17ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5c17b1, SUC_lexicon_tags_33 },
    { 0xc2bc17b1, SUC_lexicon_tags_7 },
    { 0xdf7c17b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3c17b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65c17bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9dc17c1, SUC_lexicon_tags_7 },
    { 0xe05c17c1, SUC_lexicon_tags_65 },
    { 0x871c17c3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x897c17c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbc17cb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7c17ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x889c17d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xea7c17d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdc17d8, SUC_lexicon_tags_2 },
    { 0x85dc17d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3c17de, SUC_lexicon_tags_16 },
    { 0x1e9c17df, SUC_lexicon_tags_133 },
    { 0x707c17e0, SUC_lexicon_tags_25 },
    { 0xbebc17e0, SUC_lexicon_tags_23 },
    { 0x421c17e2, SUC_lexicon_tags_15 },
    { 0xee1c17e2, SUC_lexicon_tags_27 },
    { 0x9ebc17df, SUC_lexicon_tags_37 },
    { 0xef3c17e1, SUC_lexicon_tags_2 },
    { 0xfa1c17e6, SUC_lexicon_tags_15 },
    { 0x8f1c17e6, SUC_lexicon_tags_26 },
    { 0x417c17e6, SUC_lexicon_tags_25 },
    { 0xb31c17e4, SUC_lexicon_tags_2 },
    { 0x9f9c17ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe53c17ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdc17ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5c17f2, SUC_lexicon_tags_2 },
    { 0xe63c17f2, SUC_lexicon_tags_36 },
    { 0x8c7c17f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fc17f7, SUC_lexicon_tags_5 },
    { 0xcebc17f8, SUC_lexicon_tags_60 },
    { 0xe7c17f7, SUC_lexicon_tags_28 },
    { 0x553c17f7, SUC_lexicon_tags_17 },
    { 0x4a3c17fa, SUC_lexicon_tags_17 },
    { 0x50fc17f9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1c17ff, SUC_lexicon_tags_37 },
    { 0x317c1800, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569c1805, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7c1808, SUC_lexicon_tags_37 },
    { 0xe81c1809, SUC_lexicon_tags_45 },
    { 0x96dc180a, SUC_lexicon_tags_6 },
    { 0xba9c180b, SUC_lexicon_tags_19 },
    { 0xebc180c, SUC_lexicon_tags_15 },
    { 0x2ffc180c, SUC_lexicon_tags_33 },
    { 0xdc3c180e, SUC_lexicon_tags_82 },
    { 0x3f3c180d, SUC_lexicon_tags_17 },
    { 0x583c1810, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fc1813, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffc181c, SUC_lexicon_tags_11 },
    { 0x56bc181d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5c1820, SUC_lexicon_tags_10 },
    { 0x481c1820, SUC_lexicon_tags_26 },
    { 0xf61c1822, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311c1829, SUC_lexicon_tags_27 },
    { 0x5c5c182a, SUC_lexicon_tags_14 },
    { 0x7c7c1829, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61c182e, SUC_lexicon_tags_43 },
    { 0xc31c182f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x919c1832, SUC_lexicon_tags_6 },
    { 0x4fdc1833, SUC_lexicon_tags_11 },
    { 0xab3c1833, SUC_lexicon_tags_21 },
    { 0x5edc1835, SUC_lexicon_tags_30 },
    { 0xb3c1836, SUC_lexicon_tags_17 },
    { 0xc11c1833, SUC_lexicon_tags_11 },
    { 0xbe5c1838, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299c183c, SUC_lexicon_tags_7 },
    { 0xccfc183c, SUC_lexicon_tags_55 },
    { 0x289c183d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7c1842, SUC_lexicon_tags_37 },
    { 0xbbdc1843, SUC_lexicon_tags_0 },
    { 0x65fc1842, SUC_lexicon_tags_32 },
    { 0x6d3c1845, SUC_lexicon_tags_91 },
    { 0xdedc1845, SUC_lexicon_tags_26 },
    { 0xe93c1847, SUC_lexicon_tags_37 },
    { 0x7f5c1848, SUC_lexicon_tags_7 },
    { 0x95fc1847, SUC_lexicon_tags_91 },
    { 0x6c9c1849, SUC_lexicon_tags_26 },
    { 0x87fc184b, SUC_lexicon_tags_45 },
    { 0xff7c184c, SUC_lexicon_tags_14 },
    { 0xaf1c184d, SUC_lexicon_tags_8 },
    { 0xcd7c184c, SUC_lexicon_tags_66 },
    { 0x9c1c184d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1c1853, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5c185b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xafbc185d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603c1861, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x9b7c1863, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa33c1865, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685c186d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x127c186f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fc1873, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb43c1875, SUC_lexicon_tags_29 },
    { 0x945c1875, SUC_lexicon_tags_6 },
    { 0x173c1877, SUC_lexicon_tags_66 },
    { 0x989c1877, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1c187d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97c1884, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fc1887, SUC_lexicon_tags_9 },
    { 0xc5fc1888, SUC_lexicon_tags_8 },
    { 0xc9bc1889, SUC_lexicon_tags_11 },
    { 0xce3c188a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c5c188c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85c188f, SUC_lexicon_tags_295 },
    { 0xd81c188f, SUC_lexicon_tags_47 },
    { 0x42dc188f, SUC_lexicon_tags_37 },
    { 0x9dfc188f, SUC_lexicon_tags_37 },
    { 0x4ffc1893, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fc1898, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7c189c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9c18a0, SUC_lexicon_tags_39 },
    { 0x625c18a1, SUC_lexicon_tags_33 },
    { 0x479c18a2, SUC_lexicon_tags_3 },
    { 0xb81c18a1, SUC_lexicon_tags_19 },
    { 0xb6dc18a3, SUC_lexicon_tags_27 },
    { 0xbcfc18a5, SUC_lexicon_tags_17 },
    { 0xfc3c18a3, SUC_lexicon_tags_29 },
    { 0xc21c18a2, SUC_lexicon_tags_11 },
    { 0xf29c18a6, SUC_lexicon_tags_38 },
    { 0x619c18a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x563c18ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dc18af, SUC_lexicon_tags_93 },
    { 0xeedc18b0, SUC_lexicon_tags_47 },
    { 0x55fc18b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933c18bd, SUC_lexicon_tags_37 },
    { 0xabdc18be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3c18cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9c18d0, SUC_lexicon_tags_17 },
    { 0xcb5c18d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa8bc18d3, SUC_lexicon_tags_7 },
    { 0x8f3c18d4, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x90fc18d6, SUC_lexicon_tags_38 },
    { 0xca9c18d6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x691c18d9, SUC_lexicon_tags_10 },
    { 0xc03c18da, SUC_lexicon_tags_8 },
    { 0xcd1c18d9, SUC_lexicon_tags_80 },
    { 0x591c18dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7c18e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb21c18e2, SUC_lexicon_tags_37 },
    { 0x79dc18e3, SUC_lexicon_tags_46 },
    { 0x6e3c18e3, SUC_lexicon_tags_12 },
    { 0x49c18e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5c18ea, SUC_lexicon_tags_1 },
    { 0x35bc18eb, SUC_lexicon_tags_7 },
    { 0x2efc18eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf33c18ee, SUC_lexicon_tags_15 },
    { 0x6fdc18ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcebc18f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74dc18f4, SUC_lexicon_tags_25 },
    { 0x827c18f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x83fc18f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429c18fb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dc1906, SUC_lexicon_tags_27 },
    { 0xcbbc1907, SUC_lexicon_tags_45 },
    { 0xd7bc1908, SUC_lexicon_tags_7 },
    { 0x9a7c1908, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5c190e, SUC_lexicon_tags_43 },
    { 0xcb5c190f, SUC_lexicon_tags_17 },
    { 0x361c1910, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x9b1c1912, SUC_lexicon_tags_12 },
    { 0x3e3c1913, SUC_lexicon_tags_17 },
    { 0x1a5c1913, SUC_lexicon_tags_63 },
    { 0x39bc1915, SUC_lexicon_tags_1 },
    { 0x6adc1912, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dc191a, SUC_lexicon_tags_124 },
    { 0x2c9c191a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c9c191d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fc1920, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7c1923, SUC_lexicon_tags_32 },
    { 0x96dc1924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x979c1926, SUC_lexicon_tags_65 },
    { 0x28fc1926, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71c1930, SUC_lexicon_tags_37 },
    { 0x813c1931, SUC_lexicon_tags_12 },
    { 0x30dc1930, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279c1936, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeb1c1938, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfd1c193a, SUC_lexicon_tags_477 },
    { 0x8bdc193a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x311c193e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fc1943, SUC_lexicon_tags_8 },
    { 0x795c1943, SUC_lexicon_tags_25 },
    { 0x9fdc1945, SUC_lexicon_tags_14 },
    { 0x387c1946, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x701c1948, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5abc194a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb87c194c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bc194f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321c1959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbc1963, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7c1966, SUC_lexicon_tags_65 },
    { 0xe63c1967, SUC_lexicon_tags_68 },
    { 0xf37c1968, SUC_lexicon_tags_92 },
    { 0xcebc1968, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x635c196b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb2bc196d, SUC_lexicon_tags_24 },
    { 0x1a1c196e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbc1974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bc197c, SUC_lexicon_tags_11 },
    { 0x365c197d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x929c197f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe03c1981, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fc1986, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29c198b, SUC_lexicon_tags_26 },
    { 0x1c3c198c, SUC_lexicon_tags_25 },
    { 0x3d5c198d, SUC_lexicon_tags_3 },
    { 0x19fc198d, SUC_lexicon_tags_27 },
    { 0xd93c198f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9c1995, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727c199b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bc199e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3c19a1, SUC_lexicon_tags_9 },
    { 0xa47c19a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x797c19a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfe5c19a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3c19a9, SUC_lexicon_tags_14 },
    { 0xf47c19aa, SUC_lexicon_tags_11 },
    { 0xaf1c19ab, SUC_lexicon_tags_38 },
    { 0x31bc19ac, SUC_lexicon_tags_45 },
    { 0x193c19ad, SUC_lexicon_tags_7 },
    { 0xec3c19ae, SUC_lexicon_tags_15 },
    { 0xc49c19af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91c19b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fc19be, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x617c19c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc19c4, SUC_lexicon_tags_37 },
    { 0x517c19c4, SUC_lexicon_tags_63 },
    { 0x4c1c19c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5c19c9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851c19cd, SUC_lexicon_tags_14 },
    { 0xd91c19ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf83c19d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x489c19d3, SUC_lexicon_tags_29 },
    { 0x7c1c19d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdc19d6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xebbc19d8, SUC_lexicon_tags_11 },
    { 0xff7c19d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x389c19dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x96fc19de, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x437c19e1, SUC_lexicon_tags_11 },
    { 0xf67c19e2, SUC_lexicon_tags_15 },
    { 0x5bdc19e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697c19e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x40dc19eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc19ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x701c19f2, SUC_lexicon_tags_30 },
    { 0x267c19f2, SUC_lexicon_tags_0 },
    { 0x9d3c19f3, SUC_lexicon_tags_26 },
    { 0xd23c19f5, SUC_lexicon_tags_23 },
    { 0x6c1c19f3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x993c19f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a5c19fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc09c19fc, SUC_lexicon_tags_63 },
    { 0x14dc19fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fc1a03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853c1a0a, SUC_lexicon_tags_25 },
    { 0xbf5c1a0b, SUC_lexicon_tags_151 },
    { 0xa3fc1a0b, SUC_lexicon_tags_11 },
    { 0xfefc1a0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7c1a10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91c1a15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669c1a19, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6fdc1a1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x895c1a1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1c1a2b, SUC_lexicon_tags_25 },
    { 0x5a9c1a2b, SUC_lexicon_tags_32 },
    { 0x2bbc1a2d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb45c1a2f, SUC_lexicon_tags_5 },
    { 0x449c1a30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x621c1a33, SUC_lexicon_tags_7 },
    { 0x5f1c1a34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a5c1a36, SUC_lexicon_tags_48 },
    { 0x217c1a37, SUC_lexicon_tags_30 },
    { 0x9dfc1a38, SUC_lexicon_tags_65 },
    { 0xd57c1a37, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc1a3c, SUC_lexicon_tags_25 },
    { 0x813c1a3d, SUC_lexicon_tags_59 },
    { 0x58dc1a3e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fc1a41, SUC_lexicon_tags_62 },
    { 0x3fc1a42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5c1a46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc1a49, SUC_lexicon_tags_7 },
    { 0xe73c1a49, SUC_lexicon_tags_7 },
    { 0xea5c1a4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39c1a4e, SUC_lexicon_tags_28 },
    { 0xae1c1a4f, SUC_lexicon_tags_12 },
    { 0x3f5c1a4f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401c1a56, SUC_lexicon_tags_47 },
    { 0xca7c1a57, SUC_lexicon_tags_31 },
    { 0xfbfc1a56, SUC_lexicon_tags_27 },
    { 0x4ebc1a56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x585c1a5b, SUC_lexicon_tags_10 },
    { 0xbf5c1a5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7c1a61, SUC_lexicon_tags_9 },
    { 0x631c1a61, SUC_lexicon_tags_11 },
    { 0x8b5c1a61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f5c1a65, SUC_lexicon_tags_7 },
    { 0x2e7c1a66, SUC_lexicon_tags_1 },
    { 0x5d9c1a65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5f7c1a69, SUC_lexicon_tags_27 },
    { 0xb15c1a6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe23c1a6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc1a71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bc1a75, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dc1a7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1c1a7f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dc1a85, SUC_lexicon_tags_262 },
    { 0x14bc1a86, SUC_lexicon_tags_17 },
    { 0xdb3c1a87, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fc1a8b, SUC_lexicon_tags_8 },
    { 0x825c1a8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dc1a91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa3c1a93, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bc1a98, SUC_lexicon_tags_6 },
    { 0xe47c1a98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x469c1a9c, SUC_lexicon_tags_27 },
    { 0xd8dc1a9d, SUC_lexicon_tags_14 },
    { 0x1c1c1a9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x171c1aa0, SUC_lexicon_tags_17 },
    { 0x85fc1aa0, SUC_lexicon_tags_7 },
    { 0xf8fc1aa2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x14dc1aa4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7c1aad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dc1ab2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xed7c1ab4, SUC_lexicon_tags_47 },
    { 0xa39c1ab5, SUC_lexicon_tags_60 },
    { 0x87dc1ab4, SUC_lexicon_tags_0 },
    { 0xf57c1ab7, SUC_lexicon_tags_7 },
    { 0xc69c1ab5, SUC_lexicon_tags_0 },
    { 0x68bc1ab9, SUC_lexicon_tags_14 },
    { 0x5ddc1aba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc13c1abc, SUC_lexicon_tags_4 },
    { 0x2ebc1abd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9dc1ac0, SUC_lexicon_tags_46 },
    { 0xe0fc1ac1, SUC_lexicon_tags_17 },
    { 0x27dc1ac0, SUC_lexicon_tags_11 },
    { 0x3dc1ac3, SUC_lexicon_tags_19 },
    { 0x4a1c1ac4, SUC_lexicon_tags_19 },
    { 0xcf5c1ac3, SUC_lexicon_tags_11 },
    { 0xc47c1ac3, SUC_lexicon_tags_12 },
    { 0x757c1ac3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43c1aca, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dc1acf, SUC_lexicon_tags_26 },
    { 0xbb1c1acf, SUC_lexicon_tags_7 },
    { 0xe73c1ad0, SUC_lexicon_tags_4 },
    { 0x775c1ad2, SUC_lexicon_tags_60 },
    { 0x2bdc1ad3, SUC_lexicon_tags_2 },
    { 0xc1c1ad4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf1c1ad6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa65c1ad8, SUC_lexicon_tags_0 },
    { 0xfefc1ad9, SUC_lexicon_tags_45 },
    { 0x1e1c1ada, SUC_lexicon_tags_25 },
    { 0x5bdc1adb, SUC_lexicon_tags_113 },
    { 0x843c1adc, SUC_lexicon_tags_37 },
    { 0xdd3c1adb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x969c1adf, SUC_lexicon_tags_70 },
    { 0xbfdc1adf, SUC_lexicon_tags_15 },
    { 0xce5c1ae1, SUC_lexicon_tags_0 },
    { 0x64dc1ae2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xedc1ae5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dc1aeb, SUC_lexicon_tags_12 },
    { 0x4adc1aec, SUC_lexicon_tags_37 },
    { 0x911c1aed, SUC_lexicon_tags_43 },
    { 0x60fc1aee, SUC_lexicon_tags_33 },
    { 0x4a1c1aee, SUC_lexicon_tags_28 },
    { 0x5dc1af0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57c1af3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x95fc1af5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761c1afb, SUC_lexicon_tags_25 },
    { 0x541c1afb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5c1aff, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93c1b03, SUC_lexicon_tags_47 },
    { 0xee1c1b04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbc1b07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc1b0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3fc1b0e, SUC_lexicon_tags_25 },
    { 0xe81c1b0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841c1b15, SUC_lexicon_tags_14 },
    { 0x58dc1b15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb45c1b18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddc1b1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed7c1b1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bc1b22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bfc1b24, SUC_lexicon_tags_38 },
    { 0xc61c1b25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x285c1b28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5c1b2b, SUC_lexicon_tags_11 },
    { 0x65bc1b2c, SUC_lexicon_tags_4 },
    { 0x199c1b2d, SUC_lexicon_tags_6 },
    { 0x27bc1b2d, SUC_lexicon_tags_11 },
    { 0xc5bc1b2f, SUC_lexicon_tags_47 },
    { 0xad9c1b30, SUC_lexicon_tags_11 },
    { 0x48bc1b2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dc1b34, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fc1b3c, SUC_lexicon_tags_84 },
    { 0x225c1b3c, SUC_lexicon_tags_12 },
    { 0x74bc1b3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf51c1b40, SUC_lexicon_tags_25 },
    { 0xf37c1b40, SUC_lexicon_tags_5 },
    { 0xe69c1b42, SUC_lexicon_tags_25 },
    { 0xeddc1b43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fc1b46, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529c1b4e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1c1b51, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb55c1b53, SUC_lexicon_tags_23 },
    { 0x1b5c1b54, SUC_lexicon_tags_61 },
    { 0x1b1c1b54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dc1b59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x781c1b5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5c1b5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bc1b63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3c1b67, SUC_lexicon_tags_2 },
    { 0x5c1b68, SUC_lexicon_tags_11 },
    { 0x36dc1b69, SUC_lexicon_tags_1 },
    { 0x76fc1b6a, SUC_lexicon_tags_0 },
    { 0x2a5c1b6a, SUC_lexicon_tags_103 },
    { 0x101c1b67, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x855c1b6e, SUC_lexicon_tags_4 },
    { 0x4fc1b6e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c1b74, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19c1b7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17dc1b81, SUC_lexicon_tags_25 },
    { 0x863c1b81, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5c1b86, SUC_lexicon_tags_12 },
    { 0xd6fc1b87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x983c1b89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x93fc1b8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x911c1b8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439c1b91, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73c1b96, SUC_lexicon_tags_19 },
    { 0x6e9c1b97, SUC_lexicon_tags_14 },
    { 0x42dc1b96, SUC_lexicon_tags_30 },
    { 0x217c1b99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227c1b9e, SUC_lexicon_tags_7 },
    { 0x539c1b9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fc1ba3, SUC_lexicon_tags_25 },
    { 0x181c1ba4, SUC_lexicon_tags_25 },
    { 0x4c5c1ba5, SUC_lexicon_tags_42 },
    { 0x431c1ba4, SUC_lexicon_tags_43 },
    { 0xa67c1ba7, SUC_lexicon_tags_25 },
    { 0xc6dc1ba8, SUC_lexicon_tags_26 },
    { 0x9a5c1ba9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x929c1bab, SUC_lexicon_tags_21 },
    { 0x6d1c1bac, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xc87c1bae, SUC_lexicon_tags_15 },
    { 0x4cdc1bae, SUC_lexicon_tags_38 },
    { 0xe0bc1bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959c1bb5, SUC_lexicon_tags_2 },
    { 0xb73c1bb6, SUC_lexicon_tags_11 },
    { 0x13bc1bb6, SUC_lexicon_tags_1 },
    { 0xd9fc1bb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bc1bbe, SUC_lexicon_tags_27 },
    { 0x405c1bbe, SUC_lexicon_tags_29 },
    { 0x925c1bc0, SUC_lexicon_tags_21 },
    { 0xbebc1bc1, SUC_lexicon_tags_1 },
    { 0x5cbc1bbe, SUC_lexicon_tags_66 },
    { 0xfb9c1bc3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x317c1bc5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x433c1bc8, SUC_lexicon_tags_26 },
    { 0xbadc1bc8, SUC_lexicon_tags_136 },
    { 0x9adc1bca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1c1bce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9c1bd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bc1bd6, SUC_lexicon_tags_11 },
    { 0x533c1bd7, SUC_lexicon_tags_15 },
    { 0xd83c1bd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca1c1bda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x167c1bdc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47c1be2, SUC_lexicon_tags_1 },
    { 0x77dc1be2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56bc1be5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fc1be9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2bc1beb, SUC_lexicon_tags_9 },
    { 0x631c1bec, SUC_lexicon_tags_27 },
    { 0xd4bc1bed, SUC_lexicon_tags_17 },
    { 0xffc1beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x789c1bf0, SUC_lexicon_tags_931 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dc1bf6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dc1bf9, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39c1c01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9c1c06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe0dc1c08, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x601c1c0a, SUC_lexicon_tags_15 },
    { 0x5f7c1c0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfc1c10, SUC_lexicon_tags_46 },
    { 0xc73c1c11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7c1c14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a1c1c16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbc1c1a, SUC_lexicon_tags_2 },
    { 0xa75c1c1b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x345c1c1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9c1c20, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xdebc1c23, SUC_lexicon_tags_39 },
    { 0xa41c1c23, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x4adc1c26, SUC_lexicon_tags_25 },
    { 0x6d1c1c27, SUC_lexicon_tags_2 },
    { 0xa8fc1c26, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfc1c30, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x3e1c1c32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951c1c39, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9c1c3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x889c1c3f, SUC_lexicon_tags_45 },
    { 0x819c1c40, SUC_lexicon_tags_12 },
    { 0xdd9c1c40, SUC_lexicon_tags_27 },
    { 0x5c1c1c42, SUC_lexicon_tags_30 },
    { 0x1c5c1c43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bc1c51, SUC_lexicon_tags_45 },
    { 0x7bbc1c51, SUC_lexicon_tags_0 },
    { 0x78fc1c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5c1c58, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707c1c5d, SUC_lexicon_tags_25 },
    { 0x4bdc1c5d, SUC_lexicon_tags_37 },
    { 0xcbc1c5d, SUC_lexicon_tags_37 },
    { 0x649c1c5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd79c1c62, SUC_lexicon_tags_2 },
    { 0x1a5c1c62, SUC_lexicon_tags_40 },
    { 0x4ddc1c62, SUC_lexicon_tags_1 },
    { 0xed5c1c62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x951c1c67, SUC_lexicon_tags_48 },
    { 0x5edc1c68, SUC_lexicon_tags_15 },
    { 0x785c1c67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x84fc1c6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x681c1c6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bc1c75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2fbc1c77, SUC_lexicon_tags_27 },
    { 0x1a5c1c78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63c1c7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75c1c82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x945c1c84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53c1c88, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fc1c8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89c1c8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x725c1c90, SUC_lexicon_tags_54 },
    { 0xbebc1c91, SUC_lexicon_tags_25 },
    { 0xaafc1c92, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xed7c1c94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27c1c9b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dc1ca1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5c1ca8, SUC_lexicon_tags_33 },
    { 0x5f3c1ca8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x649c1cab, SUC_lexicon_tags_19 },
    { 0x36bc1cab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7c1cb3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9c1cb6, SUC_lexicon_tags_37 },
    { 0xc31c1cb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81fc1cb9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7c1cbd, SUC_lexicon_tags_25 },
    { 0xa99c1cbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c1cc1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649c1cc5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7c1cd4, SUC_lexicon_tags_21 },
    { 0xb1dc1cd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dc1cd9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fc1cdd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x151c1cdf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc37c1ce1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3c1ce5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09c1ceb, SUC_lexicon_tags_13 },
    { 0x435c1cec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebc1cf0, SUC_lexicon_tags_33 },
    { 0xf79c1cf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3c1cf4, SUC_lexicon_tags_17 },
    { 0xc4bc1cf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x867c1cf7, SUC_lexicon_tags_9 },
    { 0xf4dc1cf7, SUC_lexicon_tags_37 },
    { 0x3c1cf9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x689c1cfb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725c1cff, SUC_lexicon_tags_25 },
    { 0xe63c1d00, SUC_lexicon_tags_66 },
    { 0x5f1c1d00, SUC_lexicon_tags_37 },
    { 0x897c1d02, SUC_lexicon_tags_13 },
    { 0x909c1d01, SUC_lexicon_tags_37 },
    { 0xaa1c1d02, SUC_lexicon_tags_10 },
    { 0x161c1d05, SUC_lexicon_tags_15 },
    { 0xfb5c1d05, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7c1d09, SUC_lexicon_tags_15 },
    { 0x799c1d09, SUC_lexicon_tags_15 },
    { 0x62dc1d0a, SUC_lexicon_tags_37 },
    { 0x691c1d0a, SUC_lexicon_tags_14 },
    { 0xe91c1d0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fc1d12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53c1d16, SUC_lexicon_tags_30 },
    { 0xb13c1d16, SUC_lexicon_tags_15 },
    { 0x5b9c1d18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fc1d1e, SUC_lexicon_tags_37 },
    { 0xbb9c1d1e, SUC_lexicon_tags_37 },
    { 0x3e3c1d20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1c1d24, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc1d2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c1d2f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dc1d32, SUC_lexicon_tags_12 },
    { 0xadc1d32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23fc1d36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7c1d39, SUC_lexicon_tags_2 },
    { 0x781c1d39, SUC_lexicon_tags_23 },
    { 0xe15c1d3b, SUC_lexicon_tags_2 },
    { 0x31fc1d39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73bc1d3e, SUC_lexicon_tags_1 },
    { 0x1bfc1d3f, SUC_lexicon_tags_7 },
    { 0xe93c1d40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1e9c1d42, SUC_lexicon_tags_38 },
    { 0x22fc1d43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7c1d47, SUC_lexicon_tags_18 },
    { 0x14bc1d48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56fc1d4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5c1d50, SUC_lexicon_tags_5 },
    { 0xf7bc1d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31fc1d53, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fc1d57, SUC_lexicon_tags_47 },
    { 0x8f5c1d58, SUC_lexicon_tags_127 },
    { 0xb25c1d57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85c1d60, SUC_lexicon_tags_25 },
    { 0xb5c1d61, SUC_lexicon_tags_37 },
    { 0x3dfc1d61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x525c1d64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5c1d67, SUC_lexicon_tags_32 },
    { 0xabfc1d68, SUC_lexicon_tags_25 },
    { 0xa3c1d68, SUC_lexicon_tags_1 },
    { 0xfadc1d6a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dc1d6e, SUC_lexicon_tags_43 },
    { 0xc67c1d6f, SUC_lexicon_tags_38 },
    { 0x1a9c1d6e, SUC_lexicon_tags_15 },
    { 0xd1bc1d71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf49c1d73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49c1d7b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xca7c1d7d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115c1d82, SUC_lexicon_tags_11 },
    { 0x493c1d82, SUC_lexicon_tags_25 },
    { 0x2f1c1d84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445c1d8c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf3bc1d8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173c1d9a, SUC_lexicon_tags_47 },
    { 0xcffc1d9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dc1da0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bc1dae, SUC_lexicon_tags_25 },
    { 0x1ffc1daf, SUC_lexicon_tags_37 },
    { 0xf27c1daf, SUC_lexicon_tags_26 },
    { 0x903c1db1, SUC_lexicon_tags_27 },
    { 0xb49c1dae, SUC_lexicon_tags_12 },
    { 0xbe9c1db3, SUC_lexicon_tags_2 },
    { 0xe01c1db0, SUC_lexicon_tags_17 },
    { 0x51dc1db5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdc1dc0, SUC_lexicon_tags_7 },
    { 0xc1bc1dc0, SUC_lexicon_tags_418 },
    { 0x79dc1dc2, SUC_lexicon_tags_17 },
    { 0x5f5c1dc1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x345c1dc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc61c1dc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce3c1dc9, SUC_lexicon_tags_7 },
    { 0x681c1dca, SUC_lexicon_tags_14 },
    { 0x9dbc1dc9, SUC_lexicon_tags_37 },
    { 0x11dc1dca, SUC_lexicon_tags_37 },
    { 0xe7bc1dca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7c1dd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x36bc1dd2, SUC_lexicon_tags_6 },
    { 0x33dc1dd3, SUC_lexicon_tags_37 },
    { 0x8b3c1dd4, SUC_lexicon_tags_15 },
    { 0xf9bc1dd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x689c1dd8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x659c1ddb, SUC_lexicon_tags_32 },
    { 0x691c1ddb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a9c1dde, SUC_lexicon_tags_60 },
    { 0x2f1c1dde, SUC_lexicon_tags_47 },
    { 0xd91c1de0, SUC_lexicon_tags_49 },
    { 0xa3fc1dde, SUC_lexicon_tags_66 },
    { 0xc1c1de2, SUC_lexicon_tags_1 },
    { 0x925c1dde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc57c1de5, SUC_lexicon_tags_65 },
    { 0x713c1de5, SUC_lexicon_tags_7 },
    { 0x821c1de7, SUC_lexicon_tags_1 },
    { 0xcbbc1de5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c1deb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa8dc1ded, SUC_lexicon_tags_2 },
    { 0xa37c1dee, SUC_lexicon_tags_2 },
    { 0x9d7c1def, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x107c1df1, SUC_lexicon_tags_37 },
    { 0x4f5c1df2, SUC_lexicon_tags_12 },
    { 0xfb5c1df3, SUC_lexicon_tags_6 },
    { 0x873c1df4, SUC_lexicon_tags_65 },
    { 0xf25c1df5, SUC_lexicon_tags_37 },
    { 0xb75c1df5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x959c1df9, SUC_lexicon_tags_47 },
    { 0xd3bc1dfa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1c1dff, SUC_lexicon_tags_15 },
    { 0xda5c1e00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x63dc1e03, SUC_lexicon_tags_48 },
    { 0x67dc1e04, SUC_lexicon_tags_2 },
    { 0x997c1e05, SUC_lexicon_tags_0 },
    { 0x253c1e03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfc1e0c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637c1e10, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97c1e13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x319c1e15, SUC_lexicon_tags_2 },
    { 0x9edc1e15, SUC_lexicon_tags_7 },
    { 0x103c1e17, SUC_lexicon_tags_17 },
    { 0x403c1e18, SUC_lexicon_tags_31 },
    { 0xf7dc1e15, SUC_lexicon_tags_11 },
    { 0xe5fc1e1a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5c1e22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9c1e28, SUC_lexicon_tags_0 },
    { 0xd6fc1e29, SUC_lexicon_tags_23 },
    { 0x505c1e2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x875c1e2c, SUC_lexicon_tags_12 },
    { 0x617c1e2d, SUC_lexicon_tags_15 },
    { 0x9edc1e2d, SUC_lexicon_tags_27 },
    { 0x7c9c1e2f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdc1e32, SUC_lexicon_tags_2 },
    { 0xb7c1e32, SUC_lexicon_tags_82 },
    { 0x15fc1e33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3ddc1e36, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe35c1e38, SUC_lexicon_tags_37 },
    { 0x495c1e39, SUC_lexicon_tags_65 },
    { 0xb47c1e38, SUC_lexicon_tags_17 },
    { 0xebbc1e3b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7c1e3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x581c1e42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1fdc1e44, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc5bc1e46, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17c1e49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5c1e4f, SUC_lexicon_tags_1 },
    { 0x48fc1e50, SUC_lexicon_tags_26 },
    { 0xd99c1e50, SUC_lexicon_tags_15 },
    { 0x623c1e52, SUC_lexicon_tags_8 },
    { 0x713c1e52, SUC_lexicon_tags_11 },
    { 0x131c1e51, SUC_lexicon_tags_23 },
    { 0xebc1e55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd3fc1e57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce5c1e59, SUC_lexicon_tags_33 },
    { 0xce7c1e59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe97c1e5c, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x763c1e5e, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1c1e62, SUC_lexicon_tags_170 },
    { 0x897c1e63, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7c1e69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x207c1e6c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135c1e74, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdebc1e76, SUC_lexicon_tags_11 },
    { 0x645c1e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abc1e7b, SUC_lexicon_tags_0 },
    { 0xb53c1e7c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc1e81, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fc1e89, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17c1e8e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383c1e93, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdc1e9a, SUC_lexicon_tags_1 },
    { 0x4a3c1e9b, SUC_lexicon_tags_17 },
    { 0x8f5c1e9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d9c1e9e, SUC_lexicon_tags_38 },
    { 0xd7bc1e9f, SUC_lexicon_tags_10 },
    { 0x141c1ea0, SUC_lexicon_tags_54 },
    { 0x29c1ea1, SUC_lexicon_tags_45 },
    { 0x50fc1ea2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533c1eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd7c1eac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bbc1eae, SUC_lexicon_tags_11 },
    { 0x71bc1eaf, SUC_lexicon_tags_37 },
    { 0xcedc1eaf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x799c1eb2, SUC_lexicon_tags_47 },
    { 0x181c1eb2, SUC_lexicon_tags_37 },
    { 0xce5c1eb3, SUC_lexicon_tags_2 },
    { 0xfcbc1eb5, SUC_lexicon_tags_37 },
    { 0x41bc1eb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1c1eb9, SUC_lexicon_tags_12 },
    { 0x207c1eba, SUC_lexicon_tags_15 },
    { 0x65c1eba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x371c1ebd, SUC_lexicon_tags_84 },
    { 0xc77c1ebe, SUC_lexicon_tags_47 },
    { 0x335c1ebf, SUC_lexicon_tags_16 },
    { 0x689c1ebe, SUC_lexicon_tags_19 },
    { 0x4bbc1ec1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd8bc1ec3, SUC_lexicon_tags_19 },
    { 0x859c1ec4, SUC_lexicon_tags_42 },
    { 0xc27c1ec5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa31c1ec7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27c1eca, SUC_lexicon_tags_46 },
    { 0x9afc1ecb, SUC_lexicon_tags_48 },
    { 0xfe9c1ecc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47c1ed2, SUC_lexicon_tags_33 },
    { 0x207c1ed2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35c1ed6, SUC_lexicon_tags_60 },
    { 0xdd5c1ed6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661c1edc, SUC_lexicon_tags_19 },
    { 0xf95c1edd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507c1ee2, SUC_lexicon_tags_47 },
    { 0xcdbc1ee3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45c1ee9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77c1eed, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9c1ef1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4fdc1ef3, SUC_lexicon_tags_4 },
    { 0x237c1ef4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbc1ef8, SUC_lexicon_tags_47 },
    { 0x7e5c1ef8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25dc1f04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefc1f08, SUC_lexicon_tags_61 },
    { 0x89bc1f09, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695c1f0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bc1f11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc1f1c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bc1f24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x22bc1f26, SUC_lexicon_tags_45 },
    { 0xe25c1f27, SUC_lexicon_tags_45 },
    { 0x7ebc1f28, SUC_lexicon_tags_60 },
    { 0xdffc1f28, SUC_lexicon_tags_11 },
    { 0x36bc1f2a, SUC_lexicon_tags_56 },
    { 0xbc1f2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3bc1f2d, SUC_lexicon_tags_37 },
    { 0x499c1f2d, SUC_lexicon_tags_29 },
    { 0xb3fc1f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1c1f32, SUC_lexicon_tags_54 },
    { 0xd4dc1f33, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17c1f3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x125c1f3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a5c1f3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffc1f43, SUC_lexicon_tags_30 },
    { 0xae9c1f43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fc1f47, SUC_lexicon_tags_2 },
    { 0x647c1f47, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bc1f4c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x713c1f4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bc1f58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c1f5d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x59dc1f5f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb5dc1f61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54fc1f63, SUC_lexicon_tags_37 },
    { 0x8c3c1f64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfcbc1f66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6e7c1f68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdc1f6c, SUC_lexicon_tags_17 },
    { 0x1afc1f6d, SUC_lexicon_tags_12 },
    { 0xd5c1f6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3c1f75, SUC_lexicon_tags_29 },
    { 0x4c5c1f76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc4fc1f78, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x2ddc1f7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe33c1f7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e5c1f7e, SUC_lexicon_tags_12 },
    { 0x56dc1f7e, SUC_lexicon_tags_7 },
    { 0x63fc1f80, SUC_lexicon_tags_62 },
    { 0x653c1f7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7c1f85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa3dc1f87, SUC_lexicon_tags_17 },
    { 0xebfc1f87, SUC_lexicon_tags_17 },
    { 0x25bc1f88, SUC_lexicon_tags_49 },
    { 0xe07c1f88, SUC_lexicon_tags_4 },
    { 0x345c1f89, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5c1f8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bc1f95, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x713c1f97, SUC_lexicon_tags_113 },
    { 0xc17c1f98, SUC_lexicon_tags_11 },
    { 0xdc9c1f98, SUC_lexicon_tags_7 },
    { 0x8ffc1f9a, SUC_lexicon_tags_26 },
    { 0xb41c1f9b, SUC_lexicon_tags_11 },
    { 0xf31c1f97, SUC_lexicon_tags_1 },
    { 0x147c1f9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb6bc1f9f, SUC_lexicon_tags_82 },
    { 0x873c1fa0, SUC_lexicon_tags_28 },
    { 0x56dc1fa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1c1fa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307c1fb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7c1fb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bc1fc3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a1c1fc5, SUC_lexicon_tags_37 },
    { 0xf43c1fc6, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3c1fca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb4bc1fcc, SUC_lexicon_tags_12 },
    { 0xfffc1fcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9c1fd0, SUC_lexicon_tags_4 },
    { 0x6c3c1fd1, SUC_lexicon_tags_1 },
    { 0xec3c1fd2, SUC_lexicon_tags_17 },
    { 0xaddc1fd2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x99fc1fd5, SUC_lexicon_tags_23 },
    { 0xc8dc1fd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dc1fdf, SUC_lexicon_tags_4 },
    { 0x5e1c1fe0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b9c1fe2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9c1fe8, SUC_lexicon_tags_33 },
    { 0xdfc1fe9, SUC_lexicon_tags_37 },
    { 0xdfc1fea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x85bc1fec, SUC_lexicon_tags_15 },
    { 0xa77c1fec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x961c1fef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe8bc1ff1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x115c1ff3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadc1ff8, SUC_lexicon_tags_56 },
    { 0x9f1c1ff8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84bc1ffb, SUC_lexicon_tags_65 },
    { 0x279c1ffc, SUC_lexicon_tags_14 },
    { 0x75c1ffd, SUC_lexicon_tags_11 },
    { 0x623c1ffe, SUC_lexicon_tags_6 },
    { 0x5afc1fff, SUC_lexicon_tags_14 },
    { 0x2f7c2000, SUC_lexicon_tags_11 },
    { 0x54fc2001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69c2003, SUC_lexicon_tags_30 },
    { 0x61c2003, SUC_lexicon_tags_37 },
    { 0xaa7c2005, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1c200e, SUC_lexicon_tags_28 },
    { 0x5cdc200f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7c2016, SUC_lexicon_tags_11 },
    { 0xfa3c2017, SUC_lexicon_tags_25 },
    { 0xca5c2018, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9c201d, SUC_lexicon_tags_23 },
    { 0x959c201e, SUC_lexicon_tags_82 },
    { 0x8f5c201e, SUC_lexicon_tags_12 },
    { 0x685c201f, SUC_lexicon_tags_8 },
    { 0x8c1c2020, SUC_lexicon_tags_15 },
    { 0xbdbc2022, SUC_lexicon_tags_7 },
    { 0x60fc2023, SUC_lexicon_tags_74 },
    { 0x65c2022, SUC_lexicon_tags_66 },
    { 0x607c2025, SUC_lexicon_tags_37 },
    { 0xbfbc2026, SUC_lexicon_tags_15 },
    { 0xc57c2027, SUC_lexicon_tags_27 },
    { 0x1ddc2025, SUC_lexicon_tags_7 },
    { 0xd27c2029, SUC_lexicon_tags_1 },
    { 0xcb3c2029, SUC_lexicon_tags_26 },
    { 0xfb7c2024, SUC_lexicon_tags_12 },
    { 0x73bc201d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7c2030, SUC_lexicon_tags_87 },
    { 0x519c2031, SUC_lexicon_tags_27 },
    { 0xcb3c2031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc2035, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77bc2037, SUC_lexicon_tags_9 },
    { 0x587c2038, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x127c203a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29c2040, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57c2044, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x101c2047, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9c204b, SUC_lexicon_tags_0 },
    { 0x907c204b, SUC_lexicon_tags_11 },
    { 0x4bdc204d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63c2050, SUC_lexicon_tags_37 },
    { 0xb59c2050, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x68dc2053, SUC_lexicon_tags_30 },
    { 0x9b5c2053, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67c205a, SUC_lexicon_tags_47 },
    { 0xed3c205b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bc205e, SUC_lexicon_tags_2 },
    { 0x6bbc205f, SUC_lexicon_tags_2 },
    { 0x38dc205f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abc2063, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5c2066, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc206c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x191c206e, SUC_lexicon_tags_81 },
    { 0xa53c206e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x443c2071, SUC_lexicon_tags_57 },
    { 0x5f7c2072, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbc2075, SUC_lexicon_tags_16 },
    { 0xe69c2076, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81c2079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17dc207b, SUC_lexicon_tags_55 },
    { 0xa49c207b, SUC_lexicon_tags_40 },
    { 0x8cdc207c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dc2081, SUC_lexicon_tags_2 },
    { 0x10dc2082, SUC_lexicon_tags_43 },
    { 0x1dc2083, SUC_lexicon_tags_745 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bc2087, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4a7c2089, SUC_lexicon_tags_7 },
    { 0xf51c208a, SUC_lexicon_tags_17 },
    { 0xae3c2089, SUC_lexicon_tags_224 },
    { 0x541c208c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x64dc208f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9c2096, SUC_lexicon_tags_12 },
    { 0x9d9c2097, SUC_lexicon_tags_23 },
    { 0x3f3c2096, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x393c209a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7c209f, SUC_lexicon_tags_7 },
    { 0xb9c209f, SUC_lexicon_tags_85 },
    { 0xe57c20a1, SUC_lexicon_tags_45 },
    { 0xd19c20a2, SUC_lexicon_tags_39 },
    { 0x3b7c20a3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x94fc20a5, SUC_lexicon_tags_37 },
    { 0x219c20a6, SUC_lexicon_tags_25 },
    { 0x839c20a6, SUC_lexicon_tags_69 },
    { 0x2d9c20a8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x89c20aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205c20b1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa7dc20b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x719c20b5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x433c20b8, SUC_lexicon_tags_84 },
    { 0x457c20b8, SUC_lexicon_tags_11 },
    { 0x13dc20b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9c20bd, SUC_lexicon_tags_25 },
    { 0x6fbc20bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7c20c2, SUC_lexicon_tags_17 },
    { 0xcedc20c3, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe07c20c5, SUC_lexicon_tags_207 },
    { 0xb33c20c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa89c20c8, SUC_lexicon_tags_0 },
    { 0xc01c20c9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x191c20cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc20ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf25c20d0, SUC_lexicon_tags_10 },
    { 0xb21c20d0, SUC_lexicon_tags_11 },
    { 0x65c20d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd35c20d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dc20d9, SUC_lexicon_tags_43 },
    { 0xeb1c20d9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afc20df, SUC_lexicon_tags_11 },
    { 0xbddc20df, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xeddc20e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33dc20e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855c20eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377c20f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dc20f9, SUC_lexicon_tags_2 },
    { 0xd97c20f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x747c20fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf9fc20ff, SUC_lexicon_tags_15 },
    { 0x6d3c2100, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597c2104, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0x6b7c2106, SUC_lexicon_tags_25 },
    { 0xcb3c2106, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4c9c2109, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533c210d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3c2112, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431c2117, SUC_lexicon_tags_37 },
    { 0xc3dc2118, SUC_lexicon_tags_19 },
    { 0x9c1c2117, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafc211c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dc2124, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1c2128, SUC_lexicon_tags_25 },
    { 0xb25c2128, SUC_lexicon_tags_0 },
    { 0x121c212a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bc212d, SUC_lexicon_tags_37 },
    { 0xe1dc212e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bc2131, SUC_lexicon_tags_37 },
    { 0x4c1c2131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf01c2134, SUC_lexicon_tags_10 },
    { 0xefc2135, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7c213a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2abc213c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477c2140, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc2143, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3c2149, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185c214f, SUC_lexicon_tags_14 },
    { 0xfb7c214f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4ffc2152, SUC_lexicon_tags_1 },
    { 0xeafc2153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb7c2155, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x775c2158, SUC_lexicon_tags_35 },
    { 0x701c2159, SUC_lexicon_tags_87 },
    { 0x9d7c215a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9c215f, SUC_lexicon_tags_14 },
    { 0x9fc2160, SUC_lexicon_tags_65 },
    { 0x2b3c2161, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x699c2163, SUC_lexicon_tags_17 },
    { 0x343c2164, SUC_lexicon_tags_12 },
    { 0x255c2164, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfc2169, SUC_lexicon_tags_45 },
    { 0xaabc216a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5c2171, SUC_lexicon_tags_8 },
    { 0xe3bc2172, SUC_lexicon_tags_7 },
    { 0xf09c2172, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2d9c2175, SUC_lexicon_tags_25 },
    { 0xad1c2176, SUC_lexicon_tags_27 },
    { 0xf9fc2176, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59c217a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fc2188, SUC_lexicon_tags_12 },
    { 0x5c9c2188, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475c218f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29c2195, SUC_lexicon_tags_7 },
    { 0x86fc2196, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3c219c, SUC_lexicon_tags_12 },
    { 0x6f5c219c, SUC_lexicon_tags_60 },
    { 0x33bc219e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x55fc21a0, SUC_lexicon_tags_39 },
    { 0xa8bc21a1, SUC_lexicon_tags_37 },
    { 0xe83c21a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1fdc21a4, SUC_lexicon_tags_33 },
    { 0x2adc21a5, SUC_lexicon_tags_66 },
    { 0x8a1c21a6, SUC_lexicon_tags_4 },
    { 0xb1dc21a7, SUC_lexicon_tags_15 },
    { 0x5a7c21a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fc21ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bc21b4, SUC_lexicon_tags_23 },
    { 0xe1c21b4, SUC_lexicon_tags_2 },
    { 0xbafc21b6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9c21ba, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddc21bd, SUC_lexicon_tags_12 },
    { 0x3c7c21bd, SUC_lexicon_tags_11 },
    { 0xecdc21be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7fc21c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa03c21c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x777c21c6, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243c21cc, SUC_lexicon_tags_25 },
    { 0xd2bc21cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe63c21cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419c21df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32fc21e1, SUC_lexicon_tags_14 },
    { 0x11fc21e2, SUC_lexicon_tags_1 },
    { 0xca9c21e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbafc21e5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67c21f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd8bc21f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667c21f9, SUC_lexicon_tags_37 },
    { 0x20bc21fa, SUC_lexicon_tags_2 },
    { 0xd37c21f9, SUC_lexicon_tags_53 },
    { 0x6a9c21fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afc2201, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x145c2204, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877c2209, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdc220f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x747c2211, SUC_lexicon_tags_34 },
    { 0xc23c2211, SUC_lexicon_tags_0 },
    { 0x4a7c2212, SUC_lexicon_tags_8 },
    { 0x12bc2214, SUC_lexicon_tags_30 },
    { 0x46dc2215, SUC_lexicon_tags_15 },
    { 0xd71c2216, SUC_lexicon_tags_27 },
    { 0xfa1c2215, SUC_lexicon_tags_14 },
    { 0x23fc2215, SUC_lexicon_tags_11 },
    { 0xa79c2219, SUC_lexicon_tags_44 },
    { 0x2bdc221a, SUC_lexicon_tags_15 },
    { 0x84fc221b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x603c221e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c2221, SUC_lexicon_tags_8 },
    { 0xd7fc2222, SUC_lexicon_tags_31 },
    { 0x513c2221, SUC_lexicon_tags_87 },
    { 0x37dc2222, SUC_lexicon_tags_30 },
    { 0x76bc2221, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3c2228, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17c222b, SUC_lexicon_tags_7 },
    { 0x8abc222c, SUC_lexicon_tags_32 },
    { 0x2f9c222d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9c2230, SUC_lexicon_tags_48 },
    { 0x359c2231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58bc2233, SUC_lexicon_tags_28 },
    { 0x457c2233, SUC_lexicon_tags_11 },
    { 0xbfdc2235, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9c2239, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419c223f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7c2244, SUC_lexicon_tags_29 },
    { 0x417c2245, SUC_lexicon_tags_11 },
    { 0xd7bc2244, SUC_lexicon_tags_68 },
    { 0xc69c2246, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1b9c2249, SUC_lexicon_tags_25 },
    { 0x19fc224a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4ebc224c, SUC_lexicon_tags_15 },
    { 0x315c224d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x555c224f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3c225a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745c2260, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5c2264, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x743c2267, SUC_lexicon_tags_8 },
    { 0xae9c2268, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbc226b, SUC_lexicon_tags_37 },
    { 0x555c226b, SUC_lexicon_tags_12 },
    { 0x37bc226d, SUC_lexicon_tags_14 },
    { 0xbc3c226d, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7c227a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84fc227c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x41bc227e, SUC_lexicon_tags_26 },
    { 0x3bfc227f, SUC_lexicon_tags_15 },
    { 0xa87c2280, SUC_lexicon_tags_77 },
    { 0x8a3c2281, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c2285, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49c228b, SUC_lexicon_tags_1 },
    { 0x671c228b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53c228f, SUC_lexicon_tags_23 },
    { 0xb93c2290, SUC_lexicon_tags_38 },
    { 0x719c2291, SUC_lexicon_tags_5 },
    { 0xfe1c2290, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c229d, SUC_lexicon_tags_11 },
    { 0x8b1c229e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7c22a1, SUC_lexicon_tags_11 },
    { 0x193c22a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e7c22a4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbffc22a6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbc22ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fc22ae, SUC_lexicon_tags_37 },
    { 0x6d5c22af, SUC_lexicon_tags_35 },
    { 0x6abc22b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x46dc22b2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcd7c22b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x679c22b6, SUC_lexicon_tags_27 },
    { 0x5a5c22b7, SUC_lexicon_tags_4 },
    { 0x65bc22b8, SUC_lexicon_tags_65 },
    { 0xdfc22b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bc22bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1c22bf, SUC_lexicon_tags_0 },
    { 0x149c22c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9c22c2, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299c22c7, SUC_lexicon_tags_11 },
    { 0xc97c22c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9c22cc, SUC_lexicon_tags_66 },
    { 0x8d7c22cd, SUC_lexicon_tags_60 },
    { 0xcfbc22cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x849c22d0, SUC_lexicon_tags_25 },
    { 0xecbc22d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadbc22d3, SUC_lexicon_tags_38 },
    { 0x5efc22d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1c22d6, SUC_lexicon_tags_26 },
    { 0xaa3c22d7, SUC_lexicon_tags_37 },
    { 0x1c7c22d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b5c22da, SUC_lexicon_tags_47 },
    { 0x827c22db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3d7c22dd, SUC_lexicon_tags_33 },
    { 0xc11c22dd, SUC_lexicon_tags_2 },
    { 0x77fc22de, SUC_lexicon_tags_70 },
    { 0x9a5c22dd, SUC_lexicon_tags_150 },
    { 0x6fbc22e1, SUC_lexicon_tags_17 },
    { 0x799c22e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe29c22e4, SUC_lexicon_tags_49 },
    { 0xbb5c22e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30fc22e7, SUC_lexicon_tags_15 },
    { 0xde5c22e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5c22eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc71c22ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321c22f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9c22f8, SUC_lexicon_tags_27 },
    { 0x4c3c22f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x355c22fb, SUC_lexicon_tags_87 },
    { 0x651c22fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4dc22fe, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x619c2300, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3dc2302, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edc2307, SUC_lexicon_tags_27 },
    { 0xc2bc2307, SUC_lexicon_tags_23 },
    { 0x9dbc2308, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa1bc230b, SUC_lexicon_tags_11 },
    { 0xd3fc230b, SUC_lexicon_tags_15 },
    { 0xf61c230d, SUC_lexicon_tags_4 },
    { 0x337c230d, SUC_lexicon_tags_2 },
    { 0x9fbc230e, SUC_lexicon_tags_17 },
    { 0x17bc230c, SUC_lexicon_tags_47 },
    { 0x1e3c2311, SUC_lexicon_tags_15 },
    { 0xc15c230e, SUC_lexicon_tags_13 },
    { 0x185c2313, SUC_lexicon_tags_25 },
    { 0x5c9c2312, SUC_lexicon_tags_193 },
    { 0x3b5c230c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9c7c2317, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb77c2319, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7c231d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873c2322, SUC_lexicon_tags_1 },
    { 0x61fc2323, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x84bc2325, SUC_lexicon_tags_12 },
    { 0x5cdc2326, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbc2330, SUC_lexicon_tags_11 },
    { 0xa9dc2330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46dc2333, SUC_lexicon_tags_14 },
    { 0xf6dc2334, SUC_lexicon_tags_15 },
    { 0x723c2333, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bc233b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f3c233d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bc2344, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85c2347, SUC_lexicon_tags_47 },
    { 0xa03c2348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfc234c, SUC_lexicon_tags_28 },
    { 0x8cfc234d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39c2352, SUC_lexicon_tags_37 },
    { 0xd99c2353, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73c2356, SUC_lexicon_tags_26 },
    { 0x1b1c2357, SUC_lexicon_tags_37 },
    { 0x7a1c2357, SUC_lexicon_tags_12 },
    { 0x1e3c2359, SUC_lexicon_tags_26 },
    { 0xaa1c2359, SUC_lexicon_tags_37 },
    { 0x2d9c235b, SUC_lexicon_tags_155 },
    { 0xa3bc235c, SUC_lexicon_tags_25 },
    { 0x541c235b, SUC_lexicon_tags_74 },
    { 0x2d9c235e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebc2362, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bc2367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd9c2369, SUC_lexicon_tags_10 },
    { 0x96dc236a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5bc236c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfc2370, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bc2373, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1c2377, SUC_lexicon_tags_12 },
    { 0x3bdc2378, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4cdc237a, SUC_lexicon_tags_37 },
    { 0xce3c237a, SUC_lexicon_tags_37 },
    { 0x19fc237c, SUC_lexicon_tags_11 },
    { 0xc7dc237d, SUC_lexicon_tags_43 },
    { 0xc3dc237e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65c2384, SUC_lexicon_tags_1 },
    { 0x9abc2385, SUC_lexicon_tags_7 },
    { 0xedc2386, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdedc238a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257c2398, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbc239b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c239e, SUC_lexicon_tags_16 },
    { 0x549c239f, SUC_lexicon_tags_140 },
    { 0x4d7c23a0, SUC_lexicon_tags_1 },
    { 0x5f9c23a1, SUC_lexicon_tags_23 },
    { 0x28dc239f, SUC_lexicon_tags_17 },
    { 0xc2dc23a2, SUC_lexicon_tags_31 },
    { 0x86dc23a3, SUC_lexicon_tags_11 },
    { 0xddfc239e, SUC_lexicon_tags_17 },
    { 0x32fc23a6, SUC_lexicon_tags_14 },
    { 0x153c23a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff3c23a9, SUC_lexicon_tags_17 },
    { 0x889c23aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x48dc23ac, SUC_lexicon_tags_29 },
    { 0x991c23ad, SUC_lexicon_tags_43 },
    { 0xf95c23ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x617c23b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17c23b4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fc23ba, SUC_lexicon_tags_59 },
    { 0x85fc23bb, SUC_lexicon_tags_11 },
    { 0xea7c23bc, SUC_lexicon_tags_61 },
    { 0x941c23bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc23c0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x78fc23c2, SUC_lexicon_tags_5 },
    { 0xb99c23c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613c23c8, SUC_lexicon_tags_37 },
    { 0xf17c23c9, SUC_lexicon_tags_48 },
    { 0xfa9c23c8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe5fc23cc, SUC_lexicon_tags_37 },
    { 0x8fc23cc, SUC_lexicon_tags_19 },
    { 0x7d5c23cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x68fc23d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x225c23d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x50bc23d5, SUC_lexicon_tags_65 },
    { 0x6e5c23d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x903c23d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487c23de, SUC_lexicon_tags_27 },
    { 0xef1c23df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dc23e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbc23ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe21c23ee, SUC_lexicon_tags_17 },
    { 0xc37c23ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3c23f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x279c23f4, SUC_lexicon_tags_17 },
    { 0xd7c23f5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xaffc23f7, SUC_lexicon_tags_34 },
    { 0xfc1c23f8, SUC_lexicon_tags_60 },
    { 0x1e7c23f8, SUC_lexicon_tags_45 },
    { 0x951c23fa, SUC_lexicon_tags_11 },
    { 0xa9bc23f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7c2401, SUC_lexicon_tags_1 },
    { 0x2a3c2402, SUC_lexicon_tags_47 },
    { 0x243c2403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb73c2405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x571c2408, SUC_lexicon_tags_7 },
    { 0x44dc2408, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5c240c, SUC_lexicon_tags_13 },
    { 0x6efc240c, SUC_lexicon_tags_2 },
    { 0xbbbc240e, SUC_lexicon_tags_30 },
    { 0x3dfc240f, SUC_lexicon_tags_17 },
    { 0x193c2410, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd89c2412, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x647c2414, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7c241d, SUC_lexicon_tags_19 },
    { 0xe89c241e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fc2422, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbc2427, SUC_lexicon_tags_1 },
    { 0x5bbc2428, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x301c242a, SUC_lexicon_tags_41 },
    { 0xbe7c242a, SUC_lexicon_tags_26 },
    { 0x483c242c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971c2434, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6bbc2436, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175c243a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb23c243c, SUC_lexicon_tags_0 },
    { 0x505c243d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbc2440, SUC_lexicon_tags_7 },
    { 0xd07c2440, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27c2446, SUC_lexicon_tags_43 },
    { 0xc21c2447, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fc244d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7c2452, SUC_lexicon_tags_17 },
    { 0x567c2453, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9c2456, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239c245c, SUC_lexicon_tags_22 },
    { 0x17c245c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf3c245f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x319c2461, SUC_lexicon_tags_37 },
    { 0xc49c2462, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bc2466, SUC_lexicon_tags_15 },
    { 0x80fc2467, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf41c2469, SUC_lexicon_tags_38 },
    { 0x5e1c2469, SUC_lexicon_tags_38 },
    { 0x35dc2469, SUC_lexicon_tags_11 },
    { 0x587c246b, SUC_lexicon_tags_12 },
    { 0x3e7c2469, SUC_lexicon_tags_17 },
    { 0x77bc246a, SUC_lexicon_tags_60 },
    { 0xb1c246b, SUC_lexicon_tags_2 },
    { 0x81dc246d, SUC_lexicon_tags_37 },
    { 0x755c2469, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5c2474, SUC_lexicon_tags_15 },
    { 0xf9bc2475, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x433c2478, SUC_lexicon_tags_33 },
    { 0x8b1c2479, SUC_lexicon_tags_17 },
    { 0x3ddc2479, SUC_lexicon_tags_33 },
    { 0xf4bc247b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95c247f, SUC_lexicon_tags_45 },
    { 0x7ebc2480, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x67c2483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5c2486, SUC_lexicon_tags_13 },
    { 0xfb3c2487, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc248a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x861c248c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275c2492, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5c2496, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc2499, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bc249d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bc24a0, SUC_lexicon_tags_87 },
    { 0x56fc24a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x25dc24a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x485c24a6, SUC_lexicon_tags_17 },
    { 0xdafc24a7, SUC_lexicon_tags_25 },
    { 0xb99c24a7, SUC_lexicon_tags_7 },
    { 0x6ddc24a9, SUC_lexicon_tags_65 },
    { 0x68dc24aa, SUC_lexicon_tags_14 },
    { 0x98dc24a8, SUC_lexicon_tags_0 },
    { 0xc4fc24ac, SUC_lexicon_tags_37 },
    { 0x955c24a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x285c24af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdc24b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bc24c3, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x6b1c24c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63fc24c7, SUC_lexicon_tags_32 },
    { 0x935c24c8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fc24ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x599c24d0, SUC_lexicon_tags_0 },
    { 0xb5fc24d0, SUC_lexicon_tags_7 },
    { 0xfe1c24d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfdbc24d4, SUC_lexicon_tags_27 },
    { 0x419c24d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877c24d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8fc24db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd87c24dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7c24e0, SUC_lexicon_tags_7 },
    { 0xd35c24e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x399c24e3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145c24e7, SUC_lexicon_tags_2 },
    { 0xcbc24e8, SUC_lexicon_tags_47 },
    { 0xb43c24e9, SUC_lexicon_tags_37 },
    { 0x955c24e8, SUC_lexicon_tags_92 },
    { 0xf2fc24e8, SUC_lexicon_tags_155 },
    { 0x3d5c24ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda3c24ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01c24f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c24f5, SUC_lexicon_tags_1 },
    { 0x8b3c24f5, SUC_lexicon_tags_25 },
    { 0x66fc24f7, SUC_lexicon_tags_19 },
    { 0x291c24f8, SUC_lexicon_tags_15 },
    { 0x211c24f9, SUC_lexicon_tags_63 },
    { 0x43dc24f9, SUC_lexicon_tags_23 },
    { 0x2e9c24fa, SUC_lexicon_tags_11 },
    { 0x1f3c24f7, SUC_lexicon_tags_4 },
    { 0x735c24fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x733c24ff, SUC_lexicon_tags_4 },
    { 0xdbfc2500, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249c2504, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3c2508, SUC_lexicon_tags_43 },
    { 0xd47c2509, SUC_lexicon_tags_38 },
    { 0x407c250a, SUC_lexicon_tags_27 },
    { 0x315c250b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbb5c250d, SUC_lexicon_tags_32 },
    { 0x313c250e, SUC_lexicon_tags_11 },
    { 0x1a1c250d, SUC_lexicon_tags_25 },
    { 0x1bbc2510, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdc2516, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1c2519, SUC_lexicon_tags_11 },
    { 0x9c7c251a, SUC_lexicon_tags_26 },
    { 0xedfc251b, SUC_lexicon_tags_14 },
    { 0x7f9c251c, SUC_lexicon_tags_11 },
    { 0xf77c251c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417c2529, SUC_lexicon_tags_30 },
    { 0xb71c252a, SUC_lexicon_tags_7 },
    { 0x9f5c252a, SUC_lexicon_tags_6 },
    { 0x78dc252a, SUC_lexicon_tags_40 },
    { 0xf23c252c, SUC_lexicon_tags_38 },
    { 0x33bc252c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3c2537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca9c2539, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x78fc253b, SUC_lexicon_tags_37 },
    { 0x603c253b, SUC_lexicon_tags_29 },
    { 0xaebc253b, SUC_lexicon_tags_6 },
    { 0x457c253e, SUC_lexicon_tags_33 },
    { 0xcdfc253e, SUC_lexicon_tags_11 },
    { 0x87bc2540, SUC_lexicon_tags_37 },
    { 0xb7fc2540, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1c2544, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21bc2547, SUC_lexicon_tags_8 },
    { 0x363c2547, SUC_lexicon_tags_63 },
    { 0xbe5c2549, SUC_lexicon_tags_13 },
    { 0x757c2547, SUC_lexicon_tags_17 },
    { 0xc3bc2549, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c254e, SUC_lexicon_tags_11 },
    { 0x629c254e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c255a, SUC_lexicon_tags_43 },
    { 0xe31c255a, SUC_lexicon_tags_17 },
    { 0xdd9c255c, SUC_lexicon_tags_7 },
    { 0xa7dc255d, SUC_lexicon_tags_11 },
    { 0x927c255d, SUC_lexicon_tags_84 },
    { 0xcddc255f, SUC_lexicon_tags_7 },
    { 0x3fc255f, SUC_lexicon_tags_11 },
    { 0x525c2561, SUC_lexicon_tags_1 },
    { 0x8cfc255d, SUC_lexicon_tags_61 },
    { 0x7fdc255f, SUC_lexicon_tags_23 },
    { 0x81dc255d, SUC_lexicon_tags_32 },
    { 0x27fc2565, SUC_lexicon_tags_26 },
    { 0x49bc2566, SUC_lexicon_tags_104 },
    { 0xc79c2567, SUC_lexicon_tags_11 },
    { 0x8c7c255f, SUC_lexicon_tags_25 },
    { 0x969c2569, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x967c256b, SUC_lexicon_tags_27 },
    { 0x6c1c256c, SUC_lexicon_tags_11 },
    { 0xfe5c256d, SUC_lexicon_tags_1 },
    { 0x53fc256b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fc2578, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x933c257a, SUC_lexicon_tags_12 },
    { 0x8ddc257a, SUC_lexicon_tags_11 },
    { 0xe1fc257c, SUC_lexicon_tags_48 },
    { 0x407c257c, SUC_lexicon_tags_2 },
    { 0x429c257e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51c2583, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627c2588, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9bc258a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb5c258c, SUC_lexicon_tags_47 },
    { 0x1cdc258d, SUC_lexicon_tags_7 },
    { 0x31bc258c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd77c2590, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47c2597, SUC_lexicon_tags_30 },
    { 0x1efc2598, SUC_lexicon_tags_25 },
    { 0x10dc2599, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x489c259b, SUC_lexicon_tags_25 },
    { 0x7bdc259c, SUC_lexicon_tags_11 },
    { 0xcd5c259c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc83c259f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xaefc25a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe2dc25a3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5c25a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x709c25a8, SUC_lexicon_tags_25 },
    { 0x5a1c25a8, SUC_lexicon_tags_9 },
    { 0x5bdc25aa, SUC_lexicon_tags_2 },
    { 0xb31c25a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x791c25ad, SUC_lexicon_tags_17 },
    { 0x2a7c25ae, SUC_lexicon_tags_7 },
    { 0x15dc25ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9c25b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb93c25b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3c25b9, SUC_lexicon_tags_28 },
    { 0x9cdc25ba, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c25c0, SUC_lexicon_tags_12 },
    { 0x9c25c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa41c25c3, SUC_lexicon_tags_9 },
    { 0x709c25c3, SUC_lexicon_tags_7 },
    { 0xafc25c5, SUC_lexicon_tags_94 },
    { 0x2afc25c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59c25cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bc25cf, SUC_lexicon_tags_25 },
    { 0x42fc25d0, SUC_lexicon_tags_40 },
    { 0xce5c25d1, SUC_lexicon_tags_12 },
    { 0xdf3c25d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fc25d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfc25db, SUC_lexicon_tags_131 },
    { 0x2cbc25dc, SUC_lexicon_tags_38 },
    { 0x223c25dd, SUC_lexicon_tags_1 },
    { 0xd4dc25dc, SUC_lexicon_tags_0 },
    { 0xb41c25dc, SUC_lexicon_tags_61 },
    { 0x46fc25e0, SUC_lexicon_tags_57 },
    { 0xdd5c25e0, SUC_lexicon_tags_6 },
    { 0xfefc25e2, SUC_lexicon_tags_45 },
    { 0x4f9c25e1, SUC_lexicon_tags_25 },
    { 0x5dc25df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c25eb, SUC_lexicon_tags_12 },
    { 0x18dc25ec, SUC_lexicon_tags_15 },
    { 0x5fbc25ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3c25f4, SUC_lexicon_tags_15 },
    { 0xa79c25f5, SUC_lexicon_tags_11 },
    { 0xf7c25f6, SUC_lexicon_tags_47 },
    { 0xa8bc25f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fc25fa, SUC_lexicon_tags_8 },
    { 0xa51c25fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf1c25fd, SUC_lexicon_tags_63 },
    { 0xc03c25fe, SUC_lexicon_tags_351 },
    { 0x671c25fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97c2609, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581c260e, SUC_lexicon_tags_33 },
    { 0x79c260f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd69c2611, SUC_lexicon_tags_3 },
    { 0x98fc2612, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x85c2614, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe9c2616, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x60fc2618, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bc261e, SUC_lexicon_tags_27 },
    { 0x603c261e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x21fc2621, SUC_lexicon_tags_17 },
    { 0x10bc2622, SUC_lexicon_tags_60 },
    { 0x433c2623, SUC_lexicon_tags_1 },
    { 0xd19c2624, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x803c2627, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x701c262a, SUC_lexicon_tags_37 },
    { 0x635c262b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677c2630, SUC_lexicon_tags_7 },
    { 0x46bc2630, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333c2638, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x59dc263b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x107c263d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadc2640, SUC_lexicon_tags_30 },
    { 0x3ffc2640, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1c2646, SUC_lexicon_tags_12 },
    { 0xf99c2647, SUC_lexicon_tags_25 },
    { 0xd99c2648, SUC_lexicon_tags_37 },
    { 0x809c2649, SUC_lexicon_tags_30 },
    { 0x1ebc264a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbc264d, SUC_lexicon_tags_4 },
    { 0x50dc264e, SUC_lexicon_tags_37 },
    { 0xbf3c264d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x205c2651, SUC_lexicon_tags_25 },
    { 0xbfdc2652, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369c2657, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c2668, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249c266d, SUC_lexicon_tags_15 },
    { 0x2d9c266e, SUC_lexicon_tags_8 },
    { 0xfc9c266d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9c2672, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x503c2674, SUC_lexicon_tags_6 },
    { 0xc93c2675, SUC_lexicon_tags_15 },
    { 0xe6bc2676, SUC_lexicon_tags_15 },
    { 0x9a7c2676, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x37bc2679, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5abc267b, SUC_lexicon_tags_9 },
    { 0x34fc267b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc2680, SUC_lexicon_tags_9 },
    { 0x91c2681, SUC_lexicon_tags_23 },
    { 0xcbc2682, SUC_lexicon_tags_2 },
    { 0x5ddc2680, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dc268a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fc268f, SUC_lexicon_tags_7 },
    { 0x75fc2690, SUC_lexicon_tags_20 },
    { 0xfc9c2690, SUC_lexicon_tags_1 },
    { 0x8e1c2691, SUC_lexicon_tags_37 },
    { 0xd07c2691, SUC_lexicon_tags_30 },
    { 0x5cbc2694, SUC_lexicon_tags_15 },
    { 0x29c2690, SUC_lexicon_tags_37 },
    { 0xca5c2696, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x649c2698, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9c26a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x70bc26a2, SUC_lexicon_tags_25 },
    { 0xd99c26a3, SUC_lexicon_tags_7 },
    { 0x6b5c26a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15c26aa, SUC_lexicon_tags_6 },
    { 0x5ddc26ab, SUC_lexicon_tags_12 },
    { 0xf69c26aa, SUC_lexicon_tags_37 },
    { 0x957c26ad, SUC_lexicon_tags_33 },
    { 0xedfc26ae, SUC_lexicon_tags_37 },
    { 0xaa1c26af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x273c26b1, SUC_lexicon_tags_2 },
    { 0x4f7c26b2, SUC_lexicon_tags_63 },
    { 0xa9dc26b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x445c26b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bc26bb, SUC_lexicon_tags_37 },
    { 0x1f3c26bc, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adc26c0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddc26c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459c26c8, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c26cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03c26d6, SUC_lexicon_tags_7 },
    { 0x5d1c26d7, SUC_lexicon_tags_2 },
    { 0xcb5c26d8, SUC_lexicon_tags_33 },
    { 0x1edc26d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf97c26db, SUC_lexicon_tags_29 },
    { 0x6b1c26db, SUC_lexicon_tags_17 },
    { 0x35fc26db, SUC_lexicon_tags_11 },
    { 0x521c26db, SUC_lexicon_tags_49 },
    { 0x9bbc26df, SUC_lexicon_tags_42 },
    { 0x455c26e0, SUC_lexicon_tags_17 },
    { 0x7b5c26e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4f5c26e3, SUC_lexicon_tags_14 },
    { 0xf2fc26e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847c26ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddc26ed, SUC_lexicon_tags_17 },
    { 0x97fc26ee, SUC_lexicon_tags_49 },
    { 0xaa1c26ef, SUC_lexicon_tags_11 },
    { 0xff9c26ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bc26f3, SUC_lexicon_tags_37 },
    { 0xdbbc26f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc26f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x721c26fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725c26fe, SUC_lexicon_tags_2 },
    { 0xb61c26ff, SUC_lexicon_tags_38 },
    { 0x5fc2700, SUC_lexicon_tags_26 },
    { 0xc49c2701, SUC_lexicon_tags_39 },
    { 0xe81c2701, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3c2706, SUC_lexicon_tags_25 },
    { 0x90fc2706, SUC_lexicon_tags_25 },
    { 0x185c2708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x197c270b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fc2710, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433c2718, SUC_lexicon_tags_43 },
    { 0xdb9c2718, SUC_lexicon_tags_30 },
    { 0xb05c271a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x155c271c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83c271f, SUC_lexicon_tags_37 },
    { 0x27fc2720, SUC_lexicon_tags_74 },
    { 0xf7c2721, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491c2727, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3c272a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfc272e, SUC_lexicon_tags_35 },
    { 0xaefc272e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdc2734, SUC_lexicon_tags_25 },
    { 0x547c2735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x639c2738, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7ebc273a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5c2741, SUC_lexicon_tags_12 },
    { 0x86bc2742, SUC_lexicon_tags_1 },
    { 0xff3c2743, SUC_lexicon_tags_0 },
    { 0x7f1c2743, SUC_lexicon_tags_17 },
    { 0x12dc2745, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1c274b, SUC_lexicon_tags_1 },
    { 0x3b3c274b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7c2751, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa05c2753, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbc275f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc91c2761, SUC_lexicon_tags_1 },
    { 0xe73c2762, SUC_lexicon_tags_29 },
    { 0x2bc2763, SUC_lexicon_tags_14 },
    { 0xbffc2764, SUC_lexicon_tags_15 },
    { 0xebdc2765, SUC_lexicon_tags_12 },
    { 0xcdbc2766, SUC_lexicon_tags_25 },
    { 0x45dc2764, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdc276d, SUC_lexicon_tags_27 },
    { 0xdcbc276d, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69c2777, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x479c2779, SUC_lexicon_tags_65 },
    { 0x50bc277a, SUC_lexicon_tags_29 },
    { 0xcedc277b, SUC_lexicon_tags_25 },
    { 0xfbbc277c, SUC_lexicon_tags_28 },
    { 0x4cdc277d, SUC_lexicon_tags_11 },
    { 0x725c277d, SUC_lexicon_tags_17 },
    { 0x84fc277c, SUC_lexicon_tags_0 },
    { 0x8a3c2780, SUC_lexicon_tags_1 },
    { 0x8cdc2780, SUC_lexicon_tags_38 },
    { 0xaa9c277f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c2788, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79c278f, SUC_lexicon_tags_56 },
    { 0x30fc278f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f3c2792, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x747c2794, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3c2797, SUC_lexicon_tags_26 },
    { 0x525c2797, SUC_lexicon_tags_14 },
    { 0x565c2797, SUC_lexicon_tags_25 },
    { 0xe69c2798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbc279e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5c27a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88fc27a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dc27ae, SUC_lexicon_tags_31 },
    { 0xc13c27af, SUC_lexicon_tags_15 },
    { 0x1c9c27b0, SUC_lexicon_tags_7 },
    { 0x5f3c27b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd77c27b3, SUC_lexicon_tags_15 },
    { 0x90dc27b4, SUC_lexicon_tags_37 },
    { 0x63fc27b5, SUC_lexicon_tags_60 },
    { 0xaafc27b6, SUC_lexicon_tags_38 },
    { 0xabbc27b4, SUC_lexicon_tags_28 },
    { 0x611c27b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x591c27bb, SUC_lexicon_tags_61 },
    { 0xf23c27bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37c27c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fbc27c9, SUC_lexicon_tags_15 },
    { 0x11dc27ca, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xd27c27cc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x329c27ce, SUC_lexicon_tags_19 },
    { 0xc1dc27cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d5c27d1, SUC_lexicon_tags_15 },
    { 0xa2fc27d1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x679c27d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedbc27d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bc27dc, SUC_lexicon_tags_30 },
    { 0xc03c27dc, SUC_lexicon_tags_17 },
    { 0x35c27de, SUC_lexicon_tags_50 },
    { 0xd27c27df, SUC_lexicon_tags_26 },
    { 0x491c27dd, SUC_lexicon_tags_14 },
    { 0xba1c27dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa15c27e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bc27e6, SUC_lexicon_tags_29 },
    { 0x41fc27e7, SUC_lexicon_tags_0 },
    { 0x251c27e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc27eb, SUC_lexicon_tags_63 },
    { 0xe3fc27ec, SUC_lexicon_tags_25 },
    { 0x6a9c27ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x747c27f0, SUC_lexicon_tags_17 },
    { 0x39fc27f1, SUC_lexicon_tags_15 },
    { 0x3f9c27f0, SUC_lexicon_tags_19 },
    { 0x7ddc27f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcabc27f5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfc27fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ddc27fd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe15c27ff, SUC_lexicon_tags_31 },
    { 0x6f7c2800, SUC_lexicon_tags_25 },
    { 0x4dbc2801, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c2805, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x655c2807, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577c280e, SUC_lexicon_tags_56 },
    { 0xe97c280f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443c2813, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46bc2815, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87c281c, SUC_lexicon_tags_65 },
    { 0xc5bc281d, SUC_lexicon_tags_1 },
    { 0x231c281e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67c2821, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5c2824, SUC_lexicon_tags_17 },
    { 0x9afc2825, SUC_lexicon_tags_19 },
    { 0x8afc2826, SUC_lexicon_tags_14 },
    { 0xa5fc2827, SUC_lexicon_tags_12 },
    { 0x8dbc2827, SUC_lexicon_tags_16 },
    { 0x649c2828, SUC_lexicon_tags_2 },
    { 0xe3fc2826, SUC_lexicon_tags_28 },
    { 0xbc1c282a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849c2830, SUC_lexicon_tags_27 },
    { 0x503c2831, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643c2835, SUC_lexicon_tags_11 },
    { 0xeffc2835, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73c283a, SUC_lexicon_tags_1 },
    { 0xef9c283a, SUC_lexicon_tags_17 },
    { 0xffc283c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efc2841, SUC_lexicon_tags_37 },
    { 0xff3c2841, SUC_lexicon_tags_17 },
    { 0x58fc2843, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8b5c2845, SUC_lexicon_tags_9 },
    { 0x191c2846, SUC_lexicon_tags_11 },
    { 0xb3c2845, SUC_lexicon_tags_15 },
    { 0xcabc2848, SUC_lexicon_tags_7 },
    { 0x19bc2848, SUC_lexicon_tags_68 },
    { 0x62bc284a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffc284d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c2850, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bc2858, SUC_lexicon_tags_23 },
    { 0x17fc2859, SUC_lexicon_tags_63 },
    { 0x303c285a, SUC_lexicon_tags_26 },
    { 0xb23c2859, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1e7c285d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7c2860, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2fc2862, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x3b7c2864, SUC_lexicon_tags_12 },
    { 0x609c2865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbc286a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c286e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bc2872, SUC_lexicon_tags_31 },
    { 0x9adc2872, SUC_lexicon_tags_15 },
    { 0xc37c2872, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd01c2876, SUC_lexicon_tags_68 },
    { 0x9fc2877, SUC_lexicon_tags_253 },
    { 0x7f7c2876, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x73c287a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec9c287c, SUC_lexicon_tags_12 },
    { 0x323c287d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadbc287f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31c2885, SUC_lexicon_tags_47 },
    { 0x9d7c2886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x893c2889, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fc288f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd13c2891, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67c2896, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfc2899, SUC_lexicon_tags_49 },
    { 0xcbfc2899, SUC_lexicon_tags_14 },
    { 0x855c289b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c289e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe57c28a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x253c28a2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7c28a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f7c28a9, SUC_lexicon_tags_11 },
    { 0xb43c28aa, SUC_lexicon_tags_15 },
    { 0x24bc28ab, SUC_lexicon_tags_25 },
    { 0x713c28aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea1c28ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5c28b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bc28b5, SUC_lexicon_tags_1 },
    { 0x507c28b6, SUC_lexicon_tags_15 },
    { 0x69fc28b7, SUC_lexicon_tags_240 },
    { 0, NULL },
    { 0x8f9c28b9, SUC_lexicon_tags_25 },
    { 0xf21c28ba, SUC_lexicon_tags_17 },
    { 0xd71c28bb, SUC_lexicon_tags_1 },
    { 0x8fc28b9, SUC_lexicon_tags_11 },
    { 0xdb7c28bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89c28c0, SUC_lexicon_tags_12 },
    { 0xc6dc28c1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebc28c5, SUC_lexicon_tags_12 },
    { 0x689c28c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xccdc28c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03c28cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fc28d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x659c28da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bc28de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9c28e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedc28e9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edc28ef, SUC_lexicon_tags_27 },
    { 0x4a7c28ef, SUC_lexicon_tags_14 },
    { 0xa41c28ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x19fc28f3, SUC_lexicon_tags_37 },
    { 0x8f3c28f4, SUC_lexicon_tags_15 },
    { 0x6adc28f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x10fc28f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc29c28f9, SUC_lexicon_tags_92 },
    { 0xc65c28fa, SUC_lexicon_tags_37 },
    { 0x595c28fa, SUC_lexicon_tags_9 },
    { 0x6adc28fc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969c2901, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73c2907, SUC_lexicon_tags_49 },
    { 0xe13c2908, SUC_lexicon_tags_31 },
    { 0xc4bc2909, SUC_lexicon_tags_30 },
    { 0xbc1c2909, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abc2917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x853c2919, SUC_lexicon_tags_12 },
    { 0xb95c291a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0dc291c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5c2922, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcc5c2924, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xefdc2926, SUC_lexicon_tags_42 },
    { 0x1e9c2927, SUC_lexicon_tags_39 },
    { 0x1efc2927, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9c292d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd57c292f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebc293a, SUC_lexicon_tags_7 },
    { 0x4dc293a, SUC_lexicon_tags_12 },
    { 0xad7c293a, SUC_lexicon_tags_9 },
    { 0xe8fc293a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109c2941, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149c2948, SUC_lexicon_tags_23 },
    { 0xb29c2948, SUC_lexicon_tags_11 },
    { 0xdb7c294a, SUC_lexicon_tags_45 },
    { 0x6adc294a, SUC_lexicon_tags_7 },
    { 0x349c294c, SUC_lexicon_tags_37 },
    { 0x3c1c294d, SUC_lexicon_tags_2 },
    { 0xd2bc294e, SUC_lexicon_tags_17 },
    { 0x801c294e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bc2956, SUC_lexicon_tags_7 },
    { 0x203c2957, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351c295c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x673c295e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x859c2960, SUC_lexicon_tags_23 },
    { 0x89bc2961, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb2bc2963, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x755c2966, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07c2969, SUC_lexicon_tags_12 },
    { 0x6f3c296a, SUC_lexicon_tags_34 },
    { 0xad9c296a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dc2971, SUC_lexicon_tags_9 },
    { 0x3c3c2971, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fc2975, SUC_lexicon_tags_11 },
    { 0x4b9c2975, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfc297e, SUC_lexicon_tags_179 },
    { 0x129c297f, SUC_lexicon_tags_11 },
    { 0xf65c2980, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x15bc2982, SUC_lexicon_tags_31 },
    { 0x8cbc2983, SUC_lexicon_tags_17 },
    { 0x32bc2984, SUC_lexicon_tags_25 },
    { 0xf19c2984, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b5c2987, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5c298c, SUC_lexicon_tags_69 },
    { 0xc09c298c, SUC_lexicon_tags_34 },
    { 0x105c298c, SUC_lexicon_tags_7 },
    { 0xd7fc298d, SUC_lexicon_tags_32 },
    { 0xc67c298c, SUC_lexicon_tags_2 },
    { 0x2b5c2991, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3c2995, SUC_lexicon_tags_58 },
    { 0xcedc2995, SUC_lexicon_tags_7 },
    { 0x599c2996, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x61c2999, SUC_lexicon_tags_37 },
    { 0x8b3c299a, SUC_lexicon_tags_17 },
    { 0x77c299b, SUC_lexicon_tags_1 },
    { 0x35c299c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x787c299e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2c9c29a0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x387c29a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fc29a5, SUC_lexicon_tags_25 },
    { 0xfadc29a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7c29aa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fc29ad, SUC_lexicon_tags_23 },
    { 0xfedc29ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09c29b1, SUC_lexicon_tags_25 },
    { 0xbf5c29b2, SUC_lexicon_tags_10 },
    { 0xc1fc29b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69c29b5, SUC_lexicon_tags_17 },
    { 0x71bc29b5, SUC_lexicon_tags_27 },
    { 0x6cbc29b7, SUC_lexicon_tags_25 },
    { 0xd1fc29b7, SUC_lexicon_tags_12 },
    { 0xfbdc29b8, SUC_lexicon_tags_25 },
    { 0x863c29b9, SUC_lexicon_tags_8 },
    { 0xc7bc29b9, SUC_lexicon_tags_29 },
    { 0x3c1c29bc, SUC_lexicon_tags_39 },
    { 0x5f3c29bd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53dc29bf, SUC_lexicon_tags_31 },
    { 0x13dc29c0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x78bc29c2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88dc29ca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bc29cd, SUC_lexicon_tags_62 },
    { 0x2afc29cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41c29d4, SUC_lexicon_tags_38 },
    { 0x719c29d5, SUC_lexicon_tags_2 },
    { 0x1e9c29d5, SUC_lexicon_tags_4 },
    { 0x821c29d7, SUC_lexicon_tags_42 },
    { 0xca1c29d7, SUC_lexicon_tags_7 },
    { 0xf6dc29d9, SUC_lexicon_tags_1 },
    { 0x7fbc29d6, SUC_lexicon_tags_48 },
    { 0x645c29db, SUC_lexicon_tags_37 },
    { 0x87bc29db, SUC_lexicon_tags_6 },
    { 0x241c29db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3c29e0, SUC_lexicon_tags_0 },
    { 0xd83c29e0, SUC_lexicon_tags_12 },
    { 0x10fc29e0, SUC_lexicon_tags_7 },
    { 0x113c29e3, SUC_lexicon_tags_11 },
    { 0x45dc29e4, SUC_lexicon_tags_0 },
    { 0xae5c29e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3c29e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x909c29ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x30bc29ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fc29f5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533c29fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc2a08, SUC_lexicon_tags_47 },
    { 0xbd3c2a08, SUC_lexicon_tags_17 },
    { 0x655c2a0a, SUC_lexicon_tags_68 },
    { 0x42dc2a0b, SUC_lexicon_tags_17 },
    { 0x8a1c2a0c, SUC_lexicon_tags_25 },
    { 0xf1dc2a0d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd7bc2a0f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fc2a16, SUC_lexicon_tags_15 },
    { 0xda3c2a16, SUC_lexicon_tags_13 },
    { 0x945c2a18, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fc2a1e, SUC_lexicon_tags_35 },
    { 0x3dbc2a1f, SUC_lexicon_tags_11 },
    { 0x583c2a20, SUC_lexicon_tags_46 },
    { 0xb7fc2a21, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709c2a29, SUC_lexicon_tags_2 },
    { 0x21dc2a2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37c2a2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dc2a31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdc2a34, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519c2a38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefc2a3c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9efc2a3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x599c2a41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c2a44, SUC_lexicon_tags_25 },
    { 0x2d7c2a45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125c2a49, SUC_lexicon_tags_11 },
    { 0x365c2a49, SUC_lexicon_tags_63 },
    { 0x733c2a4b, SUC_lexicon_tags_61 },
    { 0x2c1c2a4c, SUC_lexicon_tags_34 },
    { 0x3f9c2a4d, SUC_lexicon_tags_80 },
    { 0xaf3c2a4d, SUC_lexicon_tags_14 },
    { 0x3bdc2a4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353c2a56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x803c2a59, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x543c2a5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c2a5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x69dc2a62, SUC_lexicon_tags_11 },
    { 0x2e9c2a63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25c2a66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5dc2a68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bc2a6c, SUC_lexicon_tags_15 },
    { 0xd3c2a6d, SUC_lexicon_tags_23 },
    { 0xb75c2a6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47c2a74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfdc2a76, SUC_lexicon_tags_17 },
    { 0x7bfc2a76, SUC_lexicon_tags_38 },
    { 0x51dc2a78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd69c2a7a, SUC_lexicon_tags_11 },
    { 0xa65c2a7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x73dc2a7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x205c2a81, SUC_lexicon_tags_31 },
    { 0xbcdc2a82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x823c2a85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63c2a88, SUC_lexicon_tags_17 },
    { 0xc27c2a89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d3c2a8b, SUC_lexicon_tags_14 },
    { 0x86bc2a8b, SUC_lexicon_tags_37 },
    { 0xd9bc2a8d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x795c2a90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9c2a94, SUC_lexicon_tags_49 },
    { 0x9c9c2a95, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa17c2a97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7c2a9a, SUC_lexicon_tags_17 },
    { 0x8bfc2a9a, SUC_lexicon_tags_48 },
    { 0x933c2a9b, SUC_lexicon_tags_25 },
    { 0x445c2a9d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x343c2a9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19c2aa3, SUC_lexicon_tags_49 },
    { 0x89bc2aa4, SUC_lexicon_tags_19 },
    { 0x61dc2aa5, SUC_lexicon_tags_23 },
    { 0x4dfc2aa5, SUC_lexicon_tags_15 },
    { 0x4edc2aa7, SUC_lexicon_tags_28 },
    { 0x637c2aa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbc2aab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adc2ab0, SUC_lexicon_tags_16 },
    { 0x107c2ab1, SUC_lexicon_tags_17 },
    { 0x5fbc2ab2, SUC_lexicon_tags_17 },
    { 0x8b9c2ab2, SUC_lexicon_tags_25 },
    { 0xf83c2ab4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43c2ab7, SUC_lexicon_tags_17 },
    { 0xebdc2ab8, SUC_lexicon_tags_48 },
    { 0x89dc2ab9, SUC_lexicon_tags_7 },
    { 0x1a1c2aba, SUC_lexicon_tags_7 },
    { 0xee3c2ab8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x53bc2abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bc2ac3, SUC_lexicon_tags_1 },
    { 0x1bdc2ac4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bc2ac7, SUC_lexicon_tags_43 },
    { 0xc27c2ac8, SUC_lexicon_tags_25 },
    { 0x12fc2ac9, SUC_lexicon_tags_12 },
    { 0x925c2ac9, SUC_lexicon_tags_7 },
    { 0x381c2acb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdc2acf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5c2ad2, SUC_lexicon_tags_9 },
    { 0xcbbc2ad3, SUC_lexicon_tags_11 },
    { 0x6c7c2ad4, SUC_lexicon_tags_29 },
    { 0xdddc2ad5, SUC_lexicon_tags_37 },
    { 0xf67c2ad3, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x5c3c2ad8, SUC_lexicon_tags_23 },
    { 0xa0fc2ad8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x56bc2adb, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x889c2ade, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x161c2ae1, SUC_lexicon_tags_45 },
    { 0xe4fc2ae1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5c2ae6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7c2af3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7c2af9, SUC_lexicon_tags_0 },
    { 0xb83c2afa, SUC_lexicon_tags_37 },
    { 0x583c2afa, SUC_lexicon_tags_25 },
    { 0x573c2afc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a1c2afe, SUC_lexicon_tags_0 },
    { 0xe83c2aff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bc2b02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a5c2b04, SUC_lexicon_tags_45 },
    { 0x77c2b05, SUC_lexicon_tags_15 },
    { 0xab7c2b05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c2b09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63fc2b0b, SUC_lexicon_tags_0 },
    { 0x629c2b0c, SUC_lexicon_tags_15 },
    { 0x1bfc2b0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547c2b11, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x32bc2b13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dc2b16, SUC_lexicon_tags_46 },
    { 0x479c2b17, SUC_lexicon_tags_60 },
    { 0xc15c2b17, SUC_lexicon_tags_25 },
    { 0x6a9c2b19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dc2b1f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb97c2b21, SUC_lexicon_tags_26 },
    { 0x993c2b21, SUC_lexicon_tags_25 },
    { 0x4f1c2b23, SUC_lexicon_tags_13 },
    { 0xae1c2b23, SUC_lexicon_tags_35 },
    { 0xc7c2b25, SUC_lexicon_tags_31 },
    { 0xe7dc2b25, SUC_lexicon_tags_37 },
    { 0x337c2b26, SUC_lexicon_tags_37 },
    { 0xfedc2b28, SUC_lexicon_tags_63 },
    { 0xde5c2b28, SUC_lexicon_tags_12 },
    { 0x17dc2b28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dc2b2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c1c2b31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9c2b34, SUC_lexicon_tags_2 },
    { 0xe2fc2b35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bc2b3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdc2b42, SUC_lexicon_tags_14 },
    { 0xd9dc2b43, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbc2b4a, SUC_lexicon_tags_15 },
    { 0xc2fc2b4b, SUC_lexicon_tags_25 },
    { 0xf05c2b4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30fc2b4e, SUC_lexicon_tags_45 },
    { 0xd65c2b4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7c2b52, SUC_lexicon_tags_0 },
    { 0x85bc2b52, SUC_lexicon_tags_37 },
    { 0x167c2b54, SUC_lexicon_tags_37 },
    { 0x473c2b52, SUC_lexicon_tags_0 },
    { 0x1f3c2b56, SUC_lexicon_tags_16 },
    { 0xef9c2b57, SUC_lexicon_tags_14 },
    { 0x9afc2b58, SUC_lexicon_tags_70 },
    { 0xfebc2b59, SUC_lexicon_tags_37 },
    { 0x957c2b5a, SUC_lexicon_tags_17 },
    { 0xd69c2b5b, SUC_lexicon_tags_45 },
    { 0x50fc2b5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21c2b60, SUC_lexicon_tags_243 },
    { 0x97fc2b60, SUC_lexicon_tags_87 },
    { 0xf8dc2b60, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323c2b66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x93dc2b69, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91c2b71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbdbc2b73, SUC_lexicon_tags_10 },
    { 0xf95c2b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdc2b79, SUC_lexicon_tags_34 },
    { 0xe1fc2b79, SUC_lexicon_tags_23 },
    { 0x359c2b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x453c2b7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03c2b81, SUC_lexicon_tags_0 },
    { 0x157c2b82, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4b5c2b84, SUC_lexicon_tags_14 },
    { 0xd1dc2b85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7c2b89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193c2b8d, SUC_lexicon_tags_47 },
    { 0x6e3c2b8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3c2b93, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17c2b9a, SUC_lexicon_tags_7 },
    { 0x781c2b9a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a3c2b9d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9c2ba0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x767c2ba3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x557c2ba6, SUC_lexicon_tags_2 },
    { 0x8f9c2ba7, SUC_lexicon_tags_21 },
    { 0xf6dc2ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbc2bad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bc2bb2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd83c2bb4, SUC_lexicon_tags_43 },
    { 0x1efc2bb4, SUC_lexicon_tags_23 },
    { 0x935c2bb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223c2bba, SUC_lexicon_tags_17 },
    { 0xa77c2bba, SUC_lexicon_tags_7 },
    { 0xa79c2bbc, SUC_lexicon_tags_25 },
    { 0x1dc2bbd, SUC_lexicon_tags_7 },
    { 0x179c2bbe, SUC_lexicon_tags_19 },
    { 0x853c2bbf, SUC_lexicon_tags_10 },
    { 0xa4fc2bc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1b7c2bc2, SUC_lexicon_tags_19 },
    { 0xbafc2bc3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dc2bc7, SUC_lexicon_tags_37 },
    { 0x809c2bc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ffc2bca, SUC_lexicon_tags_11 },
    { 0xc4bc2bca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c2bce, SUC_lexicon_tags_10 },
    { 0x443c2bcf, SUC_lexicon_tags_11 },
    { 0xc31c2bd0, SUC_lexicon_tags_25 },
    { 0xdb5c2bd0, SUC_lexicon_tags_15 },
    { 0xb59c2bd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fc2bd5, SUC_lexicon_tags_8 },
    { 0xbadc2bd6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x973c2bd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7dc2bdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317c2be0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7c2be2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34dc2be4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bc2be9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x407c2bec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x61bc2bee, SUC_lexicon_tags_261 },
    { 0x693c2bef, SUC_lexicon_tags_1 },
    { 0x989c2bf0, SUC_lexicon_tags_12 },
    { 0x62dc2bf1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29c2bf9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33c2c05, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575c2c09, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c2c0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33c2c11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fc2c16, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bc2c1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb3c2c1c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x351c2c1f, SUC_lexicon_tags_31 },
    { 0xb59c2c20, SUC_lexicon_tags_1 },
    { 0xe47c2c20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c2c29, SUC_lexicon_tags_43 },
    { 0x13fc2c2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa7bc2c2c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdc2c32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfc2c3b, SUC_lexicon_tags_1 },
    { 0x937c2c3c, SUC_lexicon_tags_2 },
    { 0x29fc2c3d, SUC_lexicon_tags_38 },
    { 0x79dc2c3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c2c41, SUC_lexicon_tags_17 },
    { 0x105c2c42, SUC_lexicon_tags_25 },
    { 0xf95c2c43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe1dc2c45, SUC_lexicon_tags_12 },
    { 0x747c2c45, SUC_lexicon_tags_27 },
    { 0x2fc2c47, SUC_lexicon_tags_15 },
    { 0x155c2c48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d5c2c4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1c2c50, SUC_lexicon_tags_30 },
    { 0x777c2c51, SUC_lexicon_tags_17 },
    { 0xbb3c2c52, SUC_lexicon_tags_35 },
    { 0x6ddc2c51, SUC_lexicon_tags_1247 },
    { 0xa73c2c53, SUC_lexicon_tags_2 },
    { 0xe15c2c50, SUC_lexicon_tags_30 },
    { 0xe63c2c56, SUC_lexicon_tags_0 },
    { 0xeb7c2c56, SUC_lexicon_tags_0 },
    { 0x235c2c57, SUC_lexicon_tags_30 },
    { 0x5d5c2c58, SUC_lexicon_tags_49 },
    { 0x485c2c59, SUC_lexicon_tags_15 },
    { 0x525c2c52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bc2c5f, SUC_lexicon_tags_49 },
    { 0x49bc2c60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515c2c68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29c2c6b, SUC_lexicon_tags_14 },
    { 0xf6bc2c6c, SUC_lexicon_tags_26 },
    { 0xf57c2c6c, SUC_lexicon_tags_17 },
    { 0xfbfc2c6e, SUC_lexicon_tags_27 },
    { 0xb3bc2c6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5c2c73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x957c2c75, SUC_lexicon_tags_37 },
    { 0x699c2c75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x439c2c79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd81c2c7b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03c2c7e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5c2c86, SUC_lexicon_tags_2 },
    { 0xb1bc2c87, SUC_lexicon_tags_21 },
    { 0xff9c2c86, SUC_lexicon_tags_11 },
    { 0x61fc2c89, SUC_lexicon_tags_32 },
    { 0x22fc2c8a, SUC_lexicon_tags_6 },
    { 0xc85c2c88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3c2c8f, SUC_lexicon_tags_11 },
    { 0xfdbc2c90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82dc2c92, SUC_lexicon_tags_15 },
    { 0x275c2c93, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x4c7c2c95, SUC_lexicon_tags_11 },
    { 0x5e1c2c96, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13c2c9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01c2c9f, SUC_lexicon_tags_37 },
    { 0x1ebc2c9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75dc2ca5, SUC_lexicon_tags_12 },
    { 0x71bc2ca6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfbbc2ca8, SUC_lexicon_tags_1 },
    { 0xe9c2ca9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dc2cb2, SUC_lexicon_tags_48 },
    { 0x9efc2cb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65c2cb9, SUC_lexicon_tags_31 },
    { 0x6cbc2cb9, SUC_lexicon_tags_15 },
    { 0x2e7c2cb9, SUC_lexicon_tags_30 },
    { 0xadfc2cbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1c2cbf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5c2ccb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2edc2ccd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cbc2ccf, SUC_lexicon_tags_11 },
    { 0xab5c2cd0, SUC_lexicon_tags_44 },
    { 0x811c2ccf, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xadfc2cd3, SUC_lexicon_tags_12 },
    { 0x253c2cd4, SUC_lexicon_tags_12 },
    { 0x2abc2cd5, SUC_lexicon_tags_17 },
    { 0xfe1c2cd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7c2cdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfc2ceb, SUC_lexicon_tags_25 },
    { 0x355c2cec, SUC_lexicon_tags_33 },
    { 0x39fc2ced, SUC_lexicon_tags_30 },
    { 0x389c2cee, SUC_lexicon_tags_30 },
    { 0x29dc2cef, SUC_lexicon_tags_31 },
    { 0x143c2cef, SUC_lexicon_tags_33 },
    { 0xc81c2ceb, SUC_lexicon_tags_162 },
    { 0xa99c2cf2, SUC_lexicon_tags_12 },
    { 0x1f7c2cf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9c2cf7, SUC_lexicon_tags_2 },
    { 0xe5dc2cf7, SUC_lexicon_tags_11 },
    { 0xa31c2cf7, SUC_lexicon_tags_17 },
    { 0xe07c2cfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x895c2cfc, SUC_lexicon_tags_2 },
    { 0xc8bc2cfd, SUC_lexicon_tags_25 },
    { 0xeabc2cfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x593c2d01, SUC_lexicon_tags_49 },
    { 0xbbc2d01, SUC_lexicon_tags_47 },
    { 0x88bc2d03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46fc2d05, SUC_lexicon_tags_15 },
    { 0x349c2d06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3c2d09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35c2d0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x36bc2d11, SUC_lexicon_tags_27 },
    { 0x36fc2d12, SUC_lexicon_tags_15 },
    { 0x8c7c2d13, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fc2d1a, SUC_lexicon_tags_27 },
    { 0x969c2d1b, SUC_lexicon_tags_27 },
    { 0x785c2d1c, SUC_lexicon_tags_32 },
    { 0x1d1c2d1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f5c2d1f, SUC_lexicon_tags_37 },
    { 0x293c2d1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bc2d26, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa0dc2d28, SUC_lexicon_tags_87 },
    { 0xd39c2d29, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91c2d2d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1c2d30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9c2d33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd51c2d35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf89c2d37, SUC_lexicon_tags_53 },
    { 0x29bc2d38, SUC_lexicon_tags_17 },
    { 0xaefc2d39, SUC_lexicon_tags_1 },
    { 0x70fc2d3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3c2d3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3c2d46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbc2d4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a7c2d4d, SUC_lexicon_tags_14 },
    { 0x9ddc2d4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c2d54, SUC_lexicon_tags_11 },
    { 0x28fc2d55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35c2d59, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc2d60, SUC_lexicon_tags_1487 },
    { 0xab7c2d61, SUC_lexicon_tags_1 },
    { 0xbf5c2d62, SUC_lexicon_tags_25 },
    { 0xfefc2d63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebc2d68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x477c2d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfc2d6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9fbc2d71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207c2d77, SUC_lexicon_tags_33 },
    { 0xbd3c2d77, SUC_lexicon_tags_37 },
    { 0x4dbc2d79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbc2d7d, SUC_lexicon_tags_31 },
    { 0xfb9c2d7e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf6dc2d80, SUC_lexicon_tags_6 },
    { 0xba7c2d81, SUC_lexicon_tags_14 },
    { 0x83c2d82, SUC_lexicon_tags_6 },
    { 0x71c2d82, SUC_lexicon_tags_15 },
    { 0xd3c2d84, SUC_lexicon_tags_47 },
    { 0x1b1c2d85, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5e3c2d87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f1c2d89, SUC_lexicon_tags_2 },
    { 0x267c2d8a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabc2d91, SUC_lexicon_tags_56 },
    { 0xb0fc2d92, SUC_lexicon_tags_63 },
    { 0x237c2d92, SUC_lexicon_tags_1 },
    { 0xf01c2d94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x503c2d96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9adc2d99, SUC_lexicon_tags_45 },
    { 0xd2fc2d9a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07c2d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27c2da4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65fc2da7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fc2dab, SUC_lexicon_tags_63 },
    { 0x3abc2dac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x205c2dae, SUC_lexicon_tags_62 },
    { 0xcd9c2dae, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x273c2db1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7c2db8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dc2dbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c2dc3, SUC_lexicon_tags_49 },
    { 0xc03c2dc4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc2dca, SUC_lexicon_tags_15 },
    { 0x695c2dca, SUC_lexicon_tags_38 },
    { 0x3cbc2dcc, SUC_lexicon_tags_15 },
    { 0xc5fc2dcd, SUC_lexicon_tags_507 },
    { 0xdddc2dcc, SUC_lexicon_tags_15 },
    { 0x80fc2dcf, SUC_lexicon_tags_1 },
    { 0x6bbc2dcd, SUC_lexicon_tags_17 },
    { 0x357c2dd1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefc2dd4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bc2dd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x521c2dda, SUC_lexicon_tags_43 },
    { 0x3c9c2ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933c2ddf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfc2de3, SUC_lexicon_tags_37 },
    { 0xbb9c2de3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dc2dea, SUC_lexicon_tags_47 },
    { 0xdffc2deb, SUC_lexicon_tags_12 },
    { 0xe47c2dec, SUC_lexicon_tags_3 },
    { 0x8bfc2ded, SUC_lexicon_tags_25 },
    { 0x949c2dec, SUC_lexicon_tags_2 },
    { 0x537c2def, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x26dc2df1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bc2df5, SUC_lexicon_tags_27 },
    { 0x519c2df5, SUC_lexicon_tags_2 },
    { 0x74dc2df5, SUC_lexicon_tags_12 },
    { 0x89dc2df5, SUC_lexicon_tags_11 },
    { 0x8f1c2df9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dc2dff, SUC_lexicon_tags_17 },
    { 0xaf3c2e00, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5c2e03, SUC_lexicon_tags_30 },
    { 0x993c2e04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc19c2e06, SUC_lexicon_tags_17 },
    { 0x837c2e06, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x1ebc2e09, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x843c2e0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfc2e0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bc2e12, SUC_lexicon_tags_14 },
    { 0x21fc2e12, SUC_lexicon_tags_38 },
    { 0x28dc2e12, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79c2e1d, SUC_lexicon_tags_49 },
    { 0x9abc2e1e, SUC_lexicon_tags_30 },
    { 0x771c2e1f, SUC_lexicon_tags_60 },
    { 0x2c9c2e20, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb93c2e22, SUC_lexicon_tags_17 },
    { 0x3c7c2e22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4dc2e25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x843c2e27, SUC_lexicon_tags_30 },
    { 0xee7c2e28, SUC_lexicon_tags_37 },
    { 0xae9c2e28, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dc2e2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45c2e34, SUC_lexicon_tags_8 },
    { 0xf9bc2e35, SUC_lexicon_tags_12 },
    { 0x63bc2e36, SUC_lexicon_tags_16 },
    { 0x961c2e34, SUC_lexicon_tags_25 },
    { 0xf03c2e38, SUC_lexicon_tags_8 },
    { 0xdd1c2e35, SUC_lexicon_tags_17 },
    { 0x2b1c2e3a, SUC_lexicon_tags_8 },
    { 0x937c2e36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x409c2e3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd07c2e40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bc2e46, SUC_lexicon_tags_7 },
    { 0xa5c2e47, SUC_lexicon_tags_68 },
    { 0x45c2e48, SUC_lexicon_tags_60 },
    { 0x9e1c2e47, SUC_lexicon_tags_61 },
    { 0x941c2e4a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673c2e51, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3c2e57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x369c2e59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4fc2e5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3c2e5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddc2e61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ffc2e63, SUC_lexicon_tags_0 },
    { 0xd27c2e63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x997c2e67, SUC_lexicon_tags_60 },
    { 0x471c2e68, SUC_lexicon_tags_7 },
    { 0xd9dc2e69, SUC_lexicon_tags_0 },
    { 0x36fc2e69, SUC_lexicon_tags_1 },
    { 0x63bc2e6b, SUC_lexicon_tags_1 },
    { 0xd17c2e69, SUC_lexicon_tags_25 },
    { 0xbddc2e6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x887c2e70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59c2e72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ebc2e74, SUC_lexicon_tags_3 },
    { 0xa47c2e75, SUC_lexicon_tags_44 },
    { 0x843c2e76, SUC_lexicon_tags_9 },
    { 0xcc3c2e74, SUC_lexicon_tags_25 },
    { 0x9f9c2e76, SUC_lexicon_tags_2 },
    { 0xf6dc2e76, SUC_lexicon_tags_60 },
    { 0x989c2e7a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xde9c2e7c, SUC_lexicon_tags_14 },
    { 0x48dc2e7d, SUC_lexicon_tags_37 },
    { 0x61c2e7c, SUC_lexicon_tags_11 },
    { 0xe3fc2e7d, SUC_lexicon_tags_37 },
    { 0x5c2e80, SUC_lexicon_tags_14 },
    { 0xa7bc2e81, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7c2e86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adc2e8a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2afc2e8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x705c2e8f, SUC_lexicon_tags_4 },
    { 0xee7c2e8f, SUC_lexicon_tags_37 },
    { 0x247c2e91, SUC_lexicon_tags_17 },
    { 0x4ebc2e92, SUC_lexicon_tags_56 },
    { 0x31c2e8f, SUC_lexicon_tags_14 },
    { 0x5d5c2e94, SUC_lexicon_tags_55 },
    { 0xd8bc2e94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65c2e9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dc2ea2, SUC_lexicon_tags_25 },
    { 0x67c2ea3, SUC_lexicon_tags_27 },
    { 0xf01c2ea2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5c2eaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49c2eae, SUC_lexicon_tags_23 },
    { 0x775c2eaf, SUC_lexicon_tags_23 },
    { 0x8fbc2eb0, SUC_lexicon_tags_34 },
    { 0xcb1c2eb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd89c2eb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf1c2eb5, SUC_lexicon_tags_47 },
    { 0x527c2eb5, SUC_lexicon_tags_2 },
    { 0x66bc2eb7, SUC_lexicon_tags_6 },
    { 0xbedc2eb5, SUC_lexicon_tags_12 },
    { 0x13c2eb6, SUC_lexicon_tags_43 },
    { 0xa71c2eba, SUC_lexicon_tags_0 },
    { 0xa3bc2ebb, SUC_lexicon_tags_30 },
    { 0xea3c2ebc, SUC_lexicon_tags_46 },
    { 0x911c2ebd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ebc2ebf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x313c2ec2, SUC_lexicon_tags_19 },
    { 0x94bc2ec3, SUC_lexicon_tags_88 },
    { 0x685c2ec4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499c2ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45c2ecf, SUC_lexicon_tags_8 },
    { 0x9c5c2ed0, SUC_lexicon_tags_4 },
    { 0x933c2ed0, SUC_lexicon_tags_37 },
    { 0x8ffc2ed2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49c2ed5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09c2edc, SUC_lexicon_tags_47 },
    { 0x31dc2edd, SUC_lexicon_tags_12 },
    { 0x4ffc2edc, SUC_lexicon_tags_25 },
    { 0x4a7c2edd, SUC_lexicon_tags_0 },
    { 0xe9dc2ee0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7c2ee2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121c2eec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463c2ef0, SUC_lexicon_tags_11 },
    { 0x979c2ef0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x483c2ef3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1c2ef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1e1c2efa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bc2f01, SUC_lexicon_tags_23 },
    { 0xe7dc2f02, SUC_lexicon_tags_37 },
    { 0xd1c2f03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fc2f08, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc79c2f0a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeefc2f0c, SUC_lexicon_tags_46 },
    { 0xbf5c2f0d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3c2f10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3c2f19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9c2f1e, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x467c2f21, SUC_lexicon_tags_26 },
    { 0xe2bc2f21, SUC_lexicon_tags_25 },
    { 0x321c2f23, SUC_lexicon_tags_2 },
    { 0x3fc2f24, SUC_lexicon_tags_25 },
    { 0xed1c2f24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x431c2f28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb7fc2f2a, SUC_lexicon_tags_2 },
    { 0xd35c2f2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d9c2f2d, SUC_lexicon_tags_0 },
    { 0x97c2f2e, SUC_lexicon_tags_11 },
    { 0x15fc2f2f, SUC_lexicon_tags_27 },
    { 0x8efc2f30, SUC_lexicon_tags_11 },
    { 0x247c2f30, SUC_lexicon_tags_0 },
    { 0xcc1c2f32, SUC_lexicon_tags_342 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fc2f43, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1c2f46, SUC_lexicon_tags_25 },
    { 0x3cfc2f47, SUC_lexicon_tags_6 },
    { 0xb9c2f47, SUC_lexicon_tags_11 },
    { 0xeafc2f49, SUC_lexicon_tags_6 },
    { 0xccfc2f4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60dc2f51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x25fc2f54, SUC_lexicon_tags_5 },
    { 0xd01c2f55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x287c2f57, SUC_lexicon_tags_25 },
    { 0xa85c2f58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5c2f5e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa29c2f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabc2f67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41c2f6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705c2f6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c2f72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235c2f76, SUC_lexicon_tags_43 },
    { 0x699c2f77, SUC_lexicon_tags_68 },
    { 0x511c2f78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x541c2f7b, SUC_lexicon_tags_6 },
    { 0xde9c2f7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c2f82, SUC_lexicon_tags_16 },
    { 0xae9c2f83, SUC_lexicon_tags_8 },
    { 0x1bdc2f83, SUC_lexicon_tags_11 },
    { 0xd03c2f84, SUC_lexicon_tags_8 },
    { 0x615c2f83, SUC_lexicon_tags_165 },
    { 0x88bc2f82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fc2f95, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9c2f9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897c2fb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9c2fb7, SUC_lexicon_tags_15 },
    { 0x77fc2fb8, SUC_lexicon_tags_6 },
    { 0xe6fc2fb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x373c2fbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa61c2fbe, SUC_lexicon_tags_11 },
    { 0xe11c2fbe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dc2fc2, SUC_lexicon_tags_10 },
    { 0xb27c2fc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3c2fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdc2fc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93c2fce, SUC_lexicon_tags_49 },
    { 0x3a5c2fce, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc2fd3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1c2fd7, SUC_lexicon_tags_35 },
    { 0xa53c2fd7, SUC_lexicon_tags_15 },
    { 0xa25c2fd7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbc2fdc, SUC_lexicon_tags_7 },
    { 0x4b3c2fdd, SUC_lexicon_tags_37 },
    { 0x899c2fde, SUC_lexicon_tags_4 },
    { 0x3c5c2fdf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57c2fe2, SUC_lexicon_tags_60 },
    { 0x555c2fe3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7c2fe7, SUC_lexicon_tags_25 },
    { 0x139c2fe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15c2fed, SUC_lexicon_tags_11 },
    { 0x881c2fee, SUC_lexicon_tags_48 },
    { 0x9b3c2fed, SUC_lexicon_tags_27 },
    { 0xae5c2fee, SUC_lexicon_tags_37 },
    { 0x307c2ff0, SUC_lexicon_tags_37 },
    { 0x4bfc2ff1, SUC_lexicon_tags_39 },
    { 0xe2bc2fef, SUC_lexicon_tags_14 },
    { 0xbabc2ff4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709c2ffa, SUC_lexicon_tags_11 },
    { 0x949c2ffa, SUC_lexicon_tags_16 },
    { 0xd47c2ffc, SUC_lexicon_tags_1 },
    { 0x3cdc2ffd, SUC_lexicon_tags_27 },
    { 0x843c2ffc, SUC_lexicon_tags_39 },
    { 0xc45c2ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bfc3001, SUC_lexicon_tags_79 },
    { 0xa69c3002, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c3006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b5c3008, SUC_lexicon_tags_45 },
    { 0x485c3009, SUC_lexicon_tags_25 },
    { 0xfbc3008, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x421c300c, SUC_lexicon_tags_65 },
    { 0xdd7c300c, SUC_lexicon_tags_12 },
    { 0x6b7c300e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x231c3010, SUC_lexicon_tags_39 },
    { 0x3cfc3010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3c3014, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dc3019, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc301c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbfdc301e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133c3030, SUC_lexicon_tags_17 },
    { 0x4abc3030, SUC_lexicon_tags_68 },
    { 0x169c3030, SUC_lexicon_tags_13 },
    { 0x7d9c3032, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7c3037, SUC_lexicon_tags_61 },
    { 0x543c3037, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91c303b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fc303f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37c3042, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63c3045, SUC_lexicon_tags_11 },
    { 0xc47c3046, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdc304c, SUC_lexicon_tags_42 },
    { 0xc67c304d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1c304f, SUC_lexicon_tags_17 },
    { 0x461c3050, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x60bc3052, SUC_lexicon_tags_37 },
    { 0x9c5c3053, SUC_lexicon_tags_33 },
    { 0x63fc3054, SUC_lexicon_tags_21 },
    { 0x8c5c3055, SUC_lexicon_tags_2 },
    { 0xdb9c3056, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa79c3058, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bc305d, SUC_lexicon_tags_31 },
    { 0x96fc305e, SUC_lexicon_tags_56 },
    { 0x12fc305e, SUC_lexicon_tags_37 },
    { 0x79bc305f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797c306e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dc3072, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bc307b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeefc307d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebc3080, SUC_lexicon_tags_43 },
    { 0x81dc3081, SUC_lexicon_tags_47 },
    { 0x579c3082, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61c3087, SUC_lexicon_tags_21 },
    { 0xe4fc3088, SUC_lexicon_tags_113 },
    { 0x97dc3088, SUC_lexicon_tags_45 },
    { 0x45dc308a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdbc308c, SUC_lexicon_tags_25 },
    { 0xc17c308c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dc3096, SUC_lexicon_tags_12 },
    { 0x5bdc3097, SUC_lexicon_tags_21 },
    { 0x33dc3097, SUC_lexicon_tags_37 },
    { 0xae1c3099, SUC_lexicon_tags_37 },
    { 0x2e7c3098, SUC_lexicon_tags_7 },
    { 0xef3c3096, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523c30a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x463c30a7, SUC_lexicon_tags_1 },
    { 0x61dc30a8, SUC_lexicon_tags_39 },
    { 0x4edc30a9, SUC_lexicon_tags_47 },
    { 0x6edc30a8, SUC_lexicon_tags_11 },
    { 0x659c30aa, SUC_lexicon_tags_37 },
    { 0xfbfc30ac, SUC_lexicon_tags_25 },
    { 0x68dc30a9, SUC_lexicon_tags_32 },
    { 0xdffc30ae, SUC_lexicon_tags_65 },
    { 0xcffc30af, SUC_lexicon_tags_37 },
    { 0xdfbc30af, SUC_lexicon_tags_15 },
    { 0xfa9c30b1, SUC_lexicon_tags_2 },
    { 0x333c30ae, SUC_lexicon_tags_33 },
    { 0xb9dc30ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1c30b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x855c30b8, SUC_lexicon_tags_17 },
    { 0x669c30b9, SUC_lexicon_tags_47 },
    { 0xa43c30b8, SUC_lexicon_tags_15 },
    { 0x19bc30bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5c30bf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xce3c30c1, SUC_lexicon_tags_8 },
    { 0xea7c30c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11c30c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa1c30c7, SUC_lexicon_tags_15 },
    { 0x301c30c8, SUC_lexicon_tags_37 },
    { 0xb69c30c8, SUC_lexicon_tags_63 },
    { 0xb89c30ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1c30cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x93dc30cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f1c30d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561c30d7, SUC_lexicon_tags_15 },
    { 0x91c30d7, SUC_lexicon_tags_26 },
    { 0x877c30d9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x453c30dc, SUC_lexicon_tags_17 },
    { 0x5b9c30dd, SUC_lexicon_tags_15 },
    { 0xe47c30de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfc30e4, SUC_lexicon_tags_35 },
    { 0x553c30e5, SUC_lexicon_tags_4 },
    { 0x9d1c30e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfa3c30e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5c30ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x539c30f2, SUC_lexicon_tags_33 },
    { 0xb41c30f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7c30f5, SUC_lexicon_tags_1 },
    { 0xf93c30f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21c30f9, SUC_lexicon_tags_38 },
    { 0xe65c30fa, SUC_lexicon_tags_45 },
    { 0x2bdc30fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dc30ff, SUC_lexicon_tags_63 },
    { 0x5b3c3100, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebc3103, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dc3109, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87c310c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xca9c310e, SUC_lexicon_tags_64 },
    { 0x73fc310e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1c3112, SUC_lexicon_tags_2 },
    { 0xcd1c3113, SUC_lexicon_tags_37 },
    { 0x52bc3113, SUC_lexicon_tags_17 },
    { 0xcbbc3115, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49c3118, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5c311f, SUC_lexicon_tags_43 },
    { 0x1f3c311f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bc3123, SUC_lexicon_tags_87 },
    { 0x705c3124, SUC_lexicon_tags_31 },
    { 0xabfc3125, SUC_lexicon_tags_3 },
    { 0x349c3126, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x445c3128, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafc312b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x601c312e, SUC_lexicon_tags_17 },
    { 0xb6bc312e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27dc3133, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x73dc3136, SUC_lexicon_tags_1 },
    { 0x63dc3137, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c7c3139, SUC_lexicon_tags_25 },
    { 0xeddc313a, SUC_lexicon_tags_1 },
    { 0x13c313b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcf3c313d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3c3141, SUC_lexicon_tags_25 },
    { 0x397c3142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3c3146, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x813c3148, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x431c314b, SUC_lexicon_tags_11 },
    { 0x82fc314c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x121c314e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x669c3151, SUC_lexicon_tags_63 },
    { 0x2fbc3152, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x331c3155, SUC_lexicon_tags_2 },
    { 0xc33c3156, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x627c3158, SUC_lexicon_tags_12 },
    { 0x5d9c3159, SUC_lexicon_tags_37 },
    { 0x85bc3159, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfc315d, SUC_lexicon_tags_19 },
    { 0x6a7c315e, SUC_lexicon_tags_25 },
    { 0x665c315f, SUC_lexicon_tags_8 },
    { 0x277c3160, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe05c3162, SUC_lexicon_tags_80 },
    { 0xcdfc3162, SUC_lexicon_tags_14 },
    { 0x5e1c3162, SUC_lexicon_tags_17 },
    { 0x54dc3163, SUC_lexicon_tags_45 },
    { 0x491c3166, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x361c3168, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xb9dc316a, SUC_lexicon_tags_19 },
    { 0x26bc316b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfc316e, SUC_lexicon_tags_14 },
    { 0xdefc316f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef1c3171, SUC_lexicon_tags_17 },
    { 0x60fc3172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20dc3174, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xef3c3176, SUC_lexicon_tags_87 },
    { 0xb9fc3177, SUC_lexicon_tags_15 },
    { 0x8a7c3176, SUC_lexicon_tags_12 },
    { 0xe63c3179, SUC_lexicon_tags_48 },
    { 0x769c3176, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3c317d, SUC_lexicon_tags_2 },
    { 0x6c1c317e, SUC_lexicon_tags_2 },
    { 0xe05c317f, SUC_lexicon_tags_2 },
    { 0x4ebc3180, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc1c3182, SUC_lexicon_tags_60 },
    { 0x245c3183, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa7c3185, SUC_lexicon_tags_537 },
    { 0x71bc3186, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257c318d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x683c318f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fc3192, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefc3195, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x607c3198, SUC_lexicon_tags_1 },
    { 0xedfc3199, SUC_lexicon_tags_15 },
    { 0x83fc319a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x163c319c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8cdc319e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xb0dc31a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45c31a2, SUC_lexicon_tags_7 },
    { 0x277c31a3, SUC_lexicon_tags_11 },
    { 0xe83c31a4, SUC_lexicon_tags_80 },
    { 0xc2dc31a3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1c31a7, SUC_lexicon_tags_2 },
    { 0x51bc31a8, SUC_lexicon_tags_82 },
    { 0xaf3c31a9, SUC_lexicon_tags_29 },
    { 0xf5fc31a9, SUC_lexicon_tags_7 },
    { 0x7fdc31ab, SUC_lexicon_tags_104 },
    { 0x991c31ac, SUC_lexicon_tags_25 },
    { 0xea9c31ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c31b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7c31b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7c31bd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9c31c0, SUC_lexicon_tags_15 },
    { 0xa95c31c0, SUC_lexicon_tags_22 },
    { 0x30fc31c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12bc31c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x661c31c6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x203c31c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc27c31cb, SUC_lexicon_tags_25 },
    { 0x763c31cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fc31d3, SUC_lexicon_tags_4 },
    { 0x44dc31d4, SUC_lexicon_tags_17 },
    { 0xb3fc31d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc31d8, SUC_lexicon_tags_3 },
    { 0xe59c31d9, SUC_lexicon_tags_63 },
    { 0x363c31da, SUC_lexicon_tags_17 },
    { 0x8bc31d8, SUC_lexicon_tags_37 },
    { 0xf15c31d8, SUC_lexicon_tags_12 },
    { 0xe53c31dd, SUC_lexicon_tags_66 },
    { 0x301c31de, SUC_lexicon_tags_38 },
    { 0xdf1c31df, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3c31e2, SUC_lexicon_tags_33 },
    { 0x4c9c31e2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3c31e7, SUC_lexicon_tags_64 },
    { 0xb03c31e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b5c31ea, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xfb5c31ec, SUC_lexicon_tags_11 },
    { 0x57fc31ed, SUC_lexicon_tags_82 },
    { 0xfedc31ed, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa35c31f0, SUC_lexicon_tags_15 },
    { 0xccdc31f0, SUC_lexicon_tags_9 },
    { 0x4d5c31f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379c31f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89c31fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab1c31fc, SUC_lexicon_tags_69 },
    { 0x10fc31fd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfc3200, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x147c3202, SUC_lexicon_tags_59 },
    { 0x8e9c3202, SUC_lexicon_tags_2 },
    { 0xdcdc3204, SUC_lexicon_tags_15 },
    { 0x9cdc3205, SUC_lexicon_tags_2 },
    { 0x52bc3204, SUC_lexicon_tags_25 },
    { 0x4c3c3204, SUC_lexicon_tags_60 },
    { 0xe89c3206, SUC_lexicon_tags_5 },
    { 0x27dc3209, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbc320d, SUC_lexicon_tags_60 },
    { 0x6a9c320e, SUC_lexicon_tags_7 },
    { 0x86fc320f, SUC_lexicon_tags_15 },
    { 0x49fc3210, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1c3213, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bc3216, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9c3219, SUC_lexicon_tags_37 },
    { 0x357c321a, SUC_lexicon_tags_23 },
    { 0x6dbc321a, SUC_lexicon_tags_37 },
    { 0xa89c321a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4efc321e, SUC_lexicon_tags_11 },
    { 0xcd3c321f, SUC_lexicon_tags_55 },
    { 0x431c3220, SUC_lexicon_tags_27 },
    { 0x2b3c3221, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd93c3223, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbc3227, SUC_lexicon_tags_2 },
    { 0xadc3228, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625c3235, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fc323a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3efc323c, SUC_lexicon_tags_37 },
    { 0x11dc323d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb51c323f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdc3242, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x287c3244, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483c3249, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323c324f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x275c3251, SUC_lexicon_tags_27 },
    { 0x9ebc3252, SUC_lexicon_tags_1 },
    { 0xc7dc3252, SUC_lexicon_tags_7 },
    { 0x1b3c3251, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb57c3256, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07c3259, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3c325d, SUC_lexicon_tags_1 },
    { 0xa4fc325d, SUC_lexicon_tags_15 },
    { 0xbdbc325d, SUC_lexicon_tags_48 },
    { 0x7f5c325d, SUC_lexicon_tags_65 },
    { 0xffc325d, SUC_lexicon_tags_38 },
    { 0xae7c325f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef7c3264, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbc3267, SUC_lexicon_tags_11 },
    { 0xcfdc3267, SUC_lexicon_tags_25 },
    { 0xebfc3269, SUC_lexicon_tags_39 },
    { 0x509c326a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdc326d, SUC_lexicon_tags_45 },
    { 0x1b3c326d, SUC_lexicon_tags_46 },
    { 0x881c326d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19fc3271, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5c327a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefc3284, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883c3289, SUC_lexicon_tags_17 },
    { 0x9d5c3289, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bc328d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3c3292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79c3296, SUC_lexicon_tags_2 },
    { 0x5edc3296, SUC_lexicon_tags_25 },
    { 0x363c3296, SUC_lexicon_tags_38 },
    { 0x391c3296, SUC_lexicon_tags_15 },
    { 0xccdc3297, SUC_lexicon_tags_1 },
    { 0x487c329b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4edc329d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bc32a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57c32a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fc32aa, SUC_lexicon_tags_11 },
    { 0xe01c32ab, SUC_lexicon_tags_23 },
    { 0xc21c32ac, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2ffc32ae, SUC_lexicon_tags_1 },
    { 0xcb7c32af, SUC_lexicon_tags_0 },
    { 0x93bc32af, SUC_lexicon_tags_40 },
    { 0xa7fc32b1, SUC_lexicon_tags_63 },
    { 0x84dc32b2, SUC_lexicon_tags_0 },
    { 0xba5c32b3, SUC_lexicon_tags_817 },
    { 0xa29c32b4, SUC_lexicon_tags_0 },
    { 0x645c32b1, SUC_lexicon_tags_11 },
    { 0x7b1c32b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e1c32b8, SUC_lexicon_tags_15 },
    { 0x6c7c32b8, SUC_lexicon_tags_17 },
    { 0x3e1c32ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa3c32bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325c32c0, SUC_lexicon_tags_14 },
    { 0xc6fc32c1, SUC_lexicon_tags_17 },
    { 0xbf9c32c2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x215c32c4, SUC_lexicon_tags_65 },
    { 0x41dc32c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bc32c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa03c32cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7c32ce, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933c32d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85c32d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55c32dc, SUC_lexicon_tags_11 },
    { 0xab3c32dc, SUC_lexicon_tags_40 },
    { 0x22fc32de, SUC_lexicon_tags_0 },
    { 0xb19c32de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc89c32e2, SUC_lexicon_tags_10 },
    { 0xccdc32e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03c32e6, SUC_lexicon_tags_63 },
    { 0x6e7c32e6, SUC_lexicon_tags_12 },
    { 0x637c32e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bc32f2, SUC_lexicon_tags_9 },
    { 0x78dc32f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523c32f9, SUC_lexicon_tags_11 },
    { 0x373c32f9, SUC_lexicon_tags_31 },
    { 0x7efc32f9, SUC_lexicon_tags_11 },
    { 0x1ebc32fa, SUC_lexicon_tags_17 },
    { 0x755c32fd, SUC_lexicon_tags_0 },
    { 0xf91c32fb, SUC_lexicon_tags_2 },
    { 0x21fc32ff, SUC_lexicon_tags_16 },
    { 0xacfc32fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167c3304, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbc5c3306, SUC_lexicon_tags_87 },
    { 0xccdc3307, SUC_lexicon_tags_40 },
    { 0xfa9c3306, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x781c330a, SUC_lexicon_tags_14 },
    { 0xa8dc330a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11c3311, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77bc3313, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29c3318, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5afc331a, SUC_lexicon_tags_0 },
    { 0xc8fc331b, SUC_lexicon_tags_17 },
    { 0x2f7c331c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbadc331e, SUC_lexicon_tags_32 },
    { 0x8f9c331e, SUC_lexicon_tags_26 },
    { 0x331c3320, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1c3324, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681c3328, SUC_lexicon_tags_11 },
    { 0x307c3329, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343c3331, SUC_lexicon_tags_66 },
    { 0x74bc3331, SUC_lexicon_tags_2 },
    { 0xdfdc3333, SUC_lexicon_tags_7 },
    { 0x141c3332, SUC_lexicon_tags_12 },
    { 0x537c3335, SUC_lexicon_tags_15 },
    { 0x239c3336, SUC_lexicon_tags_17 },
    { 0x149c3334, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefc333f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26dc3344, SUC_lexicon_tags_28 },
    { 0xd0dc3345, SUC_lexicon_tags_79 },
    { 0x45fc3346, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70bc3348, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dc3353, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa85c3355, SUC_lexicon_tags_1 },
    { 0x65bc3356, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x499c3358, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x415c335b, SUC_lexicon_tags_43 },
    { 0x5e3c335c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x34fc335e, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x7b5c3360, SUC_lexicon_tags_7 },
    { 0x169c3360, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dc3365, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9c3368, SUC_lexicon_tags_9 },
    { 0x507c3368, SUC_lexicon_tags_15 },
    { 0xe5fc3368, SUC_lexicon_tags_23 },
    { 0x8cdc3368, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x153c336e, SUC_lexicon_tags_2 },
    { 0xe45c336e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc3375, SUC_lexicon_tags_55 },
    { 0x1bdc3376, SUC_lexicon_tags_84 },
    { 0xb0dc3376, SUC_lexicon_tags_27 },
    { 0xbbbc3376, SUC_lexicon_tags_30 },
    { 0x2b5c3379, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x457c337b, SUC_lexicon_tags_25 },
    { 0x60dc337c, SUC_lexicon_tags_37 },
    { 0x22fc337c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc3383, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x223c3385, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29c338b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c1c338d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bc3393, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9dc3395, SUC_lexicon_tags_7 },
    { 0xdcdc3395, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05c3399, SUC_lexicon_tags_46 },
    { 0x8c9c339a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9dc339c, SUC_lexicon_tags_0 },
    { 0x5adc339d, SUC_lexicon_tags_23 },
    { 0x73bc339e, SUC_lexicon_tags_11 },
    { 0x3bc339d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5c33a3, SUC_lexicon_tags_7 },
    { 0x9f3c33a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61c33ab, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdc33b2, SUC_lexicon_tags_60 },
    { 0x269c33b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5dc33b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc33b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fc33bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3c33be, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5c33c9, SUC_lexicon_tags_11 },
    { 0x38dc33ca, SUC_lexicon_tags_6 },
    { 0x147c33c9, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c33d5, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5c33d8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x77bc33da, SUC_lexicon_tags_32 },
    { 0xbe7c33da, SUC_lexicon_tags_53 },
    { 0x1c3c33dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3e7c33de, SUC_lexicon_tags_1 },
    { 0xb0dc33de, SUC_lexicon_tags_25 },
    { 0x433c33e0, SUC_lexicon_tags_2 },
    { 0x437c33e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5edc33e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7d5c33e5, SUC_lexicon_tags_15 },
    { 0x78bc33e6, SUC_lexicon_tags_13 },
    { 0xe4dc33e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdc33ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61c33f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09c33f6, SUC_lexicon_tags_37 },
    { 0x703c33f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x215c33fa, SUC_lexicon_tags_43 },
    { 0x5e9c33fb, SUC_lexicon_tags_45 },
    { 0x2edc33fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67c33ff, SUC_lexicon_tags_2 },
    { 0xca5c33ff, SUC_lexicon_tags_12 },
    { 0x689c3401, SUC_lexicon_tags_65 },
    { 0x8a5c3402, SUC_lexicon_tags_198 },
    { 0x28bc3403, SUC_lexicon_tags_12 },
    { 0x7d7c3404, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7c3407, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17c340c, SUC_lexicon_tags_15 },
    { 0xf43c340d, SUC_lexicon_tags_25 },
    { 0x4efc340e, SUC_lexicon_tags_39 },
    { 0x5dfc340f, SUC_lexicon_tags_4 },
    { 0x9b5c340f, SUC_lexicon_tags_23 },
    { 0x1f5c340d, SUC_lexicon_tags_15 },
    { 0xefbc3410, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x171c3415, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435c3419, SUC_lexicon_tags_12 },
    { 0x145c341a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfedc341c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7c341f, SUC_lexicon_tags_11 },
    { 0xd63c3420, SUC_lexicon_tags_30 },
    { 0xeb7c341f, SUC_lexicon_tags_7 },
    { 0xaabc341f, SUC_lexicon_tags_23 },
    { 0x91bc3422, SUC_lexicon_tags_25 },
    { 0x44bc3423, SUC_lexicon_tags_32 },
    { 0x319c341f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3c3428, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x865c342b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x553c342d, SUC_lexicon_tags_15 },
    { 0x7cdc342e, SUC_lexicon_tags_26 },
    { 0x2ffc342f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c3433, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9c3438, SUC_lexicon_tags_8 },
    { 0x335c3439, SUC_lexicon_tags_37 },
    { 0x713c3439, SUC_lexicon_tags_20 },
    { 0x519c3439, SUC_lexicon_tags_1 },
    { 0x919c343c, SUC_lexicon_tags_7 },
    { 0x9c5c343d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45fc343f, SUC_lexicon_tags_8 },
    { 0x51c3440, SUC_lexicon_tags_2 },
    { 0xab1c3441, SUC_lexicon_tags_37 },
    { 0xa8dc3441, SUC_lexicon_tags_27 },
    { 0xd45c3442, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bc3447, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bc344b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bc344f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x669c3451, SUC_lexicon_tags_14 },
    { 0x3e5c3452, SUC_lexicon_tags_37 },
    { 0x6bfc3453, SUC_lexicon_tags_0 },
    { 0x4dfc3453, SUC_lexicon_tags_25 },
    { 0xa13c3455, SUC_lexicon_tags_29 },
    { 0x663c3456, SUC_lexicon_tags_63 },
    { 0xd81c3455, SUC_lexicon_tags_14 },
    { 0xbffc3457, SUC_lexicon_tags_7 },
    { 0x203c3453, SUC_lexicon_tags_37 },
    { 0x7e1c345a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x117c345c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5c345f, SUC_lexicon_tags_12 },
    { 0x62fc3460, SUC_lexicon_tags_24 },
    { 0x281c345f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1c3465, SUC_lexicon_tags_19 },
    { 0xf6dc3465, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x441c3468, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc346d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7c3475, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723c347a, SUC_lexicon_tags_11 },
    { 0x94fc347a, SUC_lexicon_tags_38 },
    { 0x135c347c, SUC_lexicon_tags_49 },
    { 0xfb5c347b, SUC_lexicon_tags_23 },
    { 0xc85c347e, SUC_lexicon_tags_23 },
    { 0xfcbc347f, SUC_lexicon_tags_25 },
    { 0x687c347b, SUC_lexicon_tags_7 },
    { 0x389c3481, SUC_lexicon_tags_32 },
    { 0x77c347c, SUC_lexicon_tags_27 },
    { 0xf0dc3483, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x345c3486, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c7c3488, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadc348c, SUC_lexicon_tags_0 },
    { 0xe01c348c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdc3490, SUC_lexicon_tags_11 },
    { 0x365c3491, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431c3496, SUC_lexicon_tags_136 },
    { 0xb95c3497, SUC_lexicon_tags_14 },
    { 0xc85c3497, SUC_lexicon_tags_6 },
    { 0x749c3499, SUC_lexicon_tags_45 },
    { 0x50dc3499, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7c349d, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0x959c34a0, SUC_lexicon_tags_3 },
    { 0xfbdc34a0, SUC_lexicon_tags_33 },
    { 0x357c34a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11c34a5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x803c34a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3c34ab, SUC_lexicon_tags_29 },
    { 0x9dc34ac, SUC_lexicon_tags_37 },
    { 0x78dc34ad, SUC_lexicon_tags_1 },
    { 0xe27c34ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9c34b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85bc34b4, SUC_lexicon_tags_28 },
    { 0x8adc34b4, SUC_lexicon_tags_56 },
    { 0xbc7c34b6, SUC_lexicon_tags_11 },
    { 0x889c34b6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3c34be, SUC_lexicon_tags_8 },
    { 0xebdc34bf, SUC_lexicon_tags_23 },
    { 0x285c34bf, SUC_lexicon_tags_7 },
    { 0x4d5c34be, SUC_lexicon_tags_63 },
    { 0x2c1c34c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dc34c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11c34d4, SUC_lexicon_tags_15 },
    { 0x8d9c34d5, SUC_lexicon_tags_4 },
    { 0x24dc34d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379c34da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369c34de, SUC_lexicon_tags_0 },
    { 0xa3c34df, SUC_lexicon_tags_25 },
    { 0x39dc34df, SUC_lexicon_tags_37 },
    { 0x663c34e0, SUC_lexicon_tags_65 },
    { 0x259c34df, SUC_lexicon_tags_12 },
    { 0x227c34de, SUC_lexicon_tags_37 },
    { 0xabfc34e2, SUC_lexicon_tags_16 },
    { 0x82bc34e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351c34ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b9c34f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45c34f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dc34fc, SUC_lexicon_tags_160 },
    { 0x7cbc34fd, SUC_lexicon_tags_15 },
    { 0xc09c34fe, SUC_lexicon_tags_148 },
    { 0x507c34ff, SUC_lexicon_tags_11 },
    { 0xb73c34fe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x95c3502, SUC_lexicon_tags_26 },
    { 0x1c3503, SUC_lexicon_tags_28 },
    { 0xa49c3504, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff9c3506, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fc350d, SUC_lexicon_tags_657 },
    { 0xa3fc350e, SUC_lexicon_tags_11 },
    { 0xddbc350f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7c3512, SUC_lexicon_tags_148 },
    { 0x67bc3512, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499c3517, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36fc3519, SUC_lexicon_tags_37 },
    { 0x88fc351a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1c3529, SUC_lexicon_tags_45 },
    { 0x98fc352a, SUC_lexicon_tags_2 },
    { 0x7d9c3529, SUC_lexicon_tags_7 },
    { 0x3bc352c, SUC_lexicon_tags_60 },
    { 0x41bc352d, SUC_lexicon_tags_7 },
    { 0x1e9c352d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bc3531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dc3538, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdc353c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dc3544, SUC_lexicon_tags_17 },
    { 0xf45c3545, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21fc3547, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1c354f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfc3555, SUC_lexicon_tags_37 },
    { 0x13bc3556, SUC_lexicon_tags_7 },
    { 0x35fc3557, SUC_lexicon_tags_661 },
    { 0, NULL },
    { 0x9a7c3559, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x539c355c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x715c355f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x801c3561, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43c3567, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfc356d, SUC_lexicon_tags_14 },
    { 0x373c356e, SUC_lexicon_tags_17 },
    { 0xc41c356d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929c3576, SUC_lexicon_tags_33 },
    { 0xb25c3577, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9c357a, SUC_lexicon_tags_15 },
    { 0x669c357b, SUC_lexicon_tags_10 },
    { 0x3afc357c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe91c357e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127c3588, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x451c358b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfe9c358d, SUC_lexicon_tags_11 },
    { 0x217c358e, SUC_lexicon_tags_37 },
    { 0x69dc358e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657c3593, SUC_lexicon_tags_15 },
    { 0xe07c3593, SUC_lexicon_tags_11 },
    { 0x7cfc3594, SUC_lexicon_tags_47 },
    { 0x67fc3596, SUC_lexicon_tags_25 },
    { 0x783c3593, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65bc3599, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bc359c, SUC_lexicon_tags_2 },
    { 0x481c359c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afc35a4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25c35a7, SUC_lexicon_tags_6 },
    { 0xd4dc35a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bc35af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x569c35b1, SUC_lexicon_tags_15 },
    { 0x7f1c35b2, SUC_lexicon_tags_1 },
    { 0xa3fc35b2, SUC_lexicon_tags_26 },
    { 0x1bfc35b4, SUC_lexicon_tags_2 },
    { 0xb57c35b4, SUC_lexicon_tags_84 },
    { 0xa0dc35b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc35b9, SUC_lexicon_tags_62 },
    { 0xb1c35ba, SUC_lexicon_tags_30 },
    { 0x6efc35b9, SUC_lexicon_tags_43 },
    { 0x67bc35bc, SUC_lexicon_tags_56 },
    { 0xf79c35bd, SUC_lexicon_tags_2 },
    { 0x143c35bd, SUC_lexicon_tags_49 },
    { 0xfc1c35bf, SUC_lexicon_tags_11 },
    { 0xbb3c35bf, SUC_lexicon_tags_25 },
    { 0x22bc35bf, SUC_lexicon_tags_33 },
    { 0x371c35c2, SUC_lexicon_tags_11 },
    { 0xba7c35c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf51c35c5, SUC_lexicon_tags_12 },
    { 0xe99c35c5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dc35d2, SUC_lexicon_tags_60 },
    { 0x101c35d2, SUC_lexicon_tags_15 },
    { 0xe1dc35d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x457c35d6, SUC_lexicon_tags_2 },
    { 0xcc7c35d7, SUC_lexicon_tags_26 },
    { 0xf7c35d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fc35df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdc35e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4ebc35e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x797c35e6, SUC_lexicon_tags_37 },
    { 0xd8dc35e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa9c35e9, SUC_lexicon_tags_8 },
    { 0xcb3c35ea, SUC_lexicon_tags_11 },
    { 0x2e1c35eb, SUC_lexicon_tags_12 },
    { 0xdd3c35e9, SUC_lexicon_tags_0 },
    { 0x847c35ed, SUC_lexicon_tags_37 },
    { 0x7c35ee, SUC_lexicon_tags_25 },
    { 0x8f9c35ef, SUC_lexicon_tags_107 },
    { 0x6cfc35f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x987c35f3, SUC_lexicon_tags_1342 },
    { 0xda9c35f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dc35fa, SUC_lexicon_tags_26 },
    { 0x9ffc35fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c35fe, SUC_lexicon_tags_1 },
    { 0x71c35ff, SUC_lexicon_tags_38 },
    { 0x69dc35ff, SUC_lexicon_tags_26 },
    { 0x875c35fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x889c3603, SUC_lexicon_tags_17 },
    { 0xe53c3604, SUC_lexicon_tags_19 },
    { 0x813c3605, SUC_lexicon_tags_37 },
    { 0xb2fc3606, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2adc360b, SUC_lexicon_tags_25 },
    { 0x47fc360c, SUC_lexicon_tags_66 },
    { 0x897c360c, SUC_lexicon_tags_25 },
    { 0x579c360e, SUC_lexicon_tags_7 },
    { 0xb55c360e, SUC_lexicon_tags_15 },
    { 0xe3c360f, SUC_lexicon_tags_11 },
    { 0x379c3611, SUC_lexicon_tags_37 },
    { 0x303c360f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67c3617, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fc361a, SUC_lexicon_tags_1 },
    { 0xe0fc361a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49c361f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc27c3621, SUC_lexicon_tags_31 },
    { 0x2a3c3622, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x847c3625, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fc362a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adc362e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dc3633, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7c3637, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865c3645, SUC_lexicon_tags_37 },
    { 0x9f1c3646, SUC_lexicon_tags_0 },
    { 0x8c9c3647, SUC_lexicon_tags_28 },
    { 0x335c3648, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1dc364a, SUC_lexicon_tags_8 },
    { 0x25fc364b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x143c364d, SUC_lexicon_tags_25 },
    { 0x89fc364e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b3c3650, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc3654, SUC_lexicon_tags_32 },
    { 0x1e9c3655, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed7c3657, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dc365c, SUC_lexicon_tags_17 },
    { 0x7abc365d, SUC_lexicon_tags_29 },
    { 0xce3c365c, SUC_lexicon_tags_3 },
    { 0x267c365f, SUC_lexicon_tags_29 },
    { 0x3bbc3660, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53c3667, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1c366f, SUC_lexicon_tags_14 },
    { 0x4bdc3670, SUC_lexicon_tags_17 },
    { 0x4fdc3671, SUC_lexicon_tags_43 },
    { 0x6f1c3672, SUC_lexicon_tags_11 },
    { 0xac9c3672, SUC_lexicon_tags_15 },
    { 0x7bbc3670, SUC_lexicon_tags_51 },
    { 0x14dc3670, SUC_lexicon_tags_37 },
    { 0x4cbc366f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9c3679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3c367c, SUC_lexicon_tags_15 },
    { 0xb5c367d, SUC_lexicon_tags_20 },
    { 0x15dc367e, SUC_lexicon_tags_45 },
    { 0xcb5c367d, SUC_lexicon_tags_14 },
    { 0xd13c3680, SUC_lexicon_tags_12 },
    { 0x4a9c3681, SUC_lexicon_tags_49 },
    { 0x175c3681, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa05c3684, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dc368f, SUC_lexicon_tags_64 },
    { 0x6f3c368f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463c3698, SUC_lexicon_tags_25 },
    { 0x765c3699, SUC_lexicon_tags_37 },
    { 0x93c369a, SUC_lexicon_tags_37 },
    { 0x33bc369a, SUC_lexicon_tags_17 },
    { 0xd83c369a, SUC_lexicon_tags_10 },
    { 0x465c369d, SUC_lexicon_tags_37 },
    { 0xddfc3699, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3c36a1, SUC_lexicon_tags_45 },
    { 0x4b7c36a1, SUC_lexicon_tags_4 },
    { 0xedbc36a3, SUC_lexicon_tags_9 },
    { 0x6f5c36a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3c36a7, SUC_lexicon_tags_15 },
    { 0x1f7c36a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3c36ab, SUC_lexicon_tags_37 },
    { 0x443c36ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fc36af, SUC_lexicon_tags_45 },
    { 0xa2dc36b0, SUC_lexicon_tags_7 },
    { 0xe69c36b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbc36b8, SUC_lexicon_tags_481 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbc36bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579c36c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x855c36c3, SUC_lexicon_tags_12 },
    { 0xde5c36c4, SUC_lexicon_tags_11 },
    { 0x757c36c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x31c36c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c36ca, SUC_lexicon_tags_65 },
    { 0x27dc36cb, SUC_lexicon_tags_7 },
    { 0xa4fc36cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfe7c36ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1c36d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfc36d4, SUC_lexicon_tags_12 },
    { 0x711c36d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689c36d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45c36dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x379c36de, SUC_lexicon_tags_0 },
    { 0x65fc36df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb7c36e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fc36e4, SUC_lexicon_tags_25 },
    { 0xf2dc36e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c36ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7c36ef, SUC_lexicon_tags_29 },
    { 0xccfc36ef, SUC_lexicon_tags_37 },
    { 0x79bc36f1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539c36f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81c3704, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5c3708, SUC_lexicon_tags_38 },
    { 0x301c3708, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05c370d, SUC_lexicon_tags_45 },
    { 0x597c370e, SUC_lexicon_tags_27 },
    { 0x491c370f, SUC_lexicon_tags_29 },
    { 0xbb3c3710, SUC_lexicon_tags_2 },
    { 0xcc7c3711, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x433c3714, SUC_lexicon_tags_95 },
    { 0x5fbc3715, SUC_lexicon_tags_31 },
    { 0x5f3c3714, SUC_lexicon_tags_49 },
    { 0x423c3715, SUC_lexicon_tags_10 },
    { 0x31dc3716, SUC_lexicon_tags_26 },
    { 0xfd5c3719, SUC_lexicon_tags_6 },
    { 0xdabc3717, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb43c371c, SUC_lexicon_tags_66 },
    { 0x56fc371c, SUC_lexicon_tags_15 },
    { 0x157c371c, SUC_lexicon_tags_37 },
    { 0x5f5c371f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xdefc3721, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bc3727, SUC_lexicon_tags_17 },
    { 0xbcfc3727, SUC_lexicon_tags_25 },
    { 0xd8fc3729, SUC_lexicon_tags_12 },
    { 0x50dc3727, SUC_lexicon_tags_23 },
    { 0xaffc372b, SUC_lexicon_tags_11 },
    { 0xb87c372c, SUC_lexicon_tags_55 },
    { 0x29c372b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2f7c372f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931c3733, SUC_lexicon_tags_17 },
    { 0xa7fc3734, SUC_lexicon_tags_12 },
    { 0x821c3735, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x969c3737, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffc373b, SUC_lexicon_tags_12 },
    { 0x51c373c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xad3c373e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6c9c3740, SUC_lexicon_tags_7 },
    { 0x3a7c3741, SUC_lexicon_tags_4 },
    { 0xadc3742, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731c374b, SUC_lexicon_tags_7 },
    { 0x42dc374c, SUC_lexicon_tags_11 },
    { 0x9adc374d, SUC_lexicon_tags_14 },
    { 0xd8fc374d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcafc3750, SUC_lexicon_tags_14 },
    { 0x75c3750, SUC_lexicon_tags_31 },
    { 0x27bc3752, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28dc3754, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681c3759, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dc375e, SUC_lexicon_tags_65 },
    { 0xd67c375f, SUC_lexicon_tags_47 },
    { 0xa49c375e, SUC_lexicon_tags_30 },
    { 0x91dc375e, SUC_lexicon_tags_34 },
    { 0x167c3762, SUC_lexicon_tags_26 },
    { 0x51c3762, SUC_lexicon_tags_12 },
    { 0x279c3764, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c3c3766, SUC_lexicon_tags_26 },
    { 0x745c3766, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3c376b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441c3778, SUC_lexicon_tags_65 },
    { 0xb5c3779, SUC_lexicon_tags_49 },
    { 0x981c377a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6dc3780, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x85dc3783, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xeedc3785, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5c378a, SUC_lexicon_tags_33 },
    { 0x1ebc378b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaabc378d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x495c3790, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x141c3793, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73c3798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fc379e, SUC_lexicon_tags_7 },
    { 0xe19c379e, SUC_lexicon_tags_25 },
    { 0xe63c37a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697c37a4, SUC_lexicon_tags_14 },
    { 0xfcfc37a4, SUC_lexicon_tags_55 },
    { 0x5d5c37a4, SUC_lexicon_tags_47 },
    { 0x8cdc37a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bc37aa, SUC_lexicon_tags_19 },
    { 0xbe3c37ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fc37ae, SUC_lexicon_tags_68 },
    { 0x8c1c37af, SUC_lexicon_tags_23 },
    { 0x6a5c37b0, SUC_lexicon_tags_23 },
    { 0xba9c37b1, SUC_lexicon_tags_60 },
    { 0xef3c37b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291c37b7, SUC_lexicon_tags_37 },
    { 0x3c3c37b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7c37c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bc37d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x17bc37d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x281c37d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94fc37d7, SUC_lexicon_tags_39 },
    { 0x949c37d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81c37dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fc37e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469c37e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3c37e7, SUC_lexicon_tags_25 },
    { 0xd57c37e8, SUC_lexicon_tags_1 },
    { 0xaafc37e9, SUC_lexicon_tags_25 },
    { 0x5dfc37ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7c37ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9c37f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bc37fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fc3801, SUC_lexicon_tags_25 },
    { 0x161c3801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x773c3804, SUC_lexicon_tags_10 },
    { 0xcdbc3804, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x86dc3807, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97c380a, SUC_lexicon_tags_37 },
    { 0x6ffc380a, SUC_lexicon_tags_129 },
    { 0xd1dc380c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc3810, SUC_lexicon_tags_27 },
    { 0x7dfc3811, SUC_lexicon_tags_25 },
    { 0x2d1c3812, SUC_lexicon_tags_57 },
    { 0xdbdc3813, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9c3816, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3c381c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83c3820, SUC_lexicon_tags_30 },
    { 0xec7c3820, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dc3829, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3edc382b, SUC_lexicon_tags_12 },
    { 0x98dc382c, SUC_lexicon_tags_25 },
    { 0x773c382d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609c3833, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bc3836, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xef1c3838, SUC_lexicon_tags_7 },
    { 0x2abc3839, SUC_lexicon_tags_7 },
    { 0xdbfc3839, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x18fc383d, SUC_lexicon_tags_55 },
    { 0xcebc383d, SUC_lexicon_tags_15 },
    { 0xf9dc383f, SUC_lexicon_tags_1 },
    { 0x5c9c383f, SUC_lexicon_tags_46 },
    { 0x1ffc3840, SUC_lexicon_tags_15 },
    { 0x533c3842, SUC_lexicon_tags_29 },
    { 0xbddc3843, SUC_lexicon_tags_2 },
    { 0x8a1c3844, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x745c3846, SUC_lexicon_tags_23 },
    { 0x405c3847, SUC_lexicon_tags_0 },
    { 0xe3fc3847, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffc384c, SUC_lexicon_tags_35 },
    { 0x1b5c384c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x181c3850, SUC_lexicon_tags_29 },
    { 0x49c3850, SUC_lexicon_tags_2 },
    { 0x237c3852, SUC_lexicon_tags_32 },
    { 0x6adc3853, SUC_lexicon_tags_4 },
    { 0x915c3854, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafc3857, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe57c3859, SUC_lexicon_tags_85 },
    { 0x375c385a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcb9c385c, SUC_lexicon_tags_11 },
    { 0xa29c385d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x17dc385f, SUC_lexicon_tags_12 },
    { 0x7d1c385f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9c7c3862, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc3865, SUC_lexicon_tags_14 },
    { 0xab5c3866, SUC_lexicon_tags_16 },
    { 0xc47c3865, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x787c3869, SUC_lexicon_tags_34 },
    { 0xbbdc3869, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddc386f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fc3878, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd3c387a, SUC_lexicon_tags_0 },
    { 0x8d3c387b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9c387f, SUC_lexicon_tags_11 },
    { 0x16dc3880, SUC_lexicon_tags_7 },
    { 0xd77c3881, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fc3888, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x899c388a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfc388d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcf1c388f, SUC_lexicon_tags_26 },
    { 0x6d7c3890, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7c3894, SUC_lexicon_tags_17 },
    { 0xa13c3895, SUC_lexicon_tags_1 },
    { 0x8d3c3896, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bc389a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc3bc389c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fc389f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05c38a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9c38aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1c38ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa87c38af, SUC_lexicon_tags_1 },
    { 0xdcbc38af, SUC_lexicon_tags_1 },
    { 0x8e7c38b1, SUC_lexicon_tags_7 },
    { 0x427c38b1, SUC_lexicon_tags_47 },
    { 0xb3c38b0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285c38b7, SUC_lexicon_tags_7 },
    { 0xfa9c38b8, SUC_lexicon_tags_48 },
    { 0x6b9c38b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ddc38bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9c38bd, SUC_lexicon_tags_12 },
    { 0x5a7c38be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5fdc38c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3c38c4, SUC_lexicon_tags_11 },
    { 0x217c38c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e3c38c7, SUC_lexicon_tags_25 },
    { 0x7a5c38c8, SUC_lexicon_tags_22 },
    { 0x14bc38c7, SUC_lexicon_tags_17 },
    { 0xdd7c38c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67c38cf, SUC_lexicon_tags_33 },
    { 0xa47c38d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe47c38d2, SUC_lexicon_tags_47 },
    { 0x211c38d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fc38d8, SUC_lexicon_tags_11 },
    { 0xb79c38d8, SUC_lexicon_tags_2 },
    { 0xfb9c38da, SUC_lexicon_tags_17 },
    { 0x5e3c38d8, SUC_lexicon_tags_17 },
    { 0xb79c38db, SUC_lexicon_tags_33 },
    { 0x195c38d8, SUC_lexicon_tags_12 },
    { 0xdebc38da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bc38e5, SUC_lexicon_tags_87 },
    { 0x8fc38e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fc38e9, SUC_lexicon_tags_29 },
    { 0x29bc38ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5c38ef, SUC_lexicon_tags_19 },
    { 0x699c38f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x499c38f2, SUC_lexicon_tags_27 },
    { 0x2efc38f2, SUC_lexicon_tags_68 },
    { 0xf29c38f4, SUC_lexicon_tags_32 },
    { 0xaffc38f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889c38fa, SUC_lexicon_tags_9 },
    { 0x44fc38fb, SUC_lexicon_tags_54 },
    { 0xb05c38fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x915c38fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bc3906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fc3908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1c390b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607c3912, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41fc3914, SUC_lexicon_tags_7 },
    { 0x15dc3915, SUC_lexicon_tags_7 },
    { 0xd3dc3914, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717c391f, SUC_lexicon_tags_33 },
    { 0xb5dc3920, SUC_lexicon_tags_37 },
    { 0xac1c391f, SUC_lexicon_tags_27 },
    { 0xd47c3920, SUC_lexicon_tags_13 },
    { 0xf85c3923, SUC_lexicon_tags_11 },
    { 0x6bc3922, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbc3929, SUC_lexicon_tags_43 },
    { 0xfe7c3929, SUC_lexicon_tags_43 },
    { 0x81fc392b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53c392f, SUC_lexicon_tags_37 },
    { 0xa1bc3930, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x325c3932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x617c3935, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdffc3937, SUC_lexicon_tags_39 },
    { 0xe9fc3938, SUC_lexicon_tags_1 },
    { 0x179c3939, SUC_lexicon_tags_2 },
    { 0x3c9c3938, SUC_lexicon_tags_82 },
    { 0xb0bc393b, SUC_lexicon_tags_17 },
    { 0x8ffc393c, SUC_lexicon_tags_11 },
    { 0x677c393d, SUC_lexicon_tags_79 },
    { 0xabfc393d, SUC_lexicon_tags_25 },
    { 0xc33c393f, SUC_lexicon_tags_1 },
    { 0xb33c3940, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc394a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbdc394c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c3c394e, SUC_lexicon_tags_7 },
    { 0x8e9c394e, SUC_lexicon_tags_25 },
    { 0xae3c3950, SUC_lexicon_tags_47 },
    { 0x48fc3951, SUC_lexicon_tags_27 },
    { 0xfcbc3950, SUC_lexicon_tags_2 },
    { 0x721c3953, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01c3956, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7c395c, SUC_lexicon_tags_29 },
    { 0xe6dc395c, SUC_lexicon_tags_43 },
    { 0xee9c395e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95c3963, SUC_lexicon_tags_9 },
    { 0x413c3964, SUC_lexicon_tags_37 },
    { 0x5cfc3965, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb67c3967, SUC_lexicon_tags_37 },
    { 0xe33c3968, SUC_lexicon_tags_1 },
    { 0x481c3969, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fc396d, SUC_lexicon_tags_39 },
    { 0xf57c396e, SUC_lexicon_tags_31 },
    { 0x289c396f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fc3972, SUC_lexicon_tags_11 },
    { 0x8d5c3973, SUC_lexicon_tags_7 },
    { 0x14dc3973, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5c3977, SUC_lexicon_tags_16 },
    { 0xeddc3978, SUC_lexicon_tags_235 },
    { 0xce9c3979, SUC_lexicon_tags_37 },
    { 0x56bc397a, SUC_lexicon_tags_1 },
    { 0xaf9c3978, SUC_lexicon_tags_26 },
    { 0xc25c397c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x279c397e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9c3980, SUC_lexicon_tags_14 },
    { 0x3e5c3981, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3c3986, SUC_lexicon_tags_63 },
    { 0x119c3987, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35c398b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33c398f, SUC_lexicon_tags_0 },
    { 0xc35c3990, SUC_lexicon_tags_0 },
    { 0x32dc398f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3c3996, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f5c3998, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8c9c399a, SUC_lexicon_tags_2 },
    { 0xf87c399b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc399e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xab3c39a0, SUC_lexicon_tags_25 },
    { 0xee9c39a0, SUC_lexicon_tags_8 },
    { 0x7e1c39a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb07c39a4, SUC_lexicon_tags_31 },
    { 0x24dc39a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c39b2, SUC_lexicon_tags_53 },
    { 0x817c39b2, SUC_lexicon_tags_7 },
    { 0x53dc39b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9c39b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fc39c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afc39c8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8b3c39ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x703c39cc, SUC_lexicon_tags_16 },
    { 0xf87c39cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dc39d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdc39e3, SUC_lexicon_tags_23 },
    { 0xff3c39e4, SUC_lexicon_tags_43 },
    { 0x929c39e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7cdc39e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9c39ea, SUC_lexicon_tags_10 },
    { 0x673c39eb, SUC_lexicon_tags_3 },
    { 0xc0dc39eb, SUC_lexicon_tags_34 },
    { 0x4cfc39eb, SUC_lexicon_tags_16 },
    { 0xb97c39eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07c39f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xadfc39f5, SUC_lexicon_tags_14 },
    { 0xda1c39f5, SUC_lexicon_tags_11 },
    { 0x617c39f5, SUC_lexicon_tags_1 },
    { 0x209c39f8, SUC_lexicon_tags_28 },
    { 0x75dc39f6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985c39fd, SUC_lexicon_tags_12 },
    { 0x167c39fe, SUC_lexicon_tags_35 },
    { 0xba7c39fd, SUC_lexicon_tags_11 },
    { 0xc9bc3a00, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619c3a06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x953c3a08, SUC_lexicon_tags_31 },
    { 0x137c3a09, SUC_lexicon_tags_31 },
    { 0x72fc3a09, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7c3a0e, SUC_lexicon_tags_2 },
    { 0x3f9c3a0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x561c3a11, SUC_lexicon_tags_11 },
    { 0xdbbc3a12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9c3a16, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42fc3a18, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3abc3a1a, SUC_lexicon_tags_31 },
    { 0x367c3a1b, SUC_lexicon_tags_12 },
    { 0xc17c3a1c, SUC_lexicon_tags_11 },
    { 0x567c3a1c, SUC_lexicon_tags_27 },
    { 0xbe7c3a1e, SUC_lexicon_tags_17 },
    { 0xc9c3a1e, SUC_lexicon_tags_12 },
    { 0xfdfc3a20, SUC_lexicon_tags_37 },
    { 0xb5dc3a1b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfc3a26, SUC_lexicon_tags_7 },
    { 0xd2dc3a27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2efc3a29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2bc3a2b, SUC_lexicon_tags_1 },
    { 0x4c3c3a2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1fc3a2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483c3a32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23bc3a34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5c3a3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6f5c3a3c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xac1c3a3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397c3a43, SUC_lexicon_tags_0 },
    { 0xf8fc3a44, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1c3a4b, SUC_lexicon_tags_25 },
    { 0x191c3a4b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x685c3a4e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bc3a57, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37c3a5d, SUC_lexicon_tags_23 },
    { 0x56bc3a5e, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0xc25c3a60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x675c3a63, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbc3a66, SUC_lexicon_tags_14 },
    { 0xa11c3a67, SUC_lexicon_tags_33 },
    { 0x223c3a68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909c3a6e, SUC_lexicon_tags_60 },
    { 0xf73c3a6f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x70bc3a71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bc3a77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb1bc3a79, SUC_lexicon_tags_23 },
    { 0xc63c3a7a, SUC_lexicon_tags_17 },
    { 0x7c7c3a7a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7c3a7e, SUC_lexicon_tags_9 },
    { 0x9cfc3a7f, SUC_lexicon_tags_37 },
    { 0x6e1c3a80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffc3a86, SUC_lexicon_tags_11 },
    { 0xbdbc3a86, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213c3a8c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc41c3a8e, SUC_lexicon_tags_17 },
    { 0xbcbc3a8f, SUC_lexicon_tags_37 },
    { 0x7abc3a90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadbc3a92, SUC_lexicon_tags_17 },
    { 0x8a7c3a93, SUC_lexicon_tags_60 },
    { 0xb35c3a93, SUC_lexicon_tags_1 },
    { 0x25fc3a95, SUC_lexicon_tags_30 },
    { 0x9f9c3a96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x455c3a98, SUC_lexicon_tags_42 },
    { 0xca1c3a99, SUC_lexicon_tags_63 },
    { 0x2b3c3a99, SUC_lexicon_tags_26 },
    { 0xed1c3a99, SUC_lexicon_tags_7 },
    { 0xb5bc3a9c, SUC_lexicon_tags_46 },
    { 0x599c3a9d, SUC_lexicon_tags_21 },
    { 0x8ebc3a98, SUC_lexicon_tags_12 },
    { 0xe01c3a9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1c3aa2, SUC_lexicon_tags_17 },
    { 0xa53c3aa2, SUC_lexicon_tags_11 },
    { 0x949c3aa2, SUC_lexicon_tags_31 },
    { 0xeafc3aa5, SUC_lexicon_tags_7 },
    { 0xb05c3aa6, SUC_lexicon_tags_0 },
    { 0xad7c3aa7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbc3aaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x365c3aac, SUC_lexicon_tags_30 },
    { 0x7fdc3aad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f7c3aaf, SUC_lexicon_tags_108 },
    { 0x10dc3ab0, SUC_lexicon_tags_5 },
    { 0x83c3ab1, SUC_lexicon_tags_15 },
    { 0xd85c3ab1, SUC_lexicon_tags_1 },
    { 0xa13c3ab3, SUC_lexicon_tags_3 },
    { 0x9e5c3ab2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fc3ab9, SUC_lexicon_tags_12 },
    { 0xcd3c3aba, SUC_lexicon_tags_11 },
    { 0x265c3abb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fc3ac0, SUC_lexicon_tags_15 },
    { 0x22bc3ac1, SUC_lexicon_tags_0 },
    { 0xfefc3ac2, SUC_lexicon_tags_10 },
    { 0x2b7c3ac0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x84bc3ac5, SUC_lexicon_tags_12 },
    { 0x37dc3ac6, SUC_lexicon_tags_11 },
    { 0xb21c3ac7, SUC_lexicon_tags_13 },
    { 0xcc3c3ac8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x195c3acb, SUC_lexicon_tags_33 },
    { 0xd37c3acc, SUC_lexicon_tags_31 },
    { 0xdfbc3acd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dc3ad0, SUC_lexicon_tags_46 },
    { 0x445c3ad1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafc3ad6, SUC_lexicon_tags_33 },
    { 0xa85c3ad7, SUC_lexicon_tags_43 },
    { 0x8a1c3ad7, SUC_lexicon_tags_37 },
    { 0x5c9c3ad9, SUC_lexicon_tags_17 },
    { 0x111c3ada, SUC_lexicon_tags_37 },
    { 0x6dc3ad9, SUC_lexicon_tags_46 },
    { 0xcefc3adc, SUC_lexicon_tags_60 },
    { 0x417c3add, SUC_lexicon_tags_7 },
    { 0x9fdc3ade, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f5c3ae0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bc3ae3, SUC_lexicon_tags_15 },
    { 0x1e3c3ae4, SUC_lexicon_tags_30 },
    { 0x971c3ae5, SUC_lexicon_tags_2 },
    { 0x719c3ae6, SUC_lexicon_tags_7 },
    { 0x743c3ae6, SUC_lexicon_tags_37 },
    { 0x5a7c3ae8, SUC_lexicon_tags_2 },
    { 0xa75c3ae9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9c3aec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x919c3aee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc95c3af0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15c3af2, SUC_lexicon_tags_37 },
    { 0x5a7c3af3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81c3af7, SUC_lexicon_tags_15 },
    { 0x2fdc3af8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41c3b01, SUC_lexicon_tags_1 },
    { 0xef3c3b01, SUC_lexicon_tags_37 },
    { 0x43bc3b03, SUC_lexicon_tags_13 },
    { 0x483c3b04, SUC_lexicon_tags_37 },
    { 0x139c3b05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc3b11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdc3b14, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07c3b17, SUC_lexicon_tags_69 },
    { 0x577c3b18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x527c3b1a, SUC_lexicon_tags_39 },
    { 0xf55c3b1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8bc3b1d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbc3b20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbc3b23, SUC_lexicon_tags_31 },
    { 0xba1c3b24, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x167c3b26, SUC_lexicon_tags_10 },
    { 0x783c3b27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a3c3b29, SUC_lexicon_tags_25 },
    { 0xefdc3b2a, SUC_lexicon_tags_17 },
    { 0xd33c3b2b, SUC_lexicon_tags_23 },
    { 0xbe9c3b2a, SUC_lexicon_tags_42 },
    { 0x63fc3b2d, SUC_lexicon_tags_49 },
    { 0xa17c3b2b, SUC_lexicon_tags_26 },
    { 0x5a3c3b2f, SUC_lexicon_tags_7 },
    { 0xc3dc3b2f, SUC_lexicon_tags_49 },
    { 0xabc3b31, SUC_lexicon_tags_7 },
    { 0x667c3b31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147c3b37, SUC_lexicon_tags_8 },
    { 0xe6dc3b38, SUC_lexicon_tags_25 },
    { 0x72bc3b37, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdc3b45, SUC_lexicon_tags_2 },
    { 0xfa3c3b46, SUC_lexicon_tags_17 },
    { 0x201c3b47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dc3b4e, SUC_lexicon_tags_60 },
    { 0xf67c3b4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3c3b55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bc3b58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf5c3b5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1c3b5e, SUC_lexicon_tags_17 },
    { 0x58bc3b5e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163c3b65, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddc3b68, SUC_lexicon_tags_63 },
    { 0xb49c3b68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c3b6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1c3b75, SUC_lexicon_tags_1 },
    { 0x8d9c3b76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717c3b7a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219c3b7e, SUC_lexicon_tags_2 },
    { 0x175c3b7f, SUC_lexicon_tags_25 },
    { 0xb05c3b7f, SUC_lexicon_tags_11 },
    { 0x3cfc3b7f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3c3b86, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xff3c3b88, SUC_lexicon_tags_11 },
    { 0x709c3b89, SUC_lexicon_tags_25 },
    { 0xfb7c3b88, SUC_lexicon_tags_14 },
    { 0x527c3b8b, SUC_lexicon_tags_25 },
    { 0xc75c3b8c, SUC_lexicon_tags_8 },
    { 0x315c3b88, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1c3b93, SUC_lexicon_tags_1 },
    { 0x44fc3b94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4f7c3b96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffc3b99, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5c3b9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x841c3b9e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efc3ba3, SUC_lexicon_tags_47 },
    { 0x9f3c3ba4, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bc3ba7, SUC_lexicon_tags_25 },
    { 0x88fc3ba8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755c3bad, SUC_lexicon_tags_60 },
    { 0xeb7c3bae, SUC_lexicon_tags_29 },
    { 0x1dbc3bad, SUC_lexicon_tags_0 },
    { 0x159c3bb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc73c3bb2, SUC_lexicon_tags_140 },
    { 0x77c3bb3, SUC_lexicon_tags_92 },
    { 0x5f9c3bb3, SUC_lexicon_tags_12 },
    { 0x2d7c3bb3, SUC_lexicon_tags_49 },
    { 0xfc9c3bb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d7c3bb8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf1c3bba, SUC_lexicon_tags_12 },
    { 0xaf5c3bbb, SUC_lexicon_tags_65 },
    { 0x3efc3bbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39bc3bbe, SUC_lexicon_tags_64 },
    { 0xf81c3bbe, SUC_lexicon_tags_14 },
    { 0x8afc3bc0, SUC_lexicon_tags_12 },
    { 0xdf1c3bc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7c3bc5, SUC_lexicon_tags_66 },
    { 0x6afc3bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x227c3bc9, SUC_lexicon_tags_37 },
    { 0xf99c3bc9, SUC_lexicon_tags_87 },
    { 0x5d3c3bca, SUC_lexicon_tags_31 },
    { 0x8b3c3bcc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x509c3bcf, SUC_lexicon_tags_1 },
    { 0x61dc3bd0, SUC_lexicon_tags_37 },
    { 0xda7c3bd1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fc3bd4, SUC_lexicon_tags_15 },
    { 0x17c3bd5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7c3bdc, SUC_lexicon_tags_63 },
    { 0x787c3bdd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdc3be1, SUC_lexicon_tags_17 },
    { 0xb1fc3be1, SUC_lexicon_tags_6 },
    { 0xd9c3be2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x551c3be5, SUC_lexicon_tags_68 },
    { 0x2f7c3be6, SUC_lexicon_tags_11 },
    { 0xbf3c3be7, SUC_lexicon_tags_33 },
    { 0xf53c3be7, SUC_lexicon_tags_11 },
    { 0xd7dc3be9, SUC_lexicon_tags_61 },
    { 0xc73c3be9, SUC_lexicon_tags_0 },
    { 0x8f3c3be5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bc3bf3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe27c3bf5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659c3bfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa1bc3bfe, SUC_lexicon_tags_27 },
    { 0x727c3bfe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfc3c05, SUC_lexicon_tags_39 },
    { 0x9adc3c05, SUC_lexicon_tags_7 },
    { 0xc7bc3c07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fdc3c09, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dc3c0c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dc3c0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87fc3c12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fc3c16, SUC_lexicon_tags_12 },
    { 0x805c3c17, SUC_lexicon_tags_46 },
    { 0x243c3c17, SUC_lexicon_tags_2 },
    { 0xdf7c3c17, SUC_lexicon_tags_23 },
    { 0xd9dc3c18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x55dc3c1c, SUC_lexicon_tags_31 },
    { 0xeadc3c1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3c3c21, SUC_lexicon_tags_60 },
    { 0x58fc3c22, SUC_lexicon_tags_47 },
    { 0xd09c3c23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x229c3c25, SUC_lexicon_tags_12 },
    { 0x87fc3c26, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3c3c2a, SUC_lexicon_tags_39 },
    { 0x9ddc3c2b, SUC_lexicon_tags_14 },
    { 0x5dfc3c2c, SUC_lexicon_tags_2 },
    { 0xc55c3c2d, SUC_lexicon_tags_1 },
    { 0x98fc3c2b, SUC_lexicon_tags_13 },
    { 0x5a1c3c2f, SUC_lexicon_tags_29 },
    { 0x64dc3c2e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3c3c35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dc3c38, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x20bc3c3a, SUC_lexicon_tags_25 },
    { 0x421c3c3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe1c3c3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x877c3c40, SUC_lexicon_tags_42 },
    { 0xd0dc3c41, SUC_lexicon_tags_2 },
    { 0x539c3c42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bc3c47, SUC_lexicon_tags_8 },
    { 0x32fc3c48, SUC_lexicon_tags_43 },
    { 0x8ddc3c47, SUC_lexicon_tags_33 },
    { 0x9f3c3c47, SUC_lexicon_tags_88 },
    { 0xb71c3c4b, SUC_lexicon_tags_27 },
    { 0x10fc3c4c, SUC_lexicon_tags_60 },
    { 0xc05c3c4d, SUC_lexicon_tags_25 },
    { 0xbcbc3c4c, SUC_lexicon_tags_32 },
    { 0x199c3c4d, SUC_lexicon_tags_7 },
    { 0xca9c3c4c, SUC_lexicon_tags_19 },
    { 0xd65c3c51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341c3c56, SUC_lexicon_tags_6 },
    { 0xac7c3c57, SUC_lexicon_tags_46 },
    { 0xcbc3c56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dc3c5d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc3c6b, SUC_lexicon_tags_10 },
    { 0x937c3c6c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701c3c72, SUC_lexicon_tags_31 },
    { 0xc55c3c73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bc3c7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x875c3c7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd41c3c7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac3c3c81, SUC_lexicon_tags_12 },
    { 0x2f3c3c81, SUC_lexicon_tags_4 },
    { 0xe21c3c81, SUC_lexicon_tags_37 },
    { 0x86fc3c83, SUC_lexicon_tags_15 },
    { 0x29bc3c85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbc3c88, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xfb5c3c8a, SUC_lexicon_tags_65 },
    { 0x29c3c8b, SUC_lexicon_tags_10 },
    { 0xc91c3c8c, SUC_lexicon_tags_37 },
    { 0xd25c3c8a, SUC_lexicon_tags_26 },
    { 0xaa1c3c8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441c3c92, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53c3c98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735c3c9d, SUC_lexicon_tags_1 },
    { 0xb5bc3c9d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xecbc3ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7bc3ca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609c3ca6, SUC_lexicon_tags_17 },
    { 0x297c3ca7, SUC_lexicon_tags_31 },
    { 0xebdc3ca8, SUC_lexicon_tags_82 },
    { 0xaadc3ca8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x893c3cab, SUC_lexicon_tags_11 },
    { 0x7c9c3cac, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x18dc3caf, SUC_lexicon_tags_37 },
    { 0xc5bc3caf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x303c3cb3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09c3cb6, SUC_lexicon_tags_68 },
    { 0xbf7c3cb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1c3cbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9fc3cc0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5c3cc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x589c3cc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37c3cc9, SUC_lexicon_tags_37 },
    { 0x767c3cca, SUC_lexicon_tags_17 },
    { 0x253c3ccb, SUC_lexicon_tags_8 },
    { 0xc03c3ccc, SUC_lexicon_tags_16 },
    { 0x431c3cc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77c3cd1, SUC_lexicon_tags_29 },
    { 0x157c3cd2, SUC_lexicon_tags_49 },
    { 0x1bfc3cd3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fc3cd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6dc3cda, SUC_lexicon_tags_26 },
    { 0x753c3cda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87c3cdd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d3c3cdf, SUC_lexicon_tags_37 },
    { 0xd23c3ce0, SUC_lexicon_tags_47 },
    { 0xd55c3ce0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323c3ce5, SUC_lexicon_tags_16 },
    { 0xc9bc3ce5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98bc3ce8, SUC_lexicon_tags_65 },
    { 0xd11c3ce9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7c3cec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d3c3cee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7e9c3cf0, SUC_lexicon_tags_43 },
    { 0xf6bc3cf0, SUC_lexicon_tags_1 },
    { 0x403c3cf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5c3cf5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fc3cf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1c3cfb, SUC_lexicon_tags_12 },
    { 0xee7c3cfb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1c3d03, SUC_lexicon_tags_27 },
    { 0x155c3d04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91c3d10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x415c3d12, SUC_lexicon_tags_2 },
    { 0x82bc3d13, SUC_lexicon_tags_37 },
    { 0x5e9c3d14, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbd9c3d16, SUC_lexicon_tags_37 },
    { 0x87dc3d16, SUC_lexicon_tags_15 },
    { 0xa05c3d18, SUC_lexicon_tags_7 },
    { 0x161c3d19, SUC_lexicon_tags_39 },
    { 0x16dc3d1a, SUC_lexicon_tags_27 },
    { 0xbc5c3d1b, SUC_lexicon_tags_11 },
    { 0xc75c3d1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf61c3d1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x597c3d20, SUC_lexicon_tags_12 },
    { 0x9f9c3d20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7c3d29, SUC_lexicon_tags_2 },
    { 0x55bc3d2a, SUC_lexicon_tags_63 },
    { 0x635c3d2b, SUC_lexicon_tags_11 },
    { 0x45bc3d2c, SUC_lexicon_tags_602 },
    { 0, NULL },
    { 0xfedc3d2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x333c3d31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317c3d35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9c3d39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x641c3d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23c3d3f, SUC_lexicon_tags_11 },
    { 0x15c3d40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x941c3d43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dc3d48, SUC_lexicon_tags_2 },
    { 0x777c3d49, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5c3d4e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5c3d50, SUC_lexicon_tags_23 },
    { 0x67dc3d50, SUC_lexicon_tags_47 },
    { 0xf21c3d52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7c3d55, SUC_lexicon_tags_45 },
    { 0x4e3c3d55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5c3d5a, SUC_lexicon_tags_2 },
    { 0xbd7c3d5b, SUC_lexicon_tags_38 },
    { 0x4abc3d5b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411c3d61, SUC_lexicon_tags_27 },
    { 0xab3c3d62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c3d64, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33c3d6a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fc3d6d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605c3d72, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8a1c3d74, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb19c3d76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x14bc3d78, SUC_lexicon_tags_37 },
    { 0x459c3d78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1c3d7e, SUC_lexicon_tags_23 },
    { 0x3fbc3d7f, SUC_lexicon_tags_12 },
    { 0x455c3d7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fc3d83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x765c3d85, SUC_lexicon_tags_37 },
    { 0x6a5c3d85, SUC_lexicon_tags_13 },
    { 0x113c3d87, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bc3d8b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9c3d90, SUC_lexicon_tags_55 },
    { 0xa5fc3d90, SUC_lexicon_tags_15 },
    { 0x80bc3d91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdc3d99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1c3d9c, SUC_lexicon_tags_45 },
    { 0x61dc3d9d, SUC_lexicon_tags_11 },
    { 0xb65c3d9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36dc3da0, SUC_lexicon_tags_2 },
    { 0x2ffc3da1, SUC_lexicon_tags_48 },
    { 0x213c3da0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x9f5c3da4, SUC_lexicon_tags_17 },
    { 0xfc1c3da5, SUC_lexicon_tags_39 },
    { 0xacdc3da6, SUC_lexicon_tags_25 },
    { 0x145c3da6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23c3dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99c3daf, SUC_lexicon_tags_11 },
    { 0xf6fc3db0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x163c3db2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5c3db7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dc3dbb, SUC_lexicon_tags_47 },
    { 0x67fc3dbc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701c3dc1, SUC_lexicon_tags_17 },
    { 0xab9c3dc2, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x853c3dc4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97c3dc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x529c3dca, SUC_lexicon_tags_11 },
    { 0x481c3dcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6edc3dce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fc3ddf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdb3c3de1, SUC_lexicon_tags_37 },
    { 0xd81c3de2, SUC_lexicon_tags_70 },
    { 0x9a9c3de2, SUC_lexicon_tags_40 },
    { 0x179c3de3, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3c3de7, SUC_lexicon_tags_1 },
    { 0x8d7c3de7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbc3deb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5c3df4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9c3df9, SUC_lexicon_tags_29 },
    { 0xf11c3dfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9c3dfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7c3e04, SUC_lexicon_tags_10 },
    { 0x127c3e05, SUC_lexicon_tags_11 },
    { 0xb97c3e04, SUC_lexicon_tags_7 },
    { 0x41c3e04, SUC_lexicon_tags_4 },
    { 0x4afc3e04, SUC_lexicon_tags_15 },
    { 0x87dc3e09, SUC_lexicon_tags_11 },
    { 0x4d7c3e0a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c3e0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dc3e15, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x31dc3e17, SUC_lexicon_tags_27 },
    { 0xd9fc3e17, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9a9c3e1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145c3e23, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdc3e26, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7c3e29, SUC_lexicon_tags_49 },
    { 0xb8bc3e2a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dc3e31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2fc3e33, SUC_lexicon_tags_63 },
    { 0x34bc3e34, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf33c3e36, SUC_lexicon_tags_11 },
    { 0x83fc3e37, SUC_lexicon_tags_41 },
    { 0xf8bc3e36, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efc3e3b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabc3e40, SUC_lexicon_tags_25 },
    { 0x71dc3e41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edc3e47, SUC_lexicon_tags_15 },
    { 0xea5c3e48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67c3e52, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4fc3e54, SUC_lexicon_tags_9 },
    { 0x195c3e55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4ebc3e57, SUC_lexicon_tags_15 },
    { 0x7f5c3e58, SUC_lexicon_tags_25 },
    { 0x959c3e59, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebc3e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5c3e62, SUC_lexicon_tags_49 },
    { 0x71bc3e62, SUC_lexicon_tags_17 },
    { 0x93dc3e64, SUC_lexicon_tags_47 },
    { 0x6bfc3e63, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3e6a, SUC_lexicon_tags_12 },
    { 0x45c3e6a, SUC_lexicon_tags_26 },
    { 0xb63c3e6c, SUC_lexicon_tags_6 },
    { 0xb65c3e6d, SUC_lexicon_tags_26 },
    { 0x747c3e6e, SUC_lexicon_tags_12 },
    { 0x93c3e6f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8c7c3e71, SUC_lexicon_tags_11 },
    { 0xdb1c3e71, SUC_lexicon_tags_2 },
    { 0xd17c3e73, SUC_lexicon_tags_19 },
    { 0x429c3e72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1c3e7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba9c3e7c, SUC_lexicon_tags_473 },
    { 0xeebc3e7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80dc3e7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96fc3e81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56bc3e83, SUC_lexicon_tags_91 },
    { 0x3adc3e83, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137c3e88, SUC_lexicon_tags_11 },
    { 0x691c3e88, SUC_lexicon_tags_17 },
    { 0xb9dc3e89, SUC_lexicon_tags_37 },
    { 0x4b1c3e8b, SUC_lexicon_tags_19 },
    { 0x909c3e8b, SUC_lexicon_tags_47 },
    { 0xdedc3e8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa2bc3e8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1c3e93, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113c3e97, SUC_lexicon_tags_17 },
    { 0xa87c3e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73c3e9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x55bc3e9c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x399c3e9e, SUC_lexicon_tags_33 },
    { 0xc9c3e9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fc3ea2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x79bc3ea4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47c3ea7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x817c3eaa, SUC_lexicon_tags_25 },
    { 0x213c3eab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x659c3ead, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bc3eb1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbc3eb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72bc3ebc, SUC_lexicon_tags_37 },
    { 0xf21c3ebd, SUC_lexicon_tags_68 },
    { 0xa2bc3ebe, SUC_lexicon_tags_43 },
    { 0x75c3ebf, SUC_lexicon_tags_11 },
    { 0x4bdc3ec0, SUC_lexicon_tags_7 },
    { 0x803c3ec1, SUC_lexicon_tags_29 },
    { 0x861c3ec1, SUC_lexicon_tags_23 },
    { 0x77bc3ebd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x349c3ec5, SUC_lexicon_tags_91 },
    { 0x681c3ec5, SUC_lexicon_tags_15 },
    { 0xc17c3ec7, SUC_lexicon_tags_1 },
    { 0xa3fc3ec8, SUC_lexicon_tags_2 },
    { 0xb85c3ec5, SUC_lexicon_tags_112 },
    { 0xd37c3eca, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913c3ecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5c3ed2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x363c3ed5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77c3ede, SUC_lexicon_tags_32 },
    { 0xdd9c3edf, SUC_lexicon_tags_11 },
    { 0x427c3ee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf7c3ee2, SUC_lexicon_tags_7 },
    { 0x4cdc3ee3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbc3ee9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555c3ef8, SUC_lexicon_tags_17 },
    { 0xf0dc3ef9, SUC_lexicon_tags_37 },
    { 0x5f3c3ef9, SUC_lexicon_tags_27 },
    { 0xb6dc3efb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c3efe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebc3f01, SUC_lexicon_tags_7 },
    { 0x32dc3f02, SUC_lexicon_tags_26 },
    { 0xbb5c3f01, SUC_lexicon_tags_29 },
    { 0xe33c3f01, SUC_lexicon_tags_26 },
    { 0xf4bc3f05, SUC_lexicon_tags_25 },
    { 0xa4fc3f06, SUC_lexicon_tags_25 },
    { 0x26fc3f03, SUC_lexicon_tags_87 },
    { 0x771c3f07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7abc3f0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923c3f12, SUC_lexicon_tags_46 },
    { 0x75c3f12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dc3f17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153c3f1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3c3f22, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbc3f26, SUC_lexicon_tags_25 },
    { 0x537c3f27, SUC_lexicon_tags_7 },
    { 0x16bc3f28, SUC_lexicon_tags_4 },
    { 0x6e7c3f27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x945c3f2b, SUC_lexicon_tags_55 },
    { 0xae1c3f2c, SUC_lexicon_tags_46 },
    { 0xd4dc3f2d, SUC_lexicon_tags_26 },
    { 0x36dc3f2d, SUC_lexicon_tags_12 },
    { 0xefc3f2f, SUC_lexicon_tags_29 },
    { 0x3cbc3f2f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3c3f33, SUC_lexicon_tags_17 },
    { 0xa5dc3f34, SUC_lexicon_tags_11 },
    { 0x5c3c3f35, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93c3f3a, SUC_lexicon_tags_12 },
    { 0x77c3f3b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a3c3f3d, SUC_lexicon_tags_25 },
    { 0x6cdc3f3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x189c3f41, SUC_lexicon_tags_47 },
    { 0xc5bc3f42, SUC_lexicon_tags_11 },
    { 0xcf3c3f43, SUC_lexicon_tags_16 },
    { 0x8bbc3f44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xea3c3f46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf5c3f48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9c5c3f4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x799c3f4d, SUC_lexicon_tags_7 },
    { 0x50bc3f4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a3c3f50, SUC_lexicon_tags_17 },
    { 0x55fc3f51, SUC_lexicon_tags_31 },
    { 0xfebc3f52, SUC_lexicon_tags_13 },
    { 0x7f5c3f53, SUC_lexicon_tags_27 },
    { 0x2adc3f54, SUC_lexicon_tags_63 },
    { 0xdc3c3f55, SUC_lexicon_tags_23 },
    { 0xff7c3f55, SUC_lexicon_tags_28 },
    { 0x69bc3f55, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5c3f5a, SUC_lexicon_tags_168 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fc3f62, SUC_lexicon_tags_3 },
    { 0xce5c3f62, SUC_lexicon_tags_37 },
    { 0xcbdc3f64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d1c3f66, SUC_lexicon_tags_47 },
    { 0xfb5c3f67, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x729c3f6a, SUC_lexicon_tags_47 },
    { 0x19bc3f6b, SUC_lexicon_tags_85 },
    { 0x3bdc3f6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7c3f71, SUC_lexicon_tags_33 },
    { 0xd45c3f72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf0bc3f74, SUC_lexicon_tags_25 },
    { 0x6a1c3f74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf3dc3f77, SUC_lexicon_tags_1 },
    { 0xa05c3f78, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bc3f87, SUC_lexicon_tags_11 },
    { 0x53bc3f87, SUC_lexicon_tags_30 },
    { 0xc1fc3f89, SUC_lexicon_tags_14 },
    { 0x821c3f88, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623c3f91, SUC_lexicon_tags_29 },
    { 0x18bc3f91, SUC_lexicon_tags_33 },
    { 0xbc5c3f93, SUC_lexicon_tags_11 },
    { 0x621c3f93, SUC_lexicon_tags_14 },
    { 0x201c3f93, SUC_lexicon_tags_39 },
    { 0xbb7c3f96, SUC_lexicon_tags_19 },
    { 0xdc3f97, SUC_lexicon_tags_2 },
    { 0x583c3f97, SUC_lexicon_tags_25 },
    { 0x54dc3f99, SUC_lexicon_tags_7 },
    { 0x579c3f96, SUC_lexicon_tags_11 },
    { 0x8bc3f98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc75c3f9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x291c3f9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabc3fa2, SUC_lexicon_tags_25 },
    { 0xbb3c3fa3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7c3fa8, SUC_lexicon_tags_47 },
    { 0x35dc3fa9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ebc3fab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc3fae, SUC_lexicon_tags_21 },
    { 0x835c3faf, SUC_lexicon_tags_41 },
    { 0xd4fc3faf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3c3fb8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5a1c3fba, SUC_lexicon_tags_8 },
    { 0x3edc3fbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeadc3fbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1c3fc3, SUC_lexicon_tags_45 },
    { 0x96bc3fc3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xedbc3fc6, SUC_lexicon_tags_23 },
    { 0xf71c3fc7, SUC_lexicon_tags_29 },
    { 0x3a3c3fc7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfd5c3fca, SUC_lexicon_tags_33 },
    { 0x429c3fca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2fc3fcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05c3fd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22bc3fd3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3efc3fd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95dc3fd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabc3fe0, SUC_lexicon_tags_38 },
    { 0x705c3fe1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26dc3fe9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5afc3feb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1b5c3fed, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05c3ff0, SUC_lexicon_tags_8 },
    { 0x783c3ff0, SUC_lexicon_tags_14 },
    { 0xae3c3ff1, SUC_lexicon_tags_7 },
    { 0xb71c3ff3, SUC_lexicon_tags_12 },
    { 0x5b5c3ff3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c3ffc, SUC_lexicon_tags_1 },
    { 0x5fbc3ffc, SUC_lexicon_tags_15 },
    { 0x57dc3ffe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5dfc4000, SUC_lexicon_tags_3 },
    { 0xfdc4000, SUC_lexicon_tags_25 },
    { 0x46fc4000, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dc4004, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x90bc4007, SUC_lexicon_tags_38 },
    { 0xeb9c4008, SUC_lexicon_tags_23 },
    { 0x257c4007, SUC_lexicon_tags_23 },
    { 0x8f7c400a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97c400d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc83c400f, SUC_lexicon_tags_1 },
    { 0x18dc4010, SUC_lexicon_tags_9 },
    { 0x9dbc400f, SUC_lexicon_tags_37 },
    { 0x7a1c4012, SUC_lexicon_tags_1 },
    { 0x7fbc4013, SUC_lexicon_tags_2 },
    { 0xe2dc4014, SUC_lexicon_tags_17 },
    { 0x377c4015, SUC_lexicon_tags_47 },
    { 0x8f7c4016, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x351c4018, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1c401b, SUC_lexicon_tags_55 },
    { 0x625c401c, SUC_lexicon_tags_436 },
    { 0, NULL },
    { 0x6abc401e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65c4021, SUC_lexicon_tags_15 },
    { 0x709c4022, SUC_lexicon_tags_12 },
    { 0xf41c4023, SUC_lexicon_tags_14 },
    { 0xe8fc4024, SUC_lexicon_tags_12 },
    { 0xe51c4021, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7c4028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaddc402a, SUC_lexicon_tags_47 },
    { 0x8d3c402b, SUC_lexicon_tags_2 },
    { 0x33c402c, SUC_lexicon_tags_11 },
    { 0x395c402b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bc4030, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bc4033, SUC_lexicon_tags_45 },
    { 0x55dc4034, SUC_lexicon_tags_1 },
    { 0x9a5c4035, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x453c4038, SUC_lexicon_tags_37 },
    { 0xb09c4039, SUC_lexicon_tags_14 },
    { 0xce5c4038, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79c403f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b7c4041, SUC_lexicon_tags_2 },
    { 0x79bc4041, SUC_lexicon_tags_19 },
    { 0x475c4043, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731c4047, SUC_lexicon_tags_2 },
    { 0xb71c4048, SUC_lexicon_tags_12 },
    { 0x83fc4048, SUC_lexicon_tags_17 },
    { 0x2cbc4048, SUC_lexicon_tags_7 },
    { 0xd1bc404b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881c4056, SUC_lexicon_tags_37 },
    { 0xc7dc4056, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03c405a, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83c405e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeddc4060, SUC_lexicon_tags_19 },
    { 0xef1c4061, SUC_lexicon_tags_37 },
    { 0xdbbc4062, SUC_lexicon_tags_39 },
    { 0x9b9c4062, SUC_lexicon_tags_37 },
    { 0xfe9c4064, SUC_lexicon_tags_77 },
    { 0x6d7c4062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bc4068, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83c4070, SUC_lexicon_tags_47 },
    { 0x739c4071, SUC_lexicon_tags_11 },
    { 0xc7c4072, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x431c4075, SUC_lexicon_tags_10 },
    { 0x755c4076, SUC_lexicon_tags_23 },
    { 0x7d3c4077, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x597c407a, SUC_lexicon_tags_30 },
    { 0x7afc407a, SUC_lexicon_tags_37 },
    { 0xd9dc407c, SUC_lexicon_tags_19 },
    { 0x7b1c407d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dc4081, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc35c4083, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c408b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fc4090, SUC_lexicon_tags_19 },
    { 0xd51c4090, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88dc4096, SUC_lexicon_tags_60 },
    { 0xd3c4096, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7dc4099, SUC_lexicon_tags_49 },
    { 0xe1c409a, SUC_lexicon_tags_25 },
    { 0x45bc409b, SUC_lexicon_tags_46 },
    { 0x53fc409c, SUC_lexicon_tags_17 },
    { 0x3b7c409d, SUC_lexicon_tags_37 },
    { 0x849c409c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9c40a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bc40a5, SUC_lexicon_tags_23 },
    { 0x92dc40a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c40ae, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdc40b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743c40bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb3bc40be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbc40c1, SUC_lexicon_tags_33 },
    { 0x4c1c40c2, SUC_lexicon_tags_23 },
    { 0xbe1c40c3, SUC_lexicon_tags_1 },
    { 0xfe1c40c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5c40c9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x285c40cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8fbc40cd, SUC_lexicon_tags_49 },
    { 0xb05c40ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x273c40d0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x34bc40d3, SUC_lexicon_tags_7 },
    { 0xdafc40d4, SUC_lexicon_tags_11 },
    { 0xde1c40d5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1c40db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bc40e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa67c40e6, SUC_lexicon_tags_28 },
    { 0x931c40e6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x875c40e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1c40ed, SUC_lexicon_tags_40 },
    { 0x797c40ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69c40f3, SUC_lexicon_tags_12 },
    { 0x9ebc40f3, SUC_lexicon_tags_10 },
    { 0x2d3c40f5, SUC_lexicon_tags_1 },
    { 0x8cbc40f4, SUC_lexicon_tags_87 },
    { 0x6a5c40f7, SUC_lexicon_tags_63 },
    { 0x6e1c40f8, SUC_lexicon_tags_28 },
    { 0xd45c40f5, SUC_lexicon_tags_23 },
    { 0xf23c40fa, SUC_lexicon_tags_38 },
    { 0x9f5c40f6, SUC_lexicon_tags_66 },
    { 0xeb3c40fc, SUC_lexicon_tags_42 },
    { 0xf33c40fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c4103, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xedfc4105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x231c4107, SUC_lexicon_tags_0 },
    { 0x7f7c4108, SUC_lexicon_tags_79 },
    { 0xa77c4108, SUC_lexicon_tags_37 },
    { 0x56fc410a, SUC_lexicon_tags_12 },
    { 0xfadc4109, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13c410f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61c411c, SUC_lexicon_tags_28 },
    { 0xc23c411d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5cbc411f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x881c4122, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x521c4125, SUC_lexicon_tags_47 },
    { 0x673c4125, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20dc4128, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b7c412a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdc412d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdadc412f, SUC_lexicon_tags_35 },
    { 0x611c412f, SUC_lexicon_tags_32 },
    { 0xcddc4131, SUC_lexicon_tags_17 },
    { 0x53c4132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95c4135, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x785c4137, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdc413e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94dc4143, SUC_lexicon_tags_43 },
    { 0x27dc4143, SUC_lexicon_tags_25 },
    { 0xd7dc4145, SUC_lexicon_tags_38 },
    { 0xaf5c4145, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21c4152, SUC_lexicon_tags_13 },
    { 0x951c4153, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe95c4155, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9c415b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbc415e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfc4163, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01c416b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbc4170, SUC_lexicon_tags_87 },
    { 0x4bdc4171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fc4174, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4d3c4176, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x2f1c4178, SUC_lexicon_tags_84 },
    { 0x939c4179, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa59c417b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfc417f, SUC_lexicon_tags_37 },
    { 0x7b7c4180, SUC_lexicon_tags_39 },
    { 0x9c9c4181, SUC_lexicon_tags_8 },
    { 0x551c4181, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd97c4184, SUC_lexicon_tags_11 },
    { 0xb4fc4185, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871c418b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21c418e, SUC_lexicon_tags_2 },
    { 0x6cdc418f, SUC_lexicon_tags_85 },
    { 0x8fdc418e, SUC_lexicon_tags_17 },
    { 0x375c4191, SUC_lexicon_tags_15 },
    { 0xb21c4192, SUC_lexicon_tags_77 },
    { 0xbc7c4192, SUC_lexicon_tags_29 },
    { 0x65bc4194, SUC_lexicon_tags_26 },
    { 0xc43c4195, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfc419e, SUC_lexicon_tags_7 },
    { 0x9e9c419f, SUC_lexicon_tags_17 },
    { 0xab5c41a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1fc41a2, SUC_lexicon_tags_5 },
    { 0xec7c41a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x295c41a5, SUC_lexicon_tags_15 },
    { 0xadc41a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc41aa, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585c41c1, SUC_lexicon_tags_4 },
    { 0x7a5c41c2, SUC_lexicon_tags_11 },
    { 0xa25c41c3, SUC_lexicon_tags_45 },
    { 0xe83c41c2, SUC_lexicon_tags_45 },
    { 0x347c41c5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1c41c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x861c41ca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379c41d3, SUC_lexicon_tags_3 },
    { 0x82bc41d4, SUC_lexicon_tags_17 },
    { 0x1efc41d3, SUC_lexicon_tags_12 },
    { 0x5c41d3, SUC_lexicon_tags_19 },
    { 0xfe9c41d5, SUC_lexicon_tags_43 },
    { 0xe2dc41d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf4bc41da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedc41df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x477c41e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517c41ec, SUC_lexicon_tags_1 },
    { 0xda3c41ed, SUC_lexicon_tags_28 },
    { 0x3edc41ed, SUC_lexicon_tags_38 },
    { 0x1bfc41ef, SUC_lexicon_tags_6 },
    { 0xf91c41ee, SUC_lexicon_tags_25 },
    { 0x15bc41ec, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83c41f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8dc41f6, SUC_lexicon_tags_55 },
    { 0xb6dc41f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1c41fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa99c41fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfc4203, SUC_lexicon_tags_2 },
    { 0x6c7c4203, SUC_lexicon_tags_21 },
    { 0x361c4205, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3c4208, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc420d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddc4211, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf6fc4213, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751c4218, SUC_lexicon_tags_28 },
    { 0x4cbc4218, SUC_lexicon_tags_7 },
    { 0x757c421a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x687c421c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3c421f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x4b5c4221, SUC_lexicon_tags_9 },
    { 0x66fc4222, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c4225, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbe3c4227, SUC_lexicon_tags_37 },
    { 0xa4fc4228, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c4234, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x471c4237, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341c423b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6d1c423d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x305c423f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9c4242, SUC_lexicon_tags_7 },
    { 0x3f3c4242, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615c4249, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7c424d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefc4253, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xca1c4255, SUC_lexicon_tags_29 },
    { 0xfd7c4256, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x6e9c4258, SUC_lexicon_tags_25 },
    { 0x207c4259, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9c425d, SUC_lexicon_tags_26 },
    { 0xbc5c425d, SUC_lexicon_tags_65 },
    { 0xbd9c425f, SUC_lexicon_tags_33 },
    { 0x5abc425f, SUC_lexicon_tags_44 },
    { 0xaffc425f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x41c4263, SUC_lexicon_tags_14 },
    { 0x2ddc4263, SUC_lexicon_tags_1 },
    { 0xb87c4263, SUC_lexicon_tags_35 },
    { 0x747c4264, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86fc4268, SUC_lexicon_tags_25 },
    { 0x643c4269, SUC_lexicon_tags_37 },
    { 0x58bc426a, SUC_lexicon_tags_30 },
    { 0xdcbc426b, SUC_lexicon_tags_101 },
    { 0x367c426b, SUC_lexicon_tags_81 },
    { 0x8e9c426d, SUC_lexicon_tags_63 },
    { 0xc93c426a, SUC_lexicon_tags_29 },
    { 0x19c426a, SUC_lexicon_tags_14 },
    { 0xf85c4270, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259c4275, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581c4279, SUC_lexicon_tags_25 },
    { 0x78fc427a, SUC_lexicon_tags_7 },
    { 0xf3bc4279, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xb35c427d, SUC_lexicon_tags_15 },
    { 0xb9fc427e, SUC_lexicon_tags_7 },
    { 0x7f5c427f, SUC_lexicon_tags_1325 },
    { 0x4ddc4280, SUC_lexicon_tags_7 },
    { 0x7fc4281, SUC_lexicon_tags_19 },
    { 0xd61c4282, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8d9c4284, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x67bc4286, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109c428a, SUC_lexicon_tags_15 },
    { 0x523c428b, SUC_lexicon_tags_29 },
    { 0xf43c428c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9dc428e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x715c4290, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c4293, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9c429a, SUC_lexicon_tags_217 },
    { 0x761c429b, SUC_lexicon_tags_63 },
    { 0x381c429b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f7c429e, SUC_lexicon_tags_16 },
    { 0x44fc429e, SUC_lexicon_tags_2 },
    { 0x8e1c42a0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x833c42a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bfc42a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01c42b4, SUC_lexicon_tags_40 },
    { 0x8dc42b5, SUC_lexicon_tags_33 },
    { 0x2c9c42b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5c42b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4c5c42bb, SUC_lexicon_tags_11 },
    { 0x629c42bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf37c42be, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dc42c2, SUC_lexicon_tags_11 },
    { 0xf31c42c3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x7cbc42c5, SUC_lexicon_tags_11 },
    { 0x7d7c42c6, SUC_lexicon_tags_17 },
    { 0xf97c42c6, SUC_lexicon_tags_38 },
    { 0x551c42c8, SUC_lexicon_tags_12 },
    { 0x6a7c42c9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fc42d4, SUC_lexicon_tags_7 },
    { 0x92fc42d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac7c42d7, SUC_lexicon_tags_17 },
    { 0x43dc42d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf0bc42da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5bc42dc, SUC_lexicon_tags_25 },
    { 0x4a3c42dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc21c42df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63c42e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3c42e9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x703c42ec, SUC_lexicon_tags_8 },
    { 0xa65c42ed, SUC_lexicon_tags_13 },
    { 0x7bbc42ee, SUC_lexicon_tags_37 },
    { 0xbfbc42ec, SUC_lexicon_tags_25 },
    { 0xac5c42f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91c42f5, SUC_lexicon_tags_19 },
    { 0x867c42f6, SUC_lexicon_tags_49 },
    { 0x837c42f7, SUC_lexicon_tags_29 },
    { 0xfcfc42f8, SUC_lexicon_tags_11 },
    { 0x785c42f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89c42fc, SUC_lexicon_tags_17 },
    { 0x9ebc42fd, SUC_lexicon_tags_25 },
    { 0xb49c42fd, SUC_lexicon_tags_15 },
    { 0xe3dc42ff, SUC_lexicon_tags_12 },
    { 0x583c4300, SUC_lexicon_tags_1 },
    { 0x8ddc4301, SUC_lexicon_tags_38 },
    { 0x723c4302, SUC_lexicon_tags_29 },
    { 0x4a5c4303, SUC_lexicon_tags_44 },
    { 0x49c4304, SUC_lexicon_tags_10 },
    { 0x86bc42fd, SUC_lexicon_tags_91 },
    { 0xf6bc4305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05c430a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40fc430c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62dc430e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3c4311, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43c4314, SUC_lexicon_tags_38 },
    { 0x225c4314, SUC_lexicon_tags_35 },
    { 0x915c4314, SUC_lexicon_tags_5 },
    { 0x329c4317, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x405c4319, SUC_lexicon_tags_17 },
    { 0x43bc431a, SUC_lexicon_tags_49 },
    { 0xf65c431a, SUC_lexicon_tags_17 },
    { 0x481c431c, SUC_lexicon_tags_8 },
    { 0xfbdc431d, SUC_lexicon_tags_37 },
    { 0xcafc431c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x551c4321, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc95c4323, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc4328, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9c432f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2edc4331, SUC_lexicon_tags_17 },
    { 0xe59c4332, SUC_lexicon_tags_33 },
    { 0xec3c4331, SUC_lexicon_tags_7 },
    { 0x131c4333, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231c433c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbc4341, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd69c4343, SUC_lexicon_tags_12 },
    { 0xcd1c4343, SUC_lexicon_tags_49 },
    { 0x71bc4343, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13c4348, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3c434c, SUC_lexicon_tags_25 },
    { 0x3a7c434d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbc4351, SUC_lexicon_tags_11 },
    { 0xe9dc4351, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc4355, SUC_lexicon_tags_12 },
    { 0x445c4356, SUC_lexicon_tags_0 },
    { 0x75bc4356, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc65c4359, SUC_lexicon_tags_39 },
    { 0x745c435a, SUC_lexicon_tags_21 },
    { 0x2f9c435b, SUC_lexicon_tags_19 },
    { 0x3cfc435b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615c4361, SUC_lexicon_tags_47 },
    { 0xe5bc4362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabc4366, SUC_lexicon_tags_15 },
    { 0x8ebc4366, SUC_lexicon_tags_8 },
    { 0xa3fc4368, SUC_lexicon_tags_29 },
    { 0x9d3c4368, SUC_lexicon_tags_47 },
    { 0x88dc436a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7c436f, SUC_lexicon_tags_11 },
    { 0xdddc4370, SUC_lexicon_tags_15 },
    { 0xfb1c4371, SUC_lexicon_tags_68 },
    { 0xafdc436f, SUC_lexicon_tags_37 },
    { 0xbfc4372, SUC_lexicon_tags_37 },
    { 0x637c4374, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fc437c, SUC_lexicon_tags_1 },
    { 0x7fbc437d, SUC_lexicon_tags_11 },
    { 0x895c437d, SUC_lexicon_tags_37 },
    { 0x83dc437f, SUC_lexicon_tags_63 },
    { 0xd23c437c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c4384, SUC_lexicon_tags_1 },
    { 0xa5c4385, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7c438e, SUC_lexicon_tags_47 },
    { 0xb75c438e, SUC_lexicon_tags_46 },
    { 0xb8bc438e, SUC_lexicon_tags_15 },
    { 0x263c4391, SUC_lexicon_tags_9 },
    { 0x4c3c4392, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x507c4394, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7c439b, SUC_lexicon_tags_25 },
    { 0xedbc439c, SUC_lexicon_tags_15 },
    { 0x9c9c439d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1c43a1, SUC_lexicon_tags_34 },
    { 0xde5c43a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abc43a5, SUC_lexicon_tags_37 },
    { 0xec9c43a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdedc43b6, SUC_lexicon_tags_7 },
    { 0xe87c43b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc43bc, SUC_lexicon_tags_47 },
    { 0x933c43bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bc43c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74fc43c3, SUC_lexicon_tags_14 },
    { 0x8bc43c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9fc43c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b3c43c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61c43ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x82dc43d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x35c43d3, SUC_lexicon_tags_10 },
    { 0xfc3c43d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bc43d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55c43db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7c43e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x66dc43e6, SUC_lexicon_tags_61 },
    { 0xee7c43e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfc43ea, SUC_lexicon_tags_49 },
    { 0x3bbc43ea, SUC_lexicon_tags_5 },
    { 0x57dc43ea, SUC_lexicon_tags_17 },
    { 0x335c43eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc43f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbc43f7, SUC_lexicon_tags_61 },
    { 0x767c43f8, SUC_lexicon_tags_5 },
    { 0x71dc43f9, SUC_lexicon_tags_28 },
    { 0xe15c43fa, SUC_lexicon_tags_14 },
    { 0x57bc43fb, SUC_lexicon_tags_31 },
    { 0x8bdc43fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349c4400, SUC_lexicon_tags_25 },
    { 0xc5c4400, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7c4406, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197c440c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15c440f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x343c4412, SUC_lexicon_tags_63 },
    { 0x8dc4413, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x1c3c4415, SUC_lexicon_tags_25 },
    { 0xd07c4416, SUC_lexicon_tags_29 },
    { 0x2dbc4416, SUC_lexicon_tags_45 },
    { 0x33c4418, SUC_lexicon_tags_11 },
    { 0x611c4419, SUC_lexicon_tags_2 },
    { 0x74dc441a, SUC_lexicon_tags_15 },
    { 0xe6bc441b, SUC_lexicon_tags_25 },
    { 0x901c441a, SUC_lexicon_tags_39 },
    { 0x5e7c4417, SUC_lexicon_tags_37 },
    { 0x18fc4418, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adc4429, SUC_lexicon_tags_12 },
    { 0x3a3c4429, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dc4432, SUC_lexicon_tags_14 },
    { 0x8a9c4432, SUC_lexicon_tags_1 },
    { 0x415c4434, SUC_lexicon_tags_12 },
    { 0xe5dc4435, SUC_lexicon_tags_14 },
    { 0xe6fc4435, SUC_lexicon_tags_1 },
    { 0x8fbc4437, SUC_lexicon_tags_47 },
    { 0x2c5c4437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33c443b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x74bc443d, SUC_lexicon_tags_2 },
    { 0xdf5c443e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11c4443, SUC_lexicon_tags_11 },
    { 0x2ffc4444, SUC_lexicon_tags_17 },
    { 0x90dc4445, SUC_lexicon_tags_37 },
    { 0xc1c4443, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xec9c4448, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x275c444a, SUC_lexicon_tags_1 },
    { 0x26fc444b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedc4450, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x983c4453, SUC_lexicon_tags_19 },
    { 0x523c4454, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39c445a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1c445c, SUC_lexicon_tags_47 },
    { 0x67fc445d, SUC_lexicon_tags_60 },
    { 0xde3c445e, SUC_lexicon_tags_61 },
    { 0xa3fc445f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdc4464, SUC_lexicon_tags_1 },
    { 0xb7dc4465, SUC_lexicon_tags_30 },
    { 0xf77c4464, SUC_lexicon_tags_91 },
    { 0xfa1c4467, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fc446e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x585c4470, SUC_lexicon_tags_25 },
    { 0x933c4470, SUC_lexicon_tags_37 },
    { 0xe29c4472, SUC_lexicon_tags_1 },
    { 0x831c4471, SUC_lexicon_tags_12 },
    { 0x62fc4474, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fc4479, SUC_lexicon_tags_5 },
    { 0xce7c447a, SUC_lexicon_tags_45 },
    { 0xf33c447b, SUC_lexicon_tags_1 },
    { 0xa57c447c, SUC_lexicon_tags_1 },
    { 0xf53c447d, SUC_lexicon_tags_17 },
    { 0x881c447e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe27c4480, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x359c4483, SUC_lexicon_tags_11 },
    { 0x99bc4484, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c448a, SUC_lexicon_tags_12 },
    { 0x79fc448b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7c448f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679c4493, SUC_lexicon_tags_16 },
    { 0x39c4494, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x219c4497, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x787c4499, SUC_lexicon_tags_31 },
    { 0x6c7c449a, SUC_lexicon_tags_25 },
    { 0xc09c449b, SUC_lexicon_tags_12 },
    { 0x2c7c4499, SUC_lexicon_tags_16 },
    { 0xb37c449d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfc44a6, SUC_lexicon_tags_7 },
    { 0xbe3c44a6, SUC_lexicon_tags_15 },
    { 0x963c44a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afc44ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x743c44af, SUC_lexicon_tags_37 },
    { 0x21dc44b0, SUC_lexicon_tags_39 },
    { 0x89fc44b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedc44b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbddc44b8, SUC_lexicon_tags_22 },
    { 0xe9fc44b9, SUC_lexicon_tags_37 },
    { 0xcefc44ba, SUC_lexicon_tags_7 },
    { 0xa1c44ba, SUC_lexicon_tags_47 },
    { 0x659c44ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7c44c2, SUC_lexicon_tags_2 },
    { 0xe6dc44c2, SUC_lexicon_tags_17 },
    { 0x279c44c4, SUC_lexicon_tags_4 },
    { 0x631c44c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x381c44c7, SUC_lexicon_tags_2 },
    { 0x365c44c8, SUC_lexicon_tags_2 },
    { 0xc7c44c9, SUC_lexicon_tags_7 },
    { 0x30fc44c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x473c44cc, SUC_lexicon_tags_11 },
    { 0xdfdc44cd, SUC_lexicon_tags_49 },
    { 0x95c44ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35c44d3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6c5c44d5, SUC_lexicon_tags_7 },
    { 0xd5bc44d6, SUC_lexicon_tags_1 },
    { 0x419c44d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x22bc44d9, SUC_lexicon_tags_49 },
    { 0xc0fc44d9, SUC_lexicon_tags_14 },
    { 0x857c44db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95c44e7, SUC_lexicon_tags_17 },
    { 0x141c44e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeffc44ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c44ed, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adc44fb, SUC_lexicon_tags_26 },
    { 0x90dc44fc, SUC_lexicon_tags_37 },
    { 0x299c44fd, SUC_lexicon_tags_10 },
    { 0x7cfc44fd, SUC_lexicon_tags_15 },
    { 0x9c9c44fb, SUC_lexicon_tags_11 },
    { 0x8edc44fd, SUC_lexicon_tags_65 },
    { 0x33fc44ff, SUC_lexicon_tags_26 },
    { 0xb37c4502, SUC_lexicon_tags_31 },
    { 0xb0bc4502, SUC_lexicon_tags_25 },
    { 0xaf3c4502, SUC_lexicon_tags_55 },
    { 0x237c4504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x943c4508, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fc450d, SUC_lexicon_tags_37 },
    { 0x19dc450d, SUC_lexicon_tags_14 },
    { 0xa67c450f, SUC_lexicon_tags_0 },
    { 0xa83c4510, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9c4516, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdf9c4518, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353c451c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101c4523, SUC_lexicon_tags_26 },
    { 0x34fc4524, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x817c4526, SUC_lexicon_tags_15 },
    { 0xe27c4526, SUC_lexicon_tags_6 },
    { 0x353c4528, SUC_lexicon_tags_48 },
    { 0x9afc4526, SUC_lexicon_tags_12 },
    { 0x607c452a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5c452f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463c4533, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdc4535, SUC_lexicon_tags_28 },
    { 0x955c4536, SUC_lexicon_tags_7 },
    { 0xe77c4535, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfc453c, SUC_lexicon_tags_17 },
    { 0xa93c453c, SUC_lexicon_tags_11 },
    { 0x157c453e, SUC_lexicon_tags_2 },
    { 0xb45c453e, SUC_lexicon_tags_0 },
    { 0x5e9c4540, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xecbc4542, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13c4548, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbc454b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x14dc454d, SUC_lexicon_tags_37 },
    { 0x517c454d, SUC_lexicon_tags_37 },
    { 0x3d9c454f, SUC_lexicon_tags_8 },
    { 0xccfc4550, SUC_lexicon_tags_0 },
    { 0x535c4551, SUC_lexicon_tags_37 },
    { 0x5f5c4551, SUC_lexicon_tags_45 },
    { 0xd37c454d, SUC_lexicon_tags_336 },
    { 0xd8dc4553, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc4557, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3fc4559, SUC_lexicon_tags_68 },
    { 0x341c4559, SUC_lexicon_tags_68 },
    { 0x895c455b, SUC_lexicon_tags_47 },
    { 0xef5c455c, SUC_lexicon_tags_25 },
    { 0x337c455b, SUC_lexicon_tags_1 },
    { 0x8fdc455e, SUC_lexicon_tags_17 },
    { 0x96bc455f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313c4566, SUC_lexicon_tags_1 },
    { 0xddbc4567, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1c456c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa51c456f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x915c4572, SUC_lexicon_tags_2 },
    { 0xc85c4573, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8efc4575, SUC_lexicon_tags_17 },
    { 0x25c4576, SUC_lexicon_tags_12 },
    { 0x85bc4576, SUC_lexicon_tags_13 },
    { 0x589c4578, SUC_lexicon_tags_11 },
    { 0xa2dc4578, SUC_lexicon_tags_27 },
    { 0xbb3c4575, SUC_lexicon_tags_12 },
    { 0x825c4576, SUC_lexicon_tags_87 },
    { 0x7d5c4578, SUC_lexicon_tags_21 },
    { 0x919c457d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c7c457f, SUC_lexicon_tags_26 },
    { 0xf77c4580, SUC_lexicon_tags_47 },
    { 0xca1c4581, SUC_lexicon_tags_38 },
    { 0x7b9c4582, SUC_lexicon_tags_25 },
    { 0xf9fc4582, SUC_lexicon_tags_35 },
    { 0x69c4581, SUC_lexicon_tags_15 },
    { 0x3b5c4580, SUC_lexicon_tags_23 },
    { 0xe6dc4582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd93c4589, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8ffc458b, SUC_lexicon_tags_12 },
    { 0x27bc458c, SUC_lexicon_tags_23 },
    { 0x78bc458d, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xc1fc458f, SUC_lexicon_tags_65 },
    { 0x46dc458f, SUC_lexicon_tags_6 },
    { 0x579c4591, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x759c4593, SUC_lexicon_tags_12 },
    { 0x6c1c4594, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x69c4596, SUC_lexicon_tags_63 },
    { 0x385c4597, SUC_lexicon_tags_38 },
    { 0xa91c4598, SUC_lexicon_tags_103 },
    { 0xa6bc4598, SUC_lexicon_tags_17 },
    { 0x49bc4598, SUC_lexicon_tags_37 },
    { 0xa1dc459a, SUC_lexicon_tags_37 },
    { 0x251c459c, SUC_lexicon_tags_7 },
    { 0xefbc459d, SUC_lexicon_tags_43 },
    { 0x95dc459d, SUC_lexicon_tags_15 },
    { 0x847c459f, SUC_lexicon_tags_2 },
    { 0x475c459a, SUC_lexicon_tags_2 },
    { 0x74bc459a, SUC_lexicon_tags_47 },
    { 0x95bc4597, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413c45a9, SUC_lexicon_tags_3 },
    { 0x567c45aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x337c45ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd59c45ae, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafc45b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7c45b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1c45bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7dc45bf, SUC_lexicon_tags_0 },
    { 0x20dc45c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fc45c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfc45c7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55c45ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x185c45cc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebc45d0, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x271c45d2, SUC_lexicon_tags_45 },
    { 0x4d7c45d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4dc45d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2bc45d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705c45dd, SUC_lexicon_tags_8 },
    { 0xcc3c45dd, SUC_lexicon_tags_37 },
    { 0x553c45df, SUC_lexicon_tags_17 },
    { 0x8a1c45dd, SUC_lexicon_tags_63 },
    { 0x9a7c45df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcbbc45e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fc45e8, SUC_lexicon_tags_25 },
    { 0x3b1c45e9, SUC_lexicon_tags_23 },
    { 0xbfc45e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505c45ef, SUC_lexicon_tags_10 },
    { 0xa31c45f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b7c45f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea9c45f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa21c45f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3c45fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91c45ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3d7c4601, SUC_lexicon_tags_8 },
    { 0x297c4601, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x859c4605, SUC_lexicon_tags_802 },
    { 0, NULL },
    { 0x8cbc4607, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fc460d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105c4611, SUC_lexicon_tags_11 },
    { 0xc59c4611, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85c4616, SUC_lexicon_tags_14 },
    { 0xda3c4617, SUC_lexicon_tags_2 },
    { 0xdbfc4618, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefc461c, SUC_lexicon_tags_27 },
    { 0xd09c461d, SUC_lexicon_tags_61 },
    { 0x8c1c461d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbc4622, SUC_lexicon_tags_45 },
    { 0x153c4623, SUC_lexicon_tags_69 },
    { 0xc99c4624, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dc4628, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bc462d, SUC_lexicon_tags_1 },
    { 0xeb9c462e, SUC_lexicon_tags_4 },
    { 0x51bc462f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3c4632, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fc4635, SUC_lexicon_tags_49 },
    { 0x215c4635, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51c463b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadc4640, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7c4648, SUC_lexicon_tags_1 },
    { 0x8cbc4649, SUC_lexicon_tags_47 },
    { 0x6a1c464a, SUC_lexicon_tags_12 },
    { 0x535c464a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c464e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d1c4650, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae7c4652, SUC_lexicon_tags_34 },
    { 0xfe1c4653, SUC_lexicon_tags_15 },
    { 0x827c4652, SUC_lexicon_tags_14 },
    { 0x6ffc4655, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891c465f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1afc4661, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23c466a, SUC_lexicon_tags_257 },
    { 0, NULL },
    { 0xfd1c466c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x675c466e, SUC_lexicon_tags_0 },
    { 0x565c466f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5c4672, SUC_lexicon_tags_7 },
    { 0xc87c4673, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x71dc4675, SUC_lexicon_tags_11 },
    { 0xd8bc4676, SUC_lexicon_tags_23 },
    { 0x557c4675, SUC_lexicon_tags_17 },
    { 0xf4fc4678, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7c467d, SUC_lexicon_tags_25 },
    { 0xf3dc467e, SUC_lexicon_tags_0 },
    { 0x909c467f, SUC_lexicon_tags_77 },
    { 0x40dc467e, SUC_lexicon_tags_45 },
    { 0x1f7c4681, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741c4685, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81c468a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfc468e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c4691, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159c4695, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9c4699, SUC_lexicon_tags_1 },
    { 0xfd7c469a, SUC_lexicon_tags_15 },
    { 0x895c469b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5a7c469d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c46a0, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fc46a5, SUC_lexicon_tags_30 },
    { 0x19bc46a5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xea1c46a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9c46b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1c46b7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x87fc46b9, SUC_lexicon_tags_10 },
    { 0x189c46ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fc46be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33c46c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63c46c7, SUC_lexicon_tags_33 },
    { 0xf7c46c8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5c46cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9c46cf, SUC_lexicon_tags_63 },
    { 0x111c46cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9c46d3, SUC_lexicon_tags_143 },
    { 0, NULL },
    { 0xbbdc46d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45c46db, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x761c46de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x371c46e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1c46e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c3c46e6, SUC_lexicon_tags_34 },
    { 0x605c46e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x437c46ea, SUC_lexicon_tags_15 },
    { 0x2e1c46eb, SUC_lexicon_tags_25 },
    { 0x5ddc46eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x367c46ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65c46f1, SUC_lexicon_tags_19 },
    { 0xad5c46f2, SUC_lexicon_tags_1 },
    { 0x77fc46f3, SUC_lexicon_tags_15 },
    { 0xdf1c46f2, SUC_lexicon_tags_398 },
    { 0xbd9c46f5, SUC_lexicon_tags_43 },
    { 0xd7bc46f5, SUC_lexicon_tags_19 },
    { 0x431c46f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x359c46f9, SUC_lexicon_tags_49 },
    { 0xab9c46fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35c46fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297c4703, SUC_lexicon_tags_8 },
    { 0x9f5c4703, SUC_lexicon_tags_4 },
    { 0xf5bc4705, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789c470c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7c4714, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551c4719, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b1c471b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891c4722, SUC_lexicon_tags_23 },
    { 0x77dc4723, SUC_lexicon_tags_15 },
    { 0x3a9c4723, SUC_lexicon_tags_72 },
    { 0x381c4724, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x29dc4727, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1dc4729, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19c472c, SUC_lexicon_tags_38 },
    { 0x93bc472c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xfc5c472f, SUC_lexicon_tags_19 },
    { 0x33bc472f, SUC_lexicon_tags_25 },
    { 0xf71c4730, SUC_lexicon_tags_11 },
    { 0x76fc4732, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bc4736, SUC_lexicon_tags_1 },
    { 0x26dc4737, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x25c473a, SUC_lexicon_tags_11 },
    { 0x3c1c473b, SUC_lexicon_tags_65 },
    { 0x5a7c473b, SUC_lexicon_tags_4 },
    { 0xa07c473c, SUC_lexicon_tags_48 },
    { 0xf53c473c, SUC_lexicon_tags_9 },
    { 0xe99c473e, SUC_lexicon_tags_14 },
    { 0xeddc4740, SUC_lexicon_tags_2 },
    { 0xd49c473d, SUC_lexicon_tags_30 },
    { 0x5bc4741, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7bc4744, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33c474d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7c4753, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7c4756, SUC_lexicon_tags_23 },
    { 0x631c4757, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7c475a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9c4760, SUC_lexicon_tags_1 },
    { 0x5d7c4761, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7c4764, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xee1c4766, SUC_lexicon_tags_29 },
    { 0x7d5c4767, SUC_lexicon_tags_1 },
    { 0x7e1c4767, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc476b, SUC_lexicon_tags_14 },
    { 0xe71c476c, SUC_lexicon_tags_37 },
    { 0x3e5c476c, SUC_lexicon_tags_37 },
    { 0x621c476d, SUC_lexicon_tags_15 },
    { 0xe9c476f, SUC_lexicon_tags_7 },
    { 0x243c476d, SUC_lexicon_tags_17 },
    { 0x71bc4771, SUC_lexicon_tags_11 },
    { 0xf15c476c, SUC_lexicon_tags_25 },
    { 0x3fc4773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbfc4775, SUC_lexicon_tags_7 },
    { 0x509c4775, SUC_lexicon_tags_10 },
    { 0x5a1c4777, SUC_lexicon_tags_27 },
    { 0xd53c4776, SUC_lexicon_tags_13 },
    { 0x277c4779, SUC_lexicon_tags_32 },
    { 0x487c4779, SUC_lexicon_tags_81 },
    { 0xfcfc477b, SUC_lexicon_tags_10 },
    { 0xdbdc477b, SUC_lexicon_tags_1 },
    { 0x95dc477d, SUC_lexicon_tags_4 },
    { 0x795c477e, SUC_lexicon_tags_1 },
    { 0x94fc477f, SUC_lexicon_tags_0 },
    { 0xaefc477b, SUC_lexicon_tags_98 },
    { 0x79dc4776, SUC_lexicon_tags_63 },
    { 0xaf9c4777, SUC_lexicon_tags_15 },
    { 0xd4bc4783, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6dfc4785, SUC_lexicon_tags_1 },
    { 0x171c4785, SUC_lexicon_tags_23 },
    { 0x1c9c4787, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981c478d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119c4793, SUC_lexicon_tags_34 },
    { 0xe57c4793, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873c4798, SUC_lexicon_tags_63 },
    { 0x437c4799, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91c479c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x593c479f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99c47a3, SUC_lexicon_tags_68 },
    { 0x14fc47a3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281c47ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x699c47ad, SUC_lexicon_tags_25 },
    { 0x4ffc47ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb19c47b0, SUC_lexicon_tags_1 },
    { 0xdcfc47b1, SUC_lexicon_tags_15 },
    { 0xe91c47b1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xda3c47b4, SUC_lexicon_tags_1 },
    { 0xbf5c47b5, SUC_lexicon_tags_33 },
    { 0xe81c47b6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881c47ba, SUC_lexicon_tags_11 },
    { 0xf79c47bb, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x555c47bd, SUC_lexicon_tags_66 },
    { 0xc51c47bd, SUC_lexicon_tags_14 },
    { 0xc4bc47bf, SUC_lexicon_tags_106 },
    { 0x719c47bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fc47c3, SUC_lexicon_tags_15 },
    { 0xacfc47c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7c47ca, SUC_lexicon_tags_14 },
    { 0xcb5c47cb, SUC_lexicon_tags_2 },
    { 0x411c47cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x361c47cf, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bc47d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3fc47d7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62dc47de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a3c47e0, SUC_lexicon_tags_2 },
    { 0xf11c47e1, SUC_lexicon_tags_32 },
    { 0xcc1c47e2, SUC_lexicon_tags_14 },
    { 0xeb5c47e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761c47e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c1c47ea, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9c47f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebc47fb, SUC_lexicon_tags_37 },
    { 0xd0dc47fb, SUC_lexicon_tags_37 },
    { 0x405c47fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81c4800, SUC_lexicon_tags_17 },
    { 0x7c4801, SUC_lexicon_tags_62 },
    { 0xac3c4802, SUC_lexicon_tags_10 },
    { 0x1cdc4802, SUC_lexicon_tags_25 },
    { 0xccbc4801, SUC_lexicon_tags_1 },
    { 0xeddc4805, SUC_lexicon_tags_7 },
    { 0xb5dc4805, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2adc4808, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa67c480a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x475c480c, SUC_lexicon_tags_37 },
    { 0x14dc480d, SUC_lexicon_tags_37 },
    { 0x36fc480d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc4811, SUC_lexicon_tags_47 },
    { 0x559c4812, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcf7c4814, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x897c4816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9c4819, SUC_lexicon_tags_23 },
    { 0x5c9c4819, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fc4822, SUC_lexicon_tags_37 },
    { 0x667c4823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbc4826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fc4829, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9c4835, SUC_lexicon_tags_46 },
    { 0x513c4836, SUC_lexicon_tags_26 },
    { 0x1a1c4837, SUC_lexicon_tags_0 },
    { 0x821c4836, SUC_lexicon_tags_17 },
    { 0x419c4835, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x103c483c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25c4840, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cdc4842, SUC_lexicon_tags_27 },
    { 0x4fc4843, SUC_lexicon_tags_68 },
    { 0xc6dc4843, SUC_lexicon_tags_63 },
    { 0xc9fc4842, SUC_lexicon_tags_593 },
    { 0x64dc4846, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111c484a, SUC_lexicon_tags_11 },
    { 0x373c484a, SUC_lexicon_tags_19 },
    { 0x7cdc484a, SUC_lexicon_tags_27 },
    { 0xaadc484d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x507c484f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fc4855, SUC_lexicon_tags_56 },
    { 0x69dc4856, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa71c4858, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3e3c485a, SUC_lexicon_tags_5 },
    { 0xdbfc485b, SUC_lexicon_tags_126 },
    { 0x59fc485b, SUC_lexicon_tags_15 },
    { 0x21c485d, SUC_lexicon_tags_11 },
    { 0xe01c485e, SUC_lexicon_tags_26 },
    { 0x3e9c485f, SUC_lexicon_tags_2 },
    { 0xfbbc4860, SUC_lexicon_tags_37 },
    { 0xf05c485d, SUC_lexicon_tags_26 },
    { 0xde1c4862, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951c4870, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55c4875, SUC_lexicon_tags_25 },
    { 0xe27c4875, SUC_lexicon_tags_7 },
    { 0x5ffc4877, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xefbc4879, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bc487d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dc4882, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c4887, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1c488a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bc488d, SUC_lexicon_tags_37 },
    { 0xa41c488e, SUC_lexicon_tags_33 },
    { 0x99fc488f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1c4894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dc4898, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbc48a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d9c48a3, SUC_lexicon_tags_37 },
    { 0x39fc48a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x863c48a6, SUC_lexicon_tags_3 },
    { 0xc51c48a7, SUC_lexicon_tags_25 },
    { 0xbd7c48a8, SUC_lexicon_tags_28 },
    { 0xd19c48a9, SUC_lexicon_tags_1 },
    { 0x4f7c48aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725c48ae, SUC_lexicon_tags_39 },
    { 0xec9c48af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5c48b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x761c48b7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x25fc48b9, SUC_lexicon_tags_3 },
    { 0x1d5c48ba, SUC_lexicon_tags_6 },
    { 0xd5fc48bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbc48bf, SUC_lexicon_tags_63 },
    { 0xfcfc48c0, SUC_lexicon_tags_27 },
    { 0xb47c48c1, SUC_lexicon_tags_37 },
    { 0x27c48c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63c48cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa7c48cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c48d0, SUC_lexicon_tags_12 },
    { 0xffbc48d1, SUC_lexicon_tags_27 },
    { 0xa7fc48d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5c48d5, SUC_lexicon_tags_19 },
    { 0x429c48d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd4bc48d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dc48e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467c48e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdc48ea, SUC_lexicon_tags_7 },
    { 0x621c48ea, SUC_lexicon_tags_9 },
    { 0x87bc48ec, SUC_lexicon_tags_2 },
    { 0xbb1c48eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73c48f0, SUC_lexicon_tags_11 },
    { 0x4c1c48f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c48fb, SUC_lexicon_tags_46 },
    { 0x93bc48fb, SUC_lexicon_tags_61 },
    { 0x4e1c48fc, SUC_lexicon_tags_15 },
    { 0x2c9c48fe, SUC_lexicon_tags_37 },
    { 0x789c48ff, SUC_lexicon_tags_40 },
    { 0xa43c48ff, SUC_lexicon_tags_7 },
    { 0xc51c4901, SUC_lexicon_tags_37 },
    { 0xe4bc4901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81c4908, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfc490b, SUC_lexicon_tags_12 },
    { 0xe07c490b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3bdc490e, SUC_lexicon_tags_27 },
    { 0x9d1c490e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x439c4911, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x613c4914, SUC_lexicon_tags_2 },
    { 0x40fc4914, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359c4919, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c491d, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xbe5c491f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43c4923, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd19c4925, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7c4931, SUC_lexicon_tags_14 },
    { 0x2c3c4932, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x435c4935, SUC_lexicon_tags_32 },
    { 0x155c4936, SUC_lexicon_tags_274 },
    { 0xa69c4936, SUC_lexicon_tags_49 },
    { 0x2e7c4938, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x201c493a, SUC_lexicon_tags_7 },
    { 0xbf5c493b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7c493e, SUC_lexicon_tags_2 },
    { 0x6f1c493f, SUC_lexicon_tags_2 },
    { 0xdb5c493f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fc4946, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1c4949, SUC_lexicon_tags_16 },
    { 0x895c494a, SUC_lexicon_tags_30 },
    { 0xf4bc494a, SUC_lexicon_tags_77 },
    { 0xad1c494a, SUC_lexicon_tags_39 },
    { 0x5a3c494b, SUC_lexicon_tags_43 },
    { 0xcc7c494d, SUC_lexicon_tags_43 },
    { 0xde1c494f, SUC_lexicon_tags_17 },
    { 0x81dc494b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6dfc4952, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbc4959, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1f9c495b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bc4961, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fc4964, SUC_lexicon_tags_87 },
    { 0x343c4965, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fc4968, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x85dc496a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa2dc496c, SUC_lexicon_tags_17 },
    { 0x2ffc496d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x245c4970, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc497a, SUC_lexicon_tags_19 },
    { 0x64dc497a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13c4980, SUC_lexicon_tags_17 },
    { 0xc1c4980, SUC_lexicon_tags_55 },
    { 0xe7dc4980, SUC_lexicon_tags_68 },
    { 0xae5c4982, SUC_lexicon_tags_62 },
    { 0xe39c4981, SUC_lexicon_tags_17 },
    { 0xe4bc4984, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dc4989, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dc498f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdc4992, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x335c4994, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x439c4997, SUC_lexicon_tags_103 },
    { 0x1bdc4998, SUC_lexicon_tags_9 },
    { 0xa01c4998, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3ffc499b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52bc499d, SUC_lexicon_tags_55 },
    { 0x59bc499e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375c49a4, SUC_lexicon_tags_31 },
    { 0x891c49a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x871c49a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7c49b2, SUC_lexicon_tags_60 },
    { 0xb8fc49b3, SUC_lexicon_tags_66 },
    { 0x953c49b4, SUC_lexicon_tags_15 },
    { 0x421c49b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3c49b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc61c49bb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x811c49be, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773c49c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adc49c8, SUC_lexicon_tags_2 },
    { 0xa3bc49c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dc49ce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1c49d1, SUC_lexicon_tags_45 },
    { 0x66fc49d2, SUC_lexicon_tags_15 },
    { 0x537c49d3, SUC_lexicon_tags_22 },
    { 0xbe3c49d4, SUC_lexicon_tags_17 },
    { 0xba1c49d5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x889c49d7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fc49de, SUC_lexicon_tags_28 },
    { 0x8e5c49df, SUC_lexicon_tags_15 },
    { 0xe3bc49e0, SUC_lexicon_tags_37 },
    { 0x48fc49e1, SUC_lexicon_tags_17 },
    { 0xd3dc49e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x307c49e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x547c49e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcfc49e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9c49eb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47c49f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449c49f6, SUC_lexicon_tags_25 },
    { 0x43fc49f7, SUC_lexicon_tags_48 },
    { 0xd09c49f6, SUC_lexicon_tags_10 },
    { 0x5e9c49f9, SUC_lexicon_tags_17 },
    { 0x9edc49fa, SUC_lexicon_tags_82 },
    { 0xc35c49f6, SUC_lexicon_tags_28 },
    { 0xc27c49fc, SUC_lexicon_tags_9 },
    { 0x38fc49fd, SUC_lexicon_tags_1 },
    { 0x31fc49fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8dc4a00, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc4a03, SUC_lexicon_tags_17 },
    { 0xb8bc4a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fc4a0a, SUC_lexicon_tags_19 },
    { 0x965c4a0b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x60bc4a0d, SUC_lexicon_tags_2 },
    { 0xc6bc4a0e, SUC_lexicon_tags_59 },
    { 0x5b1c4a0e, SUC_lexicon_tags_17 },
    { 0x687c4a0e, SUC_lexicon_tags_37 },
    { 0x999c4a0d, SUC_lexicon_tags_17 },
    { 0x39bc4a12, SUC_lexicon_tags_11 },
    { 0xb0dc4a10, SUC_lexicon_tags_66 },
    { 0x823c4a12, SUC_lexicon_tags_17 },
    { 0x76fc4a15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e7c4a17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x769c4a19, SUC_lexicon_tags_47 },
    { 0xe89c4a1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19c4a1d, SUC_lexicon_tags_26 },
    { 0x6b7c4a1e, SUC_lexicon_tags_38 },
    { 0x11bc4a1f, SUC_lexicon_tags_19 },
    { 0xfb5c4a1f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6dc4a2a, SUC_lexicon_tags_15 },
    { 0x215c4a2a, SUC_lexicon_tags_23 },
    { 0x1a3c4a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbc4a30, SUC_lexicon_tags_47 },
    { 0x81bc4a30, SUC_lexicon_tags_37 },
    { 0xe2bc4a30, SUC_lexicon_tags_26 },
    { 0x4a5c4a32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc4a37, SUC_lexicon_tags_26 },
    { 0xa7dc4a38, SUC_lexicon_tags_1 },
    { 0x2bc4a39, SUC_lexicon_tags_37 },
    { 0x639c4a3a, SUC_lexicon_tags_15 },
    { 0x32dc4a3b, SUC_lexicon_tags_12 },
    { 0x6efc4a37, SUC_lexicon_tags_14 },
    { 0x321c4a38, SUC_lexicon_tags_25 },
    { 0xe1fc4a3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc4a41, SUC_lexicon_tags_37 },
    { 0x90dc4a41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817c4a47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xef9c4a4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615c4a4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79c4a55, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943c4a5a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc4a5d, SUC_lexicon_tags_60 },
    { 0x7c1c4a5e, SUC_lexicon_tags_37 },
    { 0x1a7c4a5e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261c4a68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7c4a6d, SUC_lexicon_tags_4 },
    { 0x219c4a6e, SUC_lexicon_tags_8 },
    { 0xcb5c4a6f, SUC_lexicon_tags_43 },
    { 0xb07c4a6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469c4a74, SUC_lexicon_tags_17 },
    { 0x16dc4a74, SUC_lexicon_tags_11 },
    { 0xe8fc4a74, SUC_lexicon_tags_49 },
    { 0xb7c4a77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x785c4a79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfbc4a7b, SUC_lexicon_tags_17 },
    { 0x649c4a7c, SUC_lexicon_tags_15 },
    { 0xe83c4a7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd9c4a7f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7c4a83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bc4a86, SUC_lexicon_tags_17 },
    { 0xb9dc4a86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93c4a8b, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465c4a95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609c4a99, SUC_lexicon_tags_17 },
    { 0x48bc4a99, SUC_lexicon_tags_32 },
    { 0xd7c4a9b, SUC_lexicon_tags_7 },
    { 0x13fc4a9b, SUC_lexicon_tags_0 },
    { 0x2d1c4a9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3c4aa0, SUC_lexicon_tags_38 },
    { 0x127c4aa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x659c4aa3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfb5c4aa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x641c4aa7, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x69c4aaa, SUC_lexicon_tags_40 },
    { 0x955c4aab, SUC_lexicon_tags_4 },
    { 0xecbc4aac, SUC_lexicon_tags_32 },
    { 0xb5fc4aad, SUC_lexicon_tags_9 },
    { 0x389c4aad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7c4ab1, SUC_lexicon_tags_80 },
    { 0xfebc4ab2, SUC_lexicon_tags_40 },
    { 0x73fc4ab3, SUC_lexicon_tags_15 },
    { 0xb2bc4ab4, SUC_lexicon_tags_26 },
    { 0x6f3c4ab5, SUC_lexicon_tags_30 },
    { 0xb05c4ab5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x687c4ab8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe73c4aba, SUC_lexicon_tags_27 },
    { 0x2f5c4aba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc65c4abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bc4ac1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4d9c4ac3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13c4ac8, SUC_lexicon_tags_19 },
    { 0x29c4ac8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x50dc4acb, SUC_lexicon_tags_12 },
    { 0x8e3c4acc, SUC_lexicon_tags_1 },
    { 0xdb7c4acd, SUC_lexicon_tags_17 },
    { 0xb55c4acb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x349c4ad1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bc4ad7, SUC_lexicon_tags_10 },
    { 0xab5c4ad8, SUC_lexicon_tags_12 },
    { 0x6c1c4ad9, SUC_lexicon_tags_30 },
    { 0xd6bc4ad9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcdc4adc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7c4adf, SUC_lexicon_tags_11 },
    { 0xf73c4adf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adc4ae8, SUC_lexicon_tags_11 },
    { 0x88fc4ae9, SUC_lexicon_tags_4 },
    { 0x235c4ae9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x887c4aec, SUC_lexicon_tags_45 },
    { 0xc93c4aec, SUC_lexicon_tags_27 },
    { 0x215c4aee, SUC_lexicon_tags_23 },
    { 0xc8dc4aec, SUC_lexicon_tags_1 },
    { 0xea9c4af0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43c4af3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01c4af6, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679c4afb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1c4afe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc4b08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85c4b0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1c4b12, SUC_lexicon_tags_15 },
    { 0x763c4b13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1c4b16, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7c4b1a, SUC_lexicon_tags_3 },
    { 0x73fc4b1b, SUC_lexicon_tags_37 },
    { 0x533c4b1b, SUC_lexicon_tags_47 },
    { 0x45bc4b1d, SUC_lexicon_tags_2 },
    { 0xab9c4b1c, SUC_lexicon_tags_43 },
    { 0x6d3c4b1f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fc4b24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1c4b27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dc4b2c, SUC_lexicon_tags_11 },
    { 0xf2dc4b2d, SUC_lexicon_tags_1 },
    { 0x141c4b2e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbc4b31, SUC_lexicon_tags_27 },
    { 0xc09c4b31, SUC_lexicon_tags_29 },
    { 0xb97c4b33, SUC_lexicon_tags_37 },
    { 0xa7fc4b34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbc4b3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bc4b42, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1f5c4b44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57c4b47, SUC_lexicon_tags_27 },
    { 0xdefc4b48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69c4b4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3dc4b4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41c4b50, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3c4b53, SUC_lexicon_tags_57 },
    { 0x34fc4b54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17c4b5a, SUC_lexicon_tags_11 },
    { 0x8bc4b5b, SUC_lexicon_tags_31 },
    { 0x317c4b5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5c4b5f, SUC_lexicon_tags_7 },
    { 0x453c4b5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5c4b62, SUC_lexicon_tags_25 },
    { 0x51fc4b62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x63c4b65, SUC_lexicon_tags_11 },
    { 0xed3c4b66, SUC_lexicon_tags_17 },
    { 0x515c4b67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41c4b6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfc4b6d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x94dc4b6f, SUC_lexicon_tags_60 },
    { 0xe03c4b70, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5c4b82, SUC_lexicon_tags_7 },
    { 0x4d9c4b82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x397c4b86, SUC_lexicon_tags_39 },
    { 0x7bc4b87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fc4b97, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fc4b9b, SUC_lexicon_tags_11 },
    { 0xd11c4b9c, SUC_lexicon_tags_25 },
    { 0x5cbc4b9d, SUC_lexicon_tags_15 },
    { 0xcadc4b9e, SUC_lexicon_tags_14 },
    { 0xe91c4b9d, SUC_lexicon_tags_63 },
    { 0x31dc4b9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc4ba3, SUC_lexicon_tags_37 },
    { 0x42fc4ba3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x70bc4ba7, SUC_lexicon_tags_79 },
    { 0xb73c4ba8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49c4bb0, SUC_lexicon_tags_25 },
    { 0x65bc4bb1, SUC_lexicon_tags_47 },
    { 0x70bc4bb2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edc4bb6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x61dc4bb8, SUC_lexicon_tags_1 },
    { 0xecbc4bb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45c4bbc, SUC_lexicon_tags_14 },
    { 0xa13c4bbd, SUC_lexicon_tags_37 },
    { 0x3a9c4bbd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1a3c4bc0, SUC_lexicon_tags_60 },
    { 0x11fc4bc1, SUC_lexicon_tags_6 },
    { 0x121c4bc2, SUC_lexicon_tags_32 },
    { 0x5afc4bc3, SUC_lexicon_tags_77 },
    { 0xaefc4bc4, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bc4bd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75c4bd3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bc4bd6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3c4bda, SUC_lexicon_tags_38 },
    { 0xb65c4bdb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x397c4bdd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5d7c4bdf, SUC_lexicon_tags_19 },
    { 0x57bc4be0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2bdc4be2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde1c4be4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133c4beb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249c4bef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30bc4bf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fc4bf5, SUC_lexicon_tags_25 },
    { 0x5bfc4bf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed3c4bf8, SUC_lexicon_tags_1 },
    { 0x173c4bf9, SUC_lexicon_tags_15 },
    { 0x235c4bfa, SUC_lexicon_tags_16 },
    { 0x7b5c4bfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1c4c07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07c4c0a, SUC_lexicon_tags_1 },
    { 0xfc4c0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905c4c0f, SUC_lexicon_tags_31 },
    { 0x101c4c10, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89c4c13, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07c4c16, SUC_lexicon_tags_11 },
    { 0x9bdc4c16, SUC_lexicon_tags_29 },
    { 0xb4fc4c17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7c4c1f, SUC_lexicon_tags_37 },
    { 0xacbc4c20, SUC_lexicon_tags_64 },
    { 0xfbfc4c20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1c4c29, SUC_lexicon_tags_229 },
    { 0xce5c4c2a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x53c4c2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x485c4c2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9c4c3a, SUC_lexicon_tags_23 },
    { 0xf4bc4c3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x15bc4c3d, SUC_lexicon_tags_26 },
    { 0x721c4c3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fc4c44, SUC_lexicon_tags_55 },
    { 0x2bfc4c44, SUC_lexicon_tags_2 },
    { 0xa9fc4c45, SUC_lexicon_tags_36 },
    { 0x661c4c47, SUC_lexicon_tags_47 },
    { 0x10bc4c47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xec9c4c4a, SUC_lexicon_tags_79 },
    { 0xc6fc4c4b, SUC_lexicon_tags_4 },
    { 0xa1bc4c4c, SUC_lexicon_tags_37 },
    { 0xa25c4c4b, SUC_lexicon_tags_201 },
    { 0x93dc4c4e, SUC_lexicon_tags_25 },
    { 0xfdfc4c4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641c4c54, SUC_lexicon_tags_47 },
    { 0x4f1c4c55, SUC_lexicon_tags_23 },
    { 0x53dc4c56, SUC_lexicon_tags_15 },
    { 0x8ddc4c57, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07c4c5d, SUC_lexicon_tags_14 },
    { 0xcbbc4c5e, SUC_lexicon_tags_25 },
    { 0x3f3c4c5f, SUC_lexicon_tags_64 },
    { 0x96bc4c5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99c4c63, SUC_lexicon_tags_49 },
    { 0xd77c4c64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5c4c67, SUC_lexicon_tags_37 },
    { 0x3b3c4c67, SUC_lexicon_tags_31 },
    { 0x2bbc4c68, SUC_lexicon_tags_3 },
    { 0x373c4c6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x415c4c6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe7dc4c6f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2cfc4c71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68dc4c76, SUC_lexicon_tags_11 },
    { 0x76bc4c76, SUC_lexicon_tags_11 },
    { 0x2b7c4c76, SUC_lexicon_tags_26 },
    { 0x2fdc4c79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc4c7d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7c4c82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525c4c89, SUC_lexicon_tags_7 },
    { 0xd8bc4c8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x655c4c8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbc4c90, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x24fc4c92, SUC_lexicon_tags_19 },
    { 0xd91c4c93, SUC_lexicon_tags_11 },
    { 0xbb9c4c94, SUC_lexicon_tags_14 },
    { 0x6dfc4c94, SUC_lexicon_tags_27 },
    { 0xa79c4c96, SUC_lexicon_tags_40 },
    { 0xfcbc4c94, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x37dc4c99, SUC_lexicon_tags_11 },
    { 0x18fc4c99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc4c9d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381c4ca1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89c4ca5, SUC_lexicon_tags_56 },
    { 0x533c4ca5, SUC_lexicon_tags_49 },
    { 0x3e9c4ca7, SUC_lexicon_tags_2 },
    { 0xdf5c4ca5, SUC_lexicon_tags_17 },
    { 0x663c4ca8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d1c4cab, SUC_lexicon_tags_12 },
    { 0xdffc4cac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bc4cb0, SUC_lexicon_tags_1 },
    { 0x7cdc4cb0, SUC_lexicon_tags_2 },
    { 0x6cdc4cb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x85fc4cb4, SUC_lexicon_tags_11 },
    { 0x3e9c4cb4, SUC_lexicon_tags_77 },
    { 0xe5c4cb4, SUC_lexicon_tags_11 },
    { 0xd47c4cb7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x907c4cba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc4cc1, SUC_lexicon_tags_47 },
    { 0x245c4cc2, SUC_lexicon_tags_38 },
    { 0xedc4cc3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293c4cc9, SUC_lexicon_tags_172 },
    { 0x5f9c4cca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7c4ccf, SUC_lexicon_tags_45 },
    { 0xf7bc4cd0, SUC_lexicon_tags_26 },
    { 0xf6dc4cd1, SUC_lexicon_tags_17 },
    { 0xc95c4cd0, SUC_lexicon_tags_21 },
    { 0x1fdc4cd3, SUC_lexicon_tags_37 },
    { 0x4d1c4cd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x369c4cd7, SUC_lexicon_tags_17 },
    { 0x2b1c4cd8, SUC_lexicon_tags_25 },
    { 0xd1fc4cd9, SUC_lexicon_tags_37 },
    { 0x6e9c4cd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x687c4cdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297c4ce7, SUC_lexicon_tags_21 },
    { 0xb57c4ce7, SUC_lexicon_tags_34 },
    { 0x161c4ce9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc9c4ceb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35c4cf0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991c4cfd, SUC_lexicon_tags_31 },
    { 0xc85c4cfe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77c4d04, SUC_lexicon_tags_32 },
    { 0x457c4d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcedc4d07, SUC_lexicon_tags_13 },
    { 0x2cbc4d08, SUC_lexicon_tags_17 },
    { 0xe81c4d09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dc4d0e, SUC_lexicon_tags_9 },
    { 0xb73c4d0f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3e7c4d11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49c4d13, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477c4d18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83dc4d1a, SUC_lexicon_tags_28 },
    { 0xb81c4d1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dc4d1f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63c4d23, SUC_lexicon_tags_17 },
    { 0xed7c4d24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbc4d27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bc4d2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7adc4d2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff3c4d2f, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x2e3c4d31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9c4d34, SUC_lexicon_tags_14 },
    { 0x6bc4d35, SUC_lexicon_tags_47 },
    { 0xa59c4d36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fc4d39, SUC_lexicon_tags_92 },
    { 0x48bc4d39, SUC_lexicon_tags_29 },
    { 0x6cbc4d3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825c4d3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69c4d43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf75c4d45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x503c4d48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d5c4d4a, SUC_lexicon_tags_32 },
    { 0x26fc4d4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bc4d5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdc4d5d, SUC_lexicon_tags_37 },
    { 0x675c4d5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4abc4d60, SUC_lexicon_tags_8 },
    { 0x5c5c4d60, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19c4d64, SUC_lexicon_tags_1 },
    { 0x7e1c4d64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9c4d6c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdc4d6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7c4d71, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c4d74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc4d78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9c4d80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3c4d82, SUC_lexicon_tags_0 },
    { 0xe29c4d82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x961c4d86, SUC_lexicon_tags_30 },
    { 0x169c4d87, SUC_lexicon_tags_1 },
    { 0xd17c4d88, SUC_lexicon_tags_31 },
    { 0x565c4d89, SUC_lexicon_tags_37 },
    { 0x97dc4d87, SUC_lexicon_tags_17 },
    { 0x921c4d8b, SUC_lexicon_tags_37 },
    { 0xc37c4d87, SUC_lexicon_tags_31 },
    { 0xb5fc4d8d, SUC_lexicon_tags_7 },
    { 0x387c4d8e, SUC_lexicon_tags_46 },
    { 0x831c4d88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5f3c4d91, SUC_lexicon_tags_70 },
    { 0x723c4d91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x903c4d95, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861c4d9a, SUC_lexicon_tags_129 },
    { 0xbfbc4d9b, SUC_lexicon_tags_14 },
    { 0xf41c4d9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd47c4d9e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x66fc4da1, SUC_lexicon_tags_79 },
    { 0xd1fc4da1, SUC_lexicon_tags_121 },
    { 0xb3bc4da3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895c4da8, SUC_lexicon_tags_37 },
    { 0x70bc4da8, SUC_lexicon_tags_8 },
    { 0x28bc4daa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa37c4dac, SUC_lexicon_tags_17 },
    { 0x4e9c4dac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd75c4daf, SUC_lexicon_tags_15 },
    { 0xd19c4db0, SUC_lexicon_tags_11 },
    { 0xc6dc4db1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5c4db5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345c4dba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3c4dbc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7c4dc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc7c4dcb, SUC_lexicon_tags_17 },
    { 0x75dc4dcc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dc4dd0, SUC_lexicon_tags_23 },
    { 0xd2dc4dd1, SUC_lexicon_tags_37 },
    { 0xcafc4dd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97c4dd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x69bc4ddc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397c4de1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17c4de6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9c4dea, SUC_lexicon_tags_12 },
    { 0xd31c4dea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x72dc4dee, SUC_lexicon_tags_7 },
    { 0x807c4def, SUC_lexicon_tags_83 },
    { 0x751c4def, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475c4df9, SUC_lexicon_tags_15 },
    { 0xf7dc4df9, SUC_lexicon_tags_63 },
    { 0xb7c4dfb, SUC_lexicon_tags_4 },
    { 0xc45c4dfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x639c4dfe, SUC_lexicon_tags_31 },
    { 0x8d3c4dff, SUC_lexicon_tags_33 },
    { 0x6e7c4e00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x915c4e02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc4e07, SUC_lexicon_tags_60 },
    { 0xaf1c4e08, SUC_lexicon_tags_6 },
    { 0x461c4e09, SUC_lexicon_tags_1 },
    { 0xc95c4e09, SUC_lexicon_tags_11 },
    { 0x1d3c4e0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc4e14, SUC_lexicon_tags_33 },
    { 0xe5bc4e15, SUC_lexicon_tags_2 },
    { 0x7f3c4e16, SUC_lexicon_tags_37 },
    { 0xe9fc4e17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x72fc4e19, SUC_lexicon_tags_68 },
    { 0x499c4e1a, SUC_lexicon_tags_46 },
    { 0xef7c4e1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9c4e24, SUC_lexicon_tags_47 },
    { 0xe27c4e24, SUC_lexicon_tags_16 },
    { 0x80fc4e24, SUC_lexicon_tags_14 },
    { 0xcbdc4e24, SUC_lexicon_tags_37 },
    { 0x339c4e28, SUC_lexicon_tags_63 },
    { 0x61c4e29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623c4e31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31dc4e35, SUC_lexicon_tags_12 },
    { 0x29dc4e36, SUC_lexicon_tags_38 },
    { 0xef9c4e36, SUC_lexicon_tags_12 },
    { 0xee1c4e38, SUC_lexicon_tags_30 },
    { 0x54dc4e39, SUC_lexicon_tags_29 },
    { 0xf25c4e3a, SUC_lexicon_tags_29 },
    { 0xeafc4e3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb95c4e3d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x109c4e3f, SUC_lexicon_tags_11 },
    { 0xe9c4e40, SUC_lexicon_tags_14 },
    { 0x7d1c4e41, SUC_lexicon_tags_17 },
    { 0xa5c4e41, SUC_lexicon_tags_1 },
    { 0xbebc4e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399c4e48, SUC_lexicon_tags_19 },
    { 0xf5fc4e49, SUC_lexicon_tags_61 },
    { 0xe23c4e49, SUC_lexicon_tags_26 },
    { 0x1bdc4e4b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x343c4e4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47c4e52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bc4e58, SUC_lexicon_tags_2 },
    { 0xe3c4e58, SUC_lexicon_tags_92 },
    { 0x2bdc4e5a, SUC_lexicon_tags_12 },
    { 0x66dc4e5b, SUC_lexicon_tags_17 },
    { 0xd83c4e5b, SUC_lexicon_tags_2 },
    { 0x385c4e5b, SUC_lexicon_tags_8 },
    { 0xfefc4e5e, SUC_lexicon_tags_25 },
    { 0xa61c4e5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697c4e69, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3c4e6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591c4e70, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddc4e75, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29c4e78, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7c4e7e, SUC_lexicon_tags_15 },
    { 0xdd1c4e7f, SUC_lexicon_tags_1 },
    { 0x849c4e80, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x963c4e82, SUC_lexicon_tags_23 },
    { 0x309c4e83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181c4e87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01c4e8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc4e92, SUC_lexicon_tags_4 },
    { 0x56fc4e93, SUC_lexicon_tags_16 },
    { 0x965c4e94, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x893c4e97, SUC_lexicon_tags_43 },
    { 0xb7bc4e98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd1c4e9a, SUC_lexicon_tags_4 },
    { 0x6f7c4e9b, SUC_lexicon_tags_47 },
    { 0xa4fc4e9b, SUC_lexicon_tags_17 },
    { 0xe51c4e9d, SUC_lexicon_tags_114 },
    { 0x66fc4e9e, SUC_lexicon_tags_37 },
    { 0x979c4e9e, SUC_lexicon_tags_87 },
    { 0x4c5c4ea0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc4ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x239c4eaa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93c4ead, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dc4eb6, SUC_lexicon_tags_2 },
    { 0x9c4eb7, SUC_lexicon_tags_43 },
    { 0xf19c4eb8, SUC_lexicon_tags_7 },
    { 0x4f9c4eb7, SUC_lexicon_tags_25 },
    { 0x587c4eb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183c4ebf, SUC_lexicon_tags_9 },
    { 0x3f1c4ec0, SUC_lexicon_tags_9 },
    { 0x4e9c4ec1, SUC_lexicon_tags_39 },
    { 0x1b1c4ec1, SUC_lexicon_tags_87 },
    { 0x32dc4ec2, SUC_lexicon_tags_2 },
    { 0x521c4ec4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4c3c4ec6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33c4ec9, SUC_lexicon_tags_60 },
    { 0x6efc4ec9, SUC_lexicon_tags_1 },
    { 0x91dc4ecb, SUC_lexicon_tags_39 },
    { 0x76fc4eca, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51c4ed0, SUC_lexicon_tags_1 },
    { 0x621c4ed1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913c4ed6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757c4edc, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691c4ee0, SUC_lexicon_tags_10 },
    { 0xaffc4ee1, SUC_lexicon_tags_27 },
    { 0x7d9c4ee1, SUC_lexicon_tags_7 },
    { 0x29bc4ee3, SUC_lexicon_tags_30 },
    { 0xfd1c4ee1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291c4eed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fc4ef1, SUC_lexicon_tags_8 },
    { 0x8d9c4ef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29c4ef7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93c4efc, SUC_lexicon_tags_10 },
    { 0x79bc4efc, SUC_lexicon_tags_65 },
    { 0x97dc4efe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x359c4f00, SUC_lexicon_tags_19 },
    { 0xfb7c4f00, SUC_lexicon_tags_47 },
    { 0xb79c4f02, SUC_lexicon_tags_39 },
    { 0x945c4f01, SUC_lexicon_tags_25 },
    { 0xdfc4f04, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebc4f08, SUC_lexicon_tags_11 },
    { 0x99bc4f09, SUC_lexicon_tags_7 },
    { 0x6ffc4f0a, SUC_lexicon_tags_15 },
    { 0x21fc4f0a, SUC_lexicon_tags_8 },
    { 0x8d7c4f08, SUC_lexicon_tags_63 },
    { 0xab7c4f0d, SUC_lexicon_tags_7 },
    { 0x8c5c4f08, SUC_lexicon_tags_31 },
    { 0xe45c4f0f, SUC_lexicon_tags_37 },
    { 0x47c4f10, SUC_lexicon_tags_68 },
    { 0x1cfc4f11, SUC_lexicon_tags_17 },
    { 0x107c4f09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xabc4f14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3c4f17, SUC_lexicon_tags_34 },
    { 0xd31c4f18, SUC_lexicon_tags_19 },
    { 0xe7bc4f19, SUC_lexicon_tags_2 },
    { 0xdedc4f1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fc4f1d, SUC_lexicon_tags_63 },
    { 0xafdc4f1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2c5c4f20, SUC_lexicon_tags_37 },
    { 0xfdbc4f20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbb3c4f23, SUC_lexicon_tags_19 },
    { 0x76dc4f23, SUC_lexicon_tags_62 },
    { 0xf47c4f25, SUC_lexicon_tags_37 },
    { 0x3a1c4f26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59bc4f28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409c4f2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efc4f36, SUC_lexicon_tags_17 },
    { 0x387c4f37, SUC_lexicon_tags_17 },
    { 0xf9fc4f38, SUC_lexicon_tags_38 },
    { 0x3d9c4f39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x387c4f3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993c4f40, SUC_lexicon_tags_7 },
    { 0xa55c4f41, SUC_lexicon_tags_48 },
    { 0x231c4f42, SUC_lexicon_tags_12 },
    { 0x609c4f43, SUC_lexicon_tags_16 },
    { 0xaf3c4f44, SUC_lexicon_tags_23 },
    { 0xbb7c4f44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dfc4f47, SUC_lexicon_tags_25 },
    { 0xdf9c4f48, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x377c4f4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf51c4f4c, SUC_lexicon_tags_14 },
    { 0x5fbc4f4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69c4f50, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x9dbc4f52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x21bc4f54, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111c4f58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07c4f5e, SUC_lexicon_tags_11 },
    { 0xc9dc4f5f, SUC_lexicon_tags_37 },
    { 0x745c4f60, SUC_lexicon_tags_25 },
    { 0xc9c4f60, SUC_lexicon_tags_56 },
    { 0x671c4f62, SUC_lexicon_tags_63 },
    { 0xbcfc4f5f, SUC_lexicon_tags_39 },
    { 0xe2bc4f61, SUC_lexicon_tags_47 },
    { 0xadbc4f65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdc4f68, SUC_lexicon_tags_17 },
    { 0x707c4f69, SUC_lexicon_tags_9 },
    { 0xe49c4f6a, SUC_lexicon_tags_14 },
    { 0xc21c4f6b, SUC_lexicon_tags_35 },
    { 0xb2bc4f6c, SUC_lexicon_tags_45 },
    { 0xa47c4f6d, SUC_lexicon_tags_45 },
    { 0x92fc4f6b, SUC_lexicon_tags_25 },
    { 0x3c5c4f6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe45c4f72, SUC_lexicon_tags_60 },
    { 0xbbc4f73, SUC_lexicon_tags_12 },
    { 0xde7c4f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x101c4f77, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f3c4f79, SUC_lexicon_tags_65 },
    { 0xd1dc4f79, SUC_lexicon_tags_15 },
    { 0xfcdc4f7b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bc4f82, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6b3c4f84, SUC_lexicon_tags_38 },
    { 0xf35c4f85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfdbc4f87, SUC_lexicon_tags_15 },
    { 0x49fc4f88, SUC_lexicon_tags_7 },
    { 0x603c4f88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671c4f8e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x61c4f90, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3c4f93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9c4f96, SUC_lexicon_tags_25 },
    { 0xf7c4f97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x38dc4f9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31c4f9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x74fc4f9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1c4fa9, SUC_lexicon_tags_2 },
    { 0x6d9c4faa, SUC_lexicon_tags_19 },
    { 0x10fc4fab, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fc4fae, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x351c4fb1, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dc4fb8, SUC_lexicon_tags_83 },
    { 0x581c4fb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8dc4fbb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7c4fbe, SUC_lexicon_tags_12 },
    { 0x875c4fbf, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xacfc4fc1, SUC_lexicon_tags_25 },
    { 0x349c4fc2, SUC_lexicon_tags_21 },
    { 0x2c5c4fc3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893c4fc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffc4fcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dc4fd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dc4fd6, SUC_lexicon_tags_9 },
    { 0xf23c4fd7, SUC_lexicon_tags_21 },
    { 0xb83c4fd8, SUC_lexicon_tags_11 },
    { 0x679c4fd8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbdbc4fdb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2bbc4fdd, SUC_lexicon_tags_14 },
    { 0x24fc4fdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa11c4fe0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dc4fe5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19c4fea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63c4fec, SUC_lexicon_tags_60 },
    { 0x8fdc4fed, SUC_lexicon_tags_82 },
    { 0xc59c4fee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227c4ff5, SUC_lexicon_tags_48 },
    { 0xf33c4ff6, SUC_lexicon_tags_7 },
    { 0x9e1c4ff7, SUC_lexicon_tags_37 },
    { 0x1d3c4ff6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafc4ffc, SUC_lexicon_tags_66 },
    { 0x6efc4ffd, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dc5000, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x289c5002, SUC_lexicon_tags_19 },
    { 0x605c5003, SUC_lexicon_tags_105 },
    { 0xc91c5004, SUC_lexicon_tags_37 },
    { 0xcd9c5003, SUC_lexicon_tags_70 },
    { 0x163c5006, SUC_lexicon_tags_3 },
    { 0x875c5007, SUC_lexicon_tags_0 },
    { 0x27fc5005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bc5015, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fc501c, SUC_lexicon_tags_1 },
    { 0xb43c501d, SUC_lexicon_tags_15 },
    { 0xf07c501e, SUC_lexicon_tags_1 },
    { 0xe1bc501c, SUC_lexicon_tags_44 },
    { 0xba5c501f, SUC_lexicon_tags_7 },
    { 0x1afc5021, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbc502b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba9c502d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x903c502f, SUC_lexicon_tags_15 },
    { 0x7bfc502f, SUC_lexicon_tags_7 },
    { 0x82dc5031, SUC_lexicon_tags_15 },
    { 0x8adc5032, SUC_lexicon_tags_17 },
    { 0xdb5c5030, SUC_lexicon_tags_38 },
    { 0x97dc5032, SUC_lexicon_tags_8 },
    { 0xde3c5035, SUC_lexicon_tags_6 },
    { 0xe69c5035, SUC_lexicon_tags_30 },
    { 0x64fc5035, SUC_lexicon_tags_63 },
    { 0x867c5038, SUC_lexicon_tags_7 },
    { 0x103c5039, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1c503c, SUC_lexicon_tags_1 },
    { 0xa09c503d, SUC_lexicon_tags_0 },
    { 0xc1bc503c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3c5043, SUC_lexicon_tags_24 },
    { 0x6a1c5043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7c5047, SUC_lexicon_tags_45 },
    { 0x89bc5047, SUC_lexicon_tags_17 },
    { 0x383c5049, SUC_lexicon_tags_1 },
    { 0xd17c504a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5c504d, SUC_lexicon_tags_12 },
    { 0x5c3c504e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92dc5051, SUC_lexicon_tags_9 },
    { 0x443c5052, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb3c5054, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7c505d, SUC_lexicon_tags_27 },
    { 0x6f7c505e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x47fc5060, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09c5064, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ddc5066, SUC_lexicon_tags_17 },
    { 0x3fbc5066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133c506c, SUC_lexicon_tags_15 },
    { 0xbadc506d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fc5072, SUC_lexicon_tags_39 },
    { 0x8e5c5073, SUC_lexicon_tags_19 },
    { 0x3b9c5074, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17c5077, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7c507a, SUC_lexicon_tags_69 },
    { 0x6f9c507a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27dc507d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3c5080, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cdc5082, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50fc5085, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fc5089, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x579c508b, SUC_lexicon_tags_10 },
    { 0x3a5c508b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf7c508e, SUC_lexicon_tags_1 },
    { 0xc35c508f, SUC_lexicon_tags_17 },
    { 0xdddc5090, SUC_lexicon_tags_8 },
    { 0x645c5091, SUC_lexicon_tags_11 },
    { 0x585c5092, SUC_lexicon_tags_33 },
    { 0x65bc5093, SUC_lexicon_tags_30 },
    { 0x997c5091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9c509c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315c50a0, SUC_lexicon_tags_31 },
    { 0x21fc50a0, SUC_lexicon_tags_17 },
    { 0xbddc50a2, SUC_lexicon_tags_17 },
    { 0xf6dc50a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05c50a6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9c50ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7c50b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x771c50b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c50bf, SUC_lexicon_tags_2 },
    { 0x729c50c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x289c50c2, SUC_lexicon_tags_17 },
    { 0x577c50c3, SUC_lexicon_tags_33 },
    { 0x9fbc50c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729c50c8, SUC_lexicon_tags_30 },
    { 0x66dc50c9, SUC_lexicon_tags_29 },
    { 0x209c50c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddc50cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x25bc50d1, SUC_lexicon_tags_47 },
    { 0xc2dc50d2, SUC_lexicon_tags_11 },
    { 0x13bc50d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x259c50d6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x589c50d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3c50e0, SUC_lexicon_tags_37 },
    { 0x959c50e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5c50e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x66bc50e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x159c50ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fc50f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bc50f7, SUC_lexicon_tags_8 },
    { 0x5b5c50f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbc50fb, SUC_lexicon_tags_15 },
    { 0xc43c50fc, SUC_lexicon_tags_17 },
    { 0xad9c50fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1e5c50ff, SUC_lexicon_tags_9 },
    { 0xad1c5100, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xae5c5102, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bc5105, SUC_lexicon_tags_43 },
    { 0xd9fc5106, SUC_lexicon_tags_47 },
    { 0x1cbc5107, SUC_lexicon_tags_16 },
    { 0x305c5108, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf0fc510a, SUC_lexicon_tags_1 },
    { 0x505c510a, SUC_lexicon_tags_30 },
    { 0xa6dc510b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x849c510f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1e1c5111, SUC_lexicon_tags_19 },
    { 0x111c5112, SUC_lexicon_tags_7 },
    { 0x1e1c5113, SUC_lexicon_tags_33 },
    { 0x411c5114, SUC_lexicon_tags_23 },
    { 0xda9c5115, SUC_lexicon_tags_44 },
    { 0xb2fc5115, SUC_lexicon_tags_1 },
    { 0x673c5117, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327c511b, SUC_lexicon_tags_32 },
    { 0x51c511c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x141c511f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fc5122, SUC_lexicon_tags_12 },
    { 0x1a9c5123, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2e1c5125, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf7c5127, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf69c5129, SUC_lexicon_tags_9 },
    { 0xc2fc5129, SUC_lexicon_tags_15 },
    { 0xed3c512b, SUC_lexicon_tags_7 },
    { 0x1ddc512c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c5130, SUC_lexicon_tags_108 },
    { 0x1c1c5130, SUC_lexicon_tags_33 },
    { 0x231c5131, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95c5137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73c513c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3c5140, SUC_lexicon_tags_12 },
    { 0x525c5141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x655c5143, SUC_lexicon_tags_19 },
    { 0xde1c5143, SUC_lexicon_tags_23 },
    { 0x743c5144, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fc514b, SUC_lexicon_tags_25 },
    { 0x61c514c, SUC_lexicon_tags_14 },
    { 0xca1c514c, SUC_lexicon_tags_425 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41c5153, SUC_lexicon_tags_47 },
    { 0x1f5c5153, SUC_lexicon_tags_1 },
    { 0xcffc5154, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fc5158, SUC_lexicon_tags_37 },
    { 0xe1c5158, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc515c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bc5160, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1c5166, SUC_lexicon_tags_46 },
    { 0x391c5167, SUC_lexicon_tags_15 },
    { 0x11fc5167, SUC_lexicon_tags_15 },
    { 0xd03c5169, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1c516d, SUC_lexicon_tags_12 },
    { 0x719c516e, SUC_lexicon_tags_2 },
    { 0x8bc516f, SUC_lexicon_tags_13 },
    { 0x4ddc5170, SUC_lexicon_tags_7 },
    { 0xb2fc5170, SUC_lexicon_tags_35 },
    { 0xd57c5170, SUC_lexicon_tags_45 },
    { 0x4abc5173, SUC_lexicon_tags_37 },
    { 0xc33c5174, SUC_lexicon_tags_11 },
    { 0xb51c5175, SUC_lexicon_tags_31 },
    { 0x299c5176, SUC_lexicon_tags_25 },
    { 0x475c5177, SUC_lexicon_tags_27 },
    { 0x38dc5176, SUC_lexicon_tags_2 },
    { 0xe2dc516f, SUC_lexicon_tags_4 },
    { 0xdbdc516d, SUC_lexicon_tags_3 },
    { 0xffdc517b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1c5180, SUC_lexicon_tags_37 },
    { 0x50fc5181, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3c5185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7c518b, SUC_lexicon_tags_74 },
    { 0x63c518b, SUC_lexicon_tags_45 },
    { 0xc03c518c, SUC_lexicon_tags_45 },
    { 0xa15c518e, SUC_lexicon_tags_10 },
    { 0xdf3c518f, SUC_lexicon_tags_0 },
    { 0x4d1c518f, SUC_lexicon_tags_17 },
    { 0x84fc518d, SUC_lexicon_tags_29 },
    { 0xbffc5192, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219c5196, SUC_lexicon_tags_15 },
    { 0x9f9c5197, SUC_lexicon_tags_11 },
    { 0x5b1c5196, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481c519c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81c519f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbc51a6, SUC_lexicon_tags_17 },
    { 0x1b3c51a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09c51aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861c51af, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05c51b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd23c51b5, SUC_lexicon_tags_12 },
    { 0xc69c51b5, SUC_lexicon_tags_37 },
    { 0x363c51b5, SUC_lexicon_tags_15 },
    { 0x22fc51b7, SUC_lexicon_tags_23 },
    { 0x13c51b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3c51bf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91c51c2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7c51c5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7c51cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x327c51cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfc51d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bc51d8, SUC_lexicon_tags_63 },
    { 0x1e9c51d9, SUC_lexicon_tags_33 },
    { 0xa49c51da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665c51de, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fc51e3, SUC_lexicon_tags_15 },
    { 0xb3fc51e4, SUC_lexicon_tags_60 },
    { 0x50fc51e4, SUC_lexicon_tags_23 },
    { 0x88bc51e6, SUC_lexicon_tags_47 },
    { 0x37bc51e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89c51ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b9c51ee, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edc51f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09c51fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x575c51fe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75c5201, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddc5206, SUC_lexicon_tags_17 },
    { 0x75dc5206, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1c520d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadc5212, SUC_lexicon_tags_12 },
    { 0x845c5212, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ffc5215, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1c521e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7c5221, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1d9c5223, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3c522a, SUC_lexicon_tags_14 },
    { 0xa87c522b, SUC_lexicon_tags_38 },
    { 0x40dc522c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983c5233, SUC_lexicon_tags_23 },
    { 0x65bc5234, SUC_lexicon_tags_10 },
    { 0xcb7c5235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07c5238, SUC_lexicon_tags_46 },
    { 0xdefc5239, SUC_lexicon_tags_15 },
    { 0xb3fc5238, SUC_lexicon_tags_15 },
    { 0x151c523a, SUC_lexicon_tags_12 },
    { 0x895c523c, SUC_lexicon_tags_15 },
    { 0x8fc523d, SUC_lexicon_tags_19 },
    { 0x6cdc523e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1b3c5240, SUC_lexicon_tags_47 },
    { 0xf1fc5241, SUC_lexicon_tags_61 },
    { 0x7fdc5240, SUC_lexicon_tags_7 },
    { 0x833c5243, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9c5247, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21c524e, SUC_lexicon_tags_87 },
    { 0x42fc524f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdc5254, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb37c5256, SUC_lexicon_tags_37 },
    { 0x449c5257, SUC_lexicon_tags_25 },
    { 0xf87c5258, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc525b, SUC_lexicon_tags_13 },
    { 0x96fc525c, SUC_lexicon_tags_25 },
    { 0xaebc525d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc7c525f, SUC_lexicon_tags_12 },
    { 0x6e3c5260, SUC_lexicon_tags_26 },
    { 0x265c5260, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbabc5263, SUC_lexicon_tags_1 },
    { 0xa59c5264, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc2fc5266, SUC_lexicon_tags_15 },
    { 0x5a7c5266, SUC_lexicon_tags_26 },
    { 0x461c5268, SUC_lexicon_tags_31 },
    { 0x83bc5269, SUC_lexicon_tags_11 },
    { 0x69c526a, SUC_lexicon_tags_26 },
    { 0x741c5269, SUC_lexicon_tags_16 },
    { 0xb11c526c, SUC_lexicon_tags_44 },
    { 0x435c526c, SUC_lexicon_tags_7 },
    { 0x933c526b, SUC_lexicon_tags_43 },
    { 0x4fdc5269, SUC_lexicon_tags_19 },
    { 0xb1bc526f, SUC_lexicon_tags_7 },
    { 0xd11c526d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x933c5274, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1c5278, SUC_lexicon_tags_38 },
    { 0x281c5278, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c527d, SUC_lexicon_tags_2 },
    { 0x45fc527e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2adc5280, SUC_lexicon_tags_37 },
    { 0x779c5281, SUC_lexicon_tags_85 },
    { 0xcd5c5282, SUC_lexicon_tags_38 },
    { 0xe1fc5283, SUC_lexicon_tags_25 },
    { 0xbdbc5282, SUC_lexicon_tags_9 },
    { 0x22bc5281, SUC_lexicon_tags_27 },
    { 0x651c5284, SUC_lexicon_tags_47 },
    { 0xae3c5283, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733c528c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ddc528e, SUC_lexicon_tags_2 },
    { 0x1e3c528e, SUC_lexicon_tags_15 },
    { 0xda1c5290, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bc5293, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fc5298, SUC_lexicon_tags_214 },
    { 0x229c5299, SUC_lexicon_tags_15 },
    { 0x80bc529a, SUC_lexicon_tags_37 },
    { 0x3e7c529b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7c529e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427c52a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3c52a6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fc52a9, SUC_lexicon_tags_63 },
    { 0x2e7c52a9, SUC_lexicon_tags_28 },
    { 0x7afc52ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f3c52ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x12dc52b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9efc52b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dc52b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7c52ba, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xd1dc52bc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x38fc52bf, SUC_lexicon_tags_11 },
    { 0xf01c52bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91c52c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7c52cb, SUC_lexicon_tags_39 },
    { 0x375c52cc, SUC_lexicon_tags_11 },
    { 0xa3fc52cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529c52d2, SUC_lexicon_tags_26 },
    { 0x847c52d2, SUC_lexicon_tags_60 },
    { 0x9a1c52d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f3c52d6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73c52dc, SUC_lexicon_tags_244 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71c52e2, SUC_lexicon_tags_25 },
    { 0x713c52e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2e7c52e5, SUC_lexicon_tags_1 },
    { 0x89fc52e6, SUC_lexicon_tags_25 },
    { 0xaa9c52e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bc52ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fc52f0, SUC_lexicon_tags_65 },
    { 0x34bc52f1, SUC_lexicon_tags_53 },
    { 0x907c52f2, SUC_lexicon_tags_9 },
    { 0xdfdc52f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765c52f7, SUC_lexicon_tags_29 },
    { 0xd5fc52f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa3c52fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bc52ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x667c5301, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1b1c5303, SUC_lexicon_tags_11 },
    { 0x6c1c5304, SUC_lexicon_tags_2 },
    { 0xe0fc5304, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa6dc5307, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fc530b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbc530f, SUC_lexicon_tags_40 },
    { 0x453c5310, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fc5313, SUC_lexicon_tags_17 },
    { 0xd1dc5314, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5c531a, SUC_lexicon_tags_0 },
    { 0xf6dc531b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dc531e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdc5323, SUC_lexicon_tags_25 },
    { 0x663c5323, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6d7c5326, SUC_lexicon_tags_2 },
    { 0x21c5327, SUC_lexicon_tags_2 },
    { 0x297c5326, SUC_lexicon_tags_2 },
    { 0x623c5329, SUC_lexicon_tags_47 },
    { 0x767c5329, SUC_lexicon_tags_65 },
    { 0x4dfc532b, SUC_lexicon_tags_7 },
    { 0xbf1c532b, SUC_lexicon_tags_23 },
    { 0x9e9c532d, SUC_lexicon_tags_27 },
    { 0x39fc532e, SUC_lexicon_tags_60 },
    { 0xaabc532f, SUC_lexicon_tags_17 },
    { 0x607c532f, SUC_lexicon_tags_84 },
    { 0xd7fc5327, SUC_lexicon_tags_37 },
    { 0x78bc532a, SUC_lexicon_tags_12 },
    { 0xb47c532d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbc5336, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bc533c, SUC_lexicon_tags_8 },
    { 0xe31c533c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbc5343, SUC_lexicon_tags_37 },
    { 0x509c5344, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9c5348, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x845c534a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbc534d, SUC_lexicon_tags_14 },
    { 0xfcfc534d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3b9c5350, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa35c5352, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e3c5354, SUC_lexicon_tags_21 },
    { 0x9afc5354, SUC_lexicon_tags_7 },
    { 0x5c1c5356, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x577c5359, SUC_lexicon_tags_12 },
    { 0x567c5359, SUC_lexicon_tags_47 },
    { 0x49c5359, SUC_lexicon_tags_8 },
    { 0xff1c535c, SUC_lexicon_tags_27 },
    { 0xf57c535d, SUC_lexicon_tags_37 },
    { 0xe55c5359, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dc5363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41c5368, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7c536c, SUC_lexicon_tags_9 },
    { 0x11dc536d, SUC_lexicon_tags_21 },
    { 0x6f3c536e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x433c5370, SUC_lexicon_tags_25 },
    { 0xdadc5370, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x307c5373, SUC_lexicon_tags_46 },
    { 0x1e3c5374, SUC_lexicon_tags_2 },
    { 0xfd3c5374, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc5378, SUC_lexicon_tags_17 },
    { 0xab7c5379, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa85c537b, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1c537e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb27c5380, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc13c5382, SUC_lexicon_tags_14 },
    { 0xddfc5383, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673c5388, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6efc538a, SUC_lexicon_tags_26 },
    { 0x5fbc538b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9c5391, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9c5394, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9c539a, SUC_lexicon_tags_0 },
    { 0xa15c539b, SUC_lexicon_tags_39 },
    { 0x481c539c, SUC_lexicon_tags_25 },
    { 0xde3c539c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bc53a1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x659c53a3, SUC_lexicon_tags_7 },
    { 0xed7c53a4, SUC_lexicon_tags_7 },
    { 0x96fc53a5, SUC_lexicon_tags_2 },
    { 0xe3fc53a3, SUC_lexicon_tags_32 },
    { 0xb17c53a7, SUC_lexicon_tags_13 },
    { 0x549c53a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bc53ac, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bc53b2, SUC_lexicon_tags_7 },
    { 0x70fc53b2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf5dc53b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7c53ba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7c53be, SUC_lexicon_tags_38 },
    { 0x74fc53be, SUC_lexicon_tags_37 },
    { 0xe45c53c0, SUC_lexicon_tags_5 },
    { 0xeb3c53c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5c53c8, SUC_lexicon_tags_95 },
    { 0x2dbc53c9, SUC_lexicon_tags_10 },
    { 0xae1c53c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x487c53cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c53d3, SUC_lexicon_tags_14 },
    { 0xaa7c53d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893c53d8, SUC_lexicon_tags_15 },
    { 0x6d3c53d9, SUC_lexicon_tags_49 },
    { 0x241c53da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3c53dd, SUC_lexicon_tags_7 },
    { 0xe3fc53dd, SUC_lexicon_tags_25 },
    { 0x13fc53df, SUC_lexicon_tags_15 },
    { 0xf65c53e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71c53e4, SUC_lexicon_tags_37 },
    { 0x2a9c53e5, SUC_lexicon_tags_35 },
    { 0xc4bc53e6, SUC_lexicon_tags_68 },
    { 0xc2fc53e6, SUC_lexicon_tags_30 },
    { 0x1c3c53e8, SUC_lexicon_tags_37 },
    { 0x76fc53e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbc53ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efc53ef, SUC_lexicon_tags_14 },
    { 0x3fdc53ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fdc53f2, SUC_lexicon_tags_7 },
    { 0xf83c53f3, SUC_lexicon_tags_26 },
    { 0x469c53f4, SUC_lexicon_tags_19 },
    { 0x77dc53f5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x61fc53f7, SUC_lexicon_tags_26 },
    { 0x8e7c53f7, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63c53fb, SUC_lexicon_tags_29 },
    { 0x1d3c53fb, SUC_lexicon_tags_2 },
    { 0x4f5c53fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x585c5400, SUC_lexicon_tags_1 },
    { 0xd95c5401, SUC_lexicon_tags_25 },
    { 0xfd9c5401, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6d3c5404, SUC_lexicon_tags_2 },
    { 0xe1dc5405, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27dc5409, SUC_lexicon_tags_47 },
    { 0x923c5409, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeefc540c, SUC_lexicon_tags_43 },
    { 0x76fc540d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbc5411, SUC_lexicon_tags_7 },
    { 0xd59c5411, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x72fc5415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bc541a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13c5420, SUC_lexicon_tags_47 },
    { 0xf9bc5421, SUC_lexicon_tags_25 },
    { 0x5b3c5422, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5c5428, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x433c542b, SUC_lexicon_tags_13 },
    { 0x95bc542c, SUC_lexicon_tags_6 },
    { 0x33bc542b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94dc542f, SUC_lexicon_tags_25 },
    { 0xd05c5430, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd87c5432, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5c5435, SUC_lexicon_tags_63 },
    { 0x78bc5435, SUC_lexicon_tags_53 },
    { 0xe0fc5437, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd77c5439, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77c5440, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dc5445, SUC_lexicon_tags_17 },
    { 0x26fc5446, SUC_lexicon_tags_8 },
    { 0x111c5446, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45dc544e, SUC_lexicon_tags_49 },
    { 0x609c544f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee9c5451, SUC_lexicon_tags_7 },
    { 0xbb1c5451, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x725c5454, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99c5461, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x19fc5463, SUC_lexicon_tags_14 },
    { 0xd2dc5464, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efc5467, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x729c546a, SUC_lexicon_tags_11 },
    { 0x321c546a, SUC_lexicon_tags_7 },
    { 0x203c546c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc33c546e, SUC_lexicon_tags_27 },
    { 0xecbc546f, SUC_lexicon_tags_105 },
    { 0xe9dc5470, SUC_lexicon_tags_39 },
    { 0x55c5471, SUC_lexicon_tags_82 },
    { 0x5b3c546e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9ddc5474, SUC_lexicon_tags_27 },
    { 0xaa1c5475, SUC_lexicon_tags_63 },
    { 0xae7c5476, SUC_lexicon_tags_11 },
    { 0x515c5474, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1c547e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xadc5481, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x409c5484, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea1c5486, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x315c5489, SUC_lexicon_tags_12 },
    { 0x94bc548a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x303c548c, SUC_lexicon_tags_25 },
    { 0x4afc548d, SUC_lexicon_tags_63 },
    { 0x87fc548e, SUC_lexicon_tags_37 },
    { 0x4f3c548e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x981c5491, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x73dc5494, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffc5497, SUC_lexicon_tags_7 },
    { 0xd37c5497, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x78dc549b, SUC_lexicon_tags_48 },
    { 0x153c549c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc9c549e, SUC_lexicon_tags_23 },
    { 0xa3c549f, SUC_lexicon_tags_0 },
    { 0x18bc54a0, SUC_lexicon_tags_47 },
    { 0x333c54a0, SUC_lexicon_tags_17 },
    { 0x893c549f, SUC_lexicon_tags_7 },
    { 0x42bc54a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x831c54a5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9bdc54a7, SUC_lexicon_tags_4 },
    { 0x959c54a7, SUC_lexicon_tags_37 },
    { 0x5abc54a8, SUC_lexicon_tags_17 },
    { 0x4cfc54a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05c54b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8edc54b4, SUC_lexicon_tags_17 },
    { 0xca9c54b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdc54bc, SUC_lexicon_tags_37 },
    { 0x541c54bc, SUC_lexicon_tags_87 },
    { 0xd59c54be, SUC_lexicon_tags_34 },
    { 0xb5c54bd, SUC_lexicon_tags_7 },
    { 0xe55c54bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e5c54c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5c54cb, SUC_lexicon_tags_117 },
    { 0x5c7c54cc, SUC_lexicon_tags_49 },
    { 0x7c3c54cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb43c54cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3c54d4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241c54da, SUC_lexicon_tags_17 },
    { 0xe6fc54db, SUC_lexicon_tags_27 },
    { 0x65dc54dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5bc54de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcfdc54e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xed3c54e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c54e5, SUC_lexicon_tags_2 },
    { 0xf5c54e6, SUC_lexicon_tags_11 },
    { 0x3ddc54e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27c54f1, SUC_lexicon_tags_45 },
    { 0x3b1c54f2, SUC_lexicon_tags_47 },
    { 0xe9dc54f2, SUC_lexicon_tags_91 },
    { 0xeb1c54f4, SUC_lexicon_tags_16 },
    { 0xe71c54f5, SUC_lexicon_tags_47 },
    { 0xe6fc54f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc54fc, SUC_lexicon_tags_2 },
    { 0xaa5c54fd, SUC_lexicon_tags_30 },
    { 0xb5c54fd, SUC_lexicon_tags_47 },
    { 0x851c54ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3c5504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1c5507, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb93c5509, SUC_lexicon_tags_25 },
    { 0x4fc550a, SUC_lexicon_tags_25 },
    { 0x6e9c550a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x147c550e, SUC_lexicon_tags_37 },
    { 0xc0fc550e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x361c5511, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xddbc5513, SUC_lexicon_tags_2 },
    { 0x7fbc5514, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05c5518, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc7c551a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x175c551d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d3c551f, SUC_lexicon_tags_47 },
    { 0xa65c5520, SUC_lexicon_tags_85 },
    { 0xe21c5521, SUC_lexicon_tags_17 },
    { 0xf63c5522, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43c5525, SUC_lexicon_tags_33 },
    { 0x1edc5525, SUC_lexicon_tags_15 },
    { 0xe25c5527, SUC_lexicon_tags_14 },
    { 0xba1c5528, SUC_lexicon_tags_65 },
    { 0xa6dc5529, SUC_lexicon_tags_1 },
    { 0x559c552a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb37c552c, SUC_lexicon_tags_29 },
    { 0xc37c552d, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xbfc552f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43c5531, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529c5538, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdc553b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01c5547, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145c554d, SUC_lexicon_tags_9 },
    { 0x227c554e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ebc5550, SUC_lexicon_tags_25 },
    { 0x393c5551, SUC_lexicon_tags_3 },
    { 0x447c5550, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x3a1c5554, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abc5559, SUC_lexicon_tags_47 },
    { 0x6f9c5559, SUC_lexicon_tags_2 },
    { 0xbe7c555b, SUC_lexicon_tags_56 },
    { 0xc51c555c, SUC_lexicon_tags_12 },
    { 0xd41c555d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe4dc555f, SUC_lexicon_tags_47 },
    { 0xe55c5560, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87c5566, SUC_lexicon_tags_44 },
    { 0x80fc5567, SUC_lexicon_tags_5 },
    { 0x3fbc5568, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x643c556a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dc556e, SUC_lexicon_tags_28 },
    { 0xf1fc556e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547c5574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411c5579, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5c557e, SUC_lexicon_tags_15 },
    { 0x4cdc557f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x697c5581, SUC_lexicon_tags_15 },
    { 0xdbbc5582, SUC_lexicon_tags_25 },
    { 0x89dc5583, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bc5589, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7c5590, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dc5595, SUC_lexicon_tags_16 },
    { 0xba9c5596, SUC_lexicon_tags_12 },
    { 0x833c5597, SUC_lexicon_tags_12 },
    { 0x387c5597, SUC_lexicon_tags_37 },
    { 0xd8fc5597, SUC_lexicon_tags_56 },
    { 0x7a1c5595, SUC_lexicon_tags_213 },
    { 0x973c5597, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50dc559d, SUC_lexicon_tags_47 },
    { 0x99c559e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7c55a1, SUC_lexicon_tags_2 },
    { 0x499c55a2, SUC_lexicon_tags_11 },
    { 0x9fbc55a3, SUC_lexicon_tags_21 },
    { 0x6f9c55a1, SUC_lexicon_tags_15 },
    { 0xbbc55a1, SUC_lexicon_tags_15 },
    { 0x933c55a2, SUC_lexicon_tags_12 },
    { 0x189c55a7, SUC_lexicon_tags_27 },
    { 0x777c55a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88dc55ad, SUC_lexicon_tags_47 },
    { 0x3cbc55ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c55b2, SUC_lexicon_tags_11 },
    { 0x953c55b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x769c55b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c55b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7c55bf, SUC_lexicon_tags_87 },
    { 0x62bc55c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933c55c9, SUC_lexicon_tags_100 },
    { 0xdfc55ca, SUC_lexicon_tags_31 },
    { 0x4b5c55cb, SUC_lexicon_tags_60 },
    { 0x59dc55cb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x39fc55ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5d3c55d0, SUC_lexicon_tags_17 },
    { 0xa73c55d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdc55d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x649c55d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb5c55d8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfc55dd, SUC_lexicon_tags_6 },
    { 0x1f7c55de, SUC_lexicon_tags_43 },
    { 0xf5c55df, SUC_lexicon_tags_11 },
    { 0x757c55e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31c55e3, SUC_lexicon_tags_55 },
    { 0xb53c55e3, SUC_lexicon_tags_27 },
    { 0xaf7c55e5, SUC_lexicon_tags_25 },
    { 0x32bc55e4, SUC_lexicon_tags_33 },
    { 0x403c55e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfc55ea, SUC_lexicon_tags_37 },
    { 0x1f5c55eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bc55f3, SUC_lexicon_tags_38 },
    { 0x205c55f4, SUC_lexicon_tags_15 },
    { 0x541c55f5, SUC_lexicon_tags_4 },
    { 0x563c55f6, SUC_lexicon_tags_29 },
    { 0x1bdc55f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bc55fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x617c55fe, SUC_lexicon_tags_2 },
    { 0x46dc55ff, SUC_lexicon_tags_43 },
    { 0x3bdc5600, SUC_lexicon_tags_10 },
    { 0x93dc5601, SUC_lexicon_tags_25 },
    { 0x675c5601, SUC_lexicon_tags_49 },
    { 0x3f9c5601, SUC_lexicon_tags_15 },
    { 0xb9c5604, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3c5608, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x66bc560a, SUC_lexicon_tags_3 },
    { 0x105c560a, SUC_lexicon_tags_23 },
    { 0xc47c560c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdc560f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffc5614, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x371c5616, SUC_lexicon_tags_27 },
    { 0x3f1c5617, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9a9c5619, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bc561c, SUC_lexicon_tags_62 },
    { 0x8b5c561d, SUC_lexicon_tags_15 },
    { 0xc4bc561c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85c5621, SUC_lexicon_tags_23 },
    { 0x1e1c5622, SUC_lexicon_tags_23 },
    { 0x441c5623, SUC_lexicon_tags_11 },
    { 0xf31c5623, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44dc562a, SUC_lexicon_tags_68 },
    { 0xe15c562b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc5632, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817c5637, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07c563a, SUC_lexicon_tags_15 },
    { 0xa97c563b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793c5640, SUC_lexicon_tags_23 },
    { 0x641c5641, SUC_lexicon_tags_3 },
    { 0xb7fc5640, SUC_lexicon_tags_37 },
    { 0xb6dc5640, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9c5648, SUC_lexicon_tags_17 },
    { 0x5e1c5649, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25dc5652, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcf3c5654, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45c5658, SUC_lexicon_tags_43 },
    { 0xaa1c5658, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x74bc565b, SUC_lexicon_tags_37 },
    { 0x62dc565b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x759c565e, SUC_lexicon_tags_12 },
    { 0x52bc565f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fc5662, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5c5665, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdc566b, SUC_lexicon_tags_84 },
    { 0x925c566c, SUC_lexicon_tags_29 },
    { 0xaa5c566d, SUC_lexicon_tags_80 },
    { 0x91fc566b, SUC_lexicon_tags_1 },
    { 0xf2dc566f, SUC_lexicon_tags_55 },
    { 0x809c566f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x235c5673, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x133c5675, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xebc5678, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x869c567b, SUC_lexicon_tags_31 },
    { 0xaf3c567c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcb5c567e, SUC_lexicon_tags_14 },
    { 0x1dbc567e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3fc5681, SUC_lexicon_tags_25 },
    { 0xf4fc5682, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7c5685, SUC_lexicon_tags_113 },
    { 0x1e1c5685, SUC_lexicon_tags_27 },
    { 0x6c1c5687, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdc568e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d1c5690, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fc5693, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17c5696, SUC_lexicon_tags_63 },
    { 0xd5dc5696, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565c569c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485c56a8, SUC_lexicon_tags_16 },
    { 0xddfc56a8, SUC_lexicon_tags_25 },
    { 0x80dc56a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dc56ae, SUC_lexicon_tags_45 },
    { 0x199c56af, SUC_lexicon_tags_37 },
    { 0xe09c56af, SUC_lexicon_tags_60 },
    { 0x353c56b1, SUC_lexicon_tags_33 },
    { 0xd5fc56b2, SUC_lexicon_tags_17 },
    { 0x2b5c56b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dc56b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15c56c0, SUC_lexicon_tags_10 },
    { 0x933c56c1, SUC_lexicon_tags_2 },
    { 0x2bc56c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x899c56c5, SUC_lexicon_tags_15 },
    { 0x299c56c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbe9c56c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd9c56ca, SUC_lexicon_tags_58 },
    { 0x785c56cb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505c56d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bc56d3, SUC_lexicon_tags_16 },
    { 0xbe1c56d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x605c56d7, SUC_lexicon_tags_55 },
    { 0x65bc56d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5c56db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x87c56de, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777c56e2, SUC_lexicon_tags_48 },
    { 0x49bc56e3, SUC_lexicon_tags_8 },
    { 0x5c3c56e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43c56ea, SUC_lexicon_tags_2 },
    { 0xba3c56eb, SUC_lexicon_tags_28 },
    { 0xc7dc56ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc56ef, SUC_lexicon_tags_37 },
    { 0x7dc56f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x615c56f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13c56f6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961c5700, SUC_lexicon_tags_25 },
    { 0x317c5700, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fc5704, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44dc5706, SUC_lexicon_tags_12 },
    { 0xa9c5706, SUC_lexicon_tags_25 },
    { 0x35c5708, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03c570c, SUC_lexicon_tags_1101 },
    { 0, NULL },
    { 0x2dfc570e, SUC_lexicon_tags_16 },
    { 0x441c570f, SUC_lexicon_tags_38 },
    { 0xf67c570f, SUC_lexicon_tags_45 },
    { 0xa41c5711, SUC_lexicon_tags_4 },
    { 0x4fc5712, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41c5716, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1c571a, SUC_lexicon_tags_11 },
    { 0x9c571b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3c571e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebc5721, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc5724, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x95bc5727, SUC_lexicon_tags_11 },
    { 0x117c5728, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x47c572b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4bbc572d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73dc5730, SUC_lexicon_tags_0 },
    { 0xa91c5730, SUC_lexicon_tags_19 },
    { 0x49c5731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x867c5735, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fc5739, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x93c573b, SUC_lexicon_tags_2 },
    { 0x115c573c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97c5740, SUC_lexicon_tags_10 },
    { 0xe4dc5741, SUC_lexicon_tags_12 },
    { 0x88bc5741, SUC_lexicon_tags_7 },
    { 0xb77c5741, SUC_lexicon_tags_12 },
    { 0xb3c5742, SUC_lexicon_tags_63 },
    { 0xdb3c5742, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3c574a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x651c574c, SUC_lexicon_tags_25 },
    { 0x59fc574d, SUC_lexicon_tags_11 },
    { 0x5d3c574c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fc5755, SUC_lexicon_tags_14 },
    { 0x621c5756, SUC_lexicon_tags_12 },
    { 0x575c5755, SUC_lexicon_tags_27 },
    { 0xeb3c5757, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x457c575a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x653c575d, SUC_lexicon_tags_12 },
    { 0x597c575d, SUC_lexicon_tags_127 },
    { 0xcc9c575f, SUC_lexicon_tags_0 },
    { 0x70dc575d, SUC_lexicon_tags_35 },
    { 0x6fdc5761, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3c5764, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x583c5766, SUC_lexicon_tags_27 },
    { 0xb83c5766, SUC_lexicon_tags_63 },
    { 0x3c3c5768, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1c576d, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9c5771, SUC_lexicon_tags_4 },
    { 0xb7bc5772, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987c577b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1c577e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfc5784, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93c578a, SUC_lexicon_tags_27 },
    { 0xa63c578a, SUC_lexicon_tags_17 },
    { 0xac1c578c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9c5793, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xabbc5795, SUC_lexicon_tags_33 },
    { 0xbe9c5796, SUC_lexicon_tags_0 },
    { 0xb3dc5795, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7c579c, SUC_lexicon_tags_37 },
    { 0xf5c579d, SUC_lexicon_tags_34 },
    { 0xe9c579c, SUC_lexicon_tags_14 },
    { 0xef5c579d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bc57a2, SUC_lexicon_tags_47 },
    { 0xa8dc57a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x41fc57a6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x989c57a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdc57ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837c57b0, SUC_lexicon_tags_11 },
    { 0x621c57b1, SUC_lexicon_tags_6 },
    { 0xeb1c57b1, SUC_lexicon_tags_2 },
    { 0x50fc57b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3c57b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51dc57ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65c57bf, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xb5dc57c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d7c57c3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffc57c8, SUC_lexicon_tags_2 },
    { 0x3e9c57c9, SUC_lexicon_tags_25 },
    { 0xa97c57ca, SUC_lexicon_tags_136 },
    { 0xf15c57cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3cfc57cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9dc57cf, SUC_lexicon_tags_33 },
    { 0x65c57d0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169c57d4, SUC_lexicon_tags_12 },
    { 0x5c7c57d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x507c57d7, SUC_lexicon_tags_6 },
    { 0xd69c57d8, SUC_lexicon_tags_25 },
    { 0xf75c57d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5c57df, SUC_lexicon_tags_47 },
    { 0x4ddc57e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efc57e7, SUC_lexicon_tags_7 },
    { 0x93bc57e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5c57ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699c57f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x567c57f5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x63c57f8, SUC_lexicon_tags_32 },
    { 0x267c57f8, SUC_lexicon_tags_24 },
    { 0xf5c57fa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bc5800, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101c5804, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c5808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159c580c, SUC_lexicon_tags_8 },
    { 0x5bc580d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7c5810, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7c5814, SUC_lexicon_tags_25 },
    { 0x309c5815, SUC_lexicon_tags_12 },
    { 0x56fc5814, SUC_lexicon_tags_37 },
    { 0xcdfc5815, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5c581c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b7c581e, SUC_lexicon_tags_15 },
    { 0xbf7c581e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fc5826, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943c582b, SUC_lexicon_tags_17 },
    { 0x1afc582b, SUC_lexicon_tags_11 },
    { 0x253c582d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa9c582f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3f1c5831, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc5835, SUC_lexicon_tags_7 },
    { 0x52bc5835, SUC_lexicon_tags_221 },
    { 0x5bbc5836, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7c583b, SUC_lexicon_tags_32 },
    { 0x4e3c583c, SUC_lexicon_tags_33 },
    { 0x117c583b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93fc583f, SUC_lexicon_tags_92 },
    { 0x7e3c5840, SUC_lexicon_tags_424 },
    { 0x1ddc5841, SUC_lexicon_tags_33 },
    { 0x41bc5842, SUC_lexicon_tags_1 },
    { 0x657c5843, SUC_lexicon_tags_15 },
    { 0x837c5840, SUC_lexicon_tags_17 },
    { 0x75fc5845, SUC_lexicon_tags_47 },
    { 0x1d5c5846, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1c584d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed9c584f, SUC_lexicon_tags_38 },
    { 0x2b7c5850, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee9c5852, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1c5856, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfc5859, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7c585d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45fc5860, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x37bc5862, SUC_lexicon_tags_27 },
    { 0x669c5863, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3c5866, SUC_lexicon_tags_11 },
    { 0xd8fc5867, SUC_lexicon_tags_63 },
    { 0x9f5c5866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1c5872, SUC_lexicon_tags_25 },
    { 0x52dc5872, SUC_lexicon_tags_12 },
    { 0x7a9c5874, SUC_lexicon_tags_25 },
    { 0x3cfc5875, SUC_lexicon_tags_19 },
    { 0x29bc5874, SUC_lexicon_tags_37 },
    { 0x26dc5877, SUC_lexicon_tags_17 },
    { 0x687c5878, SUC_lexicon_tags_47 },
    { 0x813c5877, SUC_lexicon_tags_27 },
    { 0xefdc5878, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5c5880, SUC_lexicon_tags_14 },
    { 0x54dc5881, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9c5885, SUC_lexicon_tags_0 },
    { 0xa89c5886, SUC_lexicon_tags_33 },
    { 0x189c5886, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669c588e, SUC_lexicon_tags_1 },
    { 0x759c588f, SUC_lexicon_tags_12 },
    { 0xa39c5890, SUC_lexicon_tags_2 },
    { 0xfefc5891, SUC_lexicon_tags_27 },
    { 0xd23c5892, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3fdc5894, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19c5897, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fc58a0, SUC_lexicon_tags_38 },
    { 0x8f5c58a1, SUC_lexicon_tags_15 },
    { 0x5e5c58a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1c58a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bc58a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fc58ad, SUC_lexicon_tags_60 },
    { 0x199c58ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d3c58b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fc58b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9b1c58b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x791c58b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffc58bb, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfc58be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x731c58c1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x887c58c3, SUC_lexicon_tags_87 },
    { 0x645c58c4, SUC_lexicon_tags_46 },
    { 0xcc1c58c4, SUC_lexicon_tags_10 },
    { 0x79fc58c3, SUC_lexicon_tags_47 },
    { 0xbdc58c3, SUC_lexicon_tags_11 },
    { 0x2a3c58c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17c58cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x585c58ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163c58d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717c58d8, SUC_lexicon_tags_12 },
    { 0x8ebc58d8, SUC_lexicon_tags_12 },
    { 0xff3c58da, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827c58df, SUC_lexicon_tags_11 },
    { 0x843c58df, SUC_lexicon_tags_66 },
    { 0xe9c58df, SUC_lexicon_tags_37 },
    { 0x1b9c58e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x217c58e4, SUC_lexicon_tags_9 },
    { 0x413c58e4, SUC_lexicon_tags_8 },
    { 0xc15c58e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bc58ea, SUC_lexicon_tags_2 },
    { 0x8bbc58eb, SUC_lexicon_tags_8 },
    { 0xce9c58eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61c58f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349c58f5, SUC_lexicon_tags_17 },
    { 0x891c58f6, SUC_lexicon_tags_12 },
    { 0xd7fc58f7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95c58fb, SUC_lexicon_tags_471 },
    { 0, NULL },
    { 0xf93c58fd, SUC_lexicon_tags_29 },
    { 0x43fc58fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa49c5900, SUC_lexicon_tags_48 },
    { 0xd8fc5900, SUC_lexicon_tags_37 },
    { 0xe41c5902, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9c5906, SUC_lexicon_tags_0 },
    { 0x299c5907, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xedbc5909, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1c590c, SUC_lexicon_tags_49 },
    { 0x3bdc590d, SUC_lexicon_tags_9 },
    { 0xe65c590c, SUC_lexicon_tags_15 },
    { 0xba7c590f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adc5913, SUC_lexicon_tags_29 },
    { 0xb53c5913, SUC_lexicon_tags_31 },
    { 0x275c5914, SUC_lexicon_tags_25 },
    { 0x175c5915, SUC_lexicon_tags_2 },
    { 0xf4fc5917, SUC_lexicon_tags_174 },
    { 0xbb3c5918, SUC_lexicon_tags_14 },
    { 0x8d7c5918, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xb93c591b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdebc591d, SUC_lexicon_tags_24 },
    { 0x659c591e, SUC_lexicon_tags_43 },
    { 0x9b5c591d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97bc5921, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf81c5923, SUC_lexicon_tags_21 },
    { 0xd47c5923, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5c5932, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb11c5934, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7c5936, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3c5939, SUC_lexicon_tags_25 },
    { 0x31fc5939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x305c593d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf93c593f, SUC_lexicon_tags_37 },
    { 0x191c5940, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0x71c5942, SUC_lexicon_tags_2 },
    { 0xeafc5943, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631c5947, SUC_lexicon_tags_23 },
    { 0x917c5947, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dc594b, SUC_lexicon_tags_12 },
    { 0x4d9c594b, SUC_lexicon_tags_15 },
    { 0xadbc594d, SUC_lexicon_tags_15 },
    { 0x601c594e, SUC_lexicon_tags_66 },
    { 0x87c594e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439c5954, SUC_lexicon_tags_47 },
    { 0xe7fc5955, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fc5959, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdc5960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bc5965, SUC_lexicon_tags_2 },
    { 0x69bc5965, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x82fc5968, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedc596c, SUC_lexicon_tags_1 },
    { 0x4edc596d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129c5973, SUC_lexicon_tags_14 },
    { 0xfb5c5974, SUC_lexicon_tags_25 },
    { 0x659c5975, SUC_lexicon_tags_38 },
    { 0x28bc5975, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc95c5978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3c597d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bc5981, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbf3c5983, SUC_lexicon_tags_6 },
    { 0x77c5983, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbc5987, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541c598b, SUC_lexicon_tags_0 },
    { 0xe4bc598c, SUC_lexicon_tags_12 },
    { 0xde5c598c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdc5992, SUC_lexicon_tags_11 },
    { 0x783c5993, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dc5999, SUC_lexicon_tags_0 },
    { 0x583c5999, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3c59a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225c59aa, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5c59ae, SUC_lexicon_tags_17 },
    { 0x727c59af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x61c59b2, SUC_lexicon_tags_12 },
    { 0xcfdc59b3, SUC_lexicon_tags_23 },
    { 0x16fc59b4, SUC_lexicon_tags_12 },
    { 0x8adc59b5, SUC_lexicon_tags_17 },
    { 0x70fc59b3, SUC_lexicon_tags_61 },
    { 0xf93c59b3, SUC_lexicon_tags_11 },
    { 0xfefc59b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x97bc59ba, SUC_lexicon_tags_44 },
    { 0xf8bc59bb, SUC_lexicon_tags_14 },
    { 0x471c59bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafc59c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2fc59c2, SUC_lexicon_tags_4 },
    { 0xbedc59c2, SUC_lexicon_tags_85 },
    { 0x14fc59c4, SUC_lexicon_tags_29 },
    { 0x63c59c3, SUC_lexicon_tags_7 },
    { 0xcbc59c5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa0bc59c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111c59cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64dc59d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbc59d5, SUC_lexicon_tags_45 },
    { 0xfebc59d6, SUC_lexicon_tags_15 },
    { 0x68dc59d7, SUC_lexicon_tags_27 },
    { 0x90fc59d8, SUC_lexicon_tags_11 },
    { 0xb95c59d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc5fc59db, SUC_lexicon_tags_2 },
    { 0xab1c59dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfc59e2, SUC_lexicon_tags_79 },
    { 0x97c59e3, SUC_lexicon_tags_25 },
    { 0x351c59e4, SUC_lexicon_tags_17 },
    { 0xc9fc59e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267c59eb, SUC_lexicon_tags_15 },
    { 0x54fc59ec, SUC_lexicon_tags_23 },
    { 0x9f1c59ec, SUC_lexicon_tags_25 },
    { 0x823c59ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49c59f1, SUC_lexicon_tags_16 },
    { 0xe6bc59f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39c59f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7c59fd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc5a00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391c5a04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x325c5a06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fc5a0a, SUC_lexicon_tags_31 },
    { 0xf35c5a0b, SUC_lexicon_tags_21 },
    { 0xe5fc5a0c, SUC_lexicon_tags_69 },
    { 0xfc3c5a0c, SUC_lexicon_tags_25 },
    { 0xfffc5a0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30fc5a10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5a14, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bc5a18, SUC_lexicon_tags_33 },
    { 0x741c5a18, SUC_lexicon_tags_37 },
    { 0x987c5a1a, SUC_lexicon_tags_9 },
    { 0xda3c5a1b, SUC_lexicon_tags_25 },
    { 0x717c5a1c, SUC_lexicon_tags_25 },
    { 0xc07c5a1d, SUC_lexicon_tags_17 },
    { 0xc85c5a1e, SUC_lexicon_tags_33 },
    { 0x19dc5a1f, SUC_lexicon_tags_47 },
    { 0x853c5a20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfefc5a23, SUC_lexicon_tags_12 },
    { 0xdadc5a23, SUC_lexicon_tags_25 },
    { 0xf5fc5a25, SUC_lexicon_tags_2 },
    { 0x147c5a26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6bbc5a28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7c5a2b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c5a30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dc5a33, SUC_lexicon_tags_11 },
    { 0x3c3c5a33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc39c5a36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1c5a39, SUC_lexicon_tags_7 },
    { 0x7e1c5a3a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fc5a3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c5a42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x611c5a44, SUC_lexicon_tags_63 },
    { 0x337c5a44, SUC_lexicon_tags_2 },
    { 0x1bfc5a46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbc5a49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x971c5a4c, SUC_lexicon_tags_61 },
    { 0xd59c5a4c, SUC_lexicon_tags_66 },
    { 0x681c5a4e, SUC_lexicon_tags_15 },
    { 0xe17c5a4e, SUC_lexicon_tags_17 },
    { 0x89c5a4d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x207c5a52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541c5a56, SUC_lexicon_tags_17 },
    { 0xa83c5a57, SUC_lexicon_tags_49 },
    { 0xe59c5a57, SUC_lexicon_tags_7 },
    { 0x735c5a58, SUC_lexicon_tags_25 },
    { 0x7c5c5a5a, SUC_lexicon_tags_37 },
    { 0x3a5c5a58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7c5a62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539c5a71, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x555c5a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fc5a76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7c5a7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa53c5a7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x897c5a80, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc5a89, SUC_lexicon_tags_33 },
    { 0xd83c5a8a, SUC_lexicon_tags_19 },
    { 0x283c5a8b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x933c5a8e, SUC_lexicon_tags_38 },
    { 0xe95c5a8e, SUC_lexicon_tags_47 },
    { 0x19bc5a90, SUC_lexicon_tags_2 },
    { 0xbddc5a90, SUC_lexicon_tags_1 },
    { 0x91bc5a92, SUC_lexicon_tags_14 },
    { 0x50fc5a92, SUC_lexicon_tags_13 },
    { 0x6e3c5a94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c9c5a96, SUC_lexicon_tags_37 },
    { 0x94dc5a97, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111c5a9d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x153c5aa0, SUC_lexicon_tags_26 },
    { 0xa65c5aa1, SUC_lexicon_tags_64 },
    { 0xb65c5aa0, SUC_lexicon_tags_17 },
    { 0x6cfc5aa3, SUC_lexicon_tags_6 },
    { 0xbb9c5aa4, SUC_lexicon_tags_15 },
    { 0x681c5aa5, SUC_lexicon_tags_17 },
    { 0xcf3c5aa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d5c5aa8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9c5aad, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x833c5aaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561c5aba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fdc5abc, SUC_lexicon_tags_37 },
    { 0xa97c5abd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57fc5abf, SUC_lexicon_tags_25 },
    { 0x983c5abf, SUC_lexicon_tags_25 },
    { 0x3f1c5ac1, SUC_lexicon_tags_17 },
    { 0x5c5c5ac0, SUC_lexicon_tags_7 },
    { 0xa97c5ac1, SUC_lexicon_tags_1 },
    { 0xef3c5ac4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x733c5ac7, SUC_lexicon_tags_14 },
    { 0x3e3c5ac8, SUC_lexicon_tags_83 },
    { 0x5b3c5ac9, SUC_lexicon_tags_231 },
    { 0x549c5ac9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383c5ad2, SUC_lexicon_tags_1 },
    { 0xb3c5ad2, SUC_lexicon_tags_28 },
    { 0x4bbc5ad4, SUC_lexicon_tags_0 },
    { 0x41c5ad4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dc5ada, SUC_lexicon_tags_11 },
    { 0x8dc5adb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa8bc5add, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501c5ae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21dc5ae9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e1c5aeb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc95c5aed, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c5af2, SUC_lexicon_tags_37 },
    { 0x935c5af2, SUC_lexicon_tags_11 },
    { 0x7d5c5af4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bc5af9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddc5afc, SUC_lexicon_tags_220 },
    { 0x4b7c5afc, SUC_lexicon_tags_7 },
    { 0x8efc5afc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x871c5b00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8e5c5b02, SUC_lexicon_tags_0 },
    { 0xe95c5b02, SUC_lexicon_tags_17 },
    { 0xd3fc5b03, SUC_lexicon_tags_10 },
    { 0x4afc5b02, SUC_lexicon_tags_37 },
    { 0x2afc5b05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x275c5b09, SUC_lexicon_tags_0 },
    { 0x413c5b09, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99fc5b0c, SUC_lexicon_tags_17 },
    { 0x1e9c5b0d, SUC_lexicon_tags_1 },
    { 0xb3c5b0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab7c5b10, SUC_lexicon_tags_21 },
    { 0xea1c5b11, SUC_lexicon_tags_46 },
    { 0xc97c5b11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4cbc5b14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe21c5b16, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5c5b19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x833c5b1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc5b1e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x65dc5b21, SUC_lexicon_tags_25 },
    { 0x9cdc5b22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1fc5b24, SUC_lexicon_tags_17 },
    { 0xb3dc5b24, SUC_lexicon_tags_25 },
    { 0xa3fc5b26, SUC_lexicon_tags_27 },
    { 0xdbbc5b27, SUC_lexicon_tags_2 },
    { 0x433c5b25, SUC_lexicon_tags_11 },
    { 0x9fdc5b29, SUC_lexicon_tags_7 },
    { 0xacdc5b2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30dc5b2f, SUC_lexicon_tags_2 },
    { 0x495c5b2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaabc5b32, SUC_lexicon_tags_7 },
    { 0xa47c5b32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22bc5b35, SUC_lexicon_tags_61 },
    { 0xf07c5b36, SUC_lexicon_tags_43 },
    { 0xeb5c5b37, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x973c5b39, SUC_lexicon_tags_5 },
    { 0xe2bc5b3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x17c5b3d, SUC_lexicon_tags_133 },
    { 0xdddc5b3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39c5b41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9dc5b43, SUC_lexicon_tags_27 },
    { 0xc4fc5b43, SUC_lexicon_tags_1 },
    { 0x66bc5b43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603c5b4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c1c5b4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bc5b50, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fc5b56, SUC_lexicon_tags_151 },
    { 0xf4bc5b57, SUC_lexicon_tags_7 },
    { 0x1c9c5b57, SUC_lexicon_tags_229 },
    { 0x911c5b59, SUC_lexicon_tags_82 },
    { 0x665c5b5a, SUC_lexicon_tags_38 },
    { 0xd61c5b5b, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x345c5b5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x735c5b60, SUC_lexicon_tags_0 },
    { 0x7efc5b60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x517c5b64, SUC_lexicon_tags_11 },
    { 0x825c5b64, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x819c5b68, SUC_lexicon_tags_60 },
    { 0x6c3c5b69, SUC_lexicon_tags_37 },
    { 0x269c5b6a, SUC_lexicon_tags_17 },
    { 0x545c5b6a, SUC_lexicon_tags_68 },
    { 0xf73c5b6a, SUC_lexicon_tags_11 },
    { 0x7b9c5b6d, SUC_lexicon_tags_46 },
    { 0x1d3c5b6d, SUC_lexicon_tags_46 },
    { 0x17dc5b6d, SUC_lexicon_tags_10 },
    { 0x369c5b70, SUC_lexicon_tags_63 },
    { 0xa1dc5b71, SUC_lexicon_tags_6 },
    { 0xe6dc5b71, SUC_lexicon_tags_31 },
    { 0x37dc5b73, SUC_lexicon_tags_25 },
    { 0xcbdc5b73, SUC_lexicon_tags_49 },
    { 0x73dc5b75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bc5b79, SUC_lexicon_tags_1 },
    { 0xb6bc5b7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6fc5b7c, SUC_lexicon_tags_47 },
    { 0x3d7c5b7d, SUC_lexicon_tags_25 },
    { 0x13bc5b7d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5dfc5b80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fc5b84, SUC_lexicon_tags_38 },
    { 0x57dc5b84, SUC_lexicon_tags_70 },
    { 0x723c5b86, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3c5b8a, SUC_lexicon_tags_27 },
    { 0x2b3c5b8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5c5b8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471c5b97, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x219c5b99, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x119c5b9c, SUC_lexicon_tags_37 },
    { 0xb51c5b9d, SUC_lexicon_tags_91 },
    { 0x29c5b9e, SUC_lexicon_tags_14 },
    { 0x59dc5b9f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfc5ba2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdc5ba6, SUC_lexicon_tags_15 },
    { 0x4a5c5ba7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3c5baa, SUC_lexicon_tags_23 },
    { 0x395c5bab, SUC_lexicon_tags_11 },
    { 0x841c5bab, SUC_lexicon_tags_19 },
    { 0x8b1c5bad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19c5bb3, SUC_lexicon_tags_17 },
    { 0xe7c5bb3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bc5bb7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87c5bbb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe43c5bbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259c5bc6, SUC_lexicon_tags_14 },
    { 0x833c5bc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01c5bcd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edc5bd3, SUC_lexicon_tags_4 },
    { 0x8ebc5bd3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcf3c5bd6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c5bda, SUC_lexicon_tags_13 },
    { 0x95fc5bda, SUC_lexicon_tags_47 },
    { 0x9c1c5bdc, SUC_lexicon_tags_17 },
    { 0x4d7c5bdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695c5be3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9b1c5be5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bc5bea, SUC_lexicon_tags_33 },
    { 0x185c5beb, SUC_lexicon_tags_37 },
    { 0xc9bc5bea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x819c5bee, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xce1c5bf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33c5bf8, SUC_lexicon_tags_48 },
    { 0xa51c5bf9, SUC_lexicon_tags_14 },
    { 0xf2bc5bfa, SUC_lexicon_tags_37 },
    { 0xebdc5bfa, SUC_lexicon_tags_17 },
    { 0x737c5bfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efc5c00, SUC_lexicon_tags_32 },
    { 0xddc5c01, SUC_lexicon_tags_25 },
    { 0x213c5c02, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfc5c0a, SUC_lexicon_tags_18 },
    { 0xde3c5c0b, SUC_lexicon_tags_37 },
    { 0x1dc5c0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25c5c0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65dc5c12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abc5c1c, SUC_lexicon_tags_14 },
    { 0xcd7c5c1d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1c5c29, SUC_lexicon_tags_2 },
    { 0x14fc5c29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fc5c31, SUC_lexicon_tags_2 },
    { 0xb2bc5c32, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3c5c37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddc5c39, SUC_lexicon_tags_27 },
    { 0x1dfc5c3a, SUC_lexicon_tags_14 },
    { 0x273c5c3a, SUC_lexicon_tags_1 },
    { 0xda5c5c3a, SUC_lexicon_tags_47 },
    { 0xad5c5c3d, SUC_lexicon_tags_63 },
    { 0x9f5c5c3e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81c5c41, SUC_lexicon_tags_60 },
    { 0xdf7c5c42, SUC_lexicon_tags_21 },
    { 0x4d7c5c43, SUC_lexicon_tags_198 },
    { 0x41fc5c42, SUC_lexicon_tags_26 },
    { 0x83dc5c45, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc03c5c47, SUC_lexicon_tags_38 },
    { 0xd29c5c48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedc5c4f, SUC_lexicon_tags_39 },
    { 0x9a7c5c50, SUC_lexicon_tags_1 },
    { 0xea3c5c51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bc5c55, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831c5c61, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fc5c64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c5c67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bc5c6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x567c5c6e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571c5c74, SUC_lexicon_tags_1 },
    { 0x6bc5c75, SUC_lexicon_tags_15 },
    { 0x10dc5c75, SUC_lexicon_tags_23 },
    { 0xb47c5c74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x255c5c7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x949c5c7d, SUC_lexicon_tags_10 },
    { 0xda5c5c7e, SUC_lexicon_tags_7 },
    { 0xd29c5c7f, SUC_lexicon_tags_24 },
    { 0xf5fc5c7e, SUC_lexicon_tags_37 },
    { 0xaa7c5c81, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x483c5c83, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17c5c87, SUC_lexicon_tags_25 },
    { 0x733c5c87, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf13c5c8a, SUC_lexicon_tags_4 },
    { 0x71c5c8b, SUC_lexicon_tags_37 },
    { 0x3fbc5c8c, SUC_lexicon_tags_17 },
    { 0x5dc5c8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fc5c90, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fc5c97, SUC_lexicon_tags_1 },
    { 0x66fc5c98, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x251c5c9a, SUC_lexicon_tags_12 },
    { 0x81dc5c9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdc5c9e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7c5ca2, SUC_lexicon_tags_1 },
    { 0xd15c5ca3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135c5ca9, SUC_lexicon_tags_37 },
    { 0x77bc5caa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x751c5cac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71c5cb2, SUC_lexicon_tags_23 },
    { 0x197c5cb2, SUC_lexicon_tags_38 },
    { 0x863c5cb2, SUC_lexicon_tags_18 },
    { 0xc3c5cb5, SUC_lexicon_tags_38 },
    { 0x443c5cb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4bc5cb8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x51c5cbb, SUC_lexicon_tags_37 },
    { 0x1ddc5cbb, SUC_lexicon_tags_25 },
    { 0x945c5cbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbc5cc2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xbddc5cc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c5cc8, SUC_lexicon_tags_25 },
    { 0xd43c5cc8, SUC_lexicon_tags_32 },
    { 0x851c5cca, SUC_lexicon_tags_47 },
    { 0x78fc5ccb, SUC_lexicon_tags_7 },
    { 0x793c5ccc, SUC_lexicon_tags_7 },
    { 0xfc9c5ccd, SUC_lexicon_tags_17 },
    { 0x363c5cce, SUC_lexicon_tags_16 },
    { 0x5f7c5ccb, SUC_lexicon_tags_1672 },
    { 0xae9c5cd0, SUC_lexicon_tags_15 },
    { 0x585c5cd0, SUC_lexicon_tags_9 },
    { 0x66fc5cd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa75c5cd4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5c5cd8, SUC_lexicon_tags_33 },
    { 0x2cdc5cd9, SUC_lexicon_tags_46 },
    { 0x309c5cd8, SUC_lexicon_tags_37 },
    { 0xaa7c5cd8, SUC_lexicon_tags_12 },
    { 0xd49c5cdb, SUC_lexicon_tags_11 },
    { 0x625c5cdd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53c5ce1, SUC_lexicon_tags_7 },
    { 0x439c5ce2, SUC_lexicon_tags_234 },
    { 0xa1c5ce3, SUC_lexicon_tags_65 },
    { 0x7ebc5ce4, SUC_lexicon_tags_15 },
    { 0x3c5ce5, SUC_lexicon_tags_15 },
    { 0x60fc5ce6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57bc5ce8, SUC_lexicon_tags_7 },
    { 0x21c5ce9, SUC_lexicon_tags_11 },
    { 0xbebc5cea, SUC_lexicon_tags_14 },
    { 0x889c5cea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb5c5ced, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bc5cf4, SUC_lexicon_tags_21 },
    { 0x62dc5cf5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x623c5cf8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c5cfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597c5d02, SUC_lexicon_tags_13 },
    { 0xd13c5d03, SUC_lexicon_tags_37 },
    { 0x3c5d02, SUC_lexicon_tags_87 },
    { 0x1f5c5d05, SUC_lexicon_tags_14 },
    { 0x9bdc5d06, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca5c5d09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fc5d0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9c5d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32fc5d16, SUC_lexicon_tags_7 },
    { 0x4d5c5d17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x897c5d19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x109c5d1c, SUC_lexicon_tags_17 },
    { 0x13fc5d1d, SUC_lexicon_tags_14 },
    { 0x34dc5d1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x96dc5d20, SUC_lexicon_tags_14 },
    { 0x11fc5d20, SUC_lexicon_tags_15 },
    { 0x6f7c5d20, SUC_lexicon_tags_37 },
    { 0xd07c5d23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9c5d26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867c5d35, SUC_lexicon_tags_2 },
    { 0x325c5d35, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x20dc5d39, SUC_lexicon_tags_17 },
    { 0xf69c5d3a, SUC_lexicon_tags_697 },
    { 0xcbdc5d3b, SUC_lexicon_tags_47 },
    { 0xc61c5d3b, SUC_lexicon_tags_27 },
    { 0xe5bc5d3c, SUC_lexicon_tags_37 },
    { 0xddfc5d3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x861c5d40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dc5d44, SUC_lexicon_tags_9 },
    { 0x47c5d45, SUC_lexicon_tags_37 },
    { 0xabfc5d45, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9c5d49, SUC_lexicon_tags_69 },
    { 0x60fc5d4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x271c5d4d, SUC_lexicon_tags_1 },
    { 0xc27c5d4e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x685c5d50, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fc5d59, SUC_lexicon_tags_38 },
    { 0xcffc5d5a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe7dc5d5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x181c5d5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4adc5d60, SUC_lexicon_tags_37 },
    { 0x311c5d61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c3c5d63, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7c5d66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x339c5d69, SUC_lexicon_tags_11 },
    { 0xa63c5d69, SUC_lexicon_tags_47 },
    { 0x679c5d6b, SUC_lexicon_tags_49 },
    { 0x36dc5d6c, SUC_lexicon_tags_0 },
    { 0x279c5d6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x353c5d6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7c5d72, SUC_lexicon_tags_61 },
    { 0xef9c5d73, SUC_lexicon_tags_15 },
    { 0x263c5d74, SUC_lexicon_tags_27 },
    { 0x1ddc5d74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5c5d79, SUC_lexicon_tags_25 },
    { 0xd95c5d7a, SUC_lexicon_tags_8 },
    { 0x71bc5d7b, SUC_lexicon_tags_77 },
    { 0x18bc5d7b, SUC_lexicon_tags_12 },
    { 0x72fc5d79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47fc5d7f, SUC_lexicon_tags_37 },
    { 0x6b1c5d7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7c5d84, SUC_lexicon_tags_17 },
    { 0x7fbc5d85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bc5d8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569c5d91, SUC_lexicon_tags_0 },
    { 0x31c5d91, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9afc5d94, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47c5d97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229c5d9d, SUC_lexicon_tags_52 },
    { 0x173c5d9d, SUC_lexicon_tags_37 },
    { 0x9b1c5d9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9fc5da1, SUC_lexicon_tags_11 },
    { 0x751c5da2, SUC_lexicon_tags_2 },
    { 0x2a3c5da3, SUC_lexicon_tags_17 },
    { 0x94dc5da4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bc5dab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1b7c5dad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x37dc5db0, SUC_lexicon_tags_11 },
    { 0x235c5db0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4abc5db4, SUC_lexicon_tags_7 },
    { 0xc59c5db4, SUC_lexicon_tags_12 },
    { 0xa65c5db6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x485c5db9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc27c5dbb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x989c5dbd, SUC_lexicon_tags_25 },
    { 0x63c5dbe, SUC_lexicon_tags_17 },
    { 0x97fc5dbe, SUC_lexicon_tags_17 },
    { 0x3a1c5dbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4f1c5dc2, SUC_lexicon_tags_29 },
    { 0x825c5dc2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99c5dc8, SUC_lexicon_tags_43 },
    { 0x44bc5dc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x871c5dcc, SUC_lexicon_tags_53 },
    { 0x9abc5dcc, SUC_lexicon_tags_38 },
    { 0x59dc5dce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fc5dd7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xba3c5dd9, SUC_lexicon_tags_45 },
    { 0x6abc5dd9, SUC_lexicon_tags_37 },
    { 0x677c5ddb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49dc5ddd, SUC_lexicon_tags_26 },
    { 0xf67c5dde, SUC_lexicon_tags_60 },
    { 0x3fbc5ddf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c5de9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57c5dee, SUC_lexicon_tags_32 },
    { 0xa81c5def, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abc5dfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x883c5dfc, SUC_lexicon_tags_12 },
    { 0xb05c5dfc, SUC_lexicon_tags_47 },
    { 0x5d9c5dfd, SUC_lexicon_tags_63 },
    { 0x51bc5dfd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5c5e02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c5e04, SUC_lexicon_tags_8 },
    { 0xabc5e05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dc5e0d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3c5e10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c5e18, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efc5e1c, SUC_lexicon_tags_28 },
    { 0x96dc5e1d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161c5e23, SUC_lexicon_tags_23 },
    { 0xb93c5e24, SUC_lexicon_tags_15 },
    { 0x9b7c5e25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc9bc5e27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x33c5e29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbc5e2c, SUC_lexicon_tags_15 },
    { 0x40fc5e2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1c5e31, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1c5e35, SUC_lexicon_tags_113 },
    { 0x44fc5e36, SUC_lexicon_tags_55 },
    { 0xcffc5e37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bc5e3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdc5e3d, SUC_lexicon_tags_16 },
    { 0xd5dc5e3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x833c5e41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1c5e44, SUC_lexicon_tags_30 },
    { 0x899c5e44, SUC_lexicon_tags_11 },
    { 0xd0bc5e46, SUC_lexicon_tags_33 },
    { 0x195c5e44, SUC_lexicon_tags_17 },
    { 0x6fdc5e47, SUC_lexicon_tags_10 },
    { 0x47c5e49, SUC_lexicon_tags_38 },
    { 0xaa1c5e47, SUC_lexicon_tags_9 },
    { 0xe59c5e46, SUC_lexicon_tags_32 },
    { 0x93bc5e4c, SUC_lexicon_tags_107 },
    { 0xb67c5e4d, SUC_lexicon_tags_56 },
    { 0x9d5c5e4b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x499c5e51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fc5e58, SUC_lexicon_tags_37 },
    { 0xd15c5e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09c5e5f, SUC_lexicon_tags_1 },
    { 0x895c5e60, SUC_lexicon_tags_17 },
    { 0x9bfc5e61, SUC_lexicon_tags_39 },
    { 0x123c5e5f, SUC_lexicon_tags_15 },
    { 0xcf9c5e62, SUC_lexicon_tags_173 },
    { 0x1dbc5e61, SUC_lexicon_tags_7 },
    { 0x357c5e62, SUC_lexicon_tags_13 },
    { 0x67c5e60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd43c5e68, SUC_lexicon_tags_12 },
    { 0x5c7c5e69, SUC_lexicon_tags_80 },
    { 0x2a3c5e6a, SUC_lexicon_tags_45 },
    { 0x16fc5e69, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdc5e6e, SUC_lexicon_tags_151 },
    { 0x76bc5e6e, SUC_lexicon_tags_11 },
    { 0xb3dc5e70, SUC_lexicon_tags_26 },
    { 0x7bbc5e6e, SUC_lexicon_tags_11 },
    { 0xb5fc5e72, SUC_lexicon_tags_8 },
    { 0x48fc5e73, SUC_lexicon_tags_14 },
    { 0x26fc5e74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1dc5e76, SUC_lexicon_tags_13 },
    { 0x69bc5e77, SUC_lexicon_tags_23 },
    { 0xd47c5e78, SUC_lexicon_tags_47 },
    { 0x763c5e76, SUC_lexicon_tags_66 },
    { 0x445c5e7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bc5e7e, SUC_lexicon_tags_7 },
    { 0xd29c5e7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bc5e82, SUC_lexicon_tags_66 },
    { 0xa31c5e83, SUC_lexicon_tags_4 },
    { 0xcc9c5e84, SUC_lexicon_tags_4 },
    { 0x141c5e83, SUC_lexicon_tags_64 },
    { 0xb5dc5e86, SUC_lexicon_tags_11 },
    { 0x82fc5e87, SUC_lexicon_tags_70 },
    { 0x93bc5e86, SUC_lexicon_tags_5 },
    { 0xc71c5e85, SUC_lexicon_tags_18 },
    { 0xa23c5e8a, SUC_lexicon_tags_1 },
    { 0xac3c5e84, SUC_lexicon_tags_55 },
    { 0xeb1c5e8c, SUC_lexicon_tags_51 },
    { 0x667c5e84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd21c5e8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x953c5e91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcbc5e93, SUC_lexicon_tags_30 },
    { 0x1c5e93, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x443c5e97, SUC_lexicon_tags_4 },
    { 0xf23c5e98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49c5e9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x731c5e9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bc5ea2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229c5eaa, SUC_lexicon_tags_38 },
    { 0xeedc5eab, SUC_lexicon_tags_7 },
    { 0x91dc5eab, SUC_lexicon_tags_17 },
    { 0xc1bc5ead, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc5eb1, SUC_lexicon_tags_19 },
    { 0x3dfc5eb2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9afc5eb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcdc5eb6, SUC_lexicon_tags_32 },
    { 0x607c5eb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dc5ebc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1c5ec0, SUC_lexicon_tags_11 },
    { 0x6ddc5ec0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e3c5ec3, SUC_lexicon_tags_31 },
    { 0x31dc5ec3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171c5ec8, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bc5ecb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57c5ece, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313c5ed3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515c5ed9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505c5edd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5c5ee5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dc5ee8, SUC_lexicon_tags_2 },
    { 0x2f9c5ee9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7c5eed, SUC_lexicon_tags_11 },
    { 0x6c3c5eee, SUC_lexicon_tags_10 },
    { 0x663c5eef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37c5ef2, SUC_lexicon_tags_42 },
    { 0xf6bc5ef3, SUC_lexicon_tags_2 },
    { 0x46bc5ef4, SUC_lexicon_tags_91 },
    { 0x707c5ef4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x18fc5ef8, SUC_lexicon_tags_45 },
    { 0x2f5c5ef9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11c5eff, SUC_lexicon_tags_37 },
    { 0xe37c5f00, SUC_lexicon_tags_17 },
    { 0x329c5f01, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3c5f07, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89c5f0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5c5f13, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x33c5f15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdc5f18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1c5f1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1c5f25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x463c5f27, SUC_lexicon_tags_11 },
    { 0x3bdc5f28, SUC_lexicon_tags_25 },
    { 0x8abc5f29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfc5f2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bc5f33, SUC_lexicon_tags_45 },
    { 0x33dc5f34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dc5f37, SUC_lexicon_tags_47 },
    { 0xe2fc5f38, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x65c5f3b, SUC_lexicon_tags_12 },
    { 0x69c5f3c, SUC_lexicon_tags_55 },
    { 0x9bbc5f3d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75c5f42, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc91c5f44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19c5f48, SUC_lexicon_tags_30 },
    { 0x73dc5f49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905c5f4d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dc5f51, SUC_lexicon_tags_15 },
    { 0xd4bc5f52, SUC_lexicon_tags_17 },
    { 0x109c5f53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dc5f57, SUC_lexicon_tags_46 },
    { 0xae7c5f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7c5f5d, SUC_lexicon_tags_39 },
    { 0x941c5f5e, SUC_lexicon_tags_60 },
    { 0x525c5f5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561c5f6b, SUC_lexicon_tags_47 },
    { 0xa23c5f6c, SUC_lexicon_tags_2 },
    { 0x761c5f6c, SUC_lexicon_tags_7 },
    { 0xa9dc5f6e, SUC_lexicon_tags_15 },
    { 0x8fbc5f6e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121c5f74, SUC_lexicon_tags_1 },
    { 0x385c5f74, SUC_lexicon_tags_7 },
    { 0xcf7c5f76, SUC_lexicon_tags_11 },
    { 0x151c5f74, SUC_lexicon_tags_11 },
    { 0xd99c5f76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x759c5f7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb05c5f7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17c5f81, SUC_lexicon_tags_12 },
    { 0x309c5f82, SUC_lexicon_tags_1 },
    { 0xd83c5f81, SUC_lexicon_tags_7 },
    { 0x795c5f84, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67c5f87, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37c5f91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdafc5f93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x835c5f96, SUC_lexicon_tags_49 },
    { 0x77c5f97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdfbc5f99, SUC_lexicon_tags_30 },
    { 0x8f1c5f9a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x113c5f9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc39c5f9f, SUC_lexicon_tags_17 },
    { 0x4bc5fa0, SUC_lexicon_tags_31 },
    { 0xc39c5fa1, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0x997c5fa3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bc5fa6, SUC_lexicon_tags_9 },
    { 0x3edc5fa6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99c5fb2, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0xc7bc5fb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fdc5fb6, SUC_lexicon_tags_11 },
    { 0x2d9c5fb6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5f5c5fb9, SUC_lexicon_tags_2 },
    { 0xf43c5fba, SUC_lexicon_tags_1 },
    { 0xfc7c5fbb, SUC_lexicon_tags_11 },
    { 0x331c5fbc, SUC_lexicon_tags_2 },
    { 0x4bfc5fbd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa59c5fbf, SUC_lexicon_tags_11 },
    { 0xdabc5fc0, SUC_lexicon_tags_47 },
    { 0x337c5fc1, SUC_lexicon_tags_25 },
    { 0xd03c5fc0, SUC_lexicon_tags_12 },
    { 0x893c5fc2, SUC_lexicon_tags_7 },
    { 0xca9c5fc0, SUC_lexicon_tags_60 },
    { 0xb67c5fc5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5edc5fc8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa6fc5fca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbc5fcf, SUC_lexicon_tags_33 },
    { 0xda5c5fcf, SUC_lexicon_tags_45 },
    { 0x7cbc5fd1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9c5fd9, SUC_lexicon_tags_11 },
    { 0x8f1c5fda, SUC_lexicon_tags_49 },
    { 0xd99c5fda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x88bc5fdd, SUC_lexicon_tags_1 },
    { 0x603c5fde, SUC_lexicon_tags_12 },
    { 0xacfc5fdf, SUC_lexicon_tags_11 },
    { 0x7bc5fdf, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadc5fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bc5fe8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3c5fef, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63c5ff2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49c5ff6, SUC_lexicon_tags_63 },
    { 0x969c5ff7, SUC_lexicon_tags_12 },
    { 0xadc5ff8, SUC_lexicon_tags_25 },
    { 0x7e9c5ff8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bc5fff, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x121c6002, SUC_lexicon_tags_33 },
    { 0xd27c6003, SUC_lexicon_tags_30 },
    { 0x9c1c6004, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853c600a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x635c600d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c6012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x845c6014, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbc6019, SUC_lexicon_tags_55 },
    { 0x4bc601a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41c6022, SUC_lexicon_tags_12 },
    { 0xfc5c6023, SUC_lexicon_tags_12 },
    { 0x939c6024, SUC_lexicon_tags_1 },
    { 0x7c3c6025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa61c6027, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fc602c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa53c602e, SUC_lexicon_tags_2 },
    { 0x249c602e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199c6035, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bc6039, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x367c603b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x849c603d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x823c603f, SUC_lexicon_tags_32 },
    { 0x219c6040, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadc6046, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc604d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581c6052, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x83fc6055, SUC_lexicon_tags_25 },
    { 0x577c6056, SUC_lexicon_tags_19 },
    { 0xb2dc6056, SUC_lexicon_tags_15 },
    { 0x533c6058, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bc605b, SUC_lexicon_tags_1 },
    { 0xd37c605c, SUC_lexicon_tags_26 },
    { 0x183c605d, SUC_lexicon_tags_19 },
    { 0x477c605b, SUC_lexicon_tags_25 },
    { 0x69fc605f, SUC_lexicon_tags_10 },
    { 0xddc6060, SUC_lexicon_tags_7 },
    { 0x909c6061, SUC_lexicon_tags_37 },
    { 0x937c6061, SUC_lexicon_tags_27 },
    { 0xe6bc6060, SUC_lexicon_tags_84 },
    { 0x403c6060, SUC_lexicon_tags_1 },
    { 0xd4fc6065, SUC_lexicon_tags_23 },
    { 0x4afc6061, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79c6069, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda9c606b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf7c606d, SUC_lexicon_tags_11 },
    { 0x255c606d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dc6072, SUC_lexicon_tags_33 },
    { 0x631c6073, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91c6078, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e3c607a, SUC_lexicon_tags_37 },
    { 0xe73c607a, SUC_lexicon_tags_0 },
    { 0x6a5c607b, SUC_lexicon_tags_0 },
    { 0xd05c607d, SUC_lexicon_tags_2 },
    { 0x9f5c607e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9c6081, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x503c6083, SUC_lexicon_tags_15 },
    { 0x20dc6084, SUC_lexicon_tags_15 },
    { 0x3bc6085, SUC_lexicon_tags_31 },
    { 0x6dc6083, SUC_lexicon_tags_33 },
    { 0x2c7c6087, SUC_lexicon_tags_25 },
    { 0x6d9c6088, SUC_lexicon_tags_37 },
    { 0x475c6087, SUC_lexicon_tags_37 },
    { 0xcabc6087, SUC_lexicon_tags_8 },
    { 0x3cbc608b, SUC_lexicon_tags_65 },
    { 0xe53c608b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x593c608e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7c6093, SUC_lexicon_tags_7 },
    { 0x783c6094, SUC_lexicon_tags_37 },
    { 0x673c6094, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x861c6097, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bc609c, SUC_lexicon_tags_43 },
    { 0x65fc609d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7f5c609f, SUC_lexicon_tags_11 },
    { 0x14bc60a0, SUC_lexicon_tags_9 },
    { 0xed7c60a1, SUC_lexicon_tags_13 },
    { 0x71dc60a0, SUC_lexicon_tags_21 },
    { 0xae9c60a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bc60a8, SUC_lexicon_tags_12 },
    { 0x329c60a9, SUC_lexicon_tags_60 },
    { 0xc85c60a8, SUC_lexicon_tags_27 },
    { 0x2d7c60ab, SUC_lexicon_tags_27 },
    { 0xe69c60ac, SUC_lexicon_tags_3 },
    { 0xac7c60a9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bc60b3, SUC_lexicon_tags_62 },
    { 0x613c60b4, SUC_lexicon_tags_23 },
    { 0xa85c60b5, SUC_lexicon_tags_1 },
    { 0xeafc60b6, SUC_lexicon_tags_17 },
    { 0x145c60b6, SUC_lexicon_tags_0 },
    { 0x5ebc60b8, SUC_lexicon_tags_6 },
    { 0x635c60b8, SUC_lexicon_tags_70 },
    { 0x6f9c60b8, SUC_lexicon_tags_152 },
    { 0xbcbc60b4, SUC_lexicon_tags_5 },
    { 0xa41c60b9, SUC_lexicon_tags_12 },
    { 0xecfc60bd, SUC_lexicon_tags_16 },
    { 0xf77c60be, SUC_lexicon_tags_15 },
    { 0xee7c60bc, SUC_lexicon_tags_15 },
    { 0x679c60c0, SUC_lexicon_tags_301 },
    { 0xc41c60b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddc60c8, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43c60d0, SUC_lexicon_tags_4 },
    { 0x45c60d1, SUC_lexicon_tags_37 },
    { 0x1afc60d2, SUC_lexicon_tags_45 },
    { 0xec3c60d3, SUC_lexicon_tags_12 },
    { 0xe8fc60d4, SUC_lexicon_tags_30 },
    { 0xfe1c60d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x81fc60d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x617c60db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda5c60dd, SUC_lexicon_tags_23 },
    { 0x521c60dd, SUC_lexicon_tags_15 },
    { 0x879c60dd, SUC_lexicon_tags_60 },
    { 0x8dc60e0, SUC_lexicon_tags_12 },
    { 0xa3c60dd, SUC_lexicon_tags_0 },
    { 0xe45c60e0, SUC_lexicon_tags_47 },
    { 0x113c60e3, SUC_lexicon_tags_17 },
    { 0x463c60e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfc60e7, SUC_lexicon_tags_17 },
    { 0x52bc60e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85dc60ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc60f0, SUC_lexicon_tags_38 },
    { 0xabc60f0, SUC_lexicon_tags_11 },
    { 0xa2bc60f2, SUC_lexicon_tags_12 },
    { 0xa1c60f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x72bc60f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7c60fd, SUC_lexicon_tags_37 },
    { 0x65bc60fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x36dc6100, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367c6108, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fc610b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c610f, SUC_lexicon_tags_25 },
    { 0x449c610f, SUC_lexicon_tags_11 },
    { 0xe3fc6111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bc611a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x36dc611d, SUC_lexicon_tags_26 },
    { 0x659c611d, SUC_lexicon_tags_37 },
    { 0x563c611d, SUC_lexicon_tags_33 },
    { 0xfabc6120, SUC_lexicon_tags_2 },
    { 0x203c6121, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc6127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933c612d, SUC_lexicon_tags_47 },
    { 0xfedc612d, SUC_lexicon_tags_43 },
    { 0xfcbc612f, SUC_lexicon_tags_13 },
    { 0x145c6130, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bc6134, SUC_lexicon_tags_31 },
    { 0xe4fc6135, SUC_lexicon_tags_11 },
    { 0x5d3c6136, SUC_lexicon_tags_37 },
    { 0x48dc6135, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3c613a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x317c613c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebc613f, SUC_lexicon_tags_1 },
    { 0x975c6140, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fc6151, SUC_lexicon_tags_27 },
    { 0xbf1c6151, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xae3c6154, SUC_lexicon_tags_38 },
    { 0xa29c6154, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadc6158, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dc615e, SUC_lexicon_tags_2 },
    { 0xb01c615f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69c6162, SUC_lexicon_tags_47 },
    { 0x493c6162, SUC_lexicon_tags_45 },
    { 0xaefc6164, SUC_lexicon_tags_26 },
    { 0x5e3c6164, SUC_lexicon_tags_2 },
    { 0xf61c6166, SUC_lexicon_tags_32 },
    { 0x591c6166, SUC_lexicon_tags_4 },
    { 0xf13c6168, SUC_lexicon_tags_11 },
    { 0x3c1c6166, SUC_lexicon_tags_8 },
    { 0xb13c6164, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6e1c616c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321c6171, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe49c6173, SUC_lexicon_tags_12 },
    { 0x547c6174, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11bc6176, SUC_lexicon_tags_12 },
    { 0x54bc6177, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x419c6179, SUC_lexicon_tags_1 },
    { 0xdfbc617a, SUC_lexicon_tags_17 },
    { 0x4d3c617b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfc6186, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c6189, SUC_lexicon_tags_7 },
    { 0x89bc618a, SUC_lexicon_tags_2 },
    { 0x73dc618b, SUC_lexicon_tags_15 },
    { 0x1b9c618b, SUC_lexicon_tags_23 },
    { 0xe7fc618c, SUC_lexicon_tags_17 },
    { 0x765c618e, SUC_lexicon_tags_49 },
    { 0xc5bc618e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85c6196, SUC_lexicon_tags_37 },
    { 0xbddc6197, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fc619d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dc61a1, SUC_lexicon_tags_104 },
    { 0x9b7c61a2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01c61a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9c61a9, SUC_lexicon_tags_45 },
    { 0xbfbc61aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x64fc61ad, SUC_lexicon_tags_29 },
    { 0x2b1c61ae, SUC_lexicon_tags_31 },
    { 0xe55c61ad, SUC_lexicon_tags_37 },
    { 0x119c61b0, SUC_lexicon_tags_17 },
    { 0xae7c61b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bc61b6, SUC_lexicon_tags_7 },
    { 0xa6fc61b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeb7c61b9, SUC_lexicon_tags_198 },
    { 0xce9c61ba, SUC_lexicon_tags_2 },
    { 0x25dc61ba, SUC_lexicon_tags_47 },
    { 0x8bc61bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fc61bf, SUC_lexicon_tags_37 },
    { 0x8b3c61c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6b7c61c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69c61c5, SUC_lexicon_tags_12 },
    { 0xf95c61c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81c61c8, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bc61cb, SUC_lexicon_tags_49 },
    { 0xac9c61cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8adc61cf, SUC_lexicon_tags_40 },
    { 0x445c61cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x367c61d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bfc61d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffc61d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7c61da, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dc61e7, SUC_lexicon_tags_26 },
    { 0x7f3c61e8, SUC_lexicon_tags_69 },
    { 0xce7c61e8, SUC_lexicon_tags_25 },
    { 0xf9bc61ea, SUC_lexicon_tags_15 },
    { 0x8e9c61ea, SUC_lexicon_tags_60 },
    { 0xdd5c61ec, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x627c61ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615c61f2, SUC_lexicon_tags_47 },
    { 0xd9fc61f2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd59c61f5, SUC_lexicon_tags_48 },
    { 0x80bc61f6, SUC_lexicon_tags_33 },
    { 0x495c61f6, SUC_lexicon_tags_25 },
    { 0xb53c61f8, SUC_lexicon_tags_1 },
    { 0xe11c61f9, SUC_lexicon_tags_33 },
    { 0xebbc61fa, SUC_lexicon_tags_17 },
    { 0x46fc61f5, SUC_lexicon_tags_48 },
    { 0x353c61fc, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbc6202, SUC_lexicon_tags_11 },
    { 0x319c6203, SUC_lexicon_tags_37 },
    { 0xe65c6204, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5c6208, SUC_lexicon_tags_38 },
    { 0x2f7c6209, SUC_lexicon_tags_1 },
    { 0x547c620a, SUC_lexicon_tags_4 },
    { 0x15dc620a, SUC_lexicon_tags_2 },
    { 0xf91c620c, SUC_lexicon_tags_11 },
    { 0xf37c6209, SUC_lexicon_tags_12 },
    { 0x1bdc620e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189c6216, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dc621c, SUC_lexicon_tags_11 },
    { 0xe0dc621c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bc6220, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23c6225, SUC_lexicon_tags_21 },
    { 0x1b9c6225, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x995c6228, SUC_lexicon_tags_46 },
    { 0x851c6229, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x175c622b, SUC_lexicon_tags_0 },
    { 0x8b5c622c, SUC_lexicon_tags_37 },
    { 0x713c622c, SUC_lexicon_tags_37 },
    { 0x833c622e, SUC_lexicon_tags_15 },
    { 0x581c622e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdc6232, SUC_lexicon_tags_38 },
    { 0xe25c6232, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215c6237, SUC_lexicon_tags_25 },
    { 0xa8bc6237, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x211c623a, SUC_lexicon_tags_19 },
    { 0x5d3c623b, SUC_lexicon_tags_6 },
    { 0x1e5c623b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfcfc623e, SUC_lexicon_tags_56 },
    { 0x231c623f, SUC_lexicon_tags_14 },
    { 0x90bc623f, SUC_lexicon_tags_29 },
    { 0x4a1c6241, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbc6244, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3c6249, SUC_lexicon_tags_11 },
    { 0x23bc624a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3c624c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x419c624f, SUC_lexicon_tags_23 },
    { 0x78dc624f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedc6253, SUC_lexicon_tags_38 },
    { 0x483c6254, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69c6258, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fc625e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x801c6260, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb2fc6262, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e7c6264, SUC_lexicon_tags_11 },
    { 0x9d1c6265, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54dc626a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617c6270, SUC_lexicon_tags_6 },
    { 0x6bdc6271, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x183c6273, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fc6276, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257c627b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7c627e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x28dc6280, SUC_lexicon_tags_9 },
    { 0xc7fc6281, SUC_lexicon_tags_19 },
    { 0x381c6281, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff9c6284, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bc628e, SUC_lexicon_tags_37 },
    { 0xa05c628e, SUC_lexicon_tags_11 },
    { 0xfadc6290, SUC_lexicon_tags_15 },
    { 0x5dfc6290, SUC_lexicon_tags_18 },
    { 0x93bc6292, SUC_lexicon_tags_25 },
    { 0xf07c6292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945c6298, SUC_lexicon_tags_60 },
    { 0xd03c6299, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49fc629b, SUC_lexicon_tags_7 },
    { 0xd17c629c, SUC_lexicon_tags_17 },
    { 0x9e5c629d, SUC_lexicon_tags_38 },
    { 0xfabc629e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bc62a2, SUC_lexicon_tags_0 },
    { 0x633c62a3, SUC_lexicon_tags_23 },
    { 0xacbc62a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x65fc62a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1c62ab, SUC_lexicon_tags_12 },
    { 0x35fc62ab, SUC_lexicon_tags_45 },
    { 0x137c62ac, SUC_lexicon_tags_25 },
    { 0x207c62ab, SUC_lexicon_tags_12 },
    { 0xbc9c62af, SUC_lexicon_tags_17 },
    { 0xbdc62b0, SUC_lexicon_tags_29 },
    { 0x2a1c62b1, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0x2d3c62b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dc62b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15dc62bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21c62bf, SUC_lexicon_tags_2 },
    { 0xc7bc62c0, SUC_lexicon_tags_2 },
    { 0xff7c62c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bc62c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edc62c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89dc62cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7c62d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd85c62d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bc62d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa47c62d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfc62da, SUC_lexicon_tags_27 },
    { 0x8afc62da, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51c62e0, SUC_lexicon_tags_11 },
    { 0xc73c62e1, SUC_lexicon_tags_15 },
    { 0x143c62e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3e3c62e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3c62e8, SUC_lexicon_tags_0 },
    { 0x121c62e9, SUC_lexicon_tags_15 },
    { 0x8afc62e9, SUC_lexicon_tags_37 },
    { 0x329c62eb, SUC_lexicon_tags_63 },
    { 0x4fbc62ec, SUC_lexicon_tags_1 },
    { 0x1a9c62ec, SUC_lexicon_tags_25 },
    { 0xdb1c62ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5b7c62f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879c62f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edc62fa, SUC_lexicon_tags_82 },
    { 0xc61c62fb, SUC_lexicon_tags_61 },
    { 0x9b7c62fc, SUC_lexicon_tags_9 },
    { 0x4bbc62fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbc6301, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f1c6303, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863c630c, SUC_lexicon_tags_14 },
    { 0x18fc630d, SUC_lexicon_tags_82 },
    { 0x8dbc630c, SUC_lexicon_tags_1 },
    { 0x29c630f, SUC_lexicon_tags_82 },
    { 0xcc3c6310, SUC_lexicon_tags_61 },
    { 0x88fc6311, SUC_lexicon_tags_11 },
    { 0x5b1c6311, SUC_lexicon_tags_9 },
    { 0xf43c6311, SUC_lexicon_tags_47 },
    { 0xcadc6312, SUC_lexicon_tags_6 },
    { 0xf6fc6310, SUC_lexicon_tags_38 },
    { 0xf57c6316, SUC_lexicon_tags_26 },
    { 0x6c1c630f, SUC_lexicon_tags_1 },
    { 0x37c6318, SUC_lexicon_tags_7 },
    { 0x4e3c6319, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xab1c631c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb3c631e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bc6323, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5adc6325, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85c632c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dc632f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebc6335, SUC_lexicon_tags_8 },
    { 0x341c6336, SUC_lexicon_tags_11 },
    { 0xb9fc6337, SUC_lexicon_tags_15 },
    { 0xcddc6338, SUC_lexicon_tags_65 },
    { 0xd93c6335, SUC_lexicon_tags_39 },
    { 0x8ebc6335, SUC_lexicon_tags_65 },
    { 0x813c6335, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bc633e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887c6344, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfc6348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf81c634a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffc634d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa33c634f, SUC_lexicon_tags_47 },
    { 0x2e3c634f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fc6354, SUC_lexicon_tags_17 },
    { 0x435c6354, SUC_lexicon_tags_35 },
    { 0x92dc6356, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07c635e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8c3c6360, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c6366, SUC_lexicon_tags_6 },
    { 0x68dc6367, SUC_lexicon_tags_15 },
    { 0xfbc6367, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fc636d, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9c6377, SUC_lexicon_tags_1 },
    { 0xe23c6378, SUC_lexicon_tags_17 },
    { 0xaf7c6377, SUC_lexicon_tags_48 },
    { 0x34fc637a, SUC_lexicon_tags_65 },
    { 0x861c637a, SUC_lexicon_tags_2 },
    { 0xe1c637c, SUC_lexicon_tags_11 },
    { 0x65c637a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bc6380, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb67c6382, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fc6388, SUC_lexicon_tags_15 },
    { 0x9e5c6388, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1c638f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9c6393, SUC_lexicon_tags_17 },
    { 0x87fc6394, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e5c6396, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x975c6398, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf2bc639a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x217c639d, SUC_lexicon_tags_4 },
    { 0xe41c639d, SUC_lexicon_tags_2 },
    { 0xbb1c639d, SUC_lexicon_tags_0 },
    { 0x6d5c639e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbc63a4, SUC_lexicon_tags_11 },
    { 0xb9dc63a5, SUC_lexicon_tags_11 },
    { 0xc21c63a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3c63a9, SUC_lexicon_tags_25 },
    { 0x7e5c63a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4fc63ac, SUC_lexicon_tags_23 },
    { 0x91bc63ad, SUC_lexicon_tags_15 },
    { 0x159c63ad, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73c63b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdc63bb, SUC_lexicon_tags_11 },
    { 0x86bc63bc, SUC_lexicon_tags_63 },
    { 0x6c1c63bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bc63c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x313c63c3, SUC_lexicon_tags_11 },
    { 0xdf5c63c4, SUC_lexicon_tags_37 },
    { 0x737c63c4, SUC_lexicon_tags_19 },
    { 0x6a7c63c6, SUC_lexicon_tags_17 },
    { 0x1f3c63c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9fdc63c9, SUC_lexicon_tags_17 },
    { 0xa35c63c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87c63cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bc63d9, SUC_lexicon_tags_19 },
    { 0x879c63d9, SUC_lexicon_tags_25 },
    { 0x6abc63db, SUC_lexicon_tags_11 },
    { 0x53dc63dc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917c63e3, SUC_lexicon_tags_28 },
    { 0xb99c63e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbc63ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x291c63f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3c63f9, SUC_lexicon_tags_1 },
    { 0x55c63fa, SUC_lexicon_tags_7 },
    { 0xd55c63fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efc6400, SUC_lexicon_tags_100 },
    { 0xc41c6401, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3c6405, SUC_lexicon_tags_9 },
    { 0xc2dc6406, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dc6409, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x793c640b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9c640f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9c6414, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4edc6416, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3c641b, SUC_lexicon_tags_0 },
    { 0x641c641c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69c641e, SUC_lexicon_tags_100 },
    { 0x953c641f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bc6422, SUC_lexicon_tags_62 },
    { 0x327c6423, SUC_lexicon_tags_32 },
    { 0xe13c6422, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5c6428, SUC_lexicon_tags_23 },
    { 0x489c6429, SUC_lexicon_tags_26 },
    { 0xbfc642a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3c642f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffc6432, SUC_lexicon_tags_25 },
    { 0x3e5c6433, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bc6436, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6abc6439, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x515c643b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e3c643d, SUC_lexicon_tags_25 },
    { 0x425c643d, SUC_lexicon_tags_25 },
    { 0x80dc643e, SUC_lexicon_tags_17 },
    { 0x97bc6440, SUC_lexicon_tags_11 },
    { 0xd8bc6440, SUC_lexicon_tags_21 },
    { 0x85bc6442, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffc6445, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x191c6448, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x15fc644a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edc644f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3c6453, SUC_lexicon_tags_27 },
    { 0xa6dc6454, SUC_lexicon_tags_14 },
    { 0x955c6455, SUC_lexicon_tags_0 },
    { 0x761c6456, SUC_lexicon_tags_25 },
    { 0x959c6455, SUC_lexicon_tags_2 },
    { 0x77c6458, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dc645c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afc645f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe03c6461, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7c6466, SUC_lexicon_tags_12 },
    { 0x8b9c6467, SUC_lexicon_tags_25 },
    { 0xe31c6468, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x929c646a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993c646f, SUC_lexicon_tags_38 },
    { 0x7bc646f, SUC_lexicon_tags_12 },
    { 0x37c6471, SUC_lexicon_tags_7 },
    { 0xc5fc6471, SUC_lexicon_tags_37 },
    { 0x825c6473, SUC_lexicon_tags_25 },
    { 0x7a7c6472, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbc6477, SUC_lexicon_tags_7 },
    { 0xb19c6478, SUC_lexicon_tags_8 },
    { 0x12bc6477, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7c647d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5c6482, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x711c6485, SUC_lexicon_tags_13 },
    { 0xa0dc6485, SUC_lexicon_tags_33 },
    { 0x603c6487, SUC_lexicon_tags_4 },
    { 0xc9fc6487, SUC_lexicon_tags_43 },
    { 0xc3c6488, SUC_lexicon_tags_17 },
    { 0x41bc6488, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x221c648c, SUC_lexicon_tags_17 },
    { 0xb73c648c, SUC_lexicon_tags_17 },
    { 0xbd7c648d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77c6492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc37c6494, SUC_lexicon_tags_11 },
    { 0x2e5c6495, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fc6499, SUC_lexicon_tags_3 },
    { 0x385c6499, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793c649e, SUC_lexicon_tags_45 },
    { 0x24dc649f, SUC_lexicon_tags_37 },
    { 0x579c649e, SUC_lexicon_tags_0 },
    { 0x2f1c649f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49c64a8, SUC_lexicon_tags_56 },
    { 0x51c64a9, SUC_lexicon_tags_12 },
    { 0xed5c64aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471c64ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdc64b3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37c64b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc64bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39c64c2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bc64c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7c64cb, SUC_lexicon_tags_25 },
    { 0x33c64cc, SUC_lexicon_tags_11 },
    { 0xa07c64cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dc64d0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2dc64d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361c64d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fc64db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x149c64dd, SUC_lexicon_tags_43 },
    { 0xf19c64de, SUC_lexicon_tags_15 },
    { 0x71dc64dd, SUC_lexicon_tags_17 },
    { 0x341c64df, SUC_lexicon_tags_54 },
    { 0x4b7c64df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf1c64e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbc64e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x485c64e9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x1bc64eb, SUC_lexicon_tags_6 },
    { 0x311c64ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8bc64ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17dc64f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda5c64f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf07c64f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241c64fa, SUC_lexicon_tags_7 },
    { 0xacbc64fb, SUC_lexicon_tags_31 },
    { 0xb4bc64fb, SUC_lexicon_tags_74 },
    { 0x98fc64fd, SUC_lexicon_tags_12 },
    { 0x13dc64fd, SUC_lexicon_tags_17 },
    { 0x7f1c64fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc6502, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e3c6504, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bc6507, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeefc6509, SUC_lexicon_tags_16 },
    { 0x333c6509, SUC_lexicon_tags_254 },
    { 0x7e1c6509, SUC_lexicon_tags_23 },
    { 0xec7c6509, SUC_lexicon_tags_1 },
    { 0xe6fc650a, SUC_lexicon_tags_27 },
    { 0xda5c650c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34dc6510, SUC_lexicon_tags_29 },
    { 0xc87c6511, SUC_lexicon_tags_12 },
    { 0x87fc6512, SUC_lexicon_tags_1 },
    { 0xfb5c6513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801c651a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19fc651c, SUC_lexicon_tags_14 },
    { 0x7cdc651d, SUC_lexicon_tags_60 },
    { 0x6efc651e, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x837c6521, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa2fc6523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fc6529, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc652c, SUC_lexicon_tags_11 },
    { 0x889c652d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981c6535, SUC_lexicon_tags_39 },
    { 0xcc9c6536, SUC_lexicon_tags_7 },
    { 0x7ebc6536, SUC_lexicon_tags_17 },
    { 0x28dc6536, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7c653e, SUC_lexicon_tags_37 },
    { 0xa7c653f, SUC_lexicon_tags_26 },
    { 0xf2bc653e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bbc6542, SUC_lexicon_tags_17 },
    { 0x78dc6542, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfc6546, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9c6549, SUC_lexicon_tags_1 },
    { 0x903c654a, SUC_lexicon_tags_55 },
    { 0xa43c654a, SUC_lexicon_tags_9 },
    { 0x317c654a, SUC_lexicon_tags_2 },
    { 0x2b7c654d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fc6551, SUC_lexicon_tags_17 },
    { 0xef1c6552, SUC_lexicon_tags_27 },
    { 0x739c6552, SUC_lexicon_tags_17 },
    { 0x9dbc6553, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75c655b, SUC_lexicon_tags_61 },
    { 0x61bc655c, SUC_lexicon_tags_7 },
    { 0x33bc655b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7c6560, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68fc6562, SUC_lexicon_tags_185 },
    { 0, NULL },
    { 0, NULL },
    { 0x31fc6565, SUC_lexicon_tags_17 },
    { 0x1a3c6566, SUC_lexicon_tags_12 },
    { 0xa5fc6566, SUC_lexicon_tags_43 },
    { 0xc93c6568, SUC_lexicon_tags_25 },
    { 0x257c6568, SUC_lexicon_tags_27 },
    { 0x513c6568, SUC_lexicon_tags_32 },
    { 0x8f1c656b, SUC_lexicon_tags_12 },
    { 0x75c656c, SUC_lexicon_tags_17 },
    { 0x41dc656c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddc6572, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fc6575, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a1c6577, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3c657a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5abc657c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23c657f, SUC_lexicon_tags_27 },
    { 0x343c6580, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855c6584, SUC_lexicon_tags_2 },
    { 0x8cdc6585, SUC_lexicon_tags_23 },
    { 0xda1c6586, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x53dc6589, SUC_lexicon_tags_25 },
    { 0x7f5c658a, SUC_lexicon_tags_25 },
    { 0x56fc658b, SUC_lexicon_tags_29 },
    { 0x40fc6589, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951c6590, SUC_lexicon_tags_19 },
    { 0x3fbc6591, SUC_lexicon_tags_1 },
    { 0x34dc6591, SUC_lexicon_tags_32 },
    { 0xa41c6592, SUC_lexicon_tags_7 },
    { 0xd73c6594, SUC_lexicon_tags_17 },
    { 0x451c6595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9c6599, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7c659c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dc65a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x833c65a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771c65a8, SUC_lexicon_tags_181 },
    { 0, NULL },
    { 0xc1bc65aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x931c65ac, SUC_lexicon_tags_11 },
    { 0x3c3c65ac, SUC_lexicon_tags_66 },
    { 0x6dbc65ac, SUC_lexicon_tags_11 },
    { 0x9a5c65af, SUC_lexicon_tags_63 },
    { 0x18fc65ad, SUC_lexicon_tags_45 },
    { 0xedc65b1, SUC_lexicon_tags_15 },
    { 0x6d7c65b1, SUC_lexicon_tags_0 },
    { 0x7ebc65b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1c65b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fc65bd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x8c9c65bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3c65c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9c65c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75c65cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x105c65ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2ffc65d0, SUC_lexicon_tags_17 },
    { 0xe35c65d1, SUC_lexicon_tags_55 },
    { 0x387c65d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa5c65d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bc65db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53bc65dd, SUC_lexicon_tags_37 },
    { 0x987c65de, SUC_lexicon_tags_55 },
    { 0x7c9c65df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393c65e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebc65e8, SUC_lexicon_tags_33 },
    { 0x55dc65e9, SUC_lexicon_tags_27 },
    { 0xad5c65e8, SUC_lexicon_tags_16 },
    { 0x313c65eb, SUC_lexicon_tags_12 },
    { 0x633c65ec, SUC_lexicon_tags_8 },
    { 0x761c65eb, SUC_lexicon_tags_23 },
    { 0xc5dc65e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfc65f4, SUC_lexicon_tags_5 },
    { 0x42dc65f5, SUC_lexicon_tags_14 },
    { 0x6a1c65f6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68dc65fa, SUC_lexicon_tags_84 },
    { 0x3ebc65fb, SUC_lexicon_tags_113 },
    { 0xcffc65fc, SUC_lexicon_tags_11 },
    { 0xce1c65fc, SUC_lexicon_tags_0 },
    { 0x693c65fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533c6604, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x82dc6607, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x213c660a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fc660d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x221c6610, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9c6615, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1d1c6617, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73c6622, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83c6625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33c6629, SUC_lexicon_tags_37 },
    { 0x223c662a, SUC_lexicon_tags_38 },
    { 0xc2dc662b, SUC_lexicon_tags_11 },
    { 0x8d9c662b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd1dc662e, SUC_lexicon_tags_40 },
    { 0x36dc662f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x207c6632, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd21c6634, SUC_lexicon_tags_49 },
    { 0x7d3c6634, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x713c6638, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbc663b, SUC_lexicon_tags_2 },
    { 0xa3c663b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27c663e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79c6644, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfc664a, SUC_lexicon_tags_17 },
    { 0x63c664a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9c6650, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1c6654, SUC_lexicon_tags_7 },
    { 0x829c6654, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x599c6658, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe33c665a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809c665e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fc6664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bc6668, SUC_lexicon_tags_0 },
    { 0xdc6668, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdc666c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371c6670, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x57dc6672, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddc6676, SUC_lexicon_tags_19 },
    { 0x31dc6677, SUC_lexicon_tags_1 },
    { 0x613c6678, SUC_lexicon_tags_17 },
    { 0xb51c6679, SUC_lexicon_tags_15 },
    { 0x879c667a, SUC_lexicon_tags_30 },
    { 0x9a3c667b, SUC_lexicon_tags_2 },
    { 0xaddc667a, SUC_lexicon_tags_17 },
    { 0x671c667a, SUC_lexicon_tags_40 },
    { 0x575c667a, SUC_lexicon_tags_35 },
    { 0x31dc667f, SUC_lexicon_tags_2 },
    { 0x637c6676, SUC_lexicon_tags_60 },
    { 0x82dc6678, SUC_lexicon_tags_17 },
    { 0x515c6677, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x55dc6684, SUC_lexicon_tags_652 },
    { 0, NULL },
    { 0xee5c6686, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91c6689, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681c668d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8afc668f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x34bc6691, SUC_lexicon_tags_28 },
    { 0xeb3c6692, SUC_lexicon_tags_8 },
    { 0x27dc6692, SUC_lexicon_tags_15 },
    { 0xbe9c6693, SUC_lexicon_tags_27 },
    { 0xec7c6695, SUC_lexicon_tags_7 },
    { 0xb85c6696, SUC_lexicon_tags_7 },
    { 0x7b1c6697, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdfbc6699, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x299c669c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x44dc669e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebc66a2, SUC_lexicon_tags_49 },
    { 0x439c66a2, SUC_lexicon_tags_17 },
    { 0xcc5c66a4, SUC_lexicon_tags_8 },
    { 0x435c66a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9c66a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5c66ac, SUC_lexicon_tags_57 },
    { 0xa01c66ad, SUC_lexicon_tags_68 },
    { 0x4b1c66ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x115c66b1, SUC_lexicon_tags_37 },
    { 0x71dc66b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343c66b9, SUC_lexicon_tags_2 },
    { 0x1ddc66ba, SUC_lexicon_tags_12 },
    { 0x15fc66ba, SUC_lexicon_tags_25 },
    { 0x57fc66bc, SUC_lexicon_tags_2 },
    { 0xa55c66bc, SUC_lexicon_tags_7 },
    { 0xbfdc66be, SUC_lexicon_tags_15 },
    { 0xd5fc66bf, SUC_lexicon_tags_33 },
    { 0x50bc66c0, SUC_lexicon_tags_37 },
    { 0xde3c66bb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9c66c5, SUC_lexicon_tags_1 },
    { 0x3bfc66c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xce1c66c8, SUC_lexicon_tags_47 },
    { 0x329c66c9, SUC_lexicon_tags_15 },
    { 0xb63c66ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733c66d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5c66d6, SUC_lexicon_tags_45 },
    { 0xdefc66d6, SUC_lexicon_tags_49 },
    { 0x9c9c66d8, SUC_lexicon_tags_60 },
    { 0x339c66d6, SUC_lexicon_tags_11 },
    { 0xbb5c66d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bc66e3, SUC_lexicon_tags_15 },
    { 0x265c66e4, SUC_lexicon_tags_31 },
    { 0xd49c66e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85c66ea, SUC_lexicon_tags_15 },
    { 0x7c7c66eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdc66f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8dfc66f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651c66f8, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xbd3c66fa, SUC_lexicon_tags_12 },
    { 0x34dc66fb, SUC_lexicon_tags_39 },
    { 0x49fc66fb, SUC_lexicon_tags_318 },
    { 0xf83c66fd, SUC_lexicon_tags_29 },
    { 0xee5c66fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x373c6700, SUC_lexicon_tags_11 },
    { 0x6fdc6701, SUC_lexicon_tags_25 },
    { 0x371c6702, SUC_lexicon_tags_12 },
    { 0x683c6701, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x871c6705, SUC_lexicon_tags_17 },
    { 0xf93c6706, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfc670c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf37c670e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fc6711, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7c6718, SUC_lexicon_tags_47 },
    { 0xc39c6719, SUC_lexicon_tags_25 },
    { 0x203c671a, SUC_lexicon_tags_19 },
    { 0x297c6719, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fc6720, SUC_lexicon_tags_25 },
    { 0xeb3c6720, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83c6724, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c672b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdc6732, SUC_lexicon_tags_2 },
    { 0x5c6732, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67c6737, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa01c6739, SUC_lexicon_tags_2 },
    { 0x1bc673a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x287c673c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeefc673f, SUC_lexicon_tags_17 },
    { 0x223c6740, SUC_lexicon_tags_25 },
    { 0xd73c6740, SUC_lexicon_tags_2 },
    { 0x4cdc6740, SUC_lexicon_tags_47 },
    { 0xb4fc6742, SUC_lexicon_tags_82 },
    { 0xef7c6744, SUC_lexicon_tags_0 },
    { 0xd5c673f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dc6749, SUC_lexicon_tags_17 },
    { 0x1e5c6749, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3c674d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79c6755, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffc675a, SUC_lexicon_tags_30 },
    { 0x1c7c675b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fc675f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85c6762, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fc6766, SUC_lexicon_tags_68 },
    { 0xae7c6766, SUC_lexicon_tags_25 },
    { 0x703c6767, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xde5c676a, SUC_lexicon_tags_19 },
    { 0x749c676b, SUC_lexicon_tags_37 },
    { 0x9e5c676b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x305c676f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x447c6771, SUC_lexicon_tags_23 },
    { 0x4cdc6771, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd0dc6774, SUC_lexicon_tags_62 },
    { 0xe07c6774, SUC_lexicon_tags_92 },
    { 0xf6bc6774, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe03c6778, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699c677c, SUC_lexicon_tags_15 },
    { 0xdb9c677d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbc6780, SUC_lexicon_tags_49 },
    { 0x147c6780, SUC_lexicon_tags_2 },
    { 0xef7c6782, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dc6787, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85c678f, SUC_lexicon_tags_47 },
    { 0x3cdc6790, SUC_lexicon_tags_25 },
    { 0xda5c6791, SUC_lexicon_tags_8 },
    { 0x75fc6791, SUC_lexicon_tags_61 },
    { 0x8d5c6792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4bc6795, SUC_lexicon_tags_12 },
    { 0x75c6796, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbdbc6798, SUC_lexicon_tags_12 },
    { 0xc23c6798, SUC_lexicon_tags_126 },
    { 0x615c679a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bc679d, SUC_lexicon_tags_19 },
    { 0x7cdc679e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131c67a2, SUC_lexicon_tags_33 },
    { 0xb5dc67a3, SUC_lexicon_tags_2 },
    { 0xbefc67a4, SUC_lexicon_tags_1 },
    { 0x5e5c67a3, SUC_lexicon_tags_11 },
    { 0xf6bc67a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1c67ad, SUC_lexicon_tags_49 },
    { 0x131c67ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5c67b1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x821c67b3, SUC_lexicon_tags_0 },
    { 0x13c67b4, SUC_lexicon_tags_12 },
    { 0x65c67b5, SUC_lexicon_tags_65 },
    { 0x3f1c67b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3c67b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc67cf, SUC_lexicon_tags_49 },
    { 0x21bc67d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29c67d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x355c67d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529c67e0, SUC_lexicon_tags_30 },
    { 0xabdc67e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459c67e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x183c67e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413c67eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361c67f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbc67f5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fc67f9, SUC_lexicon_tags_107 },
    { 0x509c67f9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb9c67fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x559c67ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x977c6801, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579c6807, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x639c680a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9c680d, SUC_lexicon_tags_37 },
    { 0x903c680e, SUC_lexicon_tags_14 },
    { 0x1a9c680f, SUC_lexicon_tags_17 },
    { 0x79fc6810, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd57c6812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7adc6814, SUC_lexicon_tags_27 },
    { 0xff9c6815, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813c681a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd89c681c, SUC_lexicon_tags_30 },
    { 0x375c681d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdc6820, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79c6825, SUC_lexicon_tags_66 },
    { 0x7ebc6826, SUC_lexicon_tags_9 },
    { 0xdf7c6827, SUC_lexicon_tags_1 },
    { 0xc9dc6827, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ddc682a, SUC_lexicon_tags_28 },
    { 0xb1fc682b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2c1c682d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfc6832, SUC_lexicon_tags_106 },
    { 0xda1c6832, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb03c6835, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x83c6837, SUC_lexicon_tags_15 },
    { 0x973c6838, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84dc683a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9c683f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaffc6841, SUC_lexicon_tags_21 },
    { 0x997c6842, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47c6848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bc684c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fc684f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213c6853, SUC_lexicon_tags_12 },
    { 0x2dfc6854, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9c6857, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efc685a, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dc6860, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bc6863, SUC_lexicon_tags_29 },
    { 0x8f3c6864, SUC_lexicon_tags_79 },
    { 0xd23c6865, SUC_lexicon_tags_17 },
    { 0xcdbc6866, SUC_lexicon_tags_25 },
    { 0x1dfc6866, SUC_lexicon_tags_2 },
    { 0x741c6868, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x707c686a, SUC_lexicon_tags_11 },
    { 0xae9c686b, SUC_lexicon_tags_15 },
    { 0x539c686c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afc686f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x435c6871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28dc6873, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c5c6875, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x435c6877, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xa75c6879, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1c687f, SUC_lexicon_tags_28 },
    { 0xe47c6880, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765c6884, SUC_lexicon_tags_17 },
    { 0x889c6884, SUC_lexicon_tags_12 },
    { 0x68bc6886, SUC_lexicon_tags_17 },
    { 0xf9bc6885, SUC_lexicon_tags_6 },
    { 0xee1c6888, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefc688b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedc6896, SUC_lexicon_tags_1423 },
    { 0x4b7c6896, SUC_lexicon_tags_12 },
    { 0x883c6896, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd6fc689a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3c689d, SUC_lexicon_tags_27 },
    { 0x4f7c689e, SUC_lexicon_tags_37 },
    { 0x98fc689e, SUC_lexicon_tags_31 },
    { 0x6c5c68a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x445c68a3, SUC_lexicon_tags_1 },
    { 0xbc3c68a4, SUC_lexicon_tags_45 },
    { 0xd13c68a5, SUC_lexicon_tags_38 },
    { 0x425c68a5, SUC_lexicon_tags_37 },
    { 0xd95c68a7, SUC_lexicon_tags_59 },
    { 0x393c68a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb07c68aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe75c68ac, SUC_lexicon_tags_45 },
    { 0x59c68ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x13c68b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941c68c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedc68c7, SUC_lexicon_tags_12 },
    { 0x4bbc68c8, SUC_lexicon_tags_47 },
    { 0xf77c68c9, SUC_lexicon_tags_23 },
    { 0xc53c68ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebc68cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbc68d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x653c68d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x737c68d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x621c68d8, SUC_lexicon_tags_47 },
    { 0x731c68d9, SUC_lexicon_tags_1 },
    { 0x6e1c68da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dc68dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13c68e0, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749c68e5, SUC_lexicon_tags_47 },
    { 0xfadc68e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7c68e9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb8bc68eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453c68f2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x91c68f4, SUC_lexicon_tags_11 },
    { 0x273c68f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc68f8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3c68fb, SUC_lexicon_tags_47 },
    { 0x36bc68fb, SUC_lexicon_tags_12 },
    { 0x219c68fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9dbc68ff, SUC_lexicon_tags_15 },
    { 0xd29c6900, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffc6903, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x759c6906, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x673c6909, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1c690d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9c6912, SUC_lexicon_tags_32 },
    { 0x6fc6913, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56bc6915, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451c691a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67c691d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efc6921, SUC_lexicon_tags_1 },
    { 0xedfc6922, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbf5c6924, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3c6927, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x841c6929, SUC_lexicon_tags_12 },
    { 0x335c6929, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9b9c692c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627c6936, SUC_lexicon_tags_17 },
    { 0x753c6937, SUC_lexicon_tags_2 },
    { 0x197c6936, SUC_lexicon_tags_25 },
    { 0x179c6938, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fc693d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1efc693f, SUC_lexicon_tags_9 },
    { 0x11bc6940, SUC_lexicon_tags_133 },
    { 0x14fc6941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bc6946, SUC_lexicon_tags_7 },
    { 0xb41c6946, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85fc6949, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbc694c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x455c694e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bc6951, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fc6958, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913c6961, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x241c6963, SUC_lexicon_tags_11 },
    { 0xacfc6964, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1c6967, SUC_lexicon_tags_29 },
    { 0x7fc6968, SUC_lexicon_tags_25 },
    { 0xd7c6969, SUC_lexicon_tags_33 },
    { 0x911c6969, SUC_lexicon_tags_17 },
    { 0x45c6968, SUC_lexicon_tags_7 },
    { 0x25dc6967, SUC_lexicon_tags_26 },
    { 0xabc696d, SUC_lexicon_tags_17 },
    { 0x137c696d, SUC_lexicon_tags_31 },
    { 0x363c6967, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3c5c6971, SUC_lexicon_tags_56 },
    { 0x97bc6972, SUC_lexicon_tags_2 },
    { 0x6fc6972, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfc697b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fc6980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dc6982, SUC_lexicon_tags_17 },
    { 0x781c6982, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3c6986, SUC_lexicon_tags_21 },
    { 0x4cfc6987, SUC_lexicon_tags_34 },
    { 0xfe5c6986, SUC_lexicon_tags_2 },
    { 0xa03c6989, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193c698e, SUC_lexicon_tags_12 },
    { 0x36dc698e, SUC_lexicon_tags_12 },
    { 0x421c6990, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fc6995, SUC_lexicon_tags_7 },
    { 0x4f3c6996, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239c699c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9c69a2, SUC_lexicon_tags_15 },
    { 0x1dfc69a2, SUC_lexicon_tags_27 },
    { 0xd6bc69a2, SUC_lexicon_tags_26 },
    { 0x98bc69a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb21c69a7, SUC_lexicon_tags_12 },
    { 0x78fc69a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x697c69aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x687c69ad, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bc69b0, SUC_lexicon_tags_14 },
    { 0x2b3c69b1, SUC_lexicon_tags_11 },
    { 0x7a1c69b0, SUC_lexicon_tags_5 },
    { 0x381c69b3, SUC_lexicon_tags_17 },
    { 0x957c69b4, SUC_lexicon_tags_12 },
    { 0xbf9c69b2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727c69bc, SUC_lexicon_tags_11 },
    { 0xb3c69bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbedc69bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfcbc69c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3c69c4, SUC_lexicon_tags_49 },
    { 0x4edc69c4, SUC_lexicon_tags_49 },
    { 0xa8dc69c4, SUC_lexicon_tags_37 },
    { 0x2a3c69c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bc69ca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbc69d0, SUC_lexicon_tags_43 },
    { 0xba1c69d1, SUC_lexicon_tags_15 },
    { 0xa7c69d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5c69d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd19c69da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afc69dd, SUC_lexicon_tags_37 },
    { 0x73bc69dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dc69e1, SUC_lexicon_tags_16 },
    { 0xd9dc69e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa3bc69e4, SUC_lexicon_tags_3 },
    { 0xa9dc69e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c69eb, SUC_lexicon_tags_47 },
    { 0x845c69eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fc69ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x859c69f2, SUC_lexicon_tags_17 },
    { 0x51c69f2, SUC_lexicon_tags_0 },
    { 0xfffc69f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7efc69f6, SUC_lexicon_tags_27 },
    { 0x57c69f7, SUC_lexicon_tags_43 },
    { 0x393c69f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x72bc69fa, SUC_lexicon_tags_8 },
    { 0x577c69fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x831c69fd, SUC_lexicon_tags_65 },
    { 0xd3fc69fe, SUC_lexicon_tags_17 },
    { 0x51c69ff, SUC_lexicon_tags_37 },
    { 0xb63c69fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x807c6a03, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65c6a07, SUC_lexicon_tags_11 },
    { 0x8b7c6a07, SUC_lexicon_tags_14 },
    { 0x177c6a08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55c6a0c, SUC_lexicon_tags_34 },
    { 0xf77c6a0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521c6a19, SUC_lexicon_tags_12 },
    { 0xaabc6a1a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5c6a1d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x3afc6a1f, SUC_lexicon_tags_1 },
    { 0xd1c6a1f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xea7c6a22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fc6a2f, SUC_lexicon_tags_15 },
    { 0x46fc6a30, SUC_lexicon_tags_10 },
    { 0xee7c6a31, SUC_lexicon_tags_1 },
    { 0x731c6a32, SUC_lexicon_tags_37 },
    { 0x55c6a33, SUC_lexicon_tags_7 },
    { 0xcd3c6a34, SUC_lexicon_tags_7 },
    { 0xe9dc6a33, SUC_lexicon_tags_47 },
    { 0x9ddc6a36, SUC_lexicon_tags_33 },
    { 0xb3bc6a37, SUC_lexicon_tags_43 },
    { 0x239c6a38, SUC_lexicon_tags_47 },
    { 0x76bc6a39, SUC_lexicon_tags_12 },
    { 0x6e1c6a36, SUC_lexicon_tags_49 },
    { 0x95c6a3b, SUC_lexicon_tags_53 },
    { 0xabc6a3c, SUC_lexicon_tags_23 },
    { 0x781c6a3c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x25bc6a3f, SUC_lexicon_tags_3 },
    { 0x5ddc6a40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1fc6a42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977c6a46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3c6a4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bc6a4f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bc6a53, SUC_lexicon_tags_25 },
    { 0x971c6a54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d7c6a56, SUC_lexicon_tags_103 },
    { 0x7b9c6a57, SUC_lexicon_tags_28 },
    { 0xdf1c6a57, SUC_lexicon_tags_1 },
    { 0x3e3c6a58, SUC_lexicon_tags_21 },
    { 0xe4dc6a5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75c6a5d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93c6a61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x909c6a63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71bc6a65, SUC_lexicon_tags_10 },
    { 0x585c6a66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x857c6a68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09c6a6e, SUC_lexicon_tags_13 },
    { 0x76dc6a6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd09c6a71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x827c6a74, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5c6a77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfabc6a79, SUC_lexicon_tags_37 },
    { 0xd95c6a7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafc6a7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bc6a81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abc6a88, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81c6a8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13c6a94, SUC_lexicon_tags_23 },
    { 0x875c6a95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4cdc6a97, SUC_lexicon_tags_8 },
    { 0xaf1c6a98, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9c6aa0, SUC_lexicon_tags_43 },
    { 0xd6dc6aa1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1c6aa5, SUC_lexicon_tags_26 },
    { 0xeafc6aa6, SUC_lexicon_tags_37 },
    { 0x563c6aa6, SUC_lexicon_tags_47 },
    { 0x941c6aa6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfe9c6aaa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913c6aaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfc6ab2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc95c6ab4, SUC_lexicon_tags_17 },
    { 0x851c6ab5, SUC_lexicon_tags_35 },
    { 0x657c6ab4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1edc6ab8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65c6abb, SUC_lexicon_tags_7 },
    { 0x841c6abc, SUC_lexicon_tags_21 },
    { 0x5c5c6abd, SUC_lexicon_tags_1 },
    { 0xaa7c6abd, SUC_lexicon_tags_14 },
    { 0x96fc6abe, SUC_lexicon_tags_7 },
    { 0xf65c6ac0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbc6ac5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f1c6ac7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdc6aca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x97bc6acc, SUC_lexicon_tags_37 },
    { 0xa27c6acd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde9c6acf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6fc6ad1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xea7c6ad3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7c6ada, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x241c6add, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x17dc6ae0, SUC_lexicon_tags_37 },
    { 0xb49c6ae1, SUC_lexicon_tags_25 },
    { 0x3d9c6ae0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bc6ae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdc6aec, SUC_lexicon_tags_82 },
    { 0xcbbc6aed, SUC_lexicon_tags_9 },
    { 0x5a3c6aee, SUC_lexicon_tags_40 },
    { 0x65dc6aef, SUC_lexicon_tags_37 },
    { 0xf8bc6af0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103c6af8, SUC_lexicon_tags_49 },
    { 0x645c6af9, SUC_lexicon_tags_21 },
    { 0x6b9c6af8, SUC_lexicon_tags_25 },
    { 0xdf5c6af8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef3c6afd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a3c6aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bc6b03, SUC_lexicon_tags_224 },
    { 0xf23c6b03, SUC_lexicon_tags_3 },
    { 0x627c6b03, SUC_lexicon_tags_14 },
    { 0x15bc6b05, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5c6b0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7c6b11, SUC_lexicon_tags_47 },
    { 0xf03c6b12, SUC_lexicon_tags_37 },
    { 0xb79c6b13, SUC_lexicon_tags_15 },
    { 0x7c6b13, SUC_lexicon_tags_10 },
    { 0x347c6b12, SUC_lexicon_tags_1 },
    { 0x5f9c6b16, SUC_lexicon_tags_37 },
    { 0xf1bc6b13, SUC_lexicon_tags_2 },
    { 0x49c6b16, SUC_lexicon_tags_7 },
    { 0xe93c6b19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137c6b21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafc6b26, SUC_lexicon_tags_1 },
    { 0x2c7c6b26, SUC_lexicon_tags_959 },
    { 0x59dc6b28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7c6b30, SUC_lexicon_tags_12 },
    { 0xccbc6b31, SUC_lexicon_tags_23 },
    { 0x417c6b30, SUC_lexicon_tags_17 },
    { 0x66dc6b32, SUC_lexicon_tags_37 },
    { 0xccfc6b31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc73c6b36, SUC_lexicon_tags_25 },
    { 0x35bc6b36, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bc6b3c, SUC_lexicon_tags_15 },
    { 0x287c6b3d, SUC_lexicon_tags_1 },
    { 0xdcdc6b3e, SUC_lexicon_tags_14 },
    { 0x6e9c6b3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21c6b41, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc6b44, SUC_lexicon_tags_0 },
    { 0xf0dc6b45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfc6b4a, SUC_lexicon_tags_15 },
    { 0x53fc6b4b, SUC_lexicon_tags_47 },
    { 0x8c1c6b4c, SUC_lexicon_tags_613 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5c6b52, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61c6b55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfa7c6b57, SUC_lexicon_tags_8 },
    { 0x8dfc6b58, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77c6b5e, SUC_lexicon_tags_37 },
    { 0xee1c6b5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fc6b66, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5dc6b68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fc6b6b, SUC_lexicon_tags_14 },
    { 0x6ebc6b6c, SUC_lexicon_tags_25 },
    { 0x611c6b6d, SUC_lexicon_tags_14 },
    { 0x63fc6b6e, SUC_lexicon_tags_17 },
    { 0xf39c6b6c, SUC_lexicon_tags_25 },
    { 0xdbdc6b70, SUC_lexicon_tags_15 },
    { 0x345c6b6e, SUC_lexicon_tags_7 },
    { 0x187c6b6f, SUC_lexicon_tags_29 },
    { 0x7dbc6b73, SUC_lexicon_tags_15 },
    { 0x13dc6b6f, SUC_lexicon_tags_12 },
    { 0xe5c6b75, SUC_lexicon_tags_47 },
    { 0x7e5c6b76, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x755c6b78, SUC_lexicon_tags_12 },
    { 0xb11c6b79, SUC_lexicon_tags_61 },
    { 0x677c6b7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1c6b83, SUC_lexicon_tags_23 },
    { 0x165c6b83, SUC_lexicon_tags_37 },
    { 0x6edc6b85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afc6b89, SUC_lexicon_tags_17 },
    { 0xf2dc6b89, SUC_lexicon_tags_63 },
    { 0xbbc6b8b, SUC_lexicon_tags_37 },
    { 0x255c6b8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bc6b91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb2fc6b93, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xd1fc6b95, SUC_lexicon_tags_2 },
    { 0xad5c6b95, SUC_lexicon_tags_68 },
    { 0x673c6b97, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38dc6b9b, SUC_lexicon_tags_25 },
    { 0x961c6b9c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x33fc6b9e, SUC_lexicon_tags_1 },
    { 0xf73c6b9f, SUC_lexicon_tags_7 },
    { 0x8e1c6ba0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c9c6ba2, SUC_lexicon_tags_37 },
    { 0xf1dc6ba3, SUC_lexicon_tags_19 },
    { 0xcc3c6ba4, SUC_lexicon_tags_14 },
    { 0xfd1c6ba3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2bc6ba7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955c6bac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4ebc6bae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x68dc6bb0, SUC_lexicon_tags_53 },
    { 0xcfdc6bb0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bc6bb4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99c6bb7, SUC_lexicon_tags_15 },
    { 0x5cfc6bb7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9c6bbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fc6bc0, SUC_lexicon_tags_49 },
    { 0x8b9c6bc1, SUC_lexicon_tags_14 },
    { 0xd7fc6bc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3c6bc7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe4dc6bc9, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adc6bcc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dc6bd0, SUC_lexicon_tags_23 },
    { 0x15dc6bd1, SUC_lexicon_tags_17 },
    { 0x539c6bd0, SUC_lexicon_tags_19 },
    { 0xb49c6bd3, SUC_lexicon_tags_28 },
    { 0x171c6bd1, SUC_lexicon_tags_38 },
    { 0xf05c6bd5, SUC_lexicon_tags_14 },
    { 0xf51c6bd6, SUC_lexicon_tags_43 },
    { 0x9ffc6bd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73c6bde, SUC_lexicon_tags_14 },
    { 0x28bc6bdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115c6be5, SUC_lexicon_tags_23 },
    { 0x4d5c6be6, SUC_lexicon_tags_37 },
    { 0x1efc6be7, SUC_lexicon_tags_174 },
    { 0xe49c6be7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91dc6bea, SUC_lexicon_tags_47 },
    { 0xca9c6beb, SUC_lexicon_tags_14 },
    { 0xa5bc6beb, SUC_lexicon_tags_11 },
    { 0xbe5c6bed, SUC_lexicon_tags_45 },
    { 0xae7c6beb, SUC_lexicon_tags_25 },
    { 0xf15c6bef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x577c6bf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1c6bf7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141c6bfe, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa35c6c00, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedc6c07, SUC_lexicon_tags_6 },
    { 0x1c9c6c08, SUC_lexicon_tags_17 },
    { 0x2bfc6c07, SUC_lexicon_tags_10 },
    { 0xc63c6c07, SUC_lexicon_tags_38 },
    { 0xf5bc6c0b, SUC_lexicon_tags_11 },
    { 0xa3bc6c0c, SUC_lexicon_tags_37 },
    { 0x46dc6c0d, SUC_lexicon_tags_0 },
    { 0x3ddc6c0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3c6c14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x595c6c16, SUC_lexicon_tags_20 },
    { 0x71dc6c17, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c6c1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x761c6c1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc6c1f, SUC_lexicon_tags_2 },
    { 0x3e5c6c1f, SUC_lexicon_tags_45 },
    { 0xf25c6c1f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5c6c25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x757c6c27, SUC_lexicon_tags_33 },
    { 0x3d3c6c28, SUC_lexicon_tags_11 },
    { 0x259c6c29, SUC_lexicon_tags_37 },
    { 0xf1fc6c2a, SUC_lexicon_tags_2 },
    { 0x115c6c2a, SUC_lexicon_tags_0 },
    { 0x419c6c2c, SUC_lexicon_tags_63 },
    { 0xcbc6c2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26dc6c2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x211c6c31, SUC_lexicon_tags_17 },
    { 0x537c6c32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81c6c36, SUC_lexicon_tags_37 },
    { 0x92dc6c36, SUC_lexicon_tags_66 },
    { 0x991c6c38, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c6c3b, SUC_lexicon_tags_37 },
    { 0xbc9c6c3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x729c6c3e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779c6c46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde9c6c48, SUC_lexicon_tags_11 },
    { 0x499c6c49, SUC_lexicon_tags_17 },
    { 0x4b3c6c4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585c6c50, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbc6c57, SUC_lexicon_tags_7 },
    { 0x597c6c58, SUC_lexicon_tags_31 },
    { 0x8afc6c59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03c6c5c, SUC_lexicon_tags_28 },
    { 0x57dc6c5d, SUC_lexicon_tags_12 },
    { 0x6f1c6c5e, SUC_lexicon_tags_8 },
    { 0x8afc6c5c, SUC_lexicon_tags_19 },
    { 0x885c6c5d, SUC_lexicon_tags_235 },
    { 0xb85c6c61, SUC_lexicon_tags_17 },
    { 0x7fbc6c62, SUC_lexicon_tags_17 },
    { 0xa51c6c5e, SUC_lexicon_tags_27 },
    { 0x563c6c64, SUC_lexicon_tags_30 },
    { 0xbcbc6c61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659c6c69, SUC_lexicon_tags_28 },
    { 0xccbc6c6a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbc6c76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbc6c7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf1c6c7e, SUC_lexicon_tags_4 },
    { 0x5bbc6c7e, SUC_lexicon_tags_30 },
    { 0x1ebc6c7f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bc6c8b, SUC_lexicon_tags_9 },
    { 0x89fc6c8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7cfc6c8e, SUC_lexicon_tags_82 },
    { 0x173c6c8e, SUC_lexicon_tags_25 },
    { 0x639c6c8f, SUC_lexicon_tags_43 },
    { 0xfdc6c91, SUC_lexicon_tags_28 },
    { 0xb01c6c92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e1c6c94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867c6c98, SUC_lexicon_tags_28 },
    { 0xf15c6c99, SUC_lexicon_tags_37 },
    { 0x337c6c9a, SUC_lexicon_tags_17 },
    { 0x5e5c6c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa89c6c9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x505c6ca0, SUC_lexicon_tags_4 },
    { 0xfdfc6ca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x837c6ca3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dc6ca8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b5c6caa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa19c6cac, SUC_lexicon_tags_45 },
    { 0x69c6cad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebc6cb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907c6cb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703c6cc5, SUC_lexicon_tags_27 },
    { 0x6abc6cc6, SUC_lexicon_tags_11 },
    { 0xf49c6cc7, SUC_lexicon_tags_28 },
    { 0xa25c6cc5, SUC_lexicon_tags_53 },
    { 0x98bc6cc9, SUC_lexicon_tags_30 },
    { 0x90bc6cc9, SUC_lexicon_tags_29 },
    { 0x99c6ccb, SUC_lexicon_tags_25 },
    { 0x731c6ccb, SUC_lexicon_tags_37 },
    { 0xc6bc6ccd, SUC_lexicon_tags_45 },
    { 0x9c3c6cce, SUC_lexicon_tags_37 },
    { 0xdbc6cc6, SUC_lexicon_tags_12 },
    { 0xa5bc6ccb, SUC_lexicon_tags_7 },
    { 0xe8bc6cd1, SUC_lexicon_tags_38 },
    { 0xd49c6cd2, SUC_lexicon_tags_63 },
    { 0xd1dc6cd2, SUC_lexicon_tags_12 },
    { 0x261c6cd1, SUC_lexicon_tags_28 },
    { 0x515c6cd5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dc6cd8, SUC_lexicon_tags_39 },
    { 0x687c6cd8, SUC_lexicon_tags_16 },
    { 0xe41c6cda, SUC_lexicon_tags_11 },
    { 0xbc6cd9, SUC_lexicon_tags_1 },
    { 0xebdc6cdc, SUC_lexicon_tags_17 },
    { 0xfdc6cdc, SUC_lexicon_tags_91 },
    { 0xeb1c6cda, SUC_lexicon_tags_25 },
    { 0x7a7c6cdf, SUC_lexicon_tags_4 },
    { 0x461c6cdd, SUC_lexicon_tags_11 },
    { 0x21c6ce1, SUC_lexicon_tags_9 },
    { 0x1adc6ce2, SUC_lexicon_tags_63 },
    { 0x187c6ce1, SUC_lexicon_tags_9 },
    { 0xb39c6ce4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbc6ce7, SUC_lexicon_tags_437 },
    { 0xdb7c6ce8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd3bc6cea, SUC_lexicon_tags_49 },
    { 0xaf5c6ceb, SUC_lexicon_tags_37 },
    { 0xa3dc6cea, SUC_lexicon_tags_9 },
    { 0x975c6cec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bc6cf1, SUC_lexicon_tags_11 },
    { 0x7c5c6cf1, SUC_lexicon_tags_11 },
    { 0xa8bc6cf3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x691c6cf5, SUC_lexicon_tags_27 },
    { 0xc75c6cf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaebc6cf9, SUC_lexicon_tags_25 },
    { 0xa61c6cfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fc6cfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317c6d05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75c6d09, SUC_lexicon_tags_1 },
    { 0x56fc6d09, SUC_lexicon_tags_45 },
    { 0x725c6d0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c6d0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5e1c6d10, SUC_lexicon_tags_66 },
    { 0x63fc6d11, SUC_lexicon_tags_7 },
    { 0x491c6d12, SUC_lexicon_tags_7 },
    { 0x195c6d10, SUC_lexicon_tags_25 },
    { 0x53fc6d14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x917c6d17, SUC_lexicon_tags_19 },
    { 0x4ffc6d18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefdc6d1a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773c6d1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1c6d20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1c6d24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dc6d27, SUC_lexicon_tags_26 },
    { 0xcf7c6d28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05c6d2c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa69c6d2e, SUC_lexicon_tags_12 },
    { 0xc1dc6d2e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa1dc6d31, SUC_lexicon_tags_29 },
    { 0x571c6d32, SUC_lexicon_tags_1 },
    { 0x3adc6d33, SUC_lexicon_tags_12 },
    { 0x97dc6d32, SUC_lexicon_tags_25 },
    { 0x227c6d34, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x53fc6d38, SUC_lexicon_tags_27 },
    { 0x9c9c6d38, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9c6d3c, SUC_lexicon_tags_11 },
    { 0xe43c6d3c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efc6d45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc33c6d47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d7c6d49, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb5c6d4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9c6d51, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1c6d55, SUC_lexicon_tags_7 },
    { 0x3fdc6d55, SUC_lexicon_tags_34 },
    { 0x55c6d57, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadc6d5d, SUC_lexicon_tags_32 },
    { 0x4abc6d5e, SUC_lexicon_tags_64 },
    { 0x78fc6d5e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9c6d62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89c6d66, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05c6d6a, SUC_lexicon_tags_46 },
    { 0xf63c6d6b, SUC_lexicon_tags_38 },
    { 0xad9c6d6c, SUC_lexicon_tags_28 },
    { 0x205c6d6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5c6d77, SUC_lexicon_tags_7 },
    { 0x9ebc6d78, SUC_lexicon_tags_17 },
    { 0x741c6d79, SUC_lexicon_tags_0 },
    { 0x559c6d78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dc6d85, SUC_lexicon_tags_66 },
    { 0x3edc6d86, SUC_lexicon_tags_15 },
    { 0x4f3c6d87, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x31c6d8a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5d9c6d8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895c6d90, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dc6d9d, SUC_lexicon_tags_25 },
    { 0x753c6d9e, SUC_lexicon_tags_17 },
    { 0x45dc6d9f, SUC_lexicon_tags_2 },
    { 0xa3c6d9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9c6da3, SUC_lexicon_tags_60 },
    { 0x4b9c6da4, SUC_lexicon_tags_47 },
    { 0xec1c6da3, SUC_lexicon_tags_12 },
    { 0xf0fc6da6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbc6dac, SUC_lexicon_tags_7 },
    { 0x4abc6dac, SUC_lexicon_tags_68 },
    { 0xed9c6dac, SUC_lexicon_tags_15 },
    { 0x1a3c6daf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4f1c6db1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243c6db6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277c6dbc, SUC_lexicon_tags_17 },
    { 0xaafc6dbd, SUC_lexicon_tags_1 },
    { 0xcc3c6dbd, SUC_lexicon_tags_2 },
    { 0xe59c6dbd, SUC_lexicon_tags_17 },
    { 0x8ffc6dbe, SUC_lexicon_tags_0 },
    { 0xe9c6dbf, SUC_lexicon_tags_0 },
    { 0x371c6dc2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1c6dc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x975c6dc8, SUC_lexicon_tags_38 },
    { 0xe11c6dc9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x127c6dcb, SUC_lexicon_tags_11 },
    { 0x835c6dcc, SUC_lexicon_tags_17 },
    { 0xc91c6dcd, SUC_lexicon_tags_8 },
    { 0xfd1c6dce, SUC_lexicon_tags_15 },
    { 0xa6bc6dcf, SUC_lexicon_tags_17 },
    { 0xe25c6dcf, SUC_lexicon_tags_10 },
    { 0xee7c6dd1, SUC_lexicon_tags_23 },
    { 0x37c6dcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaafc6dd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabc6dd7, SUC_lexicon_tags_25 },
    { 0xc63c6dd8, SUC_lexicon_tags_0 },
    { 0x685c6dd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01c6de0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fc6de7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfc6deb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95c6dee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x275c6df1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xeb7c6df3, SUC_lexicon_tags_0 },
    { 0xfd5c6df4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfc6dfb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dc6e03, SUC_lexicon_tags_21 },
    { 0xb43c6e04, SUC_lexicon_tags_29 },
    { 0x957c6e05, SUC_lexicon_tags_60 },
    { 0x989c6e06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edc6e0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5c6e13, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x59dc6e15, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x98fc6e17, SUC_lexicon_tags_12 },
    { 0xcb1c6e18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdefc6e1a, SUC_lexicon_tags_29 },
    { 0x98bc6e1a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369c6e20, SUC_lexicon_tags_1 },
    { 0x7c6e21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bc6e24, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbc6e29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9c6e2e, SUC_lexicon_tags_47 },
    { 0x6bbc6e2f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289c6e33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abc6e36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9c6e3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc6e44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1c6e47, SUC_lexicon_tags_1 },
    { 0xa4fc6e48, SUC_lexicon_tags_17 },
    { 0xbadc6e49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x475c6e4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797c6e4f, SUC_lexicon_tags_28 },
    { 0x60bc6e50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x56dc6e52, SUC_lexicon_tags_17 },
    { 0x96fc6e52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dc6e57, SUC_lexicon_tags_47 },
    { 0x1b5c6e57, SUC_lexicon_tags_7 },
    { 0xc5fc6e59, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c6e5e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc0bc6e60, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5a3c6e62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x551c6e65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x191c6e67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33c6e69, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47c6e6e, SUC_lexicon_tags_25 },
    { 0xf11c6e6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf59c6e71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fc6e77, SUC_lexicon_tags_33 },
    { 0x84dc6e78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5c6e7e, SUC_lexicon_tags_84 },
    { 0xfbc6e7f, SUC_lexicon_tags_31 },
    { 0x9c3c6e80, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bc6e8d, SUC_lexicon_tags_42 },
    { 0x77bc6e8d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe43c6e90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdefc6e92, SUC_lexicon_tags_11 },
    { 0x8cfc6e93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x983c6e96, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bc6e9a, SUC_lexicon_tags_7 },
    { 0x6adc6e9b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x233c6e9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa9c6e9f, SUC_lexicon_tags_7 },
    { 0x54fc6ea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x17bc6ea3, SUC_lexicon_tags_0 },
    { 0x3e9c6ea4, SUC_lexicon_tags_19 },
    { 0xb4dc6ea5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x497c6ea8, SUC_lexicon_tags_17 },
    { 0x193c6ea9, SUC_lexicon_tags_26 },
    { 0x54fc6eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x379c6ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x541c6eaf, SUC_lexicon_tags_33 },
    { 0x465c6eb0, SUC_lexicon_tags_68 },
    { 0xcc7c6eb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x877c6eb4, SUC_lexicon_tags_45 },
    { 0x971c6eb5, SUC_lexicon_tags_17 },
    { 0x38dc6eb4, SUC_lexicon_tags_7 },
    { 0x7f5c6eb5, SUC_lexicon_tags_0 },
    { 0xa71c6eb8, SUC_lexicon_tags_17 },
    { 0x753c6eb8, SUC_lexicon_tags_17 },
    { 0x9b1c6eb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x593c6ebd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x25dc6ebf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc6ec5, SUC_lexicon_tags_1 },
    { 0xd49c6ec6, SUC_lexicon_tags_1 },
    { 0xcddc6ec7, SUC_lexicon_tags_19 },
    { 0xbc3c6ec8, SUC_lexicon_tags_39 },
    { 0xa97c6ec8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e1c6ecb, SUC_lexicon_tags_9 },
    { 0xe7fc6ecb, SUC_lexicon_tags_33 },
    { 0x24fc6ecb, SUC_lexicon_tags_17 },
    { 0xd67c6ece, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171c6ed4, SUC_lexicon_tags_11 },
    { 0xaf7c6ed4, SUC_lexicon_tags_2 },
    { 0x88bc6ed5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x477c6ed8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5c6edd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abc6ee3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3c6ee7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0fc6ee9, SUC_lexicon_tags_27 },
    { 0x6ddc6ee9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c6eed, SUC_lexicon_tags_30 },
    { 0xa1bc6eee, SUC_lexicon_tags_12 },
    { 0x46dc6eee, SUC_lexicon_tags_27 },
    { 0xa79c6eee, SUC_lexicon_tags_61 },
    { 0x261c6ef1, SUC_lexicon_tags_23 },
    { 0x257c6ef2, SUC_lexicon_tags_21 },
    { 0x95fc6ef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dc6efc, SUC_lexicon_tags_9 },
    { 0x5e7c6efd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8f7c6eff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9c6f04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x471c6f06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7c6f0b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x32fc6f0e, SUC_lexicon_tags_30 },
    { 0xa8bc6f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9c6f13, SUC_lexicon_tags_36 },
    { 0x841c6f13, SUC_lexicon_tags_7 },
    { 0x2e1c6f15, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdc6f19, SUC_lexicon_tags_1 },
    { 0x11fc6f1a, SUC_lexicon_tags_11 },
    { 0x471c6f1b, SUC_lexicon_tags_60 },
    { 0xa9c6f1b, SUC_lexicon_tags_21 },
    { 0xcc5c6f1d, SUC_lexicon_tags_7 },
    { 0x86bc6f1a, SUC_lexicon_tags_14 },
    { 0x44fc6f19, SUC_lexicon_tags_37 },
    { 0xa1fc6f20, SUC_lexicon_tags_47 },
    { 0xdd3c6f20, SUC_lexicon_tags_0 },
    { 0x7dbc6f22, SUC_lexicon_tags_1 },
    { 0xd2dc6f23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf9c6f25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bc6f30, SUC_lexicon_tags_15 },
    { 0x833c6f31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23c6f39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc6f43, SUC_lexicon_tags_11 },
    { 0xbcdc6f43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfc6f4c, SUC_lexicon_tags_17 },
    { 0x1d9c6f4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbc6f54, SUC_lexicon_tags_37 },
    { 0x48bc6f54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bc6f5d, SUC_lexicon_tags_14 },
    { 0xe73c6f5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x989c6f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x78dc6f62, SUC_lexicon_tags_14 },
    { 0xc83c6f63, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x233c6f66, SUC_lexicon_tags_11 },
    { 0xe85c6f67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1c6f6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x441c6f6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dc6f70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fc6f73, SUC_lexicon_tags_33 },
    { 0xc71c6f73, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5c6f77, SUC_lexicon_tags_7 },
    { 0xe19c6f78, SUC_lexicon_tags_37 },
    { 0x699c6f79, SUC_lexicon_tags_25 },
    { 0xe9dc6f78, SUC_lexicon_tags_27 },
    { 0x31bc6f7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c6f84, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2b5c6f86, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443c6f8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1c6f8d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773c6f91, SUC_lexicon_tags_65 },
    { 0xf65c6f91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fc6f96, SUC_lexicon_tags_2 },
    { 0x7fbc6f97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f5c6f99, SUC_lexicon_tags_12 },
    { 0x24bc6f9a, SUC_lexicon_tags_6 },
    { 0xbc9c6f9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa7c6f9d, SUC_lexicon_tags_47 },
    { 0x381c6f9e, SUC_lexicon_tags_1 },
    { 0xad9c6f9f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x661c6fa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fc6fac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5c6faf, SUC_lexicon_tags_27 },
    { 0x977c6faf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa81c6fb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x633c6fb4, SUC_lexicon_tags_61 },
    { 0xd05c6fb4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dc6fbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289c6fc2, SUC_lexicon_tags_23 },
    { 0x2c7c6fc2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x4dc6fc5, SUC_lexicon_tags_14 },
    { 0x5ebc6fc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dc6fcc, SUC_lexicon_tags_33 },
    { 0xb79c6fcd, SUC_lexicon_tags_23 },
    { 0x7f9c6fce, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173c6fd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x157c6fd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173c6fdc, SUC_lexicon_tags_0 },
    { 0xe45c6fdc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fc6fe2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1c6fe5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bc6fe8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x247c6feb, SUC_lexicon_tags_19 },
    { 0x3f1c6feb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbc6fef, SUC_lexicon_tags_92 },
    { 0x963c6ff0, SUC_lexicon_tags_12 },
    { 0x4d9c6ff0, SUC_lexicon_tags_11 },
    { 0x873c6ff2, SUC_lexicon_tags_11 },
    { 0xc21c6ff3, SUC_lexicon_tags_15 },
    { 0xf95c6ff2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91c6ff8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65c7004, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb7dc7006, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443c700a, SUC_lexicon_tags_83 },
    { 0xca3c700a, SUC_lexicon_tags_69 },
    { 0xa81c700c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d3c700e, SUC_lexicon_tags_11 },
    { 0x255c700e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1c7013, SUC_lexicon_tags_37 },
    { 0x231c7014, SUC_lexicon_tags_38 },
    { 0x2dc7014, SUC_lexicon_tags_11 },
    { 0x8f5c7014, SUC_lexicon_tags_37 },
    { 0x59fc7017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3c701b, SUC_lexicon_tags_37 },
    { 0x53fc701b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9c7022, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebc7026, SUC_lexicon_tags_10 },
    { 0x619c7027, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe93c7029, SUC_lexicon_tags_25 },
    { 0x493c702a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe6fc702c, SUC_lexicon_tags_7 },
    { 0xbf3c702d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51c7031, SUC_lexicon_tags_37 },
    { 0x33dc7031, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355c703b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x155c703d, SUC_lexicon_tags_17 },
    { 0x1edc703d, SUC_lexicon_tags_7 },
    { 0x45bc703d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fc704a, SUC_lexicon_tags_45 },
    { 0xca5c704a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe55c704d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745c7056, SUC_lexicon_tags_23 },
    { 0xb99c7056, SUC_lexicon_tags_17 },
    { 0x741c7056, SUC_lexicon_tags_11 },
    { 0x885c7059, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x345c705b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x849c705e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dc7062, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17c7065, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x345c7067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x889c706a, SUC_lexicon_tags_10 },
    { 0xe25c706b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5c7070, SUC_lexicon_tags_28 },
    { 0x97bc7070, SUC_lexicon_tags_47 },
    { 0xce1c7070, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4dc7074, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5dc7076, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dc7079, SUC_lexicon_tags_2 },
    { 0x217c707a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13c7080, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bc7083, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa51c7086, SUC_lexicon_tags_11 },
    { 0xf3c7087, SUC_lexicon_tags_29 },
    { 0xfafc7087, SUC_lexicon_tags_49 },
    { 0x999c7086, SUC_lexicon_tags_17 },
    { 0x615c708a, SUC_lexicon_tags_26 },
    { 0x97fc7086, SUC_lexicon_tags_37 },
    { 0x6a7c708a, SUC_lexicon_tags_2 },
    { 0x14dc708d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765c7091, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5c7094, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913c709b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbc70a2, SUC_lexicon_tags_12 },
    { 0x969c70a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3c70a7, SUC_lexicon_tags_2 },
    { 0xd59c70a8, SUC_lexicon_tags_12 },
    { 0x8a5c70a8, SUC_lexicon_tags_25 },
    { 0xf8bc70a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89fc70ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87c70af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa07c70b1, SUC_lexicon_tags_17 },
    { 0x131c70b2, SUC_lexicon_tags_15 },
    { 0xbefc70b3, SUC_lexicon_tags_7 },
    { 0xc73c70b1, SUC_lexicon_tags_3 },
    { 0x5f7c70b5, SUC_lexicon_tags_49 },
    { 0xadfc70b1, SUC_lexicon_tags_12 },
    { 0x715c70b6, SUC_lexicon_tags_1 },
    { 0x849c70b8, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bc70bc, SUC_lexicon_tags_15 },
    { 0x447c70bd, SUC_lexicon_tags_7 },
    { 0x4b3c70bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x841c70c0, SUC_lexicon_tags_27 },
    { 0x3bdc70c1, SUC_lexicon_tags_38 },
    { 0x38fc70c2, SUC_lexicon_tags_25 },
    { 0xa4fc70c3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fc70c7, SUC_lexicon_tags_7 },
    { 0x1e7c70c7, SUC_lexicon_tags_7 },
    { 0x3e9c70c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889c70cd, SUC_lexicon_tags_7 },
    { 0x5d3c70ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1c70d1, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bc70d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdbc70d6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebc70dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc70e1, SUC_lexicon_tags_9 },
    { 0x28bc70e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747c70e6, SUC_lexicon_tags_39 },
    { 0xfb1c70e7, SUC_lexicon_tags_11 },
    { 0x5d5c70e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x65bc70ea, SUC_lexicon_tags_11 },
    { 0xa3fc70ea, SUC_lexicon_tags_36 },
    { 0xcd3c70ec, SUC_lexicon_tags_17 },
    { 0x201c70eb, SUC_lexicon_tags_2 },
    { 0x4c5c70ee, SUC_lexicon_tags_7 },
    { 0x803c70ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547c70f3, SUC_lexicon_tags_38 },
    { 0xe89c70f4, SUC_lexicon_tags_14 },
    { 0xb27c70f4, SUC_lexicon_tags_262 },
    { 0xd6fc70f6, SUC_lexicon_tags_14 },
    { 0xb5bc70f5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x321c70fa, SUC_lexicon_tags_14 },
    { 0x4bbc70fb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x40bc70fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1c7101, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeebc7103, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bc7108, SUC_lexicon_tags_31 },
    { 0x12fc7109, SUC_lexicon_tags_1 },
    { 0x2c9c7109, SUC_lexicon_tags_68 },
    { 0x35bc7109, SUC_lexicon_tags_148 },
    { 0xe6dc710c, SUC_lexicon_tags_11 },
    { 0x8c3c710b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e1c710f, SUC_lexicon_tags_42 },
    { 0xcddc7110, SUC_lexicon_tags_45 },
    { 0xb3dc7110, SUC_lexicon_tags_31 },
    { 0x28dc7112, SUC_lexicon_tags_11 },
    { 0x7cfc7113, SUC_lexicon_tags_17 },
    { 0xaf5c7114, SUC_lexicon_tags_1 },
    { 0x527c7115, SUC_lexicon_tags_38 },
    { 0x9c5c7116, SUC_lexicon_tags_68 },
    { 0x9bfc7117, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35c711c, SUC_lexicon_tags_29 },
    { 0x2e1c711d, SUC_lexicon_tags_48 },
    { 0xe5fc711d, SUC_lexicon_tags_5 },
    { 0x877c711d, SUC_lexicon_tags_8 },
    { 0xb8fc711e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685c7124, SUC_lexicon_tags_33 },
    { 0xd6dc7125, SUC_lexicon_tags_1 },
    { 0x145c7124, SUC_lexicon_tags_17 },
    { 0xebbc7127, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x16fc7129, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x799c712c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x537c712f, SUC_lexicon_tags_30 },
    { 0xa99c712f, SUC_lexicon_tags_60 },
    { 0xb71c712f, SUC_lexicon_tags_14 },
    { 0x2e1c7132, SUC_lexicon_tags_23 },
    { 0xfa3c7131, SUC_lexicon_tags_49 },
    { 0xe1c7130, SUC_lexicon_tags_17 },
    { 0x65bc7135, SUC_lexicon_tags_25 },
    { 0x6f9c7136, SUC_lexicon_tags_15 },
    { 0x39bc7137, SUC_lexicon_tags_25 },
    { 0xa6dc7138, SUC_lexicon_tags_25 },
    { 0xa43c7139, SUC_lexicon_tags_15 },
    { 0x5c9c7138, SUC_lexicon_tags_30 },
    { 0xae9c713b, SUC_lexicon_tags_13 },
    { 0xc55c713c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7c713f, SUC_lexicon_tags_38 },
    { 0xee5c713f, SUC_lexicon_tags_21 },
    { 0xde5c7141, SUC_lexicon_tags_33 },
    { 0xf17c7140, SUC_lexicon_tags_17 },
    { 0xe27c7140, SUC_lexicon_tags_70 },
    { 0xa31c7144, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfbc7146, SUC_lexicon_tags_19 },
    { 0x4d7c7147, SUC_lexicon_tags_37 },
    { 0x40dc7148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361c714f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b1c7151, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x813c7154, SUC_lexicon_tags_23 },
    { 0x11fc7155, SUC_lexicon_tags_35 },
    { 0x27c7156, SUC_lexicon_tags_23 },
    { 0x103c7157, SUC_lexicon_tags_26 },
    { 0x333c7158, SUC_lexicon_tags_25 },
    { 0xd7dc7159, SUC_lexicon_tags_0 },
    { 0x449c7155, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251c715f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5c7162, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8e5c7164, SUC_lexicon_tags_15 },
    { 0x8f7c7165, SUC_lexicon_tags_9 },
    { 0xfdbc7166, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x24fc7168, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c716c, SUC_lexicon_tags_12 },
    { 0x81dc716d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x281c716f, SUC_lexicon_tags_230 },
    { 0xa5fc7170, SUC_lexicon_tags_30 },
    { 0x173c7171, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0x863c7173, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c7176, SUC_lexicon_tags_12 },
    { 0x5f5c7176, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17c717a, SUC_lexicon_tags_7 },
    { 0x81dc717a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x493c717d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91c7183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf87c7185, SUC_lexicon_tags_198 },
    { 0x8d5c7186, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dc7189, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fc718d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x767c7190, SUC_lexicon_tags_31 },
    { 0xaddc7190, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7c7195, SUC_lexicon_tags_1 },
    { 0x6c7c7195, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf0bc7198, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8bc719a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa77c719c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277c71a0, SUC_lexicon_tags_38 },
    { 0xb0fc71a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd1bc71a3, SUC_lexicon_tags_113 },
    { 0x3dc71a3, SUC_lexicon_tags_33 },
    { 0x70fc71a5, SUC_lexicon_tags_17 },
    { 0x293c71a4, SUC_lexicon_tags_47 },
    { 0x99bc71a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x981c71a9, SUC_lexicon_tags_30 },
    { 0x941c71aa, SUC_lexicon_tags_3 },
    { 0xb1c71aa, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17c71b2, SUC_lexicon_tags_46 },
    { 0x277c71b3, SUC_lexicon_tags_12 },
    { 0x9a9c71b4, SUC_lexicon_tags_43 },
    { 0x51c71b5, SUC_lexicon_tags_13 },
    { 0x285c71b3, SUC_lexicon_tags_9 },
    { 0xefc71b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x33fc71b9, SUC_lexicon_tags_44 },
    { 0xc41c71b9, SUC_lexicon_tags_17 },
    { 0x6edc71bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc97c71bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5c71c0, SUC_lexicon_tags_46 },
    { 0xd19c71c1, SUC_lexicon_tags_28 },
    { 0xacbc71c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdc71c6, SUC_lexicon_tags_12 },
    { 0xc7dc71c7, SUC_lexicon_tags_46 },
    { 0xa7fc71c8, SUC_lexicon_tags_39 },
    { 0x2c1c71c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7c71ce, SUC_lexicon_tags_25 },
    { 0x66fc71cf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85c71d2, SUC_lexicon_tags_28 },
    { 0x5e9c71d3, SUC_lexicon_tags_33 },
    { 0xe33c71d3, SUC_lexicon_tags_37 },
    { 0xe45c71d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bc71d9, SUC_lexicon_tags_25 },
    { 0x407c71da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x315c71dc, SUC_lexicon_tags_25 },
    { 0x18bc71dd, SUC_lexicon_tags_15 },
    { 0xdfdc71de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb0fc71e0, SUC_lexicon_tags_23 },
    { 0x3d3c71e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x635c71e3, SUC_lexicon_tags_6 },
    { 0x233c71e4, SUC_lexicon_tags_47 },
    { 0x7e5c71e5, SUC_lexicon_tags_4 },
    { 0xde1c71e6, SUC_lexicon_tags_25 },
    { 0x32bc71e7, SUC_lexicon_tags_9 },
    { 0xd8fc71e8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dc71eb, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbc71f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7c71f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761c71fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fc7200, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa41c7202, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909c7209, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc720c, SUC_lexicon_tags_2 },
    { 0x301c720c, SUC_lexicon_tags_7 },
    { 0xa67c720e, SUC_lexicon_tags_30 },
    { 0x485c720e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e1c7211, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x745c7213, SUC_lexicon_tags_15 },
    { 0x4adc7213, SUC_lexicon_tags_5 },
    { 0x60dc7215, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c721e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897c7222, SUC_lexicon_tags_47 },
    { 0x16bc7223, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bc7226, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab3c7228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fc722d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781c7231, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c7234, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49dc7236, SUC_lexicon_tags_30 },
    { 0xf29c7236, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717c723f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81c7246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fc7248, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc73c724a, SUC_lexicon_tags_63 },
    { 0x575c724b, SUC_lexicon_tags_15 },
    { 0x409c724a, SUC_lexicon_tags_33 },
    { 0xdd3c724d, SUC_lexicon_tags_15 },
    { 0x2bdc724e, SUC_lexicon_tags_25 },
    { 0xe23c724e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x839c7251, SUC_lexicon_tags_23 },
    { 0xd8bc7252, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35c7257, SUC_lexicon_tags_25 },
    { 0x2a3c7258, SUC_lexicon_tags_60 },
    { 0x879c7259, SUC_lexicon_tags_61 },
    { 0xb53c7259, SUC_lexicon_tags_9 },
    { 0x353c7258, SUC_lexicon_tags_47 },
    { 0xbefc7257, SUC_lexicon_tags_13 },
    { 0x451c725b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3c7262, SUC_lexicon_tags_12 },
    { 0x9c5c7263, SUC_lexicon_tags_7 },
    { 0xda7c7262, SUC_lexicon_tags_17 },
    { 0x999c7265, SUC_lexicon_tags_55 },
    { 0xad5c7265, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7c7269, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45c726e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287c7275, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bc7279, SUC_lexicon_tags_30 },
    { 0xe75c7279, SUC_lexicon_tags_41 },
    { 0xec7c727a, SUC_lexicon_tags_32 },
    { 0x78fc7279, SUC_lexicon_tags_1 },
    { 0xc99c727b, SUC_lexicon_tags_18 },
    { 0x5dbc727e, SUC_lexicon_tags_37 },
    { 0x4cfc727e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c7283, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3c7287, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3c728a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45c7290, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51c7294, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fc7297, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe37c7299, SUC_lexicon_tags_37 },
    { 0x6e3c729a, SUC_lexicon_tags_23 },
    { 0x415c7299, SUC_lexicon_tags_37 },
    { 0x163c7299, SUC_lexicon_tags_47 },
    { 0x465c7299, SUC_lexicon_tags_60 },
    { 0xb0bc729b, SUC_lexicon_tags_11 },
    { 0x7c1c729f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xed7c72a1, SUC_lexicon_tags_4 },
    { 0x953c72a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fc72a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb9c72aa, SUC_lexicon_tags_2 },
    { 0x2a7c72ab, SUC_lexicon_tags_27 },
    { 0x6e9c72aa, SUC_lexicon_tags_9 },
    { 0xd79c72ac, SUC_lexicon_tags_11 },
    { 0x54fc72ae, SUC_lexicon_tags_26 },
    { 0x92fc72ab, SUC_lexicon_tags_29 },
    { 0x43c72af, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xc3bc72b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdc72b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbc72b9, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bc72bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c72bf, SUC_lexicon_tags_9 },
    { 0x223c72c0, SUC_lexicon_tags_23 },
    { 0x499c72c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7fc72c3, SUC_lexicon_tags_11 },
    { 0x513c72c4, SUC_lexicon_tags_11 },
    { 0x403c72c5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7c72c8, SUC_lexicon_tags_17 },
    { 0x35fc72c9, SUC_lexicon_tags_15 },
    { 0xcb1c72ca, SUC_lexicon_tags_17 },
    { 0x47dc72cb, SUC_lexicon_tags_7 },
    { 0x897c72cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd13c72ce, SUC_lexicon_tags_11 },
    { 0xadc72ce, SUC_lexicon_tags_38 },
    { 0x819c72ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x723c72d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897c72d7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6e5c72d9, SUC_lexicon_tags_45 },
    { 0x21c72d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20dc72dc, SUC_lexicon_tags_17 },
    { 0x17fc72dd, SUC_lexicon_tags_10 },
    { 0x5bbc72dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb15c72e0, SUC_lexicon_tags_30 },
    { 0x6b5c72e1, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fc72e4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99c72e8, SUC_lexicon_tags_7 },
    { 0x567c72e9, SUC_lexicon_tags_19 },
    { 0xeefc72e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779c72f2, SUC_lexicon_tags_63 },
    { 0x3cfc72f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dc7301, SUC_lexicon_tags_82 },
    { 0x221c7302, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3c7305, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96dc7308, SUC_lexicon_tags_26 },
    { 0x6bbc7308, SUC_lexicon_tags_9 },
    { 0x505c730a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bc730d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d7c730f, SUC_lexicon_tags_8 },
    { 0x7e3c7310, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dc7313, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x903c7315, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x811c7318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56bc731a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x66bc731c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fc731f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bc7328, SUC_lexicon_tags_25 },
    { 0xc75c7329, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x573c732c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x799c732f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7c7332, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5c7336, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7c733a, SUC_lexicon_tags_12 },
    { 0x60bc733b, SUC_lexicon_tags_43 },
    { 0x3c3c733c, SUC_lexicon_tags_45 },
    { 0xf5c733a, SUC_lexicon_tags_92 },
    { 0x40dc733e, SUC_lexicon_tags_11 },
    { 0x287c733f, SUC_lexicon_tags_14 },
    { 0xf63c7340, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861c7345, SUC_lexicon_tags_7 },
    { 0xe7dc7346, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x379c7348, SUC_lexicon_tags_25 },
    { 0xc27c7349, SUC_lexicon_tags_7 },
    { 0xa4fc734a, SUC_lexicon_tags_66 },
    { 0x219c734b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71c734f, SUC_lexicon_tags_40 },
    { 0xc4dc7350, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89c7355, SUC_lexicon_tags_11 },
    { 0x195c7356, SUC_lexicon_tags_11 },
    { 0xd47c7357, SUC_lexicon_tags_6 },
    { 0xa79c7355, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8f5c735a, SUC_lexicon_tags_25 },
    { 0x4c9c735a, SUC_lexicon_tags_25 },
    { 0x863c735c, SUC_lexicon_tags_47 },
    { 0x3fdc735d, SUC_lexicon_tags_12 },
    { 0x6cbc735d, SUC_lexicon_tags_7 },
    { 0x425c735f, SUC_lexicon_tags_1 },
    { 0x6efc7360, SUC_lexicon_tags_40 },
    { 0xfc7361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdc7364, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fdc7366, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd61c7368, SUC_lexicon_tags_25 },
    { 0x5fc7369, SUC_lexicon_tags_0 },
    { 0x891c736a, SUC_lexicon_tags_27 },
    { 0x99dc7369, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x127c736d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1dc736f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dc7378, SUC_lexicon_tags_47 },
    { 0x843c7378, SUC_lexicon_tags_1 },
    { 0x157c737a, SUC_lexicon_tags_25 },
    { 0xb1fc737b, SUC_lexicon_tags_0 },
    { 0x8bbc737b, SUC_lexicon_tags_7 },
    { 0xa51c737d, SUC_lexicon_tags_19 },
    { 0x1ebc737d, SUC_lexicon_tags_2 },
    { 0xcd3c737f, SUC_lexicon_tags_26 },
    { 0xc8bc737f, SUC_lexicon_tags_7 },
    { 0x4d3c737a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5c7385, SUC_lexicon_tags_17 },
    { 0xbd1c7385, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9c738c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627c7391, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1c7394, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bc7397, SUC_lexicon_tags_19 },
    { 0x7c3c7398, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x995c739b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x42dc739d, SUC_lexicon_tags_23 },
    { 0x5b9c739d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dc73a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c73a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4adc73aa, SUC_lexicon_tags_46 },
    { 0x9a5c73aa, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xfd9c73ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91c73b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dc73b5, SUC_lexicon_tags_0 },
    { 0xdb1c73b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fc73bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505c73bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43c73c3, SUC_lexicon_tags_25 },
    { 0xb41c73c4, SUC_lexicon_tags_17 },
    { 0x3e9c73c5, SUC_lexicon_tags_32 },
    { 0x8b1c73c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9c73cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119c73d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923c73d5, SUC_lexicon_tags_45 },
    { 0xb43c73d6, SUC_lexicon_tags_12 },
    { 0x41bc73d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7c73da, SUC_lexicon_tags_2 },
    { 0x413c73db, SUC_lexicon_tags_37 },
    { 0x77bc73dc, SUC_lexicon_tags_39 },
    { 0x747c73dc, SUC_lexicon_tags_19 },
    { 0xd93c73dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79c73e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7c73e9, SUC_lexicon_tags_11 },
    { 0x18fc73ea, SUC_lexicon_tags_0 },
    { 0x56bc73eb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9c73f1, SUC_lexicon_tags_49 },
    { 0x233c73f2, SUC_lexicon_tags_2 },
    { 0xf61c73f3, SUC_lexicon_tags_60 },
    { 0x3d9c73f4, SUC_lexicon_tags_37 },
    { 0xafbc73f4, SUC_lexicon_tags_35 },
    { 0xe07c73f2, SUC_lexicon_tags_66 },
    { 0x5e7c73f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09c73fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x139c7400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbc7405, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fc740d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf7c740f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x517c7411, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x981c7413, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb51c7415, SUC_lexicon_tags_12 },
    { 0x42dc7416, SUC_lexicon_tags_19 },
    { 0x39bc7416, SUC_lexicon_tags_37 },
    { 0x1ebc7418, SUC_lexicon_tags_7 },
    { 0x2dc7417, SUC_lexicon_tags_47 },
    { 0xa57c7417, SUC_lexicon_tags_13 },
    { 0xa2bc7418, SUC_lexicon_tags_45 },
    { 0x701c7417, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x761c741e, SUC_lexicon_tags_12 },
    { 0x231c741e, SUC_lexicon_tags_37 },
    { 0x60fc7420, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35c7423, SUC_lexicon_tags_38 },
    { 0x7f1c7424, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xec3c7426, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x541c7428, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915c742c, SUC_lexicon_tags_7 },
    { 0x4d5c742c, SUC_lexicon_tags_174 },
    { 0xbf1c742d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x675c7431, SUC_lexicon_tags_37 },
    { 0xef9c7432, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7c743c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x1ffc743e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c5c7440, SUC_lexicon_tags_45 },
    { 0x869c7441, SUC_lexicon_tags_31 },
    { 0x8d1c7440, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319c7446, SUC_lexicon_tags_2 },
    { 0x96bc7447, SUC_lexicon_tags_37 },
    { 0x2e9c7448, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fc744b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdc7453, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271c7459, SUC_lexicon_tags_37 },
    { 0x2abc7459, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917c7461, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5ffc7463, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81c7467, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x325c746a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7d1c746c, SUC_lexicon_tags_49 },
    { 0xdd5c746c, SUC_lexicon_tags_0 },
    { 0x3f9c746c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf3c7470, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357c7478, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97c747c, SUC_lexicon_tags_53 },
    { 0xa69c747d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x51bc747f, SUC_lexicon_tags_17 },
    { 0x8ffc747f, SUC_lexicon_tags_14 },
    { 0x477c747f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7c7485, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01c7489, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe31c748b, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357c7491, SUC_lexicon_tags_14 },
    { 0x4e1c7492, SUC_lexicon_tags_390 },
    { 0x1c7491, SUC_lexicon_tags_25 },
    { 0x80bc7493, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdc7497, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5c7499, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x879c749b, SUC_lexicon_tags_19 },
    { 0x8b5c749c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9dc749e, SUC_lexicon_tags_23 },
    { 0x8f1c749f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fc74a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x11dc74a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bc74a9, SUC_lexicon_tags_129 },
    { 0x24dc74aa, SUC_lexicon_tags_15 },
    { 0xb0dc74ab, SUC_lexicon_tags_23 },
    { 0x3d1c74aa, SUC_lexicon_tags_17 },
    { 0x18bc74ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905c74b1, SUC_lexicon_tags_47 },
    { 0xa1bc74b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fc74b5, SUC_lexicon_tags_10 },
    { 0x183c74b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa5dc74b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89c74bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bbc74be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc53c74c0, SUC_lexicon_tags_45 },
    { 0x11dc74c0, SUC_lexicon_tags_47 },
    { 0xf45c74c0, SUC_lexicon_tags_1 },
    { 0x133c74c2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe39c74c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x88fc74c7, SUC_lexicon_tags_14 },
    { 0x4f3c74c8, SUC_lexicon_tags_33 },
    { 0x367c74c8, SUC_lexicon_tags_65 },
    { 0x241c74ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43c74cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bc74d2, SUC_lexicon_tags_17 },
    { 0xeb1c74d3, SUC_lexicon_tags_61 },
    { 0xbd5c74d4, SUC_lexicon_tags_7 },
    { 0x419c74d5, SUC_lexicon_tags_43 },
    { 0xd4fc74d6, SUC_lexicon_tags_12 },
    { 0xa37c74d7, SUC_lexicon_tags_1 },
    { 0x655c74d8, SUC_lexicon_tags_27 },
    { 0x7f7c74d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fc74df, SUC_lexicon_tags_26 },
    { 0xd73c74e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe09c74e2, SUC_lexicon_tags_33 },
    { 0xf4bc74e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac1c74e5, SUC_lexicon_tags_21 },
    { 0x1bfc74e6, SUC_lexicon_tags_11 },
    { 0x15c74e7, SUC_lexicon_tags_46 },
    { 0xfbfc74e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbc74eb, SUC_lexicon_tags_46 },
    { 0xb6dc74ec, SUC_lexicon_tags_33 },
    { 0x851c74ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcf9c74ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5c74f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97c74f8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5c74fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67c7503, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x369c7505, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x58bc7507, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xe13c7509, SUC_lexicon_tags_28 },
    { 0x411c750a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x2a1c750c, SUC_lexicon_tags_12 },
    { 0xd69c750d, SUC_lexicon_tags_12 },
    { 0x495c750e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2bc7510, SUC_lexicon_tags_2 },
    { 0x8adc7511, SUC_lexicon_tags_19 },
    { 0x4ffc7511, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bc7518, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801c7522, SUC_lexicon_tags_14 },
    { 0x267c7522, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ebc7525, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x25dc7527, SUC_lexicon_tags_25 },
    { 0xe33c7527, SUC_lexicon_tags_37 },
    { 0xdd5c7528, SUC_lexicon_tags_1 },
    { 0x369c7529, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcebc752c, SUC_lexicon_tags_7 },
    { 0x6e7c752d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7c7534, SUC_lexicon_tags_7 },
    { 0xf13c7535, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95c753c, SUC_lexicon_tags_2 },
    { 0x2ebc753c, SUC_lexicon_tags_7 },
    { 0xd21c753e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69c7541, SUC_lexicon_tags_54 },
    { 0xce1c7542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533c7547, SUC_lexicon_tags_7 },
    { 0xd87c7548, SUC_lexicon_tags_8 },
    { 0x3bbc7549, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x833c754c, SUC_lexicon_tags_47 },
    { 0xb49c754d, SUC_lexicon_tags_13 },
    { 0x607c754e, SUC_lexicon_tags_27 },
    { 0xe9fc754d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc7555, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde3c7559, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x695c755b, SUC_lexicon_tags_2 },
    { 0x971c755b, SUC_lexicon_tags_34 },
    { 0xf03c755d, SUC_lexicon_tags_4 },
    { 0x3d3c755d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafc7562, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dc7566, SUC_lexicon_tags_59 },
    { 0xb91c7567, SUC_lexicon_tags_16 },
    { 0x68bc7568, SUC_lexicon_tags_17 },
    { 0x817c7569, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325c756d, SUC_lexicon_tags_25 },
    { 0x1b3c756e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfc7574, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x439c7576, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5c7579, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8abc757b, SUC_lexicon_tags_11 },
    { 0x717c757c, SUC_lexicon_tags_27 },
    { 0x14fc757b, SUC_lexicon_tags_47 },
    { 0x2d1c757c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353c7582, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5c7585, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffc7589, SUC_lexicon_tags_21 },
    { 0x197c758a, SUC_lexicon_tags_23 },
    { 0xc6bc758a, SUC_lexicon_tags_12 },
    { 0xf25c758c, SUC_lexicon_tags_15 },
    { 0x6fdc758b, SUC_lexicon_tags_60 },
    { 0xd5c758d, SUC_lexicon_tags_48 },
    { 0x6e9c758f, SUC_lexicon_tags_17 },
    { 0xefbc7590, SUC_lexicon_tags_9 },
    { 0xaf9c7591, SUC_lexicon_tags_1 },
    { 0xed7c758a, SUC_lexicon_tags_45 },
    { 0x507c7589, SUC_lexicon_tags_12 },
    { 0x5ddc7589, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x215c7596, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a5c7598, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43fc759a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddc75a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9c75a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbbc75a5, SUC_lexicon_tags_37 },
    { 0xea9c75a5, SUC_lexicon_tags_33 },
    { 0x6bfc75a7, SUC_lexicon_tags_11 },
    { 0xe2fc75a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1d1c75aa, SUC_lexicon_tags_25 },
    { 0x451c75aa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9c75ae, SUC_lexicon_tags_37 },
    { 0xa0dc75ae, SUC_lexicon_tags_15 },
    { 0xb73c75b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7c75b3, SUC_lexicon_tags_1 },
    { 0xe2dc75b4, SUC_lexicon_tags_17 },
    { 0x13bc75b4, SUC_lexicon_tags_0 },
    { 0x639c75b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5dc75b8, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x181c75bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafc75c0, SUC_lexicon_tags_25 },
    { 0x167c75c1, SUC_lexicon_tags_29 },
    { 0xd1c75c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29fc75c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d7c75c8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x331c75cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efc75d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfc75d4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x729c75d6, SUC_lexicon_tags_1 },
    { 0xc83c75d7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dc75db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abc75de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9c75e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1c75eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dc75ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b5c75f0, SUC_lexicon_tags_12 },
    { 0xf57c75f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e9c75f3, SUC_lexicon_tags_32 },
    { 0x50dc75f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bc75f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9c75fa, SUC_lexicon_tags_12 },
    { 0x95c75fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x721c75fd, SUC_lexicon_tags_37 },
    { 0x6abc75fe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589c7604, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9fc7606, SUC_lexicon_tags_56 },
    { 0x241c7606, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5c760a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x357c760c, SUC_lexicon_tags_8 },
    { 0xfd7c760d, SUC_lexicon_tags_7 },
    { 0x6d7c760c, SUC_lexicon_tags_10 },
    { 0xfffc760f, SUC_lexicon_tags_25 },
    { 0xbadc7610, SUC_lexicon_tags_2 },
    { 0x995c7611, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125c7619, SUC_lexicon_tags_2 },
    { 0xd9dc7619, SUC_lexicon_tags_7 },
    { 0x953c761b, SUC_lexicon_tags_454 },
    { 0xa29c761b, SUC_lexicon_tags_14 },
    { 0xe33c761d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7c7625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x675c7628, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dc762b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3c7632, SUC_lexicon_tags_23 },
    { 0xb51c7632, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafc7637, SUC_lexicon_tags_25 },
    { 0xad7c7638, SUC_lexicon_tags_15 },
    { 0x65bc7639, SUC_lexicon_tags_10 },
    { 0xbf7c763a, SUC_lexicon_tags_38 },
    { 0x417c763b, SUC_lexicon_tags_14 },
    { 0x425c763a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd33c763e, SUC_lexicon_tags_15 },
    { 0xd39c763f, SUC_lexicon_tags_12 },
    { 0xb0bc7640, SUC_lexicon_tags_31 },
    { 0xf97c7640, SUC_lexicon_tags_12 },
    { 0x661c7642, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31c7650, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdc7653, SUC_lexicon_tags_45 },
    { 0xd31c7654, SUC_lexicon_tags_7 },
    { 0xae1c7654, SUC_lexicon_tags_14 },
    { 0x819c7655, SUC_lexicon_tags_14 },
    { 0xbb5c7657, SUC_lexicon_tags_2 },
    { 0x66bc7656, SUC_lexicon_tags_17 },
    { 0x827c7654, SUC_lexicon_tags_28 },
    { 0xa51c7657, SUC_lexicon_tags_1 },
    { 0x219c765b, SUC_lexicon_tags_17 },
    { 0x661c7659, SUC_lexicon_tags_2 },
    { 0xc35c765c, SUC_lexicon_tags_12 },
    { 0xf3dc765d, SUC_lexicon_tags_37 },
    { 0x449c765f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27c7663, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x119c7665, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299c7669, SUC_lexicon_tags_27 },
    { 0xcf1c766a, SUC_lexicon_tags_25 },
    { 0xf35c766a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x323c766d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd83c766f, SUC_lexicon_tags_45 },
    { 0x8dbc7670, SUC_lexicon_tags_40 },
    { 0x77c7670, SUC_lexicon_tags_2 },
    { 0x9e5c7672, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bc7678, SUC_lexicon_tags_11 },
    { 0x5a1c7678, SUC_lexicon_tags_6 },
    { 0xf61c767a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19c767d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1b3c767f, SUC_lexicon_tags_7 },
    { 0x3cdc767f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273c768a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187c768f, SUC_lexicon_tags_12 },
    { 0x487c7690, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55c7693, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1c7696, SUC_lexicon_tags_1 },
    { 0x2abc7697, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddbc7699, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bc769c, SUC_lexicon_tags_63 },
    { 0xe29c769d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbcfc769f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x729c76a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345c76a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8adc76ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x371c76ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffc76b1, SUC_lexicon_tags_33 },
    { 0x781c76b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdfc76b4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x799c76b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a5c76b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bc76bc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135c76c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43c76ca, SUC_lexicon_tags_25 },
    { 0xb71c76cb, SUC_lexicon_tags_38 },
    { 0x29fc76ca, SUC_lexicon_tags_37 },
    { 0xfbbc76cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x757c76cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedc76d4, SUC_lexicon_tags_0 },
    { 0xb7dc76d5, SUC_lexicon_tags_1 },
    { 0x14dc76d5, SUC_lexicon_tags_11 },
    { 0x927c76d6, SUC_lexicon_tags_6 },
    { 0x8ddc76d8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bc76e4, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x823c76e6, SUC_lexicon_tags_25 },
    { 0x2dfc76e7, SUC_lexicon_tags_7 },
    { 0x283c76e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dc76f1, SUC_lexicon_tags_17 },
    { 0xcc7c76f1, SUC_lexicon_tags_2 },
    { 0x997c76f3, SUC_lexicon_tags_37 },
    { 0x6f1c76f4, SUC_lexicon_tags_10 },
    { 0x359c76f5, SUC_lexicon_tags_37 },
    { 0xbe7c76f4, SUC_lexicon_tags_37 },
    { 0x797c76f7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfc76fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1c7701, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99c7708, SUC_lexicon_tags_11 },
    { 0xcafc7709, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbc770c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fc7712, SUC_lexicon_tags_25 },
    { 0x81bc7712, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x775c7715, SUC_lexicon_tags_25 },
    { 0x9ffc7716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x101c7718, SUC_lexicon_tags_57 },
    { 0x617c7719, SUC_lexicon_tags_38 },
    { 0x31c771a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59c771e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bc7721, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbbdc7723, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381c7729, SUC_lexicon_tags_29 },
    { 0xe97c772a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x827c772c, SUC_lexicon_tags_17 },
    { 0x427c772c, SUC_lexicon_tags_28 },
    { 0xfedc772c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x449c7731, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fc7734, SUC_lexicon_tags_15 },
    { 0xc15c7735, SUC_lexicon_tags_45 },
    { 0x3a5c7736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bc774b, SUC_lexicon_tags_2 },
    { 0x403c774c, SUC_lexicon_tags_43 },
    { 0x1a9c774d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77dc774f, SUC_lexicon_tags_37 },
    { 0xdcbc7750, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56fc7752, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27c7758, SUC_lexicon_tags_55 },
    { 0xffdc7759, SUC_lexicon_tags_2 },
    { 0x989c775a, SUC_lexicon_tags_12 },
    { 0xfb5c775a, SUC_lexicon_tags_2 },
    { 0x29dc775a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc7766, SUC_lexicon_tags_17 },
    { 0xa51c7767, SUC_lexicon_tags_10 },
    { 0x5f3c7768, SUC_lexicon_tags_42 },
    { 0x89c7768, SUC_lexicon_tags_17 },
    { 0x427c776a, SUC_lexicon_tags_4 },
    { 0x5b7c776b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859c7771, SUC_lexicon_tags_17 },
    { 0xb39c7772, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa4dc7774, SUC_lexicon_tags_230 },
    { 0x6f9c7774, SUC_lexicon_tags_30 },
    { 0x981c7776, SUC_lexicon_tags_60 },
    { 0x5d9c7776, SUC_lexicon_tags_37 },
    { 0xf5c7777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5c777d, SUC_lexicon_tags_2 },
    { 0xfc1c777e, SUC_lexicon_tags_11 },
    { 0x93dc777f, SUC_lexicon_tags_44 },
    { 0x8e9c7780, SUC_lexicon_tags_11 },
    { 0xb95c777e, SUC_lexicon_tags_11 },
    { 0xd77c777e, SUC_lexicon_tags_37 },
    { 0x1b3c7783, SUC_lexicon_tags_1 },
    { 0xcc7c7783, SUC_lexicon_tags_37 },
    { 0x897c7785, SUC_lexicon_tags_12 },
    { 0xa93c7786, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dc778b, SUC_lexicon_tags_32 },
    { 0xb6dc778c, SUC_lexicon_tags_37 },
    { 0x175c778c, SUC_lexicon_tags_49 },
    { 0x533c778e, SUC_lexicon_tags_11 },
    { 0xa65c778f, SUC_lexicon_tags_15 },
    { 0x17c778c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbc779b, SUC_lexicon_tags_47 },
    { 0xae9c779b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49c779f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c77a3, SUC_lexicon_tags_39 },
    { 0x1afc77a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615c77a8, SUC_lexicon_tags_54 },
    { 0xfcfc77a9, SUC_lexicon_tags_2 },
    { 0xb67c77aa, SUC_lexicon_tags_39 },
    { 0x2efc77a9, SUC_lexicon_tags_9 },
    { 0xa57c77a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bc77b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe55c77b3, SUC_lexicon_tags_27 },
    { 0xaffc77b4, SUC_lexicon_tags_7 },
    { 0x131c77b5, SUC_lexicon_tags_32 },
    { 0x289c77b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17c77bb, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x92bc77bd, SUC_lexicon_tags_92 },
    { 0xee3c77be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1c1c77c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5cdc77c2, SUC_lexicon_tags_12 },
    { 0x5dc77c3, SUC_lexicon_tags_11 },
    { 0x1b7c77c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3edc77c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdc77c9, SUC_lexicon_tags_39 },
    { 0x1a9c77ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23c77ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x829c77d0, SUC_lexicon_tags_25 },
    { 0xb73c77d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41bc77d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03c77d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ddc77d8, SUC_lexicon_tags_14 },
    { 0x2bc77d9, SUC_lexicon_tags_72 },
    { 0x381c77d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc77e1, SUC_lexicon_tags_55 },
    { 0x1ffc77e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fc77e7, SUC_lexicon_tags_53 },
    { 0x339c77e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7fc77ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dc77ef, SUC_lexicon_tags_25 },
    { 0x6abc77f0, SUC_lexicon_tags_1 },
    { 0x7d3c77ef, SUC_lexicon_tags_17 },
    { 0xc45c77f0, SUC_lexicon_tags_22 },
    { 0x77bc77f3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x747c77f5, SUC_lexicon_tags_26 },
    { 0x32bc77f5, SUC_lexicon_tags_45 },
    { 0xccdc77f6, SUC_lexicon_tags_2 },
    { 0x7dbc77f5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x695c77fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3c7802, SUC_lexicon_tags_14 },
    { 0xe71c7802, SUC_lexicon_tags_11 },
    { 0xdedc7804, SUC_lexicon_tags_1 },
    { 0x80fc7804, SUC_lexicon_tags_0 },
    { 0x7b1c7805, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe39c7808, SUC_lexicon_tags_1 },
    { 0x9d9c7809, SUC_lexicon_tags_29 },
    { 0xf13c780a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1adc780c, SUC_lexicon_tags_1 },
    { 0xa63c780d, SUC_lexicon_tags_7 },
    { 0x3fc780d, SUC_lexicon_tags_16 },
    { 0xee5c780d, SUC_lexicon_tags_1 },
    { 0xa1c780d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3c7813, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x197c7816, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bc781b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dc7820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dc7823, SUC_lexicon_tags_4 },
    { 0x18dc7824, SUC_lexicon_tags_25 },
    { 0xe15c7825, SUC_lexicon_tags_15 },
    { 0xf19c7826, SUC_lexicon_tags_17 },
    { 0xb77c7826, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b7c7829, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x495c782c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87c7831, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x417c7833, SUC_lexicon_tags_28 },
    { 0x62bc7834, SUC_lexicon_tags_1 },
    { 0x991c7835, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643c783a, SUC_lexicon_tags_43 },
    { 0xe11c783b, SUC_lexicon_tags_5 },
    { 0xdfdc783c, SUC_lexicon_tags_17 },
    { 0x899c783d, SUC_lexicon_tags_10 },
    { 0x2e5c783e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfd5c7840, SUC_lexicon_tags_204 },
    { 0xde3c7841, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3c7848, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc784e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93c7851, SUC_lexicon_tags_55 },
    { 0x9d9c7852, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323c7857, SUC_lexicon_tags_1 },
    { 0x8f9c7858, SUC_lexicon_tags_124 },
    { 0xba7c7859, SUC_lexicon_tags_63 },
    { 0x6fc7857, SUC_lexicon_tags_12 },
    { 0xd7c785b, SUC_lexicon_tags_15 },
    { 0x787c785b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x339c785f, SUC_lexicon_tags_87 },
    { 0xcdc7860, SUC_lexicon_tags_402 },
    { 0x11dc7861, SUC_lexicon_tags_19 },
    { 0x56bc7860, SUC_lexicon_tags_55 },
    { 0xb97c7863, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebc7866, SUC_lexicon_tags_11 },
    { 0x315c7866, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x26fc7869, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2fc786b, SUC_lexicon_tags_7 },
    { 0xc75c786c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x895c786e, SUC_lexicon_tags_71 },
    { 0xeb5c786f, SUC_lexicon_tags_2 },
    { 0xdc3c7870, SUC_lexicon_tags_17 },
    { 0x515c7871, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c7876, SUC_lexicon_tags_25 },
    { 0x9dfc7877, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xedc7879, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd0bc787b, SUC_lexicon_tags_2 },
    { 0xd2bc787c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fc787f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x127c7881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae9c7883, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9c7886, SUC_lexicon_tags_15 },
    { 0xf29c7887, SUC_lexicon_tags_7 },
    { 0x9edc7888, SUC_lexicon_tags_15 },
    { 0x74dc7889, SUC_lexicon_tags_97 },
    { 0x6e5c788a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c788e, SUC_lexicon_tags_11 },
    { 0xbd1c788f, SUC_lexicon_tags_16 },
    { 0x9b7c7890, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50bc7892, SUC_lexicon_tags_17 },
    { 0xb25c7893, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x979c7895, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9c7899, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x259c789b, SUC_lexicon_tags_33 },
    { 0xcbbc789c, SUC_lexicon_tags_0 },
    { 0x6e1c789d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21c78a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe5bc78a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x601c78a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fc78ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89c78ae, SUC_lexicon_tags_25 },
    { 0x2fdc78af, SUC_lexicon_tags_25 },
    { 0xd7bc78b0, SUC_lexicon_tags_0 },
    { 0x54bc78b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7c78b7, SUC_lexicon_tags_0 },
    { 0xa13c78b8, SUC_lexicon_tags_43 },
    { 0x6f7c78b8, SUC_lexicon_tags_15 },
    { 0xd85c78b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x25dc78bd, SUC_lexicon_tags_10 },
    { 0x665c78be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fc78c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c78d1, SUC_lexicon_tags_25 },
    { 0xe91c78d2, SUC_lexicon_tags_38 },
    { 0x179c78d2, SUC_lexicon_tags_1 },
    { 0x4a3c78d4, SUC_lexicon_tags_14 },
    { 0xf35c78d4, SUC_lexicon_tags_37 },
    { 0x12dc78d6, SUC_lexicon_tags_30 },
    { 0xfe9c78d4, SUC_lexicon_tags_28 },
    { 0x2e5c78d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433c78dc, SUC_lexicon_tags_4 },
    { 0x803c78dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dc78e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45c78e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadc78e9, SUC_lexicon_tags_45 },
    { 0xa9bc78ea, SUC_lexicon_tags_1 },
    { 0x25c78eb, SUC_lexicon_tags_35 },
    { 0xfbdc78ec, SUC_lexicon_tags_25 },
    { 0x697c78ed, SUC_lexicon_tags_1 },
    { 0x1b3c78e9, SUC_lexicon_tags_29 },
    { 0x20bc78eb, SUC_lexicon_tags_66 },
    { 0x17c78f0, SUC_lexicon_tags_12 },
    { 0xc2dc78f1, SUC_lexicon_tags_15 },
    { 0x991c78f2, SUC_lexicon_tags_21 },
    { 0x5ebc78f0, SUC_lexicon_tags_29 },
    { 0x34fc78f4, SUC_lexicon_tags_17 },
    { 0xa45c78ee, SUC_lexicon_tags_27 },
    { 0x47dc78f6, SUC_lexicon_tags_7 },
    { 0x687c78e9, SUC_lexicon_tags_30 },
    { 0xc79c78f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03c78fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x297c78fe, SUC_lexicon_tags_7 },
    { 0x661c78ff, SUC_lexicon_tags_66 },
    { 0x871c78ff, SUC_lexicon_tags_12 },
    { 0x273c7901, SUC_lexicon_tags_12 },
    { 0x37c7902, SUC_lexicon_tags_17 },
    { 0x8fbc7900, SUC_lexicon_tags_91 },
    { 0xe7fc7903, SUC_lexicon_tags_15 },
    { 0xe85c7902, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x609c7908, SUC_lexicon_tags_15 },
    { 0xc45c7908, SUC_lexicon_tags_25 },
    { 0xdfdc790a, SUC_lexicon_tags_11 },
    { 0x9fbc7909, SUC_lexicon_tags_32 },
    { 0xa0fc790c, SUC_lexicon_tags_91 },
    { 0x8cfc7908, SUC_lexicon_tags_21 },
    { 0xe63c790a, SUC_lexicon_tags_25 },
    { 0x119c7908, SUC_lexicon_tags_15 },
    { 0x401c7910, SUC_lexicon_tags_1 },
    { 0x201c7910, SUC_lexicon_tags_7 },
    { 0x537c7912, SUC_lexicon_tags_10 },
    { 0x12fc7912, SUC_lexicon_tags_26 },
    { 0x4abc790e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963c7918, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63c791e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x345c7920, SUC_lexicon_tags_15 },
    { 0xf23c7921, SUC_lexicon_tags_25 },
    { 0x99bc7920, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x999c7925, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bc7929, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bc7935, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9c793a, SUC_lexicon_tags_6 },
    { 0xc0dc793a, SUC_lexicon_tags_1 },
    { 0xf89c793b, SUC_lexicon_tags_1 },
    { 0xa7bc793a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bc7940, SUC_lexicon_tags_1113 },
    { 0xaadc7940, SUC_lexicon_tags_37 },
    { 0x961c7941, SUC_lexicon_tags_14 },
    { 0x75bc7943, SUC_lexicon_tags_7 },
    { 0x2b1c7944, SUC_lexicon_tags_25 },
    { 0x2a1c7945, SUC_lexicon_tags_14 },
    { 0xae1c7943, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x915c7948, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3c794e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41c7952, SUC_lexicon_tags_62 },
    { 0x805c7953, SUC_lexicon_tags_34 },
    { 0x99bc7954, SUC_lexicon_tags_7 },
    { 0xd8dc7955, SUC_lexicon_tags_7 },
    { 0x841c7954, SUC_lexicon_tags_11 },
    { 0x19fc7953, SUC_lexicon_tags_15 },
    { 0xfbfc7958, SUC_lexicon_tags_1 },
    { 0xc5bc7959, SUC_lexicon_tags_48 },
    { 0x369c7954, SUC_lexicon_tags_29 },
    { 0xa6fc7958, SUC_lexicon_tags_19 },
    { 0x96fc795a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x39c795f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3e7c7961, SUC_lexicon_tags_63 },
    { 0x89fc7962, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1c7964, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x89c7967, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bc796b, SUC_lexicon_tags_25 },
    { 0x5dc796c, SUC_lexicon_tags_0 },
    { 0x99dc796d, SUC_lexicon_tags_43 },
    { 0x195c796b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x385c7970, SUC_lexicon_tags_2 },
    { 0x96dc7970, SUC_lexicon_tags_12 },
    { 0x2d7c7970, SUC_lexicon_tags_12 },
    { 0xf81c7973, SUC_lexicon_tags_39 },
    { 0xdc7974, SUC_lexicon_tags_8 },
    { 0xe65c7975, SUC_lexicon_tags_37 },
    { 0x21bc7975, SUC_lexicon_tags_25 },
    { 0x8dbc7976, SUC_lexicon_tags_15 },
    { 0xabbc7977, SUC_lexicon_tags_37 },
    { 0xda7c7976, SUC_lexicon_tags_12 },
    { 0xfa9c7971, SUC_lexicon_tags_37 },
    { 0x881c797b, SUC_lexicon_tags_15 },
    { 0x6bbc797b, SUC_lexicon_tags_12 },
    { 0xaa9c7970, SUC_lexicon_tags_23 },
    { 0x1b7c797e, SUC_lexicon_tags_9 },
    { 0x7bc797f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5c7988, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9c798b, SUC_lexicon_tags_37 },
    { 0x975c798c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1c798f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927c7993, SUC_lexicon_tags_13 },
    { 0x21fc7994, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69c7998, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bc799f, SUC_lexicon_tags_63 },
    { 0x50fc79a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x41fc79a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2a3c79a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbdbc79a6, SUC_lexicon_tags_12 },
    { 0x405c79a7, SUC_lexicon_tags_29 },
    { 0xfb9c79a7, SUC_lexicon_tags_15 },
    { 0x44dc79a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33c79ac, SUC_lexicon_tags_10 },
    { 0xa1bc79ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5c79b1, SUC_lexicon_tags_11 },
    { 0x5c5c79b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119c79bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbc79bf, SUC_lexicon_tags_140 },
    { 0xb89c79c0, SUC_lexicon_tags_9 },
    { 0x447c79c0, SUC_lexicon_tags_33 },
    { 0xa2dc79c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41fc79c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5c79c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbc79cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13c79ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x875c79d0, SUC_lexicon_tags_33 },
    { 0x889c79d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa5dc79d3, SUC_lexicon_tags_30 },
    { 0xd75c79d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53c79d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadc79dd, SUC_lexicon_tags_17 },
    { 0x383c79de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x14dc79e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x821c79e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdc79e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911c79ee, SUC_lexicon_tags_88 },
    { 0xc61c79ee, SUC_lexicon_tags_37 },
    { 0xed3c79f0, SUC_lexicon_tags_17 },
    { 0xc6bc79f1, SUC_lexicon_tags_17 },
    { 0xf37c79f2, SUC_lexicon_tags_27 },
    { 0xa37c79f3, SUC_lexicon_tags_30 },
    { 0x1afc79f2, SUC_lexicon_tags_12 },
    { 0xe33c79f3, SUC_lexicon_tags_60 },
    { 0xea3c79f0, SUC_lexicon_tags_7 },
    { 0x603c79f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bc7a00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafc7a05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7c7a0a, SUC_lexicon_tags_1 },
    { 0x601c7a0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf9dc7a0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a5c7a0f, SUC_lexicon_tags_12 },
    { 0x3dfc7a10, SUC_lexicon_tags_84 },
    { 0xc2dc7a11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39c7a19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93c7a1d, SUC_lexicon_tags_45 },
    { 0x9dbc7a1e, SUC_lexicon_tags_25 },
    { 0x285c7a1f, SUC_lexicon_tags_91 },
    { 0xe07c7a1e, SUC_lexicon_tags_23 },
    { 0xc21c7a21, SUC_lexicon_tags_26 },
    { 0x451c7a1d, SUC_lexicon_tags_29 },
    { 0x79dc7a23, SUC_lexicon_tags_27 },
    { 0xf99c7a23, SUC_lexicon_tags_17 },
    { 0x9e1c7a25, SUC_lexicon_tags_65 },
    { 0xc0fc7a26, SUC_lexicon_tags_7 },
    { 0xb95c7a1f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x71c7a29, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xfbdc7a2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x795c7a2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfc7a30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x639c7a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbc7a34, SUC_lexicon_tags_25 },
    { 0x84dc7a34, SUC_lexicon_tags_44 },
    { 0x2e3c7a36, SUC_lexicon_tags_25 },
    { 0x191c7a34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd23c7a39, SUC_lexicon_tags_38 },
    { 0x69bc7a39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87c7a40, SUC_lexicon_tags_19 },
    { 0x943c7a41, SUC_lexicon_tags_46 },
    { 0x30bc7a41, SUC_lexicon_tags_28 },
    { 0x827c7a43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x21fc7a45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bc7a49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd0fc7a4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdc7a4e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95c7a50, SUC_lexicon_tags_21 },
    { 0xbf3c7a51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c7a58, SUC_lexicon_tags_23 },
    { 0x855c7a59, SUC_lexicon_tags_47 },
    { 0x6b1c7a5a, SUC_lexicon_tags_25 },
    { 0x819c7a5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x203c7a5e, SUC_lexicon_tags_28 },
    { 0x72fc7a5f, SUC_lexicon_tags_11 },
    { 0x155c7a60, SUC_lexicon_tags_37 },
    { 0x6b5c7a61, SUC_lexicon_tags_60 },
    { 0xb81c7a60, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x235c7a64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x945c7a66, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5c7a69, SUC_lexicon_tags_39 },
    { 0x5dbc7a6a, SUC_lexicon_tags_6 },
    { 0x8efc7a69, SUC_lexicon_tags_39 },
    { 0xe7bc7a6c, SUC_lexicon_tags_33 },
    { 0x985c7a6c, SUC_lexicon_tags_25 },
    { 0x603c7a6c, SUC_lexicon_tags_45 },
    { 0xe1dc7a6f, SUC_lexicon_tags_37 },
    { 0xaf1c7a6f, SUC_lexicon_tags_17 },
    { 0x16dc7a70, SUC_lexicon_tags_31 },
    { 0xbe1c7a72, SUC_lexicon_tags_2 },
    { 0x335c7a69, SUC_lexicon_tags_1 },
    { 0x477c7a74, SUC_lexicon_tags_6 },
    { 0x57bc7a74, SUC_lexicon_tags_34 },
    { 0x335c7a74, SUC_lexicon_tags_25 },
    { 0x813c7a77, SUC_lexicon_tags_31 },
    { 0xb1dc7a78, SUC_lexicon_tags_15 },
    { 0x37c7a79, SUC_lexicon_tags_32 },
    { 0x325c7a7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe65c7a7c, SUC_lexicon_tags_10 },
    { 0x279c7a7d, SUC_lexicon_tags_7 },
    { 0x3efc7a7e, SUC_lexicon_tags_34 },
    { 0xaa5c7a7d, SUC_lexicon_tags_11 },
    { 0x77fc7a80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x467c7a82, SUC_lexicon_tags_14 },
    { 0x269c7a83, SUC_lexicon_tags_2 },
    { 0x349c7a83, SUC_lexicon_tags_25 },
    { 0xfa9c7a83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475c7a8a, SUC_lexicon_tags_1 },
    { 0xf1bc7a8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbc7a93, SUC_lexicon_tags_12 },
    { 0x653c7a94, SUC_lexicon_tags_37 },
    { 0x54dc7a95, SUC_lexicon_tags_33 },
    { 0x333c7a95, SUC_lexicon_tags_41 },
    { 0xdabc7a97, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41c7a9a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc7a9f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbbfc7aa1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bc7aa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x457c7aab, SUC_lexicon_tags_30 },
    { 0x675c7aac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78fc7aae, SUC_lexicon_tags_16 },
    { 0x581c7aae, SUC_lexicon_tags_12 },
    { 0x25dc7ab0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdc7ab6, SUC_lexicon_tags_38 },
    { 0xe75c7ab7, SUC_lexicon_tags_925 },
    { 0xc3c7ab8, SUC_lexicon_tags_18 },
    { 0x2b7c7ab6, SUC_lexicon_tags_2 },
    { 0xb17c7aba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x655c7abd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5c7ac2, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85c7ac6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bc7ac9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x541c7acb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7c7ace, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efc7ad3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9c7ad9, SUC_lexicon_tags_1 },
    { 0xaa1c7ada, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269c7ade, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbc7ae0, SUC_lexicon_tags_57 },
    { 0xeadc7ae1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1c7ae4, SUC_lexicon_tags_1 },
    { 0xd75c7ae4, SUC_lexicon_tags_2 },
    { 0xe59c7ae6, SUC_lexicon_tags_6 },
    { 0x68bc7ae4, SUC_lexicon_tags_46 },
    { 0x29bc7ae8, SUC_lexicon_tags_25 },
    { 0xcbdc7ae9, SUC_lexicon_tags_29 },
    { 0x403c7aea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcffc7aec, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4fc7aee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39c7af5, SUC_lexicon_tags_17 },
    { 0xa19c7af6, SUC_lexicon_tags_1 },
    { 0x581c7af7, SUC_lexicon_tags_25 },
    { 0xa67c7af5, SUC_lexicon_tags_37 },
    { 0x677c7af9, SUC_lexicon_tags_63 },
    { 0x8a7c7af9, SUC_lexicon_tags_49 },
    { 0xe8fc7afb, SUC_lexicon_tags_31 },
    { 0x405c7afc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35dc7b03, SUC_lexicon_tags_23 },
    { 0xaddc7b04, SUC_lexicon_tags_1 },
    { 0xc37c7b05, SUC_lexicon_tags_47 },
    { 0xe7c7b06, SUC_lexicon_tags_2 },
    { 0x771c7b07, SUC_lexicon_tags_9 },
    { 0xd5dc7b08, SUC_lexicon_tags_16 },
    { 0xfe5c7b07, SUC_lexicon_tags_17 },
    { 0x5e7c7b0a, SUC_lexicon_tags_33 },
    { 0x49c7b0b, SUC_lexicon_tags_24 },
    { 0x9bc7b06, SUC_lexicon_tags_8 },
    { 0x79dc7b0c, SUC_lexicon_tags_29 },
    { 0xb59c7b04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69bc7b10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x60fc7b12, SUC_lexicon_tags_33 },
    { 0x69bc7b13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x675c7b15, SUC_lexicon_tags_40 },
    { 0x25fc7b16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf69c7b18, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dc7b1d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdc7b20, SUC_lexicon_tags_47 },
    { 0xd3c7b21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7c7b24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9c7b29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x889c7b2c, SUC_lexicon_tags_12 },
    { 0xca5c7b2c, SUC_lexicon_tags_15 },
    { 0x4f5c7b2e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x535c7b30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99fc7b33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x117c7b36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5c7b3d, SUC_lexicon_tags_10 },
    { 0xb91c7b3e, SUC_lexicon_tags_28 },
    { 0x8adc7b3d, SUC_lexicon_tags_15 },
    { 0x3b5c7b40, SUC_lexicon_tags_60 },
    { 0x253c7b3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fc7b45, SUC_lexicon_tags_11 },
    { 0x94dc7b45, SUC_lexicon_tags_72 },
    { 0x4d7c7b45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c7b4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635c7b4f, SUC_lexicon_tags_7 },
    { 0x2d7c7b4f, SUC_lexicon_tags_17 },
    { 0x4b7c7b4f, SUC_lexicon_tags_49 },
    { 0x505c7b51, SUC_lexicon_tags_30 },
    { 0x2afc7b53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9c7b57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e5c7b59, SUC_lexicon_tags_17 },
    { 0x981c7b5a, SUC_lexicon_tags_9 },
    { 0x55bc7b5b, SUC_lexicon_tags_46 },
    { 0x877c7b5c, SUC_lexicon_tags_68 },
    { 0x61bc7b5d, SUC_lexicon_tags_29 },
    { 0xb97c7b5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dc7b62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4cbc7b64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x783c7b67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb91c7b69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59c7b73, SUC_lexicon_tags_47 },
    { 0xe05c7b74, SUC_lexicon_tags_9 },
    { 0x75fc7b74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbc7b78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa7c7b7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x163c7b7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73c7b82, SUC_lexicon_tags_30 },
    { 0x367c7b83, SUC_lexicon_tags_34 },
    { 0x2d9c7b84, SUC_lexicon_tags_11 },
    { 0x4dc7b85, SUC_lexicon_tags_10 },
    { 0xe9dc7b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243c7b8a, SUC_lexicon_tags_37 },
    { 0x4f7c7b8b, SUC_lexicon_tags_0 },
    { 0x9ebc7b8c, SUC_lexicon_tags_63 },
    { 0x997c7b8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x557c7b8f, SUC_lexicon_tags_47 },
    { 0x3c5c7b90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x559c7b93, SUC_lexicon_tags_15 },
    { 0xd27c7b94, SUC_lexicon_tags_27 },
    { 0x7a7c7b95, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167c7b99, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525c7b9e, SUC_lexicon_tags_26 },
    { 0xb73c7b9f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9c7ba5, SUC_lexicon_tags_37 },
    { 0x9ddc7ba6, SUC_lexicon_tags_37 },
    { 0x407c7ba7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3c7bab, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xca1c7bad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fc7bb0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc7bb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755c7bb8, SUC_lexicon_tags_38 },
    { 0xdb7c7bb8, SUC_lexicon_tags_170 },
    { 0x51c7bb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b3c7bbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebc7bbf, SUC_lexicon_tags_0 },
    { 0x563c7bc0, SUC_lexicon_tags_82 },
    { 0xd11c7bc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79bc7bc3, SUC_lexicon_tags_30 },
    { 0x4fdc7bc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabc7bca, SUC_lexicon_tags_13 },
    { 0xa31c7bca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x63dc7bce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd0bc7bd0, SUC_lexicon_tags_37 },
    { 0x5c9c7bd1, SUC_lexicon_tags_1 },
    { 0x95bc7bd2, SUC_lexicon_tags_1 },
    { 0x571c7bd3, SUC_lexicon_tags_7 },
    { 0xeb7c7bd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9c7bda, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53c7bdf, SUC_lexicon_tags_11 },
    { 0x821c7be0, SUC_lexicon_tags_7 },
    { 0x101c7bdf, SUC_lexicon_tags_2 },
    { 0x535c7be0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5c7be9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc01c7beb, SUC_lexicon_tags_12 },
    { 0x665c7bec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x711c7bee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x141c7bf1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61c7bf4, SUC_lexicon_tags_39 },
    { 0xa91c7bf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc7bfa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7c7bfe, SUC_lexicon_tags_23 },
    { 0x69bc7bff, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfc7c04, SUC_lexicon_tags_25 },
    { 0xefdc7c04, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x633c7c07, SUC_lexicon_tags_37 },
    { 0x257c7c07, SUC_lexicon_tags_15 },
    { 0xa49c7c08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5c7c0f, SUC_lexicon_tags_27 },
    { 0x637c7c10, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bc7c16, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3c7c19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x531c7c1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39c7c22, SUC_lexicon_tags_161 },
    { 0xcedc7c23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257c7c2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3c7c31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x40fc7c33, SUC_lexicon_tags_8 },
    { 0x9bdc7c33, SUC_lexicon_tags_37 },
    { 0x1dbc7c35, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1c7c3d, SUC_lexicon_tags_11 },
    { 0xadfc7c3d, SUC_lexicon_tags_25 },
    { 0x627c7c3f, SUC_lexicon_tags_60 },
    { 0x3ffc7c3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9c3c7c42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3c7c48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95c7c4d, SUC_lexicon_tags_55 },
    { 0xf75c7c4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52fc7c51, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23c7c55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35c7c58, SUC_lexicon_tags_1 },
    { 0x157c7c59, SUC_lexicon_tags_0 },
    { 0x375c7c58, SUC_lexicon_tags_17 },
    { 0x7e7c7c5a, SUC_lexicon_tags_11 },
    { 0x41dc7c5c, SUC_lexicon_tags_23 },
    { 0x779c7c5d, SUC_lexicon_tags_38 },
    { 0x7e3c7c5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5c7c61, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbc7c65, SUC_lexicon_tags_104 },
    { 0x45fc7c66, SUC_lexicon_tags_55 },
    { 0x58dc7c67, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407c7c6c, SUC_lexicon_tags_60 },
    { 0x7e1c7c6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17c7c70, SUC_lexicon_tags_11 },
    { 0x6e7c7c71, SUC_lexicon_tags_11 },
    { 0xf09c7c72, SUC_lexicon_tags_7 },
    { 0x1b5c7c70, SUC_lexicon_tags_63 },
    { 0xe45c7c74, SUC_lexicon_tags_7 },
    { 0x2fdc7c75, SUC_lexicon_tags_378 },
    { 0x451c7c73, SUC_lexicon_tags_25 },
    { 0x499c7c75, SUC_lexicon_tags_55 },
    { 0x347c7c70, SUC_lexicon_tags_62 },
    { 0xd5fc7c79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7c7c80, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfb5c7c82, SUC_lexicon_tags_37 },
    { 0xabdc7c83, SUC_lexicon_tags_1 },
    { 0x2e9c7c84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9c7c8e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x45dc7c90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bc7c93, SUC_lexicon_tags_82 },
    { 0xfe1c7c94, SUC_lexicon_tags_1 },
    { 0x44bc7c94, SUC_lexicon_tags_23 },
    { 0xb0bc7c95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb05c7c98, SUC_lexicon_tags_25 },
    { 0x3e7c7c98, SUC_lexicon_tags_2 },
    { 0x433c7c9a, SUC_lexicon_tags_38 },
    { 0x76dc7c9b, SUC_lexicon_tags_46 },
    { 0x7a9c7c99, SUC_lexicon_tags_37 },
    { 0xa3c7c9d, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703c7ca8, SUC_lexicon_tags_40 },
    { 0xb75c7ca9, SUC_lexicon_tags_37 },
    { 0x2edc7ca9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x879c7cac, SUC_lexicon_tags_17 },
    { 0x73bc7cad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadc7cb2, SUC_lexicon_tags_11 },
    { 0xda3c7cb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213c7cb9, SUC_lexicon_tags_63 },
    { 0x8e1c7cba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1c7cbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7c7cc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bc7cc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfb7c7cc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad1c7cc8, SUC_lexicon_tags_4 },
    { 0xec7c7cc9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e3c7ccb, SUC_lexicon_tags_665 },
    { 0xa5dc7ccb, SUC_lexicon_tags_13 },
    { 0x429c7ccd, SUC_lexicon_tags_15 },
    { 0x5adc7cce, SUC_lexicon_tags_12 },
    { 0xfdbc7ccf, SUC_lexicon_tags_17 },
    { 0xb7c7cce, SUC_lexicon_tags_11 },
    { 0xa5bc7cd0, SUC_lexicon_tags_2 },
    { 0x42bc7cd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5c7cd9, SUC_lexicon_tags_17 },
    { 0xa8bc7cda, SUC_lexicon_tags_14 },
    { 0xe13c7cdb, SUC_lexicon_tags_28 },
    { 0xc67c7cd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49c7ce0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3c7ce3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fc7ce7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x201c7ce9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee3c7ceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edc7cf0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fc7cf5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd7dc7cf7, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97c7cfa, SUC_lexicon_tags_2 },
    { 0x783c7cfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51c7d00, SUC_lexicon_tags_141 },
    { 0x7bbc7d01, SUC_lexicon_tags_37 },
    { 0x3adc7d00, SUC_lexicon_tags_56 },
    { 0xec9c7d01, SUC_lexicon_tags_63 },
    { 0x8fbc7d01, SUC_lexicon_tags_9 },
    { 0x9b7c7d05, SUC_lexicon_tags_60 },
    { 0x52fc7d06, SUC_lexicon_tags_27 },
    { 0x40dc7d01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaddc7d09, SUC_lexicon_tags_37 },
    { 0x3cfc7d0a, SUC_lexicon_tags_2 },
    { 0x67bc7d0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573c7d10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae9c7d12, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x621c7d14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x661c7d16, SUC_lexicon_tags_140 },
    { 0x581c7d16, SUC_lexicon_tags_17 },
    { 0x3fdc7d18, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xfa7c7d1a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x357c7d1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b7c7d1f, SUC_lexicon_tags_68 },
    { 0xa57c7d20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21c7d23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75c7d28, SUC_lexicon_tags_2 },
    { 0x4f3c7d29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fc7d31, SUC_lexicon_tags_13 },
    { 0x29bc7d32, SUC_lexicon_tags_23 },
    { 0xa2fc7d31, SUC_lexicon_tags_2 },
    { 0xf55c7d32, SUC_lexicon_tags_2 },
    { 0xcfdc7d35, SUC_lexicon_tags_61 },
    { 0xd27c7d35, SUC_lexicon_tags_1 },
    { 0x461c7d37, SUC_lexicon_tags_14 },
    { 0x8d9c7d38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c7d3c, SUC_lexicon_tags_26 },
    { 0x359c7d3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443c7d41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7c7d44, SUC_lexicon_tags_129 },
    { 0xccdc7d44, SUC_lexicon_tags_1 },
    { 0x4a9c7d45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c5c7d48, SUC_lexicon_tags_27 },
    { 0x84bc7d48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x797c7d4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53c7d4f, SUC_lexicon_tags_2 },
    { 0x571c7d50, SUC_lexicon_tags_37 },
    { 0xc0fc7d4f, SUC_lexicon_tags_15 },
    { 0x639c7d51, SUC_lexicon_tags_19 },
    { 0x70dc7d53, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bc7d59, SUC_lexicon_tags_11 },
    { 0x7b7c7d5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf83c7d5c, SUC_lexicon_tags_19 },
    { 0x283c7d5d, SUC_lexicon_tags_29 },
    { 0xc31c7d5d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5c7d61, SUC_lexicon_tags_11 },
    { 0xd8bc7d61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xce7c7d64, SUC_lexicon_tags_23 },
    { 0xb4fc7d65, SUC_lexicon_tags_15 },
    { 0x2d5c7d66, SUC_lexicon_tags_14 },
    { 0xa61c7d67, SUC_lexicon_tags_45 },
    { 0xbedc7d67, SUC_lexicon_tags_37 },
    { 0x547c7d69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbbc7d6b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61c7d6f, SUC_lexicon_tags_37 },
    { 0xf4dc7d6f, SUC_lexicon_tags_17 },
    { 0x82fc7d71, SUC_lexicon_tags_43 },
    { 0xc09c7d72, SUC_lexicon_tags_25 },
    { 0xe37c7d72, SUC_lexicon_tags_19 },
    { 0xa57c7d74, SUC_lexicon_tags_4 },
    { 0x135c7d72, SUC_lexicon_tags_37 },
    { 0x53dc7d72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc7d79, SUC_lexicon_tags_77 },
    { 0x8dfc7d7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5c7d81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf91c7d83, SUC_lexicon_tags_4 },
    { 0x347c7d83, SUC_lexicon_tags_32 },
    { 0x6c5c7d85, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bc7d89, SUC_lexicon_tags_25 },
    { 0xc49c7d8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4bc7d8c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4a9c7d8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x709c7d91, SUC_lexicon_tags_43 },
    { 0x4d9c7d91, SUC_lexicon_tags_11 },
    { 0x411c7d93, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5c7d98, SUC_lexicon_tags_7 },
    { 0xaa1c7d98, SUC_lexicon_tags_25 },
    { 0x33fc7d9a, SUC_lexicon_tags_37 },
    { 0xd77c7d9b, SUC_lexicon_tags_11 },
    { 0x175c7d9b, SUC_lexicon_tags_15 },
    { 0x5c5c7d98, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce5c7d9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5c7da2, SUC_lexicon_tags_37 },
    { 0x1f7c7da3, SUC_lexicon_tags_62 },
    { 0xc49c7da4, SUC_lexicon_tags_1 },
    { 0x48dc7da5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dc7da9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb7c7dab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf3c7dad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29c7db5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x781c7db7, SUC_lexicon_tags_37 },
    { 0xd0fc7db8, SUC_lexicon_tags_2 },
    { 0xde5c7db9, SUC_lexicon_tags_46 },
    { 0x32dc7dba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79fc7dbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3c7dc2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dc7dcb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47c7dce, SUC_lexicon_tags_11 },
    { 0xad1c7dcf, SUC_lexicon_tags_63 },
    { 0xeafc7dcf, SUC_lexicon_tags_79 },
    { 0x6bbc7dd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x463c7dd4, SUC_lexicon_tags_24 },
    { 0x95fc7dd5, SUC_lexicon_tags_60 },
    { 0x4e9c7dd5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x351c7dd9, SUC_lexicon_tags_45 },
    { 0xa3fc7dd9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc5fc7ddc, SUC_lexicon_tags_32 },
    { 0x9d3c7ddd, SUC_lexicon_tags_19 },
    { 0xddc7dde, SUC_lexicon_tags_12 },
    { 0x503c7ddf, SUC_lexicon_tags_17 },
    { 0x73c7ddc, SUC_lexicon_tags_82 },
    { 0xc37c7de1, SUC_lexicon_tags_17 },
    { 0x8e3c7ddd, SUC_lexicon_tags_43 },
    { 0xb6dc7de3, SUC_lexicon_tags_39 },
    { 0xee1c7de1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1c7de7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdc7deb, SUC_lexicon_tags_60 },
    { 0x65fc7dec, SUC_lexicon_tags_49 },
    { 0xb99c7dec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe3c7def, SUC_lexicon_tags_56 },
    { 0xdc1c7df0, SUC_lexicon_tags_108 },
    { 0xd39c7df1, SUC_lexicon_tags_12 },
    { 0x823c7df2, SUC_lexicon_tags_25 },
    { 0xaebc7df3, SUC_lexicon_tags_32 },
    { 0x23c7df0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17c7dfa, SUC_lexicon_tags_1 },
    { 0xfcbc7dfb, SUC_lexicon_tags_37 },
    { 0x263c7dfc, SUC_lexicon_tags_8 },
    { 0x13c7dfd, SUC_lexicon_tags_28 },
    { 0x59dc7dfd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fc7e00, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691c7e07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513c7e0c, SUC_lexicon_tags_25 },
    { 0xda1c7e0d, SUC_lexicon_tags_37 },
    { 0xb3fc7e0d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x219c7e10, SUC_lexicon_tags_12 },
    { 0x6f7c7e11, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x1f5c7e13, SUC_lexicon_tags_7 },
    { 0x7f3c7e13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb77c7e16, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9c7e1a, SUC_lexicon_tags_11 },
    { 0x38bc7e1b, SUC_lexicon_tags_17 },
    { 0xf49c7e1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dc7e29, SUC_lexicon_tags_1 },
    { 0xdc3c7e29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcafc7e2c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x285c7e2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fc7e31, SUC_lexicon_tags_17 },
    { 0x351c7e32, SUC_lexicon_tags_15 },
    { 0x5afc7e32, SUC_lexicon_tags_37 },
    { 0x909c7e32, SUC_lexicon_tags_14 },
    { 0x9b7c7e35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x12bc7e37, SUC_lexicon_tags_4 },
    { 0x869c7e38, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bc7e41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dc7e48, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d5c7e4a, SUC_lexicon_tags_34 },
    { 0xb03c7e4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3c7e4f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc9bc7e51, SUC_lexicon_tags_31 },
    { 0xf6bc7e51, SUC_lexicon_tags_28 },
    { 0x13bc7e52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19dc7e57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d9c7e59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfc7e5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc7e61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0fc7e63, SUC_lexicon_tags_26 },
    { 0xa5bc7e63, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe57c7e66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf09c7e68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fc7e6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x247c7e70, SUC_lexicon_tags_14 },
    { 0xac5c7e71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dc7e76, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe5fc7e78, SUC_lexicon_tags_1 },
    { 0x98dc7e79, SUC_lexicon_tags_25 },
    { 0xf43c7e78, SUC_lexicon_tags_46 },
    { 0x82bc7e79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbfdc7e7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9c7e89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x409c7e8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb55c7e8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55c7e91, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5c7e97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b3c7e99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24bc7e9c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfddc7e9e, SUC_lexicon_tags_4 },
    { 0xd13c7e9e, SUC_lexicon_tags_33 },
    { 0x6edc7e9f, SUC_lexicon_tags_17 },
    { 0x68bc7e9f, SUC_lexicon_tags_27 },
    { 0xbfc7ea2, SUC_lexicon_tags_19 },
    { 0xa61c7ea1, SUC_lexicon_tags_26 },
    { 0x183c7ea4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afc7ea8, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0xc9bc7eaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bc7ead, SUC_lexicon_tags_82 },
    { 0xf5fc7eae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479c7eb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbc7eb7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6edc7eba, SUC_lexicon_tags_45 },
    { 0x4e7c7eba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x77c7ebd, SUC_lexicon_tags_64 },
    { 0x5cfc7ebe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9c7ec1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19c7ec6, SUC_lexicon_tags_1 },
    { 0x953c7ec7, SUC_lexicon_tags_47 },
    { 0x6cdc7ec8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fc7ed3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83c7ed6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x395c7ed8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddc7ede, SUC_lexicon_tags_40 },
    { 0xf4bc7ede, SUC_lexicon_tags_68 },
    { 0xf15c7ee0, SUC_lexicon_tags_55 },
    { 0xe2dc7edf, SUC_lexicon_tags_6 },
    { 0xc55c7ee2, SUC_lexicon_tags_47 },
    { 0x633c7ee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9c7ee6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8fdc7ee8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913c7eed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc7ef6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51dc7ef8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3c7efb, SUC_lexicon_tags_29 },
    { 0xbc7c7efb, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0xa89c7efe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299c7f02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c7f07, SUC_lexicon_tags_62 },
    { 0x7d5c7f07, SUC_lexicon_tags_48 },
    { 0x43c7f09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe87c7f0b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2a1c7f0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bfc7f0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991c7f16, SUC_lexicon_tags_49 },
    { 0xa6dc7f17, SUC_lexicon_tags_2 },
    { 0x84dc7f18, SUC_lexicon_tags_31 },
    { 0x5ffc7f19, SUC_lexicon_tags_7 },
    { 0xbbdc7f19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x235c7f1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x271c7f1e, SUC_lexicon_tags_37 },
    { 0xf23c7f1e, SUC_lexicon_tags_7 },
    { 0xa09c7f20, SUC_lexicon_tags_12 },
    { 0x46dc7f21, SUC_lexicon_tags_10 },
    { 0x5d1c7f22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65c7f27, SUC_lexicon_tags_37 },
    { 0xe7bc7f28, SUC_lexicon_tags_62 },
    { 0x669c7f28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa5c7f2b, SUC_lexicon_tags_47 },
    { 0x3e5c7f2c, SUC_lexicon_tags_156 },
    { 0x917c7f2d, SUC_lexicon_tags_11 },
    { 0x93c7f2b, SUC_lexicon_tags_27 },
    { 0xe23c7f2c, SUC_lexicon_tags_17 },
    { 0x54fc7f2d, SUC_lexicon_tags_26 },
    { 0xc51c7f2d, SUC_lexicon_tags_4 },
    { 0x3abc7f32, SUC_lexicon_tags_10 },
    { 0x257c7f33, SUC_lexicon_tags_25 },
    { 0xc5c7f2d, SUC_lexicon_tags_1 },
    { 0x8a1c7f35, SUC_lexicon_tags_17 },
    { 0x9a1c7f36, SUC_lexicon_tags_12 },
    { 0x741c7f35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x719c7f39, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9c7f3c, SUC_lexicon_tags_65 },
    { 0xe31c7f3d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x473c7f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdc7f43, SUC_lexicon_tags_12 },
    { 0x593c7f43, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93c7f4c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x363c7f4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dbc7f51, SUC_lexicon_tags_37 },
    { 0xc9fc7f52, SUC_lexicon_tags_11 },
    { 0x2bc7f53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7dc7f55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bc7f58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b5c7f5a, SUC_lexicon_tags_7 },
    { 0xcb1c7f5b, SUC_lexicon_tags_215 },
    { 0xc23c7f5c, SUC_lexicon_tags_14 },
    { 0xb3dc7f5a, SUC_lexicon_tags_12 },
    { 0x6c5c7f5e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bc7f62, SUC_lexicon_tags_46 },
    { 0xa35c7f63, SUC_lexicon_tags_33 },
    { 0x273c7f63, SUC_lexicon_tags_17 },
    { 0x87dc7f63, SUC_lexicon_tags_2 },
    { 0x75c7f64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dc7f6a, SUC_lexicon_tags_19 },
    { 0x119c7f6a, SUC_lexicon_tags_33 },
    { 0x129c7f6b, SUC_lexicon_tags_2 },
    { 0x399c7f6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59c7f71, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x749c7f74, SUC_lexicon_tags_37 },
    { 0xa85c7f75, SUC_lexicon_tags_33 },
    { 0x1e3c7f76, SUC_lexicon_tags_38 },
    { 0x89bc7f77, SUC_lexicon_tags_2 },
    { 0x257c7f78, SUC_lexicon_tags_38 },
    { 0x23bc7f79, SUC_lexicon_tags_45 },
    { 0x6bdc7f7a, SUC_lexicon_tags_11 },
    { 0xc95c7f7b, SUC_lexicon_tags_11 },
    { 0x181c7f7c, SUC_lexicon_tags_92 },
    { 0xcf5c7f7d, SUC_lexicon_tags_38 },
    { 0xe0fc7f7d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dc7f88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x687c7f8a, SUC_lexicon_tags_1 },
    { 0x109c7f8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbc7f8e, SUC_lexicon_tags_158 },
    { 0xf41c7f8e, SUC_lexicon_tags_72 },
    { 0xbe5c7f90, SUC_lexicon_tags_6 },
    { 0x75dc7f8e, SUC_lexicon_tags_12 },
    { 0x983c7f92, SUC_lexicon_tags_63 },
    { 0x463c7f93, SUC_lexicon_tags_43 },
    { 0xd5fc7f92, SUC_lexicon_tags_68 },
    { 0x5dc7f92, SUC_lexicon_tags_0 },
    { 0x97c7f96, SUC_lexicon_tags_9 },
    { 0x591c7f94, SUC_lexicon_tags_11 },
    { 0x58dc7f98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bc7f9c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9c7fa0, SUC_lexicon_tags_37 },
    { 0xdc9c7fa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1c7fa5, SUC_lexicon_tags_57 },
    { 0xfdbc7fa5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1c7fac, SUC_lexicon_tags_1 },
    { 0x345c7fad, SUC_lexicon_tags_17 },
    { 0x8dbc7fad, SUC_lexicon_tags_11 },
    { 0x617c7fac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375c7fb5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x87bc7fb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9c7fbe, SUC_lexicon_tags_55 },
    { 0xa6fc7fbf, SUC_lexicon_tags_31 },
    { 0xd55c7fc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x625c7fc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9c7fc8, SUC_lexicon_tags_30 },
    { 0xabc7fc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbc7fcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dc7fd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x467c7fd2, SUC_lexicon_tags_17 },
    { 0x3c5c7fd3, SUC_lexicon_tags_17 },
    { 0x9f5c7fd4, SUC_lexicon_tags_1 },
    { 0xa35c7fd5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51c7fd8, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xdd9c7fda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x835c7fdc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd23c7fde, SUC_lexicon_tags_25 },
    { 0x42fc7fde, SUC_lexicon_tags_49 },
    { 0x33bc7fde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7c7fe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9c7fec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x88bc7fee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83c7ff1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5c7ff5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efc8001, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fc800d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x813c800f, SUC_lexicon_tags_11 },
    { 0x10dc800f, SUC_lexicon_tags_16 },
    { 0x20dc800f, SUC_lexicon_tags_25 },
    { 0x9ddc800f, SUC_lexicon_tags_10 },
    { 0x5c5c8011, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x831c8015, SUC_lexicon_tags_17 },
    { 0x377c8015, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7c8019, SUC_lexicon_tags_38 },
    { 0x3d5c801a, SUC_lexicon_tags_15 },
    { 0x3e7c801a, SUC_lexicon_tags_37 },
    { 0x1f3c8019, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdc8020, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2adc8023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11c8029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5c802d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387c8031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52dc8033, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2fdc8035, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367c803b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3c8044, SUC_lexicon_tags_37 },
    { 0x9e1c8044, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efc804a, SUC_lexicon_tags_12 },
    { 0xf91c804b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c804e, SUC_lexicon_tags_63 },
    { 0x343c804f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x727c8051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9c8054, SUC_lexicon_tags_1 },
    { 0xafbc8054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dc805a, SUC_lexicon_tags_7 },
    { 0xae5c805b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e5c805d, SUC_lexicon_tags_7 },
    { 0xef7c805d, SUC_lexicon_tags_37 },
    { 0x1d5c805d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dc8063, SUC_lexicon_tags_7 },
    { 0xfbdc8064, SUC_lexicon_tags_37 },
    { 0xf8bc8065, SUC_lexicon_tags_21 },
    { 0x303c8063, SUC_lexicon_tags_45 },
    { 0x217c8063, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x93dc8069, SUC_lexicon_tags_25 },
    { 0x10fc8069, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x29bc806c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x191c806e, SUC_lexicon_tags_43 },
    { 0xc95c806f, SUC_lexicon_tags_37 },
    { 0xef5c806e, SUC_lexicon_tags_27 },
    { 0xaa1c806f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x265c8074, SUC_lexicon_tags_25 },
    { 0x6e9c8074, SUC_lexicon_tags_3 },
    { 0xbfc8076, SUC_lexicon_tags_11 },
    { 0xe9bc8077, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x597c8079, SUC_lexicon_tags_7 },
    { 0x4efc807a, SUC_lexicon_tags_63 },
    { 0xe77c807a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x903c807e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x871c8080, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29c8088, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x279c808b, SUC_lexicon_tags_6 },
    { 0x5dbc808c, SUC_lexicon_tags_14 },
    { 0xbd5c808b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc81c808f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30dc8093, SUC_lexicon_tags_15 },
    { 0x785c8094, SUC_lexicon_tags_60 },
    { 0xdefc8093, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x269c8097, SUC_lexicon_tags_43 },
    { 0x279c8097, SUC_lexicon_tags_37 },
    { 0x6f3c8099, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c7c809b, SUC_lexicon_tags_16 },
    { 0xb09c809c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333c80a1, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727c80ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d7c80b0, SUC_lexicon_tags_37 },
    { 0x247c80b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dc80b6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf47c80b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdafc80ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5c80be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x431c80c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7c80c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x139c80c6, SUC_lexicon_tags_63 },
    { 0x4b7c80c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9c80ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bc80ce, SUC_lexicon_tags_25 },
    { 0x417c80ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf99c80d2, SUC_lexicon_tags_25 },
    { 0xa01c80d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc80d7, SUC_lexicon_tags_28 },
    { 0x3bfc80d7, SUC_lexicon_tags_23 },
    { 0xf3dc80d7, SUC_lexicon_tags_17 },
    { 0x4c9c80da, SUC_lexicon_tags_26 },
    { 0x8e1c80da, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dc80e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x351c80e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5c80e7, SUC_lexicon_tags_2 },
    { 0x3cdc80e8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x565c80ea, SUC_lexicon_tags_4 },
    { 0x2d3c80eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x919c80ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffc80f5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9c5c80f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07c8101, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x71bc8103, SUC_lexicon_tags_33 },
    { 0x3cdc8103, SUC_lexicon_tags_83 },
    { 0x6bc8105, SUC_lexicon_tags_4 },
    { 0xf1c8106, SUC_lexicon_tags_25 },
    { 0x4cbc8107, SUC_lexicon_tags_30 },
    { 0xefc8103, SUC_lexicon_tags_9 },
    { 0xb5c8106, SUC_lexicon_tags_127 },
    { 0x86fc810a, SUC_lexicon_tags_38 },
    { 0x2cfc810b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fc810e, SUC_lexicon_tags_62 },
    { 0xbdbc810f, SUC_lexicon_tags_0 },
    { 0xaabc8110, SUC_lexicon_tags_25 },
    { 0x43fc8110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bc8114, SUC_lexicon_tags_16 },
    { 0x77c8115, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afc811a, SUC_lexicon_tags_11 },
    { 0x9dbc811a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dc811f, SUC_lexicon_tags_57 },
    { 0x981c811f, SUC_lexicon_tags_11 },
    { 0x779c8121, SUC_lexicon_tags_140 },
    { 0x607c8122, SUC_lexicon_tags_11 },
    { 0xe25c8123, SUC_lexicon_tags_25 },
    { 0xaafc8122, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749c8129, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fc8132, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x563c8134, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x1a7c8136, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fc813b, SUC_lexicon_tags_7 },
    { 0x79c813c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbf3c813e, SUC_lexicon_tags_25 },
    { 0x9abc813f, SUC_lexicon_tags_15 },
    { 0xdf3c813e, SUC_lexicon_tags_7 },
    { 0xb7dc8140, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19dc8146, SUC_lexicon_tags_118 },
    { 0xc8bc8147, SUC_lexicon_tags_32 },
    { 0xe61c8148, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52dc814a, SUC_lexicon_tags_8 },
    { 0xa5fc814a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7c814f, SUC_lexicon_tags_17 },
    { 0xa77c8150, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dc8155, SUC_lexicon_tags_59 },
    { 0x87fc8156, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dc815b, SUC_lexicon_tags_37 },
    { 0x5efc815c, SUC_lexicon_tags_39 },
    { 0x69dc815d, SUC_lexicon_tags_14 },
    { 0x5a5c815e, SUC_lexicon_tags_3 },
    { 0x34fc815f, SUC_lexicon_tags_43 },
    { 0x997c8160, SUC_lexicon_tags_33 },
    { 0x8adc8160, SUC_lexicon_tags_11 },
    { 0x9f3c8162, SUC_lexicon_tags_17 },
    { 0x9d7c8163, SUC_lexicon_tags_38 },
    { 0x433c8162, SUC_lexicon_tags_39 },
    { 0xfa1c8165, SUC_lexicon_tags_64 },
    { 0x9f7c8164, SUC_lexicon_tags_13 },
    { 0xdc5c8167, SUC_lexicon_tags_1 },
    { 0x3b5c8161, SUC_lexicon_tags_84 },
    { 0xc11c8169, SUC_lexicon_tags_17 },
    { 0x1c9c816a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x36dc816d, SUC_lexicon_tags_17 },
    { 0xdcdc816e, SUC_lexicon_tags_15 },
    { 0xb97c816e, SUC_lexicon_tags_11 },
    { 0xafc816e, SUC_lexicon_tags_7 },
    { 0x569c816e, SUC_lexicon_tags_14 },
    { 0xb87c8170, SUC_lexicon_tags_40 },
    { 0xaf1c8173, SUC_lexicon_tags_15 },
    { 0x9e1c8174, SUC_lexicon_tags_67 },
    { 0x913c8174, SUC_lexicon_tags_31 },
    { 0xdfbc8176, SUC_lexicon_tags_48 },
    { 0xddfc8174, SUC_lexicon_tags_37 },
    { 0x575c8175, SUC_lexicon_tags_28 },
    { 0x99fc8179, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47c817c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c5c817e, SUC_lexicon_tags_4 },
    { 0xe75c817e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfc8183, SUC_lexicon_tags_15 },
    { 0x4a3c8184, SUC_lexicon_tags_60 },
    { 0x411c8183, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47c8189, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363c818d, SUC_lexicon_tags_64 },
    { 0x565c818e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679c8196, SUC_lexicon_tags_15 },
    { 0x31dc8197, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf85c8199, SUC_lexicon_tags_49 },
    { 0x109c8199, SUC_lexicon_tags_27 },
    { 0x59fc8199, SUC_lexicon_tags_25 },
    { 0x219c819b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7c81a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7c81a3, SUC_lexicon_tags_17 },
    { 0xdc9c81a4, SUC_lexicon_tags_37 },
    { 0xd77c81a3, SUC_lexicon_tags_30 },
    { 0x187c81a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25c81aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3c81ae, SUC_lexicon_tags_4 },
    { 0x273c81ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x237c81b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bc81b4, SUC_lexicon_tags_1016 },
    { 0xbcdc81b5, SUC_lexicon_tags_31 },
    { 0xbedc81b6, SUC_lexicon_tags_31 },
    { 0xaa9c81b7, SUC_lexicon_tags_11 },
    { 0xf97c81b8, SUC_lexicon_tags_7 },
    { 0x50bc81b4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143c81be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bc81c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x519c81c2, SUC_lexicon_tags_37 },
    { 0x851c81c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdc81c9, SUC_lexicon_tags_7 },
    { 0xe61c81ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf95c81cc, SUC_lexicon_tags_31 },
    { 0x631c81cd, SUC_lexicon_tags_15 },
    { 0x4d1c81cc, SUC_lexicon_tags_31 },
    { 0xd3bc81cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663c81d8, SUC_lexicon_tags_38 },
    { 0xbe5c81d9, SUC_lexicon_tags_37 },
    { 0x73bc81da, SUC_lexicon_tags_70 },
    { 0x695c81db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635c81df, SUC_lexicon_tags_25 },
    { 0xa3fc81df, SUC_lexicon_tags_63 },
    { 0xd87c81e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbc81e5, SUC_lexicon_tags_3 },
    { 0x8e1c81e6, SUC_lexicon_tags_25 },
    { 0xa9bc81e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7c81ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2ebc81f0, SUC_lexicon_tags_30 },
    { 0xdb1c81f1, SUC_lexicon_tags_69 },
    { 0x4ddc81f2, SUC_lexicon_tags_15 },
    { 0xcc7c81f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c5c81f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95c81f8, SUC_lexicon_tags_92 },
    { 0xb2fc81f9, SUC_lexicon_tags_37 },
    { 0xf9fc81f8, SUC_lexicon_tags_23 },
    { 0x6e1c81fb, SUC_lexicon_tags_65 },
    { 0x1cfc81fc, SUC_lexicon_tags_25 },
    { 0x6edc81fd, SUC_lexicon_tags_35 },
    { 0xf11c81fe, SUC_lexicon_tags_65 },
    { 0x9dbc81ff, SUC_lexicon_tags_29 },
    { 0xeb1c8200, SUC_lexicon_tags_38 },
    { 0x75fc81ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e9c8203, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeedc8205, SUC_lexicon_tags_27 },
    { 0x89fc8206, SUC_lexicon_tags_2 },
    { 0xc4bc8206, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bc820a, SUC_lexicon_tags_37 },
    { 0xe65c820b, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x9d1c820d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fc8216, SUC_lexicon_tags_11 },
    { 0xf37c8217, SUC_lexicon_tags_12 },
    { 0x2dfc8218, SUC_lexicon_tags_16 },
    { 0xaffc8219, SUC_lexicon_tags_27 },
    { 0x2d7c821a, SUC_lexicon_tags_27 },
    { 0xd17c821b, SUC_lexicon_tags_43 },
    { 0x153c821b, SUC_lexicon_tags_62 },
    { 0x6bbc821d, SUC_lexicon_tags_30 },
    { 0xde1c821a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc8223, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x137c8225, SUC_lexicon_tags_1 },
    { 0x27dc8225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afc822f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1edc8231, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd55c8233, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3c8237, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x22bc8239, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9dc823b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fc8242, SUC_lexicon_tags_65 },
    { 0xa6dc8243, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7c8246, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfc824c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9bc824e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa73c8250, SUC_lexicon_tags_17 },
    { 0x65dc8251, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7c8255, SUC_lexicon_tags_63 },
    { 0x455c8255, SUC_lexicon_tags_15 },
    { 0xe7c8255, SUC_lexicon_tags_27 },
    { 0x703c8258, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fc8263, SUC_lexicon_tags_40 },
    { 0xbb7c8264, SUC_lexicon_tags_28 },
    { 0x2d3c8264, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x741c8268, SUC_lexicon_tags_37 },
    { 0xce7c8269, SUC_lexicon_tags_43 },
    { 0x43bc826a, SUC_lexicon_tags_25 },
    { 0xa4dc8269, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x15c826e, SUC_lexicon_tags_37 },
    { 0xde1c826e, SUC_lexicon_tags_12 },
    { 0x9d5c826e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x63fc8273, SUC_lexicon_tags_84 },
    { 0x713c8273, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6cdc8276, SUC_lexicon_tags_22 },
    { 0xb39c8276, SUC_lexicon_tags_16 },
    { 0x323c8278, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5c827d, SUC_lexicon_tags_11 },
    { 0x43bc827e, SUC_lexicon_tags_45 },
    { 0xe33c827d, SUC_lexicon_tags_14 },
    { 0xabc827f, SUC_lexicon_tags_17 },
    { 0xab9c8281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95c8287, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb9c8289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151c828f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x9c8291, SUC_lexicon_tags_14 },
    { 0xadc8291, SUC_lexicon_tags_31 },
    { 0xe3bc8293, SUC_lexicon_tags_11 },
    { 0x19bc8294, SUC_lexicon_tags_48 },
    { 0x88dc8291, SUC_lexicon_tags_7 },
    { 0x8bfc8296, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1fc8298, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x689c829a, SUC_lexicon_tags_49 },
    { 0xdc829a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9c82a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7c82ab, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x605c82ad, SUC_lexicon_tags_31 },
    { 0xa29c82ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77c82b1, SUC_lexicon_tags_46 },
    { 0x89bc82b2, SUC_lexicon_tags_11 },
    { 0x155c82b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1c82b6, SUC_lexicon_tags_53 },
    { 0x551c82b7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9c82ba, SUC_lexicon_tags_1 },
    { 0xbcfc82bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41c82be, SUC_lexicon_tags_2 },
    { 0xe19c82be, SUC_lexicon_tags_7 },
    { 0x36fc82c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07c82c3, SUC_lexicon_tags_17 },
    { 0x5d9c82c4, SUC_lexicon_tags_19 },
    { 0x87c82c5, SUC_lexicon_tags_24 },
    { 0x8afc82c6, SUC_lexicon_tags_48 },
    { 0xa59c82c7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5c82ce, SUC_lexicon_tags_48 },
    { 0x401c82cf, SUC_lexicon_tags_13 },
    { 0x379c82ce, SUC_lexicon_tags_9 },
    { 0xd17c82ce, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77c82d5, SUC_lexicon_tags_15 },
    { 0x36dc82d5, SUC_lexicon_tags_1 },
    { 0x765c82d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dc82dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3c82e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83c82e3, SUC_lexicon_tags_15 },
    { 0x73fc82e4, SUC_lexicon_tags_66 },
    { 0x307c82e5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4cfc82e7, SUC_lexicon_tags_25 },
    { 0xe39c82e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51c82ec, SUC_lexicon_tags_12 },
    { 0x59fc82ed, SUC_lexicon_tags_2 },
    { 0x9f9c82ee, SUC_lexicon_tags_31 },
    { 0xe23c82ee, SUC_lexicon_tags_4 },
    { 0xd0fc82f0, SUC_lexicon_tags_12 },
    { 0xf2bc82f1, SUC_lexicon_tags_37 },
    { 0xa5c82ed, SUC_lexicon_tags_34 },
    { 0x32dc82ef, SUC_lexicon_tags_12 },
    { 0xd5dc82f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1c82f7, SUC_lexicon_tags_33 },
    { 0x14bc82f8, SUC_lexicon_tags_6 },
    { 0x575c82f7, SUC_lexicon_tags_29 },
    { 0x7afc82f7, SUC_lexicon_tags_16 },
    { 0x39fc82fb, SUC_lexicon_tags_25 },
    { 0x4cfc82fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc69c82fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01c8301, SUC_lexicon_tags_44 },
    { 0x6d3c8302, SUC_lexicon_tags_60 },
    { 0xad7c8303, SUC_lexicon_tags_7 },
    { 0xcffc8303, SUC_lexicon_tags_49 },
    { 0xcbdc8304, SUC_lexicon_tags_2 },
    { 0xd1c8303, SUC_lexicon_tags_11 },
    { 0x3afc8301, SUC_lexicon_tags_48 },
    { 0xb8bc8304, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c9c830a, SUC_lexicon_tags_37 },
    { 0xe23c830b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13c830f, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc41c8311, SUC_lexicon_tags_25 },
    { 0x699c8312, SUC_lexicon_tags_1143 },
    { 0, NULL },
    { 0x81bc8314, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x543c8316, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6e3c8318, SUC_lexicon_tags_48 },
    { 0xce7c8318, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc4fc831b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc8dc831d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd65c831f, SUC_lexicon_tags_27 },
    { 0xf33c8320, SUC_lexicon_tags_27 },
    { 0x24bc831f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7c8324, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279c8329, SUC_lexicon_tags_7 },
    { 0xfdfc832a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fc8333, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x29c8335, SUC_lexicon_tags_37 },
    { 0x617c8336, SUC_lexicon_tags_25 },
    { 0x7c5c8337, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bc833c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217c8342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1afc8344, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55c8347, SUC_lexicon_tags_17 },
    { 0xd41c8347, SUC_lexicon_tags_46 },
    { 0xc0dc8348, SUC_lexicon_tags_0 },
    { 0xd73c834a, SUC_lexicon_tags_26 },
    { 0x301c834b, SUC_lexicon_tags_60 },
    { 0x12bc834b, SUC_lexicon_tags_1468 },
    { 0xc75c834d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x243c8350, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3c8357, SUC_lexicon_tags_47 },
    { 0xa81c8358, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dc835c, SUC_lexicon_tags_7 },
    { 0xdbc835d, SUC_lexicon_tags_10 },
    { 0xe73c835c, SUC_lexicon_tags_37 },
    { 0x7fc835c, SUC_lexicon_tags_37 },
    { 0x81fc8360, SUC_lexicon_tags_15 },
    { 0x465c8360, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fc8367, SUC_lexicon_tags_17 },
    { 0x3d1c8367, SUC_lexicon_tags_39 },
    { 0xbf3c8369, SUC_lexicon_tags_34 },
    { 0x79c8368, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xde5c836c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97c836f, SUC_lexicon_tags_26 },
    { 0x76fc8370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x985c8373, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x41c8376, SUC_lexicon_tags_1 },
    { 0xf93c8377, SUC_lexicon_tags_38 },
    { 0xc8fc8378, SUC_lexicon_tags_65 },
    { 0x583c8379, SUC_lexicon_tags_37 },
    { 0x925c8378, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x32fc837d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x625c837f, SUC_lexicon_tags_14 },
    { 0x22bc8380, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49dc8383, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc5bc8385, SUC_lexicon_tags_14 },
    { 0x89c8386, SUC_lexicon_tags_15 },
    { 0x2efc8387, SUC_lexicon_tags_15 },
    { 0x645c8385, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x771c838b, SUC_lexicon_tags_27 },
    { 0x693c838c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27dc838e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49c8395, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bc8398, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddc83a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3c83ab, SUC_lexicon_tags_25 },
    { 0xf57c83ab, SUC_lexicon_tags_19 },
    { 0x94fc83ad, SUC_lexicon_tags_7 },
    { 0x7ffc83ae, SUC_lexicon_tags_16 },
    { 0xca1c83ad, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dc83be, SUC_lexicon_tags_8 },
    { 0x331c83bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27c83c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fc83c5, SUC_lexicon_tags_29 },
    { 0x3efc83c6, SUC_lexicon_tags_79 },
    { 0xc29c83c5, SUC_lexicon_tags_17 },
    { 0xa53c83c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7c83cb, SUC_lexicon_tags_187 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fc83cf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7c83d2, SUC_lexicon_tags_0 },
    { 0xeabc83d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edc83d8, SUC_lexicon_tags_49 },
    { 0xf0bc83d8, SUC_lexicon_tags_63 },
    { 0xcb9c83da, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527c83df, SUC_lexicon_tags_25 },
    { 0x159c83df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dc83e6, SUC_lexicon_tags_39 },
    { 0x11fc83e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf73c83e9, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145c83ee, SUC_lexicon_tags_64 },
    { 0x819c83ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1c83f2, SUC_lexicon_tags_17 },
    { 0x8adc83f3, SUC_lexicon_tags_14 },
    { 0xf5fc83f4, SUC_lexicon_tags_63 },
    { 0x8a3c83f5, SUC_lexicon_tags_17 },
    { 0xa7fc83f6, SUC_lexicon_tags_15 },
    { 0xee3c83f5, SUC_lexicon_tags_37 },
    { 0xe69c83f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x339c83fa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf6bc83fc, SUC_lexicon_tags_12 },
    { 0x4afc83fd, SUC_lexicon_tags_79 },
    { 0x9d1c83fe, SUC_lexicon_tags_47 },
    { 0xb2bc83ff, SUC_lexicon_tags_1 },
    { 0x925c83fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabc8403, SUC_lexicon_tags_23 },
    { 0x387c8404, SUC_lexicon_tags_37 },
    { 0xa1c8403, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb27c8407, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dc840a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7c840f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe95c8412, SUC_lexicon_tags_25 },
    { 0x7a7c8413, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7f7c8415, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed3c8417, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03c841b, SUC_lexicon_tags_37 },
    { 0xbcfc841c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2adc841f, SUC_lexicon_tags_0 },
    { 0x14dc8420, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7c8424, SUC_lexicon_tags_2 },
    { 0xadfc8425, SUC_lexicon_tags_15 },
    { 0x25c8424, SUC_lexicon_tags_315 },
    { 0x563c8427, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249c842b, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x531c842d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf39c842f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7afc8432, SUC_lexicon_tags_14 },
    { 0x957c8433, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe55c8435, SUC_lexicon_tags_12 },
    { 0xe19c8436, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1c8444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x63bc8447, SUC_lexicon_tags_148 },
    { 0xf27c8448, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x78fc844a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3c844e, SUC_lexicon_tags_17 },
    { 0x2bfc844f, SUC_lexicon_tags_12 },
    { 0xdb7c8450, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93c8454, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957c8459, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3c845c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x81c845e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1c8461, SUC_lexicon_tags_25 },
    { 0xdf1c8462, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3d5c8464, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdc846c, SUC_lexicon_tags_37 },
    { 0xc05c846d, SUC_lexicon_tags_8 },
    { 0x745c846c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dc8474, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xafc8476, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x37dc8478, SUC_lexicon_tags_27 },
    { 0x11fc8478, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7c847d, SUC_lexicon_tags_15 },
    { 0xc7dc847e, SUC_lexicon_tags_37 },
    { 0xf3dc847d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaadc8481, SUC_lexicon_tags_17 },
    { 0x913c8482, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbbbc8484, SUC_lexicon_tags_34 },
    { 0x1d3c8485, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdc8489, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565c848d, SUC_lexicon_tags_26 },
    { 0x793c848e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61c8492, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb17c8495, SUC_lexicon_tags_49 },
    { 0xfcdc8496, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa69c8498, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdc849b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cfc849d, SUC_lexicon_tags_1 },
    { 0x5c7c849d, SUC_lexicon_tags_12 },
    { 0x9fc849e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x29fc84a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf9c84a3, SUC_lexicon_tags_87 },
    { 0x15dc84a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bc84a8, SUC_lexicon_tags_25 },
    { 0xa57c84a8, SUC_lexicon_tags_56 },
    { 0x6c1c84aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x147c84ac, SUC_lexicon_tags_11 },
    { 0x6dfc84ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dc84b3, SUC_lexicon_tags_70 },
    { 0xd99c84b4, SUC_lexicon_tags_12 },
    { 0x96fc84b5, SUC_lexicon_tags_11 },
    { 0xde7c84b5, SUC_lexicon_tags_45 },
    { 0xb55c84b6, SUC_lexicon_tags_10 },
    { 0x269c84b8, SUC_lexicon_tags_6 },
    { 0x377c84b5, SUC_lexicon_tags_10 },
    { 0xdf5c84b4, SUC_lexicon_tags_23 },
    { 0x3b5c84bb, SUC_lexicon_tags_7 },
    { 0xa47c84b4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc84c3, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x72dc84c6, SUC_lexicon_tags_207 },
    { 0x5b1c84c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671c84cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afc84d0, SUC_lexicon_tags_27 },
    { 0xe53c84d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x63fc84d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe15c84d5, SUC_lexicon_tags_2 },
    { 0xacdc84d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bc84df, SUC_lexicon_tags_8 },
    { 0x6d9c84e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e5c84e2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb13c84e4, SUC_lexicon_tags_2 },
    { 0xfbfc84e4, SUC_lexicon_tags_29 },
    { 0x4ffc84e4, SUC_lexicon_tags_17 },
    { 0x33bc84e7, SUC_lexicon_tags_1 },
    { 0x15dc84e8, SUC_lexicon_tags_6 },
    { 0x78bc84e9, SUC_lexicon_tags_11 },
    { 0x599c84ea, SUC_lexicon_tags_17 },
    { 0xcdfc84ea, SUC_lexicon_tags_17 },
    { 0x6bdc84e5, SUC_lexicon_tags_4 },
    { 0x549c84e7, SUC_lexicon_tags_2 },
    { 0xdcfc84ee, SUC_lexicon_tags_30 },
    { 0x417c84e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfc84f2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xecbc84f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dc84fe, SUC_lexicon_tags_282 },
    { 0x6d9c84ff, SUC_lexicon_tags_16 },
    { 0xb17c8500, SUC_lexicon_tags_17 },
    { 0xc85c84fe, SUC_lexicon_tags_39 },
    { 0x36bc8502, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xb3dc8504, SUC_lexicon_tags_74 },
    { 0xd7dc8505, SUC_lexicon_tags_17 },
    { 0xcdc8504, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe57c8508, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbc850d, SUC_lexicon_tags_11 },
    { 0xa5bc850e, SUC_lexicon_tags_47 },
    { 0x459c850f, SUC_lexicon_tags_45 },
    { 0x2d7c850e, SUC_lexicon_tags_8 },
    { 0x4f3c8511, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689c8519, SUC_lexicon_tags_10 },
    { 0x4fc851a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dc851e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x411c8520, SUC_lexicon_tags_124 },
    { 0xf61c8521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x197c8523, SUC_lexicon_tags_0 },
    { 0xdf7c8523, SUC_lexicon_tags_63 },
    { 0xc0bc8524, SUC_lexicon_tags_43 },
    { 0x5e3c8526, SUC_lexicon_tags_32 },
    { 0x863c8527, SUC_lexicon_tags_14 },
    { 0x3b9c8528, SUC_lexicon_tags_14 },
    { 0x51c8527, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5c9c852b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe3c852d, SUC_lexicon_tags_6 },
    { 0x951c852d, SUC_lexicon_tags_2 },
    { 0xbc852d, SUC_lexicon_tags_6 },
    { 0x12bc8530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbc8536, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805c853a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05c8540, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x553c8542, SUC_lexicon_tags_15 },
    { 0xfb1c8542, SUC_lexicon_tags_21 },
    { 0x91dc8543, SUC_lexicon_tags_11 },
    { 0x5ebc8544, SUC_lexicon_tags_12 },
    { 0x4afc8544, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bc8549, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdc854c, SUC_lexicon_tags_14 },
    { 0x255c854d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bc8550, SUC_lexicon_tags_0 },
    { 0xb1c8551, SUC_lexicon_tags_12 },
    { 0xb6dc8551, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6bc8554, SUC_lexicon_tags_14 },
    { 0xae1c8554, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bc855b, SUC_lexicon_tags_8 },
    { 0x181c855c, SUC_lexicon_tags_46 },
    { 0x695c855d, SUC_lexicon_tags_11 },
    { 0xb0bc855c, SUC_lexicon_tags_45 },
    { 0x9bdc855f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2ddc8561, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bc8565, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333c856a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x793c856d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e1c856f, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xdfdc8571, SUC_lexicon_tags_17 },
    { 0x13dc8572, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7c8577, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23dc857a, SUC_lexicon_tags_12 },
    { 0x93bc857b, SUC_lexicon_tags_43 },
    { 0x56dc857c, SUC_lexicon_tags_2 },
    { 0x80fc857b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x17fc857f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7c8582, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9dc8584, SUC_lexicon_tags_11 },
    { 0x337c8585, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x89dc8587, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x18dc858a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x48dc858d, SUC_lexicon_tags_15 },
    { 0xc83c858d, SUC_lexicon_tags_37 },
    { 0x9fc858f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bc8595, SUC_lexicon_tags_25 },
    { 0xebdc8596, SUC_lexicon_tags_148 },
    { 0xf37c8596, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x103c8599, SUC_lexicon_tags_17 },
    { 0x71bc8599, SUC_lexicon_tags_70 },
    { 0x87dc8599, SUC_lexicon_tags_7 },
    { 0x43bc859c, SUC_lexicon_tags_37 },
    { 0x641c8599, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3c85a2, SUC_lexicon_tags_17 },
    { 0xdcfc85a3, SUC_lexicon_tags_66 },
    { 0x6c3c85a3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xec9c85a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5c85a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3c85ae, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x685c85b0, SUC_lexicon_tags_12 },
    { 0x8e7c85b0, SUC_lexicon_tags_71 },
    { 0x7ebc85b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd45c85b4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279c85b8, SUC_lexicon_tags_44 },
    { 0xa3dc85b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03c85be, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801c85c5, SUC_lexicon_tags_49 },
    { 0x7e9c85c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249c85cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bc85d1, SUC_lexicon_tags_56 },
    { 0x271c85d2, SUC_lexicon_tags_30 },
    { 0xc17c85d1, SUC_lexicon_tags_11 },
    { 0x92fc85d4, SUC_lexicon_tags_47 },
    { 0x589c85d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89c85d8, SUC_lexicon_tags_0 },
    { 0xb2bc85d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe27c85db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcbc85dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf5c85df, SUC_lexicon_tags_38 },
    { 0x699c85df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edc85e3, SUC_lexicon_tags_60 },
    { 0x991c85e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9c85e8, SUC_lexicon_tags_17 },
    { 0x2f1c85e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5c85ed, SUC_lexicon_tags_23 },
    { 0x9f1c85ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd7bc85f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901c85f6, SUC_lexicon_tags_1 },
    { 0x8b3c85f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1c8604, SUC_lexicon_tags_11 },
    { 0xbb3c8605, SUC_lexicon_tags_16 },
    { 0x9bbc8606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5bc8608, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7c860b, SUC_lexicon_tags_11 },
    { 0x59c860c, SUC_lexicon_tags_118 },
    { 0x6cfc860d, SUC_lexicon_tags_1 },
    { 0x5f1c860c, SUC_lexicon_tags_87 },
    { 0x2b7c860c, SUC_lexicon_tags_66 },
    { 0xdc9c8610, SUC_lexicon_tags_17 },
    { 0x907c860b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3c8617, SUC_lexicon_tags_1 },
    { 0xc5dc8618, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3c861b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe9c861d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7efc861f, SUC_lexicon_tags_17 },
    { 0x255c8620, SUC_lexicon_tags_29 },
    { 0x5cdc8621, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe05c8623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7c8629, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c9c862b, SUC_lexicon_tags_38 },
    { 0x5fc862c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8abc862e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe5bc8630, SUC_lexicon_tags_15 },
    { 0x5bbc8631, SUC_lexicon_tags_47 },
    { 0xdc7c8632, SUC_lexicon_tags_15 },
    { 0x917c8631, SUC_lexicon_tags_37 },
    { 0xa47c8634, SUC_lexicon_tags_84 },
    { 0xb1fc8635, SUC_lexicon_tags_64 },
    { 0xe95c8636, SUC_lexicon_tags_47 },
    { 0x75c8637, SUC_lexicon_tags_31 },
    { 0x427c8638, SUC_lexicon_tags_14 },
    { 0x211c8638, SUC_lexicon_tags_46 },
    { 0xedbc863a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47c863d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77c863f, SUC_lexicon_tags_17 },
    { 0xd07c8640, SUC_lexicon_tags_15 },
    { 0x87c8641, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4c5c8643, SUC_lexicon_tags_19 },
    { 0x63dc8644, SUC_lexicon_tags_37 },
    { 0xf6fc8644, SUC_lexicon_tags_38 },
    { 0xd53c8646, SUC_lexicon_tags_77 },
    { 0xdcbc8644, SUC_lexicon_tags_35 },
    { 0x493c8648, SUC_lexicon_tags_11 },
    { 0xd59c8646, SUC_lexicon_tags_37 },
    { 0xdc1c8645, SUC_lexicon_tags_8 },
    { 0xe71c8648, SUC_lexicon_tags_38 },
    { 0xac7c8645, SUC_lexicon_tags_23 },
    { 0x35bc8646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19c8651, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddc8654, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7c8658, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bc865b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfc865e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17c8665, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc77c8667, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757c866b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75c866e, SUC_lexicon_tags_25 },
    { 0xd67c866e, SUC_lexicon_tags_65 },
    { 0x669c8670, SUC_lexicon_tags_25 },
    { 0xecfc8671, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbc8674, SUC_lexicon_tags_63 },
    { 0x907c8675, SUC_lexicon_tags_12 },
    { 0x935c8674, SUC_lexicon_tags_43 },
    { 0xeb5c8677, SUC_lexicon_tags_9 },
    { 0x1d9c8677, SUC_lexicon_tags_37 },
    { 0x2fbc8679, SUC_lexicon_tags_26 },
    { 0x881c8677, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd27c867c, SUC_lexicon_tags_1 },
    { 0xedbc867d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x665c867f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe83c8681, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bc8685, SUC_lexicon_tags_37 },
    { 0x919c8686, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfc8690, SUC_lexicon_tags_7 },
    { 0x3f3c8690, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x43fc8693, SUC_lexicon_tags_0 },
    { 0xdc5c8694, SUC_lexicon_tags_47 },
    { 0x9c3c8695, SUC_lexicon_tags_31 },
    { 0xc5c8694, SUC_lexicon_tags_30 },
    { 0x6b3c8697, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fc869d, SUC_lexicon_tags_11 },
    { 0x75dc869e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7c86a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abc86a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23fc86a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc35c86ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bc86ae, SUC_lexicon_tags_5 },
    { 0x6adc86ae, SUC_lexicon_tags_18 },
    { 0x769c86af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21dc86b8, SUC_lexicon_tags_19 },
    { 0x7b1c86b9, SUC_lexicon_tags_1 },
    { 0xa6dc86b8, SUC_lexicon_tags_29 },
    { 0x989c86b8, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97c86c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbc86c5, SUC_lexicon_tags_11 },
    { 0x24dc86c6, SUC_lexicon_tags_5 },
    { 0x975c86c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447c86cd, SUC_lexicon_tags_54 },
    { 0x73fc86ce, SUC_lexicon_tags_14 },
    { 0x847c86cd, SUC_lexicon_tags_11 },
    { 0x5c7c86d0, SUC_lexicon_tags_25 },
    { 0xa43c86ce, SUC_lexicon_tags_4 },
    { 0xf93c86d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bc86d7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143c86dc, SUC_lexicon_tags_85 },
    { 0x82fc86dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fc86e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5afc86e2, SUC_lexicon_tags_46 },
    { 0xd0dc86e3, SUC_lexicon_tags_15 },
    { 0x8dc86e2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd75c86e6, SUC_lexicon_tags_1 },
    { 0xcabc86e7, SUC_lexicon_tags_29 },
    { 0x48fc86e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887c86ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x741c86ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x221c86f1, SUC_lexicon_tags_27 },
    { 0x8d1c86f2, SUC_lexicon_tags_14 },
    { 0xa91c86f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bc86f7, SUC_lexicon_tags_10 },
    { 0x2b3c86f7, SUC_lexicon_tags_12 },
    { 0x5fc86f8, SUC_lexicon_tags_31 },
    { 0x6b9c86fa, SUC_lexicon_tags_79 },
    { 0xd85c86fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fc86ff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x6edc8701, SUC_lexicon_tags_37 },
    { 0xdefc8702, SUC_lexicon_tags_17 },
    { 0xed5c8703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839c8708, SUC_lexicon_tags_12 },
    { 0xa5bc8708, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x171c870b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7dbc870d, SUC_lexicon_tags_1 },
    { 0xf1dc870e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x501c8711, SUC_lexicon_tags_1 },
    { 0x4ebc8712, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbc8715, SUC_lexicon_tags_60 },
    { 0x73fc8715, SUC_lexicon_tags_25 },
    { 0xabfc8717, SUC_lexicon_tags_16 },
    { 0xb93c8717, SUC_lexicon_tags_10 },
    { 0x8adc8719, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01c871c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x755c871f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dc8727, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x497c8729, SUC_lexicon_tags_0 },
    { 0xf15c8729, SUC_lexicon_tags_11 },
    { 0xb8fc872b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675c8730, SUC_lexicon_tags_9 },
    { 0xb73c8731, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x235c8733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x709c8735, SUC_lexicon_tags_8 },
    { 0x4e3c8736, SUC_lexicon_tags_60 },
    { 0x1bbc8735, SUC_lexicon_tags_45 },
    { 0xbefc8738, SUC_lexicon_tags_262 },
    { 0x19dc8736, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37c873c, SUC_lexicon_tags_48 },
    { 0x181c873d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93c8742, SUC_lexicon_tags_7 },
    { 0x98bc8743, SUC_lexicon_tags_26 },
    { 0xf59c8742, SUC_lexicon_tags_17 },
    { 0x169c8745, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abc8749, SUC_lexicon_tags_46 },
    { 0xf19c874a, SUC_lexicon_tags_21 },
    { 0xf61c8749, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07c8753, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3d1c8755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x939c8757, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bc875d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7bc875f, SUC_lexicon_tags_82 },
    { 0x8cdc8760, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa1c8762, SUC_lexicon_tags_46 },
    { 0x869c8763, SUC_lexicon_tags_25 },
    { 0x7edc8764, SUC_lexicon_tags_60 },
    { 0xfafc8765, SUC_lexicon_tags_8 },
    { 0xf47c8766, SUC_lexicon_tags_38 },
    { 0x155c8764, SUC_lexicon_tags_63 },
    { 0xa5dc8762, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8c5c876a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdc8777, SUC_lexicon_tags_11 },
    { 0x367c8778, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fc877e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb21c8780, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597c8788, SUC_lexicon_tags_27 },
    { 0x569c8789, SUC_lexicon_tags_25 },
    { 0x363c878a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x667c878c, SUC_lexicon_tags_37 },
    { 0xdbc878d, SUC_lexicon_tags_11 },
    { 0xb93c878e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fdc8790, SUC_lexicon_tags_28 },
    { 0xe99c8791, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9c8794, SUC_lexicon_tags_43 },
    { 0x997c8795, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa71c8797, SUC_lexicon_tags_51 },
    { 0xcddc8797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331c87a1, SUC_lexicon_tags_14 },
    { 0x6c7c87a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x777c87a4, SUC_lexicon_tags_45 },
    { 0x159c87a4, SUC_lexicon_tags_31 },
    { 0x325c87a6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd7fc87a8, SUC_lexicon_tags_21 },
    { 0x2fdc87a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd1fc87ab, SUC_lexicon_tags_65 },
    { 0x85bc87ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3c87af, SUC_lexicon_tags_7 },
    { 0xbd3c87b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09c87b4, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bc87be, SUC_lexicon_tags_17 },
    { 0xb03c87be, SUC_lexicon_tags_37 },
    { 0x4e3c87c0, SUC_lexicon_tags_17 },
    { 0x833c87be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe63c87c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd17c87c5, SUC_lexicon_tags_30 },
    { 0x983c87c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd73c87c8, SUC_lexicon_tags_11 },
    { 0x825c87c8, SUC_lexicon_tags_7 },
    { 0x2b5c87ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6f9c87cc, SUC_lexicon_tags_7 },
    { 0xef3c87cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8f9c87cf, SUC_lexicon_tags_28 },
    { 0xd29c87cf, SUC_lexicon_tags_9 },
    { 0xb67c87d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd71c87d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fc87d6, SUC_lexicon_tags_3 },
    { 0x871c87d7, SUC_lexicon_tags_15 },
    { 0x82fc87d6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x641c87db, SUC_lexicon_tags_47 },
    { 0xe8fc87dc, SUC_lexicon_tags_37 },
    { 0xaedc87db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fc87e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4f5c87e2, SUC_lexicon_tags_31 },
    { 0x473c87e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c1c87e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8adc87e8, SUC_lexicon_tags_1 },
    { 0x80dc87e9, SUC_lexicon_tags_49 },
    { 0xaa1c87ea, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9c87ee, SUC_lexicon_tags_14 },
    { 0x769c87ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bc87f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44dc87f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24dc87fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd21c87fc, SUC_lexicon_tags_11 },
    { 0xf5fc87fc, SUC_lexicon_tags_27 },
    { 0xba5c87fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fc8801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21c8804, SUC_lexicon_tags_56 },
    { 0x36fc8804, SUC_lexicon_tags_2 },
    { 0xc6dc8806, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b5c8808, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675c880d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6dc8811, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x37bc8814, SUC_lexicon_tags_27 },
    { 0xbadc8814, SUC_lexicon_tags_40 },
    { 0x9ddc8816, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x881c8818, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687c881d, SUC_lexicon_tags_7 },
    { 0xbbc881d, SUC_lexicon_tags_37 },
    { 0x573c881f, SUC_lexicon_tags_12 },
    { 0x77c881d, SUC_lexicon_tags_65 },
    { 0xf19c8821, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7c8826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5c882b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27c8837, SUC_lexicon_tags_25 },
    { 0xbc1c8838, SUC_lexicon_tags_0 },
    { 0xb89c8839, SUC_lexicon_tags_43 },
    { 0x28bc8838, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x753c883d, SUC_lexicon_tags_11 },
    { 0x8c3c883e, SUC_lexicon_tags_17 },
    { 0xe9dc883f, SUC_lexicon_tags_7 },
    { 0x147c8840, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721c8847, SUC_lexicon_tags_29 },
    { 0x4e1c8847, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa79c884a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543c884e, SUC_lexicon_tags_15 },
    { 0xeebc884f, SUC_lexicon_tags_26 },
    { 0x359c884f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fc885a, SUC_lexicon_tags_33 },
    { 0x203c885a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaebc885e, SUC_lexicon_tags_27 },
    { 0x957c885e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad7c8861, SUC_lexicon_tags_0 },
    { 0xa0fc8862, SUC_lexicon_tags_43 },
    { 0x379c8862, SUC_lexicon_tags_46 },
    { 0x365c8863, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x43fc8866, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x445c8869, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c5c886b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5c8870, SUC_lexicon_tags_11 },
    { 0xa05c8871, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf7c8873, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57c8877, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc5c8879, SUC_lexicon_tags_11 },
    { 0x627c887a, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3c8880, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x19bc8882, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afc8885, SUC_lexicon_tags_47 },
    { 0x1f9c8886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7c888a, SUC_lexicon_tags_12 },
    { 0x7f5c888b, SUC_lexicon_tags_19 },
    { 0xdcfc888c, SUC_lexicon_tags_12 },
    { 0x77bc888a, SUC_lexicon_tags_7 },
    { 0x795c888b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5fc8890, SUC_lexicon_tags_7 },
    { 0xc7bc8891, SUC_lexicon_tags_87 },
    { 0xc75c8891, SUC_lexicon_tags_28 },
    { 0x73c8890, SUC_lexicon_tags_47 },
    { 0xc7fc8894, SUC_lexicon_tags_17 },
    { 0x217c8895, SUC_lexicon_tags_6 },
    { 0xb65c8896, SUC_lexicon_tags_38 },
    { 0x6cfc8891, SUC_lexicon_tags_25 },
    { 0xb81c8898, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffc889b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4bc889d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17dc889f, SUC_lexicon_tags_17 },
    { 0x12bc88a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afc88a7, SUC_lexicon_tags_17 },
    { 0xd17c88a8, SUC_lexicon_tags_57 },
    { 0x4a5c88a9, SUC_lexicon_tags_7 },
    { 0x63c88aa, SUC_lexicon_tags_146 },
    { 0x963c88ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d9c88ad, SUC_lexicon_tags_26 },
    { 0xfc7c88ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dc88b2, SUC_lexicon_tags_28 },
    { 0xa1dc88b3, SUC_lexicon_tags_7 },
    { 0x421c88b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7fc88b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57c88bc, SUC_lexicon_tags_49 },
    { 0xc17c88bd, SUC_lexicon_tags_28 },
    { 0x5d9c88be, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1c88c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbc88c4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405c88c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7fdc88cb, SUC_lexicon_tags_0 },
    { 0x907c88cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bc88d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3c88d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa7c88d8, SUC_lexicon_tags_9 },
    { 0x361c88d9, SUC_lexicon_tags_17 },
    { 0x8afc88da, SUC_lexicon_tags_28 },
    { 0xa49c88db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5b1c88dd, SUC_lexicon_tags_1 },
    { 0xf6fc88de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa11c88e0, SUC_lexicon_tags_26 },
    { 0x1b9c88e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bc88e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bc88ed, SUC_lexicon_tags_6 },
    { 0x267c88ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dc88f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bc88f6, SUC_lexicon_tags_2 },
    { 0x237c88f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abc88fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173c88ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8adc8901, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadc8904, SUC_lexicon_tags_7 },
    { 0x3ddc8905, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf9c8907, SUC_lexicon_tags_6 },
    { 0xba5c8908, SUC_lexicon_tags_27 },
    { 0xa59c8908, SUC_lexicon_tags_64 },
    { 0xde9c8908, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x26dc890c, SUC_lexicon_tags_4 },
    { 0x967c890c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7c8911, SUC_lexicon_tags_23 },
    { 0x1bc8912, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3c8916, SUC_lexicon_tags_15 },
    { 0xccfc8916, SUC_lexicon_tags_7 },
    { 0x86dc8918, SUC_lexicon_tags_40 },
    { 0x27dc8918, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fc891d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777c8923, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3c8929, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dc892e, SUC_lexicon_tags_12 },
    { 0x25fc892f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a9c8931, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77c8936, SUC_lexicon_tags_19 },
    { 0x95dc8936, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fc893d, SUC_lexicon_tags_30 },
    { 0xacfc893e, SUC_lexicon_tags_12 },
    { 0x285c893e, SUC_lexicon_tags_26 },
    { 0xafc8940, SUC_lexicon_tags_19 },
    { 0x869c8941, SUC_lexicon_tags_60 },
    { 0xe3c893d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x64fc8945, SUC_lexicon_tags_30 },
    { 0xf5fc8946, SUC_lexicon_tags_23 },
    { 0xa4dc8947, SUC_lexicon_tags_57 },
    { 0xfdfc8948, SUC_lexicon_tags_33 },
    { 0x25dc8949, SUC_lexicon_tags_15 },
    { 0x19c8947, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x85c894d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe99c894f, SUC_lexicon_tags_2 },
    { 0xbf3c8950, SUC_lexicon_tags_37 },
    { 0xc17c894f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abc8956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb95c8958, SUC_lexicon_tags_16 },
    { 0xb85c8959, SUC_lexicon_tags_17 },
    { 0x175c8959, SUC_lexicon_tags_2 },
    { 0x275c8958, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5c8962, SUC_lexicon_tags_17 },
    { 0x8a1c8963, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x10dc8965, SUC_lexicon_tags_14 },
    { 0xaf9c8966, SUC_lexicon_tags_12 },
    { 0x5a5c8966, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfc896a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dc896e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bc8973, SUC_lexicon_tags_25 },
    { 0xd69c8973, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dc897b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887c897f, SUC_lexicon_tags_11 },
    { 0x503c897f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159c8984, SUC_lexicon_tags_7 },
    { 0x1fbc8985, SUC_lexicon_tags_0 },
    { 0xff5c8984, SUC_lexicon_tags_17 },
    { 0x357c8986, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5c5c8989, SUC_lexicon_tags_11 },
    { 0x4cfc898a, SUC_lexicon_tags_58 },
    { 0x87dc898a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67c898e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd7c8990, SUC_lexicon_tags_7 },
    { 0xb7dc8990, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x47c8993, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15c8996, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x553c8998, SUC_lexicon_tags_28 },
    { 0xbe5c8999, SUC_lexicon_tags_45 },
    { 0x625c899a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3c89a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791c89a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce5c89aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7c7c89ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x69dc89af, SUC_lexicon_tags_19 },
    { 0x92fc89b0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19c89b4, SUC_lexicon_tags_15 },
    { 0x299c89b5, SUC_lexicon_tags_63 },
    { 0x3dbc89b5, SUC_lexicon_tags_29 },
    { 0x1d5c89b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3c89bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dc89c0, SUC_lexicon_tags_11 },
    { 0x6b3c89c1, SUC_lexicon_tags_60 },
    { 0x475c89c2, SUC_lexicon_tags_55 },
    { 0xe53c89c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fbc89c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83bc89c7, SUC_lexicon_tags_26 },
    { 0xe87c89c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b1c89ca, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71c89d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x227c89d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5c89da, SUC_lexicon_tags_25 },
    { 0xc4fc89db, SUC_lexicon_tags_69 },
    { 0x1e5c89dc, SUC_lexicon_tags_37 },
    { 0xc73c89dc, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbc89e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe55c89e6, SUC_lexicon_tags_12 },
    { 0x111c89e6, SUC_lexicon_tags_12 },
    { 0x379c89e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13c89fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631c8a05, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151c8a0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597c8a10, SUC_lexicon_tags_32 },
    { 0x41bc8a11, SUC_lexicon_tags_25 },
    { 0x673c8a10, SUC_lexicon_tags_69 },
    { 0xd89c8a13, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465c8a17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5c8a1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20dc8a22, SUC_lexicon_tags_1 },
    { 0x3dfc8a23, SUC_lexicon_tags_12 },
    { 0xc4fc8a24, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bc8a28, SUC_lexicon_tags_4 },
    { 0x15dc8a29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abc8a2c, SUC_lexicon_tags_54 },
    { 0x99bc8a2d, SUC_lexicon_tags_60 },
    { 0x7bc8a2d, SUC_lexicon_tags_38 },
    { 0xb1dc8a2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9c8a32, SUC_lexicon_tags_39 },
    { 0xebc8a32, SUC_lexicon_tags_7 },
    { 0xe13c8a33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e3c8a36, SUC_lexicon_tags_91 },
    { 0xb77c8a36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2dbc8a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ddc8a3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7c8a48, SUC_lexicon_tags_27 },
    { 0xd75c8a48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21c8a4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19fc8a4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bc8a52, SUC_lexicon_tags_92 },
    { 0x9afc8a52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe99c8a55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x501c8a58, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafc8a5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fc8a62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x969c8a64, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbc5c8a66, SUC_lexicon_tags_27 },
    { 0x27fc8a67, SUC_lexicon_tags_8 },
    { 0xea1c8a68, SUC_lexicon_tags_13 },
    { 0x245c8a68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555c8a72, SUC_lexicon_tags_25 },
    { 0x163c8a72, SUC_lexicon_tags_29 },
    { 0xe99c8a74, SUC_lexicon_tags_43 },
    { 0xc8bc8a73, SUC_lexicon_tags_1 },
    { 0x123c8a76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffc8a79, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bc8a7c, SUC_lexicon_tags_0 },
    { 0x3cfc8a7c, SUC_lexicon_tags_1 },
    { 0x41bc8a7e, SUC_lexicon_tags_12 },
    { 0x847c8a7f, SUC_lexicon_tags_15 },
    { 0xe59c8a7e, SUC_lexicon_tags_7 },
    { 0x15dc8a80, SUC_lexicon_tags_37 },
    { 0x48fc8a82, SUC_lexicon_tags_31 },
    { 0x927c8a82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edc8a86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bc8a8a, SUC_lexicon_tags_38 },
    { 0xdcdc8a8b, SUC_lexicon_tags_10 },
    { 0x79fc8a8c, SUC_lexicon_tags_7 },
    { 0x541c8a8b, SUC_lexicon_tags_37 },
    { 0xbd1c8a8d, SUC_lexicon_tags_46 },
    { 0x6bdc8a8f, SUC_lexicon_tags_31 },
    { 0xee1c8a8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff5c8a92, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bc8a97, SUC_lexicon_tags_26 },
    { 0xec9c8a98, SUC_lexicon_tags_57 },
    { 0xfc5c8a98, SUC_lexicon_tags_17 },
    { 0x3adc8a99, SUC_lexicon_tags_47 },
    { 0x78bc8a9b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4b5c8a9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0fc8a9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bc8aa6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd9bc8aa8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf7c8aaa, SUC_lexicon_tags_11 },
    { 0xb6dc8aab, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fc8ab0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155c8ab4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe4fc8ab6, SUC_lexicon_tags_15 },
    { 0xdc1c8ab7, SUC_lexicon_tags_19 },
    { 0xb6bc8ab8, SUC_lexicon_tags_1 },
    { 0x1b7c8ab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41c8abc, SUC_lexicon_tags_12 },
    { 0x8d3c8abd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1c8ac2, SUC_lexicon_tags_6 },
    { 0x813c8ac3, SUC_lexicon_tags_37 },
    { 0x281c8ac4, SUC_lexicon_tags_63 },
    { 0x449c8ac4, SUC_lexicon_tags_7 },
    { 0x4a9c8ac6, SUC_lexicon_tags_25 },
    { 0x73bc8ac7, SUC_lexicon_tags_27 },
    { 0xee7c8ac7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfc8acb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5c8ace, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59c8ad7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7c8adb, SUC_lexicon_tags_47 },
    { 0x615c8adc, SUC_lexicon_tags_15 },
    { 0x79bc8add, SUC_lexicon_tags_6 },
    { 0xa41c8ade, SUC_lexicon_tags_12 },
    { 0xd4dc8adf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x521c8ae1, SUC_lexicon_tags_11 },
    { 0xa87c8ae2, SUC_lexicon_tags_37 },
    { 0x3c1c8ae1, SUC_lexicon_tags_15 },
    { 0xebdc8ae4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x811c8ae6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43c8aed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7cfc8aef, SUC_lexicon_tags_79 },
    { 0x3dc8aef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x93c8af3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bc8af6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x639c8af9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2adc8afe, SUC_lexicon_tags_2 },
    { 0x821c8aff, SUC_lexicon_tags_23 },
    { 0x233c8aff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7a1c8b02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9c8b05, SUC_lexicon_tags_18 },
    { 0x3edc8b06, SUC_lexicon_tags_38 },
    { 0xe43c8b06, SUC_lexicon_tags_43 },
    { 0xa8bc8b05, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3c8b0e, SUC_lexicon_tags_7 },
    { 0x93c8b0f, SUC_lexicon_tags_37 },
    { 0xcfbc8b0f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x321c8b12, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbc8b15, SUC_lexicon_tags_37 },
    { 0xf0dc8b16, SUC_lexicon_tags_43 },
    { 0x13dc8b16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bc8b1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17c8b1d, SUC_lexicon_tags_9 },
    { 0xadbc8b1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dc8b22, SUC_lexicon_tags_25 },
    { 0x99bc8b23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1c8b26, SUC_lexicon_tags_23 },
    { 0x15c8b26, SUC_lexicon_tags_146 },
    { 0x19c8b28, SUC_lexicon_tags_215 },
    { 0xf11c8b28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dc8b2e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09c8b33, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe65c8b35, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efc8b39, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dc8b3f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dc8b44, SUC_lexicon_tags_37 },
    { 0x7a9c8b45, SUC_lexicon_tags_2 },
    { 0xbcfc8b46, SUC_lexicon_tags_39 },
    { 0x273c8b44, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bc8b4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353c8b50, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853c8b54, SUC_lexicon_tags_8 },
    { 0xf5bc8b55, SUC_lexicon_tags_275 },
    { 0x343c8b56, SUC_lexicon_tags_15 },
    { 0x1cdc8b57, SUC_lexicon_tags_22 },
    { 0x1d1c8b57, SUC_lexicon_tags_14 },
    { 0x5d7c8b59, SUC_lexicon_tags_39 },
    { 0x7abc8b56, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45c8b60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdc8b67, SUC_lexicon_tags_37 },
    { 0x46fc8b68, SUC_lexicon_tags_2 },
    { 0x5fc8b69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adc8b78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x119c8b7b, SUC_lexicon_tags_17 },
    { 0xca3c8b7c, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0xc65c8b7e, SUC_lexicon_tags_37 },
    { 0x247c8b7e, SUC_lexicon_tags_49 },
    { 0x71dc8b7e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x169c8b83, SUC_lexicon_tags_21 },
    { 0x695c8b83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3c8b90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaf7c8b92, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x327c8b95, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bc8b9b, SUC_lexicon_tags_10 },
    { 0x9bfc8b9c, SUC_lexicon_tags_6 },
    { 0xb7dc8b9d, SUC_lexicon_tags_37 },
    { 0x171c8b9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7c8ba1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc9c8ba3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x233c8ba6, SUC_lexicon_tags_10 },
    { 0x78dc8ba6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dc8bab, SUC_lexicon_tags_463 },
    { 0xd27c8bab, SUC_lexicon_tags_37 },
    { 0x435c8bad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f5c8baf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207c8bb3, SUC_lexicon_tags_29 },
    { 0x2dfc8bb3, SUC_lexicon_tags_12 },
    { 0x9d9c8bb3, SUC_lexicon_tags_38 },
    { 0x7c3c8bb4, SUC_lexicon_tags_7 },
    { 0xd2bc8bb7, SUC_lexicon_tags_17 },
    { 0xdf9c8bb4, SUC_lexicon_tags_11 },
    { 0x659c8bb9, SUC_lexicon_tags_12 },
    { 0x88bc8bb9, SUC_lexicon_tags_43 },
    { 0xb3dc8bbb, SUC_lexicon_tags_2 },
    { 0x6bc8bbb, SUC_lexicon_tags_9 },
    { 0x915c8bb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x41dc8bbf, SUC_lexicon_tags_138 },
    { 0xb35c8bbf, SUC_lexicon_tags_33 },
    { 0x46dc8bc1, SUC_lexicon_tags_2 },
    { 0xecdc8bc2, SUC_lexicon_tags_68 },
    { 0x761c8bc3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03c8bc9, SUC_lexicon_tags_1 },
    { 0x913c8bc9, SUC_lexicon_tags_43 },
    { 0x3d7c8bca, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dc8bcf, SUC_lexicon_tags_31 },
    { 0x433c8bd0, SUC_lexicon_tags_15 },
    { 0x1bbc8bd0, SUC_lexicon_tags_6 },
    { 0x299c8bd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf63c8bd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dc8bd8, SUC_lexicon_tags_1 },
    { 0x331c8bd9, SUC_lexicon_tags_6 },
    { 0xedbc8bda, SUC_lexicon_tags_92 },
    { 0xdfdc8bdb, SUC_lexicon_tags_17 },
    { 0x537c8bda, SUC_lexicon_tags_11 },
    { 0x639c8bdd, SUC_lexicon_tags_8 },
    { 0x40bc8bde, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1c8be1, SUC_lexicon_tags_32 },
    { 0xecbc8be2, SUC_lexicon_tags_7 },
    { 0x237c8be3, SUC_lexicon_tags_13 },
    { 0x993c8be4, SUC_lexicon_tags_27 },
    { 0xcdfc8be5, SUC_lexicon_tags_34 },
    { 0xac3c8be5, SUC_lexicon_tags_37 },
    { 0xcbdc8be7, SUC_lexicon_tags_10 },
    { 0x89c8be5, SUC_lexicon_tags_37 },
    { 0x7a7c8be9, SUC_lexicon_tags_13 },
    { 0x929c8bea, SUC_lexicon_tags_14 },
    { 0x10bc8be4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83c8bee, SUC_lexicon_tags_54 },
    { 0x3dc8bef, SUC_lexicon_tags_11 },
    { 0x3ffc8bee, SUC_lexicon_tags_17 },
    { 0xde3c8bf1, SUC_lexicon_tags_37 },
    { 0x941c8bf0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc4fc8bf4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89dc8bf8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831c8c01, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x459c8c04, SUC_lexicon_tags_12 },
    { 0xd4bc8c05, SUC_lexicon_tags_7 },
    { 0x1fc8c06, SUC_lexicon_tags_30 },
    { 0x685c8c06, SUC_lexicon_tags_1 },
    { 0x811c8c07, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xac5c8c0a, SUC_lexicon_tags_7 },
    { 0x407c8c0b, SUC_lexicon_tags_25 },
    { 0xb1dc8c0b, SUC_lexicon_tags_1 },
    { 0xeddc8c0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd3fc8c0f, SUC_lexicon_tags_27 },
    { 0x6bdc8c10, SUC_lexicon_tags_0 },
    { 0xcf3c8c0f, SUC_lexicon_tags_30 },
    { 0xaffc8c12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaebc8c14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231c8c1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dc8c23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc21c8c25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dc8c29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb31c8c2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dc8c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x803c8c36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x103c8c38, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579c8c3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fc8c49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc39c8c4b, SUC_lexicon_tags_19 },
    { 0xd61c8c4c, SUC_lexicon_tags_33 },
    { 0x821c8c4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7c8c55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bc8c5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x169c8c5f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfc8c62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95c8c65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dc8c71, SUC_lexicon_tags_27 },
    { 0x37bc8c72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fc8c75, SUC_lexicon_tags_77 },
    { 0x63c8c75, SUC_lexicon_tags_30 },
    { 0x447c8c77, SUC_lexicon_tags_1 },
    { 0xbc8c78, SUC_lexicon_tags_17 },
    { 0x54bc8c79, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9c8c7f, SUC_lexicon_tags_28 },
    { 0x9fc8c7f, SUC_lexicon_tags_2 },
    { 0xd3c8c7f, SUC_lexicon_tags_2 },
    { 0x251c8c7f, SUC_lexicon_tags_12 },
    { 0x93c8c83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37c8c8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5c8c90, SUC_lexicon_tags_38 },
    { 0xdbdc8c91, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbc8c95, SUC_lexicon_tags_17 },
    { 0xc87c8c95, SUC_lexicon_tags_17 },
    { 0xb19c8c97, SUC_lexicon_tags_37 },
    { 0x819c8c98, SUC_lexicon_tags_22 },
    { 0xd17c8c99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fc8c9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397c8ca2, SUC_lexicon_tags_11 },
    { 0x85dc8ca3, SUC_lexicon_tags_45 },
    { 0x6d3c8ca4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511c8caa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc8cb2, SUC_lexicon_tags_15 },
    { 0x915c8cb2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321c8cb7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xeb3c8cb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebc8cbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bc8cc1, SUC_lexicon_tags_8 },
    { 0x901c8cc2, SUC_lexicon_tags_8 },
    { 0xbb5c8cc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7c8cc8, SUC_lexicon_tags_28 },
    { 0xc95c8cc8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3c8ccc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47c8cd0, SUC_lexicon_tags_33 },
    { 0xc59c8cd1, SUC_lexicon_tags_17 },
    { 0x8ffc8cd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x607c8cd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x201c8cd6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x613c8cd8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x355c8cda, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4a5c8cdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bc8ce3, SUC_lexicon_tags_37 },
    { 0x63fc8ce4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x343c8ce6, SUC_lexicon_tags_27 },
    { 0xc75c8ce7, SUC_lexicon_tags_71 },
    { 0x7c1c8ce8, SUC_lexicon_tags_38 },
    { 0x59c8ce9, SUC_lexicon_tags_30 },
    { 0xcbbc8ce8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x231c8cec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5c8cf2, SUC_lexicon_tags_2 },
    { 0xb6fc8cf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaddc8cf6, SUC_lexicon_tags_17 },
    { 0x3d7c8cf7, SUC_lexicon_tags_60 },
    { 0x363c8cf6, SUC_lexicon_tags_2 },
    { 0xdf9c8cf9, SUC_lexicon_tags_1 },
    { 0x4dc8cf9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x205c8cfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcdc8cff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277c8d03, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xba1c8d05, SUC_lexicon_tags_37 },
    { 0xb73c8d05, SUC_lexicon_tags_48 },
    { 0x727c8d07, SUC_lexicon_tags_0 },
    { 0x195c8d07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bc8d0d, SUC_lexicon_tags_7 },
    { 0x33fc8d0d, SUC_lexicon_tags_48 },
    { 0x547c8d0d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7c8d12, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3c8d15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9c8d1c, SUC_lexicon_tags_47 },
    { 0xd29c8d1c, SUC_lexicon_tags_8 },
    { 0x86bc8d1e, SUC_lexicon_tags_1 },
    { 0xc2fc8d1e, SUC_lexicon_tags_7 },
    { 0xf9fc8d20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fc8d23, SUC_lexicon_tags_7 },
    { 0x51bc8d24, SUC_lexicon_tags_1 },
    { 0xc8fc8d25, SUC_lexicon_tags_87 },
    { 0x515c8d26, SUC_lexicon_tags_74 },
    { 0x433c8d27, SUC_lexicon_tags_38 },
    { 0x517c8d25, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fc8d2e, SUC_lexicon_tags_1 },
    { 0x5c8d2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa17c8d31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbc8d34, SUC_lexicon_tags_11 },
    { 0xb23c8d35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd63c8d37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xca3c8d39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x497c8d3c, SUC_lexicon_tags_47 },
    { 0xb8dc8d3d, SUC_lexicon_tags_47 },
    { 0x7a7c8d3e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9c8d41, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6bfc8d43, SUC_lexicon_tags_1 },
    { 0x55c8d44, SUC_lexicon_tags_106 },
    { 0x8e1c8d45, SUC_lexicon_tags_17 },
    { 0xecfc8d45, SUC_lexicon_tags_33 },
    { 0x5b1c8d43, SUC_lexicon_tags_17 },
    { 0x671c8d48, SUC_lexicon_tags_15 },
    { 0xd1fc8d48, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x677c8d4b, SUC_lexicon_tags_43 },
    { 0xee1c8d4b, SUC_lexicon_tags_11 },
    { 0xd1bc8d4d, SUC_lexicon_tags_87 },
    { 0x345c8d4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c8d52, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2fdc8d54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11c8d57, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x717c8d59, SUC_lexicon_tags_6 },
    { 0x477c8d59, SUC_lexicon_tags_27 },
    { 0x9ffc8d59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x66dc8d5d, SUC_lexicon_tags_5 },
    { 0x383c8d5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbfdc8d60, SUC_lexicon_tags_27 },
    { 0x72fc8d61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedc8d66, SUC_lexicon_tags_38 },
    { 0x975c8d67, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5c8d6b, SUC_lexicon_tags_25 },
    { 0x9bc8d6b, SUC_lexicon_tags_47 },
    { 0xae5c8d6d, SUC_lexicon_tags_30 },
    { 0x8b5c8d6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03c8d72, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53c8d75, SUC_lexicon_tags_25 },
    { 0xa73c8d76, SUC_lexicon_tags_15 },
    { 0xf17c8d77, SUC_lexicon_tags_12 },
    { 0x81dc8d78, SUC_lexicon_tags_60 },
    { 0x67dc8d79, SUC_lexicon_tags_1 },
    { 0x131c8d79, SUC_lexicon_tags_25 },
    { 0x643c8d77, SUC_lexicon_tags_100 },
    { 0x8e5c8d7c, SUC_lexicon_tags_45 },
    { 0x6ebc8d7c, SUC_lexicon_tags_45 },
    { 0xf91c8d79, SUC_lexicon_tags_1 },
    { 0xe41c8d7f, SUC_lexicon_tags_25 },
    { 0x4f5c8d80, SUC_lexicon_tags_31 },
    { 0x3dbc8d80, SUC_lexicon_tags_15 },
    { 0x321c8d82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3c8d85, SUC_lexicon_tags_48 },
    { 0x6abc8d86, SUC_lexicon_tags_45 },
    { 0x42bc8d87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73c8d8b, SUC_lexicon_tags_73 },
    { 0x45bc8d8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9c8d93, SUC_lexicon_tags_15 },
    { 0x1e7c8d94, SUC_lexicon_tags_17 },
    { 0x15fc8d95, SUC_lexicon_tags_31 },
    { 0xc99c8d95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x143c8d99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc63c8d9b, SUC_lexicon_tags_33 },
    { 0x365c8d9c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5c8da5, SUC_lexicon_tags_27 },
    { 0xab5c8da5, SUC_lexicon_tags_47 },
    { 0x55fc8da7, SUC_lexicon_tags_1 },
    { 0x5f9c8da7, SUC_lexicon_tags_2 },
    { 0x685c8da6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5c8db4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x29dc8db6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8f9c8db8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dc8dbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539c8dc0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359c8dc4, SUC_lexicon_tags_25 },
    { 0xc8fc8dc5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267c8dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dc8dce, SUC_lexicon_tags_47 },
    { 0x145c8dcf, SUC_lexicon_tags_107 },
    { 0x8fc8dcf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4abc8dd2, SUC_lexicon_tags_11 },
    { 0x553c8dd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dc8dd6, SUC_lexicon_tags_23 },
    { 0xf63c8dd7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543c8ddb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x129c8dde, SUC_lexicon_tags_12 },
    { 0x175c8dde, SUC_lexicon_tags_37 },
    { 0xa21c8de0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3c8de4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bc8de7, SUC_lexicon_tags_37 },
    { 0x79c8de8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171c8dee, SUC_lexicon_tags_37 },
    { 0x681c8def, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47c8df4, SUC_lexicon_tags_60 },
    { 0x3b3c8df4, SUC_lexicon_tags_87 },
    { 0xf9fc8df4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3c8dfb, SUC_lexicon_tags_87 },
    { 0x78fc8dfc, SUC_lexicon_tags_2 },
    { 0x86dc8dfc, SUC_lexicon_tags_27 },
    { 0xd67c8dfe, SUC_lexicon_tags_87 },
    { 0xdd9c8dff, SUC_lexicon_tags_7 },
    { 0xb3bc8dfd, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x4f5c8e02, SUC_lexicon_tags_33 },
    { 0x68dc8e03, SUC_lexicon_tags_61 },
    { 0xd1fc8e03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x473c8e07, SUC_lexicon_tags_37 },
    { 0x761c8e08, SUC_lexicon_tags_6 },
    { 0xcbc8e07, SUC_lexicon_tags_7 },
    { 0x559c8e0a, SUC_lexicon_tags_0 },
    { 0x441c8e0b, SUC_lexicon_tags_33 },
    { 0x835c8e0c, SUC_lexicon_tags_11 },
    { 0x603c8e0d, SUC_lexicon_tags_31 },
    { 0x5cfc8e0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddc8e11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e3c8e13, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fc8e16, SUC_lexicon_tags_23 },
    { 0x6f7c8e17, SUC_lexicon_tags_9 },
    { 0xd33c8e16, SUC_lexicon_tags_12 },
    { 0xf55c8e17, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9bdc8e1b, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5c8e1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x261c8e20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c8e25, SUC_lexicon_tags_12 },
    { 0xa49c8e26, SUC_lexicon_tags_30 },
    { 0x7bbc8e26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41c8e2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25c8e34, SUC_lexicon_tags_129 },
    { 0x915c8e34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef9c8e38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x655c8e3b, SUC_lexicon_tags_25 },
    { 0x6cbc8e3b, SUC_lexicon_tags_45 },
    { 0xef1c8e3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bc8e45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x853c8e47, SUC_lexicon_tags_47 },
    { 0xbe9c8e47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x961c8e4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbc8e51, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x761c8e53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xca5c8e55, SUC_lexicon_tags_8 },
    { 0xe33c8e56, SUC_lexicon_tags_6 },
    { 0xdebc8e55, SUC_lexicon_tags_25 },
    { 0x367c8e58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823c8e5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73c8e5f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x539c8e61, SUC_lexicon_tags_15 },
    { 0xbbfc8e62, SUC_lexicon_tags_16 },
    { 0xf51c8e61, SUC_lexicon_tags_9 },
    { 0xd79c8e62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681c8e69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5c8e6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81c8e76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691c8e7b, SUC_lexicon_tags_65 },
    { 0x8a5c8e7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x357c8e7e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9c8e85, SUC_lexicon_tags_47 },
    { 0x605c8e86, SUC_lexicon_tags_2 },
    { 0x35bc8e85, SUC_lexicon_tags_48 },
    { 0xd89c8e86, SUC_lexicon_tags_25 },
    { 0x83c8e89, SUC_lexicon_tags_19 },
    { 0xf81c8e8a, SUC_lexicon_tags_14 },
    { 0x94fc8e88, SUC_lexicon_tags_17 },
    { 0x901c8e87, SUC_lexicon_tags_25 },
    { 0xcabc8e8d, SUC_lexicon_tags_50 },
    { 0x863c8e8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x569c8e90, SUC_lexicon_tags_9 },
    { 0x711c8e91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf15c8e93, SUC_lexicon_tags_37 },
    { 0xf03c8e94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebc8e99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9c8e9e, SUC_lexicon_tags_68 },
    { 0xc4bc8e9f, SUC_lexicon_tags_37 },
    { 0x967c8ea0, SUC_lexicon_tags_17 },
    { 0xd41c8ea1, SUC_lexicon_tags_12 },
    { 0x69c8ea2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377c8ea7, SUC_lexicon_tags_25 },
    { 0x3adc8ea8, SUC_lexicon_tags_19 },
    { 0xa5dc8ea9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993c8eb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcb3c8eb2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3c8eba, SUC_lexicon_tags_38 },
    { 0x315c8ebb, SUC_lexicon_tags_218 },
    { 0xedbc8ebc, SUC_lexicon_tags_45 },
    { 0xd85c8eba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb2dc8ebf, SUC_lexicon_tags_25 },
    { 0xb73c8ec0, SUC_lexicon_tags_6 },
    { 0x39dc8ec1, SUC_lexicon_tags_2 },
    { 0xe91c8ec1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bc8ec9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x907c8ecb, SUC_lexicon_tags_12 },
    { 0x52bc8ecc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1c8ed3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6bbc8ed5, SUC_lexicon_tags_16 },
    { 0x8b7c8ed6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2b3c8ed8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf99c8eda, SUC_lexicon_tags_23 },
    { 0xeb7c8eda, SUC_lexicon_tags_23 },
    { 0xaefc8eda, SUC_lexicon_tags_79 },
    { 0x497c8edd, SUC_lexicon_tags_0 },
    { 0x1d5c8edb, SUC_lexicon_tags_19 },
    { 0x54fc8edf, SUC_lexicon_tags_38 },
    { 0xc25c8ee0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52dc8ee2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dc8ee6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x735c8ee9, SUC_lexicon_tags_33 },
    { 0xf39c8eea, SUC_lexicon_tags_25 },
    { 0xe1fc8eeb, SUC_lexicon_tags_7 },
    { 0x147c8eec, SUC_lexicon_tags_37 },
    { 0x505c8eed, SUC_lexicon_tags_1 },
    { 0x8c1c8eec, SUC_lexicon_tags_37 },
    { 0x397c8eea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817c8ef6, SUC_lexicon_tags_32 },
    { 0xaa3c8ef7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221c8efe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd69c8f00, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabc8f04, SUC_lexicon_tags_12 },
    { 0xffc8f05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bc8f0a, SUC_lexicon_tags_55 },
    { 0x57dc8f0b, SUC_lexicon_tags_66 },
    { 0xa0fc8f0c, SUC_lexicon_tags_29 },
    { 0x243c8f0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x485c8f0f, SUC_lexicon_tags_17 },
    { 0xf59c8f10, SUC_lexicon_tags_1 },
    { 0x735c8f0f, SUC_lexicon_tags_4 },
    { 0x46bc8f10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95dc8f15, SUC_lexicon_tags_2 },
    { 0x27dc8f16, SUC_lexicon_tags_17 },
    { 0x5c8f15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625c8f1d, SUC_lexicon_tags_29 },
    { 0xdbbc8f1e, SUC_lexicon_tags_1 },
    { 0x569c8f1e, SUC_lexicon_tags_15 },
    { 0xce1c8f20, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xcd9c8f22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25c8f29, SUC_lexicon_tags_15 },
    { 0x291c8f2a, SUC_lexicon_tags_14 },
    { 0x5abc8f2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x265c8f2d, SUC_lexicon_tags_2 },
    { 0xf97c8f2d, SUC_lexicon_tags_30 },
    { 0x825c8f2f, SUC_lexicon_tags_15 },
    { 0xce3c8f30, SUC_lexicon_tags_47 },
    { 0x63c8f30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf59c8f33, SUC_lexicon_tags_11 },
    { 0x3cfc8f34, SUC_lexicon_tags_17 },
    { 0xd75c8f35, SUC_lexicon_tags_57 },
    { 0x629c8f35, SUC_lexicon_tags_25 },
    { 0xad1c8f36, SUC_lexicon_tags_11 },
    { 0xf75c8f37, SUC_lexicon_tags_9 },
    { 0xd83c8f37, SUC_lexicon_tags_15 },
    { 0xd59c8f39, SUC_lexicon_tags_43 },
    { 0x649c8f3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x851c8f3e, SUC_lexicon_tags_9 },
    { 0x4adc8f3e, SUC_lexicon_tags_77 },
    { 0x8e7c8f40, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5c8f46, SUC_lexicon_tags_47 },
    { 0xde9c8f47, SUC_lexicon_tags_12 },
    { 0x921c8f46, SUC_lexicon_tags_9 },
    { 0xb4bc8f49, SUC_lexicon_tags_87 },
    { 0xa75c8f47, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457c8f4e, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xd33c8f50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc8f53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1bc8f55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7c8f58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39c8f5b, SUC_lexicon_tags_63 },
    { 0xaa1c8f5c, SUC_lexicon_tags_7 },
    { 0xb07c8f5d, SUC_lexicon_tags_23 },
    { 0xfabc8f5d, SUC_lexicon_tags_14 },
    { 0x7e5c8f5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43bc8f61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19c8f64, SUC_lexicon_tags_41 },
    { 0xde9c8f64, SUC_lexicon_tags_11 },
    { 0xe47c8f66, SUC_lexicon_tags_27 },
    { 0x46fc8f67, SUC_lexicon_tags_70 },
    { 0xb13c8f64, SUC_lexicon_tags_45 },
    { 0x7b5c8f69, SUC_lexicon_tags_23 },
    { 0x3dfc8f6a, SUC_lexicon_tags_1 },
    { 0x3b3c8f6a, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3c8f75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bc8f78, SUC_lexicon_tags_1 },
    { 0x2b1c8f79, SUC_lexicon_tags_30 },
    { 0x819c8f7a, SUC_lexicon_tags_31 },
    { 0xccfc8f79, SUC_lexicon_tags_7 },
    { 0x26bc8f78, SUC_lexicon_tags_60 },
    { 0xecfc8f7b, SUC_lexicon_tags_6 },
    { 0x12dc8f7e, SUC_lexicon_tags_7 },
    { 0x7b7c8f7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17fc8f81, SUC_lexicon_tags_37 },
    { 0xb83c8f82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x205c8f85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87c8f8d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfc8f90, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9c8f93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dbc8f95, SUC_lexicon_tags_11 },
    { 0x767c8f96, SUC_lexicon_tags_8 },
    { 0x9b5c8f97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c7c8f99, SUC_lexicon_tags_17 },
    { 0x11dc8f9a, SUC_lexicon_tags_25 },
    { 0xe0bc8f9b, SUC_lexicon_tags_14 },
    { 0xc6fc8f9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5c8fa2, SUC_lexicon_tags_38 },
    { 0x9bfc8fa3, SUC_lexicon_tags_38 },
    { 0x919c8fa2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x44bc8fa6, SUC_lexicon_tags_11 },
    { 0xed1c8fa7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x555c8fa9, SUC_lexicon_tags_19 },
    { 0xfb7c8faa, SUC_lexicon_tags_15 },
    { 0xf6dc8fa9, SUC_lexicon_tags_29 },
    { 0xee3c8fa9, SUC_lexicon_tags_57 },
    { 0x96bc8fad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd39c8faf, SUC_lexicon_tags_27 },
    { 0x523c8faf, SUC_lexicon_tags_1 },
    { 0x9c3c8faf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x791c8fb3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9d1c8fb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fc8fba, SUC_lexicon_tags_2 },
    { 0x981c8fbb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87c8fbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dc8fc6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37c8fcc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143c8fd0, SUC_lexicon_tags_63 },
    { 0x6c5c8fd0, SUC_lexicon_tags_37 },
    { 0xba1c8fd2, SUC_lexicon_tags_25 },
    { 0x737c8fd1, SUC_lexicon_tags_47 },
    { 0xfbdc8fd1, SUC_lexicon_tags_37 },
    { 0x5d7c8fd5, SUC_lexicon_tags_66 },
    { 0xecfc8fd6, SUC_lexicon_tags_37 },
    { 0x621c8fd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbc8fda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3c8fdf, SUC_lexicon_tags_17 },
    { 0xdbc8fe0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfc8fe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5c8feb, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x1c1c8fed, SUC_lexicon_tags_45 },
    { 0x525c8fee, SUC_lexicon_tags_38 },
    { 0x41c8fed, SUC_lexicon_tags_16 },
    { 0x505c8ff0, SUC_lexicon_tags_25 },
    { 0x341c8ff1, SUC_lexicon_tags_7 },
    { 0x5cfc8ff2, SUC_lexicon_tags_46 },
    { 0x17c8ff3, SUC_lexicon_tags_11 },
    { 0x32fc8ff3, SUC_lexicon_tags_25 },
    { 0xb43c8fef, SUC_lexicon_tags_235 },
    { 0x4d7c8fef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17c8ff9, SUC_lexicon_tags_4 },
    { 0xa61c8ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x379c8ffc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fc9007, SUC_lexicon_tags_11 },
    { 0xc1fc9008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205c900c, SUC_lexicon_tags_10 },
    { 0x2bfc900c, SUC_lexicon_tags_2 },
    { 0x95bc900d, SUC_lexicon_tags_105 },
    { 0xa67c900f, SUC_lexicon_tags_17 },
    { 0x87fc9010, SUC_lexicon_tags_25 },
    { 0x45bc9011, SUC_lexicon_tags_80 },
    { 0x97bc9012, SUC_lexicon_tags_37 },
    { 0x2dc9013, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1c901a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x761c901c, SUC_lexicon_tags_37 },
    { 0x90bc901d, SUC_lexicon_tags_4 },
    { 0xc25c901d, SUC_lexicon_tags_14 },
    { 0x2cfc901c, SUC_lexicon_tags_11 },
    { 0x8bdc901d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fc902f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbc903b, SUC_lexicon_tags_2 },
    { 0xab5c903b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbc9040, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3c9042, SUC_lexicon_tags_19 },
    { 0xc4bc9042, SUC_lexicon_tags_25 },
    { 0x167c9044, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205c9050, SUC_lexicon_tags_4 },
    { 0x4a9c9051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53c9058, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1c905c, SUC_lexicon_tags_79 },
    { 0x887c905d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9c9062, SUC_lexicon_tags_34 },
    { 0xe69c9063, SUC_lexicon_tags_23 },
    { 0xaa3c9064, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x709c9066, SUC_lexicon_tags_11 },
    { 0x88bc9067, SUC_lexicon_tags_45 },
    { 0x379c9068, SUC_lexicon_tags_31 },
    { 0x5d5c9069, SUC_lexicon_tags_27 },
    { 0xe33c9069, SUC_lexicon_tags_35 },
    { 0x7ffc906a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793c9070, SUC_lexicon_tags_39 },
    { 0x4ebc9070, SUC_lexicon_tags_14 },
    { 0x65fc9072, SUC_lexicon_tags_47 },
    { 0xf17c9073, SUC_lexicon_tags_61 },
    { 0x319c9074, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1c9078, SUC_lexicon_tags_12 },
    { 0xe0bc9078, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3c907e, SUC_lexicon_tags_70 },
    { 0xc41c907f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63c9083, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbc9088, SUC_lexicon_tags_2 },
    { 0x879c9088, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03c908d, SUC_lexicon_tags_32 },
    { 0x535c908d, SUC_lexicon_tags_65 },
    { 0xd87c908f, SUC_lexicon_tags_17 },
    { 0x13fc908f, SUC_lexicon_tags_16 },
    { 0x91dc908d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89c9093, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdc9096, SUC_lexicon_tags_37 },
    { 0x339c9097, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbc909a, SUC_lexicon_tags_0 },
    { 0xa9bc909b, SUC_lexicon_tags_7 },
    { 0xf23c909c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fc90a2, SUC_lexicon_tags_25 },
    { 0xc95c90a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fc90a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5c90a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x247c90ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267c90b0, SUC_lexicon_tags_1 },
    { 0x3cbc90b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e5c90b3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7c90b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x763c90b9, SUC_lexicon_tags_7 },
    { 0x57c90ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccdc90bc, SUC_lexicon_tags_19 },
    { 0xdd9c90bc, SUC_lexicon_tags_2 },
    { 0x6c1c90be, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x1dc90c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1c90c5, SUC_lexicon_tags_25 },
    { 0xcd9c90c5, SUC_lexicon_tags_93 },
    { 0x5e5c90c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321c90cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adc90ce, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bc90d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x56dc90d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dc90d6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133c90da, SUC_lexicon_tags_1 },
    { 0x4b9c90db, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155c90e3, SUC_lexicon_tags_37 },
    { 0xb7bc90e3, SUC_lexicon_tags_1 },
    { 0x88fc90e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb57c90e7, SUC_lexicon_tags_23 },
    { 0xe0dc90e8, SUC_lexicon_tags_106 },
    { 0x8c1c90e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca5c90eb, SUC_lexicon_tags_1 },
    { 0x603c90ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebc90f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1c90f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x85dc90f8, SUC_lexicon_tags_17 },
    { 0x7e5c90f8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x261c90fb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbc90fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x941c9100, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1c9104, SUC_lexicon_tags_12 },
    { 0x5c9c9105, SUC_lexicon_tags_15 },
    { 0x6efc9106, SUC_lexicon_tags_1 },
    { 0xbfbc9106, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25dc9109, SUC_lexicon_tags_2 },
    { 0xc17c9109, SUC_lexicon_tags_38 },
    { 0x273c910b, SUC_lexicon_tags_47 },
    { 0x245c910b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7c9111, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5c9116, SUC_lexicon_tags_45 },
    { 0xcadc9117, SUC_lexicon_tags_37 },
    { 0xbc5c9118, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe43c911a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1e5c911c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3c9121, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bc9129, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2dfc912b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87c9131, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1c9135, SUC_lexicon_tags_57 },
    { 0xe1dc9136, SUC_lexicon_tags_14 },
    { 0x397c9137, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fc913a, SUC_lexicon_tags_26 },
    { 0x3e7c913b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc27c913d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x363c913f, SUC_lexicon_tags_43 },
    { 0x357c9140, SUC_lexicon_tags_15 },
    { 0xfcfc9140, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50bc9143, SUC_lexicon_tags_7 },
    { 0x1c1c9143, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe8fc9146, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x79bc9148, SUC_lexicon_tags_6 },
    { 0x2a7c9149, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903c914d, SUC_lexicon_tags_25 },
    { 0xb61c914e, SUC_lexicon_tags_43 },
    { 0x57fc914e, SUC_lexicon_tags_25 },
    { 0x3c7c914e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613c9156, SUC_lexicon_tags_19 },
    { 0x2bbc9157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dc915d, SUC_lexicon_tags_1 },
    { 0x1d3c915e, SUC_lexicon_tags_84 },
    { 0xc63c915e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7c9164, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x757c9167, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503c916b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679c9170, SUC_lexicon_tags_45 },
    { 0x21c9171, SUC_lexicon_tags_17 },
    { 0xe73c9171, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265c9179, SUC_lexicon_tags_27 },
    { 0x475c917a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1e9c917c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dc9180, SUC_lexicon_tags_4 },
    { 0x803c9181, SUC_lexicon_tags_0 },
    { 0xfc5c9182, SUC_lexicon_tags_0 },
    { 0x485c9182, SUC_lexicon_tags_13 },
    { 0xac5c9184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x407c9186, SUC_lexicon_tags_29 },
    { 0x10bc9187, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1c918a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x453c918d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7c9190, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bc9194, SUC_lexicon_tags_47 },
    { 0x8bdc9195, SUC_lexicon_tags_27 },
    { 0x5abc9196, SUC_lexicon_tags_14 },
    { 0xce1c9196, SUC_lexicon_tags_6 },
    { 0xbb5c9197, SUC_lexicon_tags_14 },
    { 0x4cbc9199, SUC_lexicon_tags_2 },
    { 0x265c9199, SUC_lexicon_tags_15 },
    { 0x589c919b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7c919e, SUC_lexicon_tags_31 },
    { 0x4bdc919f, SUC_lexicon_tags_38 },
    { 0x3d7c91a0, SUC_lexicon_tags_238 },
    { 0x695c919e, SUC_lexicon_tags_60 },
    { 0xd6dc91a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dc91a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5c91aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7c91b2, SUC_lexicon_tags_12 },
    { 0x227c91b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3c91bb, SUC_lexicon_tags_55 },
    { 0x573c91bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x793c91be, SUC_lexicon_tags_28 },
    { 0x7f9c91bf, SUC_lexicon_tags_38 },
    { 0x41c91c0, SUC_lexicon_tags_179 },
    { 0xc4bc91c0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x83bc91c3, SUC_lexicon_tags_25 },
    { 0x551c91c4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8a3c91c6, SUC_lexicon_tags_37 },
    { 0x835c91c7, SUC_lexicon_tags_6 },
    { 0xa5bc91c8, SUC_lexicon_tags_151 },
    { 0x565c91c9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfc91cd, SUC_lexicon_tags_11 },
    { 0xabc91ce, SUC_lexicon_tags_9 },
    { 0x6dfc91cf, SUC_lexicon_tags_27 },
    { 0xd5bc91cf, SUC_lexicon_tags_37 },
    { 0x49c91d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x67bc91d3, SUC_lexicon_tags_10 },
    { 0xb0dc91d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfffc91d6, SUC_lexicon_tags_37 },
    { 0xdf7c91d6, SUC_lexicon_tags_1 },
    { 0x751c91d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdedc91e0, SUC_lexicon_tags_26 },
    { 0x269c91e1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xf17c91e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bc91e6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dc91ea, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x75fc91ec, SUC_lexicon_tags_1 },
    { 0x3dbc91ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaedc91f0, SUC_lexicon_tags_20 },
    { 0x49dc91f0, SUC_lexicon_tags_318 },
    { 0x4f9c91f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddc91f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3c91f9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xb1bc91fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7c9202, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74dc920b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dc920f, SUC_lexicon_tags_37 },
    { 0x72fc920f, SUC_lexicon_tags_12 },
    { 0x875c9211, SUC_lexicon_tags_7 },
    { 0x499c9212, SUC_lexicon_tags_11 },
    { 0x7f5c9213, SUC_lexicon_tags_39 },
    { 0xeffc9214, SUC_lexicon_tags_29 },
    { 0x90fc9215, SUC_lexicon_tags_14 },
    { 0xd85c9216, SUC_lexicon_tags_2 },
    { 0xe23c9217, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67c9224, SUC_lexicon_tags_43 },
    { 0xf3c9225, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39c922c, SUC_lexicon_tags_15 },
    { 0xb15c922c, SUC_lexicon_tags_39 },
    { 0x3d1c922c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1cfc9230, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5c9233, SUC_lexicon_tags_69 },
    { 0xe59c9234, SUC_lexicon_tags_29 },
    { 0xd3c9235, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x101c9237, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09c923a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83c923d, SUC_lexicon_tags_26 },
    { 0x10dc923e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x50bc9241, SUC_lexicon_tags_15 },
    { 0xcb3c9242, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x563c9244, SUC_lexicon_tags_12 },
    { 0xc67c9245, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53dc9247, SUC_lexicon_tags_25 },
    { 0x801c9248, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5fc924a, SUC_lexicon_tags_38 },
    { 0xf73c924b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x109c924e, SUC_lexicon_tags_41 },
    { 0x2cfc924f, SUC_lexicon_tags_12 },
    { 0xaf3c9250, SUC_lexicon_tags_37 },
    { 0xc8fc9251, SUC_lexicon_tags_0 },
    { 0x721c9252, SUC_lexicon_tags_1 },
    { 0xadc9250, SUC_lexicon_tags_7 },
    { 0xcb7c9254, SUC_lexicon_tags_26 },
    { 0x83c9254, SUC_lexicon_tags_11 },
    { 0x6ffc9254, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc19c9258, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67c925e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63c9262, SUC_lexicon_tags_47 },
    { 0xdf9c9263, SUC_lexicon_tags_1 },
    { 0x5edc9264, SUC_lexicon_tags_542 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3c926d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbc9270, SUC_lexicon_tags_47 },
    { 0xda3c9270, SUC_lexicon_tags_47 },
    { 0xe2dc9272, SUC_lexicon_tags_1 },
    { 0xdf7c9273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41c9279, SUC_lexicon_tags_506 },
    { 0xe13c9279, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x521c927c, SUC_lexicon_tags_30 },
    { 0x59c927d, SUC_lexicon_tags_45 },
    { 0x48bc927d, SUC_lexicon_tags_55 },
    { 0x2f7c927f, SUC_lexicon_tags_45 },
    { 0x4dbc9280, SUC_lexicon_tags_33 },
    { 0x329c927f, SUC_lexicon_tags_25 },
    { 0xd37c9282, SUC_lexicon_tags_11 },
    { 0xacbc9283, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x445c9285, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x62fc9287, SUC_lexicon_tags_3 },
    { 0xb53c9288, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x159c928a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x553c928c, SUC_lexicon_tags_45 },
    { 0x5a5c928d, SUC_lexicon_tags_25 },
    { 0xb5bc928d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb27c9290, SUC_lexicon_tags_7 },
    { 0x8e1c9290, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x637c9294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bc9297, SUC_lexicon_tags_144 },
    { 0xf15c9298, SUC_lexicon_tags_2 },
    { 0x72bc9299, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33c929d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9c92a3, SUC_lexicon_tags_12 },
    { 0x683c92a4, SUC_lexicon_tags_12 },
    { 0xfb9c92a4, SUC_lexicon_tags_0 },
    { 0x6cfc92a6, SUC_lexicon_tags_46 },
    { 0x213c92a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x933c92a9, SUC_lexicon_tags_15 },
    { 0x81bc92aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fc92ae, SUC_lexicon_tags_25 },
    { 0x49fc92af, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbc92b3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03c92b6, SUC_lexicon_tags_29 },
    { 0x257c92b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd49c92b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6bdc92bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abc92c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9e1c92c2, SUC_lexicon_tags_2 },
    { 0xb6bc92c2, SUC_lexicon_tags_133 },
    { 0xfe3c92c3, SUC_lexicon_tags_27 },
    { 0x513c92c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdc92ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x465c92cd, SUC_lexicon_tags_11 },
    { 0x30bc92ce, SUC_lexicon_tags_742 },
    { 0xf5fc92ce, SUC_lexicon_tags_1 },
    { 0x843c92d0, SUC_lexicon_tags_26 },
    { 0xefc92d1, SUC_lexicon_tags_11 },
    { 0x5fdc92d2, SUC_lexicon_tags_30 },
    { 0xa85c92d3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdc92d7, SUC_lexicon_tags_11 },
    { 0x82bc92d8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7ebc92da, SUC_lexicon_tags_37 },
    { 0xc9fc92da, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dc92e0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfc92e5, SUC_lexicon_tags_15 },
    { 0x597c92e6, SUC_lexicon_tags_25 },
    { 0xb5fc92e6, SUC_lexicon_tags_23 },
    { 0x8c3c92e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbafc92ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d5c92ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3c92f0, SUC_lexicon_tags_33 },
    { 0xe75c92f1, SUC_lexicon_tags_23 },
    { 0xabdc92f2, SUC_lexicon_tags_146 },
    { 0x3efc92f2, SUC_lexicon_tags_15 },
    { 0xa79c92f4, SUC_lexicon_tags_3 },
    { 0x6b7c92f2, SUC_lexicon_tags_274 },
    { 0, NULL },
    { 0x37fc92f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa33c92f9, SUC_lexicon_tags_11 },
    { 0x419c92f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x51dc92fc, SUC_lexicon_tags_9 },
    { 0x1bfc92fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1c3c92ff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x241c9302, SUC_lexicon_tags_66 },
    { 0xbe5c9303, SUC_lexicon_tags_8 },
    { 0x103c9304, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167c930b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc79c930d, SUC_lexicon_tags_6 },
    { 0xc25c930d, SUC_lexicon_tags_37 },
    { 0x539c930f, SUC_lexicon_tags_37 },
    { 0x9a7c930f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x929c9312, SUC_lexicon_tags_57 },
    { 0x3c9c9313, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x791c9315, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9abc9317, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x157c931a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x56fc931c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe2fc931e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x379c9321, SUC_lexicon_tags_1 },
    { 0x8f3c9322, SUC_lexicon_tags_17 },
    { 0xb23c9321, SUC_lexicon_tags_17 },
    { 0x4ebc9324, SUC_lexicon_tags_1 },
    { 0x50fc9325, SUC_lexicon_tags_29 },
    { 0xb8bc9324, SUC_lexicon_tags_12 },
    { 0x213c9327, SUC_lexicon_tags_21 },
    { 0xf19c9328, SUC_lexicon_tags_17 },
    { 0xaf1c9323, SUC_lexicon_tags_37 },
    { 0x919c932a, SUC_lexicon_tags_19 },
    { 0x83fc932b, SUC_lexicon_tags_38 },
    { 0x599c932a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5c9330, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbc9334, SUC_lexicon_tags_1 },
    { 0x201c9335, SUC_lexicon_tags_35 },
    { 0xbb9c9334, SUC_lexicon_tags_30 },
    { 0x557c9336, SUC_lexicon_tags_45 },
    { 0x12fc9338, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x323c933a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5c933d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfc9340, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1d9c9342, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x377c9344, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbfdc9346, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a1c9348, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dc934d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc4dc934f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x29bc9351, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x863c9354, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe63c9356, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309c935f, SUC_lexicon_tags_37 },
    { 0x747c9360, SUC_lexicon_tags_1 },
    { 0x88dc9360, SUC_lexicon_tags_57 },
    { 0x835c9362, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x491c9364, SUC_lexicon_tags_45 },
    { 0x3c9365, SUC_lexicon_tags_1 },
    { 0xe37c9366, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123c936e, SUC_lexicon_tags_25 },
    { 0x549c936f, SUC_lexicon_tags_15 },
    { 0x525c9370, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485c9377, SUC_lexicon_tags_62 },
    { 0xbd7c9377, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15c937d, SUC_lexicon_tags_12 },
    { 0xf79c937d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x171c9381, SUC_lexicon_tags_47 },
    { 0x347c9382, SUC_lexicon_tags_68 },
    { 0x9e9c9382, SUC_lexicon_tags_43 },
    { 0xf89c9384, SUC_lexicon_tags_31 },
    { 0xf33c9382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15c9389, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc9390, SUC_lexicon_tags_1 },
    { 0x153c9391, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d3c9393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07c9397, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647c939f, SUC_lexicon_tags_12 },
    { 0xe5c93a0, SUC_lexicon_tags_25 },
    { 0x40dc93a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7c93a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x211c93a7, SUC_lexicon_tags_37 },
    { 0x9cfc93a7, SUC_lexicon_tags_70 },
    { 0x3e1c93a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bc93ad, SUC_lexicon_tags_48 },
    { 0x331c93ae, SUC_lexicon_tags_37 },
    { 0x23fc93ad, SUC_lexicon_tags_69 },
    { 0x753c93b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x299c93b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19bc93b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3fc93ba, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebc93bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1c93c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x24dc93c6, SUC_lexicon_tags_6 },
    { 0x6bfc93c7, SUC_lexicon_tags_11 },
    { 0xb3dc93c7, SUC_lexicon_tags_45 },
    { 0xb7c93c9, SUC_lexicon_tags_23 },
    { 0x8fdc93ca, SUC_lexicon_tags_11 },
    { 0x8bbc93c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddc93ce, SUC_lexicon_tags_68 },
    { 0x9c1c93cf, SUC_lexicon_tags_47 },
    { 0xe9bc93ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x987c93d3, SUC_lexicon_tags_35 },
    { 0x113c93d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87c93d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x521c93db, SUC_lexicon_tags_45 },
    { 0x8c5c93dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3c93df, SUC_lexicon_tags_26 },
    { 0x645c93e0, SUC_lexicon_tags_25 },
    { 0x7a3c93e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a1c93e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefc93e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89c93ea, SUC_lexicon_tags_9 },
    { 0xf49c93eb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1c93f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa41c93f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x799c93f7, SUC_lexicon_tags_25 },
    { 0x44dc93f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x39bc93fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb49c93fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x543c93fe, SUC_lexicon_tags_15 },
    { 0x7e7c93fe, SUC_lexicon_tags_12 },
    { 0x895c93fe, SUC_lexicon_tags_2 },
    { 0xaebc9401, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bc9409, SUC_lexicon_tags_26 },
    { 0xd41c940a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9c940d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c9411, SUC_lexicon_tags_31 },
    { 0xe5dc9412, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2b7c9414, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151c9418, SUC_lexicon_tags_28 },
    { 0xfc1c9419, SUC_lexicon_tags_15 },
    { 0x3edc941a, SUC_lexicon_tags_25 },
    { 0x143c941a, SUC_lexicon_tags_15 },
    { 0xeebc941a, SUC_lexicon_tags_0 },
    { 0xeddc941d, SUC_lexicon_tags_2 },
    { 0xa9fc9418, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05c9428, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x45fc942a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x297c942c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b7c942e, SUC_lexicon_tags_23 },
    { 0xf6fc942f, SUC_lexicon_tags_15 },
    { 0x3fdc9430, SUC_lexicon_tags_38 },
    { 0xc23c9431, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fc9434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69c9438, SUC_lexicon_tags_66 },
    { 0xe45c9438, SUC_lexicon_tags_37 },
    { 0x3c7c943a, SUC_lexicon_tags_16 },
    { 0x34fc943a, SUC_lexicon_tags_27 },
    { 0x901c943c, SUC_lexicon_tags_6 },
    { 0xddbc943d, SUC_lexicon_tags_46 },
    { 0x3e3c943d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917c9443, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf57c9445, SUC_lexicon_tags_46 },
    { 0x941c9446, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3c944a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c944f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4dc9451, SUC_lexicon_tags_60 },
    { 0x5bc9452, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x413c9454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3c9459, SUC_lexicon_tags_2 },
    { 0xee3c945a, SUC_lexicon_tags_7 },
    { 0xf6dc945a, SUC_lexicon_tags_14 },
    { 0x697c945c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179c9463, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307c9468, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf09c946a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x303c946c, SUC_lexicon_tags_7 },
    { 0x421c946d, SUC_lexicon_tags_19 },
    { 0x2e7c946d, SUC_lexicon_tags_26 },
    { 0x433c946f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bc9471, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe5c9473, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73c947f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3dc9481, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205c9487, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fc948b, SUC_lexicon_tags_49 },
    { 0x7fdc948c, SUC_lexicon_tags_25 },
    { 0xf5dc948d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3f3c948f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x143c9491, SUC_lexicon_tags_14 },
    { 0x17c9492, SUC_lexicon_tags_11 },
    { 0x23bc9493, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51c9497, SUC_lexicon_tags_46 },
    { 0xc89c9497, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281c949d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bc94a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bc94a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583c94a9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fc94ae, SUC_lexicon_tags_19 },
    { 0x485c94af, SUC_lexicon_tags_13 },
    { 0x439c94af, SUC_lexicon_tags_55 },
    { 0xcb5c94b1, SUC_lexicon_tags_1 },
    { 0x225c94b0, SUC_lexicon_tags_12 },
    { 0xc81c94b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bc94ba, SUC_lexicon_tags_1 },
    { 0xd1bc94bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9c94bf, SUC_lexicon_tags_47 },
    { 0x651c94c0, SUC_lexicon_tags_47 },
    { 0x7cbc94c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2edc94c4, SUC_lexicon_tags_15 },
    { 0xff3c94c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x699c94c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5c94cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fc94d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c94d6, SUC_lexicon_tags_37 },
    { 0xca9c94d6, SUC_lexicon_tags_81 },
    { 0x187c94d8, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x449c94da, SUC_lexicon_tags_48 },
    { 0x3e5c94da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2abc94dd, SUC_lexicon_tags_5 },
    { 0x3ffc94de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfc94e2, SUC_lexicon_tags_23 },
    { 0x69bc94e3, SUC_lexicon_tags_38 },
    { 0x835c94e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x927c94e6, SUC_lexicon_tags_66 },
    { 0xd29c94e7, SUC_lexicon_tags_12 },
    { 0xf83c94e8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387c94ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffc94f1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf37c94f3, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435c94f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef7c94fb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x85bc94fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fc9501, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x40fc9503, SUC_lexicon_tags_10 },
    { 0xebfc9504, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bc9507, SUC_lexicon_tags_172 },
    { 0xe1dc9508, SUC_lexicon_tags_19 },
    { 0xadc9507, SUC_lexicon_tags_25 },
    { 0xfdbc9507, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63c950e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493c9512, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1c9515, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x613c9518, SUC_lexicon_tags_525 },
    { 0, NULL },
    { 0x9fdc951a, SUC_lexicon_tags_8 },
    { 0x69dc951b, SUC_lexicon_tags_79 },
    { 0x9bfc951a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd17c951e, SUC_lexicon_tags_14 },
    { 0x1ddc951f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdc9c9521, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fc9524, SUC_lexicon_tags_12 },
    { 0xd83c9525, SUC_lexicon_tags_7 },
    { 0x725c9525, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf5dc9528, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623c952c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73c952f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7c9532, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1c9539, SUC_lexicon_tags_48 },
    { 0x9c9539, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x14dc953c, SUC_lexicon_tags_10 },
    { 0x59fc953d, SUC_lexicon_tags_25 },
    { 0xc21c953e, SUC_lexicon_tags_48 },
    { 0x89bc953d, SUC_lexicon_tags_7 },
    { 0xf7bc953e, SUC_lexicon_tags_23 },
    { 0x33fc9541, SUC_lexicon_tags_29 },
    { 0x331c9542, SUC_lexicon_tags_27 },
    { 0xe17c9543, SUC_lexicon_tags_15 },
    { 0x9c3c9544, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x14dc9546, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77c9549, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a5c954b, SUC_lexicon_tags_21 },
    { 0x77c954c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8adc954e, SUC_lexicon_tags_158 },
    { 0x7e3c954f, SUC_lexicon_tags_38 },
    { 0x1dbc954f, SUC_lexicon_tags_31 },
    { 0xb6fc9551, SUC_lexicon_tags_48 },
    { 0xd47c9551, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90fc9554, SUC_lexicon_tags_62 },
    { 0xe53c9555, SUC_lexicon_tags_25 },
    { 0xadfc9556, SUC_lexicon_tags_1 },
    { 0x9c7c9555, SUC_lexicon_tags_39 },
    { 0x1abc9558, SUC_lexicon_tags_2 },
    { 0x4a5c9559, SUC_lexicon_tags_7 },
    { 0xa67c9558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3c955f, SUC_lexicon_tags_0 },
    { 0xd81c9560, SUC_lexicon_tags_7 },
    { 0x9e7c9560, SUC_lexicon_tags_28 },
    { 0x7e9c9562, SUC_lexicon_tags_1 },
    { 0x32fc9560, SUC_lexicon_tags_37 },
    { 0x4b1c9564, SUC_lexicon_tags_31 },
    { 0x43bc9560, SUC_lexicon_tags_25 },
    { 0x81fc9566, SUC_lexicon_tags_87 },
    { 0x253c9567, SUC_lexicon_tags_80 },
    { 0xd91c9561, SUC_lexicon_tags_35 },
    { 0x15fc9564, SUC_lexicon_tags_14 },
    { 0x579c9566, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd09c956c, SUC_lexicon_tags_14 },
    { 0x94fc956c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adc9570, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7c9574, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87c9577, SUC_lexicon_tags_33 },
    { 0x935c9577, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1c957b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f1c957d, SUC_lexicon_tags_37 },
    { 0xee3c957e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53c9581, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebc9589, SUC_lexicon_tags_15 },
    { 0xb2dc9589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x679c958d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9c9590, SUC_lexicon_tags_49 },
    { 0x1f1c9591, SUC_lexicon_tags_16 },
    { 0x315c9592, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x161c9594, SUC_lexicon_tags_17 },
    { 0x755c9595, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe09c9597, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc63c9599, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6dbc959b, SUC_lexicon_tags_25 },
    { 0x39fc959c, SUC_lexicon_tags_11 },
    { 0xffc959d, SUC_lexicon_tags_25 },
    { 0xbf7c959e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fc95a7, SUC_lexicon_tags_13 },
    { 0x3cdc95a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bc95ab, SUC_lexicon_tags_23 },
    { 0x381c95ac, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x923c95ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9c95b1, SUC_lexicon_tags_27 },
    { 0xc5dc95b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83c95b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea7c95b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5adc95bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fc95bf, SUC_lexicon_tags_8 },
    { 0x365c95c0, SUC_lexicon_tags_101 },
    { 0x7ebc95c1, SUC_lexicon_tags_37 },
    { 0x8a5c95c2, SUC_lexicon_tags_1 },
    { 0xe59c95c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18fc95c5, SUC_lexicon_tags_17 },
    { 0x687c95c5, SUC_lexicon_tags_9 },
    { 0xc2bc95c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1c95ca, SUC_lexicon_tags_11 },
    { 0x6a9c95cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61c95cd, SUC_lexicon_tags_15 },
    { 0xa87c95ce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x18bc95d1, SUC_lexicon_tags_24 },
    { 0x76dc95d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dc95d5, SUC_lexicon_tags_10 },
    { 0xeebc95d6, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0x479c95d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf3c95da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x887c95dc, SUC_lexicon_tags_30 },
    { 0x3fdc95dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e7c95df, SUC_lexicon_tags_11 },
    { 0x411c95e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dc95ea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb47c95ec, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5c95f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x385c95f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf4bc95f4, SUC_lexicon_tags_37 },
    { 0x7e5c95f4, SUC_lexicon_tags_37 },
    { 0xfdc95f6, SUC_lexicon_tags_8 },
    { 0x3ebc95f5, SUC_lexicon_tags_25 },
    { 0x769c95f8, SUC_lexicon_tags_25 },
    { 0x97bc95f9, SUC_lexicon_tags_37 },
    { 0x725c95fa, SUC_lexicon_tags_12 },
    { 0xb0bc95fb, SUC_lexicon_tags_37 },
    { 0xe5bc95fa, SUC_lexicon_tags_23 },
    { 0x285c95fa, SUC_lexicon_tags_38 },
    { 0xf97c95f5, SUC_lexicon_tags_45 },
    { 0x195c95f9, SUC_lexicon_tags_11 },
    { 0x40bc95f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x36dc9603, SUC_lexicon_tags_10 },
    { 0x8a9c9604, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf5c9606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727c960f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x773c9612, SUC_lexicon_tags_65 },
    { 0x5a1c9613, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dc9617, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x169c961a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71c9623, SUC_lexicon_tags_25 },
    { 0x1dfc9624, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107c9629, SUC_lexicon_tags_46 },
    { 0x397c962a, SUC_lexicon_tags_11 },
    { 0x389c962a, SUC_lexicon_tags_19 },
    { 0x7c7c962c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffc962f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419c9636, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbc963d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeebc963f, SUC_lexicon_tags_25 },
    { 0x225c963f, SUC_lexicon_tags_21 },
    { 0x3e3c9641, SUC_lexicon_tags_9 },
    { 0x8a3c963f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dc9645, SUC_lexicon_tags_31 },
    { 0x88fc9646, SUC_lexicon_tags_27 },
    { 0xe11c9645, SUC_lexicon_tags_37 },
    { 0x895c9646, SUC_lexicon_tags_14 },
    { 0x525c9649, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35c9653, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1c9656, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc47c9658, SUC_lexicon_tags_41 },
    { 0x613c9658, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9c965c, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xcabc965f, SUC_lexicon_tags_32 },
    { 0xd55c9660, SUC_lexicon_tags_47 },
    { 0x163c9661, SUC_lexicon_tags_37 },
    { 0xae9c965f, SUC_lexicon_tags_17 },
    { 0xc5bc9663, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fc9667, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x287c966a, SUC_lexicon_tags_35 },
    { 0x383c966a, SUC_lexicon_tags_742 },
    { 0xa5bc966c, SUC_lexicon_tags_2 },
    { 0xb45c966c, SUC_lexicon_tags_45 },
    { 0x815c966c, SUC_lexicon_tags_79 },
    { 0x1a7c966f, SUC_lexicon_tags_8 },
    { 0x67fc966f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5c9675, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65c967a, SUC_lexicon_tags_55 },
    { 0x7cfc967b, SUC_lexicon_tags_10 },
    { 0x4cfc967a, SUC_lexicon_tags_21 },
    { 0x785c967d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1c9680, SUC_lexicon_tags_37 },
    { 0xe99c9681, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x415c9683, SUC_lexicon_tags_30 },
    { 0xdc1c9684, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571c968b, SUC_lexicon_tags_17 },
    { 0x371c968c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13c968f, SUC_lexicon_tags_37 },
    { 0x3d3c9690, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x529c9692, SUC_lexicon_tags_38 },
    { 0xed9c9692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03c9699, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x45bc969b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bc969e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745c96a4, SUC_lexicon_tags_2 },
    { 0xfedc96a5, SUC_lexicon_tags_12 },
    { 0x1cdc96a6, SUC_lexicon_tags_17 },
    { 0x1b7c96a7, SUC_lexicon_tags_2 },
    { 0x1d9c96a6, SUC_lexicon_tags_37 },
    { 0x309c96a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5c96b0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5c96b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x161c96b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153c96ba, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271c96be, SUC_lexicon_tags_11 },
    { 0x8fdc96bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2fc96c1, SUC_lexicon_tags_6 },
    { 0x1dbc96c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1c96ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3c96d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa31c96d3, SUC_lexicon_tags_1 },
    { 0x1a7c96d4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x397c96d6, SUC_lexicon_tags_17 },
    { 0x1edc96d7, SUC_lexicon_tags_2 },
    { 0x85bc96d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22dc96da, SUC_lexicon_tags_0 },
    { 0x101c96db, SUC_lexicon_tags_15 },
    { 0x825c96dc, SUC_lexicon_tags_37 },
    { 0x661c96dd, SUC_lexicon_tags_7 },
    { 0xf5bc96dd, SUC_lexicon_tags_11 },
    { 0xf1c96df, SUC_lexicon_tags_33 },
    { 0x3b7c96da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x781c96e2, SUC_lexicon_tags_17 },
    { 0xa23c96e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efc96e9, SUC_lexicon_tags_49 },
    { 0x9ddc96ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95c96ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7c96f4, SUC_lexicon_tags_17 },
    { 0x283c96f5, SUC_lexicon_tags_16 },
    { 0xc47c96f6, SUC_lexicon_tags_0 },
    { 0x905c96f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebc96fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76bc96fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x815c96ff, SUC_lexicon_tags_10 },
    { 0xe01c9700, SUC_lexicon_tags_7 },
    { 0x93fc9701, SUC_lexicon_tags_63 },
    { 0x643c9702, SUC_lexicon_tags_17 },
    { 0xba1c9703, SUC_lexicon_tags_63 },
    { 0x515c9703, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x379c9706, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bc9709, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589c970f, SUC_lexicon_tags_17 },
    { 0xfb9c970f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b9c9712, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c9716, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7c971b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddc971e, SUC_lexicon_tags_38 },
    { 0x4cbc971f, SUC_lexicon_tags_0 },
    { 0x21dc9720, SUC_lexicon_tags_26 },
    { 0xef9c9721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadc9728, SUC_lexicon_tags_7 },
    { 0xdc3c9729, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfc972c, SUC_lexicon_tags_38 },
    { 0x42bc972c, SUC_lexicon_tags_7 },
    { 0xc11c972e, SUC_lexicon_tags_46 },
    { 0x2fbc972f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3e3c9731, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcffc9734, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ddc9736, SUC_lexicon_tags_60 },
    { 0xe55c9737, SUC_lexicon_tags_15 },
    { 0x7fc9737, SUC_lexicon_tags_70 },
    { 0xfe5c9739, SUC_lexicon_tags_11 },
    { 0xd67c9738, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x271c973c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5c973f, SUC_lexicon_tags_21 },
    { 0xfebc9740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1c9743, SUC_lexicon_tags_7 },
    { 0xd83c9744, SUC_lexicon_tags_37 },
    { 0x7c3c9743, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59c974a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc4fc974c, SUC_lexicon_tags_15 },
    { 0x331c974d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dc9755, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5c975c, SUC_lexicon_tags_14 },
    { 0xbe7c975d, SUC_lexicon_tags_11 },
    { 0xacbc975c, SUC_lexicon_tags_0 },
    { 0x9e1c975d, SUC_lexicon_tags_60 },
    { 0x1a7c9760, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x333c9763, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x75fc9765, SUC_lexicon_tags_15 },
    { 0xb31c9766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x227c9768, SUC_lexicon_tags_14 },
    { 0x75dc9769, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137c9770, SUC_lexicon_tags_43 },
    { 0xf33c9770, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237c9776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdc977c, SUC_lexicon_tags_11 },
    { 0x5cfc977d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7c9783, SUC_lexicon_tags_1 },
    { 0xd3c9783, SUC_lexicon_tags_2 },
    { 0xe3c9785, SUC_lexicon_tags_11 },
    { 0xa2dc9784, SUC_lexicon_tags_13 },
    { 0x40dc9787, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303c978d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fc9790, SUC_lexicon_tags_64 },
    { 0xed5c9791, SUC_lexicon_tags_37 },
    { 0xef7c9790, SUC_lexicon_tags_10 },
    { 0x107c9793, SUC_lexicon_tags_25 },
    { 0xaa1c9794, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x807c9797, SUC_lexicon_tags_7 },
    { 0xc87c9797, SUC_lexicon_tags_31 },
    { 0x62bc9799, SUC_lexicon_tags_14 },
    { 0x517c979a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3c979e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adc97a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1c97a6, SUC_lexicon_tags_160 },
    { 0x96dc97a7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51c97ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefc97b0, SUC_lexicon_tags_23 },
    { 0x27c97b1, SUC_lexicon_tags_17 },
    { 0xda9c97b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a9c97b4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1c97b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x96fc97b9, SUC_lexicon_tags_7 },
    { 0xc0bc97ba, SUC_lexicon_tags_37 },
    { 0xb5fc97bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbc7c97bd, SUC_lexicon_tags_68 },
    { 0xedfc97be, SUC_lexicon_tags_60 },
    { 0xe59c97bf, SUC_lexicon_tags_213 },
    { 0x159c97be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc97c3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdc97c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543c97ce, SUC_lexicon_tags_48 },
    { 0xd3bc97cf, SUC_lexicon_tags_35 },
    { 0x967c97d0, SUC_lexicon_tags_1 },
    { 0x9e7c97cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebc97db, SUC_lexicon_tags_105 },
    { 0xbe1c97dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39c97df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddc97e2, SUC_lexicon_tags_11 },
    { 0xb71c97e3, SUC_lexicon_tags_17 },
    { 0x73bc97e3, SUC_lexicon_tags_7 },
    { 0x817c97e2, SUC_lexicon_tags_11 },
    { 0xd91c97e5, SUC_lexicon_tags_15 },
    { 0x291c97e7, SUC_lexicon_tags_12 },
    { 0xf1c97e5, SUC_lexicon_tags_14 },
    { 0x80fc97e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x287c97eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9c97ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdc97f2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483c97f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bc97fa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31c97ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15fc9801, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5c9804, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bc9808, SUC_lexicon_tags_47 },
    { 0xcc9c9809, SUC_lexicon_tags_14 },
    { 0x8f5c9809, SUC_lexicon_tags_60 },
    { 0x559c9809, SUC_lexicon_tags_27 },
    { 0x161c980a, SUC_lexicon_tags_17 },
    { 0xe93c980c, SUC_lexicon_tags_31 },
    { 0x863c980e, SUC_lexicon_tags_47 },
    { 0x609c980f, SUC_lexicon_tags_255 },
    { 0x199c980e, SUC_lexicon_tags_79 },
    { 0xa2bc9809, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7c9815, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9c9818, SUC_lexicon_tags_7 },
    { 0x8f9c9818, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x8adc981b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfc981f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203c9824, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97c9827, SUC_lexicon_tags_11 },
    { 0x18dc9828, SUC_lexicon_tags_15 },
    { 0x581c9829, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3c982e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31c9831, SUC_lexicon_tags_19 },
    { 0x41fc9832, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7c9839, SUC_lexicon_tags_68 },
    { 0x175c9839, SUC_lexicon_tags_37 },
    { 0xb37c9839, SUC_lexicon_tags_7 },
    { 0x755c983c, SUC_lexicon_tags_27 },
    { 0xf35c983d, SUC_lexicon_tags_12 },
    { 0x78dc983e, SUC_lexicon_tags_47 },
    { 0xb2bc983d, SUC_lexicon_tags_77 },
    { 0x975c9840, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfc9844, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183c984f, SUC_lexicon_tags_25 },
    { 0xdf7c9850, SUC_lexicon_tags_5 },
    { 0x633c9851, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45c9856, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dc9859, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38fc985b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23c985e, SUC_lexicon_tags_63 },
    { 0xa65c985e, SUC_lexicon_tags_56 },
    { 0x7dbc985e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9c9866, SUC_lexicon_tags_7 },
    { 0x2e1c9867, SUC_lexicon_tags_28 },
    { 0xe41c9868, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbf1c986a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997c9870, SUC_lexicon_tags_12 },
    { 0x2a1c9871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209c9877, SUC_lexicon_tags_25 },
    { 0xf65c9877, SUC_lexicon_tags_82 },
    { 0x187c9879, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39c987d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f9c987f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb43c9881, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc6dc9883, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9f9c9885, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54dc9887, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9c988c, SUC_lexicon_tags_15 },
    { 0xb43c988c, SUC_lexicon_tags_23 },
    { 0x99dc988e, SUC_lexicon_tags_15 },
    { 0x7e5c988f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bc9895, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31c9898, SUC_lexicon_tags_1 },
    { 0xf89c9899, SUC_lexicon_tags_2 },
    { 0xd15c989a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95c989f, SUC_lexicon_tags_17 },
    { 0x34fc989f, SUC_lexicon_tags_116 },
    { 0xb37c98a1, SUC_lexicon_tags_11 },
    { 0x859c98a2, SUC_lexicon_tags_2 },
    { 0xf41c98a3, SUC_lexicon_tags_60 },
    { 0xeb9c98a4, SUC_lexicon_tags_15 },
    { 0x851c98a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbc98ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15c98b1, SUC_lexicon_tags_23 },
    { 0xf3dc98b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x653c98b5, SUC_lexicon_tags_12 },
    { 0xcbfc98b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcc7c98b8, SUC_lexicon_tags_2 },
    { 0xe3bc98b8, SUC_lexicon_tags_32 },
    { 0x795c98b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1c98bf, SUC_lexicon_tags_25 },
    { 0xeabc98c0, SUC_lexicon_tags_37 },
    { 0x6bbc98c0, SUC_lexicon_tags_11 },
    { 0x5edc98bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcbbc98c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f5c98c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f5c98c8, SUC_lexicon_tags_25 },
    { 0xc01c98c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3c98cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fc98d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x571c98d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbc98d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7c98e0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925c98e5, SUC_lexicon_tags_11 },
    { 0x7e1c98e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe21c98e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bc98f3, SUC_lexicon_tags_29 },
    { 0xb99c98f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50fc98f6, SUC_lexicon_tags_40 },
    { 0x4c5c98f6, SUC_lexicon_tags_23 },
    { 0x7ddc98f8, SUC_lexicon_tags_39 },
    { 0x60dc98f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5c98fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fc9902, SUC_lexicon_tags_31 },
    { 0x11bc9903, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2a5c9905, SUC_lexicon_tags_7 },
    { 0xb7c9906, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x2c9c9908, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7c990c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x49dc990f, SUC_lexicon_tags_14 },
    { 0xe37c9910, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x247c9912, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdc9918, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bc991b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fc991f, SUC_lexicon_tags_92 },
    { 0xbf7c9920, SUC_lexicon_tags_61 },
    { 0x11fc9920, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587c9929, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7c992c, SUC_lexicon_tags_31 },
    { 0xeadc992d, SUC_lexicon_tags_11 },
    { 0x8c3c992c, SUC_lexicon_tags_17 },
    { 0x757c992f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdc9933, SUC_lexicon_tags_77 },
    { 0xbcbc9934, SUC_lexicon_tags_2 },
    { 0xbe7c9934, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffc993c, SUC_lexicon_tags_37 },
    { 0x82fc993d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dfc993f, SUC_lexicon_tags_1 },
    { 0x179c993f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x187c9942, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fc9947, SUC_lexicon_tags_66 },
    { 0x677c9948, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x11fc994a, SUC_lexicon_tags_7 },
    { 0xf89c994b, SUC_lexicon_tags_63 },
    { 0xe55c994c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xba7c994e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dc9953, SUC_lexicon_tags_35 },
    { 0x23c9954, SUC_lexicon_tags_7 },
    { 0xddc9955, SUC_lexicon_tags_49 },
    { 0xf97c9956, SUC_lexicon_tags_27 },
    { 0x7edc9954, SUC_lexicon_tags_45 },
    { 0xe3bc9956, SUC_lexicon_tags_25 },
    { 0xc0fc9953, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe45c995b, SUC_lexicon_tags_37 },
    { 0x379c995b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651c9967, SUC_lexicon_tags_2 },
    { 0xdc9c9967, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbc996b, SUC_lexicon_tags_20 },
    { 0x57dc996c, SUC_lexicon_tags_27 },
    { 0x501c996c, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9c9972, SUC_lexicon_tags_36 },
    { 0x1f3c9973, SUC_lexicon_tags_37 },
    { 0xe9bc9974, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bc997b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa97c997d, SUC_lexicon_tags_7 },
    { 0xd03c997e, SUC_lexicon_tags_14 },
    { 0x9d5c997e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x927c9981, SUC_lexicon_tags_17 },
    { 0xf7bc9982, SUC_lexicon_tags_48 },
    { 0x629c9981, SUC_lexicon_tags_15 },
    { 0x65dc9984, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7c9989, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bc9990, SUC_lexicon_tags_11 },
    { 0xabc9991, SUC_lexicon_tags_0 },
    { 0x481c9990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x707c9994, SUC_lexicon_tags_15 },
    { 0x6c9c9995, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45c999d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7c99a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x611c99a2, SUC_lexicon_tags_47 },
    { 0x39fc99a3, SUC_lexicon_tags_15 },
    { 0x1a9c99a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x97c99a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa0bc99a9, SUC_lexicon_tags_27 },
    { 0xe09c99a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33c99ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abc99b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x643c99b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55c99bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73c99be, SUC_lexicon_tags_46 },
    { 0x9a3c99be, SUC_lexicon_tags_37 },
    { 0x533c99c0, SUC_lexicon_tags_16 },
    { 0x43bc99bf, SUC_lexicon_tags_11 },
    { 0x883c99c2, SUC_lexicon_tags_12 },
    { 0xe4bc99c3, SUC_lexicon_tags_1 },
    { 0x203c99c4, SUC_lexicon_tags_14 },
    { 0xc85c99c3, SUC_lexicon_tags_2 },
    { 0x6adc99c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9c99c9, SUC_lexicon_tags_11 },
    { 0x381c99ca, SUC_lexicon_tags_12 },
    { 0x513c99cb, SUC_lexicon_tags_31 },
    { 0x343c99cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441c99d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd2fc99d2, SUC_lexicon_tags_49 },
    { 0x9dbc99d3, SUC_lexicon_tags_16 },
    { 0xd8bc99d2, SUC_lexicon_tags_6 },
    { 0x6b1c99d5, SUC_lexicon_tags_49 },
    { 0x821c99d3, SUC_lexicon_tags_79 },
    { 0xc79c99d7, SUC_lexicon_tags_16 },
    { 0xfefc99d7, SUC_lexicon_tags_14 },
    { 0x563c99d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fc99e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce7c99e4, SUC_lexicon_tags_36 },
    { 0x4b5c99e4, SUC_lexicon_tags_45 },
    { 0x1afc99e6, SUC_lexicon_tags_30 },
    { 0x69fc99e7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9c99ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x915c99ec, SUC_lexicon_tags_17 },
    { 0x8d1c99ed, SUC_lexicon_tags_103 },
    { 0x8e5c99ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddc99f1, SUC_lexicon_tags_14 },
    { 0xe39c99f1, SUC_lexicon_tags_19 },
    { 0x7d1c99f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405c99f7, SUC_lexicon_tags_37 },
    { 0xefdc99f8, SUC_lexicon_tags_49 },
    { 0x16bc99f8, SUC_lexicon_tags_27 },
    { 0x6e9c99f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfd7c99fc, SUC_lexicon_tags_47 },
    { 0xcd5c99fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabc9a02, SUC_lexicon_tags_25 },
    { 0xc1bc9a03, SUC_lexicon_tags_351 },
    { 0x4a7c9a03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421c9a0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9e7c9a0c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541c9a10, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37c9a16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x215c9a18, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x409c9a1b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefc9a1f, SUC_lexicon_tags_15 },
    { 0xe47c9a20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7bc9a22, SUC_lexicon_tags_8 },
    { 0x51bc9a23, SUC_lexicon_tags_29 },
    { 0xf45c9a24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83c9a28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xeb5c9a2a, SUC_lexicon_tags_33 },
    { 0xf75c9a2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7c9a33, SUC_lexicon_tags_10 },
    { 0x1fbc9a34, SUC_lexicon_tags_47 },
    { 0x837c9a35, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb29c9a37, SUC_lexicon_tags_4 },
    { 0x541c9a38, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedc9a3c, SUC_lexicon_tags_25 },
    { 0x6cfc9a3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3c9a40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09c9a49, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3c9a4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaa9c9a4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c9a55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7c9a58, SUC_lexicon_tags_82 },
    { 0xb83c9a59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3c9a5c, SUC_lexicon_tags_8 },
    { 0x163c9a5d, SUC_lexicon_tags_12 },
    { 0x5a5c9a5e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfc9a61, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5d5c9a63, SUC_lexicon_tags_37 },
    { 0x169c9a64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1c9a6b, SUC_lexicon_tags_14 },
    { 0xed5c9a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fc9a71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dc9a73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dc9a76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x421c9a78, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe19c9a7a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9c9a7d, SUC_lexicon_tags_13 },
    { 0xe3fc9a7e, SUC_lexicon_tags_11 },
    { 0x56bc9a7d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17c9a83, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173c9a87, SUC_lexicon_tags_43 },
    { 0xecfc9a88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfc9a8b, SUC_lexicon_tags_21 },
    { 0x6dfc9a8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf77c9a8e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57c9a98, SUC_lexicon_tags_8 },
    { 0x5e9c9a99, SUC_lexicon_tags_36 },
    { 0x5d1c9a9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dfc9a9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfd3c9a9e, SUC_lexicon_tags_15 },
    { 0x2fbc9a9f, SUC_lexicon_tags_85 },
    { 0x67dc9aa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fdc9aa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x929c9aa4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb17c9aa6, SUC_lexicon_tags_2 },
    { 0x347c9aa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7c9aad, SUC_lexicon_tags_43 },
    { 0x5a1c9aae, SUC_lexicon_tags_48 },
    { 0x9b9c9aae, SUC_lexicon_tags_17 },
    { 0xea1c9ab0, SUC_lexicon_tags_1636 },
    { 0x627c9ab0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9c9ab5, SUC_lexicon_tags_8 },
    { 0x825c9ab6, SUC_lexicon_tags_47 },
    { 0xf99c9ab7, SUC_lexicon_tags_7 },
    { 0xdf1c9ab8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571c9abd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5c9ac0, SUC_lexicon_tags_25 },
    { 0x13bc9ac1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5b1c9ac3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5c9ac7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7c9aca, SUC_lexicon_tags_87 },
    { 0xcd7c9acb, SUC_lexicon_tags_11 },
    { 0xd91c9acb, SUC_lexicon_tags_11 },
    { 0x597c9acd, SUC_lexicon_tags_30 },
    { 0xf83c9ace, SUC_lexicon_tags_47 },
    { 0xa93c9acf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d1c9ad1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd63c9ad3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773c9ad7, SUC_lexicon_tags_13 },
    { 0x17dc9ad8, SUC_lexicon_tags_1 },
    { 0x1bdc9ad8, SUC_lexicon_tags_17 },
    { 0x1fdc9ada, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73c9ae1, SUC_lexicon_tags_25 },
    { 0xb29c9ae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53c9ae7, SUC_lexicon_tags_37 },
    { 0x2b7c9ae8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349c9aed, SUC_lexicon_tags_15 },
    { 0xf0fc9aee, SUC_lexicon_tags_7 },
    { 0x2a7c9aef, SUC_lexicon_tags_2 },
    { 0x63dc9aef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37c9af4, SUC_lexicon_tags_23 },
    { 0xb89c9af5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1c9afa, SUC_lexicon_tags_37 },
    { 0x515c9afa, SUC_lexicon_tags_12 },
    { 0x639c9afc, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x7a3c9afe, SUC_lexicon_tags_17 },
    { 0x565c9aff, SUC_lexicon_tags_11 },
    { 0xfafc9b00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efc9b0b, SUC_lexicon_tags_11 },
    { 0x55dc9b0c, SUC_lexicon_tags_14 },
    { 0xc61c9b0d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffc9b12, SUC_lexicon_tags_1 },
    { 0xbadc9b13, SUC_lexicon_tags_11 },
    { 0x2fc9b14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7c9b17, SUC_lexicon_tags_12 },
    { 0x243c9b18, SUC_lexicon_tags_7 },
    { 0xf55c9b19, SUC_lexicon_tags_17 },
    { 0xdf9c9b1a, SUC_lexicon_tags_2 },
    { 0x1afc9b1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x755c9b1d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x367c9b1f, SUC_lexicon_tags_31 },
    { 0x423c9b20, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b1c9b22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545c9b26, SUC_lexicon_tags_1 },
    { 0xb13c9b26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619c9b2b, SUC_lexicon_tags_37 },
    { 0x2b3c9b2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf45c9b2e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dc9b38, SUC_lexicon_tags_84 },
    { 0xbcfc9b39, SUC_lexicon_tags_15 },
    { 0x9ddc9b3a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf53c9b3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfc9b3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3c9b45, SUC_lexicon_tags_15 },
    { 0xe15c9b46, SUC_lexicon_tags_1 },
    { 0xbb1c9b47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ebc9b49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afc9b51, SUC_lexicon_tags_57 },
    { 0xb7bc9b51, SUC_lexicon_tags_14 },
    { 0x3a7c9b53, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5c9b56, SUC_lexicon_tags_12 },
    { 0x87c9b57, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edc9b5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861c9b62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d9c9b64, SUC_lexicon_tags_49 },
    { 0x83c9b64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba9c9b67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc9b6a, SUC_lexicon_tags_17 },
    { 0xddc9b6b, SUC_lexicon_tags_11 },
    { 0x115c9b6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x377c9b6f, SUC_lexicon_tags_32 },
    { 0xe0bc9b70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551c9b74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1c9b77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d9c9b79, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcb1c9b7b, SUC_lexicon_tags_66 },
    { 0x797c9b7b, SUC_lexicon_tags_6 },
    { 0x3a5c9b7d, SUC_lexicon_tags_1 },
    { 0xa97c9b7c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3c9b89, SUC_lexicon_tags_28 },
    { 0x6d9c9b89, SUC_lexicon_tags_25 },
    { 0xd07c9b89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7c9b8e, SUC_lexicon_tags_66 },
    { 0x2b3c9b8f, SUC_lexicon_tags_7 },
    { 0xcdc9b8e, SUC_lexicon_tags_25 },
    { 0xa5dc9b91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edc9b96, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5c9b9a, SUC_lexicon_tags_26 },
    { 0xc83c9b9b, SUC_lexicon_tags_2 },
    { 0xc27c9b9c, SUC_lexicon_tags_17 },
    { 0x29c9b9a, SUC_lexicon_tags_29 },
    { 0x333c9b9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109c9ba4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdd7c9ba6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9c9ba9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1c9bae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9c9bb1, SUC_lexicon_tags_1 },
    { 0x2ebc9bb1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87c9bb8, SUC_lexicon_tags_15 },
    { 0xec7c9bb9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fc9bc2, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbc9bc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9c9bc8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9c9bcc, SUC_lexicon_tags_1 },
    { 0xb09c9bcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x681c9bcf, SUC_lexicon_tags_1 },
    { 0x971c9bd0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5bc9bd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bc9bd8, SUC_lexicon_tags_43 },
    { 0xf1fc9bd9, SUC_lexicon_tags_47 },
    { 0x8dbc9bda, SUC_lexicon_tags_17 },
    { 0x90fc9bdb, SUC_lexicon_tags_37 },
    { 0xa69c9bdc, SUC_lexicon_tags_33 },
    { 0xf1dc9bdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x987c9bdf, SUC_lexicon_tags_12 },
    { 0xdc1c9be0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe03c9be2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fc9be6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9c9be9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdedc9bef, SUC_lexicon_tags_199 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883c9bf4, SUC_lexicon_tags_5 },
    { 0x11dc9bf4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa19c9bf7, SUC_lexicon_tags_11 },
    { 0x13bc9bf7, SUC_lexicon_tags_7 },
    { 0xc95c9bf7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x49c9bfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x39c9bfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dc9c06, SUC_lexicon_tags_10 },
    { 0xe13c9c07, SUC_lexicon_tags_12 },
    { 0xc05c9c08, SUC_lexicon_tags_37 },
    { 0xf27c9c09, SUC_lexicon_tags_61 },
    { 0x173c9c0a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xf33c9c0c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8afc9c0e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd3bc9c10, SUC_lexicon_tags_11 },
    { 0xe29c9c11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x79c9c14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fc9c1a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9c1f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29c9c23, SUC_lexicon_tags_15 },
    { 0x5bdc9c24, SUC_lexicon_tags_6 },
    { 0xd71c9c24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265c9c2a, SUC_lexicon_tags_3 },
    { 0x14bc9c2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bc9c2e, SUC_lexicon_tags_33 },
    { 0xdfc9c2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x693c9c32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x619c9c35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27c9c38, SUC_lexicon_tags_2 },
    { 0xe6bc9c39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x817c9c3c, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169c9c43, SUC_lexicon_tags_15 },
    { 0x9bdc9c44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23bc9c46, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83c9c4a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffc9c4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09c9c51, SUC_lexicon_tags_17 },
    { 0x16bc9c52, SUC_lexicon_tags_0 },
    { 0xa0fc9c52, SUC_lexicon_tags_25 },
    { 0x5e3c9c53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dc9c59, SUC_lexicon_tags_27 },
    { 0x391c9c5a, SUC_lexicon_tags_13 },
    { 0x203c9c5a, SUC_lexicon_tags_27 },
    { 0x231c9c59, SUC_lexicon_tags_1 },
    { 0x803c9c5d, SUC_lexicon_tags_1673 },
    { 0x917c9c5e, SUC_lexicon_tags_2 },
    { 0xa51c9c5f, SUC_lexicon_tags_11 },
    { 0xf27c9c5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7f3c9c62, SUC_lexicon_tags_25 },
    { 0x30dc9c63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x63dc9c66, SUC_lexicon_tags_12 },
    { 0x407c9c67, SUC_lexicon_tags_2 },
    { 0x767c9c68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593c9c6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7ddc9c70, SUC_lexicon_tags_12 },
    { 0xc07c9c71, SUC_lexicon_tags_12 },
    { 0xdb9c9c72, SUC_lexicon_tags_10 },
    { 0x109c9c71, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715c9c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dc9c7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bc9c83, SUC_lexicon_tags_16 },
    { 0x71bc9c83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50fc9c86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdc9c8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1c9c95, SUC_lexicon_tags_2 },
    { 0x39dc9c96, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd5bc9c98, SUC_lexicon_tags_12 },
    { 0x23dc9c99, SUC_lexicon_tags_5 },
    { 0x4a9c9c9a, SUC_lexicon_tags_30 },
    { 0x45dc9c9a, SUC_lexicon_tags_25 },
    { 0xadc9c9a, SUC_lexicon_tags_7 },
    { 0x683c9c9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf69c9c9f, SUC_lexicon_tags_118 },
    { 0x45c9ca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x587c9ca3, SUC_lexicon_tags_17 },
    { 0xac1c9ca4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfc9ca8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7c9cab, SUC_lexicon_tags_2 },
    { 0x6afc9cac, SUC_lexicon_tags_11 },
    { 0xb25c9cad, SUC_lexicon_tags_0 },
    { 0xd8bc9cab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x253c9cb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd71c9cb2, SUC_lexicon_tags_29 },
    { 0xdcbc9cb3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x821c9cb5, SUC_lexicon_tags_1 },
    { 0x323c9cb6, SUC_lexicon_tags_43 },
    { 0xbe3c9cb7, SUC_lexicon_tags_15 },
    { 0x3d7c9cb8, SUC_lexicon_tags_25 },
    { 0x4c7c9cb9, SUC_lexicon_tags_27 },
    { 0xb4dc9cba, SUC_lexicon_tags_37 },
    { 0xfb7c9cba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95bc9cbd, SUC_lexicon_tags_33 },
    { 0xfb1c9cbe, SUC_lexicon_tags_2 },
    { 0x789c9cbe, SUC_lexicon_tags_47 },
    { 0x5ffc9cbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfbc9cc2, SUC_lexicon_tags_1 },
    { 0x17bc9cc2, SUC_lexicon_tags_21 },
    { 0x673c9cc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3c9cc7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893c9ccc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x309c9cce, SUC_lexicon_tags_13 },
    { 0xddc9ccf, SUC_lexicon_tags_25 },
    { 0xa9fc9cd0, SUC_lexicon_tags_9 },
    { 0xa7bc9cd0, SUC_lexicon_tags_17 },
    { 0x76fc9cd2, SUC_lexicon_tags_28 },
    { 0xc59c9cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5c9cdd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57c9ce2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x819c9ce5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddc9ce8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7c9cec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd5c9cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca3c9cf0, SUC_lexicon_tags_60 },
    { 0x82fc9cf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1c9cf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa17c9cf7, SUC_lexicon_tags_17 },
    { 0x6f3c9cf7, SUC_lexicon_tags_25 },
    { 0xb5bc9cf9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635c9cfd, SUC_lexicon_tags_37 },
    { 0x96dc9cfe, SUC_lexicon_tags_47 },
    { 0x14dc9cff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1c9d05, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x417c9d07, SUC_lexicon_tags_7 },
    { 0x475c9d08, SUC_lexicon_tags_29 },
    { 0x725c9d09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9c9d0e, SUC_lexicon_tags_28 },
    { 0xc75c9d0e, SUC_lexicon_tags_7 },
    { 0x6bc9d10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951c9d14, SUC_lexicon_tags_61 },
    { 0xbb1c9d15, SUC_lexicon_tags_2 },
    { 0x88bc9d16, SUC_lexicon_tags_21 },
    { 0xb1c9d16, SUC_lexicon_tags_40 },
    { 0x211c9d15, SUC_lexicon_tags_38 },
    { 0xba9c9d19, SUC_lexicon_tags_13 },
    { 0x7edc9d14, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa7bc9d1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdc9d20, SUC_lexicon_tags_17 },
    { 0x407c9d21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1c9d2c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xa7dc9d2e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xb33c9d30, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x977c9d32, SUC_lexicon_tags_38 },
    { 0x1a5c9d33, SUC_lexicon_tags_25 },
    { 0x59c9d34, SUC_lexicon_tags_17 },
    { 0xf5fc9d35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbc9d3a, SUC_lexicon_tags_9 },
    { 0xe85c9d3b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9c9d42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6fbc9d44, SUC_lexicon_tags_46 },
    { 0xd73c9d45, SUC_lexicon_tags_106 },
    { 0xd07c9d46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fc9d49, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8e7c9d4b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x999c9d4d, SUC_lexicon_tags_29 },
    { 0x9c9d4d, SUC_lexicon_tags_12 },
    { 0x2d9c9d4d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831c9d53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9d56, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3c9d59, SUC_lexicon_tags_1 },
    { 0xfddc9d59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05c9d63, SUC_lexicon_tags_68 },
    { 0x7d9c9d63, SUC_lexicon_tags_5 },
    { 0x589c9d65, SUC_lexicon_tags_37 },
    { 0x565c9d65, SUC_lexicon_tags_25 },
    { 0x97bc9d67, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877c9d6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfc9d72, SUC_lexicon_tags_840 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1c9d7a, SUC_lexicon_tags_26 },
    { 0x87bc9d7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fc9d7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26dc9d81, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x521c9d83, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x217c9d86, SUC_lexicon_tags_7 },
    { 0xf69c9d87, SUC_lexicon_tags_25 },
    { 0xf99c9d87, SUC_lexicon_tags_30 },
    { 0xff5c9d87, SUC_lexicon_tags_82 },
    { 0x9d3c9d86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x40dc9d8d, SUC_lexicon_tags_37 },
    { 0x2e7c9d8e, SUC_lexicon_tags_15 },
    { 0x2a3c9d8e, SUC_lexicon_tags_17 },
    { 0xcd7c9d90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x885c9d92, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bc9d97, SUC_lexicon_tags_11 },
    { 0xd59c9d97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dc9d9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bc9da0, SUC_lexicon_tags_30 },
    { 0x4bbc9da1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbc9da6, SUC_lexicon_tags_17 },
    { 0xeb3c9da7, SUC_lexicon_tags_56 },
    { 0xbf5c9da8, SUC_lexicon_tags_100 },
    { 0xb39c9da8, SUC_lexicon_tags_2 },
    { 0x665c9da8, SUC_lexicon_tags_11 },
    { 0x1b5c9da7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77c9dae, SUC_lexicon_tags_2 },
    { 0x7f3c9daf, SUC_lexicon_tags_12 },
    { 0x5a5c9dae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fc9db5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17bc9db7, SUC_lexicon_tags_15 },
    { 0x2f5c9db7, SUC_lexicon_tags_12 },
    { 0x537c9db9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25c9dc6, SUC_lexicon_tags_13 },
    { 0xe89c9dc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53c9dca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dc9dcd, SUC_lexicon_tags_26 },
    { 0xea5c9dce, SUC_lexicon_tags_33 },
    { 0xd83c9dcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3c9dd1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x375c9dd4, SUC_lexicon_tags_19 },
    { 0xe9fc9dd4, SUC_lexicon_tags_38 },
    { 0x227c9dd6, SUC_lexicon_tags_17 },
    { 0xd97c9dd4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dc9ddc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5e1c9dde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x943c9de0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2fdc9de2, SUC_lexicon_tags_15 },
    { 0x723c9de3, SUC_lexicon_tags_38 },
    { 0x2f5c9de4, SUC_lexicon_tags_63 },
    { 0x333c9de3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1c9de9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5c9dec, SUC_lexicon_tags_82 },
    { 0x5e7c9ded, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbdc9def, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x801c9df2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafc9df5, SUC_lexicon_tags_23 },
    { 0x56fc9df6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbc3c9df8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x62fc9dfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b9c9dfc, SUC_lexicon_tags_14 },
    { 0x137c9dfc, SUC_lexicon_tags_15 },
    { 0x705c9dfe, SUC_lexicon_tags_12 },
    { 0x70bc9dfe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fc9e03, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x59bc9e05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37c9e07, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bc9e0b, SUC_lexicon_tags_25 },
    { 0x29fc9e0c, SUC_lexicon_tags_37 },
    { 0xc43c9e0d, SUC_lexicon_tags_17 },
    { 0xffdc9e0e, SUC_lexicon_tags_11 },
    { 0x791c9e0e, SUC_lexicon_tags_2 },
    { 0xc03c9e10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe87c9e12, SUC_lexicon_tags_17 },
    { 0x10fc9e13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xdbc9e15, SUC_lexicon_tags_43 },
    { 0xc65c9e16, SUC_lexicon_tags_66 },
    { 0x531c9e16, SUC_lexicon_tags_244 },
    { 0x315c9e18, SUC_lexicon_tags_49 },
    { 0x999c9e17, SUC_lexicon_tags_2 },
    { 0x567c9e1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x775c9e1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x155c9e20, SUC_lexicon_tags_55 },
    { 0x803c9e21, SUC_lexicon_tags_19 },
    { 0x3b5c9e21, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fc9e28, SUC_lexicon_tags_60 },
    { 0xbefc9e28, SUC_lexicon_tags_70 },
    { 0x853c9e2a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53c9e30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1c9e35, SUC_lexicon_tags_1 },
    { 0x54bc9e36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03c9e3a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997c9e3e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x94bc9e40, SUC_lexicon_tags_12 },
    { 0x7e7c9e41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff1c9e43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665c9e48, SUC_lexicon_tags_4 },
    { 0x1f1c9e49, SUC_lexicon_tags_68 },
    { 0xf0fc9e4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9c9c9e4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37fc9e4e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151c9e52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x229c9e55, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bc9e59, SUC_lexicon_tags_40 },
    { 0xc0bc9e5a, SUC_lexicon_tags_38 },
    { 0xd67c9e5b, SUC_lexicon_tags_56 },
    { 0x3dc9e5b, SUC_lexicon_tags_34 },
    { 0xa61c9e5a, SUC_lexicon_tags_1 },
    { 0x99bc9e5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73c9e61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5c9e64, SUC_lexicon_tags_7 },
    { 0x79c9e64, SUC_lexicon_tags_17 },
    { 0xcfbc9e66, SUC_lexicon_tags_37 },
    { 0xe99c9e66, SUC_lexicon_tags_15 },
    { 0xba5c9e65, SUC_lexicon_tags_60 },
    { 0xacdc9e69, SUC_lexicon_tags_23 },
    { 0xc0bc9e6a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x321c9e6d, SUC_lexicon_tags_57 },
    { 0x29bc9e6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fc9e71, SUC_lexicon_tags_25 },
    { 0x543c9e72, SUC_lexicon_tags_37 },
    { 0xee1c9e72, SUC_lexicon_tags_59 },
    { 0xe7bc9e73, SUC_lexicon_tags_26 },
    { 0x25fc9e75, SUC_lexicon_tags_13 },
    { 0xb05c9e75, SUC_lexicon_tags_13 },
    { 0x60dc9e77, SUC_lexicon_tags_118 },
    { 0x3bc9e78, SUC_lexicon_tags_37 },
    { 0x5c5c9e75, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe7bc9e7b, SUC_lexicon_tags_11 },
    { 0xb87c9e7c, SUC_lexicon_tags_25 },
    { 0x1edc9e7b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x975c9e80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dc9e84, SUC_lexicon_tags_9 },
    { 0x8c5c9e85, SUC_lexicon_tags_29 },
    { 0xf27c9e85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfc9e89, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7e3c9e8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bc9e8d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1c9e90, SUC_lexicon_tags_7 },
    { 0xd53c9e91, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7b5c9e93, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5c9e99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3c9e9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9c9ea0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bc9ea5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27c9ea8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x143c9eaa, SUC_lexicon_tags_11 },
    { 0x1c1c9eab, SUC_lexicon_tags_23 },
    { 0xc5fc9eab, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21c9eb3, SUC_lexicon_tags_27 },
    { 0x747c9eb4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9f5c9eb6, SUC_lexicon_tags_19 },
    { 0x879c9eb6, SUC_lexicon_tags_37 },
    { 0x6d1c9eb8, SUC_lexicon_tags_15 },
    { 0xb0fc9eb8, SUC_lexicon_tags_25 },
    { 0xb29c9eba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfdfc9ebc, SUC_lexicon_tags_9 },
    { 0xc8dc9ebd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x247c9ebf, SUC_lexicon_tags_15 },
    { 0xe3fc9ec0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x833c9ec3, SUC_lexicon_tags_15 },
    { 0x669c9ec4, SUC_lexicon_tags_4 },
    { 0xaf9c9ec3, SUC_lexicon_tags_37 },
    { 0x2a3c9ec3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65c9ec9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59fc9ecb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79c9ecf, SUC_lexicon_tags_55 },
    { 0xe6fc9ed0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc5fc9ed2, SUC_lexicon_tags_33 },
    { 0xd39c9ed2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1c9ed6, SUC_lexicon_tags_61 },
    { 0x983c9ed6, SUC_lexicon_tags_25 },
    { 0x47dc9ed8, SUC_lexicon_tags_23 },
    { 0x66fc9ed9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61c9ede, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615c9ee7, SUC_lexicon_tags_8 },
    { 0xcfc9ee8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3c9eec, SUC_lexicon_tags_14 },
    { 0x3dbc9eec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dc9efa, SUC_lexicon_tags_63 },
    { 0x315c9efb, SUC_lexicon_tags_25 },
    { 0x22bc9efc, SUC_lexicon_tags_2 },
    { 0x273c9efb, SUC_lexicon_tags_23 },
    { 0xa5c9efb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abc9f01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd69c9f03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3c9f08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x38dc9f0a, SUC_lexicon_tags_42 },
    { 0x165c9f0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf0fc9f0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe05c9f0f, SUC_lexicon_tags_32 },
    { 0x1e5c9f10, SUC_lexicon_tags_11 },
    { 0xc9fc9f11, SUC_lexicon_tags_2 },
    { 0x629c9f12, SUC_lexicon_tags_12 },
    { 0x67dc9f0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x585c9f16, SUC_lexicon_tags_29 },
    { 0x591c9f17, SUC_lexicon_tags_65 },
    { 0xea5c9f18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4bc9f1a, SUC_lexicon_tags_14 },
    { 0xf4fc9f1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3c9f25, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xba3c9f27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x673c9f29, SUC_lexicon_tags_8 },
    { 0x6a3c9f2a, SUC_lexicon_tags_63 },
    { 0x36bc9f2b, SUC_lexicon_tags_8 },
    { 0x7a5c9f2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7c9f2f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bc9f33, SUC_lexicon_tags_26 },
    { 0x399c9f33, SUC_lexicon_tags_17 },
    { 0x69fc9f34, SUC_lexicon_tags_47 },
    { 0xa2fc9f36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x343c9f39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59c9f3e, SUC_lexicon_tags_7 },
    { 0x421c9f3f, SUC_lexicon_tags_158 },
    { 0xe03c9f40, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdb1c9f42, SUC_lexicon_tags_63 },
    { 0x541c9f43, SUC_lexicon_tags_57 },
    { 0xe13c9f44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1c9f4a, SUC_lexicon_tags_0 },
    { 0x8bfc9f4b, SUC_lexicon_tags_4 },
    { 0x581c9f4c, SUC_lexicon_tags_2 },
    { 0x299c9f4a, SUC_lexicon_tags_75 },
    { 0x35bc9f4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dc9f53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x309c9f56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fc9f5c, SUC_lexicon_tags_87 },
    { 0x83c9f5d, SUC_lexicon_tags_25 },
    { 0xffdc9f5d, SUC_lexicon_tags_8 },
    { 0x3efc9f5f, SUC_lexicon_tags_184 },
    { 0xaf1c9f60, SUC_lexicon_tags_27 },
    { 0x415c9f61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91c9f67, SUC_lexicon_tags_2 },
    { 0x2c3c9f67, SUC_lexicon_tags_32 },
    { 0xa7fc9f69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73dc9f6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bc9f6f, SUC_lexicon_tags_34 },
    { 0x4dfc9f6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xff3c9f72, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3c9f7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fc9f84, SUC_lexicon_tags_16 },
    { 0x4dc9f85, SUC_lexicon_tags_6 },
    { 0x85bc9f86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x339c9f88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e1c9f8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x70fc9f8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc3c9f8e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x78fc9f90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839c9f95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cfc9f97, SUC_lexicon_tags_2 },
    { 0x1b9c9f98, SUC_lexicon_tags_11 },
    { 0x4e3c9f99, SUC_lexicon_tags_37 },
    { 0x85bc9f98, SUC_lexicon_tags_9 },
    { 0x673c9f9a, SUC_lexicon_tags_43 },
    { 0xaa9c9f9c, SUC_lexicon_tags_15 },
    { 0x39bc9f9d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbcfc9f9f, SUC_lexicon_tags_26 },
    { 0xfb9c9f9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7c9fa5, SUC_lexicon_tags_14 },
    { 0x57bc9fa5, SUC_lexicon_tags_46 },
    { 0x9bdc9fa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x677c9faa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcdbc9fac, SUC_lexicon_tags_33 },
    { 0x435c9fad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4abc9fb0, SUC_lexicon_tags_7 },
    { 0xec7c9fb1, SUC_lexicon_tags_26 },
    { 0xc2dc9fb2, SUC_lexicon_tags_7 },
    { 0x939c9fb0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd95c9fb5, SUC_lexicon_tags_84 },
    { 0x1c5c9fb6, SUC_lexicon_tags_23 },
    { 0x38bc9fb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf11c9fb9, SUC_lexicon_tags_8 },
    { 0x9bbc9fba, SUC_lexicon_tags_60 },
    { 0x473c9fbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33c9fc0, SUC_lexicon_tags_25 },
    { 0x3ddc9fc1, SUC_lexicon_tags_31 },
    { 0x88dc9fc2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7c9fc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0dc9fc7, SUC_lexicon_tags_0 },
    { 0x5f9c9fc8, SUC_lexicon_tags_6 },
    { 0x2fdc9fc9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc43c9fcb, SUC_lexicon_tags_12 },
    { 0x595c9fcc, SUC_lexicon_tags_17 },
    { 0xb97c9fcd, SUC_lexicon_tags_34 },
    { 0x83dc9fce, SUC_lexicon_tags_14 },
    { 0xb4fc9fcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x469c9fd1, SUC_lexicon_tags_12 },
    { 0x6efc9fd1, SUC_lexicon_tags_11 },
    { 0x4c1c9fd1, SUC_lexicon_tags_33 },
    { 0x4bdc9fd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bc9fd9, SUC_lexicon_tags_37 },
    { 0x225c9fda, SUC_lexicon_tags_13 },
    { 0xcf1c9fdb, SUC_lexicon_tags_37 },
    { 0x2f3c9fd9, SUC_lexicon_tags_7 },
    { 0x4f9c9fda, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fc9fe0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93c9fe3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3c9c9fe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfc9fe8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bc9fed, SUC_lexicon_tags_11 },
    { 0x621c9fed, SUC_lexicon_tags_16 },
    { 0xe69c9fee, SUC_lexicon_tags_37 },
    { 0x875c9fee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5c9ff9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaefc9ffc, SUC_lexicon_tags_1 },
    { 0x6fdc9ffc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x295c9fff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5ca002, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9ca007, SUC_lexicon_tags_14 },
    { 0xedbca007, SUC_lexicon_tags_25 },
    { 0xb2dca009, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x449ca00c, SUC_lexicon_tags_25 },
    { 0x7e3ca00d, SUC_lexicon_tags_18 },
    { 0x505ca00e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bca011, SUC_lexicon_tags_12 },
    { 0xa1ca012, SUC_lexicon_tags_37 },
    { 0xa89ca013, SUC_lexicon_tags_1 },
    { 0xbf9ca014, SUC_lexicon_tags_15 },
    { 0x9edca014, SUC_lexicon_tags_77 },
    { 0xb1ca013, SUC_lexicon_tags_23 },
    { 0x4c7ca017, SUC_lexicon_tags_60 },
    { 0xa93ca011, SUC_lexicon_tags_19 },
    { 0x62bca019, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x115ca01b, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x49fca01d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475ca021, SUC_lexicon_tags_7 },
    { 0xbbfca022, SUC_lexicon_tags_402 },
    { 0x8cbca022, SUC_lexicon_tags_17 },
    { 0xf51ca021, SUC_lexicon_tags_0 },
    { 0x99ca025, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x119ca028, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1ca02c, SUC_lexicon_tags_226 },
    { 0xd3dca02d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x345ca02f, SUC_lexicon_tags_38 },
    { 0x1ddca030, SUC_lexicon_tags_30 },
    { 0xaf5ca031, SUC_lexicon_tags_6 },
    { 0x783ca032, SUC_lexicon_tags_47 },
    { 0x8a7ca02f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8dbca035, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5ca038, SUC_lexicon_tags_15 },
    { 0xe2dca039, SUC_lexicon_tags_63 },
    { 0x681ca03a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bca03d, SUC_lexicon_tags_12 },
    { 0xeefca03e, SUC_lexicon_tags_37 },
    { 0x971ca03f, SUC_lexicon_tags_82 },
    { 0x561ca040, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fca044, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x79ca046, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fca04c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc3ca04e, SUC_lexicon_tags_1 },
    { 0x929ca04f, SUC_lexicon_tags_2 },
    { 0x429ca04e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bca055, SUC_lexicon_tags_17 },
    { 0xcedca055, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4bfca058, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5ca05b, SUC_lexicon_tags_26 },
    { 0xcc1ca05c, SUC_lexicon_tags_4 },
    { 0x13dca05d, SUC_lexicon_tags_25 },
    { 0x4c3ca05e, SUC_lexicon_tags_2 },
    { 0x49bca05d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x789ca062, SUC_lexicon_tags_8 },
    { 0x459ca062, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bca066, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbca06c, SUC_lexicon_tags_77 },
    { 0x56bca06c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edca070, SUC_lexicon_tags_23 },
    { 0xb7ca070, SUC_lexicon_tags_12 },
    { 0x7f3ca071, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa77ca074, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531ca07a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x549ca07d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41ca080, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e3ca082, SUC_lexicon_tags_29 },
    { 0x219ca083, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x385ca085, SUC_lexicon_tags_17 },
    { 0xb77ca086, SUC_lexicon_tags_19 },
    { 0x44dca086, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x273ca089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5ca08d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc95ca08f, SUC_lexicon_tags_63 },
    { 0xb5ca090, SUC_lexicon_tags_38 },
    { 0x117ca090, SUC_lexicon_tags_25 },
    { 0xf13ca091, SUC_lexicon_tags_38 },
    { 0x601ca093, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fca098, SUC_lexicon_tags_282 },
    { 0x773ca098, SUC_lexicon_tags_17 },
    { 0x98fca098, SUC_lexicon_tags_12 },
    { 0x363ca09b, SUC_lexicon_tags_28 },
    { 0x933ca09c, SUC_lexicon_tags_2 },
    { 0x781ca09d, SUC_lexicon_tags_37 },
    { 0x643ca09c, SUC_lexicon_tags_8 },
    { 0x62bca09f, SUC_lexicon_tags_14 },
    { 0xa4bca0a0, SUC_lexicon_tags_56 },
    { 0x499ca09e, SUC_lexicon_tags_45 },
    { 0xc31ca0a0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219ca0a9, SUC_lexicon_tags_23 },
    { 0x981ca0aa, SUC_lexicon_tags_49 },
    { 0x38bca0ab, SUC_lexicon_tags_12 },
    { 0x54dca0ab, SUC_lexicon_tags_28 },
    { 0x1ebca0ad, SUC_lexicon_tags_15 },
    { 0xcbca0a9, SUC_lexicon_tags_56 },
    { 0x743ca0a9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891ca0b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bca0b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95ca0ba, SUC_lexicon_tags_23 },
    { 0x9cfca0bb, SUC_lexicon_tags_11 },
    { 0xa5fca0bc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263ca0c0, SUC_lexicon_tags_17 },
    { 0x5b1ca0c0, SUC_lexicon_tags_31 },
    { 0x2edca0c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83ca0c6, SUC_lexicon_tags_15 },
    { 0x4a9ca0c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfbfca0c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dca0cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a5ca0ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd25ca0d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29ca0d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15ca0dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7ddca0de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x749ca0e0, SUC_lexicon_tags_37 },
    { 0xb55ca0e1, SUC_lexicon_tags_55 },
    { 0xfa9ca0e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x727ca0e5, SUC_lexicon_tags_47 },
    { 0x381ca0e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903ca0ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfca0ed, SUC_lexicon_tags_45 },
    { 0x621ca0ee, SUC_lexicon_tags_77 },
    { 0xe7bca0ef, SUC_lexicon_tags_63 },
    { 0x3a5ca0ef, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x23dca0f2, SUC_lexicon_tags_136 },
    { 0xaafca0f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x323ca0f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549ca0fa, SUC_lexicon_tags_30 },
    { 0x7adca0fb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x197ca0fe, SUC_lexicon_tags_47 },
    { 0xabbca0ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fca105, SUC_lexicon_tags_29 },
    { 0xb0bca105, SUC_lexicon_tags_8 },
    { 0x73fca106, SUC_lexicon_tags_32 },
    { 0x51dca107, SUC_lexicon_tags_13 },
    { 0x63dca108, SUC_lexicon_tags_30 },
    { 0xe45ca107, SUC_lexicon_tags_522 },
    { 0, NULL },
    { 0xf7fca10c, SUC_lexicon_tags_7 },
    { 0xccfca10c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdca113, SUC_lexicon_tags_17 },
    { 0x5a1ca113, SUC_lexicon_tags_12 },
    { 0x10fca115, SUC_lexicon_tags_15 },
    { 0x851ca116, SUC_lexicon_tags_60 },
    { 0xebbca117, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf7ca119, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ca11c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x50fca11e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57ca122, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9ca12a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19bca131, SUC_lexicon_tags_2 },
    { 0xfe5ca132, SUC_lexicon_tags_12 },
    { 0x78fca132, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x257ca136, SUC_lexicon_tags_87 },
    { 0x433ca136, SUC_lexicon_tags_17 },
    { 0x1e9ca138, SUC_lexicon_tags_61 },
    { 0x9adca139, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xed1ca13b, SUC_lexicon_tags_7 },
    { 0xeb5ca13b, SUC_lexicon_tags_68 },
    { 0x779ca13b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41dca13f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ca142, SUC_lexicon_tags_47 },
    { 0x475ca143, SUC_lexicon_tags_7 },
    { 0xcbdca143, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abca148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7ca14b, SUC_lexicon_tags_46 },
    { 0x9d7ca14b, SUC_lexicon_tags_65 },
    { 0x879ca14b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa77ca14f, SUC_lexicon_tags_17 },
    { 0xfb9ca150, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40dca152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65ca154, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33ca15d, SUC_lexicon_tags_25 },
    { 0x5c3ca15e, SUC_lexicon_tags_32 },
    { 0x663ca15e, SUC_lexicon_tags_11 },
    { 0xdf9ca160, SUC_lexicon_tags_61 },
    { 0x6d1ca161, SUC_lexicon_tags_23 },
    { 0x22dca15f, SUC_lexicon_tags_23 },
    { 0xa25ca163, SUC_lexicon_tags_12 },
    { 0x177ca164, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601ca168, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x28dca16a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9ca16d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbca170, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfca173, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5ca178, SUC_lexicon_tags_2 },
    { 0xb57ca178, SUC_lexicon_tags_38 },
    { 0x7d5ca17a, SUC_lexicon_tags_7 },
    { 0x751ca17b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bca183, SUC_lexicon_tags_19 },
    { 0x49dca184, SUC_lexicon_tags_2 },
    { 0xc9fca184, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x881ca187, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431ca18e, SUC_lexicon_tags_65 },
    { 0x623ca18f, SUC_lexicon_tags_18 },
    { 0x753ca190, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x593ca193, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf83ca195, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5fca197, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x697ca199, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215ca19d, SUC_lexicon_tags_15 },
    { 0xae5ca19d, SUC_lexicon_tags_1 },
    { 0x6e5ca19f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x64fca1a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x853ca1a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9ca1a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fca1ab, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41ca1ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7ca1b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfca1b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ca1b9, SUC_lexicon_tags_2 },
    { 0x8b3ca1ba, SUC_lexicon_tags_15 },
    { 0x46fca1bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20dca1c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdca1c3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edca1c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1ca1ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337ca1ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167ca1d2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x83ca1d4, SUC_lexicon_tags_16 },
    { 0x1bdca1d5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x8e7ca1d7, SUC_lexicon_tags_29 },
    { 0xde7ca1d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ca1dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f5ca1df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1dca1e1, SUC_lexicon_tags_17 },
    { 0x25ca1e2, SUC_lexicon_tags_7 },
    { 0x2a9ca1e1, SUC_lexicon_tags_82 },
    { 0x135ca1e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc9ca1e6, SUC_lexicon_tags_55 },
    { 0x873ca1e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15dca1e9, SUC_lexicon_tags_33 },
    { 0x2a9ca1ea, SUC_lexicon_tags_27 },
    { 0xcc3ca1eb, SUC_lexicon_tags_15 },
    { 0x341ca1ec, SUC_lexicon_tags_37 },
    { 0x1abca1ed, SUC_lexicon_tags_43 },
    { 0x375ca1ee, SUC_lexicon_tags_7 },
    { 0x143ca1eb, SUC_lexicon_tags_13 },
    { 0xc5bca1f0, SUC_lexicon_tags_17 },
    { 0xd65ca1ee, SUC_lexicon_tags_11 },
    { 0x239ca1f2, SUC_lexicon_tags_2 },
    { 0x6a9ca1f3, SUC_lexicon_tags_11 },
    { 0x2ddca1f2, SUC_lexicon_tags_2 },
    { 0x73ca1f4, SUC_lexicon_tags_7 },
    { 0xa9dca1f5, SUC_lexicon_tags_38 },
    { 0x96fca1f3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525ca1fb, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xde7ca1fd, SUC_lexicon_tags_14 },
    { 0xcd3ca1fe, SUC_lexicon_tags_12 },
    { 0x813ca1fd, SUC_lexicon_tags_17 },
    { 0x9fca200, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919ca209, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7ca20e, SUC_lexicon_tags_48 },
    { 0xfefca20f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa3bca211, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dca217, SUC_lexicon_tags_47 },
    { 0xeb7ca218, SUC_lexicon_tags_33 },
    { 0xeefca217, SUC_lexicon_tags_2 },
    { 0xea1ca21a, SUC_lexicon_tags_12 },
    { 0x7f3ca219, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1cfca21d, SUC_lexicon_tags_28 },
    { 0xb0fca21e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfca221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc35ca223, SUC_lexicon_tags_37 },
    { 0x817ca223, SUC_lexicon_tags_37 },
    { 0xe4bca225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7ca229, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131ca22e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dca231, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa57ca233, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92bca235, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdca238, SUC_lexicon_tags_31 },
    { 0x429ca238, SUC_lexicon_tags_68 },
    { 0x649ca239, SUC_lexicon_tags_1 },
    { 0x5e5ca23b, SUC_lexicon_tags_30 },
    { 0x95ca23c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1ca23f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13ca244, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3abca246, SUC_lexicon_tags_24 },
    { 0xc53ca246, SUC_lexicon_tags_37 },
    { 0x60fca248, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bca24c, SUC_lexicon_tags_31 },
    { 0x99fca24d, SUC_lexicon_tags_2 },
    { 0x41fca24e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bca251, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bca254, SUC_lexicon_tags_1 },
    { 0x437ca255, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xfbbca257, SUC_lexicon_tags_4 },
    { 0xdabca258, SUC_lexicon_tags_35 },
    { 0x25bca258, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dca25d, SUC_lexicon_tags_14 },
    { 0x811ca25e, SUC_lexicon_tags_11 },
    { 0xbf1ca25e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x157ca262, SUC_lexicon_tags_91 },
    { 0x143ca262, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fca267, SUC_lexicon_tags_55 },
    { 0xaddca268, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7e9ca26a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fca26d, SUC_lexicon_tags_83 },
    { 0xa73ca26e, SUC_lexicon_tags_267 },
    { 0xfd3ca26f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ca273, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ca277, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293ca27b, SUC_lexicon_tags_38 },
    { 0x4f9ca27b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bca27f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb07ca281, SUC_lexicon_tags_2 },
    { 0x9fbca282, SUC_lexicon_tags_1 },
    { 0x19fca283, SUC_lexicon_tags_6 },
    { 0x26bca284, SUC_lexicon_tags_19 },
    { 0xd3dca285, SUC_lexicon_tags_37 },
    { 0x7f7ca286, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1ca289, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x667ca28c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedca293, SUC_lexicon_tags_61 },
    { 0x853ca293, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0xd4dca296, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fca29d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349ca2a1, SUC_lexicon_tags_1 },
    { 0x339ca2a2, SUC_lexicon_tags_28 },
    { 0xb1bca2a3, SUC_lexicon_tags_12 },
    { 0x647ca2a3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x237ca2a6, SUC_lexicon_tags_25 },
    { 0xc6bca2a7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fca2aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07ca2af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75ca2b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bca2b7, SUC_lexicon_tags_60 },
    { 0x877ca2b7, SUC_lexicon_tags_3 },
    { 0xe65ca2b8, SUC_lexicon_tags_0 },
    { 0x5c5ca2ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf1ca2bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdca2be, SUC_lexicon_tags_15 },
    { 0xa8dca2bf, SUC_lexicon_tags_27 },
    { 0xcadca2be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31dca2c2, SUC_lexicon_tags_47 },
    { 0x9d5ca2c3, SUC_lexicon_tags_23 },
    { 0xbdfca2c4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09ca2c7, SUC_lexicon_tags_1 },
    { 0xf59ca2c8, SUC_lexicon_tags_61 },
    { 0x35fca2c9, SUC_lexicon_tags_37 },
    { 0x3d7ca2c9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf5ca2cc, SUC_lexicon_tags_12 },
    { 0x171ca2cd, SUC_lexicon_tags_2 },
    { 0x87fca2cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87ca2d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165ca2d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43ca2d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9ca2dc, SUC_lexicon_tags_1 },
    { 0xf87ca2dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x29ca2df, SUC_lexicon_tags_25 },
    { 0x921ca2df, SUC_lexicon_tags_12 },
    { 0xa7dca2e0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fca2e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33ca2e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb57ca2eb, SUC_lexicon_tags_31 },
    { 0x191ca2ec, SUC_lexicon_tags_47 },
    { 0x1cbca2eb, SUC_lexicon_tags_6 },
    { 0xd61ca2ec, SUC_lexicon_tags_31 },
    { 0xd75ca2ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabca2f3, SUC_lexicon_tags_37 },
    { 0x729ca2f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63dca2f6, SUC_lexicon_tags_32 },
    { 0x589ca2f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x901ca2fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bca2ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe5fca301, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75ca308, SUC_lexicon_tags_25 },
    { 0xcbdca308, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc69ca30b, SUC_lexicon_tags_61 },
    { 0xdf7ca30c, SUC_lexicon_tags_39 },
    { 0x343ca30d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfca313, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dca31f, SUC_lexicon_tags_38 },
    { 0xd2dca320, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfb1ca322, SUC_lexicon_tags_19 },
    { 0x497ca322, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfca327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9ca32a, SUC_lexicon_tags_1 },
    { 0x37ca32b, SUC_lexicon_tags_12 },
    { 0x4d3ca32b, SUC_lexicon_tags_1 },
    { 0x6dca32d, SUC_lexicon_tags_32 },
    { 0xc2dca32d, SUC_lexicon_tags_1 },
    { 0xcdbca32e, SUC_lexicon_tags_17 },
    { 0xb83ca330, SUC_lexicon_tags_2 },
    { 0xea7ca331, SUC_lexicon_tags_47 },
    { 0x949ca332, SUC_lexicon_tags_12 },
    { 0xc2fca32e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fca337, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91dca339, SUC_lexicon_tags_7 },
    { 0x387ca33a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47fca33c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357ca342, SUC_lexicon_tags_7 },
    { 0x433ca343, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a5ca345, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2afca347, SUC_lexicon_tags_37 },
    { 0xe11ca348, SUC_lexicon_tags_15 },
    { 0x98fca348, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fca34f, SUC_lexicon_tags_33 },
    { 0x859ca34f, SUC_lexicon_tags_7 },
    { 0xe25ca351, SUC_lexicon_tags_17 },
    { 0xdb1ca34f, SUC_lexicon_tags_15 },
    { 0xc13ca353, SUC_lexicon_tags_12 },
    { 0x6a3ca353, SUC_lexicon_tags_2 },
    { 0x9adca355, SUC_lexicon_tags_11 },
    { 0xda7ca356, SUC_lexicon_tags_2 },
    { 0xc69ca354, SUC_lexicon_tags_87 },
    { 0xb77ca357, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x169ca35a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fca35d, SUC_lexicon_tags_49 },
    { 0x5fca35d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcf1ca360, SUC_lexicon_tags_28 },
    { 0x9e9ca361, SUC_lexicon_tags_1 },
    { 0xe61ca360, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5ca366, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1ca36b, SUC_lexicon_tags_37 },
    { 0x95bca36c, SUC_lexicon_tags_33 },
    { 0x423ca36d, SUC_lexicon_tags_66 },
    { 0x479ca36e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81ca372, SUC_lexicon_tags_12 },
    { 0x2bbca372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69ca376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bca37d, SUC_lexicon_tags_47 },
    { 0x247ca37d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3c5ca380, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11ca384, SUC_lexicon_tags_7 },
    { 0x66dca385, SUC_lexicon_tags_85 },
    { 0x893ca384, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x535ca388, SUC_lexicon_tags_38 },
    { 0xa13ca388, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3ca391, SUC_lexicon_tags_37 },
    { 0x909ca391, SUC_lexicon_tags_7 },
    { 0x261ca393, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977ca398, SUC_lexicon_tags_17 },
    { 0xc05ca399, SUC_lexicon_tags_25 },
    { 0xd4dca398, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dca39f, SUC_lexicon_tags_1 },
    { 0xe9bca39f, SUC_lexicon_tags_27 },
    { 0x8a1ca3a1, SUC_lexicon_tags_43 },
    { 0x70fca3a2, SUC_lexicon_tags_37 },
    { 0x29dca3a2, SUC_lexicon_tags_37 },
    { 0x841ca3a0, SUC_lexicon_tags_45 },
    { 0xe7fca3a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35dca3a9, SUC_lexicon_tags_9 },
    { 0xfcbca3aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe91ca3ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc77ca3ae, SUC_lexicon_tags_37 },
    { 0x78dca3af, SUC_lexicon_tags_43 },
    { 0x5cdca3af, SUC_lexicon_tags_37 },
    { 0xd25ca3b1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1ca3b6, SUC_lexicon_tags_30 },
    { 0xb91ca3b7, SUC_lexicon_tags_21 },
    { 0xfbfca3b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5f3ca3ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1fca3bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fca3c0, SUC_lexicon_tags_16 },
    { 0xcb1ca3c0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3a3ca3c3, SUC_lexicon_tags_27 },
    { 0xd93ca3c3, SUC_lexicon_tags_9 },
    { 0xf31ca3c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabca3c8, SUC_lexicon_tags_37 },
    { 0xa41ca3c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x10fca3cb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcedca3cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99ca3d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc97ca3d2, SUC_lexicon_tags_49 },
    { 0x12bca3d3, SUC_lexicon_tags_15 },
    { 0xaa7ca3d4, SUC_lexicon_tags_31 },
    { 0xc7dca3d5, SUC_lexicon_tags_64 },
    { 0x84fca3d6, SUC_lexicon_tags_79 },
    { 0xf7bca3d7, SUC_lexicon_tags_2 },
    { 0xa53ca3d8, SUC_lexicon_tags_23 },
    { 0x5fca3d9, SUC_lexicon_tags_17 },
    { 0xc4fca3da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35ca3e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4b9ca3e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8f9ca3e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03ca3e8, SUC_lexicon_tags_7 },
    { 0x5e7ca3e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fca3ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x693ca3ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x57ca3f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4adca3f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dca3f6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5ca3f9, SUC_lexicon_tags_47 },
    { 0x773ca3f9, SUC_lexicon_tags_37 },
    { 0x2fbca3fb, SUC_lexicon_tags_105 },
    { 0x397ca3f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7ca401, SUC_lexicon_tags_25 },
    { 0x8f5ca401, SUC_lexicon_tags_15 },
    { 0x743ca401, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdca40d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1ca411, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5ffca413, SUC_lexicon_tags_2 },
    { 0xe99ca414, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dca418, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadca428, SUC_lexicon_tags_15 },
    { 0xa8fca428, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x96fca42b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125ca42f, SUC_lexicon_tags_19 },
    { 0x567ca42f, SUC_lexicon_tags_38 },
    { 0x887ca42f, SUC_lexicon_tags_14 },
    { 0xe3bca432, SUC_lexicon_tags_37 },
    { 0x1efca431, SUC_lexicon_tags_47 },
    { 0x879ca434, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26fca436, SUC_lexicon_tags_2 },
    { 0x5a7ca437, SUC_lexicon_tags_25 },
    { 0xdafca437, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x667ca43a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687ca43e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fca444, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fca447, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1ca44b, SUC_lexicon_tags_1 },
    { 0x2f9ca44c, SUC_lexicon_tags_1 },
    { 0x801ca44d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x76bca44f, SUC_lexicon_tags_37 },
    { 0xe75ca450, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45ca453, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769ca457, SUC_lexicon_tags_15 },
    { 0xe69ca458, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x893ca45a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623ca45e, SUC_lexicon_tags_12 },
    { 0xbe1ca45e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x479ca462, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edca46b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffca46d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905ca473, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bca477, SUC_lexicon_tags_11 },
    { 0x90bca477, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x659ca47b, SUC_lexicon_tags_2 },
    { 0x905ca47c, SUC_lexicon_tags_29 },
    { 0xf29ca47c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x879ca480, SUC_lexicon_tags_15 },
    { 0x983ca481, SUC_lexicon_tags_15 },
    { 0xc27ca481, SUC_lexicon_tags_27 },
    { 0xa69ca481, SUC_lexicon_tags_37 },
    { 0x90dca484, SUC_lexicon_tags_85 },
    { 0x28fca482, SUC_lexicon_tags_19 },
    { 0xa51ca483, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719ca491, SUC_lexicon_tags_68 },
    { 0x9abca491, SUC_lexicon_tags_4 },
    { 0xe73ca491, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadca496, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x31bca499, SUC_lexicon_tags_49 },
    { 0xce1ca499, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddca4a0, SUC_lexicon_tags_25 },
    { 0xfc9ca4a1, SUC_lexicon_tags_17 },
    { 0x883ca4a0, SUC_lexicon_tags_23 },
    { 0x579ca4a1, SUC_lexicon_tags_103 },
    { 0x13dca4a4, SUC_lexicon_tags_43 },
    { 0xb7dca4a0, SUC_lexicon_tags_37 },
    { 0xc5dca4a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaebca4a8, SUC_lexicon_tags_1 },
    { 0x191ca4a8, SUC_lexicon_tags_31 },
    { 0x2dbca4a9, SUC_lexicon_tags_1 },
    { 0x513ca4ab, SUC_lexicon_tags_14 },
    { 0x211ca4ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad5ca4af, SUC_lexicon_tags_47 },
    { 0x38bca4b0, SUC_lexicon_tags_2 },
    { 0xe69ca4b1, SUC_lexicon_tags_37 },
    { 0x57ca4b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69ca4b6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe15ca4b8, SUC_lexicon_tags_19 },
    { 0x2a5ca4b8, SUC_lexicon_tags_33 },
    { 0x6c5ca4ba, SUC_lexicon_tags_50 },
    { 0x7dbca4ba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9ca4bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94dca4c1, SUC_lexicon_tags_15 },
    { 0x5bbca4c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x809ca4c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa9bca4c7, SUC_lexicon_tags_60 },
    { 0xdc9ca4c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9ca4d0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x773ca4d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbca4d8, SUC_lexicon_tags_65 },
    { 0x289ca4d9, SUC_lexicon_tags_74 },
    { 0x4a3ca4da, SUC_lexicon_tags_8 },
    { 0xfa9ca4d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a7ca4dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd41ca4df, SUC_lexicon_tags_1051 },
    { 0x3ebca4e0, SUC_lexicon_tags_11 },
    { 0xcffca4df, SUC_lexicon_tags_14 },
    { 0x8a3ca4e2, SUC_lexicon_tags_801 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59ca4e7, SUC_lexicon_tags_37 },
    { 0x5ca4e7, SUC_lexicon_tags_37 },
    { 0x5ffca4e7, SUC_lexicon_tags_37 },
    { 0xc3dca4e7, SUC_lexicon_tags_1 },
    { 0x325ca4e7, SUC_lexicon_tags_1 },
    { 0x3d3ca4ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97dca4ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bca4f6, SUC_lexicon_tags_1 },
    { 0xaa1ca4f7, SUC_lexicon_tags_37 },
    { 0x2f1ca4f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3ca4fc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07ca4ff, SUC_lexicon_tags_2 },
    { 0x5d7ca500, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x487ca502, SUC_lexicon_tags_17 },
    { 0xb0dca503, SUC_lexicon_tags_3 },
    { 0xa9bca504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7ca508, SUC_lexicon_tags_37 },
    { 0x95ca509, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3ca50c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x103ca50f, SUC_lexicon_tags_46 },
    { 0x70bca50f, SUC_lexicon_tags_68 },
    { 0xf65ca50f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73ca513, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dca516, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa4fca518, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fca51c, SUC_lexicon_tags_6 },
    { 0x80fca51d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbca52b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fca52e, SUC_lexicon_tags_31 },
    { 0x2ffca52f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x149ca531, SUC_lexicon_tags_27 },
    { 0xa73ca532, SUC_lexicon_tags_56 },
    { 0x6b5ca532, SUC_lexicon_tags_28 },
    { 0xb4dca534, SUC_lexicon_tags_37 },
    { 0xb49ca531, SUC_lexicon_tags_27 },
    { 0xa9ca534, SUC_lexicon_tags_7 },
    { 0x973ca537, SUC_lexicon_tags_2 },
    { 0x3bdca538, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3ca541, SUC_lexicon_tags_26 },
    { 0x4a7ca542, SUC_lexicon_tags_38 },
    { 0xf3fca541, SUC_lexicon_tags_0 },
    { 0x60bca544, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fca54a, SUC_lexicon_tags_6 },
    { 0xdefca54a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abca54e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73fca550, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xefca552, SUC_lexicon_tags_29 },
    { 0x89bca553, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60dca558, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x685ca55a, SUC_lexicon_tags_2 },
    { 0x72bca55a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ca560, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38dca566, SUC_lexicon_tags_1 },
    { 0x41ca567, SUC_lexicon_tags_27 },
    { 0x61ca567, SUC_lexicon_tags_34 },
    { 0x3bbca566, SUC_lexicon_tags_11 },
    { 0x4e3ca56a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503ca56e, SUC_lexicon_tags_57 },
    { 0xc5ca56e, SUC_lexicon_tags_11 },
    { 0xbb9ca570, SUC_lexicon_tags_8 },
    { 0x375ca570, SUC_lexicon_tags_25 },
    { 0x8f3ca572, SUC_lexicon_tags_19 },
    { 0xc01ca573, SUC_lexicon_tags_15 },
    { 0x5b3ca56e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99dca576, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bca579, SUC_lexicon_tags_15 },
    { 0xd0dca57a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbca57d, SUC_lexicon_tags_12 },
    { 0x9c3ca57d, SUC_lexicon_tags_14 },
    { 0x481ca57d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9ca582, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa33ca584, SUC_lexicon_tags_15 },
    { 0xd43ca585, SUC_lexicon_tags_24 },
    { 0x973ca586, SUC_lexicon_tags_11 },
    { 0xa1dca586, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91ca58e, SUC_lexicon_tags_25 },
    { 0x333ca58f, SUC_lexicon_tags_84 },
    { 0x58dca590, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x22fca592, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dca595, SUC_lexicon_tags_1 },
    { 0xa2fca595, SUC_lexicon_tags_17 },
    { 0xe91ca597, SUC_lexicon_tags_15 },
    { 0x59ca597, SUC_lexicon_tags_29 },
    { 0xb99ca598, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fca59c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dca59f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8ffca5a1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597ca5a6, SUC_lexicon_tags_9 },
    { 0xf65ca5a7, SUC_lexicon_tags_8 },
    { 0xe5bca5a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9ca5ac, SUC_lexicon_tags_48 },
    { 0x7abca5ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x531ca5af, SUC_lexicon_tags_60 },
    { 0xfb7ca5af, SUC_lexicon_tags_0 },
    { 0x73bca5b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd09ca5b3, SUC_lexicon_tags_19 },
    { 0x1c7ca5b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bca5b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ca5bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfca5ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375ca5cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edca5e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcd1ca5e2, SUC_lexicon_tags_7 },
    { 0x77fca5e3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb7ca5e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfca5ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1ca5ed, SUC_lexicon_tags_29 },
    { 0x92fca5ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03ca5f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9ca5f8, SUC_lexicon_tags_38 },
    { 0x5c3ca5f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7ca5fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79ca607, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xce1ca609, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bca60e, SUC_lexicon_tags_0 },
    { 0x801ca60f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdca616, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc9ca618, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x341ca61b, SUC_lexicon_tags_37 },
    { 0xb85ca61c, SUC_lexicon_tags_12 },
    { 0x583ca61c, SUC_lexicon_tags_25 },
    { 0x115ca61d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1ca623, SUC_lexicon_tags_46 },
    { 0x467ca623, SUC_lexicon_tags_37 },
    { 0xdd1ca625, SUC_lexicon_tags_48 },
    { 0x8ffca625, SUC_lexicon_tags_33 },
    { 0xe29ca625, SUC_lexicon_tags_542 },
    { 0x4bfca627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763ca62d, SUC_lexicon_tags_63 },
    { 0xaa9ca62d, SUC_lexicon_tags_7 },
    { 0xa3dca62f, SUC_lexicon_tags_1 },
    { 0x47dca630, SUC_lexicon_tags_47 },
    { 0xc37ca62d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe97ca633, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fca636, SUC_lexicon_tags_250 },
    { 0xbadca637, SUC_lexicon_tags_28 },
    { 0x85fca638, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3ca63b, SUC_lexicon_tags_11 },
    { 0x333ca63c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaf3ca63e, SUC_lexicon_tags_14 },
    { 0xc45ca63e, SUC_lexicon_tags_37 },
    { 0x9a9ca640, SUC_lexicon_tags_10 },
    { 0x9b9ca641, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fca646, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x559ca649, SUC_lexicon_tags_60 },
    { 0x3afca64a, SUC_lexicon_tags_80 },
    { 0x911ca649, SUC_lexicon_tags_11 },
    { 0x65ca649, SUC_lexicon_tags_1 },
    { 0x32fca64c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649ca652, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5ca655, SUC_lexicon_tags_33 },
    { 0x275ca656, SUC_lexicon_tags_15 },
    { 0xf71ca656, SUC_lexicon_tags_4 },
    { 0xd9ca656, SUC_lexicon_tags_1 },
    { 0x8e1ca659, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x291ca65b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88bca65f, SUC_lexicon_tags_7 },
    { 0x35ca660, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbca669, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbdca66b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdca66e, SUC_lexicon_tags_37 },
    { 0x40dca66f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bca678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac7ca67a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x299ca67c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fca68a, SUC_lexicon_tags_15 },
    { 0x317ca68b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fca68f, SUC_lexicon_tags_15 },
    { 0x7b7ca68f, SUC_lexicon_tags_4 },
    { 0xd9bca68f, SUC_lexicon_tags_81 },
    { 0x88fca68f, SUC_lexicon_tags_17 },
    { 0x49ca693, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dca697, SUC_lexicon_tags_2 },
    { 0x88dca697, SUC_lexicon_tags_12 },
    { 0x111ca699, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x773ca69b, SUC_lexicon_tags_37 },
    { 0xde1ca69c, SUC_lexicon_tags_14 },
    { 0xd4bca69d, SUC_lexicon_tags_15 },
    { 0xda5ca69d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf49ca6a0, SUC_lexicon_tags_37 },
    { 0x425ca6a1, SUC_lexicon_tags_17 },
    { 0x98fca6a2, SUC_lexicon_tags_23 },
    { 0x1fdca6a3, SUC_lexicon_tags_44 },
    { 0x3b9ca6a4, SUC_lexicon_tags_2 },
    { 0xa17ca6a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9ca6a9, SUC_lexicon_tags_12 },
    { 0xfa1ca6aa, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bca6b0, SUC_lexicon_tags_1 },
    { 0xb87ca6b1, SUC_lexicon_tags_17 },
    { 0x6b9ca6b2, SUC_lexicon_tags_27 },
    { 0x65bca6b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfadca6b5, SUC_lexicon_tags_37 },
    { 0x365ca6b5, SUC_lexicon_tags_17 },
    { 0xfa9ca6b7, SUC_lexicon_tags_87 },
    { 0x10dca6b8, SUC_lexicon_tags_27 },
    { 0xd23ca6b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81ca6c3, SUC_lexicon_tags_11 },
    { 0xb9bca6c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ca6c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3ca6d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54bca6d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bca6d9, SUC_lexicon_tags_23 },
    { 0xa4dca6da, SUC_lexicon_tags_7 },
    { 0x1fdca6d9, SUC_lexicon_tags_12 },
    { 0x735ca6d9, SUC_lexicon_tags_11 },
    { 0x54bca6dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fca6e0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x489ca6e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91dca6e6, SUC_lexicon_tags_40 },
    { 0xebca6e7, SUC_lexicon_tags_2 },
    { 0xb35ca6e8, SUC_lexicon_tags_11 },
    { 0x549ca6e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dca6ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5ca6f5, SUC_lexicon_tags_19 },
    { 0x9e9ca6f6, SUC_lexicon_tags_37 },
    { 0xbbbca6f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343ca6fc, SUC_lexicon_tags_6 },
    { 0xb6dca6fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bca701, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x651ca704, SUC_lexicon_tags_17 },
    { 0xa65ca705, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dca70b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ca70e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdca711, SUC_lexicon_tags_46 },
    { 0x37dca712, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x569ca715, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ca718, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ca71b, SUC_lexicon_tags_38 },
    { 0xb77ca71b, SUC_lexicon_tags_23 },
    { 0x833ca71d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dca720, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x137ca723, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0bca725, SUC_lexicon_tags_17 },
    { 0xff7ca726, SUC_lexicon_tags_37 },
    { 0xbbdca727, SUC_lexicon_tags_47 },
    { 0x3a9ca728, SUC_lexicon_tags_12 },
    { 0xac3ca729, SUC_lexicon_tags_8 },
    { 0x241ca72a, SUC_lexicon_tags_45 },
    { 0xe0bca72b, SUC_lexicon_tags_12 },
    { 0xb37ca729, SUC_lexicon_tags_37 },
    { 0xc39ca729, SUC_lexicon_tags_11 },
    { 0x297ca72d, SUC_lexicon_tags_60 },
    { 0xa9bca72d, SUC_lexicon_tags_43 },
    { 0xec3ca729, SUC_lexicon_tags_7 },
    { 0x595ca731, SUC_lexicon_tags_10 },
    { 0x85bca731, SUC_lexicon_tags_148 },
    { 0x54dca727, SUC_lexicon_tags_68 },
    { 0x985ca731, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83ca737, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x325ca73a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x681ca73c, SUC_lexicon_tags_7 },
    { 0x547ca73d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x5d5ca73f, SUC_lexicon_tags_6 },
    { 0x379ca740, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3adca742, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9ca748, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163ca74d, SUC_lexicon_tags_7 },
    { 0x8cbca74e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x815ca750, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bca754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adca758, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93ca75c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe3ca75e, SUC_lexicon_tags_12 },
    { 0x249ca75f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffca762, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475ca766, SUC_lexicon_tags_11 },
    { 0x54bca767, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1ca76a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1ca770, SUC_lexicon_tags_27 },
    { 0x837ca770, SUC_lexicon_tags_7 },
    { 0x975ca772, SUC_lexicon_tags_33 },
    { 0xb2fca773, SUC_lexicon_tags_25 },
    { 0x65bca774, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x209ca776, SUC_lexicon_tags_11 },
    { 0xa91ca776, SUC_lexicon_tags_2 },
    { 0xbe3ca776, SUC_lexicon_tags_68 },
    { 0x933ca779, SUC_lexicon_tags_0 },
    { 0x2bdca776, SUC_lexicon_tags_14 },
    { 0x635ca77b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeddca77e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509ca782, SUC_lexicon_tags_15 },
    { 0x55ca783, SUC_lexicon_tags_15 },
    { 0xe93ca783, SUC_lexicon_tags_63 },
    { 0xb91ca784, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53ca789, SUC_lexicon_tags_33 },
    { 0x6fca78a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x52fca78c, SUC_lexicon_tags_2 },
    { 0xcabca78d, SUC_lexicon_tags_17 },
    { 0x285ca78e, SUC_lexicon_tags_1 },
    { 0xebca78f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9ca792, SUC_lexicon_tags_49 },
    { 0x479ca793, SUC_lexicon_tags_42 },
    { 0xb0fca794, SUC_lexicon_tags_48 },
    { 0xf3dca794, SUC_lexicon_tags_11 },
    { 0xde5ca796, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31fca798, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8bbca79a, SUC_lexicon_tags_38 },
    { 0xfafca79b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357ca79f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x235ca7a2, SUC_lexicon_tags_29 },
    { 0xc21ca7a3, SUC_lexicon_tags_21 },
    { 0x141ca7a4, SUC_lexicon_tags_1 },
    { 0x42dca7a5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7ca7a9, SUC_lexicon_tags_7 },
    { 0xe25ca7aa, SUC_lexicon_tags_11 },
    { 0x801ca7aa, SUC_lexicon_tags_47 },
    { 0xa65ca7ac, SUC_lexicon_tags_17 },
    { 0x819ca7ad, SUC_lexicon_tags_10 },
    { 0xb61ca7ac, SUC_lexicon_tags_30 },
    { 0x405ca7ab, SUC_lexicon_tags_28 },
    { 0xe43ca7b0, SUC_lexicon_tags_0 },
    { 0x307ca7ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ca7b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3ca7b9, SUC_lexicon_tags_26 },
    { 0x987ca7ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27ca7bd, SUC_lexicon_tags_30 },
    { 0xd8fca7be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x975ca7c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7ca7c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927ca7ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd9ca7cc, SUC_lexicon_tags_6 },
    { 0x8d5ca7cd, SUC_lexicon_tags_17 },
    { 0xf1dca7ce, SUC_lexicon_tags_17 },
    { 0x819ca7cf, SUC_lexicon_tags_37 },
    { 0xa6fca7ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x623ca7d3, SUC_lexicon_tags_17 },
    { 0xdf3ca7d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x587ca7d6, SUC_lexicon_tags_45 },
    { 0x73bca7d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc13ca7d9, SUC_lexicon_tags_17 },
    { 0xf81ca7da, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ca7de, SUC_lexicon_tags_4 },
    { 0x70fca7df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5ca7e4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdca7e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae7ca7e9, SUC_lexicon_tags_47 },
    { 0xbf1ca7e9, SUC_lexicon_tags_34 },
    { 0xc15ca7eb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x86bca7ed, SUC_lexicon_tags_33 },
    { 0x14dca7ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeedca7f0, SUC_lexicon_tags_4 },
    { 0xea5ca7f1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xad1ca7f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bca7fa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7bca7fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ca7ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x203ca802, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x881ca804, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fca80e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5ca818, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3ca81b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc1ca81d, SUC_lexicon_tags_37 },
    { 0x19bca81d, SUC_lexicon_tags_37 },
    { 0xd6dca81d, SUC_lexicon_tags_14 },
    { 0x621ca820, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb09ca822, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x5d7ca824, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ca827, SUC_lexicon_tags_37 },
    { 0x727ca828, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb99ca82a, SUC_lexicon_tags_5 },
    { 0x7a3ca82a, SUC_lexicon_tags_33 },
    { 0x21fca82a, SUC_lexicon_tags_0 },
    { 0x805ca82d, SUC_lexicon_tags_17 },
    { 0xfa9ca82e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509ca833, SUC_lexicon_tags_6 },
    { 0x8f3ca834, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1ca83a, SUC_lexicon_tags_37 },
    { 0x98dca83b, SUC_lexicon_tags_43 },
    { 0x5fdca83b, SUC_lexicon_tags_37 },
    { 0x443ca83c, SUC_lexicon_tags_57 },
    { 0x339ca83e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43ca840, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2c5ca842, SUC_lexicon_tags_7 },
    { 0x181ca843, SUC_lexicon_tags_55 },
    { 0xea7ca844, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x821ca847, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdca84c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb1ca84e, SUC_lexicon_tags_18 },
    { 0xfc9ca84f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x639ca852, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x22dca855, SUC_lexicon_tags_951 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53ca858, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa4bca85a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xac5ca85c, SUC_lexicon_tags_26 },
    { 0x33bca85c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x42dca85f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb79ca861, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9ca873, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ca876, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdcdca878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ca87f, SUC_lexicon_tags_63 },
    { 0x1c3ca87f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55ca883, SUC_lexicon_tags_63 },
    { 0x5efca884, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dca888, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48fca893, SUC_lexicon_tags_11 },
    { 0x90bca894, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4b7ca896, SUC_lexicon_tags_53 },
    { 0x9f5ca897, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bca89c, SUC_lexicon_tags_15 },
    { 0x677ca89d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17ca8a4, SUC_lexicon_tags_31 },
    { 0xe25ca8a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2cdca8a7, SUC_lexicon_tags_17 },
    { 0xde7ca8a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bca8ab, SUC_lexicon_tags_25 },
    { 0x25ca8ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xebfca8ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7ca8b9, SUC_lexicon_tags_11 },
    { 0x139ca8ba, SUC_lexicon_tags_27 },
    { 0x7b5ca8bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd55ca8bd, SUC_lexicon_tags_11 },
    { 0x3adca8be, SUC_lexicon_tags_1 },
    { 0x46bca8bf, SUC_lexicon_tags_38 },
    { 0xf09ca8c0, SUC_lexicon_tags_6 },
    { 0xc97ca8bf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3ca8c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9ca8cd, SUC_lexicon_tags_56 },
    { 0xca3ca8ce, SUC_lexicon_tags_17 },
    { 0x17bca8cf, SUC_lexicon_tags_17 },
    { 0xf41ca8cf, SUC_lexicon_tags_37 },
    { 0x7f3ca8d1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x309ca8d4, SUC_lexicon_tags_15 },
    { 0x639ca8d4, SUC_lexicon_tags_28 },
    { 0x749ca8d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dca8db, SUC_lexicon_tags_14 },
    { 0x5bdca8dc, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbc3ca8de, SUC_lexicon_tags_25 },
    { 0x359ca8df, SUC_lexicon_tags_12 },
    { 0x6f1ca8df, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9ca8e3, SUC_lexicon_tags_43 },
    { 0x8fbca8e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30fca8e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa3dca8e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x905ca8eb, SUC_lexicon_tags_12 },
    { 0x4bdca8ec, SUC_lexicon_tags_25 },
    { 0x3b7ca8ec, SUC_lexicon_tags_11 },
    { 0xbabca8ed, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefca8f5, SUC_lexicon_tags_37 },
    { 0x813ca8f6, SUC_lexicon_tags_4 },
    { 0x54dca8f6, SUC_lexicon_tags_10 },
    { 0xa63ca8f8, SUC_lexicon_tags_15 },
    { 0x75ca8f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abca8fe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ca903, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95ca90a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01ca90d, SUC_lexicon_tags_45 },
    { 0xac9ca90d, SUC_lexicon_tags_25 },
    { 0xcdbca90e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3ca916, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8ebca918, SUC_lexicon_tags_33 },
    { 0xf5bca919, SUC_lexicon_tags_33 },
    { 0x6e7ca91a, SUC_lexicon_tags_151 },
    { 0x8bdca91b, SUC_lexicon_tags_28 },
    { 0x3cbca91a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x875ca91e, SUC_lexicon_tags_30 },
    { 0xb29ca91f, SUC_lexicon_tags_0 },
    { 0xa9dca91f, SUC_lexicon_tags_87 },
    { 0x50dca920, SUC_lexicon_tags_14 },
    { 0x8cdca922, SUC_lexicon_tags_87 },
    { 0xa17ca922, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc57ca925, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x22fca928, SUC_lexicon_tags_37 },
    { 0x6fdca929, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ca932, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x223ca935, SUC_lexicon_tags_23 },
    { 0xb93ca936, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbca939, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf87ca93b, SUC_lexicon_tags_14 },
    { 0xd4bca93c, SUC_lexicon_tags_7 },
    { 0xfffca93c, SUC_lexicon_tags_8 },
    { 0x251ca93e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9ca942, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7ca949, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bca94f, SUC_lexicon_tags_9 },
    { 0xbb9ca94f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49dca954, SUC_lexicon_tags_37 },
    { 0x595ca955, SUC_lexicon_tags_47 },
    { 0xf91ca956, SUC_lexicon_tags_12 },
    { 0xc1bca957, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe23ca959, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x66fca95b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb41ca95d, SUC_lexicon_tags_64 },
    { 0xf19ca95e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99ca962, SUC_lexicon_tags_56 },
    { 0xa73ca962, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2bdca965, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdca969, SUC_lexicon_tags_69 },
    { 0x413ca969, SUC_lexicon_tags_37 },
    { 0x90fca96b, SUC_lexicon_tags_43 },
    { 0x169ca969, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fca96f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f7ca971, SUC_lexicon_tags_11 },
    { 0x39ca972, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x26bca975, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13ca985, SUC_lexicon_tags_10 },
    { 0xc5dca985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaedca988, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x9afca98a, SUC_lexicon_tags_37 },
    { 0xac9ca98b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1b7ca98d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x965ca990, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdabca992, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3ca995, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7ca999, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1ca99d, SUC_lexicon_tags_7 },
    { 0x137ca99d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xeddca9a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x403ca9a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefca9a7, SUC_lexicon_tags_13 },
    { 0x3e1ca9a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3fca9aa, SUC_lexicon_tags_38 },
    { 0xa8fca9ab, SUC_lexicon_tags_30 },
    { 0x817ca9aa, SUC_lexicon_tags_12 },
    { 0x6a1ca9ab, SUC_lexicon_tags_8 },
    { 0xba9ca9ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509ca9b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x359ca9ba, SUC_lexicon_tags_9 },
    { 0xbe3ca9bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x417ca9be, SUC_lexicon_tags_47 },
    { 0x9b7ca9be, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bca9c2, SUC_lexicon_tags_6 },
    { 0x4cbca9c3, SUC_lexicon_tags_45 },
    { 0xf4fca9c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bca9d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x647ca9d2, SUC_lexicon_tags_37 },
    { 0x473ca9d3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x221ca9d5, SUC_lexicon_tags_280 },
    { 0, NULL },
    { 0x357ca9d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x26fca9d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1ca9de, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89ca9e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf55ca9e4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe95ca9e6, SUC_lexicon_tags_11 },
    { 0x8d1ca9e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdca9ea, SUC_lexicon_tags_7 },
    { 0x351ca9eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x421ca9ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ca9f1, SUC_lexicon_tags_25 },
    { 0x53fca9f2, SUC_lexicon_tags_7 },
    { 0xb85ca9f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbca9fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cbca9fc, SUC_lexicon_tags_7 },
    { 0x5f1ca9fd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x12dcaa00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685caa07, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45caa0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25caa10, SUC_lexicon_tags_23 },
    { 0x89dcaa11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcaa16, SUC_lexicon_tags_19 },
    { 0xfdfcaa17, SUC_lexicon_tags_154 },
    { 0x341caa16, SUC_lexicon_tags_37 },
    { 0x1c9caa18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81caa21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adcaa28, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9caa2d, SUC_lexicon_tags_45 },
    { 0x587caa2e, SUC_lexicon_tags_25 },
    { 0xb23caa2f, SUC_lexicon_tags_50 },
    { 0xeabcaa30, SUC_lexicon_tags_12 },
    { 0x53dcaa2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85caa34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7caa43, SUC_lexicon_tags_1 },
    { 0x77fcaa44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7edcaa46, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bcaa4b, SUC_lexicon_tags_30 },
    { 0xe37caa4c, SUC_lexicon_tags_46 },
    { 0x5f5caa4b, SUC_lexicon_tags_4 },
    { 0xa77caa4e, SUC_lexicon_tags_17 },
    { 0xa85caa4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe27caa51, SUC_lexicon_tags_43 },
    { 0xc4fcaa51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7caa5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85caa5e, SUC_lexicon_tags_104 },
    { 0xae9caa5f, SUC_lexicon_tags_17 },
    { 0x3bbcaa60, SUC_lexicon_tags_45 },
    { 0x55fcaa60, SUC_lexicon_tags_15 },
    { 0x16fcaa5f, SUC_lexicon_tags_117 },
    { 0x737caa63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc43caa65, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277caa69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d9caa6b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39caa6f, SUC_lexicon_tags_15 },
    { 0xd5fcaa70, SUC_lexicon_tags_47 },
    { 0xdb3caa71, SUC_lexicon_tags_33 },
    { 0xb2dcaa72, SUC_lexicon_tags_87 },
    { 0xf1caa73, SUC_lexicon_tags_63 },
    { 0x1e1caa73, SUC_lexicon_tags_25 },
    { 0x187caa73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb93caa77, SUC_lexicon_tags_42 },
    { 0x66bcaa78, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58dcaa7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1bcaa7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc95caa80, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fcaa8b, SUC_lexicon_tags_2 },
    { 0x3e3caa8c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x797caa8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5caa95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cbcaa97, SUC_lexicon_tags_79 },
    { 0xd61caa97, SUC_lexicon_tags_0 },
    { 0xc45caa98, SUC_lexicon_tags_48 },
    { 0x177caa9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc8dcaa9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x20bcaa9e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9caaa4, SUC_lexicon_tags_21 },
    { 0x723caaa5, SUC_lexicon_tags_15 },
    { 0x6d9caaa5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9caaaa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13caaae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x63dcaab1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7caab4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771caab8, SUC_lexicon_tags_2 },
    { 0xdcfcaab8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49caabc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9caabe, SUC_lexicon_tags_12 },
    { 0x77fcaabe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dcaac4, SUC_lexicon_tags_37 },
    { 0x3c5caac4, SUC_lexicon_tags_25 },
    { 0x50dcaac6, SUC_lexicon_tags_17 },
    { 0x88dcaac7, SUC_lexicon_tags_77 },
    { 0x69bcaac8, SUC_lexicon_tags_45 },
    { 0x3caac7, SUC_lexicon_tags_47 },
    { 0xb55caac7, SUC_lexicon_tags_14 },
    { 0x707caacb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d1caacd, SUC_lexicon_tags_47 },
    { 0x5e9caacd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259caad2, SUC_lexicon_tags_47 },
    { 0xb43caad3, SUC_lexicon_tags_19 },
    { 0xfedcaad4, SUC_lexicon_tags_13 },
    { 0x9bdcaad5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799caadc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fcaadf, SUC_lexicon_tags_26 },
    { 0x683caae0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfcaae4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebcaae8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bcaaeb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa7dcaaed, SUC_lexicon_tags_6 },
    { 0xf4fcaaee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413caaf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225caaf8, SUC_lexicon_tags_21 },
    { 0x85caaf8, SUC_lexicon_tags_39 },
    { 0x11bcaafa, SUC_lexicon_tags_37 },
    { 0x52fcaafb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bcab03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x865cab05, SUC_lexicon_tags_12 },
    { 0xf61cab05, SUC_lexicon_tags_8 },
    { 0x151cab07, SUC_lexicon_tags_12 },
    { 0xaafcab07, SUC_lexicon_tags_0 },
    { 0x11dcab08, SUC_lexicon_tags_37 },
    { 0xbfcab09, SUC_lexicon_tags_11 },
    { 0x1b5cab0b, SUC_lexicon_tags_33 },
    { 0x17fcab0b, SUC_lexicon_tags_25 },
    { 0x845cab08, SUC_lexicon_tags_3 },
    { 0x32bcab0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a7cab10, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x619cab12, SUC_lexicon_tags_14 },
    { 0xd81cab12, SUC_lexicon_tags_28 },
    { 0xd2dcab14, SUC_lexicon_tags_28 },
    { 0x317cab15, SUC_lexicon_tags_11 },
    { 0xeabcab16, SUC_lexicon_tags_11 },
    { 0x66fcab17, SUC_lexicon_tags_34 },
    { 0x46bcab18, SUC_lexicon_tags_37 },
    { 0x65bcab17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x861cab1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9bcab1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9cab21, SUC_lexicon_tags_43 },
    { 0x7d7cab21, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbcab2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17cab34, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3cab3e, SUC_lexicon_tags_15 },
    { 0xddfcab3f, SUC_lexicon_tags_63 },
    { 0x8fbcab40, SUC_lexicon_tags_15 },
    { 0xea1cab3f, SUC_lexicon_tags_47 },
    { 0xe7cab41, SUC_lexicon_tags_12 },
    { 0x789cab43, SUC_lexicon_tags_107 },
    { 0x2e5cab43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbcab47, SUC_lexicon_tags_15 },
    { 0x397cab47, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x771cab4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837cab4f, SUC_lexicon_tags_37 },
    { 0x41cab50, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa07cab52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383cab58, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8bfcab5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33cab5f, SUC_lexicon_tags_0 },
    { 0xc07cab60, SUC_lexicon_tags_11 },
    { 0x62fcab61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fcab64, SUC_lexicon_tags_46 },
    { 0xb6fcab65, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcab6a, SUC_lexicon_tags_23 },
    { 0xd5dcab6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bcab72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35cab7c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421cab81, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x171cab84, SUC_lexicon_tags_46 },
    { 0xf77cab85, SUC_lexicon_tags_37 },
    { 0xbb1cab85, SUC_lexicon_tags_15 },
    { 0xa47cab86, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39cab8b, SUC_lexicon_tags_0 },
    { 0x1e3cab8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efcab8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41cab96, SUC_lexicon_tags_11 },
    { 0xf0bcab96, SUC_lexicon_tags_39 },
    { 0xbc1cab96, SUC_lexicon_tags_1 },
    { 0x355cab99, SUC_lexicon_tags_16 },
    { 0x5ffcab9a, SUC_lexicon_tags_6 },
    { 0xd13cab99, SUC_lexicon_tags_60 },
    { 0x5a9cab9c, SUC_lexicon_tags_28 },
    { 0x911cab9d, SUC_lexicon_tags_34 },
    { 0x877cab9b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893caba5, SUC_lexicon_tags_29 },
    { 0xd3dcaba6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16dcaba8, SUC_lexicon_tags_19 },
    { 0xba1caba8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74bcabab, SUC_lexicon_tags_74 },
    { 0xe9dcabac, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x47fcabae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fcabb1, SUC_lexicon_tags_6 },
    { 0x321cabb2, SUC_lexicon_tags_27 },
    { 0xe03cabb3, SUC_lexicon_tags_23 },
    { 0xf37cabb4, SUC_lexicon_tags_12 },
    { 0x77cabb4, SUC_lexicon_tags_1 },
    { 0x87dcabb3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdcabbb, SUC_lexicon_tags_63 },
    { 0x70dcabbb, SUC_lexicon_tags_15 },
    { 0x60fcabbd, SUC_lexicon_tags_14 },
    { 0x681cabbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cabc1, SUC_lexicon_tags_2 },
    { 0x155cabc1, SUC_lexicon_tags_15 },
    { 0x371cabc3, SUC_lexicon_tags_23 },
    { 0xa13cabc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x639cabc6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbcabd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x945cabd5, SUC_lexicon_tags_11 },
    { 0xe71cabd5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bcabd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x771cabdc, SUC_lexicon_tags_7 },
    { 0xeb5cabdd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbcabe0, SUC_lexicon_tags_63 },
    { 0xdf1cabe1, SUC_lexicon_tags_45 },
    { 0xd13cabe1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x375cabe4, SUC_lexicon_tags_15 },
    { 0x1dfcabe5, SUC_lexicon_tags_1 },
    { 0xa2bcabe6, SUC_lexicon_tags_39 },
    { 0x4fdcabe7, SUC_lexicon_tags_79 },
    { 0x66bcabe6, SUC_lexicon_tags_12 },
    { 0x9bbcabe5, SUC_lexicon_tags_17 },
    { 0x84dcabea, SUC_lexicon_tags_0 },
    { 0x79cabea, SUC_lexicon_tags_17 },
    { 0x239cabec, SUC_lexicon_tags_82 },
    { 0xf35cabe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fcabf5, SUC_lexicon_tags_47 },
    { 0x815cabf5, SUC_lexicon_tags_15 },
    { 0x1f3cabf6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29cabfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abcabfd, SUC_lexicon_tags_15 },
    { 0x573cabfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905cac02, SUC_lexicon_tags_11 },
    { 0xa23cac02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x383cac05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x193cac07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4ddcac09, SUC_lexicon_tags_68 },
    { 0x8c3cac0a, SUC_lexicon_tags_19 },
    { 0xa63cac0b, SUC_lexicon_tags_13 },
    { 0xb2fcac0c, SUC_lexicon_tags_2 },
    { 0x6a9cac0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dcac10, SUC_lexicon_tags_15 },
    { 0xd8fcac11, SUC_lexicon_tags_37 },
    { 0x4e5cac12, SUC_lexicon_tags_66 },
    { 0x1a3cac13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf3cac15, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7cac1a, SUC_lexicon_tags_27 },
    { 0xdb7cac1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdb5cac1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x73bcac20, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347cac24, SUC_lexicon_tags_118 },
    { 0x45fcac24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x997cac28, SUC_lexicon_tags_20 },
    { 0xda7cac29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1cac2c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bcac30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x763cac32, SUC_lexicon_tags_17 },
    { 0x425cac33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501cac3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7cac3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03cac41, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867cac45, SUC_lexicon_tags_37 },
    { 0x1dcac46, SUC_lexicon_tags_60 },
    { 0x17cac47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1cac4c, SUC_lexicon_tags_0 },
    { 0xa5fcac4d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fcac50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x789cac52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1cac55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647cac5a, SUC_lexicon_tags_16 },
    { 0x8e1cac5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x177cac5d, SUC_lexicon_tags_79 },
    { 0x76bcac5e, SUC_lexicon_tags_17 },
    { 0x4e9cac5f, SUC_lexicon_tags_61 },
    { 0xfc7cac60, SUC_lexicon_tags_60 },
    { 0x367cac61, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49dcac65, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbcac68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dcac6d, SUC_lexicon_tags_40 },
    { 0x621cac6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7cac73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afcac7b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71cac7e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09cac82, SUC_lexicon_tags_43 },
    { 0xbc7cac83, SUC_lexicon_tags_11 },
    { 0xdb3cac84, SUC_lexicon_tags_1 },
    { 0x18dcac84, SUC_lexicon_tags_17 },
    { 0xce1cac84, SUC_lexicon_tags_31 },
    { 0xa5cac87, SUC_lexicon_tags_43 },
    { 0x717cac87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1cac8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7cac90, SUC_lexicon_tags_16 },
    { 0x70fcac91, SUC_lexicon_tags_8 },
    { 0x30dcac92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9cac95, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3cac98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491cac9e, SUC_lexicon_tags_43 },
    { 0x427cac9e, SUC_lexicon_tags_2 },
    { 0x3fdcac9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x833caca2, SUC_lexicon_tags_37 },
    { 0xedbcaca2, SUC_lexicon_tags_55 },
    { 0xcd7caca4, SUC_lexicon_tags_1 },
    { 0x7e1caca5, SUC_lexicon_tags_7 },
    { 0x951caca2, SUC_lexicon_tags_17 },
    { 0xa07caca3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fcacab, SUC_lexicon_tags_7 },
    { 0xcbfcacac, SUC_lexicon_tags_1 },
    { 0xfd5cacad, SUC_lexicon_tags_87 },
    { 0xc2fcacab, SUC_lexicon_tags_14 },
    { 0x18dcacad, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05cacb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce7cacb4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcacb7, SUC_lexicon_tags_43 },
    { 0x231cacb7, SUC_lexicon_tags_1 },
    { 0x7adcacb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafcacbd, SUC_lexicon_tags_23 },
    { 0x881cacbd, SUC_lexicon_tags_43 },
    { 0x783cacbf, SUC_lexicon_tags_29 },
    { 0x5b5cacbf, SUC_lexicon_tags_45 },
    { 0x18fcacbd, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fcacc5, SUC_lexicon_tags_49 },
    { 0x879cacc6, SUC_lexicon_tags_3 },
    { 0xb91cacc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4fcacc9, SUC_lexicon_tags_17 },
    { 0x643cacca, SUC_lexicon_tags_14 },
    { 0x85caccb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x835caccd, SUC_lexicon_tags_12 },
    { 0xc3bcacce, SUC_lexicon_tags_12 },
    { 0xfcbcaccd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5cacd2, SUC_lexicon_tags_17 },
    { 0x5cacd2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fcacd9, SUC_lexicon_tags_15 },
    { 0x34fcacd9, SUC_lexicon_tags_2 },
    { 0x5fcacdb, SUC_lexicon_tags_13 },
    { 0xe29cacda, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fcace0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x34bcace3, SUC_lexicon_tags_28 },
    { 0xa81cace4, SUC_lexicon_tags_11 },
    { 0x6dcace5, SUC_lexicon_tags_46 },
    { 0xa1bcace6, SUC_lexicon_tags_12 },
    { 0xee3cace7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfcaceb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x17fcaced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x399cacef, SUC_lexicon_tags_7 },
    { 0x2afcacf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435cacf4, SUC_lexicon_tags_49 },
    { 0xc69cacf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23cacf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fcad06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7cad0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7cad0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3cad10, SUC_lexicon_tags_44 },
    { 0x4edcad11, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xaddcad13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe09cad15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x749cad17, SUC_lexicon_tags_60 },
    { 0x9c1cad17, SUC_lexicon_tags_16 },
    { 0x713cad19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7cad1d, SUC_lexicon_tags_74 },
    { 0x579cad1d, SUC_lexicon_tags_55 },
    { 0x32bcad1d, SUC_lexicon_tags_31 },
    { 0x601cad20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393cad24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5cad27, SUC_lexicon_tags_17 },
    { 0xf61cad28, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x50dcad2a, SUC_lexicon_tags_82 },
    { 0x48fcad2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5cad2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1cad33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1cad37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x871cad3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669cad3e, SUC_lexicon_tags_16 },
    { 0x979cad3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbcad42, SUC_lexicon_tags_17 },
    { 0x95cad42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe07cad45, SUC_lexicon_tags_12 },
    { 0x777cad46, SUC_lexicon_tags_11 },
    { 0x9f5cad47, SUC_lexicon_tags_1 },
    { 0x8a3cad48, SUC_lexicon_tags_0 },
    { 0xae1cad49, SUC_lexicon_tags_2 },
    { 0x35fcad4a, SUC_lexicon_tags_15 },
    { 0x393cad4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f5cad4d, SUC_lexicon_tags_33 },
    { 0x2bbcad4d, SUC_lexicon_tags_87 },
    { 0x5cbcad4f, SUC_lexicon_tags_23 },
    { 0x699cad50, SUC_lexicon_tags_9 },
    { 0xa93cad50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddcad54, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x78dcad56, SUC_lexicon_tags_11 },
    { 0xc01cad57, SUC_lexicon_tags_25 },
    { 0x9cad57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86bcad5a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb45cad5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43cad63, SUC_lexicon_tags_25 },
    { 0xc3cad64, SUC_lexicon_tags_108 },
    { 0x28bcad65, SUC_lexicon_tags_3 },
    { 0xbcbcad66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1cad6b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x543cad6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197cad72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdcad75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379cad7c, SUC_lexicon_tags_11 },
    { 0xca3cad7d, SUC_lexicon_tags_33 },
    { 0xa55cad7e, SUC_lexicon_tags_12 },
    { 0xc9cad7d, SUC_lexicon_tags_7 },
    { 0xcd3cad7c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x43cad83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bcad87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x999cad8a, SUC_lexicon_tags_37 },
    { 0x8cdcad8a, SUC_lexicon_tags_25 },
    { 0x4a5cad8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3f3cad8e, SUC_lexicon_tags_217 },
    { 0, NULL },
    { 0x5fdcad90, SUC_lexicon_tags_11 },
    { 0x38dcad91, SUC_lexicon_tags_11 },
    { 0x9edcad90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1cad9d, SUC_lexicon_tags_37 },
    { 0x455cad9e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdf9cada0, SUC_lexicon_tags_416 },
    { 0x23bcada1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d3cada3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7e7cada5, SUC_lexicon_tags_55 },
    { 0xdd7cada5, SUC_lexicon_tags_15 },
    { 0xbafcada7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x51bcada9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74dcadaf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4b7cadb1, SUC_lexicon_tags_133 },
    { 0xec9cadb2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1cadb5, SUC_lexicon_tags_38 },
    { 0x6cfcadb6, SUC_lexicon_tags_11 },
    { 0x24fcadb6, SUC_lexicon_tags_15 },
    { 0xa0dcadb8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23cadbb, SUC_lexicon_tags_8 },
    { 0x925cadbc, SUC_lexicon_tags_51 },
    { 0x477cadbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3fdcadbf, SUC_lexicon_tags_0 },
    { 0xc9dcadbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9cadc4, SUC_lexicon_tags_23 },
    { 0x455cadc4, SUC_lexicon_tags_63 },
    { 0xcfdcadc6, SUC_lexicon_tags_19 },
    { 0x3f7cadc5, SUC_lexicon_tags_0 },
    { 0xf41cadc5, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1cadcb, SUC_lexicon_tags_1 },
    { 0x56dcadcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe69cadce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1cadd7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x515cadd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9caddf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7dcade1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfcade6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xec9cade8, SUC_lexicon_tags_25 },
    { 0x405cade9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967cadef, SUC_lexicon_tags_11 },
    { 0x3ffcadf0, SUC_lexicon_tags_0 },
    { 0xffcadf1, SUC_lexicon_tags_1 },
    { 0xaa5cadf2, SUC_lexicon_tags_1 },
    { 0x74bcadf3, SUC_lexicon_tags_15 },
    { 0x2a1cadf3, SUC_lexicon_tags_2 },
    { 0xaadcadf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bcadf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235cadfd, SUC_lexicon_tags_0 },
    { 0xaa1cadfd, SUC_lexicon_tags_0 },
    { 0xe83cadff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x12dcae02, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf17cae04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffcae07, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebcae0a, SUC_lexicon_tags_43 },
    { 0xf8dcae0b, SUC_lexicon_tags_17 },
    { 0xc8bcae0b, SUC_lexicon_tags_19 },
    { 0xdb3cae0d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdcae10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddcae13, SUC_lexicon_tags_2 },
    { 0xfc1cae13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x111cae16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17cae1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7cae21, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1cae24, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69cae2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdcae30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3cae33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d5cae35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03cae41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x447cae43, SUC_lexicon_tags_66 },
    { 0x8dbcae43, SUC_lexicon_tags_11 },
    { 0x415cae45, SUC_lexicon_tags_17 },
    { 0xbe9cae46, SUC_lexicon_tags_37 },
    { 0x327cae46, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3cae4b, SUC_lexicon_tags_0 },
    { 0xfabcae4c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dcae50, SUC_lexicon_tags_37 },
    { 0xa3cae50, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5cae57, SUC_lexicon_tags_1 },
    { 0xe3cae58, SUC_lexicon_tags_14 },
    { 0x4b9cae59, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431cae5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615cae66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9cae6b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83cae6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11dcae72, SUC_lexicon_tags_23 },
    { 0xf65cae73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdc3cae75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639cae7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52dcae7d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fcae81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x723cae83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x961cae86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67cae8a, SUC_lexicon_tags_47 },
    { 0x833cae8b, SUC_lexicon_tags_63 },
    { 0xf1dcae8c, SUC_lexicon_tags_1 },
    { 0xc31cae8d, SUC_lexicon_tags_1 },
    { 0x6c9cae8e, SUC_lexicon_tags_30 },
    { 0x969cae8f, SUC_lexicon_tags_7 },
    { 0xd19cae8b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31cae94, SUC_lexicon_tags_27 },
    { 0xf2bcae95, SUC_lexicon_tags_60 },
    { 0x3d1cae96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dcae99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca3cae9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa81cae9d, SUC_lexicon_tags_25 },
    { 0x4b5cae9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4bcaea0, SUC_lexicon_tags_17 },
    { 0x5bbcaea0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x491caea4, SUC_lexicon_tags_1 },
    { 0x655caea5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x707caea8, SUC_lexicon_tags_9 },
    { 0x551caea9, SUC_lexicon_tags_60 },
    { 0x3dbcaeaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfcaeb0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf4dcaeb2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fcaeb6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55caebe, SUC_lexicon_tags_17 },
    { 0x5adcaebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x109caec1, SUC_lexicon_tags_38 },
    { 0x231caec1, SUC_lexicon_tags_31 },
    { 0xdb3caec3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d7caec5, SUC_lexicon_tags_43 },
    { 0x757caec6, SUC_lexicon_tags_129 },
    { 0x2a9caec7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf8bcaec9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bcaecc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7dcaece, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dcaed6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd3dcaed8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55caedc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3caee4, SUC_lexicon_tags_7 },
    { 0x417caee5, SUC_lexicon_tags_11 },
    { 0x661caee6, SUC_lexicon_tags_8 },
    { 0xb53caee7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bcaeea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4dcaeec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d7caeee, SUC_lexicon_tags_47 },
    { 0x53fcaeef, SUC_lexicon_tags_28 },
    { 0x6adcaeee, SUC_lexicon_tags_11 },
    { 0xe99caef1, SUC_lexicon_tags_0 },
    { 0x47caef2, SUC_lexicon_tags_65 },
    { 0xe85caef2, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23caef6, SUC_lexicon_tags_62 },
    { 0x71bcaef7, SUC_lexicon_tags_38 },
    { 0x789caef8, SUC_lexicon_tags_92 },
    { 0x37caef8, SUC_lexicon_tags_17 },
    { 0xe2bcaefa, SUC_lexicon_tags_15 },
    { 0x879caefb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfefcaefd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9caf01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2b3caf03, SUC_lexicon_tags_21 },
    { 0xa31caf03, SUC_lexicon_tags_12 },
    { 0xcdbcaf03, SUC_lexicon_tags_37 },
    { 0x29dcaf04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821caf0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x72fcaf0c, SUC_lexicon_tags_47 },
    { 0x5d7caf0c, SUC_lexicon_tags_68 },
    { 0x755caf0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bcaf13, SUC_lexicon_tags_37 },
    { 0xca1caf14, SUC_lexicon_tags_2 },
    { 0xeebcaf15, SUC_lexicon_tags_19 },
    { 0xcc1caf15, SUC_lexicon_tags_2 },
    { 0xbcdcaf17, SUC_lexicon_tags_2 },
    { 0x2b5caf18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcf7caf1a, SUC_lexicon_tags_7 },
    { 0x1c7caf1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadcaf1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29caf22, SUC_lexicon_tags_7 },
    { 0xb61caf22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dcaf27, SUC_lexicon_tags_12 },
    { 0xc5dcaf27, SUC_lexicon_tags_60 },
    { 0x24dcaf27, SUC_lexicon_tags_6 },
    { 0xeb7caf29, SUC_lexicon_tags_37 },
    { 0xfffcaf2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd87caf2d, SUC_lexicon_tags_11 },
    { 0xacfcaf2e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dcaf33, SUC_lexicon_tags_27 },
    { 0xd13caf33, SUC_lexicon_tags_17 },
    { 0x719caf35, SUC_lexicon_tags_1 },
    { 0xcedcaf33, SUC_lexicon_tags_32 },
    { 0x72fcaf34, SUC_lexicon_tags_68 },
    { 0xd47caf38, SUC_lexicon_tags_28 },
    { 0x6efcaf39, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bcaf3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843caf46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dcaf52, SUC_lexicon_tags_19 },
    { 0x11caf53, SUC_lexicon_tags_25 },
    { 0xecbcaf52, SUC_lexicon_tags_37 },
    { 0x707caf55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99caf57, SUC_lexicon_tags_1 },
    { 0x775caf58, SUC_lexicon_tags_47 },
    { 0xfb7caf59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfcaf5d, SUC_lexicon_tags_25 },
    { 0x8efcaf5e, SUC_lexicon_tags_46 },
    { 0x34bcaf5f, SUC_lexicon_tags_28 },
    { 0x2ffcaf60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x309caf62, SUC_lexicon_tags_11 },
    { 0x49fcaf62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa95caf65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba1caf67, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edcaf6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x66dcaf6d, SUC_lexicon_tags_61 },
    { 0x8f1caf6e, SUC_lexicon_tags_11 },
    { 0xfefcaf6e, SUC_lexicon_tags_26 },
    { 0xb39caf6e, SUC_lexicon_tags_9 },
    { 0x363caf71, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x755caf74, SUC_lexicon_tags_25 },
    { 0xec1caf74, SUC_lexicon_tags_37 },
    { 0x27dcaf76, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2bfcaf78, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7caf7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf39caf7e, SUC_lexicon_tags_14 },
    { 0x57fcaf7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dcaf83, SUC_lexicon_tags_34 },
    { 0x6cfcaf84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9caf89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bbcaf8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x511caf8e, SUC_lexicon_tags_33 },
    { 0x9fcaf8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcb5caf91, SUC_lexicon_tags_60 },
    { 0x31caf92, SUC_lexicon_tags_19 },
    { 0xb51caf91, SUC_lexicon_tags_40 },
    { 0x215caf94, SUC_lexicon_tags_37 },
    { 0x43fcaf94, SUC_lexicon_tags_25 },
    { 0x2a5caf96, SUC_lexicon_tags_27 },
    { 0x153caf94, SUC_lexicon_tags_22 },
    { 0xb33caf98, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3a5caf9a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bcafa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9cafa5, SUC_lexicon_tags_11 },
    { 0xa65cafa6, SUC_lexicon_tags_2 },
    { 0xa83cafa7, SUC_lexicon_tags_27 },
    { 0x7c1cafa7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cafb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dcafb4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fcafbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x95dcafbd, SUC_lexicon_tags_15 },
    { 0x8adcafbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2abcafc0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3cafc3, SUC_lexicon_tags_1 },
    { 0xd87cafc4, SUC_lexicon_tags_37 },
    { 0x8d1cafc5, SUC_lexicon_tags_19 },
    { 0xb11cafc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01cafcd, SUC_lexicon_tags_47 },
    { 0x623cafce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa47cafd0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x43dcafd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fdcafd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fcafd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9d5cafda, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcb1cafdc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefcafe0, SUC_lexicon_tags_15 },
    { 0xa91cafe1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bcafe6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1cafea, SUC_lexicon_tags_14 },
    { 0x533cafeb, SUC_lexicon_tags_45 },
    { 0x97cafec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffcaff0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdcaff3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fcaff7, SUC_lexicon_tags_7 },
    { 0x241caff8, SUC_lexicon_tags_0 },
    { 0x525caff9, SUC_lexicon_tags_17 },
    { 0xb6fcaff9, SUC_lexicon_tags_14 },
    { 0x633caff7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927cafff, SUC_lexicon_tags_29 },
    { 0xd25cafff, SUC_lexicon_tags_1 },
    { 0xf4bcb001, SUC_lexicon_tags_61 },
    { 0xcbcb001, SUC_lexicon_tags_12 },
    { 0x35fcafff, SUC_lexicon_tags_12 },
    { 0xc21cb000, SUC_lexicon_tags_2 },
    { 0xdfcb005, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cb011, SUC_lexicon_tags_100 },
    { 0xca9cb012, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5cb018, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9cb01b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x84dcb01e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x189cb020, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7cb023, SUC_lexicon_tags_63 },
    { 0xb23cb023, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x905cb027, SUC_lexicon_tags_18 },
    { 0x59bcb027, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x541cb02b, SUC_lexicon_tags_25 },
    { 0x73cb02b, SUC_lexicon_tags_37 },
    { 0xeb3cb02d, SUC_lexicon_tags_34 },
    { 0x955cb02d, SUC_lexicon_tags_15 },
    { 0xe3cb02f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7c1cb031, SUC_lexicon_tags_46 },
    { 0x457cb032, SUC_lexicon_tags_19 },
    { 0x9c5cb033, SUC_lexicon_tags_15 },
    { 0x191cb033, SUC_lexicon_tags_234 },
    { 0x6ebcb034, SUC_lexicon_tags_20 },
    { 0x3d5cb036, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc79cb038, SUC_lexicon_tags_33 },
    { 0xc97cb039, SUC_lexicon_tags_17 },
    { 0xb77cb03a, SUC_lexicon_tags_12 },
    { 0xde7cb03b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9cb041, SUC_lexicon_tags_11 },
    { 0x469cb042, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9cb045, SUC_lexicon_tags_46 },
    { 0x7cb046, SUC_lexicon_tags_14 },
    { 0x673cb047, SUC_lexicon_tags_15 },
    { 0x1d5cb048, SUC_lexicon_tags_7 },
    { 0x333cb047, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9cb05b, SUC_lexicon_tags_30 },
    { 0xa15cb05b, SUC_lexicon_tags_2 },
    { 0x53dcb05d, SUC_lexicon_tags_17 },
    { 0x515cb05e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dcb062, SUC_lexicon_tags_11 },
    { 0x9cbcb063, SUC_lexicon_tags_37 },
    { 0x375cb063, SUC_lexicon_tags_15 },
    { 0x7c3cb063, SUC_lexicon_tags_62 },
    { 0x639cb064, SUC_lexicon_tags_43 },
    { 0x489cb063, SUC_lexicon_tags_12 },
    { 0xb95cb062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edcb06b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fcb070, SUC_lexicon_tags_1 },
    { 0xc7fcb071, SUC_lexicon_tags_26 },
    { 0xc11cb072, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x195cb074, SUC_lexicon_tags_18 },
    { 0x57fcb075, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xd3bcb077, SUC_lexicon_tags_33 },
    { 0x567cb078, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45cb07c, SUC_lexicon_tags_1 },
    { 0xa41cb07c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1fcb07f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edcb08a, SUC_lexicon_tags_37 },
    { 0x4afcb08b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cb08f, SUC_lexicon_tags_26 },
    { 0x305cb090, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb093, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91dcb095, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd6fcb097, SUC_lexicon_tags_77 },
    { 0x691cb098, SUC_lexicon_tags_49 },
    { 0x663cb099, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fcb0a2, SUC_lexicon_tags_37 },
    { 0x463cb0a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x58fcb0a6, SUC_lexicon_tags_9 },
    { 0xcebcb0a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdcb0aa, SUC_lexicon_tags_37 },
    { 0x6a5cb0ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9cb0b1, SUC_lexicon_tags_15 },
    { 0xf8dcb0b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfcb0c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11cb0c3, SUC_lexicon_tags_2 },
    { 0x355cb0c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21cb0c7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757cb0d0, SUC_lexicon_tags_25 },
    { 0x4c5cb0d1, SUC_lexicon_tags_27 },
    { 0xfd9cb0d0, SUC_lexicon_tags_9 },
    { 0x1f5cb0d1, SUC_lexicon_tags_213 },
    { 0x459cb0d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97cb0d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21cb0da, SUC_lexicon_tags_77 },
    { 0xdedcb0db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e9cb0dd, SUC_lexicon_tags_5 },
    { 0xd2bcb0dd, SUC_lexicon_tags_37 },
    { 0x777cb0df, SUC_lexicon_tags_42 },
    { 0xd6bcb0de, SUC_lexicon_tags_37 },
    { 0x4adcb0df, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47cb0e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x641cb0ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1cb0ed, SUC_lexicon_tags_47 },
    { 0x7dcb0ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefcb0f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87cb0f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5cb0fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x229cb0fd, SUC_lexicon_tags_31 },
    { 0x205cb0fd, SUC_lexicon_tags_29 },
    { 0xaa7cb0ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x437cb102, SUC_lexicon_tags_4 },
    { 0xc8bcb103, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7cb105, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e1cb107, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bcb10b, SUC_lexicon_tags_25 },
    { 0xb3cb10c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51cb112, SUC_lexicon_tags_23 },
    { 0x81bcb113, SUC_lexicon_tags_25 },
    { 0xb11cb114, SUC_lexicon_tags_117 },
    { 0x98dcb115, SUC_lexicon_tags_16 },
    { 0x1ffcb114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5fcb118, SUC_lexicon_tags_38 },
    { 0xf33cb119, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785cb121, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbd9cb123, SUC_lexicon_tags_57 },
    { 0x9dcb124, SUC_lexicon_tags_87 },
    { 0x1c9cb124, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa59cb127, SUC_lexicon_tags_11 },
    { 0x7f3cb128, SUC_lexicon_tags_14 },
    { 0x95cb129, SUC_lexicon_tags_12 },
    { 0xd43cb129, SUC_lexicon_tags_11 },
    { 0xa8bcb129, SUC_lexicon_tags_30 },
    { 0x883cb12c, SUC_lexicon_tags_38 },
    { 0x5d7cb12c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x955cb130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x38dcb133, SUC_lexicon_tags_38 },
    { 0xafbcb133, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3cb138, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817cb13e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb29cb140, SUC_lexicon_tags_60 },
    { 0x6e5cb140, SUC_lexicon_tags_47 },
    { 0x263cb142, SUC_lexicon_tags_0 },
    { 0xd29cb141, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fcb146, SUC_lexicon_tags_12 },
    { 0x40fcb147, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cb14c, SUC_lexicon_tags_29 },
    { 0x1f5cb14d, SUC_lexicon_tags_30 },
    { 0x37dcb14d, SUC_lexicon_tags_37 },
    { 0x517cb14d, SUC_lexicon_tags_57 },
    { 0x28bcb150, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bcb155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7fcb157, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x545cb159, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdcb15d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49cb162, SUC_lexicon_tags_2 },
    { 0x5dfcb162, SUC_lexicon_tags_12 },
    { 0xd95cb164, SUC_lexicon_tags_37 },
    { 0x865cb162, SUC_lexicon_tags_0 },
    { 0x427cb166, SUC_lexicon_tags_414 },
    { 0x907cb164, SUC_lexicon_tags_15 },
    { 0x9ffcb168, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x287cb16a, SUC_lexicon_tags_11 },
    { 0x523cb16b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93cb16f, SUC_lexicon_tags_10 },
    { 0xfebcb170, SUC_lexicon_tags_0 },
    { 0x9e7cb171, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355cb176, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389cb17d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bcb182, SUC_lexicon_tags_1 },
    { 0xa7bcb183, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fcb187, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe3fcb189, SUC_lexicon_tags_7 },
    { 0x879cb18a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x807cb18d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd3cb18f, SUC_lexicon_tags_3 },
    { 0x593cb18f, SUC_lexicon_tags_38 },
    { 0xffcb18f, SUC_lexicon_tags_63 },
    { 0x74dcb192, SUC_lexicon_tags_85 },
    { 0xbcbcb193, SUC_lexicon_tags_26 },
    { 0x6b1cb191, SUC_lexicon_tags_43 },
    { 0x325cb195, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5cb19e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x463cb1a0, SUC_lexicon_tags_37 },
    { 0xb3cb1a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x461cb1a4, SUC_lexicon_tags_33 },
    { 0x8d5cb1a5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77cb1a8, SUC_lexicon_tags_14 },
    { 0x369cb1a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dcb1ae, SUC_lexicon_tags_57 },
    { 0x19dcb1af, SUC_lexicon_tags_15 },
    { 0x7f3cb1b0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x64dcb1b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc5cb1b4, SUC_lexicon_tags_25 },
    { 0xc45cb1b5, SUC_lexicon_tags_25 },
    { 0xa11cb1b6, SUC_lexicon_tags_12 },
    { 0x995cb1b7, SUC_lexicon_tags_35 },
    { 0xa9dcb1b8, SUC_lexicon_tags_11 },
    { 0x225cb1b9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x729cb1bc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xec9cb1bf, SUC_lexicon_tags_57 },
    { 0x17fcb1c0, SUC_lexicon_tags_42 },
    { 0x369cb1c1, SUC_lexicon_tags_10 },
    { 0x88dcb1c2, SUC_lexicon_tags_39 },
    { 0x7fbcb1c3, SUC_lexicon_tags_12 },
    { 0x2b9cb1c2, SUC_lexicon_tags_7 },
    { 0x765cb1c5, SUC_lexicon_tags_35 },
    { 0xd6bcb1c5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbcb1c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x69dcb1cb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993cb1d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc1fcb1d3, SUC_lexicon_tags_38 },
    { 0x10dcb1d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfcb1da, SUC_lexicon_tags_28 },
    { 0x7edcb1db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bcb1de, SUC_lexicon_tags_17 },
    { 0x943cb1df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x239cb1e1, SUC_lexicon_tags_25 },
    { 0x213cb1e2, SUC_lexicon_tags_28 },
    { 0xf15cb1e1, SUC_lexicon_tags_15 },
    { 0xaafcb1e4, SUC_lexicon_tags_1 },
    { 0x377cb1e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3cb1e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22bcb1ea, SUC_lexicon_tags_17 },
    { 0x4dcb1eb, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dcb1ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9cb1f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfcb1f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e3cb1fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fcb1fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf63cb1ff, SUC_lexicon_tags_62 },
    { 0x8afcb200, SUC_lexicon_tags_98 },
    { 0x987cb201, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615cb205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cb20a, SUC_lexicon_tags_2 },
    { 0x7f7cb20b, SUC_lexicon_tags_39 },
    { 0x73dcb20b, SUC_lexicon_tags_7 },
    { 0xbd1cb20d, SUC_lexicon_tags_37 },
    { 0xb05cb20c, SUC_lexicon_tags_14 },
    { 0x529cb20b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dcb213, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x715cb215, SUC_lexicon_tags_29 },
    { 0x161cb216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9cb21b, SUC_lexicon_tags_25 },
    { 0x623cb21c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1dcb21e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x975cb220, SUC_lexicon_tags_1 },
    { 0x299cb221, SUC_lexicon_tags_11 },
    { 0x7f7cb222, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbcb227, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48bcb229, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfcb22e, SUC_lexicon_tags_49 },
    { 0x61dcb22f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451cb234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11cb237, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe9cb239, SUC_lexicon_tags_12 },
    { 0x6b9cb23a, SUC_lexicon_tags_2 },
    { 0xa0bcb23b, SUC_lexicon_tags_46 },
    { 0x18fcb23c, SUC_lexicon_tags_27 },
    { 0x90bcb23d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa99cb23f, SUC_lexicon_tags_1019 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485cb244, SUC_lexicon_tags_17 },
    { 0x711cb245, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7cb248, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed9cb24a, SUC_lexicon_tags_37 },
    { 0x8d5cb24b, SUC_lexicon_tags_17 },
    { 0xd47cb24b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x603cb24e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c5cb250, SUC_lexicon_tags_4 },
    { 0xab3cb251, SUC_lexicon_tags_117 },
    { 0xe6fcb252, SUC_lexicon_tags_29 },
    { 0x7a1cb250, SUC_lexicon_tags_4 },
    { 0x7c3cb253, SUC_lexicon_tags_66 },
    { 0xef9cb250, SUC_lexicon_tags_21 },
    { 0x2f5cb256, SUC_lexicon_tags_1 },
    { 0x4c3cb252, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd05cb259, SUC_lexicon_tags_25 },
    { 0x4fcb259, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x263cb25d, SUC_lexicon_tags_2 },
    { 0xddbcb25e, SUC_lexicon_tags_57 },
    { 0x357cb25f, SUC_lexicon_tags_14 },
    { 0x6ddcb260, SUC_lexicon_tags_11 },
    { 0xaa1cb260, SUC_lexicon_tags_20 },
    { 0xf33cb262, SUC_lexicon_tags_25 },
    { 0x403cb263, SUC_lexicon_tags_14 },
    { 0x5d1cb262, SUC_lexicon_tags_0 },
    { 0x7efcb263, SUC_lexicon_tags_2 },
    { 0xbe3cb263, SUC_lexicon_tags_30 },
    { 0x5c9cb266, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdcb26a, SUC_lexicon_tags_1 },
    { 0x5e3cb26a, SUC_lexicon_tags_0 },
    { 0xd4dcb26c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x495cb26e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcb273, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5cb27a, SUC_lexicon_tags_11 },
    { 0xb8fcb27b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x66bcb27d, SUC_lexicon_tags_27 },
    { 0x753cb27e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd43cb280, SUC_lexicon_tags_27 },
    { 0x96fcb280, SUC_lexicon_tags_48 },
    { 0x8ebcb282, SUC_lexicon_tags_27 },
    { 0xd61cb283, SUC_lexicon_tags_47 },
    { 0x30dcb284, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6edcb286, SUC_lexicon_tags_37 },
    { 0x919cb286, SUC_lexicon_tags_25 },
    { 0xd4fcb288, SUC_lexicon_tags_124 },
    { 0xf51cb288, SUC_lexicon_tags_16 },
    { 0x8d3cb288, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefcb292, SUC_lexicon_tags_23 },
    { 0x713cb293, SUC_lexicon_tags_12 },
    { 0x48bcb292, SUC_lexicon_tags_29 },
    { 0xcb9cb295, SUC_lexicon_tags_10 },
    { 0xea9cb296, SUC_lexicon_tags_25 },
    { 0x775cb294, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7a3cb299, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bcb2a0, SUC_lexicon_tags_7 },
    { 0xd57cb2a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3cb2a5, SUC_lexicon_tags_12 },
    { 0x6c1cb2a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcddcb2a8, SUC_lexicon_tags_12 },
    { 0x75fcb2a9, SUC_lexicon_tags_15 },
    { 0xebdcb2aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfcb2ad, SUC_lexicon_tags_16 },
    { 0x677cb2ae, SUC_lexicon_tags_60 },
    { 0xdbcb2ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3cb2b2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77cb2b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe37cb2b7, SUC_lexicon_tags_15 },
    { 0x563cb2b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9cb2bc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdcb2bf, SUC_lexicon_tags_101 },
    { 0xc43cb2bf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dcb2c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413cb2ce, SUC_lexicon_tags_1 },
    { 0xf35cb2ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x707cb2d1, SUC_lexicon_tags_34 },
    { 0x88bcb2d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61cb2dc, SUC_lexicon_tags_15 },
    { 0xa1bcb2dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789cb2e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bcb2e6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547cb2ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229cb2f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffcb2f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x429cb2fb, SUC_lexicon_tags_11 },
    { 0xffdcb2fb, SUC_lexicon_tags_9 },
    { 0x32bcb2fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x849cb300, SUC_lexicon_tags_19 },
    { 0x921cb301, SUC_lexicon_tags_25 },
    { 0xf9cb301, SUC_lexicon_tags_7 },
    { 0xa87cb300, SUC_lexicon_tags_37 },
    { 0xe4dcb304, SUC_lexicon_tags_21 },
    { 0xc73cb305, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefcb30c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99cb316, SUC_lexicon_tags_68 },
    { 0x9c1cb316, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717cb31b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83cb31f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x98dcb321, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dcb329, SUC_lexicon_tags_28 },
    { 0x479cb32a, SUC_lexicon_tags_10 },
    { 0x73fcb32a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167cb32f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e7cb331, SUC_lexicon_tags_0 },
    { 0xeb5cb332, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bcb339, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3cb33c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfbbcb33e, SUC_lexicon_tags_25 },
    { 0xc55cb33f, SUC_lexicon_tags_12 },
    { 0xf7dcb33f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x805cb342, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebcb345, SUC_lexicon_tags_29 },
    { 0x56fcb346, SUC_lexicon_tags_0 },
    { 0x4b1cb347, SUC_lexicon_tags_37 },
    { 0x907cb347, SUC_lexicon_tags_17 },
    { 0x65fcb347, SUC_lexicon_tags_45 },
    { 0xfabcb348, SUC_lexicon_tags_17 },
    { 0xcb3cb346, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dfcb34d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74fcb34f, SUC_lexicon_tags_25 },
    { 0x35dcb350, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bcb353, SUC_lexicon_tags_7 },
    { 0xd15cb354, SUC_lexicon_tags_25 },
    { 0x51cb354, SUC_lexicon_tags_12 },
    { 0x9fcb354, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493cb35c, SUC_lexicon_tags_23 },
    { 0xa6bcb35d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735cb361, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605cb36a, SUC_lexicon_tags_11 },
    { 0xa95cb36b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87cb371, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x431cb373, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39cb376, SUC_lexicon_tags_25 },
    { 0x979cb376, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbcb37c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fcb37f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87cb383, SUC_lexicon_tags_14 },
    { 0xcf9cb384, SUC_lexicon_tags_16 },
    { 0x523cb385, SUC_lexicon_tags_30 },
    { 0xf95cb386, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xb49cb389, SUC_lexicon_tags_37 },
    { 0x1efcb389, SUC_lexicon_tags_17 },
    { 0xd15cb38b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4c5cb38d, SUC_lexicon_tags_24 },
    { 0x1d5cb38e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05cb393, SUC_lexicon_tags_43 },
    { 0x641cb394, SUC_lexicon_tags_1 },
    { 0x5dcb394, SUC_lexicon_tags_13 },
    { 0x271cb395, SUC_lexicon_tags_7 },
    { 0xd6dcb397, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa31cb399, SUC_lexicon_tags_11 },
    { 0x497cb39a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeebcb39c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fcb3a3, SUC_lexicon_tags_4 },
    { 0xe4bcb3a3, SUC_lexicon_tags_25 },
    { 0x123cb3a5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x22bcb3a7, SUC_lexicon_tags_7 },
    { 0x209cb3a8, SUC_lexicon_tags_17 },
    { 0x105cb3a7, SUC_lexicon_tags_3 },
    { 0xc2dcb3aa, SUC_lexicon_tags_19 },
    { 0x423cb3ab, SUC_lexicon_tags_15 },
    { 0xe6fcb3ac, SUC_lexicon_tags_25 },
    { 0x223cb3ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fcb3b0, SUC_lexicon_tags_7 },
    { 0x2cdcb3b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129cb3b6, SUC_lexicon_tags_11 },
    { 0x481cb3b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7cb3ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3d5cb3bc, SUC_lexicon_tags_10 },
    { 0x489cb3bd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71cb3c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839cb3c6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x897cb3c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9cb3cf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x4e5cb3d1, SUC_lexicon_tags_11 },
    { 0xd3bcb3d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9cb3d6, SUC_lexicon_tags_37 },
    { 0x243cb3d7, SUC_lexicon_tags_43 },
    { 0x8d5cb3d7, SUC_lexicon_tags_25 },
    { 0xa07cb3d9, SUC_lexicon_tags_27 },
    { 0x8d3cb3da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xefdcb3dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfcb3e1, SUC_lexicon_tags_30 },
    { 0x3c7cb3e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d7cb3e4, SUC_lexicon_tags_39 },
    { 0xecbcb3e5, SUC_lexicon_tags_0 },
    { 0x389cb3e5, SUC_lexicon_tags_27 },
    { 0xce3cb3e7, SUC_lexicon_tags_2 },
    { 0x375cb3e8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x43dcb3ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cb3ef, SUC_lexicon_tags_1 },
    { 0x54bcb3f0, SUC_lexicon_tags_31 },
    { 0x80dcb3f1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dcb3f8, SUC_lexicon_tags_12 },
    { 0xe93cb3f9, SUC_lexicon_tags_11 },
    { 0x359cb3fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99bcb3fc, SUC_lexicon_tags_64 },
    { 0x7dbcb3fc, SUC_lexicon_tags_77 },
    { 0x7d9cb3fd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdcb401, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbcb404, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3efcb406, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19cb40a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdcb412, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x709cb415, SUC_lexicon_tags_12 },
    { 0x2a3cb416, SUC_lexicon_tags_27 },
    { 0x549cb417, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3cb41c, SUC_lexicon_tags_11 },
    { 0x311cb41d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483cb421, SUC_lexicon_tags_15 },
    { 0x68dcb421, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bcb426, SUC_lexicon_tags_12 },
    { 0x35dcb426, SUC_lexicon_tags_17 },
    { 0x23dcb426, SUC_lexicon_tags_17 },
    { 0xcdbcb428, SUC_lexicon_tags_30 },
    { 0xb21cb428, SUC_lexicon_tags_11 },
    { 0x973cb429, SUC_lexicon_tags_67 },
    { 0x90dcb427, SUC_lexicon_tags_9 },
    { 0x9a3cb42d, SUC_lexicon_tags_11 },
    { 0x397cb42b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71cb432, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x967cb435, SUC_lexicon_tags_37 },
    { 0xdefcb436, SUC_lexicon_tags_0 },
    { 0xe4bcb435, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129cb43b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dfcb43d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fcb442, SUC_lexicon_tags_45 },
    { 0x3d3cb443, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9cb44c, SUC_lexicon_tags_17 },
    { 0xbafcb44d, SUC_lexicon_tags_33 },
    { 0xefbcb44d, SUC_lexicon_tags_1 },
    { 0xed5cb44f, SUC_lexicon_tags_87 },
    { 0x431cb450, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91cb454, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fdcb456, SUC_lexicon_tags_19 },
    { 0x555cb457, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe1dcb459, SUC_lexicon_tags_0 },
    { 0x9ddcb45a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9cb45d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309cb462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed9cb464, SUC_lexicon_tags_37 },
    { 0x983cb464, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97cb46d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4d3cb46f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bcb476, SUC_lexicon_tags_29 },
    { 0x277cb477, SUC_lexicon_tags_12 },
    { 0xb1cb476, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xba7cb47a, SUC_lexicon_tags_15 },
    { 0xd0dcb47a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69cb47e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fcb481, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cb484, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8ffcb486, SUC_lexicon_tags_0 },
    { 0x327cb487, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x763cb48a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1fcb48c, SUC_lexicon_tags_14 },
    { 0xcedcb48d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe31cb48f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91cb495, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bcb498, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61cb49e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19dcb4a3, SUC_lexicon_tags_27 },
    { 0x147cb4a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x379cb4a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfd5cb4a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x443cb4ab, SUC_lexicon_tags_17 },
    { 0x5fbcb4ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cb4b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfcb4b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fcb4ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x519cb4bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7d7cb4be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29cb4c3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abcb4cb, SUC_lexicon_tags_39 },
    { 0x911cb4cc, SUC_lexicon_tags_37 },
    { 0xb2bcb4cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955cb4d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x977cb4da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf65cb4dc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141cb4e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5bcb4e4, SUC_lexicon_tags_11 },
    { 0x3e9cb4e5, SUC_lexicon_tags_79 },
    { 0x48fcb4e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33dcb4f5, SUC_lexicon_tags_7 },
    { 0xe6bcb4f6, SUC_lexicon_tags_27 },
    { 0x71cb4f7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x475cb4fa, SUC_lexicon_tags_17 },
    { 0x64dcb4fb, SUC_lexicon_tags_30 },
    { 0xe5bcb4fc, SUC_lexicon_tags_47 },
    { 0xaa7cb4fd, SUC_lexicon_tags_17 },
    { 0x265cb4fd, SUC_lexicon_tags_6 },
    { 0xcd1cb4ff, SUC_lexicon_tags_72 },
    { 0xacdcb500, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8adcb503, SUC_lexicon_tags_14 },
    { 0xed5cb503, SUC_lexicon_tags_27 },
    { 0xe87cb505, SUC_lexicon_tags_87 },
    { 0x7e7cb504, SUC_lexicon_tags_12 },
    { 0xc5cb507, SUC_lexicon_tags_15 },
    { 0xe9cb503, SUC_lexicon_tags_10 },
    { 0xadfcb509, SUC_lexicon_tags_7 },
    { 0xb11cb506, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903cb517, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79cb51a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x75dcb51c, SUC_lexicon_tags_12 },
    { 0x881cb51c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebcb526, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fcb52e, SUC_lexicon_tags_37 },
    { 0x349cb52f, SUC_lexicon_tags_48 },
    { 0x1abcb530, SUC_lexicon_tags_8 },
    { 0xf49cb531, SUC_lexicon_tags_60 },
    { 0x28dcb531, SUC_lexicon_tags_63 },
    { 0x9c5cb533, SUC_lexicon_tags_25 },
    { 0x445cb534, SUC_lexicon_tags_17 },
    { 0x2dbcb535, SUC_lexicon_tags_2 },
    { 0xa13cb535, SUC_lexicon_tags_45 },
    { 0xefbcb537, SUC_lexicon_tags_40 },
    { 0x61cb536, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3cb53c, SUC_lexicon_tags_29 },
    { 0x1afcb53c, SUC_lexicon_tags_108 },
    { 0x4efcb53e, SUC_lexicon_tags_4 },
    { 0xc09cb53f, SUC_lexicon_tags_11 },
    { 0xfa3cb53f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x587cb542, SUC_lexicon_tags_7 },
    { 0xc73cb542, SUC_lexicon_tags_10 },
    { 0xbc9cb542, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735cb54b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe8dcb54d, SUC_lexicon_tags_551 },
    { 0x101cb54e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771cb556, SUC_lexicon_tags_7 },
    { 0x33fcb556, SUC_lexicon_tags_9 },
    { 0xc93cb558, SUC_lexicon_tags_14 },
    { 0xb3cb557, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcb55d, SUC_lexicon_tags_33 },
    { 0x7cfcb55d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7cb567, SUC_lexicon_tags_17 },
    { 0x605cb568, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03cb56c, SUC_lexicon_tags_37 },
    { 0xbdbcb56d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3cb571, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x473cb574, SUC_lexicon_tags_15 },
    { 0xa63cb575, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35cb578, SUC_lexicon_tags_25 },
    { 0x9b1cb579, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91cb57d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbbbcb57f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fcb58c, SUC_lexicon_tags_1 },
    { 0xb53cb58c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x47dcb58f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15cb595, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bcb598, SUC_lexicon_tags_18 },
    { 0xe7fcb599, SUC_lexicon_tags_3 },
    { 0x9ebcb59a, SUC_lexicon_tags_7 },
    { 0x2efcb59b, SUC_lexicon_tags_17 },
    { 0xbbbcb59c, SUC_lexicon_tags_37 },
    { 0x555cb59d, SUC_lexicon_tags_18 },
    { 0x9fbcb59e, SUC_lexicon_tags_11 },
    { 0xbf3cb59f, SUC_lexicon_tags_2 },
    { 0x1cfcb5a0, SUC_lexicon_tags_11 },
    { 0xcefcb5a0, SUC_lexicon_tags_1 },
    { 0x653cb59b, SUC_lexicon_tags_1 },
    { 0xca9cb5a3, SUC_lexicon_tags_25 },
    { 0x509cb59d, SUC_lexicon_tags_23 },
    { 0x97cb5a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91cb5ad, SUC_lexicon_tags_7 },
    { 0x289cb5ae, SUC_lexicon_tags_11 },
    { 0x15bcb5ad, SUC_lexicon_tags_63 },
    { 0xfdcb5b0, SUC_lexicon_tags_7 },
    { 0x113cb5b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429cb5b5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2efcb5b7, SUC_lexicon_tags_11 },
    { 0xfc7cb5b8, SUC_lexicon_tags_49 },
    { 0x7c1cb5b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa7bcb5bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15cb5be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941cb5cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc95cb5cd, SUC_lexicon_tags_37 },
    { 0x89dcb5cd, SUC_lexicon_tags_87 },
    { 0x115cb5cf, SUC_lexicon_tags_12 },
    { 0x3a3cb5d0, SUC_lexicon_tags_31 },
    { 0xbfbcb5d1, SUC_lexicon_tags_6 },
    { 0x837cb5d2, SUC_lexicon_tags_32 },
    { 0x6b7cb5d1, SUC_lexicon_tags_37 },
    { 0xb9fcb5d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201cb5da, SUC_lexicon_tags_25 },
    { 0x973cb5db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3abcb5dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501cb5e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x265cb5e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fcb5e8, SUC_lexicon_tags_15 },
    { 0x24dcb5e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9cb5eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x491cb5ee, SUC_lexicon_tags_796 },
    { 0, NULL },
    { 0xc9dcb5f0, SUC_lexicon_tags_43 },
    { 0xebdcb5f1, SUC_lexicon_tags_0 },
    { 0xbfbcb5f2, SUC_lexicon_tags_28 },
    { 0x4ebcb5f2, SUC_lexicon_tags_29 },
    { 0x5bbcb5f4, SUC_lexicon_tags_60 },
    { 0x4b3cb5f5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x199cb5f7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd57cb5f9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x519cb5fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x72dcb5fe, SUC_lexicon_tags_33 },
    { 0x3f1cb5ff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fcb60c, SUC_lexicon_tags_17 },
    { 0x95dcb60c, SUC_lexicon_tags_31 },
    { 0xf55cb60e, SUC_lexicon_tags_11 },
    { 0x4c9cb60f, SUC_lexicon_tags_19 },
    { 0xc1cb60f, SUC_lexicon_tags_37 },
    { 0x5a5cb610, SUC_lexicon_tags_4 },
    { 0xee1cb612, SUC_lexicon_tags_17 },
    { 0xa35cb613, SUC_lexicon_tags_63 },
    { 0x6c1cb60e, SUC_lexicon_tags_12 },
    { 0x1f1cb615, SUC_lexicon_tags_25 },
    { 0xe1dcb612, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1dcb618, SUC_lexicon_tags_49 },
    { 0xa73cb618, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abcb620, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97cb623, SUC_lexicon_tags_62 },
    { 0x2e3cb623, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7f3cb626, SUC_lexicon_tags_65 },
    { 0xd69cb626, SUC_lexicon_tags_12 },
    { 0x831cb626, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaa3cb62a, SUC_lexicon_tags_37 },
    { 0x1c7cb62b, SUC_lexicon_tags_34 },
    { 0x847cb62c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7cb62f, SUC_lexicon_tags_27 },
    { 0x2d3cb62f, SUC_lexicon_tags_26 },
    { 0x5fdcb62f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759cb638, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x65fcb63b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f1cb63d, SUC_lexicon_tags_65 },
    { 0x411cb63e, SUC_lexicon_tags_17 },
    { 0xb23cb63e, SUC_lexicon_tags_33 },
    { 0x55fcb640, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf95cb642, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcedcb644, SUC_lexicon_tags_2 },
    { 0xfb3cb645, SUC_lexicon_tags_30 },
    { 0x79fcb646, SUC_lexicon_tags_3 },
    { 0xfdbcb644, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28dcb649, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5cb64c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebcb65d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa97cb65f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9cb663, SUC_lexicon_tags_19 },
    { 0x70dcb663, SUC_lexicon_tags_38 },
    { 0x287cb665, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873cb66a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe73cb66c, SUC_lexicon_tags_11 },
    { 0x33dcb66d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x573cb66f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7cb671, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dcb676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f3cb678, SUC_lexicon_tags_5 },
    { 0xccdcb679, SUC_lexicon_tags_26 },
    { 0x223cb679, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7cb67f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbbcb681, SUC_lexicon_tags_27 },
    { 0x44dcb682, SUC_lexicon_tags_37 },
    { 0xbd3cb683, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171cb688, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bcb68c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfcb68f, SUC_lexicon_tags_79 },
    { 0x5afcb68f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7cb69e, SUC_lexicon_tags_17 },
    { 0x55bcb69e, SUC_lexicon_tags_14 },
    { 0xcdbcb6a0, SUC_lexicon_tags_4 },
    { 0xbf5cb6a1, SUC_lexicon_tags_33 },
    { 0xac7cb6a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1adcb6a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5cb6a8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe57cb6aa, SUC_lexicon_tags_30 },
    { 0x9b3cb6ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79cb6b1, SUC_lexicon_tags_2 },
    { 0x45dcb6b1, SUC_lexicon_tags_124 },
    { 0x471cb6b1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3d3cb6b5, SUC_lexicon_tags_7 },
    { 0x559cb6b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fcb6bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9cb6be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37cb6c1, SUC_lexicon_tags_40 },
    { 0xd39cb6c2, SUC_lexicon_tags_39 },
    { 0x14dcb6c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59cb6c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fcb6c9, SUC_lexicon_tags_43 },
    { 0x5b5cb6c9, SUC_lexicon_tags_33 },
    { 0xc15cb6c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9fcb6cd, SUC_lexicon_tags_28 },
    { 0x3cbcb6ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf6fcb6d0, SUC_lexicon_tags_221 },
    { 0x3a9cb6d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x931cb6d3, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfcb6d6, SUC_lexicon_tags_4 },
    { 0xcb1cb6d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5cb6da, SUC_lexicon_tags_80 },
    { 0x97fcb6db, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe21cb6dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9b5cb6df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341cb6e4, SUC_lexicon_tags_47 },
    { 0x517cb6e4, SUC_lexicon_tags_45 },
    { 0xcd1cb6e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x549cb6e8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25cb6f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9cb6fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d9cb6ff, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x215cb702, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc29cb704, SUC_lexicon_tags_63 },
    { 0x937cb705, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcb70b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfcb710, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bcb712, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b3cb714, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcbfcb716, SUC_lexicon_tags_1 },
    { 0x425cb717, SUC_lexicon_tags_33 },
    { 0x34dcb718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb81cb71a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9dbcb71c, SUC_lexicon_tags_15 },
    { 0xc19cb71d, SUC_lexicon_tags_37 },
    { 0x44dcb71e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69cb729, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667cb72f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bcb734, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3cb73d, SUC_lexicon_tags_37 },
    { 0x339cb73e, SUC_lexicon_tags_15 },
    { 0x1e9cb73e, SUC_lexicon_tags_61 },
    { 0xc97cb740, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1cb747, SUC_lexicon_tags_11 },
    { 0x31bcb748, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9cb751, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bcb756, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07cb759, SUC_lexicon_tags_11 },
    { 0x415cb75a, SUC_lexicon_tags_9 },
    { 0xe0fcb75b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697cb769, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199cb772, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dcb776, SUC_lexicon_tags_17 },
    { 0x27cb777, SUC_lexicon_tags_33 },
    { 0x529cb778, SUC_lexicon_tags_33 },
    { 0xc7fcb779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdafcb77b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x551cb77d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x571cb780, SUC_lexicon_tags_0 },
    { 0x72dcb781, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1cb784, SUC_lexicon_tags_48 },
    { 0xbe5cb785, SUC_lexicon_tags_17 },
    { 0x7fbcb784, SUC_lexicon_tags_32 },
    { 0x745cb787, SUC_lexicon_tags_12 },
    { 0x64fcb787, SUC_lexicon_tags_31 },
    { 0xf0bcb787, SUC_lexicon_tags_7 },
    { 0x4a5cb784, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd5bcb78c, SUC_lexicon_tags_31 },
    { 0x559cb78d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x859cb78f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3cb794, SUC_lexicon_tags_37 },
    { 0x179cb794, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafcb798, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x293cb79b, SUC_lexicon_tags_15 },
    { 0xab5cb79c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7cb7a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e3cb7a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79cb7a7, SUC_lexicon_tags_8 },
    { 0x13bcb7a8, SUC_lexicon_tags_33 },
    { 0x715cb7a9, SUC_lexicon_tags_8 },
    { 0x51fcb7aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x769cb7ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95bcb7af, SUC_lexicon_tags_1 },
    { 0x107cb7af, SUC_lexicon_tags_1 },
    { 0x64dcb7b1, SUC_lexicon_tags_0 },
    { 0x4e1cb7af, SUC_lexicon_tags_3 },
    { 0x541cb7b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6fdcb7b5, SUC_lexicon_tags_7 },
    { 0xbafcb7b5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcbbcb7b8, SUC_lexicon_tags_27 },
    { 0xf8dcb7b9, SUC_lexicon_tags_32 },
    { 0xf17cb7ba, SUC_lexicon_tags_27 },
    { 0x551cb7bb, SUC_lexicon_tags_25 },
    { 0x49bcb7bc, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe95cb7be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15cb7c2, SUC_lexicon_tags_3 },
    { 0x52bcb7c3, SUC_lexicon_tags_25 },
    { 0xd9dcb7c4, SUC_lexicon_tags_15 },
    { 0x3f5cb7c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadcb7cb, SUC_lexicon_tags_37 },
    { 0xd77cb7cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfd1cb7ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a9cb7d0, SUC_lexicon_tags_15 },
    { 0x7c9cb7d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993cb7d7, SUC_lexicon_tags_37 },
    { 0x2f7cb7d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bcb7db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcb5cb7dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe5cb7df, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279cb7e3, SUC_lexicon_tags_14 },
    { 0x5dfcb7e4, SUC_lexicon_tags_62 },
    { 0x21fcb7e5, SUC_lexicon_tags_87 },
    { 0x5d1cb7e5, SUC_lexicon_tags_37 },
    { 0x27fcb7e7, SUC_lexicon_tags_13 },
    { 0xe1cb7e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28dcb7ec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdcb7ef, SUC_lexicon_tags_2 },
    { 0xc97cb7f0, SUC_lexicon_tags_0 },
    { 0x9f5cb7f1, SUC_lexicon_tags_45 },
    { 0x2a7cb7f0, SUC_lexicon_tags_25 },
    { 0x5c7cb7f3, SUC_lexicon_tags_38 },
    { 0xea1cb7ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xddcb7f7, SUC_lexicon_tags_7 },
    { 0xff1cb7f8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fcb7fb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x965cb7fd, SUC_lexicon_tags_11 },
    { 0xfefcb7fe, SUC_lexicon_tags_11 },
    { 0x245cb7ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87fcb801, SUC_lexicon_tags_12 },
    { 0x347cb802, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97cb80e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5abcb810, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dcb815, SUC_lexicon_tags_30 },
    { 0x141cb815, SUC_lexicon_tags_29 },
    { 0xf97cb817, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85cb819, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x98dcb81b, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x4c3cb81d, SUC_lexicon_tags_16 },
    { 0xd7bcb81e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x541cb820, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9cb823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb1cb825, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4ffcb827, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9cb82c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1fcb82e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9dcb830, SUC_lexicon_tags_39 },
    { 0xda1cb830, SUC_lexicon_tags_25 },
    { 0x7d3cb832, SUC_lexicon_tags_7 },
    { 0x64fcb830, SUC_lexicon_tags_17 },
    { 0xf47cb834, SUC_lexicon_tags_30 },
    { 0x2f1cb833, SUC_lexicon_tags_33 },
    { 0x6e5cb832, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebcb83b, SUC_lexicon_tags_25 },
    { 0xf01cb83b, SUC_lexicon_tags_7 },
    { 0xe7dcb83d, SUC_lexicon_tags_7 },
    { 0xdbcb83b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6fcb840, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89cb844, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237cb849, SUC_lexicon_tags_7 },
    { 0x2f9cb849, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x913cb84d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9cb852, SUC_lexicon_tags_60 },
    { 0xfd3cb853, SUC_lexicon_tags_37 },
    { 0xf3cb854, SUC_lexicon_tags_56 },
    { 0x27bcb855, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859cb85f, SUC_lexicon_tags_30 },
    { 0x2dfcb860, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bcb863, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dcb866, SUC_lexicon_tags_373 },
    { 0x3c5cb867, SUC_lexicon_tags_6 },
    { 0x2cfcb868, SUC_lexicon_tags_29 },
    { 0x4b9cb866, SUC_lexicon_tags_36 },
    { 0x3c1cb86a, SUC_lexicon_tags_30 },
    { 0x3efcb866, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xccdcb86d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf89cb86f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbebcb871, SUC_lexicon_tags_11 },
    { 0xebfcb872, SUC_lexicon_tags_37 },
    { 0xcddcb873, SUC_lexicon_tags_2 },
    { 0x14bcb874, SUC_lexicon_tags_17 },
    { 0x62bcb873, SUC_lexicon_tags_21 },
    { 0x613cb873, SUC_lexicon_tags_11 },
    { 0xa51cb876, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54bcb87a, SUC_lexicon_tags_25 },
    { 0x92dcb87b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbcb880, SUC_lexicon_tags_3 },
    { 0xd33cb881, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddcb884, SUC_lexicon_tags_25 },
    { 0x129cb884, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29cb889, SUC_lexicon_tags_14 },
    { 0x3dfcb88a, SUC_lexicon_tags_45 },
    { 0xe27cb88b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dcb890, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cb898, SUC_lexicon_tags_12 },
    { 0x929cb898, SUC_lexicon_tags_1 },
    { 0xdbbcb89a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3cb89d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc33cb89f, SUC_lexicon_tags_49 },
    { 0x6abcb8a0, SUC_lexicon_tags_25 },
    { 0x623cb8a1, SUC_lexicon_tags_7 },
    { 0xf45cb8a2, SUC_lexicon_tags_27 },
    { 0xe51cb8a0, SUC_lexicon_tags_17 },
    { 0x187cb8a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd1dcb8a6, SUC_lexicon_tags_11 },
    { 0xb07cb8a6, SUC_lexicon_tags_5 },
    { 0x509cb8a8, SUC_lexicon_tags_25 },
    { 0xf3dcb8a9, SUC_lexicon_tags_17 },
    { 0x4fcb8aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fcb8ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x463cb8b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebcb8b6, SUC_lexicon_tags_8 },
    { 0x421cb8b7, SUC_lexicon_tags_43 },
    { 0xf21cb8b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b7cb8ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fcb8bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x885cb8c0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5ddcb8c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91cb8c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dcb8cb, SUC_lexicon_tags_56 },
    { 0x32dcb8cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x319cb8ce, SUC_lexicon_tags_15 },
    { 0x5b9cb8cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61cb8d3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x68fcb8d5, SUC_lexicon_tags_2 },
    { 0x5d5cb8d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563cb8dd, SUC_lexicon_tags_65 },
    { 0x139cb8dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbcb8e2, SUC_lexicon_tags_15 },
    { 0xc27cb8e3, SUC_lexicon_tags_2 },
    { 0x867cb8e2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6f9cb8e6, SUC_lexicon_tags_47 },
    { 0x671cb8e6, SUC_lexicon_tags_19 },
    { 0x5fbcb8e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95cb8ed, SUC_lexicon_tags_1 },
    { 0x4b1cb8ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fcb8f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629cb8fb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bcb8ff, SUC_lexicon_tags_47 },
    { 0x835cb900, SUC_lexicon_tags_37 },
    { 0xa0dcb901, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59cb905, SUC_lexicon_tags_15 },
    { 0xe67cb906, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dcb90d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5cb911, SUC_lexicon_tags_49 },
    { 0xefcb912, SUC_lexicon_tags_60 },
    { 0xb35cb913, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63cb915, SUC_lexicon_tags_12 },
    { 0x725cb916, SUC_lexicon_tags_59 },
    { 0x68bcb917, SUC_lexicon_tags_28 },
    { 0x2c7cb918, SUC_lexicon_tags_34 },
    { 0xf77cb919, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2cbcb91b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefcb91f, SUC_lexicon_tags_37 },
    { 0x59bcb91f, SUC_lexicon_tags_16 },
    { 0x805cb921, SUC_lexicon_tags_49 },
    { 0x43dcb922, SUC_lexicon_tags_11 },
    { 0xa41cb922, SUC_lexicon_tags_6 },
    { 0xd5cb923, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9fcb926, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823cb92c, SUC_lexicon_tags_27 },
    { 0x621cb92d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7cb933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe11cb935, SUC_lexicon_tags_11 },
    { 0xe3dcb936, SUC_lexicon_tags_45 },
    { 0x949cb936, SUC_lexicon_tags_28 },
    { 0x181cb938, SUC_lexicon_tags_11 },
    { 0x6bbcb939, SUC_lexicon_tags_17 },
    { 0xe49cb939, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd1cb93c, SUC_lexicon_tags_234 },
    { 0x7d9cb93d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dcb941, SUC_lexicon_tags_25 },
    { 0xf6fcb942, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x635cb945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49cb948, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5cb94b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93cb950, SUC_lexicon_tags_15 },
    { 0x1bdcb950, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9cb954, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedcb95e, SUC_lexicon_tags_17 },
    { 0x9dfcb95e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfdcb961, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adcb966, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1cb969, SUC_lexicon_tags_47 },
    { 0xb7cb96a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x419cb96c, SUC_lexicon_tags_31 },
    { 0x345cb96d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735cb972, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99cb976, SUC_lexicon_tags_15 },
    { 0xe4fcb977, SUC_lexicon_tags_2 },
    { 0x8c5cb978, SUC_lexicon_tags_188 },
    { 0x74fcb977, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7cb97e, SUC_lexicon_tags_23 },
    { 0xb7dcb97f, SUC_lexicon_tags_63 },
    { 0x6afcb980, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9fdcb982, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbcb985, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1cb989, SUC_lexicon_tags_64 },
    { 0xb7dcb98a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fcb990, SUC_lexicon_tags_29 },
    { 0x2bbcb990, SUC_lexicon_tags_27 },
    { 0x2a3cb992, SUC_lexicon_tags_11 },
    { 0xd3dcb991, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x16dcb996, SUC_lexicon_tags_7 },
    { 0xb47cb996, SUC_lexicon_tags_82 },
    { 0xd2fcb998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x439cb99b, SUC_lexicon_tags_11 },
    { 0xd6dcb99c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829cb9a7, SUC_lexicon_tags_207 },
    { 0xc6bcb9a7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fcb9ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483cb9b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31cb9b5, SUC_lexicon_tags_68 },
    { 0x1bdcb9b5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7cb9bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969cb9c0, SUC_lexicon_tags_11 },
    { 0x5fdcb9c1, SUC_lexicon_tags_38 },
    { 0xf29cb9c2, SUC_lexicon_tags_56 },
    { 0x675cb9c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367cb9c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc73cb9c9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x111cb9cc, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7cb9cf, SUC_lexicon_tags_12 },
    { 0x949cb9d0, SUC_lexicon_tags_37 },
    { 0x929cb9cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bcb9d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa19cb9d8, SUC_lexicon_tags_17 },
    { 0xf7bcb9d8, SUC_lexicon_tags_2 },
    { 0xa8dcb9da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1cb9dc, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x399cb9de, SUC_lexicon_tags_43 },
    { 0xd4bcb9df, SUC_lexicon_tags_11 },
    { 0x465cb9df, SUC_lexicon_tags_14 },
    { 0x6f3cb9e1, SUC_lexicon_tags_0 },
    { 0xe09cb9e2, SUC_lexicon_tags_45 },
    { 0xa09cb9df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75cb9e7, SUC_lexicon_tags_23 },
    { 0xb81cb9e7, SUC_lexicon_tags_27 },
    { 0x917cb9e9, SUC_lexicon_tags_1 },
    { 0x1bfcb9ea, SUC_lexicon_tags_12 },
    { 0x303cb9eb, SUC_lexicon_tags_34 },
    { 0xcedcb9ea, SUC_lexicon_tags_37 },
    { 0x1d9cb9ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8cbcb9ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x141cb9f1, SUC_lexicon_tags_17 },
    { 0xedfcb9f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dcb9f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdcb9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37cb9ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fcba06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x51dcba08, SUC_lexicon_tags_54 },
    { 0xeabcba08, SUC_lexicon_tags_6 },
    { 0xae5cba0a, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0xdadcba0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bcba12, SUC_lexicon_tags_3 },
    { 0x8b9cba12, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57cba19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3cba1c, SUC_lexicon_tags_64 },
    { 0x595cba1d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639cba22, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x561cba25, SUC_lexicon_tags_39 },
    { 0x6e1cba25, SUC_lexicon_tags_9 },
    { 0xd3fcba27, SUC_lexicon_tags_82 },
    { 0x319cba26, SUC_lexicon_tags_47 },
    { 0xcebcba25, SUC_lexicon_tags_2 },
    { 0x831cba25, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3cba2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cba31, SUC_lexicon_tags_14 },
    { 0x6afcba32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x50dcba34, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8a1cba36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7cba45, SUC_lexicon_tags_17 },
    { 0x959cba46, SUC_lexicon_tags_38 },
    { 0x139cba47, SUC_lexicon_tags_34 },
    { 0xfbcba48, SUC_lexicon_tags_7 },
    { 0xa0fcba49, SUC_lexicon_tags_27 },
    { 0x219cba47, SUC_lexicon_tags_28 },
    { 0xa57cba4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9cba4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63cba51, SUC_lexicon_tags_60 },
    { 0xb43cba51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9cba55, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x601cba57, SUC_lexicon_tags_17 },
    { 0x85fcba57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdabcba5a, SUC_lexicon_tags_30 },
    { 0xf5dcba5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bcba60, SUC_lexicon_tags_13 },
    { 0xe6bcba60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x591cba63, SUC_lexicon_tags_2 },
    { 0x87fcba64, SUC_lexicon_tags_25 },
    { 0x553cba65, SUC_lexicon_tags_25 },
    { 0x55fcba64, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437cba6b, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bcba6e, SUC_lexicon_tags_11 },
    { 0x883cba6e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x185cba71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47cba74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3d5cba76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd97cba78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x511cba7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33fcba7c, SUC_lexicon_tags_79 },
    { 0xb03cba7d, SUC_lexicon_tags_64 },
    { 0x9bfcba7e, SUC_lexicon_tags_0 },
    { 0x903cba7f, SUC_lexicon_tags_15 },
    { 0xaa3cba80, SUC_lexicon_tags_31 },
    { 0xd65cba81, SUC_lexicon_tags_6 },
    { 0xacdcba80, SUC_lexicon_tags_15 },
    { 0xfa1cba7f, SUC_lexicon_tags_29 },
    { 0x7f3cba84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7cba8d, SUC_lexicon_tags_17 },
    { 0xcfbcba8d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf21cba90, SUC_lexicon_tags_12 },
    { 0xd69cba90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3cba94, SUC_lexicon_tags_40 },
    { 0xc27cba95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe07cba97, SUC_lexicon_tags_45 },
    { 0xce1cba98, SUC_lexicon_tags_21 },
    { 0x819cba99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc9cba9b, SUC_lexicon_tags_11 },
    { 0x283cba9b, SUC_lexicon_tags_65 },
    { 0x773cba9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x971cba9f, SUC_lexicon_tags_48 },
    { 0x649cbaa0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3cbaa5, SUC_lexicon_tags_6 },
    { 0xb0bcbaa6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1cbaaa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x385cbaac, SUC_lexicon_tags_11 },
    { 0x68fcbaad, SUC_lexicon_tags_47 },
    { 0xff7cbaae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbcbab5, SUC_lexicon_tags_2 },
    { 0xef9cbab6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdcbabb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7cbabf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5cbac5, SUC_lexicon_tags_27 },
    { 0xd65cbac6, SUC_lexicon_tags_77 },
    { 0x115cbac7, SUC_lexicon_tags_11 },
    { 0xca9cbac6, SUC_lexicon_tags_17 },
    { 0x52bcbac8, SUC_lexicon_tags_29 },
    { 0xe65cbaca, SUC_lexicon_tags_11 },
    { 0xfabcbac5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633cbad2, SUC_lexicon_tags_49 },
    { 0xf65cbad2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x67dcbad6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85cbada, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbcbadd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69cbae0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbae3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd1bcbae5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x623cbae7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1cbaeb, SUC_lexicon_tags_25 },
    { 0x3f1cbaec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xccbcbaee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x641cbaf0, SUC_lexicon_tags_45 },
    { 0xdd3cbaf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17cbaf6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dcbafa, SUC_lexicon_tags_19 },
    { 0x4cfcbafb, SUC_lexicon_tags_19 },
    { 0xfa1cbafc, SUC_lexicon_tags_25 },
    { 0x2a7cbafc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd27cbaff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x35cbb02, SUC_lexicon_tags_14 },
    { 0x2d7cbb02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbcbb0b, SUC_lexicon_tags_43 },
    { 0x9fcbb0b, SUC_lexicon_tags_17 },
    { 0x7c5cbb0d, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x79cbb0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19cbb13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45cbb17, SUC_lexicon_tags_33 },
    { 0x4bbcbb17, SUC_lexicon_tags_11 },
    { 0x7a5cbb19, SUC_lexicon_tags_0 },
    { 0xe6bcbb1a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x439cbb1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03cbb22, SUC_lexicon_tags_25 },
    { 0x95bcbb22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fcbb25, SUC_lexicon_tags_37 },
    { 0xc79cbb26, SUC_lexicon_tags_32 },
    { 0x52fcbb27, SUC_lexicon_tags_9 },
    { 0x43dcbb28, SUC_lexicon_tags_33 },
    { 0xb2fcbb29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x775cbb2c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99cbb32, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdcbb35, SUC_lexicon_tags_11 },
    { 0x3dfcbb36, SUC_lexicon_tags_25 },
    { 0x501cbb36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271cbb3b, SUC_lexicon_tags_29 },
    { 0x833cbb3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dcbb42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5cbb4b, SUC_lexicon_tags_3 },
    { 0x67cbb4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bcbb51, SUC_lexicon_tags_33 },
    { 0x363cbb51, SUC_lexicon_tags_15 },
    { 0x7d1cbb53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe09cbb55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659cbb5a, SUC_lexicon_tags_23 },
    { 0x2bdcbb5b, SUC_lexicon_tags_63 },
    { 0xd01cbb5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7cbb61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5cbb64, SUC_lexicon_tags_28 },
    { 0xf51cbb65, SUC_lexicon_tags_1 },
    { 0xfb5cbb65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11cbb69, SUC_lexicon_tags_49 },
    { 0x1cbcbb6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617cbb77, SUC_lexicon_tags_1 },
    { 0xe57cbb78, SUC_lexicon_tags_15 },
    { 0x86bcbb79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a1cbb7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x383cbb7d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffcbb81, SUC_lexicon_tags_15 },
    { 0x669cbb82, SUC_lexicon_tags_15 },
    { 0x1a3cbb83, SUC_lexicon_tags_37 },
    { 0x66fcbb83, SUC_lexicon_tags_2 },
    { 0xf01cbb84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdcbb89, SUC_lexicon_tags_28 },
    { 0x839cbb8a, SUC_lexicon_tags_12 },
    { 0xbf5cbb8a, SUC_lexicon_tags_7 },
    { 0x7fbcbb89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xab3cbb8e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebcbb9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cbb9d, SUC_lexicon_tags_38 },
    { 0x69cbb9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1cbba1, SUC_lexicon_tags_37 },
    { 0xfa7cbba2, SUC_lexicon_tags_11 },
    { 0x295cbba2, SUC_lexicon_tags_37 },
    { 0xec7cbba4, SUC_lexicon_tags_47 },
    { 0xd79cbba5, SUC_lexicon_tags_16 },
    { 0x5a7cbba5, SUC_lexicon_tags_1 },
    { 0x8b7cbba7, SUC_lexicon_tags_11 },
    { 0xc97cbba5, SUC_lexicon_tags_7 },
    { 0xf89cbba9, SUC_lexicon_tags_16 },
    { 0xaf3cbba8, SUC_lexicon_tags_17 },
    { 0xb3cbba6, SUC_lexicon_tags_1 },
    { 0xd63cbbac, SUC_lexicon_tags_15 },
    { 0x159cbba1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9bfcbbaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dcbbb3, SUC_lexicon_tags_42 },
    { 0xf11cbbb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb9cbbb6, SUC_lexicon_tags_31 },
    { 0x845cbbb7, SUC_lexicon_tags_49 },
    { 0x537cbbb8, SUC_lexicon_tags_45 },
    { 0x395cbbb9, SUC_lexicon_tags_0 },
    { 0x559cbbba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dcbbbd, SUC_lexicon_tags_63 },
    { 0x2a3cbbbd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x547cbbc0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x14fcbbc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dcbbc8, SUC_lexicon_tags_7 },
    { 0xa6dcbbc8, SUC_lexicon_tags_17 },
    { 0xc57cbbc9, SUC_lexicon_tags_2 },
    { 0x3fcbbcb, SUC_lexicon_tags_33 },
    { 0x901cbbc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x453cbbce, SUC_lexicon_tags_17 },
    { 0x3fdcbbcf, SUC_lexicon_tags_18 },
    { 0xe5bcbbce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37cbbd4, SUC_lexicon_tags_15 },
    { 0xcf7cbbd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abcbbd8, SUC_lexicon_tags_7 },
    { 0xcbcbbd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x495cbbdb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19cbbde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbcbbe1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x351cbbe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x497cbbe6, SUC_lexicon_tags_12 },
    { 0x9d1cbbe6, SUC_lexicon_tags_33 },
    { 0xd23cbbe8, SUC_lexicon_tags_23 },
    { 0x5d7cbbe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47cbbee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2adcbbf1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bcbbf4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x12dcbbf6, SUC_lexicon_tags_8 },
    { 0x7c3cbbf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57cbbfa, SUC_lexicon_tags_48 },
    { 0x6a3cbbfb, SUC_lexicon_tags_10 },
    { 0xfcdcbbfc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361cbc03, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd79cbc05, SUC_lexicon_tags_11 },
    { 0x2e1cbc06, SUC_lexicon_tags_11 },
    { 0x98dcbc07, SUC_lexicon_tags_65 },
    { 0xfb3cbc06, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bcbc17, SUC_lexicon_tags_25 },
    { 0x2f9cbc18, SUC_lexicon_tags_201 },
    { 0xd7cbc18, SUC_lexicon_tags_79 },
    { 0xde9cbc1a, SUC_lexicon_tags_63 },
    { 0x385cbc1b, SUC_lexicon_tags_2 },
    { 0xf67cbc1b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5cbc20, SUC_lexicon_tags_11 },
    { 0x9fdcbc20, SUC_lexicon_tags_12 },
    { 0xa55cbc20, SUC_lexicon_tags_25 },
    { 0x4c7cbc23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17bcbc25, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x20bcbc27, SUC_lexicon_tags_134 },
    { 0x10bcbc27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfcbc2b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4c1cbc2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1cbc31, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5cbc34, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3cbc3c, SUC_lexicon_tags_11 },
    { 0x519cbc3d, SUC_lexicon_tags_46 },
    { 0x941cbc3c, SUC_lexicon_tags_6 },
    { 0xd3bcbc3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5cbc45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x599cbc47, SUC_lexicon_tags_27 },
    { 0x941cbc48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fcbc4c, SUC_lexicon_tags_7 },
    { 0x83bcbc4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fcbc51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fcbc5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x463cbc5c, SUC_lexicon_tags_65 },
    { 0x1d7cbc5d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddcbc61, SUC_lexicon_tags_15 },
    { 0xba5cbc62, SUC_lexicon_tags_19 },
    { 0xa95cbc62, SUC_lexicon_tags_87 },
    { 0x92dcbc64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fcbc6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdcbc6e, SUC_lexicon_tags_26 },
    { 0x429cbc6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x86bcbc71, SUC_lexicon_tags_15 },
    { 0x417cbc72, SUC_lexicon_tags_37 },
    { 0x9abcbc73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfcbc79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa89cbc7b, SUC_lexicon_tags_7 },
    { 0x9fbcbc7c, SUC_lexicon_tags_12 },
    { 0x69cbc7b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa35cbc7f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bcbc84, SUC_lexicon_tags_30 },
    { 0x69fcbc85, SUC_lexicon_tags_14 },
    { 0xd2fcbc86, SUC_lexicon_tags_47 },
    { 0x4a7cbc87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9a7cbc89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53cbc8c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xacbcbc8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcbc93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcbc98, SUC_lexicon_tags_25 },
    { 0xdd5cbc98, SUC_lexicon_tags_37 },
    { 0xe93cbc9a, SUC_lexicon_tags_265 },
    { 0x27fcbc9b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5cbca1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41cbca5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x61dcbca8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfcbcab, SUC_lexicon_tags_14 },
    { 0x431cbcab, SUC_lexicon_tags_11 },
    { 0xb53cbcad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7cbcb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919cbcb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fcbcc1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb3cbcc3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885cbcc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27cbccc, SUC_lexicon_tags_6 },
    { 0xddcbccd, SUC_lexicon_tags_17 },
    { 0xeb7cbcce, SUC_lexicon_tags_39 },
    { 0xeb1cbcce, SUC_lexicon_tags_0 },
    { 0xb9dcbccf, SUC_lexicon_tags_37 },
    { 0xfbbcbccc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9cbcd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abcbcd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc73cbcda, SUC_lexicon_tags_11 },
    { 0xff5cbcdb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x645cbcde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671cbce3, SUC_lexicon_tags_19 },
    { 0xe15cbce4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cbcea, SUC_lexicon_tags_27 },
    { 0x93bcbcea, SUC_lexicon_tags_414 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41cbcf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x403cbcf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967cbcfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581cbd01, SUC_lexicon_tags_7 },
    { 0xee9cbd02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ebcbd04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fcbd0a, SUC_lexicon_tags_7 },
    { 0xc6fcbd0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cbd0e, SUC_lexicon_tags_31 },
    { 0xf73cbd0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fcbd13, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9cbd1c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe2dcbd1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x749cbd20, SUC_lexicon_tags_31 },
    { 0x9c3cbd20, SUC_lexicon_tags_60 },
    { 0xa13cbd20, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825cbd26, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07cbd2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x663cbd2e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x973cbd31, SUC_lexicon_tags_38 },
    { 0x751cbd32, SUC_lexicon_tags_1 },
    { 0x901cbd33, SUC_lexicon_tags_15 },
    { 0x9d7cbd34, SUC_lexicon_tags_21 },
    { 0x1dbcbd32, SUC_lexicon_tags_49 },
    { 0x375cbd32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bcbd3a, SUC_lexicon_tags_2 },
    { 0xc5fcbd3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe17cbd3d, SUC_lexicon_tags_1 },
    { 0x2bcbd3e, SUC_lexicon_tags_15 },
    { 0x847cbd3f, SUC_lexicon_tags_25 },
    { 0x7bfcbd3d, SUC_lexicon_tags_2 },
    { 0x87bcbd3e, SUC_lexicon_tags_37 },
    { 0xd7fcbd42, SUC_lexicon_tags_5 },
    { 0x285cbd42, SUC_lexicon_tags_34 },
    { 0xd81cbd43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dcbd48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53cbd4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49cbd53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775cbd57, SUC_lexicon_tags_103 },
    { 0x4d7cbd57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5cbd5c, SUC_lexicon_tags_25 },
    { 0x95fcbd5d, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0x71bcbd60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383cbd67, SUC_lexicon_tags_37 },
    { 0xee7cbd68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179cbd6c, SUC_lexicon_tags_15 },
    { 0xa75cbd6c, SUC_lexicon_tags_14 },
    { 0x211cbd6e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9cbd72, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x65dcbd74, SUC_lexicon_tags_25 },
    { 0x677cbd75, SUC_lexicon_tags_39 },
    { 0xe9dcbd74, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfcbd7a, SUC_lexicon_tags_7 },
    { 0x5f3cbd7b, SUC_lexicon_tags_7 },
    { 0xd17cbd7c, SUC_lexicon_tags_17 },
    { 0xb89cbd7c, SUC_lexicon_tags_7 },
    { 0x9ddcbd7c, SUC_lexicon_tags_48 },
    { 0xeefcbd7c, SUC_lexicon_tags_2 },
    { 0x42bcbd80, SUC_lexicon_tags_1222 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69cbd89, SUC_lexicon_tags_26 },
    { 0xd29cbd89, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1cbd8d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x457cbd8f, SUC_lexicon_tags_14 },
    { 0x2b7cbd90, SUC_lexicon_tags_29 },
    { 0xcfbcbd91, SUC_lexicon_tags_60 },
    { 0x559cbd92, SUC_lexicon_tags_235 },
    { 0xc13cbd91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973cbd98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31cbd9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8bcbd9e, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x567cbda0, SUC_lexicon_tags_15 },
    { 0xcb5cbda1, SUC_lexicon_tags_34 },
    { 0xda5cbda2, SUC_lexicon_tags_0 },
    { 0x821cbda2, SUC_lexicon_tags_11 },
    { 0xc23cbda4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6fcbda6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fcbda9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfcbdad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x47cbdaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xab7cbdb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301cbdb5, SUC_lexicon_tags_2 },
    { 0x9d3cbdb6, SUC_lexicon_tags_75 },
    { 0x841cbdb7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bcbdbe, SUC_lexicon_tags_25 },
    { 0x561cbdbf, SUC_lexicon_tags_37 },
    { 0xff5cbdc0, SUC_lexicon_tags_37 },
    { 0xa11cbdc1, SUC_lexicon_tags_74 },
    { 0x31dcbdc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97cbdc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bcbdce, SUC_lexicon_tags_12 },
    { 0xc9dcbdce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105cbdd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35cbddb, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79cbddf, SUC_lexicon_tags_12 },
    { 0x949cbde0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdcbde3, SUC_lexicon_tags_57 },
    { 0xacdcbde4, SUC_lexicon_tags_17 },
    { 0xd5fcbde5, SUC_lexicon_tags_1 },
    { 0xc5dcbde6, SUC_lexicon_tags_71 },
    { 0x43fcbde6, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xb8bcbde9, SUC_lexicon_tags_37 },
    { 0xed7cbde9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325cbdf4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3cbdf9, SUC_lexicon_tags_25 },
    { 0x209cbdfa, SUC_lexicon_tags_15 },
    { 0xe6fcbdfa, SUC_lexicon_tags_12 },
    { 0x455cbdfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abcbe01, SUC_lexicon_tags_37 },
    { 0xea1cbe02, SUC_lexicon_tags_151 },
    { 0xd11cbe02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dcbe0a, SUC_lexicon_tags_37 },
    { 0xd85cbe0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3cbe0e, SUC_lexicon_tags_37 },
    { 0x1b5cbe0e, SUC_lexicon_tags_11 },
    { 0x7c5cbe0f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xabfcbe12, SUC_lexicon_tags_24 },
    { 0x70dcbe13, SUC_lexicon_tags_65 },
    { 0xa69cbe14, SUC_lexicon_tags_1 },
    { 0x4e1cbe14, SUC_lexicon_tags_15 },
    { 0xbc7cbe16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddcbe20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971cbe24, SUC_lexicon_tags_48 },
    { 0xce1cbe24, SUC_lexicon_tags_10 },
    { 0xfadcbe26, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579cbe2a, SUC_lexicon_tags_14 },
    { 0x37bcbe2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bcbe35, SUC_lexicon_tags_7 },
    { 0x183cbe36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x345cbe39, SUC_lexicon_tags_1 },
    { 0x85cbe3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fcbe3f, SUC_lexicon_tags_37 },
    { 0xd1cbe40, SUC_lexicon_tags_2 },
    { 0xd5cbe3f, SUC_lexicon_tags_10 },
    { 0xb9cbe42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x197cbe44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7cbe47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49cbe4a, SUC_lexicon_tags_33 },
    { 0x80fcbe4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dcbe50, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb1cbe52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb2bcbe54, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26dcbe59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5cbe63, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbadcbe65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dcbe6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x909cbe6c, SUC_lexicon_tags_17 },
    { 0xb83cbe6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6efcbe6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdffcbe71, SUC_lexicon_tags_1 },
    { 0x21dcbe72, SUC_lexicon_tags_6 },
    { 0xa9cbe72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcbe76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41cbe7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b1cbe7c, SUC_lexicon_tags_39 },
    { 0x657cbe7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3cbe83, SUC_lexicon_tags_37 },
    { 0x55cbe84, SUC_lexicon_tags_32 },
    { 0x25dcbe85, SUC_lexicon_tags_1 },
    { 0x80fcbe86, SUC_lexicon_tags_12 },
    { 0x997cbe84, SUC_lexicon_tags_37 },
    { 0xb17cbe86, SUC_lexicon_tags_4 },
    { 0xdb1cbe87, SUC_lexicon_tags_65 },
    { 0x559cbe83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fcbe8e, SUC_lexicon_tags_8 },
    { 0x937cbe8e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddcbe9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaa7cbe9d, SUC_lexicon_tags_17 },
    { 0x2fbcbe9e, SUC_lexicon_tags_11 },
    { 0x2e7cbe9f, SUC_lexicon_tags_54 },
    { 0xfc1cbea0, SUC_lexicon_tags_12 },
    { 0x60fcbea0, SUC_lexicon_tags_12 },
    { 0xb57cbea2, SUC_lexicon_tags_17 },
    { 0x54dcbea3, SUC_lexicon_tags_7 },
    { 0xa39cbea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5cbea7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371cbeb2, SUC_lexicon_tags_25 },
    { 0x9bcbeb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfcbebc, SUC_lexicon_tags_12 },
    { 0x96bcbebd, SUC_lexicon_tags_1 },
    { 0xa41cbebc, SUC_lexicon_tags_30 },
    { 0xb3bcbebc, SUC_lexicon_tags_7 },
    { 0xb11cbec0, SUC_lexicon_tags_11 },
    { 0x54fcbebd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01cbec6, SUC_lexicon_tags_16 },
    { 0xf71cbec7, SUC_lexicon_tags_14 },
    { 0x583cbec8, SUC_lexicon_tags_68 },
    { 0xc03cbec9, SUC_lexicon_tags_12 },
    { 0x41bcbeca, SUC_lexicon_tags_30 },
    { 0xf67cbec6, SUC_lexicon_tags_62 },
    { 0x63cbecc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907cbed1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x249cbed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9cbedb, SUC_lexicon_tags_1 },
    { 0xbcbcbedc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27cbedf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa8bcbee1, SUC_lexicon_tags_13 },
    { 0x685cbee2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x20dcbee4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93cbee9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcbeec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3cbef0, SUC_lexicon_tags_37 },
    { 0xa5bcbef1, SUC_lexicon_tags_19 },
    { 0x273cbef1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x219cbef4, SUC_lexicon_tags_37 },
    { 0x9fbcbef5, SUC_lexicon_tags_0 },
    { 0xc13cbef6, SUC_lexicon_tags_47 },
    { 0x791cbef7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bcbefa, SUC_lexicon_tags_2 },
    { 0x489cbefb, SUC_lexicon_tags_55 },
    { 0x51cbefc, SUC_lexicon_tags_13 },
    { 0x433cbefb, SUC_lexicon_tags_6 },
    { 0x519cbefb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853cbf04, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe1dcbf06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd09cbf08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdbcbf0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fcbf0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17cbf15, SUC_lexicon_tags_69 },
    { 0x509cbf16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9cbf1a, SUC_lexicon_tags_2 },
    { 0xb95cbf1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32dcbf1e, SUC_lexicon_tags_47 },
    { 0xfd3cbf1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9cbf23, SUC_lexicon_tags_1 },
    { 0x85dcbf23, SUC_lexicon_tags_0 },
    { 0xf23cbf25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c5cbf27, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07cbf2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309cbf31, SUC_lexicon_tags_33 },
    { 0xa17cbf32, SUC_lexicon_tags_26 },
    { 0x839cbf33, SUC_lexicon_tags_27 },
    { 0x45fcbf34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dcbf39, SUC_lexicon_tags_17 },
    { 0xfb9cbf3a, SUC_lexicon_tags_25 },
    { 0x87fcbf39, SUC_lexicon_tags_15 },
    { 0x9b5cbf3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635cbf41, SUC_lexicon_tags_30 },
    { 0x1e9cbf41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7cbf45, SUC_lexicon_tags_12 },
    { 0x455cbf46, SUC_lexicon_tags_17 },
    { 0xf7fcbf47, SUC_lexicon_tags_15 },
    { 0xce9cbf47, SUC_lexicon_tags_25 },
    { 0x663cbf48, SUC_lexicon_tags_47 },
    { 0xa69cbf4a, SUC_lexicon_tags_14 },
    { 0x7ffcbf48, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x68fcbf4d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa81cbf4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x321cbf51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fcbf5b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bcbf5f, SUC_lexicon_tags_65 },
    { 0xc73cbf60, SUC_lexicon_tags_18 },
    { 0xaadcbf61, SUC_lexicon_tags_39 },
    { 0xfb7cbf5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3afcbf64, SUC_lexicon_tags_1 },
    { 0x405cbf64, SUC_lexicon_tags_32 },
    { 0x60dcbf64, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fcbf6a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xff3cbf6c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbcbf6f, SUC_lexicon_tags_35 },
    { 0x9f5cbf70, SUC_lexicon_tags_47 },
    { 0x243cbf71, SUC_lexicon_tags_63 },
    { 0x75bcbf72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9cbf76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573cbf7c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x203cbf7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdcbf84, SUC_lexicon_tags_19 },
    { 0x4e9cbf85, SUC_lexicon_tags_8 },
    { 0xbcbf86, SUC_lexicon_tags_7 },
    { 0x67cbf84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dcbf8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5b3cbf8d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcbf94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbcbf9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x451cbf9c, SUC_lexicon_tags_11 },
    { 0x1fdcbf9c, SUC_lexicon_tags_7 },
    { 0x889cbf9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4dcbfa0, SUC_lexicon_tags_25 },
    { 0xf95cbfa0, SUC_lexicon_tags_47 },
    { 0xe8fcbfa0, SUC_lexicon_tags_1 },
    { 0x8f9cbfa3, SUC_lexicon_tags_11 },
    { 0xb55cbfa3, SUC_lexicon_tags_39 },
    { 0xd3bcbfa0, SUC_lexicon_tags_23 },
    { 0x7ebcbfa6, SUC_lexicon_tags_20 },
    { 0xa11cbfa6, SUC_lexicon_tags_46 },
    { 0x397cbfa4, SUC_lexicon_tags_25 },
    { 0xbf5cbfa9, SUC_lexicon_tags_1 },
    { 0x985cbfa9, SUC_lexicon_tags_11 },
    { 0x14dcbfa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca3cbfad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x80dcbfaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa57cbfb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdcbfb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x891cbfb6, SUC_lexicon_tags_2 },
    { 0x7ddcbfb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0dcbfb9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3c3cbfbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc3cbfbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fcbfc8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bcbfce, SUC_lexicon_tags_15 },
    { 0xa07cbfcf, SUC_lexicon_tags_14 },
    { 0x999cbfd0, SUC_lexicon_tags_9 },
    { 0x7c9cbfcf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbfd4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5cbfdb, SUC_lexicon_tags_27 },
    { 0xc9bcbfdc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cbfe2, SUC_lexicon_tags_27 },
    { 0x58dcbfe3, SUC_lexicon_tags_15 },
    { 0xda1cbfe4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5cbfe7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd29cbfe9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7cbff1, SUC_lexicon_tags_15 },
    { 0xc71cbff2, SUC_lexicon_tags_1 },
    { 0xcefcbff3, SUC_lexicon_tags_1 },
    { 0xa7fcbff4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5cbff9, SUC_lexicon_tags_37 },
    { 0x35cbffa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71cc004, SUC_lexicon_tags_30 },
    { 0xfdbcc004, SUC_lexicon_tags_9 },
    { 0xda3cc005, SUC_lexicon_tags_37 },
    { 0xcebcc006, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe1cc009, SUC_lexicon_tags_9 },
    { 0x8b1cc00a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bcc00d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fcc012, SUC_lexicon_tags_2 },
    { 0x3a5cc013, SUC_lexicon_tags_17 },
    { 0xdedcc014, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad9cc016, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dcc01e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913cc025, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3cc02b, SUC_lexicon_tags_0 },
    { 0xa67cc02c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x239cc02e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdcc034, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fcc037, SUC_lexicon_tags_49 },
    { 0x277cc038, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817cc03c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x949cc03e, SUC_lexicon_tags_14 },
    { 0x271cc03e, SUC_lexicon_tags_35 },
    { 0xa7dcc03f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9cc044, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fcc049, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x74dcc04c, SUC_lexicon_tags_27 },
    { 0x223cc04d, SUC_lexicon_tags_61 },
    { 0xdc7cc04d, SUC_lexicon_tags_33 },
    { 0xa33cc04d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd71cc051, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92dcc053, SUC_lexicon_tags_17 },
    { 0xeb7cc054, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7fbcc056, SUC_lexicon_tags_30 },
    { 0x187cc057, SUC_lexicon_tags_49 },
    { 0x86bcc058, SUC_lexicon_tags_66 },
    { 0xd5dcc057, SUC_lexicon_tags_37 },
    { 0x283cc056, SUC_lexicon_tags_7 },
    { 0xb87cc05b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fcc063, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1cc06a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3cc06f, SUC_lexicon_tags_39 },
    { 0xba5cc070, SUC_lexicon_tags_12 },
    { 0x789cc071, SUC_lexicon_tags_11 },
    { 0xf91cc06f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95cc075, SUC_lexicon_tags_17 },
    { 0xab7cc075, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7cc079, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95cc07e, SUC_lexicon_tags_49 },
    { 0x985cc07f, SUC_lexicon_tags_27 },
    { 0x833cc07e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x429cc083, SUC_lexicon_tags_38 },
    { 0xc81cc084, SUC_lexicon_tags_21 },
    { 0x541cc085, SUC_lexicon_tags_25 },
    { 0xb09cc085, SUC_lexicon_tags_41 },
    { 0x6cbcc083, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925cc08b, SUC_lexicon_tags_37 },
    { 0x7f5cc08c, SUC_lexicon_tags_141 },
    { 0x5e5cc08d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x99cc08f, SUC_lexicon_tags_17 },
    { 0xdabcc090, SUC_lexicon_tags_25 },
    { 0xa4fcc091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fcc099, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfcc09d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7cc0a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479cc0aa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf67cc0ac, SUC_lexicon_tags_17 },
    { 0xe99cc0ac, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bcc0b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761cc0b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b5cc0bb, SUC_lexicon_tags_25 },
    { 0x34fcc0bc, SUC_lexicon_tags_6 },
    { 0xbcdcc0bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf81cc0bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f3cc0c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8fdcc0c3, SUC_lexicon_tags_25 },
    { 0x995cc0c4, SUC_lexicon_tags_1 },
    { 0xe15cc0c4, SUC_lexicon_tags_37 },
    { 0x7b5cc0c3, SUC_lexicon_tags_19 },
    { 0x403cc0c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81cc0cc, SUC_lexicon_tags_12 },
    { 0xe81cc0cc, SUC_lexicon_tags_15 },
    { 0x63dcc0cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d3cc0d0, SUC_lexicon_tags_7 },
    { 0x16dcc0d1, SUC_lexicon_tags_28 },
    { 0x7c3cc0d2, SUC_lexicon_tags_87 },
    { 0x839cc0d2, SUC_lexicon_tags_65 },
    { 0x3e5cc0d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dcc0d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a5cc0db, SUC_lexicon_tags_11 },
    { 0x45dcc0db, SUC_lexicon_tags_12 },
    { 0x5dbcc0dd, SUC_lexicon_tags_17 },
    { 0xd0dcc0de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bcc0e2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x92bcc0e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9cc0e7, SUC_lexicon_tags_1 },
    { 0x51bcc0e8, SUC_lexicon_tags_47 },
    { 0x6fbcc0e9, SUC_lexicon_tags_23 },
    { 0xb0dcc0ea, SUC_lexicon_tags_11 },
    { 0x589cc0eb, SUC_lexicon_tags_14 },
    { 0xe19cc0eb, SUC_lexicon_tags_2 },
    { 0x961cc0ec, SUC_lexicon_tags_37 },
    { 0x1b5cc0ee, SUC_lexicon_tags_15 },
    { 0x5b7cc0ef, SUC_lexicon_tags_87 },
    { 0x60dcc0f0, SUC_lexicon_tags_32 },
    { 0x11bcc0f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79cc0f5, SUC_lexicon_tags_27 },
    { 0xff3cc0f6, SUC_lexicon_tags_43 },
    { 0x881cc0f6, SUC_lexicon_tags_2 },
    { 0x27dcc0f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcc0fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677cc102, SUC_lexicon_tags_124 },
    { 0x69dcc103, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645cc108, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf35cc10a, SUC_lexicon_tags_107 },
    { 0x9bfcc10a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd71cc10d, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79cc111, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c3cc113, SUC_lexicon_tags_60 },
    { 0x345cc113, SUC_lexicon_tags_25 },
    { 0xc8bcc114, SUC_lexicon_tags_15 },
    { 0x8a5cc116, SUC_lexicon_tags_25 },
    { 0x957cc113, SUC_lexicon_tags_85 },
    { 0xdfbcc113, SUC_lexicon_tags_25 },
    { 0x7f1cc119, SUC_lexicon_tags_15 },
    { 0xe55cc118, SUC_lexicon_tags_17 },
    { 0x403cc11b, SUC_lexicon_tags_14 },
    { 0xc27cc11c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dcc121, SUC_lexicon_tags_19 },
    { 0xeafcc122, SUC_lexicon_tags_17 },
    { 0x26dcc121, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x907cc125, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xafbcc127, SUC_lexicon_tags_1 },
    { 0xe7bcc128, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd99cc12a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993cc130, SUC_lexicon_tags_12 },
    { 0x37bcc130, SUC_lexicon_tags_79 },
    { 0xcefcc132, SUC_lexicon_tags_11 },
    { 0x1c1cc131, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dcc13a, SUC_lexicon_tags_29 },
    { 0x561cc13a, SUC_lexicon_tags_2 },
    { 0x87fcc13c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753cc140, SUC_lexicon_tags_45 },
    { 0x37fcc141, SUC_lexicon_tags_21 },
    { 0x4ffcc142, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955cc149, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fcc14c, SUC_lexicon_tags_37 },
    { 0xa47cc14d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505cc153, SUC_lexicon_tags_12 },
    { 0x67bcc154, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5cc158, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9cc15b, SUC_lexicon_tags_33 },
    { 0x8c9cc15b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b9cc15e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabcc164, SUC_lexicon_tags_47 },
    { 0x7b3cc165, SUC_lexicon_tags_2 },
    { 0xa69cc164, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127cc16a, SUC_lexicon_tags_10 },
    { 0x511cc16b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7cc16e, SUC_lexicon_tags_19 },
    { 0x913cc16f, SUC_lexicon_tags_82 },
    { 0x6ddcc170, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157cc177, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x257cc17a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46fcc17c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5fcc17e, SUC_lexicon_tags_62 },
    { 0xc39cc17f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1cc186, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd61cc188, SUC_lexicon_tags_0 },
    { 0x69bcc189, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e9cc18b, SUC_lexicon_tags_13 },
    { 0x333cc18c, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0x455cc18f, SUC_lexicon_tags_18 },
    { 0x1ebcc190, SUC_lexicon_tags_1 },
    { 0x4e9cc191, SUC_lexicon_tags_33 },
    { 0x865cc192, SUC_lexicon_tags_12 },
    { 0x357cc192, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e3cc195, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x243cc198, SUC_lexicon_tags_11 },
    { 0x27cc198, SUC_lexicon_tags_2 },
    { 0x291cc19a, SUC_lexicon_tags_2 },
    { 0x18bcc199, SUC_lexicon_tags_77 },
    { 0x423cc19c, SUC_lexicon_tags_1 },
    { 0x3cfcc19c, SUC_lexicon_tags_41 },
    { 0xbb1cc19e, SUC_lexicon_tags_141 },
    { 0x161cc19f, SUC_lexicon_tags_7 },
    { 0xdafcc19b, SUC_lexicon_tags_33 },
    { 0xbc5cc1a1, SUC_lexicon_tags_7 },
    { 0xe4dcc1a2, SUC_lexicon_tags_12 },
    { 0x5edcc1a3, SUC_lexicon_tags_34 },
    { 0x77fcc19f, SUC_lexicon_tags_11 },
    { 0xcc9cc1a5, SUC_lexicon_tags_13 },
    { 0xa73cc1a6, SUC_lexicon_tags_25 },
    { 0x2ebcc1a7, SUC_lexicon_tags_5 },
    { 0x43bcc1a7, SUC_lexicon_tags_63 },
    { 0xfdfcc1a9, SUC_lexicon_tags_12 },
    { 0x4ffcc1aa, SUC_lexicon_tags_17 },
    { 0xa8dcc1a8, SUC_lexicon_tags_15 },
    { 0xa0fcc1ac, SUC_lexicon_tags_6 },
    { 0xf31cc1ac, SUC_lexicon_tags_29 },
    { 0x5a3cc1ae, SUC_lexicon_tags_14 },
    { 0x5f9cc1ae, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdcc1b2, SUC_lexicon_tags_43 },
    { 0x6bdcc1b3, SUC_lexicon_tags_37 },
    { 0x711cc1b4, SUC_lexicon_tags_2 },
    { 0xfdfcc1b5, SUC_lexicon_tags_1 },
    { 0xc43cc1b3, SUC_lexicon_tags_11 },
    { 0xb61cc1b7, SUC_lexicon_tags_87 },
    { 0x153cc1b7, SUC_lexicon_tags_13 },
    { 0xa85cc1b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x47dcc1bb, SUC_lexicon_tags_11 },
    { 0x899cc1bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcc1c0, SUC_lexicon_tags_11 },
    { 0x995cc1c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad5cc1c3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x911cc1c6, SUC_lexicon_tags_17 },
    { 0x82dcc1c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3cc1cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5cc1cf, SUC_lexicon_tags_17 },
    { 0x217cc1d0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5cc1d5, SUC_lexicon_tags_26 },
    { 0x91bcc1d6, SUC_lexicon_tags_25 },
    { 0x94dcc1d6, SUC_lexicon_tags_17 },
    { 0x48fcc1d8, SUC_lexicon_tags_12 },
    { 0xf87cc1d8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271cc1de, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x593cc1e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3cc1e3, SUC_lexicon_tags_7 },
    { 0xe73cc1e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23cc1e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233cc1ed, SUC_lexicon_tags_10 },
    { 0x7e9cc1ed, SUC_lexicon_tags_11 },
    { 0xf13cc1ef, SUC_lexicon_tags_74 },
    { 0x191cc1f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfcc1f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5dcc1f5, SUC_lexicon_tags_23 },
    { 0x127cc1f5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3cc1fa, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9cc1ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x529cc201, SUC_lexicon_tags_37 },
    { 0xcf9cc202, SUC_lexicon_tags_91 },
    { 0x47bcc203, SUC_lexicon_tags_7 },
    { 0xecdcc202, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955cc209, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57dcc20b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cc20e, SUC_lexicon_tags_37 },
    { 0x1bfcc20f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5cc215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa69cc217, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x211cc219, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1fcc21b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdcc21e, SUC_lexicon_tags_21 },
    { 0x689cc21f, SUC_lexicon_tags_0 },
    { 0x857cc220, SUC_lexicon_tags_25 },
    { 0x159cc221, SUC_lexicon_tags_11 },
    { 0x519cc21e, SUC_lexicon_tags_19 },
    { 0xd19cc223, SUC_lexicon_tags_15 },
    { 0x64dcc223, SUC_lexicon_tags_37 },
    { 0xea5cc225, SUC_lexicon_tags_25 },
    { 0xc6dcc225, SUC_lexicon_tags_37 },
    { 0xd27cc227, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a7cc229, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5cc22d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x139cc22f, SUC_lexicon_tags_25 },
    { 0x933cc22f, SUC_lexicon_tags_11 },
    { 0x1a3cc231, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1f7cc233, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb17cc236, SUC_lexicon_tags_2 },
    { 0xfdfcc237, SUC_lexicon_tags_15 },
    { 0xde1cc238, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x57fcc23a, SUC_lexicon_tags_15 },
    { 0x467cc23a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fcc23e, SUC_lexicon_tags_31 },
    { 0x7abcc23f, SUC_lexicon_tags_5 },
    { 0xcfcc23f, SUC_lexicon_tags_12 },
    { 0x6cfcc241, SUC_lexicon_tags_17 },
    { 0x97dcc241, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dcc246, SUC_lexicon_tags_2 },
    { 0x457cc246, SUC_lexicon_tags_31 },
    { 0xb75cc248, SUC_lexicon_tags_2 },
    { 0xbfdcc249, SUC_lexicon_tags_11 },
    { 0x43cc246, SUC_lexicon_tags_46 },
    { 0x509cc247, SUC_lexicon_tags_19 },
    { 0x52fcc24c, SUC_lexicon_tags_15 },
    { 0xaa5cc24a, SUC_lexicon_tags_27 },
    { 0xfdfcc249, SUC_lexicon_tags_11 },
    { 0x5f1cc24f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3cc258, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5cc25d, SUC_lexicon_tags_7 },
    { 0x5fdcc25e, SUC_lexicon_tags_31 },
    { 0x28fcc25d, SUC_lexicon_tags_38 },
    { 0x8ffcc260, SUC_lexicon_tags_7 },
    { 0x1e5cc25d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x687cc263, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15cc269, SUC_lexicon_tags_27 },
    { 0xc6bcc26a, SUC_lexicon_tags_25 },
    { 0x827cc26b, SUC_lexicon_tags_29 },
    { 0x95dcc26c, SUC_lexicon_tags_47 },
    { 0xae1cc26d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27dcc26f, SUC_lexicon_tags_6 },
    { 0xc77cc270, SUC_lexicon_tags_17 },
    { 0xdadcc270, SUC_lexicon_tags_33 },
    { 0xf3dcc270, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3cc276, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdcc27b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fcc280, SUC_lexicon_tags_1 },
    { 0x11fcc281, SUC_lexicon_tags_37 },
    { 0x163cc281, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x12fcc284, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dcc28f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9cc293, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x125cc295, SUC_lexicon_tags_33 },
    { 0x46fcc296, SUC_lexicon_tags_7 },
    { 0xcb5cc295, SUC_lexicon_tags_17 },
    { 0xa5dcc298, SUC_lexicon_tags_11 },
    { 0x9bfcc298, SUC_lexicon_tags_17 },
    { 0x9d3cc29a, SUC_lexicon_tags_17 },
    { 0x627cc29b, SUC_lexicon_tags_34 },
    { 0xbc1cc29c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bcc2a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3cc2a3, SUC_lexicon_tags_13 },
    { 0xae5cc2a4, SUC_lexicon_tags_27 },
    { 0xe4fcc2a3, SUC_lexicon_tags_7 },
    { 0xe4bcc2a6, SUC_lexicon_tags_11 },
    { 0x6a3cc2a3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x83fcc2aa, SUC_lexicon_tags_37 },
    { 0x2d7cc2ab, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1cc2b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609cc2b4, SUC_lexicon_tags_7 },
    { 0x3e5cc2b5, SUC_lexicon_tags_37 },
    { 0x629cc2b4, SUC_lexicon_tags_14 },
    { 0x213cc2b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbcc2ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x179cc2bd, SUC_lexicon_tags_63 },
    { 0xabbcc2bd, SUC_lexicon_tags_18 },
    { 0x7dcc2bd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fcc2c3, SUC_lexicon_tags_3 },
    { 0x48fcc2c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x223cc2c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffcc2cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe55cc2ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9cc2d1, SUC_lexicon_tags_5 },
    { 0x79cc2d2, SUC_lexicon_tags_45 },
    { 0xde1cc2d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcc2d7, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5cc2db, SUC_lexicon_tags_3 },
    { 0x201cc2dc, SUC_lexicon_tags_3 },
    { 0x92bcc2dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abcc2e0, SUC_lexicon_tags_66 },
    { 0x331cc2e1, SUC_lexicon_tags_28 },
    { 0xd37cc2e2, SUC_lexicon_tags_25 },
    { 0x43fcc2e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfcc2e8, SUC_lexicon_tags_3 },
    { 0xe77cc2e9, SUC_lexicon_tags_15 },
    { 0xf57cc2e9, SUC_lexicon_tags_39 },
    { 0x46dcc2eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bcc2f0, SUC_lexicon_tags_17 },
    { 0x6c3cc2f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x729cc2f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bcc2f7, SUC_lexicon_tags_25 },
    { 0x647cc2f8, SUC_lexicon_tags_2 },
    { 0xf97cc2f9, SUC_lexicon_tags_10 },
    { 0x9dbcc2f8, SUC_lexicon_tags_25 },
    { 0x86dcc2fb, SUC_lexicon_tags_85 },
    { 0xc69cc2fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45cc302, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5cc308, SUC_lexicon_tags_37 },
    { 0xbafcc309, SUC_lexicon_tags_26 },
    { 0xc9bcc30a, SUC_lexicon_tags_66 },
    { 0x205cc30a, SUC_lexicon_tags_25 },
    { 0xf49cc30a, SUC_lexicon_tags_14 },
    { 0x4f9cc30d, SUC_lexicon_tags_11 },
    { 0x4dcc30e, SUC_lexicon_tags_27 },
    { 0x413cc30f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9cc315, SUC_lexicon_tags_28 },
    { 0xe41cc316, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adcc31a, SUC_lexicon_tags_25 },
    { 0x3a1cc31b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99cc320, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x849cc322, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffcc327, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991cc32e, SUC_lexicon_tags_43 },
    { 0x701cc32f, SUC_lexicon_tags_1 },
    { 0x513cc330, SUC_lexicon_tags_21 },
    { 0xf7cc330, SUC_lexicon_tags_25 },
    { 0x54bcc32f, SUC_lexicon_tags_49 },
    { 0xacdcc333, SUC_lexicon_tags_37 },
    { 0xc5bcc331, SUC_lexicon_tags_2 },
    { 0x70dcc333, SUC_lexicon_tags_33 },
    { 0x9bbcc336, SUC_lexicon_tags_37 },
    { 0xd73cc336, SUC_lexicon_tags_11 },
    { 0xb0fcc335, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fcc33b, SUC_lexicon_tags_1 },
    { 0x871cc33b, SUC_lexicon_tags_31 },
    { 0x42dcc33c, SUC_lexicon_tags_19 },
    { 0x6d5cc33e, SUC_lexicon_tags_27 },
    { 0x93cc33e, SUC_lexicon_tags_4 },
    { 0x67cc340, SUC_lexicon_tags_66 },
    { 0x317cc340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fcc343, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x699cc345, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cc349, SUC_lexicon_tags_65 },
    { 0xeabcc349, SUC_lexicon_tags_31 },
    { 0xa35cc349, SUC_lexicon_tags_2 },
    { 0xeebcc349, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03cc350, SUC_lexicon_tags_47 },
    { 0x3fcc351, SUC_lexicon_tags_106 },
    { 0x62dcc351, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dcc356, SUC_lexicon_tags_23 },
    { 0x7c3cc357, SUC_lexicon_tags_47 },
    { 0xc51cc357, SUC_lexicon_tags_25 },
    { 0xd0bcc359, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbcc35e, SUC_lexicon_tags_37 },
    { 0x9f7cc35f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9cc362, SUC_lexicon_tags_11 },
    { 0xf4dcc363, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x631cc365, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7cfcc367, SUC_lexicon_tags_74 },
    { 0xb53cc367, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5cc372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65cc379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x699cc37b, SUC_lexicon_tags_46 },
    { 0x2c9cc37b, SUC_lexicon_tags_15 },
    { 0x4bfcc37d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11cc382, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dcc386, SUC_lexicon_tags_68 },
    { 0x311cc387, SUC_lexicon_tags_30 },
    { 0xbf9cc388, SUC_lexicon_tags_1 },
    { 0x6e3cc389, SUC_lexicon_tags_30 },
    { 0x50dcc389, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x909cc38c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fcc38f, SUC_lexicon_tags_21 },
    { 0x1cc38f, SUC_lexicon_tags_16 },
    { 0x3dfcc391, SUC_lexicon_tags_2 },
    { 0x7b5cc392, SUC_lexicon_tags_37 },
    { 0x129cc38f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xebcc396, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803cc39b, SUC_lexicon_tags_19 },
    { 0x477cc39c, SUC_lexicon_tags_11 },
    { 0x789cc39c, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1cc3a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41fcc3a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bcc3a6, SUC_lexicon_tags_71 },
    { 0x82fcc3a6, SUC_lexicon_tags_27 },
    { 0xc43cc3a8, SUC_lexicon_tags_2 },
    { 0xcfdcc3a9, SUC_lexicon_tags_66 },
    { 0x633cc3aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x38fcc3ac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73cc3b5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559cc3b9, SUC_lexicon_tags_15 },
    { 0xa47cc3b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161cc3c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dcc3c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfefcc3c7, SUC_lexicon_tags_11 },
    { 0x28dcc3c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa5fcc3ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd47cc3cc, SUC_lexicon_tags_87 },
    { 0x633cc3cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda1cc3cf, SUC_lexicon_tags_30 },
    { 0xd8fcc3cf, SUC_lexicon_tags_32 },
    { 0xb97cc3d1, SUC_lexicon_tags_21 },
    { 0x2d3cc3d0, SUC_lexicon_tags_37 },
    { 0x7adcc3d3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bcc3dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x967cc3de, SUC_lexicon_tags_11 },
    { 0x95cc3de, SUC_lexicon_tags_15 },
    { 0x6a9cc3e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573cc3e6, SUC_lexicon_tags_19 },
    { 0x621cc3e7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91cc3ee, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bcc3f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fcc3f4, SUC_lexicon_tags_34 },
    { 0x6f3cc3f5, SUC_lexicon_tags_14 },
    { 0xb1bcc3f4, SUC_lexicon_tags_296 },
    { 0xa3bcc3f4, SUC_lexicon_tags_14 },
    { 0x863cc3f5, SUC_lexicon_tags_37 },
    { 0xdcdcc3f4, SUC_lexicon_tags_11 },
    { 0xab7cc3fa, SUC_lexicon_tags_1 },
    { 0xb21cc3fb, SUC_lexicon_tags_11 },
    { 0xd19cc3fc, SUC_lexicon_tags_46 },
    { 0x409cc3fd, SUC_lexicon_tags_17 },
    { 0xd8dcc3fe, SUC_lexicon_tags_37 },
    { 0x213cc3fd, SUC_lexicon_tags_30 },
    { 0x18fcc3fc, SUC_lexicon_tags_25 },
    { 0x911cc3fe, SUC_lexicon_tags_43 },
    { 0xe9bcc3fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa01cc404, SUC_lexicon_tags_69 },
    { 0x387cc405, SUC_lexicon_tags_1 },
    { 0x1b3cc404, SUC_lexicon_tags_2 },
    { 0xb5cc407, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x341cc409, SUC_lexicon_tags_38 },
    { 0x5bdcc40a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7dcc40c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dcc416, SUC_lexicon_tags_7 },
    { 0x9d7cc417, SUC_lexicon_tags_15 },
    { 0x56dcc417, SUC_lexicon_tags_48 },
    { 0x647cc419, SUC_lexicon_tags_15 },
    { 0x83cc419, SUC_lexicon_tags_23 },
    { 0x685cc41b, SUC_lexicon_tags_41 },
    { 0x2bcc41c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x975cc41e, SUC_lexicon_tags_17 },
    { 0x1cc41e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651cc424, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1cc42a, SUC_lexicon_tags_33 },
    { 0x885cc42b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd55cc42d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c1cc42f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99dcc431, SUC_lexicon_tags_10 },
    { 0x1dfcc432, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bcc436, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681cc43a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01cc441, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x991cc443, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8ddcc445, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221cc449, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55cc44d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x285cc44f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bcc453, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cc457, SUC_lexicon_tags_16 },
    { 0xdabcc458, SUC_lexicon_tags_11 },
    { 0x761cc459, SUC_lexicon_tags_43 },
    { 0x2a9cc45a, SUC_lexicon_tags_16 },
    { 0x743cc45b, SUC_lexicon_tags_11 },
    { 0x983cc45c, SUC_lexicon_tags_37 },
    { 0xcc5cc45d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287cc46e, SUC_lexicon_tags_25 },
    { 0x31bcc46f, SUC_lexicon_tags_7 },
    { 0xd97cc470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945cc474, SUC_lexicon_tags_55 },
    { 0xc15cc474, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fcc481, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273cc485, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfcc488, SUC_lexicon_tags_11 },
    { 0xfbfcc489, SUC_lexicon_tags_60 },
    { 0x9c7cc489, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5cc48d, SUC_lexicon_tags_66 },
    { 0x705cc48e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7cc492, SUC_lexicon_tags_1494 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dcc496, SUC_lexicon_tags_18 },
    { 0xefcc497, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5cc49a, SUC_lexicon_tags_148 },
    { 0xe2bcc49a, SUC_lexicon_tags_2 },
    { 0x9f1cc49a, SUC_lexicon_tags_8 },
    { 0xdcc49d, SUC_lexicon_tags_25 },
    { 0x7c9cc49e, SUC_lexicon_tags_28 },
    { 0x67cc49f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7cc4a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x611cc4a4, SUC_lexicon_tags_15 },
    { 0xd4fcc4a5, SUC_lexicon_tags_2 },
    { 0x789cc4a4, SUC_lexicon_tags_87 },
    { 0xe11cc4a5, SUC_lexicon_tags_17 },
    { 0x995cc4a5, SUC_lexicon_tags_25 },
    { 0x3b3cc4a8, SUC_lexicon_tags_37 },
    { 0xd8fcc4aa, SUC_lexicon_tags_2 },
    { 0x90dcc4a7, SUC_lexicon_tags_12 },
    { 0x36bcc4ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24bcc4af, SUC_lexicon_tags_56 },
    { 0x7c3cc4af, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1cc4b3, SUC_lexicon_tags_25 },
    { 0x437cc4b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287cc4bd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xac3cc4bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x679cc4c2, SUC_lexicon_tags_37 },
    { 0xa41cc4c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x881cc4c6, SUC_lexicon_tags_287 },
    { 0x247cc4c7, SUC_lexicon_tags_17 },
    { 0xa95cc4c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf5cc4ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49cc4cf, SUC_lexicon_tags_47 },
    { 0x519cc4d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2bcc4d2, SUC_lexicon_tags_11 },
    { 0x59fcc4d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x19bcc4d6, SUC_lexicon_tags_55 },
    { 0x925cc4d7, SUC_lexicon_tags_201 },
    { 0xff9cc4d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59cc4db, SUC_lexicon_tags_2 },
    { 0xa81cc4db, SUC_lexicon_tags_7 },
    { 0xae5cc4dd, SUC_lexicon_tags_63 },
    { 0x8e7cc4dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcc4e3, SUC_lexicon_tags_12 },
    { 0x19dcc4e3, SUC_lexicon_tags_84 },
    { 0x33bcc4e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdcc4ee, SUC_lexicon_tags_45 },
    { 0xb99cc4ef, SUC_lexicon_tags_84 },
    { 0x58dcc4f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x963cc4f3, SUC_lexicon_tags_1 },
    { 0xa63cc4f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffcc4f9, SUC_lexicon_tags_47 },
    { 0x39bcc4fa, SUC_lexicon_tags_7 },
    { 0x313cc4fb, SUC_lexicon_tags_37 },
    { 0x717cc4fa, SUC_lexicon_tags_27 },
    { 0xa77cc4fa, SUC_lexicon_tags_7 },
    { 0x61dcc4fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdcc504, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45cc50a, SUC_lexicon_tags_49 },
    { 0xb1dcc50a, SUC_lexicon_tags_293 },
    { 0x38fcc50c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe7bcc50e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3c1cc510, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39cc514, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x29fcc517, SUC_lexicon_tags_1 },
    { 0xd1cc517, SUC_lexicon_tags_15 },
    { 0x9d1cc517, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdcc51e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1cc528, SUC_lexicon_tags_65 },
    { 0xf03cc529, SUC_lexicon_tags_11 },
    { 0x30bcc52a, SUC_lexicon_tags_25 },
    { 0x98bcc52b, SUC_lexicon_tags_9 },
    { 0x957cc52b, SUC_lexicon_tags_299 },
    { 0x909cc52d, SUC_lexicon_tags_1 },
    { 0xd15cc52a, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0xa43cc530, SUC_lexicon_tags_25 },
    { 0x361cc530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x745cc533, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x297cc535, SUC_lexicon_tags_57 },
    { 0xf97cc536, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xea9cc538, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107cc53c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85dcc53e, SUC_lexicon_tags_31 },
    { 0x1afcc53f, SUC_lexicon_tags_10 },
    { 0xc7fcc53f, SUC_lexicon_tags_179 },
    { 0x1cc53e, SUC_lexicon_tags_0 },
    { 0xe3bcc542, SUC_lexicon_tags_63 },
    { 0x9b7cc543, SUC_lexicon_tags_14 },
    { 0xaa1cc542, SUC_lexicon_tags_63 },
    { 0x7c9cc544, SUC_lexicon_tags_8 },
    { 0xdadcc545, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa87cc548, SUC_lexicon_tags_45 },
    { 0x51dcc548, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175cc54d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa27cc54f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85cc555, SUC_lexicon_tags_43 },
    { 0xc37cc555, SUC_lexicon_tags_80 },
    { 0xb8fcc557, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcc55a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671cc55e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dcc562, SUC_lexicon_tags_47 },
    { 0x737cc563, SUC_lexicon_tags_63 },
    { 0x6fbcc562, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1cc567, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd03cc569, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11cc56c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239cc570, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x71cc572, SUC_lexicon_tags_6 },
    { 0x7dbcc573, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x469cc575, SUC_lexicon_tags_0 },
    { 0x775cc576, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x119cc578, SUC_lexicon_tags_17 },
    { 0xebdcc579, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x439cc57c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x391cc57e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bcc581, SUC_lexicon_tags_252 },
    { 0x387cc582, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91cc587, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fcc58f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x385cc591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x435cc593, SUC_lexicon_tags_171 },
    { 0xc91cc593, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbcc599, SUC_lexicon_tags_37 },
    { 0x70bcc59a, SUC_lexicon_tags_11 },
    { 0x3cbcc59b, SUC_lexicon_tags_47 },
    { 0x5ffcc59c, SUC_lexicon_tags_7 },
    { 0xf91cc59d, SUC_lexicon_tags_11 },
    { 0xaffcc59a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x20fcc5a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afcc5aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5cc5af, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539cc5b8, SUC_lexicon_tags_19 },
    { 0xafcc5b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbdbcc5bb, SUC_lexicon_tags_11 },
    { 0x43bcc5bb, SUC_lexicon_tags_37 },
    { 0x281cc5bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe17cc5bf, SUC_lexicon_tags_7 },
    { 0x745cc5bf, SUC_lexicon_tags_2 },
    { 0xab3cc5c1, SUC_lexicon_tags_45 },
    { 0xac3cc5c2, SUC_lexicon_tags_37 },
    { 0x7cbcc5c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xefcc5c5, SUC_lexicon_tags_0 },
    { 0xad1cc5c5, SUC_lexicon_tags_10 },
    { 0x6f1cc5c7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa1fcc5c9, SUC_lexicon_tags_84 },
    { 0x4f7cc5ca, SUC_lexicon_tags_28 },
    { 0x47fcc5c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8afcc5cd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x415cc5d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995cc5d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdcc5db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x853cc5de, SUC_lexicon_tags_37 },
    { 0xbfdcc5df, SUC_lexicon_tags_65 },
    { 0x405cc5df, SUC_lexicon_tags_29 },
    { 0x7b5cc5e1, SUC_lexicon_tags_2 },
    { 0x181cc5de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb3cc5e4, SUC_lexicon_tags_85 },
    { 0x9e3cc5e5, SUC_lexicon_tags_24 },
    { 0xe61cc5e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7cc5e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x60dcc5ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143cc5f0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7f7cc5f2, SUC_lexicon_tags_55 },
    { 0x411cc5f3, SUC_lexicon_tags_1 },
    { 0xad1cc5f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fcc5f7, SUC_lexicon_tags_9 },
    { 0x83cc5f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1cc5fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdcc602, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8fdcc604, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3cc608, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f5cc60a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733cc60f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9cc611, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dcc616, SUC_lexicon_tags_42 },
    { 0x1c7cc616, SUC_lexicon_tags_2 },
    { 0x6abcc618, SUC_lexicon_tags_27 },
    { 0x27dcc619, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bcc61f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x359cc622, SUC_lexicon_tags_11 },
    { 0xbd5cc623, SUC_lexicon_tags_19 },
    { 0x79cc624, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9bdcc626, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xf59cc628, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bcc62c, SUC_lexicon_tags_49 },
    { 0x599cc62d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5cc633, SUC_lexicon_tags_60 },
    { 0x96bcc634, SUC_lexicon_tags_108 },
    { 0xaf3cc633, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7cc63c, SUC_lexicon_tags_65 },
    { 0xc5fcc63d, SUC_lexicon_tags_25 },
    { 0x35fcc63e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa8dcc640, SUC_lexicon_tags_3 },
    { 0xd99cc640, SUC_lexicon_tags_37 },
    { 0x6e9cc640, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dcc645, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdcc648, SUC_lexicon_tags_25 },
    { 0x68bcc649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7cc64d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x115cc650, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x727cc652, SUC_lexicon_tags_12 },
    { 0xe5fcc653, SUC_lexicon_tags_43 },
    { 0xcb9cc653, SUC_lexicon_tags_25 },
    { 0x90dcc655, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3cc65a, SUC_lexicon_tags_2 },
    { 0xb91cc65b, SUC_lexicon_tags_29 },
    { 0xc73cc65c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95cc660, SUC_lexicon_tags_12 },
    { 0x79dcc660, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x109cc663, SUC_lexicon_tags_60 },
    { 0xf69cc664, SUC_lexicon_tags_25 },
    { 0xb7fcc664, SUC_lexicon_tags_14 },
    { 0x895cc666, SUC_lexicon_tags_11 },
    { 0xa9fcc667, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3cc66a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58fcc66c, SUC_lexicon_tags_39 },
    { 0x119cc66c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fcc670, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x1cc672, SUC_lexicon_tags_60 },
    { 0xafbcc673, SUC_lexicon_tags_17 },
    { 0x1fbcc674, SUC_lexicon_tags_1 },
    { 0x92bcc673, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9cc677, SUC_lexicon_tags_12 },
    { 0xc09cc677, SUC_lexicon_tags_29 },
    { 0x4e1cc677, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xca5cc67b, SUC_lexicon_tags_63 },
    { 0x875cc67b, SUC_lexicon_tags_17 },
    { 0xd65cc67d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x483cc67f, SUC_lexicon_tags_22 },
    { 0xdd5cc67f, SUC_lexicon_tags_3 },
    { 0x675cc681, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839cc68a, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbcc68e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe7cc690, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5cc697, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x697cc699, SUC_lexicon_tags_11 },
    { 0xe0dcc699, SUC_lexicon_tags_15 },
    { 0x83cc69b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfcc6a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe95cc6a2, SUC_lexicon_tags_1 },
    { 0x835cc6a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x78bcc6a5, SUC_lexicon_tags_60 },
    { 0xc4dcc6a6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf21cc6a8, SUC_lexicon_tags_1 },
    { 0x6c9cc6a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185cc6ae, SUC_lexicon_tags_46 },
    { 0xf47cc6af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5cc6b2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9cdcc6b4, SUC_lexicon_tags_16 },
    { 0xe13cc6b4, SUC_lexicon_tags_37 },
    { 0x9fdcc6b6, SUC_lexicon_tags_30 },
    { 0xf75cc6b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x77fcc6b9, SUC_lexicon_tags_1 },
    { 0xeabcc6ba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9cc6bf, SUC_lexicon_tags_12 },
    { 0xb1dcc6bf, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9cc6c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90bcc6c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1cc6c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9cc6cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd93cc6ce, SUC_lexicon_tags_17 },
    { 0xe15cc6ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fcc6d4, SUC_lexicon_tags_2 },
    { 0x237cc6d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09cc6d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61cc6dd, SUC_lexicon_tags_1 },
    { 0x6a7cc6de, SUC_lexicon_tags_2 },
    { 0x5c1cc6de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2bcc6e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe81cc6e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9cc6e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bcc6ea, SUC_lexicon_tags_2 },
    { 0x87cc6eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41cc6f0, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1cc6f3, SUC_lexicon_tags_45 },
    { 0x6a1cc6f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x899cc6f6, SUC_lexicon_tags_10 },
    { 0xbe5cc6f7, SUC_lexicon_tags_47 },
    { 0x11bcc6f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9fcc6fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdcc6fe, SUC_lexicon_tags_26 },
    { 0x48fcc6fe, SUC_lexicon_tags_23 },
    { 0xa6fcc700, SUC_lexicon_tags_43 },
    { 0x979cc6ff, SUC_lexicon_tags_12 },
    { 0x8efcc702, SUC_lexicon_tags_25 },
    { 0xa27cc703, SUC_lexicon_tags_141 },
    { 0xa5fcc6ff, SUC_lexicon_tags_44 },
    { 0x34fcc705, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5cc70b, SUC_lexicon_tags_42 },
    { 0x207cc70b, SUC_lexicon_tags_30 },
    { 0x63fcc70d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79cc710, SUC_lexicon_tags_82 },
    { 0x3b7cc711, SUC_lexicon_tags_12 },
    { 0x8a1cc711, SUC_lexicon_tags_21 },
    { 0xb49cc712, SUC_lexicon_tags_65 },
    { 0xbe5cc714, SUC_lexicon_tags_11 },
    { 0x5bfcc713, SUC_lexicon_tags_7 },
    { 0x68bcc710, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cc720, SUC_lexicon_tags_33 },
    { 0x90dcc720, SUC_lexicon_tags_10 },
    { 0x17cc720, SUC_lexicon_tags_79 },
    { 0xa81cc723, SUC_lexicon_tags_19 },
    { 0xd45cc724, SUC_lexicon_tags_60 },
    { 0x111cc723, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa9dcc727, SUC_lexicon_tags_0 },
    { 0xb35cc728, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x57cc72a, SUC_lexicon_tags_16 },
    { 0x25cc72b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19cc72f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x75fcc731, SUC_lexicon_tags_1 },
    { 0x42fcc732, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x949cc735, SUC_lexicon_tags_454 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807cc73b, SUC_lexicon_tags_0 },
    { 0xfc9cc73b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2dcc73e, SUC_lexicon_tags_17 },
    { 0x9f3cc73e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcbcc741, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7cc745, SUC_lexicon_tags_29 },
    { 0x6bbcc746, SUC_lexicon_tags_16 },
    { 0x1b3cc746, SUC_lexicon_tags_65 },
    { 0xdbbcc746, SUC_lexicon_tags_31 },
    { 0xc43cc749, SUC_lexicon_tags_15 },
    { 0x9e7cc749, SUC_lexicon_tags_39 },
    { 0x179cc749, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bcc74e, SUC_lexicon_tags_38 },
    { 0x71dcc74e, SUC_lexicon_tags_33 },
    { 0x67cc750, SUC_lexicon_tags_2 },
    { 0xba3cc751, SUC_lexicon_tags_40 },
    { 0x995cc752, SUC_lexicon_tags_27 },
    { 0x2e1cc753, SUC_lexicon_tags_1 },
    { 0xd33cc754, SUC_lexicon_tags_8 },
    { 0xfc1cc755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bcc759, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abcc75d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa51cc75f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55cc762, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x80fcc764, SUC_lexicon_tags_15 },
    { 0x63cc764, SUC_lexicon_tags_343 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fcc768, SUC_lexicon_tags_23 },
    { 0xb17cc768, SUC_lexicon_tags_25 },
    { 0x743cc76a, SUC_lexicon_tags_25 },
    { 0x64dcc76b, SUC_lexicon_tags_38 },
    { 0x1fbcc768, SUC_lexicon_tags_47 },
    { 0xeabcc76d, SUC_lexicon_tags_25 },
    { 0xdb7cc76e, SUC_lexicon_tags_23 },
    { 0xb97cc76f, SUC_lexicon_tags_37 },
    { 0x963cc770, SUC_lexicon_tags_32 },
    { 0x397cc770, SUC_lexicon_tags_61 },
    { 0x583cc76b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9cc776, SUC_lexicon_tags_23 },
    { 0xc03cc777, SUC_lexicon_tags_38 },
    { 0xfb7cc777, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xacfcc77a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc85cc77c, SUC_lexicon_tags_25 },
    { 0x25fcc77c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dcc782, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fcc786, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x511cc788, SUC_lexicon_tags_37 },
    { 0xf0bcc789, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3cc78c, SUC_lexicon_tags_25 },
    { 0x7fbcc78d, SUC_lexicon_tags_0 },
    { 0xbd9cc78d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8fcc790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x65fcc792, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdcc797, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afcc7a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bcc7a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dcc7af, SUC_lexicon_tags_13 },
    { 0x227cc7af, SUC_lexicon_tags_28 },
    { 0xcc9cc7b0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13cc7b9, SUC_lexicon_tags_49 },
    { 0xd03cc7b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e5cc7bc, SUC_lexicon_tags_45 },
    { 0x8dcc7bc, SUC_lexicon_tags_2 },
    { 0x6fdcc7be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bcc7c4, SUC_lexicon_tags_17 },
    { 0xd37cc7c5, SUC_lexicon_tags_49 },
    { 0x7f1cc7c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x173cc7c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8dcc7ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7cc7cd, SUC_lexicon_tags_4 },
    { 0x8ddcc7ce, SUC_lexicon_tags_17 },
    { 0x1b3cc7cd, SUC_lexicon_tags_12 },
    { 0x817cc7d0, SUC_lexicon_tags_47 },
    { 0x861cc7d0, SUC_lexicon_tags_65 },
    { 0x435cc7d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x137cc7d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229cc7db, SUC_lexicon_tags_25 },
    { 0x141cc7db, SUC_lexicon_tags_25 },
    { 0xf79cc7dd, SUC_lexicon_tags_15 },
    { 0xa75cc7dd, SUC_lexicon_tags_11 },
    { 0x7efcc7df, SUC_lexicon_tags_0 },
    { 0x897cc7de, SUC_lexicon_tags_17 },
    { 0x737cc7e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19cc7e4, SUC_lexicon_tags_63 },
    { 0xab9cc7e5, SUC_lexicon_tags_2 },
    { 0x3efcc7e6, SUC_lexicon_tags_2 },
    { 0x87fcc7e6, SUC_lexicon_tags_15 },
    { 0x689cc7e4, SUC_lexicon_tags_2 },
    { 0x3ffcc7e9, SUC_lexicon_tags_25 },
    { 0x28bcc7ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb85cc7ec, SUC_lexicon_tags_10 },
    { 0x36bcc7ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fcc7f1, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xdcc7f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421cc7fb, SUC_lexicon_tags_37 },
    { 0xe0dcc7fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845cc800, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x841cc802, SUC_lexicon_tags_37 },
    { 0x13cc803, SUC_lexicon_tags_60 },
    { 0xcd3cc803, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebcc808, SUC_lexicon_tags_59 },
    { 0x1f1cc809, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1cc80c, SUC_lexicon_tags_49 },
    { 0x33fcc80d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733cc813, SUC_lexicon_tags_17 },
    { 0xe15cc813, SUC_lexicon_tags_27 },
    { 0xd61cc815, SUC_lexicon_tags_1 },
    { 0xc9bcc816, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87dcc818, SUC_lexicon_tags_17 },
    { 0xf7fcc818, SUC_lexicon_tags_204 },
    { 0x40fcc818, SUC_lexicon_tags_25 },
    { 0x6e5cc81b, SUC_lexicon_tags_32 },
    { 0x46fcc81c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3bcc81e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x397cc821, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1cc827, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3cc82a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x537cc82c, SUC_lexicon_tags_2 },
    { 0x40bcc82c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165cc832, SUC_lexicon_tags_37 },
    { 0x983cc832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x16fcc835, SUC_lexicon_tags_47 },
    { 0xd03cc835, SUC_lexicon_tags_8 },
    { 0x7cfcc835, SUC_lexicon_tags_64 },
    { 0x6a7cc835, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bcc83b, SUC_lexicon_tags_49 },
    { 0xc1dcc83c, SUC_lexicon_tags_43 },
    { 0x84bcc83c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbcc840, SUC_lexicon_tags_65 },
    { 0x65dcc840, SUC_lexicon_tags_19 },
    { 0x5a5cc842, SUC_lexicon_tags_12 },
    { 0x847cc841, SUC_lexicon_tags_61 },
    { 0x855cc841, SUC_lexicon_tags_63 },
    { 0x165cc843, SUC_lexicon_tags_235 },
    { 0xe4fcc846, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dcc84a, SUC_lexicon_tags_66 },
    { 0x6b7cc84a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1cc84e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5cc851, SUC_lexicon_tags_87 },
    { 0x407cc852, SUC_lexicon_tags_160 },
    { 0xe31cc852, SUC_lexicon_tags_33 },
    { 0x7d7cc854, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dcc858, SUC_lexicon_tags_6 },
    { 0xed1cc859, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafcc85c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7cc865, SUC_lexicon_tags_15 },
    { 0xd1dcc866, SUC_lexicon_tags_11 },
    { 0xaf7cc865, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x94dcc869, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45cc871, SUC_lexicon_tags_30 },
    { 0x765cc872, SUC_lexicon_tags_8 },
    { 0x351cc873, SUC_lexicon_tags_35 },
    { 0xcfcc873, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5cc877, SUC_lexicon_tags_2 },
    { 0x9e5cc877, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43cc87d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05cc880, SUC_lexicon_tags_25 },
    { 0x213cc881, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x91bcc883, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3cc888, SUC_lexicon_tags_8 },
    { 0x4a1cc889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddcc88c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bcc890, SUC_lexicon_tags_34 },
    { 0x483cc891, SUC_lexicon_tags_49 },
    { 0x271cc892, SUC_lexicon_tags_0 },
    { 0x60fcc893, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3cc89d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815cc8a2, SUC_lexicon_tags_45 },
    { 0x5e5cc8a2, SUC_lexicon_tags_95 },
    { 0x24fcc8a4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9cc8ab, SUC_lexicon_tags_98 },
    { 0xebcc8ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bcc8b1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11cc8b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5cc8b9, SUC_lexicon_tags_17 },
    { 0x9efcc8b9, SUC_lexicon_tags_15 },
    { 0x52dcc8bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbcc8be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc93cc8c0, SUC_lexicon_tags_63 },
    { 0x781cc8c1, SUC_lexicon_tags_43 },
    { 0xb97cc8c1, SUC_lexicon_tags_1 },
    { 0xbbcc8c2, SUC_lexicon_tags_116 },
    { 0xa67cc8c4, SUC_lexicon_tags_7 },
    { 0x653cc8c2, SUC_lexicon_tags_12 },
    { 0x2a1cc8c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03cc8ca, SUC_lexicon_tags_37 },
    { 0xfbdcc8cb, SUC_lexicon_tags_14 },
    { 0xb37cc8cc, SUC_lexicon_tags_17 },
    { 0xe83cc8cc, SUC_lexicon_tags_10 },
    { 0xc8dcc8cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfcc8d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87cc8d8, SUC_lexicon_tags_1 },
    { 0xf05cc8d9, SUC_lexicon_tags_35 },
    { 0xc23cc8da, SUC_lexicon_tags_37 },
    { 0x5a5cc8d8, SUC_lexicon_tags_17 },
    { 0xaebcc8d8, SUC_lexicon_tags_47 },
    { 0x65cc8dc, SUC_lexicon_tags_9 },
    { 0x145cc8de, SUC_lexicon_tags_1 },
    { 0xbcbcc8de, SUC_lexicon_tags_27 },
    { 0xaf5cc8e0, SUC_lexicon_tags_32 },
    { 0x2d3cc8df, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x439cc8e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63fcc8e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcc8e8, SUC_lexicon_tags_11 },
    { 0x3b9cc8e9, SUC_lexicon_tags_37 },
    { 0x911cc8e9, SUC_lexicon_tags_31 },
    { 0xa41cc8eb, SUC_lexicon_tags_12 },
    { 0x903cc8ea, SUC_lexicon_tags_87 },
    { 0xe1bcc8e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bcc8f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45cc8f7, SUC_lexicon_tags_7 },
    { 0x525cc8f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73cc8fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3cc901, SUC_lexicon_tags_48 },
    { 0x7d9cc902, SUC_lexicon_tags_0 },
    { 0xfc5cc903, SUC_lexicon_tags_37 },
    { 0x97bcc904, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553cc908, SUC_lexicon_tags_15 },
    { 0xe6dcc909, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5cc90d, SUC_lexicon_tags_27 },
    { 0xf5bcc90e, SUC_lexicon_tags_11 },
    { 0xdc1cc90e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa47cc911, SUC_lexicon_tags_11 },
    { 0x807cc912, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305cc916, SUC_lexicon_tags_11 },
    { 0xd1cc917, SUC_lexicon_tags_9 },
    { 0xe05cc916, SUC_lexicon_tags_31 },
    { 0xf41cc919, SUC_lexicon_tags_29 },
    { 0x93dcc919, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xc05cc91c, SUC_lexicon_tags_23 },
    { 0x1d3cc91d, SUC_lexicon_tags_37 },
    { 0x291cc91e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59cc920, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733cc925, SUC_lexicon_tags_17 },
    { 0x935cc925, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x859cc928, SUC_lexicon_tags_17 },
    { 0xb4bcc929, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9cc92d, SUC_lexicon_tags_47 },
    { 0x945cc92e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cdcc930, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x10bcc932, SUC_lexicon_tags_25 },
    { 0x3bfcc933, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85cc936, SUC_lexicon_tags_12 },
    { 0xe63cc937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1cc939, SUC_lexicon_tags_151 },
    { 0x785cc93a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe85cc93c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x505cc93e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85cc941, SUC_lexicon_tags_30 },
    { 0x1a7cc941, SUC_lexicon_tags_77 },
    { 0xfbfcc943, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5adcc945, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dcc949, SUC_lexicon_tags_45 },
    { 0xc8bcc949, SUC_lexicon_tags_13 },
    { 0x885cc94a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x871cc94d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1cc950, SUC_lexicon_tags_37 },
    { 0x33cc950, SUC_lexicon_tags_12 },
    { 0xd73cc950, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13cc955, SUC_lexicon_tags_12 },
    { 0xc4fcc955, SUC_lexicon_tags_14 },
    { 0x5c7cc957, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3a5cc959, SUC_lexicon_tags_57 },
    { 0xfcc95a, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75cc95d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x881cc95f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x53cc961, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51cc967, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcc96b, SUC_lexicon_tags_14 },
    { 0xa4fcc96c, SUC_lexicon_tags_5 },
    { 0xf6dcc96c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcc970, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457cc975, SUC_lexicon_tags_63 },
    { 0xbb9cc975, SUC_lexicon_tags_37 },
    { 0x62dcc976, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29cc979, SUC_lexicon_tags_19 },
    { 0x9d1cc979, SUC_lexicon_tags_14 },
    { 0xe65cc979, SUC_lexicon_tags_48 },
    { 0xfddcc97c, SUC_lexicon_tags_11 },
    { 0x7a5cc97c, SUC_lexicon_tags_7 },
    { 0x163cc97c, SUC_lexicon_tags_70 },
    { 0x613cc97f, SUC_lexicon_tags_66 },
    { 0x9b7cc980, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x903cc982, SUC_lexicon_tags_27 },
    { 0x99bcc983, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5cc988, SUC_lexicon_tags_48 },
    { 0xd6dcc988, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fdcc98b, SUC_lexicon_tags_14 },
    { 0x5d3cc98c, SUC_lexicon_tags_48 },
    { 0xacdcc98c, SUC_lexicon_tags_40 },
    { 0xa99cc98d, SUC_lexicon_tags_25 },
    { 0x3a7cc98d, SUC_lexicon_tags_12 },
    { 0x4b1cc990, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375cc998, SUC_lexicon_tags_4 },
    { 0xce3cc998, SUC_lexicon_tags_7 },
    { 0xd65cc99a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x505cc99c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3cc9a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45cc9a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cc9aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dcc9b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5cc9b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbcc9b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179cc9c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x33fcc9c5, SUC_lexicon_tags_7 },
    { 0xfedcc9c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cfcc9c8, SUC_lexicon_tags_15 },
    { 0x97cc9c9, SUC_lexicon_tags_91 },
    { 0x33fcc9ca, SUC_lexicon_tags_2 },
    { 0x185cc9ca, SUC_lexicon_tags_38 },
    { 0x2dbcc9ca, SUC_lexicon_tags_0 },
    { 0x5bcc9c9, SUC_lexicon_tags_31 },
    { 0x549cc9ce, SUC_lexicon_tags_37 },
    { 0x7a5cc9ce, SUC_lexicon_tags_25 },
    { 0xf8fcc9cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93cc9d3, SUC_lexicon_tags_4 },
    { 0x433cc9d3, SUC_lexicon_tags_7 },
    { 0x81cc9d3, SUC_lexicon_tags_12 },
    { 0x849cc9d6, SUC_lexicon_tags_29 },
    { 0xe07cc9d5, SUC_lexicon_tags_37 },
    { 0xb61cc9d6, SUC_lexicon_tags_17 },
    { 0xb89cc9d9, SUC_lexicon_tags_15 },
    { 0x907cc9d9, SUC_lexicon_tags_7 },
    { 0x3a5cc9db, SUC_lexicon_tags_55 },
    { 0x86bcc9dc, SUC_lexicon_tags_34 },
    { 0x993cc9dd, SUC_lexicon_tags_2 },
    { 0xd13cc9d9, SUC_lexicon_tags_11 },
    { 0x3e9cc9d9, SUC_lexicon_tags_11 },
    { 0x561cc9df, SUC_lexicon_tags_27 },
    { 0x275cc9e0, SUC_lexicon_tags_31 },
    { 0xe5bcc9d4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67cc9e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8adcc9ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x26dcc9ef, SUC_lexicon_tags_23 },
    { 0x7fcc9f0, SUC_lexicon_tags_37 },
    { 0x1d1cc9f1, SUC_lexicon_tags_15 },
    { 0x71bcc9f1, SUC_lexicon_tags_6 },
    { 0x947cc9ef, SUC_lexicon_tags_25 },
    { 0xee1cc9f4, SUC_lexicon_tags_61 },
    { 0xcfcc9f4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1cc9fd, SUC_lexicon_tags_33 },
    { 0xf0fcc9fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bcca02, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135cca0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9cca0d, SUC_lexicon_tags_0 },
    { 0xc69cca0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebcca13, SUC_lexicon_tags_44 },
    { 0x90bcca14, SUC_lexicon_tags_14 },
    { 0xb7cca15, SUC_lexicon_tags_16 },
    { 0xeb9cca16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfcca19, SUC_lexicon_tags_2 },
    { 0xb7bcca19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbcca1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3cca25, SUC_lexicon_tags_15 },
    { 0xdcca26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd87cca28, SUC_lexicon_tags_37 },
    { 0xe71cca29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfcca2c, SUC_lexicon_tags_25 },
    { 0x4a9cca2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59cca30, SUC_lexicon_tags_29 },
    { 0x379cca31, SUC_lexicon_tags_15 },
    { 0x6a1cca32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dcca37, SUC_lexicon_tags_25 },
    { 0x4fcca37, SUC_lexicon_tags_29 },
    { 0x8c7cca38, SUC_lexicon_tags_25 },
    { 0x4d5cca3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dcca40, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6cdcca42, SUC_lexicon_tags_65 },
    { 0xeb9cca43, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553cca49, SUC_lexicon_tags_17 },
    { 0xfb7cca4a, SUC_lexicon_tags_12 },
    { 0x991cca4b, SUC_lexicon_tags_74 },
    { 0xafcca4c, SUC_lexicon_tags_77 },
    { 0xe25cca4d, SUC_lexicon_tags_27 },
    { 0xeabcca4e, SUC_lexicon_tags_47 },
    { 0xb25cca4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bcca52, SUC_lexicon_tags_12 },
    { 0x3f5cca53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dcca58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73cca64, SUC_lexicon_tags_69 },
    { 0x1d1cca65, SUC_lexicon_tags_37 },
    { 0xcfbcca64, SUC_lexicon_tags_17 },
    { 0xe2bcca66, SUC_lexicon_tags_7 },
    { 0x29cca66, SUC_lexicon_tags_49 },
    { 0xbf5cca69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb53cca6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6efcca6d, SUC_lexicon_tags_0 },
    { 0x907cca6e, SUC_lexicon_tags_85 },
    { 0x8f5cca6f, SUC_lexicon_tags_2 },
    { 0x88fcca70, SUC_lexicon_tags_13 },
    { 0xca9cca71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bcca74, SUC_lexicon_tags_15 },
    { 0x5e3cca75, SUC_lexicon_tags_59 },
    { 0x4c9cca76, SUC_lexicon_tags_2 },
    { 0x879cca77, SUC_lexicon_tags_12 },
    { 0xf99cca78, SUC_lexicon_tags_1 },
    { 0x91bcca79, SUC_lexicon_tags_17 },
    { 0xfc3cca7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7cca7d, SUC_lexicon_tags_63 },
    { 0x7f1cca7e, SUC_lexicon_tags_39 },
    { 0xf3fcca7f, SUC_lexicon_tags_6 },
    { 0x97fcca80, SUC_lexicon_tags_4 },
    { 0x355cca81, SUC_lexicon_tags_25 },
    { 0x681cca82, SUC_lexicon_tags_0 },
    { 0xf3cca83, SUC_lexicon_tags_9 },
    { 0xa4fcca83, SUC_lexicon_tags_6 },
    { 0xa69cca85, SUC_lexicon_tags_29 },
    { 0xedcca85, SUC_lexicon_tags_1 },
    { 0x6fcca87, SUC_lexicon_tags_45 },
    { 0xf5cca87, SUC_lexicon_tags_2 },
    { 0xce1cca88, SUC_lexicon_tags_12 },
    { 0x181cca85, SUC_lexicon_tags_7 },
    { 0x7a7cca80, SUC_lexicon_tags_33 },
    { 0xf03cca7d, SUC_lexicon_tags_12 },
    { 0x127cca84, SUC_lexicon_tags_17 },
    { 0x73bcca8e, SUC_lexicon_tags_39 },
    { 0xdc1cca7f, SUC_lexicon_tags_60 },
    { 0xc55cca87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bcca93, SUC_lexicon_tags_23 },
    { 0xac9cca93, SUC_lexicon_tags_37 },
    { 0xc0dcca95, SUC_lexicon_tags_11 },
    { 0x2b3cca96, SUC_lexicon_tags_33 },
    { 0xbafcca97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfcca9c, SUC_lexicon_tags_25 },
    { 0x897cca9d, SUC_lexicon_tags_17 },
    { 0x92fcca9e, SUC_lexicon_tags_38 },
    { 0x665cca9f, SUC_lexicon_tags_25 },
    { 0xe79cca9d, SUC_lexicon_tags_39 },
    { 0x77fcca9e, SUC_lexicon_tags_4 },
    { 0x357ccaa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x731ccaa4, SUC_lexicon_tags_28 },
    { 0x1f7ccaa5, SUC_lexicon_tags_29 },
    { 0x543ccaa5, SUC_lexicon_tags_19 },
    { 0xe5ccaa7, SUC_lexicon_tags_11 },
    { 0xae5ccaa6, SUC_lexicon_tags_17 },
    { 0xa69ccaa8, SUC_lexicon_tags_15 },
    { 0x70dccaaa, SUC_lexicon_tags_10 },
    { 0x3ccaab, SUC_lexicon_tags_1 },
    { 0xaafccaa7, SUC_lexicon_tags_0 },
    { 0xc3dccaad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x319ccaaf, SUC_lexicon_tags_17 },
    { 0x1dfccab0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7ccab4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabccab8, SUC_lexicon_tags_2 },
    { 0xdc1ccab9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45ccac0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x339ccac3, SUC_lexicon_tags_17 },
    { 0x233ccac4, SUC_lexicon_tags_38 },
    { 0xd45ccac5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87ccac8, SUC_lexicon_tags_11 },
    { 0x47bccac9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x92dccacb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bccad0, SUC_lexicon_tags_45 },
    { 0xe11ccad1, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0x539ccad3, SUC_lexicon_tags_1 },
    { 0x835ccad4, SUC_lexicon_tags_11 },
    { 0x171ccad4, SUC_lexicon_tags_11 },
    { 0x4f7ccad6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465ccada, SUC_lexicon_tags_28 },
    { 0x1bfccadb, SUC_lexicon_tags_19 },
    { 0xdd9ccadc, SUC_lexicon_tags_16 },
    { 0x409ccadd, SUC_lexicon_tags_12 },
    { 0x16dccadc, SUC_lexicon_tags_4 },
    { 0x937ccadf, SUC_lexicon_tags_25 },
    { 0xe1fccae0, SUC_lexicon_tags_37 },
    { 0xb0fccae1, SUC_lexicon_tags_17 },
    { 0x2ffccadc, SUC_lexicon_tags_10 },
    { 0x5b1ccae3, SUC_lexicon_tags_25 },
    { 0x417ccae4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x803ccae7, SUC_lexicon_tags_33 },
    { 0xbbccae8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afccaeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x643ccaee, SUC_lexicon_tags_37 },
    { 0xf0bccaee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c5ccaf1, SUC_lexicon_tags_32 },
    { 0x7d1ccaf2, SUC_lexicon_tags_17 },
    { 0xa7ccaf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adccaf7, SUC_lexicon_tags_62 },
    { 0xe0dccaf8, SUC_lexicon_tags_12 },
    { 0xb81ccaf9, SUC_lexicon_tags_23 },
    { 0x60bccafa, SUC_lexicon_tags_15 },
    { 0xafccafb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fccaff, SUC_lexicon_tags_29 },
    { 0x251ccb00, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x96fccb02, SUC_lexicon_tags_15 },
    { 0x73dccb03, SUC_lexicon_tags_31 },
    { 0x261ccb03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3ccb0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715ccb13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x819ccb15, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7ccb1d, SUC_lexicon_tags_2 },
    { 0x87dccb1d, SUC_lexicon_tags_39 },
    { 0x4e7ccb1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353ccb23, SUC_lexicon_tags_62 },
    { 0x229ccb23, SUC_lexicon_tags_9 },
    { 0x2bccb25, SUC_lexicon_tags_87 },
    { 0x509ccb25, SUC_lexicon_tags_1 },
    { 0xc05ccb27, SUC_lexicon_tags_23 },
    { 0x225ccb28, SUC_lexicon_tags_7 },
    { 0x145ccb27, SUC_lexicon_tags_15 },
    { 0xa6fccb29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bccb2d, SUC_lexicon_tags_0 },
    { 0xfafccb2d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05ccb37, SUC_lexicon_tags_68 },
    { 0x2d9ccb38, SUC_lexicon_tags_35 },
    { 0xa97ccb39, SUC_lexicon_tags_23 },
    { 0xe9dccb39, SUC_lexicon_tags_32 },
    { 0x9d1ccb38, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x625ccb3e, SUC_lexicon_tags_17 },
    { 0x849ccb3e, SUC_lexicon_tags_42 },
    { 0xe75ccb40, SUC_lexicon_tags_60 },
    { 0xa6fccb3e, SUC_lexicon_tags_7 },
    { 0x7adccb42, SUC_lexicon_tags_11 },
    { 0x347ccb43, SUC_lexicon_tags_12 },
    { 0x3e5ccb40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x875ccb46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbccb48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3cbccb4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa59ccb4c, SUC_lexicon_tags_15 },
    { 0x4f9ccb4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afccb50, SUC_lexicon_tags_17 },
    { 0x5edccb51, SUC_lexicon_tags_1 },
    { 0xb1bccb52, SUC_lexicon_tags_17 },
    { 0x445ccb52, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x415ccb55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3ccb5a, SUC_lexicon_tags_29 },
    { 0x545ccb5a, SUC_lexicon_tags_17 },
    { 0x5dfccb5c, SUC_lexicon_tags_27 },
    { 0x605ccb5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ccb62, SUC_lexicon_tags_28 },
    { 0xf1fccb63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47bccb6a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423ccb71, SUC_lexicon_tags_7 },
    { 0x495ccb72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9ccb75, SUC_lexicon_tags_7 },
    { 0x4afccb75, SUC_lexicon_tags_0 },
    { 0x23ccb77, SUC_lexicon_tags_10 },
    { 0x807ccb77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edccb7c, SUC_lexicon_tags_3 },
    { 0x4b1ccb7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abccb82, SUC_lexicon_tags_1 },
    { 0x1efccb83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x175ccb85, SUC_lexicon_tags_31 },
    { 0x6b9ccb86, SUC_lexicon_tags_17 },
    { 0x271ccb87, SUC_lexicon_tags_17 },
    { 0xbbfccb88, SUC_lexicon_tags_40 },
    { 0x921ccb89, SUC_lexicon_tags_15 },
    { 0x44bccb8a, SUC_lexicon_tags_16 },
    { 0x24bccb8b, SUC_lexicon_tags_57 },
    { 0x2f1ccb89, SUC_lexicon_tags_31 },
    { 0x831ccb85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x357ccb8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbccb94, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5ccb9d, SUC_lexicon_tags_8 },
    { 0x587ccb9e, SUC_lexicon_tags_25 },
    { 0xa1ccb9f, SUC_lexicon_tags_19 },
    { 0x6dfccb9e, SUC_lexicon_tags_38 },
    { 0xe7bccba1, SUC_lexicon_tags_17 },
    { 0xab7ccba2, SUC_lexicon_tags_1 },
    { 0xfadccba3, SUC_lexicon_tags_26 },
    { 0x295ccba0, SUC_lexicon_tags_12 },
    { 0xffdccba5, SUC_lexicon_tags_49 },
    { 0x3f7ccba6, SUC_lexicon_tags_11 },
    { 0x6b5ccba5, SUC_lexicon_tags_15 },
    { 0xd27ccba1, SUC_lexicon_tags_85 },
    { 0x22fccba9, SUC_lexicon_tags_25 },
    { 0xe3ccba9, SUC_lexicon_tags_10 },
    { 0x527ccba0, SUC_lexicon_tags_93 },
    { 0x84fccbac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdccbb2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x17dccbb4, SUC_lexicon_tags_45 },
    { 0x6f1ccbb5, SUC_lexicon_tags_1 },
    { 0x985ccbb6, SUC_lexicon_tags_29 },
    { 0x67fccbb7, SUC_lexicon_tags_27 },
    { 0x549ccbb8, SUC_lexicon_tags_25 },
    { 0x163ccbb9, SUC_lexicon_tags_7 },
    { 0x56dccbba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8f5ccbbc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81ccbc0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07ccbc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ddccbc5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9ccbc8, SUC_lexicon_tags_25 },
    { 0xa89ccbc9, SUC_lexicon_tags_29 },
    { 0x1c9ccbca, SUC_lexicon_tags_12 },
    { 0x6efccbcb, SUC_lexicon_tags_46 },
    { 0x9adccbcc, SUC_lexicon_tags_2 },
    { 0xe81ccbca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbccbd2, SUC_lexicon_tags_26 },
    { 0xf15ccbd3, SUC_lexicon_tags_33 },
    { 0x29dccbd4, SUC_lexicon_tags_7 },
    { 0x617ccbd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9ccbd9, SUC_lexicon_tags_17 },
    { 0x17dccbda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbccbdd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x877ccbe0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5ccbe3, SUC_lexicon_tags_38 },
    { 0x3bccbe4, SUC_lexicon_tags_45 },
    { 0x2c3ccbe5, SUC_lexicon_tags_63 },
    { 0xfcfccbe4, SUC_lexicon_tags_17 },
    { 0x61fccbe4, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57ccbeb, SUC_lexicon_tags_17 },
    { 0xd8fccbec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3bdccbee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x129ccbf0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x79dccbf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ccbf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x359ccbf9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x693ccbfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabbccbfd, SUC_lexicon_tags_27 },
    { 0xd27ccbfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x237ccc00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x567ccc03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffccc07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80dccc0a, SUC_lexicon_tags_17 },
    { 0x13ccc0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7ccc0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11ccc12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dccc16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5ccc1c, SUC_lexicon_tags_12 },
    { 0xa5fccc1d, SUC_lexicon_tags_7 },
    { 0x7dccc1e, SUC_lexicon_tags_7 },
    { 0xe27ccc1f, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfccc28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x837ccc2b, SUC_lexicon_tags_31 },
    { 0x815ccc2c, SUC_lexicon_tags_28 },
    { 0xaebccc2d, SUC_lexicon_tags_15 },
    { 0x6b7ccc2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xef9ccc31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc13ccc33, SUC_lexicon_tags_35 },
    { 0x879ccc33, SUC_lexicon_tags_40 },
    { 0x3e5ccc35, SUC_lexicon_tags_48 },
    { 0xe69ccc36, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x17bccc38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf27ccc3a, SUC_lexicon_tags_2 },
    { 0x325ccc3b, SUC_lexicon_tags_15 },
    { 0xa93ccc3b, SUC_lexicon_tags_583 },
    { 0x9c3ccc3d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31ccc43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3ccc46, SUC_lexicon_tags_0 },
    { 0xa1dccc47, SUC_lexicon_tags_43 },
    { 0xaa7ccc48, SUC_lexicon_tags_25 },
    { 0xff7ccc48, SUC_lexicon_tags_25 },
    { 0xc8bccc47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e1ccc4c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xdf5ccc4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bccc51, SUC_lexicon_tags_25 },
    { 0x1e3ccc51, SUC_lexicon_tags_0 },
    { 0x731ccc53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fccc58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695ccc5d, SUC_lexicon_tags_25 },
    { 0xc31ccc5d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5f3ccc60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71bccc62, SUC_lexicon_tags_23 },
    { 0x2bccc63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bccc6b, SUC_lexicon_tags_46 },
    { 0x99bccc6c, SUC_lexicon_tags_49 },
    { 0x703ccc6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dccc70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x921ccc72, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bccc75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411ccc87, SUC_lexicon_tags_2 },
    { 0x7d1ccc88, SUC_lexicon_tags_7 },
    { 0x9fdccc89, SUC_lexicon_tags_7 },
    { 0xb0fccc8a, SUC_lexicon_tags_2 },
    { 0x973ccc89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ccc8e, SUC_lexicon_tags_4 },
    { 0x655ccc8f, SUC_lexicon_tags_25 },
    { 0x73fccc8f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dccc94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70dccc96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf91ccc98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ccc9d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xa31ccc9f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ccca2, SUC_lexicon_tags_25 },
    { 0x759ccca3, SUC_lexicon_tags_7 },
    { 0x627ccca2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9cccaa, SUC_lexicon_tags_7 },
    { 0xa9bcccaa, SUC_lexicon_tags_237 },
    { 0, NULL },
    { 0x577cccad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15dcccb3, SUC_lexicon_tags_45 },
    { 0xdbbcccb3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465cccb9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x357cccbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60dcccbd, SUC_lexicon_tags_33 },
    { 0x381cccbe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x73bcccc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x581cccc2, SUC_lexicon_tags_1 },
    { 0x52bcccc2, SUC_lexicon_tags_12 },
    { 0xc5fcccc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x825cccc6, SUC_lexicon_tags_49 },
    { 0x3d9cccc7, SUC_lexicon_tags_2 },
    { 0x951cccc8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6f3cccca, SUC_lexicon_tags_33 },
    { 0x437ccccb, SUC_lexicon_tags_25 },
    { 0x83fccccc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3cccd0, SUC_lexicon_tags_9 },
    { 0xebbcccd0, SUC_lexicon_tags_11 },
    { 0x863cccd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cccd8, SUC_lexicon_tags_11 },
    { 0x16bcccd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x563cccdc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511ccce6, SUC_lexicon_tags_77 },
    { 0x743ccce7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x899cccea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x441ccced, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xedcccef, SUC_lexicon_tags_69 },
    { 0xfb9cccf0, SUC_lexicon_tags_11 },
    { 0xd03cccf0, SUC_lexicon_tags_4 },
    { 0xd1dcccf2, SUC_lexicon_tags_27 },
    { 0x10fcccf2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813cccf7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75cccfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69cccfd, SUC_lexicon_tags_23 },
    { 0xe6dcccfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfafccd00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33ccd08, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x887ccd0b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fccd0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x455ccd11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85ccd15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbccd18, SUC_lexicon_tags_43 },
    { 0x521ccd18, SUC_lexicon_tags_0 },
    { 0x93bccd19, SUC_lexicon_tags_11 },
    { 0x257ccd1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01ccd1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95bccd21, SUC_lexicon_tags_15 },
    { 0x9d1ccd22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88fccd25, SUC_lexicon_tags_60 },
    { 0xc4fccd26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fccd2d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x993ccd30, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ccd36, SUC_lexicon_tags_27 },
    { 0xe55ccd37, SUC_lexicon_tags_12 },
    { 0x9b3ccd38, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb3ccd3a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21ccd3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe1ccd3f, SUC_lexicon_tags_66 },
    { 0x3b5ccd40, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83ccd43, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x185ccd46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5ccd4a, SUC_lexicon_tags_15 },
    { 0xb39ccd4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bccd50, SUC_lexicon_tags_14 },
    { 0xff1ccd51, SUC_lexicon_tags_27 },
    { 0x9adccd50, SUC_lexicon_tags_15 },
    { 0x1c1ccd52, SUC_lexicon_tags_37 },
    { 0x207ccd50, SUC_lexicon_tags_33 },
    { 0x275ccd55, SUC_lexicon_tags_40 },
    { 0x333ccd55, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fccd59, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5adccd5b, SUC_lexicon_tags_2 },
    { 0xb9dccd5b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95ccd5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dccd63, SUC_lexicon_tags_7 },
    { 0xae7ccd64, SUC_lexicon_tags_12 },
    { 0x39ccd63, SUC_lexicon_tags_12 },
    { 0xe11ccd66, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651ccd6a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc65ccd6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15ccd70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x305ccd72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119ccd78, SUC_lexicon_tags_8 },
    { 0x6bccd79, SUC_lexicon_tags_162 },
    { 0x9d3ccd78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bccd7c, SUC_lexicon_tags_32 },
    { 0x3bdccd7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a9ccd7f, SUC_lexicon_tags_68 },
    { 0xcc3ccd80, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1ccd84, SUC_lexicon_tags_0 },
    { 0x317ccd85, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6b9ccd87, SUC_lexicon_tags_37 },
    { 0xb53ccd88, SUC_lexicon_tags_3 },
    { 0x5c7ccd89, SUC_lexicon_tags_23 },
    { 0x149ccd8a, SUC_lexicon_tags_47 },
    { 0x96bccd8b, SUC_lexicon_tags_15 },
    { 0xb1bccd89, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fccd8f, SUC_lexicon_tags_25 },
    { 0xbabccd90, SUC_lexicon_tags_37 },
    { 0xc01ccd8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367ccd97, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfccd9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x837ccd9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x991ccda2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x82fccda4, SUC_lexicon_tags_2 },
    { 0x89bccda5, SUC_lexicon_tags_17 },
    { 0x83dccda4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x65fccda8, SUC_lexicon_tags_27 },
    { 0xf7fccda9, SUC_lexicon_tags_29 },
    { 0xd83ccda9, SUC_lexicon_tags_25 },
    { 0x7bdccdab, SUC_lexicon_tags_37 },
    { 0xf53ccdab, SUC_lexicon_tags_15 },
    { 0x6fccdac, SUC_lexicon_tags_11 },
    { 0x3c7ccdab, SUC_lexicon_tags_11 },
    { 0x231ccdad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb7ccdb1, SUC_lexicon_tags_7 },
    { 0x743ccdb1, SUC_lexicon_tags_32 },
    { 0xe61ccdb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x243ccdb5, SUC_lexicon_tags_17 },
    { 0xab7ccdb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x173ccdb8, SUC_lexicon_tags_10 },
    { 0x71dccdb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1afccdbb, SUC_lexicon_tags_12 },
    { 0xcbccdbb, SUC_lexicon_tags_25 },
    { 0x925ccdbd, SUC_lexicon_tags_47 },
    { 0x1bbccdbb, SUC_lexicon_tags_11 },
    { 0xa85ccdbf, SUC_lexicon_tags_23 },
    { 0x87fccdc0, SUC_lexicon_tags_44 },
    { 0xdf3ccdc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9ccdc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd37ccdc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3ccdce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475ccdd9, SUC_lexicon_tags_17 },
    { 0x33dccdda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbccdde, SUC_lexicon_tags_37 },
    { 0x773ccddf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fccde7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ccdea, SUC_lexicon_tags_31 },
    { 0x32fccdeb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x135ccded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afccdf0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7ccdf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89dccdf5, SUC_lexicon_tags_15 },
    { 0xd0fccdf6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743ccdfb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1ccdfe, SUC_lexicon_tags_49 },
    { 0x231ccdff, SUC_lexicon_tags_6 },
    { 0x74bccdfe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfcce04, SUC_lexicon_tags_7 },
    { 0x3a3cce05, SUC_lexicon_tags_68 },
    { 0xda9cce05, SUC_lexicon_tags_21 },
    { 0xe7dcce07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1cce0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x197cce0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x393cce11, SUC_lexicon_tags_48 },
    { 0x637cce12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfcce1a, SUC_lexicon_tags_2 },
    { 0x16dcce1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x169cce1d, SUC_lexicon_tags_0 },
    { 0x137cce1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bcce27, SUC_lexicon_tags_38 },
    { 0xda1cce27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7ddcce2a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fcce31, SUC_lexicon_tags_38 },
    { 0x7d1cce32, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dcce39, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x1d7cce3b, SUC_lexicon_tags_61 },
    { 0xa89cce3c, SUC_lexicon_tags_25 },
    { 0x869cce3b, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7cce40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x32bcce42, SUC_lexicon_tags_11 },
    { 0xa7fcce43, SUC_lexicon_tags_45 },
    { 0x4d1cce43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bcce47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5edcce49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x989cce4c, SUC_lexicon_tags_63 },
    { 0x9a7cce4c, SUC_lexicon_tags_183 },
    { 0xed7cce4d, SUC_lexicon_tags_1606 },
    { 0x771cce4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61fcce51, SUC_lexicon_tags_11 },
    { 0xf25cce52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63cce55, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13cce58, SUC_lexicon_tags_37 },
    { 0xc95cce59, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01cce5c, SUC_lexicon_tags_37 },
    { 0x905cce5d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdcce62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fcce67, SUC_lexicon_tags_25 },
    { 0x36bcce68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fcce6b, SUC_lexicon_tags_12 },
    { 0x98dcce6b, SUC_lexicon_tags_17 },
    { 0x7cce6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfcce70, SUC_lexicon_tags_207 },
    { 0x20bcce71, SUC_lexicon_tags_4 },
    { 0x779cce71, SUC_lexicon_tags_47 },
    { 0x7edcce73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67dcce75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13cce7c, SUC_lexicon_tags_47 },
    { 0xeb5cce7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bcce80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75dcce83, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xbdbcce85, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55cce88, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x9d7cce8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1dcce8c, SUC_lexicon_tags_25 },
    { 0x585cce8d, SUC_lexicon_tags_37 },
    { 0xd07cce8e, SUC_lexicon_tags_30 },
    { 0x19dcce8e, SUC_lexicon_tags_11 },
    { 0x64bcce90, SUC_lexicon_tags_23 },
    { 0xb2bcce8e, SUC_lexicon_tags_37 },
    { 0x72bcce92, SUC_lexicon_tags_37 },
    { 0x9afcce8e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157cce97, SUC_lexicon_tags_37 },
    { 0x21fcce98, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63ccea7, SUC_lexicon_tags_7 },
    { 0x22fccea7, SUC_lexicon_tags_25 },
    { 0x4ddccea9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2dcceab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaedccead, SUC_lexicon_tags_25 },
    { 0x3e9cceae, SUC_lexicon_tags_27 },
    { 0xb21cceae, SUC_lexicon_tags_31 },
    { 0x4e9ccead, SUC_lexicon_tags_9 },
    { 0xbf1ccead, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1cceb5, SUC_lexicon_tags_23 },
    { 0xfdbcceb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e1cceb8, SUC_lexicon_tags_61 },
    { 0xe97cceb8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xc65ccebb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf1dccebd, SUC_lexicon_tags_27 },
    { 0xb57ccebe, SUC_lexicon_tags_35 },
    { 0x19dccebf, SUC_lexicon_tags_10 },
    { 0xb01ccebd, SUC_lexicon_tags_37 },
    { 0x86fccec1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bccec6, SUC_lexicon_tags_15 },
    { 0x75fccec6, SUC_lexicon_tags_11 },
    { 0xa01ccec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dccecc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1ccecf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bcced3, SUC_lexicon_tags_37 },
    { 0x1e1cced4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x955cced7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf39cced9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31ccedc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x12bccede, SUC_lexicon_tags_0 },
    { 0x1b9ccedf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abccee6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1a1ccee8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c5cceea, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dccef1, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969ccef5, SUC_lexicon_tags_79 },
    { 0xfccef6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dccefe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7ccf01, SUC_lexicon_tags_7 },
    { 0xbafccf02, SUC_lexicon_tags_11 },
    { 0xed5ccf03, SUC_lexicon_tags_34 },
    { 0xc95ccf02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f7ccf06, SUC_lexicon_tags_1 },
    { 0x24dccf06, SUC_lexicon_tags_25 },
    { 0xf17ccf06, SUC_lexicon_tags_31 },
    { 0x113ccf07, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fccf0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x35ccf11, SUC_lexicon_tags_19 },
    { 0x821ccf11, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19ccf16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffccf1c, SUC_lexicon_tags_17 },
    { 0x533ccf1c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9ccf20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1ccf26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5c1ccf28, SUC_lexicon_tags_43 },
    { 0xc07ccf29, SUC_lexicon_tags_15 },
    { 0x8d9ccf2a, SUC_lexicon_tags_6 },
    { 0x4b3ccf2a, SUC_lexicon_tags_25 },
    { 0x1ccf2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83ccf30, SUC_lexicon_tags_60 },
    { 0x553ccf31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffccf36, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15ccf39, SUC_lexicon_tags_11 },
    { 0x423ccf39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91ccf3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x569ccf41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x331ccf43, SUC_lexicon_tags_14 },
    { 0xea3ccf44, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1ccf49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5f1ccf4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd4fccf4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dccf50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5ccf55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ccf59, SUC_lexicon_tags_30 },
    { 0x8ddccf5a, SUC_lexicon_tags_17 },
    { 0xc27ccf5a, SUC_lexicon_tags_31 },
    { 0xef1ccf5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fccf69, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5ccf71, SUC_lexicon_tags_21 },
    { 0x5f3ccf72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fccf75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a1ccf77, SUC_lexicon_tags_2 },
    { 0x8d5ccf77, SUC_lexicon_tags_19 },
    { 0x7e3ccf79, SUC_lexicon_tags_68 },
    { 0x519ccf7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867ccf80, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x895ccf82, SUC_lexicon_tags_26 },
    { 0x5efccf83, SUC_lexicon_tags_7 },
    { 0xe6fccf84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bccf88, SUC_lexicon_tags_37 },
    { 0x271ccf89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747ccf8e, SUC_lexicon_tags_28 },
    { 0x9ddccf8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe45ccf91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x711ccf93, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x745ccf96, SUC_lexicon_tags_87 },
    { 0xb6bccf97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85ccf9a, SUC_lexicon_tags_234 },
    { 0xebbccf9a, SUC_lexicon_tags_6 },
    { 0x369ccf9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa8fccf9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5ccfa1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fccfa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a3ccfa6, SUC_lexicon_tags_7 },
    { 0x43dccfa6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ccfaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bccfae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23ccfb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48fccfb7, SUC_lexicon_tags_15 },
    { 0xcd1ccfb8, SUC_lexicon_tags_47 },
    { 0x7cbccfb7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcf3ccfbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7ccfbf, SUC_lexicon_tags_30 },
    { 0x323ccfc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8dccfc2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ccfc5, SUC_lexicon_tags_46 },
    { 0xc35ccfc5, SUC_lexicon_tags_45 },
    { 0x357ccfc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355ccfcf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3ccfd3, SUC_lexicon_tags_2 },
    { 0x9f9ccfd4, SUC_lexicon_tags_32 },
    { 0x643ccfd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbccfd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9ccfdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3b7ccfde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ccfe6, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ccfed, SUC_lexicon_tags_33 },
    { 0x5fbccfee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fccff1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbccff6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31ccffa, SUC_lexicon_tags_11 },
    { 0xfc3ccffb, SUC_lexicon_tags_60 },
    { 0x527ccffa, SUC_lexicon_tags_55 },
    { 0xe15ccffd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x391ccfff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x833cd001, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3e1cd003, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x6c1cd005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x405cd007, SUC_lexicon_tags_19 },
    { 0x733cd008, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6cfcd00a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x817cd00c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x839cd00f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbcd014, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7cd017, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dcd01e, SUC_lexicon_tags_45 },
    { 0xde3cd01e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x753cd021, SUC_lexicon_tags_35 },
    { 0x195cd021, SUC_lexicon_tags_3 },
    { 0x843cd022, SUC_lexicon_tags_16 },
    { 0x8b3cd021, SUC_lexicon_tags_23 },
    { 0xe4bcd025, SUC_lexicon_tags_17 },
    { 0x9d5cd026, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91dcd02b, SUC_lexicon_tags_11 },
    { 0x841cd02b, SUC_lexicon_tags_66 },
    { 0xa6dcd02d, SUC_lexicon_tags_652 },
    { 0x76dcd02c, SUC_lexicon_tags_0 },
    { 0x8e1cd02d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15cd035, SUC_lexicon_tags_17 },
    { 0x9fdcd036, SUC_lexicon_tags_33 },
    { 0xf3cd035, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fcd03a, SUC_lexicon_tags_61 },
    { 0x61bcd03b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9cd03f, SUC_lexicon_tags_49 },
    { 0xbf3cd040, SUC_lexicon_tags_12 },
    { 0x249cd041, SUC_lexicon_tags_66 },
    { 0xd45cd041, SUC_lexicon_tags_10 },
    { 0x5f7cd043, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x2f3cd045, SUC_lexicon_tags_25 },
    { 0x9fdcd046, SUC_lexicon_tags_11 },
    { 0xd7cd046, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3cd04b, SUC_lexicon_tags_17 },
    { 0xadcd04c, SUC_lexicon_tags_28 },
    { 0x1dbcd04d, SUC_lexicon_tags_21 },
    { 0xf35cd04e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7cd055, SUC_lexicon_tags_32 },
    { 0xb85cd056, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xce7cd058, SUC_lexicon_tags_25 },
    { 0xe09cd059, SUC_lexicon_tags_3 },
    { 0x9e1cd058, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dcd060, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55cd06e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x473cd070, SUC_lexicon_tags_33 },
    { 0x547cd070, SUC_lexicon_tags_12 },
    { 0x6d5cd072, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x287cd074, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dcd07d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25cd082, SUC_lexicon_tags_14 },
    { 0x65bcd083, SUC_lexicon_tags_7 },
    { 0x7c9cd084, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd41cd086, SUC_lexicon_tags_38 },
    { 0x1dfcd086, SUC_lexicon_tags_17 },
    { 0x803cd087, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x365cd08a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x865cd08d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefcd091, SUC_lexicon_tags_6 },
    { 0x351cd092, SUC_lexicon_tags_39 },
    { 0x923cd093, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59cd097, SUC_lexicon_tags_15 },
    { 0xbcdcd098, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5cd09c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x765cd09f, SUC_lexicon_tags_47 },
    { 0x1afcd0a0, SUC_lexicon_tags_14 },
    { 0x921cd0a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bcd0a5, SUC_lexicon_tags_12 },
    { 0x787cd0a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a7cd0a8, SUC_lexicon_tags_141 },
    { 0xa57cd0a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edcd0ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc81cd0ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb01cd0b0, SUC_lexicon_tags_60 },
    { 0xc09cd0b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3cd0b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24bcd0b6, SUC_lexicon_tags_343 },
    { 0x14fcd0b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x719cd0b9, SUC_lexicon_tags_19 },
    { 0xdd7cd0b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7cd0be, SUC_lexicon_tags_15 },
    { 0x2bbcd0be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e1cd0c1, SUC_lexicon_tags_37 },
    { 0x3cd0c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bcd0ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dcd0cd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe99cd0cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78fcd0d1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb59cd0d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdfbcd0d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfcd0d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bcd0e3, SUC_lexicon_tags_37 },
    { 0xcd5cd0e4, SUC_lexicon_tags_17 },
    { 0x80fcd0e5, SUC_lexicon_tags_39 },
    { 0xf73cd0e6, SUC_lexicon_tags_32 },
    { 0x53cd0e7, SUC_lexicon_tags_33 },
    { 0xacdcd0e7, SUC_lexicon_tags_25 },
    { 0x62bcd0e9, SUC_lexicon_tags_8 },
    { 0xf97cd0e6, SUC_lexicon_tags_2 },
    { 0xe13cd0e6, SUC_lexicon_tags_12 },
    { 0x59fcd0ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729cd0f2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x47dcd0f4, SUC_lexicon_tags_11 },
    { 0x255cd0f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993cd0fd, SUC_lexicon_tags_66 },
    { 0x9b3cd0fe, SUC_lexicon_tags_65 },
    { 0xff7cd0fe, SUC_lexicon_tags_2 },
    { 0xe1cd0fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459cd106, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x103cd108, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bcd111, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbcd113, SUC_lexicon_tags_17 },
    { 0x663cd114, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bcd118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa03cd11a, SUC_lexicon_tags_12 },
    { 0x71dcd11b, SUC_lexicon_tags_47 },
    { 0x1f3cd11b, SUC_lexicon_tags_49 },
    { 0xd31cd11c, SUC_lexicon_tags_2 },
    { 0x5f9cd11c, SUC_lexicon_tags_0 },
    { 0x581cd11f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cd12b, SUC_lexicon_tags_8 },
    { 0xd5bcd12c, SUC_lexicon_tags_21 },
    { 0x419cd12d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25cd130, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbcd136, SUC_lexicon_tags_7 },
    { 0x37bcd137, SUC_lexicon_tags_17 },
    { 0xf3bcd137, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fcd13c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x953cd13e, SUC_lexicon_tags_7 },
    { 0x1adcd13e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffcd142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3cd145, SUC_lexicon_tags_25 },
    { 0x13cd146, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xeb3cd148, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0bcd14a, SUC_lexicon_tags_0 },
    { 0xccfcd14b, SUC_lexicon_tags_11 },
    { 0xb81cd14c, SUC_lexicon_tags_138 },
    { 0xbd7cd14d, SUC_lexicon_tags_38 },
    { 0x1c3cd14a, SUC_lexicon_tags_17 },
    { 0x1ffcd14d, SUC_lexicon_tags_54 },
    { 0x5a9cd14c, SUC_lexicon_tags_16 },
    { 0x2bbcd151, SUC_lexicon_tags_31 },
    { 0xe81cd152, SUC_lexicon_tags_68 },
    { 0x59bcd153, SUC_lexicon_tags_25 },
    { 0xf07cd154, SUC_lexicon_tags_112 },
    { 0x2f1cd155, SUC_lexicon_tags_65 },
    { 0xbd3cd155, SUC_lexicon_tags_12 },
    { 0x4b5cd151, SUC_lexicon_tags_29 },
    { 0xc4bcd158, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x347cd15b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd7bcd15d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcd164, SUC_lexicon_tags_25 },
    { 0x6dcd165, SUC_lexicon_tags_10 },
    { 0xb59cd166, SUC_lexicon_tags_45 },
    { 0x809cd167, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bcd170, SUC_lexicon_tags_12 },
    { 0xd9dcd171, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fcd174, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81cd177, SUC_lexicon_tags_23 },
    { 0x719cd177, SUC_lexicon_tags_7 },
    { 0x28bcd178, SUC_lexicon_tags_15 },
    { 0x89cd17a, SUC_lexicon_tags_17 },
    { 0xaebcd17b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5cd17e, SUC_lexicon_tags_108 },
    { 0xb87cd17f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe3bcd181, SUC_lexicon_tags_9 },
    { 0x5dbcd181, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5cd185, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x96dcd187, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63cd191, SUC_lexicon_tags_66 },
    { 0xbdfcd191, SUC_lexicon_tags_9 },
    { 0x527cd193, SUC_lexicon_tags_47 },
    { 0x95cd194, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861cd19e, SUC_lexicon_tags_25 },
    { 0xd9dcd19f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7cd1a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa45cd1a5, SUC_lexicon_tags_2 },
    { 0xe57cd1a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbcd1a9, SUC_lexicon_tags_21 },
    { 0x3a1cd1aa, SUC_lexicon_tags_11 },
    { 0xdefcd1ab, SUC_lexicon_tags_37 },
    { 0x42fcd1ac, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x667cd1af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe31cd1b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf01cd1b3, SUC_lexicon_tags_25 },
    { 0xd63cd1b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabcd1b8, SUC_lexicon_tags_3 },
    { 0xa0fcd1b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdfdcd1bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7cd1be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x29fcd1c1, SUC_lexicon_tags_65 },
    { 0xeb9cd1c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bcd1c6, SUC_lexicon_tags_38 },
    { 0x11cd1c7, SUC_lexicon_tags_25 },
    { 0x471cd1c7, SUC_lexicon_tags_12 },
    { 0x6bcd1c7, SUC_lexicon_tags_45 },
    { 0xfabcd1c6, SUC_lexicon_tags_9 },
    { 0x9b1cd1cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x72dcd1cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41cd1d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3cd1d9, SUC_lexicon_tags_146 },
    { 0x741cd1da, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x335cd1dc, SUC_lexicon_tags_47 },
    { 0x617cd1dd, SUC_lexicon_tags_23 },
    { 0x11dcd1dd, SUC_lexicon_tags_77 },
    { 0xf47cd1dc, SUC_lexicon_tags_65 },
    { 0xe8fcd1e0, SUC_lexicon_tags_405 },
    { 0x68fcd1e1, SUC_lexicon_tags_17 },
    { 0x575cd1e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x437cd1e4, SUC_lexicon_tags_38 },
    { 0xcc5cd1e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e7cd1e7, SUC_lexicon_tags_28 },
    { 0x17fcd1e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543cd1ee, SUC_lexicon_tags_43 },
    { 0xc39cd1ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf9bcd1f1, SUC_lexicon_tags_21 },
    { 0xf09cd1f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf9cd1f4, SUC_lexicon_tags_15 },
    { 0x29dcd1f4, SUC_lexicon_tags_10 },
    { 0x427cd1f6, SUC_lexicon_tags_17 },
    { 0x7efcd1f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa63cd1f9, SUC_lexicon_tags_7 },
    { 0xb53cd1fa, SUC_lexicon_tags_37 },
    { 0xc75cd1fa, SUC_lexicon_tags_37 },
    { 0xc3dcd1fb, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x1bfcd1fe, SUC_lexicon_tags_25 },
    { 0xf13cd1ff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23cd202, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5b9cd204, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbd7cd206, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x319cd209, SUC_lexicon_tags_11 },
    { 0x969cd20a, SUC_lexicon_tags_31 },
    { 0x2c9cd209, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35cd20f, SUC_lexicon_tags_48 },
    { 0x89fcd210, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9bbcd212, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65cd216, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x19bcd219, SUC_lexicon_tags_6 },
    { 0xb53cd219, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51fcd21c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf49cd21e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafcd226, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7cd22a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7cd230, SUC_lexicon_tags_23 },
    { 0xeafcd231, SUC_lexicon_tags_65 },
    { 0x283cd232, SUC_lexicon_tags_2 },
    { 0x3d1cd232, SUC_lexicon_tags_14 },
    { 0xaf5cd233, SUC_lexicon_tags_14 },
    { 0xd41cd233, SUC_lexicon_tags_26 },
    { 0x337cd234, SUC_lexicon_tags_43 },
    { 0x303cd235, SUC_lexicon_tags_15 },
    { 0xc27cd238, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3cd246, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bcd249, SUC_lexicon_tags_6 },
    { 0xb8dcd24a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabcd24f, SUC_lexicon_tags_30 },
    { 0xae1cd250, SUC_lexicon_tags_37 },
    { 0x2dfcd24f, SUC_lexicon_tags_25 },
    { 0xa7fcd251, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x175cd254, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fcd259, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbcd25c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43cd260, SUC_lexicon_tags_9 },
    { 0x423cd261, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1cd264, SUC_lexicon_tags_43 },
    { 0xaf5cd265, SUC_lexicon_tags_15 },
    { 0x647cd266, SUC_lexicon_tags_38 },
    { 0x953cd266, SUC_lexicon_tags_25 },
    { 0xfedcd268, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x423cd26a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x77cd26c, SUC_lexicon_tags_15 },
    { 0x919cd26d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933cd271, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25cd275, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebcd27e, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689cd282, SUC_lexicon_tags_17 },
    { 0x2f7cd282, SUC_lexicon_tags_8 },
    { 0xc11cd284, SUC_lexicon_tags_19 },
    { 0x947cd283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x367cd287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3cd28a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6bcd28c, SUC_lexicon_tags_12 },
    { 0xcafcd28c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799cd291, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bcd299, SUC_lexicon_tags_286 },
    { 0x8afcd299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b3cd29c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf33cd29e, SUC_lexicon_tags_25 },
    { 0x129cd29e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x99dcd2a1, SUC_lexicon_tags_11 },
    { 0xebbcd2a2, SUC_lexicon_tags_11 },
    { 0xb1cd2a3, SUC_lexicon_tags_25 },
    { 0xc27cd2a4, SUC_lexicon_tags_17 },
    { 0xca5cd2a3, SUC_lexicon_tags_23 },
    { 0x9b1cd2a6, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7cd2ab, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x113cd2ad, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715cd2b5, SUC_lexicon_tags_1 },
    { 0x5c3cd2b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfcd2bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281cd2bf, SUC_lexicon_tags_8 },
    { 0x9dcd2c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bcd2c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x333cd2cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9cd2d0, SUC_lexicon_tags_15 },
    { 0xf1cd2d1, SUC_lexicon_tags_12 },
    { 0xe93cd2d2, SUC_lexicon_tags_7 },
    { 0x259cd2d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb47cd2d5, SUC_lexicon_tags_29 },
    { 0x969cd2d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x167cd2d8, SUC_lexicon_tags_230 },
    { 0x90dcd2d8, SUC_lexicon_tags_107 },
    { 0xe07cd2d9, SUC_lexicon_tags_8 },
    { 0x147cd2db, SUC_lexicon_tags_2 },
    { 0xcd3cd2dc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x175cd2de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd5dcd2e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233cd2e7, SUC_lexicon_tags_12 },
    { 0x8d1cd2e7, SUC_lexicon_tags_19 },
    { 0xf7bcd2e9, SUC_lexicon_tags_15 },
    { 0xe5bcd2ea, SUC_lexicon_tags_79 },
    { 0x1d7cd2e9, SUC_lexicon_tags_12 },
    { 0xf63cd2ec, SUC_lexicon_tags_26 },
    { 0x1dbcd2ed, SUC_lexicon_tags_27 },
    { 0xc65cd2e8, SUC_lexicon_tags_17 },
    { 0x3efcd2ea, SUC_lexicon_tags_16 },
    { 0x7d1cd2ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x643cd2f2, SUC_lexicon_tags_15 },
    { 0xaa7cd2f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x251cd2f5, SUC_lexicon_tags_11 },
    { 0x957cd2f6, SUC_lexicon_tags_47 },
    { 0xc73cd2f5, SUC_lexicon_tags_14 },
    { 0x297cd2f8, SUC_lexicon_tags_23 },
    { 0x523cd2f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5cd2fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x903cd2fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705cd303, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x14bcd305, SUC_lexicon_tags_11 },
    { 0x7cbcd306, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5cd309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc1cd30b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x715cd30e, SUC_lexicon_tags_16 },
    { 0x3f1cd30e, SUC_lexicon_tags_11 },
    { 0xdddcd310, SUC_lexicon_tags_15 },
    { 0x30fcd311, SUC_lexicon_tags_1 },
    { 0x3f9cd310, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7cd319, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73cd322, SUC_lexicon_tags_14 },
    { 0xdf9cd323, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x44fcd325, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf5cd327, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x841cd32a, SUC_lexicon_tags_17 },
    { 0xf3dcd32a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517cd336, SUC_lexicon_tags_43 },
    { 0xd19cd337, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3cd33c, SUC_lexicon_tags_61 },
    { 0x27cd33d, SUC_lexicon_tags_7 },
    { 0xb35cd33e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fcd341, SUC_lexicon_tags_6 },
    { 0xc8fcd342, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5cd348, SUC_lexicon_tags_10 },
    { 0xde5cd348, SUC_lexicon_tags_9 },
    { 0x213cd34a, SUC_lexicon_tags_25 },
    { 0x109cd34a, SUC_lexicon_tags_6 },
    { 0xbabcd34c, SUC_lexicon_tags_31 },
    { 0x247cd34c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219cd356, SUC_lexicon_tags_37 },
    { 0x66bcd357, SUC_lexicon_tags_18 },
    { 0xb1bcd357, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9cd35c, SUC_lexicon_tags_17 },
    { 0x8fcd35d, SUC_lexicon_tags_38 },
    { 0x551cd35e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719cd364, SUC_lexicon_tags_32 },
    { 0x78bcd365, SUC_lexicon_tags_26 },
    { 0xd6dcd365, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bcd369, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dcd36e, SUC_lexicon_tags_17 },
    { 0x2dfcd36f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x565cd372, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1cd379, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x129cd37c, SUC_lexicon_tags_16 },
    { 0xbddcd37d, SUC_lexicon_tags_8 },
    { 0xc01cd37d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7c5cd380, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3cd383, SUC_lexicon_tags_38 },
    { 0xb5dcd384, SUC_lexicon_tags_2 },
    { 0xd05cd385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783cd38b, SUC_lexicon_tags_1 },
    { 0x4ebcd38b, SUC_lexicon_tags_17 },
    { 0x7f7cd38d, SUC_lexicon_tags_31 },
    { 0xeb9cd38e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d3cd390, SUC_lexicon_tags_11 },
    { 0xebdcd391, SUC_lexicon_tags_85 },
    { 0xf71cd392, SUC_lexicon_tags_31 },
    { 0x12fcd393, SUC_lexicon_tags_12 },
    { 0x6ddcd391, SUC_lexicon_tags_24 },
    { 0xcc9cd393, SUC_lexicon_tags_12 },
    { 0x759cd396, SUC_lexicon_tags_12 },
    { 0x509cd397, SUC_lexicon_tags_4 },
    { 0xe3cd398, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55cd39c, SUC_lexicon_tags_2 },
    { 0x5e1cd39d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5cd3a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bcd3a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fcd3ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6afcd3b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x915cd3b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dcd3bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bcd3c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x241cd3c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bcd3c9, SUC_lexicon_tags_16 },
    { 0xd0fcd3ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67cd3d1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x133cd3d4, SUC_lexicon_tags_47 },
    { 0xa5cd3d5, SUC_lexicon_tags_37 },
    { 0x2b5cd3d6, SUC_lexicon_tags_47 },
    { 0xbffcd3d6, SUC_lexicon_tags_1 },
    { 0x83bcd3d8, SUC_lexicon_tags_45 },
    { 0x7dfcd3d9, SUC_lexicon_tags_17 },
    { 0xa19cd3d6, SUC_lexicon_tags_25 },
    { 0xc21cd3d4, SUC_lexicon_tags_47 },
    { 0x45fcd3d4, SUC_lexicon_tags_17 },
    { 0xa29cd3d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3cd3e0, SUC_lexicon_tags_25 },
    { 0xcbbcd3e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x86dcd3e3, SUC_lexicon_tags_11 },
    { 0x60bcd3e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68bcd3e6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979cd3ed, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x733cd3ef, SUC_lexicon_tags_17 },
    { 0x89cd3ef, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9cd3f4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x685cd3f7, SUC_lexicon_tags_15 },
    { 0x785cd3f8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdcd3fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bcd402, SUC_lexicon_tags_30 },
    { 0x8d9cd403, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1bcd405, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bcd408, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9cd40c, SUC_lexicon_tags_8 },
    { 0xc41cd40d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5cd411, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21cd414, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721cd419, SUC_lexicon_tags_151 },
    { 0x631cd41a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x805cd41d, SUC_lexicon_tags_10 },
    { 0x4cfcd41e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7cd422, SUC_lexicon_tags_7 },
    { 0xb47cd422, SUC_lexicon_tags_11 },
    { 0x9bcd424, SUC_lexicon_tags_32 },
    { 0x345cd425, SUC_lexicon_tags_14 },
    { 0xc1bcd425, SUC_lexicon_tags_29 },
    { 0x12fcd426, SUC_lexicon_tags_1 },
    { 0x5a9cd422, SUC_lexicon_tags_0 },
    { 0x81fcd425, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f5cd42b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b7cd42d, SUC_lexicon_tags_17 },
    { 0xe15cd42d, SUC_lexicon_tags_17 },
    { 0x2bcd42f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf2bcd431, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfcd43d, SUC_lexicon_tags_7 },
    { 0x787cd43d, SUC_lexicon_tags_16 },
    { 0x141cd43d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x31fcd442, SUC_lexicon_tags_0 },
    { 0xdc1cd443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x715cd446, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3cd453, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5cd457, SUC_lexicon_tags_87 },
    { 0x681cd458, SUC_lexicon_tags_37 },
    { 0xec7cd459, SUC_lexicon_tags_77 },
    { 0xfe3cd45a, SUC_lexicon_tags_60 },
    { 0xf57cd45a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7cd45f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xf29cd462, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbcd466, SUC_lexicon_tags_38 },
    { 0x7bbcd466, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc2fcd469, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47cd471, SUC_lexicon_tags_27 },
    { 0xe15cd472, SUC_lexicon_tags_22 },
    { 0x109cd472, SUC_lexicon_tags_21 },
    { 0x70fcd474, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9cd478, SUC_lexicon_tags_2 },
    { 0x813cd479, SUC_lexicon_tags_30 },
    { 0xcdbcd47a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457cd47e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5cd484, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09cd48a, SUC_lexicon_tags_82 },
    { 0xcafcd48b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7cd490, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x26fcd492, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf3cd494, SUC_lexicon_tags_8 },
    { 0x4c5cd495, SUC_lexicon_tags_43 },
    { 0x957cd496, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5cd49a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd7cd49c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x725cd49f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x381cd4a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9cd4a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5cd4a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245cd4ae, SUC_lexicon_tags_27 },
    { 0xda9cd4af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7cd4b3, SUC_lexicon_tags_17 },
    { 0xe9dcd4b4, SUC_lexicon_tags_1 },
    { 0x98fcd4b5, SUC_lexicon_tags_11 },
    { 0xc23cd4b6, SUC_lexicon_tags_16 },
    { 0x4dfcd4b7, SUC_lexicon_tags_18 },
    { 0xe1cd4b7, SUC_lexicon_tags_12 },
    { 0x5cfcd4b7, SUC_lexicon_tags_38 },
    { 0x77fcd4b9, SUC_lexicon_tags_17 },
    { 0xc9cd4bb, SUC_lexicon_tags_25 },
    { 0x6bfcd4bb, SUC_lexicon_tags_64 },
    { 0xd5fcd4bb, SUC_lexicon_tags_7 },
    { 0x1c9cd4ba, SUC_lexicon_tags_6 },
    { 0x13bcd4bf, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x557cd4c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe33cd4c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43cd4c8, SUC_lexicon_tags_23 },
    { 0xaf3cd4c9, SUC_lexicon_tags_10 },
    { 0x1e9cd4ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175cd4d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bcd4d6, SUC_lexicon_tags_63 },
    { 0x701cd4d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x595cd4d9, SUC_lexicon_tags_57 },
    { 0x58bcd4da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x235cd4dc, SUC_lexicon_tags_25 },
    { 0x633cd4dd, SUC_lexicon_tags_65 },
    { 0x8bfcd4de, SUC_lexicon_tags_17 },
    { 0x475cd4de, SUC_lexicon_tags_33 },
    { 0xc6dcd4e0, SUC_lexicon_tags_30 },
    { 0x7cfcd4de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7cd4e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817cd4ea, SUC_lexicon_tags_7 },
    { 0x96bcd4eb, SUC_lexicon_tags_29 },
    { 0x7b7cd4eb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd87cd4ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1cd4f1, SUC_lexicon_tags_31 },
    { 0xe7cd4f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53cd4f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5cd4fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641cd4fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x647cd501, SUC_lexicon_tags_11 },
    { 0x1a5cd501, SUC_lexicon_tags_14 },
    { 0xdbcd503, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cd507, SUC_lexicon_tags_30 },
    { 0x355cd508, SUC_lexicon_tags_37 },
    { 0xc17cd509, SUC_lexicon_tags_47 },
    { 0x44fcd50a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1cd50d, SUC_lexicon_tags_27 },
    { 0x4d5cd50e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485cd513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85cd518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bcd51b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x941cd51e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabcd522, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf99cd524, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efcd52b, SUC_lexicon_tags_2 },
    { 0xf99cd52b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705cd532, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbffcd534, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7dbcd536, SUC_lexicon_tags_2 },
    { 0xff5cd537, SUC_lexicon_tags_15 },
    { 0x78bcd536, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5cd53b, SUC_lexicon_tags_60 },
    { 0xb7cd53c, SUC_lexicon_tags_2 },
    { 0xf0bcd53b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8dcd53f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4c9cd541, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x251cd543, SUC_lexicon_tags_9 },
    { 0xa85cd544, SUC_lexicon_tags_25 },
    { 0x47bcd545, SUC_lexicon_tags_38 },
    { 0x12bcd546, SUC_lexicon_tags_31 },
    { 0x893cd545, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1cd54e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc3cd550, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf55cd552, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x217cd555, SUC_lexicon_tags_21 },
    { 0xf75cd556, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcfcd558, SUC_lexicon_tags_11 },
    { 0xafbcd559, SUC_lexicon_tags_48 },
    { 0x2e9cd55a, SUC_lexicon_tags_8 },
    { 0x5c7cd55b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfcd55e, SUC_lexicon_tags_2 },
    { 0x559cd55f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x617cd561, SUC_lexicon_tags_32 },
    { 0xafdcd562, SUC_lexicon_tags_37 },
    { 0x21cd563, SUC_lexicon_tags_2 },
    { 0x2c9cd564, SUC_lexicon_tags_13 },
    { 0x8cdcd562, SUC_lexicon_tags_9 },
    { 0x3bfcd566, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05cd569, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81cd56e, SUC_lexicon_tags_47 },
    { 0xacdcd56f, SUC_lexicon_tags_5 },
    { 0x701cd570, SUC_lexicon_tags_46 },
    { 0x7b3cd570, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fcd576, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68dcd57c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x959cd57e, SUC_lexicon_tags_8 },
    { 0x1cbcd57f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd1dcd581, SUC_lexicon_tags_39 },
    { 0x157cd581, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7cd589, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abcd58c, SUC_lexicon_tags_16 },
    { 0xa19cd58d, SUC_lexicon_tags_23 },
    { 0xb33cd58c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95cd593, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9cd596, SUC_lexicon_tags_25 },
    { 0xb8dcd596, SUC_lexicon_tags_25 },
    { 0x73cd598, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x60dcd59a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x727cd59d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21dcd59f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41cd5a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffcd5a5, SUC_lexicon_tags_11 },
    { 0xec5cd5a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fcd5ad, SUC_lexicon_tags_47 },
    { 0x42dcd5ad, SUC_lexicon_tags_37 },
    { 0x951cd5af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x761cd5b2, SUC_lexicon_tags_43 },
    { 0xd0dcd5b3, SUC_lexicon_tags_25 },
    { 0x4d3cd5b4, SUC_lexicon_tags_45 },
    { 0x49cd5b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c9cd5b7, SUC_lexicon_tags_45 },
    { 0x5abcd5b8, SUC_lexicon_tags_68 },
    { 0x38fcd5b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bcd5be, SUC_lexicon_tags_15 },
    { 0x2c5cd5be, SUC_lexicon_tags_14 },
    { 0xa2fcd5c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cd5c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5cd5cd, SUC_lexicon_tags_43 },
    { 0xae7cd5ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99cd5d2, SUC_lexicon_tags_2 },
    { 0x5d9cd5d3, SUC_lexicon_tags_7 },
    { 0xfa9cd5d4, SUC_lexicon_tags_1 },
    { 0x267cd5d4, SUC_lexicon_tags_45 },
    { 0xb7dcd5d6, SUC_lexicon_tags_12 },
    { 0x98bcd5d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x51bcd5da, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01cd5df, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x851cd5e1, SUC_lexicon_tags_57 },
    { 0xda7cd5e2, SUC_lexicon_tags_26 },
    { 0x1ebcd5e3, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0xa1cd5e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27cd5e7, SUC_lexicon_tags_65 },
    { 0xb95cd5e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11cd5eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x45bcd5ee, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8c3cd5f0, SUC_lexicon_tags_43 },
    { 0xcdfcd5f1, SUC_lexicon_tags_27 },
    { 0xd4dcd5f2, SUC_lexicon_tags_1 },
    { 0xacdcd5f3, SUC_lexicon_tags_7 },
    { 0x41fcd5f4, SUC_lexicon_tags_27 },
    { 0xa29cd5f4, SUC_lexicon_tags_49 },
    { 0x6c3cd5f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef9cd5f8, SUC_lexicon_tags_7 },
    { 0x49fcd5f9, SUC_lexicon_tags_46 },
    { 0x141cd5fa, SUC_lexicon_tags_32 },
    { 0xe1fcd5fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fcd600, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7cd608, SUC_lexicon_tags_2 },
    { 0xe27cd609, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9b3cd60b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd29cd60d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x83dcd60f, SUC_lexicon_tags_49 },
    { 0xc53cd610, SUC_lexicon_tags_14 },
    { 0xcfdcd611, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833cd61c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831cd622, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbcd62b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83cd633, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5cd639, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa61cd63b, SUC_lexicon_tags_12 },
    { 0xc7cd63b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4bcd63e, SUC_lexicon_tags_66 },
    { 0x7abcd63e, SUC_lexicon_tags_38 },
    { 0xe73cd640, SUC_lexicon_tags_0 },
    { 0x805cd641, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5cd644, SUC_lexicon_tags_13 },
    { 0x7c7cd644, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dcd64c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739cd655, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x18bcd657, SUC_lexicon_tags_7 },
    { 0xc99cd658, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c7cd65a, SUC_lexicon_tags_68 },
    { 0x9a5cd65a, SUC_lexicon_tags_55 },
    { 0xaf7cd65b, SUC_lexicon_tags_43 },
    { 0x123cd65a, SUC_lexicon_tags_15 },
    { 0xe29cd65e, SUC_lexicon_tags_26 },
    { 0x52bcd65f, SUC_lexicon_tags_31 },
    { 0x697cd65d, SUC_lexicon_tags_37 },
    { 0x837cd660, SUC_lexicon_tags_69 },
    { 0xc81cd662, SUC_lexicon_tags_49 },
    { 0x389cd663, SUC_lexicon_tags_11 },
    { 0x2dcd65e, SUC_lexicon_tags_1 },
    { 0xebcd665, SUC_lexicon_tags_9 },
    { 0x73dcd65a, SUC_lexicon_tags_7 },
    { 0xf1cd665, SUC_lexicon_tags_38 },
    { 0x73fcd668, SUC_lexicon_tags_2 },
    { 0xab1cd660, SUC_lexicon_tags_29 },
    { 0x9c5cd668, SUC_lexicon_tags_15 },
    { 0xeb1cd664, SUC_lexicon_tags_12 },
    { 0x8dfcd65e, SUC_lexicon_tags_27 },
    { 0x791cd66d, SUC_lexicon_tags_2 },
    { 0x4b9cd66c, SUC_lexicon_tags_15 },
    { 0x4d9cd66f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdfdcd671, SUC_lexicon_tags_91 },
    { 0x83dcd671, SUC_lexicon_tags_7 },
    { 0x241cd672, SUC_lexicon_tags_11 },
    { 0xae5cd674, SUC_lexicon_tags_17 },
    { 0x4ddcd675, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11cd678, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x133cd67a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5cd67d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5cd681, SUC_lexicon_tags_60 },
    { 0x183cd682, SUC_lexicon_tags_0 },
    { 0x357cd682, SUC_lexicon_tags_25 },
    { 0x32bcd681, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fcd687, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa6fcd689, SUC_lexicon_tags_12 },
    { 0x185cd68a, SUC_lexicon_tags_2 },
    { 0x27dcd68b, SUC_lexicon_tags_23 },
    { 0x42bcd68c, SUC_lexicon_tags_2 },
    { 0xbc5cd68d, SUC_lexicon_tags_26 },
    { 0xf1fcd68a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x425cd690, SUC_lexicon_tags_4 },
    { 0x109cd691, SUC_lexicon_tags_49 },
    { 0x753cd691, SUC_lexicon_tags_27 },
    { 0xb37cd693, SUC_lexicon_tags_11 },
    { 0x443cd692, SUC_lexicon_tags_60 },
    { 0x843cd695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26bcd697, SUC_lexicon_tags_0 },
    { 0xe2dcd697, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb4bcd69a, SUC_lexicon_tags_37 },
    { 0x93cd69b, SUC_lexicon_tags_29 },
    { 0x851cd69a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957cd6a1, SUC_lexicon_tags_1 },
    { 0x82dcd6a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bcd6a5, SUC_lexicon_tags_4 },
    { 0x15fcd6a5, SUC_lexicon_tags_7 },
    { 0xcb5cd6a7, SUC_lexicon_tags_17 },
    { 0xb69cd6a8, SUC_lexicon_tags_13 },
    { 0x587cd6a9, SUC_lexicon_tags_37 },
    { 0x9a1cd6a9, SUC_lexicon_tags_33 },
    { 0x1d3cd6a7, SUC_lexicon_tags_9 },
    { 0xe87cd6a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3cd6b2, SUC_lexicon_tags_14 },
    { 0x27dcd6b2, SUC_lexicon_tags_34 },
    { 0x87dcd6b4, SUC_lexicon_tags_11 },
    { 0x92bcd6b5, SUC_lexicon_tags_45 },
    { 0x8e5cd6b6, SUC_lexicon_tags_7 },
    { 0x9ffcd6b6, SUC_lexicon_tags_27 },
    { 0xa7cd6b5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fcd6bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63cd6c0, SUC_lexicon_tags_83 },
    { 0x5f3cd6c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7cd6c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1cd6c8, SUC_lexicon_tags_15 },
    { 0xa57cd6c8, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x21cd6cb, SUC_lexicon_tags_6 },
    { 0x8d1cd6cb, SUC_lexicon_tags_25 },
    { 0x43fcd6cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadcd6d4, SUC_lexicon_tags_25 },
    { 0x3c9cd6d5, SUC_lexicon_tags_2 },
    { 0x655cd6d6, SUC_lexicon_tags_61 },
    { 0x79dcd6d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x599cd6d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1cbcd6db, SUC_lexicon_tags_2 },
    { 0xa45cd6dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe07cd6de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbcd6e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55dcd6eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x53dcd6ee, SUC_lexicon_tags_26 },
    { 0x1c9cd6ee, SUC_lexicon_tags_2 },
    { 0x23fcd6ee, SUC_lexicon_tags_11 },
    { 0x81fcd6f1, SUC_lexicon_tags_15 },
    { 0x761cd6f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19cd6f5, SUC_lexicon_tags_9 },
    { 0x1f9cd6f6, SUC_lexicon_tags_63 },
    { 0xf5bcd6f5, SUC_lexicon_tags_15 },
    { 0x877cd6f8, SUC_lexicon_tags_8 },
    { 0x2afcd6f8, SUC_lexicon_tags_25 },
    { 0x76dcd6f9, SUC_lexicon_tags_27 },
    { 0x53fcd6fb, SUC_lexicon_tags_6 },
    { 0xf97cd6fc, SUC_lexicon_tags_49 },
    { 0x647cd6f9, SUC_lexicon_tags_45 },
    { 0x527cd6fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3cd701, SUC_lexicon_tags_25 },
    { 0x717cd702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53cd707, SUC_lexicon_tags_1 },
    { 0x4a1cd708, SUC_lexicon_tags_39 },
    { 0x5b3cd709, SUC_lexicon_tags_20 },
    { 0xffbcd70a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa77cd70c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adcd711, SUC_lexicon_tags_39 },
    { 0x25bcd712, SUC_lexicon_tags_12 },
    { 0x6d1cd711, SUC_lexicon_tags_30 },
    { 0xb93cd713, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeb3cd716, SUC_lexicon_tags_29 },
    { 0x145cd717, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73bcd719, SUC_lexicon_tags_43 },
    { 0x78dcd71a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dcd71e, SUC_lexicon_tags_28 },
    { 0x2e7cd71e, SUC_lexicon_tags_68 },
    { 0x799cd720, SUC_lexicon_tags_61 },
    { 0x2f5cd71f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bcd72b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x31bcd72d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x297cd730, SUC_lexicon_tags_12 },
    { 0x681cd731, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35cd738, SUC_lexicon_tags_47 },
    { 0x483cd739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987cd740, SUC_lexicon_tags_12 },
    { 0xff5cd740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x145cd743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfcd74d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5cd752, SUC_lexicon_tags_60 },
    { 0x409cd753, SUC_lexicon_tags_118 },
    { 0x409cd754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579cd759, SUC_lexicon_tags_21 },
    { 0xa9bcd75a, SUC_lexicon_tags_193 },
    { 0x86fcd75a, SUC_lexicon_tags_2 },
    { 0x24dcd75a, SUC_lexicon_tags_49 },
    { 0x837cd75d, SUC_lexicon_tags_43 },
    { 0x377cd75d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbcd767, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5cd76a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x235cd76c, SUC_lexicon_tags_1 },
    { 0xd0bcd76c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x411cd76f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7abcd771, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86fcd773, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dcd778, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617cd77d, SUC_lexicon_tags_84 },
    { 0x651cd77e, SUC_lexicon_tags_0 },
    { 0xe5cd77f, SUC_lexicon_tags_33 },
    { 0x18bcd780, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe3fcd782, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd55cd784, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8dbcd786, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41cd78a, SUC_lexicon_tags_1 },
    { 0xc3bcd78a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x80dcd78d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7cd794, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3efcd796, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3cd799, SUC_lexicon_tags_11 },
    { 0x463cd79a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dcd79e, SUC_lexicon_tags_51 },
    { 0x7bfcd79e, SUC_lexicon_tags_63 },
    { 0x689cd7a0, SUC_lexicon_tags_30 },
    { 0x59bcd7a1, SUC_lexicon_tags_34 },
    { 0xa63cd7a1, SUC_lexicon_tags_11 },
    { 0x29fcd7a3, SUC_lexicon_tags_17 },
    { 0x433cd7a0, SUC_lexicon_tags_17 },
    { 0x849cd7a4, SUC_lexicon_tags_37 },
    { 0x109cd7a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdcd7aa, SUC_lexicon_tags_43 },
    { 0x39dcd7aa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7efcd7ad, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01cd7b0, SUC_lexicon_tags_43 },
    { 0x215cd7b1, SUC_lexicon_tags_11 },
    { 0x60bcd7b1, SUC_lexicon_tags_17 },
    { 0x2edcd7b3, SUC_lexicon_tags_37 },
    { 0x9bfcd7b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedcd7bc, SUC_lexicon_tags_43 },
    { 0x133cd7bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7cd7c2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3cd7c6, SUC_lexicon_tags_33 },
    { 0x5a5cd7c7, SUC_lexicon_tags_17 },
    { 0xcb9cd7c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24fcd7ca, SUC_lexicon_tags_63 },
    { 0x443cd7cb, SUC_lexicon_tags_37 },
    { 0x915cd7cc, SUC_lexicon_tags_43 },
    { 0xd8dcd7ca, SUC_lexicon_tags_11 },
    { 0xc69cd7cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x497cd7d0, SUC_lexicon_tags_11 },
    { 0x86fcd7d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7cd7d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3cd7de, SUC_lexicon_tags_29 },
    { 0x705cd7df, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x449cd7e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3cd7e5, SUC_lexicon_tags_12 },
    { 0xaa7cd7e6, SUC_lexicon_tags_26 },
    { 0xeb9cd7e7, SUC_lexicon_tags_38 },
    { 0x257cd7e8, SUC_lexicon_tags_23 },
    { 0x401cd7e9, SUC_lexicon_tags_2 },
    { 0x96bcd7ea, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fcd7ed, SUC_lexicon_tags_4 },
    { 0xf23cd7ee, SUC_lexicon_tags_8 },
    { 0x2cfcd7ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabcd7f3, SUC_lexicon_tags_1 },
    { 0xdcdcd7f4, SUC_lexicon_tags_9 },
    { 0x95bcd7f4, SUC_lexicon_tags_15 },
    { 0x243cd7f6, SUC_lexicon_tags_12 },
    { 0x90fcd7f6, SUC_lexicon_tags_6 },
    { 0x79fcd7f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bcd7fd, SUC_lexicon_tags_1 },
    { 0xebbcd7fe, SUC_lexicon_tags_2 },
    { 0x263cd7ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdcd806, SUC_lexicon_tags_75 },
    { 0xdedcd807, SUC_lexicon_tags_17 },
    { 0xb17cd807, SUC_lexicon_tags_60 },
    { 0x5a1cd809, SUC_lexicon_tags_2 },
    { 0xfbbcd80a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedcd80f, SUC_lexicon_tags_12 },
    { 0x913cd810, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75cd813, SUC_lexicon_tags_43 },
    { 0x34bcd814, SUC_lexicon_tags_15 },
    { 0xacdcd814, SUC_lexicon_tags_60 },
    { 0x6f3cd815, SUC_lexicon_tags_54 },
    { 0x3cbcd814, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2cfcd819, SUC_lexicon_tags_49 },
    { 0x155cd81a, SUC_lexicon_tags_567 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411cd81e, SUC_lexicon_tags_17 },
    { 0xaffcd81f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11cd824, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd0dcd826, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799cd82b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb7fcd82d, SUC_lexicon_tags_25 },
    { 0x975cd82d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4fcd830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307cd834, SUC_lexicon_tags_197 },
    { 0xefcd835, SUC_lexicon_tags_17 },
    { 0x765cd836, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcddcd838, SUC_lexicon_tags_21 },
    { 0x765cd839, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51cd841, SUC_lexicon_tags_15 },
    { 0xa1cd842, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbcd84c, SUC_lexicon_tags_70 },
    { 0x9fdcd84d, SUC_lexicon_tags_44 },
    { 0xa41cd84d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x133cd850, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffcd854, SUC_lexicon_tags_12 },
    { 0x295cd855, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf5fcd857, SUC_lexicon_tags_113 },
    { 0xdf9cd858, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdcd861, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x469cd864, SUC_lexicon_tags_40 },
    { 0xc5fcd865, SUC_lexicon_tags_57 },
    { 0xe51cd865, SUC_lexicon_tags_11 },
    { 0xce1cd867, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bcd86a, SUC_lexicon_tags_23 },
    { 0x803cd86a, SUC_lexicon_tags_17 },
    { 0x58fcd86b, SUC_lexicon_tags_108 },
    { 0x7f1cd86d, SUC_lexicon_tags_45 },
    { 0xe7dcd86d, SUC_lexicon_tags_47 },
    { 0x9edcd86f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dcd873, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7cd876, SUC_lexicon_tags_32 },
    { 0xf17cd876, SUC_lexicon_tags_1 },
    { 0x86fcd878, SUC_lexicon_tags_12 },
    { 0xb57cd876, SUC_lexicon_tags_1 },
    { 0xda1cd87a, SUC_lexicon_tags_49 },
    { 0x2e9cd87b, SUC_lexicon_tags_12 },
    { 0x6fdcd87c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3cd87f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135cd883, SUC_lexicon_tags_17 },
    { 0x281cd884, SUC_lexicon_tags_12 },
    { 0xf53cd885, SUC_lexicon_tags_8 },
    { 0xec3cd885, SUC_lexicon_tags_35 },
    { 0x74dcd887, SUC_lexicon_tags_27 },
    { 0x9a9cd887, SUC_lexicon_tags_38 },
    { 0x7f7cd888, SUC_lexicon_tags_37 },
    { 0x221cd887, SUC_lexicon_tags_3 },
    { 0x84dcd88b, SUC_lexicon_tags_1 },
    { 0xa7cd889, SUC_lexicon_tags_47 },
    { 0x857cd88d, SUC_lexicon_tags_0 },
    { 0xb39cd88e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59cd890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bcd895, SUC_lexicon_tags_60 },
    { 0x22bcd896, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dcd89b, SUC_lexicon_tags_79 },
    { 0x2fdcd89c, SUC_lexicon_tags_16 },
    { 0x5cd89c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fcd8a5, SUC_lexicon_tags_83 },
    { 0x49cd8a5, SUC_lexicon_tags_4 },
    { 0x35dcd8a7, SUC_lexicon_tags_11 },
    { 0x5d7cd8a7, SUC_lexicon_tags_47 },
    { 0xa4bcd8a9, SUC_lexicon_tags_2 },
    { 0x22fcd8aa, SUC_lexicon_tags_2 },
    { 0xbdfcd8a5, SUC_lexicon_tags_12 },
    { 0x803cd8aa, SUC_lexicon_tags_63 },
    { 0xbc3cd8ad, SUC_lexicon_tags_35 },
    { 0x47bcd8ae, SUC_lexicon_tags_11 },
    { 0xb73cd8ad, SUC_lexicon_tags_25 },
    { 0x623cd8b0, SUC_lexicon_tags_49 },
    { 0xbbcd8b1, SUC_lexicon_tags_48 },
    { 0xc0fcd8b1, SUC_lexicon_tags_11 },
    { 0x4f3cd8b3, SUC_lexicon_tags_19 },
    { 0xcb1cd8b4, SUC_lexicon_tags_30 },
    { 0xcb5cd8b5, SUC_lexicon_tags_7 },
    { 0xa17cd8ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcd8bc, SUC_lexicon_tags_17 },
    { 0xe1fcd8bd, SUC_lexicon_tags_55 },
    { 0x20dcd8be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bcd8c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x867cd8c4, SUC_lexicon_tags_26 },
    { 0x909cd8c5, SUC_lexicon_tags_11 },
    { 0x5cdcd8c6, SUC_lexicon_tags_11 },
    { 0x815cd8c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bcd8d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37cd8dc, SUC_lexicon_tags_11 },
    { 0x627cd8dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dcd8e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635cd8e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7cd8e7, SUC_lexicon_tags_2 },
    { 0x831cd8e7, SUC_lexicon_tags_661 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dcd8ec, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x585cd8ee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41cd8f2, SUC_lexicon_tags_34 },
    { 0xcbdcd8f3, SUC_lexicon_tags_17 },
    { 0x763cd8f2, SUC_lexicon_tags_37 },
    { 0xf23cd8f2, SUC_lexicon_tags_12 },
    { 0x4b9cd8f6, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69cd8fb, SUC_lexicon_tags_2 },
    { 0x57bcd8fb, SUC_lexicon_tags_11 },
    { 0x963cd8fd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x13bcd8ff, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bcd903, SUC_lexicon_tags_38 },
    { 0x785cd904, SUC_lexicon_tags_12 },
    { 0x9f5cd905, SUC_lexicon_tags_37 },
    { 0xaedcd903, SUC_lexicon_tags_7 },
    { 0xb5bcd904, SUC_lexicon_tags_25 },
    { 0xdebcd908, SUC_lexicon_tags_16 },
    { 0xf0fcd909, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fcd90d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bcd910, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dcd919, SUC_lexicon_tags_439 },
    { 0xc8fcd91a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adcd923, SUC_lexicon_tags_15 },
    { 0x5e7cd923, SUC_lexicon_tags_15 },
    { 0xa07cd925, SUC_lexicon_tags_2 },
    { 0xf77cd924, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5cd932, SUC_lexicon_tags_17 },
    { 0xcefcd933, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1cd936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5cd93b, SUC_lexicon_tags_11 },
    { 0xff7cd93c, SUC_lexicon_tags_7 },
    { 0x35fcd93d, SUC_lexicon_tags_82 },
    { 0x9b3cd93c, SUC_lexicon_tags_25 },
    { 0x5b1cd93f, SUC_lexicon_tags_13 },
    { 0xffdcd93f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5cd947, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x119cd949, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9cd94f, SUC_lexicon_tags_31 },
    { 0xb7bcd950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fcd954, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83cd958, SUC_lexicon_tags_47 },
    { 0xd0fcd959, SUC_lexicon_tags_15 },
    { 0x963cd958, SUC_lexicon_tags_15 },
    { 0xf6dcd95b, SUC_lexicon_tags_82 },
    { 0xd8bcd959, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dfcd95e, SUC_lexicon_tags_25 },
    { 0xb89cd95e, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9cd963, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x291cd965, SUC_lexicon_tags_10 },
    { 0x9e1cd966, SUC_lexicon_tags_9 },
    { 0xf83cd966, SUC_lexicon_tags_37 },
    { 0xa61cd966, SUC_lexicon_tags_105 },
    { 0x71bcd969, SUC_lexicon_tags_32 },
    { 0xdbbcd96a, SUC_lexicon_tags_6 },
    { 0xa71cd96b, SUC_lexicon_tags_17 },
    { 0x383cd96a, SUC_lexicon_tags_17 },
    { 0x9bdcd96a, SUC_lexicon_tags_7 },
    { 0x1e7cd968, SUC_lexicon_tags_9 },
    { 0xb97cd96c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbcd973, SUC_lexicon_tags_37 },
    { 0x5adcd974, SUC_lexicon_tags_37 },
    { 0x4e9cd975, SUC_lexicon_tags_63 },
    { 0x5e9cd975, SUC_lexicon_tags_62 },
    { 0x1c7cd977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x435cd97a, SUC_lexicon_tags_37 },
    { 0x357cd97b, SUC_lexicon_tags_55 },
    { 0xdbbcd97c, SUC_lexicon_tags_82 },
    { 0x39cd97d, SUC_lexicon_tags_10 },
    { 0x9a7cd97e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9cd983, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559cd988, SUC_lexicon_tags_30 },
    { 0xf53cd989, SUC_lexicon_tags_2 },
    { 0xdcfcd98a, SUC_lexicon_tags_15 },
    { 0x9fdcd989, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bcd98f, SUC_lexicon_tags_11 },
    { 0x245cd990, SUC_lexicon_tags_17 },
    { 0xc99cd991, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5cd996, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dcd99c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa6fcd99e, SUC_lexicon_tags_27 },
    { 0x83bcd99f, SUC_lexicon_tags_27 },
    { 0x79cd9a0, SUC_lexicon_tags_87 },
    { 0x279cd9a1, SUC_lexicon_tags_37 },
    { 0x6e1cd9a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc3cd9a4, SUC_lexicon_tags_1 },
    { 0xc47cd9a5, SUC_lexicon_tags_57 },
    { 0x35fcd9a4, SUC_lexicon_tags_87 },
    { 0xdedcd9a7, SUC_lexicon_tags_10 },
    { 0xcd1cd9a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5f1cd9aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x865cd9ad, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x749cd9b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x915cd9b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fcd9b5, SUC_lexicon_tags_43 },
    { 0xc4fcd9b6, SUC_lexicon_tags_17 },
    { 0xd0bcd9b6, SUC_lexicon_tags_4 },
    { 0xbddcd9b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x477cd9ba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fcd9bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fcd9c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7cd9c7, SUC_lexicon_tags_15 },
    { 0xa79cd9c8, SUC_lexicon_tags_49 },
    { 0xf21cd9c7, SUC_lexicon_tags_43 },
    { 0x9a3cd9ca, SUC_lexicon_tags_43 },
    { 0xb95cd9cb, SUC_lexicon_tags_37 },
    { 0xc59cd9cc, SUC_lexicon_tags_17 },
    { 0x701cd9cd, SUC_lexicon_tags_37 },
    { 0x379cd9ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5cd9d3, SUC_lexicon_tags_27 },
    { 0x1fdcd9d4, SUC_lexicon_tags_33 },
    { 0x42fcd9d5, SUC_lexicon_tags_3 },
    { 0x73bcd9d6, SUC_lexicon_tags_23 },
    { 0x65cd9d7, SUC_lexicon_tags_65 },
    { 0x985cd9d7, SUC_lexicon_tags_2 },
    { 0x6e7cd9d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9cd9dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1cd9e1, SUC_lexicon_tags_1 },
    { 0xf29cd9e2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59cd9e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x135cd9e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5cd9ee, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fcd9f1, SUC_lexicon_tags_38 },
    { 0xf1fcd9f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1fcd9f4, SUC_lexicon_tags_33 },
    { 0x919cd9f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64dcd9fa, SUC_lexicon_tags_17 },
    { 0x8a9cd9fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dcda00, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1cda06, SUC_lexicon_tags_43 },
    { 0xb39cda07, SUC_lexicon_tags_15 },
    { 0x481cda08, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3cda0d, SUC_lexicon_tags_37 },
    { 0xb59cda0e, SUC_lexicon_tags_25 },
    { 0xd01cda0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb99cda11, SUC_lexicon_tags_37 },
    { 0x42dcda12, SUC_lexicon_tags_45 },
    { 0x645cda13, SUC_lexicon_tags_45 },
    { 0x255cda14, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1bdcda16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519cda1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fcda1d, SUC_lexicon_tags_1 },
    { 0xb39cda1d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999cda23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bcda27, SUC_lexicon_tags_10 },
    { 0xe2dcda27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd49cda2a, SUC_lexicon_tags_7 },
    { 0xaddcda2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fcda2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bcda32, SUC_lexicon_tags_15 },
    { 0x743cda33, SUC_lexicon_tags_37 },
    { 0x677cda34, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fcda39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3f7cda3b, SUC_lexicon_tags_25 },
    { 0x135cda3c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433cda45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x561cda48, SUC_lexicon_tags_14 },
    { 0x753cda49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fcda4c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x181cda4f, SUC_lexicon_tags_2 },
    { 0x4f7cda50, SUC_lexicon_tags_25 },
    { 0xd5bcda50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x777cda54, SUC_lexicon_tags_46 },
    { 0xcbfcda55, SUC_lexicon_tags_49 },
    { 0x89dcda56, SUC_lexicon_tags_7 },
    { 0xf21cda57, SUC_lexicon_tags_6 },
    { 0x1e9cda58, SUC_lexicon_tags_27 },
    { 0xa53cda59, SUC_lexicon_tags_49 },
    { 0x179cda5a, SUC_lexicon_tags_17 },
    { 0x42bcda54, SUC_lexicon_tags_25 },
    { 0x649cda5c, SUC_lexicon_tags_70 },
    { 0x22dcda54, SUC_lexicon_tags_37 },
    { 0x54dcda5e, SUC_lexicon_tags_15 },
    { 0xef3cda56, SUC_lexicon_tags_6 },
    { 0x4f3cda55, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c1cda62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231cda6b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dcda72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bcda75, SUC_lexicon_tags_6 },
    { 0xc5dcda76, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x843cda78, SUC_lexicon_tags_25 },
    { 0x243cda78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315cda7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407cda84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3cda87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bcda8c, SUC_lexicon_tags_1 },
    { 0xf77cda8c, SUC_lexicon_tags_43 },
    { 0x39bcda8d, SUC_lexicon_tags_93 },
    { 0xe41cda8f, SUC_lexicon_tags_10 },
    { 0xd1bcda8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedcda94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19bcda96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3cda9b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xed1cda9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7abcda9f, SUC_lexicon_tags_37 },
    { 0xf1cdaa0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7fdcdaa2, SUC_lexicon_tags_79 },
    { 0x10fcdaa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fcdaa6, SUC_lexicon_tags_138 },
    { 0x135cdaa7, SUC_lexicon_tags_29 },
    { 0xe7fcdaa8, SUC_lexicon_tags_25 },
    { 0x3b7cdaa9, SUC_lexicon_tags_25 },
    { 0x4bcdaaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf23cdaac, SUC_lexicon_tags_2 },
    { 0x417cdaad, SUC_lexicon_tags_17 },
    { 0x3fcdaae, SUC_lexicon_tags_44 },
    { 0x86bcdaaf, SUC_lexicon_tags_0 },
    { 0xa8dcdab0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b1cdab2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bcdab5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd21cdab7, SUC_lexicon_tags_19 },
    { 0xaedcdab8, SUC_lexicon_tags_37 },
    { 0xae3cdab8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21cdabc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64dcdac0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bcdac7, SUC_lexicon_tags_14 },
    { 0xfafcdac8, SUC_lexicon_tags_11 },
    { 0x487cdac9, SUC_lexicon_tags_17 },
    { 0x8b9cdaca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5cdace, SUC_lexicon_tags_5 },
    { 0x4b3cdace, SUC_lexicon_tags_63 },
    { 0x4f9cdad0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fcdad3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5efcdad5, SUC_lexicon_tags_37 },
    { 0xb5bcdad6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x66fcdad8, SUC_lexicon_tags_7 },
    { 0x4f9cdad9, SUC_lexicon_tags_14 },
    { 0xca7cdad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c7cdadc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55cdae2, SUC_lexicon_tags_55 },
    { 0xefcdae2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39cdae7, SUC_lexicon_tags_46 },
    { 0xeb7cdae8, SUC_lexicon_tags_15 },
    { 0xd45cdae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x76bcdaec, SUC_lexicon_tags_37 },
    { 0xd87cdaed, SUC_lexicon_tags_12 },
    { 0x751cdaec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afcdaf2, SUC_lexicon_tags_6 },
    { 0x975cdaf3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335cdafa, SUC_lexicon_tags_46 },
    { 0x65bcdafb, SUC_lexicon_tags_30 },
    { 0x973cdafc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5cdb06, SUC_lexicon_tags_6 },
    { 0x46dcdb06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc33cdb09, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5b9cdb0b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287cdb11, SUC_lexicon_tags_29 },
    { 0x181cdb12, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf2fcdb14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e3cdb16, SUC_lexicon_tags_9 },
    { 0x94bcdb16, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fcdb1b, SUC_lexicon_tags_37 },
    { 0x44fcdb1c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3dfcdb1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7cdb23, SUC_lexicon_tags_1 },
    { 0x203cdb24, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227cdb29, SUC_lexicon_tags_1 },
    { 0x685cdb2a, SUC_lexicon_tags_7 },
    { 0x429cdb2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d3cdb2d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x113cdb30, SUC_lexicon_tags_27 },
    { 0xca7cdb31, SUC_lexicon_tags_11 },
    { 0x935cdb32, SUC_lexicon_tags_15 },
    { 0x37cdb31, SUC_lexicon_tags_26 },
    { 0xe9bcdb34, SUC_lexicon_tags_0 },
    { 0x7a7cdb34, SUC_lexicon_tags_2 },
    { 0x77fcdb35, SUC_lexicon_tags_11 },
    { 0xecfcdb36, SUC_lexicon_tags_17 },
    { 0x1bcdb38, SUC_lexicon_tags_165 },
    { 0x46bcdb39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bcdb3b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dcdb3e, SUC_lexicon_tags_43 },
    { 0xf2dcdb3f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589cdb44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bcdb4a, SUC_lexicon_tags_17 },
    { 0x68dcdb4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdcdb4e, SUC_lexicon_tags_13 },
    { 0xbf7cdb4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfcdb52, SUC_lexicon_tags_37 },
    { 0x427cdb52, SUC_lexicon_tags_37 },
    { 0xc93cdb54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117cdb5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61cdb5f, SUC_lexicon_tags_19 },
    { 0x2ebcdb60, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59cdb68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3cdb6b, SUC_lexicon_tags_25 },
    { 0xe33cdb6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40fcdb6e, SUC_lexicon_tags_38 },
    { 0xcbcdb6f, SUC_lexicon_tags_45 },
    { 0xcafcdb70, SUC_lexicon_tags_55 },
    { 0x7c3cdb6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56bcdb73, SUC_lexicon_tags_14 },
    { 0x3adcdb73, SUC_lexicon_tags_17 },
    { 0x77bcdb75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13fcdb77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bcdb7a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67cdb80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x217cdb83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfcdb87, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3cdb8a, SUC_lexicon_tags_7 },
    { 0x279cdb8b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45cdb91, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0x121cdb94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fdcdb96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfcdb9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3cdba0, SUC_lexicon_tags_47 },
    { 0x165cdba0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6cdcdba3, SUC_lexicon_tags_63 },
    { 0x801cdba4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x191cdba6, SUC_lexicon_tags_17 },
    { 0x643cdba7, SUC_lexicon_tags_46 },
    { 0xef1cdba7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23cdbaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x503cdbac, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x451cdbae, SUC_lexicon_tags_12 },
    { 0xbadcdbaf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc0bcdbb1, SUC_lexicon_tags_63 },
    { 0xbbcdbb1, SUC_lexicon_tags_26 },
    { 0x8abcdbb3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75cdbb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447cdbbf, SUC_lexicon_tags_69 },
    { 0x2d3cdbc0, SUC_lexicon_tags_6 },
    { 0xc47cdbc0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x657cdbc3, SUC_lexicon_tags_47 },
    { 0x6dfcdbc4, SUC_lexicon_tags_87 },
    { 0x8f3cdbc5, SUC_lexicon_tags_65 },
    { 0x187cdbc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbcdbc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x991cdbcb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x929cdbce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x135cdbd1, SUC_lexicon_tags_40 },
    { 0x7a3cdbd2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1cdbd5, SUC_lexicon_tags_1 },
    { 0x2efcdbd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51cdbd9, SUC_lexicon_tags_17 },
    { 0x313cdbd9, SUC_lexicon_tags_38 },
    { 0x4d9cdbda, SUC_lexicon_tags_17 },
    { 0xa4bcdbdc, SUC_lexicon_tags_31 },
    { 0xc53cdbdc, SUC_lexicon_tags_48 },
    { 0xe19cdbde, SUC_lexicon_tags_27 },
    { 0xbe1cdbde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12dcdbe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555cdbe6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1e9cdbe8, SUC_lexicon_tags_14 },
    { 0x2dcdbe9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfcdbec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3cdbf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdcdbf4, SUC_lexicon_tags_9 },
    { 0x5e9cdbf5, SUC_lexicon_tags_14 },
    { 0xc15cdbf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05cdbfa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x589cdbfd, SUC_lexicon_tags_6 },
    { 0x587cdbfe, SUC_lexicon_tags_1 },
    { 0x511cdbff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebcdc04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x401cdc06, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x273cdc08, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dcdc0b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3cdc0f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x933cdc12, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9efcdc14, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2bbcdc16, SUC_lexicon_tags_33 },
    { 0x6fdcdc17, SUC_lexicon_tags_88 },
    { 0xc49cdc18, SUC_lexicon_tags_4 },
    { 0x7dcdc18, SUC_lexicon_tags_77 },
    { 0xc91cdc16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddcdc21, SUC_lexicon_tags_14 },
    { 0xe57cdc22, SUC_lexicon_tags_49 },
    { 0x56dcdc23, SUC_lexicon_tags_47 },
    { 0x137cdc24, SUC_lexicon_tags_17 },
    { 0xd21cdc25, SUC_lexicon_tags_42 },
    { 0xd49cdc22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319cdc2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03cdc2e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67cdc37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101cdc3f, SUC_lexicon_tags_27 },
    { 0xdbcdc40, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xf13cdc42, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8d3cdc44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5cdc47, SUC_lexicon_tags_17 },
    { 0xdedcdc47, SUC_lexicon_tags_1 },
    { 0x133cdc49, SUC_lexicon_tags_7 },
    { 0x4fcdc47, SUC_lexicon_tags_6 },
    { 0x8a9cdc4b, SUC_lexicon_tags_19 },
    { 0xe5cdc4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51cdc56, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9a1cdc58, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dcdc5c, SUC_lexicon_tags_7 },
    { 0xc37cdc5d, SUC_lexicon_tags_15 },
    { 0x2a7cdc5e, SUC_lexicon_tags_37 },
    { 0x179cdc5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x15dcdc62, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1cdc66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcdc6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9cdc6f, SUC_lexicon_tags_7 },
    { 0xf4fcdc70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941cdc75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbcdc7a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3bdcdc7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61dcdc7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fcdc86, SUC_lexicon_tags_17 },
    { 0x6afcdc86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x577cdc8a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727cdc8e, SUC_lexicon_tags_1 },
    { 0xa83cdc8e, SUC_lexicon_tags_66 },
    { 0x1bcdc90, SUC_lexicon_tags_55 },
    { 0xb2fcdc91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fcdc95, SUC_lexicon_tags_33 },
    { 0x39cdc96, SUC_lexicon_tags_26 },
    { 0xa13cdc95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267cdc9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dcdc9f, SUC_lexicon_tags_0 },
    { 0xc7fcdca0, SUC_lexicon_tags_11 },
    { 0xdd7cdc9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9cdca4, SUC_lexicon_tags_11 },
    { 0x441cdca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3cdcaa, SUC_lexicon_tags_60 },
    { 0x89fcdcab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbcdcb5, SUC_lexicon_tags_2 },
    { 0x499cdcb6, SUC_lexicon_tags_29 },
    { 0x51fcdcb7, SUC_lexicon_tags_17 },
    { 0xd2bcdcb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5cdcbb, SUC_lexicon_tags_11 },
    { 0xa05cdcbc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x923cdcbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x909cdcc0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bcdcc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3cdcca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cdccd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1cdcd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bcdcd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377cdcd8, SUC_lexicon_tags_25 },
    { 0x80dcdcd9, SUC_lexicon_tags_38 },
    { 0x55dcdcda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f9cdcdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d1cdcde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bcdce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa07cdce9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88bcdced, SUC_lexicon_tags_68 },
    { 0xa7cdcee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x82bcdcf1, SUC_lexicon_tags_2 },
    { 0x969cdcf1, SUC_lexicon_tags_60 },
    { 0x6f7cdcf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07cdcf8, SUC_lexicon_tags_27 },
    { 0x555cdcf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3cdcfe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fcdd02, SUC_lexicon_tags_206 },
    { 0x535cdd02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895cdd09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ebcdd0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5cdd0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x701cdd12, SUC_lexicon_tags_84 },
    { 0x6b7cdd13, SUC_lexicon_tags_0 },
    { 0xc43cdd14, SUC_lexicon_tags_17 },
    { 0xebfcdd15, SUC_lexicon_tags_11 },
    { 0x9c1cdd16, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531cdd1e, SUC_lexicon_tags_15 },
    { 0x47bcdd1f, SUC_lexicon_tags_37 },
    { 0xd1bcdd20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367cdd26, SUC_lexicon_tags_60 },
    { 0x71cdd27, SUC_lexicon_tags_7 },
    { 0x74dcdd26, SUC_lexicon_tags_6 },
    { 0x763cdd29, SUC_lexicon_tags_60 },
    { 0x9d5cdd2a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bcdd2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd3cdd31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9cdd33, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dcdd36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfcdd39, SUC_lexicon_tags_12 },
    { 0x29cdd3a, SUC_lexicon_tags_6 },
    { 0x1c3cdd3b, SUC_lexicon_tags_257 },
    { 0x92fcdd3c, SUC_lexicon_tags_12 },
    { 0x157cdd39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edcdd40, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fcdd43, SUC_lexicon_tags_27 },
    { 0xd1bcdd44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dcdd49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fcdd4e, SUC_lexicon_tags_7 },
    { 0xe4bcdd4f, SUC_lexicon_tags_60 },
    { 0xc59cdd4e, SUC_lexicon_tags_17 },
    { 0x841cdd51, SUC_lexicon_tags_12 },
    { 0x52bcdd52, SUC_lexicon_tags_43 },
    { 0xd19cdd4e, SUC_lexicon_tags_23 },
    { 0xbcdd54, SUC_lexicon_tags_11 },
    { 0xb25cdd54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbcdd59, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x277cdd5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91cdd5e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6fbcdd60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643cdd64, SUC_lexicon_tags_9 },
    { 0xe3cdd65, SUC_lexicon_tags_12 },
    { 0x3cfcdd66, SUC_lexicon_tags_11 },
    { 0x297cdd66, SUC_lexicon_tags_7 },
    { 0x42dcdd64, SUC_lexicon_tags_11 },
    { 0xaa5cdd68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fcdd6f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf4bcdd71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x487cdd74, SUC_lexicon_tags_37 },
    { 0xa5dcdd75, SUC_lexicon_tags_11 },
    { 0xca9cdd74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadcdd7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55fcdd7c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcdcdd7e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efcdd82, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13cdd88, SUC_lexicon_tags_37 },
    { 0x14dcdd89, SUC_lexicon_tags_12 },
    { 0xc6fcdd89, SUC_lexicon_tags_21 },
    { 0x7e9cdd88, SUC_lexicon_tags_11 },
    { 0x20bcdd8c, SUC_lexicon_tags_11 },
    { 0x9b3cdd8d, SUC_lexicon_tags_37 },
    { 0x4b1cdd8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdcbcdd90, SUC_lexicon_tags_19 },
    { 0xbbbcdd91, SUC_lexicon_tags_27 },
    { 0xa3dcdd91, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa53cdd94, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3cdd97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5cdd9d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bcdda0, SUC_lexicon_tags_4 },
    { 0xe11cdda1, SUC_lexicon_tags_12 },
    { 0xbb5cdda1, SUC_lexicon_tags_4 },
    { 0x49dcdda1, SUC_lexicon_tags_19 },
    { 0xb6dcdda3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe29cdda6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7cddad, SUC_lexicon_tags_15 },
    { 0xee9cddae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x517cddb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bcddbf, SUC_lexicon_tags_25 },
    { 0xec5cddc0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x62fcddc2, SUC_lexicon_tags_19 },
    { 0x887cddc2, SUC_lexicon_tags_44 },
    { 0x8d1cddc4, SUC_lexicon_tags_29 },
    { 0x8c7cddc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603cddcc, SUC_lexicon_tags_11 },
    { 0xe5cddcd, SUC_lexicon_tags_8 },
    { 0xa35cddce, SUC_lexicon_tags_25 },
    { 0x675cddce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbcddd6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849cdddf, SUC_lexicon_tags_0 },
    { 0xf33cdde0, SUC_lexicon_tags_117 },
    { 0x51bcdde1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffdcdde3, SUC_lexicon_tags_1 },
    { 0xf3fcdde3, SUC_lexicon_tags_33 },
    { 0xe3cdde4, SUC_lexicon_tags_7 },
    { 0x50bcdde6, SUC_lexicon_tags_17 },
    { 0x515cdde6, SUC_lexicon_tags_37 },
    { 0xa01cdde8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb11cddeb, SUC_lexicon_tags_2 },
    { 0x567cddec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bcddef, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bcddf3, SUC_lexicon_tags_0 },
    { 0xf47cddf4, SUC_lexicon_tags_15 },
    { 0xb4dcddf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76dcddf7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x745cddf9, SUC_lexicon_tags_23 },
    { 0x56bcddfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb71cddfd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9bcddff, SUC_lexicon_tags_38 },
    { 0x779cde00, SUC_lexicon_tags_25 },
    { 0x3fcde00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92fcde03, SUC_lexicon_tags_31 },
    { 0x71cde04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff1cde06, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09cde0b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dcde0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x783cde11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511cde15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3cde17, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x3edcde1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607cde22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x335cde25, SUC_lexicon_tags_2 },
    { 0xbfbcde26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c3cde28, SUC_lexicon_tags_30 },
    { 0x7afcde28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5cde2c, SUC_lexicon_tags_44 },
    { 0x969cde2d, SUC_lexicon_tags_65 },
    { 0x899cde2e, SUC_lexicon_tags_0 },
    { 0x86dcde2c, SUC_lexicon_tags_25 },
    { 0x11bcde30, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3cde34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fcde3a, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05cde3e, SUC_lexicon_tags_49 },
    { 0xed7cde3f, SUC_lexicon_tags_45 },
    { 0xbd3cde3f, SUC_lexicon_tags_9 },
    { 0x28fcde3e, SUC_lexicon_tags_68 },
    { 0x7abcde3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7cde4a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fcde4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdb1cde4f, SUC_lexicon_tags_15 },
    { 0x217cde50, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x29cde52, SUC_lexicon_tags_9 },
    { 0x4b1cde52, SUC_lexicon_tags_7 },
    { 0x6bdcde52, SUC_lexicon_tags_27 },
    { 0xad7cde55, SUC_lexicon_tags_15 },
    { 0xd3cde56, SUC_lexicon_tags_27 },
    { 0x617cde55, SUC_lexicon_tags_25 },
    { 0x743cde52, SUC_lexicon_tags_4 },
    { 0x967cde59, SUC_lexicon_tags_1 },
    { 0x757cde54, SUC_lexicon_tags_1 },
    { 0x929cde53, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03cde5e, SUC_lexicon_tags_17 },
    { 0xdd7cde5e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x673cde62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7d3cde64, SUC_lexicon_tags_781 },
    { 0, NULL },
    { 0xe8dcde66, SUC_lexicon_tags_17 },
    { 0xcd3cde67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bcde6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x625cde70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabcde75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557cde7a, SUC_lexicon_tags_154 },
    { 0x697cde7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfcde7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc61cde80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359cde84, SUC_lexicon_tags_27 },
    { 0x587cde85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49cde88, SUC_lexicon_tags_17 },
    { 0x51cde88, SUC_lexicon_tags_11 },
    { 0x64dcde89, SUC_lexicon_tags_12 },
    { 0x107cde8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaebcde8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871cde92, SUC_lexicon_tags_15 },
    { 0x301cde93, SUC_lexicon_tags_240 },
    { 0x63fcde92, SUC_lexicon_tags_25 },
    { 0xc79cde92, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dcde99, SUC_lexicon_tags_28 },
    { 0xef7cde9a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe4dcde9c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9cde9f, SUC_lexicon_tags_17 },
    { 0xdedcdea0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5cdea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67cdeab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1cdeb1, SUC_lexicon_tags_47 },
    { 0x431cdeb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9cdeb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bcdebb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137cdebf, SUC_lexicon_tags_82 },
    { 0x35cdebf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fcdec4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91cdec7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3cdeca, SUC_lexicon_tags_45 },
    { 0xa15cdecb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3cdecf, SUC_lexicon_tags_1 },
    { 0x37cded0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf57cded2, SUC_lexicon_tags_7 },
    { 0x635cded3, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211cded7, SUC_lexicon_tags_2 },
    { 0xd73cded7, SUC_lexicon_tags_57 },
    { 0x39dcded7, SUC_lexicon_tags_27 },
    { 0xf7fcdeda, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3cdedd, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xde5cdedf, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbcdee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9cdee6, SUC_lexicon_tags_68 },
    { 0x95cdee6, SUC_lexicon_tags_25 },
    { 0x47cdee8, SUC_lexicon_tags_0 },
    { 0x88fcdee6, SUC_lexicon_tags_25 },
    { 0x7d5cdee7, SUC_lexicon_tags_254 },
    { 0xc7fcdeeb, SUC_lexicon_tags_37 },
    { 0xbcdeec, SUC_lexicon_tags_17 },
    { 0xf93cdee6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fcdef1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8bcdef3, SUC_lexicon_tags_29 },
    { 0x479cdef4, SUC_lexicon_tags_22 },
    { 0xd6bcdef3, SUC_lexicon_tags_19 },
    { 0xe57cdef6, SUC_lexicon_tags_23 },
    { 0x753cdef7, SUC_lexicon_tags_1 },
    { 0xd97cdef7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe89cdefa, SUC_lexicon_tags_29 },
    { 0xbf5cdefb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03cdf00, SUC_lexicon_tags_55 },
    { 0xc5fcdf00, SUC_lexicon_tags_123 },
    { 0x441cdf00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bcdf05, SUC_lexicon_tags_17 },
    { 0x4c1cdf05, SUC_lexicon_tags_47 },
    { 0xbbbcdf07, SUC_lexicon_tags_3 },
    { 0xddcdf08, SUC_lexicon_tags_39 },
    { 0xb5bcdf06, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29cdf0e, SUC_lexicon_tags_28 },
    { 0x129cdf0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e1cdf11, SUC_lexicon_tags_12 },
    { 0x5b1cdf12, SUC_lexicon_tags_12 },
    { 0xe27cdf13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dcdf18, SUC_lexicon_tags_1 },
    { 0x60bcdf19, SUC_lexicon_tags_12 },
    { 0xe63cdf1a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dcdf20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x983cdf23, SUC_lexicon_tags_15 },
    { 0x2a7cdf23, SUC_lexicon_tags_60 },
    { 0xde7cdf24, SUC_lexicon_tags_4 },
    { 0x9f1cdf26, SUC_lexicon_tags_11 },
    { 0xffcdf26, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfcdf2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7cdf2f, SUC_lexicon_tags_37 },
    { 0x8b7cdf2f, SUC_lexicon_tags_47 },
    { 0x5d9cdf31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56fcdf33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3cdf3d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbcdf42, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9f3cdf44, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0x281cdf46, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9cdf4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fcdf53, SUC_lexicon_tags_31 },
    { 0x4edcdf54, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539cdf58, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1ebcdf5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d7cdf5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcdf60, SUC_lexicon_tags_35 },
    { 0x375cdf61, SUC_lexicon_tags_31 },
    { 0x1cbcdf61, SUC_lexicon_tags_2 },
    { 0xeefcdf63, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe29cdf65, SUC_lexicon_tags_12 },
    { 0x347cdf66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7b9cdf68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71cdf6c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861cdf74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fcdf79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x845cdf7c, SUC_lexicon_tags_303 },
    { 0xedcdf7d, SUC_lexicon_tags_2 },
    { 0x14fcdf7e, SUC_lexicon_tags_64 },
    { 0xb25cdf7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01cdf82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x427cdf85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bcdf89, SUC_lexicon_tags_6 },
    { 0xfb3cdf8a, SUC_lexicon_tags_63 },
    { 0x939cdf89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf29cdf8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15cdf90, SUC_lexicon_tags_30 },
    { 0x463cdf90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397cdf9b, SUC_lexicon_tags_15 },
    { 0xdd7cdf9c, SUC_lexicon_tags_2 },
    { 0x9c9cdf9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409cdfa1, SUC_lexicon_tags_18 },
    { 0xfa1cdfa2, SUC_lexicon_tags_38 },
    { 0x4ffcdfa3, SUC_lexicon_tags_14 },
    { 0xbabcdfa4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483cdfaa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7cdfad, SUC_lexicon_tags_64 },
    { 0xf5cdfae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fcdfb8, SUC_lexicon_tags_37 },
    { 0x833cdfb9, SUC_lexicon_tags_28 },
    { 0x845cdfba, SUC_lexicon_tags_15 },
    { 0x703cdfbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5cdfbe, SUC_lexicon_tags_43 },
    { 0x98bcdfbe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483cdfc5, SUC_lexicon_tags_7 },
    { 0xe29cdfc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a3cdfc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5dcdfca, SUC_lexicon_tags_65 },
    { 0xda3cdfcb, SUC_lexicon_tags_15 },
    { 0xe01cdfcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1cdfcf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9cdfd8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x94fcdfda, SUC_lexicon_tags_28 },
    { 0x4f3cdfdb, SUC_lexicon_tags_23 },
    { 0x4f3cdfdb, SUC_lexicon_tags_60 },
    { 0x527cdfdb, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dcdfe3, SUC_lexicon_tags_30 },
    { 0x42fcdfe4, SUC_lexicon_tags_61 },
    { 0xe0bcdfe5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x559cdfe7, SUC_lexicon_tags_37 },
    { 0x1abcdfe8, SUC_lexicon_tags_21 },
    { 0xc57cdfe9, SUC_lexicon_tags_45 },
    { 0x63fcdfea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa89cdfec, SUC_lexicon_tags_14 },
    { 0xea1cdfed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x54bcdfef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289cdffb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x345cdffd, SUC_lexicon_tags_25 },
    { 0x6cfcdffe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3ce001, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabce007, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7ce00a, SUC_lexicon_tags_31 },
    { 0x7c5ce00b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x349ce00d, SUC_lexicon_tags_49 },
    { 0xddbce00d, SUC_lexicon_tags_11 },
    { 0x9f1ce00f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21fce011, SUC_lexicon_tags_17 },
    { 0xdb7ce012, SUC_lexicon_tags_1 },
    { 0x1a9ce011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadce019, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe19ce01b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc53ce01d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997ce022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ce029, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x61bce02b, SUC_lexicon_tags_26 },
    { 0x809ce02c, SUC_lexicon_tags_17 },
    { 0x271ce02b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x153ce02f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x459ce032, SUC_lexicon_tags_17 },
    { 0xe39ce033, SUC_lexicon_tags_63 },
    { 0x387ce034, SUC_lexicon_tags_38 },
    { 0xf3ce035, SUC_lexicon_tags_17 },
    { 0x4a1ce036, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x829ce038, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03ce03d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6fdce03f, SUC_lexicon_tags_761 },
    { 0xf89ce040, SUC_lexicon_tags_11 },
    { 0xfa3ce040, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd8fce043, SUC_lexicon_tags_48 },
    { 0x881ce043, SUC_lexicon_tags_63 },
    { 0x623ce044, SUC_lexicon_tags_49 },
    { 0xbefce044, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdce04a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7e3ce04c, SUC_lexicon_tags_381 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279ce050, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f5ce052, SUC_lexicon_tags_47 },
    { 0xf1ce053, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d7ce055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x563ce057, SUC_lexicon_tags_14 },
    { 0xcbfce058, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6dce05a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5ce05d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13ce062, SUC_lexicon_tags_66 },
    { 0xb79ce063, SUC_lexicon_tags_7 },
    { 0x7adce064, SUC_lexicon_tags_11 },
    { 0x189ce062, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7ce069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x597ce06c, SUC_lexicon_tags_15 },
    { 0x857ce06d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3ce072, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf73ce074, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3ce078, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x805ce07b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf63ce07d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501ce081, SUC_lexicon_tags_47 },
    { 0x3b1ce082, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99ce085, SUC_lexicon_tags_160 },
    { 0xcb9ce085, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb6dce088, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1ce08b, SUC_lexicon_tags_12 },
    { 0xd99ce08c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abce08f, SUC_lexicon_tags_23 },
    { 0x5adce090, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfce094, SUC_lexicon_tags_17 },
    { 0x425ce095, SUC_lexicon_tags_0 },
    { 0xc87ce096, SUC_lexicon_tags_8 },
    { 0x6ffce097, SUC_lexicon_tags_31 },
    { 0xa1fce096, SUC_lexicon_tags_2 },
    { 0x1a1ce099, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xef9ce09b, SUC_lexicon_tags_17 },
    { 0x905ce09c, SUC_lexicon_tags_25 },
    { 0xe33ce09d, SUC_lexicon_tags_14 },
    { 0x233ce09d, SUC_lexicon_tags_16 },
    { 0xb7bce09d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbce0a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ce0ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5ce0af, SUC_lexicon_tags_11 },
    { 0xf89ce0af, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fce0ba, SUC_lexicon_tags_32 },
    { 0x37bce0ba, SUC_lexicon_tags_46 },
    { 0xd79ce0bc, SUC_lexicon_tags_25 },
    { 0x643ce0bd, SUC_lexicon_tags_9 },
    { 0xe89ce0be, SUC_lexicon_tags_64 },
    { 0x7c1ce0bf, SUC_lexicon_tags_25 },
    { 0xdc5ce0c0, SUC_lexicon_tags_15 },
    { 0x603ce0bf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5ebce0c3, SUC_lexicon_tags_56 },
    { 0x5afce0c4, SUC_lexicon_tags_7 },
    { 0x6c5ce0c5, SUC_lexicon_tags_26 },
    { 0x359ce0c6, SUC_lexicon_tags_25 },
    { 0xcc5ce0c7, SUC_lexicon_tags_37 },
    { 0x49bce0c7, SUC_lexicon_tags_23 },
    { 0x637ce0c9, SUC_lexicon_tags_1 },
    { 0x513ce0c6, SUC_lexicon_tags_10 },
    { 0xe5bce0cb, SUC_lexicon_tags_29 },
    { 0x1e5ce0c5, SUC_lexicon_tags_14 },
    { 0x87fce0cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449ce0d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbce0d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9ce0e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3ce0e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dce0e7, SUC_lexicon_tags_11 },
    { 0x8f1ce0e8, SUC_lexicon_tags_38 },
    { 0xdd9ce0e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x295ce0eb, SUC_lexicon_tags_37 },
    { 0xf41ce0ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ffce0ee, SUC_lexicon_tags_17 },
    { 0xfc5ce0ee, SUC_lexicon_tags_27 },
    { 0x341ce0f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fce0f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd6dce0f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fce0f8, SUC_lexicon_tags_208 },
    { 0xc8dce0f9, SUC_lexicon_tags_42 },
    { 0x50fce0f8, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83ce0ff, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x593ce101, SUC_lexicon_tags_11 },
    { 0x959ce102, SUC_lexicon_tags_13 },
    { 0x57fce102, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x2e1ce105, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11ce108, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbce10f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bce115, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83ce118, SUC_lexicon_tags_63 },
    { 0xfcdce119, SUC_lexicon_tags_31 },
    { 0xbddce11a, SUC_lexicon_tags_39 },
    { 0x507ce11b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91ce11e, SUC_lexicon_tags_17 },
    { 0x395ce11f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a1ce121, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x539ce123, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x935ce125, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfce129, SUC_lexicon_tags_38 },
    { 0x2dfce12a, SUC_lexicon_tags_60 },
    { 0xb5fce12b, SUC_lexicon_tags_6 },
    { 0x79fce129, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf93ce12e, SUC_lexicon_tags_15 },
    { 0xdcfce12f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61ce132, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x43bce135, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x551ce137, SUC_lexicon_tags_25 },
    { 0x97ce138, SUC_lexicon_tags_7 },
    { 0x6ffce139, SUC_lexicon_tags_19 },
    { 0x203ce139, SUC_lexicon_tags_103 },
    { 0xb41ce13a, SUC_lexicon_tags_15 },
    { 0x4a7ce13c, SUC_lexicon_tags_1 },
    { 0x4efce13d, SUC_lexicon_tags_45 },
    { 0x7bbce13d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651ce142, SUC_lexicon_tags_63 },
    { 0x193ce143, SUC_lexicon_tags_10 },
    { 0x37ce143, SUC_lexicon_tags_14 },
    { 0x5a1ce145, SUC_lexicon_tags_11 },
    { 0x585ce145, SUC_lexicon_tags_11 },
    { 0x6e9ce147, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x809ce149, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe63ce14b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585ce14f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb3bce151, SUC_lexicon_tags_49 },
    { 0xbfbce151, SUC_lexicon_tags_25 },
    { 0xdbdce151, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803ce157, SUC_lexicon_tags_38 },
    { 0xe87ce157, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511ce15c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x185ce15f, SUC_lexicon_tags_29 },
    { 0x181ce160, SUC_lexicon_tags_25 },
    { 0xc3ce15f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dce165, SUC_lexicon_tags_37 },
    { 0x555ce166, SUC_lexicon_tags_112 },
    { 0x21fce166, SUC_lexicon_tags_20 },
    { 0x4f1ce168, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dce171, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fce17a, SUC_lexicon_tags_7 },
    { 0x109ce17b, SUC_lexicon_tags_7 },
    { 0x3c5ce17c, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x385ce17e, SUC_lexicon_tags_46 },
    { 0x755ce17f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x59ce181, SUC_lexicon_tags_69 },
    { 0x19bce181, SUC_lexicon_tags_3 },
    { 0xf69ce181, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9d9ce185, SUC_lexicon_tags_2 },
    { 0x7dce186, SUC_lexicon_tags_7 },
    { 0x4f3ce187, SUC_lexicon_tags_17 },
    { 0xe2fce188, SUC_lexicon_tags_44 },
    { 0xb15ce189, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xaa3ce18b, SUC_lexicon_tags_15 },
    { 0xc27ce18c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabce18f, SUC_lexicon_tags_35 },
    { 0x77fce190, SUC_lexicon_tags_37 },
    { 0x531ce18f, SUC_lexicon_tags_12 },
    { 0xe7dce191, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793ce1a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97ce1a9, SUC_lexicon_tags_23 },
    { 0xb19ce1aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0dce1ac, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x2c1ce1ae, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13ce1b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1ce1c3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2fce1c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e5ce1c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x913ce1ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5ce1ce, SUC_lexicon_tags_23 },
    { 0x2f9ce1ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7ce1d7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255ce1dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa73ce1de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadce1e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeffce1e7, SUC_lexicon_tags_49 },
    { 0x7cbce1e8, SUC_lexicon_tags_77 },
    { 0x92dce1e9, SUC_lexicon_tags_11 },
    { 0x901ce1e9, SUC_lexicon_tags_23 },
    { 0xc89ce1ea, SUC_lexicon_tags_50 },
    { 0xb51ce1ec, SUC_lexicon_tags_28 },
    { 0x13fce1eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1ce1f0, SUC_lexicon_tags_7 },
    { 0x217ce1f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307ce1f6, SUC_lexicon_tags_26 },
    { 0x3fdce1f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdfce1f9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb81ce1fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dce1ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xce1ce201, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89dce204, SUC_lexicon_tags_10 },
    { 0xe77ce205, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaa9ce207, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ce20c, SUC_lexicon_tags_37 },
    { 0x68fce20c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x275ce20f, SUC_lexicon_tags_49 },
    { 0xf67ce210, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff5ce212, SUC_lexicon_tags_12 },
    { 0x455ce212, SUC_lexicon_tags_39 },
    { 0xbc1ce214, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99ce217, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5ce21b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x429ce21d, SUC_lexicon_tags_11 },
    { 0xe5bce21d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97ce222, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd77ce224, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453ce229, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7edce22b, SUC_lexicon_tags_0 },
    { 0x561ce22b, SUC_lexicon_tags_7 },
    { 0x889ce22c, SUC_lexicon_tags_17 },
    { 0xb21ce22e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7ce237, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869ce23f, SUC_lexicon_tags_27 },
    { 0x189ce240, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x959ce242, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x56fce244, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13ce246, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x449ce248, SUC_lexicon_tags_14 },
    { 0x14fce249, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x817ce24b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9ce24e, SUC_lexicon_tags_49 },
    { 0xc6bce24f, SUC_lexicon_tags_12 },
    { 0x763ce24f, SUC_lexicon_tags_21 },
    { 0xb9ce251, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x87bce254, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fce258, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efce264, SUC_lexicon_tags_14 },
    { 0xef5ce265, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd65ce267, SUC_lexicon_tags_74 },
    { 0x6ffce268, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67ce26c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbce274, SUC_lexicon_tags_47 },
    { 0x8f5ce274, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbce278, SUC_lexicon_tags_25 },
    { 0xc1ce279, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ce280, SUC_lexicon_tags_8 },
    { 0x607ce280, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17ce284, SUC_lexicon_tags_14 },
    { 0x155ce284, SUC_lexicon_tags_87 },
    { 0x18bce284, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491ce28c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaafce28e, SUC_lexicon_tags_19 },
    { 0xc19ce28f, SUC_lexicon_tags_19 },
    { 0x895ce290, SUC_lexicon_tags_37 },
    { 0xe4dce291, SUC_lexicon_tags_49 },
    { 0x4cfce292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb61ce294, SUC_lexicon_tags_25 },
    { 0x1bce295, SUC_lexicon_tags_29 },
    { 0x3f1ce296, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc59ce298, SUC_lexicon_tags_45 },
    { 0xf79ce298, SUC_lexicon_tags_27 },
    { 0xe01ce29a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dce29d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7ebce29f, SUC_lexicon_tags_57 },
    { 0xccfce2a0, SUC_lexicon_tags_55 },
    { 0xd91ce2a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd69ce2a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39ce2a7, SUC_lexicon_tags_38 },
    { 0xb3ce2a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52fce2aa, SUC_lexicon_tags_44 },
    { 0x749ce2ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3ce2af, SUC_lexicon_tags_8 },
    { 0x6cbce2b0, SUC_lexicon_tags_60 },
    { 0x345ce2b1, SUC_lexicon_tags_33 },
    { 0x9c1ce2b2, SUC_lexicon_tags_2 },
    { 0xd47ce2b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x74bce2b5, SUC_lexicon_tags_34 },
    { 0x2f3ce2b6, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3ce2b9, SUC_lexicon_tags_12 },
    { 0x74dce2b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x157ce2bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3edce2be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555ce2c2, SUC_lexicon_tags_33 },
    { 0x7c1ce2c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dce2c8, SUC_lexicon_tags_8 },
    { 0x3adce2c8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadce2cf, SUC_lexicon_tags_27 },
    { 0x47fce2cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc73ce2d2, SUC_lexicon_tags_65 },
    { 0xcf5ce2d3, SUC_lexicon_tags_116 },
    { 0xf0dce2d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90fce2d6, SUC_lexicon_tags_37 },
    { 0xb23ce2d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f3ce2d9, SUC_lexicon_tags_47 },
    { 0x5a3ce2d9, SUC_lexicon_tags_47 },
    { 0x709ce2db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efce2df, SUC_lexicon_tags_15 },
    { 0x483ce2e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39ce2e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6fbce2e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb79ce2e7, SUC_lexicon_tags_17 },
    { 0x3ddce2e7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xdf5ce2ea, SUC_lexicon_tags_2 },
    { 0xab1ce2eb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dce2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7adce2f0, SUC_lexicon_tags_31 },
    { 0x4f7ce2f1, SUC_lexicon_tags_66 },
    { 0xafdce2f2, SUC_lexicon_tags_38 },
    { 0x6ebce2f3, SUC_lexicon_tags_16 },
    { 0x6d5ce2f4, SUC_lexicon_tags_37 },
    { 0x8d5ce2f5, SUC_lexicon_tags_47 },
    { 0x63dce2f6, SUC_lexicon_tags_6 },
    { 0xca5ce2f6, SUC_lexicon_tags_25 },
    { 0xc43ce2f3, SUC_lexicon_tags_40 },
    { 0x97ce2f9, SUC_lexicon_tags_21 },
    { 0x7c3ce2fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63ce2fd, SUC_lexicon_tags_49 },
    { 0x3bce2fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921ce302, SUC_lexicon_tags_37 },
    { 0x2e3ce303, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x307ce306, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1ce30a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efce313, SUC_lexicon_tags_160 },
    { 0xd19ce314, SUC_lexicon_tags_17 },
    { 0xd4dce315, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bce317, SUC_lexicon_tags_45 },
    { 0xd79ce317, SUC_lexicon_tags_29 },
    { 0x771ce319, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65ce324, SUC_lexicon_tags_37 },
    { 0xfc9ce325, SUC_lexicon_tags_25 },
    { 0xbefce324, SUC_lexicon_tags_43 },
    { 0x82fce325, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259ce32b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6edce32e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe39ce330, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bce338, SUC_lexicon_tags_19 },
    { 0x90dce339, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b3ce33b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5b3ce33d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffce340, SUC_lexicon_tags_9 },
    { 0xb73ce341, SUC_lexicon_tags_6 },
    { 0xef3ce340, SUC_lexicon_tags_1 },
    { 0x24fce343, SUC_lexicon_tags_81 },
    { 0xd27ce343, SUC_lexicon_tags_25 },
    { 0xd5fce345, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7ce34b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dce34e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bce351, SUC_lexicon_tags_0 },
    { 0xdbbce352, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bce355, SUC_lexicon_tags_17 },
    { 0x35fce356, SUC_lexicon_tags_60 },
    { 0x27dce357, SUC_lexicon_tags_60 },
    { 0x5bbce358, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x301ce35a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xcdfce35c, SUC_lexicon_tags_2 },
    { 0xa05ce35d, SUC_lexicon_tags_38 },
    { 0xa39ce35e, SUC_lexicon_tags_1 },
    { 0x68dce35d, SUC_lexicon_tags_11 },
    { 0x829ce35c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fce365, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afce369, SUC_lexicon_tags_19 },
    { 0x529ce369, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7ce36d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ce370, SUC_lexicon_tags_37 },
    { 0xe61ce371, SUC_lexicon_tags_37 },
    { 0xe61ce372, SUC_lexicon_tags_31 },
    { 0x89dce372, SUC_lexicon_tags_204 },
    { 0x80dce371, SUC_lexicon_tags_12 },
    { 0xb45ce375, SUC_lexicon_tags_14 },
    { 0xbf9ce371, SUC_lexicon_tags_11 },
    { 0x383ce377, SUC_lexicon_tags_1 },
    { 0xe01ce378, SUC_lexicon_tags_8 },
    { 0x3e9ce378, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x10fce37b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1ce37e, SUC_lexicon_tags_47 },
    { 0x93fce37e, SUC_lexicon_tags_12 },
    { 0x8f3ce37f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac9ce382, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957ce389, SUC_lexicon_tags_11 },
    { 0x6cfce38a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddce38d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbce390, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x841ce393, SUC_lexicon_tags_0 },
    { 0x605ce394, SUC_lexicon_tags_25 },
    { 0x663ce395, SUC_lexicon_tags_62 },
    { 0xec1ce396, SUC_lexicon_tags_79 },
    { 0x457ce397, SUC_lexicon_tags_1 },
    { 0x5f9ce397, SUC_lexicon_tags_2 },
    { 0xe41ce397, SUC_lexicon_tags_2 },
    { 0x885ce39a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9ce39f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66dce3a2, SUC_lexicon_tags_30 },
    { 0x1abce3a3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ce3a6, SUC_lexicon_tags_28 },
    { 0x515ce3a7, SUC_lexicon_tags_27 },
    { 0x5cdce3a8, SUC_lexicon_tags_15 },
    { 0x251ce3a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfce3ac, SUC_lexicon_tags_12 },
    { 0x91fce3ad, SUC_lexicon_tags_11 },
    { 0x6a5ce3ae, SUC_lexicon_tags_9 },
    { 0xb79ce3ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x197ce3b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x421ce3b3, SUC_lexicon_tags_37 },
    { 0xf3fce3b4, SUC_lexicon_tags_37 },
    { 0x905ce3b4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33ce3b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x485ce3bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ce3bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb7ce3c1, SUC_lexicon_tags_60 },
    { 0x37dce3c2, SUC_lexicon_tags_4 },
    { 0x43ce3c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1ce3c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac3ce3c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9ce3cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315ce3d4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219ce3d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2cfce3db, SUC_lexicon_tags_257 },
    { 0x3d9ce3db, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bce3df, SUC_lexicon_tags_7 },
    { 0x291ce3e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf53ce3e2, SUC_lexicon_tags_31 },
    { 0xaedce3e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185ce3e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x897ce3ea, SUC_lexicon_tags_25 },
    { 0x75fce3eb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5ce3ee, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x96dce3f0, SUC_lexicon_tags_0 },
    { 0xc6bce3f1, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fce3f7, SUC_lexicon_tags_27 },
    { 0xdd1ce3f8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ce3fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9ce400, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5ce403, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1ce409, SUC_lexicon_tags_12 },
    { 0x30fce409, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbce411, SUC_lexicon_tags_61 },
    { 0x123ce411, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1ce415, SUC_lexicon_tags_17 },
    { 0x6bbce416, SUC_lexicon_tags_15 },
    { 0x67ce417, SUC_lexicon_tags_60 },
    { 0x811ce417, SUC_lexicon_tags_11 },
    { 0xc7fce419, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x61bce41b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15ce41e, SUC_lexicon_tags_29 },
    { 0x7a5ce41e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x761ce421, SUC_lexicon_tags_9 },
    { 0x859ce422, SUC_lexicon_tags_27 },
    { 0xf4bce422, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4dbce425, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74bce427, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7ce433, SUC_lexicon_tags_21 },
    { 0x8b3ce433, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9ce439, SUC_lexicon_tags_17 },
    { 0x529ce43a, SUC_lexicon_tags_29 },
    { 0xef7ce43b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ce43e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dce442, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5ce445, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xba1ce447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5ce44b, SUC_lexicon_tags_129 },
    { 0xbb7ce44b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9ce455, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ce45b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ce45e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x14fce460, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa91ce462, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe99ce464, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedce467, SUC_lexicon_tags_43 },
    { 0xea1ce468, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xabfce46b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdce46f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddfce471, SUC_lexicon_tags_15 },
    { 0xd33ce471, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x67bce474, SUC_lexicon_tags_87 },
    { 0x271ce475, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadce478, SUC_lexicon_tags_1 },
    { 0xe27ce478, SUC_lexicon_tags_17 },
    { 0xc7bce47a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x281ce47d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9ce481, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xb41ce483, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x94dce485, SUC_lexicon_tags_43 },
    { 0x54dce486, SUC_lexicon_tags_75 },
    { 0x351ce487, SUC_lexicon_tags_25 },
    { 0xf27ce487, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adce48b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ce48f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7adce491, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xabfce493, SUC_lexicon_tags_10 },
    { 0x3c1ce494, SUC_lexicon_tags_13 },
    { 0xaafce494, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21ce498, SUC_lexicon_tags_604 },
    { 0xb49ce499, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bce49c, SUC_lexicon_tags_10 },
    { 0x359ce49d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddce4a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xffce4a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99ce4a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357ce4aa, SUC_lexicon_tags_423 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477ce4ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5ce4b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939ce4b6, SUC_lexicon_tags_27 },
    { 0x579ce4b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3ce4bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73ce4c0, SUC_lexicon_tags_32 },
    { 0x227ce4c1, SUC_lexicon_tags_11 },
    { 0x32bce4c2, SUC_lexicon_tags_204 },
    { 0x3d5ce4c3, SUC_lexicon_tags_47 },
    { 0x3e5ce4c0, SUC_lexicon_tags_25 },
    { 0xcd5ce4c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x275ce4c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ffce4ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bce4d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x459ce4d2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x193ce4d5, SUC_lexicon_tags_11 },
    { 0xa85ce4d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bce4db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbce4df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfd7ce4e1, SUC_lexicon_tags_23 },
    { 0x389ce4e2, SUC_lexicon_tags_7 },
    { 0xfddce4e2, SUC_lexicon_tags_47 },
    { 0x289ce4e2, SUC_lexicon_tags_2 },
    { 0xff1ce4e3, SUC_lexicon_tags_47 },
    { 0x7ce4e6, SUC_lexicon_tags_43 },
    { 0x3d7ce4e5, SUC_lexicon_tags_84 },
    { 0xb11ce4e8, SUC_lexicon_tags_17 },
    { 0x14fce4e8, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x785ce4eb, SUC_lexicon_tags_116 },
    { 0xe7ce4eb, SUC_lexicon_tags_7 },
    { 0xf85ce4eb, SUC_lexicon_tags_33 },
    { 0x87ce4ee, SUC_lexicon_tags_81 },
    { 0x57fce4ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ce4f3, SUC_lexicon_tags_25 },
    { 0x71dce4f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf5ce4f6, SUC_lexicon_tags_37 },
    { 0xdbbce4f7, SUC_lexicon_tags_1 },
    { 0xaa5ce4f8, SUC_lexicon_tags_39 },
    { 0x7cbce4f6, SUC_lexicon_tags_7 },
    { 0x5c5ce4fa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5ce500, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x471ce502, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46bce504, SUC_lexicon_tags_39 },
    { 0x86bce505, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617ce50d, SUC_lexicon_tags_37 },
    { 0x8c9ce50e, SUC_lexicon_tags_95 },
    { 0xe91ce50e, SUC_lexicon_tags_63 },
    { 0x97fce510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfce513, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bce51c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xed7ce51e, SUC_lexicon_tags_38 },
    { 0x65bce51f, SUC_lexicon_tags_336 },
    { 0x60fce51f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc2fce522, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x979ce525, SUC_lexicon_tags_0 },
    { 0xb15ce526, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xed1ce528, SUC_lexicon_tags_13 },
    { 0xdce528, SUC_lexicon_tags_25 },
    { 0xc2dce528, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe93ce52c, SUC_lexicon_tags_87 },
    { 0x6dce52d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68fce52f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473ce539, SUC_lexicon_tags_31 },
    { 0xfc1ce53a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191ce53e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x883ce540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x809ce543, SUC_lexicon_tags_19 },
    { 0x4c7ce544, SUC_lexicon_tags_55 },
    { 0x4e7ce544, SUC_lexicon_tags_17 },
    { 0x7e7ce546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39ce54d, SUC_lexicon_tags_37 },
    { 0xf85ce54e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba7ce551, SUC_lexicon_tags_92 },
    { 0x3a1ce551, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x1a1ce554, SUC_lexicon_tags_34 },
    { 0x1dfce555, SUC_lexicon_tags_25 },
    { 0x9f1ce555, SUC_lexicon_tags_30 },
    { 0x365ce557, SUC_lexicon_tags_17 },
    { 0x185ce558, SUC_lexicon_tags_25 },
    { 0x6edce559, SUC_lexicon_tags_25 },
    { 0x2adce55a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeffce55c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc1ce55e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fdce560, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3ce563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9ce56e, SUC_lexicon_tags_19 },
    { 0xa3fce56e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdce575, SUC_lexicon_tags_15 },
    { 0x867ce576, SUC_lexicon_tags_2 },
    { 0x3bce576, SUC_lexicon_tags_53 },
    { 0x41dce577, SUC_lexicon_tags_25 },
    { 0xbebce576, SUC_lexicon_tags_19 },
    { 0xd19ce576, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58dce57d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb37ce57f, SUC_lexicon_tags_25 },
    { 0x355ce580, SUC_lexicon_tags_25 },
    { 0x919ce581, SUC_lexicon_tags_23 },
    { 0x843ce581, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x659ce584, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbce588, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323ce58d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x69dce58f, SUC_lexicon_tags_17 },
    { 0x3b7ce590, SUC_lexicon_tags_19 },
    { 0xddce591, SUC_lexicon_tags_38 },
    { 0x295ce592, SUC_lexicon_tags_11 },
    { 0xfedce58f, SUC_lexicon_tags_12 },
    { 0xfebce58f, SUC_lexicon_tags_2 },
    { 0xf01ce595, SUC_lexicon_tags_27 },
    { 0x511ce595, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9ce599, SUC_lexicon_tags_63 },
    { 0x6afce599, SUC_lexicon_tags_17 },
    { 0x619ce59a, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dce59f, SUC_lexicon_tags_65 },
    { 0x493ce5a0, SUC_lexicon_tags_15 },
    { 0xafdce5a1, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7ce5ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a3ce5ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xee1ce5af, SUC_lexicon_tags_12 },
    { 0x58fce5b0, SUC_lexicon_tags_11 },
    { 0xfd9ce5b1, SUC_lexicon_tags_0 },
    { 0xc21ce5b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fce5b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ce5bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7efce5be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dce5c3, SUC_lexicon_tags_37 },
    { 0x6a3ce5c4, SUC_lexicon_tags_62 },
    { 0x72fce5c4, SUC_lexicon_tags_37 },
    { 0x70fce5c6, SUC_lexicon_tags_17 },
    { 0x96fce5c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfce5cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fce5cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x88dce5d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5ce5d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675ce5d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfc9ce5da, SUC_lexicon_tags_1 },
    { 0x71dce5db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc81ce5dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x597ce5df, SUC_lexicon_tags_65 },
    { 0x6cbce5e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5dfce5e2, SUC_lexicon_tags_33 },
    { 0x1c3ce5e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bce5e9, SUC_lexicon_tags_0 },
    { 0xc19ce5e9, SUC_lexicon_tags_37 },
    { 0x665ce5ea, SUC_lexicon_tags_16 },
    { 0xf7dce5ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x315ce5ef, SUC_lexicon_tags_11 },
    { 0x6a7ce5f0, SUC_lexicon_tags_2 },
    { 0x5a1ce5f0, SUC_lexicon_tags_33 },
    { 0xbcdce5ef, SUC_lexicon_tags_28 },
    { 0x355ce5f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf7bce5f5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55ce5f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5ce5fb, SUC_lexicon_tags_30 },
    { 0xca5ce5fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9ce602, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bce605, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x67fce607, SUC_lexicon_tags_16 },
    { 0xd79ce607, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bce60e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669ce612, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x40fce614, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda7ce616, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1ce619, SUC_lexicon_tags_19 },
    { 0xd37ce61a, SUC_lexicon_tags_30 },
    { 0x117ce61a, SUC_lexicon_tags_43 },
    { 0x36fce619, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93ce620, SUC_lexicon_tags_11 },
    { 0xdf5ce621, SUC_lexicon_tags_15 },
    { 0x773ce620, SUC_lexicon_tags_15 },
    { 0xca1ce623, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ce62d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x42fce62f, SUC_lexicon_tags_8 },
    { 0xf53ce630, SUC_lexicon_tags_26 },
    { 0x3bbce631, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fce634, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bce638, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x27dce63b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9ce63e, SUC_lexicon_tags_43 },
    { 0x6adce63f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa13ce641, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6edce643, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3ce649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5ce64d, SUC_lexicon_tags_6 },
    { 0x533ce64e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5ce652, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeebce654, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571ce658, SUC_lexicon_tags_91 },
    { 0xda9ce659, SUC_lexicon_tags_15 },
    { 0xdffce65a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ce65d, SUC_lexicon_tags_68 },
    { 0x84bce65d, SUC_lexicon_tags_84 },
    { 0xae3ce65f, SUC_lexicon_tags_11 },
    { 0xa2dce65e, SUC_lexicon_tags_15 },
    { 0x6cfce660, SUC_lexicon_tags_17 },
    { 0xa39ce65e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e1ce664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bce668, SUC_lexicon_tags_37 },
    { 0xf63ce669, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dce66c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbce670, SUC_lexicon_tags_45 },
    { 0xb77ce670, SUC_lexicon_tags_37 },
    { 0x57dce672, SUC_lexicon_tags_39 },
    { 0x989ce673, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fce676, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x149ce678, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6b5ce67a, SUC_lexicon_tags_15 },
    { 0x87bce67b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bce681, SUC_lexicon_tags_25 },
    { 0xf71ce682, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803ce68c, SUC_lexicon_tags_17 },
    { 0x3cbce68d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4d7ce68f, SUC_lexicon_tags_17 },
    { 0xbb3ce68f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4e7ce692, SUC_lexicon_tags_11 },
    { 0x34dce692, SUC_lexicon_tags_144 },
    { 0x2d3ce694, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x743ce696, SUC_lexicon_tags_15 },
    { 0x6b1ce696, SUC_lexicon_tags_10 },
    { 0xa83ce697, SUC_lexicon_tags_14 },
    { 0x745ce699, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x71dce69b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61ce6a5, SUC_lexicon_tags_2 },
    { 0x3b9ce6a5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fce6aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cdce6ac, SUC_lexicon_tags_25 },
    { 0x5b1ce6ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71ce6b1, SUC_lexicon_tags_84 },
    { 0xef3ce6b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x785ce6b4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79ce6b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa67ce6b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ce6bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b9ce6c1, SUC_lexicon_tags_19 },
    { 0x997ce6c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3e9ce6c4, SUC_lexicon_tags_37 },
    { 0xdf1ce6c5, SUC_lexicon_tags_25 },
    { 0x441ce6c5, SUC_lexicon_tags_9 },
    { 0x379ce6c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313ce6d1, SUC_lexicon_tags_16 },
    { 0x785ce6d2, SUC_lexicon_tags_47 },
    { 0xf67ce6d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x25ce6d5, SUC_lexicon_tags_1 },
    { 0xdcfce6d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4f9ce6d8, SUC_lexicon_tags_37 },
    { 0x52fce6d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48fce6db, SUC_lexicon_tags_25 },
    { 0xffce6dc, SUC_lexicon_tags_30 },
    { 0xfe1ce6dd, SUC_lexicon_tags_7 },
    { 0x9afce6dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dce6e2, SUC_lexicon_tags_9 },
    { 0x467ce6e3, SUC_lexicon_tags_7 },
    { 0xe99ce6e4, SUC_lexicon_tags_28 },
    { 0xfefce6e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed3ce6e7, SUC_lexicon_tags_25 },
    { 0xa47ce6e7, SUC_lexicon_tags_33 },
    { 0xa91ce6e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25ce6ec, SUC_lexicon_tags_32 },
    { 0xa6fce6ec, SUC_lexicon_tags_8 },
    { 0xa67ce6ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x851ce6f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa81ce6f2, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17ce6f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955ce6fa, SUC_lexicon_tags_65 },
    { 0xe57ce6fb, SUC_lexicon_tags_7 },
    { 0x3c1ce6fc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dce704, SUC_lexicon_tags_2 },
    { 0xf6fce704, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x42dce708, SUC_lexicon_tags_181 },
    { 0x941ce709, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdce715, SUC_lexicon_tags_60 },
    { 0x67ce716, SUC_lexicon_tags_319 },
    { 0, NULL },
    { 0x25bce718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fce71f, SUC_lexicon_tags_37 },
    { 0xe3fce71f, SUC_lexicon_tags_46 },
    { 0xe3fce721, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605ce725, SUC_lexicon_tags_1 },
    { 0xb77ce726, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509ce72b, SUC_lexicon_tags_37 },
    { 0xbb5ce72c, SUC_lexicon_tags_38 },
    { 0x6a1ce72d, SUC_lexicon_tags_33 },
    { 0x3d7ce72d, SUC_lexicon_tags_28 },
    { 0x6a3ce72e, SUC_lexicon_tags_2 },
    { 0x117ce72c, SUC_lexicon_tags_12 },
    { 0x477ce731, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9e5ce733, SUC_lexicon_tags_26 },
    { 0x72fce734, SUC_lexicon_tags_77 },
    { 0xa9fce735, SUC_lexicon_tags_7 },
    { 0xff9ce736, SUC_lexicon_tags_11 },
    { 0x5fdce736, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebce73c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfce73f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bce742, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x91dce744, SUC_lexicon_tags_14 },
    { 0x3b5ce744, SUC_lexicon_tags_14 },
    { 0xdf7ce746, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95ce749, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fce74c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409ce750, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd1dce752, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bce757, SUC_lexicon_tags_62 },
    { 0xc65ce757, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85ce75c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe27ce75e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf39ce760, SUC_lexicon_tags_25 },
    { 0x80fce761, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ce76b, SUC_lexicon_tags_63 },
    { 0xc13ce76b, SUC_lexicon_tags_34 },
    { 0x7c9ce76b, SUC_lexicon_tags_23 },
    { 0x681ce76d, SUC_lexicon_tags_1 },
    { 0x1e3ce76f, SUC_lexicon_tags_84 },
    { 0x3cfce770, SUC_lexicon_tags_37 },
    { 0xb49ce76b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85ce777, SUC_lexicon_tags_34 },
    { 0x4c1ce778, SUC_lexicon_tags_37 },
    { 0xd49ce779, SUC_lexicon_tags_12 },
    { 0x95ce779, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc49ce77c, SUC_lexicon_tags_37 },
    { 0xcf5ce77c, SUC_lexicon_tags_25 },
    { 0x7ddce77e, SUC_lexicon_tags_21 },
    { 0x757ce77f, SUC_lexicon_tags_19 },
    { 0xed1ce780, SUC_lexicon_tags_1 },
    { 0x4ebce77f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535ce787, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5ce78b, SUC_lexicon_tags_30 },
    { 0x271ce78c, SUC_lexicon_tags_21 },
    { 0x941ce78d, SUC_lexicon_tags_54 },
    { 0x859ce78e, SUC_lexicon_tags_14 },
    { 0xfe7ce78c, SUC_lexicon_tags_25 },
    { 0xdb7ce790, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5ce793, SUC_lexicon_tags_23 },
    { 0xd15ce793, SUC_lexicon_tags_9 },
    { 0x91fce795, SUC_lexicon_tags_2 },
    { 0x845ce795, SUC_lexicon_tags_9 },
    { 0xc6bce797, SUC_lexicon_tags_7 },
    { 0xa8fce795, SUC_lexicon_tags_12 },
    { 0xb53ce797, SUC_lexicon_tags_199 },
    { 0xbefce79a, SUC_lexicon_tags_25 },
    { 0x243ce79b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdebce79d, SUC_lexicon_tags_87 },
    { 0x85ce79e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771ce7b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x89fce7b3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc69ce7b5, SUC_lexicon_tags_87 },
    { 0x819ce7b6, SUC_lexicon_tags_33 },
    { 0x2d3ce7b7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddce7bb, SUC_lexicon_tags_23 },
    { 0x103ce7bc, SUC_lexicon_tags_37 },
    { 0x47ce7bd, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1ce7c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1ce7c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11ce7c9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bce7cc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dce7d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69ce7d3, SUC_lexicon_tags_293 },
    { 0x5adce7d4, SUC_lexicon_tags_49 },
    { 0x347ce7d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8fce7d7, SUC_lexicon_tags_6 },
    { 0x169ce7d8, SUC_lexicon_tags_37 },
    { 0x52dce7d8, SUC_lexicon_tags_37 },
    { 0x8bce7da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855ce7e0, SUC_lexicon_tags_11 },
    { 0xe7dce7e1, SUC_lexicon_tags_87 },
    { 0xf27ce7e2, SUC_lexicon_tags_8 },
    { 0x5f1ce7e3, SUC_lexicon_tags_70 },
    { 0xb9bce7e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ce7ee, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afce7f4, SUC_lexicon_tags_19 },
    { 0xc43ce7f5, SUC_lexicon_tags_12 },
    { 0xd77ce7f6, SUC_lexicon_tags_7 },
    { 0xc6fce7f4, SUC_lexicon_tags_0 },
    { 0x7b7ce7f7, SUC_lexicon_tags_2 },
    { 0x5f7ce7f5, SUC_lexicon_tags_0 },
    { 0x6ffce7fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe8dce7fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ce801, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39ce807, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4bce809, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7ce80c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1ce80f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfce818, SUC_lexicon_tags_66 },
    { 0x367ce819, SUC_lexicon_tags_28 },
    { 0xa5dce81a, SUC_lexicon_tags_7 },
    { 0x27bce818, SUC_lexicon_tags_47 },
    { 0x13bce818, SUC_lexicon_tags_35 },
    { 0x5b3ce818, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ce821, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee5ce823, SUC_lexicon_tags_63 },
    { 0xf6bce823, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x31dce826, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bce82b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x72dce82e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9ce834, SUC_lexicon_tags_7 },
    { 0x80dce835, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ce83d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abce841, SUC_lexicon_tags_7 },
    { 0x70dce841, SUC_lexicon_tags_37 },
    { 0xe4bce843, SUC_lexicon_tags_15 },
    { 0x5b3ce841, SUC_lexicon_tags_47 },
    { 0xb8bce845, SUC_lexicon_tags_1 },
    { 0x6fce846, SUC_lexicon_tags_37 },
    { 0xba9ce846, SUC_lexicon_tags_49 },
    { 0x111ce848, SUC_lexicon_tags_27 },
    { 0x515ce848, SUC_lexicon_tags_29 },
    { 0xf65ce84a, SUC_lexicon_tags_25 },
    { 0x453ce84b, SUC_lexicon_tags_31 },
    { 0x413ce846, SUC_lexicon_tags_47 },
    { 0x2ffce84d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9ce850, SUC_lexicon_tags_2 },
    { 0x83bce850, SUC_lexicon_tags_7 },
    { 0x3dce852, SUC_lexicon_tags_0 },
    { 0xd0fce853, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9ce85b, SUC_lexicon_tags_25 },
    { 0xe3ce85c, SUC_lexicon_tags_478 },
    { 0x8bdce85c, SUC_lexicon_tags_63 },
    { 0x4afce85e, SUC_lexicon_tags_25 },
    { 0x6adce85f, SUC_lexicon_tags_1 },
    { 0x903ce860, SUC_lexicon_tags_27 },
    { 0xeb3ce861, SUC_lexicon_tags_32 },
    { 0x129ce85f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ce866, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x37dce868, SUC_lexicon_tags_21 },
    { 0xb5ce868, SUC_lexicon_tags_7 },
    { 0x8fce868, SUC_lexicon_tags_16 },
    { 0xf93ce86b, SUC_lexicon_tags_30 },
    { 0xe97ce86b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bce871, SUC_lexicon_tags_17 },
    { 0x49bce872, SUC_lexicon_tags_32 },
    { 0x405ce873, SUC_lexicon_tags_7 },
    { 0xcf5ce874, SUC_lexicon_tags_56 },
    { 0xb5dce875, SUC_lexicon_tags_45 },
    { 0x769ce876, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30fce878, SUC_lexicon_tags_43 },
    { 0x87ce879, SUC_lexicon_tags_8 },
    { 0x3c3ce87a, SUC_lexicon_tags_236 },
    { 0x2d5ce87b, SUC_lexicon_tags_23 },
    { 0x48fce87b, SUC_lexicon_tags_11 },
    { 0xb49ce87d, SUC_lexicon_tags_0 },
    { 0xb35ce87a, SUC_lexicon_tags_17 },
    { 0x9f1ce87f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1ce882, SUC_lexicon_tags_33 },
    { 0x261ce882, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fce888, SUC_lexicon_tags_34 },
    { 0xc0fce889, SUC_lexicon_tags_15 },
    { 0xddbce88a, SUC_lexicon_tags_8 },
    { 0xea5ce88a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc29ce88d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efce896, SUC_lexicon_tags_25 },
    { 0x939ce897, SUC_lexicon_tags_39 },
    { 0x203ce898, SUC_lexicon_tags_60 },
    { 0x3cbce899, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3bdce89b, SUC_lexicon_tags_3 },
    { 0x429ce89c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5ce89f, SUC_lexicon_tags_37 },
    { 0x503ce8a0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9ce8a6, SUC_lexicon_tags_37 },
    { 0xd59ce8a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fce8ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabce8af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdcfce8b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x199ce8b4, SUC_lexicon_tags_15 },
    { 0xa8dce8b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66fce8b8, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xff7ce8ba, SUC_lexicon_tags_17 },
    { 0xbbdce8ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1ce8c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfce8c5, SUC_lexicon_tags_15 },
    { 0xbbbce8c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x741ce8c8, SUC_lexicon_tags_12 },
    { 0xf23ce8c9, SUC_lexicon_tags_43 },
    { 0x94bce8c8, SUC_lexicon_tags_17 },
    { 0x191ce8cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa7dce8cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701ce8d1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfa1ce8d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435ce8d7, SUC_lexicon_tags_8 },
    { 0x303ce8d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5ce8dc, SUC_lexicon_tags_30 },
    { 0xe05ce8dd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa9ce8df, SUC_lexicon_tags_2 },
    { 0x537ce8e0, SUC_lexicon_tags_38 },
    { 0xaffce8e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4a3ce8e3, SUC_lexicon_tags_17 },
    { 0xc1fce8e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3ce8e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x539ce8ea, SUC_lexicon_tags_37 },
    { 0x24fce8eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab5ce8ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dce8f9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01ce8fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7ce900, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x33ce902, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x639ce905, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315ce90c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x665ce90f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfce912, SUC_lexicon_tags_27 },
    { 0x1efce913, SUC_lexicon_tags_7 },
    { 0xa81ce913, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9efce916, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdce91c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ce920, SUC_lexicon_tags_47 },
    { 0xa1ce921, SUC_lexicon_tags_65 },
    { 0xb7dce922, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973ce92c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbabce92e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3ce933, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5ce937, SUC_lexicon_tags_43 },
    { 0x751ce937, SUC_lexicon_tags_170 },
    { 0xf5bce938, SUC_lexicon_tags_2 },
    { 0xad1ce93a, SUC_lexicon_tags_23 },
    { 0x47bce937, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dce941, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x595ce944, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12bce946, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11ce94a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9ce94f, SUC_lexicon_tags_7 },
    { 0x83ce950, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3ce956, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2b3ce958, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2edce95a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73ce95c, SUC_lexicon_tags_37 },
    { 0xeadce95c, SUC_lexicon_tags_46 },
    { 0x7f9ce95c, SUC_lexicon_tags_38 },
    { 0x377ce95d, SUC_lexicon_tags_26 },
    { 0x309ce95d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdce966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ddce968, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x68fce96b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5e9ce96d, SUC_lexicon_tags_37 },
    { 0x259ce96e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x893ce970, SUC_lexicon_tags_63 },
    { 0x105ce971, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31bce973, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bce976, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe37ce978, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9ce97c, SUC_lexicon_tags_1 },
    { 0x19dce97c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59ce982, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x90dce984, SUC_lexicon_tags_43 },
    { 0x619ce985, SUC_lexicon_tags_1 },
    { 0x607ce986, SUC_lexicon_tags_43 },
    { 0xdf5ce987, SUC_lexicon_tags_26 },
    { 0x66dce987, SUC_lexicon_tags_62 },
    { 0x93ce988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26dce98c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbce98e, SUC_lexicon_tags_25 },
    { 0x659ce98e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x50fce991, SUC_lexicon_tags_15 },
    { 0xa1ce992, SUC_lexicon_tags_0 },
    { 0xb3fce991, SUC_lexicon_tags_65 },
    { 0x7cfce994, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697ce998, SUC_lexicon_tags_15 },
    { 0x123ce998, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789ce9a0, SUC_lexicon_tags_25 },
    { 0xfbbce9a1, SUC_lexicon_tags_47 },
    { 0x8f9ce9a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x447ce9a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x683ce9a6, SUC_lexicon_tags_55 },
    { 0xb3dce9a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811ce9ac, SUC_lexicon_tags_16 },
    { 0x3dce9ad, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffce9b1, SUC_lexicon_tags_7 },
    { 0x157ce9b1, SUC_lexicon_tags_17 },
    { 0x3bfce9b3, SUC_lexicon_tags_0 },
    { 0xcdfce9b4, SUC_lexicon_tags_29 },
    { 0xf15ce9b5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7bce9b7, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xe61ce9b9, SUC_lexicon_tags_14 },
    { 0xaa9ce9ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55ce9bf, SUC_lexicon_tags_37 },
    { 0xadce9c0, SUC_lexicon_tags_45 },
    { 0x70fce9bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1ce9c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbce9c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x53fce9ca, SUC_lexicon_tags_9 },
    { 0x62dce9ca, SUC_lexicon_tags_160 },
    { 0x433ce9ca, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x543ce9ce, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x72fce9d1, SUC_lexicon_tags_33 },
    { 0x537ce9d2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa9dce9d4, SUC_lexicon_tags_29 },
    { 0xd81ce9d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9ce9d9, SUC_lexicon_tags_140 },
    { 0x7dbce9da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x503ce9dd, SUC_lexicon_tags_25 },
    { 0xf23ce9de, SUC_lexicon_tags_2 },
    { 0x6ffce9dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd35ce9e1, SUC_lexicon_tags_30 },
    { 0x67dce9e2, SUC_lexicon_tags_30 },
    { 0x1e3ce9e3, SUC_lexicon_tags_30 },
    { 0x38dce9e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b1ce9e6, SUC_lexicon_tags_45 },
    { 0xaedce9e6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x981ce9ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x403ce9ec, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x23dce9ef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159ce9f4, SUC_lexicon_tags_38 },
    { 0xe59ce9f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37ce9f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881cea05, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x29dcea08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x30bcea0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf21cea0c, SUC_lexicon_tags_11 },
    { 0xab3cea0d, SUC_lexicon_tags_27 },
    { 0xde3cea0e, SUC_lexicon_tags_30 },
    { 0xe05cea0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f5cea11, SUC_lexicon_tags_23 },
    { 0x793cea12, SUC_lexicon_tags_47 },
    { 0x775cea11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5ddcea15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbcea18, SUC_lexicon_tags_37 },
    { 0x251cea19, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x863cea1b, SUC_lexicon_tags_27 },
    { 0x9a7cea1b, SUC_lexicon_tags_63 },
    { 0x903cea1c, SUC_lexicon_tags_30 },
    { 0x131cea1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb7cea20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4dcea22, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fcea27, SUC_lexicon_tags_37 },
    { 0x467cea28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dcea2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x98dcea30, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x167cea33, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7cea36, SUC_lexicon_tags_37 },
    { 0x9d7cea37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x935cea39, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bcea3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x665cea3f, SUC_lexicon_tags_60 },
    { 0xd35cea40, SUC_lexicon_tags_17 },
    { 0xd1dcea40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9cea47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfebcea49, SUC_lexicon_tags_29 },
    { 0x745cea49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9cea50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb6bcea52, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58dcea57, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3cea5c, SUC_lexicon_tags_0 },
    { 0x53dcea5d, SUC_lexicon_tags_33 },
    { 0x3cea5e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd63cea60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x885cea63, SUC_lexicon_tags_77 },
    { 0xdedcea63, SUC_lexicon_tags_12 },
    { 0x8fbcea63, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899cea6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b1cea6c, SUC_lexicon_tags_27 },
    { 0xc71cea6d, SUC_lexicon_tags_26 },
    { 0x7ebcea6e, SUC_lexicon_tags_26 },
    { 0xe2fcea6f, SUC_lexicon_tags_27 },
    { 0xd01cea6c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21cea75, SUC_lexicon_tags_32 },
    { 0x607cea76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451cea7b, SUC_lexicon_tags_37 },
    { 0x4ddcea7c, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbcea81, SUC_lexicon_tags_0 },
    { 0xd4bcea81, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9cea88, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81cea91, SUC_lexicon_tags_11 },
    { 0xe19cea92, SUC_lexicon_tags_66 },
    { 0xc39cea93, SUC_lexicon_tags_63 },
    { 0xf7fcea94, SUC_lexicon_tags_11 },
    { 0x525cea95, SUC_lexicon_tags_37 },
    { 0x54bcea96, SUC_lexicon_tags_10 },
    { 0xb0dcea97, SUC_lexicon_tags_11 },
    { 0xbe3cea94, SUC_lexicon_tags_14 },
    { 0xb33cea99, SUC_lexicon_tags_0 },
    { 0xe1dcea9a, SUC_lexicon_tags_11 },
    { 0x53fcea99, SUC_lexicon_tags_13 },
    { 0x937cea96, SUC_lexicon_tags_1 },
    { 0xbadcea9d, SUC_lexicon_tags_15 },
    { 0x8dfcea9d, SUC_lexicon_tags_14 },
    { 0x7dbcea95, SUC_lexicon_tags_15 },
    { 0x8d5ceaa0, SUC_lexicon_tags_33 },
    { 0xdc9ceaa1, SUC_lexicon_tags_11 },
    { 0x827ceaa0, SUC_lexicon_tags_17 },
    { 0x493ceaa3, SUC_lexicon_tags_12 },
    { 0xc45ceaa4, SUC_lexicon_tags_23 },
    { 0x849cea97, SUC_lexicon_tags_1 },
    { 0x5a7cea92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27ceaaa, SUC_lexicon_tags_7 },
    { 0x5f9ceaaa, SUC_lexicon_tags_25 },
    { 0xb3bceaac, SUC_lexicon_tags_7 },
    { 0x4abceaad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41ceab0, SUC_lexicon_tags_19 },
    { 0x66bceab1, SUC_lexicon_tags_5 },
    { 0xf93ceab1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x493ceab5, SUC_lexicon_tags_7 },
    { 0x303ceab6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdceaba, SUC_lexicon_tags_33 },
    { 0xa0bceabb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x245ceabd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bceac3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xeadceac5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fceac8, SUC_lexicon_tags_47 },
    { 0x11bceac9, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0x6bceacb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7cead0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fcead4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x935cead6, SUC_lexicon_tags_17 },
    { 0x8d5cead7, SUC_lexicon_tags_2 },
    { 0x5b3cead8, SUC_lexicon_tags_15 },
    { 0x9b3cead9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9ceadd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1ceae0, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xf13ceae2, SUC_lexicon_tags_11 },
    { 0x363ceae3, SUC_lexicon_tags_11 },
    { 0xb91ceae3, SUC_lexicon_tags_29 },
    { 0xd93ceae5, SUC_lexicon_tags_1 },
    { 0x43dceae6, SUC_lexicon_tags_35 },
    { 0x50bceae7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893ceaec, SUC_lexicon_tags_293 },
    { 0x72bceaed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89ceaf2, SUC_lexicon_tags_12 },
    { 0x41dceaf3, SUC_lexicon_tags_12 },
    { 0x171ceaf4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fceafb, SUC_lexicon_tags_17 },
    { 0x3b7ceafb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801ceb00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857ceb07, SUC_lexicon_tags_1 },
    { 0x873ceb08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb31ceb0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbceb10, SUC_lexicon_tags_7 },
    { 0x269ceb11, SUC_lexicon_tags_8 },
    { 0xc7ceb12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x23dceb14, SUC_lexicon_tags_11 },
    { 0x995ceb15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb41ceb17, SUC_lexicon_tags_136 },
    { 0xebbceb18, SUC_lexicon_tags_9 },
    { 0xcb5ceb19, SUC_lexicon_tags_17 },
    { 0x46fceb1a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x13ceb1d, SUC_lexicon_tags_34 },
    { 0x7f5ceb1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddceb23, SUC_lexicon_tags_2 },
    { 0x671ceb24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdbceb26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbceb29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x40dceb2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591ceb33, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddceb39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809ceb3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985ceb41, SUC_lexicon_tags_7 },
    { 0xa15ceb41, SUC_lexicon_tags_57 },
    { 0x33ceb41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575ceb47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1ceb4c, SUC_lexicon_tags_28 },
    { 0x4abceb4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153ceb52, SUC_lexicon_tags_47 },
    { 0xa03ceb53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc4fceb55, SUC_lexicon_tags_7 },
    { 0xb29ceb56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadceb5e, SUC_lexicon_tags_44 },
    { 0xf7dceb5f, SUC_lexicon_tags_25 },
    { 0x8fdceb60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x53fceb63, SUC_lexicon_tags_1 },
    { 0x663ceb64, SUC_lexicon_tags_11 },
    { 0x571ceb65, SUC_lexicon_tags_68 },
    { 0x23ceb65, SUC_lexicon_tags_17 },
    { 0x9bceb63, SUC_lexicon_tags_7 },
    { 0x345ceb64, SUC_lexicon_tags_45 },
    { 0x7b1ceb69, SUC_lexicon_tags_38 },
    { 0x243ceb6a, SUC_lexicon_tags_25 },
    { 0x21bceb69, SUC_lexicon_tags_31 },
    { 0xf01ceb6c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ceb70, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21ceb73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2afceb75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd89ceb77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229ceb7b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5ceb7e, SUC_lexicon_tags_1 },
    { 0x9e1ceb7f, SUC_lexicon_tags_34 },
    { 0x4edceb7f, SUC_lexicon_tags_25 },
    { 0x1fdceb81, SUC_lexicon_tags_2 },
    { 0x1f3ceb7f, SUC_lexicon_tags_11 },
    { 0xb0dceb7f, SUC_lexicon_tags_15 },
    { 0x47fceb84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ceb88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffceb90, SUC_lexicon_tags_37 },
    { 0x66dceb90, SUC_lexicon_tags_68 },
    { 0xa7dceb92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7ceb96, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13ceb9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x637ceb9d, SUC_lexicon_tags_49 },
    { 0xdfbceb9d, SUC_lexicon_tags_7 },
    { 0x74bceb9e, SUC_lexicon_tags_25 },
    { 0xb91ceb9e, SUC_lexicon_tags_0 },
    { 0xe55ceba1, SUC_lexicon_tags_19 },
    { 0x3a5ceba2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835ceba8, SUC_lexicon_tags_45 },
    { 0x5dceba9, SUC_lexicon_tags_2 },
    { 0x859cebaa, SUC_lexicon_tags_25 },
    { 0xf79cebab, SUC_lexicon_tags_46 },
    { 0x577cebab, SUC_lexicon_tags_17 },
    { 0xa71cebac, SUC_lexicon_tags_17 },
    { 0x743ceba9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9edcebb0, SUC_lexicon_tags_27 },
    { 0xf3bcebb0, SUC_lexicon_tags_17 },
    { 0x1c7cebb1, SUC_lexicon_tags_17 },
    { 0xd71cebb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a3cebb5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81cebbb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995cebc7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3cbcebc9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bcebd6, SUC_lexicon_tags_27 },
    { 0x4d5cebd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483cebdd, SUC_lexicon_tags_7 },
    { 0x117cebdd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x283cebe0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdcebe3, SUC_lexicon_tags_2 },
    { 0x9cdcebe4, SUC_lexicon_tags_14 },
    { 0x7d7cebe5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf77cebe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cebed, SUC_lexicon_tags_15 },
    { 0xc23cebee, SUC_lexicon_tags_2 },
    { 0x25fcebef, SUC_lexicon_tags_12 },
    { 0x563cebef, SUC_lexicon_tags_39 },
    { 0xf09cebef, SUC_lexicon_tags_11 },
    { 0xb2dcebf2, SUC_lexicon_tags_7 },
    { 0x297cebf3, SUC_lexicon_tags_12 },
    { 0x23fcebf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fcebf7, SUC_lexicon_tags_17 },
    { 0x52fcebf7, SUC_lexicon_tags_17 },
    { 0xd3dcebf8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30dcebfb, SUC_lexicon_tags_11 },
    { 0x623cebfb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dcec00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7cec03, SUC_lexicon_tags_40 },
    { 0x8fbcec03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe1fcec06, SUC_lexicon_tags_25 },
    { 0xc29cec07, SUC_lexicon_tags_28 },
    { 0x1a1cec07, SUC_lexicon_tags_17 },
    { 0x62fcec09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dcec0e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbcec11, SUC_lexicon_tags_49 },
    { 0x5dbcec12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77cec15, SUC_lexicon_tags_10 },
    { 0xd27cec15, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107cec1b, SUC_lexicon_tags_1 },
    { 0x193cec1c, SUC_lexicon_tags_7 },
    { 0xf0dcec1d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9cec24, SUC_lexicon_tags_15 },
    { 0x5f5cec25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dcec29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3e5cec2b, SUC_lexicon_tags_1 },
    { 0x575cec2c, SUC_lexicon_tags_23 },
    { 0xd9bcec2d, SUC_lexicon_tags_17 },
    { 0xc77cec2e, SUC_lexicon_tags_12 },
    { 0xb65cec2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dcec36, SUC_lexicon_tags_45 },
    { 0x1afcec36, SUC_lexicon_tags_24 },
    { 0x8c1cec38, SUC_lexicon_tags_2 },
    { 0x415cec39, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5cec42, SUC_lexicon_tags_14 },
    { 0xf47cec43, SUC_lexicon_tags_37 },
    { 0xf7fcec44, SUC_lexicon_tags_20 },
    { 0xb91cec45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101cec49, SUC_lexicon_tags_65 },
    { 0xda5cec4a, SUC_lexicon_tags_0 },
    { 0xa3cec4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61dcec4e, SUC_lexicon_tags_60 },
    { 0x5afcec4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5cec52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249cec56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfcec59, SUC_lexicon_tags_17 },
    { 0xbc1cec5a, SUC_lexicon_tags_21 },
    { 0x75bcec5b, SUC_lexicon_tags_30 },
    { 0x99fcec5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1cdcec5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329cec63, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x487cec65, SUC_lexicon_tags_34 },
    { 0x62bcec66, SUC_lexicon_tags_8 },
    { 0xa43cec66, SUC_lexicon_tags_15 },
    { 0xf91cec65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x34fcec6a, SUC_lexicon_tags_19 },
    { 0x413cec6b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bcec6f, SUC_lexicon_tags_39 },
    { 0x24bcec70, SUC_lexicon_tags_37 },
    { 0x56dcec70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73cec76, SUC_lexicon_tags_63 },
    { 0x8a5cec77, SUC_lexicon_tags_11 },
    { 0x33bcec78, SUC_lexicon_tags_17 },
    { 0x545cec79, SUC_lexicon_tags_17 },
    { 0x9b3cec76, SUC_lexicon_tags_32 },
    { 0x68bcec7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffcec80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x877cec82, SUC_lexicon_tags_2 },
    { 0x2e5cec83, SUC_lexicon_tags_1 },
    { 0xf4dcec84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7cec8d, SUC_lexicon_tags_37 },
    { 0xc3bcec8e, SUC_lexicon_tags_7 },
    { 0x28fcec8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3dcec91, SUC_lexicon_tags_129 },
    { 0x2bcec92, SUC_lexicon_tags_15 },
    { 0x3f1cec92, SUC_lexicon_tags_17 },
    { 0xbcdcec94, SUC_lexicon_tags_1 },
    { 0x813cec95, SUC_lexicon_tags_15 },
    { 0x8b5cec93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91cec99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfcec9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7ceca1, SUC_lexicon_tags_15 },
    { 0x333ceca2, SUC_lexicon_tags_25 },
    { 0x2b1ceca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23fceca5, SUC_lexicon_tags_28 },
    { 0xe8bceca6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705cecaa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bcecb2, SUC_lexicon_tags_79 },
    { 0xbffcecb3, SUC_lexicon_tags_39 },
    { 0x85cecb2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457cecb8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x429cecba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cfcecbc, SUC_lexicon_tags_38 },
    { 0xfa9cecbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc35cecbf, SUC_lexicon_tags_63 },
    { 0x903cecc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dcecc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bcecc9, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0xdedceccb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcecd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a1cecd5, SUC_lexicon_tags_25 },
    { 0xbc3cecd6, SUC_lexicon_tags_11 },
    { 0xeb1cecd5, SUC_lexicon_tags_19 },
    { 0xdcecd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ebcecda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dcecdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3cece2, SUC_lexicon_tags_63 },
    { 0xdf1cece2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91cece5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdcece8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bcecef, SUC_lexicon_tags_13 },
    { 0xefcecf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bcecf8, SUC_lexicon_tags_87 },
    { 0xfbfcecf9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229cecfd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23ced00, SUC_lexicon_tags_16 },
    { 0xbd9ced01, SUC_lexicon_tags_23 },
    { 0x3efced02, SUC_lexicon_tags_15 },
    { 0xcb3ced03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x295ced05, SUC_lexicon_tags_11 },
    { 0x1bbced06, SUC_lexicon_tags_38 },
    { 0xba9ced06, SUC_lexicon_tags_16 },
    { 0xe29ced08, SUC_lexicon_tags_11 },
    { 0x9a5ced09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7ced0e, SUC_lexicon_tags_15 },
    { 0x8dbced0e, SUC_lexicon_tags_1 },
    { 0x7fbced0e, SUC_lexicon_tags_11 },
    { 0xd1ced0f, SUC_lexicon_tags_39 },
    { 0xb53ced0e, SUC_lexicon_tags_1 },
    { 0xbbdced0e, SUC_lexicon_tags_17 },
    { 0xce3ced11, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xf37ced16, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5fced18, SUC_lexicon_tags_14 },
    { 0x125ced19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fced21, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x511ced23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abced28, SUC_lexicon_tags_45 },
    { 0x77bced28, SUC_lexicon_tags_17 },
    { 0x3f5ced2a, SUC_lexicon_tags_45 },
    { 0xe51ced2b, SUC_lexicon_tags_11 },
    { 0x765ced2a, SUC_lexicon_tags_100 },
    { 0xbcbced2d, SUC_lexicon_tags_45 },
    { 0xccbced2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x219ced31, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe9bced33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bced3a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875ced3f, SUC_lexicon_tags_794 },
    { 0, NULL },
    { 0xac5ced41, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667ced46, SUC_lexicon_tags_15 },
    { 0xa51ced47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09ced4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b3ced4c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fced4f, SUC_lexicon_tags_37 },
    { 0xd0fced50, SUC_lexicon_tags_25 },
    { 0xb75ced50, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xdbced53, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x9f3ced55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37ced5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ced60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49ced66, SUC_lexicon_tags_2 },
    { 0xa9ced67, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6abced6a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd3dced6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281ced72, SUC_lexicon_tags_15 },
    { 0xaa3ced73, SUC_lexicon_tags_12 },
    { 0xe5fced74, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xab9ced76, SUC_lexicon_tags_30 },
    { 0x9f1ced76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889ced7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51ced80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f9ced82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7ced85, SUC_lexicon_tags_7 },
    { 0x9a1ced85, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01ced8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7ced91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19ced94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x949ced96, SUC_lexicon_tags_12 },
    { 0xba5ced97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbced9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a1ceda0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649ceda5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93ceda8, SUC_lexicon_tags_37 },
    { 0xb1bceda9, SUC_lexicon_tags_1 },
    { 0x86fceda8, SUC_lexicon_tags_38 },
    { 0xf85ceda9, SUC_lexicon_tags_14 },
    { 0xa5dcedab, SUC_lexicon_tags_0 },
    { 0xd0bcedad, SUC_lexicon_tags_107 },
    { 0x6b3cedae, SUC_lexicon_tags_15 },
    { 0x9f9cedae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfcedb2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x175cedb5, SUC_lexicon_tags_21 },
    { 0x6e7cedb6, SUC_lexicon_tags_7 },
    { 0xbf9cedb7, SUC_lexicon_tags_1 },
    { 0x4d3cedb8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35cedbc, SUC_lexicon_tags_15 },
    { 0xf7fcedbd, SUC_lexicon_tags_37 },
    { 0x75bcedbe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491cedc9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x745cedcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe55cedce, SUC_lexicon_tags_17 },
    { 0xcb1cedcf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffcedd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bceddc, SUC_lexicon_tags_17 },
    { 0xf81ceddc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbcede1, SUC_lexicon_tags_12 },
    { 0x647cede1, SUC_lexicon_tags_47 },
    { 0x3f3cede3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3cede8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dceded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59fcedef, SUC_lexicon_tags_17 },
    { 0x211cedef, SUC_lexicon_tags_15 },
    { 0xeedcedf1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53cedf4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2d5cedf6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09cedfb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1cedff, SUC_lexicon_tags_1 },
    { 0x4c5cee00, SUC_lexicon_tags_17 },
    { 0x615cee01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x465cee03, SUC_lexicon_tags_29 },
    { 0x793cee03, SUC_lexicon_tags_3 },
    { 0xd2bcee04, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeadcee07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa61cee09, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6f5cee0b, SUC_lexicon_tags_25 },
    { 0x757cee0c, SUC_lexicon_tags_29 },
    { 0x409cee0c, SUC_lexicon_tags_4 },
    { 0xfbfcee0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x293cee11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dcee14, SUC_lexicon_tags_28 },
    { 0x105cee15, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bcee18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fcee1b, SUC_lexicon_tags_15 },
    { 0x6bcee1c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98dcee20, SUC_lexicon_tags_47 },
    { 0xc05cee21, SUC_lexicon_tags_12 },
    { 0x58dcee22, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcfcee24, SUC_lexicon_tags_17 },
    { 0x8a7cee25, SUC_lexicon_tags_48 },
    { 0x24bcee24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59bcee28, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x213cee2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7fcee2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25fcee30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbcee34, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75dcee39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x635cee3b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5cee3f, SUC_lexicon_tags_25 },
    { 0x89dcee40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc19cee42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6fcee44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fcee4b, SUC_lexicon_tags_17 },
    { 0x87dcee4b, SUC_lexicon_tags_299 },
    { 0x777cee4d, SUC_lexicon_tags_0 },
    { 0xd83cee4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf9cee50, SUC_lexicon_tags_11 },
    { 0x837cee50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e9cee53, SUC_lexicon_tags_136 },
    { 0x9ffcee54, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1cee58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8fcee5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fcee5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5c3cee5f, SUC_lexicon_tags_2 },
    { 0x475cee5f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bcee68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c3cee6a, SUC_lexicon_tags_47 },
    { 0x2b1cee6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x811cee6d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1cee70, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fbcee72, SUC_lexicon_tags_65 },
    { 0x125cee73, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x929cee75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfcee79, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88bcee7b, SUC_lexicon_tags_36 },
    { 0x295cee7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bcee84, SUC_lexicon_tags_61 },
    { 0xd5cee85, SUC_lexicon_tags_31 },
    { 0xd0bcee86, SUC_lexicon_tags_33 },
    { 0x263cee87, SUC_lexicon_tags_21 },
    { 0x9cee85, SUC_lexicon_tags_27 },
    { 0x3fdcee89, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0x37bcee8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55cee90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1cee93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfb1cee95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67cee9b, SUC_lexicon_tags_37 },
    { 0xd71cee9b, SUC_lexicon_tags_12 },
    { 0xc1cee9b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbfdcee9f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bceea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x791ceea5, SUC_lexicon_tags_25 },
    { 0xecbceea6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97ceea9, SUC_lexicon_tags_37 },
    { 0x67fceeaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52dceeac, SUC_lexicon_tags_224 },
    { 0xb87ceead, SUC_lexicon_tags_37 },
    { 0x7c5ceeae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3ceeb1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x389ceeb3, SUC_lexicon_tags_1 },
    { 0xd19ceeb3, SUC_lexicon_tags_47 },
    { 0x59dceeb3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7ceeb9, SUC_lexicon_tags_8 },
    { 0x3f7ceeb9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9ceec2, SUC_lexicon_tags_11 },
    { 0x197ceec2, SUC_lexicon_tags_25 },
    { 0x779ceec2, SUC_lexicon_tags_17 },
    { 0x4b7ceec2, SUC_lexicon_tags_1 },
    { 0x12fceec6, SUC_lexicon_tags_37 },
    { 0x751ceec7, SUC_lexicon_tags_26 },
    { 0x22dceec8, SUC_lexicon_tags_32 },
    { 0xd25ceec8, SUC_lexicon_tags_31 },
    { 0xb55ceec8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd11ceecc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x173ceece, SUC_lexicon_tags_77 },
    { 0xebfceece, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x23ceed2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563ceedb, SUC_lexicon_tags_68 },
    { 0x6d9ceedb, SUC_lexicon_tags_151 },
    { 0xc4dceedd, SUC_lexicon_tags_2 },
    { 0x983ceedd, SUC_lexicon_tags_0 },
    { 0x7f9ceedd, SUC_lexicon_tags_33 },
    { 0x13fceedd, SUC_lexicon_tags_25 },
    { 0x8efceee1, SUC_lexicon_tags_1 },
    { 0xdefceee1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x205ceee4, SUC_lexicon_tags_62 },
    { 0xe23ceee5, SUC_lexicon_tags_1489 },
    { 0, NULL },
    { 0xfe1ceee7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x51bceeea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71ceeed, SUC_lexicon_tags_7 },
    { 0xd11ceeee, SUC_lexicon_tags_17 },
    { 0x6c3ceeee, SUC_lexicon_tags_29 },
    { 0xc01ceef0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bceef6, SUC_lexicon_tags_37 },
    { 0xd13ceef7, SUC_lexicon_tags_60 },
    { 0xb31ceef7, SUC_lexicon_tags_12 },
    { 0x979ceef9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ceeff, SUC_lexicon_tags_37 },
    { 0x8e9cef00, SUC_lexicon_tags_19 },
    { 0xbfbcef00, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc5fcef03, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb57cef05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1cef09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9cef0c, SUC_lexicon_tags_68 },
    { 0xe3bcef0c, SUC_lexicon_tags_14 },
    { 0xb9cef0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x297cef11, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503cef15, SUC_lexicon_tags_38 },
    { 0x59bcef16, SUC_lexicon_tags_25 },
    { 0x36fcef16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9cef1b, SUC_lexicon_tags_45 },
    { 0x55bcef1c, SUC_lexicon_tags_25 },
    { 0x149cef1d, SUC_lexicon_tags_11 },
    { 0x675cef1d, SUC_lexicon_tags_48 },
    { 0x16bcef1d, SUC_lexicon_tags_7 },
    { 0xf93cef1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31cef24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09cef2b, SUC_lexicon_tags_29 },
    { 0x1f3cef2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x549cef2e, SUC_lexicon_tags_17 },
    { 0xcb5cef2f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5cef33, SUC_lexicon_tags_37 },
    { 0x147cef33, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x47cef36, SUC_lexicon_tags_12 },
    { 0xcdfcef37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5cef3f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x347cef42, SUC_lexicon_tags_323 },
    { 0x5f7cef43, SUC_lexicon_tags_19 },
    { 0xd83cef44, SUC_lexicon_tags_8 },
    { 0xac5cef45, SUC_lexicon_tags_75 },
    { 0x1c9cef43, SUC_lexicon_tags_28 },
    { 0x4c5cef47, SUC_lexicon_tags_17 },
    { 0xf35cef48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x93fcef4a, SUC_lexicon_tags_7 },
    { 0x18dcef4a, SUC_lexicon_tags_156 },
    { 0x3e5cef4c, SUC_lexicon_tags_3 },
    { 0xe91cef4d, SUC_lexicon_tags_19 },
    { 0xe8dcef4e, SUC_lexicon_tags_2 },
    { 0x925cef4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd5fcef51, SUC_lexicon_tags_68 },
    { 0xa87cef51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fcef56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4afcef58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3cef5a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c3cef5c, SUC_lexicon_tags_37 },
    { 0x8a3cef5d, SUC_lexicon_tags_2 },
    { 0x9d7cef5e, SUC_lexicon_tags_77 },
    { 0xefcef5f, SUC_lexicon_tags_641 },
    { 0x1a5cef5f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x529cef63, SUC_lexicon_tags_27 },
    { 0x457cef64, SUC_lexicon_tags_39 },
    { 0xe4dcef64, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5cef68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e5cef6a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6edcef6c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bcef7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77cef7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55cef82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x119cef85, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dcef8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x30dcef90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7cef94, SUC_lexicon_tags_43 },
    { 0x86dcef94, SUC_lexicon_tags_26 },
    { 0x213cef96, SUC_lexicon_tags_49 },
    { 0xe29cef97, SUC_lexicon_tags_14 },
    { 0xd3cef94, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x145cef9a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd79cef9c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75cef9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7a9cefa1, SUC_lexicon_tags_15 },
    { 0x3edcefa2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3cefae, SUC_lexicon_tags_19 },
    { 0xfd1cefaf, SUC_lexicon_tags_12 },
    { 0x63dcefb0, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xaf7cefb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05cefbd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9cefc3, SUC_lexicon_tags_1 },
    { 0x19fcefc3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9cefc9, SUC_lexicon_tags_38 },
    { 0x2f5cefca, SUC_lexicon_tags_14 },
    { 0x4d5cefcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2bcefcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf7cefcf, SUC_lexicon_tags_32 },
    { 0xcbdcefd0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x837cefd3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373cefd9, SUC_lexicon_tags_140 },
    { 0x5abcefd9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1cefde, SUC_lexicon_tags_37 },
    { 0xc93cefdf, SUC_lexicon_tags_2 },
    { 0xa87cefdf, SUC_lexicon_tags_11 },
    { 0x85cefdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d5cefe3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x425cefe6, SUC_lexicon_tags_11 },
    { 0xdc1cefe6, SUC_lexicon_tags_107 },
    { 0x2f9cefe8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967cefef, SUC_lexicon_tags_6 },
    { 0x135ceff0, SUC_lexicon_tags_17 },
    { 0xc6dceff1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2ebceff3, SUC_lexicon_tags_26 },
    { 0x307ceff3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497ceffb, SUC_lexicon_tags_11 },
    { 0x4fceffb, SUC_lexicon_tags_1 },
    { 0x9d7ceffd, SUC_lexicon_tags_55 },
    { 0xcdceffe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bcf006, SUC_lexicon_tags_14 },
    { 0xba5cf007, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa5bcf009, SUC_lexicon_tags_11 },
    { 0x8fdcf00a, SUC_lexicon_tags_11 },
    { 0x8c9cf00a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fcf00f, SUC_lexicon_tags_27 },
    { 0x24bcf00f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf13cf012, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdb1cf014, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dcf01a, SUC_lexicon_tags_15 },
    { 0x5bfcf01b, SUC_lexicon_tags_15 },
    { 0x35cf01b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x473cf01f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23cf022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4fbcf024, SUC_lexicon_tags_65 },
    { 0xe07cf024, SUC_lexicon_tags_25 },
    { 0x58bcf026, SUC_lexicon_tags_1 },
    { 0xb37cf024, SUC_lexicon_tags_57 },
    { 0x60dcf028, SUC_lexicon_tags_17 },
    { 0xc55cf025, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bcf034, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffcf037, SUC_lexicon_tags_17 },
    { 0x8c7cf038, SUC_lexicon_tags_32 },
    { 0xc6fcf039, SUC_lexicon_tags_12 },
    { 0xc2bcf03a, SUC_lexicon_tags_15 },
    { 0xc89cf039, SUC_lexicon_tags_7 },
    { 0xd63cf03b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43fcf03e, SUC_lexicon_tags_11 },
    { 0x32dcf03e, SUC_lexicon_tags_11 },
    { 0x3fcf03f, SUC_lexicon_tags_48 },
    { 0x715cf03e, SUC_lexicon_tags_1 },
    { 0x25cf042, SUC_lexicon_tags_15 },
    { 0xed7cf043, SUC_lexicon_tags_27 },
    { 0x35cf044, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d3cf046, SUC_lexicon_tags_2 },
    { 0xd91cf047, SUC_lexicon_tags_17 },
    { 0x1d9cf048, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x815cf04b, SUC_lexicon_tags_19 },
    { 0x881cf04b, SUC_lexicon_tags_49 },
    { 0xf77cf04d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bcf053, SUC_lexicon_tags_17 },
    { 0x599cf053, SUC_lexicon_tags_15 },
    { 0xe6bcf054, SUC_lexicon_tags_107 },
    { 0xa9fcf054, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1cf059, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dcf05d, SUC_lexicon_tags_37 },
    { 0xa25cf05e, SUC_lexicon_tags_84 },
    { 0xbefcf05f, SUC_lexicon_tags_2 },
    { 0x171cf060, SUC_lexicon_tags_25 },
    { 0xcebcf061, SUC_lexicon_tags_33 },
    { 0x53cf05f, SUC_lexicon_tags_19 },
    { 0xda5cf062, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17cf068, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607cf06c, SUC_lexicon_tags_28 },
    { 0x7a5cf06d, SUC_lexicon_tags_28 },
    { 0x265cf06e, SUC_lexicon_tags_38 },
    { 0x88bcf06e, SUC_lexicon_tags_1 },
    { 0x927cf06f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffcf073, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x337cf076, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dcf079, SUC_lexicon_tags_31 },
    { 0xb0dcf079, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc3cf07c, SUC_lexicon_tags_26 },
    { 0x8e7cf07c, SUC_lexicon_tags_25 },
    { 0xd7cf07e, SUC_lexicon_tags_44 },
    { 0xb9cf07e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3cf083, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x231cf085, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35cf08c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bcf091, SUC_lexicon_tags_63 },
    { 0xc8dcf091, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611cf09c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdcf09f, SUC_lexicon_tags_17 },
    { 0xc97cf0a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe99cf0a2, SUC_lexicon_tags_25 },
    { 0xc73cf0a3, SUC_lexicon_tags_47 },
    { 0x765cf0a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9e9cf0a6, SUC_lexicon_tags_1 },
    { 0xb27cf0a7, SUC_lexicon_tags_17 },
    { 0x17bcf0a8, SUC_lexicon_tags_37 },
    { 0x1cfcf0a7, SUC_lexicon_tags_47 },
    { 0xb4dcf0aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x95cf0ac, SUC_lexicon_tags_38 },
    { 0x9c3cf0ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbcf0b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x459cf0b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bcf0b9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x64fcf0bb, SUC_lexicon_tags_10 },
    { 0x591cf0bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xaedcf0be, SUC_lexicon_tags_33 },
    { 0xd65cf0bf, SUC_lexicon_tags_4 },
    { 0x82fcf0c0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bcf0c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479cf0cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9cf0ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3fcf0d0, SUC_lexicon_tags_87 },
    { 0x69dcf0d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94dcf0d3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7cf0d6, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xb55cf0d8, SUC_lexicon_tags_37 },
    { 0x7f7cf0d9, SUC_lexicon_tags_12 },
    { 0x57bcf0da, SUC_lexicon_tags_21 },
    { 0x6d3cf0db, SUC_lexicon_tags_57 },
    { 0x2e7cf0dc, SUC_lexicon_tags_38 },
    { 0x637cf0dc, SUC_lexicon_tags_12 },
    { 0x2cfcf0db, SUC_lexicon_tags_17 },
    { 0x8c1cf0da, SUC_lexicon_tags_56 },
    { 0xcd9cf0da, SUC_lexicon_tags_31 },
    { 0xecdcf0d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1cf0e6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9cf0e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787cf0ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xba7cf0f2, SUC_lexicon_tags_25 },
    { 0x199cf0f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5cf0f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb5cf0f9, SUC_lexicon_tags_33 },
    { 0x989cf0fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5cf102, SUC_lexicon_tags_79 },
    { 0x331cf103, SUC_lexicon_tags_56 },
    { 0x99bcf104, SUC_lexicon_tags_37 },
    { 0x56bcf103, SUC_lexicon_tags_11 },
    { 0x42fcf106, SUC_lexicon_tags_26 },
    { 0x697cf107, SUC_lexicon_tags_12 },
    { 0x57fcf108, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fcf10d, SUC_lexicon_tags_59 },
    { 0x6bfcf10e, SUC_lexicon_tags_15 },
    { 0x99fcf10e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf23cf111, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43cf11b, SUC_lexicon_tags_15 },
    { 0xd6bcf11c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa05cf11e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x57cf120, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bcf123, SUC_lexicon_tags_31 },
    { 0x935cf123, SUC_lexicon_tags_61 },
    { 0x895cf125, SUC_lexicon_tags_7 },
    { 0x893cf126, SUC_lexicon_tags_12 },
    { 0xf51cf126, SUC_lexicon_tags_60 },
    { 0x2d1cf128, SUC_lexicon_tags_27 },
    { 0xe9fcf128, SUC_lexicon_tags_25 },
    { 0x83fcf12a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x251cf12c, SUC_lexicon_tags_15 },
    { 0x77bcf12d, SUC_lexicon_tags_47 },
    { 0xaf7cf12e, SUC_lexicon_tags_38 },
    { 0x725cf12d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc49cf131, SUC_lexicon_tags_25 },
    { 0xfedcf132, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9cf138, SUC_lexicon_tags_98 },
    { 0x447cf139, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17cf13e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x651cf140, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bcf144, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x7cbcf146, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13cf149, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7fdcf14b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7dcf14d, SUC_lexicon_tags_40 },
    { 0x43dcf14e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x173cf150, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa69cf152, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05cf156, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeefcf158, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bcf15d, SUC_lexicon_tags_61 },
    { 0x669cf15d, SUC_lexicon_tags_38 },
    { 0xca7cf15e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x591cf162, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39cf165, SUC_lexicon_tags_51 },
    { 0xc1bcf166, SUC_lexicon_tags_9 },
    { 0xd51cf167, SUC_lexicon_tags_10 },
    { 0x14fcf168, SUC_lexicon_tags_15 },
    { 0x979cf167, SUC_lexicon_tags_68 },
    { 0xca9cf16a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4bbcf16c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377cf174, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fcf178, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x69cf17a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37cf17d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bcf181, SUC_lexicon_tags_7 },
    { 0xfd7cf181, SUC_lexicon_tags_2 },
    { 0xc33cf183, SUC_lexicon_tags_68 },
    { 0xf95cf182, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7cf188, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7cf18f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133cf195, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69cf19b, SUC_lexicon_tags_2 },
    { 0x24dcf19c, SUC_lexicon_tags_32 },
    { 0xec9cf19d, SUC_lexicon_tags_13 },
    { 0xcafcf19c, SUC_lexicon_tags_151 },
    { 0x503cf19c, SUC_lexicon_tags_11 },
    { 0xc49cf19e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1b9cf1a2, SUC_lexicon_tags_27 },
    { 0x9b1cf1a3, SUC_lexicon_tags_19 },
    { 0xfebcf1a4, SUC_lexicon_tags_38 },
    { 0x2fcf1a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665cf1aa, SUC_lexicon_tags_12 },
    { 0xe15cf1ab, SUC_lexicon_tags_13 },
    { 0x2ebcf1ac, SUC_lexicon_tags_22 },
    { 0x743cf1ab, SUC_lexicon_tags_12 },
    { 0xfb9cf1ae, SUC_lexicon_tags_25 },
    { 0x8d1cf1af, SUC_lexicon_tags_60 },
    { 0xf95cf1ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xddcf1b3, SUC_lexicon_tags_68 },
    { 0xd89cf1b4, SUC_lexicon_tags_17 },
    { 0x78bcf1b5, SUC_lexicon_tags_9 },
    { 0xcf9cf1b5, SUC_lexicon_tags_37 },
    { 0x5a1cf1b7, SUC_lexicon_tags_107 },
    { 0xb75cf1b8, SUC_lexicon_tags_49 },
    { 0x6b7cf1b8, SUC_lexicon_tags_11 },
    { 0x733cf1b4, SUC_lexicon_tags_12 },
    { 0xbe5cf1b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899cf1bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb5cf1c1, SUC_lexicon_tags_37 },
    { 0x1f3cf1c1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbcf1c9, SUC_lexicon_tags_15 },
    { 0xc9dcf1ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9cf1cd, SUC_lexicon_tags_2 },
    { 0x971cf1ce, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf39cf1d0, SUC_lexicon_tags_2 },
    { 0x5ffcf1d1, SUC_lexicon_tags_37 },
    { 0x4f9cf1d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09cf1d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x563cf1d9, SUC_lexicon_tags_87 },
    { 0x40dcf1da, SUC_lexicon_tags_1 },
    { 0xdb3cf1da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9cf1de, SUC_lexicon_tags_0 },
    { 0xb23cf1df, SUC_lexicon_tags_33 },
    { 0xb43cf1e0, SUC_lexicon_tags_42 },
    { 0xebdcf1df, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01cf1e5, SUC_lexicon_tags_38 },
    { 0xb5cf1e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78dcf1e9, SUC_lexicon_tags_11 },
    { 0x2dcf1ea, SUC_lexicon_tags_25 },
    { 0xa7fcf1eb, SUC_lexicon_tags_11 },
    { 0x43fcf1eb, SUC_lexicon_tags_38 },
    { 0x147cf1ea, SUC_lexicon_tags_7 },
    { 0xcf5cf1ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcf1f3, SUC_lexicon_tags_27 },
    { 0xcd1cf1f3, SUC_lexicon_tags_14 },
    { 0x395cf1f5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309cf1fa, SUC_lexicon_tags_7 },
    { 0xa5cf1fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81fcf1fd, SUC_lexicon_tags_25 },
    { 0xc17cf1fe, SUC_lexicon_tags_2 },
    { 0xedfcf1fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x757cf202, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1cf206, SUC_lexicon_tags_2 },
    { 0x31bcf207, SUC_lexicon_tags_7 },
    { 0x843cf208, SUC_lexicon_tags_37 },
    { 0x171cf208, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdcf211, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x551cf214, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9cf218, SUC_lexicon_tags_12 },
    { 0xbf3cf219, SUC_lexicon_tags_17 },
    { 0xe4dcf21a, SUC_lexicon_tags_12 },
    { 0xe5cf219, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fcf21f, SUC_lexicon_tags_11 },
    { 0xcd5cf220, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dcf223, SUC_lexicon_tags_14 },
    { 0xaf1cf223, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fcf230, SUC_lexicon_tags_3 },
    { 0xb4fcf230, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x835cf234, SUC_lexicon_tags_26 },
    { 0xb7bcf235, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc85cf237, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x351cf239, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdcf23d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5cf247, SUC_lexicon_tags_11 },
    { 0xba7cf247, SUC_lexicon_tags_11 },
    { 0x239cf249, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x221cf24b, SUC_lexicon_tags_47 },
    { 0xe81cf24c, SUC_lexicon_tags_37 },
    { 0x30bcf24d, SUC_lexicon_tags_40 },
    { 0xfa5cf24d, SUC_lexicon_tags_7 },
    { 0xaddcf24c, SUC_lexicon_tags_25 },
    { 0x443cf250, SUC_lexicon_tags_1 },
    { 0xf43cf250, SUC_lexicon_tags_25 },
    { 0x461cf252, SUC_lexicon_tags_49 },
    { 0x22dcf253, SUC_lexicon_tags_15 },
    { 0xef9cf254, SUC_lexicon_tags_2 },
    { 0x3b3cf251, SUC_lexicon_tags_29 },
    { 0xdbbcf255, SUC_lexicon_tags_1 },
    { 0xc5cf253, SUC_lexicon_tags_23 },
    { 0x5bcf24c, SUC_lexicon_tags_37 },
    { 0xadbcf259, SUC_lexicon_tags_60 },
    { 0xe69cf25a, SUC_lexicon_tags_25 },
    { 0x5c9cf25b, SUC_lexicon_tags_63 },
    { 0xb7dcf25b, SUC_lexicon_tags_43 },
    { 0xc71cf25b, SUC_lexicon_tags_7 },
    { 0x6b5cf25b, SUC_lexicon_tags_26 },
    { 0x83fcf259, SUC_lexicon_tags_23 },
    { 0xdedcf25b, SUC_lexicon_tags_57 },
    { 0xa3fcf260, SUC_lexicon_tags_57 },
    { 0xc6bcf25c, SUC_lexicon_tags_35 },
    { 0xe33cf252, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x615cf265, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfcf26b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231cf270, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb4dcf272, SUC_lexicon_tags_16 },
    { 0x7c1cf273, SUC_lexicon_tags_77 },
    { 0x9ffcf274, SUC_lexicon_tags_17 },
    { 0xbfdcf275, SUC_lexicon_tags_8 },
    { 0x971cf276, SUC_lexicon_tags_11 },
    { 0xb35cf277, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fcf27b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dcf27f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x229cf282, SUC_lexicon_tags_1 },
    { 0xb85cf283, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x711cf286, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6ddcf288, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1cf28b, SUC_lexicon_tags_37 },
    { 0xcbbcf28c, SUC_lexicon_tags_23 },
    { 0x6c1cf28b, SUC_lexicon_tags_15 },
    { 0xc15cf28b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53fcf290, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc1dcf292, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231cf296, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfcf299, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x20fcf29b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fcf29e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11fcf2a2, SUC_lexicon_tags_7 },
    { 0xe25cf2a2, SUC_lexicon_tags_25 },
    { 0xf1cf2a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9cf2a7, SUC_lexicon_tags_53 },
    { 0x5f5cf2a8, SUC_lexicon_tags_25 },
    { 0x35fcf2a9, SUC_lexicon_tags_12 },
    { 0x9f7cf2aa, SUC_lexicon_tags_63 },
    { 0xa3dcf2ab, SUC_lexicon_tags_15 },
    { 0xda7cf2ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc55cf2ae, SUC_lexicon_tags_7 },
    { 0x40dcf2ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x917cf2b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xca5cf2b4, SUC_lexicon_tags_33 },
    { 0xd93cf2b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x805cf2b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cf2ba, SUC_lexicon_tags_17 },
    { 0xaa9cf2bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf05cf2bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x76bcf2c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28fcf2c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957cf2c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7cf2cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dcf2d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcc1cf2d3, SUC_lexicon_tags_7 },
    { 0x81cf2d3, SUC_lexicon_tags_15 },
    { 0x8fbcf2d5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x899cf2d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa61cf2d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3fcf2db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bcf2e1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x201cf2e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fcf2ea, SUC_lexicon_tags_17 },
    { 0x84bcf2eb, SUC_lexicon_tags_15 },
    { 0x999cf2eb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68dcf2f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57cf2f3, SUC_lexicon_tags_17 },
    { 0x88dcf2f4, SUC_lexicon_tags_15 },
    { 0x4a5cf2f5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1cf2f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5cf300, SUC_lexicon_tags_57 },
    { 0x8f9cf300, SUC_lexicon_tags_48 },
    { 0xd65cf302, SUC_lexicon_tags_33 },
    { 0x35dcf303, SUC_lexicon_tags_1 },
    { 0x9afcf301, SUC_lexicon_tags_28 },
    { 0x739cf305, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723cf30b, SUC_lexicon_tags_17 },
    { 0xc79cf30c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x141cf30e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7cf312, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815cf321, SUC_lexicon_tags_25 },
    { 0x6bfcf322, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3bcf324, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47dcf326, SUC_lexicon_tags_17 },
    { 0xfefcf327, SUC_lexicon_tags_25 },
    { 0x781cf328, SUC_lexicon_tags_77 },
    { 0xb57cf329, SUC_lexicon_tags_11 },
    { 0x72dcf329, SUC_lexicon_tags_37 },
    { 0xfd7cf32a, SUC_lexicon_tags_1 },
    { 0xbe3cf32c, SUC_lexicon_tags_19 },
    { 0x7adcf32d, SUC_lexicon_tags_25 },
    { 0xb5cf32d, SUC_lexicon_tags_27 },
    { 0xd73cf32f, SUC_lexicon_tags_15 },
    { 0xe99cf32f, SUC_lexicon_tags_12 },
    { 0x4a1cf32f, SUC_lexicon_tags_37 },
    { 0x433cf332, SUC_lexicon_tags_15 },
    { 0x723cf330, SUC_lexicon_tags_47 },
    { 0xf5fcf32e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc53cf336, SUC_lexicon_tags_2 },
    { 0xb01cf337, SUC_lexicon_tags_77 },
    { 0xc2fcf338, SUC_lexicon_tags_25 },
    { 0x719cf339, SUC_lexicon_tags_60 },
    { 0x857cf33a, SUC_lexicon_tags_2 },
    { 0x5a5cf33b, SUC_lexicon_tags_11 },
    { 0xefbcf33c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbcf340, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653cf349, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81cf350, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x985cf352, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7cf359, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9cf366, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe8dcf368, SUC_lexicon_tags_25 },
    { 0x77fcf368, SUC_lexicon_tags_43 },
    { 0x8cfcf36a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe19cf36c, SUC_lexicon_tags_55 },
    { 0xfb3cf36d, SUC_lexicon_tags_17 },
    { 0x8e1cf36c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bcf373, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7cf376, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85cf379, SUC_lexicon_tags_37 },
    { 0x789cf37a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67cf37e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3cf384, SUC_lexicon_tags_37 },
    { 0x841cf385, SUC_lexicon_tags_27 },
    { 0x5c7cf385, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x96fcf388, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fcf38b, SUC_lexicon_tags_26 },
    { 0xaefcf38b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe11cf38e, SUC_lexicon_tags_15 },
    { 0xd85cf38f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bcf394, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efcf398, SUC_lexicon_tags_66 },
    { 0xe49cf399, SUC_lexicon_tags_17 },
    { 0xc8dcf39a, SUC_lexicon_tags_7 },
    { 0x4f9cf39b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1cf39f, SUC_lexicon_tags_74 },
    { 0xc59cf3a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77cf3a5, SUC_lexicon_tags_25 },
    { 0xb99cf3a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3dcf3a8, SUC_lexicon_tags_49 },
    { 0xb81cf3a9, SUC_lexicon_tags_31 },
    { 0x195cf3a8, SUC_lexicon_tags_60 },
    { 0x761cf3aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x819cf3ad, SUC_lexicon_tags_7 },
    { 0x133cf3ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29cf3b6, SUC_lexicon_tags_231 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafcf3ba, SUC_lexicon_tags_2 },
    { 0x37cf3ba, SUC_lexicon_tags_10 },
    { 0xf47cf3bc, SUC_lexicon_tags_1 },
    { 0xd7cf3bd, SUC_lexicon_tags_11 },
    { 0x35dcf3bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85cf3c1, SUC_lexicon_tags_57 },
    { 0xb67cf3c1, SUC_lexicon_tags_11 },
    { 0xacdcf3c2, SUC_lexicon_tags_1 },
    { 0xff7cf3c1, SUC_lexicon_tags_27 },
    { 0x7e3cf3c5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa05cf3c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7cf3cb, SUC_lexicon_tags_23 },
    { 0xb6dcf3cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d7cf3ce, SUC_lexicon_tags_11 },
    { 0x2edcf3cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07cf3d3, SUC_lexicon_tags_15 },
    { 0xbcfcf3d3, SUC_lexicon_tags_70 },
    { 0x95cf3d5, SUC_lexicon_tags_6 },
    { 0x44dcf3d6, SUC_lexicon_tags_29 },
    { 0x269cf3d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9cf3da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7cf3de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1cf3e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259cf3ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fcf3ee, SUC_lexicon_tags_7 },
    { 0x651cf3ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x351cf3f2, SUC_lexicon_tags_0 },
    { 0x9a5cf3f3, SUC_lexicon_tags_27 },
    { 0xf33cf3f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dfcf3f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbcf3fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dcf3fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf37cf400, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fcf406, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831cf40d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adcf410, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfa5cf412, SUC_lexicon_tags_29 },
    { 0xb55cf413, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x251cf416, SUC_lexicon_tags_33 },
    { 0xac3cf416, SUC_lexicon_tags_8 },
    { 0xa8dcf418, SUC_lexicon_tags_12 },
    { 0x11dcf419, SUC_lexicon_tags_10 },
    { 0x1c5cf41a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dcf41e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x337cf420, SUC_lexicon_tags_37 },
    { 0xc1bcf421, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd3dcf423, SUC_lexicon_tags_17 },
    { 0x1cf424, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xd6bcf426, SUC_lexicon_tags_32 },
    { 0x29cf426, SUC_lexicon_tags_12 },
    { 0x343cf426, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3cf42b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x227cf42d, SUC_lexicon_tags_11 },
    { 0x393cf42e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x25dcf431, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689cf435, SUC_lexicon_tags_23 },
    { 0xb91cf435, SUC_lexicon_tags_19 },
    { 0xfe1cf437, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x713cf43a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79cf43e, SUC_lexicon_tags_15 },
    { 0xb13cf43e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5cf442, SUC_lexicon_tags_15 },
    { 0x4edcf443, SUC_lexicon_tags_25 },
    { 0xc29cf444, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26dcf446, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f7cf448, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3cf44b, SUC_lexicon_tags_11 },
    { 0x2efcf44c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567cf452, SUC_lexicon_tags_12 },
    { 0x22fcf453, SUC_lexicon_tags_37 },
    { 0x44fcf454, SUC_lexicon_tags_27 },
    { 0xc17cf455, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5cf459, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebcf45f, SUC_lexicon_tags_63 },
    { 0x763cf45f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xdfbcf462, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1cf469, SUC_lexicon_tags_19 },
    { 0x9b7cf469, SUC_lexicon_tags_14 },
    { 0xa2dcf469, SUC_lexicon_tags_32 },
    { 0xa1bcf46c, SUC_lexicon_tags_33 },
    { 0xea9cf46d, SUC_lexicon_tags_11 },
    { 0x2a1cf46e, SUC_lexicon_tags_12 },
    { 0x621cf46f, SUC_lexicon_tags_17 },
    { 0x12dcf470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327cf477, SUC_lexicon_tags_7 },
    { 0x6f5cf478, SUC_lexicon_tags_19 },
    { 0xb8dcf479, SUC_lexicon_tags_18 },
    { 0xc03cf478, SUC_lexicon_tags_10 },
    { 0xf03cf47b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5cf483, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51cf486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1cf48e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fcf491, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x151cf494, SUC_lexicon_tags_7 },
    { 0x723cf494, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb4fcf497, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd51cf499, SUC_lexicon_tags_45 },
    { 0x537cf49a, SUC_lexicon_tags_7 },
    { 0x66fcf49a, SUC_lexicon_tags_82 },
    { 0x929cf49b, SUC_lexicon_tags_25 },
    { 0x97bcf49a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8f5cf49f, SUC_lexicon_tags_7 },
    { 0x683cf4a0, SUC_lexicon_tags_60 },
    { 0x1a9cf4a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dcf4a4, SUC_lexicon_tags_26 },
    { 0xc8dcf4a4, SUC_lexicon_tags_60 },
    { 0xd87cf4a6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e3cf4a8, SUC_lexicon_tags_27 },
    { 0xadcf4a9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcfcf4ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05cf4ae, SUC_lexicon_tags_15 },
    { 0x811cf4af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219cf4b5, SUC_lexicon_tags_12 },
    { 0x82dcf4b6, SUC_lexicon_tags_37 },
    { 0x863cf4b5, SUC_lexicon_tags_28 },
    { 0x85cf4b8, SUC_lexicon_tags_14 },
    { 0x167cf4b9, SUC_lexicon_tags_25 },
    { 0xd51cf4b9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afcf4c0, SUC_lexicon_tags_37 },
    { 0xdf3cf4c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567cf4c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bcf4c8, SUC_lexicon_tags_8 },
    { 0xa5dcf4c9, SUC_lexicon_tags_10 },
    { 0x553cf4ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbcf4cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbcf4d1, SUC_lexicon_tags_3 },
    { 0xa9fcf4d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551cf4d8, SUC_lexicon_tags_11 },
    { 0x6bdcf4d9, SUC_lexicon_tags_91 },
    { 0xe73cf4d8, SUC_lexicon_tags_16 },
    { 0xfd7cf4db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x16bcf4dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb5cf4df, SUC_lexicon_tags_2 },
    { 0x1a3cf4df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349cf4e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119cf4eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3e9cf4ed, SUC_lexicon_tags_16 },
    { 0xfb7cf4ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9cf4f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afcf4f4, SUC_lexicon_tags_15 },
    { 0x67dcf4f4, SUC_lexicon_tags_30 },
    { 0xbebcf4f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47bcf4f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987cf4ff, SUC_lexicon_tags_0 },
    { 0xadfcf500, SUC_lexicon_tags_0 },
    { 0x4c1cf501, SUC_lexicon_tags_37 },
    { 0xb07cf501, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93cf50a, SUC_lexicon_tags_9 },
    { 0x6c9cf50b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3cf50e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xda9cf510, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2a7cf512, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x487cf514, SUC_lexicon_tags_11 },
    { 0xac5cf515, SUC_lexicon_tags_1 },
    { 0x769cf516, SUC_lexicon_tags_6 },
    { 0x2d5cf517, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x3bcf519, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x917cf51b, SUC_lexicon_tags_21 },
    { 0x91cf51c, SUC_lexicon_tags_401 },
    { 0, NULL },
    { 0xdc3cf51e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6a9cf520, SUC_lexicon_tags_17 },
    { 0x241cf520, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed7cf523, SUC_lexicon_tags_68 },
    { 0x881cf523, SUC_lexicon_tags_25 },
    { 0xb99cf525, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x691cf528, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x507cf52a, SUC_lexicon_tags_25 },
    { 0xde9cf52a, SUC_lexicon_tags_17 },
    { 0x5abcf52c, SUC_lexicon_tags_45 },
    { 0xa97cf52c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7cf537, SUC_lexicon_tags_47 },
    { 0x3cf538, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45cf53d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bcf542, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x201cf545, SUC_lexicon_tags_107 },
    { 0xc0dcf546, SUC_lexicon_tags_39 },
    { 0x805cf547, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x731cf54a, SUC_lexicon_tags_0 },
    { 0xaa1cf54b, SUC_lexicon_tags_12 },
    { 0x5a3cf54b, SUC_lexicon_tags_17 },
    { 0x1f5cf54b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61cf553, SUC_lexicon_tags_25 },
    { 0x25dcf554, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43cf558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x835cf55a, SUC_lexicon_tags_37 },
    { 0xfafcf55b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x58bcf55d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1cf560, SUC_lexicon_tags_37 },
    { 0xc99cf561, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadcf56b, SUC_lexicon_tags_43 },
    { 0xbb1cf56c, SUC_lexicon_tags_7 },
    { 0x13bcf56d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23cf571, SUC_lexicon_tags_37 },
    { 0x85dcf572, SUC_lexicon_tags_63 },
    { 0xa59cf573, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1cf576, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebcf579, SUC_lexicon_tags_19 },
    { 0xdc1cf579, SUC_lexicon_tags_17 },
    { 0x7b9cf57b, SUC_lexicon_tags_6 },
    { 0x105cf57c, SUC_lexicon_tags_43 },
    { 0x99cf57a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abcf583, SUC_lexicon_tags_37 },
    { 0x329cf583, SUC_lexicon_tags_37 },
    { 0x1efcf585, SUC_lexicon_tags_12 },
    { 0x573cf586, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x633cf588, SUC_lexicon_tags_9 },
    { 0x86bcf589, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8bdcf58b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x85bcf58e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcf596, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x597cf598, SUC_lexicon_tags_51 },
    { 0x3cf599, SUC_lexicon_tags_15 },
    { 0xa9cf599, SUC_lexicon_tags_10 },
    { 0x83dcf59a, SUC_lexicon_tags_12 },
    { 0xaf3cf59c, SUC_lexicon_tags_56 },
    { 0x6cbcf59d, SUC_lexicon_tags_68 },
    { 0xaafcf59b, SUC_lexicon_tags_1 },
    { 0x40bcf598, SUC_lexicon_tags_51 },
    { 0x3abcf59b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x453cf5a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadcf5a5, SUC_lexicon_tags_15 },
    { 0x9edcf5a6, SUC_lexicon_tags_11 },
    { 0x641cf5a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x957cf5aa, SUC_lexicon_tags_26 },
    { 0x683cf5aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7cf5b4, SUC_lexicon_tags_2 },
    { 0xdbcf5b4, SUC_lexicon_tags_37 },
    { 0x7a7cf5b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499cf5bc, SUC_lexicon_tags_17 },
    { 0xd55cf5bd, SUC_lexicon_tags_2 },
    { 0x9c9cf5be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e5cf5c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399cf5c5, SUC_lexicon_tags_30 },
    { 0xd3bcf5c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21cf5ca, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5cf5cd, SUC_lexicon_tags_25 },
    { 0x8f1cf5cd, SUC_lexicon_tags_12 },
    { 0xc71cf5cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfe9cf5d1, SUC_lexicon_tags_25 },
    { 0xc07cf5d2, SUC_lexicon_tags_38 },
    { 0x975cf5d3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x331cf5d5, SUC_lexicon_tags_27 },
    { 0xd3bcf5d6, SUC_lexicon_tags_37 },
    { 0xcafcf5d7, SUC_lexicon_tags_25 },
    { 0x24dcf5d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbcf5e0, SUC_lexicon_tags_37 },
    { 0xb5cf5e0, SUC_lexicon_tags_12 },
    { 0xc77cf5e2, SUC_lexicon_tags_23 },
    { 0x595cf5e1, SUC_lexicon_tags_37 },
    { 0x943cf5e4, SUC_lexicon_tags_1 },
    { 0x44bcf5e4, SUC_lexicon_tags_47 },
    { 0xdc5cf5e1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15cf5e9, SUC_lexicon_tags_29 },
    { 0xaafcf5ea, SUC_lexicon_tags_87 },
    { 0xaffcf5eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fcf5ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75dcf5f0, SUC_lexicon_tags_28 },
    { 0xfa7cf5f1, SUC_lexicon_tags_25 },
    { 0x8bcf5f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7cf5f7, SUC_lexicon_tags_2 },
    { 0xdd7cf5f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x709cf5fa, SUC_lexicon_tags_15 },
    { 0x3cdcf5fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf89cf5fd, SUC_lexicon_tags_25 },
    { 0xc53cf5fe, SUC_lexicon_tags_12 },
    { 0x6a9cf5ff, SUC_lexicon_tags_25 },
    { 0x1efcf600, SUC_lexicon_tags_38 },
    { 0xe7dcf5fd, SUC_lexicon_tags_66 },
    { 0x235cf602, SUC_lexicon_tags_7 },
    { 0x373cf600, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cf60a, SUC_lexicon_tags_37 },
    { 0xd81cf60a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x82bcf60e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x919cf610, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd21cf612, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4bcf614, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9cf616, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x17dcf618, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7cf626, SUC_lexicon_tags_8 },
    { 0x335cf627, SUC_lexicon_tags_7 },
    { 0xac9cf628, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595cf62e, SUC_lexicon_tags_11 },
    { 0x753cf62e, SUC_lexicon_tags_16 },
    { 0x781cf630, SUC_lexicon_tags_37 },
    { 0xe01cf62f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bcf634, SUC_lexicon_tags_46 },
    { 0xe6dcf635, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1cf638, SUC_lexicon_tags_16 },
    { 0x689cf639, SUC_lexicon_tags_17 },
    { 0x727cf63a, SUC_lexicon_tags_49 },
    { 0x15fcf63a, SUC_lexicon_tags_45 },
    { 0xf03cf638, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bcf644, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x155cf646, SUC_lexicon_tags_15 },
    { 0xe57cf647, SUC_lexicon_tags_19 },
    { 0xc5dcf648, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35dcf64a, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305cf64f, SUC_lexicon_tags_19 },
    { 0x9a3cf64f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7cf656, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce5cf658, SUC_lexicon_tags_25 },
    { 0xec3cf658, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971cf660, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bcf664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7cf668, SUC_lexicon_tags_1 },
    { 0xab7cf669, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27cf66d, SUC_lexicon_tags_87 },
    { 0x2c1cf66e, SUC_lexicon_tags_49 },
    { 0x925cf66d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99cf672, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595cf676, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x18fcf678, SUC_lexicon_tags_37 },
    { 0xfb3cf679, SUC_lexicon_tags_11 },
    { 0x4f9cf67a, SUC_lexicon_tags_2 },
    { 0xc0fcf67b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fcf67e, SUC_lexicon_tags_45 },
    { 0x2cbcf67e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9cfcf681, SUC_lexicon_tags_25 },
    { 0xf85cf682, SUC_lexicon_tags_31 },
    { 0x239cf681, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bcf688, SUC_lexicon_tags_2 },
    { 0xd53cf689, SUC_lexicon_tags_23 },
    { 0xfc1cf68a, SUC_lexicon_tags_45 },
    { 0xfcf689, SUC_lexicon_tags_26 },
    { 0xdc1cf68c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05cf690, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bcf693, SUC_lexicon_tags_7 },
    { 0xf63cf693, SUC_lexicon_tags_17 },
    { 0xc3bcf694, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7cf699, SUC_lexicon_tags_60 },
    { 0xcf5cf69a, SUC_lexicon_tags_37 },
    { 0xbf1cf69b, SUC_lexicon_tags_27 },
    { 0x883cf69b, SUC_lexicon_tags_39 },
    { 0xd09cf69c, SUC_lexicon_tags_37 },
    { 0x64fcf69b, SUC_lexicon_tags_11 },
    { 0xc9bcf69a, SUC_lexicon_tags_2 },
    { 0x301cf6a0, SUC_lexicon_tags_0 },
    { 0xebfcf6a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3cf6a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9cf6a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7e7cf6ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x44bcf6ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe33cf6af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa99cf6b1, SUC_lexicon_tags_9 },
    { 0x813cf6b2, SUC_lexicon_tags_65 },
    { 0x58bcf6b3, SUC_lexicon_tags_25 },
    { 0x839cf6b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83cf6b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xff9cf6b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759cf6bd, SUC_lexicon_tags_12 },
    { 0xc33cf6be, SUC_lexicon_tags_40 },
    { 0x3e9cf6bf, SUC_lexicon_tags_14 },
    { 0x885cf6be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99cf6c7, SUC_lexicon_tags_37 },
    { 0x4c3cf6c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bcf6cd, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911cf6d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1cf6d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdcf6d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x627cf6db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x699cf6de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbcf6e3, SUC_lexicon_tags_25 },
    { 0x4a5cf6e4, SUC_lexicon_tags_11 },
    { 0x677cf6e5, SUC_lexicon_tags_7 },
    { 0xc5dcf6e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7cf6e9, SUC_lexicon_tags_12 },
    { 0x2dcf6ea, SUC_lexicon_tags_25 },
    { 0x9a5cf6eb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x315cf6ed, SUC_lexicon_tags_15 },
    { 0x667cf6ee, SUC_lexicon_tags_27 },
    { 0xc87cf6ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29cf6f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa59cf6f4, SUC_lexicon_tags_17 },
    { 0xd3fcf6f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d1cf6f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5cf6fb, SUC_lexicon_tags_68 },
    { 0x129cf6fc, SUC_lexicon_tags_92 },
    { 0x81fcf6fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5cf701, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adcf704, SUC_lexicon_tags_33 },
    { 0x28dcf705, SUC_lexicon_tags_7 },
    { 0xba3cf704, SUC_lexicon_tags_1 },
    { 0x4edcf706, SUC_lexicon_tags_46 },
    { 0xa1cf708, SUC_lexicon_tags_11 },
    { 0x513cf708, SUC_lexicon_tags_2 },
    { 0xc1bcf70a, SUC_lexicon_tags_1 },
    { 0x6edcf708, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fcf70f, SUC_lexicon_tags_17 },
    { 0xa07cf710, SUC_lexicon_tags_43 },
    { 0x435cf710, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67cf714, SUC_lexicon_tags_4 },
    { 0x589cf715, SUC_lexicon_tags_21 },
    { 0x953cf716, SUC_lexicon_tags_72 },
    { 0xbebcf716, SUC_lexicon_tags_28 },
    { 0xdcfcf717, SUC_lexicon_tags_0 },
    { 0xb83cf718, SUC_lexicon_tags_8 },
    { 0x849cf719, SUC_lexicon_tags_15 },
    { 0xfc9cf71b, SUC_lexicon_tags_0 },
    { 0xd37cf716, SUC_lexicon_tags_37 },
    { 0xc49cf71c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555cf721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335cf725, SUC_lexicon_tags_17 },
    { 0xb91cf726, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1cf73a, SUC_lexicon_tags_6 },
    { 0x7d7cf73a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c7cf73d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fcf74a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7cf74d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x211cf74f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fcf753, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdcf755, SUC_lexicon_tags_23 },
    { 0x3abcf756, SUC_lexicon_tags_38 },
    { 0x88bcf757, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdcf75b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07cf75e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xcc7cf760, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x893cf762, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535cf766, SUC_lexicon_tags_25 },
    { 0xeb3cf767, SUC_lexicon_tags_11 },
    { 0x3e5cf768, SUC_lexicon_tags_2 },
    { 0xb6dcf769, SUC_lexicon_tags_60 },
    { 0x2fbcf76a, SUC_lexicon_tags_26 },
    { 0xa19cf76b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaafcf76d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55cf770, SUC_lexicon_tags_37 },
    { 0xd8dcf771, SUC_lexicon_tags_12 },
    { 0xf09cf772, SUC_lexicon_tags_19 },
    { 0xdfbcf773, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749cf77e, SUC_lexicon_tags_33 },
    { 0x2c5cf77e, SUC_lexicon_tags_220 },
    { 0, NULL },
    { 0xcd1cf781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21cf785, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbbfcf787, SUC_lexicon_tags_56 },
    { 0xaabcf787, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fcf78b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7cf78e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dcf791, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b5cf793, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1bcf795, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fcf798, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7cf79b, SUC_lexicon_tags_2 },
    { 0xb69cf79c, SUC_lexicon_tags_15 },
    { 0x407cf79d, SUC_lexicon_tags_26 },
    { 0x3d5cf79e, SUC_lexicon_tags_19 },
    { 0x677cf79f, SUC_lexicon_tags_19 },
    { 0xb51cf7a0, SUC_lexicon_tags_63 },
    { 0x19dcf7a0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb7bcf7a3, SUC_lexicon_tags_29 },
    { 0x121cf7a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x461cf7a7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc9cf7a9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3cf7af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x607cf7b2, SUC_lexicon_tags_87 },
    { 0x37fcf7b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bcf7ba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769cf7c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x169cf7c3, SUC_lexicon_tags_33 },
    { 0xe0bcf7c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1cf7c8, SUC_lexicon_tags_13 },
    { 0x4fcf7c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe3fcf7cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x891cf7cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3dcf7cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bcf7d3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dcf7d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc23cf7d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bcf7dd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x367cf7e0, SUC_lexicon_tags_12 },
    { 0xf23cf7e0, SUC_lexicon_tags_12 },
    { 0x75bcf7e0, SUC_lexicon_tags_38 },
    { 0x165cf7e0, SUC_lexicon_tags_32 },
    { 0x1abcf7e2, SUC_lexicon_tags_33 },
    { 0xa2fcf7e5, SUC_lexicon_tags_8 },
    { 0x5d7cf7e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613cf7f0, SUC_lexicon_tags_37 },
    { 0xf63cf7f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593cf7f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5cf7fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279cf806, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1cf80f, SUC_lexicon_tags_33 },
    { 0x9b3cf80f, SUC_lexicon_tags_26 },
    { 0x97cf811, SUC_lexicon_tags_79 },
    { 0x4f9cf812, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe07cf814, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c3cf816, SUC_lexicon_tags_37 },
    { 0xeadcf816, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x923cf81a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf3fcf81c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe5dcf81e, SUC_lexicon_tags_49 },
    { 0x237cf81e, SUC_lexicon_tags_8 },
    { 0x667cf81e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc7cf822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf61cf824, SUC_lexicon_tags_23 },
    { 0x17dcf825, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfcf829, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259cf837, SUC_lexicon_tags_2 },
    { 0x18bcf838, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07cf83b, SUC_lexicon_tags_63 },
    { 0xa83cf83c, SUC_lexicon_tags_2 },
    { 0x4edcf83d, SUC_lexicon_tags_35 },
    { 0xb39cf83e, SUC_lexicon_tags_7 },
    { 0x44fcf83f, SUC_lexicon_tags_16 },
    { 0xed5cf83b, SUC_lexicon_tags_77 },
    { 0xe2fcf841, SUC_lexicon_tags_27 },
    { 0xfefcf83c, SUC_lexicon_tags_57 },
    { 0xe21cf840, SUC_lexicon_tags_7 },
    { 0xc73cf844, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5cf847, SUC_lexicon_tags_11 },
    { 0x473cf848, SUC_lexicon_tags_63 },
    { 0xe9bcf847, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29dcf84b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cf84f, SUC_lexicon_tags_12 },
    { 0xcbdcf850, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89cf852, SUC_lexicon_tags_17 },
    { 0x7d3cf853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x565cf855, SUC_lexicon_tags_29 },
    { 0x889cf856, SUC_lexicon_tags_17 },
    { 0x51dcf857, SUC_lexicon_tags_0 },
    { 0xae9cf858, SUC_lexicon_tags_47 },
    { 0x3f1cf859, SUC_lexicon_tags_2 },
    { 0x355cf85a, SUC_lexicon_tags_1 },
    { 0xe7dcf85b, SUC_lexicon_tags_38 },
    { 0xa1cf85c, SUC_lexicon_tags_45 },
    { 0xbdcf85d, SUC_lexicon_tags_2 },
    { 0x859cf85e, SUC_lexicon_tags_3 },
    { 0xd03cf85d, SUC_lexicon_tags_10 },
    { 0x441cf860, SUC_lexicon_tags_15 },
    { 0xf1bcf85c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47cf86a, SUC_lexicon_tags_47 },
    { 0xa4fcf86a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109cf871, SUC_lexicon_tags_19 },
    { 0x1b3cf871, SUC_lexicon_tags_17 },
    { 0xfd7cf873, SUC_lexicon_tags_31 },
    { 0x507cf874, SUC_lexicon_tags_6 },
    { 0x747cf875, SUC_lexicon_tags_9 },
    { 0x221cf874, SUC_lexicon_tags_27 },
    { 0xd2bcf874, SUC_lexicon_tags_15 },
    { 0x483cf878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x81cf87b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fcf883, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdc5cf885, SUC_lexicon_tags_25 },
    { 0xf43cf886, SUC_lexicon_tags_17 },
    { 0x6f9cf886, SUC_lexicon_tags_2 },
    { 0x34dcf887, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9cf898, SUC_lexicon_tags_8 },
    { 0x67bcf899, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6e3cf89b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x137cf89e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c5cf8a0, SUC_lexicon_tags_59 },
    { 0x4b7cf8a1, SUC_lexicon_tags_22 },
    { 0xf83cf8a1, SUC_lexicon_tags_64 },
    { 0x877cf8a2, SUC_lexicon_tags_7 },
    { 0x631cf8a2, SUC_lexicon_tags_66 },
    { 0x691cf8a5, SUC_lexicon_tags_7 },
    { 0xe6fcf8a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x54dcf8a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x241cf8aa, SUC_lexicon_tags_43 },
    { 0x1fbcf8ab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbcf8af, SUC_lexicon_tags_55 },
    { 0x6a5cf8af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03cf8b9, SUC_lexicon_tags_11 },
    { 0x93dcf8b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x199cf8bc, SUC_lexicon_tags_19 },
    { 0xc8dcf8bc, SUC_lexicon_tags_47 },
    { 0x67cf8bd, SUC_lexicon_tags_45 },
    { 0x35fcf8bf, SUC_lexicon_tags_1 },
    { 0x5dcf8c0, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55cf8c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3cf8c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ddcf8ca, SUC_lexicon_tags_22 },
    { 0x56dcf8cb, SUC_lexicon_tags_7 },
    { 0x739cf8cc, SUC_lexicon_tags_7 },
    { 0x89bcf8cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda5cf8cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc91cf8d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x685cf8d4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdcf8d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925cf8db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907cf8df, SUC_lexicon_tags_4 },
    { 0x5e5cf8e0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x735cf8e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105cf8e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fcf8ed, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261cf8f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc05cf8f3, SUC_lexicon_tags_23 },
    { 0xe9bcf8f3, SUC_lexicon_tags_11 },
    { 0x263cf8f5, SUC_lexicon_tags_47 },
    { 0x175cf8f6, SUC_lexicon_tags_2 },
    { 0x973cf8f5, SUC_lexicon_tags_0 },
    { 0x235cf8f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67cf8fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x579cf8ff, SUC_lexicon_tags_0 },
    { 0x465cf900, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xf2dcf902, SUC_lexicon_tags_31 },
    { 0x23fcf903, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19dcf90b, SUC_lexicon_tags_40 },
    { 0xa71cf90b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92fcf90e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9cf913, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x649cf915, SUC_lexicon_tags_37 },
    { 0xf37cf916, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5cf91a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa93cf91c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7cf91f, SUC_lexicon_tags_33 },
    { 0xfb3cf91f, SUC_lexicon_tags_37 },
    { 0x2f3cf920, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3cf926, SUC_lexicon_tags_25 },
    { 0xa65cf926, SUC_lexicon_tags_33 },
    { 0x137cf928, SUC_lexicon_tags_7 },
    { 0x787cf929, SUC_lexicon_tags_138 },
    { 0x245cf92a, SUC_lexicon_tags_17 },
    { 0x48bcf92b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27cf933, SUC_lexicon_tags_37 },
    { 0x125cf934, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25cf937, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdcf93a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbcf93d, SUC_lexicon_tags_13 },
    { 0xc45cf93e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbcf942, SUC_lexicon_tags_60 },
    { 0xcf5cf943, SUC_lexicon_tags_17 },
    { 0x715cf944, SUC_lexicon_tags_11 },
    { 0xc57cf942, SUC_lexicon_tags_63 },
    { 0x3fcf946, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x245cf948, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33cf94c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa23cf94e, SUC_lexicon_tags_27 },
    { 0xe5cf94f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45cf952, SUC_lexicon_tags_55 },
    { 0x2dcf953, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25cf958, SUC_lexicon_tags_31 },
    { 0x76bcf959, SUC_lexicon_tags_25 },
    { 0x579cf95a, SUC_lexicon_tags_25 },
    { 0x799cf95b, SUC_lexicon_tags_2 },
    { 0x203cf95c, SUC_lexicon_tags_48 },
    { 0x44fcf95d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x581cf960, SUC_lexicon_tags_45 },
    { 0xe59cf961, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85dcf966, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x527cf969, SUC_lexicon_tags_1 },
    { 0xe07cf96a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22dcf96c, SUC_lexicon_tags_17 },
    { 0x843cf96d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc8fcf96f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209cf976, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9cf97a, SUC_lexicon_tags_14 },
    { 0xd81cf97a, SUC_lexicon_tags_226 },
    { 0x1fdcf97b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdcf97f, SUC_lexicon_tags_29 },
    { 0xacdcf97f, SUC_lexicon_tags_25 },
    { 0xcafcf981, SUC_lexicon_tags_48 },
    { 0xe4fcf97f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x84fcf984, SUC_lexicon_tags_23 },
    { 0x4ffcf985, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23cf989, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3cf98f, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x1bfcf991, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x11dcf993, SUC_lexicon_tags_30 },
    { 0xb57cf994, SUC_lexicon_tags_14 },
    { 0x279cf994, SUC_lexicon_tags_12 },
    { 0xfcf993, SUC_lexicon_tags_19 },
    { 0x9adcf997, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3cf99e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc8dcf9a0, SUC_lexicon_tags_10 },
    { 0xb05cf9a1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63cf9ab, SUC_lexicon_tags_11 },
    { 0xcc3cf9ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241cf9b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafcf9b8, SUC_lexicon_tags_27 },
    { 0x3e1cf9b9, SUC_lexicon_tags_17 },
    { 0xc89cf9b8, SUC_lexicon_tags_2 },
    { 0x5dbcf9b8, SUC_lexicon_tags_45 },
    { 0xcedcf9b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x989cf9be, SUC_lexicon_tags_1 },
    { 0x9f3cf9be, SUC_lexicon_tags_25 },
    { 0xbbdcf9be, SUC_lexicon_tags_31 },
    { 0x3e7cf9bf, SUC_lexicon_tags_25 },
    { 0xd63cf9c2, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9cf9c6, SUC_lexicon_tags_1 },
    { 0xaa7cf9c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dcf9cc, SUC_lexicon_tags_3 },
    { 0x589cf9cd, SUC_lexicon_tags_47 },
    { 0x389cf9ce, SUC_lexicon_tags_19 },
    { 0xa87cf9cf, SUC_lexicon_tags_17 },
    { 0xfa1cf9ce, SUC_lexicon_tags_6 },
    { 0xc91cf9d0, SUC_lexicon_tags_63 },
    { 0xa6fcf9d1, SUC_lexicon_tags_12 },
    { 0x1cfcf9d3, SUC_lexicon_tags_11 },
    { 0xf0dcf9d1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443cf9db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6b1cf9dd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913cf9e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfcf9e4, SUC_lexicon_tags_12 },
    { 0x181cf9e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x953cf9e7, SUC_lexicon_tags_37 },
    { 0x5fdcf9e8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x645cf9ea, SUC_lexicon_tags_1 },
    { 0xc31cf9ea, SUC_lexicon_tags_65 },
    { 0xc5cf9ea, SUC_lexicon_tags_30 },
    { 0x70fcf9ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3cf9f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53cf9f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x269cf9fa, SUC_lexicon_tags_1 },
    { 0x38fcf9fb, SUC_lexicon_tags_30 },
    { 0xc1dcf9fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dcfa00, SUC_lexicon_tags_32 },
    { 0xf73cfa00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317cfa05, SUC_lexicon_tags_32 },
    { 0x861cfa05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fcfa0a, SUC_lexicon_tags_28 },
    { 0x8d3cfa0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fcfa0e, SUC_lexicon_tags_37 },
    { 0xad3cfa0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfcfa16, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723cfa21, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc23cfa23, SUC_lexicon_tags_7 },
    { 0x87cfa24, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdcfa27, SUC_lexicon_tags_68 },
    { 0x587cfa28, SUC_lexicon_tags_42 },
    { 0xf33cfa27, SUC_lexicon_tags_49 },
    { 0xb3bcfa2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87cfa2f, SUC_lexicon_tags_12 },
    { 0x3ffcfa30, SUC_lexicon_tags_12 },
    { 0x1a1cfa31, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97cfa34, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x55bcfa36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235cfa3e, SUC_lexicon_tags_37 },
    { 0xfefcfa3e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x745cfa41, SUC_lexicon_tags_17 },
    { 0x4f1cfa42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afcfa45, SUC_lexicon_tags_37 },
    { 0xfcfcfa45, SUC_lexicon_tags_17 },
    { 0x1f3cfa47, SUC_lexicon_tags_11 },
    { 0xac1cfa46, SUC_lexicon_tags_11 },
    { 0xfd1cfa49, SUC_lexicon_tags_12 },
    { 0x935cfa48, SUC_lexicon_tags_21 },
    { 0xd9dcfa49, SUC_lexicon_tags_45 },
    { 0x145cfa4a, SUC_lexicon_tags_39 },
    { 0xad1cfa4d, SUC_lexicon_tags_11 },
    { 0xe23cfa46, SUC_lexicon_tags_7 },
    { 0x167cfa4e, SUC_lexicon_tags_65 },
    { 0xf77cfa50, SUC_lexicon_tags_25 },
    { 0xec1cfa51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x169cfa53, SUC_lexicon_tags_29 },
    { 0x6b9cfa54, SUC_lexicon_tags_103 },
    { 0xc91cfa55, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0xf4bcfa57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a7cfa59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3cfa5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3e9cfa5e, SUC_lexicon_tags_12 },
    { 0x9ddcfa5f, SUC_lexicon_tags_7 },
    { 0xbddcfa5f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x47cfa62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x435cfa65, SUC_lexicon_tags_56 },
    { 0xc11cfa66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x899cfa68, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3cfa71, SUC_lexicon_tags_37 },
    { 0xf95cfa72, SUC_lexicon_tags_28 },
    { 0x7cbcfa73, SUC_lexicon_tags_11 },
    { 0xfedcfa74, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dcfa81, SUC_lexicon_tags_1 },
    { 0x829cfa82, SUC_lexicon_tags_49 },
    { 0x603cfa83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfcfcfa85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe43cfa87, SUC_lexicon_tags_11 },
    { 0x3e5cfa87, SUC_lexicon_tags_63 },
    { 0x3adcfa89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21cfa99, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9cfa9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3cfaa0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdfbcfaa2, SUC_lexicon_tags_33 },
    { 0x413cfaa3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x847cfaa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x655cfaa8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17fcfaaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9cfaae, SUC_lexicon_tags_1 },
    { 0xfa7cfaaf, SUC_lexicon_tags_10 },
    { 0x6f7cfaaf, SUC_lexicon_tags_0 },
    { 0xf7fcfab0, SUC_lexicon_tags_25 },
    { 0x8b3cfab1, SUC_lexicon_tags_49 },
    { 0x1bdcfaaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c7cfab5, SUC_lexicon_tags_38 },
    { 0xa9fcfab6, SUC_lexicon_tags_15 },
    { 0x577cfab6, SUC_lexicon_tags_15 },
    { 0xbc3cfab8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc05cfaba, SUC_lexicon_tags_17 },
    { 0xf77cfaba, SUC_lexicon_tags_37 },
    { 0x35bcfabc, SUC_lexicon_tags_49 },
    { 0xbbbcfabd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec7cfabf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dcfac5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbffcfac7, SUC_lexicon_tags_1 },
    { 0x4c1cfac7, SUC_lexicon_tags_31 },
    { 0xb3dcfac8, SUC_lexicon_tags_2 },
    { 0x90fcfaca, SUC_lexicon_tags_30 },
    { 0xa35cfacb, SUC_lexicon_tags_65 },
    { 0xa85cfacb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e7cface, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x88bcfad0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x579cfad3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdcfad8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x82fcfada, SUC_lexicon_tags_21 },
    { 0xfb5cfadb, SUC_lexicon_tags_30 },
    { 0x5b7cfadb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x48bcfade, SUC_lexicon_tags_11 },
    { 0xb47cfadf, SUC_lexicon_tags_11 },
    { 0x453cfae0, SUC_lexicon_tags_3 },
    { 0x695cfae0, SUC_lexicon_tags_23 },
    { 0xddcfae2, SUC_lexicon_tags_47 },
    { 0x5b1cfae3, SUC_lexicon_tags_47 },
    { 0xbc5cfae4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bcfae7, SUC_lexicon_tags_7 },
    { 0x2ffcfae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfcfaeb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x323cfaee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01cfaf5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fcfaff, SUC_lexicon_tags_2 },
    { 0xa39cfaff, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1cfb03, SUC_lexicon_tags_46 },
    { 0x25dcfb04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x169cfb06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x659cfb08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639cfb0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153cfb11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7cfb14, SUC_lexicon_tags_71 },
    { 0xb47cfb15, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfb1f, SUC_lexicon_tags_29 },
    { 0xeb9cfb20, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dcfb24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73bcfb26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x593cfb28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abcfb2f, SUC_lexicon_tags_23 },
    { 0x969cfb30, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc9bcfb32, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bcfb38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903cfb3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3cfb40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x795cfb42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0bcfb44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacfcfb46, SUC_lexicon_tags_46 },
    { 0xad7cfb47, SUC_lexicon_tags_17 },
    { 0xe35cfb48, SUC_lexicon_tags_0 },
    { 0x72dcfb49, SUC_lexicon_tags_25 },
    { 0x483cfb4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63cfb4d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1cfb53, SUC_lexicon_tags_4 },
    { 0x987cfb54, SUC_lexicon_tags_65 },
    { 0xeabcfb55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x953cfb58, SUC_lexicon_tags_17 },
    { 0x97bcfb59, SUC_lexicon_tags_25 },
    { 0xb05cfb59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x309cfb5c, SUC_lexicon_tags_82 },
    { 0x773cfb5d, SUC_lexicon_tags_8 },
    { 0x281cfb5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x693cfb60, SUC_lexicon_tags_33 },
    { 0x427cfb61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1cfb64, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7cfb71, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9cfb77, SUC_lexicon_tags_351 },
    { 0x937cfb77, SUC_lexicon_tags_66 },
    { 0xa75cfb79, SUC_lexicon_tags_31 },
    { 0x35bcfb79, SUC_lexicon_tags_12 },
    { 0xcc3cfb7b, SUC_lexicon_tags_26 },
    { 0x4ffcfb7b, SUC_lexicon_tags_26 },
    { 0x8d7cfb7d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x61dcfb80, SUC_lexicon_tags_37 },
    { 0x3d1cfb81, SUC_lexicon_tags_37 },
    { 0xb31cfb82, SUC_lexicon_tags_9 },
    { 0x1e3cfb82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xae9cfb85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dcfb8a, SUC_lexicon_tags_45 },
    { 0x4cfcfb8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327cfb90, SUC_lexicon_tags_37 },
    { 0xc37cfb91, SUC_lexicon_tags_37 },
    { 0x7bfcfb91, SUC_lexicon_tags_17 },
    { 0x32dcfb90, SUC_lexicon_tags_34 },
    { 0xf71cfb94, SUC_lexicon_tags_9 },
    { 0xa19cfb94, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5cfb98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf49cfb9a, SUC_lexicon_tags_30 },
    { 0xb77cfb9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x20bcfb9d, SUC_lexicon_tags_7 },
    { 0xba3cfb9e, SUC_lexicon_tags_6 },
    { 0xed5cfb9d, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x6bbcfba1, SUC_lexicon_tags_12 },
    { 0xabdcfba2, SUC_lexicon_tags_38 },
    { 0xdf5cfba3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1cfba6, SUC_lexicon_tags_37 },
    { 0x353cfba6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01cfbb3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x749cfbb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdcfbbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77cfbc3, SUC_lexicon_tags_45 },
    { 0x5dcfbc4, SUC_lexicon_tags_17 },
    { 0xd87cfbc5, SUC_lexicon_tags_19 },
    { 0x2cfcfbc6, SUC_lexicon_tags_21 },
    { 0xcddcfbc7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xd6bcfbc9, SUC_lexicon_tags_30 },
    { 0x505cfbc9, SUC_lexicon_tags_10 },
    { 0xcf3cfbcb, SUC_lexicon_tags_3 },
    { 0x853cfbcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5b3cfbce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1cfbd7, SUC_lexicon_tags_12 },
    { 0xa05cfbd8, SUC_lexicon_tags_11 },
    { 0xcf5cfbd7, SUC_lexicon_tags_11 },
    { 0x5cbcfbd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197cfbde, SUC_lexicon_tags_8 },
    { 0x5e3cfbdf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3cfbea, SUC_lexicon_tags_33 },
    { 0xcfcfbea, SUC_lexicon_tags_31 },
    { 0xc5fcfbec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x915cfbef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27cfbf2, SUC_lexicon_tags_7 },
    { 0x1ddcfbf3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bcfbf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35cfbf9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xff5cfbfb, SUC_lexicon_tags_87 },
    { 0x739cfbfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfcfc00, SUC_lexicon_tags_37 },
    { 0xf93cfc00, SUC_lexicon_tags_1 },
    { 0xfcfc02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78bcfc04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5cfc0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd5cfc0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x70fcfc10, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x903cfc12, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bcfc15, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701cfc19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf61cfc1b, SUC_lexicon_tags_2 },
    { 0x111cfc1c, SUC_lexicon_tags_38 },
    { 0x963cfc1d, SUC_lexicon_tags_7 },
    { 0xcdfcfc1e, SUC_lexicon_tags_0 },
    { 0x3d3cfc1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351cfc24, SUC_lexicon_tags_7 },
    { 0x483cfc25, SUC_lexicon_tags_7 },
    { 0x98dcfc26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771cfc2a, SUC_lexicon_tags_15 },
    { 0xa3bcfc2b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x8bcfc2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bcfc33, SUC_lexicon_tags_21 },
    { 0x7bfcfc34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23cfc3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621cfc40, SUC_lexicon_tags_12 },
    { 0x893cfc41, SUC_lexicon_tags_47 },
    { 0xac3cfc41, SUC_lexicon_tags_55 },
    { 0xd89cfc42, SUC_lexicon_tags_12 },
    { 0x251cfc44, SUC_lexicon_tags_17 },
    { 0x197cfc40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x355cfc47, SUC_lexicon_tags_15 },
    { 0xd01cfc47, SUC_lexicon_tags_29 },
    { 0x271cfc49, SUC_lexicon_tags_25 },
    { 0xfbbcfc48, SUC_lexicon_tags_7 },
    { 0xf37cfc47, SUC_lexicon_tags_17 },
    { 0x85dcfc4c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x695cfc4e, SUC_lexicon_tags_25 },
    { 0xd27cfc4e, SUC_lexicon_tags_46 },
    { 0x417cfc50, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fcfc53, SUC_lexicon_tags_15 },
    { 0xdf5cfc54, SUC_lexicon_tags_47 },
    { 0xa2bcfc55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x117cfc57, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x765cfc59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7c1cfc5b, SUC_lexicon_tags_32 },
    { 0x749cfc5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93cfc60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec3cfc62, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2fcfc64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x345cfc66, SUC_lexicon_tags_1 },
    { 0xd79cfc66, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7cfc74, SUC_lexicon_tags_43 },
    { 0x5b7cfc75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae9cfc77, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x15dcfc79, SUC_lexicon_tags_2 },
    { 0xebdcfc7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555cfc85, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bcfc88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fcfc92, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x243cfc95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74dcfc99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd3cfc9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01cfca1, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5cfcac, SUC_lexicon_tags_28 },
    { 0x115cfcac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dcfcb0, SUC_lexicon_tags_11 },
    { 0xdc9cfcb1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bcfcb4, SUC_lexicon_tags_1047 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7cfcb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fcfcbb, SUC_lexicon_tags_160 },
    { 0xa29cfcbc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x129cfcbe, SUC_lexicon_tags_11 },
    { 0xa3cfcbf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fbcfcc1, SUC_lexicon_tags_29 },
    { 0xa93cfcc2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1cfcca, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97cfcce, SUC_lexicon_tags_15 },
    { 0xeadcfccf, SUC_lexicon_tags_15 },
    { 0x79dcfcd0, SUC_lexicon_tags_1 },
    { 0x1a5cfcd1, SUC_lexicon_tags_60 },
    { 0x877cfcd0, SUC_lexicon_tags_12 },
    { 0xa7cfcd3, SUC_lexicon_tags_37 },
    { 0x6bdcfcd4, SUC_lexicon_tags_25 },
    { 0x20fcfcd5, SUC_lexicon_tags_3 },
    { 0x7efcfcd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x96bcfcd9, SUC_lexicon_tags_2 },
    { 0x10fcfcd9, SUC_lexicon_tags_43 },
    { 0x789cfcda, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x903cfcdd, SUC_lexicon_tags_28 },
    { 0x301cfcdd, SUC_lexicon_tags_12 },
    { 0xcafcfcdf, SUC_lexicon_tags_25 },
    { 0xe35cfce0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x747cfce3, SUC_lexicon_tags_48 },
    { 0x3c3cfce3, SUC_lexicon_tags_84 },
    { 0x28dcfce5, SUC_lexicon_tags_2 },
    { 0x4b3cfce6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dcfce9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf3cfceb, SUC_lexicon_tags_0 },
    { 0x7f1cfceb, SUC_lexicon_tags_48 },
    { 0x607cfced, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655cfcf1, SUC_lexicon_tags_68 },
    { 0x8d5cfcf2, SUC_lexicon_tags_7 },
    { 0x851cfcf1, SUC_lexicon_tags_11 },
    { 0xe7dcfcf3, SUC_lexicon_tags_11 },
    { 0x1bfcfcf5, SUC_lexicon_tags_16 },
    { 0xed1cfcf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433cfcfa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43cfcff, SUC_lexicon_tags_25 },
    { 0x2b3cfd00, SUC_lexicon_tags_15 },
    { 0x893cfcff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd33cfd03, SUC_lexicon_tags_26 },
    { 0x4d3cfd04, SUC_lexicon_tags_47 },
    { 0x889cfd05, SUC_lexicon_tags_11 },
    { 0xd3dcfd03, SUC_lexicon_tags_48 },
    { 0x6e3cfd07, SUC_lexicon_tags_27 },
    { 0x8b7cfd08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333cfd0c, SUC_lexicon_tags_23 },
    { 0x56dcfd0d, SUC_lexicon_tags_15 },
    { 0x42fcfd0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3cfd13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd41cfd15, SUC_lexicon_tags_19 },
    { 0x515cfd15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1dcfd18, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fcfd1d, SUC_lexicon_tags_12 },
    { 0xb1fcfd1e, SUC_lexicon_tags_47 },
    { 0x47cfd1f, SUC_lexicon_tags_25 },
    { 0x625cfd1f, SUC_lexicon_tags_37 },
    { 0x341cfd1e, SUC_lexicon_tags_17 },
    { 0x37dcfd20, SUC_lexicon_tags_33 },
    { 0xba3cfd23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97cfd29, SUC_lexicon_tags_30 },
    { 0xc7fcfd29, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35cfd2e, SUC_lexicon_tags_7 },
    { 0x3b5cfd2f, SUC_lexicon_tags_4 },
    { 0xab1cfd2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fcfd35, SUC_lexicon_tags_2 },
    { 0xf49cfd36, SUC_lexicon_tags_27 },
    { 0x61bcfd35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedcfd3f, SUC_lexicon_tags_17 },
    { 0xb11cfd3f, SUC_lexicon_tags_1 },
    { 0xbd3cfd41, SUC_lexicon_tags_45 },
    { 0x5d5cfd42, SUC_lexicon_tags_10 },
    { 0x70dcfd42, SUC_lexicon_tags_26 },
    { 0xf91cfd44, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3cfd47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955cfd4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4fbcfd4e, SUC_lexicon_tags_25 },
    { 0xdcdcfd4f, SUC_lexicon_tags_49 },
    { 0x1d5cfd4f, SUC_lexicon_tags_17 },
    { 0xa1cfd51, SUC_lexicon_tags_12 },
    { 0x55bcfd52, SUC_lexicon_tags_32 },
    { 0x845cfd53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdcfd56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf7cfd58, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3cfd5c, SUC_lexicon_tags_11 },
    { 0xeb9cfd5d, SUC_lexicon_tags_46 },
    { 0x3b5cfd5e, SUC_lexicon_tags_1 },
    { 0x4bbcfd5c, SUC_lexicon_tags_30 },
    { 0x85bcfd60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x22bcfd62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723cfd66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7cfd6a, SUC_lexicon_tags_98 },
    { 0xd1bcfd6b, SUC_lexicon_tags_9 },
    { 0x89cfd6a, SUC_lexicon_tags_27 },
    { 0xfefcfd6d, SUC_lexicon_tags_3 },
    { 0xd13cfd6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x923cfd70, SUC_lexicon_tags_77 },
    { 0x9f5cfd70, SUC_lexicon_tags_45 },
    { 0x4f7cfd72, SUC_lexicon_tags_11 },
    { 0x90bcfd72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81cfd7b, SUC_lexicon_tags_25 },
    { 0x2fbcfd7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x379cfd7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7cfd82, SUC_lexicon_tags_15 },
    { 0x67fcfd83, SUC_lexicon_tags_37 },
    { 0x93dcfd84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95cfd87, SUC_lexicon_tags_26 },
    { 0xca9cfd87, SUC_lexicon_tags_17 },
    { 0xea7cfd89, SUC_lexicon_tags_10 },
    { 0xc5dcfd8a, SUC_lexicon_tags_201 },
    { 0x2d9cfd8b, SUC_lexicon_tags_4 },
    { 0x87bcfd8b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15cfd8f, SUC_lexicon_tags_2 },
    { 0xccfcfd8f, SUC_lexicon_tags_10 },
    { 0xcd7cfd90, SUC_lexicon_tags_17 },
    { 0x1c1cfd92, SUC_lexicon_tags_43 },
    { 0x72fcfd92, SUC_lexicon_tags_38 },
    { 0xb19cfd94, SUC_lexicon_tags_15 },
    { 0x9afcfd94, SUC_lexicon_tags_1 },
    { 0x683cfd93, SUC_lexicon_tags_25 },
    { 0xb69cfd8f, SUC_lexicon_tags_45 },
    { 0xd73cfd91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881cfd9e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbcfda9, SUC_lexicon_tags_66 },
    { 0xb85cfda9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ffcfdac, SUC_lexicon_tags_23 },
    { 0x58bcfdad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41fcfdaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x223cfdb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1b5cfdb3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x203cfdb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x693cfdb9, SUC_lexicon_tags_33 },
    { 0x1fcfdb9, SUC_lexicon_tags_33 },
    { 0xe51cfdbb, SUC_lexicon_tags_33 },
    { 0x2b7cfdbc, SUC_lexicon_tags_15 },
    { 0xb17cfdbc, SUC_lexicon_tags_112 },
    { 0xcf3cfdbe, SUC_lexicon_tags_25 },
    { 0xe9dcfdbf, SUC_lexicon_tags_0 },
    { 0xb0dcfdbe, SUC_lexicon_tags_17 },
    { 0x8e9cfdc1, SUC_lexicon_tags_2 },
    { 0xc75cfdc2, SUC_lexicon_tags_32 },
    { 0x533cfdc2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d5cfdc5, SUC_lexicon_tags_6 },
    { 0x711cfdc5, SUC_lexicon_tags_25 },
    { 0x121cfdc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfcfdca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd97cfdcc, SUC_lexicon_tags_63 },
    { 0xdd7cfdcd, SUC_lexicon_tags_23 },
    { 0x5fbcfdcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bcfdd1, SUC_lexicon_tags_60 },
    { 0x751cfdd2, SUC_lexicon_tags_55 },
    { 0x30bcfdd2, SUC_lexicon_tags_43 },
    { 0x179cfdd1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x701cfdd6, SUC_lexicon_tags_27 },
    { 0x7afcfdd6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfcfdda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x293cfddd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x4dbcfddf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5cfde3, SUC_lexicon_tags_11 },
    { 0xda9cfde4, SUC_lexicon_tags_15 },
    { 0x59fcfde5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57cfde8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5dbcfdea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723cfdef, SUC_lexicon_tags_12 },
    { 0xf93cfdef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541cfdf4, SUC_lexicon_tags_45 },
    { 0x72fcfdf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89cfdfa, SUC_lexicon_tags_1 },
    { 0xe35cfdfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67cfe04, SUC_lexicon_tags_27 },
    { 0x4c5cfe04, SUC_lexicon_tags_15 },
    { 0x557cfe06, SUC_lexicon_tags_34 },
    { 0x907cfe04, SUC_lexicon_tags_43 },
    { 0xde9cfe05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5cfe0e, SUC_lexicon_tags_7 },
    { 0xef3cfe0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1cfe14, SUC_lexicon_tags_47 },
    { 0xe25cfe15, SUC_lexicon_tags_2 },
    { 0x471cfe16, SUC_lexicon_tags_16 },
    { 0xbf5cfe17, SUC_lexicon_tags_17 },
    { 0x109cfe16, SUC_lexicon_tags_33 },
    { 0x139cfe17, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x433cfe1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105cfe21, SUC_lexicon_tags_1 },
    { 0xadcfe21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86bcfe24, SUC_lexicon_tags_93 },
    { 0x1e9cfe25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9cfe29, SUC_lexicon_tags_63 },
    { 0x101cfe2a, SUC_lexicon_tags_48 },
    { 0x517cfe2a, SUC_lexicon_tags_37 },
    { 0xb37cfe2b, SUC_lexicon_tags_17 },
    { 0xbe7cfe2d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41cfe32, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x605cfe35, SUC_lexicon_tags_17 },
    { 0xfc3cfe36, SUC_lexicon_tags_77 },
    { 0xe61cfe37, SUC_lexicon_tags_26 },
    { 0x63dcfe38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe8fcfe3a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x305cfe3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c5cfe3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x83dcfe40, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x71dcfe42, SUC_lexicon_tags_17 },
    { 0x889cfe43, SUC_lexicon_tags_11 },
    { 0x9bcfe44, SUC_lexicon_tags_17 },
    { 0x2c1cfe45, SUC_lexicon_tags_25 },
    { 0x111cfe46, SUC_lexicon_tags_61 },
    { 0x715cfe44, SUC_lexicon_tags_16 },
    { 0x413cfe45, SUC_lexicon_tags_45 },
    { 0x84dcfe49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fcfe52, SUC_lexicon_tags_1 },
    { 0xe15cfe52, SUC_lexicon_tags_9 },
    { 0x6c9cfe53, SUC_lexicon_tags_49 },
    { 0xa11cfe55, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xf49cfe57, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83cfe5a, SUC_lexicon_tags_5 },
    { 0xdb3cfe5b, SUC_lexicon_tags_17 },
    { 0xce9cfe5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb41cfe5e, SUC_lexicon_tags_13 },
    { 0x8a7cfe5f, SUC_lexicon_tags_37 },
    { 0x7e1cfe60, SUC_lexicon_tags_43 },
    { 0xb71cfe60, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0xa29cfe63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabcfe6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3cfe72, SUC_lexicon_tags_23 },
    { 0x4d1cfe73, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1cfe77, SUC_lexicon_tags_25 },
    { 0x8f1cfe78, SUC_lexicon_tags_2 },
    { 0xed1cfe79, SUC_lexicon_tags_37 },
    { 0x16bcfe77, SUC_lexicon_tags_46 },
    { 0x1bcfe77, SUC_lexicon_tags_29 },
    { 0xcc1cfe78, SUC_lexicon_tags_14 },
    { 0x23cfe7d, SUC_lexicon_tags_63 },
    { 0x273cfe7e, SUC_lexicon_tags_46 },
    { 0x917cfe7e, SUC_lexicon_tags_4 },
    { 0x405cfe80, SUC_lexicon_tags_26 },
    { 0x9abcfe81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x281cfe83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fcfe86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x519cfe89, SUC_lexicon_tags_9 },
    { 0x839cfe8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bcfe8c, SUC_lexicon_tags_7 },
    { 0xa59cfe8d, SUC_lexicon_tags_0 },
    { 0x1efcfe8c, SUC_lexicon_tags_43 },
    { 0xc13cfe8d, SUC_lexicon_tags_47 },
    { 0x169cfe8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01cfe93, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885cfe98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4d7cfe9a, SUC_lexicon_tags_11 },
    { 0xc21cfe9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabcfe9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dcfea5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdbcfea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29cfeab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0fcfead, SUC_lexicon_tags_28 },
    { 0x737cfeae, SUC_lexicon_tags_47 },
    { 0x73dcfeae, SUC_lexicon_tags_47 },
    { 0x10bcfeae, SUC_lexicon_tags_15 },
    { 0xdbfcfeb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdcfeb5, SUC_lexicon_tags_11 },
    { 0x73bcfeb5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffcfebf, SUC_lexicon_tags_25 },
    { 0xf31cfec0, SUC_lexicon_tags_1 },
    { 0xea9cfec0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87cfec4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5cfec8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbcfecc, SUC_lexicon_tags_27 },
    { 0x451cfecd, SUC_lexicon_tags_1 },
    { 0xfdbcfecc, SUC_lexicon_tags_6 },
    { 0x3ffcfecf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x943cfed1, SUC_lexicon_tags_65 },
    { 0x4fcfed1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa9cfed4, SUC_lexicon_tags_39 },
    { 0xbb5cfed5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bcfeda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61cfedd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237cfee1, SUC_lexicon_tags_1 },
    { 0xe03cfee1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x78bcfee4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bcfeea, SUC_lexicon_tags_49 },
    { 0xa5bcfeea, SUC_lexicon_tags_45 },
    { 0xb57cfeec, SUC_lexicon_tags_57 },
    { 0x8c9cfeec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaefcfef0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5cfef3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1cfef6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fcfefa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7bcfefc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x285cfefe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633cff03, SUC_lexicon_tags_14 },
    { 0xa27cff04, SUC_lexicon_tags_0 },
    { 0x9f7cff05, SUC_lexicon_tags_68 },
    { 0xa23cff05, SUC_lexicon_tags_46 },
    { 0xcd1cff04, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa0dcff09, SUC_lexicon_tags_15 },
    { 0x2d3cff0a, SUC_lexicon_tags_37 },
    { 0x25fcff0a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4f5cff0d, SUC_lexicon_tags_38 },
    { 0x9c3cff0d, SUC_lexicon_tags_17 },
    { 0x2b3cff0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fcff12, SUC_lexicon_tags_47 },
    { 0x317cff13, SUC_lexicon_tags_2 },
    { 0x81fcff14, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9cff18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x157cff1a, SUC_lexicon_tags_17 },
    { 0x971cff1b, SUC_lexicon_tags_35 },
    { 0xeabcff1b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x477cff1f, SUC_lexicon_tags_37 },
    { 0x4a5cff20, SUC_lexicon_tags_37 },
    { 0x6dfcff20, SUC_lexicon_tags_28 },
    { 0x477cff20, SUC_lexicon_tags_31 },
    { 0x6fdcff23, SUC_lexicon_tags_66 },
    { 0x399cff24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62dcff2a, SUC_lexicon_tags_37 },
    { 0xc9dcff2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69cff2f, SUC_lexicon_tags_12 },
    { 0xcefcff30, SUC_lexicon_tags_46 },
    { 0xff1cff30, SUC_lexicon_tags_1 },
    { 0xa43cff31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x481cff35, SUC_lexicon_tags_37 },
    { 0x535cff36, SUC_lexicon_tags_92 },
    { 0xb6fcff36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71cff3a, SUC_lexicon_tags_1 },
    { 0xbe7cff3b, SUC_lexicon_tags_47 },
    { 0x789cff3a, SUC_lexicon_tags_17 },
    { 0x2d9cff3a, SUC_lexicon_tags_66 },
    { 0xfafcff3c, SUC_lexicon_tags_46 },
    { 0xb5dcff3a, SUC_lexicon_tags_17 },
    { 0x2f9cff40, SUC_lexicon_tags_37 },
    { 0xc79cff40, SUC_lexicon_tags_7 },
    { 0x7efcff40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x583cff45, SUC_lexicon_tags_2 },
    { 0x805cff46, SUC_lexicon_tags_15 },
    { 0xcafcff46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffcff4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35cff50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdcff53, SUC_lexicon_tags_30 },
    { 0xb33cff54, SUC_lexicon_tags_12 },
    { 0x643cff54, SUC_lexicon_tags_8 },
    { 0x909cff56, SUC_lexicon_tags_1 },
    { 0xc37cff57, SUC_lexicon_tags_63 },
    { 0x9a7cff58, SUC_lexicon_tags_21 },
    { 0xb3bcff59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb7cff5b, SUC_lexicon_tags_7 },
    { 0xe85cff5c, SUC_lexicon_tags_15 },
    { 0xa83cff5c, SUC_lexicon_tags_85 },
    { 0xc5cff5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fcff63, SUC_lexicon_tags_1 },
    { 0xd11cff63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483cff68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bcff6a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9cff6d, SUC_lexicon_tags_33 },
    { 0xbb3cff6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9cff78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x40dcff7a, SUC_lexicon_tags_15 },
    { 0x411cff7a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1cff7f, SUC_lexicon_tags_4 },
    { 0xfa7cff7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa41cff82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x323cff84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x275cff87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3cff8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fcff8e, SUC_lexicon_tags_15 },
    { 0xebbcff8e, SUC_lexicon_tags_37 },
    { 0x15fcff90, SUC_lexicon_tags_11 },
    { 0x9cbcff91, SUC_lexicon_tags_11 },
    { 0xa11cff92, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x609cff94, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71cff9e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fcffa2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7cffa5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91cffac, SUC_lexicon_tags_7 },
    { 0xe55cffad, SUC_lexicon_tags_14 },
    { 0x881cffad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x599cffb1, SUC_lexicon_tags_32 },
    { 0x681cffb2, SUC_lexicon_tags_2 },
    { 0x38fcffb2, SUC_lexicon_tags_49 },
    { 0x84fcffb4, SUC_lexicon_tags_11 },
    { 0xb87cffb4, SUC_lexicon_tags_11 },
    { 0x6c9cffb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45cffb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68bcffba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1bcffbc, SUC_lexicon_tags_25 },
    { 0x775cffbd, SUC_lexicon_tags_15 },
    { 0x691cffbe, SUC_lexicon_tags_0 },
    { 0x935cffbe, SUC_lexicon_tags_25 },
    { 0x961cffbe, SUC_lexicon_tags_39 },
    { 0x911cffbf, SUC_lexicon_tags_14 },
    { 0xdf1cffbe, SUC_lexicon_tags_11 },
    { 0xab5cffc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fdcffc5, SUC_lexicon_tags_8 },
    { 0xd9dcffc6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfbcffc8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81cffcc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x291cffce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3cffd1, SUC_lexicon_tags_63 },
    { 0xb6bcffd2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x363cffd4, SUC_lexicon_tags_25 },
    { 0x1f5cffd5, SUC_lexicon_tags_14 },
    { 0x29fcffd4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbbcffd8, SUC_lexicon_tags_1 },
    { 0xdf3cffd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeddcffdb, SUC_lexicon_tags_12 },
    { 0xbf7cffdb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dcffea, SUC_lexicon_tags_7 },
    { 0x5f7cffea, SUC_lexicon_tags_2 },
    { 0x24fcffec, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1fdcffee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x85dcfff0, SUC_lexicon_tags_65 },
    { 0x53bcfff0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x871cfff3, SUC_lexicon_tags_9 },
    { 0x8e7cfff4, SUC_lexicon_tags_38 },
    { 0xb09cfff4, SUC_lexicon_tags_25 },
    { 0x6fcfff3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443cfffa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5d0000, SUC_lexicon_tags_19 },
    { 0xa63d0000, SUC_lexicon_tags_60 },
    { 0xea3d0000, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3d000a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99bd000c, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x10bd000e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfd0012, SUC_lexicon_tags_37 },
    { 0x463d0013, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0x1fdd0015, SUC_lexicon_tags_77 },
    { 0xc5dd0016, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7b5d0018, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339d001e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x91fd0020, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edd0024, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x847d0027, SUC_lexicon_tags_19 },
    { 0x9f3d0028, SUC_lexicon_tags_7 },
    { 0xce7d0027, SUC_lexicon_tags_25 },
    { 0xa75d002a, SUC_lexicon_tags_11 },
    { 0xca5d002b, SUC_lexicon_tags_60 },
    { 0xd17d002b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43d0032, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x193d0034, SUC_lexicon_tags_7 },
    { 0xeb7d0034, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9d0038, SUC_lexicon_tags_27 },
    { 0x37fd0038, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x33dd003b, SUC_lexicon_tags_25 },
    { 0xb91d003b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305d0043, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1bd0045, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fd0048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fd0050, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71d0056, SUC_lexicon_tags_14 },
    { 0x821d0056, SUC_lexicon_tags_6 },
    { 0x9edd0058, SUC_lexicon_tags_12 },
    { 0xbb9d0056, SUC_lexicon_tags_27 },
    { 0x665d0058, SUC_lexicon_tags_2 },
    { 0x2e1d005a, SUC_lexicon_tags_6 },
    { 0xd03d005c, SUC_lexicon_tags_38 },
    { 0xe9d005d, SUC_lexicon_tags_2 },
    { 0x953d005c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03d0061, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4bdd0063, SUC_lexicon_tags_13 },
    { 0x32bd0064, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbffd0066, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe33d0068, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x753d006a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe8bd006c, SUC_lexicon_tags_32 },
    { 0xa2fd006d, SUC_lexicon_tags_32 },
    { 0x293d006c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x317d0070, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x267d0072, SUC_lexicon_tags_12 },
    { 0x46dd0072, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317d0077, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bd007b, SUC_lexicon_tags_8 },
    { 0xf87d007c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd0083, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc0fd0085, SUC_lexicon_tags_7 },
    { 0xca5d0086, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x951d0089, SUC_lexicon_tags_14 },
    { 0x3c5d0089, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7d008d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f3d008f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc6fd0091, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74dd0093, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebd0099, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddd009e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb67d00a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bd00a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7a1d00a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x551d00a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bd00aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31d00b2, SUC_lexicon_tags_17 },
    { 0x1e7d00b3, SUC_lexicon_tags_11 },
    { 0xa59d00b3, SUC_lexicon_tags_48 },
    { 0xf5bd00b4, SUC_lexicon_tags_60 },
    { 0xef1d00b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27d00ba, SUC_lexicon_tags_37 },
    { 0xb79d00ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x439d00be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3d00c2, SUC_lexicon_tags_17 },
    { 0x7b9d00c3, SUC_lexicon_tags_28 },
    { 0xdfd00c3, SUC_lexicon_tags_39 },
    { 0xa8dd00c3, SUC_lexicon_tags_85 },
    { 0xd8fd00c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49d00ca, SUC_lexicon_tags_23 },
    { 0x773d00ca, SUC_lexicon_tags_1 },
    { 0xee1d00cc, SUC_lexicon_tags_15 },
    { 0xffd00cd, SUC_lexicon_tags_17 },
    { 0x18fd00ce, SUC_lexicon_tags_2 },
    { 0xdfdd00cf, SUC_lexicon_tags_63 },
    { 0xd03d00d0, SUC_lexicon_tags_70 },
    { 0xfabd00d0, SUC_lexicon_tags_27 },
    { 0x30dd00cf, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x951d00d5, SUC_lexicon_tags_37 },
    { 0xc0fd00d6, SUC_lexicon_tags_66 },
    { 0xdcbd00d5, SUC_lexicon_tags_18 },
    { 0xd65d00d8, SUC_lexicon_tags_8 },
    { 0x8efd00d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x23bd00db, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x807d00dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fd00e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb69d00e2, SUC_lexicon_tags_15 },
    { 0x563d00e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf49d00e5, SUC_lexicon_tags_11 },
    { 0xda7d00e5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1d00ed, SUC_lexicon_tags_21 },
    { 0x2dbd00ed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1d00f2, SUC_lexicon_tags_45 },
    { 0xc9dd00f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abd00f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d7d00f8, SUC_lexicon_tags_57 },
    { 0x7abd00f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdd0103, SUC_lexicon_tags_43 },
    { 0x27fd0103, SUC_lexicon_tags_11 },
    { 0x5cdd0105, SUC_lexicon_tags_25 },
    { 0xa63d0106, SUC_lexicon_tags_62 },
    { 0xd47d0107, SUC_lexicon_tags_14 },
    { 0xc1d0107, SUC_lexicon_tags_19 },
    { 0x8e3d0107, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65d010c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57d0114, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bfd0116, SUC_lexicon_tags_44 },
    { 0x341d0117, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785d011b, SUC_lexicon_tags_4 },
    { 0xe1d011c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99d011f, SUC_lexicon_tags_7 },
    { 0x88fd0120, SUC_lexicon_tags_7 },
    { 0x97dd0121, SUC_lexicon_tags_25 },
    { 0x26fd0122, SUC_lexicon_tags_25 },
    { 0xcfdd0122, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x175d0125, SUC_lexicon_tags_48 },
    { 0x58bd0126, SUC_lexicon_tags_17 },
    { 0x9d3d0125, SUC_lexicon_tags_87 },
    { 0x6add0125, SUC_lexicon_tags_14 },
    { 0x867d0129, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8cdd012b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x551d012d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745d0133, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1d0136, SUC_lexicon_tags_4 },
    { 0xd53d0137, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bd013a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21d013d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15dd0142, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73d0148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x133d014b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafd014e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30fd0150, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfd0155, SUC_lexicon_tags_1 },
    { 0x903d0155, SUC_lexicon_tags_14 },
    { 0x2b5d0157, SUC_lexicon_tags_12 },
    { 0x5edd0157, SUC_lexicon_tags_11 },
    { 0x9add0159, SUC_lexicon_tags_19 },
    { 0x4c5d015a, SUC_lexicon_tags_48 },
    { 0x897d0156, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5d015f, SUC_lexicon_tags_79 },
    { 0xcbd0160, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fd0163, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb1d0165, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563d016d, SUC_lexicon_tags_31 },
    { 0xaefd016d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d0175, SUC_lexicon_tags_38 },
    { 0x35dd0175, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcdd0178, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1d017e, SUC_lexicon_tags_7 },
    { 0x31dd017f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf6bd0181, SUC_lexicon_tags_25 },
    { 0x641d0181, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58dd0184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51d0189, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd91d018b, SUC_lexicon_tags_33 },
    { 0xb83d018b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c9d018e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fd0191, SUC_lexicon_tags_25 },
    { 0x7d1d0192, SUC_lexicon_tags_1 },
    { 0xaabd0193, SUC_lexicon_tags_7 },
    { 0x61d0193, SUC_lexicon_tags_25 },
    { 0x39dd0193, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b5d0197, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a9d0199, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d01a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe57d01a8, SUC_lexicon_tags_2 },
    { 0x4e5d01a9, SUC_lexicon_tags_184 },
    { 0xa49d01aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7edd01ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fd01b2, SUC_lexicon_tags_33 },
    { 0x691d01b3, SUC_lexicon_tags_17 },
    { 0x83dd01b3, SUC_lexicon_tags_37 },
    { 0x745d01b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177d01b9, SUC_lexicon_tags_33 },
    { 0xaf3d01ba, SUC_lexicon_tags_21 },
    { 0x305d01ba, SUC_lexicon_tags_43 },
    { 0x78bd01ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399d01c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x115d01c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x493d01c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x39fd01ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33d01ce, SUC_lexicon_tags_11 },
    { 0x997d01cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59d01d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa75d01da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d01dd, SUC_lexicon_tags_25 },
    { 0x70bd01de, SUC_lexicon_tags_37 },
    { 0x8ffd01df, SUC_lexicon_tags_10 },
    { 0xba1d01df, SUC_lexicon_tags_28 },
    { 0x673d01e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3d01e3, SUC_lexicon_tags_25 },
    { 0x4edd01e4, SUC_lexicon_tags_11 },
    { 0x9b5d01e5, SUC_lexicon_tags_14 },
    { 0x417d01e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x819d01e9, SUC_lexicon_tags_99 },
    { 0xb4fd01ea, SUC_lexicon_tags_2 },
    { 0x6a5d01e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda1d01ed, SUC_lexicon_tags_30 },
    { 0xbadd01ed, SUC_lexicon_tags_11 },
    { 0x147d01ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x619d01f2, SUC_lexicon_tags_29 },
    { 0xfbdd01f3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433d01f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3add01fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbd01ff, SUC_lexicon_tags_7 },
    { 0xeefd0200, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe15d0202, SUC_lexicon_tags_1 },
    { 0x85d0203, SUC_lexicon_tags_55 },
    { 0x3bd0203, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e3d0206, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbd020b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fd020e, SUC_lexicon_tags_19 },
    { 0xdddd020f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30dd0211, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20dd0213, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba3d0215, SUC_lexicon_tags_47 },
    { 0xdb3d0216, SUC_lexicon_tags_27 },
    { 0x82bd0217, SUC_lexicon_tags_1 },
    { 0xcddd0218, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd021b, SUC_lexicon_tags_11 },
    { 0xa9bd021b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x865d021e, SUC_lexicon_tags_27 },
    { 0x157d021f, SUC_lexicon_tags_24 },
    { 0x8ebd021f, SUC_lexicon_tags_7 },
    { 0xef5d0221, SUC_lexicon_tags_15 },
    { 0x99fd0220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd19d0224, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e5d0226, SUC_lexicon_tags_7 },
    { 0xc73d0227, SUC_lexicon_tags_1 },
    { 0x56bd0228, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fd022f, SUC_lexicon_tags_0 },
    { 0xe61d022f, SUC_lexicon_tags_25 },
    { 0xfd3d0230, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb71d0233, SUC_lexicon_tags_30 },
    { 0xf21d0234, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbd0237, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1d023c, SUC_lexicon_tags_11 },
    { 0xe43d023d, SUC_lexicon_tags_2 },
    { 0xed7d023e, SUC_lexicon_tags_14 },
    { 0xbb1d023e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ffd0241, SUC_lexicon_tags_47 },
    { 0x58dd0242, SUC_lexicon_tags_17 },
    { 0x33bd0243, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bd0246, SUC_lexicon_tags_35 },
    { 0xb61d0247, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1d024a, SUC_lexicon_tags_37 },
    { 0x4cbd024b, SUC_lexicon_tags_12 },
    { 0x98dd024c, SUC_lexicon_tags_37 },
    { 0xa73d024d, SUC_lexicon_tags_11 },
    { 0x3d5d024b, SUC_lexicon_tags_14 },
    { 0x89d024b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673d0253, SUC_lexicon_tags_70 },
    { 0x21fd0253, SUC_lexicon_tags_14 },
    { 0x123d0255, SUC_lexicon_tags_2 },
    { 0xe15d0256, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fd025d, SUC_lexicon_tags_47 },
    { 0xf41d025d, SUC_lexicon_tags_25 },
    { 0x2c3d025d, SUC_lexicon_tags_37 },
    { 0xb01d025d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3d0267, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dd026a, SUC_lexicon_tags_25 },
    { 0xd55d026b, SUC_lexicon_tags_4 },
    { 0xebbd026c, SUC_lexicon_tags_12 },
    { 0xf83d026b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95d0273, SUC_lexicon_tags_63 },
    { 0xc03d0273, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3d0278, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7d027b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x845d027e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8cbd0280, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9d0283, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367d0287, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x55fd0289, SUC_lexicon_tags_37 },
    { 0x80bd0289, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5add028c, SUC_lexicon_tags_27 },
    { 0xd2dd028d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9bd028f, SUC_lexicon_tags_11 },
    { 0xbc1d0290, SUC_lexicon_tags_15 },
    { 0x449d0290, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x529d0293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3d0296, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dd029c, SUC_lexicon_tags_25 },
    { 0x27fd029d, SUC_lexicon_tags_42 },
    { 0x8abd029e, SUC_lexicon_tags_529 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9d02a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc3d02a3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x949d02a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3d02aa, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1e5d02ac, SUC_lexicon_tags_37 },
    { 0x675d02ad, SUC_lexicon_tags_37 },
    { 0xc7d02ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe53d02b0, SUC_lexicon_tags_38 },
    { 0x9a3d02b1, SUC_lexicon_tags_54 },
    { 0xd91d02b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedd02b7, SUC_lexicon_tags_4 },
    { 0x711d02b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1d02c0, SUC_lexicon_tags_60 },
    { 0xdefd02c1, SUC_lexicon_tags_33 },
    { 0xd4fd02c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbd7d02c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7bd02c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fd02ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67dd02cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e1d02ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bd02d2, SUC_lexicon_tags_8 },
    { 0x237d02d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3d02d6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4bbd02d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fd02dc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73d02e0, SUC_lexicon_tags_17 },
    { 0x6f3d02e0, SUC_lexicon_tags_12 },
    { 0x81fd02e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x471d02e5, SUC_lexicon_tags_14 },
    { 0x65fd02e6, SUC_lexicon_tags_37 },
    { 0xc37d02e6, SUC_lexicon_tags_43 },
    { 0x1fdd02e6, SUC_lexicon_tags_37 },
    { 0xb89d02e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61d02f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273d02fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619d0300, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dd0305, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305d0311, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7d0316, SUC_lexicon_tags_38 },
    { 0x11dd0316, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d031a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1d031f, SUC_lexicon_tags_11 },
    { 0xf19d0320, SUC_lexicon_tags_68 },
    { 0xe0fd0320, SUC_lexicon_tags_29 },
    { 0xdd9d0322, SUC_lexicon_tags_15 },
    { 0x381d0320, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bbd0325, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x51bd0327, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611d032c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x12dd032f, SUC_lexicon_tags_25 },
    { 0xe79d032f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd0338, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdd033d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x435d033f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x135d0342, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549d0347, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe1fd0349, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7d034d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x555d034f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x74fd0351, SUC_lexicon_tags_2 },
    { 0xcf5d0352, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9d0355, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd0bd0357, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82dd035a, SUC_lexicon_tags_1 },
    { 0x9f7d035b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfd0363, SUC_lexicon_tags_29 },
    { 0xbd9d0363, SUC_lexicon_tags_31 },
    { 0xd59d0363, SUC_lexicon_tags_2 },
    { 0x1b3d0363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5d036c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fd0370, SUC_lexicon_tags_45 },
    { 0x1b5d0371, SUC_lexicon_tags_37 },
    { 0x55fd0372, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x62dd0374, SUC_lexicon_tags_27 },
    { 0xdf1d0374, SUC_lexicon_tags_33 },
    { 0x5dbd0376, SUC_lexicon_tags_15 },
    { 0xc91d0376, SUC_lexicon_tags_17 },
    { 0xe01d0376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bd037b, SUC_lexicon_tags_61 },
    { 0xe89d037b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341d0380, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x933d0383, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9add0386, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19d0389, SUC_lexicon_tags_1 },
    { 0x69bd0389, SUC_lexicon_tags_14 },
    { 0x6a7d038b, SUC_lexicon_tags_1 },
    { 0xccbd038c, SUC_lexicon_tags_85 },
    { 0x7a5d038d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x997d038f, SUC_lexicon_tags_1 },
    { 0x129d0390, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x12dd0392, SUC_lexicon_tags_16 },
    { 0x781d0393, SUC_lexicon_tags_47 },
    { 0x949d0394, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fd039a, SUC_lexicon_tags_63 },
    { 0x2cfd039b, SUC_lexicon_tags_8 },
    { 0xa27d039c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dd039f, SUC_lexicon_tags_43 },
    { 0xda7d03a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411d03a5, SUC_lexicon_tags_33 },
    { 0xe9fd03a6, SUC_lexicon_tags_55 },
    { 0xb0dd03a7, SUC_lexicon_tags_1 },
    { 0x2bbd03a7, SUC_lexicon_tags_8 },
    { 0xdf9d03a5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x491d03ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5bd03ad, SUC_lexicon_tags_12 },
    { 0x271d03ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x693d03b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd3dd03b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x90dd03b5, SUC_lexicon_tags_17 },
    { 0x2e1d03b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93d03bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d03be, SUC_lexicon_tags_1 },
    { 0x13bd03bf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dd03c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7d03c8, SUC_lexicon_tags_11 },
    { 0x78bd03c9, SUC_lexicon_tags_0 },
    { 0x999d03ca, SUC_lexicon_tags_6 },
    { 0x7edd03cb, SUC_lexicon_tags_65 },
    { 0xe85d03cc, SUC_lexicon_tags_11 },
    { 0x711d03cd, SUC_lexicon_tags_25 },
    { 0x65dd03ce, SUC_lexicon_tags_25 },
    { 0x793d03cf, SUC_lexicon_tags_18 },
    { 0x1efd03d0, SUC_lexicon_tags_14 },
    { 0xeb5d03d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc77d03d3, SUC_lexicon_tags_25 },
    { 0xef7d03d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fd03dd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3afd03df, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x6edd03e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc87d03e3, SUC_lexicon_tags_14 },
    { 0x723d03e4, SUC_lexicon_tags_12 },
    { 0x6c5d03e4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc4dd03e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x135d03e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59d03ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfedd03ee, SUC_lexicon_tags_104 },
    { 0x5bbd03ef, SUC_lexicon_tags_62 },
    { 0xaedd03f0, SUC_lexicon_tags_38 },
    { 0x1d3d03f1, SUC_lexicon_tags_37 },
    { 0xc13d03f1, SUC_lexicon_tags_11 },
    { 0x813d03f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x183d03f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x775d03f8, SUC_lexicon_tags_29 },
    { 0x67fd03f9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7d03fc, SUC_lexicon_tags_0 },
    { 0x8f5d03fc, SUC_lexicon_tags_39 },
    { 0x7bdd03fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9dd0402, SUC_lexicon_tags_7 },
    { 0x20dd0402, SUC_lexicon_tags_7 },
    { 0xa1fd0403, SUC_lexicon_tags_21 },
    { 0xf99d0405, SUC_lexicon_tags_37 },
    { 0x429d0402, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x15dd0409, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x307d040b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebd040f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x801d0411, SUC_lexicon_tags_18 },
    { 0xbcfd0411, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bd0415, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc87d0417, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469d041d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdd0421, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9d042a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xffdd042d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3d0430, SUC_lexicon_tags_60 },
    { 0xfbfd0431, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2b5d0433, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121d043b, SUC_lexicon_tags_0 },
    { 0x57fd043c, SUC_lexicon_tags_25 },
    { 0x345d043b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffd0443, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef5d0445, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9d044a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dd044e, SUC_lexicon_tags_25 },
    { 0x53fd044e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x435d0452, SUC_lexicon_tags_11 },
    { 0x66bd0452, SUC_lexicon_tags_1 },
    { 0x8ffd0454, SUC_lexicon_tags_37 },
    { 0x68dd0452, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x9cdd0457, SUC_lexicon_tags_43 },
    { 0x7efd0458, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd045d, SUC_lexicon_tags_25 },
    { 0xd0dd045e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60dd0460, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fd0465, SUC_lexicon_tags_1 },
    { 0xf7dd0466, SUC_lexicon_tags_7 },
    { 0x5c3d0467, SUC_lexicon_tags_25 },
    { 0x1a5d0467, SUC_lexicon_tags_1 },
    { 0x273d0467, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fd046f, SUC_lexicon_tags_17 },
    { 0x6e3d0470, SUC_lexicon_tags_33 },
    { 0xc1bd0470, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85d0474, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x697d0477, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe61d0479, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27d047c, SUC_lexicon_tags_19 },
    { 0x735d047d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dd0481, SUC_lexicon_tags_14 },
    { 0x1c9d0482, SUC_lexicon_tags_37 },
    { 0x667d0482, SUC_lexicon_tags_11 },
    { 0x507d0484, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1bfd0486, SUC_lexicon_tags_10 },
    { 0x325d0486, SUC_lexicon_tags_60 },
    { 0x1afd0487, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x77dd048a, SUC_lexicon_tags_15 },
    { 0x3f3d048a, SUC_lexicon_tags_37 },
    { 0x31d048c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615d0493, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1d0497, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd5d0499, SUC_lexicon_tags_3 },
    { 0x5dfd0499, SUC_lexicon_tags_8 },
    { 0x5c9d049b, SUC_lexicon_tags_25 },
    { 0x61dd049b, SUC_lexicon_tags_2 },
    { 0x5abd049b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba7d049f, SUC_lexicon_tags_35 },
    { 0x41fd049f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13d04a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26dd04a9, SUC_lexicon_tags_49 },
    { 0x4e9d04aa, SUC_lexicon_tags_22 },
    { 0xbdd04aa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9d04ae, SUC_lexicon_tags_1 },
    { 0x6b3d04ae, SUC_lexicon_tags_37 },
    { 0x399d04b0, SUC_lexicon_tags_2 },
    { 0x9d7d04b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373d04b5, SUC_lexicon_tags_12 },
    { 0xcf9d04b5, SUC_lexicon_tags_39 },
    { 0x791d04b6, SUC_lexicon_tags_37 },
    { 0x307d04b6, SUC_lexicon_tags_4 },
    { 0x9d9d04b6, SUC_lexicon_tags_27 },
    { 0x259d04b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07d04bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x30bd04bf, SUC_lexicon_tags_33 },
    { 0x70fd04c0, SUC_lexicon_tags_1 },
    { 0x4a9d04c1, SUC_lexicon_tags_96 },
    { 0x9bd04c1, SUC_lexicon_tags_19 },
    { 0x7b7d04c3, SUC_lexicon_tags_84 },
    { 0x9edd04c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7d04cc, SUC_lexicon_tags_37 },
    { 0xf1fd04cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdd04d1, SUC_lexicon_tags_23 },
    { 0xa35d04d2, SUC_lexicon_tags_33 },
    { 0x383d04d3, SUC_lexicon_tags_17 },
    { 0x145d04d4, SUC_lexicon_tags_11 },
    { 0x78fd04d5, SUC_lexicon_tags_26 },
    { 0xfc9d04d1, SUC_lexicon_tags_29 },
    { 0xbdd04d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15d04da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf35d04dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8cfd04de, SUC_lexicon_tags_62 },
    { 0xb97d04df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fdd04e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bd04e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761d04e8, SUC_lexicon_tags_37 },
    { 0x2c7d04e8, SUC_lexicon_tags_15 },
    { 0x773d04ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1d04f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54bd04f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfd04fa, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7d04fd, SUC_lexicon_tags_21 },
    { 0x7c1d04fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa61d0500, SUC_lexicon_tags_605 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73d0504, SUC_lexicon_tags_26 },
    { 0xa6bd0505, SUC_lexicon_tags_37 },
    { 0x285d0505, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731d050a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543d050f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfd0515, SUC_lexicon_tags_46 },
    { 0x4add0516, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1d051a, SUC_lexicon_tags_15 },
    { 0xd9d051a, SUC_lexicon_tags_15 },
    { 0xcb1d051c, SUC_lexicon_tags_1 },
    { 0x7a5d051a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6f9d051f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1d0522, SUC_lexicon_tags_14 },
    { 0xc11d0523, SUC_lexicon_tags_12 },
    { 0xd33d0523, SUC_lexicon_tags_2 },
    { 0xa4dd0524, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99d0529, SUC_lexicon_tags_37 },
    { 0xf17d052a, SUC_lexicon_tags_63 },
    { 0x799d0529, SUC_lexicon_tags_11 },
    { 0xaa1d052b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bd052f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37d053a, SUC_lexicon_tags_19 },
    { 0x415d053b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29d053e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725d0543, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dd0546, SUC_lexicon_tags_47 },
    { 0x977d0547, SUC_lexicon_tags_23 },
    { 0xe27d0547, SUC_lexicon_tags_19 },
    { 0x52fd0549, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8a7d054b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429d0550, SUC_lexicon_tags_6 },
    { 0x263d0550, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x293d0553, SUC_lexicon_tags_25 },
    { 0x9d7d0554, SUC_lexicon_tags_17 },
    { 0x707d0555, SUC_lexicon_tags_32 },
    { 0xbf1d0556, SUC_lexicon_tags_2 },
    { 0x8add0553, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x56dd0559, SUC_lexicon_tags_11 },
    { 0x87d0559, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0xe23d055c, SUC_lexicon_tags_15 },
    { 0xdd1d055c, SUC_lexicon_tags_17 },
    { 0x2cfd055e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d0563, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bd0567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721d056c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95dd056e, SUC_lexicon_tags_12 },
    { 0xd53d056e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x75bd0571, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c3d0573, SUC_lexicon_tags_37 },
    { 0x91bd0574, SUC_lexicon_tags_17 },
    { 0x6e9d0573, SUC_lexicon_tags_17 },
    { 0xedbd0576, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81fd0578, SUC_lexicon_tags_0 },
    { 0x349d0578, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855d057d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bd0581, SUC_lexicon_tags_49 },
    { 0xa87d0582, SUC_lexicon_tags_12 },
    { 0x799d0583, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4e5d0585, SUC_lexicon_tags_7 },
    { 0x8c5d0586, SUC_lexicon_tags_49 },
    { 0xcdbd0587, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xf1d0589, SUC_lexicon_tags_6 },
    { 0x56dd058a, SUC_lexicon_tags_48 },
    { 0x7a9d058b, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x8a1d058d, SUC_lexicon_tags_65 },
    { 0x705d058e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd0593, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bd0596, SUC_lexicon_tags_47 },
    { 0x9fbd0597, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc0fd0599, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x797d059b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67d059e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc07d05a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x683d05a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fd05ae, SUC_lexicon_tags_12 },
    { 0x43bd05ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817d05b3, SUC_lexicon_tags_14 },
    { 0xd0dd05b4, SUC_lexicon_tags_12 },
    { 0x4dbd05b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5d05b9, SUC_lexicon_tags_1 },
    { 0x9c7d05ba, SUC_lexicon_tags_32 },
    { 0xfdd05bb, SUC_lexicon_tags_113 },
    { 0xafdd05bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x475d05bf, SUC_lexicon_tags_2 },
    { 0x6a1d05c0, SUC_lexicon_tags_34 },
    { 0x617d05c1, SUC_lexicon_tags_3 },
    { 0x51dd05c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9d05c6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6f3d05c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85dd05ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x867d05cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd3bd05cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9d05d6, SUC_lexicon_tags_87 },
    { 0xcf3d05d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x467d05d9, SUC_lexicon_tags_45 },
    { 0x54dd05da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d05de, SUC_lexicon_tags_7 },
    { 0x569d05df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5d05e3, SUC_lexicon_tags_15 },
    { 0x5cfd05e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd05e8, SUC_lexicon_tags_27 },
    { 0x3f1d05e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x821d05eb, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x11d05ee, SUC_lexicon_tags_15 },
    { 0xe83d05ef, SUC_lexicon_tags_15 },
    { 0x98dd05f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473d05f6, SUC_lexicon_tags_9 },
    { 0x89bd05f7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xae1d05f9, SUC_lexicon_tags_29 },
    { 0x71dd05f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52bd05fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad3d05fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9afd0600, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8bd0602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3d060b, SUC_lexicon_tags_33 },
    { 0x10fd060b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x751d060e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9d0611, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09d0614, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd83d0616, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbd0620, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe51d0622, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39d0628, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27bd062a, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x6cdd062c, SUC_lexicon_tags_33 },
    { 0x5fdd062d, SUC_lexicon_tags_1 },
    { 0x78bd062c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5fd0630, SUC_lexicon_tags_7 },
    { 0x4a3d0630, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb6bd0633, SUC_lexicon_tags_57 },
    { 0x30dd0634, SUC_lexicon_tags_30 },
    { 0xb5bd0633, SUC_lexicon_tags_8 },
    { 0xc89d0634, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d0639, SUC_lexicon_tags_21 },
    { 0xefd0639, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeffd063c, SUC_lexicon_tags_55 },
    { 0xc45d063d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3d0643, SUC_lexicon_tags_12 },
    { 0x5d9d0644, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3d0647, SUC_lexicon_tags_15 },
    { 0xa91d0647, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21d0652, SUC_lexicon_tags_37 },
    { 0xf7dd0653, SUC_lexicon_tags_26 },
    { 0xe6dd0653, SUC_lexicon_tags_33 },
    { 0x241d0654, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3d0658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abd065d, SUC_lexicon_tags_57 },
    { 0xd3bd065e, SUC_lexicon_tags_55 },
    { 0x3afd065f, SUC_lexicon_tags_25 },
    { 0x4d9d065d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d0663, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75d0667, SUC_lexicon_tags_25 },
    { 0x161d0668, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7d066c, SUC_lexicon_tags_49 },
    { 0xf7dd066d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fd0672, SUC_lexicon_tags_28 },
    { 0x11fd0672, SUC_lexicon_tags_15 },
    { 0xbb5d0674, SUC_lexicon_tags_60 },
    { 0xefbd0674, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919d0679, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695d067f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23d0683, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9d0686, SUC_lexicon_tags_37 },
    { 0x465d0687, SUC_lexicon_tags_1 },
    { 0xbdfd0688, SUC_lexicon_tags_37 },
    { 0xc0bd0686, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd068d, SUC_lexicon_tags_26 },
    { 0x771d068e, SUC_lexicon_tags_37 },
    { 0x3dd068e, SUC_lexicon_tags_48 },
    { 0x2d1d0690, SUC_lexicon_tags_28 },
    { 0x397d0691, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7d069c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3add069e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23d06a1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x683d06a4, SUC_lexicon_tags_6 },
    { 0x4e7d06a5, SUC_lexicon_tags_63 },
    { 0x919d06a6, SUC_lexicon_tags_55 },
    { 0xf1fd06a7, SUC_lexicon_tags_23 },
    { 0xa49d06a6, SUC_lexicon_tags_14 },
    { 0xab3d06a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9d06ab, SUC_lexicon_tags_7 },
    { 0x5ffd06ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfdfd06ae, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x565d06b0, SUC_lexicon_tags_16 },
    { 0x97d06b1, SUC_lexicon_tags_17 },
    { 0x74fd06b2, SUC_lexicon_tags_61 },
    { 0xfd7d06b3, SUC_lexicon_tags_7 },
    { 0x3f3d06b3, SUC_lexicon_tags_23 },
    { 0x6dfd06b5, SUC_lexicon_tags_1 },
    { 0x953d06b3, SUC_lexicon_tags_15 },
    { 0xb63d06b6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07d06ba, SUC_lexicon_tags_12 },
    { 0x1c9d06bb, SUC_lexicon_tags_236 },
    { 0xa3d06bc, SUC_lexicon_tags_17 },
    { 0xd77d06bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7d06c1, SUC_lexicon_tags_23 },
    { 0x35dd06c2, SUC_lexicon_tags_17 },
    { 0x67fd06c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x855d06c6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9d06c9, SUC_lexicon_tags_1 },
    { 0xf1dd06c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d06d1, SUC_lexicon_tags_65 },
    { 0x8b7d06d2, SUC_lexicon_tags_29 },
    { 0xf79d06d3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637d06da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa77d06dc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5d06e0, SUC_lexicon_tags_37 },
    { 0x2fd06e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe39d06e3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x623d06e5, SUC_lexicon_tags_60 },
    { 0xb75d06e5, SUC_lexicon_tags_10 },
    { 0x603d06e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9d06ea, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x607d06ec, SUC_lexicon_tags_25 },
    { 0x277d06ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fd06f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651d06fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bd06fd, SUC_lexicon_tags_23 },
    { 0x9c9d06fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe3dd0700, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x507d0702, SUC_lexicon_tags_7 },
    { 0xa0bd0703, SUC_lexicon_tags_15 },
    { 0x633d0704, SUC_lexicon_tags_12 },
    { 0x46dd0705, SUC_lexicon_tags_2 },
    { 0x271d0706, SUC_lexicon_tags_2 },
    { 0x67bd0706, SUC_lexicon_tags_7 },
    { 0x613d0708, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429d070d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bd0716, SUC_lexicon_tags_2 },
    { 0xdbbd0717, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6f9d0719, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bd071d, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xadbd071f, SUC_lexicon_tags_60 },
    { 0x7d3d0720, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x47bd0722, SUC_lexicon_tags_24 },
    { 0xbf1d0722, SUC_lexicon_tags_21 },
    { 0xa5d0724, SUC_lexicon_tags_8 },
    { 0x543d0723, SUC_lexicon_tags_17 },
    { 0xa65d0726, SUC_lexicon_tags_17 },
    { 0x6c9d0726, SUC_lexicon_tags_66 },
    { 0xf3fd0724, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fd072b, SUC_lexicon_tags_7 },
    { 0xb6dd072b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x88dd072e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbd0736, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fd073a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbd073f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217d0745, SUC_lexicon_tags_27 },
    { 0x9c9d0746, SUC_lexicon_tags_37 },
    { 0x485d0747, SUC_lexicon_tags_30 },
    { 0x1d5d0748, SUC_lexicon_tags_7 },
    { 0x811d0745, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3d074d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dd0750, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe63d0752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x911d0754, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x437d0757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbd0760, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb0fd0762, SUC_lexicon_tags_19 },
    { 0xfe9d0763, SUC_lexicon_tags_32 },
    { 0x5d7d0763, SUC_lexicon_tags_1 },
    { 0xa8bd0765, SUC_lexicon_tags_2 },
    { 0x869d0762, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd076a, SUC_lexicon_tags_27 },
    { 0x1bbd076a, SUC_lexicon_tags_1 },
    { 0x915d076c, SUC_lexicon_tags_25 },
    { 0xb59d076d, SUC_lexicon_tags_15 },
    { 0x6f3d076e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fd0778, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bd077b, SUC_lexicon_tags_37 },
    { 0xcd1d077b, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfd0784, SUC_lexicon_tags_17 },
    { 0xd39d0785, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdd0788, SUC_lexicon_tags_12 },
    { 0x3dbd0788, SUC_lexicon_tags_12 },
    { 0x2f9d0789, SUC_lexicon_tags_6 },
    { 0x4afd0789, SUC_lexicon_tags_62 },
    { 0xa6dd078b, SUC_lexicon_tags_12 },
    { 0xf3fd0788, SUC_lexicon_tags_40 },
    { 0xc11d078e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbd0790, SUC_lexicon_tags_33 },
    { 0x7bd0790, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169d0795, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3d079b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xec3d079d, SUC_lexicon_tags_63 },
    { 0x1ddd079e, SUC_lexicon_tags_3 },
    { 0x525d079f, SUC_lexicon_tags_32 },
    { 0xc05d07a0, SUC_lexicon_tags_26 },
    { 0x3edd07a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2abd07a3, SUC_lexicon_tags_49 },
    { 0x537d07a4, SUC_lexicon_tags_23 },
    { 0xd91d07a4, SUC_lexicon_tags_17 },
    { 0xc79d07a6, SUC_lexicon_tags_19 },
    { 0x6edd07a6, SUC_lexicon_tags_17 },
    { 0xfdbd07a8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x671d07aa, SUC_lexicon_tags_0 },
    { 0xe73d07aa, SUC_lexicon_tags_17 },
    { 0x607d07ac, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xefbd07ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49d07b1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9d07b4, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xddd07b6, SUC_lexicon_tags_29 },
    { 0x2afd07b7, SUC_lexicon_tags_38 },
    { 0x375d07b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4bdd07ba, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33d07bd, SUC_lexicon_tags_43 },
    { 0x239d07be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3d07c3, SUC_lexicon_tags_37 },
    { 0xdbfd07c4, SUC_lexicon_tags_32 },
    { 0x2a5d07c5, SUC_lexicon_tags_12 },
    { 0xd93d07c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25d07ca, SUC_lexicon_tags_36 },
    { 0x9c7d07cb, SUC_lexicon_tags_7 },
    { 0x3e5d07ca, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75d07d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5d07d4, SUC_lexicon_tags_23 },
    { 0xd73d07d5, SUC_lexicon_tags_15 },
    { 0x981d07d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x47dd07d8, SUC_lexicon_tags_253 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5d07e0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3d07e3, SUC_lexicon_tags_28 },
    { 0x25fd07e3, SUC_lexicon_tags_46 },
    { 0x8d9d07e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd07e8, SUC_lexicon_tags_7 },
    { 0x9c7d07e9, SUC_lexicon_tags_19 },
    { 0x2f9d07ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd07ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93d07f9, SUC_lexicon_tags_37 },
    { 0x43dd07f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddd07ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7d0802, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddd080a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95d080d, SUC_lexicon_tags_42 },
    { 0x865d080e, SUC_lexicon_tags_9 },
    { 0x555d080e, SUC_lexicon_tags_2 },
    { 0x569d080d, SUC_lexicon_tags_7 },
    { 0xa9fd080e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfd0814, SUC_lexicon_tags_33 },
    { 0x4b7d0815, SUC_lexicon_tags_267 },
    { 0, NULL },
    { 0x7f5d0817, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x78fd0819, SUC_lexicon_tags_11 },
    { 0x207d081a, SUC_lexicon_tags_33 },
    { 0x1ddd081b, SUC_lexicon_tags_25 },
    { 0xbabd081c, SUC_lexicon_tags_19 },
    { 0x267d081d, SUC_lexicon_tags_1 },
    { 0x9c1d081e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9d0821, SUC_lexicon_tags_0 },
    { 0x625d0821, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe0fd0824, SUC_lexicon_tags_8 },
    { 0xb73d0824, SUC_lexicon_tags_17 },
    { 0x1a3d0824, SUC_lexicon_tags_43 },
    { 0x2ddd0827, SUC_lexicon_tags_43 },
    { 0xdc5d0828, SUC_lexicon_tags_26 },
    { 0x4bbd0828, SUC_lexicon_tags_25 },
    { 0xeb1d082a, SUC_lexicon_tags_7 },
    { 0xaf3d082a, SUC_lexicon_tags_49 },
    { 0x78dd082b, SUC_lexicon_tags_17 },
    { 0x9cdd082d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5d0830, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41d0834, SUC_lexicon_tags_12 },
    { 0x3d9d0834, SUC_lexicon_tags_15 },
    { 0x829d0834, SUC_lexicon_tags_9 },
    { 0x5c7d0837, SUC_lexicon_tags_65 },
    { 0xda3d0838, SUC_lexicon_tags_37 },
    { 0x2b9d0839, SUC_lexicon_tags_39 },
    { 0x585d0836, SUC_lexicon_tags_11 },
    { 0x37d0835, SUC_lexicon_tags_37 },
    { 0xe53d083c, SUC_lexicon_tags_16 },
    { 0x3d083d, SUC_lexicon_tags_16 },
    { 0xe6dd083a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97d0843, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271d084d, SUC_lexicon_tags_38 },
    { 0x7fbd084d, SUC_lexicon_tags_27 },
    { 0xf0bd084f, SUC_lexicon_tags_37 },
    { 0xc01d0850, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc31d0852, SUC_lexicon_tags_17 },
    { 0xbabd0853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451d0857, SUC_lexicon_tags_19 },
    { 0x67dd0858, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xebbd085a, SUC_lexicon_tags_14 },
    { 0xf8dd085b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891d085f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd0862, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05d0866, SUC_lexicon_tags_7 },
    { 0xe69d0867, SUC_lexicon_tags_14 },
    { 0xea9d0868, SUC_lexicon_tags_35 },
    { 0xf39d0866, SUC_lexicon_tags_25 },
    { 0x98bd086a, SUC_lexicon_tags_26 },
    { 0x74dd0869, SUC_lexicon_tags_63 },
    { 0x4b3d086c, SUC_lexicon_tags_0 },
    { 0xd57d086c, SUC_lexicon_tags_167 },
    { 0xaf3d086d, SUC_lexicon_tags_1 },
    { 0x6a5d086f, SUC_lexicon_tags_23 },
    { 0x989d0867, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfd0876, SUC_lexicon_tags_2 },
    { 0xd4bd0877, SUC_lexicon_tags_7 },
    { 0xdd0877, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51d087c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37dd087f, SUC_lexicon_tags_30 },
    { 0x193d0880, SUC_lexicon_tags_2 },
    { 0xe61d0881, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x149d0883, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141d0888, SUC_lexicon_tags_25 },
    { 0xe79d0888, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95d0890, SUC_lexicon_tags_49 },
    { 0x661d0891, SUC_lexicon_tags_2 },
    { 0x6a3d0892, SUC_lexicon_tags_26 },
    { 0x8a1d0892, SUC_lexicon_tags_91 },
    { 0x45d0891, SUC_lexicon_tags_25 },
    { 0x7a1d0892, SUC_lexicon_tags_30 },
    { 0xe45d0896, SUC_lexicon_tags_11 },
    { 0x5fd0897, SUC_lexicon_tags_19 },
    { 0x717d0898, SUC_lexicon_tags_31 },
    { 0xcd9d0899, SUC_lexicon_tags_11 },
    { 0x9afd0897, SUC_lexicon_tags_12 },
    { 0x1a9d0899, SUC_lexicon_tags_113 },
    { 0x993d089c, SUC_lexicon_tags_29 },
    { 0x24fd089d, SUC_lexicon_tags_15 },
    { 0xc03d089e, SUC_lexicon_tags_11 },
    { 0x7d9d089d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe45d08a2, SUC_lexicon_tags_1 },
    { 0x1e9d08a3, SUC_lexicon_tags_63 },
    { 0x25d08a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe61d08a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333d08ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fd08b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13d08bd, SUC_lexicon_tags_2 },
    { 0xe2bd08be, SUC_lexicon_tags_15 },
    { 0x567d08be, SUC_lexicon_tags_28 },
    { 0x259d08be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfd08c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d1d08c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413d08ca, SUC_lexicon_tags_23 },
    { 0x2a3d08ca, SUC_lexicon_tags_7 },
    { 0xee1d08cc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3d08d1, SUC_lexicon_tags_32 },
    { 0x221d08d1, SUC_lexicon_tags_15 },
    { 0x677d08d3, SUC_lexicon_tags_2 },
    { 0x5e3d08d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddd08d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845d08e3, SUC_lexicon_tags_15 },
    { 0x749d08e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe07d08e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d08e9, SUC_lexicon_tags_12 },
    { 0xbc9d08ea, SUC_lexicon_tags_27 },
    { 0x64dd08eb, SUC_lexicon_tags_17 },
    { 0x127d08eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x35fd08ee, SUC_lexicon_tags_37 },
    { 0xd67d08ef, SUC_lexicon_tags_38 },
    { 0x9fdd08f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1d08f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fd08f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fd08fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d08ff, SUC_lexicon_tags_17 },
    { 0xaf3d0900, SUC_lexicon_tags_9 },
    { 0x2f5d0900, SUC_lexicon_tags_15 },
    { 0x311d0900, SUC_lexicon_tags_27 },
    { 0xb17d0903, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeb7d0905, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x745d0907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x495d0909, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x893d090c, SUC_lexicon_tags_37 },
    { 0x4edd090c, SUC_lexicon_tags_6 },
    { 0xc11d090c, SUC_lexicon_tags_25 },
    { 0x3afd090d, SUC_lexicon_tags_23 },
    { 0x3cdd090e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x719d0913, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fd0917, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x525d0919, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bd091c, SUC_lexicon_tags_14 },
    { 0xdbd091d, SUC_lexicon_tags_12 },
    { 0x765d091e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbebd0920, SUC_lexicon_tags_32 },
    { 0x3dd0921, SUC_lexicon_tags_8 },
    { 0xcc5d0921, SUC_lexicon_tags_7 },
    { 0xaf3d0923, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419d0928, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x611d092a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dd092e, SUC_lexicon_tags_7 },
    { 0xa9dd092f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51d0932, SUC_lexicon_tags_55 },
    { 0xb63d0933, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41d0936, SUC_lexicon_tags_49 },
    { 0x911d0936, SUC_lexicon_tags_12 },
    { 0x89bd0938, SUC_lexicon_tags_19 },
    { 0x6bdd0939, SUC_lexicon_tags_2 },
    { 0xb1dd0938, SUC_lexicon_tags_66 },
    { 0xcb5d093a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d0940, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3d0948, SUC_lexicon_tags_15 },
    { 0xf79d0948, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917d0951, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dd095a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dd0960, SUC_lexicon_tags_7 },
    { 0xf23d0961, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90dd0964, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dd0968, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95d096b, SUC_lexicon_tags_7 },
    { 0x479d096b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fd0972, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d0978, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe01d097a, SUC_lexicon_tags_1 },
    { 0xdb1d097b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x95fd097d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7d0982, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x10fd0984, SUC_lexicon_tags_27 },
    { 0x341d0985, SUC_lexicon_tags_25 },
    { 0xd57d0984, SUC_lexicon_tags_56 },
    { 0xbb1d0985, SUC_lexicon_tags_68 },
    { 0x905d0988, SUC_lexicon_tags_16 },
    { 0x331d0986, SUC_lexicon_tags_17 },
    { 0xb8dd0987, SUC_lexicon_tags_26 },
    { 0xac1d0986, SUC_lexicon_tags_11 },
    { 0xd5dd098c, SUC_lexicon_tags_1 },
    { 0xb75d0989, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15d0991, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3d0998, SUC_lexicon_tags_27 },
    { 0x6ffd0998, SUC_lexicon_tags_15 },
    { 0x29fd099a, SUC_lexicon_tags_34 },
    { 0x2f1d099b, SUC_lexicon_tags_37 },
    { 0x35fd099b, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bd099f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bd09a2, SUC_lexicon_tags_25 },
    { 0x803d09a3, SUC_lexicon_tags_7 },
    { 0x87d09a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x279d09a7, SUC_lexicon_tags_12 },
    { 0x29bd09a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739d09ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9d09af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327d09b3, SUC_lexicon_tags_71 },
    { 0xe1bd09b3, SUC_lexicon_tags_19 },
    { 0xc51d09b5, SUC_lexicon_tags_25 },
    { 0xdf7d09b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x301d09b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805d09bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9d09c0, SUC_lexicon_tags_12 },
    { 0x49dd09c0, SUC_lexicon_tags_11 },
    { 0x3a1d09c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbd09c5, SUC_lexicon_tags_11 },
    { 0x8f9d09c6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbd09cc, SUC_lexicon_tags_29 },
    { 0x483d09cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa11d09cf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd8dd09d1, SUC_lexicon_tags_34 },
    { 0xbb7d09d2, SUC_lexicon_tags_11 },
    { 0x129d09d3, SUC_lexicon_tags_23 },
    { 0x547d09d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d09d8, SUC_lexicon_tags_49 },
    { 0xe6dd09d9, SUC_lexicon_tags_68 },
    { 0x14fd09d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1d09e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfdfd09e4, SUC_lexicon_tags_25 },
    { 0xc83d09e5, SUC_lexicon_tags_28 },
    { 0x815d09e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x759d09e9, SUC_lexicon_tags_48 },
    { 0x705d09ea, SUC_lexicon_tags_26 },
    { 0xb5dd09eb, SUC_lexicon_tags_2 },
    { 0x8f1d09e9, SUC_lexicon_tags_29 },
    { 0x465d09ed, SUC_lexicon_tags_7 },
    { 0xa71d09ee, SUC_lexicon_tags_2 },
    { 0xf29d09ef, SUC_lexicon_tags_47 },
    { 0x43d09ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24bd09f2, SUC_lexicon_tags_29 },
    { 0xcf3d09f3, SUC_lexicon_tags_37 },
    { 0xc15d09f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5d09fb, SUC_lexicon_tags_15 },
    { 0x289d09fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89bd09fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffd0a01, SUC_lexicon_tags_4 },
    { 0xac7d0a02, SUC_lexicon_tags_39 },
    { 0xdf5d0a03, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x137d0a05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1d0a08, SUC_lexicon_tags_84 },
    { 0x15fd0a09, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bd0a0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4abd0a0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63dd0a11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x87fd0a13, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x553d0a15, SUC_lexicon_tags_23 },
    { 0xf47d0a16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875d0a1a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9d0a22, SUC_lexicon_tags_61 },
    { 0xe21d0a22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51d0a26, SUC_lexicon_tags_63 },
    { 0x8b7d0a27, SUC_lexicon_tags_43 },
    { 0x875d0a28, SUC_lexicon_tags_14 },
    { 0x617d0a29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa0bd0a2b, SUC_lexicon_tags_10 },
    { 0xbafd0a2c, SUC_lexicon_tags_37 },
    { 0x39bd0a2b, SUC_lexicon_tags_9 },
    { 0x37bd0a2c, SUC_lexicon_tags_11 },
    { 0x7b3d0a2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfd0a33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x513d0a35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91d0a39, SUC_lexicon_tags_37 },
    { 0x7e1d0a39, SUC_lexicon_tags_37 },
    { 0xe3fd0a3b, SUC_lexicon_tags_63 },
    { 0xa1bd0a3c, SUC_lexicon_tags_25 },
    { 0x489d0a39, SUC_lexicon_tags_12 },
    { 0xeefd0a3e, SUC_lexicon_tags_2 },
    { 0xa5d0a3f, SUC_lexicon_tags_12 },
    { 0x489d0a40, SUC_lexicon_tags_27 },
    { 0xdfd0a3f, SUC_lexicon_tags_1 },
    { 0xa15d0a42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147d0a47, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x58dd0a49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7d0a4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43d0a52, SUC_lexicon_tags_31 },
    { 0xf35d0a53, SUC_lexicon_tags_29 },
    { 0xfe9d0a54, SUC_lexicon_tags_15 },
    { 0x879d0a54, SUC_lexicon_tags_25 },
    { 0x69d0a54, SUC_lexicon_tags_405 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d0a5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc11d0a5f, SUC_lexicon_tags_15 },
    { 0x51dd0a60, SUC_lexicon_tags_6 },
    { 0xe93d0a60, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965d0a66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcfdd0a68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759d0a6c, SUC_lexicon_tags_25 },
    { 0x875d0a6c, SUC_lexicon_tags_25 },
    { 0x8a3d0a6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7d0a72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1d0a77, SUC_lexicon_tags_14 },
    { 0x951d0a77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa29d0a7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf1d0a7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x269d0a7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x60dd0a80, SUC_lexicon_tags_17 },
    { 0xa91d0a81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645d0a85, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81d0a8a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x813d0a8c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x603d0a8e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfd9d0a90, SUC_lexicon_tags_25 },
    { 0x319d0a91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d9d0a93, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xabfd0a96, SUC_lexicon_tags_1 },
    { 0xcebd0a97, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf67d0a99, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa7fd0a9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfd0a9f, SUC_lexicon_tags_55 },
    { 0x957d0aa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdfd0aa2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fd0aa5, SUC_lexicon_tags_23 },
    { 0xfbd0aa6, SUC_lexicon_tags_10 },
    { 0x941d0aa6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95d0aad, SUC_lexicon_tags_45 },
    { 0xa7fd0aae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8dd0ab0, SUC_lexicon_tags_46 },
    { 0x129d0ab1, SUC_lexicon_tags_49 },
    { 0xe55d0ab0, SUC_lexicon_tags_3 },
    { 0x5afd0ab3, SUC_lexicon_tags_37 },
    { 0xa37d0ab4, SUC_lexicon_tags_2 },
    { 0x77fd0ab4, SUC_lexicon_tags_7 },
    { 0x375d0ab2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x461d0ab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x281d0aba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x123d0abc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x289d0abf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55d0ac6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc1d0ac8, SUC_lexicon_tags_15 },
    { 0x911d0ac9, SUC_lexicon_tags_26 },
    { 0x185d0ac9, SUC_lexicon_tags_63 },
    { 0x2f3d0aca, SUC_lexicon_tags_27 },
    { 0x26dd0acb, SUC_lexicon_tags_45 },
    { 0x421d0acd, SUC_lexicon_tags_7 },
    { 0x697d0ace, SUC_lexicon_tags_6 },
    { 0x649d0acf, SUC_lexicon_tags_11 },
    { 0x5d0ad0, SUC_lexicon_tags_11 },
    { 0xea7d0ad1, SUC_lexicon_tags_7 },
    { 0xd3bd0ad2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa51d0ad4, SUC_lexicon_tags_27 },
    { 0x357d0ad4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dd0ad9, SUC_lexicon_tags_27 },
    { 0x81bd0ad9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bd0ae1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bd0ae4, SUC_lexicon_tags_0 },
    { 0x92bd0ae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1d0ae9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9d0af0, SUC_lexicon_tags_25 },
    { 0xd4dd0af1, SUC_lexicon_tags_23 },
    { 0xae9d0af1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a1d0af4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25d0af9, SUC_lexicon_tags_25 },
    { 0x9dd0afa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf41d0afc, SUC_lexicon_tags_40 },
    { 0xba9d0afc, SUC_lexicon_tags_7 },
    { 0x72bd0afe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d1d0b00, SUC_lexicon_tags_15 },
    { 0xa6dd0b01, SUC_lexicon_tags_43 },
    { 0xfdfd0b02, SUC_lexicon_tags_87 },
    { 0x8fd0b02, SUC_lexicon_tags_2 },
    { 0xb15d0b04, SUC_lexicon_tags_28 },
    { 0xa29d0b01, SUC_lexicon_tags_11 },
    { 0x40fd0b05, SUC_lexicon_tags_31 },
    { 0x37dd0b03, SUC_lexicon_tags_77 },
    { 0x273d0b03, SUC_lexicon_tags_6 },
    { 0xb75d0b00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f3d0b0b, SUC_lexicon_tags_11 },
    { 0x76bd0b0b, SUC_lexicon_tags_37 },
    { 0x3d9d0b0d, SUC_lexicon_tags_37 },
    { 0x617d0b0e, SUC_lexicon_tags_79 },
    { 0x6ebd0b0e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x43dd0b12, SUC_lexicon_tags_46 },
    { 0xe71d0b13, SUC_lexicon_tags_37 },
    { 0x225d0b13, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3d0b18, SUC_lexicon_tags_7 },
    { 0x393d0b18, SUC_lexicon_tags_103 },
    { 0x4ebd0b1a, SUC_lexicon_tags_38 },
    { 0xcd5d0b1b, SUC_lexicon_tags_38 },
    { 0xf87d0b1c, SUC_lexicon_tags_17 },
    { 0xd21d0b1d, SUC_lexicon_tags_15 },
    { 0xe2dd0b1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31d0b25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9d0b2c, SUC_lexicon_tags_12 },
    { 0x66bd0b2d, SUC_lexicon_tags_6 },
    { 0xbcdd0b2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x817d0b30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25d0b33, SUC_lexicon_tags_60 },
    { 0x5ebd0b34, SUC_lexicon_tags_13 },
    { 0x673d0b35, SUC_lexicon_tags_38 },
    { 0xa9bd0b36, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1d0b39, SUC_lexicon_tags_29 },
    { 0xf01d0b39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9d0b48, SUC_lexicon_tags_12 },
    { 0xf81d0b49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbd0b4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3d0b4f, SUC_lexicon_tags_26 },
    { 0xba7d0b50, SUC_lexicon_tags_46 },
    { 0x169d0b51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61d0b55, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf59d0b57, SUC_lexicon_tags_60 },
    { 0x27dd0b58, SUC_lexicon_tags_2 },
    { 0x58fd0b59, SUC_lexicon_tags_26 },
    { 0x553d0b5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfd0b60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdd0b64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99d0b68, SUC_lexicon_tags_82 },
    { 0xacbd0b68, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d0b6f, SUC_lexicon_tags_4 },
    { 0xd1d0b70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9d0b75, SUC_lexicon_tags_6 },
    { 0xc1d0b76, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fd0b7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x761d0b7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3d0b81, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb9d0b83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25d0b87, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdd0b8d, SUC_lexicon_tags_45 },
    { 0x4b5d0b8e, SUC_lexicon_tags_21 },
    { 0x56dd0b8f, SUC_lexicon_tags_49 },
    { 0x91fd0b90, SUC_lexicon_tags_11 },
    { 0xa7d0b8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd0b94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x965d0b97, SUC_lexicon_tags_28 },
    { 0x99fd0b98, SUC_lexicon_tags_15 },
    { 0x837d0b99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1d0b9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dd0bad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xba1d0baf, SUC_lexicon_tags_60 },
    { 0xb31d0baf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bd0bb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5d0bbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x703d0bbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9d0bc5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bd0bc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d0bcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3d0bd1, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fd0bd9, SUC_lexicon_tags_13 },
    { 0x717d0bda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x64bd0bdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1d0be4, SUC_lexicon_tags_0 },
    { 0x7b1d0be4, SUC_lexicon_tags_79 },
    { 0xea7d0be6, SUC_lexicon_tags_15 },
    { 0xd03d0be7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655d0bec, SUC_lexicon_tags_37 },
    { 0x987d0bec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9b1d0bef, SUC_lexicon_tags_15 },
    { 0xea7d0bf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1d0bf4, SUC_lexicon_tags_11 },
    { 0xf7dd0bf4, SUC_lexicon_tags_1 },
    { 0xb03d0bf4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x409d0bf9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7e5d0bfb, SUC_lexicon_tags_17 },
    { 0x2f3d0bfc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17d0bff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3d0c02, SUC_lexicon_tags_17 },
    { 0x805d0c03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fd0c08, SUC_lexicon_tags_12 },
    { 0x9c7d0c09, SUC_lexicon_tags_15 },
    { 0x77dd0c0a, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0xaa1d0c0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x849d0c0f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa97d0c11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4edd0c13, SUC_lexicon_tags_0 },
    { 0x731d0c14, SUC_lexicon_tags_15 },
    { 0x803d0c13, SUC_lexicon_tags_47 },
    { 0xa07d0c13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65d0c23, SUC_lexicon_tags_47 },
    { 0xc6bd0c23, SUC_lexicon_tags_63 },
    { 0xf33d0c25, SUC_lexicon_tags_47 },
    { 0xc97d0c25, SUC_lexicon_tags_0 },
    { 0x6bbd0c24, SUC_lexicon_tags_37 },
    { 0x8c3d0c26, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbd0c2c, SUC_lexicon_tags_6 },
    { 0xce3d0c2d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd0bd0c2f, SUC_lexicon_tags_30 },
    { 0x553d0c2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe9d0c32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x103d0c34, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45d0c37, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xad5d0c39, SUC_lexicon_tags_45 },
    { 0x7bd0c3a, SUC_lexicon_tags_25 },
    { 0x333d0c3b, SUC_lexicon_tags_38 },
    { 0xa89d0c3b, SUC_lexicon_tags_17 },
    { 0x795d0c3a, SUC_lexicon_tags_2 },
    { 0x3a5d0c3b, SUC_lexicon_tags_57 },
    { 0x543d0c3b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01d0c42, SUC_lexicon_tags_25 },
    { 0x589d0c43, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6a9d0c45, SUC_lexicon_tags_14 },
    { 0x5e9d0c46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb1d0c48, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x16bd0c4a, SUC_lexicon_tags_47 },
    { 0x955d0c4a, SUC_lexicon_tags_12 },
    { 0x83d0c4c, SUC_lexicon_tags_19 },
    { 0x219d0c4b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9ddd0c4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4bd0c51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcabd0c53, SUC_lexicon_tags_12 },
    { 0x45dd0c53, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263d0c59, SUC_lexicon_tags_11 },
    { 0x451d0c5a, SUC_lexicon_tags_55 },
    { 0xc8bd0c5b, SUC_lexicon_tags_15 },
    { 0xc9bd0c5b, SUC_lexicon_tags_43 },
    { 0xa3fd0c59, SUC_lexicon_tags_27 },
    { 0xda5d0c5e, SUC_lexicon_tags_10 },
    { 0xb77d0c5e, SUC_lexicon_tags_15 },
    { 0xdb1d0c5a, SUC_lexicon_tags_11 },
    { 0xa87d0c59, SUC_lexicon_tags_7 },
    { 0x453d0c5a, SUC_lexicon_tags_37 },
    { 0x603d0c63, SUC_lexicon_tags_62 },
    { 0x2ffd0c64, SUC_lexicon_tags_17 },
    { 0xd47d0c5b, SUC_lexicon_tags_37 },
    { 0x679d0c5c, SUC_lexicon_tags_14 },
    { 0xca5d0c67, SUC_lexicon_tags_9 },
    { 0xea7d0c62, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d0c70, SUC_lexicon_tags_179 },
    { 0xd4fd0c71, SUC_lexicon_tags_7 },
    { 0xf9fd0c71, SUC_lexicon_tags_12 },
    { 0xdffd0c72, SUC_lexicon_tags_2 },
    { 0xa11d0c74, SUC_lexicon_tags_19 },
    { 0x91bd0c75, SUC_lexicon_tags_37 },
    { 0xef3d0c76, SUC_lexicon_tags_0 },
    { 0x3afd0c77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bd0c7c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x1bbd0c7e, SUC_lexicon_tags_43 },
    { 0xb4dd0c7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d0c82, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fd0c85, SUC_lexicon_tags_0 },
    { 0xd9dd0c85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5d0c89, SUC_lexicon_tags_7 },
    { 0x541d0c8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bd0c8d, SUC_lexicon_tags_45 },
    { 0x835d0c8e, SUC_lexicon_tags_37 },
    { 0x4d5d0c8f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x961d0c92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbd0c9a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe13d0c9c, SUC_lexicon_tags_49 },
    { 0xad1d0c9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd0ca1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3e9d0ca3, SUC_lexicon_tags_79 },
    { 0xbc1d0ca4, SUC_lexicon_tags_30 },
    { 0x9c1d0ca3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179d0cab, SUC_lexicon_tags_37 },
    { 0xb67d0cac, SUC_lexicon_tags_17 },
    { 0x205d0cad, SUC_lexicon_tags_37 },
    { 0x767d0cad, SUC_lexicon_tags_33 },
    { 0xd37d0cad, SUC_lexicon_tags_1 },
    { 0xf4fd0cb0, SUC_lexicon_tags_17 },
    { 0xaa5d0cb1, SUC_lexicon_tags_17 },
    { 0x3dd0cb1, SUC_lexicon_tags_49 },
    { 0xfd1d0caf, SUC_lexicon_tags_17 },
    { 0x333d0cae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x61fd0cb6, SUC_lexicon_tags_4 },
    { 0xc71d0cb7, SUC_lexicon_tags_23 },
    { 0x355d0cb7, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x91d0cba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48bd0cbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37d0cc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebd0cc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abd0cc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbd0ccd, SUC_lexicon_tags_15 },
    { 0x70bd0cce, SUC_lexicon_tags_33 },
    { 0xb73d0ccf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bd0cd2, SUC_lexicon_tags_60 },
    { 0xe2fd0cd3, SUC_lexicon_tags_17 },
    { 0xa55d0cd2, SUC_lexicon_tags_15 },
    { 0x45dd0cd5, SUC_lexicon_tags_55 },
    { 0x285d0cd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x691d0cd8, SUC_lexicon_tags_43 },
    { 0x93bd0cd8, SUC_lexicon_tags_25 },
    { 0x963d0cda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abd0cdd, SUC_lexicon_tags_11 },
    { 0xc0dd0cde, SUC_lexicon_tags_45 },
    { 0x17dd0cdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69d0ce9, SUC_lexicon_tags_6 },
    { 0xc9d0cea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x199d0ced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d0cf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51d0cf5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd0cfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x693d0d01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93d0d07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x353d0d0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee9d0d0c, SUC_lexicon_tags_46 },
    { 0x22dd0d0d, SUC_lexicon_tags_15 },
    { 0x525d0d0d, SUC_lexicon_tags_22 },
    { 0xcb1d0d0e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31d0d13, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9d0d17, SUC_lexicon_tags_47 },
    { 0xcc3d0d18, SUC_lexicon_tags_11 },
    { 0x95fd0d18, SUC_lexicon_tags_69 },
    { 0xa91d0d17, SUC_lexicon_tags_23 },
    { 0xfcbd0d1b, SUC_lexicon_tags_37 },
    { 0xc1fd0d1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3d0d1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf87d0d21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x299d0d23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1d0d29, SUC_lexicon_tags_26 },
    { 0x24dd0d2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2bbd0d2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5d0d2f, SUC_lexicon_tags_37 },
    { 0xd5dd0d30, SUC_lexicon_tags_25 },
    { 0xfb9d0d31, SUC_lexicon_tags_2 },
    { 0x2d3d0d31, SUC_lexicon_tags_77 },
    { 0x1c9d0d33, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735d0d37, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdd0d3e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fd0d42, SUC_lexicon_tags_27 },
    { 0x6b5d0d42, SUC_lexicon_tags_17 },
    { 0xc0dd0d42, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde1d0d46, SUC_lexicon_tags_28 },
    { 0x7bdd0d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77d0d4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd0d4f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3d0d52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb15d0d54, SUC_lexicon_tags_25 },
    { 0xed5d0d54, SUC_lexicon_tags_23 },
    { 0x15fd0d55, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1d0d59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5d0d5c, SUC_lexicon_tags_17 },
    { 0x159d0d5d, SUC_lexicon_tags_37 },
    { 0x1a1d0d5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653d0d63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcdd0d65, SUC_lexicon_tags_3 },
    { 0x9c1d0d66, SUC_lexicon_tags_11 },
    { 0x985d0d65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707d0d6b, SUC_lexicon_tags_23 },
    { 0xca3d0d6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d0d6f, SUC_lexicon_tags_66 },
    { 0x97dd0d70, SUC_lexicon_tags_37 },
    { 0x4d3d0d71, SUC_lexicon_tags_38 },
    { 0x395d0d71, SUC_lexicon_tags_0 },
    { 0x57bd0d72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505d0d7c, SUC_lexicon_tags_7 },
    { 0x60dd0d7d, SUC_lexicon_tags_11 },
    { 0xc6bd0d7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xccfd0d80, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831d0d86, SUC_lexicon_tags_8 },
    { 0xdf3d0d87, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5bbd0d89, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbd0d90, SUC_lexicon_tags_1 },
    { 0x1fdd0d91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8abd0d93, SUC_lexicon_tags_421 },
    { 0xb29d0d93, SUC_lexicon_tags_56 },
    { 0x93fd0d95, SUC_lexicon_tags_26 },
    { 0x69d0d96, SUC_lexicon_tags_3 },
    { 0xc25d0d94, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bd0d9b, SUC_lexicon_tags_12 },
    { 0x3a5d0d9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xffbd0d9f, SUC_lexicon_tags_29 },
    { 0x739d0d9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x727d0da3, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0x38fd0da6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7d0da9, SUC_lexicon_tags_31 },
    { 0xb99d0daa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d0dad, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293d0dc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5d0dc7, SUC_lexicon_tags_7 },
    { 0x53dd0dc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54dd0dcc, SUC_lexicon_tags_45 },
    { 0xad1d0dcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7d0dd3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1f1d0dd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x34dd0dd8, SUC_lexicon_tags_15 },
    { 0x4b3d0dd9, SUC_lexicon_tags_91 },
    { 0x3c7d0dd9, SUC_lexicon_tags_14 },
    { 0xc3fd0ddb, SUC_lexicon_tags_14 },
    { 0xa01d0dda, SUC_lexicon_tags_92 },
    { 0x667d0ddc, SUC_lexicon_tags_37 },
    { 0x4abd0ddc, SUC_lexicon_tags_17 },
    { 0x115d0dd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d0de5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fd0def, SUC_lexicon_tags_37 },
    { 0x7a5d0df0, SUC_lexicon_tags_25 },
    { 0x9abd0df1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541d0df6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d0df9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf53d0dfb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xcfd0dfd, SUC_lexicon_tags_12 },
    { 0x5bdd0dfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21d0e00, SUC_lexicon_tags_15 },
    { 0x397d0e01, SUC_lexicon_tags_43 },
    { 0x785d0e02, SUC_lexicon_tags_17 },
    { 0xdd9d0e03, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221d0e09, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29d0e0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3d0e11, SUC_lexicon_tags_33 },
    { 0x29dd0e12, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993d0e1a, SUC_lexicon_tags_7 },
    { 0xf29d0e1b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x795d0e1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd0e26, SUC_lexicon_tags_11 },
    { 0xa1dd0e27, SUC_lexicon_tags_4 },
    { 0xdb5d0e28, SUC_lexicon_tags_23 },
    { 0xd4fd0e28, SUC_lexicon_tags_21 },
    { 0x92dd0e27, SUC_lexicon_tags_7 },
    { 0x45d0e29, SUC_lexicon_tags_30 },
    { 0x253d0e2c, SUC_lexicon_tags_65 },
    { 0x905d0e27, SUC_lexicon_tags_7 },
    { 0x9b3d0e2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7d0e39, SUC_lexicon_tags_16 },
    { 0x751d0e3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x36bd0e3c, SUC_lexicon_tags_29 },
    { 0xff1d0e3d, SUC_lexicon_tags_69 },
    { 0xae3d0e3c, SUC_lexicon_tags_12 },
    { 0xc3bd0e3f, SUC_lexicon_tags_15 },
    { 0xbd9d0e40, SUC_lexicon_tags_15 },
    { 0xdf9d0e41, SUC_lexicon_tags_25 },
    { 0xc0bd0e42, SUC_lexicon_tags_9 },
    { 0xf45d0e43, SUC_lexicon_tags_11 },
    { 0xfbfd0e44, SUC_lexicon_tags_15 },
    { 0x5a5d0e40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663d0e4b, SUC_lexicon_tags_2 },
    { 0x1d0e4b, SUC_lexicon_tags_64 },
    { 0x97d0e4d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187d0e51, SUC_lexicon_tags_47 },
    { 0xcadd0e52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d0e59, SUC_lexicon_tags_35 },
    { 0xbe9d0e59, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7d0e5f, SUC_lexicon_tags_29 },
    { 0x501d0e5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637d0e64, SUC_lexicon_tags_2 },
    { 0x49d0e65, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd47d0e67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x885d0e69, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7d0e6c, SUC_lexicon_tags_37 },
    { 0xbdd0e6d, SUC_lexicon_tags_2 },
    { 0xc01d0e6d, SUC_lexicon_tags_1 },
    { 0xc99d0e6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d0e72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x275d0e75, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x277d0e77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfd0e7b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dd0e81, SUC_lexicon_tags_49 },
    { 0x855d0e81, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afd0e89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9d0e8d, SUC_lexicon_tags_17 },
    { 0xd3bd0e8e, SUC_lexicon_tags_11 },
    { 0xd97d0e8f, SUC_lexicon_tags_18 },
    { 0x889d0e90, SUC_lexicon_tags_1 },
    { 0x93d0e91, SUC_lexicon_tags_45 },
    { 0x4b9d0e8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbd0e98, SUC_lexicon_tags_15 },
    { 0x3add0e98, SUC_lexicon_tags_39 },
    { 0x5b9d0e9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7d0e9d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dd0ea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5d0ea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x57d0ea7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567d0eac, SUC_lexicon_tags_15 },
    { 0xf53d0ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7d0eb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9efd0eb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd0eb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fd0eba, SUC_lexicon_tags_13 },
    { 0x2add0ebb, SUC_lexicon_tags_11 },
    { 0x4add0ebc, SUC_lexicon_tags_25 },
    { 0x43bd0ebd, SUC_lexicon_tags_40 },
    { 0x247d0ebe, SUC_lexicon_tags_31 },
    { 0xc27d0ebf, SUC_lexicon_tags_31 },
    { 0x34dd0ebf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbf3d0ec2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1d0ec6, SUC_lexicon_tags_10 },
    { 0x211d0ec7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dd0ecb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1d0ece, SUC_lexicon_tags_14 },
    { 0x995d0ece, SUC_lexicon_tags_282 },
    { 0x5ffd0ecf, SUC_lexicon_tags_30 },
    { 0x25dd0ed1, SUC_lexicon_tags_77 },
    { 0xa9dd0ece, SUC_lexicon_tags_2 },
    { 0x12fd0ed3, SUC_lexicon_tags_47 },
    { 0x567d0ed4, SUC_lexicon_tags_56 },
    { 0x26bd0ed1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327d0eda, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dd0edd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bd0ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25d0eec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbd0eef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa1d0ef1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddd0ef6, SUC_lexicon_tags_30 },
    { 0xa1fd0ef6, SUC_lexicon_tags_7 },
    { 0xdc9d0ef7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbd0efb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdd0f01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4add0f04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf9d0f06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8bbd0f08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfd0f0c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9d0f11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dd0f19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb3d0f1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95d0f21, SUC_lexicon_tags_0 },
    { 0x5e7d0f22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46fd0f24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xaf9d0f26, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495d0f2e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x68fd0f31, SUC_lexicon_tags_6 },
    { 0x78dd0f32, SUC_lexicon_tags_46 },
    { 0x7ebd0f33, SUC_lexicon_tags_16 },
    { 0xf9bd0f33, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dd0f3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58bd0f3d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x563d0f3f, SUC_lexicon_tags_63 },
    { 0x16fd0f40, SUC_lexicon_tags_25 },
    { 0xaddd0f41, SUC_lexicon_tags_1 },
    { 0x711d0f41, SUC_lexicon_tags_19 },
    { 0x5d7d0f3f, SUC_lexicon_tags_30 },
    { 0x62fd0f40, SUC_lexicon_tags_60 },
    { 0xeb1d0f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edd0f48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b1d0f4a, SUC_lexicon_tags_7 },
    { 0x4bdd0f4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x781d0f4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc65d0f50, SUC_lexicon_tags_14 },
    { 0x3cfd0f51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3d0f56, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1e3d0f58, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dd0f5b, SUC_lexicon_tags_35 },
    { 0x469d0f5c, SUC_lexicon_tags_25 },
    { 0x715d0f5d, SUC_lexicon_tags_15 },
    { 0x59bd0f5c, SUC_lexicon_tags_30 },
    { 0x5bfd0f5b, SUC_lexicon_tags_11 },
    { 0xfcdd0f60, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x499d0f63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bd0f69, SUC_lexicon_tags_7 },
    { 0x507d0f69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x321d0f6c, SUC_lexicon_tags_17 },
    { 0xbbdd0f6d, SUC_lexicon_tags_17 },
    { 0x577d0f6d, SUC_lexicon_tags_61 },
    { 0x559d0f6f, SUC_lexicon_tags_65 },
    { 0x881d0f70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5d0f77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d5d0f79, SUC_lexicon_tags_15 },
    { 0x4fd0f7a, SUC_lexicon_tags_31 },
    { 0x455d0f7b, SUC_lexicon_tags_11 },
    { 0x131d0f7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b3d0f7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5d0f82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7d0f85, SUC_lexicon_tags_64 },
    { 0x9dfd0f85, SUC_lexicon_tags_15 },
    { 0xfb3d0f87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x571d0f89, SUC_lexicon_tags_17 },
    { 0x939d0f89, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fd0f90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x68dd0f93, SUC_lexicon_tags_8 },
    { 0x95dd0f94, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x983d0f97, SUC_lexicon_tags_49 },
    { 0xf1bd0f98, SUC_lexicon_tags_30 },
    { 0xadd0f97, SUC_lexicon_tags_23 },
    { 0x431d0f9a, SUC_lexicon_tags_25 },
    { 0x2b5d0f9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5d0fa5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35d0fa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7f7d0faa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd0fad, SUC_lexicon_tags_2 },
    { 0xf5dd0fae, SUC_lexicon_tags_17 },
    { 0x605d0faf, SUC_lexicon_tags_15 },
    { 0x1b1d0faf, SUC_lexicon_tags_7 },
    { 0x771d0fad, SUC_lexicon_tags_37 },
    { 0xa0fd0fb2, SUC_lexicon_tags_61 },
    { 0x8a7d0fb2, SUC_lexicon_tags_160 },
    { 0x3fbd0fae, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9d0fb7, SUC_lexicon_tags_1 },
    { 0xf59d0fb7, SUC_lexicon_tags_27 },
    { 0xed5d0fb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfc7d0fbb, SUC_lexicon_tags_27 },
    { 0x96bd0fbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x949d0fbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fd0fc3, SUC_lexicon_tags_32 },
    { 0x973d0fc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x131d0fc6, SUC_lexicon_tags_42 },
    { 0x451d0fc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d0fcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dd0fd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21dd0fd7, SUC_lexicon_tags_15 },
    { 0x5fbd0fd8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bd0fde, SUC_lexicon_tags_12 },
    { 0x907d0fdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd69d0fe1, SUC_lexicon_tags_12 },
    { 0x775d0fe1, SUC_lexicon_tags_17 },
    { 0x7edd0fe1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1d0fe6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedd0fe9, SUC_lexicon_tags_7 },
    { 0x66bd0fea, SUC_lexicon_tags_16 },
    { 0x32bd0fea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1d0ff1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1d0ff5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b5d0ff7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x703d0ffa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6fd0ffc, SUC_lexicon_tags_15 },
    { 0xaf7d0ffc, SUC_lexicon_tags_25 },
    { 0x7f5d0ffd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x819d1001, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669d1005, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x171d1008, SUC_lexicon_tags_2 },
    { 0x707d1009, SUC_lexicon_tags_12 },
    { 0xb43d100a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc29d100c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bd1012, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdd1d1014, SUC_lexicon_tags_35 },
    { 0xbc3d1015, SUC_lexicon_tags_1594 },
    { 0x1edd1016, SUC_lexicon_tags_17 },
    { 0xf3dd1017, SUC_lexicon_tags_15 },
    { 0xd4dd1017, SUC_lexicon_tags_17 },
    { 0xe15d1018, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331d1021, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x85d1024, SUC_lexicon_tags_25 },
    { 0x455d1025, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebd1028, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d102c, SUC_lexicon_tags_33 },
    { 0x94dd102c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x629d1030, SUC_lexicon_tags_68 },
    { 0x64bd1031, SUC_lexicon_tags_17 },
    { 0xde3d1032, SUC_lexicon_tags_17 },
    { 0x795d1032, SUC_lexicon_tags_33 },
    { 0xc71d1034, SUC_lexicon_tags_8 },
    { 0x9abd1034, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x7f9d1037, SUC_lexicon_tags_37 },
    { 0xcadd1038, SUC_lexicon_tags_17 },
    { 0xf43d1039, SUC_lexicon_tags_80 },
    { 0x105d103a, SUC_lexicon_tags_63 },
    { 0x603d1039, SUC_lexicon_tags_23 },
    { 0x981d103c, SUC_lexicon_tags_148 },
    { 0xd9d1038, SUC_lexicon_tags_37 },
    { 0x3add103e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19d1046, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x4d7d1048, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xd81d104a, SUC_lexicon_tags_7 },
    { 0xa07d104b, SUC_lexicon_tags_9 },
    { 0x4fbd104b, SUC_lexicon_tags_37 },
    { 0x593d104b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fd1051, SUC_lexicon_tags_7 },
    { 0x5cbd1051, SUC_lexicon_tags_12 },
    { 0xee9d1053, SUC_lexicon_tags_21 },
    { 0xef5d1052, SUC_lexicon_tags_11 },
    { 0xf7bd1052, SUC_lexicon_tags_62 },
    { 0xd5bd1056, SUC_lexicon_tags_38 },
    { 0x6bfd1057, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7b1d1059, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fd105d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395d1065, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73d1068, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x519d106a, SUC_lexicon_tags_37 },
    { 0xc8dd106a, SUC_lexicon_tags_435 },
    { 0, NULL },
    { 0, NULL },
    { 0x941d106e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdd1074, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb45d1077, SUC_lexicon_tags_26 },
    { 0xb2dd1077, SUC_lexicon_tags_25 },
    { 0xa27d1079, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf59d107c, SUC_lexicon_tags_33 },
    { 0x2c3d107d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c5d107f, SUC_lexicon_tags_37 },
    { 0x263d1080, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253d1088, SUC_lexicon_tags_66 },
    { 0x49dd1089, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5f5d108b, SUC_lexicon_tags_3 },
    { 0xd4bd108c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75d1091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97d1094, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745d10a4, SUC_lexicon_tags_37 },
    { 0x6c7d10a5, SUC_lexicon_tags_40 },
    { 0xe7dd10a6, SUC_lexicon_tags_30 },
    { 0xf91d10a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc71d10a9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4bfd10ab, SUC_lexicon_tags_6 },
    { 0x385d10ac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb41d10ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fd10b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17bd10b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fd10ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x227d10bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9d10c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97d10cb, SUC_lexicon_tags_179 },
    { 0x227d10cb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x52dd10cf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587d10dc, SUC_lexicon_tags_25 },
    { 0x17fd10dc, SUC_lexicon_tags_19 },
    { 0xf03d10de, SUC_lexicon_tags_306 },
    { 0xe45d10de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fbd10e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57d10e5, SUC_lexicon_tags_15 },
    { 0x31dd10e6, SUC_lexicon_tags_12 },
    { 0xb45d10e5, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bd10ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe03d10ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7d10f3, SUC_lexicon_tags_2 },
    { 0x651d10f4, SUC_lexicon_tags_46 },
    { 0xcb9d10f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x641d10f7, SUC_lexicon_tags_19 },
    { 0x615d10f8, SUC_lexicon_tags_33 },
    { 0x255d10f8, SUC_lexicon_tags_37 },
    { 0xbcfd10f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731d10ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x53dd1101, SUC_lexicon_tags_2 },
    { 0x78dd1101, SUC_lexicon_tags_15 },
    { 0x5a3d1103, SUC_lexicon_tags_18 },
    { 0x757d1103, SUC_lexicon_tags_17 },
    { 0x527d1103, SUC_lexicon_tags_86 },
    { 0x3c1d1106, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfd1109, SUC_lexicon_tags_29 },
    { 0xac1d1109, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x425d110c, SUC_lexicon_tags_55 },
    { 0x9cfd110c, SUC_lexicon_tags_10 },
    { 0xbd1d110e, SUC_lexicon_tags_17 },
    { 0x189d110c, SUC_lexicon_tags_1 },
    { 0xc83d1110, SUC_lexicon_tags_37 },
    { 0x741d110e, SUC_lexicon_tags_16 },
    { 0xbcbd1112, SUC_lexicon_tags_12 },
    { 0xa13d1113, SUC_lexicon_tags_11 },
    { 0xe47d1113, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3d1117, SUC_lexicon_tags_0 },
    { 0x80bd1117, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fd111c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1d1120, SUC_lexicon_tags_37 },
    { 0x271d1120, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x439d1123, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abd1127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1d112a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bd1133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7add1137, SUC_lexicon_tags_37 },
    { 0xce5d1137, SUC_lexicon_tags_25 },
    { 0x23bd1139, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afd113c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21d1142, SUC_lexicon_tags_14 },
    { 0xcc1d1143, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x145d1145, SUC_lexicon_tags_19 },
    { 0xe1fd1146, SUC_lexicon_tags_4 },
    { 0xadbd1146, SUC_lexicon_tags_87 },
    { 0xb9d1147, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd47d114a, SUC_lexicon_tags_12 },
    { 0xe5fd114b, SUC_lexicon_tags_9 },
    { 0xc77d114a, SUC_lexicon_tags_15 },
    { 0x74bd114a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbd1151, SUC_lexicon_tags_27 },
    { 0xb1dd1152, SUC_lexicon_tags_2 },
    { 0x1e9d1153, SUC_lexicon_tags_7 },
    { 0x22fd1154, SUC_lexicon_tags_9 },
    { 0x30fd1153, SUC_lexicon_tags_7 },
    { 0x953d1155, SUC_lexicon_tags_12 },
    { 0x6b7d1154, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7e7d1159, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc17d115b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29d115e, SUC_lexicon_tags_18 },
    { 0x825d115f, SUC_lexicon_tags_14 },
    { 0xa73d1160, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x42fd1162, SUC_lexicon_tags_60 },
    { 0x39dd1162, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcf7d1165, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dd116a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x573d116c, SUC_lexicon_tags_79 },
    { 0x4f1d116d, SUC_lexicon_tags_55 },
    { 0xcf1d116d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3cbd1170, SUC_lexicon_tags_7 },
    { 0x7bbd1171, SUC_lexicon_tags_11 },
    { 0x85d1171, SUC_lexicon_tags_21 },
    { 0x99dd1171, SUC_lexicon_tags_30 },
    { 0x3cfd1172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x34bd1177, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edd117e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8fd1180, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x151d1182, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3d1185, SUC_lexicon_tags_32 },
    { 0x79d1186, SUC_lexicon_tags_25 },
    { 0x7b7d1186, SUC_lexicon_tags_0 },
    { 0x605d1188, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783d118d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1d1192, SUC_lexicon_tags_48 },
    { 0xfe7d1193, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd33d1195, SUC_lexicon_tags_12 },
    { 0x8cbd1195, SUC_lexicon_tags_63 },
    { 0x32dd1196, SUC_lexicon_tags_35 },
    { 0x99d1198, SUC_lexicon_tags_1 },
    { 0x51d1198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fd119d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x38dd11a0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xef1d11a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcedd11a5, SUC_lexicon_tags_7 },
    { 0x439d11a6, SUC_lexicon_tags_6 },
    { 0x135d11a7, SUC_lexicon_tags_27 },
    { 0x4c3d11a8, SUC_lexicon_tags_30 },
    { 0x2f3d11a8, SUC_lexicon_tags_68 },
    { 0xcdbd11aa, SUC_lexicon_tags_45 },
    { 0x28bd11ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31d11ae, SUC_lexicon_tags_8 },
    { 0x8a7d11af, SUC_lexicon_tags_103 },
    { 0xe31d11b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xddfd11b3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3d11b8, SUC_lexicon_tags_2 },
    { 0x557d11b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1d11c2, SUC_lexicon_tags_27 },
    { 0x455d11c3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafd11c9, SUC_lexicon_tags_82 },
    { 0x613d11c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfd11d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x185d11d2, SUC_lexicon_tags_38 },
    { 0xbdfd11d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3d11d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3d11da, SUC_lexicon_tags_31 },
    { 0x399d11db, SUC_lexicon_tags_65 },
    { 0x389d11dc, SUC_lexicon_tags_27 },
    { 0x46fd11dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11d11e0, SUC_lexicon_tags_2 },
    { 0xf9d11e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8bd11e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x817d11e6, SUC_lexicon_tags_11 },
    { 0xeb9d11e6, SUC_lexicon_tags_37 },
    { 0x48bd11e8, SUC_lexicon_tags_8 },
    { 0x9dfd11e8, SUC_lexicon_tags_12 },
    { 0xe51d11e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3d11f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877d11f7, SUC_lexicon_tags_17 },
    { 0x99bd11f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3d11fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x69fd11ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1d1d1201, SUC_lexicon_tags_14 },
    { 0x3b7d1202, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33d1206, SUC_lexicon_tags_77 },
    { 0x815d1207, SUC_lexicon_tags_15 },
    { 0x429d1208, SUC_lexicon_tags_12 },
    { 0x303d1208, SUC_lexicon_tags_17 },
    { 0xe83d1208, SUC_lexicon_tags_15 },
    { 0xba9d120b, SUC_lexicon_tags_28 },
    { 0xd57d120a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bd120f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb7dd1211, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97d1213, SUC_lexicon_tags_23 },
    { 0xfbbd1214, SUC_lexicon_tags_38 },
    { 0x4d9d1214, SUC_lexicon_tags_15 },
    { 0x827d1216, SUC_lexicon_tags_49 },
    { 0x329d1217, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dd121a, SUC_lexicon_tags_37 },
    { 0xc5fd121b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x391d121d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x913d1220, SUC_lexicon_tags_15 },
    { 0x525d1220, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcb3d1223, SUC_lexicon_tags_19 },
    { 0x93dd1223, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x787d1226, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x781d1228, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7d122c, SUC_lexicon_tags_47 },
    { 0x1a9d122c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdedd1230, SUC_lexicon_tags_2 },
    { 0xf6bd1230, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x487d1234, SUC_lexicon_tags_43 },
    { 0x421d1235, SUC_lexicon_tags_25 },
    { 0xfe3d1236, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489d123a, SUC_lexicon_tags_2 },
    { 0x38dd123b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83bd123d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9d1241, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d1248, SUC_lexicon_tags_17 },
    { 0x26dd1248, SUC_lexicon_tags_39 },
    { 0xf51d124a, SUC_lexicon_tags_27 },
    { 0xbbfd124a, SUC_lexicon_tags_60 },
    { 0x6fd124b, SUC_lexicon_tags_47 },
    { 0xc77d124c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x829d124f, SUC_lexicon_tags_12 },
    { 0x959d124f, SUC_lexicon_tags_17 },
    { 0xafd1251, SUC_lexicon_tags_62 },
    { 0x765d1252, SUC_lexicon_tags_11 },
    { 0x46bd1253, SUC_lexicon_tags_1 },
    { 0xb75d1254, SUC_lexicon_tags_26 },
    { 0xbddd1252, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bd1259, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x257d125b, SUC_lexicon_tags_12 },
    { 0x5c7d125c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7d125f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d1263, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273d1267, SUC_lexicon_tags_47 },
    { 0xd8fd1268, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1d126c, SUC_lexicon_tags_4 },
    { 0xfb1d126d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ffd126f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469d1273, SUC_lexicon_tags_27 },
    { 0xbc3d1273, SUC_lexicon_tags_30 },
    { 0x8cbd1275, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdefd1277, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7d127b, SUC_lexicon_tags_17 },
    { 0x943d127b, SUC_lexicon_tags_45 },
    { 0xb61d127d, SUC_lexicon_tags_14 },
    { 0x37d127c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf95d1280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9d1283, SUC_lexicon_tags_25 },
    { 0x6abd1284, SUC_lexicon_tags_37 },
    { 0xf8dd1284, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9d1289, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15d1292, SUC_lexicon_tags_79 },
    { 0xec9d1293, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd1296, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1d129a, SUC_lexicon_tags_25 },
    { 0xafbd129a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2c1d129d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fd12a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77d12a4, SUC_lexicon_tags_37 },
    { 0x81dd12a5, SUC_lexicon_tags_39 },
    { 0xc43d12a6, SUC_lexicon_tags_25 },
    { 0xe49d12a6, SUC_lexicon_tags_66 },
    { 0x209d12a6, SUC_lexicon_tags_43 },
    { 0xc07d12a9, SUC_lexicon_tags_45 },
    { 0xc8dd12aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x863d12ac, SUC_lexicon_tags_12 },
    { 0x251d12ad, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7d12b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x355d12b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7d12bc, SUC_lexicon_tags_13 },
    { 0x4efd12bd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3d12ce, SUC_lexicon_tags_7 },
    { 0xfedd12cf, SUC_lexicon_tags_7 },
    { 0xd13d12d0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabd12d3, SUC_lexicon_tags_17 },
    { 0x167d12d4, SUC_lexicon_tags_8 },
    { 0x19dd12d5, SUC_lexicon_tags_39 },
    { 0xc31d12d6, SUC_lexicon_tags_63 },
    { 0x7abd12d7, SUC_lexicon_tags_1 },
    { 0x679d12d5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bd12df, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fd12e5, SUC_lexicon_tags_9 },
    { 0xc83d12e6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x641d12e9, SUC_lexicon_tags_63 },
    { 0x60fd12ea, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xeb5d12ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdd12f2, SUC_lexicon_tags_2 },
    { 0xa07d12f2, SUC_lexicon_tags_66 },
    { 0x6add12f4, SUC_lexicon_tags_28 },
    { 0xe7dd12f5, SUC_lexicon_tags_34 },
    { 0xda1d12f3, SUC_lexicon_tags_2 },
    { 0x6bbd12f7, SUC_lexicon_tags_911 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5d12fd, SUC_lexicon_tags_26 },
    { 0xe3dd12fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x74dd1300, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3b9d1302, SUC_lexicon_tags_27 },
    { 0x8e3d1303, SUC_lexicon_tags_47 },
    { 0x213d1303, SUC_lexicon_tags_30 },
    { 0x133d1305, SUC_lexicon_tags_56 },
    { 0x83fd1306, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x74bd1308, SUC_lexicon_tags_17 },
    { 0x4dd1309, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x457d130b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3d130f, SUC_lexicon_tags_31 },
    { 0xce7d1310, SUC_lexicon_tags_85 },
    { 0x409d130f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x433d1314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bd1318, SUC_lexicon_tags_12 },
    { 0xff7d1319, SUC_lexicon_tags_28 },
    { 0xaa7d1319, SUC_lexicon_tags_19 },
    { 0x425d131b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bd131e, SUC_lexicon_tags_12 },
    { 0x36bd131f, SUC_lexicon_tags_25 },
    { 0x97bd1320, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87d1324, SUC_lexicon_tags_60 },
    { 0xf83d1324, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd19d1327, SUC_lexicon_tags_31 },
    { 0x69d1328, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x137d132b, SUC_lexicon_tags_46 },
    { 0xe05d132c, SUC_lexicon_tags_4 },
    { 0xf65d132d, SUC_lexicon_tags_7 },
    { 0xf41d132d, SUC_lexicon_tags_42 },
    { 0xf41d132f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd1335, SUC_lexicon_tags_151 },
    { 0x1f5d1336, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x129d1338, SUC_lexicon_tags_37 },
    { 0xf4dd1339, SUC_lexicon_tags_37 },
    { 0x767d1339, SUC_lexicon_tags_17 },
    { 0x1bd133b, SUC_lexicon_tags_33 },
    { 0x17bd133a, SUC_lexicon_tags_79 },
    { 0x81bd1339, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbbfd133f, SUC_lexicon_tags_7 },
    { 0x507d1340, SUC_lexicon_tags_151 },
    { 0xbb9d1340, SUC_lexicon_tags_37 },
    { 0xb03d1342, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x11bd1344, SUC_lexicon_tags_15 },
    { 0x1e9d1345, SUC_lexicon_tags_25 },
    { 0xda9d1346, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dd134b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6fbd134d, SUC_lexicon_tags_15 },
    { 0x8c7d134d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31d1352, SUC_lexicon_tags_45 },
    { 0x53bd1352, SUC_lexicon_tags_27 },
    { 0x85bd1354, SUC_lexicon_tags_11 },
    { 0x6c9d1354, SUC_lexicon_tags_37 },
    { 0x627d1356, SUC_lexicon_tags_17 },
    { 0x255d1356, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x317d135a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdd135d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x43dd135f, SUC_lexicon_tags_30 },
    { 0x19d135f, SUC_lexicon_tags_207 },
    { 0x747d1360, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd1365, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11d1368, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb1d136a, SUC_lexicon_tags_15 },
    { 0x1bd136b, SUC_lexicon_tags_38 },
    { 0x419d136c, SUC_lexicon_tags_23 },
    { 0x2e3d136d, SUC_lexicon_tags_2 },
    { 0xc35d136e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9d1371, SUC_lexicon_tags_1 },
    { 0xf97d1372, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103d1377, SUC_lexicon_tags_25 },
    { 0xe6fd1377, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x435d137a, SUC_lexicon_tags_28 },
    { 0x8b1d137b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7d137f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5fd1381, SUC_lexicon_tags_218 },
    { 0x94fd1382, SUC_lexicon_tags_2 },
    { 0x1a9d1383, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x27bd1385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x625d1387, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7d138d, SUC_lexicon_tags_45 },
    { 0x99bd138e, SUC_lexicon_tags_17 },
    { 0x23bd138f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91d1393, SUC_lexicon_tags_31 },
    { 0xde7d1393, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1398, SUC_lexicon_tags_38 },
    { 0x45bd1398, SUC_lexicon_tags_49 },
    { 0x211d139a, SUC_lexicon_tags_49 },
    { 0xdc1d139a, SUC_lexicon_tags_12 },
    { 0x61bd139c, SUC_lexicon_tags_7 },
    { 0xd4dd139b, SUC_lexicon_tags_14 },
    { 0x43fd139b, SUC_lexicon_tags_33 },
    { 0xf5dd1399, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x217d13a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bd13a5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bd13ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4fd13ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7abd13b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03d13b9, SUC_lexicon_tags_60 },
    { 0x5f7d13ba, SUC_lexicon_tags_15 },
    { 0x7bfd13bb, SUC_lexicon_tags_37 },
    { 0xc01d13bb, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdd13c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1d13c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19d13cb, SUC_lexicon_tags_17 },
    { 0x91bd13cc, SUC_lexicon_tags_1 },
    { 0x3d1d13cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87bd13d0, SUC_lexicon_tags_17 },
    { 0x7cbd13d1, SUC_lexicon_tags_37 },
    { 0x75fd13d2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedd13d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bd13da, SUC_lexicon_tags_65 },
    { 0xdd13db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751d13df, SUC_lexicon_tags_25 },
    { 0x1d7d13df, SUC_lexicon_tags_11 },
    { 0x9a1d13df, SUC_lexicon_tags_33 },
    { 0x47d13e0, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0xcedd13e5, SUC_lexicon_tags_27 },
    { 0x859d13e6, SUC_lexicon_tags_8 },
    { 0x775d13e6, SUC_lexicon_tags_45 },
    { 0xdedd13e8, SUC_lexicon_tags_14 },
    { 0x109d13e7, SUC_lexicon_tags_17 },
    { 0xe07d13ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdd13ef, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xde9d13f1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41d13f4, SUC_lexicon_tags_71 },
    { 0xfdbd13f5, SUC_lexicon_tags_47 },
    { 0x61bd13f4, SUC_lexicon_tags_37 },
    { 0x9a3d13f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905d13fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3edd13fd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fd1404, SUC_lexicon_tags_14 },
    { 0xb55d1405, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fd1408, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe6dd140a, SUC_lexicon_tags_17 },
    { 0x27dd140a, SUC_lexicon_tags_39 },
    { 0x2efd140c, SUC_lexicon_tags_26 },
    { 0x655d140d, SUC_lexicon_tags_1 },
    { 0x631d140e, SUC_lexicon_tags_25 },
    { 0x53d140e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675d1414, SUC_lexicon_tags_2 },
    { 0x461d1415, SUC_lexicon_tags_4 },
    { 0xcafd1416, SUC_lexicon_tags_92 },
    { 0x7b1d1415, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x707d141a, SUC_lexicon_tags_15 },
    { 0xc81d141a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66fd141d, SUC_lexicon_tags_42 },
    { 0xa1fd141e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1d1427, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3bdd1429, SUC_lexicon_tags_60 },
    { 0x5f7d142a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x107d142d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89d1432, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd77d1434, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981d1439, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725d143e, SUC_lexicon_tags_296 },
    { 0x3f7d143e, SUC_lexicon_tags_17 },
    { 0x33bd143e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x673d1443, SUC_lexicon_tags_47 },
    { 0x481d1444, SUC_lexicon_tags_30 },
    { 0x64dd1445, SUC_lexicon_tags_17 },
    { 0x909d1446, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5fd1448, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fd144b, SUC_lexicon_tags_82 },
    { 0x96fd144c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3d1454, SUC_lexicon_tags_15 },
    { 0xa5dd1455, SUC_lexicon_tags_15 },
    { 0x289d1456, SUC_lexicon_tags_44 },
    { 0xd01d1455, SUC_lexicon_tags_37 },
    { 0xf19d1457, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47d1462, SUC_lexicon_tags_2 },
    { 0xc0dd1463, SUC_lexicon_tags_27 },
    { 0x8dbd1464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x32dd1467, SUC_lexicon_tags_7 },
    { 0x695d1468, SUC_lexicon_tags_27 },
    { 0xf3dd1469, SUC_lexicon_tags_1 },
    { 0xd49d146a, SUC_lexicon_tags_1 },
    { 0xabdd146b, SUC_lexicon_tags_37 },
    { 0x59d146a, SUC_lexicon_tags_5 },
    { 0xd3d146d, SUC_lexicon_tags_14 },
    { 0xa2bd146e, SUC_lexicon_tags_12 },
    { 0x1a9d1467, SUC_lexicon_tags_38 },
    { 0xd2dd146d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bd1473, SUC_lexicon_tags_11 },
    { 0xce1d1473, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e3d1476, SUC_lexicon_tags_71 },
    { 0xd95d1477, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbd147b, SUC_lexicon_tags_8 },
    { 0xf03d147c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xabbd147e, SUC_lexicon_tags_27 },
    { 0xc0bd147f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01d1482, SUC_lexicon_tags_29 },
    { 0xbfd1483, SUC_lexicon_tags_29 },
    { 0x501d1484, SUC_lexicon_tags_69 },
    { 0xb73d1485, SUC_lexicon_tags_17 },
    { 0x1afd1486, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x785d1488, SUC_lexicon_tags_29 },
    { 0xb5fd1489, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1d148d, SUC_lexicon_tags_27 },
    { 0x5d3d148e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5d1491, SUC_lexicon_tags_17 },
    { 0x83dd1491, SUC_lexicon_tags_46 },
    { 0xafbd1493, SUC_lexicon_tags_17 },
    { 0x8bdd1493, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223d1498, SUC_lexicon_tags_2 },
    { 0x5f3d1499, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x19bd149c, SUC_lexicon_tags_63 },
    { 0xf3dd149d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5d14a1, SUC_lexicon_tags_7 },
    { 0xe63d14a2, SUC_lexicon_tags_13 },
    { 0x7b7d14a3, SUC_lexicon_tags_29 },
    { 0xd33d14a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d14ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35d14af, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe91d14b1, SUC_lexicon_tags_47 },
    { 0x933d14b2, SUC_lexicon_tags_27 },
    { 0x12dd14b2, SUC_lexicon_tags_17 },
    { 0x5ddd14b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x295d14b6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1d14bc, SUC_lexicon_tags_25 },
    { 0x50dd14bd, SUC_lexicon_tags_7 },
    { 0x567d14be, SUC_lexicon_tags_7 },
    { 0xa39d14be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81d14c6, SUC_lexicon_tags_11 },
    { 0x8fbd14c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x41dd14ca, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7d14cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1add14cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1bfd14d1, SUC_lexicon_tags_34 },
    { 0x4dfd14d2, SUC_lexicon_tags_81 },
    { 0xe33d14d1, SUC_lexicon_tags_84 },
    { 0x921d14d3, SUC_lexicon_tags_11 },
    { 0xad1d14d1, SUC_lexicon_tags_47 },
    { 0x821d14d3, SUC_lexicon_tags_2 },
    { 0x4a9d14d7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0x843d14da, SUC_lexicon_tags_17 },
    { 0x20dd14da, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa41d14dd, SUC_lexicon_tags_47 },
    { 0x283d14dd, SUC_lexicon_tags_29 },
    { 0xf45d14df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7b5d14e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x927d14e4, SUC_lexicon_tags_55 },
    { 0xa77d14e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x285d14e8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd14ec, SUC_lexicon_tags_19 },
    { 0x163d14ec, SUC_lexicon_tags_37 },
    { 0x777d14ee, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d14fd, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd17d14ff, SUC_lexicon_tags_4 },
    { 0x151d1500, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4bfd1502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99d1504, SUC_lexicon_tags_16 },
    { 0x667d1505, SUC_lexicon_tags_13 },
    { 0x985d1506, SUC_lexicon_tags_77 },
    { 0x743d1506, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323d150d, SUC_lexicon_tags_37 },
    { 0x5b9d150d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xeebd1510, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb29d1512, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb03d1514, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd1517, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5d151c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc49d151e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x921d1521, SUC_lexicon_tags_100 },
    { 0x259d1522, SUC_lexicon_tags_43 },
    { 0xf8fd1521, SUC_lexicon_tags_27 },
    { 0x9bd1522, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbd1528, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x337d152b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dd152e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbffd1530, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x271d1533, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xafbd1535, SUC_lexicon_tags_0 },
    { 0xf13d1535, SUC_lexicon_tags_37 },
    { 0x275d1537, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x565d153a, SUC_lexicon_tags_37 },
    { 0x8add153b, SUC_lexicon_tags_11 },
    { 0xd59d153c, SUC_lexicon_tags_19 },
    { 0x1d9d153d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcf9d153f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x845d1541, SUC_lexicon_tags_17 },
    { 0x885d1542, SUC_lexicon_tags_63 },
    { 0xc0bd1541, SUC_lexicon_tags_7 },
    { 0xdd7d1544, SUC_lexicon_tags_25 },
    { 0xa67d1544, SUC_lexicon_tags_15 },
    { 0x483d1546, SUC_lexicon_tags_28 },
    { 0x817d1547, SUC_lexicon_tags_68 },
    { 0x629d1548, SUC_lexicon_tags_38 },
    { 0x10dd1549, SUC_lexicon_tags_15 },
    { 0xe65d1544, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19d154f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691d1554, SUC_lexicon_tags_25 },
    { 0x891d1555, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1d1558, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x595d155b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63d1561, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x107d1563, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631d156a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fd1574, SUC_lexicon_tags_12 },
    { 0x71d1574, SUC_lexicon_tags_27 },
    { 0xe5fd1574, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73fd1578, SUC_lexicon_tags_7 },
    { 0xed9d1579, SUC_lexicon_tags_39 },
    { 0xe1bd1578, SUC_lexicon_tags_11 },
    { 0xc4dd157b, SUC_lexicon_tags_25 },
    { 0x247d157b, SUC_lexicon_tags_15 },
    { 0x9f7d157b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x109d1580, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bd1585, SUC_lexicon_tags_15 },
    { 0x179d1585, SUC_lexicon_tags_30 },
    { 0xeb5d1587, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fd158a, SUC_lexicon_tags_19 },
    { 0xd37d158b, SUC_lexicon_tags_64 },
    { 0x355d158b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5d1592, SUC_lexicon_tags_49 },
    { 0x71d1592, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69dd1595, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d1599, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dd15a1, SUC_lexicon_tags_43 },
    { 0xa4bd15a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fd15aa, SUC_lexicon_tags_17 },
    { 0xc51d15aa, SUC_lexicon_tags_54 },
    { 0x877d15ab, SUC_lexicon_tags_44 },
    { 0xc8dd15aa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfd15b2, SUC_lexicon_tags_9 },
    { 0x4b9d15b3, SUC_lexicon_tags_33 },
    { 0xfd7d15b4, SUC_lexicon_tags_12 },
    { 0xb63d15b5, SUC_lexicon_tags_7 },
    { 0x2b7d15b4, SUC_lexicon_tags_46 },
    { 0xa33d15b2, SUC_lexicon_tags_14 },
    { 0x237d15b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x573d15ba, SUC_lexicon_tags_1 },
    { 0x209d15bb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x915d15be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7d15c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x68fd15c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5d15cb, SUC_lexicon_tags_11 },
    { 0xebdd15cc, SUC_lexicon_tags_29 },
    { 0x9efd15cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79d15d1, SUC_lexicon_tags_15 },
    { 0x567d15d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb03d15d4, SUC_lexicon_tags_1 },
    { 0x7bfd15d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bd15e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7d15e3, SUC_lexicon_tags_6 },
    { 0x9a3d15e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521d15e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x713d15ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39d15ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x779d15f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x765d15f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfd15f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d15f9, SUC_lexicon_tags_2 },
    { 0xd97d15fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7d15fd, SUC_lexicon_tags_7 },
    { 0xdc3d15fe, SUC_lexicon_tags_7 },
    { 0x7b1d15fe, SUC_lexicon_tags_37 },
    { 0x7b3d15ff, SUC_lexicon_tags_2 },
    { 0x4f3d15fe, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53d1605, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87d160d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7d1610, SUC_lexicon_tags_23 },
    { 0xa0fd1611, SUC_lexicon_tags_65 },
    { 0xeffd1612, SUC_lexicon_tags_37 },
    { 0x6ebd1613, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5d161b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0x9dd161d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x355d161f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5d1627, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90dd1629, SUC_lexicon_tags_26 },
    { 0x513d1629, SUC_lexicon_tags_7 },
    { 0x23bd162b, SUC_lexicon_tags_2 },
    { 0x2f7d162c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7d1633, SUC_lexicon_tags_10 },
    { 0xabfd1634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583d1639, SUC_lexicon_tags_0 },
    { 0x301d1639, SUC_lexicon_tags_60 },
    { 0xf99d163b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39d1647, SUC_lexicon_tags_11 },
    { 0xcf3d1648, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbd164c, SUC_lexicon_tags_108 },
    { 0x2c9d164d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd1654, SUC_lexicon_tags_18 },
    { 0xb21d1654, SUC_lexicon_tags_2 },
    { 0x821d1656, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd165d, SUC_lexicon_tags_64 },
    { 0x127d165e, SUC_lexicon_tags_62 },
    { 0x843d165f, SUC_lexicon_tags_87 },
    { 0xb5fd165e, SUC_lexicon_tags_0 },
    { 0xf87d165f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x453d1663, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd61d1665, SUC_lexicon_tags_17 },
    { 0x741d1665, SUC_lexicon_tags_11 },
    { 0xed1d1667, SUC_lexicon_tags_17 },
    { 0xfebd1668, SUC_lexicon_tags_15 },
    { 0x553d1665, SUC_lexicon_tags_23 },
    { 0xe17d1668, SUC_lexicon_tags_47 },
    { 0xa95d166a, SUC_lexicon_tags_65 },
    { 0x9bd1668, SUC_lexicon_tags_7 },
    { 0x867d166d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627d1671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9dd1673, SUC_lexicon_tags_15 },
    { 0x995d1673, SUC_lexicon_tags_11 },
    { 0x567d1673, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbd1679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7d167f, SUC_lexicon_tags_17 },
    { 0x389d1680, SUC_lexicon_tags_31 },
    { 0xe39d167f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dd1686, SUC_lexicon_tags_23 },
    { 0xe3d1687, SUC_lexicon_tags_25 },
    { 0xdddd1687, SUC_lexicon_tags_18 },
    { 0xc79d1689, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d1692, SUC_lexicon_tags_57 },
    { 0x5a7d1693, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9c7d1695, SUC_lexicon_tags_66 },
    { 0x2a3d1696, SUC_lexicon_tags_19 },
    { 0x4bfd1697, SUC_lexicon_tags_14 },
    { 0x7c3d1697, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x267d169a, SUC_lexicon_tags_0 },
    { 0xd2dd169b, SUC_lexicon_tags_11 },
    { 0x8c5d169c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbf5d169e, SUC_lexicon_tags_49 },
    { 0x413d169e, SUC_lexicon_tags_25 },
    { 0xc49d16a0, SUC_lexicon_tags_25 },
    { 0xb97d16a1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29d16a5, SUC_lexicon_tags_17 },
    { 0xcb3d16a6, SUC_lexicon_tags_46 },
    { 0x75fd16a7, SUC_lexicon_tags_12 },
    { 0xabfd16a8, SUC_lexicon_tags_61 },
    { 0xbc3d16a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf09d16ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd01d16ad, SUC_lexicon_tags_17 },
    { 0xcbdd16ae, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7d16b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fd16b7, SUC_lexicon_tags_2 },
    { 0xb3d16b7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfd16be, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15d16c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x36dd16ca, SUC_lexicon_tags_8 },
    { 0x931d16cb, SUC_lexicon_tags_105 },
    { 0x4b5d16cc, SUC_lexicon_tags_19 },
    { 0x2e3d16cd, SUC_lexicon_tags_146 },
    { 0x4dd16cd, SUC_lexicon_tags_4 },
    { 0xdd5d16cf, SUC_lexicon_tags_0 },
    { 0xa63d16d0, SUC_lexicon_tags_30 },
    { 0xa21d16d0, SUC_lexicon_tags_7 },
    { 0xf13d16d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8f7d16d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9d16d7, SUC_lexicon_tags_0 },
    { 0x66bd16d8, SUC_lexicon_tags_7 },
    { 0x95bd16d8, SUC_lexicon_tags_129 },
    { 0x1a1d16da, SUC_lexicon_tags_17 },
    { 0x4cfd16d8, SUC_lexicon_tags_7 },
    { 0x899d16dc, SUC_lexicon_tags_1 },
    { 0x865d16dc, SUC_lexicon_tags_37 },
    { 0x5a3d16dd, SUC_lexicon_tags_49 },
    { 0x52fd16df, SUC_lexicon_tags_1 },
    { 0x65fd16e0, SUC_lexicon_tags_30 },
    { 0x215d16d7, SUC_lexicon_tags_2 },
    { 0x977d16e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26fd16e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bd16e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fd16ee, SUC_lexicon_tags_14 },
    { 0xe9bd16ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x939d16f1, SUC_lexicon_tags_33 },
    { 0x9bdd16f2, SUC_lexicon_tags_37 },
    { 0xc9fd16f3, SUC_lexicon_tags_11 },
    { 0x2f3d16f4, SUC_lexicon_tags_10 },
    { 0x7bdd16f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19d16f8, SUC_lexicon_tags_6 },
    { 0xbbd16f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37bd16fb, SUC_lexicon_tags_9 },
    { 0xcc1d16fb, SUC_lexicon_tags_25 },
    { 0x40fd16fd, SUC_lexicon_tags_17 },
    { 0x35d16fd, SUC_lexicon_tags_12 },
    { 0x795d16ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bd1707, SUC_lexicon_tags_25 },
    { 0xcffd1707, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67d170b, SUC_lexicon_tags_45 },
    { 0x12fd170c, SUC_lexicon_tags_8 },
    { 0x83d170d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e1d170f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5d1714, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5bd1716, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd7bd1718, SUC_lexicon_tags_37 },
    { 0x88fd1718, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30dd171e, SUC_lexicon_tags_6 },
    { 0x3a5d171e, SUC_lexicon_tags_30 },
    { 0xbf3d171f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x38dd1722, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddd1725, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa53d1727, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dd172e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99d1736, SUC_lexicon_tags_19 },
    { 0xed1d1737, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bd173b, SUC_lexicon_tags_17 },
    { 0xc45d173c, SUC_lexicon_tags_2 },
    { 0xbf9d173d, SUC_lexicon_tags_27 },
    { 0x959d173d, SUC_lexicon_tags_25 },
    { 0x891d173f, SUC_lexicon_tags_37 },
    { 0xfdbd173f, SUC_lexicon_tags_17 },
    { 0x1b9d1740, SUC_lexicon_tags_1 },
    { 0xdc5d1742, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdd1747, SUC_lexicon_tags_52 },
    { 0x543d1748, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69d174b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5fd174d, SUC_lexicon_tags_11 },
    { 0x49dd174d, SUC_lexicon_tags_129 },
    { 0xec9d174f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24dd1751, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x157d1753, SUC_lexicon_tags_45 },
    { 0xd09d1753, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd1758, SUC_lexicon_tags_63 },
    { 0x703d1759, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7ddd175b, SUC_lexicon_tags_12 },
    { 0x911d175b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb7d175e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x231d1761, SUC_lexicon_tags_13 },
    { 0xbc5d1762, SUC_lexicon_tags_0 },
    { 0xc1dd1762, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf39d1765, SUC_lexicon_tags_84 },
    { 0x3e7d1765, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edd176a, SUC_lexicon_tags_37 },
    { 0xbbbd176b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd99d176d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1d1772, SUC_lexicon_tags_9 },
    { 0x4c3d1773, SUC_lexicon_tags_6 },
    { 0x60fd1772, SUC_lexicon_tags_60 },
    { 0x291d1775, SUC_lexicon_tags_37 },
    { 0xd6bd1776, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd177a, SUC_lexicon_tags_63 },
    { 0xb69d177a, SUC_lexicon_tags_15 },
    { 0xa1bd177c, SUC_lexicon_tags_48 },
    { 0xd15d177b, SUC_lexicon_tags_7 },
    { 0x9a9d177b, SUC_lexicon_tags_87 },
    { 0x91d177f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d1783, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bd1787, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fd178b, SUC_lexicon_tags_12 },
    { 0x8f9d178c, SUC_lexicon_tags_6 },
    { 0x881d178b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bd1790, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbd17a2, SUC_lexicon_tags_23 },
    { 0x4add17a2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787d17a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb05d17a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159d17ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9d17b1, SUC_lexicon_tags_25 },
    { 0x953d17b2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x63bd17b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7d17bd, SUC_lexicon_tags_45 },
    { 0x325d17be, SUC_lexicon_tags_7 },
    { 0xddfd17bd, SUC_lexicon_tags_17 },
    { 0x6fd17c0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fd17c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1d17ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fd17d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1d17d9, SUC_lexicon_tags_1 },
    { 0xab5d17da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bd17dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe81d17df, SUC_lexicon_tags_11 },
    { 0xa51d17e0, SUC_lexicon_tags_25 },
    { 0x74dd17e1, SUC_lexicon_tags_14 },
    { 0xd5dd17e2, SUC_lexicon_tags_37 },
    { 0xf8bd17e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbd17e6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc97d17e8, SUC_lexicon_tags_12 },
    { 0xc15d17e8, SUC_lexicon_tags_37 },
    { 0xeedd17e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bd17ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11bd17f0, SUC_lexicon_tags_30 },
    { 0xb9fd17f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x729d17f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x88dd17f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90dd17f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb7bd17fa, SUC_lexicon_tags_14 },
    { 0x687d17fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1d17fe, SUC_lexicon_tags_14 },
    { 0x237d17ff, SUC_lexicon_tags_45 },
    { 0x9b1d1800, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfcbd1802, SUC_lexicon_tags_30 },
    { 0x847d1803, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789d180a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x141d180d, SUC_lexicon_tags_25 },
    { 0xee3d180e, SUC_lexicon_tags_17 },
    { 0x2efd180f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33d1812, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5d1818, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355d1822, SUC_lexicon_tags_1 },
    { 0x54dd1823, SUC_lexicon_tags_62 },
    { 0x3f3d1823, SUC_lexicon_tags_11 },
    { 0xe83d1825, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61d1829, SUC_lexicon_tags_60 },
    { 0x4cdd182a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9d182e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x99fd1831, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7dbd1833, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3d1839, SUC_lexicon_tags_7 },
    { 0x9f3d183a, SUC_lexicon_tags_11 },
    { 0xbf7d183b, SUC_lexicon_tags_27 },
    { 0xc4dd183a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xde3d183e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fd1841, SUC_lexicon_tags_17 },
    { 0xd39d1841, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fd1845, SUC_lexicon_tags_64 },
    { 0xf2fd1845, SUC_lexicon_tags_39 },
    { 0xe29d1847, SUC_lexicon_tags_141 },
    { 0x1b5d1848, SUC_lexicon_tags_15 },
    { 0x971d1849, SUC_lexicon_tags_12 },
    { 0x85d1848, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187d184e, SUC_lexicon_tags_6 },
    { 0x7a7d184e, SUC_lexicon_tags_87 },
    { 0xe53d1850, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69dd1852, SUC_lexicon_tags_14 },
    { 0xc77d1853, SUC_lexicon_tags_4 },
    { 0x5add1852, SUC_lexicon_tags_56 },
    { 0x42fd1855, SUC_lexicon_tags_11 },
    { 0xd83d1856, SUC_lexicon_tags_1 },
    { 0x61fd1853, SUC_lexicon_tags_25 },
    { 0x423d1858, SUC_lexicon_tags_0 },
    { 0x573d1859, SUC_lexicon_tags_29 },
    { 0xaefd1859, SUC_lexicon_tags_11 },
    { 0x211d1858, SUC_lexicon_tags_23 },
    { 0xaefd1859, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5f9d185e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77d1862, SUC_lexicon_tags_0 },
    { 0x3b5d1863, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xadd1865, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24fd1867, SUC_lexicon_tags_87 },
    { 0xc67d1867, SUC_lexicon_tags_11 },
    { 0xd89d1868, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf61d186c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbd1871, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf0bd1873, SUC_lexicon_tags_45 },
    { 0x1f7d1874, SUC_lexicon_tags_1 },
    { 0x8fd1875, SUC_lexicon_tags_27 },
    { 0xa3dd1876, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7dd1878, SUC_lexicon_tags_16 },
    { 0x575d1879, SUC_lexicon_tags_7 },
    { 0x61bd187a, SUC_lexicon_tags_8 },
    { 0xc13d187a, SUC_lexicon_tags_55 },
    { 0x197d187c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa23d187e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb07d1880, SUC_lexicon_tags_7 },
    { 0xb1d1881, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffd1886, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7d1889, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdd188e, SUC_lexicon_tags_30 },
    { 0xa8dd188f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fd1892, SUC_lexicon_tags_15 },
    { 0x79fd1893, SUC_lexicon_tags_25 },
    { 0x7ebd1894, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3d189b, SUC_lexicon_tags_77 },
    { 0x455d189b, SUC_lexicon_tags_60 },
    { 0x9dfd189d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7dd189f, SUC_lexicon_tags_30 },
    { 0x5b3d189f, SUC_lexicon_tags_72 },
    { 0x5d1d18a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03d18a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x727d18a7, SUC_lexicon_tags_25 },
    { 0x1cdd18a8, SUC_lexicon_tags_15 },
    { 0x8bbd18a8, SUC_lexicon_tags_83 },
    { 0x633d18a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x255d18ad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcd9d18af, SUC_lexicon_tags_60 },
    { 0x7c1d18b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fd18b4, SUC_lexicon_tags_0 },
    { 0x4a9d18b5, SUC_lexicon_tags_12 },
    { 0xf3fd18b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81d18ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadd18bd, SUC_lexicon_tags_47 },
    { 0xce9d18be, SUC_lexicon_tags_37 },
    { 0xb5bd18be, SUC_lexicon_tags_2 },
    { 0x583d18c0, SUC_lexicon_tags_35 },
    { 0x137d18c1, SUC_lexicon_tags_37 },
    { 0x581d18c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dd18c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29d18c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x461d18cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x995d18ce, SUC_lexicon_tags_15 },
    { 0xfd7d18cf, SUC_lexicon_tags_45 },
    { 0x639d18d0, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351d18e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9d18f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3f1d18f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3d18f9, SUC_lexicon_tags_14 },
    { 0xd0bd18fa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xfddd18fc, SUC_lexicon_tags_21 },
    { 0x7abd18fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x57dd18ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493d1904, SUC_lexicon_tags_37 },
    { 0x5afd1905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7d1908, SUC_lexicon_tags_17 },
    { 0x7fd1909, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0xeffd190b, SUC_lexicon_tags_37 },
    { 0xd7d190c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b3d190e, SUC_lexicon_tags_1 },
    { 0x679d190f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9d1912, SUC_lexicon_tags_26 },
    { 0xb43d1912, SUC_lexicon_tags_38 },
    { 0xb8bd1912, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x131d1917, SUC_lexicon_tags_8 },
    { 0x165d1917, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555d191f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd1922, SUC_lexicon_tags_1 },
    { 0xe67d1922, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfd1928, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953d1939, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dd193e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xee9d1941, SUC_lexicon_tags_161 },
    { 0x421d1942, SUC_lexicon_tags_11 },
    { 0x583d1941, SUC_lexicon_tags_433 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fd1947, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dd194c, SUC_lexicon_tags_8 },
    { 0x6e1d194c, SUC_lexicon_tags_43 },
    { 0x91fd194e, SUC_lexicon_tags_14 },
    { 0x2e1d194e, SUC_lexicon_tags_9 },
    { 0x54dd194d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831d1958, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x893d195b, SUC_lexicon_tags_37 },
    { 0x525d195b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9d195f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3d1962, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb85d1964, SUC_lexicon_tags_25 },
    { 0x15dd1965, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x161d1967, SUC_lexicon_tags_27 },
    { 0x56dd1967, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1d196d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9c5d196f, SUC_lexicon_tags_8 },
    { 0xa7fd196f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd31d1972, SUC_lexicon_tags_7 },
    { 0x9a9d1972, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bd1976, SUC_lexicon_tags_37 },
    { 0xe7d1976, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1d197a, SUC_lexicon_tags_17 },
    { 0x97d197b, SUC_lexicon_tags_60 },
    { 0x3a1d197c, SUC_lexicon_tags_56 },
    { 0xc3d197c, SUC_lexicon_tags_2 },
    { 0x1cbd197e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413d1987, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa03d1989, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7dd198b, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0x855d198d, SUC_lexicon_tags_47 },
    { 0xe9fd198d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb07d1990, SUC_lexicon_tags_2 },
    { 0x597d1990, SUC_lexicon_tags_37 },
    { 0x92fd1991, SUC_lexicon_tags_21 },
    { 0x62fd1993, SUC_lexicon_tags_19 },
    { 0x7d5d1994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cbd1996, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x8cdd1998, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01d199b, SUC_lexicon_tags_26 },
    { 0xcfd199c, SUC_lexicon_tags_15 },
    { 0x347d199c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x939d199f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc0bd19a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x20fd19a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6a9d19a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef1d19a8, SUC_lexicon_tags_17 },
    { 0xfe3d19a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3d19af, SUC_lexicon_tags_298 },
    { 0x919d19b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffd19b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x319d19b5, SUC_lexicon_tags_174 },
    { 0x5dd19b6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bd19bb, SUC_lexicon_tags_63 },
    { 0xe53d19bc, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xa49d19be, SUC_lexicon_tags_37 },
    { 0xd51d19be, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x451d19c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1d19cc, SUC_lexicon_tags_8 },
    { 0xf83d19cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85d19d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7dbd19d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdd19d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7d19dc, SUC_lexicon_tags_25 },
    { 0xb1dd19dc, SUC_lexicon_tags_10 },
    { 0xe4dd19de, SUC_lexicon_tags_6 },
    { 0x7b9d19dc, SUC_lexicon_tags_6 },
    { 0x5add19df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf29d19e3, SUC_lexicon_tags_37 },
    { 0x4f9d19e4, SUC_lexicon_tags_30 },
    { 0xd03d19e4, SUC_lexicon_tags_6 },
    { 0xbbd19e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7d19e9, SUC_lexicon_tags_25 },
    { 0x6c3d19e9, SUC_lexicon_tags_37 },
    { 0x97dd19eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3bd19ed, SUC_lexicon_tags_25 },
    { 0xa79d19ed, SUC_lexicon_tags_45 },
    { 0xf15d19ef, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19d19f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd7dd19fa, SUC_lexicon_tags_14 },
    { 0x67fd19fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7d1a01, SUC_lexicon_tags_8 },
    { 0x45dd1a01, SUC_lexicon_tags_25 },
    { 0x32fd1a01, SUC_lexicon_tags_17 },
    { 0x25dd1a04, SUC_lexicon_tags_32 },
    { 0xc61d1a04, SUC_lexicon_tags_12 },
    { 0x14dd1a06, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdd1a09, SUC_lexicon_tags_45 },
    { 0x4c9d1a0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7d1a14, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x405d1a17, SUC_lexicon_tags_39 },
    { 0xcb3d1a18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7d1a1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9d1a24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1bd1a26, SUC_lexicon_tags_11 },
    { 0x21d1a27, SUC_lexicon_tags_23 },
    { 0x859d1a27, SUC_lexicon_tags_11 },
    { 0x721d1a28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x92dd1a2b, SUC_lexicon_tags_25 },
    { 0xb13d1a2b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd1a30, SUC_lexicon_tags_17 },
    { 0xfbfd1a31, SUC_lexicon_tags_15 },
    { 0x9a5d1a31, SUC_lexicon_tags_2 },
    { 0x5cdd1a33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3d1a36, SUC_lexicon_tags_2 },
    { 0x475d1a37, SUC_lexicon_tags_25 },
    { 0x23dd1a37, SUC_lexicon_tags_17 },
    { 0x779d1a39, SUC_lexicon_tags_47 },
    { 0x65d1a3a, SUC_lexicon_tags_1 },
    { 0x1edd1a3a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26fd1a3d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dd1a40, SUC_lexicon_tags_477 },
    { 0xe1fd1a41, SUC_lexicon_tags_8 },
    { 0xfdbd1a42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41d1a47, SUC_lexicon_tags_25 },
    { 0x2b9d1a48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x769d1a4a, SUC_lexicon_tags_11 },
    { 0x9f9d1a4a, SUC_lexicon_tags_43 },
    { 0x975d1a4b, SUC_lexicon_tags_44 },
    { 0x59dd1a4c, SUC_lexicon_tags_27 },
    { 0x127d1a4e, SUC_lexicon_tags_12 },
    { 0xec5d1a4c, SUC_lexicon_tags_63 },
    { 0xbd5d1a50, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x143d1a52, SUC_lexicon_tags_23 },
    { 0xa9fd1a53, SUC_lexicon_tags_106 },
    { 0x295d1a54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdd1a5a, SUC_lexicon_tags_31 },
    { 0x551d1a5b, SUC_lexicon_tags_8 },
    { 0x533d1a5a, SUC_lexicon_tags_15 },
    { 0x8bfd1a5d, SUC_lexicon_tags_7 },
    { 0x73fd1a5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5d1a63, SUC_lexicon_tags_2 },
    { 0xf83d1a63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x135d1a66, SUC_lexicon_tags_23 },
    { 0x373d1a66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf35d1a69, SUC_lexicon_tags_12 },
    { 0x90dd1a6a, SUC_lexicon_tags_15 },
    { 0x149d1a6a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x869d1a6d, SUC_lexicon_tags_17 },
    { 0x3c3d1a6e, SUC_lexicon_tags_27 },
    { 0xa7dd1a6f, SUC_lexicon_tags_25 },
    { 0xf5d1a70, SUC_lexicon_tags_1 },
    { 0x1dfd1a71, SUC_lexicon_tags_9 },
    { 0x24fd1a70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bd1a78, SUC_lexicon_tags_26 },
    { 0x1c1d1a79, SUC_lexicon_tags_27 },
    { 0x483d1a7a, SUC_lexicon_tags_27 },
    { 0x67dd1a7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6bd1a7d, SUC_lexicon_tags_1 },
    { 0xb77d1a7e, SUC_lexicon_tags_40 },
    { 0x88bd1a7d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957d1a83, SUC_lexicon_tags_6 },
    { 0x805d1a83, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d1a8d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadd1a91, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47d1a96, SUC_lexicon_tags_37 },
    { 0xab3d1a96, SUC_lexicon_tags_13 },
    { 0xbbdd1a98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fd1a9d, SUC_lexicon_tags_2 },
    { 0xabfd1a9e, SUC_lexicon_tags_37 },
    { 0x161d1a9f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe15d1aa1, SUC_lexicon_tags_11 },
    { 0xc59d1aa2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x24bd1aa4, SUC_lexicon_tags_12 },
    { 0x42bd1aa5, SUC_lexicon_tags_11 },
    { 0x4dbd1aa6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1d1aab, SUC_lexicon_tags_15 },
    { 0xfa1d1aac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x1bfd1aae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3f1d1ab0, SUC_lexicon_tags_2 },
    { 0x607d1ab1, SUC_lexicon_tags_34 },
    { 0x213d1ab2, SUC_lexicon_tags_30 },
    { 0x605d1ab3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7d1d1ab5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x40dd1ab7, SUC_lexicon_tags_1 },
    { 0x6bfd1ab8, SUC_lexicon_tags_63 },
    { 0x523d1ab9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493d1abe, SUC_lexicon_tags_27 },
    { 0x9f1d1abf, SUC_lexicon_tags_37 },
    { 0xba7d1ac0, SUC_lexicon_tags_25 },
    { 0x989d1abe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fd1ac9, SUC_lexicon_tags_8 },
    { 0x3b3d1aca, SUC_lexicon_tags_11 },
    { 0x25fd1aca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbbd1acd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563d1ad1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9d1ad4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x879d1ad6, SUC_lexicon_tags_39 },
    { 0x64fd1ad6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3d1add, SUC_lexicon_tags_2 },
    { 0x8dd1ade, SUC_lexicon_tags_6 },
    { 0xc27d1adf, SUC_lexicon_tags_43 },
    { 0x625d1ade, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeabd1ae2, SUC_lexicon_tags_11 },
    { 0x38bd1ae2, SUC_lexicon_tags_37 },
    { 0xce1d1ae4, SUC_lexicon_tags_7 },
    { 0x853d1ae3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29d1aed, SUC_lexicon_tags_68 },
    { 0x5add1aee, SUC_lexicon_tags_17 },
    { 0x4e1d1aef, SUC_lexicon_tags_66 },
    { 0x989d1af0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe75d1af3, SUC_lexicon_tags_38 },
    { 0x6f3d1af4, SUC_lexicon_tags_7 },
    { 0x899d1af5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33d1af9, SUC_lexicon_tags_1 },
    { 0xabdd1af9, SUC_lexicon_tags_11 },
    { 0x5e7d1afa, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fd1afe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f7d1b00, SUC_lexicon_tags_37 },
    { 0x20bd1b01, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc3fd1b03, SUC_lexicon_tags_1 },
    { 0x553d1b04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f3d1b06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec1d1b08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x55d1b0a, SUC_lexicon_tags_46 },
    { 0x61d1b0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x423d1b0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4d7d1b10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dd1b13, SUC_lexicon_tags_37 },
    { 0x219d1b13, SUC_lexicon_tags_7 },
    { 0xe71d1b14, SUC_lexicon_tags_19 },
    { 0xf39d1b16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449d1b1c, SUC_lexicon_tags_19 },
    { 0xcc9d1b1c, SUC_lexicon_tags_25 },
    { 0x7d5d1b1e, SUC_lexicon_tags_60 },
    { 0x53fd1b1f, SUC_lexicon_tags_32 },
    { 0xb87d1b1e, SUC_lexicon_tags_37 },
    { 0xd81d1b21, SUC_lexicon_tags_26 },
    { 0x757d1b22, SUC_lexicon_tags_26 },
    { 0xcadd1b23, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafd1b27, SUC_lexicon_tags_13 },
    { 0xf7fd1b27, SUC_lexicon_tags_1 },
    { 0x3dbd1b29, SUC_lexicon_tags_17 },
    { 0xd5bd1b2a, SUC_lexicon_tags_14 },
    { 0xaa9d1b27, SUC_lexicon_tags_28 },
    { 0x7e1d1b29, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19d1b35, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x93d1b37, SUC_lexicon_tags_1 },
    { 0x30fd1b37, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6c5d1b3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f9d1b3c, SUC_lexicon_tags_12 },
    { 0xc91d1b3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51d1b42, SUC_lexicon_tags_17 },
    { 0xf7d1b43, SUC_lexicon_tags_87 },
    { 0x365d1b44, SUC_lexicon_tags_17 },
    { 0x243d1b45, SUC_lexicon_tags_2 },
    { 0x1ddd1b44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc07d1b48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe67d1b4a, SUC_lexicon_tags_17 },
    { 0xe9d1b4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bd1b50, SUC_lexicon_tags_9 },
    { 0x837d1b50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4a9d1b53, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddd1b59, SUC_lexicon_tags_7 },
    { 0xf75d1b59, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x527d1b5d, SUC_lexicon_tags_43 },
    { 0xf31d1b5e, SUC_lexicon_tags_10 },
    { 0xa53d1b5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x79bd1b61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x45dd1b63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf91d1b65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x449d1b67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x905d1b6a, SUC_lexicon_tags_25 },
    { 0x3a9d1b6b, SUC_lexicon_tags_25 },
    { 0xdc3d1b6c, SUC_lexicon_tags_17 },
    { 0xc77d1b6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3d1b78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97bd1b7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3a9d1b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb3d1b7e, SUC_lexicon_tags_19 },
    { 0x4bd1b7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dd1b88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857d1b8c, SUC_lexicon_tags_47 },
    { 0xe25d1b8c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639d1b91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd89d1b93, SUC_lexicon_tags_23 },
    { 0xe67d1b94, SUC_lexicon_tags_47 },
    { 0xab5d1b95, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07d1b98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5d1b9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf65d1b9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635d1ba2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7d1ba7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1d1baa, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9d1bae, SUC_lexicon_tags_0 },
    { 0xe97d1baf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9d1bb6, SUC_lexicon_tags_7 },
    { 0x81fd1bb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d1bbb, SUC_lexicon_tags_14 },
    { 0x817d1bbc, SUC_lexicon_tags_33 },
    { 0x31d1bbc, SUC_lexicon_tags_26 },
    { 0xf07d1bbc, SUC_lexicon_tags_2 },
    { 0xf1fd1bbe, SUC_lexicon_tags_19 },
    { 0x23dd1bc0, SUC_lexicon_tags_2 },
    { 0x60bd1bc1, SUC_lexicon_tags_25 },
    { 0x959d1bc1, SUC_lexicon_tags_4 },
    { 0x78bd1bc2, SUC_lexicon_tags_14 },
    { 0x57bd1bc4, SUC_lexicon_tags_14 },
    { 0x685d1bbc, SUC_lexicon_tags_7 },
    { 0x8cdd1bc6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35dd1bce, SUC_lexicon_tags_7 },
    { 0x5c9d1bcf, SUC_lexicon_tags_43 },
    { 0xc1bd1bcf, SUC_lexicon_tags_17 },
    { 0x717d1bcf, SUC_lexicon_tags_10 },
    { 0x6d9d1bd2, SUC_lexicon_tags_37 },
    { 0xd0bd1bd3, SUC_lexicon_tags_23 },
    { 0xb33d1bd2, SUC_lexicon_tags_7 },
    { 0x757d1bd5, SUC_lexicon_tags_25 },
    { 0x819d1bd0, SUC_lexicon_tags_226 },
    { 0x405d1bd0, SUC_lexicon_tags_201 },
    { 0xdb3d1bd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5d1bdd, SUC_lexicon_tags_8 },
    { 0xdedd1bde, SUC_lexicon_tags_33 },
    { 0x8dfd1bdf, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3d1be2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadd1be7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bd1beb, SUC_lexicon_tags_11 },
    { 0xba7d1bec, SUC_lexicon_tags_43 },
    { 0x7c3d1bec, SUC_lexicon_tags_11 },
    { 0x499d1beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x625d1bf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bd1bf5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x387d1bf7, SUC_lexicon_tags_32 },
    { 0x75dd1bf8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43d1bfb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x865d1bfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x33fd1bff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9d1c05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41d1c08, SUC_lexicon_tags_65 },
    { 0xfe5d1c09, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x73d1c0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597d1c0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9d1c13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x167d1c16, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d1c1d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5d1c22, SUC_lexicon_tags_2 },
    { 0xa53d1c23, SUC_lexicon_tags_60 },
    { 0xc33d1c24, SUC_lexicon_tags_2 },
    { 0x17bd1c25, SUC_lexicon_tags_12 },
    { 0x45d1c26, SUC_lexicon_tags_37 },
    { 0xc55d1c23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbd1c2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b7d1c2f, SUC_lexicon_tags_11 },
    { 0x2bd1c30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcdbd1c32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xac5d1c34, SUC_lexicon_tags_27 },
    { 0xfcbd1c34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7d1c38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa97d1c3a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x13d1c3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x875d1c3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97d1c40, SUC_lexicon_tags_31 },
    { 0xffd1c41, SUC_lexicon_tags_19 },
    { 0x741d1c41, SUC_lexicon_tags_2 },
    { 0xc81d1c40, SUC_lexicon_tags_35 },
    { 0xd43d1c40, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x99d1c47, SUC_lexicon_tags_25 },
    { 0x833d1c47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbd1c4f, SUC_lexicon_tags_5 },
    { 0x757d1c4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603d1c55, SUC_lexicon_tags_33 },
    { 0x8c7d1c56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x649d1c59, SUC_lexicon_tags_43 },
    { 0xed5d1c5a, SUC_lexicon_tags_12 },
    { 0x4cbd1c5b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85d1c60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9d1c66, SUC_lexicon_tags_41 },
    { 0x767d1c66, SUC_lexicon_tags_11 },
    { 0x935d1c68, SUC_lexicon_tags_11 },
    { 0xda9d1c66, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2f9d1c6b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bd1c6f, SUC_lexicon_tags_17 },
    { 0x9e7d1c6f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91d1c73, SUC_lexicon_tags_17 },
    { 0xa83d1c73, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x12dd1c76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5bd1c78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x293d1c7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9d1c7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x239d1c80, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957d1c84, SUC_lexicon_tags_7 },
    { 0xe1bd1c85, SUC_lexicon_tags_1 },
    { 0x2ffd1c85, SUC_lexicon_tags_1 },
    { 0x569d1c85, SUC_lexicon_tags_27 },
    { 0x50dd1c84, SUC_lexicon_tags_15 },
    { 0x8a3d1c89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1d1c8f, SUC_lexicon_tags_47 },
    { 0x3ffd1c90, SUC_lexicon_tags_56 },
    { 0x729d1c91, SUC_lexicon_tags_14 },
    { 0xd67d1c90, SUC_lexicon_tags_43 },
    { 0xad5d1c93, SUC_lexicon_tags_1 },
    { 0x93fd1c94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bd1c9b, SUC_lexicon_tags_2 },
    { 0x279d1c9c, SUC_lexicon_tags_49 },
    { 0xc25d1c9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b5d1c9f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207d1ca7, SUC_lexicon_tags_10 },
    { 0x10fd1ca7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d1cae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x221d1cb0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7d1cb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9d1cb6, SUC_lexicon_tags_25 },
    { 0xbe3d1cb7, SUC_lexicon_tags_38 },
    { 0xe87d1cb7, SUC_lexicon_tags_7 },
    { 0xbb1d1cb7, SUC_lexicon_tags_25 },
    { 0x50dd1cba, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc2dd1cbc, SUC_lexicon_tags_12 },
    { 0x2a1d1cbd, SUC_lexicon_tags_25 },
    { 0xf3fd1cbe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fd1cc2, SUC_lexicon_tags_30 },
    { 0xc2dd1cc2, SUC_lexicon_tags_8 },
    { 0xdbbd1cc4, SUC_lexicon_tags_27 },
    { 0x2cbd1cc5, SUC_lexicon_tags_27 },
    { 0xd03d1cc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457d1cd1, SUC_lexicon_tags_7 },
    { 0x1bfd1cd2, SUC_lexicon_tags_947 },
    { 0x97bd1cd3, SUC_lexicon_tags_10 },
    { 0x721d1cd4, SUC_lexicon_tags_25 },
    { 0x6cfd1cd2, SUC_lexicon_tags_47 },
    { 0xa71d1cd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52dd1cd8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x30dd1cda, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47d1cdd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1d1ce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fd1ce4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x78dd1ce6, SUC_lexicon_tags_13 },
    { 0x5c9d1ce6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571d1cf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33d1cfa, SUC_lexicon_tags_14 },
    { 0xdadd1cfa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85d1cff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fbd1d01, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x82fd1d03, SUC_lexicon_tags_1 },
    { 0xb5fd1d04, SUC_lexicon_tags_6 },
    { 0x745d1d05, SUC_lexicon_tags_2 },
    { 0x837d1d06, SUC_lexicon_tags_25 },
    { 0x725d1d06, SUC_lexicon_tags_55 },
    { 0xf2fd1d08, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01d1d0b, SUC_lexicon_tags_15 },
    { 0x4fbd1d0c, SUC_lexicon_tags_15 },
    { 0x3b1d1d0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddd1d10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bd1d13, SUC_lexicon_tags_17 },
    { 0x25dd1d14, SUC_lexicon_tags_39 },
    { 0xdafd1d15, SUC_lexicon_tags_12 },
    { 0xddd1d14, SUC_lexicon_tags_115 },
    { 0, NULL },
    { 0xb87d1d18, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5d1d1c, SUC_lexicon_tags_32 },
    { 0x7a5d1d1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbfd1d1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90dd1d22, SUC_lexicon_tags_60 },
    { 0xe7dd1d23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x151d1d26, SUC_lexicon_tags_19 },
    { 0x333d1d27, SUC_lexicon_tags_1 },
    { 0x99dd1d28, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xefdd1d2a, SUC_lexicon_tags_12 },
    { 0x961d1d2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1d1d32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fd1d35, SUC_lexicon_tags_12 },
    { 0xb1dd1d36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1d1d39, SUC_lexicon_tags_26 },
    { 0xfdd1d39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6bd1d3c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fd1d41, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9d1d45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x72bd1d47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71d1d4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dd1d51, SUC_lexicon_tags_17 },
    { 0xf79d1d52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41d1d56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dd1d5b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91d1d62, SUC_lexicon_tags_49 },
    { 0xefd1d63, SUC_lexicon_tags_60 },
    { 0x913d1d64, SUC_lexicon_tags_25 },
    { 0xb15d1d65, SUC_lexicon_tags_84 },
    { 0xcf7d1d66, SUC_lexicon_tags_7 },
    { 0x945d1d66, SUC_lexicon_tags_7 },
    { 0x6c3d1d68, SUC_lexicon_tags_25 },
    { 0x423d1d68, SUC_lexicon_tags_25 },
    { 0xe01d1d67, SUC_lexicon_tags_10 },
    { 0x14fd1d6b, SUC_lexicon_tags_1 },
    { 0x8f5d1d6c, SUC_lexicon_tags_37 },
    { 0x9b7d1d6b, SUC_lexicon_tags_1 },
    { 0xe05d1d6e, SUC_lexicon_tags_47 },
    { 0x18bd1d6e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x2fd1d71, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855d1d79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb63d1d7b, SUC_lexicon_tags_13 },
    { 0x4d7d1d7c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5d1d80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bd1d90, SUC_lexicon_tags_37 },
    { 0x3bdd1d91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3d1d95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b5d1d97, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793d1d9b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47d1d9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bd1da6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbd1dac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e7d1dae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbd1db1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7d1db4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x809d1db6, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x1bdd1db8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x13d1dba, SUC_lexicon_tags_19 },
    { 0xa19d1dbb, SUC_lexicon_tags_17 },
    { 0xec5d1dba, SUC_lexicon_tags_15 },
    { 0x2b7d1dbd, SUC_lexicon_tags_77 },
    { 0x60bd1dbd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fd1dc6, SUC_lexicon_tags_33 },
    { 0xd79d1dc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73d1dca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0dd1dcc, SUC_lexicon_tags_17 },
    { 0xe85d1dcd, SUC_lexicon_tags_47 },
    { 0x47dd1dce, SUC_lexicon_tags_11 },
    { 0x10dd1dcf, SUC_lexicon_tags_29 },
    { 0xbfd1dcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5d1dd3, SUC_lexicon_tags_1 },
    { 0x61fd1dd4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311d1dda, SUC_lexicon_tags_11 },
    { 0x19bd1ddb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7d3d1ddd, SUC_lexicon_tags_37 },
    { 0xeefd1dde, SUC_lexicon_tags_25 },
    { 0x2dbd1dde, SUC_lexicon_tags_17 },
    { 0xc8dd1dde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fd1de3, SUC_lexicon_tags_12 },
    { 0xdbd1de4, SUC_lexicon_tags_16 },
    { 0x915d1de5, SUC_lexicon_tags_65 },
    { 0xe15d1de5, SUC_lexicon_tags_7 },
    { 0xdd1d1de7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5d1ded, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49d1df5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07d1dfb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefd1e00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5c7d1e02, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275d1e06, SUC_lexicon_tags_0 },
    { 0xb91d1e06, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3d1e0e, SUC_lexicon_tags_25 },
    { 0x43bd1e0f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63d1e1a, SUC_lexicon_tags_0 },
    { 0x4dd1e1b, SUC_lexicon_tags_62 },
    { 0x943d1e1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x735d1e1e, SUC_lexicon_tags_33 },
    { 0x5dbd1e1e, SUC_lexicon_tags_31 },
    { 0x883d1e20, SUC_lexicon_tags_6 },
    { 0xd45d1e21, SUC_lexicon_tags_13 },
    { 0xb27d1e21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd45d1e24, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493d1e29, SUC_lexicon_tags_79 },
    { 0x2ddd1e2a, SUC_lexicon_tags_42 },
    { 0xc55d1e2a, SUC_lexicon_tags_55 },
    { 0x47dd1e2a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d1e31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fd1e36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7d1e39, SUC_lexicon_tags_37 },
    { 0x34dd1e39, SUC_lexicon_tags_33 },
    { 0x861d1e3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ddd1e3d, SUC_lexicon_tags_14 },
    { 0x7d9d1e3d, SUC_lexicon_tags_39 },
    { 0xd7d1e3d, SUC_lexicon_tags_15 },
    { 0x663d1e3d, SUC_lexicon_tags_37 },
    { 0x3ebd1e41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x491d1e43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fd1e49, SUC_lexicon_tags_11 },
    { 0xff5d1e4a, SUC_lexicon_tags_0 },
    { 0x245d1e4a, SUC_lexicon_tags_37 },
    { 0x895d1e4c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb2dd1e4e, SUC_lexicon_tags_17 },
    { 0x2a7d1e4f, SUC_lexicon_tags_79 },
    { 0x3b3d1e50, SUC_lexicon_tags_15 },
    { 0x965d1e4f, SUC_lexicon_tags_28 },
    { 0xba9d1e52, SUC_lexicon_tags_12 },
    { 0x469d1e53, SUC_lexicon_tags_2 },
    { 0x67dd1e54, SUC_lexicon_tags_45 },
    { 0x3dd1e51, SUC_lexicon_tags_45 },
    { 0xf6dd1e55, SUC_lexicon_tags_37 },
    { 0x955d1e52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdbd1e59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbd1e60, SUC_lexicon_tags_79 },
    { 0x87dd1e60, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5d1e66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb77d1e68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d1e6b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa99d1e6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbd1e71, SUC_lexicon_tags_7 },
    { 0xe3fd1e72, SUC_lexicon_tags_30 },
    { 0x11d1e73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fd1e75, SUC_lexicon_tags_12 },
    { 0x41dd1e76, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbd1e79, SUC_lexicon_tags_3 },
    { 0xf81d1e79, SUC_lexicon_tags_10 },
    { 0x46bd1e7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d1e7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5d1e80, SUC_lexicon_tags_33 },
    { 0xc8dd1e80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd29d1e83, SUC_lexicon_tags_7 },
    { 0xb03d1e84, SUC_lexicon_tags_151 },
    { 0xea3d1e84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95d1e89, SUC_lexicon_tags_43 },
    { 0xa47d1e89, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x615d1e8c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x27dd1e8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a5d1e91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77d1e9b, SUC_lexicon_tags_26 },
    { 0x5efd1e9c, SUC_lexicon_tags_25 },
    { 0x6fdd1e9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fd1ea0, SUC_lexicon_tags_2 },
    { 0xaa7d1ea0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x997d1ea4, SUC_lexicon_tags_38 },
    { 0xa4fd1ea4, SUC_lexicon_tags_63 },
    { 0x501d1ea6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x331d1ea8, SUC_lexicon_tags_60 },
    { 0x5afd1ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09d1ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dd1eb1, SUC_lexicon_tags_1 },
    { 0x5fd1eb1, SUC_lexicon_tags_35 },
    { 0x861d1eb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5d1eb8, SUC_lexicon_tags_33 },
    { 0x5cbd1eb8, SUC_lexicon_tags_12 },
    { 0x123d1eba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf1d1ebc, SUC_lexicon_tags_60 },
    { 0x6d7d1ebd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1dbd1ebf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe67d1ec1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6e7d1ec3, SUC_lexicon_tags_12 },
    { 0x27d1ec3, SUC_lexicon_tags_7 },
    { 0x77dd1ec3, SUC_lexicon_tags_17 },
    { 0x40dd1ec4, SUC_lexicon_tags_25 },
    { 0xc99d1ec7, SUC_lexicon_tags_146 },
    { 0x3bbd1ec4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d1ecb, SUC_lexicon_tags_33 },
    { 0x757d1ecc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4fd1ece, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e1d1ed0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575d1edb, SUC_lexicon_tags_12 },
    { 0x659d1edb, SUC_lexicon_tags_17 },
    { 0xa9dd1edc, SUC_lexicon_tags_83 },
    { 0x229d1ede, SUC_lexicon_tags_17 },
    { 0xf77d1edf, SUC_lexicon_tags_29 },
    { 0xff1d1edf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf69d1ee2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73d1ee7, SUC_lexicon_tags_17 },
    { 0xb4fd1ee7, SUC_lexicon_tags_2 },
    { 0x1bfd1ee9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79d1eee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x721d1ef0, SUC_lexicon_tags_39 },
    { 0xc91d1ef1, SUC_lexicon_tags_37 },
    { 0x9ebd1ef2, SUC_lexicon_tags_37 },
    { 0xae3d1ef3, SUC_lexicon_tags_1 },
    { 0xdb5d1ef4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb91d1ef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9d1ef9, SUC_lexicon_tags_0 },
    { 0x243d1efa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5d1efd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbd1f02, SUC_lexicon_tags_25 },
    { 0x3b9d1f03, SUC_lexicon_tags_25 },
    { 0x63fd1f04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87d1f0a, SUC_lexicon_tags_26 },
    { 0xd4fd1f0b, SUC_lexicon_tags_46 },
    { 0xd1bd1f0a, SUC_lexicon_tags_2 },
    { 0xc5bd1f0a, SUC_lexicon_tags_2 },
    { 0xb01d1f0c, SUC_lexicon_tags_1 },
    { 0xb31d1f0f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcbd1f11, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x949d1f13, SUC_lexicon_tags_45 },
    { 0xe55d1f13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x857d1f16, SUC_lexicon_tags_30 },
    { 0xa1bd1f16, SUC_lexicon_tags_107 },
    { 0xea7d1f17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15d1f1e, SUC_lexicon_tags_19 },
    { 0x4dd1f1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dfd1f21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x831d1f23, SUC_lexicon_tags_2 },
    { 0x23bd1f24, SUC_lexicon_tags_45 },
    { 0xf25d1f25, SUC_lexicon_tags_27 },
    { 0x8b7d1f25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdd1f2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5d1f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7d1f32, SUC_lexicon_tags_31 },
    { 0xb61d1f33, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429d1f3b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x105d1f3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddd1f43, SUC_lexicon_tags_1 },
    { 0x40fd1f44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e3d1f46, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fd1f49, SUC_lexicon_tags_28 },
    { 0xa43d1f4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5d1f4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadd1f52, SUC_lexicon_tags_91 },
    { 0x951d1f53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bd1f58, SUC_lexicon_tags_23 },
    { 0x9bdd1f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e7d1f5b, SUC_lexicon_tags_47 },
    { 0x213d1f5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3d1f63, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dd1f67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bd1f6a, SUC_lexicon_tags_84 },
    { 0x5e3d1f6b, SUC_lexicon_tags_37 },
    { 0xb03d1f6c, SUC_lexicon_tags_82 },
    { 0xa4dd1f6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd53d1f6f, SUC_lexicon_tags_15 },
    { 0xf5dd1f70, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x50fd1f72, SUC_lexicon_tags_2 },
    { 0x5c7d1f73, SUC_lexicon_tags_37 },
    { 0xe63d1f72, SUC_lexicon_tags_17 },
    { 0x573d1f74, SUC_lexicon_tags_25 },
    { 0x73bd1f75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559d1f7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63bd1f7d, SUC_lexicon_tags_26 },
    { 0xfdfd1f7d, SUC_lexicon_tags_0 },
    { 0x247d1f7f, SUC_lexicon_tags_15 },
    { 0x923d1f7e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x199d1f82, SUC_lexicon_tags_23 },
    { 0x2f3d1f82, SUC_lexicon_tags_7 },
    { 0x9edd1f82, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2a5d1f86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf85d1f88, SUC_lexicon_tags_45 },
    { 0x621d1f88, SUC_lexicon_tags_25 },
    { 0x4bd1f88, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dd1f91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5add1f93, SUC_lexicon_tags_15 },
    { 0x8a9d1f94, SUC_lexicon_tags_14 },
    { 0x89d1f95, SUC_lexicon_tags_10 },
    { 0xeabd1f95, SUC_lexicon_tags_38 },
    { 0x489d1f95, SUC_lexicon_tags_47 },
    { 0x56dd1f96, SUC_lexicon_tags_37 },
    { 0xacfd1f97, SUC_lexicon_tags_17 },
    { 0xc69d1f94, SUC_lexicon_tags_1 },
    { 0xe71d1f99, SUC_lexicon_tags_20 },
    { 0x2fbd1f99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dd1f9f, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0x371d1fa1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddd1fa6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533d1fab, SUC_lexicon_tags_7 },
    { 0x321d1fac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1d1fb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fd1fb4, SUC_lexicon_tags_37 },
    { 0x769d1fb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53d1fb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685d1fbd, SUC_lexicon_tags_37 },
    { 0xc4bd1fbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657d1fc3, SUC_lexicon_tags_31 },
    { 0xb59d1fc4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbd1fc7, SUC_lexicon_tags_18 },
    { 0x54dd1fc8, SUC_lexicon_tags_14 },
    { 0xa99d1fc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdd1fcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x371d1fce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85d1fd2, SUC_lexicon_tags_37 },
    { 0x651d1fd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3d1fd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27d1fda, SUC_lexicon_tags_1 },
    { 0xb7bd1fdb, SUC_lexicon_tags_15 },
    { 0x747d1fda, SUC_lexicon_tags_64 },
    { 0x7b3d1fda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65d1fe0, SUC_lexicon_tags_7 },
    { 0x2fbd1fe1, SUC_lexicon_tags_2 },
    { 0xaa3d1fe0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35d1fe8, SUC_lexicon_tags_12 },
    { 0x433d1fe9, SUC_lexicon_tags_63 },
    { 0x9bbd1fe9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d9d1fec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abd1ff5, SUC_lexicon_tags_14 },
    { 0x72bd1ff6, SUC_lexicon_tags_118 },
    { 0xb1fd1ff6, SUC_lexicon_tags_66 },
    { 0xe2bd1ff8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb5d1ffa, SUC_lexicon_tags_37 },
    { 0x1ffd1ffa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x29fd1ffd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7fd1fff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa63d2001, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bd2004, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87d2007, SUC_lexicon_tags_2 },
    { 0x7b7d2008, SUC_lexicon_tags_63 },
    { 0xee5d2008, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fd200e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3d2016, SUC_lexicon_tags_10 },
    { 0x84bd2017, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x719d2019, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a9d201b, SUC_lexicon_tags_2 },
    { 0x31d201c, SUC_lexicon_tags_8 },
    { 0x8abd201c, SUC_lexicon_tags_1 },
    { 0x85d201d, SUC_lexicon_tags_19 },
    { 0xf5fd201f, SUC_lexicon_tags_23 },
    { 0x74dd2020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efd2024, SUC_lexicon_tags_7 },
    { 0xc11d2025, SUC_lexicon_tags_37 },
    { 0x9e1d2026, SUC_lexicon_tags_27 },
    { 0xbbfd2024, SUC_lexicon_tags_25 },
    { 0x667d2028, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fd202b, SUC_lexicon_tags_37 },
    { 0x36dd202c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47d2030, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x94bd2032, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc0fd2034, SUC_lexicon_tags_37 },
    { 0x26fd2034, SUC_lexicon_tags_43 },
    { 0x32bd2036, SUC_lexicon_tags_343 },
    { 0x19fd2037, SUC_lexicon_tags_23 },
    { 0x985d2037, SUC_lexicon_tags_68 },
    { 0xadbd2038, SUC_lexicon_tags_17 },
    { 0x995d203a, SUC_lexicon_tags_29 },
    { 0xe77d203b, SUC_lexicon_tags_28 },
    { 0xe5bd2037, SUC_lexicon_tags_23 },
    { 0xc5d203d, SUC_lexicon_tags_31 },
    { 0x26bd203d, SUC_lexicon_tags_37 },
    { 0xa69d203f, SUC_lexicon_tags_12 },
    { 0xe45d2040, SUC_lexicon_tags_22 },
    { 0x7fbd2040, SUC_lexicon_tags_17 },
    { 0xb63d203f, SUC_lexicon_tags_14 },
    { 0x41d2040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dd2047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fd204d, SUC_lexicon_tags_84 },
    { 0xa93d204d, SUC_lexicon_tags_30 },
    { 0xbedd204f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bd2052, SUC_lexicon_tags_87 },
    { 0x8d1d2052, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbd205a, SUC_lexicon_tags_46 },
    { 0xb65d205b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217d205f, SUC_lexicon_tags_28 },
    { 0x2add2060, SUC_lexicon_tags_37 },
    { 0xec5d2061, SUC_lexicon_tags_11 },
    { 0xd07d2061, SUC_lexicon_tags_2 },
    { 0x1a5d2063, SUC_lexicon_tags_37 },
    { 0x2c1d2061, SUC_lexicon_tags_19 },
    { 0xb4fd2062, SUC_lexicon_tags_63 },
    { 0xd21d2062, SUC_lexicon_tags_47 },
    { 0x2edd2067, SUC_lexicon_tags_33 },
    { 0xf09d2068, SUC_lexicon_tags_31 },
    { 0xa6bd2067, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x31fd206b, SUC_lexicon_tags_15 },
    { 0x35dd206c, SUC_lexicon_tags_108 },
    { 0x5a3d206b, SUC_lexicon_tags_57 },
    { 0x213d206d, SUC_lexicon_tags_11 },
    { 0xe8fd206f, SUC_lexicon_tags_17 },
    { 0x75d2070, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdb7d2072, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa5bd2074, SUC_lexicon_tags_15 },
    { 0x233d2074, SUC_lexicon_tags_12 },
    { 0x7d5d2076, SUC_lexicon_tags_46 },
    { 0xe9dd2077, SUC_lexicon_tags_33 },
    { 0x53fd2077, SUC_lexicon_tags_29 },
    { 0xf1bd2077, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293d207e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadd2082, SUC_lexicon_tags_23 },
    { 0xe3bd2083, SUC_lexicon_tags_38 },
    { 0xdedd2084, SUC_lexicon_tags_25 },
    { 0xb51d2085, SUC_lexicon_tags_37 },
    { 0xa95d2086, SUC_lexicon_tags_27 },
    { 0x225d2083, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa87d2089, SUC_lexicon_tags_43 },
    { 0xf25d2089, SUC_lexicon_tags_63 },
    { 0x845d208b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d208e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3d2093, SUC_lexicon_tags_17 },
    { 0x389d2094, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcf9d2096, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9efd2098, SUC_lexicon_tags_25 },
    { 0x123d2098, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21d209c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x549d209e, SUC_lexicon_tags_33 },
    { 0xa27d209f, SUC_lexicon_tags_11 },
    { 0xb51d20a0, SUC_lexicon_tags_13 },
    { 0x8c5d20a0, SUC_lexicon_tags_15 },
    { 0x321d20a0, SUC_lexicon_tags_296 },
    { 0xd97d20a1, SUC_lexicon_tags_37 },
    { 0xe29d20a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x81d20a7, SUC_lexicon_tags_17 },
    { 0x5bdd20a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29d20ad, SUC_lexicon_tags_33 },
    { 0x775d20ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755d20b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1d20b7, SUC_lexicon_tags_2 },
    { 0x6a1d20b8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33d20be, SUC_lexicon_tags_30 },
    { 0x7d7d20bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55d20c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d20c6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x673d20c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x299d20ca, SUC_lexicon_tags_94 },
    { 0x9f3d20cb, SUC_lexicon_tags_378 },
    { 0x7edd20cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dd20d0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfd20d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61d20d8, SUC_lexicon_tags_19 },
    { 0x8bdd20d8, SUC_lexicon_tags_37 },
    { 0xc71d20da, SUC_lexicon_tags_2 },
    { 0xe63d20d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9add20e0, SUC_lexicon_tags_25 },
    { 0xe5bd20e1, SUC_lexicon_tags_12 },
    { 0x871d20e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489d20e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfd20ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x1d7d20ef, SUC_lexicon_tags_15 },
    { 0x41dd20f0, SUC_lexicon_tags_15 },
    { 0x673d20f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31d20fb, SUC_lexicon_tags_28 },
    { 0xeffd20fb, SUC_lexicon_tags_45 },
    { 0xce7d20fd, SUC_lexicon_tags_26 },
    { 0xe75d20fd, SUC_lexicon_tags_37 },
    { 0x489d20fe, SUC_lexicon_tags_34 },
    { 0x71bd20fc, SUC_lexicon_tags_29 },
    { 0x1fd2101, SUC_lexicon_tags_38 },
    { 0xb23d2102, SUC_lexicon_tags_19 },
    { 0x65d2102, SUC_lexicon_tags_2 },
    { 0x759d2104, SUC_lexicon_tags_30 },
    { 0xc03d2104, SUC_lexicon_tags_14 },
    { 0x8ebd2103, SUC_lexicon_tags_14 },
    { 0xfc9d2107, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773d210e, SUC_lexicon_tags_7 },
    { 0x1ddd210f, SUC_lexicon_tags_15 },
    { 0xbadd2110, SUC_lexicon_tags_59 },
    { 0xe97d2111, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60dd2115, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9d211e, SUC_lexicon_tags_2 },
    { 0x269d211e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x27fd2121, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85d2127, SUC_lexicon_tags_66 },
    { 0xdf7d2128, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8fd212a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afd2131, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefd2135, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xfe3d2137, SUC_lexicon_tags_60 },
    { 0x8b3d2137, SUC_lexicon_tags_12 },
    { 0xbe5d2139, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9d213c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885d2141, SUC_lexicon_tags_37 },
    { 0x55d2141, SUC_lexicon_tags_1 },
    { 0x5cbd2143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17d2145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1d2149, SUC_lexicon_tags_24 },
    { 0x791d2149, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x19d214c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x483d214e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x50bd2150, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607d2154, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x353d2157, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafd215a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dd215e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd2162, SUC_lexicon_tags_23 },
    { 0x1cbd2163, SUC_lexicon_tags_37 },
    { 0x2b3d2164, SUC_lexicon_tags_65 },
    { 0x5f7d2165, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdd216a, SUC_lexicon_tags_55 },
    { 0x7dd216b, SUC_lexicon_tags_17 },
    { 0xb83d216c, SUC_lexicon_tags_17 },
    { 0x3c1d216c, SUC_lexicon_tags_12 },
    { 0x179d216b, SUC_lexicon_tags_2 },
    { 0x3bdd216f, SUC_lexicon_tags_17 },
    { 0x401d216f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dd2177, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fbd2179, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd217e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe89d2180, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd2183, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x22bd2185, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65d2189, SUC_lexicon_tags_19 },
    { 0xa1d2189, SUC_lexicon_tags_55 },
    { 0x7add218b, SUC_lexicon_tags_17 },
    { 0x7dbd218c, SUC_lexicon_tags_37 },
    { 0xf31d218d, SUC_lexicon_tags_138 },
    { 0x73d218e, SUC_lexicon_tags_17 },
    { 0x4c7d218a, SUC_lexicon_tags_63 },
    { 0xac5d2190, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd5d2192, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dd219a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x92bd219c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd21a0, SUC_lexicon_tags_0 },
    { 0xfdd21a1, SUC_lexicon_tags_31 },
    { 0x97dd21a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x34fd21a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8add21ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7d21b1, SUC_lexicon_tags_43 },
    { 0x209d21b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13d21b6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x953d21b9, SUC_lexicon_tags_1 },
    { 0xa0fd21b9, SUC_lexicon_tags_11 },
    { 0x11dd21bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x561d21bd, SUC_lexicon_tags_15 },
    { 0xb87d21bd, SUC_lexicon_tags_47 },
    { 0x2bd21bf, SUC_lexicon_tags_25 },
    { 0x467d21c0, SUC_lexicon_tags_12 },
    { 0xc65d21c1, SUC_lexicon_tags_7 },
    { 0x793d21c0, SUC_lexicon_tags_12 },
    { 0x775d21c0, SUC_lexicon_tags_87 },
    { 0x27dd21c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9d21cb, SUC_lexicon_tags_15 },
    { 0x4f9d21cc, SUC_lexicon_tags_47 },
    { 0xf2fd21cd, SUC_lexicon_tags_17 },
    { 0xfd5d21cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5d21d3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d21de, SUC_lexicon_tags_37 },
    { 0xeedd21df, SUC_lexicon_tags_37 },
    { 0x78fd21e0, SUC_lexicon_tags_32 },
    { 0x57bd21df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe9d21e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdd21e9, SUC_lexicon_tags_47 },
    { 0xf99d21ea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705d21f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3d21f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe83d21f7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bd21fc, SUC_lexicon_tags_4 },
    { 0x5d5d21fd, SUC_lexicon_tags_11 },
    { 0x233d21fe, SUC_lexicon_tags_31 },
    { 0x159d21ff, SUC_lexicon_tags_49 },
    { 0x2c7d21fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc89d2202, SUC_lexicon_tags_15 },
    { 0xe37d2202, SUC_lexicon_tags_27 },
    { 0x7add2204, SUC_lexicon_tags_37 },
    { 0x1b7d2205, SUC_lexicon_tags_60 },
    { 0xc8dd2206, SUC_lexicon_tags_10 },
    { 0xddd2206, SUC_lexicon_tags_2 },
    { 0x9d3d2207, SUC_lexicon_tags_1256 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5d220e, SUC_lexicon_tags_1 },
    { 0xa5d220f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x59fd2211, SUC_lexicon_tags_0 },
    { 0xccfd2212, SUC_lexicon_tags_47 },
    { 0x6abd2212, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155d221d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3d2223, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5add2225, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88fd2228, SUC_lexicon_tags_6 },
    { 0xb59d2229, SUC_lexicon_tags_29 },
    { 0x895d2228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd222d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x283d222f, SUC_lexicon_tags_583 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621d2234, SUC_lexicon_tags_65 },
    { 0x6f5d2235, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfd2238, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b7d223a, SUC_lexicon_tags_0 },
    { 0xd3dd223b, SUC_lexicon_tags_31 },
    { 0x913d223c, SUC_lexicon_tags_9 },
    { 0x12dd223d, SUC_lexicon_tags_11 },
    { 0xb71d223d, SUC_lexicon_tags_1 },
    { 0xa0bd223f, SUC_lexicon_tags_15 },
    { 0xcf5d223f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fd2245, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1bd2247, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x143d224a, SUC_lexicon_tags_11 },
    { 0xa0bd224b, SUC_lexicon_tags_45 },
    { 0x8a5d224b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x151d224f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dd2252, SUC_lexicon_tags_15 },
    { 0xe91d2252, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fd2259, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35d225f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199d2263, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x733d2265, SUC_lexicon_tags_12 },
    { 0x377d2265, SUC_lexicon_tags_21 },
    { 0xdebd2266, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdcbd2269, SUC_lexicon_tags_2 },
    { 0x11fd226a, SUC_lexicon_tags_15 },
    { 0x97bd226b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851d226f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d2277, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x669d227a, SUC_lexicon_tags_65 },
    { 0x9cfd227a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d227e, SUC_lexicon_tags_12 },
    { 0x23fd227f, SUC_lexicon_tags_1 },
    { 0x219d2280, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x837d2283, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7d2286, SUC_lexicon_tags_38 },
    { 0xec9d2287, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbd228b, SUC_lexicon_tags_16 },
    { 0xdcbd228c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bd2291, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64fd2293, SUC_lexicon_tags_15 },
    { 0x65dd2294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255d22a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x579d22a4, SUC_lexicon_tags_55 },
    { 0x715d22a4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xabbd22a7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadd22ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31fd22ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103d22b3, SUC_lexicon_tags_12 },
    { 0x6cfd22b3, SUC_lexicon_tags_30 },
    { 0xef7d22b5, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5d22b8, SUC_lexicon_tags_2 },
    { 0x54bd22b9, SUC_lexicon_tags_37 },
    { 0xe3d22b9, SUC_lexicon_tags_47 },
    { 0xbf3d22ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3d22be, SUC_lexicon_tags_12 },
    { 0xf77d22bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79d22c3, SUC_lexicon_tags_57 },
    { 0x18dd22c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1d22c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dd22cd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x43dd22d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x281d22d2, SUC_lexicon_tags_711 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7d22d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa5bd22d7, SUC_lexicon_tags_60 },
    { 0x6f5d22d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efd22dd, SUC_lexicon_tags_25 },
    { 0x23bd22dd, SUC_lexicon_tags_30 },
    { 0x8fdd22df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9d22e3, SUC_lexicon_tags_2 },
    { 0x201d22e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7d22e9, SUC_lexicon_tags_215 },
    { 0x78dd22e9, SUC_lexicon_tags_13 },
    { 0xb8dd22eb, SUC_lexicon_tags_2 },
    { 0x4bbd22ec, SUC_lexicon_tags_47 },
    { 0x12fd22ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x53dd22f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6add22f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x477d22f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd49d22f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc9d22f8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561d22fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7d22ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6d3d2301, SUC_lexicon_tags_7 },
    { 0xb0dd2301, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695d2309, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x75dd230b, SUC_lexicon_tags_37 },
    { 0xcbfd230b, SUC_lexicon_tags_17 },
    { 0xba3d230d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xc55d230f, SUC_lexicon_tags_47 },
    { 0x85fd2310, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6b7d2312, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91d2315, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473d231d, SUC_lexicon_tags_55 },
    { 0x3bbd231e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97d2320, SUC_lexicon_tags_439 },
    { 0, NULL },
    { 0x5cbd2322, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x229d2324, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fd232c, SUC_lexicon_tags_63 },
    { 0x259d232d, SUC_lexicon_tags_15 },
    { 0x41bd232d, SUC_lexicon_tags_98 },
    { 0x3d9d232f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d2332, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x887d2334, SUC_lexicon_tags_7 },
    { 0xad7d2334, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25fd2337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x803d2339, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xae3d233b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43d2345, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879d234b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fd2350, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x92fd2352, SUC_lexicon_tags_13 },
    { 0x499d2352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x53dd2355, SUC_lexicon_tags_65 },
    { 0x1c3d2356, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad1d2358, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x42fd235b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5d2360, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7d2366, SUC_lexicon_tags_11 },
    { 0xbf3d2367, SUC_lexicon_tags_6 },
    { 0x7a9d2368, SUC_lexicon_tags_32 },
    { 0xff5d2367, SUC_lexicon_tags_65 },
    { 0x381d2369, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x749d236c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dd2372, SUC_lexicon_tags_12 },
    { 0x1e1d2372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x713d2375, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd2378, SUC_lexicon_tags_8 },
    { 0xb3bd2379, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7d237c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fd237f, SUC_lexicon_tags_30 },
    { 0x351d237f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5f5d2382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dd2385, SUC_lexicon_tags_46 },
    { 0x14bd2385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x743d2388, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6bd238a, SUC_lexicon_tags_242 },
    { 0xcefd238b, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x5f1d238d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcabd2390, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5d2394, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883d2398, SUC_lexicon_tags_10 },
    { 0xf17d2399, SUC_lexicon_tags_8 },
    { 0x9edd239a, SUC_lexicon_tags_38 },
    { 0xdbd2398, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91d239e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d23a2, SUC_lexicon_tags_63 },
    { 0xe6bd23a2, SUC_lexicon_tags_25 },
    { 0xc11d23a3, SUC_lexicon_tags_17 },
    { 0x8ebd23a2, SUC_lexicon_tags_4 },
    { 0x11dd23a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x42dd23a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xea9d23ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871d23b0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331d23b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245d23b8, SUC_lexicon_tags_25 },
    { 0xed1d23b9, SUC_lexicon_tags_19 },
    { 0xce7d23b8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x485d23bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9d23c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7d23c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75d23ce, SUC_lexicon_tags_182 },
    { 0x137d23ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x947d23d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755d23d6, SUC_lexicon_tags_17 },
    { 0xc2bd23d7, SUC_lexicon_tags_25 },
    { 0x9f7d23d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5d23e2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb1bd23e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fd23e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bd23eb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfd23ee, SUC_lexicon_tags_2 },
    { 0x535d23ef, SUC_lexicon_tags_84 },
    { 0x84bd23f0, SUC_lexicon_tags_61 },
    { 0xa71d23f1, SUC_lexicon_tags_16 },
    { 0xdfdd23f0, SUC_lexicon_tags_26 },
    { 0x91dd23f3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897d23f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d23ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135d240c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x601d240e, SUC_lexicon_tags_57 },
    { 0xcfd240f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3d2412, SUC_lexicon_tags_11 },
    { 0x6d9d2412, SUC_lexicon_tags_10 },
    { 0xcf7d2414, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3d2418, SUC_lexicon_tags_28 },
    { 0x1fd2419, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebd241e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc09d2420, SUC_lexicon_tags_71 },
    { 0x761d2421, SUC_lexicon_tags_27 },
    { 0x50bd2421, SUC_lexicon_tags_27 },
    { 0x2c1d2423, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5add2425, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf77d2427, SUC_lexicon_tags_2 },
    { 0xd79d2427, SUC_lexicon_tags_10 },
    { 0xcd3d2427, SUC_lexicon_tags_2 },
    { 0x289d242a, SUC_lexicon_tags_9 },
    { 0x981d242b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbd242f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77d2437, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x561d2439, SUC_lexicon_tags_37 },
    { 0x2c1d2439, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dd243d, SUC_lexicon_tags_33 },
    { 0xf07d243d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bd2443, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11d244c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf2fd244e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf89d2450, SUC_lexicon_tags_4 },
    { 0x171d2451, SUC_lexicon_tags_54 },
    { 0x5cbd2451, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeefd2454, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3d2459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15d245d, SUC_lexicon_tags_32 },
    { 0x7c5d245e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x407d2460, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fd2464, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507d2468, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf83d246a, SUC_lexicon_tags_11 },
    { 0xf25d246b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5d2470, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bd247c, SUC_lexicon_tags_17 },
    { 0x34bd247d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e1d247f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97d2484, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bd2487, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdd248e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x90dd2490, SUC_lexicon_tags_55 },
    { 0xefd2490, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39d2496, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dd249b, SUC_lexicon_tags_8 },
    { 0xe99d249c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dd24a6, SUC_lexicon_tags_27 },
    { 0xcebd24a7, SUC_lexicon_tags_17 },
    { 0xd17d24a8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf5dd24aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd24ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacdd24af, SUC_lexicon_tags_17 },
    { 0xf77d24b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26dd24b6, SUC_lexicon_tags_8 },
    { 0xef7d24b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01d24ba, SUC_lexicon_tags_10 },
    { 0xe0fd24ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fd24c0, SUC_lexicon_tags_11 },
    { 0xd15d24c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdd24c9, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fd24d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bd24d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5d24dc, SUC_lexicon_tags_2 },
    { 0xb23d24dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c9d24df, SUC_lexicon_tags_25 },
    { 0x585d24df, SUC_lexicon_tags_19 },
    { 0xee1d24e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d24e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc57d24e7, SUC_lexicon_tags_17 },
    { 0xea5d24e7, SUC_lexicon_tags_43 },
    { 0x81bd24e9, SUC_lexicon_tags_62 },
    { 0xb49d24ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839d24f3, SUC_lexicon_tags_37 },
    { 0xfe1d24f3, SUC_lexicon_tags_27 },
    { 0x8c3d24f3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x905d24f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x677d24fa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa13d24fc, SUC_lexicon_tags_1 },
    { 0x433d24fd, SUC_lexicon_tags_38 },
    { 0x8dbd24fe, SUC_lexicon_tags_37 },
    { 0xd7fd24fe, SUC_lexicon_tags_45 },
    { 0x83fd24fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fd2508, SUC_lexicon_tags_21 },
    { 0xd15d2508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe63d250b, SUC_lexicon_tags_60 },
    { 0xf8fd250b, SUC_lexicon_tags_12 },
    { 0x835d250c, SUC_lexicon_tags_2 },
    { 0xc09d250b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44fd2510, SUC_lexicon_tags_5 },
    { 0xb9dd2511, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x63dd2513, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x23dd2515, SUC_lexicon_tags_3 },
    { 0x6dbd2516, SUC_lexicon_tags_45 },
    { 0xd33d2516, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc0fd2519, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x957d251c, SUC_lexicon_tags_23 },
    { 0x617d251c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d2521, SUC_lexicon_tags_40 },
    { 0x7fd2522, SUC_lexicon_tags_9 },
    { 0xd39d2522, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebd2529, SUC_lexicon_tags_15 },
    { 0x821d252a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97d252d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fd2532, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5d2538, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dd253c, SUC_lexicon_tags_11 },
    { 0x141d253d, SUC_lexicon_tags_19 },
    { 0xefd253e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebd2542, SUC_lexicon_tags_37 },
    { 0x2bfd2542, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bd2547, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x37d254a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfd254d, SUC_lexicon_tags_61 },
    { 0xd81d254e, SUC_lexicon_tags_9 },
    { 0xd15d254f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871d2558, SUC_lexicon_tags_27 },
    { 0x74bd2559, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76fd255b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edd255e, SUC_lexicon_tags_17 },
    { 0x3cbd255e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367d2568, SUC_lexicon_tags_23 },
    { 0x271d2568, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb7dd256b, SUC_lexicon_tags_39 },
    { 0x63bd256b, SUC_lexicon_tags_12 },
    { 0x6cfd256d, SUC_lexicon_tags_2 },
    { 0x43bd256e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515d2572, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbd2576, SUC_lexicon_tags_8 },
    { 0x79bd2577, SUC_lexicon_tags_14 },
    { 0xda5d2577, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235d257c, SUC_lexicon_tags_0 },
    { 0x681d257c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827d2581, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bd2585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf4dd2587, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dd258c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdd258f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309d2599, SUC_lexicon_tags_1 },
    { 0x39dd259a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dd259e, SUC_lexicon_tags_19 },
    { 0xf51d259e, SUC_lexicon_tags_55 },
    { 0x4d1d25a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5d25a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283d25aa, SUC_lexicon_tags_25 },
    { 0x735d25aa, SUC_lexicon_tags_1 },
    { 0xc7d25ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9d25af, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51d25b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29d25b8, SUC_lexicon_tags_2 },
    { 0x3cbd25b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73bd25bb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9a7d25bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd25c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729d25c6, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dd25ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dd25d5, SUC_lexicon_tags_11 },
    { 0x899d25d6, SUC_lexicon_tags_7 },
    { 0x859d25d7, SUC_lexicon_tags_17 },
    { 0x61dd25d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x16bd25da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09d25e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133d25e8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5d25ec, SUC_lexicon_tags_26 },
    { 0xc5fd25ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5d25f0, SUC_lexicon_tags_14 },
    { 0x17bd25f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dd25f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67dd25f7, SUC_lexicon_tags_12 },
    { 0x1ddd25f8, SUC_lexicon_tags_37 },
    { 0x627d25f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4cdd25fb, SUC_lexicon_tags_37 },
    { 0x391d25fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39d2607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fd260a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dd260e, SUC_lexicon_tags_15 },
    { 0xe3fd260f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd67d2611, SUC_lexicon_tags_19 },
    { 0xab7d2611, SUC_lexicon_tags_25 },
    { 0x149d2613, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783d2618, SUC_lexicon_tags_34 },
    { 0xe2bd2619, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x181d261b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xf61d261d, SUC_lexicon_tags_1 },
    { 0x885d261d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x771d2621, SUC_lexicon_tags_60 },
    { 0xef7d2622, SUC_lexicon_tags_37 },
    { 0x57dd2622, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fd2626, SUC_lexicon_tags_17 },
    { 0xf01d2627, SUC_lexicon_tags_2 },
    { 0xdb5d2628, SUC_lexicon_tags_15 },
    { 0x3c9d2629, SUC_lexicon_tags_45 },
    { 0xdbbd262a, SUC_lexicon_tags_27 },
    { 0x37fd262b, SUC_lexicon_tags_25 },
    { 0x883d2629, SUC_lexicon_tags_47 },
    { 0xf89d262d, SUC_lexicon_tags_37 },
    { 0xeefd262c, SUC_lexicon_tags_38 },
    { 0x3c1d262a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7d263c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9add263e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9d2646, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca3d2648, SUC_lexicon_tags_15 },
    { 0xe3dd2649, SUC_lexicon_tags_37 },
    { 0xaf1d264a, SUC_lexicon_tags_11 },
    { 0xa85d264b, SUC_lexicon_tags_43 },
    { 0x9b5d264c, SUC_lexicon_tags_6 },
    { 0xf07d264b, SUC_lexicon_tags_49 },
    { 0x427d264d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x781d2650, SUC_lexicon_tags_47 },
    { 0x82dd2651, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbd2655, SUC_lexicon_tags_7 },
    { 0xbf7d2656, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x7c1d2658, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bd265e, SUC_lexicon_tags_17 },
    { 0xf5fd265f, SUC_lexicon_tags_60 },
    { 0xb17d265f, SUC_lexicon_tags_1 },
    { 0x1dd2661, SUC_lexicon_tags_1 },
    { 0x77d265f, SUC_lexicon_tags_32 },
    { 0xff7d265f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xffdd2666, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31d266e, SUC_lexicon_tags_31 },
    { 0x279d266f, SUC_lexicon_tags_60 },
    { 0x389d2670, SUC_lexicon_tags_2 },
    { 0xc85d2670, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5d2674, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76fd2676, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1d2680, SUC_lexicon_tags_27 },
    { 0x80fd2681, SUC_lexicon_tags_3 },
    { 0x16fd2680, SUC_lexicon_tags_17 },
    { 0xc1dd2683, SUC_lexicon_tags_23 },
    { 0xf3dd2681, SUC_lexicon_tags_79 },
    { 0xe63d2682, SUC_lexicon_tags_141 },
    { 0xe2dd2682, SUC_lexicon_tags_11 },
    { 0x2abd2686, SUC_lexicon_tags_37 },
    { 0x5ddd2688, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2e5d268a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21d2691, SUC_lexicon_tags_15 },
    { 0xd89d2691, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9d2696, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857d26a1, SUC_lexicon_tags_68 },
    { 0xc33d26a2, SUC_lexicon_tags_9 },
    { 0xe21d26a2, SUC_lexicon_tags_49 },
    { 0x115d26a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd26ab, SUC_lexicon_tags_4 },
    { 0x64dd26ac, SUC_lexicon_tags_46 },
    { 0x7ffd26ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311d26b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf4fd26b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e5d26ba, SUC_lexicon_tags_65 },
    { 0x4d3d26bb, SUC_lexicon_tags_37 },
    { 0x6fd26ba, SUC_lexicon_tags_48 },
    { 0x5cfd26bd, SUC_lexicon_tags_7 },
    { 0x9d7d26bb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xd15d26c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdd26c8, SUC_lexicon_tags_37 },
    { 0xcdd26c9, SUC_lexicon_tags_37 },
    { 0xd71d26ca, SUC_lexicon_tags_4 },
    { 0xa43d26cb, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47d26d0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x253d26d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbd26d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645d26db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9d26e0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaadd26e2, SUC_lexicon_tags_7 },
    { 0xc8dd26e2, SUC_lexicon_tags_37 },
    { 0xb2bd26e4, SUC_lexicon_tags_29 },
    { 0x1ebd26e5, SUC_lexicon_tags_17 },
    { 0xe49d26e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19d26ec, SUC_lexicon_tags_4 },
    { 0xf1fd26ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d5d26ef, SUC_lexicon_tags_17 },
    { 0x9c5d26f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb65d26f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbd26fa, SUC_lexicon_tags_19 },
    { 0xbe1d26fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x217d26fd, SUC_lexicon_tags_33 },
    { 0xe3dd26fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd53d2700, SUC_lexicon_tags_82 },
    { 0xb5d2701, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41d2708, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bd2710, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x53dd2712, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d1d2714, SUC_lexicon_tags_13 },
    { 0x837d2714, SUC_lexicon_tags_17 },
    { 0xbe1d2715, SUC_lexicon_tags_43 },
    { 0xaf3d2717, SUC_lexicon_tags_0 },
    { 0x511d2718, SUC_lexicon_tags_19 },
    { 0x573d2718, SUC_lexicon_tags_7 },
    { 0x61fd2718, SUC_lexicon_tags_11 },
    { 0x935d271a, SUC_lexicon_tags_10 },
    { 0xf11d271a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x237d271e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189d272a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fd272d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d3d272f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79d2734, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725d2739, SUC_lexicon_tags_17 },
    { 0x781d273a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99d273d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6dd2740, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa55d2742, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x55bd2744, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245d274d, SUC_lexicon_tags_3 },
    { 0x8e9d274e, SUC_lexicon_tags_49 },
    { 0x15d274f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbd2757, SUC_lexicon_tags_2 },
    { 0x7bbd2757, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x653d275a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afd275d, SUC_lexicon_tags_11 },
    { 0x9a3d275e, SUC_lexicon_tags_25 },
    { 0xc8dd275f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b1d2761, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5d2764, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69d2766, SUC_lexicon_tags_54 },
    { 0x513d2766, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d2773, SUC_lexicon_tags_49 },
    { 0xc9bd2773, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fd2777, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fd277a, SUC_lexicon_tags_37 },
    { 0x34dd277a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfd2780, SUC_lexicon_tags_721 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3d2784, SUC_lexicon_tags_27 },
    { 0xad1d2785, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539d2789, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17d2790, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd2794, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1d2797, SUC_lexicon_tags_28 },
    { 0xacdd2798, SUC_lexicon_tags_2 },
    { 0xbbdd2798, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c3d279b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991d27a0, SUC_lexicon_tags_11 },
    { 0xf33d27a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d27aa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9c3d27ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7d27af, SUC_lexicon_tags_4 },
    { 0xf1d27af, SUC_lexicon_tags_17 },
    { 0xfb1d27b1, SUC_lexicon_tags_63 },
    { 0xa3fd27b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa05d27b4, SUC_lexicon_tags_25 },
    { 0xecfd27b4, SUC_lexicon_tags_65 },
    { 0xefd27b4, SUC_lexicon_tags_63 },
    { 0xc29d27b7, SUC_lexicon_tags_11 },
    { 0x193d27b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x511d27ba, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bd27bd, SUC_lexicon_tags_23 },
    { 0xafbd27be, SUC_lexicon_tags_25 },
    { 0xf5d27bf, SUC_lexicon_tags_15 },
    { 0xf5bd27bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x571d27c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701d27c9, SUC_lexicon_tags_1 },
    { 0x4ddd27ca, SUC_lexicon_tags_60 },
    { 0xe01d27ca, SUC_lexicon_tags_49 },
    { 0x847d27cc, SUC_lexicon_tags_67 },
    { 0x9bbd27cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3d27d1, SUC_lexicon_tags_10 },
    { 0xb01d27d1, SUC_lexicon_tags_40 },
    { 0x9abd27d3, SUC_lexicon_tags_11 },
    { 0x3dd27d3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343d27d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763d27df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9d27e8, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0xd29d27ea, SUC_lexicon_tags_11 },
    { 0xc39d27ea, SUC_lexicon_tags_28 },
    { 0x121d27ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49d27f1, SUC_lexicon_tags_2 },
    { 0x5d27f1, SUC_lexicon_tags_17 },
    { 0x5f3d27f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afd27f6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99d27fb, SUC_lexicon_tags_1 },
    { 0x63bd27fb, SUC_lexicon_tags_47 },
    { 0xbcbd27fc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x689d2800, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73d2804, SUC_lexicon_tags_33 },
    { 0xc27d2805, SUC_lexicon_tags_47 },
    { 0xcfd2804, SUC_lexicon_tags_19 },
    { 0x69bd2807, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fd280b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7d280e, SUC_lexicon_tags_87 },
    { 0xeddd280e, SUC_lexicon_tags_25 },
    { 0x417d2810, SUC_lexicon_tags_2 },
    { 0x60bd2811, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc3d2813, SUC_lexicon_tags_8 },
    { 0x9cbd2814, SUC_lexicon_tags_37 },
    { 0xe0dd2813, SUC_lexicon_tags_43 },
    { 0x5e7d2815, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x515d2819, SUC_lexicon_tags_37 },
    { 0xb85d281a, SUC_lexicon_tags_1 },
    { 0xd3dd281b, SUC_lexicon_tags_0 },
    { 0xde3d281a, SUC_lexicon_tags_37 },
    { 0x6e5d2819, SUC_lexicon_tags_14 },
    { 0xff5d281e, SUC_lexicon_tags_68 },
    { 0xaf3d281f, SUC_lexicon_tags_23 },
    { 0xbf7d281e, SUC_lexicon_tags_14 },
    { 0x5add281b, SUC_lexicon_tags_38 },
    { 0xf7dd2822, SUC_lexicon_tags_43 },
    { 0x195d2822, SUC_lexicon_tags_0 },
    { 0x8b9d2824, SUC_lexicon_tags_8 },
    { 0xff5d2822, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bd2829, SUC_lexicon_tags_62 },
    { 0x2dbd282a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dd2830, SUC_lexicon_tags_43 },
    { 0xfadd2831, SUC_lexicon_tags_11 },
    { 0x797d2832, SUC_lexicon_tags_16 },
    { 0x57d2831, SUC_lexicon_tags_43 },
    { 0xe43d2830, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72bd2839, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dd283d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29d2840, SUC_lexicon_tags_25 },
    { 0xa09d2841, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbd2845, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f3d2847, SUC_lexicon_tags_1 },
    { 0xe73d2848, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dd284d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d2852, SUC_lexicon_tags_2 },
    { 0xb3fd2853, SUC_lexicon_tags_0 },
    { 0xc55d2854, SUC_lexicon_tags_0 },
    { 0x1abd2855, SUC_lexicon_tags_38 },
    { 0x1d1d2855, SUC_lexicon_tags_45 },
    { 0x68fd2855, SUC_lexicon_tags_25 },
    { 0x379d2856, SUC_lexicon_tags_45 },
    { 0xba3d2855, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51d285e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x587d2861, SUC_lexicon_tags_9 },
    { 0x3e9d2862, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163d2868, SUC_lexicon_tags_11 },
    { 0x6d9d2868, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bd286c, SUC_lexicon_tags_7 },
    { 0xdd5d286d, SUC_lexicon_tags_1 },
    { 0xc09d286d, SUC_lexicon_tags_11 },
    { 0x6dd286f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d2877, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577d287b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x455d287e, SUC_lexicon_tags_37 },
    { 0xa97d287f, SUC_lexicon_tags_15 },
    { 0x8a1d2880, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fd2884, SUC_lexicon_tags_1 },
    { 0xc89d2884, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7d2889, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x95d288b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1d288e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4cdd2890, SUC_lexicon_tags_12 },
    { 0x3b5d2891, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3d2897, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1fd2899, SUC_lexicon_tags_86 },
    { 0x571d2899, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dd28a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3d28a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bd28af, SUC_lexicon_tags_45 },
    { 0x995d28b0, SUC_lexicon_tags_12 },
    { 0xdb5d28b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x655d28b3, SUC_lexicon_tags_35 },
    { 0x773d28b4, SUC_lexicon_tags_11 },
    { 0xac1d28b4, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217d28be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909d28c2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941d28c8, SUC_lexicon_tags_9 },
    { 0xb87d28c9, SUC_lexicon_tags_21 },
    { 0x931d28c9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4e5d28cc, SUC_lexicon_tags_47 },
    { 0xec9d28cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fd28d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3d28da, SUC_lexicon_tags_1032 },
    { 0, NULL },
    { 0x7f1d28dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x937d28de, SUC_lexicon_tags_0 },
    { 0xcf7d28df, SUC_lexicon_tags_30 },
    { 0xeb1d28e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9d28e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe13d28e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x663d28e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09d28eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51bd28ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3bfd28ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3d28f5, SUC_lexicon_tags_60 },
    { 0xeffd28f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d28fa, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5d28fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fd2901, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3d2908, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71d290f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82bd2911, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251d2915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dd2919, SUC_lexicon_tags_21 },
    { 0xb1bd291a, SUC_lexicon_tags_47 },
    { 0x66bd2919, SUC_lexicon_tags_25 },
    { 0x7b5d291b, SUC_lexicon_tags_37 },
    { 0xa4dd291d, SUC_lexicon_tags_35 },
    { 0x859d291a, SUC_lexicon_tags_37 },
    { 0x55bd291f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5d2923, SUC_lexicon_tags_59 },
    { 0x405d2923, SUC_lexicon_tags_60 },
    { 0x431d2925, SUC_lexicon_tags_2 },
    { 0xa89d2924, SUC_lexicon_tags_65 },
    { 0x8bdd2927, SUC_lexicon_tags_1 },
    { 0xc5dd2923, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbafd292a, SUC_lexicon_tags_3 },
    { 0x573d292a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3d292f, SUC_lexicon_tags_23 },
    { 0x84bd2930, SUC_lexicon_tags_19 },
    { 0x179d2931, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfd2935, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fd293c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159d2942, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x419d2945, SUC_lexicon_tags_12 },
    { 0x1e3d2945, SUC_lexicon_tags_30 },
    { 0x6dd2947, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4b7d2949, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x327d294c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdd294f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4edd2951, SUC_lexicon_tags_27 },
    { 0xd1d2952, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x633d2955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a5d2957, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199d295c, SUC_lexicon_tags_32 },
    { 0xddbd295d, SUC_lexicon_tags_27 },
    { 0xe29d295e, SUC_lexicon_tags_11 },
    { 0x793d295f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x96bd2961, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0dd2963, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41d296a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9d296d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x819d2970, SUC_lexicon_tags_8 },
    { 0xf65d2971, SUC_lexicon_tags_6 },
    { 0xd5bd2971, SUC_lexicon_tags_3 },
    { 0x27bd2973, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd19d2975, SUC_lexicon_tags_38 },
    { 0x46dd2976, SUC_lexicon_tags_56 },
    { 0x4fbd2977, SUC_lexicon_tags_57 },
    { 0xe4fd2978, SUC_lexicon_tags_2 },
    { 0x823d2979, SUC_lexicon_tags_27 },
    { 0x699d297a, SUC_lexicon_tags_5 },
    { 0x43dd2977, SUC_lexicon_tags_29 },
    { 0x8f7d2976, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x431d297f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbd2983, SUC_lexicon_tags_9 },
    { 0x461d2983, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5d298a, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x713d298c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x787d298e, SUC_lexicon_tags_23 },
    { 0xe55d298f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x263d2992, SUC_lexicon_tags_37 },
    { 0xb29d2993, SUC_lexicon_tags_19 },
    { 0xe0dd2992, SUC_lexicon_tags_2 },
    { 0x31fd2995, SUC_lexicon_tags_1 },
    { 0x3a1d2996, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3d2999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa45d299b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d29a0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbedd29a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3c3d29a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fd29aa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95d29af, SUC_lexicon_tags_7 },
    { 0xdd1d29af, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5d29b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61d29b6, SUC_lexicon_tags_0 },
    { 0x709d29b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7d29be, SUC_lexicon_tags_155 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d29cb, SUC_lexicon_tags_19 },
    { 0x60dd29cc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x79fd29ce, SUC_lexicon_tags_1 },
    { 0x779d29cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39d29d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5d29dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x24dd29df, SUC_lexicon_tags_39 },
    { 0x43bd29df, SUC_lexicon_tags_1 },
    { 0x2e9d29e1, SUC_lexicon_tags_8 },
    { 0x9b9d29e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fd29e6, SUC_lexicon_tags_17 },
    { 0xbd9d29e7, SUC_lexicon_tags_11 },
    { 0x7d7d29e8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffd29eb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dd29f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb53d29f3, SUC_lexicon_tags_27 },
    { 0x4b9d29f3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591d29f8, SUC_lexicon_tags_65 },
    { 0xfe7d29f9, SUC_lexicon_tags_28 },
    { 0x80fd29fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x945d29fd, SUC_lexicon_tags_7 },
    { 0x9dbd29fd, SUC_lexicon_tags_65 },
    { 0x463d29fd, SUC_lexicon_tags_38 },
    { 0x9abd2a00, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x83fd2a03, SUC_lexicon_tags_38 },
    { 0x891d2a04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2fd2a06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187d2a0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda7d2a0d, SUC_lexicon_tags_68 },
    { 0xc61d2a0d, SUC_lexicon_tags_40 },
    { 0x615d2a0f, SUC_lexicon_tags_2 },
    { 0xa61d2a10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803d2a14, SUC_lexicon_tags_29 },
    { 0xfb7d2a14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1edd2a17, SUC_lexicon_tags_29 },
    { 0xc63d2a17, SUC_lexicon_tags_9 },
    { 0xb4fd2a19, SUC_lexicon_tags_2 },
    { 0x8c7d2a1a, SUC_lexicon_tags_82 },
    { 0xa8bd2a1b, SUC_lexicon_tags_1 },
    { 0xa2bd2a1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bd2a1f, SUC_lexicon_tags_29 },
    { 0xfabd2a1f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde7d2a22, SUC_lexicon_tags_47 },
    { 0x3b5d2a23, SUC_lexicon_tags_33 },
    { 0x713d2a24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dd2a2e, SUC_lexicon_tags_32 },
    { 0x43d2a2f, SUC_lexicon_tags_2 },
    { 0x273d2a2f, SUC_lexicon_tags_17 },
    { 0xe4bd2a2e, SUC_lexicon_tags_45 },
    { 0x699d2a32, SUC_lexicon_tags_11 },
    { 0xb29d2a32, SUC_lexicon_tags_23 },
    { 0x61d2a31, SUC_lexicon_tags_47 },
    { 0xda7d2a34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1d2a39, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x52fd2a3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bd2a41, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf11d2a43, SUC_lexicon_tags_23 },
    { 0x385d2a44, SUC_lexicon_tags_25 },
    { 0x56fd2a45, SUC_lexicon_tags_65 },
    { 0xa0fd2a45, SUC_lexicon_tags_25 },
    { 0x60fd2a47, SUC_lexicon_tags_7 },
    { 0x957d2a47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x491d2a4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d2a4e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd19d2a50, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fd2a54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfd2a57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957d2a5b, SUC_lexicon_tags_7 },
    { 0x217d2a5b, SUC_lexicon_tags_82 },
    { 0xc3dd2a5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781d2a65, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb1d2a67, SUC_lexicon_tags_23 },
    { 0x77bd2a68, SUC_lexicon_tags_33 },
    { 0x39bd2a67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcafd2a6b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfa5d2a6d, SUC_lexicon_tags_9 },
    { 0x243d2a6d, SUC_lexicon_tags_28 },
    { 0x595d2a6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bd2a74, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x181d2a76, SUC_lexicon_tags_38 },
    { 0x9b5d2a77, SUC_lexicon_tags_65 },
    { 0xb87d2a78, SUC_lexicon_tags_12 },
    { 0xefdd2a79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959d2a80, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfb7d2a82, SUC_lexicon_tags_14 },
    { 0x131d2a83, SUC_lexicon_tags_37 },
    { 0x7ffd2a84, SUC_lexicon_tags_32 },
    { 0x981d2a83, SUC_lexicon_tags_4 },
    { 0xfe9d2a86, SUC_lexicon_tags_56 },
    { 0x87fd2a87, SUC_lexicon_tags_65 },
    { 0xdcbd2a87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1d2a8b, SUC_lexicon_tags_23 },
    { 0xe43d2a8c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe01d2a8e, SUC_lexicon_tags_17 },
    { 0x9f3d2a8e, SUC_lexicon_tags_47 },
    { 0x215d2a90, SUC_lexicon_tags_25 },
    { 0x5d3d2a91, SUC_lexicon_tags_25 },
    { 0x869d2a92, SUC_lexicon_tags_1 },
    { 0xd1d2a93, SUC_lexicon_tags_2 },
    { 0x69dd2a93, SUC_lexicon_tags_29 },
    { 0x1afd2a93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261d2a9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559d2aa1, SUC_lexicon_tags_45 },
    { 0xf7dd2aa2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x609d2aa4, SUC_lexicon_tags_46 },
    { 0x967d2aa4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bd2aaa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd2aae, SUC_lexicon_tags_2 },
    { 0x983d2aae, SUC_lexicon_tags_15 },
    { 0x779d2aae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809d2ab4, SUC_lexicon_tags_37 },
    { 0xcadd2ab5, SUC_lexicon_tags_31 },
    { 0xf3d2ab5, SUC_lexicon_tags_2 },
    { 0xb11d2ab4, SUC_lexicon_tags_11 },
    { 0xa4bd2ab8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x869d2aba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bd2abd, SUC_lexicon_tags_31 },
    { 0xb5dd2abd, SUC_lexicon_tags_1 },
    { 0x79d2abf, SUC_lexicon_tags_11 },
    { 0xc45d2abe, SUC_lexicon_tags_53 },
    { 0xde7d2abe, SUC_lexicon_tags_2 },
    { 0xf51d2ac2, SUC_lexicon_tags_7 },
    { 0x137d2ac2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d2ac6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebd2ace, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x147d2ad0, SUC_lexicon_tags_12 },
    { 0xdcbd2ad1, SUC_lexicon_tags_45 },
    { 0x47bd2ad2, SUC_lexicon_tags_2 },
    { 0x87fd2ad1, SUC_lexicon_tags_28 },
    { 0x7e3d2ad4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1d2ad8, SUC_lexicon_tags_14 },
    { 0x9d2ad9, SUC_lexicon_tags_12 },
    { 0xf19d2ada, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dd2ade, SUC_lexicon_tags_63 },
    { 0x31fd2ade, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fd2ae2, SUC_lexicon_tags_29 },
    { 0x74dd2ae3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3d2ae9, SUC_lexicon_tags_1 },
    { 0xcadd2aea, SUC_lexicon_tags_37 },
    { 0x881d2aeb, SUC_lexicon_tags_57 },
    { 0x839d2aec, SUC_lexicon_tags_37 },
    { 0xdbbd2aed, SUC_lexicon_tags_37 },
    { 0x7e3d2aee, SUC_lexicon_tags_25 },
    { 0xc61d2aeb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6afd2af1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x119d2af4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e1d2af6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fd2af9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5d2afc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xaa5d2afe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x225d2b01, SUC_lexicon_tags_83 },
    { 0x203d2b01, SUC_lexicon_tags_1 },
    { 0xe81d2b01, SUC_lexicon_tags_37 },
    { 0xbe7d2b03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dd2b08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301d2b0d, SUC_lexicon_tags_19 },
    { 0x6abd2b0e, SUC_lexicon_tags_17 },
    { 0x893d2b0f, SUC_lexicon_tags_17 },
    { 0x715d2b0d, SUC_lexicon_tags_16 },
    { 0x255d2b0e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bd2b14, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x73fd2b16, SUC_lexicon_tags_8 },
    { 0xb63d2b16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb01d2b19, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5d2b1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bd2b21, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59dd2b23, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x253d2b25, SUC_lexicon_tags_1 },
    { 0x16fd2b26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecfd2b28, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71d2b2c, SUC_lexicon_tags_17 },
    { 0x47dd2b2d, SUC_lexicon_tags_7 },
    { 0x10dd2b2e, SUC_lexicon_tags_17 },
    { 0x8c3d2b2f, SUC_lexicon_tags_82 },
    { 0xc1d2b30, SUC_lexicon_tags_1 },
    { 0x6b7d2b31, SUC_lexicon_tags_17 },
    { 0xd45d2b31, SUC_lexicon_tags_37 },
    { 0xaf7d2b31, SUC_lexicon_tags_43 },
    { 0x28fd2b32, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97d2b37, SUC_lexicon_tags_15 },
    { 0x341d2b38, SUC_lexicon_tags_25 },
    { 0xe39d2b37, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1d2b3c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29d2b42, SUC_lexicon_tags_39 },
    { 0xa5dd2b43, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29d2b4d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9d2b52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fd2b5d, SUC_lexicon_tags_45 },
    { 0xe25d2b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dd2b62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4ebd2b64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9d2b67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45d2b6a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf7bd2b6c, SUC_lexicon_tags_17 },
    { 0xed3d2b6c, SUC_lexicon_tags_1 },
    { 0x8bdd2b6c, SUC_lexicon_tags_15 },
    { 0xef3d2b6e, SUC_lexicon_tags_37 },
    { 0x37dd2b6f, SUC_lexicon_tags_12 },
    { 0xc7bd2b6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb3d2b73, SUC_lexicon_tags_82 },
    { 0xa8bd2b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabd2b79, SUC_lexicon_tags_12 },
    { 0xa41d2b7a, SUC_lexicon_tags_1 },
    { 0x8fbd2b7b, SUC_lexicon_tags_47 },
    { 0x3add2b7c, SUC_lexicon_tags_8 },
    { 0xacdd2b7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47d2b81, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fd2b89, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad1d2b8b, SUC_lexicon_tags_1 },
    { 0xdf1d2b8c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadd2b91, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdd2b95, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5d2b98, SUC_lexicon_tags_33 },
    { 0x13dd2b99, SUC_lexicon_tags_37 },
    { 0x2e7d2b99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb8dd2b9c, SUC_lexicon_tags_8 },
    { 0xc95d2b9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbd2ba1, SUC_lexicon_tags_37 },
    { 0xfd2ba2, SUC_lexicon_tags_43 },
    { 0x23bd2ba3, SUC_lexicon_tags_37 },
    { 0x2dd2ba4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x473d2ba7, SUC_lexicon_tags_49 },
    { 0x921d2ba8, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x8f9d2baa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbfdd2bac, SUC_lexicon_tags_25 },
    { 0x6a5d2bad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c1d2baf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7d2bb5, SUC_lexicon_tags_6 },
    { 0xe85d2bb6, SUC_lexicon_tags_1 },
    { 0x743d2bb6, SUC_lexicon_tags_37 },
    { 0xa25d2bb6, SUC_lexicon_tags_4 },
    { 0x53d2bb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29d2bbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d7d2bbe, SUC_lexicon_tags_11 },
    { 0x643d2bbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x637d2bc1, SUC_lexicon_tags_17 },
    { 0xc35d2bc2, SUC_lexicon_tags_11 },
    { 0x1a3d2bc3, SUC_lexicon_tags_49 },
    { 0x22bd2bc4, SUC_lexicon_tags_12 },
    { 0x343d2bc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6dd2bc7, SUC_lexicon_tags_14 },
    { 0xbd3d2bc7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83d2bcb, SUC_lexicon_tags_12 },
    { 0x1d3d2bcc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebd2bd2, SUC_lexicon_tags_25 },
    { 0xd01d2bd3, SUC_lexicon_tags_63 },
    { 0x8c1d2bd2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267d2bd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xed1d2bda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1d2bde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1d2be4, SUC_lexicon_tags_70 },
    { 0x50bd2be5, SUC_lexicon_tags_11 },
    { 0x975d2be4, SUC_lexicon_tags_7 },
    { 0x52bd2be7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d2bec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc3d2bee, SUC_lexicon_tags_33 },
    { 0xc83d2bef, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x763d2bf2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefd2bf6, SUC_lexicon_tags_3 },
    { 0xf2bd2bf7, SUC_lexicon_tags_37 },
    { 0xd25d2bf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363d2bfc, SUC_lexicon_tags_49 },
    { 0xbadd2bfd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x89dd2c00, SUC_lexicon_tags_7 },
    { 0x499d2c00, SUC_lexicon_tags_79 },
    { 0x2bfd2c00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83d2c06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7d2c0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505d2c1f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbd2c28, SUC_lexicon_tags_1 },
    { 0x5a7d2c28, SUC_lexicon_tags_1 },
    { 0xe7dd2c29, SUC_lexicon_tags_25 },
    { 0x65dd2c2a, SUC_lexicon_tags_11 },
    { 0x49d2c2a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fd2c32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fd2c3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x827d2c3d, SUC_lexicon_tags_2 },
    { 0x1b5d2c3d, SUC_lexicon_tags_1 },
    { 0xffd2c3f, SUC_lexicon_tags_17 },
    { 0x7ebd2c40, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x433d2c43, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe85d2c45, SUC_lexicon_tags_31 },
    { 0x4cdd2c46, SUC_lexicon_tags_17 },
    { 0x109d2c45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fd2c4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbd2c50, SUC_lexicon_tags_0 },
    { 0x35fd2c51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fd2c56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23bd2c58, SUC_lexicon_tags_9 },
    { 0xce7d2c59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfd2c5e, SUC_lexicon_tags_27 },
    { 0x237d2c5f, SUC_lexicon_tags_29 },
    { 0xe47d2c60, SUC_lexicon_tags_61 },
    { 0xa33d2c61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45d2c67, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5d2c6a, SUC_lexicon_tags_15 },
    { 0x9dbd2c6a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc79d2c6d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365d2c72, SUC_lexicon_tags_38 },
    { 0x155d2c73, SUC_lexicon_tags_72 },
    { 0x94fd2c72, SUC_lexicon_tags_25 },
    { 0xbbdd2c75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73d2c7a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fd2c80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x91dd2c82, SUC_lexicon_tags_27 },
    { 0x39dd2c83, SUC_lexicon_tags_13 },
    { 0x365d2c83, SUC_lexicon_tags_68 },
    { 0xe47d2c85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bd2c89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45d2c8e, SUC_lexicon_tags_26 },
    { 0x443d2c8f, SUC_lexicon_tags_11 },
    { 0x177d2c90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35d2c98, SUC_lexicon_tags_1 },
    { 0xb77d2c99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1d2c9c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x879d2c9f, SUC_lexicon_tags_70 },
    { 0x3fd2c9f, SUC_lexicon_tags_63 },
    { 0x209d2ca1, SUC_lexicon_tags_47 },
    { 0x815d2c9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fd2ca7, SUC_lexicon_tags_7 },
    { 0x13bd2ca8, SUC_lexicon_tags_12 },
    { 0xc43d2ca9, SUC_lexicon_tags_9 },
    { 0x413d2caa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411d2caf, SUC_lexicon_tags_17 },
    { 0xe75d2cb0, SUC_lexicon_tags_2 },
    { 0x47bd2cb1, SUC_lexicon_tags_12 },
    { 0xdf3d2cb1, SUC_lexicon_tags_37 },
    { 0x797d2cb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcb9d2cb5, SUC_lexicon_tags_47 },
    { 0x85bd2cb6, SUC_lexicon_tags_25 },
    { 0x199d2cb7, SUC_lexicon_tags_9 },
    { 0x61bd2cb7, SUC_lexicon_tags_4 },
    { 0x93fd2cb9, SUC_lexicon_tags_60 },
    { 0x81bd2cb6, SUC_lexicon_tags_0 },
    { 0x1d2cb6, SUC_lexicon_tags_7 },
    { 0x83dd2cbc, SUC_lexicon_tags_43 },
    { 0xe3dd2cbd, SUC_lexicon_tags_17 },
    { 0xf5dd2cbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89d2cc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7d2cc7, SUC_lexicon_tags_7 },
    { 0x203d2cc8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bd2ccd, SUC_lexicon_tags_25 },
    { 0x42bd2cce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d2cd6, SUC_lexicon_tags_37 },
    { 0x81fd2cd7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99d2cdb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x729d2cde, SUC_lexicon_tags_19 },
    { 0x171d2cde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x731d2ce1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35d2ce9, SUC_lexicon_tags_17 },
    { 0xcb1d2cea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe71d2cec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55d2cf2, SUC_lexicon_tags_46 },
    { 0xb91d2cf3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfd2cf6, SUC_lexicon_tags_56 },
    { 0xa43d2cf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7d2cfb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747d2d05, SUC_lexicon_tags_2 },
    { 0xa57d2d05, SUC_lexicon_tags_66 },
    { 0x53bd2d06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae7d2d09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x571d2d0b, SUC_lexicon_tags_23 },
    { 0xb6bd2d0b, SUC_lexicon_tags_45 },
    { 0x623d2d0d, SUC_lexicon_tags_46 },
    { 0x171d2d0d, SUC_lexicon_tags_2 },
    { 0x999d2d0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719d2d13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e7d2d15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8f3d2d17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4a3d2d19, SUC_lexicon_tags_17 },
    { 0xf71d2d1a, SUC_lexicon_tags_37 },
    { 0xcf1d2d19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1afd2d1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53d2d25, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabd2d2e, SUC_lexicon_tags_17 },
    { 0x6b5d2d2f, SUC_lexicon_tags_12 },
    { 0xea7d2d2f, SUC_lexicon_tags_42 },
    { 0x1fbd2d31, SUC_lexicon_tags_37 },
    { 0x911d2d31, SUC_lexicon_tags_6 },
    { 0xdd9d2d33, SUC_lexicon_tags_7 },
    { 0x3fd2d33, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xed9d2d36, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43d2d3e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985d2d43, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143d2d4b, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x105d2d4d, SUC_lexicon_tags_84 },
    { 0x7d2d4e, SUC_lexicon_tags_6 },
    { 0x89bd2d4f, SUC_lexicon_tags_17 },
    { 0x167d2d50, SUC_lexicon_tags_60 },
    { 0x873d2d51, SUC_lexicon_tags_25 },
    { 0xea3d2d52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9d2d55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bd2d5c, SUC_lexicon_tags_15 },
    { 0x14fd2d5d, SUC_lexicon_tags_25 },
    { 0x6b3d2d5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5d2d63, SUC_lexicon_tags_39 },
    { 0xd8bd2d64, SUC_lexicon_tags_50 },
    { 0x2dd2d65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2fd2d67, SUC_lexicon_tags_55 },
    { 0xd41d2d68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd5fd2d6a, SUC_lexicon_tags_33 },
    { 0xde7d2d6b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6a9d2d6d, SUC_lexicon_tags_63 },
    { 0xac7d2d6e, SUC_lexicon_tags_80 },
    { 0xbfbd2d6e, SUC_lexicon_tags_25 },
    { 0xa03d2d6f, SUC_lexicon_tags_15 },
    { 0x1ddd2d70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3d2d74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715d2d79, SUC_lexicon_tags_47 },
    { 0xa5d2d79, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x40dd2d7c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb6fd2d7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edd2d83, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd2d87, SUC_lexicon_tags_25 },
    { 0x2e3d2d87, SUC_lexicon_tags_15 },
    { 0x9d1d2d89, SUC_lexicon_tags_33 },
    { 0x7fbd2d8a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x22fd2d8c, SUC_lexicon_tags_201 },
    { 0x961d2d8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bd2d91, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x89d2d93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x597d2d95, SUC_lexicon_tags_36 },
    { 0x96fd2d96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753d2d9b, SUC_lexicon_tags_17 },
    { 0xfd5d2d9c, SUC_lexicon_tags_12 },
    { 0xe9dd2d9c, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x755d2d9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc19d2da1, SUC_lexicon_tags_38 },
    { 0x2a5d2da1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe81d2da4, SUC_lexicon_tags_37 },
    { 0x57dd2da4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc03d2da7, SUC_lexicon_tags_37 },
    { 0x40dd2da7, SUC_lexicon_tags_23 },
    { 0x605d2da9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fd2dac, SUC_lexicon_tags_15 },
    { 0xf2dd2dac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae9d2daf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41d2db3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd2db9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa11d2dbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fd2dbf, SUC_lexicon_tags_2 },
    { 0xb47d2dc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdd2dc3, SUC_lexicon_tags_34 },
    { 0xeb5d2dc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef3d2dc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57d2dce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9d2dd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x587d2dd7, SUC_lexicon_tags_11 },
    { 0x587d2dd8, SUC_lexicon_tags_45 },
    { 0x325d2dd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97d2de2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329d2de6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9d2dea, SUC_lexicon_tags_25 },
    { 0x3d1d2deb, SUC_lexicon_tags_25 },
    { 0x69fd2dec, SUC_lexicon_tags_17 },
    { 0x863d2dec, SUC_lexicon_tags_56 },
    { 0xb31d2dee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bd2df4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22dd2df8, SUC_lexicon_tags_62 },
    { 0xd4fd2df9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7d2dfc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bd2dff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1d2e02, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x273d2e04, SUC_lexicon_tags_55 },
    { 0xa8dd2e05, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d2e08, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bd2e0b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3d2e11, SUC_lexicon_tags_83 },
    { 0x1a3d2e11, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5d2e15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddd2e18, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183d2e20, SUC_lexicon_tags_23 },
    { 0x39dd2e21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207d2e2b, SUC_lexicon_tags_17 },
    { 0xdd5d2e2c, SUC_lexicon_tags_3 },
    { 0x613d2e2d, SUC_lexicon_tags_25 },
    { 0x7e7d2e2d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d2e37, SUC_lexicon_tags_9 },
    { 0x11dd2e38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd2e3d, SUC_lexicon_tags_47 },
    { 0xcc9d2e3e, SUC_lexicon_tags_28 },
    { 0xcffd2e3f, SUC_lexicon_tags_46 },
    { 0x81dd2e40, SUC_lexicon_tags_7 },
    { 0x747d2e3f, SUC_lexicon_tags_9 },
    { 0x8c9d2e42, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x45fd2e45, SUC_lexicon_tags_14 },
    { 0xbd5d2e46, SUC_lexicon_tags_11 },
    { 0x695d2e46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x90bd2e49, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x747d2e4c, SUC_lexicon_tags_15 },
    { 0x661d2e4d, SUC_lexicon_tags_10 },
    { 0x17dd2e4e, SUC_lexicon_tags_48 },
    { 0x313d2e4e, SUC_lexicon_tags_33 },
    { 0xafdd2e50, SUC_lexicon_tags_30 },
    { 0x567d2e51, SUC_lexicon_tags_23 },
    { 0xe03d2e51, SUC_lexicon_tags_15 },
    { 0x783d2e4d, SUC_lexicon_tags_158 },
    { 0x573d2e54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb87d2e56, SUC_lexicon_tags_19 },
    { 0x707d2e57, SUC_lexicon_tags_0 },
    { 0xcafd2e57, SUC_lexicon_tags_35 },
    { 0x9f3d2e59, SUC_lexicon_tags_17 },
    { 0xa97d2e59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x777d2e5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725d2e62, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x443d2e64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x101d2e66, SUC_lexicon_tags_17 },
    { 0xbb3d2e66, SUC_lexicon_tags_45 },
    { 0x45bd2e68, SUC_lexicon_tags_30 },
    { 0x535d2e66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683d2e72, SUC_lexicon_tags_9 },
    { 0x1bfd2e72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11d2e79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dd2e7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x93dd2e7f, SUC_lexicon_tags_1 },
    { 0xa65d2e80, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x223d2e82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x997d2e85, SUC_lexicon_tags_6 },
    { 0x573d2e85, SUC_lexicon_tags_1 },
    { 0xd3d2e85, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83d2e8b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x805d2e8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dd2e92, SUC_lexicon_tags_7 },
    { 0xcd9d2e93, SUC_lexicon_tags_2 },
    { 0x6f9d2e92, SUC_lexicon_tags_47 },
    { 0x58fd2e93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbd2e98, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bd2e9b, SUC_lexicon_tags_27 },
    { 0x919d2e9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdadd2e9e, SUC_lexicon_tags_1 },
    { 0x4e1d2e9e, SUC_lexicon_tags_33 },
    { 0x8abd2e9f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x199d2ea2, SUC_lexicon_tags_11 },
    { 0x96bd2ea3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4dd2ea5, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbd2ea8, SUC_lexicon_tags_2 },
    { 0x7bdd2ea9, SUC_lexicon_tags_68 },
    { 0x353d2eaa, SUC_lexicon_tags_26 },
    { 0x12fd2ea9, SUC_lexicon_tags_1 },
    { 0xd51d2eab, SUC_lexicon_tags_38 },
    { 0xb05d2eab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e9d2eaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819d2eb3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1d2eb7, SUC_lexicon_tags_26 },
    { 0x1abd2eb8, SUC_lexicon_tags_4 },
    { 0x279d2eb8, SUC_lexicon_tags_15 },
    { 0x87fd2eb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x861d2ebc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5d2ebf, SUC_lexicon_tags_14 },
    { 0x243d2ec0, SUC_lexicon_tags_9 },
    { 0x49bd2ec0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99fd2ec3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51fd2ec6, SUC_lexicon_tags_27 },
    { 0x523d2ec6, SUC_lexicon_tags_364 },
    { 0x647d2ec8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x567d2eca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d2ed2, SUC_lexicon_tags_55 },
    { 0x6e9d2ed3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1d2ed8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bd2edc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x32dd2edf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbd2ee5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb55d2ee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadd2eeb, SUC_lexicon_tags_37 },
    { 0x529d2eec, SUC_lexicon_tags_27 },
    { 0xdabd2eed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edd2ef1, SUC_lexicon_tags_47 },
    { 0xf47d2ef2, SUC_lexicon_tags_15 },
    { 0x9a7d2ef3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9d2ef7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3d2efc, SUC_lexicon_tags_26 },
    { 0x9bbd2efd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5d2f00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169d2f09, SUC_lexicon_tags_28 },
    { 0x165d2f0a, SUC_lexicon_tags_37 },
    { 0x443d2f09, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7d2f0e, SUC_lexicon_tags_1 },
    { 0x4e1d2f0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7d2f12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdd2f15, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa8fd2f17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bd2f1d, SUC_lexicon_tags_56 },
    { 0x805d2f1e, SUC_lexicon_tags_21 },
    { 0x349d2f1f, SUC_lexicon_tags_46 },
    { 0x9cbd2f20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d2f25, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x595d2f28, SUC_lexicon_tags_37 },
    { 0x12dd2f28, SUC_lexicon_tags_1 },
    { 0x39bd2f29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbd2f31, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bd2f34, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fd2f37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dd2f3c, SUC_lexicon_tags_37 },
    { 0x18dd2f3d, SUC_lexicon_tags_30 },
    { 0xb55d2f3d, SUC_lexicon_tags_37 },
    { 0xf5d2f3f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7d2f44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fd2f48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89d2f4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687d2f4f, SUC_lexicon_tags_47 },
    { 0x739d2f50, SUC_lexicon_tags_57 },
    { 0xf1dd2f51, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9d2f55, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9d2f5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3d2f6a, SUC_lexicon_tags_27 },
    { 0x47fd2f6a, SUC_lexicon_tags_1 },
    { 0x965d2f6c, SUC_lexicon_tags_17 },
    { 0xab7d2f6d, SUC_lexicon_tags_31 },
    { 0x22dd2f6e, SUC_lexicon_tags_2 },
    { 0xfd2f6f, SUC_lexicon_tags_8 },
    { 0x955d2f6f, SUC_lexicon_tags_7 },
    { 0xe6bd2f71, SUC_lexicon_tags_33 },
    { 0x5e1d2f6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bd2f75, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15d2f83, SUC_lexicon_tags_21 },
    { 0x13d2f83, SUC_lexicon_tags_7 },
    { 0x23fd2f85, SUC_lexicon_tags_37 },
    { 0x2e3d2f86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55d2f89, SUC_lexicon_tags_16 },
    { 0x4e5d2f8a, SUC_lexicon_tags_100 },
    { 0x68bd2f8b, SUC_lexicon_tags_57 },
    { 0x69d2f8b, SUC_lexicon_tags_1 },
    { 0x6afd2f8d, SUC_lexicon_tags_106 },
    { 0x3dd2f89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe5dd2f90, SUC_lexicon_tags_0 },
    { 0xd1d2f90, SUC_lexicon_tags_1 },
    { 0x84bd2f92, SUC_lexicon_tags_2 },
    { 0x4e9d2f93, SUC_lexicon_tags_2 },
    { 0x483d2f92, SUC_lexicon_tags_11 },
    { 0x94fd2f95, SUC_lexicon_tags_2 },
    { 0xed5d2f95, SUC_lexicon_tags_19 },
    { 0x121d2f90, SUC_lexicon_tags_32 },
    { 0x623d2f96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbd2f9b, SUC_lexicon_tags_215 },
    { 0xd69d2f9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873d2fa7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709d2fab, SUC_lexicon_tags_43 },
    { 0xcb5d2fab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e1d2fae, SUC_lexicon_tags_7 },
    { 0x7a9d2faf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x337d2fb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135d2fb7, SUC_lexicon_tags_27 },
    { 0x789d2fb8, SUC_lexicon_tags_33 },
    { 0x183d2fb8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781d2fc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5d2fc3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xff5d2fc5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1d2fcc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3d2fd6, SUC_lexicon_tags_5 },
    { 0x8b3d2fd7, SUC_lexicon_tags_15 },
    { 0xd57d2fd6, SUC_lexicon_tags_27 },
    { 0x9c9d2fd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fd2fde, SUC_lexicon_tags_11 },
    { 0x9afd2fdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601d2fe4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb4dd2fe6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dd2ff0, SUC_lexicon_tags_2 },
    { 0x95fd2ff0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9d2ff6, SUC_lexicon_tags_38 },
    { 0xa43d2ff7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1d2ffa, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xcd5d2ffc, SUC_lexicon_tags_12 },
    { 0xdadd2ffd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13d3008, SUC_lexicon_tags_33 },
    { 0xf45d3008, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bd300c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x589d300e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edd3013, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9d3019, SUC_lexicon_tags_77 },
    { 0xa67d301a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313d301e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdbd3020, SUC_lexicon_tags_35 },
    { 0xdabd3020, SUC_lexicon_tags_23 },
    { 0x6c5d3020, SUC_lexicon_tags_9 },
    { 0xdbd3023, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fd302c, SUC_lexicon_tags_25 },
    { 0x17bd302d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21d3030, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6dd3032, SUC_lexicon_tags_10 },
    { 0x4cbd3032, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x429d3035, SUC_lexicon_tags_60 },
    { 0x3dfd3036, SUC_lexicon_tags_11 },
    { 0xc7bd3035, SUC_lexicon_tags_15 },
    { 0x4bd3036, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x11fd303a, SUC_lexicon_tags_54 },
    { 0x23fd303b, SUC_lexicon_tags_47 },
    { 0xd15d303a, SUC_lexicon_tags_37 },
    { 0xfdbd303a, SUC_lexicon_tags_17 },
    { 0xa3d303e, SUC_lexicon_tags_11 },
    { 0x775d303f, SUC_lexicon_tags_37 },
    { 0x929d303b, SUC_lexicon_tags_25 },
    { 0x55bd303a, SUC_lexicon_tags_68 },
    { 0x781d3041, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd29d3044, SUC_lexicon_tags_63 },
    { 0x8c9d3044, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5d3048, SUC_lexicon_tags_1 },
    { 0x7d1d3049, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xa83d304b, SUC_lexicon_tags_7 },
    { 0x195d304c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2edd304e, SUC_lexicon_tags_35 },
    { 0xf8bd304f, SUC_lexicon_tags_55 },
    { 0x5c9d3050, SUC_lexicon_tags_29 },
    { 0xd2bd3050, SUC_lexicon_tags_23 },
    { 0x59d304f, SUC_lexicon_tags_37 },
    { 0x8b1d3051, SUC_lexicon_tags_87 },
    { 0x76dd304f, SUC_lexicon_tags_43 },
    { 0x723d304e, SUC_lexicon_tags_10 },
    { 0x15dd3056, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65d3059, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52dd305c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97d3061, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fd3064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b1d3066, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529d306a, SUC_lexicon_tags_4 },
    { 0x20fd306a, SUC_lexicon_tags_11 },
    { 0xcedd306c, SUC_lexicon_tags_117 },
    { 0x5a3d306d, SUC_lexicon_tags_8 },
    { 0x98dd306d, SUC_lexicon_tags_151 },
    { 0x1ffd306f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda7d3071, SUC_lexicon_tags_30 },
    { 0xfdfd3072, SUC_lexicon_tags_15 },
    { 0xe13d3073, SUC_lexicon_tags_30 },
    { 0x701d3074, SUC_lexicon_tags_14 },
    { 0x931d3075, SUC_lexicon_tags_12 },
    { 0x727d3075, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb03d3078, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9d307e, SUC_lexicon_tags_151 },
    { 0x665d307f, SUC_lexicon_tags_1 },
    { 0xa1bd3080, SUC_lexicon_tags_37 },
    { 0x8b5d3081, SUC_lexicon_tags_17 },
    { 0x711d3082, SUC_lexicon_tags_6 },
    { 0xdf5d3082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5d3089, SUC_lexicon_tags_30 },
    { 0xd0bd308a, SUC_lexicon_tags_4 },
    { 0x4c1d308b, SUC_lexicon_tags_30 },
    { 0x2d1d308c, SUC_lexicon_tags_34 },
    { 0xa83d308d, SUC_lexicon_tags_14 },
    { 0xcc7d308b, SUC_lexicon_tags_39 },
    { 0x3fdd308f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x657d3092, SUC_lexicon_tags_11 },
    { 0xa1fd3092, SUC_lexicon_tags_11 },
    { 0x9bd3094, SUC_lexicon_tags_12 },
    { 0xa67d3095, SUC_lexicon_tags_17 },
    { 0x79bd3094, SUC_lexicon_tags_17 },
    { 0xcfd3097, SUC_lexicon_tags_2 },
    { 0x843d3098, SUC_lexicon_tags_2 },
    { 0xec1d3099, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8d5d309b, SUC_lexicon_tags_7 },
    { 0xa71d309b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bd309f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33d30a2, SUC_lexicon_tags_25 },
    { 0x14bd30a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea9d30a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebd30aa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1afd30ac, SUC_lexicon_tags_63 },
    { 0xd25d30ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9d30b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd30b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x445d30b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x107d30ba, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbd30be, SUC_lexicon_tags_66 },
    { 0x759d30bf, SUC_lexicon_tags_7 },
    { 0x121d30be, SUC_lexicon_tags_48 },
    { 0xdcdd30c1, SUC_lexicon_tags_27 },
    { 0xbf7d30c2, SUC_lexicon_tags_1 },
    { 0xef1d30be, SUC_lexicon_tags_26 },
    { 0xe4bd30c2, SUC_lexicon_tags_39 },
    { 0xdbd30c5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x347d30c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1d30ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3fd30cc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdd30d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9d30d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7d30de, SUC_lexicon_tags_19 },
    { 0x5c7d30df, SUC_lexicon_tags_103 },
    { 0xe39d30df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x411d30e3, SUC_lexicon_tags_38 },
    { 0x82dd30e4, SUC_lexicon_tags_23 },
    { 0xd53d30e5, SUC_lexicon_tags_30 },
    { 0x7dd30e6, SUC_lexicon_tags_37 },
    { 0xbb9d30e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59d30ef, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d30f4, SUC_lexicon_tags_45 },
    { 0xc05d30f4, SUC_lexicon_tags_26 },
    { 0xcfd30f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9dd30f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x993d30fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19d30ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9bdd3101, SUC_lexicon_tags_39 },
    { 0x14bd3102, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dd3106, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa27d3109, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bd310d, SUC_lexicon_tags_43 },
    { 0xc5bd310d, SUC_lexicon_tags_10 },
    { 0x7f7d310f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb15d3111, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fd3114, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9d3117, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc65d3119, SUC_lexicon_tags_1 },
    { 0x8e9d311a, SUC_lexicon_tags_27 },
    { 0xc83d311b, SUC_lexicon_tags_1 },
    { 0x8add311c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc0fd311e, SUC_lexicon_tags_15 },
    { 0x607d311e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd9bd3121, SUC_lexicon_tags_31 },
    { 0x39dd3122, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5d3127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x497d312a, SUC_lexicon_tags_46 },
    { 0x2c5d312a, SUC_lexicon_tags_4 },
    { 0xb89d312c, SUC_lexicon_tags_17 },
    { 0x6e1d312d, SUC_lexicon_tags_63 },
    { 0xd03d312b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1d3131, SUC_lexicon_tags_1 },
    { 0x7add3132, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7e3d3134, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531d3138, SUC_lexicon_tags_15 },
    { 0x423d3139, SUC_lexicon_tags_29 },
    { 0x7b3d3138, SUC_lexicon_tags_45 },
    { 0x5efd313b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x983d313e, SUC_lexicon_tags_25 },
    { 0x485d313e, SUC_lexicon_tags_79 },
    { 0x40dd3140, SUC_lexicon_tags_1 },
    { 0x3cdd3141, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x607d3143, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6d1d3145, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dd3149, SUC_lexicon_tags_32 },
    { 0xa1dd3149, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77d314d, SUC_lexicon_tags_2 },
    { 0xa5d314e, SUC_lexicon_tags_15 },
    { 0x823d314e, SUC_lexicon_tags_11 },
    { 0x215d3150, SUC_lexicon_tags_25 },
    { 0x4abd3151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fd3157, SUC_lexicon_tags_16 },
    { 0x8a3d3157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bd315b, SUC_lexicon_tags_29 },
    { 0xfdbd315b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1dd315e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x36fd3161, SUC_lexicon_tags_9 },
    { 0xe7bd3162, SUC_lexicon_tags_2 },
    { 0x131d3163, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdd316a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535d316e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cfd3170, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf47d3172, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddd3178, SUC_lexicon_tags_38 },
    { 0xd7d3179, SUC_lexicon_tags_66 },
    { 0xc4bd317a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bd317f, SUC_lexicon_tags_17 },
    { 0x961d317f, SUC_lexicon_tags_17 },
    { 0x5d7d3181, SUC_lexicon_tags_12 },
    { 0x47d3182, SUC_lexicon_tags_25 },
    { 0x963d3181, SUC_lexicon_tags_17 },
    { 0x82bd3180, SUC_lexicon_tags_37 },
    { 0xb8fd3185, SUC_lexicon_tags_16 },
    { 0x1fbd3186, SUC_lexicon_tags_2 },
    { 0x9e3d3186, SUC_lexicon_tags_0 },
    { 0x889d3188, SUC_lexicon_tags_48 },
    { 0xea7d3185, SUC_lexicon_tags_29 },
    { 0xec3d3186, SUC_lexicon_tags_19 },
    { 0xac7d3188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x90fd318d, SUC_lexicon_tags_13 },
    { 0x97d318e, SUC_lexicon_tags_60 },
    { 0x24dd318d, SUC_lexicon_tags_47 },
    { 0xbe9d3190, SUC_lexicon_tags_8 },
    { 0xb03d3191, SUC_lexicon_tags_1 },
    { 0x97d3191, SUC_lexicon_tags_85 },
    { 0xd29d3192, SUC_lexicon_tags_13 },
    { 0x66fd3194, SUC_lexicon_tags_7 },
    { 0x333d3191, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01d3198, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfd319b, SUC_lexicon_tags_37 },
    { 0xbc7d319b, SUC_lexicon_tags_7 },
    { 0x7dfd319d, SUC_lexicon_tags_34 },
    { 0x809d319e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3d31a3, SUC_lexicon_tags_10 },
    { 0xf7fd31a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9d31ad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301d31b3, SUC_lexicon_tags_3 },
    { 0x539d31b3, SUC_lexicon_tags_1 },
    { 0x3cdd31b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x121d31b7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9add31bb, SUC_lexicon_tags_33 },
    { 0xf1bd31bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3d31c1, SUC_lexicon_tags_25 },
    { 0x6b9d31c2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1b5d31c4, SUC_lexicon_tags_63 },
    { 0xd03d31c5, SUC_lexicon_tags_11 },
    { 0x2a5d31c6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1f7d31c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639d31cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1d31cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49d31d2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fd31d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dd31db, SUC_lexicon_tags_45 },
    { 0x31dd31db, SUC_lexicon_tags_14 },
    { 0x3f9d31dc, SUC_lexicon_tags_8 },
    { 0xdb7d31de, SUC_lexicon_tags_38 },
    { 0x42bd31de, SUC_lexicon_tags_37 },
    { 0x509d31e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb9d31e2, SUC_lexicon_tags_65 },
    { 0x96fd31e2, SUC_lexicon_tags_2 },
    { 0xd8fd31e4, SUC_lexicon_tags_29 },
    { 0x815d31e4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3d31eb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x817d31ed, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xbefd31ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x817d31f1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdd31f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x56bd31f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x463d31f9, SUC_lexicon_tags_37 },
    { 0xedbd31fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x583d31fc, SUC_lexicon_tags_15 },
    { 0xe79d31fd, SUC_lexicon_tags_25 },
    { 0xe9fd31fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bd3204, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2add3207, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3d320a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9d320e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc69d3210, SUC_lexicon_tags_11 },
    { 0x32fd3211, SUC_lexicon_tags_2 },
    { 0x36fd3212, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d3215, SUC_lexicon_tags_11 },
    { 0x279d3215, SUC_lexicon_tags_27 },
    { 0xf65d3217, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x177d3219, SUC_lexicon_tags_37 },
    { 0x67dd321a, SUC_lexicon_tags_2 },
    { 0xf21d321b, SUC_lexicon_tags_3 },
    { 0x6b3d321c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43d3226, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5d3229, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x86dd322b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xabbd322d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x975d3230, SUC_lexicon_tags_17 },
    { 0x8f7d3231, SUC_lexicon_tags_10 },
    { 0x5e1d3232, SUC_lexicon_tags_9 },
    { 0xc4dd3233, SUC_lexicon_tags_49 },
    { 0x483d3233, SUC_lexicon_tags_37 },
    { 0x32fd3231, SUC_lexicon_tags_15 },
    { 0xd8fd3236, SUC_lexicon_tags_13 },
    { 0x907d3234, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fd323b, SUC_lexicon_tags_37 },
    { 0x155d323c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69d3243, SUC_lexicon_tags_38 },
    { 0xf3bd3244, SUC_lexicon_tags_1 },
    { 0x53bd3245, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x415d3247, SUC_lexicon_tags_60 },
    { 0xc61d3248, SUC_lexicon_tags_84 },
    { 0x71dd3247, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673d324d, SUC_lexicon_tags_37 },
    { 0x483d324e, SUC_lexicon_tags_7 },
    { 0x663d324f, SUC_lexicon_tags_38 },
    { 0xc01d3250, SUC_lexicon_tags_11 },
    { 0xa27d3251, SUC_lexicon_tags_7 },
    { 0xb57d3251, SUC_lexicon_tags_63 },
    { 0x923d3251, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5d3256, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x715d3259, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bd325d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571d3262, SUC_lexicon_tags_23 },
    { 0xabdd3263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fd3269, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x353d326c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805d3270, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d3273, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57d3277, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9d3282, SUC_lexicon_tags_29 },
    { 0x98bd3283, SUC_lexicon_tags_32 },
    { 0xdc9d3284, SUC_lexicon_tags_53 },
    { 0xc63d3283, SUC_lexicon_tags_19 },
    { 0x863d3283, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d328c, SUC_lexicon_tags_15 },
    { 0xa5bd328c, SUC_lexicon_tags_37 },
    { 0x65dd328e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x219d3290, SUC_lexicon_tags_17 },
    { 0xc8dd3290, SUC_lexicon_tags_15 },
    { 0x26bd3291, SUC_lexicon_tags_10 },
    { 0x553d3291, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x23d3296, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdd329a, SUC_lexicon_tags_17 },
    { 0x6c9d329b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5d32a0, SUC_lexicon_tags_1 },
    { 0x11dd32a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d1d32a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159d32a8, SUC_lexicon_tags_17 },
    { 0xca3d32a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793d32ad, SUC_lexicon_tags_25 },
    { 0xa79d32ae, SUC_lexicon_tags_14 },
    { 0x105d32af, SUC_lexicon_tags_8 },
    { 0xfd3d32ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x66fd32b3, SUC_lexicon_tags_1 },
    { 0x407d32b4, SUC_lexicon_tags_18 },
    { 0x90fd32b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x233d32b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x31d32ba, SUC_lexicon_tags_48 },
    { 0xc57d32bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd32c1, SUC_lexicon_tags_11 },
    { 0xc0fd32c2, SUC_lexicon_tags_16 },
    { 0x755d32c3, SUC_lexicon_tags_79 },
    { 0x735d32c3, SUC_lexicon_tags_15 },
    { 0x85bd32c5, SUC_lexicon_tags_37 },
    { 0xa65d32c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x111d32c8, SUC_lexicon_tags_40 },
    { 0xfbbd32c8, SUC_lexicon_tags_15 },
    { 0x32bd32c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7d32cd, SUC_lexicon_tags_70 },
    { 0xfe7d32cd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdfbd32d0, SUC_lexicon_tags_23 },
    { 0x643d32d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3d32d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5d32d7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bd32da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5d32de, SUC_lexicon_tags_39 },
    { 0xf17d32df, SUC_lexicon_tags_69 },
    { 0x71bd32e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x70bd32e2, SUC_lexicon_tags_31 },
    { 0x7f3d32e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x115d32e5, SUC_lexicon_tags_23 },
    { 0xf29d32e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbd32ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bd32f5, SUC_lexicon_tags_63 },
    { 0x849d32f5, SUC_lexicon_tags_25 },
    { 0x57bd32f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3d32fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf17d32fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9fd3300, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe01d3302, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f5d3304, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29d3306, SUC_lexicon_tags_11 },
    { 0xbd1d3306, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x229d3309, SUC_lexicon_tags_15 },
    { 0x4fdd330a, SUC_lexicon_tags_47 },
    { 0x6bbd330a, SUC_lexicon_tags_48 },
    { 0x7a5d330c, SUC_lexicon_tags_17 },
    { 0xdefd330d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37d3311, SUC_lexicon_tags_14 },
    { 0x77d3311, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfa7d3314, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17d3318, SUC_lexicon_tags_2 },
    { 0xb1d3319, SUC_lexicon_tags_23 },
    { 0x1c1d3318, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6add331d, SUC_lexicon_tags_12 },
    { 0x525d331e, SUC_lexicon_tags_65 },
    { 0xdd9d331f, SUC_lexicon_tags_7 },
    { 0xde3d3320, SUC_lexicon_tags_23 },
    { 0x91dd3321, SUC_lexicon_tags_0 },
    { 0xcf9d3321, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x76fd3324, SUC_lexicon_tags_21 },
    { 0x35dd3325, SUC_lexicon_tags_22 },
    { 0x72bd3326, SUC_lexicon_tags_9 },
    { 0x155d3327, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7d332a, SUC_lexicon_tags_7 },
    { 0x4cdd332b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac9d332d, SUC_lexicon_tags_3 },
    { 0xd0fd332e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28dd3332, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bd3335, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fd333a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dd334a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeadd334d, SUC_lexicon_tags_26 },
    { 0x6a7d334e, SUC_lexicon_tags_45 },
    { 0x12fd334e, SUC_lexicon_tags_1 },
    { 0xce5d334f, SUC_lexicon_tags_26 },
    { 0x28fd3351, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbd3354, SUC_lexicon_tags_17 },
    { 0xa41d3354, SUC_lexicon_tags_15 },
    { 0xfbd3354, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7cbd3358, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x937d335b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fd3362, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x11fd3364, SUC_lexicon_tags_45 },
    { 0xce9d3364, SUC_lexicon_tags_0 },
    { 0x671d3366, SUC_lexicon_tags_77 },
    { 0xf03d3367, SUC_lexicon_tags_761 },
    { 0xd3dd3368, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325d336e, SUC_lexicon_tags_19 },
    { 0x5d1d336f, SUC_lexicon_tags_10 },
    { 0xef1d3370, SUC_lexicon_tags_27 },
    { 0x767d336f, SUC_lexicon_tags_12 },
    { 0x809d3372, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x129d3374, SUC_lexicon_tags_17 },
    { 0x475d3374, SUC_lexicon_tags_17 },
    { 0x859d3376, SUC_lexicon_tags_17 },
    { 0x701d3377, SUC_lexicon_tags_23 },
    { 0x985d3378, SUC_lexicon_tags_14 },
    { 0xd1bd3378, SUC_lexicon_tags_56 },
    { 0xfa5d337a, SUC_lexicon_tags_31 },
    { 0xd37d337a, SUC_lexicon_tags_17 },
    { 0x853d337c, SUC_lexicon_tags_17 },
    { 0x285d337c, SUC_lexicon_tags_27 },
    { 0x98dd3377, SUC_lexicon_tags_7 },
    { 0xa3fd3379, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x289d3381, SUC_lexicon_tags_12 },
    { 0x903d3381, SUC_lexicon_tags_11 },
    { 0xa4bd3381, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ddd3385, SUC_lexicon_tags_5 },
    { 0x663d3386, SUC_lexicon_tags_38 },
    { 0x939d3387, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8bbd3389, SUC_lexicon_tags_1 },
    { 0xedd3389, SUC_lexicon_tags_2 },
    { 0x297d3389, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x279d338d, SUC_lexicon_tags_15 },
    { 0xdcbd338e, SUC_lexicon_tags_1 },
    { 0xaafd338e, SUC_lexicon_tags_23 },
    { 0xc55d338f, SUC_lexicon_tags_17 },
    { 0x4a9d338f, SUC_lexicon_tags_0 },
    { 0xeb3d3392, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x181d3394, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf3d3396, SUC_lexicon_tags_43 },
    { 0xc43d3397, SUC_lexicon_tags_49 },
    { 0xb57d3398, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x499d339a, SUC_lexicon_tags_13 },
    { 0x741d339a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x797d339d, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d33a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4dbd33a2, SUC_lexicon_tags_15 },
    { 0xe79d33a3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x11bd33a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdd33a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xabfd33aa, SUC_lexicon_tags_7 },
    { 0x695d33ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5d33af, SUC_lexicon_tags_6 },
    { 0x387d33b0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dd33b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bd33b8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe3fd33ba, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d33c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3d33c5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x503d33c7, SUC_lexicon_tags_7 },
    { 0x709d33c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dd33cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bd33d1, SUC_lexicon_tags_37 },
    { 0xe65d33d2, SUC_lexicon_tags_15 },
    { 0xd85d33d1, SUC_lexicon_tags_14 },
    { 0x995d33d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x35fd33d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9d33db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d33de, SUC_lexicon_tags_17 },
    { 0x31bd33df, SUC_lexicon_tags_2 },
    { 0x7c7d33de, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c1d33e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959d33e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801d33ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835d33f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5d33fc, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x999d33fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d340c, SUC_lexicon_tags_17 },
    { 0xf6fd340d, SUC_lexicon_tags_17 },
    { 0x61fd340d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d3412, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895d341c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3d3424, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9d3428, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd342d, SUC_lexicon_tags_11 },
    { 0x15dd342e, SUC_lexicon_tags_2 },
    { 0x94dd342f, SUC_lexicon_tags_31 },
    { 0xa47d3430, SUC_lexicon_tags_12 },
    { 0xa4dd342e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219d3435, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd3437, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x857d343a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159d3441, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bd3446, SUC_lexicon_tags_23 },
    { 0x833d3447, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837d344b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x501d344d, SUC_lexicon_tags_11 },
    { 0xb55d344e, SUC_lexicon_tags_33 },
    { 0x439d344f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x907d3452, SUC_lexicon_tags_737 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797d345b, SUC_lexicon_tags_0 },
    { 0x9a3d345c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfcdd345e, SUC_lexicon_tags_9 },
    { 0x3f5d345f, SUC_lexicon_tags_39 },
    { 0xae9d3460, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3d3466, SUC_lexicon_tags_8 },
    { 0x6c7d3467, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bd346d, SUC_lexicon_tags_38 },
    { 0xd51d346d, SUC_lexicon_tags_49 },
    { 0x85d346f, SUC_lexicon_tags_17 },
    { 0xda5d346e, SUC_lexicon_tags_2 },
    { 0xe11d3471, SUC_lexicon_tags_16 },
    { 0x357d3472, SUC_lexicon_tags_60 },
    { 0xaf9d3471, SUC_lexicon_tags_45 },
    { 0x701d3474, SUC_lexicon_tags_2 },
    { 0xa13d3474, SUC_lexicon_tags_22 },
    { 0xb97d3476, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedd347a, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3d347d, SUC_lexicon_tags_2 },
    { 0xa2bd347e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1d3484, SUC_lexicon_tags_2 },
    { 0x867d3485, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbbd3487, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xad7d3489, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x351d348b, SUC_lexicon_tags_29 },
    { 0x44fd348c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61d3491, SUC_lexicon_tags_23 },
    { 0x811d3492, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11d3496, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa5bd3498, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dd349c, SUC_lexicon_tags_15 },
    { 0x631d349c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bd34a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x675d34a5, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7d34a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x557d34ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3d34b2, SUC_lexicon_tags_21 },
    { 0xef9d34b2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265d34b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05d34bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dd34c4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9d34cb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x15dd34cd, SUC_lexicon_tags_15 },
    { 0x1e1d34ce, SUC_lexicon_tags_26 },
    { 0x83d34cd, SUC_lexicon_tags_26 },
    { 0x41bd34d0, SUC_lexicon_tags_0 },
    { 0xe9dd34d1, SUC_lexicon_tags_7 },
    { 0x23bd34d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x307d34d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11d34db, SUC_lexicon_tags_47 },
    { 0xf33d34dc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x265d34de, SUC_lexicon_tags_77 },
    { 0x31d34df, SUC_lexicon_tags_48 },
    { 0x3d1d34e0, SUC_lexicon_tags_47 },
    { 0x4cbd34df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fd34e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75d34eb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1e7d34ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe53d34ef, SUC_lexicon_tags_12 },
    { 0xf7d34f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455d34f4, SUC_lexicon_tags_3 },
    { 0xc13d34f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3d34f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39dd34fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55dd34ff, SUC_lexicon_tags_80 },
    { 0x87d3500, SUC_lexicon_tags_32 },
    { 0x4bdd3500, SUC_lexicon_tags_45 },
    { 0x317d3502, SUC_lexicon_tags_17 },
    { 0xbb3d3502, SUC_lexicon_tags_1 },
    { 0x737d3501, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631d3508, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x879d350b, SUC_lexicon_tags_30 },
    { 0xf9dd350b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dd3511, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a5d3513, SUC_lexicon_tags_1 },
    { 0xbf5d3513, SUC_lexicon_tags_3 },
    { 0x133d3515, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dd3518, SUC_lexicon_tags_25 },
    { 0x89fd3519, SUC_lexicon_tags_30 },
    { 0xb1fd351a, SUC_lexicon_tags_11 },
    { 0x331d3518, SUC_lexicon_tags_38 },
    { 0x2a7d3519, SUC_lexicon_tags_978 },
    { 0x399d351d, SUC_lexicon_tags_703 },
    { 0xdcbd351d, SUC_lexicon_tags_1 },
    { 0x1d5d351e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74dd3527, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x42bd352a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x575d352c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdd353f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1d3542, SUC_lexicon_tags_376 },
    { 0xf49d3543, SUC_lexicon_tags_29 },
    { 0x8b9d3544, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x729d3547, SUC_lexicon_tags_33 },
    { 0x711d3548, SUC_lexicon_tags_11 },
    { 0xf6bd3547, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x66bd354b, SUC_lexicon_tags_37 },
    { 0xf3bd354c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13d354f, SUC_lexicon_tags_12 },
    { 0x8d7d3550, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2fd3552, SUC_lexicon_tags_8 },
    { 0xbebd3552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbd3556, SUC_lexicon_tags_105 },
    { 0x469d3556, SUC_lexicon_tags_19 },
    { 0xde9d3557, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905d355c, SUC_lexicon_tags_4 },
    { 0x5add355d, SUC_lexicon_tags_11 },
    { 0xfd1d355d, SUC_lexicon_tags_25 },
    { 0xbebd355f, SUC_lexicon_tags_10 },
    { 0x919d355f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463d3566, SUC_lexicon_tags_32 },
    { 0x37fd3567, SUC_lexicon_tags_33 },
    { 0x9b7d3568, SUC_lexicon_tags_2 },
    { 0x1a9d3569, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bd356c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5d3570, SUC_lexicon_tags_21 },
    { 0x83fd3571, SUC_lexicon_tags_45 },
    { 0x893d3571, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdd1d3574, SUC_lexicon_tags_12 },
    { 0x25d3575, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaa7d3577, SUC_lexicon_tags_17 },
    { 0x4ebd3578, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9d357b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565d357f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bd3584, SUC_lexicon_tags_19 },
    { 0xae9d3585, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511d358b, SUC_lexicon_tags_4 },
    { 0x11d358c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651d3590, SUC_lexicon_tags_25 },
    { 0xd1dd3590, SUC_lexicon_tags_0 },
    { 0x2ffd3592, SUC_lexicon_tags_12 },
    { 0x5fd3591, SUC_lexicon_tags_3 },
    { 0xb89d3590, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3d3597, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bd359c, SUC_lexicon_tags_38 },
    { 0x5d9d359d, SUC_lexicon_tags_15 },
    { 0xb1bd359c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fd35a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x45fd35a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x725d35a7, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdd35ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8dd35ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47d35b2, SUC_lexicon_tags_2 },
    { 0xd61d35b3, SUC_lexicon_tags_65 },
    { 0x321d35b3, SUC_lexicon_tags_38 },
    { 0xd49d35b5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fd35bb, SUC_lexicon_tags_14 },
    { 0x705d35bc, SUC_lexicon_tags_68 },
    { 0x70bd35bd, SUC_lexicon_tags_33 },
    { 0xf7fd35be, SUC_lexicon_tags_32 },
    { 0x23fd35be, SUC_lexicon_tags_17 },
    { 0xe7bd35bf, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bd35c4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51d35c8, SUC_lexicon_tags_45 },
    { 0x8abd35c8, SUC_lexicon_tags_17 },
    { 0xe99d35c9, SUC_lexicon_tags_11 },
    { 0x8b7d35ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9d35d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x399d35d3, SUC_lexicon_tags_15 },
    { 0x8add35d4, SUC_lexicon_tags_26 },
    { 0xf0fd35d4, SUC_lexicon_tags_47 },
    { 0x81bd35d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x445d35d9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x101d35dc, SUC_lexicon_tags_11 },
    { 0x291d35dd, SUC_lexicon_tags_12 },
    { 0xb9fd35dd, SUC_lexicon_tags_47 },
    { 0x519d35de, SUC_lexicon_tags_27 },
    { 0x6c5d35e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x781d35e2, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7d35e7, SUC_lexicon_tags_27 },
    { 0xcdfd35e7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf61d35ea, SUC_lexicon_tags_14 },
    { 0xab5d35eb, SUC_lexicon_tags_25 },
    { 0xc9fd35ec, SUC_lexicon_tags_85 },
    { 0x9ddd35ed, SUC_lexicon_tags_11 },
    { 0xc1d35ee, SUC_lexicon_tags_7 },
    { 0x65bd35ee, SUC_lexicon_tags_30 },
    { 0x125d35ee, SUC_lexicon_tags_14 },
    { 0x3fd35ef, SUC_lexicon_tags_10 },
    { 0x2c5d35f2, SUC_lexicon_tags_146 },
    { 0x885d35f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dd35f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f3d35f9, SUC_lexicon_tags_38 },
    { 0xefd35fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc41d35fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x497d35ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x365d3601, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673d3606, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadd3609, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44bd360b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dd3611, SUC_lexicon_tags_37 },
    { 0x4f5d3612, SUC_lexicon_tags_60 },
    { 0xb89d3611, SUC_lexicon_tags_43 },
    { 0x6f7d3614, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bd361b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffd361e, SUC_lexicon_tags_63 },
    { 0x1c5d361e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xae7d3621, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4a9d3623, SUC_lexicon_tags_11 },
    { 0x5c3d3623, SUC_lexicon_tags_11 },
    { 0xc83d3625, SUC_lexicon_tags_1 },
    { 0x1e3d3623, SUC_lexicon_tags_11 },
    { 0x32fd3623, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4e3d3629, SUC_lexicon_tags_7 },
    { 0xdbbd362a, SUC_lexicon_tags_11 },
    { 0x47bd362b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787d3631, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943d3635, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01d3638, SUC_lexicon_tags_47 },
    { 0xe11d3639, SUC_lexicon_tags_7 },
    { 0x4efd363a, SUC_lexicon_tags_7 },
    { 0x6afd363b, SUC_lexicon_tags_7 },
    { 0xe6bd363c, SUC_lexicon_tags_25 },
    { 0x287d363d, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3d3643, SUC_lexicon_tags_43 },
    { 0xde5d3643, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b1d3646, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7dd3648, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67d364d, SUC_lexicon_tags_37 },
    { 0x2a1d364e, SUC_lexicon_tags_32 },
    { 0x4f7d364f, SUC_lexicon_tags_17 },
    { 0x57bd3650, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x335d3652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd3655, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x301d3658, SUC_lexicon_tags_8 },
    { 0x325d3659, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58dd365b, SUC_lexicon_tags_23 },
    { 0xe65d365c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeedd365e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1d3661, SUC_lexicon_tags_27 },
    { 0xf5dd3661, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d366c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xacdd366e, SUC_lexicon_tags_27 },
    { 0xb3d366f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x57fd3671, SUC_lexicon_tags_14 },
    { 0xd93d3671, SUC_lexicon_tags_11 },
    { 0x6f9d3673, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x623d3675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333d367c, SUC_lexicon_tags_17 },
    { 0x289d367d, SUC_lexicon_tags_8 },
    { 0xf3bd367c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdd3682, SUC_lexicon_tags_43 },
    { 0x665d3683, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x93fd3685, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaafd3687, SUC_lexicon_tags_0 },
    { 0x57bd3688, SUC_lexicon_tags_14 },
    { 0x209d3689, SUC_lexicon_tags_30 },
    { 0x8c3d3689, SUC_lexicon_tags_46 },
    { 0x715d3689, SUC_lexicon_tags_1 },
    { 0x6b5d368c, SUC_lexicon_tags_7 },
    { 0x309d368a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3d3690, SUC_lexicon_tags_17 },
    { 0xd3fd3691, SUC_lexicon_tags_37 },
    { 0xb83d3691, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x8cfd3694, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7d3697, SUC_lexicon_tags_112 },
    { 0x4cbd3697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffd36a0, SUC_lexicon_tags_25 },
    { 0xf25d36a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85d36a4, SUC_lexicon_tags_29 },
    { 0x191d36a5, SUC_lexicon_tags_37 },
    { 0xbb1d36a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x34fd36a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dd36ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291d36b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x247d36ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadd36bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259d36c3, SUC_lexicon_tags_23 },
    { 0xe91d36c4, SUC_lexicon_tags_12 },
    { 0x9afd36c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dd36c8, SUC_lexicon_tags_25 },
    { 0x9e5d36c9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9d36cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35d36cf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d36d2, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0xca7d36d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb63d36d6, SUC_lexicon_tags_17 },
    { 0xab3d36d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09d36df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3d36e3, SUC_lexicon_tags_9 },
    { 0x111d36e4, SUC_lexicon_tags_25 },
    { 0xebd36e5, SUC_lexicon_tags_14 },
    { 0x28dd36e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1d36e9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffd36ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85d36f2, SUC_lexicon_tags_14 },
    { 0xaadd36f2, SUC_lexicon_tags_17 },
    { 0x789d36f4, SUC_lexicon_tags_261 },
    { 0xd2bd36f4, SUC_lexicon_tags_29 },
    { 0xb5d36f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9d36f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95d36fe, SUC_lexicon_tags_23 },
    { 0x8add36fe, SUC_lexicon_tags_1 },
    { 0xcefd3700, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd1d3702, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa8dd3704, SUC_lexicon_tags_8 },
    { 0x5f7d3704, SUC_lexicon_tags_87 },
    { 0x263d3705, SUC_lexicon_tags_1 },
    { 0x897d3707, SUC_lexicon_tags_45 },
    { 0xff5d3708, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x43bd370a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdd370d, SUC_lexicon_tags_0 },
    { 0x771d370e, SUC_lexicon_tags_2 },
    { 0x7c7d370e, SUC_lexicon_tags_15 },
    { 0x921d3710, SUC_lexicon_tags_37 },
    { 0x747d3710, SUC_lexicon_tags_37 },
    { 0x6dbd3710, SUC_lexicon_tags_32 },
    { 0x98bd3713, SUC_lexicon_tags_19 },
    { 0xf0fd3714, SUC_lexicon_tags_17 },
    { 0x17dd3715, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x1f3d3717, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d371a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fd371d, SUC_lexicon_tags_12 },
    { 0xa0fd371e, SUC_lexicon_tags_56 },
    { 0x91fd371f, SUC_lexicon_tags_1 },
    { 0x813d3720, SUC_lexicon_tags_15 },
    { 0x24bd3721, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881d3726, SUC_lexicon_tags_26 },
    { 0x379d3726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x697d3729, SUC_lexicon_tags_30 },
    { 0x843d372a, SUC_lexicon_tags_37 },
    { 0xa6bd372b, SUC_lexicon_tags_7 },
    { 0x8fbd372c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7d372f, SUC_lexicon_tags_46 },
    { 0x6efd3730, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9d3738, SUC_lexicon_tags_17 },
    { 0x617d3739, SUC_lexicon_tags_45 },
    { 0x6a9d3739, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bd373f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd3743, SUC_lexicon_tags_19 },
    { 0x879d3743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183d3748, SUC_lexicon_tags_47 },
    { 0xaa7d3749, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x479d374c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7d3754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fd3758, SUC_lexicon_tags_12 },
    { 0x2abd3759, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627d375d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24dd375f, SUC_lexicon_tags_11 },
    { 0x4a5d3760, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33d3764, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbd3768, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x201d376b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb4bd376d, SUC_lexicon_tags_25 },
    { 0xb6bd376e, SUC_lexicon_tags_12 },
    { 0xde5d376f, SUC_lexicon_tags_65 },
    { 0x577d3770, SUC_lexicon_tags_17 },
    { 0x4f3d3771, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bd3776, SUC_lexicon_tags_11 },
    { 0x5e5d3776, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x479d3779, SUC_lexicon_tags_26 },
    { 0xb2bd377a, SUC_lexicon_tags_0 },
    { 0x5dd377b, SUC_lexicon_tags_17 },
    { 0xa1d3779, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x11dd377e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfefd3780, SUC_lexicon_tags_66 },
    { 0xe31d3781, SUC_lexicon_tags_15 },
    { 0x7a9d3782, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x11dd3784, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x29fd3787, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3d378d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa8dd378f, SUC_lexicon_tags_2 },
    { 0x3dfd378f, SUC_lexicon_tags_6 },
    { 0x3e9d3790, SUC_lexicon_tags_6 },
    { 0xb83d3792, SUC_lexicon_tags_33 },
    { 0xa2dd3791, SUC_lexicon_tags_1 },
    { 0x781d3793, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b5d3796, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd379b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7d379e, SUC_lexicon_tags_37 },
    { 0xe9fd379f, SUC_lexicon_tags_0 },
    { 0xd55d379e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a9d37a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba7d37a4, SUC_lexicon_tags_11 },
    { 0x6d5d37a5, SUC_lexicon_tags_37 },
    { 0x6cfd37a6, SUC_lexicon_tags_27 },
    { 0x425d37a7, SUC_lexicon_tags_0 },
    { 0x81fd37a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8bd37aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x399d37ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577d37b2, SUC_lexicon_tags_15 },
    { 0x173d37b3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc09d37b5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc49d37b7, SUC_lexicon_tags_37 },
    { 0xcfbd37b7, SUC_lexicon_tags_1 },
    { 0x7d3d37b9, SUC_lexicon_tags_37 },
    { 0xa97d37ba, SUC_lexicon_tags_16 },
    { 0xe4fd37ba, SUC_lexicon_tags_28 },
    { 0x609d37b9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x45dd37bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x647d37c1, SUC_lexicon_tags_8 },
    { 0xf07d37c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebd37c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bd37ce, SUC_lexicon_tags_25 },
    { 0xd33d37ce, SUC_lexicon_tags_13 },
    { 0x337d37cf, SUC_lexicon_tags_17 },
    { 0xc55d37ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x787d37d3, SUC_lexicon_tags_1 },
    { 0x3e5d37d4, SUC_lexicon_tags_37 },
    { 0xd0fd37d5, SUC_lexicon_tags_14 },
    { 0xf37d37d5, SUC_lexicon_tags_37 },
    { 0x94bd37d6, SUC_lexicon_tags_60 },
    { 0xa23d37d8, SUC_lexicon_tags_37 },
    { 0x673d37d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1d1d37db, SUC_lexicon_tags_12 },
    { 0x2f3d37dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec1d37de, SUC_lexicon_tags_15 },
    { 0x553d37df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fd37e7, SUC_lexicon_tags_17 },
    { 0x31d37e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fd37f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6a7d37f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1d37fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2dbd3800, SUC_lexicon_tags_7 },
    { 0x33d3801, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x493d3803, SUC_lexicon_tags_37 },
    { 0x167d3804, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d3d3806, SUC_lexicon_tags_35 },
    { 0x335d3807, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd380e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07d3811, SUC_lexicon_tags_6 },
    { 0x687d3811, SUC_lexicon_tags_15 },
    { 0x561d3811, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dd3818, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bd381c, SUC_lexicon_tags_25 },
    { 0x70dd381d, SUC_lexicon_tags_7 },
    { 0xf41d381d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117d3822, SUC_lexicon_tags_12 },
    { 0x9b9d3822, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fd382d, SUC_lexicon_tags_47 },
    { 0xd1fd382e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995d3838, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1a7d383a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabd3843, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x675d3845, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd384f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01d3852, SUC_lexicon_tags_108 },
    { 0x4d1d3853, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x589d3855, SUC_lexicon_tags_45 },
    { 0x7dbd3856, SUC_lexicon_tags_63 },
    { 0x249d3857, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fd385f, SUC_lexicon_tags_48 },
    { 0xf35d3860, SUC_lexicon_tags_38 },
    { 0x99dd3860, SUC_lexicon_tags_7 },
    { 0x26fd3862, SUC_lexicon_tags_31 },
    { 0x20dd3863, SUC_lexicon_tags_12 },
    { 0x201d3863, SUC_lexicon_tags_29 },
    { 0x33d3865, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5d386b, SUC_lexicon_tags_33 },
    { 0x7a5d386b, SUC_lexicon_tags_6 },
    { 0x681d386d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x53fd3870, SUC_lexicon_tags_17 },
    { 0xff1d3870, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dd3875, SUC_lexicon_tags_74 },
    { 0xaffd3875, SUC_lexicon_tags_57 },
    { 0xc53d3876, SUC_lexicon_tags_25 },
    { 0x81dd3875, SUC_lexicon_tags_25 },
    { 0x557d3878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x355d387c, SUC_lexicon_tags_37 },
    { 0x9d5d387d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa41d387f, SUC_lexicon_tags_17 },
    { 0x1c5d387f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89bd3882, SUC_lexicon_tags_17 },
    { 0x391d3882, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9d3887, SUC_lexicon_tags_11 },
    { 0x7efd3888, SUC_lexicon_tags_27 },
    { 0xf75d3889, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bd3891, SUC_lexicon_tags_37 },
    { 0x965d3892, SUC_lexicon_tags_15 },
    { 0xe19d3892, SUC_lexicon_tags_7 },
    { 0x511d3892, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x913d3896, SUC_lexicon_tags_55 },
    { 0xe05d3897, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdd389e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bd38a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57d38a7, SUC_lexicon_tags_40 },
    { 0xf4fd38a7, SUC_lexicon_tags_37 },
    { 0x581d38a8, SUC_lexicon_tags_47 },
    { 0xf43d38aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1d38ad, SUC_lexicon_tags_16 },
    { 0x84bd38ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xff3d38b0, SUC_lexicon_tags_12 },
    { 0xb8bd38b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5d38b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdc7d38b6, SUC_lexicon_tags_12 },
    { 0xae7d38b6, SUC_lexicon_tags_2 },
    { 0x8b1d38b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaafd38ba, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37d38c0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1d38c9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdd38d4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5d38d9, SUC_lexicon_tags_21 },
    { 0xf91d38da, SUC_lexicon_tags_17 },
    { 0x9bfd38da, SUC_lexicon_tags_30 },
    { 0xb25d38da, SUC_lexicon_tags_15 },
    { 0x26bd38dc, SUC_lexicon_tags_2 },
    { 0xd65d38de, SUC_lexicon_tags_61 },
    { 0xd3dd38de, SUC_lexicon_tags_17 },
    { 0x575d38e0, SUC_lexicon_tags_79 },
    { 0x8a9d38e0, SUC_lexicon_tags_15 },
    { 0xa0fd38e2, SUC_lexicon_tags_28 },
    { 0xfedd38e3, SUC_lexicon_tags_11 },
    { 0xfe5d38e2, SUC_lexicon_tags_12 },
    { 0xb8bd38e4, SUC_lexicon_tags_11 },
    { 0xdcbd38e2, SUC_lexicon_tags_40 },
    { 0x639d38e3, SUC_lexicon_tags_65 },
    { 0xefbd38e8, SUC_lexicon_tags_7 },
    { 0x69d38de, SUC_lexicon_tags_11 },
    { 0x6a5d38ea, SUC_lexicon_tags_26 },
    { 0xcf7d38eb, SUC_lexicon_tags_25 },
    { 0xa65d38eb, SUC_lexicon_tags_43 },
    { 0x6f7d38ed, SUC_lexicon_tags_11 },
    { 0x47fd38ee, SUC_lexicon_tags_2 },
    { 0x75bd38d9, SUC_lexicon_tags_23 },
    { 0x32dd38f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffbd38f2, SUC_lexicon_tags_25 },
    { 0xd5d38f2, SUC_lexicon_tags_11 },
    { 0xd1dd38f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xedfd38f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719d38fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe41d38fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdd3902, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d3906, SUC_lexicon_tags_12 },
    { 0xca9d3907, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fd390b, SUC_lexicon_tags_63 },
    { 0x98dd390b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffd3910, SUC_lexicon_tags_15 },
    { 0x7ffd3911, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbd3914, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x635d3916, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x255d3918, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1d391c, SUC_lexicon_tags_30 },
    { 0xc4fd391d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x96bd391f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcc3d3921, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7d3929, SUC_lexicon_tags_30 },
    { 0xa4bd392a, SUC_lexicon_tags_31 },
    { 0x985d392b, SUC_lexicon_tags_11 },
    { 0x4b5d392a, SUC_lexicon_tags_17 },
    { 0xa6bd392d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3bdd392f, SUC_lexicon_tags_31 },
    { 0x8f1d3930, SUC_lexicon_tags_49 },
    { 0x677d3930, SUC_lexicon_tags_17 },
    { 0x7bdd3932, SUC_lexicon_tags_15 },
    { 0x9f9d3933, SUC_lexicon_tags_12 },
    { 0x805d3932, SUC_lexicon_tags_19 },
    { 0x67fd3932, SUC_lexicon_tags_7 },
    { 0x72fd3935, SUC_lexicon_tags_65 },
    { 0xaabd3933, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x8c3d3939, SUC_lexicon_tags_66 },
    { 0xd35d3939, SUC_lexicon_tags_44 },
    { 0xb01d393b, SUC_lexicon_tags_43 },
    { 0xae5d393c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d3940, SUC_lexicon_tags_1 },
    { 0xb5bd3941, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35d3944, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57d3946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29d3949, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19d394c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75d394f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6cbd3951, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9d3954, SUC_lexicon_tags_17 },
    { 0x89d3955, SUC_lexicon_tags_29 },
    { 0xc3d3956, SUC_lexicon_tags_25 },
    { 0xf61d3956, SUC_lexicon_tags_12 },
    { 0x325d3954, SUC_lexicon_tags_4 },
    { 0x9ebd3956, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc1fd395b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebd395e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x895d3961, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9d3965, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5d396f, SUC_lexicon_tags_4 },
    { 0x49d3970, SUC_lexicon_tags_23 },
    { 0xfe9d3971, SUC_lexicon_tags_15 },
    { 0xfbd396f, SUC_lexicon_tags_60 },
    { 0x87bd3973, SUC_lexicon_tags_1 },
    { 0x27fd3974, SUC_lexicon_tags_46 },
    { 0xedbd3975, SUC_lexicon_tags_37 },
    { 0xc3bd3975, SUC_lexicon_tags_39 },
    { 0x25fd3976, SUC_lexicon_tags_31 },
    { 0x121d3978, SUC_lexicon_tags_7 },
    { 0x9cbd3979, SUC_lexicon_tags_17 },
    { 0x47d397a, SUC_lexicon_tags_30 },
    { 0xbbfd3977, SUC_lexicon_tags_12 },
    { 0x7e1d397c, SUC_lexicon_tags_1 },
    { 0x691d397d, SUC_lexicon_tags_14 },
    { 0xdbd397e, SUC_lexicon_tags_32 },
    { 0x9f1d397f, SUC_lexicon_tags_14 },
    { 0xd9dd397c, SUC_lexicon_tags_38 },
    { 0xb19d397a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281d398b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bd398e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309d3992, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdd3997, SUC_lexicon_tags_7 },
    { 0x189d3998, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x317d399b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3fbd399d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bd39a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abd39a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5d39aa, SUC_lexicon_tags_63 },
    { 0x2c5d39ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef9d39ad, SUC_lexicon_tags_19 },
    { 0xb6bd39ae, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xebfd39b0, SUC_lexicon_tags_13 },
    { 0x767d39b0, SUC_lexicon_tags_11 },
    { 0xdd3d39b0, SUC_lexicon_tags_17 },
    { 0xe9bd39b3, SUC_lexicon_tags_4 },
    { 0x9a1d39b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57d39b7, SUC_lexicon_tags_4 },
    { 0x38fd39b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1d39bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x335d39bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89bd39bf, SUC_lexicon_tags_60 },
    { 0xea1d39c0, SUC_lexicon_tags_27 },
    { 0xb77d39bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d39c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7d39c8, SUC_lexicon_tags_27 },
    { 0x951d39c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7d39d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbd39d7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2bbd39d9, SUC_lexicon_tags_37 },
    { 0xd6bd39d9, SUC_lexicon_tags_52 },
    { 0xe71d39db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7d39de, SUC_lexicon_tags_17 },
    { 0x53d39df, SUC_lexicon_tags_113 },
    { 0xb73d39de, SUC_lexicon_tags_37 },
    { 0x5afd39e1, SUC_lexicon_tags_77 },
    { 0xdb7d39e1, SUC_lexicon_tags_32 },
    { 0x77fd39e3, SUC_lexicon_tags_17 },
    { 0x9cbd39e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dd39ec, SUC_lexicon_tags_17 },
    { 0x221d39ec, SUC_lexicon_tags_4 },
    { 0x32bd39ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861d39f3, SUC_lexicon_tags_38 },
    { 0x925d39f4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bd3a00, SUC_lexicon_tags_15 },
    { 0x301d3a01, SUC_lexicon_tags_15 },
    { 0xeebd3a02, SUC_lexicon_tags_34 },
    { 0x6e9d3a03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4fd3a05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3d3a0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879d3a11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bd3a17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afd3a1a, SUC_lexicon_tags_4 },
    { 0x623d3a1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d3a1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5d3a26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bd3a2a, SUC_lexicon_tags_15 },
    { 0x67fd3a2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cdd3a2d, SUC_lexicon_tags_33 },
    { 0xbdbd3a2d, SUC_lexicon_tags_37 },
    { 0xb33d3a2f, SUC_lexicon_tags_2 },
    { 0x1c1d3a2e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe8dd3a32, SUC_lexicon_tags_13 },
    { 0x223d3a33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51d3a36, SUC_lexicon_tags_64 },
    { 0xdbbd3a36, SUC_lexicon_tags_0 },
    { 0xb2bd3a37, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfd3a3d, SUC_lexicon_tags_32 },
    { 0x5c3d3a3e, SUC_lexicon_tags_49 },
    { 0x9fd3a3d, SUC_lexicon_tags_29 },
    { 0xf5d3a40, SUC_lexicon_tags_25 },
    { 0x997d3a3e, SUC_lexicon_tags_12 },
    { 0x7f7d3a42, SUC_lexicon_tags_165 },
    { 0x52bd3a43, SUC_lexicon_tags_14 },
    { 0x159d3a44, SUC_lexicon_tags_31 },
    { 0xd6bd3a44, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5d3a48, SUC_lexicon_tags_27 },
    { 0x2c3d3a49, SUC_lexicon_tags_38 },
    { 0x451d3a48, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05d3a55, SUC_lexicon_tags_6 },
    { 0x493d3a55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7d3a5c, SUC_lexicon_tags_7 },
    { 0xd69d3a5d, SUC_lexicon_tags_49 },
    { 0xdd7d3a5c, SUC_lexicon_tags_7 },
    { 0xd3fd3a5c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45d3a66, SUC_lexicon_tags_25 },
    { 0x501d3a67, SUC_lexicon_tags_63 },
    { 0x48bd3a68, SUC_lexicon_tags_25 },
    { 0xd83d3a69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bd3a6c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x141d3a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45bd3a70, SUC_lexicon_tags_2 },
    { 0x8b3d3a70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1d3a74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x959d3a76, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xddd3a78, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d3a7b, SUC_lexicon_tags_27 },
    { 0x199d3a7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313d3a84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2d5d3a86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe71d3a88, SUC_lexicon_tags_11 },
    { 0x393d3a88, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee9d3a8b, SUC_lexicon_tags_11 },
    { 0x631d3a8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69d3a90, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fd3a93, SUC_lexicon_tags_6 },
    { 0xdf1d3a94, SUC_lexicon_tags_11 },
    { 0xd17d3a95, SUC_lexicon_tags_33 },
    { 0xc9fd3a95, SUC_lexicon_tags_43 },
    { 0x3f3d3a96, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xa11d3a99, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x7bd3a9b, SUC_lexicon_tags_7 },
    { 0x109d3a9b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf67d3a9e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79d3aa3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x755d3aa6, SUC_lexicon_tags_11 },
    { 0x4d5d3aa7, SUC_lexicon_tags_0 },
    { 0xb7fd3aa8, SUC_lexicon_tags_1 },
    { 0xb21d3aa9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dd3ab2, SUC_lexicon_tags_33 },
    { 0x23d3ab3, SUC_lexicon_tags_15 },
    { 0xcd3d3ab3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x103d3ab7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x2afd3ab9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3d3abc, SUC_lexicon_tags_27 },
    { 0x959d3abd, SUC_lexicon_tags_60 },
    { 0x575d3abd, SUC_lexicon_tags_57 },
    { 0x3bbd3abf, SUC_lexicon_tags_17 },
    { 0xc2fd3abf, SUC_lexicon_tags_25 },
    { 0x52fd3abe, SUC_lexicon_tags_12 },
    { 0xdadd3ac2, SUC_lexicon_tags_12 },
    { 0xed9d3abe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd3ac8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x53fd3acb, SUC_lexicon_tags_17 },
    { 0xff7d3acb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59d3ad3, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbd3ad8, SUC_lexicon_tags_39 },
    { 0x479d3ad9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bd3ae2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3ae5, SUC_lexicon_tags_37 },
    { 0x3a9d3ae6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5d3aef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x453d3af2, SUC_lexicon_tags_65 },
    { 0xf87d3af2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77d3af7, SUC_lexicon_tags_32 },
    { 0x189d3af7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddd3afb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7d3b00, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7d3b03, SUC_lexicon_tags_15 },
    { 0x5d7d3b03, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x57d3b06, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x64fd3b08, SUC_lexicon_tags_8 },
    { 0xbdd3b08, SUC_lexicon_tags_81 },
    { 0xff1d3b0a, SUC_lexicon_tags_11 },
    { 0xeb7d3b0b, SUC_lexicon_tags_70 },
    { 0x8b1d3b0c, SUC_lexicon_tags_19 },
    { 0x33dd3b0a, SUC_lexicon_tags_23 },
    { 0xebfd3b0e, SUC_lexicon_tags_2 },
    { 0x8c1d3b0f, SUC_lexicon_tags_57 },
    { 0xddd3b0e, SUC_lexicon_tags_1 },
    { 0xe89d3b0f, SUC_lexicon_tags_48 },
    { 0xd0bd3b12, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x89dd3b15, SUC_lexicon_tags_1 },
    { 0x165d3b15, SUC_lexicon_tags_12 },
    { 0xcebd3b15, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5d3b1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303d3b25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebd3b2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x911d3b2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x305d3b30, SUC_lexicon_tags_32 },
    { 0x793d3b31, SUC_lexicon_tags_30 },
    { 0x16dd3b32, SUC_lexicon_tags_1 },
    { 0x389d3b31, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x46dd3b35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807d3b3a, SUC_lexicon_tags_11 },
    { 0x8e3d3b3b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33d3b3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1d3b42, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7d3b46, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5d3b49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9d3b50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fd3b53, SUC_lexicon_tags_45 },
    { 0x11fd3b54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7d3b58, SUC_lexicon_tags_610 },
    { 0, NULL },
    { 0x65bd3b5a, SUC_lexicon_tags_46 },
    { 0x269d3b5a, SUC_lexicon_tags_1 },
    { 0xe55d3b5a, SUC_lexicon_tags_12 },
    { 0x685d3b5d, SUC_lexicon_tags_15 },
    { 0xc5fd3b5c, SUC_lexicon_tags_0 },
    { 0xd8dd3b5a, SUC_lexicon_tags_10 },
    { 0x465d3b5a, SUC_lexicon_tags_17 },
    { 0x907d3b61, SUC_lexicon_tags_2 },
    { 0xadbd3b5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf7d3b64, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dd3b75, SUC_lexicon_tags_17 },
    { 0x61dd3b75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dd3b79, SUC_lexicon_tags_9 },
    { 0x2dd3b7a, SUC_lexicon_tags_95 },
    { 0x39dd3b79, SUC_lexicon_tags_27 },
    { 0x121d3b7c, SUC_lexicon_tags_37 },
    { 0x493d3b7d, SUC_lexicon_tags_19 },
    { 0xc51d3b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fd3b81, SUC_lexicon_tags_514 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dd3b85, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe93d3b87, SUC_lexicon_tags_152 },
    { 0x635d3b88, SUC_lexicon_tags_19 },
    { 0xc6dd3b89, SUC_lexicon_tags_7 },
    { 0xc6dd3b88, SUC_lexicon_tags_37 },
    { 0x3b5d3b8a, SUC_lexicon_tags_43 },
    { 0x2c7d3b8a, SUC_lexicon_tags_9 },
    { 0x563d3b8d, SUC_lexicon_tags_0 },
    { 0x461d3b8d, SUC_lexicon_tags_12 },
    { 0xa6dd3b8f, SUC_lexicon_tags_116 },
    { 0x339d3b8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x11fd3b92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817d3ba3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7d3ba9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa0fd3bab, SUC_lexicon_tags_9 },
    { 0x9c5d3bac, SUC_lexicon_tags_38 },
    { 0x22dd3bad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe79d3baf, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x277d3bb1, SUC_lexicon_tags_47 },
    { 0xea7d3bb2, SUC_lexicon_tags_2 },
    { 0xdefd3bb3, SUC_lexicon_tags_46 },
    { 0xf19d3bb3, SUC_lexicon_tags_12 },
    { 0x235d3bb5, SUC_lexicon_tags_19 },
    { 0xc0bd3bb3, SUC_lexicon_tags_21 },
    { 0x88dd3bb7, SUC_lexicon_tags_15 },
    { 0x34fd3bb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3d3bbb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7d3bbe, SUC_lexicon_tags_4 },
    { 0x423d3bbf, SUC_lexicon_tags_27 },
    { 0xdfdd3bc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dd3bc4, SUC_lexicon_tags_14 },
    { 0x661d3bc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3d3bcb, SUC_lexicon_tags_77 },
    { 0x317d3bcc, SUC_lexicon_tags_47 },
    { 0x289d3bcd, SUC_lexicon_tags_390 },
    { 0x683d3bce, SUC_lexicon_tags_37 },
    { 0xbb3d3bcf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9a3d3bd1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dd3bd5, SUC_lexicon_tags_37 },
    { 0xb89d3bd5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45dd3be1, SUC_lexicon_tags_8 },
    { 0x34fd3be1, SUC_lexicon_tags_49 },
    { 0xc9dd3be3, SUC_lexicon_tags_25 },
    { 0x485d3be4, SUC_lexicon_tags_31 },
    { 0x3fbd3be3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6a3d3be7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xafbd3be9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3d3bed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd3bf1, SUC_lexicon_tags_42 },
    { 0xb61d3bf2, SUC_lexicon_tags_43 },
    { 0xcd3d3bf2, SUC_lexicon_tags_2 },
    { 0x865d3bf4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf09d3bf6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd9d3bf8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fd3bfc, SUC_lexicon_tags_46 },
    { 0xb03d3bfd, SUC_lexicon_tags_25 },
    { 0x883d3bfe, SUC_lexicon_tags_7 },
    { 0xf1bd3bff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35d3c03, SUC_lexicon_tags_11 },
    { 0x191d3c04, SUC_lexicon_tags_37 },
    { 0x281d3c04, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577d3c09, SUC_lexicon_tags_25 },
    { 0x695d3c0a, SUC_lexicon_tags_234 },
    { 0x163d3c09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3d3c0e, SUC_lexicon_tags_11 },
    { 0x769d3c0f, SUC_lexicon_tags_7 },
    { 0xc57d3c0f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983d3c16, SUC_lexicon_tags_1 },
    { 0x149d3c17, SUC_lexicon_tags_47 },
    { 0x37d3c17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29bd3c1a, SUC_lexicon_tags_45 },
    { 0xe93d3c1a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79d3c20, SUC_lexicon_tags_11 },
    { 0x45fd3c21, SUC_lexicon_tags_9 },
    { 0x61d3c21, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285d3c27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xecfd3c29, SUC_lexicon_tags_45 },
    { 0xe0fd3c2a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5d3c30, SUC_lexicon_tags_62 },
    { 0x1ebd3c31, SUC_lexicon_tags_1 },
    { 0x443d3c32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1afd3c34, SUC_lexicon_tags_17 },
    { 0xecbd3c35, SUC_lexicon_tags_15 },
    { 0xc29d3c36, SUC_lexicon_tags_47 },
    { 0x3edd3c37, SUC_lexicon_tags_1 },
    { 0x2dfd3c38, SUC_lexicon_tags_17 },
    { 0xce9d3c39, SUC_lexicon_tags_17 },
    { 0xed7d3c37, SUC_lexicon_tags_15 },
    { 0xb69d3c3b, SUC_lexicon_tags_49 },
    { 0x7d1d3c39, SUC_lexicon_tags_25 },
    { 0x43bd3c3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedd3c43, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fd3c4f, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x395d3c52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3d3c55, SUC_lexicon_tags_12 },
    { 0xfcfd3c56, SUC_lexicon_tags_53 },
    { 0xbb7d3c57, SUC_lexicon_tags_15 },
    { 0x5cbd3c58, SUC_lexicon_tags_2 },
    { 0x5add3c58, SUC_lexicon_tags_26 },
    { 0x571d3c57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x411d3c5d, SUC_lexicon_tags_26 },
    { 0xac1d3c5e, SUC_lexicon_tags_2 },
    { 0xdc3d3c5e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4b3d3c61, SUC_lexicon_tags_12 },
    { 0x6b1d3c62, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fd3c67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13d3c6b, SUC_lexicon_tags_52 },
    { 0x82fd3c6c, SUC_lexicon_tags_21 },
    { 0x76fd3c6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5d3c72, SUC_lexicon_tags_29 },
    { 0x1e3d3c73, SUC_lexicon_tags_55 },
    { 0xbf1d3c73, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651d3c7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7d3c80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf43d3c82, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4dfd3c84, SUC_lexicon_tags_8 },
    { 0x9e1d3c85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x935d3c88, SUC_lexicon_tags_2 },
    { 0x27bd3c89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d3c8c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863d3c90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1d3c96, SUC_lexicon_tags_17 },
    { 0x43fd3c97, SUC_lexicon_tags_25 },
    { 0x555d3c97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fd3c9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25d3ca4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57d3ca7, SUC_lexicon_tags_2 },
    { 0x867d3ca8, SUC_lexicon_tags_15 },
    { 0x8f5d3ca9, SUC_lexicon_tags_15 },
    { 0xa4fd3caa, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x281d3cad, SUC_lexicon_tags_17 },
    { 0xa2bd3cad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3d3cb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60dd3cb4, SUC_lexicon_tags_2 },
    { 0x945d3cb5, SUC_lexicon_tags_65 },
    { 0x863d3cb6, SUC_lexicon_tags_66 },
    { 0x9add3cb7, SUC_lexicon_tags_25 },
    { 0x4bfd3cb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587d3cbe, SUC_lexicon_tags_44 },
    { 0xbd1d3cbe, SUC_lexicon_tags_2 },
    { 0xfcbd3cbf, SUC_lexicon_tags_9 },
    { 0xf1d3cc1, SUC_lexicon_tags_38 },
    { 0xba7d3cc2, SUC_lexicon_tags_23 },
    { 0xfd9d3cc1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469d3cc7, SUC_lexicon_tags_114 },
    { 0x691d3cc8, SUC_lexicon_tags_1 },
    { 0xc19d3cc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7d3ccd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367d3cd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1d3cd6, SUC_lexicon_tags_25 },
    { 0x917d3cd7, SUC_lexicon_tags_19 },
    { 0x169d3cd8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6add3cdc, SUC_lexicon_tags_25 },
    { 0xc9dd3cdd, SUC_lexicon_tags_38 },
    { 0x95dd3cde, SUC_lexicon_tags_63 },
    { 0x741d3cdf, SUC_lexicon_tags_23 },
    { 0xaadd3ce0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7d3ce4, SUC_lexicon_tags_7 },
    { 0x8a1d3ce5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bd3ce8, SUC_lexicon_tags_2 },
    { 0x841d3ce9, SUC_lexicon_tags_38 },
    { 0x6cfd3cea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3d3cee, SUC_lexicon_tags_17 },
    { 0x4a7d3cee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdd3cf2, SUC_lexicon_tags_63 },
    { 0x46fd3cf2, SUC_lexicon_tags_12 },
    { 0xba1d3cf4, SUC_lexicon_tags_0 },
    { 0x3b1d3cf4, SUC_lexicon_tags_12 },
    { 0xdd1d3cf4, SUC_lexicon_tags_7 },
    { 0x5a1d3cf7, SUC_lexicon_tags_23 },
    { 0x67bd3cf8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebd3cfe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x421d3d00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x587d3d03, SUC_lexicon_tags_27 },
    { 0x60dd3d04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddd3d08, SUC_lexicon_tags_47 },
    { 0x9abd3d09, SUC_lexicon_tags_70 },
    { 0x9e3d3d0a, SUC_lexicon_tags_12 },
    { 0x999d3d0b, SUC_lexicon_tags_64 },
    { 0x7a3d3d08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57d3d0f, SUC_lexicon_tags_82 },
    { 0x8f3d3d10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x749d3d12, SUC_lexicon_tags_15 },
    { 0xa11d3d13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x671d3d15, SUC_lexicon_tags_27 },
    { 0xa23d3d15, SUC_lexicon_tags_26 },
    { 0xa17d3d15, SUC_lexicon_tags_73 },
    { 0xe6bd3d18, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403d3d20, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d3d24, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x29fd3d27, SUC_lexicon_tags_12 },
    { 0xa81d3d28, SUC_lexicon_tags_12 },
    { 0x85d3d27, SUC_lexicon_tags_8 },
    { 0xf17d3d29, SUC_lexicon_tags_31 },
    { 0x6a5d3d27, SUC_lexicon_tags_26 },
    { 0x325d3d2c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe07d3d2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x91d3d31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bd3d34, SUC_lexicon_tags_12 },
    { 0xd6dd3d35, SUC_lexicon_tags_47 },
    { 0xbf3d3d36, SUC_lexicon_tags_9 },
    { 0xa55d3d37, SUC_lexicon_tags_2 },
    { 0x7abd3d38, SUC_lexicon_tags_11 },
    { 0x3dbd3d39, SUC_lexicon_tags_9 },
    { 0x527d3d37, SUC_lexicon_tags_1 },
    { 0xf0dd3d38, SUC_lexicon_tags_17 },
    { 0xca7d3d3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffd3d40, SUC_lexicon_tags_37 },
    { 0xc39d3d41, SUC_lexicon_tags_33 },
    { 0x46bd3d42, SUC_lexicon_tags_26 },
    { 0x681d3d42, SUC_lexicon_tags_19 },
    { 0x495d3d42, SUC_lexicon_tags_37 },
    { 0xb4bd3d42, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdddd3d47, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99d3d51, SUC_lexicon_tags_23 },
    { 0x9e3d3d51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x655d3d54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1d3d57, SUC_lexicon_tags_27 },
    { 0x25d3d58, SUC_lexicon_tags_17 },
    { 0x39bd3d58, SUC_lexicon_tags_17 },
    { 0xb73d3d5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x711d3d5c, SUC_lexicon_tags_1 },
    { 0xffdd3d5c, SUC_lexicon_tags_63 },
    { 0xea7d3d5e, SUC_lexicon_tags_7 },
    { 0x5b1d3d5f, SUC_lexicon_tags_6 },
    { 0xca7d3d5f, SUC_lexicon_tags_6 },
    { 0x1b3d3d5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x639d3d64, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x1a1d3d66, SUC_lexicon_tags_12 },
    { 0xd47d3d67, SUC_lexicon_tags_6 },
    { 0x32fd3d67, SUC_lexicon_tags_17 },
    { 0xa0bd3d69, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297d3d6d, SUC_lexicon_tags_1 },
    { 0xf73d3d6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afd3d74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdd3d77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x52dd3d79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x321d3d7c, SUC_lexicon_tags_107 },
    { 0x9b1d3d7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc3d3d7f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x949d3d82, SUC_lexicon_tags_33 },
    { 0x37fd3d83, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9d3d8a, SUC_lexicon_tags_8 },
    { 0xc43d3d8b, SUC_lexicon_tags_15 },
    { 0xd9dd3d8c, SUC_lexicon_tags_5 },
    { 0x6c5d3d8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdbd3d8f, SUC_lexicon_tags_33 },
    { 0x339d3d90, SUC_lexicon_tags_11 },
    { 0xcefd3d91, SUC_lexicon_tags_38 },
    { 0xea9d3d92, SUC_lexicon_tags_124 },
    { 0x643d3d91, SUC_lexicon_tags_37 },
    { 0x709d3d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9d3d97, SUC_lexicon_tags_11 },
    { 0xe1fd3d98, SUC_lexicon_tags_47 },
    { 0xbbd3d97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335d3da0, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x21dd3da2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x819d3da5, SUC_lexicon_tags_168 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7d3dab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x319d3dae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x919d3db1, SUC_lexicon_tags_1 },
    { 0x7f3d3db2, SUC_lexicon_tags_30 },
    { 0x277d3db1, SUC_lexicon_tags_14 },
    { 0x557d3db1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc2bd3db6, SUC_lexicon_tags_27 },
    { 0xc91d3db7, SUC_lexicon_tags_66 },
    { 0x5ddd3db6, SUC_lexicon_tags_80 },
    { 0x521d3db9, SUC_lexicon_tags_15 },
    { 0xd89d3dba, SUC_lexicon_tags_10 },
    { 0x737d3dbb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x293d3dbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fd3dc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x32dd3dc6, SUC_lexicon_tags_25 },
    { 0xefd3dc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93d3dca, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5d3dcd, SUC_lexicon_tags_11 },
    { 0x1e9d3dce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895d3dd3, SUC_lexicon_tags_35 },
    { 0xdabd3dd3, SUC_lexicon_tags_91 },
    { 0xd0bd3dd5, SUC_lexicon_tags_38 },
    { 0xebbd3dd6, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327d3ddb, SUC_lexicon_tags_25 },
    { 0x7b3d3ddc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9d3ddf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdd3de2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533d3de9, SUC_lexicon_tags_87 },
    { 0xdbd3dea, SUC_lexicon_tags_11 },
    { 0x391d3dea, SUC_lexicon_tags_17 },
    { 0x9b7d3dec, SUC_lexicon_tags_77 },
    { 0x1add3dea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0fd3def, SUC_lexicon_tags_13 },
    { 0xddfd3df0, SUC_lexicon_tags_1 },
    { 0xcdd3def, SUC_lexicon_tags_1 },
    { 0x6a3d3def, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17d3df5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x621d3df7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669d3e03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x749d3e06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfd3e09, SUC_lexicon_tags_11 },
    { 0xa79d3e09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x867d3e0d, SUC_lexicon_tags_37 },
    { 0x959d3e0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89d3e12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95d3e17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xffdd3e1a, SUC_lexicon_tags_38 },
    { 0x3fd3e1b, SUC_lexicon_tags_29 },
    { 0x519d3e1a, SUC_lexicon_tags_11 },
    { 0xd9d3e1d, SUC_lexicon_tags_0 },
    { 0xebbd3e1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91dd3e22, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5d3e29, SUC_lexicon_tags_2 },
    { 0x953d3e2a, SUC_lexicon_tags_84 },
    { 0x5efd3e2b, SUC_lexicon_tags_21 },
    { 0xc61d3e2c, SUC_lexicon_tags_16 },
    { 0x5f5d3e2d, SUC_lexicon_tags_7 },
    { 0xb1dd3e2e, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49dd3e33, SUC_lexicon_tags_25 },
    { 0xf93d3e34, SUC_lexicon_tags_43 },
    { 0x7d3d3e35, SUC_lexicon_tags_7 },
    { 0x26dd3e35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7bbd3e38, SUC_lexicon_tags_17 },
    { 0xd95d3e39, SUC_lexicon_tags_25 },
    { 0x131d3e39, SUC_lexicon_tags_7 },
    { 0x507d3e3b, SUC_lexicon_tags_11 },
    { 0xab7d3e3c, SUC_lexicon_tags_9 },
    { 0xe85d3e3d, SUC_lexicon_tags_69 },
    { 0x6c7d3e3e, SUC_lexicon_tags_68 },
    { 0x239d3e3f, SUC_lexicon_tags_37 },
    { 0x465d3e3f, SUC_lexicon_tags_56 },
    { 0x369d3e3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x84dd3e43, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x951d3e45, SUC_lexicon_tags_12 },
    { 0xb35d3e46, SUC_lexicon_tags_10 },
    { 0x5d9d3e45, SUC_lexicon_tags_65 },
    { 0xff7d3e48, SUC_lexicon_tags_11 },
    { 0x529d3e46, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebd3e4e, SUC_lexicon_tags_1 },
    { 0xe19d3e4e, SUC_lexicon_tags_37 },
    { 0x567d3e50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bd3e53, SUC_lexicon_tags_43 },
    { 0x8d7d3e54, SUC_lexicon_tags_14 },
    { 0xd15d3e55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dd3e5c, SUC_lexicon_tags_12 },
    { 0xb83d3e5d, SUC_lexicon_tags_46 },
    { 0xae1d3e5e, SUC_lexicon_tags_49 },
    { 0x927d3e5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x137d3e62, SUC_lexicon_tags_11 },
    { 0x57d3e63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5d3e67, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dd3e6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3fd3e6c, SUC_lexicon_tags_33 },
    { 0x893d3e6d, SUC_lexicon_tags_5 },
    { 0xd8dd3e6d, SUC_lexicon_tags_12 },
    { 0x61fd3e6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efd3e73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7d3e76, SUC_lexicon_tags_15 },
    { 0x58fd3e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63d3e7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dd3e83, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33d3e86, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33d3e8d, SUC_lexicon_tags_47 },
    { 0xaf9d3e8e, SUC_lexicon_tags_1 },
    { 0x27dd3e8e, SUC_lexicon_tags_37 },
    { 0xa81d3e90, SUC_lexicon_tags_15 },
    { 0xa43d3e91, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x495d3e93, SUC_lexicon_tags_37 },
    { 0x359d3e94, SUC_lexicon_tags_8 },
    { 0xf35d3e95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fd3e98, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc2dd3e9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x441d3e9c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb07d3e9e, SUC_lexicon_tags_4 },
    { 0xf47d3e9e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbd3ea1, SUC_lexicon_tags_37 },
    { 0x6a7d3ea2, SUC_lexicon_tags_43 },
    { 0x57d3ea2, SUC_lexicon_tags_11 },
    { 0x60bd3ea3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bd3ea8, SUC_lexicon_tags_1 },
    { 0x95dd3ea9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dd3eb6, SUC_lexicon_tags_10 },
    { 0xfafd3eb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe5dd3eb9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3d3ebd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfd3ec0, SUC_lexicon_tags_11 },
    { 0xd87d3ec1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815d3ec5, SUC_lexicon_tags_1 },
    { 0xc11d3ec5, SUC_lexicon_tags_12 },
    { 0x837d3ec5, SUC_lexicon_tags_25 },
    { 0x2dd3ec8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5d3ecb, SUC_lexicon_tags_1 },
    { 0x535d3ecc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39d3ed1, SUC_lexicon_tags_6 },
    { 0x141d3ed2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56dd3ed4, SUC_lexicon_tags_7 },
    { 0xf85d3ed5, SUC_lexicon_tags_12 },
    { 0xfc1d3ed6, SUC_lexicon_tags_26 },
    { 0x467d3ed5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1d3edb, SUC_lexicon_tags_8 },
    { 0xe0fd3edc, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3d3ee2, SUC_lexicon_tags_7 },
    { 0x8c1d3ee2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9d3eec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925d3ef0, SUC_lexicon_tags_25 },
    { 0x199d3ef1, SUC_lexicon_tags_7 },
    { 0xc0dd3ef2, SUC_lexicon_tags_25 },
    { 0x523d3ef3, SUC_lexicon_tags_31 },
    { 0x1add3ef3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fd3ef7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bd3efc, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9d3eff, SUC_lexicon_tags_19 },
    { 0x8f5d3eff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bd3f09, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3d3f0d, SUC_lexicon_tags_63 },
    { 0xa2bd3f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9d3f11, SUC_lexicon_tags_49 },
    { 0x613d3f11, SUC_lexicon_tags_15 },
    { 0x441d3f13, SUC_lexicon_tags_27 },
    { 0xec5d3f14, SUC_lexicon_tags_19 },
    { 0x127d3f13, SUC_lexicon_tags_38 },
    { 0xd05d3f14, SUC_lexicon_tags_224 },
    { 0xb1dd3f17, SUC_lexicon_tags_17 },
    { 0x7bdd3f18, SUC_lexicon_tags_17 },
    { 0x1f7d3f19, SUC_lexicon_tags_12 },
    { 0x305d3f1a, SUC_lexicon_tags_37 },
    { 0x7f3d3f1a, SUC_lexicon_tags_17 },
    { 0x6fbd3f1c, SUC_lexicon_tags_31 },
    { 0xebbd3f1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dd3f21, SUC_lexicon_tags_43 },
    { 0x2dbd3f22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11d3f25, SUC_lexicon_tags_11 },
    { 0xd03d3f26, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d9d3f28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833d3f2d, SUC_lexicon_tags_47 },
    { 0xa21d3f2d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6bfd3f30, SUC_lexicon_tags_37 },
    { 0xb65d3f30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35d3f34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1abd3f36, SUC_lexicon_tags_33 },
    { 0x92fd3f36, SUC_lexicon_tags_25 },
    { 0x52dd3f38, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddd3f3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dd3f43, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9d3f46, SUC_lexicon_tags_7 },
    { 0xb95d3f47, SUC_lexicon_tags_17 },
    { 0xb63d3f48, SUC_lexicon_tags_17 },
    { 0xf25d3f48, SUC_lexicon_tags_65 },
    { 0xb85d3f46, SUC_lexicon_tags_17 },
    { 0x5ebd3f47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55dd3f50, SUC_lexicon_tags_29 },
    { 0x653d3f51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc71d3f53, SUC_lexicon_tags_17 },
    { 0xdadd3f54, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x177d3f56, SUC_lexicon_tags_8 },
    { 0x25d3f57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd9fd3f59, SUC_lexicon_tags_390 },
    { 0, NULL },
    { 0x5b1d3f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bd3f60, SUC_lexicon_tags_38 },
    { 0xc4dd3f61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99d3f69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x171d3f6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d3f70, SUC_lexicon_tags_8 },
    { 0xb9d3f71, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95d3f77, SUC_lexicon_tags_17 },
    { 0x57d3f78, SUC_lexicon_tags_39 },
    { 0xd49d3f79, SUC_lexicon_tags_28 },
    { 0x2d1d3f79, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341d3f83, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe13d3f85, SUC_lexicon_tags_30 },
    { 0x863d3f85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafd3f8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9d3f91, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711d3f95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x381d3f98, SUC_lexicon_tags_7 },
    { 0x201d3f99, SUC_lexicon_tags_1 },
    { 0x8bd3f9a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xcffd3f9c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7d3fa0, SUC_lexicon_tags_17 },
    { 0xc0fd3fa1, SUC_lexicon_tags_37 },
    { 0xbfbd3fa2, SUC_lexicon_tags_30 },
    { 0xd41d3fa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1afd3fa5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747d3fa9, SUC_lexicon_tags_9 },
    { 0x5e9d3faa, SUC_lexicon_tags_7 },
    { 0x1f9d3fab, SUC_lexicon_tags_12 },
    { 0xe17d3fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bd3fb0, SUC_lexicon_tags_3 },
    { 0xa85d3fb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x253d3fb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfd3fb7, SUC_lexicon_tags_11 },
    { 0x13fd3fb8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfd3fc1, SUC_lexicon_tags_6 },
    { 0xae5d3fc1, SUC_lexicon_tags_0 },
    { 0x389d3fc2, SUC_lexicon_tags_14 },
    { 0x615d3fc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7d3fc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x981d3fcb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13d3fce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5d3fd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x455d3fd6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9d3fdb, SUC_lexicon_tags_10 },
    { 0x9f5d3fdc, SUC_lexicon_tags_10 },
    { 0x975d3fdc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fd3fe2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2f5d3fe4, SUC_lexicon_tags_12 },
    { 0xd5fd3fe5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abd3fe8, SUC_lexicon_tags_61 },
    { 0xd83d3fe8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa11d3feb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889d3ff3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bd3ff6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9d3ff9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbd3ffd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437d4001, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fd4005, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x571d4007, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x239d4009, SUC_lexicon_tags_15 },
    { 0xea3d400a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1dd400c, SUC_lexicon_tags_11 },
    { 0x369d400c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x159d400f, SUC_lexicon_tags_11 },
    { 0x399d4010, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9c3d4012, SUC_lexicon_tags_14 },
    { 0xcf3d4013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21d4016, SUC_lexicon_tags_12 },
    { 0xae3d4017, SUC_lexicon_tags_1 },
    { 0xb41d4016, SUC_lexicon_tags_29 },
    { 0xd29d4016, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bd401f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xefbd4021, SUC_lexicon_tags_25 },
    { 0xbadd4022, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fd402d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x317d4030, SUC_lexicon_tags_12 },
    { 0x3cbd4031, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebd4034, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dd4037, SUC_lexicon_tags_31 },
    { 0x39bd4038, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13d4044, SUC_lexicon_tags_14 },
    { 0xd2fd4044, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9d404a, SUC_lexicon_tags_37 },
    { 0xd4fd404b, SUC_lexicon_tags_65 },
    { 0xc1dd404c, SUC_lexicon_tags_47 },
    { 0xa3dd404d, SUC_lexicon_tags_17 },
    { 0x4b3d404c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53d4052, SUC_lexicon_tags_15 },
    { 0x303d4053, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebd4056, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfd405b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bd405e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753d4065, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf43d4067, SUC_lexicon_tags_12 },
    { 0x573d4067, SUC_lexicon_tags_56 },
    { 0x66bd4069, SUC_lexicon_tags_15 },
    { 0x78bd4067, SUC_lexicon_tags_7 },
    { 0x6afd406b, SUC_lexicon_tags_61 },
    { 0xef5d406c, SUC_lexicon_tags_37 },
    { 0x453d406d, SUC_lexicon_tags_29 },
    { 0x9f3d406a, SUC_lexicon_tags_15 },
    { 0x7b9d4068, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabd407a, SUC_lexicon_tags_35 },
    { 0xe2bd407b, SUC_lexicon_tags_37 },
    { 0x657d407c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1d407f, SUC_lexicon_tags_25 },
    { 0xbb9d407f, SUC_lexicon_tags_1 },
    { 0x6c5d4081, SUC_lexicon_tags_1 },
    { 0x3dbd4082, SUC_lexicon_tags_15 },
    { 0x40fd4082, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3d5d4085, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bd4088, SUC_lexicon_tags_1 },
    { 0x835d4088, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6efd408b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3f3d408d, SUC_lexicon_tags_1 },
    { 0xfefd408d, SUC_lexicon_tags_1 },
    { 0x41d408f, SUC_lexicon_tags_7 },
    { 0x117d408f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3d4093, SUC_lexicon_tags_9 },
    { 0x657d4094, SUC_lexicon_tags_30 },
    { 0x9a3d4095, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9b1d4097, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdd40a7, SUC_lexicon_tags_59 },
    { 0xa8bd40a8, SUC_lexicon_tags_1 },
    { 0xe1dd40a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171d40af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddd40b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7d40bb, SUC_lexicon_tags_49 },
    { 0x529d40bc, SUC_lexicon_tags_60 },
    { 0xec7d40bc, SUC_lexicon_tags_26 },
    { 0x7bbd40be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x813d40c0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e7d40c2, SUC_lexicon_tags_14 },
    { 0x52dd40c2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x92dd40c6, SUC_lexicon_tags_6 },
    { 0xb71d40c6, SUC_lexicon_tags_25 },
    { 0xf1d40c6, SUC_lexicon_tags_12 },
    { 0xf37d40c6, SUC_lexicon_tags_4 },
    { 0xee3d40ca, SUC_lexicon_tags_17 },
    { 0xcd1d40cb, SUC_lexicon_tags_15 },
    { 0x6c9d40cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37dd40ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef5d40d0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x97bd40d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x997d40d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x73bd40d6, SUC_lexicon_tags_11 },
    { 0xe1d40d7, SUC_lexicon_tags_12 },
    { 0x3d7d40d8, SUC_lexicon_tags_46 },
    { 0x4b3d40d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc09d40db, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5d40e1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x781d40e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fd40e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bd40ec, SUC_lexicon_tags_71 },
    { 0x295d40ed, SUC_lexicon_tags_17 },
    { 0x757d40ee, SUC_lexicon_tags_30 },
    { 0xe67d40ef, SUC_lexicon_tags_0 },
    { 0x3dd40ed, SUC_lexicon_tags_37 },
    { 0x575d40f1, SUC_lexicon_tags_2 },
    { 0x269d40ed, SUC_lexicon_tags_8 },
    { 0xb7dd40f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19d40f9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dd40fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x929d4100, SUC_lexicon_tags_28 },
    { 0x725d4100, SUC_lexicon_tags_17 },
    { 0xdb7d4102, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65d4110, SUC_lexicon_tags_33 },
    { 0xcd3d4110, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7d4117, SUC_lexicon_tags_37 },
    { 0xbfbd4118, SUC_lexicon_tags_60 },
    { 0xa9d4117, SUC_lexicon_tags_1 },
    { 0x20bd411a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x819d411c, SUC_lexicon_tags_54 },
    { 0xdb1d411d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabd4120, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x561d4123, SUC_lexicon_tags_11 },
    { 0xb95d4123, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7ffd4126, SUC_lexicon_tags_25 },
    { 0x9fd4127, SUC_lexicon_tags_0 },
    { 0x6e3d4127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x157d412a, SUC_lexicon_tags_23 },
    { 0x5dfd412a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fd412e, SUC_lexicon_tags_0 },
    { 0x905d412e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x89bd4131, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9d4135, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x833d4137, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d413a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1d413d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89d4141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x717d4143, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7bd4145, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69d4149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3d414d, SUC_lexicon_tags_1 },
    { 0x987d414e, SUC_lexicon_tags_19 },
    { 0x3c3d414d, SUC_lexicon_tags_31 },
    { 0x55dd4150, SUC_lexicon_tags_25 },
    { 0x3fdd414f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd45d4153, SUC_lexicon_tags_38 },
    { 0x4cbd4153, SUC_lexicon_tags_16 },
    { 0x8ddd4155, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205d4159, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x76bd415b, SUC_lexicon_tags_23 },
    { 0xf35d415c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x541d415f, SUC_lexicon_tags_351 },
    { 0x723d4160, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bd4165, SUC_lexicon_tags_53 },
    { 0x657d4166, SUC_lexicon_tags_26 },
    { 0xacdd4167, SUC_lexicon_tags_2 },
    { 0x85d4166, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7bd416a, SUC_lexicon_tags_17 },
    { 0xb6dd416b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffd4170, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89d4175, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9d417a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3a9d417c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441d4181, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d4183, SUC_lexicon_tags_11 },
    { 0xd57d4183, SUC_lexicon_tags_113 },
    { 0x86dd4183, SUC_lexicon_tags_27 },
    { 0xa09d4183, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6dbd4188, SUC_lexicon_tags_160 },
    { 0x6d9d4189, SUC_lexicon_tags_30 },
    { 0x4f3d4188, SUC_lexicon_tags_60 },
    { 0xcafd4188, SUC_lexicon_tags_23 },
    { 0xfc7d418c, SUC_lexicon_tags_12 },
    { 0x1f9d418d, SUC_lexicon_tags_11 },
    { 0x4a9d4188, SUC_lexicon_tags_7 },
    { 0x7ddd418b, SUC_lexicon_tags_1 },
    { 0xf1fd4190, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x80dd4192, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bd4195, SUC_lexicon_tags_28 },
    { 0x289d4196, SUC_lexicon_tags_8 },
    { 0x249d4196, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebd419a, SUC_lexicon_tags_10 },
    { 0x519d419a, SUC_lexicon_tags_60 },
    { 0xd63d419a, SUC_lexicon_tags_15 },
    { 0x675d419d, SUC_lexicon_tags_0 },
    { 0x2abd419e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fd41a2, SUC_lexicon_tags_13 },
    { 0xe3dd41a3, SUC_lexicon_tags_2 },
    { 0x3b3d41a4, SUC_lexicon_tags_27 },
    { 0x7c9d41a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bd41a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd41ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52dd41ae, SUC_lexicon_tags_38 },
    { 0x693d41af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4dd41b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xaabd41b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfd41bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55dd41be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1d41c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9d41c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60dd41ce, SUC_lexicon_tags_31 },
    { 0x2d9d41cf, SUC_lexicon_tags_14 },
    { 0x761d41d0, SUC_lexicon_tags_30 },
    { 0x807d41d1, SUC_lexicon_tags_29 },
    { 0x817d41d0, SUC_lexicon_tags_23 },
    { 0x4f1d41d2, SUC_lexicon_tags_12 },
    { 0x733d41d2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfd41dc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x62bd41de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3d41e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfd41eb, SUC_lexicon_tags_11 },
    { 0xfedd41ec, SUC_lexicon_tags_10 },
    { 0x1e5d41ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbadd41ef, SUC_lexicon_tags_17 },
    { 0xe29d41f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa39d41f2, SUC_lexicon_tags_12 },
    { 0xf2dd41f3, SUC_lexicon_tags_7 },
    { 0x5ffd41f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7d41f7, SUC_lexicon_tags_12 },
    { 0xff7d41f8, SUC_lexicon_tags_12 },
    { 0xcfdd41f8, SUC_lexicon_tags_68 },
    { 0x6c9d41fa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd7d41fc, SUC_lexicon_tags_64 },
    { 0xe6fd41fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59d4201, SUC_lexicon_tags_27 },
    { 0xc1dd4202, SUC_lexicon_tags_17 },
    { 0x371d4203, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3d4206, SUC_lexicon_tags_17 },
    { 0x857d4207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63d420c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5cdd420e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bd4217, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9d421a, SUC_lexicon_tags_25 },
    { 0x391d421b, SUC_lexicon_tags_26 },
    { 0xd6bd421a, SUC_lexicon_tags_92 },
    { 0xf7d421c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5d4223, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdabd4225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bd4229, SUC_lexicon_tags_26 },
    { 0x64fd422a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x381d422c, SUC_lexicon_tags_32 },
    { 0xc35d422c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479d4232, SUC_lexicon_tags_46 },
    { 0x725d4232, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7d4237, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27d423a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2fd423c, SUC_lexicon_tags_106 },
    { 0x937d423d, SUC_lexicon_tags_60 },
    { 0x1cbd423e, SUC_lexicon_tags_12 },
    { 0x3efd423f, SUC_lexicon_tags_25 },
    { 0x68bd4240, SUC_lexicon_tags_10 },
    { 0xa43d4241, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddd4249, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd424c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f7d424e, SUC_lexicon_tags_30 },
    { 0x687d424f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d4253, SUC_lexicon_tags_1 },
    { 0x4fbd4254, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bd4257, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffd425a, SUC_lexicon_tags_47 },
    { 0x9ffd425b, SUC_lexicon_tags_19 },
    { 0xfb7d425b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd4261, SUC_lexicon_tags_29 },
    { 0x3bd4262, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dd4267, SUC_lexicon_tags_47 },
    { 0x39dd4268, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7d426b, SUC_lexicon_tags_45 },
    { 0xc0dd426b, SUC_lexicon_tags_106 },
    { 0x2cbd426d, SUC_lexicon_tags_14 },
    { 0xe21d426e, SUC_lexicon_tags_0 },
    { 0x831d426f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51bd4271, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fdd4273, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x8f5d4275, SUC_lexicon_tags_25 },
    { 0x921d4276, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfbdd4278, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfb9d427a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81d4283, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebd4286, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbd428a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1f3d428c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9ebd428e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3d4291, SUC_lexicon_tags_1 },
    { 0x92dd4292, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8add4295, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739d4299, SUC_lexicon_tags_43 },
    { 0x905d429a, SUC_lexicon_tags_11 },
    { 0x437d429b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x991d429d, SUC_lexicon_tags_37 },
    { 0xcf7d429e, SUC_lexicon_tags_16 },
    { 0xb1bd429d, SUC_lexicon_tags_2 },
    { 0x2afd42a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39d42a2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6e3d42a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bd42aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdd42ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x833d42af, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815d42b3, SUC_lexicon_tags_7 },
    { 0x9b5d42b4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fd42b8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bd42bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381d42c7, SUC_lexicon_tags_43 },
    { 0x9dbd42c8, SUC_lexicon_tags_17 },
    { 0x981d42c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71d42cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf27d42ce, SUC_lexicon_tags_23 },
    { 0x4f1d42cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x631d42d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755d42d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dd42db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1d42df, SUC_lexicon_tags_60 },
    { 0x2dfd42e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddd42e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26dd42ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xccfd42ec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf73d42ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7d42f2, SUC_lexicon_tags_11 },
    { 0x1ddd42f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb5bd42f5, SUC_lexicon_tags_11 },
    { 0x5d42f6, SUC_lexicon_tags_39 },
    { 0x7bbd42f7, SUC_lexicon_tags_25 },
    { 0x2c7d42f8, SUC_lexicon_tags_17 },
    { 0x73fd42f9, SUC_lexicon_tags_47 },
    { 0xacdd42f9, SUC_lexicon_tags_63 },
    { 0xa1fd42fa, SUC_lexicon_tags_11 },
    { 0xa87d42f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709d4301, SUC_lexicon_tags_25 },
    { 0xe11d4302, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1d4305, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xc51d4307, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3d430a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1d430d, SUC_lexicon_tags_1 },
    { 0x2b9d430e, SUC_lexicon_tags_12 },
    { 0xc23d430f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x48bd4311, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8add4314, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f3d4316, SUC_lexicon_tags_11 },
    { 0x4cbd4316, SUC_lexicon_tags_68 },
    { 0x50bd4318, SUC_lexicon_tags_23 },
    { 0x97dd4318, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9d4320, SUC_lexicon_tags_11 },
    { 0x3dfd4321, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bd4324, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x22bd4327, SUC_lexicon_tags_17 },
    { 0xfa3d4327, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85d432c, SUC_lexicon_tags_15 },
    { 0xe69d432c, SUC_lexicon_tags_32 },
    { 0x2d7d432c, SUC_lexicon_tags_31 },
    { 0xd0dd432f, SUC_lexicon_tags_19 },
    { 0x7cdd432d, SUC_lexicon_tags_477 },
    { 0, NULL },
    { 0x3d7d4332, SUC_lexicon_tags_59 },
    { 0x899d4332, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x819d4336, SUC_lexicon_tags_25 },
    { 0x295d4337, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7d433e, SUC_lexicon_tags_12 },
    { 0x23fd433e, SUC_lexicon_tags_37 },
    { 0x5c5d433f, SUC_lexicon_tags_19 },
    { 0x7e9d4340, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d4346, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fd434b, SUC_lexicon_tags_30 },
    { 0xc45d434c, SUC_lexicon_tags_79 },
    { 0x6f7d434c, SUC_lexicon_tags_11 },
    { 0x6c5d434e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbabd4350, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9d4353, SUC_lexicon_tags_11 },
    { 0x419d4354, SUC_lexicon_tags_12 },
    { 0x19d4355, SUC_lexicon_tags_1 },
    { 0x903d4356, SUC_lexicon_tags_10 },
    { 0xa0bd4357, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547d4363, SUC_lexicon_tags_62 },
    { 0x34fd4364, SUC_lexicon_tags_28 },
    { 0xc7bd4365, SUC_lexicon_tags_17 },
    { 0xf05d4366, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x53dd4368, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb8bd436a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8add436e, SUC_lexicon_tags_15 },
    { 0x871d436f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x231d4372, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xde5d4374, SUC_lexicon_tags_36 },
    { 0x80bd4375, SUC_lexicon_tags_15 },
    { 0x15d4376, SUC_lexicon_tags_11 },
    { 0x43d4374, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1d4383, SUC_lexicon_tags_34 },
    { 0x17dd4384, SUC_lexicon_tags_2 },
    { 0xcedd4383, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1d4389, SUC_lexicon_tags_61 },
    { 0x80dd438a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xeffd438c, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbd4390, SUC_lexicon_tags_47 },
    { 0x689d4390, SUC_lexicon_tags_6 },
    { 0x8b7d4392, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf5dd4394, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735d439c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bd43a3, SUC_lexicon_tags_12 },
    { 0xf29d43a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x997d43a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ffd43a8, SUC_lexicon_tags_1 },
    { 0x457d43a9, SUC_lexicon_tags_33 },
    { 0xb37d43aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x679d43ac, SUC_lexicon_tags_28 },
    { 0xac7d43ac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bd43b0, SUC_lexicon_tags_87 },
    { 0x1a5d43b0, SUC_lexicon_tags_26 },
    { 0x54dd43b2, SUC_lexicon_tags_1 },
    { 0x611d43b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8ebd43b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdd43b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389d43bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d43c2, SUC_lexicon_tags_47 },
    { 0x5c7d43c2, SUC_lexicon_tags_66 },
    { 0xa15d43c4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531d43cb, SUC_lexicon_tags_82 },
    { 0xc9d43cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd43d2, SUC_lexicon_tags_2 },
    { 0x8b3d43d2, SUC_lexicon_tags_19 },
    { 0xccdd43d4, SUC_lexicon_tags_17 },
    { 0xa27d43d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe1dd43d7, SUC_lexicon_tags_28 },
    { 0xc85d43d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923d43df, SUC_lexicon_tags_12 },
    { 0x757d43e0, SUC_lexicon_tags_4 },
    { 0x6a7d43df, SUC_lexicon_tags_26 },
    { 0xb89d43e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5d43e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2c5d43e6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x241d43e9, SUC_lexicon_tags_77 },
    { 0x833d43ea, SUC_lexicon_tags_47 },
    { 0x1c7d43eb, SUC_lexicon_tags_11 },
    { 0xf5fd43ec, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcc3d43ee, SUC_lexicon_tags_1 },
    { 0x785d43ef, SUC_lexicon_tags_47 },
    { 0xd59d43ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fd43f4, SUC_lexicon_tags_0 },
    { 0xe1fd43f4, SUC_lexicon_tags_0 },
    { 0xa3d43f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31d43fd, SUC_lexicon_tags_1 },
    { 0xe85d43fd, SUC_lexicon_tags_11 },
    { 0x54fd43ff, SUC_lexicon_tags_11 },
    { 0x387d43ff, SUC_lexicon_tags_1 },
    { 0x491d4401, SUC_lexicon_tags_33 },
    { 0xb7bd4402, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bd4406, SUC_lexicon_tags_33 },
    { 0x899d4407, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb41d4409, SUC_lexicon_tags_33 },
    { 0x46bd440a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5d4411, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517d4418, SUC_lexicon_tags_8 },
    { 0xeffd4419, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x85dd441c, SUC_lexicon_tags_37 },
    { 0x2ffd441d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7d4422, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7d4426, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807d442b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc05d442d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcc3d442f, SUC_lexicon_tags_25 },
    { 0x309d4430, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119d4435, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21dd4438, SUC_lexicon_tags_39 },
    { 0xd27d4438, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7d443d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edd4442, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9d444c, SUC_lexicon_tags_29 },
    { 0x7bbd444d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bd4450, SUC_lexicon_tags_79 },
    { 0x937d4450, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfd9d4453, SUC_lexicon_tags_29 },
    { 0xedbd4453, SUC_lexicon_tags_0 },
    { 0x69dd4455, SUC_lexicon_tags_17 },
    { 0x319d4456, SUC_lexicon_tags_11 },
    { 0x481d4454, SUC_lexicon_tags_25 },
    { 0x3bd4458, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbd445c, SUC_lexicon_tags_65 },
    { 0xcb5d445d, SUC_lexicon_tags_37 },
    { 0x577d445d, SUC_lexicon_tags_11 },
    { 0x83dd445f, SUC_lexicon_tags_1 },
    { 0x77bd445e, SUC_lexicon_tags_9 },
    { 0x16fd4461, SUC_lexicon_tags_0 },
    { 0xaf7d445c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221d4466, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdb1d4468, SUC_lexicon_tags_26 },
    { 0xdcfd4469, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78fd446b, SUC_lexicon_tags_55 },
    { 0xe23d446c, SUC_lexicon_tags_27 },
    { 0xa6dd446c, SUC_lexicon_tags_25 },
    { 0xf4bd446d, SUC_lexicon_tags_37 },
    { 0x2fbd446c, SUC_lexicon_tags_28 },
    { 0xeb5d4470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffd4474, SUC_lexicon_tags_37 },
    { 0x985d4475, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbd4479, SUC_lexicon_tags_19 },
    { 0x217d447a, SUC_lexicon_tags_8 },
    { 0xf7fd447b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdbbd447d, SUC_lexicon_tags_10 },
    { 0x46dd447e, SUC_lexicon_tags_48 },
    { 0xbc3d447f, SUC_lexicon_tags_1 },
    { 0x9dbd447f, SUC_lexicon_tags_1 },
    { 0x329d4481, SUC_lexicon_tags_57 },
    { 0x29bd4482, SUC_lexicon_tags_38 },
    { 0xa8dd4480, SUC_lexicon_tags_37 },
    { 0xa57d4484, SUC_lexicon_tags_2 },
    { 0x4b5d4485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79d4488, SUC_lexicon_tags_46 },
    { 0xc8bd4489, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57d448f, SUC_lexicon_tags_17 },
    { 0xb15d448f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb7dd4492, SUC_lexicon_tags_218 },
    { 0x355d4492, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3d4496, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x225d4498, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x141d449a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dd449f, SUC_lexicon_tags_15 },
    { 0x287d44a0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf51d44a2, SUC_lexicon_tags_27 },
    { 0xf7bd44a3, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779d44a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad9d44aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x58fd44ac, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bd44b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3d44b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7dd44b4, SUC_lexicon_tags_12 },
    { 0x81dd44b4, SUC_lexicon_tags_25 },
    { 0xac7d44b4, SUC_lexicon_tags_9 },
    { 0x1a7d44b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1d44bb, SUC_lexicon_tags_15 },
    { 0x651d44bc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdc9d44be, SUC_lexicon_tags_37 },
    { 0xbcfd44bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa67d44c1, SUC_lexicon_tags_55 },
    { 0xab7d44c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493d44c6, SUC_lexicon_tags_42 },
    { 0x153d44c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3d44d0, SUC_lexicon_tags_77 },
    { 0xa43d44d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3d44d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995d44d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fd44dd, SUC_lexicon_tags_1 },
    { 0x72dd44de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x567d44e0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03d44ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1d44f3, SUC_lexicon_tags_27 },
    { 0x9cbd44f3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13d44fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bd4500, SUC_lexicon_tags_473 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09d4503, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe41d4505, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9e1d4507, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xed9d4509, SUC_lexicon_tags_3 },
    { 0x8dfd4509, SUC_lexicon_tags_1 },
    { 0x84dd450b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xeafd450d, SUC_lexicon_tags_8 },
    { 0xaf7d450e, SUC_lexicon_tags_57 },
    { 0xa97d450f, SUC_lexicon_tags_105 },
    { 0x47d450f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3d4513, SUC_lexicon_tags_60 },
    { 0x1dd4514, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbd451c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fd4523, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbd4527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa7d4529, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfd4530, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9d4535, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x60bd4537, SUC_lexicon_tags_16 },
    { 0x8bdd4538, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d453b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x693d453e, SUC_lexicon_tags_65 },
    { 0xb93d453f, SUC_lexicon_tags_17 },
    { 0x2b5d453f, SUC_lexicon_tags_33 },
    { 0x133d453f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfd4547, SUC_lexicon_tags_31 },
    { 0xd2fd4548, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697d454f, SUC_lexicon_tags_11 },
    { 0x77fd4550, SUC_lexicon_tags_66 },
    { 0x365d454f, SUC_lexicon_tags_29 },
    { 0x471d4552, SUC_lexicon_tags_2 },
    { 0xa19d4552, SUC_lexicon_tags_25 },
    { 0xc3d4554, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1d4557, SUC_lexicon_tags_37 },
    { 0xfd4557, SUC_lexicon_tags_46 },
    { 0xaa9d4559, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x931d455c, SUC_lexicon_tags_45 },
    { 0xc3d455d, SUC_lexicon_tags_60 },
    { 0x82dd455e, SUC_lexicon_tags_26 },
    { 0xc71d455f, SUC_lexicon_tags_1 },
    { 0x32fd455c, SUC_lexicon_tags_1 },
    { 0xa03d455d, SUC_lexicon_tags_60 },
    { 0x21bd4561, SUC_lexicon_tags_28 },
    { 0x4f9d4562, SUC_lexicon_tags_68 },
    { 0x2ffd4561, SUC_lexicon_tags_45 },
    { 0x4e5d4565, SUC_lexicon_tags_17 },
    { 0xb31d4565, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325d456f, SUC_lexicon_tags_9 },
    { 0xcd1d4570, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d4573, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211d457c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdd457f, SUC_lexicon_tags_1 },
    { 0xa7dd4580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfd4587, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x39dd4589, SUC_lexicon_tags_2 },
    { 0x7b1d458a, SUC_lexicon_tags_16 },
    { 0x51bd4589, SUC_lexicon_tags_25 },
    { 0xd93d458c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf07d458e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x401d4590, SUC_lexicon_tags_11 },
    { 0xb85d4591, SUC_lexicon_tags_11 },
    { 0x353d4592, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7d4595, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9cdd4597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68dd4599, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dd45a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc59d45a2, SUC_lexicon_tags_47 },
    { 0x2bd45a3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d45a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89dd45a9, SUC_lexicon_tags_11 },
    { 0x66bd45a9, SUC_lexicon_tags_54 },
    { 0x905d45ab, SUC_lexicon_tags_11 },
    { 0x931d45ab, SUC_lexicon_tags_15 },
    { 0xe71d45ac, SUC_lexicon_tags_38 },
    { 0x27d45ae, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39d45b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5d45b5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf8bd45b7, SUC_lexicon_tags_12 },
    { 0x28bd45b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdd45bc, SUC_lexicon_tags_25 },
    { 0xf13d45bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x683d45c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafd45c4, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x8a5d45c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x329d45c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x807d45cb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fd45d1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa09d45d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf5d45d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97d45d8, SUC_lexicon_tags_11 },
    { 0x4fd45d9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83d45de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d1d45e0, SUC_lexicon_tags_51 },
    { 0xcb1d45e0, SUC_lexicon_tags_20 },
    { 0x9a3d45e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175d45e6, SUC_lexicon_tags_11 },
    { 0xe6fd45e7, SUC_lexicon_tags_25 },
    { 0x8bd45e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1d45eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d45ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc9d45f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60dd45f5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5a5d45f7, SUC_lexicon_tags_11 },
    { 0x987d45f8, SUC_lexicon_tags_43 },
    { 0x201d45f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd4fd45fb, SUC_lexicon_tags_29 },
    { 0x78dd45fb, SUC_lexicon_tags_11 },
    { 0x41fd45fc, SUC_lexicon_tags_4 },
    { 0x957d45fe, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fd4602, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dd4608, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfd460e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55d4612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7d4616, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763d461b, SUC_lexicon_tags_35 },
    { 0xc8fd461b, SUC_lexicon_tags_15 },
    { 0xac1d461b, SUC_lexicon_tags_10 },
    { 0xcddd461e, SUC_lexicon_tags_49 },
    { 0xe4dd461d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1d4622, SUC_lexicon_tags_57 },
    { 0x7a1d4623, SUC_lexicon_tags_16 },
    { 0xf31d4624, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7d7d4626, SUC_lexicon_tags_7 },
    { 0xb8dd4626, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x975d4629, SUC_lexicon_tags_10 },
    { 0x5b7d462a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d4630, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dd4635, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d463c, SUC_lexicon_tags_17 },
    { 0x1c3d463c, SUC_lexicon_tags_37 },
    { 0x639d463d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bd4644, SUC_lexicon_tags_29 },
    { 0x187d4645, SUC_lexicon_tags_30 },
    { 0xacdd4645, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41d4649, SUC_lexicon_tags_55 },
    { 0xf2bd4649, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x519d464c, SUC_lexicon_tags_7 },
    { 0x379d464c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x27fd464f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5abd4651, SUC_lexicon_tags_37 },
    { 0x99dd4651, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abd4655, SUC_lexicon_tags_11 },
    { 0x2bdd4655, SUC_lexicon_tags_17 },
    { 0x619d4657, SUC_lexicon_tags_27 },
    { 0xc9d4657, SUC_lexicon_tags_25 },
    { 0x7bd4659, SUC_lexicon_tags_12 },
    { 0xc8dd465a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe3d465c, SUC_lexicon_tags_17 },
    { 0x78bd465c, SUC_lexicon_tags_7 },
    { 0x473d465c, SUC_lexicon_tags_12 },
    { 0x2b3d465f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9bd4661, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x691d4663, SUC_lexicon_tags_57 },
    { 0xce5d4663, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x673d4667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dd466d, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17d4670, SUC_lexicon_tags_34 },
    { 0x69fd4671, SUC_lexicon_tags_25 },
    { 0xa17d4670, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261d4676, SUC_lexicon_tags_45 },
    { 0x24bd4677, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99d467e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x587d4680, SUC_lexicon_tags_2 },
    { 0xe55d4681, SUC_lexicon_tags_38 },
    { 0x85dd4682, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x63fd4684, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5d4688, SUC_lexicon_tags_11 },
    { 0xa69d4689, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d468e, SUC_lexicon_tags_25 },
    { 0xa51d468f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27d4692, SUC_lexicon_tags_37 },
    { 0x6b3d4692, SUC_lexicon_tags_15 },
    { 0xc8dd4692, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5d4699, SUC_lexicon_tags_7 },
    { 0x329d469a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdd469e, SUC_lexicon_tags_168 },
    { 0xea1d469e, SUC_lexicon_tags_23 },
    { 0xc3fd46a0, SUC_lexicon_tags_29 },
    { 0x40dd46a1, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fd46a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabd46a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd6bd46ab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9fdd46ad, SUC_lexicon_tags_0 },
    { 0x5d3d46ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dd46b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x429d46b6, SUC_lexicon_tags_37 },
    { 0x95fd46b6, SUC_lexicon_tags_57 },
    { 0xae7d46b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7d9d46ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9d46be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf17d46c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1d46c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5d46cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5d46d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xba5d46d2, SUC_lexicon_tags_15 },
    { 0xdb5d46d2, SUC_lexicon_tags_37 },
    { 0xfedd46d4, SUC_lexicon_tags_7 },
    { 0xec3d46d5, SUC_lexicon_tags_14 },
    { 0x63bd46d4, SUC_lexicon_tags_33 },
    { 0x267d46d7, SUC_lexicon_tags_43 },
    { 0x951d46d3, SUC_lexicon_tags_15 },
    { 0x747d46d6, SUC_lexicon_tags_11 },
    { 0xf51d46d2, SUC_lexicon_tags_30 },
    { 0x721d46db, SUC_lexicon_tags_34 },
    { 0x337d46dc, SUC_lexicon_tags_12 },
    { 0x5b3d46dd, SUC_lexicon_tags_37 },
    { 0x909d46d3, SUC_lexicon_tags_15 },
    { 0xa57d46df, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209d46e3, SUC_lexicon_tags_7 },
    { 0x8efd46e4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8a5d46e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e7d46e8, SUC_lexicon_tags_45 },
    { 0x7bfd46e8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x347d46ec, SUC_lexicon_tags_25 },
    { 0xb09d46ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dd46f0, SUC_lexicon_tags_43 },
    { 0x9f9d46f0, SUC_lexicon_tags_27 },
    { 0x3abd46f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdd46fd, SUC_lexicon_tags_10 },
    { 0x777d46fe, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dd4701, SUC_lexicon_tags_19 },
    { 0xbc7d4701, SUC_lexicon_tags_6 },
    { 0xe5fd4701, SUC_lexicon_tags_9 },
    { 0x9d4704, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd1d4706, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3d470a, SUC_lexicon_tags_19 },
    { 0x5c9d470b, SUC_lexicon_tags_17 },
    { 0x389d470a, SUC_lexicon_tags_17 },
    { 0x9afd470d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11d4714, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55d471d, SUC_lexicon_tags_11 },
    { 0x523d471e, SUC_lexicon_tags_12 },
    { 0xe25d471f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebd4722, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9d4725, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15d4728, SUC_lexicon_tags_50 },
    { 0x9add4729, SUC_lexicon_tags_16 },
    { 0x439d4729, SUC_lexicon_tags_14 },
    { 0x9f3d472b, SUC_lexicon_tags_8 },
    { 0x4bdd472a, SUC_lexicon_tags_9 },
    { 0x539d472d, SUC_lexicon_tags_37 },
    { 0xc45d4729, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7ffd4730, SUC_lexicon_tags_15 },
    { 0xdbdd4731, SUC_lexicon_tags_8 },
    { 0xc51d4730, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9d4735, SUC_lexicon_tags_0 },
    { 0x511d4736, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5e7d4738, SUC_lexicon_tags_37 },
    { 0xc9dd4739, SUC_lexicon_tags_37 },
    { 0x849d473a, SUC_lexicon_tags_55 },
    { 0xa05d473b, SUC_lexicon_tags_37 },
    { 0xcb1d473c, SUC_lexicon_tags_47 },
    { 0xbffd473d, SUC_lexicon_tags_7 },
    { 0xe1d473c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x457d4741, SUC_lexicon_tags_2 },
    { 0xb7dd4741, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdd4746, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x333d4749, SUC_lexicon_tags_25 },
    { 0x1e9d4749, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8b1d474c, SUC_lexicon_tags_37 },
    { 0xe9bd474d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x883d474f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9d4752, SUC_lexicon_tags_1 },
    { 0x671d4752, SUC_lexicon_tags_17 },
    { 0x4e5d4752, SUC_lexicon_tags_17 },
    { 0x84dd4755, SUC_lexicon_tags_47 },
    { 0xce3d4754, SUC_lexicon_tags_23 },
    { 0x875d4754, SUC_lexicon_tags_2 },
    { 0xd45d4755, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc69d475a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165d475f, SUC_lexicon_tags_45 },
    { 0x8bdd475f, SUC_lexicon_tags_12 },
    { 0x6efd475f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3d4766, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463d476d, SUC_lexicon_tags_27 },
    { 0x5a7d476d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdefd4770, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bd4776, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x799d4778, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1d477b, SUC_lexicon_tags_17 },
    { 0x51fd477b, SUC_lexicon_tags_2 },
    { 0xd77d477c, SUC_lexicon_tags_2 },
    { 0x8ebd477d, SUC_lexicon_tags_25 },
    { 0x333d477b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297d4788, SUC_lexicon_tags_29 },
    { 0x39bd4788, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fd478f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebd4795, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x809d4798, SUC_lexicon_tags_9 },
    { 0x751d4798, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba7d479b, SUC_lexicon_tags_2 },
    { 0x591d479b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecfd479e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf7bd47a0, SUC_lexicon_tags_27 },
    { 0x333d47a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5d47a3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x57fd47a6, SUC_lexicon_tags_19 },
    { 0xce9d47a6, SUC_lexicon_tags_19 },
    { 0x31fd47a6, SUC_lexicon_tags_25 },
    { 0x31dd47a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191d47ad, SUC_lexicon_tags_37 },
    { 0x79fd47ae, SUC_lexicon_tags_5 },
    { 0x7a1d47af, SUC_lexicon_tags_25 },
    { 0x561d47ae, SUC_lexicon_tags_14 },
    { 0x6a1d47b0, SUC_lexicon_tags_11 },
    { 0x27fd47ad, SUC_lexicon_tags_47 },
    { 0x2abd47b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d47b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61d47bb, SUC_lexicon_tags_15 },
    { 0x655d47bb, SUC_lexicon_tags_12 },
    { 0xa6bd47bb, SUC_lexicon_tags_11 },
    { 0x9c7d47bc, SUC_lexicon_tags_37 },
    { 0x4f9d47bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fd47ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71dd47cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fd47cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bd47d8, SUC_lexicon_tags_49 },
    { 0x243d47d9, SUC_lexicon_tags_6 },
    { 0xebbd47da, SUC_lexicon_tags_11 },
    { 0x109d47db, SUC_lexicon_tags_44 },
    { 0xad7d47da, SUC_lexicon_tags_19 },
    { 0x6ffd47dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x57fd47df, SUC_lexicon_tags_2 },
    { 0xfcfd47e0, SUC_lexicon_tags_25 },
    { 0x133d47e1, SUC_lexicon_tags_15 },
    { 0x299d47e1, SUC_lexicon_tags_6 },
    { 0xefd47e3, SUC_lexicon_tags_12 },
    { 0xa8dd47e4, SUC_lexicon_tags_0 },
    { 0xb7dd47e5, SUC_lexicon_tags_12 },
    { 0x923d47e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dd47e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x101d47eb, SUC_lexicon_tags_1 },
    { 0xb49d47ec, SUC_lexicon_tags_39 },
    { 0xa51d47ed, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529d47f1, SUC_lexicon_tags_11 },
    { 0xe2dd47f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fd47fa, SUC_lexicon_tags_82 },
    { 0xb63d47fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05d47fe, SUC_lexicon_tags_17 },
    { 0x979d47ff, SUC_lexicon_tags_1 },
    { 0xe7fd47ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1add4802, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fd4806, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe63d4808, SUC_lexicon_tags_8 },
    { 0x6e3d4809, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x297d480b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249d480f, SUC_lexicon_tags_37 },
    { 0xb4bd4810, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb15d4812, SUC_lexicon_tags_200 },
    { 0, NULL },
    { 0x919d4814, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x59dd4817, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x823d4819, SUC_lexicon_tags_2 },
    { 0x24dd4819, SUC_lexicon_tags_45 },
    { 0x715d481a, SUC_lexicon_tags_9 },
    { 0x595d4819, SUC_lexicon_tags_60 },
    { 0xe69d481c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8e3d481f, SUC_lexicon_tags_2 },
    { 0xc97d4820, SUC_lexicon_tags_31 },
    { 0x8d5d4821, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf0dd4823, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1d4826, SUC_lexicon_tags_47 },
    { 0x247d4826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fd482a, SUC_lexicon_tags_2 },
    { 0xa51d482b, SUC_lexicon_tags_29 },
    { 0x111d482a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x30dd482e, SUC_lexicon_tags_7 },
    { 0xa6fd482f, SUC_lexicon_tags_3 },
    { 0xea1d4830, SUC_lexicon_tags_91 },
    { 0x2c3d4831, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa89d4833, SUC_lexicon_tags_23 },
    { 0x49dd4834, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89d4839, SUC_lexicon_tags_60 },
    { 0x6a5d483a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x42bd483c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd4844, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffd484a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x977d484c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9fd484e, SUC_lexicon_tags_32 },
    { 0x7fdd484f, SUC_lexicon_tags_15 },
    { 0xc85d484f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x465d4852, SUC_lexicon_tags_15 },
    { 0x393d4853, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7d4858, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bd485b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x457d485e, SUC_lexicon_tags_19 },
    { 0xfddd485f, SUC_lexicon_tags_0 },
    { 0x153d4860, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c9d4862, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5d4866, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x475d4868, SUC_lexicon_tags_37 },
    { 0xd91d4869, SUC_lexicon_tags_37 },
    { 0x775d4869, SUC_lexicon_tags_2 },
    { 0xc83d486b, SUC_lexicon_tags_39 },
    { 0xb11d486c, SUC_lexicon_tags_15 },
    { 0x8c1d486d, SUC_lexicon_tags_1 },
    { 0x249d486d, SUC_lexicon_tags_14 },
    { 0x553d4869, SUC_lexicon_tags_65 },
    { 0x627d4870, SUC_lexicon_tags_15 },
    { 0xcadd4871, SUC_lexicon_tags_55 },
    { 0x741d486c, SUC_lexicon_tags_7 },
    { 0x49bd4868, SUC_lexicon_tags_664 },
    { 0x35fd4874, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9d4878, SUC_lexicon_tags_37 },
    { 0x21bd4879, SUC_lexicon_tags_37 },
    { 0x733d487a, SUC_lexicon_tags_37 },
    { 0x439d487b, SUC_lexicon_tags_14 },
    { 0xde7d487c, SUC_lexicon_tags_11 },
    { 0x547d487d, SUC_lexicon_tags_47 },
    { 0x789d487e, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481d4882, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9e3d4884, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3d7d4886, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x991d4888, SUC_lexicon_tags_17 },
    { 0xfb9d4889, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efd4891, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49bd4893, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12fd4898, SUC_lexicon_tags_0 },
    { 0xe4fd4899, SUC_lexicon_tags_37 },
    { 0x1bbd4899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x993d489c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575d48a3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fd48aa, SUC_lexicon_tags_11 },
    { 0x33d48ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfd48af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26fd48b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd9bd48b3, SUC_lexicon_tags_7 },
    { 0x385d48b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x589d48b6, SUC_lexicon_tags_4 },
    { 0xbb1d48b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf15d48b9, SUC_lexicon_tags_6 },
    { 0xbe5d48b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fd48c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaf5d48c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d48d3, SUC_lexicon_tags_23 },
    { 0x1d48d4, SUC_lexicon_tags_37 },
    { 0xd51d48d5, SUC_lexicon_tags_66 },
    { 0x68fd48d6, SUC_lexicon_tags_15 },
    { 0x5bbd48d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75d48db, SUC_lexicon_tags_47 },
    { 0x8fbd48db, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8c3d48de, SUC_lexicon_tags_7 },
    { 0xdc9d48df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x639d48e1, SUC_lexicon_tags_25 },
    { 0x97bd48e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbd48e8, SUC_lexicon_tags_12 },
    { 0x2bd48e8, SUC_lexicon_tags_26 },
    { 0x667d48ea, SUC_lexicon_tags_60 },
    { 0xba9d48e8, SUC_lexicon_tags_23 },
    { 0x567d48eb, SUC_lexicon_tags_17 },
    { 0x631d48ed, SUC_lexicon_tags_37 },
    { 0x18dd48ee, SUC_lexicon_tags_26 },
    { 0x6e5d48ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8add48f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xce7d48f3, SUC_lexicon_tags_28 },
    { 0xd6fd48f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435d48f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5bbd48fa, SUC_lexicon_tags_33 },
    { 0x3bd48fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273d4902, SUC_lexicon_tags_2 },
    { 0xa6fd4902, SUC_lexicon_tags_63 },
    { 0xcafd4904, SUC_lexicon_tags_37 },
    { 0xdd3d4904, SUC_lexicon_tags_37 },
    { 0x4bdd4906, SUC_lexicon_tags_131 },
    { 0x66dd4906, SUC_lexicon_tags_33 },
    { 0x1efd4908, SUC_lexicon_tags_1 },
    { 0x1f3d4903, SUC_lexicon_tags_65 },
    { 0x45dd490a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fd4911, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x253d4913, SUC_lexicon_tags_1 },
    { 0x237d4914, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x81dd4916, SUC_lexicon_tags_31 },
    { 0x125d4916, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bd491a, SUC_lexicon_tags_25 },
    { 0xe65d491b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923d491f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1d4923, SUC_lexicon_tags_11 },
    { 0xbafd4923, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xf45d4926, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fd492b, SUC_lexicon_tags_47 },
    { 0xc95d492b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddd4930, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xac7d4932, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a5d4934, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddd4937, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75d493c, SUC_lexicon_tags_129 },
    { 0xa6bd493d, SUC_lexicon_tags_23 },
    { 0x19bd493d, SUC_lexicon_tags_8 },
    { 0xbddd493e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3d4942, SUC_lexicon_tags_1 },
    { 0xd8dd4942, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1d4946, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5d494f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2b7d4951, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69d495d, SUC_lexicon_tags_7 },
    { 0x973d495d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fd4961, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fd4967, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3b3d4969, SUC_lexicon_tags_26 },
    { 0xab5d496a, SUC_lexicon_tags_49 },
    { 0x347d496b, SUC_lexicon_tags_2 },
    { 0x71dd496b, SUC_lexicon_tags_29 },
    { 0xd27d496c, SUC_lexicon_tags_29 },
    { 0xa3fd496c, SUC_lexicon_tags_66 },
    { 0x967d496f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x151d4972, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9d4974, SUC_lexicon_tags_33 },
    { 0xd5d4974, SUC_lexicon_tags_11 },
    { 0xfafd4975, SUC_lexicon_tags_2 },
    { 0x141d4977, SUC_lexicon_tags_7 },
    { 0x4a3d4978, SUC_lexicon_tags_31 },
    { 0xaabd4979, SUC_lexicon_tags_34 },
    { 0x5add4979, SUC_lexicon_tags_47 },
    { 0xa9d497a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61d4981, SUC_lexicon_tags_14 },
    { 0x3add4982, SUC_lexicon_tags_37 },
    { 0x9abd4983, SUC_lexicon_tags_62 },
    { 0x927d4984, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7e3d4986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadd4992, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fd4997, SUC_lexicon_tags_12 },
    { 0x7ffd4998, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86bd499b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bd49a0, SUC_lexicon_tags_6 },
    { 0xf2fd49a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9d49a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4b5d49a6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1d49ae, SUC_lexicon_tags_15 },
    { 0xb9dd49ae, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3d49b6, SUC_lexicon_tags_12 },
    { 0x4e1d49b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe65d49b9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x193d49bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3d49c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67d49c3, SUC_lexicon_tags_7 },
    { 0x3efd49c3, SUC_lexicon_tags_21 },
    { 0x491d49c4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x993d49c7, SUC_lexicon_tags_14 },
    { 0xc69d49c8, SUC_lexicon_tags_1 },
    { 0x123d49c9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xae1d49cb, SUC_lexicon_tags_37 },
    { 0x75bd49cb, SUC_lexicon_tags_33 },
    { 0x751d49cd, SUC_lexicon_tags_151 },
    { 0x2a5d49ce, SUC_lexicon_tags_25 },
    { 0xb4dd49cb, SUC_lexicon_tags_1 },
    { 0xe1bd49d0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd49d5, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7d49d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05d49db, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8bdd49dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc4fd49df, SUC_lexicon_tags_23 },
    { 0xc83d49df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x771d49e3, SUC_lexicon_tags_33 },
    { 0x3d5d49e4, SUC_lexicon_tags_31 },
    { 0xcfdd49e5, SUC_lexicon_tags_151 },
    { 0xf85d49e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dd49f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaa9d49f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbd49f5, SUC_lexicon_tags_17 },
    { 0x4a7d49f6, SUC_lexicon_tags_19 },
    { 0x573d49f5, SUC_lexicon_tags_25 },
    { 0x4c1d49f6, SUC_lexicon_tags_25 },
    { 0x133d49f9, SUC_lexicon_tags_2 },
    { 0xcd3d49f7, SUC_lexicon_tags_14 },
    { 0xc69d49f6, SUC_lexicon_tags_1 },
    { 0xfd7d49fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabd4a00, SUC_lexicon_tags_7 },
    { 0xcadd4a01, SUC_lexicon_tags_1 },
    { 0xf25d4a02, SUC_lexicon_tags_11 },
    { 0x7f9d4a02, SUC_lexicon_tags_11 },
    { 0x31bd4a04, SUC_lexicon_tags_11 },
    { 0x221d4a05, SUC_lexicon_tags_23 },
    { 0x143d4a06, SUC_lexicon_tags_2 },
    { 0x29d4a04, SUC_lexicon_tags_17 },
    { 0x9cdd4a08, SUC_lexicon_tags_25 },
    { 0x307d4a02, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fd4a0d, SUC_lexicon_tags_60 },
    { 0x695d4a0e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x203d4a10, SUC_lexicon_tags_63 },
    { 0x67d4a11, SUC_lexicon_tags_23 },
    { 0xd57d4a12, SUC_lexicon_tags_103 },
    { 0xb47d4a13, SUC_lexicon_tags_2 },
    { 0x6e7d4a14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x80bd4a16, SUC_lexicon_tags_25 },
    { 0xefd4a17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bd4a1b, SUC_lexicon_tags_82 },
    { 0x1b7d4a1b, SUC_lexicon_tags_37 },
    { 0x3dbd4a1d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x811d4a1f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1d4a2a, SUC_lexicon_tags_27 },
    { 0xa09d4a2b, SUC_lexicon_tags_27 },
    { 0xba1d4a2c, SUC_lexicon_tags_25 },
    { 0x2e7d4a2d, SUC_lexicon_tags_11 },
    { 0xc35d4a2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69d4a32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181d4a37, SUC_lexicon_tags_47 },
    { 0x1ebd4a38, SUC_lexicon_tags_15 },
    { 0xd9d4a39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc85d4a3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825d4a41, SUC_lexicon_tags_27 },
    { 0xaedd4a42, SUC_lexicon_tags_37 },
    { 0x35bd4a43, SUC_lexicon_tags_27 },
    { 0x381d4a44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929d4a4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedd4a52, SUC_lexicon_tags_1 },
    { 0xcc9d4a53, SUC_lexicon_tags_38 },
    { 0x1e1d4a54, SUC_lexicon_tags_0 },
    { 0x6fdd4a55, SUC_lexicon_tags_37 },
    { 0xc47d4a54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bd4a5e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1d4a65, SUC_lexicon_tags_79 },
    { 0xeebd4a65, SUC_lexicon_tags_7 },
    { 0x2edd4a67, SUC_lexicon_tags_38 },
    { 0x159d4a68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53d4a6d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x953d4a70, SUC_lexicon_tags_25 },
    { 0x385d4a70, SUC_lexicon_tags_25 },
    { 0x8b7d4a71, SUC_lexicon_tags_60 },
    { 0xe11d4a71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90dd4a75, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x33fd4a77, SUC_lexicon_tags_23 },
    { 0xb79d4a77, SUC_lexicon_tags_23 },
    { 0x601d4a79, SUC_lexicon_tags_70 },
    { 0x7efd4a7a, SUC_lexicon_tags_49 },
    { 0xe37d4a79, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6bdd4a7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x981d4a7f, SUC_lexicon_tags_43 },
    { 0xbc9d4a7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701d4a85, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9d4a89, SUC_lexicon_tags_27 },
    { 0x6d5d4a8a, SUC_lexicon_tags_7 },
    { 0x2f9d4a8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28dd4a8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x713d4a92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fd4a99, SUC_lexicon_tags_21 },
    { 0xd47d4a9a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d4a9e, SUC_lexicon_tags_37 },
    { 0x4a3d4a9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bd4aa8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdd4aac, SUC_lexicon_tags_7 },
    { 0x399d4aad, SUC_lexicon_tags_17 },
    { 0x523d4aad, SUC_lexicon_tags_60 },
    { 0xe8dd4aaf, SUC_lexicon_tags_38 },
    { 0x8e3d4ab0, SUC_lexicon_tags_7 },
    { 0x2b3d4aaf, SUC_lexicon_tags_8 },
    { 0x9fdd4ab1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x781d4ab4, SUC_lexicon_tags_77 },
    { 0x31d4ab5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6abd4ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbd4ac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55d4aca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcddd4acc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5d4ad0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb17d4ad2, SUC_lexicon_tags_0 },
    { 0xb87d4ad3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9d4ad6, SUC_lexicon_tags_45 },
    { 0xd09d4ad7, SUC_lexicon_tags_8 },
    { 0x7d1d4ad8, SUC_lexicon_tags_15 },
    { 0xa6dd4ad8, SUC_lexicon_tags_39 },
    { 0x261d4ada, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7d4ade, SUC_lexicon_tags_2 },
    { 0xbdfd4adf, SUC_lexicon_tags_49 },
    { 0x1fdd4adf, SUC_lexicon_tags_10 },
    { 0xe17d4ae1, SUC_lexicon_tags_60 },
    { 0xc89d4ae0, SUC_lexicon_tags_11 },
    { 0x3abd4ae3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dd4ae6, SUC_lexicon_tags_44 },
    { 0xa9fd4ae6, SUC_lexicon_tags_12 },
    { 0xbe5d4ae7, SUC_lexicon_tags_2 },
    { 0x66bd4ae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fd4af7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1fdd4af9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7d4afe, SUC_lexicon_tags_0 },
    { 0x30fd4afe, SUC_lexicon_tags_38 },
    { 0xa21d4aff, SUC_lexicon_tags_17 },
    { 0x523d4b00, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdd3d4b03, SUC_lexicon_tags_27 },
    { 0x307d4b04, SUC_lexicon_tags_12 },
    { 0xa4fd4b05, SUC_lexicon_tags_27 },
    { 0x8a3d4b06, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03d4b09, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x56dd4b0b, SUC_lexicon_tags_25 },
    { 0xbf3d4b0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x213d4b0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x149d4b10, SUC_lexicon_tags_498 },
    { 0xc7bd4b11, SUC_lexicon_tags_2 },
    { 0xfe1d4b10, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fd4b15, SUC_lexicon_tags_0 },
    { 0x29bd4b16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bd4b1d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63d4b22, SUC_lexicon_tags_16 },
    { 0x20fd4b23, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x567d4b26, SUC_lexicon_tags_28 },
    { 0x25d4b27, SUC_lexicon_tags_17 },
    { 0x8ffd4b26, SUC_lexicon_tags_37 },
    { 0xffbd4b29, SUC_lexicon_tags_6 },
    { 0x6bbd4b2a, SUC_lexicon_tags_1 },
    { 0xaf7d4b29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x23bd4b2d, SUC_lexicon_tags_12 },
    { 0xe15d4b2d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dd4b32, SUC_lexicon_tags_63 },
    { 0xce9d4b32, SUC_lexicon_tags_9 },
    { 0x46dd4b34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fd4b38, SUC_lexicon_tags_37 },
    { 0xbf3d4b39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5d4b3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e3d4b3e, SUC_lexicon_tags_12 },
    { 0x8b1d4b3f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x21bd4b41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505d4b46, SUC_lexicon_tags_10 },
    { 0x333d4b47, SUC_lexicon_tags_77 },
    { 0x44dd4b46, SUC_lexicon_tags_124 },
    { 0x2f7d4b49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653d4b4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dd4b52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83d4b57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x32fd4b59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf87d4b5b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xebbd4b5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7d4b61, SUC_lexicon_tags_27 },
    { 0x69fd4b62, SUC_lexicon_tags_42 },
    { 0x783d4b62, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07d4b68, SUC_lexicon_tags_7 },
    { 0xa45d4b69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bd4b76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63d4b79, SUC_lexicon_tags_17 },
    { 0x8e5d4b7a, SUC_lexicon_tags_26 },
    { 0x2c1d4b79, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x28dd4b7d, SUC_lexicon_tags_4 },
    { 0xa0dd4b7e, SUC_lexicon_tags_87 },
    { 0x5abd4b7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x269d4b81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41d4b84, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861d4b89, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fd4b8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x667d4b8f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x885d4b91, SUC_lexicon_tags_34 },
    { 0xacbd4b91, SUC_lexicon_tags_45 },
    { 0x69d4b93, SUC_lexicon_tags_30 },
    { 0xee1d4b94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x677d4b96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3ffd4b98, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dd4ba1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x511d4ba3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7dd4ba5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7d4ba8, SUC_lexicon_tags_17 },
    { 0xee7d4ba9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509d4bad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x745d4bb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b1d4bb2, SUC_lexicon_tags_7 },
    { 0x659d4bb3, SUC_lexicon_tags_17 },
    { 0x36bd4bb3, SUC_lexicon_tags_6 },
    { 0x613d4bb3, SUC_lexicon_tags_66 },
    { 0x32bd4bb6, SUC_lexicon_tags_37 },
    { 0x4f1d4bb7, SUC_lexicon_tags_12 },
    { 0x309d4bb8, SUC_lexicon_tags_47 },
    { 0x6b7d4bb8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9d4bc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x125d4bc2, SUC_lexicon_tags_46 },
    { 0x23dd4bc3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x52dd4bc5, SUC_lexicon_tags_27 },
    { 0xb29d4bc5, SUC_lexicon_tags_53 },
    { 0x31d4bc7, SUC_lexicon_tags_9 },
    { 0xec5d4bc5, SUC_lexicon_tags_66 },
    { 0x55d4bc9, SUC_lexicon_tags_7 },
    { 0xebfd4bca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1d4bce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x103d4bd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d3d4bd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479d4bd7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269d4bdd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145d4be4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbd4be7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b7d4be9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bd4bee, SUC_lexicon_tags_26 },
    { 0x1a7d4bee, SUC_lexicon_tags_27 },
    { 0x7f7d4bf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759d4bf4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5fd4bf6, SUC_lexicon_tags_12 },
    { 0x8f7d4bf7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fd4bfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d4bfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3d4c01, SUC_lexicon_tags_14 },
    { 0x97dd4c02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8cdd4c04, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663d4c09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x311d4c0b, SUC_lexicon_tags_17 },
    { 0x9ffd4c0c, SUC_lexicon_tags_0 },
    { 0xcb9d4c0d, SUC_lexicon_tags_37 },
    { 0xa1d4c0d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7efd4c10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1d4c16, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33d4c1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3d4c1e, SUC_lexicon_tags_55 },
    { 0x741d4c1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea5d4c21, SUC_lexicon_tags_651 },
    { 0, NULL },
    { 0x151d4c23, SUC_lexicon_tags_14 },
    { 0x2d9d4c24, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1d4c2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x117d4c2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9d4c32, SUC_lexicon_tags_17 },
    { 0x5ffd4c33, SUC_lexicon_tags_17 },
    { 0x73dd4c34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7d4c38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x769d4c3a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bd4c42, SUC_lexicon_tags_26 },
    { 0xd13d4c43, SUC_lexicon_tags_15 },
    { 0xe9fd4c44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31d4c4b, SUC_lexicon_tags_170 },
    { 0x707d4c4c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x571d4c4e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadd4c52, SUC_lexicon_tags_17 },
    { 0xad9d4c52, SUC_lexicon_tags_2 },
    { 0xad7d4c52, SUC_lexicon_tags_17 },
    { 0x8c7d4c55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d4c5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa8bd4c5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbd4c63, SUC_lexicon_tags_87 },
    { 0x227d4c64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d4c6a, SUC_lexicon_tags_19 },
    { 0x8e1d4c6a, SUC_lexicon_tags_37 },
    { 0x541d4c6c, SUC_lexicon_tags_1 },
    { 0x61bd4c6c, SUC_lexicon_tags_22 },
    { 0x9afd4c6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x84fd4c70, SUC_lexicon_tags_15 },
    { 0x5b1d4c71, SUC_lexicon_tags_7 },
    { 0x459d4c70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fd4c76, SUC_lexicon_tags_2 },
    { 0x885d4c77, SUC_lexicon_tags_25 },
    { 0xbffd4c78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75d4c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97d4c89, SUC_lexicon_tags_11 },
    { 0xf3dd4c89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69d4c8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75dd4c8f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x599d4c91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3d4c95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43d4c9b, SUC_lexicon_tags_6 },
    { 0xa17d4c9b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401d4ca1, SUC_lexicon_tags_28 },
    { 0x63dd4ca2, SUC_lexicon_tags_1 },
    { 0x72fd4ca3, SUC_lexicon_tags_2 },
    { 0x77fd4ca4, SUC_lexicon_tags_7 },
    { 0x757d4ca5, SUC_lexicon_tags_15 },
    { 0x51fd4ca5, SUC_lexicon_tags_46 },
    { 0x355d4ca7, SUC_lexicon_tags_11 },
    { 0x6f9d4ca6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725d4caf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933d4cb4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dd4cb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89fd4cba, SUC_lexicon_tags_12 },
    { 0xd99d4cbb, SUC_lexicon_tags_9 },
    { 0x84bd4cbc, SUC_lexicon_tags_42 },
    { 0xa21d4cba, SUC_lexicon_tags_25 },
    { 0xe53d4cbe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abd4cc6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x389d4cc8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xdb5d4cca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fd4ccf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9d4cd4, SUC_lexicon_tags_55 },
    { 0xcd7d4cd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x249d4cd7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bd4cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bd4ce4, SUC_lexicon_tags_9 },
    { 0x859d4ce4, SUC_lexicon_tags_14 },
    { 0x587d4ce6, SUC_lexicon_tags_7 },
    { 0x3d1d4ce6, SUC_lexicon_tags_15 },
    { 0xdb1d4ce6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639d4cec, SUC_lexicon_tags_37 },
    { 0x2b7d4ced, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5d4cf0, SUC_lexicon_tags_33 },
    { 0xf3bd4cf0, SUC_lexicon_tags_17 },
    { 0x691d4cf0, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bd4cf6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9d4cf9, SUC_lexicon_tags_79 },
    { 0xe71d4cfa, SUC_lexicon_tags_87 },
    { 0x1edd4cf9, SUC_lexicon_tags_4 },
    { 0x1a1d4cfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313d4d02, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dd4d08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbedd4d0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac5d4d0d, SUC_lexicon_tags_27 },
    { 0x1bdd4d0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fd4d12, SUC_lexicon_tags_17 },
    { 0xaf9d4d13, SUC_lexicon_tags_17 },
    { 0x333d4d13, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33d4d17, SUC_lexicon_tags_37 },
    { 0x713d4d18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dbd4d1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9d4d1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d3d4d1f, SUC_lexicon_tags_1 },
    { 0x6bd4d20, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x683d4d22, SUC_lexicon_tags_10 },
    { 0x67fd4d23, SUC_lexicon_tags_17 },
    { 0x9c1d4d24, SUC_lexicon_tags_37 },
    { 0x927d4d24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x127d4d28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1d4d2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1d4d2f, SUC_lexicon_tags_27 },
    { 0xb29d4d30, SUC_lexicon_tags_87 },
    { 0xf35d4d31, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7d4d37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6ffd4d39, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7f5d4d3b, SUC_lexicon_tags_27 },
    { 0xc13d4d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bd4d42, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd4d4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd4d54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dd4d57, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209d4d5c, SUC_lexicon_tags_37 },
    { 0x19bd4d5d, SUC_lexicon_tags_126 },
    { 0x631d4d5c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x55fd4d60, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x973d4d63, SUC_lexicon_tags_82 },
    { 0xf83d4d64, SUC_lexicon_tags_82 },
    { 0xc69d4d65, SUC_lexicon_tags_48 },
    { 0xb5fd4d66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fd4d6a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37d4d6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x74fd4d70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dd4d74, SUC_lexicon_tags_15 },
    { 0xf03d4d74, SUC_lexicon_tags_11 },
    { 0xf5d4d76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd4d7a, SUC_lexicon_tags_6 },
    { 0x5fd4d7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1d4d7f, SUC_lexicon_tags_43 },
    { 0x2c1d4d80, SUC_lexicon_tags_60 },
    { 0xb59d4d7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4afd4d8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503d4d8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7c3d4d91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa53d4d93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9d4d99, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dd4d9d, SUC_lexicon_tags_25 },
    { 0xafd4d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x843d4da0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x879d4da2, SUC_lexicon_tags_37 },
    { 0x8cbd4da3, SUC_lexicon_tags_87 },
    { 0x43dd4da4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bd4da9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1d4db1, SUC_lexicon_tags_45 },
    { 0x981d4db2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x705d4db4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1d4db7, SUC_lexicon_tags_37 },
    { 0x89dd4db7, SUC_lexicon_tags_9 },
    { 0x27d4db8, SUC_lexicon_tags_43 },
    { 0x41dd4db7, SUC_lexicon_tags_56 },
    { 0xa97d4dbb, SUC_lexicon_tags_9 },
    { 0xfa7d4dbc, SUC_lexicon_tags_12 },
    { 0x6dfd4dbd, SUC_lexicon_tags_45 },
    { 0xc3bd4dbb, SUC_lexicon_tags_45 },
    { 0x111d4db9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xccbd4dc1, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0x687d4dc4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x829d4dc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d1d4dc8, SUC_lexicon_tags_7 },
    { 0xa35d4dc9, SUC_lexicon_tags_14 },
    { 0xd0bd4dc9, SUC_lexicon_tags_17 },
    { 0x5b7d4dcb, SUC_lexicon_tags_11 },
    { 0x34dd4dc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe69d4dce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3d4dd3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25d4dd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d4ddd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x395d4de0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dbd4de2, SUC_lexicon_tags_27 },
    { 0x5add4de3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fd4de8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9d4dee, SUC_lexicon_tags_37 },
    { 0x13fd4dee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535d4df4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x5afd4df7, SUC_lexicon_tags_0 },
    { 0xbe5d4df8, SUC_lexicon_tags_25 },
    { 0xd6dd4df8, SUC_lexicon_tags_12 },
    { 0x709d4dfa, SUC_lexicon_tags_7 },
    { 0x13d4dfb, SUC_lexicon_tags_7 },
    { 0x85bd4dfc, SUC_lexicon_tags_12 },
    { 0x571d4dfd, SUC_lexicon_tags_7 },
    { 0xb49d4dfe, SUC_lexicon_tags_15 },
    { 0xadfd4dfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bd4e02, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x315d4e04, SUC_lexicon_tags_56 },
    { 0xecdd4e04, SUC_lexicon_tags_38 },
    { 0x1edd4e05, SUC_lexicon_tags_43 },
    { 0x9d3d4e06, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x67d4e0a, SUC_lexicon_tags_2 },
    { 0x9a7d4e0b, SUC_lexicon_tags_28 },
    { 0xd7fd4e0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129d4e16, SUC_lexicon_tags_27 },
    { 0x303d4e17, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61d4e1a, SUC_lexicon_tags_26 },
    { 0x84dd4e1a, SUC_lexicon_tags_25 },
    { 0x959d4e1b, SUC_lexicon_tags_35 },
    { 0xa4bd4e1d, SUC_lexicon_tags_46 },
    { 0x8c5d4e1d, SUC_lexicon_tags_7 },
    { 0xcebd4e1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7f9d4e21, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215d4e28, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53d4e2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd4e30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd2dd4e32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7d4e35, SUC_lexicon_tags_33 },
    { 0x291d4e36, SUC_lexicon_tags_27 },
    { 0x86bd4e37, SUC_lexicon_tags_19 },
    { 0xff5d4e37, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x971d4e3a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x62dd4e3d, SUC_lexicon_tags_11 },
    { 0x435d4e3d, SUC_lexicon_tags_23 },
    { 0x3fd4e3e, SUC_lexicon_tags_1 },
    { 0x381d4e3e, SUC_lexicon_tags_37 },
    { 0x613d4e41, SUC_lexicon_tags_1 },
    { 0x249d4e42, SUC_lexicon_tags_2 },
    { 0x41dd4e43, SUC_lexicon_tags_26 },
    { 0x60fd4e44, SUC_lexicon_tags_43 },
    { 0xfe1d4e43, SUC_lexicon_tags_12 },
    { 0xdf9d4e45, SUC_lexicon_tags_47 },
    { 0x25d4e43, SUC_lexicon_tags_17 },
    { 0x8b1d4e48, SUC_lexicon_tags_17 },
    { 0x1ebd4e41, SUC_lexicon_tags_17 },
    { 0x21bd4e49, SUC_lexicon_tags_39 },
    { 0x291d4e45, SUC_lexicon_tags_25 },
    { 0x3dbd4e4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb21d4e4e, SUC_lexicon_tags_95 },
    { 0xf27d4e4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3d4e54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fd4e5b, SUC_lexicon_tags_56 },
    { 0xd19d4e5c, SUC_lexicon_tags_19 },
    { 0x22dd4e5d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23d4e62, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1d4e65, SUC_lexicon_tags_11 },
    { 0x311d4e66, SUC_lexicon_tags_9 },
    { 0x403d4e67, SUC_lexicon_tags_14 },
    { 0x7d1d4e68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b3d4e6a, SUC_lexicon_tags_9 },
    { 0x9c7d4e6b, SUC_lexicon_tags_30 },
    { 0x52fd4e6c, SUC_lexicon_tags_4 },
    { 0x7c1d4e6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadd4e70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107d4e7a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87d4e80, SUC_lexicon_tags_8 },
    { 0x2d9d4e81, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341d4e85, SUC_lexicon_tags_1 },
    { 0x833d4e86, SUC_lexicon_tags_6 },
    { 0x7efd4e86, SUC_lexicon_tags_11 },
    { 0xbd1d4e86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fd4e8c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x139d4e8e, SUC_lexicon_tags_37 },
    { 0x78fd4e8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5d4e92, SUC_lexicon_tags_77 },
    { 0x661d4e92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9d4e96, SUC_lexicon_tags_4 },
    { 0x76dd4e97, SUC_lexicon_tags_16 },
    { 0x5c5d4e98, SUC_lexicon_tags_0 },
    { 0xf87d4e99, SUC_lexicon_tags_47 },
    { 0xcf1d4e9a, SUC_lexicon_tags_84 },
    { 0x45dd4e98, SUC_lexicon_tags_0 },
    { 0x6c9d4e9b, SUC_lexicon_tags_17 },
    { 0x71d4e98, SUC_lexicon_tags_11 },
    { 0x88fd4e9b, SUC_lexicon_tags_37 },
    { 0x787d4e9f, SUC_lexicon_tags_15 },
    { 0xe35d4ea0, SUC_lexicon_tags_26 },
    { 0xfcfd4ea1, SUC_lexicon_tags_19 },
    { 0x2c5d4ea2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371d4ea6, SUC_lexicon_tags_7 },
    { 0x797d4ea7, SUC_lexicon_tags_21 },
    { 0x89d4ea7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d4eaa, SUC_lexicon_tags_7 },
    { 0xe2fd4eab, SUC_lexicon_tags_107 },
    { 0x971d4eac, SUC_lexicon_tags_16 },
    { 0x48bd4ead, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdd4eb2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x513d4eb4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07d4eb7, SUC_lexicon_tags_64 },
    { 0x4f7d4eb7, SUC_lexicon_tags_7 },
    { 0xe27d4eb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb1d4ebb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x75dd4ebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499d4ec4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbd4ec8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xfa7d4eca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509d4ed1, SUC_lexicon_tags_11 },
    { 0xcb3d4ed1, SUC_lexicon_tags_25 },
    { 0x1c3d4ed3, SUC_lexicon_tags_82 },
    { 0x9a9d4ed4, SUC_lexicon_tags_72 },
    { 0xd77d4ed4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x74dd4ed8, SUC_lexicon_tags_47 },
    { 0xcd7d4ed9, SUC_lexicon_tags_15 },
    { 0x245d4ed9, SUC_lexicon_tags_23 },
    { 0xf5d4edb, SUC_lexicon_tags_17 },
    { 0x51fd4edc, SUC_lexicon_tags_37 },
    { 0xb13d4edd, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471d4ee1, SUC_lexicon_tags_10 },
    { 0x419d4ee2, SUC_lexicon_tags_43 },
    { 0x4bbd4ee3, SUC_lexicon_tags_28 },
    { 0xb53d4ee3, SUC_lexicon_tags_1 },
    { 0x6c5d4ee4, SUC_lexicon_tags_1 },
    { 0xd8bd4ee6, SUC_lexicon_tags_1 },
    { 0x85dd4ee6, SUC_lexicon_tags_2 },
    { 0xd93d4ee8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x473d4eeb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe03d4eed, SUC_lexicon_tags_47 },
    { 0xfd9d4eed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fd4ef2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc71d4ef4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985d4efa, SUC_lexicon_tags_33 },
    { 0xe5dd4efa, SUC_lexicon_tags_13 },
    { 0x1dbd4efa, SUC_lexicon_tags_37 },
    { 0x87fd4efd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d4f03, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xf33d4f05, SUC_lexicon_tags_7 },
    { 0x8d3d4f05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8fbd4f08, SUC_lexicon_tags_17 },
    { 0xd65d4f08, SUC_lexicon_tags_23 },
    { 0xe81d4f0a, SUC_lexicon_tags_37 },
    { 0x929d4f0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfdbd4f0d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc3d4f0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cdd4f11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb63d4f13, SUC_lexicon_tags_38 },
    { 0xe1bd4f14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd4f17, SUC_lexicon_tags_15 },
    { 0xdd9d4f18, SUC_lexicon_tags_33 },
    { 0xa7fd4f19, SUC_lexicon_tags_61 },
    { 0xc59d4f1a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbd4f1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fd4f20, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xed7d4f22, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7d4f2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd0dd4f2c, SUC_lexicon_tags_17 },
    { 0xe33d4f2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64fd4f2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3d4f32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x337d4f34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa17d4f36, SUC_lexicon_tags_11 },
    { 0xf2bd4f37, SUC_lexicon_tags_14 },
    { 0x287d4f36, SUC_lexicon_tags_2 },
    { 0xebd4f36, SUC_lexicon_tags_33 },
    { 0xb5bd4f3a, SUC_lexicon_tags_9 },
    { 0x905d4f3a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11d4f3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bd4f41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfd4f46, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951d4f4b, SUC_lexicon_tags_11 },
    { 0xedd4f4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917d4f50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82fd4f52, SUC_lexicon_tags_12 },
    { 0x727d4f53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3d4f57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3d4f5a, SUC_lexicon_tags_11 },
    { 0xdbbd4f5b, SUC_lexicon_tags_12 },
    { 0x127d4f5b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd4f60, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09d4f64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9d4f68, SUC_lexicon_tags_12 },
    { 0x6bdd4f69, SUC_lexicon_tags_1 },
    { 0xe9dd4f68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd4f6d, SUC_lexicon_tags_45 },
    { 0x21d4f6e, SUC_lexicon_tags_45 },
    { 0xe0fd4f6e, SUC_lexicon_tags_29 },
    { 0x3dfd4f6f, SUC_lexicon_tags_19 },
    { 0xf5dd4f71, SUC_lexicon_tags_37 },
    { 0x61fd4f72, SUC_lexicon_tags_9 },
    { 0xca3d4f72, SUC_lexicon_tags_85 },
    { 0x685d4f71, SUC_lexicon_tags_14 },
    { 0x411d4f73, SUC_lexicon_tags_19 },
    { 0x3efd4f76, SUC_lexicon_tags_27 },
    { 0x81bd4f77, SUC_lexicon_tags_26 },
    { 0x283d4f78, SUC_lexicon_tags_15 },
    { 0xdb1d4f79, SUC_lexicon_tags_17 },
    { 0xf5bd4f7a, SUC_lexicon_tags_49 },
    { 0x735d4f7b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x63bd4f7d, SUC_lexicon_tags_11 },
    { 0xd69d4f7d, SUC_lexicon_tags_6 },
    { 0x4ffd4f7f, SUC_lexicon_tags_8 },
    { 0x3dbd4f7f, SUC_lexicon_tags_11 },
    { 0x269d4f81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x325d4f84, SUC_lexicon_tags_14 },
    { 0x439d4f84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5d4f89, SUC_lexicon_tags_31 },
    { 0x109d4f8a, SUC_lexicon_tags_2 },
    { 0xe01d4f8b, SUC_lexicon_tags_597 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9d4f8e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x1e3d4f90, SUC_lexicon_tags_47 },
    { 0xfbbd4f91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69d4f94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x191d4f96, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1d4f9b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x843d4f9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abd4fa5, SUC_lexicon_tags_19 },
    { 0xadbd4fa5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53d4fac, SUC_lexicon_tags_11 },
    { 0x479d4fad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5d4fb5, SUC_lexicon_tags_23 },
    { 0xe99d4fb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7d4fbb, SUC_lexicon_tags_14 },
    { 0xeedd4fbc, SUC_lexicon_tags_17 },
    { 0x8f1d4fbd, SUC_lexicon_tags_37 },
    { 0x13fd4fbe, SUC_lexicon_tags_23 },
    { 0x56dd4fbe, SUC_lexicon_tags_37 },
    { 0x92bd4fbf, SUC_lexicon_tags_32 },
    { 0x6f3d4fbd, SUC_lexicon_tags_25 },
    { 0x59dd4fbd, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7d4fca, SUC_lexicon_tags_47 },
    { 0xb13d4fca, SUC_lexicon_tags_11 },
    { 0x369d4fcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3d4fd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bd4fd5, SUC_lexicon_tags_37 },
    { 0xe6dd4fd5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5d4fe1, SUC_lexicon_tags_43 },
    { 0xbafd4fe2, SUC_lexicon_tags_454 },
    { 0x3afd4fe3, SUC_lexicon_tags_11 },
    { 0xee7d4fe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3d4ff1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23d4ff4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x331d4ff6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1d4ffc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fd5005, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981d5009, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787d500d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd25d500f, SUC_lexicon_tags_3 },
    { 0xd11d5010, SUC_lexicon_tags_6 },
    { 0x18fd5011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb9d5013, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd501a, SUC_lexicon_tags_63 },
    { 0x9d7d501a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9d5022, SUC_lexicon_tags_7 },
    { 0x26bd5023, SUC_lexicon_tags_11 },
    { 0x2d5d5023, SUC_lexicon_tags_37 },
    { 0xf07d5025, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x485d5028, SUC_lexicon_tags_15 },
    { 0x7b7d5029, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb6dd502b, SUC_lexicon_tags_1 },
    { 0x97dd502c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x647d502e, SUC_lexicon_tags_47 },
    { 0xf7fd502e, SUC_lexicon_tags_13 },
    { 0xa0fd502e, SUC_lexicon_tags_14 },
    { 0xaadd5030, SUC_lexicon_tags_12 },
    { 0x29fd5031, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71d5038, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdbd503a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x719d503c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49d5040, SUC_lexicon_tags_17 },
    { 0xb7bd5041, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc5d5043, SUC_lexicon_tags_25 },
    { 0xef1d5043, SUC_lexicon_tags_82 },
    { 0xe9d5045, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa47d5047, SUC_lexicon_tags_15 },
    { 0x2add5048, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7d5051, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x101d5053, SUC_lexicon_tags_46 },
    { 0xa57d5054, SUC_lexicon_tags_14 },
    { 0x7d1d5055, SUC_lexicon_tags_27 },
    { 0xa15d5055, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadd505c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741d5069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abd5071, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdcdd5073, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdadd5075, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3d5078, SUC_lexicon_tags_2 },
    { 0x52bd5078, SUC_lexicon_tags_1 },
    { 0x8d1d5079, SUC_lexicon_tags_14 },
    { 0x9c1d507b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bd5082, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a7d5084, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3d5087, SUC_lexicon_tags_11 },
    { 0xb53d5087, SUC_lexicon_tags_12 },
    { 0x69d5089, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1d508d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59d5090, SUC_lexicon_tags_22 },
    { 0xe6bd5090, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65d509b, SUC_lexicon_tags_49 },
    { 0xc79d509c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89d509f, SUC_lexicon_tags_2 },
    { 0x41dd50a0, SUC_lexicon_tags_23 },
    { 0xda5d50a1, SUC_lexicon_tags_5 },
    { 0xb83d50a2, SUC_lexicon_tags_47 },
    { 0x29d509f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fd50ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e7d50ae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3d50b1, SUC_lexicon_tags_82 },
    { 0xc17d50b1, SUC_lexicon_tags_14 },
    { 0x499d50b1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x941d50b5, SUC_lexicon_tags_33 },
    { 0xe6bd50b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd8dd50b8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbd50bc, SUC_lexicon_tags_37 },
    { 0xa81d50bd, SUC_lexicon_tags_0 },
    { 0x905d50bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x513d50c0, SUC_lexicon_tags_1 },
    { 0x401d50c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd7fd50c3, SUC_lexicon_tags_9 },
    { 0x207d50c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1d50c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615d50cb, SUC_lexicon_tags_77 },
    { 0x1ddd50cc, SUC_lexicon_tags_1 },
    { 0x75dd50cd, SUC_lexicon_tags_27 },
    { 0x2fdd50ce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3d50d1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x12dd50d3, SUC_lexicon_tags_16 },
    { 0x615d50d4, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0x475d50d7, SUC_lexicon_tags_11 },
    { 0xfd5d50d8, SUC_lexicon_tags_25 },
    { 0x547d50d8, SUC_lexicon_tags_63 },
    { 0xfb9d50da, SUC_lexicon_tags_25 },
    { 0x70dd50db, SUC_lexicon_tags_47 },
    { 0x81fd50dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bd50e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fd50e3, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0x5e5d50e5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11fd50f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x259d50fb, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3d50ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3d5102, SUC_lexicon_tags_184 },
    { 0x109d5102, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xaa3d5105, SUC_lexicon_tags_13 },
    { 0xa69d5106, SUC_lexicon_tags_30 },
    { 0x6e3d5107, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fd510b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1a7d510d, SUC_lexicon_tags_0 },
    { 0xb75d510d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa57d5110, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd89d5112, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x753d5115, SUC_lexicon_tags_15 },
    { 0x979d5116, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x19d5118, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbebd511a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dd5122, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691d5127, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35d512c, SUC_lexicon_tags_2 },
    { 0xed5d512c, SUC_lexicon_tags_27 },
    { 0x75fd512e, SUC_lexicon_tags_2 },
    { 0xd17d512f, SUC_lexicon_tags_27 },
    { 0xd4dd512e, SUC_lexicon_tags_96 },
    { 0x883d5131, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929d5140, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483d5144, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07d5147, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5d514a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x341d514c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29d5157, SUC_lexicon_tags_49 },
    { 0x225d5157, SUC_lexicon_tags_10 },
    { 0x6abd5158, SUC_lexicon_tags_40 },
    { 0x6d9d5159, SUC_lexicon_tags_17 },
    { 0x4a5d515b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13d5163, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31dd5167, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1add516a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x87dd516c, SUC_lexicon_tags_11 },
    { 0x4fd516d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d5170, SUC_lexicon_tags_8 },
    { 0xcd3d5171, SUC_lexicon_tags_25 },
    { 0xdd9d5172, SUC_lexicon_tags_32 },
    { 0xdcdd5173, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907d517e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bd5181, SUC_lexicon_tags_17 },
    { 0x7ddd5181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe57d5184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91d5187, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bd5195, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fd5198, SUC_lexicon_tags_37 },
    { 0x3c5d5198, SUC_lexicon_tags_38 },
    { 0x323d519a, SUC_lexicon_tags_56 },
    { 0xe77d519b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x833d519d, SUC_lexicon_tags_7 },
    { 0x60bd519d, SUC_lexicon_tags_19 },
    { 0xde1d519f, SUC_lexicon_tags_21 },
    { 0x7c7d51a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3d51a6, SUC_lexicon_tags_60 },
    { 0x86dd51a7, SUC_lexicon_tags_17 },
    { 0xeebd51a8, SUC_lexicon_tags_18 },
    { 0xcddd51a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dd51ac, SUC_lexicon_tags_32 },
    { 0x2a9d51ac, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xc41d51af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x183d51b1, SUC_lexicon_tags_12 },
    { 0xcd7d51b1, SUC_lexicon_tags_7 },
    { 0x12fd51b3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xbb3d51b5, SUC_lexicon_tags_37 },
    { 0x59fd51b6, SUC_lexicon_tags_522 },
    { 0x647d51b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfbdd51b9, SUC_lexicon_tags_19 },
    { 0xcffd51ba, SUC_lexicon_tags_24 },
    { 0x5a3d51b9, SUC_lexicon_tags_21 },
    { 0xa17d51bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfd51c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc17d51c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111d51c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dd51cd, SUC_lexicon_tags_6 },
    { 0xf0bd51cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5d51d1, SUC_lexicon_tags_25 },
    { 0x56fd51d2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5d51d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6dd51d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d51dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48dd51e0, SUC_lexicon_tags_8 },
    { 0x2d1d51e1, SUC_lexicon_tags_135 },
    { 0, NULL },
    { 0x883d51e3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc53d51e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabd51e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc15d51e9, SUC_lexicon_tags_37 },
    { 0x7b7d51ea, SUC_lexicon_tags_37 },
    { 0x969d51eb, SUC_lexicon_tags_85 },
    { 0xa1bd51ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225d51f0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd4fd51f2, SUC_lexicon_tags_29 },
    { 0x69bd51f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1dd51f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fd51f8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x879d51fa, SUC_lexicon_tags_31 },
    { 0xf61d51fb, SUC_lexicon_tags_37 },
    { 0x13bd51fc, SUC_lexicon_tags_11 },
    { 0x3b1d51fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441d5203, SUC_lexicon_tags_47 },
    { 0xb17d5203, SUC_lexicon_tags_103 },
    { 0xe95d5205, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dd5209, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941d520d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8e5d520f, SUC_lexicon_tags_59 },
    { 0xf47d5210, SUC_lexicon_tags_29 },
    { 0x631d5211, SUC_lexicon_tags_17 },
    { 0xfe5d5212, SUC_lexicon_tags_30 },
    { 0xb07d5213, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1d5217, SUC_lexicon_tags_25 },
    { 0xf77d5218, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99d521d, SUC_lexicon_tags_17 },
    { 0x16dd521e, SUC_lexicon_tags_17 },
    { 0xd73d521f, SUC_lexicon_tags_35 },
    { 0x46fd5220, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffd5223, SUC_lexicon_tags_140 },
    { 0xbcdd5223, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d522f, SUC_lexicon_tags_2 },
    { 0x34dd5230, SUC_lexicon_tags_13 },
    { 0xfd1d522f, SUC_lexicon_tags_19 },
    { 0x4a3d5232, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5d5237, SUC_lexicon_tags_17 },
    { 0xb21d5238, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d523e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927d5243, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc17d5245, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbd524b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad1d524d, SUC_lexicon_tags_2 },
    { 0x1fdd524e, SUC_lexicon_tags_9 },
    { 0x78bd524e, SUC_lexicon_tags_25 },
    { 0xb9dd524d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9d5255, SUC_lexicon_tags_39 },
    { 0x177d5255, SUC_lexicon_tags_2 },
    { 0x78fd5257, SUC_lexicon_tags_47 },
    { 0x783d5255, SUC_lexicon_tags_19 },
    { 0xda9d5259, SUC_lexicon_tags_6 },
    { 0x3b9d5259, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12bd525c, SUC_lexicon_tags_1 },
    { 0x1fd525c, SUC_lexicon_tags_12 },
    { 0x2bfd525d, SUC_lexicon_tags_19 },
    { 0x55fd525f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659d5263, SUC_lexicon_tags_25 },
    { 0x871d5264, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3d5267, SUC_lexicon_tags_48 },
    { 0x249d5268, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe8fd526a, SUC_lexicon_tags_26 },
    { 0x1cbd526b, SUC_lexicon_tags_31 },
    { 0xef5d526b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf37d526f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb1d5271, SUC_lexicon_tags_0 },
    { 0xb79d5271, SUC_lexicon_tags_14 },
    { 0x565d5273, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x779d5275, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x951d5277, SUC_lexicon_tags_15 },
    { 0x3b3d5278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267d527e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xd57d5280, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x679d5282, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1d5289, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07d528e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07d5292, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997d5298, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3d529b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7d52a5, SUC_lexicon_tags_32 },
    { 0x231d52a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741d52aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdd52af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa07d52b1, SUC_lexicon_tags_77 },
    { 0x71fd52b2, SUC_lexicon_tags_7 },
    { 0xf7dd52b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x923d52b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x747d52b7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7d52ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9d52be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9d52c3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fd52cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9d52d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf77d52d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd52d7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953d52db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dd52dd, SUC_lexicon_tags_49 },
    { 0x4efd52de, SUC_lexicon_tags_11 },
    { 0x63bd52df, SUC_lexicon_tags_15 },
    { 0xb0fd52e0, SUC_lexicon_tags_6 },
    { 0x215d52e1, SUC_lexicon_tags_11 },
    { 0xb4dd52e0, SUC_lexicon_tags_63 },
    { 0xe8fd52e2, SUC_lexicon_tags_84 },
    { 0x917d52e1, SUC_lexicon_tags_30 },
    { 0xdcdd52e5, SUC_lexicon_tags_17 },
    { 0x10dd52e4, SUC_lexicon_tags_11 },
    { 0x737d52dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fd52ea, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x18dd52ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9c3d52ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x985d52f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe51d52f2, SUC_lexicon_tags_25 },
    { 0x59d52f2, SUC_lexicon_tags_24 },
    { 0xa93d52f4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x183d52f6, SUC_lexicon_tags_17 },
    { 0x94fd52f7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253d52fc, SUC_lexicon_tags_61 },
    { 0x9e5d52fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69d5300, SUC_lexicon_tags_17 },
    { 0xd71d5300, SUC_lexicon_tags_151 },
    { 0x623d5301, SUC_lexicon_tags_15 },
    { 0xa53d5303, SUC_lexicon_tags_12 },
    { 0x889d5304, SUC_lexicon_tags_19 },
    { 0x285d5300, SUC_lexicon_tags_56 },
    { 0x9e1d5306, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bd5309, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6edd530b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13d530e, SUC_lexicon_tags_23 },
    { 0x4f7d530f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1d5312, SUC_lexicon_tags_14 },
    { 0x72fd5313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fd5317, SUC_lexicon_tags_49 },
    { 0xa47d5317, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1d531e, SUC_lexicon_tags_29 },
    { 0x21d531f, SUC_lexicon_tags_27 },
    { 0x7dbd5320, SUC_lexicon_tags_34 },
    { 0x5f1d5321, SUC_lexicon_tags_14 },
    { 0xc63d5322, SUC_lexicon_tags_37 },
    { 0x98bd5322, SUC_lexicon_tags_1 },
    { 0x863d5324, SUC_lexicon_tags_1547 },
    { 0x8c9d5325, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e3d5327, SUC_lexicon_tags_48 },
    { 0x501d5328, SUC_lexicon_tags_11 },
    { 0xd0fd5328, SUC_lexicon_tags_25 },
    { 0xb61d5327, SUC_lexicon_tags_386 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5d532d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79d5330, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14bd5332, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfabd5334, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dd5338, SUC_lexicon_tags_37 },
    { 0x13d5338, SUC_lexicon_tags_12 },
    { 0x4c7d533a, SUC_lexicon_tags_4 },
    { 0x243d5338, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x93fd533d, SUC_lexicon_tags_1 },
    { 0x677d533e, SUC_lexicon_tags_25 },
    { 0x375d533e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdd5344, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7d534e, SUC_lexicon_tags_11 },
    { 0x8add534f, SUC_lexicon_tags_14 },
    { 0x4a3d534f, SUC_lexicon_tags_63 },
    { 0x343d534f, SUC_lexicon_tags_37 },
    { 0x561d534f, SUC_lexicon_tags_8 },
    { 0xfbdd5353, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03d5358, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3d535b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x897d535e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8ddd5360, SUC_lexicon_tags_31 },
    { 0x147d5361, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7d5367, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807d536b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1d536f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7d5375, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873d5379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdd537f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fd5382, SUC_lexicon_tags_1 },
    { 0x801d5383, SUC_lexicon_tags_37 },
    { 0x539d5384, SUC_lexicon_tags_23 },
    { 0x40dd5384, SUC_lexicon_tags_12 },
    { 0x83bd5385, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x11d5388, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fd538c, SUC_lexicon_tags_12 },
    { 0xb59d538d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3d5396, SUC_lexicon_tags_26 },
    { 0xfffd5397, SUC_lexicon_tags_7 },
    { 0xa43d5398, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69d539d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x823d53a0, SUC_lexicon_tags_17 },
    { 0xddd53a1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335d53a7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x903d53aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91d53b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bd53b8, SUC_lexicon_tags_17 },
    { 0x5e5d53b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465d53c0, SUC_lexicon_tags_11 },
    { 0x9cbd53c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9d53c4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fd53ca, SUC_lexicon_tags_57 },
    { 0xacbd53ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x257d53cd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fd53d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bd53d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbd53da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47d53de, SUC_lexicon_tags_45 },
    { 0xb91d53de, SUC_lexicon_tags_10 },
    { 0x741d53e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53d53e4, SUC_lexicon_tags_37 },
    { 0x73bd53e5, SUC_lexicon_tags_25 },
    { 0x7c5d53e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcb7d53e8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xaffd53ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69d53ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddd53f3, SUC_lexicon_tags_87 },
    { 0xb7d53f4, SUC_lexicon_tags_37 },
    { 0x2a9d53f5, SUC_lexicon_tags_25 },
    { 0xbe5d53f6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaddd53f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0bd53fa, SUC_lexicon_tags_2 },
    { 0x917d53fa, SUC_lexicon_tags_70 },
    { 0x8a9d53fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffd5402, SUC_lexicon_tags_124 },
    { 0x771d5402, SUC_lexicon_tags_45 },
    { 0x457d5402, SUC_lexicon_tags_17 },
    { 0x8e1d5405, SUC_lexicon_tags_17 },
    { 0xbbdd5406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15d5408, SUC_lexicon_tags_23 },
    { 0xed1d5409, SUC_lexicon_tags_11 },
    { 0xb8bd5408, SUC_lexicon_tags_37 },
    { 0x23fd540b, SUC_lexicon_tags_28 },
    { 0xe43d540b, SUC_lexicon_tags_28 },
    { 0x979d540d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71d5412, SUC_lexicon_tags_7 },
    { 0x361d5413, SUC_lexicon_tags_45 },
    { 0xc53d5413, SUC_lexicon_tags_14 },
    { 0x9b5d5415, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9d541a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6bd541c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x341d541e, SUC_lexicon_tags_45 },
    { 0xd69d541e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dd5422, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47d5425, SUC_lexicon_tags_56 },
    { 0x2fd5426, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x795d5428, SUC_lexicon_tags_85 },
    { 0x3e3d5429, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbd542d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9d5431, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5d5437, SUC_lexicon_tags_17 },
    { 0x291d5438, SUC_lexicon_tags_6 },
    { 0xa8fd5439, SUC_lexicon_tags_19 },
    { 0x247d543a, SUC_lexicon_tags_6 },
    { 0x90dd543b, SUC_lexicon_tags_2 },
    { 0xd59d543c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43d5442, SUC_lexicon_tags_57 },
    { 0xe81d5443, SUC_lexicon_tags_25 },
    { 0x37bd5444, SUC_lexicon_tags_11 },
    { 0xd91d5444, SUC_lexicon_tags_3 },
    { 0x285d5446, SUC_lexicon_tags_5 },
    { 0x885d5445, SUC_lexicon_tags_63 },
    { 0xb17d5448, SUC_lexicon_tags_87 },
    { 0x3d5448, SUC_lexicon_tags_29 },
    { 0x7bbd5444, SUC_lexicon_tags_43 },
    { 0x4fd5448, SUC_lexicon_tags_14 },
    { 0xa43d544c, SUC_lexicon_tags_1 },
    { 0x8b7d544c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1d5451, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x277d5453, SUC_lexicon_tags_43 },
    { 0xd93d5454, SUC_lexicon_tags_15 },
    { 0xf85d5455, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7add545a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x411d545c, SUC_lexicon_tags_1 },
    { 0x38fd545c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x37fd545f, SUC_lexicon_tags_1 },
    { 0x583d5460, SUC_lexicon_tags_10 },
    { 0x1e9d5460, SUC_lexicon_tags_11 },
    { 0xb69d5461, SUC_lexicon_tags_0 },
    { 0x821d5462, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x13fd5465, SUC_lexicon_tags_19 },
    { 0x919d5466, SUC_lexicon_tags_63 },
    { 0x9c7d5465, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x797d5469, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdd546c, SUC_lexicon_tags_12 },
    { 0xa6fd546c, SUC_lexicon_tags_25 },
    { 0xb61d546e, SUC_lexicon_tags_29 },
    { 0x739d546e, SUC_lexicon_tags_7 },
    { 0x255d546d, SUC_lexicon_tags_17 },
    { 0x8d1d5471, SUC_lexicon_tags_7 },
    { 0xe21d5472, SUC_lexicon_tags_7 },
    { 0xe5bd546d, SUC_lexicon_tags_12 },
    { 0x4fd5472, SUC_lexicon_tags_25 },
    { 0x8bdd5475, SUC_lexicon_tags_65 },
    { 0x5f9d5475, SUC_lexicon_tags_30 },
    { 0x2c3d5475, SUC_lexicon_tags_0 },
    { 0x20dd5472, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd35d547a, SUC_lexicon_tags_11 },
    { 0x9cfd547b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1cfd547d, SUC_lexicon_tags_39 },
    { 0x1dfd547e, SUC_lexicon_tags_56 },
    { 0xfefd547d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x143d5482, SUC_lexicon_tags_27 },
    { 0x32bd5482, SUC_lexicon_tags_29 },
    { 0x3edd5484, SUC_lexicon_tags_1 },
    { 0xf85d5485, SUC_lexicon_tags_25 },
    { 0x603d5486, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x203d5488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dd548b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ffd548d, SUC_lexicon_tags_17 },
    { 0xe3dd548e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567d5492, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223d5497, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73d549a, SUC_lexicon_tags_26 },
    { 0xae1d549b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dd549f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x225d54a2, SUC_lexicon_tags_47 },
    { 0x9a9d54a3, SUC_lexicon_tags_12 },
    { 0xa65d54a4, SUC_lexicon_tags_6 },
    { 0x37dd54a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dd54a8, SUC_lexicon_tags_15 },
    { 0xe85d54a9, SUC_lexicon_tags_27 },
    { 0x615d54aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x595d54ad, SUC_lexicon_tags_68 },
    { 0x3c7d54ae, SUC_lexicon_tags_17 },
    { 0x63dd54af, SUC_lexicon_tags_84 },
    { 0x4abd54ae, SUC_lexicon_tags_48 },
    { 0x485d54b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77d54b5, SUC_lexicon_tags_32 },
    { 0x4b3d54b5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9f9d54b8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7d54ba, SUC_lexicon_tags_7 },
    { 0xb05d54bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe71d54bd, SUC_lexicon_tags_38 },
    { 0xd6dd54be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x96fd54c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdb9d54c2, SUC_lexicon_tags_13 },
    { 0xe27d54c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe53d54c5, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5d54c8, SUC_lexicon_tags_16 },
    { 0xafdd54c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dd54cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xef9d54cf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d54d2, SUC_lexicon_tags_1 },
    { 0xc83d54d2, SUC_lexicon_tags_12 },
    { 0xecdd54d2, SUC_lexicon_tags_23 },
    { 0x831d54d5, SUC_lexicon_tags_2 },
    { 0xa69d54d6, SUC_lexicon_tags_17 },
    { 0x171d54d3, SUC_lexicon_tags_1 },
    { 0xdc1d54d8, SUC_lexicon_tags_30 },
    { 0xeadd54d6, SUC_lexicon_tags_37 },
    { 0x1b9d54da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcbbd54dc, SUC_lexicon_tags_10 },
    { 0xf37d54dd, SUC_lexicon_tags_38 },
    { 0x321d54dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x335d54e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37d54e4, SUC_lexicon_tags_23 },
    { 0x6bbd54e4, SUC_lexicon_tags_33 },
    { 0xaadd54e4, SUC_lexicon_tags_26 },
    { 0x9b3d54e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd19d54e9, SUC_lexicon_tags_37 },
    { 0x1fd54ea, SUC_lexicon_tags_7 },
    { 0x347d54eb, SUC_lexicon_tags_16 },
    { 0xd31d54ec, SUC_lexicon_tags_7 },
    { 0xe1d54ed, SUC_lexicon_tags_68 },
    { 0xeffd54ee, SUC_lexicon_tags_23 },
    { 0x9c5d54ec, SUC_lexicon_tags_37 },
    { 0x427d54f0, SUC_lexicon_tags_15 },
    { 0x315d54ec, SUC_lexicon_tags_2 },
    { 0xee9d54f0, SUC_lexicon_tags_7 },
    { 0xa83d54f3, SUC_lexicon_tags_49 },
    { 0xa43d54f3, SUC_lexicon_tags_0 },
    { 0xa7bd54f4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc39d54f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbd54fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29d5502, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfd5509, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7d550c, SUC_lexicon_tags_15 },
    { 0xf4fd550d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x399d5510, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x453d5512, SUC_lexicon_tags_30 },
    { 0xdbbd5513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x869d5516, SUC_lexicon_tags_14 },
    { 0x1edd5517, SUC_lexicon_tags_48 },
    { 0x171d5517, SUC_lexicon_tags_15 },
    { 0x259d5517, SUC_lexicon_tags_106 },
    { 0x3d551a, SUC_lexicon_tags_38 },
    { 0xc03d551b, SUC_lexicon_tags_6 },
    { 0x2ebd551a, SUC_lexicon_tags_1 },
    { 0xe3d551c, SUC_lexicon_tags_23 },
    { 0x9e3d551e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde3d5520, SUC_lexicon_tags_32 },
    { 0x829d5521, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54dd5529, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x70bd552c, SUC_lexicon_tags_15 },
    { 0x619d552d, SUC_lexicon_tags_37 },
    { 0x1dbd552e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdc9d5530, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969d5534, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbf3d5536, SUC_lexicon_tags_2 },
    { 0xb63d5537, SUC_lexicon_tags_47 },
    { 0x3c1d5538, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x579d553a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x183d553c, SUC_lexicon_tags_47 },
    { 0x93dd553d, SUC_lexicon_tags_17 },
    { 0x1f3d553e, SUC_lexicon_tags_474 },
    { 0xf97d553e, SUC_lexicon_tags_28 },
    { 0xe43d553e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d5543, SUC_lexicon_tags_47 },
    { 0xb0bd5544, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35d554b, SUC_lexicon_tags_37 },
    { 0xfedd554b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x879d554f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521d5555, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9d555a, SUC_lexicon_tags_31 },
    { 0x4bdd555a, SUC_lexicon_tags_11 },
    { 0x853d555c, SUC_lexicon_tags_16 },
    { 0xab1d555d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x28bd555f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23d5562, SUC_lexicon_tags_51 },
    { 0x30bd5563, SUC_lexicon_tags_2 },
    { 0x2d5d5564, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fd556c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803d5574, SUC_lexicon_tags_26 },
    { 0x3f9d5575, SUC_lexicon_tags_4 },
    { 0xc95d5576, SUC_lexicon_tags_11 },
    { 0xe7d5577, SUC_lexicon_tags_11 },
    { 0x2fdd5578, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbd557f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x127d5582, SUC_lexicon_tags_7 },
    { 0x9d5d5583, SUC_lexicon_tags_0 },
    { 0x94fd5584, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x19bd5587, SUC_lexicon_tags_17 },
    { 0x711d5588, SUC_lexicon_tags_65 },
    { 0xfa7d5589, SUC_lexicon_tags_15 },
    { 0xb0fd558a, SUC_lexicon_tags_35 },
    { 0xfd558b, SUC_lexicon_tags_11 },
    { 0x395d558a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5d558f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9d5592, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dd5597, SUC_lexicon_tags_37 },
    { 0x2e5d5598, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd41d559a, SUC_lexicon_tags_18 },
    { 0x83dd559b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc5dd559d, SUC_lexicon_tags_68 },
    { 0xc8dd559e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d55a1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57d55a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa1d55a8, SUC_lexicon_tags_45 },
    { 0xa51d55a8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x333d55ac, SUC_lexicon_tags_2 },
    { 0xebfd55ad, SUC_lexicon_tags_8 },
    { 0xa69d55ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x375d55b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425d55b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09d55bc, SUC_lexicon_tags_84 },
    { 0xaf7d55bd, SUC_lexicon_tags_7 },
    { 0x617d55be, SUC_lexicon_tags_19 },
    { 0x291d55bf, SUC_lexicon_tags_1 },
    { 0xa73d55c0, SUC_lexicon_tags_68 },
    { 0xbfd55c1, SUC_lexicon_tags_17 },
    { 0x66dd55be, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x60fd55c4, SUC_lexicon_tags_25 },
    { 0x2a5d55c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9d55c8, SUC_lexicon_tags_33 },
    { 0x265d55c8, SUC_lexicon_tags_7 },
    { 0x725d55ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x83fd55cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1d55d1, SUC_lexicon_tags_7 },
    { 0x9cfd55d2, SUC_lexicon_tags_5 },
    { 0x1bfd55d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9d55dd, SUC_lexicon_tags_37 },
    { 0xaabd55dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x56dd55e0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbd55e8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85d55ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebd55f2, SUC_lexicon_tags_14 },
    { 0x507d55f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x943d55f5, SUC_lexicon_tags_7 },
    { 0x4b7d55f5, SUC_lexicon_tags_65 },
    { 0xb77d55f5, SUC_lexicon_tags_15 },
    { 0xc7fd55f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409d55fc, SUC_lexicon_tags_7 },
    { 0xedfd55fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebd5602, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abd5606, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247d560a, SUC_lexicon_tags_7 },
    { 0xecbd560a, SUC_lexicon_tags_70 },
    { 0x35dd560c, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d5611, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11d5616, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d9d5618, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bbd561a, SUC_lexicon_tags_9 },
    { 0x165d561a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x547d561e, SUC_lexicon_tags_13 },
    { 0xd83d561e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bd5624, SUC_lexicon_tags_11 },
    { 0x8c3d5625, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x3c7d5627, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d5629, SUC_lexicon_tags_10 },
    { 0xa13d562a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x83dd562d, SUC_lexicon_tags_11 },
    { 0x6a7d562d, SUC_lexicon_tags_12 },
    { 0xa89d562d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xec9d5631, SUC_lexicon_tags_2 },
    { 0xd03d5631, SUC_lexicon_tags_140 },
    { 0x6fd5633, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf0fd5635, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909d5639, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x933d563b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edd563e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbd5642, SUC_lexicon_tags_23 },
    { 0x6fdd5643, SUC_lexicon_tags_4 },
    { 0x5f7d5644, SUC_lexicon_tags_2 },
    { 0x123d5643, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afd5648, SUC_lexicon_tags_23 },
    { 0xd77d5649, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fd564e, SUC_lexicon_tags_14 },
    { 0x7dfd564e, SUC_lexicon_tags_19 },
    { 0xa27d5650, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2f9d5652, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65d5657, SUC_lexicon_tags_45 },
    { 0x425d5657, SUC_lexicon_tags_15 },
    { 0xe11d5659, SUC_lexicon_tags_10 },
    { 0xa25d5659, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3d5660, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc1bd5662, SUC_lexicon_tags_351 },
    { 0xfa7d5662, SUC_lexicon_tags_47 },
    { 0x41bd5664, SUC_lexicon_tags_7 },
    { 0xf49d5663, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58dd5667, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5d5670, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447d5676, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587d567a, SUC_lexicon_tags_29 },
    { 0x8a1d567b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x883d567d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x929d567f, SUC_lexicon_tags_49 },
    { 0x307d5680, SUC_lexicon_tags_12 },
    { 0x603d5681, SUC_lexicon_tags_39 },
    { 0xa45d5680, SUC_lexicon_tags_33 },
    { 0x5f1d5681, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7d5686, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xee5d5688, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89d5692, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1d5699, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xdcdd569b, SUC_lexicon_tags_7 },
    { 0xc03d569c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9d56a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x29fd56a3, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x3add56a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x157d56a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bd56ad, SUC_lexicon_tags_26 },
    { 0x5a1d56ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63d56b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b5d56b4, SUC_lexicon_tags_37 },
    { 0x6cdd56b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbd56bd, SUC_lexicon_tags_7 },
    { 0xa9fd56bd, SUC_lexicon_tags_2 },
    { 0x461d56bf, SUC_lexicon_tags_35 },
    { 0x33fd56bf, SUC_lexicon_tags_15 },
    { 0xc8dd56c1, SUC_lexicon_tags_39 },
    { 0x493d56bd, SUC_lexicon_tags_79 },
    { 0x26dd56c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x189d56c6, SUC_lexicon_tags_17 },
    { 0xc3bd56c7, SUC_lexicon_tags_8 },
    { 0x937d56c8, SUC_lexicon_tags_47 },
    { 0x905d56c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x287d56cb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x31d56cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x699d56cf, SUC_lexicon_tags_7 },
    { 0xb81d56d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2e1d56d2, SUC_lexicon_tags_38 },
    { 0x14bd56d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49d56d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dd56db, SUC_lexicon_tags_11 },
    { 0xc71d56db, SUC_lexicon_tags_37 },
    { 0x23fd56dd, SUC_lexicon_tags_37 },
    { 0x3b3d56de, SUC_lexicon_tags_0 },
    { 0x49bd56df, SUC_lexicon_tags_37 },
    { 0x7abd56e0, SUC_lexicon_tags_7 },
    { 0x5edd56e1, SUC_lexicon_tags_27 },
    { 0x673d56e2, SUC_lexicon_tags_30 },
    { 0xecfd56de, SUC_lexicon_tags_47 },
    { 0x4b9d56e4, SUC_lexicon_tags_21 },
    { 0x965d56dc, SUC_lexicon_tags_18 },
    { 0x38dd56de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1d56ed, SUC_lexicon_tags_0 },
    { 0x3cfd56ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75d56f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf1d56f4, SUC_lexicon_tags_38 },
    { 0x615d56f4, SUC_lexicon_tags_30 },
    { 0xf79d56f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x377d56f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x69bd56fb, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7d56fe, SUC_lexicon_tags_9 },
    { 0x5e9d56fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffd5702, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393d5706, SUC_lexicon_tags_45 },
    { 0x7add5707, SUC_lexicon_tags_34 },
    { 0x237d5708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09d570b, SUC_lexicon_tags_19 },
    { 0xcedd570c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05d5713, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fd5717, SUC_lexicon_tags_212 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7d571e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x233d5720, SUC_lexicon_tags_17 },
    { 0x851d5721, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217d5726, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597d572a, SUC_lexicon_tags_12 },
    { 0xde3d572a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bd572f, SUC_lexicon_tags_10 },
    { 0x461d5730, SUC_lexicon_tags_15 },
    { 0xa83d5731, SUC_lexicon_tags_16 },
    { 0x61d572f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1d5737, SUC_lexicon_tags_28 },
    { 0x2bfd5738, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fd573b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebd573f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fd5744, SUC_lexicon_tags_11 },
    { 0x6bbd5744, SUC_lexicon_tags_11 },
    { 0x885d5746, SUC_lexicon_tags_65 },
    { 0xa97d5747, SUC_lexicon_tags_26 },
    { 0xc65d5748, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa25d574a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbd574d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x63d574f, SUC_lexicon_tags_9 },
    { 0xfbbd5750, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x18fd5752, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9d5756, SUC_lexicon_tags_3 },
    { 0x531d5757, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6add5759, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9d575c, SUC_lexicon_tags_27 },
    { 0x799d575d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadd5760, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973d5766, SUC_lexicon_tags_26 },
    { 0x929d5767, SUC_lexicon_tags_11 },
    { 0x441d5768, SUC_lexicon_tags_29 },
    { 0x425d5768, SUC_lexicon_tags_37 },
    { 0xc25d5769, SUC_lexicon_tags_15 },
    { 0xcfd576b, SUC_lexicon_tags_37 },
    { 0x641d576c, SUC_lexicon_tags_25 },
    { 0x61dd576c, SUC_lexicon_tags_5 },
    { 0x935d5768, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xdc1d5770, SUC_lexicon_tags_11 },
    { 0x6b3d5771, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa83d5773, SUC_lexicon_tags_19 },
    { 0x309d5774, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x871d5776, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c7d5778, SUC_lexicon_tags_108 },
    { 0x29fd5778, SUC_lexicon_tags_2 },
    { 0x5b3d577a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddd577d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x643d577f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5d5783, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631d5789, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835d578e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65d5791, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11fd5799, SUC_lexicon_tags_1 },
    { 0x6d9d579a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fd579d, SUC_lexicon_tags_1 },
    { 0xd71d579d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715d57a2, SUC_lexicon_tags_141 },
    { 0xcb9d57a2, SUC_lexicon_tags_12 },
    { 0xe6bd57a4, SUC_lexicon_tags_25 },
    { 0xb73d57a3, SUC_lexicon_tags_1 },
    { 0xab9d57a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117d57ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdf7d57ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fd57b2, SUC_lexicon_tags_27 },
    { 0x6f7d57b2, SUC_lexicon_tags_11 },
    { 0xa81d57b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x289d57b6, SUC_lexicon_tags_25 },
    { 0xdbd57b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9d57c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521d57c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bd57cb, SUC_lexicon_tags_1 },
    { 0x829d57cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843d57d1, SUC_lexicon_tags_12 },
    { 0x5a7d57d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa39d57d4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89d57d7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75d57db, SUC_lexicon_tags_38 },
    { 0x267d57dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63d57e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd33d57e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fd57e6, SUC_lexicon_tags_10 },
    { 0x573d57e7, SUC_lexicon_tags_17 },
    { 0xf49d57e8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fd57eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafd57ef, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bd57f3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x773d57f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10bd57f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bd57fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691d5804, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb5d5806, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d580d, SUC_lexicon_tags_11 },
    { 0xc7d580e, SUC_lexicon_tags_17 },
    { 0xae9d580e, SUC_lexicon_tags_27 },
    { 0xe6fd5810, SUC_lexicon_tags_11 },
    { 0xd7bd5811, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x189d5814, SUC_lexicon_tags_8 },
    { 0xf9fd5814, SUC_lexicon_tags_1 },
    { 0x69bd5816, SUC_lexicon_tags_9 },
    { 0x1b3d5817, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61fd5819, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591d5822, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x773d5825, SUC_lexicon_tags_7 },
    { 0xacdd5826, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5d582c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x25d582e, SUC_lexicon_tags_2 },
    { 0x125d582f, SUC_lexicon_tags_27 },
    { 0x85dd582f, SUC_lexicon_tags_10 },
    { 0xd09d582e, SUC_lexicon_tags_44 },
    { 0xbc7d5832, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bd5838, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91d583b, SUC_lexicon_tags_7 },
    { 0x945d583b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf45d583e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17d5841, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3d5844, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x655d5846, SUC_lexicon_tags_17 },
    { 0x57bd5847, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85d584a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5d584d, SUC_lexicon_tags_35 },
    { 0x453d584d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xef7d5850, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7d5853, SUC_lexicon_tags_25 },
    { 0xf5dd5854, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5d5857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fd5860, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d5863, SUC_lexicon_tags_25 },
    { 0x2b1d5864, SUC_lexicon_tags_1 },
    { 0xbf7d5865, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7d5868, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x979d586a, SUC_lexicon_tags_47 },
    { 0x5f3d586b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd19d586d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe63d586f, SUC_lexicon_tags_17 },
    { 0x32fd5870, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbd5876, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529d587b, SUC_lexicon_tags_15 },
    { 0xb89d587c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1b5d587e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391d5886, SUC_lexicon_tags_83 },
    { 0xbadd5887, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa7bd5889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635d588d, SUC_lexicon_tags_33 },
    { 0xe2bd588e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x437d5891, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8b3d5893, SUC_lexicon_tags_7 },
    { 0x587d5894, SUC_lexicon_tags_16 },
    { 0x657d5894, SUC_lexicon_tags_10 },
    { 0x6c3d5896, SUC_lexicon_tags_5 },
    { 0x503d5895, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x60dd589a, SUC_lexicon_tags_17 },
    { 0xbbd589b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dd589e, SUC_lexicon_tags_11 },
    { 0x707d589f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x63d58a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fd58a5, SUC_lexicon_tags_15 },
    { 0xc41d58a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x323d58a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xffbd58aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5d58ad, SUC_lexicon_tags_19 },
    { 0x123d58ae, SUC_lexicon_tags_25 },
    { 0x11d58af, SUC_lexicon_tags_45 },
    { 0x151d58b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4bbd58b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d3d58b4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x7a3d58b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab9d58b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x14bd58ba, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97d58bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x81fd58bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdffd58c1, SUC_lexicon_tags_12 },
    { 0x307d58c2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xdbfd58c4, SUC_lexicon_tags_14 },
    { 0xe21d58c5, SUC_lexicon_tags_11 },
    { 0x919d58c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991d58cc, SUC_lexicon_tags_15 },
    { 0xa1fd58cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ebd58cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63d58d2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaa9d58d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17d58da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3d58dd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x86dd58df, SUC_lexicon_tags_29 },
    { 0xaadd58e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655d58e4, SUC_lexicon_tags_37 },
    { 0x213d58e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc85d58e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x295d58ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581d58ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb15d58f0, SUC_lexicon_tags_10 },
    { 0x359d58f1, SUC_lexicon_tags_30 },
    { 0xe8fd58f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x325d58f4, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x749d58f6, SUC_lexicon_tags_26 },
    { 0x913d58f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbd58fa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abd58fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x93d5900, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573d5905, SUC_lexicon_tags_17 },
    { 0xba5d5906, SUC_lexicon_tags_13 },
    { 0x383d5907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa0dd5909, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeafd590b, SUC_lexicon_tags_15 },
    { 0xa7dd590c, SUC_lexicon_tags_25 },
    { 0xd33d590b, SUC_lexicon_tags_10 },
    { 0x97dd590e, SUC_lexicon_tags_28 },
    { 0x299d590e, SUC_lexicon_tags_11 },
    { 0xbcfd590f, SUC_lexicon_tags_60 },
    { 0x35bd5911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd29d5913, SUC_lexicon_tags_15 },
    { 0xe15d5914, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dd5917, SUC_lexicon_tags_38 },
    { 0x167d5918, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bd591b, SUC_lexicon_tags_5 },
    { 0x38bd591c, SUC_lexicon_tags_12 },
    { 0xb9bd591b, SUC_lexicon_tags_37 },
    { 0xbdfd591e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77d5921, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x983d5924, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x645d5927, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x3ddd5929, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfd592d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x295d592f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x88dd5932, SUC_lexicon_tags_37 },
    { 0xc29d5932, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x949d5936, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3d593a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85dd593c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5d593f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbd5946, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bd594a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d594d, SUC_lexicon_tags_2 },
    { 0xd9bd594e, SUC_lexicon_tags_37 },
    { 0x3e5d594e, SUC_lexicon_tags_10 },
    { 0x4c3d594d, SUC_lexicon_tags_31 },
    { 0xa77d5951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x309d5953, SUC_lexicon_tags_29 },
    { 0x6fbd5953, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87d595c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1d595f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbd5963, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb21d5965, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbe9d5967, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x779d5969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x441d596b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7a1d596d, SUC_lexicon_tags_48 },
    { 0xae1d596d, SUC_lexicon_tags_30 },
    { 0x3efd596f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51d5973, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9e5d5975, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xab3d5977, SUC_lexicon_tags_11 },
    { 0x51fd5978, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45d597e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x415d5980, SUC_lexicon_tags_29 },
    { 0x54dd5980, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d5986, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x431d5989, SUC_lexicon_tags_27 },
    { 0x759d598a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x609d598c, SUC_lexicon_tags_33 },
    { 0x53dd598d, SUC_lexicon_tags_37 },
    { 0xfbfd598d, SUC_lexicon_tags_47 },
    { 0x92bd598f, SUC_lexicon_tags_124 },
    { 0x21fd598e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe01d5992, SUC_lexicon_tags_7 },
    { 0x9bfd5992, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8b9d5995, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfbfd5997, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2ddd5999, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1d599e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfd59a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863d59aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305d59ae, SUC_lexicon_tags_25 },
    { 0xc2bd59af, SUC_lexicon_tags_37 },
    { 0xdefd59b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499d59b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519d59bc, SUC_lexicon_tags_43 },
    { 0xac7d59bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdb7d59bf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57d59c4, SUC_lexicon_tags_17 },
    { 0xe59d59c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1d59c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201d59cf, SUC_lexicon_tags_15 },
    { 0xce7d59d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8d1d59d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x191d59d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x863d59d8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dd59dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd59e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3abd59e6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237d59ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9d59f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfd59f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69d59f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbbd59fa, SUC_lexicon_tags_17 },
    { 0x965d59fb, SUC_lexicon_tags_17 },
    { 0x2f3d59fc, SUC_lexicon_tags_11 },
    { 0x219d59fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7d5a02, SUC_lexicon_tags_37 },
    { 0xe01d5a02, SUC_lexicon_tags_25 },
    { 0x2afd5a04, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x673d5a07, SUC_lexicon_tags_49 },
    { 0x6f5d5a08, SUC_lexicon_tags_46 },
    { 0xbfdd5a09, SUC_lexicon_tags_37 },
    { 0xbc9d5a09, SUC_lexicon_tags_60 },
    { 0xb77d5a0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcebd5a0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5d5a13, SUC_lexicon_tags_49 },
    { 0x68dd5a13, SUC_lexicon_tags_28 },
    { 0xe49d5a13, SUC_lexicon_tags_37 },
    { 0x9d7d5a13, SUC_lexicon_tags_4 },
    { 0x269d5a15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5d5a1a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd99d5a1c, SUC_lexicon_tags_7 },
    { 0x799d5a1d, SUC_lexicon_tags_38 },
    { 0xb97d5a1d, SUC_lexicon_tags_33 },
    { 0x5b9d5a1f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3d5a23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x475d5a25, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07d5a29, SUC_lexicon_tags_49 },
    { 0x861d5a29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9d5a2d, SUC_lexicon_tags_26 },
    { 0x4e3d5a2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327d5a32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x287d5a34, SUC_lexicon_tags_4 },
    { 0xb1d5a35, SUC_lexicon_tags_11 },
    { 0x6fbd5a36, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1d5a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7d5a3c, SUC_lexicon_tags_11 },
    { 0x231d5a3d, SUC_lexicon_tags_65 },
    { 0x255d5a3e, SUC_lexicon_tags_15 },
    { 0x1dfd5a3f, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd5a42, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xf5bd5a44, SUC_lexicon_tags_37 },
    { 0xee5d5a44, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7d5a48, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3d5a51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903d5a56, SUC_lexicon_tags_2 },
    { 0xa9dd5a57, SUC_lexicon_tags_4 },
    { 0x91dd5a57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x765d5a5a, SUC_lexicon_tags_25 },
    { 0x287d5a5a, SUC_lexicon_tags_17 },
    { 0xdf1d5a5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3d5a5f, SUC_lexicon_tags_7 },
    { 0x4e5d5a60, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x333d5a63, SUC_lexicon_tags_11 },
    { 0xa9dd5a64, SUC_lexicon_tags_14 },
    { 0x9ebd5a64, SUC_lexicon_tags_27 },
    { 0xd93d5a66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5d5a6b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x78fd5a6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5d5a70, SUC_lexicon_tags_37 },
    { 0x8f7d5a71, SUC_lexicon_tags_82 },
    { 0x4afd5a72, SUC_lexicon_tags_37 },
    { 0xd81d5a72, SUC_lexicon_tags_14 },
    { 0x2c5d5a73, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d5a77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d5a80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3d5a8d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd5a99, SUC_lexicon_tags_25 },
    { 0x371d5a9a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e7d5a9c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d5aa0, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd5aa7, SUC_lexicon_tags_25 },
    { 0xf4fd5aa7, SUC_lexicon_tags_23 },
    { 0x7b5d5aa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9d5ab1, SUC_lexicon_tags_25 },
    { 0xcb7d5ab1, SUC_lexicon_tags_2 },
    { 0xe75d5ab3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271d5ab7, SUC_lexicon_tags_25 },
    { 0x5e5d5ab8, SUC_lexicon_tags_49 },
    { 0xbffd5ab8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x383d5abb, SUC_lexicon_tags_30 },
    { 0x499d5abc, SUC_lexicon_tags_39 },
    { 0x323d5abd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb37d5abf, SUC_lexicon_tags_37 },
    { 0x2dbd5ac0, SUC_lexicon_tags_44 },
    { 0xc21d5ac1, SUC_lexicon_tags_11 },
    { 0x165d5ac2, SUC_lexicon_tags_79 },
    { 0xc1dd5ac2, SUC_lexicon_tags_7 },
    { 0xae5d5ac3, SUC_lexicon_tags_11 },
    { 0x5fbd5ac5, SUC_lexicon_tags_31 },
    { 0x961d5ac6, SUC_lexicon_tags_1 },
    { 0x905d5ac5, SUC_lexicon_tags_14 },
    { 0xc77d5ac7, SUC_lexicon_tags_14 },
    { 0x4a1d5ac1, SUC_lexicon_tags_47 },
    { 0x243d5aca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529d5acf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dd5ad4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9d5ad7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdd5adc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3d5ae1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7d5ae5, SUC_lexicon_tags_38 },
    { 0x61fd5ae6, SUC_lexicon_tags_16 },
    { 0xc39d5ae6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4efd5ae9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x40dd5aeb, SUC_lexicon_tags_38 },
    { 0x413d5aec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1d3d5aee, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x405d5af0, SUC_lexicon_tags_1 },
    { 0x91fd5af0, SUC_lexicon_tags_19 },
    { 0x1c3d5af0, SUC_lexicon_tags_38 },
    { 0xd1dd5af3, SUC_lexicon_tags_104 },
    { 0x239d5af3, SUC_lexicon_tags_33 },
    { 0xf9d5af4, SUC_lexicon_tags_27 },
    { 0x18fd5af4, SUC_lexicon_tags_25 },
    { 0xe27d5af7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e7d5af9, SUC_lexicon_tags_28 },
    { 0xe9bd5afa, SUC_lexicon_tags_49 },
    { 0xb05d5afa, SUC_lexicon_tags_11 },
    { 0xff3d5afa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x83fd5aff, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x441d5b01, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5d5b03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759d5b09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffd5b0f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cdd5b11, SUC_lexicon_tags_15 },
    { 0x439d5b11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dd5b16, SUC_lexicon_tags_25 },
    { 0x94bd5b17, SUC_lexicon_tags_49 },
    { 0xf95d5b18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x925d5b1b, SUC_lexicon_tags_15 },
    { 0xb23d5b1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd5b21, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadd5b26, SUC_lexicon_tags_12 },
    { 0xc59d5b27, SUC_lexicon_tags_4 },
    { 0xb9d5b28, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dd5b2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343d5b30, SUC_lexicon_tags_11 },
    { 0x29bd5b30, SUC_lexicon_tags_131 },
    { 0x497d5b30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6f7d5b34, SUC_lexicon_tags_30 },
    { 0x173d5b35, SUC_lexicon_tags_6 },
    { 0x11d5b36, SUC_lexicon_tags_0 },
    { 0x657d5b37, SUC_lexicon_tags_14 },
    { 0x1e7d5b38, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xefd5b3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35d5b3f, SUC_lexicon_tags_10 },
    { 0xc2bd5b40, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4fdd5b42, SUC_lexicon_tags_25 },
    { 0xe3fd5b43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x899d5b46, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x613d5b48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5d5b51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35d5b56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bd5b63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7d5b68, SUC_lexicon_tags_45 },
    { 0x9fdd5b68, SUC_lexicon_tags_25 },
    { 0xfddd5b6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa91d5b6c, SUC_lexicon_tags_8 },
    { 0x8fdd5b6d, SUC_lexicon_tags_37 },
    { 0x3d9d5b6d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x86bd5b70, SUC_lexicon_tags_2 },
    { 0xd63d5b70, SUC_lexicon_tags_26 },
    { 0xb81d5b70, SUC_lexicon_tags_1 },
    { 0x7a7d5b73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81dd5b75, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadd5b7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5d1d5b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x31d5b80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf9dd5b82, SUC_lexicon_tags_48 },
    { 0x2cbd5b83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b5d5b85, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681d5b8d, SUC_lexicon_tags_9 },
    { 0x989d5b8e, SUC_lexicon_tags_4 },
    { 0x55bd5b8f, SUC_lexicon_tags_17 },
    { 0x1dbd5b8f, SUC_lexicon_tags_15 },
    { 0xe97d5b91, SUC_lexicon_tags_33 },
    { 0x56bd5b92, SUC_lexicon_tags_27 },
    { 0xa63d5b93, SUC_lexicon_tags_7 },
    { 0x71fd5b94, SUC_lexicon_tags_17 },
    { 0xcdd5b95, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x12fd5b97, SUC_lexicon_tags_15 },
    { 0x15bd5b98, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163d5b9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1d5ba0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x91dd5ba3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769d5bab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdefd5bad, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbd5bb0, SUC_lexicon_tags_30 },
    { 0x669d5bb0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x94fd5bb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb0fd5bb5, SUC_lexicon_tags_17 },
    { 0xd87d5bb6, SUC_lexicon_tags_19 },
    { 0x103d5bb7, SUC_lexicon_tags_9 },
    { 0xbd3d5bb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5d5bbb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12dd5bbd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x90fd5bbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61d5bc2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x331d5bc4, SUC_lexicon_tags_25 },
    { 0x735d5bc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x563d5bc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1d5bd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95fd5bd2, SUC_lexicon_tags_17 },
    { 0x9dd5bd3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97d5bd6, SUC_lexicon_tags_17 },
    { 0xe81d5bd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x363d5bda, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fd5be4, SUC_lexicon_tags_37 },
    { 0xe15d5be5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305d5be9, SUC_lexicon_tags_33 },
    { 0xe0bd5be9, SUC_lexicon_tags_4 },
    { 0x389d5beb, SUC_lexicon_tags_2 },
    { 0x179d5bec, SUC_lexicon_tags_7 },
    { 0xc8dd5bed, SUC_lexicon_tags_37 },
    { 0x17bd5bee, SUC_lexicon_tags_25 },
    { 0xfd7d5bee, SUC_lexicon_tags_37 },
    { 0xbf3d5bf0, SUC_lexicon_tags_82 },
    { 0x281d5bf1, SUC_lexicon_tags_37 },
    { 0x6fdd5bf2, SUC_lexicon_tags_30 },
    { 0xbe9d5bf3, SUC_lexicon_tags_13 },
    { 0xe1bd5bf4, SUC_lexicon_tags_8 },
    { 0xaf1d5bf4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3d5bf8, SUC_lexicon_tags_49 },
    { 0x167d5bf8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6c9d5bfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3fd5bfd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963d5c01, SUC_lexicon_tags_77 },
    { 0xd37d5c01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x209d5c04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397d5c08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1d5c0b, SUC_lexicon_tags_17 },
    { 0x129d5c0c, SUC_lexicon_tags_28 },
    { 0xb4bd5c0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bd5c10, SUC_lexicon_tags_47 },
    { 0xfbd5c10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac7d5c13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173d5c17, SUC_lexicon_tags_46 },
    { 0xc0bd5c18, SUC_lexicon_tags_68 },
    { 0xb39d5c18, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fd5c1c, SUC_lexicon_tags_9 },
    { 0x70dd5c1d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb29d5c1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5add5c24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495d5c29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x933d5c2c, SUC_lexicon_tags_1 },
    { 0x6b1d5c2d, SUC_lexicon_tags_5 },
    { 0x315d5c2e, SUC_lexicon_tags_29 },
    { 0x617d5c2e, SUC_lexicon_tags_11 },
    { 0x295d5c2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fd5c34, SUC_lexicon_tags_136 },
    { 0xf3d5c35, SUC_lexicon_tags_201 },
    { 0xc55d5c36, SUC_lexicon_tags_15 },
    { 0x9e7d5c37, SUC_lexicon_tags_25 },
    { 0xfc3d5c37, SUC_lexicon_tags_45 },
    { 0x873d5c37, SUC_lexicon_tags_27 },
    { 0x99fd5c37, SUC_lexicon_tags_27 },
    { 0x38dd5c3b, SUC_lexicon_tags_47 },
    { 0xc7fd5c37, SUC_lexicon_tags_254 },
    { 0xbb3d5c35, SUC_lexicon_tags_39 },
    { 0x8f9d5c3e, SUC_lexicon_tags_25 },
    { 0xb27d5c3e, SUC_lexicon_tags_9 },
    { 0x6cbd5c3e, SUC_lexicon_tags_15 },
    { 0xdedd5c3e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd8fd5c43, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bd5c47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbd5c4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d5c51, SUC_lexicon_tags_35 },
    { 0xca7d5c52, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa51d5c54, SUC_lexicon_tags_37 },
    { 0x26fd5c55, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa71d5c57, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebd5c5b, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4add5c67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1cdd5c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43d5c6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5d5c71, SUC_lexicon_tags_26 },
    { 0xaf9d5c72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd5d5c74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf51d5c76, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1bbd5c78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9bd5c7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4dbd5c7c, SUC_lexicon_tags_26 },
    { 0x615d5c7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edd5c81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f3d5c83, SUC_lexicon_tags_17 },
    { 0x14fd5c84, SUC_lexicon_tags_30 },
    { 0x765d5c85, SUC_lexicon_tags_37 },
    { 0x741d5c86, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bd5c93, SUC_lexicon_tags_6 },
    { 0xb37d5c94, SUC_lexicon_tags_46 },
    { 0x815d5c93, SUC_lexicon_tags_68 },
    { 0xa4fd5c96, SUC_lexicon_tags_27 },
    { 0xba7d5c97, SUC_lexicon_tags_2 },
    { 0x7e1d5c97, SUC_lexicon_tags_47 },
    { 0x323d5c98, SUC_lexicon_tags_60 },
    { 0xf87d5c9a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2e1d5c9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x993d5c9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x127d5ca0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x291d5ca2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997d5ca6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257d5cb2, SUC_lexicon_tags_1 },
    { 0xf21d5cb3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bd5cb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfd5cba, SUC_lexicon_tags_22 },
    { 0x963d5cba, SUC_lexicon_tags_12 },
    { 0x839d5cbb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x65bd5cbf, SUC_lexicon_tags_25 },
    { 0x87dd5cc0, SUC_lexicon_tags_23 },
    { 0x93d5cc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639d5cc6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xea7d5cc8, SUC_lexicon_tags_11 },
    { 0xa93d5cc9, SUC_lexicon_tags_37 },
    { 0xf4bd5cca, SUC_lexicon_tags_11 },
    { 0xcbfd5cc9, SUC_lexicon_tags_60 },
    { 0xc99d5cca, SUC_lexicon_tags_11 },
    { 0x459d5ccd, SUC_lexicon_tags_12 },
    { 0x26dd5ccd, SUC_lexicon_tags_22 },
    { 0x91bd5ccb, SUC_lexicon_tags_7 },
    { 0xd97d5ccf, SUC_lexicon_tags_7 },
    { 0x363d5cd1, SUC_lexicon_tags_43 },
    { 0x661d5cd2, SUC_lexicon_tags_7 },
    { 0xd33d5cd2, SUC_lexicon_tags_12 },
    { 0x2b5d5cd4, SUC_lexicon_tags_28 },
    { 0xa2fd5ccb, SUC_lexicon_tags_48 },
    { 0x357d5cd6, SUC_lexicon_tags_37 },
    { 0x5c3d5cd6, SUC_lexicon_tags_37 },
    { 0x54dd5cd8, SUC_lexicon_tags_65 },
    { 0xff1d5cd9, SUC_lexicon_tags_47 },
    { 0x807d5cd1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74fd5cde, SUC_lexicon_tags_25 },
    { 0x691d5cdf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9d5ce4, SUC_lexicon_tags_19 },
    { 0x2f1d5ce5, SUC_lexicon_tags_26 },
    { 0x9abd5ce6, SUC_lexicon_tags_11 },
    { 0xdf5d5ce6, SUC_lexicon_tags_25 },
    { 0x8ddd5ce6, SUC_lexicon_tags_7 },
    { 0xecfd5ce9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x93fd5ceb, SUC_lexicon_tags_37 },
    { 0x789d5ceb, SUC_lexicon_tags_60 },
    { 0x5b3d5ceb, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0x7e9d5cef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67fd5cf1, SUC_lexicon_tags_17 },
    { 0x5edd5cf2, SUC_lexicon_tags_25 },
    { 0x81dd5cf3, SUC_lexicon_tags_79 },
    { 0x431d5cf3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x735d5cf7, SUC_lexicon_tags_11 },
    { 0x35bd5cf8, SUC_lexicon_tags_25 },
    { 0xdd5d5cf9, SUC_lexicon_tags_26 },
    { 0xf8bd5cf7, SUC_lexicon_tags_1 },
    { 0xf33d5cf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8f7d5cfd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d7d5cff, SUC_lexicon_tags_106 },
    { 0x259d5cff, SUC_lexicon_tags_2 },
    { 0x70bd5d01, SUC_lexicon_tags_15 },
    { 0xbbbd5d01, SUC_lexicon_tags_11 },
    { 0x5e5d5d02, SUC_lexicon_tags_63 },
    { 0xbdbd5d04, SUC_lexicon_tags_57 },
    { 0xa07d5d04, SUC_lexicon_tags_15 },
    { 0x96bd5d00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bd5d0c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dd5d12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9d5d14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x547d5d16, SUC_lexicon_tags_26 },
    { 0x231d5d16, SUC_lexicon_tags_33 },
    { 0x277d5d18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8fd5d1a, SUC_lexicon_tags_15 },
    { 0xf41d5d1b, SUC_lexicon_tags_8 },
    { 0x979d5d1c, SUC_lexicon_tags_14 },
    { 0xc7bd5d1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x927d5d1f, SUC_lexicon_tags_17 },
    { 0x12bd5d20, SUC_lexicon_tags_21 },
    { 0x9b9d5d20, SUC_lexicon_tags_30 },
    { 0x295d5d22, SUC_lexicon_tags_37 },
    { 0x8afd5d23, SUC_lexicon_tags_38 },
    { 0x471d5d23, SUC_lexicon_tags_1 },
    { 0xc85d5d25, SUC_lexicon_tags_388 },
    { 0x793d5d24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afd5d29, SUC_lexicon_tags_27 },
    { 0x70bd5d29, SUC_lexicon_tags_68 },
    { 0x15d5d2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffd5d30, SUC_lexicon_tags_25 },
    { 0x1e7d5d31, SUC_lexicon_tags_148 },
    { 0xc9dd5d31, SUC_lexicon_tags_0 },
    { 0xbbdd5d30, SUC_lexicon_tags_25 },
    { 0xb97d5d34, SUC_lexicon_tags_17 },
    { 0x711d5d31, SUC_lexicon_tags_27 },
    { 0xc37d5d31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbd5d3e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49d5d41, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3d5d45, SUC_lexicon_tags_1 },
    { 0x7b7d5d45, SUC_lexicon_tags_47 },
    { 0x7add5d47, SUC_lexicon_tags_11 },
    { 0xdc9d5d47, SUC_lexicon_tags_37 },
    { 0x469d5d45, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0x997d5d4b, SUC_lexicon_tags_45 },
    { 0x563d5d4c, SUC_lexicon_tags_14 },
    { 0x2b1d5d4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5d5d51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8bd5d53, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345d5d57, SUC_lexicon_tags_7 },
    { 0x83fd5d58, SUC_lexicon_tags_2 },
    { 0xf15d5d59, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109d5d5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6dd5d60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x993d5d62, SUC_lexicon_tags_17 },
    { 0x4a7d5d63, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x841d5d65, SUC_lexicon_tags_29 },
    { 0x3a1d5d65, SUC_lexicon_tags_27 },
    { 0x7abd5d66, SUC_lexicon_tags_11 },
    { 0x6fdd5d66, SUC_lexicon_tags_49 },
    { 0x44bd5d69, SUC_lexicon_tags_2 },
    { 0x60dd5d65, SUC_lexicon_tags_8 },
    { 0x8efd5d6a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x553d5d6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b5d5d70, SUC_lexicon_tags_60 },
    { 0x4c9d5d70, SUC_lexicon_tags_3 },
    { 0x1edd5d72, SUC_lexicon_tags_35 },
    { 0x1e9d5d73, SUC_lexicon_tags_12 },
    { 0xdd1d5d71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83fd5d76, SUC_lexicon_tags_14 },
    { 0x8e3d5d76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47d5d7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dd5d85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb1fd5d87, SUC_lexicon_tags_11 },
    { 0x3ffd5d87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x661d5d8b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8ebd5d8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x417d5d8f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bd5d97, SUC_lexicon_tags_14 },
    { 0x677d5d97, SUC_lexicon_tags_7 },
    { 0xaedd5d99, SUC_lexicon_tags_0 },
    { 0x1e3d5d99, SUC_lexicon_tags_2 },
    { 0x215d5d9b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x96bd5d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x925d5da0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd75d5da2, SUC_lexicon_tags_1 },
    { 0xe69d5da3, SUC_lexicon_tags_2 },
    { 0x745d5da4, SUC_lexicon_tags_37 },
    { 0xc4fd5da5, SUC_lexicon_tags_46 },
    { 0x7afd5da6, SUC_lexicon_tags_31 },
    { 0xc75d5da3, SUC_lexicon_tags_11 },
    { 0xe3dd5da7, SUC_lexicon_tags_82 },
    { 0x813d5da9, SUC_lexicon_tags_23 },
    { 0x3dd5daa, SUC_lexicon_tags_33 },
    { 0x1a5d5daa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf43d5dad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9d5db0, SUC_lexicon_tags_33 },
    { 0x3e7d5db1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x707d5db3, SUC_lexicon_tags_38 },
    { 0xcd7d5db3, SUC_lexicon_tags_37 },
    { 0xc2dd5db5, SUC_lexicon_tags_7 },
    { 0xa8fd5db3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x709d5db8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xce3d5dba, SUC_lexicon_tags_21 },
    { 0x211d5dbb, SUC_lexicon_tags_19 },
    { 0xe2bd5dbc, SUC_lexicon_tags_30 },
    { 0x3abd5dbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfd5dc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefd5dc5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa15d5dc7, SUC_lexicon_tags_66 },
    { 0x67fd5dc7, SUC_lexicon_tags_19 },
    { 0xa1dd5dc9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xadfd5dcc, SUC_lexicon_tags_49 },
    { 0xf87d5dcc, SUC_lexicon_tags_2 },
    { 0x3b5d5dcc, SUC_lexicon_tags_33 },
    { 0x9e3d5dcd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f9d5dd1, SUC_lexicon_tags_29 },
    { 0x63dd5dd1, SUC_lexicon_tags_27 },
    { 0x23fd5dd3, SUC_lexicon_tags_19 },
    { 0x7f9d5dd4, SUC_lexicon_tags_0 },
    { 0xdcdd5dd5, SUC_lexicon_tags_2 },
    { 0x14fd5dd5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf57d5dd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3d5ddb, SUC_lexicon_tags_38 },
    { 0x7cdd5ddb, SUC_lexicon_tags_9 },
    { 0x3efd5ddd, SUC_lexicon_tags_11 },
    { 0xe0bd5ddb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9d5de5, SUC_lexicon_tags_25 },
    { 0x769d5de6, SUC_lexicon_tags_60 },
    { 0x765d5de7, SUC_lexicon_tags_17 },
    { 0x3f5d5de8, SUC_lexicon_tags_37 },
    { 0x107d5de9, SUC_lexicon_tags_45 },
    { 0x359d5de5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89d5ded, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27d5df3, SUC_lexicon_tags_25 },
    { 0xba5d5df4, SUC_lexicon_tags_11 },
    { 0x113d5df4, SUC_lexicon_tags_61 },
    { 0x80fd5df4, SUC_lexicon_tags_2 },
    { 0xa59d5df4, SUC_lexicon_tags_27 },
    { 0xf33d5df8, SUC_lexicon_tags_16 },
    { 0xcd1d5df9, SUC_lexicon_tags_87 },
    { 0x40bd5dfa, SUC_lexicon_tags_2 },
    { 0x93bd5dfa, SUC_lexicon_tags_2 },
    { 0x2ebd5dfc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x925d5dff, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7d5e09, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d5e0c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3a7d5e0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdd5e11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbd5e14, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb97d5e16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273d5e1a, SUC_lexicon_tags_9 },
    { 0x967d5e1a, SUC_lexicon_tags_29 },
    { 0xfa9d5e1c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fd5e1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dd5e23, SUC_lexicon_tags_21 },
    { 0xcafd5e24, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x25d5e26, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719d5e2b, SUC_lexicon_tags_17 },
    { 0xc57d5e2c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcffd5e2e, SUC_lexicon_tags_6 },
    { 0x26fd5e2e, SUC_lexicon_tags_16 },
    { 0x9afd5e2f, SUC_lexicon_tags_2 },
    { 0xf3dd5e31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fd5e3a, SUC_lexicon_tags_15 },
    { 0x1cbd5e3a, SUC_lexicon_tags_17 },
    { 0x54dd5e3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537d5e40, SUC_lexicon_tags_11 },
    { 0x557d5e41, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd5e44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699d5e4d, SUC_lexicon_tags_17 },
    { 0xbb1d5e4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdd5e51, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47d5e56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xedfd5e58, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3d5e5c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x723d5e5e, SUC_lexicon_tags_17 },
    { 0x6f7d5e5e, SUC_lexicon_tags_31 },
    { 0x485d5e60, SUC_lexicon_tags_26 },
    { 0x599d5e5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5d5e64, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe73d5e66, SUC_lexicon_tags_2 },
    { 0x9b1d5e67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163d5e6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5d5e72, SUC_lexicon_tags_15 },
    { 0x5b5d5e73, SUC_lexicon_tags_56 },
    { 0xf89d5e74, SUC_lexicon_tags_39 },
    { 0xb09d5e73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x369d5e77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfd5e7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x54fd5e7d, SUC_lexicon_tags_29 },
    { 0x605d5e7e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdd5e81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2a1d5e83, SUC_lexicon_tags_14 },
    { 0x26dd5e84, SUC_lexicon_tags_59 },
    { 0x3a7d5e85, SUC_lexicon_tags_65 },
    { 0x8f5d5e86, SUC_lexicon_tags_45 },
    { 0x60bd5e87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdd5e8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdd5e90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x44bd5e92, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bd5e95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c5d5e97, SUC_lexicon_tags_12 },
    { 0x15dd5e97, SUC_lexicon_tags_17 },
    { 0xa5bd5e97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699d5e9d, SUC_lexicon_tags_15 },
    { 0xb1d5e9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45d5ea2, SUC_lexicon_tags_37 },
    { 0x8cfd5ea3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bd5ea8, SUC_lexicon_tags_0 },
    { 0x929d5ea8, SUC_lexicon_tags_8 },
    { 0xf8fd5eaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffd5eb0, SUC_lexicon_tags_49 },
    { 0xd9bd5eb1, SUC_lexicon_tags_38 },
    { 0x9b7d5eb2, SUC_lexicon_tags_35 },
    { 0x3dbd5eb1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf0dd5eb5, SUC_lexicon_tags_85 },
    { 0x189d5eb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbd5ec0, SUC_lexicon_tags_282 },
    { 0x2e5d5ec1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3d5ec5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77d5ec9, SUC_lexicon_tags_11 },
    { 0xb1bd5ec9, SUC_lexicon_tags_9 },
    { 0x163d5eca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x153d5ecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a7d5ecf, SUC_lexicon_tags_17 },
    { 0x899d5ecf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97dd5ed2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03d5ed6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc51d5ed8, SUC_lexicon_tags_6 },
    { 0x631d5ed8, SUC_lexicon_tags_27 },
    { 0x253d5ed8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc59d5edc, SUC_lexicon_tags_47 },
    { 0xf2dd5edd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95d5ee1, SUC_lexicon_tags_27 },
    { 0x177d5ee1, SUC_lexicon_tags_4 },
    { 0x543d5ee3, SUC_lexicon_tags_12 },
    { 0xefbd5ee3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x393d5ee6, SUC_lexicon_tags_11 },
    { 0x31bd5ee7, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xc9dd5ee9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23d5eec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dd5ef0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95d5ef8, SUC_lexicon_tags_87 },
    { 0x1edd5ef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50fd5efc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x13bd5eff, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafd5f06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfd5f0f, SUC_lexicon_tags_46 },
    { 0x971d5f0f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9d5f13, SUC_lexicon_tags_37 },
    { 0x759d5f14, SUC_lexicon_tags_8 },
    { 0x2add5f14, SUC_lexicon_tags_1 },
    { 0xd47d5f16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afd5f21, SUC_lexicon_tags_3 },
    { 0x5a9d5f21, SUC_lexicon_tags_2 },
    { 0x451d5f21, SUC_lexicon_tags_12 },
    { 0xa77d5f21, SUC_lexicon_tags_0 },
    { 0xa85d5f25, SUC_lexicon_tags_7 },
    { 0x311d5f23, SUC_lexicon_tags_5 },
    { 0x911d5f24, SUC_lexicon_tags_25 },
    { 0x72dd5f28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85d5f2b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bd5f2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1fd5f30, SUC_lexicon_tags_15 },
    { 0x6a5d5f30, SUC_lexicon_tags_16 },
    { 0xf71d5f32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdd5f35, SUC_lexicon_tags_82 },
    { 0x8afd5f35, SUC_lexicon_tags_25 },
    { 0xc5fd5f37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5a1d5f39, SUC_lexicon_tags_15 },
    { 0xfedd5f3a, SUC_lexicon_tags_17 },
    { 0xbcdd5f3b, SUC_lexicon_tags_11 },
    { 0x66dd5f3b, SUC_lexicon_tags_6 },
    { 0x80dd5f3d, SUC_lexicon_tags_19 },
    { 0x12bd5f3e, SUC_lexicon_tags_2 },
    { 0x9ffd5f3b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45d5f45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdd5f4e, SUC_lexicon_tags_15 },
    { 0x4c7d5f4f, SUC_lexicon_tags_151 },
    { 0xc0fd5f4e, SUC_lexicon_tags_9 },
    { 0xcf3d5f50, SUC_lexicon_tags_21 },
    { 0xe45d5f52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ddd5f54, SUC_lexicon_tags_31 },
    { 0x5f1d5f55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x531d5f57, SUC_lexicon_tags_31 },
    { 0xba7d5f58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99d5f5f, SUC_lexicon_tags_37 },
    { 0xe91d5f60, SUC_lexicon_tags_37 },
    { 0x5a9d5f61, SUC_lexicon_tags_47 },
    { 0xe89d5f62, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3d5f68, SUC_lexicon_tags_418 },
    { 0xb2bd5f68, SUC_lexicon_tags_16 },
    { 0x555d5f6a, SUC_lexicon_tags_2 },
    { 0xff5d5f69, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fd5f6e, SUC_lexicon_tags_17 },
    { 0x73d5f6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x505d5f72, SUC_lexicon_tags_17 },
    { 0xe5dd5f73, SUC_lexicon_tags_15 },
    { 0x5c5d5f74, SUC_lexicon_tags_79 },
    { 0xa63d5f74, SUC_lexicon_tags_37 },
    { 0xcdfd5f75, SUC_lexicon_tags_10 },
    { 0x45fd5f77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75d5f7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969d5f82, SUC_lexicon_tags_41 },
    { 0xc89d5f83, SUC_lexicon_tags_30 },
    { 0x92dd5f82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1a3d5f86, SUC_lexicon_tags_14 },
    { 0x183d5f87, SUC_lexicon_tags_56 },
    { 0x6d3d5f86, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7d5f8d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe93d5f8f, SUC_lexicon_tags_11 },
    { 0x84bd5f90, SUC_lexicon_tags_9 },
    { 0xdb1d5f91, SUC_lexicon_tags_2 },
    { 0x771d5f92, SUC_lexicon_tags_82 },
    { 0x3ffd5f90, SUC_lexicon_tags_11 },
    { 0xabd5f91, SUC_lexicon_tags_10 },
    { 0xac9d5f8f, SUC_lexicon_tags_37 },
    { 0x675d5f96, SUC_lexicon_tags_26 },
    { 0x595d5f97, SUC_lexicon_tags_11 },
    { 0x84fd5f98, SUC_lexicon_tags_40 },
    { 0x6a5d5f99, SUC_lexicon_tags_9 },
    { 0xeedd5f98, SUC_lexicon_tags_61 },
    { 0xdedd5f9b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbd5f9e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x87fd5fa0, SUC_lexicon_tags_23 },
    { 0x39bd5fa0, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddd5fa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55d5fad, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xec9d5faf, SUC_lexicon_tags_23 },
    { 0xef5d5faf, SUC_lexicon_tags_48 },
    { 0xa2fd5faf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x485d5fb3, SUC_lexicon_tags_65 },
    { 0x5e9d5fb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abd5fb9, SUC_lexicon_tags_15 },
    { 0x37fd5fb9, SUC_lexicon_tags_25 },
    { 0x2b7d5fba, SUC_lexicon_tags_7 },
    { 0x1cfd5fbc, SUC_lexicon_tags_8 },
    { 0xd1bd5fbd, SUC_lexicon_tags_31 },
    { 0x2f1d5fbe, SUC_lexicon_tags_23 },
    { 0x9f5d5fbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefd5fc3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79d5fc6, SUC_lexicon_tags_27 },
    { 0x3abd5fc6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x5fbd5fc9, SUC_lexicon_tags_12 },
    { 0x325d5fc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa21d5fcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481d5fd1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fd5fd4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5a5d5fd6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbd9d5fd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bd5fe0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef3d5fe2, SUC_lexicon_tags_0 },
    { 0xcabd5fe3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1d5fe8, SUC_lexicon_tags_11 },
    { 0x69bd5fe9, SUC_lexicon_tags_3 },
    { 0x9edd5fea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe0bd5fec, SUC_lexicon_tags_11 },
    { 0xc7dd5fed, SUC_lexicon_tags_25 },
    { 0xbebd5fed, SUC_lexicon_tags_25 },
    { 0xc15d5fef, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x60dd5ff1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa67d5ff3, SUC_lexicon_tags_25 },
    { 0x90dd5ff3, SUC_lexicon_tags_14 },
    { 0x671d5ff4, SUC_lexicon_tags_26 },
    { 0x721d5ff6, SUC_lexicon_tags_15 },
    { 0x1abd5ff7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1d5ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569d6001, SUC_lexicon_tags_37 },
    { 0xbf1d6002, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39d6005, SUC_lexicon_tags_14 },
    { 0xf3fd6006, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801d600c, SUC_lexicon_tags_8 },
    { 0x8d3d600d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9d6011, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459d6015, SUC_lexicon_tags_1 },
    { 0xe35d6016, SUC_lexicon_tags_1 },
    { 0x26fd6017, SUC_lexicon_tags_17 },
    { 0xde9d6018, SUC_lexicon_tags_1 },
    { 0x9e9d6016, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0x963d601c, SUC_lexicon_tags_11 },
    { 0x4f9d601d, SUC_lexicon_tags_7 },
    { 0x9d3d601e, SUC_lexicon_tags_46 },
    { 0x21bd601e, SUC_lexicon_tags_25 },
    { 0x8d3d6020, SUC_lexicon_tags_19 },
    { 0x7b5d6020, SUC_lexicon_tags_11 },
    { 0x3a5d6020, SUC_lexicon_tags_41 },
    { 0xdf7d6023, SUC_lexicon_tags_19 },
    { 0x823d6023, SUC_lexicon_tags_65 },
    { 0xf1d6025, SUC_lexicon_tags_6 },
    { 0x56dd6026, SUC_lexicon_tags_11 },
    { 0x32dd6021, SUC_lexicon_tags_11 },
    { 0x639d6028, SUC_lexicon_tags_31 },
    { 0x9e9d6029, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bd602c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bd6032, SUC_lexicon_tags_105 },
    { 0xee3d6032, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78dd6035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf7d6037, SUC_lexicon_tags_45 },
    { 0x87d6037, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dd603d, SUC_lexicon_tags_49 },
    { 0x937d603e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xca1d6040, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1e1d6042, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3d6046, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba7d6048, SUC_lexicon_tags_12 },
    { 0x33dd6048, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc37d604b, SUC_lexicon_tags_1 },
    { 0xb09d604b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x995d604e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1bd6050, SUC_lexicon_tags_16 },
    { 0x4fdd6051, SUC_lexicon_tags_9 },
    { 0x407d6052, SUC_lexicon_tags_103 },
    { 0xb9fd6053, SUC_lexicon_tags_0 },
    { 0x137d6054, SUC_lexicon_tags_47 },
    { 0x809d6055, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647d605a, SUC_lexicon_tags_33 },
    { 0x219d605a, SUC_lexicon_tags_7 },
    { 0xad3d605c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x82fd605e, SUC_lexicon_tags_1 },
    { 0x3bfd605e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fd6062, SUC_lexicon_tags_2 },
    { 0xe15d6063, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x871d6066, SUC_lexicon_tags_9 },
    { 0xe07d6067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabd606f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x865d6072, SUC_lexicon_tags_25 },
    { 0xb87d6072, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x91bd6075, SUC_lexicon_tags_14 },
    { 0x9d1d6076, SUC_lexicon_tags_30 },
    { 0xdd1d6077, SUC_lexicon_tags_7 },
    { 0x2c9d6078, SUC_lexicon_tags_37 },
    { 0xeb3d6078, SUC_lexicon_tags_2 },
    { 0x759d6077, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f1d607c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe39d607e, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7d6081, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31d6085, SUC_lexicon_tags_7 },
    { 0x8fbd6085, SUC_lexicon_tags_9 },
    { 0x52dd6086, SUC_lexicon_tags_63 },
    { 0xeabd6088, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9d608f, SUC_lexicon_tags_37 },
    { 0x451d608f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907d6094, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x965d6096, SUC_lexicon_tags_25 },
    { 0x4dd6096, SUC_lexicon_tags_82 },
    { 0x9f3d6098, SUC_lexicon_tags_82 },
    { 0x169d6099, SUC_lexicon_tags_10 },
    { 0x345d609a, SUC_lexicon_tags_37 },
    { 0x27dd6098, SUC_lexicon_tags_25 },
    { 0xc43d609c, SUC_lexicon_tags_84 },
    { 0xebd609c, SUC_lexicon_tags_37 },
    { 0x6abd609e, SUC_lexicon_tags_8 },
    { 0x2f7d6097, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9d60a3, SUC_lexicon_tags_15 },
    { 0x8f3d60a4, SUC_lexicon_tags_22 },
    { 0x7f1d60a4, SUC_lexicon_tags_11 },
    { 0xb89d60a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbfbd60a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4bd60aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xefdd60ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbd60b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63dd60bb, SUC_lexicon_tags_4 },
    { 0xef9d60bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4fbd60be, SUC_lexicon_tags_25 },
    { 0x1afd60be, SUC_lexicon_tags_23 },
    { 0x7dd60bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fd60c4, SUC_lexicon_tags_37 },
    { 0xa83d60c5, SUC_lexicon_tags_1 },
    { 0x319d60c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfd60c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xafbd60cc, SUC_lexicon_tags_60 },
    { 0xee3d60cc, SUC_lexicon_tags_32 },
    { 0x567d60ce, SUC_lexicon_tags_9 },
    { 0xcddd60cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531d60d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1d60d9, SUC_lexicon_tags_63 },
    { 0xeafd60da, SUC_lexicon_tags_33 },
    { 0x927d60db, SUC_lexicon_tags_33 },
    { 0x867d60dc, SUC_lexicon_tags_15 },
    { 0x173d60dc, SUC_lexicon_tags_61 },
    { 0xddfd60de, SUC_lexicon_tags_7 },
    { 0xf91d60d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb71d60e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713d60e5, SUC_lexicon_tags_17 },
    { 0xfedd60e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587d60eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9d60ef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7d60fe, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d6103, SUC_lexicon_tags_15 },
    { 0x5bdd6104, SUC_lexicon_tags_37 },
    { 0xf47d6105, SUC_lexicon_tags_29 },
    { 0xdebd6105, SUC_lexicon_tags_1 },
    { 0x4d1d6107, SUC_lexicon_tags_28 },
    { 0x679d6105, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe11d610a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bd610d, SUC_lexicon_tags_46 },
    { 0xcf7d610e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37d6112, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe8dd6114, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bd6119, SUC_lexicon_tags_45 },
    { 0x399d611a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63d611d, SUC_lexicon_tags_29 },
    { 0xb9dd611e, SUC_lexicon_tags_11 },
    { 0xe79d611f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x257d6121, SUC_lexicon_tags_7 },
    { 0x999d6122, SUC_lexicon_tags_32 },
    { 0x2bbd6122, SUC_lexicon_tags_6 },
    { 0x143d6124, SUC_lexicon_tags_45 },
    { 0x295d6125, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bd6128, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dd612e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x401d6131, SUC_lexicon_tags_37 },
    { 0x891d6131, SUC_lexicon_tags_49 },
    { 0x793d6133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf5d6135, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dd6139, SUC_lexicon_tags_38 },
    { 0xecdd613a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bd613e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb3d6140, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7d6144, SUC_lexicon_tags_29 },
    { 0x175d6144, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7d614c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3d6150, SUC_lexicon_tags_35 },
    { 0x789d6150, SUC_lexicon_tags_11 },
    { 0x453d6152, SUC_lexicon_tags_37 },
    { 0xe87d6152, SUC_lexicon_tags_7 },
    { 0x515d6154, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bd6158, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xea7d615a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa59d615c, SUC_lexicon_tags_25 },
    { 0x52fd615c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3d6164, SUC_lexicon_tags_2 },
    { 0x813d6164, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x467d6168, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39d616b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfd616e, SUC_lexicon_tags_29 },
    { 0x385d616e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4fdd6171, SUC_lexicon_tags_8 },
    { 0x3dfd6172, SUC_lexicon_tags_49 },
    { 0x597d6171, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x671d6176, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x73bd6178, SUC_lexicon_tags_30 },
    { 0xf8fd6178, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7d617e, SUC_lexicon_tags_9 },
    { 0xfdbd617e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7d6183, SUC_lexicon_tags_70 },
    { 0x9bd6183, SUC_lexicon_tags_25 },
    { 0xb17d6184, SUC_lexicon_tags_17 },
    { 0xaedd6186, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9d6189, SUC_lexicon_tags_39 },
    { 0xa85d618a, SUC_lexicon_tags_15 },
    { 0x915d618a, SUC_lexicon_tags_25 },
    { 0x6c7d618c, SUC_lexicon_tags_37 },
    { 0xd6bd618d, SUC_lexicon_tags_60 },
    { 0xb4bd6189, SUC_lexicon_tags_63 },
    { 0x58fd618f, SUC_lexicon_tags_26 },
    { 0x74bd6190, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7d6193, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3d6197, SUC_lexicon_tags_2 },
    { 0x857d6197, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5f9d619a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371d619e, SUC_lexicon_tags_12 },
    { 0x7d7d619e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4add61a2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x301d61a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xdedd61a6, SUC_lexicon_tags_194 },
    { 0x6ebd61a6, SUC_lexicon_tags_31 },
    { 0xd67d61a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d61ad, SUC_lexicon_tags_5 },
    { 0x375d61ad, SUC_lexicon_tags_37 },
    { 0xd5dd61ae, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfd61b6, SUC_lexicon_tags_1 },
    { 0x537d61b7, SUC_lexicon_tags_7 },
    { 0x877d61b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3d61be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3dfd61c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09d61c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57d61c7, SUC_lexicon_tags_25 },
    { 0x6a1d61c8, SUC_lexicon_tags_43 },
    { 0x7ebd61c9, SUC_lexicon_tags_33 },
    { 0x11dd61ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5d61cd, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xfc1d61cf, SUC_lexicon_tags_68 },
    { 0xdfbd61d0, SUC_lexicon_tags_32 },
    { 0xf41d61d1, SUC_lexicon_tags_25 },
    { 0x687d61d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x367d61d4, SUC_lexicon_tags_26 },
    { 0xa09d61d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5d61d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x445d61db, SUC_lexicon_tags_49 },
    { 0x1e5d61dc, SUC_lexicon_tags_37 },
    { 0x31fd61dd, SUC_lexicon_tags_15 },
    { 0x985d61de, SUC_lexicon_tags_14 },
    { 0xb05d61dd, SUC_lexicon_tags_68 },
    { 0x2e1d61e0, SUC_lexicon_tags_63 },
    { 0x623d61e1, SUC_lexicon_tags_17 },
    { 0x995d61e2, SUC_lexicon_tags_17 },
    { 0x24bd61e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577d61e7, SUC_lexicon_tags_34 },
    { 0x1ebd61e7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5d61eb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x563d61ed, SUC_lexicon_tags_12 },
    { 0x35dd61ee, SUC_lexicon_tags_64 },
    { 0x37fd61ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbdd61f1, SUC_lexicon_tags_29 },
    { 0xd0dd61f2, SUC_lexicon_tags_25 },
    { 0x719d61f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb37d61f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81d61fb, SUC_lexicon_tags_26 },
    { 0xac7d61fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b1d61fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abd6207, SUC_lexicon_tags_25 },
    { 0xf05d6208, SUC_lexicon_tags_12 },
    { 0xa3fd6209, SUC_lexicon_tags_38 },
    { 0x5c9d620a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bd620e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x42fd6211, SUC_lexicon_tags_23 },
    { 0xbb7d6212, SUC_lexicon_tags_0 },
    { 0x6dbd6213, SUC_lexicon_tags_3 },
    { 0x5bbd6214, SUC_lexicon_tags_17 },
    { 0x41fd6215, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1d6218, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9d621b, SUC_lexicon_tags_39 },
    { 0x729d621c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a5d621e, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdd6223, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b3d6225, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dd622b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x819d622d, SUC_lexicon_tags_0 },
    { 0xd7fd622e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91d6231, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d6235, SUC_lexicon_tags_49 },
    { 0x359d6236, SUC_lexicon_tags_0 },
    { 0x6f5d6237, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bd623a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffd623d, SUC_lexicon_tags_23 },
    { 0x2c9d623e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbd6244, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fd6258, SUC_lexicon_tags_11 },
    { 0xb77d6259, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bd625c, SUC_lexicon_tags_56 },
    { 0x365d625c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3ebd625f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d6262, SUC_lexicon_tags_21 },
    { 0x101d6262, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7d6266, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe6dd6268, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d626b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb53d626d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f7d626f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbe3d6271, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdd6274, SUC_lexicon_tags_48 },
    { 0x213d6275, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67d627e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efd6282, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255d6288, SUC_lexicon_tags_15 },
    { 0xb37d6289, SUC_lexicon_tags_55 },
    { 0xcbdd628a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1d1d628c, SUC_lexicon_tags_9 },
    { 0x7bfd628c, SUC_lexicon_tags_1 },
    { 0x36dd628c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdd6291, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd33d6293, SUC_lexicon_tags_25 },
    { 0x1e9d6294, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9d629d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3d62a5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbd62af, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5d62b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63d62b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767d62bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7d62c0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x103d62c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfd62c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9d62c8, SUC_lexicon_tags_7 },
    { 0xf25d62c8, SUC_lexicon_tags_63 },
    { 0xea5d62c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x621d62cc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bd62cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdd5d62d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941d62d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6cfd62d8, SUC_lexicon_tags_37 },
    { 0xc3dd62d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefd62dc, SUC_lexicon_tags_13 },
    { 0x819d62dc, SUC_lexicon_tags_47 },
    { 0xd1dd62de, SUC_lexicon_tags_2 },
    { 0x185d62dc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09d62e3, SUC_lexicon_tags_37 },
    { 0x7dd62e3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97d62e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac9d62ea, SUC_lexicon_tags_17 },
    { 0x975d62ea, SUC_lexicon_tags_53 },
    { 0x8efd62eb, SUC_lexicon_tags_37 },
    { 0x2ffd62ed, SUC_lexicon_tags_15 },
    { 0x607d62ee, SUC_lexicon_tags_13 },
    { 0x491d62ec, SUC_lexicon_tags_37 },
    { 0xea1d62f0, SUC_lexicon_tags_45 },
    { 0x2b1d62f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fd62f4, SUC_lexicon_tags_2 },
    { 0xf0fd62f5, SUC_lexicon_tags_164 },
    { 0x2ffd62f6, SUC_lexicon_tags_31 },
    { 0x305d62f6, SUC_lexicon_tags_38 },
    { 0x2bbd62f5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5d62fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe1d6300, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbd6305, SUC_lexicon_tags_19 },
    { 0x1b3d6306, SUC_lexicon_tags_3 },
    { 0xcebd6307, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d6310, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5f7d6312, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb49d6314, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317d6318, SUC_lexicon_tags_8 },
    { 0x8b7d6318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x939d631b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68fd631d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7d6322, SUC_lexicon_tags_33 },
    { 0x8f1d6322, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35d6327, SUC_lexicon_tags_27 },
    { 0x4e7d6328, SUC_lexicon_tags_23 },
    { 0xc6dd6329, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111d632d, SUC_lexicon_tags_27 },
    { 0x57d632e, SUC_lexicon_tags_12 },
    { 0x61d632f, SUC_lexicon_tags_61 },
    { 0x94fd6330, SUC_lexicon_tags_0 },
    { 0x1e9d632e, SUC_lexicon_tags_28 },
    { 0xb89d6332, SUC_lexicon_tags_17 },
    { 0xbefd6333, SUC_lexicon_tags_7 },
    { 0x99fd6333, SUC_lexicon_tags_31 },
    { 0xa77d6335, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9d6338, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x423d633a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbd633d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7d6345, SUC_lexicon_tags_25 },
    { 0x7e7d6346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca9d6348, SUC_lexicon_tags_37 },
    { 0x9a9d6348, SUC_lexicon_tags_53 },
    { 0xf09d634a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3d6351, SUC_lexicon_tags_31 },
    { 0x561d6351, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe5fd6354, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3add635a, SUC_lexicon_tags_38 },
    { 0x287d635a, SUC_lexicon_tags_10 },
    { 0x229d635a, SUC_lexicon_tags_11 },
    { 0xc2bd635d, SUC_lexicon_tags_37 },
    { 0xb8bd635e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb91d6360, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddd6364, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47d6368, SUC_lexicon_tags_74 },
    { 0x555d6368, SUC_lexicon_tags_9 },
    { 0xb2bd6368, SUC_lexicon_tags_28 },
    { 0xbfdd636b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1d636e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38dd6373, SUC_lexicon_tags_0 },
    { 0x6f1d6374, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd6379, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x951d637b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x86dd637d, SUC_lexicon_tags_238 },
    { 0x337d637e, SUC_lexicon_tags_2 },
    { 0x24bd637d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fd6389, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2ddd638b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513d6390, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81d6393, SUC_lexicon_tags_15 },
    { 0x793d6394, SUC_lexicon_tags_1 },
    { 0x6fdd6393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d639c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99d639f, SUC_lexicon_tags_33 },
    { 0xba1d63a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaddd63a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45d63a7, SUC_lexicon_tags_37 },
    { 0x13fd63a8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x2b3d63aa, SUC_lexicon_tags_26 },
    { 0xf5dd63aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcabd63ad, SUC_lexicon_tags_2 },
    { 0xeb3d63ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5d63b1, SUC_lexicon_tags_25 },
    { 0x16fd63b2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1d63b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdd63b8, SUC_lexicon_tags_0 },
    { 0xc93d63b9, SUC_lexicon_tags_115 },
    { 0xe03d63ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d63c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x827d63c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61d63ca, SUC_lexicon_tags_25 },
    { 0xe4dd63cb, SUC_lexicon_tags_25 },
    { 0x445d63cc, SUC_lexicon_tags_45 },
    { 0x177d63cc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x263d63cf, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9d63d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x88fd63d4, SUC_lexicon_tags_2 },
    { 0x299d63d5, SUC_lexicon_tags_17 },
    { 0x5fbd63d5, SUC_lexicon_tags_17 },
    { 0xc27d63d4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fd63da, SUC_lexicon_tags_11 },
    { 0x67fd63db, SUC_lexicon_tags_19 },
    { 0xfc7d63dc, SUC_lexicon_tags_14 },
    { 0xb05d63db, SUC_lexicon_tags_9 },
    { 0xb87d63de, SUC_lexicon_tags_148 },
    { 0x407d63df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3d63e2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47bd63e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd63ec, SUC_lexicon_tags_2 },
    { 0x7b3d63ed, SUC_lexicon_tags_33 },
    { 0xf03d63ee, SUC_lexicon_tags_54 },
    { 0x4a7d63ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65d63f3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9add63fc, SUC_lexicon_tags_43 },
    { 0x94fd63fc, SUC_lexicon_tags_218 },
    { 0x295d63fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe19d6400, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509d6405, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe6dd6407, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x36fd640a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbd640d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbd6410, SUC_lexicon_tags_38 },
    { 0x681d6411, SUC_lexicon_tags_29 },
    { 0xab1d6410, SUC_lexicon_tags_2 },
    { 0xc9dd6413, SUC_lexicon_tags_43 },
    { 0xd1bd6411, SUC_lexicon_tags_2 },
    { 0x881d6412, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb65d6417, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x171d6419, SUC_lexicon_tags_38 },
    { 0xf03d641a, SUC_lexicon_tags_14 },
    { 0xd8dd641b, SUC_lexicon_tags_17 },
    { 0xf89d641b, SUC_lexicon_tags_63 },
    { 0x9fbd641d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x157d641f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797d6424, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3cfd6426, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3d6429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995d642d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd6439, SUC_lexicon_tags_31 },
    { 0xe3d643a, SUC_lexicon_tags_26 },
    { 0x2b7d643b, SUC_lexicon_tags_27 },
    { 0x909d643c, SUC_lexicon_tags_43 },
    { 0x179d643b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x873d643f, SUC_lexicon_tags_34 },
    { 0x467d6440, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfdd6442, SUC_lexicon_tags_45 },
    { 0x435d6443, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3d6447, SUC_lexicon_tags_19 },
    { 0xb57d6448, SUC_lexicon_tags_37 },
    { 0xaafd6447, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdd644c, SUC_lexicon_tags_15 },
    { 0xbcdd644c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6f1d644f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x799d6452, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723d6458, SUC_lexicon_tags_1 },
    { 0x17fd6459, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c9d645b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dd6460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb7fd6462, SUC_lexicon_tags_37 },
    { 0xff7d6463, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35d6468, SUC_lexicon_tags_15 },
    { 0x4add6469, SUC_lexicon_tags_37 },
    { 0x8abd646a, SUC_lexicon_tags_37 },
    { 0xf3dd6468, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55d646f, SUC_lexicon_tags_39 },
    { 0x913d646f, SUC_lexicon_tags_17 },
    { 0x199d6471, SUC_lexicon_tags_11 },
    { 0x73fd6471, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3edd6474, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57d6476, SUC_lexicon_tags_80 },
    { 0x7abd6477, SUC_lexicon_tags_11 },
    { 0x669d6476, SUC_lexicon_tags_28 },
    { 0x1fdd6479, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587d647f, SUC_lexicon_tags_26 },
    { 0xdc5d647f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x799d6483, SUC_lexicon_tags_17 },
    { 0xaedd6484, SUC_lexicon_tags_9 },
    { 0xdbdd6485, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fd648a, SUC_lexicon_tags_47 },
    { 0xe5dd648b, SUC_lexicon_tags_9 },
    { 0x7d3d648b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edd6496, SUC_lexicon_tags_0 },
    { 0x1bbd6497, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d649e, SUC_lexicon_tags_29 },
    { 0xe99d649f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5bfd64a1, SUC_lexicon_tags_25 },
    { 0x1cdd64a1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x183d64a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x127d64a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7dbd64a9, SUC_lexicon_tags_88 },
    { 0x2f7d64a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fd64ae, SUC_lexicon_tags_33 },
    { 0x25fd64ae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afd64b2, SUC_lexicon_tags_0 },
    { 0x6bbd64b2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5f5d64b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803d64b9, SUC_lexicon_tags_0 },
    { 0x83dd64b9, SUC_lexicon_tags_77 },
    { 0xcdbd64ba, SUC_lexicon_tags_14 },
    { 0xc19d64b9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67d64c6, SUC_lexicon_tags_25 },
    { 0x57fd64c7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x90bd64c9, SUC_lexicon_tags_0 },
    { 0x1ddd64ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x983d64cd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dd64d0, SUC_lexicon_tags_15 },
    { 0xd71d64d1, SUC_lexicon_tags_7 },
    { 0x511d64d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x51d64d4, SUC_lexicon_tags_37 },
    { 0x7fd64d5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fd64d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1d64dd, SUC_lexicon_tags_12 },
    { 0xba7d64de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dd64e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fd64ea, SUC_lexicon_tags_27 },
    { 0x535d64ea, SUC_lexicon_tags_37 },
    { 0x95d64ea, SUC_lexicon_tags_32 },
    { 0xaf3d64ed, SUC_lexicon_tags_31 },
    { 0x49fd64ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bd64f3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfd64f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc61d64fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa95d64fc, SUC_lexicon_tags_19 },
    { 0xd0fd64fd, SUC_lexicon_tags_60 },
    { 0x7b7d64fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb29d6500, SUC_lexicon_tags_9 },
    { 0xcdfd6501, SUC_lexicon_tags_114 },
    { 0xf7d6500, SUC_lexicon_tags_1 },
    { 0xa6dd6503, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xebdd6505, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdd650b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dd6511, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55d6517, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x253d6519, SUC_lexicon_tags_140 },
    { 0x3edd6519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x467d651c, SUC_lexicon_tags_37 },
    { 0x8f1d651d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1d6520, SUC_lexicon_tags_19 },
    { 0xacdd6521, SUC_lexicon_tags_17 },
    { 0x15bd6521, SUC_lexicon_tags_11 },
    { 0x92fd6523, SUC_lexicon_tags_27 },
    { 0xd2bd6524, SUC_lexicon_tags_37 },
    { 0x7d1d6524, SUC_lexicon_tags_0 },
    { 0x51dd6526, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdd6529, SUC_lexicon_tags_0 },
    { 0x5e7d652a, SUC_lexicon_tags_4 },
    { 0x7a3d652b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57d652f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9bbd6531, SUC_lexicon_tags_2 },
    { 0x49fd6532, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25d6535, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3d6538, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf19d653a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x911d653d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fd6540, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe39d6542, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53d654e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x341d6551, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x615d6553, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5d6557, SUC_lexicon_tags_37 },
    { 0x9abd6558, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719d655f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafd6563, SUC_lexicon_tags_56 },
    { 0x6ebd6564, SUC_lexicon_tags_36 },
    { 0x491d6565, SUC_lexicon_tags_37 },
    { 0x389d6566, SUC_lexicon_tags_23 },
    { 0xcdbd6566, SUC_lexicon_tags_9 },
    { 0xb0dd6563, SUC_lexicon_tags_4 },
    { 0x46bd6569, SUC_lexicon_tags_38 },
    { 0x2ebd6566, SUC_lexicon_tags_6 },
    { 0xeefd656b, SUC_lexicon_tags_7 },
    { 0xc05d6565, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fd656f, SUC_lexicon_tags_28 },
    { 0x6c5d6570, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fd6573, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377d6579, SUC_lexicon_tags_37 },
    { 0xd0dd657a, SUC_lexicon_tags_37 },
    { 0xbd657b, SUC_lexicon_tags_109 },
    { 0xa69d657c, SUC_lexicon_tags_61 },
    { 0x3d3d657b, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd6588, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d658f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc71d6591, SUC_lexicon_tags_14 },
    { 0xb21d6592, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bd6598, SUC_lexicon_tags_11 },
    { 0xa5d6598, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x43bd659b, SUC_lexicon_tags_15 },
    { 0x13fd659b, SUC_lexicon_tags_63 },
    { 0xa2fd659d, SUC_lexicon_tags_17 },
    { 0x5dfd659e, SUC_lexicon_tags_17 },
    { 0x5ffd659e, SUC_lexicon_tags_43 },
    { 0xeb9d659d, SUC_lexicon_tags_37 },
    { 0x9cbd65a1, SUC_lexicon_tags_32 },
    { 0x26bd659c, SUC_lexicon_tags_11 },
    { 0x5d5d65a3, SUC_lexicon_tags_12 },
    { 0xad1d65a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb19d65a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33d65aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d65ad, SUC_lexicon_tags_14 },
    { 0xcffd65ae, SUC_lexicon_tags_13 },
    { 0x527d65ad, SUC_lexicon_tags_30 },
    { 0x21d65b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x443d65b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x697d65b5, SUC_lexicon_tags_25 },
    { 0x515d65b6, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7d65c2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bd65c5, SUC_lexicon_tags_46 },
    { 0x73dd65c6, SUC_lexicon_tags_104 },
    { 0x7e7d65c6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1d65cc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75d65db, SUC_lexicon_tags_63 },
    { 0x429d65dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1d65df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5d65e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bd65e7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfd65ea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x239d65ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf45d65ee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1d65f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21d65f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7d65fb, SUC_lexicon_tags_15 },
    { 0x605d65fc, SUC_lexicon_tags_2 },
    { 0x57d65fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x313d6600, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde7d6602, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x435d6605, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dd6608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd660b, SUC_lexicon_tags_5 },
    { 0x3efd660c, SUC_lexicon_tags_7 },
    { 0xa4dd660d, SUC_lexicon_tags_14 },
    { 0xc0fd660e, SUC_lexicon_tags_15 },
    { 0x5edd660f, SUC_lexicon_tags_85 },
    { 0xb45d6610, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd5fd6612, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdd6615, SUC_lexicon_tags_29 },
    { 0x4efd6616, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741d661c, SUC_lexicon_tags_11 },
    { 0x7f1d661c, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x751d661f, SUC_lexicon_tags_25 },
    { 0xa5fd6620, SUC_lexicon_tags_11 },
    { 0x68dd6620, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfd6628, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc77d662a, SUC_lexicon_tags_37 },
    { 0xc85d662a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373d6631, SUC_lexicon_tags_17 },
    { 0xacbd6632, SUC_lexicon_tags_1 },
    { 0xbbd6633, SUC_lexicon_tags_11 },
    { 0x9e1d6633, SUC_lexicon_tags_17 },
    { 0x587d6635, SUC_lexicon_tags_12 },
    { 0xd5bd6636, SUC_lexicon_tags_15 },
    { 0x587d6636, SUC_lexicon_tags_25 },
    { 0x9e9d6635, SUC_lexicon_tags_2 },
    { 0x2dbd6639, SUC_lexicon_tags_33 },
    { 0x5d663a, SUC_lexicon_tags_60 },
    { 0x1fd663b, SUC_lexicon_tags_63 },
    { 0x65d6638, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921d6640, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9d6644, SUC_lexicon_tags_2 },
    { 0x3dfd6644, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f1d6647, SUC_lexicon_tags_15 },
    { 0xabd6647, SUC_lexicon_tags_38 },
    { 0xf2bd6647, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07d664d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x441d6650, SUC_lexicon_tags_47 },
    { 0xf0dd6651, SUC_lexicon_tags_47 },
    { 0xdb1d6652, SUC_lexicon_tags_50 },
    { 0x9ddd6651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa1d6655, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309d665b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x94fd665d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29dd6660, SUC_lexicon_tags_158 },
    { 0x351d6660, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xb27d6663, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949d6667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7cfd6669, SUC_lexicon_tags_5 },
    { 0x83fd6669, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d6677, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbd667a, SUC_lexicon_tags_49 },
    { 0xae7d667a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435d6682, SUC_lexicon_tags_68 },
    { 0x557d6683, SUC_lexicon_tags_23 },
    { 0xfa3d6683, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x403d6687, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9d6691, SUC_lexicon_tags_12 },
    { 0xf11d6692, SUC_lexicon_tags_17 },
    { 0x529d6693, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdd6699, SUC_lexicon_tags_46 },
    { 0xbefd669a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665d669e, SUC_lexicon_tags_46 },
    { 0x43d669f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x335d66a2, SUC_lexicon_tags_17 },
    { 0xd21d66a2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825d66a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17d66ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbd66b0, SUC_lexicon_tags_31 },
    { 0xd9d66b0, SUC_lexicon_tags_28 },
    { 0x593d66b1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31dd66b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb7dd66b9, SUC_lexicon_tags_23 },
    { 0x821d66b9, SUC_lexicon_tags_17 },
    { 0x6dfd66bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dd66c0, SUC_lexicon_tags_26 },
    { 0xd47d66c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbd66c6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dd66c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefd66ce, SUC_lexicon_tags_7 },
    { 0xee5d66cf, SUC_lexicon_tags_23 },
    { 0xc13d66ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bd66d4, SUC_lexicon_tags_11 },
    { 0x5dd66d5, SUC_lexicon_tags_29 },
    { 0x77d66d6, SUC_lexicon_tags_87 },
    { 0x775d66d7, SUC_lexicon_tags_2 },
    { 0x4dbd66d8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xab9d66da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bd66dd, SUC_lexicon_tags_10 },
    { 0xb31d66de, SUC_lexicon_tags_4 },
    { 0xcf1d66dd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325d66e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbd66e9, SUC_lexicon_tags_7 },
    { 0x88bd66ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad7d66ec, SUC_lexicon_tags_7 },
    { 0xea3d66ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7d66f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6efd66f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727d66f7, SUC_lexicon_tags_23 },
    { 0x5d1d66f7, SUC_lexicon_tags_0 },
    { 0x32fd66f7, SUC_lexicon_tags_1 },
    { 0xeddd66f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137d66fe, SUC_lexicon_tags_84 },
    { 0xbebd66ff, SUC_lexicon_tags_48 },
    { 0x513d66ff, SUC_lexicon_tags_25 },
    { 0xeb3d6700, SUC_lexicon_tags_15 },
    { 0xd5d6702, SUC_lexicon_tags_11 },
    { 0x91d6702, SUC_lexicon_tags_25 },
    { 0xc5dd6703, SUC_lexicon_tags_11 },
    { 0x68bd6705, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9d670e, SUC_lexicon_tags_11 },
    { 0xb11d670f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa0dd6711, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351d6716, SUC_lexicon_tags_19 },
    { 0xbdfd6716, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bd671c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c9d671e, SUC_lexicon_tags_235 },
    { 0xe3fd671e, SUC_lexicon_tags_15 },
    { 0xadd671e, SUC_lexicon_tags_68 },
    { 0xe4dd671f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x19fd6723, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dd6729, SUC_lexicon_tags_11 },
    { 0x799d672a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb71d672c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcd9d672e, SUC_lexicon_tags_43 },
    { 0xc01d672f, SUC_lexicon_tags_11 },
    { 0x65dd672f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3d6735, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdd673a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55d673f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21fd6741, SUC_lexicon_tags_63 },
    { 0x345d6742, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9dfd6744, SUC_lexicon_tags_32 },
    { 0x7b5d6745, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3a3d6747, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5bdd6749, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x65d674b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd9d674d, SUC_lexicon_tags_15 },
    { 0x6f5d674d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbd6752, SUC_lexicon_tags_65 },
    { 0xaa9d6753, SUC_lexicon_tags_3 },
    { 0x6abd6754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd41d6756, SUC_lexicon_tags_25 },
    { 0x39d6757, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11d675a, SUC_lexicon_tags_28 },
    { 0x3fdd675a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5d675e, SUC_lexicon_tags_23 },
    { 0xea7d675e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea3d6761, SUC_lexicon_tags_31 },
    { 0x113d6762, SUC_lexicon_tags_46 },
    { 0x1bbd6762, SUC_lexicon_tags_11 },
    { 0x813d6761, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39dd6767, SUC_lexicon_tags_17 },
    { 0xa93d6768, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dd676c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6afd676e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd83d6770, SUC_lexicon_tags_15 },
    { 0xdc7d6770, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddd6775, SUC_lexicon_tags_15 },
    { 0x689d6776, SUC_lexicon_tags_31 },
    { 0x6bfd6775, SUC_lexicon_tags_1 },
    { 0xee7d6777, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabd677d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1d6789, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5d678c, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0x105d678e, SUC_lexicon_tags_17 },
    { 0x981d678f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x733d6792, SUC_lexicon_tags_13 },
    { 0x4fbd6792, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afd679c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd679f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabd67a3, SUC_lexicon_tags_55 },
    { 0x4c3d67a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x277d67a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccbd67a8, SUC_lexicon_tags_118 },
    { 0xc2dd67a8, SUC_lexicon_tags_23 },
    { 0x67fd67aa, SUC_lexicon_tags_23 },
    { 0x989d67a9, SUC_lexicon_tags_15 },
    { 0xe01d67ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbd67af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79d67b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367d67b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bd67bd, SUC_lexicon_tags_31 },
    { 0x24bd67bd, SUC_lexicon_tags_30 },
    { 0xe59d67bf, SUC_lexicon_tags_46 },
    { 0x7e3d67c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa15d67c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3d67c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80bd67c7, SUC_lexicon_tags_14 },
    { 0xe5d67c8, SUC_lexicon_tags_25 },
    { 0x16bd67c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53d67cd, SUC_lexicon_tags_11 },
    { 0xc8fd67ce, SUC_lexicon_tags_8 },
    { 0xf6dd67cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad5d67d1, SUC_lexicon_tags_9 },
    { 0x21dd67d2, SUC_lexicon_tags_17 },
    { 0xf49d67d3, SUC_lexicon_tags_40 },
    { 0x51d67d4, SUC_lexicon_tags_46 },
    { 0x781d67d4, SUC_lexicon_tags_11 },
    { 0x68fd67d6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5a1d67d8, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x613d67db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59bd67dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31d67e1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x479d67e3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7d67e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xba7d67e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811d67ec, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7d67f1, SUC_lexicon_tags_17 },
    { 0x373d67f2, SUC_lexicon_tags_87 },
    { 0x749d67f1, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5d67f9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35d67fe, SUC_lexicon_tags_37 },
    { 0x14bd67ff, SUC_lexicon_tags_15 },
    { 0xe6dd67ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07d6804, SUC_lexicon_tags_8 },
    { 0x18bd6805, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa89d6807, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1bbd6809, SUC_lexicon_tags_1 },
    { 0xbb9d680a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fd680e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dd6811, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58fd6813, SUC_lexicon_tags_47 },
    { 0x32fd6814, SUC_lexicon_tags_26 },
    { 0xd4fd6815, SUC_lexicon_tags_107 },
    { 0x6ebd6815, SUC_lexicon_tags_11 },
    { 0x8b5d6814, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5fd6819, SUC_lexicon_tags_6 },
    { 0x5f7d681a, SUC_lexicon_tags_9 },
    { 0xd33d681b, SUC_lexicon_tags_0 },
    { 0xc49d681c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155d6820, SUC_lexicon_tags_46 },
    { 0x305d6821, SUC_lexicon_tags_15 },
    { 0xbd5d6822, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23d6824, SUC_lexicon_tags_37 },
    { 0xebfd6825, SUC_lexicon_tags_13 },
    { 0xeefd6826, SUC_lexicon_tags_15 },
    { 0xd71d6826, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7d682d, SUC_lexicon_tags_29 },
    { 0x481d682e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47d6833, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bd6836, SUC_lexicon_tags_27 },
    { 0x8a9d6836, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bd683c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x355d683e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bd6847, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4bfd6849, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xdffd684c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75d684f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3afd6851, SUC_lexicon_tags_37 },
    { 0x41dd6852, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1fd6854, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdd6857, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275d685d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1d6861, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89d686e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebd6871, SUC_lexicon_tags_11 },
    { 0x255d6871, SUC_lexicon_tags_35 },
    { 0x8d9d6873, SUC_lexicon_tags_43 },
    { 0x177d6872, SUC_lexicon_tags_1 },
    { 0xedfd6874, SUC_lexicon_tags_11 },
    { 0xa35d6876, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2e3d6878, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bd687d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x91fd687f, SUC_lexicon_tags_7 },
    { 0x5abd6880, SUC_lexicon_tags_0 },
    { 0xbfd687f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fd6886, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x955d6888, SUC_lexicon_tags_65 },
    { 0x9e9d6888, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbd688e, SUC_lexicon_tags_2 },
    { 0x613d688e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5d6894, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67d6897, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95d689a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dd68a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41d68a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d68ae, SUC_lexicon_tags_170 },
    { 0xdefd68ae, SUC_lexicon_tags_47 },
    { 0x723d68af, SUC_lexicon_tags_37 },
    { 0xc1fd68ae, SUC_lexicon_tags_15 },
    { 0xeadd68af, SUC_lexicon_tags_49 },
    { 0x193d68b3, SUC_lexicon_tags_63 },
    { 0x27d68b4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213d68b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd68bb, SUC_lexicon_tags_7 },
    { 0xc21d68bc, SUC_lexicon_tags_14 },
    { 0x251d68bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3d68c0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd68c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea7d68c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa8dd68c9, SUC_lexicon_tags_60 },
    { 0xc01d68c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701d68d2, SUC_lexicon_tags_25 },
    { 0x495d68d3, SUC_lexicon_tags_11 },
    { 0x3fbd68d3, SUC_lexicon_tags_17 },
    { 0xd7fd68d5, SUC_lexicon_tags_2 },
    { 0x703d68d6, SUC_lexicon_tags_17 },
    { 0x42bd68d4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67d68dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x19d68df, SUC_lexicon_tags_79 },
    { 0x75fd68df, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645d68e4, SUC_lexicon_tags_0 },
    { 0x39d68e5, SUC_lexicon_tags_7 },
    { 0xa51d68e5, SUC_lexicon_tags_4 },
    { 0xbe7d68e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae3d68e9, SUC_lexicon_tags_1 },
    { 0xc15d68e9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x25dd68ed, SUC_lexicon_tags_79 },
    { 0x547d68ed, SUC_lexicon_tags_17 },
    { 0x3cbd68ef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15d68f4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee7d68f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x527d68f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdd6901, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bd6905, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62dd6909, SUC_lexicon_tags_8 },
    { 0x195d690a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd6912, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bd6915, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x231d6917, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x533d6919, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x887d691b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17d6920, SUC_lexicon_tags_37 },
    { 0x5dbd6921, SUC_lexicon_tags_17 },
    { 0x13dd6922, SUC_lexicon_tags_37 },
    { 0x9f9d6923, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7d6929, SUC_lexicon_tags_25 },
    { 0xb6fd692a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5d692d, SUC_lexicon_tags_33 },
    { 0x459d692e, SUC_lexicon_tags_23 },
    { 0x529d692f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9d6935, SUC_lexicon_tags_1 },
    { 0x48fd6936, SUC_lexicon_tags_43 },
    { 0x6b1d6937, SUC_lexicon_tags_108 },
    { 0xfc3d6938, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667d693c, SUC_lexicon_tags_43 },
    { 0x2fbd693c, SUC_lexicon_tags_7 },
    { 0x72fd693e, SUC_lexicon_tags_68 },
    { 0xa69d693f, SUC_lexicon_tags_60 },
    { 0xe17d6940, SUC_lexicon_tags_7 },
    { 0xee1d6941, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35d6944, SUC_lexicon_tags_4 },
    { 0xd85d6945, SUC_lexicon_tags_27 },
    { 0x99d6945, SUC_lexicon_tags_48 },
    { 0xd9fd6945, SUC_lexicon_tags_62 },
    { 0x5bdd6948, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xefbd694a, SUC_lexicon_tags_1 },
    { 0xb0bd694b, SUC_lexicon_tags_15 },
    { 0x981d694b, SUC_lexicon_tags_0 },
    { 0x10fd694d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3d6951, SUC_lexicon_tags_25 },
    { 0x357d6951, SUC_lexicon_tags_47 },
    { 0x47dd6951, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3e3d6955, SUC_lexicon_tags_63 },
    { 0x7edd6956, SUC_lexicon_tags_13 },
    { 0xc19d6957, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x507d6959, SUC_lexicon_tags_4 },
    { 0xe9d6959, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf71d695c, SUC_lexicon_tags_19 },
    { 0x605d695d, SUC_lexicon_tags_8 },
    { 0xd99d695d, SUC_lexicon_tags_138 },
    { 0x1e7d695d, SUC_lexicon_tags_25 },
    { 0xdfdd695f, SUC_lexicon_tags_45 },
    { 0xfedd6961, SUC_lexicon_tags_63 },
    { 0x3fdd6961, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3d6967, SUC_lexicon_tags_70 },
    { 0x431d6968, SUC_lexicon_tags_12 },
    { 0x81fd6969, SUC_lexicon_tags_11 },
    { 0x45dd6969, SUC_lexicon_tags_17 },
    { 0xbedd696b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x95bd696d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85d6972, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x7fdd6974, SUC_lexicon_tags_17 },
    { 0xc6dd6975, SUC_lexicon_tags_4 },
    { 0x7d7d6976, SUC_lexicon_tags_216 },
    { 0xe6dd6977, SUC_lexicon_tags_0 },
    { 0x6a7d6974, SUC_lexicon_tags_1 },
    { 0xbabd6979, SUC_lexicon_tags_17 },
    { 0xba9d6978, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x379d697c, SUC_lexicon_tags_28 },
    { 0x7fdd697d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07d6980, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03d6983, SUC_lexicon_tags_15 },
    { 0x413d6984, SUC_lexicon_tags_22 },
    { 0xd9d6985, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x41fd6987, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x935d6989, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe6fd698b, SUC_lexicon_tags_65 },
    { 0xdf5d698c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2c7d698e, SUC_lexicon_tags_72 },
    { 0xae9d698e, SUC_lexicon_tags_23 },
    { 0xa5fd6990, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bd6999, SUC_lexicon_tags_47 },
    { 0x94fd6999, SUC_lexicon_tags_33 },
    { 0xa61d699b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9d699e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xd1fd69a0, SUC_lexicon_tags_33 },
    { 0xd13d69a0, SUC_lexicon_tags_37 },
    { 0x209d69a1, SUC_lexicon_tags_2 },
    { 0xf47d69a2, SUC_lexicon_tags_17 },
    { 0x45d69a4, SUC_lexicon_tags_15 },
    { 0xadbd69a5, SUC_lexicon_tags_1 },
    { 0x4a3d69a6, SUC_lexicon_tags_6 },
    { 0x817d69a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x93d69a9, SUC_lexicon_tags_47 },
    { 0x4b7d69a9, SUC_lexicon_tags_68 },
    { 0xad9d69ab, SUC_lexicon_tags_25 },
    { 0xdedd69ab, SUC_lexicon_tags_11 },
    { 0xf5fd69ad, SUC_lexicon_tags_11 },
    { 0x9ebd69ad, SUC_lexicon_tags_17 },
    { 0x17dd69af, SUC_lexicon_tags_43 },
    { 0xb2bd69ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1cdd69b2, SUC_lexicon_tags_28 },
    { 0x239d69b3, SUC_lexicon_tags_25 },
    { 0x14fd69b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5d69b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e1d69b9, SUC_lexicon_tags_25 },
    { 0x329d69b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4efd69bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7d69c0, SUC_lexicon_tags_10 },
    { 0x7ffd69c0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x3c3d69c3, SUC_lexicon_tags_25 },
    { 0x44fd69c4, SUC_lexicon_tags_66 },
    { 0x75bd69c5, SUC_lexicon_tags_14 },
    { 0xe8bd69c6, SUC_lexicon_tags_19 },
    { 0xd05d69c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x321d69ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f9d69cc, SUC_lexicon_tags_12 },
    { 0x529d69cd, SUC_lexicon_tags_23 },
    { 0xda5d69ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc29d69d0, SUC_lexicon_tags_30 },
    { 0x163d69d1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x59fd69d4, SUC_lexicon_tags_22 },
    { 0xe85d69d4, SUC_lexicon_tags_23 },
    { 0x5cdd69d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dd69da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bd69e0, SUC_lexicon_tags_30 },
    { 0xdf1d69e1, SUC_lexicon_tags_2 },
    { 0x4dfd69e2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x16fd69e4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1ffd69e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77d69eb, SUC_lexicon_tags_2 },
    { 0xe6bd69ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x327d69ee, SUC_lexicon_tags_1 },
    { 0x275d69ef, SUC_lexicon_tags_0 },
    { 0xdcbd69f0, SUC_lexicon_tags_27 },
    { 0x317d69f1, SUC_lexicon_tags_26 },
    { 0xe8bd69f1, SUC_lexicon_tags_11 },
    { 0xf33d69f3, SUC_lexicon_tags_61 },
    { 0x4bfd69f3, SUC_lexicon_tags_37 },
    { 0x5c9d69f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x38bd69f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253d6a00, SUC_lexicon_tags_33 },
    { 0x481d6a00, SUC_lexicon_tags_43 },
    { 0x529d6a00, SUC_lexicon_tags_7 },
    { 0x1f3d6a02, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x745d6a05, SUC_lexicon_tags_30 },
    { 0xf3fd6a06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abd6a09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb77d6a0b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3d6a0f, SUC_lexicon_tags_37 },
    { 0x7f3d6a10, SUC_lexicon_tags_39 },
    { 0xfa7d6a0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeadd6a13, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fd6a16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd6a19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaa9d6a1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3d6a25, SUC_lexicon_tags_15 },
    { 0xb4dd6a25, SUC_lexicon_tags_60 },
    { 0xeabd6a26, SUC_lexicon_tags_17 },
    { 0x153d6a28, SUC_lexicon_tags_15 },
    { 0xb7fd6a29, SUC_lexicon_tags_14 },
    { 0xfbd6a29, SUC_lexicon_tags_26 },
    { 0xabd6a2b, SUC_lexicon_tags_57 },
    { 0x8b1d6a2a, SUC_lexicon_tags_47 },
    { 0x125d6a2a, SUC_lexicon_tags_9 },
    { 0xa45d6a2e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873d6a38, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebd6a3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f9d6a3d, SUC_lexicon_tags_33 },
    { 0xa33d6a3e, SUC_lexicon_tags_7 },
    { 0x45dd6a3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339d6a46, SUC_lexicon_tags_9 },
    { 0xda9d6a47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41d6a4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x659d6a4c, SUC_lexicon_tags_57 },
    { 0x25fd6a4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x433d6a4f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8c5d6a51, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fd6a57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1d6a5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61d6a5f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe61d6a61, SUC_lexicon_tags_30 },
    { 0xf9bd6a62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b5d6a64, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafd6a6b, SUC_lexicon_tags_11 },
    { 0x32bd6a6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5d6a6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x205d6a71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747d6a75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843d6a7b, SUC_lexicon_tags_82 },
    { 0xb4fd6a7b, SUC_lexicon_tags_37 },
    { 0xbcdd6a7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1d6a80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ffd6a82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bd6a89, SUC_lexicon_tags_2 },
    { 0xa3d6a8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x145d6a8c, SUC_lexicon_tags_7 },
    { 0xa1d6a8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fd6a90, SUC_lexicon_tags_26 },
    { 0xcefd6a91, SUC_lexicon_tags_17 },
    { 0x8d3d6a92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x65fd6a94, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd6a98, SUC_lexicon_tags_25 },
    { 0x867d6a99, SUC_lexicon_tags_43 },
    { 0x165d6a9a, SUC_lexicon_tags_66 },
    { 0xa03d6a98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bd6a9e, SUC_lexicon_tags_12 },
    { 0x54dd6a9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9d6aa3, SUC_lexicon_tags_12 },
    { 0x701d6aa4, SUC_lexicon_tags_27 },
    { 0xf4bd6aa5, SUC_lexicon_tags_61 },
    { 0x83dd6aa6, SUC_lexicon_tags_19 },
    { 0x83fd6aa5, SUC_lexicon_tags_0 },
    { 0xe83d6aa8, SUC_lexicon_tags_47 },
    { 0x383d6aa8, SUC_lexicon_tags_15 },
    { 0x947d6aaa, SUC_lexicon_tags_11 },
    { 0xd33d6aab, SUC_lexicon_tags_26 },
    { 0xdb1d6aac, SUC_lexicon_tags_0 },
    { 0xc05d6aac, SUC_lexicon_tags_49 },
    { 0x53bd6aac, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x697d6ab1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bd6ab4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x759d6ab6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349d6abb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeffd6abd, SUC_lexicon_tags_11 },
    { 0xcb7d6abe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdd6ac2, SUC_lexicon_tags_62 },
    { 0x6ddd6ac2, SUC_lexicon_tags_27 },
    { 0xb81d6ac3, SUC_lexicon_tags_12 },
    { 0x17fd6ac5, SUC_lexicon_tags_12 },
    { 0x2f3d6ac5, SUC_lexicon_tags_25 },
    { 0xd45d6ac7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x51d6ac9, SUC_lexicon_tags_11 },
    { 0x411d6ac9, SUC_lexicon_tags_14 },
    { 0x73dd6acb, SUC_lexicon_tags_7 },
    { 0x171d6acc, SUC_lexicon_tags_1 },
    { 0x847d6acc, SUC_lexicon_tags_16 },
    { 0x761d6acb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc89d6ad0, SUC_lexicon_tags_38 },
    { 0x467d6ad1, SUC_lexicon_tags_10 },
    { 0xbdbd6ad2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x2bfd6ad4, SUC_lexicon_tags_11 },
    { 0xc1dd6ad4, SUC_lexicon_tags_1 },
    { 0x32fd6ad6, SUC_lexicon_tags_37 },
    { 0x653d6ad7, SUC_lexicon_tags_25 },
    { 0xb7d6ad5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5d6adb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dd6ade, SUC_lexicon_tags_4 },
    { 0x4c9d6adf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671d6ae6, SUC_lexicon_tags_63 },
    { 0x321d6ae7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x257d6ae9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb75d6aeb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2cdd6aed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbd6af3, SUC_lexicon_tags_15 },
    { 0x5d1d6af3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x19bd6af6, SUC_lexicon_tags_15 },
    { 0x7a3d6af7, SUC_lexicon_tags_7 },
    { 0x353d6af8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfd6afb, SUC_lexicon_tags_23 },
    { 0xd6dd6afc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcabd6afe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3d6b08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x881d6b0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebd6b0d, SUC_lexicon_tags_0 },
    { 0x1d5d6b0d, SUC_lexicon_tags_12 },
    { 0x31fd6b0f, SUC_lexicon_tags_25 },
    { 0xb97d6b0f, SUC_lexicon_tags_7 },
    { 0xd2dd6b0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dd6b14, SUC_lexicon_tags_11 },
    { 0xde3d6b14, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x82bd6b18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x697d6b1b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87d6b1e, SUC_lexicon_tags_15 },
    { 0x40bd6b1e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x33d6b21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x693d6b23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487d6b27, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fd6b2c, SUC_lexicon_tags_2 },
    { 0x3ffd6b2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317d6b31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x707d6b33, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x229d6b36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63dd6b39, SUC_lexicon_tags_38 },
    { 0x929d6b39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xabdd6b3d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3d6b44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbd6b48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc99d6b4a, SUC_lexicon_tags_154 },
    { 0x2f9d6b4b, SUC_lexicon_tags_23 },
    { 0x259d6b4c, SUC_lexicon_tags_14 },
    { 0xb65d6b4d, SUC_lexicon_tags_37 },
    { 0xdf1d6b4d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x729d6b50, SUC_lexicon_tags_28 },
    { 0x4c9d6b51, SUC_lexicon_tags_14 },
    { 0x23bd6b52, SUC_lexicon_tags_27 },
    { 0x3f9d6b53, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffd6b5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafd6b64, SUC_lexicon_tags_37 },
    { 0xb45d6b64, SUC_lexicon_tags_11 },
    { 0x4a7d6b66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3d6b69, SUC_lexicon_tags_25 },
    { 0xfe5d6b6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d6b6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1d6b70, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6ddd6b72, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x455d6b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675d6b78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x305d6b7a, SUC_lexicon_tags_11 },
    { 0x2bdd6b7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13d6b7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62dd6b81, SUC_lexicon_tags_17 },
    { 0x353d6b82, SUC_lexicon_tags_7 },
    { 0x447d6b82, SUC_lexicon_tags_1 },
    { 0x2dfd6b81, SUC_lexicon_tags_7 },
    { 0x2c7d6b85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5edd6b88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d6b8c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69d6b93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x46dd6b95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417d6b99, SUC_lexicon_tags_12 },
    { 0x7cbd6b9a, SUC_lexicon_tags_47 },
    { 0xfbdd6b99, SUC_lexicon_tags_29 },
    { 0x4afd6b9b, SUC_lexicon_tags_33 },
    { 0xc99d6b99, SUC_lexicon_tags_49 },
    { 0xa3dd6b9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdd6ba2, SUC_lexicon_tags_54 },
    { 0xd91d6ba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7d6ba8, SUC_lexicon_tags_2 },
    { 0x509d6ba9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x413d6bab, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9d6baf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb19d6bb1, SUC_lexicon_tags_4 },
    { 0xad1d6bb2, SUC_lexicon_tags_8 },
    { 0x197d6bb3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3bfd6bb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fd6bb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45d6bbc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x679d6bbf, SUC_lexicon_tags_16 },
    { 0x35bd6bc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabfd6bc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85d6bcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x91fd6bcf, SUC_lexicon_tags_17 },
    { 0xeb7d6bd0, SUC_lexicon_tags_33 },
    { 0x775d6bd0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xadd6bd3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbd6bd6, SUC_lexicon_tags_10 },
    { 0x6ddd6bd7, SUC_lexicon_tags_1 },
    { 0x905d6bd7, SUC_lexicon_tags_11 },
    { 0x637d6bd9, SUC_lexicon_tags_19 },
    { 0x7e5d6bda, SUC_lexicon_tags_1 },
    { 0xe15d6bda, SUC_lexicon_tags_7 },
    { 0xaf7d6bdc, SUC_lexicon_tags_1 },
    { 0xb75d6bdd, SUC_lexicon_tags_25 },
    { 0xa97d6bde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbd6be2, SUC_lexicon_tags_27 },
    { 0x33bd6be3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa6bd6be5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdd6bec, SUC_lexicon_tags_189 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebd6bef, SUC_lexicon_tags_7 },
    { 0xa97d6bf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99d6bf3, SUC_lexicon_tags_12 },
    { 0xb55d6bf4, SUC_lexicon_tags_37 },
    { 0xff5d6bf5, SUC_lexicon_tags_47 },
    { 0xa47d6bf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7d6bfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x577d6bfc, SUC_lexicon_tags_11 },
    { 0x87fd6bfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fd6c01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9d6c09, SUC_lexicon_tags_31 },
    { 0xa5fd6c0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1dfd6c0c, SUC_lexicon_tags_7 },
    { 0xfbfd6c0c, SUC_lexicon_tags_11 },
    { 0xbddd6c0c, SUC_lexicon_tags_7 },
    { 0xf2dd6c0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07d6c13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547d6c17, SUC_lexicon_tags_3 },
    { 0x2f3d6c18, SUC_lexicon_tags_7 },
    { 0x3d1d6c19, SUC_lexicon_tags_6 },
    { 0xaa1d6c1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffd6c21, SUC_lexicon_tags_11 },
    { 0x829d6c22, SUC_lexicon_tags_8 },
    { 0x129d6c23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bd6c28, SUC_lexicon_tags_38 },
    { 0x517d6c28, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07d6c32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb29d6c34, SUC_lexicon_tags_37 },
    { 0xee3d6c35, SUC_lexicon_tags_15 },
    { 0x37fd6c36, SUC_lexicon_tags_25 },
    { 0xacbd6c37, SUC_lexicon_tags_61 },
    { 0x9bdd6c38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x145d6c3a, SUC_lexicon_tags_0 },
    { 0x467d6c3a, SUC_lexicon_tags_11 },
    { 0x5f1d6c3c, SUC_lexicon_tags_2 },
    { 0xcadd6c3b, SUC_lexicon_tags_63 },
    { 0x62dd6c3e, SUC_lexicon_tags_29 },
    { 0xee3d6c3e, SUC_lexicon_tags_15 },
    { 0xa37d6c40, SUC_lexicon_tags_7 },
    { 0x72dd6c3f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x875d6c44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdd6c48, SUC_lexicon_tags_38 },
    { 0xba3d6c49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2fd6c4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1d6c52, SUC_lexicon_tags_1345 },
    { 0, NULL },
    { 0, NULL },
    { 0x547d6c55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa59d6c57, SUC_lexicon_tags_48 },
    { 0xe31d6c58, SUC_lexicon_tags_14 },
    { 0xe99d6c59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7d6c63, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9d6c66, SUC_lexicon_tags_29 },
    { 0xd5d6c67, SUC_lexicon_tags_1 },
    { 0x7c5d6c68, SUC_lexicon_tags_25 },
    { 0x37d6c69, SUC_lexicon_tags_17 },
    { 0x321d6c6a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5d6c6f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bd6c73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d6c78, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3add6c7b, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bd6c7e, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dd6c81, SUC_lexicon_tags_9 },
    { 0x343d6c82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x861d6c85, SUC_lexicon_tags_47 },
    { 0x6a1d6c85, SUC_lexicon_tags_75 },
    { 0x839d6c87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39d6c8a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fd6c8f, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5d6c92, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d6c95, SUC_lexicon_tags_11 },
    { 0xb21d6c95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3d6c99, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xed9d6c9b, SUC_lexicon_tags_15 },
    { 0x94bd6c9b, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x65d6c9e, SUC_lexicon_tags_6 },
    { 0xedd6c9f, SUC_lexicon_tags_55 },
    { 0xcadd6c9f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609d6ca8, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x14dd6cab, SUC_lexicon_tags_64 },
    { 0xc85d6cac, SUC_lexicon_tags_46 },
    { 0x85fd6cad, SUC_lexicon_tags_7 },
    { 0xdfbd6cae, SUC_lexicon_tags_25 },
    { 0xce9d6caf, SUC_lexicon_tags_37 },
    { 0x699d6cb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45d6cb3, SUC_lexicon_tags_17 },
    { 0x3ffd6cb4, SUC_lexicon_tags_25 },
    { 0xd53d6cb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd99d6cb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeedd6cba, SUC_lexicon_tags_39 },
    { 0x117d6cba, SUC_lexicon_tags_49 },
    { 0x103d6cbc, SUC_lexicon_tags_63 },
    { 0x5e7d6cbd, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0x149d6cbf, SUC_lexicon_tags_25 },
    { 0xb4bd6cc0, SUC_lexicon_tags_30 },
    { 0x459d6cc0, SUC_lexicon_tags_25 },
    { 0x1dbd6cc2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x24bd6cc4, SUC_lexicon_tags_87 },
    { 0x97fd6cc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525d6ccb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3d6cd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6bbd6cd6, SUC_lexicon_tags_45 },
    { 0xec7d6cd6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x913d6cd9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7d6cdf, SUC_lexicon_tags_25 },
    { 0x9a3d6ce0, SUC_lexicon_tags_48 },
    { 0xfabd6ce1, SUC_lexicon_tags_15 },
    { 0x5cbd6ce1, SUC_lexicon_tags_43 },
    { 0x755d6ce3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xda3d6ce5, SUC_lexicon_tags_22 },
    { 0xf55d6ce6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdd6ceb, SUC_lexicon_tags_47 },
    { 0x359d6ceb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb9d6cee, SUC_lexicon_tags_31 },
    { 0xbcdd6cef, SUC_lexicon_tags_45 },
    { 0x72dd6cee, SUC_lexicon_tags_18 },
    { 0xed7d6cf1, SUC_lexicon_tags_29 },
    { 0xe69d6cf1, SUC_lexicon_tags_17 },
    { 0x343d6cf2, SUC_lexicon_tags_6 },
    { 0x2e3d6cf4, SUC_lexicon_tags_16 },
    { 0xc43d6cf5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xebbd6cf7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159d6cfb, SUC_lexicon_tags_12 },
    { 0xe9fd6cfb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdd6cff, SUC_lexicon_tags_31 },
    { 0x469d6d00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x191d6d02, SUC_lexicon_tags_23 },
    { 0xe87d6d03, SUC_lexicon_tags_12 },
    { 0xd39d6d03, SUC_lexicon_tags_1 },
    { 0x427d6d05, SUC_lexicon_tags_29 },
    { 0x29dd6d06, SUC_lexicon_tags_27 },
    { 0x94dd6d06, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1d6d0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bd6d0e, SUC_lexicon_tags_7 },
    { 0x447d6d0e, SUC_lexicon_tags_17 },
    { 0xdf3d6d0e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfd6d12, SUC_lexicon_tags_48 },
    { 0x8c3d6d13, SUC_lexicon_tags_10 },
    { 0xa35d6d14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19d6d16, SUC_lexicon_tags_37 },
    { 0xe31d6d16, SUC_lexicon_tags_12 },
    { 0xd7d6d16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5d6d1f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x641d6d21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd6d25, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221d6d29, SUC_lexicon_tags_12 },
    { 0xb49d6d29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbdd6d2c, SUC_lexicon_tags_37 },
    { 0x299d6d2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22bd6d2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7abd6d31, SUC_lexicon_tags_35 },
    { 0xb97d6d32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37d6d35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb43d6d37, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6efd6d39, SUC_lexicon_tags_8 },
    { 0x43bd6d3a, SUC_lexicon_tags_7 },
    { 0xfe5d6d3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9d6d3f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5d6d43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dd6d4c, SUC_lexicon_tags_2 },
    { 0xa33d6d4d, SUC_lexicon_tags_29 },
    { 0xce9d6d4e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x585d6d51, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x93dd6d54, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x85fd6d56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fd6d5d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49d6d61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebd6d65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbd6d69, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbd6d78, SUC_lexicon_tags_0 },
    { 0x41bd6d78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721d6d7d, SUC_lexicon_tags_47 },
    { 0x281d6d7e, SUC_lexicon_tags_7 },
    { 0xcadd6d7d, SUC_lexicon_tags_6 },
    { 0x3b5d6d7f, SUC_lexicon_tags_214 },
    { 0x6dfd6d81, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101d6d86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbd6d8a, SUC_lexicon_tags_29 },
    { 0x60fd6d8a, SUC_lexicon_tags_37 },
    { 0xaafd6d8a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbd6d90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x473d6d92, SUC_lexicon_tags_45 },
    { 0x8b9d6d92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dd6d96, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fd6d9b, SUC_lexicon_tags_6 },
    { 0xcdd6d9b, SUC_lexicon_tags_1 },
    { 0xde9d6d9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fd6da3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bd6da6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x587d6da9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf7fd6dab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1d6db1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x449d6db4, SUC_lexicon_tags_15 },
    { 0xa3d6db4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37d6db9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01d6dc1, SUC_lexicon_tags_17 },
    { 0x327d6dc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129d6dc7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3d6dcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1d6dd5, SUC_lexicon_tags_17 },
    { 0x96dd6dd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe55d6dd8, SUC_lexicon_tags_27 },
    { 0x6dd6dd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dd6ddf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x85d6de1, SUC_lexicon_tags_57 },
    { 0x3f9d6de1, SUC_lexicon_tags_45 },
    { 0xed1d6de3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509d6de7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe15d6de9, SUC_lexicon_tags_7 },
    { 0xce7d6dea, SUC_lexicon_tags_60 },
    { 0xc63d6deb, SUC_lexicon_tags_10 },
    { 0x953d6dea, SUC_lexicon_tags_15 },
    { 0x343d6dea, SUC_lexicon_tags_6 },
    { 0x83bd6ded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecfd6df0, SUC_lexicon_tags_27 },
    { 0xa2fd6df1, SUC_lexicon_tags_32 },
    { 0xd9fd6df1, SUC_lexicon_tags_12 },
    { 0xa89d6df0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x583d6df5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5d6dfa, SUC_lexicon_tags_37 },
    { 0x781d6dfb, SUC_lexicon_tags_12 },
    { 0x5ffd6dfc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe95d6dfe, SUC_lexicon_tags_25 },
    { 0x903d6dfe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bd6e04, SUC_lexicon_tags_15 },
    { 0x467d6e05, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa9d6e07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffd6e0d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fd6e10, SUC_lexicon_tags_10 },
    { 0x28fd6e11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x18bd6e14, SUC_lexicon_tags_25 },
    { 0x7f5d6e15, SUC_lexicon_tags_7 },
    { 0xdcfd6e14, SUC_lexicon_tags_17 },
    { 0xe4fd6e17, SUC_lexicon_tags_39 },
    { 0xd23d6e17, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbd6e1e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xffbd6e21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xebdd6e23, SUC_lexicon_tags_2 },
    { 0xfb3d6e23, SUC_lexicon_tags_82 },
    { 0x65fd6e23, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x187d6e27, SUC_lexicon_tags_23 },
    { 0x16fd6e28, SUC_lexicon_tags_27 },
    { 0x473d6e29, SUC_lexicon_tags_17 },
    { 0x6b3d6e2a, SUC_lexicon_tags_11 },
    { 0x5fbd6e2b, SUC_lexicon_tags_1 },
    { 0x3b9d6e2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5d6e31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb8dd6e33, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dd6e37, SUC_lexicon_tags_0 },
    { 0xf9bd6e37, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389d6e3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2dd6e3f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7f3d6e41, SUC_lexicon_tags_12 },
    { 0xbf7d6e41, SUC_lexicon_tags_25 },
    { 0xd2dd6e43, SUC_lexicon_tags_19 },
    { 0x517d6e41, SUC_lexicon_tags_87 },
    { 0xce3d6e45, SUC_lexicon_tags_38 },
    { 0xf6bd6e42, SUC_lexicon_tags_12 },
    { 0xfb7d6e47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33d6e4e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dd6e54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11d6e5b, SUC_lexicon_tags_2 },
    { 0xcd7d6e5b, SUC_lexicon_tags_37 },
    { 0xf8dd6e5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fd6e60, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x92fd6e62, SUC_lexicon_tags_49 },
    { 0x513d6e63, SUC_lexicon_tags_25 },
    { 0x6c3d6e64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x331d6e67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9d6e6a, SUC_lexicon_tags_2 },
    { 0xb9dd6e6b, SUC_lexicon_tags_14 },
    { 0x971d6e6b, SUC_lexicon_tags_1 },
    { 0x2a1d6e6d, SUC_lexicon_tags_45 },
    { 0x173d6e6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109d6e73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fd6e76, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf4dd6e78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x17bd6e7a, SUC_lexicon_tags_25 },
    { 0xb83d6e7b, SUC_lexicon_tags_48 },
    { 0x241d6e7b, SUC_lexicon_tags_37 },
    { 0x4c3d6e7d, SUC_lexicon_tags_27 },
    { 0xa43d6e7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x451d6e80, SUC_lexicon_tags_29 },
    { 0xbe1d6e81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53d6e84, SUC_lexicon_tags_8 },
    { 0x8b5d6e85, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421d6e89, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35d6e8c, SUC_lexicon_tags_23 },
    { 0x875d6e8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9bbd6e8f, SUC_lexicon_tags_25 },
    { 0x40bd6e90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef1d6e92, SUC_lexicon_tags_17 },
    { 0x65fd6e93, SUC_lexicon_tags_26 },
    { 0x8b7d6e92, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x903d6e97, SUC_lexicon_tags_26 },
    { 0x729d6e98, SUC_lexicon_tags_11 },
    { 0xa9fd6e99, SUC_lexicon_tags_10 },
    { 0x6f3d6e97, SUC_lexicon_tags_2 },
    { 0xbf3d6e97, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdd6e9f, SUC_lexicon_tags_65 },
    { 0x98dd6ea0, SUC_lexicon_tags_45 },
    { 0xbfd6ea1, SUC_lexicon_tags_11 },
    { 0x79bd6e9f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4d9d6ea4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65d6ea8, SUC_lexicon_tags_23 },
    { 0x35fd6ea8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23d6eac, SUC_lexicon_tags_178 },
    { 0x481d6ead, SUC_lexicon_tags_41 },
    { 0x98fd6eae, SUC_lexicon_tags_3 },
    { 0xd7dd6eae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597d6eb5, SUC_lexicon_tags_12 },
    { 0x779d6eb5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfd6eb9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x727d6ebc, SUC_lexicon_tags_10 },
    { 0x749d6ebd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd49d6ebf, SUC_lexicon_tags_15 },
    { 0x887d6ec0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9d6ec3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb63d6ec5, SUC_lexicon_tags_49 },
    { 0x6fd6ec6, SUC_lexicon_tags_30 },
    { 0xd47d6ec7, SUC_lexicon_tags_12 },
    { 0xdfdd6ec8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc5dd6eca, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xe3fd6ecc, SUC_lexicon_tags_61 },
    { 0x21fd6ecc, SUC_lexicon_tags_37 },
    { 0x98fd6ece, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7d6ed2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa2dd6ed4, SUC_lexicon_tags_39 },
    { 0xe97d6ed4, SUC_lexicon_tags_37 },
    { 0x865d6ed6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x35dd6ed8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7d6edb, SUC_lexicon_tags_2 },
    { 0x217d6edc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981d6ee0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3d6ee4, SUC_lexicon_tags_15 },
    { 0x9efd6ee5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bd6ee8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x989d6eea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779d6eef, SUC_lexicon_tags_17 },
    { 0xad7d6eef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4c7d6ef2, SUC_lexicon_tags_25 },
    { 0xb7bd6ef3, SUC_lexicon_tags_77 },
    { 0x1d3d6ef3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abd6ef8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x323d6efb, SUC_lexicon_tags_11 },
    { 0x7bd6efc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9d6f00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x589d6f02, SUC_lexicon_tags_29 },
    { 0x753d6f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3d6f09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631d6f0d, SUC_lexicon_tags_29 },
    { 0x109d6f0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77dd6f10, SUC_lexicon_tags_30 },
    { 0xffbd6f11, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb99d6f13, SUC_lexicon_tags_17 },
    { 0xe09d6f14, SUC_lexicon_tags_25 },
    { 0xe4fd6f13, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dd6f1a, SUC_lexicon_tags_14 },
    { 0x593d6f1b, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x263d6f1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1d6f25, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d6f28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafd6f2e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dd6f36, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4cbd6f38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bd6f3f, SUC_lexicon_tags_15 },
    { 0x489d6f3f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x85dd6f42, SUC_lexicon_tags_15 },
    { 0x8d1d6f42, SUC_lexicon_tags_7 },
    { 0xeefd6f43, SUC_lexicon_tags_7 },
    { 0x27bd6f45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1d6f49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa05d6f4b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25d6f4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x389d6f50, SUC_lexicon_tags_17 },
    { 0xe63d6f51, SUC_lexicon_tags_80 },
    { 0xfd1d6f51, SUC_lexicon_tags_48 },
    { 0x347d6f51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727d6f58, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd33d6f5a, SUC_lexicon_tags_14 },
    { 0x5f7d6f5b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x563d6f5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdd6f66, SUC_lexicon_tags_2 },
    { 0xdadd6f66, SUC_lexicon_tags_38 },
    { 0xb73d6f67, SUC_lexicon_tags_25 },
    { 0x621d6f69, SUC_lexicon_tags_28 },
    { 0xf03d6f6a, SUC_lexicon_tags_11 },
    { 0xa67d6f6b, SUC_lexicon_tags_56 },
    { 0xfd3d6f6a, SUC_lexicon_tags_27 },
    { 0x87d6f6b, SUC_lexicon_tags_37 },
    { 0xbafd6f68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43d6f71, SUC_lexicon_tags_43 },
    { 0xcfdd6f72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9d6f76, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fd6f79, SUC_lexicon_tags_31 },
    { 0x81dd6f7a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcafd6f7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x981d6f7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fd6f83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11d6f86, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fd6f8d, SUC_lexicon_tags_47 },
    { 0xa71d6f8e, SUC_lexicon_tags_25 },
    { 0x961d6f8f, SUC_lexicon_tags_1 },
    { 0x4a5d6f8e, SUC_lexicon_tags_7 },
    { 0x1edd6f8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7a7d6f93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833d6f97, SUC_lexicon_tags_200 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fd6f9d, SUC_lexicon_tags_38 },
    { 0x295d6f9e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x289d6fa0, SUC_lexicon_tags_49 },
    { 0x2f5d6fa1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769d6fa5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5d6fad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bd6fb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2f3d6fb8, SUC_lexicon_tags_25 },
    { 0x1add6fb9, SUC_lexicon_tags_17 },
    { 0x38dd6fba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25d6fc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5dbd6fc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf1d6fc4, SUC_lexicon_tags_23 },
    { 0xa7fd6fc4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x21bd6fc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1d6fcc, SUC_lexicon_tags_1 },
    { 0x1c9d6fcd, SUC_lexicon_tags_28 },
    { 0x95dd6fce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907d6fda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf93d6fdc, SUC_lexicon_tags_19 },
    { 0x61bd6fdd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb7d6fdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77d6fe4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79d6ff5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65d6ff9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb1bd6ffb, SUC_lexicon_tags_15 },
    { 0x98bd6ffc, SUC_lexicon_tags_774 },
    { 0x395d6ffb, SUC_lexicon_tags_59 },
    { 0x605d6ffb, SUC_lexicon_tags_23 },
    { 0xfe5d6ffd, SUC_lexicon_tags_84 },
    { 0x11bd7000, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x623d7002, SUC_lexicon_tags_32 },
    { 0x61fd7002, SUC_lexicon_tags_66 },
    { 0xa6fd7003, SUC_lexicon_tags_2 },
    { 0x301d7003, SUC_lexicon_tags_1 },
    { 0x49d7006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x255d7008, SUC_lexicon_tags_2 },
    { 0x843d7008, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3dfd700b, SUC_lexicon_tags_12 },
    { 0x1cdd700c, SUC_lexicon_tags_30 },
    { 0x93dd700b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdd7011, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95d7015, SUC_lexicon_tags_0 },
    { 0x571d7016, SUC_lexicon_tags_12 },
    { 0xe91d7016, SUC_lexicon_tags_16 },
    { 0x7d5d7015, SUC_lexicon_tags_1 },
    { 0xddd7018, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781d701f, SUC_lexicon_tags_25 },
    { 0x6fd701f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x703d7022, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05d7026, SUC_lexicon_tags_24 },
    { 0x6e5d7026, SUC_lexicon_tags_17 },
    { 0x691d7027, SUC_lexicon_tags_37 },
    { 0x8d7d7029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9d702e, SUC_lexicon_tags_47 },
    { 0x3d5d702f, SUC_lexicon_tags_1 },
    { 0xea7d702f, SUC_lexicon_tags_1 },
    { 0xd61d7031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfd7035, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817d7039, SUC_lexicon_tags_15 },
    { 0xdf3d703a, SUC_lexicon_tags_2 },
    { 0x11fd703b, SUC_lexicon_tags_37 },
    { 0x7d3d703c, SUC_lexicon_tags_17 },
    { 0xf5bd703d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fd7041, SUC_lexicon_tags_8 },
    { 0x683d7041, SUC_lexicon_tags_2 },
    { 0xb4bd7042, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc11d7045, SUC_lexicon_tags_2 },
    { 0xb5dd7046, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c1d7048, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfd704b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x769d704d, SUC_lexicon_tags_4 },
    { 0xc91d704e, SUC_lexicon_tags_17 },
    { 0xf25d704d, SUC_lexicon_tags_11 },
    { 0xd2dd7050, SUC_lexicon_tags_25 },
    { 0xf5fd7051, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1d7055, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d7058, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa61d705b, SUC_lexicon_tags_2 },
    { 0x5c9d705c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x851d705e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bd7063, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf3d7065, SUC_lexicon_tags_8 },
    { 0x8a5d7065, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdd7069, SUC_lexicon_tags_1 },
    { 0x8c5d7069, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b9d706c, SUC_lexicon_tags_59 },
    { 0x669d706d, SUC_lexicon_tags_23 },
    { 0x1bfd706e, SUC_lexicon_tags_64 },
    { 0x21d706f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdd7073, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x367d7076, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdd1d7078, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dd707c, SUC_lexicon_tags_2 },
    { 0xdd3d707d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfc9d707f, SUC_lexicon_tags_25 },
    { 0x38bd707f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x347d7082, SUC_lexicon_tags_1 },
    { 0xda1d7083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61d7086, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5d708f, SUC_lexicon_tags_17 },
    { 0xff5d7090, SUC_lexicon_tags_25 },
    { 0xde1d7090, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54bd7093, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955d7098, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9f9d709a, SUC_lexicon_tags_77 },
    { 0x811d709b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7d709f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3d70a8, SUC_lexicon_tags_7 },
    { 0xa31d70a9, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x9bd70ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933d70b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x763d70b3, SUC_lexicon_tags_29 },
    { 0xd8fd70b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4e7d70b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbd70b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71d70bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75d70c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737d70c6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdd70c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dd70ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfd70d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1d70da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x105d70dd, SUC_lexicon_tags_37 },
    { 0xd37d70de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3cdd70e0, SUC_lexicon_tags_0 },
    { 0x405d70e0, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xfc5d70e3, SUC_lexicon_tags_783 },
    { 0x779d70e3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93d70eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bd70f0, SUC_lexicon_tags_19 },
    { 0xf99d70f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bd70f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bd70f9, SUC_lexicon_tags_11 },
    { 0x5e5d70f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d70ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fd7102, SUC_lexicon_tags_14 },
    { 0x807d7102, SUC_lexicon_tags_46 },
    { 0x131d7104, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91d710c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfbdd710e, SUC_lexicon_tags_14 },
    { 0x50dd710e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x527d7111, SUC_lexicon_tags_25 },
    { 0xcbfd7112, SUC_lexicon_tags_38 },
    { 0x5a1d7111, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x835d7116, SUC_lexicon_tags_2 },
    { 0x1e1d7116, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1d711a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bd711e, SUC_lexicon_tags_56 },
    { 0x117d711f, SUC_lexicon_tags_68 },
    { 0x317d7120, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fd7123, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fd7126, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17d7129, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bd712d, SUC_lexicon_tags_66 },
    { 0xa0fd712e, SUC_lexicon_tags_11 },
    { 0x569d712d, SUC_lexicon_tags_48 },
    { 0xd83d712e, SUC_lexicon_tags_37 },
    { 0xb4fd712d, SUC_lexicon_tags_17 },
    { 0x9f3d712d, SUC_lexicon_tags_14 },
    { 0x32bd7133, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123d7137, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9d713d, SUC_lexicon_tags_33 },
    { 0x61dd713e, SUC_lexicon_tags_1 },
    { 0x59bd713f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb31d7141, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11d7144, SUC_lexicon_tags_6 },
    { 0x38dd7145, SUC_lexicon_tags_25 },
    { 0x6dfd7146, SUC_lexicon_tags_15 },
    { 0x3e7d7145, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05d7155, SUC_lexicon_tags_18 },
    { 0x623d7156, SUC_lexicon_tags_32 },
    { 0x229d7155, SUC_lexicon_tags_151 },
    { 0x48dd7156, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x529d715a, SUC_lexicon_tags_12 },
    { 0x39fd715b, SUC_lexicon_tags_11 },
    { 0xe69d715b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdb3d715e, SUC_lexicon_tags_37 },
    { 0xdfd715f, SUC_lexicon_tags_25 },
    { 0x3d1d7160, SUC_lexicon_tags_11 },
    { 0xd1bd7160, SUC_lexicon_tags_15 },
    { 0x5d3d715f, SUC_lexicon_tags_1 },
    { 0x213d715f, SUC_lexicon_tags_11 },
    { 0x49d7162, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xde5d7166, SUC_lexicon_tags_1 },
    { 0xfdbd7167, SUC_lexicon_tags_11 },
    { 0xd75d7167, SUC_lexicon_tags_38 },
    { 0xf7bd7168, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1d716d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dd7170, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3b1d7172, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fd717b, SUC_lexicon_tags_17 },
    { 0xa71d717c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115d7181, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1d7185, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bd718a, SUC_lexicon_tags_14 },
    { 0x821d718b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375d7193, SUC_lexicon_tags_25 },
    { 0xb45d7193, SUC_lexicon_tags_34 },
    { 0x5bbd7195, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfd719a, SUC_lexicon_tags_7 },
    { 0x581d719b, SUC_lexicon_tags_27 },
    { 0x9e7d719a, SUC_lexicon_tags_1 },
    { 0x8cfd719d, SUC_lexicon_tags_1 },
    { 0x51d719d, SUC_lexicon_tags_25 },
    { 0xd5bd719f, SUC_lexicon_tags_1 },
    { 0xf3fd71a0, SUC_lexicon_tags_11 },
    { 0x793d71a1, SUC_lexicon_tags_12 },
    { 0xfddd71a0, SUC_lexicon_tags_37 },
    { 0x5dd71a0, SUC_lexicon_tags_6 },
    { 0xb29d71a4, SUC_lexicon_tags_56 },
    { 0x453d71a5, SUC_lexicon_tags_39 },
    { 0xa6fd71a4, SUC_lexicon_tags_11 },
    { 0xa25d71a4, SUC_lexicon_tags_37 },
    { 0x5a1d71a5, SUC_lexicon_tags_43 },
    { 0xfe3d71a9, SUC_lexicon_tags_37 },
    { 0x177d71a3, SUC_lexicon_tags_37 },
    { 0x69d71a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x343d71ad, SUC_lexicon_tags_1 },
    { 0x76fd71ad, SUC_lexicon_tags_37 },
    { 0xc75d71af, SUC_lexicon_tags_1 },
    { 0xb01d71b0, SUC_lexicon_tags_47 },
    { 0x5a7d71b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9d71b4, SUC_lexicon_tags_25 },
    { 0x7cfd71b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2d7d71b7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bd71bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc05d71c1, SUC_lexicon_tags_12 },
    { 0x9bbd71c2, SUC_lexicon_tags_46 },
    { 0x80fd71c3, SUC_lexicon_tags_16 },
    { 0x415d71c2, SUC_lexicon_tags_2 },
    { 0xb27d71c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fd71cb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abd71d1, SUC_lexicon_tags_11 },
    { 0x2bfd71d1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fd71d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc2fd71d8, SUC_lexicon_tags_45 },
    { 0xe17d71d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49bd71db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7cfd71dd, SUC_lexicon_tags_15 },
    { 0x699d71de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbd71e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bd71e8, SUC_lexicon_tags_33 },
    { 0xe53d71e9, SUC_lexicon_tags_11 },
    { 0xca1d71ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbd71ed, SUC_lexicon_tags_4 },
    { 0x2f9d71ee, SUC_lexicon_tags_30 },
    { 0xc09d71ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x453d71f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3d71fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5e7d7200, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84fd7202, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd9d7204, SUC_lexicon_tags_17 },
    { 0x741d7204, SUC_lexicon_tags_15 },
    { 0x977d7205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dd7212, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x45bd7214, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x277d7217, SUC_lexicon_tags_1 },
    { 0x855d7218, SUC_lexicon_tags_65 },
    { 0xbadd7217, SUC_lexicon_tags_21 },
    { 0x5f7d721a, SUC_lexicon_tags_55 },
    { 0x89bd721b, SUC_lexicon_tags_43 },
    { 0xf8fd721c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c3d721e, SUC_lexicon_tags_17 },
    { 0x25dd721f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0bd7222, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd7227, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03d7230, SUC_lexicon_tags_45 },
    { 0xaefd7230, SUC_lexicon_tags_16 },
    { 0xd75d7231, SUC_lexicon_tags_7 },
    { 0x3e9d7233, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x989d7236, SUC_lexicon_tags_37 },
    { 0xbbdd7236, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x533d7239, SUC_lexicon_tags_7 },
    { 0xb15d723a, SUC_lexicon_tags_26 },
    { 0xed5d723b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5d7242, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5d7246, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abd7249, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbd724c, SUC_lexicon_tags_63 },
    { 0x1a5d724c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1d7254, SUC_lexicon_tags_21 },
    { 0x84dd7255, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03d7259, SUC_lexicon_tags_8 },
    { 0x837d725a, SUC_lexicon_tags_0 },
    { 0x85fd725a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3d7260, SUC_lexicon_tags_45 },
    { 0xaa9d7261, SUC_lexicon_tags_26 },
    { 0xab9d7262, SUC_lexicon_tags_7 },
    { 0x159d7262, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe5dd7265, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7dd7267, SUC_lexicon_tags_45 },
    { 0xcdfd7268, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x851d726a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f9d726c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715d7271, SUC_lexicon_tags_1 },
    { 0x329d7272, SUC_lexicon_tags_10 },
    { 0x9c7d7271, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa1d7275, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x321d7278, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dd727c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7d7281, SUC_lexicon_tags_9 },
    { 0x625d7281, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9d7287, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x87bd728a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71d7291, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x99d7293, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3d7296, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dd729c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4bdd729e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1c1d72a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x855d72a2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539d72ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67d72b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba9d72b3, SUC_lexicon_tags_43 },
    { 0x255d72b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dd72c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87d72c8, SUC_lexicon_tags_61 },
    { 0x1f9d72c9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1d72ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x51bd72d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107d72d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03d72da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x231d72dc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90fd72de, SUC_lexicon_tags_25 },
    { 0x621d72df, SUC_lexicon_tags_1 },
    { 0x321d72e0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939d72e8, SUC_lexicon_tags_26 },
    { 0x603d72e9, SUC_lexicon_tags_37 },
    { 0x2b9d72ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x211d72ed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93d72f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bd72f7, SUC_lexicon_tags_37 },
    { 0x565d72f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc7dd72fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9d7301, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63d7304, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd49d7306, SUC_lexicon_tags_14 },
    { 0xc4dd7307, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bd730b, SUC_lexicon_tags_15 },
    { 0x3bfd730c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3d7316, SUC_lexicon_tags_47 },
    { 0x5c3d7317, SUC_lexicon_tags_25 },
    { 0x287d7318, SUC_lexicon_tags_14 },
    { 0x3b5d7316, SUC_lexicon_tags_34 },
    { 0x19dd7318, SUC_lexicon_tags_15 },
    { 0xb4dd731b, SUC_lexicon_tags_42 },
    { 0xbffd731c, SUC_lexicon_tags_32 },
    { 0xd47d731d, SUC_lexicon_tags_17 },
    { 0xe2dd731e, SUC_lexicon_tags_45 },
    { 0x78dd731b, SUC_lexicon_tags_47 },
    { 0x63dd7320, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x157d7322, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb57d7324, SUC_lexicon_tags_1 },
    { 0xdd7324, SUC_lexicon_tags_7 },
    { 0xcadd7326, SUC_lexicon_tags_17 },
    { 0xba1d7327, SUC_lexicon_tags_7 },
    { 0x9e7d7326, SUC_lexicon_tags_29 },
    { 0x289d7329, SUC_lexicon_tags_0 },
    { 0x5bd7324, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae7d732c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1d733b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1d7347, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd734a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x353d734c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x12bd734e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaebd7350, SUC_lexicon_tags_15 },
    { 0x63d7350, SUC_lexicon_tags_19 },
    { 0xeabd7350, SUC_lexicon_tags_38 },
    { 0xeb7d7353, SUC_lexicon_tags_47 },
    { 0x809d7351, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3d7359, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dd735d, SUC_lexicon_tags_12 },
    { 0x773d735e, SUC_lexicon_tags_47 },
    { 0x231d735f, SUC_lexicon_tags_1 },
    { 0x633d7360, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7d7367, SUC_lexicon_tags_11 },
    { 0x78dd7368, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bd736f, SUC_lexicon_tags_2 },
    { 0x533d7370, SUC_lexicon_tags_3 },
    { 0x4f1d7371, SUC_lexicon_tags_1 },
    { 0x953d7372, SUC_lexicon_tags_17 },
    { 0xa27d7373, SUC_lexicon_tags_37 },
    { 0x997d7374, SUC_lexicon_tags_40 },
    { 0x64fd7370, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x129d7377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1d737b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bd737f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bd7384, SUC_lexicon_tags_33 },
    { 0x503d7385, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x20bd7387, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979d738c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x161d738f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dd7395, SUC_lexicon_tags_14 },
    { 0xf05d7396, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf57d7398, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17fd739a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x717d739c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3d739f, SUC_lexicon_tags_15 },
    { 0xae7d73a0, SUC_lexicon_tags_0 },
    { 0xa69d73a1, SUC_lexicon_tags_26 },
    { 0x71fd739f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x291d73a5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61d73a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd73ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xffbd73ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x401d73b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dd73b5, SUC_lexicon_tags_43 },
    { 0x149d73b6, SUC_lexicon_tags_30 },
    { 0x4a7d73b6, SUC_lexicon_tags_179 },
    { 0xadbd73b8, SUC_lexicon_tags_37 },
    { 0x80dd73b6, SUC_lexicon_tags_19 },
    { 0xa45d73ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbabd73bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41d73bf, SUC_lexicon_tags_15 },
    { 0x3efd73bf, SUC_lexicon_tags_11 },
    { 0xcefd73c1, SUC_lexicon_tags_27 },
    { 0x30dd73c2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd73c6, SUC_lexicon_tags_8 },
    { 0x78bd73c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfd73ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37d73cc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x937d73ce, SUC_lexicon_tags_26 },
    { 0x33d73cf, SUC_lexicon_tags_60 },
    { 0x56fd73d0, SUC_lexicon_tags_2 },
    { 0x5c1d73d1, SUC_lexicon_tags_25 },
    { 0x6b1d73d2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x711d73d5, SUC_lexicon_tags_10 },
    { 0x139d73d6, SUC_lexicon_tags_14 },
    { 0xdedd73d7, SUC_lexicon_tags_27 },
    { 0x3b5d73d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171d73dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeabd73df, SUC_lexicon_tags_23 },
    { 0x83fd73e0, SUC_lexicon_tags_37 },
    { 0xeb1d73e1, SUC_lexicon_tags_12 },
    { 0x9bfd73e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd83d73e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453d73e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39d73ec, SUC_lexicon_tags_37 },
    { 0x147d73ec, SUC_lexicon_tags_23 },
    { 0x40dd73ee, SUC_lexicon_tags_11 },
    { 0xadd73ee, SUC_lexicon_tags_8 },
    { 0x2c7d73f0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x55d73f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1d73f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x33d73f9, SUC_lexicon_tags_6 },
    { 0x8b3d73f9, SUC_lexicon_tags_36 },
    { 0xa23d73fb, SUC_lexicon_tags_28 },
    { 0x89fd73f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69d73ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bd7405, SUC_lexicon_tags_37 },
    { 0xd7bd7406, SUC_lexicon_tags_60 },
    { 0x47d7407, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5d740c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc43d740e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x187d7410, SUC_lexicon_tags_42 },
    { 0x9a1d7410, SUC_lexicon_tags_17 },
    { 0x239d7410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x25d7414, SUC_lexicon_tags_63 },
    { 0x371d7415, SUC_lexicon_tags_1 },
    { 0xb1dd7415, SUC_lexicon_tags_15 },
    { 0x8c9d7417, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x741d7419, SUC_lexicon_tags_11 },
    { 0xecfd7419, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5d741d, SUC_lexicon_tags_11 },
    { 0x44fd741e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173d7425, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bd7428, SUC_lexicon_tags_4 },
    { 0x55dd7429, SUC_lexicon_tags_12 },
    { 0x5b5d742a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x449d742d, SUC_lexicon_tags_14 },
    { 0x255d742e, SUC_lexicon_tags_113 },
    { 0x889d742f, SUC_lexicon_tags_38 },
    { 0xdd5d742d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x971d7433, SUC_lexicon_tags_7 },
    { 0x15dd7433, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdd7438, SUC_lexicon_tags_44 },
    { 0x65fd7438, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4bd743b, SUC_lexicon_tags_45 },
    { 0xffbd743c, SUC_lexicon_tags_29 },
    { 0x1ddd743c, SUC_lexicon_tags_15 },
    { 0x829d743e, SUC_lexicon_tags_17 },
    { 0x915d743f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5d7443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9d7445, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5d7449, SUC_lexicon_tags_52 },
    { 0xd57d744a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x41bd744c, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xce1d744e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27dd7455, SUC_lexicon_tags_37 },
    { 0xe41d7455, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedd745c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79d7463, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4fbd7465, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bd7468, SUC_lexicon_tags_37 },
    { 0x969d7469, SUC_lexicon_tags_28 },
    { 0x4c5d7469, SUC_lexicon_tags_2 },
    { 0x49fd746b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x13dd746d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7d7470, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169d7476, SUC_lexicon_tags_0 },
    { 0x4a5d7476, SUC_lexicon_tags_37 },
    { 0x421d7477, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7d747b, SUC_lexicon_tags_46 },
    { 0x8dbd747b, SUC_lexicon_tags_25 },
    { 0xd91d747d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3d7480, SUC_lexicon_tags_229 },
    { 0x60dd7481, SUC_lexicon_tags_23 },
    { 0x88bd7481, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dd7485, SUC_lexicon_tags_17 },
    { 0x9e1d7485, SUC_lexicon_tags_13 },
    { 0xdf3d7485, SUC_lexicon_tags_26 },
    { 0xa53d7488, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf71d748a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffd748f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d7493, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433d7497, SUC_lexicon_tags_10 },
    { 0xe35d7497, SUC_lexicon_tags_1 },
    { 0xfe9d7499, SUC_lexicon_tags_66 },
    { 0x3f5d7498, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abd74a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dd74a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5d74ac, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949d74b3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555d74ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157d74bf, SUC_lexicon_tags_33 },
    { 0x71bd74c0, SUC_lexicon_tags_21 },
    { 0x131d74c0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x47fd74c3, SUC_lexicon_tags_23 },
    { 0x69d74c4, SUC_lexicon_tags_17 },
    { 0x9d74c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdd74cd, SUC_lexicon_tags_37 },
    { 0xe6bd74ce, SUC_lexicon_tags_27 },
    { 0x977d74ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fd74d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79d74da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bd74dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d74e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17d74e4, SUC_lexicon_tags_14 },
    { 0xc63d74e5, SUC_lexicon_tags_2 },
    { 0x42dd74e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72fd74e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987d74ec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bd74f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacbd74f3, SUC_lexicon_tags_63 },
    { 0x285d74f4, SUC_lexicon_tags_27 },
    { 0x391d74f5, SUC_lexicon_tags_7 },
    { 0x795d74f6, SUC_lexicon_tags_2 },
    { 0x81dd74f7, SUC_lexicon_tags_43 },
    { 0xf33d74f8, SUC_lexicon_tags_12 },
    { 0x3efd74f9, SUC_lexicon_tags_10 },
    { 0xee3d74f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x151d74fc, SUC_lexicon_tags_6 },
    { 0xa6dd74fd, SUC_lexicon_tags_4 },
    { 0x8afd74fe, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3f7d7500, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5d7509, SUC_lexicon_tags_51 },
    { 0xda9d7509, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x48bd750c, SUC_lexicon_tags_17 },
    { 0xb25d750d, SUC_lexicon_tags_31 },
    { 0x4bbd750d, SUC_lexicon_tags_26 },
    { 0x9c7d750c, SUC_lexicon_tags_15 },
    { 0xebd7510, SUC_lexicon_tags_14 },
    { 0x55dd7511, SUC_lexicon_tags_37 },
    { 0x4bdd750c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3d7515, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54dd751b, SUC_lexicon_tags_49 },
    { 0x989d751c, SUC_lexicon_tags_15 },
    { 0x329d751d, SUC_lexicon_tags_30 },
    { 0x6edd751b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x327d7520, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7d7524, SUC_lexicon_tags_2 },
    { 0xeadd7524, SUC_lexicon_tags_30 },
    { 0xe1d7526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x803d7529, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x89d752c, SUC_lexicon_tags_9 },
    { 0x867d752c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1d752f, SUC_lexicon_tags_62 },
    { 0x7cfd7530, SUC_lexicon_tags_1 },
    { 0x965d752f, SUC_lexicon_tags_7 },
    { 0xe45d7530, SUC_lexicon_tags_7 },
    { 0x8dd752f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917d7538, SUC_lexicon_tags_456 },
    { 0x313d7539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fd753c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x21dd753e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x461d7540, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1d7545, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d754a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113d7550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf19d7552, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9d755a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dd7561, SUC_lexicon_tags_31 },
    { 0xa67d7562, SUC_lexicon_tags_17 },
    { 0x8cfd7563, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbd7566, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa43d7568, SUC_lexicon_tags_49 },
    { 0xc11d7569, SUC_lexicon_tags_23 },
    { 0xd7d7569, SUC_lexicon_tags_160 },
    { 0x989d7569, SUC_lexicon_tags_15 },
    { 0xbf3d756c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13d756f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfd7572, SUC_lexicon_tags_26 },
    { 0xb33d7573, SUC_lexicon_tags_33 },
    { 0x301d7574, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x801d7577, SUC_lexicon_tags_25 },
    { 0x731d7578, SUC_lexicon_tags_37 },
    { 0xaabd7579, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bd757d, SUC_lexicon_tags_9 },
    { 0xc23d757e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x18fd7580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bd7585, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7d7589, SUC_lexicon_tags_15 },
    { 0x57d758a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddd758d, SUC_lexicon_tags_39 },
    { 0x5d758e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d9d7590, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd7596, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93d759a, SUC_lexicon_tags_5 },
    { 0xca3d759a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dd759f, SUC_lexicon_tags_25 },
    { 0x8efd75a0, SUC_lexicon_tags_19 },
    { 0xb85d75a1, SUC_lexicon_tags_38 },
    { 0xa97d75a2, SUC_lexicon_tags_49 },
    { 0xf0fd75a3, SUC_lexicon_tags_7 },
    { 0xc9dd75a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x321d75a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9d75ac, SUC_lexicon_tags_16 },
    { 0x7ffd75ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x44bd75b0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781d75b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7e3d75b9, SUC_lexicon_tags_22 },
    { 0xa43d75ba, SUC_lexicon_tags_43 },
    { 0x9d5d75bb, SUC_lexicon_tags_7 },
    { 0x1d75bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe91d75be, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3d75c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x211d75c3, SUC_lexicon_tags_11 },
    { 0x62dd75c4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bd75c7, SUC_lexicon_tags_1 },
    { 0x23bd75c8, SUC_lexicon_tags_38 },
    { 0x455d75c8, SUC_lexicon_tags_17 },
    { 0x21d75ca, SUC_lexicon_tags_15 },
    { 0xb63d75cb, SUC_lexicon_tags_19 },
    { 0x87dd75cc, SUC_lexicon_tags_45 },
    { 0xa27d75cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd75cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b5d75d1, SUC_lexicon_tags_7 },
    { 0x789d75d1, SUC_lexicon_tags_19 },
    { 0xfdfd75d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fd75d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdf9d75d8, SUC_lexicon_tags_55 },
    { 0x187d75d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211d75dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf0bd75df, SUC_lexicon_tags_40 },
    { 0x5e5d75e0, SUC_lexicon_tags_23 },
    { 0x515d75df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd5d75e3, SUC_lexicon_tags_11 },
    { 0x57dd75e4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x311d75e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365d75eb, SUC_lexicon_tags_29 },
    { 0x7f1d75ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fd75f1, SUC_lexicon_tags_7 },
    { 0x5ebd75f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8efd75f4, SUC_lexicon_tags_194 },
    { 0, NULL },
    { 0xf3d75f6, SUC_lexicon_tags_27 },
    { 0xc4bd75f7, SUC_lexicon_tags_31 },
    { 0x745d75f8, SUC_lexicon_tags_17 },
    { 0xf0fd75f9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x189d75fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xddfd75fd, SUC_lexicon_tags_47 },
    { 0x4edd75fd, SUC_lexicon_tags_0 },
    { 0xd35d75ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fd7602, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381d7606, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fd7609, SUC_lexicon_tags_9 },
    { 0xd23d7609, SUC_lexicon_tags_11 },
    { 0xfd9d760b, SUC_lexicon_tags_106 },
    { 0xab1d760b, SUC_lexicon_tags_7 },
    { 0x745d760d, SUC_lexicon_tags_11 },
    { 0xeb5d760e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dd7614, SUC_lexicon_tags_29 },
    { 0x815d7615, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd61d7617, SUC_lexicon_tags_15 },
    { 0x9edd7618, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7d761e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabd7627, SUC_lexicon_tags_17 },
    { 0x4c9d7628, SUC_lexicon_tags_15 },
    { 0xa57d7629, SUC_lexicon_tags_19 },
    { 0x2a1d762a, SUC_lexicon_tags_7 },
    { 0xe11d762b, SUC_lexicon_tags_2 },
    { 0xb9dd762c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29d7635, SUC_lexicon_tags_39 },
    { 0xeedd7636, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155d763a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303d7642, SUC_lexicon_tags_12 },
    { 0x253d7643, SUC_lexicon_tags_29 },
    { 0x461d7643, SUC_lexicon_tags_98 },
    { 0x771d7645, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x349d7648, SUC_lexicon_tags_25 },
    { 0x9afd7648, SUC_lexicon_tags_11 },
    { 0x46dd764a, SUC_lexicon_tags_61 },
    { 0x921d7649, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361d7650, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd8bd7652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdc7d7654, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931d7658, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x29d765b, SUC_lexicon_tags_29 },
    { 0xefbd765b, SUC_lexicon_tags_38 },
    { 0xdd9d765d, SUC_lexicon_tags_35 },
    { 0x3add765d, SUC_lexicon_tags_4 },
    { 0xecbd765d, SUC_lexicon_tags_33 },
    { 0x1cdd765e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2fbd7662, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bd766c, SUC_lexicon_tags_14 },
    { 0xf31d766d, SUC_lexicon_tags_17 },
    { 0x12fd766e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dd7676, SUC_lexicon_tags_0 },
    { 0xf0bd7676, SUC_lexicon_tags_27 },
    { 0xf55d7676, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05d767e, SUC_lexicon_tags_15 },
    { 0x321d767f, SUC_lexicon_tags_31 },
    { 0x4cfd7680, SUC_lexicon_tags_25 },
    { 0xd21d7681, SUC_lexicon_tags_56 },
    { 0x6a1d7682, SUC_lexicon_tags_2 },
    { 0x96dd7681, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d7686, SUC_lexicon_tags_17 },
    { 0xc6fd7686, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfd768b, SUC_lexicon_tags_60 },
    { 0x351d768b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x77dd768e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce3d7690, SUC_lexicon_tags_11 },
    { 0xe7fd7691, SUC_lexicon_tags_15 },
    { 0xf69d7692, SUC_lexicon_tags_91 },
    { 0x579d7693, SUC_lexicon_tags_29 },
    { 0x143d7694, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d7698, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61d769d, SUC_lexicon_tags_37 },
    { 0x837d769e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf63d76a0, SUC_lexicon_tags_43 },
    { 0xfbd76a1, SUC_lexicon_tags_68 },
    { 0x305d76a0, SUC_lexicon_tags_14 },
    { 0x5e7d76a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653d76a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x249d76ab, SUC_lexicon_tags_13 },
    { 0x5ddd76ac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1d76b2, SUC_lexicon_tags_0 },
    { 0xf65d76b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf37d76b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc7d76b7, SUC_lexicon_tags_49 },
    { 0x6bfd76b8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd76bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x56fd76c0, SUC_lexicon_tags_25 },
    { 0x671d76c1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abd76c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fd76c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cfd76ca, SUC_lexicon_tags_12 },
    { 0xacbd76cb, SUC_lexicon_tags_19 },
    { 0x7f1d76cc, SUC_lexicon_tags_19 },
    { 0x2dbd76cd, SUC_lexicon_tags_9 },
    { 0x25dd76cd, SUC_lexicon_tags_29 },
    { 0x14bd76cf, SUC_lexicon_tags_17 },
    { 0x369d76d0, SUC_lexicon_tags_187 },
    { 0xabfd76d0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87d76d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb91d76d7, SUC_lexicon_tags_2 },
    { 0x4fbd76d8, SUC_lexicon_tags_29 },
    { 0x23fd76d7, SUC_lexicon_tags_31 },
    { 0x417d76da, SUC_lexicon_tags_2 },
    { 0xd03d76d9, SUC_lexicon_tags_37 },
    { 0x505d76dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa1d76de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x18bd76e0, SUC_lexicon_tags_77 },
    { 0x113d76e0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x389d76e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bd76e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa17d76e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fd76eb, SUC_lexicon_tags_11 },
    { 0xafbd76eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9d76f2, SUC_lexicon_tags_12 },
    { 0xf2dd76f3, SUC_lexicon_tags_43 },
    { 0xb41d76f3, SUC_lexicon_tags_17 },
    { 0xe3bd76f5, SUC_lexicon_tags_37 },
    { 0x293d76f4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afd76ff, SUC_lexicon_tags_12 },
    { 0x9ffd7700, SUC_lexicon_tags_17 },
    { 0x25fd7700, SUC_lexicon_tags_29 },
    { 0xdd3d7702, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x107d7705, SUC_lexicon_tags_12 },
    { 0x9a9d7706, SUC_lexicon_tags_15 },
    { 0xefd7706, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879d770b, SUC_lexicon_tags_33 },
    { 0xacfd770b, SUC_lexicon_tags_60 },
    { 0x6f5d770b, SUC_lexicon_tags_37 },
    { 0xc29d770d, SUC_lexicon_tags_7 },
    { 0xeb3d770e, SUC_lexicon_tags_62 },
    { 0x25d7710, SUC_lexicon_tags_56 },
    { 0xae3d7711, SUC_lexicon_tags_23 },
    { 0x115d7712, SUC_lexicon_tags_25 },
    { 0xbe1d7710, SUC_lexicon_tags_35 },
    { 0xd65d770e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bd771e, SUC_lexicon_tags_63 },
    { 0xd7dd771f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dd7723, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efd7727, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21d7730, SUC_lexicon_tags_208 },
    { 0x7a5d7731, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdd7738, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dd7740, SUC_lexicon_tags_16 },
    { 0x211d7741, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd41d7743, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xaf3d7745, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d774c, SUC_lexicon_tags_60 },
    { 0xaadd774d, SUC_lexicon_tags_15 },
    { 0xbf7d774e, SUC_lexicon_tags_1 },
    { 0x61d774c, SUC_lexicon_tags_37 },
    { 0x36dd774c, SUC_lexicon_tags_12 },
    { 0xc51d774e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x387d7754, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56dd7756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdfbd7758, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fd7761, SUC_lexicon_tags_9 },
    { 0x747d7762, SUC_lexicon_tags_25 },
    { 0x131d7763, SUC_lexicon_tags_14 },
    { 0xe47d7764, SUC_lexicon_tags_16 },
    { 0x7cbd7763, SUC_lexicon_tags_2 },
    { 0xe39d7766, SUC_lexicon_tags_23 },
    { 0x115d7766, SUC_lexicon_tags_49 },
    { 0x9c1d7766, SUC_lexicon_tags_53 },
    { 0x573d7769, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddd776d, SUC_lexicon_tags_84 },
    { 0xa6fd776e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebd7779, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x499d777b, SUC_lexicon_tags_72 },
    { 0x9a5d777c, SUC_lexicon_tags_7 },
    { 0x597d777b, SUC_lexicon_tags_60 },
    { 0x74fd777e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8dd7780, SUC_lexicon_tags_7 },
    { 0xe3d7781, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd19d7783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525d7789, SUC_lexicon_tags_7 },
    { 0x23fd778a, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1d778e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x64dd7790, SUC_lexicon_tags_7 },
    { 0xcb3d7791, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73d7797, SUC_lexicon_tags_1 },
    { 0xcf5d7798, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdd779f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dd77a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd1d77a5, SUC_lexicon_tags_43 },
    { 0x70fd77a6, SUC_lexicon_tags_57 },
    { 0xf73d77a7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbd77ab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ebd77ad, SUC_lexicon_tags_15 },
    { 0xadfd77ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1d77b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1d77b6, SUC_lexicon_tags_63 },
    { 0xd4dd77b7, SUC_lexicon_tags_55 },
    { 0x51fd77b6, SUC_lexicon_tags_12 },
    { 0x645d77b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411d77be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefd77c6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39d77cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadd77d1, SUC_lexicon_tags_26 },
    { 0x831d77d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc7d77d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e5d77d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3d77da, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc37d77dc, SUC_lexicon_tags_29 },
    { 0x4f9d77dd, SUC_lexicon_tags_17 },
    { 0x493d77de, SUC_lexicon_tags_1 },
    { 0xc1bd77de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa0fd77e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7dd77e3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe91d77e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa9fd77e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bd77eb, SUC_lexicon_tags_17 },
    { 0xb41d77ec, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x745d77ee, SUC_lexicon_tags_15 },
    { 0x8fbd77ee, SUC_lexicon_tags_20 },
    { 0xdc3d77f0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493d77f4, SUC_lexicon_tags_6 },
    { 0x1f5d77f4, SUC_lexicon_tags_47 },
    { 0x7fdd77f6, SUC_lexicon_tags_7 },
    { 0x34dd77f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5cbd77f9, SUC_lexicon_tags_7 },
    { 0x693d77fa, SUC_lexicon_tags_23 },
    { 0x985d77fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1d7800, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fd7803, SUC_lexicon_tags_37 },
    { 0x9dbd7803, SUC_lexicon_tags_45 },
    { 0x2f5d7804, SUC_lexicon_tags_27 },
    { 0x59bd7805, SUC_lexicon_tags_11 },
    { 0xcc9d7807, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc11d7809, SUC_lexicon_tags_25 },
    { 0x831d780a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dd7810, SUC_lexicon_tags_7 },
    { 0x199d7811, SUC_lexicon_tags_11 },
    { 0xe71d7812, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x465d7814, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeedd7816, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdd781b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b9d781d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddd7825, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9a9d7827, SUC_lexicon_tags_47 },
    { 0xfe3d7828, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7d782c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc03d782e, SUC_lexicon_tags_14 },
    { 0x65fd782e, SUC_lexicon_tags_208 },
    { 0x2bbd7830, SUC_lexicon_tags_12 },
    { 0x46fd7831, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x405d7833, SUC_lexicon_tags_30 },
    { 0xcfd7834, SUC_lexicon_tags_12 },
    { 0x68dd7835, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bd7839, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3ebd783b, SUC_lexicon_tags_11 },
    { 0x4cdd783c, SUC_lexicon_tags_17 },
    { 0xc7d783c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dd7840, SUC_lexicon_tags_23 },
    { 0xc07d7840, SUC_lexicon_tags_25 },
    { 0x1ddd7841, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bd7844, SUC_lexicon_tags_11 },
    { 0x615d7845, SUC_lexicon_tags_25 },
    { 0x23bd7846, SUC_lexicon_tags_40 },
    { 0x635d7844, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfd784f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x571d7851, SUC_lexicon_tags_25 },
    { 0x8d5d7851, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415d7857, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa45d7859, SUC_lexicon_tags_11 },
    { 0x8f7d785a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb0dd785c, SUC_lexicon_tags_15 },
    { 0x493d785d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361d7863, SUC_lexicon_tags_25 },
    { 0xd0fd7864, SUC_lexicon_tags_17 },
    { 0x3d9d7863, SUC_lexicon_tags_43 },
    { 0xc07d7866, SUC_lexicon_tags_23 },
    { 0x755d7866, SUC_lexicon_tags_23 },
    { 0x6f7d7868, SUC_lexicon_tags_46 },
    { 0x90dd7869, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafd786d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283d7875, SUC_lexicon_tags_11 },
    { 0x7bd7875, SUC_lexicon_tags_7 },
    { 0x44dd7877, SUC_lexicon_tags_7 },
    { 0xeefd7878, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffd787c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x31fd787e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9d7886, SUC_lexicon_tags_53 },
    { 0x715d7886, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcb1d7889, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x321d788c, SUC_lexicon_tags_46 },
    { 0xe97d788d, SUC_lexicon_tags_16 },
    { 0xcc5d788e, SUC_lexicon_tags_9 },
    { 0x81dd788e, SUC_lexicon_tags_47 },
    { 0x831d7890, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xebbd7892, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xbedd7895, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95dd7898, SUC_lexicon_tags_37 },
    { 0x7b1d7898, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43d789d, SUC_lexicon_tags_0 },
    { 0x8c7d789e, SUC_lexicon_tags_45 },
    { 0x6f1d789f, SUC_lexicon_tags_45 },
    { 0x123d78a0, SUC_lexicon_tags_19 },
    { 0xc19d78a1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3d78a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7dd78a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x46fd78a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd23d78a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfd78ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1d78b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2efd78ba, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x131d78bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3d78c4, SUC_lexicon_tags_38 },
    { 0x247d78c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd2bd78c7, SUC_lexicon_tags_17 },
    { 0x18dd78c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21dd78cb, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913d78d1, SUC_lexicon_tags_34 },
    { 0xdf9d78d1, SUC_lexicon_tags_30 },
    { 0x3d3d78d3, SUC_lexicon_tags_19 },
    { 0x3b5d78d4, SUC_lexicon_tags_11 },
    { 0x4d3d78d3, SUC_lexicon_tags_3 },
    { 0x59d78d6, SUC_lexicon_tags_1 },
    { 0x71bd78d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13d78e1, SUC_lexicon_tags_17 },
    { 0xd91d78e2, SUC_lexicon_tags_37 },
    { 0xc37d78e3, SUC_lexicon_tags_60 },
    { 0xfd1d78e4, SUC_lexicon_tags_62 },
    { 0x5f1d78e5, SUC_lexicon_tags_33 },
    { 0x927d78e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1bdd78e8, SUC_lexicon_tags_14 },
    { 0xacfd78e8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7d78ec, SUC_lexicon_tags_17 },
    { 0x6dbd78ed, SUC_lexicon_tags_28 },
    { 0x51fd78ee, SUC_lexicon_tags_25 },
    { 0x731d78ef, SUC_lexicon_tags_91 },
    { 0xaadd78f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3d78f3, SUC_lexicon_tags_14 },
    { 0x295d78f3, SUC_lexicon_tags_26 },
    { 0x38dd78f5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5abd78f7, SUC_lexicon_tags_61 },
    { 0x7d78f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbd78fc, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x2dd78fe, SUC_lexicon_tags_15 },
    { 0x78fd78ff, SUC_lexicon_tags_14 },
    { 0x1f7d7900, SUC_lexicon_tags_25 },
    { 0x1d9d78ff, SUC_lexicon_tags_12 },
    { 0x99fd7902, SUC_lexicon_tags_2 },
    { 0xa93d7903, SUC_lexicon_tags_15 },
    { 0xc0fd7902, SUC_lexicon_tags_37 },
    { 0xd29d7900, SUC_lexicon_tags_8 },
    { 0x8f1d7905, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fd7909, SUC_lexicon_tags_12 },
    { 0x269d7909, SUC_lexicon_tags_87 },
    { 0x2edd790b, SUC_lexicon_tags_2 },
    { 0x639d790b, SUC_lexicon_tags_31 },
    { 0x9a7d790d, SUC_lexicon_tags_7 },
    { 0x943d790b, SUC_lexicon_tags_7 },
    { 0x6b3d790a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bd7914, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x52bd7917, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5d791a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3d791f, SUC_lexicon_tags_23 },
    { 0xfd7920, SUC_lexicon_tags_16 },
    { 0x141d791f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbd7928, SUC_lexicon_tags_12 },
    { 0x879d7928, SUC_lexicon_tags_37 },
    { 0x28bd792a, SUC_lexicon_tags_29 },
    { 0x371d7929, SUC_lexicon_tags_32 },
    { 0x51fd792b, SUC_lexicon_tags_15 },
    { 0xf97d792d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9edd792f, SUC_lexicon_tags_11 },
    { 0xb0fd792f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbd7933, SUC_lexicon_tags_49 },
    { 0x379d7934, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbd7937, SUC_lexicon_tags_37 },
    { 0xa97d7938, SUC_lexicon_tags_7 },
    { 0x23fd7939, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33d793c, SUC_lexicon_tags_84 },
    { 0x41fd793c, SUC_lexicon_tags_23 },
    { 0xa87d793e, SUC_lexicon_tags_19 },
    { 0xe03d793f, SUC_lexicon_tags_25 },
    { 0x2dfd793e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4fd7942, SUC_lexicon_tags_31 },
    { 0x503d7942, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd5d7945, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a7d7947, SUC_lexicon_tags_19 },
    { 0xbd9d7948, SUC_lexicon_tags_129 },
    { 0x619d7947, SUC_lexicon_tags_6 },
    { 0x95bd7948, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1d794f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bd7955, SUC_lexicon_tags_32 },
    { 0x627d7956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbd795b, SUC_lexicon_tags_7 },
    { 0x2b7d795b, SUC_lexicon_tags_25 },
    { 0x767d795c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfd7963, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9d7966, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf5dd7968, SUC_lexicon_tags_127 },
    { 0xeffd7969, SUC_lexicon_tags_62 },
    { 0xc7d796a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc15d796c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdd7971, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dd7974, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x813d7977, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x549d7979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x44bd797b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x267d797d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7980, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9d7985, SUC_lexicon_tags_2 },
    { 0x24fd7986, SUC_lexicon_tags_7 },
    { 0xef5d7987, SUC_lexicon_tags_17 },
    { 0x25bd7986, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x57fd798a, SUC_lexicon_tags_61 },
    { 0x743d798b, SUC_lexicon_tags_26 },
    { 0x2dfd798c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfd7995, SUC_lexicon_tags_31 },
    { 0x937d7996, SUC_lexicon_tags_8 },
    { 0x63d7996, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89fd7999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5d799d, SUC_lexicon_tags_37 },
    { 0xb67d799e, SUC_lexicon_tags_6 },
    { 0xc55d799e, SUC_lexicon_tags_23 },
    { 0x467d799f, SUC_lexicon_tags_38 },
    { 0x3c9d79a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69d79a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bd79ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37d79b3, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x589d79b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d79ba, SUC_lexicon_tags_63 },
    { 0x37fd79bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x401d79be, SUC_lexicon_tags_0 },
    { 0x881d79bf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9d79c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x95bd79c6, SUC_lexicon_tags_37 },
    { 0x6a5d79c7, SUC_lexicon_tags_43 },
    { 0x2bdd79c7, SUC_lexicon_tags_15 },
    { 0xbd1d79c6, SUC_lexicon_tags_19 },
    { 0x75bd79c7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9d79cd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9d79d0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xee9d79d2, SUC_lexicon_tags_37 },
    { 0x9ffd79d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d79d7, SUC_lexicon_tags_49 },
    { 0x47bd79d7, SUC_lexicon_tags_1 },
    { 0x717d79d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x985d79db, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01d79df, SUC_lexicon_tags_25 },
    { 0xffbd79e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7fd79e2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x489d79e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd8fd79e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525d79ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fd79ed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdd79f4, SUC_lexicon_tags_17 },
    { 0x7c9d79f5, SUC_lexicon_tags_17 },
    { 0x395d79f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x76dd79f9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341d7a00, SUC_lexicon_tags_0 },
    { 0x917d7a00, SUC_lexicon_tags_95 },
    { 0xc65d7a00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85d7a08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x977d7a0b, SUC_lexicon_tags_49 },
    { 0x54fd7a0b, SUC_lexicon_tags_26 },
    { 0xe7bd7a0d, SUC_lexicon_tags_1 },
    { 0x6bd7a0d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x75bd7a10, SUC_lexicon_tags_0 },
    { 0x3bd7a11, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7d7a16, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45d7a1a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d7a1d, SUC_lexicon_tags_45 },
    { 0xf37d7a1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x37bd7a20, SUC_lexicon_tags_33 },
    { 0x341d7a20, SUC_lexicon_tags_1 },
    { 0x639d7a20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187d7a26, SUC_lexicon_tags_29 },
    { 0xf21d7a27, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe59d7a29, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x857d7a2b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7d7a2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8cdd7a30, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd7a34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79d7a37, SUC_lexicon_tags_25 },
    { 0x7a5d7a38, SUC_lexicon_tags_14 },
    { 0x7ddd7a39, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbd7a3d, SUC_lexicon_tags_17 },
    { 0x37fd7a3e, SUC_lexicon_tags_11 },
    { 0x56bd7a3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x71dd7a41, SUC_lexicon_tags_31 },
    { 0x6b1d7a41, SUC_lexicon_tags_12 },
    { 0x707d7a42, SUC_lexicon_tags_14 },
    { 0x287d7a44, SUC_lexicon_tags_8 },
    { 0xcdfd7a44, SUC_lexicon_tags_49 },
    { 0x381d7a45, SUC_lexicon_tags_17 },
    { 0x57bd7a47, SUC_lexicon_tags_20 },
    { 0x683d7a48, SUC_lexicon_tags_30 },
    { 0x5a9d7a49, SUC_lexicon_tags_60 },
    { 0xe7dd7a4a, SUC_lexicon_tags_27 },
    { 0x363d7a4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf73d7a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0dd7a50, SUC_lexicon_tags_8 },
    { 0xacdd7a51, SUC_lexicon_tags_23 },
    { 0xd9bd7a51, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c5d7a54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419d7a59, SUC_lexicon_tags_33 },
    { 0x6bd7a5a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xda1d7a5c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abd7a5f, SUC_lexicon_tags_2 },
    { 0x64fd7a60, SUC_lexicon_tags_12 },
    { 0x193d7a61, SUC_lexicon_tags_38 },
    { 0xd9bd7a62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbf1d7a64, SUC_lexicon_tags_10 },
    { 0x37fd7a64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b7d7a67, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73d7a6d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1d7a70, SUC_lexicon_tags_14 },
    { 0xa6fd7a71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7d7a74, SUC_lexicon_tags_32 },
    { 0xe33d7a74, SUC_lexicon_tags_7 },
    { 0x89bd7a74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9d7a7a, SUC_lexicon_tags_41 },
    { 0x1ebd7a7a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x123d7a7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29d7a83, SUC_lexicon_tags_11 },
    { 0xab5d7a83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3d7a87, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7d7a8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x247d7a8d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b9d7a8f, SUC_lexicon_tags_38 },
    { 0xbe9d7a90, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4bbd7a92, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7d7a95, SUC_lexicon_tags_51 },
    { 0x70dd7a96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b5d7a98, SUC_lexicon_tags_57 },
    { 0x7d9d7a99, SUC_lexicon_tags_11 },
    { 0xa03d7a99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1e3d7a9c, SUC_lexicon_tags_47 },
    { 0xe01d7a9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x115d7aa0, SUC_lexicon_tags_0 },
    { 0x4d9d7aa1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d3d7aa3, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d7aa9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2b7d7aab, SUC_lexicon_tags_33 },
    { 0x25dd7aac, SUC_lexicon_tags_17 },
    { 0xc83d7aad, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791d7ab1, SUC_lexicon_tags_64 },
    { 0xa8dd7ab1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3cbd7ab4, SUC_lexicon_tags_37 },
    { 0x30fd7ab5, SUC_lexicon_tags_2 },
    { 0x283d7ab5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edd7abb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x831d7abd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x681d7abf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x861d7ac1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61bd7ac3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe1d7ac5, SUC_lexicon_tags_37 },
    { 0x3c5d7ac5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe9bd7ac8, SUC_lexicon_tags_12 },
    { 0x845d7ac9, SUC_lexicon_tags_34 },
    { 0xecfd7aca, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x4c7d7acc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437d7ad1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3bd7ad3, SUC_lexicon_tags_17 },
    { 0xb57d7ad4, SUC_lexicon_tags_37 },
    { 0xd1fd7ad4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdd7ada, SUC_lexicon_tags_17 },
    { 0xccbd7adb, SUC_lexicon_tags_43 },
    { 0x847d7adc, SUC_lexicon_tags_7 },
    { 0x1f9d7adb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebd7ae2, SUC_lexicon_tags_477 },
    { 0xd7fd7ae3, SUC_lexicon_tags_1 },
    { 0xd77d7ae4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dd7aea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bd7aee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4f5d7af0, SUC_lexicon_tags_17 },
    { 0xec1d7af1, SUC_lexicon_tags_17 },
    { 0x417d7af2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6add7af5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd7af8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdd7afb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc99d7afd, SUC_lexicon_tags_18 },
    { 0xbc9d7afd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1d7b02, SUC_lexicon_tags_27 },
    { 0xd03d7b03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x789d7b05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fd7b15, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdd7b18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101d7b1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xad7d7b1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dd7b27, SUC_lexicon_tags_37 },
    { 0x6e7d7b28, SUC_lexicon_tags_2 },
    { 0x3f7d7b27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddd7b2d, SUC_lexicon_tags_69 },
    { 0x1dd7b2e, SUC_lexicon_tags_32 },
    { 0x2f5d7b2f, SUC_lexicon_tags_63 },
    { 0x47bd7b2e, SUC_lexicon_tags_61 },
    { 0xa7fd7b31, SUC_lexicon_tags_31 },
    { 0x60dd7b2d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbafd7b34, SUC_lexicon_tags_4 },
    { 0xf1d7b35, SUC_lexicon_tags_37 },
    { 0x36dd7b36, SUC_lexicon_tags_12 },
    { 0x93bd7b37, SUC_lexicon_tags_37 },
    { 0x381d7b38, SUC_lexicon_tags_25 },
    { 0x237d7b39, SUC_lexicon_tags_25 },
    { 0xb45d7b37, SUC_lexicon_tags_57 },
    { 0x777d7b35, SUC_lexicon_tags_25 },
    { 0x2d1d7b3c, SUC_lexicon_tags_1 },
    { 0xd23d7b39, SUC_lexicon_tags_26 },
    { 0x4bdd7b3e, SUC_lexicon_tags_17 },
    { 0x48bd7b3f, SUC_lexicon_tags_7 },
    { 0x997d7b38, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fd7b46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bd7b4a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdd7b4e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7d7b52, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259d7b60, SUC_lexicon_tags_11 },
    { 0x69dd7b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91d7b64, SUC_lexicon_tags_12 },
    { 0x221d7b65, SUC_lexicon_tags_65 },
    { 0x6fbd7b65, SUC_lexicon_tags_43 },
    { 0xd5fd7b67, SUC_lexicon_tags_2 },
    { 0xa9d7b68, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5d7b6c, SUC_lexicon_tags_17 },
    { 0x97d7b6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d9d7b6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9d7b73, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4cbd7b75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fd7b78, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41d7b7c, SUC_lexicon_tags_33 },
    { 0x2c5d7b7d, SUC_lexicon_tags_15 },
    { 0x60bd7b7d, SUC_lexicon_tags_19 },
    { 0xad9d7b7f, SUC_lexicon_tags_32 },
    { 0xc4fd7b80, SUC_lexicon_tags_17 },
    { 0x94dd7b81, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bd7b86, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bd7b8e, SUC_lexicon_tags_15 },
    { 0xcbfd7b8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x569d7b91, SUC_lexicon_tags_16 },
    { 0xd0bd7b92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d7b96, SUC_lexicon_tags_17 },
    { 0x181d7b96, SUC_lexicon_tags_0 },
    { 0xc17d7b98, SUC_lexicon_tags_14 },
    { 0xdabd7b99, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811d7b9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x177d7ba0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbd7ba4, SUC_lexicon_tags_4 },
    { 0xdf7d7ba5, SUC_lexicon_tags_2 },
    { 0xb1d7ba6, SUC_lexicon_tags_65 },
    { 0xe5fd7ba7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54bd7ba9, SUC_lexicon_tags_39 },
    { 0x9c9d7ba9, SUC_lexicon_tags_37 },
    { 0x3a3d7bab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35d7bae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1d7bb3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afd7bb9, SUC_lexicon_tags_23 },
    { 0xedfd7bba, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d7bbd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329d7bc3, SUC_lexicon_tags_27 },
    { 0xb7bd7bc3, SUC_lexicon_tags_7 },
    { 0x8add7bc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9d7bc8, SUC_lexicon_tags_27 },
    { 0x50dd7bc9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3d7bcc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbd7bcf, SUC_lexicon_tags_1 },
    { 0xae1d7bd0, SUC_lexicon_tags_8 },
    { 0x3cfd7bd1, SUC_lexicon_tags_0 },
    { 0xf1bd7bd2, SUC_lexicon_tags_43 },
    { 0xdc3d7bd2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7afd7bd6, SUC_lexicon_tags_26 },
    { 0x665d7bd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edd7be0, SUC_lexicon_tags_15 },
    { 0x15d7be1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabd7be5, SUC_lexicon_tags_14 },
    { 0x4f9d7be6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1d7bf1, SUC_lexicon_tags_25 },
    { 0xc73d7bf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843d7bf6, SUC_lexicon_tags_6 },
    { 0x903d7bf7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x147d7bf9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdcfd7bfb, SUC_lexicon_tags_37 },
    { 0x541d7bfb, SUC_lexicon_tags_87 },
    { 0x96bd7bfd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa7fd7bff, SUC_lexicon_tags_11 },
    { 0xc93d7c00, SUC_lexicon_tags_95 },
    { 0xa23d7c01, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xcb5d7c03, SUC_lexicon_tags_17 },
    { 0x985d7c03, SUC_lexicon_tags_49 },
    { 0xb29d7c05, SUC_lexicon_tags_11 },
    { 0x4cfd7c06, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5d7c09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15d7c0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x87d7c0f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5d7c15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d7d7c17, SUC_lexicon_tags_47 },
    { 0x151d7c18, SUC_lexicon_tags_47 },
    { 0xd95d7c19, SUC_lexicon_tags_11 },
    { 0x901d7c17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185d7c1f, SUC_lexicon_tags_0 },
    { 0xc23d7c20, SUC_lexicon_tags_158 },
    { 0xd23d7c1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb1d7c23, SUC_lexicon_tags_17 },
    { 0x3d7d7c24, SUC_lexicon_tags_13 },
    { 0xac3d7c24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x277d7c28, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4d1d7c2a, SUC_lexicon_tags_47 },
    { 0x9dbd7c2b, SUC_lexicon_tags_64 },
    { 0x33dd7c2c, SUC_lexicon_tags_1 },
    { 0x285d7c2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb15d7c2f, SUC_lexicon_tags_17 },
    { 0xeebd7c2f, SUC_lexicon_tags_17 },
    { 0x175d7c31, SUC_lexicon_tags_2 },
    { 0x8ebd7c32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x651d7c35, SUC_lexicon_tags_6 },
    { 0x6bbd7c36, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb4dd7c38, SUC_lexicon_tags_14 },
    { 0x65dd7c39, SUC_lexicon_tags_26 },
    { 0xe4bd7c39, SUC_lexicon_tags_154 },
    { 0x793d7c38, SUC_lexicon_tags_28 },
    { 0x1e9d7c3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abd7c43, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x275d7c46, SUC_lexicon_tags_17 },
    { 0x409d7c47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc1bd7c49, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dd7c4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5d7c52, SUC_lexicon_tags_26 },
    { 0x8bbd7c53, SUC_lexicon_tags_1 },
    { 0xa57d7c53, SUC_lexicon_tags_62 },
    { 0x6e7d7c52, SUC_lexicon_tags_31 },
    { 0x70fd7c56, SUC_lexicon_tags_11 },
    { 0x783d7c56, SUC_lexicon_tags_15 },
    { 0xc47d7c57, SUC_lexicon_tags_0 },
    { 0xa23d7c58, SUC_lexicon_tags_17 },
    { 0x23bd7c52, SUC_lexicon_tags_23 },
    { 0x951d7c5b, SUC_lexicon_tags_32 },
    { 0x221d7c5c, SUC_lexicon_tags_45 },
    { 0xfbfd7c5d, SUC_lexicon_tags_2 },
    { 0xd3fd7c5e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x415d7c60, SUC_lexicon_tags_14 },
    { 0xd1bd7c61, SUC_lexicon_tags_2 },
    { 0xf4bd7c61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809d7c66, SUC_lexicon_tags_9 },
    { 0x2b5d7c67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x745d7c6a, SUC_lexicon_tags_2 },
    { 0x2e1d7c6a, SUC_lexicon_tags_17 },
    { 0x7cbd7c6b, SUC_lexicon_tags_1 },
    { 0x663d7c6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3d7c70, SUC_lexicon_tags_33 },
    { 0xea7d7c71, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x631d7c73, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7d7c77, SUC_lexicon_tags_0 },
    { 0x6cbd7c78, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7b5d7c7a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fd7c7d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x283d7c7f, SUC_lexicon_tags_35 },
    { 0xd6fd7c80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9d7c83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9d7c8c, SUC_lexicon_tags_49 },
    { 0x2b1d7c8c, SUC_lexicon_tags_11 },
    { 0xc6dd7c8e, SUC_lexicon_tags_25 },
    { 0xdd1d7c8f, SUC_lexicon_tags_2 },
    { 0x2c1d7c90, SUC_lexicon_tags_87 },
    { 0x8bbd7c8e, SUC_lexicon_tags_2 },
    { 0xdf1d7c92, SUC_lexicon_tags_25 },
    { 0xd6dd7c93, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3d7c9a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfbfd7c9c, SUC_lexicon_tags_26 },
    { 0x5b3d7c9c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb1fd7c9f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x757d7ca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x397d7ca3, SUC_lexicon_tags_6 },
    { 0x385d7ca4, SUC_lexicon_tags_26 },
    { 0xe75d7ca4, SUC_lexicon_tags_7 },
    { 0x5f1d7ca4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x495d7ca9, SUC_lexicon_tags_31 },
    { 0xd89d7ca9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25d7caf, SUC_lexicon_tags_37 },
    { 0x913d7cb0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x169d7cb3, SUC_lexicon_tags_19 },
    { 0x44dd7cb3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x99fd7cb6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x267d7cb8, SUC_lexicon_tags_15 },
    { 0x79bd7cb8, SUC_lexicon_tags_38 },
    { 0xba9d7cba, SUC_lexicon_tags_2 },
    { 0xefbd7cbb, SUC_lexicon_tags_37 },
    { 0x1a3d7cba, SUC_lexicon_tags_44 },
    { 0x21fd7cbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x48dd7cc0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39dd7cc5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edd7cc8, SUC_lexicon_tags_37 },
    { 0x651d7cc8, SUC_lexicon_tags_11 },
    { 0xa29d7cca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1d7ccf, SUC_lexicon_tags_33 },
    { 0xf7bd7ccf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf79d7cd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbd7cd5, SUC_lexicon_tags_12 },
    { 0xa85d7cd6, SUC_lexicon_tags_37 },
    { 0x911d7cd7, SUC_lexicon_tags_0 },
    { 0xf7fd7cd8, SUC_lexicon_tags_93 },
    { 0xa05d7cd9, SUC_lexicon_tags_10 },
    { 0x9fdd7cd5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5add7cde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc83d7ce0, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dd7ce5, SUC_lexicon_tags_14 },
    { 0x6ffd7ce5, SUC_lexicon_tags_31 },
    { 0x121d7ce7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d7cf2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9f9d7cf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb51d7cf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x81bd7cf8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xed3d7cfa, SUC_lexicon_tags_12 },
    { 0x99fd7cfb, SUC_lexicon_tags_129 },
    { 0xc91d7cfc, SUC_lexicon_tags_16 },
    { 0x255d7cfb, SUC_lexicon_tags_37 },
    { 0x5add7cfe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501d7d04, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41d7d09, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dd7d0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5d7d16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebd7d1a, SUC_lexicon_tags_43 },
    { 0x2fd7d1a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5d7d21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77fd7d23, SUC_lexicon_tags_2 },
    { 0x1b7d7d23, SUC_lexicon_tags_2 },
    { 0x667d7d25, SUC_lexicon_tags_47 },
    { 0xda1d7d26, SUC_lexicon_tags_17 },
    { 0x6dfd7d27, SUC_lexicon_tags_11 },
    { 0xc4fd7d28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b3d7d2a, SUC_lexicon_tags_79 },
    { 0xc41d7d2b, SUC_lexicon_tags_40 },
    { 0xa71d7d2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717d7d30, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75d7d37, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6cbd7d39, SUC_lexicon_tags_65 },
    { 0x6e9d7d3a, SUC_lexicon_tags_17 },
    { 0x5d3d7d3b, SUC_lexicon_tags_49 },
    { 0xfbdd7d3c, SUC_lexicon_tags_15 },
    { 0xb33d7d3d, SUC_lexicon_tags_7 },
    { 0x3cbd7d3d, SUC_lexicon_tags_68 },
    { 0xb81d7d3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fd7d47, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x189d7d49, SUC_lexicon_tags_15 },
    { 0x887d7d49, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f7d7d4c, SUC_lexicon_tags_26 },
    { 0xae5d7d4c, SUC_lexicon_tags_45 },
    { 0xf5fd7d4e, SUC_lexicon_tags_37 },
    { 0xa4dd7d4f, SUC_lexicon_tags_14 },
    { 0xcedd7d4d, SUC_lexicon_tags_45 },
    { 0x721d7d51, SUC_lexicon_tags_38 },
    { 0x509d7d4c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fd7d56, SUC_lexicon_tags_15 },
    { 0x81fd7d56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fd7d5c, SUC_lexicon_tags_1 },
    { 0xd59d7d5d, SUC_lexicon_tags_25 },
    { 0xb3d7d5e, SUC_lexicon_tags_13 },
    { 0xb0bd7d5f, SUC_lexicon_tags_61 },
    { 0x17bd7d60, SUC_lexicon_tags_15 },
    { 0xbdfd7d61, SUC_lexicon_tags_0 },
    { 0xd41d7d61, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847d7d66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x181d7d68, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541d7d6c, SUC_lexicon_tags_63 },
    { 0xd51d7d6c, SUC_lexicon_tags_16 },
    { 0xecbd7d6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9d7d73, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x299d7d76, SUC_lexicon_tags_70 },
    { 0x451d7d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x809d7d79, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdd7d7c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x29bd7d7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa21d7d80, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xffdd7d82, SUC_lexicon_tags_17 },
    { 0x8bd7d82, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xed3d7d85, SUC_lexicon_tags_21 },
    { 0x425d7d85, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371d7d8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbcdd7d8d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1d7d90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743d7d99, SUC_lexicon_tags_38 },
    { 0x9b5d7d99, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773d7d9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dd7da2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x81dd7da4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x277d7da6, SUC_lexicon_tags_25 },
    { 0xbefd7da7, SUC_lexicon_tags_12 },
    { 0xc79d7da7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3efd7daa, SUC_lexicon_tags_17 },
    { 0xf35d7daa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x53fd7dad, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd7db1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffd7db4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53d7db7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53d7db9, SUC_lexicon_tags_27 },
    { 0x56dd7dba, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7d7dbd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x307d7dc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1d7dc3, SUC_lexicon_tags_25 },
    { 0x67fd7dc4, SUC_lexicon_tags_12 },
    { 0xe6fd7dc5, SUC_lexicon_tags_66 },
    { 0xf5dd7dc6, SUC_lexicon_tags_2 },
    { 0xbe9d7dc6, SUC_lexicon_tags_31 },
    { 0xc7d7dc5, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01d7dcb, SUC_lexicon_tags_17 },
    { 0xeb9d7dcb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbd7dd3, SUC_lexicon_tags_2 },
    { 0x5c5d7dd4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdd7ddc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48fd7dde, SUC_lexicon_tags_2 },
    { 0x7ddd7ddf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x12dd7de2, SUC_lexicon_tags_7 },
    { 0xb6dd7de3, SUC_lexicon_tags_17 },
    { 0xcafd7de3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31d7de9, SUC_lexicon_tags_27 },
    { 0x301d7dea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb0dd7dec, SUC_lexicon_tags_35 },
    { 0x871d7ded, SUC_lexicon_tags_37 },
    { 0x857d7dec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x59fd7df0, SUC_lexicon_tags_1 },
    { 0xaefd7df0, SUC_lexicon_tags_7 },
    { 0xc25d7df2, SUC_lexicon_tags_26 },
    { 0xc2bd7df0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afd7dfa, SUC_lexicon_tags_12 },
    { 0x7f3d7dfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bd7dff, SUC_lexicon_tags_11 },
    { 0x995d7dff, SUC_lexicon_tags_60 },
    { 0x5ddd7dff, SUC_lexicon_tags_27 },
    { 0x1abd7e01, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3d7e09, SUC_lexicon_tags_15 },
    { 0x65bd7e0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb99d7e0c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dd7e13, SUC_lexicon_tags_45 },
    { 0xc31d7e14, SUC_lexicon_tags_30 },
    { 0x379d7e15, SUC_lexicon_tags_32 },
    { 0x14fd7e16, SUC_lexicon_tags_21 },
    { 0x637d7e16, SUC_lexicon_tags_27 },
    { 0xcbfd7e16, SUC_lexicon_tags_17 },
    { 0xdffd7e19, SUC_lexicon_tags_35 },
    { 0x5cbd7e17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441d7e20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbd7e25, SUC_lexicon_tags_1 },
    { 0x425d7e26, SUC_lexicon_tags_39 },
    { 0x1abd7e27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7d7e2a, SUC_lexicon_tags_107 },
    { 0xff5d7e2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x879d7e2d, SUC_lexicon_tags_9 },
    { 0xf8fd7e2e, SUC_lexicon_tags_21 },
    { 0xb8dd7e2f, SUC_lexicon_tags_33 },
    { 0x82fd7e30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9d7e35, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05d7e38, SUC_lexicon_tags_47 },
    { 0xd4dd7e39, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d7e3e, SUC_lexicon_tags_7 },
    { 0x905d7e3f, SUC_lexicon_tags_1 },
    { 0x1a5d7e40, SUC_lexicon_tags_8 },
    { 0x391d7e3e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95d7e46, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7d7e4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9d7e50, SUC_lexicon_tags_23 },
    { 0x2efd7e50, SUC_lexicon_tags_65 },
    { 0xfb9d7e52, SUC_lexicon_tags_17 },
    { 0x621d7e53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x789d7e55, SUC_lexicon_tags_7 },
    { 0xcebd7e56, SUC_lexicon_tags_17 },
    { 0x34fd7e57, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x85d7e59, SUC_lexicon_tags_45 },
    { 0x91d7e5a, SUC_lexicon_tags_7 },
    { 0x481d7e5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9d7e5f, SUC_lexicon_tags_2 },
    { 0x81d7e60, SUC_lexicon_tags_2 },
    { 0x4cdd7e60, SUC_lexicon_tags_12 },
    { 0xf1bd7e62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77d7e74, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x92bd7e76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc0dd7e78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675d7e81, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9d7e85, SUC_lexicon_tags_7 },
    { 0x36bd7e86, SUC_lexicon_tags_32 },
    { 0x69bd7e85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x477d7e89, SUC_lexicon_tags_63 },
    { 0x57dd7e89, SUC_lexicon_tags_47 },
    { 0x51dd7e89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9d7e8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x787d7e91, SUC_lexicon_tags_33 },
    { 0x943d7e92, SUC_lexicon_tags_1 },
    { 0xc59d7e93, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5afd7e96, SUC_lexicon_tags_15 },
    { 0xa1d7e97, SUC_lexicon_tags_49 },
    { 0x59dd7e98, SUC_lexicon_tags_63 },
    { 0x8afd7e98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x20dd7e9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb11d7e9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x409d7ea1, SUC_lexicon_tags_43 },
    { 0x3a3d7ea2, SUC_lexicon_tags_13 },
    { 0xad1d7ea3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59d7ea9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7f1d7eab, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bd7eae, SUC_lexicon_tags_11 },
    { 0x299d7eae, SUC_lexicon_tags_11 },
    { 0x7fbd7eaf, SUC_lexicon_tags_33 },
    { 0x285d7eb1, SUC_lexicon_tags_15 },
    { 0x133d7eb2, SUC_lexicon_tags_100 },
    { 0x81d7eb1, SUC_lexicon_tags_39 },
    { 0xfd7eb4, SUC_lexicon_tags_28 },
    { 0x39dd7eb5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97d7eb9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07d7ebf, SUC_lexicon_tags_70 },
    { 0x535d7ec0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdfdd7ec2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x24bd7ec4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x109d7ec6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x36fd7ec9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x61dd7ecb, SUC_lexicon_tags_46 },
    { 0x46bd7ecc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07d7ed5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x363d7ed8, SUC_lexicon_tags_29 },
    { 0x1fdd7ed9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877d7ede, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x45dd7ee1, SUC_lexicon_tags_11 },
    { 0x2f9d7ee2, SUC_lexicon_tags_32 },
    { 0x761d7ee2, SUC_lexicon_tags_11 },
    { 0xc75d7ee4, SUC_lexicon_tags_15 },
    { 0x17d7ee4, SUC_lexicon_tags_1 },
    { 0x395d7ee6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfebd7ee8, SUC_lexicon_tags_13 },
    { 0x5bbd7ee9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd7eec, SUC_lexicon_tags_4 },
    { 0x6a5d7eec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fd7ef0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51bd7ef2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x2cbd7ef4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dd7ef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfd7efc, SUC_lexicon_tags_4 },
    { 0x7afd7efd, SUC_lexicon_tags_14 },
    { 0x99bd7efc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x149d7f01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999d7f07, SUC_lexicon_tags_81 },
    { 0x997d7f07, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21d7f10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fd7f16, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41d7f19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb7d7f1b, SUC_lexicon_tags_37 },
    { 0xf21d7f1c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bd7f20, SUC_lexicon_tags_17 },
    { 0x4dd7f21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9d7f27, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3d7f2a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1dfd7f2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92fd7f2e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9d7f32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29dd7f34, SUC_lexicon_tags_19 },
    { 0x9e3d7f35, SUC_lexicon_tags_17 },
    { 0x421d7f36, SUC_lexicon_tags_7 },
    { 0x685d7f36, SUC_lexicon_tags_19 },
    { 0xcbbd7f36, SUC_lexicon_tags_37 },
    { 0x115d7f39, SUC_lexicon_tags_51 },
    { 0xca3d7f3a, SUC_lexicon_tags_30 },
    { 0x5d3d7f3b, SUC_lexicon_tags_23 },
    { 0xbabd7f3c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7d7f42, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x12fd7f44, SUC_lexicon_tags_4 },
    { 0xe1d7f45, SUC_lexicon_tags_47 },
    { 0xa6bd7f46, SUC_lexicon_tags_2 },
    { 0x385d7f47, SUC_lexicon_tags_12 },
    { 0xad9d7f47, SUC_lexicon_tags_7 },
    { 0xb13d7f48, SUC_lexicon_tags_3 },
    { 0x3ffd7f4a, SUC_lexicon_tags_83 },
    { 0x40fd7f4b, SUC_lexicon_tags_37 },
    { 0xd7d7f49, SUC_lexicon_tags_63 },
    { 0xafd7f4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd7f51, SUC_lexicon_tags_17 },
    { 0xc9dd7f52, SUC_lexicon_tags_2 },
    { 0xb11d7f53, SUC_lexicon_tags_13 },
    { 0x55bd7f54, SUC_lexicon_tags_12 },
    { 0x229d7f52, SUC_lexicon_tags_136 },
    { 0x811d7f56, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9d7f59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb1d7f5b, SUC_lexicon_tags_60 },
    { 0x5add7f5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825d7f60, SUC_lexicon_tags_37 },
    { 0x407d7f60, SUC_lexicon_tags_87 },
    { 0x2edd7f62, SUC_lexicon_tags_15 },
    { 0x9a1d7f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x23dd7f66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfd7f74, SUC_lexicon_tags_37 },
    { 0xee7d7f75, SUC_lexicon_tags_17 },
    { 0xb43d7f75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1d7f7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dd7f7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x379d7f81, SUC_lexicon_tags_82 },
    { 0xc13d7f81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10dd7f89, SUC_lexicon_tags_14 },
    { 0x2a7d7f89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc67d7f8c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x30bd7f8f, SUC_lexicon_tags_63 },
    { 0x127d7f8f, SUC_lexicon_tags_82 },
    { 0x267d7f8f, SUC_lexicon_tags_2 },
    { 0x6fdd7f8f, SUC_lexicon_tags_10 },
    { 0xb79d7f90, SUC_lexicon_tags_25 },
    { 0x15dd7f8f, SUC_lexicon_tags_25 },
    { 0xccfd7f95, SUC_lexicon_tags_63 },
    { 0x81d7f96, SUC_lexicon_tags_35 },
    { 0x46dd7f96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffd7f9d, SUC_lexicon_tags_0 },
    { 0xffdd7f9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25d7fa1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fd7fa6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33d7fad, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x96dd7fb0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765d7fb7, SUC_lexicon_tags_10 },
    { 0x7b5d7fb7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857d7fbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bd7fc0, SUC_lexicon_tags_46 },
    { 0x657d7fc1, SUC_lexicon_tags_27 },
    { 0xb75d7fc1, SUC_lexicon_tags_33 },
    { 0xd83d7fc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdd7fc7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9d7fcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61d7fd0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375d7fd6, SUC_lexicon_tags_37 },
    { 0xfbbd7fd7, SUC_lexicon_tags_75 },
    { 0xfb5d7fd7, SUC_lexicon_tags_7 },
    { 0x65d7fd9, SUC_lexicon_tags_0 },
    { 0x777d7fd8, SUC_lexicon_tags_37 },
    { 0x55d7fd7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9d7fe0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1e7d7fe2, SUC_lexicon_tags_33 },
    { 0x61bd7fe3, SUC_lexicon_tags_17 },
    { 0xe7dd7fe3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf77d7fe6, SUC_lexicon_tags_37 },
    { 0xc0bd7fe7, SUC_lexicon_tags_38 },
    { 0x6ebd7fe8, SUC_lexicon_tags_2 },
    { 0xa1d7fe6, SUC_lexicon_tags_26 },
    { 0x4abd7fea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x821d7fed, SUC_lexicon_tags_15 },
    { 0xc19d7fee, SUC_lexicon_tags_14 },
    { 0x325d7fef, SUC_lexicon_tags_25 },
    { 0x8b9d7ff0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37d7ff4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfd7ff7, SUC_lexicon_tags_79 },
    { 0x491d7ff8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x143d7ffa, SUC_lexicon_tags_17 },
    { 0xeedd7ffa, SUC_lexicon_tags_2 },
    { 0xdafd7ffc, SUC_lexicon_tags_768 },
    { 0x8b9d7ffd, SUC_lexicon_tags_7 },
    { 0xc3dd7ffe, SUC_lexicon_tags_23 },
    { 0x683d7ffe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933d8007, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fd800d, SUC_lexicon_tags_11 },
    { 0x221d800e, SUC_lexicon_tags_11 },
    { 0x723d800f, SUC_lexicon_tags_7 },
    { 0xeadd8010, SUC_lexicon_tags_8 },
    { 0x399d8011, SUC_lexicon_tags_8 },
    { 0x931d8012, SUC_lexicon_tags_7 },
    { 0xa21d800e, SUC_lexicon_tags_77 },
    { 0xaefd8014, SUC_lexicon_tags_7 },
    { 0x905d8014, SUC_lexicon_tags_7 },
    { 0x32bd8016, SUC_lexicon_tags_17 },
    { 0xb3fd8013, SUC_lexicon_tags_5 },
    { 0xba7d8011, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109d801d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61d8022, SUC_lexicon_tags_15 },
    { 0xcf9d8022, SUC_lexicon_tags_34 },
    { 0x303d8022, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x62fd8026, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d8029, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f1d802b, SUC_lexicon_tags_10 },
    { 0xd11d802b, SUC_lexicon_tags_0 },
    { 0xe99d802d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x271d8030, SUC_lexicon_tags_37 },
    { 0x61fd8031, SUC_lexicon_tags_15 },
    { 0xba7d8032, SUC_lexicon_tags_92 },
    { 0x74bd8031, SUC_lexicon_tags_7 },
    { 0x36fd8030, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xe0fd8036, SUC_lexicon_tags_11 },
    { 0xc0fd8036, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423d8041, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdd804b, SUC_lexicon_tags_80 },
    { 0x699d804c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7dd804e, SUC_lexicon_tags_37 },
    { 0xbabd804f, SUC_lexicon_tags_12 },
    { 0xc35d804e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb3dd8052, SUC_lexicon_tags_37 },
    { 0x237d8053, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x145d8056, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1d8059, SUC_lexicon_tags_2 },
    { 0x7b5d805a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x91d805c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279d8060, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x93bd8062, SUC_lexicon_tags_32 },
    { 0x567d8062, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x28dd8066, SUC_lexicon_tags_2 },
    { 0x269d8066, SUC_lexicon_tags_28 },
    { 0xeb9d8067, SUC_lexicon_tags_49 },
    { 0x519d8069, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d8072, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fd807b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bd807f, SUC_lexicon_tags_8 },
    { 0xcddd8080, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x117d8083, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa83d8085, SUC_lexicon_tags_12 },
    { 0xb59d8086, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5fdd8088, SUC_lexicon_tags_63 },
    { 0xf9dd8089, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdd808c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea9d808e, SUC_lexicon_tags_17 },
    { 0xcddd808f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x711d8091, SUC_lexicon_tags_2 },
    { 0x7ddd8092, SUC_lexicon_tags_2 },
    { 0xadfd8092, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x265d8095, SUC_lexicon_tags_11 },
    { 0x713d8096, SUC_lexicon_tags_37 },
    { 0x9e1d8097, SUC_lexicon_tags_16 },
    { 0x6a1d8096, SUC_lexicon_tags_11 },
    { 0x7abd8096, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4d3d809b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7d809f, SUC_lexicon_tags_68 },
    { 0xb81d80a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9d80a3, SUC_lexicon_tags_25 },
    { 0x1d7d80a4, SUC_lexicon_tags_17 },
    { 0x1dbd80a5, SUC_lexicon_tags_37 },
    { 0x7b7d80a6, SUC_lexicon_tags_12 },
    { 0xc51d80a7, SUC_lexicon_tags_1 },
    { 0xc7d80a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3fd80aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x319d80ad, SUC_lexicon_tags_179 },
    { 0x81d80ae, SUC_lexicon_tags_63 },
    { 0x707d80af, SUC_lexicon_tags_12 },
    { 0xc23d80b0, SUC_lexicon_tags_60 },
    { 0xc87d80b1, SUC_lexicon_tags_17 },
    { 0x739d80b2, SUC_lexicon_tags_8 },
    { 0x293d80b0, SUC_lexicon_tags_25 },
    { 0xbd5d80b2, SUC_lexicon_tags_11 },
    { 0x897d80b5, SUC_lexicon_tags_17 },
    { 0x633d80b6, SUC_lexicon_tags_17 },
    { 0x45d80b7, SUC_lexicon_tags_25 },
    { 0xa0fd80b8, SUC_lexicon_tags_1 },
    { 0xb3dd80b4, SUC_lexicon_tags_1 },
    { 0x2c1d80b8, SUC_lexicon_tags_158 },
    { 0xb21d80b3, SUC_lexicon_tags_12 },
    { 0x395d80bc, SUC_lexicon_tags_38 },
    { 0x9a1d80ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x817d80bf, SUC_lexicon_tags_28 },
    { 0x4f9d80bf, SUC_lexicon_tags_64 },
    { 0x64bd80c1, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x661d80c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1d80c7, SUC_lexicon_tags_48 },
    { 0x6cdd80c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3d80d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3d80d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9ebd80d7, SUC_lexicon_tags_43 },
    { 0x781d80d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5d80db, SUC_lexicon_tags_18 },
    { 0x60dd80db, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0x21fd80de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb69d80e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x929d80e2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x28bd80e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09d80e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5d80e9, SUC_lexicon_tags_65 },
    { 0x96fd80ea, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x739d80ec, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dd80f1, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9d80fa, SUC_lexicon_tags_87 },
    { 0x2f9d80fa, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5d80ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbd8103, SUC_lexicon_tags_10 },
    { 0x1e7d8103, SUC_lexicon_tags_30 },
    { 0x205d8105, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9d8108, SUC_lexicon_tags_25 },
    { 0x335d8109, SUC_lexicon_tags_37 },
    { 0x9cdd810a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd59d810c, SUC_lexicon_tags_82 },
    { 0xe37d810d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacbd810f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d8114, SUC_lexicon_tags_19 },
    { 0x27fd8115, SUC_lexicon_tags_8 },
    { 0xc33d8115, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bd8119, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1d5d811b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fd8122, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7d8125, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5d8128, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf2fd812a, SUC_lexicon_tags_11 },
    { 0x457d812a, SUC_lexicon_tags_17 },
    { 0x6efd812c, SUC_lexicon_tags_42 },
    { 0x9dfd812c, SUC_lexicon_tags_23 },
    { 0x515d812e, SUC_lexicon_tags_63 },
    { 0x69bd812f, SUC_lexicon_tags_11 },
    { 0x617d812c, SUC_lexicon_tags_7 },
    { 0xfb9d8131, SUC_lexicon_tags_0 },
    { 0xecdd8132, SUC_lexicon_tags_12 },
    { 0x6fd8133, SUC_lexicon_tags_37 },
    { 0x597d8132, SUC_lexicon_tags_66 },
    { 0xbbd8135, SUC_lexicon_tags_63 },
    { 0x7bd8136, SUC_lexicon_tags_8 },
    { 0x8fdd8137, SUC_lexicon_tags_17 },
    { 0x44bd8134, SUC_lexicon_tags_68 },
    { 0x3e9d8135, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4edd813b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d8142, SUC_lexicon_tags_12 },
    { 0x715d8142, SUC_lexicon_tags_31 },
    { 0xd17d8144, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcb3d8146, SUC_lexicon_tags_17 },
    { 0x49d8147, SUC_lexicon_tags_38 },
    { 0xad1d8148, SUC_lexicon_tags_118 },
    { 0xeedd8149, SUC_lexicon_tags_60 },
    { 0x1d1d8148, SUC_lexicon_tags_25 },
    { 0x7bd8147, SUC_lexicon_tags_25 },
    { 0xcadd8148, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbbfd814e, SUC_lexicon_tags_38 },
    { 0x7e1d814f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443d8158, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1d815d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf13d815f, SUC_lexicon_tags_2 },
    { 0x79fd8160, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55d8165, SUC_lexicon_tags_160 },
    { 0xa3dd8166, SUC_lexicon_tags_43 },
    { 0xabd8166, SUC_lexicon_tags_7 },
    { 0x285d8165, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259d816c, SUC_lexicon_tags_43 },
    { 0x8e3d816d, SUC_lexicon_tags_46 },
    { 0xe2dd816d, SUC_lexicon_tags_26 },
    { 0xd35d816f, SUC_lexicon_tags_12 },
    { 0x91fd816c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2e1d8172, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bd8176, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xde1d8178, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233d8180, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3d8186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfc7d8188, SUC_lexicon_tags_48 },
    { 0x55bd8188, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97d818e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975d8193, SUC_lexicon_tags_25 },
    { 0xa27d8194, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fd8197, SUC_lexicon_tags_17 },
    { 0xfedd8197, SUC_lexicon_tags_2 },
    { 0xb0fd8199, SUC_lexicon_tags_7 },
    { 0x1b3d819a, SUC_lexicon_tags_25 },
    { 0x4b7d819b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bd81a8, SUC_lexicon_tags_15 },
    { 0x323d81a9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdd81ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7c9d81ae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x887d81b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4fd81b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4b3d81b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7d81b7, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99d81bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17d81bf, SUC_lexicon_tags_23 },
    { 0xc01d81c0, SUC_lexicon_tags_11 },
    { 0xb61d81c1, SUC_lexicon_tags_63 },
    { 0xebd81c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27d81c8, SUC_lexicon_tags_2 },
    { 0xcd1d81c8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5d81cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179d81d4, SUC_lexicon_tags_7 },
    { 0xd87d81d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6edd81d7, SUC_lexicon_tags_60 },
    { 0x227d81d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5d81e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbd81e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9d81ec, SUC_lexicon_tags_70 },
    { 0xcc5d81ed, SUC_lexicon_tags_27 },
    { 0xf57d81ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fd81f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x41bd81f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9d81f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x829d81f9, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x899d81fc, SUC_lexicon_tags_12 },
    { 0xb93d81fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d9d81ff, SUC_lexicon_tags_37 },
    { 0x80bd8200, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x8ffd8202, SUC_lexicon_tags_124 },
    { 0xa9d8203, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7abd8205, SUC_lexicon_tags_1 },
    { 0x67bd8206, SUC_lexicon_tags_32 },
    { 0x79bd8206, SUC_lexicon_tags_37 },
    { 0xcbdd8205, SUC_lexicon_tags_33 },
    { 0x925d8208, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x295d820c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc97d820e, SUC_lexicon_tags_38 },
    { 0xfbd820f, SUC_lexicon_tags_30 },
    { 0x283d8210, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261d8215, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x767d8218, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c3d821a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1d8220, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x50dd8222, SUC_lexicon_tags_7 },
    { 0x92bd8223, SUC_lexicon_tags_2 },
    { 0xe21d8224, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21d8227, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x65fd8229, SUC_lexicon_tags_2 },
    { 0xd9fd822a, SUC_lexicon_tags_2 },
    { 0x15d822b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899d8231, SUC_lexicon_tags_63 },
    { 0xa97d8232, SUC_lexicon_tags_6 },
    { 0x2dbd8231, SUC_lexicon_tags_37 },
    { 0x92bd8234, SUC_lexicon_tags_1 },
    { 0x4e5d8232, SUC_lexicon_tags_37 },
    { 0xf5d8233, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbd8239, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bdd823b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x745d823d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bd8243, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9d8247, SUC_lexicon_tags_17 },
    { 0xf03d8248, SUC_lexicon_tags_25 },
    { 0x8b5d8249, SUC_lexicon_tags_45 },
    { 0x5f5d824a, SUC_lexicon_tags_8 },
    { 0xbfbd824b, SUC_lexicon_tags_3 },
    { 0xa95d8248, SUC_lexicon_tags_1 },
    { 0xeedd824d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9d8251, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3fd8253, SUC_lexicon_tags_37 },
    { 0xd8fd8254, SUC_lexicon_tags_31 },
    { 0x721d8255, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5e7d8257, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7d825a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69bd825c, SUC_lexicon_tags_17 },
    { 0x8e5d825d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679d8261, SUC_lexicon_tags_12 },
    { 0x4b9d8262, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe45d8264, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3d8267, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4a7d8269, SUC_lexicon_tags_9 },
    { 0x179d8269, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x991d826c, SUC_lexicon_tags_27 },
    { 0x235d826c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbefd826f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31d8276, SUC_lexicon_tags_11 },
    { 0x835d8277, SUC_lexicon_tags_12 },
    { 0x9bd8277, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x847d827b, SUC_lexicon_tags_7 },
    { 0x847d827c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dd8280, SUC_lexicon_tags_28 },
    { 0xefd8281, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bd8284, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x383d8286, SUC_lexicon_tags_31 },
    { 0xafbd8287, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fd828a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25d8292, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25d8296, SUC_lexicon_tags_1 },
    { 0xacbd8297, SUC_lexicon_tags_17 },
    { 0x7dd8298, SUC_lexicon_tags_16 },
    { 0xee5d8299, SUC_lexicon_tags_15 },
    { 0xecdd829a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9d829f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf03d82a1, SUC_lexicon_tags_17 },
    { 0x221d82a2, SUC_lexicon_tags_49 },
    { 0xf63d82a3, SUC_lexicon_tags_45 },
    { 0xcffd82a4, SUC_lexicon_tags_37 },
    { 0x91dd82a5, SUC_lexicon_tags_0 },
    { 0x13bd82a4, SUC_lexicon_tags_27 },
    { 0xad9d82a3, SUC_lexicon_tags_1 },
    { 0x4b3d82a8, SUC_lexicon_tags_14 },
    { 0x733d82a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x261d82ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efd82af, SUC_lexicon_tags_7 },
    { 0xb43d82b0, SUC_lexicon_tags_29 },
    { 0x591d82b1, SUC_lexicon_tags_48 },
    { 0x5b9d82b2, SUC_lexicon_tags_230 },
    { 0xed1d82b3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebd82b6, SUC_lexicon_tags_60 },
    { 0x2e3d82b7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5d82bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9d82c2, SUC_lexicon_tags_29 },
    { 0x469d82c3, SUC_lexicon_tags_15 },
    { 0x4ddd82c3, SUC_lexicon_tags_4 },
    { 0xb75d82c5, SUC_lexicon_tags_0 },
    { 0x137d82c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fd82cc, SUC_lexicon_tags_25 },
    { 0xa5d82cc, SUC_lexicon_tags_42 },
    { 0x875d82cd, SUC_lexicon_tags_46 },
    { 0x69bd82cf, SUC_lexicon_tags_25 },
    { 0x3c9d82d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3d82d3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddd82d7, SUC_lexicon_tags_68 },
    { 0x45fd82d7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dd82dc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadd82e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x663d82e3, SUC_lexicon_tags_21 },
    { 0x4c1d82e3, SUC_lexicon_tags_23 },
    { 0x917d82e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafd82eb, SUC_lexicon_tags_47 },
    { 0xbdbd82ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bd82ef, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x1f9d82f1, SUC_lexicon_tags_8 },
    { 0xdadd82f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfd82f5, SUC_lexicon_tags_7 },
    { 0x58bd82f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58dd82fe, SUC_lexicon_tags_15 },
    { 0xaf5d82ff, SUC_lexicon_tags_27 },
    { 0x407d82ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x701d8302, SUC_lexicon_tags_15 },
    { 0x193d8303, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bd8306, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fd830a, SUC_lexicon_tags_10 },
    { 0xbd9d830b, SUC_lexicon_tags_15 },
    { 0x29dd830a, SUC_lexicon_tags_87 },
    { 0x8d1d830d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x847d830f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcddd8311, SUC_lexicon_tags_23 },
    { 0x87bd8312, SUC_lexicon_tags_11 },
    { 0xb9bd8311, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1d8317, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd831f, SUC_lexicon_tags_2 },
    { 0x475d831f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x149d8322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbd8327, SUC_lexicon_tags_12 },
    { 0x83d8327, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01d832b, SUC_lexicon_tags_15 },
    { 0x92bd832b, SUC_lexicon_tags_364 },
    { 0xd89d832d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb1bd832f, SUC_lexicon_tags_60 },
    { 0x245d8330, SUC_lexicon_tags_17 },
    { 0xdd3d8331, SUC_lexicon_tags_37 },
    { 0x189d8332, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9d8336, SUC_lexicon_tags_7 },
    { 0xcd3d8336, SUC_lexicon_tags_9 },
    { 0x1c5d8337, SUC_lexicon_tags_65 },
    { 0x2cbd8339, SUC_lexicon_tags_38 },
    { 0x999d8339, SUC_lexicon_tags_19 },
    { 0xce5d833b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dd833e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x339d8340, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x635d8342, SUC_lexicon_tags_2 },
    { 0x7ebd8342, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dd834b, SUC_lexicon_tags_17 },
    { 0x983d834b, SUC_lexicon_tags_37 },
    { 0xe99d834d, SUC_lexicon_tags_37 },
    { 0x68bd834b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xddd8351, SUC_lexicon_tags_45 },
    { 0x9dbd8352, SUC_lexicon_tags_11 },
    { 0x389d8353, SUC_lexicon_tags_60 },
    { 0xc45d8354, SUC_lexicon_tags_37 },
    { 0xfd1d8353, SUC_lexicon_tags_6 },
    { 0xb99d8353, SUC_lexicon_tags_46 },
    { 0x393d8352, SUC_lexicon_tags_12 },
    { 0x885d8354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63d835a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91d8360, SUC_lexicon_tags_33 },
    { 0x9e5d8361, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe25d8363, SUC_lexicon_tags_7 },
    { 0x15bd8364, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fd836b, SUC_lexicon_tags_0 },
    { 0x6bbd836c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3d8370, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe91d8372, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9d8379, SUC_lexicon_tags_15 },
    { 0xb77d8379, SUC_lexicon_tags_37 },
    { 0x997d8379, SUC_lexicon_tags_68 },
    { 0x619d837c, SUC_lexicon_tags_39 },
    { 0x1a9d837a, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bd8381, SUC_lexicon_tags_15 },
    { 0xfc9d8382, SUC_lexicon_tags_12 },
    { 0xd11d8383, SUC_lexicon_tags_30 },
    { 0xb0dd8384, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfd8388, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdd838c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1d838f, SUC_lexicon_tags_4 },
    { 0xf71d8390, SUC_lexicon_tags_14 },
    { 0xd87d8390, SUC_lexicon_tags_29 },
    { 0x4a1d8390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13d839b, SUC_lexicon_tags_15 },
    { 0xa55d839c, SUC_lexicon_tags_47 },
    { 0x3c3d839d, SUC_lexicon_tags_63 },
    { 0x26dd839b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc59d83a0, SUC_lexicon_tags_23 },
    { 0x131d83a1, SUC_lexicon_tags_37 },
    { 0xf07d83a1, SUC_lexicon_tags_4 },
    { 0x261d83a3, SUC_lexicon_tags_14 },
    { 0x859d83a2, SUC_lexicon_tags_27 },
    { 0x1d9d83a5, SUC_lexicon_tags_9 },
    { 0x7ffd83a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9d83af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bd83b5, SUC_lexicon_tags_2 },
    { 0x38bd83b5, SUC_lexicon_tags_25 },
    { 0x63bd83b7, SUC_lexicon_tags_15 },
    { 0x455d83b8, SUC_lexicon_tags_44 },
    { 0x61bd83b7, SUC_lexicon_tags_42 },
    { 0xaedd83ba, SUC_lexicon_tags_27 },
    { 0x3b3d83b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507d83bf, SUC_lexicon_tags_7 },
    { 0x4edd83c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fd83c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437d83c8, SUC_lexicon_tags_2 },
    { 0x373d83c8, SUC_lexicon_tags_15 },
    { 0xc1dd83ca, SUC_lexicon_tags_0 },
    { 0xb3dd83c8, SUC_lexicon_tags_1 },
    { 0x35fd83c9, SUC_lexicon_tags_7 },
    { 0x425d83cd, SUC_lexicon_tags_49 },
    { 0x4b7d83cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x64fd83d0, SUC_lexicon_tags_25 },
    { 0xdb5d83d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed9d83d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583d83da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52dd83dd, SUC_lexicon_tags_19 },
    { 0xa71d83de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xadbd83e0, SUC_lexicon_tags_15 },
    { 0x5efd83e1, SUC_lexicon_tags_25 },
    { 0x147d83e2, SUC_lexicon_tags_12 },
    { 0xd5bd83e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b3d83e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47bd83e9, SUC_lexicon_tags_25 },
    { 0x683d83ea, SUC_lexicon_tags_33 },
    { 0xa55d83ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x297d83ed, SUC_lexicon_tags_57 },
    { 0x353d83ee, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1d83f6, SUC_lexicon_tags_25 },
    { 0x32dd83f7, SUC_lexicon_tags_12 },
    { 0xde3d83f8, SUC_lexicon_tags_14 },
    { 0xce5d83f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d8400, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fd8404, SUC_lexicon_tags_80 },
    { 0x5b1d8405, SUC_lexicon_tags_12 },
    { 0x6add8406, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bd840a, SUC_lexicon_tags_11 },
    { 0xc51d840b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bd840f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x239d8412, SUC_lexicon_tags_2 },
    { 0xf19d8413, SUC_lexicon_tags_28 },
    { 0xef5d8414, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x833d8416, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x781d8418, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bd841f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca3d8421, SUC_lexicon_tags_55 },
    { 0x1c9d8422, SUC_lexicon_tags_66 },
    { 0xe4dd8421, SUC_lexicon_tags_0 },
    { 0xc41d8423, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89d842c, SUC_lexicon_tags_15 },
    { 0x3bbd842c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x417d842f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855d8433, SUC_lexicon_tags_7 },
    { 0x127d8433, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x73fd8436, SUC_lexicon_tags_23 },
    { 0x2ffd8436, SUC_lexicon_tags_37 },
    { 0xfa9d8438, SUC_lexicon_tags_19 },
    { 0x1d3d8439, SUC_lexicon_tags_65 },
    { 0x41dd8439, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3d843e, SUC_lexicon_tags_1 },
    { 0xaafd843f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9d8445, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5d8448, SUC_lexicon_tags_20 },
    { 0xfcbd8449, SUC_lexicon_tags_31 },
    { 0x91fd8448, SUC_lexicon_tags_47 },
    { 0x4e5d844b, SUC_lexicon_tags_48 },
    { 0xaedd844a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45d8454, SUC_lexicon_tags_30 },
    { 0xdabd8454, SUC_lexicon_tags_60 },
    { 0xc05d8455, SUC_lexicon_tags_31 },
    { 0x2d7d8455, SUC_lexicon_tags_60 },
    { 0xd83d8458, SUC_lexicon_tags_12 },
    { 0x765d8459, SUC_lexicon_tags_27 },
    { 0x9add8458, SUC_lexicon_tags_25 },
    { 0xb73d8459, SUC_lexicon_tags_37 },
    { 0xf85d845c, SUC_lexicon_tags_15 },
    { 0x25d845c, SUC_lexicon_tags_33 },
    { 0x70dd845c, SUC_lexicon_tags_19 },
    { 0x4c5d845f, SUC_lexicon_tags_37 },
    { 0x33fd845f, SUC_lexicon_tags_27 },
    { 0xa43d8460, SUC_lexicon_tags_351 },
    { 0xd3fd8462, SUC_lexicon_tags_7 },
    { 0x67dd8463, SUC_lexicon_tags_12 },
    { 0x197d8462, SUC_lexicon_tags_33 },
    { 0x7c3d845c, SUC_lexicon_tags_15 },
    { 0x77dd8466, SUC_lexicon_tags_16 },
    { 0x2abd8466, SUC_lexicon_tags_0 },
    { 0xd11d8468, SUC_lexicon_tags_31 },
    { 0x233d8469, SUC_lexicon_tags_23 },
    { 0x29dd8467, SUC_lexicon_tags_45 },
    { 0x191d8466, SUC_lexicon_tags_0 },
    { 0x83fd8467, SUC_lexicon_tags_107 },
    { 0x82fd846c, SUC_lexicon_tags_25 },
    { 0x9d7d846e, SUC_lexicon_tags_61 },
    { 0x9bd846f, SUC_lexicon_tags_15 },
    { 0xe09d8470, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x19bd8472, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x521d8474, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fd847a, SUC_lexicon_tags_21 },
    { 0xd97d847b, SUC_lexicon_tags_23 },
    { 0x67fd847c, SUC_lexicon_tags_30 },
    { 0x9fbd847d, SUC_lexicon_tags_158 },
    { 0xe33d847e, SUC_lexicon_tags_11 },
    { 0xb9d847f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97d8485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadd848a, SUC_lexicon_tags_17 },
    { 0x78dd848b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45d8490, SUC_lexicon_tags_10 },
    { 0x45bd8490, SUC_lexicon_tags_2 },
    { 0xb47d8490, SUC_lexicon_tags_37 },
    { 0xd73d8493, SUC_lexicon_tags_43 },
    { 0x8dbd8494, SUC_lexicon_tags_11 },
    { 0xaa3d8491, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5add849c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bd849f, SUC_lexicon_tags_12 },
    { 0xd93d849f, SUC_lexicon_tags_33 },
    { 0xe7fd84a0, SUC_lexicon_tags_11 },
    { 0xd59d84a1, SUC_lexicon_tags_0 },
    { 0x5b3d84a0, SUC_lexicon_tags_12 },
    { 0xb63d84a0, SUC_lexicon_tags_37 },
    { 0x475d84a5, SUC_lexicon_tags_8 },
    { 0xb91d84a6, SUC_lexicon_tags_12 },
    { 0xbcdd84a5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebd84ab, SUC_lexicon_tags_11 },
    { 0xf33d84ac, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbd84b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x533d84b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bd84bb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x331d84bd, SUC_lexicon_tags_16 },
    { 0x985d84be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57d84c2, SUC_lexicon_tags_63 },
    { 0xd89d84c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadd84c6, SUC_lexicon_tags_45 },
    { 0x9afd84c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc95d84c9, SUC_lexicon_tags_1 },
    { 0x625d84ca, SUC_lexicon_tags_27 },
    { 0x8c3d84cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dd84cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x771d84d1, SUC_lexicon_tags_39 },
    { 0x9bfd84d2, SUC_lexicon_tags_1 },
    { 0xb8bd84d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609d84d8, SUC_lexicon_tags_12 },
    { 0x4e5d84d9, SUC_lexicon_tags_11 },
    { 0xf39d84d8, SUC_lexicon_tags_66 },
    { 0xc3dd84db, SUC_lexicon_tags_37 },
    { 0x82fd84da, SUC_lexicon_tags_14 },
    { 0xdc1d84da, SUC_lexicon_tags_15 },
    { 0xa43d84dd, SUC_lexicon_tags_12 },
    { 0xd5fd84d8, SUC_lexicon_tags_7 },
    { 0xb0bd84df, SUC_lexicon_tags_84 },
    { 0x84fd84e1, SUC_lexicon_tags_61 },
    { 0xa2bd84e2, SUC_lexicon_tags_7 },
    { 0xee3d84e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fd84e7, SUC_lexicon_tags_1 },
    { 0x309d84e7, SUC_lexicon_tags_37 },
    { 0x47d84e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3d84ec, SUC_lexicon_tags_12 },
    { 0xae1d84ec, SUC_lexicon_tags_25 },
    { 0x613d84ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefd84f3, SUC_lexicon_tags_40 },
    { 0x43d84f4, SUC_lexicon_tags_37 },
    { 0x7a7d84f4, SUC_lexicon_tags_201 },
    { 0xeb7d84f6, SUC_lexicon_tags_7 },
    { 0x5a3d84f4, SUC_lexicon_tags_23 },
    { 0xf61d84f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43d84fa, SUC_lexicon_tags_9 },
    { 0x4add84fb, SUC_lexicon_tags_9 },
    { 0x245d84fc, SUC_lexicon_tags_37 },
    { 0x631d84fd, SUC_lexicon_tags_11 },
    { 0x723d84fe, SUC_lexicon_tags_3 },
    { 0x4b9d84fb, SUC_lexicon_tags_1 },
    { 0x321d84fd, SUC_lexicon_tags_42 },
    { 0xce7d8501, SUC_lexicon_tags_7 },
    { 0xed9d8502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7efd8505, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5d850a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73d850d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47d8510, SUC_lexicon_tags_11 },
    { 0x605d8511, SUC_lexicon_tags_28 },
    { 0x42fd8512, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141d8517, SUC_lexicon_tags_31 },
    { 0x1e5d8518, SUC_lexicon_tags_21 },
    { 0x875d8517, SUC_lexicon_tags_21 },
    { 0xaf3d8518, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6bd851c, SUC_lexicon_tags_8 },
    { 0xdbbd851c, SUC_lexicon_tags_17 },
    { 0x471d851e, SUC_lexicon_tags_33 },
    { 0x139d851f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59d8524, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bd8526, SUC_lexicon_tags_30 },
    { 0x1cfd8526, SUC_lexicon_tags_15 },
    { 0x28fd8528, SUC_lexicon_tags_38 },
    { 0x25fd8529, SUC_lexicon_tags_47 },
    { 0x73bd8526, SUC_lexicon_tags_12 },
    { 0x1b1d852b, SUC_lexicon_tags_1 },
    { 0x5add8526, SUC_lexicon_tags_25 },
    { 0xc7d852d, SUC_lexicon_tags_29 },
    { 0x141d852e, SUC_lexicon_tags_37 },
    { 0x7efd852f, SUC_lexicon_tags_275 },
    { 0x6d9d852e, SUC_lexicon_tags_1 },
    { 0xbd5d8531, SUC_lexicon_tags_10 },
    { 0xbe3d852b, SUC_lexicon_tags_13 },
    { 0xd25d8533, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fd8537, SUC_lexicon_tags_160 },
    { 0x279d8538, SUC_lexicon_tags_49 },
    { 0xd2dd8538, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1d853d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5d8543, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x2a3d8545, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfd854a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33d854d, SUC_lexicon_tags_49 },
    { 0xcebd854e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeefd8551, SUC_lexicon_tags_37 },
    { 0x8e5d8551, SUC_lexicon_tags_37 },
    { 0x5c5d8552, SUC_lexicon_tags_14 },
    { 0x3a5d8553, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9d8557, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d855d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xff7d855f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63d8563, SUC_lexicon_tags_14 },
    { 0xb2fd8564, SUC_lexicon_tags_65 },
    { 0x697d8564, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35d8568, SUC_lexicon_tags_25 },
    { 0x88fd8569, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9d856d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9d8570, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb37d8572, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bd8578, SUC_lexicon_tags_43 },
    { 0xf59d8579, SUC_lexicon_tags_19 },
    { 0x291d857a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dd857d, SUC_lexicon_tags_33 },
    { 0x2a9d857e, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07d8582, SUC_lexicon_tags_11 },
    { 0x82dd8582, SUC_lexicon_tags_37 },
    { 0x7f7d8584, SUC_lexicon_tags_19 },
    { 0xdd1d8585, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329d858b, SUC_lexicon_tags_118 },
    { 0x30bd858c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xa1dd858e, SUC_lexicon_tags_37 },
    { 0xc9bd858f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d859b, SUC_lexicon_tags_45 },
    { 0x359d859c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd93d859e, SUC_lexicon_tags_79 },
    { 0x291d859f, SUC_lexicon_tags_37 },
    { 0x749d859e, SUC_lexicon_tags_15 },
    { 0x94bd859f, SUC_lexicon_tags_1 },
    { 0xe87d859f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23d85a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5d85a8, SUC_lexicon_tags_646 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51d85ab, SUC_lexicon_tags_10 },
    { 0xb41d85ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x997d85ae, SUC_lexicon_tags_17 },
    { 0x55bd85af, SUC_lexicon_tags_60 },
    { 0xf37d85b0, SUC_lexicon_tags_7 },
    { 0x40bd85b1, SUC_lexicon_tags_38 },
    { 0x483d85b2, SUC_lexicon_tags_55 },
    { 0x31d85b1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe51d85b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddd85b8, SUC_lexicon_tags_19 },
    { 0x509d85b9, SUC_lexicon_tags_7 },
    { 0x711d85ba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x769d85bd, SUC_lexicon_tags_84 },
    { 0xbfdd85be, SUC_lexicon_tags_10 },
    { 0xdadd85bf, SUC_lexicon_tags_15 },
    { 0xe1dd85bd, SUC_lexicon_tags_63 },
    { 0x979d85c1, SUC_lexicon_tags_12 },
    { 0x46bd85c2, SUC_lexicon_tags_55 },
    { 0xfd85c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadd85c8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x539d85ca, SUC_lexicon_tags_17 },
    { 0x6bfd85ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71d85d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7d85d3, SUC_lexicon_tags_47 },
    { 0x1d3d85d4, SUC_lexicon_tags_1 },
    { 0x8ddd85d3, SUC_lexicon_tags_14 },
    { 0xc63d85d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6abd85d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bd85e2, SUC_lexicon_tags_31 },
    { 0x69dd85e3, SUC_lexicon_tags_187 },
    { 0x13fd85e4, SUC_lexicon_tags_10 },
    { 0x39fd85e4, SUC_lexicon_tags_29 },
    { 0x4bd85e6, SUC_lexicon_tags_2 },
    { 0x81dd85e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e7d85e9, SUC_lexicon_tags_17 },
    { 0x913d85e9, SUC_lexicon_tags_11 },
    { 0x461d85eb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45d85ee, SUC_lexicon_tags_57 },
    { 0x25bd85ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20dd85f3, SUC_lexicon_tags_11 },
    { 0x86bd85f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x319d85f6, SUC_lexicon_tags_7 },
    { 0xf67d85f7, SUC_lexicon_tags_33 },
    { 0x7d1d85f8, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x245d85fb, SUC_lexicon_tags_11 },
    { 0x50fd85fb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7d8602, SUC_lexicon_tags_3 },
    { 0x593d8603, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1d860a, SUC_lexicon_tags_25 },
    { 0x9abd860b, SUC_lexicon_tags_1 },
    { 0x665d860c, SUC_lexicon_tags_37 },
    { 0xafd860d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103d861a, SUC_lexicon_tags_42 },
    { 0xe33d861b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbd8620, SUC_lexicon_tags_0 },
    { 0x479d8621, SUC_lexicon_tags_17 },
    { 0x6a1d8621, SUC_lexicon_tags_27 },
    { 0x4e1d8620, SUC_lexicon_tags_47 },
    { 0x527d8624, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7d862a, SUC_lexicon_tags_37 },
    { 0x47dd862a, SUC_lexicon_tags_28 },
    { 0xaf7d862b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653d8631, SUC_lexicon_tags_17 },
    { 0xc35d8632, SUC_lexicon_tags_0 },
    { 0xe29d8633, SUC_lexicon_tags_14 },
    { 0xb4dd8633, SUC_lexicon_tags_111 },
    { 0x349d8632, SUC_lexicon_tags_43 },
    { 0xb13d8633, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7d863a, SUC_lexicon_tags_30 },
    { 0x269d863b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331d8643, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb67d8645, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe2bd8647, SUC_lexicon_tags_8 },
    { 0xfe1d8648, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17d8652, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d8655, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x199d8658, SUC_lexicon_tags_11 },
    { 0xdbdd8658, SUC_lexicon_tags_43 },
    { 0xcc3d865a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edd865e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79d8661, SUC_lexicon_tags_11 },
    { 0x9a3d8662, SUC_lexicon_tags_29 },
    { 0x779d8662, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fd8668, SUC_lexicon_tags_31 },
    { 0x33bd8668, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x157d866b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179d866f, SUC_lexicon_tags_23 },
    { 0xcc9d8670, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7d8673, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711d8677, SUC_lexicon_tags_1 },
    { 0x241d8678, SUC_lexicon_tags_29 },
    { 0xff3d8679, SUC_lexicon_tags_67 },
    { 0xe11d8679, SUC_lexicon_tags_2 },
    { 0x357d867b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd33d867d, SUC_lexicon_tags_1 },
    { 0xc59d867e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edd8682, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03d8687, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9d868d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x303d868f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33d8693, SUC_lexicon_tags_26 },
    { 0xde5d8694, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53d8699, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abd869e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x283d86a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09d86a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9d86ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dd86b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fd86b6, SUC_lexicon_tags_39 },
    { 0x63fd86b7, SUC_lexicon_tags_13 },
    { 0x81d86b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bd86bc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bd86cc, SUC_lexicon_tags_47 },
    { 0xa99d86cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x491d86d0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x60dd86d3, SUC_lexicon_tags_12 },
    { 0xd77d86d3, SUC_lexicon_tags_46 },
    { 0xcb1d86d5, SUC_lexicon_tags_1 },
    { 0x5abd86d4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7d86d9, SUC_lexicon_tags_11 },
    { 0x49bd86da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81d86dd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69d86e1, SUC_lexicon_tags_2 },
    { 0x42fd86e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bd86e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa03d86e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x27d86ea, SUC_lexicon_tags_25 },
    { 0x79dd86ea, SUC_lexicon_tags_33 },
    { 0x519d86ec, SUC_lexicon_tags_43 },
    { 0xbb5d86ea, SUC_lexicon_tags_25 },
    { 0xd63d86eb, SUC_lexicon_tags_64 },
    { 0x215d86ed, SUC_lexicon_tags_8 },
    { 0x99fd86f0, SUC_lexicon_tags_37 },
    { 0x6d9d86eb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x655d86f3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xfbd86f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd8fd86f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91d86fb, SUC_lexicon_tags_13 },
    { 0x3b7d86fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf21d86fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3d8703, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945d8709, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7d870c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfd8711, SUC_lexicon_tags_21 },
    { 0xf99d8712, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2a9d8714, SUC_lexicon_tags_46 },
    { 0x651d8715, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x139d8718, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x421d871b, SUC_lexicon_tags_4 },
    { 0xed3d871c, SUC_lexicon_tags_65 },
    { 0xb09d871c, SUC_lexicon_tags_2 },
    { 0xbd1d871b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6bfd8720, SUC_lexicon_tags_2 },
    { 0x63fd8721, SUC_lexicon_tags_37 },
    { 0x20fd8721, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93fd8724, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd7fd8726, SUC_lexicon_tags_25 },
    { 0x333d8727, SUC_lexicon_tags_11 },
    { 0x5fbd8728, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41d872c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a3d872e, SUC_lexicon_tags_3 },
    { 0x931d872e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfd8736, SUC_lexicon_tags_12 },
    { 0x483d8737, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bd873a, SUC_lexicon_tags_15 },
    { 0x805d873b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561d8742, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb9d8744, SUC_lexicon_tags_26 },
    { 0x779d8745, SUC_lexicon_tags_1 },
    { 0x629d8746, SUC_lexicon_tags_15 },
    { 0xf8fd8747, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215d874b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bd8750, SUC_lexicon_tags_7 },
    { 0xfb7d8750, SUC_lexicon_tags_12 },
    { 0xc8fd8752, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x581d8755, SUC_lexicon_tags_11 },
    { 0x7c9d8755, SUC_lexicon_tags_485 },
    { 0x9b1d8757, SUC_lexicon_tags_2 },
    { 0x97dd8755, SUC_lexicon_tags_15 },
    { 0x21d8759, SUC_lexicon_tags_7 },
    { 0xeafd875a, SUC_lexicon_tags_7 },
    { 0x737d875b, SUC_lexicon_tags_11 },
    { 0x589d875c, SUC_lexicon_tags_38 },
    { 0x255d875b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97dd8764, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x311d8766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d876e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dd8771, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53d8774, SUC_lexicon_tags_40 },
    { 0x237d8775, SUC_lexicon_tags_23 },
    { 0x767d8776, SUC_lexicon_tags_43 },
    { 0x9afd8776, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1d877b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5dfd877d, SUC_lexicon_tags_23 },
    { 0x48dd877d, SUC_lexicon_tags_1 },
    { 0xecfd877f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81d8786, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fd878c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2e9d878e, SUC_lexicon_tags_25 },
    { 0x7a7d878f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xdcdd8791, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fd8795, SUC_lexicon_tags_82 },
    { 0xf9d8795, SUC_lexicon_tags_46 },
    { 0x2dbd8796, SUC_lexicon_tags_12 },
    { 0x9a5d8798, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x379d879a, SUC_lexicon_tags_37 },
    { 0xa39d879a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383d879f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d87a7, SUC_lexicon_tags_23 },
    { 0xc21d87a8, SUC_lexicon_tags_82 },
    { 0x9b5d87a7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983d87af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5d87b5, SUC_lexicon_tags_40 },
    { 0x137d87b6, SUC_lexicon_tags_5 },
    { 0xe83d87b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x49fd87b9, SUC_lexicon_tags_37 },
    { 0x25dd87b9, SUC_lexicon_tags_0 },
    { 0x3d1d87bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561d87c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01d87d3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afd87d6, SUC_lexicon_tags_8 },
    { 0x91fd87d7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bd87df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5dd87e1, SUC_lexicon_tags_28 },
    { 0x793d87e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91d87e5, SUC_lexicon_tags_17 },
    { 0x1b5d87e5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5d87ee, SUC_lexicon_tags_75 },
    { 0xb7bd87ef, SUC_lexicon_tags_79 },
    { 0xa99d87f0, SUC_lexicon_tags_60 },
    { 0x967d87ef, SUC_lexicon_tags_82 },
    { 0x1dbd87ee, SUC_lexicon_tags_70 },
    { 0x37bd87f3, SUC_lexicon_tags_65 },
    { 0xf5fd87f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3d87fb, SUC_lexicon_tags_56 },
    { 0x81fd87fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5d8802, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9d8805, SUC_lexicon_tags_28 },
    { 0xb0dd8805, SUC_lexicon_tags_1 },
    { 0xdebd8807, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9d880e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d5d8810, SUC_lexicon_tags_15 },
    { 0x9dd8811, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dd8817, SUC_lexicon_tags_0 },
    { 0x561d8817, SUC_lexicon_tags_13 },
    { 0x101d8819, SUC_lexicon_tags_12 },
    { 0x5f5d8819, SUC_lexicon_tags_1 },
    { 0xaf5d881b, SUC_lexicon_tags_25 },
    { 0x13fd881c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d881f, SUC_lexicon_tags_46 },
    { 0x80fd8820, SUC_lexicon_tags_11 },
    { 0x29dd8820, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d8827, SUC_lexicon_tags_12 },
    { 0xa17d8828, SUC_lexicon_tags_15 },
    { 0x303d8828, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57d8832, SUC_lexicon_tags_38 },
    { 0xa5bd8833, SUC_lexicon_tags_70 },
    { 0xc4dd8833, SUC_lexicon_tags_29 },
    { 0x425d8835, SUC_lexicon_tags_37 },
    { 0x2c5d8836, SUC_lexicon_tags_45 },
    { 0xde7d8836, SUC_lexicon_tags_66 },
    { 0xa89d8835, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5d883c, SUC_lexicon_tags_7 },
    { 0xddbd883c, SUC_lexicon_tags_26 },
    { 0xd29d883e, SUC_lexicon_tags_129 },
    { 0x6bfd883e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x79dd8841, SUC_lexicon_tags_15 },
    { 0xdd7d8841, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65d8846, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xd45d8848, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43d884b, SUC_lexicon_tags_17 },
    { 0x9add884c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9b1d884e, SUC_lexicon_tags_17 },
    { 0x211d884f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bd8855, SUC_lexicon_tags_11 },
    { 0x473d8856, SUC_lexicon_tags_7 },
    { 0xfbd8856, SUC_lexicon_tags_60 },
    { 0xbe7d8856, SUC_lexicon_tags_15 },
    { 0x8fbd8859, SUC_lexicon_tags_30 },
    { 0xdafd8859, SUC_lexicon_tags_37 },
    { 0x289d885b, SUC_lexicon_tags_10 },
    { 0xdfbd885c, SUC_lexicon_tags_14 },
    { 0x94fd885c, SUC_lexicon_tags_3 },
    { 0x8a9d885d, SUC_lexicon_tags_2 },
    { 0x3e7d885d, SUC_lexicon_tags_28 },
    { 0xf7bd8860, SUC_lexicon_tags_114 },
    { 0xec1d885f, SUC_lexicon_tags_14 },
    { 0x5afd8862, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc8bd8864, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebbd8866, SUC_lexicon_tags_37 },
    { 0x889d8867, SUC_lexicon_tags_38 },
    { 0xd73d8868, SUC_lexicon_tags_37 },
    { 0x7e9d8869, SUC_lexicon_tags_61 },
    { 0x213d886a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x833d886d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bd8870, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9d8876, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92dd8878, SUC_lexicon_tags_12 },
    { 0x423d8878, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47d887f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fd8883, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103d8887, SUC_lexicon_tags_55 },
    { 0xb27d8887, SUC_lexicon_tags_17 },
    { 0x16dd8889, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x98fd888c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fd8890, SUC_lexicon_tags_49 },
    { 0xd9d8891, SUC_lexicon_tags_5 },
    { 0xc17d8892, SUC_lexicon_tags_49 },
    { 0x9a1d8891, SUC_lexicon_tags_37 },
    { 0xbc7d8894, SUC_lexicon_tags_29 },
    { 0x65dd8894, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe63d8897, SUC_lexicon_tags_37 },
    { 0x30fd8897, SUC_lexicon_tags_29 },
    { 0x9ebd8898, SUC_lexicon_tags_17 },
    { 0x103d889a, SUC_lexicon_tags_13 },
    { 0x62bd889b, SUC_lexicon_tags_2 },
    { 0x62dd889c, SUC_lexicon_tags_9 },
    { 0x9c7d889c, SUC_lexicon_tags_15 },
    { 0xe5d889e, SUC_lexicon_tags_11 },
    { 0xe29d889e, SUC_lexicon_tags_25 },
    { 0x7f3d88a0, SUC_lexicon_tags_0 },
    { 0x685d889f, SUC_lexicon_tags_0 },
    { 0xb1d8898, SUC_lexicon_tags_63 },
    { 0xa59d889a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x467d88a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fd88aa, SUC_lexicon_tags_16 },
    { 0x1d3d88aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861d88b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffd88b7, SUC_lexicon_tags_8 },
    { 0xcf1d88b8, SUC_lexicon_tags_13 },
    { 0x777d88b9, SUC_lexicon_tags_1 },
    { 0xa5dd88ba, SUC_lexicon_tags_25 },
    { 0xedd88bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3d88bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbd88c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937d88c7, SUC_lexicon_tags_17 },
    { 0xb61d88c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x227d88cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc6fd88cd, SUC_lexicon_tags_2 },
    { 0x4edd88ce, SUC_lexicon_tags_1 },
    { 0x7f7d88cf, SUC_lexicon_tags_17 },
    { 0x619d88cd, SUC_lexicon_tags_26 },
    { 0x159d88d1, SUC_lexicon_tags_26 },
    { 0x261d88ce, SUC_lexicon_tags_37 },
    { 0x4e5d88d0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01d88d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cbd88d9, SUC_lexicon_tags_25 },
    { 0x3c9d88da, SUC_lexicon_tags_12 },
    { 0x619d88db, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xdabd88dd, SUC_lexicon_tags_65 },
    { 0xb39d88dd, SUC_lexicon_tags_24 },
    { 0x1c1d88dd, SUC_lexicon_tags_25 },
    { 0x637d88e0, SUC_lexicon_tags_17 },
    { 0x797d88e1, SUC_lexicon_tags_2 },
    { 0x993d88e2, SUC_lexicon_tags_12 },
    { 0xb8dd88e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d88ea, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4a3d88ec, SUC_lexicon_tags_11 },
    { 0xe09d88ed, SUC_lexicon_tags_6 },
    { 0xccfd88ed, SUC_lexicon_tags_25 },
    { 0x5a3d88ec, SUC_lexicon_tags_10 },
    { 0xa85d88ee, SUC_lexicon_tags_14 },
    { 0x363d88f0, SUC_lexicon_tags_15 },
    { 0x151d88f2, SUC_lexicon_tags_79 },
    { 0xb97d88ee, SUC_lexicon_tags_1 },
    { 0xd27d88ef, SUC_lexicon_tags_10 },
    { 0xfd88f5, SUC_lexicon_tags_31 },
    { 0x787d88f5, SUC_lexicon_tags_8 },
    { 0xddd88f7, SUC_lexicon_tags_29 },
    { 0x61fd88f8, SUC_lexicon_tags_15 },
    { 0x205d88f9, SUC_lexicon_tags_48 },
    { 0x31fd88fa, SUC_lexicon_tags_60 },
    { 0xb77d88f6, SUC_lexicon_tags_23 },
    { 0xa49d88fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc2dd88fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x74fd8901, SUC_lexicon_tags_8 },
    { 0xa6bd8902, SUC_lexicon_tags_37 },
    { 0x64dd8903, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x873d8905, SUC_lexicon_tags_15 },
    { 0x6e7d8906, SUC_lexicon_tags_23 },
    { 0x71dd8906, SUC_lexicon_tags_7 },
    { 0xa91d8907, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3d890d, SUC_lexicon_tags_12 },
    { 0xb29d890e, SUC_lexicon_tags_53 },
    { 0xb29d890d, SUC_lexicon_tags_12 },
    { 0xd45d890f, SUC_lexicon_tags_15 },
    { 0x7a9d8911, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bd8914, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13d8917, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1f7d8919, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669d891e, SUC_lexicon_tags_12 },
    { 0x279d891e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x791d8922, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbd8929, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29d892c, SUC_lexicon_tags_63 },
    { 0xcc1d892d, SUC_lexicon_tags_33 },
    { 0x4e9d892e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x35bd8930, SUC_lexicon_tags_2 },
    { 0x3edd8930, SUC_lexicon_tags_55 },
    { 0xf5dd8932, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19d8937, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf99d8939, SUC_lexicon_tags_48 },
    { 0x863d893a, SUC_lexicon_tags_0 },
    { 0xfbbd893a, SUC_lexicon_tags_19 },
    { 0x80bd893c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97d893f, SUC_lexicon_tags_15 },
    { 0x52dd8940, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd05d8942, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdebd8944, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1d8949, SUC_lexicon_tags_11 },
    { 0x8bbd894a, SUC_lexicon_tags_42 },
    { 0x59dd894b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29fd894d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9d8950, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73d8953, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x753d8955, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bd8959, SUC_lexicon_tags_56 },
    { 0x3e5d8959, SUC_lexicon_tags_63 },
    { 0x81dd895a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95d895e, SUC_lexicon_tags_39 },
    { 0x3d895e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfcfd8961, SUC_lexicon_tags_19 },
    { 0x631d8961, SUC_lexicon_tags_60 },
    { 0x7f1d8961, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x703d8966, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfa7d8968, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfd896b, SUC_lexicon_tags_275 },
    { 0xdcdd896b, SUC_lexicon_tags_37 },
    { 0x6b3d896c, SUC_lexicon_tags_21 },
    { 0xb75d896d, SUC_lexicon_tags_40 },
    { 0xec5d896f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97d8973, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dd8977, SUC_lexicon_tags_15 },
    { 0xcb9d8978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x699d897b, SUC_lexicon_tags_17 },
    { 0x50bd897c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353d8981, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x491d8984, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x989d8986, SUC_lexicon_tags_19 },
    { 0x965d8986, SUC_lexicon_tags_15 },
    { 0x33fd8988, SUC_lexicon_tags_27 },
    { 0x5a7d8987, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07d898c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3bdd898e, SUC_lexicon_tags_11 },
    { 0xf15d898f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3d8992, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf5d8994, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5d89a2, SUC_lexicon_tags_15 },
    { 0x5f9d89a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5d89a7, SUC_lexicon_tags_1 },
    { 0xddfd89a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbffd89aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23d89ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbd89b2, SUC_lexicon_tags_11 },
    { 0x59fd89b3, SUC_lexicon_tags_2 },
    { 0x2fbd89b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433d89b9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27dd89bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dfd89bf, SUC_lexicon_tags_25 },
    { 0x863d89bf, SUC_lexicon_tags_11 },
    { 0xd4dd89bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1d89c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9dd89c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401d89d1, SUC_lexicon_tags_23 },
    { 0xcedd89d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x387d89d5, SUC_lexicon_tags_61 },
    { 0x731d89d5, SUC_lexicon_tags_77 },
    { 0x65d89d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d89de, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf57d89e0, SUC_lexicon_tags_47 },
    { 0xdf5d89e1, SUC_lexicon_tags_2 },
    { 0xd45d89e0, SUC_lexicon_tags_29 },
    { 0x9bd89e0, SUC_lexicon_tags_61 },
    { 0x1ddd89e3, SUC_lexicon_tags_25 },
    { 0x775d89e5, SUC_lexicon_tags_47 },
    { 0x915d89e6, SUC_lexicon_tags_60 },
    { 0xa25d89e1, SUC_lexicon_tags_25 },
    { 0x5c9d89e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dd89ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5d89f0, SUC_lexicon_tags_12 },
    { 0x1dfd89f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3d89f8, SUC_lexicon_tags_17 },
    { 0xdafd89f9, SUC_lexicon_tags_79 },
    { 0x161d89fa, SUC_lexicon_tags_17 },
    { 0x81fd89f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x283d89fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79d8a02, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfbbd8a04, SUC_lexicon_tags_79 },
    { 0xb6bd8a05, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985d8a09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc29d8a0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a7d8a0d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x40dd8a0f, SUC_lexicon_tags_10 },
    { 0x717d8a0f, SUC_lexicon_tags_27 },
    { 0xc5fd8a11, SUC_lexicon_tags_8 },
    { 0x341d8a0f, SUC_lexicon_tags_7 },
    { 0x58fd8a13, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9d8a16, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1d8a23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bd8a27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa33d8a29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bd8a2d, SUC_lexicon_tags_27 },
    { 0xfdbd8a2e, SUC_lexicon_tags_6 },
    { 0x561d8a2e, SUC_lexicon_tags_27 },
    { 0x677d8a30, SUC_lexicon_tags_47 },
    { 0x8a9d8a31, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xea1d8a33, SUC_lexicon_tags_25 },
    { 0x113d8a33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82dd8a36, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bd8a3a, SUC_lexicon_tags_8 },
    { 0xbb9d8a3a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763d8a41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf83d8a43, SUC_lexicon_tags_0 },
    { 0x323d8a44, SUC_lexicon_tags_5 },
    { 0x7a3d8a45, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9d8a4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x79d8a52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dd8a56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d8a5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07d8a60, SUC_lexicon_tags_55 },
    { 0xe31d8a60, SUC_lexicon_tags_15 },
    { 0x47fd8a62, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x30dd8a65, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5d8a6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dd8a6e, SUC_lexicon_tags_27 },
    { 0x8cbd8a6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdefd8a71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dd8a7b, SUC_lexicon_tags_87 },
    { 0x215d8a7b, SUC_lexicon_tags_25 },
    { 0xd65d8a7c, SUC_lexicon_tags_25 },
    { 0xc49d8a7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567d8a82, SUC_lexicon_tags_11 },
    { 0x12bd8a82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfd8a87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfb3d8a89, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf7d8a8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4c7d8a8d, SUC_lexicon_tags_12 },
    { 0x18bd8a8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89d8a96, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29fd8a9c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x297d8a9e, SUC_lexicon_tags_25 },
    { 0xdbfd8a9f, SUC_lexicon_tags_87 },
    { 0x167d8a9e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb67d8aa2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe17d8aa4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dd8aa8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bd8aab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59d8aae, SUC_lexicon_tags_3 },
    { 0xfb9d8aaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcedd8ab2, SUC_lexicon_tags_15 },
    { 0x6b7d8ab3, SUC_lexicon_tags_25 },
    { 0x709d8ab2, SUC_lexicon_tags_2 },
    { 0xe61d8ab2, SUC_lexicon_tags_11 },
    { 0x297d8ab3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55d8ab9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf6fd8abb, SUC_lexicon_tags_47 },
    { 0x6a3d8abb, SUC_lexicon_tags_1 },
    { 0x123d8abc, SUC_lexicon_tags_29 },
    { 0xaf7d8abe, SUC_lexicon_tags_4 },
    { 0x26dd8abf, SUC_lexicon_tags_25 },
    { 0xc19d8abf, SUC_lexicon_tags_4 },
    { 0x7fdd8abd, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb5d8ac3, SUC_lexicon_tags_87 },
    { 0x279d8ac3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbd8ac8, SUC_lexicon_tags_14 },
    { 0xea3d8ac9, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d8ad2, SUC_lexicon_tags_17 },
    { 0x38bd8ad3, SUC_lexicon_tags_12 },
    { 0x7ebd8ad4, SUC_lexicon_tags_15 },
    { 0xa73d8ad5, SUC_lexicon_tags_63 },
    { 0xc0bd8ad4, SUC_lexicon_tags_8 },
    { 0x4b9d8ad7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa8bd8ad9, SUC_lexicon_tags_26 },
    { 0x747d8ada, SUC_lexicon_tags_12 },
    { 0x6d5d8ada, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fd8ade, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dd8ae3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371d8ae7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffd8ae9, SUC_lexicon_tags_25 },
    { 0x769d8aea, SUC_lexicon_tags_49 },
    { 0xbafd8aeb, SUC_lexicon_tags_21 },
    { 0xee9d8aea, SUC_lexicon_tags_31 },
    { 0x7bd8aed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x137d8aef, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x17fd8af1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9d8af4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf7dd8af6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa79d8af8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa05d8afa, SUC_lexicon_tags_0 },
    { 0x4a3d8afb, SUC_lexicon_tags_66 },
    { 0x9f9d8afc, SUC_lexicon_tags_32 },
    { 0x231d8afd, SUC_lexicon_tags_12 },
    { 0x6f7d8afe, SUC_lexicon_tags_12 },
    { 0x75dd8aff, SUC_lexicon_tags_26 },
    { 0xcd3d8aff, SUC_lexicon_tags_61 },
    { 0x5e7d8b00, SUC_lexicon_tags_151 },
    { 0x4d9d8afd, SUC_lexicon_tags_79 },
    { 0xf91d8b03, SUC_lexicon_tags_140 },
    { 0x5e5d8b03, SUC_lexicon_tags_57 },
    { 0x8c5d8aff, SUC_lexicon_tags_38 },
    { 0xa01d8b06, SUC_lexicon_tags_60 },
    { 0x2ddd8afa, SUC_lexicon_tags_25 },
    { 0x35dd8b08, SUC_lexicon_tags_0 },
    { 0x325d8b08, SUC_lexicon_tags_2 },
    { 0x48dd8b0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbd8b0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321d8b14, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fd8b1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9d8b24, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x187d8b26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc5d8b28, SUC_lexicon_tags_17 },
    { 0x6d7d8b29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fd8b2e, SUC_lexicon_tags_146 },
    { 0x169d8b2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3d8b32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x569d8b34, SUC_lexicon_tags_11 },
    { 0x827d8b34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe25d8b37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x955d8b39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6add8b3c, SUC_lexicon_tags_12 },
    { 0x857d8b3d, SUC_lexicon_tags_15 },
    { 0x1a5d8b3e, SUC_lexicon_tags_9 },
    { 0xf7bd8b3f, SUC_lexicon_tags_25 },
    { 0x3a3d8b3d, SUC_lexicon_tags_12 },
    { 0xa07d8b41, SUC_lexicon_tags_10 },
    { 0x429d8b3f, SUC_lexicon_tags_27 },
    { 0x1cdd8b42, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x29d8b45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x295d8b47, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x40dd8b49, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x85bd8b4c, SUC_lexicon_tags_15 },
    { 0x3add8b4d, SUC_lexicon_tags_27 },
    { 0x331d8b4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe39d8b50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73fd8b52, SUC_lexicon_tags_8 },
    { 0xd21d8b53, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6dfd8b55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23d8b58, SUC_lexicon_tags_31 },
    { 0x56dd8b58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7d8b61, SUC_lexicon_tags_2 },
    { 0xe9dd8b62, SUC_lexicon_tags_3 },
    { 0x1d5d8b63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x109d8b65, SUC_lexicon_tags_37 },
    { 0x89dd8b66, SUC_lexicon_tags_43 },
    { 0xfedd8b66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5cbd8b69, SUC_lexicon_tags_15 },
    { 0x35bd8b6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dd8b6d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27d8b72, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x619d8b74, SUC_lexicon_tags_50 },
    { 0x2cbd8b75, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3d8b78, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x853d8b7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdd8b7e, SUC_lexicon_tags_85 },
    { 0x6f3d8b7f, SUC_lexicon_tags_17 },
    { 0x405d8b80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf79d8b83, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x77dd8b85, SUC_lexicon_tags_17 },
    { 0x24bd8b86, SUC_lexicon_tags_46 },
    { 0xd37d8b85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57d8b8c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x917d8b8e, SUC_lexicon_tags_12 },
    { 0xe1d8b8f, SUC_lexicon_tags_2 },
    { 0x17bd8b90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91d8b95, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87d8b99, SUC_lexicon_tags_1 },
    { 0xd5fd8b9a, SUC_lexicon_tags_31 },
    { 0xb59d8b9b, SUC_lexicon_tags_32 },
    { 0xe51d8b9a, SUC_lexicon_tags_49 },
    { 0xf1fd8b99, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x759d8b9f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97d8ba3, SUC_lexicon_tags_37 },
    { 0xde7d8ba4, SUC_lexicon_tags_7 },
    { 0x35bd8ba4, SUC_lexicon_tags_8 },
    { 0x9fdd8ba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07d8baa, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fd8bad, SUC_lexicon_tags_47 },
    { 0x909d8bad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9d8bb1, SUC_lexicon_tags_6 },
    { 0xf8bd8bb2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd5dd8bb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49d8bbb, SUC_lexicon_tags_1 },
    { 0x22fd8bbc, SUC_lexicon_tags_37 },
    { 0x58bd8bbb, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537d8bc2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309d8bc6, SUC_lexicon_tags_47 },
    { 0x9f9d8bc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2e3d8bc9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757d8bcd, SUC_lexicon_tags_1 },
    { 0xdb1d8bcd, SUC_lexicon_tags_1173 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5d8bd7, SUC_lexicon_tags_34 },
    { 0x1c5d8bd8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdd8bdd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3d8be1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81d8be4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x815d8be7, SUC_lexicon_tags_510 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5d8bea, SUC_lexicon_tags_63 },
    { 0x58dd8beb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67d8bf1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc43d8bf3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71d8bfa, SUC_lexicon_tags_3 },
    { 0x767d8bfa, SUC_lexicon_tags_37 },
    { 0x3d9d8bfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfd8c00, SUC_lexicon_tags_47 },
    { 0x7f1d8c01, SUC_lexicon_tags_14 },
    { 0x639d8c00, SUC_lexicon_tags_12 },
    { 0xde5d8c02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99dd8c05, SUC_lexicon_tags_19 },
    { 0xab1d8c05, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x217d8c08, SUC_lexicon_tags_63 },
    { 0xf65d8c08, SUC_lexicon_tags_14 },
    { 0x557d8c0a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98dd8c0f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1d8c13, SUC_lexicon_tags_38 },
    { 0x855d8c14, SUC_lexicon_tags_11 },
    { 0x483d8c15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3a5d8c17, SUC_lexicon_tags_27 },
    { 0x433d8c18, SUC_lexicon_tags_38 },
    { 0x1ddd8c19, SUC_lexicon_tags_2 },
    { 0x3e3d8c1a, SUC_lexicon_tags_7 },
    { 0xda5d8c18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x195d8c1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfd8c22, SUC_lexicon_tags_17 },
    { 0x79d8c23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3dd8c25, SUC_lexicon_tags_79 },
    { 0xe2dd8c26, SUC_lexicon_tags_27 },
    { 0x9efd8c25, SUC_lexicon_tags_17 },
    { 0x65bd8c25, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x3d3d8c2a, SUC_lexicon_tags_37 },
    { 0xbd7d8c2b, SUC_lexicon_tags_37 },
    { 0xce5d8c2c, SUC_lexicon_tags_8 },
    { 0x697d8c2c, SUC_lexicon_tags_27 },
    { 0x69dd8c2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47d8c32, SUC_lexicon_tags_26 },
    { 0x9cdd8c33, SUC_lexicon_tags_25 },
    { 0xc09d8c34, SUC_lexicon_tags_29 },
    { 0x2bbd8c32, SUC_lexicon_tags_43 },
    { 0x195d8c36, SUC_lexicon_tags_25 },
    { 0xb69d8c36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dd8c3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21d8c3d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbd8c41, SUC_lexicon_tags_25 },
    { 0x59fd8c42, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153d8c49, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dd8c4f, SUC_lexicon_tags_21 },
    { 0x2b9d8c4f, SUC_lexicon_tags_15 },
    { 0x1c7d8c50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9dd8c53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1d8c5a, SUC_lexicon_tags_0 },
    { 0x7cbd8c5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43d8c5f, SUC_lexicon_tags_28 },
    { 0xd17d8c60, SUC_lexicon_tags_19 },
    { 0x9d8c5f, SUC_lexicon_tags_1615 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbd8c6a, SUC_lexicon_tags_38 },
    { 0x759d8c6b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x983d8c6d, SUC_lexicon_tags_48 },
    { 0x387d8c6d, SUC_lexicon_tags_82 },
    { 0xaf1d8c6f, SUC_lexicon_tags_21 },
    { 0x971d8c6f, SUC_lexicon_tags_1 },
    { 0xbc7d8c70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fd8c74, SUC_lexicon_tags_11 },
    { 0x777d8c74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9d8c78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3d8c7b, SUC_lexicon_tags_105 },
    { 0x759d8c7c, SUC_lexicon_tags_25 },
    { 0x467d8c7d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13d8c85, SUC_lexicon_tags_49 },
    { 0x611d8c85, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x80dd8c88, SUC_lexicon_tags_1 },
    { 0xc33d8c88, SUC_lexicon_tags_60 },
    { 0xba7d8c8a, SUC_lexicon_tags_66 },
    { 0x833d8c8b, SUC_lexicon_tags_26 },
    { 0xe09d8c8c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x9a1d8c8e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7d9d8c90, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf5d8c92, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3d8c95, SUC_lexicon_tags_38 },
    { 0xb93d8c96, SUC_lexicon_tags_33 },
    { 0x4f1d8c97, SUC_lexicon_tags_22 },
    { 0x861d8c95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37d8c9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeffd8c9f, SUC_lexicon_tags_27 },
    { 0xd45d8c9f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2a1d8ca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdd8ca5, SUC_lexicon_tags_31 },
    { 0x12bd8ca6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdd8cae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x497d8cb0, SUC_lexicon_tags_60 },
    { 0x431d8cb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83d8cb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x271d8cbb, SUC_lexicon_tags_45 },
    { 0xa69d8cbc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7d8cc0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddd8cc4, SUC_lexicon_tags_9 },
    { 0xc0fd8cc5, SUC_lexicon_tags_30 },
    { 0xd19d8cc6, SUC_lexicon_tags_1 },
    { 0xe97d8cc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd65d8cc9, SUC_lexicon_tags_12 },
    { 0x6cbd8cca, SUC_lexicon_tags_33 },
    { 0x919d8cca, SUC_lexicon_tags_47 },
    { 0x4a9d8ccb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fd8ccf, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3d8cd3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1d8cd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ddd8cdb, SUC_lexicon_tags_25 },
    { 0x39dd8cdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86fd8cde, SUC_lexicon_tags_2 },
    { 0x447d8cdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9d8ce3, SUC_lexicon_tags_55 },
    { 0xa7dd8ce4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2a1d8ce6, SUC_lexicon_tags_37 },
    { 0x4b9d8ce7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01d8ceb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x929d8ced, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165d8cf2, SUC_lexicon_tags_15 },
    { 0xfa5d8cf2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x19bd8cf5, SUC_lexicon_tags_45 },
    { 0xa13d8cf6, SUC_lexicon_tags_1 },
    { 0x3b7d8cf7, SUC_lexicon_tags_38 },
    { 0x42fd8cf8, SUC_lexicon_tags_43 },
    { 0xc5bd8cf8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x13d8cfb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x701d8cfd, SUC_lexicon_tags_49 },
    { 0xcfd8cfe, SUC_lexicon_tags_7 },
    { 0xc55d8cfe, SUC_lexicon_tags_60 },
    { 0xc5d8d00, SUC_lexicon_tags_15 },
    { 0xec5d8cfd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dd8d0a, SUC_lexicon_tags_63 },
    { 0x15bd8d0b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbd8d10, SUC_lexicon_tags_49 },
    { 0xa3fd8d11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3abd8d13, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17d8d16, SUC_lexicon_tags_7 },
    { 0xdd1d8d17, SUC_lexicon_tags_60 },
    { 0xe1fd8d18, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd8d1b, SUC_lexicon_tags_62 },
    { 0x55d8d1b, SUC_lexicon_tags_11 },
    { 0xdffd8d1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21bd8d20, SUC_lexicon_tags_8 },
    { 0xb17d8d21, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xedbd8d23, SUC_lexicon_tags_111 },
    { 0xaffd8d23, SUC_lexicon_tags_35 },
    { 0x477d8d25, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x221d8d27, SUC_lexicon_tags_7 },
    { 0xfa5d8d28, SUC_lexicon_tags_16 },
    { 0xca9d8d27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c5d8d2b, SUC_lexicon_tags_63 },
    { 0xa9dd8d2c, SUC_lexicon_tags_11 },
    { 0xb19d8d2c, SUC_lexicon_tags_29 },
    { 0x353d8d2e, SUC_lexicon_tags_27 },
    { 0x7a5d8d2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebd8d36, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835d8d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0bd8d3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffd8d42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c5d8d44, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58dd8d51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefdd8d53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x609d8d55, SUC_lexicon_tags_15 },
    { 0x5a3d8d56, SUC_lexicon_tags_1 },
    { 0xc33d8d57, SUC_lexicon_tags_34 },
    { 0x531d8d56, SUC_lexicon_tags_58 },
    { 0xda3d8d59, SUC_lexicon_tags_15 },
    { 0xf5fd8d5a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x69fd8d5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfd8d62, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf55d8d64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaadd8d66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50dd8d68, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfd8d6b, SUC_lexicon_tags_31 },
    { 0x9a9d8d6c, SUC_lexicon_tags_229 },
    { 0x527d8d6b, SUC_lexicon_tags_15 },
    { 0xe0dd8d6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fd8d72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbd8d76, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x563d8d78, SUC_lexicon_tags_2 },
    { 0x1c9d8d79, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495d8d7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55d8d83, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b1d8d85, SUC_lexicon_tags_15 },
    { 0xdddd8d86, SUC_lexicon_tags_9 },
    { 0xeddd8d87, SUC_lexicon_tags_25 },
    { 0x3cbd8d86, SUC_lexicon_tags_7 },
    { 0x1c5d8d89, SUC_lexicon_tags_11 },
    { 0x6bd8d89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x759d8d8d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5fbd8d8f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x99bd8d91, SUC_lexicon_tags_30 },
    { 0xda9d8d92, SUC_lexicon_tags_0 },
    { 0x1d1d8d93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89d8d9a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf45d8d9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9d8da0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9d8da6, SUC_lexicon_tags_25 },
    { 0xc17d8da7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991d8dab, SUC_lexicon_tags_6 },
    { 0xa35d8dac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x245d8daf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125d8dba, SUC_lexicon_tags_14 },
    { 0x925d8dbb, SUC_lexicon_tags_47 },
    { 0xc63d8dbc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571d8dc2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee5d8dc4, SUC_lexicon_tags_9 },
    { 0x10dd8dc5, SUC_lexicon_tags_45 },
    { 0x8bdd8dc6, SUC_lexicon_tags_11 },
    { 0x28fd8dc4, SUC_lexicon_tags_11 },
    { 0x529d8dc8, SUC_lexicon_tags_15 },
    { 0x623d8dc9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51d8dcc, SUC_lexicon_tags_95 },
    { 0x2fdd8dcc, SUC_lexicon_tags_38 },
    { 0x9dbd8dce, SUC_lexicon_tags_14 },
    { 0x199d8dcf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8e9d8dd1, SUC_lexicon_tags_24 },
    { 0x3b7d8dd1, SUC_lexicon_tags_27 },
    { 0x8c1d8dd1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5d8ddc, SUC_lexicon_tags_37 },
    { 0xe03d8ddd, SUC_lexicon_tags_38 },
    { 0x94dd8dde, SUC_lexicon_tags_4 },
    { 0xa9d8ddf, SUC_lexicon_tags_0 },
    { 0x293d8dde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x143d8de3, SUC_lexicon_tags_63 },
    { 0x5cbd8de4, SUC_lexicon_tags_26 },
    { 0xe77d8de3, SUC_lexicon_tags_43 },
    { 0x7fbd8de3, SUC_lexicon_tags_61 },
    { 0x7dd8de5, SUC_lexicon_tags_2 },
    { 0xd37d8de8, SUC_lexicon_tags_4 },
    { 0xe53d8de9, SUC_lexicon_tags_46 },
    { 0x9bdd8de7, SUC_lexicon_tags_19 },
    { 0xdc1d8deb, SUC_lexicon_tags_23 },
    { 0xa73d8dec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531d8df0, SUC_lexicon_tags_2 },
    { 0x54dd8df1, SUC_lexicon_tags_25 },
    { 0xee7d8df2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x21dd8df5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdafd8df7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeafd8df9, SUC_lexicon_tags_99 },
    { 0x6add8dfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x88dd8dfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd03d8dfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dd8e04, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3d8e09, SUC_lexicon_tags_0 },
    { 0xfe3d8e0a, SUC_lexicon_tags_43 },
    { 0x39dd8e0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bd8e0f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577d8e14, SUC_lexicon_tags_11 },
    { 0x2d1d8e15, SUC_lexicon_tags_15 },
    { 0x2fbd8e16, SUC_lexicon_tags_9 },
    { 0x971d8e17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1d8e1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661d8e22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bd8e27, SUC_lexicon_tags_60 },
    { 0x6cfd8e28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe57d8e2a, SUC_lexicon_tags_25 },
    { 0xcb3d8e2b, SUC_lexicon_tags_28 },
    { 0x5cdd8e2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7d5d8e2e, SUC_lexicon_tags_66 },
    { 0x8b9d8e2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x819d8e32, SUC_lexicon_tags_43 },
    { 0x5cdd8e33, SUC_lexicon_tags_7 },
    { 0x85d8e32, SUC_lexicon_tags_15 },
    { 0xf4fd8e33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5d8e3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5d8e40, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fd8e44, SUC_lexicon_tags_16 },
    { 0x6a5d8e45, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8f3d8e47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x975d8e4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999d8e4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a1d8e50, SUC_lexicon_tags_9 },
    { 0xc77d8e51, SUC_lexicon_tags_7 },
    { 0x333d8e51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efd8e55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf1d8e57, SUC_lexicon_tags_25 },
    { 0x249d8e58, SUC_lexicon_tags_37 },
    { 0x491d8e58, SUC_lexicon_tags_15 },
    { 0x4d5d8e5a, SUC_lexicon_tags_17 },
    { 0x2f5d8e59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x627d8e5d, SUC_lexicon_tags_2 },
    { 0x7d7d8e5e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7d8e64, SUC_lexicon_tags_25 },
    { 0xfa3d8e65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bd8e69, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7d8e6d, SUC_lexicon_tags_33 },
    { 0x1a5d8e6e, SUC_lexicon_tags_23 },
    { 0x5ddd8e6f, SUC_lexicon_tags_37 },
    { 0x955d8e70, SUC_lexicon_tags_87 },
    { 0x4dbd8e71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdd8e74, SUC_lexicon_tags_17 },
    { 0x433d8e75, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xcb9d8e77, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dd8e7b, SUC_lexicon_tags_11 },
    { 0x49dd8e7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x40fd8e7e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdd8e83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x541d8e86, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffd8e8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163d8e8f, SUC_lexicon_tags_17 },
    { 0x8f1d8e90, SUC_lexicon_tags_39 },
    { 0xbd8e91, SUC_lexicon_tags_2 },
    { 0x891d8e91, SUC_lexicon_tags_38 },
    { 0xebfd8e92, SUC_lexicon_tags_11 },
    { 0x3c3d8e94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdd8e99, SUC_lexicon_tags_33 },
    { 0x485d8e9a, SUC_lexicon_tags_30 },
    { 0x5e9d8e99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bd8e9f, SUC_lexicon_tags_49 },
    { 0xc6bd8ea0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7d8ea6, SUC_lexicon_tags_11 },
    { 0xa3dd8ea7, SUC_lexicon_tags_106 },
    { 0x8abd8ea8, SUC_lexicon_tags_2 },
    { 0x55d8ea9, SUC_lexicon_tags_46 },
    { 0x227d8eaa, SUC_lexicon_tags_37 },
    { 0xb4fd8ea8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fd8eae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07d8eb5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dd8ebb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b7d8ebd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429d8ec5, SUC_lexicon_tags_17 },
    { 0x555d8ec6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x351d8ec8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x149d8eca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dd8ecf, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbd8ed7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dd8edb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf5d8edd, SUC_lexicon_tags_11 },
    { 0x749d8edd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3d8ee2, SUC_lexicon_tags_57 },
    { 0x4f3d8ee3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55d8ee6, SUC_lexicon_tags_47 },
    { 0x265d8ee7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147d8eec, SUC_lexicon_tags_63 },
    { 0x12fd8eec, SUC_lexicon_tags_3 },
    { 0xf33d8eed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23d8ef1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x657d8ef4, SUC_lexicon_tags_8 },
    { 0x3fbd8ef4, SUC_lexicon_tags_4 },
    { 0xb3d8ef5, SUC_lexicon_tags_45 },
    { 0xc9d8ef7, SUC_lexicon_tags_1 },
    { 0xb49d8ef8, SUC_lexicon_tags_4 },
    { 0xbdd8ef9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf29d8efb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143d8f04, SUC_lexicon_tags_43 },
    { 0x471d8f04, SUC_lexicon_tags_37 },
    { 0xfdbd8f06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x24bd8f09, SUC_lexicon_tags_12 },
    { 0x7a9d8f0a, SUC_lexicon_tags_14 },
    { 0x52bd8f0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8bd8f0d, SUC_lexicon_tags_4 },
    { 0xb59d8f0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd71d8f10, SUC_lexicon_tags_79 },
    { 0x7f7d8f11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4cfd8f13, SUC_lexicon_tags_2 },
    { 0xa09d8f13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x691d8f16, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dd8f1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x947d8f1e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe0dd8f20, SUC_lexicon_tags_25 },
    { 0x5efd8f20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63d8f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25d8f2a, SUC_lexicon_tags_9 },
    { 0x43dd8f2b, SUC_lexicon_tags_32 },
    { 0x4d1d8f2c, SUC_lexicon_tags_11 },
    { 0xc5d8f2d, SUC_lexicon_tags_46 },
    { 0xb23d8f2e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbbbd8f30, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fd8f35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17d8f3e, SUC_lexicon_tags_47 },
    { 0xb5d8f3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25d8f49, SUC_lexicon_tags_6 },
    { 0xcdd8f4a, SUC_lexicon_tags_15 },
    { 0x851d8f49, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dd8f53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5d8f56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535d8f5b, SUC_lexicon_tags_34 },
    { 0xb87d8f5c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x713d8f5f, SUC_lexicon_tags_0 },
    { 0x167d8f60, SUC_lexicon_tags_2 },
    { 0xd0fd8f60, SUC_lexicon_tags_37 },
    { 0xf9fd8f62, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5d8f67, SUC_lexicon_tags_39 },
    { 0x97fd8f67, SUC_lexicon_tags_7 },
    { 0x107d8f68, SUC_lexicon_tags_49 },
    { 0x4bdd8f6a, SUC_lexicon_tags_27 },
    { 0x42dd8f6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3d8f70, SUC_lexicon_tags_7 },
    { 0xa1dd8f71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffd8f76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff5d8f78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4c5d8f7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa49d8f7c, SUC_lexicon_tags_6 },
    { 0x189d8f7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93d8f81, SUC_lexicon_tags_23 },
    { 0xf3dd8f81, SUC_lexicon_tags_7 },
    { 0x7c5d8f81, SUC_lexicon_tags_7 },
    { 0xb17d8f82, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abd8f89, SUC_lexicon_tags_21 },
    { 0x169d8f89, SUC_lexicon_tags_14 },
    { 0x82dd8f8b, SUC_lexicon_tags_47 },
    { 0x225d8f8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9d8f91, SUC_lexicon_tags_61 },
    { 0x141d8f92, SUC_lexicon_tags_46 },
    { 0xb59d8f93, SUC_lexicon_tags_68 },
    { 0xa85d8f91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbd8f97, SUC_lexicon_tags_7 },
    { 0xd0dd8f98, SUC_lexicon_tags_13 },
    { 0x9d5d8f99, SUC_lexicon_tags_2 },
    { 0xca9d8f97, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x893d8f9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455d8fa1, SUC_lexicon_tags_14 },
    { 0xe83d8fa2, SUC_lexicon_tags_63 },
    { 0xe8fd8fa3, SUC_lexicon_tags_1 },
    { 0xad1d8fa3, SUC_lexicon_tags_77 },
    { 0x4dbd8fa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa67d8fa7, SUC_lexicon_tags_25 },
    { 0x31bd8fa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x153d8fab, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb63d8fad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x197d8faf, SUC_lexicon_tags_13 },
    { 0xca5d8fb0, SUC_lexicon_tags_0 },
    { 0x711d8faf, SUC_lexicon_tags_2 },
    { 0xb03d8fb0, SUC_lexicon_tags_17 },
    { 0x70fd8fb1, SUC_lexicon_tags_17 },
    { 0x86bd8fb4, SUC_lexicon_tags_15 },
    { 0xdfbd8fb5, SUC_lexicon_tags_17 },
    { 0x65fd8fb6, SUC_lexicon_tags_15 },
    { 0xb85d8fb4, SUC_lexicon_tags_26 },
    { 0x451d8fb1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693d8fbe, SUC_lexicon_tags_14 },
    { 0x171d8fbe, SUC_lexicon_tags_11 },
    { 0xec5d8fbf, SUC_lexicon_tags_17 },
    { 0xfc9d8fbf, SUC_lexicon_tags_27 },
    { 0xcc1d8fc1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc63d8fc4, SUC_lexicon_tags_17 },
    { 0xc99d8fc4, SUC_lexicon_tags_0 },
    { 0xf13d8fc6, SUC_lexicon_tags_23 },
    { 0x4bd8fc7, SUC_lexicon_tags_17 },
    { 0x5c1d8fc7, SUC_lexicon_tags_45 },
    { 0x12bd8fc9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153d8fcf, SUC_lexicon_tags_11 },
    { 0xe6bd8fd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835d8fd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x58fd8fdb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65d8fdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59bd8fe1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921d8fe6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91d8fe9, SUC_lexicon_tags_45 },
    { 0x695d8fea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bd8ff2, SUC_lexicon_tags_19 },
    { 0xda7d8ff3, SUC_lexicon_tags_17 },
    { 0x1c5d8ff4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bd8ffe, SUC_lexicon_tags_17 },
    { 0x1f7d8fff, SUC_lexicon_tags_32 },
    { 0x181d8fff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x147d9002, SUC_lexicon_tags_63 },
    { 0xc6dd9002, SUC_lexicon_tags_31 },
    { 0x7f9d9004, SUC_lexicon_tags_13 },
    { 0x6d1d9005, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fd9009, SUC_lexicon_tags_7 },
    { 0x43d900a, SUC_lexicon_tags_1 },
    { 0xf0bd900b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565d9010, SUC_lexicon_tags_45 },
    { 0x643d9011, SUC_lexicon_tags_29 },
    { 0x165d9011, SUC_lexicon_tags_33 },
    { 0xe7fd9013, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x59d9016, SUC_lexicon_tags_31 },
    { 0x6c3d9017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d901b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfd901e, SUC_lexicon_tags_2 },
    { 0xa4bd901e, SUC_lexicon_tags_17 },
    { 0xe1dd901f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd9027, SUC_lexicon_tags_37 },
    { 0x5f3d9028, SUC_lexicon_tags_14 },
    { 0x931d9029, SUC_lexicon_tags_33 },
    { 0x7bfd902a, SUC_lexicon_tags_29 },
    { 0x41fd902a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5d9035, SUC_lexicon_tags_35 },
    { 0xe5d9036, SUC_lexicon_tags_79 },
    { 0x737d9036, SUC_lexicon_tags_65 },
    { 0xcd9d9038, SUC_lexicon_tags_38 },
    { 0xa53d9039, SUC_lexicon_tags_11 },
    { 0x6c7d903a, SUC_lexicon_tags_12 },
    { 0x1e3d9038, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8cfd903d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x157d903f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xec7d9041, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7d9044, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5d9047, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719d9051, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33d9058, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x87fd905a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125d905e, SUC_lexicon_tags_27 },
    { 0xa27d905f, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5d9063, SUC_lexicon_tags_82 },
    { 0x445d9063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbd9067, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x13bd9069, SUC_lexicon_tags_39 },
    { 0xcd7d9069, SUC_lexicon_tags_1 },
    { 0xc2dd906b, SUC_lexicon_tags_37 },
    { 0x6bd906a, SUC_lexicon_tags_11 },
    { 0xf63d906d, SUC_lexicon_tags_11 },
    { 0xd31d906e, SUC_lexicon_tags_11 },
    { 0xfd3d906f, SUC_lexicon_tags_33 },
    { 0xb7bd9070, SUC_lexicon_tags_47 },
    { 0x51d906e, SUC_lexicon_tags_37 },
    { 0xb03d9070, SUC_lexicon_tags_1 },
    { 0x83bd906e, SUC_lexicon_tags_53 },
    { 0x51dd906d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941d9078, SUC_lexicon_tags_17 },
    { 0x989d9079, SUC_lexicon_tags_47 },
    { 0xb11d9078, SUC_lexicon_tags_70 },
    { 0xdc9d907b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191d907f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7d9086, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x77dd9088, SUC_lexicon_tags_82 },
    { 0x869d9088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe6bd908b, SUC_lexicon_tags_49 },
    { 0x9f9d908c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679d9097, SUC_lexicon_tags_87 },
    { 0xa97d9097, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345d90a0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x723d90a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bd90a6, SUC_lexicon_tags_43 },
    { 0x1bdd90a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245d90ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e1d90ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edd90b2, SUC_lexicon_tags_11 },
    { 0x93bd90b3, SUC_lexicon_tags_7 },
    { 0xfdd90b4, SUC_lexicon_tags_17 },
    { 0x65dd90b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9d90ba, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fd90bd, SUC_lexicon_tags_17 },
    { 0xa6bd90be, SUC_lexicon_tags_1 },
    { 0x299d90bf, SUC_lexicon_tags_38 },
    { 0x57fd90c0, SUC_lexicon_tags_3 },
    { 0xfa3d90bf, SUC_lexicon_tags_1 },
    { 0x517d90c2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dd90c7, SUC_lexicon_tags_15 },
    { 0xf65d90c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebd90cc, SUC_lexicon_tags_33 },
    { 0x84bd90cd, SUC_lexicon_tags_12 },
    { 0xdd9d90cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5d90d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1c1d90d3, SUC_lexicon_tags_33 },
    { 0x68dd90d4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5b1d90d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfd90dd, SUC_lexicon_tags_33 },
    { 0x4a1d90de, SUC_lexicon_tags_61 },
    { 0x4d5d90df, SUC_lexicon_tags_15 },
    { 0x4bbd90df, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bd90e4, SUC_lexicon_tags_49 },
    { 0x50fd90e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25d90e9, SUC_lexicon_tags_43 },
    { 0xedfd90e9, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7d90ef, SUC_lexicon_tags_17 },
    { 0xd13d90ef, SUC_lexicon_tags_82 },
    { 0x2abd90f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59dd90f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3d90f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4bbd90f9, SUC_lexicon_tags_0 },
    { 0xb8fd90fa, SUC_lexicon_tags_14 },
    { 0x845d90fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bd90ff, SUC_lexicon_tags_33 },
    { 0x7ddd9100, SUC_lexicon_tags_40 },
    { 0xb6bd9101, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29d9104, SUC_lexicon_tags_9 },
    { 0xfafd9105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89d910f, SUC_lexicon_tags_37 },
    { 0x7fbd9110, SUC_lexicon_tags_37 },
    { 0x75d9111, SUC_lexicon_tags_7 },
    { 0x2e1d9111, SUC_lexicon_tags_14 },
    { 0x911d9113, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbd9116, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd99d9118, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x709d911a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x35bd911c, SUC_lexicon_tags_25 },
    { 0x7bd911c, SUC_lexicon_tags_23 },
    { 0xa1d911e, SUC_lexicon_tags_33 },
    { 0x151d911f, SUC_lexicon_tags_10 },
    { 0x5dbd911e, SUC_lexicon_tags_15 },
    { 0x531d9121, SUC_lexicon_tags_10 },
    { 0x77bd9121, SUC_lexicon_tags_33 },
    { 0x3c7d9123, SUC_lexicon_tags_77 },
    { 0xb1bd911d, SUC_lexicon_tags_15 },
    { 0xfa7d911e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ffd9127, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dd912c, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1d9130, SUC_lexicon_tags_45 },
    { 0x3c7d9130, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x271d9133, SUC_lexicon_tags_198 },
    { 0x6bfd9134, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfd9138, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x953d913a, SUC_lexicon_tags_15 },
    { 0xd93d913b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3a3d913d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dd9140, SUC_lexicon_tags_27 },
    { 0x921d9141, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9f9d9143, SUC_lexicon_tags_25 },
    { 0x117d9143, SUC_lexicon_tags_9 },
    { 0x743d9144, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21d9148, SUC_lexicon_tags_87 },
    { 0x887d9149, SUC_lexicon_tags_37 },
    { 0xdcbd9149, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xabbd914c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x509d914e, SUC_lexicon_tags_30 },
    { 0xb51d914f, SUC_lexicon_tags_28 },
    { 0x725d914e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e7d9152, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5d9157, SUC_lexicon_tags_25 },
    { 0xe2bd9157, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x723d915a, SUC_lexicon_tags_37 },
    { 0xd05d915b, SUC_lexicon_tags_43 },
    { 0xe05d915c, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829d9163, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x39dd9165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a9d9167, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66bd9169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb11d916b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdd916e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbd9172, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2cbd9174, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dd9177, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3d917e, SUC_lexicon_tags_11 },
    { 0xdfbd917e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xefbd9181, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x681d9183, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1d9186, SUC_lexicon_tags_17 },
    { 0x541d9187, SUC_lexicon_tags_2 },
    { 0xe69d9187, SUC_lexicon_tags_37 },
    { 0x29fd9188, SUC_lexicon_tags_31 },
    { 0x94fd918a, SUC_lexicon_tags_75 },
    { 0x3cbd918a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943d9194, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0x7e7d9196, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3c3d9198, SUC_lexicon_tags_37 },
    { 0xd09d9199, SUC_lexicon_tags_11 },
    { 0xf7fd919a, SUC_lexicon_tags_57 },
    { 0x363d919b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe0fd919d, SUC_lexicon_tags_229 },
    { 0x4a3d919d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5d91a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5d91a5, SUC_lexicon_tags_14 },
    { 0xe4bd91a6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1d91aa, SUC_lexicon_tags_37 },
    { 0x6bd91ab, SUC_lexicon_tags_46 },
    { 0x8cbd91ac, SUC_lexicon_tags_6 },
    { 0xa47d91ad, SUC_lexicon_tags_37 },
    { 0x31d91ae, SUC_lexicon_tags_15 },
    { 0xfd9d91af, SUC_lexicon_tags_65 },
    { 0x397d91b0, SUC_lexicon_tags_28 },
    { 0xd65d91b1, SUC_lexicon_tags_14 },
    { 0x9ffd91aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fd91b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7d91bc, SUC_lexicon_tags_17 },
    { 0xcd3d91bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3d91c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc13d91c2, SUC_lexicon_tags_27 },
    { 0x971d91c3, SUC_lexicon_tags_65 },
    { 0x719d91c3, SUC_lexicon_tags_10 },
    { 0x48fd91c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd91c9, SUC_lexicon_tags_62 },
    { 0xf01d91ca, SUC_lexicon_tags_32 },
    { 0xb8fd91cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fd91ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x813d91d1, SUC_lexicon_tags_45 },
    { 0xc5d91d2, SUC_lexicon_tags_37 },
    { 0x9f5d91d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d91d7, SUC_lexicon_tags_17 },
    { 0xc31d91d7, SUC_lexicon_tags_107 },
    { 0xd4fd91d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x821d91db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701d91e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d91e7, SUC_lexicon_tags_34 },
    { 0xe03d91e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf3d91ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59d91f0, SUC_lexicon_tags_27 },
    { 0x409d91f1, SUC_lexicon_tags_11 },
    { 0x6cdd91f2, SUC_lexicon_tags_7 },
    { 0xf8bd91f3, SUC_lexicon_tags_25 },
    { 0x94bd91f3, SUC_lexicon_tags_49 },
    { 0x1e7d91f5, SUC_lexicon_tags_117 },
    { 0x55fd91f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x191d91f9, SUC_lexicon_tags_33 },
    { 0x9add91fa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dd91fd, SUC_lexicon_tags_1 },
    { 0x685d91fe, SUC_lexicon_tags_23 },
    { 0xc0bd91fd, SUC_lexicon_tags_1 },
    { 0xde9d9200, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbd920a, SUC_lexicon_tags_0 },
    { 0x505d920b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcedd920e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205d9212, SUC_lexicon_tags_8 },
    { 0xa4bd9212, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaf3d9215, SUC_lexicon_tags_301 },
    { 0xf49d9216, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9d921c, SUC_lexicon_tags_4 },
    { 0x9edd921d, SUC_lexicon_tags_2 },
    { 0xa75d921c, SUC_lexicon_tags_25 },
    { 0x753d921f, SUC_lexicon_tags_55 },
    { 0xe01d921f, SUC_lexicon_tags_29 },
    { 0xe83d921f, SUC_lexicon_tags_43 },
    { 0x823d9222, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x129d9224, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfd9227, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd19d9229, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3d9233, SUC_lexicon_tags_12 },
    { 0xc3dd9234, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493d923a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7d9241, SUC_lexicon_tags_29 },
    { 0x255d9242, SUC_lexicon_tags_63 },
    { 0x24dd9243, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d924e, SUC_lexicon_tags_0 },
    { 0x7d3d924e, SUC_lexicon_tags_11 },
    { 0x18dd924f, SUC_lexicon_tags_37 },
    { 0x3d7d924e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765d9255, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53d925a, SUC_lexicon_tags_7 },
    { 0x315d925b, SUC_lexicon_tags_7 },
    { 0xb93d925c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdd925f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523d9263, SUC_lexicon_tags_1 },
    { 0x7e1d9264, SUC_lexicon_tags_49 },
    { 0xbc3d9264, SUC_lexicon_tags_68 },
    { 0x829d9266, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cfd9268, SUC_lexicon_tags_17 },
    { 0xb53d9269, SUC_lexicon_tags_28 },
    { 0xfafd926a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1d9270, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385d9274, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x25d9277, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bdd9279, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddd9282, SUC_lexicon_tags_1 },
    { 0x209d9283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1d9288, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05d928c, SUC_lexicon_tags_11 },
    { 0x21dd928d, SUC_lexicon_tags_33 },
    { 0x7bbd928e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523d9295, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xca9d9297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x74bd9299, SUC_lexicon_tags_46 },
    { 0x1a5d9299, SUC_lexicon_tags_10 },
    { 0x80fd929b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553d92a3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1d92ae, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x913d92b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31dd92b2, SUC_lexicon_tags_2 },
    { 0xfdbd92b3, SUC_lexicon_tags_37 },
    { 0x74dd92b4, SUC_lexicon_tags_8 },
    { 0x34bd92b5, SUC_lexicon_tags_0 },
    { 0x15d92b5, SUC_lexicon_tags_55 },
    { 0x795d92b7, SUC_lexicon_tags_1 },
    { 0x94bd92b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x447d92bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4dd92bd, SUC_lexicon_tags_46 },
    { 0xed9d92be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9d92c2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdd92c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x881d92c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cdd92ca, SUC_lexicon_tags_28 },
    { 0xe9fd92ca, SUC_lexicon_tags_68 },
    { 0x35dd92cc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe4dd92ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dd92d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf3dd92d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x485d92d6, SUC_lexicon_tags_15 },
    { 0x437d92d7, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb47d92d9, SUC_lexicon_tags_61 },
    { 0x497d92da, SUC_lexicon_tags_12 },
    { 0xf8fd92db, SUC_lexicon_tags_14 },
    { 0x453d92da, SUC_lexicon_tags_2 },
    { 0xee9d92db, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x807d92df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf61d92e1, SUC_lexicon_tags_12 },
    { 0x457d92e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x161d92e4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x29dd92e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bd92ee, SUC_lexicon_tags_17 },
    { 0x17bd92ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa73d92f1, SUC_lexicon_tags_11 },
    { 0xaffd92f1, SUC_lexicon_tags_11 },
    { 0x56bd92f3, SUC_lexicon_tags_37 },
    { 0x317d92f4, SUC_lexicon_tags_30 },
    { 0xf61d92f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3bdd92f7, SUC_lexicon_tags_15 },
    { 0xc21d92f7, SUC_lexicon_tags_25 },
    { 0x80fd92f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x735d92fb, SUC_lexicon_tags_10 },
    { 0xf91d92fc, SUC_lexicon_tags_17 },
    { 0x337d92fd, SUC_lexicon_tags_297 },
    { 0, NULL },
    { 0xb57d92ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41d9302, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3d9305, SUC_lexicon_tags_1 },
    { 0x349d9306, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x81fd9308, SUC_lexicon_tags_15 },
    { 0xe3fd9308, SUC_lexicon_tags_2 },
    { 0x4f7d930a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffd931d, SUC_lexicon_tags_11 },
    { 0xa25d931d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513d9322, SUC_lexicon_tags_14 },
    { 0xe1bd9322, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf99d9326, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67d9329, SUC_lexicon_tags_12 },
    { 0xd61d932a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53d932f, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xe9fd9331, SUC_lexicon_tags_27 },
    { 0xe65d9331, SUC_lexicon_tags_8 },
    { 0x53bd9332, SUC_lexicon_tags_7 },
    { 0xadd9334, SUC_lexicon_tags_14 },
    { 0xf57d9335, SUC_lexicon_tags_7 },
    { 0x75dd9335, SUC_lexicon_tags_21 },
    { 0xe3d9334, SUC_lexicon_tags_17 },
    { 0xef1d9335, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x911d933b, SUC_lexicon_tags_1 },
    { 0x31dd933b, SUC_lexicon_tags_57 },
    { 0x49fd933d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3d9341, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51d9345, SUC_lexicon_tags_14 },
    { 0x589d9346, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155d934a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1bd934c, SUC_lexicon_tags_28 },
    { 0x5f5d934d, SUC_lexicon_tags_7 },
    { 0x38bd934e, SUC_lexicon_tags_37 },
    { 0xb0bd934e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7d9d9351, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabd9355, SUC_lexicon_tags_17 },
    { 0xeafd9356, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3d935b, SUC_lexicon_tags_19 },
    { 0x27dd935b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1d935f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55d9362, SUC_lexicon_tags_37 },
    { 0x1d7d9363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd1d9365, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7d936a, SUC_lexicon_tags_8 },
    { 0x337d936a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1a9d936d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x685d9370, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f5d9372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dd9379, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24bd9380, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67d9384, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bd9389, SUC_lexicon_tags_87 },
    { 0x993d938a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3d938e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae7d9391, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3d9395, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x533d9398, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bd939c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x4e1d939e, SUC_lexicon_tags_49 },
    { 0x69bd939f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc85d93a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345d93a6, SUC_lexicon_tags_11 },
    { 0x4e5d93a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699d93ac, SUC_lexicon_tags_1 },
    { 0x5ffd93ad, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dd93b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebd93bc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33d93c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x92fd93c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xdd93c6, SUC_lexicon_tags_48 },
    { 0xf6dd93c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa49d93c9, SUC_lexicon_tags_87 },
    { 0x963d93c9, SUC_lexicon_tags_43 },
    { 0x501d93cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1add93ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bd93d2, SUC_lexicon_tags_0 },
    { 0xabd93d3, SUC_lexicon_tags_88 },
    { 0xf7bd93d3, SUC_lexicon_tags_15 },
    { 0x92bd93d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bd93d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ffd93da, SUC_lexicon_tags_11 },
    { 0x135d93da, SUC_lexicon_tags_7 },
    { 0x16bd93db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bd93e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7dd93e2, SUC_lexicon_tags_38 },
    { 0xbbfd93e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x179d93e5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbd93ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbd93f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2a5d93f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6b3d93f6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x865d93f8, SUC_lexicon_tags_45 },
    { 0xac9d93f9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb33d93fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0fd93fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1d9401, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfd9404, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49d940b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9d9416, SUC_lexicon_tags_47 },
    { 0x2e3d9417, SUC_lexicon_tags_0 },
    { 0x69fd9418, SUC_lexicon_tags_23 },
    { 0xe49d9416, SUC_lexicon_tags_17 },
    { 0xabbd9416, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5d941d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ffd941f, SUC_lexicon_tags_7 },
    { 0xbd7d941f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbd9423, SUC_lexicon_tags_17 },
    { 0xbd1d9424, SUC_lexicon_tags_1 },
    { 0xebd9423, SUC_lexicon_tags_37 },
    { 0x783d9426, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69d9428, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691d942e, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x655d9431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2fd9433, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c5d9435, SUC_lexicon_tags_45 },
    { 0xc51d9436, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x52dd9438, SUC_lexicon_tags_11 },
    { 0x361d9439, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5d943f, SUC_lexicon_tags_39 },
    { 0x8b9d943f, SUC_lexicon_tags_63 },
    { 0xacdd9441, SUC_lexicon_tags_49 },
    { 0xf5bd9442, SUC_lexicon_tags_37 },
    { 0x6d1d9443, SUC_lexicon_tags_37 },
    { 0xf03d9444, SUC_lexicon_tags_11 },
    { 0xdd9442, SUC_lexicon_tags_2 },
    { 0x74bd9446, SUC_lexicon_tags_7 },
    { 0x475d9443, SUC_lexicon_tags_1 },
    { 0xf2dd9448, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65d944f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdafd9451, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801d9457, SUC_lexicon_tags_43 },
    { 0x7e3d9457, SUC_lexicon_tags_0 },
    { 0x35dd9457, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5d945c, SUC_lexicon_tags_2 },
    { 0xacbd945c, SUC_lexicon_tags_17 },
    { 0xc9d945e, SUC_lexicon_tags_3 },
    { 0x391d945d, SUC_lexicon_tags_17 },
    { 0xcf9d9460, SUC_lexicon_tags_31 },
    { 0x411d945e, SUC_lexicon_tags_2 },
    { 0x11dd9460, SUC_lexicon_tags_19 },
    { 0x3d7d945d, SUC_lexicon_tags_21 },
    { 0xdcbd9464, SUC_lexicon_tags_58 },
    { 0x4f1d9465, SUC_lexicon_tags_30 },
    { 0x52bd9466, SUC_lexicon_tags_33 },
    { 0x2c9d9466, SUC_lexicon_tags_17 },
    { 0x403d9468, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65d946b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad7d946d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fd9472, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3d9478, SUC_lexicon_tags_19 },
    { 0xfc9d9479, SUC_lexicon_tags_2 },
    { 0xd99d947a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9d947f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd61d9481, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fd9484, SUC_lexicon_tags_27 },
    { 0xa51d9485, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1d9488, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493d9494, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbd94a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9d94a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3d94aa, SUC_lexicon_tags_12 },
    { 0xb19d94ab, SUC_lexicon_tags_30 },
    { 0x8e7d94ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb15d94ae, SUC_lexicon_tags_11 },
    { 0xdf1d94af, SUC_lexicon_tags_197 },
    { 0x4fbd94b0, SUC_lexicon_tags_10 },
    { 0x947d94b0, SUC_lexicon_tags_2 },
    { 0x813d94b1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x719d94b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x103d94b7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5d94bb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x827d94bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x367d94c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x149d94c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9c9d94c5, SUC_lexicon_tags_10 },
    { 0x475d94c6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2d7d94c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x38dd94ca, SUC_lexicon_tags_1 },
    { 0x753d94cb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff5d94cd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x99d94cf, SUC_lexicon_tags_49 },
    { 0xd47d94d0, SUC_lexicon_tags_0 },
    { 0x5b9d94d1, SUC_lexicon_tags_7 },
    { 0x8efd94d0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe11d94d4, SUC_lexicon_tags_46 },
    { 0xd97d94d5, SUC_lexicon_tags_37 },
    { 0x14bd94d6, SUC_lexicon_tags_9 },
    { 0x8e1d94d7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfd94da, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc5d94dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd3d94de, SUC_lexicon_tags_25 },
    { 0xc39d94de, SUC_lexicon_tags_63 },
    { 0x451d94de, SUC_lexicon_tags_8 },
    { 0x207d94e1, SUC_lexicon_tags_17 },
    { 0xb9fd94df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdd94e7, SUC_lexicon_tags_11 },
    { 0xd0fd94e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1d94eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7d94ee, SUC_lexicon_tags_547 },
    { 0x82bd94ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61d94f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5d94f8, SUC_lexicon_tags_63 },
    { 0xf87d94f9, SUC_lexicon_tags_15 },
    { 0x83d94fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dd94fe, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe4bd9500, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dd9508, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x201d950a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dd950d, SUC_lexicon_tags_61 },
    { 0x77bd950d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dd9512, SUC_lexicon_tags_33 },
    { 0x391d9513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e1d9515, SUC_lexicon_tags_140 },
    { 0x2c7d9516, SUC_lexicon_tags_9 },
    { 0x661d9515, SUC_lexicon_tags_17 },
    { 0x775d9517, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81d951c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bd9520, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29d9525, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abd952b, SUC_lexicon_tags_25 },
    { 0xd49d952b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbd9535, SUC_lexicon_tags_93 },
    { 0xb9d9535, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x35dd9539, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x32fd953b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95d953e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951d9548, SUC_lexicon_tags_160 },
    { 0x491d9549, SUC_lexicon_tags_56 },
    { 0xaa7d954a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x56dd954c, SUC_lexicon_tags_12 },
    { 0xfe5d954d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8a9d954f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1d9552, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2c5d9554, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3d955a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5d955d, SUC_lexicon_tags_47 },
    { 0x7f7d955e, SUC_lexicon_tags_19 },
    { 0x1f5d955f, SUC_lexicon_tags_21 },
    { 0x881d955f, SUC_lexicon_tags_25 },
    { 0x359d955e, SUC_lexicon_tags_18 },
    { 0xfd7d9562, SUC_lexicon_tags_11 },
    { 0xedd9562, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9d9566, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dd9569, SUC_lexicon_tags_1 },
    { 0x97fd956a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x715d956c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55d9570, SUC_lexicon_tags_27 },
    { 0x889d9570, SUC_lexicon_tags_29 },
    { 0x2e3d9572, SUC_lexicon_tags_11 },
    { 0x79fd9572, SUC_lexicon_tags_60 },
    { 0x4ffd9574, SUC_lexicon_tags_6 },
    { 0x4add9575, SUC_lexicon_tags_2 },
    { 0x1b3d9576, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dd9579, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803d957e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19d9586, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17d958d, SUC_lexicon_tags_15 },
    { 0xa9fd958e, SUC_lexicon_tags_2 },
    { 0x1d3d958e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2cfd9591, SUC_lexicon_tags_62 },
    { 0xe3fd9592, SUC_lexicon_tags_8 },
    { 0xb59d9593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fd9597, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181d959e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdf3d95a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893d95a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x85bd95a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7d95ac, SUC_lexicon_tags_12 },
    { 0x6e3d95ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x459d95b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15d95b3, SUC_lexicon_tags_7 },
    { 0xa33d95b3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5d95b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5d95bc, SUC_lexicon_tags_12 },
    { 0x9d5d95bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53d95c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x751d95c3, SUC_lexicon_tags_30 },
    { 0xfd7d95c3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xef5d95c6, SUC_lexicon_tags_2 },
    { 0xde3d95c7, SUC_lexicon_tags_30 },
    { 0xbafd95c8, SUC_lexicon_tags_12 },
    { 0xb7fd95c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x523d95cc, SUC_lexicon_tags_63 },
    { 0xd69d95cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3bbd95cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x75dd95d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd37d95d5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803d95df, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edd95e5, SUC_lexicon_tags_25 },
    { 0xd3dd95e5, SUC_lexicon_tags_31 },
    { 0xdf7d95e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fd95ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9d95f0, SUC_lexicon_tags_0 },
    { 0xa65d95f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x731d95f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21d95f7, SUC_lexicon_tags_23 },
    { 0x1add95f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd95ff, SUC_lexicon_tags_7 },
    { 0x731d9600, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x871d9602, SUC_lexicon_tags_19 },
    { 0x2dfd9602, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fd9608, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227d960d, SUC_lexicon_tags_23 },
    { 0x347d960d, SUC_lexicon_tags_11 },
    { 0xd91d960f, SUC_lexicon_tags_6 },
    { 0x695d9610, SUC_lexicon_tags_14 },
    { 0x181d960f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa69d9613, SUC_lexicon_tags_71 },
    { 0x4d9d9613, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f7d9616, SUC_lexicon_tags_7 },
    { 0x741d9616, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31d961c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x267d961e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341d9622, SUC_lexicon_tags_56 },
    { 0x961d9623, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fd9626, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3d9631, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b1d9633, SUC_lexicon_tags_11 },
    { 0xccbd9634, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21d963f, SUC_lexicon_tags_72 },
    { 0xaebd9640, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5bd9642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x383d9644, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8b7d9646, SUC_lexicon_tags_41 },
    { 0xcb1d9647, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x135d9649, SUC_lexicon_tags_9 },
    { 0xf53d9649, SUC_lexicon_tags_60 },
    { 0xb35d964b, SUC_lexicon_tags_35 },
    { 0xb27d964c, SUC_lexicon_tags_6 },
    { 0xf17d964d, SUC_lexicon_tags_34 },
    { 0x131d964e, SUC_lexicon_tags_8 },
    { 0x9fbd964e, SUC_lexicon_tags_19 },
    { 0x2c3d9649, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbd9657, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5d965c, SUC_lexicon_tags_11 },
    { 0xfdfd965d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1d9662, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdedd9667, SUC_lexicon_tags_19 },
    { 0x63d9668, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501d966e, SUC_lexicon_tags_37 },
    { 0x7ebd966e, SUC_lexicon_tags_104 },
    { 0x347d966e, SUC_lexicon_tags_23 },
    { 0x24bd9671, SUC_lexicon_tags_45 },
    { 0xdbdd9672, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70dd9676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x741d9679, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedd9680, SUC_lexicon_tags_56 },
    { 0xe7dd9681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x551d9684, SUC_lexicon_tags_0 },
    { 0x8a7d9684, SUC_lexicon_tags_38 },
    { 0xcc3d9686, SUC_lexicon_tags_60 },
    { 0x511d9686, SUC_lexicon_tags_37 },
    { 0xfe5d9685, SUC_lexicon_tags_27 },
    { 0x131d9689, SUC_lexicon_tags_101 },
    { 0x3b3d968a, SUC_lexicon_tags_11 },
    { 0x623d968b, SUC_lexicon_tags_26 },
    { 0x1d5d9687, SUC_lexicon_tags_0 },
    { 0xb6dd9688, SUC_lexicon_tags_2 },
    { 0x971d968a, SUC_lexicon_tags_12 },
    { 0x6ebd9685, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfd9693, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89d9697, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbd969a, SUC_lexicon_tags_14 },
    { 0x4f7d969a, SUC_lexicon_tags_4 },
    { 0x9f1d969c, SUC_lexicon_tags_2 },
    { 0x98fd969d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dd96a2, SUC_lexicon_tags_17 },
    { 0x555d96a3, SUC_lexicon_tags_36 },
    { 0x7abd96a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x235d96a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381d96ab, SUC_lexicon_tags_21 },
    { 0x199d96ac, SUC_lexicon_tags_35 },
    { 0x627d96ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x789d96b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497d96b4, SUC_lexicon_tags_17 },
    { 0xa7d96b4, SUC_lexicon_tags_11 },
    { 0xd2fd96b5, SUC_lexicon_tags_82 },
    { 0xda3d96b7, SUC_lexicon_tags_43 },
    { 0xe1dd96b7, SUC_lexicon_tags_7 },
    { 0x90dd96b8, SUC_lexicon_tags_33 },
    { 0xee3d96ba, SUC_lexicon_tags_27 },
    { 0xdf5d96bb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fd96c2, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07d96c5, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3d96ca, SUC_lexicon_tags_33 },
    { 0x3e7d96cb, SUC_lexicon_tags_1 },
    { 0x969d96cb, SUC_lexicon_tags_1 },
    { 0xfffd96cc, SUC_lexicon_tags_48 },
    { 0x76bd96cc, SUC_lexicon_tags_25 },
    { 0x43fd96ca, SUC_lexicon_tags_14 },
    { 0x3f9d96d0, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d96d6, SUC_lexicon_tags_17 },
    { 0xdc3d96d7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1d96da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x31bd96dc, SUC_lexicon_tags_37 },
    { 0x111d96dc, SUC_lexicon_tags_17 },
    { 0x713d96dc, SUC_lexicon_tags_6 },
    { 0xc5fd96df, SUC_lexicon_tags_30 },
    { 0xdbfd96e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x719d96e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbd96e5, SUC_lexicon_tags_11 },
    { 0x537d96e6, SUC_lexicon_tags_1 },
    { 0xfcdd96e7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23d96ea, SUC_lexicon_tags_29 },
    { 0x161d96eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47d96ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe23d96f1, SUC_lexicon_tags_26 },
    { 0x5a1d96f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bd96f8, SUC_lexicon_tags_10 },
    { 0x3a1d96f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bd96fc, SUC_lexicon_tags_27 },
    { 0x95fd96fd, SUC_lexicon_tags_37 },
    { 0x711d96fe, SUC_lexicon_tags_19 },
    { 0x1f1d96fc, SUC_lexicon_tags_45 },
    { 0x9a9d9700, SUC_lexicon_tags_85 },
    { 0x565d9701, SUC_lexicon_tags_9 },
    { 0xd3fd9702, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9ebd9704, SUC_lexicon_tags_26 },
    { 0x71dd9705, SUC_lexicon_tags_60 },
    { 0x65fd9705, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefd9709, SUC_lexicon_tags_12 },
    { 0xb23d970a, SUC_lexicon_tags_30 },
    { 0xc27d970b, SUC_lexicon_tags_30 },
    { 0x365d970c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661d9712, SUC_lexicon_tags_37 },
    { 0x7d3d9712, SUC_lexicon_tags_47 },
    { 0x6ebd9712, SUC_lexicon_tags_38 },
    { 0x405d9714, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dd971c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x647d971e, SUC_lexicon_tags_25 },
    { 0x433d971e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x585d9721, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2bfd9723, SUC_lexicon_tags_87 },
    { 0x585d9724, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33d9727, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba3d9729, SUC_lexicon_tags_17 },
    { 0x7a3d9729, SUC_lexicon_tags_17 },
    { 0x785d972a, SUC_lexicon_tags_9 },
    { 0x413d972c, SUC_lexicon_tags_55 },
    { 0x4b1d972d, SUC_lexicon_tags_60 },
    { 0x685d972e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5dd9730, SUC_lexicon_tags_12 },
    { 0x131d9731, SUC_lexicon_tags_37 },
    { 0x1bfd9731, SUC_lexicon_tags_37 },
    { 0xbbd9732, SUC_lexicon_tags_39 },
    { 0x18dd9734, SUC_lexicon_tags_55 },
    { 0x51d9735, SUC_lexicon_tags_9 },
    { 0xafdd9735, SUC_lexicon_tags_7 },
    { 0xc7dd9733, SUC_lexicon_tags_26 },
    { 0x5dfd9738, SUC_lexicon_tags_29 },
    { 0xe97d9739, SUC_lexicon_tags_45 },
    { 0x2e7d9730, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361d9740, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fd9744, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfd9746, SUC_lexicon_tags_25 },
    { 0xce9d9747, SUC_lexicon_tags_0 },
    { 0x541d9748, SUC_lexicon_tags_7 },
    { 0x5b1d9748, SUC_lexicon_tags_33 },
    { 0xc71d9748, SUC_lexicon_tags_61 },
    { 0x175d974b, SUC_lexicon_tags_14 },
    { 0x9f5d974c, SUC_lexicon_tags_49 },
    { 0x2f7d9748, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xfedd974f, SUC_lexicon_tags_7 },
    { 0x86bd974f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcffd9752, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6abd9754, SUC_lexicon_tags_1 },
    { 0x31fd9755, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd975a, SUC_lexicon_tags_47 },
    { 0x8b7d975b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebd975f, SUC_lexicon_tags_25 },
    { 0xfcbd975f, SUC_lexicon_tags_1 },
    { 0x83d9761, SUC_lexicon_tags_11 },
    { 0xa85d9762, SUC_lexicon_tags_63 },
    { 0x4d5d9763, SUC_lexicon_tags_3 },
    { 0xf71d9764, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe1d9766, SUC_lexicon_tags_13 },
    { 0x7d9767, SUC_lexicon_tags_19 },
    { 0x6bd9768, SUC_lexicon_tags_11 },
    { 0x5c3d9768, SUC_lexicon_tags_56 },
    { 0x4f3d9769, SUC_lexicon_tags_25 },
    { 0xfdfd976b, SUC_lexicon_tags_30 },
    { 0xb31d976a, SUC_lexicon_tags_25 },
    { 0xaadd976d, SUC_lexicon_tags_29 },
    { 0x78dd9769, SUC_lexicon_tags_28 },
    { 0xddbd976f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x299d9772, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145d9777, SUC_lexicon_tags_63 },
    { 0xdb7d9778, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683d977d, SUC_lexicon_tags_0 },
    { 0x355d977d, SUC_lexicon_tags_37 },
    { 0x71d977f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753d9784, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3b1d9786, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x289d9789, SUC_lexicon_tags_38 },
    { 0x3e3d978a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdcbd978c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3d9790, SUC_lexicon_tags_28 },
    { 0xb3fd9791, SUC_lexicon_tags_32 },
    { 0xbebd9791, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47d9795, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe6fd9797, SUC_lexicon_tags_8 },
    { 0x40fd9798, SUC_lexicon_tags_45 },
    { 0xd17d9798, SUC_lexicon_tags_2 },
    { 0xd67d979a, SUC_lexicon_tags_49 },
    { 0x421d979a, SUC_lexicon_tags_61 },
    { 0x563d979c, SUC_lexicon_tags_0 },
    { 0xe99d979a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbd979f, SUC_lexicon_tags_27 },
    { 0x70dd97a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc09d97a2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x747d97a4, SUC_lexicon_tags_37 },
    { 0x497d97a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedd97ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb77d97af, SUC_lexicon_tags_10 },
    { 0xf8bd97b0, SUC_lexicon_tags_25 },
    { 0xb33d97b1, SUC_lexicon_tags_0 },
    { 0xee7d97b2, SUC_lexicon_tags_66 },
    { 0x3a1d97b0, SUC_lexicon_tags_25 },
    { 0x76dd97b4, SUC_lexicon_tags_2 },
    { 0x499d97b5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87d97ba, SUC_lexicon_tags_62 },
    { 0x32fd97bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5d97c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3d97c7, SUC_lexicon_tags_39 },
    { 0xc9fd97c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddd97cc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5d97d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979d97dc, SUC_lexicon_tags_38 },
    { 0x845d97dd, SUC_lexicon_tags_23 },
    { 0x15dd97de, SUC_lexicon_tags_37 },
    { 0x887d97df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bd97e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbd97e7, SUC_lexicon_tags_25 },
    { 0x261d97e8, SUC_lexicon_tags_32 },
    { 0xd1bd97e9, SUC_lexicon_tags_0 },
    { 0x229d97e9, SUC_lexicon_tags_25 },
    { 0x2c5d97eb, SUC_lexicon_tags_33 },
    { 0x139d97e8, SUC_lexicon_tags_12 },
    { 0x7fdd97ed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xcc7d97ef, SUC_lexicon_tags_7 },
    { 0x8f9d97f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fd97f5, SUC_lexicon_tags_2 },
    { 0x365d97f6, SUC_lexicon_tags_11 },
    { 0x815d97f5, SUC_lexicon_tags_158 },
    { 0x47bd97f8, SUC_lexicon_tags_43 },
    { 0x23dd97f8, SUC_lexicon_tags_29 },
    { 0xaf9d97fa, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x335d97fc, SUC_lexicon_tags_1 },
    { 0x15d97fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d7d97ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x929d9801, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xee1d9803, SUC_lexicon_tags_7 },
    { 0x43bd9804, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba5d9806, SUC_lexicon_tags_37 },
    { 0x747d9807, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9d980a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fd9810, SUC_lexicon_tags_26 },
    { 0x70bd9810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7d9814, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4c5d9816, SUC_lexicon_tags_18 },
    { 0xd6fd9817, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcfd9819, SUC_lexicon_tags_15 },
    { 0x5ddd9819, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66dd9821, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fd9824, SUC_lexicon_tags_17 },
    { 0xe8dd9825, SUC_lexicon_tags_2 },
    { 0xb21d9825, SUC_lexicon_tags_6 },
    { 0x9e1d9827, SUC_lexicon_tags_16 },
    { 0xec1d9827, SUC_lexicon_tags_46 },
    { 0xd37d9828, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x62fd982b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455d982f, SUC_lexicon_tags_17 },
    { 0x5c5d9830, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdd9837, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43d983a, SUC_lexicon_tags_37 },
    { 0xe8bd983b, SUC_lexicon_tags_48 },
    { 0xd3bd983c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbefd983e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51d9842, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdd9847, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fd984a, SUC_lexicon_tags_85 },
    { 0x5fbd984b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa0fd984d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3d9850, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999d9855, SUC_lexicon_tags_30 },
    { 0xd77d9855, SUC_lexicon_tags_25 },
    { 0x53d9857, SUC_lexicon_tags_33 },
    { 0xe25d9856, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe0dd985a, SUC_lexicon_tags_63 },
    { 0x8afd985a, SUC_lexicon_tags_12 },
    { 0xc57d985c, SUC_lexicon_tags_31 },
    { 0xe8dd985d, SUC_lexicon_tags_2 },
    { 0x933d985e, SUC_lexicon_tags_79 },
    { 0xe7bd985d, SUC_lexicon_tags_11 },
    { 0xc2fd9860, SUC_lexicon_tags_17 },
    { 0xe5d985b, SUC_lexicon_tags_17 },
    { 0x4dfd9862, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177d9868, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81d9870, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15d9875, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dd9879, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x535d987b, SUC_lexicon_tags_15 },
    { 0xdebd987c, SUC_lexicon_tags_11 },
    { 0x249d987b, SUC_lexicon_tags_29 },
    { 0x3cdd987b, SUC_lexicon_tags_0 },
    { 0xdc7d987f, SUC_lexicon_tags_11 },
    { 0x291d987f, SUC_lexicon_tags_25 },
    { 0x823d9880, SUC_lexicon_tags_17 },
    { 0xb09d9882, SUC_lexicon_tags_37 },
    { 0xf63d9881, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe7d9885, SUC_lexicon_tags_37 },
    { 0x41dd9886, SUC_lexicon_tags_37 },
    { 0x46fd9887, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x97fd988a, SUC_lexicon_tags_0 },
    { 0xe9dd988a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57d988d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7d9891, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffd9894, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1d9898, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01d98a1, SUC_lexicon_tags_19 },
    { 0xc0fd98a2, SUC_lexicon_tags_49 },
    { 0xf5d98a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9d98a6, SUC_lexicon_tags_38 },
    { 0x9bfd98a7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xad5d98a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebd98ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1d98af, SUC_lexicon_tags_25 },
    { 0xe39d98b0, SUC_lexicon_tags_17 },
    { 0xa4dd98b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbc1d98b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x609d98b6, SUC_lexicon_tags_38 },
    { 0xebfd98b7, SUC_lexicon_tags_37 },
    { 0x99fd98b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x56bd98bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fd98bf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bd98c4, SUC_lexicon_tags_24 },
    { 0xb2fd98c4, SUC_lexicon_tags_0 },
    { 0x927d98c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dd98ca, SUC_lexicon_tags_15 },
    { 0xe07d98cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29d98d2, SUC_lexicon_tags_25 },
    { 0x6fbd98d2, SUC_lexicon_tags_37 },
    { 0xa69d98d4, SUC_lexicon_tags_2 },
    { 0x52dd98d2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe33d98d8, SUC_lexicon_tags_27 },
    { 0xa5d98d9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5a3d98db, SUC_lexicon_tags_25 },
    { 0x51dd98dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddd98e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7d98e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dd98e9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4e3d98eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcbfd98ed, SUC_lexicon_tags_37 },
    { 0xf1d98ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245d98fc, SUC_lexicon_tags_14 },
    { 0x4efd98fd, SUC_lexicon_tags_4 },
    { 0xb1dd98fe, SUC_lexicon_tags_47 },
    { 0x85dd98ff, SUC_lexicon_tags_2 },
    { 0x131d98ff, SUC_lexicon_tags_38 },
    { 0x5e9d9900, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4dd9904, SUC_lexicon_tags_2 },
    { 0x2c1d9904, SUC_lexicon_tags_25 },
    { 0xadfd9906, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc8fd9908, SUC_lexicon_tags_27 },
    { 0xffdd9908, SUC_lexicon_tags_7 },
    { 0xc37d990a, SUC_lexicon_tags_11 },
    { 0xc1dd9908, SUC_lexicon_tags_1674 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bd990f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x437d9912, SUC_lexicon_tags_13 },
    { 0x703d9913, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9d9916, SUC_lexicon_tags_14 },
    { 0xfedd9917, SUC_lexicon_tags_29 },
    { 0x645d9916, SUC_lexicon_tags_45 },
    { 0xdbfd9919, SUC_lexicon_tags_65 },
    { 0xd07d9919, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdd991d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6add9929, SUC_lexicon_tags_63 },
    { 0xa7bd9929, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x539d992c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921d9933, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x405d9936, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dd9939, SUC_lexicon_tags_77 },
    { 0xdbbd993a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7d993d, SUC_lexicon_tags_27 },
    { 0x5dd993e, SUC_lexicon_tags_2 },
    { 0xf2bd993f, SUC_lexicon_tags_15 },
    { 0xa37d993f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5d9948, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dd994c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9dd994f, SUC_lexicon_tags_55 },
    { 0xa45d9950, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x653d9953, SUC_lexicon_tags_15 },
    { 0x93d9954, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fd9957, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fd995a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x271d995c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe1bd995e, SUC_lexicon_tags_68 },
    { 0xc5bd995f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x157d9962, SUC_lexicon_tags_1 },
    { 0x6fd9963, SUC_lexicon_tags_47 },
    { 0xfffd9964, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe13d9966, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9d9969, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15d9970, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x283d9972, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbd9979, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x133d997c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x909d997e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x955d9980, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd9985, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdd9988, SUC_lexicon_tags_25 },
    { 0xf31d9988, SUC_lexicon_tags_37 },
    { 0xefdd9989, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5dd998c, SUC_lexicon_tags_87 },
    { 0xc19d998d, SUC_lexicon_tags_31 },
    { 0xcd1d998d, SUC_lexicon_tags_45 },
    { 0xc3d998f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x175d9992, SUC_lexicon_tags_7 },
    { 0xd7dd9993, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb1d9995, SUC_lexicon_tags_3 },
    { 0x629d9995, SUC_lexicon_tags_43 },
    { 0x639d9997, SUC_lexicon_tags_43 },
    { 0x297d9998, SUC_lexicon_tags_12 },
    { 0x5e3d9998, SUC_lexicon_tags_29 },
    { 0x2bbd9995, SUC_lexicon_tags_37 },
    { 0xb9fd999b, SUC_lexicon_tags_26 },
    { 0x20dd999c, SUC_lexicon_tags_15 },
    { 0x267d999c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23d99a0, SUC_lexicon_tags_2 },
    { 0x13dd99a0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685d99a7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563d99ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacdd99ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb35d99af, SUC_lexicon_tags_64 },
    { 0xff5d99b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffd99b3, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xca5d99b6, SUC_lexicon_tags_63 },
    { 0x67fd99b7, SUC_lexicon_tags_2 },
    { 0x849d99b8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fd99bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3d99c0, SUC_lexicon_tags_1 },
    { 0x171d99c1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1d99c5, SUC_lexicon_tags_3 },
    { 0x2c3d99c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedd99ce, SUC_lexicon_tags_31 },
    { 0xa57d99ce, SUC_lexicon_tags_12 },
    { 0x3dbd99d0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x383d99d2, SUC_lexicon_tags_17 },
    { 0xcbdd99d3, SUC_lexicon_tags_11 },
    { 0xb51d99d4, SUC_lexicon_tags_37 },
    { 0x751d99d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905d99d9, SUC_lexicon_tags_15 },
    { 0x30bd99da, SUC_lexicon_tags_46 },
    { 0x5e9d99db, SUC_lexicon_tags_12 },
    { 0x3b9d99dc, SUC_lexicon_tags_6 },
    { 0xc4fd99db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda3d99df, SUC_lexicon_tags_6 },
    { 0x42bd99e0, SUC_lexicon_tags_79 },
    { 0x251d99e1, SUC_lexicon_tags_7 },
    { 0x773d99e2, SUC_lexicon_tags_4 },
    { 0xa87d99e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x931d99e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fd99ea, SUC_lexicon_tags_25 },
    { 0xcb1d99eb, SUC_lexicon_tags_27 },
    { 0xcb5d99ec, SUC_lexicon_tags_15 },
    { 0xfb5d99ec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57d99f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x737d99f2, SUC_lexicon_tags_11 },
    { 0xc8dd99f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1cbd99f5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefd99f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23d99fc, SUC_lexicon_tags_47 },
    { 0x325d99fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1d9a01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9d9a04, SUC_lexicon_tags_161 },
    { 0xc3fd9a05, SUC_lexicon_tags_2 },
    { 0x9dbd9a06, SUC_lexicon_tags_151 },
    { 0xb2bd9a06, SUC_lexicon_tags_11 },
    { 0xaf7d9a04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x88fd9a0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x789d9a0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3d9a11, SUC_lexicon_tags_9 },
    { 0xc2bd9a12, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x23bd9a14, SUC_lexicon_tags_63 },
    { 0x791d9a14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7d9a1c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6f7d9a1e, SUC_lexicon_tags_2 },
    { 0x863d9a1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdd9a24, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5d9a28, SUC_lexicon_tags_30 },
    { 0x65bd9a28, SUC_lexicon_tags_7 },
    { 0x5c9d9a2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3d9a2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb9d9a30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb8bd9a32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf3bd9a34, SUC_lexicon_tags_2 },
    { 0xd8fd9a34, SUC_lexicon_tags_2 },
    { 0x45bd9a35, SUC_lexicon_tags_45 },
    { 0x5fbd9a36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xef7d9a39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xcc1d9a3b, SUC_lexicon_tags_37 },
    { 0x6fdd9a3c, SUC_lexicon_tags_47 },
    { 0x883d9a3d, SUC_lexicon_tags_0 },
    { 0x251d9a3b, SUC_lexicon_tags_11 },
    { 0xcdfd9a3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xab1d9a42, SUC_lexicon_tags_19 },
    { 0x439d9a43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75d9a47, SUC_lexicon_tags_15 },
    { 0x1bd9a48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf55d9a4a, SUC_lexicon_tags_6 },
    { 0xebd9a4a, SUC_lexicon_tags_60 },
    { 0xcfbd9a4a, SUC_lexicon_tags_12 },
    { 0x731d9a4d, SUC_lexicon_tags_7 },
    { 0xab1d9a4e, SUC_lexicon_tags_2 },
    { 0xc69d9a4f, SUC_lexicon_tags_16 },
    { 0x165d9a4f, SUC_lexicon_tags_15 },
    { 0x317d9a50, SUC_lexicon_tags_57 },
    { 0x1bd9a52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x50fd9a54, SUC_lexicon_tags_14 },
    { 0x309d9a55, SUC_lexicon_tags_2 },
    { 0x9abd9a56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf61d9a59, SUC_lexicon_tags_49 },
    { 0xea3d9a5a, SUC_lexicon_tags_15 },
    { 0xcdd9a5b, SUC_lexicon_tags_2 },
    { 0xcf5d9a5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7d9a5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x391d9a61, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x72bd9a63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9d9a67, SUC_lexicon_tags_98 },
    { 0xab5d9a68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97d9a6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3d9a71, SUC_lexicon_tags_17 },
    { 0x4b1d9a72, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135d9a77, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddd9a7b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x773d9a7e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dd9a81, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51d9a86, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x253d9a89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x969d9a8b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe93d9a8d, SUC_lexicon_tags_68 },
    { 0x57fd9a8e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4a5d9a90, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddd9a96, SUC_lexicon_tags_12 },
    { 0x84bd9a97, SUC_lexicon_tags_12 },
    { 0x69dd9a97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fd9a9b, SUC_lexicon_tags_63 },
    { 0xcffd9a9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x287d9a9e, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0x24fd9aa0, SUC_lexicon_tags_77 },
    { 0x6edd9aa1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xad3d9aa3, SUC_lexicon_tags_15 },
    { 0x185d9aa3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x545d9aa6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47fd9aa8, SUC_lexicon_tags_12 },
    { 0x46dd9aa8, SUC_lexicon_tags_49 },
    { 0xa1dd9aa9, SUC_lexicon_tags_32 },
    { 0x4ffd9aab, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373d9ab1, SUC_lexicon_tags_45 },
    { 0x7f9d9ab2, SUC_lexicon_tags_15 },
    { 0x689d9ab3, SUC_lexicon_tags_14 },
    { 0x9add9ab1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57d9ab7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d9d9ab9, SUC_lexicon_tags_12 },
    { 0x8a3d9aba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5f7d9abc, SUC_lexicon_tags_0 },
    { 0xfe7d9abd, SUC_lexicon_tags_14 },
    { 0x76bd9abe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47d9ac3, SUC_lexicon_tags_15 },
    { 0x33d9ac4, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65d9ac7, SUC_lexicon_tags_1 },
    { 0xdcdd9ac8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0fd9aca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc93d9acc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x257d9ace, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd59d9ad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb13d9ad2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229d9ad7, SUC_lexicon_tags_37 },
    { 0x331d9ad8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf1d9ada, SUC_lexicon_tags_2 },
    { 0xcfbd9adb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x959d9add, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x4cdd9adf, SUC_lexicon_tags_25 },
    { 0xe51d9adf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b1d9ae2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49d9aea, SUC_lexicon_tags_37 },
    { 0x177d9aeb, SUC_lexicon_tags_66 },
    { 0xbcdd9aec, SUC_lexicon_tags_11 },
    { 0xc39d9aeb, SUC_lexicon_tags_25 },
    { 0x135d9aee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdd9af3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xea1d9af5, SUC_lexicon_tags_45 },
    { 0x103d9af6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x48fd9af9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1d9afe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffd9b09, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19d9b0d, SUC_lexicon_tags_31 },
    { 0xc3fd9b0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bd9b13, SUC_lexicon_tags_0 },
    { 0x283d9b14, SUC_lexicon_tags_25 },
    { 0xa07d9b15, SUC_lexicon_tags_31 },
    { 0xaedd9b13, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x197d9b18, SUC_lexicon_tags_84 },
    { 0x8a3d9b19, SUC_lexicon_tags_17 },
    { 0xa7bd9b1a, SUC_lexicon_tags_0 },
    { 0xddfd9b19, SUC_lexicon_tags_1016 },
    { 0x22fd9b1c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3d9b1f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xfd9d9b21, SUC_lexicon_tags_37 },
    { 0xc61d9b21, SUC_lexicon_tags_47 },
    { 0xabfd9b23, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9d9b26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fd9b2c, SUC_lexicon_tags_37 },
    { 0x9d9d9b2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dd9b35, SUC_lexicon_tags_60 },
    { 0xdd9d9b36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfd9b3c, SUC_lexicon_tags_21 },
    { 0x611d9b3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xba1d9b3f, SUC_lexicon_tags_7 },
    { 0x61dd9b40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fd9b45, SUC_lexicon_tags_12 },
    { 0xe25d9b46, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3d9b4c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x93d9b4e, SUC_lexicon_tags_63 },
    { 0xa97d9b4f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad9d9b51, SUC_lexicon_tags_37 },
    { 0xc4fd9b52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21d9b59, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61d9b68, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb3d9b6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1d9b6d, SUC_lexicon_tags_8 },
    { 0x7fdd9b6e, SUC_lexicon_tags_29 },
    { 0x9a9d9b6f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3d9b72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249d9b76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9d9b7a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2a1d9b7c, SUC_lexicon_tags_17 },
    { 0x525d9b7d, SUC_lexicon_tags_2 },
    { 0x7c7d9b7e, SUC_lexicon_tags_55 },
    { 0x3bfd9b7f, SUC_lexicon_tags_47 },
    { 0xb45d9b80, SUC_lexicon_tags_12 },
    { 0xaedd9b81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755d9b86, SUC_lexicon_tags_15 },
    { 0xca3d9b87, SUC_lexicon_tags_11 },
    { 0xa43d9b88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bd9b8d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3c3d9b8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadd9b92, SUC_lexicon_tags_15 },
    { 0x50bd9b93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb4bd9b95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849d9b9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47d9b9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x501d9b9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaefd9ba1, SUC_lexicon_tags_60 },
    { 0x3ebd9ba1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfd9ba9, SUC_lexicon_tags_37 },
    { 0x6efd9ba9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb81d9bac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7d9baf, SUC_lexicon_tags_17 },
    { 0x735d9bb0, SUC_lexicon_tags_1 },
    { 0x59bd9bb0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x307d9bb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x845d9bb7, SUC_lexicon_tags_1 },
    { 0x9dfd9bb7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8c3d9bba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd3d9bbc, SUC_lexicon_tags_7 },
    { 0x45fd9bbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fd9bc1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x58bd9bc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x737d9bc5, SUC_lexicon_tags_28 },
    { 0x79d9bc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55dd9bc8, SUC_lexicon_tags_44 },
    { 0x663d9bc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebd9bcc, SUC_lexicon_tags_11 },
    { 0xf2bd9bcd, SUC_lexicon_tags_11 },
    { 0x6f5d9bce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed9d9bd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1d9bd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bd9bd6, SUC_lexicon_tags_77 },
    { 0x8b1d9bd7, SUC_lexicon_tags_37 },
    { 0xcd1d9bd6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x171d9bdb, SUC_lexicon_tags_29 },
    { 0x709d9bdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd41d9bde, SUC_lexicon_tags_30 },
    { 0xb67d9bde, SUC_lexicon_tags_28 },
    { 0x31d9be0, SUC_lexicon_tags_23 },
    { 0xc9d9bdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa65d9be3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73d9be6, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0x579d9be8, SUC_lexicon_tags_11 },
    { 0xc81d9be9, SUC_lexicon_tags_4 },
    { 0x8dbd9bea, SUC_lexicon_tags_60 },
    { 0x61fd9beb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x721d9bee, SUC_lexicon_tags_39 },
    { 0xa49d9bef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf33d9bf1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e3d9bf3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x897d9bf6, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0x8d1d9bf8, SUC_lexicon_tags_25 },
    { 0xdfd9bf8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x80fd9bfb, SUC_lexicon_tags_63 },
    { 0xc9d9bfc, SUC_lexicon_tags_47 },
    { 0x4e5d9bfd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffd9c01, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9c06, SUC_lexicon_tags_23 },
    { 0xf6dd9c06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbf5d9c09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbd9c0f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67d9c14, SUC_lexicon_tags_14 },
    { 0x7cfd9c15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83fd9c17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efd9c1e, SUC_lexicon_tags_8 },
    { 0x1f5d9c1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759d9c27, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x85d9c29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afd9c30, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527d9c39, SUC_lexicon_tags_5 },
    { 0xecdd9c3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbd9c3d, SUC_lexicon_tags_37 },
    { 0x42fd9c3d, SUC_lexicon_tags_11 },
    { 0xd63d9c3f, SUC_lexicon_tags_12 },
    { 0x9e1d9c40, SUC_lexicon_tags_37 },
    { 0xaffd9c41, SUC_lexicon_tags_21 },
    { 0x7c7d9c3d, SUC_lexicon_tags_47 },
    { 0xafbd9c43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37dd9c45, SUC_lexicon_tags_17 },
    { 0xdb1d9c45, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bd9c49, SUC_lexicon_tags_1 },
    { 0x9c9d9c49, SUC_lexicon_tags_0 },
    { 0x6d5d9c4b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213d9c4f, SUC_lexicon_tags_47 },
    { 0xaa5d9c50, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3d9c53, SUC_lexicon_tags_8 },
    { 0xfc1d9c53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe63d9c56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x71fd9c58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bd9c5d, SUC_lexicon_tags_1 },
    { 0xcb7d9c5e, SUC_lexicon_tags_3 },
    { 0x967d9c5f, SUC_lexicon_tags_42 },
    { 0x49fd9c60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bd9c6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09d9c6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677d9c72, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05d9c76, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfd9c7b, SUC_lexicon_tags_12 },
    { 0x3f5d9c7b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x241d9c7f, SUC_lexicon_tags_2 },
    { 0x64fd9c7f, SUC_lexicon_tags_7 },
    { 0x817d9c81, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc53d9c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa29d9c85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05d9c88, SUC_lexicon_tags_15 },
    { 0x771d9c89, SUC_lexicon_tags_37 },
    { 0x519d9c8a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfd9c8d, SUC_lexicon_tags_35 },
    { 0x3a9d9c8e, SUC_lexicon_tags_37 },
    { 0x8bd9c8f, SUC_lexicon_tags_1 },
    { 0x9fd9c90, SUC_lexicon_tags_49 },
    { 0xb35d9c91, SUC_lexicon_tags_19 },
    { 0x5a5d9c91, SUC_lexicon_tags_37 },
    { 0x3ebd9c92, SUC_lexicon_tags_7 },
    { 0x90bd9c91, SUC_lexicon_tags_17 },
    { 0x7d3d9c91, SUC_lexicon_tags_25 },
    { 0x30bd9c90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67d9c99, SUC_lexicon_tags_1 },
    { 0x84fd9c9a, SUC_lexicon_tags_579 },
    { 0, NULL },
    { 0x6d9d9c9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615d9ca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fd9ca9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xff5d9cab, SUC_lexicon_tags_30 },
    { 0x2add9cab, SUC_lexicon_tags_15 },
    { 0x28dd9cad, SUC_lexicon_tags_77 },
    { 0x7cfd9cae, SUC_lexicon_tags_33 },
    { 0xa73d9caf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a9d9cb1, SUC_lexicon_tags_2 },
    { 0xc63d9cb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbd9cb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3d9cba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x561d9cbc, SUC_lexicon_tags_38 },
    { 0xe73d9cbd, SUC_lexicon_tags_17 },
    { 0x293d9cbd, SUC_lexicon_tags_9 },
    { 0x4cbd9cbf, SUC_lexicon_tags_47 },
    { 0x2c7d9cbf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x659d9cc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91d9cc9, SUC_lexicon_tags_2 },
    { 0x641d9cca, SUC_lexicon_tags_15 },
    { 0xb4fd9ccb, SUC_lexicon_tags_27 },
    { 0xc4bd9cc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5d9ccf, SUC_lexicon_tags_62 },
    { 0xea7d9ccf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1d9cd4, SUC_lexicon_tags_3 },
    { 0xb2dd9cd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817d9cde, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9d9ce3, SUC_lexicon_tags_15 },
    { 0x619d9ce3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x157d9ce6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5d9ce9, SUC_lexicon_tags_17 },
    { 0xaf1d9cea, SUC_lexicon_tags_37 },
    { 0x4bdd9ceb, SUC_lexicon_tags_60 },
    { 0x631d9ceb, SUC_lexicon_tags_7 },
    { 0x3afd9ceb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa3d9cef, SUC_lexicon_tags_41 },
    { 0xba3d9cf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfa7d9cf2, SUC_lexicon_tags_17 },
    { 0xa69d9cf3, SUC_lexicon_tags_15 },
    { 0xb79d9cf4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01d9cfc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfd9d00, SUC_lexicon_tags_47 },
    { 0x435d9d01, SUC_lexicon_tags_17 },
    { 0x343d9d01, SUC_lexicon_tags_17 },
    { 0x7e9d9d03, SUC_lexicon_tags_37 },
    { 0xefbd9d02, SUC_lexicon_tags_17 },
    { 0x31bd9d05, SUC_lexicon_tags_14 },
    { 0x15dd9d06, SUC_lexicon_tags_29 },
    { 0x60fd9d07, SUC_lexicon_tags_38 },
    { 0x763d9d08, SUC_lexicon_tags_53 },
    { 0x79dd9d08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bd9d0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7d9d11, SUC_lexicon_tags_9 },
    { 0x4b5d9d11, SUC_lexicon_tags_25 },
    { 0x9efd9d11, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x94dd9d15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dd9d1a, SUC_lexicon_tags_37 },
    { 0xd4fd9d1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb39d9d1e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf33d9d20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbd9d2b, SUC_lexicon_tags_9 },
    { 0x2dfd9d2c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dd9d31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99d9d34, SUC_lexicon_tags_4 },
    { 0x151d9d34, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85d9d38, SUC_lexicon_tags_25 },
    { 0xbfd9d39, SUC_lexicon_tags_37 },
    { 0x64dd9d38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x177d9d3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x673d9d3f, SUC_lexicon_tags_15 },
    { 0x95dd9d40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35dd9d43, SUC_lexicon_tags_11 },
    { 0xbe5d9d43, SUC_lexicon_tags_11 },
    { 0x7fdd9d44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7d9d48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bd9d4b, SUC_lexicon_tags_2 },
    { 0x51d9d4b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479d9d50, SUC_lexicon_tags_17 },
    { 0xb97d9d50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01d9d54, SUC_lexicon_tags_42 },
    { 0x807d9d54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63d9d5a, SUC_lexicon_tags_11 },
    { 0xcb3d9d5b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9d9d5e, SUC_lexicon_tags_12 },
    { 0xb25d9d5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd49d9d61, SUC_lexicon_tags_43 },
    { 0xf9fd9d62, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x477d9d64, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfd9d67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdd9d6f, SUC_lexicon_tags_12 },
    { 0xf27d9d70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x397d9d73, SUC_lexicon_tags_17 },
    { 0x5d1d9d74, SUC_lexicon_tags_25 },
    { 0x659d9d74, SUC_lexicon_tags_45 },
    { 0x4e3d9d75, SUC_lexicon_tags_21 },
    { 0x75bd9d75, SUC_lexicon_tags_30 },
    { 0x65bd9d75, SUC_lexicon_tags_37 },
    { 0xfb7d9d74, SUC_lexicon_tags_15 },
    { 0xf15d9d74, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdd9d80, SUC_lexicon_tags_11 },
    { 0xd2dd9d80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1d9d87, SUC_lexicon_tags_11 },
    { 0xd2dd9d87, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93d9d8f, SUC_lexicon_tags_31 },
    { 0x323d9d90, SUC_lexicon_tags_38 },
    { 0x57d9d91, SUC_lexicon_tags_2 },
    { 0xd53d9d90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55dd9d97, SUC_lexicon_tags_27 },
    { 0x873d9d97, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55d9d9c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fd9d9f, SUC_lexicon_tags_21 },
    { 0xf17d9da0, SUC_lexicon_tags_56 },
    { 0xb63d9da0, SUC_lexicon_tags_49 },
    { 0x9ffd9da1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99d9da5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765d9daa, SUC_lexicon_tags_23 },
    { 0xef5d9daa, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf5bd9dad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9d9db0, SUC_lexicon_tags_15 },
    { 0x3a1d9db0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7d9db6, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0x70bd9db8, SUC_lexicon_tags_7 },
    { 0x7d5d9db8, SUC_lexicon_tags_17 },
    { 0x86dd9dba, SUC_lexicon_tags_17 },
    { 0x571d9dbb, SUC_lexicon_tags_9 },
    { 0x89dd9dbc, SUC_lexicon_tags_47 },
    { 0x655d9db8, SUC_lexicon_tags_24 },
    { 0x611d9dbe, SUC_lexicon_tags_26 },
    { 0xe53d9dbf, SUC_lexicon_tags_11 },
    { 0x5bdd9dbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95d9dc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585d9dca, SUC_lexicon_tags_14 },
    { 0x315d9dcb, SUC_lexicon_tags_1 },
    { 0x49bd9dcb, SUC_lexicon_tags_2 },
    { 0x86dd9dcd, SUC_lexicon_tags_37 },
    { 0x5d5d9dce, SUC_lexicon_tags_23 },
    { 0x8cdd9dcf, SUC_lexicon_tags_17 },
    { 0xd2dd9dce, SUC_lexicon_tags_1596 },
    { 0, NULL },
    { 0xb75d9dd2, SUC_lexicon_tags_28 },
    { 0x327d9dd3, SUC_lexicon_tags_12 },
    { 0xbc1d9dd4, SUC_lexicon_tags_37 },
    { 0xcb5d9dd5, SUC_lexicon_tags_11 },
    { 0x699d9dd6, SUC_lexicon_tags_29 },
    { 0x557d9dd7, SUC_lexicon_tags_65 },
    { 0xa6bd9dd6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a1d9dda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6a5d9ddc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x969d9dde, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835d9de3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317d9de8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa53d9dea, SUC_lexicon_tags_1 },
    { 0xbadd9deb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7d9def, SUC_lexicon_tags_25 },
    { 0x7a3d9df0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69d9df3, SUC_lexicon_tags_9 },
    { 0x59d9df4, SUC_lexicon_tags_17 },
    { 0xfebd9df4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x39bd9df7, SUC_lexicon_tags_12 },
    { 0x2dbd9df8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dd9dfe, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77d9e02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fbd9e04, SUC_lexicon_tags_10 },
    { 0x2f5d9e05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9d9e08, SUC_lexicon_tags_11 },
    { 0x105d9e09, SUC_lexicon_tags_15 },
    { 0xd71d9e0a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bd9e0f, SUC_lexicon_tags_116 },
    { 0x595d9e10, SUC_lexicon_tags_11 },
    { 0xc5d9e11, SUC_lexicon_tags_39 },
    { 0xd85d9e12, SUC_lexicon_tags_48 },
    { 0xa2bd9e10, SUC_lexicon_tags_23 },
    { 0xefdd9e10, SUC_lexicon_tags_17 },
    { 0x3f1d9e15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x855d9e17, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdd9e1b, SUC_lexicon_tags_12 },
    { 0xf3bd9e1b, SUC_lexicon_tags_1 },
    { 0x797d9e1c, SUC_lexicon_tags_31 },
    { 0xd7fd9e1e, SUC_lexicon_tags_31 },
    { 0x2dfd9e1d, SUC_lexicon_tags_14 },
    { 0x4abd9e20, SUC_lexicon_tags_1 },
    { 0x2b9d9e1d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45d9e2b, SUC_lexicon_tags_27 },
    { 0xac1d9e2c, SUC_lexicon_tags_31 },
    { 0x5d9d9e2d, SUC_lexicon_tags_61 },
    { 0x649d9e2e, SUC_lexicon_tags_17 },
    { 0x519d9e2e, SUC_lexicon_tags_47 },
    { 0x7e5d9e30, SUC_lexicon_tags_25 },
    { 0x68dd9e30, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7d9e34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x169d9e36, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x555d9e38, SUC_lexicon_tags_37 },
    { 0x8fbd9e39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdf1d9e3b, SUC_lexicon_tags_12 },
    { 0x477d9e3c, SUC_lexicon_tags_7 },
    { 0x55bd9e3d, SUC_lexicon_tags_14 },
    { 0x9f9d9e3e, SUC_lexicon_tags_47 },
    { 0xd9d9e3e, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbd9e4c, SUC_lexicon_tags_12 },
    { 0xa6fd9e4d, SUC_lexicon_tags_30 },
    { 0x8bfd9e4e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bd9e53, SUC_lexicon_tags_83 },
    { 0x6b9d9e54, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405d9e5f, SUC_lexicon_tags_6 },
    { 0x7afd9e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9d9e66, SUC_lexicon_tags_0 },
    { 0xc79d9e67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc3d9e69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319d9e6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x78dd9e6f, SUC_lexicon_tags_17 },
    { 0xe4fd9e6f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6dd9e73, SUC_lexicon_tags_16 },
    { 0x1d5d9e73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1d9e79, SUC_lexicon_tags_37 },
    { 0x299d9e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fd9e7d, SUC_lexicon_tags_17 },
    { 0x4b1d9e7e, SUC_lexicon_tags_141 },
    { 0xdc1d9e7e, SUC_lexicon_tags_1 },
    { 0x843d9e80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9efd9e83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c1d9e85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x257d9e87, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x11dd9e8a, SUC_lexicon_tags_38 },
    { 0x65bd9e8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223d9e8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x231d9e91, SUC_lexicon_tags_87 },
    { 0x843d9e92, SUC_lexicon_tags_25 },
    { 0xd21d9e92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x101d9e96, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221d9e9c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xacbd9e9e, SUC_lexicon_tags_38 },
    { 0xd93d9e9f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbc9d9ea1, SUC_lexicon_tags_1 },
    { 0xa55d9ea2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbd9ea5, SUC_lexicon_tags_79 },
    { 0x549d9ea6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9a9d9ea8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dd9eac, SUC_lexicon_tags_17 },
    { 0x307d9eac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bd9eb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879d9eb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edd9eb8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4afd9eba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14dd9ebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9d9ec4, SUC_lexicon_tags_2 },
    { 0xcfbd9ec4, SUC_lexicon_tags_18 },
    { 0xda3d9ec4, SUC_lexicon_tags_14 },
    { 0x7edd9ec7, SUC_lexicon_tags_1 },
    { 0x3c5d9ec7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdebd9eca, SUC_lexicon_tags_17 },
    { 0x93dd9ecb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcbbd9ecd, SUC_lexicon_tags_61 },
    { 0x669d9ecd, SUC_lexicon_tags_33 },
    { 0x3cfd9ecd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91d9ed2, SUC_lexicon_tags_12 },
    { 0xdd3d9ed3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fd9edd, SUC_lexicon_tags_65 },
    { 0xa67d9edd, SUC_lexicon_tags_37 },
    { 0xdc9d9edf, SUC_lexicon_tags_1 },
    { 0x8efd9ee0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449d9ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47d9ee8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x645d9eea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1d9eee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7d9ef2, SUC_lexicon_tags_10 },
    { 0x307d9ef3, SUC_lexicon_tags_17 },
    { 0x9ffd9ef4, SUC_lexicon_tags_63 },
    { 0x511d9ef5, SUC_lexicon_tags_25 },
    { 0x583d9ef6, SUC_lexicon_tags_49 },
    { 0x1b1d9ef6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x453d9efa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2efd9efc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x317d9efe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd0bd9f00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393d9f04, SUC_lexicon_tags_60 },
    { 0x13d9f04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dd9f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1d9f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edd9f17, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3d9f1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x729d9f1e, SUC_lexicon_tags_1 },
    { 0xcd9d9f1e, SUC_lexicon_tags_34 },
    { 0xf07d9f1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd87d9f22, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175d9f28, SUC_lexicon_tags_42 },
    { 0xebbd9f28, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6dfd9f2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf33d9f2d, SUC_lexicon_tags_27 },
    { 0xcb1d9f2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9d9f32, SUC_lexicon_tags_2 },
    { 0x337d9f33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267d9f38, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfd9f3d, SUC_lexicon_tags_30 },
    { 0xbffd9f3e, SUC_lexicon_tags_37 },
    { 0x85d9f3f, SUC_lexicon_tags_25 },
    { 0x8f3d9f40, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95d9f47, SUC_lexicon_tags_7 },
    { 0x62bd9f47, SUC_lexicon_tags_784 },
    { 0xeafd9f49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fd9f4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161d9f52, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01d9f55, SUC_lexicon_tags_17 },
    { 0xda5d9f56, SUC_lexicon_tags_11 },
    { 0x463d9f57, SUC_lexicon_tags_63 },
    { 0xccdd9f57, SUC_lexicon_tags_65 },
    { 0x277d9f59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57d9f5c, SUC_lexicon_tags_47 },
    { 0xad3d9f5d, SUC_lexicon_tags_37 },
    { 0x4a5d9f5e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa9d9f60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbd9f64, SUC_lexicon_tags_38 },
    { 0x7b7d9f65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf39d9f67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x567d9f69, SUC_lexicon_tags_25 },
    { 0x575d9f6a, SUC_lexicon_tags_60 },
    { 0xf37d9f6b, SUC_lexicon_tags_33 },
    { 0xfc1d9f6a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x503d9f6e, SUC_lexicon_tags_26 },
    { 0x1b1d9f6f, SUC_lexicon_tags_10 },
    { 0x985d9f70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbd9f75, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9c1d9f77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7d9f7a, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x93d9f7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53d9f83, SUC_lexicon_tags_33 },
    { 0x29bd9f84, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bd9f8c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x38bd9f8e, SUC_lexicon_tags_17 },
    { 0x725d9f8f, SUC_lexicon_tags_12 },
    { 0x96fd9f8f, SUC_lexicon_tags_25 },
    { 0x2cfd9f90, SUC_lexicon_tags_87 },
    { 0xb49d9f92, SUC_lexicon_tags_15 },
    { 0xfe1d9f93, SUC_lexicon_tags_23 },
    { 0xa43d9f94, SUC_lexicon_tags_47 },
    { 0x1d5d9f95, SUC_lexicon_tags_49 },
    { 0x1bfd9f94, SUC_lexicon_tags_1 },
    { 0xe07d9f95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x381d9f99, SUC_lexicon_tags_33 },
    { 0x97fd9f9a, SUC_lexicon_tags_27 },
    { 0x413d9f9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57d9fa1, SUC_lexicon_tags_34 },
    { 0x3f7d9fa2, SUC_lexicon_tags_114 },
    { 0xfdbd9fa1, SUC_lexicon_tags_57 },
    { 0x1b9d9fa3, SUC_lexicon_tags_4 },
    { 0x98bd9fa4, SUC_lexicon_tags_25 },
    { 0x79d9fa1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5d9faf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb13d9fb1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fd9fb4, SUC_lexicon_tags_37 },
    { 0x351d9fb5, SUC_lexicon_tags_14 },
    { 0x5e1d9fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7d9fbc, SUC_lexicon_tags_25 },
    { 0x8bfd9fbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4bdd9fbf, SUC_lexicon_tags_37 },
    { 0x181d9fbf, SUC_lexicon_tags_15 },
    { 0xa25d9fc1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bd9fc5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbd9fcb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1d9fd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19d9fd5, SUC_lexicon_tags_39 },
    { 0x907d9fd5, SUC_lexicon_tags_15 },
    { 0x441d9fd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7d9fdb, SUC_lexicon_tags_15 },
    { 0x9bd9fdb, SUC_lexicon_tags_17 },
    { 0x429d9fdd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9e5d9fdf, SUC_lexicon_tags_60 },
    { 0xb0fd9fdf, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7e9d9fe2, SUC_lexicon_tags_11 },
    { 0xf1fd9fe3, SUC_lexicon_tags_87 },
    { 0x3c9d9fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfddd9fe6, SUC_lexicon_tags_49 },
    { 0xe3bd9fe7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bd9fec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe99d9fee, SUC_lexicon_tags_47 },
    { 0xd35d9fee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbd9ff2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9d9ff7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bd9ffd, SUC_lexicon_tags_4 },
    { 0xb79d9ffd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957da002, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd1dda004, SUC_lexicon_tags_70 },
    { 0x327da005, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175da00b, SUC_lexicon_tags_47 },
    { 0x6a9da00c, SUC_lexicon_tags_37 },
    { 0xcd3da00d, SUC_lexicon_tags_25 },
    { 0x2e5da00e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x5bfda010, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503da014, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81da017, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55da01e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fda022, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fda026, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf1dda028, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe35da02a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfa5da02c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb39da02e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffbda030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x249da032, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x249da034, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x311da036, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7fda038, SUC_lexicon_tags_28 },
    { 0x5b3da039, SUC_lexicon_tags_37 },
    { 0x37bda038, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdda03e, SUC_lexicon_tags_17 },
    { 0xa3bda03f, SUC_lexicon_tags_11 },
    { 0x7c9da040, SUC_lexicon_tags_25 },
    { 0xed3da03f, SUC_lexicon_tags_1 },
    { 0x69bda042, SUC_lexicon_tags_32 },
    { 0xc83da041, SUC_lexicon_tags_60 },
    { 0xdf5da040, SUC_lexicon_tags_33 },
    { 0x5e1da045, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7da048, SUC_lexicon_tags_53 },
    { 0x32fda048, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbda04c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9da051, SUC_lexicon_tags_7 },
    { 0x1d3da052, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbda055, SUC_lexicon_tags_17 },
    { 0xec7da055, SUC_lexicon_tags_26 },
    { 0x23bda057, SUC_lexicon_tags_17 },
    { 0xb8bda058, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bda05d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfda06b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725da071, SUC_lexicon_tags_2 },
    { 0xbb5da072, SUC_lexicon_tags_25 },
    { 0xc6bda073, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bda07a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dda07d, SUC_lexicon_tags_47 },
    { 0xb1da07e, SUC_lexicon_tags_1 },
    { 0x75da07f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67da081, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5da087, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcdfda089, SUC_lexicon_tags_34 },
    { 0xd63da089, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x32dda08c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x149da08e, SUC_lexicon_tags_33 },
    { 0x6f5da08f, SUC_lexicon_tags_45 },
    { 0x7e5da08e, SUC_lexicon_tags_2 },
    { 0x3b5da091, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475da098, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7da09e, SUC_lexicon_tags_6 },
    { 0x95da09f, SUC_lexicon_tags_47 },
    { 0x363da0a0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xdf9da0a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddda0a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72fda0aa, SUC_lexicon_tags_17 },
    { 0x285da0aa, SUC_lexicon_tags_47 },
    { 0x20bda0ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bda0b0, SUC_lexicon_tags_38 },
    { 0x711da0b1, SUC_lexicon_tags_7 },
    { 0x5d3da0b2, SUC_lexicon_tags_30 },
    { 0xef9da0b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7efda0b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1da0b9, SUC_lexicon_tags_25 },
    { 0x195da0b9, SUC_lexicon_tags_13 },
    { 0xa83da0bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83da0be, SUC_lexicon_tags_12 },
    { 0xdc1da0be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdda0c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x907da0c6, SUC_lexicon_tags_1 },
    { 0xf45da0c6, SUC_lexicon_tags_4 },
    { 0xc1dda0c8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdda0cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc99da0ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x39dda0d1, SUC_lexicon_tags_60 },
    { 0x73dda0d2, SUC_lexicon_tags_29 },
    { 0x70fda0d2, SUC_lexicon_tags_17 },
    { 0x327da0d2, SUC_lexicon_tags_19 },
    { 0xb1da0d3, SUC_lexicon_tags_9 },
    { 0xd61da0d2, SUC_lexicon_tags_49 },
    { 0x211da0d7, SUC_lexicon_tags_28 },
    { 0x4b7da0d5, SUC_lexicon_tags_46 },
    { 0xe61da0d9, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abda0e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x317da0e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x431da0e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x867da0e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bda0f3, SUC_lexicon_tags_17 },
    { 0x221da0f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dda0fb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3da0fe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9da104, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbda10d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43bda10f, SUC_lexicon_tags_7 },
    { 0xa27da110, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5da113, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8c9da115, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25fda119, SUC_lexicon_tags_25 },
    { 0x9da11a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479da121, SUC_lexicon_tags_269 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdda12a, SUC_lexicon_tags_25 },
    { 0x281da12b, SUC_lexicon_tags_66 },
    { 0x1e9da12c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc67da12e, SUC_lexicon_tags_48 },
    { 0x6bbda12f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x475da131, SUC_lexicon_tags_28 },
    { 0x5dda132, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcabda134, SUC_lexicon_tags_27 },
    { 0x19fda134, SUC_lexicon_tags_47 },
    { 0x519da136, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x1d3da138, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3da13f, SUC_lexicon_tags_17 },
    { 0xd0bda140, SUC_lexicon_tags_15 },
    { 0x32bda141, SUC_lexicon_tags_15 },
    { 0x1c3da142, SUC_lexicon_tags_25 },
    { 0x77dda143, SUC_lexicon_tags_38 },
    { 0xd49da144, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edda147, SUC_lexicon_tags_46 },
    { 0x37bda148, SUC_lexicon_tags_33 },
    { 0x639da149, SUC_lexicon_tags_37 },
    { 0x279da148, SUC_lexicon_tags_5 },
    { 0xc5da148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23da151, SUC_lexicon_tags_1 },
    { 0x4fdda152, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fda155, SUC_lexicon_tags_37 },
    { 0x237da156, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x447da158, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xddfda15b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81da160, SUC_lexicon_tags_33 },
    { 0xe85da160, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35da165, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4dda167, SUC_lexicon_tags_12 },
    { 0x3dda167, SUC_lexicon_tags_12 },
    { 0x1b1da168, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9da16e, SUC_lexicon_tags_33 },
    { 0x9f5da16f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7da172, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbda176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11da17a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dda183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccbda185, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dda18a, SUC_lexicon_tags_16 },
    { 0x779da18b, SUC_lexicon_tags_12 },
    { 0xf7dda18c, SUC_lexicon_tags_68 },
    { 0xac7da18c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5da193, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebda196, SUC_lexicon_tags_2 },
    { 0xa2bda197, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd43da199, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dda19c, SUC_lexicon_tags_37 },
    { 0x961da19d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfd5da19f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87da1a2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37da1a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec7da1aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9da1ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5da1b1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfda1b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bda1b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf45da1b9, SUC_lexicon_tags_37 },
    { 0xc5bda1ba, SUC_lexicon_tags_7 },
    { 0xa87da1b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45da1c2, SUC_lexicon_tags_25 },
    { 0xfe5da1c3, SUC_lexicon_tags_49 },
    { 0xcb9da1c2, SUC_lexicon_tags_29 },
    { 0x995da1c4, SUC_lexicon_tags_24 },
    { 0x4d1da1c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbda1cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x831da1ce, SUC_lexicon_tags_45 },
    { 0x84bda1ce, SUC_lexicon_tags_42 },
    { 0x4afda1d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fda1d3, SUC_lexicon_tags_3 },
    { 0xda5da1d4, SUC_lexicon_tags_14 },
    { 0x299da1d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x619da1d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17da1db, SUC_lexicon_tags_7 },
    { 0xe3da1dc, SUC_lexicon_tags_9 },
    { 0x5dfda1dd, SUC_lexicon_tags_57 },
    { 0x2e1da1dc, SUC_lexicon_tags_12 },
    { 0xefdda1df, SUC_lexicon_tags_75 },
    { 0x699da1df, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421da1e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91da1e9, SUC_lexicon_tags_39 },
    { 0x1ddda1e9, SUC_lexicon_tags_33 },
    { 0x31bda1e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x933da1ee, SUC_lexicon_tags_49 },
    { 0x3f9da1ee, SUC_lexicon_tags_25 },
    { 0x5d3da1f0, SUC_lexicon_tags_46 },
    { 0x45fda1f1, SUC_lexicon_tags_37 },
    { 0x8d1da1ef, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309da1fc, SUC_lexicon_tags_15 },
    { 0x4d1da1fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x731da1ff, SUC_lexicon_tags_7 },
    { 0x66bda1ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dda203, SUC_lexicon_tags_61 },
    { 0x515da204, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x455da207, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8fdda209, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x137da20b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcb5da20d, SUC_lexicon_tags_17 },
    { 0x1a5da20e, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafda217, SUC_lexicon_tags_0 },
    { 0xb3da218, SUC_lexicon_tags_17 },
    { 0x823da219, SUC_lexicon_tags_37 },
    { 0x5a5da218, SUC_lexicon_tags_0 },
    { 0x753da21a, SUC_lexicon_tags_16 },
    { 0x441da21c, SUC_lexicon_tags_55 },
    { 0x949da21c, SUC_lexicon_tags_2 },
    { 0xae1da21c, SUC_lexicon_tags_19 },
    { 0xcb3da21f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5da22b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x47dda22d, SUC_lexicon_tags_572 },
    { 0x58fda22e, SUC_lexicon_tags_2 },
    { 0xcfbda22e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bda233, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07da238, SUC_lexicon_tags_45 },
    { 0x127da239, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1da23d, SUC_lexicon_tags_8 },
    { 0xf73da23e, SUC_lexicon_tags_57 },
    { 0xc77da23f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x581da241, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fda246, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x197da248, SUC_lexicon_tags_48 },
    { 0x247da249, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7da24e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd8fda250, SUC_lexicon_tags_2 },
    { 0x1c9da250, SUC_lexicon_tags_37 },
    { 0xf87da250, SUC_lexicon_tags_75 },
    { 0x41da253, SUC_lexicon_tags_25 },
    { 0x311da254, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1da258, SUC_lexicon_tags_4 },
    { 0x22fda259, SUC_lexicon_tags_10 },
    { 0x849da258, SUC_lexicon_tags_16 },
    { 0x3abda25b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x241da25d, SUC_lexicon_tags_37 },
    { 0xcc3da25e, SUC_lexicon_tags_39 },
    { 0x355da25f, SUC_lexicon_tags_29 },
    { 0xe6bda260, SUC_lexicon_tags_4 },
    { 0x6f5da25f, SUC_lexicon_tags_26 },
    { 0xed9da262, SUC_lexicon_tags_30 },
    { 0x123da262, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fda26b, SUC_lexicon_tags_63 },
    { 0x3c7da26c, SUC_lexicon_tags_29 },
    { 0xd93da26b, SUC_lexicon_tags_25 },
    { 0x9c9da26e, SUC_lexicon_tags_17 },
    { 0x86fda26f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x465da271, SUC_lexicon_tags_23 },
    { 0x5c9da272, SUC_lexicon_tags_33 },
    { 0x9a3da273, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x861da275, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc41da277, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dda27b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edda27f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76dda281, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187da28a, SUC_lexicon_tags_8 },
    { 0x5f9da28a, SUC_lexicon_tags_12 },
    { 0xc87da28b, SUC_lexicon_tags_33 },
    { 0xe7da28d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1da291, SUC_lexicon_tags_17 },
    { 0x7c3da291, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07da298, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39da29d, SUC_lexicon_tags_66 },
    { 0xb37da29e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdda2a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595da2a9, SUC_lexicon_tags_33 },
    { 0x79da2aa, SUC_lexicon_tags_14 },
    { 0x581da2ab, SUC_lexicon_tags_2 },
    { 0xbc7da2ac, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35da2af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50bda2b1, SUC_lexicon_tags_7 },
    { 0x44bda2b2, SUC_lexicon_tags_11 },
    { 0x67bda2b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67da2b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697da2ba, SUC_lexicon_tags_30 },
    { 0x147da2bb, SUC_lexicon_tags_27 },
    { 0xee9da2ba, SUC_lexicon_tags_0 },
    { 0x9f9da2bc, SUC_lexicon_tags_55 },
    { 0x95bda2bd, SUC_lexicon_tags_25 },
    { 0x12fda2bb, SUC_lexicon_tags_15 },
    { 0x3cfda2bc, SUC_lexicon_tags_29 },
    { 0x7c3da2c1, SUC_lexicon_tags_60 },
    { 0xee1da2c2, SUC_lexicon_tags_42 },
    { 0xd69da2c0, SUC_lexicon_tags_7 },
    { 0xabdda2c4, SUC_lexicon_tags_11 },
    { 0x9dbda2c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07da2c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x207da2cb, SUC_lexicon_tags_11 },
    { 0x993da2cb, SUC_lexicon_tags_45 },
    { 0xe5da2cd, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41da2d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7da2d3, SUC_lexicon_tags_38 },
    { 0x4ffda2d4, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x26fda2d6, SUC_lexicon_tags_21 },
    { 0xb7bda2d6, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0xecfda2da, SUC_lexicon_tags_0 },
    { 0xc43da2da, SUC_lexicon_tags_8 },
    { 0x659da2dc, SUC_lexicon_tags_39 },
    { 0xd61da2dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xeadda2e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1da2e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9da2ea, SUC_lexicon_tags_1 },
    { 0x13fda2eb, SUC_lexicon_tags_38 },
    { 0x981da2ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783da2f4, SUC_lexicon_tags_37 },
    { 0x79bda2f4, SUC_lexicon_tags_23 },
    { 0x197da2f6, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39da2f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653da2fe, SUC_lexicon_tags_15 },
    { 0xc63da2ff, SUC_lexicon_tags_30 },
    { 0x319da300, SUC_lexicon_tags_8 },
    { 0x70bda300, SUC_lexicon_tags_15 },
    { 0x4dda301, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe73da304, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bda30d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37dda30f, SUC_lexicon_tags_12 },
    { 0xbbdda30f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfda319, SUC_lexicon_tags_17 },
    { 0xcf9da319, SUC_lexicon_tags_37 },
    { 0x9afda31b, SUC_lexicon_tags_14 },
    { 0xc3da31b, SUC_lexicon_tags_43 },
    { 0xab5da31c, SUC_lexicon_tags_14 },
    { 0x8adda31b, SUC_lexicon_tags_17 },
    { 0xc7dda31f, SUC_lexicon_tags_37 },
    { 0x7afda320, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x41da322, SUC_lexicon_tags_15 },
    { 0x5bda323, SUC_lexicon_tags_11 },
    { 0xcfbda324, SUC_lexicon_tags_25 },
    { 0xab3da325, SUC_lexicon_tags_1 },
    { 0xb59da326, SUC_lexicon_tags_45 },
    { 0xcbbda326, SUC_lexicon_tags_31 },
    { 0xe41da328, SUC_lexicon_tags_39 },
    { 0x619da323, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x33dda32c, SUC_lexicon_tags_9 },
    { 0x73fda32d, SUC_lexicon_tags_11 },
    { 0x885da32e, SUC_lexicon_tags_7 },
    { 0x4a1da32f, SUC_lexicon_tags_15 },
    { 0x6bfda330, SUC_lexicon_tags_16 },
    { 0x503da331, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xae9da333, SUC_lexicon_tags_1 },
    { 0xcb1da334, SUC_lexicon_tags_46 },
    { 0x797da335, SUC_lexicon_tags_14 },
    { 0xdc3da336, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd9da338, SUC_lexicon_tags_25 },
    { 0x9bda339, SUC_lexicon_tags_10 },
    { 0x9f7da33a, SUC_lexicon_tags_31 },
    { 0x971da33a, SUC_lexicon_tags_572 },
    { 0xaabda33a, SUC_lexicon_tags_58 },
    { 0xe17da33c, SUC_lexicon_tags_28 },
    { 0xb8bda33e, SUC_lexicon_tags_47 },
    { 0x579da33e, SUC_lexicon_tags_14 },
    { 0x8afda33f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5d1da342, SUC_lexicon_tags_9 },
    { 0x629da342, SUC_lexicon_tags_2 },
    { 0x357da343, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321da349, SUC_lexicon_tags_37 },
    { 0x573da34a, SUC_lexicon_tags_37 },
    { 0x527da34a, SUC_lexicon_tags_23 },
    { 0xefda34c, SUC_lexicon_tags_75 },
    { 0x135da34c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dda352, SUC_lexicon_tags_207 },
    { 0x27bda353, SUC_lexicon_tags_57 },
    { 0x9dbda353, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7da357, SUC_lexicon_tags_17 },
    { 0x8b3da357, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda9da35a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713da361, SUC_lexicon_tags_49 },
    { 0x979da362, SUC_lexicon_tags_31 },
    { 0xff5da361, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x577da366, SUC_lexicon_tags_45 },
    { 0x48bda367, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dda36e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bda371, SUC_lexicon_tags_17 },
    { 0xe3fda372, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffda376, SUC_lexicon_tags_11 },
    { 0xffdda376, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9da37c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81da386, SUC_lexicon_tags_17 },
    { 0x707da387, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855da38b, SUC_lexicon_tags_15 },
    { 0xfa3da38c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf7da38e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x93bda390, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99da395, SUC_lexicon_tags_11 },
    { 0xd8bda396, SUC_lexicon_tags_49 },
    { 0xd85da396, SUC_lexicon_tags_11 },
    { 0x183da397, SUC_lexicon_tags_41 },
    { 0x1dfda399, SUC_lexicon_tags_37 },
    { 0x1a5da399, SUC_lexicon_tags_2 },
    { 0x1b5da395, SUC_lexicon_tags_4 },
    { 0x363da39c, SUC_lexicon_tags_65 },
    { 0xea5da39d, SUC_lexicon_tags_37 },
    { 0x151da39c, SUC_lexicon_tags_12 },
    { 0xe7da39c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x71da3a1, SUC_lexicon_tags_201 },
    { 0x565da3a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefda3a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xed5da3a8, SUC_lexicon_tags_15 },
    { 0x6d9da3a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadda3ad, SUC_lexicon_tags_2 },
    { 0x331da3ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbda3b3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dda3b8, SUC_lexicon_tags_37 },
    { 0x41dda3b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21da3be, SUC_lexicon_tags_11 },
    { 0x1fdda3bf, SUC_lexicon_tags_29 },
    { 0x9fda3c0, SUC_lexicon_tags_11 },
    { 0x12fda3c1, SUC_lexicon_tags_47 },
    { 0xabbda3c2, SUC_lexicon_tags_11 },
    { 0xc6dda3c2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6b7da3c5, SUC_lexicon_tags_104 },
    { 0x41da3c6, SUC_lexicon_tags_49 },
    { 0x231da3c7, SUC_lexicon_tags_1 },
    { 0xc55da3c7, SUC_lexicon_tags_9 },
    { 0x45bda3c6, SUC_lexicon_tags_7 },
    { 0xed5da3ca, SUC_lexicon_tags_17 },
    { 0x7c1da3ca, SUC_lexicon_tags_34 },
    { 0x839da3c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x529da3ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7da3d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3dda3d3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1b9da3d5, SUC_lexicon_tags_7 },
    { 0xb5fda3d6, SUC_lexicon_tags_14 },
    { 0x7fbda3d6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dda3da, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5da3de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc93da3e0, SUC_lexicon_tags_49 },
    { 0x4b5da3e1, SUC_lexicon_tags_17 },
    { 0xa9bda3e2, SUC_lexicon_tags_1 },
    { 0xe77da3e2, SUC_lexicon_tags_2 },
    { 0x81dda3e1, SUC_lexicon_tags_28 },
    { 0x4e7da3e5, SUC_lexicon_tags_31 },
    { 0xea1da3e5, SUC_lexicon_tags_38 },
    { 0x3d1da3e7, SUC_lexicon_tags_38 },
    { 0xdb9da3e7, SUC_lexicon_tags_14 },
    { 0xfb9da3e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x105da3eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3f7da3ed, SUC_lexicon_tags_37 },
    { 0x437da3ed, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dda3f1, SUC_lexicon_tags_0 },
    { 0xf5fda3f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3edda3f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1da3f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7da3fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31da402, SUC_lexicon_tags_25 },
    { 0x76bda403, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bda407, SUC_lexicon_tags_9 },
    { 0x2ebda407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x64dda40b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe33da40d, SUC_lexicon_tags_6 },
    { 0xb6bda40e, SUC_lexicon_tags_25 },
    { 0xbbdda40f, SUC_lexicon_tags_23 },
    { 0x5e1da40f, SUC_lexicon_tags_49 },
    { 0xc3fda40e, SUC_lexicon_tags_29 },
    { 0x9c5da40e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb71da414, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583da418, SUC_lexicon_tags_15 },
    { 0xb09da419, SUC_lexicon_tags_55 },
    { 0x307da418, SUC_lexicon_tags_7 },
    { 0x4e3da41b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49dda41f, SUC_lexicon_tags_4 },
    { 0xb77da420, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3da429, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835da42e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b1da430, SUC_lexicon_tags_23 },
    { 0xa49da430, SUC_lexicon_tags_37 },
    { 0xad5da432, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bda439, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601da43d, SUC_lexicon_tags_28 },
    { 0xf55da43e, SUC_lexicon_tags_45 },
    { 0xc2dda43f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353da446, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7bfda448, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1da44c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885da452, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6a3da454, SUC_lexicon_tags_38 },
    { 0xf49da455, SUC_lexicon_tags_7 },
    { 0xea1da454, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bda45b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fda45f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x485da461, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7adda463, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fda468, SUC_lexicon_tags_30 },
    { 0x7e3da469, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8f5da46b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc17da46d, SUC_lexicon_tags_1 },
    { 0xa3da46d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x791da470, SUC_lexicon_tags_9 },
    { 0xa8bda471, SUC_lexicon_tags_19 },
    { 0x89dda471, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdda475, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251da47b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x90dda47d, SUC_lexicon_tags_7 },
    { 0xf5da47e, SUC_lexicon_tags_63 },
    { 0x50dda47d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bda487, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7da48a, SUC_lexicon_tags_27 },
    { 0x415da48a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd5da48d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51da492, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bda495, SUC_lexicon_tags_19 },
    { 0xa7da495, SUC_lexicon_tags_7 },
    { 0x653da497, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaabda499, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b3da49b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9da49f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabda4a2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbda4a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb17da4a7, SUC_lexicon_tags_29 },
    { 0x933da4a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9da4ab, SUC_lexicon_tags_8 },
    { 0xaadda4ac, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x77bda4ae, SUC_lexicon_tags_11 },
    { 0x8dfda4ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bda4b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fda4b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5da4bc, SUC_lexicon_tags_11 },
    { 0xf79da4bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bda4c1, SUC_lexicon_tags_37 },
    { 0xd2fda4c2, SUC_lexicon_tags_21 },
    { 0xfcfda4c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x113da4c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xad7da4c8, SUC_lexicon_tags_17 },
    { 0x2cdda4c9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x29da4cc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdda4d0, SUC_lexicon_tags_21 },
    { 0xe5fda4d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9da4d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bda4d9, SUC_lexicon_tags_15 },
    { 0x1efda4da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951da4df, SUC_lexicon_tags_2 },
    { 0xf05da4df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dda4e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x105da4e6, SUC_lexicon_tags_37 },
    { 0xc45da4e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1da4eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bda4f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fda4f9, SUC_lexicon_tags_8 },
    { 0xf53da4fa, SUC_lexicon_tags_68 },
    { 0x88fda4fb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05da4fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x923da500, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7da503, SUC_lexicon_tags_35 },
    { 0xfdbda504, SUC_lexicon_tags_70 },
    { 0x789da504, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x52dda508, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2fda50a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xeb1da50c, SUC_lexicon_tags_22 },
    { 0x2fda50d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fda512, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89da515, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb7da517, SUC_lexicon_tags_30 },
    { 0xae3da518, SUC_lexicon_tags_11 },
    { 0x8edda519, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb77da51b, SUC_lexicon_tags_7 },
    { 0x7c7da51c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x98fda51e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bda521, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fda524, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997da528, SUC_lexicon_tags_25 },
    { 0x779da528, SUC_lexicon_tags_11 },
    { 0x511da52a, SUC_lexicon_tags_11 },
    { 0x92fda52a, SUC_lexicon_tags_49 },
    { 0x551da52c, SUC_lexicon_tags_56 },
    { 0xebfda52b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x407da530, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x549da532, SUC_lexicon_tags_3 },
    { 0xb3dda533, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76bda535, SUC_lexicon_tags_68 },
    { 0x6b9da536, SUC_lexicon_tags_7 },
    { 0xc15da537, SUC_lexicon_tags_28 },
    { 0x79da538, SUC_lexicon_tags_15 },
    { 0x5a1da539, SUC_lexicon_tags_15 },
    { 0x20dda53a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9da53e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7da541, SUC_lexicon_tags_27 },
    { 0x945da542, SUC_lexicon_tags_17 },
    { 0xd47da543, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddda546, SUC_lexicon_tags_38 },
    { 0x659da546, SUC_lexicon_tags_15 },
    { 0x7b9da548, SUC_lexicon_tags_25 },
    { 0x10dda549, SUC_lexicon_tags_49 },
    { 0x483da54a, SUC_lexicon_tags_85 },
    { 0x9d9da54a, SUC_lexicon_tags_7 },
    { 0x723da548, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efda553, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x61da555, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9adda557, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7da55a, SUC_lexicon_tags_77 },
    { 0x15bda55a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc97da55d, SUC_lexicon_tags_12 },
    { 0x687da55e, SUC_lexicon_tags_37 },
    { 0x54bda55f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5da568, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307da56f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x849da571, SUC_lexicon_tags_38 },
    { 0xb05da572, SUC_lexicon_tags_17 },
    { 0x4ebda573, SUC_lexicon_tags_15 },
    { 0xb15da574, SUC_lexicon_tags_66 },
    { 0xfa1da574, SUC_lexicon_tags_3 },
    { 0x767da571, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93da579, SUC_lexicon_tags_17 },
    { 0x9e7da57a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x59bda57c, SUC_lexicon_tags_2 },
    { 0x94fda57c, SUC_lexicon_tags_63 },
    { 0x50fda57e, SUC_lexicon_tags_77 },
    { 0x3cbda57f, SUC_lexicon_tags_3 },
    { 0xb71da57f, SUC_lexicon_tags_63 },
    { 0x357da581, SUC_lexicon_tags_48 },
    { 0x6afda57f, SUC_lexicon_tags_57 },
    { 0xc69da57c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x595da585, SUC_lexicon_tags_17 },
    { 0x3fda586, SUC_lexicon_tags_10 },
    { 0xb09da586, SUC_lexicon_tags_25 },
    { 0xf4fda588, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf39da58a, SUC_lexicon_tags_15 },
    { 0x90bda58b, SUC_lexicon_tags_22 },
    { 0xde7da58b, SUC_lexicon_tags_47 },
    { 0xe8dda58d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfda59a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd99da59c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27da5a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11da5a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc53da5ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe63da5ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dda5b1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadda5b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77da5bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445da5bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7da5c4, SUC_lexicon_tags_10 },
    { 0x8d9da5c4, SUC_lexicon_tags_37 },
    { 0x75dda5c4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5da5c9, SUC_lexicon_tags_45 },
    { 0xe8bda5c9, SUC_lexicon_tags_24 },
    { 0x1cdda5ca, SUC_lexicon_tags_1 },
    { 0xd57da5ca, SUC_lexicon_tags_11 },
    { 0x931da5cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799da5d3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53dda5d5, SUC_lexicon_tags_11 },
    { 0xbebda5d6, SUC_lexicon_tags_28 },
    { 0x451da5d7, SUC_lexicon_tags_8 },
    { 0x899da5d8, SUC_lexicon_tags_60 },
    { 0x285da5d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fda5e2, SUC_lexicon_tags_27 },
    { 0xe9bda5e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9da5e5, SUC_lexicon_tags_0 },
    { 0x439da5e6, SUC_lexicon_tags_18 },
    { 0x157da5e6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1da5ee, SUC_lexicon_tags_9 },
    { 0xa65da5ef, SUC_lexicon_tags_57 },
    { 0xfc7da5ee, SUC_lexicon_tags_7 },
    { 0xed3da5f1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fda5f4, SUC_lexicon_tags_249 },
    { 0, NULL },
    { 0xf01da5f6, SUC_lexicon_tags_8 },
    { 0xbcbda5f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa7dda5f9, SUC_lexicon_tags_179 },
    { 0xe0dda5f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x14dda5fd, SUC_lexicon_tags_47 },
    { 0x5fda5fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf09da600, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x977da602, SUC_lexicon_tags_45 },
    { 0xa5fda602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2a9da605, SUC_lexicon_tags_7 },
    { 0x5e7da606, SUC_lexicon_tags_10 },
    { 0x71fda606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab1da609, SUC_lexicon_tags_38 },
    { 0x7e3da60a, SUC_lexicon_tags_43 },
    { 0x42dda609, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3da60e, SUC_lexicon_tags_17 },
    { 0x53fda60e, SUC_lexicon_tags_38 },
    { 0xbf5da60f, SUC_lexicon_tags_48 },
    { 0x18dda611, SUC_lexicon_tags_47 },
    { 0xdfbda612, SUC_lexicon_tags_45 },
    { 0x461da612, SUC_lexicon_tags_92 },
    { 0x551da614, SUC_lexicon_tags_2 },
    { 0x41bda615, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73da61a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773da61e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x315da620, SUC_lexicon_tags_29 },
    { 0xbe7da621, SUC_lexicon_tags_19 },
    { 0x42bda622, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9da625, SUC_lexicon_tags_49 },
    { 0x565da625, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46dda629, SUC_lexicon_tags_15 },
    { 0x6b9da62a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09da62d, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63da631, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e5da633, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fda63c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7da642, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x48dda645, SUC_lexicon_tags_11 },
    { 0xb93da645, SUC_lexicon_tags_38 },
    { 0x8b1da647, SUC_lexicon_tags_9 },
    { 0xda5da645, SUC_lexicon_tags_39 },
    { 0xfb7da649, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5da656, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437da65e, SUC_lexicon_tags_47 },
    { 0xd5dda65e, SUC_lexicon_tags_2 },
    { 0xf61da65e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fda664, SUC_lexicon_tags_37 },
    { 0xe1fda665, SUC_lexicon_tags_69 },
    { 0x29dda666, SUC_lexicon_tags_7 },
    { 0x649da666, SUC_lexicon_tags_12 },
    { 0xe29da668, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x279da66a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1da66e, SUC_lexicon_tags_6 },
    { 0xa95da66e, SUC_lexicon_tags_38 },
    { 0x4c5da670, SUC_lexicon_tags_19 },
    { 0xaafda66e, SUC_lexicon_tags_44 },
    { 0xd03da672, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffda677, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xcf3da679, SUC_lexicon_tags_1 },
    { 0x31fda67a, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0xc39da67c, SUC_lexicon_tags_49 },
    { 0x42fda67d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9da681, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42dda685, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbda689, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7da692, SUC_lexicon_tags_55 },
    { 0x49bda692, SUC_lexicon_tags_6 },
    { 0x11da694, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1b3da696, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbfda698, SUC_lexicon_tags_1 },
    { 0x8b5da698, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x59da69b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29da69e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c1da6a0, SUC_lexicon_tags_15 },
    { 0xdbda6a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9da6a8, SUC_lexicon_tags_26 },
    { 0xc9bda6a9, SUC_lexicon_tags_49 },
    { 0xa9dda6aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd7da6ac, SUC_lexicon_tags_38 },
    { 0x8ffda6ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05da6b0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bda6b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dda6b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3da6bc, SUC_lexicon_tags_17 },
    { 0x69fda6bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515da6c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fda6c4, SUC_lexicon_tags_36 },
    { 0x89da6c5, SUC_lexicon_tags_0 },
    { 0xef9da6c5, SUC_lexicon_tags_31 },
    { 0x895da6c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x40bda6c9, SUC_lexicon_tags_46 },
    { 0x813da6c9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dda6d0, SUC_lexicon_tags_25 },
    { 0x5a7da6d1, SUC_lexicon_tags_11 },
    { 0x8abda6d1, SUC_lexicon_tags_12 },
    { 0x303da6d3, SUC_lexicon_tags_178 },
    { 0x1b1da6d4, SUC_lexicon_tags_2 },
    { 0x1bdda6d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c9da6d7, SUC_lexicon_tags_27 },
    { 0xbd3da6d8, SUC_lexicon_tags_45 },
    { 0x2e5da6d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9da6dc, SUC_lexicon_tags_47 },
    { 0x3bbda6dd, SUC_lexicon_tags_38 },
    { 0x271da6de, SUC_lexicon_tags_66 },
    { 0xf65da6de, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbda6e6, SUC_lexicon_tags_13 },
    { 0x8ddda6e7, SUC_lexicon_tags_47 },
    { 0x679da6e8, SUC_lexicon_tags_15 },
    { 0xf75da6e8, SUC_lexicon_tags_21 },
    { 0x8f9da6ea, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bda6ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5da6f0, SUC_lexicon_tags_11 },
    { 0xb5fda6f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83da6f4, SUC_lexicon_tags_27 },
    { 0xca3da6f5, SUC_lexicon_tags_47 },
    { 0x15bda6f6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbda6fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2f3da6fd, SUC_lexicon_tags_25 },
    { 0xf3bda6fe, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xb1da700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefda702, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x34fda704, SUC_lexicon_tags_1 },
    { 0x763da705, SUC_lexicon_tags_25 },
    { 0xb1da706, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e5da708, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbda70b, SUC_lexicon_tags_37 },
    { 0x3a5da70c, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bda713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adda717, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191da71d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd67da71f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x207da721, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689da726, SUC_lexicon_tags_37 },
    { 0xf91da727, SUC_lexicon_tags_2 },
    { 0xf95da727, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x491da72a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb17da72d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedda738, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3da743, SUC_lexicon_tags_84 },
    { 0xefbda744, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca7da746, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd99da748, SUC_lexicon_tags_23 },
    { 0xcbbda748, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285da750, SUC_lexicon_tags_14 },
    { 0xf3fda751, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61da755, SUC_lexicon_tags_17 },
    { 0xb83da755, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x95dda758, SUC_lexicon_tags_25 },
    { 0x57dda758, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33dda75d, SUC_lexicon_tags_33 },
    { 0x11dda75e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdb3da760, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x55da762, SUC_lexicon_tags_25 },
    { 0x95dda762, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3da766, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985da76c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bda76f, SUC_lexicon_tags_45 },
    { 0x399da770, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xad9da772, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdda777, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x99da77a, SUC_lexicon_tags_27 },
    { 0xe63da77b, SUC_lexicon_tags_26 },
    { 0xfd7da77c, SUC_lexicon_tags_23 },
    { 0x967da77a, SUC_lexicon_tags_21 },
    { 0x217da77e, SUC_lexicon_tags_17 },
    { 0x7ffda77f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37da782, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105da786, SUC_lexicon_tags_15 },
    { 0xb1dda787, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259da792, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fda795, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5da798, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68dda79d, SUC_lexicon_tags_65 },
    { 0x28bda79e, SUC_lexicon_tags_12 },
    { 0xecbda79f, SUC_lexicon_tags_2 },
    { 0x955da7a0, SUC_lexicon_tags_0 },
    { 0x181da7a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x69da7a3, SUC_lexicon_tags_11 },
    { 0x9b3da7a4, SUC_lexicon_tags_37 },
    { 0x88fda7a3, SUC_lexicon_tags_60 },
    { 0xc1da7a5, SUC_lexicon_tags_7 },
    { 0xfc9da7a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9da7ac, SUC_lexicon_tags_33 },
    { 0xc17da7ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa19da7af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x683da7b1, SUC_lexicon_tags_33 },
    { 0x905da7b2, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85da7b6, SUC_lexicon_tags_25 },
    { 0x973da7b7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7da7bb, SUC_lexicon_tags_61 },
    { 0xce1da7bc, SUC_lexicon_tags_37 },
    { 0xf5bda7bb, SUC_lexicon_tags_37 },
    { 0xaedda7be, SUC_lexicon_tags_9 },
    { 0x61da7bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fda7c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461da7c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9da7cc, SUC_lexicon_tags_1 },
    { 0xc9fda7cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fdda7cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fda7d3, SUC_lexicon_tags_64 },
    { 0xe91da7d4, SUC_lexicon_tags_7 },
    { 0x8a5da7d5, SUC_lexicon_tags_38 },
    { 0xc5da7d3, SUC_lexicon_tags_148 },
    { 0x1b1da7d5, SUC_lexicon_tags_31 },
    { 0xb9bda7d8, SUC_lexicon_tags_25 },
    { 0x85bda7d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdda7dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53bda7de, SUC_lexicon_tags_13 },
    { 0xd1da7df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4adda7e1, SUC_lexicon_tags_49 },
    { 0x6c1da7e2, SUC_lexicon_tags_7 },
    { 0xf05da7e2, SUC_lexicon_tags_2 },
    { 0xc89da7e2, SUC_lexicon_tags_61 },
    { 0xa69da7e4, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x6f7da7e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7da7eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bda7ee, SUC_lexicon_tags_33 },
    { 0x2e7da7ef, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fda7f3, SUC_lexicon_tags_21 },
    { 0x63dda7f4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3ddda7f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bda7fa, SUC_lexicon_tags_6 },
    { 0x8ebda7fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dda7ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f5da801, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x60fda803, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639da807, SUC_lexicon_tags_15 },
    { 0xbd7da807, SUC_lexicon_tags_32 },
    { 0xd6dda809, SUC_lexicon_tags_11 },
    { 0x877da80a, SUC_lexicon_tags_12 },
    { 0x77bda80b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dda80f, SUC_lexicon_tags_19 },
    { 0x76dda810, SUC_lexicon_tags_7 },
    { 0xb25da811, SUC_lexicon_tags_3 },
    { 0xd21da812, SUC_lexicon_tags_85 },
    { 0x345da813, SUC_lexicon_tags_68 },
    { 0x627da814, SUC_lexicon_tags_15 },
    { 0x48dda813, SUC_lexicon_tags_11 },
    { 0x4d7da816, SUC_lexicon_tags_0 },
    { 0xc1dda816, SUC_lexicon_tags_7 },
    { 0xc07da816, SUC_lexicon_tags_11 },
    { 0x561da814, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9da81c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2cfda81e, SUC_lexicon_tags_11 },
    { 0x597da81e, SUC_lexicon_tags_30 },
    { 0x37da820, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbda824, SUC_lexicon_tags_34 },
    { 0x443da825, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3da828, SUC_lexicon_tags_61 },
    { 0xd63da829, SUC_lexicon_tags_17 },
    { 0x22bda828, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193da82f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fda833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x115da835, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfda838, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x459da83a, SUC_lexicon_tags_25 },
    { 0xdc5da83a, SUC_lexicon_tags_46 },
    { 0xc0bda83c, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73da843, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbda849, SUC_lexicon_tags_15 },
    { 0xa79da84a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45da854, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9da85a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x445da85d, SUC_lexicon_tags_68 },
    { 0x4d5da85e, SUC_lexicon_tags_46 },
    { 0xbabda85f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbda862, SUC_lexicon_tags_37 },
    { 0xd5da863, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bda866, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7da86d, SUC_lexicon_tags_84 },
    { 0xcf3da86d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x825da870, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33da873, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19da87f, SUC_lexicon_tags_45 },
    { 0x81da87f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x241da883, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953da887, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bda88c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb7fda88e, SUC_lexicon_tags_37 },
    { 0xaf3da88f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefda894, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xec9da896, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323da89b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe31da89d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99da8a1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7da8a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc65da8a6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97da8ab, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559da8af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97da8b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x263da8b7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebda8ba, SUC_lexicon_tags_13 },
    { 0x961da8ba, SUC_lexicon_tags_26 },
    { 0x975da8bc, SUC_lexicon_tags_68 },
    { 0xa81da8bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1da8c1, SUC_lexicon_tags_160 },
    { 0x1dfda8c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27da8c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x499da8ca, SUC_lexicon_tags_37 },
    { 0xeb5da8ca, SUC_lexicon_tags_12 },
    { 0x2bda8cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bda8d0, SUC_lexicon_tags_9 },
    { 0x681da8d1, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0x2dbda8d3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83da8d7, SUC_lexicon_tags_7 },
    { 0x17dda8d7, SUC_lexicon_tags_25 },
    { 0x939da8d8, SUC_lexicon_tags_32 },
    { 0x2c7da8d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9da8dd, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5da8e0, SUC_lexicon_tags_7 },
    { 0x82bda8e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dda8e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdda8ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa9dda8ec, SUC_lexicon_tags_27 },
    { 0x3d7da8ec, SUC_lexicon_tags_27 },
    { 0x64bda8ed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999da8f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3da8fa, SUC_lexicon_tags_38 },
    { 0x177da8fa, SUC_lexicon_tags_12 },
    { 0xe5fda8fc, SUC_lexicon_tags_7 },
    { 0xd9da8fd, SUC_lexicon_tags_7 },
    { 0xdf3da8fe, SUC_lexicon_tags_66 },
    { 0x775da8fd, SUC_lexicon_tags_2 },
    { 0x24fda900, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x945da902, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfffda906, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931da90a, SUC_lexicon_tags_47 },
    { 0x72fda90a, SUC_lexicon_tags_11 },
    { 0xdcbda90c, SUC_lexicon_tags_37 },
    { 0x1e1da90b, SUC_lexicon_tags_1 },
    { 0x7c5da90b, SUC_lexicon_tags_37 },
    { 0x4afda90f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dda915, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9da91c, SUC_lexicon_tags_39 },
    { 0x6dbda91c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2bda91f, SUC_lexicon_tags_4 },
    { 0xe97da91f, SUC_lexicon_tags_2 },
    { 0xe0dda921, SUC_lexicon_tags_11 },
    { 0xf2fda921, SUC_lexicon_tags_26 },
    { 0xb9dda922, SUC_lexicon_tags_11 },
    { 0x723da922, SUC_lexicon_tags_16 },
    { 0x829da925, SUC_lexicon_tags_23 },
    { 0xe85da924, SUC_lexicon_tags_68 },
    { 0xf23da922, SUC_lexicon_tags_15 },
    { 0x74dda928, SUC_lexicon_tags_0 },
    { 0xe5dda926, SUC_lexicon_tags_12 },
    { 0xfc5da924, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afda92d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa89da92f, SUC_lexicon_tags_23 },
    { 0x3c7da930, SUC_lexicon_tags_23 },
    { 0xd93da931, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbda934, SUC_lexicon_tags_13 },
    { 0x237da935, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc47da937, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10dda939, SUC_lexicon_tags_28 },
    { 0x69bda93a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bbda93c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249da942, SUC_lexicon_tags_17 },
    { 0x45dda943, SUC_lexicon_tags_33 },
    { 0x4ebda944, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2bda946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477da94c, SUC_lexicon_tags_7 },
    { 0x83bda94d, SUC_lexicon_tags_85 },
    { 0x10bda94e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x149da951, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bda956, SUC_lexicon_tags_48 },
    { 0xc4fda957, SUC_lexicon_tags_56 },
    { 0x603da958, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x705da95b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657da961, SUC_lexicon_tags_28 },
    { 0x66bda962, SUC_lexicon_tags_26 },
    { 0x941da961, SUC_lexicon_tags_8 },
    { 0xebdda961, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1da968, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf57da96a, SUC_lexicon_tags_37 },
    { 0x49da96b, SUC_lexicon_tags_1 },
    { 0x1a1da96a, SUC_lexicon_tags_16 },
    { 0xc1fda96d, SUC_lexicon_tags_46 },
    { 0x373da96d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x23dda970, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfda975, SUC_lexicon_tags_37 },
    { 0xe59da976, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579da97a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9da97d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fda983, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fda988, SUC_lexicon_tags_27 },
    { 0xe7bda989, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877da995, SUC_lexicon_tags_61 },
    { 0x2b3da996, SUC_lexicon_tags_37 },
    { 0x3bbda997, SUC_lexicon_tags_11 },
    { 0x35dda998, SUC_lexicon_tags_2 },
    { 0x619da999, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf5bda99b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaedda99e, SUC_lexicon_tags_11 },
    { 0xd63da99f, SUC_lexicon_tags_9 },
    { 0xf79da9a0, SUC_lexicon_tags_25 },
    { 0xff7da9a0, SUC_lexicon_tags_15 },
    { 0x91dda99e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a9da9a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x21dda9a6, SUC_lexicon_tags_14 },
    { 0xba7da9a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09da9ab, SUC_lexicon_tags_12 },
    { 0x9c1da9ac, SUC_lexicon_tags_2 },
    { 0x89fda9ad, SUC_lexicon_tags_85 },
    { 0xc95da9ad, SUC_lexicon_tags_17 },
    { 0x1e9da9af, SUC_lexicon_tags_15 },
    { 0xc99da9af, SUC_lexicon_tags_27 },
    { 0x407da9b0, SUC_lexicon_tags_37 },
    { 0x35dda9b0, SUC_lexicon_tags_23 },
    { 0xbabda9b0, SUC_lexicon_tags_17 },
    { 0x13da9ab, SUC_lexicon_tags_47 },
    { 0x2fdda9ac, SUC_lexicon_tags_47 },
    { 0xc7dda9b5, SUC_lexicon_tags_87 },
    { 0x53da9ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1b7da9b9, SUC_lexicon_tags_45 },
    { 0xd39da9ba, SUC_lexicon_tags_31 },
    { 0xe55da9bb, SUC_lexicon_tags_46 },
    { 0x795da9bb, SUC_lexicon_tags_12 },
    { 0x22dda9bb, SUC_lexicon_tags_12 },
    { 0x7edda9be, SUC_lexicon_tags_12 },
    { 0x4abda9bc, SUC_lexicon_tags_0 },
    { 0xfd9da9c0, SUC_lexicon_tags_7 },
    { 0xde5da9c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7efda9c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d3da9c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bda9c7, SUC_lexicon_tags_92 },
    { 0xa15da9c7, SUC_lexicon_tags_15 },
    { 0x3e3da9c7, SUC_lexicon_tags_23 },
    { 0x393da9ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523da9ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57da9d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ddda9d7, SUC_lexicon_tags_37 },
    { 0x283da9d8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7da9db, SUC_lexicon_tags_2 },
    { 0x76bda9dc, SUC_lexicon_tags_117 },
    { 0x111da9db, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221da9e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5da9eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3da9ef, SUC_lexicon_tags_2 },
    { 0xc3bda9ef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb87da9f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63fda9f4, SUC_lexicon_tags_45 },
    { 0xd11da9f5, SUC_lexicon_tags_19 },
    { 0xb5dda9f6, SUC_lexicon_tags_42 },
    { 0x567da9f7, SUC_lexicon_tags_7 },
    { 0x8efda9f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x481da9fa, SUC_lexicon_tags_8 },
    { 0xdffda9fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdaa05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdabdaa07, SUC_lexicon_tags_17 },
    { 0xed7daa08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6a5daa0a, SUC_lexicon_tags_43 },
    { 0xa83daa0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6fdaa0d, SUC_lexicon_tags_12 },
    { 0xc6bdaa0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfdaa12, SUC_lexicon_tags_19 },
    { 0xbcbdaa13, SUC_lexicon_tags_10 },
    { 0xacddaa12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63daa17, SUC_lexicon_tags_14 },
    { 0xe3ddaa17, SUC_lexicon_tags_60 },
    { 0x58fdaa19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bdaa1c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bdaa21, SUC_lexicon_tags_14 },
    { 0x3c5daa22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaa7daa24, SUC_lexicon_tags_7 },
    { 0x371daa25, SUC_lexicon_tags_82 },
    { 0x947daa24, SUC_lexicon_tags_67 },
    { 0xb67daa27, SUC_lexicon_tags_25 },
    { 0xa63daa26, SUC_lexicon_tags_14 },
    { 0xde1daa29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x85bdaa2c, SUC_lexicon_tags_60 },
    { 0x711daa2d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151daa32, SUC_lexicon_tags_12 },
    { 0x205daa33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x565daa35, SUC_lexicon_tags_10 },
    { 0xf3daa36, SUC_lexicon_tags_5 },
    { 0x9cddaa36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17daa3e, SUC_lexicon_tags_17 },
    { 0xedfdaa3e, SUC_lexicon_tags_37 },
    { 0xae5daa3f, SUC_lexicon_tags_25 },
    { 0x79ddaa41, SUC_lexicon_tags_25 },
    { 0x837daa42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x777daa45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdaa48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b7daa4a, SUC_lexicon_tags_4 },
    { 0x547daa4b, SUC_lexicon_tags_14 },
    { 0xe7fdaa4c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03daa4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387daa53, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ddaa56, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ddaa59, SUC_lexicon_tags_70 },
    { 0xec9daa5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bdaa5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeaddaa5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bdaa65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fdaa6a, SUC_lexicon_tags_25 },
    { 0x3c1daa6b, SUC_lexicon_tags_2 },
    { 0x505daa6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89daa74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x791daa77, SUC_lexicon_tags_30 },
    { 0x223daa78, SUC_lexicon_tags_0 },
    { 0x397daa78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fdaa7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1daa83, SUC_lexicon_tags_38 },
    { 0x587daa83, SUC_lexicon_tags_19 },
    { 0xf8fdaa85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x497daa87, SUC_lexicon_tags_172 },
    { 0xbe1daa88, SUC_lexicon_tags_23 },
    { 0x3d9daa89, SUC_lexicon_tags_63 },
    { 0x237daa8a, SUC_lexicon_tags_14 },
    { 0x703daa8b, SUC_lexicon_tags_7 },
    { 0x7fdaa8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5daa91, SUC_lexicon_tags_46 },
    { 0x845daa92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29daa99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3daa9c, SUC_lexicon_tags_23 },
    { 0x691daa9c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcfddaa9f, SUC_lexicon_tags_11 },
    { 0x7dddaaa0, SUC_lexicon_tags_12 },
    { 0x2e1daaa1, SUC_lexicon_tags_2 },
    { 0x547daa9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9daaa5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xc23daaa7, SUC_lexicon_tags_37 },
    { 0x6cbdaaa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32ddaaaa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9daaad, SUC_lexicon_tags_11 },
    { 0x6c1daaad, SUC_lexicon_tags_31 },
    { 0x5e7daaad, SUC_lexicon_tags_1 },
    { 0xc5ddaaae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa27daab3, SUC_lexicon_tags_15 },
    { 0x4f3daab4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2c9daab6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x115daab9, SUC_lexicon_tags_12 },
    { 0x4abdaaba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4efdaabc, SUC_lexicon_tags_17 },
    { 0x37fdaabd, SUC_lexicon_tags_60 },
    { 0x3d3daabe, SUC_lexicon_tags_7 },
    { 0x8e1daabf, SUC_lexicon_tags_28 },
    { 0x33ddaabf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x981daac2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffdaacb, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0xf5bdaacd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa61daacf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11daad8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x7bfdaada, SUC_lexicon_tags_15 },
    { 0xae9daadb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x597daade, SUC_lexicon_tags_14 },
    { 0x6d1daade, SUC_lexicon_tags_17 },
    { 0x635daae0, SUC_lexicon_tags_35 },
    { 0x18ddaae0, SUC_lexicon_tags_0 },
    { 0x549daae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcddaae4, SUC_lexicon_tags_27 },
    { 0x5b9daae4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x531daae8, SUC_lexicon_tags_17 },
    { 0xf53daae8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x22ddaaeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1daaf1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x173daaf3, SUC_lexicon_tags_7 },
    { 0x357daaf4, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251daaf8, SUC_lexicon_tags_17 },
    { 0xaefdaaf8, SUC_lexicon_tags_26 },
    { 0x6b9daafa, SUC_lexicon_tags_7 },
    { 0xa79daafb, SUC_lexicon_tags_17 },
    { 0x8e1daafc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x253daafe, SUC_lexicon_tags_12 },
    { 0xb5fdaaff, SUC_lexicon_tags_37 },
    { 0xa25dab00, SUC_lexicon_tags_8 },
    { 0xe1ddaafe, SUC_lexicon_tags_17 },
    { 0x8e3dab02, SUC_lexicon_tags_5 },
    { 0x77ddab01, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x369dab06, SUC_lexicon_tags_17 },
    { 0x9e3dab07, SUC_lexicon_tags_19 },
    { 0x9cddab08, SUC_lexicon_tags_12 },
    { 0x4f5dab08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bddab0d, SUC_lexicon_tags_82 },
    { 0x923dab0d, SUC_lexicon_tags_10 },
    { 0xeebdab0f, SUC_lexicon_tags_30 },
    { 0x79fdab10, SUC_lexicon_tags_2 },
    { 0x2c5dab0f, SUC_lexicon_tags_27 },
    { 0x3dddab12, SUC_lexicon_tags_45 },
    { 0xf51dab11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb85dab15, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3d3dab17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe47dab19, SUC_lexicon_tags_87 },
    { 0x807dab1a, SUC_lexicon_tags_28 },
    { 0xa2bdab19, SUC_lexicon_tags_54 },
    { 0xbbfdab1a, SUC_lexicon_tags_1 },
    { 0x471dab1d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25dab24, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf69dab26, SUC_lexicon_tags_1 },
    { 0xe99dab27, SUC_lexicon_tags_14 },
    { 0x1abdab26, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61dab2b, SUC_lexicon_tags_0 },
    { 0x5dddab2b, SUC_lexicon_tags_21 },
    { 0xe8bdab2d, SUC_lexicon_tags_23 },
    { 0x1f3dab2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfdab31, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7dab3a, SUC_lexicon_tags_37 },
    { 0xf6ddab3b, SUC_lexicon_tags_15 },
    { 0xf3fdab3c, SUC_lexicon_tags_14 },
    { 0xc43dab3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6c5dab3f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x771dab42, SUC_lexicon_tags_1 },
    { 0x68ddab43, SUC_lexicon_tags_55 },
    { 0x355dab44, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xddab47, SUC_lexicon_tags_11 },
    { 0x57fdab48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x78ddab4b, SUC_lexicon_tags_2 },
    { 0xeb7dab4c, SUC_lexicon_tags_0 },
    { 0x18ddab4d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1efdab4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0fdab51, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1dab55, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3dab5a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825dab65, SUC_lexicon_tags_63 },
    { 0x751dab66, SUC_lexicon_tags_8 },
    { 0xf1ddab67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49dab6a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1dab6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215dab74, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661dab7b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79dab7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x985dab81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5dab84, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadfdab86, SUC_lexicon_tags_7 },
    { 0x35bdab86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7addab89, SUC_lexicon_tags_10 },
    { 0xfa5dab8a, SUC_lexicon_tags_18 },
    { 0x865dab8a, SUC_lexicon_tags_95 },
    { 0x3c7dab8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1dab92, SUC_lexicon_tags_7 },
    { 0x727dab93, SUC_lexicon_tags_7 },
    { 0x129dab94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7dab97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c9dab99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5dab9c, SUC_lexicon_tags_6 },
    { 0x9a7dab9d, SUC_lexicon_tags_2 },
    { 0x88ddab9e, SUC_lexicon_tags_12 },
    { 0x74fdab9e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bdaba4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15daba7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xb4fdaba9, SUC_lexicon_tags_9 },
    { 0xeabdaba9, SUC_lexicon_tags_27 },
    { 0xb69daba9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159dabb1, SUC_lexicon_tags_8 },
    { 0x43ddabb1, SUC_lexicon_tags_15 },
    { 0x1e3dabb2, SUC_lexicon_tags_53 },
    { 0x5a5dabb2, SUC_lexicon_tags_31 },
    { 0x59bdabb3, SUC_lexicon_tags_37 },
    { 0xb15dabb5, SUC_lexicon_tags_12 },
    { 0xbd7dabb5, SUC_lexicon_tags_15 },
    { 0x84ddabb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fdabbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaebdabbe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x615dabc1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fdabcb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x49ddabcd, SUC_lexicon_tags_32 },
    { 0x127dabce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x391dabd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdabd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x91dabd6, SUC_lexicon_tags_21 },
    { 0x38bdabd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x293dabda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1dabdd, SUC_lexicon_tags_43 },
    { 0x83fdabdd, SUC_lexicon_tags_11 },
    { 0x943dabdf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x459dabe1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9dabe5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x91bdabe7, SUC_lexicon_tags_15 },
    { 0x37ddabe7, SUC_lexicon_tags_56 },
    { 0x89bdabe9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05dabef, SUC_lexicon_tags_44 },
    { 0x6e1dabf0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa23dabf2, SUC_lexicon_tags_0 },
    { 0x829dabf2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x509dabf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x447dabf8, SUC_lexicon_tags_48 },
    { 0x2b7dabf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155dabfd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3dac01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f9dac03, SUC_lexicon_tags_37 },
    { 0x397dac04, SUC_lexicon_tags_45 },
    { 0x45ddac04, SUC_lexicon_tags_82 },
    { 0x4eddac05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5dac0d, SUC_lexicon_tags_48 },
    { 0xa27dac0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ddac11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb57dac13, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bdac17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bdac1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x86bdac1d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1dac21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3dac24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227dac28, SUC_lexicon_tags_0 },
    { 0xbaddac29, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abdac2c, SUC_lexicon_tags_7 },
    { 0x86bdac2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23dac34, SUC_lexicon_tags_1 },
    { 0x145dac35, SUC_lexicon_tags_28 },
    { 0x3b1dac36, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349dac3f, SUC_lexicon_tags_11 },
    { 0x175dac40, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd43dac42, SUC_lexicon_tags_49 },
    { 0xaf7dac42, SUC_lexicon_tags_39 },
    { 0xd03dac44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddac47, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x895dac49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x86fdac4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ddac55, SUC_lexicon_tags_12 },
    { 0x2e5dac55, SUC_lexicon_tags_45 },
    { 0xf41dac56, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf79dac59, SUC_lexicon_tags_65 },
    { 0x473dac59, SUC_lexicon_tags_7 },
    { 0x3b3dac5b, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xf6fdac5d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fdac64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153dac6b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x315dac6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fdac73, SUC_lexicon_tags_11 },
    { 0x1b9dac74, SUC_lexicon_tags_15 },
    { 0x953dac74, SUC_lexicon_tags_10 },
    { 0x29fdac75, SUC_lexicon_tags_25 },
    { 0xaf5dac77, SUC_lexicon_tags_47 },
    { 0x961dac75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689dac80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fdac87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fdac8e, SUC_lexicon_tags_28 },
    { 0x86fdac8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x659dac91, SUC_lexicon_tags_74 },
    { 0x9c3dac92, SUC_lexicon_tags_60 },
    { 0x317dac93, SUC_lexicon_tags_25 },
    { 0x949dac93, SUC_lexicon_tags_63 },
    { 0xbefdac94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x275dac98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeefdac9b, SUC_lexicon_tags_28 },
    { 0x7d5dac9c, SUC_lexicon_tags_12 },
    { 0xc55dac9d, SUC_lexicon_tags_14 },
    { 0x8cddac9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x635daca0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafdaca3, SUC_lexicon_tags_87 },
    { 0xa9fdaca4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbdaca9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dddacae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ddacb4, SUC_lexicon_tags_49 },
    { 0xc4fdacb4, SUC_lexicon_tags_19 },
    { 0x837dacb6, SUC_lexicon_tags_12 },
    { 0x6d9dacb5, SUC_lexicon_tags_29 },
    { 0xad1dacb6, SUC_lexicon_tags_55 },
    { 0x5b7dacb9, SUC_lexicon_tags_80 },
    { 0xbc1dacba, SUC_lexicon_tags_25 },
    { 0x563dacb8, SUC_lexicon_tags_11 },
    { 0x90fdacbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2c1dacbe, SUC_lexicon_tags_0 },
    { 0xdc9dacbe, SUC_lexicon_tags_82 },
    { 0x7ebdacc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcabdacc3, SUC_lexicon_tags_12 },
    { 0x2dddacc4, SUC_lexicon_tags_63 },
    { 0x3a1dacc4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9dacc8, SUC_lexicon_tags_10 },
    { 0xf3bdacc9, SUC_lexicon_tags_37 },
    { 0xf13dacca, SUC_lexicon_tags_17 },
    { 0x271daccb, SUC_lexicon_tags_7 },
    { 0xfc7daccb, SUC_lexicon_tags_65 },
    { 0xaa1daccd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x175daccf, SUC_lexicon_tags_17 },
    { 0x9f5dacd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x23fdacd2, SUC_lexicon_tags_7 },
    { 0x8d1dacd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05dacd9, SUC_lexicon_tags_0 },
    { 0xc1fdacda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d5dacdc, SUC_lexicon_tags_31 },
    { 0x9cddacdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4ebdacdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa79dace1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x161dace3, SUC_lexicon_tags_29 },
    { 0x2bddace4, SUC_lexicon_tags_42 },
    { 0x835dace5, SUC_lexicon_tags_1057 },
    { 0xfdbdace6, SUC_lexicon_tags_15 },
    { 0xe3bdace5, SUC_lexicon_tags_1 },
    { 0x2d9dace8, SUC_lexicon_tags_17 },
    { 0xf77dace9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadbdaceb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dacef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795dacf8, SUC_lexicon_tags_8 },
    { 0xf4bdacf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c3dacfb, SUC_lexicon_tags_11 },
    { 0x3b3dacfc, SUC_lexicon_tags_1 },
    { 0x375dacfd, SUC_lexicon_tags_23 },
    { 0x197dacfb, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xf29dad00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5dad05, SUC_lexicon_tags_17 },
    { 0x7cbdad06, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1dad09, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fddad0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39dad10, SUC_lexicon_tags_27 },
    { 0x44ddad11, SUC_lexicon_tags_10 },
    { 0x82bdad12, SUC_lexicon_tags_11 },
    { 0x529dad12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x331dad15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b9dad17, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1dad1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9addad1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47dad21, SUC_lexicon_tags_17 },
    { 0xb39dad22, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131dad2c, SUC_lexicon_tags_15 },
    { 0xef5dad2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dad31, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1dad36, SUC_lexicon_tags_15 },
    { 0x61bdad37, SUC_lexicon_tags_62 },
    { 0x793dad38, SUC_lexicon_tags_12 },
    { 0x6a1dad39, SUC_lexicon_tags_37 },
    { 0x9bfdad3a, SUC_lexicon_tags_220 },
    { 0xfd9dad3b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3addad40, SUC_lexicon_tags_63 },
    { 0xe43dad41, SUC_lexicon_tags_0 },
    { 0x57dad42, SUC_lexicon_tags_9 },
    { 0x5c7dad43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853dad48, SUC_lexicon_tags_17 },
    { 0x615dad49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337dad4d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe0fdad4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3dad53, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09dad5a, SUC_lexicon_tags_47 },
    { 0x68ddad5b, SUC_lexicon_tags_25 },
    { 0x6a1dad5c, SUC_lexicon_tags_17 },
    { 0xebddad5d, SUC_lexicon_tags_11 },
    { 0xa89dad5e, SUC_lexicon_tags_1 },
    { 0xbabdad5f, SUC_lexicon_tags_46 },
    { 0x891dad60, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fdad63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x743dad66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffdad69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c7dad6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bdad6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bbdad71, SUC_lexicon_tags_17 },
    { 0xc49dad71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbdad81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1dad84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5dad89, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x251dad8c, SUC_lexicon_tags_28 },
    { 0xea3dad8c, SUC_lexicon_tags_34 },
    { 0x49ddad8c, SUC_lexicon_tags_47 },
    { 0x4b1dad8f, SUC_lexicon_tags_27 },
    { 0xcaddad90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ddad93, SUC_lexicon_tags_0 },
    { 0x8a1dad94, SUC_lexicon_tags_25 },
    { 0x3bbdad95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fdad9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12bdad9c, SUC_lexicon_tags_31 },
    { 0x2bdad9d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdada5, SUC_lexicon_tags_0 },
    { 0x899dada6, SUC_lexicon_tags_0 },
    { 0x51dada7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x943dadaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f5dadac, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1dadb1, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0x763dadb3, SUC_lexicon_tags_47 },
    { 0xa9dadb4, SUC_lexicon_tags_17 },
    { 0xd6ddadb5, SUC_lexicon_tags_7 },
    { 0x7d3dadb3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x53ddadb8, SUC_lexicon_tags_60 },
    { 0xce9dadb9, SUC_lexicon_tags_60 },
    { 0x58fdadba, SUC_lexicon_tags_28 },
    { 0xeb7dadbb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fdadbf, SUC_lexicon_tags_12 },
    { 0xdabdadbf, SUC_lexicon_tags_12 },
    { 0x72ddadc1, SUC_lexicon_tags_8 },
    { 0xc2fdadc2, SUC_lexicon_tags_11 },
    { 0xbf7dadc0, SUC_lexicon_tags_27 },
    { 0x89dadc4, SUC_lexicon_tags_37 },
    { 0xe3ddadc5, SUC_lexicon_tags_33 },
    { 0xbd7dadc5, SUC_lexicon_tags_56 },
    { 0xf0ddadc7, SUC_lexicon_tags_37 },
    { 0xe9dadc4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967dadcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3dadcf, SUC_lexicon_tags_15 },
    { 0x5a3dadcf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdadd3, SUC_lexicon_tags_55 },
    { 0x9abdadd4, SUC_lexicon_tags_25 },
    { 0x3a1dadd4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd57dadd7, SUC_lexicon_tags_25 },
    { 0xe4bdadd7, SUC_lexicon_tags_37 },
    { 0x159dadd8, SUC_lexicon_tags_8 },
    { 0xaa7dadd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9dade0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x531dade2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3dade5, SUC_lexicon_tags_68 },
    { 0xea3dade6, SUC_lexicon_tags_12 },
    { 0xf3bdade7, SUC_lexicon_tags_10 },
    { 0x4d9dade8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fddadec, SUC_lexicon_tags_10 },
    { 0x125dadec, SUC_lexicon_tags_7 },
    { 0x297dadee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe49dadf0, SUC_lexicon_tags_12 },
    { 0x71fdadf1, SUC_lexicon_tags_9 },
    { 0x895dadf2, SUC_lexicon_tags_11 },
    { 0x1c5dadf3, SUC_lexicon_tags_10 },
    { 0xf85dadf0, SUC_lexicon_tags_15 },
    { 0x3ebdadf1, SUC_lexicon_tags_8 },
    { 0xf53dadf6, SUC_lexicon_tags_31 },
    { 0x585dadf6, SUC_lexicon_tags_79 },
    { 0xd89dadf8, SUC_lexicon_tags_2 },
    { 0xc8fdadf9, SUC_lexicon_tags_11 },
    { 0xe01dadf6, SUC_lexicon_tags_12 },
    { 0x56fdadfb, SUC_lexicon_tags_15 },
    { 0x283dadfb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba5dadfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dae02, SUC_lexicon_tags_7 },
    { 0xcffdae02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe65dae05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fdae11, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcddae14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3dae18, SUC_lexicon_tags_37 },
    { 0xb7fdae18, SUC_lexicon_tags_38 },
    { 0xc03dae19, SUC_lexicon_tags_11 },
    { 0x6bbdae19, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161dae25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3dae2b, SUC_lexicon_tags_29 },
    { 0xb51dae2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x559dae2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5dae37, SUC_lexicon_tags_0 },
    { 0xd73dae38, SUC_lexicon_tags_43 },
    { 0xef9dae39, SUC_lexicon_tags_25 },
    { 0x273dae39, SUC_lexicon_tags_2 },
    { 0xd0ddae39, SUC_lexicon_tags_37 },
    { 0x517dae3c, SUC_lexicon_tags_104 },
    { 0xe63dae3d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31fdae3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7dae42, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xae9dae44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb09dae46, SUC_lexicon_tags_35 },
    { 0xb03dae46, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95dae4b, SUC_lexicon_tags_17 },
    { 0x7b3dae4b, SUC_lexicon_tags_47 },
    { 0x469dae4b, SUC_lexicon_tags_62 },
    { 0x737dae4e, SUC_lexicon_tags_25 },
    { 0x95fdae4f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fdae53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97dae58, SUC_lexicon_tags_17 },
    { 0xe59dae59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe4fdae5b, SUC_lexicon_tags_6 },
    { 0x9b1dae5c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9dae5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1dae65, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb93dae67, SUC_lexicon_tags_39 },
    { 0x99ddae67, SUC_lexicon_tags_4 },
    { 0xfbddae67, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaf7dae6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfdae6e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ddae73, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf77dae75, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ddae78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bdae7b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe87dae7d, SUC_lexicon_tags_19 },
    { 0x53dae7e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8bdae80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1dae87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703dae8d, SUC_lexicon_tags_17 },
    { 0xa57dae8d, SUC_lexicon_tags_26 },
    { 0x785dae8f, SUC_lexicon_tags_45 },
    { 0x389dae8d, SUC_lexicon_tags_37 },
    { 0xc3bdae91, SUC_lexicon_tags_60 },
    { 0x7e7dae8e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xccddae95, SUC_lexicon_tags_22 },
    { 0xebbdae96, SUC_lexicon_tags_15 },
    { 0x1f5dae95, SUC_lexicon_tags_15 },
    { 0x6cbdae97, SUC_lexicon_tags_26 },
    { 0x2bddae96, SUC_lexicon_tags_82 },
    { 0x2e1dae98, SUC_lexicon_tags_37 },
    { 0xbafdae98, SUC_lexicon_tags_47 },
    { 0xf07dae9c, SUC_lexicon_tags_32 },
    { 0xf5bdae9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355daea1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x82bdaea3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43daea9, SUC_lexicon_tags_27 },
    { 0x9d9daea9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7f1daeac, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x30fdaeae, SUC_lexicon_tags_14 },
    { 0xba1daeae, SUC_lexicon_tags_60 },
    { 0xd33daeb0, SUC_lexicon_tags_3 },
    { 0xcf1daeae, SUC_lexicon_tags_1 },
    { 0xe55daeb1, SUC_lexicon_tags_29 },
    { 0xf53daeb3, SUC_lexicon_tags_1 },
    { 0xedbdaeb4, SUC_lexicon_tags_25 },
    { 0x83fdaeb3, SUC_lexicon_tags_27 },
    { 0x239daeb6, SUC_lexicon_tags_1 },
    { 0x4c1daeb7, SUC_lexicon_tags_17 },
    { 0xddddaeb3, SUC_lexicon_tags_11 },
    { 0x9addaeb9, SUC_lexicon_tags_33 },
    { 0xd35daeb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85daebe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87daec1, SUC_lexicon_tags_49 },
    { 0x7cddaec2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x233daec5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x37fdaec8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891daece, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabdaed2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebfdaed4, SUC_lexicon_tags_1 },
    { 0x60bdaed5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fdaee4, SUC_lexicon_tags_8 },
    { 0xbefdaee5, SUC_lexicon_tags_27 },
    { 0x4addaee5, SUC_lexicon_tags_60 },
    { 0x403daee7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297daeeb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7daef0, SUC_lexicon_tags_30 },
    { 0x2fdaef1, SUC_lexicon_tags_7 },
    { 0x2fdaef2, SUC_lexicon_tags_2 },
    { 0xac1daef2, SUC_lexicon_tags_45 },
    { 0xa97daef2, SUC_lexicon_tags_32 },
    { 0x8f3daef4, SUC_lexicon_tags_25 },
    { 0x157daef3, SUC_lexicon_tags_63 },
    { 0x2fddaef7, SUC_lexicon_tags_12 },
    { 0x4cfdaef7, SUC_lexicon_tags_25 },
    { 0x2c5daef1, SUC_lexicon_tags_17 },
    { 0x1f9daef6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1daefe, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993daf02, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23daf06, SUC_lexicon_tags_1 },
    { 0xc61daf06, SUC_lexicon_tags_15 },
    { 0x535daf08, SUC_lexicon_tags_25 },
    { 0xc37daf08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb63daf0b, SUC_lexicon_tags_11 },
    { 0x325daf0c, SUC_lexicon_tags_48 },
    { 0x9d9daf0b, SUC_lexicon_tags_47 },
    { 0x2e9daf0e, SUC_lexicon_tags_37 },
    { 0x43bdaf0e, SUC_lexicon_tags_15 },
    { 0x4e1daf0e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebdaf1b, SUC_lexicon_tags_1 },
    { 0x7cfdaf1c, SUC_lexicon_tags_25 },
    { 0x247daf1d, SUC_lexicon_tags_23 },
    { 0xb67daf1b, SUC_lexicon_tags_7 },
    { 0xfc1daf1f, SUC_lexicon_tags_57 },
    { 0xcc7daf1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefdaf23, SUC_lexicon_tags_17 },
    { 0x4eddaf23, SUC_lexicon_tags_43 },
    { 0x9e7daf23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7a7daf27, SUC_lexicon_tags_49 },
    { 0xbc1daf28, SUC_lexicon_tags_15 },
    { 0x5d9daf28, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9a7daf2b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1daf2e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdaf31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39daf34, SUC_lexicon_tags_60 },
    { 0x8d1daf34, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x941daf37, SUC_lexicon_tags_37 },
    { 0x29daf38, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bdaf3b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x147daf3d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x27fdaf3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9daf43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39daf46, SUC_lexicon_tags_15 },
    { 0xbbdaf47, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xb53daf49, SUC_lexicon_tags_7 },
    { 0xc85daf4a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9daf50, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd27daf52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x563daf54, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3daf59, SUC_lexicon_tags_66 },
    { 0xf8bdaf59, SUC_lexicon_tags_10 },
    { 0xd7fdaf59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27daf5d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x943daf5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fdaf62, SUC_lexicon_tags_10 },
    { 0x3a1daf63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc99daf65, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x343daf68, SUC_lexicon_tags_33 },
    { 0x98fdaf69, SUC_lexicon_tags_82 },
    { 0xed3daf69, SUC_lexicon_tags_12 },
    { 0x2cbdaf6b, SUC_lexicon_tags_10 },
    { 0xdd7daf6c, SUC_lexicon_tags_0 },
    { 0xa2ddaf6d, SUC_lexicon_tags_45 },
    { 0x2f1daf6a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59daf71, SUC_lexicon_tags_25 },
    { 0xc21daf72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8a9daf74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7daf77, SUC_lexicon_tags_25 },
    { 0x7e1daf77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3daf7d, SUC_lexicon_tags_12 },
    { 0xb91daf7d, SUC_lexicon_tags_12 },
    { 0x5b3daf7f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fdaf82, SUC_lexicon_tags_37 },
    { 0x69fdaf83, SUC_lexicon_tags_7 },
    { 0x961daf84, SUC_lexicon_tags_55 },
    { 0x729daf83, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x23ddaf87, SUC_lexicon_tags_11 },
    { 0xc67daf88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9daf8c, SUC_lexicon_tags_37 },
    { 0xbfddaf8c, SUC_lexicon_tags_37 },
    { 0xb55daf8e, SUC_lexicon_tags_2 },
    { 0x9efdaf8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe63daf91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x423daf93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9daf98, SUC_lexicon_tags_25 },
    { 0x3c7daf99, SUC_lexicon_tags_140 },
    { 0xbfddaf98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ddaf9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5fdafa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadbdafa2, SUC_lexicon_tags_28 },
    { 0x8a9dafa3, SUC_lexicon_tags_47 },
    { 0x2f9dafa4, SUC_lexicon_tags_25 },
    { 0x7bdafa5, SUC_lexicon_tags_9 },
    { 0xf3fdafa4, SUC_lexicon_tags_37 },
    { 0x55ddafa7, SUC_lexicon_tags_28 },
    { 0x165dafa7, SUC_lexicon_tags_29 },
    { 0xca7dafa8, SUC_lexicon_tags_7 },
    { 0xab7dafaa, SUC_lexicon_tags_174 },
    { 0xa69dafab, SUC_lexicon_tags_7 },
    { 0xb4fdafa5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdafaf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x743dafb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3fdafb3, SUC_lexicon_tags_3 },
    { 0x8dddafb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaa9dafb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x13bdafb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3addafba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541dafc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25dafc3, SUC_lexicon_tags_1 },
    { 0xbebdafc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83dafc7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89dafcd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667dafd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fdafd7, SUC_lexicon_tags_15 },
    { 0xa55dafd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83bdafda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281dafe3, SUC_lexicon_tags_87 },
    { 0xea9dafe4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9dafe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4efdafe9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x541dafeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11daff1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abdaff4, SUC_lexicon_tags_17 },
    { 0xfe5daff4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfdaffd, SUC_lexicon_tags_9 },
    { 0x94ddaffe, SUC_lexicon_tags_37 },
    { 0xecddafff, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993db005, SUC_lexicon_tags_43 },
    { 0x775db006, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x99fdb008, SUC_lexicon_tags_37 },
    { 0x2d9db009, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x535db00b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fdb014, SUC_lexicon_tags_28 },
    { 0x581db015, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2f9db017, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1db01a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931db023, SUC_lexicon_tags_10 },
    { 0x295db024, SUC_lexicon_tags_15 },
    { 0x473db024, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63db029, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5db02f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbbdb031, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe1db033, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83db038, SUC_lexicon_tags_26 },
    { 0xbc3db038, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x373db03b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3db042, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71fdb044, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fdb048, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fdb04c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x285db04e, SUC_lexicon_tags_12 },
    { 0x71ddb04f, SUC_lexicon_tags_25 },
    { 0xd09db050, SUC_lexicon_tags_55 },
    { 0xc6ddb04f, SUC_lexicon_tags_47 },
    { 0x8ebdb04f, SUC_lexicon_tags_12 },
    { 0x8e1db04e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15bdb055, SUC_lexicon_tags_25 },
    { 0x1ebdb056, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693db05a, SUC_lexicon_tags_27 },
    { 0xacbdb05a, SUC_lexicon_tags_37 },
    { 0xbc3db05c, SUC_lexicon_tags_179 },
    { 0xc6bdb05d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3db065, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5db069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bdb06d, SUC_lexicon_tags_31 },
    { 0x517db06d, SUC_lexicon_tags_1 },
    { 0xd63db06f, SUC_lexicon_tags_27 },
    { 0xb19db070, SUC_lexicon_tags_37 },
    { 0xbc3db071, SUC_lexicon_tags_1 },
    { 0x481db072, SUC_lexicon_tags_7 },
    { 0x937db070, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd65db075, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x503db078, SUC_lexicon_tags_21 },
    { 0xb9db079, SUC_lexicon_tags_17 },
    { 0xd23db07a, SUC_lexicon_tags_1 },
    { 0xc45db079, SUC_lexicon_tags_17 },
    { 0x2dbdb078, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfd1db07e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x683db080, SUC_lexicon_tags_82 },
    { 0x3cbdb080, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ddb085, SUC_lexicon_tags_15 },
    { 0x573db085, SUC_lexicon_tags_45 },
    { 0xe01db087, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x861db08a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfdb08f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefdb093, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ddb096, SUC_lexicon_tags_37 },
    { 0x78ddb097, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc6fdb099, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bdb0a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bdb0a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6fbdb0a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1db0ac, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bddb0b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173db0b7, SUC_lexicon_tags_26 },
    { 0xd1ddb0b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebdb0be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efdb0c2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1db0cc, SUC_lexicon_tags_43 },
    { 0xec7db0cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461db0d2, SUC_lexicon_tags_654 },
    { 0xfe5db0d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x741db0d6, SUC_lexicon_tags_25 },
    { 0x5c7db0d7, SUC_lexicon_tags_25 },
    { 0xd73db0d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7db0dd, SUC_lexicon_tags_12 },
    { 0x30bdb0dd, SUC_lexicon_tags_6 },
    { 0xe5db0df, SUC_lexicon_tags_3 },
    { 0xeeddb0e0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabdb0e5, SUC_lexicon_tags_25 },
    { 0x193db0e6, SUC_lexicon_tags_17 },
    { 0xe79db0e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9ebdb0e9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849db0ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63fdb0f0, SUC_lexicon_tags_11 },
    { 0xef3db0f0, SUC_lexicon_tags_40 },
    { 0xaa1db0f1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975db0f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffdb0fb, SUC_lexicon_tags_0 },
    { 0xc45db0fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafdb0ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fddb103, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fdb108, SUC_lexicon_tags_12 },
    { 0x715db109, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69db10d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x927db10f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61db113, SUC_lexicon_tags_24 },
    { 0xb2fdb114, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5db118, SUC_lexicon_tags_14 },
    { 0x99fdb118, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ddb11e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6c9db120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb31db122, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fdb126, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2fdb128, SUC_lexicon_tags_6 },
    { 0xa47db129, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bdb130, SUC_lexicon_tags_15 },
    { 0x11db130, SUC_lexicon_tags_17 },
    { 0x6b1db132, SUC_lexicon_tags_46 },
    { 0xe99db132, SUC_lexicon_tags_25 },
    { 0x7b3db131, SUC_lexicon_tags_32 },
    { 0x471db130, SUC_lexicon_tags_4 },
    { 0x3db136, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bdb13a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdddb13c, SUC_lexicon_tags_10 },
    { 0xab9db13c, SUC_lexicon_tags_16 },
    { 0x515db13e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x70ddb140, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35db143, SUC_lexicon_tags_14 },
    { 0x90ddb143, SUC_lexicon_tags_28 },
    { 0x175db145, SUC_lexicon_tags_54 },
    { 0xf27db143, SUC_lexicon_tags_27 },
    { 0x157db147, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5addb14b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x96fdb14d, SUC_lexicon_tags_43 },
    { 0x1ddb14d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35fdb150, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa77db152, SUC_lexicon_tags_47 },
    { 0x1b7db152, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7db156, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x853db158, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59db166, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43db16d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x567db170, SUC_lexicon_tags_11 },
    { 0xfa1db171, SUC_lexicon_tags_17 },
    { 0x78fdb172, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41db177, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x785db179, SUC_lexicon_tags_27 },
    { 0x719db17a, SUC_lexicon_tags_19 },
    { 0x1e5db17b, SUC_lexicon_tags_4 },
    { 0x2d5db17c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x289db17e, SUC_lexicon_tags_10 },
    { 0x1ebdb17f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x97ddb181, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fdb185, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x423db187, SUC_lexicon_tags_25 },
    { 0x19db188, SUC_lexicon_tags_47 },
    { 0x5a9db189, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x713db18b, SUC_lexicon_tags_87 },
    { 0xeb3db18b, SUC_lexicon_tags_37 },
    { 0x871db18c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9a1db18f, SUC_lexicon_tags_46 },
    { 0x429db18f, SUC_lexicon_tags_54 },
    { 0xb93db190, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x683db193, SUC_lexicon_tags_30 },
    { 0xe9ddb194, SUC_lexicon_tags_37 },
    { 0xc3bdb193, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x917db197, SUC_lexicon_tags_35 },
    { 0x9d9db198, SUC_lexicon_tags_1 },
    { 0x7f1db199, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x207db19c, SUC_lexicon_tags_11 },
    { 0xb6bdb19d, SUC_lexicon_tags_1 },
    { 0x4f3db19c, SUC_lexicon_tags_12 },
    { 0xf89db19f, SUC_lexicon_tags_19 },
    { 0x3c7db1a0, SUC_lexicon_tags_23 },
    { 0xd03db1a1, SUC_lexicon_tags_14 },
    { 0x2b9db19f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe65db1a4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x329db1a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35db1ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ddb1af, SUC_lexicon_tags_178 },
    { 0xf6fdb1b0, SUC_lexicon_tags_11 },
    { 0xf6bdb1b1, SUC_lexicon_tags_11 },
    { 0x9f3db1b0, SUC_lexicon_tags_7 },
    { 0xe7ddb1b2, SUC_lexicon_tags_11 },
    { 0x9ddb1af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9db1be, SUC_lexicon_tags_8 },
    { 0xffdb1bf, SUC_lexicon_tags_27 },
    { 0xe3ddb1c0, SUC_lexicon_tags_7 },
    { 0x98ddb1c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x179db1c4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffdb1c8, SUC_lexicon_tags_23 },
    { 0xcf3db1c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f3db1cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961db1d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe8fdb1d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf49db1d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fdb1da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75db1e8, SUC_lexicon_tags_25 },
    { 0xd77db1e9, SUC_lexicon_tags_12 },
    { 0x7addb1ea, SUC_lexicon_tags_44 },
    { 0x153db1e9, SUC_lexicon_tags_15 },
    { 0xd49db1ec, SUC_lexicon_tags_3 },
    { 0x5bbdb1ec, SUC_lexicon_tags_10 },
    { 0xa7ddb1e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fdb1f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3f9db1f6, SUC_lexicon_tags_68 },
    { 0xf27db1f6, SUC_lexicon_tags_63 },
    { 0x29db1f8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5fdb1fa, SUC_lexicon_tags_57 },
    { 0x7b7db1fb, SUC_lexicon_tags_21 },
    { 0xe57db1fb, SUC_lexicon_tags_11 },
    { 0xbc9db1fa, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fdb204, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5db208, SUC_lexicon_tags_14 },
    { 0xae1db209, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249db20f, SUC_lexicon_tags_37 },
    { 0xa89db210, SUC_lexicon_tags_17 },
    { 0x66ddb20f, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xf21db213, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd49db215, SUC_lexicon_tags_6 },
    { 0x855db216, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca7db218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f5db21a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fdb222, SUC_lexicon_tags_14 },
    { 0x9f9db223, SUC_lexicon_tags_3 },
    { 0x639db223, SUC_lexicon_tags_17 },
    { 0x3e9db222, SUC_lexicon_tags_17 },
    { 0x1f3db223, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x547db228, SUC_lexicon_tags_6 },
    { 0x94ddb228, SUC_lexicon_tags_19 },
    { 0x627db22a, SUC_lexicon_tags_17 },
    { 0xf2ddb22b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fdb22e, SUC_lexicon_tags_47 },
    { 0xfbbdb22f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb47db231, SUC_lexicon_tags_29 },
    { 0x96bdb232, SUC_lexicon_tags_26 },
    { 0xa1ddb233, SUC_lexicon_tags_37 },
    { 0xf39db234, SUC_lexicon_tags_25 },
    { 0x161db235, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bddb238, SUC_lexicon_tags_12 },
    { 0xc83db238, SUC_lexicon_tags_25 },
    { 0xe4ddb23a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bdb240, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5db248, SUC_lexicon_tags_187 },
    { 0xa1bdb249, SUC_lexicon_tags_37 },
    { 0xa11db24a, SUC_lexicon_tags_17 },
    { 0x84ddb24b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6ffdb24d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x87db250, SUC_lexicon_tags_7 },
    { 0x28ddb250, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afdb255, SUC_lexicon_tags_0 },
    { 0xbfddb256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae5db258, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe15db25a, SUC_lexicon_tags_8 },
    { 0x181db25b, SUC_lexicon_tags_24 },
    { 0xaabdb25c, SUC_lexicon_tags_77 },
    { 0x5c5db25d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x43db260, SUC_lexicon_tags_14 },
    { 0x387db261, SUC_lexicon_tags_37 },
    { 0xabddb262, SUC_lexicon_tags_12 },
    { 0xa8ddb261, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bdb268, SUC_lexicon_tags_29 },
    { 0xa45db268, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x771db26b, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3db26f, SUC_lexicon_tags_11 },
    { 0xbf1db270, SUC_lexicon_tags_49 },
    { 0xf7fdb270, SUC_lexicon_tags_13 },
    { 0x861db272, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ddb274, SUC_lexicon_tags_12 },
    { 0xa11db275, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xfe1db277, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1bdb279, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bdb27c, SUC_lexicon_tags_37 },
    { 0xdb5db27c, SUC_lexicon_tags_14 },
    { 0xa35db27e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61fdb280, SUC_lexicon_tags_25 },
    { 0x3b3db280, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc73db283, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765db28a, SUC_lexicon_tags_9 },
    { 0xc13db28b, SUC_lexicon_tags_10 },
    { 0x993db28b, SUC_lexicon_tags_9 },
    { 0xd99db28a, SUC_lexicon_tags_38 },
    { 0xb3db28e, SUC_lexicon_tags_92 },
    { 0x6f5db28f, SUC_lexicon_tags_7 },
    { 0x7e9db28d, SUC_lexicon_tags_14 },
    { 0x7d7db28f, SUC_lexicon_tags_8 },
    { 0xdefdb28c, SUC_lexicon_tags_12 },
    { 0xb89db28d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749db29c, SUC_lexicon_tags_38 },
    { 0xabbdb29c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7db2a1, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0x899db2a4, SUC_lexicon_tags_46 },
    { 0x1db2a5, SUC_lexicon_tags_17 },
    { 0xa4fdb2a6, SUC_lexicon_tags_11 },
    { 0x5cbdb2a6, SUC_lexicon_tags_30 },
    { 0x265db2a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x891db2aa, SUC_lexicon_tags_61 },
    { 0x9d5db2ab, SUC_lexicon_tags_11 },
    { 0x39db2ac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1db2b4, SUC_lexicon_tags_57 },
    { 0xd01db2b5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x19bdb2b8, SUC_lexicon_tags_14 },
    { 0xa7bdb2b9, SUC_lexicon_tags_25 },
    { 0x16fdb2ba, SUC_lexicon_tags_31 },
    { 0xfa1db2b8, SUC_lexicon_tags_1 },
    { 0xeb3db2b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75db2bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165db2c8, SUC_lexicon_tags_79 },
    { 0xaf1db2c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ddb2cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x779db2cf, SUC_lexicon_tags_7 },
    { 0xc19db2cf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6b1db2d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547db2d6, SUC_lexicon_tags_33 },
    { 0x5c9db2d7, SUC_lexicon_tags_7 },
    { 0x7efdb2d8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295db2df, SUC_lexicon_tags_15 },
    { 0xbefdb2e0, SUC_lexicon_tags_60 },
    { 0xf07db2e1, SUC_lexicon_tags_19 },
    { 0xacfdb2e1, SUC_lexicon_tags_11 },
    { 0xc25db2e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x425db2e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1db2e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55db2eb, SUC_lexicon_tags_29 },
    { 0xdb7db2ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5dbdb2ee, SUC_lexicon_tags_2 },
    { 0x84bdb2ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1db2f2, SUC_lexicon_tags_26 },
    { 0x169db2f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x577db2f6, SUC_lexicon_tags_17 },
    { 0x435db2f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9ddb2f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3db303, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23db307, SUC_lexicon_tags_162 },
    { 0x599db308, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fdb30c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5db311, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xaebdb313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189db317, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x507db319, SUC_lexicon_tags_35 },
    { 0x879db319, SUC_lexicon_tags_25 },
    { 0x6a1db31a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfdb31f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3cddb321, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfddb324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7db327, SUC_lexicon_tags_11 },
    { 0x3f1db327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddb32d, SUC_lexicon_tags_60 },
    { 0xe2bdb32e, SUC_lexicon_tags_19 },
    { 0x7bfdb32f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7db333, SUC_lexicon_tags_37 },
    { 0x897db333, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x45db336, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x22ddb338, SUC_lexicon_tags_17 },
    { 0x11fdb339, SUC_lexicon_tags_47 },
    { 0xbafdb338, SUC_lexicon_tags_17 },
    { 0xa39db338, SUC_lexicon_tags_11 },
    { 0x821db33c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x355db33e, SUC_lexicon_tags_37 },
    { 0xf37db33e, SUC_lexicon_tags_2 },
    { 0xf0bdb33e, SUC_lexicon_tags_11 },
    { 0x415db33e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x549db344, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x41fdb346, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd53db348, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcf5db34a, SUC_lexicon_tags_17 },
    { 0xda9db34a, SUC_lexicon_tags_45 },
    { 0x277db34c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7db350, SUC_lexicon_tags_47 },
    { 0xda9db351, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x379db353, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbdb35a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc31db35c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1db35f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e5db361, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x871db364, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ddb367, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdfddb369, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9ddb36b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ddb378, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x747db37a, SUC_lexicon_tags_12 },
    { 0x753db37b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5db386, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x601db389, SUC_lexicon_tags_15 },
    { 0x7e7db38a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xca9db38d, SUC_lexicon_tags_15 },
    { 0x1dddb38e, SUC_lexicon_tags_17 },
    { 0x717db38e, SUC_lexicon_tags_37 },
    { 0x407db390, SUC_lexicon_tags_35 },
    { 0xe4fdb38d, SUC_lexicon_tags_15 },
    { 0xec5db38d, SUC_lexicon_tags_12 },
    { 0x219db393, SUC_lexicon_tags_2 },
    { 0xdabdb391, SUC_lexicon_tags_25 },
    { 0x90bdb395, SUC_lexicon_tags_35 },
    { 0xec1db396, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635db39b, SUC_lexicon_tags_25 },
    { 0xbcfdb39c, SUC_lexicon_tags_37 },
    { 0x3c5db39c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbdb3a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efdb3a6, SUC_lexicon_tags_36 },
    { 0x7cddb3a7, SUC_lexicon_tags_29 },
    { 0xf19db3a6, SUC_lexicon_tags_0 },
    { 0xa75db3a9, SUC_lexicon_tags_33 },
    { 0x885db3aa, SUC_lexicon_tags_70 },
    { 0xb6bdb3ab, SUC_lexicon_tags_29 },
    { 0x6f5db3a7, SUC_lexicon_tags_37 },
    { 0x58ddb3a9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x483db3b0, SUC_lexicon_tags_38 },
    { 0x423db3b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7fdb3b3, SUC_lexicon_tags_7 },
    { 0x63fdb3b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9db3b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cbdb3b9, SUC_lexicon_tags_37 },
    { 0xcf3db3ba, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb9fdb3bc, SUC_lexicon_tags_17 },
    { 0x7b9db3bd, SUC_lexicon_tags_45 },
    { 0x493db3bd, SUC_lexicon_tags_29 },
    { 0x9dbdb3bf, SUC_lexicon_tags_15 },
    { 0x6c5db3c0, SUC_lexicon_tags_47 },
    { 0x645db3c0, SUC_lexicon_tags_63 },
    { 0x8f3db3bd, SUC_lexicon_tags_37 },
    { 0xd5db3c3, SUC_lexicon_tags_1 },
    { 0xd57db3c4, SUC_lexicon_tags_33 },
    { 0x67ddb3c5, SUC_lexicon_tags_37 },
    { 0xebdb3c6, SUC_lexicon_tags_37 },
    { 0x7e3db3c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bdb3ca, SUC_lexicon_tags_9 },
    { 0x6b9db3cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbdb3cd, SUC_lexicon_tags_12 },
    { 0xe7ddb3ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bdb3d2, SUC_lexicon_tags_6 },
    { 0xa7db3d2, SUC_lexicon_tags_32 },
    { 0x53ddb3d4, SUC_lexicon_tags_7 },
    { 0x873db3d5, SUC_lexicon_tags_23 },
    { 0x1b7db3d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x863db3d8, SUC_lexicon_tags_8 },
    { 0x2b5db3d9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd7ddb3db, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183db3e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9db3e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x891db3e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf23db3e8, SUC_lexicon_tags_28 },
    { 0x4dddb3e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb4fdb3eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5db3ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fdb3f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fdb3f6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761db400, SUC_lexicon_tags_55 },
    { 0xa95db401, SUC_lexicon_tags_13 },
    { 0x89fdb401, SUC_lexicon_tags_17 },
    { 0x5dbdb400, SUC_lexicon_tags_37 },
    { 0x80fdb404, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9db408, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95db40b, SUC_lexicon_tags_108 },
    { 0xadddb40c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3db412, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc8ddb414, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x73fdb416, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47db41a, SUC_lexicon_tags_65 },
    { 0xbc3db41b, SUC_lexicon_tags_7 },
    { 0xf55db41c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1fdb41e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6addb420, SUC_lexicon_tags_11 },
    { 0xf9bdb421, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cddb424, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x625db426, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xad7db428, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543db42e, SUC_lexicon_tags_33 },
    { 0xb19db42f, SUC_lexicon_tags_3 },
    { 0xa2ddb42e, SUC_lexicon_tags_11 },
    { 0xed5db431, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519db436, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145db43a, SUC_lexicon_tags_15 },
    { 0x459db43a, SUC_lexicon_tags_37 },
    { 0x4c1db43c, SUC_lexicon_tags_38 },
    { 0xc65db43b, SUC_lexicon_tags_1 },
    { 0x2dddb43e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ddb442, SUC_lexicon_tags_14 },
    { 0xfc3db442, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc89db445, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8d9db447, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9fdb449, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bdb44c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ddb44f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187db453, SUC_lexicon_tags_37 },
    { 0xf09db454, SUC_lexicon_tags_25 },
    { 0x205db453, SUC_lexicon_tags_4 },
    { 0xa51db456, SUC_lexicon_tags_49 },
    { 0xc4ddb455, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ddb45a, SUC_lexicon_tags_25 },
    { 0x295db45b, SUC_lexicon_tags_37 },
    { 0xafbdb45c, SUC_lexicon_tags_11 },
    { 0xab7db45c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951db467, SUC_lexicon_tags_37 },
    { 0x6ffdb468, SUC_lexicon_tags_30 },
    { 0xfdb469, SUC_lexicon_tags_17 },
    { 0xc4ddb46a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e5db46c, SUC_lexicon_tags_68 },
    { 0x9c3db46d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7db470, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x273db473, SUC_lexicon_tags_29 },
    { 0xc99db474, SUC_lexicon_tags_26 },
    { 0x827db475, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45db47c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91db482, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xfefdb485, SUC_lexicon_tags_1 },
    { 0xc49db485, SUC_lexicon_tags_29 },
    { 0x449db487, SUC_lexicon_tags_82 },
    { 0x1cbdb487, SUC_lexicon_tags_45 },
    { 0x1abdb489, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bdb48d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddb491, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05db495, SUC_lexicon_tags_19 },
    { 0x629db496, SUC_lexicon_tags_15 },
    { 0xc1db497, SUC_lexicon_tags_77 },
    { 0x8a5db498, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x881db49b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eddb49f, SUC_lexicon_tags_1 },
    { 0xa5ddb4a0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fdb4a5, SUC_lexicon_tags_37 },
    { 0x195db4a6, SUC_lexicon_tags_15 },
    { 0x151db4a6, SUC_lexicon_tags_12 },
    { 0x37ddb4a8, SUC_lexicon_tags_4 },
    { 0x201db4a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5db4ad, SUC_lexicon_tags_38 },
    { 0x6c7db4ae, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x693db4b1, SUC_lexicon_tags_8 },
    { 0x413db4b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4bdb4b4, SUC_lexicon_tags_8 },
    { 0x587db4b4, SUC_lexicon_tags_17 },
    { 0x2dfdb4b5, SUC_lexicon_tags_172 },
    { 0xe3bdb4b4, SUC_lexicon_tags_15 },
    { 0xbbddb4b5, SUC_lexicon_tags_31 },
    { 0x3b9db4b9, SUC_lexicon_tags_2 },
    { 0x98fdb4b9, SUC_lexicon_tags_49 },
    { 0x70bdb4b9, SUC_lexicon_tags_66 },
    { 0x44ddb4bc, SUC_lexicon_tags_8 },
    { 0x69fdb4b5, SUC_lexicon_tags_26 },
    { 0xbffdb4bb, SUC_lexicon_tags_13 },
    { 0x4e9db4b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fdb4c6, SUC_lexicon_tags_38 },
    { 0xb9ddb4c7, SUC_lexicon_tags_82 },
    { 0x1c5db4c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x58fdb4ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x765db4cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbc9db4cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7db4d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbdb4d5, SUC_lexicon_tags_37 },
    { 0x275db4d6, SUC_lexicon_tags_10 },
    { 0xbe7db4d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bdb4da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7db4de, SUC_lexicon_tags_12 },
    { 0xea5db4df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fdb4e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bdb4e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf1fdb4ea, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29db4ee, SUC_lexicon_tags_37 },
    { 0x3ddb4ee, SUC_lexicon_tags_29 },
    { 0x3fdb4f0, SUC_lexicon_tags_7 },
    { 0x4ddb4ef, SUC_lexicon_tags_37 },
    { 0x1bddb4f0, SUC_lexicon_tags_13 },
    { 0x467db4f3, SUC_lexicon_tags_21 },
    { 0x8f5db4f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57db4f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xddddb4fa, SUC_lexicon_tags_7 },
    { 0x447db4fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefdb4fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x25fdb500, SUC_lexicon_tags_17 },
    { 0xea3db500, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5cfdb503, SUC_lexicon_tags_3 },
    { 0x611db504, SUC_lexicon_tags_1 },
    { 0xf9ddb505, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc11db507, SUC_lexicon_tags_15 },
    { 0x4e5db508, SUC_lexicon_tags_33 },
    { 0x237db508, SUC_lexicon_tags_33 },
    { 0x4c3db509, SUC_lexicon_tags_12 },
    { 0x4b9db50b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1bddb50d, SUC_lexicon_tags_38 },
    { 0xfe9db50d, SUC_lexicon_tags_37 },
    { 0x9f7db50d, SUC_lexicon_tags_46 },
    { 0xf2fdb510, SUC_lexicon_tags_1 },
    { 0x72bdb511, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x54ddb513, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x837db515, SUC_lexicon_tags_1 },
    { 0x2d5db515, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5ebdb518, SUC_lexicon_tags_37 },
    { 0x69db519, SUC_lexicon_tags_43 },
    { 0x885db519, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfa9db51c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x73fdb51e, SUC_lexicon_tags_149 },
    { 0x7d9db51f, SUC_lexicon_tags_147 },
    { 0x85db520, SUC_lexicon_tags_37 },
    { 0x805db520, SUC_lexicon_tags_15 },
    { 0x1c3db520, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9db527, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5db52a, SUC_lexicon_tags_27 },
    { 0x6b3db52a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561db530, SUC_lexicon_tags_65 },
    { 0x3f3db530, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57db534, SUC_lexicon_tags_1 },
    { 0x603db534, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4dfdb537, SUC_lexicon_tags_8 },
    { 0xf27db538, SUC_lexicon_tags_48 },
    { 0x223db539, SUC_lexicon_tags_15 },
    { 0x33bdb539, SUC_lexicon_tags_37 },
    { 0xafdb53b, SUC_lexicon_tags_38 },
    { 0x68ddb537, SUC_lexicon_tags_32 },
    { 0x21ddb53a, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11db540, SUC_lexicon_tags_0 },
    { 0x27ddb540, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x51db543, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b9db545, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95db548, SUC_lexicon_tags_33 },
    { 0x3efdb549, SUC_lexicon_tags_37 },
    { 0xc09db54a, SUC_lexicon_tags_37 },
    { 0x2e1db548, SUC_lexicon_tags_12 },
    { 0x69db54a, SUC_lexicon_tags_11 },
    { 0xf3db54a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5f9db54f, SUC_lexicon_tags_37 },
    { 0xd79db54f, SUC_lexicon_tags_15 },
    { 0xfb5db54f, SUC_lexicon_tags_17 },
    { 0x999db552, SUC_lexicon_tags_49 },
    { 0x803db551, SUC_lexicon_tags_118 },
    { 0xfeddb554, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x895db557, SUC_lexicon_tags_6 },
    { 0x30bdb558, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5db560, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x539db562, SUC_lexicon_tags_21 },
    { 0xc77db563, SUC_lexicon_tags_124 },
    { 0xf8bdb564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdb569, SUC_lexicon_tags_15 },
    { 0xe3db56a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67bdb56c, SUC_lexicon_tags_25 },
    { 0x261db56d, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x8b1db56f, SUC_lexicon_tags_7 },
    { 0x623db570, SUC_lexicon_tags_25 },
    { 0xdffdb571, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11db577, SUC_lexicon_tags_0 },
    { 0x29bdb578, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bdb57c, SUC_lexicon_tags_1 },
    { 0x8d7db57d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x70bdb57f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fdb583, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3db585, SUC_lexicon_tags_39 },
    { 0x2d7db586, SUC_lexicon_tags_27 },
    { 0x78fdb586, SUC_lexicon_tags_23 },
    { 0xa7ddb586, SUC_lexicon_tags_7 },
    { 0xb7db587, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe6fdb58b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7db590, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d7db592, SUC_lexicon_tags_27 },
    { 0x73bdb593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903db598, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x257db59b, SUC_lexicon_tags_68 },
    { 0xdf7db59c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabdb5a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d1db5a6, SUC_lexicon_tags_66 },
    { 0x333db5a7, SUC_lexicon_tags_11 },
    { 0xde5db5a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bdb5ac, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fdb5af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1db5b2, SUC_lexicon_tags_11 },
    { 0x235db5b3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x5e7db5b5, SUC_lexicon_tags_7 },
    { 0x2f3db5b6, SUC_lexicon_tags_66 },
    { 0xa2fdb5b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddb5bb, SUC_lexicon_tags_25 },
    { 0x5e7db5bc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647db5c3, SUC_lexicon_tags_29 },
    { 0x879db5c4, SUC_lexicon_tags_41 },
    { 0xd3bdb5c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fdb5c8, SUC_lexicon_tags_15 },
    { 0xa09db5c8, SUC_lexicon_tags_7 },
    { 0xde5db5ca, SUC_lexicon_tags_32 },
    { 0x75fdb5ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fdb5d0, SUC_lexicon_tags_25 },
    { 0x695db5d1, SUC_lexicon_tags_15 },
    { 0xe65db5d2, SUC_lexicon_tags_14 },
    { 0x7db5d1, SUC_lexicon_tags_45 },
    { 0x437db5d4, SUC_lexicon_tags_37 },
    { 0x68fdb5d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde7db5d7, SUC_lexicon_tags_15 },
    { 0x861db5d8, SUC_lexicon_tags_24 },
    { 0x973db5d9, SUC_lexicon_tags_10 },
    { 0x88bdb5da, SUC_lexicon_tags_45 },
    { 0xfddb5db, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaeddb5e2, SUC_lexicon_tags_27 },
    { 0x3a9db5e3, SUC_lexicon_tags_37 },
    { 0x219db5e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9db5eb, SUC_lexicon_tags_25 },
    { 0x2e5db5eb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebdb5ef, SUC_lexicon_tags_2 },
    { 0xeb1db5f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x281db5f3, SUC_lexicon_tags_23 },
    { 0x785db5f4, SUC_lexicon_tags_7 },
    { 0xdf1db5f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e7db5f7, SUC_lexicon_tags_14 },
    { 0xc6ddb5f8, SUC_lexicon_tags_17 },
    { 0xb37db5f9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xcc5db5fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931db604, SUC_lexicon_tags_1 },
    { 0x9fddb605, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9db608, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ddb60d, SUC_lexicon_tags_62 },
    { 0xa9fdb60e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7db611, SUC_lexicon_tags_77 },
    { 0x17ddb612, SUC_lexicon_tags_39 },
    { 0x4a5db611, SUC_lexicon_tags_15 },
    { 0xa77db614, SUC_lexicon_tags_23 },
    { 0x911db611, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fdb619, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x38ddb61b, SUC_lexicon_tags_26 },
    { 0xc4fdb61c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ddb620, SUC_lexicon_tags_1 },
    { 0x207db620, SUC_lexicon_tags_2 },
    { 0x195db621, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4d1db624, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5db629, SUC_lexicon_tags_26 },
    { 0xe1ddb62a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x803db62d, SUC_lexicon_tags_23 },
    { 0xb1db62d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb1db630, SUC_lexicon_tags_33 },
    { 0x16bdb630, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x191db633, SUC_lexicon_tags_1 },
    { 0x24ddb634, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17db639, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d7db63b, SUC_lexicon_tags_6 },
    { 0xf6fdb63c, SUC_lexicon_tags_65 },
    { 0x283db63b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9db641, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xad1db643, SUC_lexicon_tags_17 },
    { 0x529db644, SUC_lexicon_tags_43 },
    { 0xed3db645, SUC_lexicon_tags_1 },
    { 0x5c9db645, SUC_lexicon_tags_33 },
    { 0xa99db645, SUC_lexicon_tags_26 },
    { 0x537db648, SUC_lexicon_tags_11 },
    { 0xc81db648, SUC_lexicon_tags_27 },
    { 0xfa3db645, SUC_lexicon_tags_12 },
    { 0xf93db64b, SUC_lexicon_tags_23 },
    { 0xe9ddb64c, SUC_lexicon_tags_45 },
    { 0x17bdb647, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7db654, SUC_lexicon_tags_2 },
    { 0x4c1db655, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x885db658, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7db65b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefdb661, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x871db663, SUC_lexicon_tags_43 },
    { 0xca9db664, SUC_lexicon_tags_13 },
    { 0x5d9db663, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ddb66c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ddb66f, SUC_lexicon_tags_12 },
    { 0xdaddb670, SUC_lexicon_tags_18 },
    { 0xe71db671, SUC_lexicon_tags_1 },
    { 0x957db672, SUC_lexicon_tags_7 },
    { 0x839db672, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27db676, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11db67c, SUC_lexicon_tags_9 },
    { 0x1a7db67d, SUC_lexicon_tags_31 },
    { 0x2a5db67d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9db682, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbdb689, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fdb68c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ddb68f, SUC_lexicon_tags_57 },
    { 0x205db690, SUC_lexicon_tags_57 },
    { 0x6fdb691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869db696, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa53db698, SUC_lexicon_tags_19 },
    { 0xa5db698, SUC_lexicon_tags_41 },
    { 0x697db69a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x133db69c, SUC_lexicon_tags_15 },
    { 0x5fbdb69d, SUC_lexicon_tags_27 },
    { 0xd65db69e, SUC_lexicon_tags_11 },
    { 0xd9ddb69d, SUC_lexicon_tags_33 },
    { 0x163db6a0, SUC_lexicon_tags_29 },
    { 0x889db69e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbddb6a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee5db6a8, SUC_lexicon_tags_75 },
    { 0x22ddb6a9, SUC_lexicon_tags_1 },
    { 0x57bdb6aa, SUC_lexicon_tags_2 },
    { 0x907db6aa, SUC_lexicon_tags_38 },
    { 0x9e7db6ac, SUC_lexicon_tags_14 },
    { 0x18bdb6ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ddb6b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2abdb6b4, SUC_lexicon_tags_13 },
    { 0x96bdb6b4, SUC_lexicon_tags_0 },
    { 0xb85db6b4, SUC_lexicon_tags_56 },
    { 0x873db6b7, SUC_lexicon_tags_63 },
    { 0xbb1db6b8, SUC_lexicon_tags_87 },
    { 0x55db6b8, SUC_lexicon_tags_17 },
    { 0x653db6ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24fdb6bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x369db6be, SUC_lexicon_tags_63 },
    { 0xca9db6bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x39fdb6c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xef1db6c4, SUC_lexicon_tags_31 },
    { 0xf6fdb6c5, SUC_lexicon_tags_7 },
    { 0xacddb6c6, SUC_lexicon_tags_28 },
    { 0xf39db6c7, SUC_lexicon_tags_29 },
    { 0xd2bdb6c7, SUC_lexicon_tags_25 },
    { 0xfc3db6c9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fdb6cc, SUC_lexicon_tags_29 },
    { 0x9c5db6cc, SUC_lexicon_tags_17 },
    { 0x9c3db6ce, SUC_lexicon_tags_12 },
    { 0xce3db6cf, SUC_lexicon_tags_0 },
    { 0x735db6d0, SUC_lexicon_tags_21 },
    { 0xf0ddb6d1, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eddb6d6, SUC_lexicon_tags_17 },
    { 0xab7db6d7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9db6e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9db6e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfdb6ec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xff5db6ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399db6f9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe7ddb6fb, SUC_lexicon_tags_61 },
    { 0x993db6fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2f5db6fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355db703, SUC_lexicon_tags_25 },
    { 0xb45db704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x681db706, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3db709, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc4ddb70b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffdb710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf25db712, SUC_lexicon_tags_14 },
    { 0xd6bdb713, SUC_lexicon_tags_151 },
    { 0x3cbdb712, SUC_lexicon_tags_60 },
    { 0x4efdb715, SUC_lexicon_tags_17 },
    { 0x2a1db715, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bdb719, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3db71e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc63db720, SUC_lexicon_tags_15 },
    { 0xd1db721, SUC_lexicon_tags_47 },
    { 0xb59db721, SUC_lexicon_tags_11 },
    { 0x2c7db720, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe27db725, SUC_lexicon_tags_25 },
    { 0xfdfdb725, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6ffdb728, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x417db72b, SUC_lexicon_tags_7 },
    { 0x661db72c, SUC_lexicon_tags_12 },
    { 0x97ddb72d, SUC_lexicon_tags_12 },
    { 0xe8ddb72d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3db732, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ddb737, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe61db739, SUC_lexicon_tags_43 },
    { 0xdd5db73a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1db73d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x815db740, SUC_lexicon_tags_12 },
    { 0x717db741, SUC_lexicon_tags_14 },
    { 0x5abdb742, SUC_lexicon_tags_1 },
    { 0xaebdb743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5db746, SUC_lexicon_tags_37 },
    { 0x5e7db747, SUC_lexicon_tags_49 },
    { 0xf95db748, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bddb74e, SUC_lexicon_tags_19 },
    { 0xa2bdb74f, SUC_lexicon_tags_15 },
    { 0x23bdb74e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9db757, SUC_lexicon_tags_25 },
    { 0x4c9db758, SUC_lexicon_tags_37 },
    { 0x59fdb757, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803db75d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749db762, SUC_lexicon_tags_7 },
    { 0x303db762, SUC_lexicon_tags_30 },
    { 0xf89db764, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fdb769, SUC_lexicon_tags_12 },
    { 0x1e7db769, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65bdb76c, SUC_lexicon_tags_45 },
    { 0xd4bdb76d, SUC_lexicon_tags_7 },
    { 0x7c5db76e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37db771, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bdb774, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fdb777, SUC_lexicon_tags_25 },
    { 0xdc9db778, SUC_lexicon_tags_6 },
    { 0xedbdb778, SUC_lexicon_tags_25 },
    { 0xc85db77a, SUC_lexicon_tags_49 },
    { 0x3f7db77b, SUC_lexicon_tags_41 },
    { 0x261db77c, SUC_lexicon_tags_32 },
    { 0xeefdb77d, SUC_lexicon_tags_28 },
    { 0x52ddb77d, SUC_lexicon_tags_61 },
    { 0xa77db77f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19db785, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbdb78b, SUC_lexicon_tags_11 },
    { 0xb87db78c, SUC_lexicon_tags_6 },
    { 0xffddb78d, SUC_lexicon_tags_25 },
    { 0x8d9db78e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x445db790, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc69db792, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7db79b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb0bdb79d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327db7a6, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x443db7a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x67bdb7ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ddb7b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x91bdb7b4, SUC_lexicon_tags_47 },
    { 0x1c1db7b4, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x485db7b7, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5db7bc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7db7bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1cbdb7c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffdb7c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe1db7c7, SUC_lexicon_tags_49 },
    { 0x78fdb7c7, SUC_lexicon_tags_17 },
    { 0x43ddb7c9, SUC_lexicon_tags_7 },
    { 0x19fdb7ca, SUC_lexicon_tags_11 },
    { 0xcdddb7c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2c7db7cd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbddb7d1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafddb7d6, SUC_lexicon_tags_6 },
    { 0x6fddb7d7, SUC_lexicon_tags_60 },
    { 0xed3db7d7, SUC_lexicon_tags_66 },
    { 0xadbdb7d9, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffdb7dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdcddb7de, SUC_lexicon_tags_17 },
    { 0x6d7db7de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fddb7e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9db7f0, SUC_lexicon_tags_26 },
    { 0x6c5db7f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853db7f6, SUC_lexicon_tags_15 },
    { 0x8d5db7f7, SUC_lexicon_tags_11 },
    { 0x95ddb7f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77db7fb, SUC_lexicon_tags_65 },
    { 0xa37db7fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd39db7fe, SUC_lexicon_tags_15 },
    { 0x51db7ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x263db801, SUC_lexicon_tags_43 },
    { 0x2d1db801, SUC_lexicon_tags_11 },
    { 0x4a9db803, SUC_lexicon_tags_34 },
    { 0x995db802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81db808, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x45bdb80a, SUC_lexicon_tags_37 },
    { 0x569db80b, SUC_lexicon_tags_37 },
    { 0xa25db80c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xcd1db80e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15db813, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa2ddb815, SUC_lexicon_tags_63 },
    { 0x4ffdb815, SUC_lexicon_tags_7 },
    { 0xfc1db815, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x255db819, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bdb81e, SUC_lexicon_tags_37 },
    { 0x30ddb81e, SUC_lexicon_tags_74 },
    { 0x129db820, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe43db822, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287db82b, SUC_lexicon_tags_1 },
    { 0x1f5db82b, SUC_lexicon_tags_11 },
    { 0x52bdb82d, SUC_lexicon_tags_47 },
    { 0xc51db82b, SUC_lexicon_tags_0 },
    { 0x8c3db82f, SUC_lexicon_tags_2 },
    { 0x795db82f, SUC_lexicon_tags_2 },
    { 0xcb7db82d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x403db833, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bdb839, SUC_lexicon_tags_84 },
    { 0x717db83a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x68fdb83c, SUC_lexicon_tags_2 },
    { 0xb29db83c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7db840, SUC_lexicon_tags_95 },
    { 0x19bdb841, SUC_lexicon_tags_34 },
    { 0x9eddb842, SUC_lexicon_tags_7 },
    { 0x5bbdb843, SUC_lexicon_tags_37 },
    { 0x4f5db840, SUC_lexicon_tags_47 },
    { 0xcfbdb840, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9db848, SUC_lexicon_tags_27 },
    { 0xccddb848, SUC_lexicon_tags_62 },
    { 0xb17db84a, SUC_lexicon_tags_19 },
    { 0x699db84b, SUC_lexicon_tags_14 },
    { 0x649db84c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7db859, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2c7db85b, SUC_lexicon_tags_4 },
    { 0x315db85c, SUC_lexicon_tags_31 },
    { 0x189db85d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdb860, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe5ddb862, SUC_lexicon_tags_1 },
    { 0x795db863, SUC_lexicon_tags_0 },
    { 0x21fdb864, SUC_lexicon_tags_38 },
    { 0x395db864, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd2fdb867, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3db869, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2bddb86b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x867db86d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfdb871, SUC_lexicon_tags_17 },
    { 0x89bdb871, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bdb876, SUC_lexicon_tags_2 },
    { 0x225db877, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa97db879, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907db880, SUC_lexicon_tags_31 },
    { 0x895db881, SUC_lexicon_tags_12 },
    { 0x4d5db881, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3db886, SUC_lexicon_tags_37 },
    { 0xd79db887, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55db88d, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdb897, SUC_lexicon_tags_14 },
    { 0x29db898, SUC_lexicon_tags_1 },
    { 0xe27db898, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ddb89e, SUC_lexicon_tags_14 },
    { 0xc2ddb89f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9db8a3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x45fdb8a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x809db8a8, SUC_lexicon_tags_30 },
    { 0x8bbdb8a9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6cbdb8ab, SUC_lexicon_tags_14 },
    { 0xa4fdb8ac, SUC_lexicon_tags_1 },
    { 0x3f3db8ab, SUC_lexicon_tags_84 },
    { 0x77ddb8ad, SUC_lexicon_tags_12 },
    { 0xc35db8ae, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fdb8b4, SUC_lexicon_tags_8 },
    { 0xa6fdb8b5, SUC_lexicon_tags_68 },
    { 0xfdddb8b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1db8b9, SUC_lexicon_tags_31 },
    { 0x9fddb8b9, SUC_lexicon_tags_15 },
    { 0x679db8bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29db8be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1fddb8c0, SUC_lexicon_tags_33 },
    { 0x53db8c1, SUC_lexicon_tags_17 },
    { 0xdb5db8c2, SUC_lexicon_tags_17 },
    { 0x393db8c3, SUC_lexicon_tags_17 },
    { 0x9addb8c3, SUC_lexicon_tags_15 },
    { 0xf3ddb8c1, SUC_lexicon_tags_99 },
    { 0x3fbdb8c6, SUC_lexicon_tags_15 },
    { 0xc95db8c6, SUC_lexicon_tags_12 },
    { 0x7ffdb8c6, SUC_lexicon_tags_37 },
    { 0x1bddb8c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01db8cd, SUC_lexicon_tags_25 },
    { 0x4d3db8ce, SUC_lexicon_tags_17 },
    { 0xdf3db8cf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbdb8d2, SUC_lexicon_tags_44 },
    { 0x647db8d3, SUC_lexicon_tags_15 },
    { 0xd87db8d3, SUC_lexicon_tags_25 },
    { 0xfa3db8d5, SUC_lexicon_tags_25 },
    { 0x409db8d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bdb8d9, SUC_lexicon_tags_65 },
    { 0xa91db8da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7db8dd, SUC_lexicon_tags_63 },
    { 0x6cbdb8de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xabbdb8e0, SUC_lexicon_tags_11 },
    { 0x86fdb8e1, SUC_lexicon_tags_9 },
    { 0x5addb8e1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdc1db8e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743db8ed, SUC_lexicon_tags_14 },
    { 0xca7db8ee, SUC_lexicon_tags_1 },
    { 0x105db8ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7db8f7, SUC_lexicon_tags_15 },
    { 0x265db8f8, SUC_lexicon_tags_45 },
    { 0xa85db8f8, SUC_lexicon_tags_63 },
    { 0x663db8f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x769db8fc, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0xc15db8fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc5db900, SUC_lexicon_tags_0 },
    { 0xfb7db901, SUC_lexicon_tags_27 },
    { 0x79fdb902, SUC_lexicon_tags_37 },
    { 0xa37db903, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcf5db905, SUC_lexicon_tags_16 },
    { 0x875db906, SUC_lexicon_tags_160 },
    { 0x37fdb905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fdb90b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x339db90d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fdb911, SUC_lexicon_tags_1 },
    { 0xba9db912, SUC_lexicon_tags_54 },
    { 0x2dfdb913, SUC_lexicon_tags_25 },
    { 0xffddb913, SUC_lexicon_tags_23 },
    { 0xabbdb913, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x759db917, SUC_lexicon_tags_25 },
    { 0xabdb917, SUC_lexicon_tags_27 },
    { 0x567db919, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x70ddb91b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbdb91f, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbddb922, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801db929, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7db92c, SUC_lexicon_tags_10 },
    { 0xc55db92d, SUC_lexicon_tags_1 },
    { 0x505db92d, SUC_lexicon_tags_17 },
    { 0x5ddb92f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x325db931, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fdb93a, SUC_lexicon_tags_17 },
    { 0xf53db93b, SUC_lexicon_tags_2 },
    { 0x2e7db93a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbdb93e, SUC_lexicon_tags_17 },
    { 0xbf9db93f, SUC_lexicon_tags_47 },
    { 0x4d3db93f, SUC_lexicon_tags_38 },
    { 0x167db941, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abdb944, SUC_lexicon_tags_65 },
    { 0xe7db945, SUC_lexicon_tags_17 },
    { 0xbc7db946, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3db94d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8ddb94f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x245db951, SUC_lexicon_tags_1 },
    { 0x429db952, SUC_lexicon_tags_27 },
    { 0x41ddb953, SUC_lexicon_tags_34 },
    { 0xbc9db954, SUC_lexicon_tags_18 },
    { 0x2c1db951, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x321db957, SUC_lexicon_tags_27 },
    { 0x31fdb957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaddb95d, SUC_lexicon_tags_1 },
    { 0xd03db95d, SUC_lexicon_tags_16 },
    { 0x17ddb95f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9db965, SUC_lexicon_tags_25 },
    { 0x36ddb966, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdb96a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1db973, SUC_lexicon_tags_37 },
    { 0x2b3db974, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7db97b, SUC_lexicon_tags_12 },
    { 0xa87db97b, SUC_lexicon_tags_68 },
    { 0x4b5db97b, SUC_lexicon_tags_26 },
    { 0xdd3db97c, SUC_lexicon_tags_57 },
    { 0xaa9db97f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43db984, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537db989, SUC_lexicon_tags_17 },
    { 0xc63db98a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269db98f, SUC_lexicon_tags_11 },
    { 0xd17db990, SUC_lexicon_tags_47 },
    { 0xc69db991, SUC_lexicon_tags_7 },
    { 0xb6fdb992, SUC_lexicon_tags_17 },
    { 0xda9db992, SUC_lexicon_tags_12 },
    { 0xe8bdb994, SUC_lexicon_tags_15 },
    { 0x4dddb98f, SUC_lexicon_tags_60 },
    { 0xb83db996, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x85fdb998, SUC_lexicon_tags_2 },
    { 0xfcbdb999, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfdb99f, SUC_lexicon_tags_47 },
    { 0x8d5db9a0, SUC_lexicon_tags_29 },
    { 0xd1bdb9a1, SUC_lexicon_tags_27 },
    { 0x9b1db9a0, SUC_lexicon_tags_25 },
    { 0x8c7db9a3, SUC_lexicon_tags_1 },
    { 0x849db9a1, SUC_lexicon_tags_2 },
    { 0x6fdb9a5, SUC_lexicon_tags_1 },
    { 0x13db9a6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39db9b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bdb9b7, SUC_lexicon_tags_37 },
    { 0xc39db9b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xafddb9ba, SUC_lexicon_tags_7 },
    { 0x14bdb9ba, SUC_lexicon_tags_12 },
    { 0xaf5db9ba, SUC_lexicon_tags_11 },
    { 0x331db9bd, SUC_lexicon_tags_7 },
    { 0xea1db9bd, SUC_lexicon_tags_26 },
    { 0xcb1db9bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1db9c8, SUC_lexicon_tags_27 },
    { 0x21ddb9c9, SUC_lexicon_tags_9 },
    { 0xde5db9ca, SUC_lexicon_tags_12 },
    { 0x243db9c9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813db9d3, SUC_lexicon_tags_55 },
    { 0x323db9d4, SUC_lexicon_tags_12 },
    { 0x341db9d3, SUC_lexicon_tags_25 },
    { 0x3e5db9d6, SUC_lexicon_tags_38 },
    { 0x613db9d7, SUC_lexicon_tags_45 },
    { 0xfc1db9d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x673db9db, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739db9e1, SUC_lexicon_tags_17 },
    { 0x1abdb9e2, SUC_lexicon_tags_47 },
    { 0x20ddb9e2, SUC_lexicon_tags_16 },
    { 0xc8fdb9e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9db9ea, SUC_lexicon_tags_17 },
    { 0xe6bdb9eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9db9f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd5db9f3, SUC_lexicon_tags_25 },
    { 0x34ddb9f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5cfdb9f6, SUC_lexicon_tags_63 },
    { 0xe13db9f6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213db9fd, SUC_lexicon_tags_23 },
    { 0x1d5db9fe, SUC_lexicon_tags_23 },
    { 0x779db9fd, SUC_lexicon_tags_1 },
    { 0x2fbdba00, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe99dba02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71dba04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bdba08, SUC_lexicon_tags_25 },
    { 0xb31dba09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebdba0b, SUC_lexicon_tags_1 },
    { 0x99bdba0c, SUC_lexicon_tags_23 },
    { 0xfafdba0c, SUC_lexicon_tags_37 },
    { 0x2f7dba0e, SUC_lexicon_tags_2 },
    { 0x7c9dba0f, SUC_lexicon_tags_29 },
    { 0x80ddba0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97dba13, SUC_lexicon_tags_61 },
    { 0xf97dba14, SUC_lexicon_tags_79 },
    { 0xa03dba13, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95dba19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1dba1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4eddba1f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe27dba21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9dba27, SUC_lexicon_tags_146 },
    { 0xb45dba28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x40fdba2a, SUC_lexicon_tags_11 },
    { 0xcf9dba2b, SUC_lexicon_tags_37 },
    { 0x92fdba2c, SUC_lexicon_tags_17 },
    { 0xaffdba2d, SUC_lexicon_tags_61 },
    { 0xb91dba2e, SUC_lexicon_tags_25 },
    { 0x45bdba2e, SUC_lexicon_tags_37 },
    { 0xf79dba2f, SUC_lexicon_tags_2 },
    { 0xe93dba2f, SUC_lexicon_tags_68 },
    { 0xfffdba2d, SUC_lexicon_tags_17 },
    { 0xeaddba30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05dba38, SUC_lexicon_tags_39 },
    { 0x2c5dba39, SUC_lexicon_tags_1 },
    { 0x599dba39, SUC_lexicon_tags_12 },
    { 0xc99dba39, SUC_lexicon_tags_15 },
    { 0x1b7dba3c, SUC_lexicon_tags_2 },
    { 0x2a7dba3c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x7dba3f, SUC_lexicon_tags_7 },
    { 0xb19dba3f, SUC_lexicon_tags_19 },
    { 0x437dba41, SUC_lexicon_tags_39 },
    { 0x2f1dba42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bdba46, SUC_lexicon_tags_70 },
    { 0x911dba46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3bfdba49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb27dba4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8a9dba4d, SUC_lexicon_tags_36 },
    { 0x8cbdba4e, SUC_lexicon_tags_11 },
    { 0x6bfdba4f, SUC_lexicon_tags_61 },
    { 0xebddba50, SUC_lexicon_tags_19 },
    { 0xc21dba51, SUC_lexicon_tags_84 },
    { 0xdd5dba52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xde9dba54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f9dba56, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xe9ddba58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5dba5b, SUC_lexicon_tags_9 },
    { 0x799dba5c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb29dba5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac5dba61, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5f9dba63, SUC_lexicon_tags_37 },
    { 0x82bdba64, SUC_lexicon_tags_134 },
    { 0xbddba63, SUC_lexicon_tags_9 },
    { 0x9e1dba66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269dba6e, SUC_lexicon_tags_68 },
    { 0xd11dba6e, SUC_lexicon_tags_9 },
    { 0x579dba70, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371dba77, SUC_lexicon_tags_13 },
    { 0x29dba77, SUC_lexicon_tags_7 },
    { 0x787dba79, SUC_lexicon_tags_62 },
    { 0x61bdba79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69dba7d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71dba84, SUC_lexicon_tags_15 },
    { 0xfa1dba85, SUC_lexicon_tags_1 },
    { 0x29ddba86, SUC_lexicon_tags_4 },
    { 0x9afdba87, SUC_lexicon_tags_40 },
    { 0xf9ddba87, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bdba8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ddba8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdbfdba90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x433dba92, SUC_lexicon_tags_29 },
    { 0xa3fdba93, SUC_lexicon_tags_8 },
    { 0x26fdba94, SUC_lexicon_tags_45 },
    { 0x45fdba95, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9dba98, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x981dba9a, SUC_lexicon_tags_17 },
    { 0xba9dba9a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x22fdba9d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa29dba9f, SUC_lexicon_tags_15 },
    { 0xe93dbaa0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe13dbaa2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbdbaa7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6d3dbaa9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597dbaaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2ffdbab1, SUC_lexicon_tags_4 },
    { 0x6d7dbab2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf7dbab4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3dbabf, SUC_lexicon_tags_49 },
    { 0x6fddbac0, SUC_lexicon_tags_11 },
    { 0x757dbac1, SUC_lexicon_tags_68 },
    { 0x3b9dbac2, SUC_lexicon_tags_26 },
    { 0x2dfdbabf, SUC_lexicon_tags_53 },
    { 0x8cfdbac4, SUC_lexicon_tags_37 },
    { 0xefbdbac5, SUC_lexicon_tags_37 },
    { 0xde7dbac1, SUC_lexicon_tags_47 },
    { 0xb7fdbac7, SUC_lexicon_tags_57 },
    { 0x11fdbac4, SUC_lexicon_tags_36 },
    { 0xcf7dbac9, SUC_lexicon_tags_12 },
    { 0x469dbac8, SUC_lexicon_tags_14 },
    { 0x4e9dbacb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9dbad7, SUC_lexicon_tags_33 },
    { 0x9d5dbad8, SUC_lexicon_tags_22 },
    { 0x425dbad8, SUC_lexicon_tags_1 },
    { 0xa91dbada, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2a1dbadc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x65dbade, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc75dbae0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddbaee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717dbaf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fdbaf9, SUC_lexicon_tags_68 },
    { 0x6d7dbaf9, SUC_lexicon_tags_37 },
    { 0x7c7dbafb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23dbb01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d9dbb03, SUC_lexicon_tags_87 },
    { 0x17fdbb03, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x933dbb07, SUC_lexicon_tags_9 },
    { 0x27fdbb07, SUC_lexicon_tags_25 },
    { 0xf17dbb07, SUC_lexicon_tags_104 },
    { 0x857dbb07, SUC_lexicon_tags_3 },
    { 0xc1fdbb0b, SUC_lexicon_tags_23 },
    { 0xe35dbb0b, SUC_lexicon_tags_15 },
    { 0x13bdbb0d, SUC_lexicon_tags_46 },
    { 0x11bdbb0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbd7dbb10, SUC_lexicon_tags_15 },
    { 0x469dbb10, SUC_lexicon_tags_11 },
    { 0xd4bdbb10, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7dbb16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdddbb18, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e5dbb1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cddbb20, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bdbb24, SUC_lexicon_tags_69 },
    { 0xfa5dbb25, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x577dbb27, SUC_lexicon_tags_11 },
    { 0xb99dbb28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fdbb2b, SUC_lexicon_tags_49 },
    { 0xb77dbb2b, SUC_lexicon_tags_12 },
    { 0x8cbdbb2d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbb30, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dbb33, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x723dbb35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x625dbb37, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x91ddbb39, SUC_lexicon_tags_56 },
    { 0x205dbb3a, SUC_lexicon_tags_37 },
    { 0x143dbb3a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fdbb40, SUC_lexicon_tags_32 },
    { 0xf4ddbb41, SUC_lexicon_tags_15 },
    { 0x85ddbb41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7efdbb44, SUC_lexicon_tags_19 },
    { 0x3fdbb45, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71dbb48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9dbb4f, SUC_lexicon_tags_82 },
    { 0xb03dbb50, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42bdbb52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373dbb56, SUC_lexicon_tags_9 },
    { 0x815dbb57, SUC_lexicon_tags_19 },
    { 0x2bbdbb58, SUC_lexicon_tags_4 },
    { 0x6fbdbb58, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0x3e1dbb5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x673dbb5d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2b5dbb5f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x57ddbb61, SUC_lexicon_tags_15 },
    { 0xcd7dbb62, SUC_lexicon_tags_11 },
    { 0x5dbb63, SUC_lexicon_tags_141 },
    { 0xb87dbb63, SUC_lexicon_tags_45 },
    { 0xc95dbb64, SUC_lexicon_tags_37 },
    { 0x40ddbb62, SUC_lexicon_tags_55 },
    { 0x15ddbb67, SUC_lexicon_tags_45 },
    { 0x7addbb67, SUC_lexicon_tags_32 },
    { 0x21bdbb62, SUC_lexicon_tags_1 },
    { 0x24fdbb6a, SUC_lexicon_tags_17 },
    { 0x141dbb66, SUC_lexicon_tags_12 },
    { 0x70fdbb6c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49dbb71, SUC_lexicon_tags_15 },
    { 0xd65dbb72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x603dbb75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe49dbb77, SUC_lexicon_tags_136 },
    { 0xe59dbb77, SUC_lexicon_tags_9 },
    { 0x587dbb79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85dbb7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x131dbb81, SUC_lexicon_tags_45 },
    { 0xea3dbb82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9dbb87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87fdbb89, SUC_lexicon_tags_38 },
    { 0x8cfdbb8a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9dbb8e, SUC_lexicon_tags_48 },
    { 0x41dbb8e, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5dbb92, SUC_lexicon_tags_37 },
    { 0x1c1dbb93, SUC_lexicon_tags_77 },
    { 0x651dbb93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9dbb9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5dbba1, SUC_lexicon_tags_37 },
    { 0x8c3dbba1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc17dbba4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ddbba8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683dbbb2, SUC_lexicon_tags_44 },
    { 0x4dbdbbb3, SUC_lexicon_tags_61 },
    { 0x31bdbbb3, SUC_lexicon_tags_26 },
    { 0xbdfdbbb4, SUC_lexicon_tags_4 },
    { 0xb1dbbb4, SUC_lexicon_tags_77 },
    { 0xde1dbbb3, SUC_lexicon_tags_5 },
    { 0x74ddbbb2, SUC_lexicon_tags_39 },
    { 0x9c1dbbb9, SUC_lexicon_tags_1 },
    { 0x2e5dbbb9, SUC_lexicon_tags_14 },
    { 0xb03dbbb3, SUC_lexicon_tags_47 },
    { 0x7b5dbbb7, SUC_lexicon_tags_1 },
    { 0x485dbbbd, SUC_lexicon_tags_11 },
    { 0xd31dbbbd, SUC_lexicon_tags_68 },
    { 0x1a1dbbbd, SUC_lexicon_tags_37 },
    { 0x4bddbbc0, SUC_lexicon_tags_63 },
    { 0x7fbdbbc1, SUC_lexicon_tags_11 },
    { 0xdddbbbf, SUC_lexicon_tags_11 },
    { 0x7cfdbbc3, SUC_lexicon_tags_2 },
    { 0xe97dbbc2, SUC_lexicon_tags_33 },
    { 0xc45dbbbc, SUC_lexicon_tags_21 },
    { 0x651dbbc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5dbbca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x33fdbbcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1dbbd2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d3dbbd4, SUC_lexicon_tags_4 },
    { 0xe0ddbbd4, SUC_lexicon_tags_65 },
    { 0xa2fdbbd6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dddbbe1, SUC_lexicon_tags_7 },
    { 0x13fdbbe1, SUC_lexicon_tags_15 },
    { 0x9c7dbbe3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x437dbbe5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x531dbbe7, SUC_lexicon_tags_37 },
    { 0xff5dbbe8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x997dbbea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7a9dbbec, SUC_lexicon_tags_14 },
    { 0x66bdbbed, SUC_lexicon_tags_17 },
    { 0x871dbbed, SUC_lexicon_tags_9 },
    { 0x381dbbee, SUC_lexicon_tags_26 },
    { 0xf7bdbbed, SUC_lexicon_tags_1 },
    { 0x4d9dbbf1, SUC_lexicon_tags_27 },
    { 0xbbfdbbf2, SUC_lexicon_tags_27 },
    { 0xe31dbbf3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x1bbdbbf5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x285dbbf7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bdbbfd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9dbc02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebdbc08, SUC_lexicon_tags_33 },
    { 0x8c1dbc08, SUC_lexicon_tags_37 },
    { 0x5f3dbc0a, SUC_lexicon_tags_11 },
    { 0xec1dbc0b, SUC_lexicon_tags_19 },
    { 0x99ddbc09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7dbc11, SUC_lexicon_tags_11 },
    { 0x53dbc12, SUC_lexicon_tags_4 },
    { 0xaa3dbc12, SUC_lexicon_tags_12 },
    { 0xd7dbc12, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb89dbc16, SUC_lexicon_tags_25 },
    { 0x5f9dbc17, SUC_lexicon_tags_47 },
    { 0xe43dbc18, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbc1f, SUC_lexicon_tags_12 },
    { 0xf1bdbc1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17bdbc23, SUC_lexicon_tags_2 },
    { 0x921dbc23, SUC_lexicon_tags_38 },
    { 0x10ddbc24, SUC_lexicon_tags_45 },
    { 0xf97dbc24, SUC_lexicon_tags_8 },
    { 0x48ddbc27, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23dbc2b, SUC_lexicon_tags_0 },
    { 0x397dbc2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x975dbc2e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ddbc32, SUC_lexicon_tags_27 },
    { 0x28ddbc32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x56fdbc35, SUC_lexicon_tags_37 },
    { 0x4d1dbc36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffdbc3b, SUC_lexicon_tags_11 },
    { 0xdd9dbc3c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x671dbc3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337dbc43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbdbc46, SUC_lexicon_tags_26 },
    { 0x453dbc46, SUC_lexicon_tags_65 },
    { 0x39ddbc48, SUC_lexicon_tags_33 },
    { 0xd05dbc49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c9dbc4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x403dbc4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd25dbc50, SUC_lexicon_tags_39 },
    { 0xeb3dbc50, SUC_lexicon_tags_45 },
    { 0xf8bdbc51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa0fdbc54, SUC_lexicon_tags_43 },
    { 0x681dbc55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7dbc58, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x369dbc5a, SUC_lexicon_tags_32 },
    { 0x125dbc5b, SUC_lexicon_tags_25 },
    { 0xd65dbc5a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf53dbc5e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09dbc63, SUC_lexicon_tags_26 },
    { 0x6addbc63, SUC_lexicon_tags_17 },
    { 0x34ddbc65, SUC_lexicon_tags_31 },
    { 0x12fdbc66, SUC_lexicon_tags_146 },
    { 0x3b7dbc67, SUC_lexicon_tags_37 },
    { 0x413dbc66, SUC_lexicon_tags_6 },
    { 0xc1fdbc69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bdbc6d, SUC_lexicon_tags_56 },
    { 0x805dbc6e, SUC_lexicon_tags_12 },
    { 0xfdddbc6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fdbc73, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7c5dbc75, SUC_lexicon_tags_19 },
    { 0x9e3dbc76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ddbc7b, SUC_lexicon_tags_25 },
    { 0xc57dbc7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bdbc82, SUC_lexicon_tags_33 },
    { 0x2abdbc82, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x60bdbc85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ddbc88, SUC_lexicon_tags_7 },
    { 0x79ddbc88, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3dbc8d, SUC_lexicon_tags_30 },
    { 0xfe5dbc8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfddbc91, SUC_lexicon_tags_91 },
    { 0x791dbc92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1dbc98, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbdbc9b, SUC_lexicon_tags_77 },
    { 0x90fdbc9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21dbca0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x97dbca2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ddbca6, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423dbcbb, SUC_lexicon_tags_60 },
    { 0xef9dbcbc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595dbcc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x599dbcc4, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbdbcc8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59dbccf, SUC_lexicon_tags_8 },
    { 0x31fdbcd0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfdbcd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3dbcda, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bdbcdf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x68ddbce2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7dbce5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xed9dbce7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523dbcf1, SUC_lexicon_tags_14 },
    { 0xfe9dbcf2, SUC_lexicon_tags_7 },
    { 0x821dbcf3, SUC_lexicon_tags_49 },
    { 0x39bdbcf4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fdbcf8, SUC_lexicon_tags_48 },
    { 0xa1ddbcf9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x37bdbcfb, SUC_lexicon_tags_39 },
    { 0x263dbcfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe19dbcfe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607dbd07, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x983dbd09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x52bdbd0c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cddbd11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb19dbd13, SUC_lexicon_tags_83 },
    { 0x895dbd14, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xb55dbd16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x153dbd19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195dbd1e, SUC_lexicon_tags_14 },
    { 0x3fddbd1f, SUC_lexicon_tags_80 },
    { 0xff7dbd1e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x199dbd23, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaddbd26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1dddbd28, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dbd2c, SUC_lexicon_tags_28 },
    { 0xf1dbd2c, SUC_lexicon_tags_47 },
    { 0xcf1dbd2e, SUC_lexicon_tags_25 },
    { 0xde5dbd2f, SUC_lexicon_tags_1463 },
    { 0x6d7dbd2d, SUC_lexicon_tags_33 },
    { 0xd17dbd31, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5dbd34, SUC_lexicon_tags_84 },
    { 0x88fdbd34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551dbd3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ddbd41, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xc7fdbd43, SUC_lexicon_tags_15 },
    { 0xa37dbd43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43dbd47, SUC_lexicon_tags_31 },
    { 0xee7dbd47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfdbd4c, SUC_lexicon_tags_63 },
    { 0xdfdbd4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaefdbd4f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xdf7dbd51, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fdbd54, SUC_lexicon_tags_11 },
    { 0x4addbd55, SUC_lexicon_tags_7 },
    { 0x457dbd54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcddbd5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967dbd61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3dbd67, SUC_lexicon_tags_17 },
    { 0x887dbd67, SUC_lexicon_tags_11 },
    { 0xea1dbd69, SUC_lexicon_tags_8 },
    { 0x799dbd67, SUC_lexicon_tags_23 },
    { 0x831dbd6b, SUC_lexicon_tags_70 },
    { 0x45ddbd6b, SUC_lexicon_tags_13 },
    { 0xe5dbd6d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9dbd70, SUC_lexicon_tags_60 },
    { 0x32bdbd71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44bdbd73, SUC_lexicon_tags_46 },
    { 0xb17dbd73, SUC_lexicon_tags_1 },
    { 0x31bdbd73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21dbd7c, SUC_lexicon_tags_28 },
    { 0x6afdbd7d, SUC_lexicon_tags_7 },
    { 0xe7bdbd7e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efdbd81, SUC_lexicon_tags_12 },
    { 0xf17dbd81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13dbd85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffdbd8a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf0bdbd8c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x69fdbd8e, SUC_lexicon_tags_43 },
    { 0x5ffdbd8e, SUC_lexicon_tags_34 },
    { 0x78bdbd90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x505dbd93, SUC_lexicon_tags_25 },
    { 0x979dbd93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabdbd97, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79dbd9a, SUC_lexicon_tags_1 },
    { 0x11dbd9a, SUC_lexicon_tags_14 },
    { 0xb2ddbd9b, SUC_lexicon_tags_5 },
    { 0x543dbd9c, SUC_lexicon_tags_37 },
    { 0x895dbd9e, SUC_lexicon_tags_10 },
    { 0xf25dbd9e, SUC_lexicon_tags_15 },
    { 0x4f5dbd9d, SUC_lexicon_tags_38 },
    { 0x429dbda1, SUC_lexicon_tags_55 },
    { 0xe0ddbda2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1ffdbda4, SUC_lexicon_tags_44 },
    { 0xffbdbda4, SUC_lexicon_tags_12 },
    { 0x5f1dbda6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7dbda9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x459dbdab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989dbdb0, SUC_lexicon_tags_33 },
    { 0x545dbdb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7cddbdb3, SUC_lexicon_tags_47 },
    { 0x893dbdb3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbdb7, SUC_lexicon_tags_22 },
    { 0x189dbdb8, SUC_lexicon_tags_15 },
    { 0x7fddbdb8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbdbdbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae5dbdbf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x261dbdc1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfa1dbdc3, SUC_lexicon_tags_37 },
    { 0xf3dbdc4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x78ddbdc6, SUC_lexicon_tags_1 },
    { 0x761dbdc6, SUC_lexicon_tags_37 },
    { 0xcc1dbdc7, SUC_lexicon_tags_32 },
    { 0xdc3dbdc9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ddbdce, SUC_lexicon_tags_49 },
    { 0xab7dbdcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ddbdd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x775dbdd5, SUC_lexicon_tags_37 },
    { 0x807dbdd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe21dbdd8, SUC_lexicon_tags_38 },
    { 0xaf9dbdd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989dbddd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafdbde2, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fdbde8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781dbdec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bddbdf1, SUC_lexicon_tags_25 },
    { 0x259dbdf1, SUC_lexicon_tags_25 },
    { 0xf55dbdf3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf4ddbdf5, SUC_lexicon_tags_87 },
    { 0xe77dbdf6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x80ddbdf8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x85fdbdfa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x17dbdfd, SUC_lexicon_tags_60 },
    { 0xb7dbdfe, SUC_lexicon_tags_15 },
    { 0x2f7dbdfe, SUC_lexicon_tags_63 },
    { 0x579dbdfe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x305dbe02, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3abdbe04, SUC_lexicon_tags_61 },
    { 0x1dfdbe05, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dbe09, SUC_lexicon_tags_15 },
    { 0xa2bdbe0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd0fdbe0c, SUC_lexicon_tags_47 },
    { 0x847dbe0d, SUC_lexicon_tags_7 },
    { 0x99fdbe0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fdbe12, SUC_lexicon_tags_68 },
    { 0x2ffdbe12, SUC_lexicon_tags_23 },
    { 0x57ddbe14, SUC_lexicon_tags_82 },
    { 0x7bdbe15, SUC_lexicon_tags_45 },
    { 0xdf7dbe12, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5dbe1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e5dbe1c, SUC_lexicon_tags_15 },
    { 0xf9dbe1d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69dbe22, SUC_lexicon_tags_25 },
    { 0x5e9dbe22, SUC_lexicon_tags_1 },
    { 0x761dbe24, SUC_lexicon_tags_17 },
    { 0x59fdbe22, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ddbe29, SUC_lexicon_tags_80 },
    { 0xba7dbe2a, SUC_lexicon_tags_15 },
    { 0xb0bdbe2b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fdbe31, SUC_lexicon_tags_29 },
    { 0x717dbe32, SUC_lexicon_tags_77 },
    { 0xe9ddbe32, SUC_lexicon_tags_17 },
    { 0x81fdbe32, SUC_lexicon_tags_7 },
    { 0x1abdbe35, SUC_lexicon_tags_1 },
    { 0xd91dbe36, SUC_lexicon_tags_6 },
    { 0x1fbdbe34, SUC_lexicon_tags_27 },
    { 0x885dbe33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x45ddbe3a, SUC_lexicon_tags_19 },
    { 0x20fdbe3b, SUC_lexicon_tags_136 },
    { 0xa7dbe3c, SUC_lexicon_tags_68 },
    { 0xee5dbe3d, SUC_lexicon_tags_47 },
    { 0x4d5dbe3e, SUC_lexicon_tags_23 },
    { 0x745dbe3f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb3fdbe41, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787dbe46, SUC_lexicon_tags_10 },
    { 0x671dbe47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x439dbe49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35dbe53, SUC_lexicon_tags_25 },
    { 0x68ddbe54, SUC_lexicon_tags_8 },
    { 0x30fdbe53, SUC_lexicon_tags_39 },
    { 0xcc1dbe54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1dbe5d, SUC_lexicon_tags_33 },
    { 0x701dbe5d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fdbe62, SUC_lexicon_tags_4 },
    { 0x5afdbe62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5dbe66, SUC_lexicon_tags_25 },
    { 0xe87dbe67, SUC_lexicon_tags_7 },
    { 0xa01dbe66, SUC_lexicon_tags_77 },
    { 0x179dbe69, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ebdbe6b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4bdbe6d, SUC_lexicon_tags_2 },
    { 0xb83dbe6d, SUC_lexicon_tags_37 },
    { 0xb87dbe6e, SUC_lexicon_tags_11 },
    { 0x9d5dbe6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdbbdbe72, SUC_lexicon_tags_40 },
    { 0x835dbe72, SUC_lexicon_tags_31 },
    { 0xc67dbe74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fdbe7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ddbe81, SUC_lexicon_tags_66 },
    { 0x26fdbe81, SUC_lexicon_tags_35 },
    { 0xe4bdbe83, SUC_lexicon_tags_10 },
    { 0xf0fdbe84, SUC_lexicon_tags_33 },
    { 0x329dbe85, SUC_lexicon_tags_0 },
    { 0x5cfdbe84, SUC_lexicon_tags_17 },
    { 0x763dbe87, SUC_lexicon_tags_33 },
    { 0xde1dbe88, SUC_lexicon_tags_15 },
    { 0xd75dbe89, SUC_lexicon_tags_17 },
    { 0xfa7dbe89, SUC_lexicon_tags_17 },
    { 0x92fdbe85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efdbe8f, SUC_lexicon_tags_803 },
    { 0x6b9dbe90, SUC_lexicon_tags_28 },
    { 0x6c1dbe90, SUC_lexicon_tags_14 },
    { 0x815dbe92, SUC_lexicon_tags_17 },
    { 0xc23dbe93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47dbe99, SUC_lexicon_tags_61 },
    { 0x619dbe9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1dbe9d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaafdbe9f, SUC_lexicon_tags_25 },
    { 0xd7bdbe9f, SUC_lexicon_tags_17 },
    { 0x379dbea0, SUC_lexicon_tags_36 },
    { 0xd17dbea2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bdbea5, SUC_lexicon_tags_14 },
    { 0x30bdbea5, SUC_lexicon_tags_15 },
    { 0x5ddbea6, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fdbeaa, SUC_lexicon_tags_49 },
    { 0xf57dbeab, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a5dbead, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb25dbeaf, SUC_lexicon_tags_63 },
    { 0xe4fdbeb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95dbeb3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x383dbeb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x57ddbeb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bdbec3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xebdbec5, SUC_lexicon_tags_27 },
    { 0x26fdbec5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xba1dbec8, SUC_lexicon_tags_17 },
    { 0xcdfdbec9, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7dbece, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7bfdbed0, SUC_lexicon_tags_11 },
    { 0x7a3dbed1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf43dbed3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdbed8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701dbede, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eddbee2, SUC_lexicon_tags_27 },
    { 0xd4fdbee2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9dbee8, SUC_lexicon_tags_7 },
    { 0x5cddbee8, SUC_lexicon_tags_49 },
    { 0xd1ddbee9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x509dbeec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ddbef0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121dbef4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241dbef8, SUC_lexicon_tags_88 },
    { 0xed9dbef9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609dbefd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x605dbf00, SUC_lexicon_tags_19 },
    { 0xfdddbf00, SUC_lexicon_tags_25 },
    { 0xebddbf02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ddbf06, SUC_lexicon_tags_87 },
    { 0xb2bdbf07, SUC_lexicon_tags_27 },
    { 0x70fdbf08, SUC_lexicon_tags_37 },
    { 0xbd7dbf09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x58fdbf0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x22ddbf0d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeddbf0f, SUC_lexicon_tags_60 },
    { 0xa63dbf10, SUC_lexicon_tags_41 },
    { 0x295dbf11, SUC_lexicon_tags_38 },
    { 0x717dbf12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb5fdbf14, SUC_lexicon_tags_47 },
    { 0x72ddbf14, SUC_lexicon_tags_12 },
    { 0x9f9dbf14, SUC_lexicon_tags_1 },
    { 0x77ddbf17, SUC_lexicon_tags_11 },
    { 0x133dbf18, SUC_lexicon_tags_16 },
    { 0x247dbf16, SUC_lexicon_tags_7 },
    { 0xc5ddbf1a, SUC_lexicon_tags_56 },
    { 0xe91dbf1a, SUC_lexicon_tags_33 },
    { 0xbddbf1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03dbf20, SUC_lexicon_tags_46 },
    { 0x5e1dbf21, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfdbf26, SUC_lexicon_tags_68 },
    { 0x7c3dbf27, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95dbf2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fdbf2e, SUC_lexicon_tags_15 },
    { 0xacbdbf2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5dbf34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5dbf38, SUC_lexicon_tags_23 },
    { 0x49ddbf39, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35dbf3f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dddbf45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bdbf50, SUC_lexicon_tags_15 },
    { 0x89ddbf51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x103dbf54, SUC_lexicon_tags_12 },
    { 0xaa7dbf55, SUC_lexicon_tags_11 },
    { 0xeebdbf56, SUC_lexicon_tags_17 },
    { 0xd47dbf54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebdbf5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffdbf63, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dbf66, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fdbf6d, SUC_lexicon_tags_10 },
    { 0x645dbf6e, SUC_lexicon_tags_35 },
    { 0xd57dbf6f, SUC_lexicon_tags_56 },
    { 0x3bdbf6e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc2ddbf72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x939dbf75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dbf78, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1dbf81, SUC_lexicon_tags_17 },
    { 0xc43dbf82, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83dbf85, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ddbf8b, SUC_lexicon_tags_17 },
    { 0x755dbf8c, SUC_lexicon_tags_19 },
    { 0x831dbf8b, SUC_lexicon_tags_1 },
    { 0xae5dbf8e, SUC_lexicon_tags_2 },
    { 0x40ddbf8c, SUC_lexicon_tags_17 },
    { 0x53fdbf8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383dbf95, SUC_lexicon_tags_4 },
    { 0xf1ddbf95, SUC_lexicon_tags_2 },
    { 0xca3dbf97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1eddbf99, SUC_lexicon_tags_63 },
    { 0xd6bdbf9a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47dbfa1, SUC_lexicon_tags_28 },
    { 0xd59dbfa1, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7dbfa5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f5dbfa7, SUC_lexicon_tags_7 },
    { 0x91ddbfa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfdbfab, SUC_lexicon_tags_11 },
    { 0x2b5dbfab, SUC_lexicon_tags_7 },
    { 0xd49dbfab, SUC_lexicon_tags_19 },
    { 0xe71dbfae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffdbfb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fdbfb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d3dbfbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf47dbfbd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3dbfcb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5dbfce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43dbfd0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabdbfd3, SUC_lexicon_tags_25 },
    { 0xd7bdbfd3, SUC_lexicon_tags_39 },
    { 0x8a3dbfd5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xba7dbfd7, SUC_lexicon_tags_10 },
    { 0xf95dbfd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafdbfde, SUC_lexicon_tags_14 },
    { 0x619dbfdf, SUC_lexicon_tags_17 },
    { 0x9f7dbfde, SUC_lexicon_tags_23 },
    { 0x335dbfe1, SUC_lexicon_tags_46 },
    { 0x8bdbfe2, SUC_lexicon_tags_23 },
    { 0x453dbfde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x429dbfe5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7dbfeb, SUC_lexicon_tags_17 },
    { 0x55fdbfec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a9dbfee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21dbff1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe1ddbff3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x243dbff6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x10fdbff8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fdbffd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fdc002, SUC_lexicon_tags_38 },
    { 0xa37dc002, SUC_lexicon_tags_19 },
    { 0x357dc004, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x279dc007, SUC_lexicon_tags_77 },
    { 0x445dc008, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x393dc00b, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0x81dc00d, SUC_lexicon_tags_8 },
    { 0xe9dc00d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x69ddc010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbdc014, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361dc01c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41dc027, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed1dc029, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869dc02d, SUC_lexicon_tags_47 },
    { 0xbb5dc02e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44bdc030, SUC_lexicon_tags_37 },
    { 0x197dc031, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa9dc033, SUC_lexicon_tags_18 },
    { 0x601dc034, SUC_lexicon_tags_7 },
    { 0xf81dc033, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9dc03a, SUC_lexicon_tags_33 },
    { 0x54fdc03b, SUC_lexicon_tags_1 },
    { 0xb2bdc03a, SUC_lexicon_tags_88 },
    { 0x6b1dc03d, SUC_lexicon_tags_2 },
    { 0xf71dc03c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x831dc040, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7fbdc042, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dc045, SUC_lexicon_tags_1 },
    { 0xa29dc046, SUC_lexicon_tags_68 },
    { 0x1f5dc047, SUC_lexicon_tags_37 },
    { 0xebdc046, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafbdc04a, SUC_lexicon_tags_16 },
    { 0x61ddc04a, SUC_lexicon_tags_48 },
    { 0x58fdc04b, SUC_lexicon_tags_27 },
    { 0xae7dc04d, SUC_lexicon_tags_7 },
    { 0x2bfdc04e, SUC_lexicon_tags_11 },
    { 0xb1ddc04f, SUC_lexicon_tags_23 },
    { 0xd3dc050, SUC_lexicon_tags_2 },
    { 0x653dc051, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x91fdc054, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1dc058, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ddc05c, SUC_lexicon_tags_25 },
    { 0xfd7dc05d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa6ddc05f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07dc062, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x487dc065, SUC_lexicon_tags_25 },
    { 0xdd1dc066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769dc070, SUC_lexicon_tags_60 },
    { 0x901dc071, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33dc074, SUC_lexicon_tags_30 },
    { 0x45ddc075, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1dc079, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69ddc07b, SUC_lexicon_tags_26 },
    { 0x247dc07c, SUC_lexicon_tags_1 },
    { 0x9f3dc07d, SUC_lexicon_tags_45 },
    { 0xffdc07d, SUC_lexicon_tags_27 },
    { 0xc6fdc07f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e5dc081, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd3ddc083, SUC_lexicon_tags_9 },
    { 0x583dc083, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bdc087, SUC_lexicon_tags_1 },
    { 0x1d3dc088, SUC_lexicon_tags_23 },
    { 0x739dc088, SUC_lexicon_tags_7 },
    { 0x609dc088, SUC_lexicon_tags_56 },
    { 0x51bdc088, SUC_lexicon_tags_12 },
    { 0x19dc08c, SUC_lexicon_tags_38 },
    { 0x6c7dc08c, SUC_lexicon_tags_0 },
    { 0xa0ddc08e, SUC_lexicon_tags_16 },
    { 0x78fdc08e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x20ddc091, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7dc09c, SUC_lexicon_tags_15 },
    { 0xc51dc09d, SUC_lexicon_tags_49 },
    { 0x8cfdc09e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3dc0a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfdc0a7, SUC_lexicon_tags_25 },
    { 0x14ddc0a8, SUC_lexicon_tags_37 },
    { 0xec3dc0a9, SUC_lexicon_tags_87 },
    { 0x3e7dc0aa, SUC_lexicon_tags_7 },
    { 0x111dc0ab, SUC_lexicon_tags_23 },
    { 0x967dc0ac, SUC_lexicon_tags_63 },
    { 0xa5fdc0ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x921dc0b0, SUC_lexicon_tags_79 },
    { 0xde3dc0b0, SUC_lexicon_tags_37 },
    { 0x85fdc0b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fdc0bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21bdc0bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1dc0c2, SUC_lexicon_tags_27 },
    { 0xc53dc0c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73dc0c7, SUC_lexicon_tags_37 },
    { 0x4d7dc0c7, SUC_lexicon_tags_17 },
    { 0xdffdc0c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ddc0cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x13bdc0cf, SUC_lexicon_tags_26 },
    { 0x277dc0d0, SUC_lexicon_tags_51 },
    { 0x90ddc0cf, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bdc0d4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3dc0d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebdc0de, SUC_lexicon_tags_14 },
    { 0xa09dc0df, SUC_lexicon_tags_12 },
    { 0x637dc0e0, SUC_lexicon_tags_39 },
    { 0x29ddc0de, SUC_lexicon_tags_37 },
    { 0xbdc0de, SUC_lexicon_tags_80 },
    { 0xc1fdc0e2, SUC_lexicon_tags_47 },
    { 0x3ddc0e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x751dc0e7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61dc0ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595dc0f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2dfdc0f5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31dc0f9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1dc0fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3dc101, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67ddc103, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3dc106, SUC_lexicon_tags_43 },
    { 0x333dc107, SUC_lexicon_tags_31 },
    { 0xb97dc107, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9dc10d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66fdc10f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4c9dc111, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbdc114, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1dc117, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791dc11d, SUC_lexicon_tags_2 },
    { 0xddbdc11e, SUC_lexicon_tags_37 },
    { 0xfabdc11f, SUC_lexicon_tags_30 },
    { 0x77bdc120, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc65dc122, SUC_lexicon_tags_32 },
    { 0x391dc122, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe1bdc125, SUC_lexicon_tags_17 },
    { 0xc57dc126, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xefddc129, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121dc12d, SUC_lexicon_tags_13 },
    { 0x14fdc12d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfdc136, SUC_lexicon_tags_77 },
    { 0xc27dc137, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x26fdc139, SUC_lexicon_tags_37 },
    { 0x10bdc13a, SUC_lexicon_tags_38 },
    { 0x30fdc13a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb3dc13d, SUC_lexicon_tags_10 },
    { 0xadddc13e, SUC_lexicon_tags_3 },
    { 0x525dc13f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3dc142, SUC_lexicon_tags_47 },
    { 0x855dc143, SUC_lexicon_tags_12 },
    { 0x7e7dc142, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd89dc146, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa49dc148, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fdc14d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xce9dc14f, SUC_lexicon_tags_17 },
    { 0x909dc150, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49dc156, SUC_lexicon_tags_16 },
    { 0x80fdc157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bdc15a, SUC_lexicon_tags_15 },
    { 0x347dc15b, SUC_lexicon_tags_25 },
    { 0x243dc15a, SUC_lexicon_tags_27 },
    { 0x28fdc15c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9dc160, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdc163, SUC_lexicon_tags_37 },
    { 0xed9dc164, SUC_lexicon_tags_15 },
    { 0x62fdc164, SUC_lexicon_tags_37 },
    { 0x569dc166, SUC_lexicon_tags_30 },
    { 0x513dc167, SUC_lexicon_tags_37 },
    { 0x21bdc163, SUC_lexicon_tags_17 },
    { 0x2fddc169, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe2bdc16b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x257dc16d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27dc174, SUC_lexicon_tags_1 },
    { 0x67bdc175, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fdc178, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a1dc17a, SUC_lexicon_tags_43 },
    { 0xee5dc17a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x271dc17e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885dc182, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09dc18c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651dc191, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21dc196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e9dc198, SUC_lexicon_tags_0 },
    { 0x43fdc199, SUC_lexicon_tags_7 },
    { 0x701dc19a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdc19d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3dc1a2, SUC_lexicon_tags_37 },
    { 0x917dc1a3, SUC_lexicon_tags_49 },
    { 0xf51dc1a2, SUC_lexicon_tags_8 },
    { 0x819dc1a5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617dc1ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85dc1af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc1b2, SUC_lexicon_tags_0 },
    { 0xfd7dc1b3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xacfdc1b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe69dc1b7, SUC_lexicon_tags_3 },
    { 0x20bdc1b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1dc1bb, SUC_lexicon_tags_19 },
    { 0x8a7dc1bc, SUC_lexicon_tags_11 },
    { 0x78ddc1bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21dc1c0, SUC_lexicon_tags_23 },
    { 0xf7dc1c1, SUC_lexicon_tags_106 },
    { 0xd6ddc1c1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843dc1c6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4bddc1c8, SUC_lexicon_tags_60 },
    { 0x1f7dc1c8, SUC_lexicon_tags_68 },
    { 0xe63dc1ca, SUC_lexicon_tags_1 },
    { 0xf79dc1c8, SUC_lexicon_tags_7 },
    { 0x12bdc1cc, SUC_lexicon_tags_15 },
    { 0xe59dc1cd, SUC_lexicon_tags_37 },
    { 0x13dc1cd, SUC_lexicon_tags_49 },
    { 0xffdc1c8, SUC_lexicon_tags_32 },
    { 0x993dc1c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdc1d3, SUC_lexicon_tags_13 },
    { 0xb6fdc1d4, SUC_lexicon_tags_25 },
    { 0x26fdc1d5, SUC_lexicon_tags_37 },
    { 0xe0bdc1d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29dc1d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fdc1dc, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3dc1e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb87dc1e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267dc1e8, SUC_lexicon_tags_29 },
    { 0x285dc1e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x335dc1ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41dc1ef, SUC_lexicon_tags_14 },
    { 0x871dc1ef, SUC_lexicon_tags_39 },
    { 0xc05dc1f1, SUC_lexicon_tags_7 },
    { 0xb09dc1f1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x219dc1f4, SUC_lexicon_tags_8 },
    { 0x62fdc1f5, SUC_lexicon_tags_11 },
    { 0xe0fdc1f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xea9dc1f8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03dc1fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47dc201, SUC_lexicon_tags_226 },
    { 0x14bdc201, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc87dc204, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdc207, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813dc20e, SUC_lexicon_tags_7 },
    { 0xcb7dc20f, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817dc215, SUC_lexicon_tags_56 },
    { 0x651dc216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5dc219, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165dc21d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x51bdc21f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643dc223, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05dc227, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x83ddc229, SUC_lexicon_tags_2 },
    { 0xb01dc229, SUC_lexicon_tags_7 },
    { 0x869dc22b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27dc22f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3dc235, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa47dc237, SUC_lexicon_tags_25 },
    { 0x6efdc237, SUC_lexicon_tags_47 },
    { 0xfa7dc239, SUC_lexicon_tags_33 },
    { 0x57fdc239, SUC_lexicon_tags_25 },
    { 0xe49dc237, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3dc242, SUC_lexicon_tags_10 },
    { 0x91bdc243, SUC_lexicon_tags_47 },
    { 0xcb5dc243, SUC_lexicon_tags_15 },
    { 0xbf5dc245, SUC_lexicon_tags_0 },
    { 0x1d7dc246, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7dc249, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ddc24c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69dc253, SUC_lexicon_tags_29 },
    { 0x38bdc253, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491dc25a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25dc25f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61dc263, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3dc267, SUC_lexicon_tags_45 },
    { 0xdc3dc268, SUC_lexicon_tags_62 },
    { 0x2dddc269, SUC_lexicon_tags_475 },
    { 0x649dc269, SUC_lexicon_tags_7 },
    { 0xae3dc268, SUC_lexicon_tags_25 },
    { 0x7addc269, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fdc272, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x137dc275, SUC_lexicon_tags_17 },
    { 0x297dc276, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195dc27d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbffdc27f, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0x367dc281, SUC_lexicon_tags_15 },
    { 0x37bdc282, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3dc286, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x739dc288, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ddc28d, SUC_lexicon_tags_12 },
    { 0x313dc28d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x695dc291, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479dc299, SUC_lexicon_tags_2 },
    { 0x673dc29a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x673dc29d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461dc2a4, SUC_lexicon_tags_47 },
    { 0xeeddc2a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311dc2a9, SUC_lexicon_tags_26 },
    { 0xd9dc2a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbdc2ad, SUC_lexicon_tags_2 },
    { 0x40ddc2ae, SUC_lexicon_tags_65 },
    { 0x1cfdc2af, SUC_lexicon_tags_49 },
    { 0x4c9dc2b0, SUC_lexicon_tags_14 },
    { 0x4fdc2b1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21bdc2b3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bdc2b6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xe37dc2b8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bdc2bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7dc2c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4c5dc2c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987dc2c6, SUC_lexicon_tags_45 },
    { 0x8ddc2c7, SUC_lexicon_tags_0 },
    { 0xc05dc2c7, SUC_lexicon_tags_25 },
    { 0xd9fdc2c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fdc2ce, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2e7dc2d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7dc2d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931dc2de, SUC_lexicon_tags_522 },
    { 0x9e3dc2df, SUC_lexicon_tags_25 },
    { 0x489dc2e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa05dc2e2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcb7dc2e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa95dc2e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf77dc2e8, SUC_lexicon_tags_25 },
    { 0xddc2e8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efdc2ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9b7dc2ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615dc2f4, SUC_lexicon_tags_74 },
    { 0xcf1dc2f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x725dc2f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ddc2fb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd27dc2fd, SUC_lexicon_tags_12 },
    { 0xb89dc2fd, SUC_lexicon_tags_0 },
    { 0xb6bdc2ff, SUC_lexicon_tags_11 },
    { 0xfcbdc2fd, SUC_lexicon_tags_17 },
    { 0xa43dc301, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x125dc303, SUC_lexicon_tags_317 },
    { 0x49dc304, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3dc308, SUC_lexicon_tags_19 },
    { 0x815dc308, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57dc30c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x21bdc30f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dddc314, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x307dc316, SUC_lexicon_tags_45 },
    { 0x771dc317, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ddc327, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577dc333, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x279dc336, SUC_lexicon_tags_7 },
    { 0x4addc336, SUC_lexicon_tags_1 },
    { 0xe1bdc337, SUC_lexicon_tags_11 },
    { 0x879dc339, SUC_lexicon_tags_106 },
    { 0x3d3dc336, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x213dc33c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa3dc33e, SUC_lexicon_tags_29 },
    { 0xdf5dc33e, SUC_lexicon_tags_7 },
    { 0x201dc340, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33dc344, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd27dc346, SUC_lexicon_tags_31 },
    { 0x9d1dc347, SUC_lexicon_tags_27 },
    { 0x705dc348, SUC_lexicon_tags_29 },
    { 0xff5dc347, SUC_lexicon_tags_12 },
    { 0xda9dc34a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5dc34d, SUC_lexicon_tags_11 },
    { 0x839dc34e, SUC_lexicon_tags_2 },
    { 0x5e5dc34e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1dc356, SUC_lexicon_tags_11 },
    { 0x809dc357, SUC_lexicon_tags_17 },
    { 0x69ddc358, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839dc360, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf7dc362, SUC_lexicon_tags_56 },
    { 0xa83dc362, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x855dc365, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907dc36a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf03dc36c, SUC_lexicon_tags_37 },
    { 0x989dc36d, SUC_lexicon_tags_37 },
    { 0x53bdc36d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbbfdc370, SUC_lexicon_tags_7 },
    { 0x465dc371, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21dc376, SUC_lexicon_tags_47 },
    { 0x499dc376, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f7dc379, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3dc37f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa85dc381, SUC_lexicon_tags_60 },
    { 0xa6ddc381, SUC_lexicon_tags_23 },
    { 0xe95dc381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d5dc385, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc77dc388, SUC_lexicon_tags_35 },
    { 0xbf3dc389, SUC_lexicon_tags_28 },
    { 0xaf1dc38a, SUC_lexicon_tags_11 },
    { 0x8fdc38b, SUC_lexicon_tags_29 },
    { 0xdaddc38c, SUC_lexicon_tags_7 },
    { 0xe73dc38d, SUC_lexicon_tags_33 },
    { 0x95dc38c, SUC_lexicon_tags_11 },
    { 0xe87dc38d, SUC_lexicon_tags_17 },
    { 0xa8ddc38d, SUC_lexicon_tags_16 },
    { 0x885dc391, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ddc399, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fdc39d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fdc3a0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bdc3aa, SUC_lexicon_tags_17 },
    { 0x81bdc3ab, SUC_lexicon_tags_23 },
    { 0xb79dc3ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fdc3b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x783dc3b4, SUC_lexicon_tags_32 },
    { 0x7d1dc3b4, SUC_lexicon_tags_6 },
    { 0x961dc3b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcddc3bd, SUC_lexicon_tags_49 },
    { 0x94bdc3bd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dddc3c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x997dc3c4, SUC_lexicon_tags_1 },
    { 0x8e1dc3c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819dc3c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x82bdc3cc, SUC_lexicon_tags_82 },
    { 0x2a3dc3cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fdc3d1, SUC_lexicon_tags_1 },
    { 0x373dc3d2, SUC_lexicon_tags_29 },
    { 0xcf7dc3d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x72fdc3d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41ddc3d7, SUC_lexicon_tags_45 },
    { 0x579dc3d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x245dc3db, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237dc3e0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649dc3e6, SUC_lexicon_tags_47 },
    { 0x70fdc3e7, SUC_lexicon_tags_8 },
    { 0x4d5dc3e8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x811dc3ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x113dc3ec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x5bbdc3ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdc3f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x359dc3f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637dc3f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977dc3fd, SUC_lexicon_tags_12 },
    { 0xf2fdc3fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2afdc400, SUC_lexicon_tags_84 },
    { 0xffbdc401, SUC_lexicon_tags_14 },
    { 0x27fdc401, SUC_lexicon_tags_37 },
    { 0x559dc401, SUC_lexicon_tags_25 },
    { 0x24bdc404, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8a3dc406, SUC_lexicon_tags_12 },
    { 0xd71dc407, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7dc40c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8b5dc40e, SUC_lexicon_tags_11 },
    { 0xf63dc40f, SUC_lexicon_tags_138 },
    { 0x165dc410, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbdc414, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ddc41b, SUC_lexicon_tags_23 },
    { 0xc7ddc41c, SUC_lexicon_tags_15 },
    { 0x5fddc41c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd55dc41f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bdc423, SUC_lexicon_tags_30 },
    { 0xd2ddc424, SUC_lexicon_tags_1 },
    { 0xe2fdc424, SUC_lexicon_tags_25 },
    { 0xc55dc426, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47dc42e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x473dc430, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb7dc432, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x139dc434, SUC_lexicon_tags_37 },
    { 0xac3dc435, SUC_lexicon_tags_323 },
    { 0xde1dc436, SUC_lexicon_tags_11 },
    { 0x33fdc437, SUC_lexicon_tags_17 },
    { 0x837dc438, SUC_lexicon_tags_45 },
    { 0x731dc439, SUC_lexicon_tags_79 },
    { 0x84bdc43a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x27fdc43c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfdc43f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bdc443, SUC_lexicon_tags_37 },
    { 0x7abdc443, SUC_lexicon_tags_0 },
    { 0x479dc445, SUC_lexicon_tags_43 },
    { 0x427dc443, SUC_lexicon_tags_44 },
    { 0xa09dc447, SUC_lexicon_tags_25 },
    { 0x60bdc447, SUC_lexicon_tags_34 },
    { 0x9fddc445, SUC_lexicon_tags_7 },
    { 0xc77dc443, SUC_lexicon_tags_87 },
    { 0xeafdc44b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709dc452, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559dc456, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xefdc459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7dc45d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdc460, SUC_lexicon_tags_25 },
    { 0xa17dc461, SUC_lexicon_tags_27 },
    { 0xaf9dc462, SUC_lexicon_tags_29 },
    { 0x1bddc461, SUC_lexicon_tags_45 },
    { 0x59ddc464, SUC_lexicon_tags_87 },
    { 0x1f5dc460, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66fdc467, SUC_lexicon_tags_15 },
    { 0x5d1dc468, SUC_lexicon_tags_60 },
    { 0xda7dc468, SUC_lexicon_tags_1 },
    { 0x1dbdc467, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423dc46f, SUC_lexicon_tags_2 },
    { 0xa19dc46f, SUC_lexicon_tags_1 },
    { 0x8b3dc46f, SUC_lexicon_tags_45 },
    { 0x89bdc472, SUC_lexicon_tags_31 },
    { 0xacddc46f, SUC_lexicon_tags_37 },
    { 0x4c3dc474, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773dc47b, SUC_lexicon_tags_15 },
    { 0xb53dc47b, SUC_lexicon_tags_17 },
    { 0x20ddc47d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5dc481, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9dc484, SUC_lexicon_tags_31 },
    { 0x2d3dc484, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc97dc487, SUC_lexicon_tags_37 },
    { 0xb5bdc488, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153dc48c, SUC_lexicon_tags_23 },
    { 0x725dc48c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9dc490, SUC_lexicon_tags_17 },
    { 0x287dc491, SUC_lexicon_tags_60 },
    { 0x685dc490, SUC_lexicon_tags_19 },
    { 0x6ebdc490, SUC_lexicon_tags_2 },
    { 0xca1dc490, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1dc498, SUC_lexicon_tags_11 },
    { 0x5addc499, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x433dc49c, SUC_lexicon_tags_43 },
    { 0x2c9dc49d, SUC_lexicon_tags_11 },
    { 0x93dc49e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd4ddc4a0, SUC_lexicon_tags_129 },
    { 0xdcbdc4a0, SUC_lexicon_tags_23 },
    { 0x1d9dc4a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbdc4a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x709dc4a7, SUC_lexicon_tags_47 },
    { 0x333dc4a7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1dc4ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45dc4b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdaddc4b4, SUC_lexicon_tags_14 },
    { 0x79dc4b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbdc4ba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x643dc4bd, SUC_lexicon_tags_37 },
    { 0x7c5dc4be, SUC_lexicon_tags_11 },
    { 0x41dc4be, SUC_lexicon_tags_113 },
    { 0xe1bdc4c0, SUC_lexicon_tags_25 },
    { 0x5e9dc4c1, SUC_lexicon_tags_177 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fdc4c4, SUC_lexicon_tags_19 },
    { 0x2b3dc4c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x695dc4c7, SUC_lexicon_tags_16 },
    { 0x8cfdc4c7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5dc4cd, SUC_lexicon_tags_25 },
    { 0xdd7dc4ce, SUC_lexicon_tags_4 },
    { 0x63fdc4cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8fbdc4d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1dc4d5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bdc4d8, SUC_lexicon_tags_48 },
    { 0xb7bdc4d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67fdc4dc, SUC_lexicon_tags_4 },
    { 0x3fbdc4dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe45dc4e0, SUC_lexicon_tags_33 },
    { 0x803dc4e0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1dc4e4, SUC_lexicon_tags_38 },
    { 0x945dc4e5, SUC_lexicon_tags_26 },
    { 0x9dbdc4e6, SUC_lexicon_tags_19 },
    { 0xc91dc4e5, SUC_lexicon_tags_7 },
    { 0x9b9dc4e8, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1dc4ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11dc4f0, SUC_lexicon_tags_7 },
    { 0x691dc4f1, SUC_lexicon_tags_33 },
    { 0x9f3dc4f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdc4f6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1addc4fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xda5dc4fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7dc500, SUC_lexicon_tags_6 },
    { 0x511dc500, SUC_lexicon_tags_12 },
    { 0xf39dc501, SUC_lexicon_tags_47 },
    { 0xfcddc503, SUC_lexicon_tags_38 },
    { 0xee5dc500, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05dc508, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcddc50e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x947dc510, SUC_lexicon_tags_49 },
    { 0x14ddc511, SUC_lexicon_tags_15 },
    { 0x355dc512, SUC_lexicon_tags_198 },
    { 0x671dc513, SUC_lexicon_tags_57 },
    { 0x1e5dc514, SUC_lexicon_tags_2 },
    { 0xa6ddc515, SUC_lexicon_tags_65 },
    { 0x511dc515, SUC_lexicon_tags_11 },
    { 0x991dc517, SUC_lexicon_tags_38 },
    { 0x983dc518, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855dc51d, SUC_lexicon_tags_1 },
    { 0x283dc51e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bdc521, SUC_lexicon_tags_11 },
    { 0x629dc521, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5ebdc524, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d3dc526, SUC_lexicon_tags_60 },
    { 0x6b5dc527, SUC_lexicon_tags_14 },
    { 0xfabdc526, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7dc52b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dc530, SUC_lexicon_tags_84 },
    { 0x837dc531, SUC_lexicon_tags_17 },
    { 0xd0ddc532, SUC_lexicon_tags_11 },
    { 0x8ebdc533, SUC_lexicon_tags_28 },
    { 0x5d9dc534, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf67dc536, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x11fdc538, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7dc53b, SUC_lexicon_tags_17 },
    { 0xc81dc53c, SUC_lexicon_tags_12 },
    { 0x9cddc53d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5dc543, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x561dc546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa39dc548, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9dc54b, SUC_lexicon_tags_30 },
    { 0x77dc54c, SUC_lexicon_tags_25 },
    { 0x793dc54b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23dc550, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89dc556, SUC_lexicon_tags_15 },
    { 0x6ebdc556, SUC_lexicon_tags_39 },
    { 0xd95dc558, SUC_lexicon_tags_16 },
    { 0xd4bdc559, SUC_lexicon_tags_11 },
    { 0xcf7dc558, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x461dc55c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x547dc55f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1d3dc561, SUC_lexicon_tags_25 },
    { 0xda9dc562, SUC_lexicon_tags_17 },
    { 0x89bdc563, SUC_lexicon_tags_10 },
    { 0x927dc564, SUC_lexicon_tags_25 },
    { 0x24ddc565, SUC_lexicon_tags_17 },
    { 0x28bdc566, SUC_lexicon_tags_32 },
    { 0xa09dc562, SUC_lexicon_tags_0 },
    { 0x901dc568, SUC_lexicon_tags_47 },
    { 0xba3dc565, SUC_lexicon_tags_15 },
    { 0x633dc563, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x2bfdc56c, SUC_lexicon_tags_1 },
    { 0xb87dc56d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x739dc56f, SUC_lexicon_tags_37 },
    { 0x343dc56f, SUC_lexicon_tags_37 },
    { 0x1bdc571, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5dc576, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x445dc579, SUC_lexicon_tags_25 },
    { 0x69bdc579, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7b5dc57c, SUC_lexicon_tags_7 },
    { 0xb89dc57c, SUC_lexicon_tags_17 },
    { 0xf5ddc57c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529dc583, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa09dc585, SUC_lexicon_tags_8 },
    { 0x39ddc586, SUC_lexicon_tags_2 },
    { 0x77ddc587, SUC_lexicon_tags_25 },
    { 0x87bdc587, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb93dc58b, SUC_lexicon_tags_33 },
    { 0xf69dc58b, SUC_lexicon_tags_25 },
    { 0x81bdc58d, SUC_lexicon_tags_61 },
    { 0x5fbdc58b, SUC_lexicon_tags_25 },
    { 0x787dc58d, SUC_lexicon_tags_23 },
    { 0x68fdc58f, SUC_lexicon_tags_2 },
    { 0xcc5dc58c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bdc595, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429dc59d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bdc5a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x439dc5a5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafdc5a8, SUC_lexicon_tags_4 },
    { 0x8ebdc5a9, SUC_lexicon_tags_64 },
    { 0x697dc5aa, SUC_lexicon_tags_11 },
    { 0x27bdc5aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xffddc5ad, SUC_lexicon_tags_2 },
    { 0x9cbdc5ad, SUC_lexicon_tags_19 },
    { 0xc5dc5ae, SUC_lexicon_tags_11 },
    { 0xdeddc5ae, SUC_lexicon_tags_1 },
    { 0xb77dc5b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2e1dc5b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69dc5b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x533dc5ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45dc5bf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc6ddc5c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76fdc5c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bdc5c9, SUC_lexicon_tags_15 },
    { 0xe31dc5ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533dc5d0, SUC_lexicon_tags_55 },
    { 0x84ddc5d0, SUC_lexicon_tags_24 },
    { 0xa47dc5d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505dc5dc, SUC_lexicon_tags_16 },
    { 0x6a5dc5dd, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75dc5e1, SUC_lexicon_tags_37 },
    { 0xcb1dc5e1, SUC_lexicon_tags_1 },
    { 0x759dc5e3, SUC_lexicon_tags_60 },
    { 0x4cddc5e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x30fdc5e6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x81bdc5e8, SUC_lexicon_tags_15 },
    { 0x1cbdc5e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffdc5f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97dc5f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7dc5f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaefdc5fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x56fdc5fd, SUC_lexicon_tags_11 },
    { 0x2eddc5fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb79dc600, SUC_lexicon_tags_0 },
    { 0xf3bdc601, SUC_lexicon_tags_60 },
    { 0x24ddc601, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc1bdc604, SUC_lexicon_tags_5 },
    { 0x7b1dc605, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x70fdc607, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fdc60b, SUC_lexicon_tags_17 },
    { 0xf8ddc60c, SUC_lexicon_tags_14 },
    { 0x9c7dc60d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc15dc60f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897dc619, SUC_lexicon_tags_6 },
    { 0x34fdc619, SUC_lexicon_tags_2 },
    { 0xef7dc61a, SUC_lexicon_tags_61 },
    { 0xbdddc61c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719dc621, SUC_lexicon_tags_11 },
    { 0x6f3dc621, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7dc625, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddc629, SUC_lexicon_tags_13 },
    { 0xea5dc62a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb89dc62c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5dc631, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9dc634, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ddc636, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x647dc638, SUC_lexicon_tags_7 },
    { 0xfd7dc638, SUC_lexicon_tags_11 },
    { 0xc03dc63a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fdc640, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91ddc644, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899dc648, SUC_lexicon_tags_38 },
    { 0x7d1dc649, SUC_lexicon_tags_50 },
    { 0x2bfdc64a, SUC_lexicon_tags_60 },
    { 0x7a5dc64b, SUC_lexicon_tags_2 },
    { 0xfcfdc64b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9dc650, SUC_lexicon_tags_17 },
    { 0xbb5dc650, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x685dc654, SUC_lexicon_tags_1 },
    { 0x1f1dc654, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdc658, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd1dc65a, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bdc65e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfdc661, SUC_lexicon_tags_27 },
    { 0x1dc662, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1e5dc664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233dc669, SUC_lexicon_tags_45 },
    { 0xaf9dc66a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7dc66d, SUC_lexicon_tags_17 },
    { 0xe9fdc66d, SUC_lexicon_tags_37 },
    { 0x7bfdc66e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbb9dc671, SUC_lexicon_tags_9 },
    { 0x9efdc672, SUC_lexicon_tags_39 },
    { 0x5a1dc673, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x16ddc675, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2addc677, SUC_lexicon_tags_17 },
    { 0x671dc678, SUC_lexicon_tags_37 },
    { 0x72ddc679, SUC_lexicon_tags_63 },
    { 0xdc1dc679, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccddc680, SUC_lexicon_tags_14 },
    { 0x4d1dc681, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc2bdc683, SUC_lexicon_tags_2 },
    { 0x489dc683, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x385dc687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x273dc689, SUC_lexicon_tags_7 },
    { 0x39dc68a, SUC_lexicon_tags_25 },
    { 0xa3dc68a, SUC_lexicon_tags_17 },
    { 0x8d3dc68c, SUC_lexicon_tags_68 },
    { 0xe27dc68c, SUC_lexicon_tags_38 },
    { 0x427dc68e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x155dc690, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883dc696, SUC_lexicon_tags_17 },
    { 0x91fdc697, SUC_lexicon_tags_7 },
    { 0x245dc696, SUC_lexicon_tags_1 },
    { 0x177dc699, SUC_lexicon_tags_15 },
    { 0x487dc69a, SUC_lexicon_tags_14 },
    { 0xe33dc69b, SUC_lexicon_tags_35 },
    { 0x187dc699, SUC_lexicon_tags_33 },
    { 0x123dc699, SUC_lexicon_tags_0 },
    { 0xed9dc69e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fdc6a1, SUC_lexicon_tags_10 },
    { 0xb61dc6a2, SUC_lexicon_tags_37 },
    { 0x7e1dc6a3, SUC_lexicon_tags_56 },
    { 0x131dc6a4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x70bdc6a6, SUC_lexicon_tags_23 },
    { 0xdcfdc6a7, SUC_lexicon_tags_138 },
    { 0x923dc6a8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x26fdc6aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21dc6ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e7dc6ae, SUC_lexicon_tags_27 },
    { 0xd9ddc6af, SUC_lexicon_tags_38 },
    { 0xa69dc6b0, SUC_lexicon_tags_26 },
    { 0x331dc6b0, SUC_lexicon_tags_12 },
    { 0xe39dc6b1, SUC_lexicon_tags_10 },
    { 0xa33dc6ae, SUC_lexicon_tags_7 },
    { 0x415dc6b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x723dc6b6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bdc6ba, SUC_lexicon_tags_25 },
    { 0x7afdc6bb, SUC_lexicon_tags_2 },
    { 0x41bdc6bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa9dc6be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x57bdc6c0, SUC_lexicon_tags_37 },
    { 0xbc5dc6c1, SUC_lexicon_tags_37 },
    { 0xfb3dc6c1, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x3abdc6c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1dc6cb, SUC_lexicon_tags_55 },
    { 0x1a9dc6cb, SUC_lexicon_tags_17 },
    { 0x815dc6cd, SUC_lexicon_tags_17 },
    { 0x5cfdc6ce, SUC_lexicon_tags_25 },
    { 0x8b5dc6cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcafdc6d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf1ddc6d3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x96fdc6d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121dc6db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89dc6de, SUC_lexicon_tags_17 },
    { 0xc01dc6df, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1dc6e2, SUC_lexicon_tags_25 },
    { 0x28fdc6e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddc6ea, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543dc6f1, SUC_lexicon_tags_12 },
    { 0x17bdc6f1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5dc6f4, SUC_lexicon_tags_29 },
    { 0xdddc6f4, SUC_lexicon_tags_17 },
    { 0x58bdc6f6, SUC_lexicon_tags_2 },
    { 0xb9fdc6f7, SUC_lexicon_tags_38 },
    { 0xbcfdc6f5, SUC_lexicon_tags_23 },
    { 0xc33dc6f5, SUC_lexicon_tags_48 },
    { 0xe37dc6f5, SUC_lexicon_tags_1 },
    { 0xa69dc6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x885dc6fd, SUC_lexicon_tags_38 },
    { 0x6e1dc6fe, SUC_lexicon_tags_30 },
    { 0x67dc6fe, SUC_lexicon_tags_12 },
    { 0xeb7dc6fe, SUC_lexicon_tags_77 },
    { 0xc49dc701, SUC_lexicon_tags_49 },
    { 0x4b9dc701, SUC_lexicon_tags_63 },
    { 0xf11dc703, SUC_lexicon_tags_15 },
    { 0x68bdc704, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573dc70e, SUC_lexicon_tags_23 },
    { 0x729dc70f, SUC_lexicon_tags_27 },
    { 0x13fdc710, SUC_lexicon_tags_30 },
    { 0x795dc711, SUC_lexicon_tags_38 },
    { 0x13dc712, SUC_lexicon_tags_11 },
    { 0xaf7dc713, SUC_lexicon_tags_37 },
    { 0xf2ddc70f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bddc719, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x535dc71b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525dc71f, SUC_lexicon_tags_14 },
    { 0xc51dc720, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7dc723, SUC_lexicon_tags_11 },
    { 0x541dc724, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe03dc726, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fdc72b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc85dc72d, SUC_lexicon_tags_11 },
    { 0x781dc72e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x719dc731, SUC_lexicon_tags_37 },
    { 0xfcddc732, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321dc73b, SUC_lexicon_tags_12 },
    { 0x839dc73c, SUC_lexicon_tags_14 },
    { 0x803dc73d, SUC_lexicon_tags_11 },
    { 0xe33dc73d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221dc746, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ddc74a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x869dc74c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf63dc74e, SUC_lexicon_tags_15 },
    { 0xebbdc74e, SUC_lexicon_tags_4 },
    { 0x283dc750, SUC_lexicon_tags_25 },
    { 0x8a9dc74e, SUC_lexicon_tags_63 },
    { 0x3d5dc750, SUC_lexicon_tags_87 },
    { 0x311dc753, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3dc756, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb43dc758, SUC_lexicon_tags_4 },
    { 0x5c5dc759, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7dc75f, SUC_lexicon_tags_37 },
    { 0xe85dc75f, SUC_lexicon_tags_17 },
    { 0x5dc760, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fdc766, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53dc76b, SUC_lexicon_tags_26 },
    { 0x8f1dc76b, SUC_lexicon_tags_27 },
    { 0x893dc76d, SUC_lexicon_tags_11 },
    { 0x9d1dc76b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb95dc770, SUC_lexicon_tags_140 },
    { 0x6d9dc770, SUC_lexicon_tags_68 },
    { 0x539dc772, SUC_lexicon_tags_14 },
    { 0x2f7dc773, SUC_lexicon_tags_14 },
    { 0x70bdc772, SUC_lexicon_tags_12 },
    { 0x373dc775, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x38ddc777, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe77dc779, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x30fdc77b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5dc77e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9dc782, SUC_lexicon_tags_9 },
    { 0x709dc783, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x24fdc785, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dc788, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03dc78b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8fddc78d, SUC_lexicon_tags_17 },
    { 0x94ddc78d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75dc790, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x787dc792, SUC_lexicon_tags_47 },
    { 0xdc9dc793, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1dc797, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595dc79d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bddc7a1, SUC_lexicon_tags_23 },
    { 0x65fdc7a2, SUC_lexicon_tags_26 },
    { 0x605dc7a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecddc7a7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x92fdc7a9, SUC_lexicon_tags_12 },
    { 0xac5dc7a9, SUC_lexicon_tags_9 },
    { 0x139dc7ab, SUC_lexicon_tags_107 },
    { 0xdebdc7ac, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x62bdc7ae, SUC_lexicon_tags_65 },
    { 0xacbdc7af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ddc7b5, SUC_lexicon_tags_1 },
    { 0xf2ddc7b5, SUC_lexicon_tags_15 },
    { 0x2bdc7b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309dc7bc, SUC_lexicon_tags_12 },
    { 0xf37dc7bd, SUC_lexicon_tags_7 },
    { 0x275dc7be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cddc7c1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3dc7c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fdc7ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09dc7d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7dfdc7d4, SUC_lexicon_tags_70 },
    { 0x377dc7d4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83dc7da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x589dc7dd, SUC_lexicon_tags_9 },
    { 0xf47dc7dd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643dc7e2, SUC_lexicon_tags_2 },
    { 0x611dc7e3, SUC_lexicon_tags_23 },
    { 0xb0bdc7e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81bdc7e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xec5dc7e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7dc7ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3dc7f2, SUC_lexicon_tags_0 },
    { 0x6d7dc7f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe63dc7f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21dc7fc, SUC_lexicon_tags_23 },
    { 0x41bdc7fd, SUC_lexicon_tags_15 },
    { 0x583dc7fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eddc802, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x141dc804, SUC_lexicon_tags_148 },
    { 0xe23dc805, SUC_lexicon_tags_37 },
    { 0x11fdc804, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x929dc808, SUC_lexicon_tags_19 },
    { 0x43bdc808, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281dc80d, SUC_lexicon_tags_39 },
    { 0xa4ddc80e, SUC_lexicon_tags_17 },
    { 0x4b5dc80d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb49dc811, SUC_lexicon_tags_17 },
    { 0xe2fdc812, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf05dc814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97dc817, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8ddc819, SUC_lexicon_tags_17 },
    { 0xbd9dc819, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fdc81e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf11dc820, SUC_lexicon_tags_14 },
    { 0x187dc821, SUC_lexicon_tags_37 },
    { 0x1ffdc822, SUC_lexicon_tags_37 },
    { 0x90bdc822, SUC_lexicon_tags_84 },
    { 0x95ddc821, SUC_lexicon_tags_39 },
    { 0xbafdc820, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57dc828, SUC_lexicon_tags_49 },
    { 0x9ffdc828, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbdc82c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9cfdc82e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fdc835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x979dc838, SUC_lexicon_tags_57 },
    { 0x46bdc838, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fdc83d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1dc840, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7dc844, SUC_lexicon_tags_17 },
    { 0xcbdc845, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x823dc848, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5bfdc84a, SUC_lexicon_tags_12 },
    { 0x9d7dc84b, SUC_lexicon_tags_49 },
    { 0x1addc84a, SUC_lexicon_tags_17 },
    { 0x31dc84d, SUC_lexicon_tags_8 },
    { 0xd4ddc84d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fdc851, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x357dc853, SUC_lexicon_tags_7 },
    { 0xc45dc853, SUC_lexicon_tags_9 },
    { 0xb47dc854, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebdc859, SUC_lexicon_tags_2 },
    { 0x5d3dc85a, SUC_lexicon_tags_34 },
    { 0x189dc85a, SUC_lexicon_tags_27 },
    { 0x5dddc85c, SUC_lexicon_tags_39 },
    { 0xdddc85a, SUC_lexicon_tags_17 },
    { 0x8dddc859, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ddc861, SUC_lexicon_tags_17 },
    { 0x247dc862, SUC_lexicon_tags_156 },
    { 0x85bdc861, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x483dc865, SUC_lexicon_tags_31 },
    { 0x661dc865, SUC_lexicon_tags_60 },
    { 0x195dc866, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb45dc86a, SUC_lexicon_tags_37 },
    { 0x2a9dc86b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3dc86e, SUC_lexicon_tags_19 },
    { 0xaf3dc86f, SUC_lexicon_tags_15 },
    { 0x63ddc870, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x489dc872, SUC_lexicon_tags_37 },
    { 0x9a5dc873, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x203dc876, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2ddc878, SUC_lexicon_tags_7 },
    { 0x9fbdc879, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511dc87e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fdc882, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13bdc884, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfdc888, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd25dc88a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x959dc88c, SUC_lexicon_tags_7 },
    { 0xf87dc88c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc53dc88f, SUC_lexicon_tags_15 },
    { 0x7a9dc890, SUC_lexicon_tags_25 },
    { 0xa67dc891, SUC_lexicon_tags_38 },
    { 0x505dc890, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fdc895, SUC_lexicon_tags_1 },
    { 0x651dc895, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43dc899, SUC_lexicon_tags_17 },
    { 0x9a3dc899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbddc89f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3dc8a3, SUC_lexicon_tags_47 },
    { 0xb6bdc8a4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3dc8a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x493dc8a9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79dc8ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf85dc8ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bdc8b1, SUC_lexicon_tags_19 },
    { 0x7eddc8b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d7dc8b4, SUC_lexicon_tags_21 },
    { 0x3a7dc8b5, SUC_lexicon_tags_26 },
    { 0x1abdc8b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x547dc8b8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bdc8bb, SUC_lexicon_tags_27 },
    { 0xe65dc8bc, SUC_lexicon_tags_64 },
    { 0x55bdc8bd, SUC_lexicon_tags_33 },
    { 0xa87dc8be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x775dc8c1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1dc8c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfddc8cc, SUC_lexicon_tags_17 },
    { 0x90bdc8cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fbdc8cf, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ddc8d9, SUC_lexicon_tags_27 },
    { 0x4dbdc8da, SUC_lexicon_tags_25 },
    { 0xd1fdc8db, SUC_lexicon_tags_68 },
    { 0xe61dc8da, SUC_lexicon_tags_12 },
    { 0x4c7dc8dd, SUC_lexicon_tags_7 },
    { 0x61bdc8da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467dc8e4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xbddc8e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebdc8f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973dc8f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fdc8fa, SUC_lexicon_tags_2 },
    { 0xa61dc8fb, SUC_lexicon_tags_25 },
    { 0xc4ddc8fc, SUC_lexicon_tags_0 },
    { 0xd8fdc8fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x23bdc8ff, SUC_lexicon_tags_37 },
    { 0x283dc8ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe75dc902, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dc905, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bdc908, SUC_lexicon_tags_31 },
    { 0xfbbdc909, SUC_lexicon_tags_55 },
    { 0x67fdc909, SUC_lexicon_tags_37 },
    { 0x9fdc90b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x729dc90d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7d7dc90f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c5dc911, SUC_lexicon_tags_2 },
    { 0x3b9dc912, SUC_lexicon_tags_39 },
    { 0x62bdc913, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399dc917, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x395dc919, SUC_lexicon_tags_1 },
    { 0x49bdc91a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6fddc91c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x897dc91f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x92ddc921, SUC_lexicon_tags_26 },
    { 0x27fdc921, SUC_lexicon_tags_37 },
    { 0x63dc922, SUC_lexicon_tags_6 },
    { 0x79ddc922, SUC_lexicon_tags_9 },
    { 0x3a3dc921, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ddc928, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7dc92c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dddc931, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11dc934, SUC_lexicon_tags_87 },
    { 0x765dc934, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83dc938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397dc93c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3dc93f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593dc944, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x803dc946, SUC_lexicon_tags_3 },
    { 0xffddc947, SUC_lexicon_tags_997 },
    { 0x57bdc947, SUC_lexicon_tags_11 },
    { 0xb03dc946, SUC_lexicon_tags_378 },
    { 0x65ddc94a, SUC_lexicon_tags_45 },
    { 0xeddc946, SUC_lexicon_tags_62 },
    { 0x2bddc94c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fdc950, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc19dc952, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641dc958, SUC_lexicon_tags_6 },
    { 0x777dc959, SUC_lexicon_tags_2 },
    { 0x9dfdc959, SUC_lexicon_tags_7 },
    { 0xb97dc95b, SUC_lexicon_tags_49 },
    { 0xd93dc95b, SUC_lexicon_tags_16 },
    { 0xb03dc95b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663dc963, SUC_lexicon_tags_7 },
    { 0x741dc963, SUC_lexicon_tags_37 },
    { 0x9efdc964, SUC_lexicon_tags_17 },
    { 0xb6ddc966, SUC_lexicon_tags_9 },
    { 0xeb7dc967, SUC_lexicon_tags_7 },
    { 0xbc9dc963, SUC_lexicon_tags_15 },
    { 0xe4ddc967, SUC_lexicon_tags_17 },
    { 0xcbdc96a, SUC_lexicon_tags_38 },
    { 0xd17dc963, SUC_lexicon_tags_37 },
    { 0x263dc96b, SUC_lexicon_tags_27 },
    { 0x20fdc96d, SUC_lexicon_tags_19 },
    { 0xc85dc96e, SUC_lexicon_tags_77 },
    { 0x95ddc96d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe6bdc971, SUC_lexicon_tags_31 },
    { 0x97dc972, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bdc979, SUC_lexicon_tags_61 },
    { 0xe13dc97a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fdc97d, SUC_lexicon_tags_29 },
    { 0xd91dc97e, SUC_lexicon_tags_2 },
    { 0x38fdc97e, SUC_lexicon_tags_43 },
    { 0x27fdc980, SUC_lexicon_tags_7 },
    { 0x6f5dc980, SUC_lexicon_tags_3 },
    { 0x6a1dc980, SUC_lexicon_tags_17 },
    { 0xc4fdc97d, SUC_lexicon_tags_17 },
    { 0x77fdc984, SUC_lexicon_tags_14 },
    { 0x30ddc985, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99dc98a, SUC_lexicon_tags_28 },
    { 0xba5dc98a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9dc98f, SUC_lexicon_tags_108 },
    { 0x793dc990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ddc994, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffdc999, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dc99e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x273dc9a1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5f3dc9a3, SUC_lexicon_tags_11 },
    { 0x91bdc9a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5dc9a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4d7dc9ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65dc9af, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dc9b5, SUC_lexicon_tags_28 },
    { 0x243dc9b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59dc9b9, SUC_lexicon_tags_16 },
    { 0x779dc9ba, SUC_lexicon_tags_14 },
    { 0x6f7dc9ba, SUC_lexicon_tags_12 },
    { 0x789dc9b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf59dc9be, SUC_lexicon_tags_23 },
    { 0xf77dc9bf, SUC_lexicon_tags_32 },
    { 0x491dc9bf, SUC_lexicon_tags_23 },
    { 0x715dc9c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3afdc9c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdafdc9c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ddc9cc, SUC_lexicon_tags_1 },
    { 0xbc3dc9cd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bdc9d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4eddc9d3, SUC_lexicon_tags_18 },
    { 0x6a9dc9d4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5dc9df, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9eddc9e2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa09dc9e4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5dc9e8, SUC_lexicon_tags_25 },
    { 0x45fdc9e9, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13dc9ed, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51dc9f0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7dc9f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x369dc9f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9ddc9fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43dca02, SUC_lexicon_tags_111 },
    { 0xc8ddca02, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fdca08, SUC_lexicon_tags_1 },
    { 0xf41dca09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ddca11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75dca15, SUC_lexicon_tags_35 },
    { 0xe1dca15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x39dca18, SUC_lexicon_tags_37 },
    { 0x23bdca18, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bdca21, SUC_lexicon_tags_17 },
    { 0x747dca22, SUC_lexicon_tags_25 },
    { 0x241dca22, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x839dca25, SUC_lexicon_tags_2 },
    { 0xf19dca25, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x523dca28, SUC_lexicon_tags_11 },
    { 0x239dca28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5dca33, SUC_lexicon_tags_43 },
    { 0xbdbdca34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x671dca36, SUC_lexicon_tags_15 },
    { 0x683dca37, SUC_lexicon_tags_47 },
    { 0xed9dca37, SUC_lexicon_tags_60 },
    { 0xabddca38, SUC_lexicon_tags_7 },
    { 0xf87dca3a, SUC_lexicon_tags_32 },
    { 0x30bdca3b, SUC_lexicon_tags_21 },
    { 0x4afdca3c, SUC_lexicon_tags_25 },
    { 0x379dca3d, SUC_lexicon_tags_11 },
    { 0xdb7dca3e, SUC_lexicon_tags_3 },
    { 0x2b5dca3b, SUC_lexicon_tags_39 },
    { 0x5fddca38, SUC_lexicon_tags_42 },
    { 0xaf1dca41, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xef5dca43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x217dca45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bdca4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3dca4f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ddca52, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefdca55, SUC_lexicon_tags_18 },
    { 0xaffdca56, SUC_lexicon_tags_2 },
    { 0x7f5dca57, SUC_lexicon_tags_7 },
    { 0xb67dca58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb69dca5a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9dca5d, SUC_lexicon_tags_25 },
    { 0x7fdca5d, SUC_lexicon_tags_17 },
    { 0x735dca5d, SUC_lexicon_tags_1 },
    { 0x8dfdca5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921dca65, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffdca68, SUC_lexicon_tags_22 },
    { 0x1e3dca68, SUC_lexicon_tags_12 },
    { 0xae7dca6a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bdca6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fdca72, SUC_lexicon_tags_12 },
    { 0x585dca72, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125dca7a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547dca8d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3addca8f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855dca94, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf1dca96, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x303dca99, SUC_lexicon_tags_49 },
    { 0x3a1dca9a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x265dca9c, SUC_lexicon_tags_15 },
    { 0xb5fdca9d, SUC_lexicon_tags_25 },
    { 0xd93dca9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cddcaa2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x44bdcaa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9dcaa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bdcaab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ddcaaf, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3dddcab1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbdcab4, SUC_lexicon_tags_1 },
    { 0x6e7dcab5, SUC_lexicon_tags_558 },
    { 0, NULL },
    { 0x76fdcab7, SUC_lexicon_tags_2 },
    { 0x55ddcab8, SUC_lexicon_tags_29 },
    { 0xc6bdcab9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99dcabd, SUC_lexicon_tags_0 },
    { 0xbe5dcabe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3dcac3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1dcac5, SUC_lexicon_tags_37 },
    { 0x9addcac6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xad7dcac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbdcacc, SUC_lexicon_tags_63 },
    { 0x6f5dcacd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xec5dcacf, SUC_lexicon_tags_27 },
    { 0x8bfdcad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfdcad4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x785dcad6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbdcadc, SUC_lexicon_tags_37 },
    { 0xfb1dcadc, SUC_lexicon_tags_49 },
    { 0x177dcade, SUC_lexicon_tags_39 },
    { 0xfdcadf, SUC_lexicon_tags_10 },
    { 0xd49dcae0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb7dcae2, SUC_lexicon_tags_88 },
    { 0xc47dcae3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa3fdcae5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53dcae8, SUC_lexicon_tags_9 },
    { 0x4b5dcae9, SUC_lexicon_tags_15 },
    { 0x919dcaea, SUC_lexicon_tags_27 },
    { 0x4a3dcae9, SUC_lexicon_tags_27 },
    { 0x3a1dcaea, SUC_lexicon_tags_30 },
    { 0x453dcaec, SUC_lexicon_tags_19 },
    { 0x371dcaec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bdcaf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fdcafa, SUC_lexicon_tags_2 },
    { 0x615dcafb, SUC_lexicon_tags_4 },
    { 0xa8ddcafb, SUC_lexicon_tags_19 },
    { 0xfbfdcafa, SUC_lexicon_tags_31 },
    { 0xf45dcafe, SUC_lexicon_tags_44 },
    { 0xfb1dcaff, SUC_lexicon_tags_11 },
    { 0xdafdcb00, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17dcb04, SUC_lexicon_tags_14 },
    { 0xd13dcb05, SUC_lexicon_tags_2 },
    { 0xb85dcb06, SUC_lexicon_tags_35 },
    { 0x313dcb07, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbdddcb09, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc01dcb0b, SUC_lexicon_tags_7 },
    { 0x59dcb0c, SUC_lexicon_tags_12 },
    { 0x397dcb0d, SUC_lexicon_tags_1 },
    { 0x9bbdcb0e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bdcb12, SUC_lexicon_tags_27 },
    { 0xb71dcb13, SUC_lexicon_tags_2 },
    { 0xd09dcb13, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5dcb1b, SUC_lexicon_tags_14 },
    { 0xfe9dcb1c, SUC_lexicon_tags_623 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bdcb21, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995dcb2b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841dcb32, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x461dcb35, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6b9dcb37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bdcb3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x271dcb3d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7dcb40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473dcb44, SUC_lexicon_tags_15 },
    { 0xc83dcb45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55dcb48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x22bdcb4b, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0xa25dcb4d, SUC_lexicon_tags_30 },
    { 0xfdbdcb4d, SUC_lexicon_tags_70 },
    { 0xc9dcb4f, SUC_lexicon_tags_17 },
    { 0x335dcb4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1dcb57, SUC_lexicon_tags_25 },
    { 0x61fdcb57, SUC_lexicon_tags_1 },
    { 0x79bdcb57, SUC_lexicon_tags_27 },
    { 0xf2ddcb59, SUC_lexicon_tags_15 },
    { 0xb31dcb5b, SUC_lexicon_tags_28 },
    { 0xb75dcb5c, SUC_lexicon_tags_63 },
    { 0xa17dcb5b, SUC_lexicon_tags_7 },
    { 0x759dcb59, SUC_lexicon_tags_25 },
    { 0x6e1dcb5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ddcb64, SUC_lexicon_tags_17 },
    { 0x81bdcb65, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521dcb69, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91dcb70, SUC_lexicon_tags_3 },
    { 0xe4ddcb71, SUC_lexicon_tags_25 },
    { 0x615dcb72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65dcb76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb2ddcb78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319dcb7c, SUC_lexicon_tags_62 },
    { 0x501dcb7d, SUC_lexicon_tags_39 },
    { 0x4c5dcb7d, SUC_lexicon_tags_17 },
    { 0x355dcb7f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xecbdcb81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7dcb83, SUC_lexicon_tags_17 },
    { 0x6e1dcb83, SUC_lexicon_tags_37 },
    { 0xd0bdcb85, SUC_lexicon_tags_14 },
    { 0x599dcb86, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fdcb89, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9dcb8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e9dcb8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287dcb95, SUC_lexicon_tags_43 },
    { 0xc91dcb96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33bdcb98, SUC_lexicon_tags_5 },
    { 0x223dcb98, SUC_lexicon_tags_11 },
    { 0xd1fdcb99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881dcb9e, SUC_lexicon_tags_25 },
    { 0xdddcb9f, SUC_lexicon_tags_60 },
    { 0x687dcba0, SUC_lexicon_tags_61 },
    { 0x331dcba1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x571dcba3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35dcba8, SUC_lexicon_tags_49 },
    { 0x7addcba8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd5dcbab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fdcbae, SUC_lexicon_tags_38 },
    { 0x499dcbaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fdcbb2, SUC_lexicon_tags_33 },
    { 0x315dcbb2, SUC_lexicon_tags_2 },
    { 0x4b5dcbb4, SUC_lexicon_tags_32 },
    { 0x769dcbb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ddcbba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x447dcbbd, SUC_lexicon_tags_60 },
    { 0xd03dcbbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9cbdcbc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0fdcbc2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x295dcbc4, SUC_lexicon_tags_10 },
    { 0xc05dcbc5, SUC_lexicon_tags_29 },
    { 0xb33dcbc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x21bdcbc9, SUC_lexicon_tags_23 },
    { 0xeffdcbc9, SUC_lexicon_tags_9 },
    { 0x327dcbca, SUC_lexicon_tags_43 },
    { 0xa1bdcbcc, SUC_lexicon_tags_12 },
    { 0xbd7dcbca, SUC_lexicon_tags_13 },
    { 0x9e1dcbca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fdcbd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233dcbd7, SUC_lexicon_tags_11 },
    { 0x93ddcbd8, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe61dcbda, SUC_lexicon_tags_17 },
    { 0x3e7dcbdb, SUC_lexicon_tags_37 },
    { 0xd8bdcbda, SUC_lexicon_tags_12 },
    { 0xa11dcbdd, SUC_lexicon_tags_33 },
    { 0x7a3dcbde, SUC_lexicon_tags_12 },
    { 0x957dcbda, SUC_lexicon_tags_1 },
    { 0x49fdcbe0, SUC_lexicon_tags_11 },
    { 0xd75dcbe1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x153dcbe4, SUC_lexicon_tags_37 },
    { 0x73fdcbe5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9dcbe9, SUC_lexicon_tags_14 },
    { 0xc89dcbea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x899dcbec, SUC_lexicon_tags_43 },
    { 0x791dcbed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x617dcbef, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fdcbf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x359dcbf5, SUC_lexicon_tags_7 },
    { 0xaf5dcbf5, SUC_lexicon_tags_6 },
    { 0xd55dcbf7, SUC_lexicon_tags_11 },
    { 0x4c9dcbf7, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xeabdcbfa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221dcc00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcddcc03, SUC_lexicon_tags_3 },
    { 0xebddcc04, SUC_lexicon_tags_48 },
    { 0xddcc04, SUC_lexicon_tags_15 },
    { 0x9bdcc06, SUC_lexicon_tags_37 },
    { 0x9a3dcc07, SUC_lexicon_tags_49 },
    { 0xb45dcc03, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465dcc12, SUC_lexicon_tags_15 },
    { 0xb69dcc13, SUC_lexicon_tags_26 },
    { 0xa67dcc14, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc45dcc16, SUC_lexicon_tags_17 },
    { 0x713dcc17, SUC_lexicon_tags_5 },
    { 0x45bdcc18, SUC_lexicon_tags_25 },
    { 0xbf7dcc18, SUC_lexicon_tags_11 },
    { 0x89dcc18, SUC_lexicon_tags_60 },
    { 0xfdbdcc1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dcc22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x131dcc24, SUC_lexicon_tags_3 },
    { 0x727dcc24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41ddcc27, SUC_lexicon_tags_21 },
    { 0x9fbdcc28, SUC_lexicon_tags_17 },
    { 0x797dcc28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31dcc2c, SUC_lexicon_tags_43 },
    { 0x8a3dcc2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x277dcc2f, SUC_lexicon_tags_8 },
    { 0x7b1dcc2f, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ddcc36, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5dcc39, SUC_lexicon_tags_14 },
    { 0xc63dcc39, SUC_lexicon_tags_12 },
    { 0x63bdcc3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2afdcc3e, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47dcc45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ddcc49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xff9dcc4b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5dcc51, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x971dcc53, SUC_lexicon_tags_384 },
    { 0, NULL },
    { 0xa83dcc55, SUC_lexicon_tags_0 },
    { 0x1ddcc56, SUC_lexicon_tags_8 },
    { 0x63ddcc57, SUC_lexicon_tags_25 },
    { 0xfb9dcc58, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfabdcc5a, SUC_lexicon_tags_2 },
    { 0xefdcc5b, SUC_lexicon_tags_7 },
    { 0x315dcc5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3dcc60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1dcc66, SUC_lexicon_tags_11 },
    { 0x80ddcc67, SUC_lexicon_tags_23 },
    { 0x5efdcc67, SUC_lexicon_tags_34 },
    { 0x91bdcc69, SUC_lexicon_tags_37 },
    { 0xe2bdcc69, SUC_lexicon_tags_33 },
    { 0x2a3dcc69, SUC_lexicon_tags_10 },
    { 0x343dcc6b, SUC_lexicon_tags_7 },
    { 0x401dcc6d, SUC_lexicon_tags_62 },
    { 0xd79dcc6e, SUC_lexicon_tags_7 },
    { 0xf57dcc6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x511dcc72, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaddcc74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfddcc76, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1dcc79, SUC_lexicon_tags_21 },
    { 0x1b7dcc7a, SUC_lexicon_tags_17 },
    { 0x48ddcc7a, SUC_lexicon_tags_8 },
    { 0xbc9dcc7a, SUC_lexicon_tags_63 },
    { 0xcfbdcc7d, SUC_lexicon_tags_63 },
    { 0xcb7dcc7b, SUC_lexicon_tags_38 },
    { 0xb1ddcc7f, SUC_lexicon_tags_17 },
    { 0x14fdcc80, SUC_lexicon_tags_45 },
    { 0xd07dcc7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bdcc86, SUC_lexicon_tags_19 },
    { 0x745dcc86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bdcc8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28ddcc8e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebdcc91, SUC_lexicon_tags_8 },
    { 0x399dcc92, SUC_lexicon_tags_43 },
    { 0x6bdcc91, SUC_lexicon_tags_1 },
    { 0x149dcc94, SUC_lexicon_tags_57 },
    { 0x5bbdcc91, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddcc99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27dcc9f, SUC_lexicon_tags_60 },
    { 0x789dcc9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7dcca4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bdcca9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83dccad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5dccb7, SUC_lexicon_tags_25 },
    { 0xa3dccb7, SUC_lexicon_tags_25 },
    { 0x6bfdccb9, SUC_lexicon_tags_47 },
    { 0xd8ddccb9, SUC_lexicon_tags_1 },
    { 0xa33dccbb, SUC_lexicon_tags_2 },
    { 0x57bdccbc, SUC_lexicon_tags_79 },
    { 0xd03dccb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7dccc0, SUC_lexicon_tags_11 },
    { 0xe5bdccc1, SUC_lexicon_tags_23 },
    { 0x785dccc2, SUC_lexicon_tags_15 },
    { 0xe43dccc2, SUC_lexicon_tags_64 },
    { 0xcb3dccc2, SUC_lexicon_tags_7 },
    { 0xeb3dccc5, SUC_lexicon_tags_0 },
    { 0xf09dccc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5dccc9, SUC_lexicon_tags_32 },
    { 0x483dccca, SUC_lexicon_tags_27 },
    { 0xd5bdcccb, SUC_lexicon_tags_12 },
    { 0xdf9dcccc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe69dccce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95dccd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x147dccd5, SUC_lexicon_tags_39 },
    { 0xb99dccd6, SUC_lexicon_tags_4 },
    { 0x6eddccd7, SUC_lexicon_tags_14 },
    { 0xb2fdccd8, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x637dccda, SUC_lexicon_tags_33 },
    { 0xc43dccdb, SUC_lexicon_tags_11 },
    { 0xa3bdccdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x521dccdf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa37dcce1, SUC_lexicon_tags_3 },
    { 0x463dcce1, SUC_lexicon_tags_37 },
    { 0xd9ddcce3, SUC_lexicon_tags_37 },
    { 0x7d3dcce3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07dccec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99dccee, SUC_lexicon_tags_12 },
    { 0x26fdccef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x853dccf1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd7dccf3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8dddccf5, SUC_lexicon_tags_25 },
    { 0x585dccf6, SUC_lexicon_tags_37 },
    { 0x101dccf6, SUC_lexicon_tags_11 },
    { 0x5d7dccf8, SUC_lexicon_tags_16 },
    { 0x21bdccf9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbdccfc, SUC_lexicon_tags_7 },
    { 0xbf3dccfd, SUC_lexicon_tags_37 },
    { 0x705dccfe, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ddcd06, SUC_lexicon_tags_45 },
    { 0xf2fdcd06, SUC_lexicon_tags_17 },
    { 0x487dcd08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bbdcd0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ddcd11, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfdcd14, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629dcd1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e7dcd1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793dcd21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7dcd2a, SUC_lexicon_tags_2 },
    { 0x3d9dcd2b, SUC_lexicon_tags_31 },
    { 0xb7fdcd2c, SUC_lexicon_tags_7 },
    { 0xc9fdcd2d, SUC_lexicon_tags_37 },
    { 0x419dcd2e, SUC_lexicon_tags_12 },
    { 0xf87dcd2f, SUC_lexicon_tags_14 },
    { 0xa17dcd2e, SUC_lexicon_tags_12 },
    { 0x7c7dcd2f, SUC_lexicon_tags_25 },
    { 0xaf1dcd32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ddcd37, SUC_lexicon_tags_38 },
    { 0xfb9dcd38, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfdcd3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7dcd48, SUC_lexicon_tags_29 },
    { 0x537dcd49, SUC_lexicon_tags_10 },
    { 0x187dcd4a, SUC_lexicon_tags_54 },
    { 0x26bdcd4b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5dcd54, SUC_lexicon_tags_7 },
    { 0x9f7dcd55, SUC_lexicon_tags_15 },
    { 0xb23dcd56, SUC_lexicon_tags_15 },
    { 0x1ffdcd55, SUC_lexicon_tags_0 },
    { 0xaa1dcd57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcebdcd5a, SUC_lexicon_tags_40 },
    { 0x25bdcd5a, SUC_lexicon_tags_11 },
    { 0xbdcd5a, SUC_lexicon_tags_11 },
    { 0x88ddcd5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x269dcd5f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891dcd64, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5bddcd66, SUC_lexicon_tags_1 },
    { 0x225dcd67, SUC_lexicon_tags_0 },
    { 0x74bdcd68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763dcd6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319dcd71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbdcd74, SUC_lexicon_tags_37 },
    { 0x3f7dcd74, SUC_lexicon_tags_37 },
    { 0x3dddcd74, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdbfdcd78, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279dcd7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61dcd81, SUC_lexicon_tags_3 },
    { 0x67ddcd82, SUC_lexicon_tags_37 },
    { 0x893dcd82, SUC_lexicon_tags_14 },
    { 0x89ddcd81, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c5dcd86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fdcd8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x923dcd8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d1dcd8f, SUC_lexicon_tags_27 },
    { 0xb4fdcd90, SUC_lexicon_tags_9 },
    { 0xa09dcd91, SUC_lexicon_tags_12 },
    { 0xf25dcd8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf85dcd94, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7dcd97, SUC_lexicon_tags_14 },
    { 0x13fdcd98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843dcd9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611dcda2, SUC_lexicon_tags_9 },
    { 0x85dcda3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947dcda7, SUC_lexicon_tags_66 },
    { 0xf05dcda8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfdcdab, SUC_lexicon_tags_14 },
    { 0xc97dcdac, SUC_lexicon_tags_11 },
    { 0xed9dcdad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3dcdb2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa5fdcdb4, SUC_lexicon_tags_0 },
    { 0xa45dcdb4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x36ddcdb7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ddcdbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377dcdc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5dcdc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfdcdca, SUC_lexicon_tags_7 },
    { 0xa55dcdcb, SUC_lexicon_tags_0 },
    { 0x5e9dcdcb, SUC_lexicon_tags_2 },
    { 0x2ddcdcd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5dcdd2, SUC_lexicon_tags_38 },
    { 0x841dcdd3, SUC_lexicon_tags_30 },
    { 0xa8fdcdd4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87dcdd9, SUC_lexicon_tags_46 },
    { 0xb9fdcdda, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x44bdcddc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd59dcdde, SUC_lexicon_tags_38 },
    { 0x65fdcddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fdcde3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3dcde8, SUC_lexicon_tags_14 },
    { 0xb61dcde8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581dcdf0, SUC_lexicon_tags_83 },
    { 0xadbdcdf1, SUC_lexicon_tags_2 },
    { 0xa2ddcdf1, SUC_lexicon_tags_1368 },
    { 0x51dcdf3, SUC_lexicon_tags_19 },
    { 0xda5dcdf3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9dcdfa, SUC_lexicon_tags_33 },
    { 0x9bfdcdfb, SUC_lexicon_tags_0 },
    { 0xc19dcdfa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fdce01, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fdce05, SUC_lexicon_tags_37 },
    { 0xe03dce05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc25dce08, SUC_lexicon_tags_47 },
    { 0x59ddce09, SUC_lexicon_tags_37 },
    { 0x7d9dce0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977dce0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbdce11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8efdce13, SUC_lexicon_tags_37 },
    { 0x12ddce14, SUC_lexicon_tags_45 },
    { 0xbf5dce15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7cddce17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbdce1b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcd3dce1d, SUC_lexicon_tags_26 },
    { 0xa73dce1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x289dce20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52bdce22, SUC_lexicon_tags_6 },
    { 0x9b7dce23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x295dce26, SUC_lexicon_tags_25 },
    { 0x5ebdce26, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1dfdce29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7dce2b, SUC_lexicon_tags_1 },
    { 0x7a7dce2b, SUC_lexicon_tags_16 },
    { 0xd91dce2d, SUC_lexicon_tags_25 },
    { 0x803dce2e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fdce32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7dce36, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x943dce39, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbdce3f, SUC_lexicon_tags_15 },
    { 0x311dce40, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x507dce43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bdce47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27dce4d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bdce50, SUC_lexicon_tags_1 },
    { 0x391dce50, SUC_lexicon_tags_4 },
    { 0xfc7dce52, SUC_lexicon_tags_2 },
    { 0x31ddce51, SUC_lexicon_tags_15 },
    { 0xc3ddce54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1dce57, SUC_lexicon_tags_37 },
    { 0x5c5dce58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9dce5c, SUC_lexicon_tags_26 },
    { 0x8a7dce5c, SUC_lexicon_tags_37 },
    { 0x48fdce5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fdce63, SUC_lexicon_tags_14 },
    { 0x425dce64, SUC_lexicon_tags_60 },
    { 0xd6fdce65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bdce6b, SUC_lexicon_tags_25 },
    { 0x25bdce6c, SUC_lexicon_tags_33 },
    { 0x1f9dce6d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9dce72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913dce77, SUC_lexicon_tags_0 },
    { 0xafbdce77, SUC_lexicon_tags_2 },
    { 0x9e5dce77, SUC_lexicon_tags_45 },
    { 0x8a3dce7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7f1dce7c, SUC_lexicon_tags_4 },
    { 0x851dce7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7dce80, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ddce85, SUC_lexicon_tags_25 },
    { 0x84fdce85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fdce8a, SUC_lexicon_tags_11 },
    { 0xfe5dce8a, SUC_lexicon_tags_3 },
    { 0x7d5dce8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5e9dce8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5d5dce90, SUC_lexicon_tags_37 },
    { 0x67dce91, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x627dce93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x341dce96, SUC_lexicon_tags_23 },
    { 0x191dce97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dce9b, SUC_lexicon_tags_79 },
    { 0xba3dce9c, SUC_lexicon_tags_37 },
    { 0x3a9dce9c, SUC_lexicon_tags_17 },
    { 0x8addce9b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5dbdcea0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37dcea8, SUC_lexicon_tags_37 },
    { 0x1cbdcea9, SUC_lexicon_tags_26 },
    { 0xbc3dceaa, SUC_lexicon_tags_25 },
    { 0xef5dceab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dceaf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6bddceb1, SUC_lexicon_tags_87 },
    { 0x5f5dceb1, SUC_lexicon_tags_16 },
    { 0x5a7dceb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fdceb8, SUC_lexicon_tags_19 },
    { 0x425dceb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dcec1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3dcec5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73fdcec7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf59dcec9, SUC_lexicon_tags_25 },
    { 0xa77dcec9, SUC_lexicon_tags_19 },
    { 0xa17dcecb, SUC_lexicon_tags_33 },
    { 0x7e1dcecb, SUC_lexicon_tags_11 },
    { 0x35fdcecd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ddceda, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7dcedd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xca5dcee0, SUC_lexicon_tags_17 },
    { 0x9eddcee1, SUC_lexicon_tags_27 },
    { 0x505dcee1, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9dcee9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37dceed, SUC_lexicon_tags_17 },
    { 0x3c3dceee, SUC_lexicon_tags_19 },
    { 0xeb5dceee, SUC_lexicon_tags_1 },
    { 0xf5fdceee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45dcef3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x57dcef6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ddcefe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bdcf02, SUC_lexicon_tags_12 },
    { 0x65fdcf02, SUC_lexicon_tags_27 },
    { 0xc9ddcf02, SUC_lexicon_tags_15 },
    { 0xf19dcf03, SUC_lexicon_tags_33 },
    { 0x61ddcf04, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bdcf09, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fdcf0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183dcf14, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5dcf17, SUC_lexicon_tags_79 },
    { 0xc57dcf18, SUC_lexicon_tags_28 },
    { 0x9f7dcf18, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddcf1d, SUC_lexicon_tags_13 },
    { 0x659dcf1e, SUC_lexicon_tags_23 },
    { 0xc35dcf1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x621dcf22, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb71dcf25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39dcf2e, SUC_lexicon_tags_30 },
    { 0xdb5dcf2f, SUC_lexicon_tags_15 },
    { 0xea5dcf30, SUC_lexicon_tags_39 },
    { 0x4b5dcf2f, SUC_lexicon_tags_17 },
    { 0xf37dcf30, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x85ddcf34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffdcf39, SUC_lexicon_tags_17 },
    { 0x565dcf3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x96bdcf3d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697dcf42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddcf45, SUC_lexicon_tags_33 },
    { 0x909dcf46, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bdcf4a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4e9dcf4c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55dcf54, SUC_lexicon_tags_30 },
    { 0x66ddcf55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ddcf59, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x383dcf5c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05dcf62, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ddcf67, SUC_lexicon_tags_11 },
    { 0x1f1dcf67, SUC_lexicon_tags_15 },
    { 0x349dcf69, SUC_lexicon_tags_12 },
    { 0x313dcf69, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebdcf71, SUC_lexicon_tags_15 },
    { 0x3ddcf72, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xcbddcf74, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x417dcf76, SUC_lexicon_tags_60 },
    { 0x5a5dcf77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x909dcf79, SUC_lexicon_tags_37 },
    { 0xac7dcf79, SUC_lexicon_tags_21 },
    { 0x4a1dcf7b, SUC_lexicon_tags_25 },
    { 0x7d7dcf7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5dcf83, SUC_lexicon_tags_17 },
    { 0x23fdcf84, SUC_lexicon_tags_15 },
    { 0xe99dcf85, SUC_lexicon_tags_38 },
    { 0x4a5dcf86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75dcf89, SUC_lexicon_tags_2 },
    { 0xa97dcf8a, SUC_lexicon_tags_15 },
    { 0xe0fdcf8a, SUC_lexicon_tags_7 },
    { 0x5e5dcf8c, SUC_lexicon_tags_23 },
    { 0x2bbdcf8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90fdcf8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1e9dcf91, SUC_lexicon_tags_47 },
    { 0x161dcf91, SUC_lexicon_tags_49 },
    { 0xe3fdcf93, SUC_lexicon_tags_17 },
    { 0xf7dcf92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175dcf98, SUC_lexicon_tags_7 },
    { 0x483dcf99, SUC_lexicon_tags_17 },
    { 0xa37dcf99, SUC_lexicon_tags_31 },
    { 0x57fdcf9b, SUC_lexicon_tags_33 },
    { 0x103dcf9b, SUC_lexicon_tags_11 },
    { 0x4efdcf9c, SUC_lexicon_tags_45 },
    { 0x241dcf9e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x68fdcfa0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51dcfa4, SUC_lexicon_tags_17 },
    { 0x2cbdcfa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695dcfab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377dcfb2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3dcfb9, SUC_lexicon_tags_8 },
    { 0x4b3dcfba, SUC_lexicon_tags_1165 },
    { 0x87ddcfbb, SUC_lexicon_tags_17 },
    { 0xf5dcfbc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fdcfc1, SUC_lexicon_tags_2 },
    { 0x4addcfc2, SUC_lexicon_tags_23 },
    { 0xac7dcfc3, SUC_lexicon_tags_26 },
    { 0x4e5dcfc4, SUC_lexicon_tags_27 },
    { 0x853dcfc5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb2ddcfc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7dcfcc, SUC_lexicon_tags_7 },
    { 0x2fdcfcd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x81fdcfcf, SUC_lexicon_tags_9 },
    { 0x71ddcfd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x925dcfd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xab5dcfd4, SUC_lexicon_tags_12 },
    { 0xc03dcfd4, SUC_lexicon_tags_0 },
    { 0xaa1dcfd5, SUC_lexicon_tags_171 },
    { 0x217dcfd7, SUC_lexicon_tags_15 },
    { 0xf17dcfd7, SUC_lexicon_tags_23 },
    { 0x8eddcfd9, SUC_lexicon_tags_14 },
    { 0xa79dcfda, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbdcfdd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dcfe0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc6ddcfe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fdcfe5, SUC_lexicon_tags_12 },
    { 0xfbdcfe6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25dcfea, SUC_lexicon_tags_11 },
    { 0x103dcfeb, SUC_lexicon_tags_39 },
    { 0xf2bdcfec, SUC_lexicon_tags_28 },
    { 0xcb9dcfed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe5dcfef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dcff5, SUC_lexicon_tags_2 },
    { 0xf05dcff6, SUC_lexicon_tags_2 },
    { 0x6c5dcff7, SUC_lexicon_tags_17 },
    { 0x627dcff8, SUC_lexicon_tags_29 },
    { 0x251dcff9, SUC_lexicon_tags_37 },
    { 0xa69dcff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69dcffd, SUC_lexicon_tags_37 },
    { 0x2fbdcffe, SUC_lexicon_tags_68 },
    { 0x2addcffe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5dd003, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ddd009, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba7dd00b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x931dd00d, SUC_lexicon_tags_26 },
    { 0x897dd00e, SUC_lexicon_tags_37 },
    { 0xcd9dd00e, SUC_lexicon_tags_29 },
    { 0xb0ddd010, SUC_lexicon_tags_32 },
    { 0x469dd010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebdd015, SUC_lexicon_tags_19 },
    { 0x201dd016, SUC_lexicon_tags_8 },
    { 0xbbdd017, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6b1dd019, SUC_lexicon_tags_60 },
    { 0x88ddd019, SUC_lexicon_tags_15 },
    { 0x189dd01b, SUC_lexicon_tags_25 },
    { 0x5a3dd01c, SUC_lexicon_tags_15 },
    { 0x783dd01d, SUC_lexicon_tags_31 },
    { 0x499dd01e, SUC_lexicon_tags_2 },
    { 0xfcddd01b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x827dd021, SUC_lexicon_tags_45 },
    { 0x717dd022, SUC_lexicon_tags_0 },
    { 0xefbdd023, SUC_lexicon_tags_10 },
    { 0xdb3dd024, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961dd02b, SUC_lexicon_tags_19 },
    { 0x727dd02c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd7dd02e, SUC_lexicon_tags_37 },
    { 0x965dd02f, SUC_lexicon_tags_43 },
    { 0xa0ddd02e, SUC_lexicon_tags_47 },
    { 0xafbdd031, SUC_lexicon_tags_88 },
    { 0xff3dd031, SUC_lexicon_tags_26 },
    { 0x841dd031, SUC_lexicon_tags_0 },
    { 0x579dd031, SUC_lexicon_tags_7 },
    { 0xa63dd035, SUC_lexicon_tags_19 },
    { 0x85bdd036, SUC_lexicon_tags_29 },
    { 0xcc3dd035, SUC_lexicon_tags_65 },
    { 0xd3ddd038, SUC_lexicon_tags_8 },
    { 0xd13dd038, SUC_lexicon_tags_17 },
    { 0xeeddd03a, SUC_lexicon_tags_26 },
    { 0x497dd03a, SUC_lexicon_tags_25 },
    { 0x4c1dd03a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1dd040, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x37ddd042, SUC_lexicon_tags_12 },
    { 0x5cddd043, SUC_lexicon_tags_8 },
    { 0x94ddd044, SUC_lexicon_tags_57 },
    { 0x7a3dd043, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7dd048, SUC_lexicon_tags_39 },
    { 0x5a5dd048, SUC_lexicon_tags_25 },
    { 0x197dd048, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fdd051, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe9bdd053, SUC_lexicon_tags_17 },
    { 0xbdbdd053, SUC_lexicon_tags_2 },
    { 0x729dd055, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bdd05a, SUC_lexicon_tags_40 },
    { 0x717dd05b, SUC_lexicon_tags_49 },
    { 0x8b7dd05a, SUC_lexicon_tags_2 },
    { 0x245dd05c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc87dd05f, SUC_lexicon_tags_7 },
    { 0x9bbdd05f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91dd063, SUC_lexicon_tags_7 },
    { 0x117dd064, SUC_lexicon_tags_27 },
    { 0x9f3dd065, SUC_lexicon_tags_25 },
    { 0x2b7dd063, SUC_lexicon_tags_2 },
    { 0xbcddd067, SUC_lexicon_tags_21 },
    { 0x16fdd066, SUC_lexicon_tags_12 },
    { 0xe07dd069, SUC_lexicon_tags_14 },
    { 0x237dd069, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9dd071, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ddd078, SUC_lexicon_tags_25 },
    { 0xe7fdd079, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x359dd07b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x427dd07d, SUC_lexicon_tags_68 },
    { 0xe8fdd07d, SUC_lexicon_tags_2 },
    { 0xbf5dd07e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903dd084, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ebdd086, SUC_lexicon_tags_103 },
    { 0x7d5dd087, SUC_lexicon_tags_1 },
    { 0xa3fdd088, SUC_lexicon_tags_15 },
    { 0x9a7dd089, SUC_lexicon_tags_47 },
    { 0xa27dd089, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfa7dd08c, SUC_lexicon_tags_11 },
    { 0xe2ddd08d, SUC_lexicon_tags_7 },
    { 0x16bdd08e, SUC_lexicon_tags_11 },
    { 0xc25dd08e, SUC_lexicon_tags_38 },
    { 0x47bdd08d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9dd093, SUC_lexicon_tags_28 },
    { 0x243dd094, SUC_lexicon_tags_65 },
    { 0x1afdd095, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bdd09b, SUC_lexicon_tags_25 },
    { 0xc87dd09c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2ddd09e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ddd0a2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb11dd0a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487dd0ae, SUC_lexicon_tags_17 },
    { 0x859dd0af, SUC_lexicon_tags_55 },
    { 0x807dd0b0, SUC_lexicon_tags_1 },
    { 0x769dd0ae, SUC_lexicon_tags_12 },
    { 0x1bbdd0b2, SUC_lexicon_tags_15 },
    { 0xce1dd0b2, SUC_lexicon_tags_27 },
    { 0x49dd0b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2f7dd0b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfdd0b9, SUC_lexicon_tags_15 },
    { 0xb35dd0ba, SUC_lexicon_tags_17 },
    { 0x71ddd0bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x727dd0bd, SUC_lexicon_tags_204 },
    { 0x79fdd0be, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x917dd0c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ddd0c6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfdd0c9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3dd0cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09dd0d6, SUC_lexicon_tags_38 },
    { 0x379dd0d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x957dd0d9, SUC_lexicon_tags_12 },
    { 0x7c7dd0da, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbdd0de, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x585dd0e1, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xfa3dd0e3, SUC_lexicon_tags_12 },
    { 0xd83dd0e3, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7dd0ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbddd0f6, SUC_lexicon_tags_37 },
    { 0xd73dd0f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617dd0fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd102, SUC_lexicon_tags_60 },
    { 0x7a7dd102, SUC_lexicon_tags_6 },
    { 0x2eddd104, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3fdd106, SUC_lexicon_tags_15 },
    { 0x3dd107, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473dd10d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffdd111, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x693dd113, SUC_lexicon_tags_11 },
    { 0xf7ddd114, SUC_lexicon_tags_43 },
    { 0x3f9dd114, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x673dd118, SUC_lexicon_tags_29 },
    { 0xe6fdd118, SUC_lexicon_tags_32 },
    { 0xd89dd118, SUC_lexicon_tags_15 },
    { 0x351dd11b, SUC_lexicon_tags_25 },
    { 0x9a5dd11c, SUC_lexicon_tags_2 },
    { 0x8a7dd11c, SUC_lexicon_tags_1 },
    { 0xd2fdd11c, SUC_lexicon_tags_17 },
    { 0xdefdd11f, SUC_lexicon_tags_33 },
    { 0xf47dd120, SUC_lexicon_tags_13 },
    { 0xb4bdd121, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5dd124, SUC_lexicon_tags_7 },
    { 0x8c1dd125, SUC_lexicon_tags_2 },
    { 0x821dd125, SUC_lexicon_tags_1 },
    { 0x27bdd126, SUC_lexicon_tags_27 },
    { 0x3c1dd128, SUC_lexicon_tags_14 },
    { 0x761dd129, SUC_lexicon_tags_1 },
    { 0xf29dd129, SUC_lexicon_tags_25 },
    { 0xf7ddd12b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dd130, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3dd137, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887dd13c, SUC_lexicon_tags_25 },
    { 0xc6ddd13d, SUC_lexicon_tags_8 },
    { 0x31ddd13d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed1dd140, SUC_lexicon_tags_8 },
    { 0x2a3dd140, SUC_lexicon_tags_32 },
    { 0xd1fdd142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde1dd144, SUC_lexicon_tags_25 },
    { 0xf6bdd145, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47dd14b, SUC_lexicon_tags_55 },
    { 0x1bdd14c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa25dd14e, SUC_lexicon_tags_12 },
    { 0x837dd14e, SUC_lexicon_tags_37 },
    { 0x7addd150, SUC_lexicon_tags_0 },
    { 0xcb1dd151, SUC_lexicon_tags_75 },
    { 0xd79dd152, SUC_lexicon_tags_1 },
    { 0x745dd152, SUC_lexicon_tags_11 },
    { 0xbcbdd154, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x75dd157, SUC_lexicon_tags_17 },
    { 0xc05dd158, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x53fdd15a, SUC_lexicon_tags_25 },
    { 0xf3fdd15a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afdd160, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdd163, SUC_lexicon_tags_23 },
    { 0x67ddd164, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6f7dd166, SUC_lexicon_tags_16 },
    { 0x4f3dd167, SUC_lexicon_tags_17 },
    { 0x17bdd168, SUC_lexicon_tags_13 },
    { 0xf75dd168, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfdd16d, SUC_lexicon_tags_37 },
    { 0x441dd16e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbbdd170, SUC_lexicon_tags_8 },
    { 0x623dd171, SUC_lexicon_tags_11 },
    { 0xa13dd172, SUC_lexicon_tags_1 },
    { 0xeaddd171, SUC_lexicon_tags_7 },
    { 0xe3dd174, SUC_lexicon_tags_17 },
    { 0xeeddd174, SUC_lexicon_tags_17 },
    { 0x905dd176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeabdd178, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe37dd17a, SUC_lexicon_tags_63 },
    { 0x217dd17a, SUC_lexicon_tags_28 },
    { 0xdb9dd17c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc3dd17e, SUC_lexicon_tags_11 },
    { 0xe1ddd17e, SUC_lexicon_tags_11 },
    { 0xc55dd180, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ddd183, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xadddd186, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8fbdd188, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67dd18c, SUC_lexicon_tags_1 },
    { 0x60bdd18d, SUC_lexicon_tags_12 },
    { 0x197dd18e, SUC_lexicon_tags_29 },
    { 0x51ddd18f, SUC_lexicon_tags_82 },
    { 0xe59dd190, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x85ddd192, SUC_lexicon_tags_17 },
    { 0x1fbdd193, SUC_lexicon_tags_15 },
    { 0x9a9dd192, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb9dd196, SUC_lexicon_tags_25 },
    { 0x685dd197, SUC_lexicon_tags_15 },
    { 0x5fbdd198, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1dd19d, SUC_lexicon_tags_19 },
    { 0x479dd19e, SUC_lexicon_tags_7 },
    { 0x989dd19f, SUC_lexicon_tags_30 },
    { 0x3fbdd19d, SUC_lexicon_tags_63 },
    { 0xf4bdd19f, SUC_lexicon_tags_63 },
    { 0x861dd1a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43fdd1a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdfbdd1a6, SUC_lexicon_tags_30 },
    { 0x67fdd1a6, SUC_lexicon_tags_2 },
    { 0x913dd1a8, SUC_lexicon_tags_27 },
    { 0x5f3dd1a8, SUC_lexicon_tags_28 },
    { 0x24fdd1a9, SUC_lexicon_tags_15 },
    { 0xd39dd1ab, SUC_lexicon_tags_28 },
    { 0xdc7dd1aa, SUC_lexicon_tags_12 },
    { 0x7ffdd1a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93dd1b5, SUC_lexicon_tags_48 },
    { 0xb05dd1b5, SUC_lexicon_tags_11 },
    { 0x9afdd1b6, SUC_lexicon_tags_204 },
    { 0xa11dd1b5, SUC_lexicon_tags_2 },
    { 0x8bbdd1b9, SUC_lexicon_tags_19 },
    { 0x207dd1b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2b7dd1bc, SUC_lexicon_tags_12 },
    { 0x5ffdd1bc, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25dd1c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7dd1ca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x791dd1cc, SUC_lexicon_tags_8 },
    { 0xb6bdd1cd, SUC_lexicon_tags_25 },
    { 0xe97dd1ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc77dd1d0, SUC_lexicon_tags_87 },
    { 0xaa7dd1d1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907dd1d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48bdd1d7, SUC_lexicon_tags_15 },
    { 0x5d1dd1d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531dd1dc, SUC_lexicon_tags_1 },
    { 0x97dd1dd, SUC_lexicon_tags_36 },
    { 0x9cddd1dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ebdd1e0, SUC_lexicon_tags_7 },
    { 0xc8bdd1e1, SUC_lexicon_tags_14 },
    { 0xd9ddd1e2, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1dd1ed, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x583dd1ef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3dd1f8, SUC_lexicon_tags_12 },
    { 0x9ffdd1f9, SUC_lexicon_tags_33 },
    { 0x129dd1fa, SUC_lexicon_tags_17 },
    { 0xc73dd1fa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x94bdd1fd, SUC_lexicon_tags_49 },
    { 0x361dd1fd, SUC_lexicon_tags_67 },
    { 0x971dd1ff, SUC_lexicon_tags_29 },
    { 0xc5dd1fd, SUC_lexicon_tags_23 },
    { 0x35ddd201, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55dd204, SUC_lexicon_tags_6 },
    { 0x3d7dd205, SUC_lexicon_tags_49 },
    { 0x3b9dd206, SUC_lexicon_tags_17 },
    { 0xd5dd205, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x791dd20a, SUC_lexicon_tags_23 },
    { 0x9fdd20b, SUC_lexicon_tags_11 },
    { 0xd73dd20c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3ebdd20e, SUC_lexicon_tags_1 },
    { 0xa25dd20f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48fdd211, SUC_lexicon_tags_37 },
    { 0xd49dd212, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x47dd215, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1dd221, SUC_lexicon_tags_17 },
    { 0x48fdd222, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371dd226, SUC_lexicon_tags_9 },
    { 0x715dd227, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x485dd229, SUC_lexicon_tags_21 },
    { 0x583dd229, SUC_lexicon_tags_60 },
    { 0x807dd22b, SUC_lexicon_tags_12 },
    { 0x697dd22c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ddd230, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fdd234, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x48bdd236, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eddd23d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef3dd23f, SUC_lexicon_tags_29 },
    { 0x629dd23f, SUC_lexicon_tags_49 },
    { 0xbd1dd241, SUC_lexicon_tags_32 },
    { 0xa95dd242, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7dd245, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeaddd247, SUC_lexicon_tags_37 },
    { 0x3e7dd248, SUC_lexicon_tags_8 },
    { 0x569dd249, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9dd250, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x40bdd252, SUC_lexicon_tags_17 },
    { 0xc6ddd252, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219dd259, SUC_lexicon_tags_37 },
    { 0xcddd25a, SUC_lexicon_tags_37 },
    { 0xe59dd25b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585dd262, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dddd267, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379dd26f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5dd274, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x37ddd276, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dd279, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7dd27c, SUC_lexicon_tags_11 },
    { 0xef5dd27d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5dd284, SUC_lexicon_tags_30 },
    { 0x829dd285, SUC_lexicon_tags_3 },
    { 0x801dd286, SUC_lexicon_tags_17 },
    { 0x8d9dd287, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf83dd289, SUC_lexicon_tags_14 },
    { 0x637dd289, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649dd290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e7dd292, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8abdd294, SUC_lexicon_tags_16 },
    { 0x525dd295, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd9fdd297, SUC_lexicon_tags_23 },
    { 0x8addd298, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x605dd29a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x17dd29c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4addd2a2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x287dd2a4, SUC_lexicon_tags_17 },
    { 0x619dd2a5, SUC_lexicon_tags_11 },
    { 0x4efdd2a6, SUC_lexicon_tags_17 },
    { 0xa6ddd2a7, SUC_lexicon_tags_3 },
    { 0xd59dd2a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41dd2af, SUC_lexicon_tags_45 },
    { 0x245dd2b0, SUC_lexicon_tags_55 },
    { 0x773dd2b1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x815dd2b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x189dd2b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x84ddd2b7, SUC_lexicon_tags_25 },
    { 0x28ddd2b8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31dd2bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fdd2c0, SUC_lexicon_tags_47 },
    { 0x49ddd2c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fbdd2c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831dd2ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bdd2ce, SUC_lexicon_tags_49 },
    { 0xb49dd2cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8fdd2d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97dd2d6, SUC_lexicon_tags_25 },
    { 0x50fdd2d7, SUC_lexicon_tags_2 },
    { 0x569dd2d7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x75bdd2da, SUC_lexicon_tags_0 },
    { 0x3dfdd2db, SUC_lexicon_tags_25 },
    { 0xe0bdd2da, SUC_lexicon_tags_60 },
    { 0x77dd2dd, SUC_lexicon_tags_55 },
    { 0xa83dd2da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7dd2e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03dd2e7, SUC_lexicon_tags_62 },
    { 0xf6ddd2e7, SUC_lexicon_tags_45 },
    { 0xef1dd2e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ddd2ed, SUC_lexicon_tags_38 },
    { 0xc15dd2ee, SUC_lexicon_tags_12 },
    { 0x537dd2ee, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbdd2f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfdd2f6, SUC_lexicon_tags_25 },
    { 0x5d5dd2f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe7dd2f9, SUC_lexicon_tags_2 },
    { 0x543dd2fa, SUC_lexicon_tags_31 },
    { 0x6c3dd2fb, SUC_lexicon_tags_69 },
    { 0x96ddd2fc, SUC_lexicon_tags_56 },
    { 0x40fdd2fa, SUC_lexicon_tags_11 },
    { 0xbeddd2fb, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fdd301, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ddd309, SUC_lexicon_tags_15 },
    { 0x53fdd30a, SUC_lexicon_tags_87 },
    { 0x7ddd30b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x929dd30d, SUC_lexicon_tags_37 },
    { 0x47bdd30e, SUC_lexicon_tags_2 },
    { 0xb9ddd30d, SUC_lexicon_tags_8 },
    { 0xc23dd310, SUC_lexicon_tags_38 },
    { 0x213dd311, SUC_lexicon_tags_25 },
    { 0x619dd310, SUC_lexicon_tags_4 },
    { 0x5dfdd313, SUC_lexicon_tags_25 },
    { 0xcafdd314, SUC_lexicon_tags_30 },
    { 0xd25dd312, SUC_lexicon_tags_77 },
    { 0x4fdd316, SUC_lexicon_tags_37 },
    { 0x18bdd316, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9dd31a, SUC_lexicon_tags_7 },
    { 0xbc7dd31b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x77dd31e, SUC_lexicon_tags_201 },
    { 0x383dd31f, SUC_lexicon_tags_17 },
    { 0x769dd31f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669dd32e, SUC_lexicon_tags_6 },
    { 0x44ddd32f, SUC_lexicon_tags_17 },
    { 0x8eddd330, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ddd334, SUC_lexicon_tags_25 },
    { 0xed1dd335, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213dd33a, SUC_lexicon_tags_63 },
    { 0x787dd33b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbdd342, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bddd34e, SUC_lexicon_tags_82 },
    { 0x941dd34f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x925dd352, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a9dd354, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x54bdd357, SUC_lexicon_tags_30 },
    { 0xa69dd358, SUC_lexicon_tags_46 },
    { 0x84bdd357, SUC_lexicon_tags_12 },
    { 0xe6fdd35a, SUC_lexicon_tags_14 },
    { 0x9dfdd35a, SUC_lexicon_tags_8 },
    { 0x77ddd35c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25dd35f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7dd364, SUC_lexicon_tags_6 },
    { 0x5d3dd365, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffdd369, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7dd372, SUC_lexicon_tags_17 },
    { 0xc41dd373, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45dd37a, SUC_lexicon_tags_15 },
    { 0xc09dd37a, SUC_lexicon_tags_14 },
    { 0x581dd37c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x141dd37f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x891dd381, SUC_lexicon_tags_39 },
    { 0xb3fdd382, SUC_lexicon_tags_21 },
    { 0x1afdd383, SUC_lexicon_tags_79 },
    { 0xe43dd383, SUC_lexicon_tags_296 },
    { 0xd09dd382, SUC_lexicon_tags_28 },
    { 0x3f5dd385, SUC_lexicon_tags_2 },
    { 0x45bdd383, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3abdd389, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6eddd38c, SUC_lexicon_tags_19 },
    { 0x30fdd38d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34bdd391, SUC_lexicon_tags_55 },
    { 0x2a1dd392, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3dd399, SUC_lexicon_tags_25 },
    { 0xd45dd39a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7dd3a6, SUC_lexicon_tags_14 },
    { 0x977dd3a7, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bdd3ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1dd3b3, SUC_lexicon_tags_1 },
    { 0xe2fdd3b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbfdd3b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691dd3ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77dd3bd, SUC_lexicon_tags_103 },
    { 0xeb9dd3bd, SUC_lexicon_tags_11 },
    { 0xf1dd3be, SUC_lexicon_tags_45 },
    { 0x58fdd3bd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa87dd3c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31dd3cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dd3cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dd3d2, SUC_lexicon_tags_2 },
    { 0xeb7dd3d3, SUC_lexicon_tags_37 },
    { 0x135dd3d4, SUC_lexicon_tags_60 },
    { 0xd0bdd3d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x257dd3d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7dd3df, SUC_lexicon_tags_16 },
    { 0xe19dd3df, SUC_lexicon_tags_27 },
    { 0xf31dd3e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fdd3e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bdd3eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x36ddd3ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4a7dd3ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd49dd3f1, SUC_lexicon_tags_25 },
    { 0xb07dd3f2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd63dd3f4, SUC_lexicon_tags_8 },
    { 0x581dd3f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x423dd3f7, SUC_lexicon_tags_7 },
    { 0xfdbdd3f8, SUC_lexicon_tags_25 },
    { 0x5b3dd3f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61dd3fd, SUC_lexicon_tags_34 },
    { 0xd2ddd3fe, SUC_lexicon_tags_14 },
    { 0xc69dd3fe, SUC_lexicon_tags_28 },
    { 0xb4fdd3ff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79dd404, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bdd40c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1dd40f, SUC_lexicon_tags_0 },
    { 0x1e7dd40f, SUC_lexicon_tags_49 },
    { 0x407dd411, SUC_lexicon_tags_35 },
    { 0xf81dd412, SUC_lexicon_tags_14 },
    { 0xc2fdd413, SUC_lexicon_tags_35 },
    { 0xb27dd414, SUC_lexicon_tags_2 },
    { 0x995dd410, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x863dd418, SUC_lexicon_tags_250 },
    { 0xd3bdd418, SUC_lexicon_tags_7 },
    { 0x7f7dd41a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf5fdd41c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87dd423, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3dd426, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153dd42d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3dd430, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fdd435, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x76fdd437, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329dd43d, SUC_lexicon_tags_1 },
    { 0xfc3dd43d, SUC_lexicon_tags_11 },
    { 0x1ddd43f, SUC_lexicon_tags_40 },
    { 0xae9dd43e, SUC_lexicon_tags_25 },
    { 0xd47dd440, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfdd448, SUC_lexicon_tags_43 },
    { 0x959dd448, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x791dd44b, SUC_lexicon_tags_1 },
    { 0x3b9dd44c, SUC_lexicon_tags_54 },
    { 0x6b5dd44d, SUC_lexicon_tags_7 },
    { 0x9f7dd44e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebdd451, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5dd456, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29ddd458, SUC_lexicon_tags_29 },
    { 0xc55dd458, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x45dd45c, SUC_lexicon_tags_7 },
    { 0x58fdd45d, SUC_lexicon_tags_0 },
    { 0x6a1dd45e, SUC_lexicon_tags_17 },
    { 0x489dd45f, SUC_lexicon_tags_0 },
    { 0x81ddd460, SUC_lexicon_tags_17 },
    { 0x19bdd460, SUC_lexicon_tags_2 },
    { 0xe65dd45f, SUC_lexicon_tags_25 },
    { 0xb19dd463, SUC_lexicon_tags_47 },
    { 0x2c1dd463, SUC_lexicon_tags_43 },
    { 0x5d9dd465, SUC_lexicon_tags_60 },
    { 0x48fdd464, SUC_lexicon_tags_63 },
    { 0x727dd462, SUC_lexicon_tags_0 },
    { 0xae9dd468, SUC_lexicon_tags_14 },
    { 0x763dd463, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9dd46d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ddd470, SUC_lexicon_tags_12 },
    { 0xd69dd471, SUC_lexicon_tags_30 },
    { 0x4addd471, SUC_lexicon_tags_25 },
    { 0x605dd473, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x7dfdd475, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x609dd477, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x563dd479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9fdd47b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x423dd47d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x717dd47f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3dd482, SUC_lexicon_tags_28 },
    { 0xf95dd483, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x919dd486, SUC_lexicon_tags_25 },
    { 0x18bdd487, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95dd48c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bdd496, SUC_lexicon_tags_11 },
    { 0x425dd497, SUC_lexicon_tags_37 },
    { 0x3cfdd498, SUC_lexicon_tags_49 },
    { 0x31bdd498, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ddd49c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x309dd49e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4dddd4a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x477dd4a2, SUC_lexicon_tags_13 },
    { 0x605dd4a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1dd4a8, SUC_lexicon_tags_25 },
    { 0x763dd4a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4abdd4ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437dd4b2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709dd4b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bdd4b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe59dd4bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73dd4c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9dd4c3, SUC_lexicon_tags_40 },
    { 0xa27dd4c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc4bdd4c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14fdd4c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe85dd4ca, SUC_lexicon_tags_1 },
    { 0x58ddd4cb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ddd4ce, SUC_lexicon_tags_17 },
    { 0xae9dd4cf, SUC_lexicon_tags_14 },
    { 0x5b5dd4cf, SUC_lexicon_tags_48 },
    { 0x9ebdd4d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc95dd4d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29dd4d9, SUC_lexicon_tags_1 },
    { 0xa87dd4d9, SUC_lexicon_tags_17 },
    { 0xafbdd4db, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959dd4df, SUC_lexicon_tags_28 },
    { 0x447dd4e0, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0x253dd4e3, SUC_lexicon_tags_1 },
    { 0x233dd4e4, SUC_lexicon_tags_37 },
    { 0x81fdd4e4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3dd4ea, SUC_lexicon_tags_37 },
    { 0x88ddd4eb, SUC_lexicon_tags_25 },
    { 0xfffdd4ec, SUC_lexicon_tags_14 },
    { 0x4e7dd4ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x431dd4ef, SUC_lexicon_tags_28 },
    { 0xd0fdd4f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x48fdd4f2, SUC_lexicon_tags_27 },
    { 0x6c1dd4f3, SUC_lexicon_tags_10 },
    { 0x69dd4f2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x811dd4f6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5dd4fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3dd500, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x439dd502, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcbdd504, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1addd507, SUC_lexicon_tags_17 },
    { 0x28bdd508, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fdd50d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1dd510, SUC_lexicon_tags_37 },
    { 0x4e9dd511, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfcbdd513, SUC_lexicon_tags_8 },
    { 0x7d9dd514, SUC_lexicon_tags_17 },
    { 0xa73dd515, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ddd51c, SUC_lexicon_tags_37 },
    { 0xc6fdd51d, SUC_lexicon_tags_11 },
    { 0x48bdd51c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37dd523, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5dd52b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bdd535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291dd539, SUC_lexicon_tags_63 },
    { 0x4dbdd53a, SUC_lexicon_tags_7 },
    { 0xb0fdd53b, SUC_lexicon_tags_47 },
    { 0xedfdd53c, SUC_lexicon_tags_34 },
    { 0xddddd539, SUC_lexicon_tags_29 },
    { 0x5b5dd53a, SUC_lexicon_tags_25 },
    { 0xdfdd53f, SUC_lexicon_tags_2 },
    { 0x35dd540, SUC_lexicon_tags_1 },
    { 0x9c7dd541, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa13dd543, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3dd548, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ddd54c, SUC_lexicon_tags_30 },
    { 0xd6fdd54d, SUC_lexicon_tags_32 },
    { 0x9b5dd54e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdebdd550, SUC_lexicon_tags_37 },
    { 0xa99dd551, SUC_lexicon_tags_37 },
    { 0x857dd552, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9dd555, SUC_lexicon_tags_46 },
    { 0x943dd556, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa6bdd558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3dd55b, SUC_lexicon_tags_12 },
    { 0x95bdd55c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbdd562, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ffdd564, SUC_lexicon_tags_21 },
    { 0x80bdd564, SUC_lexicon_tags_7 },
    { 0x675dd564, SUC_lexicon_tags_45 },
    { 0xffdd567, SUC_lexicon_tags_11 },
    { 0x15dd568, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbdd56d, SUC_lexicon_tags_9 },
    { 0xe6bdd56d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867dd576, SUC_lexicon_tags_17 },
    { 0xdbbdd577, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bdd57b, SUC_lexicon_tags_43 },
    { 0xa91dd57c, SUC_lexicon_tags_1 },
    { 0xdb3dd57d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fdd582, SUC_lexicon_tags_7 },
    { 0x9cbdd583, SUC_lexicon_tags_12 },
    { 0x7b7dd584, SUC_lexicon_tags_37 },
    { 0x52bdd585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a5dd587, SUC_lexicon_tags_45 },
    { 0x969dd588, SUC_lexicon_tags_15 },
    { 0x117dd589, SUC_lexicon_tags_17 },
    { 0x4d7dd589, SUC_lexicon_tags_12 },
    { 0x29fdd58b, SUC_lexicon_tags_28 },
    { 0x5a9dd58a, SUC_lexicon_tags_35 },
    { 0xb89dd58b, SUC_lexicon_tags_2 },
    { 0xa99dd587, SUC_lexicon_tags_11 },
    { 0x893dd58f, SUC_lexicon_tags_12 },
    { 0x11dd590, SUC_lexicon_tags_29 },
    { 0xb31dd591, SUC_lexicon_tags_10 },
    { 0x469dd592, SUC_lexicon_tags_282 },
    { 0x133dd591, SUC_lexicon_tags_7 },
    { 0x241dd593, SUC_lexicon_tags_13 },
    { 0x58fdd595, SUC_lexicon_tags_85 },
    { 0xfb5dd590, SUC_lexicon_tags_14 },
    { 0x443dd58e, SUC_lexicon_tags_8 },
    { 0x19bdd598, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x677dd59a, SUC_lexicon_tags_17 },
    { 0x4f7dd59b, SUC_lexicon_tags_17 },
    { 0x5e9dd59c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x249dd59f, SUC_lexicon_tags_15 },
    { 0xc77dd59f, SUC_lexicon_tags_2 },
    { 0x6ddd59f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9dd5a4, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xc03dd5a6, SUC_lexicon_tags_37 },
    { 0xac3dd5a6, SUC_lexicon_tags_15 },
    { 0xcd3dd5a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bdd5ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13dd5b2, SUC_lexicon_tags_28 },
    { 0x5ebdd5b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaddd5ba, SUC_lexicon_tags_25 },
    { 0x2c5dd5bb, SUC_lexicon_tags_49 },
    { 0xffdd5bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc09dd5be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3a7dd5c0, SUC_lexicon_tags_65 },
    { 0xfcfdd5c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfdd5c6, SUC_lexicon_tags_218 },
    { 0, NULL },
    { 0xe39dd5c8, SUC_lexicon_tags_9 },
    { 0xe09dd5c9, SUC_lexicon_tags_15 },
    { 0x46ddd5c9, SUC_lexicon_tags_31 },
    { 0x76fdd5cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x247dd5cd, SUC_lexicon_tags_1472 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5dd5d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163dd5d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x285dd5d6, SUC_lexicon_tags_107 },
    { 0xc6ddd5d6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fdd5de, SUC_lexicon_tags_0 },
    { 0x41bdd5df, SUC_lexicon_tags_11 },
    { 0x8ffdd5e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bdd5e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x60ddd5e6, SUC_lexicon_tags_17 },
    { 0x80ddd5e7, SUC_lexicon_tags_1 },
    { 0x973dd5e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x407dd5eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbdd5f0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeddd5f4, SUC_lexicon_tags_64 },
    { 0xf63dd5f5, SUC_lexicon_tags_47 },
    { 0xb17dd5f5, SUC_lexicon_tags_11 },
    { 0x7d1dd5f7, SUC_lexicon_tags_2 },
    { 0x7fddd5f5, SUC_lexicon_tags_46 },
    { 0x637dd5f9, SUC_lexicon_tags_1 },
    { 0x90bdd5fa, SUC_lexicon_tags_37 },
    { 0xdc3dd5fb, SUC_lexicon_tags_47 },
    { 0x785dd5fc, SUC_lexicon_tags_28 },
    { 0xe1bdd5fb, SUC_lexicon_tags_25 },
    { 0xd69dd5fa, SUC_lexicon_tags_27 },
    { 0xda5dd5fe, SUC_lexicon_tags_80 },
    { 0x9e9dd5f9, SUC_lexicon_tags_37 },
    { 0xc41dd601, SUC_lexicon_tags_39 },
    { 0xbebdd602, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ddd606, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ddd609, SUC_lexicon_tags_37 },
    { 0x9d9dd60a, SUC_lexicon_tags_11 },
    { 0x58bdd60a, SUC_lexicon_tags_2 },
    { 0xa55dd60c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bdd60f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdd614, SUC_lexicon_tags_1 },
    { 0xc35dd615, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603dd619, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959dd620, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a7dd622, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x473dd625, SUC_lexicon_tags_15 },
    { 0xdc7dd625, SUC_lexicon_tags_85 },
    { 0x2f5dd626, SUC_lexicon_tags_3 },
    { 0x45bdd628, SUC_lexicon_tags_9 },
    { 0x5bddd625, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7dd62b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd31dd62d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471dd634, SUC_lexicon_tags_1 },
    { 0xd09dd635, SUC_lexicon_tags_25 },
    { 0xcefdd635, SUC_lexicon_tags_11 },
    { 0x7e5dd637, SUC_lexicon_tags_9 },
    { 0x951dd638, SUC_lexicon_tags_47 },
    { 0x9c3dd638, SUC_lexicon_tags_39 },
    { 0xddfdd63a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffdd63e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x87fdd641, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5f9dd643, SUC_lexicon_tags_25 },
    { 0xa13dd644, SUC_lexicon_tags_41 },
    { 0x1c1dd644, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dd64a, SUC_lexicon_tags_15 },
    { 0x631dd64a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5dd651, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xca5dd653, SUC_lexicon_tags_17 },
    { 0xcafdd654, SUC_lexicon_tags_25 },
    { 0x4addd655, SUC_lexicon_tags_47 },
    { 0xde5dd656, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1c9dd658, SUC_lexicon_tags_32 },
    { 0x907dd659, SUC_lexicon_tags_11 },
    { 0x75bdd659, SUC_lexicon_tags_14 },
    { 0x8fbdd658, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bdd663, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bdd66a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x67fdd66c, SUC_lexicon_tags_1 },
    { 0x543dd66d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5dd675, SUC_lexicon_tags_37 },
    { 0xfb9dd676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab7dd678, SUC_lexicon_tags_61 },
    { 0x95dd678, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51dd67d, SUC_lexicon_tags_4 },
    { 0xf01dd67e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x613dd681, SUC_lexicon_tags_1 },
    { 0xd9ddd682, SUC_lexicon_tags_61 },
    { 0x75ddd681, SUC_lexicon_tags_2 },
    { 0x1a7dd684, SUC_lexicon_tags_37 },
    { 0x361dd681, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x923dd687, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d9dd689, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673dd68d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3dd695, SUC_lexicon_tags_23 },
    { 0xd8bdd695, SUC_lexicon_tags_171 },
    { 0xf71dd697, SUC_lexicon_tags_1 },
    { 0x3fbdd698, SUC_lexicon_tags_7 },
    { 0x63bdd699, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25dd6a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf7bdd6a2, SUC_lexicon_tags_25 },
    { 0x3bfdd6a3, SUC_lexicon_tags_37 },
    { 0x57dd6a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4dddd6a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd81dd6a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91dd6ae, SUC_lexicon_tags_28 },
    { 0xcd5dd6af, SUC_lexicon_tags_2 },
    { 0xbe3dd6b0, SUC_lexicon_tags_25 },
    { 0x6ffdd6b1, SUC_lexicon_tags_7 },
    { 0xa49dd6b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xffbdd6b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59dd6b7, SUC_lexicon_tags_14 },
    { 0x9d5dd6b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffdd6c6, SUC_lexicon_tags_23 },
    { 0xff9dd6c6, SUC_lexicon_tags_66 },
    { 0x559dd6c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bdd6cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfa1dd6ce, SUC_lexicon_tags_7 },
    { 0x249dd6ce, SUC_lexicon_tags_11 },
    { 0x431dd6d0, SUC_lexicon_tags_6 },
    { 0xf9bdd6d1, SUC_lexicon_tags_11 },
    { 0x597dd6d0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf93dd6d4, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9dd6d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf7bdd6da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1dd6df, SUC_lexicon_tags_17 },
    { 0x6e7dd6df, SUC_lexicon_tags_25 },
    { 0xf8bdd6e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6b1dd6e3, SUC_lexicon_tags_10 },
    { 0xab9dd6e4, SUC_lexicon_tags_1 },
    { 0x1ebdd6e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1dd6ed, SUC_lexicon_tags_1 },
    { 0x743dd6ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x559dd6f0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddd6f9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x901dd6fc, SUC_lexicon_tags_15 },
    { 0x893dd6fd, SUC_lexicon_tags_17 },
    { 0x82fdd6fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ddd702, SUC_lexicon_tags_17 },
    { 0xc81dd702, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5c1dd705, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ebdd707, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85dd70b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ddd70e, SUC_lexicon_tags_65 },
    { 0xb7fdd70e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bdd713, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdd719, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2f5dd71b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ddd71f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bdd724, SUC_lexicon_tags_11 },
    { 0xda9dd725, SUC_lexicon_tags_7 },
    { 0xa05dd724, SUC_lexicon_tags_67 },
    { 0x23dd727, SUC_lexicon_tags_38 },
    { 0xad1dd727, SUC_lexicon_tags_11 },
    { 0x8d7dd729, SUC_lexicon_tags_9 },
    { 0x5e9dd729, SUC_lexicon_tags_17 },
    { 0xc9fdd72b, SUC_lexicon_tags_1 },
    { 0x889dd72c, SUC_lexicon_tags_15 },
    { 0x6e1dd72c, SUC_lexicon_tags_11 },
    { 0xcc1dd72c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15dd732, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c9dd734, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8e1dd736, SUC_lexicon_tags_1 },
    { 0xc5fdd737, SUC_lexicon_tags_49 },
    { 0x65dd736, SUC_lexicon_tags_49 },
    { 0xa8fdd739, SUC_lexicon_tags_16 },
    { 0x8a3dd73a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81dd73e, SUC_lexicon_tags_1 },
    { 0xe91dd73f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bdd743, SUC_lexicon_tags_12 },
    { 0x899dd744, SUC_lexicon_tags_9 },
    { 0xe29dd745, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdd749, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7dd74f, SUC_lexicon_tags_37 },
    { 0x277dd750, SUC_lexicon_tags_77 },
    { 0x879dd751, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853dd755, SUC_lexicon_tags_27 },
    { 0x15fdd755, SUC_lexicon_tags_44 },
    { 0xcb3dd755, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfdd75a, SUC_lexicon_tags_15 },
    { 0xcf9dd75b, SUC_lexicon_tags_17 },
    { 0xebfdd75b, SUC_lexicon_tags_37 },
    { 0xb8fdd75d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1dd760, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebdd765, SUC_lexicon_tags_17 },
    { 0xcb9dd766, SUC_lexicon_tags_6 },
    { 0x5e1dd765, SUC_lexicon_tags_64 },
    { 0x469dd768, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1dd76c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed3dd76e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5dd772, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xccfdd774, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ddd777, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bdd77c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x59fdd77e, SUC_lexicon_tags_1 },
    { 0xaf3dd77f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fdd785, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x717dd788, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaddd78c, SUC_lexicon_tags_17 },
    { 0x6d3dd78c, SUC_lexicon_tags_62 },
    { 0xdafdd78d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeddd791, SUC_lexicon_tags_82 },
    { 0x319dd792, SUC_lexicon_tags_2 },
    { 0xf21dd791, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65dd797, SUC_lexicon_tags_14 },
    { 0x639dd797, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1dd79c, SUC_lexicon_tags_11 },
    { 0x3ffdd79c, SUC_lexicon_tags_15 },
    { 0x56ddd79c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x445dd7a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83dd7a4, SUC_lexicon_tags_0 },
    { 0xde7dd7a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x567dd7a7, SUC_lexicon_tags_15 },
    { 0xf9ddd7a8, SUC_lexicon_tags_14 },
    { 0x25ddd7a9, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45dd7ac, SUC_lexicon_tags_0 },
    { 0xa83dd7ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x797dd7af, SUC_lexicon_tags_4 },
    { 0x475dd7af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fdd7b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x967dd7b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x801dd7b9, SUC_lexicon_tags_11 },
    { 0xa1fdd7ba, SUC_lexicon_tags_63 },
    { 0x7b3dd7bb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffdd7bf, SUC_lexicon_tags_17 },
    { 0x76ddd7c0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xafdd7c2, SUC_lexicon_tags_9 },
    { 0x9e9dd7c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29dd7c9, SUC_lexicon_tags_49 },
    { 0x415dd7c9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6bdd7cc, SUC_lexicon_tags_35 },
    { 0x50bdd7cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13dd7d0, SUC_lexicon_tags_23 },
    { 0x4a5dd7d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75dd7d5, SUC_lexicon_tags_12 },
    { 0x679dd7d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70fdd7d8, SUC_lexicon_tags_25 },
    { 0xed3dd7d9, SUC_lexicon_tags_49 },
    { 0x745dd7d9, SUC_lexicon_tags_4 },
    { 0xd1ddd7db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415dd7e4, SUC_lexicon_tags_84 },
    { 0x2fbdd7e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7dd7e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c7dd7eb, SUC_lexicon_tags_45 },
    { 0x445dd7eb, SUC_lexicon_tags_7 },
    { 0xa69dd7ec, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf3fdd7ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a3dd7f1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x13dd7f3, SUC_lexicon_tags_30 },
    { 0xe15dd7f3, SUC_lexicon_tags_34 },
    { 0xa65dd7f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865dd7f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49dd7fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4abdd7ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6dbdd801, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89dd805, SUC_lexicon_tags_25 },
    { 0x3b5dd806, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x621dd808, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6ffdd80a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdffdd80c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbfdd80e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dd812, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x329dd814, SUC_lexicon_tags_1 },
    { 0x84fdd814, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4bdd817, SUC_lexicon_tags_65 },
    { 0xd47dd818, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bdd81b, SUC_lexicon_tags_1 },
    { 0xed5dd81c, SUC_lexicon_tags_27 },
    { 0x127dd81c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fdd822, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd19dd825, SUC_lexicon_tags_25 },
    { 0xa5bdd826, SUC_lexicon_tags_36 },
    { 0xba9dd825, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ddd830, SUC_lexicon_tags_6 },
    { 0xf71dd831, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ddd836, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6a7dd838, SUC_lexicon_tags_22 },
    { 0x6a1dd838, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1dd840, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x30ddd842, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfdd845, SUC_lexicon_tags_26 },
    { 0x2dbdd846, SUC_lexicon_tags_226 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767dd84a, SUC_lexicon_tags_11 },
    { 0x6eddd84b, SUC_lexicon_tags_7 },
    { 0x5c3dd84b, SUC_lexicon_tags_74 },
    { 0xb39dd84d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdd853, SUC_lexicon_tags_21 },
    { 0xff7dd854, SUC_lexicon_tags_17 },
    { 0x441dd855, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bdd858, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabddd85c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54bdd85f, SUC_lexicon_tags_14 },
    { 0x5b3dd860, SUC_lexicon_tags_47 },
    { 0x47fdd860, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ddd865, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9a3dd867, SUC_lexicon_tags_37 },
    { 0xadfdd868, SUC_lexicon_tags_1 },
    { 0x24ddd867, SUC_lexicon_tags_11 },
    { 0x4a3dd86a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3dd86d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761dd873, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xe9fdd875, SUC_lexicon_tags_31 },
    { 0x5dd876, SUC_lexicon_tags_10 },
    { 0xc2fdd877, SUC_lexicon_tags_61 },
    { 0xe77dd878, SUC_lexicon_tags_35 },
    { 0x413dd877, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51dd87d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb3dd87f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfefdd881, SUC_lexicon_tags_27 },
    { 0x265dd881, SUC_lexicon_tags_9 },
    { 0xfd9dd882, SUC_lexicon_tags_2 },
    { 0xd21dd884, SUC_lexicon_tags_15 },
    { 0xba9dd885, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fdd88b, SUC_lexicon_tags_33 },
    { 0xa61dd88b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x39bdd88e, SUC_lexicon_tags_27 },
    { 0x807dd88f, SUC_lexicon_tags_30 },
    { 0xdb5dd890, SUC_lexicon_tags_26 },
    { 0xacbdd890, SUC_lexicon_tags_12 },
    { 0xc05dd892, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687dd898, SUC_lexicon_tags_37 },
    { 0x2b3dd899, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7dd89f, SUC_lexicon_tags_33 },
    { 0xf4ddd8a0, SUC_lexicon_tags_45 },
    { 0x8e3dd8a1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fdd8a7, SUC_lexicon_tags_47 },
    { 0x15fdd8a8, SUC_lexicon_tags_66 },
    { 0x6bddd8a9, SUC_lexicon_tags_9 },
    { 0x3dddd8aa, SUC_lexicon_tags_25 },
    { 0xf57dd8aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fdd8af, SUC_lexicon_tags_47 },
    { 0x3cfdd8b0, SUC_lexicon_tags_37 },
    { 0x6b7dd8b0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x863dd8b3, SUC_lexicon_tags_19 },
    { 0x85bdd8b3, SUC_lexicon_tags_7 },
    { 0xa89dd8b5, SUC_lexicon_tags_12 },
    { 0x771dd8b6, SUC_lexicon_tags_10 },
    { 0xa55dd8b3, SUC_lexicon_tags_15 },
    { 0x8d5dd8b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x725dd8bc, SUC_lexicon_tags_28 },
    { 0xa43dd8bd, SUC_lexicon_tags_43 },
    { 0x2dfdd8bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf79dd8c0, SUC_lexicon_tags_11 },
    { 0xc0bdd8c1, SUC_lexicon_tags_25 },
    { 0xbabdd8c2, SUC_lexicon_tags_23 },
    { 0x261dd8c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ddd8cd, SUC_lexicon_tags_87 },
    { 0xe1bdd8ce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x827dd8d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87dd8d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xac9dd8d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6a3dd8d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x711dd8d9, SUC_lexicon_tags_0 },
    { 0x74ddd8da, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x77ddd8dc, SUC_lexicon_tags_38 },
    { 0xcdbdd8dc, SUC_lexicon_tags_21 },
    { 0x381dd8dd, SUC_lexicon_tags_11 },
    { 0xe6bdd8de, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x8e7dd8e1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fdd8e5, SUC_lexicon_tags_60 },
    { 0xd01dd8e5, SUC_lexicon_tags_235 },
    { 0x443dd8e7, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3dd8eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fdd8f0, SUC_lexicon_tags_17 },
    { 0x433dd8f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddd8f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247dd8f8, SUC_lexicon_tags_23 },
    { 0x419dd8f9, SUC_lexicon_tags_55 },
    { 0xf8bdd8fa, SUC_lexicon_tags_2 },
    { 0xf97dd8fb, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0xf63dd8fd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fdd904, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa13dd906, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x673dd908, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x51ddd90b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bdd910, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7b5dd912, SUC_lexicon_tags_27 },
    { 0xc5fdd913, SUC_lexicon_tags_12 },
    { 0xe2fdd912, SUC_lexicon_tags_38 },
    { 0x6afdd915, SUC_lexicon_tags_14 },
    { 0xbcfdd912, SUC_lexicon_tags_1 },
    { 0xc8fdd917, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e9dd919, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673dd920, SUC_lexicon_tags_9 },
    { 0xc49dd920, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ddd924, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x61bdd926, SUC_lexicon_tags_1 },
    { 0xbddd927, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc17dd929, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3dd931, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c9dd933, SUC_lexicon_tags_27 },
    { 0xf65dd934, SUC_lexicon_tags_17 },
    { 0x493dd935, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fddd93b, SUC_lexicon_tags_11 },
    { 0x139dd93b, SUC_lexicon_tags_0 },
    { 0xdf7dd93c, SUC_lexicon_tags_15 },
    { 0x57dd93e, SUC_lexicon_tags_15 },
    { 0xd9ddd93f, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafddd947, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbcfdd949, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd7bdd94b, SUC_lexicon_tags_68 },
    { 0xfabdd94b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bdd94f, SUC_lexicon_tags_57 },
    { 0x36ddd950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619dd955, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x79ddd957, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x373dd95a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13dd967, SUC_lexicon_tags_41 },
    { 0x2bddd968, SUC_lexicon_tags_129 },
    { 0xd9dd969, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79dd96f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbdd972, SUC_lexicon_tags_9 },
    { 0xcbddd973, SUC_lexicon_tags_17 },
    { 0xa2ddd973, SUC_lexicon_tags_39 },
    { 0x87ddd975, SUC_lexicon_tags_1073 },
    { 0xbc9dd974, SUC_lexicon_tags_7 },
    { 0xce3dd977, SUC_lexicon_tags_66 },
    { 0x657dd977, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bdd97b, SUC_lexicon_tags_17 },
    { 0x489dd97b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc1dd97e, SUC_lexicon_tags_17 },
    { 0xb0bdd97e, SUC_lexicon_tags_16 },
    { 0x8b7dd97e, SUC_lexicon_tags_45 },
    { 0x99fdd981, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e7dd983, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x21dd985, SUC_lexicon_tags_11 },
    { 0x911dd986, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5dd989, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825dd98d, SUC_lexicon_tags_33 },
    { 0x5dfdd98e, SUC_lexicon_tags_45 },
    { 0xf87dd98f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123dd993, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xe79dd995, SUC_lexicon_tags_15 },
    { 0x859dd996, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bdd99a, SUC_lexicon_tags_2 },
    { 0x6fdd99b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x147dd99e, SUC_lexicon_tags_37 },
    { 0x2a3dd99f, SUC_lexicon_tags_2 },
    { 0xa59dd99e, SUC_lexicon_tags_31 },
    { 0xb41dd99e, SUC_lexicon_tags_1 },
    { 0xed7dd9a2, SUC_lexicon_tags_47 },
    { 0x46fdd9a3, SUC_lexicon_tags_4 },
    { 0xcd9dd9a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b7dd9a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899dd9ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc9dd9ad, SUC_lexicon_tags_17 },
    { 0x659dd9ad, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ddd9b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x427dd9b3, SUC_lexicon_tags_1 },
    { 0x873dd9b4, SUC_lexicon_tags_18 },
    { 0x41bdd9b3, SUC_lexicon_tags_62 },
    { 0x5cddd9b6, SUC_lexicon_tags_17 },
    { 0x20bdd9b7, SUC_lexicon_tags_4 },
    { 0xcbddd9b5, SUC_lexicon_tags_7 },
    { 0xabddd9b9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47dd9be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771dd9c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27dd9c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dddd9cb, SUC_lexicon_tags_17 },
    { 0x43bdd9cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd49dd9ce, SUC_lexicon_tags_33 },
    { 0x1fddd9cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3e5dd9d1, SUC_lexicon_tags_12 },
    { 0x66fdd9d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77dd9d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7dd9d9, SUC_lexicon_tags_14 },
    { 0xa15dd9da, SUC_lexicon_tags_26 },
    { 0x61dd9d9, SUC_lexicon_tags_25 },
    { 0xc69dd9dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x827dd9de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fdd9e2, SUC_lexicon_tags_23 },
    { 0xd73dd9e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x427dd9e6, SUC_lexicon_tags_63 },
    { 0xe8bdd9e6, SUC_lexicon_tags_1 },
    { 0x5fddd9e7, SUC_lexicon_tags_17 },
    { 0x4ddd9e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x179dd9ec, SUC_lexicon_tags_44 },
    { 0x8dbdd9ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5dd9f5, SUC_lexicon_tags_9 },
    { 0xe93dd9f6, SUC_lexicon_tags_9 },
    { 0xe77dd9f7, SUC_lexicon_tags_17 },
    { 0x6bddd9f8, SUC_lexicon_tags_297 },
    { 0x265dd9f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bdd9fd, SUC_lexicon_tags_11 },
    { 0xedbdd9fe, SUC_lexicon_tags_12 },
    { 0xb5dd9ff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57dda02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87ddda04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97dda08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x98fdda0a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x263dda0d, SUC_lexicon_tags_55 },
    { 0xc1fdda0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dda11, SUC_lexicon_tags_11 },
    { 0xd15dda11, SUC_lexicon_tags_37 },
    { 0xe7dda13, SUC_lexicon_tags_38 },
    { 0xdc1dda12, SUC_lexicon_tags_8 },
    { 0xaf5dda14, SUC_lexicon_tags_37 },
    { 0x195dda16, SUC_lexicon_tags_82 },
    { 0x5dfdda16, SUC_lexicon_tags_11 },
    { 0xafbdda12, SUC_lexicon_tags_19 },
    { 0x503dda18, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8b7dda1b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5dda20, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x3b9dda22, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ddda2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dda37, SUC_lexicon_tags_17 },
    { 0x709dda38, SUC_lexicon_tags_37 },
    { 0xa31dda39, SUC_lexicon_tags_17 },
    { 0xdd5dda38, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fdda3d, SUC_lexicon_tags_8 },
    { 0x3abdda3d, SUC_lexicon_tags_15 },
    { 0x623dda3f, SUC_lexicon_tags_61 },
    { 0x685dda3f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x60ddda42, SUC_lexicon_tags_40 },
    { 0x355dda43, SUC_lexicon_tags_498 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51dda46, SUC_lexicon_tags_1 },
    { 0xcb9dda47, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb29dda49, SUC_lexicon_tags_29 },
    { 0x793dda4a, SUC_lexicon_tags_11 },
    { 0x985dda4a, SUC_lexicon_tags_11 },
    { 0xb7ddda4c, SUC_lexicon_tags_37 },
    { 0xdc9dda4d, SUC_lexicon_tags_37 },
    { 0x495dda4e, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xd0ddda50, SUC_lexicon_tags_17 },
    { 0x1dddda51, SUC_lexicon_tags_4 },
    { 0x61bdda52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793dda56, SUC_lexicon_tags_46 },
    { 0x693dda56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ddda5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dda60, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x15ddda62, SUC_lexicon_tags_12 },
    { 0x161dda63, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97dda6e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8ddda70, SUC_lexicon_tags_164 },
    { 0x51ddda70, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4f9dda73, SUC_lexicon_tags_15 },
    { 0xb15dda73, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ddda79, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa37dda7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353dda7f, SUC_lexicon_tags_7 },
    { 0x63bdda7f, SUC_lexicon_tags_1 },
    { 0xdcddda81, SUC_lexicon_tags_15 },
    { 0x2e5dda80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1dfdda84, SUC_lexicon_tags_30 },
    { 0x4a7dda84, SUC_lexicon_tags_28 },
    { 0x6f1dda86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9dda8b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109dda8f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb93dda91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789dda95, SUC_lexicon_tags_632 },
    { 0x707dda95, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb53dda98, SUC_lexicon_tags_47 },
    { 0xaaddda99, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xbd9dda9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbddaa1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fddaa4, SUC_lexicon_tags_34 },
    { 0xed1ddaa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfddaa8, SUC_lexicon_tags_0 },
    { 0x73fddaa8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7ddaac, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03ddaaf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bddab3, SUC_lexicon_tags_82 },
    { 0x65bddab4, SUC_lexicon_tags_65 },
    { 0x679ddab5, SUC_lexicon_tags_69 },
    { 0x173ddab6, SUC_lexicon_tags_1 },
    { 0xa25ddab7, SUC_lexicon_tags_1 },
    { 0xe4dddab7, SUC_lexicon_tags_15 },
    { 0x177ddab9, SUC_lexicon_tags_33 },
    { 0xd99ddaba, SUC_lexicon_tags_25 },
    { 0x53fddab6, SUC_lexicon_tags_103 },
    { 0xddddaba, SUC_lexicon_tags_7 },
    { 0xf03ddabd, SUC_lexicon_tags_12 },
    { 0x685ddabd, SUC_lexicon_tags_29 },
    { 0xe1dddabf, SUC_lexicon_tags_63 },
    { 0xdd7ddab8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3ddac7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93fddaca, SUC_lexicon_tags_60 },
    { 0x6a5ddaca, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9ddad1, SUC_lexicon_tags_37 },
    { 0x71ddad1, SUC_lexicon_tags_12 },
    { 0xcc3ddad1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81ddad6, SUC_lexicon_tags_9 },
    { 0x6f9ddad7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe1ddad9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x393ddadb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x17ddadd, SUC_lexicon_tags_37 },
    { 0x6b5ddade, SUC_lexicon_tags_31 },
    { 0x78bddade, SUC_lexicon_tags_6 },
    { 0xa3ddadf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89ddae5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5ddae8, SUC_lexicon_tags_27 },
    { 0xc45ddae8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811ddaef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x451ddaf1, SUC_lexicon_tags_23 },
    { 0x27fddaf2, SUC_lexicon_tags_60 },
    { 0xc8dddaf2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x963ddaf6, SUC_lexicon_tags_80 },
    { 0xb09ddaf6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x945ddaf9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x757ddafc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x187ddaff, SUC_lexicon_tags_17 },
    { 0x691ddb00, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3d1ddb02, SUC_lexicon_tags_25 },
    { 0x3cbddb03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe2dddb05, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x85fddb07, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21ddb0d, SUC_lexicon_tags_11 },
    { 0xf2dddb0e, SUC_lexicon_tags_1 },
    { 0xdefddb0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bddb14, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adddb19, SUC_lexicon_tags_47 },
    { 0xf7bddb19, SUC_lexicon_tags_37 },
    { 0x2fdddb19, SUC_lexicon_tags_37 },
    { 0xc05ddb1c, SUC_lexicon_tags_29 },
    { 0xd19ddb1d, SUC_lexicon_tags_17 },
    { 0x805ddb1d, SUC_lexicon_tags_12 },
    { 0x4f1ddb1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bddb22, SUC_lexicon_tags_39 },
    { 0xd1ddb23, SUC_lexicon_tags_63 },
    { 0xccfddb24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43ddb2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bddb30, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9ddb35, SUC_lexicon_tags_9 },
    { 0xca1ddb35, SUC_lexicon_tags_376 },
    { 0x835ddb37, SUC_lexicon_tags_100 },
    { 0x46fddb37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857ddb43, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65ddb47, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dddb4b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fddb57, SUC_lexicon_tags_12 },
    { 0x3afddb58, SUC_lexicon_tags_15 },
    { 0x205ddb58, SUC_lexicon_tags_0 },
    { 0x36dddb57, SUC_lexicon_tags_11 },
    { 0xaedddb59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7ddb62, SUC_lexicon_tags_43 },
    { 0xb3ddb63, SUC_lexicon_tags_2 },
    { 0xe39ddb64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85ddb6c, SUC_lexicon_tags_11 },
    { 0xc9bddb6c, SUC_lexicon_tags_23 },
    { 0xcc5ddb6d, SUC_lexicon_tags_4 },
    { 0x9e9ddb6f, SUC_lexicon_tags_97 },
    { 0x619ddb6f, SUC_lexicon_tags_17 },
    { 0x9fbddb71, SUC_lexicon_tags_21 },
    { 0xba7ddb71, SUC_lexicon_tags_0 },
    { 0x625ddb6d, SUC_lexicon_tags_7 },
    { 0x8ebddb74, SUC_lexicon_tags_1 },
    { 0xe95ddb75, SUC_lexicon_tags_15 },
    { 0xef3ddb73, SUC_lexicon_tags_12 },
    { 0xaedddb77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x48fddb79, SUC_lexicon_tags_17 },
    { 0xbefddb79, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x221ddb7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb1ddb7e, SUC_lexicon_tags_19 },
    { 0x991ddb7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca9ddb81, SUC_lexicon_tags_39 },
    { 0x467ddb82, SUC_lexicon_tags_30 },
    { 0x369ddb81, SUC_lexicon_tags_47 },
    { 0x39fddb83, SUC_lexicon_tags_1 },
    { 0xcddddb85, SUC_lexicon_tags_15 },
    { 0x5bfddb86, SUC_lexicon_tags_39 },
    { 0xebdddb87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9e5ddb89, SUC_lexicon_tags_1 },
    { 0xc01ddb8a, SUC_lexicon_tags_138 },
    { 0xc63ddb8b, SUC_lexicon_tags_7 },
    { 0xbd1ddb8c, SUC_lexicon_tags_60 },
    { 0x6a1ddb8a, SUC_lexicon_tags_37 },
    { 0xc29ddb8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x50fddb90, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bddb93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7ddb99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03ddb9c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ddba7, SUC_lexicon_tags_65 },
    { 0x809ddba8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dddbab, SUC_lexicon_tags_25 },
    { 0xf91ddbab, SUC_lexicon_tags_17 },
    { 0x721ddbac, SUC_lexicon_tags_17 },
    { 0xa6fddbad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89ddbb2, SUC_lexicon_tags_2 },
    { 0xab1ddbb3, SUC_lexicon_tags_37 },
    { 0xef1ddbb4, SUC_lexicon_tags_2 },
    { 0xa57ddbb5, SUC_lexicon_tags_46 },
    { 0x105ddbb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151ddbba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbddbc0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bddbc3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x941ddbc6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e7ddbc8, SUC_lexicon_tags_108 },
    { 0xff3ddbc9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x591ddbcb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7ddbd1, SUC_lexicon_tags_47 },
    { 0x1d9ddbd2, SUC_lexicon_tags_45 },
    { 0xe89ddbd1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x805ddbd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3ddbdd, SUC_lexicon_tags_43 },
    { 0x5e3ddbde, SUC_lexicon_tags_39 },
    { 0x691ddbde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37ddbe3, SUC_lexicon_tags_29 },
    { 0xf7fddbe3, SUC_lexicon_tags_37 },
    { 0xc7ddbe3, SUC_lexicon_tags_2 },
    { 0xa15ddbe5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1a3ddbe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bddbeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf6fddbed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1ddbf2, SUC_lexicon_tags_64 },
    { 0x79dddbf2, SUC_lexicon_tags_12 },
    { 0xc21ddbf4, SUC_lexicon_tags_47 },
    { 0x9e7ddbf5, SUC_lexicon_tags_12 },
    { 0x4dfddbf6, SUC_lexicon_tags_11 },
    { 0xd8fddbf7, SUC_lexicon_tags_31 },
    { 0xbb7ddbf4, SUC_lexicon_tags_45 },
    { 0x2a3ddbf8, SUC_lexicon_tags_1 },
    { 0xdb7ddbfa, SUC_lexicon_tags_26 },
    { 0xc55ddbfb, SUC_lexicon_tags_1 },
    { 0x211ddbf9, SUC_lexicon_tags_66 },
    { 0x3bbddbf6, SUC_lexicon_tags_27 },
    { 0x969ddbfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbddc01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x801ddc04, SUC_lexicon_tags_33 },
    { 0x561ddc05, SUC_lexicon_tags_37 },
    { 0xd6dddc04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26bddc08, SUC_lexicon_tags_1 },
    { 0xd93ddc09, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabddc0e, SUC_lexicon_tags_11 },
    { 0xa51ddc0f, SUC_lexicon_tags_14 },
    { 0xbffddc10, SUC_lexicon_tags_25 },
    { 0x9a1ddc11, SUC_lexicon_tags_21 },
    { 0xa77ddc11, SUC_lexicon_tags_18 },
    { 0x65fddc11, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x381ddc15, SUC_lexicon_tags_12 },
    { 0xeb3ddc15, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x539ddc19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47ddc1c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87ddc1f, SUC_lexicon_tags_33 },
    { 0xc39ddc1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9ddc25, SUC_lexicon_tags_56 },
    { 0xf21ddc25, SUC_lexicon_tags_64 },
    { 0x689ddc27, SUC_lexicon_tags_160 },
    { 0x15fddc28, SUC_lexicon_tags_17 },
    { 0xe01ddc27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9ddc2c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xebfddc2f, SUC_lexicon_tags_1 },
    { 0xa69ddc30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65ddc33, SUC_lexicon_tags_15 },
    { 0x3b5ddc33, SUC_lexicon_tags_0 },
    { 0x581ddc35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147ddc43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee7ddc45, SUC_lexicon_tags_17 },
    { 0xf1fddc46, SUC_lexicon_tags_65 },
    { 0x381ddc47, SUC_lexicon_tags_37 },
    { 0xd41ddc47, SUC_lexicon_tags_13 },
    { 0x3d3ddc48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fddc4f, SUC_lexicon_tags_1 },
    { 0x3b9ddc50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd13ddc52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x817ddc55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c1ddc57, SUC_lexicon_tags_60 },
    { 0x20bddc58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1efddc5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6ebddc5c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbddc5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae9ddc61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdddc68, SUC_lexicon_tags_25 },
    { 0x263ddc69, SUC_lexicon_tags_30 },
    { 0xcebddc68, SUC_lexicon_tags_3 },
    { 0x7f3ddc6b, SUC_lexicon_tags_63 },
    { 0x4ddddc6c, SUC_lexicon_tags_8 },
    { 0x41bddc6b, SUC_lexicon_tags_63 },
    { 0x9edddc6e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x37dddc70, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x221ddc72, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ddc75, SUC_lexicon_tags_19 },
    { 0xdc3ddc75, SUC_lexicon_tags_37 },
    { 0xb1bddc77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5ddc7b, SUC_lexicon_tags_30 },
    { 0xafddc7c, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21ddc7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1ddc82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fddc89, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9ddc8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x375ddc8f, SUC_lexicon_tags_11 },
    { 0x451ddc90, SUC_lexicon_tags_87 },
    { 0xa1fddc91, SUC_lexicon_tags_11 },
    { 0xc27ddc8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9ddc95, SUC_lexicon_tags_2 },
    { 0xba5ddc96, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x809ddc98, SUC_lexicon_tags_8 },
    { 0xd55ddc99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x459ddc9b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353ddca1, SUC_lexicon_tags_46 },
    { 0xdfddca2, SUC_lexicon_tags_47 },
    { 0x977ddca2, SUC_lexicon_tags_37 },
    { 0xabbddca2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc27ddca6, SUC_lexicon_tags_26 },
    { 0x45fddca6, SUC_lexicon_tags_60 },
    { 0xab1ddca8, SUC_lexicon_tags_25 },
    { 0x425ddca8, SUC_lexicon_tags_25 },
    { 0xfefddca9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2a9ddcac, SUC_lexicon_tags_0 },
    { 0xe4bddcad, SUC_lexicon_tags_2 },
    { 0x87ddcae, SUC_lexicon_tags_21 },
    { 0x62fddcac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dbddcb1, SUC_lexicon_tags_31 },
    { 0x5cdddcb2, SUC_lexicon_tags_27 },
    { 0xa7dddcb3, SUC_lexicon_tags_11 },
    { 0x539ddcb1, SUC_lexicon_tags_7 },
    { 0x277ddcb1, SUC_lexicon_tags_27 },
    { 0xd23ddcb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bddcb9, SUC_lexicon_tags_0 },
    { 0x7b1ddcba, SUC_lexicon_tags_117 },
    { 0x2bddcba, SUC_lexicon_tags_35 },
    { 0xd77ddcba, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1a5ddcbe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edddcc2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3ddcc5, SUC_lexicon_tags_49 },
    { 0xfadddcc6, SUC_lexicon_tags_14 },
    { 0xd43ddcc7, SUC_lexicon_tags_18 },
    { 0xf17ddcc7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x95fddcca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5ddcd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x597ddcd8, SUC_lexicon_tags_26 },
    { 0x7bfddcd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x963ddcdc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8afddcde, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9ddce1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bddce9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x897ddceb, SUC_lexicon_tags_2 },
    { 0x85ddcec, SUC_lexicon_tags_31 },
    { 0x2f9ddceb, SUC_lexicon_tags_1 },
    { 0xd05ddcec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xff5ddcf0, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1ddcf3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dddcf6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x87ddcf8, SUC_lexicon_tags_23 },
    { 0x637ddcf9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fddcfd, SUC_lexicon_tags_17 },
    { 0x9dbddcfe, SUC_lexicon_tags_57 },
    { 0x18dddcfe, SUC_lexicon_tags_7 },
    { 0x339ddcfe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3abddd02, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0xeabddd04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1ddd08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75fddd0a, SUC_lexicon_tags_84 },
    { 0xfa7ddd0b, SUC_lexicon_tags_37 },
    { 0x5c5ddd0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fddd0f, SUC_lexicon_tags_9 },
    { 0xf61ddd10, SUC_lexicon_tags_1 },
    { 0x713ddd11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fddd14, SUC_lexicon_tags_47 },
    { 0xe6dddd14, SUC_lexicon_tags_2 },
    { 0x48dddd15, SUC_lexicon_tags_17 },
    { 0x5d9ddd17, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83ddd1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dddd1e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3ddd27, SUC_lexicon_tags_55 },
    { 0x991ddd27, SUC_lexicon_tags_26 },
    { 0x11bddd29, SUC_lexicon_tags_0 },
    { 0x7fdddd2a, SUC_lexicon_tags_0 },
    { 0xf47ddd2b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe1ddd2d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x431ddd2f, SUC_lexicon_tags_47 },
    { 0x593ddd2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x495ddd32, SUC_lexicon_tags_25 },
    { 0x54bddd33, SUC_lexicon_tags_11 },
    { 0xf59ddd34, SUC_lexicon_tags_2 },
    { 0x115ddd34, SUC_lexicon_tags_15 },
    { 0xfd1ddd36, SUC_lexicon_tags_0 },
    { 0x665ddd37, SUC_lexicon_tags_28 },
    { 0xb4bddd38, SUC_lexicon_tags_17 },
    { 0xdc7ddd36, SUC_lexicon_tags_15 },
    { 0x5cfddd37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55fddd3c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3ddd3f, SUC_lexicon_tags_83 },
    { 0x32dddd40, SUC_lexicon_tags_63 },
    { 0x33dddd41, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bddd48, SUC_lexicon_tags_19 },
    { 0xce9ddd49, SUC_lexicon_tags_11 },
    { 0xa3fddd49, SUC_lexicon_tags_11 },
    { 0xd09ddd4b, SUC_lexicon_tags_37 },
    { 0xcb7ddd4c, SUC_lexicon_tags_17 },
    { 0x5efddd4d, SUC_lexicon_tags_37 },
    { 0x76bddd4e, SUC_lexicon_tags_0 },
    { 0x237ddd4e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdddd53, SUC_lexicon_tags_43 },
    { 0xde9ddd54, SUC_lexicon_tags_17 },
    { 0x3f3ddd55, SUC_lexicon_tags_25 },
    { 0xb27ddd55, SUC_lexicon_tags_65 },
    { 0x881ddd56, SUC_lexicon_tags_53 },
    { 0x22dddd58, SUC_lexicon_tags_15 },
    { 0xbb3ddd59, SUC_lexicon_tags_23 },
    { 0xedbddd5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657ddd60, SUC_lexicon_tags_37 },
    { 0x69fddd60, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x737ddd64, SUC_lexicon_tags_26 },
    { 0x85bddd65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfddd6f, SUC_lexicon_tags_68 },
    { 0xc1dddd70, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7ddd74, SUC_lexicon_tags_32 },
    { 0x13bddd75, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x665ddd78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1ddd7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bddd81, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa1fddd83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bddd88, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bddd8b, SUC_lexicon_tags_15 },
    { 0xe65ddd8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07ddd90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83ddd9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bddda5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaddddaa, SUC_lexicon_tags_1 },
    { 0xe9fdddaa, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bdddae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03dddb4, SUC_lexicon_tags_3 },
    { 0x5ddddb4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67dddba, SUC_lexicon_tags_45 },
    { 0x739dddbb, SUC_lexicon_tags_17 },
    { 0x545dddbb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9dddbe, SUC_lexicon_tags_17 },
    { 0x69dddbf, SUC_lexicon_tags_2 },
    { 0xcf1dddbf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd09dddc2, SUC_lexicon_tags_70 },
    { 0x9cddddc3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27dddc7, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bdddcc, SUC_lexicon_tags_15 },
    { 0x86ddddcd, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0x5a7dddcf, SUC_lexicon_tags_15 },
    { 0x27bdddd0, SUC_lexicon_tags_7 },
    { 0x81bdddd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bddddd6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73dddd9, SUC_lexicon_tags_39 },
    { 0x117dddda, SUC_lexicon_tags_151 },
    { 0xa43ddddb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf99ddddd, SUC_lexicon_tags_23 },
    { 0x255dddde, SUC_lexicon_tags_33 },
    { 0xc3dddde, SUC_lexicon_tags_12 },
    { 0xb91dddde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x611ddde2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bdddec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3dddf0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bdddf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103dddf7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfdddfa, SUC_lexicon_tags_27 },
    { 0xc19dddfb, SUC_lexicon_tags_15 },
    { 0x89bdddfb, SUC_lexicon_tags_29 },
    { 0x3dfdddfd, SUC_lexicon_tags_47 },
    { 0xac9dddfe, SUC_lexicon_tags_7 },
    { 0x20bdddfd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3dde03, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bdde07, SUC_lexicon_tags_16 },
    { 0x651dde08, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457dde0c, SUC_lexicon_tags_11 },
    { 0xdbbdde0d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efdde14, SUC_lexicon_tags_43 },
    { 0x97fdde15, SUC_lexicon_tags_29 },
    { 0x6cbdde15, SUC_lexicon_tags_6 },
    { 0x1c1dde17, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x34bdde19, SUC_lexicon_tags_15 },
    { 0x153dde1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683dde22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84fdde28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc79dde2a, SUC_lexicon_tags_84 },
    { 0x5f9dde2a, SUC_lexicon_tags_11 },
    { 0x321dde2c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5addde2e, SUC_lexicon_tags_12 },
    { 0xa85dde2f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbdde35, SUC_lexicon_tags_25 },
    { 0x147dde36, SUC_lexicon_tags_16 },
    { 0x921dde37, SUC_lexicon_tags_1 },
    { 0x385dde38, SUC_lexicon_tags_13 },
    { 0x79ddde39, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7e1dde3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad7dde3d, SUC_lexicon_tags_57 },
    { 0xbe7dde3d, SUC_lexicon_tags_1 },
    { 0x515dde3e, SUC_lexicon_tags_17 },
    { 0xf03dde40, SUC_lexicon_tags_27 },
    { 0x373dde40, SUC_lexicon_tags_38 },
    { 0x957dde42, SUC_lexicon_tags_6 },
    { 0x805dde43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x43dde45, SUC_lexicon_tags_27 },
    { 0x76ddde46, SUC_lexicon_tags_28 },
    { 0x425dde45, SUC_lexicon_tags_6 },
    { 0x27dde48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57ddde4a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x601dde4c, SUC_lexicon_tags_23 },
    { 0xd23dde4d, SUC_lexicon_tags_79 },
    { 0x169dde4e, SUC_lexicon_tags_7 },
    { 0xa4ddde4f, SUC_lexicon_tags_37 },
    { 0xee1dde50, SUC_lexicon_tags_31 },
    { 0x635dde50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdde54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x937dde56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07dde59, SUC_lexicon_tags_19 },
    { 0x979dde5a, SUC_lexicon_tags_37 },
    { 0xb81dde5b, SUC_lexicon_tags_10 },
    { 0xb4fdde5c, SUC_lexicon_tags_0 },
    { 0x4b7dde5d, SUC_lexicon_tags_160 },
    { 0x1c5dde5b, SUC_lexicon_tags_19 },
    { 0x713dde59, SUC_lexicon_tags_25 },
    { 0xbdfdde60, SUC_lexicon_tags_15 },
    { 0x6ffdde61, SUC_lexicon_tags_3 },
    { 0x679dde62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fdde65, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bdde68, SUC_lexicon_tags_37 },
    { 0xaf7dde68, SUC_lexicon_tags_165 },
    { 0x1cfdde6a, SUC_lexicon_tags_25 },
    { 0xbd5dde6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fdde6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdddde71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x871dde73, SUC_lexicon_tags_12 },
    { 0x3a9dde74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99dde79, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xef5dde7b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fddde7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfdde86, SUC_lexicon_tags_25 },
    { 0x36ddde87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fddde8b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9dde8e, SUC_lexicon_tags_15 },
    { 0x25dde8f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x41fdde92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe03dde94, SUC_lexicon_tags_47 },
    { 0x201dde95, SUC_lexicon_tags_19 },
    { 0x961dde95, SUC_lexicon_tags_35 },
    { 0x63ddde97, SUC_lexicon_tags_29 },
    { 0x743dde98, SUC_lexicon_tags_19 },
    { 0x9bdde97, SUC_lexicon_tags_55 },
    { 0x6a7dde9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8fdde9c, SUC_lexicon_tags_1 },
    { 0xd1fdde9d, SUC_lexicon_tags_445 },
    { 0xaa5dde9e, SUC_lexicon_tags_7 },
    { 0xa6ddde9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1a7ddea1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69ddea6, SUC_lexicon_tags_37 },
    { 0x31dddea7, SUC_lexicon_tags_107 },
    { 0x905ddea8, SUC_lexicon_tags_37 },
    { 0xaedddea9, SUC_lexicon_tags_15 },
    { 0x35dddea9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20dddeaf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf81ddeb1, SUC_lexicon_tags_45 },
    { 0xd7fddeb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915ddeba, SUC_lexicon_tags_1 },
    { 0x221ddeba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899ddec1, SUC_lexicon_tags_14 },
    { 0x333ddec2, SUC_lexicon_tags_33 },
    { 0xa15ddec2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1ddec8, SUC_lexicon_tags_2 },
    { 0x2c3ddec9, SUC_lexicon_tags_7 },
    { 0xc8bddeca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x419ddecc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1dded1, SUC_lexicon_tags_9 },
    { 0x8c5dded1, SUC_lexicon_tags_11 },
    { 0xea3dded3, SUC_lexicon_tags_47 },
    { 0x9f9dded4, SUC_lexicon_tags_25 },
    { 0x445dded4, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73dded8, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bddedf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1ddee5, SUC_lexicon_tags_15 },
    { 0x67ddee6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397ddeee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdddef8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5ddefa, SUC_lexicon_tags_8 },
    { 0x597ddefa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46dddefd, SUC_lexicon_tags_65 },
    { 0x23dddefd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91ddf01, SUC_lexicon_tags_0 },
    { 0xedddf02, SUC_lexicon_tags_84 },
    { 0xc9dddf03, SUC_lexicon_tags_11 },
    { 0xe2bddf04, SUC_lexicon_tags_17 },
    { 0x1e3ddf03, SUC_lexicon_tags_1 },
    { 0xa77ddf03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47ddf0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe37ddf0c, SUC_lexicon_tags_6 },
    { 0x3ddf0c, SUC_lexicon_tags_49 },
    { 0x8cdddf0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x881ddf10, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x439ddf12, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x559ddf14, SUC_lexicon_tags_13 },
    { 0x121ddf15, SUC_lexicon_tags_10 },
    { 0xd39ddf14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc7dddf18, SUC_lexicon_tags_19 },
    { 0x773ddf18, SUC_lexicon_tags_451 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3ddf1c, SUC_lexicon_tags_25 },
    { 0xd77ddf1c, SUC_lexicon_tags_26 },
    { 0xd75ddf1e, SUC_lexicon_tags_11 },
    { 0x775ddf1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe21ddf21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdddf27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf93ddf29, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717ddf2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ddf36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7ddf3e, SUC_lexicon_tags_11 },
    { 0xa35ddf3f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5ddf45, SUC_lexicon_tags_14 },
    { 0x589ddf46, SUC_lexicon_tags_198 },
    { 0xd6fddf47, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b5ddf49, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd49ddf4b, SUC_lexicon_tags_17 },
    { 0xc63ddf4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63ddf4f, SUC_lexicon_tags_10 },
    { 0xe1ddf4f, SUC_lexicon_tags_49 },
    { 0x9c1ddf51, SUC_lexicon_tags_2 },
    { 0x8c3ddf51, SUC_lexicon_tags_37 },
    { 0xb69ddf50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4a7ddf55, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339ddf5a, SUC_lexicon_tags_7 },
    { 0xda3ddf5b, SUC_lexicon_tags_15 },
    { 0x2adddf5c, SUC_lexicon_tags_57 },
    { 0xfc9ddf5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb91ddf5f, SUC_lexicon_tags_11 },
    { 0xd7ddf60, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bddf66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27ddf69, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbddddf6b, SUC_lexicon_tags_12 },
    { 0x2dfddf6c, SUC_lexicon_tags_17 },
    { 0x62fddf6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbddf73, SUC_lexicon_tags_553 },
    { 0xe1fddf73, SUC_lexicon_tags_46 },
    { 0xeb3ddf75, SUC_lexicon_tags_2 },
    { 0x147ddf74, SUC_lexicon_tags_17 },
    { 0x743ddf77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x18dddf79, SUC_lexicon_tags_19 },
    { 0x2b1ddf7a, SUC_lexicon_tags_2 },
    { 0x701ddf7b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7e1ddf7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe4fddf7f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2adddf81, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x39bddf83, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1ddf86, SUC_lexicon_tags_1 },
    { 0xc4bddf86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x97fddf8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7dddf8c, SUC_lexicon_tags_19 },
    { 0xea7ddf8c, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x8ffddf8f, SUC_lexicon_tags_23 },
    { 0x377ddf90, SUC_lexicon_tags_45 },
    { 0x27bddf91, SUC_lexicon_tags_30 },
    { 0x98dddf90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae3ddf94, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435ddf99, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbddf9d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171ddfa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dddfa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3dfddfa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bddfac, SUC_lexicon_tags_1 },
    { 0x53fddfad, SUC_lexicon_tags_17 },
    { 0x9b5ddfae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fddfb1, SUC_lexicon_tags_15 },
    { 0x9fddfb2, SUC_lexicon_tags_31 },
    { 0x167ddfb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfddfb8, SUC_lexicon_tags_11 },
    { 0xc71ddfb9, SUC_lexicon_tags_87 },
    { 0x597ddfba, SUC_lexicon_tags_12 },
    { 0x637ddfbb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3ddfbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575ddfc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1c5ddfc7, SUC_lexicon_tags_392 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1ddfcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27ddfd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd89ddfda, SUC_lexicon_tags_14 },
    { 0x219ddfdb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86fddfdd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829ddfe1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfddfe5, SUC_lexicon_tags_64 },
    { 0xfd7ddfe5, SUC_lexicon_tags_53 },
    { 0xa7fddfe6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ddfea, SUC_lexicon_tags_37 },
    { 0xb19ddfeb, SUC_lexicon_tags_37 },
    { 0xa77ddfec, SUC_lexicon_tags_17 },
    { 0x6e7ddfec, SUC_lexicon_tags_43 },
    { 0x327ddfee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3ddff1, SUC_lexicon_tags_7 },
    { 0xbd3ddff2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9ddff5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x883ddff7, SUC_lexicon_tags_7 },
    { 0xc89ddff8, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfddffb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5e3ddffd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x87bde000, SUC_lexicon_tags_45 },
    { 0xf27de001, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bde005, SUC_lexicon_tags_15 },
    { 0xbe5de005, SUC_lexicon_tags_12 },
    { 0xa73de007, SUC_lexicon_tags_25 },
    { 0xd9fde007, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe4dde00a, SUC_lexicon_tags_14 },
    { 0x333de00b, SUC_lexicon_tags_37 },
    { 0xd41de00b, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x147de00f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dde014, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3cdde016, SUC_lexicon_tags_44 },
    { 0x86bde017, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65de019, SUC_lexicon_tags_19 },
    { 0xf7de019, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1de01e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa1de020, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50dde024, SUC_lexicon_tags_32 },
    { 0x8a5de025, SUC_lexicon_tags_55 },
    { 0xba1de025, SUC_lexicon_tags_8 },
    { 0x2e7de025, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1de02e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301de032, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efde035, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397de03c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x411de03e, SUC_lexicon_tags_63 },
    { 0x51bde03e, SUC_lexicon_tags_54 },
    { 0x187de040, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883de044, SUC_lexicon_tags_31 },
    { 0x291de045, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9de04d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd7de04f, SUC_lexicon_tags_2 },
    { 0xb35de050, SUC_lexicon_tags_15 },
    { 0x6a5de051, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x893de053, SUC_lexicon_tags_26 },
    { 0x1fde054, SUC_lexicon_tags_47 },
    { 0xeffde055, SUC_lexicon_tags_84 },
    { 0x493de055, SUC_lexicon_tags_11 },
    { 0xb67de057, SUC_lexicon_tags_30 },
    { 0x45dde058, SUC_lexicon_tags_31 },
    { 0x6b3de059, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf1de05b, SUC_lexicon_tags_17 },
    { 0x71fde05c, SUC_lexicon_tags_44 },
    { 0xffde05b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xa71de05f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203de063, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945de068, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x897de06b, SUC_lexicon_tags_25 },
    { 0x4fdde06b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191de070, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbdde072, SUC_lexicon_tags_23 },
    { 0xa9bde073, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x56dde076, SUC_lexicon_tags_17 },
    { 0xe61de077, SUC_lexicon_tags_2 },
    { 0x21bde077, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29de080, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19de086, SUC_lexicon_tags_2 },
    { 0x90fde087, SUC_lexicon_tags_11 },
    { 0xe41de087, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x617de08b, SUC_lexicon_tags_7 },
    { 0x25fde08b, SUC_lexicon_tags_12 },
    { 0x6f9de08b, SUC_lexicon_tags_80 },
    { 0xb7dde08c, SUC_lexicon_tags_37 },
    { 0xadde08d, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb21de091, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fde097, SUC_lexicon_tags_29 },
    { 0xa1fde098, SUC_lexicon_tags_7 },
    { 0x1f5de099, SUC_lexicon_tags_15 },
    { 0x59fde098, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x1cfde09c, SUC_lexicon_tags_43 },
    { 0x65de09d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x621de09f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x45fde0a1, SUC_lexicon_tags_48 },
    { 0x6dbde0a2, SUC_lexicon_tags_28 },
    { 0x3e1de0a3, SUC_lexicon_tags_15 },
    { 0xcd7de0a4, SUC_lexicon_tags_38 },
    { 0x51fde0a5, SUC_lexicon_tags_2 },
    { 0x6cdde0a4, SUC_lexicon_tags_2 },
    { 0xa93de0a2, SUC_lexicon_tags_151 },
    { 0xb47de0a3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x257de0ab, SUC_lexicon_tags_29 },
    { 0xa89de0ab, SUC_lexicon_tags_14 },
    { 0xe01de0ac, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xccbde0af, SUC_lexicon_tags_25 },
    { 0x82bde0b0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5de0b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x439de0b7, SUC_lexicon_tags_17 },
    { 0x6a1de0b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3dde0c0, SUC_lexicon_tags_11 },
    { 0xf7dde0c1, SUC_lexicon_tags_27 },
    { 0x1a1de0c2, SUC_lexicon_tags_26 },
    { 0xd0dde0c3, SUC_lexicon_tags_37 },
    { 0x1ddde0c0, SUC_lexicon_tags_1 },
    { 0x5adde0c5, SUC_lexicon_tags_2 },
    { 0x22dde0c1, SUC_lexicon_tags_57 },
    { 0xd5bde0c7, SUC_lexicon_tags_25 },
    { 0xfa1de0c8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647de0d0, SUC_lexicon_tags_68 },
    { 0xbd3de0d0, SUC_lexicon_tags_2 },
    { 0x8e3de0d2, SUC_lexicon_tags_17 },
    { 0x375de0d1, SUC_lexicon_tags_33 },
    { 0x729de0d0, SUC_lexicon_tags_31 },
    { 0x8afde0d5, SUC_lexicon_tags_15 },
    { 0x1f1de0d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd75de0d8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535de0dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81de0e2, SUC_lexicon_tags_12 },
    { 0x325de0e2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bde0eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3de0ee, SUC_lexicon_tags_38 },
    { 0x167de0ef, SUC_lexicon_tags_2 },
    { 0x72bde0f0, SUC_lexicon_tags_14 },
    { 0xeffde0f1, SUC_lexicon_tags_8 },
    { 0x69bde0f1, SUC_lexicon_tags_87 },
    { 0xdddde0f2, SUC_lexicon_tags_2 },
    { 0x1dfde0f0, SUC_lexicon_tags_25 },
    { 0xa5bde0f5, SUC_lexicon_tags_16 },
    { 0x437de0f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x159de0f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89de0ff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa59de101, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5de104, SUC_lexicon_tags_37 },
    { 0xb89de105, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3de10c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907de110, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90fde112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfde115, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffde119, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207de11e, SUC_lexicon_tags_7 },
    { 0xaedde11e, SUC_lexicon_tags_1 },
    { 0x9d1de120, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211de124, SUC_lexicon_tags_15 },
    { 0xe3de124, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffde128, SUC_lexicon_tags_15 },
    { 0x2c9de129, SUC_lexicon_tags_49 },
    { 0x385de129, SUC_lexicon_tags_27 },
    { 0x98dde12b, SUC_lexicon_tags_31 },
    { 0x681de12c, SUC_lexicon_tags_47 },
    { 0x903de12d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1bde12f, SUC_lexicon_tags_38 },
    { 0xfe9de130, SUC_lexicon_tags_37 },
    { 0x6b3de12f, SUC_lexicon_tags_31 },
    { 0x95bde12f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d5de134, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xab3de136, SUC_lexicon_tags_12 },
    { 0xd77de136, SUC_lexicon_tags_63 },
    { 0x5c5de138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1de13b, SUC_lexicon_tags_12 },
    { 0xb1fde13c, SUC_lexicon_tags_37 },
    { 0x957de13d, SUC_lexicon_tags_0 },
    { 0xe1de13e, SUC_lexicon_tags_29 },
    { 0xb13de13f, SUC_lexicon_tags_15 },
    { 0x483de13f, SUC_lexicon_tags_23 },
    { 0x3b5de13f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9a5de143, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xc03de145, SUC_lexicon_tags_0 },
    { 0x47fde145, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99de14c, SUC_lexicon_tags_15 },
    { 0x7b3de14d, SUC_lexicon_tags_75 },
    { 0xedfde14e, SUC_lexicon_tags_14 },
    { 0xad9de14f, SUC_lexicon_tags_68 },
    { 0x86bde14e, SUC_lexicon_tags_15 },
    { 0x9edde151, SUC_lexicon_tags_37 },
    { 0x777de151, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01de15a, SUC_lexicon_tags_45 },
    { 0x1e5de15b, SUC_lexicon_tags_14 },
    { 0xa43de15b, SUC_lexicon_tags_27 },
    { 0x37dde15c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381de162, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x65bde165, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd8fde167, SUC_lexicon_tags_27 },
    { 0x663de168, SUC_lexicon_tags_62 },
    { 0x493de169, SUC_lexicon_tags_17 },
    { 0x55fde16a, SUC_lexicon_tags_11 },
    { 0x77de167, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3de16e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599de172, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fde176, SUC_lexicon_tags_33 },
    { 0xb81de176, SUC_lexicon_tags_47 },
    { 0x7e1de178, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bde17c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85de184, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3de18a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x713de18d, SUC_lexicon_tags_56 },
    { 0x5dfde18e, SUC_lexicon_tags_45 },
    { 0x6f9de18f, SUC_lexicon_tags_61 },
    { 0xf51de18e, SUC_lexicon_tags_12 },
    { 0x439de191, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69de199, SUC_lexicon_tags_5 },
    { 0xfa1de199, SUC_lexicon_tags_17 },
    { 0x90bde19b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92dde19e, SUC_lexicon_tags_2 },
    { 0xee9de19f, SUC_lexicon_tags_2 },
    { 0xef9de1a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x423de1a2, SUC_lexicon_tags_79 },
    { 0x8b9de1a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afde1ac, SUC_lexicon_tags_27 },
    { 0xd75de1ac, SUC_lexicon_tags_5 },
    { 0xdd7de1ac, SUC_lexicon_tags_29 },
    { 0x567de1af, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9de1b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dde1bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdde1c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73de1c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7de1c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801de1cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb75de1cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3de1d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7de1d6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5de1de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3de1e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8c3de1e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689de1e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555de1ed, SUC_lexicon_tags_25 },
    { 0x73de1ee, SUC_lexicon_tags_2 },
    { 0xca7de1ef, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcf1de1f1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bde1f7, SUC_lexicon_tags_11 },
    { 0xf41de1f8, SUC_lexicon_tags_7 },
    { 0xf27de1f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531de1fe, SUC_lexicon_tags_7 },
    { 0x45bde1fe, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x291de201, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x865de204, SUC_lexicon_tags_12 },
    { 0x991de204, SUC_lexicon_tags_12 },
    { 0xe8bde206, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xee9de208, SUC_lexicon_tags_12 },
    { 0xf25de209, SUC_lexicon_tags_12 },
    { 0x47bde20a, SUC_lexicon_tags_33 },
    { 0x1edde20a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc3de20d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fde211, SUC_lexicon_tags_107 },
    { 0xcf1de212, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75de216, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3cfde218, SUC_lexicon_tags_1 },
    { 0x13dde219, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5de21d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x24fde21f, SUC_lexicon_tags_28 },
    { 0xebfde21f, SUC_lexicon_tags_37 },
    { 0xb41de221, SUC_lexicon_tags_15 },
    { 0x709de222, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8dfde224, SUC_lexicon_tags_60 },
    { 0x1e5de225, SUC_lexicon_tags_6 },
    { 0x561de224, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ddde228, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93fde22a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1de22e, SUC_lexicon_tags_2 },
    { 0x3cbde22f, SUC_lexicon_tags_37 },
    { 0x46fde22f, SUC_lexicon_tags_19 },
    { 0x4f1de22f, SUC_lexicon_tags_47 },
    { 0x2fbde230, SUC_lexicon_tags_0 },
    { 0x15de231, SUC_lexicon_tags_7 },
    { 0xbe7de234, SUC_lexicon_tags_15 },
    { 0xc65de22f, SUC_lexicon_tags_15 },
    { 0xbadde236, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bde239, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bde23e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafde243, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x781de245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x725de247, SUC_lexicon_tags_49 },
    { 0x165de248, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bde24b, SUC_lexicon_tags_38 },
    { 0xe7de24b, SUC_lexicon_tags_27 },
    { 0xf25de24d, SUC_lexicon_tags_11 },
    { 0x9adde24e, SUC_lexicon_tags_37 },
    { 0xc23de24f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebde253, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91de257, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2ddde259, SUC_lexicon_tags_25 },
    { 0xf2dde25a, SUC_lexicon_tags_1 },
    { 0xb11de25a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bde261, SUC_lexicon_tags_33 },
    { 0xba1de262, SUC_lexicon_tags_28 },
    { 0xb3de263, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x227de266, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67de269, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfde270, SUC_lexicon_tags_37 },
    { 0x9b1de271, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efde27c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5de27e, SUC_lexicon_tags_27 },
    { 0x2bde27e, SUC_lexicon_tags_7 },
    { 0x4edde280, SUC_lexicon_tags_118 },
    { 0xdd7de281, SUC_lexicon_tags_7 },
    { 0xf5fde281, SUC_lexicon_tags_65 },
    { 0x7e5de283, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x629de286, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7de28a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23de28d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69de290, SUC_lexicon_tags_19 },
    { 0xb3dde290, SUC_lexicon_tags_2 },
    { 0x655de292, SUC_lexicon_tags_2 },
    { 0xf5fde293, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dde296, SUC_lexicon_tags_38 },
    { 0x671de297, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91de29b, SUC_lexicon_tags_32 },
    { 0x8d5de29b, SUC_lexicon_tags_17 },
    { 0xf61de29b, SUC_lexicon_tags_4 },
    { 0xffbde29c, SUC_lexicon_tags_7 },
    { 0xedbde29f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1de2aa, SUC_lexicon_tags_23 },
    { 0x37de2ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa05de2ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14fde2af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9de2b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13de2b8, SUC_lexicon_tags_148 },
    { 0xb9dde2b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9de2bc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefde2c2, SUC_lexicon_tags_11 },
    { 0xc15de2c3, SUC_lexicon_tags_25 },
    { 0xbd5de2c4, SUC_lexicon_tags_6 },
    { 0x635de2c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439de2c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991de2cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefde2d5, SUC_lexicon_tags_56 },
    { 0x905de2d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183de2dc, SUC_lexicon_tags_7 },
    { 0xfa3de2dd, SUC_lexicon_tags_38 },
    { 0x6dbde2de, SUC_lexicon_tags_29 },
    { 0x813de2df, SUC_lexicon_tags_1 },
    { 0xa79de2e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33de2e3, SUC_lexicon_tags_1 },
    { 0xd01de2e4, SUC_lexicon_tags_7 },
    { 0x9a3de2e5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa31de2e7, SUC_lexicon_tags_9 },
    { 0x735de2e8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9de2ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fde2f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557de2f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1edde2f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdde2ff, SUC_lexicon_tags_37 },
    { 0x811de300, SUC_lexicon_tags_64 },
    { 0x5fde2ff, SUC_lexicon_tags_4 },
    { 0x183de302, SUC_lexicon_tags_17 },
    { 0x3c5de303, SUC_lexicon_tags_4 },
    { 0xc71de2ff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x437de306, SUC_lexicon_tags_45 },
    { 0x79de307, SUC_lexicon_tags_63 },
    { 0x3ffde307, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4b3de30a, SUC_lexicon_tags_29 },
    { 0x1d5de30b, SUC_lexicon_tags_10 },
    { 0x37dde30b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x195de30e, SUC_lexicon_tags_11 },
    { 0xa2dde30e, SUC_lexicon_tags_22 },
    { 0x1efde310, SUC_lexicon_tags_25 },
    { 0x53fde310, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7de314, SUC_lexicon_tags_25 },
    { 0xb1bde315, SUC_lexicon_tags_17 },
    { 0x27dde316, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddde319, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5b5de31b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bde31e, SUC_lexicon_tags_34 },
    { 0x171de31e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11de325, SUC_lexicon_tags_37 },
    { 0x647de326, SUC_lexicon_tags_15 },
    { 0x575de327, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3de32d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffde338, SUC_lexicon_tags_2 },
    { 0xa7bde339, SUC_lexicon_tags_17 },
    { 0x77bde33a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x85fde33c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289de340, SUC_lexicon_tags_23 },
    { 0x81de341, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dde348, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x72dde34a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa29de34c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263de350, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabde353, SUC_lexicon_tags_27 },
    { 0x90fde353, SUC_lexicon_tags_29 },
    { 0x3d5de355, SUC_lexicon_tags_43 },
    { 0xfa1de356, SUC_lexicon_tags_6 },
    { 0x1d5de355, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6dde359, SUC_lexicon_tags_39 },
    { 0x6b9de35a, SUC_lexicon_tags_15 },
    { 0x39fde35b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bde35e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xeefde360, SUC_lexicon_tags_54 },
    { 0x4f1de360, SUC_lexicon_tags_17 },
    { 0xd75de362, SUC_lexicon_tags_12 },
    { 0xb17de363, SUC_lexicon_tags_37 },
    { 0x3d7de363, SUC_lexicon_tags_19 },
    { 0x967de365, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fde368, SUC_lexicon_tags_17 },
    { 0xc1bde369, SUC_lexicon_tags_21 },
    { 0x265de36a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285de372, SUC_lexicon_tags_29 },
    { 0x535de373, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3de376, SUC_lexicon_tags_1 },
    { 0xc73de377, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x807de37a, SUC_lexicon_tags_47 },
    { 0xa07de37a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23fde37d, SUC_lexicon_tags_25 },
    { 0x9bdde37d, SUC_lexicon_tags_17 },
    { 0x119de37f, SUC_lexicon_tags_28 },
    { 0xbffde380, SUC_lexicon_tags_28 },
    { 0x53de381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fde388, SUC_lexicon_tags_2 },
    { 0x28bde389, SUC_lexicon_tags_719 },
    { 0x231de388, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bde38d, SUC_lexicon_tags_47 },
    { 0xb43de38e, SUC_lexicon_tags_26 },
    { 0x855de38f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bde392, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1de396, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2bde398, SUC_lexicon_tags_11 },
    { 0xc8fde399, SUC_lexicon_tags_87 },
    { 0x445de398, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959de39e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6a3de3a0, SUC_lexicon_tags_3 },
    { 0x30dde3a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349de3a8, SUC_lexicon_tags_0 },
    { 0x601de3a9, SUC_lexicon_tags_46 },
    { 0x4b5de3aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf97de3ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a3de3ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa47de3b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd87de3b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bde3ba, SUC_lexicon_tags_38 },
    { 0x65bde3ba, SUC_lexicon_tags_12 },
    { 0x8b7de3bc, SUC_lexicon_tags_17 },
    { 0xd99de3ba, SUC_lexicon_tags_12 },
    { 0xfa3de3be, SUC_lexicon_tags_26 },
    { 0xdf9de3be, SUC_lexicon_tags_47 },
    { 0xe25de3bb, SUC_lexicon_tags_1 },
    { 0x22fde3c1, SUC_lexicon_tags_84 },
    { 0xad9de3c2, SUC_lexicon_tags_46 },
    { 0x685de3c3, SUC_lexicon_tags_19 },
    { 0xffdde3c4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x45dde3c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593de3cc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xeddde3ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1de3d2, SUC_lexicon_tags_63 },
    { 0x7ebde3d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dde3d9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7de3de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fde3e2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x953de3e5, SUC_lexicon_tags_17 },
    { 0x801de3e5, SUC_lexicon_tags_1 },
    { 0x845de3e7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe83de3e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xee5de3eb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb97de3ed, SUC_lexicon_tags_31 },
    { 0x651de3ee, SUC_lexicon_tags_9 },
    { 0xc27de3ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17de3f3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fde3f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105de3fe, SUC_lexicon_tags_17 },
    { 0x5d7de3fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa55de401, SUC_lexicon_tags_0 },
    { 0x4b5de402, SUC_lexicon_tags_47 },
    { 0x237de403, SUC_lexicon_tags_87 },
    { 0x9ddde401, SUC_lexicon_tags_7 },
    { 0xf61de405, SUC_lexicon_tags_39 },
    { 0x677de406, SUC_lexicon_tags_45 },
    { 0xebde403, SUC_lexicon_tags_33 },
    { 0xff9de404, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfde40c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9de41d, SUC_lexicon_tags_69 },
    { 0x363de41e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49de422, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fde428, SUC_lexicon_tags_12 },
    { 0x74dde429, SUC_lexicon_tags_25 },
    { 0xa9de429, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fde42f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd9de431, SUC_lexicon_tags_12 },
    { 0xa31de432, SUC_lexicon_tags_8 },
    { 0xd7bde431, SUC_lexicon_tags_63 },
    { 0xdcfde434, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207de439, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xf2dde43b, SUC_lexicon_tags_15 },
    { 0x5bfde43b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7de443, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25dde446, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce3de449, SUC_lexicon_tags_1 },
    { 0xcb1de44a, SUC_lexicon_tags_0 },
    { 0xccdde44b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6e5de44d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83de450, SUC_lexicon_tags_7 },
    { 0xd6dde451, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7de458, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7de45d, SUC_lexicon_tags_2 },
    { 0x82fde45e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x545de461, SUC_lexicon_tags_37 },
    { 0x10fde461, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x851de465, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01de468, SUC_lexicon_tags_87 },
    { 0xc65de469, SUC_lexicon_tags_2 },
    { 0xb5bde468, SUC_lexicon_tags_21 },
    { 0xa41de46b, SUC_lexicon_tags_12 },
    { 0xa93de46c, SUC_lexicon_tags_82 },
    { 0xd33de46d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xda9de46f, SUC_lexicon_tags_25 },
    { 0xce5de46f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfde473, SUC_lexicon_tags_29 },
    { 0x13fde474, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb03de476, SUC_lexicon_tags_19 },
    { 0x42bde477, SUC_lexicon_tags_37 },
    { 0xbf3de478, SUC_lexicon_tags_5 },
    { 0x709de479, SUC_lexicon_tags_7 },
    { 0x1b1de47a, SUC_lexicon_tags_11 },
    { 0x655de47a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5de47e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85de481, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3de485, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83de48e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afde493, SUC_lexicon_tags_12 },
    { 0x5f9de494, SUC_lexicon_tags_6 },
    { 0x71fde495, SUC_lexicon_tags_45 },
    { 0x2c9de495, SUC_lexicon_tags_23 },
    { 0x90bde494, SUC_lexicon_tags_7 },
    { 0x9b3de493, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe09de49a, SUC_lexicon_tags_45 },
    { 0x1b7de49a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xecfde49e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105de4a2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x68bde4a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71de4a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x95bde4ab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33de4b1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55dde4b3, SUC_lexicon_tags_29 },
    { 0x96dde4b4, SUC_lexicon_tags_37 },
    { 0x907de4b4, SUC_lexicon_tags_12 },
    { 0xee7de4b4, SUC_lexicon_tags_9 },
    { 0xbc1de4b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb9bde4b9, SUC_lexicon_tags_29 },
    { 0xf69de4ba, SUC_lexicon_tags_3 },
    { 0xa49de4bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7de4bf, SUC_lexicon_tags_1 },
    { 0x50dde4c0, SUC_lexicon_tags_60 },
    { 0x427de4c1, SUC_lexicon_tags_7 },
    { 0xef5de4c1, SUC_lexicon_tags_12 },
    { 0xc7dde4c3, SUC_lexicon_tags_45 },
    { 0xba5de4c0, SUC_lexicon_tags_11 },
    { 0x69de4c5, SUC_lexicon_tags_27 },
    { 0x553de4c0, SUC_lexicon_tags_1 },
    { 0x9a1de4c7, SUC_lexicon_tags_60 },
    { 0xd5dde4c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbdde4ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3de4cd, SUC_lexicon_tags_9 },
    { 0x7ddde4ce, SUC_lexicon_tags_37 },
    { 0x479de4cf, SUC_lexicon_tags_14 },
    { 0x315de4d0, SUC_lexicon_tags_38 },
    { 0x429de4cf, SUC_lexicon_tags_65 },
    { 0x8a7de4d2, SUC_lexicon_tags_38 },
    { 0xf33de4d2, SUC_lexicon_tags_47 },
    { 0x289de4d4, SUC_lexicon_tags_49 },
    { 0xac7de4d5, SUC_lexicon_tags_14 },
    { 0xfc1de4d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9de4da, SUC_lexicon_tags_19 },
    { 0xd1dde4db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1de4dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13de4e1, SUC_lexicon_tags_12 },
    { 0x96bde4e2, SUC_lexicon_tags_10 },
    { 0x19fde4e3, SUC_lexicon_tags_38 },
    { 0x1efde4e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9de4e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c3de4eb, SUC_lexicon_tags_2 },
    { 0x3b5de4ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403de4f4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69de4f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1de4fa, SUC_lexicon_tags_27 },
    { 0xd29de4fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x245de4fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafde501, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebde505, SUC_lexicon_tags_17 },
    { 0xa23de505, SUC_lexicon_tags_11 },
    { 0x881de505, SUC_lexicon_tags_14 },
    { 0x43dde508, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x773de50a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdde50d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe1fde50f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edde516, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbde51a, SUC_lexicon_tags_55 },
    { 0xa05de51b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x319de51e, SUC_lexicon_tags_23 },
    { 0x941de51f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb51de521, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x21de524, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45de528, SUC_lexicon_tags_0 },
    { 0x519de529, SUC_lexicon_tags_15 },
    { 0xd4fde528, SUC_lexicon_tags_15 },
    { 0xa37de529, SUC_lexicon_tags_30 },
    { 0x687de52c, SUC_lexicon_tags_16 },
    { 0xadbde52d, SUC_lexicon_tags_2 },
    { 0xfe9de52e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc23de530, SUC_lexicon_tags_1082 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bde533, SUC_lexicon_tags_11 },
    { 0xb93de534, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97de53c, SUC_lexicon_tags_7 },
    { 0x8fde53d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63bde53f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdde544, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbde54a, SUC_lexicon_tags_33 },
    { 0x8cbde54b, SUC_lexicon_tags_15 },
    { 0x5bde54c, SUC_lexicon_tags_17 },
    { 0x707de54d, SUC_lexicon_tags_11 },
    { 0xa47de54d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7de551, SUC_lexicon_tags_37 },
    { 0xc25de552, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849de55e, SUC_lexicon_tags_34 },
    { 0x97dde55f, SUC_lexicon_tags_15 },
    { 0x471de560, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbde566, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79fde568, SUC_lexicon_tags_19 },
    { 0x269de569, SUC_lexicon_tags_8 },
    { 0xd9fde56a, SUC_lexicon_tags_12 },
    { 0x899de56a, SUC_lexicon_tags_63 },
    { 0xfd1de56c, SUC_lexicon_tags_1 },
    { 0xb23de56d, SUC_lexicon_tags_79 },
    { 0x3b5de56e, SUC_lexicon_tags_9 },
    { 0xdd9de56f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x301de572, SUC_lexicon_tags_23 },
    { 0xcc3de573, SUC_lexicon_tags_31 },
    { 0x931de572, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5de57a, SUC_lexicon_tags_60 },
    { 0xdb9de57a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x743de57d, SUC_lexicon_tags_11 },
    { 0xc0fde57d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bde581, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dde584, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafde587, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9de58c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63de590, SUC_lexicon_tags_77 },
    { 0xf07de591, SUC_lexicon_tags_25 },
    { 0x14fde592, SUC_lexicon_tags_37 },
    { 0xa3fde591, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x733de595, SUC_lexicon_tags_7 },
    { 0xb9bde596, SUC_lexicon_tags_26 },
    { 0x5a3de595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22bde599, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x525de59b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87fde59d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x107de59f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafde5a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1de5a9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95de5b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x829de5b4, SUC_lexicon_tags_17 },
    { 0x177de5b5, SUC_lexicon_tags_1 },
    { 0x76fde5b6, SUC_lexicon_tags_23 },
    { 0x2b9de5b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17de5be, SUC_lexicon_tags_11 },
    { 0x8dde5bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199de5c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbde5cc, SUC_lexicon_tags_25 },
    { 0xadbde5cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe71de5cf, SUC_lexicon_tags_23 },
    { 0x8d9de5d0, SUC_lexicon_tags_22 },
    { 0xb09de5d1, SUC_lexicon_tags_37 },
    { 0x2e1de5d1, SUC_lexicon_tags_11 },
    { 0xda3de5d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x861de5d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa97de5d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8efde5da, SUC_lexicon_tags_7 },
    { 0x3e1de5da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bde5de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59de5e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbde5e6, SUC_lexicon_tags_33 },
    { 0x3d5de5e7, SUC_lexicon_tags_12 },
    { 0x6b3de5e8, SUC_lexicon_tags_170 },
    { 0x3a3de5e7, SUC_lexicon_tags_34 },
    { 0x645de5ea, SUC_lexicon_tags_11 },
    { 0x321de5eb, SUC_lexicon_tags_11 },
    { 0x2d5de5ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367de5f0, SUC_lexicon_tags_8 },
    { 0xc2fde5f1, SUC_lexicon_tags_23 },
    { 0x3ddde5f1, SUC_lexicon_tags_16 },
    { 0x3ffde5f3, SUC_lexicon_tags_15 },
    { 0xdb1de5f4, SUC_lexicon_tags_12 },
    { 0xf01de5f5, SUC_lexicon_tags_0 },
    { 0xdb9de5f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7de602, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x733de604, SUC_lexicon_tags_31 },
    { 0xb7fde604, SUC_lexicon_tags_23 },
    { 0x1ebde606, SUC_lexicon_tags_2 },
    { 0x59bde606, SUC_lexicon_tags_47 },
    { 0x551de604, SUC_lexicon_tags_6 },
    { 0xfa9de609, SUC_lexicon_tags_28 },
    { 0xf37de607, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537de60e, SUC_lexicon_tags_0 },
    { 0x21de60f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7de616, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee7de618, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x321de61a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bde61e, SUC_lexicon_tags_0 },
    { 0xd65de61f, SUC_lexicon_tags_65 },
    { 0x14bde620, SUC_lexicon_tags_33 },
    { 0xb91de621, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969de625, SUC_lexicon_tags_60 },
    { 0xf11de626, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8afde629, SUC_lexicon_tags_1 },
    { 0xd61de62a, SUC_lexicon_tags_11 },
    { 0x849de629, SUC_lexicon_tags_12 },
    { 0x689de62a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaabde62e, SUC_lexicon_tags_29 },
    { 0x66bde62f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2d5de631, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xffdde633, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbdfde635, SUC_lexicon_tags_1 },
    { 0x36fde636, SUC_lexicon_tags_37 },
    { 0xa59de636, SUC_lexicon_tags_2 },
    { 0x705de638, SUC_lexicon_tags_26 },
    { 0x13dde636, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113de63d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x415de63f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9de643, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9de646, SUC_lexicon_tags_9 },
    { 0xab1de646, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x619de64a, SUC_lexicon_tags_43 },
    { 0x509de64a, SUC_lexicon_tags_14 },
    { 0xad9de64a, SUC_lexicon_tags_43 },
    { 0x817de64b, SUC_lexicon_tags_16 },
    { 0x2fde64a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffde656, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5de65a, SUC_lexicon_tags_14 },
    { 0xeedde65b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3dde65d, SUC_lexicon_tags_12 },
    { 0xc4bde65e, SUC_lexicon_tags_16 },
    { 0x679de65e, SUC_lexicon_tags_37 },
    { 0x2ebde660, SUC_lexicon_tags_30 },
    { 0xe4bde65f, SUC_lexicon_tags_47 },
    { 0xd97de65d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x769de665, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x779de667, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5de66a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x97fde66d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x805de670, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67bde672, SUC_lexicon_tags_12 },
    { 0xeb3de673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5de676, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebde67f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9de685, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x373de688, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf95de68a, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0x60bde68c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649de692, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59de696, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13de69a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3de69e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf73de6a0, SUC_lexicon_tags_1 },
    { 0xdb1de6a0, SUC_lexicon_tags_37 },
    { 0x697de6a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53dde6a4, SUC_lexicon_tags_15 },
    { 0x519de6a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74dde6a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x525de6aa, SUC_lexicon_tags_17 },
    { 0xdb3de6ab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7de6ae, SUC_lexicon_tags_17 },
    { 0x3a3de6ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7de6b2, SUC_lexicon_tags_68 },
    { 0x9a1de6b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x527de6b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471de6be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xeadde6c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7de6c6, SUC_lexicon_tags_0 },
    { 0x9ffde6c7, SUC_lexicon_tags_14 },
    { 0x58bde6c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09de6cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13de6d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x46fde6d6, SUC_lexicon_tags_1 },
    { 0x5ddde6d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bde6db, SUC_lexicon_tags_2 },
    { 0xac9de6db, SUC_lexicon_tags_27 },
    { 0xae1de6db, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13de6e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbde6e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x301de6e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803de6eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefde6f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xecfde6f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf45de6f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa21de6f6, SUC_lexicon_tags_29 },
    { 0x1bbde6f7, SUC_lexicon_tags_15 },
    { 0x9a9de6f8, SUC_lexicon_tags_63 },
    { 0xf0bde6f9, SUC_lexicon_tags_2 },
    { 0x4b9de6fa, SUC_lexicon_tags_43 },
    { 0x6bbde6fa, SUC_lexicon_tags_25 },
    { 0x977de6fc, SUC_lexicon_tags_35 },
    { 0x2afde6fd, SUC_lexicon_tags_7 },
    { 0x17fde6fd, SUC_lexicon_tags_2 },
    { 0x343de6ff, SUC_lexicon_tags_66 },
    { 0xb41de6fb, SUC_lexicon_tags_129 },
    { 0xc99de6fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dde70c, SUC_lexicon_tags_17 },
    { 0xa4fde70d, SUC_lexicon_tags_2 },
    { 0x8dde70e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x833de711, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3de714, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8d9de716, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9de71c, SUC_lexicon_tags_39 },
    { 0x4abde71c, SUC_lexicon_tags_47 },
    { 0x559de71d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x271de720, SUC_lexicon_tags_12 },
    { 0x51de720, SUC_lexicon_tags_38 },
    { 0xfc3de722, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1de727, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdde72b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7adde72d, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0x505de72f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587de73a, SUC_lexicon_tags_25 },
    { 0x7efde73b, SUC_lexicon_tags_46 },
    { 0x5f7de73a, SUC_lexicon_tags_103 },
    { 0x59de73d, SUC_lexicon_tags_25 },
    { 0x1c1de73e, SUC_lexicon_tags_63 },
    { 0x8a5de73e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5de743, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f9de745, SUC_lexicon_tags_37 },
    { 0x13bde746, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9de74a, SUC_lexicon_tags_37 },
    { 0x48dde74b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x26fde74d, SUC_lexicon_tags_49 },
    { 0xc61de74e, SUC_lexicon_tags_15 },
    { 0x789de74f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493de753, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3de758, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8a9de75a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37de764, SUC_lexicon_tags_25 },
    { 0x783de764, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebde76a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fde775, SUC_lexicon_tags_14 },
    { 0xe2bde775, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5bde778, SUC_lexicon_tags_55 },
    { 0xc49de778, SUC_lexicon_tags_14 },
    { 0xe83de77a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91dde77e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9afde780, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21de78a, SUC_lexicon_tags_7 },
    { 0x9b7de78b, SUC_lexicon_tags_33 },
    { 0x1e1de78c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fde791, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dde795, SUC_lexicon_tags_12 },
    { 0xb41de796, SUC_lexicon_tags_7 },
    { 0xa35de796, SUC_lexicon_tags_37 },
    { 0x27fde798, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655de79c, SUC_lexicon_tags_37 },
    { 0xdabde79d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3fde79f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xee9de7a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbde7a4, SUC_lexicon_tags_11 },
    { 0xdcbde7a5, SUC_lexicon_tags_65 },
    { 0xf33de7a6, SUC_lexicon_tags_9 },
    { 0x9c5de7a6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xfefde7a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339de7af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfde7b2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xfbdde7b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd71de7b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07de7bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbde7c0, SUC_lexicon_tags_46 },
    { 0x129de7c1, SUC_lexicon_tags_204 },
    { 0xae5de7c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7de7c5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbde7d3, SUC_lexicon_tags_30 },
    { 0x1c3de7d4, SUC_lexicon_tags_4 },
    { 0x461de7d4, SUC_lexicon_tags_37 },
    { 0xc87de7d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905de7dc, SUC_lexicon_tags_25 },
    { 0xa8bde7dd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dde7e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x735de7e3, SUC_lexicon_tags_34 },
    { 0x5fde7e3, SUC_lexicon_tags_21 },
    { 0x9b3de7e5, SUC_lexicon_tags_43 },
    { 0x7efde7e6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9de7ea, SUC_lexicon_tags_27 },
    { 0xdb9de7eb, SUC_lexicon_tags_56 },
    { 0x209de7eb, SUC_lexicon_tags_33 },
    { 0x4dbde7ec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbde7f1, SUC_lexicon_tags_10 },
    { 0xa7de7f1, SUC_lexicon_tags_6 },
    { 0x887de7f1, SUC_lexicon_tags_26 },
    { 0x149de7f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181de7f8, SUC_lexicon_tags_17 },
    { 0x9bbde7f8, SUC_lexicon_tags_12 },
    { 0x7a7de7fa, SUC_lexicon_tags_9 },
    { 0x2f5de7f8, SUC_lexicon_tags_28 },
    { 0xc3de7fc, SUC_lexicon_tags_42 },
    { 0xe11de7fb, SUC_lexicon_tags_25 },
    { 0xf49de7fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b3de800, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fde808, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587de80c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x117de80e, SUC_lexicon_tags_47 },
    { 0x1a7de80e, SUC_lexicon_tags_25 },
    { 0x22fde810, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacdde812, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x825de815, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa5dde817, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1c5de819, SUC_lexicon_tags_9 },
    { 0x2a3de81a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2dde81c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67dde820, SUC_lexicon_tags_9 },
    { 0xcdfde821, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2bfde823, SUC_lexicon_tags_15 },
    { 0x801de824, SUC_lexicon_tags_16 },
    { 0xf27de824, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41de828, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b3de82a, SUC_lexicon_tags_84 },
    { 0x339de82b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7de82e, SUC_lexicon_tags_11 },
    { 0x385de82f, SUC_lexicon_tags_4 },
    { 0x64bde82e, SUC_lexicon_tags_1 },
    { 0x41dde831, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385de837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dde83a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb7dde83c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893de843, SUC_lexicon_tags_11 },
    { 0x687de843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f3de846, SUC_lexicon_tags_49 },
    { 0x697de847, SUC_lexicon_tags_12 },
    { 0x5d3de848, SUC_lexicon_tags_6 },
    { 0xc49de849, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5bde84b, SUC_lexicon_tags_47 },
    { 0x33bde84c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9de850, SUC_lexicon_tags_1 },
    { 0xbd3de851, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fde85f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x963de861, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801de86d, SUC_lexicon_tags_1 },
    { 0x521de86e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53de871, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfde874, SUC_lexicon_tags_37 },
    { 0xcd3de875, SUC_lexicon_tags_4 },
    { 0xe0fde876, SUC_lexicon_tags_11 },
    { 0x87bde877, SUC_lexicon_tags_11 },
    { 0x8bde878, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37de87c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793de880, SUC_lexicon_tags_15 },
    { 0x62fde881, SUC_lexicon_tags_23 },
    { 0xf1de880, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39de888, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x459de88a, SUC_lexicon_tags_2 },
    { 0xa19de88a, SUC_lexicon_tags_17 },
    { 0xee9de88c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59de890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfde894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125de89b, SUC_lexicon_tags_19 },
    { 0xd4fde89c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfde89f, SUC_lexicon_tags_45 },
    { 0x6d7de89f, SUC_lexicon_tags_0 },
    { 0x663de8a0, SUC_lexicon_tags_79 },
    { 0x38dde8a0, SUC_lexicon_tags_63 },
    { 0x5dfde8a0, SUC_lexicon_tags_17 },
    { 0xa35de8a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27de8a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3dbde8ab, SUC_lexicon_tags_10 },
    { 0x881de8ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4abde8ae, SUC_lexicon_tags_38 },
    { 0x1ffde8af, SUC_lexicon_tags_24 },
    { 0x47dde8b0, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757de8b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7a7de8b6, SUC_lexicon_tags_25 },
    { 0xaddde8b7, SUC_lexicon_tags_17 },
    { 0xa13de8b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dde8bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51de8be, SUC_lexicon_tags_63 },
    { 0x70fde8be, SUC_lexicon_tags_14 },
    { 0x5a9de8be, SUC_lexicon_tags_11 },
    { 0x485de8be, SUC_lexicon_tags_280 },
    { 0xc7de8c2, SUC_lexicon_tags_28 },
    { 0xbc3de8c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x389de8c5, SUC_lexicon_tags_17 },
    { 0xe9dde8c6, SUC_lexicon_tags_2 },
    { 0x711de8c6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe7de8c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93de8d3, SUC_lexicon_tags_2 },
    { 0x6a3de8d4, SUC_lexicon_tags_33 },
    { 0x4b5de8d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfde8d9, SUC_lexicon_tags_8 },
    { 0x41de8da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9de8e0, SUC_lexicon_tags_63 },
    { 0x1d9de8e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dde8e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fde8eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efde8ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3de8f1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8c1de8f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfde8f6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86bde8f9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39de8fc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9de903, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1de909, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedde916, SUC_lexicon_tags_2 },
    { 0x64bde917, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x851de919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x237de91c, SUC_lexicon_tags_37 },
    { 0x555de91d, SUC_lexicon_tags_61 },
    { 0xc39de91e, SUC_lexicon_tags_30 },
    { 0xd4bde91f, SUC_lexicon_tags_11 },
    { 0xb5dde91e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc85de922, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaebde924, SUC_lexicon_tags_31 },
    { 0xbe5de925, SUC_lexicon_tags_9 },
    { 0x6a5de926, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x855de929, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59de92f, SUC_lexicon_tags_25 },
    { 0xe6fde92f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x56fde932, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81de937, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x239de939, SUC_lexicon_tags_15 },
    { 0x119de93a, SUC_lexicon_tags_77 },
    { 0xac3de93a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x457de93d, SUC_lexicon_tags_65 },
    { 0x16dde93e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x555de940, SUC_lexicon_tags_7 },
    { 0x5de941, SUC_lexicon_tags_10 },
    { 0xe71de942, SUC_lexicon_tags_7 },
    { 0xe51de943, SUC_lexicon_tags_11 },
    { 0xa6dde942, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdde94c, SUC_lexicon_tags_37 },
    { 0xef1de94c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bde951, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49de955, SUC_lexicon_tags_15 },
    { 0x13bde955, SUC_lexicon_tags_46 },
    { 0x815de957, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1de95a, SUC_lexicon_tags_11 },
    { 0xd3dde95a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bde95f, SUC_lexicon_tags_92 },
    { 0x839de95f, SUC_lexicon_tags_47 },
    { 0xecdde95f, SUC_lexicon_tags_37 },
    { 0xc6fde962, SUC_lexicon_tags_63 },
    { 0x7bde963, SUC_lexicon_tags_35 },
    { 0x5c9de964, SUC_lexicon_tags_43 },
    { 0x96dde964, SUC_lexicon_tags_14 },
    { 0xc61de964, SUC_lexicon_tags_1230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1de970, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1de973, SUC_lexicon_tags_23 },
    { 0xec1de974, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1de979, SUC_lexicon_tags_65 },
    { 0xf5bde97a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dde97d, SUC_lexicon_tags_87 },
    { 0xad1de97e, SUC_lexicon_tags_39 },
    { 0x70fde97f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461de983, SUC_lexicon_tags_38 },
    { 0x36bde983, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x73bde987, SUC_lexicon_tags_25 },
    { 0x737de988, SUC_lexicon_tags_25 },
    { 0x37dde989, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1de98f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7de992, SUC_lexicon_tags_65 },
    { 0xf8bde992, SUC_lexicon_tags_2 },
    { 0x117de993, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf9de996, SUC_lexicon_tags_1 },
    { 0xec3de997, SUC_lexicon_tags_6 },
    { 0x45de998, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x271de99b, SUC_lexicon_tags_11 },
    { 0x677de99c, SUC_lexicon_tags_25 },
    { 0x651de99b, SUC_lexicon_tags_60 },
    { 0xa83de99e, SUC_lexicon_tags_23 },
    { 0xa19de99e, SUC_lexicon_tags_27 },
    { 0x5afde9a0, SUC_lexicon_tags_1 },
    { 0x67dde9a0, SUC_lexicon_tags_14 },
    { 0x3dfde9a0, SUC_lexicon_tags_49 },
    { 0x11dde9a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9de9a8, SUC_lexicon_tags_15 },
    { 0x85dde9a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf1de9ab, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xab1de9ad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5de9b1, SUC_lexicon_tags_56 },
    { 0xf4dde9b2, SUC_lexicon_tags_25 },
    { 0x8ffde9b3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44dde9bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87de9bf, SUC_lexicon_tags_27 },
    { 0xb43de9c0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x821de9c3, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa77de9c5, SUC_lexicon_tags_138 },
    { 0x513de9c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bde9ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbde9ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681de9d6, SUC_lexicon_tags_11 },
    { 0x525de9d6, SUC_lexicon_tags_39 },
    { 0x38fde9d6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe91de9db, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bde9de, SUC_lexicon_tags_84 },
    { 0x259de9df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x237de9e2, SUC_lexicon_tags_60 },
    { 0x2ebde9e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57de9e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xabdde9e8, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x26fde9ea, SUC_lexicon_tags_7 },
    { 0x221de9eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdde9f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a3de9f2, SUC_lexicon_tags_31 },
    { 0xd41de9f3, SUC_lexicon_tags_11 },
    { 0x21fde9f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x231de9f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf19de9f8, SUC_lexicon_tags_170 },
    { 0x2afde9f9, SUC_lexicon_tags_1 },
    { 0x335de9fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9de9fd, SUC_lexicon_tags_40 },
    { 0xfbbde9fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dea02, SUC_lexicon_tags_17 },
    { 0x58bdea03, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575dea07, SUC_lexicon_tags_2 },
    { 0xf3fdea07, SUC_lexicon_tags_37 },
    { 0xf63dea07, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfb5dea0b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8bdea0d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85fdea0f, SUC_lexicon_tags_7 },
    { 0xacddea0f, SUC_lexicon_tags_47 },
    { 0x441dea11, SUC_lexicon_tags_11 },
    { 0xecfdea12, SUC_lexicon_tags_46 },
    { 0x1f9dea13, SUC_lexicon_tags_18 },
    { 0x751dea14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3ddea16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x815dea18, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707dea1c, SUC_lexicon_tags_63 },
    { 0xcd3dea1c, SUC_lexicon_tags_29 },
    { 0xcffdea1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1dea21, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efdea25, SUC_lexicon_tags_7 },
    { 0xaa7dea26, SUC_lexicon_tags_112 },
    { 0xf35dea26, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7dea2c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x171dea2e, SUC_lexicon_tags_8 },
    { 0x7b7dea2e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x119dea31, SUC_lexicon_tags_25 },
    { 0x8e5dea31, SUC_lexicon_tags_37 },
    { 0xafdea33, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ddea37, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x71ddea39, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd39dea3b, SUC_lexicon_tags_17 },
    { 0x801dea3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabddea3f, SUC_lexicon_tags_25 },
    { 0xedfdea40, SUC_lexicon_tags_21 },
    { 0xd31dea3f, SUC_lexicon_tags_16 },
    { 0xc93dea41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x17ddea44, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2bbdea46, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371dea50, SUC_lexicon_tags_11 },
    { 0x139dea51, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x911dea53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6fdea55, SUC_lexicon_tags_38 },
    { 0xff1dea56, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x781dea58, SUC_lexicon_tags_2 },
    { 0x41ddea58, SUC_lexicon_tags_40 },
    { 0xd09dea5a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb9fdea5c, SUC_lexicon_tags_85 },
    { 0x301dea5c, SUC_lexicon_tags_37 },
    { 0xa6ddea5e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf0ddea60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239dea66, SUC_lexicon_tags_47 },
    { 0x713dea66, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2addea69, SUC_lexicon_tags_63 },
    { 0x969dea6a, SUC_lexicon_tags_11 },
    { 0x6ffdea69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xffbdea6d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x273dea6f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x93dea71, SUC_lexicon_tags_7 },
    { 0x141dea72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x165dea75, SUC_lexicon_tags_20 },
    { 0xa29dea75, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebdea7a, SUC_lexicon_tags_66 },
    { 0x931dea7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ddea81, SUC_lexicon_tags_1 },
    { 0xaf1dea82, SUC_lexicon_tags_60 },
    { 0xf7fdea81, SUC_lexicon_tags_0 },
    { 0x769dea82, SUC_lexicon_tags_25 },
    { 0xb71dea85, SUC_lexicon_tags_37 },
    { 0x757dea85, SUC_lexicon_tags_11 },
    { 0x829dea87, SUC_lexicon_tags_17 },
    { 0x7b9dea86, SUC_lexicon_tags_0 },
    { 0xa81dea86, SUC_lexicon_tags_44 },
    { 0xda1dea87, SUC_lexicon_tags_7 },
    { 0xaa1dea89, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fdea91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5dea94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x929dea96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ddea99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffdea9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3deaa3, SUC_lexicon_tags_48 },
    { 0x371deaa4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603deaa8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01deaaf, SUC_lexicon_tags_2 },
    { 0xe71deaaf, SUC_lexicon_tags_37 },
    { 0xb83deaaf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9deab6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3c1deab8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x22ddeabb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5deac1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2bddeac3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16fdeac5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7deacb, SUC_lexicon_tags_2 },
    { 0xee7deacc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xfddeace, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379dead2, SUC_lexicon_tags_47 },
    { 0xb97dead2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3dead8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xedddeada, SUC_lexicon_tags_60 },
    { 0xe91deada, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69deade, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d5deae0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7deae3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x871deae6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ddeaeb, SUC_lexicon_tags_7 },
    { 0x943deaec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eddeaf2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23deafd, SUC_lexicon_tags_25 },
    { 0xc37deafd, SUC_lexicon_tags_12 },
    { 0x3dfdeafd, SUC_lexicon_tags_25 },
    { 0x65bdeaff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c7deb02, SUC_lexicon_tags_1 },
    { 0x95ddeb03, SUC_lexicon_tags_25 },
    { 0xfcbdeb03, SUC_lexicon_tags_6 },
    { 0x171deb03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41deb0c, SUC_lexicon_tags_2 },
    { 0xd89deb0c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3deb14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347deb1c, SUC_lexicon_tags_70 },
    { 0xd45deb1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47deb23, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fdeb27, SUC_lexicon_tags_2 },
    { 0xb95deb28, SUC_lexicon_tags_33 },
    { 0x93ddeb29, SUC_lexicon_tags_2 },
    { 0x3deb29, SUC_lexicon_tags_15 },
    { 0x285deb2b, SUC_lexicon_tags_47 },
    { 0x9b1deb28, SUC_lexicon_tags_15 },
    { 0x15ddeb2d, SUC_lexicon_tags_42 },
    { 0x913deb2d, SUC_lexicon_tags_33 },
    { 0x7d3deb2f, SUC_lexicon_tags_0 },
    { 0x811deb2f, SUC_lexicon_tags_47 },
    { 0x77fdeb30, SUC_lexicon_tags_31 },
    { 0xd0ddeb31, SUC_lexicon_tags_1 },
    { 0x2e1deb33, SUC_lexicon_tags_65 },
    { 0x489deb30, SUC_lexicon_tags_47 },
    { 0xd53deb2a, SUC_lexicon_tags_8 },
    { 0xefddeb2a, SUC_lexicon_tags_11 },
    { 0x713deb2d, SUC_lexicon_tags_15 },
    { 0xebdeb38, SUC_lexicon_tags_25 },
    { 0xdefdeb39, SUC_lexicon_tags_2 },
    { 0x8d1deb38, SUC_lexicon_tags_11 },
    { 0x707deb39, SUC_lexicon_tags_6 },
    { 0x875deb3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63fdeb3f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631deb43, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889deb49, SUC_lexicon_tags_47 },
    { 0x297deb4a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21deb4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205deb55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1ddeb57, SUC_lexicon_tags_31 },
    { 0x19fdeb58, SUC_lexicon_tags_1 },
    { 0x787deb59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbdeb5c, SUC_lexicon_tags_98 },
    { 0x7c7deb5d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcdddeb5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7b1deb61, SUC_lexicon_tags_37 },
    { 0x54bdeb62, SUC_lexicon_tags_11 },
    { 0x877deb63, SUC_lexicon_tags_27 },
    { 0xec9deb64, SUC_lexicon_tags_14 },
    { 0x543deb63, SUC_lexicon_tags_30 },
    { 0x44fdeb66, SUC_lexicon_tags_43 },
    { 0x693deb67, SUC_lexicon_tags_0 },
    { 0xf5bdeb61, SUC_lexicon_tags_2 },
    { 0xbfdeb69, SUC_lexicon_tags_0 },
    { 0xb89deb69, SUC_lexicon_tags_37 },
    { 0xaabdeb69, SUC_lexicon_tags_11 },
    { 0xb3bdeb6c, SUC_lexicon_tags_12 },
    { 0x9b5deb6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09deb70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05deb75, SUC_lexicon_tags_35 },
    { 0x303deb76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07deb79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113deb80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07deb85, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863deb8a, SUC_lexicon_tags_7 },
    { 0xba5deb8a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x921deb8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb51deb8f, SUC_lexicon_tags_48 },
    { 0xd51deb8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bfdeb92, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fdeb98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47bdeb9a, SUC_lexicon_tags_47 },
    { 0x207deb9b, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85deb9e, SUC_lexicon_tags_2 },
    { 0xae9deb9f, SUC_lexicon_tags_46 },
    { 0x65ddeb9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2bdeba2, SUC_lexicon_tags_25 },
    { 0x75fdeba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79deba7, SUC_lexicon_tags_2 },
    { 0x435deba8, SUC_lexicon_tags_9 },
    { 0x3c3deba9, SUC_lexicon_tags_2 },
    { 0x9c5deba9, SUC_lexicon_tags_26 },
    { 0x149debab, SUC_lexicon_tags_39 },
    { 0xfe5deba9, SUC_lexicon_tags_11 },
    { 0x72bdebad, SUC_lexicon_tags_1 },
    { 0x8c5debac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc61debb0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bdebb3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3debb6, SUC_lexicon_tags_48 },
    { 0x30bdebb6, SUC_lexicon_tags_15 },
    { 0x6dfdebb8, SUC_lexicon_tags_25 },
    { 0xc8bdebb9, SUC_lexicon_tags_1 },
    { 0x9fddebba, SUC_lexicon_tags_11 },
    { 0xab9debbb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x317debbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5debc1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xca9debc3, SUC_lexicon_tags_4 },
    { 0x13bdebc3, SUC_lexicon_tags_884 },
    { 0xb43debc4, SUC_lexicon_tags_27 },
    { 0x86ddebc6, SUC_lexicon_tags_17 },
    { 0xb75debc7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fdebca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa21debcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853debd2, SUC_lexicon_tags_27 },
    { 0x9d3debd2, SUC_lexicon_tags_22 },
    { 0x9a5debd4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x835debd6, SUC_lexicon_tags_15 },
    { 0x59ddebd7, SUC_lexicon_tags_25 },
    { 0xeb9debd8, SUC_lexicon_tags_48 },
    { 0xf65debd9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ddebdc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x54bdebde, SUC_lexicon_tags_2 },
    { 0x213debdf, SUC_lexicon_tags_12 },
    { 0xdffdebe0, SUC_lexicon_tags_34 },
    { 0x8bddebe0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdddebe6, SUC_lexicon_tags_62 },
    { 0xc6fdebe7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59debea, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fdebf0, SUC_lexicon_tags_12 },
    { 0x67bdebf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67debf5, SUC_lexicon_tags_38 },
    { 0x299debf5, SUC_lexicon_tags_12 },
    { 0x2a7debf5, SUC_lexicon_tags_25 },
    { 0xb17debf8, SUC_lexicon_tags_38 },
    { 0xa5debf9, SUC_lexicon_tags_9 },
    { 0xd01debfa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x523debfd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21debff, SUC_lexicon_tags_9 },
    { 0x967dec00, SUC_lexicon_tags_0 },
    { 0x985debff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7dec07, SUC_lexicon_tags_250 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fdec11, SUC_lexicon_tags_32 },
    { 0xf99dec12, SUC_lexicon_tags_45 },
    { 0x683dec13, SUC_lexicon_tags_47 },
    { 0xd9bdec12, SUC_lexicon_tags_15 },
    { 0x16fdec14, SUC_lexicon_tags_11 },
    { 0x87ddec16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fdec1b, SUC_lexicon_tags_68 },
    { 0x3cfdec1c, SUC_lexicon_tags_87 },
    { 0x757dec1c, SUC_lexicon_tags_37 },
    { 0xc01dec1e, SUC_lexicon_tags_17 },
    { 0xefdec1d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3dec22, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbddec2c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9dec30, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafdec39, SUC_lexicon_tags_17 },
    { 0x62fdec3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1dec3d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bdec40, SUC_lexicon_tags_14 },
    { 0x251dec41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefdec46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fdec49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8a9dec4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1dec51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd69dec53, SUC_lexicon_tags_27 },
    { 0xc23dec54, SUC_lexicon_tags_26 },
    { 0xc57dec55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc43dec57, SUC_lexicon_tags_17 },
    { 0x745dec58, SUC_lexicon_tags_17 },
    { 0xe69dec57, SUC_lexicon_tags_7 },
    { 0xe91dec59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9fddec5c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb7bdec5e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fdec6a, SUC_lexicon_tags_10 },
    { 0x65dec6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x801dec6d, SUC_lexicon_tags_30 },
    { 0x6addec6d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23dec72, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fdec76, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffdec79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21bdec7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb31dec7d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x709dec7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfdec87, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17dec8b, SUC_lexicon_tags_8 },
    { 0x655dec8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61dec91, SUC_lexicon_tags_0 },
    { 0x849dec91, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333dec99, SUC_lexicon_tags_37 },
    { 0xca3dec99, SUC_lexicon_tags_11 },
    { 0x88ddec9b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51deca5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499decab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7decae, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bdecb1, SUC_lexicon_tags_17 },
    { 0xb97decb2, SUC_lexicon_tags_65 },
    { 0xcbbdecb3, SUC_lexicon_tags_66 },
    { 0x16fdecb4, SUC_lexicon_tags_11 },
    { 0xc21decb5, SUC_lexicon_tags_45 },
    { 0xc99decb4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5decba, SUC_lexicon_tags_15 },
    { 0x3f5decba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3decc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x57bdecc9, SUC_lexicon_tags_2 },
    { 0x897decca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abdecd2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3decd5, SUC_lexicon_tags_63 },
    { 0xee9decd6, SUC_lexicon_tags_49 },
    { 0xeaddecd7, SUC_lexicon_tags_2 },
    { 0x26ddecd7, SUC_lexicon_tags_17 },
    { 0x977decd6, SUC_lexicon_tags_12 },
    { 0xf0ddecd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9decdd, SUC_lexicon_tags_25 },
    { 0x9f5decdd, SUC_lexicon_tags_27 },
    { 0xdfdecdf, SUC_lexicon_tags_2 },
    { 0x273decdf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fdece4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffdece7, SUC_lexicon_tags_15 },
    { 0x511dece8, SUC_lexicon_tags_37 },
    { 0xb59dece8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ddecec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867decf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61decf9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd2bdecfb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffdecfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x923ded00, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1ded0a, SUC_lexicon_tags_17 },
    { 0xf7fded0a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ded0e, SUC_lexicon_tags_33 },
    { 0xfdded0f, SUC_lexicon_tags_26 },
    { 0x8efded0f, SUC_lexicon_tags_277 },
    { 0xe11ded10, SUC_lexicon_tags_38 },
    { 0x9e5ded12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3ded15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4a5ded17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffded1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3dded1c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ded20, SUC_lexicon_tags_49 },
    { 0x545ded20, SUC_lexicon_tags_17 },
    { 0xedfded21, SUC_lexicon_tags_17 },
    { 0x1b9ded23, SUC_lexicon_tags_1 },
    { 0x3b7ded23, SUC_lexicon_tags_1 },
    { 0xd95ded25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65ded28, SUC_lexicon_tags_11 },
    { 0x319ded29, SUC_lexicon_tags_47 },
    { 0x9c7ded29, SUC_lexicon_tags_15 },
    { 0x4e3ded2b, SUC_lexicon_tags_108 },
    { 0xda1ded2c, SUC_lexicon_tags_45 },
    { 0x241ded2c, SUC_lexicon_tags_4 },
    { 0xec7ded2e, SUC_lexicon_tags_29 },
    { 0x399ded2c, SUC_lexicon_tags_32 },
    { 0xc0bded30, SUC_lexicon_tags_16 },
    { 0x897ded31, SUC_lexicon_tags_23 },
    { 0xb81ded32, SUC_lexicon_tags_31 },
    { 0xea9ded29, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13ded36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3ded39, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1ded3d, SUC_lexicon_tags_9 },
    { 0xb5dded3e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdded44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad7ded46, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x75fded48, SUC_lexicon_tags_12 },
    { 0x1c9ded48, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x33fded4c, SUC_lexicon_tags_38 },
    { 0xa63ded4c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfded51, SUC_lexicon_tags_12 },
    { 0xb3dded51, SUC_lexicon_tags_12 },
    { 0x8edded53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb07ded55, SUC_lexicon_tags_25 },
    { 0xd21ded56, SUC_lexicon_tags_11 },
    { 0xf7fded56, SUC_lexicon_tags_46 },
    { 0xd21ded58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x78dded5b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abded60, SUC_lexicon_tags_7 },
    { 0xdabded60, SUC_lexicon_tags_37 },
    { 0x5a3ded62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfdded64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83ded67, SUC_lexicon_tags_11 },
    { 0x95fded67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9cbded6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bded6d, SUC_lexicon_tags_98 },
    { 0x4d3ded6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fded79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9dded7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bded7e, SUC_lexicon_tags_37 },
    { 0xb6dded7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fded82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe19ded84, SUC_lexicon_tags_46 },
    { 0xf7ded85, SUC_lexicon_tags_2 },
    { 0xc3bded84, SUC_lexicon_tags_33 },
    { 0xa37ded87, SUC_lexicon_tags_7 },
    { 0x929ded88, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1ded90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf97ded92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787ded97, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x559ded99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fdeda0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3deda8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bdedac, SUC_lexicon_tags_47 },
    { 0x5dbdedac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x291dedaf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bdedb4, SUC_lexicon_tags_108 },
    { 0xabbdedb5, SUC_lexicon_tags_15 },
    { 0x361dedb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7dedc1, SUC_lexicon_tags_37 },
    { 0xb71dedc2, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349dedc7, SUC_lexicon_tags_11 },
    { 0x6f1dedc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x447dedca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbdedce, SUC_lexicon_tags_3 },
    { 0x735dedcf, SUC_lexicon_tags_25 },
    { 0x89dedd0, SUC_lexicon_tags_27 },
    { 0xbbdedd0, SUC_lexicon_tags_3 },
    { 0x509dedd1, SUC_lexicon_tags_14 },
    { 0xaf9dedcf, SUC_lexicon_tags_26 },
    { 0x5ebdedd4, SUC_lexicon_tags_66 },
    { 0xf1fdedd5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3dedd8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3deddc, SUC_lexicon_tags_27 },
    { 0x9d3deddc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaabdeddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d1dede1, SUC_lexicon_tags_1 },
    { 0x3fdede1, SUC_lexicon_tags_7 },
    { 0x23fdede3, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17dedee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1dedf1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35dedf4, SUC_lexicon_tags_2 },
    { 0xdb9dedf5, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abdedf9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fdedfd, SUC_lexicon_tags_27 },
    { 0x51dedfe, SUC_lexicon_tags_2 },
    { 0x47bdedff, SUC_lexicon_tags_37 },
    { 0xe59dedfe, SUC_lexicon_tags_49 },
    { 0xd83dedfe, SUC_lexicon_tags_15 },
    { 0x969dee02, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efdee06, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fdee10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfdee17, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xd8ddee19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bdee1d, SUC_lexicon_tags_12 },
    { 0xbd7dee1e, SUC_lexicon_tags_17 },
    { 0xba1dee1f, SUC_lexicon_tags_14 },
    { 0x3dbdee1d, SUC_lexicon_tags_9 },
    { 0x2f3dee21, SUC_lexicon_tags_7 },
    { 0xbf1dee1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea5dee25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5dee2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fdee2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa61dee31, SUC_lexicon_tags_23 },
    { 0x61bdee32, SUC_lexicon_tags_49 },
    { 0x163dee33, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841dee3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145dee3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x91bdee41, SUC_lexicon_tags_2 },
    { 0xbbddee41, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bdee48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bdee4f, SUC_lexicon_tags_37 },
    { 0xfe7dee50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x423dee52, SUC_lexicon_tags_21 },
    { 0x887dee53, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0x3d5dee55, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7dee64, SUC_lexicon_tags_29 },
    { 0xe2ddee65, SUC_lexicon_tags_1 },
    { 0x479dee66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x251dee68, SUC_lexicon_tags_17 },
    { 0x435dee68, SUC_lexicon_tags_15 },
    { 0x2a1dee6a, SUC_lexicon_tags_35 },
    { 0x6e5dee6b, SUC_lexicon_tags_7 },
    { 0xc49dee6c, SUC_lexicon_tags_12 },
    { 0xf81dee6d, SUC_lexicon_tags_45 },
    { 0xfcddee69, SUC_lexicon_tags_47 },
    { 0xe21dee6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49dee72, SUC_lexicon_tags_59 },
    { 0xad5dee73, SUC_lexicon_tags_1 },
    { 0x249dee73, SUC_lexicon_tags_65 },
    { 0x20bdee73, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x887dee77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x309dee7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7dee7e, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x7f1dee80, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb95dee82, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbdee86, SUC_lexicon_tags_15 },
    { 0x2e1dee87, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dee8b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x383dee8d, SUC_lexicon_tags_15 },
    { 0x2c1dee8e, SUC_lexicon_tags_7 },
    { 0xfd3dee8e, SUC_lexicon_tags_47 },
    { 0xeb5dee90, SUC_lexicon_tags_15 },
    { 0x6b1dee90, SUC_lexicon_tags_12 },
    { 0xbf1dee8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58fdee94, SUC_lexicon_tags_35 },
    { 0x3e1dee95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51dee99, SUC_lexicon_tags_17 },
    { 0x573dee99, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bdeea1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d1deea3, SUC_lexicon_tags_17 },
    { 0x483deea4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fdeea8, SUC_lexicon_tags_25 },
    { 0x8a7deea9, SUC_lexicon_tags_19 },
    { 0xed5deea8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bdeeb1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5deeb5, SUC_lexicon_tags_49 },
    { 0xf61deeb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdeeb9, SUC_lexicon_tags_37 },
    { 0x8e1deeb9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x711deebc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453deec1, SUC_lexicon_tags_37 },
    { 0x24fdeec2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57bdeec4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x223deec7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fddeed0, SUC_lexicon_tags_6 },
    { 0x805deed1, SUC_lexicon_tags_17 },
    { 0x9f5deed2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb6ddeed4, SUC_lexicon_tags_30 },
    { 0x1d1deed5, SUC_lexicon_tags_3 },
    { 0x4e9deed6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bdeeda, SUC_lexicon_tags_14 },
    { 0xfdeeda, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21deee0, SUC_lexicon_tags_12 },
    { 0x823deee0, SUC_lexicon_tags_11 },
    { 0x90fdeee2, SUC_lexicon_tags_2 },
    { 0x9a5deee3, SUC_lexicon_tags_39 },
    { 0x82ddeee4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5a9deee6, SUC_lexicon_tags_25 },
    { 0x3b3deee7, SUC_lexicon_tags_2 },
    { 0xb67deee8, SUC_lexicon_tags_2 },
    { 0xcbdeee9, SUC_lexicon_tags_15 },
    { 0x5deee9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bdeeef, SUC_lexicon_tags_7 },
    { 0xaa5deef0, SUC_lexicon_tags_7 },
    { 0x78bdeeef, SUC_lexicon_tags_17 },
    { 0xeabdeef2, SUC_lexicon_tags_43 },
    { 0xe0bdeef3, SUC_lexicon_tags_19 },
    { 0x7cfdeef4, SUC_lexicon_tags_47 },
    { 0x691deef5, SUC_lexicon_tags_8 },
    { 0x455deef6, SUC_lexicon_tags_7 },
    { 0xc9fdeef7, SUC_lexicon_tags_53 },
    { 0x7e9deef8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x681deefa, SUC_lexicon_tags_47 },
    { 0xe2bdeefa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x93bdeefd, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1def00, SUC_lexicon_tags_27 },
    { 0xa09def01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5fdef03, SUC_lexicon_tags_562 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdef0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x349def0f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe27def11, SUC_lexicon_tags_14 },
    { 0x7fdef11, SUC_lexicon_tags_10 },
    { 0xc7ddef12, SUC_lexicon_tags_11 },
    { 0xad5def12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ddef17, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ddef20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1def24, SUC_lexicon_tags_14 },
    { 0x19bdef25, SUC_lexicon_tags_17 },
    { 0x5cbdef24, SUC_lexicon_tags_15 },
    { 0x9f5def27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfdef2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3def34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479def38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977def3c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7def43, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967def47, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7ddef49, SUC_lexicon_tags_57 },
    { 0xd37def4a, SUC_lexicon_tags_15 },
    { 0x6b9def4b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ddef57, SUC_lexicon_tags_7 },
    { 0xadddef58, SUC_lexicon_tags_63 },
    { 0x729def58, SUC_lexicon_tags_39 },
    { 0x781def5a, SUC_lexicon_tags_37 },
    { 0x6bddef5b, SUC_lexicon_tags_11 },
    { 0x351def5c, SUC_lexicon_tags_14 },
    { 0x7e3def5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2bdef5f, SUC_lexicon_tags_47 },
    { 0xa27def60, SUC_lexicon_tags_25 },
    { 0x249def61, SUC_lexicon_tags_14 },
    { 0xd77def61, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ddef69, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965def6e, SUC_lexicon_tags_37 },
    { 0xb05def6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb75def71, SUC_lexicon_tags_11 },
    { 0x33fdef72, SUC_lexicon_tags_17 },
    { 0xf81def71, SUC_lexicon_tags_8 },
    { 0xb43def74, SUC_lexicon_tags_7 },
    { 0x4e5def75, SUC_lexicon_tags_56 },
    { 0xefdef71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdef79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xecddef7b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x979def7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x457def80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5bdef82, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebdef85, SUC_lexicon_tags_12 },
    { 0xd5bdef86, SUC_lexicon_tags_25 },
    { 0x10fdef87, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x621def89, SUC_lexicon_tags_17 },
    { 0x64bdef8a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7def8d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x58ddef8f, SUC_lexicon_tags_15 },
    { 0x16bdef90, SUC_lexicon_tags_16 },
    { 0xf5ddef91, SUC_lexicon_tags_7 },
    { 0x77fdef91, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21def95, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71fdef99, SUC_lexicon_tags_17 },
    { 0xd67def99, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fddefa1, SUC_lexicon_tags_0 },
    { 0xc5ddefa2, SUC_lexicon_tags_15 },
    { 0x3a1defa2, SUC_lexicon_tags_37 },
    { 0x99defa2, SUC_lexicon_tags_31 },
    { 0xaddefa5, SUC_lexicon_tags_2 },
    { 0xcc5defa5, SUC_lexicon_tags_6 },
    { 0x34ddefa1, SUC_lexicon_tags_37 },
    { 0x6c7defa8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fdefab, SUC_lexicon_tags_27 },
    { 0x30fdefac, SUC_lexicon_tags_7 },
    { 0x46bdefad, SUC_lexicon_tags_71 },
    { 0x5c9defae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1defb1, SUC_lexicon_tags_12 },
    { 0x5d5defb2, SUC_lexicon_tags_49 },
    { 0xd81defb1, SUC_lexicon_tags_32 },
    { 0x963defb4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x865defb6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc39defb8, SUC_lexicon_tags_2 },
    { 0x7d5defb9, SUC_lexicon_tags_2 },
    { 0xfdbdefba, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x929defbc, SUC_lexicon_tags_77 },
    { 0xc2bdefbd, SUC_lexicon_tags_25 },
    { 0x307defbd, SUC_lexicon_tags_79 },
    { 0x173defbe, SUC_lexicon_tags_17 },
    { 0x9c9defbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf67defc2, SUC_lexicon_tags_17 },
    { 0xbabdefc2, SUC_lexicon_tags_11 },
    { 0x2c9defc3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x97defc6, SUC_lexicon_tags_37 },
    { 0xfbbdefc6, SUC_lexicon_tags_26 },
    { 0x4d3defc7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb89defca, SUC_lexicon_tags_37 },
    { 0x97bdefca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x49ddefcd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc31defcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x137defd2, SUC_lexicon_tags_36 },
    { 0x6c7defd3, SUC_lexicon_tags_47 },
    { 0x649defd4, SUC_lexicon_tags_27 },
    { 0x11fdefd5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7defd9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603defdd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x789defdf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bddefe5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xaa3defe7, SUC_lexicon_tags_38 },
    { 0xf59defe7, SUC_lexicon_tags_11 },
    { 0x305defe9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddefee, SUC_lexicon_tags_47 },
    { 0x89defef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x919deff1, SUC_lexicon_tags_67 },
    { 0x6f5deff2, SUC_lexicon_tags_293 },
    { 0xb19deff3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5deff7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x559deff9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x645deffc, SUC_lexicon_tags_37 },
    { 0xcf7deffd, SUC_lexicon_tags_1 },
    { 0x42bdeffe, SUC_lexicon_tags_2 },
    { 0x775defff, SUC_lexicon_tags_17 },
    { 0x97bdeffd, SUC_lexicon_tags_66 },
    { 0xe79df001, SUC_lexicon_tags_0 },
    { 0x1bfdf001, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x517df004, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fdf009, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837df012, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63df020, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb37df022, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03df025, SUC_lexicon_tags_49 },
    { 0x4a7df026, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x62bdf028, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x411df02a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63df032, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17df036, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ddf03d, SUC_lexicon_tags_4 },
    { 0xe97df03e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x52fdf040, SUC_lexicon_tags_11 },
    { 0xc21df040, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfdf046, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8bfdf048, SUC_lexicon_tags_63 },
    { 0xc75df049, SUC_lexicon_tags_27 },
    { 0x81bdf04a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe17df04c, SUC_lexicon_tags_12 },
    { 0xc8fdf04d, SUC_lexicon_tags_25 },
    { 0x991df04d, SUC_lexicon_tags_14 },
    { 0x1afdf04f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x149df051, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33df054, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5d3df056, SUC_lexicon_tags_37 },
    { 0x609df057, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x33df059, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9df05c, SUC_lexicon_tags_30 },
    { 0x645df05d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3efdf05f, SUC_lexicon_tags_41 },
    { 0xcf7df060, SUC_lexicon_tags_17 },
    { 0x4bdf060, SUC_lexicon_tags_11 },
    { 0x7ebdf062, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1df069, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bddf06e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee7df070, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cddf072, SUC_lexicon_tags_23 },
    { 0x617df072, SUC_lexicon_tags_60 },
    { 0xe19df074, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x13fdf076, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9df079, SUC_lexicon_tags_37 },
    { 0x9bfdf07a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17df07f, SUC_lexicon_tags_172 },
    { 0x3d5df07f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x163df083, SUC_lexicon_tags_0 },
    { 0x487df084, SUC_lexicon_tags_10 },
    { 0xd4bdf085, SUC_lexicon_tags_15 },
    { 0xc1bdf086, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11df08c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321df092, SUC_lexicon_tags_39 },
    { 0xaffdf093, SUC_lexicon_tags_6 },
    { 0xc7df094, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1df097, SUC_lexicon_tags_23 },
    { 0x845df098, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x757df09b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x853df09d, SUC_lexicon_tags_33 },
    { 0xb73df09e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x663df0a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf9df0a3, SUC_lexicon_tags_12 },
    { 0x787df0a4, SUC_lexicon_tags_25 },
    { 0x931df0a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5df0b0, SUC_lexicon_tags_23 },
    { 0xe3ddf0b0, SUC_lexicon_tags_33 },
    { 0x113df0b2, SUC_lexicon_tags_28 },
    { 0x44ddf0b3, SUC_lexicon_tags_28 },
    { 0x4b9df0b4, SUC_lexicon_tags_56 },
    { 0xccfdf0b1, SUC_lexicon_tags_27 },
    { 0xf27df0b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475df0bd, SUC_lexicon_tags_1163 },
    { 0, NULL },
    { 0x1bbdf0bf, SUC_lexicon_tags_37 },
    { 0x8f9df0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fddf0cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2addf0ce, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ddf0d8, SUC_lexicon_tags_2 },
    { 0xda9df0d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d1df0db, SUC_lexicon_tags_17 },
    { 0x467df0db, SUC_lexicon_tags_43 },
    { 0xd95df0db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xcaddf0df, SUC_lexicon_tags_0 },
    { 0x619df0e0, SUC_lexicon_tags_15 },
    { 0x93ddf0e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc3df0e3, SUC_lexicon_tags_10 },
    { 0x499df0e3, SUC_lexicon_tags_17 },
    { 0x6efdf0e5, SUC_lexicon_tags_2 },
    { 0xd49df0e6, SUC_lexicon_tags_25 },
    { 0xce1df0e7, SUC_lexicon_tags_37 },
    { 0x5addf0e8, SUC_lexicon_tags_10 },
    { 0xe3ddf0e9, SUC_lexicon_tags_15 },
    { 0xab5df0e7, SUC_lexicon_tags_2 },
    { 0xc5fdf0e6, SUC_lexicon_tags_19 },
    { 0x23df0e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdf0f2, SUC_lexicon_tags_31 },
    { 0xbe9df0f3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x897df0f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9df0f8, SUC_lexicon_tags_38 },
    { 0x613df0f9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa07df0fb, SUC_lexicon_tags_156 },
    { 0xd61df0fc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x495df0fe, SUC_lexicon_tags_45 },
    { 0x3dbdf0ff, SUC_lexicon_tags_43 },
    { 0x67ddf0fe, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x8b5df102, SUC_lexicon_tags_11 },
    { 0x813df102, SUC_lexicon_tags_43 },
    { 0x449df104, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1df107, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x595df109, SUC_lexicon_tags_47 },
    { 0xadddf10a, SUC_lexicon_tags_43 },
    { 0x76ddf109, SUC_lexicon_tags_16 },
    { 0x51bdf10b, SUC_lexicon_tags_12 },
    { 0x3ebdf10d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9df110, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf01df112, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf4fdf114, SUC_lexicon_tags_1 },
    { 0x1ffdf115, SUC_lexicon_tags_25 },
    { 0x89ddf116, SUC_lexicon_tags_38 },
    { 0x521df117, SUC_lexicon_tags_37 },
    { 0xd1bdf116, SUC_lexicon_tags_30 },
    { 0x4b5df119, SUC_lexicon_tags_33 },
    { 0xcc7df119, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1df11d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fdf123, SUC_lexicon_tags_12 },
    { 0x12bdf124, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7df127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf47df129, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01df12f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ddf137, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69df13a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb19df13c, SUC_lexicon_tags_6 },
    { 0xa77df13c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7eddf146, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe23df148, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407df14d, SUC_lexicon_tags_25 },
    { 0x67df14e, SUC_lexicon_tags_26 },
    { 0x46ddf14f, SUC_lexicon_tags_17 },
    { 0x749df14f, SUC_lexicon_tags_43 },
    { 0xab5df151, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3df154, SUC_lexicon_tags_11 },
    { 0xa01df154, SUC_lexicon_tags_11 },
    { 0x981df156, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf3fdf158, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717df15e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7ffdf160, SUC_lexicon_tags_32 },
    { 0xc8bdf160, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x817df164, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97df167, SUC_lexicon_tags_31 },
    { 0xe45df168, SUC_lexicon_tags_129 },
    { 0xdfddf168, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ddf16c, SUC_lexicon_tags_58 },
    { 0xd77df16d, SUC_lexicon_tags_48 },
    { 0x5dbdf16e, SUC_lexicon_tags_37 },
    { 0x35df16f, SUC_lexicon_tags_33 },
    { 0x781df16f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x2e5df172, SUC_lexicon_tags_0 },
    { 0xc53df173, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9dddf175, SUC_lexicon_tags_65 },
    { 0x8b7df175, SUC_lexicon_tags_13 },
    { 0x41bdf177, SUC_lexicon_tags_7 },
    { 0xc43df178, SUC_lexicon_tags_30 },
    { 0xfafdf179, SUC_lexicon_tags_349 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857df17e, SUC_lexicon_tags_33 },
    { 0x225df17f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x13bdf181, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ddf189, SUC_lexicon_tags_87 },
    { 0xa0fdf189, SUC_lexicon_tags_47 },
    { 0x5a3df189, SUC_lexicon_tags_1 },
    { 0x44bdf18c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe4bdf18e, SUC_lexicon_tags_11 },
    { 0x2bdf18f, SUC_lexicon_tags_2 },
    { 0xc4ddf190, SUC_lexicon_tags_1 },
    { 0x6a3df191, SUC_lexicon_tags_45 },
    { 0x911df190, SUC_lexicon_tags_23 },
    { 0xa9bdf193, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x119df195, SUC_lexicon_tags_857 },
    { 0, NULL },
    { 0x20fdf197, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x533df199, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x56bdf19b, SUC_lexicon_tags_45 },
    { 0xbbbdf19b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35df1a0, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xbf1df1a2, SUC_lexicon_tags_37 },
    { 0xdb5df1a3, SUC_lexicon_tags_124 },
    { 0x8afdf1a2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xbffdf1a6, SUC_lexicon_tags_49 },
    { 0x3fdf1a7, SUC_lexicon_tags_38 },
    { 0xc3bdf1a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x25ddf1aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x925df1ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33df1b0, SUC_lexicon_tags_34 },
    { 0x851df1b1, SUC_lexicon_tags_37 },
    { 0x999df1b2, SUC_lexicon_tags_33 },
    { 0xceddf1b2, SUC_lexicon_tags_43 },
    { 0x79fdf1b2, SUC_lexicon_tags_1 },
    { 0x441df1b2, SUC_lexicon_tags_14 },
    { 0x449df1b5, SUC_lexicon_tags_28 },
    { 0x1ddf1b4, SUC_lexicon_tags_87 },
    { 0xe8ddf1b2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39df1bb, SUC_lexicon_tags_0 },
    { 0x665df1bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1df1bf, SUC_lexicon_tags_23 },
    { 0x6efdf1c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3df1c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9df1c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x235df1c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf07df1ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4d5df1cc, SUC_lexicon_tags_8 },
    { 0xe99df1cd, SUC_lexicon_tags_63 },
    { 0xcbddf1cd, SUC_lexicon_tags_1 },
    { 0xc2ddf1cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fdf1d2, SUC_lexicon_tags_17 },
    { 0x3eddf1d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf5df1d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xce5df1d7, SUC_lexicon_tags_14 },
    { 0x9d1df1d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13df1de, SUC_lexicon_tags_11 },
    { 0xb73df1df, SUC_lexicon_tags_8 },
    { 0x55ddf1df, SUC_lexicon_tags_11 },
    { 0xaf7df1e1, SUC_lexicon_tags_38 },
    { 0x9addf1df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bdf1e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8e5df1e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bdf1ee, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5df1f2, SUC_lexicon_tags_17 },
    { 0x9f7df1f3, SUC_lexicon_tags_64 },
    { 0x7b1df1f2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x257df1f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ddf1fb, SUC_lexicon_tags_25 },
    { 0x6afdf1fc, SUC_lexicon_tags_7 },
    { 0xe5fdf1fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bdf202, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57fdf204, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x293df206, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ebdf208, SUC_lexicon_tags_48 },
    { 0xb09df208, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x939df20b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48bdf20d, SUC_lexicon_tags_17 },
    { 0xc41df20e, SUC_lexicon_tags_30 },
    { 0x813df20f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbc7df211, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fdf214, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdf21a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8d3df21c, SUC_lexicon_tags_37 },
    { 0x135df21d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x759df220, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1df223, SUC_lexicon_tags_29 },
    { 0x439df223, SUC_lexicon_tags_38 },
    { 0x337df225, SUC_lexicon_tags_19 },
    { 0xc91df226, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bdf22c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfdf230, SUC_lexicon_tags_30 },
    { 0x193df231, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fdf234, SUC_lexicon_tags_37 },
    { 0x255df235, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775df23c, SUC_lexicon_tags_80 },
    { 0x8f5df23d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9df240, SUC_lexicon_tags_39 },
    { 0x95ddf240, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x139df244, SUC_lexicon_tags_11 },
    { 0xfdbdf245, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985df24b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5bddf24d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bdf250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1df259, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111df25e, SUC_lexicon_tags_65 },
    { 0x11ddf25f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43df264, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bdf271, SUC_lexicon_tags_65 },
    { 0x64fdf272, SUC_lexicon_tags_61 },
    { 0x50bdf273, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3df278, SUC_lexicon_tags_15 },
    { 0xfc5df279, SUC_lexicon_tags_7 },
    { 0xe35df27a, SUC_lexicon_tags_25 },
    { 0xa1fdf27b, SUC_lexicon_tags_2 },
    { 0x377df27c, SUC_lexicon_tags_12 },
    { 0x237df27b, SUC_lexicon_tags_2 },
    { 0xdf7df27e, SUC_lexicon_tags_27 },
    { 0x551df27a, SUC_lexicon_tags_26 },
    { 0x12ddf27a, SUC_lexicon_tags_47 },
    { 0x839df27d, SUC_lexicon_tags_22 },
    { 0xab5df282, SUC_lexicon_tags_45 },
    { 0xca7df280, SUC_lexicon_tags_17 },
    { 0x36fdf284, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x61bdf287, SUC_lexicon_tags_11 },
    { 0x3dfdf287, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x883df28a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabdf28d, SUC_lexicon_tags_0 },
    { 0x839df28d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd49df290, SUC_lexicon_tags_15 },
    { 0x3bbdf291, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9df296, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x709df298, SUC_lexicon_tags_0 },
    { 0xf85df298, SUC_lexicon_tags_17 },
    { 0x331df29a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2bbdf29c, SUC_lexicon_tags_11 },
    { 0x295df29d, SUC_lexicon_tags_21 },
    { 0x6d5df29d, SUC_lexicon_tags_37 },
    { 0x8bddf29f, SUC_lexicon_tags_26 },
    { 0x535df2a0, SUC_lexicon_tags_29 },
    { 0x93ddf2a1, SUC_lexicon_tags_56 },
    { 0xf9ddf2a2, SUC_lexicon_tags_15 },
    { 0x109df2a3, SUC_lexicon_tags_26 },
    { 0xd9bdf2a4, SUC_lexicon_tags_23 },
    { 0x1e9df2a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811df2aa, SUC_lexicon_tags_12 },
    { 0x3e9df2aa, SUC_lexicon_tags_6 },
    { 0xf29df2ac, SUC_lexicon_tags_26 },
    { 0xf41df2ac, SUC_lexicon_tags_1 },
    { 0x2ffdf2ac, SUC_lexicon_tags_49 },
    { 0xd53df2ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5df2b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafdf2b6, SUC_lexicon_tags_12 },
    { 0x745df2b7, SUC_lexicon_tags_32 },
    { 0xcf7df2b7, SUC_lexicon_tags_21 },
    { 0x9b9df2b7, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x931df2bb, SUC_lexicon_tags_1 },
    { 0x3a1df2bc, SUC_lexicon_tags_19 },
    { 0xb89df2bc, SUC_lexicon_tags_43 },
    { 0x27ddf2be, SUC_lexicon_tags_11 },
    { 0x593df2bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5c1df2c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa7ddf2c3, SUC_lexicon_tags_46 },
    { 0x7f5df2c3, SUC_lexicon_tags_12 },
    { 0x221df2c4, SUC_lexicon_tags_39 },
    { 0xcd1df2c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225df2ca, SUC_lexicon_tags_15 },
    { 0x28bdf2ca, SUC_lexicon_tags_12 },
    { 0x65ddf2cc, SUC_lexicon_tags_15 },
    { 0xca1df2ca, SUC_lexicon_tags_43 },
    { 0x7b9df2cd, SUC_lexicon_tags_11 },
    { 0x1e5df2cf, SUC_lexicon_tags_60 },
    { 0xdebdf2cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x23fdf2d2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5df2d7, SUC_lexicon_tags_14 },
    { 0x42bdf2d8, SUC_lexicon_tags_0 },
    { 0xd69df2d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x81ddf2db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bdf2df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31df2e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb49df2e6, SUC_lexicon_tags_11 },
    { 0x84fdf2e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5cddf2e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fdf2f3, SUC_lexicon_tags_48 },
    { 0xdc7df2f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1addf2f6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699df2fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fdf2fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddf301, SUC_lexicon_tags_12 },
    { 0x8fddf302, SUC_lexicon_tags_2 },
    { 0xedbdf303, SUC_lexicon_tags_28 },
    { 0xaa9df304, SUC_lexicon_tags_55 },
    { 0x963df304, SUC_lexicon_tags_68 },
    { 0xa9fdf303, SUC_lexicon_tags_9 },
    { 0x559df305, SUC_lexicon_tags_77 },
    { 0x3e1df308, SUC_lexicon_tags_49 },
    { 0x761df304, SUC_lexicon_tags_0 },
    { 0x13df30a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ddf30f, SUC_lexicon_tags_19 },
    { 0xd39df310, SUC_lexicon_tags_33 },
    { 0xa6bdf311, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x113df314, SUC_lexicon_tags_47 },
    { 0xed5df315, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7df318, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ddf31d, SUC_lexicon_tags_5 },
    { 0x6f1df31e, SUC_lexicon_tags_43 },
    { 0xc3bdf31f, SUC_lexicon_tags_12 },
    { 0x211df31f, SUC_lexicon_tags_28 },
    { 0x633df321, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbdf329, SUC_lexicon_tags_9 },
    { 0x165df329, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1df32e, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13df332, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bdf33a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51df33d, SUC_lexicon_tags_15 },
    { 0xb65df33e, SUC_lexicon_tags_43 },
    { 0xe23df33f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9df342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xec9df345, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xadfdf347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bdf34a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5df34e, SUC_lexicon_tags_1 },
    { 0x72fdf34f, SUC_lexicon_tags_38 },
    { 0x9e3df34f, SUC_lexicon_tags_60 },
    { 0x5c5df350, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77df355, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3df358, SUC_lexicon_tags_6 },
    { 0xad5df359, SUC_lexicon_tags_25 },
    { 0xe47df358, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09df363, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c5df365, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ddf369, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x857df36b, SUC_lexicon_tags_19 },
    { 0x665df36c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3ddf36e, SUC_lexicon_tags_1 },
    { 0x60ddf36f, SUC_lexicon_tags_37 },
    { 0xe45df370, SUC_lexicon_tags_7 },
    { 0xebfdf371, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceddf379, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bdf37c, SUC_lexicon_tags_27 },
    { 0x69bdf37c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1df381, SUC_lexicon_tags_12 },
    { 0x4cbdf382, SUC_lexicon_tags_37 },
    { 0x1bbdf383, SUC_lexicon_tags_13 },
    { 0xd87df384, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa55df386, SUC_lexicon_tags_47 },
    { 0xffdf386, SUC_lexicon_tags_17 },
    { 0x1b9df388, SUC_lexicon_tags_15 },
    { 0x2b5df388, SUC_lexicon_tags_31 },
    { 0x8a1df38a, SUC_lexicon_tags_37 },
    { 0xd6bdf38b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41df38e, SUC_lexicon_tags_29 },
    { 0x5e9df38e, SUC_lexicon_tags_43 },
    { 0x5cbdf38e, SUC_lexicon_tags_45 },
    { 0x9a5df38e, SUC_lexicon_tags_47 },
    { 0x627df392, SUC_lexicon_tags_14 },
    { 0x685df393, SUC_lexicon_tags_14 },
    { 0xacddf392, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3df397, SUC_lexicon_tags_65 },
    { 0xde1df397, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ddf39b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99df3a1, SUC_lexicon_tags_1 },
    { 0x271df3a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd5ddf3a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5d1df3a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17bdf3a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8ffdf3aa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fdf3ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7fddf3b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3dfdf3b2, SUC_lexicon_tags_49 },
    { 0x9b3df3b3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x631df3b6, SUC_lexicon_tags_8 },
    { 0xd09df3b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8ddf3b9, SUC_lexicon_tags_597 },
    { 0x8c5df3ba, SUC_lexicon_tags_107 },
    { 0xb5ddf3b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x639df3be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x733df3c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17df3c4, SUC_lexicon_tags_85 },
    { 0xf5ddf3c5, SUC_lexicon_tags_12 },
    { 0x82fdf3c5, SUC_lexicon_tags_17 },
    { 0x6d3df3c6, SUC_lexicon_tags_17 },
    { 0xfffdf3c4, SUC_lexicon_tags_15 },
    { 0x8e7df3c9, SUC_lexicon_tags_30 },
    { 0x5a9df3ca, SUC_lexicon_tags_66 },
    { 0xc15df3c5, SUC_lexicon_tags_19 },
    { 0x7ffdf3cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7df3cf, SUC_lexicon_tags_25 },
    { 0xb5df3cf, SUC_lexicon_tags_20 },
    { 0x421df3d0, SUC_lexicon_tags_23 },
    { 0xe7df3d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x931df3d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddf3d9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56fdf3dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5df3e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xefbdf3e4, SUC_lexicon_tags_17 },
    { 0xb97df3e5, SUC_lexicon_tags_23 },
    { 0x3d1df3e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x399df3e8, SUC_lexicon_tags_12 },
    { 0x201df3e9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75df3ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ddf3f7, SUC_lexicon_tags_11 },
    { 0xc2ddf3f8, SUC_lexicon_tags_68 },
    { 0xa3bdf3f8, SUC_lexicon_tags_1 },
    { 0x5c3df3f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3df3fe, SUC_lexicon_tags_19 },
    { 0xa61df3ff, SUC_lexicon_tags_8 },
    { 0x1cddf400, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x71fdf402, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x989df404, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbdf407, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x43bdf409, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x527df40c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99df40f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc85df411, SUC_lexicon_tags_38 },
    { 0xff3df412, SUC_lexicon_tags_25 },
    { 0x1d1df413, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6dbdf415, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455df41d, SUC_lexicon_tags_43 },
    { 0x249df41e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53df421, SUC_lexicon_tags_15 },
    { 0xa59df422, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc7df424, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561df42c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67df42f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3ddf431, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149df435, SUC_lexicon_tags_25 },
    { 0x64fdf436, SUC_lexicon_tags_15 },
    { 0xdcfdf437, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x721df439, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x111df43b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ddf446, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1df450, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fdf454, SUC_lexicon_tags_46 },
    { 0xab5df455, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821df459, SUC_lexicon_tags_11 },
    { 0x749df459, SUC_lexicon_tags_9 },
    { 0xc95df459, SUC_lexicon_tags_30 },
    { 0x129df459, SUC_lexicon_tags_21 },
    { 0xaafdf45d, SUC_lexicon_tags_0 },
    { 0x6bfdf45e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51df465, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa4fdf467, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x643df469, SUC_lexicon_tags_594 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfdf46d, SUC_lexicon_tags_33 },
    { 0xf61df46d, SUC_lexicon_tags_87 },
    { 0xa99df46f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9df473, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bdf478, SUC_lexicon_tags_37 },
    { 0xfeddf479, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fdf47f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2cddf481, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xebdf483, SUC_lexicon_tags_47 },
    { 0xc97df483, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa17df486, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73df489, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73df496, SUC_lexicon_tags_15 },
    { 0x3bfdf497, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bdf49a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xee5df49c, SUC_lexicon_tags_17 },
    { 0x6b3df49d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3df4a7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x44bdf4a9, SUC_lexicon_tags_63 },
    { 0x37ddf4aa, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x361df4ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffdf4b0, SUC_lexicon_tags_16 },
    { 0x4a7df4b1, SUC_lexicon_tags_37 },
    { 0x5fbdf4b2, SUC_lexicon_tags_14 },
    { 0x477df4b2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bdf4b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x761df4b9, SUC_lexicon_tags_8 },
    { 0x97df4ba, SUC_lexicon_tags_7 },
    { 0xcaddf4ba, SUC_lexicon_tags_11 },
    { 0xa2fdf4b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82ddf4be, SUC_lexicon_tags_37 },
    { 0xb7fdf4bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59df4c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395df4c9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8f5df4cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97ddf4cd, SUC_lexicon_tags_17 },
    { 0x44fdf4ce, SUC_lexicon_tags_11 },
    { 0xbf3df4cf, SUC_lexicon_tags_26 },
    { 0x4d3df4d0, SUC_lexicon_tags_37 },
    { 0x639df4d1, SUC_lexicon_tags_16 },
    { 0xfdfdf4d1, SUC_lexicon_tags_32 },
    { 0x917df4d3, SUC_lexicon_tags_15 },
    { 0x777df4d4, SUC_lexicon_tags_7 },
    { 0xf47df4d0, SUC_lexicon_tags_17 },
    { 0x66fdf4d6, SUC_lexicon_tags_25 },
    { 0x421df4d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x815df4da, SUC_lexicon_tags_39 },
    { 0xbcfdf4da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x40bdf4dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x30bdf4df, SUC_lexicon_tags_12 },
    { 0x695df4e0, SUC_lexicon_tags_27 },
    { 0x709df4e1, SUC_lexicon_tags_124 },
    { 0x7bddf4df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcfdf4e4, SUC_lexicon_tags_7 },
    { 0x2cfdf4e4, SUC_lexicon_tags_37 },
    { 0x5f1df4e4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2c9df4e8, SUC_lexicon_tags_37 },
    { 0x589df4e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b9df4eb, SUC_lexicon_tags_37 },
    { 0x39fdf4ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535df4f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafdf4f6, SUC_lexicon_tags_13 },
    { 0x8a9df4f7, SUC_lexicon_tags_14 },
    { 0xf3df4f8, SUC_lexicon_tags_27 },
    { 0x5d7df4f9, SUC_lexicon_tags_17 },
    { 0x869df4f6, SUC_lexicon_tags_15 },
    { 0x8b5df4f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x395df4fe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabdf501, SUC_lexicon_tags_38 },
    { 0x899df502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x163df505, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3df50a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x63ddf50c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cddf50e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bdf512, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5df516, SUC_lexicon_tags_25 },
    { 0x791df516, SUC_lexicon_tags_25 },
    { 0x241df518, SUC_lexicon_tags_30 },
    { 0xf6ddf519, SUC_lexicon_tags_2 },
    { 0xb73df51a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd63df51c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129df526, SUC_lexicon_tags_14 },
    { 0x2f9df527, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7df52c, SUC_lexicon_tags_68 },
    { 0x265df52d, SUC_lexicon_tags_5 },
    { 0xe05df52e, SUC_lexicon_tags_45 },
    { 0xee9df52e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19df533, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bfdf535, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05df538, SUC_lexicon_tags_45 },
    { 0xe5ddf539, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x51fdf53b, SUC_lexicon_tags_64 },
    { 0xb0bdf53c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689df541, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3df546, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x787df548, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffdf54d, SUC_lexicon_tags_63 },
    { 0x54bdf54e, SUC_lexicon_tags_37 },
    { 0xfd3df54e, SUC_lexicon_tags_752 },
    { 0, NULL },
    { 0, NULL },
    { 0x375df552, SUC_lexicon_tags_4 },
    { 0x777df553, SUC_lexicon_tags_37 },
    { 0xea9df552, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11df55a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175df55f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11df563, SUC_lexicon_tags_30 },
    { 0xc71df564, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bdf56b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501df56f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfdf573, SUC_lexicon_tags_8 },
    { 0xc9fdf574, SUC_lexicon_tags_32 },
    { 0xa83df575, SUC_lexicon_tags_33 },
    { 0x7e1df576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339df57b, SUC_lexicon_tags_43 },
    { 0x17fdf57c, SUC_lexicon_tags_2 },
    { 0x1fddf57d, SUC_lexicon_tags_47 },
    { 0xc27df57d, SUC_lexicon_tags_9 },
    { 0xee9df57c, SUC_lexicon_tags_33 },
    { 0x1fbdf57f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7df58a, SUC_lexicon_tags_6 },
    { 0xe6ddf58b, SUC_lexicon_tags_25 },
    { 0xf49df58c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x943df58f, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ddf598, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaf5df59a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeefdf59e, SUC_lexicon_tags_1 },
    { 0xf79df59f, SUC_lexicon_tags_10 },
    { 0x42fdf5a0, SUC_lexicon_tags_85 },
    { 0x747df5a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94ddf5a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bdf5aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7df5ad, SUC_lexicon_tags_11 },
    { 0x259df5ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bdf5b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbdf5b8, SUC_lexicon_tags_30 },
    { 0x519df5b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fdf5bc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdf5c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd21df5c5, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x6e9df5c7, SUC_lexicon_tags_14 },
    { 0x6b7df5c8, SUC_lexicon_tags_37 },
    { 0x585df5c9, SUC_lexicon_tags_62 },
    { 0xc67df5ca, SUC_lexicon_tags_28 },
    { 0x31df5cb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf89df5cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd27df5cf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x42fdf5d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37ddf5d3, SUC_lexicon_tags_12 },
    { 0xcf5df5d4, SUC_lexicon_tags_5 },
    { 0x403df5d5, SUC_lexicon_tags_37 },
    { 0x63bdf5d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515df5de, SUC_lexicon_tags_12 },
    { 0x801df5df, SUC_lexicon_tags_17 },
    { 0x641df5de, SUC_lexicon_tags_23 },
    { 0x241df5e0, SUC_lexicon_tags_47 },
    { 0x879df5de, SUC_lexicon_tags_0 },
    { 0x48fdf5e0, SUC_lexicon_tags_32 },
    { 0xa99df5e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05df5e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9df5ed, SUC_lexicon_tags_16 },
    { 0xd15df5ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447df5f2, SUC_lexicon_tags_37 },
    { 0x56fdf5f2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afdf5f8, SUC_lexicon_tags_85 },
    { 0xfafdf5f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabdf5fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x301df5fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bbdf600, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ddf603, SUC_lexicon_tags_33 },
    { 0xf29df604, SUC_lexicon_tags_11 },
    { 0xbbddf604, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21df609, SUC_lexicon_tags_15 },
    { 0x7e3df60a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebdf610, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb89df612, SUC_lexicon_tags_12 },
    { 0x211df612, SUC_lexicon_tags_108 },
    { 0x20fdf614, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319df619, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa05df61b, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bdf623, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9df629, SUC_lexicon_tags_32 },
    { 0xbb7df62a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23df62e, SUC_lexicon_tags_77 },
    { 0x1e9df62f, SUC_lexicon_tags_48 },
    { 0xb9fdf62f, SUC_lexicon_tags_82 },
    { 0xe31df631, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57df636, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51df639, SUC_lexicon_tags_15 },
    { 0x773df639, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x411df63d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x259df63f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x709df642, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xc49df644, SUC_lexicon_tags_49 },
    { 0x83ddf645, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfdf648, SUC_lexicon_tags_29 },
    { 0xcbdf649, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1bbdf64b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x187df64e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfdf652, SUC_lexicon_tags_1 },
    { 0x1efdf653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xba7df655, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7df65c, SUC_lexicon_tags_17 },
    { 0x4a5df65c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf661, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3df665, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xed7df667, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745df66b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81df66e, SUC_lexicon_tags_34 },
    { 0x313df66f, SUC_lexicon_tags_17 },
    { 0xefbdf66e, SUC_lexicon_tags_33 },
    { 0xd2ddf671, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x847df674, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbdbdf676, SUC_lexicon_tags_25 },
    { 0xdf1df677, SUC_lexicon_tags_0 },
    { 0x663df678, SUC_lexicon_tags_6 },
    { 0x3b1df679, SUC_lexicon_tags_31 },
    { 0x52ddf67a, SUC_lexicon_tags_48 },
    { 0x8c5df67b, SUC_lexicon_tags_33 },
    { 0xb3ddf67c, SUC_lexicon_tags_48 },
    { 0x677df677, SUC_lexicon_tags_17 },
    { 0x1e7df67e, SUC_lexicon_tags_25 },
    { 0x53fdf679, SUC_lexicon_tags_1 },
    { 0xb8bdf67c, SUC_lexicon_tags_26 },
    { 0x3e1df67a, SUC_lexicon_tags_29 },
    { 0xca3df682, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7efdf684, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3df687, SUC_lexicon_tags_65 },
    { 0xb83df687, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x98bdf68a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb83df68c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x351df68e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375df692, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257df698, SUC_lexicon_tags_2 },
    { 0x11bdf699, SUC_lexicon_tags_6 },
    { 0x27ddf69a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11df69e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b7df6a0, SUC_lexicon_tags_60 },
    { 0x7e9df6a0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1df6a5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fddf6aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5df6ad, SUC_lexicon_tags_37 },
    { 0xecfdf6ad, SUC_lexicon_tags_37 },
    { 0x483df6ae, SUC_lexicon_tags_1 },
    { 0xe91df6ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01df6b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdddf6b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1df6bc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2f7df6be, SUC_lexicon_tags_2 },
    { 0x65df6be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1df6c2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bdf6c7, SUC_lexicon_tags_49 },
    { 0xbc3df6c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5df6cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7d9df6cd, SUC_lexicon_tags_37 },
    { 0x6cfdf6ce, SUC_lexicon_tags_8 },
    { 0x429df6cf, SUC_lexicon_tags_9 },
    { 0x527df6cf, SUC_lexicon_tags_49 },
    { 0x127df6cd, SUC_lexicon_tags_25 },
    { 0xd5ddf6d2, SUC_lexicon_tags_13 },
    { 0xa1ddf6d3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x739df6d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x55ddf6d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb49df6da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x877df6dc, SUC_lexicon_tags_3 },
    { 0xbfdf6dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671df6e1, SUC_lexicon_tags_11 },
    { 0xf89df6e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddf6e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbdf6ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91df6f1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87df6f4, SUC_lexicon_tags_49 },
    { 0x53bdf6f5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3df6fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x139df6fd, SUC_lexicon_tags_14 },
    { 0x1bfdf6fd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x761df701, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7df705, SUC_lexicon_tags_1 },
    { 0x373df705, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85df70a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53ddf70c, SUC_lexicon_tags_47 },
    { 0x3d3df70d, SUC_lexicon_tags_66 },
    { 0xbddf70d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x14fdf710, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3df713, SUC_lexicon_tags_2 },
    { 0xe51df714, SUC_lexicon_tags_30 },
    { 0x70bdf714, SUC_lexicon_tags_37 },
    { 0x2bdf713, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x865df719, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5df71f, SUC_lexicon_tags_66 },
    { 0x625df71f, SUC_lexicon_tags_12 },
    { 0x143df721, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3addf724, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ddf727, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x671df72a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38ddf72c, SUC_lexicon_tags_37 },
    { 0xd0bdf72d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5df734, SUC_lexicon_tags_13 },
    { 0x9bdf734, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fdf739, SUC_lexicon_tags_4 },
    { 0x27ddf739, SUC_lexicon_tags_12 },
    { 0x2d7df73b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3df73f, SUC_lexicon_tags_21 },
    { 0xe93df73f, SUC_lexicon_tags_25 },
    { 0xf05df73f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223df748, SUC_lexicon_tags_15 },
    { 0x83ddf748, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575df74d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165df753, SUC_lexicon_tags_68 },
    { 0xb53df754, SUC_lexicon_tags_17 },
    { 0xa3fdf755, SUC_lexicon_tags_74 },
    { 0x6eddf754, SUC_lexicon_tags_19 },
    { 0x9a7df757, SUC_lexicon_tags_8 },
    { 0x519df758, SUC_lexicon_tags_13 },
    { 0x12ddf758, SUC_lexicon_tags_37 },
    { 0xab5df75a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefdf75d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969df763, SUC_lexicon_tags_0 },
    { 0x7e3df764, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x27ddf766, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfdf769, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebddf774, SUC_lexicon_tags_27 },
    { 0x21ddf775, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ddf779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7df780, SUC_lexicon_tags_12 },
    { 0x611df780, SUC_lexicon_tags_27 },
    { 0x643df782, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa1df784, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x513df786, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7df789, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x337df78b, SUC_lexicon_tags_45 },
    { 0x7a1df78c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d1df78e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x48fdf790, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddf797, SUC_lexicon_tags_26 },
    { 0xabdf798, SUC_lexicon_tags_25 },
    { 0x85fdf799, SUC_lexicon_tags_28 },
    { 0xddbdf799, SUC_lexicon_tags_25 },
    { 0xd05df797, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49df79e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3df7a1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x94bdf7a3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x527df7a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x551df7a8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xa3df7aa, SUC_lexicon_tags_15 },
    { 0x9efdf7aa, SUC_lexicon_tags_14 },
    { 0xad1df7ac, SUC_lexicon_tags_101 },
    { 0x56bdf7aa, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9df7b2, SUC_lexicon_tags_38 },
    { 0x5e1df7b3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc39df7b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bdf7ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671df7be, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x799df7c1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x939df7c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81df7c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cddf7ca, SUC_lexicon_tags_12 },
    { 0x32bdf7cb, SUC_lexicon_tags_37 },
    { 0x681df7cb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d5df7ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5df7d5, SUC_lexicon_tags_23 },
    { 0x951df7d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x56bdf7d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627df7dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x221df7de, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489df7e4, SUC_lexicon_tags_11 },
    { 0x5f5df7e5, SUC_lexicon_tags_37 },
    { 0xedddf7e6, SUC_lexicon_tags_8 },
    { 0xc81df7e5, SUC_lexicon_tags_23 },
    { 0x7e3df7e5, SUC_lexicon_tags_10 },
    { 0xf73df7e9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdf7ee, SUC_lexicon_tags_4 },
    { 0x84bdf7ef, SUC_lexicon_tags_7 },
    { 0xab5df7f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcddf7f2, SUC_lexicon_tags_2 },
    { 0xfabdf7f2, SUC_lexicon_tags_15 },
    { 0x147df7f4, SUC_lexicon_tags_63 },
    { 0x865df7f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8bdf7f7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7df7fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9df7fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x949df800, SUC_lexicon_tags_68 },
    { 0x8a1df801, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9df806, SUC_lexicon_tags_17 },
    { 0x81bdf806, SUC_lexicon_tags_29 },
    { 0x523df807, SUC_lexicon_tags_11 },
    { 0x341df809, SUC_lexicon_tags_32 },
    { 0x773df80a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac3df80c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65df811, SUC_lexicon_tags_12 },
    { 0x339df811, SUC_lexicon_tags_37 },
    { 0xf49df813, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdf81c, SUC_lexicon_tags_33 },
    { 0x9bddf81d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541df821, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x1d9df823, SUC_lexicon_tags_8 },
    { 0xe23df824, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1df828, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63df82d, SUC_lexicon_tags_46 },
    { 0xfd1df82e, SUC_lexicon_tags_47 },
    { 0x987df82d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x811df831, SUC_lexicon_tags_70 },
    { 0xb6fdf832, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bdf836, SUC_lexicon_tags_9 },
    { 0x8ffdf837, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ddf83b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ddf83f, SUC_lexicon_tags_55 },
    { 0x953df840, SUC_lexicon_tags_38 },
    { 0x161df840, SUC_lexicon_tags_17 },
    { 0xdf7df842, SUC_lexicon_tags_33 },
    { 0xd89df843, SUC_lexicon_tags_12 },
    { 0x2ebdf843, SUC_lexicon_tags_14 },
    { 0xa19df842, SUC_lexicon_tags_33 },
    { 0xcf9df845, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbdf849, SUC_lexicon_tags_25 },
    { 0xd75df84a, SUC_lexicon_tags_47 },
    { 0xe15df84b, SUC_lexicon_tags_1 },
    { 0x445df84c, SUC_lexicon_tags_11 },
    { 0xadfdf84c, SUC_lexicon_tags_65 },
    { 0x723df84e, SUC_lexicon_tags_14 },
    { 0x1bfdf84d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bdf853, SUC_lexicon_tags_66 },
    { 0x941df854, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2a7df856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69df85b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bdf85f, SUC_lexicon_tags_28 },
    { 0xddfdf860, SUC_lexicon_tags_12 },
    { 0xcc9df860, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb7df863, SUC_lexicon_tags_45 },
    { 0xb9df863, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ddf869, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fdf86c, SUC_lexicon_tags_15 },
    { 0xd95df86d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713df872, SUC_lexicon_tags_25 },
    { 0x99bdf873, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdf876, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1df879, SUC_lexicon_tags_168 },
    { 0xbebdf87a, SUC_lexicon_tags_28 },
    { 0x60ddf87b, SUC_lexicon_tags_84 },
    { 0xc8ddf87c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869df887, SUC_lexicon_tags_11 },
    { 0xe43df887, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69df88b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc97df88d, SUC_lexicon_tags_37 },
    { 0xb75df88e, SUC_lexicon_tags_15 },
    { 0xafbdf88d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ddf894, SUC_lexicon_tags_47 },
    { 0x461df895, SUC_lexicon_tags_9 },
    { 0xc11df896, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd71df898, SUC_lexicon_tags_12 },
    { 0x35df899, SUC_lexicon_tags_2 },
    { 0xfc7df89a, SUC_lexicon_tags_13 },
    { 0x65fdf89b, SUC_lexicon_tags_37 },
    { 0x1d3df89a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x66fdf89f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73df8a4, SUC_lexicon_tags_40 },
    { 0x4e9df8a5, SUC_lexicon_tags_37 },
    { 0x51df8a5, SUC_lexicon_tags_84 },
    { 0x5c3df8a7, SUC_lexicon_tags_17 },
    { 0x2ddf8a8, SUC_lexicon_tags_31 },
    { 0xb89df8a7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x35df8ab, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fdf8b2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59df8b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4ddf8b9, SUC_lexicon_tags_19 },
    { 0xa59df8ba, SUC_lexicon_tags_65 },
    { 0x1abdf8b9, SUC_lexicon_tags_25 },
    { 0x603df8ba, SUC_lexicon_tags_63 },
    { 0xc3df8bd, SUC_lexicon_tags_17 },
    { 0xf47df8ba, SUC_lexicon_tags_1 },
    { 0x1bbdf8b9, SUC_lexicon_tags_39 },
    { 0x609df8c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x671df8c2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xdaddf8c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ddf8c7, SUC_lexicon_tags_6 },
    { 0x703df8c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323df8d1, SUC_lexicon_tags_41 },
    { 0x2abdf8d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19df8d6, SUC_lexicon_tags_74 },
    { 0x9ffdf8d7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3df8dd, SUC_lexicon_tags_15 },
    { 0x44bdf8de, SUC_lexicon_tags_1 },
    { 0x983df8df, SUC_lexicon_tags_60 },
    { 0x20bdf8e0, SUC_lexicon_tags_19 },
    { 0x295df8e0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9a1df8e3, SUC_lexicon_tags_43 },
    { 0x8d7df8e4, SUC_lexicon_tags_47 },
    { 0xf37df8e4, SUC_lexicon_tags_77 },
    { 0x107df8e3, SUC_lexicon_tags_7 },
    { 0xdf1df8e6, SUC_lexicon_tags_57 },
    { 0xeb1df8e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591df8ed, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x71bdf8ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fdf8f3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x277df8f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29df8ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f7df901, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x727df904, SUC_lexicon_tags_12 },
    { 0x881df905, SUC_lexicon_tags_45 },
    { 0xaefdf906, SUC_lexicon_tags_49 },
    { 0xb1ddf906, SUC_lexicon_tags_2 },
    { 0xbe3df907, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x725df90a, SUC_lexicon_tags_7 },
    { 0x7b9df90b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x607df90e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5cbdf910, SUC_lexicon_tags_37 },
    { 0x897df911, SUC_lexicon_tags_15 },
    { 0x8ebdf912, SUC_lexicon_tags_56 },
    { 0xac3df913, SUC_lexicon_tags_15 },
    { 0x909df911, SUC_lexicon_tags_87 },
    { 0x531df913, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf33df917, SUC_lexicon_tags_37 },
    { 0x82fdf918, SUC_lexicon_tags_6 },
    { 0x343df918, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bdf920, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5df923, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcb9df925, SUC_lexicon_tags_17 },
    { 0x18fdf926, SUC_lexicon_tags_17 },
    { 0xabfdf926, SUC_lexicon_tags_3 },
    { 0x617df925, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd99df92a, SUC_lexicon_tags_12 },
    { 0xb59df92a, SUC_lexicon_tags_19 },
    { 0xf63df92a, SUC_lexicon_tags_23 },
    { 0xf9bdf92b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cddf930, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4d1df932, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339df93c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565df948, SUC_lexicon_tags_23 },
    { 0x6cfdf949, SUC_lexicon_tags_1 },
    { 0xa6ddf94a, SUC_lexicon_tags_12 },
    { 0x5fdf949, SUC_lexicon_tags_19 },
    { 0x757df94a, SUC_lexicon_tags_14 },
    { 0xefddf94a, SUC_lexicon_tags_280 },
    { 0, NULL },
    { 0, NULL },
    { 0x137df950, SUC_lexicon_tags_37 },
    { 0x677df950, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf09df953, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ddf957, SUC_lexicon_tags_12 },
    { 0xa09df958, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbdf95b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23df95f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1df963, SUC_lexicon_tags_2 },
    { 0xfcfdf963, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc3fdf966, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xac7df968, SUC_lexicon_tags_11 },
    { 0xc2bdf969, SUC_lexicon_tags_23 },
    { 0xa43df96a, SUC_lexicon_tags_43 },
    { 0x8b1df96a, SUC_lexicon_tags_45 },
    { 0xdfbdf96c, SUC_lexicon_tags_45 },
    { 0x1efdf96b, SUC_lexicon_tags_29 },
    { 0xd81df96e, SUC_lexicon_tags_17 },
    { 0x8b1df969, SUC_lexicon_tags_25 },
    { 0xf0bdf96c, SUC_lexicon_tags_46 },
    { 0x905df971, SUC_lexicon_tags_33 },
    { 0xce9df971, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdf978, SUC_lexicon_tags_27 },
    { 0x43fdf979, SUC_lexicon_tags_1 },
    { 0xa3ddf978, SUC_lexicon_tags_30 },
    { 0x8c7df97b, SUC_lexicon_tags_17 },
    { 0x19df97c, SUC_lexicon_tags_7 },
    { 0x7b3df97b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bdf981, SUC_lexicon_tags_37 },
    { 0x7c7df981, SUC_lexicon_tags_37 },
    { 0x155df981, SUC_lexicon_tags_49 },
    { 0xc7df983, SUC_lexicon_tags_31 },
    { 0x215df985, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0xa15df987, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9df98b, SUC_lexicon_tags_0 },
    { 0x815df98c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x79fdf98e, SUC_lexicon_tags_25 },
    { 0xbfbdf98f, SUC_lexicon_tags_10 },
    { 0xce3df990, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7df995, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61fdf997, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431df99b, SUC_lexicon_tags_19 },
    { 0xc99df99b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbcfdf99e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x539df9a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53df9a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x259df9a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27df9ac, SUC_lexicon_tags_11 },
    { 0x501df9ad, SUC_lexicon_tags_26 },
    { 0x4abdf9ac, SUC_lexicon_tags_1 },
    { 0x44ddf9af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a9df9b1, SUC_lexicon_tags_85 },
    { 0xad3df9b2, SUC_lexicon_tags_37 },
    { 0x89df9b3, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47df9b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57df9b9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd85df9bb, SUC_lexicon_tags_14 },
    { 0xf41df9bc, SUC_lexicon_tags_37 },
    { 0x469df9bd, SUC_lexicon_tags_12 },
    { 0xcb1df9bd, SUC_lexicon_tags_25 },
    { 0x2fdf9be, SUC_lexicon_tags_119 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9df9c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x541df9c4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1df9c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141df9cd, SUC_lexicon_tags_2 },
    { 0x4e3df9ce, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751df9d3, SUC_lexicon_tags_11 },
    { 0x353df9d3, SUC_lexicon_tags_37 },
    { 0x9ddf9d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x609df9d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bdf9da, SUC_lexicon_tags_35 },
    { 0xf33df9da, SUC_lexicon_tags_2 },
    { 0x60bdf9db, SUC_lexicon_tags_47 },
    { 0x29bdf9dd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8d5df9df, SUC_lexicon_tags_3 },
    { 0xebdf9e0, SUC_lexicon_tags_11 },
    { 0x757df9e1, SUC_lexicon_tags_19 },
    { 0xcb7df9e1, SUC_lexicon_tags_8 },
    { 0x641df9e0, SUC_lexicon_tags_15 },
    { 0x6b3df9e4, SUC_lexicon_tags_37 },
    { 0x865df9e4, SUC_lexicon_tags_31 },
    { 0xb5ddf9e6, SUC_lexicon_tags_38 },
    { 0x5fbdf9e3, SUC_lexicon_tags_25 },
    { 0x4eddf9df, SUC_lexicon_tags_48 },
    { 0x177df9e9, SUC_lexicon_tags_7 },
    { 0xe3df9ea, SUC_lexicon_tags_14 },
    { 0xebfdf9eb, SUC_lexicon_tags_37 },
    { 0x295df9ec, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xffdf9ee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35df9f4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x125df9f6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7df9fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x313df9fc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x433df9ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac1dfa01, SUC_lexicon_tags_38 },
    { 0xfe9dfa02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x433dfa04, SUC_lexicon_tags_21 },
    { 0xa8ddfa05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55dfa0f, SUC_lexicon_tags_45 },
    { 0xf79dfa10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaebdfa12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ddfa18, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xe47dfa1a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbdfa1d, SUC_lexicon_tags_17 },
    { 0xab9dfa1e, SUC_lexicon_tags_37 },
    { 0xa7dfa1f, SUC_lexicon_tags_14 },
    { 0x41ddfa20, SUC_lexicon_tags_29 },
    { 0x89ddfa1e, SUC_lexicon_tags_14 },
    { 0x69ddfa1f, SUC_lexicon_tags_25 },
    { 0xc79dfa1e, SUC_lexicon_tags_0 },
    { 0x8d1dfa1f, SUC_lexicon_tags_10 },
    { 0x5ffdfa25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x513dfa27, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efdfa2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67dfa2f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9dfa33, SUC_lexicon_tags_45 },
    { 0x8b5dfa34, SUC_lexicon_tags_160 },
    { 0x9f5dfa34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6e1dfa37, SUC_lexicon_tags_11 },
    { 0x197dfa38, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39dfa3d, SUC_lexicon_tags_28 },
    { 0x6b1dfa3e, SUC_lexicon_tags_53 },
    { 0x2cbdfa3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bdfa43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7dfa48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49dfa4e, SUC_lexicon_tags_0 },
    { 0x1efdfa4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1dfa53, SUC_lexicon_tags_49 },
    { 0x743dfa54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bdfa57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfddfa5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x61fdfa60, SUC_lexicon_tags_61 },
    { 0x889dfa61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3bdfa63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fdfa68, SUC_lexicon_tags_1245 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607dfa6f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9dfa74, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1dfa7b, SUC_lexicon_tags_15 },
    { 0xccddfa7c, SUC_lexicon_tags_11 },
    { 0x4e9dfa7c, SUC_lexicon_tags_19 },
    { 0x2e9dfa7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fdfa82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cddfa86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3bddfa88, SUC_lexicon_tags_7 },
    { 0xb7ddfa88, SUC_lexicon_tags_12 },
    { 0x10ddfa89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x775dfa8c, SUC_lexicon_tags_27 },
    { 0x37fdfa8d, SUC_lexicon_tags_37 },
    { 0x46bdfa8e, SUC_lexicon_tags_31 },
    { 0x1cfdfa8e, SUC_lexicon_tags_19 },
    { 0x27bdfa90, SUC_lexicon_tags_37 },
    { 0x6ebdfa91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd39dfa93, SUC_lexicon_tags_25 },
    { 0x841dfa94, SUC_lexicon_tags_37 },
    { 0xc0bdfa93, SUC_lexicon_tags_17 },
    { 0x9b5dfa95, SUC_lexicon_tags_17 },
    { 0x559dfa97, SUC_lexicon_tags_12 },
    { 0x335dfa98, SUC_lexicon_tags_17 },
    { 0xea7dfa99, SUC_lexicon_tags_45 },
    { 0x6d3dfa9a, SUC_lexicon_tags_46 },
    { 0xdf5dfa9a, SUC_lexicon_tags_43 },
    { 0x591dfa99, SUC_lexicon_tags_11 },
    { 0x3e9dfa95, SUC_lexicon_tags_12 },
    { 0x477dfa9e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727dfaa4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2efdfaab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9dfaae, SUC_lexicon_tags_106 },
    { 0x815dfaaf, SUC_lexicon_tags_7 },
    { 0xbb3dfab0, SUC_lexicon_tags_7 },
    { 0x233dfab1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7dfab5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7dfabb, SUC_lexicon_tags_1408 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35dfac1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19dfac4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbabdfac6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x29dfac8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7dfacc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bdfad2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x72bdfad4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9a9dfad6, SUC_lexicon_tags_55 },
    { 0x74ddfad7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7dfadb, SUC_lexicon_tags_30 },
    { 0x55dfadc, SUC_lexicon_tags_31 },
    { 0xd93dfadc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc67dfadf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x525dfae1, SUC_lexicon_tags_11 },
    { 0xef1dfae1, SUC_lexicon_tags_26 },
    { 0xd87dfae1, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851dfae8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x155dfaea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5dfaef, SUC_lexicon_tags_12 },
    { 0xe3fdfaef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81bdfaf5, SUC_lexicon_tags_2 },
    { 0x509dfaf5, SUC_lexicon_tags_12 },
    { 0xefbdfaf6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63dfafb, SUC_lexicon_tags_38 },
    { 0xbdfafc, SUC_lexicon_tags_25 },
    { 0xe29dfafb, SUC_lexicon_tags_11 },
    { 0x945dfafe, SUC_lexicon_tags_47 },
    { 0xbdfaff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbdfb04, SUC_lexicon_tags_25 },
    { 0x991dfb04, SUC_lexicon_tags_25 },
    { 0xfc5dfb06, SUC_lexicon_tags_11 },
    { 0x5e5dfb05, SUC_lexicon_tags_43 },
    { 0x6ebdfb08, SUC_lexicon_tags_17 },
    { 0xe29dfb09, SUC_lexicon_tags_60 },
    { 0xd57dfb0a, SUC_lexicon_tags_38 },
    { 0xef3dfb0a, SUC_lexicon_tags_27 },
    { 0x505dfb0b, SUC_lexicon_tags_17 },
    { 0x443dfb0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68bdfb0f, SUC_lexicon_tags_33 },
    { 0xd75dfb0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5dfb13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x895dfb15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465dfb1b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x57bdfb1d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ddfb20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b3dfb22, SUC_lexicon_tags_38 },
    { 0x25bdfb23, SUC_lexicon_tags_12 },
    { 0xbefdfb23, SUC_lexicon_tags_68 },
    { 0xc7bdfb25, SUC_lexicon_tags_7 },
    { 0xb55dfb26, SUC_lexicon_tags_1 },
    { 0xa21dfb22, SUC_lexicon_tags_13 },
    { 0xcb3dfb28, SUC_lexicon_tags_63 },
    { 0x1ffdfb28, SUC_lexicon_tags_65 },
    { 0xc83dfb2a, SUC_lexicon_tags_2 },
    { 0x159dfb25, SUC_lexicon_tags_15 },
    { 0xc35dfb2c, SUC_lexicon_tags_27 },
    { 0x685dfb2c, SUC_lexicon_tags_32 },
    { 0xc7dfb2e, SUC_lexicon_tags_130 },
    { 0xb1dfb2d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed7dfb31, SUC_lexicon_tags_16 },
    { 0x199dfb32, SUC_lexicon_tags_55 },
    { 0x14fdfb33, SUC_lexicon_tags_17 },
    { 0x2c7dfb34, SUC_lexicon_tags_1 },
    { 0x87bdfb32, SUC_lexicon_tags_204 },
    { 0x8f7dfb36, SUC_lexicon_tags_17 },
    { 0x583dfb37, SUC_lexicon_tags_23 },
    { 0xa85dfb32, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefdfb3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3dfb3f, SUC_lexicon_tags_2 },
    { 0x889dfb3f, SUC_lexicon_tags_7 },
    { 0xf61dfb41, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969dfb48, SUC_lexicon_tags_30 },
    { 0xafbdfb49, SUC_lexicon_tags_1 },
    { 0xb5ddfb48, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefdfb4f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ddfb55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9dfb59, SUC_lexicon_tags_17 },
    { 0xd83dfb5a, SUC_lexicon_tags_17 },
    { 0xbbddfb5b, SUC_lexicon_tags_28 },
    { 0xcfdfb5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68bdfb5e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99dfb60, SUC_lexicon_tags_38 },
    { 0x9cbdfb61, SUC_lexicon_tags_9 },
    { 0x1f9dfb62, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe43dfb64, SUC_lexicon_tags_11 },
    { 0xa7ddfb65, SUC_lexicon_tags_11 },
    { 0xdefdfb64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6d7dfb68, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fdfb6b, SUC_lexicon_tags_87 },
    { 0xd7ddfb6c, SUC_lexicon_tags_14 },
    { 0x751dfb6d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x159dfb70, SUC_lexicon_tags_55 },
    { 0x815dfb70, SUC_lexicon_tags_375 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5dfb79, SUC_lexicon_tags_25 },
    { 0x32bdfb7a, SUC_lexicon_tags_0 },
    { 0x8d9dfb7a, SUC_lexicon_tags_1 },
    { 0xd5bdfb7c, SUC_lexicon_tags_2 },
    { 0x889dfb7d, SUC_lexicon_tags_66 },
    { 0x907dfb7e, SUC_lexicon_tags_37 },
    { 0xabbdfb7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31dfb83, SUC_lexicon_tags_11 },
    { 0x7e5dfb84, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701dfb8a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fdfb8f, SUC_lexicon_tags_2 },
    { 0x8b9dfb90, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb05dfb92, SUC_lexicon_tags_0 },
    { 0xf9bdfb93, SUC_lexicon_tags_1 },
    { 0x52bdfb92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461dfb9d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbdfba1, SUC_lexicon_tags_46 },
    { 0x231dfba2, SUC_lexicon_tags_37 },
    { 0xfabdfba3, SUC_lexicon_tags_2 },
    { 0x66bdfba4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffddfba8, SUC_lexicon_tags_15 },
    { 0x9d3dfba9, SUC_lexicon_tags_17 },
    { 0xaa7dfbaa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3dfbad, SUC_lexicon_tags_7 },
    { 0xd41dfbae, SUC_lexicon_tags_14 },
    { 0xcafdfbaf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x209dfbb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x785dfbb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dfbb7, SUC_lexicon_tags_28 },
    { 0x9c7dfbb7, SUC_lexicon_tags_37 },
    { 0x891dfbb9, SUC_lexicon_tags_49 },
    { 0xed5dfbba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fdfbbf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981dfbc5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbdfbcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa4fdfbcd, SUC_lexicon_tags_38 },
    { 0x40bdfbce, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd49dfbd0, SUC_lexicon_tags_37 },
    { 0x799dfbd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5dfbd8, SUC_lexicon_tags_9 },
    { 0x32bdfbd9, SUC_lexicon_tags_8 },
    { 0x66fdfbda, SUC_lexicon_tags_23 },
    { 0xa5ddfbdb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddfbe5, SUC_lexicon_tags_12 },
    { 0x157dfbe6, SUC_lexicon_tags_25 },
    { 0xa79dfbe7, SUC_lexicon_tags_49 },
    { 0x59dfbe6, SUC_lexicon_tags_1 },
    { 0x725dfbe9, SUC_lexicon_tags_8 },
    { 0x851dfbea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dfbed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x755dfbef, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x9eddfbf1, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xf4bdfbf3, SUC_lexicon_tags_1 },
    { 0x341dfbf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x167dfbf6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcbdfbf8, SUC_lexicon_tags_33 },
    { 0xabdfbf9, SUC_lexicon_tags_39 },
    { 0x4bdfbfa, SUC_lexicon_tags_11 },
    { 0xb77dfbf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ddfbfe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfc3dfc00, SUC_lexicon_tags_30 },
    { 0x97bdfc01, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3dfc05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9cfdfc07, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ddfc0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fdfc0e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fddfc19, SUC_lexicon_tags_31 },
    { 0xe81dfc19, SUC_lexicon_tags_2 },
    { 0xcefdfc1b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdf9dfc1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd1dfc1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x443dfc21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93dfc24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dfc27, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x543dfc29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7f1dfc2b, SUC_lexicon_tags_37 },
    { 0x36ddfc2c, SUC_lexicon_tags_27 },
    { 0xd81dfc2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709dfc34, SUC_lexicon_tags_43 },
    { 0x5c5dfc35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbdfc39, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x88bdfc3b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe41dfc3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61dfc42, SUC_lexicon_tags_25 },
    { 0x69dfc42, SUC_lexicon_tags_0 },
    { 0xee7dfc44, SUC_lexicon_tags_17 },
    { 0x447dfc45, SUC_lexicon_tags_27 },
    { 0x7fddfc46, SUC_lexicon_tags_25 },
    { 0x1f5dfc47, SUC_lexicon_tags_11 },
    { 0x571dfc48, SUC_lexicon_tags_25 },
    { 0xf7bdfc48, SUC_lexicon_tags_2 },
    { 0xcffdfc42, SUC_lexicon_tags_25 },
    { 0x4f1dfc4b, SUC_lexicon_tags_48 },
    { 0x74bdfc4c, SUC_lexicon_tags_19 },
    { 0x7a1dfc4d, SUC_lexicon_tags_16 },
    { 0xb01dfc42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4ddfc50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe05dfc53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x323dfc56, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ddfc59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fdfc5d, SUC_lexicon_tags_37 },
    { 0xa91dfc5e, SUC_lexicon_tags_25 },
    { 0xbc5dfc5e, SUC_lexicon_tags_37 },
    { 0x121dfc5e, SUC_lexicon_tags_14 },
    { 0xa55dfc61, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19dfc70, SUC_lexicon_tags_8 },
    { 0xa43dfc71, SUC_lexicon_tags_25 },
    { 0xce7dfc72, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x851dfc74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xce1dfc76, SUC_lexicon_tags_25 },
    { 0xf93dfc76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09dfc7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeffdfc7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1dfc84, SUC_lexicon_tags_14 },
    { 0x5ddfc84, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7dfc88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfdfc8c, SUC_lexicon_tags_55 },
    { 0x4e3dfc8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ffdfc8f, SUC_lexicon_tags_29 },
    { 0x377dfc8f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xceddfc92, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x811dfc95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bdfc99, SUC_lexicon_tags_70 },
    { 0x5bfdfc9a, SUC_lexicon_tags_45 },
    { 0xecbdfc9a, SUC_lexicon_tags_25 },
    { 0x4e7dfc9c, SUC_lexicon_tags_63 },
    { 0x4d5dfc99, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ddfca0, SUC_lexicon_tags_15 },
    { 0x40fdfca1, SUC_lexicon_tags_23 },
    { 0xf9fdfca2, SUC_lexicon_tags_1 },
    { 0x7e9dfca1, SUC_lexicon_tags_5 },
    { 0x609dfca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xab3dfca6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9dfca9, SUC_lexicon_tags_14 },
    { 0xf19dfcaa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x91fdfcac, SUC_lexicon_tags_2 },
    { 0x7a9dfcad, SUC_lexicon_tags_61 },
    { 0x7ebdfcad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5dfcb1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaddfcb7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb35dfcb9, SUC_lexicon_tags_16 },
    { 0x7fdfcb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9dfcc0, SUC_lexicon_tags_29 },
    { 0x219dfcc1, SUC_lexicon_tags_14 },
    { 0x6a7dfcc2, SUC_lexicon_tags_15 },
    { 0xa47dfcc3, SUC_lexicon_tags_25 },
    { 0x4a7dfcc2, SUC_lexicon_tags_17 },
    { 0xe0bdfcc5, SUC_lexicon_tags_27 },
    { 0x2b5dfcc4, SUC_lexicon_tags_25 },
    { 0x32ddfcc7, SUC_lexicon_tags_47 },
    { 0x259dfcc8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ddfcd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15dfcd9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8eddfcdb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x5bddfcdd, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1dfce0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x701dfce3, SUC_lexicon_tags_19 },
    { 0x4a3dfce4, SUC_lexicon_tags_17 },
    { 0x1cfdfce5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x701dfce7, SUC_lexicon_tags_11 },
    { 0xcd7dfce7, SUC_lexicon_tags_25 },
    { 0x177dfce9, SUC_lexicon_tags_1 },
    { 0xd87dfcea, SUC_lexicon_tags_17 },
    { 0x589dfcea, SUC_lexicon_tags_2 },
    { 0x8b5dfcec, SUC_lexicon_tags_45 },
    { 0x24bdfcea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdfcf0, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bddfcf5, SUC_lexicon_tags_4 },
    { 0x17bdfcf6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b9dfcf8, SUC_lexicon_tags_8 },
    { 0x2a5dfcf8, SUC_lexicon_tags_17 },
    { 0x523dfcfa, SUC_lexicon_tags_17 },
    { 0x489dfcf8, SUC_lexicon_tags_87 },
    { 0xe4ddfcfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa45dfcfe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x80bdfd00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101dfd05, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x625dfd08, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17dfd0e, SUC_lexicon_tags_15 },
    { 0x423dfd0e, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0x339dfd11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd75dfd13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95dfd18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfdfd1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7dfd20, SUC_lexicon_tags_0 },
    { 0x4afdfd21, SUC_lexicon_tags_25 },
    { 0xbddfd22, SUC_lexicon_tags_47 },
    { 0xd05dfd23, SUC_lexicon_tags_1 },
    { 0x7d7dfd23, SUC_lexicon_tags_85 },
    { 0xb9dfd25, SUC_lexicon_tags_37 },
    { 0xce1dfd26, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77dfd2d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x199dfd2f, SUC_lexicon_tags_14 },
    { 0x519dfd30, SUC_lexicon_tags_17 },
    { 0x17ddfd31, SUC_lexicon_tags_92 },
    { 0x643dfd2f, SUC_lexicon_tags_33 },
    { 0x555dfd2f, SUC_lexicon_tags_37 },
    { 0xb09dfd33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x747dfd37, SUC_lexicon_tags_57 },
    { 0xc83dfd38, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xf8fdfd3a, SUC_lexicon_tags_25 },
    { 0xb55dfd3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5dfd40, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1dfd46, SUC_lexicon_tags_9 },
    { 0x869dfd46, SUC_lexicon_tags_66 },
    { 0x9c1dfd48, SUC_lexicon_tags_8 },
    { 0xb43dfd48, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fdfd4c, SUC_lexicon_tags_15 },
    { 0x71fdfd4d, SUC_lexicon_tags_68 },
    { 0x5dfdfd4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe0bdfd50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7dfd53, SUC_lexicon_tags_17 },
    { 0xd1fdfd54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705dfd5a, SUC_lexicon_tags_49 },
    { 0x4d7dfd5b, SUC_lexicon_tags_0 },
    { 0x601dfd5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x417dfd5f, SUC_lexicon_tags_30 },
    { 0xc55dfd60, SUC_lexicon_tags_45 },
    { 0x891dfd60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1dfd66, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8f7dfd68, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505dfd6c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9dfd71, SUC_lexicon_tags_15 },
    { 0x7e7dfd72, SUC_lexicon_tags_21 },
    { 0x8eddfd73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf91dfd75, SUC_lexicon_tags_214 },
    { 0, NULL },
    { 0x5dddfd77, SUC_lexicon_tags_37 },
    { 0x813dfd78, SUC_lexicon_tags_37 },
    { 0x153dfd78, SUC_lexicon_tags_58 },
    { 0xc35dfd79, SUC_lexicon_tags_11 },
    { 0xa7bdfd7b, SUC_lexicon_tags_46 },
    { 0xe0bdfd7c, SUC_lexicon_tags_82 },
    { 0xe09dfd7d, SUC_lexicon_tags_2 },
    { 0x25fdfd78, SUC_lexicon_tags_17 },
    { 0x91fdfd7f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x76bdfd81, SUC_lexicon_tags_69 },
    { 0xbd9dfd81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15dfd86, SUC_lexicon_tags_29 },
    { 0xb05dfd87, SUC_lexicon_tags_37 },
    { 0xe0ddfd88, SUC_lexicon_tags_9 },
    { 0x785dfd89, SUC_lexicon_tags_12 },
    { 0x83ddfd8a, SUC_lexicon_tags_5 },
    { 0x8abdfd88, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7dfd90, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfdfd9b, SUC_lexicon_tags_91 },
    { 0x8c5dfd9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c7dfd9e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597dfda7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xafdfda9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9dfdb2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xafddfdb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b3dfdb7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5dfdba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccddfdc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dfdc6, SUC_lexicon_tags_59 },
    { 0xd81dfdc7, SUC_lexicon_tags_37 },
    { 0x75ddfdc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x541dfdca, SUC_lexicon_tags_2 },
    { 0xc03dfdca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9dfdd0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x33dfdd2, SUC_lexicon_tags_1 },
    { 0x1addfdd3, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x5f1dfdd5, SUC_lexicon_tags_0 },
    { 0x82ddfdd6, SUC_lexicon_tags_7 },
    { 0xee7dfdd7, SUC_lexicon_tags_31 },
    { 0xf5fdfdd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x445dfdda, SUC_lexicon_tags_1 },
    { 0xeeddfdda, SUC_lexicon_tags_37 },
    { 0x88bdfddc, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bdfddf, SUC_lexicon_tags_49 },
    { 0x65dfde0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dfde6, SUC_lexicon_tags_79 },
    { 0xfe1dfde7, SUC_lexicon_tags_80 },
    { 0xf9ddfde8, SUC_lexicon_tags_11 },
    { 0xb93dfde8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67dfded, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ddfdf3, SUC_lexicon_tags_29 },
    { 0xf25dfdf4, SUC_lexicon_tags_31 },
    { 0xdefdfdf3, SUC_lexicon_tags_37 },
    { 0xc21dfdf6, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xed5dfdf8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6bdfdfa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebdfdfd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1dfe00, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc1ddfe02, SUC_lexicon_tags_11 },
    { 0x101dfe03, SUC_lexicon_tags_49 },
    { 0xd35dfe04, SUC_lexicon_tags_37 },
    { 0x755dfe05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x921dfe08, SUC_lexicon_tags_15 },
    { 0x12ddfe08, SUC_lexicon_tags_28 },
    { 0xbe1dfe0a, SUC_lexicon_tags_0 },
    { 0x177dfe0a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x275dfe0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eddfe13, SUC_lexicon_tags_27 },
    { 0x863dfe13, SUC_lexicon_tags_37 },
    { 0x2d1dfe14, SUC_lexicon_tags_10 },
    { 0x4ddfe13, SUC_lexicon_tags_2 },
    { 0xc17dfe17, SUC_lexicon_tags_7 },
    { 0x571dfe17, SUC_lexicon_tags_11 },
    { 0x7b1dfe17, SUC_lexicon_tags_65 },
    { 0xc39dfe13, SUC_lexicon_tags_53 },
    { 0x8e1dfe1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81dfe21, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddfe25, SUC_lexicon_tags_49 },
    { 0xeb3dfe25, SUC_lexicon_tags_7 },
    { 0xce3dfe27, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bdfe2a, SUC_lexicon_tags_8 },
    { 0x331dfe2a, SUC_lexicon_tags_0 },
    { 0xf21dfe2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7b1dfe2e, SUC_lexicon_tags_17 },
    { 0x6abdfe2f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x757dfe31, SUC_lexicon_tags_37 },
    { 0xe35dfe31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173dfe37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x581dfe3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7dfe3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebdfe3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x847dfe41, SUC_lexicon_tags_146 },
    { 0x12ddfe41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1bddfe44, SUC_lexicon_tags_7 },
    { 0x5c3dfe45, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfdfe4a, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0xe89dfe4d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7dfe53, SUC_lexicon_tags_46 },
    { 0x95bdfe53, SUC_lexicon_tags_7 },
    { 0xa21dfe55, SUC_lexicon_tags_12 },
    { 0x317dfe54, SUC_lexicon_tags_43 },
    { 0x637dfe56, SUC_lexicon_tags_63 },
    { 0xa5ddfe55, SUC_lexicon_tags_45 },
    { 0xe53dfe59, SUC_lexicon_tags_12 },
    { 0x885dfe5a, SUC_lexicon_tags_33 },
    { 0x733dfe5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3addfe5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751dfe65, SUC_lexicon_tags_2 },
    { 0x121dfe66, SUC_lexicon_tags_30 },
    { 0xb25dfe67, SUC_lexicon_tags_29 },
    { 0x643dfe65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827dfe72, SUC_lexicon_tags_63 },
    { 0xcd7dfe73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x835dfe75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffdfe79, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fdfe7d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ddfe81, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2a7dfe83, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07dfe8b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdfe8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bdfe98, SUC_lexicon_tags_2 },
    { 0xb9dfe99, SUC_lexicon_tags_37 },
    { 0xf39dfe9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541dfe9f, SUC_lexicon_tags_17 },
    { 0x9d1dfea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe49dfea2, SUC_lexicon_tags_23 },
    { 0x1e1dfea2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2dddfea5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91dfea7, SUC_lexicon_tags_11 },
    { 0xab5dfea8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3dbdfeaa, SUC_lexicon_tags_26 },
    { 0xc65dfeab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x763dfead, SUC_lexicon_tags_12 },
    { 0x3b1dfeae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1dfeb2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7dfebd, SUC_lexicon_tags_12 },
    { 0x59fdfebe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51dfec5, SUC_lexicon_tags_25 },
    { 0x3a3dfec6, SUC_lexicon_tags_38 },
    { 0x323dfec6, SUC_lexicon_tags_6 },
    { 0xda7dfec7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9dfeca, SUC_lexicon_tags_66 },
    { 0xf09dfeca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaddfed2, SUC_lexicon_tags_46 },
    { 0xfa3dfed3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3dfed8, SUC_lexicon_tags_2 },
    { 0x277dfed8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebdfedc, SUC_lexicon_tags_84 },
    { 0xd7bdfedd, SUC_lexicon_tags_14 },
    { 0x97dfedc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fdfee3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafdfee6, SUC_lexicon_tags_17 },
    { 0x655dfee7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89dfeeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe47dfeed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7dfef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5dfef6, SUC_lexicon_tags_37 },
    { 0x265dfef7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x195dfefa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x851dfefc, SUC_lexicon_tags_17 },
    { 0xc69dfefc, SUC_lexicon_tags_37 },
    { 0xf15dfefe, SUC_lexicon_tags_26 },
    { 0xc69dfefd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7dff02, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x639dff04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7ddff06, SUC_lexicon_tags_7 },
    { 0x703dff07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5dff11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271dff15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fdff1a, SUC_lexicon_tags_15 },
    { 0x3e1dff1a, SUC_lexicon_tags_21 },
    { 0x2fdff1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dddff1f, SUC_lexicon_tags_39 },
    { 0x735dff20, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359dff24, SUC_lexicon_tags_1523 },
    { 0x5fbdff25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83dff27, SUC_lexicon_tags_14 },
    { 0x3cddff27, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989dff2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31dff30, SUC_lexicon_tags_31 },
    { 0xfcddff31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd99dff33, SUC_lexicon_tags_45 },
    { 0xebddff33, SUC_lexicon_tags_295 },
    { 0x725dff35, SUC_lexicon_tags_7 },
    { 0x60ddff33, SUC_lexicon_tags_11 },
    { 0x28bdff37, SUC_lexicon_tags_23 },
    { 0x513dff37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe09dff3a, SUC_lexicon_tags_11 },
    { 0xaf7dff3b, SUC_lexicon_tags_7 },
    { 0xc4bdff3b, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95dff3f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47dff4a, SUC_lexicon_tags_2 },
    { 0xc07dff4b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997dff4f, SUC_lexicon_tags_33 },
    { 0xcdfdff50, SUC_lexicon_tags_48 },
    { 0xb89dff50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dff57, SUC_lexicon_tags_39 },
    { 0xcdfdff57, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x551dff5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809dff62, SUC_lexicon_tags_49 },
    { 0xbfddff63, SUC_lexicon_tags_11 },
    { 0xf5fdff64, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1dff69, SUC_lexicon_tags_6 },
    { 0x399dff69, SUC_lexicon_tags_15 },
    { 0xfe7dff6b, SUC_lexicon_tags_26 },
    { 0x94ddff6c, SUC_lexicon_tags_49 },
    { 0xbb3dff6d, SUC_lexicon_tags_37 },
    { 0x19bdff6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x71dff71, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5dff76, SUC_lexicon_tags_29 },
    { 0xfe9dff77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83dff7a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7eddff7c, SUC_lexicon_tags_45 },
    { 0xf63dff7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ddff83, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xda9dff85, SUC_lexicon_tags_11 },
    { 0xf93dff86, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967dff8f, SUC_lexicon_tags_11 },
    { 0x5a1dff90, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dff99, SUC_lexicon_tags_8 },
    { 0x569dff9a, SUC_lexicon_tags_6 },
    { 0xb3bdff9b, SUC_lexicon_tags_17 },
    { 0x965dff9c, SUC_lexicon_tags_25 },
    { 0x541dff9d, SUC_lexicon_tags_37 },
    { 0x91fdff99, SUC_lexicon_tags_33 },
    { 0x55ddff9f, SUC_lexicon_tags_37 },
    { 0xf5fdff9f, SUC_lexicon_tags_13 },
    { 0x5c1dffa1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5dffab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bdffb0, SUC_lexicon_tags_30 },
    { 0x907dffb0, SUC_lexicon_tags_34 },
    { 0xe37dffb2, SUC_lexicon_tags_15 },
    { 0xbd5dffb3, SUC_lexicon_tags_38 },
    { 0xfefdffb4, SUC_lexicon_tags_54 },
    { 0xf0bdffb4, SUC_lexicon_tags_37 },
    { 0xe1dffb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561dffc3, SUC_lexicon_tags_25 },
    { 0x5ddffc3, SUC_lexicon_tags_87 },
    { 0xd6fdffc3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbdffc9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91dffcf, SUC_lexicon_tags_25 },
    { 0x639dffd0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x529dffd2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afdffd7, SUC_lexicon_tags_30 },
    { 0x531dffd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dffdb, SUC_lexicon_tags_17 },
    { 0x9fbdffdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bdffdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21dffe2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfbdffe4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdffeb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3dffef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe75dfff1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9dfff9, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x725dfffc, SUC_lexicon_tags_12 },
    { 0xeebdfffd, SUC_lexicon_tags_136 },
    { 0x9addfffd, SUC_lexicon_tags_45 },
    { 0xc87dfffc, SUC_lexicon_tags_47 },
    { 0x591e0000, SUC_lexicon_tags_11 },
    { 0xe75e0001, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63e0004, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9e0009, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775e0010, SUC_lexicon_tags_31 },
    { 0x515e0011, SUC_lexicon_tags_38 },
    { 0xe6be0012, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x623e0015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeede0017, SUC_lexicon_tags_20 },
    { 0xf69e0018, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92de001c, SUC_lexicon_tags_38 },
    { 0x4f3e001c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5de0023, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41e0026, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3be0029, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x249e002b, SUC_lexicon_tags_12 },
    { 0xe27e002c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x33de002e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x559e0030, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fe0034, SUC_lexicon_tags_20 },
    { 0xf71e0035, SUC_lexicon_tags_29 },
    { 0xd39e0036, SUC_lexicon_tags_11 },
    { 0x9dde0035, SUC_lexicon_tags_14 },
    { 0x5d7e0037, SUC_lexicon_tags_43 },
    { 0x73be0039, SUC_lexicon_tags_25 },
    { 0x11fe003a, SUC_lexicon_tags_0 },
    { 0xc81e003b, SUC_lexicon_tags_1 },
    { 0xc21e003b, SUC_lexicon_tags_26 },
    { 0xc59e0036, SUC_lexicon_tags_58 },
    { 0x6c3e003e, SUC_lexicon_tags_15 },
    { 0x5d1e0039, SUC_lexicon_tags_0 },
    { 0x1c9e0040, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53de004a, SUC_lexicon_tags_21 },
    { 0x457e004b, SUC_lexicon_tags_10 },
    { 0xa71e004a, SUC_lexicon_tags_27 },
    { 0x9fbe004d, SUC_lexicon_tags_29 },
    { 0xea7e004e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x113e0050, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1e0053, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x931e0055, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e0058, SUC_lexicon_tags_12 },
    { 0xed9e0059, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7f7e005b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3d7e005d, SUC_lexicon_tags_37 },
    { 0x2b3e005e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de0063, SUC_lexicon_tags_43 },
    { 0xee1e0064, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x50be0067, SUC_lexicon_tags_25 },
    { 0x74be0067, SUC_lexicon_tags_37 },
    { 0xce5e0069, SUC_lexicon_tags_87 },
    { 0xa17e006a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29e006d, SUC_lexicon_tags_11 },
    { 0x377e006d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1fe0070, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5de0072, SUC_lexicon_tags_11 },
    { 0x345e0072, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x855e0075, SUC_lexicon_tags_25 },
    { 0x287e0075, SUC_lexicon_tags_33 },
    { 0x9ede0075, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x53be0079, SUC_lexicon_tags_60 },
    { 0x65fe007a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa4de007c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5e007f, SUC_lexicon_tags_15 },
    { 0xc0de007f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1e0083, SUC_lexicon_tags_293 },
    { 0x213e0084, SUC_lexicon_tags_7 },
    { 0xcdfe0085, SUC_lexicon_tags_37 },
    { 0xa93e0086, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1e0094, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x749e0097, SUC_lexicon_tags_0 },
    { 0x7fe0098, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfffe009a, SUC_lexicon_tags_25 },
    { 0x64fe009b, SUC_lexicon_tags_1 },
    { 0x647e009c, SUC_lexicon_tags_1 },
    { 0x405e009c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2cde009f, SUC_lexicon_tags_17 },
    { 0xfe009f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fe00a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x461e00a8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x9fbe00aa, SUC_lexicon_tags_82 },
    { 0x9f7e00ab, SUC_lexicon_tags_17 },
    { 0xd5be00ab, SUC_lexicon_tags_33 },
    { 0xa0de00ad, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf33e00af, SUC_lexicon_tags_11 },
    { 0xc81e00b0, SUC_lexicon_tags_8 },
    { 0xa15e00b0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e00bd, SUC_lexicon_tags_6 },
    { 0xcdbe00be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa13e00c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x76fe00c3, SUC_lexicon_tags_3 },
    { 0x51be00c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6afe00c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603e00d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203e00d9, SUC_lexicon_tags_19 },
    { 0xec3e00da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51e00de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873e00e4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x287e00e6, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0x791e00e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6de00f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b9e00f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x275e00f4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59e00f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81e00fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1e00fd, SUC_lexicon_tags_93 },
    { 0x6bfe00fe, SUC_lexicon_tags_6 },
    { 0x937e00ff, SUC_lexicon_tags_79 },
    { 0xf4de0100, SUC_lexicon_tags_37 },
    { 0x2fe0101, SUC_lexicon_tags_2 },
    { 0x173e0102, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x441e0105, SUC_lexicon_tags_25 },
    { 0x239e0105, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7b7e0108, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6e7e010a, SUC_lexicon_tags_14 },
    { 0x15de010b, SUC_lexicon_tags_1 },
    { 0xa87e010c, SUC_lexicon_tags_1 },
    { 0x805e010c, SUC_lexicon_tags_15 },
    { 0x4bfe010c, SUC_lexicon_tags_11 },
    { 0xb3fe010f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917e0113, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3e0117, SUC_lexicon_tags_7 },
    { 0x335e0118, SUC_lexicon_tags_10 },
    { 0xf23e0118, SUC_lexicon_tags_0 },
    { 0x6d1e0119, SUC_lexicon_tags_17 },
    { 0x25e0119, SUC_lexicon_tags_109 },
    { 0xcdde011b, SUC_lexicon_tags_47 },
    { 0x5e3e011d, SUC_lexicon_tags_17 },
    { 0x73e011b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87e0127, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x389e0129, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dfe012d, SUC_lexicon_tags_60 },
    { 0x465e012e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1a5e0130, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5e0134, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22de0139, SUC_lexicon_tags_55 },
    { 0xd57e013a, SUC_lexicon_tags_45 },
    { 0x50be013b, SUC_lexicon_tags_2 },
    { 0xddfe013c, SUC_lexicon_tags_30 },
    { 0xa6de0139, SUC_lexicon_tags_11 },
    { 0x12fe013e, SUC_lexicon_tags_2 },
    { 0x4fe013f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95e0143, SUC_lexicon_tags_45 },
    { 0x669e0144, SUC_lexicon_tags_31 },
    { 0xecde0143, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfe014c, SUC_lexicon_tags_17 },
    { 0x37e014d, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7e0155, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x60be0158, SUC_lexicon_tags_7 },
    { 0x303e0159, SUC_lexicon_tags_55 },
    { 0xa8be0158, SUC_lexicon_tags_12 },
    { 0x9fde015a, SUC_lexicon_tags_27 },
    { 0xffde015c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79de0160, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59e0163, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb31e0165, SUC_lexicon_tags_14 },
    { 0x101e0165, SUC_lexicon_tags_49 },
    { 0x98de0167, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805e016b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ebe016d, SUC_lexicon_tags_49 },
    { 0xa09e016e, SUC_lexicon_tags_7 },
    { 0x8fe016f, SUC_lexicon_tags_12 },
    { 0x59be016e, SUC_lexicon_tags_12 },
    { 0xc05e0171, SUC_lexicon_tags_14 },
    { 0xec1e0172, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa1e0174, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01e0177, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1e017c, SUC_lexicon_tags_23 },
    { 0x817e017d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93be0181, SUC_lexicon_tags_60 },
    { 0xaf7e0181, SUC_lexicon_tags_6 },
    { 0x159e0181, SUC_lexicon_tags_15 },
    { 0x127e0184, SUC_lexicon_tags_25 },
    { 0xf35e0185, SUC_lexicon_tags_19 },
    { 0x45e0186, SUC_lexicon_tags_201 },
    { 0x7b5e0184, SUC_lexicon_tags_7 },
    { 0x801e0188, SUC_lexicon_tags_336 },
    { 0xec7e0189, SUC_lexicon_tags_7 },
    { 0x9bfe0184, SUC_lexicon_tags_27 },
    { 0x86be0183, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x47e018d, SUC_lexicon_tags_15 },
    { 0x979e018e, SUC_lexicon_tags_12 },
    { 0xa37e018f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x253e0192, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9e0199, SUC_lexicon_tags_37 },
    { 0x51e019a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe019d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfbde019f, SUC_lexicon_tags_29 },
    { 0xb2be019f, SUC_lexicon_tags_118 },
    { 0x907e01a1, SUC_lexicon_tags_15 },
    { 0xb37e019f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3a7e01a4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe3de01a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe01ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7e01af, SUC_lexicon_tags_60 },
    { 0x8f3e01af, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1e5e01b2, SUC_lexicon_tags_15 },
    { 0x8f9e01b3, SUC_lexicon_tags_87 },
    { 0xda5e01b2, SUC_lexicon_tags_25 },
    { 0xe43e01b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40fe01b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83e01ba, SUC_lexicon_tags_1 },
    { 0xe89e01bb, SUC_lexicon_tags_1 },
    { 0xed9e01bc, SUC_lexicon_tags_32 },
    { 0xa11e01bd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x105e01bf, SUC_lexicon_tags_15 },
    { 0x2dbe01c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65e01c3, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9e01c6, SUC_lexicon_tags_40 },
    { 0xd45e01c7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x419e01c9, SUC_lexicon_tags_9 },
    { 0x6afe01c9, SUC_lexicon_tags_25 },
    { 0xb4fe01cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78be01d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe01e01d7, SUC_lexicon_tags_7 },
    { 0x3b9e01d7, SUC_lexicon_tags_6 },
    { 0x377e01d8, SUC_lexicon_tags_33 },
    { 0x3b7e01da, SUC_lexicon_tags_45 },
    { 0xcf9e01d9, SUC_lexicon_tags_28 },
    { 0x637e01d9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe43e01de, SUC_lexicon_tags_0 },
    { 0xc89e01df, SUC_lexicon_tags_1 },
    { 0x527e01e0, SUC_lexicon_tags_112 },
    { 0x2e7e01e1, SUC_lexicon_tags_10 },
    { 0x263e01e0, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0x21e01e4, SUC_lexicon_tags_33 },
    { 0x953e01e5, SUC_lexicon_tags_13 },
    { 0x6cfe01e6, SUC_lexicon_tags_8 },
    { 0x70be01e6, SUC_lexicon_tags_12 },
    { 0x8d7e01e8, SUC_lexicon_tags_49 },
    { 0xc87e01e8, SUC_lexicon_tags_33 },
    { 0x8b3e01ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91de01ee, SUC_lexicon_tags_62 },
    { 0xfd9e01ee, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x149e01f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7e01fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfe0202, SUC_lexicon_tags_41 },
    { 0x185e0202, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc57e0205, SUC_lexicon_tags_49 },
    { 0x30de0206, SUC_lexicon_tags_152 },
    { 0x113e0206, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25e020e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9e0211, SUC_lexicon_tags_11 },
    { 0x3c5e0212, SUC_lexicon_tags_23 },
    { 0x7f5e0213, SUC_lexicon_tags_39 },
    { 0xcc7e0214, SUC_lexicon_tags_12 },
    { 0xe31e0215, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd1e0217, SUC_lexicon_tags_11 },
    { 0x165e0217, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e021d, SUC_lexicon_tags_37 },
    { 0xd79e021e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51e0221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189e0225, SUC_lexicon_tags_8 },
    { 0xc49e0225, SUC_lexicon_tags_38 },
    { 0x389e0227, SUC_lexicon_tags_14 },
    { 0x97e0227, SUC_lexicon_tags_16 },
    { 0x347e0228, SUC_lexicon_tags_27 },
    { 0xdc9e022a, SUC_lexicon_tags_12 },
    { 0xbd9e0227, SUC_lexicon_tags_8 },
    { 0xd3de022c, SUC_lexicon_tags_25 },
    { 0xcd5e022d, SUC_lexicon_tags_1 },
    { 0x3e9e0225, SUC_lexicon_tags_10 },
    { 0x417e0225, SUC_lexicon_tags_11 },
    { 0xa3fe0230, SUC_lexicon_tags_151 },
    { 0x6e7e0230, SUC_lexicon_tags_14 },
    { 0xf3fe0231, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0be0238, SUC_lexicon_tags_14 },
    { 0xe79e0239, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x957e023b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7e023e, SUC_lexicon_tags_39 },
    { 0xa3fe023f, SUC_lexicon_tags_27 },
    { 0x52be0240, SUC_lexicon_tags_13 },
    { 0xc45e0241, SUC_lexicon_tags_15 },
    { 0x835e0242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fe0248, SUC_lexicon_tags_55 },
    { 0xeede0248, SUC_lexicon_tags_48 },
    { 0x5dde0249, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa3e024c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x833e024f, SUC_lexicon_tags_30 },
    { 0x9e1e0250, SUC_lexicon_tags_12 },
    { 0xf1fe0250, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18be0255, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1e0259, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1e025c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9e0269, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ffe026b, SUC_lexicon_tags_19 },
    { 0x323e026b, SUC_lexicon_tags_39 },
    { 0x129e026d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x70be026f, SUC_lexicon_tags_10 },
    { 0x37de026f, SUC_lexicon_tags_34 },
    { 0xbdbe0271, SUC_lexicon_tags_17 },
    { 0x21be026f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07e0275, SUC_lexicon_tags_1 },
    { 0x9a5e0275, SUC_lexicon_tags_1 },
    { 0xbdde0277, SUC_lexicon_tags_57 },
    { 0x9ebe0278, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfe0281, SUC_lexicon_tags_28 },
    { 0x73e0281, SUC_lexicon_tags_2 },
    { 0x959e0283, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x26de0285, SUC_lexicon_tags_12 },
    { 0xb1de0286, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fde028a, SUC_lexicon_tags_14 },
    { 0x839e028b, SUC_lexicon_tags_17 },
    { 0x7afe028c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafe028f, SUC_lexicon_tags_25 },
    { 0xd4fe028f, SUC_lexicon_tags_15 },
    { 0x60be028f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5e0295, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x361e0297, SUC_lexicon_tags_46 },
    { 0x209e0298, SUC_lexicon_tags_7 },
    { 0x7afe0299, SUC_lexicon_tags_161 },
    { 0xe3de0297, SUC_lexicon_tags_37 },
    { 0x693e029b, SUC_lexicon_tags_37 },
    { 0xd21e029b, SUC_lexicon_tags_4 },
    { 0x139e029b, SUC_lexicon_tags_9 },
    { 0xe53e029e, SUC_lexicon_tags_34 },
    { 0xbffe029c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbe02a4, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x89de02a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e02aa, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x67fe02ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc9e02ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989e02b4, SUC_lexicon_tags_0 },
    { 0x45e02b4, SUC_lexicon_tags_37 },
    { 0xbd3e02b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fe02bc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1e02c1, SUC_lexicon_tags_194 },
    { 0x689e02c2, SUC_lexicon_tags_37 },
    { 0xcb9e02c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x4cfe02c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1e02cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9e02d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77de02d9, SUC_lexicon_tags_41 },
    { 0x92fe02da, SUC_lexicon_tags_47 },
    { 0xa75e02d9, SUC_lexicon_tags_37 },
    { 0xdc3e02dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x817e02de, SUC_lexicon_tags_6 },
    { 0x853e02de, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x945e02e1, SUC_lexicon_tags_26 },
    { 0x9b1e02e2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xee9e02e4, SUC_lexicon_tags_8 },
    { 0x9ede02e4, SUC_lexicon_tags_25 },
    { 0x2be02e6, SUC_lexicon_tags_37 },
    { 0x23de02e7, SUC_lexicon_tags_17 },
    { 0xfbfe02e7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fe02ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b5e02ee, SUC_lexicon_tags_53 },
    { 0x43fe02ee, SUC_lexicon_tags_2 },
    { 0x8fbe02ef, SUC_lexicon_tags_11 },
    { 0xde3e02f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4b1e02f3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x39be02f6, SUC_lexicon_tags_8 },
    { 0xf3de02f7, SUC_lexicon_tags_5 },
    { 0x999e02f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x461e02fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1be02fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf73e02ff, SUC_lexicon_tags_14 },
    { 0x985e0300, SUC_lexicon_tags_37 },
    { 0x267e0301, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa53e0303, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69e0306, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47e0309, SUC_lexicon_tags_37 },
    { 0x725e0309, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1be0311, SUC_lexicon_tags_39 },
    { 0x741e0312, SUC_lexicon_tags_25 },
    { 0x8d3e0312, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5e0316, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fde0319, SUC_lexicon_tags_15 },
    { 0xb87e031a, SUC_lexicon_tags_53 },
    { 0xb8fe031b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7e031f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695e0325, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf39e0327, SUC_lexicon_tags_7 },
    { 0xbde0327, SUC_lexicon_tags_57 },
    { 0xfa9e0329, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc9e032b, SUC_lexicon_tags_37 },
    { 0x4e7e032c, SUC_lexicon_tags_12 },
    { 0xf5fe032d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xd4be032f, SUC_lexicon_tags_8 },
    { 0xff5e032f, SUC_lexicon_tags_46 },
    { 0x8cbe0330, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0de0333, SUC_lexicon_tags_77 },
    { 0xff5e0334, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x847e0336, SUC_lexicon_tags_8 },
    { 0x133e0337, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49e033b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7e0340, SUC_lexicon_tags_47 },
    { 0x187e0341, SUC_lexicon_tags_39 },
    { 0x60de0342, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fe0345, SUC_lexicon_tags_26 },
    { 0xc47e0345, SUC_lexicon_tags_11 },
    { 0x967e0347, SUC_lexicon_tags_26 },
    { 0xfafe0347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cde034c, SUC_lexicon_tags_25 },
    { 0x27de034c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93e0351, SUC_lexicon_tags_25 },
    { 0x15e0352, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31e0356, SUC_lexicon_tags_79 },
    { 0x6fe0357, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x593e035a, SUC_lexicon_tags_14 },
    { 0x9c3e035b, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0xef9e035d, SUC_lexicon_tags_149 },
    { 0x69be035e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x885e0360, SUC_lexicon_tags_66 },
    { 0x121e0360, SUC_lexicon_tags_30 },
    { 0x559e0362, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x80be0364, SUC_lexicon_tags_87 },
    { 0x9be0364, SUC_lexicon_tags_37 },
    { 0xed9e0365, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x297e0368, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0de0372, SUC_lexicon_tags_37 },
    { 0x205e0373, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5e0377, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x43fe0379, SUC_lexicon_tags_12 },
    { 0xec3e0379, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddde0383, SUC_lexicon_tags_23 },
    { 0x133e0384, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6de0389, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4fe038c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fe0390, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x75e0392, SUC_lexicon_tags_17 },
    { 0x5a5e0393, SUC_lexicon_tags_38 },
    { 0x229e0393, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x25e0396, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1de0399, SUC_lexicon_tags_44 },
    { 0xed3e039a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669e039e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50be03a2, SUC_lexicon_tags_40 },
    { 0xe9be03a2, SUC_lexicon_tags_37 },
    { 0x7e7e03a2, SUC_lexicon_tags_44 },
    { 0x1e3e03a5, SUC_lexicon_tags_25 },
    { 0x1bde03a6, SUC_lexicon_tags_570 },
    { 0xdbfe03a3, SUC_lexicon_tags_55 },
    { 0xf91e03a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8d7e03aa, SUC_lexicon_tags_47 },
    { 0xdabe03ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x181e03ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfe03b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9a7e03b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xff3e03b5, SUC_lexicon_tags_17 },
    { 0xc07e03b6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xff7e03b8, SUC_lexicon_tags_12 },
    { 0xf43e03b9, SUC_lexicon_tags_25 },
    { 0x503e03b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e9e03bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x64de03be, SUC_lexicon_tags_2 },
    { 0xe51e03bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fe03c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87de03c8, SUC_lexicon_tags_37 },
    { 0x8b7e03c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x919e03cc, SUC_lexicon_tags_10 },
    { 0x63e03cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469e03d1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fe03db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9e03e0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35e03e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x60be03e9, SUC_lexicon_tags_25 },
    { 0x167e03ea, SUC_lexicon_tags_1 },
    { 0x36fe03ea, SUC_lexicon_tags_6 },
    { 0x27de03ec, SUC_lexicon_tags_60 },
    { 0x149e03ed, SUC_lexicon_tags_25 },
    { 0xaade03ee, SUC_lexicon_tags_11 },
    { 0x365e03ef, SUC_lexicon_tags_12 },
    { 0xff5e03ee, SUC_lexicon_tags_16 },
    { 0xa71e03f1, SUC_lexicon_tags_17 },
    { 0x9d3e03f2, SUC_lexicon_tags_30 },
    { 0x887e03f3, SUC_lexicon_tags_43 },
    { 0x459e03f4, SUC_lexicon_tags_17 },
    { 0x5cfe03ee, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfe03f9, SUC_lexicon_tags_38 },
    { 0xf63e03fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1e03fe, SUC_lexicon_tags_11 },
    { 0xb37e03ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55e0403, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd35e0405, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x819e0407, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de0412, SUC_lexicon_tags_28 },
    { 0x215e0413, SUC_lexicon_tags_133 },
    { 0x91fe0414, SUC_lexicon_tags_17 },
    { 0x643e0414, SUC_lexicon_tags_38 },
    { 0x705e0415, SUC_lexicon_tags_15 },
    { 0x6dde0417, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8b7e0419, SUC_lexicon_tags_6 },
    { 0x91fe041a, SUC_lexicon_tags_11 },
    { 0x14de041a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x781e041e, SUC_lexicon_tags_21 },
    { 0x2be041e, SUC_lexicon_tags_14 },
    { 0x21e0420, SUC_lexicon_tags_17 },
    { 0xce7e0420, SUC_lexicon_tags_1335 },
    { 0, NULL },
    { 0xdc5e0423, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xaebe0425, SUC_lexicon_tags_66 },
    { 0x1f3e0426, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1e042a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1de042d, SUC_lexicon_tags_17 },
    { 0xbafe042e, SUC_lexicon_tags_68 },
    { 0x213e042f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73e0433, SUC_lexicon_tags_61 },
    { 0x75e0433, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9b1e0436, SUC_lexicon_tags_42 },
    { 0x8afe0436, SUC_lexicon_tags_17 },
    { 0x959e0438, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa21e043a, SUC_lexicon_tags_37 },
    { 0x187e043b, SUC_lexicon_tags_1 },
    { 0xf71e043a, SUC_lexicon_tags_11 },
    { 0xf05e043c, SUC_lexicon_tags_7 },
    { 0xb2de043b, SUC_lexicon_tags_27 },
    { 0xce3e043e, SUC_lexicon_tags_23 },
    { 0xb87e0440, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b9e0442, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e0445, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x623e0447, SUC_lexicon_tags_63 },
    { 0x9de0448, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9e044b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fe044e, SUC_lexicon_tags_24 },
    { 0xf01e044e, SUC_lexicon_tags_15 },
    { 0xdf7e044e, SUC_lexicon_tags_8 },
    { 0xae5e0451, SUC_lexicon_tags_19 },
    { 0x99e0450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61e0457, SUC_lexicon_tags_7 },
    { 0xef3e0458, SUC_lexicon_tags_30 },
    { 0xa73e0457, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277e045f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9e0465, SUC_lexicon_tags_1 },
    { 0x94be0466, SUC_lexicon_tags_33 },
    { 0x92de0467, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9e046a, SUC_lexicon_tags_37 },
    { 0x92de046a, SUC_lexicon_tags_15 },
    { 0xe73e046b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be0470, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x31be0472, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x945e0475, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfbde0477, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x71be0479, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x389e047c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9cde047e, SUC_lexicon_tags_49 },
    { 0x8f1e047e, SUC_lexicon_tags_33 },
    { 0x4c9e0480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98fe0482, SUC_lexicon_tags_47 },
    { 0x693e0483, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e0488, SUC_lexicon_tags_55 },
    { 0xfe5e0488, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x887e048b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53e048d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995e0492, SUC_lexicon_tags_21 },
    { 0x503e0493, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xaf7e0495, SUC_lexicon_tags_7 },
    { 0x915e0496, SUC_lexicon_tags_0 },
    { 0x7efe0497, SUC_lexicon_tags_19 },
    { 0x163e0498, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7be049b, SUC_lexicon_tags_26 },
    { 0x1ede049c, SUC_lexicon_tags_38 },
    { 0x2cde049b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43e04a0, SUC_lexicon_tags_171 },
    { 0x63e04a0, SUC_lexicon_tags_7 },
    { 0xb09e04a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9e04a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fe04ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x465e04ad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63e04b4, SUC_lexicon_tags_17 },
    { 0x821e04b5, SUC_lexicon_tags_27 },
    { 0xfe5e04b6, SUC_lexicon_tags_2 },
    { 0x5a3e04b7, SUC_lexicon_tags_28 },
    { 0x87e04b8, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x4dbe04ba, SUC_lexicon_tags_17 },
    { 0x6c7e04ba, SUC_lexicon_tags_17 },
    { 0x74fe04bc, SUC_lexicon_tags_10 },
    { 0x2e7e04bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fe04c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x91fe04c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4be04c6, SUC_lexicon_tags_16 },
    { 0x715e04c7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fe04cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc53e04cd, SUC_lexicon_tags_64 },
    { 0x5de04ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70be04d8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x4b3e04da, SUC_lexicon_tags_25 },
    { 0x9c1e04db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x83de04de, SUC_lexicon_tags_10 },
    { 0x46de04df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2be04e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3fde04e3, SUC_lexicon_tags_33 },
    { 0xbe1e04e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x64be04e6, SUC_lexicon_tags_3 },
    { 0x511e04e7, SUC_lexicon_tags_23 },
    { 0xce1e04e8, SUC_lexicon_tags_1 },
    { 0x65e04e9, SUC_lexicon_tags_43 },
    { 0xd01e04e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7e04ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9e04f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83e04f4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x12fe04f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb53e04f8, SUC_lexicon_tags_61 },
    { 0x4fbe04f9, SUC_lexicon_tags_151 },
    { 0x129e04fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x119e04fc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbfe04fe, SUC_lexicon_tags_1 },
    { 0x3fe04ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x777e0501, SUC_lexicon_tags_17 },
    { 0xe71e0502, SUC_lexicon_tags_17 },
    { 0x111e0503, SUC_lexicon_tags_1 },
    { 0x43be0504, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30de050b, SUC_lexicon_tags_0 },
    { 0x543e050c, SUC_lexicon_tags_43 },
    { 0x35be050c, SUC_lexicon_tags_45 },
    { 0x505e050d, SUC_lexicon_tags_17 },
    { 0xab9e050d, SUC_lexicon_tags_37 },
    { 0x7d7e0510, SUC_lexicon_tags_77 },
    { 0xcdde0510, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x917e0513, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409e0518, SUC_lexicon_tags_17 },
    { 0x4e3e0518, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e0520, SUC_lexicon_tags_13 },
    { 0x1ade0520, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x2fde0523, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3e0526, SUC_lexicon_tags_1 },
    { 0x767e0526, SUC_lexicon_tags_15 },
    { 0x8a7e0527, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47e052d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x27e052f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x307e0532, SUC_lexicon_tags_7 },
    { 0x667e0533, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x115e0536, SUC_lexicon_tags_7 },
    { 0xbd5e0537, SUC_lexicon_tags_23 },
    { 0x9e7e0538, SUC_lexicon_tags_15 },
    { 0x8a7e0536, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fe0541, SUC_lexicon_tags_11 },
    { 0xf0fe0542, SUC_lexicon_tags_56 },
    { 0x475e0542, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761e0547, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fe054a, SUC_lexicon_tags_30 },
    { 0x509e054b, SUC_lexicon_tags_87 },
    { 0xe5de054b, SUC_lexicon_tags_37 },
    { 0x36be054d, SUC_lexicon_tags_1 },
    { 0x78be054e, SUC_lexicon_tags_19 },
    { 0x591e054b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71de0559, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e0560, SUC_lexicon_tags_81 },
    { 0x8d3e0561, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbe056d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc2fe056f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fe0577, SUC_lexicon_tags_15 },
    { 0x85be0578, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757e057d, SUC_lexicon_tags_7 },
    { 0x8f9e057e, SUC_lexicon_tags_15 },
    { 0xcf7e057e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbe0586, SUC_lexicon_tags_4 },
    { 0x9ede0587, SUC_lexicon_tags_2 },
    { 0x951e0586, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3e058c, SUC_lexicon_tags_82 },
    { 0x2cbe058d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x681e058f, SUC_lexicon_tags_12 },
    { 0x5c5e0590, SUC_lexicon_tags_25 },
    { 0x10fe0591, SUC_lexicon_tags_34 },
    { 0xd0fe0592, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccde0596, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4fe059a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8f7e059c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x907e059f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x443e05a2, SUC_lexicon_tags_31 },
    { 0xb47e05a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fde05af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d5e05b1, SUC_lexicon_tags_37 },
    { 0xbb1e05b1, SUC_lexicon_tags_87 },
    { 0x397e05b3, SUC_lexicon_tags_25 },
    { 0x907e05b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9e05b7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63be05c2, SUC_lexicon_tags_12 },
    { 0xd8fe05c3, SUC_lexicon_tags_11 },
    { 0x5c7e05c3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dde05ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11e05d0, SUC_lexicon_tags_29 },
    { 0x289e05d0, SUC_lexicon_tags_30 },
    { 0x9e1e05d2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7de05d6, SUC_lexicon_tags_35 },
    { 0xe9fe05d7, SUC_lexicon_tags_11 },
    { 0x22be05d6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3de05de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399e05e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55e05e5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abe05e9, SUC_lexicon_tags_4 },
    { 0x4f9e05ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91e05f2, SUC_lexicon_tags_23 },
    { 0x51be05f3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7e05f7, SUC_lexicon_tags_56 },
    { 0xc11e05f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1de05fb, SUC_lexicon_tags_37 },
    { 0xe1be05fb, SUC_lexicon_tags_28 },
    { 0x73e05fc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19e0604, SUC_lexicon_tags_60 },
    { 0x45e0604, SUC_lexicon_tags_38 },
    { 0xd11e0606, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc0fe0608, SUC_lexicon_tags_79 },
    { 0x43be0609, SUC_lexicon_tags_1 },
    { 0xf7de060a, SUC_lexicon_tags_30 },
    { 0x887e060b, SUC_lexicon_tags_1 },
    { 0xdf5e060c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3e060f, SUC_lexicon_tags_11 },
    { 0x483e0610, SUC_lexicon_tags_17 },
    { 0xc1de0611, SUC_lexicon_tags_37 },
    { 0xee9e0612, SUC_lexicon_tags_39 },
    { 0xdede0610, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x957e0615, SUC_lexicon_tags_17 },
    { 0x225e0615, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329e061b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1e061f, SUC_lexicon_tags_11 },
    { 0x489e0620, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebe0623, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35fe0625, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe0628, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dde062d, SUC_lexicon_tags_17 },
    { 0x2f1e062e, SUC_lexicon_tags_27 },
    { 0x653e062e, SUC_lexicon_tags_17 },
    { 0xdd7e062d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e0635, SUC_lexicon_tags_39 },
    { 0xe87e0636, SUC_lexicon_tags_62 },
    { 0xfefe0635, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fe063b, SUC_lexicon_tags_100 },
    { 0x3b9e063c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebe063f, SUC_lexicon_tags_47 },
    { 0xe3be0640, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963e0645, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8be0649, SUC_lexicon_tags_45 },
    { 0x35fe0649, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1e064d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x533e064f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5e0652, SUC_lexicon_tags_22 },
    { 0x247e0653, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3de0655, SUC_lexicon_tags_13 },
    { 0x7e5e0655, SUC_lexicon_tags_1 },
    { 0xd43e0657, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffe065b, SUC_lexicon_tags_2 },
    { 0x5e7e065c, SUC_lexicon_tags_7 },
    { 0x267e065c, SUC_lexicon_tags_32 },
    { 0xe29e065e, SUC_lexicon_tags_11 },
    { 0xfabe065f, SUC_lexicon_tags_12 },
    { 0x5afe065e, SUC_lexicon_tags_9 },
    { 0x669e0661, SUC_lexicon_tags_14 },
    { 0xd9be0662, SUC_lexicon_tags_43 },
    { 0x137e0662, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1e0666, SUC_lexicon_tags_10 },
    { 0xbd5e0667, SUC_lexicon_tags_2 },
    { 0x519e0666, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x205e066b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66be0672, SUC_lexicon_tags_44 },
    { 0xbbbe0672, SUC_lexicon_tags_22 },
    { 0x9d1e0674, SUC_lexicon_tags_15 },
    { 0x187e0673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61e067b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fe0684, SUC_lexicon_tags_30 },
    { 0x711e0685, SUC_lexicon_tags_45 },
    { 0x97de0686, SUC_lexicon_tags_15 },
    { 0xe79e0687, SUC_lexicon_tags_14 },
    { 0x119e0687, SUC_lexicon_tags_27 },
    { 0x7d5e0688, SUC_lexicon_tags_61 },
    { 0xaf3e0686, SUC_lexicon_tags_1329 },
    { 0xcfde068b, SUC_lexicon_tags_27 },
    { 0xbb3e0688, SUC_lexicon_tags_25 },
    { 0xe07e068b, SUC_lexicon_tags_63 },
    { 0x6c3e0688, SUC_lexicon_tags_37 },
    { 0x98fe0686, SUC_lexicon_tags_31 },
    { 0xb93e0684, SUC_lexicon_tags_87 },
    { 0x3f1e0684, SUC_lexicon_tags_26 },
    { 0x257e0687, SUC_lexicon_tags_17 },
    { 0x8fe0693, SUC_lexicon_tags_63 },
    { 0x5cbe0684, SUC_lexicon_tags_38 },
    { 0x1ebe0695, SUC_lexicon_tags_17 },
    { 0xc65e0695, SUC_lexicon_tags_2 },
    { 0x5d9e0697, SUC_lexicon_tags_26 },
    { 0x2e9e0696, SUC_lexicon_tags_30 },
    { 0x551e068c, SUC_lexicon_tags_95 },
    { 0xe0be069a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e06a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abe06a6, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09e06a9, SUC_lexicon_tags_65 },
    { 0x8e1e06a9, SUC_lexicon_tags_13 },
    { 0x5d5e06ab, SUC_lexicon_tags_43 },
    { 0x197e06aa, SUC_lexicon_tags_87 },
    { 0x159e06ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9de06af, SUC_lexicon_tags_11 },
    { 0xdcbe06af, SUC_lexicon_tags_25 },
    { 0xcf9e06b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5e06b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79e06b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3e06be, SUC_lexicon_tags_26 },
    { 0x2f1e06bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x379e06c2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xfb7e06c4, SUC_lexicon_tags_9 },
    { 0x61fe06c4, SUC_lexicon_tags_7 },
    { 0xcefe06c4, SUC_lexicon_tags_93 },
    { 0x103e06c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2a7e06c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x851e06cb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfde06cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xeade06d1, SUC_lexicon_tags_45 },
    { 0x79fe06d1, SUC_lexicon_tags_2 },
    { 0x92be06d1, SUC_lexicon_tags_2 },
    { 0xc2be06d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7be06db, SUC_lexicon_tags_29 },
    { 0xb2fe06dc, SUC_lexicon_tags_15 },
    { 0x457e06dd, SUC_lexicon_tags_57 },
    { 0x2b1e06dd, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1e06e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5e06e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf1e06e7, SUC_lexicon_tags_17 },
    { 0xfa1e06e7, SUC_lexicon_tags_2 },
    { 0x2f5e06e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x30be06ec, SUC_lexicon_tags_14 },
    { 0x8f7e06ed, SUC_lexicon_tags_12 },
    { 0x3fbe06ec, SUC_lexicon_tags_25 },
    { 0x169e06ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5e06f3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7e06f6, SUC_lexicon_tags_9 },
    { 0xae9e06f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51fe06fb, SUC_lexicon_tags_23 },
    { 0x125e06fc, SUC_lexicon_tags_17 },
    { 0x1d1e06fd, SUC_lexicon_tags_28 },
    { 0x209e06fe, SUC_lexicon_tags_7 },
    { 0x45fe06fe, SUC_lexicon_tags_16 },
    { 0xe43e0700, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x51de0703, SUC_lexicon_tags_12 },
    { 0xfb9e0704, SUC_lexicon_tags_86 },
    { 0x9a3e0704, SUC_lexicon_tags_7 },
    { 0x8f1e0706, SUC_lexicon_tags_14 },
    { 0x29fe0703, SUC_lexicon_tags_37 },
    { 0xbc9e0708, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x349e070b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x98de070d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x47be0710, SUC_lexicon_tags_77 },
    { 0xfc5e0711, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687e071b, SUC_lexicon_tags_1 },
    { 0x739e071c, SUC_lexicon_tags_7 },
    { 0xbffe071c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7e0722, SUC_lexicon_tags_26 },
    { 0x63fe0723, SUC_lexicon_tags_25 },
    { 0x1fde0723, SUC_lexicon_tags_1 },
    { 0xb45e0725, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc8fe0727, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f1e0729, SUC_lexicon_tags_27 },
    { 0x7b5e0729, SUC_lexicon_tags_17 },
    { 0xe55e072a, SUC_lexicon_tags_60 },
    { 0xccde072a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155e0730, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x74de0732, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xc17e0734, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fe0738, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19e073b, SUC_lexicon_tags_7 },
    { 0x9be073c, SUC_lexicon_tags_7 },
    { 0xab7e073c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xdcfe073f, SUC_lexicon_tags_11 },
    { 0x37fe073f, SUC_lexicon_tags_35 },
    { 0x415e0741, SUC_lexicon_tags_47 },
    { 0xfe7e0741, SUC_lexicon_tags_17 },
    { 0x68de073f, SUC_lexicon_tags_15 },
    { 0x335e073f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x46de0746, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ede0748, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9be074d, SUC_lexicon_tags_37 },
    { 0x7a5e074e, SUC_lexicon_tags_14 },
    { 0xb9e074d, SUC_lexicon_tags_37 },
    { 0x4e5e0750, SUC_lexicon_tags_0 },
    { 0xd5e074d, SUC_lexicon_tags_28 },
    { 0x1f7e0751, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbe0759, SUC_lexicon_tags_7 },
    { 0xa3be075a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a9e075c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9e0760, SUC_lexicon_tags_16 },
    { 0x8c7e0761, SUC_lexicon_tags_11 },
    { 0x54de0760, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e0768, SUC_lexicon_tags_6 },
    { 0x3c7e0768, SUC_lexicon_tags_37 },
    { 0x80fe076a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7e076e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xa5de0770, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfe0773, SUC_lexicon_tags_17 },
    { 0xaf3e0774, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbe0778, SUC_lexicon_tags_23 },
    { 0x365e0778, SUC_lexicon_tags_12 },
    { 0x679e077a, SUC_lexicon_tags_29 },
    { 0x8afe077b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03e077e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89e0783, SUC_lexicon_tags_2 },
    { 0x1bfe0783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7d3e0786, SUC_lexicon_tags_25 },
    { 0xf07e0787, SUC_lexicon_tags_15 },
    { 0x4b5e0788, SUC_lexicon_tags_47 },
    { 0x245e0786, SUC_lexicon_tags_11 },
    { 0x223e078a, SUC_lexicon_tags_12 },
    { 0x5b5e078a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359e078f, SUC_lexicon_tags_33 },
    { 0x22de078f, SUC_lexicon_tags_60 },
    { 0x9dfe0791, SUC_lexicon_tags_27 },
    { 0x803e078f, SUC_lexicon_tags_1517 },
    { 0x5a3e0791, SUC_lexicon_tags_17 },
    { 0x2a3e0794, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0x92be0796, SUC_lexicon_tags_65 },
    { 0xcb5e0797, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2fe0799, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb49e079b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb59e079d, SUC_lexicon_tags_87 },
    { 0x9fe079d, SUC_lexicon_tags_2 },
    { 0x88be079f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55e07a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81be07ae, SUC_lexicon_tags_15 },
    { 0x1a7e07af, SUC_lexicon_tags_12 },
    { 0x287e07b0, SUC_lexicon_tags_11 },
    { 0x9ebe07b1, SUC_lexicon_tags_49 },
    { 0xfabe07b2, SUC_lexicon_tags_25 },
    { 0x9abe07b3, SUC_lexicon_tags_15 },
    { 0xfde07b3, SUC_lexicon_tags_30 },
    { 0x1be07ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x671e07b8, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x7cfe07ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39e07bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9de07bf, SUC_lexicon_tags_17 },
    { 0xcd5e07bf, SUC_lexicon_tags_22 },
    { 0xf3be07c1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd5e07c3, SUC_lexicon_tags_4 },
    { 0x965e07c4, SUC_lexicon_tags_12 },
    { 0x431e07c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8cde07c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x74fe07ca, SUC_lexicon_tags_15 },
    { 0xc51e07ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x197e07cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fe07d2, SUC_lexicon_tags_65 },
    { 0x7bde07d3, SUC_lexicon_tags_17 },
    { 0xe19e07d4, SUC_lexicon_tags_17 },
    { 0xdefe07d5, SUC_lexicon_tags_17 },
    { 0x98fe07d6, SUC_lexicon_tags_17 },
    { 0x54be07d3, SUC_lexicon_tags_79 },
    { 0x37be07d7, SUC_lexicon_tags_23 },
    { 0x7de07d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3e07dd, SUC_lexicon_tags_33 },
    { 0xd39e07de, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fe07e3, SUC_lexicon_tags_25 },
    { 0x27e07e4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5e07e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5e07eb, SUC_lexicon_tags_30 },
    { 0xbe07eb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1f1e07ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6de07f0, SUC_lexicon_tags_25 },
    { 0xd1de07f0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x211e07f3, SUC_lexicon_tags_25 },
    { 0x7c9e07f4, SUC_lexicon_tags_23 },
    { 0x71fe07f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe05e07f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2d1e07f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa89e07fb, SUC_lexicon_tags_11 },
    { 0x5b9e07fb, SUC_lexicon_tags_17 },
    { 0xdbde07fd, SUC_lexicon_tags_15 },
    { 0x6cfe07fe, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cde0804, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x351e0806, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97e080b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7e080f, SUC_lexicon_tags_14 },
    { 0x65e0810, SUC_lexicon_tags_65 },
    { 0x31be080f, SUC_lexicon_tags_8 },
    { 0x47fe080f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2de0819, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3e081d, SUC_lexicon_tags_17 },
    { 0x679e081d, SUC_lexicon_tags_40 },
    { 0x889e081f, SUC_lexicon_tags_45 },
    { 0x5de0820, SUC_lexicon_tags_38 },
    { 0x939e0820, SUC_lexicon_tags_40 },
    { 0x3a9e0822, SUC_lexicon_tags_2 },
    { 0x679e0820, SUC_lexicon_tags_2 },
    { 0x467e0822, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9e0829, SUC_lexicon_tags_23 },
    { 0x493e082a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x365e082c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683e0834, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8afe083d, SUC_lexicon_tags_77 },
    { 0x4a5e083d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0be0843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x285e0845, SUC_lexicon_tags_9 },
    { 0xddde0846, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189e084c, SUC_lexicon_tags_17 },
    { 0x669e084d, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49de0852, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7de0859, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x903e085b, SUC_lexicon_tags_2 },
    { 0xa61e085c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8de0862, SUC_lexicon_tags_38 },
    { 0x42de0863, SUC_lexicon_tags_17 },
    { 0x397e0863, SUC_lexicon_tags_26 },
    { 0x299e0862, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5e0869, SUC_lexicon_tags_11 },
    { 0x50de086a, SUC_lexicon_tags_11 },
    { 0x21be0869, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfade086f, SUC_lexicon_tags_25 },
    { 0x5cfe086f, SUC_lexicon_tags_15 },
    { 0x8e1e0870, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x685e0873, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3e0877, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x14de087a, SUC_lexicon_tags_65 },
    { 0xdebe087b, SUC_lexicon_tags_17 },
    { 0x217e087a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225e0882, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3e0885, SUC_lexicon_tags_27 },
    { 0xcd1e0886, SUC_lexicon_tags_1 },
    { 0x685e0885, SUC_lexicon_tags_30 },
    { 0x1b5e0888, SUC_lexicon_tags_11 },
    { 0xd9be0885, SUC_lexicon_tags_31 },
    { 0xe4de088a, SUC_lexicon_tags_56 },
    { 0x69fe0889, SUC_lexicon_tags_15 },
    { 0xbdde088c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5fe088e, SUC_lexicon_tags_15 },
    { 0x287e088e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe0892, SUC_lexicon_tags_17 },
    { 0xecde0893, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9e0897, SUC_lexicon_tags_4 },
    { 0xd5de0898, SUC_lexicon_tags_66 },
    { 0xc25e0897, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb25e089b, SUC_lexicon_tags_17 },
    { 0xaf5e089b, SUC_lexicon_tags_25 },
    { 0x969e089b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375e08a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27be08a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5e08aa, SUC_lexicon_tags_2 },
    { 0xf07e08ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x75be08ae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x771e08b0, SUC_lexicon_tags_25 },
    { 0x91be08b1, SUC_lexicon_tags_17 },
    { 0x77de08b1, SUC_lexicon_tags_12 },
    { 0x99fe08b3, SUC_lexicon_tags_15 },
    { 0x6bbe08b4, SUC_lexicon_tags_4 },
    { 0x55be08b3, SUC_lexicon_tags_80 },
    { 0x25be08b6, SUC_lexicon_tags_92 },
    { 0xd17e08b7, SUC_lexicon_tags_12 },
    { 0x6a1e08b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe08bd, SUC_lexicon_tags_17 },
    { 0xcffe08be, SUC_lexicon_tags_11 },
    { 0x6d9e08bf, SUC_lexicon_tags_63 },
    { 0xf97e08bd, SUC_lexicon_tags_0 },
    { 0xcafe08c1, SUC_lexicon_tags_19 },
    { 0xb1de08c0, SUC_lexicon_tags_95 },
    { 0xacbe08c3, SUC_lexicon_tags_31 },
    { 0x6d9e08c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961e08c9, SUC_lexicon_tags_8 },
    { 0x4d5e08ca, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7e08cd, SUC_lexicon_tags_43 },
    { 0x5b3e08ce, SUC_lexicon_tags_23 },
    { 0x1ade08ce, SUC_lexicon_tags_1 },
    { 0x4bfe08d0, SUC_lexicon_tags_68 },
    { 0x673e08cf, SUC_lexicon_tags_45 },
    { 0xaa3e08d2, SUC_lexicon_tags_7 },
    { 0x47be08d3, SUC_lexicon_tags_87 },
    { 0xd3be08d2, SUC_lexicon_tags_15 },
    { 0x1cde08d5, SUC_lexicon_tags_12 },
    { 0x239e08d3, SUC_lexicon_tags_17 },
    { 0xe8fe08d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x67be08d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e08dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6a5e08de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x369e08e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7e08e4, SUC_lexicon_tags_37 },
    { 0xf05e08e4, SUC_lexicon_tags_37 },
    { 0x4e5e08e6, SUC_lexicon_tags_12 },
    { 0x3b3e08e7, SUC_lexicon_tags_2 },
    { 0x72fe08e4, SUC_lexicon_tags_14 },
    { 0x305e08e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33e08eb, SUC_lexicon_tags_11 },
    { 0x55fe08eb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdf1e08ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc53e08f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae9e08f2, SUC_lexicon_tags_12 },
    { 0x509e08f3, SUC_lexicon_tags_47 },
    { 0xfb7e08f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x419e08f7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efe08fa, SUC_lexicon_tags_11 },
    { 0xfb9e08fb, SUC_lexicon_tags_31 },
    { 0x2d7e08fc, SUC_lexicon_tags_17 },
    { 0x24fe08fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x187e08ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229e0906, SUC_lexicon_tags_31 },
    { 0x99e0907, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75e090f, SUC_lexicon_tags_19 },
    { 0x847e090f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497e0916, SUC_lexicon_tags_55 },
    { 0xf2de0917, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3e091d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebe0926, SUC_lexicon_tags_25 },
    { 0x5dde0926, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1e092a, SUC_lexicon_tags_20 },
    { 0xc3de092b, SUC_lexicon_tags_1 },
    { 0x797e092c, SUC_lexicon_tags_47 },
    { 0x13e092d, SUC_lexicon_tags_11 },
    { 0x515e092d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abe0932, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afe0935, SUC_lexicon_tags_12 },
    { 0x33be0936, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41de0938, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbe093b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209e0940, SUC_lexicon_tags_0 },
    { 0xbd3e0941, SUC_lexicon_tags_7 },
    { 0xafde0942, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95e0946, SUC_lexicon_tags_38 },
    { 0x891e0946, SUC_lexicon_tags_37 },
    { 0x4a9e0946, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fde094d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb83e094f, SUC_lexicon_tags_37 },
    { 0x121e094f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9e0953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28be0958, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbe095c, SUC_lexicon_tags_37 },
    { 0x29be095d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed5e095f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59e096b, SUC_lexicon_tags_11 },
    { 0x65de096b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x781e096f, SUC_lexicon_tags_37 },
    { 0xf21e0970, SUC_lexicon_tags_8 },
    { 0xbe0971, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5de0974, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07e097c, SUC_lexicon_tags_27 },
    { 0x449e097d, SUC_lexicon_tags_158 },
    { 0x9c9e097c, SUC_lexicon_tags_57 },
    { 0xe79e097f, SUC_lexicon_tags_9 },
    { 0xf39e0980, SUC_lexicon_tags_35 },
    { 0x5a3e0981, SUC_lexicon_tags_17 },
    { 0x56fe0982, SUC_lexicon_tags_25 },
    { 0x69de0980, SUC_lexicon_tags_63 },
    { 0x8a3e0984, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x873e0986, SUC_lexicon_tags_14 },
    { 0x5c5e0986, SUC_lexicon_tags_37 },
    { 0x223e0987, SUC_lexicon_tags_15 },
    { 0x4f7e0986, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ade098f, SUC_lexicon_tags_4 },
    { 0xd29e098f, SUC_lexicon_tags_0 },
    { 0x235e0991, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abe0995, SUC_lexicon_tags_1137 },
    { 0, NULL },
    { 0xff3e0997, SUC_lexicon_tags_82 },
    { 0xd7e0998, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99e099b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9e09a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3e09a5, SUC_lexicon_tags_45 },
    { 0x743e09a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfe09a9, SUC_lexicon_tags_27 },
    { 0xaa5e09aa, SUC_lexicon_tags_17 },
    { 0xd89e09ab, SUC_lexicon_tags_11 },
    { 0xa37e09ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e09b2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdade09b7, SUC_lexicon_tags_11 },
    { 0xd0be09b7, SUC_lexicon_tags_48 },
    { 0xbde09b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x571e09bb, SUC_lexicon_tags_1 },
    { 0x2fbe09bb, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xc47e09be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x47be09c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bde09c4, SUC_lexicon_tags_15 },
    { 0x2abe09c5, SUC_lexicon_tags_46 },
    { 0x21fe09c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x713e09c8, SUC_lexicon_tags_7 },
    { 0x43fe09c9, SUC_lexicon_tags_37 },
    { 0x843e09ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2ffe09cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fe09d4, SUC_lexicon_tags_45 },
    { 0xaa1e09d4, SUC_lexicon_tags_55 },
    { 0xccfe09d5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x953e09d9, SUC_lexicon_tags_11 },
    { 0x21fe09d9, SUC_lexicon_tags_37 },
    { 0xe83e09db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbde09e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73e09e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeede09e7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9e09eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9e09ef, SUC_lexicon_tags_8 },
    { 0xb19e09ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49be09f4, SUC_lexicon_tags_10 },
    { 0x383e09f4, SUC_lexicon_tags_38 },
    { 0x481e09f6, SUC_lexicon_tags_30 },
    { 0xf83e09f6, SUC_lexicon_tags_25 },
    { 0x677e09f8, SUC_lexicon_tags_15 },
    { 0x68fe09f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3be09ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fe0a02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x289e0a04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33e0a0a, SUC_lexicon_tags_28 },
    { 0x5d5e0a0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fe0a10, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e0a13, SUC_lexicon_tags_15 },
    { 0x39e0a14, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245e0a19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdd7e0a1b, SUC_lexicon_tags_5 },
    { 0x955e0a1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383e0a21, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1e0a25, SUC_lexicon_tags_21 },
    { 0x1f1e0a26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc39e0a28, SUC_lexicon_tags_47 },
    { 0x83fe0a28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64fe0a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67e0a2e, SUC_lexicon_tags_47 },
    { 0x70fe0a2e, SUC_lexicon_tags_43 },
    { 0xcefe0a2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e0a36, SUC_lexicon_tags_82 },
    { 0x7bde0a37, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7e0a3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57e0a40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfe0a48, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13e0a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cfe0a4f, SUC_lexicon_tags_25 },
    { 0xb75e0a4f, SUC_lexicon_tags_7 },
    { 0xabde0a51, SUC_lexicon_tags_25 },
    { 0x7d5e0a51, SUC_lexicon_tags_11 },
    { 0x215e0a53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbe0a5e, SUC_lexicon_tags_25 },
    { 0x5ede0a5e, SUC_lexicon_tags_43 },
    { 0xcfe0a5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x105e0a62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329e0a66, SUC_lexicon_tags_45 },
    { 0xadbe0a66, SUC_lexicon_tags_25 },
    { 0x3c5e0a68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fe0a6d, SUC_lexicon_tags_37 },
    { 0x395e0a6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89e0a73, SUC_lexicon_tags_32 },
    { 0x31e0a74, SUC_lexicon_tags_2 },
    { 0x963e0a74, SUC_lexicon_tags_1 },
    { 0x157e0a76, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x885e0a78, SUC_lexicon_tags_12 },
    { 0x71e0a79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41e0a80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9e0a84, SUC_lexicon_tags_7 },
    { 0xecbe0a85, SUC_lexicon_tags_37 },
    { 0x5d3e0a84, SUC_lexicon_tags_87 },
    { 0xf15e0a87, SUC_lexicon_tags_12 },
    { 0x47fe0a88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8de0a8b, SUC_lexicon_tags_15 },
    { 0x96fe0a8c, SUC_lexicon_tags_37 },
    { 0xbdbe0a8d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9e0a92, SUC_lexicon_tags_12 },
    { 0xadbe0a93, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x911e0a95, SUC_lexicon_tags_3 },
    { 0xf87e0a96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5e0a9b, SUC_lexicon_tags_2 },
    { 0x8be0a9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ade0a9e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x283e0aa0, SUC_lexicon_tags_47 },
    { 0x81fe0aa1, SUC_lexicon_tags_2 },
    { 0x8c3e0aa2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xadde0aa5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x583e0aa7, SUC_lexicon_tags_15 },
    { 0xa0be0aa8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde1e0aaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57e0ab1, SUC_lexicon_tags_66 },
    { 0xdb9e0ab1, SUC_lexicon_tags_0 },
    { 0xdb7e0ab1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6de0ab9, SUC_lexicon_tags_2 },
    { 0x3b9e0aba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44fe0abc, SUC_lexicon_tags_30 },
    { 0xa35e0abd, SUC_lexicon_tags_4 },
    { 0x24be0abc, SUC_lexicon_tags_15 },
    { 0x251e0abc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7e0ac7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85e0ad1, SUC_lexicon_tags_32 },
    { 0x597e0ad2, SUC_lexicon_tags_17 },
    { 0xc1e0ad3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fe0ad6, SUC_lexicon_tags_23 },
    { 0xe9fe0ad7, SUC_lexicon_tags_27 },
    { 0x5e1e0ad6, SUC_lexicon_tags_48 },
    { 0x789e0ad9, SUC_lexicon_tags_37 },
    { 0xf45e0ada, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497e0adf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fe0ae4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74de0ae6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1e0aed, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7e0af0, SUC_lexicon_tags_37 },
    { 0xb0fe0af1, SUC_lexicon_tags_16 },
    { 0x34fe0af1, SUC_lexicon_tags_17 },
    { 0xc15e0af3, SUC_lexicon_tags_40 },
    { 0x9ebe0af4, SUC_lexicon_tags_16 },
    { 0x709e0af4, SUC_lexicon_tags_47 },
    { 0xc1e0af4, SUC_lexicon_tags_37 },
    { 0x4e1e0af5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23be0afb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14de0b05, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x397e0b08, SUC_lexicon_tags_60 },
    { 0xd33e0b09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x211e0b0b, SUC_lexicon_tags_17 },
    { 0xbf7e0b0b, SUC_lexicon_tags_25 },
    { 0xbc3e0b0b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ffe0b0f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x45de0b12, SUC_lexicon_tags_12 },
    { 0x265e0b13, SUC_lexicon_tags_7 },
    { 0x683e0b12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fde0b17, SUC_lexicon_tags_15 },
    { 0xd7de0b17, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x7ebe0b1a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x669e0b1d, SUC_lexicon_tags_37 },
    { 0x745e0b1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbbbe0b20, SUC_lexicon_tags_11 },
    { 0xc99e0b21, SUC_lexicon_tags_17 },
    { 0x679e0b21, SUC_lexicon_tags_32 },
    { 0xddbe0b21, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3e0b26, SUC_lexicon_tags_1 },
    { 0x40fe0b26, SUC_lexicon_tags_96 },
    { 0x24de0b27, SUC_lexicon_tags_2 },
    { 0x961e0b29, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbe0b2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86de0b31, SUC_lexicon_tags_4 },
    { 0x61fe0b32, SUC_lexicon_tags_17 },
    { 0x917e0b32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaafe0b35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9b9e0b37, SUC_lexicon_tags_25 },
    { 0x3dfe0b38, SUC_lexicon_tags_60 },
    { 0xa45e0b38, SUC_lexicon_tags_17 },
    { 0xdb9e0b39, SUC_lexicon_tags_33 },
    { 0x965e0b3b, SUC_lexicon_tags_47 },
    { 0xc95e0b3c, SUC_lexicon_tags_30 },
    { 0xcd1e0b3b, SUC_lexicon_tags_0 },
    { 0xf85e0b38, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425e0b43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbe0b49, SUC_lexicon_tags_0 },
    { 0xf17e0b49, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeafe0b4d, SUC_lexicon_tags_26 },
    { 0x341e0b4e, SUC_lexicon_tags_77 },
    { 0xe4be0b4f, SUC_lexicon_tags_17 },
    { 0x4f3e0b4f, SUC_lexicon_tags_37 },
    { 0x753e0b51, SUC_lexicon_tags_46 },
    { 0xb53e0b4e, SUC_lexicon_tags_31 },
    { 0xcf7e0b53, SUC_lexicon_tags_37 },
    { 0x735e0b54, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe83e0b56, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfc7e0b58, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8be0b5e, SUC_lexicon_tags_2 },
    { 0x3b5e0b5f, SUC_lexicon_tags_43 },
    { 0xf13e0b5f, SUC_lexicon_tags_1 },
    { 0x213e0b61, SUC_lexicon_tags_70 },
    { 0x855e0b61, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa11e0b64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5f7e0b66, SUC_lexicon_tags_25 },
    { 0x129e0b67, SUC_lexicon_tags_47 },
    { 0x5f3e0b68, SUC_lexicon_tags_80 },
    { 0xd95e0b66, SUC_lexicon_tags_10 },
    { 0x903e0b6a, SUC_lexicon_tags_17 },
    { 0xba3e0b6a, SUC_lexicon_tags_12 },
    { 0x401e0b6a, SUC_lexicon_tags_129 },
    { 0x615e0b6d, SUC_lexicon_tags_3 },
    { 0x16de0b6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1afe0b70, SUC_lexicon_tags_12 },
    { 0x761e0b71, SUC_lexicon_tags_15 },
    { 0x369e0b70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9e0b75, SUC_lexicon_tags_29 },
    { 0xfb3e0b75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3d7e0b78, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323e0b7c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99e0b80, SUC_lexicon_tags_49 },
    { 0x7f3e0b80, SUC_lexicon_tags_25 },
    { 0x825e0b82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd2fe0b84, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481e0b8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x573e0b8e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481e0b92, SUC_lexicon_tags_4 },
    { 0x77e0b92, SUC_lexicon_tags_15 },
    { 0x337e0b94, SUC_lexicon_tags_7 },
    { 0xfd1e0b95, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803e0b9c, SUC_lexicon_tags_37 },
    { 0x4b9e0b9d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59e0ba3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abe0ba7, SUC_lexicon_tags_15 },
    { 0xdfde0ba8, SUC_lexicon_tags_25 },
    { 0xa5de0ba9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafe0bad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe0bb1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73e0bbb, SUC_lexicon_tags_12 },
    { 0xbb1e0bbb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e9e0bbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70de0bc3, SUC_lexicon_tags_15 },
    { 0xa57e0bc3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09e0bc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abe0bcb, SUC_lexicon_tags_25 },
    { 0x96be0bcc, SUC_lexicon_tags_1 },
    { 0x743e0bcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997e0bd4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35e0be6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07e0be9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7e0bf3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751e0bfa, SUC_lexicon_tags_27 },
    { 0xbcfe0bfb, SUC_lexicon_tags_15 },
    { 0x18de0bfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93e0c02, SUC_lexicon_tags_25 },
    { 0x2c3e0c03, SUC_lexicon_tags_1 },
    { 0xf23e0c03, SUC_lexicon_tags_61 },
    { 0xebbe0c04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebe0c0a, SUC_lexicon_tags_37 },
    { 0x437e0c0b, SUC_lexicon_tags_12 },
    { 0x289e0c0c, SUC_lexicon_tags_125 },
    { 0, NULL },
    { 0, NULL },
    { 0x505e0c0f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5e0c14, SUC_lexicon_tags_37 },
    { 0x16fe0c15, SUC_lexicon_tags_37 },
    { 0x57de0c16, SUC_lexicon_tags_77 },
    { 0xbdde0c17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bde0c1a, SUC_lexicon_tags_9 },
    { 0xc4be0c1a, SUC_lexicon_tags_11 },
    { 0x8dbe0c1c, SUC_lexicon_tags_14 },
    { 0x5d7e0c1a, SUC_lexicon_tags_84 },
    { 0xc19e0c1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3de0c22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7e0c25, SUC_lexicon_tags_14 },
    { 0x8f5e0c26, SUC_lexicon_tags_60 },
    { 0x88fe0c26, SUC_lexicon_tags_1 },
    { 0x735e0c27, SUC_lexicon_tags_21 },
    { 0xfd5e0c28, SUC_lexicon_tags_60 },
    { 0xbbbe0c2a, SUC_lexicon_tags_30 },
    { 0x8b1e0c2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583e0c2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x923e0c32, SUC_lexicon_tags_15 },
    { 0x971e0c33, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x819e0c36, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x62de0c38, SUC_lexicon_tags_77 },
    { 0x5bde0c39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fde0c3f, SUC_lexicon_tags_37 },
    { 0xc8fe0c40, SUC_lexicon_tags_17 },
    { 0x7b1e0c41, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201e0c45, SUC_lexicon_tags_37 },
    { 0xeb7e0c46, SUC_lexicon_tags_63 },
    { 0xb21e0c45, SUC_lexicon_tags_25 },
    { 0x555e0c48, SUC_lexicon_tags_1 },
    { 0xc59e0c46, SUC_lexicon_tags_21 },
    { 0x2b1e0c47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fe0c4f, SUC_lexicon_tags_15 },
    { 0x7f5e0c50, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9e0c52, SUC_lexicon_tags_28 },
    { 0xbd5e0c53, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xd1e0c55, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfe0c59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77e0c61, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfe0c66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe0c69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa91e0c6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fe0c6e, SUC_lexicon_tags_14 },
    { 0xb6fe0c6e, SUC_lexicon_tags_1 },
    { 0x30be0c70, SUC_lexicon_tags_12 },
    { 0xaa1e0c6e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfe0c74, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebe0c77, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc0be0c79, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655e0c7e, SUC_lexicon_tags_16 },
    { 0x95de0c7f, SUC_lexicon_tags_37 },
    { 0xe2de0c80, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9e0c84, SUC_lexicon_tags_19 },
    { 0x3ebe0c85, SUC_lexicon_tags_17 },
    { 0x871e0c86, SUC_lexicon_tags_35 },
    { 0x4f9e0c87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26be0c8c, SUC_lexicon_tags_12 },
    { 0xff7e0c8c, SUC_lexicon_tags_289 },
    { 0xb75e0c8d, SUC_lexicon_tags_19 },
    { 0xe3e0c8f, SUC_lexicon_tags_29 },
    { 0x8e1e0c8f, SUC_lexicon_tags_12 },
    { 0x935e0c8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1de0c93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9be0c95, SUC_lexicon_tags_0 },
    { 0x2dbe0c95, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x96de0c98, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97e0c9d, SUC_lexicon_tags_11 },
    { 0x60fe0c9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de0ca8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa93e0caa, SUC_lexicon_tags_37 },
    { 0x42de0cab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145e0cb0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e0cb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9e0cb8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xea1e0cba, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x433e0cbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x74de0cbe, SUC_lexicon_tags_1 },
    { 0x1a1e0cbe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x42be0cc1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e9e0cc3, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1e0cc9, SUC_lexicon_tags_63 },
    { 0x997e0cca, SUC_lexicon_tags_17 },
    { 0xa03e0cc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455e0ccf, SUC_lexicon_tags_0 },
    { 0x8e7e0cd0, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x7a7e0cd2, SUC_lexicon_tags_1 },
    { 0xd71e0cd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7e0cd7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xde7e0cd9, SUC_lexicon_tags_17 },
    { 0x1dbe0cd9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xca3e0cdd, SUC_lexicon_tags_37 },
    { 0x7d3e0cde, SUC_lexicon_tags_12 },
    { 0x7a7e0cdd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbe0ce4, SUC_lexicon_tags_11 },
    { 0x413e0ce5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb29e0ce7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d7e0ce9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x513e0cec, SUC_lexicon_tags_38 },
    { 0x951e0ced, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc2de0cef, SUC_lexicon_tags_11 },
    { 0xd4fe0cef, SUC_lexicon_tags_87 },
    { 0x94fe0cf1, SUC_lexicon_tags_12 },
    { 0xbdde0cf1, SUC_lexicon_tags_38 },
    { 0x2cfe0cf2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x8e7e0cf5, SUC_lexicon_tags_29 },
    { 0x987e0cf6, SUC_lexicon_tags_66 },
    { 0x801e0cf5, SUC_lexicon_tags_6 },
    { 0xb4de0cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x833e0cfa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x399e0cfd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fe0d03, SUC_lexicon_tags_8 },
    { 0x731e0d04, SUC_lexicon_tags_17 },
    { 0xb2de0d03, SUC_lexicon_tags_11 },
    { 0x4c1e0d06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7e0d0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa61e0d10, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5d3e0d12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fe0d15, SUC_lexicon_tags_7 },
    { 0xea5e0d15, SUC_lexicon_tags_19 },
    { 0x323e0d17, SUC_lexicon_tags_26 },
    { 0x7c3e0d18, SUC_lexicon_tags_2 },
    { 0xb5be0d18, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207e0d1d, SUC_lexicon_tags_151 },
    { 0x3e9e0d1e, SUC_lexicon_tags_60 },
    { 0x145e0d1f, SUC_lexicon_tags_2 },
    { 0xd3e0d1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27fe0d22, SUC_lexicon_tags_37 },
    { 0xff1e0d23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bde0d26, SUC_lexicon_tags_12 },
    { 0xd75e0d26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x173e0d29, SUC_lexicon_tags_3 },
    { 0x6bbe0d29, SUC_lexicon_tags_11 },
    { 0x4be0d2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2afe0d2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835e0d32, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2cde0d34, SUC_lexicon_tags_7 },
    { 0x7fe0d34, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8bde0d37, SUC_lexicon_tags_11 },
    { 0x2b7e0d37, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9e0d3c, SUC_lexicon_tags_3 },
    { 0x86fe0d3d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xecfe0d3f, SUC_lexicon_tags_2 },
    { 0x415e0d40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fbe0d42, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd09e0d44, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597e0d4c, SUC_lexicon_tags_12 },
    { 0xc17e0d4c, SUC_lexicon_tags_15 },
    { 0x437e0d4c, SUC_lexicon_tags_12 },
    { 0xcffe0d4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91e0d53, SUC_lexicon_tags_37 },
    { 0x283e0d53, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x14fe0d57, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4bde0d59, SUC_lexicon_tags_4 },
    { 0xb65e0d5a, SUC_lexicon_tags_14 },
    { 0xbe7e0d5b, SUC_lexicon_tags_2 },
    { 0xabbe0d5c, SUC_lexicon_tags_37 },
    { 0xf8fe0d5d, SUC_lexicon_tags_17 },
    { 0xb63e0d5d, SUC_lexicon_tags_11 },
    { 0x74de0d5f, SUC_lexicon_tags_34 },
    { 0x3c7e0d59, SUC_lexicon_tags_95 },
    { 0xcabe0d61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfe0d64, SUC_lexicon_tags_82 },
    { 0x859e0d65, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbe0d68, SUC_lexicon_tags_47 },
    { 0x73de0d69, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x125e0d6b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe0d6f, SUC_lexicon_tags_97 },
    { 0x6a3e0d70, SUC_lexicon_tags_69 },
    { 0x2abe0d71, SUC_lexicon_tags_27 },
    { 0xc3fe0d70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbe0d7f, SUC_lexicon_tags_2 },
    { 0x34fe0d80, SUC_lexicon_tags_1 },
    { 0xb31e0d7f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91e0d84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x407e0d86, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x571e0d89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743e0d8f, SUC_lexicon_tags_318 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315e0d99, SUC_lexicon_tags_12 },
    { 0x165e0d9a, SUC_lexicon_tags_0 },
    { 0x85be0d9b, SUC_lexicon_tags_1 },
    { 0xb17e0d9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x663e0d9e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735e0da5, SUC_lexicon_tags_37 },
    { 0x305e0da6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6c1e0da8, SUC_lexicon_tags_25 },
    { 0x29de0da8, SUC_lexicon_tags_43 },
    { 0x423e0daa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3e0dad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1e0db2, SUC_lexicon_tags_19 },
    { 0x145e0db3, SUC_lexicon_tags_8 },
    { 0x9c9e0db2, SUC_lexicon_tags_37 },
    { 0x9c9e0db4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x389e0db8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88de0dbf, SUC_lexicon_tags_17 },
    { 0xc15e0dbf, SUC_lexicon_tags_17 },
    { 0x83de0dbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1e0dca, SUC_lexicon_tags_57 },
    { 0x3f1e0dca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9e0dce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555e0dd3, SUC_lexicon_tags_46 },
    { 0x1ffe0dd4, SUC_lexicon_tags_17 },
    { 0x905e0dd5, SUC_lexicon_tags_17 },
    { 0x429e0dd6, SUC_lexicon_tags_15 },
    { 0xb7be0dd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x177e0dda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc25e0ddc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x587e0ddf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9fe0de1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361e0de6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17e0dea, SUC_lexicon_tags_48 },
    { 0xf1de0deb, SUC_lexicon_tags_37 },
    { 0xf31e0deb, SUC_lexicon_tags_12 },
    { 0xa17e0ded, SUC_lexicon_tags_19 },
    { 0x97de0dee, SUC_lexicon_tags_11 },
    { 0x353e0deb, SUC_lexicon_tags_198 },
    { 0x15de0df0, SUC_lexicon_tags_30 },
    { 0x719e0df1, SUC_lexicon_tags_2 },
    { 0x5b3e0df2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9e0dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79e0dff, SUC_lexicon_tags_1 },
    { 0xcd3e0e00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50be0e05, SUC_lexicon_tags_37 },
    { 0xb2be0e05, SUC_lexicon_tags_12 },
    { 0xf71e0e06, SUC_lexicon_tags_7 },
    { 0xa55e0e08, SUC_lexicon_tags_0 },
    { 0xfd3e0e09, SUC_lexicon_tags_17 },
    { 0xbefe0e0a, SUC_lexicon_tags_8 },
    { 0x65de0e0b, SUC_lexicon_tags_9 },
    { 0x447e0e0c, SUC_lexicon_tags_37 },
    { 0xba9e0e08, SUC_lexicon_tags_25 },
    { 0xfe0e08, SUC_lexicon_tags_11 },
    { 0x569e0e0f, SUC_lexicon_tags_6 },
    { 0x80fe0e10, SUC_lexicon_tags_48 },
    { 0x4ffe0e11, SUC_lexicon_tags_17 },
    { 0x947e0e0f, SUC_lexicon_tags_9 },
    { 0x391e0e10, SUC_lexicon_tags_17 },
    { 0x44fe0e0e, SUC_lexicon_tags_12 },
    { 0x979e0e15, SUC_lexicon_tags_42 },
    { 0xb5e0e12, SUC_lexicon_tags_17 },
    { 0xd33e0e17, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcb3e0e19, SUC_lexicon_tags_30 },
    { 0xdd9e0e19, SUC_lexicon_tags_17 },
    { 0x2abe0e1a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffe0e21, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fe0e27, SUC_lexicon_tags_25 },
    { 0x551e0e28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x78be0e2a, SUC_lexicon_tags_60 },
    { 0xa97e0e2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x419e0e2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8cfe0e2f, SUC_lexicon_tags_19 },
    { 0xe73e0e30, SUC_lexicon_tags_17 },
    { 0x35fe0e2f, SUC_lexicon_tags_13 },
    { 0x9dbe0e31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415e0e36, SUC_lexicon_tags_29 },
    { 0x295e0e37, SUC_lexicon_tags_37 },
    { 0x9cde0e38, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e0e3b, SUC_lexicon_tags_47 },
    { 0xd4be0e3c, SUC_lexicon_tags_15 },
    { 0xeefe0e3c, SUC_lexicon_tags_1 },
    { 0xaade0e3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99e0e43, SUC_lexicon_tags_47 },
    { 0x44de0e44, SUC_lexicon_tags_7 },
    { 0x227e0e45, SUC_lexicon_tags_1 },
    { 0xef9e0e45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97fe0e4c, SUC_lexicon_tags_179 },
    { 0x871e0e4d, SUC_lexicon_tags_44 },
    { 0x671e0e4e, SUC_lexicon_tags_2 },
    { 0x28de0e4e, SUC_lexicon_tags_6 },
    { 0xdcfe0e50, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755e0e5b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebe0e5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbe0e61, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc3e0e63, SUC_lexicon_tags_11 },
    { 0x6a7e0e63, SUC_lexicon_tags_29 },
    { 0xfc9e0e65, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x793e0e67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbe0e6b, SUC_lexicon_tags_0 },
    { 0xf05e0e6c, SUC_lexicon_tags_49 },
    { 0xa31e0e6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x445e0e6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x68be0e72, SUC_lexicon_tags_11 },
    { 0xb1fe0e73, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x539e0e75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f7e0e77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x505e0e79, SUC_lexicon_tags_1 },
    { 0xd2fe0e7a, SUC_lexicon_tags_23 },
    { 0x38de0e7a, SUC_lexicon_tags_11 },
    { 0x6cbe0e79, SUC_lexicon_tags_38 },
    { 0x30fe0e7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x91fe0e7f, SUC_lexicon_tags_2 },
    { 0xfdfe0e7f, SUC_lexicon_tags_0 },
    { 0x533e0e81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5d7e0e83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47e0e87, SUC_lexicon_tags_14 },
    { 0xd05e0e88, SUC_lexicon_tags_9 },
    { 0xe0de0e89, SUC_lexicon_tags_23 },
    { 0xf8be0e87, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7e0e96, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x121e0e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efe0e9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc43e0e9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05e0ea1, SUC_lexicon_tags_39 },
    { 0x753e0ea2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfde0ea4, SUC_lexicon_tags_37 },
    { 0x98be0ea5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e5e0ea7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ede0ea9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28be0ead, SUC_lexicon_tags_17 },
    { 0xb83e0eae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de0eb1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2fbe0eb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfb3e0eb5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x609e0eb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf1e0eba, SUC_lexicon_tags_37 },
    { 0x86fe0eba, SUC_lexicon_tags_11 },
    { 0xfbfe0eba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x133e0ebe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cbe0ec0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43e0ec3, SUC_lexicon_tags_37 },
    { 0x57e0ec4, SUC_lexicon_tags_0 },
    { 0xedfe0ec5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735e0ec9, SUC_lexicon_tags_42 },
    { 0x4a1e0ec9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21e0ecc, SUC_lexicon_tags_92 },
    { 0x44be0ecd, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51e0ed5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfde0ed9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7de0edc, SUC_lexicon_tags_29 },
    { 0xf7be0edc, SUC_lexicon_tags_13 },
    { 0xf83e0edd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e0ee1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfe0ee3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9e0ee9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31e0eec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717e0ef1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcbbe0ef3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7e0ef8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7e0efd, SUC_lexicon_tags_2 },
    { 0xa15e0efe, SUC_lexicon_tags_13 },
    { 0xfbbe0eff, SUC_lexicon_tags_2 },
    { 0xdfe0eff, SUC_lexicon_tags_11 },
    { 0x105e0f01, SUC_lexicon_tags_12 },
    { 0x3fbe0f01, SUC_lexicon_tags_15 },
    { 0x875e0f01, SUC_lexicon_tags_37 },
    { 0x98fe0f03, SUC_lexicon_tags_45 },
    { 0x2fbe0eff, SUC_lexicon_tags_8 },
    { 0x9a3e0f06, SUC_lexicon_tags_17 },
    { 0xdc3e0f06, SUC_lexicon_tags_7 },
    { 0xa91e0f07, SUC_lexicon_tags_37 },
    { 0xe9fe0f06, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x2bde0f0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3e0f10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcafe0f12, SUC_lexicon_tags_43 },
    { 0x243e0f12, SUC_lexicon_tags_37 },
    { 0x9d9e0f13, SUC_lexicon_tags_35 },
    { 0x2cfe0f12, SUC_lexicon_tags_11 },
    { 0x409e0f12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x935e0f19, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65e0f1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6d1e0f1e, SUC_lexicon_tags_1 },
    { 0xe4be0f1f, SUC_lexicon_tags_19 },
    { 0x869e0f1f, SUC_lexicon_tags_71 },
    { 0xe83e0f1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805e0f26, SUC_lexicon_tags_66 },
    { 0x6fe0f27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3de0f2b, SUC_lexicon_tags_7 },
    { 0x33fe0f2b, SUC_lexicon_tags_45 },
    { 0x31be0f2b, SUC_lexicon_tags_18 },
    { 0xfbde0f2e, SUC_lexicon_tags_0 },
    { 0xdb7e0f2f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x409e0f31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3e0f3b, SUC_lexicon_tags_17 },
    { 0xa05e0f3c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdfde0f3e, SUC_lexicon_tags_11 },
    { 0x2dfe0f3f, SUC_lexicon_tags_33 },
    { 0x5e1e0f3f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe0f43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fe0f48, SUC_lexicon_tags_29 },
    { 0xfa3e0f49, SUC_lexicon_tags_37 },
    { 0x501e0f49, SUC_lexicon_tags_66 },
    { 0x1bfe0f4b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be0f4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7e0f53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9e0f56, SUC_lexicon_tags_65 },
    { 0xbffe0f57, SUC_lexicon_tags_1 },
    { 0x327e0f58, SUC_lexicon_tags_113 },
    { 0x517e0f56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34fe0f5b, SUC_lexicon_tags_25 },
    { 0xb69e0f5b, SUC_lexicon_tags_96 },
    { 0xc09e0f5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1e0f62, SUC_lexicon_tags_25 },
    { 0x17be0f62, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x92de0f65, SUC_lexicon_tags_15 },
    { 0x4d9e0f66, SUC_lexicon_tags_11 },
    { 0xdfde0f67, SUC_lexicon_tags_14 },
    { 0x671e0f68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x253e0f6b, SUC_lexicon_tags_17 },
    { 0xaefe0f6c, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x8ade0f6e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4e1e0f70, SUC_lexicon_tags_7 },
    { 0x525e0f71, SUC_lexicon_tags_12 },
    { 0x653e0f71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67e0f74, SUC_lexicon_tags_15 },
    { 0x4f1e0f75, SUC_lexicon_tags_11 },
    { 0xc7fe0f76, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99e0f7a, SUC_lexicon_tags_17 },
    { 0xfede0f7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbe0f7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed5e0f81, SUC_lexicon_tags_40 },
    { 0xa4be0f81, SUC_lexicon_tags_7 },
    { 0x6f5e0f83, SUC_lexicon_tags_38 },
    { 0x871e0f82, SUC_lexicon_tags_1 },
    { 0xf79e0f81, SUC_lexicon_tags_12 },
    { 0x91e0f86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5e0f8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf47e0f90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x619e0f92, SUC_lexicon_tags_43 },
    { 0xd9e0f93, SUC_lexicon_tags_15 },
    { 0xe59e0f94, SUC_lexicon_tags_8 },
    { 0xf6fe0f93, SUC_lexicon_tags_63 },
    { 0x409e0f95, SUC_lexicon_tags_28 },
    { 0xe83e0f97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1e0f9c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5e0f9f, SUC_lexicon_tags_29 },
    { 0x21de0f9f, SUC_lexicon_tags_17 },
    { 0xebde0fa0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xeb7e0fa3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53e0fae, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc09e0fb0, SUC_lexicon_tags_39 },
    { 0xea9e0fb1, SUC_lexicon_tags_46 },
    { 0x4c9e0fb1, SUC_lexicon_tags_64 },
    { 0x2bde0fb3, SUC_lexicon_tags_47 },
    { 0x80be0fb4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fe0fb8, SUC_lexicon_tags_23 },
    { 0x37e0fb9, SUC_lexicon_tags_170 },
    { 0xbd1e0fb8, SUC_lexicon_tags_37 },
    { 0x533e0fbb, SUC_lexicon_tags_2 },
    { 0xe91e0fbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fe0fc0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x143e0fc3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefe0fca, SUC_lexicon_tags_2 },
    { 0xdafe0fcb, SUC_lexicon_tags_60 },
    { 0x2bfe0fcb, SUC_lexicon_tags_25 },
    { 0x97fe0fcd, SUC_lexicon_tags_68 },
    { 0x94be0fcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1e0fd3, SUC_lexicon_tags_11 },
    { 0x9bfe0fd4, SUC_lexicon_tags_26 },
    { 0x7e5e0fd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90be0fda, SUC_lexicon_tags_7 },
    { 0x23e0fda, SUC_lexicon_tags_33 },
    { 0x7c3e0fdb, SUC_lexicon_tags_56 },
    { 0x931e0fdd, SUC_lexicon_tags_7 },
    { 0x4ede0fde, SUC_lexicon_tags_26 },
    { 0x195e0fdf, SUC_lexicon_tags_11 },
    { 0xcdde0fe0, SUC_lexicon_tags_39 },
    { 0xb53e0fe1, SUC_lexicon_tags_11 },
    { 0x54be0fdd, SUC_lexicon_tags_61 },
    { 0x731e0fe3, SUC_lexicon_tags_33 },
    { 0x895e0fde, SUC_lexicon_tags_1 },
    { 0xf5de0fe0, SUC_lexicon_tags_12 },
    { 0xe1fe0fe0, SUC_lexicon_tags_1 },
    { 0x9b1e0fe3, SUC_lexicon_tags_8 },
    { 0xfade0fdf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa35e0fea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469e0fee, SUC_lexicon_tags_104 },
    { 0xd37e0fef, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969e0ff3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781e0ff7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7e0ffc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa07e0ffe, SUC_lexicon_tags_15 },
    { 0x877e0ffe, SUC_lexicon_tags_1 },
    { 0x8fe1000, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171e1006, SUC_lexicon_tags_7 },
    { 0xcb5e1007, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x931e100a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x177e100c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x15be100e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x825e1010, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2c1e1012, SUC_lexicon_tags_25 },
    { 0x50fe1012, SUC_lexicon_tags_10 },
    { 0xd23e1014, SUC_lexicon_tags_37 },
    { 0x9b5e1014, SUC_lexicon_tags_55 },
    { 0x689e1012, SUC_lexicon_tags_0 },
    { 0xca5e1017, SUC_lexicon_tags_2 },
    { 0x66be1014, SUC_lexicon_tags_11 },
    { 0x153e1019, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc75e101b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33e101f, SUC_lexicon_tags_17 },
    { 0x9e101f, SUC_lexicon_tags_1 },
    { 0x89be1021, SUC_lexicon_tags_25 },
    { 0x89e1021, SUC_lexicon_tags_5 },
    { 0xf39e101f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3de102c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dde102f, SUC_lexicon_tags_9 },
    { 0xa3e102f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847e1038, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x2f7e103a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x693e103d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959e1045, SUC_lexicon_tags_39 },
    { 0xfabe1046, SUC_lexicon_tags_2 },
    { 0x1e7e1047, SUC_lexicon_tags_17 },
    { 0x7f1e1046, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5de1050, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13de1056, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e105b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1e105e, SUC_lexicon_tags_17 },
    { 0x821e105f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3e1062, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x301e1065, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x807e1067, SUC_lexicon_tags_16 },
    { 0x143e1068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73be106c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45e1074, SUC_lexicon_tags_149 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3e107b, SUC_lexicon_tags_7 },
    { 0x787e107c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51be107e, SUC_lexicon_tags_15 },
    { 0xaede107f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40be1081, SUC_lexicon_tags_68 },
    { 0xd8de1082, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9e1086, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5e108c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x77fe108e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8c7e1090, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbde1092, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29e1096, SUC_lexicon_tags_19 },
    { 0x149e1097, SUC_lexicon_tags_18 },
    { 0xc07e1097, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafde109b, SUC_lexicon_tags_2 },
    { 0x37de109b, SUC_lexicon_tags_11 },
    { 0x939e109d, SUC_lexicon_tags_17 },
    { 0x8d1e109e, SUC_lexicon_tags_43 },
    { 0xd37e109c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61de10a3, SUC_lexicon_tags_31 },
    { 0x423e10a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13e10a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75e10ab, SUC_lexicon_tags_17 },
    { 0xae5e10ab, SUC_lexicon_tags_15 },
    { 0xef1e10ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313e10b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80de10b7, SUC_lexicon_tags_11 },
    { 0x9dde10b7, SUC_lexicon_tags_37 },
    { 0xc41e10b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64de10bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827e10c2, SUC_lexicon_tags_11 },
    { 0xc07e10c2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293e10cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ede10ce, SUC_lexicon_tags_11 },
    { 0x679e10cf, SUC_lexicon_tags_10 },
    { 0xd13e10ce, SUC_lexicon_tags_30 },
    { 0x831e10d1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1e10d8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x307e10db, SUC_lexicon_tags_37 },
    { 0xb57e10db, SUC_lexicon_tags_17 },
    { 0x8fbe10dd, SUC_lexicon_tags_9 },
    { 0x9f5e10de, SUC_lexicon_tags_14 },
    { 0xcb7e10dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbe10e4, SUC_lexicon_tags_27 },
    { 0xc87e10e5, SUC_lexicon_tags_39 },
    { 0x60de10e6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7e10e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6fe10eb, SUC_lexicon_tags_37 },
    { 0x941e10ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393e10f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbe10f4, SUC_lexicon_tags_31 },
    { 0x17fe10f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe37e10f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41e10fa, SUC_lexicon_tags_11 },
    { 0x9cbe10fb, SUC_lexicon_tags_0 },
    { 0xed7e10fc, SUC_lexicon_tags_25 },
    { 0x153e10fc, SUC_lexicon_tags_29 },
    { 0xf37e10fd, SUC_lexicon_tags_9 },
    { 0x119e10ff, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5e1103, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x63be1105, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929e110c, SUC_lexicon_tags_15 },
    { 0x347e110c, SUC_lexicon_tags_31 },
    { 0xcb5e110e, SUC_lexicon_tags_56 },
    { 0x847e110f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511e1115, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd97e1117, SUC_lexicon_tags_17 },
    { 0x1ebe1118, SUC_lexicon_tags_25 },
    { 0x8e5e1119, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21e111d, SUC_lexicon_tags_6 },
    { 0x3dde111d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9e1122, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xfa1e1124, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd53e1127, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddde112d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x8afe112f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x52de1131, SUC_lexicon_tags_15 },
    { 0xf5e1132, SUC_lexicon_tags_15 },
    { 0x645e1133, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7e113d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09e1142, SUC_lexicon_tags_17 },
    { 0xd4de1143, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87e1147, SUC_lexicon_tags_11 },
    { 0x183e1148, SUC_lexicon_tags_11 },
    { 0x987e1149, SUC_lexicon_tags_11 },
    { 0x1c9e1147, SUC_lexicon_tags_47 },
    { 0xe6be114b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1e114e, SUC_lexicon_tags_43 },
    { 0xadde114f, SUC_lexicon_tags_21 },
    { 0xe21e114f, SUC_lexicon_tags_27 },
    { 0x5be1151, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x697e1154, SUC_lexicon_tags_15 },
    { 0xb31e1154, SUC_lexicon_tags_2 },
    { 0x2bde1154, SUC_lexicon_tags_1 },
    { 0x71e1154, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc9fe1159, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e115e, SUC_lexicon_tags_30 },
    { 0x9efe115f, SUC_lexicon_tags_10 },
    { 0x32de115f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x851e1163, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabe1168, SUC_lexicon_tags_87 },
    { 0xd89e1168, SUC_lexicon_tags_43 },
    { 0xb89e1168, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679e1171, SUC_lexicon_tags_12 },
    { 0xf4be1171, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55e1176, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359e117a, SUC_lexicon_tags_98 },
    { 0x143e117b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa81e117d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd47e117f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fde1183, SUC_lexicon_tags_63 },
    { 0xd47e1183, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbe118a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff5e118c, SUC_lexicon_tags_47 },
    { 0x123e118d, SUC_lexicon_tags_15 },
    { 0xfdbe118e, SUC_lexicon_tags_12 },
    { 0x8c3e118f, SUC_lexicon_tags_27 },
    { 0x261e118f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23e1198, SUC_lexicon_tags_30 },
    { 0x72be1199, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9e11a1, SUC_lexicon_tags_28 },
    { 0x7f7e11a2, SUC_lexicon_tags_14 },
    { 0xbe1e11a3, SUC_lexicon_tags_151 },
    { 0xef9e11a4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fe11a7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91e11ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x671e11ae, SUC_lexicon_tags_45 },
    { 0xf15e11af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3cfe11b1, SUC_lexicon_tags_43 },
    { 0xb19e11b2, SUC_lexicon_tags_37 },
    { 0x8ede11b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7de11b5, SUC_lexicon_tags_17 },
    { 0xfe1e11b6, SUC_lexicon_tags_29 },
    { 0x753e11b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3e11ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x577e11bc, SUC_lexicon_tags_37 },
    { 0xc6de11bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcde11c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd91e11c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdde11c4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0de11c9, SUC_lexicon_tags_0 },
    { 0x349e11ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95e11d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x137e11d4, SUC_lexicon_tags_63 },
    { 0x43de11d4, SUC_lexicon_tags_5 },
    { 0xa61e11d6, SUC_lexicon_tags_12 },
    { 0xa07e11d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x66de11d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e3e11db, SUC_lexicon_tags_11 },
    { 0x785e11dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61de11e0, SUC_lexicon_tags_25 },
    { 0x77fe11e0, SUC_lexicon_tags_2 },
    { 0x6c7e11e2, SUC_lexicon_tags_1 },
    { 0x731e11e2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7e11e7, SUC_lexicon_tags_37 },
    { 0x5a3e11e8, SUC_lexicon_tags_6 },
    { 0x3abe11e9, SUC_lexicon_tags_14 },
    { 0xa5e11e8, SUC_lexicon_tags_25 },
    { 0xf95e11ea, SUC_lexicon_tags_11 },
    { 0xb75e11ec, SUC_lexicon_tags_34 },
    { 0xc57e11ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767e11f1, SUC_lexicon_tags_25 },
    { 0xbbe11f2, SUC_lexicon_tags_2 },
    { 0xa29e11f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55e11fa, SUC_lexicon_tags_25 },
    { 0xcede11fb, SUC_lexicon_tags_48 },
    { 0xa63e11fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fe1206, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6be120d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfcde120f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed3e1211, SUC_lexicon_tags_21 },
    { 0xd9be1212, SUC_lexicon_tags_37 },
    { 0x3ade1213, SUC_lexicon_tags_1 },
    { 0x96fe1214, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4bfe1216, SUC_lexicon_tags_11 },
    { 0xadbe1216, SUC_lexicon_tags_12 },
    { 0x333e1217, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x6b3e121a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc87e121c, SUC_lexicon_tags_37 },
    { 0xf0fe121d, SUC_lexicon_tags_15 },
    { 0x741e121e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1e1221, SUC_lexicon_tags_14 },
    { 0xd4fe1221, SUC_lexicon_tags_37 },
    { 0xebbe1223, SUC_lexicon_tags_25 },
    { 0xad3e1223, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9ffe1226, SUC_lexicon_tags_12 },
    { 0xcbde1227, SUC_lexicon_tags_37 },
    { 0x85de1228, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x593e122b, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7e122e, SUC_lexicon_tags_31 },
    { 0x96de122f, SUC_lexicon_tags_60 },
    { 0x16be122f, SUC_lexicon_tags_37 },
    { 0xc45e1231, SUC_lexicon_tags_25 },
    { 0xc25e1232, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41e1239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9e123c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7e1242, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49e124a, SUC_lexicon_tags_4 },
    { 0xf75e124a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2b5e124d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771e1251, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ede1259, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7e125c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1e1261, SUC_lexicon_tags_27 },
    { 0x3de1262, SUC_lexicon_tags_37 },
    { 0xea9e1261, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x615e1265, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x56be1268, SUC_lexicon_tags_91 },
    { 0xdd1e1269, SUC_lexicon_tags_7 },
    { 0x915e1269, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931e126e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x14de1270, SUC_lexicon_tags_15 },
    { 0x351e1271, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fe1276, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e127b, SUC_lexicon_tags_0 },
    { 0x5dfe127c, SUC_lexicon_tags_37 },
    { 0xb4de127d, SUC_lexicon_tags_2 },
    { 0xf87e127e, SUC_lexicon_tags_16 },
    { 0xfd3e127f, SUC_lexicon_tags_65 },
    { 0x2ffe127d, SUC_lexicon_tags_45 },
    { 0x16fe127d, SUC_lexicon_tags_19 },
    { 0x16de127b, SUC_lexicon_tags_25 },
    { 0xa05e127b, SUC_lexicon_tags_19 },
    { 0x589e1281, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbe1286, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09e128c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5e1292, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fe1295, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcade1299, SUC_lexicon_tags_1 },
    { 0xa91e1299, SUC_lexicon_tags_37 },
    { 0x435e129a, SUC_lexicon_tags_34 },
    { 0x9d1e129c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e129f, SUC_lexicon_tags_46 },
    { 0x73be129f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x50fe12a3, SUC_lexicon_tags_45 },
    { 0x5c5e12a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333e12a8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x133e12aa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb29e12ac, SUC_lexicon_tags_12 },
    { 0xe7de12ad, SUC_lexicon_tags_43 },
    { 0x5ede12ae, SUC_lexicon_tags_27 },
    { 0xb8be12af, SUC_lexicon_tags_2 },
    { 0xd39e12b0, SUC_lexicon_tags_37 },
    { 0x907e12b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e12b9, SUC_lexicon_tags_15 },
    { 0x621e12ba, SUC_lexicon_tags_1 },
    { 0xf39e12bb, SUC_lexicon_tags_45 },
    { 0xfd5e12ba, SUC_lexicon_tags_47 },
    { 0xc8fe12ba, SUC_lexicon_tags_11 },
    { 0xb41e12be, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63e12c2, SUC_lexicon_tags_3 },
    { 0x343e12c3, SUC_lexicon_tags_17 },
    { 0x667e12c4, SUC_lexicon_tags_32 },
    { 0x7de12c5, SUC_lexicon_tags_37 },
    { 0x63e12c2, SUC_lexicon_tags_65 },
    { 0xe3e12c7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81e12ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfe12cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffe12d3, SUC_lexicon_tags_17 },
    { 0xfabe12d4, SUC_lexicon_tags_63 },
    { 0x935e12d4, SUC_lexicon_tags_28 },
    { 0xf4be12d6, SUC_lexicon_tags_39 },
    { 0xf4de12d4, SUC_lexicon_tags_38 },
    { 0xe71e12d8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8be12df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37fe12e1, SUC_lexicon_tags_1 },
    { 0x237e12e2, SUC_lexicon_tags_15 },
    { 0x71e12e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4de12e7, SUC_lexicon_tags_28 },
    { 0x577e12e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33de12f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fe12f3, SUC_lexicon_tags_184 },
    { 0x747e12f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb17e12f6, SUC_lexicon_tags_11 },
    { 0xc1de12f7, SUC_lexicon_tags_47 },
    { 0x465e12f7, SUC_lexicon_tags_678 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9e12fb, SUC_lexicon_tags_65 },
    { 0x22be12fc, SUC_lexicon_tags_27 },
    { 0x46be12fc, SUC_lexicon_tags_74 },
    { 0x493e12fd, SUC_lexicon_tags_1 },
    { 0x563e12ff, SUC_lexicon_tags_37 },
    { 0x9c9e1300, SUC_lexicon_tags_0 },
    { 0x6e1e12fd, SUC_lexicon_tags_37 },
    { 0xdefe1301, SUC_lexicon_tags_15 },
    { 0x597e1303, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9e130c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abe1311, SUC_lexicon_tags_43 },
    { 0xe6de1312, SUC_lexicon_tags_437 },
    { 0xa05e1312, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99fe1315, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x94de1318, SUC_lexicon_tags_26 },
    { 0x2d1e1318, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xce7e131c, SUC_lexicon_tags_7 },
    { 0x64de131d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x81e1320, SUC_lexicon_tags_37 },
    { 0x725e1321, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x68de1324, SUC_lexicon_tags_25 },
    { 0xad1e1324, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86de132a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf61e132d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x899e1330, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505e133a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3e133f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebde1343, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc41e1345, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xccde1348, SUC_lexicon_tags_7 },
    { 0x529e1349, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8fe134b, SUC_lexicon_tags_36 },
    { 0x4d3e134b, SUC_lexicon_tags_47 },
    { 0x32de134d, SUC_lexicon_tags_7 },
    { 0xd51e134d, SUC_lexicon_tags_11 },
    { 0xf6de134f, SUC_lexicon_tags_37 },
    { 0xe5fe134f, SUC_lexicon_tags_25 },
    { 0x675e134f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5e1357, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3e135c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969e1361, SUC_lexicon_tags_7 },
    { 0x941e1362, SUC_lexicon_tags_15 },
    { 0x50de1363, SUC_lexicon_tags_45 },
    { 0x789e1364, SUC_lexicon_tags_551 },
    { 0x5e1364, SUC_lexicon_tags_25 },
    { 0x67be1366, SUC_lexicon_tags_77 },
    { 0xd0de1365, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x985e1369, SUC_lexicon_tags_11 },
    { 0x4a9e1369, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3e136e, SUC_lexicon_tags_0 },
    { 0x645e136f, SUC_lexicon_tags_33 },
    { 0x273e1370, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5ebe1372, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbe1375, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x917e1378, SUC_lexicon_tags_11 },
    { 0xf6fe1378, SUC_lexicon_tags_34 },
    { 0x511e137a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4be137d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdcfe137f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52de1383, SUC_lexicon_tags_47 },
    { 0xa0be1384, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x597e1386, SUC_lexicon_tags_43 },
    { 0xf33e1387, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96de138b, SUC_lexicon_tags_21 },
    { 0xd01e138c, SUC_lexicon_tags_25 },
    { 0x63de138d, SUC_lexicon_tags_23 },
    { 0x653e138e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e1393, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277e1397, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8be139f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fe13a9, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbe13ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547e13b4, SUC_lexicon_tags_1 },
    { 0x4d1e13b5, SUC_lexicon_tags_238 },
    { 0x8fde13b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x667e13b8, SUC_lexicon_tags_9 },
    { 0x1be13b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x991e13bc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfe13c1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe13c4, SUC_lexicon_tags_2 },
    { 0xd5de13c5, SUC_lexicon_tags_12 },
    { 0x13de13c5, SUC_lexicon_tags_1 },
    { 0xc19e13c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x223e13cf, SUC_lexicon_tags_17 },
    { 0x9efe13d0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc23e13d2, SUC_lexicon_tags_37 },
    { 0x5d1e13d3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x15de13d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3e13dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59be13e4, SUC_lexicon_tags_141 },
    { 0x949e13e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85be13ec, SUC_lexicon_tags_17 },
    { 0xc05e13ed, SUC_lexicon_tags_17 },
    { 0x24de13ee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867e13f2, SUC_lexicon_tags_7 },
    { 0x59fe13f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781e13f7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bde13fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18de13fd, SUC_lexicon_tags_27 },
    { 0x301e13fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdde1400, SUC_lexicon_tags_25 },
    { 0x8dfe1401, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x455e1403, SUC_lexicon_tags_153 },
    { 0, NULL },
    { 0x257e1405, SUC_lexicon_tags_43 },
    { 0x705e1406, SUC_lexicon_tags_34 },
    { 0xc7fe1407, SUC_lexicon_tags_15 },
    { 0x9f9e1408, SUC_lexicon_tags_1 },
    { 0xb8be1408, SUC_lexicon_tags_2 },
    { 0x5be1409, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x57e140c, SUC_lexicon_tags_10 },
    { 0xecfe140d, SUC_lexicon_tags_25 },
    { 0x8abe140e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9e1411, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1bbe1413, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a3e1415, SUC_lexicon_tags_4 },
    { 0x649e1416, SUC_lexicon_tags_37 },
    { 0xb25e1417, SUC_lexicon_tags_7 },
    { 0x9ade1418, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x465e141a, SUC_lexicon_tags_23 },
    { 0xca9e141a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5e141e, SUC_lexicon_tags_127 },
    { 0x6b1e141e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707e1423, SUC_lexicon_tags_45 },
    { 0xf15e1424, SUC_lexicon_tags_0 },
    { 0x799e1425, SUC_lexicon_tags_0 },
    { 0x47de1425, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18de1428, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7e1430, SUC_lexicon_tags_37 },
    { 0xcb3e1430, SUC_lexicon_tags_27 },
    { 0x711e1431, SUC_lexicon_tags_12 },
    { 0x1ebe1433, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1e1438, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4fe143a, SUC_lexicon_tags_15 },
    { 0xb4fe143a, SUC_lexicon_tags_25 },
    { 0x25be143a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfe143f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97de144a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757e144f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6abe1451, SUC_lexicon_tags_29 },
    { 0x981e1452, SUC_lexicon_tags_37 },
    { 0x183e1453, SUC_lexicon_tags_17 },
    { 0xf7fe1454, SUC_lexicon_tags_37 },
    { 0x90be1454, SUC_lexicon_tags_25 },
    { 0x947e1454, SUC_lexicon_tags_17 },
    { 0x86fe1457, SUC_lexicon_tags_2 },
    { 0x257e1458, SUC_lexicon_tags_11 },
    { 0x8c1e1456, SUC_lexicon_tags_7 },
    { 0x457e145a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bde1463, SUC_lexicon_tags_25 },
    { 0xefe1463, SUC_lexicon_tags_25 },
    { 0x777e1463, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469e1469, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7be146c, SUC_lexicon_tags_27 },
    { 0x7a5e146d, SUC_lexicon_tags_17 },
    { 0x53be146d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1d1e1470, SUC_lexicon_tags_45 },
    { 0x64be1471, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fe1477, SUC_lexicon_tags_17 },
    { 0x4c5e1478, SUC_lexicon_tags_17 },
    { 0x1b3e1479, SUC_lexicon_tags_31 },
    { 0x74de1479, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99e147e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x83de1480, SUC_lexicon_tags_87 },
    { 0xeebe1481, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x167e1484, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x76be1486, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x623e1489, SUC_lexicon_tags_66 },
    { 0x45e1489, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x831e148c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdb7e148e, SUC_lexicon_tags_12 },
    { 0x2a5e148e, SUC_lexicon_tags_39 },
    { 0xe7de148f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1be1493, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1496, SUC_lexicon_tags_17 },
    { 0xd35e1497, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfe149c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1fde149e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe14a2, SUC_lexicon_tags_26 },
    { 0x613e14a2, SUC_lexicon_tags_17 },
    { 0x321e14a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513e14aa, SUC_lexicon_tags_15 },
    { 0xd7fe14ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11e14ae, SUC_lexicon_tags_0 },
    { 0x299e14af, SUC_lexicon_tags_287 },
    { 0x715e14b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2afe14b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1de14b7, SUC_lexicon_tags_25 },
    { 0xc11e14b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x813e14ba, SUC_lexicon_tags_0 },
    { 0x5dfe14bb, SUC_lexicon_tags_30 },
    { 0x13de14bc, SUC_lexicon_tags_15 },
    { 0x4c9e14bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41be14c3, SUC_lexicon_tags_343 },
    { 0x5f1e14c4, SUC_lexicon_tags_7 },
    { 0xa69e14c5, SUC_lexicon_tags_23 },
    { 0x219e14c6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fe14d0, SUC_lexicon_tags_25 },
    { 0x801e14d0, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95e14d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1cfe14d6, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599e14db, SUC_lexicon_tags_2 },
    { 0x81fe14db, SUC_lexicon_tags_7 },
    { 0xdebe14dd, SUC_lexicon_tags_54 },
    { 0x4d3e14dd, SUC_lexicon_tags_30 },
    { 0xec1e14df, SUC_lexicon_tags_11 },
    { 0x6c7e14db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xefbe14e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68fe14e8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6cfe14ea, SUC_lexicon_tags_17 },
    { 0x543e14ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf61e14ee, SUC_lexicon_tags_12 },
    { 0x411e14ef, SUC_lexicon_tags_0 },
    { 0x85de14ef, SUC_lexicon_tags_7 },
    { 0x945e14f0, SUC_lexicon_tags_11 },
    { 0x7d1e14f2, SUC_lexicon_tags_65 },
    { 0xafe14f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3e14f8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xc11e14fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebde14fd, SUC_lexicon_tags_14 },
    { 0x527e14fd, SUC_lexicon_tags_1 },
    { 0xc7de14ff, SUC_lexicon_tags_172 },
    { 0x93be1500, SUC_lexicon_tags_1 },
    { 0xcfbe1501, SUC_lexicon_tags_9 },
    { 0x897e1502, SUC_lexicon_tags_37 },
    { 0xad7e1503, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x68fe1505, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e150a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa87e150d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xccde1510, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x641e1513, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efe1517, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63de151a, SUC_lexicon_tags_87 },
    { 0x6de151b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7e1521, SUC_lexicon_tags_174 },
    { 0xc51e1522, SUC_lexicon_tags_23 },
    { 0xc8fe1523, SUC_lexicon_tags_15 },
    { 0xdfde1524, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87de1527, SUC_lexicon_tags_47 },
    { 0xfbe1528, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5e152c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x483e152e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfe1532, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x14de1534, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721e1538, SUC_lexicon_tags_62 },
    { 0x735e1538, SUC_lexicon_tags_7 },
    { 0x88fe1538, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3e153d, SUC_lexicon_tags_63 },
    { 0x167e153d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73e1542, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11e1546, SUC_lexicon_tags_31 },
    { 0x165e1547, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703e154f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7de1557, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x333e1559, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x90be155c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25e1564, SUC_lexicon_tags_68 },
    { 0xb4fe1565, SUC_lexicon_tags_62 },
    { 0xc8de1566, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe156a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x749e156d, SUC_lexicon_tags_7 },
    { 0x955e156e, SUC_lexicon_tags_15 },
    { 0xf17e156f, SUC_lexicon_tags_14 },
    { 0x1a5e1570, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x72de1572, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659e1576, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe8de1578, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x699e157b, SUC_lexicon_tags_11 },
    { 0xdffe157b, SUC_lexicon_tags_25 },
    { 0xb11e157d, SUC_lexicon_tags_31 },
    { 0x91fe157e, SUC_lexicon_tags_17 },
    { 0xe45e157f, SUC_lexicon_tags_14 },
    { 0x5bbe157f, SUC_lexicon_tags_65 },
    { 0xcb5e1581, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9c1e1583, SUC_lexicon_tags_17 },
    { 0xee5e1584, SUC_lexicon_tags_7 },
    { 0x9b7e1585, SUC_lexicon_tags_25 },
    { 0xd1be1586, SUC_lexicon_tags_12 },
    { 0x927e1585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x82fe1589, SUC_lexicon_tags_14 },
    { 0x885e158a, SUC_lexicon_tags_77 },
    { 0x5c3e158b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbe1591, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679e1599, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3e159c, SUC_lexicon_tags_37 },
    { 0x43e159d, SUC_lexicon_tags_63 },
    { 0x787e159e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbe15a1, SUC_lexicon_tags_39 },
    { 0x239e15a2, SUC_lexicon_tags_37 },
    { 0x2b5e15a3, SUC_lexicon_tags_7 },
    { 0x973e15a4, SUC_lexicon_tags_37 },
    { 0x53de15a5, SUC_lexicon_tags_17 },
    { 0x3cde15a6, SUC_lexicon_tags_14 },
    { 0x969e15a6, SUC_lexicon_tags_25 },
    { 0xb29e15a5, SUC_lexicon_tags_37 },
    { 0x2dbe15a8, SUC_lexicon_tags_11 },
    { 0x12fe15a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1d7e15ac, SUC_lexicon_tags_129 },
    { 0xf9e15ad, SUC_lexicon_tags_11 },
    { 0x465e15ae, SUC_lexicon_tags_2 },
    { 0x49be15af, SUC_lexicon_tags_32 },
    { 0x6c5e15af, SUC_lexicon_tags_7 },
    { 0xb4be15b1, SUC_lexicon_tags_30 },
    { 0xfe7e15b2, SUC_lexicon_tags_11 },
    { 0xe83e15b1, SUC_lexicon_tags_7 },
    { 0x1a7e15b1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfe15b9, SUC_lexicon_tags_11 },
    { 0x8ffe15b9, SUC_lexicon_tags_27 },
    { 0x5a5e15bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fe15c2, SUC_lexicon_tags_12 },
    { 0xb55e15c3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfe15c7, SUC_lexicon_tags_12 },
    { 0x9dfe15c8, SUC_lexicon_tags_15 },
    { 0xa85e15c9, SUC_lexicon_tags_9 },
    { 0x49e15ca, SUC_lexicon_tags_17 },
    { 0x729e15c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x52be15cd, SUC_lexicon_tags_27 },
    { 0xc7fe15ce, SUC_lexicon_tags_45 },
    { 0x3bde15cf, SUC_lexicon_tags_12 },
    { 0xf7be15ce, SUC_lexicon_tags_0 },
    { 0x7a1e15cd, SUC_lexicon_tags_23 },
    { 0xa55e15d2, SUC_lexicon_tags_12 },
    { 0x927e15d2, SUC_lexicon_tags_14 },
    { 0xcde15cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x78fe15d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3ade15d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbe15dc, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9be15e5, SUC_lexicon_tags_25 },
    { 0x951e15e6, SUC_lexicon_tags_25 },
    { 0x50fe15e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fe15ea, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11e15ee, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17e15f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc45e15f3, SUC_lexicon_tags_17 },
    { 0x195e15f3, SUC_lexicon_tags_11 },
    { 0xffbe15f4, SUC_lexicon_tags_19 },
    { 0xfdfe15f3, SUC_lexicon_tags_17 },
    { 0xdd5e15f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x511e15f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28fe15fb, SUC_lexicon_tags_17 },
    { 0xeb9e15fb, SUC_lexicon_tags_17 },
    { 0xfcde15fc, SUC_lexicon_tags_1 },
    { 0xcafe15fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x315e1600, SUC_lexicon_tags_25 },
    { 0xd27e1601, SUC_lexicon_tags_7 },
    { 0xba3e1600, SUC_lexicon_tags_7 },
    { 0x94de1600, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fe1607, SUC_lexicon_tags_65 },
    { 0x277e1608, SUC_lexicon_tags_60 },
    { 0x3e7e1607, SUC_lexicon_tags_46 },
    { 0x225e1608, SUC_lexicon_tags_14 },
    { 0xd15e160b, SUC_lexicon_tags_7 },
    { 0x9be160c, SUC_lexicon_tags_45 },
    { 0x391e160d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abe1619, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29e1620, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xffbe1622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9e1627, SUC_lexicon_tags_29 },
    { 0xf57e1627, SUC_lexicon_tags_68 },
    { 0xf39e1629, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe17e162b, SUC_lexicon_tags_37 },
    { 0xa33e162b, SUC_lexicon_tags_63 },
    { 0xfa7e162c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e1634, SUC_lexicon_tags_23 },
    { 0xecbe1635, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2be1639, SUC_lexicon_tags_17 },
    { 0x58fe1639, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x935e163c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88be1640, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x99be1642, SUC_lexicon_tags_38 },
    { 0x19fe1643, SUC_lexicon_tags_28 },
    { 0xf77e1644, SUC_lexicon_tags_87 },
    { 0x863e1645, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x619e1647, SUC_lexicon_tags_2 },
    { 0x8dde1648, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x819e164a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4be164f, SUC_lexicon_tags_11 },
    { 0xe09e1650, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7e1656, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3e165a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf7fe165c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fe1665, SUC_lexicon_tags_17 },
    { 0xaf5e1666, SUC_lexicon_tags_1 },
    { 0xdade1666, SUC_lexicon_tags_7 },
    { 0x693e1668, SUC_lexicon_tags_2 },
    { 0x77be1669, SUC_lexicon_tags_37 },
    { 0xf65e1668, SUC_lexicon_tags_201 },
    { 0xc3fe166b, SUC_lexicon_tags_15 },
    { 0x455e1668, SUC_lexicon_tags_7 },
    { 0xfe1e166d, SUC_lexicon_tags_48 },
    { 0x8d7e1668, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e1673, SUC_lexicon_tags_7 },
    { 0xa6de1674, SUC_lexicon_tags_15 },
    { 0x913e1674, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x775e1677, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x34de1679, SUC_lexicon_tags_40 },
    { 0xa1be1679, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x84fe167c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cde1680, SUC_lexicon_tags_27 },
    { 0xf69e1680, SUC_lexicon_tags_17 },
    { 0x873e1680, SUC_lexicon_tags_1 },
    { 0x4e7e1682, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85e1688, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77be168a, SUC_lexicon_tags_17 },
    { 0xa4fe168b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4de168d, SUC_lexicon_tags_2 },
    { 0x195e168e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7e1691, SUC_lexicon_tags_1 },
    { 0xaffe1691, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x683e1695, SUC_lexicon_tags_38 },
    { 0xd73e1695, SUC_lexicon_tags_11 },
    { 0x1a3e1697, SUC_lexicon_tags_11 },
    { 0xb19e1698, SUC_lexicon_tags_43 },
    { 0x4e7e1699, SUC_lexicon_tags_15 },
    { 0x761e1697, SUC_lexicon_tags_7 },
    { 0xfd1e169a, SUC_lexicon_tags_47 },
    { 0x245e1695, SUC_lexicon_tags_15 },
    { 0xea7e1697, SUC_lexicon_tags_2 },
    { 0x263e1697, SUC_lexicon_tags_30 },
    { 0x80be169f, SUC_lexicon_tags_17 },
    { 0x7bde169f, SUC_lexicon_tags_11 },
    { 0xc2de16a1, SUC_lexicon_tags_82 },
    { 0x6c5e169f, SUC_lexicon_tags_13 },
    { 0xb89e16a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323e16a7, SUC_lexicon_tags_47 },
    { 0x293e16a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa13e16aa, SUC_lexicon_tags_14 },
    { 0x249e16ab, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9e16ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5de16b2, SUC_lexicon_tags_37 },
    { 0x79de16b3, SUC_lexicon_tags_7 },
    { 0xb5fe16b3, SUC_lexicon_tags_47 },
    { 0xf5fe16b5, SUC_lexicon_tags_2 },
    { 0x855e16b4, SUC_lexicon_tags_15 },
    { 0xb3fe16b7, SUC_lexicon_tags_60 },
    { 0xbdde16b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3e16bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x649e16bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5e16c5, SUC_lexicon_tags_13 },
    { 0x2f5e16c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43e16cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47e16ce, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87e16d3, SUC_lexicon_tags_25 },
    { 0x8ebe16d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbfe16d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fe16da, SUC_lexicon_tags_21 },
    { 0x38be16db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1e16e2, SUC_lexicon_tags_14 },
    { 0x845e16e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe16e6, SUC_lexicon_tags_2 },
    { 0x9be16e7, SUC_lexicon_tags_0 },
    { 0xac1e16e8, SUC_lexicon_tags_37 },
    { 0x66be16e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63e16ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4be16ef, SUC_lexicon_tags_26 },
    { 0x235e16ef, SUC_lexicon_tags_27 },
    { 0x321e16f1, SUC_lexicon_tags_25 },
    { 0x923e16f2, SUC_lexicon_tags_6 },
    { 0x4e7e16f2, SUC_lexicon_tags_8 },
    { 0x5de16f0, SUC_lexicon_tags_45 },
    { 0x895e16f5, SUC_lexicon_tags_54 },
    { 0x6a7e16f5, SUC_lexicon_tags_17 },
    { 0x28de16f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf23e16f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x12fe16fb, SUC_lexicon_tags_14 },
    { 0x681e16fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7e1700, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91e1705, SUC_lexicon_tags_10 },
    { 0x161e1706, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9e170c, SUC_lexicon_tags_26 },
    { 0x22de170d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98be170f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6be1716, SUC_lexicon_tags_43 },
    { 0x5c3e1717, SUC_lexicon_tags_43 },
    { 0xd25e1717, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x927e171b, SUC_lexicon_tags_19 },
    { 0xa59e171c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605e1720, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47e1724, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243e1729, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7e172d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ade1730, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfde1739, SUC_lexicon_tags_11 },
    { 0xcade173a, SUC_lexicon_tags_38 },
    { 0x96fe1739, SUC_lexicon_tags_47 },
    { 0xb11e173c, SUC_lexicon_tags_30 },
    { 0xaa7e173d, SUC_lexicon_tags_33 },
    { 0xcd3e1739, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x631e1740, SUC_lexicon_tags_37 },
    { 0xa5be1740, SUC_lexicon_tags_7 },
    { 0x2d3e1742, SUC_lexicon_tags_19 },
    { 0x871e1743, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bde1746, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe1749, SUC_lexicon_tags_49 },
    { 0xf5be1749, SUC_lexicon_tags_2 },
    { 0xa2de174a, SUC_lexicon_tags_12 },
    { 0x973e174c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48de1750, SUC_lexicon_tags_7 },
    { 0xa85e1751, SUC_lexicon_tags_2 },
    { 0xa0be1752, SUC_lexicon_tags_25 },
    { 0x3bbe1753, SUC_lexicon_tags_43 },
    { 0xdc9e1751, SUC_lexicon_tags_37 },
    { 0x489e1755, SUC_lexicon_tags_38 },
    { 0xb7de1754, SUC_lexicon_tags_19 },
    { 0xbbbe1757, SUC_lexicon_tags_10 },
    { 0x6d9e1758, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9be175e, SUC_lexicon_tags_1 },
    { 0xf4be175f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x649e1761, SUC_lexicon_tags_25 },
    { 0x803e1762, SUC_lexicon_tags_17 },
    { 0x5ede1762, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x123e1765, SUC_lexicon_tags_13 },
    { 0xf7be1766, SUC_lexicon_tags_16 },
    { 0x907e1767, SUC_lexicon_tags_14 },
    { 0xd69e1768, SUC_lexicon_tags_6 },
    { 0xa19e1769, SUC_lexicon_tags_7 },
    { 0x825e1765, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd1e176c, SUC_lexicon_tags_81 },
    { 0x90de176c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x897e1770, SUC_lexicon_tags_2 },
    { 0x251e1771, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9e1775, SUC_lexicon_tags_9 },
    { 0xdbbe1775, SUC_lexicon_tags_37 },
    { 0x579e1777, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd91e1779, SUC_lexicon_tags_28 },
    { 0x2e7e177a, SUC_lexicon_tags_7 },
    { 0x9cfe177b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9e177e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437e1782, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x781e1784, SUC_lexicon_tags_63 },
    { 0x247e1784, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x77de1787, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1de178c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531e1792, SUC_lexicon_tags_1 },
    { 0xeafe1793, SUC_lexicon_tags_60 },
    { 0x987e1792, SUC_lexicon_tags_15 },
    { 0xa9be1795, SUC_lexicon_tags_1 },
    { 0xe95e1796, SUC_lexicon_tags_12 },
    { 0x7c9e1797, SUC_lexicon_tags_37 },
    { 0x757e1796, SUC_lexicon_tags_49 },
    { 0x889e1798, SUC_lexicon_tags_68 },
    { 0x85fe179a, SUC_lexicon_tags_25 },
    { 0x787e179a, SUC_lexicon_tags_12 },
    { 0x87be1795, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xb37e179e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe17a3, SUC_lexicon_tags_79 },
    { 0x5fde17a4, SUC_lexicon_tags_1 },
    { 0xb91e17a4, SUC_lexicon_tags_47 },
    { 0x1b9e17a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333e17ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb25e17ad, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x74fe17af, SUC_lexicon_tags_7 },
    { 0xa1fe17b0, SUC_lexicon_tags_0 },
    { 0x4a5e17b1, SUC_lexicon_tags_38 },
    { 0x79de17b0, SUC_lexicon_tags_17 },
    { 0x3cfe17b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639e17b9, SUC_lexicon_tags_0 },
    { 0x753e17ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbde17c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x595e17c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f5e17c7, SUC_lexicon_tags_1 },
    { 0x2b9e17c8, SUC_lexicon_tags_152 },
    { 0x1e7e17c8, SUC_lexicon_tags_39 },
    { 0xcbfe17ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe77e17cc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3e17cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb4fe17d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241e17d7, SUC_lexicon_tags_9 },
    { 0xf81e17d8, SUC_lexicon_tags_56 },
    { 0x89be17d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199e17de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7fe17e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69e17e4, SUC_lexicon_tags_3 },
    { 0x6d3e17e4, SUC_lexicon_tags_45 },
    { 0x63e17e6, SUC_lexicon_tags_16 },
    { 0xf3e17e7, SUC_lexicon_tags_25 },
    { 0xdbe17e8, SUC_lexicon_tags_17 },
    { 0xc49e17e6, SUC_lexicon_tags_17 },
    { 0xa57e17e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181e17ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5e17f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3e17f8, SUC_lexicon_tags_84 },
    { 0x5a9e17f8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf8be17fb, SUC_lexicon_tags_1 },
    { 0xa89e17fc, SUC_lexicon_tags_66 },
    { 0xb5de17fd, SUC_lexicon_tags_11 },
    { 0x8dfe17fe, SUC_lexicon_tags_14 },
    { 0x86de17ff, SUC_lexicon_tags_2 },
    { 0x83de17ff, SUC_lexicon_tags_1 },
    { 0x73fe1801, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3be1804, SUC_lexicon_tags_31 },
    { 0xcf7e1805, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821e180a, SUC_lexicon_tags_0 },
    { 0x22fe180b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357e181a, SUC_lexicon_tags_30 },
    { 0x61de181a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbe181e, SUC_lexicon_tags_25 },
    { 0x5c7e181e, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xaa9e1821, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfe1824, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347e182a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39e1830, SUC_lexicon_tags_37 },
    { 0x21e1830, SUC_lexicon_tags_26 },
    { 0xa71e1831, SUC_lexicon_tags_11 },
    { 0x4b9e1830, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xad9e1835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53e183a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x101e183d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5e1842, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03e1845, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x80de1847, SUC_lexicon_tags_27 },
    { 0x20fe1848, SUC_lexicon_tags_32 },
    { 0x9b7e1849, SUC_lexicon_tags_32 },
    { 0xc01e184a, SUC_lexicon_tags_37 },
    { 0xb03e184b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be1855, SUC_lexicon_tags_16 },
    { 0x69fe1856, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33e185b, SUC_lexicon_tags_40 },
    { 0xdf3e185c, SUC_lexicon_tags_25 },
    { 0xd85e185b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29e1861, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x84fe1863, SUC_lexicon_tags_7 },
    { 0xb1e1863, SUC_lexicon_tags_11 },
    { 0x523e1863, SUC_lexicon_tags_1320 },
    { 0, NULL },
    { 0x49e1867, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc1e1869, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffde186e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe45e1870, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8be1875, SUC_lexicon_tags_12 },
    { 0xfcde1876, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1e187c, SUC_lexicon_tags_27 },
    { 0x2bfe187d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477e1884, SUC_lexicon_tags_12 },
    { 0xe4de1884, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbe1888, SUC_lexicon_tags_17 },
    { 0x9e1888, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1e188c, SUC_lexicon_tags_17 },
    { 0xb95e188c, SUC_lexicon_tags_27 },
    { 0xa93e188e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe1de1890, SUC_lexicon_tags_26 },
    { 0xe7de1891, SUC_lexicon_tags_39 },
    { 0xfdde1892, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc03e1894, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37e1898, SUC_lexicon_tags_37 },
    { 0xf61e1899, SUC_lexicon_tags_79 },
    { 0x957e1899, SUC_lexicon_tags_14 },
    { 0x111e1898, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x929e189e, SUC_lexicon_tags_45 },
    { 0xf9de189f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821e18a6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9be18a9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x977e18ab, SUC_lexicon_tags_1 },
    { 0x889e18ac, SUC_lexicon_tags_61 },
    { 0x165e18ac, SUC_lexicon_tags_37 },
    { 0x149e18ae, SUC_lexicon_tags_1 },
    { 0xe19e18af, SUC_lexicon_tags_47 },
    { 0x94de18ae, SUC_lexicon_tags_1 },
    { 0xd7de18b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1e18b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55fe18bb, SUC_lexicon_tags_7 },
    { 0xea7e18bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9e18bf, SUC_lexicon_tags_14 },
    { 0xa39e18c0, SUC_lexicon_tags_14 },
    { 0xd51e18c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91e18c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fe18c9, SUC_lexicon_tags_25 },
    { 0x1cbe18ca, SUC_lexicon_tags_19 },
    { 0x8d1e18cb, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91e18ce, SUC_lexicon_tags_30 },
    { 0xd1fe18ce, SUC_lexicon_tags_7 },
    { 0x769e18d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11e18d5, SUC_lexicon_tags_29 },
    { 0x6e9e18d6, SUC_lexicon_tags_48 },
    { 0xd81e18d7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xbede18d9, SUC_lexicon_tags_33 },
    { 0x731e18da, SUC_lexicon_tags_12 },
    { 0x55fe18db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73e18e2, SUC_lexicon_tags_812 },
    { 0x795e18e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89e18e8, SUC_lexicon_tags_2 },
    { 0xd6be18e8, SUC_lexicon_tags_11 },
    { 0xad9e18ea, SUC_lexicon_tags_37 },
    { 0x1f3e18ea, SUC_lexicon_tags_2 },
    { 0xa01e18ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71e18ef, SUC_lexicon_tags_6 },
    { 0xf3fe18ef, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf67e18f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25be18f4, SUC_lexicon_tags_26 },
    { 0x1f3e18f5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeade18fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6dfe18fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x233e1901, SUC_lexicon_tags_28 },
    { 0x945e1902, SUC_lexicon_tags_0 },
    { 0x329e1903, SUC_lexicon_tags_14 },
    { 0xfcde1903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbe1907, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307e190b, SUC_lexicon_tags_54 },
    { 0x505e190c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5e190f, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0x739e1911, SUC_lexicon_tags_7 },
    { 0x867e1912, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39e1916, SUC_lexicon_tags_7 },
    { 0x19fe1917, SUC_lexicon_tags_12 },
    { 0xea5e1918, SUC_lexicon_tags_27 },
    { 0xe73e1919, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf5e191b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xab9e191d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407e1923, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe41e1925, SUC_lexicon_tags_7 },
    { 0xe43e1925, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99fe1928, SUC_lexicon_tags_37 },
    { 0x9b9e1929, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xafde192c, SUC_lexicon_tags_65 },
    { 0xebbe192c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb5fe192f, SUC_lexicon_tags_14 },
    { 0x893e1930, SUC_lexicon_tags_12 },
    { 0x67e1931, SUC_lexicon_tags_38 },
    { 0x97e1931, SUC_lexicon_tags_45 },
    { 0x923e1933, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4fe1935, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1e1938, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5e193b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61e193e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcbde1940, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x163e1943, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x4ade1945, SUC_lexicon_tags_37 },
    { 0xf9be1946, SUC_lexicon_tags_17 },
    { 0x6f7e1947, SUC_lexicon_tags_0 },
    { 0x157e1947, SUC_lexicon_tags_21 },
    { 0x997e1949, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbf7e194b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde194e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e1951, SUC_lexicon_tags_12 },
    { 0x931e1952, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cbe1954, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6f5e1956, SUC_lexicon_tags_87 },
    { 0xf37e1957, SUC_lexicon_tags_15 },
    { 0xfebe1957, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x55e195a, SUC_lexicon_tags_15 },
    { 0xc73e195b, SUC_lexicon_tags_6 },
    { 0xbfde195c, SUC_lexicon_tags_19 },
    { 0x9e7e195c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2e5e195f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf13e1961, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xde3e1963, SUC_lexicon_tags_37 },
    { 0x2e7e1964, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fe196d, SUC_lexicon_tags_11 },
    { 0x2de196d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48de1970, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fe1973, SUC_lexicon_tags_25 },
    { 0xf7fe1974, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x287e1976, SUC_lexicon_tags_45 },
    { 0x4b5e1977, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x983e1979, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179e197d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7de1980, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565e1984, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c7e1986, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe4de1988, SUC_lexicon_tags_17 },
    { 0x5fde1988, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x793e198b, SUC_lexicon_tags_46 },
    { 0x7f9e198b, SUC_lexicon_tags_47 },
    { 0x1a9e198d, SUC_lexicon_tags_25 },
    { 0xd7e198e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e1991, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259e1995, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffe199d, SUC_lexicon_tags_17 },
    { 0x6dbe199e, SUC_lexicon_tags_19 },
    { 0xb99e199e, SUC_lexicon_tags_45 },
    { 0xc09e199f, SUC_lexicon_tags_33 },
    { 0x77fe19a1, SUC_lexicon_tags_14 },
    { 0x8b9e199e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8c1e19a4, SUC_lexicon_tags_7 },
    { 0xdf1e19a5, SUC_lexicon_tags_25 },
    { 0x39be19a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421e19ae, SUC_lexicon_tags_8 },
    { 0xc4de19af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7e19b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99e19c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe59e19c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4de19c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17e19c9, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xac5e19cb, SUC_lexicon_tags_13 },
    { 0x1bde19cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd73e19ce, SUC_lexicon_tags_49 },
    { 0x1cde19ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x299e19d2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5e19de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4fde19e0, SUC_lexicon_tags_62 },
    { 0xe6de19e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d3e19e3, SUC_lexicon_tags_14 },
    { 0x6f3e19e4, SUC_lexicon_tags_81 },
    { 0xd07e19e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x561e19e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb51e19e9, SUC_lexicon_tags_1 },
    { 0x2e7e19ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7e19ef, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18de19ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x163e1a01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x99fe1a03, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e1a08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889e1a0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e1a0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1de1a14, SUC_lexicon_tags_33 },
    { 0xd59e1a14, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xa9de1a17, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31be1a1e, SUC_lexicon_tags_7 },
    { 0xa93e1a1f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xec3e1a21, SUC_lexicon_tags_17 },
    { 0x7f7e1a21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1e1a26, SUC_lexicon_tags_17 },
    { 0x8a7e1a27, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1e1a2b, SUC_lexicon_tags_17 },
    { 0xd03e1a2c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x903e1a2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15e1a32, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xecfe1a35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69e1a41, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x259e1a43, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafe1a48, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673e1a50, SUC_lexicon_tags_12 },
    { 0xb2fe1a51, SUC_lexicon_tags_29 },
    { 0x961e1a52, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x68fe1a54, SUC_lexicon_tags_47 },
    { 0x259e1a55, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211e1a5a, SUC_lexicon_tags_25 },
    { 0x29e1a5b, SUC_lexicon_tags_27 },
    { 0xdede1a5c, SUC_lexicon_tags_1077 },
    { 0xee9e1a5d, SUC_lexicon_tags_31 },
    { 0xbe1a5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e1a67, SUC_lexicon_tags_1 },
    { 0x23be1a68, SUC_lexicon_tags_11 },
    { 0xfebe1a69, SUC_lexicon_tags_12 },
    { 0x30be1a69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a7e1a6c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfe1a72, SUC_lexicon_tags_63 },
    { 0x60be1a73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x399e1a75, SUC_lexicon_tags_5 },
    { 0x613e1a75, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x54be1a79, SUC_lexicon_tags_11 },
    { 0xebde1a7a, SUC_lexicon_tags_6 },
    { 0xa11e1a7b, SUC_lexicon_tags_40 },
    { 0x16be1a7b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53e1a84, SUC_lexicon_tags_27 },
    { 0xe29e1a85, SUC_lexicon_tags_2 },
    { 0x4cbe1a86, SUC_lexicon_tags_15 },
    { 0x721e1a86, SUC_lexicon_tags_12 },
    { 0xa77e1a88, SUC_lexicon_tags_39 },
    { 0x81e1a85, SUC_lexicon_tags_43 },
    { 0xa5e1a8a, SUC_lexicon_tags_17 },
    { 0xef9e1a8b, SUC_lexicon_tags_118 },
    { 0x543e1a8a, SUC_lexicon_tags_37 },
    { 0xdbfe1a8d, SUC_lexicon_tags_12 },
    { 0x4c1e1a8e, SUC_lexicon_tags_34 },
    { 0xc0fe1a8f, SUC_lexicon_tags_1 },
    { 0xd39e1a8d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8efe1a92, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa7e1a94, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dde1a9b, SUC_lexicon_tags_45 },
    { 0x1f9e1a9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fe1aa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe75e1aa2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fe1aa5, SUC_lexicon_tags_25 },
    { 0xed1e1aa5, SUC_lexicon_tags_37 },
    { 0x9d5e1aa7, SUC_lexicon_tags_37 },
    { 0x72de1aa8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3de1aab, SUC_lexicon_tags_7 },
    { 0x77e1aac, SUC_lexicon_tags_2 },
    { 0x239e1aad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fe1ab6, SUC_lexicon_tags_25 },
    { 0x757e1ab7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x933e1ab9, SUC_lexicon_tags_7 },
    { 0x4e3e1aba, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb39e1abc, SUC_lexicon_tags_17 },
    { 0xeede1abc, SUC_lexicon_tags_15 },
    { 0x9ebe1abd, SUC_lexicon_tags_25 },
    { 0xf49e1abf, SUC_lexicon_tags_1 },
    { 0xc31e1abc, SUC_lexicon_tags_17 },
    { 0xdcde1ac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e1ac5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfbe1ac7, SUC_lexicon_tags_2 },
    { 0x26be1ac8, SUC_lexicon_tags_30 },
    { 0x74fe1ac9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37e1acd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1e1ad2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x789e1ad5, SUC_lexicon_tags_17 },
    { 0x4a7e1ad6, SUC_lexicon_tags_9 },
    { 0x4f5e1ad7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffde1ad9, SUC_lexicon_tags_7 },
    { 0xf31e1ada, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8d5e1adc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x457e1ade, SUC_lexicon_tags_7 },
    { 0x969e1adf, SUC_lexicon_tags_37 },
    { 0xd55e1ade, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x323e1ae3, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebe1ae6, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e1aeb, SUC_lexicon_tags_47 },
    { 0x545e1aec, SUC_lexicon_tags_14 },
    { 0x843e1aed, SUC_lexicon_tags_28 },
    { 0x23e1aeb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e1af1, SUC_lexicon_tags_12 },
    { 0x8c9e1af2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe3de1af4, SUC_lexicon_tags_2 },
    { 0x2cfe1af5, SUC_lexicon_tags_16 },
    { 0x621e1af5, SUC_lexicon_tags_38 },
    { 0x36be1af7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37e1af9, SUC_lexicon_tags_14 },
    { 0xad5e1af9, SUC_lexicon_tags_14 },
    { 0xd2be1afb, SUC_lexicon_tags_28 },
    { 0xb6fe1af9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7be1b00, SUC_lexicon_tags_37 },
    { 0xd07e1b01, SUC_lexicon_tags_65 },
    { 0xed9e1b02, SUC_lexicon_tags_25 },
    { 0x69de1b03, SUC_lexicon_tags_17 },
    { 0xfbfe1b04, SUC_lexicon_tags_1 },
    { 0xb83e1b03, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x345e1b07, SUC_lexicon_tags_17 },
    { 0xebfe1b08, SUC_lexicon_tags_25 },
    { 0x44fe1b09, SUC_lexicon_tags_15 },
    { 0x177e1b0a, SUC_lexicon_tags_7 },
    { 0x709e1b0a, SUC_lexicon_tags_11 },
    { 0x253e1b0c, SUC_lexicon_tags_7 },
    { 0xb59e1b08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e1b13, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667e1b17, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x96be1b19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07e1b1c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x709e1b1e, SUC_lexicon_tags_14 },
    { 0xabde1b1f, SUC_lexicon_tags_23 },
    { 0x70fe1b1f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fe1b25, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x86fe1b27, SUC_lexicon_tags_82 },
    { 0x451e1b27, SUC_lexicon_tags_55 },
    { 0x731e1b28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7e1b2c, SUC_lexicon_tags_38 },
    { 0x117e1b2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb1e1b2f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7dde1b31, SUC_lexicon_tags_17 },
    { 0x1c1e1b32, SUC_lexicon_tags_55 },
    { 0xf5e1b33, SUC_lexicon_tags_4 },
    { 0x753e1b34, SUC_lexicon_tags_37 },
    { 0x597e1b35, SUC_lexicon_tags_42 },
    { 0x801e1b32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e1b3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afe1b3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfe1b40, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51e1b43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779e1b4a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3e1b4d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x25e1b4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa59e1b51, SUC_lexicon_tags_3 },
    { 0x7f7e1b52, SUC_lexicon_tags_49 },
    { 0xfc7e1b53, SUC_lexicon_tags_19 },
    { 0x44be1b53, SUC_lexicon_tags_25 },
    { 0x151e1b55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbe1b5a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x57be1b5d, SUC_lexicon_tags_12 },
    { 0xe7be1b5e, SUC_lexicon_tags_1 },
    { 0xc7fe1b5e, SUC_lexicon_tags_26 },
    { 0xee1e1b5f, SUC_lexicon_tags_38 },
    { 0xf67e1b61, SUC_lexicon_tags_30 },
    { 0x7e9e1b61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9e1b69, SUC_lexicon_tags_60 },
    { 0x641e1b6a, SUC_lexicon_tags_11 },
    { 0xb1fe1b6b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbe1b70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609e1b74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa71e1b76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661e1b7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa45e1b7d, SUC_lexicon_tags_27 },
    { 0x7a1e1b7e, SUC_lexicon_tags_47 },
    { 0xe33e1b7f, SUC_lexicon_tags_11 },
    { 0x273e1b80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x961e1b82, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x96de1b84, SUC_lexicon_tags_17 },
    { 0xb93e1b85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x59e1b88, SUC_lexicon_tags_37 },
    { 0x6dfe1b89, SUC_lexicon_tags_1 },
    { 0x12de1b8a, SUC_lexicon_tags_11 },
    { 0xb69e1b89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x733e1b8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fe1b95, SUC_lexicon_tags_37 },
    { 0x87be1b96, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1e1b9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc03e1b9d, SUC_lexicon_tags_6 },
    { 0x847e1b9d, SUC_lexicon_tags_103 },
    { 0xe1e1b9f, SUC_lexicon_tags_55 },
    { 0x729e1ba0, SUC_lexicon_tags_4 },
    { 0xe55e1ba1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861e1ba5, SUC_lexicon_tags_2 },
    { 0xd09e1ba5, SUC_lexicon_tags_16 },
    { 0xd5e1ba7, SUC_lexicon_tags_17 },
    { 0x7de1ba8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfe1baf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1bb5, SUC_lexicon_tags_49 },
    { 0xc7be1bb6, SUC_lexicon_tags_2 },
    { 0xd5de1bb7, SUC_lexicon_tags_33 },
    { 0xb1fe1bb8, SUC_lexicon_tags_0 },
    { 0x689e1bb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29be1bc4, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0x845e1bc7, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x1bde1bc9, SUC_lexicon_tags_5 },
    { 0xe6fe1bca, SUC_lexicon_tags_38 },
    { 0x5a7e1bcb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb59e1bcd, SUC_lexicon_tags_37 },
    { 0x263e1bce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15e1bd4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xac3e1bd6, SUC_lexicon_tags_13 },
    { 0xfa3e1bd7, SUC_lexicon_tags_37 },
    { 0xd41e1bd8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57e1bdb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65be1be4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681e1beb, SUC_lexicon_tags_63 },
    { 0xd8fe1bec, SUC_lexicon_tags_17 },
    { 0x66be1bed, SUC_lexicon_tags_1 },
    { 0x7f3e1bed, SUC_lexicon_tags_1 },
    { 0x29be1bef, SUC_lexicon_tags_37 },
    { 0xa9fe1bef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1e1bf3, SUC_lexicon_tags_15 },
    { 0xe99e1bf3, SUC_lexicon_tags_0 },
    { 0x84de1bf5, SUC_lexicon_tags_26 },
    { 0xc7de1bf6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xae9e1bf8, SUC_lexicon_tags_63 },
    { 0x829e1bf9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x765e1bfb, SUC_lexicon_tags_8 },
    { 0x265e1bfc, SUC_lexicon_tags_87 },
    { 0x52be1bfc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941e1c01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbe1c05, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x73de1c08, SUC_lexicon_tags_15 },
    { 0x43e1c09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6de1c0e, SUC_lexicon_tags_15 },
    { 0xb6be1c0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495e1c19, SUC_lexicon_tags_19 },
    { 0x8fbe1c1a, SUC_lexicon_tags_37 },
    { 0x4a3e1c1a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabe1c26, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273e1c2c, SUC_lexicon_tags_1717 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fe1c31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e1e1c33, SUC_lexicon_tags_26 },
    { 0x621e1c33, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459e1c39, SUC_lexicon_tags_0 },
    { 0x8c3e1c39, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbe1c3d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fde1c44, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfe1c47, SUC_lexicon_tags_37 },
    { 0xdbfe1c48, SUC_lexicon_tags_30 },
    { 0x70be1c49, SUC_lexicon_tags_28 },
    { 0xeb3e1c4a, SUC_lexicon_tags_21 },
    { 0x907e1c48, SUC_lexicon_tags_25 },
    { 0x425e1c4c, SUC_lexicon_tags_27 },
    { 0xfc7e1c4c, SUC_lexicon_tags_11 },
    { 0xdb9e1c4e, SUC_lexicon_tags_22 },
    { 0x36be1c4f, SUC_lexicon_tags_27 },
    { 0x325e1c50, SUC_lexicon_tags_17 },
    { 0x881e1c4f, SUC_lexicon_tags_39 },
    { 0x871e1c52, SUC_lexicon_tags_27 },
    { 0xa0be1c53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215e1c57, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe9be1c59, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7e1c5d, SUC_lexicon_tags_38 },
    { 0xdfde1c5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdfe1c60, SUC_lexicon_tags_37 },
    { 0x423e1c61, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4bbe1c63, SUC_lexicon_tags_7 },
    { 0x333e1c64, SUC_lexicon_tags_12 },
    { 0xe29e1c63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x205e1c67, SUC_lexicon_tags_11 },
    { 0x7efe1c68, SUC_lexicon_tags_8 },
    { 0x6a1e1c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x939e1c6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0de1c70, SUC_lexicon_tags_37 },
    { 0x421e1c71, SUC_lexicon_tags_31 },
    { 0x3efe1c72, SUC_lexicon_tags_25 },
    { 0xfade1c71, SUC_lexicon_tags_5 },
    { 0x4b5e1c70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x757e1c76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf93e1c78, SUC_lexicon_tags_79 },
    { 0x56de1c79, SUC_lexicon_tags_26 },
    { 0x4be1c7a, SUC_lexicon_tags_31 },
    { 0xbc5e1c7a, SUC_lexicon_tags_37 },
    { 0xcf5e1c7c, SUC_lexicon_tags_6 },
    { 0x213e1c7c, SUC_lexicon_tags_2 },
    { 0xfd9e1c7d, SUC_lexicon_tags_47 },
    { 0xc69e1c7f, SUC_lexicon_tags_33 },
    { 0x54fe1c7d, SUC_lexicon_tags_38 },
    { 0x7a7e1c79, SUC_lexicon_tags_27 },
    { 0x761e1c7b, SUC_lexicon_tags_11 },
    { 0xefde1c83, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x64de1c86, SUC_lexicon_tags_16 },
    { 0x369e1c87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbfe1c89, SUC_lexicon_tags_27 },
    { 0x79e1c8a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fe1c93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5be1c96, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23e1c9f, SUC_lexicon_tags_37 },
    { 0xbc5e1ca0, SUC_lexicon_tags_33 },
    { 0x54fe1ca0, SUC_lexicon_tags_0 },
    { 0x60be1ca1, SUC_lexicon_tags_10 },
    { 0xb0de1ca3, SUC_lexicon_tags_60 },
    { 0x557e1ca4, SUC_lexicon_tags_25 },
    { 0xa87e1ca5, SUC_lexicon_tags_87 },
    { 0x129e1ca5, SUC_lexicon_tags_15 },
    { 0x6bfe1c9f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51e1cab, SUC_lexicon_tags_2 },
    { 0xa41e1cac, SUC_lexicon_tags_60 },
    { 0xa79e1cad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe1cb1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x503e1cb3, SUC_lexicon_tags_33 },
    { 0x75e1cb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x29e1cb6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x343e1cb8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x249e1cba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbe1cbe, SUC_lexicon_tags_12 },
    { 0x339e1cbe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x61be1cc1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01e1cc5, SUC_lexicon_tags_25 },
    { 0x169e1cc6, SUC_lexicon_tags_38 },
    { 0xab7e1cc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591e1cd2, SUC_lexicon_tags_7 },
    { 0xde3e1cd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x221e1cd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x323e1cd8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92be1cde, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19e1ce3, SUC_lexicon_tags_2 },
    { 0xf5be1ce4, SUC_lexicon_tags_17 },
    { 0x8be1ce5, SUC_lexicon_tags_1 },
    { 0x6d1e1ce6, SUC_lexicon_tags_30 },
    { 0x5c1e1ce4, SUC_lexicon_tags_43 },
    { 0x839e1ce3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb63e1cea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95e1cf1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e1cf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fe1cfa, SUC_lexicon_tags_25 },
    { 0x93fe1cfa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfede1d01, SUC_lexicon_tags_8 },
    { 0x1efe1d02, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1d06, SUC_lexicon_tags_125 },
    { 0xc5e1d06, SUC_lexicon_tags_38 },
    { 0x121e1d07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x61e1d0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183e1d0f, SUC_lexicon_tags_15 },
    { 0x2ade1d10, SUC_lexicon_tags_79 },
    { 0xcc7e1d10, SUC_lexicon_tags_27 },
    { 0xde5e1d0f, SUC_lexicon_tags_7 },
    { 0xce9e1d13, SUC_lexicon_tags_19 },
    { 0x6fe1d10, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50de1d16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbe1d19, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfcbe1d1b, SUC_lexicon_tags_79 },
    { 0x56fe1d1c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73e1d20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x367e1d23, SUC_lexicon_tags_17 },
    { 0x28de1d24, SUC_lexicon_tags_37 },
    { 0xb33e1d24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631e1d29, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359e1d2f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x359e1d32, SUC_lexicon_tags_26 },
    { 0x3efe1d32, SUC_lexicon_tags_25 },
    { 0x911e1d34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75e1d41, SUC_lexicon_tags_37 },
    { 0x9bfe1d41, SUC_lexicon_tags_45 },
    { 0x67e1d43, SUC_lexicon_tags_17 },
    { 0x8f7e1d42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16be1d48, SUC_lexicon_tags_37 },
    { 0x9d3e1d48, SUC_lexicon_tags_17 },
    { 0xf3de1d49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c3e1d4c, SUC_lexicon_tags_25 },
    { 0xdcbe1d4d, SUC_lexicon_tags_14 },
    { 0xe69e1d4e, SUC_lexicon_tags_47 },
    { 0x7a1e1d4f, SUC_lexicon_tags_67 },
    { 0xffbe1d50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x909e1d53, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbe1d59, SUC_lexicon_tags_45 },
    { 0x26fe1d5a, SUC_lexicon_tags_87 },
    { 0x793e1d5b, SUC_lexicon_tags_8 },
    { 0x86fe1d5a, SUC_lexicon_tags_15 },
    { 0x443e1d5d, SUC_lexicon_tags_11 },
    { 0x26de1d5a, SUC_lexicon_tags_2 },
    { 0x65fe1d5f, SUC_lexicon_tags_3 },
    { 0xc5de1d60, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xfb5e1d62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x845e1d65, SUC_lexicon_tags_66 },
    { 0xea3e1d66, SUC_lexicon_tags_15 },
    { 0x667e1d66, SUC_lexicon_tags_2 },
    { 0x487e1d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13e1d6c, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xc23e1d6e, SUC_lexicon_tags_146 },
    { 0x4e5e1d6e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xffbe1d71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89e1d76, SUC_lexicon_tags_25 },
    { 0x6ede1d77, SUC_lexicon_tags_32 },
    { 0x701e1d78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde5e1d7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43fe1d7c, SUC_lexicon_tags_33 },
    { 0x7b1e1d7c, SUC_lexicon_tags_47 },
    { 0xd59e1d7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe1d83, SUC_lexicon_tags_12 },
    { 0x493e1d84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e1d87, SUC_lexicon_tags_15 },
    { 0x64be1d87, SUC_lexicon_tags_6 },
    { 0x5afe1d89, SUC_lexicon_tags_7 },
    { 0x77e1d8a, SUC_lexicon_tags_37 },
    { 0xcf7e1d8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9e1d90, SUC_lexicon_tags_7 },
    { 0xd53e1d90, SUC_lexicon_tags_25 },
    { 0xa8de1d92, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf21e1d94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x979e1d97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x271e1d9a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633e1da4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7e1da7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe1daa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731e1db0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x727e1db2, SUC_lexicon_tags_7 },
    { 0x769e1db3, SUC_lexicon_tags_17 },
    { 0x783e1db4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3e1dbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacfe1dc4, SUC_lexicon_tags_1319 },
    { 0xa09e1dc5, SUC_lexicon_tags_13 },
    { 0xd13e1dc5, SUC_lexicon_tags_45 },
    { 0xbf3e1dc7, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x50de1dc9, SUC_lexicon_tags_10 },
    { 0x93e1dc9, SUC_lexicon_tags_14 },
    { 0xbf9e1dcb, SUC_lexicon_tags_15 },
    { 0xcbe1dcc, SUC_lexicon_tags_33 },
    { 0x585e1dcc, SUC_lexicon_tags_184 },
    { 0x97be1dce, SUC_lexicon_tags_39 },
    { 0x429e1dcc, SUC_lexicon_tags_20 },
    { 0x19fe1dca, SUC_lexicon_tags_47 },
    { 0x77e1dd1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa07e1dd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21e1dd8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa9de1dda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbbe1ddc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabe1dec, SUC_lexicon_tags_11 },
    { 0xc97e1ded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf37e1def, SUC_lexicon_tags_8 },
    { 0x5c9e1df0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ade1df6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x717e1df8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e1dfe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873e1e02, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21e1e06, SUC_lexicon_tags_1 },
    { 0xd41e1e06, SUC_lexicon_tags_45 },
    { 0xa37e1e07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9de1e0a, SUC_lexicon_tags_7 },
    { 0x53e1e0b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6bfe1e0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cde1e14, SUC_lexicon_tags_7 },
    { 0xb2be1e14, SUC_lexicon_tags_37 },
    { 0x1f1e1e16, SUC_lexicon_tags_1 },
    { 0xe13e1e16, SUC_lexicon_tags_47 },
    { 0xfd7e1e15, SUC_lexicon_tags_45 },
    { 0xd6fe1e16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedde1e1d, SUC_lexicon_tags_9 },
    { 0x66fe1e1e, SUC_lexicon_tags_12 },
    { 0x3c1e1e1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551e1e27, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19be1e2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9e1e30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91e1e35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x43be1e38, SUC_lexicon_tags_35 },
    { 0x335e1e39, SUC_lexicon_tags_8 },
    { 0x50fe1e3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a1e1e3c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3e1e47, SUC_lexicon_tags_12 },
    { 0xd77e1e48, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb1e1e4a, SUC_lexicon_tags_17 },
    { 0xaa9e1e4b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85be1e4f, SUC_lexicon_tags_19 },
    { 0x1de1e50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9e1e59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x895e1e5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47be1e5f, SUC_lexicon_tags_55 },
    { 0xcf9e1e60, SUC_lexicon_tags_2 },
    { 0x885e1e61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19be1e66, SUC_lexicon_tags_30 },
    { 0x6e3e1e67, SUC_lexicon_tags_12 },
    { 0xe35e1e68, SUC_lexicon_tags_7 },
    { 0xdc9e1e69, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525e1e6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933e1e74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xab5e1e76, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x555e1e79, SUC_lexicon_tags_12 },
    { 0x45fe1e79, SUC_lexicon_tags_11 },
    { 0x24be1e7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf93e1e7d, SUC_lexicon_tags_11 },
    { 0xecfe1e7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1e1e82, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f1e1e84, SUC_lexicon_tags_7 },
    { 0x15de1e85, SUC_lexicon_tags_11 },
    { 0x237e1e85, SUC_lexicon_tags_47 },
    { 0xbf7e1e85, SUC_lexicon_tags_60 },
    { 0xb21e1e88, SUC_lexicon_tags_34 },
    { 0x539e1e89, SUC_lexicon_tags_56 },
    { 0x81be1e85, SUC_lexicon_tags_28 },
    { 0xc01e1e8b, SUC_lexicon_tags_60 },
    { 0xe83e1e84, SUC_lexicon_tags_43 },
    { 0x1e5e1e86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdde1e90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bde1e94, SUC_lexicon_tags_178 },
    { 0x10fe1e94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffe1e99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fe1ea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33be1eac, SUC_lexicon_tags_61 },
    { 0x73be1eac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7e1eb0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x17de1eb3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x88fe1eb5, SUC_lexicon_tags_63 },
    { 0xb4fe1eb6, SUC_lexicon_tags_46 },
    { 0xa0be1eb5, SUC_lexicon_tags_43 },
    { 0x591e1eb8, SUC_lexicon_tags_6 },
    { 0x149e1eb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbe1ebe, SUC_lexicon_tags_32 },
    { 0xe9be1ebf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239e1ec3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa17e1ec5, SUC_lexicon_tags_60 },
    { 0x3a3e1ec6, SUC_lexicon_tags_17 },
    { 0x8f1e1ec7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfde1ecb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x969e1ece, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebe1ed4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbebe1ed6, SUC_lexicon_tags_15 },
    { 0x735e1ed7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x757e1ed9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x219e1edb, SUC_lexicon_tags_2 },
    { 0x1efe1edc, SUC_lexicon_tags_11 },
    { 0xf8be1edd, SUC_lexicon_tags_11 },
    { 0x7f1e1edd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88fe1ee0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x727e1ee3, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3e1ee7, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59e1eeb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x209e1eee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7e1ef1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1e1ef5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcde1efa, SUC_lexicon_tags_39 },
    { 0x979e1efb, SUC_lexicon_tags_60 },
    { 0x235e1efa, SUC_lexicon_tags_37 },
    { 0x873e1efd, SUC_lexicon_tags_14 },
    { 0x5d1e1efe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e1f02, SUC_lexicon_tags_31 },
    { 0xb27e1f02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37e1f07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65e1f09, SUC_lexicon_tags_45 },
    { 0x21e1f0a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e1f0e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681e1f16, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fe1f1c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9e1f20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe99e1f22, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947e1f27, SUC_lexicon_tags_17 },
    { 0xe25e1f27, SUC_lexicon_tags_42 },
    { 0xc13e1f29, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e1f2f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1e1f32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfe1f35, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1e5e1f37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7e1f3a, SUC_lexicon_tags_37 },
    { 0xff5e1f3a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x47be1f3d, SUC_lexicon_tags_17 },
    { 0x26fe1f3d, SUC_lexicon_tags_63 },
    { 0x841e1f3e, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcede1f41, SUC_lexicon_tags_27 },
    { 0x869e1f41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5e1f45, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa8be1f47, SUC_lexicon_tags_2 },
    { 0x825e1f47, SUC_lexicon_tags_6 },
    { 0x21be1f49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x107e1f4b, SUC_lexicon_tags_11 },
    { 0x8f1e1f4b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe1f51, SUC_lexicon_tags_48 },
    { 0xab1e1f51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x391e1f55, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1e1f58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefe1f5c, SUC_lexicon_tags_408 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41e1f5f, SUC_lexicon_tags_19 },
    { 0x22be1f60, SUC_lexicon_tags_1 },
    { 0xeede1f5f, SUC_lexicon_tags_12 },
    { 0x581e1f62, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x537e1f65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x55fe1f67, SUC_lexicon_tags_11 },
    { 0x9afe1f67, SUC_lexicon_tags_37 },
    { 0x747e1f68, SUC_lexicon_tags_17 },
    { 0xacbe1f6a, SUC_lexicon_tags_11 },
    { 0xa5fe1f6b, SUC_lexicon_tags_15 },
    { 0x1d1e1f6c, SUC_lexicon_tags_10 },
    { 0xf15e1f6a, SUC_lexicon_tags_0 },
    { 0xcfe1f6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x515e1f70, SUC_lexicon_tags_37 },
    { 0x153e1f71, SUC_lexicon_tags_23 },
    { 0x169e1f72, SUC_lexicon_tags_0 },
    { 0x1e9e1f72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x98de1f76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889e1f7f, SUC_lexicon_tags_22 },
    { 0xc07e1f80, SUC_lexicon_tags_17 },
    { 0xa0de1f81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x85be1f84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb1be1f86, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5afe1f89, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7e1f8e, SUC_lexicon_tags_15 },
    { 0x109e1f8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07e1f92, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947e1f96, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1e1f9a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ede1f9e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fe1fa2, SUC_lexicon_tags_3 },
    { 0xc65e1fa2, SUC_lexicon_tags_4 },
    { 0xb2fe1fa4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c9e1fa6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73be1fad, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf9e1faf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7e1fb1, SUC_lexicon_tags_14 },
    { 0x7b1e1fb2, SUC_lexicon_tags_38 },
    { 0x499e1fb3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfe1fb9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6de1fbc, SUC_lexicon_tags_21 },
    { 0xd07e1fbd, SUC_lexicon_tags_14 },
    { 0x14be1fbd, SUC_lexicon_tags_66 },
    { 0x87fe1fbf, SUC_lexicon_tags_2 },
    { 0x593e1fbf, SUC_lexicon_tags_17 },
    { 0xbe1e1fc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23fe1fc3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fe1fc6, SUC_lexicon_tags_63 },
    { 0xd1de1fc7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe1fcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5f9e1fce, SUC_lexicon_tags_0 },
    { 0x37fe1fcf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fe1fd3, SUC_lexicon_tags_37 },
    { 0x397e1fd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7e1fd9, SUC_lexicon_tags_15 },
    { 0x9b3e1fd9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7e1fde, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41e1fe1, SUC_lexicon_tags_47 },
    { 0x633e1fe1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x699e1fe5, SUC_lexicon_tags_67 },
    { 0x7dde1fe6, SUC_lexicon_tags_45 },
    { 0xfdfe1fe6, SUC_lexicon_tags_0 },
    { 0xa49e1fe8, SUC_lexicon_tags_3 },
    { 0x25be1fe8, SUC_lexicon_tags_2 },
    { 0xc35e1fe8, SUC_lexicon_tags_25 },
    { 0xd03e1fe7, SUC_lexicon_tags_25 },
    { 0x41be1fec, SUC_lexicon_tags_14 },
    { 0x573e1fe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9bbe1fef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7e1ff4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1e1ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27e2005, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x62be2007, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdade2009, SUC_lexicon_tags_21 },
    { 0xedbe200a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x28be200d, SUC_lexicon_tags_2 },
    { 0xf1be200d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb59e2010, SUC_lexicon_tags_15 },
    { 0xfd1e2010, SUC_lexicon_tags_7 },
    { 0xba9e2012, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x761e2014, SUC_lexicon_tags_25 },
    { 0x38fe2015, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be201a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb93e201c, SUC_lexicon_tags_7 },
    { 0x5efe201d, SUC_lexicon_tags_17 },
    { 0xb01e201d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd93e2020, SUC_lexicon_tags_15 },
    { 0x42de2020, SUC_lexicon_tags_48 },
    { 0x175e2022, SUC_lexicon_tags_55 },
    { 0x7b7e2023, SUC_lexicon_tags_11 },
    { 0xb7be2024, SUC_lexicon_tags_30 },
    { 0x395e2023, SUC_lexicon_tags_45 },
    { 0x935e2024, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfde202d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21e2030, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaade2033, SUC_lexicon_tags_57 },
    { 0xcede2034, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5e2038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x375e203a, SUC_lexicon_tags_46 },
    { 0x4fe203b, SUC_lexicon_tags_17 },
    { 0xfc7e203a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25e2042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fe2046, SUC_lexicon_tags_19 },
    { 0xb05e2047, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x91de2049, SUC_lexicon_tags_61 },
    { 0x6a5e204a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7dfe204c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e2050, SUC_lexicon_tags_2 },
    { 0x67e2051, SUC_lexicon_tags_0 },
    { 0x57fe2052, SUC_lexicon_tags_17 },
    { 0x29be2051, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xce5e2055, SUC_lexicon_tags_572 },
    { 0xb4be2056, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10be205b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fe205e, SUC_lexicon_tags_15 },
    { 0xd7be205f, SUC_lexicon_tags_6 },
    { 0xfdbe205e, SUC_lexicon_tags_32 },
    { 0xb71e2061, SUC_lexicon_tags_37 },
    { 0x9bfe205f, SUC_lexicon_tags_25 },
    { 0x97fe2063, SUC_lexicon_tags_0 },
    { 0x583e2061, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709e2069, SUC_lexicon_tags_11 },
    { 0xfde206a, SUC_lexicon_tags_1 },
    { 0xb03e206b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39de206e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd09e2070, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcf7e2072, SUC_lexicon_tags_15 },
    { 0xe9e2072, SUC_lexicon_tags_43 },
    { 0x8f9e2074, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x559e2077, SUC_lexicon_tags_14 },
    { 0x81de2077, SUC_lexicon_tags_7 },
    { 0x7f7e2079, SUC_lexicon_tags_17 },
    { 0x94de207a, SUC_lexicon_tags_2 },
    { 0xc81e207a, SUC_lexicon_tags_29 },
    { 0xd3fe207c, SUC_lexicon_tags_14 },
    { 0x4bde207d, SUC_lexicon_tags_13 },
    { 0x605e207d, SUC_lexicon_tags_25 },
    { 0x5fe2078, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x281e2081, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcde2084, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x967e2086, SUC_lexicon_tags_25 },
    { 0x8b9e2086, SUC_lexicon_tags_22 },
    { 0x74fe2087, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0x7c3e208a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x685e208c, SUC_lexicon_tags_31 },
    { 0xb61e208c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515e2091, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70de2093, SUC_lexicon_tags_56 },
    { 0xa63e2093, SUC_lexicon_tags_60 },
    { 0x67be2093, SUC_lexicon_tags_29 },
    { 0xb75e2096, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x915e2099, SUC_lexicon_tags_7 },
    { 0xd21e2099, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1e20a4, SUC_lexicon_tags_37 },
    { 0xc97e20a5, SUC_lexicon_tags_60 },
    { 0x43de20a6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3e20a8, SUC_lexicon_tags_47 },
    { 0x9f5e20a9, SUC_lexicon_tags_25 },
    { 0xab7e20aa, SUC_lexicon_tags_13 },
    { 0xaa3e20ab, SUC_lexicon_tags_12 },
    { 0x5c7e20ab, SUC_lexicon_tags_19 },
    { 0x8cbe20a8, SUC_lexicon_tags_43 },
    { 0xd3e20ae, SUC_lexicon_tags_49 },
    { 0xee5e20ac, SUC_lexicon_tags_37 },
    { 0x4a1e20aa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x853e20b2, SUC_lexicon_tags_26 },
    { 0x393e20b3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fe20b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedbe20b8, SUC_lexicon_tags_11 },
    { 0xb3be20b9, SUC_lexicon_tags_0 },
    { 0x6b7e20b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x24fe20bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc39e20be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x535e20c0, SUC_lexicon_tags_11 },
    { 0xdfbe20c0, SUC_lexicon_tags_14 },
    { 0x4cbe20c2, SUC_lexicon_tags_7 },
    { 0x7ade20c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x241e20c6, SUC_lexicon_tags_37 },
    { 0x36be20c7, SUC_lexicon_tags_45 },
    { 0xc9fe20c8, SUC_lexicon_tags_17 },
    { 0x2dbe20c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3fe20cb, SUC_lexicon_tags_27 },
    { 0x2e1e20cb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x973e20ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7e20d4, SUC_lexicon_tags_282 },
    { 0, NULL },
    { 0x47e20d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9b5e20d8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9de20db, SUC_lexicon_tags_19 },
    { 0xfdde20db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7e20e3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf15e20e5, SUC_lexicon_tags_15 },
    { 0xc99e20e5, SUC_lexicon_tags_1 },
    { 0x7e7e20e7, SUC_lexicon_tags_11 },
    { 0xa11e20e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7fe20ea, SUC_lexicon_tags_17 },
    { 0x10de20ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07e20ee, SUC_lexicon_tags_9 },
    { 0x5dde20ef, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x79fe20f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8d1e20f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32de20f8, SUC_lexicon_tags_17 },
    { 0x455e20f8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13e20fc, SUC_lexicon_tags_87 },
    { 0x765e20fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda9e20ff, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x107e2102, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3e2109, SUC_lexicon_tags_17 },
    { 0xacbe210a, SUC_lexicon_tags_33 },
    { 0x9c9e2109, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813e210f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73fe2111, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8be2114, SUC_lexicon_tags_15 },
    { 0x63fe2115, SUC_lexicon_tags_12 },
    { 0x84be2114, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x837e2118, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99e211c, SUC_lexicon_tags_16 },
    { 0x5b7e211d, SUC_lexicon_tags_5 },
    { 0xf57e211e, SUC_lexicon_tags_60 },
    { 0x89e211d, SUC_lexicon_tags_45 },
    { 0x88de211e, SUC_lexicon_tags_49 },
    { 0xe91e211d, SUC_lexicon_tags_17 },
    { 0x141e2122, SUC_lexicon_tags_14 },
    { 0xefde2123, SUC_lexicon_tags_12 },
    { 0xe83e2122, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc63e2126, SUC_lexicon_tags_26 },
    { 0xdf3e2127, SUC_lexicon_tags_37 },
    { 0x4afe2127, SUC_lexicon_tags_37 },
    { 0x11de2126, SUC_lexicon_tags_30 },
    { 0x1d3e2126, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xdcde212c, SUC_lexicon_tags_463 },
    { 0x79e212d, SUC_lexicon_tags_1 },
    { 0x2e1e212d, SUC_lexicon_tags_1 },
    { 0x9f9e212f, SUC_lexicon_tags_1 },
    { 0xecbe212e, SUC_lexicon_tags_26 },
    { 0x503e2131, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbde2135, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479e2139, SUC_lexicon_tags_1 },
    { 0x4e3e213a, SUC_lexicon_tags_2 },
    { 0x7e7e213a, SUC_lexicon_tags_49 },
    { 0xf8de2139, SUC_lexicon_tags_17 },
    { 0x8f7e213b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x679e213f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb91e2141, SUC_lexicon_tags_4 },
    { 0x26de2142, SUC_lexicon_tags_11 },
    { 0x591e2143, SUC_lexicon_tags_71 },
    { 0x6f3e2142, SUC_lexicon_tags_3 },
    { 0xb71e2145, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x10be2148, SUC_lexicon_tags_30 },
    { 0x2cde2149, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5be214e, SUC_lexicon_tags_18 },
    { 0x2c1e214f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfde2160, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13e2164, SUC_lexicon_tags_31 },
    { 0xd17e2165, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6a7e2167, SUC_lexicon_tags_37 },
    { 0x7fde2168, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5de216b, SUC_lexicon_tags_6 },
    { 0x851e216b, SUC_lexicon_tags_37 },
    { 0xd8de216d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade2173, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbe2179, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71e217d, SUC_lexicon_tags_7 },
    { 0x249e217e, SUC_lexicon_tags_47 },
    { 0x4ade217f, SUC_lexicon_tags_17 },
    { 0xae7e2180, SUC_lexicon_tags_27 },
    { 0xcede2180, SUC_lexicon_tags_49 },
    { 0x463e2180, SUC_lexicon_tags_49 },
    { 0x401e217d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789e218e, SUC_lexicon_tags_0 },
    { 0xcbbe218f, SUC_lexicon_tags_13 },
    { 0x7b5e218e, SUC_lexicon_tags_25 },
    { 0xd0fe218f, SUC_lexicon_tags_49 },
    { 0x501e2192, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x709e2194, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x217e2196, SUC_lexicon_tags_11 },
    { 0x127e2197, SUC_lexicon_tags_38 },
    { 0x521e2196, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40de219a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x817e219c, SUC_lexicon_tags_28 },
    { 0x273e219d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x455e219f, SUC_lexicon_tags_2 },
    { 0x9e5e21a0, SUC_lexicon_tags_11 },
    { 0x3d5e21a1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05e21a7, SUC_lexicon_tags_23 },
    { 0x4ade21a8, SUC_lexicon_tags_17 },
    { 0x3bde21a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfe21ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83be21b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x71be21b3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5be21b5, SUC_lexicon_tags_23 },
    { 0x52fe21b6, SUC_lexicon_tags_55 },
    { 0xafbe21b6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283e21bd, SUC_lexicon_tags_46 },
    { 0x7a3e21be, SUC_lexicon_tags_2 },
    { 0x89de21be, SUC_lexicon_tags_11 },
    { 0xe65e21c0, SUC_lexicon_tags_15 },
    { 0xf61e21c0, SUC_lexicon_tags_7 },
    { 0xd8de21be, SUC_lexicon_tags_86 },
    { 0xb4de21c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89e21cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3e21ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5de21d3, SUC_lexicon_tags_66 },
    { 0xb05e21d4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fe21db, SUC_lexicon_tags_2 },
    { 0x585e21dc, SUC_lexicon_tags_91 },
    { 0x139e21db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e21e0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875e21e8, SUC_lexicon_tags_11 },
    { 0xed5e21e9, SUC_lexicon_tags_37 },
    { 0xabfe21e8, SUC_lexicon_tags_17 },
    { 0xba1e21ea, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc37e21ed, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afe21f0, SUC_lexicon_tags_2 },
    { 0xf05e21f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627e21ff, SUC_lexicon_tags_19 },
    { 0xbefe21ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1e2204, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efe220e, SUC_lexicon_tags_25 },
    { 0x16be220e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569e2213, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x76fe2216, SUC_lexicon_tags_106 },
    { 0x9b7e2216, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afe221c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205e2221, SUC_lexicon_tags_0 },
    { 0xc49e2222, SUC_lexicon_tags_3 },
    { 0x287e2221, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x347e2226, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x22fe2229, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fe2230, SUC_lexicon_tags_39 },
    { 0xdf1e2231, SUC_lexicon_tags_7 },
    { 0xfcfe2232, SUC_lexicon_tags_33 },
    { 0xc2fe2233, SUC_lexicon_tags_65 },
    { 0x895e2234, SUC_lexicon_tags_25 },
    { 0x371e2235, SUC_lexicon_tags_11 },
    { 0x5f1e2236, SUC_lexicon_tags_29 },
    { 0x2ebe2234, SUC_lexicon_tags_17 },
    { 0xa95e2238, SUC_lexicon_tags_37 },
    { 0xf87e2239, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x369e223d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6cbe223f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x503e2242, SUC_lexicon_tags_33 },
    { 0x109e2242, SUC_lexicon_tags_49 },
    { 0x487e2244, SUC_lexicon_tags_37 },
    { 0x329e2245, SUC_lexicon_tags_11 },
    { 0xbafe2244, SUC_lexicon_tags_47 },
    { 0x919e2247, SUC_lexicon_tags_23 },
    { 0xeabe2246, SUC_lexicon_tags_37 },
    { 0xd19e2249, SUC_lexicon_tags_33 },
    { 0x86de2245, SUC_lexicon_tags_11 },
    { 0xe85e224a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7e224e, SUC_lexicon_tags_46 },
    { 0x85de224e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2be2255, SUC_lexicon_tags_1 },
    { 0x1be2255, SUC_lexicon_tags_12 },
    { 0xd0fe2257, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95e225a, SUC_lexicon_tags_12 },
    { 0xb3de225a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e225f, SUC_lexicon_tags_43 },
    { 0x3c1e225f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefe2265, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x109e2267, SUC_lexicon_tags_91 },
    { 0xcfde2268, SUC_lexicon_tags_45 },
    { 0x65fe2267, SUC_lexicon_tags_11 },
    { 0xa51e226a, SUC_lexicon_tags_37 },
    { 0x59e2267, SUC_lexicon_tags_11 },
    { 0xffde226c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60be2271, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8ebe2273, SUC_lexicon_tags_25 },
    { 0xd7fe2274, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x225e2276, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xadde2279, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8fbe227b, SUC_lexicon_tags_21 },
    { 0xfbde227c, SUC_lexicon_tags_26 },
    { 0xe75e227d, SUC_lexicon_tags_60 },
    { 0x4d5e227e, SUC_lexicon_tags_7 },
    { 0xe01e227f, SUC_lexicon_tags_11 },
    { 0x7f7e2280, SUC_lexicon_tags_60 },
    { 0x585e2281, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x75de2283, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9e2286, SUC_lexicon_tags_26 },
    { 0xebfe2286, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1e228a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6be228d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb9de228f, SUC_lexicon_tags_2 },
    { 0xe4fe2290, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19e2294, SUC_lexicon_tags_15 },
    { 0xa95e2295, SUC_lexicon_tags_2 },
    { 0xfa9e2296, SUC_lexicon_tags_15 },
    { 0xa1fe2297, SUC_lexicon_tags_25 },
    { 0x23de2297, SUC_lexicon_tags_12 },
    { 0x12fe2299, SUC_lexicon_tags_27 },
    { 0x7ade229a, SUC_lexicon_tags_0 },
    { 0x571e2294, SUC_lexicon_tags_112 },
    { 0x6f5e229c, SUC_lexicon_tags_38 },
    { 0xc6fe229d, SUC_lexicon_tags_2 },
    { 0x2f7e2294, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38de22a0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd59e22a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x163e22a6, SUC_lexicon_tags_37 },
    { 0x72be22a7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xade22a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3e22ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72fe22ae, SUC_lexicon_tags_27 },
    { 0x30be22ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf79e22ba, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x667e22bc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x333e22be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8dbe22c0, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0xf27e22c3, SUC_lexicon_tags_37 },
    { 0x24fe22c4, SUC_lexicon_tags_38 },
    { 0x44be22c5, SUC_lexicon_tags_25 },
    { 0xaf7e22c6, SUC_lexicon_tags_23 },
    { 0xc17e22c7, SUC_lexicon_tags_15 },
    { 0x289e22c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xedde22ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3e22ce, SUC_lexicon_tags_2 },
    { 0x9c7e22cf, SUC_lexicon_tags_19 },
    { 0xe71e22cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf9e22d2, SUC_lexicon_tags_37 },
    { 0xbfe22d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd85e22d5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfefe22d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92be22db, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7e22e2, SUC_lexicon_tags_17 },
    { 0x89fe22e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e22e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399e22eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x68de22ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5e22f2, SUC_lexicon_tags_7 },
    { 0x5a3e22f3, SUC_lexicon_tags_15 },
    { 0x611e22f3, SUC_lexicon_tags_11 },
    { 0x86de22f5, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fe22f8, SUC_lexicon_tags_12 },
    { 0x6dbe22f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bde22fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd31e22ff, SUC_lexicon_tags_28 },
    { 0xf19e2300, SUC_lexicon_tags_12 },
    { 0x115e2300, SUC_lexicon_tags_15 },
    { 0x6e1e22ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe2308, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x501e230b, SUC_lexicon_tags_25 },
    { 0x965e230c, SUC_lexicon_tags_1 },
    { 0xcbbe230d, SUC_lexicon_tags_7 },
    { 0xfc9e230c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9d3e2310, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8be2313, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bde2316, SUC_lexicon_tags_11 },
    { 0x9c5e2316, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0de2319, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205e2320, SUC_lexicon_tags_8 },
    { 0x361e2321, SUC_lexicon_tags_43 },
    { 0x2bfe2320, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c5e2324, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afe2328, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865e2330, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11fe2334, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x715e2336, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe69e2338, SUC_lexicon_tags_17 },
    { 0xc15e2339, SUC_lexicon_tags_26 },
    { 0xd85e233a, SUC_lexicon_tags_19 },
    { 0xf73e233a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229e2341, SUC_lexicon_tags_25 },
    { 0x77fe2342, SUC_lexicon_tags_70 },
    { 0x713e2342, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2de2346, SUC_lexicon_tags_33 },
    { 0x4d1e2346, SUC_lexicon_tags_32 },
    { 0x24de2348, SUC_lexicon_tags_69 },
    { 0x131e2347, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x613e234c, SUC_lexicon_tags_31 },
    { 0x92de234c, SUC_lexicon_tags_63 },
    { 0x60be234e, SUC_lexicon_tags_17 },
    { 0x23de234f, SUC_lexicon_tags_11 },
    { 0xf91e234e, SUC_lexicon_tags_7 },
    { 0x6f1e2351, SUC_lexicon_tags_0 },
    { 0xbbe2351, SUC_lexicon_tags_15 },
    { 0x5ede2353, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe2356, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427e235b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa93e235d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x875e235f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fe2363, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979e236a, SUC_lexicon_tags_32 },
    { 0x199e236b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x18fe236d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe2370, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xacfe2372, SUC_lexicon_tags_7 },
    { 0xfd7e2372, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe2377, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fe2380, SUC_lexicon_tags_26 },
    { 0xcbfe2380, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155e2385, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3e2389, SUC_lexicon_tags_47 },
    { 0x527e238a, SUC_lexicon_tags_30 },
    { 0xddde238a, SUC_lexicon_tags_17 },
    { 0x8f3e238c, SUC_lexicon_tags_49 },
    { 0xd83e238b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79be2393, SUC_lexicon_tags_37 },
    { 0x813e2394, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efe2399, SUC_lexicon_tags_4 },
    { 0x545e2399, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fe239f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x619e23a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaede23a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7e23a9, SUC_lexicon_tags_2 },
    { 0xe41e23a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d3e23ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfcfe23ae, SUC_lexicon_tags_14 },
    { 0x2bbe23af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfe23b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9e23b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efe23bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1e23be, SUC_lexicon_tags_6 },
    { 0x1bfe23bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fe23c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939e23c8, SUC_lexicon_tags_19 },
    { 0xeb9e23c9, SUC_lexicon_tags_66 },
    { 0xc0be23c8, SUC_lexicon_tags_27 },
    { 0xb75e23cb, SUC_lexicon_tags_12 },
    { 0x8e5e23cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1e23cf, SUC_lexicon_tags_25 },
    { 0x91de23d0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1de23d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x13de23d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x481e23d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53de23da, SUC_lexicon_tags_15 },
    { 0x6a1e23db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x553e23de, SUC_lexicon_tags_42 },
    { 0x44fe23de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1e23e3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbe23ea, SUC_lexicon_tags_25 },
    { 0xf97e23eb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x825e23ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x735e23f0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29fe23f4, SUC_lexicon_tags_47 },
    { 0xf47e23f4, SUC_lexicon_tags_47 },
    { 0xca5e23f6, SUC_lexicon_tags_12 },
    { 0xb17e23f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77e23f9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xdde23fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xacde23ff, SUC_lexicon_tags_2 },
    { 0x3d3e2400, SUC_lexicon_tags_11 },
    { 0x969e2401, SUC_lexicon_tags_15 },
    { 0xb17e2402, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9be2405, SUC_lexicon_tags_3 },
    { 0x935e2406, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75fe2408, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e240c, SUC_lexicon_tags_25 },
    { 0x5a5e240d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x53de240f, SUC_lexicon_tags_1 },
    { 0x2fde2410, SUC_lexicon_tags_28 },
    { 0x9be2411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373e2417, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357e241b, SUC_lexicon_tags_17 },
    { 0x8dbe241b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187e2427, SUC_lexicon_tags_23 },
    { 0x941e2428, SUC_lexicon_tags_0 },
    { 0xf53e2429, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x87fe242c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xc9fe242e, SUC_lexicon_tags_15 },
    { 0xc2de242e, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x993e2431, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x57fe2433, SUC_lexicon_tags_1 },
    { 0xefbe2433, SUC_lexicon_tags_160 },
    { 0xc43e2435, SUC_lexicon_tags_37 },
    { 0x7e7e2436, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fe243b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fe243f, SUC_lexicon_tags_15 },
    { 0xa49e2440, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fe2446, SUC_lexicon_tags_27 },
    { 0x9d5e2446, SUC_lexicon_tags_30 },
    { 0x819e2446, SUC_lexicon_tags_39 },
    { 0xc4de2448, SUC_lexicon_tags_26 },
    { 0x10de2446, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe2450, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3e2454, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34de245c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3e2460, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1e2463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x26de2465, SUC_lexicon_tags_11 },
    { 0x7a1e2465, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17e246c, SUC_lexicon_tags_27 },
    { 0x9a9e246d, SUC_lexicon_tags_7 },
    { 0x9c7e246e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xc5de2470, SUC_lexicon_tags_17 },
    { 0xdf5e2470, SUC_lexicon_tags_1 },
    { 0x9fe2472, SUC_lexicon_tags_17 },
    { 0xbe2472, SUC_lexicon_tags_12 },
    { 0xee9e2474, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x29e2476, SUC_lexicon_tags_12 },
    { 0xe9be2477, SUC_lexicon_tags_15 },
    { 0xe9e2478, SUC_lexicon_tags_38 },
    { 0xf05e2477, SUC_lexicon_tags_1 },
    { 0x439e2478, SUC_lexicon_tags_0 },
    { 0x165e2477, SUC_lexicon_tags_69 },
    { 0x669e247c, SUC_lexicon_tags_49 },
    { 0x1c3e247c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7e2480, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23e2484, SUC_lexicon_tags_6 },
    { 0x15e2485, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539e2492, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5e2498, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xadfe249a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x33e249c, SUC_lexicon_tags_2 },
    { 0x37be249d, SUC_lexicon_tags_16 },
    { 0xf6fe249d, SUC_lexicon_tags_11 },
    { 0x395e249f, SUC_lexicon_tags_23 },
    { 0x775e249e, SUC_lexicon_tags_17 },
    { 0x195e249d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x313e24a3, SUC_lexicon_tags_148 },
    { 0x8f9e24a3, SUC_lexicon_tags_68 },
    { 0xef9e24a5, SUC_lexicon_tags_25 },
    { 0x8dbe24a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5e24af, SUC_lexicon_tags_25 },
    { 0xc9fe24b0, SUC_lexicon_tags_26 },
    { 0xbbde24b1, SUC_lexicon_tags_39 },
    { 0xa8be24b2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5a5e24b4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb0de24b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9e24b9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x833e24bb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc9de24bd, SUC_lexicon_tags_7 },
    { 0x89de24be, SUC_lexicon_tags_28 },
    { 0xbf3e24be, SUC_lexicon_tags_2 },
    { 0x4ede24bf, SUC_lexicon_tags_5 },
    { 0x775e24c1, SUC_lexicon_tags_17 },
    { 0x263e24c2, SUC_lexicon_tags_27 },
    { 0xbfbe24c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xad5e24c5, SUC_lexicon_tags_26 },
    { 0x707e24c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23e24ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9c1e24d0, SUC_lexicon_tags_32 },
    { 0x551e24d1, SUC_lexicon_tags_12 },
    { 0x8f5e24d2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x233e24d4, SUC_lexicon_tags_37 },
    { 0x5b5e24d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x329e24d7, SUC_lexicon_tags_31 },
    { 0x1d3e24d8, SUC_lexicon_tags_148 },
    { 0xa6fe24d8, SUC_lexicon_tags_11 },
    { 0xd57e24da, SUC_lexicon_tags_37 },
    { 0x5f1e24d7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x13de24de, SUC_lexicon_tags_178 },
    { 0x99e24df, SUC_lexicon_tags_1 },
    { 0x2fe24e0, SUC_lexicon_tags_30 },
    { 0xfdde24e1, SUC_lexicon_tags_45 },
    { 0xea9e24df, SUC_lexicon_tags_1 },
    { 0x199e24e3, SUC_lexicon_tags_14 },
    { 0x26be24e3, SUC_lexicon_tags_12 },
    { 0xf9be24e5, SUC_lexicon_tags_10 },
    { 0x563e24e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x231e24e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x79fe24ea, SUC_lexicon_tags_12 },
    { 0x425e24ea, SUC_lexicon_tags_12 },
    { 0xe67e24ec, SUC_lexicon_tags_7 },
    { 0xd23e24ed, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe9de24ef, SUC_lexicon_tags_17 },
    { 0xe4be24ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531e24f7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5e24fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe2501, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfefe2503, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbe2505, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883e250d, SUC_lexicon_tags_17 },
    { 0xc7e250e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebfe2510, SUC_lexicon_tags_14 },
    { 0x7fe2510, SUC_lexicon_tags_25 },
    { 0x9ebe2512, SUC_lexicon_tags_28 },
    { 0xc07e2513, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfe2516, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78fe251d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1e2525, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc9e2527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x487e2529, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53e252e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181e2532, SUC_lexicon_tags_49 },
    { 0x629e2533, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x127e2535, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95e2538, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9e253b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x261e253d, SUC_lexicon_tags_37 },
    { 0xb67e253d, SUC_lexicon_tags_28 },
    { 0xb89e253f, SUC_lexicon_tags_25 },
    { 0x8bfe2540, SUC_lexicon_tags_33 },
    { 0x8a3e2541, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xb53e2543, SUC_lexicon_tags_11 },
    { 0x19de2544, SUC_lexicon_tags_37 },
    { 0x3a5e2545, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1e2548, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75be254d, SUC_lexicon_tags_54 },
    { 0x47de254d, SUC_lexicon_tags_9 },
    { 0x8ede254f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281e255b, SUC_lexicon_tags_19 },
    { 0xe99e255c, SUC_lexicon_tags_7 },
    { 0x2ede255d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85e2562, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x61be2565, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7e2569, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3e256c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5de2574, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x87e2576, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5e2579, SUC_lexicon_tags_1 },
    { 0x17de2579, SUC_lexicon_tags_11 },
    { 0x263e257a, SUC_lexicon_tags_34 },
    { 0x18be257c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x777e257e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb55e2580, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x62be2582, SUC_lexicon_tags_2 },
    { 0xb15e2582, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe2588, SUC_lexicon_tags_27 },
    { 0x1e3e2589, SUC_lexicon_tags_37 },
    { 0xb5fe2589, SUC_lexicon_tags_15 },
    { 0xe81e258b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbe258e, SUC_lexicon_tags_68 },
    { 0x4ede258f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315e2595, SUC_lexicon_tags_17 },
    { 0x6abe2596, SUC_lexicon_tags_45 },
    { 0x3a9e2597, SUC_lexicon_tags_44 },
    { 0x1bbe2598, SUC_lexicon_tags_11 },
    { 0xadfe2599, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbbfe259b, SUC_lexicon_tags_11 },
    { 0xdf1e259c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217e25a1, SUC_lexicon_tags_33 },
    { 0xe8de25a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1be25a5, SUC_lexicon_tags_7 },
    { 0xdcfe25a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3e25ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fe25b0, SUC_lexicon_tags_15 },
    { 0x895e25b1, SUC_lexicon_tags_45 },
    { 0x569e25b2, SUC_lexicon_tags_84 },
    { 0x6f5e25b3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fe25b6, SUC_lexicon_tags_47 },
    { 0x245e25b7, SUC_lexicon_tags_37 },
    { 0x71de25b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65e25bd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91e25c2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe65e25c4, SUC_lexicon_tags_33 },
    { 0x3e5e25c5, SUC_lexicon_tags_17 },
    { 0x76fe25c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99e25cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7e25d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45de25dc, SUC_lexicon_tags_11 },
    { 0x879e25dd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xa3fe25df, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xee5e25e1, SUC_lexicon_tags_1 },
    { 0x597e25e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389e25e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x44fe25e8, SUC_lexicon_tags_8 },
    { 0xbfe25e8, SUC_lexicon_tags_28 },
    { 0xf03e25e9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7e25ef, SUC_lexicon_tags_11 },
    { 0xe5e25f0, SUC_lexicon_tags_30 },
    { 0x7de25f1, SUC_lexicon_tags_2 },
    { 0xc29e25ef, SUC_lexicon_tags_14 },
    { 0x6fe25f3, SUC_lexicon_tags_23 },
    { 0x735e25f4, SUC_lexicon_tags_65 },
    { 0xfe5e25f5, SUC_lexicon_tags_66 },
    { 0x7fbe25f0, SUC_lexicon_tags_27 },
    { 0x137e25f7, SUC_lexicon_tags_70 },
    { 0x221e25f1, SUC_lexicon_tags_17 },
    { 0xa7de25f9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59e25fd, SUC_lexicon_tags_2 },
    { 0xa3be25fe, SUC_lexicon_tags_43 },
    { 0xe37e25ff, SUC_lexicon_tags_27 },
    { 0x111e25ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631e2608, SUC_lexicon_tags_11 },
    { 0x4be2608, SUC_lexicon_tags_8 },
    { 0x1a5e260a, SUC_lexicon_tags_7 },
    { 0xf19e260b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3e2610, SUC_lexicon_tags_40 },
    { 0x8f7e2611, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3ade2613, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887e261a, SUC_lexicon_tags_1410 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455e2620, SUC_lexicon_tags_49 },
    { 0x91de2621, SUC_lexicon_tags_6 },
    { 0x6ede2622, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49e2629, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9be262c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbe2630, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b3e2632, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699e2636, SUC_lexicon_tags_7 },
    { 0x81fe2637, SUC_lexicon_tags_1 },
    { 0x515e2636, SUC_lexicon_tags_84 },
    { 0x671e2637, SUC_lexicon_tags_17 },
    { 0xd77e2636, SUC_lexicon_tags_15 },
    { 0x8f7e2637, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0be2641, SUC_lexicon_tags_43 },
    { 0x231e2642, SUC_lexicon_tags_15 },
    { 0x277e2643, SUC_lexicon_tags_30 },
    { 0xa4de2644, SUC_lexicon_tags_31 },
    { 0x7abe2644, SUC_lexicon_tags_11 },
    { 0x719e2644, SUC_lexicon_tags_11 },
    { 0x28de2647, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb47e2649, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851e264e, SUC_lexicon_tags_17 },
    { 0xf13e264f, SUC_lexicon_tags_11 },
    { 0x4bde264f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xea9e2652, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ebe2654, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57e2658, SUC_lexicon_tags_2 },
    { 0xd8fe2658, SUC_lexicon_tags_1 },
    { 0xa5de2658, SUC_lexicon_tags_38 },
    { 0xaf1e265b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fe2660, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd45e2662, SUC_lexicon_tags_47 },
    { 0x7d7e2662, SUC_lexicon_tags_43 },
    { 0xd57e2664, SUC_lexicon_tags_37 },
    { 0x64de2664, SUC_lexicon_tags_468 },
    { 0, NULL },
    { 0xb17e2667, SUC_lexicon_tags_10 },
    { 0x6e9e2668, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46de266c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97fe267d, SUC_lexicon_tags_37 },
    { 0x583e267d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd95e2680, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x2fde2682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69e2686, SUC_lexicon_tags_1 },
    { 0x67de2687, SUC_lexicon_tags_2 },
    { 0x667e2688, SUC_lexicon_tags_11 },
    { 0x719e2689, SUC_lexicon_tags_12 },
    { 0x5e9e268a, SUC_lexicon_tags_27 },
    { 0xa1de2689, SUC_lexicon_tags_44 },
    { 0x56fe2688, SUC_lexicon_tags_25 },
    { 0x7e5e268d, SUC_lexicon_tags_38 },
    { 0x1a7e268d, SUC_lexicon_tags_11 },
    { 0x6ffe268d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe2693, SUC_lexicon_tags_15 },
    { 0xc6de2693, SUC_lexicon_tags_47 },
    { 0x27e2695, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9e269f, SUC_lexicon_tags_12 },
    { 0xab5e269f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5e26a3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e26a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x675e26a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2be26ac, SUC_lexicon_tags_37 },
    { 0xe07e26ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891e26b1, SUC_lexicon_tags_43 },
    { 0x8d9e26b2, SUC_lexicon_tags_7 },
    { 0x409e26b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9e26bc, SUC_lexicon_tags_70 },
    { 0xe39e26bc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e26c0, SUC_lexicon_tags_17 },
    { 0x78de26c1, SUC_lexicon_tags_9 },
    { 0x28de26c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56fe26c4, SUC_lexicon_tags_37 },
    { 0x66de26c4, SUC_lexicon_tags_7 },
    { 0x32fe26c4, SUC_lexicon_tags_37 },
    { 0xa97e26c4, SUC_lexicon_tags_82 },
    { 0x4fbe26c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311e26d0, SUC_lexicon_tags_2 },
    { 0x5c1e26d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x911e26d4, SUC_lexicon_tags_49 },
    { 0x75e26d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679e26d9, SUC_lexicon_tags_15 },
    { 0x899e26d9, SUC_lexicon_tags_26 },
    { 0x7e9e26d9, SUC_lexicon_tags_37 },
    { 0x25be26dc, SUC_lexicon_tags_151 },
    { 0x23fe26dd, SUC_lexicon_tags_32 },
    { 0xe9be26dc, SUC_lexicon_tags_9 },
    { 0xfbde26df, SUC_lexicon_tags_2 },
    { 0x263e26dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60be26e5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05e26e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8fbe26ea, SUC_lexicon_tags_15 },
    { 0x7d7e26eb, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1e26f3, SUC_lexicon_tags_17 },
    { 0x265e26f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd67e26f7, SUC_lexicon_tags_12 },
    { 0xe8fe26f7, SUC_lexicon_tags_8 },
    { 0xf55e26f9, SUC_lexicon_tags_25 },
    { 0x95de26fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe01e26fc, SUC_lexicon_tags_2 },
    { 0x375e26fd, SUC_lexicon_tags_11 },
    { 0xc9de26fe, SUC_lexicon_tags_11 },
    { 0xe57e26fd, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fe2702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75e2708, SUC_lexicon_tags_12 },
    { 0xa17e2709, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3e270c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7e2712, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635e2716, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x271e2718, SUC_lexicon_tags_37 },
    { 0xe53e2719, SUC_lexicon_tags_17 },
    { 0xab7e2718, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa51e271c, SUC_lexicon_tags_29 },
    { 0xba1e271d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbe2723, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e2727, SUC_lexicon_tags_84 },
    { 0x3fe2727, SUC_lexicon_tags_2 },
    { 0x763e2729, SUC_lexicon_tags_33 },
    { 0x6f7e272a, SUC_lexicon_tags_37 },
    { 0x345e272b, SUC_lexicon_tags_70 },
    { 0x7d1e2727, SUC_lexicon_tags_11 },
    { 0xffbe2729, SUC_lexicon_tags_1 },
    { 0x8f5e272e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905e2735, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x371e2738, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fe2741, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93e2744, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafe274b, SUC_lexicon_tags_17 },
    { 0x111e274c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5e2750, SUC_lexicon_tags_1 },
    { 0x51e2750, SUC_lexicon_tags_14 },
    { 0x629e2752, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x79be2755, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd0de2757, SUC_lexicon_tags_38 },
    { 0x903e2758, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571e275c, SUC_lexicon_tags_18 },
    { 0x47e275d, SUC_lexicon_tags_15 },
    { 0xf7fe275d, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87e2763, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5e2766, SUC_lexicon_tags_37 },
    { 0xf1e2767, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345e276c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x359e276e, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x4a1e2770, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98be2775, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x263e2778, SUC_lexicon_tags_28 },
    { 0xc07e2779, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143e2785, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61e2789, SUC_lexicon_tags_47 },
    { 0x759e278a, SUC_lexicon_tags_79 },
    { 0xff7e278b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x40be278e, SUC_lexicon_tags_11 },
    { 0xbd7e278f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf5e2791, SUC_lexicon_tags_11 },
    { 0xc3be2792, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb49e2794, SUC_lexicon_tags_23 },
    { 0x57de2795, SUC_lexicon_tags_35 },
    { 0x485e2794, SUC_lexicon_tags_35 },
    { 0x709e2796, SUC_lexicon_tags_60 },
    { 0x65be2798, SUC_lexicon_tags_12 },
    { 0x7b7e2795, SUC_lexicon_tags_15 },
    { 0x757e2795, SUC_lexicon_tags_19 },
    { 0xbbbe279b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x60fe279d, SUC_lexicon_tags_15 },
    { 0x5d9e279e, SUC_lexicon_tags_7 },
    { 0xc09e279f, SUC_lexicon_tags_7 },
    { 0xf7fe27a0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85e27a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdede27aa, SUC_lexicon_tags_17 },
    { 0xcc9e27aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x911e27ad, SUC_lexicon_tags_11 },
    { 0x765e27ae, SUC_lexicon_tags_87 },
    { 0xd2fe27ad, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdbfe27b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623e27b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09e27ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3e27be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b3e27c0, SUC_lexicon_tags_45 },
    { 0xf3e27c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbade27c4, SUC_lexicon_tags_47 },
    { 0x3abe27c5, SUC_lexicon_tags_32 },
    { 0x139e27c6, SUC_lexicon_tags_2 },
    { 0xa19e27c4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x36be27c9, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29e27ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdfde27d0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x823e27d2, SUC_lexicon_tags_15 },
    { 0x7c7e27d2, SUC_lexicon_tags_37 },
    { 0xb55e27d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcc1e27d6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x101e27d8, SUC_lexicon_tags_25 },
    { 0x9f5e27d9, SUC_lexicon_tags_16 },
    { 0xf55e27d8, SUC_lexicon_tags_1 },
    { 0x66de27d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb49e27dd, SUC_lexicon_tags_25 },
    { 0xb53e27de, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcede27e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e27e6, SUC_lexicon_tags_15 },
    { 0xa89e27e6, SUC_lexicon_tags_6 },
    { 0x5d3e27e7, SUC_lexicon_tags_28 },
    { 0xebde27e9, SUC_lexicon_tags_15 },
    { 0xb7fe27ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44de27f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0be27f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fde27f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x133e27fb, SUC_lexicon_tags_11 },
    { 0x32fe27fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebe2804, SUC_lexicon_tags_38 },
    { 0xfd1e2805, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x707e2808, SUC_lexicon_tags_174 },
    { 0x515e2809, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e280e, SUC_lexicon_tags_2 },
    { 0xe19e280f, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfe2813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x339e2816, SUC_lexicon_tags_55 },
    { 0x341e2817, SUC_lexicon_tags_15 },
    { 0x9ffe2817, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x729e281b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0de2821, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x53fe2823, SUC_lexicon_tags_23 },
    { 0x3b3e2823, SUC_lexicon_tags_33 },
    { 0x787e2824, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x36de2828, SUC_lexicon_tags_25 },
    { 0x1b1e2829, SUC_lexicon_tags_14 },
    { 0x10de282a, SUC_lexicon_tags_14 },
    { 0x35fe2829, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2dfe282d, SUC_lexicon_tags_1 },
    { 0x3c7e282e, SUC_lexicon_tags_12 },
    { 0x9a5e282f, SUC_lexicon_tags_47 },
    { 0xd5be2830, SUC_lexicon_tags_63 },
    { 0xd9de2830, SUC_lexicon_tags_1 },
    { 0xc73e2831, SUC_lexicon_tags_6 },
    { 0x18be2830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5e2836, SUC_lexicon_tags_15 },
    { 0x8ade2837, SUC_lexicon_tags_17 },
    { 0x759e2838, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x75e283a, SUC_lexicon_tags_14 },
    { 0xaafe283b, SUC_lexicon_tags_3 },
    { 0xb3e283a, SUC_lexicon_tags_10 },
    { 0xc5fe283d, SUC_lexicon_tags_40 },
    { 0x427e283a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83e2842, SUC_lexicon_tags_32 },
    { 0x5f3e2843, SUC_lexicon_tags_63 },
    { 0xb57e2844, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ade284d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1be2852, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5e2856, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4de285a, SUC_lexicon_tags_45 },
    { 0x239e285b, SUC_lexicon_tags_27 },
    { 0xaafe285c, SUC_lexicon_tags_39 },
    { 0x543e285d, SUC_lexicon_tags_25 },
    { 0xe1de285a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30be2860, SUC_lexicon_tags_16 },
    { 0x19e2861, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x3fde2863, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95e2866, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573e286c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe61e286e, SUC_lexicon_tags_15 },
    { 0xf0de286e, SUC_lexicon_tags_25 },
    { 0xfdde2870, SUC_lexicon_tags_299 },
    { 0x1a3e2871, SUC_lexicon_tags_38 },
    { 0x225e2872, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f7e2874, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50be2877, SUC_lexicon_tags_9 },
    { 0x9d1e2878, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x765e287b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79fe2881, SUC_lexicon_tags_25 },
    { 0x901e2882, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95e2887, SUC_lexicon_tags_15 },
    { 0x997e2888, SUC_lexicon_tags_23 },
    { 0x7ede2887, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xde9e288b, SUC_lexicon_tags_33 },
    { 0x17e288c, SUC_lexicon_tags_37 },
    { 0xe4be288c, SUC_lexicon_tags_15 },
    { 0x8fde288e, SUC_lexicon_tags_11 },
    { 0x775e288f, SUC_lexicon_tags_29 },
    { 0x721e288c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733e2899, SUC_lexicon_tags_25 },
    { 0xce3e289a, SUC_lexicon_tags_13 },
    { 0xd2fe289b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93be28a3, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1e28a8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xe7de28aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51e28ad, SUC_lexicon_tags_1 },
    { 0xadbe28ae, SUC_lexicon_tags_49 },
    { 0x98be28af, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7bde28b1, SUC_lexicon_tags_12 },
    { 0x825e28b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc63e28b4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38de28b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fe28bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e28c2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6a9e28c4, SUC_lexicon_tags_57 },
    { 0xc77e28c4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5e28c8, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x849e28cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805e28d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533e28d9, SUC_lexicon_tags_27 },
    { 0x995e28da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fe28ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7e28f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4be28f4, SUC_lexicon_tags_8 },
    { 0x7abe28f5, SUC_lexicon_tags_55 },
    { 0xd8be28f6, SUC_lexicon_tags_23 },
    { 0x81be28f7, SUC_lexicon_tags_17 },
    { 0x767e28f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9e28fb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475e28ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469e2904, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449e2908, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1e290c, SUC_lexicon_tags_1 },
    { 0xbffe290d, SUC_lexicon_tags_31 },
    { 0x995e290e, SUC_lexicon_tags_17 },
    { 0xa97e290f, SUC_lexicon_tags_19 },
    { 0xa1de2910, SUC_lexicon_tags_15 },
    { 0xff9e2911, SUC_lexicon_tags_1 },
    { 0x745e2912, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4ade2914, SUC_lexicon_tags_7 },
    { 0xf37e2915, SUC_lexicon_tags_12 },
    { 0x55be2915, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefe291b, SUC_lexicon_tags_63 },
    { 0x9be291c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfe2921, SUC_lexicon_tags_0 },
    { 0x5ffe2922, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xbc9e2924, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xa27e2926, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44de292f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5e2936, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa71e2938, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x9e7e293a, SUC_lexicon_tags_5 },
    { 0xfefe293b, SUC_lexicon_tags_27 },
    { 0xb97e293b, SUC_lexicon_tags_15 },
    { 0xda1e293a, SUC_lexicon_tags_43 },
    { 0xf11e293a, SUC_lexicon_tags_77 },
    { 0xef9e293d, SUC_lexicon_tags_1 },
    { 0xc2de293a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3be2942, SUC_lexicon_tags_12 },
    { 0xff9e2942, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe99e2945, SUC_lexicon_tags_80 },
    { 0xa13e2945, SUC_lexicon_tags_31 },
    { 0x307e2946, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57e294a, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e294d, SUC_lexicon_tags_63 },
    { 0x689e294d, SUC_lexicon_tags_108 },
    { 0xde294e, SUC_lexicon_tags_26 },
    { 0xa57e2950, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73e2953, SUC_lexicon_tags_2 },
    { 0x1efe2953, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607e295b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fe2960, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17e2964, SUC_lexicon_tags_33 },
    { 0x629e2965, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe2be2967, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45be296c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7fbe296e, SUC_lexicon_tags_25 },
    { 0x985e296e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x993e2971, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17fe2973, SUC_lexicon_tags_87 },
    { 0x601e2973, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1de2979, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67e297d, SUC_lexicon_tags_15 },
    { 0xeafe297d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9e2987, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21e298a, SUC_lexicon_tags_37 },
    { 0x4abe298b, SUC_lexicon_tags_14 },
    { 0x3c1e298c, SUC_lexicon_tags_29 },
    { 0x90fe298b, SUC_lexicon_tags_8 },
    { 0x65be298e, SUC_lexicon_tags_27 },
    { 0x529e298c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe2994, SUC_lexicon_tags_5 },
    { 0x849e2995, SUC_lexicon_tags_10 },
    { 0xf4fe2994, SUC_lexicon_tags_26 },
    { 0x329e2997, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe37e2999, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x48fe299b, SUC_lexicon_tags_2 },
    { 0x71e299c, SUC_lexicon_tags_26 },
    { 0x4b3e299d, SUC_lexicon_tags_37 },
    { 0x997e299c, SUC_lexicon_tags_48 },
    { 0x423e299d, SUC_lexicon_tags_46 },
    { 0xb31e29a0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22be29a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5be29aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77be29ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf27e29ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7e29b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5e29b6, SUC_lexicon_tags_17 },
    { 0x3dde29b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5de29c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fe29cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afe29ce, SUC_lexicon_tags_10 },
    { 0xa9fe29ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa6be29d1, SUC_lexicon_tags_12 },
    { 0x843e29d1, SUC_lexicon_tags_15 },
    { 0xdc1e29d1, SUC_lexicon_tags_14 },
    { 0x78be29d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c5e29d6, SUC_lexicon_tags_56 },
    { 0x74be29d7, SUC_lexicon_tags_29 },
    { 0x2bde29d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cde29db, SUC_lexicon_tags_1 },
    { 0xfade29dc, SUC_lexicon_tags_25 },
    { 0xc3be29dd, SUC_lexicon_tags_27 },
    { 0x2c9e29de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x803e29e0, SUC_lexicon_tags_18 },
    { 0xc51e29e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7e29e4, SUC_lexicon_tags_91 },
    { 0x709e29e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdade29eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93e29f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x87de29f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603e29fc, SUC_lexicon_tags_25 },
    { 0xa95e29fd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3d7e29ff, SUC_lexicon_tags_4 },
    { 0xe47e2a00, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1e2a06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7e2a09, SUC_lexicon_tags_19 },
    { 0x515e2a0a, SUC_lexicon_tags_61 },
    { 0x485e2a09, SUC_lexicon_tags_27 },
    { 0xafe2a0c, SUC_lexicon_tags_1 },
    { 0x2c5e2a0d, SUC_lexicon_tags_4 },
    { 0x4afe2a09, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf5e2a10, SUC_lexicon_tags_11 },
    { 0xe1fe2a11, SUC_lexicon_tags_43 },
    { 0xb67e2a12, SUC_lexicon_tags_17 },
    { 0xbc9e2a13, SUC_lexicon_tags_22 },
    { 0xe6fe2a10, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23e2a2a, SUC_lexicon_tags_37 },
    { 0xa25e2a2a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x425e2a2d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f7e2a2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03e2a35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1e2a39, SUC_lexicon_tags_63 },
    { 0x299e2a3a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd57e2a3c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1e2a3f, SUC_lexicon_tags_62 },
    { 0x3b7e2a40, SUC_lexicon_tags_11 },
    { 0xffe2a41, SUC_lexicon_tags_19 },
    { 0x5c5e2a41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01e2a45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x285e2a48, SUC_lexicon_tags_1 },
    { 0xa7e2a49, SUC_lexicon_tags_33 },
    { 0x6b5e2a48, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573e2a50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01e2a60, SUC_lexicon_tags_0 },
    { 0xdbfe2a60, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abe2a66, SUC_lexicon_tags_19 },
    { 0x217e2a67, SUC_lexicon_tags_1 },
    { 0xd67e2a68, SUC_lexicon_tags_63 },
    { 0x66de2a66, SUC_lexicon_tags_16 },
    { 0xf79e2a6a, SUC_lexicon_tags_11 },
    { 0x73de2a6b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3d3e2a6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0be2a6f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89be2a76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1e2a78, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x749e2a7b, SUC_lexicon_tags_43 },
    { 0x5bbe2a7b, SUC_lexicon_tags_7 },
    { 0x993e2a7d, SUC_lexicon_tags_7 },
    { 0xaade2a7e, SUC_lexicon_tags_26 },
    { 0xe87e2a7d, SUC_lexicon_tags_56 },
    { 0x885e2a7f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x585e2a82, SUC_lexicon_tags_12 },
    { 0x69de2a83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebe2a86, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fe2a8a, SUC_lexicon_tags_23 },
    { 0x1f9e2a8b, SUC_lexicon_tags_57 },
    { 0xccbe2a8b, SUC_lexicon_tags_0 },
    { 0x45be2a8c, SUC_lexicon_tags_60 },
    { 0x183e2a8d, SUC_lexicon_tags_17 },
    { 0x64de2a8d, SUC_lexicon_tags_17 },
    { 0x1fe2a90, SUC_lexicon_tags_38 },
    { 0xedde2a91, SUC_lexicon_tags_25 },
    { 0x14de2a91, SUC_lexicon_tags_33 },
    { 0xa4be2a93, SUC_lexicon_tags_11 },
    { 0x241e2a93, SUC_lexicon_tags_43 },
    { 0xab9e2a92, SUC_lexicon_tags_11 },
    { 0x9efe2a96, SUC_lexicon_tags_25 },
    { 0x663e2a96, SUC_lexicon_tags_7 },
    { 0x3e7e2a90, SUC_lexicon_tags_15 },
    { 0x409e2a8a, SUC_lexicon_tags_60 },
    { 0x9c3e2a93, SUC_lexicon_tags_14 },
    { 0x1c7e2a97, SUC_lexicon_tags_29 },
    { 0xdebe2a99, SUC_lexicon_tags_85 },
    { 0xe11e2a9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa83e2a9f, SUC_lexicon_tags_11 },
    { 0xe29e2a9f, SUC_lexicon_tags_31 },
    { 0xae5e2aa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6fe2aa3, SUC_lexicon_tags_17 },
    { 0xcf3e2aa4, SUC_lexicon_tags_32 },
    { 0x695e2aa4, SUC_lexicon_tags_1 },
    { 0xcdbe2aa6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9de2aae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x767e2ab1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59e2ab4, SUC_lexicon_tags_12 },
    { 0x7c5e2ab4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3e2abd, SUC_lexicon_tags_37 },
    { 0x8b7e2abd, SUC_lexicon_tags_33 },
    { 0x62be2abf, SUC_lexicon_tags_27 },
    { 0xd41e2ac0, SUC_lexicon_tags_2 },
    { 0x4c9e2abe, SUC_lexicon_tags_15 },
    { 0x8cfe2ac2, SUC_lexicon_tags_16 },
    { 0xac5e2ac0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fe2ac7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2de2acd, SUC_lexicon_tags_8 },
    { 0x73be2ace, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137e2ad5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x729e2ad8, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99be2ae4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbede2ae6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51be2ae8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3e2aeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47e2aee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3e2af3, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6de2af7, SUC_lexicon_tags_15 },
    { 0xe3de2af8, SUC_lexicon_tags_23 },
    { 0x81be2af8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50be2afb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x985e2afd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf67e2aff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93e2b03, SUC_lexicon_tags_17 },
    { 0xe39e2b03, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fe2b0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5e2b0f, SUC_lexicon_tags_9 },
    { 0x29be2b0f, SUC_lexicon_tags_43 },
    { 0xcbfe2b10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdade2b13, SUC_lexicon_tags_15 },
    { 0x4cde2b14, SUC_lexicon_tags_23 },
    { 0x871e2b15, SUC_lexicon_tags_15 },
    { 0x9cbe2b16, SUC_lexicon_tags_46 },
    { 0x213e2b15, SUC_lexicon_tags_29 },
    { 0x57e2b15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de2b21, SUC_lexicon_tags_45 },
    { 0xc09e2b22, SUC_lexicon_tags_11 },
    { 0x405e2b22, SUC_lexicon_tags_17 },
    { 0xe61e2b24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95e2b28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9e2b2c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x645e2b2e, SUC_lexicon_tags_42 },
    { 0x4d7e2b2f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63e2b33, SUC_lexicon_tags_37 },
    { 0xc27e2b34, SUC_lexicon_tags_37 },
    { 0x289e2b35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa91e2b37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8de2b3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67e2b3e, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75e2b47, SUC_lexicon_tags_8 },
    { 0x5d5e2b48, SUC_lexicon_tags_7 },
    { 0xe1be2b49, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x591e2b4b, SUC_lexicon_tags_2 },
    { 0xd15e2b4c, SUC_lexicon_tags_12 },
    { 0xe95e2b4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afe2b50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfe2b55, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x76be2b58, SUC_lexicon_tags_0 },
    { 0x6f7e2b58, SUC_lexicon_tags_15 },
    { 0x881e2b59, SUC_lexicon_tags_15 },
    { 0x2c9e2b5b, SUC_lexicon_tags_30 },
    { 0xb25e2b5c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1efe2b5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5e2b62, SUC_lexicon_tags_12 },
    { 0x86fe2b63, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe5de2b65, SUC_lexicon_tags_158 },
    { 0xdd7e2b66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x387e2b69, SUC_lexicon_tags_61 },
    { 0x33de2b6a, SUC_lexicon_tags_23 },
    { 0x44fe2b6b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc87e2b6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1e2b71, SUC_lexicon_tags_2 },
    { 0x615e2b72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xefe2b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27be2b79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3e2b7c, SUC_lexicon_tags_32 },
    { 0x2c5e2b7d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x963e2b7f, SUC_lexicon_tags_37 },
    { 0xb7be2b80, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x681e2b83, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5e2b88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f3e2b8a, SUC_lexicon_tags_0 },
    { 0x2ede2b8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9be2b8d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x51de2b8f, SUC_lexicon_tags_19 },
    { 0x217e2b8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5e2b93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x48be2b96, SUC_lexicon_tags_63 },
    { 0xc67e2b96, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x58de2b99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111e2ba3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32fe2ba5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x481e2ba8, SUC_lexicon_tags_14 },
    { 0xb6fe2ba9, SUC_lexicon_tags_23 },
    { 0x6d9e2baa, SUC_lexicon_tags_47 },
    { 0x61e2bab, SUC_lexicon_tags_47 },
    { 0x21de2bac, SUC_lexicon_tags_14 },
    { 0x7f9e2baa, SUC_lexicon_tags_2 },
    { 0xc6fe2baa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9e2bb1, SUC_lexicon_tags_7 },
    { 0x7cbe2bb2, SUC_lexicon_tags_31 },
    { 0x585e2bb3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7e2bbc, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x997e2bbf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc65e2bc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf07e2bc3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971e2bc7, SUC_lexicon_tags_23 },
    { 0xbdde2bc7, SUC_lexicon_tags_17 },
    { 0xb7be2bc9, SUC_lexicon_tags_4 },
    { 0x3f5e2bca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x415e2bcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc65e2bcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333e2bd3, SUC_lexicon_tags_12 },
    { 0xf5be2bd3, SUC_lexicon_tags_37 },
    { 0x2a3e2bd5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1e2bdf, SUC_lexicon_tags_37 },
    { 0x89de2be0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe45e2be2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59e2be5, SUC_lexicon_tags_25 },
    { 0xe61e2be5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15fe2be8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39e2bec, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x213e2bef, SUC_lexicon_tags_17 },
    { 0xab1e2bef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabe2bf3, SUC_lexicon_tags_14 },
    { 0x57e2bf3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefe2bf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3ffe2bfa, SUC_lexicon_tags_17 },
    { 0x4f5e2bfa, SUC_lexicon_tags_19 },
    { 0x33de2bfc, SUC_lexicon_tags_10 },
    { 0xbbfe2bfd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813e2c02, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe2c07, SUC_lexicon_tags_7 },
    { 0xf2de2c08, SUC_lexicon_tags_19 },
    { 0x3bfe2c07, SUC_lexicon_tags_25 },
    { 0x359e2c0a, SUC_lexicon_tags_15 },
    { 0xc4fe2c0b, SUC_lexicon_tags_43 },
    { 0xb1be2c0c, SUC_lexicon_tags_17 },
    { 0xcbe2c0d, SUC_lexicon_tags_1 },
    { 0x72fe2c0e, SUC_lexicon_tags_63 },
    { 0xa9de2c09, SUC_lexicon_tags_15 },
    { 0x955e2c0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe95e2c12, SUC_lexicon_tags_25 },
    { 0x56de2c12, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x907e2c15, SUC_lexicon_tags_84 },
    { 0xf29e2c16, SUC_lexicon_tags_2 },
    { 0xb53e2c15, SUC_lexicon_tags_5 },
    { 0x159e2c18, SUC_lexicon_tags_1 },
    { 0x465e2c19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5e2c1c, SUC_lexicon_tags_47 },
    { 0x3fde2c1d, SUC_lexicon_tags_1 },
    { 0x279e2c1d, SUC_lexicon_tags_12 },
    { 0x2c3e2c1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffe2c22, SUC_lexicon_tags_29 },
    { 0x545e2c23, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xc0de2c25, SUC_lexicon_tags_33 },
    { 0xf0fe2c25, SUC_lexicon_tags_106 },
    { 0x24be2c26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c1e2c29, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefe2c2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f3e2c31, SUC_lexicon_tags_14 },
    { 0x205e2c32, SUC_lexicon_tags_43 },
    { 0x203e2c31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x857e2c35, SUC_lexicon_tags_46 },
    { 0x565e2c35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911e2c3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25e2c40, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x13be2c42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x89be2c45, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfe2c4b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857e2c51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd93e2c53, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e2c56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfafe2c5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5cde2c5e, SUC_lexicon_tags_2 },
    { 0x355e2c5f, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743e2c65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6dde2c67, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0x7d5e2c69, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x485e2c6c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311e2c71, SUC_lexicon_tags_2 },
    { 0xbc5e2c72, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489e2c77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837e2c7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x283e2c7d, SUC_lexicon_tags_47 },
    { 0xaabe2c7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e3e2c80, SUC_lexicon_tags_25 },
    { 0xa3be2c81, SUC_lexicon_tags_14 },
    { 0xc9de2c82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc07e2c84, SUC_lexicon_tags_11 },
    { 0x5efe2c84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b3e2c87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5e2c8a, SUC_lexicon_tags_25 },
    { 0x989e2c8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9e2c95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1e2c9f, SUC_lexicon_tags_38 },
    { 0x917e2ca0, SUC_lexicon_tags_92 },
    { 0xd67e2c9f, SUC_lexicon_tags_12 },
    { 0x55be2ca2, SUC_lexicon_tags_37 },
    { 0xd85e2ca2, SUC_lexicon_tags_17 },
    { 0x7ffe2ca2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7e2caa, SUC_lexicon_tags_25 },
    { 0xdd5e2cab, SUC_lexicon_tags_15 },
    { 0x30be2cac, SUC_lexicon_tags_25 },
    { 0xdfe2caa, SUC_lexicon_tags_65 },
    { 0x53e2cad, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27e2cb3, SUC_lexicon_tags_47 },
    { 0x285e2cb4, SUC_lexicon_tags_17 },
    { 0xaede2cb5, SUC_lexicon_tags_2 },
    { 0x15e2cb4, SUC_lexicon_tags_2 },
    { 0x329e2cb7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa05e2cb9, SUC_lexicon_tags_75 },
    { 0xc6de2cba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91e2cbe, SUC_lexicon_tags_83 },
    { 0xa7fe2cbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5fe2cc1, SUC_lexicon_tags_43 },
    { 0x701e2cc2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2be2cc7, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xcefe2cc9, SUC_lexicon_tags_27 },
    { 0x705e2cc9, SUC_lexicon_tags_8 },
    { 0x19be2ccb, SUC_lexicon_tags_38 },
    { 0x6fde2ccc, SUC_lexicon_tags_17 },
    { 0x8b1e2ccd, SUC_lexicon_tags_30 },
    { 0x2d9e2ccd, SUC_lexicon_tags_11 },
    { 0xc8fe2ccc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcbe2cd1, SUC_lexicon_tags_46 },
    { 0x333e2cd2, SUC_lexicon_tags_7 },
    { 0x447e2cd2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4be2cd7, SUC_lexicon_tags_26 },
    { 0xcc5e2cd8, SUC_lexicon_tags_28 },
    { 0x783e2cd9, SUC_lexicon_tags_17 },
    { 0xdb9e2cd9, SUC_lexicon_tags_37 },
    { 0x271e2cdb, SUC_lexicon_tags_15 },
    { 0x63e2cdc, SUC_lexicon_tags_47 },
    { 0xfa7e2cdc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xef7e2cdf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17e2ce1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfa1e2ce3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4fe2ce5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40de2ce8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbe2cec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x229e2cee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71e2cf1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63e2cf5, SUC_lexicon_tags_1 },
    { 0xfd7e2cf5, SUC_lexicon_tags_9 },
    { 0x68be2cf5, SUC_lexicon_tags_179 },
    { 0x611e2cf6, SUC_lexicon_tags_29 },
    { 0x2d1e2cf9, SUC_lexicon_tags_2 },
    { 0x55de2cf9, SUC_lexicon_tags_11 },
    { 0x137e2cf5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7e2cff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9f1e2d01, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75e2d07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac9e2d09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697e2d0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59e2d1e, SUC_lexicon_tags_0 },
    { 0x18de2d1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15e2d25, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x561e2d27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14de2d2b, SUC_lexicon_tags_14 },
    { 0x3b7e2d2c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1e2d35, SUC_lexicon_tags_10 },
    { 0xf8de2d36, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fe2d3a, SUC_lexicon_tags_15 },
    { 0xa23e2d3b, SUC_lexicon_tags_77 },
    { 0xcede2d3b, SUC_lexicon_tags_0 },
    { 0x1a9e2d3c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771e2d43, SUC_lexicon_tags_15 },
    { 0x4e3e2d43, SUC_lexicon_tags_16 },
    { 0x5a9e2d44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe2d4a, SUC_lexicon_tags_29 },
    { 0xe55e2d4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe73e2d4d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x45de2d4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcd3e2d51, SUC_lexicon_tags_7 },
    { 0xb5e2d51, SUC_lexicon_tags_31 },
    { 0x8d1e2d53, SUC_lexicon_tags_60 },
    { 0x657e2d53, SUC_lexicon_tags_38 },
    { 0x3dde2d53, SUC_lexicon_tags_49 },
    { 0x29be2d56, SUC_lexicon_tags_9 },
    { 0xbefe2d56, SUC_lexicon_tags_164 },
    { 0, NULL },
    { 0x567e2d59, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x23be2d5b, SUC_lexicon_tags_17 },
    { 0xf57e2d5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe79e2d5e, SUC_lexicon_tags_37 },
    { 0x36de2d5f, SUC_lexicon_tags_158 },
    { 0x1e5e2d60, SUC_lexicon_tags_2 },
    { 0x82be2d5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93be2d67, SUC_lexicon_tags_12 },
    { 0x5ede2d67, SUC_lexicon_tags_15 },
    { 0xdc9e2d69, SUC_lexicon_tags_63 },
    { 0x11e2d6a, SUC_lexicon_tags_38 },
    { 0xf9fe2d67, SUC_lexicon_tags_63 },
    { 0x63fe2d6c, SUC_lexicon_tags_43 },
    { 0xecde2d6c, SUC_lexicon_tags_2 },
    { 0x60de2d6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347e2d72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3e2d75, SUC_lexicon_tags_79 },
    { 0x7e9e2d76, SUC_lexicon_tags_48 },
    { 0xeb3e2d77, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x67be2d79, SUC_lexicon_tags_37 },
    { 0x74de2d79, SUC_lexicon_tags_26 },
    { 0x3d5e2d7a, SUC_lexicon_tags_2 },
    { 0x949e2d7b, SUC_lexicon_tags_29 },
    { 0x2ebe2d79, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35e2d81, SUC_lexicon_tags_36 },
    { 0xd23e2d81, SUC_lexicon_tags_31 },
    { 0x919e2d83, SUC_lexicon_tags_25 },
    { 0x255e2d84, SUC_lexicon_tags_47 },
    { 0xb55e2d84, SUC_lexicon_tags_25 },
    { 0x86de2d86, SUC_lexicon_tags_63 },
    { 0x31be2d83, SUC_lexicon_tags_17 },
    { 0xe6de2d83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fe2d8e, SUC_lexicon_tags_11 },
    { 0xb59e2d8f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abe2d93, SUC_lexicon_tags_1 },
    { 0x171e2d94, SUC_lexicon_tags_61 },
    { 0xa05e2d95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991e2d99, SUC_lexicon_tags_60 },
    { 0xc0de2d99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf57e2d9c, SUC_lexicon_tags_1 },
    { 0x6cbe2d9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1a5e2d9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfe2da3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbe2da6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb91e2da8, SUC_lexicon_tags_37 },
    { 0x5ebe2da9, SUC_lexicon_tags_82 },
    { 0x20be2daa, SUC_lexicon_tags_33 },
    { 0x289e2daa, SUC_lexicon_tags_23 },
    { 0x475e2dac, SUC_lexicon_tags_26 },
    { 0x891e2dac, SUC_lexicon_tags_37 },
    { 0xa05e2da9, SUC_lexicon_tags_47 },
    { 0xee1e2daf, SUC_lexicon_tags_33 },
    { 0xbd5e2db0, SUC_lexicon_tags_12 },
    { 0x25de2dad, SUC_lexicon_tags_11 },
    { 0x453e2db2, SUC_lexicon_tags_14 },
    { 0xc5de2db2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5abe2db5, SUC_lexicon_tags_25 },
    { 0x5f3e2db5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3e2dbb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5e2dc4, SUC_lexicon_tags_2 },
    { 0xb81e2dc5, SUC_lexicon_tags_25 },
    { 0xbc3e2dc4, SUC_lexicon_tags_0 },
    { 0xc07e2dc7, SUC_lexicon_tags_25 },
    { 0xe5de2dc8, SUC_lexicon_tags_7 },
    { 0x967e2dc9, SUC_lexicon_tags_55 },
    { 0xf79e2dc9, SUC_lexicon_tags_1 },
    { 0x477e2dc8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301e2dd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fe2dda, SUC_lexicon_tags_17 },
    { 0x973e2ddb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe2dde, SUC_lexicon_tags_3 },
    { 0xaa7e2dde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe9e2de1, SUC_lexicon_tags_17 },
    { 0x765e2de2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x807e2de4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7a3e2de6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347e2dee, SUC_lexicon_tags_47 },
    { 0xd55e2def, SUC_lexicon_tags_23 },
    { 0xbc7e2def, SUC_lexicon_tags_37 },
    { 0xe8be2df0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x611e2df3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51e2df7, SUC_lexicon_tags_14 },
    { 0xabbe2df8, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55e2dfb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe2dff, SUC_lexicon_tags_83 },
    { 0xf61e2dff, SUC_lexicon_tags_85 },
    { 0xf5be2e01, SUC_lexicon_tags_37 },
    { 0x859e2e02, SUC_lexicon_tags_11 },
    { 0xf85e2e03, SUC_lexicon_tags_25 },
    { 0xa27e2e04, SUC_lexicon_tags_2 },
    { 0x6f3e2e05, SUC_lexicon_tags_30 },
    { 0xaafe2e05, SUC_lexicon_tags_2 },
    { 0xfbde2e03, SUC_lexicon_tags_11 },
    { 0xe07e2e02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9e2e0f, SUC_lexicon_tags_4 },
    { 0xd55e2e10, SUC_lexicon_tags_15 },
    { 0xe03e2e11, SUC_lexicon_tags_37 },
    { 0x505e2e11, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbe2e16, SUC_lexicon_tags_63 },
    { 0x6be2e16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x58fe2e19, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x825e2e1c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x305e2e1e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x431e2e20, SUC_lexicon_tags_21 },
    { 0x177e2e21, SUC_lexicon_tags_37 },
    { 0xc89e2e22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21e2e25, SUC_lexicon_tags_55 },
    { 0xd8fe2e26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835e2e2a, SUC_lexicon_tags_11 },
    { 0xa45e2e2a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xaf5e2e2d, SUC_lexicon_tags_37 },
    { 0xf3de2e2e, SUC_lexicon_tags_27 },
    { 0x7a5e2e2f, SUC_lexicon_tags_14 },
    { 0xc0fe2e2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303e2e34, SUC_lexicon_tags_1 },
    { 0xd23e2e35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5e2e39, SUC_lexicon_tags_11 },
    { 0xc63e2e3a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe2e46, SUC_lexicon_tags_12 },
    { 0x8bfe2e46, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45e2e4f, SUC_lexicon_tags_25 },
    { 0x315e2e50, SUC_lexicon_tags_11 },
    { 0xa87e2e50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6de2e54, SUC_lexicon_tags_37 },
    { 0xb01e2e54, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589e2e59, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95de2e5d, SUC_lexicon_tags_37 },
    { 0xe63e2e5e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfe2e61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe71e2e63, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8be2e6a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7e2e6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4efe2e6f, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3e2e74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfe2e79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x95e2e7b, SUC_lexicon_tags_0 },
    { 0xbf7e2e7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x68de2e7e, SUC_lexicon_tags_25 },
    { 0x67be2e7f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1e2e90, SUC_lexicon_tags_45 },
    { 0x47de2e90, SUC_lexicon_tags_37 },
    { 0xa09e2e90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1e2e95, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1e2e99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3e2e9c, SUC_lexicon_tags_9 },
    { 0x52fe2e9d, SUC_lexicon_tags_25 },
    { 0xf59e2e9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c9e2ea0, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2be2ea4, SUC_lexicon_tags_77 },
    { 0xbc3e2ea5, SUC_lexicon_tags_18 },
    { 0xd85e2ea6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebde2eac, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651e2eb0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47e2eb4, SUC_lexicon_tags_25 },
    { 0xc47e2eb4, SUC_lexicon_tags_7 },
    { 0xd11e2eb6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1e2ebb, SUC_lexicon_tags_0 },
    { 0x94be2ebc, SUC_lexicon_tags_9 },
    { 0x953e2ebd, SUC_lexicon_tags_6 },
    { 0xf05e2ebe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe75e2ec0, SUC_lexicon_tags_0 },
    { 0x8c1e2ec0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58be2ec3, SUC_lexicon_tags_107 },
    { 0x88fe2ec3, SUC_lexicon_tags_26 },
    { 0xb79e2ec4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15e2ec9, SUC_lexicon_tags_19 },
    { 0xd87e2eca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43e2ed4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xed7e2ed6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1e2ede, SUC_lexicon_tags_60 },
    { 0x75e2ede, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dde2ee2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5be2ee4, SUC_lexicon_tags_87 },
    { 0xa1fe2ee5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf27e2ee7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe9de2ee9, SUC_lexicon_tags_6 },
    { 0xf5be2eea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbe2ef1, SUC_lexicon_tags_27 },
    { 0x151e2ef2, SUC_lexicon_tags_48 },
    { 0x957e2ef2, SUC_lexicon_tags_2 },
    { 0xac3e2ef4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcbfe2ef6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de2efe, SUC_lexicon_tags_28 },
    { 0x38be2efe, SUC_lexicon_tags_0 },
    { 0xddfe2f00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x23de2f02, SUC_lexicon_tags_17 },
    { 0xcb7e2f02, SUC_lexicon_tags_3 },
    { 0x719e2f02, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9e2f08, SUC_lexicon_tags_25 },
    { 0x205e2f09, SUC_lexicon_tags_37 },
    { 0xec1e2f09, SUC_lexicon_tags_11 },
    { 0x8c7e2f08, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5e2f11, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc7e2f13, SUC_lexicon_tags_17 },
    { 0xd1fe2f14, SUC_lexicon_tags_46 },
    { 0x301e2f14, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41e2f1a, SUC_lexicon_tags_70 },
    { 0x4e7e2f1b, SUC_lexicon_tags_65 },
    { 0x9a3e2f1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fe2f1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33be2f23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823e2f2b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23e2f31, SUC_lexicon_tags_57 },
    { 0xb3fe2f32, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e2f35, SUC_lexicon_tags_10 },
    { 0xad3e2f36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x155e2f38, SUC_lexicon_tags_25 },
    { 0x1f7e2f39, SUC_lexicon_tags_49 },
    { 0xb4fe2f3a, SUC_lexicon_tags_19 },
    { 0x5bfe2f39, SUC_lexicon_tags_38 },
    { 0x1a9e2f3c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf79e2f3f, SUC_lexicon_tags_47 },
    { 0x7abe2f3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5be2f4c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf89e2f4f, SUC_lexicon_tags_23 },
    { 0xa89e2f50, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8ede2f52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be2f55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6dfe2f57, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7e1e2f59, SUC_lexicon_tags_37 },
    { 0x545e2f5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x487e2f5d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xade2f5f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bde2f63, SUC_lexicon_tags_30 },
    { 0xb5de2f64, SUC_lexicon_tags_61 },
    { 0x2fde2f65, SUC_lexicon_tags_14 },
    { 0x855e2f64, SUC_lexicon_tags_27 },
    { 0xc69e2f67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abe2f6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87e2f6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5d3e2f6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9be2f71, SUC_lexicon_tags_7 },
    { 0x6f7e2f72, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x61de2f74, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x89de2f77, SUC_lexicon_tags_33 },
    { 0x585e2f77, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83e2f85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x52de2f87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1e2f8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc33e2f90, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3e2f93, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3be2f95, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889e2f9a, SUC_lexicon_tags_17 },
    { 0xbf1e2f9b, SUC_lexicon_tags_21 },
    { 0x1a5e2f9b, SUC_lexicon_tags_1 },
    { 0x241e2f9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf2fe2f9f, SUC_lexicon_tags_57 },
    { 0xf7e2f9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbe1e2fa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfe2fa7, SUC_lexicon_tags_8 },
    { 0xd71e2fa7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83e2fab, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747e2faf, SUC_lexicon_tags_4 },
    { 0x847e2fb0, SUC_lexicon_tags_25 },
    { 0xef7e2fb1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fe2fb6, SUC_lexicon_tags_108 },
    { 0x227e2fb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f1e2fb9, SUC_lexicon_tags_7 },
    { 0x2e1e2fb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc3be2fbc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x609e2fbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abe2fc1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883e2fc6, SUC_lexicon_tags_49 },
    { 0x5bbe2fc7, SUC_lexicon_tags_38 },
    { 0xce3e2fc8, SUC_lexicon_tags_2 },
    { 0x615e2fc7, SUC_lexicon_tags_2 },
    { 0xddfe2fc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7e2fce, SUC_lexicon_tags_47 },
    { 0xa9e2fcf, SUC_lexicon_tags_2 },
    { 0x11de2fce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86de2fd5, SUC_lexicon_tags_25 },
    { 0x619e2fd5, SUC_lexicon_tags_2 },
    { 0xb4be2fd7, SUC_lexicon_tags_57 },
    { 0x565e2fd8, SUC_lexicon_tags_15 },
    { 0xa93e2fd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe01e2fdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf7fe2fdd, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0xe5de2fdf, SUC_lexicon_tags_37 },
    { 0x779e2fdf, SUC_lexicon_tags_108 },
    { 0x7e3e2fe0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa43e2fe3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65e2fe6, SUC_lexicon_tags_13 },
    { 0xc29e2fe7, SUC_lexicon_tags_7 },
    { 0x239e2fe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xede2fea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1e2fed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x96be2ff0, SUC_lexicon_tags_19 },
    { 0x92fe2ff1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xebe2ff4, SUC_lexicon_tags_39 },
    { 0xa27e2ff4, SUC_lexicon_tags_27 },
    { 0x889e2ff6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9e2ffa, SUC_lexicon_tags_645 },
    { 0, NULL },
    { 0x2b9e2ffc, SUC_lexicon_tags_29 },
    { 0x4a7e2ffd, SUC_lexicon_tags_138 },
    { 0xb57e2ffe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x18be3000, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x785e3003, SUC_lexicon_tags_11 },
    { 0xfefe3004, SUC_lexicon_tags_1 },
    { 0x295e3005, SUC_lexicon_tags_2 },
    { 0xd97e3006, SUC_lexicon_tags_54 },
    { 0x3ede3005, SUC_lexicon_tags_3 },
    { 0x1bde3003, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fe300f, SUC_lexicon_tags_28 },
    { 0x7d9e3010, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0xfabe3012, SUC_lexicon_tags_38 },
    { 0xdd7e3012, SUC_lexicon_tags_16 },
    { 0x55de3014, SUC_lexicon_tags_11 },
    { 0x61fe3015, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8de3019, SUC_lexicon_tags_37 },
    { 0xbde301a, SUC_lexicon_tags_11 },
    { 0xbcbe3019, SUC_lexicon_tags_1 },
    { 0x625e301c, SUC_lexicon_tags_63 },
    { 0x313e301c, SUC_lexicon_tags_25 },
    { 0xcbe301e, SUC_lexicon_tags_97 },
    { 0x307e301f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63be3023, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77e3027, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x299e302a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1e302f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf7e3031, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe3039, SUC_lexicon_tags_65 },
    { 0xd9be303a, SUC_lexicon_tags_47 },
    { 0xcabe303b, SUC_lexicon_tags_7 },
    { 0xd61e303b, SUC_lexicon_tags_2 },
    { 0x60fe303c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841e3045, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ede304b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4be304e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fe3053, SUC_lexicon_tags_47 },
    { 0x945e3053, SUC_lexicon_tags_2 },
    { 0xa9de3055, SUC_lexicon_tags_11 },
    { 0x297e3056, SUC_lexicon_tags_364 },
    { 0x83e3057, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05e305a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xe6fe305c, SUC_lexicon_tags_11 },
    { 0x5b5e305c, SUC_lexicon_tags_207 },
    { 0x57be305e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa99e3060, SUC_lexicon_tags_7 },
    { 0xadde3061, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x28fe3063, SUC_lexicon_tags_1 },
    { 0x64de3064, SUC_lexicon_tags_92 },
    { 0xd11e3065, SUC_lexicon_tags_47 },
    { 0x19e3066, SUC_lexicon_tags_12 },
    { 0xade3066, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21e3070, SUC_lexicon_tags_18 },
    { 0x515e3070, SUC_lexicon_tags_2 },
    { 0xa8fe3072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3be3074, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9e307e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e3081, SUC_lexicon_tags_2 },
    { 0x907e3082, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77e3086, SUC_lexicon_tags_0 },
    { 0xe75e3086, SUC_lexicon_tags_49 },
    { 0xe9be3088, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x26fe308a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33e3090, SUC_lexicon_tags_8 },
    { 0x587e3091, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x583e3094, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbe3097, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375e309d, SUC_lexicon_tags_14 },
    { 0x71be309e, SUC_lexicon_tags_25 },
    { 0xd85e309f, SUC_lexicon_tags_11 },
    { 0xd97e309f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7be30a3, SUC_lexicon_tags_33 },
    { 0xb81e30a4, SUC_lexicon_tags_37 },
    { 0x3e3e30a5, SUC_lexicon_tags_19 },
    { 0x24fe30a6, SUC_lexicon_tags_22 },
    { 0xb9e30a4, SUC_lexicon_tags_65 },
    { 0xe4be30a7, SUC_lexicon_tags_43 },
    { 0x157e30a9, SUC_lexicon_tags_28 },
    { 0x873e30a9, SUC_lexicon_tags_45 },
    { 0xcb5e30aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fe30b0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x639e30b3, SUC_lexicon_tags_25 },
    { 0x1be30b4, SUC_lexicon_tags_37 },
    { 0xdbfe30b3, SUC_lexicon_tags_38 },
    { 0xf47e30b6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb21e30b8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7d9e30ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41de30bd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xede30bf, SUC_lexicon_tags_7 },
    { 0x5b1e30c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917e30c4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2be30c8, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fe30cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcabe30ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49e30d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9e30d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461e30dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e30e3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15de30e8, SUC_lexicon_tags_16 },
    { 0xbdbe30e8, SUC_lexicon_tags_25 },
    { 0x1be30e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5e30ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbe30f7, SUC_lexicon_tags_30 },
    { 0x13fe30f8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5e30fb, SUC_lexicon_tags_12 },
    { 0xd19e30fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39e3101, SUC_lexicon_tags_23 },
    { 0xf11e3102, SUC_lexicon_tags_21 },
    { 0xde5e3102, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x601e3105, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729e3116, SUC_lexicon_tags_15 },
    { 0xa39e3117, SUC_lexicon_tags_66 },
    { 0x581e3118, SUC_lexicon_tags_17 },
    { 0x69fe3119, SUC_lexicon_tags_8 },
    { 0xbe7e3118, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2bfe311c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23e3121, SUC_lexicon_tags_70 },
    { 0x4ede3122, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3be3126, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fe312b, SUC_lexicon_tags_45 },
    { 0xcbfe312c, SUC_lexicon_tags_12 },
    { 0xf71e312d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27be3131, SUC_lexicon_tags_14 },
    { 0x6be3132, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3e3135, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x209e3137, SUC_lexicon_tags_32 },
    { 0xa43e3137, SUC_lexicon_tags_85 },
    { 0xd9fe3139, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadfe313b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7be3142, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5e3145, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x691e3148, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3e314d, SUC_lexicon_tags_7 },
    { 0x237e314e, SUC_lexicon_tags_64 },
    { 0x6e5e314f, SUC_lexicon_tags_37 },
    { 0x365e314f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xadfe3152, SUC_lexicon_tags_37 },
    { 0x4f1e3153, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xa0be3155, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177e315a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e315d, SUC_lexicon_tags_17 },
    { 0x123e315e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x123e3160, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165e3164, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x563e3166, SUC_lexicon_tags_11 },
    { 0x5dbe3167, SUC_lexicon_tags_15 },
    { 0xbede3167, SUC_lexicon_tags_31 },
    { 0x753e3169, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd43e316b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66be316d, SUC_lexicon_tags_1 },
    { 0xb75e316e, SUC_lexicon_tags_25 },
    { 0xcd5e316f, SUC_lexicon_tags_23 },
    { 0xb0de316d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10de3175, SUC_lexicon_tags_10 },
    { 0xc33e3176, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789e317b, SUC_lexicon_tags_47 },
    { 0xc69e317c, SUC_lexicon_tags_6 },
    { 0xff3e317b, SUC_lexicon_tags_47 },
    { 0x609e317e, SUC_lexicon_tags_26 },
    { 0xc49e317e, SUC_lexicon_tags_82 },
    { 0x3c7e3180, SUC_lexicon_tags_37 },
    { 0xfb9e3181, SUC_lexicon_tags_23 },
    { 0xfe3e3182, SUC_lexicon_tags_37 },
    { 0xf35e3183, SUC_lexicon_tags_8 },
    { 0x9ede3184, SUC_lexicon_tags_0 },
    { 0x199e317f, SUC_lexicon_tags_8 },
    { 0xa3fe317d, SUC_lexicon_tags_22 },
    { 0xa45e3187, SUC_lexicon_tags_2 },
    { 0x7b1e3188, SUC_lexicon_tags_47 },
    { 0x1a1e3189, SUC_lexicon_tags_18 },
    { 0x133e317d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x51de318d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82de3195, SUC_lexicon_tags_23 },
    { 0xa5e3195, SUC_lexicon_tags_47 },
    { 0x39e3195, SUC_lexicon_tags_22 },
    { 0xafbe3198, SUC_lexicon_tags_60 },
    { 0xf57e3199, SUC_lexicon_tags_38 },
    { 0x5e9e3195, SUC_lexicon_tags_45 },
    { 0x2fde3199, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfe5e319d, SUC_lexicon_tags_17 },
    { 0xa41e319e, SUC_lexicon_tags_37 },
    { 0xbcfe319f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x287e31a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e31ab, SUC_lexicon_tags_14 },
    { 0xf1fe31ac, SUC_lexicon_tags_8 },
    { 0xd9e31ad, SUC_lexicon_tags_25 },
    { 0xe5de31ae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e31b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d1e31b4, SUC_lexicon_tags_25 },
    { 0x679e31b5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25de31bd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64be31c1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x111e31c3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fe31c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3e31cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5ebe31ce, SUC_lexicon_tags_27 },
    { 0xf6be31cf, SUC_lexicon_tags_37 },
    { 0xcfde31d0, SUC_lexicon_tags_23 },
    { 0xca1e31d1, SUC_lexicon_tags_12 },
    { 0xc89e31d2, SUC_lexicon_tags_12 },
    { 0x729e31d2, SUC_lexicon_tags_25 },
    { 0x259e31d2, SUC_lexicon_tags_45 },
    { 0x1c7e31cf, SUC_lexicon_tags_0 },
    { 0x735e31d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35e31dc, SUC_lexicon_tags_1 },
    { 0xcbe31dc, SUC_lexicon_tags_25 },
    { 0x20be31de, SUC_lexicon_tags_1 },
    { 0x9ebe31df, SUC_lexicon_tags_60 },
    { 0xf05e31df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9e31e4, SUC_lexicon_tags_17 },
    { 0xd5e31e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48be31ec, SUC_lexicon_tags_27 },
    { 0x1ede31ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe31f0, SUC_lexicon_tags_25 },
    { 0x64fe31f1, SUC_lexicon_tags_25 },
    { 0xb7e31f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7e31f7, SUC_lexicon_tags_31 },
    { 0xaa1e31f8, SUC_lexicon_tags_7 },
    { 0xaf9e31f9, SUC_lexicon_tags_6 },
    { 0x147e31fa, SUC_lexicon_tags_17 },
    { 0x2dfe31f9, SUC_lexicon_tags_71 },
    { 0x6f3e31f7, SUC_lexicon_tags_12 },
    { 0x2fde31fc, SUC_lexicon_tags_27 },
    { 0x581e31fa, SUC_lexicon_tags_25 },
    { 0x31de31ff, SUC_lexicon_tags_18 },
    { 0x995e3200, SUC_lexicon_tags_1 },
    { 0xd05e3200, SUC_lexicon_tags_45 },
    { 0x1a9e3201, SUC_lexicon_tags_7 },
    { 0xdb9e31f8, SUC_lexicon_tags_43 },
    { 0x38be3204, SUC_lexicon_tags_48 },
    { 0x1ffe3205, SUC_lexicon_tags_37 },
    { 0x487e3206, SUC_lexicon_tags_11 },
    { 0x2afe3207, SUC_lexicon_tags_12 },
    { 0x5abe3208, SUC_lexicon_tags_31 },
    { 0xe1e3200, SUC_lexicon_tags_7 },
    { 0x4ede320a, SUC_lexicon_tags_1 },
    { 0x6dbe3209, SUC_lexicon_tags_0 },
    { 0x3c7e3203, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x29de320e, SUC_lexicon_tags_6 },
    { 0x971e320e, SUC_lexicon_tags_2 },
    { 0x2b5e3210, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fe3215, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x121e3217, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3be321c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57e3220, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x799e3222, SUC_lexicon_tags_45 },
    { 0xba7e3223, SUC_lexicon_tags_1 },
    { 0x29e3224, SUC_lexicon_tags_17 },
    { 0x987e3224, SUC_lexicon_tags_27 },
    { 0xf9be3225, SUC_lexicon_tags_48 },
    { 0x7cfe3227, SUC_lexicon_tags_11 },
    { 0x465e3223, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71de322c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c5e322e, SUC_lexicon_tags_15 },
    { 0xd1e322e, SUC_lexicon_tags_1 },
    { 0xf41e3230, SUC_lexicon_tags_17 },
    { 0x787e3231, SUC_lexicon_tags_16 },
    { 0x7f9e3232, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fe3235, SUC_lexicon_tags_25 },
    { 0x533e3235, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10de323b, SUC_lexicon_tags_2 },
    { 0x7f7e323b, SUC_lexicon_tags_30 },
    { 0x239e323b, SUC_lexicon_tags_52 },
    { 0xcf3e323e, SUC_lexicon_tags_11 },
    { 0xdf7e323f, SUC_lexicon_tags_60 },
    { 0x2f7e323f, SUC_lexicon_tags_47 },
    { 0x7c1e3241, SUC_lexicon_tags_204 },
    { 0xd75e3242, SUC_lexicon_tags_0 },
    { 0x757e3241, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7e3247, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9e324a, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x7fbe324c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x57be324e, SUC_lexicon_tags_11 },
    { 0xf6fe324e, SUC_lexicon_tags_61 },
    { 0x767e3250, SUC_lexicon_tags_17 },
    { 0x32be3251, SUC_lexicon_tags_37 },
    { 0x55be324e, SUC_lexicon_tags_33 },
    { 0x95e3253, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15e325b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd1e325d, SUC_lexicon_tags_12 },
    { 0x1fe325e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb07e3260, SUC_lexicon_tags_33 },
    { 0x7f9e3261, SUC_lexicon_tags_11 },
    { 0x453e3262, SUC_lexicon_tags_27 },
    { 0x97be3263, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fe326a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5e326c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb41e326e, SUC_lexicon_tags_11 },
    { 0x8cfe326e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5de3274, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbe3278, SUC_lexicon_tags_68 },
    { 0x35e3278, SUC_lexicon_tags_49 },
    { 0x2c3e327a, SUC_lexicon_tags_46 },
    { 0xfcfe327b, SUC_lexicon_tags_11 },
    { 0xd79e327b, SUC_lexicon_tags_48 },
    { 0xbe7e327a, SUC_lexicon_tags_60 },
    { 0x6e9e327d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa9de3280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf03e3282, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x573e3284, SUC_lexicon_tags_49 },
    { 0x993e3285, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183e3289, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887e328e, SUC_lexicon_tags_15 },
    { 0x1f5e328e, SUC_lexicon_tags_9 },
    { 0xb95e328e, SUC_lexicon_tags_49 },
    { 0x605e3291, SUC_lexicon_tags_14 },
    { 0x649e328e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5e329a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x885e329c, SUC_lexicon_tags_17 },
    { 0xc3e329c, SUC_lexicon_tags_28 },
    { 0x9dfe329e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbd5e32a0, SUC_lexicon_tags_11 },
    { 0xabde32a1, SUC_lexicon_tags_12 },
    { 0x2fde32a2, SUC_lexicon_tags_15 },
    { 0x5dfe32a3, SUC_lexicon_tags_11 },
    { 0xf53e32a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05e32af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9f7e32b1, SUC_lexicon_tags_25 },
    { 0x689e32b1, SUC_lexicon_tags_1 },
    { 0x527e32b2, SUC_lexicon_tags_30 },
    { 0xd65e32b4, SUC_lexicon_tags_127 },
    { 0xcb5e32b5, SUC_lexicon_tags_17 },
    { 0x885e32b6, SUC_lexicon_tags_14 },
    { 0x2a1e32b6, SUC_lexicon_tags_69 },
    { 0xaafe32b5, SUC_lexicon_tags_7 },
    { 0xf99e32b9, SUC_lexicon_tags_11 },
    { 0x3d7e32b8, SUC_lexicon_tags_17 },
    { 0xe67e32bb, SUC_lexicon_tags_49 },
    { 0xbe7e32b9, SUC_lexicon_tags_8 },
    { 0x563e32b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e32c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79de32c7, SUC_lexicon_tags_6 },
    { 0xe31e32c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14be32ca, SUC_lexicon_tags_1 },
    { 0x3a3e32cb, SUC_lexicon_tags_17 },
    { 0x535e32cc, SUC_lexicon_tags_40 },
    { 0x81e32ca, SUC_lexicon_tags_33 },
    { 0xd1fe32ce, SUC_lexicon_tags_12 },
    { 0x405e32cf, SUC_lexicon_tags_37 },
    { 0x8ade32ce, SUC_lexicon_tags_17 },
    { 0xaf7e32ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde32d4, SUC_lexicon_tags_11 },
    { 0x42fe32d4, SUC_lexicon_tags_19 },
    { 0xba9e32d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4a5e32d8, SUC_lexicon_tags_17 },
    { 0x52fe32d9, SUC_lexicon_tags_33 },
    { 0x49de32da, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7e32e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51de32e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70de32eb, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01e32ef, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x4e1e32f1, SUC_lexicon_tags_38 },
    { 0xf71e32f1, SUC_lexicon_tags_37 },
    { 0x1a9e32f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7e32f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xacfe32f8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc77e32fa, SUC_lexicon_tags_148 },
    { 0x7d3e32fb, SUC_lexicon_tags_23 },
    { 0x2f1e32fb, SUC_lexicon_tags_4 },
    { 0xa93e32fd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9e3302, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fe3305, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6de330a, SUC_lexicon_tags_23 },
    { 0x177e330a, SUC_lexicon_tags_37 },
    { 0x67de330c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbe3310, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x44be3313, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875e331c, SUC_lexicon_tags_11 },
    { 0xebde331d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987e3325, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1f1e3327, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1e3330, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x539e3332, SUC_lexicon_tags_47 },
    { 0xcf5e3333, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9e333c, SUC_lexicon_tags_2 },
    { 0x257e333c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fe3341, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347e3345, SUC_lexicon_tags_4 },
    { 0xfb1e3345, SUC_lexicon_tags_0 },
    { 0x16be3347, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85e334d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fe3352, SUC_lexicon_tags_43 },
    { 0x7d1e3352, SUC_lexicon_tags_17 },
    { 0x581e3354, SUC_lexicon_tags_45 },
    { 0xd5e3353, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebe3358, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49e335b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x9cfe335d, SUC_lexicon_tags_11 },
    { 0x349e335e, SUC_lexicon_tags_38 },
    { 0xcbfe335f, SUC_lexicon_tags_17 },
    { 0xe09e335f, SUC_lexicon_tags_17 },
    { 0x3ebe3361, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7e3364, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35be3368, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x513e336a, SUC_lexicon_tags_21 },
    { 0xea5e336b, SUC_lexicon_tags_17 },
    { 0xfcfe336a, SUC_lexicon_tags_87 },
    { 0x4dfe336d, SUC_lexicon_tags_2 },
    { 0x46be336e, SUC_lexicon_tags_17 },
    { 0x925e336a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x85be3372, SUC_lexicon_tags_14 },
    { 0xe9fe3373, SUC_lexicon_tags_47 },
    { 0x451e3374, SUC_lexicon_tags_7 },
    { 0x765e3373, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ade3379, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e3380, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5e3384, SUC_lexicon_tags_26 },
    { 0xd3e3384, SUC_lexicon_tags_15 },
    { 0x31be3386, SUC_lexicon_tags_11 },
    { 0x6d1e3387, SUC_lexicon_tags_47 },
    { 0xbc5e3388, SUC_lexicon_tags_0 },
    { 0x5dde3386, SUC_lexicon_tags_19 },
    { 0x9f7e3386, SUC_lexicon_tags_25 },
    { 0xccfe338b, SUC_lexicon_tags_87 },
    { 0x7c3e338c, SUC_lexicon_tags_25 },
    { 0xc1be338b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5fe338f, SUC_lexicon_tags_37 },
    { 0xb39e3390, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6c3e3392, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x767e3394, SUC_lexicon_tags_39 },
    { 0xf9e3394, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x41fe3397, SUC_lexicon_tags_15 },
    { 0x26fe3398, SUC_lexicon_tags_29 },
    { 0xf2fe3398, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5e339c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x649e339f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb2fe33a1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffe33a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63be33aa, SUC_lexicon_tags_1 },
    { 0xa5de33aa, SUC_lexicon_tags_12 },
    { 0xf6be33ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65e33af, SUC_lexicon_tags_25 },
    { 0xf73e33b0, SUC_lexicon_tags_56 },
    { 0xd3fe33b1, SUC_lexicon_tags_11 },
    { 0x907e33b2, SUC_lexicon_tags_25 },
    { 0x8b3e33af, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xe3fe33b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbe9e33b7, SUC_lexicon_tags_12 },
    { 0x1e33b8, SUC_lexicon_tags_8 },
    { 0x185e33b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d9e33bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x33be33be, SUC_lexicon_tags_17 },
    { 0x2b7e33be, SUC_lexicon_tags_37 },
    { 0x3efe33c0, SUC_lexicon_tags_1 },
    { 0x653e33c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5e33c6, SUC_lexicon_tags_31 },
    { 0x25de33c7, SUC_lexicon_tags_37 },
    { 0x10fe33c7, SUC_lexicon_tags_26 },
    { 0x16be33c9, SUC_lexicon_tags_66 },
    { 0xa3fe33ca, SUC_lexicon_tags_17 },
    { 0x3a9e33c7, SUC_lexicon_tags_37 },
    { 0xc1fe33cc, SUC_lexicon_tags_37 },
    { 0x67de33cc, SUC_lexicon_tags_49 },
    { 0x27be33cc, SUC_lexicon_tags_15 },
    { 0x209e33cb, SUC_lexicon_tags_6 },
    { 0xd39e33d0, SUC_lexicon_tags_37 },
    { 0x7dde33d0, SUC_lexicon_tags_47 },
    { 0xe91e33d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82de33d7, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede33db, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7ffe33dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161e33e1, SUC_lexicon_tags_8 },
    { 0x28de33e2, SUC_lexicon_tags_12 },
    { 0xd05e33e3, SUC_lexicon_tags_37 },
    { 0x8fde33e4, SUC_lexicon_tags_33 },
    { 0x7dde33e5, SUC_lexicon_tags_17 },
    { 0x131e33e5, SUC_lexicon_tags_28 },
    { 0xfc7e33e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x43be33e9, SUC_lexicon_tags_25 },
    { 0xb15e33e9, SUC_lexicon_tags_15 },
    { 0xaf9e33eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd45e33ed, SUC_lexicon_tags_25 },
    { 0x92be33ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94be33f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb45e33f4, SUC_lexicon_tags_1 },
    { 0xeabe33f4, SUC_lexicon_tags_2 },
    { 0xc8fe33f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dde33fa, SUC_lexicon_tags_15 },
    { 0xf5be33fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79e3402, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x57de3404, SUC_lexicon_tags_50 },
    { 0x291e3404, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbe340a, SUC_lexicon_tags_14 },
    { 0x613e340b, SUC_lexicon_tags_27 },
    { 0x4f5e340c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fe3413, SUC_lexicon_tags_29 },
    { 0x7a3e3413, SUC_lexicon_tags_14 },
    { 0xcbbe3413, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553e341a, SUC_lexicon_tags_11 },
    { 0x4f1e341b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10de341f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x809e3422, SUC_lexicon_tags_33 },
    { 0xc25e3423, SUC_lexicon_tags_26 },
    { 0x317e3424, SUC_lexicon_tags_45 },
    { 0xc8fe3423, SUC_lexicon_tags_25 },
    { 0xac1e3426, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x731e3429, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8be342d, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ede3437, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaade3439, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x213e343c, SUC_lexicon_tags_39 },
    { 0xf1fe343d, SUC_lexicon_tags_37 },
    { 0x7e7e343d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x127e3441, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaade3443, SUC_lexicon_tags_7 },
    { 0x791e3444, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42be344b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57be3450, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30be3452, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fe3455, SUC_lexicon_tags_33 },
    { 0x58be3456, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cde345a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9a9e345c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x569e345e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ebe3460, SUC_lexicon_tags_15 },
    { 0x241e3461, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3e3464, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e3467, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46fe3469, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e346d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81e3470, SUC_lexicon_tags_20 },
    { 0x4e1e3471, SUC_lexicon_tags_14 },
    { 0x929e3472, SUC_lexicon_tags_1 },
    { 0x76fe3473, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21e3476, SUC_lexicon_tags_21 },
    { 0x731e3476, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x743e347a, SUC_lexicon_tags_32 },
    { 0x635e347b, SUC_lexicon_tags_4 },
    { 0xc7de347c, SUC_lexicon_tags_7 },
    { 0x909e347d, SUC_lexicon_tags_25 },
    { 0x601e347e, SUC_lexicon_tags_7 },
    { 0x7f1e347d, SUC_lexicon_tags_12 },
    { 0x7c9e3480, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71e3485, SUC_lexicon_tags_37 },
    { 0xbbde3486, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11e348d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777e3497, SUC_lexicon_tags_2 },
    { 0x1f7e3497, SUC_lexicon_tags_9 },
    { 0x7cfe3499, SUC_lexicon_tags_1 },
    { 0x177e349a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775e349f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xf2be34a1, SUC_lexicon_tags_2 },
    { 0x74be34a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42de34a4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1de34a7, SUC_lexicon_tags_31 },
    { 0x3f9e34a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb93e34aa, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27e34b3, SUC_lexicon_tags_29 },
    { 0xf3fe34b4, SUC_lexicon_tags_74 },
    { 0x47de34b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97e34b8, SUC_lexicon_tags_39 },
    { 0x153e34b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efe34bc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67e34c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb31e34c4, SUC_lexicon_tags_32 },
    { 0xed3e34c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1e34cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf5e34ce, SUC_lexicon_tags_33 },
    { 0xab5e34ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x47fe34d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9e34d5, SUC_lexicon_tags_18 },
    { 0xf39e34d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9e34dc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1e34e0, SUC_lexicon_tags_63 },
    { 0x5cde34e1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93be34e7, SUC_lexicon_tags_61 },
    { 0xdfe34e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x21de34ea, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb55e34ec, SUC_lexicon_tags_33 },
    { 0x6de34ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5e34f2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957e34fa, SUC_lexicon_tags_45 },
    { 0x725e34fa, SUC_lexicon_tags_37 },
    { 0xb11e34fb, SUC_lexicon_tags_17 },
    { 0x169e34fc, SUC_lexicon_tags_26 },
    { 0x917e34fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x337e3500, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa9fe3502, SUC_lexicon_tags_17 },
    { 0x9dbe3502, SUC_lexicon_tags_29 },
    { 0x237e3503, SUC_lexicon_tags_19 },
    { 0x7e9e3505, SUC_lexicon_tags_14 },
    { 0x781e3502, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x797e3509, SUC_lexicon_tags_2 },
    { 0x51fe350a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7e3510, SUC_lexicon_tags_49 },
    { 0xfe5e3511, SUC_lexicon_tags_37 },
    { 0x737e3511, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x715e3514, SUC_lexicon_tags_30 },
    { 0xb8be3514, SUC_lexicon_tags_25 },
    { 0x54de3515, SUC_lexicon_tags_6 },
    { 0x66be3517, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbe351c, SUC_lexicon_tags_0 },
    { 0x385e351d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905e3522, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x595e3525, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x67de3527, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09e352b, SUC_lexicon_tags_39 },
    { 0x1e7e352c, SUC_lexicon_tags_7 },
    { 0x8a5e352d, SUC_lexicon_tags_15 },
    { 0x509e352e, SUC_lexicon_tags_2 },
    { 0xb95e352b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebe3532, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb95e3534, SUC_lexicon_tags_9 },
    { 0x56fe3535, SUC_lexicon_tags_15 },
    { 0x917e3536, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84be3538, SUC_lexicon_tags_7 },
    { 0x3de3538, SUC_lexicon_tags_31 },
    { 0x675e353a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x5f9e353c, SUC_lexicon_tags_47 },
    { 0xff7e353c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x985e3540, SUC_lexicon_tags_15 },
    { 0x31fe3541, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9e3546, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfe3549, SUC_lexicon_tags_37 },
    { 0xf89e3549, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cde354d, SUC_lexicon_tags_21 },
    { 0x55e354e, SUC_lexicon_tags_38 },
    { 0xdf1e354f, SUC_lexicon_tags_43 },
    { 0x499e354e, SUC_lexicon_tags_85 },
    { 0x161e354f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebe3554, SUC_lexicon_tags_9 },
    { 0x9f7e3554, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7de3558, SUC_lexicon_tags_37 },
    { 0xd9e3559, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4fe355b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51e3560, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dde3567, SUC_lexicon_tags_46 },
    { 0xb27e3568, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99e356c, SUC_lexicon_tags_6 },
    { 0xb9de356d, SUC_lexicon_tags_31 },
    { 0x879e356d, SUC_lexicon_tags_53 },
    { 0xbb1e356f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbc9e3571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3e3575, SUC_lexicon_tags_25 },
    { 0xd01e3576, SUC_lexicon_tags_63 },
    { 0xdfe3577, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51fe357a, SUC_lexicon_tags_38 },
    { 0x837e357b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x719e357d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35e3580, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb37e3582, SUC_lexicon_tags_9 },
    { 0x27de3583, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x743e3585, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x715e3587, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33e358b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91de358f, SUC_lexicon_tags_37 },
    { 0x539e3590, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597e3599, SUC_lexicon_tags_6 },
    { 0x31e3599, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17e359e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343e35a4, SUC_lexicon_tags_14 },
    { 0x1d1e35a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x27de35a7, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x6dfe35a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1c7e35ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795e35b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9e35b5, SUC_lexicon_tags_7 },
    { 0xcf9e35b5, SUC_lexicon_tags_216 },
    { 0x775e35b6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7e35ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745e35c0, SUC_lexicon_tags_17 },
    { 0x5a7e35c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb31e35c3, SUC_lexicon_tags_68 },
    { 0x345e35c4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffe35c7, SUC_lexicon_tags_29 },
    { 0x679e35c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x983e35ca, SUC_lexicon_tags_37 },
    { 0xa31e35ca, SUC_lexicon_tags_49 },
    { 0x24de35cc, SUC_lexicon_tags_37 },
    { 0x701e35ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5e35d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad7e35d3, SUC_lexicon_tags_12 },
    { 0xb7e35d3, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xb47e35d6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfb1e35d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5e35dc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfe35e0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1e35e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x973e35e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a1e35e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5e35ec, SUC_lexicon_tags_0 },
    { 0xc2be35ed, SUC_lexicon_tags_16 },
    { 0xdc7e35ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80be35f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x77de35f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c1e35f6, SUC_lexicon_tags_35 },
    { 0x7fde35f7, SUC_lexicon_tags_28 },
    { 0xe21e35f7, SUC_lexicon_tags_17 },
    { 0x755e35f9, SUC_lexicon_tags_37 },
    { 0x6e7e35f9, SUC_lexicon_tags_17 },
    { 0xbf1e35fa, SUC_lexicon_tags_11 },
    { 0x719e35fc, SUC_lexicon_tags_79 },
    { 0x99be35fd, SUC_lexicon_tags_37 },
    { 0x601e35fe, SUC_lexicon_tags_49 },
    { 0xe21e35fe, SUC_lexicon_tags_13 },
    { 0x927e35ff, SUC_lexicon_tags_37 },
    { 0xc93e35f8, SUC_lexicon_tags_0 },
    { 0x163e35f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743e360e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5e3614, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e7e3616, SUC_lexicon_tags_7 },
    { 0xc47e3616, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc5de3619, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcc1e361b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf09e361d, SUC_lexicon_tags_8 },
    { 0x171e361e, SUC_lexicon_tags_7 },
    { 0x821e361e, SUC_lexicon_tags_23 },
    { 0x733e361d, SUC_lexicon_tags_140 },
    { 0xdcbe361e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7e3625, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94de362a, SUC_lexicon_tags_7 },
    { 0x28be362b, SUC_lexicon_tags_37 },
    { 0xa1fe362c, SUC_lexicon_tags_25 },
    { 0xa4de362d, SUC_lexicon_tags_37 },
    { 0xf71e362e, SUC_lexicon_tags_17 },
    { 0xd31e362a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x245e3631, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf87e3634, SUC_lexicon_tags_12 },
    { 0x241e3635, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cde363a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9e363f, SUC_lexicon_tags_8 },
    { 0x959e3640, SUC_lexicon_tags_45 },
    { 0x3ebe3640, SUC_lexicon_tags_46 },
    { 0xa3de3640, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf0be3644, SUC_lexicon_tags_26 },
    { 0xc95e3644, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x87be3647, SUC_lexicon_tags_37 },
    { 0x5a3e3648, SUC_lexicon_tags_201 },
    { 0x2e5e3648, SUC_lexicon_tags_1 },
    { 0xcede3648, SUC_lexicon_tags_11 },
    { 0xf6be3648, SUC_lexicon_tags_45 },
    { 0x13de364c, SUC_lexicon_tags_25 },
    { 0x8a1e3647, SUC_lexicon_tags_1 },
    { 0x107e364b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x121e3651, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55e3654, SUC_lexicon_tags_44 },
    { 0x407e3654, SUC_lexicon_tags_12 },
    { 0x67fe3654, SUC_lexicon_tags_44 },
    { 0x45de3657, SUC_lexicon_tags_19 },
    { 0x679e3658, SUC_lexicon_tags_0 },
    { 0xad3e3659, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x25be365c, SUC_lexicon_tags_15 },
    { 0x355e365d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca3e365f, SUC_lexicon_tags_15 },
    { 0x81be3660, SUC_lexicon_tags_47 },
    { 0x745e3661, SUC_lexicon_tags_3 },
    { 0xe71e3660, SUC_lexicon_tags_33 },
    { 0xda1e3663, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x217e3666, SUC_lexicon_tags_17 },
    { 0x6a7e3666, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe95e3669, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13be3672, SUC_lexicon_tags_26 },
    { 0x749e3673, SUC_lexicon_tags_49 },
    { 0xa1e3674, SUC_lexicon_tags_9 },
    { 0x689e3675, SUC_lexicon_tags_14 },
    { 0xfffe3676, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23e3679, SUC_lexicon_tags_12 },
    { 0x607e3679, SUC_lexicon_tags_95 },
    { 0x643e367a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fe3683, SUC_lexicon_tags_13 },
    { 0x36de3684, SUC_lexicon_tags_40 },
    { 0x9b7e3685, SUC_lexicon_tags_29 },
    { 0x1bbe3686, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2de3689, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923e368d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55de3691, SUC_lexicon_tags_25 },
    { 0xfa3e3692, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x337e3694, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5cde3696, SUC_lexicon_tags_60 },
    { 0xa61e3697, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5e369d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefe36a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4b5e36a4, SUC_lexicon_tags_37 },
    { 0x61be36a5, SUC_lexicon_tags_12 },
    { 0x1e1e36a6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x10de36a9, SUC_lexicon_tags_69 },
    { 0x597e36a9, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x9d9e36ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x64fe36ae, SUC_lexicon_tags_14 },
    { 0x9d9e36ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fe36b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42be36bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9f3e36be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c5e36c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc9e36c2, SUC_lexicon_tags_27 },
    { 0x853e36c3, SUC_lexicon_tags_1 },
    { 0xe69e36c3, SUC_lexicon_tags_23 },
    { 0x9dfe36c3, SUC_lexicon_tags_6 },
    { 0x555e36c6, SUC_lexicon_tags_43 },
    { 0x2de36c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e1e36c9, SUC_lexicon_tags_37 },
    { 0x7a5e36c9, SUC_lexicon_tags_7 },
    { 0xdbfe36cb, SUC_lexicon_tags_33 },
    { 0x6f3e36cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1de36d5, SUC_lexicon_tags_17 },
    { 0x86be36d5, SUC_lexicon_tags_37 },
    { 0x4efe36d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3c9e36d9, SUC_lexicon_tags_45 },
    { 0xb55e36da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86de36dc, SUC_lexicon_tags_47 },
    { 0x865e36dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd1fe36df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1be36e1, SUC_lexicon_tags_1 },
    { 0x8d5e36e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd4be36e4, SUC_lexicon_tags_15 },
    { 0xabe36e5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73e36ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbe36f2, SUC_lexicon_tags_2 },
    { 0x161e36f3, SUC_lexicon_tags_55 },
    { 0x9b9e36f2, SUC_lexicon_tags_47 },
    { 0x15be36f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe89e36f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9de36fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9be36ff, SUC_lexicon_tags_55 },
    { 0xd05e3700, SUC_lexicon_tags_39 },
    { 0xb73e3701, SUC_lexicon_tags_38 },
    { 0x569e3702, SUC_lexicon_tags_12 },
    { 0xc9fe3701, SUC_lexicon_tags_25 },
    { 0xad3e3704, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2b9e3706, SUC_lexicon_tags_35 },
    { 0x753e3707, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01e370c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63e370f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447e3713, SUC_lexicon_tags_77 },
    { 0xb93e3714, SUC_lexicon_tags_25 },
    { 0x601e3715, SUC_lexicon_tags_2 },
    { 0x7afe3716, SUC_lexicon_tags_29 },
    { 0xda3e3717, SUC_lexicon_tags_6 },
    { 0xfe9e3717, SUC_lexicon_tags_11 },
    { 0x72be3719, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xeefe371b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245e3721, SUC_lexicon_tags_15 },
    { 0x513e3721, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1be3725, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28fe3727, SUC_lexicon_tags_37 },
    { 0x4ffe3728, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ade372b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501e372f, SUC_lexicon_tags_12 },
    { 0xab3e372f, SUC_lexicon_tags_11 },
    { 0x39be3731, SUC_lexicon_tags_65 },
    { 0x465e3731, SUC_lexicon_tags_13 },
    { 0x143e372f, SUC_lexicon_tags_47 },
    { 0x991e3734, SUC_lexicon_tags_12 },
    { 0x4b9e3735, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919e3739, SUC_lexicon_tags_17 },
    { 0x623e373a, SUC_lexicon_tags_26 },
    { 0x281e373b, SUC_lexicon_tags_12 },
    { 0x325e373c, SUC_lexicon_tags_48 },
    { 0xb37e373c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x97de3740, SUC_lexicon_tags_63 },
    { 0xdd1e3741, SUC_lexicon_tags_25 },
    { 0xf65e3741, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffde3748, SUC_lexicon_tags_38 },
    { 0xa23e3749, SUC_lexicon_tags_27 },
    { 0xf77e3749, SUC_lexicon_tags_3 },
    { 0xc99e3749, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaede374d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267e3751, SUC_lexicon_tags_33 },
    { 0xb7de3752, SUC_lexicon_tags_39 },
    { 0x9fde3752, SUC_lexicon_tags_10 },
    { 0x74fe3754, SUC_lexicon_tags_19 },
    { 0xa3be3755, SUC_lexicon_tags_63 },
    { 0xca9e3752, SUC_lexicon_tags_25 },
    { 0xc5be3756, SUC_lexicon_tags_0 },
    { 0x7d5e3758, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619e375e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9e3764, SUC_lexicon_tags_31 },
    { 0x6d9e3764, SUC_lexicon_tags_11 },
    { 0xba9e3766, SUC_lexicon_tags_0 },
    { 0x17be3766, SUC_lexicon_tags_1 },
    { 0x383e3768, SUC_lexicon_tags_39 },
    { 0x48de3769, SUC_lexicon_tags_25 },
    { 0x72be3768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fe3772, SUC_lexicon_tags_7 },
    { 0x3abe3772, SUC_lexicon_tags_29 },
    { 0x4ade3772, SUC_lexicon_tags_1 },
    { 0x4e9e3775, SUC_lexicon_tags_63 },
    { 0xe85e3776, SUC_lexicon_tags_28 },
    { 0xabe3776, SUC_lexicon_tags_19 },
    { 0xdbbe3777, SUC_lexicon_tags_15 },
    { 0xb41e3779, SUC_lexicon_tags_23 },
    { 0x77e377a, SUC_lexicon_tags_9 },
    { 0xd51e377b, SUC_lexicon_tags_6 },
    { 0xa6be3779, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1a5e377e, SUC_lexicon_tags_1 },
    { 0x9ade377f, SUC_lexicon_tags_7 },
    { 0xa4fe3780, SUC_lexicon_tags_12 },
    { 0x297e3781, SUC_lexicon_tags_12 },
    { 0xb99e3780, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9e3787, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1ebe3789, SUC_lexicon_tags_2 },
    { 0x6d1e378a, SUC_lexicon_tags_15 },
    { 0x693e378b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0be3799, SUC_lexicon_tags_12 },
    { 0xe91e379a, SUC_lexicon_tags_23 },
    { 0x379e379b, SUC_lexicon_tags_10 },
    { 0x5bde379b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07e379f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e37a4, SUC_lexicon_tags_45 },
    { 0x953e37a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5e37a8, SUC_lexicon_tags_1 },
    { 0xe25e37a9, SUC_lexicon_tags_7 },
    { 0xee3e37aa, SUC_lexicon_tags_25 },
    { 0xd07e37ab, SUC_lexicon_tags_151 },
    { 0xdb1e37ac, SUC_lexicon_tags_45 },
    { 0x195e37a8, SUC_lexicon_tags_15 },
    { 0x38de37ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fde37b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x58be37b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fe37b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbfe37b9, SUC_lexicon_tags_7 },
    { 0x3fde37ba, SUC_lexicon_tags_0 },
    { 0x1dbe37bb, SUC_lexicon_tags_7 },
    { 0x66fe37bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe37bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161e37c7, SUC_lexicon_tags_37 },
    { 0x26be37c8, SUC_lexicon_tags_11 },
    { 0xfb7e37c9, SUC_lexicon_tags_1 },
    { 0x51de37ca, SUC_lexicon_tags_17 },
    { 0x8a7e37c7, SUC_lexicon_tags_2 },
    { 0x801e37c8, SUC_lexicon_tags_1 },
    { 0x399e37cd, SUC_lexicon_tags_82 },
    { 0x427e37ce, SUC_lexicon_tags_7 },
    { 0xd4de37cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5de37d7, SUC_lexicon_tags_45 },
    { 0x66fe37d7, SUC_lexicon_tags_11 },
    { 0x36be37d9, SUC_lexicon_tags_17 },
    { 0x2ebe37d9, SUC_lexicon_tags_10 },
    { 0x567e37d8, SUC_lexicon_tags_5 },
    { 0x3a7e37d9, SUC_lexicon_tags_31 },
    { 0x9cfe37db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61e37e5, SUC_lexicon_tags_33 },
    { 0xb7be37e6, SUC_lexicon_tags_17 },
    { 0xd03e37e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x483e37ea, SUC_lexicon_tags_33 },
    { 0x1e9e37ea, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1be37ef, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77e37f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x765e37f8, SUC_lexicon_tags_79 },
    { 0x913e37f8, SUC_lexicon_tags_31 },
    { 0x5a3e37fa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4e9e37fc, SUC_lexicon_tags_25 },
    { 0x555e37fd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5bde37ff, SUC_lexicon_tags_27 },
    { 0x14be3800, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe3803, SUC_lexicon_tags_37 },
    { 0xf07e3803, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x591e3806, SUC_lexicon_tags_94 },
    { 0x37e3807, SUC_lexicon_tags_6 },
    { 0x5b3e3807, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x765e380b, SUC_lexicon_tags_21 },
    { 0x81e380b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x405e380e, SUC_lexicon_tags_7 },
    { 0x841e380f, SUC_lexicon_tags_17 },
    { 0xcd9e380f, SUC_lexicon_tags_7 },
    { 0x935e3810, SUC_lexicon_tags_25 },
    { 0xc15e3812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d3e3814, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9e3819, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917e381d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd59e381f, SUC_lexicon_tags_15 },
    { 0x33be381f, SUC_lexicon_tags_11 },
    { 0xcd5e3821, SUC_lexicon_tags_7 },
    { 0x3a3e3822, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x569e3824, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fe3827, SUC_lexicon_tags_551 },
    { 0xb09e3828, SUC_lexicon_tags_17 },
    { 0xcde3829, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x40fe382b, SUC_lexicon_tags_37 },
    { 0x9fbe382c, SUC_lexicon_tags_82 },
    { 0x5dde382c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543e3831, SUC_lexicon_tags_7 },
    { 0x7a1e3832, SUC_lexicon_tags_37 },
    { 0xcfde3833, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36be3835, SUC_lexicon_tags_33 },
    { 0x6fe3835, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe09e3838, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x109e383a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x12fe383c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x16de383e, SUC_lexicon_tags_7 },
    { 0x1e7e383e, SUC_lexicon_tags_25 },
    { 0x251e383e, SUC_lexicon_tags_14 },
    { 0xff1e383f, SUC_lexicon_tags_37 },
    { 0xdc5e3840, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3efe3844, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5e3e3846, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97de384a, SUC_lexicon_tags_63 },
    { 0x401e384b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x775e384d, SUC_lexicon_tags_1 },
    { 0x109e384d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491e3853, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xebfe3856, SUC_lexicon_tags_25 },
    { 0xdbde3857, SUC_lexicon_tags_19 },
    { 0xd2fe3857, SUC_lexicon_tags_25 },
    { 0xb29e3859, SUC_lexicon_tags_11 },
    { 0x727e3856, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10be385c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5e385f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487e3863, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x547e3865, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6b3e3867, SUC_lexicon_tags_12 },
    { 0xb4de3868, SUC_lexicon_tags_10 },
    { 0x5c9e3869, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a9e386b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d3e386d, SUC_lexicon_tags_7 },
    { 0x79de386e, SUC_lexicon_tags_49 },
    { 0xcc9e386d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x131e3872, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x207e3874, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22be3878, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471e387f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983e3885, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b9e3887, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1e388a, SUC_lexicon_tags_7 },
    { 0x7cfe388b, SUC_lexicon_tags_32 },
    { 0xf51e388b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2be388e, SUC_lexicon_tags_92 },
    { 0x59de388f, SUC_lexicon_tags_17 },
    { 0xa71e3890, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44be3892, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6c5e3894, SUC_lexicon_tags_25 },
    { 0x39e3895, SUC_lexicon_tags_2 },
    { 0x6f3e3896, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e3899, SUC_lexicon_tags_60 },
    { 0x5fe389a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6f5e389c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x2a7e389e, SUC_lexicon_tags_19 },
    { 0x373e389e, SUC_lexicon_tags_9 },
    { 0x32be38a0, SUC_lexicon_tags_11 },
    { 0x3bfe38a1, SUC_lexicon_tags_68 },
    { 0x66fe38a0, SUC_lexicon_tags_16 },
    { 0xfb5e389e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fe38aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903e38ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x829e38b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x311e38b2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45be38b4, SUC_lexicon_tags_16 },
    { 0x3afe38b5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c5e38b7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255e38bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65e38bf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5e38c2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x73e38c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e38c9, SUC_lexicon_tags_11 },
    { 0x781e38c9, SUC_lexicon_tags_17 },
    { 0xc69e38cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41de38cd, SUC_lexicon_tags_2 },
    { 0xa01e38ce, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe15e38d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90de38d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563e38d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbe38dc, SUC_lexicon_tags_29 },
    { 0xe25e38dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7ade38df, SUC_lexicon_tags_7 },
    { 0x4efe38e0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x535e38e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55be38e4, SUC_lexicon_tags_1 },
    { 0x4d7e38e5, SUC_lexicon_tags_47 },
    { 0xebe38e6, SUC_lexicon_tags_37 },
    { 0xb1fe38e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3e38ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e3e38ef, SUC_lexicon_tags_63 },
    { 0x193e38ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x887e38f2, SUC_lexicon_tags_13 },
    { 0x521e38f2, SUC_lexicon_tags_3 },
    { 0x593e38f4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x655e38f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b1e38f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7e38fb, SUC_lexicon_tags_33 },
    { 0x43e38fc, SUC_lexicon_tags_0 },
    { 0x1c9e38fd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x397e3900, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x951e3902, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1e3905, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9e3909, SUC_lexicon_tags_14 },
    { 0x195e390a, SUC_lexicon_tags_11 },
    { 0x5e1e390a, SUC_lexicon_tags_45 },
    { 0xd01e390c, SUC_lexicon_tags_37 },
    { 0xd4fe390c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635e3913, SUC_lexicon_tags_11 },
    { 0xf81e3913, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9c3e3916, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101e391b, SUC_lexicon_tags_10 },
    { 0x1fde391c, SUC_lexicon_tags_66 },
    { 0x5ade391c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf6be391f, SUC_lexicon_tags_45 },
    { 0x199e3920, SUC_lexicon_tags_48 },
    { 0x791e3921, SUC_lexicon_tags_49 },
    { 0xf81e391f, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3e3926, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc7de3928, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77e392d, SUC_lexicon_tags_31 },
    { 0x2ede392d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4be3935, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x319e3937, SUC_lexicon_tags_37 },
    { 0xb5be3938, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3e393b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53e3943, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x343e3945, SUC_lexicon_tags_17 },
    { 0x8b3e3945, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7e394a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x43e394c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x86be394f, SUC_lexicon_tags_49 },
    { 0x4b5e394f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cde3956, SUC_lexicon_tags_12 },
    { 0x99be3956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x883e395a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81be3961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1de3963, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ede3967, SUC_lexicon_tags_77 },
    { 0xc93e3968, SUC_lexicon_tags_14 },
    { 0xa41e3967, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7e3972, SUC_lexicon_tags_15 },
    { 0x4dbe3973, SUC_lexicon_tags_33 },
    { 0xf17e3973, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x175e3977, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xd33e3979, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859e3984, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfe3987, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97e398a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1fe398c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92be3990, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abe3997, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43e3999, SUC_lexicon_tags_9 },
    { 0xc59e399a, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67e399f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf65e39a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5e39a5, SUC_lexicon_tags_19 },
    { 0x547e39a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605e39ab, SUC_lexicon_tags_12 },
    { 0x11fe39ac, SUC_lexicon_tags_0 },
    { 0xf69e39ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45e39b5, SUC_lexicon_tags_16 },
    { 0xaafe39b6, SUC_lexicon_tags_11 },
    { 0xdb7e39b7, SUC_lexicon_tags_9 },
    { 0xf3fe39b5, SUC_lexicon_tags_16 },
    { 0x43e39b9, SUC_lexicon_tags_3 },
    { 0x5d3e39b9, SUC_lexicon_tags_6 },
    { 0xcb1e39ba, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959e39bf, SUC_lexicon_tags_68 },
    { 0xf39e39c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97fe39c2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e39c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x985e39c7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5be39cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39e39cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e39d2, SUC_lexicon_tags_1 },
    { 0x2c9e39d3, SUC_lexicon_tags_11 },
    { 0x215e39d4, SUC_lexicon_tags_21 },
    { 0x519e39d5, SUC_lexicon_tags_2 },
    { 0xcc3e39d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3e39e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235e39e6, SUC_lexicon_tags_0 },
    { 0x6a9e39e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x713e39ea, SUC_lexicon_tags_37 },
    { 0x8d3e39eb, SUC_lexicon_tags_3 },
    { 0x8fe39eb, SUC_lexicon_tags_12 },
    { 0x7afe39ed, SUC_lexicon_tags_40 },
    { 0x819e39ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867e39f9, SUC_lexicon_tags_1 },
    { 0xb55e39f9, SUC_lexicon_tags_11 },
    { 0xb9fe39fb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679e39ff, SUC_lexicon_tags_12 },
    { 0xabfe39ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fe3a03, SUC_lexicon_tags_25 },
    { 0x3bfe3a04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5e3a0b, SUC_lexicon_tags_14 },
    { 0xae9e3a0c, SUC_lexicon_tags_7 },
    { 0xf61e3a0d, SUC_lexicon_tags_45 },
    { 0xedbe3a0b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783e3a12, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79e3a15, SUC_lexicon_tags_10 },
    { 0x735e3a16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3a1c, SUC_lexicon_tags_45 },
    { 0xdafe3a1d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3e3a20, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1e3a23, SUC_lexicon_tags_1 },
    { 0x8c9e3a24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x413e3a27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x985e3a2a, SUC_lexicon_tags_49 },
    { 0x159e3a2a, SUC_lexicon_tags_25 },
    { 0x367e3a2a, SUC_lexicon_tags_33 },
    { 0x901e3a2d, SUC_lexicon_tags_23 },
    { 0xd2be3a2e, SUC_lexicon_tags_37 },
    { 0xe71e3a2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd77e3a31, SUC_lexicon_tags_7 },
    { 0xfe3a32, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfa3e3a34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc39e3a36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11de3a38, SUC_lexicon_tags_29 },
    { 0x695e3a38, SUC_lexicon_tags_3 },
    { 0xb4be3a3a, SUC_lexicon_tags_87 },
    { 0xbbde3a38, SUC_lexicon_tags_37 },
    { 0x30fe3a3b, SUC_lexicon_tags_17 },
    { 0xa1e3a3d, SUC_lexicon_tags_37 },
    { 0x42fe3a39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fe3a41, SUC_lexicon_tags_25 },
    { 0x37de3a42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x465e3a44, SUC_lexicon_tags_30 },
    { 0x98fe3a44, SUC_lexicon_tags_38 },
    { 0x76be3a44, SUC_lexicon_tags_37 },
    { 0x8bde3a45, SUC_lexicon_tags_32 },
    { 0xbb7e3a44, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5e3a54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6afe3a56, SUC_lexicon_tags_25 },
    { 0xd49e3a57, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1be3a5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6be3a5f, SUC_lexicon_tags_12 },
    { 0xa2fe3a60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1e3a65, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bde3a72, SUC_lexicon_tags_68 },
    { 0xca7e3a73, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49e3a76, SUC_lexicon_tags_50 },
    { 0x5a1e3a77, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49e3a7c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de3a7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611e3a85, SUC_lexicon_tags_11 },
    { 0x219e3a85, SUC_lexicon_tags_6 },
    { 0x179e3a86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8be3a8d, SUC_lexicon_tags_11 },
    { 0x687e3a8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7e3a91, SUC_lexicon_tags_25 },
    { 0xaffe3a91, SUC_lexicon_tags_19 },
    { 0x147e3a93, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e3e3a95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cbe3a97, SUC_lexicon_tags_6 },
    { 0xdcde3a97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe3a9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05e3aa4, SUC_lexicon_tags_1 },
    { 0x59de3aa5, SUC_lexicon_tags_10 },
    { 0x4f1e3aa6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x415e3aa9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87de3aad, SUC_lexicon_tags_23 },
    { 0xae7e3aae, SUC_lexicon_tags_153 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5e3ab1, SUC_lexicon_tags_11 },
    { 0xf9be3ab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x721e3ab5, SUC_lexicon_tags_55 },
    { 0xd0fe3ab6, SUC_lexicon_tags_19 },
    { 0xb71e3ab7, SUC_lexicon_tags_26 },
    { 0x8d7e3ab8, SUC_lexicon_tags_7 },
    { 0x993e3ab5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47e3abe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1de3ac0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1de3ac2, SUC_lexicon_tags_35 },
    { 0xa25e3ac3, SUC_lexicon_tags_37 },
    { 0x857e3ac2, SUC_lexicon_tags_11 },
    { 0x7ede3ac3, SUC_lexicon_tags_27 },
    { 0x7e3ac6, SUC_lexicon_tags_62 },
    { 0xfd7e3ac4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4c1e3ac9, SUC_lexicon_tags_21 },
    { 0xb65e3aca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bde3acd, SUC_lexicon_tags_40 },
    { 0x989e3ace, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ade3ad0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1e3ad8, SUC_lexicon_tags_15 },
    { 0x991e3ad9, SUC_lexicon_tags_47 },
    { 0xca9e3ad9, SUC_lexicon_tags_14 },
    { 0x377e3adb, SUC_lexicon_tags_10 },
    { 0x687e3adc, SUC_lexicon_tags_23 },
    { 0xcbfe3add, SUC_lexicon_tags_11 },
    { 0xa43e3add, SUC_lexicon_tags_87 },
    { 0xfdfe3add, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x455e3ae2, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abe3ae8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437e3aec, SUC_lexicon_tags_6 },
    { 0xca3e3aed, SUC_lexicon_tags_1 },
    { 0xfa3e3aed, SUC_lexicon_tags_12 },
    { 0xc5e3aed, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793e3af4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9e3af9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63e3afd, SUC_lexicon_tags_15 },
    { 0xd07e3afe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1fe3b00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fe3b03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41e3b06, SUC_lexicon_tags_19 },
    { 0x317e3b06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf37e3b09, SUC_lexicon_tags_15 },
    { 0xe11e3b0a, SUC_lexicon_tags_25 },
    { 0xe41e3b09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9be3b11, SUC_lexicon_tags_2 },
    { 0xfd1e3b12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc77e3b15, SUC_lexicon_tags_60 },
    { 0x4a5e3b16, SUC_lexicon_tags_11 },
    { 0xd85e3b17, SUC_lexicon_tags_37 },
    { 0x279e3b18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d9e3b1a, SUC_lexicon_tags_4 },
    { 0x5fde3b1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa73e3b1d, SUC_lexicon_tags_30 },
    { 0x5e1e3b1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de3b21, SUC_lexicon_tags_14 },
    { 0x4e9e3b22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb49e3b24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbe3b2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb5e3b2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5e3b30, SUC_lexicon_tags_48 },
    { 0xafbe3b31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3e3b34, SUC_lexicon_tags_27 },
    { 0x179e3b35, SUC_lexicon_tags_17 },
    { 0x82be3b36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ade3b3a, SUC_lexicon_tags_14 },
    { 0xdf3e3b3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755e3b3f, SUC_lexicon_tags_15 },
    { 0x513e3b40, SUC_lexicon_tags_63 },
    { 0x7fde3b41, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb4fe3b43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5e3b47, SUC_lexicon_tags_25 },
    { 0x3c7e3b48, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x45de3b4a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xed1e3b4c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fe3b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccde3b51, SUC_lexicon_tags_27 },
    { 0xcb7e3b52, SUC_lexicon_tags_6 },
    { 0x119e3b53, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7e3b56, SUC_lexicon_tags_14 },
    { 0xd93e3b57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebbe3b59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc19e3b5b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x13e3b5d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2fe3b5f, SUC_lexicon_tags_0 },
    { 0xb5fe3b5f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5ebe3b62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86de3b68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x965e3b6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x683e3b6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafe3b73, SUC_lexicon_tags_565 },
    { 0x32de3b74, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4de3b76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e3b79, SUC_lexicon_tags_7 },
    { 0xfa5e3b79, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9e3b7d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76be3b82, SUC_lexicon_tags_15 },
    { 0xefbe3b83, SUC_lexicon_tags_17 },
    { 0x51be3b84, SUC_lexicon_tags_36 },
    { 0x7a7e3b84, SUC_lexicon_tags_2 },
    { 0x445e3b83, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x383e3b88, SUC_lexicon_tags_39 },
    { 0xcf3e3b89, SUC_lexicon_tags_12 },
    { 0x1f7e3b8a, SUC_lexicon_tags_31 },
    { 0xdcfe3b8b, SUC_lexicon_tags_1 },
    { 0x95be3b8c, SUC_lexicon_tags_34 },
    { 0xa75e3b8d, SUC_lexicon_tags_160 },
    { 0xb17e3b8e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbe3b91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x235e3b94, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbb3e3b96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x35be3b98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb19e3b9a, SUC_lexicon_tags_31 },
    { 0xe1de3b9b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fe3ba0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3ba3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5ffe3ba5, SUC_lexicon_tags_2 },
    { 0xe19e3ba6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13e3ba9, SUC_lexicon_tags_23 },
    { 0xc21e3ba9, SUC_lexicon_tags_17 },
    { 0x8be3bab, SUC_lexicon_tags_224 },
    { 0x4b7e3bac, SUC_lexicon_tags_33 },
    { 0x501e3bab, SUC_lexicon_tags_12 },
    { 0x6ebe3baa, SUC_lexicon_tags_14 },
    { 0xb05e3baf, SUC_lexicon_tags_2 },
    { 0xfffe3baf, SUC_lexicon_tags_11 },
    { 0x7afe3bb0, SUC_lexicon_tags_25 },
    { 0xb65e3bb2, SUC_lexicon_tags_253 },
    { 0x43de3bb0, SUC_lexicon_tags_3 },
    { 0xa9de3bb4, SUC_lexicon_tags_15 },
    { 0xa0fe3bb5, SUC_lexicon_tags_55 },
    { 0xff5e3bb1, SUC_lexicon_tags_25 },
    { 0x569e3bb7, SUC_lexicon_tags_13 },
    { 0x411e3bb7, SUC_lexicon_tags_29 },
    { 0x111e3bac, SUC_lexicon_tags_12 },
    { 0x9dbe3bba, SUC_lexicon_tags_30 },
    { 0xb35e3bb3, SUC_lexicon_tags_63 },
    { 0x435e3bad, SUC_lexicon_tags_645 },
    { 0x35e3bb0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5e3bc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e3bc4, SUC_lexicon_tags_15 },
    { 0x3c7e3bc5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39e3bc9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xeebe3bcb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x35de3bcd, SUC_lexicon_tags_46 },
    { 0xeafe3bcd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75e3bd4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51e3bda, SUC_lexicon_tags_10 },
    { 0x28be3bda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4de3be1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa95e3be3, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0x73be3be5, SUC_lexicon_tags_2 },
    { 0x619e3be6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25be3bea, SUC_lexicon_tags_37 },
    { 0x8b3e3beb, SUC_lexicon_tags_37 },
    { 0x93fe3bec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519e3bf2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331e3bfa, SUC_lexicon_tags_79 },
    { 0x945e3bfb, SUC_lexicon_tags_52 },
    { 0x6d1e3bfc, SUC_lexicon_tags_17 },
    { 0xfcfe3bfb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x209e3bff, SUC_lexicon_tags_43 },
    { 0x295e3c00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73e3c04, SUC_lexicon_tags_42 },
    { 0xfabe3c05, SUC_lexicon_tags_30 },
    { 0x8d3e3c06, SUC_lexicon_tags_118 },
    { 0xa2be3c04, SUC_lexicon_tags_11 },
    { 0x8d3e3c08, SUC_lexicon_tags_8 },
    { 0x7e9e3c08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x849e3c0c, SUC_lexicon_tags_4 },
    { 0x45fe3c0d, SUC_lexicon_tags_49 },
    { 0x1e3e3c0e, SUC_lexicon_tags_2 },
    { 0xb3be3c0e, SUC_lexicon_tags_25 },
    { 0x3f1e3c0c, SUC_lexicon_tags_17 },
    { 0x5fbe3c10, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6efe3c13, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x837e3c15, SUC_lexicon_tags_34 },
    { 0xc0de3c16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3be3c1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x941e3c1c, SUC_lexicon_tags_1 },
    { 0x591e3c1d, SUC_lexicon_tags_7 },
    { 0xce5e3c1e, SUC_lexicon_tags_112 },
    { 0xf47e3c1f, SUC_lexicon_tags_91 },
    { 0x569e3c1f, SUC_lexicon_tags_25 },
    { 0x623e3c21, SUC_lexicon_tags_46 },
    { 0x20fe3c20, SUC_lexicon_tags_47 },
    { 0x9f3e3c23, SUC_lexicon_tags_146 },
    { 0x4f9e3c23, SUC_lexicon_tags_12 },
    { 0xc23e3c25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5e3c28, SUC_lexicon_tags_23 },
    { 0x4dde3c29, SUC_lexicon_tags_25 },
    { 0x31fe3c2a, SUC_lexicon_tags_8 },
    { 0x61de3c28, SUC_lexicon_tags_31 },
    { 0xbd9e3c2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x293e3c2e, SUC_lexicon_tags_9 },
    { 0xa23e3c2f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbe3c32, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9e3c35, SUC_lexicon_tags_12 },
    { 0xf21e3c36, SUC_lexicon_tags_45 },
    { 0x431e3c36, SUC_lexicon_tags_17 },
    { 0x5ebe3c37, SUC_lexicon_tags_31 },
    { 0x18de3c39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3cfe3c3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913e3c42, SUC_lexicon_tags_1 },
    { 0xf27e3c42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x417e3c46, SUC_lexicon_tags_10 },
    { 0x109e3c47, SUC_lexicon_tags_23 },
    { 0x821e3c47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565e3c4c, SUC_lexicon_tags_37 },
    { 0xd9de3c4d, SUC_lexicon_tags_7 },
    { 0x2e9e3c4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87e3c51, SUC_lexicon_tags_2 },
    { 0x54de3c52, SUC_lexicon_tags_38 },
    { 0x69fe3c52, SUC_lexicon_tags_7 },
    { 0xce9e3c52, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf95e3c56, SUC_lexicon_tags_23 },
    { 0x887e3c57, SUC_lexicon_tags_32 },
    { 0x605e3c58, SUC_lexicon_tags_25 },
    { 0xf87e3c59, SUC_lexicon_tags_26 },
    { 0xddfe3c58, SUC_lexicon_tags_45 },
    { 0xacfe3c59, SUC_lexicon_tags_6 },
    { 0x759e3c5c, SUC_lexicon_tags_0 },
    { 0x903e3c5d, SUC_lexicon_tags_12 },
    { 0xecde3c5e, SUC_lexicon_tags_82 },
    { 0xdc3e3c5f, SUC_lexicon_tags_12 },
    { 0xfe1e3c5f, SUC_lexicon_tags_10 },
    { 0x38fe3c61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee5e3c63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebe3c66, SUC_lexicon_tags_26 },
    { 0x125e3c67, SUC_lexicon_tags_1 },
    { 0x197e3c68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e3c6b, SUC_lexicon_tags_57 },
    { 0xb29e3c6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9cfe3c6e, SUC_lexicon_tags_34 },
    { 0x5f3e3c6f, SUC_lexicon_tags_29 },
    { 0x907e3c6e, SUC_lexicon_tags_2 },
    { 0x213e3c70, SUC_lexicon_tags_1 },
    { 0x591e3c71, SUC_lexicon_tags_1 },
    { 0xd83e3c73, SUC_lexicon_tags_48 },
    { 0xa5de3c74, SUC_lexicon_tags_25 },
    { 0xcabe3c75, SUC_lexicon_tags_2 },
    { 0x3c7e3c76, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3ede3c78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e3c7e, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e3c82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe63e3c84, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde3c8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfe3c93, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133e3c99, SUC_lexicon_tags_11 },
    { 0x1dbe3c9a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xba9e3c9c, SUC_lexicon_tags_49 },
    { 0xcb3e3c9d, SUC_lexicon_tags_37 },
    { 0xdf9e3c9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3e3ca1, SUC_lexicon_tags_37 },
    { 0xb0be3ca2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99be3ca6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fe3cab, SUC_lexicon_tags_25 },
    { 0xb9de3cab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9afe3cae, SUC_lexicon_tags_17 },
    { 0x697e3caf, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x743e3cb1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5e3cb5, SUC_lexicon_tags_37 },
    { 0x385e3cb5, SUC_lexicon_tags_16 },
    { 0x7d7e3cb5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc1be3cb9, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55e3cbc, SUC_lexicon_tags_17 },
    { 0xd8fe3cbd, SUC_lexicon_tags_38 },
    { 0x3dbe3cbd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57e3cc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03e3cca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559e3cce, SUC_lexicon_tags_60 },
    { 0xc81e3ccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x61e3cd1, SUC_lexicon_tags_37 },
    { 0xdffe3cd2, SUC_lexicon_tags_1 },
    { 0x2f7e3cd2, SUC_lexicon_tags_27 },
    { 0x127e3cd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd33e3cd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbe3cdd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x607e3ce0, SUC_lexicon_tags_21 },
    { 0x5dbe3ce0, SUC_lexicon_tags_1 },
    { 0xf31e3ce2, SUC_lexicon_tags_4 },
    { 0x27e3ce3, SUC_lexicon_tags_11 },
    { 0x2abe3ce2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fe3ced, SUC_lexicon_tags_12 },
    { 0x47e3ced, SUC_lexicon_tags_107 },
    { 0x237e3cee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3e3cf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe3cfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35be3cfc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x17de3cfe, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99e3d02, SUC_lexicon_tags_15 },
    { 0x5ebe3d02, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fe3d06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31de3d0a, SUC_lexicon_tags_1 },
    { 0xe8de3d0b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x823e3d0e, SUC_lexicon_tags_47 },
    { 0x225e3d0f, SUC_lexicon_tags_15 },
    { 0x82fe3d0f, SUC_lexicon_tags_49 },
    { 0x6d1e3d11, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881e3d19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67e3d1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe27e3d1e, SUC_lexicon_tags_17 },
    { 0x519e3d1f, SUC_lexicon_tags_21 },
    { 0xfe7e3d20, SUC_lexicon_tags_12 },
    { 0x685e3d20, SUC_lexicon_tags_11 },
    { 0xa57e3d1e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a7e3d24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abe3d29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x463e3d2b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1e3d36, SUC_lexicon_tags_60 },
    { 0x837e3d37, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963e3d3b, SUC_lexicon_tags_42 },
    { 0x505e3d3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5e3d3f, SUC_lexicon_tags_17 },
    { 0x389e3d40, SUC_lexicon_tags_2 },
    { 0x88de3d41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc27e3d43, SUC_lexicon_tags_7 },
    { 0x403e3d44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9e3e3d46, SUC_lexicon_tags_12 },
    { 0x785e3d47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x179e3d49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59be3d4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28be3d56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8a3e3d58, SUC_lexicon_tags_25 },
    { 0xd07e3d58, SUC_lexicon_tags_7 },
    { 0x5bde3d59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x711e3d5c, SUC_lexicon_tags_25 },
    { 0x7a9e3d5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749e3d61, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x583e3d63, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495e3d67, SUC_lexicon_tags_7 },
    { 0xc0be3d68, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e3d6b, SUC_lexicon_tags_25 },
    { 0xa75e3d6c, SUC_lexicon_tags_45 },
    { 0xeb9e3d6d, SUC_lexicon_tags_3 },
    { 0x55be3d6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9de3d70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16de3d72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc17e3d74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x417e3d76, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7e3d79, SUC_lexicon_tags_0 },
    { 0x349e3d79, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x70be3d7d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94fe3d80, SUC_lexicon_tags_18 },
    { 0xa6be3d81, SUC_lexicon_tags_43 },
    { 0xeede3d80, SUC_lexicon_tags_17 },
    { 0x2ade3d83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67e3d87, SUC_lexicon_tags_47 },
    { 0x75be3d87, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89e3d8b, SUC_lexicon_tags_30 },
    { 0x9f3e3d8c, SUC_lexicon_tags_18 },
    { 0x46be3d8d, SUC_lexicon_tags_9 },
    { 0xd35e3d8e, SUC_lexicon_tags_1 },
    { 0x601e3d8f, SUC_lexicon_tags_17 },
    { 0x5fe3d90, SUC_lexicon_tags_8 },
    { 0x94fe3d8f, SUC_lexicon_tags_25 },
    { 0x9e9e3d92, SUC_lexicon_tags_37 },
    { 0x72de3d92, SUC_lexicon_tags_47 },
    { 0x303e3d94, SUC_lexicon_tags_15 },
    { 0xc13e3d95, SUC_lexicon_tags_23 },
    { 0xe4de3d96, SUC_lexicon_tags_27 },
    { 0xfdbe3d95, SUC_lexicon_tags_43 },
    { 0x5f7e3d93, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfe3d9b, SUC_lexicon_tags_33 },
    { 0x40be3d9c, SUC_lexicon_tags_7 },
    { 0xce1e3d9d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c1e3d9f, SUC_lexicon_tags_27 },
    { 0xdd5e3da0, SUC_lexicon_tags_11 },
    { 0xb11e3da1, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1e3da6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbe9e3da8, SUC_lexicon_tags_21 },
    { 0xfe3e3da8, SUC_lexicon_tags_0 },
    { 0xcb5e3da8, SUC_lexicon_tags_54 },
    { 0xbbde3dab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47e3dae, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x163e3db0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fe3db5, SUC_lexicon_tags_26 },
    { 0x369e3db6, SUC_lexicon_tags_23 },
    { 0x3dfe3db7, SUC_lexicon_tags_33 },
    { 0x33e3db7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5e3dbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x325e3dbe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3de3dc4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3e3dcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1e3dd0, SUC_lexicon_tags_26 },
    { 0xa7fe3dd0, SUC_lexicon_tags_37 },
    { 0xb13e3dd2, SUC_lexicon_tags_21 },
    { 0xa71e3dd0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51e3dd9, SUC_lexicon_tags_37 },
    { 0x23de3dda, SUC_lexicon_tags_1 },
    { 0x3fe3dd9, SUC_lexicon_tags_33 },
    { 0xaa3e3ddc, SUC_lexicon_tags_83 },
    { 0x45be3ddd, SUC_lexicon_tags_26 },
    { 0xbc5e3ddc, SUC_lexicon_tags_48 },
    { 0xdfe3ddf, SUC_lexicon_tags_2 },
    { 0x271e3de0, SUC_lexicon_tags_81 },
    { 0xc4de3de1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fe3de5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5e3dea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd7e3dec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169e3df0, SUC_lexicon_tags_48 },
    { 0xc95e3df0, SUC_lexicon_tags_15 },
    { 0x7a3e3df2, SUC_lexicon_tags_26 },
    { 0xddfe3df3, SUC_lexicon_tags_45 },
    { 0x6abe3df2, SUC_lexicon_tags_8 },
    { 0xda9e3df5, SUC_lexicon_tags_17 },
    { 0xd97e3df5, SUC_lexicon_tags_9 },
    { 0xa05e3df5, SUC_lexicon_tags_26 },
    { 0x951e3df8, SUC_lexicon_tags_11 },
    { 0x59e3df8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7e3dfc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7e3e02, SUC_lexicon_tags_1 },
    { 0xbb3e3e02, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59e3e08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd05e3e0a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x573e3e0d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x7b1e3e0f, SUC_lexicon_tags_19 },
    { 0x327e3e10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fe3e12, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x95be3e14, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1e3e18, SUC_lexicon_tags_37 },
    { 0x3d1e3e19, SUC_lexicon_tags_37 },
    { 0x395e3e1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcde3e1f, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xcade3e21, SUC_lexicon_tags_45 },
    { 0xb39e3e22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e3e25, SUC_lexicon_tags_26 },
    { 0x71fe3e26, SUC_lexicon_tags_3 },
    { 0xa15e3e25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbe3e2e, SUC_lexicon_tags_16 },
    { 0xc09e3e2f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfde3e35, SUC_lexicon_tags_17 },
    { 0xeb9e3e36, SUC_lexicon_tags_60 },
    { 0xf31e3e36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc01e3e39, SUC_lexicon_tags_25 },
    { 0x68be3e39, SUC_lexicon_tags_38 },
    { 0xa5de3e3b, SUC_lexicon_tags_21 },
    { 0x2ede3e3b, SUC_lexicon_tags_11 },
    { 0xad3e3e3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efe3e41, SUC_lexicon_tags_9 },
    { 0xb5fe3e42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21e3e46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7e3e48, SUC_lexicon_tags_66 },
    { 0xae5e3e49, SUC_lexicon_tags_65 },
    { 0xebe3e48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29de3e52, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x34de3e54, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcde3e57, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x55de3e59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667e3e63, SUC_lexicon_tags_12 },
    { 0xf43e3e64, SUC_lexicon_tags_15 },
    { 0x96de3e63, SUC_lexicon_tags_14 },
    { 0x9d3e3e64, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5e3e6c, SUC_lexicon_tags_68 },
    { 0x94de3e6c, SUC_lexicon_tags_5 },
    { 0xc11e3e6e, SUC_lexicon_tags_15 },
    { 0xeb7e3e6f, SUC_lexicon_tags_8 },
    { 0xa1e3e6e, SUC_lexicon_tags_15 },
    { 0x799e3e6d, SUC_lexicon_tags_0 },
    { 0x9b3e3e72, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7e3e75, SUC_lexicon_tags_15 },
    { 0x269e3e75, SUC_lexicon_tags_26 },
    { 0x459e3e77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545e3e7f, SUC_lexicon_tags_4 },
    { 0xcb9e3e80, SUC_lexicon_tags_12 },
    { 0x445e3e81, SUC_lexicon_tags_17 },
    { 0x987e3e80, SUC_lexicon_tags_15 },
    { 0x42fe3e80, SUC_lexicon_tags_4 },
    { 0xb8be3e82, SUC_lexicon_tags_27 },
    { 0xb55e3e85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9e3e89, SUC_lexicon_tags_1 },
    { 0xf49e3e89, SUC_lexicon_tags_6 },
    { 0x6e9e3e8b, SUC_lexicon_tags_1 },
    { 0x35de3e8b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbde3e96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67e3e9b, SUC_lexicon_tags_60 },
    { 0x3bbe3e9b, SUC_lexicon_tags_11 },
    { 0xa8be3e9d, SUC_lexicon_tags_1 },
    { 0xa17e3e9e, SUC_lexicon_tags_43 },
    { 0x59e3e9f, SUC_lexicon_tags_49 },
    { 0x2c5e3ea0, SUC_lexicon_tags_39 },
    { 0xd1be3e9e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebe3ea9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e3eb2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd6de3eb4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495e3eb8, SUC_lexicon_tags_19 },
    { 0x447e3eb9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x289e3ebb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x409e3ebd, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4de3ec2, SUC_lexicon_tags_17 },
    { 0xeb5e3ec3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1e3ecb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3e3ece, SUC_lexicon_tags_12 },
    { 0xcefe3ecf, SUC_lexicon_tags_25 },
    { 0x839e3ecf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fe3ed4, SUC_lexicon_tags_32 },
    { 0xf63e3ed4, SUC_lexicon_tags_8 },
    { 0xb9fe3ed6, SUC_lexicon_tags_26 },
    { 0xe63e3ed6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb0de3ed9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23e3edc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505e3ee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x973e3ee3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb47e3ee5, SUC_lexicon_tags_38 },
    { 0x50de3ee6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4e5e3ee8, SUC_lexicon_tags_61 },
    { 0xa8be3ee9, SUC_lexicon_tags_60 },
    { 0x84de3eea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a9e3eec, SUC_lexicon_tags_29 },
    { 0x6c5e3eed, SUC_lexicon_tags_9 },
    { 0x551e3eed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467e3ef3, SUC_lexicon_tags_11 },
    { 0x51de3ef3, SUC_lexicon_tags_17 },
    { 0x8cfe3ef4, SUC_lexicon_tags_4 },
    { 0xe1be3ef6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x453e3ef8, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7e3efc, SUC_lexicon_tags_56 },
    { 0x487e3efd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fe3f02, SUC_lexicon_tags_10 },
    { 0xdd5e3f02, SUC_lexicon_tags_9 },
    { 0x809e3f04, SUC_lexicon_tags_17 },
    { 0x913e3f05, SUC_lexicon_tags_7 },
    { 0x745e3f06, SUC_lexicon_tags_2 },
    { 0x69e3f07, SUC_lexicon_tags_45 },
    { 0xaffe3f07, SUC_lexicon_tags_182 },
    { 0xd91e3f09, SUC_lexicon_tags_1 },
    { 0x50be3f0a, SUC_lexicon_tags_9 },
    { 0x66fe3f0b, SUC_lexicon_tags_11 },
    { 0xaf3e3f0a, SUC_lexicon_tags_39 },
    { 0x991e3f0d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4cbe3f0f, SUC_lexicon_tags_15 },
    { 0x641e3f10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcde3f14, SUC_lexicon_tags_1 },
    { 0x7a3e3f15, SUC_lexicon_tags_14 },
    { 0x719e3f15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x195e3f19, SUC_lexicon_tags_6 },
    { 0xfe1e3f1a, SUC_lexicon_tags_8 },
    { 0x5d3e3f1b, SUC_lexicon_tags_56 },
    { 0xf0be3f1c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2d7e3f1e, SUC_lexicon_tags_49 },
    { 0x211e3f1f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xda9e3f21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc99e3f23, SUC_lexicon_tags_14 },
    { 0xbfbe3f23, SUC_lexicon_tags_25 },
    { 0xdb9e3f25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x449e3f27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3e3f2a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571e3f30, SUC_lexicon_tags_27 },
    { 0x981e3f31, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33be3f35, SUC_lexicon_tags_46 },
    { 0xfb5e3f36, SUC_lexicon_tags_1 },
    { 0x991e3f37, SUC_lexicon_tags_1 },
    { 0x3bde3f37, SUC_lexicon_tags_1 },
    { 0x22de3f38, SUC_lexicon_tags_11 },
    { 0x565e3f35, SUC_lexicon_tags_35 },
    { 0x935e3f38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fe3f43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x93fe3f45, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7e3f4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5e3f4d, SUC_lexicon_tags_63 },
    { 0x89be3f4e, SUC_lexicon_tags_17 },
    { 0xad7e3f4f, SUC_lexicon_tags_25 },
    { 0x269e3f50, SUC_lexicon_tags_17 },
    { 0x4a9e3f4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63fe3f53, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e3f56, SUC_lexicon_tags_19 },
    { 0xb33e3f57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x29be3f5a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x941e3f5c, SUC_lexicon_tags_25 },
    { 0xabfe3f5c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xda3e3f5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fe3f64, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdd3e3f66, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1e3f6d, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121e3f75, SUC_lexicon_tags_2 },
    { 0x6e1e3f76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785e3f7a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xb71e3f7c, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0xbde3f7e, SUC_lexicon_tags_7 },
    { 0x67be3f7e, SUC_lexicon_tags_11 },
    { 0x825e3f80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebe3f88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99e3f8a, SUC_lexicon_tags_11 },
    { 0x643e3f8b, SUC_lexicon_tags_29 },
    { 0xb47e3f8c, SUC_lexicon_tags_1 },
    { 0x215e3f8c, SUC_lexicon_tags_29 },
    { 0xf1be3f8e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3e3f94, SUC_lexicon_tags_7 },
    { 0xcf9e3f95, SUC_lexicon_tags_29 },
    { 0xcc1e3f95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8e1e3f98, SUC_lexicon_tags_25 },
    { 0x78fe3f98, SUC_lexicon_tags_37 },
    { 0x4a9e3f99, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xadfe3f9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb55e3f9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9de3fa3, SUC_lexicon_tags_68 },
    { 0x2abe3fa4, SUC_lexicon_tags_16 },
    { 0xc61e3fa4, SUC_lexicon_tags_30 },
    { 0x80be3fa6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9e3fac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad9e3fae, SUC_lexicon_tags_45 },
    { 0xff5e3fae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689e3fb6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfe3fbb, SUC_lexicon_tags_43 },
    { 0x909e3fbc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37e3fbf, SUC_lexicon_tags_55 },
    { 0x287e3fc0, SUC_lexicon_tags_15 },
    { 0xe49e3fc0, SUC_lexicon_tags_1 },
    { 0xccde3fc2, SUC_lexicon_tags_0 },
    { 0x87fe3fc3, SUC_lexicon_tags_2 },
    { 0x9de3fc4, SUC_lexicon_tags_17 },
    { 0x8a3e3fc5, SUC_lexicon_tags_8 },
    { 0x441e3fc4, SUC_lexicon_tags_7 },
    { 0x8ade3fc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727e3fcc, SUC_lexicon_tags_62 },
    { 0x1e3fcd, SUC_lexicon_tags_65 },
    { 0x55be3fce, SUC_lexicon_tags_0 },
    { 0x1cde3fcf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fe3fd3, SUC_lexicon_tags_31 },
    { 0x64de3fd4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1e3fda, SUC_lexicon_tags_17 },
    { 0x1dbe3fda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd21e3fdd, SUC_lexicon_tags_6 },
    { 0x45fe3fde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xae1e3fe0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15be3fe6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xf29e3fe9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17e3ff2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9e3ff6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7a3e3ff8, SUC_lexicon_tags_42 },
    { 0x40de3ff9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbd5e3ffb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x467e3ffe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373e4009, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x73fe400b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf0de400d, SUC_lexicon_tags_6 },
    { 0x977e400e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79e4012, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01e4019, SUC_lexicon_tags_10 },
    { 0xc7be4019, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x609e401c, SUC_lexicon_tags_2 },
    { 0xb75e401d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397e4021, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd7de4023, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585e402b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x5abe402d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x819e4030, SUC_lexicon_tags_7 },
    { 0x429e4031, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xca1e4033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x447e4035, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x711e4037, SUC_lexicon_tags_37 },
    { 0xba5e4038, SUC_lexicon_tags_10 },
    { 0x7c3e4039, SUC_lexicon_tags_35 },
    { 0xbe1e4038, SUC_lexicon_tags_27 },
    { 0x4afe403b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3e4041, SUC_lexicon_tags_25 },
    { 0x203e4042, SUC_lexicon_tags_11 },
    { 0xa71e4043, SUC_lexicon_tags_17 },
    { 0x63fe4044, SUC_lexicon_tags_17 },
    { 0x849e4045, SUC_lexicon_tags_4 },
    { 0x321e4044, SUC_lexicon_tags_8 },
    { 0x819e4044, SUC_lexicon_tags_37 },
    { 0xc97e4048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bde404a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fe4050, SUC_lexicon_tags_2 },
    { 0xde4051, SUC_lexicon_tags_2 },
    { 0x94fe4050, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x977e4054, SUC_lexicon_tags_15 },
    { 0xbde4055, SUC_lexicon_tags_17 },
    { 0x641e4055, SUC_lexicon_tags_34 },
    { 0x81fe4054, SUC_lexicon_tags_23 },
    { 0x1bde4057, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6d7e405a, SUC_lexicon_tags_15 },
    { 0xddbe405a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbe405e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e4066, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x545e4069, SUC_lexicon_tags_79 },
    { 0x9e1e406a, SUC_lexicon_tags_33 },
    { 0x8e3e406b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bde406f, SUC_lexicon_tags_15 },
    { 0xb7de4070, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5e4075, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169e407a, SUC_lexicon_tags_55 },
    { 0x7de407a, SUC_lexicon_tags_92 },
    { 0x589e407c, SUC_lexicon_tags_7 },
    { 0xcc7e407d, SUC_lexicon_tags_2 },
    { 0xe85e407c, SUC_lexicon_tags_13 },
    { 0xc7be407f, SUC_lexicon_tags_15 },
    { 0x5c9e407a, SUC_lexicon_tags_9 },
    { 0x6abe4081, SUC_lexicon_tags_0 },
    { 0xea9e4082, SUC_lexicon_tags_22 },
    { 0x1f5e4083, SUC_lexicon_tags_238 },
    { 0xe47e4083, SUC_lexicon_tags_43 },
    { 0x2a1e4085, SUC_lexicon_tags_37 },
    { 0xb5de4081, SUC_lexicon_tags_112 },
    { 0x669e4086, SUC_lexicon_tags_27 },
    { 0x143e4088, SUC_lexicon_tags_12 },
    { 0xb85e4089, SUC_lexicon_tags_60 },
    { 0x1e3e408a, SUC_lexicon_tags_43 },
    { 0x119e408b, SUC_lexicon_tags_31 },
    { 0x21fe408c, SUC_lexicon_tags_15 },
    { 0xbf9e408d, SUC_lexicon_tags_26 },
    { 0xb3de4085, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbabe4090, SUC_lexicon_tags_19 },
    { 0xdfbe4091, SUC_lexicon_tags_8 },
    { 0xf91e4091, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abe4097, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589e409c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5e40a1, SUC_lexicon_tags_25 },
    { 0x3dfe40a2, SUC_lexicon_tags_25 },
    { 0x2cbe40a1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x781e40a5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fe40aa, SUC_lexicon_tags_10 },
    { 0xbabe40ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dde40ad, SUC_lexicon_tags_67 },
    { 0x8f9e40ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7e40b1, SUC_lexicon_tags_30 },
    { 0x525e40b2, SUC_lexicon_tags_43 },
    { 0xfd5e40b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb0fe40b5, SUC_lexicon_tags_38 },
    { 0x38be40b6, SUC_lexicon_tags_17 },
    { 0x28de40b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3e40bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcafe40c1, SUC_lexicon_tags_1 },
    { 0xd21e40c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953e40c6, SUC_lexicon_tags_25 },
    { 0xe13e40c7, SUC_lexicon_tags_48 },
    { 0xd8de40c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75de40cc, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9be40d2, SUC_lexicon_tags_12 },
    { 0xb5de40d3, SUC_lexicon_tags_6 },
    { 0x1a7e40d4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb79e40d6, SUC_lexicon_tags_28 },
    { 0x4afe40d7, SUC_lexicon_tags_43 },
    { 0xe99e40d6, SUC_lexicon_tags_38 },
    { 0xf51e40d9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345e40df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1e40e4, SUC_lexicon_tags_17 },
    { 0xfebe40e4, SUC_lexicon_tags_43 },
    { 0x2f1e40e4, SUC_lexicon_tags_7 },
    { 0xd21e40e7, SUC_lexicon_tags_10 },
    { 0x1e1e40e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x579e40eb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x497e40ee, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x353e40f1, SUC_lexicon_tags_6 },
    { 0x5f1e40f1, SUC_lexicon_tags_19 },
    { 0x147e40f2, SUC_lexicon_tags_11 },
    { 0xfd9e40f4, SUC_lexicon_tags_11 },
    { 0xa5fe40f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe89e40f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e40fa, SUC_lexicon_tags_9 },
    { 0xc2be40fa, SUC_lexicon_tags_37 },
    { 0xfbe40fc, SUC_lexicon_tags_0 },
    { 0xf4de40fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67e4103, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd81e4105, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xec3e4107, SUC_lexicon_tags_11 },
    { 0x635e4108, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf57e410a, SUC_lexicon_tags_63 },
    { 0x233e410b, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33e4110, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbe5e4112, SUC_lexicon_tags_2 },
    { 0xba5e4113, SUC_lexicon_tags_15 },
    { 0xb4fe4114, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19e4119, SUC_lexicon_tags_27 },
    { 0x46be411a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1afe4127, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5e4130, SUC_lexicon_tags_45 },
    { 0x5abe4130, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe0be4133, SUC_lexicon_tags_27 },
    { 0xcfbe4133, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf57e4136, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7e413b, SUC_lexicon_tags_17 },
    { 0x607e413c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x58de413f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1e4142, SUC_lexicon_tags_13 },
    { 0x4ebe4143, SUC_lexicon_tags_47 },
    { 0x101e4144, SUC_lexicon_tags_11 },
    { 0x1fde4145, SUC_lexicon_tags_12 },
    { 0x3cbe4144, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66de414b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357e414f, SUC_lexicon_tags_49 },
    { 0xcd5e4150, SUC_lexicon_tags_30 },
    { 0xefe414f, SUC_lexicon_tags_12 },
    { 0xcefe4150, SUC_lexicon_tags_30 },
    { 0x643e4151, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8de4159, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1e415f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879e4166, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x49e4169, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8a7e416b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x577e416e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509e4175, SUC_lexicon_tags_25 },
    { 0x949e4176, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5e417b, SUC_lexicon_tags_84 },
    { 0x457e417c, SUC_lexicon_tags_66 },
    { 0x6ade417c, SUC_lexicon_tags_1 },
    { 0x1a9e417e, SUC_lexicon_tags_47 },
    { 0xbde417c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabe4183, SUC_lexicon_tags_0 },
    { 0x8dfe4183, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x823e4187, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805e418c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x963e418f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9e4192, SUC_lexicon_tags_46 },
    { 0x969e4192, SUC_lexicon_tags_17 },
    { 0xa39e4193, SUC_lexicon_tags_11 },
    { 0x971e4195, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x769e4198, SUC_lexicon_tags_17 },
    { 0x9e4199, SUC_lexicon_tags_12 },
    { 0x1be419a, SUC_lexicon_tags_49 },
    { 0xf51e419b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x151e419d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0de41a0, SUC_lexicon_tags_37 },
    { 0x813e41a1, SUC_lexicon_tags_37 },
    { 0xf19e41a2, SUC_lexicon_tags_11 },
    { 0xbede41a2, SUC_lexicon_tags_42 },
    { 0x4f9e41a4, SUC_lexicon_tags_1 },
    { 0xf99e41a3, SUC_lexicon_tags_9 },
    { 0x13fe41a3, SUC_lexicon_tags_206 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231e41ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92de41ae, SUC_lexicon_tags_27 },
    { 0x2afe41af, SUC_lexicon_tags_2 },
    { 0x1afe41af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa23e41b2, SUC_lexicon_tags_25 },
    { 0x87be41b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fe41b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e41bb, SUC_lexicon_tags_17 },
    { 0x831e41bc, SUC_lexicon_tags_37 },
    { 0x307e41bd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3de41c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x2fde41c2, SUC_lexicon_tags_30 },
    { 0x805e41c2, SUC_lexicon_tags_37 },
    { 0xc9be41c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fde41c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643e41cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dde41cf, SUC_lexicon_tags_38 },
    { 0x68fe41d0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x893e41d3, SUC_lexicon_tags_7 },
    { 0x303e41d3, SUC_lexicon_tags_38 },
    { 0x9b1e41d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x353e41d7, SUC_lexicon_tags_33 },
    { 0x67be41d8, SUC_lexicon_tags_30 },
    { 0x2dbe41d9, SUC_lexicon_tags_25 },
    { 0x147e41da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60de41dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x90fe41e0, SUC_lexicon_tags_43 },
    { 0x30fe41e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7de41ee, SUC_lexicon_tags_234 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e41f2, SUC_lexicon_tags_0 },
    { 0x24be41f3, SUC_lexicon_tags_63 },
    { 0x745e41f4, SUC_lexicon_tags_12 },
    { 0x51e41f2, SUC_lexicon_tags_8 },
    { 0x215e41f3, SUC_lexicon_tags_45 },
    { 0xf3e41f4, SUC_lexicon_tags_17 },
    { 0xe1be41f5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fe4202, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbe420a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7e420e, SUC_lexicon_tags_1 },
    { 0xa07e420f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313e4217, SUC_lexicon_tags_0 },
    { 0x35be4217, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x27be421a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x20fe421c, SUC_lexicon_tags_6 },
    { 0xf19e421d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fe4221, SUC_lexicon_tags_12 },
    { 0xf13e4222, SUC_lexicon_tags_14 },
    { 0x473e4223, SUC_lexicon_tags_17 },
    { 0xd83e4222, SUC_lexicon_tags_25 },
    { 0xd09e4225, SUC_lexicon_tags_14 },
    { 0xd15e4226, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5e422a, SUC_lexicon_tags_23 },
    { 0x981e422a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fe4230, SUC_lexicon_tags_30 },
    { 0x311e4231, SUC_lexicon_tags_37 },
    { 0xf7e4232, SUC_lexicon_tags_6 },
    { 0x801e4233, SUC_lexicon_tags_2 },
    { 0xeede4230, SUC_lexicon_tags_11 },
    { 0x26fe4235, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fe4238, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9e423f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6c5e4241, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x517e4243, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199e424b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7e424e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x53fe4250, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03e4253, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd27e4255, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x103e4257, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597e425f, SUC_lexicon_tags_25 },
    { 0x313e4260, SUC_lexicon_tags_87 },
    { 0x709e4261, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43e4264, SUC_lexicon_tags_25 },
    { 0xfc3e4265, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecde426a, SUC_lexicon_tags_57 },
    { 0xdf5e426b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4be4273, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec1e4275, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb49e4277, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3e427a, SUC_lexicon_tags_45 },
    { 0x7bbe427b, SUC_lexicon_tags_7 },
    { 0xdede427b, SUC_lexicon_tags_8 },
    { 0xde7e427b, SUC_lexicon_tags_63 },
    { 0xb59e427e, SUC_lexicon_tags_1 },
    { 0xc8be427a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb39e4281, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5e4287, SUC_lexicon_tags_25 },
    { 0xd51e4287, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x38be428b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7e4291, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdde4297, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x15e4299, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ade42a2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x911e42a5, SUC_lexicon_tags_12 },
    { 0xa3e42a5, SUC_lexicon_tags_16 },
    { 0x137e42a7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x68de42a9, SUC_lexicon_tags_47 },
    { 0xb95e42a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fe42b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc31e42b4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x81e42b6, SUC_lexicon_tags_39 },
    { 0x3c1e42b7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe49e42b9, SUC_lexicon_tags_37 },
    { 0x56fe42b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xca5e42bd, SUC_lexicon_tags_63 },
    { 0xa15e42be, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb61e42c0, SUC_lexicon_tags_46 },
    { 0x10be42c1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7e42c9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15e42cc, SUC_lexicon_tags_23 },
    { 0x509e42cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ade42d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdf9e42d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe3e42d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x411e42d6, SUC_lexicon_tags_12 },
    { 0x3cbe42d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x805e42d9, SUC_lexicon_tags_25 },
    { 0xc1e42da, SUC_lexicon_tags_7 },
    { 0xf83e42db, SUC_lexicon_tags_1 },
    { 0xcf7e42dc, SUC_lexicon_tags_17 },
    { 0x349e42dd, SUC_lexicon_tags_118 },
    { 0x2b7e42de, SUC_lexicon_tags_17 },
    { 0x5a7e42df, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3de42e2, SUC_lexicon_tags_17 },
    { 0x84fe42e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x71e42e5, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fe42ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59de42ef, SUC_lexicon_tags_27 },
    { 0xbede42ef, SUC_lexicon_tags_23 },
    { 0x405e42f1, SUC_lexicon_tags_43 },
    { 0x729e42f2, SUC_lexicon_tags_87 },
    { 0xcf5e42f3, SUC_lexicon_tags_10 },
    { 0xefbe42f4, SUC_lexicon_tags_12 },
    { 0x417e42f5, SUC_lexicon_tags_6 },
    { 0x29de42f6, SUC_lexicon_tags_23 },
    { 0x21fe42f7, SUC_lexicon_tags_84 },
    { 0x19be42f7, SUC_lexicon_tags_14 },
    { 0xd0de42f1, SUC_lexicon_tags_12 },
    { 0xafde42f2, SUC_lexicon_tags_1 },
    { 0x365e42f1, SUC_lexicon_tags_15 },
    { 0x1c7e42fb, SUC_lexicon_tags_92 },
    { 0x6f9e42fd, SUC_lexicon_tags_19 },
    { 0x92fe42fb, SUC_lexicon_tags_37 },
    { 0xb27e42f1, SUC_lexicon_tags_31 },
    { 0xe49e42f3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19e4303, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739e4307, SUC_lexicon_tags_301 },
    { 0xec7e4308, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbe430b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedde4310, SUC_lexicon_tags_37 },
    { 0xcd9e4311, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861e4315, SUC_lexicon_tags_48 },
    { 0xc11e4316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7e4319, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbe431c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7e4326, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe29e4328, SUC_lexicon_tags_87 },
    { 0x18be4328, SUC_lexicon_tags_25 },
    { 0x303e432a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9e432d, SUC_lexicon_tags_7 },
    { 0x4a1e432e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7e4332, SUC_lexicon_tags_11 },
    { 0xa5de4333, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fde4336, SUC_lexicon_tags_39 },
    { 0x3afe4336, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf0fe4339, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891e4345, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbe4352, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fe435d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x379e435f, SUC_lexicon_tags_91 },
    { 0x845e4360, SUC_lexicon_tags_72 },
    { 0x20de435f, SUC_lexicon_tags_2 },
    { 0x18de4362, SUC_lexicon_tags_7 },
    { 0x201e4363, SUC_lexicon_tags_12 },
    { 0x195e435f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73be4366, SUC_lexicon_tags_11 },
    { 0xe3de4367, SUC_lexicon_tags_63 },
    { 0xd21e4368, SUC_lexicon_tags_0 },
    { 0x5f7e4367, SUC_lexicon_tags_32 },
    { 0x1cfe436a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2de4371, SUC_lexicon_tags_46 },
    { 0x6ade4371, SUC_lexicon_tags_7 },
    { 0x64de4373, SUC_lexicon_tags_56 },
    { 0xa0fe4373, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7f1e4376, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fe437a, SUC_lexicon_tags_7 },
    { 0xc59e437b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4dde437d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31e4380, SUC_lexicon_tags_25 },
    { 0x84de4381, SUC_lexicon_tags_43 },
    { 0xd05e4381, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0xe7de4384, SUC_lexicon_tags_49 },
    { 0xa4fe4385, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x13e4388, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01e438c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3f3e438e, SUC_lexicon_tags_37 },
    { 0xd29e438f, SUC_lexicon_tags_25 },
    { 0x8c7e438e, SUC_lexicon_tags_12 },
    { 0x83fe4391, SUC_lexicon_tags_11 },
    { 0x58be438e, SUC_lexicon_tags_19 },
    { 0x6be438e, SUC_lexicon_tags_37 },
    { 0xca9e4394, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x397e4396, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81de439a, SUC_lexicon_tags_7 },
    { 0xf55e439b, SUC_lexicon_tags_6 },
    { 0x133e439a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fe43a1, SUC_lexicon_tags_11 },
    { 0xe61e43a2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9e43a9, SUC_lexicon_tags_551 },
    { 0, NULL },
    { 0x93be43ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x985e43ad, SUC_lexicon_tags_6 },
    { 0x163e43ae, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfe43b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381e43bc, SUC_lexicon_tags_25 },
    { 0xcf9e43bd, SUC_lexicon_tags_48 },
    { 0xadbe43bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72de43c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x857e43cb, SUC_lexicon_tags_11 },
    { 0xf35e43cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcade43df, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x605e43e1, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x43e43e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fe43e9, SUC_lexicon_tags_11 },
    { 0x2cfe43e9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecfe43f1, SUC_lexicon_tags_37 },
    { 0x7bbe43f1, SUC_lexicon_tags_47 },
    { 0xd31e43f3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5e43f7, SUC_lexicon_tags_34 },
    { 0x3a9e43f7, SUC_lexicon_tags_1 },
    { 0x4fe43f9, SUC_lexicon_tags_60 },
    { 0x467e43fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fe43fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4fe4400, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf4be4402, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955e4407, SUC_lexicon_tags_37 },
    { 0x5bbe4408, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x22fe440a, SUC_lexicon_tags_7 },
    { 0x6ede440b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x525e440d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46fe440f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6fe4411, SUC_lexicon_tags_11 },
    { 0x379e4411, SUC_lexicon_tags_81 },
    { 0x587e4411, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf99e4415, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5efe4417, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xa5be4419, SUC_lexicon_tags_9 },
    { 0xedbe441a, SUC_lexicon_tags_14 },
    { 0xb91e441a, SUC_lexicon_tags_26 },
    { 0x8dbe441a, SUC_lexicon_tags_32 },
    { 0x3ade441d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99e4420, SUC_lexicon_tags_37 },
    { 0xb33e4421, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x61be4423, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fe4427, SUC_lexicon_tags_45 },
    { 0xdafe4428, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e442c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46de4437, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be443b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e443e, SUC_lexicon_tags_15 },
    { 0xd5de443f, SUC_lexicon_tags_26 },
    { 0x57e443f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x885e4442, SUC_lexicon_tags_87 },
    { 0xfc7e4443, SUC_lexicon_tags_11 },
    { 0x469e4444, SUC_lexicon_tags_37 },
    { 0xc35e4443, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f3e4447, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x90fe4449, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9e444c, SUC_lexicon_tags_25 },
    { 0x165e444d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5abe444f, SUC_lexicon_tags_82 },
    { 0x335e444f, SUC_lexicon_tags_31 },
    { 0x595e444f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1fe4453, SUC_lexicon_tags_62 },
    { 0xaffe4454, SUC_lexicon_tags_17 },
    { 0xda7e4455, SUC_lexicon_tags_1 },
    { 0x9b5e4455, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ade445b, SUC_lexicon_tags_25 },
    { 0x82de445b, SUC_lexicon_tags_17 },
    { 0x431e445d, SUC_lexicon_tags_11 },
    { 0xb59e445c, SUC_lexicon_tags_11 },
    { 0x181e445f, SUC_lexicon_tags_15 },
    { 0x24fe445d, SUC_lexicon_tags_13 },
    { 0x26de4461, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9e4464, SUC_lexicon_tags_26 },
    { 0x335e4465, SUC_lexicon_tags_31 },
    { 0x3d1e4466, SUC_lexicon_tags_11 },
    { 0x471e4467, SUC_lexicon_tags_12 },
    { 0x4dfe4467, SUC_lexicon_tags_14 },
    { 0xb7fe4469, SUC_lexicon_tags_2 },
    { 0x95de4467, SUC_lexicon_tags_37 },
    { 0x3d5e446b, SUC_lexicon_tags_31 },
    { 0xefbe4467, SUC_lexicon_tags_15 },
    { 0xc65e4464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603e4474, SUC_lexicon_tags_38 },
    { 0x4be4474, SUC_lexicon_tags_79 },
    { 0xf5de4476, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1e4479, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2efe447b, SUC_lexicon_tags_2 },
    { 0x50de447c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91e4482, SUC_lexicon_tags_17 },
    { 0x323e4482, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4de4485, SUC_lexicon_tags_14 },
    { 0x3d5e4486, SUC_lexicon_tags_32 },
    { 0x21fe4487, SUC_lexicon_tags_25 },
    { 0xb3de4488, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1de448c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb09e448e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x591e4491, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4abe4494, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403e449d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbe44a0, SUC_lexicon_tags_35 },
    { 0xdf1e44a1, SUC_lexicon_tags_38 },
    { 0xf19e44a1, SUC_lexicon_tags_46 },
    { 0xf1fe44a3, SUC_lexicon_tags_30 },
    { 0xab1e44a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7e44a8, SUC_lexicon_tags_37 },
    { 0x7e5e44a9, SUC_lexicon_tags_11 },
    { 0xbdbe44a8, SUC_lexicon_tags_0 },
    { 0xa7de44ab, SUC_lexicon_tags_12 },
    { 0x777e44ac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa83e44ae, SUC_lexicon_tags_1 },
    { 0x9d9e44ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdf3e44b1, SUC_lexicon_tags_15 },
    { 0x331e44b2, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63e44b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161e44c8, SUC_lexicon_tags_7 },
    { 0x2fe44c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271e44ce, SUC_lexicon_tags_2 },
    { 0x1fe44ce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74be44d1, SUC_lexicon_tags_31 },
    { 0xb2be44d2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x29be44d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91e44da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x737e44dc, SUC_lexicon_tags_37 },
    { 0xce3e44dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd57e44df, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58be44e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8ffe44e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf3e44e7, SUC_lexicon_tags_32 },
    { 0x337e44e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19e44eb, SUC_lexicon_tags_47 },
    { 0xee3e44ec, SUC_lexicon_tags_43 },
    { 0xa19e44ed, SUC_lexicon_tags_37 },
    { 0xbc7e44ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17be44f7, SUC_lexicon_tags_56 },
    { 0x763e44f8, SUC_lexicon_tags_37 },
    { 0xf5de44f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3e44fc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe93e44fe, SUC_lexicon_tags_2 },
    { 0x2a9e44ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55e4504, SUC_lexicon_tags_32 },
    { 0x48be4504, SUC_lexicon_tags_2 },
    { 0xf05e4504, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0de450c, SUC_lexicon_tags_11 },
    { 0x5b7e450d, SUC_lexicon_tags_1 },
    { 0x56de450c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x717e4511, SUC_lexicon_tags_17 },
    { 0x6fde4512, SUC_lexicon_tags_68 },
    { 0x155e4513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc15e4515, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afe4519, SUC_lexicon_tags_16 },
    { 0x807e451a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73e4521, SUC_lexicon_tags_23 },
    { 0x451e4522, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1e4530, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9e4535, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a3e4537, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x541e4539, SUC_lexicon_tags_2 },
    { 0xf49e453a, SUC_lexicon_tags_11 },
    { 0xfa7e4539, SUC_lexicon_tags_65 },
    { 0x93be453c, SUC_lexicon_tags_27 },
    { 0xfc3e453d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x373e4540, SUC_lexicon_tags_26 },
    { 0xb5be4540, SUC_lexicon_tags_1 },
    { 0x967e4541, SUC_lexicon_tags_37 },
    { 0x43e4543, SUC_lexicon_tags_42 },
    { 0xb29e4544, SUC_lexicon_tags_12 },
    { 0xb1be4545, SUC_lexicon_tags_11 },
    { 0x6efe4544, SUC_lexicon_tags_14 },
    { 0xa57e4544, SUC_lexicon_tags_37 },
    { 0xaafe4548, SUC_lexicon_tags_0 },
    { 0x101e4548, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xffde454c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3de454e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fe4551, SUC_lexicon_tags_7 },
    { 0x2a9e4552, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc15e4554, SUC_lexicon_tags_4 },
    { 0xa21e4555, SUC_lexicon_tags_15 },
    { 0xe1e4556, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2ebe4558, SUC_lexicon_tags_7 },
    { 0xee5e4559, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc7e455b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbe455e, SUC_lexicon_tags_7 },
    { 0x9c3e455f, SUC_lexicon_tags_26 },
    { 0x84fe4560, SUC_lexicon_tags_112 },
    { 0xd1be4560, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1e4564, SUC_lexicon_tags_37 },
    { 0xf05e4564, SUC_lexicon_tags_56 },
    { 0x58de4566, SUC_lexicon_tags_10 },
    { 0xfb5e4566, SUC_lexicon_tags_7 },
    { 0x989e4568, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1de456b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x491e456d, SUC_lexicon_tags_55 },
    { 0x193e456e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x6e1e4570, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34fe4572, SUC_lexicon_tags_10 },
    { 0x1b7e4573, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845e457a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xbade457c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551e4580, SUC_lexicon_tags_16 },
    { 0xe8be4581, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x887e4583, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fe4586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x709e4588, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfe458c, SUC_lexicon_tags_14 },
    { 0xdc9e458d, SUC_lexicon_tags_8 },
    { 0xc8be458d, SUC_lexicon_tags_17 },
    { 0xdd7e458e, SUC_lexicon_tags_17 },
    { 0x207e4590, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47e4595, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1de4599, SUC_lexicon_tags_17 },
    { 0x97fe459a, SUC_lexicon_tags_49 },
    { 0x307e4599, SUC_lexicon_tags_4 },
    { 0x481e459c, SUC_lexicon_tags_23 },
    { 0x589e459b, SUC_lexicon_tags_6 },
    { 0xfdde459d, SUC_lexicon_tags_33 },
    { 0xeb1e459a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5dbe45a1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x743e45a4, SUC_lexicon_tags_35 },
    { 0x507e45a5, SUC_lexicon_tags_38 },
    { 0x6b9e45a5, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873e45ad, SUC_lexicon_tags_43 },
    { 0x9efe45ad, SUC_lexicon_tags_92 },
    { 0x3fde45ae, SUC_lexicon_tags_0 },
    { 0x933e45b0, SUC_lexicon_tags_14 },
    { 0x66be45b0, SUC_lexicon_tags_1 },
    { 0xe15e45b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7e45b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9e45bb, SUC_lexicon_tags_17 },
    { 0xabe45bc, SUC_lexicon_tags_38 },
    { 0x2d7e45bc, SUC_lexicon_tags_23 },
    { 0xfe9e45bd, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217e45c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97e45ca, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf47e45cc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81e45d4, SUC_lexicon_tags_11 },
    { 0x11e45d4, SUC_lexicon_tags_11 },
    { 0xf39e45d5, SUC_lexicon_tags_7 },
    { 0xaafe45d7, SUC_lexicon_tags_2 },
    { 0x3f7e45d8, SUC_lexicon_tags_32 },
    { 0x875e45d9, SUC_lexicon_tags_37 },
    { 0xa65e45d8, SUC_lexicon_tags_47 },
    { 0x3fe45db, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x3d1e45dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x615e45df, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x17fe45e1, SUC_lexicon_tags_63 },
    { 0xe71e45e1, SUC_lexicon_tags_5 },
    { 0x325e45e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x68fe45e5, SUC_lexicon_tags_41 },
    { 0x553e45e6, SUC_lexicon_tags_0 },
    { 0x50de45e5, SUC_lexicon_tags_64 },
    { 0xf5e45e8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x835e45eb, SUC_lexicon_tags_5 },
    { 0x969e45ec, SUC_lexicon_tags_32 },
    { 0xd85e45ed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fe45f0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb7fe45f2, SUC_lexicon_tags_65 },
    { 0xeade45f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fe45fa, SUC_lexicon_tags_17 },
    { 0x707e45fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953e45ff, SUC_lexicon_tags_27 },
    { 0x501e4600, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835e4604, SUC_lexicon_tags_47 },
    { 0x4c1e4605, SUC_lexicon_tags_30 },
    { 0x577e4606, SUC_lexicon_tags_14 },
    { 0xf47e4605, SUC_lexicon_tags_37 },
    { 0xb0be4606, SUC_lexicon_tags_55 },
    { 0xe55e4604, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabe460f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcde4613, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9e4618, SUC_lexicon_tags_37 },
    { 0x2fbe4619, SUC_lexicon_tags_37 },
    { 0xf43e461a, SUC_lexicon_tags_34 },
    { 0xf13e4618, SUC_lexicon_tags_2 },
    { 0xc4fe4618, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe4be461e, SUC_lexicon_tags_1 },
    { 0xd1de461f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfafe4621, SUC_lexicon_tags_231 },
    { 0xbcde4621, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x229e4624, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49e4628, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x25e462b, SUC_lexicon_tags_341 },
    { 0x8c3e462c, SUC_lexicon_tags_2 },
    { 0xf0be462d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54fe4633, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcdbe4635, SUC_lexicon_tags_79 },
    { 0xf0de4635, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcf3e4638, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93be463a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6d1e463c, SUC_lexicon_tags_65 },
    { 0x9b7e463d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbe4640, SUC_lexicon_tags_25 },
    { 0xc3be4641, SUC_lexicon_tags_12 },
    { 0x60fe4641, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x385e4644, SUC_lexicon_tags_37 },
    { 0x8cbe4644, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb59e4647, SUC_lexicon_tags_17 },
    { 0x633e4648, SUC_lexicon_tags_12 },
    { 0xd75e4649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fe464d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x175e464f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8de4651, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x197e4653, SUC_lexicon_tags_1 },
    { 0x9be4653, SUC_lexicon_tags_13 },
    { 0x1ffe4655, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x85e4657, SUC_lexicon_tags_0 },
    { 0x197e4658, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcade465b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e465e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e4664, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x229e4666, SUC_lexicon_tags_23 },
    { 0x675e4667, SUC_lexicon_tags_8 },
    { 0xd5de4667, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x6efe466b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afe466e, SUC_lexicon_tags_37 },
    { 0x8bbe466f, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27e4672, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5e4676, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa97e4678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36be467a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5e4681, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbe4686, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657e468d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5e4694, SUC_lexicon_tags_12 },
    { 0xa6de4695, SUC_lexicon_tags_15 },
    { 0x449e4695, SUC_lexicon_tags_12 },
    { 0x3b5e4695, SUC_lexicon_tags_43 },
    { 0x541e4698, SUC_lexicon_tags_12 },
    { 0x11fe4699, SUC_lexicon_tags_1 },
    { 0x4cde469a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535e469f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe47e46a1, SUC_lexicon_tags_81 },
    { 0x8dbe46a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafe46a5, SUC_lexicon_tags_12 },
    { 0x36de46a6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fe46a9, SUC_lexicon_tags_11 },
    { 0xb13e46a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9e46ae, SUC_lexicon_tags_35 },
    { 0xcb9e46ae, SUC_lexicon_tags_81 },
    { 0xc37e46b0, SUC_lexicon_tags_7 },
    { 0xe95e46b0, SUC_lexicon_tags_17 },
    { 0xabfe46b1, SUC_lexicon_tags_25 },
    { 0x1c9e46b3, SUC_lexicon_tags_15 },
    { 0xd33e46ae, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3e46b8, SUC_lexicon_tags_12 },
    { 0x5f9e46b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811e46be, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6afe46c0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13e46cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8de46d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fe46d5, SUC_lexicon_tags_45 },
    { 0xd9be46d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafe46db, SUC_lexicon_tags_17 },
    { 0xccfe46dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa87e46df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbb9e46e1, SUC_lexicon_tags_15 },
    { 0x2fde46e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x51e46e4, SUC_lexicon_tags_12 },
    { 0xba3e46e5, SUC_lexicon_tags_15 },
    { 0xdf7e46e6, SUC_lexicon_tags_1 },
    { 0x6bde46e6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x219e46ea, SUC_lexicon_tags_1 },
    { 0x8dfe46eb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x249e46ed, SUC_lexicon_tags_43 },
    { 0xf25e46ed, SUC_lexicon_tags_11 },
    { 0xc45e46ef, SUC_lexicon_tags_43 },
    { 0x9b9e46ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f7e46f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x881e46f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64de46fc, SUC_lexicon_tags_17 },
    { 0xe19e46fc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfade46ff, SUC_lexicon_tags_55 },
    { 0xddde4700, SUC_lexicon_tags_11 },
    { 0xe43e4700, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97e4706, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49de470a, SUC_lexicon_tags_11 },
    { 0x173e470a, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2cbe470d, SUC_lexicon_tags_38 },
    { 0xa83e470e, SUC_lexicon_tags_138 },
    { 0xd3e470f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9e4713, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffe4716, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x339e4718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55de471f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0de4722, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x70de4725, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25e472a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9e4732, SUC_lexicon_tags_15 },
    { 0x4b7e4733, SUC_lexicon_tags_284 },
    { 0xbdde4734, SUC_lexicon_tags_22 },
    { 0x61e4734, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3e4738, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69e473f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9dbe4741, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81be4745, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x265e4747, SUC_lexicon_tags_10 },
    { 0x5ade4747, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9e474b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509e474f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a9e4751, SUC_lexicon_tags_37 },
    { 0x805e4751, SUC_lexicon_tags_12 },
    { 0xb47e4753, SUC_lexicon_tags_37 },
    { 0xcf1e4754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be4759, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf71e475b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ade475e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fe4766, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x387e476c, SUC_lexicon_tags_47 },
    { 0x4f7e476d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9be4771, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x767e4774, SUC_lexicon_tags_1 },
    { 0xd51e4775, SUC_lexicon_tags_74 },
    { 0x971e4776, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6de4779, SUC_lexicon_tags_61 },
    { 0xd15e477a, SUC_lexicon_tags_151 },
    { 0x7ffe477b, SUC_lexicon_tags_49 },
    { 0x603e477a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5e477e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf59e4780, SUC_lexicon_tags_424 },
    { 0, NULL },
    { 0x1d1e4782, SUC_lexicon_tags_11 },
    { 0x315e4783, SUC_lexicon_tags_1 },
    { 0xcfe4783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1e4787, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3e478c, SUC_lexicon_tags_26 },
    { 0x543e478c, SUC_lexicon_tags_4 },
    { 0xd49e478e, SUC_lexicon_tags_63 },
    { 0xcf3e478c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4799, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95fe479b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x527e479d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5e47a0, SUC_lexicon_tags_17 },
    { 0xcfe47a1, SUC_lexicon_tags_7 },
    { 0xb11e47a0, SUC_lexicon_tags_55 },
    { 0xdbbe47a3, SUC_lexicon_tags_1 },
    { 0x619e47a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e7e47a6, SUC_lexicon_tags_7 },
    { 0xd5de47a6, SUC_lexicon_tags_7 },
    { 0xb1fe47a7, SUC_lexicon_tags_49 },
    { 0x521e47a9, SUC_lexicon_tags_2 },
    { 0x509e47aa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487e47b3, SUC_lexicon_tags_8 },
    { 0x7a7e47b4, SUC_lexicon_tags_2 },
    { 0xe75e47b5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3e47ba, SUC_lexicon_tags_19 },
    { 0xe4be47bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791e47bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x94be47c1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de47c6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9e47cc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6be47ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cde47d2, SUC_lexicon_tags_25 },
    { 0x84de47d3, SUC_lexicon_tags_17 },
    { 0x89de47d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaa7e47d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e47db, SUC_lexicon_tags_63 },
    { 0x521e47dc, SUC_lexicon_tags_1 },
    { 0x117e47dd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x779e47e0, SUC_lexicon_tags_6 },
    { 0xc59e47e0, SUC_lexicon_tags_55 },
    { 0xcdbe47e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x41de47e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1de47e7, SUC_lexicon_tags_27 },
    { 0xc6fe47e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fe47f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfade47f2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ede47f6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x953e47f9, SUC_lexicon_tags_0 },
    { 0x947e47fa, SUC_lexicon_tags_65 },
    { 0x9cbe47fb, SUC_lexicon_tags_19 },
    { 0xef1e47fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4a5e47fe, SUC_lexicon_tags_752 },
    { 0, NULL },
    { 0, NULL },
    { 0x907e4801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x17be4803, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc4be4805, SUC_lexicon_tags_37 },
    { 0x6d1e4805, SUC_lexicon_tags_12 },
    { 0xf3e4807, SUC_lexicon_tags_47 },
    { 0x76de4807, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e480b, SUC_lexicon_tags_21 },
    { 0xc45e480c, SUC_lexicon_tags_17 },
    { 0xfefe480b, SUC_lexicon_tags_39 },
    { 0x57e480e, SUC_lexicon_tags_46 },
    { 0x633e480e, SUC_lexicon_tags_47 },
    { 0x53e480e, SUC_lexicon_tags_25 },
    { 0x83de480c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e1e4813, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5e4818, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe481e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51de4823, SUC_lexicon_tags_10 },
    { 0xc8fe4824, SUC_lexicon_tags_17 },
    { 0xd67e4825, SUC_lexicon_tags_8 },
    { 0xdb1e4825, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de482a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x593e482d, SUC_lexicon_tags_3 },
    { 0x905e482e, SUC_lexicon_tags_15 },
    { 0x161e482f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fe4834, SUC_lexicon_tags_17 },
    { 0x1cbe4835, SUC_lexicon_tags_1 },
    { 0xf81e4835, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x563e4838, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3e483c, SUC_lexicon_tags_25 },
    { 0x273e483d, SUC_lexicon_tags_49 },
    { 0x155e483d, SUC_lexicon_tags_11 },
    { 0xba7e483f, SUC_lexicon_tags_10 },
    { 0xb77e4840, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40be4848, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x87be484a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69e4852, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5be485e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4be4861, SUC_lexicon_tags_7 },
    { 0x785e4861, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6a3e4864, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x607e4867, SUC_lexicon_tags_4 },
    { 0xb3fe4868, SUC_lexicon_tags_33 },
    { 0x6a9e4869, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7e486f, SUC_lexicon_tags_23 },
    { 0xba7e4870, SUC_lexicon_tags_63 },
    { 0xa7de4870, SUC_lexicon_tags_47 },
    { 0xa7be4872, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa73e4874, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa55e4876, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07e487c, SUC_lexicon_tags_3 },
    { 0xbfde487d, SUC_lexicon_tags_37 },
    { 0x131e487e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x44de4880, SUC_lexicon_tags_7 },
    { 0xc0be4881, SUC_lexicon_tags_9 },
    { 0x217e4881, SUC_lexicon_tags_1 },
    { 0x759e4883, SUC_lexicon_tags_14 },
    { 0xa49e4881, SUC_lexicon_tags_49 },
    { 0xbc1e4883, SUC_lexicon_tags_27 },
    { 0x399e4880, SUC_lexicon_tags_11 },
    { 0x4ffe4887, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4e9e4889, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83e488f, SUC_lexicon_tags_34 },
    { 0x455e4890, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc5e4892, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157e489b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f5e489d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903e48a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaede48ab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99de48b8, SUC_lexicon_tags_2 },
    { 0x73fe48b9, SUC_lexicon_tags_2 },
    { 0xb0be48b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37de48c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723e48c4, SUC_lexicon_tags_42 },
    { 0x40be48c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5e48c9, SUC_lexicon_tags_350 },
    { 0, NULL },
    { 0, NULL },
    { 0x921e48cc, SUC_lexicon_tags_2 },
    { 0x355e48cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeede48d1, SUC_lexicon_tags_25 },
    { 0xe57e48d1, SUC_lexicon_tags_13 },
    { 0xcdbe48d2, SUC_lexicon_tags_12 },
    { 0xe8be48d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26de48d6, SUC_lexicon_tags_8 },
    { 0x299e48d7, SUC_lexicon_tags_26 },
    { 0xf85e48d8, SUC_lexicon_tags_30 },
    { 0xcdde48d8, SUC_lexicon_tags_6 },
    { 0x1a3e48d8, SUC_lexicon_tags_53 },
    { 0x85e48d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61e48e0, SUC_lexicon_tags_15 },
    { 0x7a1e48e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3e48e4, SUC_lexicon_tags_12 },
    { 0x1f5e48e5, SUC_lexicon_tags_27 },
    { 0x645e48e4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x26be48e8, SUC_lexicon_tags_12 },
    { 0x2b3e48e9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30be48ee, SUC_lexicon_tags_26 },
    { 0xb47e48ee, SUC_lexicon_tags_14 },
    { 0x63de48f0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4de48f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfe48f9, SUC_lexicon_tags_7 },
    { 0x42de48fa, SUC_lexicon_tags_17 },
    { 0xbc7e48fb, SUC_lexicon_tags_11 },
    { 0xd25e48fc, SUC_lexicon_tags_28 },
    { 0x63e48fd, SUC_lexicon_tags_30 },
    { 0xbfe48fb, SUC_lexicon_tags_14 },
    { 0xfe48f9, SUC_lexicon_tags_17 },
    { 0x7cbe4900, SUC_lexicon_tags_15 },
    { 0x49e4901, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc85e4903, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7abe4905, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5e4909, SUC_lexicon_tags_26 },
    { 0x369e490a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76be4912, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x471e4914, SUC_lexicon_tags_29 },
    { 0xc39e4915, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe491a, SUC_lexicon_tags_14 },
    { 0x94de491b, SUC_lexicon_tags_165 },
    { 0xdede491b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0be4928, SUC_lexicon_tags_37 },
    { 0xf21e4929, SUC_lexicon_tags_15 },
    { 0x5b7e492a, SUC_lexicon_tags_38 },
    { 0xc3fe4928, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07e492f, SUC_lexicon_tags_61 },
    { 0xa43e4930, SUC_lexicon_tags_17 },
    { 0x9b5e4930, SUC_lexicon_tags_27 },
    { 0x3b5e492f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339e493c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe3fe493e, SUC_lexicon_tags_19 },
    { 0xc29e493e, SUC_lexicon_tags_37 },
    { 0xecfe493f, SUC_lexicon_tags_2 },
    { 0x533e4941, SUC_lexicon_tags_25 },
    { 0x453e4942, SUC_lexicon_tags_15 },
    { 0xe2de493f, SUC_lexicon_tags_17 },
    { 0xbfe4944, SUC_lexicon_tags_37 },
    { 0xeede4945, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32fe494d, SUC_lexicon_tags_2 },
    { 0xe7de494e, SUC_lexicon_tags_12 },
    { 0xed1e494f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03e4953, SUC_lexicon_tags_21 },
    { 0xc69e4953, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x433e4956, SUC_lexicon_tags_25 },
    { 0xbf5e4956, SUC_lexicon_tags_37 },
    { 0x9a1e4958, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb65e495a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651e4962, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcade4964, SUC_lexicon_tags_25 },
    { 0x1d7e4965, SUC_lexicon_tags_1 },
    { 0x24de4966, SUC_lexicon_tags_7 },
    { 0x707e4967, SUC_lexicon_tags_55 },
    { 0x75be4967, SUC_lexicon_tags_1 },
    { 0xcbbe4966, SUC_lexicon_tags_2 },
    { 0xd89e4969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbe496d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17e4984, SUC_lexicon_tags_2 },
    { 0x799e4984, SUC_lexicon_tags_47 },
    { 0xefe4984, SUC_lexicon_tags_16 },
    { 0x7fbe4985, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9e498e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17be4990, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x705e4992, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xfe3e4994, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8bde4996, SUC_lexicon_tags_60 },
    { 0x963e4997, SUC_lexicon_tags_12 },
    { 0x637e4997, SUC_lexicon_tags_17 },
    { 0x2a9e4997, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503e499f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3e49a9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x453e49ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x121e49ad, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x54be49af, SUC_lexicon_tags_69 },
    { 0xc07e49b0, SUC_lexicon_tags_23 },
    { 0x385e49b1, SUC_lexicon_tags_11 },
    { 0x419e49b1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x98be49b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe49b9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6de49bd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99de49c2, SUC_lexicon_tags_4 },
    { 0xfdbe49c3, SUC_lexicon_tags_1 },
    { 0xb93e49c2, SUC_lexicon_tags_5 },
    { 0xd35e49c2, SUC_lexicon_tags_49 },
    { 0x4c9e49c6, SUC_lexicon_tags_29 },
    { 0xde7e49c7, SUC_lexicon_tags_12 },
    { 0x677e49c8, SUC_lexicon_tags_47 },
    { 0xaffe49c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x47de49cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x28be49ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb79e49d0, SUC_lexicon_tags_15 },
    { 0xa99e49d1, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fe49d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x309e49d6, SUC_lexicon_tags_27 },
    { 0xd25e49d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67be49da, SUC_lexicon_tags_15 },
    { 0x7e9e49db, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901e49e0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7e49ec, SUC_lexicon_tags_17 },
    { 0xc0fe49ec, SUC_lexicon_tags_27 },
    { 0xbfde49ed, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb0fe49f0, SUC_lexicon_tags_28 },
    { 0xb95e49f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xedde49f3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0de49f7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7e49fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a7e49ff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x995e4a01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ade4a07, SUC_lexicon_tags_37 },
    { 0xfcfe4a08, SUC_lexicon_tags_30 },
    { 0xc7fe4a09, SUC_lexicon_tags_6 },
    { 0x1d3e4a09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f3e4a0c, SUC_lexicon_tags_31 },
    { 0xdf5e4a0d, SUC_lexicon_tags_27 },
    { 0xd01e4a0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dde4a15, SUC_lexicon_tags_14 },
    { 0xb65e4a16, SUC_lexicon_tags_84 },
    { 0x1bfe4a16, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447e4a1c, SUC_lexicon_tags_22 },
    { 0x8dbe4a1c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477e4a21, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fe4a27, SUC_lexicon_tags_1 },
    { 0xc5be4a28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6fe4a2a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x709e4a2d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x45de4a2f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5ede4a31, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x95be4a33, SUC_lexicon_tags_8 },
    { 0xbb1e4a33, SUC_lexicon_tags_37 },
    { 0x4c5e4a33, SUC_lexicon_tags_66 },
    { 0xa67e4a36, SUC_lexicon_tags_6 },
    { 0x9b1e4a37, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157e4a3d, SUC_lexicon_tags_47 },
    { 0xd1e4a3e, SUC_lexicon_tags_2 },
    { 0x1a9e4a3d, SUC_lexicon_tags_2 },
    { 0x8b1e4a40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17de4a47, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x58be4a4a, SUC_lexicon_tags_25 },
    { 0xbbe4a4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cbe4a4d, SUC_lexicon_tags_34 },
    { 0xaa7e4a4e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3bde4a50, SUC_lexicon_tags_57 },
    { 0xf5de4a50, SUC_lexicon_tags_14 },
    { 0x627e4a51, SUC_lexicon_tags_15 },
    { 0x413e4a53, SUC_lexicon_tags_38 },
    { 0x11e4a53, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x101e4a56, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1e4a5d, SUC_lexicon_tags_21 },
    { 0x63be4a5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e1e4a60, SUC_lexicon_tags_11 },
    { 0xedbe4a60, SUC_lexicon_tags_0 },
    { 0xb19e4a62, SUC_lexicon_tags_38 },
    { 0x3abe4a63, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e4a68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x169e4a6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6dfe4a6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x10de4a6f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3e4a73, SUC_lexicon_tags_45 },
    { 0xfc7e4a74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1e4a7a, SUC_lexicon_tags_11 },
    { 0x5c9e4a7b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf89e4a7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafe4a80, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefe4a84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187e4a88, SUC_lexicon_tags_47 },
    { 0x567e4a89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeabe4a8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x387e4a8d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2d5e4a8f, SUC_lexicon_tags_27 },
    { 0x6b9e4a8f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603e4a99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24be4aa2, SUC_lexicon_tags_12 },
    { 0xfc3e4aa3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397e4aac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e4ab0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc69e4ab2, SUC_lexicon_tags_37 },
    { 0x499e4ab2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffe4aba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75e4abe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xae1e4ac0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8ede4ac2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fe4ac6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5e4acc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637e4ad1, SUC_lexicon_tags_1 },
    { 0x2d9e4ad2, SUC_lexicon_tags_87 },
    { 0xacfe4ad2, SUC_lexicon_tags_1 },
    { 0x795e4ad4, SUC_lexicon_tags_1 },
    { 0xb8de4ad4, SUC_lexicon_tags_79 },
    { 0x6fe4ad1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x287e4ad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb3e4ada, SUC_lexicon_tags_12 },
    { 0x6abe4adb, SUC_lexicon_tags_2 },
    { 0x61e4adc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ade4ae0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x791e4ae2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x23de4ae5, SUC_lexicon_tags_43 },
    { 0x49be4ae5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf7e4ae8, SUC_lexicon_tags_37 },
    { 0xa91e4ae8, SUC_lexicon_tags_26 },
    { 0x2f1e4ae8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253e4aef, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1e4af2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xabde4af4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891e4af8, SUC_lexicon_tags_12 },
    { 0x25be4af9, SUC_lexicon_tags_37 },
    { 0x5abe4afa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383e4b00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5e4b06, SUC_lexicon_tags_30 },
    { 0xe75e4b07, SUC_lexicon_tags_25 },
    { 0x39de4b08, SUC_lexicon_tags_11 },
    { 0x64fe4b07, SUC_lexicon_tags_45 },
    { 0xf09e4b07, SUC_lexicon_tags_17 },
    { 0xe99e4b0a, SUC_lexicon_tags_47 },
    { 0xc63e4b07, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5e4b0f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6b5e4b11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fe4b16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7e7e4b18, SUC_lexicon_tags_31 },
    { 0x601e4b18, SUC_lexicon_tags_12 },
    { 0xd79e4b1a, SUC_lexicon_tags_61 },
    { 0x549e4b1a, SUC_lexicon_tags_33 },
    { 0x407e4b1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7e4b1f, SUC_lexicon_tags_12 },
    { 0xe85e4b20, SUC_lexicon_tags_30 },
    { 0x1a9e4b1f, SUC_lexicon_tags_25 },
    { 0xa21e4b22, SUC_lexicon_tags_7 },
    { 0xc49e4b23, SUC_lexicon_tags_37 },
    { 0x459e4b24, SUC_lexicon_tags_15 },
    { 0x96fe4b20, SUC_lexicon_tags_11 },
    { 0xe11e4b26, SUC_lexicon_tags_7 },
    { 0xeabe4b27, SUC_lexicon_tags_17 },
    { 0x669e4b28, SUC_lexicon_tags_49 },
    { 0x141e4b29, SUC_lexicon_tags_43 },
    { 0x73fe4b28, SUC_lexicon_tags_11 },
    { 0x753e4b2b, SUC_lexicon_tags_27 },
    { 0x493e4b2c, SUC_lexicon_tags_12 },
    { 0x4efe4b2b, SUC_lexicon_tags_11 },
    { 0x903e4b2e, SUC_lexicon_tags_19 },
    { 0x1b7e4b2f, SUC_lexicon_tags_37 },
    { 0xb5de4b2d, SUC_lexicon_tags_48 },
    { 0xbefe4b31, SUC_lexicon_tags_25 },
    { 0xe79e4b2c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217e4b38, SUC_lexicon_tags_47 },
    { 0x7fde4b39, SUC_lexicon_tags_7 },
    { 0xedbe4b3a, SUC_lexicon_tags_17 },
    { 0x583e4b3b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x26de4b3e, SUC_lexicon_tags_30 },
    { 0xf43e4b3f, SUC_lexicon_tags_63 },
    { 0xb65e4b3e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4e5e4b42, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x927e4b44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1e4b4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0de4b4e, SUC_lexicon_tags_12 },
    { 0x455e4b4f, SUC_lexicon_tags_31 },
    { 0x989e4b4f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb05e4b52, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x423e4b54, SUC_lexicon_tags_47 },
    { 0x307e4b54, SUC_lexicon_tags_47 },
    { 0x15e4b56, SUC_lexicon_tags_53 },
    { 0x53de4b57, SUC_lexicon_tags_25 },
    { 0x49e4b57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd51e4b5a, SUC_lexicon_tags_6 },
    { 0x111e4b5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99e4b66, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fe4b74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf29e4b76, SUC_lexicon_tags_10 },
    { 0xd6fe4b76, SUC_lexicon_tags_37 },
    { 0xb9be4b77, SUC_lexicon_tags_11 },
    { 0x489e4b79, SUC_lexicon_tags_11 },
    { 0x901e4b79, SUC_lexicon_tags_12 },
    { 0xc7de4b7b, SUC_lexicon_tags_45 },
    { 0xa99e4b7c, SUC_lexicon_tags_11 },
    { 0x531e4b79, SUC_lexicon_tags_21 },
    { 0x6dde4b7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfe4b83, SUC_lexicon_tags_37 },
    { 0xaa9e4b84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbde4b88, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xecfe4b8a, SUC_lexicon_tags_19 },
    { 0xf9de4b8b, SUC_lexicon_tags_23 },
    { 0x19be4b8c, SUC_lexicon_tags_7 },
    { 0xa9e4b8d, SUC_lexicon_tags_30 },
    { 0x901e4b8e, SUC_lexicon_tags_29 },
    { 0x7abe4b8b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x189e4b91, SUC_lexicon_tags_1 },
    { 0xdf5e4b92, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x963e4b95, SUC_lexicon_tags_7 },
    { 0x1ede4b96, SUC_lexicon_tags_65 },
    { 0x473e4b97, SUC_lexicon_tags_79 },
    { 0xdede4b97, SUC_lexicon_tags_19 },
    { 0xa7de4b99, SUC_lexicon_tags_37 },
    { 0xffde4b98, SUC_lexicon_tags_0 },
    { 0x7e1e4b9b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4fe4b9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79be4ba0, SUC_lexicon_tags_37 },
    { 0xaa1e4ba1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1e4ba3, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x353e4ba5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe4ba9, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x901e4bab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5de4bb4, SUC_lexicon_tags_29 },
    { 0xc0fe4bb5, SUC_lexicon_tags_12 },
    { 0xf67e4bb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3e4bb9, SUC_lexicon_tags_25 },
    { 0xf59e4bba, SUC_lexicon_tags_63 },
    { 0xe3de4bbb, SUC_lexicon_tags_37 },
    { 0xd33e4bbc, SUC_lexicon_tags_25 },
    { 0xf55e4bb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x19be4bbf, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5be4bc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x30fe4bc4, SUC_lexicon_tags_11 },
    { 0x25de4bc5, SUC_lexicon_tags_60 },
    { 0xed9e4bc5, SUC_lexicon_tags_45 },
    { 0x265e4bc7, SUC_lexicon_tags_25 },
    { 0xcfbe4bc7, SUC_lexicon_tags_30 },
    { 0x41de4bc6, SUC_lexicon_tags_15 },
    { 0x99be4bca, SUC_lexicon_tags_1 },
    { 0x1dbe4bcb, SUC_lexicon_tags_37 },
    { 0xeb9e4bcb, SUC_lexicon_tags_25 },
    { 0xdede4bcd, SUC_lexicon_tags_0 },
    { 0x77fe4bcd, SUC_lexicon_tags_11 },
    { 0xb95e4bcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e4bd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941e4bdb, SUC_lexicon_tags_11 },
    { 0x489e4bdb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfe4be1, SUC_lexicon_tags_39 },
    { 0xf47e4be1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebe4be5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27e4be9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x567e4bec, SUC_lexicon_tags_7 },
    { 0xa3e4bec, SUC_lexicon_tags_1 },
    { 0x4ebe4bee, SUC_lexicon_tags_39 },
    { 0x367e4bee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fe4bf2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffe4bf5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe4bfa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4ffe4bfc, SUC_lexicon_tags_19 },
    { 0x3e7e4bfc, SUC_lexicon_tags_44 },
    { 0x10de4bfd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1be4c03, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x669e4c06, SUC_lexicon_tags_11 },
    { 0x323e4c07, SUC_lexicon_tags_33 },
    { 0x37fe4c08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6de4c0c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6ede4c0e, SUC_lexicon_tags_11 },
    { 0xb33e4c0f, SUC_lexicon_tags_37 },
    { 0x23e4c0f, SUC_lexicon_tags_15 },
    { 0x765e4c11, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71e4c17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955e4c1d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9de4c23, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf85e4c25, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x305e4c28, SUC_lexicon_tags_103 },
    { 0x851e4c29, SUC_lexicon_tags_572 },
    { 0, NULL },
    { 0x7ebe4c2b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9be4c2f, SUC_lexicon_tags_15 },
    { 0xa2fe4c30, SUC_lexicon_tags_4 },
    { 0xc05e4c31, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7e4c34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fe4c3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9a5e4c3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x191e4c3f, SUC_lexicon_tags_11 },
    { 0x113e4c40, SUC_lexicon_tags_77 },
    { 0xf49e4c3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69e4c45, SUC_lexicon_tags_11 },
    { 0x22de4c46, SUC_lexicon_tags_11 },
    { 0xfbe4c45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40de4c4d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe2de4c4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9e4c53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xadde4c55, SUC_lexicon_tags_7 },
    { 0x77be4c55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fe4c65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccde4c67, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1e4c6b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd37e4c6d, SUC_lexicon_tags_43 },
    { 0x5b5e4c6d, SUC_lexicon_tags_37 },
    { 0xc7e4c6e, SUC_lexicon_tags_65 },
    { 0xa7be4c6f, SUC_lexicon_tags_49 },
    { 0x29fe4c71, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61fe4c73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1de4c78, SUC_lexicon_tags_28 },
    { 0xd0fe4c78, SUC_lexicon_tags_33 },
    { 0x8a7e4c7a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x349e4c7d, SUC_lexicon_tags_1 },
    { 0x2d9e4c7d, SUC_lexicon_tags_12 },
    { 0x879e4c7f, SUC_lexicon_tags_26 },
    { 0xdf3e4c80, SUC_lexicon_tags_6 },
    { 0x4f9e4c81, SUC_lexicon_tags_13 },
    { 0x1e1e4c82, SUC_lexicon_tags_2 },
    { 0xd65e4c83, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8afe4c86, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fe4c8a, SUC_lexicon_tags_25 },
    { 0xd45e4c8b, SUC_lexicon_tags_56 },
    { 0x237e4c8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf49e4c8e, SUC_lexicon_tags_14 },
    { 0x695e4c8f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x603e4c91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x90de4c94, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1e4c98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x575e4c9a, SUC_lexicon_tags_2 },
    { 0xed3e4c9a, SUC_lexicon_tags_23 },
    { 0x1d3e4c9c, SUC_lexicon_tags_12 },
    { 0xfa9e4c9d, SUC_lexicon_tags_10 },
    { 0xb3e4c9e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7e4ca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423e4ca8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993e4cb5, SUC_lexicon_tags_17 },
    { 0xc45e4cb6, SUC_lexicon_tags_37 },
    { 0x431e4cb7, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7e4cbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5e4cc0, SUC_lexicon_tags_8 },
    { 0x507e4cc1, SUC_lexicon_tags_17 },
    { 0x8e5e4cc0, SUC_lexicon_tags_19 },
    { 0xd6de4cc3, SUC_lexicon_tags_7 },
    { 0x833e4cc4, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x88fe4cc6, SUC_lexicon_tags_12 },
    { 0xa1be4cc7, SUC_lexicon_tags_23 },
    { 0xe77e4cc8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11fe4ccc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311e4cd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3e4cda, SUC_lexicon_tags_81 },
    { 0xc73e4cda, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x699e4cdd, SUC_lexicon_tags_15 },
    { 0x541e4cde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fe4ce6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f5e4ce8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x58be4ceb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x92de4ced, SUC_lexicon_tags_55 },
    { 0xf3fe4cee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x111e4cf0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa91e4cf2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fe4cf5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebe4cf9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9dde4cfb, SUC_lexicon_tags_31 },
    { 0x8fe4cfc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95e4cff, SUC_lexicon_tags_26 },
    { 0xeb3e4cff, SUC_lexicon_tags_2 },
    { 0x737e4d01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96de4d05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1e4d08, SUC_lexicon_tags_87 },
    { 0x935e4d08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x469e4d0b, SUC_lexicon_tags_11 },
    { 0xe5be4d0b, SUC_lexicon_tags_37 },
    { 0x761e4d0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efe4d13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x887e4d15, SUC_lexicon_tags_261 },
    { 0x6fbe4d15, SUC_lexicon_tags_0 },
    { 0xd03e4d17, SUC_lexicon_tags_12 },
    { 0xd13e4d17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d9e4d1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x689e4d1d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9e4d20, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa95e4d22, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e4d26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4be4d28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1be4d2c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x65e4d2e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x33de4d30, SUC_lexicon_tags_77 },
    { 0xf37e4d30, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565e4d39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffe4d3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9e4d3f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x189e4d42, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd87e4d44, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377e4d4b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x795e4d4e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fe4d52, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e4d55, SUC_lexicon_tags_25 },
    { 0xa75e4d55, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1e4d59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x991e4d5c, SUC_lexicon_tags_1 },
    { 0x8afe4d5d, SUC_lexicon_tags_39 },
    { 0xa9be4d5e, SUC_lexicon_tags_113 },
    { 0x14be4d5c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x8c5e4d61, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x92fe4d63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd13e4d65, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c7e4d67, SUC_lexicon_tags_10 },
    { 0xe05e4d68, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fe4d6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7e4d70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95e4d75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37be4d77, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51de4d7b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xde1e4d7d, SUC_lexicon_tags_26 },
    { 0xee3e4d7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa8be4d80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x843e4d83, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf5e4d85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x347e4d87, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffe4d8f, SUC_lexicon_tags_17 },
    { 0x37de4d90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabe4d96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e4d9b, SUC_lexicon_tags_12 },
    { 0xdf5e4d9c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79e4da4, SUC_lexicon_tags_6 },
    { 0xd7de4da5, SUC_lexicon_tags_6 },
    { 0x38de4da4, SUC_lexicon_tags_63 },
    { 0xa19e4da5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbe4dac, SUC_lexicon_tags_15 },
    { 0xac5e4dad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd85e4daf, SUC_lexicon_tags_10 },
    { 0xcd9e4daf, SUC_lexicon_tags_46 },
    { 0xcb5e4db1, SUC_lexicon_tags_37 },
    { 0xd0de4db2, SUC_lexicon_tags_14 },
    { 0xf91e4db1, SUC_lexicon_tags_27 },
    { 0xa17e4db4, SUC_lexicon_tags_30 },
    { 0x4d3e4daf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81e4dbb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc71e4dbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3e4dc2, SUC_lexicon_tags_27 },
    { 0x897e4dc3, SUC_lexicon_tags_7 },
    { 0x821e4dc4, SUC_lexicon_tags_63 },
    { 0xae5e4dc3, SUC_lexicon_tags_23 },
    { 0x651e4dc6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9e4dc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf21e4dcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97e4dcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12be4dcf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe53e4dd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53e4ddf, SUC_lexicon_tags_26 },
    { 0xd1de4ddf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69e4de4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x983e4de6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1e4dea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e4ded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809e4df3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbe4dfb, SUC_lexicon_tags_45 },
    { 0x2b5e4dfb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x64fe4dff, SUC_lexicon_tags_27 },
    { 0xb5fe4dff, SUC_lexicon_tags_63 },
    { 0xbd1e4e01, SUC_lexicon_tags_37 },
    { 0x6e3e4e01, SUC_lexicon_tags_9 },
    { 0x96fe4e03, SUC_lexicon_tags_15 },
    { 0xec1e4e04, SUC_lexicon_tags_7 },
    { 0x3cde4e05, SUC_lexicon_tags_39 },
    { 0x473e4e05, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9e4e09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x197e4e0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9e4e16, SUC_lexicon_tags_2 },
    { 0x263e4e17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115e4e1c, SUC_lexicon_tags_30 },
    { 0xc87e4e1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x509e4e20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263e4e28, SUC_lexicon_tags_12 },
    { 0xef3e4e28, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x29be4e2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625e4e30, SUC_lexicon_tags_47 },
    { 0x74be4e31, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e4e39, SUC_lexicon_tags_17 },
    { 0x175e4e3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733e4e3e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7e4e41, SUC_lexicon_tags_14 },
    { 0xade4e42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x62de4e44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9e4e47, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x81be4e49, SUC_lexicon_tags_16 },
    { 0x7f1e4e49, SUC_lexicon_tags_42 },
    { 0x4a5e4e49, SUC_lexicon_tags_25 },
    { 0xcdbe4e4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x96be4e4e, SUC_lexicon_tags_25 },
    { 0x929e4e4f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x805e4e51, SUC_lexicon_tags_17 },
    { 0xbd1e4e51, SUC_lexicon_tags_25 },
    { 0x795e4e53, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x87de4e56, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45e4e59, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x433e4e5c, SUC_lexicon_tags_103 },
    { 0xecbe4e5c, SUC_lexicon_tags_10 },
    { 0x257e4e5e, SUC_lexicon_tags_2 },
    { 0xefde4e5f, SUC_lexicon_tags_17 },
    { 0xcc9e4e60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3e4e63, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9de4e68, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14de4e6c, SUC_lexicon_tags_17 },
    { 0xccde4e6d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49de4e71, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5b7e4e73, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x693e4e75, SUC_lexicon_tags_1 },
    { 0xa8fe4e76, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xf8de4e78, SUC_lexicon_tags_28 },
    { 0x665e4e79, SUC_lexicon_tags_45 },
    { 0xbfe4e7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fe4e81, SUC_lexicon_tags_17 },
    { 0x437e4e82, SUC_lexicon_tags_17 },
    { 0xf7fe4e83, SUC_lexicon_tags_6 },
    { 0x72de4e82, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fe4e8e, SUC_lexicon_tags_56 },
    { 0x20fe4e8f, SUC_lexicon_tags_16 },
    { 0x46be4e8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973e4e95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbe4e98, SUC_lexicon_tags_79 },
    { 0xbc7e4e99, SUC_lexicon_tags_12 },
    { 0x161e4e9a, SUC_lexicon_tags_10 },
    { 0xbfbe4e99, SUC_lexicon_tags_37 },
    { 0xafe4e9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3de4e9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbe4ea0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01e4ea6, SUC_lexicon_tags_15 },
    { 0xc1fe4ea6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x39de4ea9, SUC_lexicon_tags_39 },
    { 0x355e4eaa, SUC_lexicon_tags_15 },
    { 0x929e4eaa, SUC_lexicon_tags_37 },
    { 0xc85e4eab, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cde4eb0, SUC_lexicon_tags_37 },
    { 0x62be4eb1, SUC_lexicon_tags_25 },
    { 0x7c1e4eb2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x32de4eb4, SUC_lexicon_tags_17 },
    { 0xf8de4eb5, SUC_lexicon_tags_7 },
    { 0xdcbe4eb5, SUC_lexicon_tags_37 },
    { 0x8ffe4eb6, SUC_lexicon_tags_25 },
    { 0x355e4eb8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21be4eba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd67e4ebc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x733e4ebe, SUC_lexicon_tags_37 },
    { 0x8f5e4ebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x70be4ec2, SUC_lexicon_tags_14 },
    { 0x435e4ec3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe4ec8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbe4ecd, SUC_lexicon_tags_31 },
    { 0x1e1e4ecd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e4ed1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929e4ed5, SUC_lexicon_tags_19 },
    { 0x6fbe4ed6, SUC_lexicon_tags_28 },
    { 0xc5de4ed7, SUC_lexicon_tags_29 },
    { 0xeb9e4ed7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x603e4edb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25e4ede, SUC_lexicon_tags_45 },
    { 0xb3fe4edf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x69de4ee2, SUC_lexicon_tags_25 },
    { 0xc23e4ee3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5e4ee8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7f3e4eea, SUC_lexicon_tags_92 },
    { 0xfbe4eea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x99de4eed, SUC_lexicon_tags_7 },
    { 0x6ffe4eee, SUC_lexicon_tags_5 },
    { 0x845e4eee, SUC_lexicon_tags_30 },
    { 0x6f5e4eef, SUC_lexicon_tags_37 },
    { 0x83be4ef1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x363e4ef3, SUC_lexicon_tags_12 },
    { 0xed9e4ef4, SUC_lexicon_tags_60 },
    { 0xe7be4ef5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9e4efb, SUC_lexicon_tags_37 },
    { 0x8bde4efb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a3e4efe, SUC_lexicon_tags_7 },
    { 0xe05e4efe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649e4f05, SUC_lexicon_tags_30 },
    { 0xfe1e4f06, SUC_lexicon_tags_7 },
    { 0xdcbe4f07, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395e4f0b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3dbe4f0d, SUC_lexicon_tags_7 },
    { 0x6a1e4f0d, SUC_lexicon_tags_11 },
    { 0x8a1e4f0f, SUC_lexicon_tags_466 },
    { 0x4d7e4f0d, SUC_lexicon_tags_17 },
    { 0xdde4f11, SUC_lexicon_tags_140 },
    { 0xb5be4f12, SUC_lexicon_tags_17 },
    { 0x82be4f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xea5e4f15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdafe4f17, SUC_lexicon_tags_15 },
    { 0xd29e4f18, SUC_lexicon_tags_27 },
    { 0x4efe4f18, SUC_lexicon_tags_31 },
    { 0xfd1e4f1a, SUC_lexicon_tags_2 },
    { 0xd01e4f1b, SUC_lexicon_tags_2 },
    { 0x9e1e4f1c, SUC_lexicon_tags_47 },
    { 0xd43e4f1d, SUC_lexicon_tags_17 },
    { 0x183e4f1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x395e4f21, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd9e4f23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b5e4f25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9e4f28, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x59de4f2a, SUC_lexicon_tags_17 },
    { 0x5d7e4f2b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de4f2e, SUC_lexicon_tags_8 },
    { 0xdb7e4f2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd1e4f31, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf35e4f33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebe4f36, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173e4f3a, SUC_lexicon_tags_29 },
    { 0x241e4f3a, SUC_lexicon_tags_27 },
    { 0xdd3e4f3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899e4f40, SUC_lexicon_tags_65 },
    { 0xf97e4f41, SUC_lexicon_tags_52 },
    { 0x75de4f41, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc81e4f44, SUC_lexicon_tags_14 },
    { 0x277e4f45, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7e4f48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa69e4f4a, SUC_lexicon_tags_46 },
    { 0x759e4f4b, SUC_lexicon_tags_17 },
    { 0x83be4f4a, SUC_lexicon_tags_20 },
    { 0xe87e4f4a, SUC_lexicon_tags_37 },
    { 0x391e4f4e, SUC_lexicon_tags_1 },
    { 0xbc3e4f4f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23e4f52, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xe63e4f54, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe4f57, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c7e4f59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4be4f5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x73e4f5e, SUC_lexicon_tags_1 },
    { 0x589e4f5f, SUC_lexicon_tags_17 },
    { 0xd93e4f60, SUC_lexicon_tags_11 },
    { 0x11e4f61, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4c5e4f63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc9e4f65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863e4f6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69de4f70, SUC_lexicon_tags_25 },
    { 0x86de4f71, SUC_lexicon_tags_38 },
    { 0xc75e4f71, SUC_lexicon_tags_12 },
    { 0x191e4f71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3e4f78, SUC_lexicon_tags_4 },
    { 0x829e4f79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8be4f7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78de4f88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3e4f8e, SUC_lexicon_tags_17 },
    { 0x2dbe4f8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x907e4f91, SUC_lexicon_tags_15 },
    { 0x9e7e4f92, SUC_lexicon_tags_7 },
    { 0xd53e4f93, SUC_lexicon_tags_65 },
    { 0x6dde4f91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49be4f96, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93e4f9d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17e4fa2, SUC_lexicon_tags_49 },
    { 0x791e4fa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe4fa6, SUC_lexicon_tags_12 },
    { 0xcf5e4fa6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2d5e4fa9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5e4faf, SUC_lexicon_tags_2 },
    { 0xaefe4fb0, SUC_lexicon_tags_11 },
    { 0x287e4fb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x42be4fb3, SUC_lexicon_tags_2 },
    { 0xa17e4fb4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa5de4fb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51fe4fb8, SUC_lexicon_tags_38 },
    { 0x165e4fb9, SUC_lexicon_tags_8 },
    { 0xe97e4fb8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67e4fbe, SUC_lexicon_tags_12 },
    { 0x36be4fbf, SUC_lexicon_tags_12 },
    { 0x20be4fbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e1e4fc2, SUC_lexicon_tags_45 },
    { 0xbcfe4fc3, SUC_lexicon_tags_11 },
    { 0x9b5e4fc2, SUC_lexicon_tags_2 },
    { 0xc7be4fc5, SUC_lexicon_tags_1 },
    { 0x899e4fc3, SUC_lexicon_tags_37 },
    { 0xce9e4fc7, SUC_lexicon_tags_23 },
    { 0x667e4fc6, SUC_lexicon_tags_63 },
    { 0x817e4fc8, SUC_lexicon_tags_43 },
    { 0x9d1e4fc9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x447e4fcd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x335e4fcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x17be4fd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x821e4fd4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x555e4fd6, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x939e4fd8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18be4fdd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31e4fe1, SUC_lexicon_tags_25 },
    { 0x28be4fe2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341e4fe8, SUC_lexicon_tags_81 },
    { 0x58fe4fe9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde4feb, SUC_lexicon_tags_1 },
    { 0x7de4fec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3e4ff1, SUC_lexicon_tags_55 },
    { 0x2fde4ff2, SUC_lexicon_tags_25 },
    { 0x41be4ff2, SUC_lexicon_tags_62 },
    { 0xd65e4ff1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x883e4ff7, SUC_lexicon_tags_47 },
    { 0x40fe4ff8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7e4ffb, SUC_lexicon_tags_7 },
    { 0xbe3e4ffb, SUC_lexicon_tags_1 },
    { 0x225e4ffd, SUC_lexicon_tags_25 },
    { 0xb1de4ffe, SUC_lexicon_tags_14 },
    { 0x4cfe4fff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03e5002, SUC_lexicon_tags_11 },
    { 0x795e5002, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a3e5005, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95e500a, SUC_lexicon_tags_9 },
    { 0x565e500b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf7e500d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627e5012, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe501c, SUC_lexicon_tags_25 },
    { 0x7b9e501d, SUC_lexicon_tags_49 },
    { 0x579e501e, SUC_lexicon_tags_14 },
    { 0xc35e501e, SUC_lexicon_tags_100 },
    { 0x16de501f, SUC_lexicon_tags_37 },
    { 0x867e5021, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6f7e5023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37de502a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7be5030, SUC_lexicon_tags_17 },
    { 0x48fe5030, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13e5033, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6dfe5035, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67fe5037, SUC_lexicon_tags_1 },
    { 0x643e5038, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa0de503a, SUC_lexicon_tags_10 },
    { 0xd47e503a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5e5045, SUC_lexicon_tags_37 },
    { 0xcb9e5045, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x425e5048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295e504d, SUC_lexicon_tags_19 },
    { 0x5a1e504d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1e5054, SUC_lexicon_tags_37 },
    { 0x9b7e5055, SUC_lexicon_tags_37 },
    { 0x163e5055, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3e5059, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x12de505c, SUC_lexicon_tags_25 },
    { 0x125e505d, SUC_lexicon_tags_60 },
    { 0x2afe505c, SUC_lexicon_tags_1 },
    { 0x683e505c, SUC_lexicon_tags_25 },
    { 0x151e5060, SUC_lexicon_tags_45 },
    { 0x921e5061, SUC_lexicon_tags_11 },
    { 0x863e5062, SUC_lexicon_tags_1 },
    { 0x48de5060, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf91e5065, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xad9e5067, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5e506b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89be506d, SUC_lexicon_tags_33 },
    { 0x237e506e, SUC_lexicon_tags_43 },
    { 0xab1e506e, SUC_lexicon_tags_19 },
    { 0xef7e506f, SUC_lexicon_tags_15 },
    { 0x827e506d, SUC_lexicon_tags_0 },
    { 0x603e5072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x863e5074, SUC_lexicon_tags_28 },
    { 0x653e5075, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14be5078, SUC_lexicon_tags_30 },
    { 0xd43e5079, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1e5081, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65e5085, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x66de5087, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39e5090, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x95e5092, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xafbe5094, SUC_lexicon_tags_60 },
    { 0xd41e5095, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x313e5097, SUC_lexicon_tags_2 },
    { 0xb41e5098, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe9de509a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31e509d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x737e509f, SUC_lexicon_tags_14 },
    { 0x39de50a0, SUC_lexicon_tags_17 },
    { 0xf9de50a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487e50a7, SUC_lexicon_tags_19 },
    { 0x91fe50a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x723e50aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ffe50ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851e50b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abe50be, SUC_lexicon_tags_15 },
    { 0xd39e50be, SUC_lexicon_tags_49 },
    { 0x67fe50bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc5be50c2, SUC_lexicon_tags_15 },
    { 0x6dfe50c3, SUC_lexicon_tags_58 },
    { 0x9a3e50c4, SUC_lexicon_tags_12 },
    { 0x133e50c3, SUC_lexicon_tags_25 },
    { 0xef1e50c2, SUC_lexicon_tags_15 },
    { 0x3fbe50c5, SUC_lexicon_tags_45 },
    { 0x9e9e50c8, SUC_lexicon_tags_25 },
    { 0x93e50c9, SUC_lexicon_tags_63 },
    { 0xeebe50c7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x209e50cd, SUC_lexicon_tags_27 },
    { 0xf05e50ce, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x779e50d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31e50d5, SUC_lexicon_tags_2 },
    { 0x47de50d5, SUC_lexicon_tags_12 },
    { 0x51e50d7, SUC_lexicon_tags_7 },
    { 0x18fe50d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9be50dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5e50e0, SUC_lexicon_tags_23 },
    { 0xd51e50e0, SUC_lexicon_tags_19 },
    { 0xc2be50e0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbdfe50e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639e50e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb91e50ea, SUC_lexicon_tags_2 },
    { 0xbd9e50eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973e50f1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefe50f5, SUC_lexicon_tags_11 },
    { 0x80fe50f5, SUC_lexicon_tags_46 },
    { 0xe8de50f6, SUC_lexicon_tags_16 },
    { 0x161e50f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf1e50fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5e50ff, SUC_lexicon_tags_15 },
    { 0x151e5100, SUC_lexicon_tags_37 },
    { 0x441e5101, SUC_lexicon_tags_17 },
    { 0x29fe5102, SUC_lexicon_tags_25 },
    { 0x343e5103, SUC_lexicon_tags_64 },
    { 0xdabe5103, SUC_lexicon_tags_15 },
    { 0xe69e5105, SUC_lexicon_tags_165 },
    { 0xd25e5106, SUC_lexicon_tags_2 },
    { 0x81e5107, SUC_lexicon_tags_49 },
    { 0xb87e5108, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x955e510a, SUC_lexicon_tags_47 },
    { 0xf9be510b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf5fe510d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35e5110, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57de511a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x96de511c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbe511f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c5e5121, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3e5124, SUC_lexicon_tags_7 },
    { 0x335e5125, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bfe5127, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455e5130, SUC_lexicon_tags_122 },
    { 0xa7e5131, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15e5136, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x843e5139, SUC_lexicon_tags_13 },
    { 0x1ede513a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1e5140, SUC_lexicon_tags_15 },
    { 0x6a9e5140, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdade5143, SUC_lexicon_tags_78 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25de5149, SUC_lexicon_tags_2 },
    { 0xa7de514a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x943e514d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105e5153, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ede5158, SUC_lexicon_tags_37 },
    { 0xbb1e5159, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfe515d, SUC_lexicon_tags_14 },
    { 0x341e515d, SUC_lexicon_tags_2 },
    { 0x18fe515e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3de5161, SUC_lexicon_tags_17 },
    { 0x4bbe5162, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973e5168, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x321e516a, SUC_lexicon_tags_11 },
    { 0xa89e516a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x507e516e, SUC_lexicon_tags_17 },
    { 0x4afe516e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21e5175, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85e5180, SUC_lexicon_tags_7 },
    { 0xe31e5180, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf25e5183, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x935e5185, SUC_lexicon_tags_7 },
    { 0x513e5185, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x56be5189, SUC_lexicon_tags_37 },
    { 0x1c7e518a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219e518f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a5e5191, SUC_lexicon_tags_47 },
    { 0x567e5192, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x157e5194, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5e5197, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x163e5199, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba9e519b, SUC_lexicon_tags_12 },
    { 0xf53e519c, SUC_lexicon_tags_26 },
    { 0x603e519d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31e51a3, SUC_lexicon_tags_8 },
    { 0x423e51a4, SUC_lexicon_tags_37 },
    { 0x14fe51a5, SUC_lexicon_tags_25 },
    { 0x503e51a6, SUC_lexicon_tags_11 },
    { 0x3b1e51a6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295e51ac, SUC_lexicon_tags_12 },
    { 0x4a9e51ac, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e51b0, SUC_lexicon_tags_12 },
    { 0xd5de51b1, SUC_lexicon_tags_11 },
    { 0x6e1e51b2, SUC_lexicon_tags_61 },
    { 0x7afe51b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe51b8, SUC_lexicon_tags_11 },
    { 0xe31e51b9, SUC_lexicon_tags_30 },
    { 0xcede51ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x193e51bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5e51c1, SUC_lexicon_tags_25 },
    { 0x9dbe51c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75e51c7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd63e51c9, SUC_lexicon_tags_17 },
    { 0xcd9e51c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71e51ce, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9e51d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x503e51d4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4de51d6, SUC_lexicon_tags_1 },
    { 0xca9e51d7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4c9e51d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05e51dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x8d7e51de, SUC_lexicon_tags_4 },
    { 0xbcde51de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48be51e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835e51e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x981e51eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x297e51ed, SUC_lexicon_tags_25 },
    { 0xdbe51ed, SUC_lexicon_tags_318 },
    { 0xd1e51ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7de51f2, SUC_lexicon_tags_26 },
    { 0xbbde51f2, SUC_lexicon_tags_11 },
    { 0x953e51f2, SUC_lexicon_tags_7 },
    { 0xa95e51f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15e51fe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91e5202, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3e5205, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31de5209, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf3de520b, SUC_lexicon_tags_6 },
    { 0xed1e520b, SUC_lexicon_tags_37 },
    { 0xc65e520d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117e5217, SUC_lexicon_tags_38 },
    { 0xd99e5218, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31e521b, SUC_lexicon_tags_11 },
    { 0x2c1e521b, SUC_lexicon_tags_17 },
    { 0x50de521c, SUC_lexicon_tags_37 },
    { 0x801e521e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xda3e5221, SUC_lexicon_tags_0 },
    { 0xae5e5222, SUC_lexicon_tags_47 },
    { 0x557e5223, SUC_lexicon_tags_2 },
    { 0x7c5e5224, SUC_lexicon_tags_12 },
    { 0xe33e5225, SUC_lexicon_tags_10 },
    { 0x96fe5224, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abe522c, SUC_lexicon_tags_2 },
    { 0xbfde522d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5231, SUC_lexicon_tags_33 },
    { 0xabde5232, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x749e5235, SUC_lexicon_tags_7 },
    { 0x349e5236, SUC_lexicon_tags_12 },
    { 0x297e5237, SUC_lexicon_tags_23 },
    { 0xee9e5238, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x249e523b, SUC_lexicon_tags_1 },
    { 0x645e523c, SUC_lexicon_tags_17 },
    { 0x405e523d, SUC_lexicon_tags_11 },
    { 0x4e7e523d, SUC_lexicon_tags_33 },
    { 0x6f1e523d, SUC_lexicon_tags_32 },
    { 0x33e523f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70be5246, SUC_lexicon_tags_25 },
    { 0x529e5246, SUC_lexicon_tags_66 },
    { 0x787e5246, SUC_lexicon_tags_17 },
    { 0x39fe5248, SUC_lexicon_tags_37 },
    { 0x9cde5249, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83e524d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73fe524f, SUC_lexicon_tags_15 },
    { 0xe3e524f, SUC_lexicon_tags_1 },
    { 0x58be5250, SUC_lexicon_tags_11 },
    { 0xa51e5252, SUC_lexicon_tags_37 },
    { 0x733e5253, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25e525a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7be525c, SUC_lexicon_tags_29 },
    { 0x27be525d, SUC_lexicon_tags_40 },
    { 0xa1be525e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x549e5261, SUC_lexicon_tags_47 },
    { 0x3d3e5261, SUC_lexicon_tags_1 },
    { 0xd25e5263, SUC_lexicon_tags_25 },
    { 0xddde5264, SUC_lexicon_tags_15 },
    { 0x783e5264, SUC_lexicon_tags_12 },
    { 0x45de5264, SUC_lexicon_tags_46 },
    { 0x811e5261, SUC_lexicon_tags_17 },
    { 0x223e5268, SUC_lexicon_tags_15 },
    { 0xeb9e5261, SUC_lexicon_tags_38 },
    { 0x2e7e5266, SUC_lexicon_tags_33 },
    { 0x10fe5266, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0x143e526d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x49be526f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7e5272, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89e5278, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fde527b, SUC_lexicon_tags_37 },
    { 0x9d9e527c, SUC_lexicon_tags_1 },
    { 0x971e527c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3e5280, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd67e5282, SUC_lexicon_tags_25 },
    { 0x445e5283, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbe528a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1de5291, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x155e5293, SUC_lexicon_tags_31 },
    { 0x487e5293, SUC_lexicon_tags_11 },
    { 0xf4de5295, SUC_lexicon_tags_7 },
    { 0x9f5e5294, SUC_lexicon_tags_15 },
    { 0x491e5297, SUC_lexicon_tags_17 },
    { 0x493e5298, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x527e529a, SUC_lexicon_tags_25 },
    { 0xa25e529b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933e52a2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x19fe52a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155e52ab, SUC_lexicon_tags_30 },
    { 0x19e52ab, SUC_lexicon_tags_37 },
    { 0xfa7e52ad, SUC_lexicon_tags_28 },
    { 0x65e52ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865e52b4, SUC_lexicon_tags_23 },
    { 0x965e52b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdbe52b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc1e52b9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x53be52bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x991e52bf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb91e52c1, SUC_lexicon_tags_37 },
    { 0xff9e52c2, SUC_lexicon_tags_38 },
    { 0x7fe52c3, SUC_lexicon_tags_15 },
    { 0x97de52c2, SUC_lexicon_tags_17 },
    { 0x7ade52c5, SUC_lexicon_tags_37 },
    { 0xeafe52c5, SUC_lexicon_tags_8 },
    { 0x95fe52c7, SUC_lexicon_tags_2 },
    { 0xbc3e52c7, SUC_lexicon_tags_58 },
    { 0x5cde52c9, SUC_lexicon_tags_47 },
    { 0xb01e52c8, SUC_lexicon_tags_23 },
    { 0xe71e52c7, SUC_lexicon_tags_1 },
    { 0x9bbe52cc, SUC_lexicon_tags_17 },
    { 0x593e52c5, SUC_lexicon_tags_7 },
    { 0x7a1e52c9, SUC_lexicon_tags_1 },
    { 0xf6de52cf, SUC_lexicon_tags_46 },
    { 0xac1e52cf, SUC_lexicon_tags_25 },
    { 0x6dde52cf, SUC_lexicon_tags_49 },
    { 0x3cfe52d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcafe52d5, SUC_lexicon_tags_182 },
    { 0x99e52d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x435e52d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x371e52dc, SUC_lexicon_tags_17 },
    { 0x3e52dd, SUC_lexicon_tags_7 },
    { 0xdd1e52dd, SUC_lexicon_tags_12 },
    { 0x70fe52de, SUC_lexicon_tags_17 },
    { 0xbe9e52dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999e52e5, SUC_lexicon_tags_17 },
    { 0xc9be52e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5e52ea, SUC_lexicon_tags_34 },
    { 0xb53e52eb, SUC_lexicon_tags_11 },
    { 0x9f5e52ea, SUC_lexicon_tags_14 },
    { 0x1d3e52ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97e52f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x587e52f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa7e52f4, SUC_lexicon_tags_21 },
    { 0xe65e52f5, SUC_lexicon_tags_29 },
    { 0xd3de52f6, SUC_lexicon_tags_25 },
    { 0x18fe52f7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x24de52fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x45de52fc, SUC_lexicon_tags_15 },
    { 0x63e52fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x53e5300, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcde5303, SUC_lexicon_tags_433 },
    { 0x8fbe5304, SUC_lexicon_tags_41 },
    { 0xac1e5305, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d5e5307, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x99be5309, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x8c3e530b, SUC_lexicon_tags_17 },
    { 0x455e530c, SUC_lexicon_tags_26 },
    { 0xa8be530d, SUC_lexicon_tags_27 },
    { 0xf6be530e, SUC_lexicon_tags_26 },
    { 0x297e530e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0be5315, SUC_lexicon_tags_9 },
    { 0x3de5316, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7e5318, SUC_lexicon_tags_1 },
    { 0x5c3e5318, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0xa55e531b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e5320, SUC_lexicon_tags_12 },
    { 0x695e5321, SUC_lexicon_tags_45 },
    { 0x22de5322, SUC_lexicon_tags_37 },
    { 0xed3e5323, SUC_lexicon_tags_49 },
    { 0x6a1e5323, SUC_lexicon_tags_45 },
    { 0x997e5325, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27e5327, SUC_lexicon_tags_17 },
    { 0x63e5327, SUC_lexicon_tags_87 },
    { 0x375e5329, SUC_lexicon_tags_29 },
    { 0x727e532a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fe532f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa7fe5331, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x231e5334, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x927e5337, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e533a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x197e533d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbe5342, SUC_lexicon_tags_11 },
    { 0x151e5343, SUC_lexicon_tags_45 },
    { 0xd65e5344, SUC_lexicon_tags_37 },
    { 0xfe5e5343, SUC_lexicon_tags_47 },
    { 0xf43e5346, SUC_lexicon_tags_40 },
    { 0x5ede5346, SUC_lexicon_tags_14 },
    { 0x16be5348, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbe534d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581e5351, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5e5356, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x329e5359, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaa1e535b, SUC_lexicon_tags_12 },
    { 0x27e535b, SUC_lexicon_tags_13 },
    { 0x461e535b, SUC_lexicon_tags_97 },
    { 0x38de535c, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89e5362, SUC_lexicon_tags_7 },
    { 0xcb3e5362, SUC_lexicon_tags_47 },
    { 0x27be5362, SUC_lexicon_tags_37 },
    { 0xd5e5365, SUC_lexicon_tags_85 },
    { 0xe0de5366, SUC_lexicon_tags_19 },
    { 0x2a9e5364, SUC_lexicon_tags_17 },
    { 0xc89e5368, SUC_lexicon_tags_37 },
    { 0x877e5362, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03e536e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989e5374, SUC_lexicon_tags_6 },
    { 0x23e5375, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbf5e5377, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fe537a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b7e537c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf8be537e, SUC_lexicon_tags_12 },
    { 0x223e537e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x233e5382, SUC_lexicon_tags_17 },
    { 0xd8be5383, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfe5387, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85e538d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1e5392, SUC_lexicon_tags_17 },
    { 0x7a9e5393, SUC_lexicon_tags_10 },
    { 0xfe5392, SUC_lexicon_tags_23 },
    { 0xd9e5395, SUC_lexicon_tags_7 },
    { 0xaf9e5396, SUC_lexicon_tags_25 },
    { 0x5e5e5396, SUC_lexicon_tags_17 },
    { 0x2f1e5398, SUC_lexicon_tags_38 },
    { 0xc7e5392, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x769e539b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1e539e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1e53a4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ffe53a6, SUC_lexicon_tags_7 },
    { 0x2ffe53a6, SUC_lexicon_tags_11 },
    { 0x1d7e53a8, SUC_lexicon_tags_37 },
    { 0x4f7e53a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dde53ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f1e53ae, SUC_lexicon_tags_47 },
    { 0xe57e53af, SUC_lexicon_tags_207 },
    { 0x571e53b0, SUC_lexicon_tags_25 },
    { 0xe01e53b1, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45e53b5, SUC_lexicon_tags_17 },
    { 0x56fe53b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7e53b9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x5cbe53bb, SUC_lexicon_tags_1299 },
    { 0, NULL },
    { 0x5ebe53bd, SUC_lexicon_tags_14 },
    { 0x261e53bd, SUC_lexicon_tags_15 },
    { 0x157e53bf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe53c2, SUC_lexicon_tags_17 },
    { 0x137e53c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x651e53c6, SUC_lexicon_tags_17 },
    { 0x533e53c7, SUC_lexicon_tags_253 },
    { 0x5c7e53c6, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71e53d1, SUC_lexicon_tags_12 },
    { 0x195e53d1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x18de53d5, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5e53da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa03e53dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfe53e4, SUC_lexicon_tags_2 },
    { 0x223e53e4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfe53e8, SUC_lexicon_tags_43 },
    { 0x943e53e8, SUC_lexicon_tags_15 },
    { 0x7fbe53ea, SUC_lexicon_tags_25 },
    { 0xdfe53e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103e53f0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5e53f3, SUC_lexicon_tags_12 },
    { 0xa7fe53f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8ebe53f6, SUC_lexicon_tags_21 },
    { 0xe57e53f7, SUC_lexicon_tags_33 },
    { 0x7abe53f8, SUC_lexicon_tags_17 },
    { 0x8d9e53f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbe53fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1e5401, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x741e5404, SUC_lexicon_tags_23 },
    { 0x509e5405, SUC_lexicon_tags_11 },
    { 0xd77e5404, SUC_lexicon_tags_17 },
    { 0x2f1e5407, SUC_lexicon_tags_46 },
    { 0xb5be5407, SUC_lexicon_tags_38 },
    { 0x913e5409, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc7de540b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57e5411, SUC_lexicon_tags_19 },
    { 0xe7fe5412, SUC_lexicon_tags_25 },
    { 0xa0de5411, SUC_lexicon_tags_60 },
    { 0x39e5414, SUC_lexicon_tags_25 },
    { 0x4e3e5414, SUC_lexicon_tags_15 },
    { 0x40be5416, SUC_lexicon_tags_37 },
    { 0x877e5414, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4be541c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37e541f, SUC_lexicon_tags_30 },
    { 0xab7e5420, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7e5426, SUC_lexicon_tags_17 },
    { 0x881e5427, SUC_lexicon_tags_37 },
    { 0x53de5428, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4c9e542a, SUC_lexicon_tags_43 },
    { 0xf15e542b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3e542e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x975e5430, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fe5434, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9be5437, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbe543d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1c1e543f, SUC_lexicon_tags_25 },
    { 0xd01e5440, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83e5443, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x91e5446, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe544b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13be544d, SUC_lexicon_tags_15 },
    { 0x841e544e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09e5456, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x509e5458, SUC_lexicon_tags_28 },
    { 0xb99e5458, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x531e545b, SUC_lexicon_tags_37 },
    { 0x843e545b, SUC_lexicon_tags_32 },
    { 0x589e545d, SUC_lexicon_tags_29 },
    { 0xe47e545b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x831e5460, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x67be5462, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x927e5465, SUC_lexicon_tags_15 },
    { 0xabbe5466, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783e546a, SUC_lexicon_tags_11 },
    { 0x38be546b, SUC_lexicon_tags_151 },
    { 0x1ade546b, SUC_lexicon_tags_62 },
    { 0x831e546d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879e5472, SUC_lexicon_tags_1 },
    { 0xc79e5473, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abe5479, SUC_lexicon_tags_11 },
    { 0x587e547a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x31e547c, SUC_lexicon_tags_33 },
    { 0x101e547d, SUC_lexicon_tags_42 },
    { 0x1ede547e, SUC_lexicon_tags_25 },
    { 0x8e1e547e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1e5487, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9e548a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbe548d, SUC_lexicon_tags_2 },
    { 0x1c9e548e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x179e5490, SUC_lexicon_tags_27 },
    { 0x4ebe5490, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x24fe5493, SUC_lexicon_tags_7 },
    { 0x35be5494, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x985e5496, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74be54a2, SUC_lexicon_tags_47 },
    { 0x437e54a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbe54a9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241e54b0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1e54b4, SUC_lexicon_tags_16 },
    { 0x491e54b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851e54bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5b3e54bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa1de54c1, SUC_lexicon_tags_27 },
    { 0xab5e54c1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9e54c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x957e54c7, SUC_lexicon_tags_82 },
    { 0xbc1e54c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x897e54ca, SUC_lexicon_tags_16 },
    { 0x5f5e54ca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe54d0, SUC_lexicon_tags_0 },
    { 0xfe54d1, SUC_lexicon_tags_17 },
    { 0x63fe54d1, SUC_lexicon_tags_45 },
    { 0x327e54d3, SUC_lexicon_tags_1 },
    { 0x78be54d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5e54db, SUC_lexicon_tags_11 },
    { 0x949e54dc, SUC_lexicon_tags_19 },
    { 0x3ade54dd, SUC_lexicon_tags_14 },
    { 0x4a3e54db, SUC_lexicon_tags_1 },
    { 0x9afe54df, SUC_lexicon_tags_17 },
    { 0x9c3e54dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7e54e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbe54e9, SUC_lexicon_tags_0 },
    { 0x7ede54e9, SUC_lexicon_tags_298 },
    { 0xbd3e54eb, SUC_lexicon_tags_17 },
    { 0x803e54ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5efe54ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bde54f3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb03e54f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x989e54f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23e54fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x281e54fd, SUC_lexicon_tags_94 },
    { 0xb0be54fe, SUC_lexicon_tags_2 },
    { 0x5d3e54fd, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x53de5502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfe5505, SUC_lexicon_tags_2 },
    { 0x52de5506, SUC_lexicon_tags_82 },
    { 0xa75e5506, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561e550b, SUC_lexicon_tags_1 },
    { 0xa8fe550b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7e550f, SUC_lexicon_tags_63 },
    { 0xd9be5510, SUC_lexicon_tags_23 },
    { 0x609e5510, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25e5515, SUC_lexicon_tags_47 },
    { 0x79be5515, SUC_lexicon_tags_21 },
    { 0xa77e5517, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x75fe5519, SUC_lexicon_tags_32 },
    { 0xc53e551a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb2fe551c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6d5e551e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x11be5520, SUC_lexicon_tags_1 },
    { 0x80fe5521, SUC_lexicon_tags_136 },
    { 0xca3e5520, SUC_lexicon_tags_14 },
    { 0xa67e5521, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fe5529, SUC_lexicon_tags_19 },
    { 0xf19e5529, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x78be552d, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3e5534, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3e553f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3be5543, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbe5547, SUC_lexicon_tags_19 },
    { 0x905e5547, SUC_lexicon_tags_37 },
    { 0xbfde5549, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x749e554c, SUC_lexicon_tags_37 },
    { 0x38be554d, SUC_lexicon_tags_61 },
    { 0x3c9e554c, SUC_lexicon_tags_74 },
    { 0x8b3e554c, SUC_lexicon_tags_90 },
    { 0xa0de554f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51de5552, SUC_lexicon_tags_2 },
    { 0x25be5553, SUC_lexicon_tags_12 },
    { 0x17de5552, SUC_lexicon_tags_2 },
    { 0xf73e5555, SUC_lexicon_tags_12 },
    { 0xa9fe5555, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75de555f, SUC_lexicon_tags_439 },
    { 0x135e555f, SUC_lexicon_tags_9 },
    { 0xbfbe5561, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9afe5563, SUC_lexicon_tags_17 },
    { 0x387e5563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bfe5566, SUC_lexicon_tags_2 },
    { 0xe31e5566, SUC_lexicon_tags_0 },
    { 0x707e5568, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x675e556b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fe556f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x653e5572, SUC_lexicon_tags_30 },
    { 0x45fe5573, SUC_lexicon_tags_14 },
    { 0xdcde5573, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb23e5576, SUC_lexicon_tags_82 },
    { 0xd33e5576, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x373e5579, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95e557d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcf3e557f, SUC_lexicon_tags_8 },
    { 0xb27e557f, SUC_lexicon_tags_87 },
    { 0x839e5581, SUC_lexicon_tags_10 },
    { 0xa11e5581, SUC_lexicon_tags_91 },
    { 0xbcde557f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf7de5585, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83e558c, SUC_lexicon_tags_17 },
    { 0xc33e558c, SUC_lexicon_tags_26 },
    { 0xdc3e558c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x219e5591, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663e5597, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfe559b, SUC_lexicon_tags_27 },
    { 0xa87e559c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cde559f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efe55a3, SUC_lexicon_tags_7 },
    { 0x11fe55a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6a7e55a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dde55aa, SUC_lexicon_tags_38 },
    { 0x78fe55ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe51e55ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abe55b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801e55ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95e55bf, SUC_lexicon_tags_2 },
    { 0xf0de55bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0be55c5, SUC_lexicon_tags_17 },
    { 0xddfe55c6, SUC_lexicon_tags_15 },
    { 0xeb3e55c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efe55cc, SUC_lexicon_tags_1 },
    { 0x14de55cd, SUC_lexicon_tags_55 },
    { 0x27fe55cd, SUC_lexicon_tags_2 },
    { 0x415e55cd, SUC_lexicon_tags_37 },
    { 0x675e55cd, SUC_lexicon_tags_19 },
    { 0x159e55d1, SUC_lexicon_tags_23 },
    { 0xc81e55d2, SUC_lexicon_tags_17 },
    { 0xaabe55d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x697e55d6, SUC_lexicon_tags_11 },
    { 0xc6de55d7, SUC_lexicon_tags_27 },
    { 0xc4fe55d7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4be55dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x89be55df, SUC_lexicon_tags_133 },
    { 0xafbe55df, SUC_lexicon_tags_0 },
    { 0x145e55e1, SUC_lexicon_tags_27 },
    { 0x7d3e55e2, SUC_lexicon_tags_28 },
    { 0xf7fe55e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243e55e9, SUC_lexicon_tags_23 },
    { 0xec1e55ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfe55f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4de55f6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7b1e55f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x549e55fb, SUC_lexicon_tags_35 },
    { 0x4d5e55fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xede55fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbe5601, SUC_lexicon_tags_27 },
    { 0x279e5602, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x559e5604, SUC_lexicon_tags_118 },
    { 0x933e5605, SUC_lexicon_tags_37 },
    { 0xe3e5606, SUC_lexicon_tags_63 },
    { 0xecfe5606, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01e560b, SUC_lexicon_tags_80 },
    { 0x753e560c, SUC_lexicon_tags_56 },
    { 0xf15e560d, SUC_lexicon_tags_133 },
    { 0x919e560d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377e5612, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d7e5614, SUC_lexicon_tags_37 },
    { 0xdc3e5614, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60be5618, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9de561d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x239e561f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf27e5621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8ebe5623, SUC_lexicon_tags_2 },
    { 0xc4be5624, SUC_lexicon_tags_25 },
    { 0x7cfe5625, SUC_lexicon_tags_14 },
    { 0xcdfe5626, SUC_lexicon_tags_1 },
    { 0x7e1e5626, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1e562a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71e562d, SUC_lexicon_tags_15 },
    { 0x33be562e, SUC_lexicon_tags_49 },
    { 0x823e562f, SUC_lexicon_tags_3 },
    { 0x271e5630, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72de5634, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39de5638, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x361e563b, SUC_lexicon_tags_17 },
    { 0xe4de563b, SUC_lexicon_tags_13 },
    { 0xf8fe563d, SUC_lexicon_tags_4 },
    { 0x281e563e, SUC_lexicon_tags_59 },
    { 0x7e9e563d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193e564a, SUC_lexicon_tags_0 },
    { 0x573e564a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781e5654, SUC_lexicon_tags_12 },
    { 0x12de5654, SUC_lexicon_tags_14 },
    { 0xd73e5654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685e565d, SUC_lexicon_tags_2 },
    { 0xcf1e565e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4be5661, SUC_lexicon_tags_11 },
    { 0x79be5662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9e5664, SUC_lexicon_tags_23 },
    { 0x3c1e5664, SUC_lexicon_tags_32 },
    { 0xff3e5666, SUC_lexicon_tags_29 },
    { 0x293e5666, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabe566d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fe5672, SUC_lexicon_tags_47 },
    { 0xbc9e5673, SUC_lexicon_tags_37 },
    { 0xb63e5673, SUC_lexicon_tags_8 },
    { 0xf9fe5675, SUC_lexicon_tags_0 },
    { 0x88fe5675, SUC_lexicon_tags_46 },
    { 0x745e5677, SUC_lexicon_tags_17 },
    { 0x5bbe5672, SUC_lexicon_tags_25 },
    { 0x8d7e5674, SUC_lexicon_tags_30 },
    { 0x70de5678, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131e5684, SUC_lexicon_tags_17 },
    { 0xf45e5685, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49e568a, SUC_lexicon_tags_419 },
    { 0xccfe568b, SUC_lexicon_tags_23 },
    { 0xc99e568c, SUC_lexicon_tags_24 },
    { 0x77de568c, SUC_lexicon_tags_12 },
    { 0x3d1e568e, SUC_lexicon_tags_4 },
    { 0xa6fe568f, SUC_lexicon_tags_25 },
    { 0x2b9e568d, SUC_lexicon_tags_29 },
    { 0x103e5691, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe5695, SUC_lexicon_tags_11 },
    { 0x5afe5696, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9fe5698, SUC_lexicon_tags_60 },
    { 0x987e5699, SUC_lexicon_tags_37 },
    { 0xc93e569a, SUC_lexicon_tags_27 },
    { 0x4e7e569b, SUC_lexicon_tags_10 },
    { 0x82de569b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x731e569e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x73de56a1, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbe56a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x573e56a6, SUC_lexicon_tags_7 },
    { 0xe13e56a7, SUC_lexicon_tags_25 },
    { 0x6d3e56a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1e56ab, SUC_lexicon_tags_7 },
    { 0x4f3e56ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbe56b2, SUC_lexicon_tags_12 },
    { 0x589e56b3, SUC_lexicon_tags_19 },
    { 0xb63e56b4, SUC_lexicon_tags_46 },
    { 0x5b3e56b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x121e56b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x301e56b9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f9e56bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31e56be, SUC_lexicon_tags_16 },
    { 0x295e56bf, SUC_lexicon_tags_22 },
    { 0x6be56c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127e56c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x149e56ca, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x2e7e56cc, SUC_lexicon_tags_2 },
    { 0x885e56cd, SUC_lexicon_tags_11 },
    { 0xe63e56cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75be56d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347e56da, SUC_lexicon_tags_31 },
    { 0x33be56db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1e56de, SUC_lexicon_tags_6 },
    { 0xb5fe56df, SUC_lexicon_tags_2 },
    { 0x453e56e0, SUC_lexicon_tags_7 },
    { 0x2dbe56e0, SUC_lexicon_tags_60 },
    { 0x4c1e56e2, SUC_lexicon_tags_0 },
    { 0x37fe56e3, SUC_lexicon_tags_26 },
    { 0xfcfe56e4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4be56e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133e56ec, SUC_lexicon_tags_25 },
    { 0x8e3e56ec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55e56f1, SUC_lexicon_tags_28 },
    { 0x42be56f2, SUC_lexicon_tags_15 },
    { 0xdf7e56f1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbe56f6, SUC_lexicon_tags_6 },
    { 0x57fe56f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x611e56f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839e56fd, SUC_lexicon_tags_37 },
    { 0x383e56fe, SUC_lexicon_tags_37 },
    { 0xcc9e56ff, SUC_lexicon_tags_61 },
    { 0x63de5700, SUC_lexicon_tags_27 },
    { 0x459e56fd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fe5704, SUC_lexicon_tags_2 },
    { 0xb4de5705, SUC_lexicon_tags_66 },
    { 0x4a3e5705, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7e570b, SUC_lexicon_tags_32 },
    { 0xe73e570b, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72be5712, SUC_lexicon_tags_2 },
    { 0x193e5712, SUC_lexicon_tags_19 },
    { 0xef9e5714, SUC_lexicon_tags_37 },
    { 0x79e5714, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353e5719, SUC_lexicon_tags_87 },
    { 0xf27e571a, SUC_lexicon_tags_45 },
    { 0x499e571a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ede5720, SUC_lexicon_tags_15 },
    { 0xeabe5721, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x75be5723, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371e572a, SUC_lexicon_tags_61 },
    { 0x88fe572b, SUC_lexicon_tags_82 },
    { 0xf9e572c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253e5731, SUC_lexicon_tags_27 },
    { 0x867e5732, SUC_lexicon_tags_11 },
    { 0x2e1e5733, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x3e5e5735, SUC_lexicon_tags_9 },
    { 0x541e5736, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb83e5738, SUC_lexicon_tags_37 },
    { 0x851e5739, SUC_lexicon_tags_63 },
    { 0x699e573a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfe5740, SUC_lexicon_tags_22 },
    { 0x607e5740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21e5745, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x755e5747, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99e574b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37de5752, SUC_lexicon_tags_47 },
    { 0x359e5752, SUC_lexicon_tags_0 },
    { 0x13de5754, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x141e5757, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x503e5759, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379e575e, SUC_lexicon_tags_11 },
    { 0x83be575e, SUC_lexicon_tags_19 },
    { 0x157e575e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa91e5762, SUC_lexicon_tags_37 },
    { 0xf7de5763, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x929e5765, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291e576a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49e5772, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebe5775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9e577b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x633e577e, SUC_lexicon_tags_21 },
    { 0xeb1e577f, SUC_lexicon_tags_84 },
    { 0xfcbe5780, SUC_lexicon_tags_17 },
    { 0x701e577e, SUC_lexicon_tags_8 },
    { 0x303e5782, SUC_lexicon_tags_25 },
    { 0xfb7e5783, SUC_lexicon_tags_7 },
    { 0x431e577e, SUC_lexicon_tags_26 },
    { 0x207e5783, SUC_lexicon_tags_25 },
    { 0xffde5783, SUC_lexicon_tags_37 },
    { 0x87de5782, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3bfe5789, SUC_lexicon_tags_31 },
    { 0x637e578a, SUC_lexicon_tags_91 },
    { 0x3a3e578b, SUC_lexicon_tags_37 },
    { 0xedfe5789, SUC_lexicon_tags_33 },
    { 0x393e578d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45e5790, SUC_lexicon_tags_87 },
    { 0x3c9e5791, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa73e5793, SUC_lexicon_tags_55 },
    { 0xb49e5794, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fe5798, SUC_lexicon_tags_38 },
    { 0xabbe5799, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fe57a0, SUC_lexicon_tags_11 },
    { 0xc23e57a1, SUC_lexicon_tags_47 },
    { 0x585e57a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7be57a4, SUC_lexicon_tags_11 },
    { 0xf21e57a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa81e57a7, SUC_lexicon_tags_31 },
    { 0x2b5e57a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7e57ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6abe57af, SUC_lexicon_tags_3 },
    { 0x7b9e57af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfdbe57b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51e57b5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffe57b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395e57bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x271e57be, SUC_lexicon_tags_49 },
    { 0xd99e57be, SUC_lexicon_tags_37 },
    { 0x2d5e57be, SUC_lexicon_tags_12 },
    { 0xafe57be, SUC_lexicon_tags_47 },
    { 0xfcde57c2, SUC_lexicon_tags_63 },
    { 0xa7e57bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97e57c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abe57cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66de57d0, SUC_lexicon_tags_15 },
    { 0xc5be57d0, SUC_lexicon_tags_12 },
    { 0xd9de57d2, SUC_lexicon_tags_37 },
    { 0x149e57d3, SUC_lexicon_tags_27 },
    { 0x2bbe57d4, SUC_lexicon_tags_7 },
    { 0x141e57d5, SUC_lexicon_tags_30 },
    { 0x50be57d6, SUC_lexicon_tags_11 },
    { 0x89de57d4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1e57da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e57df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfe57e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4b1e57e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80be57e8, SUC_lexicon_tags_87 },
    { 0xb4be57e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbe57ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x24be57ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5e57f2, SUC_lexicon_tags_25 },
    { 0x3b9e57f3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xaabe57f5, SUC_lexicon_tags_17 },
    { 0xfefe57f5, SUC_lexicon_tags_25 },
    { 0xb9fe57f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afe57fa, SUC_lexicon_tags_25 },
    { 0x8d5e57fa, SUC_lexicon_tags_14 },
    { 0x2f3e57fc, SUC_lexicon_tags_11 },
    { 0x497e57fb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1e5800, SUC_lexicon_tags_11 },
    { 0xc1e5801, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79e5803, SUC_lexicon_tags_11 },
    { 0x4f5e5803, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfe7e5806, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89e580a, SUC_lexicon_tags_49 },
    { 0x6cbe580b, SUC_lexicon_tags_47 },
    { 0x27de580a, SUC_lexicon_tags_21 },
    { 0x33e580d, SUC_lexicon_tags_8 },
    { 0x3c5e580b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8be5813, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f7e5815, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbe581a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd41e581c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x315e581e, SUC_lexicon_tags_68 },
    { 0xa95e581f, SUC_lexicon_tags_324 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887e5823, SUC_lexicon_tags_12 },
    { 0x639e5823, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69fe5826, SUC_lexicon_tags_12 },
    { 0xc4be5827, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7e582a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x36fe582c, SUC_lexicon_tags_1648 },
    { 0, NULL },
    { 0xe73e582e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09e5831, SUC_lexicon_tags_21 },
    { 0x4a1e5832, SUC_lexicon_tags_37 },
    { 0xcd9e5833, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dde5836, SUC_lexicon_tags_47 },
    { 0xdede5837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x48be5839, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbe583c, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909e5840, SUC_lexicon_tags_15 },
    { 0x8c5e5841, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91e5847, SUC_lexicon_tags_15 },
    { 0xb2fe5848, SUC_lexicon_tags_2 },
    { 0x477e5849, SUC_lexicon_tags_15 },
    { 0x609e584a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe97e584c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x213e584f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfe5855, SUC_lexicon_tags_47 },
    { 0xe5de5856, SUC_lexicon_tags_19 },
    { 0x9b7e5856, SUC_lexicon_tags_68 },
    { 0x94de5858, SUC_lexicon_tags_11 },
    { 0x515e5855, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397e5860, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9e5863, SUC_lexicon_tags_14 },
    { 0xb0be5864, SUC_lexicon_tags_43 },
    { 0x83e5864, SUC_lexicon_tags_14 },
    { 0xc8de5865, SUC_lexicon_tags_1 },
    { 0xfade5866, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc75e5869, SUC_lexicon_tags_33 },
    { 0x40de586a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x237e586c, SUC_lexicon_tags_2 },
    { 0x84fe586d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa15e586f, SUC_lexicon_tags_23 },
    { 0xf2de5870, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75e5874, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x331e5876, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87be587a, SUC_lexicon_tags_25 },
    { 0xb93e587b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x911e587e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x977e5880, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41fe5882, SUC_lexicon_tags_37 },
    { 0xd67e5882, SUC_lexicon_tags_17 },
    { 0x90be5884, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295e588a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb77e588c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x477e588f, SUC_lexicon_tags_7 },
    { 0xf7be5890, SUC_lexicon_tags_61 },
    { 0xec5e5891, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51e5896, SUC_lexicon_tags_37 },
    { 0xb4de5897, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81e589a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3e589d, SUC_lexicon_tags_14 },
    { 0x187e589e, SUC_lexicon_tags_11 },
    { 0x16be589f, SUC_lexicon_tags_13 },
    { 0x513e589f, SUC_lexicon_tags_43 },
    { 0x9d5e589d, SUC_lexicon_tags_2 },
    { 0x6efe58a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x521e58a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7e58aa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x903e58ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xeb1e58ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x68de58b0, SUC_lexicon_tags_27 },
    { 0xcf1e58b1, SUC_lexicon_tags_25 },
    { 0xe35e58b2, SUC_lexicon_tags_12 },
    { 0x51e58b1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e58b6, SUC_lexicon_tags_15 },
    { 0x87be58b7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fe58ba, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xff9e58bc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa79e58bf, SUC_lexicon_tags_1 },
    { 0xa7de58c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63e58c7, SUC_lexicon_tags_0 },
    { 0x7f9e58c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x333e58cb, SUC_lexicon_tags_11 },
    { 0xfcfe58cc, SUC_lexicon_tags_29 },
    { 0x953e58cc, SUC_lexicon_tags_37 },
    { 0xde1e58ce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdcbe58d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01e58d3, SUC_lexicon_tags_15 },
    { 0xa73e58d4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffe58da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9e58df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9e58e5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5b1e58e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c5e58e9, SUC_lexicon_tags_1 },
    { 0x733e58ea, SUC_lexicon_tags_27 },
    { 0x4fde58eb, SUC_lexicon_tags_11 },
    { 0x24fe58eb, SUC_lexicon_tags_7 },
    { 0x3dde58ed, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd7de58ef, SUC_lexicon_tags_11 },
    { 0x147e58f0, SUC_lexicon_tags_5 },
    { 0xbc5e58f1, SUC_lexicon_tags_11 },
    { 0xa13e58f2, SUC_lexicon_tags_17 },
    { 0xc17e58f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x505e58f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68de58fa, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14de5900, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2de590e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x243e5910, SUC_lexicon_tags_152 },
    { 0xc27e5911, SUC_lexicon_tags_2 },
    { 0x133e5912, SUC_lexicon_tags_9 },
    { 0x699e5910, SUC_lexicon_tags_60 },
    { 0xcc7e5911, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x503e5917, SUC_lexicon_tags_22 },
    { 0x30fe5918, SUC_lexicon_tags_29 },
    { 0xb2fe5919, SUC_lexicon_tags_8 },
    { 0x8be5918, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7fbe591c, SUC_lexicon_tags_61 },
    { 0x70be591d, SUC_lexicon_tags_30 },
    { 0x591e591e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe3e5920, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbb3e5922, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa3fe5924, SUC_lexicon_tags_1 },
    { 0x7d3e5924, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x397e5927, SUC_lexicon_tags_33 },
    { 0x471e5928, SUC_lexicon_tags_29 },
    { 0x7d5e5927, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95be592d, SUC_lexicon_tags_6 },
    { 0x459e592e, SUC_lexicon_tags_7 },
    { 0x1ade592f, SUC_lexicon_tags_11 },
    { 0xb93e592e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x297e5933, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7abe5935, SUC_lexicon_tags_87 },
    { 0xa7be5935, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4be5939, SUC_lexicon_tags_37 },
    { 0x8ffe593a, SUC_lexicon_tags_17 },
    { 0xa47e5939, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135e5942, SUC_lexicon_tags_11 },
    { 0x7bde5943, SUC_lexicon_tags_12 },
    { 0x6a1e5943, SUC_lexicon_tags_11 },
    { 0x55fe5943, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7e5948, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x623e594a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x281e594c, SUC_lexicon_tags_49 },
    { 0x87de594c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e5950, SUC_lexicon_tags_1 },
    { 0xfbbe5951, SUC_lexicon_tags_12 },
    { 0x3a3e5951, SUC_lexicon_tags_57 },
    { 0x5a5e5953, SUC_lexicon_tags_19 },
    { 0x44de5950, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5e5961, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57e5967, SUC_lexicon_tags_7 },
    { 0x837e5968, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efe5970, SUC_lexicon_tags_17 },
    { 0x9ade5971, SUC_lexicon_tags_37 },
    { 0x62be5970, SUC_lexicon_tags_37 },
    { 0xeb7e5970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05e5976, SUC_lexicon_tags_23 },
    { 0xf7be5977, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3d7e5979, SUC_lexicon_tags_49 },
    { 0x4abe597a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1e597f, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9e5983, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x2e9e5985, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8de5989, SUC_lexicon_tags_17 },
    { 0x9e3e598a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41e5998, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x575e599a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959e599e, SUC_lexicon_tags_11 },
    { 0x959e599e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fe59a6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbe59a9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbe59b4, SUC_lexicon_tags_15 },
    { 0x7d3e59b5, SUC_lexicon_tags_6 },
    { 0xad9e59b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbe59c0, SUC_lexicon_tags_631 },
    { 0xd85e59c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x287e59c4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497e59c8, SUC_lexicon_tags_11 },
    { 0xd2be59c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609e59cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xba3e59d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x74be59d4, SUC_lexicon_tags_87 },
    { 0x53de59d5, SUC_lexicon_tags_25 },
    { 0x793e59d6, SUC_lexicon_tags_95 },
    { 0x21be59d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87fe59db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8e7e59dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6be59e5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5e59e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbe59eb, SUC_lexicon_tags_37 },
    { 0x967e59ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61e59f0, SUC_lexicon_tags_33 },
    { 0x35fe59f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5e59f4, SUC_lexicon_tags_19 },
    { 0x3bbe59f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x511e59f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcb9e59fa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefe5a00, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fe5a03, SUC_lexicon_tags_11 },
    { 0xa33e5a03, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1c3e5a06, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abe5a0a, SUC_lexicon_tags_2 },
    { 0x35de5a0b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9dbe5a0d, SUC_lexicon_tags_84 },
    { 0xbf9e5a0e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8be5a10, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xb9be5a12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4be5a18, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3e5a1e, SUC_lexicon_tags_17 },
    { 0x261e5a1e, SUC_lexicon_tags_7 },
    { 0x7cfe5a20, SUC_lexicon_tags_48 },
    { 0xd33e5a20, SUC_lexicon_tags_7 },
    { 0x371e5a1e, SUC_lexicon_tags_23 },
    { 0x64be5a23, SUC_lexicon_tags_37 },
    { 0xcd3e5a23, SUC_lexicon_tags_14 },
    { 0x41fe5a24, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbe5a29, SUC_lexicon_tags_2 },
    { 0xe1be5a29, SUC_lexicon_tags_6 },
    { 0x535e5a2a, SUC_lexicon_tags_32 },
    { 0x3f9e5a29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee1e5a2e, SUC_lexicon_tags_15 },
    { 0xb73e5a2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc97e5a31, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5e5a37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8abe5a3a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5e5a3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13be5a3f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3e5a44, SUC_lexicon_tags_28 },
    { 0xdd9e5a45, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abe5a4e, SUC_lexicon_tags_25 },
    { 0xe71e5a4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4cde5a51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dde5a55, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfe5a5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55de5a5c, SUC_lexicon_tags_33 },
    { 0x40de5a5d, SUC_lexicon_tags_37 },
    { 0x3cfe5a5d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0x8ffe5a60, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x36de5a62, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x985e5a65, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x881e5a67, SUC_lexicon_tags_108 },
    { 0x207e5a68, SUC_lexicon_tags_46 },
    { 0x709e5a69, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5e5a6c, SUC_lexicon_tags_1 },
    { 0xec7e5a6c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x86de5a6f, SUC_lexicon_tags_33 },
    { 0x6e5e5a70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67e5a76, SUC_lexicon_tags_43 },
    { 0x3b9e5a77, SUC_lexicon_tags_31 },
    { 0xa6fe5a76, SUC_lexicon_tags_43 },
    { 0xf0be5a78, SUC_lexicon_tags_0 },
    { 0x9a9e5a7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x225e5a7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9e5a81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x293e5a84, SUC_lexicon_tags_10 },
    { 0x31fe5a85, SUC_lexicon_tags_15 },
    { 0x1fde5a84, SUC_lexicon_tags_138 },
    { 0x8a3e5a87, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99e5a8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x129e5a8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61e5a93, SUC_lexicon_tags_63 },
    { 0xa57e5a93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ede5a98, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07e5a9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbe5aa2, SUC_lexicon_tags_28 },
    { 0xa5be5aa3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe5aa7, SUC_lexicon_tags_16 },
    { 0xe83e5aa8, SUC_lexicon_tags_48 },
    { 0x9b5e5aa9, SUC_lexicon_tags_37 },
    { 0x707e5aaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdebe5ab1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x28fe5ab3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd8de5ab5, SUC_lexicon_tags_2 },
    { 0x233e5ab6, SUC_lexicon_tags_42 },
    { 0x3ffe5ab6, SUC_lexicon_tags_2 },
    { 0x945e5ab8, SUC_lexicon_tags_25 },
    { 0xf3de5ab9, SUC_lexicon_tags_11 },
    { 0x7cbe5aba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb9fe5abc, SUC_lexicon_tags_84 },
    { 0x25fe5abc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x835e5ac0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5e5ac3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x333e5ac6, SUC_lexicon_tags_15 },
    { 0xd23e5ac7, SUC_lexicon_tags_136 },
    { 0x59e5ac8, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfe5acb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf1fe5acd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbe5ad0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609e5ad4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41e5ad9, SUC_lexicon_tags_45 },
    { 0xb21e5ad9, SUC_lexicon_tags_21 },
    { 0x1fbe5adb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25e5adf, SUC_lexicon_tags_60 },
    { 0x1ffe5ae0, SUC_lexicon_tags_17 },
    { 0x5d1e5ae1, SUC_lexicon_tags_11 },
    { 0x2c7e5ae0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x185e5ae4, SUC_lexicon_tags_0 },
    { 0x743e5ae4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0be5ae9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x303e5aec, SUC_lexicon_tags_46 },
    { 0x14fe5aed, SUC_lexicon_tags_25 },
    { 0x29be5aee, SUC_lexicon_tags_47 },
    { 0xf17e5aed, SUC_lexicon_tags_33 },
    { 0x18be5af0, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xec9e5af2, SUC_lexicon_tags_37 },
    { 0xca9e5af2, SUC_lexicon_tags_13 },
    { 0x58be5af4, SUC_lexicon_tags_38 },
    { 0x38fe5af5, SUC_lexicon_tags_29 },
    { 0xa1be5af4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17e5afb, SUC_lexicon_tags_30 },
    { 0x23de5afc, SUC_lexicon_tags_60 },
    { 0x293e5afd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fe5b01, SUC_lexicon_tags_27 },
    { 0xf8be5b02, SUC_lexicon_tags_12 },
    { 0xa45e5b03, SUC_lexicon_tags_1 },
    { 0xbbe5b04, SUC_lexicon_tags_37 },
    { 0xfb9e5b04, SUC_lexicon_tags_2 },
    { 0xc55e5b01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x65be5b08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fe5b12, SUC_lexicon_tags_37 },
    { 0x555e5b12, SUC_lexicon_tags_11 },
    { 0x891e5b14, SUC_lexicon_tags_63 },
    { 0xd3de5b15, SUC_lexicon_tags_49 },
    { 0x19fe5b14, SUC_lexicon_tags_47 },
    { 0xc1e5b14, SUC_lexicon_tags_56 },
    { 0x66de5b18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993e5b1e, SUC_lexicon_tags_10 },
    { 0x95be5b1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29e5b22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fe5b25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf31e5b27, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfe5b2c, SUC_lexicon_tags_37 },
    { 0x139e5b2d, SUC_lexicon_tags_48 },
    { 0x3bfe5b2e, SUC_lexicon_tags_11 },
    { 0x649e5b2f, SUC_lexicon_tags_21 },
    { 0xc95e5b30, SUC_lexicon_tags_1 },
    { 0x349e5b30, SUC_lexicon_tags_10 },
    { 0xb77e5b30, SUC_lexicon_tags_25 },
    { 0xb83e5b33, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffe5b3c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x61be5b3f, SUC_lexicon_tags_17 },
    { 0xd11e5b40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1de5b43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14fe5b45, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6c1e5b47, SUC_lexicon_tags_15 },
    { 0xa41e5b47, SUC_lexicon_tags_38 },
    { 0xd47e5b48, SUC_lexicon_tags_11 },
    { 0xe79e5b47, SUC_lexicon_tags_23 },
    { 0x1e5b49, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xadfe5b4d, SUC_lexicon_tags_33 },
    { 0xd89e5b4d, SUC_lexicon_tags_4 },
    { 0xe45e5b4f, SUC_lexicon_tags_37 },
    { 0x8fe5b50, SUC_lexicon_tags_23 },
    { 0xf71e5b4f, SUC_lexicon_tags_2 },
    { 0x863e5b52, SUC_lexicon_tags_7 },
    { 0x187e5b4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdf5e5b55, SUC_lexicon_tags_38 },
    { 0x251e5b56, SUC_lexicon_tags_17 },
    { 0xedbe5b57, SUC_lexicon_tags_27 },
    { 0xb7e5b58, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe7be5b5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x285e5b5d, SUC_lexicon_tags_7 },
    { 0x39fe5b5d, SUC_lexicon_tags_0 },
    { 0xf45e5b5d, SUC_lexicon_tags_51 },
    { 0x5d9e5b5d, SUC_lexicon_tags_126 },
    { 0x2e1e5b5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447e5b66, SUC_lexicon_tags_15 },
    { 0x629e5b66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7e5b6a, SUC_lexicon_tags_15 },
    { 0x4d5e5b6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971e5b6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89fe5b71, SUC_lexicon_tags_137 },
    { 0x645e5b72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afe5b75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x23de5b78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9e5b7e, SUC_lexicon_tags_66 },
    { 0x7dbe5b7e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09e5b82, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217e5b86, SUC_lexicon_tags_25 },
    { 0x61fe5b87, SUC_lexicon_tags_4 },
    { 0x9e3e5b88, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bde5b8d, SUC_lexicon_tags_424 },
    { 0x8b3e5b8e, SUC_lexicon_tags_38 },
    { 0x937e5b8f, SUC_lexicon_tags_12 },
    { 0x617e5b90, SUC_lexicon_tags_1 },
    { 0x7c5e5b91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95e5b98, SUC_lexicon_tags_178 },
    { 0x6b5e5b99, SUC_lexicon_tags_65 },
    { 0x22fe5b98, SUC_lexicon_tags_17 },
    { 0xbbe5b9b, SUC_lexicon_tags_11 },
    { 0xe3e5b9c, SUC_lexicon_tags_27 },
    { 0xa21e5b9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3e5ba1, SUC_lexicon_tags_17 },
    { 0x737e5ba2, SUC_lexicon_tags_12 },
    { 0x553e5ba3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x159e5ba6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47de5bb0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xd25e5bb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599e5bb6, SUC_lexicon_tags_27 },
    { 0x833e5bb6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x155e5bba, SUC_lexicon_tags_37 },
    { 0x629e5bba, SUC_lexicon_tags_6 },
    { 0x847e5bbc, SUC_lexicon_tags_37 },
    { 0x4dde5bbd, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0x449e5bbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fe5bc3, SUC_lexicon_tags_17 },
    { 0x689e5bc3, SUC_lexicon_tags_1 },
    { 0xa9be5bc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x289e5bc8, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xcb1e5bca, SUC_lexicon_tags_28 },
    { 0x62de5bcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de5bce, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65e5bd2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd27e5bd4, SUC_lexicon_tags_11 },
    { 0x7cfe5bd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93e5bda, SUC_lexicon_tags_63 },
    { 0x7ede5bda, SUC_lexicon_tags_47 },
    { 0xd15e5bdc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261e5be1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea3e5be3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9e5be8, SUC_lexicon_tags_7 },
    { 0x995e5be8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1e5bec, SUC_lexicon_tags_11 },
    { 0xc03e5bec, SUC_lexicon_tags_19 },
    { 0x767e5bee, SUC_lexicon_tags_47 },
    { 0x3e7e5bef, SUC_lexicon_tags_33 },
    { 0xd5de5bed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8e7e5bf2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151e5bfa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fde5c00, SUC_lexicon_tags_31 },
    { 0x80de5c01, SUC_lexicon_tags_11 },
    { 0x557e5c01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x591e5c05, SUC_lexicon_tags_17 },
    { 0x5afe5c05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7be5c08, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07e5c0b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbabe5c0d, SUC_lexicon_tags_33 },
    { 0x993e5c0e, SUC_lexicon_tags_7 },
    { 0xa83e5c0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe5c14, SUC_lexicon_tags_2 },
    { 0x7b5e5c15, SUC_lexicon_tags_7 },
    { 0x7c1e5c16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfe5c1f, SUC_lexicon_tags_27 },
    { 0x649e5c20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x303e5c22, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39e5c25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe5c29, SUC_lexicon_tags_7 },
    { 0x913e5c29, SUC_lexicon_tags_17 },
    { 0xefbe5c2b, SUC_lexicon_tags_0 },
    { 0x211e5c2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe5c30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7e5c33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfe5c38, SUC_lexicon_tags_10 },
    { 0xfe3e5c38, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x953e5c3c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xda3e5c3e, SUC_lexicon_tags_15 },
    { 0x51be5c3e, SUC_lexicon_tags_10 },
    { 0xd75e5c40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51e5c43, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51be5c47, SUC_lexicon_tags_25 },
    { 0xb3e5c47, SUC_lexicon_tags_25 },
    { 0x21e5c48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fe5c4c, SUC_lexicon_tags_12 },
    { 0x2e1e5c4c, SUC_lexicon_tags_3 },
    { 0x29be5c4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19e5c52, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319e5c56, SUC_lexicon_tags_65 },
    { 0x871e5c57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbe5c5b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19e5c64, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbc7e5c66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fde5c69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33e5c6e, SUC_lexicon_tags_82 },
    { 0x865e5c6e, SUC_lexicon_tags_1 },
    { 0xfdbe5c6e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97be5c7c, SUC_lexicon_tags_37 },
    { 0xbe3e5c7d, SUC_lexicon_tags_27 },
    { 0x569e5c7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fe5c83, SUC_lexicon_tags_191 },
    { 0x83be5c84, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6cde5c86, SUC_lexicon_tags_7 },
    { 0xf9fe5c87, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x30fe5c89, SUC_lexicon_tags_45 },
    { 0xede5c89, SUC_lexicon_tags_17 },
    { 0x9c7e5c8b, SUC_lexicon_tags_0 },
    { 0xf57e5c8c, SUC_lexicon_tags_39 },
    { 0xe47e5c8d, SUC_lexicon_tags_25 },
    { 0xb3fe5c8c, SUC_lexicon_tags_6 },
    { 0x951e5c8f, SUC_lexicon_tags_2 },
    { 0x26fe5c8c, SUC_lexicon_tags_47 },
    { 0x993e5c91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe89e5c93, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9de5c9c, SUC_lexicon_tags_47 },
    { 0xee9e5c9c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x51be5ca0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3be5ca4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x993e5ca6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89e5caa, SUC_lexicon_tags_17 },
    { 0xde5cab, SUC_lexicon_tags_11 },
    { 0xc9fe5cab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9e5caf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1e5cb3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7e5cb6, SUC_lexicon_tags_15 },
    { 0xa31e5cb7, SUC_lexicon_tags_15 },
    { 0xd11e5cb6, SUC_lexicon_tags_27 },
    { 0xd41e5cb7, SUC_lexicon_tags_207 },
    { 0x455e5cba, SUC_lexicon_tags_82 },
    { 0x5f5e5cba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec5e5cbd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5cc4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe83e5cc6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x53e5cc8, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0xacde5ccb, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fe5ccf, SUC_lexicon_tags_37 },
    { 0x4e5e5ccf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3de5cd3, SUC_lexicon_tags_17 },
    { 0xb1e5cd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe69e5cd6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fe5ce0, SUC_lexicon_tags_37 },
    { 0x61e5ce0, SUC_lexicon_tags_10 },
    { 0x9b1e5ce2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45be5ce5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5e5ce8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa1be5cea, SUC_lexicon_tags_70 },
    { 0xe3de5ceb, SUC_lexicon_tags_11 },
    { 0x63fe5cec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xee7e5cee, SUC_lexicon_tags_7 },
    { 0x48fe5cef, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfe5cf2, SUC_lexicon_tags_15 },
    { 0x2c5e5cf3, SUC_lexicon_tags_12 },
    { 0x877e5cf4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6c5e5cf6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x40be5cf8, SUC_lexicon_tags_23 },
    { 0x64fe5cf9, SUC_lexicon_tags_62 },
    { 0x2fde5cf8, SUC_lexicon_tags_37 },
    { 0x567e5cf8, SUC_lexicon_tags_15 },
    { 0x78fe5cfb, SUC_lexicon_tags_17 },
    { 0xfe9e5cf8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29de5d02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9e5d06, SUC_lexicon_tags_1 },
    { 0xd8de5d07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x53de5d09, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x7be5d0b, SUC_lexicon_tags_0 },
    { 0x205e5d0c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597e5d10, SUC_lexicon_tags_0 },
    { 0x289e5d10, SUC_lexicon_tags_12 },
    { 0x999e5d11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe5d15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799e5d1d, SUC_lexicon_tags_17 },
    { 0x26be5d1e, SUC_lexicon_tags_14 },
    { 0xe03e5d1d, SUC_lexicon_tags_10 },
    { 0x9f3e5d20, SUC_lexicon_tags_21 },
    { 0x58fe5d1f, SUC_lexicon_tags_84 },
    { 0xdcfe5d22, SUC_lexicon_tags_402 },
    { 0, NULL },
    { 0xb95e5d24, SUC_lexicon_tags_63 },
    { 0xaabe5d25, SUC_lexicon_tags_12 },
    { 0xb19e5d26, SUC_lexicon_tags_65 },
    { 0x6d1e5d27, SUC_lexicon_tags_1 },
    { 0xb47e5d27, SUC_lexicon_tags_37 },
    { 0xd8de5d29, SUC_lexicon_tags_25 },
    { 0xbb5e5d2a, SUC_lexicon_tags_27 },
    { 0x349e5d25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fe5d2f, SUC_lexicon_tags_6 },
    { 0xd75e5d2f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87e5d37, SUC_lexicon_tags_17 },
    { 0x1bde5d38, SUC_lexicon_tags_48 },
    { 0x9bbe5d39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x373e5d3c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2de5d40, SUC_lexicon_tags_6 },
    { 0x63e5d41, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x23e5d44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5e5d4d, SUC_lexicon_tags_30 },
    { 0xf7fe5d4d, SUC_lexicon_tags_10 },
    { 0x43fe5d4f, SUC_lexicon_tags_63 },
    { 0x43de5d4d, SUC_lexicon_tags_9 },
    { 0xae7e5d51, SUC_lexicon_tags_15 },
    { 0xfb7e5d4f, SUC_lexicon_tags_15 },
    { 0xa37e5d4f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7e5d56, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5e5d59, SUC_lexicon_tags_2 },
    { 0xf5be5d59, SUC_lexicon_tags_37 },
    { 0x7dbe5d5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fe5d61, SUC_lexicon_tags_29 },
    { 0x301e5d62, SUC_lexicon_tags_38 },
    { 0x9fe5d63, SUC_lexicon_tags_25 },
    { 0x6dbe5d64, SUC_lexicon_tags_2 },
    { 0xcd3e5d63, SUC_lexicon_tags_31 },
    { 0xb9be5d63, SUC_lexicon_tags_15 },
    { 0x66be5d67, SUC_lexicon_tags_7 },
    { 0xb7be5d67, SUC_lexicon_tags_15 },
    { 0x40be5d69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbe5d72, SUC_lexicon_tags_61 },
    { 0x7f3e5d73, SUC_lexicon_tags_11 },
    { 0xf85e5d73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x449e5d76, SUC_lexicon_tags_27 },
    { 0x593e5d76, SUC_lexicon_tags_7 },
    { 0x9c9e5d78, SUC_lexicon_tags_56 },
    { 0xa65e5d76, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xbebe5d7c, SUC_lexicon_tags_47 },
    { 0xd1de5d7d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9e5d89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1e5d91, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3de5d94, SUC_lexicon_tags_60 },
    { 0x899e5d95, SUC_lexicon_tags_28 },
    { 0x9afe5d96, SUC_lexicon_tags_7 },
    { 0x337e5d94, SUC_lexicon_tags_7 },
    { 0x5cde5d98, SUC_lexicon_tags_37 },
    { 0x7f9e5d99, SUC_lexicon_tags_23 },
    { 0x855e5d99, SUC_lexicon_tags_9 },
    { 0x3a1e5d9b, SUC_lexicon_tags_55 },
    { 0x97de5d99, SUC_lexicon_tags_8 },
    { 0x473e5d9c, SUC_lexicon_tags_1 },
    { 0x13be5d9e, SUC_lexicon_tags_43 },
    { 0x595e5d99, SUC_lexicon_tags_37 },
    { 0xd0de5da0, SUC_lexicon_tags_42 },
    { 0x52fe5d9c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1e5da4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fe5daa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x54fe5dad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507e5db1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97de5db6, SUC_lexicon_tags_37 },
    { 0xc27e5db6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bde5db9, SUC_lexicon_tags_35 },
    { 0x989e5dba, SUC_lexicon_tags_45 },
    { 0xdfde5dba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf03e5dbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3e5dc7, SUC_lexicon_tags_47 },
    { 0xa35e5dc7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74de5dd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9fe5dd2, SUC_lexicon_tags_27 },
    { 0x21fe5dd3, SUC_lexicon_tags_33 },
    { 0xe65e5dd3, SUC_lexicon_tags_60 },
    { 0xf17e5dd5, SUC_lexicon_tags_25 },
    { 0x15de5dd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa3e5dd8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x797e5dda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfe5dde, SUC_lexicon_tags_15 },
    { 0xdb7e5dde, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3c1e5de1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155e5de6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfe5de9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf97e5deb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x749e5ded, SUC_lexicon_tags_28 },
    { 0xc85e5ded, SUC_lexicon_tags_28 },
    { 0x43e5dee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3e5df2, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1e5df5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdbe5df7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc57e5df9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67e5dfe, SUC_lexicon_tags_25 },
    { 0x98be5dff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x933e5e02, SUC_lexicon_tags_37 },
    { 0x3b3e5e02, SUC_lexicon_tags_49 },
    { 0xd1be5e03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x31fe5e06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ade5e08, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3be5e0c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15e5e10, SUC_lexicon_tags_4 },
    { 0xb65e5e11, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fe5e15, SUC_lexicon_tags_60 },
    { 0xc2be5e16, SUC_lexicon_tags_37 },
    { 0xf83e5e17, SUC_lexicon_tags_55 },
    { 0x6cbe5e16, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5e5e1d, SUC_lexicon_tags_39 },
    { 0x4ffe5e1d, SUC_lexicon_tags_60 },
    { 0x973e5e1f, SUC_lexicon_tags_60 },
    { 0xf79e5e1f, SUC_lexicon_tags_82 },
    { 0x51e5e1f, SUC_lexicon_tags_63 },
    { 0xdb5e5e21, SUC_lexicon_tags_49 },
    { 0x221e5e22, SUC_lexicon_tags_15 },
    { 0x69be5e24, SUC_lexicon_tags_2 },
    { 0x7e9e5e1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39be5e2a, SUC_lexicon_tags_25 },
    { 0x6dde5e2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf6fe5e2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afe5e30, SUC_lexicon_tags_45 },
    { 0x6e3e5e31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x559e5e33, SUC_lexicon_tags_11 },
    { 0xd85e5e34, SUC_lexicon_tags_12 },
    { 0xe73e5e34, SUC_lexicon_tags_25 },
    { 0x28de5e36, SUC_lexicon_tags_2 },
    { 0x80de5e34, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x31de5e39, SUC_lexicon_tags_37 },
    { 0xcf5e5e3a, SUC_lexicon_tags_17 },
    { 0xa6be5e3b, SUC_lexicon_tags_17 },
    { 0x4a7e5e39, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9e5e3f, SUC_lexicon_tags_34 },
    { 0xc9de5e3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x64fe5e43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29e5e47, SUC_lexicon_tags_3 },
    { 0x9de5e48, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5e4d, SUC_lexicon_tags_37 },
    { 0xe0fe5e4e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc6be5e50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x645e5e53, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfe5e56, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ede5e5b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3e5e60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fe5e64, SUC_lexicon_tags_37 },
    { 0xedfe5e64, SUC_lexicon_tags_17 },
    { 0x1dbe5e66, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3e5e6c, SUC_lexicon_tags_38 },
    { 0x99de5e6d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfe5e70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x85de5e72, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x589e5e75, SUC_lexicon_tags_3 },
    { 0x5e1e5e76, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb3fe5e78, SUC_lexicon_tags_15 },
    { 0xb27e5e79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe33e5e7b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x409e5e7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55de5e81, SUC_lexicon_tags_37 },
    { 0x73fe5e81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x459e5e85, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x54de5e88, SUC_lexicon_tags_17 },
    { 0xb47e5e89, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x16de5e8b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xee1e5e8e, SUC_lexicon_tags_15 },
    { 0xb99e5e8f, SUC_lexicon_tags_1 },
    { 0xe1de5e8f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4de5e95, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeafe5e97, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc43e5e99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xab9e5e9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabe5ea0, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x883e5ea3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1be5ea6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5e5eaa, SUC_lexicon_tags_15 },
    { 0xf03e5eab, SUC_lexicon_tags_1 },
    { 0x893e5eab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5eb7, SUC_lexicon_tags_33 },
    { 0x7c1e5eb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fe5ebb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe33e5ebd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581e5ec3, SUC_lexicon_tags_29 },
    { 0xc1de5ec4, SUC_lexicon_tags_80 },
    { 0x77e5ec5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe49e5ec7, SUC_lexicon_tags_37 },
    { 0xe61e5ec8, SUC_lexicon_tags_49 },
    { 0xa11e5ec7, SUC_lexicon_tags_14 },
    { 0xbede5ec7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5e5ed2, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfe5eda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd55e5edc, SUC_lexicon_tags_37 },
    { 0x613e5edc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9e5ee1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dde5ee4, SUC_lexicon_tags_1 },
    { 0x6d5e5ee5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc85e5ee7, SUC_lexicon_tags_17 },
    { 0x9f5e5ee7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3e5eed, SUC_lexicon_tags_10 },
    { 0x7fbe5eee, SUC_lexicon_tags_15 },
    { 0xb99e5eef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80fe5ef3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x85de5ef6, SUC_lexicon_tags_12 },
    { 0xa75e5ef7, SUC_lexicon_tags_60 },
    { 0xb47e5ef8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x19de5efa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77e5efe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d1e5f00, SUC_lexicon_tags_25 },
    { 0x76de5f01, SUC_lexicon_tags_47 },
    { 0xc33e5f01, SUC_lexicon_tags_15 },
    { 0xb03e5f03, SUC_lexicon_tags_23 },
    { 0x893e5f04, SUC_lexicon_tags_8 },
    { 0x703e5f05, SUC_lexicon_tags_25 },
    { 0x6abe5f05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7e5f09, SUC_lexicon_tags_17 },
    { 0xe93e5f0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69e5f0e, SUC_lexicon_tags_15 },
    { 0xb57e5f0f, SUC_lexicon_tags_31 },
    { 0xc87e5f0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5e5f13, SUC_lexicon_tags_0 },
    { 0xcd3e5f14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5de5f16, SUC_lexicon_tags_11 },
    { 0x543e5f16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361e5f1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1e5f23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513e5f27, SUC_lexicon_tags_61 },
    { 0xef9e5f28, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4de5f2b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23e5f2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33fe5f31, SUC_lexicon_tags_7 },
    { 0x4e3e5f32, SUC_lexicon_tags_23 },
    { 0xeb3e5f33, SUC_lexicon_tags_60 },
    { 0x21fe5f34, SUC_lexicon_tags_349 },
    { 0x691e5f33, SUC_lexicon_tags_28 },
    { 0xcc3e5f35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1e5f39, SUC_lexicon_tags_17 },
    { 0xe8fe5f3a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9e5f41, SUC_lexicon_tags_46 },
    { 0x925e5f42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27e5f48, SUC_lexicon_tags_25 },
    { 0x369e5f49, SUC_lexicon_tags_12 },
    { 0xdd1e5f48, SUC_lexicon_tags_14 },
    { 0x67fe5f48, SUC_lexicon_tags_55 },
    { 0x6e5e5f49, SUC_lexicon_tags_40 },
    { 0xd0fe5f4c, SUC_lexicon_tags_60 },
    { 0x90fe5f4e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6de5f52, SUC_lexicon_tags_63 },
    { 0x273e5f53, SUC_lexicon_tags_25 },
    { 0x89e5f54, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x217e5f57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e5f5d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xa29e5f5f, SUC_lexicon_tags_16 },
    { 0x38fe5f5f, SUC_lexicon_tags_45 },
    { 0xc8be5f61, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2de5f68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51e5f72, SUC_lexicon_tags_9 },
    { 0xbdde5f72, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7be5f78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93e5f7b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21e5f7f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x701e5f81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8de5f88, SUC_lexicon_tags_865 },
    { 0, NULL },
    { 0x6b1e5f8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275e5f8e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x961e5f91, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fe5f99, SUC_lexicon_tags_0 },
    { 0x479e5f9a, SUC_lexicon_tags_37 },
    { 0x473e5f9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x887e5f9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5e5fa4, SUC_lexicon_tags_40 },
    { 0x7b9e5fa4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe41e5fa7, SUC_lexicon_tags_14 },
    { 0x76de5fa8, SUC_lexicon_tags_17 },
    { 0x49be5fa9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dde5fad, SUC_lexicon_tags_25 },
    { 0x209e5fad, SUC_lexicon_tags_12 },
    { 0xb7e5faf, SUC_lexicon_tags_17 },
    { 0xc93e5fb0, SUC_lexicon_tags_21 },
    { 0x779e5fb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2be5fb6, SUC_lexicon_tags_32 },
    { 0x4b1e5fb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xaffe5fb9, SUC_lexicon_tags_17 },
    { 0xe7be5fba, SUC_lexicon_tags_19 },
    { 0x1b3e5fba, SUC_lexicon_tags_8 },
    { 0xcbfe5fbc, SUC_lexicon_tags_37 },
    { 0x1e5e5fbd, SUC_lexicon_tags_7 },
    { 0xfdbe5fbe, SUC_lexicon_tags_7 },
    { 0x349e5fbc, SUC_lexicon_tags_66 },
    { 0x6c9e5fbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4e9e5fc2, SUC_lexicon_tags_148 },
    { 0xc95e5fc3, SUC_lexicon_tags_5 },
    { 0x1dde5fc4, SUC_lexicon_tags_23 },
    { 0x73de5fc5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x531e5fc7, SUC_lexicon_tags_15 },
    { 0xc15e5fc8, SUC_lexicon_tags_7 },
    { 0x511e5fc9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1e5fd0, SUC_lexicon_tags_32 },
    { 0x51fe5fd1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x28de5fd4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82be5fd9, SUC_lexicon_tags_19 },
    { 0x63e5fda, SUC_lexicon_tags_39 },
    { 0xc19e5fda, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x53fe5fdd, SUC_lexicon_tags_25 },
    { 0x96de5fde, SUC_lexicon_tags_7 },
    { 0x325e5fde, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9de5fe2, SUC_lexicon_tags_2 },
    { 0x193e5fe3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3e5fe6, SUC_lexicon_tags_7 },
    { 0x879e5fe6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5e5feb, SUC_lexicon_tags_38 },
    { 0x98be5fec, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefe5ff2, SUC_lexicon_tags_25 },
    { 0x909e5ff2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe9de5ff5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15be5ffa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf83e5ffc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5e6001, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xaf3e6003, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739e600b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1e600e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49e6012, SUC_lexicon_tags_7 },
    { 0x457e6013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9de6015, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8afe6018, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda9e601a, SUC_lexicon_tags_34 },
    { 0x9a3e601b, SUC_lexicon_tags_264 },
    { 0, NULL },
    { 0x777e601d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbe6021, SUC_lexicon_tags_49 },
    { 0x36be6021, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x997e6024, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17e6027, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x505e602a, SUC_lexicon_tags_11 },
    { 0x8f9e602b, SUC_lexicon_tags_25 },
    { 0x13de602c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f1e602e, SUC_lexicon_tags_0 },
    { 0x24be602e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x66fe6031, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa83e6033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x415e6036, SUC_lexicon_tags_37 },
    { 0x347e6037, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0de603b, SUC_lexicon_tags_45 },
    { 0x941e603b, SUC_lexicon_tags_19 },
    { 0xb11e603b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa31e603f, SUC_lexicon_tags_11 },
    { 0x651e603f, SUC_lexicon_tags_8 },
    { 0xe51e6041, SUC_lexicon_tags_40 },
    { 0x8ede6040, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71be6047, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7be604a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe604e, SUC_lexicon_tags_29 },
    { 0x3a5e604f, SUC_lexicon_tags_39 },
    { 0x9f7e604e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1e6055, SUC_lexicon_tags_49 },
    { 0xc1de6055, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d1e6058, SUC_lexicon_tags_36 },
    { 0xe47e6059, SUC_lexicon_tags_37 },
    { 0x23e6058, SUC_lexicon_tags_47 },
    { 0x903e6059, SUC_lexicon_tags_15 },
    { 0xee1e605c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07e6060, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe6064, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d5e6066, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9e606a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x997e606d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665e6076, SUC_lexicon_tags_10 },
    { 0xa71e6077, SUC_lexicon_tags_23 },
    { 0xa21e6078, SUC_lexicon_tags_15 },
    { 0x2e9e6079, SUC_lexicon_tags_25 },
    { 0x2ede607a, SUC_lexicon_tags_11 },
    { 0x9dbe6079, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1e607e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e6081, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbe6085, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb81e6087, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3e6089, SUC_lexicon_tags_34 },
    { 0x3f3e608a, SUC_lexicon_tags_1 },
    { 0xb0be608a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x99e608d, SUC_lexicon_tags_37 },
    { 0xadbe608e, SUC_lexicon_tags_43 },
    { 0xeb7e608f, SUC_lexicon_tags_11 },
    { 0xfc1e608f, SUC_lexicon_tags_19 },
    { 0x9a7e608f, SUC_lexicon_tags_25 },
    { 0xce7e608e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe55e6094, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6be6097, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x595e6099, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989e60a1, SUC_lexicon_tags_7 },
    { 0x83de60a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf55e60a4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb51e60a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd7e60a8, SUC_lexicon_tags_6 },
    { 0xe9fe60a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5e60b0, SUC_lexicon_tags_38 },
    { 0xb6fe60b1, SUC_lexicon_tags_11 },
    { 0x5fde60b1, SUC_lexicon_tags_14 },
    { 0x431e60b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfe60b9, SUC_lexicon_tags_17 },
    { 0x897e60ba, SUC_lexicon_tags_65 },
    { 0x7ede60bb, SUC_lexicon_tags_17 },
    { 0x8f5e60bb, SUC_lexicon_tags_17 },
    { 0xef1e60ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7be60bf, SUC_lexicon_tags_9 },
    { 0x815e60c0, SUC_lexicon_tags_60 },
    { 0x15fe60bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601e60c7, SUC_lexicon_tags_1 },
    { 0x921e60c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x67be60ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5e60ce, SUC_lexicon_tags_11 },
    { 0x1f7e60cf, SUC_lexicon_tags_4 },
    { 0xa8de60d0, SUC_lexicon_tags_27 },
    { 0xa79e60d1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74be60d5, SUC_lexicon_tags_15 },
    { 0xee9e60d6, SUC_lexicon_tags_27 },
    { 0x115e60d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4be60db, SUC_lexicon_tags_38 },
    { 0x4ebe60dc, SUC_lexicon_tags_26 },
    { 0x119e60dd, SUC_lexicon_tags_12 },
    { 0x755e60dd, SUC_lexicon_tags_39 },
    { 0x53fe60de, SUC_lexicon_tags_14 },
    { 0xcede60e0, SUC_lexicon_tags_17 },
    { 0x759e60e1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999e60e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79e60ea, SUC_lexicon_tags_25 },
    { 0xbdde60eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50be60ef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebe60f5, SUC_lexicon_tags_15 },
    { 0x707e60f6, SUC_lexicon_tags_15 },
    { 0xebe60f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ebe60f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd53e60fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbe60ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4de6105, SUC_lexicon_tags_9 },
    { 0x875e6106, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x99be6108, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3e610f, SUC_lexicon_tags_33 },
    { 0x367e6110, SUC_lexicon_tags_43 },
    { 0xb1fe6110, SUC_lexicon_tags_51 },
    { 0xf4fe6112, SUC_lexicon_tags_17 },
    { 0xe23e6110, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x90de6115, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885e6119, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e611c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129e6123, SUC_lexicon_tags_31 },
    { 0xe05e6123, SUC_lexicon_tags_87 },
    { 0x77e6125, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0de6132, SUC_lexicon_tags_23 },
    { 0x89e6132, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x447e6135, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afe613a, SUC_lexicon_tags_172 },
    { 0xb43e613a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5e613f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3afe6144, SUC_lexicon_tags_60 },
    { 0x837e6145, SUC_lexicon_tags_1 },
    { 0x295e6146, SUC_lexicon_tags_33 },
    { 0x5e6144, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x973e6149, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1e614e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb81e6150, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x715e6152, SUC_lexicon_tags_108 },
    { 0x3e5e6152, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfe7e6155, SUC_lexicon_tags_35 },
    { 0x191e6156, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x957e6158, SUC_lexicon_tags_25 },
    { 0x4e9e6158, SUC_lexicon_tags_17 },
    { 0xe0fe615a, SUC_lexicon_tags_11 },
    { 0x369e615b, SUC_lexicon_tags_23 },
    { 0x69fe615c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8dde615e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeabe6160, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf53e6162, SUC_lexicon_tags_12 },
    { 0x65fe6163, SUC_lexicon_tags_23 },
    { 0x49be6163, SUC_lexicon_tags_85 },
    { 0x613e6162, SUC_lexicon_tags_11 },
    { 0xcc5e6163, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fe616a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9de616e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275e6173, SUC_lexicon_tags_1316 },
    { 0x3e9e6173, SUC_lexicon_tags_17 },
    { 0x88de6175, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197e6179, SUC_lexicon_tags_35 },
    { 0x177e6179, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1e9e617c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21e6181, SUC_lexicon_tags_15 },
    { 0xa9fe6181, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13e6185, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9e6189, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa39e618b, SUC_lexicon_tags_25 },
    { 0x48be618c, SUC_lexicon_tags_11 },
    { 0xefde618d, SUC_lexicon_tags_33 },
    { 0x9e7e618e, SUC_lexicon_tags_1 },
    { 0xe45e618f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33de6194, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5e619d, SUC_lexicon_tags_33 },
    { 0xa6fe619e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7e61ae, SUC_lexicon_tags_19 },
    { 0x7a3e61af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd5be61b1, SUC_lexicon_tags_11 },
    { 0x665e61b2, SUC_lexicon_tags_4 },
    { 0xc0de61b3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25e61b6, SUC_lexicon_tags_454 },
    { 0, NULL },
    { 0xc13e61b8, SUC_lexicon_tags_4 },
    { 0xf7be61b8, SUC_lexicon_tags_57 },
    { 0x47de61ba, SUC_lexicon_tags_31 },
    { 0x237e61bb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfffe61be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abe61c4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xed9e61c6, SUC_lexicon_tags_12 },
    { 0xedfe61c7, SUC_lexicon_tags_11 },
    { 0x7cbe61c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99e61cb, SUC_lexicon_tags_30 },
    { 0x223e61cb, SUC_lexicon_tags_33 },
    { 0x7ade61cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a5e61cf, SUC_lexicon_tags_33 },
    { 0xb47e61cf, SUC_lexicon_tags_37 },
    { 0x44be61cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829e61d6, SUC_lexicon_tags_17 },
    { 0xcf7e61d7, SUC_lexicon_tags_23 },
    { 0x93e61d8, SUC_lexicon_tags_9 },
    { 0xb11e61d7, SUC_lexicon_tags_26 },
    { 0x1a5e61da, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x33e61dc, SUC_lexicon_tags_25 },
    { 0xbe5e61dc, SUC_lexicon_tags_9 },
    { 0x7c5e61de, SUC_lexicon_tags_8 },
    { 0x283e61dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc8de61e1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94de61e5, SUC_lexicon_tags_49 },
    { 0x469e61e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599e61ea, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x1d9e61ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7f1e61ee, SUC_lexicon_tags_14 },
    { 0xc51e61ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23e61f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3e61f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57e61fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7e61ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3bde6201, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09e620a, SUC_lexicon_tags_39 },
    { 0xd63e620b, SUC_lexicon_tags_33 },
    { 0x981e620c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb33e620e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc6be6210, SUC_lexicon_tags_37 },
    { 0x183e6211, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x72de6214, SUC_lexicon_tags_19 },
    { 0xe3be6215, SUC_lexicon_tags_1 },
    { 0xd83e6214, SUC_lexicon_tags_12 },
    { 0x4b3e6215, SUC_lexicon_tags_37 },
    { 0xfabe6218, SUC_lexicon_tags_12 },
    { 0x2dde6219, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1e621c, SUC_lexicon_tags_19 },
    { 0x491e621d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x56be6220, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fe6223, SUC_lexicon_tags_6 },
    { 0xfdbe6224, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x4ade6226, SUC_lexicon_tags_3 },
    { 0x43be6226, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x83be622a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07e6230, SUC_lexicon_tags_19 },
    { 0x79be6231, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cde6239, SUC_lexicon_tags_11 },
    { 0x557e623a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87e623c, SUC_lexicon_tags_15 },
    { 0x685e623c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323e6248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfe624d, SUC_lexicon_tags_10 },
    { 0x7d5e624e, SUC_lexicon_tags_0 },
    { 0xe19e624d, SUC_lexicon_tags_47 },
    { 0x139e6250, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9e6254, SUC_lexicon_tags_37 },
    { 0x9fbe6255, SUC_lexicon_tags_17 },
    { 0x3fbe6256, SUC_lexicon_tags_4 },
    { 0x48fe6257, SUC_lexicon_tags_61 },
    { 0xb5e6257, SUC_lexicon_tags_45 },
    { 0xe2fe6254, SUC_lexicon_tags_26 },
    { 0xac3e6254, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c3e625c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23be6262, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1be6267, SUC_lexicon_tags_12 },
    { 0xa2fe6268, SUC_lexicon_tags_12 },
    { 0xd47e6268, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fe626f, SUC_lexicon_tags_12 },
    { 0xe7de626f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x243e6272, SUC_lexicon_tags_87 },
    { 0xeebe6273, SUC_lexicon_tags_26 },
    { 0xffe6274, SUC_lexicon_tags_1 },
    { 0xdbbe6272, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557e6279, SUC_lexicon_tags_27 },
    { 0xae7e627a, SUC_lexicon_tags_17 },
    { 0x419e627a, SUC_lexicon_tags_60 },
    { 0xdb1e627a, SUC_lexicon_tags_26 },
    { 0x3f1e6279, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa0be627f, SUC_lexicon_tags_37 },
    { 0xf1de627f, SUC_lexicon_tags_7 },
    { 0x8c7e6281, SUC_lexicon_tags_1 },
    { 0x1cbe6281, SUC_lexicon_tags_37 },
    { 0x7d1e6281, SUC_lexicon_tags_25 },
    { 0xa55e6284, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x153e6286, SUC_lexicon_tags_37 },
    { 0x147e6287, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807e628b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47e628d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fe6290, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1e6295, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e629e, SUC_lexicon_tags_138 },
    { 0x7c5e629f, SUC_lexicon_tags_12 },
    { 0xa99e62a0, SUC_lexicon_tags_30 },
    { 0x5cbe629f, SUC_lexicon_tags_9 },
    { 0xf2de62a2, SUC_lexicon_tags_56 },
    { 0x511e629e, SUC_lexicon_tags_31 },
    { 0x215e62a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3e62a8, SUC_lexicon_tags_37 },
    { 0xc99e62a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x227e62ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7e62b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x365e62b5, SUC_lexicon_tags_33 },
    { 0x4e5e62b6, SUC_lexicon_tags_19 },
    { 0x595e62b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7e62bc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e62c0, SUC_lexicon_tags_60 },
    { 0x423e62c1, SUC_lexicon_tags_63 },
    { 0x603e62c2, SUC_lexicon_tags_12 },
    { 0xf77e62c3, SUC_lexicon_tags_1 },
    { 0x6f7e62c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fe62c7, SUC_lexicon_tags_32 },
    { 0xac7e62c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc13e62ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dde62cd, SUC_lexicon_tags_25 },
    { 0x8f3e62ce, SUC_lexicon_tags_15 },
    { 0xd4be62cf, SUC_lexicon_tags_49 },
    { 0x41be62cf, SUC_lexicon_tags_14 },
    { 0xe6de62d0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79e62db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffe62de, SUC_lexicon_tags_41 },
    { 0xd51e62df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x145e62e1, SUC_lexicon_tags_3 },
    { 0x2dde62e2, SUC_lexicon_tags_2 },
    { 0xbe7e62e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fe62e8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7e62ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x467e62f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ffe62f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa23e62f7, SUC_lexicon_tags_7 },
    { 0xfede62f8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01e62fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x343e62fe, SUC_lexicon_tags_26 },
    { 0x2afe62ff, SUC_lexicon_tags_12 },
    { 0x96de62fe, SUC_lexicon_tags_15 },
    { 0xe41e6301, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e6308, SUC_lexicon_tags_17 },
    { 0x59e6309, SUC_lexicon_tags_19 },
    { 0x959e630a, SUC_lexicon_tags_17 },
    { 0x1c3e630a, SUC_lexicon_tags_29 },
    { 0xb1e630c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407e6314, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28de6316, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59e631b, SUC_lexicon_tags_25 },
    { 0x7dde631c, SUC_lexicon_tags_60 },
    { 0x747e631d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbb7e631f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61e6323, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37e632a, SUC_lexicon_tags_2 },
    { 0x4d5e632b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33e6332, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x579e6335, SUC_lexicon_tags_37 },
    { 0x24fe6335, SUC_lexicon_tags_33 },
    { 0xcbfe6337, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261e633b, SUC_lexicon_tags_7 },
    { 0x75e633b, SUC_lexicon_tags_11 },
    { 0xee9e633b, SUC_lexicon_tags_28 },
    { 0xaede633e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93de6340, SUC_lexicon_tags_38 },
    { 0x53be6341, SUC_lexicon_tags_17 },
    { 0x90be6342, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebe6345, SUC_lexicon_tags_11 },
    { 0x251e6346, SUC_lexicon_tags_12 },
    { 0x43be6346, SUC_lexicon_tags_15 },
    { 0x11fe6346, SUC_lexicon_tags_29 },
    { 0xc17e6345, SUC_lexicon_tags_11 },
    { 0x523e634a, SUC_lexicon_tags_91 },
    { 0xb45e6349, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dde634f, SUC_lexicon_tags_3 },
    { 0xf27e6350, SUC_lexicon_tags_69 },
    { 0x4f1e634f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe6355, SUC_lexicon_tags_2 },
    { 0xd45e6355, SUC_lexicon_tags_23 },
    { 0x3b5e6357, SUC_lexicon_tags_46 },
    { 0x10fe6358, SUC_lexicon_tags_15 },
    { 0xc9e6359, SUC_lexicon_tags_37 },
    { 0x551e6358, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xebfe635d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667e6361, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x981e6364, SUC_lexicon_tags_46 },
    { 0xf29e6365, SUC_lexicon_tags_1 },
    { 0x5bfe6366, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bfe6368, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31e636b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99e636e, SUC_lexicon_tags_12 },
    { 0xf2be636f, SUC_lexicon_tags_66 },
    { 0xdabe636e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa73e6372, SUC_lexicon_tags_25 },
    { 0xe2de6372, SUC_lexicon_tags_37 },
    { 0x19fe6372, SUC_lexicon_tags_17 },
    { 0x48be6372, SUC_lexicon_tags_525 },
    { 0xf1fe6374, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x223e6378, SUC_lexicon_tags_60 },
    { 0x719e6379, SUC_lexicon_tags_3 },
    { 0x1a7e637a, SUC_lexicon_tags_47 },
    { 0x5a9e637a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb91e637d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fe6381, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4de6389, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73e638b, SUC_lexicon_tags_25 },
    { 0xed7e638b, SUC_lexicon_tags_9 },
    { 0x103e638b, SUC_lexicon_tags_33 },
    { 0x513e638e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x921e6390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa83e6392, SUC_lexicon_tags_37 },
    { 0xd87e6393, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3e6397, SUC_lexicon_tags_11 },
    { 0xa15e6398, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x193e639b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd35e639d, SUC_lexicon_tags_11 },
    { 0xf3de639e, SUC_lexicon_tags_17 },
    { 0x7a7e639f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6dde63a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fe63a7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07e63aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1e63af, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ade63b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30de63ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1e63be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3fe63c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5e63c6, SUC_lexicon_tags_1 },
    { 0x865e63c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9e63ce, SUC_lexicon_tags_11 },
    { 0xc59e63cf, SUC_lexicon_tags_8 },
    { 0x40fe63d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x749e63d2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8cde63d4, SUC_lexicon_tags_49 },
    { 0xc5be63d5, SUC_lexicon_tags_11 },
    { 0x35be63d6, SUC_lexicon_tags_33 },
    { 0xf9e63d6, SUC_lexicon_tags_16 },
    { 0x8f1e63d6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597e63dd, SUC_lexicon_tags_47 },
    { 0x3fde63dd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9e63e1, SUC_lexicon_tags_0 },
    { 0x1c5e63e2, SUC_lexicon_tags_37 },
    { 0x60de63e3, SUC_lexicon_tags_11 },
    { 0xe2de63e4, SUC_lexicon_tags_17 },
    { 0x1b5e63e3, SUC_lexicon_tags_48 },
    { 0x4d3e63e1, SUC_lexicon_tags_7 },
    { 0x4dfe63e1, SUC_lexicon_tags_17 },
    { 0xf33e63e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9be63ee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45fe63f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563e63fc, SUC_lexicon_tags_12 },
    { 0x651e63fc, SUC_lexicon_tags_14 },
    { 0xa0de63fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x677e6400, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5de6402, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9e6406, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8dfe6408, SUC_lexicon_tags_17 },
    { 0xfb9e6409, SUC_lexicon_tags_9 },
    { 0x23e6408, SUC_lexicon_tags_46 },
    { 0xc51e640a, SUC_lexicon_tags_16 },
    { 0xae3e640c, SUC_lexicon_tags_17 },
    { 0xd83e640c, SUC_lexicon_tags_100 },
    { 0x4a5e640b, SUC_lexicon_tags_37 },
    { 0x49fe640e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8be6413, SUC_lexicon_tags_15 },
    { 0x7de6414, SUC_lexicon_tags_174 },
    { 0xbb7e6415, SUC_lexicon_tags_21 },
    { 0xfa1e6414, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89be6419, SUC_lexicon_tags_19 },
    { 0xe05e641a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7de641e, SUC_lexicon_tags_10 },
    { 0xe71e641f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41e6423, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x373e6425, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e6428, SUC_lexicon_tags_1 },
    { 0x273e6429, SUC_lexicon_tags_15 },
    { 0x34be642a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1e642d, SUC_lexicon_tags_38 },
    { 0xadfe642e, SUC_lexicon_tags_17 },
    { 0x235e642f, SUC_lexicon_tags_17 },
    { 0xd81e6430, SUC_lexicon_tags_15 },
    { 0x4f3e6431, SUC_lexicon_tags_32 },
    { 0xeb9e6432, SUC_lexicon_tags_118 },
    { 0xc73e6433, SUC_lexicon_tags_0 },
    { 0x93fe6432, SUC_lexicon_tags_1 },
    { 0xcc7e6435, SUC_lexicon_tags_32 },
    { 0xd5fe642f, SUC_lexicon_tags_7 },
    { 0x99fe6437, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5de643a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91e6440, SUC_lexicon_tags_215 },
    { 0xf79e6441, SUC_lexicon_tags_1 },
    { 0xb75e6441, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x195e6444, SUC_lexicon_tags_37 },
    { 0x17fe6445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x19e6447, SUC_lexicon_tags_49 },
    { 0x655e6448, SUC_lexicon_tags_16 },
    { 0x6a5e6449, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59e644f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593e6453, SUC_lexicon_tags_12 },
    { 0xb9be6454, SUC_lexicon_tags_7 },
    { 0xb31e6454, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe6459, SUC_lexicon_tags_25 },
    { 0x1cbe6459, SUC_lexicon_tags_2 },
    { 0x8d7e6459, SUC_lexicon_tags_27 },
    { 0x431e645c, SUC_lexicon_tags_24 },
    { 0x37be645d, SUC_lexicon_tags_17 },
    { 0xd6de645e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x87fe6460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35be6463, SUC_lexicon_tags_63 },
    { 0x4b5e6463, SUC_lexicon_tags_0 },
    { 0x3fe6463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf5e6467, SUC_lexicon_tags_29 },
    { 0x3c9e6467, SUC_lexicon_tags_37 },
    { 0x6fbe6469, SUC_lexicon_tags_11 },
    { 0xe15e6468, SUC_lexicon_tags_48 },
    { 0x3f1e646b, SUC_lexicon_tags_38 },
    { 0x9d5e646c, SUC_lexicon_tags_25 },
    { 0x1a1e646b, SUC_lexicon_tags_17 },
    { 0x4d7e6469, SUC_lexicon_tags_38 },
    { 0x735e6467, SUC_lexicon_tags_10 },
    { 0xf31e6469, SUC_lexicon_tags_37 },
    { 0xb0fe6470, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9be6475, SUC_lexicon_tags_7 },
    { 0x555e6476, SUC_lexicon_tags_47 },
    { 0xd89e6475, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5e647a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fe6481, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fe6485, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf29e6487, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x54be648a, SUC_lexicon_tags_8 },
    { 0x5fde648b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc27e648d, SUC_lexicon_tags_0 },
    { 0x183e648d, SUC_lexicon_tags_60 },
    { 0xbf1e648d, SUC_lexicon_tags_7 },
    { 0xda9e6490, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x255e6492, SUC_lexicon_tags_87 },
    { 0x81be6492, SUC_lexicon_tags_12 },
    { 0xab3e6492, SUC_lexicon_tags_12 },
    { 0x187e6495, SUC_lexicon_tags_11 },
    { 0x575e6496, SUC_lexicon_tags_15 },
    { 0x2a7e6497, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e649a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1e649d, SUC_lexicon_tags_10 },
    { 0xf6be649e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdf1e64a0, SUC_lexicon_tags_68 },
    { 0x41e64a1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865e64aa, SUC_lexicon_tags_60 },
    { 0xb9de64ab, SUC_lexicon_tags_9 },
    { 0x81e64ac, SUC_lexicon_tags_33 },
    { 0x611e64ac, SUC_lexicon_tags_61 },
    { 0x537e64ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x74de64b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e64b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfe64bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc1e64bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc9e64c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1e64c4, SUC_lexicon_tags_37 },
    { 0xa71e64c4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ade64cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fe64cf, SUC_lexicon_tags_63 },
    { 0x8ffe64cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4fbe64d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2d3e64d4, SUC_lexicon_tags_2 },
    { 0x57be64d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7e64de, SUC_lexicon_tags_37 },
    { 0x49fe64df, SUC_lexicon_tags_55 },
    { 0xc7fe64df, SUC_lexicon_tags_17 },
    { 0x271e64e1, SUC_lexicon_tags_37 },
    { 0x117e64e2, SUC_lexicon_tags_9 },
    { 0x4b1e64de, SUC_lexicon_tags_11 },
    { 0xe23e64df, SUC_lexicon_tags_17 },
    { 0x8cfe64e2, SUC_lexicon_tags_40 },
    { 0x75de64e4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53e64ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46de64f2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x767e64f4, SUC_lexicon_tags_27 },
    { 0xd71e64f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41e64fc, SUC_lexicon_tags_37 },
    { 0xdbe64fd, SUC_lexicon_tags_1 },
    { 0x48de64fe, SUC_lexicon_tags_14 },
    { 0xe5e64fc, SUC_lexicon_tags_15 },
    { 0xab3e6500, SUC_lexicon_tags_12 },
    { 0x867e64fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x693e6503, SUC_lexicon_tags_37 },
    { 0x5be6503, SUC_lexicon_tags_33 },
    { 0xe4fe6505, SUC_lexicon_tags_14 },
    { 0xd87e6506, SUC_lexicon_tags_7 },
    { 0x789e6507, SUC_lexicon_tags_25 },
    { 0x7f3e6508, SUC_lexicon_tags_11 },
    { 0xa95e6507, SUC_lexicon_tags_12 },
    { 0xc27e650a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e650d, SUC_lexicon_tags_29 },
    { 0x78fe650e, SUC_lexicon_tags_37 },
    { 0x867e650d, SUC_lexicon_tags_7 },
    { 0xb19e6510, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe6515, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505e651a, SUC_lexicon_tags_17 },
    { 0xb35e651b, SUC_lexicon_tags_68 },
    { 0xf31e651c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485e6523, SUC_lexicon_tags_14 },
    { 0x33e6524, SUC_lexicon_tags_9 },
    { 0x671e6524, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0de6528, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a1e652a, SUC_lexicon_tags_34 },
    { 0xf65e652b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x365e652e, SUC_lexicon_tags_24 },
    { 0x7c5e652f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03e6535, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fe653b, SUC_lexicon_tags_25 },
    { 0xd3e653b, SUC_lexicon_tags_25 },
    { 0x9b7e653d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9e6540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbe6543, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa83e6545, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461e654a, SUC_lexicon_tags_11 },
    { 0xd61e654a, SUC_lexicon_tags_2 },
    { 0xfffe654c, SUC_lexicon_tags_47 },
    { 0xd4be654a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31e6554, SUC_lexicon_tags_29 },
    { 0x7a5e6555, SUC_lexicon_tags_87 },
    { 0x1be6556, SUC_lexicon_tags_33 },
    { 0xbe5e6557, SUC_lexicon_tags_37 },
    { 0x437e6558, SUC_lexicon_tags_6 },
    { 0xc89e6558, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7e655d, SUC_lexicon_tags_37 },
    { 0x881e655e, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x563e6561, SUC_lexicon_tags_49 },
    { 0x311e6561, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435e6566, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6fe6568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa3e656a, SUC_lexicon_tags_7 },
    { 0x7de656b, SUC_lexicon_tags_19 },
    { 0xcd9e656c, SUC_lexicon_tags_19 },
    { 0xd0fe656c, SUC_lexicon_tags_9 },
    { 0xa49e656e, SUC_lexicon_tags_17 },
    { 0xda3e656e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x707e6571, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861e6577, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e657a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd83e657c, SUC_lexicon_tags_16 },
    { 0x77de657c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x88de657f, SUC_lexicon_tags_7 },
    { 0x209e6580, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x27be6582, SUC_lexicon_tags_9 },
    { 0xebde6583, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7e6586, SUC_lexicon_tags_11 },
    { 0x26fe6586, SUC_lexicon_tags_25 },
    { 0x765e6588, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499e6590, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe6593, SUC_lexicon_tags_2 },
    { 0x43de6594, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdcde6596, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa15e6598, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x271e659a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531e659e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfe65a1, SUC_lexicon_tags_23 },
    { 0xbe65a2, SUC_lexicon_tags_1 },
    { 0xba9e65a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfe65a8, SUC_lexicon_tags_23 },
    { 0xf69e65a8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x913e65ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd87e65ad, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffe65b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23be65b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19e65bb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3e65bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbde65c2, SUC_lexicon_tags_43 },
    { 0xabfe65c3, SUC_lexicon_tags_31 },
    { 0x6ade65c2, SUC_lexicon_tags_7 },
    { 0x32de65c3, SUC_lexicon_tags_30 },
    { 0x1f3e65c6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359e65cc, SUC_lexicon_tags_34 },
    { 0x503e65cd, SUC_lexicon_tags_21 },
    { 0x5fbe65ce, SUC_lexicon_tags_84 },
    { 0x5ede65cf, SUC_lexicon_tags_61 },
    { 0x793e65ce, SUC_lexicon_tags_65 },
    { 0x805e65ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19de65d5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3f3e65d7, SUC_lexicon_tags_60 },
    { 0xb7e65d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2be65db, SUC_lexicon_tags_17 },
    { 0x19fe65db, SUC_lexicon_tags_37 },
    { 0xa1be65dd, SUC_lexicon_tags_9 },
    { 0x669e65dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fe65e1, SUC_lexicon_tags_60 },
    { 0x3e7e65e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fe65e9, SUC_lexicon_tags_79 },
    { 0x43be65ea, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe65ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb95e65f2, SUC_lexicon_tags_28 },
    { 0xce9e65f3, SUC_lexicon_tags_17 },
    { 0xf07e65f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43be65f6, SUC_lexicon_tags_17 },
    { 0x4d3e65f6, SUC_lexicon_tags_2 },
    { 0xe35e65f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38be65fb, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xc9fe65fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fde6600, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x18be6603, SUC_lexicon_tags_62 },
    { 0xc23e6604, SUC_lexicon_tags_17 },
    { 0xbc7e6605, SUC_lexicon_tags_25 },
    { 0xf87e6605, SUC_lexicon_tags_28 },
    { 0x539e6607, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66be660c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed1e660e, SUC_lexicon_tags_70 },
    { 0x547e660e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2be6612, SUC_lexicon_tags_10 },
    { 0x21fe6612, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x759e6615, SUC_lexicon_tags_66 },
    { 0x89fe6616, SUC_lexicon_tags_7 },
    { 0x68be6615, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe661b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77e661f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb47e6621, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6dde6623, SUC_lexicon_tags_8 },
    { 0x24de6623, SUC_lexicon_tags_8 },
    { 0x6f1e6625, SUC_lexicon_tags_32 },
    { 0x421e6624, SUC_lexicon_tags_10 },
    { 0x4ade6627, SUC_lexicon_tags_37 },
    { 0x98de6628, SUC_lexicon_tags_11 },
    { 0x9b7e6629, SUC_lexicon_tags_7 },
    { 0x5f9e662a, SUC_lexicon_tags_25 },
    { 0xa59e6629, SUC_lexicon_tags_12 },
    { 0x521e662c, SUC_lexicon_tags_30 },
    { 0x625e662d, SUC_lexicon_tags_25 },
    { 0x531e662e, SUC_lexicon_tags_17 },
    { 0xb93e662e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfe6632, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbe6636, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b9e6638, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x529e663a, SUC_lexicon_tags_11 },
    { 0x2dde663b, SUC_lexicon_tags_29 },
    { 0xc6de663a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf1de663e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcb3e6640, SUC_lexicon_tags_31 },
    { 0x685e6641, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13e6646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38be664c, SUC_lexicon_tags_29 },
    { 0xf27e664d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1e6653, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25e665e, SUC_lexicon_tags_1 },
    { 0xe43e665f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb11e6661, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fe6664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4be6668, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x4ffe666a, SUC_lexicon_tags_66 },
    { 0x88be666b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11de6671, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6de6675, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbe6679, SUC_lexicon_tags_23 },
    { 0x303e667a, SUC_lexicon_tags_12 },
    { 0x499e667b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6e3e667d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x845e6680, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17e6684, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fe6688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1e668f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8fe6691, SUC_lexicon_tags_25 },
    { 0xa4de6692, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x607e6694, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49e6697, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x669e6699, SUC_lexicon_tags_7 },
    { 0x641e669a, SUC_lexicon_tags_56 },
    { 0x71e6699, SUC_lexicon_tags_0 },
    { 0xbfbe669a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9e66a0, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x9ede66a2, SUC_lexicon_tags_38 },
    { 0x741e66a3, SUC_lexicon_tags_15 },
    { 0x4b3e66a3, SUC_lexicon_tags_27 },
    { 0x26be66a5, SUC_lexicon_tags_60 },
    { 0x4cfe66a6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xede66a8, SUC_lexicon_tags_17 },
    { 0xb6de66a9, SUC_lexicon_tags_11 },
    { 0xaade66aa, SUC_lexicon_tags_19 },
    { 0xdcbe66ab, SUC_lexicon_tags_29 },
    { 0xe93e66aa, SUC_lexicon_tags_25 },
    { 0x317e66ad, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x461e66b0, SUC_lexicon_tags_572 },
    { 0xf5fe66b0, SUC_lexicon_tags_103 },
    { 0x9c5e66b1, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73e66b6, SUC_lexicon_tags_57 },
    { 0x7de66b6, SUC_lexicon_tags_17 },
    { 0xff9e66b6, SUC_lexicon_tags_1 },
    { 0xe2be66b9, SUC_lexicon_tags_23 },
    { 0x13e66b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3e66bf, SUC_lexicon_tags_23 },
    { 0xb6fe66c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961e66c6, SUC_lexicon_tags_45 },
    { 0xd57e66c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa73e66c9, SUC_lexicon_tags_6 },
    { 0xef5e66ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x477e66cc, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949e66d8, SUC_lexicon_tags_17 },
    { 0x9dde66d8, SUC_lexicon_tags_37 },
    { 0xa01e66d8, SUC_lexicon_tags_7 },
    { 0xb6fe66db, SUC_lexicon_tags_19 },
    { 0x201e66d9, SUC_lexicon_tags_55 },
    { 0x74de66dd, SUC_lexicon_tags_37 },
    { 0x12fe66de, SUC_lexicon_tags_39 },
    { 0xee1e66de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fe66e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd23e66e5, SUC_lexicon_tags_28 },
    { 0xd6be66e6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x865e66e9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x2cbe66eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e66ef, SUC_lexicon_tags_34 },
    { 0xcade66ef, SUC_lexicon_tags_16 },
    { 0x657e66ef, SUC_lexicon_tags_25 },
    { 0xac3e66f2, SUC_lexicon_tags_10 },
    { 0xf7fe66f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fe66f6, SUC_lexicon_tags_82 },
    { 0x52de66f6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba7e66fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56fe66ff, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4ffe6701, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87e6703, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebe670f, SUC_lexicon_tags_49 },
    { 0x67e6710, SUC_lexicon_tags_2 },
    { 0x47e6711, SUC_lexicon_tags_15 },
    { 0xc77e6712, SUC_lexicon_tags_49 },
    { 0x2f1e6710, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefe671b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78de6721, SUC_lexicon_tags_25 },
    { 0x501e6722, SUC_lexicon_tags_0 },
    { 0xba9e6721, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x36de6725, SUC_lexicon_tags_11 },
    { 0xea7e6726, SUC_lexicon_tags_11 },
    { 0xa53e6727, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21be672b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbe672e, SUC_lexicon_tags_37 },
    { 0xd17e672f, SUC_lexicon_tags_27 },
    { 0x6fde6730, SUC_lexicon_tags_7 },
    { 0x3bde672f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909e673a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3e673f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55e6743, SUC_lexicon_tags_33 },
    { 0x637e6744, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x367e6747, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x229e674a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x59fe674c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19be6754, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3e675a, SUC_lexicon_tags_26 },
    { 0xa17e675b, SUC_lexicon_tags_11 },
    { 0x577e675c, SUC_lexicon_tags_25 },
    { 0x133e675d, SUC_lexicon_tags_1 },
    { 0xa2de675b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7e6760, SUC_lexicon_tags_25 },
    { 0xff1e6761, SUC_lexicon_tags_23 },
    { 0x9d7e6762, SUC_lexicon_tags_37 },
    { 0x897e6762, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fe6768, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19e676c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbd9e676e, SUC_lexicon_tags_25 },
    { 0x22be676f, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13de6773, SUC_lexicon_tags_17 },
    { 0x135e6774, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05e677d, SUC_lexicon_tags_17 },
    { 0xf27e677e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc3de6780, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687e6784, SUC_lexicon_tags_1 },
    { 0xe5be6784, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1e678a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x78de678c, SUC_lexicon_tags_23 },
    { 0x4be678c, SUC_lexicon_tags_47 },
    { 0xb23e678e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x66be6791, SUC_lexicon_tags_23 },
    { 0x13fe6791, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7be6795, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249e679a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1e679f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31e67a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3e67ac, SUC_lexicon_tags_7 },
    { 0x247e67ad, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa43e67af, SUC_lexicon_tags_11 },
    { 0xa6fe67b0, SUC_lexicon_tags_50 },
    { 0x59de67b1, SUC_lexicon_tags_12 },
    { 0xbf1e67af, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5e67b5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fe67b8, SUC_lexicon_tags_12 },
    { 0xde67b8, SUC_lexicon_tags_83 },
    { 0x663e67ba, SUC_lexicon_tags_0 },
    { 0x58be67b8, SUC_lexicon_tags_17 },
    { 0xf8de67bb, SUC_lexicon_tags_37 },
    { 0xd33e67bd, SUC_lexicon_tags_38 },
    { 0xe1de67be, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89de67c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1e67c7, SUC_lexicon_tags_65 },
    { 0x989e67c8, SUC_lexicon_tags_69 },
    { 0x5e9e67c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x18be67cb, SUC_lexicon_tags_66 },
    { 0x42fe67cc, SUC_lexicon_tags_29 },
    { 0x377e67cc, SUC_lexicon_tags_118 },
    { 0xbb7e67ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x973e67d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92de67d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35fe67d7, SUC_lexicon_tags_3 },
    { 0x907e67d8, SUC_lexicon_tags_79 },
    { 0x177e67d8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x127e67db, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0x61be67dd, SUC_lexicon_tags_0 },
    { 0x3ebe67de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x39e67e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1de67e2, SUC_lexicon_tags_15 },
    { 0xeebe67e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5be67e6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5de67e9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5e67ed, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4be67f1, SUC_lexicon_tags_4 },
    { 0xdbbe67f1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93e67f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539e67f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x41de67fb, SUC_lexicon_tags_7 },
    { 0x879e67fc, SUC_lexicon_tags_61 },
    { 0x7a5e67fc, SUC_lexicon_tags_6 },
    { 0xa5de67fe, SUC_lexicon_tags_14 },
    { 0xe9be67fe, SUC_lexicon_tags_49 },
    { 0xb03e6800, SUC_lexicon_tags_66 },
    { 0xe5fe6801, SUC_lexicon_tags_2 },
    { 0xf79e6800, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe1de6804, SUC_lexicon_tags_6 },
    { 0x2abe6804, SUC_lexicon_tags_11 },
    { 0x803e6805, SUC_lexicon_tags_53 },
    { 0xc15e6806, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05e680a, SUC_lexicon_tags_25 },
    { 0x8abe680b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1abe680e, SUC_lexicon_tags_11 },
    { 0x38be680f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5e6812, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfe6815, SUC_lexicon_tags_37 },
    { 0xd63e6816, SUC_lexicon_tags_49 },
    { 0xd45e6816, SUC_lexicon_tags_14 },
    { 0x7f9e6817, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb27e681a, SUC_lexicon_tags_66 },
    { 0xb1e681b, SUC_lexicon_tags_63 },
    { 0xb97e681c, SUC_lexicon_tags_25 },
    { 0xe8de681c, SUC_lexicon_tags_38 },
    { 0x3ade681e, SUC_lexicon_tags_15 },
    { 0x1be681e, SUC_lexicon_tags_15 },
    { 0xd1e681f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fe6824, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbf7e6826, SUC_lexicon_tags_37 },
    { 0xd93e6826, SUC_lexicon_tags_151 },
    { 0xffde6828, SUC_lexicon_tags_12 },
    { 0x303e6829, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7b3e682b, SUC_lexicon_tags_25 },
    { 0x177e682c, SUC_lexicon_tags_2 },
    { 0x13e682d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09e6837, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79de683b, SUC_lexicon_tags_11 },
    { 0x153e683c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837e6842, SUC_lexicon_tags_37 },
    { 0x9d1e6843, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x9f1e6845, SUC_lexicon_tags_37 },
    { 0x20fe6846, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc35e6848, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba3e684a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4f3e684c, SUC_lexicon_tags_43 },
    { 0x503e684c, SUC_lexicon_tags_11 },
    { 0xa19e684d, SUC_lexicon_tags_42 },
    { 0xfc5e684d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x725e6851, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373e6859, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f9e685b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x103e685e, SUC_lexicon_tags_27 },
    { 0xddde685e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x71e6861, SUC_lexicon_tags_4 },
    { 0x8dbe6861, SUC_lexicon_tags_47 },
    { 0x891e6863, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73fe6865, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229e686a, SUC_lexicon_tags_79 },
    { 0x58be686b, SUC_lexicon_tags_27 },
    { 0xe15e686c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf81e686e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5be6878, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99be687c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x595e687e, SUC_lexicon_tags_87 },
    { 0xf3fe687e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491e6884, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67e6889, SUC_lexicon_tags_32 },
    { 0xf69e6889, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6f1e688c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3e688f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x745e6892, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe55e6894, SUC_lexicon_tags_2 },
    { 0x383e6894, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e689c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9e68a1, SUC_lexicon_tags_25 },
    { 0x481e68a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01e68a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73e68a9, SUC_lexicon_tags_28 },
    { 0xf7de68aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7e68af, SUC_lexicon_tags_15 },
    { 0x9e9e68b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd89e68b2, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbe68b7, SUC_lexicon_tags_32 },
    { 0x6ebe68b8, SUC_lexicon_tags_0 },
    { 0xb71e68b9, SUC_lexicon_tags_55 },
    { 0x863e68ba, SUC_lexicon_tags_3 },
    { 0xf41e68ba, SUC_lexicon_tags_17 },
    { 0xc3fe68ba, SUC_lexicon_tags_17 },
    { 0x267e68bd, SUC_lexicon_tags_42 },
    { 0xc2fe68b9, SUC_lexicon_tags_38 },
    { 0x3a3e68b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc3e68c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x155e68c3, SUC_lexicon_tags_2 },
    { 0xd83e68c4, SUC_lexicon_tags_12 },
    { 0x42fe68c5, SUC_lexicon_tags_14 },
    { 0xd19e68c6, SUC_lexicon_tags_45 },
    { 0xc19e68c7, SUC_lexicon_tags_49 },
    { 0x97be68c8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197e68cc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xedde68cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa89e68d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35e68d6, SUC_lexicon_tags_30 },
    { 0x32de68d7, SUC_lexicon_tags_25 },
    { 0xeede68d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3e68dc, SUC_lexicon_tags_11 },
    { 0xe1be68dd, SUC_lexicon_tags_37 },
    { 0xecfe68de, SUC_lexicon_tags_2 },
    { 0xf27e68df, SUC_lexicon_tags_17 },
    { 0x21fe68e0, SUC_lexicon_tags_38 },
    { 0x6e3e68e1, SUC_lexicon_tags_28 },
    { 0xcc7e68e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x68de68e4, SUC_lexicon_tags_47 },
    { 0xa8de68e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51be68e7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1e68ea, SUC_lexicon_tags_45 },
    { 0xab1e68eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x271e68ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fe68f1, SUC_lexicon_tags_17 },
    { 0xba3e68f2, SUC_lexicon_tags_23 },
    { 0x1cbe68f3, SUC_lexicon_tags_14 },
    { 0x18be68f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x70de68f7, SUC_lexicon_tags_15 },
    { 0x359e68f8, SUC_lexicon_tags_12 },
    { 0x333e68f8, SUC_lexicon_tags_279 },
    { 0, NULL },
    { 0x7c1e68fb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2de6904, SUC_lexicon_tags_44 },
    { 0x2f3e6905, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfe6913, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x769e6916, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x691e6918, SUC_lexicon_tags_25 },
    { 0xa57e6919, SUC_lexicon_tags_17 },
    { 0x69e691a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x201e691c, SUC_lexicon_tags_17 },
    { 0x4d7e691c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x347e691f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b9e6921, SUC_lexicon_tags_30 },
    { 0x74fe6922, SUC_lexicon_tags_23 },
    { 0x8bbe6922, SUC_lexicon_tags_46 },
    { 0xe63e6922, SUC_lexicon_tags_66 },
    { 0x331e6922, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfe6928, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afe692f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x711e6931, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b1e6933, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x265e6935, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0x409e6938, SUC_lexicon_tags_29 },
    { 0x11e6939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc1e693b, SUC_lexicon_tags_25 },
    { 0xbd5e693c, SUC_lexicon_tags_65 },
    { 0x8c9e693b, SUC_lexicon_tags_30 },
    { 0x273e693e, SUC_lexicon_tags_82 },
    { 0x275e693d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5efe6941, SUC_lexicon_tags_25 },
    { 0x8b7e6941, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfe6945, SUC_lexicon_tags_15 },
    { 0x627e6946, SUC_lexicon_tags_25 },
    { 0xddfe6947, SUC_lexicon_tags_1 },
    { 0x271e6946, SUC_lexicon_tags_14 },
    { 0x595e6945, SUC_lexicon_tags_43 },
    { 0xbd9e694a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb9e694c, SUC_lexicon_tags_11 },
    { 0x3d7e694d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f3e694f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcf5e6951, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x12de6953, SUC_lexicon_tags_33 },
    { 0xd63e6954, SUC_lexicon_tags_0 },
    { 0xa47e6954, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9a3e6957, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x797e6959, SUC_lexicon_tags_25 },
    { 0x885e6959, SUC_lexicon_tags_37 },
    { 0xae1e695b, SUC_lexicon_tags_23 },
    { 0x947e695c, SUC_lexicon_tags_87 },
    { 0xf23e6959, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5be6960, SUC_lexicon_tags_15 },
    { 0x151e6960, SUC_lexicon_tags_33 },
    { 0x469e6962, SUC_lexicon_tags_33 },
    { 0x413e6963, SUC_lexicon_tags_2 },
    { 0xb01e6964, SUC_lexicon_tags_48 },
    { 0x309e6964, SUC_lexicon_tags_26 },
    { 0x2b1e6966, SUC_lexicon_tags_11 },
    { 0x36be6965, SUC_lexicon_tags_66 },
    { 0x9d1e6964, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fe696e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x551e6971, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6be6974, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x987e6977, SUC_lexicon_tags_63 },
    { 0x6f5e6978, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fe6980, SUC_lexicon_tags_689 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbe6984, SUC_lexicon_tags_47 },
    { 0x6e5e6984, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57e698b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5fe698d, SUC_lexicon_tags_43 },
    { 0xa17e698e, SUC_lexicon_tags_25 },
    { 0xb4be698d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7de6993, SUC_lexicon_tags_57 },
    { 0xca3e6994, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfe69a1, SUC_lexicon_tags_11 },
    { 0x761e69a1, SUC_lexicon_tags_65 },
    { 0x39de69a3, SUC_lexicon_tags_23 },
    { 0x6efe69a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bde69a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34be69a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfde69aa, SUC_lexicon_tags_7 },
    { 0xcd5e69ab, SUC_lexicon_tags_23 },
    { 0xa83e69ac, SUC_lexicon_tags_17 },
    { 0x6e5e69ab, SUC_lexicon_tags_12 },
    { 0xd4be69ae, SUC_lexicon_tags_1 },
    { 0xf65e69af, SUC_lexicon_tags_37 },
    { 0xf93e69ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205e69b8, SUC_lexicon_tags_53 },
    { 0x8b9e69b8, SUC_lexicon_tags_29 },
    { 0x359e69ba, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7e69bd, SUC_lexicon_tags_17 },
    { 0xb61e69be, SUC_lexicon_tags_7 },
    { 0x24be69be, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d1e69c1, SUC_lexicon_tags_7 },
    { 0xb1fe69c2, SUC_lexicon_tags_28 },
    { 0x1b9e69c2, SUC_lexicon_tags_37 },
    { 0xbb7e69c4, SUC_lexicon_tags_21 },
    { 0xcc7e69c5, SUC_lexicon_tags_27 },
    { 0x3e1e69c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x655e69c9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397e69cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x54be69d1, SUC_lexicon_tags_56 },
    { 0xc2be69d2, SUC_lexicon_tags_23 },
    { 0xd61e69d2, SUC_lexicon_tags_37 },
    { 0xaf1e69d4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x157e69d6, SUC_lexicon_tags_235 },
    { 0xcd1e69d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x603e69d9, SUC_lexicon_tags_1 },
    { 0x167e69d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607e69de, SUC_lexicon_tags_17 },
    { 0xa7be69de, SUC_lexicon_tags_23 },
    { 0x9be69df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34de69e2, SUC_lexicon_tags_45 },
    { 0x1e9e69e2, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825e69e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7e69eb, SUC_lexicon_tags_54 },
    { 0x869e69eb, SUC_lexicon_tags_47 },
    { 0x8de69ed, SUC_lexicon_tags_1 },
    { 0x351e69ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x671e69f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x403e69f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95e69f6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557e69fb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77e6a01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7e6a04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x665e6a06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x863e6a09, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423e6a0d, SUC_lexicon_tags_49 },
    { 0x8b1e6a0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bde6a11, SUC_lexicon_tags_17 },
    { 0xda1e6a12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dbe6a14, SUC_lexicon_tags_7 },
    { 0x583e6a15, SUC_lexicon_tags_14 },
    { 0xbc7e6a16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x191e6a18, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723e6a20, SUC_lexicon_tags_12 },
    { 0x6b1e6a21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfe6a24, SUC_lexicon_tags_38 },
    { 0x8a9e6a24, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x771e6a28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78de6a2e, SUC_lexicon_tags_16 },
    { 0xf0de6a2e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdf3e6a31, SUC_lexicon_tags_0 },
    { 0xbd7e6a31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91e6a35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe25e6a37, SUC_lexicon_tags_45 },
    { 0x731e6a38, SUC_lexicon_tags_17 },
    { 0x1d5e6a38, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fe6a3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xddde6a3e, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xb63e6a40, SUC_lexicon_tags_17 },
    { 0x52de6a40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7e6a44, SUC_lexicon_tags_55 },
    { 0xe31e6a45, SUC_lexicon_tags_34 },
    { 0xf4de6a46, SUC_lexicon_tags_38 },
    { 0xf11e6a47, SUC_lexicon_tags_8 },
    { 0x613e6a48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7e6a4b, SUC_lexicon_tags_15 },
    { 0xb45e6a4c, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ade6a52, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e6a56, SUC_lexicon_tags_30 },
    { 0x33fe6a57, SUC_lexicon_tags_21 },
    { 0xdb3e6a58, SUC_lexicon_tags_60 },
    { 0x337e6a56, SUC_lexicon_tags_60 },
    { 0x1a1e6a5a, SUC_lexicon_tags_49 },
    { 0x841e6a5b, SUC_lexicon_tags_1 },
    { 0xa81e6a5b, SUC_lexicon_tags_2 },
    { 0x853e6a5c, SUC_lexicon_tags_87 },
    { 0x511e6a5e, SUC_lexicon_tags_1 },
    { 0x3cde6a5f, SUC_lexicon_tags_11 },
    { 0x5c3e6a5e, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebe6a69, SUC_lexicon_tags_38 },
    { 0xe4be6a69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253e6a73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865e6a77, SUC_lexicon_tags_8 },
    { 0x61be6a77, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xec9e6a7b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbf7e6a7d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7e6a86, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e6a8c, SUC_lexicon_tags_37 },
    { 0x185e6a8d, SUC_lexicon_tags_19 },
    { 0x369e6a8d, SUC_lexicon_tags_38 },
    { 0xdc1e6a8c, SUC_lexicon_tags_31 },
    { 0xe9be6a90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x347e6a93, SUC_lexicon_tags_37 },
    { 0x5f9e6a94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3e6a9b, SUC_lexicon_tags_87 },
    { 0xee5e6a9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfe6aa2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19e6aa6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdde6aac, SUC_lexicon_tags_17 },
    { 0xcf5e6aac, SUC_lexicon_tags_47 },
    { 0x2b7e6aae, SUC_lexicon_tags_11 },
    { 0xe3fe6aaf, SUC_lexicon_tags_8 },
    { 0x437e6aae, SUC_lexicon_tags_27 },
    { 0xe2fe6aae, SUC_lexicon_tags_15 },
    { 0xb0fe6ab1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37be6ab6, SUC_lexicon_tags_43 },
    { 0x96be6ab7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfe6abd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9e6ac1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e6ac4, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283e6ac9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fe6acc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417e6ad1, SUC_lexicon_tags_14 },
    { 0x7f7e6ad1, SUC_lexicon_tags_33 },
    { 0x62be6ad3, SUC_lexicon_tags_11 },
    { 0xf83e6ad4, SUC_lexicon_tags_37 },
    { 0x379e6ad2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5b1e6ad7, SUC_lexicon_tags_2 },
    { 0x9e5e6ad8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90be6ade, SUC_lexicon_tags_1 },
    { 0x31be6adf, SUC_lexicon_tags_17 },
    { 0xd5e6ae0, SUC_lexicon_tags_0 },
    { 0xe75e6ae1, SUC_lexicon_tags_87 },
    { 0xc95e6ae2, SUC_lexicon_tags_25 },
    { 0xb73e6ae3, SUC_lexicon_tags_590 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fe6ae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d3e6ae8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x9a7e6aea, SUC_lexicon_tags_17 },
    { 0x7bde6aeb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x88be6aed, SUC_lexicon_tags_7 },
    { 0xdc3e6aed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e9e6af0, SUC_lexicon_tags_0 },
    { 0xbd1e6af1, SUC_lexicon_tags_59 },
    { 0xedbe6af2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5e6af5, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xdcbe6af7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7dbe6af9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9e6afe, SUC_lexicon_tags_17 },
    { 0x13de6aff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebe6b05, SUC_lexicon_tags_38 },
    { 0x39fe6b05, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x9e9e6b08, SUC_lexicon_tags_29 },
    { 0x75be6b09, SUC_lexicon_tags_11 },
    { 0xa7e6b0a, SUC_lexicon_tags_296 },
    { 0, NULL },
    { 0, NULL },
    { 0x33e6b0d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785e6b11, SUC_lexicon_tags_38 },
    { 0x1de6b11, SUC_lexicon_tags_19 },
    { 0x71de6b13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3de6b19, SUC_lexicon_tags_55 },
    { 0x63be6b1a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xafe6b1d, SUC_lexicon_tags_23 },
    { 0xa59e6b1e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x72be6b20, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f3e6b22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5fe6b24, SUC_lexicon_tags_11 },
    { 0x803e6b25, SUC_lexicon_tags_2 },
    { 0x133e6b24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafe6b2a, SUC_lexicon_tags_25 },
    { 0xe7de6b2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2de6b2e, SUC_lexicon_tags_77 },
    { 0xd17e6b2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347e6b33, SUC_lexicon_tags_30 },
    { 0xc2de6b34, SUC_lexicon_tags_14 },
    { 0x231e6b35, SUC_lexicon_tags_17 },
    { 0x8b1e6b36, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1de6b39, SUC_lexicon_tags_16 },
    { 0xdcbe6b3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x64be6b3c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2c3e6b3e, SUC_lexicon_tags_11 },
    { 0xd3fe6b3e, SUC_lexicon_tags_14 },
    { 0x289e6b40, SUC_lexicon_tags_23 },
    { 0x241e6b41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3e6b49, SUC_lexicon_tags_45 },
    { 0xa3de6b4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21e6b4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5e6b50, SUC_lexicon_tags_27 },
    { 0xcc5e6b50, SUC_lexicon_tags_1 },
    { 0xe65e6b50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf89e6b54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fe6b57, SUC_lexicon_tags_38 },
    { 0x501e6b58, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973e6b60, SUC_lexicon_tags_37 },
    { 0x817e6b61, SUC_lexicon_tags_33 },
    { 0x859e6b61, SUC_lexicon_tags_12 },
    { 0xbcde6b63, SUC_lexicon_tags_12 },
    { 0xeffe6b64, SUC_lexicon_tags_30 },
    { 0x8bde6b65, SUC_lexicon_tags_224 },
    { 0x28de6b66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbcde6b68, SUC_lexicon_tags_17 },
    { 0x1ede6b69, SUC_lexicon_tags_33 },
    { 0x69fe6b68, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbe6b6d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9de6b6f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xedfe6b71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x42fe6b73, SUC_lexicon_tags_68 },
    { 0x51be6b74, SUC_lexicon_tags_17 },
    { 0x2e1e6b74, SUC_lexicon_tags_31 },
    { 0x923e6b76, SUC_lexicon_tags_8 },
    { 0xb7de6b77, SUC_lexicon_tags_11 },
    { 0x9f7e6b78, SUC_lexicon_tags_60 },
    { 0x4b7e6b74, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efe6b7d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xabe6b7f, SUC_lexicon_tags_17 },
    { 0xe37e6b7f, SUC_lexicon_tags_7 },
    { 0x9bfe6b81, SUC_lexicon_tags_29 },
    { 0x53e6b82, SUC_lexicon_tags_12 },
    { 0x4e5e6b81, SUC_lexicon_tags_63 },
    { 0xa29e6b84, SUC_lexicon_tags_37 },
    { 0xd69e6b84, SUC_lexicon_tags_17 },
    { 0x8fe6b85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995e6b8a, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13fe6b8f, SUC_lexicon_tags_29 },
    { 0xc6be6b90, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x217e6b92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5e6b96, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x185e6b99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe6b9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebde6ba1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1de6ba5, SUC_lexicon_tags_3 },
    { 0xd03e6ba6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf5de6ba8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x39be6bab, SUC_lexicon_tags_15 },
    { 0x535e6bac, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x53be6baf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x241e6bb1, SUC_lexicon_tags_2 },
    { 0xdf1e6bb1, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cde6bb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd79e6bb7, SUC_lexicon_tags_2 },
    { 0xc13e6bb8, SUC_lexicon_tags_17 },
    { 0x5a7e6bb9, SUC_lexicon_tags_61 },
    { 0x13e6bba, SUC_lexicon_tags_8 },
    { 0x89be6bbb, SUC_lexicon_tags_2 },
    { 0xd27e6bbb, SUC_lexicon_tags_25 },
    { 0x80de6bbd, SUC_lexicon_tags_47 },
    { 0xfafe6bbd, SUC_lexicon_tags_79 },
    { 0x851e6bbf, SUC_lexicon_tags_28 },
    { 0xcd9e6bc0, SUC_lexicon_tags_17 },
    { 0xe73e6bbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbebe6bc3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbe6bc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abe6bcf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x207e6bd1, SUC_lexicon_tags_63 },
    { 0x8d9e6bd1, SUC_lexicon_tags_2 },
    { 0xa1fe6bd2, SUC_lexicon_tags_33 },
    { 0x1efe6bd1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8e7e6bd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x50fe6bd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19e6bdb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa75e6bdd, SUC_lexicon_tags_33 },
    { 0xdfde6bde, SUC_lexicon_tags_74 },
    { 0xc95e6bdf, SUC_lexicon_tags_15 },
    { 0x13e6be0, SUC_lexicon_tags_26 },
    { 0xb45e6be1, SUC_lexicon_tags_61 },
    { 0x5b3e6be1, SUC_lexicon_tags_45 },
    { 0x7cbe6be3, SUC_lexicon_tags_37 },
    { 0x315e6be4, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xc19e6be6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75e6bea, SUC_lexicon_tags_7 },
    { 0xcd3e6beb, SUC_lexicon_tags_39 },
    { 0x8f1e6beb, SUC_lexicon_tags_1 },
    { 0x6b5e6bed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fe6bf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb2de6bf2, SUC_lexicon_tags_9 },
    { 0x965e6bf2, SUC_lexicon_tags_47 },
    { 0xda5e6bf4, SUC_lexicon_tags_7 },
    { 0x5a1e6bf4, SUC_lexicon_tags_10 },
    { 0x999e6bf6, SUC_lexicon_tags_1 },
    { 0xc97e6bf7, SUC_lexicon_tags_12 },
    { 0x851e6bf6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2de6bfc, SUC_lexicon_tags_1 },
    { 0x469e6bfc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x309e6bff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3e6c03, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x7e5e6c05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6de6c0d, SUC_lexicon_tags_45 },
    { 0x725e6c0e, SUC_lexicon_tags_26 },
    { 0x9b5e6c0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97e6c12, SUC_lexicon_tags_11 },
    { 0x41e6c12, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11de6c15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x93e6c17, SUC_lexicon_tags_17 },
    { 0x34fe6c18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa55e6c1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb1e6c1c, SUC_lexicon_tags_7 },
    { 0xa01e6c1c, SUC_lexicon_tags_31 },
    { 0xbd1e6c1c, SUC_lexicon_tags_61 },
    { 0xdafe6c1f, SUC_lexicon_tags_25 },
    { 0x27be6c1f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x621e6c22, SUC_lexicon_tags_14 },
    { 0xd9fe6c23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7c7e6c25, SUC_lexicon_tags_12 },
    { 0x32de6c26, SUC_lexicon_tags_1 },
    { 0xf85e6c26, SUC_lexicon_tags_17 },
    { 0x117e6c28, SUC_lexicon_tags_27 },
    { 0xa99e6c29, SUC_lexicon_tags_7 },
    { 0x91be6c27, SUC_lexicon_tags_14 },
    { 0xe41e6c2b, SUC_lexicon_tags_79 },
    { 0xe0de6c2c, SUC_lexicon_tags_16 },
    { 0xc5de6c2c, SUC_lexicon_tags_15 },
    { 0x90be6c26, SUC_lexicon_tags_17 },
    { 0xde9e6c2f, SUC_lexicon_tags_39 },
    { 0xbe1e6c30, SUC_lexicon_tags_113 },
    { 0xdabe6c31, SUC_lexicon_tags_39 },
    { 0x197e6c30, SUC_lexicon_tags_26 },
    { 0xe39e6c33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3e6c36, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88fe6c3d, SUC_lexicon_tags_21 },
    { 0x9cde6c3e, SUC_lexicon_tags_0 },
    { 0x92fe6c3e, SUC_lexicon_tags_11 },
    { 0x3b5e6c3e, SUC_lexicon_tags_32 },
    { 0x135e6c41, SUC_lexicon_tags_17 },
    { 0x9d5e6c3e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57de6c46, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12be6c4f, SUC_lexicon_tags_49 },
    { 0x41de6c4f, SUC_lexicon_tags_31 },
    { 0x80be6c50, SUC_lexicon_tags_63 },
    { 0x39de6c4f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fe6c57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x207e6c59, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fe6c5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf9be6c5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbe6c65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15e6c6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9e6c6d, SUC_lexicon_tags_19 },
    { 0x951e6c6e, SUC_lexicon_tags_1 },
    { 0xfe7e6c6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7be6c72, SUC_lexicon_tags_17 },
    { 0xf95e6c73, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5e6c76, SUC_lexicon_tags_45 },
    { 0xcc5e6c77, SUC_lexicon_tags_65 },
    { 0xb71e6c78, SUC_lexicon_tags_30 },
    { 0x295e6c79, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809e6c7d, SUC_lexicon_tags_17 },
    { 0xcc3e6c7e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbe6c83, SUC_lexicon_tags_12 },
    { 0x779e6c84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75be6c8a, SUC_lexicon_tags_33 },
    { 0x675e6c8b, SUC_lexicon_tags_31 },
    { 0xdd9e6c8b, SUC_lexicon_tags_1 },
    { 0x51fe6c8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x577e6c8f, SUC_lexicon_tags_15 },
    { 0xac7e6c8f, SUC_lexicon_tags_11 },
    { 0x6f5e6c91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x261e6c94, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91de6c99, SUC_lexicon_tags_8 },
    { 0x139e6c9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fe6c9d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x677e6ca0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1b5e6ca2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0be6ca7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xbbbe6ca9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32de6cad, SUC_lexicon_tags_35 },
    { 0x85de6cad, SUC_lexicon_tags_1 },
    { 0x715e6caf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5e6cb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203e6cb7, SUC_lexicon_tags_2 },
    { 0x9ade6cb8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05e6cbd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697e6cc2, SUC_lexicon_tags_2 },
    { 0xfdde6cc3, SUC_lexicon_tags_16 },
    { 0x1e6cc3, SUC_lexicon_tags_63 },
    { 0xeebe6cc5, SUC_lexicon_tags_22 },
    { 0xf81e6cc6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1e6cc9, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63e6ccf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x489e6cd1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cde6cd6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf71e6cd8, SUC_lexicon_tags_47 },
    { 0xc59e6cd9, SUC_lexicon_tags_46 },
    { 0x1e3e6cd8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29be6cdf, SUC_lexicon_tags_0 },
    { 0x8e1e6ce0, SUC_lexicon_tags_47 },
    { 0x7bbe6ce1, SUC_lexicon_tags_49 },
    { 0x7bde6ce2, SUC_lexicon_tags_7 },
    { 0xbbde6ce0, SUC_lexicon_tags_20 },
    { 0x9fe6cdf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x245e6ce7, SUC_lexicon_tags_87 },
    { 0xa19e6ce8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe0be6cea, SUC_lexicon_tags_2 },
    { 0x669e6ceb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x443e6ced, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afe6cf0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29e6cf3, SUC_lexicon_tags_38 },
    { 0x53e6cf3, SUC_lexicon_tags_405 },
    { 0xe21e6cf5, SUC_lexicon_tags_26 },
    { 0x107e6cf4, SUC_lexicon_tags_1 },
    { 0x4d5e6cf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fe6d02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567e6d0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3e6d0c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf3e6d0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9e6d12, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x15e6d14, SUC_lexicon_tags_6 },
    { 0x487e6d15, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68be6d1a, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0xf15e6d1c, SUC_lexicon_tags_17 },
    { 0x37e6d1d, SUC_lexicon_tags_25 },
    { 0x135e6d1e, SUC_lexicon_tags_87 },
    { 0xe43e6d1e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1e6d26, SUC_lexicon_tags_37 },
    { 0x4b7e6d27, SUC_lexicon_tags_14 },
    { 0x267e6d28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1e6d2b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfe6d31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5afe6d33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fe6d40, SUC_lexicon_tags_85 },
    { 0x65de6d41, SUC_lexicon_tags_63 },
    { 0xabde6d42, SUC_lexicon_tags_368 },
    { 0, NULL },
    { 0x5c5e6d44, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93e6d47, SUC_lexicon_tags_23 },
    { 0xe7fe6d48, SUC_lexicon_tags_25 },
    { 0x97de6d48, SUC_lexicon_tags_0 },
    { 0xb4de6d4a, SUC_lexicon_tags_12 },
    { 0x9de6d4a, SUC_lexicon_tags_77 },
    { 0x1a9e6d47, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703e6d50, SUC_lexicon_tags_61 },
    { 0xa5de6d51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b9e6d53, SUC_lexicon_tags_28 },
    { 0x53fe6d54, SUC_lexicon_tags_43 },
    { 0xb1de6d55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5e6d5d, SUC_lexicon_tags_18 },
    { 0x3d3e6d5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcde6d64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5e6d67, SUC_lexicon_tags_25 },
    { 0x123e6d68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9e6d6c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfe6d70, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdd5e6d72, SUC_lexicon_tags_11 },
    { 0xa45e6d73, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fe6d76, SUC_lexicon_tags_25 },
    { 0xbbbe6d77, SUC_lexicon_tags_29 },
    { 0xbabe6d78, SUC_lexicon_tags_9 },
    { 0x181e6d77, SUC_lexicon_tags_46 },
    { 0x20be6d7a, SUC_lexicon_tags_47 },
    { 0xdf7e6d79, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x759e6d7e, SUC_lexicon_tags_60 },
    { 0x401e6d7e, SUC_lexicon_tags_43 },
    { 0x9efe6d7f, SUC_lexicon_tags_0 },
    { 0x191e6d7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5dde6d83, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7e6d87, SUC_lexicon_tags_23 },
    { 0x693e6d87, SUC_lexicon_tags_2 },
    { 0x393e6d89, SUC_lexicon_tags_7 },
    { 0x91be6d8a, SUC_lexicon_tags_15 },
    { 0x95fe6d8b, SUC_lexicon_tags_12 },
    { 0xbdbe6d8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3e6d92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x687e6d94, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993e6d9b, SUC_lexicon_tags_1 },
    { 0xb0be6d9c, SUC_lexicon_tags_82 },
    { 0xabfe6d9d, SUC_lexicon_tags_33 },
    { 0x1bfe6d9c, SUC_lexicon_tags_61 },
    { 0x607e6d9f, SUC_lexicon_tags_15 },
    { 0x9ede6da0, SUC_lexicon_tags_11 },
    { 0x439e6d9e, SUC_lexicon_tags_87 },
    { 0x511e6da2, SUC_lexicon_tags_7 },
    { 0xd55e6da1, SUC_lexicon_tags_25 },
    { 0xf8fe6da4, SUC_lexicon_tags_91 },
    { 0x65e6d9d, SUC_lexicon_tags_38 },
    { 0xe53e6da6, SUC_lexicon_tags_16 },
    { 0xe47e6da7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc2de6da9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3e6dae, SUC_lexicon_tags_37 },
    { 0xe37e6daf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x75de6db1, SUC_lexicon_tags_68 },
    { 0xddfe6db1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9be6dbb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x93be6dbd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9c7e6dbf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfe6dc3, SUC_lexicon_tags_11 },
    { 0xeb1e6dc4, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0x5c5e6dc6, SUC_lexicon_tags_60 },
    { 0x2c1e6dc7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xefde6dc9, SUC_lexicon_tags_14 },
    { 0xac1e6dca, SUC_lexicon_tags_4 },
    { 0x1e3e6dcb, SUC_lexicon_tags_116 },
    { 0x14be6dcc, SUC_lexicon_tags_6 },
    { 0x64be6dcd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad3e6dcf, SUC_lexicon_tags_15 },
    { 0x945e6dcf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5e6dd3, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x8d5e6dd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x101e6dd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9e6ddb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbe6ddd, SUC_lexicon_tags_38 },
    { 0x23be6dde, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9e6de3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x489e6de5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9e6de8, SUC_lexicon_tags_37 },
    { 0xd1fe6de9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05e6ded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdde6df3, SUC_lexicon_tags_49 },
    { 0x1f1e6df4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bde6df8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x367e6dfb, SUC_lexicon_tags_15 },
    { 0xb7de6dfb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x271e6e01, SUC_lexicon_tags_11 },
    { 0x51fe6e02, SUC_lexicon_tags_38 },
    { 0xde6e03, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0de6e06, SUC_lexicon_tags_2 },
    { 0xc07e6e07, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79e6e0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857e6e11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43fe6e14, SUC_lexicon_tags_43 },
    { 0xb67e6e15, SUC_lexicon_tags_25 },
    { 0xc35e6e15, SUC_lexicon_tags_25 },
    { 0xea3e6e15, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e6e1a, SUC_lexicon_tags_14 },
    { 0x88fe6e1a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25e6e1e, SUC_lexicon_tags_68 },
    { 0x6b5e6e1f, SUC_lexicon_tags_0 },
    { 0x863e6e1f, SUC_lexicon_tags_7 },
    { 0x80fe6e1f, SUC_lexicon_tags_27 },
    { 0x74de6e21, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495e6e2b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x681e6e2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fe6e35, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13e6e38, SUC_lexicon_tags_5 },
    { 0xea1e6e39, SUC_lexicon_tags_12 },
    { 0x50de6e3a, SUC_lexicon_tags_60 },
    { 0x9f5e6e39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3e6e3f, SUC_lexicon_tags_85 },
    { 0xe23e6e40, SUC_lexicon_tags_70 },
    { 0xd21e6e41, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597e6e47, SUC_lexicon_tags_0 },
    { 0x9b1e6e48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x501e6e4b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ebe6e4d, SUC_lexicon_tags_4 },
    { 0xb1fe6e4d, SUC_lexicon_tags_12 },
    { 0x9afe6e4f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbcbe6e51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x533e6e54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ede6e56, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57e6e5e, SUC_lexicon_tags_11 },
    { 0x99de6e5f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7ede6e61, SUC_lexicon_tags_27 },
    { 0xc4be6e62, SUC_lexicon_tags_27 },
    { 0x6ede6e62, SUC_lexicon_tags_11 },
    { 0x491e6e64, SUC_lexicon_tags_0 },
    { 0x1b3e6e63, SUC_lexicon_tags_65 },
    { 0xa83e6e66, SUC_lexicon_tags_9 },
    { 0xfe5e6e66, SUC_lexicon_tags_28 },
    { 0x4e5e6e67, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83de6e71, SUC_lexicon_tags_13 },
    { 0xfcde6e72, SUC_lexicon_tags_47 },
    { 0x813e6e73, SUC_lexicon_tags_38 },
    { 0xf41e6e74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93e6e7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577e6e86, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfe6e8f, SUC_lexicon_tags_37 },
    { 0xa1de6e90, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf85e6e92, SUC_lexicon_tags_63 },
    { 0xfe5e6e93, SUC_lexicon_tags_47 },
    { 0x2d3e6e94, SUC_lexicon_tags_1 },
    { 0x9b7e6e94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9e6e97, SUC_lexicon_tags_84 },
    { 0xf09e6e97, SUC_lexicon_tags_45 },
    { 0x579e6e99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x147e6e9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb81e6e9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93e6ea0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fde6ea2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827e6ea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527e6ead, SUC_lexicon_tags_25 },
    { 0xf9e6ead, SUC_lexicon_tags_14 },
    { 0xaebe6eaf, SUC_lexicon_tags_37 },
    { 0xe25e6eae, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x337e6eb2, SUC_lexicon_tags_1 },
    { 0x9afe6eb3, SUC_lexicon_tags_2 },
    { 0xb17e6eb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x92be6eb6, SUC_lexicon_tags_9 },
    { 0xccfe6eb7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x297e6eba, SUC_lexicon_tags_14 },
    { 0x9b9e6ebb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ede6ebe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xca1e6ec0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa6de6ec2, SUC_lexicon_tags_14 },
    { 0x89be6ec2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xee1e6ec5, SUC_lexicon_tags_60 },
    { 0x665e6ec6, SUC_lexicon_tags_9 },
    { 0x597e6ec5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3e6eca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xddbe6ecc, SUC_lexicon_tags_61 },
    { 0xb77e6ecd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe6ed0, SUC_lexicon_tags_15 },
    { 0x275e6ed1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93be6ed6, SUC_lexicon_tags_25 },
    { 0xc33e6ed6, SUC_lexicon_tags_15 },
    { 0x33fe6ed6, SUC_lexicon_tags_31 },
    { 0xe19e6ed9, SUC_lexicon_tags_61 },
    { 0xd65e6eda, SUC_lexicon_tags_7 },
    { 0x83be6edb, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9be6ede, SUC_lexicon_tags_56 },
    { 0x901e6ede, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cde6ee2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x589e6ee4, SUC_lexicon_tags_63 },
    { 0xebfe6ee5, SUC_lexicon_tags_2 },
    { 0xf69e6ee5, SUC_lexicon_tags_38 },
    { 0xce9e6ee6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1e6eea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e6eee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd37e6ef0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56be6ef7, SUC_lexicon_tags_62 },
    { 0xc27e6ef7, SUC_lexicon_tags_29 },
    { 0xd05e6ef9, SUC_lexicon_tags_53 },
    { 0xb9de6efa, SUC_lexicon_tags_31 },
    { 0xc9be6efb, SUC_lexicon_tags_0 },
    { 0x91e6efc, SUC_lexicon_tags_295 },
    { 0, NULL },
    { 0x817e6efe, SUC_lexicon_tags_17 },
    { 0x5f7e6eff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12be6f04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5e6f08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c3e6f0a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355e6f11, SUC_lexicon_tags_12 },
    { 0xed9e6f12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf63e6f14, SUC_lexicon_tags_33 },
    { 0xd89e6f15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x739e6f17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbe6f1b, SUC_lexicon_tags_17 },
    { 0xfb1e6f1b, SUC_lexicon_tags_484 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3e6f24, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb3e6f26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85e6f2b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637e6f30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xde9e6f32, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1be6f38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x779e6f3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xba7e6f3d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39e6f41, SUC_lexicon_tags_13 },
    { 0xd2be6f42, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x955e6f44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdebe6f46, SUC_lexicon_tags_37 },
    { 0xbe6f47, SUC_lexicon_tags_37 },
    { 0x51fe6f48, SUC_lexicon_tags_17 },
    { 0x1d7e6f49, SUC_lexicon_tags_19 },
    { 0x21fe6f4a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77e6f4d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1d7e6f4f, SUC_lexicon_tags_28 },
    { 0x7fbe6f50, SUC_lexicon_tags_2 },
    { 0xb8be6f50, SUC_lexicon_tags_15 },
    { 0x43fe6f52, SUC_lexicon_tags_45 },
    { 0x59be6f52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe51e6f55, SUC_lexicon_tags_66 },
    { 0x21e6f56, SUC_lexicon_tags_37 },
    { 0x5bfe6f57, SUC_lexicon_tags_10 },
    { 0x2f3e6f57, SUC_lexicon_tags_31 },
    { 0x633e6f56, SUC_lexicon_tags_15 },
    { 0x2e5e6f5a, SUC_lexicon_tags_15 },
    { 0xafbe6f5b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e6f65, SUC_lexicon_tags_7 },
    { 0x31e6f66, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfdfe6f68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x497e6f6b, SUC_lexicon_tags_14 },
    { 0xbd5e6f6c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695e6f74, SUC_lexicon_tags_7 },
    { 0xbb5e6f74, SUC_lexicon_tags_15 },
    { 0xdf7e6f74, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x691e6f78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59e6f7c, SUC_lexicon_tags_17 },
    { 0x607e6f7c, SUC_lexicon_tags_27 },
    { 0xc43e6f7e, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x623e6f80, SUC_lexicon_tags_35 },
    { 0xcbde6f80, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xebe6f83, SUC_lexicon_tags_25 },
    { 0x6bbe6f83, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaafe6f87, SUC_lexicon_tags_33 },
    { 0xb5de6f87, SUC_lexicon_tags_27 },
    { 0x3ebe6f88, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59e6f8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x887e6f90, SUC_lexicon_tags_17 },
    { 0xe99e6f91, SUC_lexicon_tags_172 },
    { 0xacfe6f92, SUC_lexicon_tags_35 },
    { 0xecfe6f93, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3fe6f95, SUC_lexicon_tags_8 },
    { 0x41be6f95, SUC_lexicon_tags_33 },
    { 0x3f9e6f97, SUC_lexicon_tags_136 },
    { 0x33de6f98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9afe6f9a, SUC_lexicon_tags_17 },
    { 0xec5e6f9b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8fbe6f9d, SUC_lexicon_tags_1 },
    { 0x7b5e6f9e, SUC_lexicon_tags_43 },
    { 0xaa5e6f9e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x825e6fa1, SUC_lexicon_tags_49 },
    { 0xee9e6fa2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407e6fa6, SUC_lexicon_tags_14 },
    { 0x5ade6fa7, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91e6faa, SUC_lexicon_tags_37 },
    { 0xf41e6fab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fe6fb5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3e6fb9, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55e6fc0, SUC_lexicon_tags_39 },
    { 0x2d3e6fc1, SUC_lexicon_tags_7 },
    { 0xebfe6fc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcfde6fc4, SUC_lexicon_tags_17 },
    { 0x9f9e6fc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2de6fc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4f1e6fca, SUC_lexicon_tags_65 },
    { 0x931e6fcb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x20de6fcd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x331e6fcf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8be6fd4, SUC_lexicon_tags_37 },
    { 0xd73e6fd5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x569e6fd8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67fe6fda, SUC_lexicon_tags_0 },
    { 0x9fde6fdb, SUC_lexicon_tags_30 },
    { 0xe17e6fdc, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4be6fe0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x243e6fe3, SUC_lexicon_tags_11 },
    { 0x36fe6fe4, SUC_lexicon_tags_17 },
    { 0xdcbe6fe5, SUC_lexicon_tags_19 },
    { 0x8e1e6fe6, SUC_lexicon_tags_2 },
    { 0x46fe6fe3, SUC_lexicon_tags_1 },
    { 0xa6de6fe6, SUC_lexicon_tags_4 },
    { 0x1cfe6fe8, SUC_lexicon_tags_37 },
    { 0x3fde6fea, SUC_lexicon_tags_21 },
    { 0xc3de6fe3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9e6fee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823e6ff3, SUC_lexicon_tags_17 },
    { 0xb05e6ff4, SUC_lexicon_tags_68 },
    { 0x1c3e6ff4, SUC_lexicon_tags_27 },
    { 0x711e6ff6, SUC_lexicon_tags_12 },
    { 0x30de6ff7, SUC_lexicon_tags_15 },
    { 0xfffe6ff8, SUC_lexicon_tags_33 },
    { 0x385e6ff9, SUC_lexicon_tags_2 },
    { 0x457e6ff6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x631e6ffc, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e7001, SUC_lexicon_tags_15 },
    { 0x271e7002, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x337e7004, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69fe7006, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x23fe7009, SUC_lexicon_tags_14 },
    { 0x9f5e7009, SUC_lexicon_tags_15 },
    { 0x7d7e700b, SUC_lexicon_tags_4 },
    { 0x3b1e7009, SUC_lexicon_tags_17 },
    { 0x993e700d, SUC_lexicon_tags_12 },
    { 0x33be700e, SUC_lexicon_tags_37 },
    { 0xa97e700f, SUC_lexicon_tags_17 },
    { 0xa05e700f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbdfe7012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78de701b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x559e701e, SUC_lexicon_tags_4 },
    { 0x9e3e701f, SUC_lexicon_tags_4 },
    { 0x4abe7020, SUC_lexicon_tags_28 },
    { 0x76fe7021, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9e7024, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecde7026, SUC_lexicon_tags_29 },
    { 0x9d5e7027, SUC_lexicon_tags_1 },
    { 0xc87e7028, SUC_lexicon_tags_82 },
    { 0x2ede7029, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x209e702b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7e702f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847e7033, SUC_lexicon_tags_7 },
    { 0xf0de7033, SUC_lexicon_tags_39 },
    { 0xd81e7033, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7e7038, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3e703b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfe7042, SUC_lexicon_tags_15 },
    { 0xd35e7043, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9e704a, SUC_lexicon_tags_218 },
    { 0xc0de704b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55e704e, SUC_lexicon_tags_11 },
    { 0xb6be704f, SUC_lexicon_tags_56 },
    { 0x4b9e7050, SUC_lexicon_tags_15 },
    { 0xb1de7051, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa97e7053, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x579e7056, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7e705a, SUC_lexicon_tags_17 },
    { 0xabe705b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xcede705d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x769e7060, SUC_lexicon_tags_37 },
    { 0x4e9e7060, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1e7066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc13e7068, SUC_lexicon_tags_2 },
    { 0x855e7069, SUC_lexicon_tags_6 },
    { 0x525e706a, SUC_lexicon_tags_77 },
    { 0xbf7e7068, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x883e706d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x493e706f, SUC_lexicon_tags_55 },
    { 0xa3e706f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55de7072, SUC_lexicon_tags_12 },
    { 0xba1e7073, SUC_lexicon_tags_68 },
    { 0x89fe7074, SUC_lexicon_tags_13 },
    { 0xbb1e7073, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803e7079, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3e707d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeafe7080, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x899e7083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb11e7085, SUC_lexicon_tags_28 },
    { 0x2f9e7086, SUC_lexicon_tags_25 },
    { 0xd35e7085, SUC_lexicon_tags_12 },
    { 0x76de7085, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231e708d, SUC_lexicon_tags_26 },
    { 0x49be708e, SUC_lexicon_tags_17 },
    { 0xff1e708d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfe7097, SUC_lexicon_tags_23 },
    { 0x147e7097, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e709d, SUC_lexicon_tags_25 },
    { 0x8ebe709d, SUC_lexicon_tags_25 },
    { 0xa99e709f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e70a3, SUC_lexicon_tags_1 },
    { 0xd35e70a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99e70a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x703e70aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fe70ad, SUC_lexicon_tags_38 },
    { 0xa5fe70ae, SUC_lexicon_tags_46 },
    { 0xa4fe70af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xede70b2, SUC_lexicon_tags_66 },
    { 0xa47e70b3, SUC_lexicon_tags_23 },
    { 0x24de70b2, SUC_lexicon_tags_16 },
    { 0x2b7e70b2, SUC_lexicon_tags_15 },
    { 0xcf9e70b4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81e70bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9e70c9, SUC_lexicon_tags_39 },
    { 0xd2de70c9, SUC_lexicon_tags_48 },
    { 0x4f3e70cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9e70d0, SUC_lexicon_tags_11 },
    { 0x53de70d1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x183e70d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2de70d9, SUC_lexicon_tags_11 },
    { 0x687e70d9, SUC_lexicon_tags_33 },
    { 0x3a7e70d9, SUC_lexicon_tags_48 },
    { 0x77be70dc, SUC_lexicon_tags_12 },
    { 0xb2be70db, SUC_lexicon_tags_8 },
    { 0xff1e70de, SUC_lexicon_tags_14 },
    { 0x741e70d9, SUC_lexicon_tags_79 },
    { 0xedbe70de, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dde70e5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35e70ec, SUC_lexicon_tags_47 },
    { 0x7f5e70ed, SUC_lexicon_tags_14 },
    { 0x739e70ee, SUC_lexicon_tags_31 },
    { 0x123e70ed, SUC_lexicon_tags_12 },
    { 0x7e9e70f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7e70f5, SUC_lexicon_tags_272 },
    { 0, NULL },
    { 0x2ffe70f7, SUC_lexicon_tags_43 },
    { 0x12de70f8, SUC_lexicon_tags_7 },
    { 0x76fe70f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1e70fc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4de70ff, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x15e7102, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fe7105, SUC_lexicon_tags_40 },
    { 0xd49e7105, SUC_lexicon_tags_7 },
    { 0x3b1e7107, SUC_lexicon_tags_23 },
    { 0x8de7106, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x919e710a, SUC_lexicon_tags_17 },
    { 0x87de710a, SUC_lexicon_tags_17 },
    { 0x581e710b, SUC_lexicon_tags_0 },
    { 0xa09e710a, SUC_lexicon_tags_11 },
    { 0x7b5e710c, SUC_lexicon_tags_38 },
    { 0x8c1e710f, SUC_lexicon_tags_47 },
    { 0xeffe7110, SUC_lexicon_tags_17 },
    { 0xa5de7110, SUC_lexicon_tags_8 },
    { 0xf37e710f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe7116, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dde7119, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xdf5e711b, SUC_lexicon_tags_77 },
    { 0xd45e711c, SUC_lexicon_tags_25 },
    { 0xe65e711d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4f3e711f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e7122, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1a5e7124, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9e7128, SUC_lexicon_tags_21 },
    { 0x497e7129, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x74fe712b, SUC_lexicon_tags_47 },
    { 0xb9be712b, SUC_lexicon_tags_15 },
    { 0x41be712b, SUC_lexicon_tags_11 },
    { 0x905e712c, SUC_lexicon_tags_40 },
    { 0xb6de712f, SUC_lexicon_tags_19 },
    { 0xb99e712f, SUC_lexicon_tags_15 },
    { 0xe97e7131, SUC_lexicon_tags_25 },
    { 0xea1e7132, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3e7137, SUC_lexicon_tags_46 },
    { 0x745e7138, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x435e713b, SUC_lexicon_tags_37 },
    { 0x401e713c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfbe713e, SUC_lexicon_tags_0 },
    { 0x281e713e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677e7144, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd1e7146, SUC_lexicon_tags_23 },
    { 0xa51e7146, SUC_lexicon_tags_98 },
    { 0x2fbe7147, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24fe714a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xacde714c, SUC_lexicon_tags_23 },
    { 0xadfe714d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xcc5e714f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e7151, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf2be7153, SUC_lexicon_tags_7 },
    { 0x793e7154, SUC_lexicon_tags_33 },
    { 0x95be7155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39e715f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6de7162, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fe7165, SUC_lexicon_tags_12 },
    { 0x14de7166, SUC_lexicon_tags_25 },
    { 0x501e7166, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5e7170, SUC_lexicon_tags_26 },
    { 0x12fe7171, SUC_lexicon_tags_47 },
    { 0xfd9e7171, SUC_lexicon_tags_6 },
    { 0xde9e7170, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb7fe7175, SUC_lexicon_tags_37 },
    { 0x87e7176, SUC_lexicon_tags_38 },
    { 0x159e7177, SUC_lexicon_tags_37 },
    { 0x145e7177, SUC_lexicon_tags_12 },
    { 0x85de7175, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7fe717b, SUC_lexicon_tags_11 },
    { 0x32fe717b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b9e7181, SUC_lexicon_tags_17 },
    { 0x7cde7182, SUC_lexicon_tags_0 },
    { 0xf37e7181, SUC_lexicon_tags_66 },
    { 0xcd9e7184, SUC_lexicon_tags_43 },
    { 0xdade7184, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb93e7188, SUC_lexicon_tags_28 },
    { 0xcefe7189, SUC_lexicon_tags_17 },
    { 0xdfe7189, SUC_lexicon_tags_2 },
    { 0x70be7188, SUC_lexicon_tags_16 },
    { 0xe7e718b, SUC_lexicon_tags_17 },
    { 0xb4de718d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x901e7190, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x711e7192, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1e7197, SUC_lexicon_tags_49 },
    { 0x653e7198, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7e719a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59e719f, SUC_lexicon_tags_15 },
    { 0x72fe719f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163e71a5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x635e71a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7e71aa, SUC_lexicon_tags_215 },
    { 0, NULL },
    { 0, NULL },
    { 0xc61e71ad, SUC_lexicon_tags_28 },
    { 0xacde71ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x229e71b0, SUC_lexicon_tags_17 },
    { 0x2ebe71b1, SUC_lexicon_tags_46 },
    { 0x321e71b1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67e71b9, SUC_lexicon_tags_26 },
    { 0x1e7e71b9, SUC_lexicon_tags_30 },
    { 0xf9be71bb, SUC_lexicon_tags_21 },
    { 0x4cbe71b9, SUC_lexicon_tags_7 },
    { 0xa5e71bb, SUC_lexicon_tags_2 },
    { 0x64be71b9, SUC_lexicon_tags_1 },
    { 0x559e71bf, SUC_lexicon_tags_44 },
    { 0x7afe71bf, SUC_lexicon_tags_38 },
    { 0x7e71bf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3de71c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39be71d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e71d4, SUC_lexicon_tags_11 },
    { 0xd23e71d5, SUC_lexicon_tags_9 },
    { 0xeb5e71d5, SUC_lexicon_tags_11 },
    { 0xecde71d5, SUC_lexicon_tags_1 },
    { 0xd15e71d8, SUC_lexicon_tags_25 },
    { 0x2f7e71d9, SUC_lexicon_tags_148 },
    { 0xfe3e71da, SUC_lexicon_tags_25 },
    { 0x73de71da, SUC_lexicon_tags_141 },
    { 0x7d3e71d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x721e71de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89de71e3, SUC_lexicon_tags_47 },
    { 0x341e71e3, SUC_lexicon_tags_27 },
    { 0x833e71e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e71e9, SUC_lexicon_tags_14 },
    { 0x963e71ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9d5e71ec, SUC_lexicon_tags_9 },
    { 0x23be71ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x573e71ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9e71f3, SUC_lexicon_tags_11 },
    { 0xfdde71f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf6fe71f6, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51e71fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fe71fe, SUC_lexicon_tags_70 },
    { 0x7a9e71ff, SUC_lexicon_tags_37 },
    { 0xdb1e71fe, SUC_lexicon_tags_63 },
    { 0x5a1e7201, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527e720d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13e7212, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5e7217, SUC_lexicon_tags_27 },
    { 0xb01e7218, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93e7220, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3e7225, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20be7228, SUC_lexicon_tags_44 },
    { 0x933e7229, SUC_lexicon_tags_65 },
    { 0x3e9e7228, SUC_lexicon_tags_25 },
    { 0x3bde7229, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed5e722d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x47e7230, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7c9e7232, SUC_lexicon_tags_1 },
    { 0x8de7233, SUC_lexicon_tags_68 },
    { 0xe5be7233, SUC_lexicon_tags_160 },
    { 0x367e7234, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x977e7237, SUC_lexicon_tags_12 },
    { 0x4a9e7238, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf51e723a, SUC_lexicon_tags_15 },
    { 0xb17e723a, SUC_lexicon_tags_26 },
    { 0xd45e723c, SUC_lexicon_tags_15 },
    { 0x9b7e723b, SUC_lexicon_tags_65 },
    { 0xc3e723d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45e7241, SUC_lexicon_tags_37 },
    { 0xdd1e7242, SUC_lexicon_tags_60 },
    { 0x467e7241, SUC_lexicon_tags_61 },
    { 0xc29e7242, SUC_lexicon_tags_12 },
    { 0x25de7243, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbe724b, SUC_lexicon_tags_2 },
    { 0xdabe724c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x963e724e, SUC_lexicon_tags_49 },
    { 0x8a1e724f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9e7254, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877e7258, SUC_lexicon_tags_31 },
    { 0x903e7259, SUC_lexicon_tags_33 },
    { 0x701e7258, SUC_lexicon_tags_27 },
    { 0x1a1e7259, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf13e7262, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x91de7265, SUC_lexicon_tags_7 },
    { 0x21de7266, SUC_lexicon_tags_12 },
    { 0x38fe7267, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x13fe7269, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa05e726b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90fe726d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05e7275, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xadde7277, SUC_lexicon_tags_112 },
    { 0x63de7278, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x927e727a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fe727f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x299e7281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9e7285, SUC_lexicon_tags_47 },
    { 0x827e7285, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ade7291, SUC_lexicon_tags_14 },
    { 0x169e7292, SUC_lexicon_tags_17 },
    { 0x959e7292, SUC_lexicon_tags_27 },
    { 0xdb9e7294, SUC_lexicon_tags_47 },
    { 0x687e7294, SUC_lexicon_tags_7 },
    { 0xa57e7295, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x555e7298, SUC_lexicon_tags_25 },
    { 0x6cde7299, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2be72a5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2de72a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77e72b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bde72b6, SUC_lexicon_tags_10 },
    { 0xd9fe72b7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5e72c0, SUC_lexicon_tags_66 },
    { 0x637e72c1, SUC_lexicon_tags_124 },
    { 0xb4de72c0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x16fe72c5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xec5e72c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfe72cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x507e72ce, SUC_lexicon_tags_29 },
    { 0xa6de72ce, SUC_lexicon_tags_2 },
    { 0x5d3e72d0, SUC_lexicon_tags_11 },
    { 0x767e72d1, SUC_lexicon_tags_7 },
    { 0x5ffe72d2, SUC_lexicon_tags_37 },
    { 0xa6be72d1, SUC_lexicon_tags_10 },
    { 0xc49e72d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9e72d7, SUC_lexicon_tags_11 },
    { 0xe6be72d7, SUC_lexicon_tags_17 },
    { 0x3c1e72d9, SUC_lexicon_tags_2 },
    { 0x539e72d8, SUC_lexicon_tags_6 },
    { 0xb17e72d7, SUC_lexicon_tags_11 },
    { 0x555e72db, SUC_lexicon_tags_1 },
    { 0xbfde72dd, SUC_lexicon_tags_39 },
    { 0xa15e72dc, SUC_lexicon_tags_23 },
    { 0x92fe72df, SUC_lexicon_tags_53 },
    { 0x45be72df, SUC_lexicon_tags_14 },
    { 0x681e72e1, SUC_lexicon_tags_37 },
    { 0x1d7e72e2, SUC_lexicon_tags_27 },
    { 0xf63e72e2, SUC_lexicon_tags_68 },
    { 0x3e1e72e4, SUC_lexicon_tags_29 },
    { 0x8f3e72de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe72e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fe72ed, SUC_lexicon_tags_7 },
    { 0x96be72ee, SUC_lexicon_tags_12 },
    { 0x28be72ef, SUC_lexicon_tags_17 },
    { 0x255e72ef, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x53de72f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867e72f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xed7e72f8, SUC_lexicon_tags_28 },
    { 0x8dfe72f8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x41de72fb, SUC_lexicon_tags_17 },
    { 0x357e72fc, SUC_lexicon_tags_47 },
    { 0x513e72fd, SUC_lexicon_tags_15 },
    { 0x97be72fe, SUC_lexicon_tags_15 },
    { 0x8fde72ff, SUC_lexicon_tags_14 },
    { 0x277e7300, SUC_lexicon_tags_19 },
    { 0x64fe7301, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66be7308, SUC_lexicon_tags_11 },
    { 0xc87e7308, SUC_lexicon_tags_83 },
    { 0xbb7e7308, SUC_lexicon_tags_37 },
    { 0x741e7308, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5e730f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb7e7311, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x509e7313, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17e731c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3fbe731e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x825e7320, SUC_lexicon_tags_17 },
    { 0x359e7320, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7e732e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28de7333, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e7337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe29e7339, SUC_lexicon_tags_2 },
    { 0xe31e733a, SUC_lexicon_tags_33 },
    { 0x82be733a, SUC_lexicon_tags_46 },
    { 0x257e733b, SUC_lexicon_tags_30 },
    { 0x1f1e733c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbe7344, SUC_lexicon_tags_7 },
    { 0xcb3e7345, SUC_lexicon_tags_47 },
    { 0xbc5e7345, SUC_lexicon_tags_45 },
    { 0x409e7347, SUC_lexicon_tags_49 },
    { 0x805e7348, SUC_lexicon_tags_63 },
    { 0x2d7e7344, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x389e734b, SUC_lexicon_tags_9 },
    { 0x681e734b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3ade734e, SUC_lexicon_tags_17 },
    { 0x4a9e734f, SUC_lexicon_tags_1 },
    { 0x1b5e7350, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ede7353, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x803e7355, SUC_lexicon_tags_1 },
    { 0xe9e7356, SUC_lexicon_tags_14 },
    { 0xa25e7357, SUC_lexicon_tags_29 },
    { 0x89e7358, SUC_lexicon_tags_9 },
    { 0x567e7359, SUC_lexicon_tags_27 },
    { 0x7bde7356, SUC_lexicon_tags_47 },
    { 0x249e7359, SUC_lexicon_tags_38 },
    { 0x8efe735c, SUC_lexicon_tags_45 },
    { 0xf67e7359, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0de7360, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x115e7363, SUC_lexicon_tags_11 },
    { 0xb15e7364, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa7de7366, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd31e7368, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3e736d, SUC_lexicon_tags_3 },
    { 0x84de736e, SUC_lexicon_tags_1 },
    { 0x471e736d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957e7373, SUC_lexicon_tags_12 },
    { 0x63fe7373, SUC_lexicon_tags_14 },
    { 0xcf3e7373, SUC_lexicon_tags_28 },
    { 0xc11e7375, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x65fe7378, SUC_lexicon_tags_49 },
    { 0x4f3e7378, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebe737d, SUC_lexicon_tags_19 },
    { 0x47e737d, SUC_lexicon_tags_2 },
    { 0x6d9e737d, SUC_lexicon_tags_11 },
    { 0xea7e737d, SUC_lexicon_tags_608 },
    { 0xac1e737e, SUC_lexicon_tags_14 },
    { 0xa7de7380, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xcde7384, SUC_lexicon_tags_66 },
    { 0xf7de7385, SUC_lexicon_tags_17 },
    { 0xf99e7386, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98de738c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3e738f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xeede7392, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41e7396, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9be7398, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1e739b, SUC_lexicon_tags_23 },
    { 0xd9de739b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaade73a0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdade73a7, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fe73b1, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x85e73b3, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97e73b7, SUC_lexicon_tags_7 },
    { 0x211e73b7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fe73bd, SUC_lexicon_tags_2 },
    { 0x4e5e73be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5f1e73c0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xafbe73c3, SUC_lexicon_tags_27 },
    { 0x83fe73c4, SUC_lexicon_tags_25 },
    { 0xd6fe73c5, SUC_lexicon_tags_63 },
    { 0xc99e73c3, SUC_lexicon_tags_7 },
    { 0xb85e73c7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5e73cb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32de73cf, SUC_lexicon_tags_1 },
    { 0xcd3e73d0, SUC_lexicon_tags_23 },
    { 0xaa1e73d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9e73d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5e73db, SUC_lexicon_tags_25 },
    { 0xede73dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ebe73de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x471e73e1, SUC_lexicon_tags_39 },
    { 0x64be73e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791e73e7, SUC_lexicon_tags_12 },
    { 0xe0fe73e7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xebfe73eb, SUC_lexicon_tags_46 },
    { 0x96de73ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1e73f3, SUC_lexicon_tags_14 },
    { 0xa63e73f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5e73f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ede73fb, SUC_lexicon_tags_7 },
    { 0x35be73fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xb93e73ff, SUC_lexicon_tags_2 },
    { 0x2c7e73ff, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0xd03e7402, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1e7405, SUC_lexicon_tags_0 },
    { 0x1dbe7406, SUC_lexicon_tags_7 },
    { 0x45de7406, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97e740a, SUC_lexicon_tags_63 },
    { 0x5a5e740b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbe740e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x647e7411, SUC_lexicon_tags_17 },
    { 0x89be7412, SUC_lexicon_tags_17 },
    { 0x697e7413, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85e7417, SUC_lexicon_tags_1 },
    { 0xdbfe7418, SUC_lexicon_tags_36 },
    { 0x9ade7419, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf23e741b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791e741f, SUC_lexicon_tags_201 },
    { 0xa71e7420, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1e7426, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9e7431, SUC_lexicon_tags_1 },
    { 0x131e7432, SUC_lexicon_tags_31 },
    { 0x6afe7432, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913e7437, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x17de743a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3e7440, SUC_lexicon_tags_1 },
    { 0xf59e7441, SUC_lexicon_tags_17 },
    { 0xf6de7440, SUC_lexicon_tags_39 },
    { 0xda9e7442, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1e7447, SUC_lexicon_tags_25 },
    { 0x127e7447, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1e744d, SUC_lexicon_tags_31 },
    { 0x559e744e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1efe7452, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5e7455, SUC_lexicon_tags_25 },
    { 0xa81e7456, SUC_lexicon_tags_14 },
    { 0x70be7455, SUC_lexicon_tags_37 },
    { 0x3e1e7458, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b3e745a, SUC_lexicon_tags_17 },
    { 0xd77e745a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03e7461, SUC_lexicon_tags_23 },
    { 0xa55e7462, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775e7466, SUC_lexicon_tags_17 },
    { 0x857e7467, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9e746c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x389e746e, SUC_lexicon_tags_25 },
    { 0xe19e746f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xeabe7471, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663e747b, SUC_lexicon_tags_38 },
    { 0x853e747b, SUC_lexicon_tags_502 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9e7483, SUC_lexicon_tags_30 },
    { 0x653e7483, SUC_lexicon_tags_247 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fe7488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825e7493, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38de7497, SUC_lexicon_tags_11 },
    { 0x79e7498, SUC_lexicon_tags_17 },
    { 0xabde7499, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe2fe749b, SUC_lexicon_tags_7 },
    { 0xeffe749c, SUC_lexicon_tags_15 },
    { 0xa2de749c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x12be749f, SUC_lexicon_tags_37 },
    { 0x63fe749f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x273e74a3, SUC_lexicon_tags_49 },
    { 0xa7de74a4, SUC_lexicon_tags_63 },
    { 0xc15e74a3, SUC_lexicon_tags_7 },
    { 0xe21e74a6, SUC_lexicon_tags_7 },
    { 0xb29e74a7, SUC_lexicon_tags_29 },
    { 0x777e74a7, SUC_lexicon_tags_37 },
    { 0xf65e74a4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7e74b4, SUC_lexicon_tags_2 },
    { 0xf3e74b5, SUC_lexicon_tags_19 },
    { 0x9e7e74b4, SUC_lexicon_tags_12 },
    { 0xa4be74b6, SUC_lexicon_tags_23 },
    { 0xe03e74b8, SUC_lexicon_tags_15 },
    { 0x771e74b9, SUC_lexicon_tags_25 },
    { 0x5e5e74b9, SUC_lexicon_tags_61 },
    { 0x64de74b8, SUC_lexicon_tags_19 },
    { 0x979e74b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43de74c0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x879e74c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259e74cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x39de74ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221e74d4, SUC_lexicon_tags_2 },
    { 0x30be74d4, SUC_lexicon_tags_1 },
    { 0xd91e74d4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7de74e1, SUC_lexicon_tags_28 },
    { 0x537e74e1, SUC_lexicon_tags_17 },
    { 0x46be74e3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x911e74e5, SUC_lexicon_tags_29 },
    { 0x7a1e74e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe74ea, SUC_lexicon_tags_23 },
    { 0xfd5e74eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ffe74ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xca9e74ef, SUC_lexicon_tags_26 },
    { 0x7a5e74f0, SUC_lexicon_tags_45 },
    { 0x92be74f1, SUC_lexicon_tags_37 },
    { 0xc7de74ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x35de74f5, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x223e74f7, SUC_lexicon_tags_63 },
    { 0x33de74f7, SUC_lexicon_tags_8 },
    { 0x323e74f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x96de74fc, SUC_lexicon_tags_26 },
    { 0x83e74fd, SUC_lexicon_tags_25 },
    { 0xf0be74fe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b5e7500, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9e7505, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x685e7508, SUC_lexicon_tags_12 },
    { 0x373e7509, SUC_lexicon_tags_69 },
    { 0xef7e7509, SUC_lexicon_tags_10 },
    { 0x2ede7509, SUC_lexicon_tags_17 },
    { 0xc8be750a, SUC_lexicon_tags_15 },
    { 0xbbde750d, SUC_lexicon_tags_17 },
    { 0xc3fe750d, SUC_lexicon_tags_56 },
    { 0x7dbe750f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9de7512, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7efe7514, SUC_lexicon_tags_55 },
    { 0x285e7515, SUC_lexicon_tags_84 },
    { 0xb33e7516, SUC_lexicon_tags_37 },
    { 0x85e7514, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x321e751a, SUC_lexicon_tags_18 },
    { 0x6c3e751b, SUC_lexicon_tags_14 },
    { 0xf17e751a, SUC_lexicon_tags_44 },
    { 0xd87e751c, SUC_lexicon_tags_16 },
    { 0x57e751a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x449e7521, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7e7528, SUC_lexicon_tags_26 },
    { 0x4dfe7528, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fe752c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x14fe752e, SUC_lexicon_tags_19 },
    { 0xb09e752f, SUC_lexicon_tags_37 },
    { 0xb1e7530, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8cfe7532, SUC_lexicon_tags_38 },
    { 0x9f3e7533, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51e7539, SUC_lexicon_tags_37 },
    { 0x39fe753a, SUC_lexicon_tags_7 },
    { 0x2d3e753b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71be753f, SUC_lexicon_tags_15 },
    { 0x917e7540, SUC_lexicon_tags_14 },
    { 0xb5be7541, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xc89e7543, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3e7547, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57be754c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fe7550, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x397e7552, SUC_lexicon_tags_21 },
    { 0x703e7553, SUC_lexicon_tags_4 },
    { 0x3d5e7554, SUC_lexicon_tags_47 },
    { 0xaa5e7553, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x55e7558, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e755c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9e7560, SUC_lexicon_tags_60 },
    { 0xb4fe7561, SUC_lexicon_tags_36 },
    { 0x937e7562, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x94be7564, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x143e7567, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7e756b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed5e756d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765e7572, SUC_lexicon_tags_15 },
    { 0x2a7e7573, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x963e7575, SUC_lexicon_tags_32 },
    { 0x923e7576, SUC_lexicon_tags_17 },
    { 0x567e7577, SUC_lexicon_tags_17 },
    { 0xdf9e7578, SUC_lexicon_tags_68 },
    { 0xd75e7579, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffe757d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0be7580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379e7586, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965e758a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7e758e, SUC_lexicon_tags_10 },
    { 0xadbe758e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x471e7591, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88de7596, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5e759b, SUC_lexicon_tags_65 },
    { 0x803e759c, SUC_lexicon_tags_7 },
    { 0xc9e759c, SUC_lexicon_tags_26 },
    { 0x5d9e759c, SUC_lexicon_tags_11 },
    { 0xc41e759b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239e75a3, SUC_lexicon_tags_12 },
    { 0x647e75a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285e75a9, SUC_lexicon_tags_26 },
    { 0xf9be75aa, SUC_lexicon_tags_37 },
    { 0x5efe75a9, SUC_lexicon_tags_11 },
    { 0x6e3e75a9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3e75af, SUC_lexicon_tags_65 },
    { 0x1bfe75b0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11fe75b8, SUC_lexicon_tags_17 },
    { 0x2fde75b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe75c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33e75c3, SUC_lexicon_tags_49 },
    { 0x895e75c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17de75c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1e75cd, SUC_lexicon_tags_37 },
    { 0x37de75ce, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x1e9e75d0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28be75d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fe75d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9de75da, SUC_lexicon_tags_31 },
    { 0xc73e75db, SUC_lexicon_tags_15 },
    { 0x393e75db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ede75e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dde75e4, SUC_lexicon_tags_15 },
    { 0xdade75e5, SUC_lexicon_tags_21 },
    { 0xbcfe75e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef5e75e8, SUC_lexicon_tags_15 },
    { 0x46fe75e8, SUC_lexicon_tags_2 },
    { 0xe3be75ea, SUC_lexicon_tags_37 },
    { 0xd39e75eb, SUC_lexicon_tags_17 },
    { 0xcf1e75eb, SUC_lexicon_tags_25 },
    { 0xfb1e75ed, SUC_lexicon_tags_37 },
    { 0x49e75ee, SUC_lexicon_tags_70 },
    { 0xdf9e75ef, SUC_lexicon_tags_2 },
    { 0xdd7e75ee, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1f1e75f2, SUC_lexicon_tags_25 },
    { 0xbb1e75f3, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7e9e75f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11e75f8, SUC_lexicon_tags_107 },
    { 0xfd1e75f8, SUC_lexicon_tags_17 },
    { 0xc57e75fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5e75fd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x179e7600, SUC_lexicon_tags_37 },
    { 0xe7e7601, SUC_lexicon_tags_11 },
    { 0x931e7602, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x967e7604, SUC_lexicon_tags_25 },
    { 0x13be7605, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc1e7607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5e760f, SUC_lexicon_tags_59 },
    { 0xde1e760f, SUC_lexicon_tags_11 },
    { 0xfefe7611, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3e7615, SUC_lexicon_tags_11 },
    { 0x979e7616, SUC_lexicon_tags_45 },
    { 0x8e5e7616, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4de761a, SUC_lexicon_tags_45 },
    { 0x4ade761b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e761f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefe7626, SUC_lexicon_tags_63 },
    { 0x945e7627, SUC_lexicon_tags_9 },
    { 0x187e7627, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1e762f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x515e7631, SUC_lexicon_tags_7 },
    { 0x303e7632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98fe7636, SUC_lexicon_tags_17 },
    { 0x895e7636, SUC_lexicon_tags_45 },
    { 0x4ffe7636, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93e763b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3e7640, SUC_lexicon_tags_19 },
    { 0x5efe7641, SUC_lexicon_tags_12 },
    { 0x6fbe7642, SUC_lexicon_tags_27 },
    { 0x29e7642, SUC_lexicon_tags_19 },
    { 0xe21e7642, SUC_lexicon_tags_29 },
    { 0xb69e7642, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbe764a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x92be764d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d9e764f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73de7656, SUC_lexicon_tags_7 },
    { 0x299e7656, SUC_lexicon_tags_17 },
    { 0x5d3e7656, SUC_lexicon_tags_38 },
    { 0x3efe7659, SUC_lexicon_tags_17 },
    { 0x9c7e765a, SUC_lexicon_tags_7 },
    { 0x8f1e765b, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x8f7e765d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd67e765f, SUC_lexicon_tags_37 },
    { 0x3dfe7660, SUC_lexicon_tags_60 },
    { 0xa1de765f, SUC_lexicon_tags_7 },
    { 0x6b5e7662, SUC_lexicon_tags_47 },
    { 0x39de7663, SUC_lexicon_tags_7 },
    { 0x2b7e7664, SUC_lexicon_tags_1 },
    { 0x443e7665, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdbbe7667, SUC_lexicon_tags_25 },
    { 0x26fe7668, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675e766d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fe7675, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf2de7677, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x313e767a, SUC_lexicon_tags_15 },
    { 0x41de767b, SUC_lexicon_tags_39 },
    { 0x7d3e767c, SUC_lexicon_tags_656 },
    { 0x259e767a, SUC_lexicon_tags_37 },
    { 0xbb9e767e, SUC_lexicon_tags_7 },
    { 0x6f9e767f, SUC_lexicon_tags_30 },
    { 0x5d5e7680, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4be7684, SUC_lexicon_tags_197 },
    { 0x4b3e7685, SUC_lexicon_tags_37 },
    { 0x425e7685, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3e768a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61e768e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1d5e7690, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7de7693, SUC_lexicon_tags_17 },
    { 0x88be7693, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e7697, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x1b1e7699, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fe769d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1e76a3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd6fe76a5, SUC_lexicon_tags_2 },
    { 0x599e76a5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfe76ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5e76b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abe76b7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7e76ba, SUC_lexicon_tags_38 },
    { 0x43be76bb, SUC_lexicon_tags_37 },
    { 0x573e76bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x147e76c3, SUC_lexicon_tags_37 },
    { 0x4bde76c4, SUC_lexicon_tags_15 },
    { 0x9c5e76c5, SUC_lexicon_tags_26 },
    { 0xda9e76c3, SUC_lexicon_tags_7 },
    { 0xac1e76c7, SUC_lexicon_tags_12 },
    { 0x897e76c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fe76cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89de76cd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe3e76cf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fe76d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa55e76d7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe83e76d9, SUC_lexicon_tags_276 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fe76de, SUC_lexicon_tags_55 },
    { 0x353e76de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de76e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31de76e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1e76ed, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9e76fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbc5e76fc, SUC_lexicon_tags_18 },
    { 0xade76fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x35be7700, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9e7708, SUC_lexicon_tags_16 },
    { 0xa69e7708, SUC_lexicon_tags_7 },
    { 0x895e7708, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x309e770d, SUC_lexicon_tags_49 },
    { 0xd2fe770e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x177e7711, SUC_lexicon_tags_1 },
    { 0xdbbe7712, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x59be7714, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x49e7716, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5afe7719, SUC_lexicon_tags_38 },
    { 0x747e7719, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fe7720, SUC_lexicon_tags_43 },
    { 0xb3e7720, SUC_lexicon_tags_65 },
    { 0x4abe7722, SUC_lexicon_tags_8 },
    { 0xadbe7723, SUC_lexicon_tags_79 },
    { 0xb07e7721, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07e7727, SUC_lexicon_tags_11 },
    { 0x8fe7728, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcffe772a, SUC_lexicon_tags_17 },
    { 0x4ede772b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e772e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe41e7730, SUC_lexicon_tags_52 },
    { 0xe05e7731, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x147e7733, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473e7737, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33e773f, SUC_lexicon_tags_56 },
    { 0x959e7740, SUC_lexicon_tags_29 },
    { 0xf91e7741, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbe7744, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x17de7747, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc39e7749, SUC_lexicon_tags_25 },
    { 0xe97e774a, SUC_lexicon_tags_7 },
    { 0xb2be774a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fde7750, SUC_lexicon_tags_1 },
    { 0x5a1e7751, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fe7754, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xecbe7756, SUC_lexicon_tags_61 },
    { 0xe51e7757, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xe53e7759, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x413e775b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1e775e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe3e7760, SUC_lexicon_tags_343 },
    { 0xb11e7761, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x911e7763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151e7768, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdbfe776a, SUC_lexicon_tags_6 },
    { 0x3a1e776b, SUC_lexicon_tags_25 },
    { 0x431e776c, SUC_lexicon_tags_27 },
    { 0xe21e776d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8f9e776f, SUC_lexicon_tags_7 },
    { 0xc81e7770, SUC_lexicon_tags_2 },
    { 0xd55e7771, SUC_lexicon_tags_2 },
    { 0x42de7772, SUC_lexicon_tags_17 },
    { 0xc2fe7771, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x66be7775, SUC_lexicon_tags_39 },
    { 0x8afe7775, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x77de7778, SUC_lexicon_tags_11 },
    { 0x171e7779, SUC_lexicon_tags_65 },
    { 0x36de777a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fe777f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7e7782, SUC_lexicon_tags_11 },
    { 0xf67e7783, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7e7789, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7e778c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9e7791, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fe7794, SUC_lexicon_tags_65 },
    { 0x89e7794, SUC_lexicon_tags_49 },
    { 0xc27e7795, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xff5e7798, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe15e779a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fe779d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9e1e779f, SUC_lexicon_tags_15 },
    { 0x7a5e77a0, SUC_lexicon_tags_15 },
    { 0x1b3e77a1, SUC_lexicon_tags_33 },
    { 0x761e77a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9e77a6, SUC_lexicon_tags_4 },
    { 0xfe1e77a7, SUC_lexicon_tags_37 },
    { 0xf03e77a8, SUC_lexicon_tags_47 },
    { 0xfc9e77a6, SUC_lexicon_tags_19 },
    { 0x81de77a7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259e77af, SUC_lexicon_tags_30 },
    { 0x249e77b0, SUC_lexicon_tags_32 },
    { 0x2f3e77b0, SUC_lexicon_tags_4 },
    { 0x2bbe77b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311e77b6, SUC_lexicon_tags_4 },
    { 0xc5e77b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61be77bc, SUC_lexicon_tags_1 },
    { 0x265e77bc, SUC_lexicon_tags_7 },
    { 0x117e77be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9e77c1, SUC_lexicon_tags_37 },
    { 0x443e77c2, SUC_lexicon_tags_25 },
    { 0x3fbe77c3, SUC_lexicon_tags_16 },
    { 0x8d9e77c3, SUC_lexicon_tags_7 },
    { 0x447e77c5, SUC_lexicon_tags_10 },
    { 0x901e77c5, SUC_lexicon_tags_63 },
    { 0x48fe77c7, SUC_lexicon_tags_60 },
    { 0x2c1e77c3, SUC_lexicon_tags_34 },
    { 0xfe1e77c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x91be77cb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3e77d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x46de77d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb17e77d6, SUC_lexicon_tags_17 },
    { 0x521e77d6, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5be77da, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x70de77dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x743e77df, SUC_lexicon_tags_37 },
    { 0xd1fe77e0, SUC_lexicon_tags_11 },
    { 0x9b7e77e0, SUC_lexicon_tags_7 },
    { 0xa7de77e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1e77e6, SUC_lexicon_tags_4 },
    { 0x93be77e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fe77ed, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07e77f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9e77fb, SUC_lexicon_tags_11 },
    { 0xcc5e77fb, SUC_lexicon_tags_1 },
    { 0x59fe77fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3e7801, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x743e7803, SUC_lexicon_tags_11 },
    { 0x125e7804, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e7807, SUC_lexicon_tags_2 },
    { 0x99fe7808, SUC_lexicon_tags_26 },
    { 0x4c3e7809, SUC_lexicon_tags_37 },
    { 0x66de7809, SUC_lexicon_tags_27 },
    { 0xf9de780a, SUC_lexicon_tags_33 },
    { 0x8bde780a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x465e780e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5be7811, SUC_lexicon_tags_12 },
    { 0x415e7811, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x711e7814, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x2be7816, SUC_lexicon_tags_47 },
    { 0x3fe7816, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1e781a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x623e781d, SUC_lexicon_tags_15 },
    { 0x83fe781e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87e7822, SUC_lexicon_tags_12 },
    { 0x263e7823, SUC_lexicon_tags_1 },
    { 0x797e7823, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaf3e7826, SUC_lexicon_tags_11 },
    { 0xa93e7827, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x875e7829, SUC_lexicon_tags_23 },
    { 0x9f7e782a, SUC_lexicon_tags_12 },
    { 0xa7de782b, SUC_lexicon_tags_30 },
    { 0x4e1e782c, SUC_lexicon_tags_61 },
    { 0xf7e782d, SUC_lexicon_tags_56 },
    { 0x1fde782e, SUC_lexicon_tags_38 },
    { 0x90be782f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbe7832, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3e7835, SUC_lexicon_tags_11 },
    { 0xa1be7836, SUC_lexicon_tags_45 },
    { 0xa89e7835, SUC_lexicon_tags_37 },
    { 0x609e7837, SUC_lexicon_tags_37 },
    { 0xfc5e7839, SUC_lexicon_tags_30 },
    { 0x135e7838, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc11e783c, SUC_lexicon_tags_28 },
    { 0xd7fe783c, SUC_lexicon_tags_47 },
    { 0xaa3e783e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e7e7840, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x581e7843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279e7849, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbde784d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1e7850, SUC_lexicon_tags_4 },
    { 0xa8be7851, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x903e7854, SUC_lexicon_tags_37 },
    { 0x7b5e7855, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x841e7857, SUC_lexicon_tags_2 },
    { 0x29de7858, SUC_lexicon_tags_47 },
    { 0xdf5e7859, SUC_lexicon_tags_38 },
    { 0x12be7858, SUC_lexicon_tags_7 },
    { 0x295e7858, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x40de785d, SUC_lexicon_tags_92 },
    { 0x879e785d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fe7861, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x76be7864, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x453e7866, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a3e7868, SUC_lexicon_tags_2 },
    { 0xb6de7869, SUC_lexicon_tags_17 },
    { 0xc11e7869, SUC_lexicon_tags_11 },
    { 0x161e786a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x677e786e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5ebe7870, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e7876, SUC_lexicon_tags_1 },
    { 0xcf7e7876, SUC_lexicon_tags_35 },
    { 0x41e7878, SUC_lexicon_tags_16 },
    { 0x5efe7878, SUC_lexicon_tags_82 },
    { 0x47e787a, SUC_lexicon_tags_17 },
    { 0x65be7879, SUC_lexicon_tags_33 },
    { 0x34de787c, SUC_lexicon_tags_26 },
    { 0xb71e787d, SUC_lexicon_tags_1 },
    { 0xa4fe787e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9e7883, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdde7889, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4be788b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xcf9e788d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x959e788f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe9be7891, SUC_lexicon_tags_55 },
    { 0xc7be7892, SUC_lexicon_tags_37 },
    { 0x8f1e7893, SUC_lexicon_tags_2 },
    { 0x47be7894, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3e7897, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe789b, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0xdd3e789d, SUC_lexicon_tags_14 },
    { 0xca9e789d, SUC_lexicon_tags_15 },
    { 0xbc1e789f, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x761e78a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7e78a4, SUC_lexicon_tags_11 },
    { 0x275e78a5, SUC_lexicon_tags_12 },
    { 0xd25e78a5, SUC_lexicon_tags_2 },
    { 0xc03e78a5, SUC_lexicon_tags_47 },
    { 0x457e78a7, SUC_lexicon_tags_88 },
    { 0xf5be78a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5de78af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fe78b5, SUC_lexicon_tags_21 },
    { 0xc1be78b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc75e78b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2d7e78ba, SUC_lexicon_tags_4 },
    { 0xa5fe78bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7fe78bd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3e78c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe78c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88be78c6, SUC_lexicon_tags_61 },
    { 0x185e78c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfb9e78c9, SUC_lexicon_tags_11 },
    { 0x4b5e78ca, SUC_lexicon_tags_7 },
    { 0xd89e78cb, SUC_lexicon_tags_1 },
    { 0x76de78cc, SUC_lexicon_tags_4 },
    { 0x9b3e78cd, SUC_lexicon_tags_15 },
    { 0x907e78ce, SUC_lexicon_tags_40 },
    { 0xa0de78cd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979e78d4, SUC_lexicon_tags_14 },
    { 0x479e78d4, SUC_lexicon_tags_37 },
    { 0x283e78d5, SUC_lexicon_tags_45 },
    { 0xb57e78d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797e78db, SUC_lexicon_tags_3 },
    { 0x14de78db, SUC_lexicon_tags_66 },
    { 0xbf5e78dd, SUC_lexicon_tags_27 },
    { 0x18fe78de, SUC_lexicon_tags_1 },
    { 0x703e78dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccbe78e1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x229e78e4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x811e78e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x387e78e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3c1e78ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9cbe78ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61e78ef, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc83e78f1, SUC_lexicon_tags_11 },
    { 0xa3de78f1, SUC_lexicon_tags_30 },
    { 0xfebe78f3, SUC_lexicon_tags_37 },
    { 0x763e78f4, SUC_lexicon_tags_11 },
    { 0xd79e78f3, SUC_lexicon_tags_23 },
    { 0x621e78f6, SUC_lexicon_tags_0 },
    { 0x8cfe78f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3be7903, SUC_lexicon_tags_12 },
    { 0x223e7904, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c1e7906, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7e7909, SUC_lexicon_tags_12 },
    { 0x5d3e790a, SUC_lexicon_tags_1 },
    { 0xa3e790b, SUC_lexicon_tags_11 },
    { 0x1a5e790a, SUC_lexicon_tags_17 },
    { 0xb53e790b, SUC_lexicon_tags_28 },
    { 0x145e7909, SUC_lexicon_tags_27 },
    { 0xa8be7909, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fe7912, SUC_lexicon_tags_14 },
    { 0x21de7913, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1e7917, SUC_lexicon_tags_34 },
    { 0xd11e7917, SUC_lexicon_tags_25 },
    { 0x82be7919, SUC_lexicon_tags_65 },
    { 0x2e3e791a, SUC_lexicon_tags_27 },
    { 0xe33e7917, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12de791f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77e7923, SUC_lexicon_tags_25 },
    { 0xc13e7923, SUC_lexicon_tags_7 },
    { 0x441e7925, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7e7928, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8de7933, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e1e7935, SUC_lexicon_tags_63 },
    { 0x6a5e7936, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcde7938, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fe793c, SUC_lexicon_tags_45 },
    { 0x479e793d, SUC_lexicon_tags_12 },
    { 0x23e793e, SUC_lexicon_tags_9 },
    { 0xec1e793f, SUC_lexicon_tags_31 },
    { 0x605e793d, SUC_lexicon_tags_47 },
    { 0xade7941, SUC_lexicon_tags_11 },
    { 0xdf7e7942, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x93de7945, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xade7949, SUC_lexicon_tags_42 },
    { 0x61be794a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x241e794c, SUC_lexicon_tags_7 },
    { 0x125e794d, SUC_lexicon_tags_45 },
    { 0x265e794e, SUC_lexicon_tags_12 },
    { 0xaa5e794d, SUC_lexicon_tags_29 },
    { 0xa6fe794c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169e7954, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbde795a, SUC_lexicon_tags_11 },
    { 0x327e795b, SUC_lexicon_tags_8 },
    { 0xffe795a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47be7962, SUC_lexicon_tags_2 },
    { 0x32be7962, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x259e7966, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x299e7968, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x61e796a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77e796d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e7970, SUC_lexicon_tags_25 },
    { 0x319e7971, SUC_lexicon_tags_37 },
    { 0x2cde7972, SUC_lexicon_tags_2 },
    { 0x577e7970, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe797f, SUC_lexicon_tags_1 },
    { 0x891e797f, SUC_lexicon_tags_17 },
    { 0xf15e7980, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abe7985, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6be798c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x25fe798e, SUC_lexicon_tags_2 },
    { 0xa07e798e, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfde7992, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44be799a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933e79a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55de79a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf55e79a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf05e79ab, SUC_lexicon_tags_15 },
    { 0x8a5e79ac, SUC_lexicon_tags_20 },
    { 0x567e79ac, SUC_lexicon_tags_19 },
    { 0x3b7e79ac, SUC_lexicon_tags_23 },
    { 0x349e79af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59be79b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x925e79b5, SUC_lexicon_tags_11 },
    { 0x147e79b6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9e79bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8d7e79bf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbe79c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb31e79c4, SUC_lexicon_tags_295 },
    { 0xe47e79c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x939e79c7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70de79cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x659e79ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efe79d3, SUC_lexicon_tags_79 },
    { 0xfde79d3, SUC_lexicon_tags_25 },
    { 0x70de79d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7e79dc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x67de79de, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x61fe79e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95e79e3, SUC_lexicon_tags_60 },
    { 0x497e79e4, SUC_lexicon_tags_37 },
    { 0x225e79e5, SUC_lexicon_tags_25 },
    { 0x601e79e6, SUC_lexicon_tags_28 },
    { 0x7dbe79e4, SUC_lexicon_tags_11 },
    { 0x8efe79e8, SUC_lexicon_tags_12 },
    { 0x39de79e9, SUC_lexicon_tags_15 },
    { 0x7c9e79ea, SUC_lexicon_tags_11 },
    { 0xee9e79eb, SUC_lexicon_tags_25 },
    { 0x3f3e79ec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x979e79ef, SUC_lexicon_tags_14 },
    { 0xca9e79f0, SUC_lexicon_tags_37 },
    { 0x833e79f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ade79f5, SUC_lexicon_tags_11 },
    { 0x5e3e79f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64be79fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e79fe, SUC_lexicon_tags_65 },
    { 0x127e79fe, SUC_lexicon_tags_8 },
    { 0x54be79ff, SUC_lexicon_tags_62 },
    { 0x6cfe79ff, SUC_lexicon_tags_12 },
    { 0x143e7a02, SUC_lexicon_tags_12 },
    { 0xd09e7a02, SUC_lexicon_tags_4 },
    { 0x4cbe7a02, SUC_lexicon_tags_11 },
    { 0xbe7e7a05, SUC_lexicon_tags_49 },
    { 0xe07e7a06, SUC_lexicon_tags_27 },
    { 0x879e7a07, SUC_lexicon_tags_17 },
    { 0x201e7a05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39e7a15, SUC_lexicon_tags_8 },
    { 0x79e7a15, SUC_lexicon_tags_47 },
    { 0x77e7a17, SUC_lexicon_tags_2 },
    { 0xf81e7a18, SUC_lexicon_tags_57 },
    { 0x995e7a19, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x29be7a1c, SUC_lexicon_tags_7 },
    { 0xf13e7a1c, SUC_lexicon_tags_37 },
    { 0xb1fe7a1e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9e7a21, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x619e7a24, SUC_lexicon_tags_9 },
    { 0xf11e7a25, SUC_lexicon_tags_6 },
    { 0xacde7a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e7a2a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xfd9e7a2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865e7a30, SUC_lexicon_tags_34 },
    { 0xfebe7a31, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71e7a35, SUC_lexicon_tags_47 },
    { 0x105e7a36, SUC_lexicon_tags_570 },
    { 0, NULL },
    { 0x1d1e7a38, SUC_lexicon_tags_30 },
    { 0xc99e7a39, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5e7a40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x45e7a43, SUC_lexicon_tags_38 },
    { 0xe3be7a44, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01e7a48, SUC_lexicon_tags_7 },
    { 0xa73e7a49, SUC_lexicon_tags_66 },
    { 0x509e7a4a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429e7a4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbe7a52, SUC_lexicon_tags_146 },
    { 0x43fe7a52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279e7a58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcde7a5a, SUC_lexicon_tags_63 },
    { 0xcc1e7a5b, SUC_lexicon_tags_61 },
    { 0x4bde7a5c, SUC_lexicon_tags_0 },
    { 0x3a9e7a5d, SUC_lexicon_tags_35 },
    { 0x2de7a5e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3e7a61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xac7e7a64, SUC_lexicon_tags_43 },
    { 0x2b5e7a65, SUC_lexicon_tags_17 },
    { 0x631e7a66, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbebe7a68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4dde7a6a, SUC_lexicon_tags_1 },
    { 0xa7de7a6b, SUC_lexicon_tags_10 },
    { 0xae9e7a6b, SUC_lexicon_tags_45 },
    { 0xbc1e7a6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331e7a73, SUC_lexicon_tags_47 },
    { 0x5abe7a73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ede7a79, SUC_lexicon_tags_43 },
    { 0xd99e7a7a, SUC_lexicon_tags_17 },
    { 0x5fde7a79, SUC_lexicon_tags_25 },
    { 0x477e7a7a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbede7a83, SUC_lexicon_tags_12 },
    { 0x1bde7a84, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xaf9e7a86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfe7a8a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafe7a90, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x863e7a93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd5e7a95, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7a9c, SUC_lexicon_tags_11 },
    { 0xc25e7a9d, SUC_lexicon_tags_2 },
    { 0xf57e7a9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d1e7aa0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5e7aa6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761e7aad, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41e7ab0, SUC_lexicon_tags_37 },
    { 0xd8fe7ab0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865e7ab5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3e7abd, SUC_lexicon_tags_11 },
    { 0xd89e7abe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f3e7ac0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9be7ac3, SUC_lexicon_tags_38 },
    { 0xf87e7ac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3e7aca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29e7acd, SUC_lexicon_tags_2 },
    { 0x945e7acd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80be7ad4, SUC_lexicon_tags_25 },
    { 0x21de7ad5, SUC_lexicon_tags_25 },
    { 0xa05e7ad5, SUC_lexicon_tags_25 },
    { 0x11de7ad4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17de7ade, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2fe7ae0, SUC_lexicon_tags_15 },
    { 0xb9fe7ae1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x989e7ae3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x125e7ae6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75e7aef, SUC_lexicon_tags_1 },
    { 0x1b3e7af0, SUC_lexicon_tags_14 },
    { 0xb3e7af1, SUC_lexicon_tags_66 },
    { 0x7ebe7af0, SUC_lexicon_tags_37 },
    { 0x5b1e7af1, SUC_lexicon_tags_71 },
    { 0xf53e7af4, SUC_lexicon_tags_14 },
    { 0xb8be7af3, SUC_lexicon_tags_45 },
    { 0xd3e7aef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x715e7af9, SUC_lexicon_tags_37 },
    { 0xc39e7afa, SUC_lexicon_tags_64 },
    { 0xc93e7afb, SUC_lexicon_tags_1 },
    { 0x377e7afc, SUC_lexicon_tags_21 },
    { 0x3e3e7afc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e7b01, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fe7b05, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc4de7b07, SUC_lexicon_tags_33 },
    { 0x435e7b07, SUC_lexicon_tags_25 },
    { 0x77fe7b09, SUC_lexicon_tags_27 },
    { 0x937e7b0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847e7b0e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe5de7b10, SUC_lexicon_tags_29 },
    { 0xa5e7b11, SUC_lexicon_tags_11 },
    { 0x40fe7b12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7abe7b14, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25e7b17, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831e7b1d, SUC_lexicon_tags_2 },
    { 0xc27e7b1e, SUC_lexicon_tags_12 },
    { 0x881e7b1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9e7b25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7e7e7b27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3e7b2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa8be7b2e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9dde7b30, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdc9e7b32, SUC_lexicon_tags_17 },
    { 0x9dbe7b33, SUC_lexicon_tags_12 },
    { 0xde5e7b34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4fe7b39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3e7b3d, SUC_lexicon_tags_2 },
    { 0x251e7b3e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x62fe7b40, SUC_lexicon_tags_2 },
    { 0x74de7b41, SUC_lexicon_tags_65 },
    { 0xd0fe7b42, SUC_lexicon_tags_14 },
    { 0x5fe7b42, SUC_lexicon_tags_37 },
    { 0xddde7b44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4be7b4f, SUC_lexicon_tags_17 },
    { 0x425e7b50, SUC_lexicon_tags_38 },
    { 0xc5be7b50, SUC_lexicon_tags_16 },
    { 0xfa5e7b50, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707e7b5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x231e7b62, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05e7b69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3e7b6c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01e7b6f, SUC_lexicon_tags_1 },
    { 0x7ffe7b70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0de7b75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdffe7b77, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x969e7b7a, SUC_lexicon_tags_33 },
    { 0x2e9e7b7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc93e7b7d, SUC_lexicon_tags_14 },
    { 0x8fbe7b7d, SUC_lexicon_tags_29 },
    { 0x39e7b7f, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4be7b82, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x6ade7b84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x41e7b86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7e7b91, SUC_lexicon_tags_28 },
    { 0xddde7b92, SUC_lexicon_tags_55 },
    { 0xb45e7b93, SUC_lexicon_tags_0 },
    { 0x2c3e7b94, SUC_lexicon_tags_188 },
    { 0x77be7b95, SUC_lexicon_tags_7 },
    { 0x2b1e7b96, SUC_lexicon_tags_570 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9e7b99, SUC_lexicon_tags_60 },
    { 0x3c5e7b99, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb3e7b9c, SUC_lexicon_tags_7 },
    { 0x5a9e7b9d, SUC_lexicon_tags_16 },
    { 0x3be7b9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2d9e7ba0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e7ba4, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95e7ba8, SUC_lexicon_tags_31 },
    { 0xa51e7ba8, SUC_lexicon_tags_14 },
    { 0x385e7baa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x94de7bac, SUC_lexicon_tags_25 },
    { 0x8fbe7bad, SUC_lexicon_tags_79 },
    { 0x18de7bad, SUC_lexicon_tags_6 },
    { 0x95de7bae, SUC_lexicon_tags_2 },
    { 0xf97e7bb0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddde7bb4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xebfe7bb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x513e7bb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x567e7bbb, SUC_lexicon_tags_17 },
    { 0x723e7bbc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9dde7bbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2ede7bc0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfebe7bc3, SUC_lexicon_tags_32 },
    { 0xac9e7bc4, SUC_lexicon_tags_60 },
    { 0x599e7bc3, SUC_lexicon_tags_49 },
    { 0x4f1e7bc6, SUC_lexicon_tags_37 },
    { 0x943e7bc6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91e7bcd, SUC_lexicon_tags_25 },
    { 0xb3be7bce, SUC_lexicon_tags_21 },
    { 0x111e7bcf, SUC_lexicon_tags_7 },
    { 0x2dfe7bd0, SUC_lexicon_tags_23 },
    { 0x5e1e7bd1, SUC_lexicon_tags_47 },
    { 0xddbe7bce, SUC_lexicon_tags_28 },
    { 0xf3de7bd3, SUC_lexicon_tags_21 },
    { 0x869e7bd4, SUC_lexicon_tags_17 },
    { 0xa27e7bd4, SUC_lexicon_tags_5 },
    { 0xf71e7bd6, SUC_lexicon_tags_4 },
    { 0x487e7bd7, SUC_lexicon_tags_12 },
    { 0x9c3e7bcd, SUC_lexicon_tags_118 },
    { 0x635e7bd2, SUC_lexicon_tags_11 },
    { 0xf3e7bda, SUC_lexicon_tags_15 },
    { 0x985e7bdb, SUC_lexicon_tags_1 },
    { 0xd57e7bd6, SUC_lexicon_tags_25 },
    { 0x5b9e7bd7, SUC_lexicon_tags_25 },
    { 0xaa9e7bda, SUC_lexicon_tags_65 },
    { 0xb7de7bdf, SUC_lexicon_tags_0 },
    { 0x2e3e7bd8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fe7beb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3e7bee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc7e7bf0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3be7bf4, SUC_lexicon_tags_62 },
    { 0x7e7bf5, SUC_lexicon_tags_11 },
    { 0xfcde7bf6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x769e7bf8, SUC_lexicon_tags_14 },
    { 0x995e7bf8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1e7c00, SUC_lexicon_tags_14 },
    { 0xea3e7c00, SUC_lexicon_tags_171 },
    { 0xbe7e7c00, SUC_lexicon_tags_12 },
    { 0xe51e7c03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x59de7c05, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x171e7c07, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9e7c0e, SUC_lexicon_tags_63 },
    { 0x719e7c0f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ede7c12, SUC_lexicon_tags_19 },
    { 0x379e7c13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffe7c16, SUC_lexicon_tags_13 },
    { 0x823e7c17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbe7c22, SUC_lexicon_tags_7 },
    { 0xdc7e7c23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25be7c2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96be7c33, SUC_lexicon_tags_282 },
    { 0x6f9e7c33, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79be7c3a, SUC_lexicon_tags_1 },
    { 0xacde7c3b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5e7c3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65e7c42, SUC_lexicon_tags_2 },
    { 0xf41e7c42, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7f9e7c45, SUC_lexicon_tags_12 },
    { 0x52de7c45, SUC_lexicon_tags_15 },
    { 0xa8fe7c46, SUC_lexicon_tags_11 },
    { 0x299e7c45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999e7c4c, SUC_lexicon_tags_19 },
    { 0x6b3e7c4d, SUC_lexicon_tags_14 },
    { 0xdb5e7c4e, SUC_lexicon_tags_6 },
    { 0x2e1e7c4e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9e7c57, SUC_lexicon_tags_1 },
    { 0xcfe7c58, SUC_lexicon_tags_12 },
    { 0x7d3e7c59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x605e7c5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3d9e7c5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xee3e7c5f, SUC_lexicon_tags_48 },
    { 0x9be7c5f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43e7c65, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcdfe7c67, SUC_lexicon_tags_47 },
    { 0x609e7c67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebe7c70, SUC_lexicon_tags_33 },
    { 0x973e7c70, SUC_lexicon_tags_40 },
    { 0xefde7c72, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfe7c75, SUC_lexicon_tags_82 },
    { 0x767e7c76, SUC_lexicon_tags_8 },
    { 0xcebe7c77, SUC_lexicon_tags_12 },
    { 0xf41e7c75, SUC_lexicon_tags_30 },
    { 0xb23e7c76, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x753e7c7b, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0xf91e7c7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc53e7c80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9e7c83, SUC_lexicon_tags_7 },
    { 0x267e7c83, SUC_lexicon_tags_47 },
    { 0xcf5e7c84, SUC_lexicon_tags_37 },
    { 0x793e7c84, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3e7c8a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4ade7c8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a7e7c8e, SUC_lexicon_tags_27 },
    { 0xe0de7c8f, SUC_lexicon_tags_12 },
    { 0x4d9e7c8f, SUC_lexicon_tags_66 },
    { 0xa51e7c91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc67e7c93, SUC_lexicon_tags_28 },
    { 0xa15e7c94, SUC_lexicon_tags_47 },
    { 0x43be7c95, SUC_lexicon_tags_63 },
    { 0x97fe7c96, SUC_lexicon_tags_54 },
    { 0x77be7c97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249e7c9b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xda9e7c9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6a5e7c9f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29e7ca4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c3e7ca6, SUC_lexicon_tags_163 },
    { 0xe3de7ca7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fe7cab, SUC_lexicon_tags_63 },
    { 0x78be7cac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x75de7cae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x969e7cb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed3e7cb2, SUC_lexicon_tags_60 },
    { 0xc39e7cb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe89e7cb5, SUC_lexicon_tags_38 },
    { 0xfade7cb6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3cbe7cb8, SUC_lexicon_tags_10 },
    { 0x553e7cb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5e1e7cbb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x911e7cbd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x115e7cbf, SUC_lexicon_tags_77 },
    { 0x7dfe7cbf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179e7cc4, SUC_lexicon_tags_48 },
    { 0x529e7cc5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebe7cc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93e7ccf, SUC_lexicon_tags_13 },
    { 0x6d7e7cd0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643e7cd4, SUC_lexicon_tags_55 },
    { 0xdbbe7cd5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fe7cdb, SUC_lexicon_tags_179 },
    { 0xd6be7cdb, SUC_lexicon_tags_38 },
    { 0x18be7cdb, SUC_lexicon_tags_17 },
    { 0x4efe7cde, SUC_lexicon_tags_12 },
    { 0x96de7cde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x659e7ce1, SUC_lexicon_tags_82 },
    { 0xa8fe7ce2, SUC_lexicon_tags_28 },
    { 0x29fe7ce3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65e7ce7, SUC_lexicon_tags_6 },
    { 0xa81e7ce8, SUC_lexicon_tags_37 },
    { 0xc11e7ce9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79be7cf1, SUC_lexicon_tags_25 },
    { 0xd73e7cf2, SUC_lexicon_tags_37 },
    { 0xd0be7cf2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x407e7cf5, SUC_lexicon_tags_11 },
    { 0x4c3e7cf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1e7cfb, SUC_lexicon_tags_37 },
    { 0x2b5e7cfc, SUC_lexicon_tags_2 },
    { 0x183e7cfd, SUC_lexicon_tags_47 },
    { 0x6d3e7cfe, SUC_lexicon_tags_15 },
    { 0x10be7cff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe7d03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8efe7d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9e7d08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x313e7d0a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x79de7d0d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x75e7d10, SUC_lexicon_tags_38 },
    { 0x5bde7d10, SUC_lexicon_tags_25 },
    { 0x663e7d12, SUC_lexicon_tags_32 },
    { 0x9b1e7d12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3e7d16, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9e7d1a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe35e7d1c, SUC_lexicon_tags_23 },
    { 0xa71e7d1c, SUC_lexicon_tags_79 },
    { 0x161e7d1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xba9e7d20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x105e7d23, SUC_lexicon_tags_7 },
    { 0xeefe7d24, SUC_lexicon_tags_14 },
    { 0x525e7d25, SUC_lexicon_tags_12 },
    { 0x793e7d24, SUC_lexicon_tags_47 },
    { 0x61de7d23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb3e7d29, SUC_lexicon_tags_14 },
    { 0xc85e7d2a, SUC_lexicon_tags_15 },
    { 0x563e7d2b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbe7d2f, SUC_lexicon_tags_27 },
    { 0xa9be7d2f, SUC_lexicon_tags_10 },
    { 0x6ade7d31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6de7d34, SUC_lexicon_tags_68 },
    { 0x467e7d35, SUC_lexicon_tags_37 },
    { 0x669e7d34, SUC_lexicon_tags_9 },
    { 0xe29e7d35, SUC_lexicon_tags_15 },
    { 0xcfde7d37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae3e7d3a, SUC_lexicon_tags_30 },
    { 0x4dbe7d3b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fe7d3e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3be7d43, SUC_lexicon_tags_1 },
    { 0xf33e7d44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbfde7d46, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163e7d4b, SUC_lexicon_tags_7 },
    { 0x7b7e7d4c, SUC_lexicon_tags_49 },
    { 0x701e7d4d, SUC_lexicon_tags_16 },
    { 0x7b5e7d4b, SUC_lexicon_tags_15 },
    { 0xcefe7d4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5e7d53, SUC_lexicon_tags_87 },
    { 0x5f9e7d53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3e7d57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xff5e7d5a, SUC_lexicon_tags_15 },
    { 0xc55e7d5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9e7d63, SUC_lexicon_tags_15 },
    { 0x8f5e7d64, SUC_lexicon_tags_4 },
    { 0x1b3e7d65, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf45e7d67, SUC_lexicon_tags_23 },
    { 0x22be7d67, SUC_lexicon_tags_11 },
    { 0xfc3e7d69, SUC_lexicon_tags_27 },
    { 0xf57e7d6a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3e7d6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1e7d72, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601e7d77, SUC_lexicon_tags_11 },
    { 0xb45e7d78, SUC_lexicon_tags_27 },
    { 0xeade7d78, SUC_lexicon_tags_33 },
    { 0xff9e7d7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99be7d7e, SUC_lexicon_tags_17 },
    { 0x8e5e7d7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbe7d85, SUC_lexicon_tags_32 },
    { 0x7dde7d86, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9efe7d89, SUC_lexicon_tags_62 },
    { 0x74fe7d8a, SUC_lexicon_tags_37 },
    { 0x687e7d8a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe4be7d8d, SUC_lexicon_tags_2 },
    { 0xbbe7d8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbe7d96, SUC_lexicon_tags_11 },
    { 0x9be7d96, SUC_lexicon_tags_15 },
    { 0x8c1e7d98, SUC_lexicon_tags_7 },
    { 0xa45e7d99, SUC_lexicon_tags_55 },
    { 0x95be7d9a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe81e7d9c, SUC_lexicon_tags_17 },
    { 0x16be7d9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7de7da5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1e7dae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527e7db2, SUC_lexicon_tags_27 },
    { 0xd51e7db3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1bbe7db5, SUC_lexicon_tags_44 },
    { 0x4e3e7db6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x26fe7db8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc0de7dba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51e7dbd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x75de7dc0, SUC_lexicon_tags_25 },
    { 0x315e7dc1, SUC_lexicon_tags_23 },
    { 0x563e7dc2, SUC_lexicon_tags_2 },
    { 0x267e7dc3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x995e7dc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbe7dc8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd77e7dca, SUC_lexicon_tags_27 },
    { 0x539e7dcb, SUC_lexicon_tags_9 },
    { 0xc67e7dca, SUC_lexicon_tags_3 },
    { 0x933e7dcd, SUC_lexicon_tags_1590 },
    { 0xf07e7dcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61e7dd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdabe7dd5, SUC_lexicon_tags_17 },
    { 0xf45e7dd6, SUC_lexicon_tags_26 },
    { 0x48be7dd7, SUC_lexicon_tags_37 },
    { 0x88de7dd6, SUC_lexicon_tags_1 },
    { 0x91e7dd8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xaa1e7ddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xffbe7dde, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x69fe7de1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7e7de8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf8fe7dea, SUC_lexicon_tags_33 },
    { 0x1fe7deb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1de7ded, SUC_lexicon_tags_11 },
    { 0xb79e7dee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1e7df2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01e7df5, SUC_lexicon_tags_10 },
    { 0x4e5e7df6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x611e7df8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9e7dfc, SUC_lexicon_tags_48 },
    { 0x731e7dfd, SUC_lexicon_tags_8 },
    { 0xb0be7dfe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9e7e02, SUC_lexicon_tags_29 },
    { 0x95de7e02, SUC_lexicon_tags_17 },
    { 0xb1e7e04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9de7e07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81de7e0b, SUC_lexicon_tags_12 },
    { 0xbbbe7e0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfdde7e0e, SUC_lexicon_tags_208 },
    { 0x5e9e7e0f, SUC_lexicon_tags_49 },
    { 0x181e7e10, SUC_lexicon_tags_12 },
    { 0x9d9e7e10, SUC_lexicon_tags_7 },
    { 0x591e7e0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x433e7e14, SUC_lexicon_tags_17 },
    { 0xa2de7e15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x94be7e18, SUC_lexicon_tags_55 },
    { 0xb79e7e19, SUC_lexicon_tags_21 },
    { 0xbd7e7e1a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fe7e20, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x85de7e22, SUC_lexicon_tags_70 },
    { 0xe35e7e23, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dfe7e25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cfe7e28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fe7e2d, SUC_lexicon_tags_19 },
    { 0xa19e7e2e, SUC_lexicon_tags_17 },
    { 0xe5e7e2f, SUC_lexicon_tags_65 },
    { 0xa79e7e30, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x629e7e32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1e7e38, SUC_lexicon_tags_37 },
    { 0x9f5e7e39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x665e7e3c, SUC_lexicon_tags_23 },
    { 0x183e7e3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xafe7e3f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01e7e46, SUC_lexicon_tags_29 },
    { 0x451e7e46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87e7e4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3de7e52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe87e7e54, SUC_lexicon_tags_61 },
    { 0x763e7e55, SUC_lexicon_tags_31 },
    { 0xab3e7e56, SUC_lexicon_tags_5 },
    { 0xa73e7e56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25e7e5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x797e7e5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e7e63, SUC_lexicon_tags_44 },
    { 0x4a3e7e64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fe7e68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f1e7e6a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99e7e6d, SUC_lexicon_tags_14 },
    { 0x473e7e6d, SUC_lexicon_tags_37 },
    { 0xce7e7e6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb09e7e71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2bfe7e73, SUC_lexicon_tags_37 },
    { 0x303e7e73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6bbe7e76, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95e7e7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c1e7e7f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5de7e83, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x525e7e85, SUC_lexicon_tags_25 },
    { 0xcb5e7e86, SUC_lexicon_tags_12 },
    { 0x8c5e7e85, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdb5e7e89, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8ade7e8b, SUC_lexicon_tags_7 },
    { 0x197e7e8c, SUC_lexicon_tags_17 },
    { 0xa63e7e8d, SUC_lexicon_tags_28 },
    { 0x2b5e7e8c, SUC_lexicon_tags_92 },
    { 0x11be7e8e, SUC_lexicon_tags_16 },
    { 0xd3fe7e8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bde7e94, SUC_lexicon_tags_7 },
    { 0x547e7e95, SUC_lexicon_tags_37 },
    { 0x9de7e94, SUC_lexicon_tags_60 },
    { 0xf63e7e97, SUC_lexicon_tags_21 },
    { 0x241e7e98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5fe7e9a, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xd2fe7e9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xde3e7e9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53e7ea1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x69be7ea4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41e7eab, SUC_lexicon_tags_15 },
    { 0xd2fe7eab, SUC_lexicon_tags_7 },
    { 0x507e7ead, SUC_lexicon_tags_44 },
    { 0xcfbe7eac, SUC_lexicon_tags_17 },
    { 0x183e7eaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67de7eb7, SUC_lexicon_tags_25 },
    { 0xfade7eb8, SUC_lexicon_tags_49 },
    { 0x333e7eb9, SUC_lexicon_tags_7 },
    { 0xb3e7eba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39be7ebc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1e7ec1, SUC_lexicon_tags_25 },
    { 0x639e7ec2, SUC_lexicon_tags_11 },
    { 0x657e7ec3, SUC_lexicon_tags_1 },
    { 0xe55e7ec4, SUC_lexicon_tags_27 },
    { 0x413e7ec5, SUC_lexicon_tags_9 },
    { 0x49e7ec6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5e7eca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x281e7ecd, SUC_lexicon_tags_1 },
    { 0x697e7ecd, SUC_lexicon_tags_19 },
    { 0x30de7ece, SUC_lexicon_tags_62 },
    { 0xa5de7ecf, SUC_lexicon_tags_7 },
    { 0x8f5e7ed1, SUC_lexicon_tags_37 },
    { 0x155e7ed2, SUC_lexicon_tags_36 },
    { 0x537e7ecf, SUC_lexicon_tags_2 },
    { 0xa29e7ed4, SUC_lexicon_tags_25 },
    { 0x577e7ed1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987e7edb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937e7ee2, SUC_lexicon_tags_7 },
    { 0x5c1e7ee2, SUC_lexicon_tags_37 },
    { 0xdbe7ee4, SUC_lexicon_tags_15 },
    { 0x975e7ee5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5e7ee9, SUC_lexicon_tags_0 },
    { 0xc0be7eea, SUC_lexicon_tags_43 },
    { 0xe4fe7eeb, SUC_lexicon_tags_45 },
    { 0x907e7eeb, SUC_lexicon_tags_15 },
    { 0x221e7eed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03e7f00, SUC_lexicon_tags_3 },
    { 0xb5de7f01, SUC_lexicon_tags_14 },
    { 0x65be7f01, SUC_lexicon_tags_31 },
    { 0xda5e7f03, SUC_lexicon_tags_8 },
    { 0xf7de7f04, SUC_lexicon_tags_11 },
    { 0x4b7e7f03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83de7f07, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x503e7f09, SUC_lexicon_tags_12 },
    { 0x715e7f09, SUC_lexicon_tags_15 },
    { 0x2d7e7f09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63e7f11, SUC_lexicon_tags_43 },
    { 0x1dbe7f11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9e7f16, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675e7f1b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf23e7f1d, SUC_lexicon_tags_44 },
    { 0x3fde7f1e, SUC_lexicon_tags_48 },
    { 0x5d7e7f1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9d3e7f21, SUC_lexicon_tags_8 },
    { 0x2e7e7f22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xea5e7f24, SUC_lexicon_tags_15 },
    { 0x245e7f25, SUC_lexicon_tags_25 },
    { 0xbc7e7f26, SUC_lexicon_tags_7 },
    { 0xb0fe7f26, SUC_lexicon_tags_485 },
    { 0xf91e7f28, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x41e7f2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7e7f2d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37e7f36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefe7f39, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca7e7f3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3e7f41, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xede7f45, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89be7f4b, SUC_lexicon_tags_55 },
    { 0x309e7f4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xff7e7f4f, SUC_lexicon_tags_10 },
    { 0xbafe7f4f, SUC_lexicon_tags_37 },
    { 0x947e7f51, SUC_lexicon_tags_15 },
    { 0x115e7f52, SUC_lexicon_tags_1 },
    { 0x119e7f51, SUC_lexicon_tags_2 },
    { 0xd91e7f53, SUC_lexicon_tags_37 },
    { 0x6dbe7f51, SUC_lexicon_tags_23 },
    { 0x8a9e7f51, SUC_lexicon_tags_9 },
    { 0x5cbe7f55, SUC_lexicon_tags_2 },
    { 0x1f1e7f58, SUC_lexicon_tags_19 },
    { 0xa81e7f59, SUC_lexicon_tags_23 },
    { 0xb5be7f5a, SUC_lexicon_tags_37 },
    { 0x3d5e7f5b, SUC_lexicon_tags_15 },
    { 0x49e7f5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897e7f61, SUC_lexicon_tags_87 },
    { 0x6d7e7f62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x713e7f64, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349e7f68, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0de7f6e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6dbe7f70, SUC_lexicon_tags_31 },
    { 0x6a3e7f70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629e7f77, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa0de7f79, SUC_lexicon_tags_27 },
    { 0x633e7f7a, SUC_lexicon_tags_17 },
    { 0xc5fe7f79, SUC_lexicon_tags_27 },
    { 0x6fde7f7c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75de7f81, SUC_lexicon_tags_23 },
    { 0x63fe7f82, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0x84de7f85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43e7f8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7e7f8f, SUC_lexicon_tags_15 },
    { 0x311e7f90, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa33e7f92, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x369e7f95, SUC_lexicon_tags_11 },
    { 0xa47e7f96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5e7f99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbe7f9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x247e7fa1, SUC_lexicon_tags_113 },
    { 0x997e7fa2, SUC_lexicon_tags_32 },
    { 0x393e7fa2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cde7faa, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65e7fae, SUC_lexicon_tags_60 },
    { 0xa0fe7faf, SUC_lexicon_tags_14 },
    { 0x9ffe7fb0, SUC_lexicon_tags_33 },
    { 0x69de7fb0, SUC_lexicon_tags_46 },
    { 0x925e7fae, SUC_lexicon_tags_9 },
    { 0x791e7fb3, SUC_lexicon_tags_19 },
    { 0x3bbe7fb1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x12de7fb6, SUC_lexicon_tags_77 },
    { 0xa47e7fb6, SUC_lexicon_tags_12 },
    { 0xc3de7fb8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691e7fbc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x529e7fbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83be7fc2, SUC_lexicon_tags_43 },
    { 0xb59e7fc2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3de7fc9, SUC_lexicon_tags_15 },
    { 0xd31e7fca, SUC_lexicon_tags_17 },
    { 0xfe1e7fcb, SUC_lexicon_tags_33 },
    { 0x259e7fcc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3e7fcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb7e7fd1, SUC_lexicon_tags_37 },
    { 0x67e7fd2, SUC_lexicon_tags_15 },
    { 0x353e7fd3, SUC_lexicon_tags_25 },
    { 0xa87e7fd3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x28be7fd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23e7fda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63e7fe0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x929e7fe3, SUC_lexicon_tags_61 },
    { 0xf6fe7fe3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x51e7fe7, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07e7feb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfe7fef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xee9e7ff1, SUC_lexicon_tags_30 },
    { 0x7e7ff2, SUC_lexicon_tags_49 },
    { 0x545e7ff1, SUC_lexicon_tags_37 },
    { 0x459e7ff3, SUC_lexicon_tags_47 },
    { 0x851e7ff5, SUC_lexicon_tags_17 },
    { 0xdbde7ff6, SUC_lexicon_tags_11 },
    { 0xf9be7ff6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abe8000, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x705e8002, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50de8006, SUC_lexicon_tags_35 },
    { 0xd21e8007, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fe800a, SUC_lexicon_tags_16 },
    { 0x15de800b, SUC_lexicon_tags_7 },
    { 0xcade800c, SUC_lexicon_tags_41 },
    { 0xfebe800c, SUC_lexicon_tags_10 },
    { 0x357e800c, SUC_lexicon_tags_15 },
    { 0x817e800c, SUC_lexicon_tags_17 },
    { 0x2f3e8010, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3e8013, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4de801a, SUC_lexicon_tags_1 },
    { 0x949e801a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe801e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b9e8020, SUC_lexicon_tags_38 },
    { 0x14fe8021, SUC_lexicon_tags_12 },
    { 0x693e8022, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28fe8028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1e802c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65e8030, SUC_lexicon_tags_625 },
    { 0x527e8031, SUC_lexicon_tags_11 },
    { 0xc39e8032, SUC_lexicon_tags_84 },
    { 0x61e8033, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7e8037, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45e803a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43e803e, SUC_lexicon_tags_17 },
    { 0xe43e803e, SUC_lexicon_tags_0 },
    { 0x683e8040, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x661e8042, SUC_lexicon_tags_68 },
    { 0xe3de8043, SUC_lexicon_tags_27 },
    { 0x1f9e8044, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05e8048, SUC_lexicon_tags_17 },
    { 0xa77e8049, SUC_lexicon_tags_0 },
    { 0x873e804a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e8050, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fe8057, SUC_lexicon_tags_47 },
    { 0xa41e8058, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebe805d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa35e805f, SUC_lexicon_tags_37 },
    { 0x3ede8060, SUC_lexicon_tags_16 },
    { 0x13de8060, SUC_lexicon_tags_16 },
    { 0x9a7e8061, SUC_lexicon_tags_17 },
    { 0x7efe8060, SUC_lexicon_tags_33 },
    { 0x9bfe8064, SUC_lexicon_tags_1 },
    { 0xf81e8065, SUC_lexicon_tags_14 },
    { 0xf37e8066, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7e8069, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6be806b, SUC_lexicon_tags_12 },
    { 0x185e806c, SUC_lexicon_tags_43 },
    { 0x27de806d, SUC_lexicon_tags_15 },
    { 0x2c5e806d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9de8071, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x91de8074, SUC_lexicon_tags_25 },
    { 0x1b3e8074, SUC_lexicon_tags_37 },
    { 0x99fe8076, SUC_lexicon_tags_26 },
    { 0x795e8076, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465e807c, SUC_lexicon_tags_11 },
    { 0x389e807c, SUC_lexicon_tags_63 },
    { 0xaa3e807c, SUC_lexicon_tags_136 },
    { 0x76de807c, SUC_lexicon_tags_4 },
    { 0x57be807d, SUC_lexicon_tags_60 },
    { 0xefde8081, SUC_lexicon_tags_37 },
    { 0xd37e8082, SUC_lexicon_tags_37 },
    { 0x17be8083, SUC_lexicon_tags_66 },
    { 0x81be8084, SUC_lexicon_tags_62 },
    { 0x70fe8083, SUC_lexicon_tags_6 },
    { 0x80fe8084, SUC_lexicon_tags_1 },
    { 0xa6fe8083, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x897e8089, SUC_lexicon_tags_12 },
    { 0x2d3e808a, SUC_lexicon_tags_28 },
    { 0x5e5e808a, SUC_lexicon_tags_25 },
    { 0xb31e808c, SUC_lexicon_tags_37 },
    { 0xd3de808d, SUC_lexicon_tags_25 },
    { 0x545e808e, SUC_lexicon_tags_17 },
    { 0x82de808d, SUC_lexicon_tags_25 },
    { 0xc9e8090, SUC_lexicon_tags_25 },
    { 0xd8be8091, SUC_lexicon_tags_26 },
    { 0x55de808e, SUC_lexicon_tags_11 },
    { 0x10fe808e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xce3e8095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf33e8098, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xafde809b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fe809f, SUC_lexicon_tags_32 },
    { 0xf7fe80a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5de80a3, SUC_lexicon_tags_4 },
    { 0xcefe80a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd17e80a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe31e80a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55e80ac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x909e80ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf9e80b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x52de80b2, SUC_lexicon_tags_17 },
    { 0xcdbe80b3, SUC_lexicon_tags_8 },
    { 0xbebe80b4, SUC_lexicon_tags_60 },
    { 0x389e80b2, SUC_lexicon_tags_49 },
    { 0x5f5e80b6, SUC_lexicon_tags_37 },
    { 0x3cbe80b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x755e80b9, SUC_lexicon_tags_37 },
    { 0x4e1e80ba, SUC_lexicon_tags_9 },
    { 0x811e80ba, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd6fe80bd, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0, NULL },
    { 0xd69e80c0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3e80c9, SUC_lexicon_tags_26 },
    { 0xf35e80ca, SUC_lexicon_tags_8 },
    { 0xe6be80ca, SUC_lexicon_tags_15 },
    { 0xcf3e80cc, SUC_lexicon_tags_27 },
    { 0x66fe80cd, SUC_lexicon_tags_37 },
    { 0xf7de80c9, SUC_lexicon_tags_2 },
    { 0x285e80cf, SUC_lexicon_tags_39 },
    { 0x3f5e80ca, SUC_lexicon_tags_1 },
    { 0x447e80cd, SUC_lexicon_tags_28 },
    { 0xc31e80d2, SUC_lexicon_tags_23 },
    { 0x651e80ce, SUC_lexicon_tags_84 },
    { 0xd7e80d4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x25de80d6, SUC_lexicon_tags_7 },
    { 0xc7be80d7, SUC_lexicon_tags_25 },
    { 0x73be80d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98be80db, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4ede80dd, SUC_lexicon_tags_11 },
    { 0x10fe80dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9e80e1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165e80e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09e80ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x931e80ef, SUC_lexicon_tags_49 },
    { 0x93fe80f0, SUC_lexicon_tags_28 },
    { 0x877e80ef, SUC_lexicon_tags_55 },
    { 0x453e80ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95de80f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe80fb, SUC_lexicon_tags_17 },
    { 0x23fe80fc, SUC_lexicon_tags_2 },
    { 0x91de80fd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755e8105, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65e810e, SUC_lexicon_tags_12 },
    { 0xa8fe810e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9e8114, SUC_lexicon_tags_29 },
    { 0xe97e8114, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541e811f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x811e8121, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcade8124, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15be8128, SUC_lexicon_tags_6 },
    { 0xd3fe8129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x421e812c, SUC_lexicon_tags_23 },
    { 0x183e812c, SUC_lexicon_tags_12 },
    { 0xef1e812e, SUC_lexicon_tags_17 },
    { 0xfa9e812f, SUC_lexicon_tags_1 },
    { 0x3f9e812f, SUC_lexicon_tags_0 },
    { 0xe19e812f, SUC_lexicon_tags_9 },
    { 0x54be812c, SUC_lexicon_tags_27 },
    { 0x9be8133, SUC_lexicon_tags_138 },
    { 0x77be8134, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2abe8136, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcdfe8138, SUC_lexicon_tags_32 },
    { 0xb3be8139, SUC_lexicon_tags_15 },
    { 0x5a9e8139, SUC_lexicon_tags_1076 },
    { 0, NULL },
    { 0xb5e813c, SUC_lexicon_tags_7 },
    { 0x1e1e813d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fde8142, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8be8145, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9e8149, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc45e814b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x57de814d, SUC_lexicon_tags_11 },
    { 0x6afe814e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fe8155, SUC_lexicon_tags_14 },
    { 0x8ffe8156, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7be8159, SUC_lexicon_tags_37 },
    { 0x5de815a, SUC_lexicon_tags_55 },
    { 0x211e815a, SUC_lexicon_tags_15 },
    { 0xe3de815c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x217e815f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9e8162, SUC_lexicon_tags_25 },
    { 0x46de8162, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85e8168, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b5e816a, SUC_lexicon_tags_437 },
    { 0, NULL },
    { 0x289e816c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x101e816e, SUC_lexicon_tags_8 },
    { 0x549e816f, SUC_lexicon_tags_19 },
    { 0x743e8170, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ede8173, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3be8176, SUC_lexicon_tags_46 },
    { 0x7e7e8176, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x99fe817a, SUC_lexicon_tags_49 },
    { 0x575e817b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf97e817d, SUC_lexicon_tags_28 },
    { 0x4ede817e, SUC_lexicon_tags_64 },
    { 0x74fe817d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x433e8182, SUC_lexicon_tags_16 },
    { 0xc0be8183, SUC_lexicon_tags_37 },
    { 0xac3e8182, SUC_lexicon_tags_190 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5e8188, SUC_lexicon_tags_25 },
    { 0x737e8188, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95de818d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225e8191, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x4d7e8193, SUC_lexicon_tags_0 },
    { 0x19fe8194, SUC_lexicon_tags_11 },
    { 0xb7be8193, SUC_lexicon_tags_5 },
    { 0x437e8196, SUC_lexicon_tags_6 },
    { 0xaa1e8197, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x253e8199, SUC_lexicon_tags_107 },
    { 0x2d1e8199, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65e819d, SUC_lexicon_tags_11 },
    { 0xe83e819e, SUC_lexicon_tags_1 },
    { 0x3de819f, SUC_lexicon_tags_25 },
    { 0x867e819f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63de81a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2be81a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875e81ac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebe81b3, SUC_lexicon_tags_37 },
    { 0xeebe81b4, SUC_lexicon_tags_17 },
    { 0xebde81b5, SUC_lexicon_tags_60 },
    { 0x753e81b3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa39e81b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacde81bc, SUC_lexicon_tags_25 },
    { 0x1c1e81bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfe81c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ade81c2, SUC_lexicon_tags_77 },
    { 0xd39e81c3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x217e81c5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x915e81c8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf39e81ca, SUC_lexicon_tags_31 },
    { 0x60fe81cb, SUC_lexicon_tags_7 },
    { 0x9a3e81cc, SUC_lexicon_tags_47 },
    { 0xbbbe81cc, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fe81d0, SUC_lexicon_tags_11 },
    { 0xa9be81d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5e81d6, SUC_lexicon_tags_38 },
    { 0x44de81d7, SUC_lexicon_tags_0 },
    { 0x901e81d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4dfe81da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fe81dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8dbe81df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22de81e7, SUC_lexicon_tags_38 },
    { 0xecde81e8, SUC_lexicon_tags_27 },
    { 0x1cfe81e9, SUC_lexicon_tags_6 },
    { 0xc61e81e8, SUC_lexicon_tags_7 },
    { 0xc55e81eb, SUC_lexicon_tags_31 },
    { 0x3cfe81eb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3e81f2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389e81f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaede81fa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5de81fc, SUC_lexicon_tags_7 },
    { 0x66de81fc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfafe81ff, SUC_lexicon_tags_11 },
    { 0x923e81ff, SUC_lexicon_tags_4 },
    { 0xb9be8201, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdade8204, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7e8209, SUC_lexicon_tags_31 },
    { 0xe87e820a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfe820e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6a9e8210, SUC_lexicon_tags_19 },
    { 0x19be8211, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15e8216, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ede821b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fde821e, SUC_lexicon_tags_28 },
    { 0xbbe821f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1e8226, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x32de8229, SUC_lexicon_tags_11 },
    { 0xf09e822a, SUC_lexicon_tags_25 },
    { 0x1b3e822a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63e8234, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0x911e8236, SUC_lexicon_tags_11 },
    { 0x54be8237, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3e823b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc63e823d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9e8244, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x40be8247, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73de8249, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb59e824b, SUC_lexicon_tags_11 },
    { 0xa1be824c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9e8250, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47e8257, SUC_lexicon_tags_33 },
    { 0x67de8258, SUC_lexicon_tags_15 },
    { 0xe21e8259, SUC_lexicon_tags_7 },
    { 0xf97e825a, SUC_lexicon_tags_2 },
    { 0x4a1e8258, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe5fe825d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6de8260, SUC_lexicon_tags_0 },
    { 0x437e8261, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95be8265, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5d9e8267, SUC_lexicon_tags_14 },
    { 0x3b5e8268, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1ade826a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893e826f, SUC_lexicon_tags_49 },
    { 0xcb5e8270, SUC_lexicon_tags_25 },
    { 0x557e8270, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1e7e8273, SUC_lexicon_tags_11 },
    { 0x1d3e8274, SUC_lexicon_tags_11 },
    { 0x43fe8274, SUC_lexicon_tags_49 },
    { 0x4c9e8276, SUC_lexicon_tags_34 },
    { 0xe6de8276, SUC_lexicon_tags_25 },
    { 0x7fbe8275, SUC_lexicon_tags_26 },
    { 0x7a3e8279, SUC_lexicon_tags_17 },
    { 0x429e827a, SUC_lexicon_tags_59 },
    { 0x1de827b, SUC_lexicon_tags_12 },
    { 0x3fde827c, SUC_lexicon_tags_37 },
    { 0xb41e827c, SUC_lexicon_tags_0 },
    { 0xd1de827d, SUC_lexicon_tags_12 },
    { 0x857e827f, SUC_lexicon_tags_27 },
    { 0x565e8280, SUC_lexicon_tags_17 },
    { 0x96de8277, SUC_lexicon_tags_19 },
    { 0xf3fe8282, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbe8286, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309e828e, SUC_lexicon_tags_14 },
    { 0xfd7e828f, SUC_lexicon_tags_38 },
    { 0x9f9e828e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fe8294, SUC_lexicon_tags_26 },
    { 0x7bbe8295, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bde8299, SUC_lexicon_tags_9 },
    { 0x369e829a, SUC_lexicon_tags_37 },
    { 0xab5e829b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9e829e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x621e82a0, SUC_lexicon_tags_6 },
    { 0xaffe82a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce1e82a3, SUC_lexicon_tags_17 },
    { 0x215e82a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc79e82a6, SUC_lexicon_tags_11 },
    { 0xb1be82a7, SUC_lexicon_tags_37 },
    { 0x213e82a7, SUC_lexicon_tags_6 },
    { 0x5f7e82a9, SUC_lexicon_tags_63 },
    { 0xb75e82aa, SUC_lexicon_tags_7 },
    { 0x497e82a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d9e82ad, SUC_lexicon_tags_47 },
    { 0xdb5e82ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143e82b3, SUC_lexicon_tags_486 },
    { 0, NULL },
    { 0x391e82b5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xc1be82b7, SUC_lexicon_tags_15 },
    { 0x5f5e82b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47e82bb, SUC_lexicon_tags_33 },
    { 0x10fe82bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1e82c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125e82cc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xe8be82ce, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5e82d1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fde82d4, SUC_lexicon_tags_5 },
    { 0x1e7e82d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fe82d8, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x87e82db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe4be82dd, SUC_lexicon_tags_44 },
    { 0x53fe82de, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x225e82e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8be82e6, SUC_lexicon_tags_1 },
    { 0x705e82e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8b3e82e9, SUC_lexicon_tags_4 },
    { 0xf13e82ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9e82ef, SUC_lexicon_tags_27 },
    { 0x543e82f0, SUC_lexicon_tags_17 },
    { 0x293e82f1, SUC_lexicon_tags_87 },
    { 0x5cbe82f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7be82f6, SUC_lexicon_tags_12 },
    { 0x4c9e82f7, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fde82fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1e82fe, SUC_lexicon_tags_14 },
    { 0x711e82ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911e8304, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb39e8306, SUC_lexicon_tags_11 },
    { 0xc5be8307, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe93e8309, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55e830e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6b1e8310, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1e8313, SUC_lexicon_tags_221 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaade8317, SUC_lexicon_tags_66 },
    { 0xb7be8318, SUC_lexicon_tags_87 },
    { 0x36be8319, SUC_lexicon_tags_14 },
    { 0xdc7e831a, SUC_lexicon_tags_5 },
    { 0xc21e8318, SUC_lexicon_tags_14 },
    { 0xafe831c, SUC_lexicon_tags_14 },
    { 0x4c3e831a, SUC_lexicon_tags_15 },
    { 0x3b5e8317, SUC_lexicon_tags_43 },
    { 0xcd1e8317, SUC_lexicon_tags_46 },
    { 0x4a3e8320, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1e8324, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffe8328, SUC_lexicon_tags_17 },
    { 0xec9e8329, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4b7e832b, SUC_lexicon_tags_12 },
    { 0x9a9e832c, SUC_lexicon_tags_3 },
    { 0x70fe832c, SUC_lexicon_tags_19 },
    { 0xc3de832c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25e8333, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xefe8335, SUC_lexicon_tags_1 },
    { 0xdb9e8336, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x1e8338, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e3e833a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143e8342, SUC_lexicon_tags_32 },
    { 0xc41e8342, SUC_lexicon_tags_15 },
    { 0x355e8342, SUC_lexicon_tags_12 },
    { 0x99de8345, SUC_lexicon_tags_25 },
    { 0xd5be8345, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99e8349, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x35fe834b, SUC_lexicon_tags_68 },
    { 0x549e834b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e834e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x561e8350, SUC_lexicon_tags_6 },
    { 0x5d7e8350, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3e8354, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x38fe8356, SUC_lexicon_tags_87 },
    { 0xed7e8357, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeefe8359, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177e835e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9e8361, SUC_lexicon_tags_70 },
    { 0xa6be8362, SUC_lexicon_tags_43 },
    { 0xa67e8363, SUC_lexicon_tags_23 },
    { 0xef5e8363, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x88de8367, SUC_lexicon_tags_0 },
    { 0xde9e8368, SUC_lexicon_tags_39 },
    { 0x27fe8369, SUC_lexicon_tags_66 },
    { 0x787e836a, SUC_lexicon_tags_37 },
    { 0xe97e836b, SUC_lexicon_tags_8 },
    { 0x9a9e836a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf0be836e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6cfe8370, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe7fe8372, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa17e8374, SUC_lexicon_tags_12 },
    { 0xa29e8375, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e8380, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbe8384, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x90de8386, SUC_lexicon_tags_45 },
    { 0xbede8387, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e9e8389, SUC_lexicon_tags_9 },
    { 0xa4be838a, SUC_lexicon_tags_0 },
    { 0x403e838b, SUC_lexicon_tags_11 },
    { 0x72de838c, SUC_lexicon_tags_17 },
    { 0x6e1e838d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70de838f, SUC_lexicon_tags_2 },
    { 0x48be8390, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x973e8393, SUC_lexicon_tags_8 },
    { 0xb11e8393, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0de8397, SUC_lexicon_tags_1 },
    { 0x71e8397, SUC_lexicon_tags_12 },
    { 0x6b5e8399, SUC_lexicon_tags_29 },
    { 0xb2fe839a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x327e839c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfdfe839e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd5be83a0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11e83a5, SUC_lexicon_tags_7 },
    { 0x905e83a5, SUC_lexicon_tags_15 },
    { 0x69fe83a5, SUC_lexicon_tags_37 },
    { 0x6fde83a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5e83ab, SUC_lexicon_tags_42 },
    { 0xe8be83ac, SUC_lexicon_tags_12 },
    { 0xc0be83ad, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x631e83af, SUC_lexicon_tags_27 },
    { 0xa0be83b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b7e83b2, SUC_lexicon_tags_12 },
    { 0xbb9e83b3, SUC_lexicon_tags_38 },
    { 0x3dfe83b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77be83b7, SUC_lexicon_tags_47 },
    { 0x107e83b8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc93e83ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x609e83bd, SUC_lexicon_tags_65 },
    { 0xcffe83be, SUC_lexicon_tags_11 },
    { 0xe65e83bd, SUC_lexicon_tags_2 },
    { 0x787e83c0, SUC_lexicon_tags_7 },
    { 0x309e83c0, SUC_lexicon_tags_6 },
    { 0xd9fe83c2, SUC_lexicon_tags_4 },
    { 0xa89e83c2, SUC_lexicon_tags_17 },
    { 0x969e83bf, SUC_lexicon_tags_2 },
    { 0x93be83bf, SUC_lexicon_tags_17 },
    { 0x99de83c2, SUC_lexicon_tags_14 },
    { 0x68be83c7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441e83cb, SUC_lexicon_tags_6 },
    { 0x93be83cc, SUC_lexicon_tags_43 },
    { 0xffde83cc, SUC_lexicon_tags_31 },
    { 0xc79e83ce, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6ade83d0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xeafe83d2, SUC_lexicon_tags_25 },
    { 0xbb9e83d2, SUC_lexicon_tags_37 },
    { 0x4d1e83d4, SUC_lexicon_tags_7 },
    { 0x727e83d5, SUC_lexicon_tags_19 },
    { 0xb8de83d5, SUC_lexicon_tags_53 },
    { 0x7dde83d3, SUC_lexicon_tags_25 },
    { 0xb25e83d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5be83da, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb6be83dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf53e83de, SUC_lexicon_tags_37 },
    { 0xe81e83df, SUC_lexicon_tags_25 },
    { 0x1ade83e0, SUC_lexicon_tags_18 },
    { 0xc05e83e0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31e83e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x339e83e6, SUC_lexicon_tags_47 },
    { 0x46de83e6, SUC_lexicon_tags_17 },
    { 0x491e83e8, SUC_lexicon_tags_12 },
    { 0x17de83e8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29fe83ef, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0x17be83f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175e83f5, SUC_lexicon_tags_7 },
    { 0x1bde83f6, SUC_lexicon_tags_55 },
    { 0x4b3e83f6, SUC_lexicon_tags_48 },
    { 0xb93e83f7, SUC_lexicon_tags_96 },
    { 0xeffe83f9, SUC_lexicon_tags_2 },
    { 0xc81e83fa, SUC_lexicon_tags_47 },
    { 0x70fe83fb, SUC_lexicon_tags_17 },
    { 0x3dde83fc, SUC_lexicon_tags_27 },
    { 0x7d5e83fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfe8401, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1de8405, SUC_lexicon_tags_7 },
    { 0xcbbe8406, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2fe8408, SUC_lexicon_tags_11 },
    { 0xaade8409, SUC_lexicon_tags_17 },
    { 0x29fe840a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc49e840c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbebe840e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x445e8410, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x719e8412, SUC_lexicon_tags_15 },
    { 0xa29e8412, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231e841c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbe841e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709e8423, SUC_lexicon_tags_34 },
    { 0x5b5e8423, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87e8428, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5e842e, SUC_lexicon_tags_12 },
    { 0xaf9e842f, SUC_lexicon_tags_12 },
    { 0x505e842f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x599e8432, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1e8438, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x903e843a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4ffe843c, SUC_lexicon_tags_8 },
    { 0xb5be843d, SUC_lexicon_tags_19 },
    { 0x8c7e843c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abe8443, SUC_lexicon_tags_14 },
    { 0xa51e8444, SUC_lexicon_tags_9 },
    { 0x845e8445, SUC_lexicon_tags_49 },
    { 0xdbe8446, SUC_lexicon_tags_45 },
    { 0x493e8447, SUC_lexicon_tags_17 },
    { 0x96fe8448, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x39de844b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd89e844d, SUC_lexicon_tags_45 },
    { 0x20fe844e, SUC_lexicon_tags_23 },
    { 0xf6fe844f, SUC_lexicon_tags_30 },
    { 0x39de8450, SUC_lexicon_tags_6 },
    { 0x643e8451, SUC_lexicon_tags_37 },
    { 0xdc3e8452, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x78de8454, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ebe8456, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893e845b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x445e845e, SUC_lexicon_tags_2 },
    { 0x4efe845f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1e8463, SUC_lexicon_tags_60 },
    { 0x671e8464, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa95e8466, SUC_lexicon_tags_28 },
    { 0x70de8466, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71e846c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4e5e846e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01e8475, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcbfe8477, SUC_lexicon_tags_12 },
    { 0xb19e8478, SUC_lexicon_tags_47 },
    { 0x1de8479, SUC_lexicon_tags_25 },
    { 0x31e8478, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529e8480, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ade8484, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dde848b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7e848e, SUC_lexicon_tags_11 },
    { 0xd5fe848e, SUC_lexicon_tags_48 },
    { 0x403e848e, SUC_lexicon_tags_15 },
    { 0x5ede8490, SUC_lexicon_tags_43 },
    { 0xf71e8492, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfe849a, SUC_lexicon_tags_7 },
    { 0x401e849b, SUC_lexicon_tags_27 },
    { 0x46fe849c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85e84a3, SUC_lexicon_tags_48 },
    { 0xaebe84a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9de84a6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403e84aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x387e84ad, SUC_lexicon_tags_30 },
    { 0xad1e84ae, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295e84b6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83e84bb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1e84c0, SUC_lexicon_tags_15 },
    { 0x4bfe84c1, SUC_lexicon_tags_15 },
    { 0xbfde84c1, SUC_lexicon_tags_15 },
    { 0x843e84c3, SUC_lexicon_tags_15 },
    { 0xc2fe84c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab3e84c6, SUC_lexicon_tags_1 },
    { 0x7c3e84c7, SUC_lexicon_tags_216 },
    { 0x9cfe84c8, SUC_lexicon_tags_33 },
    { 0x3b7e84c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59e84cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x95e84d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda5e84d2, SUC_lexicon_tags_17 },
    { 0x641e84d2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbe84d6, SUC_lexicon_tags_12 },
    { 0x469e84d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fde84da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3fe84dc, SUC_lexicon_tags_66 },
    { 0x579e84dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf35e84df, SUC_lexicon_tags_31 },
    { 0xedfe84df, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfc7e84e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbb7e84e4, SUC_lexicon_tags_82 },
    { 0xe5fe84e4, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5e84e8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x565e84ea, SUC_lexicon_tags_37 },
    { 0x389e84eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf41e84ed, SUC_lexicon_tags_10 },
    { 0x483e84ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58be84f0, SUC_lexicon_tags_11 },
    { 0x923e84f1, SUC_lexicon_tags_1 },
    { 0x921e84f2, SUC_lexicon_tags_0 },
    { 0x59be84f3, SUC_lexicon_tags_29 },
    { 0x37de84f1, SUC_lexicon_tags_17 },
    { 0xab3e84f5, SUC_lexicon_tags_34 },
    { 0xde3e84f4, SUC_lexicon_tags_43 },
    { 0x4be84f7, SUC_lexicon_tags_25 },
    { 0x963e84f5, SUC_lexicon_tags_6 },
    { 0xc19e84f9, SUC_lexicon_tags_7 },
    { 0x1a1e84f9, SUC_lexicon_tags_8 },
    { 0x9e9e84fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589e850a, SUC_lexicon_tags_14 },
    { 0x273e850b, SUC_lexicon_tags_31 },
    { 0x4dbe850c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbafe8514, SUC_lexicon_tags_2 },
    { 0x8d3e8514, SUC_lexicon_tags_27 },
    { 0xe03e8515, SUC_lexicon_tags_63 },
    { 0xd1fe8517, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727e8522, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca9e8524, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe99e8526, SUC_lexicon_tags_6 },
    { 0x5d9e8527, SUC_lexicon_tags_15 },
    { 0xdcbe8527, SUC_lexicon_tags_60 },
    { 0xe19e8529, SUC_lexicon_tags_160 },
    { 0x29de852a, SUC_lexicon_tags_45 },
    { 0x115e852a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfbde852d, SUC_lexicon_tags_15 },
    { 0xcc7e852e, SUC_lexicon_tags_11 },
    { 0xef9e852f, SUC_lexicon_tags_40 },
    { 0x897e8530, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28de8534, SUC_lexicon_tags_0 },
    { 0x1c3e8535, SUC_lexicon_tags_16 },
    { 0xa79e8536, SUC_lexicon_tags_27 },
    { 0x33e8537, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617e853c, SUC_lexicon_tags_1 },
    { 0xecbe853d, SUC_lexicon_tags_14 },
    { 0xa27e853d, SUC_lexicon_tags_14 },
    { 0x22de853f, SUC_lexicon_tags_12 },
    { 0xe8de8540, SUC_lexicon_tags_38 },
    { 0x195e8541, SUC_lexicon_tags_2 },
    { 0xc6de853e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7e8549, SUC_lexicon_tags_7 },
    { 0xa0fe854a, SUC_lexicon_tags_63 },
    { 0x447e854b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81be854e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43e8553, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3be8557, SUC_lexicon_tags_14 },
    { 0x8d9e8558, SUC_lexicon_tags_57 },
    { 0x9d5e8559, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaefe855c, SUC_lexicon_tags_11 },
    { 0x9c3e855d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcfde855f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5e8565, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dde8568, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3e856d, SUC_lexicon_tags_38 },
    { 0x53fe856d, SUC_lexicon_tags_47 },
    { 0x21e856f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34fe8574, SUC_lexicon_tags_17 },
    { 0x207e8575, SUC_lexicon_tags_25 },
    { 0x35de8574, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x60be8579, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbe857d, SUC_lexicon_tags_82 },
    { 0xb55e857e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38fe8580, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7e8585, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x841e8588, SUC_lexicon_tags_11 },
    { 0xb2fe8589, SUC_lexicon_tags_43 },
    { 0x10fe8588, SUC_lexicon_tags_129 },
    { 0x16be858b, SUC_lexicon_tags_7 },
    { 0x1efe858c, SUC_lexicon_tags_27 },
    { 0xef1e858c, SUC_lexicon_tags_37 },
    { 0xebde858e, SUC_lexicon_tags_45 },
    { 0xf95e858e, SUC_lexicon_tags_1 },
    { 0xb65e8590, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2abe8592, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29e8595, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5e859c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbe85a1, SUC_lexicon_tags_56 },
    { 0x305e85a2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x415e85a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89de85ab, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abe85ae, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x61de85b1, SUC_lexicon_tags_70 },
    { 0x627e85b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3e85b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb67e85b9, SUC_lexicon_tags_12 },
    { 0x717e85ba, SUC_lexicon_tags_14 },
    { 0x7c9e85ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x883e85be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x26be85c0, SUC_lexicon_tags_12 },
    { 0x70fe85c1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fde85c6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9e85c9, SUC_lexicon_tags_15 },
    { 0xf6de85c9, SUC_lexicon_tags_11 },
    { 0xed1e85cb, SUC_lexicon_tags_14 },
    { 0xbde85c9, SUC_lexicon_tags_68 },
    { 0x3ede85cc, SUC_lexicon_tags_1 },
    { 0xfa1e85cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247e85d5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x561e85d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdb7e85d9, SUC_lexicon_tags_32 },
    { 0x40fe85da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03e85de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27e85e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ade85f3, SUC_lexicon_tags_15 },
    { 0x113e85f4, SUC_lexicon_tags_1 },
    { 0x1b3e85f3, SUC_lexicon_tags_43 },
    { 0x187e85f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455e85fa, SUC_lexicon_tags_49 },
    { 0x27be85fb, SUC_lexicon_tags_16 },
    { 0xc7be85fa, SUC_lexicon_tags_11 },
    { 0xdd5e85fc, SUC_lexicon_tags_47 },
    { 0xca5e85fa, SUC_lexicon_tags_57 },
    { 0x399e85fd, SUC_lexicon_tags_49 },
    { 0xb95e8600, SUC_lexicon_tags_9 },
    { 0xc65e8601, SUC_lexicon_tags_11 },
    { 0x783e85fa, SUC_lexicon_tags_17 },
    { 0x103e85fd, SUC_lexicon_tags_2 },
    { 0x1ede85ff, SUC_lexicon_tags_6 },
    { 0x75be8601, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5e860c, SUC_lexicon_tags_40 },
    { 0x177e860c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdf9e860f, SUC_lexicon_tags_38 },
    { 0x257e860f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1e8614, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbe5e8616, SUC_lexicon_tags_66 },
    { 0x76de8616, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x17be861a, SUC_lexicon_tags_13 },
    { 0xe63e861b, SUC_lexicon_tags_34 },
    { 0x809e861c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xaa3e861e, SUC_lexicon_tags_38 },
    { 0xb9de861e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497e8625, SUC_lexicon_tags_38 },
    { 0x589e8625, SUC_lexicon_tags_1 },
    { 0xa4fe8627, SUC_lexicon_tags_2 },
    { 0x591e8628, SUC_lexicon_tags_11 },
    { 0x725e8629, SUC_lexicon_tags_12 },
    { 0xd73e8625, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661e862e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x169e8631, SUC_lexicon_tags_1 },
    { 0x355e8632, SUC_lexicon_tags_11 },
    { 0x25fe8631, SUC_lexicon_tags_29 },
    { 0xcb7e8633, SUC_lexicon_tags_30 },
    { 0x23e8635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x237e8637, SUC_lexicon_tags_17 },
    { 0xb7fe8637, SUC_lexicon_tags_33 },
    { 0x39fe8637, SUC_lexicon_tags_85 },
    { 0x205e863a, SUC_lexicon_tags_33 },
    { 0x619e863b, SUC_lexicon_tags_21 },
    { 0x943e863c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5e8646, SUC_lexicon_tags_13 },
    { 0xe21e8646, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2be864a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafe864e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfe8651, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929e865a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10de8662, SUC_lexicon_tags_12 },
    { 0x645e8663, SUC_lexicon_tags_17 },
    { 0x8efe8664, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afe8667, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85be866c, SUC_lexicon_tags_11 },
    { 0xde7e866c, SUC_lexicon_tags_3 },
    { 0xe73e866d, SUC_lexicon_tags_68 },
    { 0x609e866e, SUC_lexicon_tags_27 },
    { 0x5c7e8670, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459e8675, SUC_lexicon_tags_53 },
    { 0x823e8676, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77e8679, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21e867d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd25e867f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x217e8682, SUC_lexicon_tags_15 },
    { 0x287e8682, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1e8686, SUC_lexicon_tags_60 },
    { 0xde9e8686, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a9e8689, SUC_lexicon_tags_55 },
    { 0xd99e868a, SUC_lexicon_tags_39 },
    { 0x5ebe868a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43e868e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95e8697, SUC_lexicon_tags_32 },
    { 0xcbde8698, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1e86a0, SUC_lexicon_tags_28 },
    { 0x6e7e86a1, SUC_lexicon_tags_14 },
    { 0xf0fe86a2, SUC_lexicon_tags_63 },
    { 0xccbe86a3, SUC_lexicon_tags_34 },
    { 0x58fe86a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1e86a9, SUC_lexicon_tags_57 },
    { 0x30de86aa, SUC_lexicon_tags_5 },
    { 0xf45e86aa, SUC_lexicon_tags_32 },
    { 0x37de86ab, SUC_lexicon_tags_53 },
    { 0x67e86aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe86b0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbe86b3, SUC_lexicon_tags_0 },
    { 0xecde86b4, SUC_lexicon_tags_17 },
    { 0xfe5e86b5, SUC_lexicon_tags_9 },
    { 0x2e1e86b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89e86ba, SUC_lexicon_tags_45 },
    { 0x551e86bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5e86be, SUC_lexicon_tags_0 },
    { 0x23fe86be, SUC_lexicon_tags_27 },
    { 0x329e86c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc1e86c2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5e86c5, SUC_lexicon_tags_34 },
    { 0xd15e86c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dde86cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fe86ce, SUC_lexicon_tags_17 },
    { 0x469e86cf, SUC_lexicon_tags_27 },
    { 0xdc5e86d0, SUC_lexicon_tags_25 },
    { 0xa9e86d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x24fe86d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x27fe86d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x863e86d8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77e86e3, SUC_lexicon_tags_2 },
    { 0x753e86e4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afe86e9, SUC_lexicon_tags_0 },
    { 0x903e86e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17be86ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb6fe86f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6a3e86f2, SUC_lexicon_tags_8 },
    { 0xb17e86f2, SUC_lexicon_tags_26 },
    { 0xaa9e86f4, SUC_lexicon_tags_37 },
    { 0xa0be86f4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x887e86f7, SUC_lexicon_tags_0 },
    { 0xd6de86f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xacfe86fb, SUC_lexicon_tags_17 },
    { 0xd5be86fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361e8700, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbe8703, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753e8708, SUC_lexicon_tags_30 },
    { 0xb59e8709, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1de8710, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81e8713, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7e8716, SUC_lexicon_tags_61 },
    { 0x133e8717, SUC_lexicon_tags_15 },
    { 0x60de8717, SUC_lexicon_tags_6 },
    { 0x807e8718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xea7e871c, SUC_lexicon_tags_118 },
    { 0xdc7e871d, SUC_lexicon_tags_112 },
    { 0xf13e871d, SUC_lexicon_tags_12 },
    { 0xe0be871f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x401e8722, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3be8724, SUC_lexicon_tags_17 },
    { 0xbc3e8725, SUC_lexicon_tags_2 },
    { 0xd37e8726, SUC_lexicon_tags_17 },
    { 0x24fe8725, SUC_lexicon_tags_17 },
    { 0x84be8727, SUC_lexicon_tags_37 },
    { 0xd05e8729, SUC_lexicon_tags_25 },
    { 0x76de872a, SUC_lexicon_tags_45 },
    { 0xed7e8727, SUC_lexicon_tags_85 },
    { 0xa91e8728, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x46be872e, SUC_lexicon_tags_11 },
    { 0x24be872f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3e8738, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5e873b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xa2fe873d, SUC_lexicon_tags_8 },
    { 0x947e873d, SUC_lexicon_tags_11 },
    { 0xf3fe873d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27e8745, SUC_lexicon_tags_0 },
    { 0x2f1e8746, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cde8749, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfe874d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x223e8750, SUC_lexicon_tags_33 },
    { 0xdc3e8750, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46de8758, SUC_lexicon_tags_17 },
    { 0xe8de8758, SUC_lexicon_tags_30 },
    { 0x6dbe875a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x43e875d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7e8763, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x497e8766, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x737e8768, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9e8773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37de8779, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x29fe877b, SUC_lexicon_tags_79 },
    { 0xc73e877b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7e8786, SUC_lexicon_tags_0 },
    { 0x959e8786, SUC_lexicon_tags_30 },
    { 0xb39e8788, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbe878c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf1be878e, SUC_lexicon_tags_17 },
    { 0xc25e878e, SUC_lexicon_tags_45 },
    { 0x283e878e, SUC_lexicon_tags_46 },
    { 0x473e8791, SUC_lexicon_tags_23 },
    { 0xbdbe878f, SUC_lexicon_tags_55 },
    { 0x7dde8793, SUC_lexicon_tags_32 },
    { 0x96de8794, SUC_lexicon_tags_85 },
    { 0x263e8794, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffe879a, SUC_lexicon_tags_45 },
    { 0xf23e879b, SUC_lexicon_tags_29 },
    { 0x22de879a, SUC_lexicon_tags_7 },
    { 0xf3fe879d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5e87a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfe87a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc53e87a5, SUC_lexicon_tags_2 },
    { 0xb7e87a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63e87ac, SUC_lexicon_tags_45 },
    { 0xad1e87ac, SUC_lexicon_tags_1 },
    { 0x23fe87ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3e87b5, SUC_lexicon_tags_14 },
    { 0xf1fe87b5, SUC_lexicon_tags_17 },
    { 0xd13e87b7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27e87bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533e87c4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x869e87c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x80be87c9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x541e87cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x19de87ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafde87d2, SUC_lexicon_tags_1 },
    { 0xf53e87d3, SUC_lexicon_tags_12 },
    { 0xc9fe87d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9e87d9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xba7e87db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85e87df, SUC_lexicon_tags_15 },
    { 0x229e87df, SUC_lexicon_tags_37 },
    { 0x31e87e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58de87e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ffe87e9, SUC_lexicon_tags_37 },
    { 0x7bfe87ea, SUC_lexicon_tags_47 },
    { 0x873e87eb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55e87ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be87f2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79e87f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6de87f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x831e87fb, SUC_lexicon_tags_47 },
    { 0xe93e87fc, SUC_lexicon_tags_25 },
    { 0xb9fe87fd, SUC_lexicon_tags_11 },
    { 0x943e87fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc6fe8800, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cde8804, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ede8809, SUC_lexicon_tags_15 },
    { 0x477e880a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbe880d, SUC_lexicon_tags_772 },
    { 0xf8fe880d, SUC_lexicon_tags_2 },
    { 0xa89e880f, SUC_lexicon_tags_19 },
    { 0x249e8810, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9e8814, SUC_lexicon_tags_87 },
    { 0x571e8815, SUC_lexicon_tags_25 },
    { 0xd91e8814, SUC_lexicon_tags_37 },
    { 0x149e8814, SUC_lexicon_tags_49 },
    { 0x2b9e8818, SUC_lexicon_tags_38 },
    { 0xd77e8819, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955e881f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8be8822, SUC_lexicon_tags_1 },
    { 0xa9de8823, SUC_lexicon_tags_15 },
    { 0x49be8824, SUC_lexicon_tags_12 },
    { 0xc1fe8824, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x73be8827, SUC_lexicon_tags_22 },
    { 0xf01e8828, SUC_lexicon_tags_15 },
    { 0xcd5e8827, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1d5e882b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe71e882d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf99e882f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x16de8832, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x281e8835, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35e8839, SUC_lexicon_tags_7 },
    { 0xb93e883a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703e8843, SUC_lexicon_tags_17 },
    { 0x4bde8844, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789e8854, SUC_lexicon_tags_103 },
    { 0xacde8855, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xf4fe8857, SUC_lexicon_tags_49 },
    { 0x981e8858, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ade885e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbe8862, SUC_lexicon_tags_9 },
    { 0xfd3e8863, SUC_lexicon_tags_37 },
    { 0xeefe8862, SUC_lexicon_tags_55 },
    { 0xb3fe8865, SUC_lexicon_tags_14 },
    { 0xcb7e8866, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6de8869, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3ede886b, SUC_lexicon_tags_11 },
    { 0x11e886c, SUC_lexicon_tags_1 },
    { 0x5de886d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccfe8874, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4ffe8876, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x13de8878, SUC_lexicon_tags_15 },
    { 0xdade8878, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169e887e, SUC_lexicon_tags_37 },
    { 0xc61e887f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4be888a, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747e8890, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129e8894, SUC_lexicon_tags_45 },
    { 0x339e8895, SUC_lexicon_tags_37 },
    { 0x4ade8895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5e889b, SUC_lexicon_tags_21 },
    { 0xa3e889c, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fe88a4, SUC_lexicon_tags_49 },
    { 0x4dde88a5, SUC_lexicon_tags_47 },
    { 0x321e88a5, SUC_lexicon_tags_38 },
    { 0x669e88a4, SUC_lexicon_tags_39 },
    { 0x4b7e88a4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77de88ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8de88b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71e88b4, SUC_lexicon_tags_43 },
    { 0x323e88b5, SUC_lexicon_tags_45 },
    { 0x9b5e88b4, SUC_lexicon_tags_17 },
    { 0x55fe88b6, SUC_lexicon_tags_33 },
    { 0x36fe88b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x76fe88ba, SUC_lexicon_tags_17 },
    { 0xa4be88ba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72fe88c2, SUC_lexicon_tags_151 },
    { 0xce7e88c3, SUC_lexicon_tags_10 },
    { 0xa49e88c3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe69e88c6, SUC_lexicon_tags_10 },
    { 0x513e88c7, SUC_lexicon_tags_63 },
    { 0x8dde88c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x253e88cb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdc9e88cd, SUC_lexicon_tags_27 },
    { 0x1d5e88ce, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7e88d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25de88d8, SUC_lexicon_tags_15 },
    { 0x473e88d8, SUC_lexicon_tags_17 },
    { 0x935e88d8, SUC_lexicon_tags_21 },
    { 0x2f7e88db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96be88e8, SUC_lexicon_tags_25 },
    { 0xf37e88e9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x901e88eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417e88fe, SUC_lexicon_tags_29 },
    { 0xed7e88ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1e8902, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fde8908, SUC_lexicon_tags_207 },
    { 0xa75e8909, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xacfe890b, SUC_lexicon_tags_7 },
    { 0x28fe890c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x191e890e, SUC_lexicon_tags_37 },
    { 0xb71e890f, SUC_lexicon_tags_2 },
    { 0x9d5e890f, SUC_lexicon_tags_56 },
    { 0x2d1e8911, SUC_lexicon_tags_0 },
    { 0xd37e8910, SUC_lexicon_tags_33 },
    { 0x9f3e8913, SUC_lexicon_tags_48 },
    { 0x1f3e8914, SUC_lexicon_tags_15 },
    { 0xc55e890f, SUC_lexicon_tags_17 },
    { 0x4bde8913, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7e891a, SUC_lexicon_tags_12 },
    { 0x977e891b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abe891f, SUC_lexicon_tags_25 },
    { 0x16fe891f, SUC_lexicon_tags_15 },
    { 0x385e8921, SUC_lexicon_tags_4 },
    { 0x97be8921, SUC_lexicon_tags_12 },
    { 0x447e891f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafe892a, SUC_lexicon_tags_39 },
    { 0xe87e892a, SUC_lexicon_tags_25 },
    { 0xbdfe892c, SUC_lexicon_tags_92 },
    { 0x6a1e892a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81e8930, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x771e8933, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbe8936, SUC_lexicon_tags_28 },
    { 0xe67e8937, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3fe8939, SUC_lexicon_tags_6 },
    { 0xfe7e893a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x11de893c, SUC_lexicon_tags_21 },
    { 0x8abe893d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99e8942, SUC_lexicon_tags_2 },
    { 0x9a9e8942, SUC_lexicon_tags_11 },
    { 0x667e8944, SUC_lexicon_tags_55 },
    { 0x9bde8945, SUC_lexicon_tags_14 },
    { 0x87fe8945, SUC_lexicon_tags_33 },
    { 0xdd3e8947, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbe894d, SUC_lexicon_tags_1 },
    { 0x819e894d, SUC_lexicon_tags_27 },
    { 0x66be894f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb8be8951, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9e8955, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbe895c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcede8960, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcdfe8962, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13e8965, SUC_lexicon_tags_11 },
    { 0x82de8966, SUC_lexicon_tags_7 },
    { 0x48fe8965, SUC_lexicon_tags_26 },
    { 0xf63e8968, SUC_lexicon_tags_60 },
    { 0xaffe8968, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1e896f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2be8976, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5e897a, SUC_lexicon_tags_47 },
    { 0xcabe897b, SUC_lexicon_tags_39 },
    { 0x793e897a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x80de897f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9b3e8981, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x53de8984, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fe898b, SUC_lexicon_tags_25 },
    { 0xb9e898c, SUC_lexicon_tags_1 },
    { 0xcc3e898b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7e8992, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4be8994, SUC_lexicon_tags_37 },
    { 0x7d7e8995, SUC_lexicon_tags_11 },
    { 0x6d5e8994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfde8998, SUC_lexicon_tags_19 },
    { 0x911e8999, SUC_lexicon_tags_17 },
    { 0x599e899a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x833e899c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x517e899e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1e89a2, SUC_lexicon_tags_1 },
    { 0x1b9e89a3, SUC_lexicon_tags_11 },
    { 0x157e89a4, SUC_lexicon_tags_47 },
    { 0xd0de89a2, SUC_lexicon_tags_37 },
    { 0x591e89a2, SUC_lexicon_tags_11 },
    { 0x2bfe89a7, SUC_lexicon_tags_2 },
    { 0xec1e89a8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767e89ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95e89b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9f9e89b6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431e89bc, SUC_lexicon_tags_47 },
    { 0x55be89bc, SUC_lexicon_tags_17 },
    { 0x1d9e89be, SUC_lexicon_tags_113 },
    { 0x115e89be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc9e89c1, SUC_lexicon_tags_11 },
    { 0xbf1e89c2, SUC_lexicon_tags_37 },
    { 0x811e89c2, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xbc5e89c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbe89cf, SUC_lexicon_tags_4 },
    { 0x21be89d0, SUC_lexicon_tags_23 },
    { 0x967e89d0, SUC_lexicon_tags_37 },
    { 0x297e89d2, SUC_lexicon_tags_15 },
    { 0xb43e89d3, SUC_lexicon_tags_65 },
    { 0x2f1e89d2, SUC_lexicon_tags_66 },
    { 0x725e89d0, SUC_lexicon_tags_8 },
    { 0xb9de89d6, SUC_lexicon_tags_63 },
    { 0x4ade89d7, SUC_lexicon_tags_47 },
    { 0x63be89d6, SUC_lexicon_tags_31 },
    { 0x5f7e89cf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbe89de, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbe89e3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805e89eb, SUC_lexicon_tags_79 },
    { 0xae5e89eb, SUC_lexicon_tags_13 },
    { 0x699e89ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7e89f0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949e89f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0de89f7, SUC_lexicon_tags_1 },
    { 0x995e89f7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x135e89fb, SUC_lexicon_tags_43 },
    { 0xcb9e89fc, SUC_lexicon_tags_188 },
    { 0x9f5e89fc, SUC_lexicon_tags_43 },
    { 0x6d7e89fe, SUC_lexicon_tags_27 },
    { 0x74fe89ff, SUC_lexicon_tags_15 },
    { 0x4e1e89ff, SUC_lexicon_tags_47 },
    { 0xcd9e89fc, SUC_lexicon_tags_23 },
    { 0x667e8a00, SUC_lexicon_tags_2 },
    { 0x625e89fc, SUC_lexicon_tags_45 },
    { 0xaa3e8a04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x487e8a07, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23e8a0a, SUC_lexicon_tags_14 },
    { 0x583e8a0a, SUC_lexicon_tags_11 },
    { 0xd23e8a0a, SUC_lexicon_tags_10 },
    { 0xe27e8a0d, SUC_lexicon_tags_12 },
    { 0x7a3e8a0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2de8a14, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7e8a17, SUC_lexicon_tags_32 },
    { 0x971e8a18, SUC_lexicon_tags_9 },
    { 0xe6de8a18, SUC_lexicon_tags_11 },
    { 0x463e8a19, SUC_lexicon_tags_38 },
    { 0xe5e8a1b, SUC_lexicon_tags_28 },
    { 0xec5e8a1b, SUC_lexicon_tags_49 },
    { 0xd43e8a1d, SUC_lexicon_tags_32 },
    { 0x32de8a1e, SUC_lexicon_tags_37 },
    { 0x689e8a1e, SUC_lexicon_tags_253 },
    { 0xd61e8a20, SUC_lexicon_tags_37 },
    { 0xcbfe8a20, SUC_lexicon_tags_1 },
    { 0x2be8a22, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27e8a26, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x67be8a28, SUC_lexicon_tags_44 },
    { 0xaf1e8a29, SUC_lexicon_tags_33 },
    { 0x9e8a2a, SUC_lexicon_tags_37 },
    { 0xc8fe8a28, SUC_lexicon_tags_17 },
    { 0x94de8a29, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51e8a37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e8a3a, SUC_lexicon_tags_12 },
    { 0x3d7e8a3a, SUC_lexicon_tags_25 },
    { 0x461e8a3b, SUC_lexicon_tags_60 },
    { 0xc8be8a3c, SUC_lexicon_tags_57 },
    { 0x807e8a3d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fe8a44, SUC_lexicon_tags_2 },
    { 0xb73e8a44, SUC_lexicon_tags_2 },
    { 0xcdde8a46, SUC_lexicon_tags_167 },
    { 0xd05e8a46, SUC_lexicon_tags_12 },
    { 0xa13e8a48, SUC_lexicon_tags_17 },
    { 0x6de8a46, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9be8a4c, SUC_lexicon_tags_26 },
    { 0xa9e8a4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa79e8a4f, SUC_lexicon_tags_82 },
    { 0x12be8a4f, SUC_lexicon_tags_47 },
    { 0x1b7e8a4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491e8a55, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457e8a5e, SUC_lexicon_tags_2 },
    { 0xe7fe8a5e, SUC_lexicon_tags_11 },
    { 0x6fde8a60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7e8a63, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x937e8a65, SUC_lexicon_tags_25 },
    { 0xc8fe8a65, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf15e8a68, SUC_lexicon_tags_37 },
    { 0xe2be8a69, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11e8a6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x809e8a6f, SUC_lexicon_tags_25 },
    { 0xebfe8a6f, SUC_lexicon_tags_1 },
    { 0x8f3e8a71, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fe8a76, SUC_lexicon_tags_29 },
    { 0x46fe8a77, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07e8a7b, SUC_lexicon_tags_33 },
    { 0x8a5e8a7b, SUC_lexicon_tags_26 },
    { 0x7c7e8a7d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5e8a80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7e8a86, SUC_lexicon_tags_49 },
    { 0xc65e8a87, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbe8a8a, SUC_lexicon_tags_15 },
    { 0x7f9e8a8b, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41fe8a94, SUC_lexicon_tags_46 },
    { 0xfa7e8a94, SUC_lexicon_tags_12 },
    { 0x387e8a96, SUC_lexicon_tags_68 },
    { 0xbebe8a94, SUC_lexicon_tags_25 },
    { 0x8ede8a95, SUC_lexicon_tags_60 },
    { 0x2fde8a98, SUC_lexicon_tags_26 },
    { 0xf65e8a9a, SUC_lexicon_tags_17 },
    { 0x65be8a99, SUC_lexicon_tags_34 },
    { 0x941e8a96, SUC_lexicon_tags_63 },
    { 0x345e8a9d, SUC_lexicon_tags_43 },
    { 0xd51e8a9e, SUC_lexicon_tags_12 },
    { 0xc87e8a9f, SUC_lexicon_tags_29 },
    { 0x9dde8aa0, SUC_lexicon_tags_0 },
    { 0x9be8aa1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1e8aa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfdfe8aaa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfe8aaf, SUC_lexicon_tags_29 },
    { 0x3e5e8ab0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fde8ab3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605e8abb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53be8abf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91e8ac4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84be8ac9, SUC_lexicon_tags_26 },
    { 0xb17e8ac9, SUC_lexicon_tags_15 },
    { 0xe15e8acb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7be8ace, SUC_lexicon_tags_15 },
    { 0xad1e8acf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48be8ad4, SUC_lexicon_tags_12 },
    { 0x613e8ad5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x489e8ad7, SUC_lexicon_tags_38 },
    { 0xff5e8ad7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x93be8ada, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143e8ae1, SUC_lexicon_tags_49 },
    { 0xf7fe8ae2, SUC_lexicon_tags_37 },
    { 0x477e8ae3, SUC_lexicon_tags_33 },
    { 0xb95e8ae4, SUC_lexicon_tags_2 },
    { 0xeb7e8ae3, SUC_lexicon_tags_17 },
    { 0xd0be8ae1, SUC_lexicon_tags_2 },
    { 0xebe8ae7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173e8aec, SUC_lexicon_tags_15 },
    { 0x4a7e8aec, SUC_lexicon_tags_49 },
    { 0xc0be8aed, SUC_lexicon_tags_0 },
    { 0xf15e8aef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcede8afe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x89be8b01, SUC_lexicon_tags_68 },
    { 0xb6be8b02, SUC_lexicon_tags_0 },
    { 0xce3e8b02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf89e8b05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7e8b08, SUC_lexicon_tags_31 },
    { 0xd2be8b08, SUC_lexicon_tags_19 },
    { 0x8c7e8b08, SUC_lexicon_tags_351 },
    { 0x801e8b09, SUC_lexicon_tags_33 },
    { 0x22fe8b0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7ebe8b0e, SUC_lexicon_tags_1 },
    { 0x131e8b0f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9e8b12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf29e8b15, SUC_lexicon_tags_11 },
    { 0x321e8b16, SUC_lexicon_tags_37 },
    { 0x8dbe8b15, SUC_lexicon_tags_62 },
    { 0x32de8b18, SUC_lexicon_tags_6 },
    { 0xa51e8b19, SUC_lexicon_tags_160 },
    { 0xc49e8b1a, SUC_lexicon_tags_112 },
    { 0x243e8b19, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fe8b1e, SUC_lexicon_tags_33 },
    { 0x81e8b1f, SUC_lexicon_tags_8 },
    { 0x9a9e8b1f, SUC_lexicon_tags_19 },
    { 0x1b5e8b21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x873e8b23, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x375e8b25, SUC_lexicon_tags_47 },
    { 0x399e8b26, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfe8b2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabe8b2f, SUC_lexicon_tags_235 },
    { 0xfd9e8b2f, SUC_lexicon_tags_17 },
    { 0x5c1e8b31, SUC_lexicon_tags_29 },
    { 0xa57e8b32, SUC_lexicon_tags_2 },
    { 0xbd5e8b31, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5a7e8b35, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x27fe8b37, SUC_lexicon_tags_13 },
    { 0x8fbe8b37, SUC_lexicon_tags_9 },
    { 0xf39e8b37, SUC_lexicon_tags_61 },
    { 0x2a9e8b38, SUC_lexicon_tags_25 },
    { 0x295e8b38, SUC_lexicon_tags_23 },
    { 0x1cde8b39, SUC_lexicon_tags_29 },
    { 0x273e8b3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x295e8b40, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7bde8b42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fe8b45, SUC_lexicon_tags_175 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ade8b49, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fe8b4e, SUC_lexicon_tags_19 },
    { 0x947e8b4e, SUC_lexicon_tags_46 },
    { 0xfb3e8b50, SUC_lexicon_tags_30 },
    { 0xc91e8b51, SUC_lexicon_tags_25 },
    { 0xaa9e8b52, SUC_lexicon_tags_15 },
    { 0x24fe8b53, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcde8b55, SUC_lexicon_tags_8 },
    { 0x6fbe8b56, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3fe8b58, SUC_lexicon_tags_31 },
    { 0xf05e8b58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x46de8b5b, SUC_lexicon_tags_2 },
    { 0x3a7e8b5c, SUC_lexicon_tags_39 },
    { 0xfbfe8b5d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaffe8b5f, SUC_lexicon_tags_33 },
    { 0xc23e8b5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393e8b6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2efe8b6d, SUC_lexicon_tags_44 },
    { 0x73e8b6e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7e8b74, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401e8b7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x845e8b81, SUC_lexicon_tags_11 },
    { 0x275e8b82, SUC_lexicon_tags_16 },
    { 0xff9e8b81, SUC_lexicon_tags_1 },
    { 0x3b7e8b84, SUC_lexicon_tags_11 },
    { 0x6f3e8b83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1e8b88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9e8b8b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9e8b8f, SUC_lexicon_tags_0 },
    { 0x617e8b90, SUC_lexicon_tags_108 },
    { 0x65e8b91, SUC_lexicon_tags_27 },
    { 0xe1e8b90, SUC_lexicon_tags_37 },
    { 0x9be8b92, SUC_lexicon_tags_17 },
    { 0x1f1e8b94, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303e8b98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe65e8b9a, SUC_lexicon_tags_55 },
    { 0xc05e8b9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797e8ba6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x747e8ba9, SUC_lexicon_tags_38 },
    { 0xd21e8baa, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbb7e8bac, SUC_lexicon_tags_66 },
    { 0x6a5e8bad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x86be8baf, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92de8bb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1e8bb6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdabe8bb8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbbe8bba, SUC_lexicon_tags_30 },
    { 0x28fe8bba, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x11de8bbd, SUC_lexicon_tags_12 },
    { 0x8fe8bbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fe8bc2, SUC_lexicon_tags_17 },
    { 0xda1e8bc2, SUC_lexicon_tags_38 },
    { 0x725e8bc4, SUC_lexicon_tags_11 },
    { 0x361e8bc5, SUC_lexicon_tags_17 },
    { 0x79e8bc5, SUC_lexicon_tags_13 },
    { 0x385e8bc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc99e8bc9, SUC_lexicon_tags_47 },
    { 0x873e8bc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5e8bcd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9e8bd0, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4abe8bd2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe7fe8bd4, SUC_lexicon_tags_15 },
    { 0x27de8bd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x36de8bd7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219e8bdb, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x45de8bde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85be8be6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7a3e8be8, SUC_lexicon_tags_7 },
    { 0xbabe8be9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbe8bec, SUC_lexicon_tags_3 },
    { 0x37e8bec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda7e8bef, SUC_lexicon_tags_27 },
    { 0x5e9e8bf0, SUC_lexicon_tags_11 },
    { 0xcc7e8bf1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1b7e8bf3, SUC_lexicon_tags_0 },
    { 0x18fe8bf4, SUC_lexicon_tags_10 },
    { 0x5e8bf5, SUC_lexicon_tags_12 },
    { 0x685e8bf3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x72de8bf8, SUC_lexicon_tags_45 },
    { 0xb4fe8bf9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5e8bfe, SUC_lexicon_tags_6 },
    { 0x997e8bfe, SUC_lexicon_tags_28 },
    { 0x36fe8bfe, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5e8c09, SUC_lexicon_tags_25 },
    { 0xbd9e8c09, SUC_lexicon_tags_26 },
    { 0x207e8c0b, SUC_lexicon_tags_10 },
    { 0xb57e8c0c, SUC_lexicon_tags_17 },
    { 0xdc3e8c0d, SUC_lexicon_tags_17 },
    { 0xde1e8c0c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89e8c12, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957e8c16, SUC_lexicon_tags_14 },
    { 0x541e8c17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x575e8c1a, SUC_lexicon_tags_5 },
    { 0x59e8c1a, SUC_lexicon_tags_37 },
    { 0x943e8c1c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffe8c1f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa9fe8c21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x921e8c24, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195e8c29, SUC_lexicon_tags_60 },
    { 0x7afe8c29, SUC_lexicon_tags_92 },
    { 0xb01e8c2b, SUC_lexicon_tags_47 },
    { 0x7cfe8c2c, SUC_lexicon_tags_63 },
    { 0xc5de8c2d, SUC_lexicon_tags_27 },
    { 0xb8fe8c2e, SUC_lexicon_tags_43 },
    { 0x33be8c2e, SUC_lexicon_tags_55 },
    { 0x64be8c2d, SUC_lexicon_tags_2 },
    { 0x7ebe8c2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa9e8c33, SUC_lexicon_tags_26 },
    { 0x41e8c34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf7be8c36, SUC_lexicon_tags_12 },
    { 0x963e8c37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17e8c3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaefe8c3d, SUC_lexicon_tags_17 },
    { 0x769e8c3d, SUC_lexicon_tags_31 },
    { 0x2bbe8c3f, SUC_lexicon_tags_63 },
    { 0xb6de8c3f, SUC_lexicon_tags_14 },
    { 0xce7e8c41, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x30fe8c43, SUC_lexicon_tags_17 },
    { 0xb7de8c44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x779e8c46, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84be8c4c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e3e8c4e, SUC_lexicon_tags_23 },
    { 0x717e8c4e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7e8c52, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9e8c56, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbc9e8c58, SUC_lexicon_tags_23 },
    { 0x6c5e8c59, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1e8c5c, SUC_lexicon_tags_530 },
    { 0, NULL },
    { 0xb33e8c5e, SUC_lexicon_tags_55 },
    { 0x787e8c5f, SUC_lexicon_tags_12 },
    { 0x2e3e8c5f, SUC_lexicon_tags_17 },
    { 0x2b3e8c61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe5de8c63, SUC_lexicon_tags_1 },
    { 0x515e8c64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29de8c6a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebde8c71, SUC_lexicon_tags_11 },
    { 0xe09e8c72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9be8c76, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd33e8c78, SUC_lexicon_tags_17 },
    { 0x1c9e8c79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x259e8c7c, SUC_lexicon_tags_12 },
    { 0x7fbe8c7d, SUC_lexicon_tags_33 },
    { 0x6d1e8c7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727e8c88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6fe8c8a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fe8c8d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xa21e8c8f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbede8c91, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61e8c95, SUC_lexicon_tags_27 },
    { 0x6f5e8c96, SUC_lexicon_tags_17 },
    { 0x679e8c97, SUC_lexicon_tags_25 },
    { 0xa11e8c98, SUC_lexicon_tags_37 },
    { 0x1c5e8c95, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbe8c9f, SUC_lexicon_tags_31 },
    { 0xf9be8c9f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65e8ca6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9de8ca8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa31e8caa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65e8caf, SUC_lexicon_tags_15 },
    { 0x34be8cb0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebe8cb9, SUC_lexicon_tags_34 },
    { 0xc7de8cba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa07e8cbc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3e8cbf, SUC_lexicon_tags_15 },
    { 0x499e8cc0, SUC_lexicon_tags_11 },
    { 0x883e8cc1, SUC_lexicon_tags_12 },
    { 0xc3de8cbf, SUC_lexicon_tags_118 },
    { 0x57fe8cc3, SUC_lexicon_tags_28 },
    { 0xfe3e8cc4, SUC_lexicon_tags_37 },
    { 0x955e8cc4, SUC_lexicon_tags_23 },
    { 0xf4fe8cc2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd19e8cc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa33e8ccb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fde8cce, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb87e8cd0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61de8cdd, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59e8ce6, SUC_lexicon_tags_101 },
    { 0xeb7e8ce7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95e8cea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xffe8cec, SUC_lexicon_tags_37 },
    { 0x8cbe8ced, SUC_lexicon_tags_12 },
    { 0x229e8cee, SUC_lexicon_tags_28 },
    { 0xea7e8cee, SUC_lexicon_tags_27 },
    { 0x39be8cf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85e8cf5, SUC_lexicon_tags_37 },
    { 0x95e8cf6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fe8cf9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5e8cfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77be8d05, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x3d3e8d07, SUC_lexicon_tags_34 },
    { 0x5cfe8d08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fe8d0a, SUC_lexicon_tags_15 },
    { 0x713e8d0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x29be8d0d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xf03e8d0f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5e8d14, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x739e8d17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fe8d1d, SUC_lexicon_tags_25 },
    { 0x765e8d1e, SUC_lexicon_tags_43 },
    { 0x375e8d1f, SUC_lexicon_tags_23 },
    { 0x5fe8d1d, SUC_lexicon_tags_2 },
    { 0x5ffe8d1e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x59be8d23, SUC_lexicon_tags_63 },
    { 0x5dfe8d24, SUC_lexicon_tags_7 },
    { 0x1b5e8d25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4fe8d27, SUC_lexicon_tags_7 },
    { 0x1bde8d28, SUC_lexicon_tags_82 },
    { 0xc2de8d29, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56be8d2d, SUC_lexicon_tags_1050 },
    { 0x171e8d2e, SUC_lexicon_tags_15 },
    { 0xbdfe8d2e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cfe8d35, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe63e8d37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6de8d39, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x235e8d3c, SUC_lexicon_tags_0 },
    { 0x5c7e8d3c, SUC_lexicon_tags_27 },
    { 0x381e8d3d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfe8d43, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1e8d47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7e8d4c, SUC_lexicon_tags_65 },
    { 0x38be8d4d, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5be8d50, SUC_lexicon_tags_33 },
    { 0x62de8d51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95fe8d53, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7e8d57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8b9e8d59, SUC_lexicon_tags_15 },
    { 0x875e8d5a, SUC_lexicon_tags_45 },
    { 0x4ebe8d5b, SUC_lexicon_tags_37 },
    { 0xe1fe8d5b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fe8d60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfffe8d62, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fe8d65, SUC_lexicon_tags_17 },
    { 0x3b7e8d65, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd39e8d68, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9e8d6d, SUC_lexicon_tags_27 },
    { 0xe2de8d6e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc0be8d70, SUC_lexicon_tags_10 },
    { 0x4dde8d70, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9e8d74, SUC_lexicon_tags_11 },
    { 0x86de8d75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899e8d7a, SUC_lexicon_tags_11 },
    { 0xb5be8d7b, SUC_lexicon_tags_0 },
    { 0xfa1e8d7b, SUC_lexicon_tags_47 },
    { 0xa6be8d7c, SUC_lexicon_tags_15 },
    { 0x493e8d7e, SUC_lexicon_tags_38 },
    { 0x31de8d7f, SUC_lexicon_tags_29 },
    { 0xc71e8d7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fe8d83, SUC_lexicon_tags_19 },
    { 0x2f5e8d84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x36fe8d86, SUC_lexicon_tags_87 },
    { 0xaa9e8d86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb53e8d89, SUC_lexicon_tags_12 },
    { 0x845e8d89, SUC_lexicon_tags_11 },
    { 0xa19e8d8b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe11e8d8d, SUC_lexicon_tags_8 },
    { 0xd1be8d8e, SUC_lexicon_tags_49 },
    { 0x619e8d8e, SUC_lexicon_tags_74 },
    { 0xe1e8d90, SUC_lexicon_tags_4 },
    { 0x467e8d8f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59e8d96, SUC_lexicon_tags_6 },
    { 0x2e1e8d97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47e8d99, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3e8d9e, SUC_lexicon_tags_47 },
    { 0x37fe8d9f, SUC_lexicon_tags_31 },
    { 0xa8be8da0, SUC_lexicon_tags_11 },
    { 0xc5e8da1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x505e8da3, SUC_lexicon_tags_77 },
    { 0x549e8da4, SUC_lexicon_tags_43 },
    { 0x181e8da5, SUC_lexicon_tags_11 },
    { 0xc3be8da6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37e8dad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffe8db1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x341e8db3, SUC_lexicon_tags_32 },
    { 0xa3e8db4, SUC_lexicon_tags_66 },
    { 0x61de8db4, SUC_lexicon_tags_56 },
    { 0xf2de8db4, SUC_lexicon_tags_9 },
    { 0xa1de8db7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8de8dba, SUC_lexicon_tags_37 },
    { 0x3dfe8dbb, SUC_lexicon_tags_1 },
    { 0xa57e8dbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd57e8dbe, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x10fe8dc1, SUC_lexicon_tags_7 },
    { 0x1c3e8dc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5a3e8dc4, SUC_lexicon_tags_23 },
    { 0x53fe8dc5, SUC_lexicon_tags_32 },
    { 0x2c9e8dc4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a3e8dc8, SUC_lexicon_tags_8 },
    { 0x8c9e8dc9, SUC_lexicon_tags_12 },
    { 0x75fe8dca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfe8dd3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd2fe8dd5, SUC_lexicon_tags_47 },
    { 0xda3e8dd6, SUC_lexicon_tags_8 },
    { 0x2d3e8dd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1e8ddb, SUC_lexicon_tags_16 },
    { 0xf0be8ddc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901e8de4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe8de7, SUC_lexicon_tags_275 },
    { 0x59fe8de8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffe8dec, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x739e8dee, SUC_lexicon_tags_25 },
    { 0xafde8def, SUC_lexicon_tags_63 },
    { 0x191e8dee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1e8df4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd01e8df6, SUC_lexicon_tags_33 },
    { 0x1d1e8df7, SUC_lexicon_tags_2 },
    { 0xde5e8df8, SUC_lexicon_tags_70 },
    { 0x869e8df7, SUC_lexicon_tags_17 },
    { 0x3f1e8dfa, SUC_lexicon_tags_17 },
    { 0x699e8dfb, SUC_lexicon_tags_47 },
    { 0xa93e8dfc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fe8e03, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xda7e8e05, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x63be8e07, SUC_lexicon_tags_13 },
    { 0x5c3e8e08, SUC_lexicon_tags_11 },
    { 0x105e8e08, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x59be8e0b, SUC_lexicon_tags_2 },
    { 0x47de8e0c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x371e8e0e, SUC_lexicon_tags_15 },
    { 0xf2be8e0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x21e8e11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703e8e17, SUC_lexicon_tags_0 },
    { 0x31de8e17, SUC_lexicon_tags_30 },
    { 0xfc1e8e19, SUC_lexicon_tags_60 },
    { 0x68be8e19, SUC_lexicon_tags_18 },
    { 0xac3e8e1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x453e8e1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9e8e20, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x68de8e22, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03e8e25, SUC_lexicon_tags_15 },
    { 0xeede8e26, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb5e8e28, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd1de8e2a, SUC_lexicon_tags_27 },
    { 0xa7e8e2a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fe8e2e, SUC_lexicon_tags_45 },
    { 0x69be8e2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfe8e33, SUC_lexicon_tags_38 },
    { 0xfd3e8e34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bfe8e36, SUC_lexicon_tags_25 },
    { 0xd3de8e36, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3de8e3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfe8e40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbe8e46, SUC_lexicon_tags_28 },
    { 0xccbe8e47, SUC_lexicon_tags_1 },
    { 0xd07e8e48, SUC_lexicon_tags_1 },
    { 0x5bbe8e48, SUC_lexicon_tags_25 },
    { 0xd1e8e49, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ade8e4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe8e4f, SUC_lexicon_tags_17 },
    { 0x571e8e50, SUC_lexicon_tags_4 },
    { 0xe8fe8e51, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901e8e55, SUC_lexicon_tags_38 },
    { 0xee9e8e55, SUC_lexicon_tags_12 },
    { 0x977e8e57, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xdc1e8e59, SUC_lexicon_tags_10 },
    { 0xe9fe8e5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5e8e5d, SUC_lexicon_tags_21 },
    { 0xd83e8e5e, SUC_lexicon_tags_33 },
    { 0xb5e8e5f, SUC_lexicon_tags_12 },
    { 0xe73e8e60, SUC_lexicon_tags_1 },
    { 0xaefe8e5f, SUC_lexicon_tags_25 },
    { 0xfe3e8e61, SUC_lexicon_tags_19 },
    { 0x457e8e60, SUC_lexicon_tags_95 },
    { 0xa23e8e62, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5e8e69, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51be8e6f, SUC_lexicon_tags_24 },
    { 0x9a9e8e6f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9e8e73, SUC_lexicon_tags_2 },
    { 0xb1fe8e74, SUC_lexicon_tags_12 },
    { 0x935e8e75, SUC_lexicon_tags_32 },
    { 0x92fe8e76, SUC_lexicon_tags_25 },
    { 0x3e1e8e73, SUC_lexicon_tags_558 },
    { 0, NULL },
    { 0x8d3e8e79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x67be8e7b, SUC_lexicon_tags_1 },
    { 0x98be8e7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677e8e84, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6f5e8e86, SUC_lexicon_tags_14 },
    { 0xcf1e8e87, SUC_lexicon_tags_12 },
    { 0x461e8e88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9e8e8b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11e8e91, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa67e8e93, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xbfde8e95, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fe8e9c, SUC_lexicon_tags_106 },
    { 0x195e8e9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17e8ea1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9be8ea3, SUC_lexicon_tags_46 },
    { 0x75fe8ea3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb5e8ea6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x72de8ea9, SUC_lexicon_tags_77 },
    { 0xfbbe8eaa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x34be8ead, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbe8eb0, SUC_lexicon_tags_15 },
    { 0xcade8eb1, SUC_lexicon_tags_113 },
    { 0x2afe8eb1, SUC_lexicon_tags_7 },
    { 0x733e8eb1, SUC_lexicon_tags_23 },
    { 0xa95e8eb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe9de8eb6, SUC_lexicon_tags_38 },
    { 0x58fe8eb6, SUC_lexicon_tags_9 },
    { 0xb9be8eb6, SUC_lexicon_tags_27 },
    { 0x7fde8eb6, SUC_lexicon_tags_37 },
    { 0x6f1e8eba, SUC_lexicon_tags_106 },
    { 0x985e8ebb, SUC_lexicon_tags_17 },
    { 0x613e8ebc, SUC_lexicon_tags_27 },
    { 0xb15e8ebd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd59e8ebf, SUC_lexicon_tags_17 },
    { 0xe13e8ec0, SUC_lexicon_tags_2 },
    { 0x31de8ebf, SUC_lexicon_tags_6 },
    { 0xe51e8ec2, SUC_lexicon_tags_7 },
    { 0x56fe8ec3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9be8eca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x81e8ecc, SUC_lexicon_tags_12 },
    { 0x437e8ecd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09e8ed3, SUC_lexicon_tags_25 },
    { 0x9a9e8ed3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x681e8ed6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5e8ede, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa29e8ee0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9e8ee3, SUC_lexicon_tags_25 },
    { 0x683e8ee3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfc9e8ee6, SUC_lexicon_tags_17 },
    { 0x647e8ee7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25de8eeb, SUC_lexicon_tags_34 },
    { 0x933e8eec, SUC_lexicon_tags_11 },
    { 0xb9be8eed, SUC_lexicon_tags_37 },
    { 0xa1fe8eed, SUC_lexicon_tags_172 },
    { 0x55e8eef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x759e8ef1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cde8ef4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9e8ef9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7e8efc, SUC_lexicon_tags_31 },
    { 0xb79e8efc, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x949e8eff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779e8f04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbe8f07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe17e8f09, SUC_lexicon_tags_26 },
    { 0x227e8f0a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fe8f0d, SUC_lexicon_tags_7 },
    { 0x6bbe8f0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x753e8f11, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91be8f18, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd31e8f1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77e8f1e, SUC_lexicon_tags_29 },
    { 0x9de8f1f, SUC_lexicon_tags_45 },
    { 0x53fe8f20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x129e8f22, SUC_lexicon_tags_4 },
    { 0x3a9e8f22, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x729e8f25, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5e8f2a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe8f2d, SUC_lexicon_tags_32 },
    { 0x8cbe8f2e, SUC_lexicon_tags_11 },
    { 0xc9e8f2f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9e8f32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe93e8f34, SUC_lexicon_tags_38 },
    { 0xd71e8f34, SUC_lexicon_tags_38 },
    { 0x3d1e8f34, SUC_lexicon_tags_33 },
    { 0x82fe8f35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5e8f3b, SUC_lexicon_tags_7 },
    { 0xd23e8f3b, SUC_lexicon_tags_63 },
    { 0x583e8f3b, SUC_lexicon_tags_11 },
    { 0xd37e8f3d, SUC_lexicon_tags_33 },
    { 0x8e9e8f3f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x451e8f41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abe8f45, SUC_lexicon_tags_17 },
    { 0x88de8f46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfe8f4c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x563e8f4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59e8f55, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x93de8f58, SUC_lexicon_tags_11 },
    { 0x66fe8f59, SUC_lexicon_tags_26 },
    { 0xa37e8f59, SUC_lexicon_tags_56 },
    { 0xe01e8f5a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x245e8f5d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebe8f60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x611e8f62, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x841e8f64, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73e8f6a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x221e8f6c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7e8f74, SUC_lexicon_tags_1258 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9e8f7a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x625e8f7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x67fe8f7e, SUC_lexicon_tags_46 },
    { 0x87de8f7f, SUC_lexicon_tags_23 },
    { 0x8a9e8f7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571e8f85, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5e8f8b, SUC_lexicon_tags_14 },
    { 0xff7e8f8c, SUC_lexicon_tags_17 },
    { 0xbc7e8f8c, SUC_lexicon_tags_11 },
    { 0x995e8f8e, SUC_lexicon_tags_57 },
    { 0xd13e8f8e, SUC_lexicon_tags_1 },
    { 0xecde8f8b, SUC_lexicon_tags_16 },
    { 0xb7be8f91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xaf9e8f93, SUC_lexicon_tags_104 },
    { 0xc05e8f94, SUC_lexicon_tags_12 },
    { 0x9d3e8f95, SUC_lexicon_tags_30 },
    { 0x893e8f93, SUC_lexicon_tags_63 },
    { 0x101e8f97, SUC_lexicon_tags_15 },
    { 0xcf9e8f98, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75e8f9b, SUC_lexicon_tags_15 },
    { 0x579e8f9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x681e8f9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bde8fa2, SUC_lexicon_tags_31 },
    { 0x1abe8fa3, SUC_lexicon_tags_39 },
    { 0x1e7e8fa3, SUC_lexicon_tags_14 },
    { 0x18de8fa5, SUC_lexicon_tags_17 },
    { 0xddbe8fa6, SUC_lexicon_tags_11 },
    { 0xc31e8fa7, SUC_lexicon_tags_47 },
    { 0xa89e8fa8, SUC_lexicon_tags_11 },
    { 0x75be8fa4, SUC_lexicon_tags_43 },
    { 0xf59e8faa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd65e8fac, SUC_lexicon_tags_27 },
    { 0x851e8fad, SUC_lexicon_tags_7 },
    { 0xe19e8fae, SUC_lexicon_tags_17 },
    { 0x9dfe8fae, SUC_lexicon_tags_12 },
    { 0xe1fe8fb0, SUC_lexicon_tags_26 },
    { 0xebde8fb1, SUC_lexicon_tags_45 },
    { 0x839e8fb2, SUC_lexicon_tags_6 },
    { 0x489e8fb3, SUC_lexicon_tags_37 },
    { 0xf69e8fb3, SUC_lexicon_tags_6 },
    { 0x5e1e8fb2, SUC_lexicon_tags_15 },
    { 0x9fe8fb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3e8fb9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fe8fbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1e8fc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x879e8fc6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39be8fca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x265e8fcc, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x217e8fce, SUC_lexicon_tags_84 },
    { 0x7efe8fcf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221e8fd3, SUC_lexicon_tags_79 },
    { 0x683e8fd3, SUC_lexicon_tags_31 },
    { 0x2f5e8fd4, SUC_lexicon_tags_4 },
    { 0xe07e8fd4, SUC_lexicon_tags_1 },
    { 0xa9e8fd7, SUC_lexicon_tags_47 },
    { 0xc5fe8fd8, SUC_lexicon_tags_45 },
    { 0x179e8fd6, SUC_lexicon_tags_17 },
    { 0xb7e8fd7, SUC_lexicon_tags_1 },
    { 0x97fe8fd9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x433e8fde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21e8fe7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x785e8fea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe45e8fed, SUC_lexicon_tags_28 },
    { 0xa25e8fed, SUC_lexicon_tags_25 },
    { 0xe45e8fef, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72be8ff4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fe8ff7, SUC_lexicon_tags_11 },
    { 0x6a1e8ff8, SUC_lexicon_tags_25 },
    { 0x17de8ff7, SUC_lexicon_tags_71 },
    { 0xad9e8ffa, SUC_lexicon_tags_66 },
    { 0xbc5e8ffa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3e9002, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x453e9004, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1de900d, SUC_lexicon_tags_9 },
    { 0x1b9e900e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x497e9011, SUC_lexicon_tags_23 },
    { 0xf1e9012, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3e9016, SUC_lexicon_tags_51 },
    { 0xb01e9016, SUC_lexicon_tags_14 },
    { 0x30be9018, SUC_lexicon_tags_17 },
    { 0x9f9e9018, SUC_lexicon_tags_38 },
    { 0xf3e901a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd7de901c, SUC_lexicon_tags_17 },
    { 0x775e901d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x789e9020, SUC_lexicon_tags_12 },
    { 0x6cde9020, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5e9025, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x339e9027, SUC_lexicon_tags_23 },
    { 0xbd3e9028, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1e902b, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fe902f, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fe9036, SUC_lexicon_tags_17 },
    { 0x415e9037, SUC_lexicon_tags_25 },
    { 0x853e9038, SUC_lexicon_tags_11 },
    { 0x7ffe9037, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43e903d, SUC_lexicon_tags_15 },
    { 0x651e903e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7e9043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717e9047, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13e904a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63e9059, SUC_lexicon_tags_88 },
    { 0xf2fe9059, SUC_lexicon_tags_5 },
    { 0x381e905b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5e905e, SUC_lexicon_tags_43 },
    { 0xb7de905f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbe906a, SUC_lexicon_tags_17 },
    { 0xd19e906a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x517e906e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x98de9071, SUC_lexicon_tags_11 },
    { 0xc79e9072, SUC_lexicon_tags_17 },
    { 0x1be9073, SUC_lexicon_tags_15 },
    { 0xe19e9071, SUC_lexicon_tags_12 },
    { 0x353e9073, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x901e9077, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa07e9079, SUC_lexicon_tags_61 },
    { 0x2f7e907a, SUC_lexicon_tags_25 },
    { 0x82de907b, SUC_lexicon_tags_28 },
    { 0xdb3e907c, SUC_lexicon_tags_0 },
    { 0xd9de907d, SUC_lexicon_tags_15 },
    { 0xb8be907c, SUC_lexicon_tags_1 },
    { 0x79de907f, SUC_lexicon_tags_37 },
    { 0xe07e907f, SUC_lexicon_tags_38 },
    { 0x401e907f, SUC_lexicon_tags_45 },
    { 0xa93e907a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x977e9084, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0de908a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7cfe908c, SUC_lexicon_tags_17 },
    { 0x393e908c, SUC_lexicon_tags_28 },
    { 0x221e908e, SUC_lexicon_tags_11 },
    { 0x731e908c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xffe9091, SUC_lexicon_tags_15 },
    { 0x9b1e9091, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62fe9097, SUC_lexicon_tags_8 },
    { 0xec9e9098, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb17e909a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe87e909d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x6bfe909f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3e90a3, SUC_lexicon_tags_17 },
    { 0x995e90a4, SUC_lexicon_tags_12 },
    { 0x81de90a4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbc5e90a7, SUC_lexicon_tags_37 },
    { 0xc23e90a8, SUC_lexicon_tags_77 },
    { 0x4fde90a9, SUC_lexicon_tags_2 },
    { 0xd07e90aa, SUC_lexicon_tags_37 },
    { 0x2e1e90a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445e90af, SUC_lexicon_tags_39 },
    { 0x5d5e90b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xddde90b2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe61e90b4, SUC_lexicon_tags_43 },
    { 0xb8de90b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383e90c1, SUC_lexicon_tags_10 },
    { 0xca7e90c2, SUC_lexicon_tags_17 },
    { 0xfa9e90c1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7e90c6, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45e90cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x279e90d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a5e90d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0de90d8, SUC_lexicon_tags_11 },
    { 0x9b3e90d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81e90dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fde90e3, SUC_lexicon_tags_55 },
    { 0x1e1e90e3, SUC_lexicon_tags_2 },
    { 0x725e90e5, SUC_lexicon_tags_10 },
    { 0x10be90e6, SUC_lexicon_tags_34 },
    { 0x16be90e7, SUC_lexicon_tags_0 },
    { 0xab9e90e3, SUC_lexicon_tags_12 },
    { 0xa6de90e9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3e90f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275e90f6, SUC_lexicon_tags_38 },
    { 0xa51e90f7, SUC_lexicon_tags_2 },
    { 0xf5e90f8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd69e90fa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x307e90fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadbe9100, SUC_lexicon_tags_25 },
    { 0x67fe9100, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811e9106, SUC_lexicon_tags_43 },
    { 0xe19e9107, SUC_lexicon_tags_37 },
    { 0xe4fe9108, SUC_lexicon_tags_68 },
    { 0xf9de9109, SUC_lexicon_tags_25 },
    { 0x8c7e9107, SUC_lexicon_tags_207 },
    { 0x267e9108, SUC_lexicon_tags_39 },
    { 0xc6be910c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfe9111, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56be9118, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafde911a, SUC_lexicon_tags_14 },
    { 0x51de911b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x763e911d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbffe9120, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9c7e9122, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x269e9125, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x507e9127, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969e912b, SUC_lexicon_tags_38 },
    { 0x9b9e912c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5e912f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x31fe9131, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x43e9133, SUC_lexicon_tags_45 },
    { 0x951e9134, SUC_lexicon_tags_0 },
    { 0x281e9134, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7e913a, SUC_lexicon_tags_43 },
    { 0xcbbe913a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x687e913e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x213e9140, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fe9143, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x671e9145, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d1e9147, SUC_lexicon_tags_38 },
    { 0xe03e9147, SUC_lexicon_tags_47 },
    { 0xd31e9149, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995e9150, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x71fe9152, SUC_lexicon_tags_106 },
    { 0xfafe9152, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dde9157, SUC_lexicon_tags_61 },
    { 0x4dbe9158, SUC_lexicon_tags_25 },
    { 0xbfde9158, SUC_lexicon_tags_11 },
    { 0xcd7e9159, SUC_lexicon_tags_10 },
    { 0x17e915b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487e915f, SUC_lexicon_tags_0 },
    { 0x4d9e9160, SUC_lexicon_tags_37 },
    { 0xf85e9160, SUC_lexicon_tags_25 },
    { 0xf49e9161, SUC_lexicon_tags_7 },
    { 0x96de9163, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x661e9165, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e9168, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf1de916a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9e916f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9172, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12de9174, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacfe9176, SUC_lexicon_tags_13 },
    { 0xd19e9176, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfade9179, SUC_lexicon_tags_6 },
    { 0xf9de9179, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83be917f, SUC_lexicon_tags_8 },
    { 0xb7de9180, SUC_lexicon_tags_8 },
    { 0x5e9e9181, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21e9188, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb35e918a, SUC_lexicon_tags_14 },
    { 0x2dbe918b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x967e918e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07e9197, SUC_lexicon_tags_38 },
    { 0xb91e9198, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8efe919c, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x71be919f, SUC_lexicon_tags_11 },
    { 0x8e1e919f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6a1e91a2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fe91a7, SUC_lexicon_tags_14 },
    { 0x475e91a8, SUC_lexicon_tags_47 },
    { 0x675e91a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44fe91af, SUC_lexicon_tags_1 },
    { 0x129e91af, SUC_lexicon_tags_26 },
    { 0x2d1e91b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45e91b8, SUC_lexicon_tags_23 },
    { 0x587e91b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x303e91bb, SUC_lexicon_tags_0 },
    { 0x7a3e91bc, SUC_lexicon_tags_8 },
    { 0x8d9e91bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcf9e91bf, SUC_lexicon_tags_12 },
    { 0xecde91c0, SUC_lexicon_tags_0 },
    { 0xca7e91c1, SUC_lexicon_tags_37 },
    { 0x9c5e91c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xeb5e91c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3e91c8, SUC_lexicon_tags_11 },
    { 0x19be91c9, SUC_lexicon_tags_55 },
    { 0xade91ca, SUC_lexicon_tags_48 },
    { 0xf59e91ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x965e91cd, SUC_lexicon_tags_9 },
    { 0xc63e91ce, SUC_lexicon_tags_38 },
    { 0x193e91ce, SUC_lexicon_tags_37 },
    { 0x79de91ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x77be91d3, SUC_lexicon_tags_0 },
    { 0xfbde91d3, SUC_lexicon_tags_23 },
    { 0x635e91d5, SUC_lexicon_tags_11 },
    { 0xf65e91d3, SUC_lexicon_tags_6 },
    { 0x4d3e91d4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3e91dc, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x7dde91de, SUC_lexicon_tags_23 },
    { 0xdc5e91df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37e91e3, SUC_lexicon_tags_19 },
    { 0x377e91e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x975e91e6, SUC_lexicon_tags_12 },
    { 0xe3e91e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd03e91e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdbe91eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x743e91ed, SUC_lexicon_tags_12 },
    { 0x111e91ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43be91f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6abe91f4, SUC_lexicon_tags_49 },
    { 0x375e91f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65e9200, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x399e9202, SUC_lexicon_tags_37 },
    { 0x1d1e9203, SUC_lexicon_tags_37 },
    { 0x4c7e9204, SUC_lexicon_tags_17 },
    { 0xd87e9204, SUC_lexicon_tags_82 },
    { 0xc5de9206, SUC_lexicon_tags_37 },
    { 0xd8fe9206, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e9e9209, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x951e920b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x365e920e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x71de9210, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x75be9213, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39e921c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5e9222, SUC_lexicon_tags_60 },
    { 0x783e9223, SUC_lexicon_tags_11 },
    { 0x7b5e9222, SUC_lexicon_tags_55 },
    { 0xf89e9225, SUC_lexicon_tags_15 },
    { 0xc7e9225, SUC_lexicon_tags_0 },
    { 0xe35e9226, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43e922d, SUC_lexicon_tags_30 },
    { 0x1e1e922e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27fe9232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539e9236, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7a9e9238, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa6fe923a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2be923d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5e9242, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x985e9244, SUC_lexicon_tags_30 },
    { 0x845e9245, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1c1e9247, SUC_lexicon_tags_12 },
    { 0x455e9247, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbde924b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9e924e, SUC_lexicon_tags_7 },
    { 0xfd7e924e, SUC_lexicon_tags_43 },
    { 0x7f9e9250, SUC_lexicon_tags_43 },
    { 0x589e9251, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xe95e9253, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9e9257, SUC_lexicon_tags_7 },
    { 0xa3de9257, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1e925b, SUC_lexicon_tags_37 },
    { 0xfafe925c, SUC_lexicon_tags_11 },
    { 0xc09e925c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa09e925f, SUC_lexicon_tags_7 },
    { 0x705e9260, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3e9263, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273e9267, SUC_lexicon_tags_160 },
    { 0x597e9267, SUC_lexicon_tags_0 },
    { 0xca3e9269, SUC_lexicon_tags_21 },
    { 0x763e9268, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83e926e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4c1e9270, SUC_lexicon_tags_15 },
    { 0xe35e9271, SUC_lexicon_tags_4 },
    { 0x5d3e9272, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9e9277, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x27be927a, SUC_lexicon_tags_14 },
    { 0xeebe927b, SUC_lexicon_tags_9 },
    { 0x205e927a, SUC_lexicon_tags_17 },
    { 0xdefe927a, SUC_lexicon_tags_56 },
    { 0xde1e927c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x431e9280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90fe9282, SUC_lexicon_tags_7 },
    { 0x9d7e9283, SUC_lexicon_tags_25 },
    { 0x583e9283, SUC_lexicon_tags_17 },
    { 0x85e9282, SUC_lexicon_tags_17 },
    { 0x81fe9286, SUC_lexicon_tags_56 },
    { 0xaa3e9284, SUC_lexicon_tags_2 },
    { 0xccde9288, SUC_lexicon_tags_17 },
    { 0x6ede9289, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeb7e928b, SUC_lexicon_tags_30 },
    { 0xc1de928b, SUC_lexicon_tags_17 },
    { 0x653e928d, SUC_lexicon_tags_60 },
    { 0x757e928e, SUC_lexicon_tags_17 },
    { 0xb09e928e, SUC_lexicon_tags_37 },
    { 0x9d9e9290, SUC_lexicon_tags_7 },
    { 0x5a1e928f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19e9296, SUC_lexicon_tags_37 },
    { 0xbde9297, SUC_lexicon_tags_53 },
    { 0xdfe9298, SUC_lexicon_tags_11 },
    { 0x489e9299, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1e929f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb1e92a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5e92a5, SUC_lexicon_tags_26 },
    { 0xeebe92a6, SUC_lexicon_tags_15 },
    { 0x6be92a6, SUC_lexicon_tags_38 },
    { 0x227e92a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x381e92aa, SUC_lexicon_tags_48 },
    { 0x5c7e92ab, SUC_lexicon_tags_32 },
    { 0xc1fe92aa, SUC_lexicon_tags_37 },
    { 0xf93e92ad, SUC_lexicon_tags_22 },
    { 0xec9e92ad, SUC_lexicon_tags_15 },
    { 0xed3e92af, SUC_lexicon_tags_17 },
    { 0x2a1e92ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x42be92b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x189e92b5, SUC_lexicon_tags_19 },
    { 0xabe92b6, SUC_lexicon_tags_7 },
    { 0x4fbe92b7, SUC_lexicon_tags_45 },
    { 0xd03e92b6, SUC_lexicon_tags_16 },
    { 0x887e92b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7e92bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdbe92be, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x24be92c0, SUC_lexicon_tags_10 },
    { 0x3e5e92c0, SUC_lexicon_tags_27 },
    { 0x253e92c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1e92c6, SUC_lexicon_tags_1 },
    { 0xb1e92c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35e92cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871e92d4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x931e92d6, SUC_lexicon_tags_8 },
    { 0xdb9e92d6, SUC_lexicon_tags_3 },
    { 0xdafe92d8, SUC_lexicon_tags_25 },
    { 0x237e92d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1e92dd, SUC_lexicon_tags_17 },
    { 0xa3e92de, SUC_lexicon_tags_14 },
    { 0xf9e92dd, SUC_lexicon_tags_126 },
    { 0x35e92e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71e92e5, SUC_lexicon_tags_17 },
    { 0xe6be92e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfc7e92e8, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ade92eb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d9e92ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83e92f2, SUC_lexicon_tags_25 },
    { 0xc2be92f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651e92f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x419e92f9, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x9bfe92fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe39e92fd, SUC_lexicon_tags_8 },
    { 0x71de92fd, SUC_lexicon_tags_27 },
    { 0x4c5e92fd, SUC_lexicon_tags_11 },
    { 0xcd1e9300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x77de9302, SUC_lexicon_tags_55 },
    { 0xca1e9302, SUC_lexicon_tags_25 },
    { 0x159e9302, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbf9e9306, SUC_lexicon_tags_17 },
    { 0x563e9306, SUC_lexicon_tags_66 },
    { 0xc03e9306, SUC_lexicon_tags_7 },
    { 0x655e9309, SUC_lexicon_tags_53 },
    { 0x87be930a, SUC_lexicon_tags_17 },
    { 0x9b9e930a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x217e930d, SUC_lexicon_tags_19 },
    { 0xa2fe930e, SUC_lexicon_tags_44 },
    { 0x55be930d, SUC_lexicon_tags_27 },
    { 0xc15e930f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xefe9313, SUC_lexicon_tags_3 },
    { 0x1efe9314, SUC_lexicon_tags_37 },
    { 0xa6be9314, SUC_lexicon_tags_1 },
    { 0x7d9e9315, SUC_lexicon_tags_0 },
    { 0xa1de9317, SUC_lexicon_tags_25 },
    { 0xf41e9318, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x383e931a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x993e931c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95e931e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1e9324, SUC_lexicon_tags_8 },
    { 0xda3e9325, SUC_lexicon_tags_84 },
    { 0x68de9326, SUC_lexicon_tags_28 },
    { 0x5ede9326, SUC_lexicon_tags_25 },
    { 0x29e9328, SUC_lexicon_tags_17 },
    { 0x4dbe9329, SUC_lexicon_tags_0 },
    { 0xe8be9329, SUC_lexicon_tags_2 },
    { 0x2b5e932b, SUC_lexicon_tags_9 },
    { 0x435e932a, SUC_lexicon_tags_37 },
    { 0x139e932d, SUC_lexicon_tags_43 },
    { 0x56fe932e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf5e9330, SUC_lexicon_tags_15 },
    { 0xd2fe9331, SUC_lexicon_tags_5 },
    { 0x45fe9331, SUC_lexicon_tags_87 },
    { 0xfbfe9333, SUC_lexicon_tags_30 },
    { 0xd99e9333, SUC_lexicon_tags_11 },
    { 0xa67e9332, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69e9339, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9be933f, SUC_lexicon_tags_37 },
    { 0xaade9340, SUC_lexicon_tags_11 },
    { 0x64fe9341, SUC_lexicon_tags_14 },
    { 0xc2fe933f, SUC_lexicon_tags_39 },
    { 0x42de9343, SUC_lexicon_tags_19 },
    { 0x87be9344, SUC_lexicon_tags_108 },
    { 0x5e9e9345, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555e9349, SUC_lexicon_tags_31 },
    { 0xaa5e9349, SUC_lexicon_tags_45 },
    { 0x7d1e934b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x431e934d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7e9351, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723e9355, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299e935b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x89e935e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x355e9361, SUC_lexicon_tags_82 },
    { 0xd95e9362, SUC_lexicon_tags_2 },
    { 0x887e9363, SUC_lexicon_tags_114 },
    { 0x6be9362, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9e936b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fe936e, SUC_lexicon_tags_46 },
    { 0x8c5e936f, SUC_lexicon_tags_17 },
    { 0xcade9370, SUC_lexicon_tags_37 },
    { 0x2c3e9371, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115e9375, SUC_lexicon_tags_82 },
    { 0x83fe9376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5e9379, SUC_lexicon_tags_27 },
    { 0x947e937a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdffe937c, SUC_lexicon_tags_17 },
    { 0xcfe937d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x653e9380, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x187e9382, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb45e9384, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bde9387, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7e938c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5e9395, SUC_lexicon_tags_968 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0be9398, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7e93a8, SUC_lexicon_tags_7 },
    { 0x7cde93a8, SUC_lexicon_tags_11 },
    { 0x4dde93aa, SUC_lexicon_tags_33 },
    { 0xefe93aa, SUC_lexicon_tags_4 },
    { 0x9a3e93a8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc0de93ae, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23de93b2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x22be93b5, SUC_lexicon_tags_29 },
    { 0xa4de93b6, SUC_lexicon_tags_12 },
    { 0xe5e93b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd6de93b9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee3e93bd, SUC_lexicon_tags_5 },
    { 0x6b7e93be, SUC_lexicon_tags_37 },
    { 0xe8fe93bf, SUC_lexicon_tags_17 },
    { 0xb5e93c0, SUC_lexicon_tags_2 },
    { 0x1b1e93c1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a1e93c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567e93c8, SUC_lexicon_tags_36 },
    { 0x8b9e93c9, SUC_lexicon_tags_39 },
    { 0x9dde93ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xac9e93cc, SUC_lexicon_tags_88 },
    { 0x9a7e93cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x299e93cf, SUC_lexicon_tags_6 },
    { 0xa37e93d0, SUC_lexicon_tags_39 },
    { 0x207e93d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5e93d7, SUC_lexicon_tags_25 },
    { 0x6c1e93d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25e93dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb37e93de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851e93e2, SUC_lexicon_tags_7 },
    { 0xef3e93e2, SUC_lexicon_tags_28 },
    { 0x10be93e4, SUC_lexicon_tags_4 },
    { 0x5f7e93e4, SUC_lexicon_tags_19 },
    { 0xc9e93e4, SUC_lexicon_tags_23 },
    { 0xba3e93e6, SUC_lexicon_tags_30 },
    { 0x279e93e8, SUC_lexicon_tags_37 },
    { 0xac5e93e2, SUC_lexicon_tags_25 },
    { 0xfc5e93e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdde93ee, SUC_lexicon_tags_2 },
    { 0x639e93ee, SUC_lexicon_tags_25 },
    { 0x8efe93f0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6ede93f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3e93f6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef5e93fc, SUC_lexicon_tags_235 },
    { 0xad9e93fd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xee5e93ff, SUC_lexicon_tags_0 },
    { 0xb53e9400, SUC_lexicon_tags_16 },
    { 0xafde9401, SUC_lexicon_tags_23 },
    { 0x89de93ff, SUC_lexicon_tags_47 },
    { 0x2c9e9403, SUC_lexicon_tags_9 },
    { 0xb79e9404, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9e9408, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7e940c, SUC_lexicon_tags_25 },
    { 0x639e940d, SUC_lexicon_tags_1 },
    { 0x2ffe940c, SUC_lexicon_tags_2 },
    { 0xecde940f, SUC_lexicon_tags_38 },
    { 0xfd1e940d, SUC_lexicon_tags_37 },
    { 0xbdbe9411, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1e9414, SUC_lexicon_tags_47 },
    { 0x3cbe9415, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5efe9417, SUC_lexicon_tags_17 },
    { 0x47be9418, SUC_lexicon_tags_38 },
    { 0x7e5e9419, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe941c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x397e941e, SUC_lexicon_tags_47 },
    { 0x7b9e941f, SUC_lexicon_tags_55 },
    { 0x251e9420, SUC_lexicon_tags_25 },
    { 0xa01e9421, SUC_lexicon_tags_0 },
    { 0x999e941f, SUC_lexicon_tags_21 },
    { 0xdd3e9421, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1e9427, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71e942c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x29fe942f, SUC_lexicon_tags_1 },
    { 0x65de942f, SUC_lexicon_tags_17 },
    { 0x8a7e9430, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1e9438, SUC_lexicon_tags_7 },
    { 0xb3de9438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xed1e943b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbe9440, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55e9444, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x717e9446, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7e9449, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7dde944b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa99e944d, SUC_lexicon_tags_31 },
    { 0xc25e944e, SUC_lexicon_tags_9 },
    { 0x3bde944d, SUC_lexicon_tags_0 },
    { 0x59fe944d, SUC_lexicon_tags_37 },
    { 0x3b9e944d, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x7b5e9453, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8de9456, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99e945b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa75e945d, SUC_lexicon_tags_38 },
    { 0x9b3e945e, SUC_lexicon_tags_63 },
    { 0xcede945f, SUC_lexicon_tags_10 },
    { 0x7f5e9460, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x79e9462, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61e9467, SUC_lexicon_tags_11 },
    { 0xf9be9468, SUC_lexicon_tags_60 },
    { 0xca3e9469, SUC_lexicon_tags_25 },
    { 0xf4fe946a, SUC_lexicon_tags_25 },
    { 0xcd3e946b, SUC_lexicon_tags_82 },
    { 0x697e946a, SUC_lexicon_tags_2 },
    { 0x3ade946d, SUC_lexicon_tags_1 },
    { 0xa83e946d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1e9472, SUC_lexicon_tags_38 },
    { 0x863e9473, SUC_lexicon_tags_15 },
    { 0x8f7e9474, SUC_lexicon_tags_8 },
    { 0x251e9475, SUC_lexicon_tags_27 },
    { 0x3de9475, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xecbe9479, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd5e947b, SUC_lexicon_tags_23 },
    { 0x76be947c, SUC_lexicon_tags_23 },
    { 0xef9e947d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x887e9480, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215e9489, SUC_lexicon_tags_37 },
    { 0xb37e948a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x443e948c, SUC_lexicon_tags_65 },
    { 0xb87e948d, SUC_lexicon_tags_25 },
    { 0xd3fe948d, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbe9493, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3de9496, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275e949a, SUC_lexicon_tags_33 },
    { 0xa63e949b, SUC_lexicon_tags_9 },
    { 0xeafe949a, SUC_lexicon_tags_60 },
    { 0x885e949b, SUC_lexicon_tags_17 },
    { 0xb01e949e, SUC_lexicon_tags_17 },
    { 0x7ffe949b, SUC_lexicon_tags_7 },
    { 0x663e949c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf45e94a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe11e94a4, SUC_lexicon_tags_7 },
    { 0x525e94a4, SUC_lexicon_tags_15 },
    { 0xfdbe94a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7be94a8, SUC_lexicon_tags_33 },
    { 0xe8be94a9, SUC_lexicon_tags_31 },
    { 0x64be94a8, SUC_lexicon_tags_47 },
    { 0x913e94ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x67de94ae, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0x8c5e94b0, SUC_lexicon_tags_33 },
    { 0xffe94b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9de94b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfe3e94b7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727e94bb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1e94bf, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25e94c1, SUC_lexicon_tags_7 },
    { 0x4a7e94c2, SUC_lexicon_tags_2 },
    { 0xb77e94c2, SUC_lexicon_tags_29 },
    { 0xc5be94c4, SUC_lexicon_tags_7 },
    { 0x1c9e94c5, SUC_lexicon_tags_33 },
    { 0x7cde94c6, SUC_lexicon_tags_47 },
    { 0x6ade94c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa5de94c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221e94cd, SUC_lexicon_tags_47 },
    { 0xe19e94ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x257e94d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad9e94d3, SUC_lexicon_tags_11 },
    { 0x34de94d4, SUC_lexicon_tags_15 },
    { 0x74de94d4, SUC_lexicon_tags_1 },
    { 0x669e94d6, SUC_lexicon_tags_37 },
    { 0xfffe94d6, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22de94dc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfe94e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7e94e6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbe94ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x529e94ed, SUC_lexicon_tags_25 },
    { 0x6f7e94ed, SUC_lexicon_tags_13 },
    { 0x6b5e94ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721e94f5, SUC_lexicon_tags_14 },
    { 0x54fe94f5, SUC_lexicon_tags_26 },
    { 0xa59e94f7, SUC_lexicon_tags_59 },
    { 0x11be94f8, SUC_lexicon_tags_12 },
    { 0x1b7e94f5, SUC_lexicon_tags_15 },
    { 0x25be94fa, SUC_lexicon_tags_37 },
    { 0x69e94fb, SUC_lexicon_tags_2 },
    { 0x5d7e94fb, SUC_lexicon_tags_2 },
    { 0x365e94fc, SUC_lexicon_tags_27 },
    { 0x5d7e94fe, SUC_lexicon_tags_25 },
    { 0xb1be94fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x31e9501, SUC_lexicon_tags_27 },
    { 0xb89e9502, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x81e9504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x469e9506, SUC_lexicon_tags_31 },
    { 0xe8fe9507, SUC_lexicon_tags_11 },
    { 0x569e9506, SUC_lexicon_tags_7 },
    { 0x58fe9506, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffe950c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3e9513, SUC_lexicon_tags_2 },
    { 0xb37e9514, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39e9517, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec9e9519, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x92be951b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x781e951d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92fe9524, SUC_lexicon_tags_32 },
    { 0x90be9524, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85e9528, SUC_lexicon_tags_11 },
    { 0x655e9529, SUC_lexicon_tags_86 },
    { 0xcebe952a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fde952d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1e9535, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1e9538, SUC_lexicon_tags_25 },
    { 0x7c1e9538, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x963e953b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2c5e953d, SUC_lexicon_tags_39 },
    { 0xfdfe953e, SUC_lexicon_tags_39 },
    { 0x733e953f, SUC_lexicon_tags_2 },
    { 0x229e953f, SUC_lexicon_tags_7 },
    { 0x6a5e9541, SUC_lexicon_tags_11 },
    { 0x861e953f, SUC_lexicon_tags_7 },
    { 0xe0de953f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x771e9545, SUC_lexicon_tags_12 },
    { 0x64be9545, SUC_lexicon_tags_12 },
    { 0x323e9546, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7e954a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9e9554, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279e9559, SUC_lexicon_tags_7 },
    { 0xe7be955a, SUC_lexicon_tags_6 },
    { 0x19e9559, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbede9560, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fde956d, SUC_lexicon_tags_0 },
    { 0x913e956e, SUC_lexicon_tags_27 },
    { 0x971e956f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x78fe9572, SUC_lexicon_tags_47 },
    { 0xb11e9572, SUC_lexicon_tags_0 },
    { 0x27be9574, SUC_lexicon_tags_1 },
    { 0x9b3e9575, SUC_lexicon_tags_9 },
    { 0x551e9576, SUC_lexicon_tags_37 },
    { 0x239e9577, SUC_lexicon_tags_12 },
    { 0x8ade9578, SUC_lexicon_tags_13 },
    { 0x873e9578, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x289e957b, SUC_lexicon_tags_31 },
    { 0x5ebe957b, SUC_lexicon_tags_15 },
    { 0x2d7e957d, SUC_lexicon_tags_84 },
    { 0x655e957c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x76be9581, SUC_lexicon_tags_17 },
    { 0x1bbe9581, SUC_lexicon_tags_37 },
    { 0xddde9583, SUC_lexicon_tags_60 },
    { 0x649e9582, SUC_lexicon_tags_32 },
    { 0x6b9e9585, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fe958c, SUC_lexicon_tags_63 },
    { 0x17de958c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421e9594, SUC_lexicon_tags_28 },
    { 0xae5e9595, SUC_lexicon_tags_43 },
    { 0x49de9596, SUC_lexicon_tags_2 },
    { 0x43fe9596, SUC_lexicon_tags_42 },
    { 0x961e9598, SUC_lexicon_tags_4 },
    { 0x82fe9599, SUC_lexicon_tags_23 },
    { 0x34de9599, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffe959d, SUC_lexicon_tags_13 },
    { 0x1e9e959e, SUC_lexicon_tags_11 },
    { 0xf19e959d, SUC_lexicon_tags_12 },
    { 0x215e95a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x533e95a2, SUC_lexicon_tags_9 },
    { 0xb21e95a3, SUC_lexicon_tags_37 },
    { 0x7a1e95a3, SUC_lexicon_tags_7 },
    { 0xbdfe95a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfffe95a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9e95ab, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x181e95ae, SUC_lexicon_tags_38 },
    { 0xee7e95ae, SUC_lexicon_tags_2 },
    { 0x1de95b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1be95b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x869e95b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe95be, SUC_lexicon_tags_57 },
    { 0xaf9e95bf, SUC_lexicon_tags_49 },
    { 0x51be95bf, SUC_lexicon_tags_46 },
    { 0xd17e95c0, SUC_lexicon_tags_15 },
    { 0xd25e95c2, SUC_lexicon_tags_15 },
    { 0x87fe95c3, SUC_lexicon_tags_151 },
    { 0xa2fe95c4, SUC_lexicon_tags_2 },
    { 0xf17e95c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1dbe95c7, SUC_lexicon_tags_12 },
    { 0xca7e95c8, SUC_lexicon_tags_230 },
    { 0x473e95c7, SUC_lexicon_tags_2 },
    { 0x5bfe95ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7e95cf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1e95d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfffe95d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3e95d8, SUC_lexicon_tags_17 },
    { 0x761e95d8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5e95e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5be95e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x461e95e9, SUC_lexicon_tags_16 },
    { 0x94fe95ea, SUC_lexicon_tags_37 },
    { 0xf33e95ea, SUC_lexicon_tags_37 },
    { 0x805e95ea, SUC_lexicon_tags_38 },
    { 0x7a3e95ec, SUC_lexicon_tags_37 },
    { 0xf81e95ea, SUC_lexicon_tags_6 },
    { 0x137e95ef, SUC_lexicon_tags_21 },
    { 0xdf7e95e9, SUC_lexicon_tags_26 },
    { 0x1c9e95ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x655e95f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4be95f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1e5e95f8, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11e95fd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd31e95ff, SUC_lexicon_tags_7 },
    { 0xf2de95ff, SUC_lexicon_tags_28 },
    { 0x203e95ff, SUC_lexicon_tags_29 },
    { 0xe9be9602, SUC_lexicon_tags_60 },
    { 0x101e9603, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52de9605, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09e960b, SUC_lexicon_tags_4 },
    { 0x543e960c, SUC_lexicon_tags_70 },
    { 0x8b3e960d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3e9610, SUC_lexicon_tags_14 },
    { 0x99be9611, SUC_lexicon_tags_79 },
    { 0xbb3e9612, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x93de9615, SUC_lexicon_tags_11 },
    { 0x213e9616, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbd7e9618, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb93e961a, SUC_lexicon_tags_1 },
    { 0xe15e961b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfafe961d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fe9621, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fe9625, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbade9628, SUC_lexicon_tags_87 },
    { 0xf0de9629, SUC_lexicon_tags_26 },
    { 0x71de9628, SUC_lexicon_tags_17 },
    { 0x839e962b, SUC_lexicon_tags_9 },
    { 0x11de9628, SUC_lexicon_tags_63 },
    { 0x26de962d, SUC_lexicon_tags_60 },
    { 0xfbde962d, SUC_lexicon_tags_25 },
    { 0x341e962f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x675e9631, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebe9635, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x16de9637, SUC_lexicon_tags_1 },
    { 0x49fe9638, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf07e963a, SUC_lexicon_tags_15 },
    { 0xf37e963a, SUC_lexicon_tags_39 },
    { 0x5e9e963c, SUC_lexicon_tags_33 },
    { 0x581e963d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfe9644, SUC_lexicon_tags_17 },
    { 0xcdde9645, SUC_lexicon_tags_37 },
    { 0xf59e9645, SUC_lexicon_tags_28 },
    { 0x9f7e9647, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7afe9649, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fe9650, SUC_lexicon_tags_2 },
    { 0xc05e9651, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263e9656, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfede9658, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481e965c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23e9663, SUC_lexicon_tags_29 },
    { 0xcdde9664, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x663e9667, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef5e9669, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb9e966b, SUC_lexicon_tags_1 },
    { 0x7e3e966c, SUC_lexicon_tags_25 },
    { 0x357e966b, SUC_lexicon_tags_25 },
    { 0xe6fe966e, SUC_lexicon_tags_9 },
    { 0x5d9e966f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fe9673, SUC_lexicon_tags_46 },
    { 0x989e9673, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7e9677, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbe967a, SUC_lexicon_tags_11 },
    { 0xe93e967a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e9682, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3e9686, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7e9e9688, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x771e968a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x697e968c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1e9691, SUC_lexicon_tags_14 },
    { 0xb7be9692, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33e9696, SUC_lexicon_tags_23 },
    { 0x579e9696, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x241e9699, SUC_lexicon_tags_2 },
    { 0xc93e969a, SUC_lexicon_tags_27 },
    { 0xf39e969b, SUC_lexicon_tags_17 },
    { 0x403e9699, SUC_lexicon_tags_8 },
    { 0xffe969d, SUC_lexicon_tags_19 },
    { 0x18de969e, SUC_lexicon_tags_23 },
    { 0x3fe969d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x185e96a2, SUC_lexicon_tags_60 },
    { 0xdfde96a3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabe96a8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93de96aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xabde96ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x115e96ae, SUC_lexicon_tags_17 },
    { 0x4a9e96ae, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1e96b7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5e96bb, SUC_lexicon_tags_2 },
    { 0xe7de96bb, SUC_lexicon_tags_38 },
    { 0xa0be96bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1e96c0, SUC_lexicon_tags_30 },
    { 0x59be96c1, SUC_lexicon_tags_7 },
    { 0xeebe96c2, SUC_lexicon_tags_65 },
    { 0x5fe96c3, SUC_lexicon_tags_30 },
    { 0x845e96c2, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x9c5e96c6, SUC_lexicon_tags_9 },
    { 0x585e96c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87be96ce, SUC_lexicon_tags_1 },
    { 0x99be96cf, SUC_lexicon_tags_37 },
    { 0xd7de96d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47de96d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1e96d5, SUC_lexicon_tags_37 },
    { 0x867e96d5, SUC_lexicon_tags_1 },
    { 0x825e96d7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23e96da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a1e96dc, SUC_lexicon_tags_28 },
    { 0x695e96dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xde3e96df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fde96e2, SUC_lexicon_tags_63 },
    { 0xff9e96e3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317e96e7, SUC_lexicon_tags_17 },
    { 0x40de96e7, SUC_lexicon_tags_2 },
    { 0xff9e96e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7fe96eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9fbe96ed, SUC_lexicon_tags_15 },
    { 0x181e96ee, SUC_lexicon_tags_12 },
    { 0x77e96ed, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341e96f3, SUC_lexicon_tags_14 },
    { 0x343e96f4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1de96f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x487e96f9, SUC_lexicon_tags_118 },
    { 0x23fe96f9, SUC_lexicon_tags_38 },
    { 0x855e96fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63be9700, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9e9704, SUC_lexicon_tags_12 },
    { 0xbf3e9705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3e970b, SUC_lexicon_tags_13 },
    { 0xb4be970b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fe9710, SUC_lexicon_tags_7 },
    { 0x72be9711, SUC_lexicon_tags_27 },
    { 0x37de9711, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafe9718, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41e9721, SUC_lexicon_tags_17 },
    { 0xccbe9722, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389e9728, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x755e972b, SUC_lexicon_tags_2 },
    { 0x219e972c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x87be972e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53be9733, SUC_lexicon_tags_7 },
    { 0x33de9734, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87fe9736, SUC_lexicon_tags_49 },
    { 0x9fde9736, SUC_lexicon_tags_27 },
    { 0x383e9738, SUC_lexicon_tags_39 },
    { 0x1f7e9739, SUC_lexicon_tags_4 },
    { 0xbbde9739, SUC_lexicon_tags_88 },
    { 0x9afe9737, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x667e973e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfb9e9740, SUC_lexicon_tags_8 },
    { 0x233e9740, SUC_lexicon_tags_49 },
    { 0xac7e9742, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x461e9744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6be9747, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x403e9749, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x391e974c, SUC_lexicon_tags_22 },
    { 0x42de974c, SUC_lexicon_tags_25 },
    { 0x4e1e974d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfade9750, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5e9757, SUC_lexicon_tags_46 },
    { 0x693e9757, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8e5e975a, SUC_lexicon_tags_43 },
    { 0xadde975b, SUC_lexicon_tags_47 },
    { 0x57e975b, SUC_lexicon_tags_31 },
    { 0x9bde975b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777e9761, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5e976b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1e9770, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73e9776, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23de9778, SUC_lexicon_tags_7 },
    { 0x543e9779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x187e977b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3e9786, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x253e9788, SUC_lexicon_tags_9 },
    { 0x629e9788, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0x8fbe978b, SUC_lexicon_tags_25 },
    { 0xc55e978c, SUC_lexicon_tags_0 },
    { 0xe7be978d, SUC_lexicon_tags_1 },
    { 0x44be978e, SUC_lexicon_tags_129 },
    { 0xc53e978e, SUC_lexicon_tags_37 },
    { 0x797e978c, SUC_lexicon_tags_2 },
    { 0xf45e9791, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf3e9793, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x363e9795, SUC_lexicon_tags_37 },
    { 0xc75e9795, SUC_lexicon_tags_25 },
    { 0xc1de9795, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x5dbe9799, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25e97a0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x25fe97a2, SUC_lexicon_tags_12 },
    { 0xafe97a2, SUC_lexicon_tags_46 },
    { 0xb95e97a4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc8fe97a6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x799e97a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45e97ac, SUC_lexicon_tags_24 },
    { 0x62be97ad, SUC_lexicon_tags_23 },
    { 0x99de97ae, SUC_lexicon_tags_15 },
    { 0x91be97af, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95e97b4, SUC_lexicon_tags_8 },
    { 0x177e97b5, SUC_lexicon_tags_30 },
    { 0xcade97b6, SUC_lexicon_tags_4 },
    { 0xf27e97b6, SUC_lexicon_tags_11 },
    { 0xe27e97b8, SUC_lexicon_tags_2 },
    { 0x3b1e97b8, SUC_lexicon_tags_28 },
    { 0xee9e97b8, SUC_lexicon_tags_48 },
    { 0xd1e97b5, SUC_lexicon_tags_38 },
    { 0x593e97bc, SUC_lexicon_tags_0 },
    { 0xed5e97bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf79e97bf, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebe97c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd71e97ca, SUC_lexicon_tags_38 },
    { 0xbb5e97ca, SUC_lexicon_tags_15 },
    { 0x6f3e97cc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x73fe97ce, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xf49e97d0, SUC_lexicon_tags_63 },
    { 0xa97e97d0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9e97d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3fe97d6, SUC_lexicon_tags_25 },
    { 0x7fbe97d6, SUC_lexicon_tags_7 },
    { 0x745e97d8, SUC_lexicon_tags_37 },
    { 0x963e97d8, SUC_lexicon_tags_16 },
    { 0x535e97d9, SUC_lexicon_tags_37 },
    { 0x24be97db, SUC_lexicon_tags_17 },
    { 0xe47e97dc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09e97f4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1e97f7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x687e97f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6fe97fc, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeade9801, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3e9807, SUC_lexicon_tags_18 },
    { 0xd71e9808, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf5e980a, SUC_lexicon_tags_48 },
    { 0x115e980a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95e9811, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719e981a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5e981d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7e9825, SUC_lexicon_tags_37 },
    { 0x1a3e9825, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb9fe9828, SUC_lexicon_tags_116 },
    { 0xeffe9829, SUC_lexicon_tags_11 },
    { 0x4a1e982a, SUC_lexicon_tags_12 },
    { 0x6dbe9829, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xef9e982d, SUC_lexicon_tags_13 },
    { 0x94fe982d, SUC_lexicon_tags_1 },
    { 0xa4fe982e, SUC_lexicon_tags_11 },
    { 0x817e9830, SUC_lexicon_tags_27 },
    { 0x4afe9831, SUC_lexicon_tags_17 },
    { 0x8bde9830, SUC_lexicon_tags_48 },
    { 0x175e9833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x627e9836, SUC_lexicon_tags_1 },
    { 0x645e9837, SUC_lexicon_tags_25 },
    { 0xc3fe9838, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xeb7e983a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdd7e983c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x485e983f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71e9843, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50de9849, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81e9850, SUC_lexicon_tags_12 },
    { 0x7dde9850, SUC_lexicon_tags_17 },
    { 0xb19e9852, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50de9854, SUC_lexicon_tags_62 },
    { 0xf39e9855, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5e9858, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbe9869, SUC_lexicon_tags_9 },
    { 0x725e9869, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35e986e, SUC_lexicon_tags_15 },
    { 0x42de986f, SUC_lexicon_tags_11 },
    { 0x209e9870, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789e9878, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x857e987b, SUC_lexicon_tags_1 },
    { 0x21e987c, SUC_lexicon_tags_13 },
    { 0xd3e987c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x477e987f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabe9882, SUC_lexicon_tags_11 },
    { 0x871e9883, SUC_lexicon_tags_65 },
    { 0xc75e9884, SUC_lexicon_tags_1 },
    { 0x3a7e9885, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fe9889, SUC_lexicon_tags_46 },
    { 0x467e9889, SUC_lexicon_tags_33 },
    { 0x62fe988b, SUC_lexicon_tags_30 },
    { 0x763e988a, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd2fe988e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x20de9891, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfa9e9893, SUC_lexicon_tags_28 },
    { 0xcafe9894, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85be9896, SUC_lexicon_tags_7 },
    { 0xbbbe9897, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3e989a, SUC_lexicon_tags_15 },
    { 0xb55e989b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x197e989d, SUC_lexicon_tags_12 },
    { 0xf91e989e, SUC_lexicon_tags_0 },
    { 0x987e989f, SUC_lexicon_tags_3 },
    { 0xd53e989d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bde98a5, SUC_lexicon_tags_7 },
    { 0xa2fe98a5, SUC_lexicon_tags_23 },
    { 0xb31e98a7, SUC_lexicon_tags_2 },
    { 0x259e98a8, SUC_lexicon_tags_2 },
    { 0x3a3e98a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d1e98ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1e98ae, SUC_lexicon_tags_14 },
    { 0x52fe98ae, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161e98b3, SUC_lexicon_tags_82 },
    { 0x1de98b4, SUC_lexicon_tags_7 },
    { 0xc9e98b5, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51e98b8, SUC_lexicon_tags_29 },
    { 0xdf3e98b9, SUC_lexicon_tags_28 },
    { 0xf1e98ba, SUC_lexicon_tags_39 },
    { 0x42de98ba, SUC_lexicon_tags_17 },
    { 0x7c3e98b9, SUC_lexicon_tags_37 },
    { 0x391e98bb, SUC_lexicon_tags_60 },
    { 0xbdbe98be, SUC_lexicon_tags_17 },
    { 0x89be98bf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489e98c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5e98c6, SUC_lexicon_tags_15 },
    { 0xdf5e98c6, SUC_lexicon_tags_6 },
    { 0x253e98c8, SUC_lexicon_tags_17 },
    { 0x833e98c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x94de98cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663e98cf, SUC_lexicon_tags_11 },
    { 0x1abe98cf, SUC_lexicon_tags_47 },
    { 0xa8de98d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x623e98d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e98d7, SUC_lexicon_tags_2 },
    { 0x255e98d8, SUC_lexicon_tags_13 },
    { 0x791e98d9, SUC_lexicon_tags_11 },
    { 0xb01e98da, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4de98de, SUC_lexicon_tags_29 },
    { 0x4afe98de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x20be98e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fe98e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5e98ef, SUC_lexicon_tags_59 },
    { 0x6b1e98f0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4ede98f2, SUC_lexicon_tags_19 },
    { 0x599e98f3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51e98ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79e9905, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x139e9907, SUC_lexicon_tags_15 },
    { 0xc7be9907, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1e990e, SUC_lexicon_tags_9 },
    { 0xe3de990f, SUC_lexicon_tags_67 },
    { 0x89fe9910, SUC_lexicon_tags_9 },
    { 0xe19e9911, SUC_lexicon_tags_11 },
    { 0xb97e9912, SUC_lexicon_tags_84 },
    { 0x5b7e9913, SUC_lexicon_tags_47 },
    { 0xf1be9912, SUC_lexicon_tags_28 },
    { 0x709e9915, SUC_lexicon_tags_26 },
    { 0xc37e9910, SUC_lexicon_tags_12 },
    { 0x6b9e9915, SUC_lexicon_tags_2 },
    { 0x1cde990f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9e991b, SUC_lexicon_tags_11 },
    { 0xf83e991c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893e9921, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6c3e9923, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf19e9925, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb63e9927, SUC_lexicon_tags_25 },
    { 0x819e9927, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x387e992b, SUC_lexicon_tags_39 },
    { 0x61be992c, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15de9930, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d1e9932, SUC_lexicon_tags_21 },
    { 0x38de9933, SUC_lexicon_tags_37 },
    { 0x17be9933, SUC_lexicon_tags_82 },
    { 0xc33e9933, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0fe9938, SUC_lexicon_tags_262 },
    { 0x64fe9939, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251e993d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x623e9940, SUC_lexicon_tags_14 },
    { 0x7b3e9940, SUC_lexicon_tags_1 },
    { 0x69e9940, SUC_lexicon_tags_37 },
    { 0x529e9943, SUC_lexicon_tags_17 },
    { 0xa6fe9944, SUC_lexicon_tags_45 },
    { 0xff7e9945, SUC_lexicon_tags_12 },
    { 0xbcfe9946, SUC_lexicon_tags_25 },
    { 0xa5be9945, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x601e9949, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x247e994b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x981e994d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9de9955, SUC_lexicon_tags_11 },
    { 0xa09e9956, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x883e9958, SUC_lexicon_tags_47 },
    { 0xb7de9959, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605e995d, SUC_lexicon_tags_28 },
    { 0x785e995d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbe9961, SUC_lexicon_tags_7 },
    { 0x6e7e9961, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327e9967, SUC_lexicon_tags_14 },
    { 0x775e9967, SUC_lexicon_tags_7 },
    { 0xf3fe9967, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9e996c, SUC_lexicon_tags_7 },
    { 0x395e996d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefe9970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4be9973, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65e9977, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221e997e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9e9981, SUC_lexicon_tags_63 },
    { 0x21de9982, SUC_lexicon_tags_30 },
    { 0xa9de9982, SUC_lexicon_tags_14 },
    { 0x3f5e9981, SUC_lexicon_tags_60 },
    { 0x7bde9984, SUC_lexicon_tags_17 },
    { 0xb8be9986, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x569e9988, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x43e998a, SUC_lexicon_tags_60 },
    { 0xa21e998b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x383e998e, SUC_lexicon_tags_17 },
    { 0x181e998f, SUC_lexicon_tags_79 },
    { 0xbebe9990, SUC_lexicon_tags_34 },
    { 0x211e9990, SUC_lexicon_tags_11 },
    { 0xd7e9990, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751e999b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3e999e, SUC_lexicon_tags_32 },
    { 0x2d3e999f, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0x289e99a1, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x4ebe99a3, SUC_lexicon_tags_28 },
    { 0xa77e99a4, SUC_lexicon_tags_7 },
    { 0x907e99a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911e99aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7e99ad, SUC_lexicon_tags_12 },
    { 0xf91e99ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97de99b3, SUC_lexicon_tags_17 },
    { 0x22fe99b4, SUC_lexicon_tags_77 },
    { 0xc8fe99b3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x9afe99b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3e7e99b9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x397e99bb, SUC_lexicon_tags_15 },
    { 0xd03e99bc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7e99c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcfe99c3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59e99c8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffe99cb, SUC_lexicon_tags_68 },
    { 0x237e99cc, SUC_lexicon_tags_25 },
    { 0xabbe99cb, SUC_lexicon_tags_16 },
    { 0x3f7e99ce, SUC_lexicon_tags_12 },
    { 0xc39e99ce, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5e99d3, SUC_lexicon_tags_9 },
    { 0x33be99d4, SUC_lexicon_tags_68 },
    { 0x72fe99d5, SUC_lexicon_tags_1 },
    { 0x8f7e99d6, SUC_lexicon_tags_37 },
    { 0x391e99d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1e99db, SUC_lexicon_tags_17 },
    { 0x243e99dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7e99df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5e99e7, SUC_lexicon_tags_47 },
    { 0x457e99e8, SUC_lexicon_tags_27 },
    { 0x5d5e99e8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3e99ef, SUC_lexicon_tags_17 },
    { 0x77be99ef, SUC_lexicon_tags_12 },
    { 0x5d7e99f1, SUC_lexicon_tags_45 },
    { 0x2dbe99f2, SUC_lexicon_tags_26 },
    { 0x8bde99f2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9b7e99f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7e99f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x845e99fb, SUC_lexicon_tags_57 },
    { 0xfffe99fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9cde99fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1e9a01, SUC_lexicon_tags_2 },
    { 0x631e9a02, SUC_lexicon_tags_15 },
    { 0x9d5e9a03, SUC_lexicon_tags_27 },
    { 0xea1e9a01, SUC_lexicon_tags_2 },
    { 0xabde9a03, SUC_lexicon_tags_35 },
    { 0xa1fe9a06, SUC_lexicon_tags_45 },
    { 0x44be9a07, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799e9a0c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5c7e9a0e, SUC_lexicon_tags_12 },
    { 0xafe9a0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36de9a11, SUC_lexicon_tags_0 },
    { 0x679e9a12, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90fe9a19, SUC_lexicon_tags_106 },
    { 0x5c1e9a19, SUC_lexicon_tags_37 },
    { 0xf3de9a19, SUC_lexicon_tags_17 },
    { 0xe31e9a19, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809e9a23, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x55de9a26, SUC_lexicon_tags_12 },
    { 0xa17e9a27, SUC_lexicon_tags_38 },
    { 0xadbe9a28, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5de9a2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fde9a32, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bde9a37, SUC_lexicon_tags_2 },
    { 0x797e9a38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55e9a3b, SUC_lexicon_tags_40 },
    { 0x75be9a3c, SUC_lexicon_tags_63 },
    { 0x2ede9a3c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7d1e9a3f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9e9a42, SUC_lexicon_tags_1 },
    { 0xcfde9a42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfe9a46, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45be9a4c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89e9a52, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fe9a55, SUC_lexicon_tags_29 },
    { 0x343e9a56, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xac9e9a58, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5e9a5e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x371e9a61, SUC_lexicon_tags_12 },
    { 0x63fe9a62, SUC_lexicon_tags_65 },
    { 0x7c1e9a62, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xedde9a65, SUC_lexicon_tags_87 },
    { 0x911e9a65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7e9a6a, SUC_lexicon_tags_8 },
    { 0x8fe9a6b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedfe9a6f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ade9a72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63e9a76, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebe9a7d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbe9a81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x619e9a83, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27e9a89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6fe9a8c, SUC_lexicon_tags_0 },
    { 0xa63e9a8d, SUC_lexicon_tags_33 },
    { 0x8fe9a8e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x571e9a90, SUC_lexicon_tags_23 },
    { 0x6a5e9a90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9e9a95, SUC_lexicon_tags_2 },
    { 0x57be9a95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x61be9a98, SUC_lexicon_tags_1 },
    { 0xdabe9a98, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299e9a9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5efe9aa5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x817e9aa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb29e9aa9, SUC_lexicon_tags_0 },
    { 0x8b5e9aaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119e9ab1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabde9ab5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe11e9ab7, SUC_lexicon_tags_23 },
    { 0x955e9ab8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227e9abd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86de9ac4, SUC_lexicon_tags_53 },
    { 0x17fe9ac5, SUC_lexicon_tags_2 },
    { 0x69fe9ac5, SUC_lexicon_tags_10 },
    { 0x911e9ac6, SUC_lexicon_tags_17 },
    { 0xbe1e9ac8, SUC_lexicon_tags_14 },
    { 0x423e9ac5, SUC_lexicon_tags_17 },
    { 0x66de9aca, SUC_lexicon_tags_27 },
    { 0x6a3e9ac9, SUC_lexicon_tags_12 },
    { 0xb2de9acc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75be9ad3, SUC_lexicon_tags_21 },
    { 0xe87e9ad4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x765e9ad6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd07e9ad8, SUC_lexicon_tags_6 },
    { 0xb07e9ad9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x37fe9adc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbe9adf, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dde9ae2, SUC_lexicon_tags_37 },
    { 0xaf3e9ae3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x7be9ae5, SUC_lexicon_tags_15 },
    { 0xfb7e9ae6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9e9aea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543e9aee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9b7e9af0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695e9af5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98de9afa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x20fe9afd, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fe9b06, SUC_lexicon_tags_68 },
    { 0xe13e9b07, SUC_lexicon_tags_65 },
    { 0x701e9b07, SUC_lexicon_tags_15 },
    { 0x4d5e9b09, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1e9b0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989e9b11, SUC_lexicon_tags_17 },
    { 0x6e3e9b11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedde9b14, SUC_lexicon_tags_0 },
    { 0x83fe9b15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7e9b18, SUC_lexicon_tags_30 },
    { 0x639e9b18, SUC_lexicon_tags_2 },
    { 0xc77e9b1a, SUC_lexicon_tags_27 },
    { 0xa57e9b1b, SUC_lexicon_tags_25 },
    { 0x479e9b1c, SUC_lexicon_tags_17 },
    { 0x371e9b1d, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0x195e9b20, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6be9b24, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x383e9b26, SUC_lexicon_tags_37 },
    { 0x209e9b27, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699e9b2c, SUC_lexicon_tags_0 },
    { 0x559e9b2c, SUC_lexicon_tags_25 },
    { 0x3e7e9b2d, SUC_lexicon_tags_12 },
    { 0x1b1e9b2d, SUC_lexicon_tags_21 },
    { 0x467e9b30, SUC_lexicon_tags_15 },
    { 0xf3fe9b30, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbe9b35, SUC_lexicon_tags_8 },
    { 0x521e9b36, SUC_lexicon_tags_17 },
    { 0xf75e9b37, SUC_lexicon_tags_2 },
    { 0x891e9b36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x195e9b3a, SUC_lexicon_tags_1 },
    { 0x589e9b3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3e9b3f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb8fe9b41, SUC_lexicon_tags_7 },
    { 0x8fde9b42, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x75be9b45, SUC_lexicon_tags_60 },
    { 0xe83e9b45, SUC_lexicon_tags_13 },
    { 0x173e9b47, SUC_lexicon_tags_11 },
    { 0x2c3e9b48, SUC_lexicon_tags_29 },
    { 0x64be9b48, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0xc11e9b4b, SUC_lexicon_tags_11 },
    { 0xdc7e9b4b, SUC_lexicon_tags_17 },
    { 0xd35e9b4b, SUC_lexicon_tags_11 },
    { 0x3c7e9b4d, SUC_lexicon_tags_33 },
    { 0xf11e9b4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953e9b53, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697e9b59, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x521e9b5c, SUC_lexicon_tags_12 },
    { 0x203e9b5d, SUC_lexicon_tags_1 },
    { 0xc29e9b5e, SUC_lexicon_tags_31 },
    { 0x937e9b5e, SUC_lexicon_tags_25 },
    { 0x5dde9b60, SUC_lexicon_tags_1 },
    { 0x657e9b61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x473e9b63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41e9b66, SUC_lexicon_tags_33 },
    { 0x377e9b66, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6be9b71, SUC_lexicon_tags_15 },
    { 0x62de9b71, SUC_lexicon_tags_12 },
    { 0xbafe9b71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7e9b7b, SUC_lexicon_tags_30 },
    { 0x80fe9b7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d5e9b7e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x875e9b81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299e9b86, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3e9b8a, SUC_lexicon_tags_0 },
    { 0x88fe9b8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x361e9b8d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94fe9b92, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45e9b96, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x37fe9b99, SUC_lexicon_tags_31 },
    { 0x4d1e9b9a, SUC_lexicon_tags_17 },
    { 0x88be9b9a, SUC_lexicon_tags_47 },
    { 0x6e5e9b9a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdb1e9b9e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531e9ba3, SUC_lexicon_tags_7 },
    { 0xdade9ba4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x111e9ba6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9bb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f9e9bb3, SUC_lexicon_tags_55 },
    { 0xc19e9bb3, SUC_lexicon_tags_6 },
    { 0xc07e9bb5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x651e9bb7, SUC_lexicon_tags_37 },
    { 0x343e9bb8, SUC_lexicon_tags_12 },
    { 0xa49e9bb9, SUC_lexicon_tags_57 },
    { 0x6fbe9bba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69e9bc3, SUC_lexicon_tags_11 },
    { 0xdf1e9bc4, SUC_lexicon_tags_49 },
    { 0x39de9bc5, SUC_lexicon_tags_7 },
    { 0x215e9bc6, SUC_lexicon_tags_1 },
    { 0x83de9bc7, SUC_lexicon_tags_1 },
    { 0x53be9bc7, SUC_lexicon_tags_61 },
    { 0x3c9e9bc6, SUC_lexicon_tags_82 },
    { 0x5bde9bca, SUC_lexicon_tags_31 },
    { 0x7ebe9bcb, SUC_lexicon_tags_27 },
    { 0x803e9bcc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf85e9bce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbde9bd3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabe9bd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x833e9bd9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171e9bde, SUC_lexicon_tags_12 },
    { 0x37e9bde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1e9be3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7e9beb, SUC_lexicon_tags_37 },
    { 0xf55e9beb, SUC_lexicon_tags_27 },
    { 0x7c7e9bec, SUC_lexicon_tags_26 },
    { 0x241e9bee, SUC_lexicon_tags_7 },
    { 0x43fe9bee, SUC_lexicon_tags_87 },
    { 0x8f1e9bee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9ede9bf2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5de9bfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x709e9bfd, SUC_lexicon_tags_30 },
    { 0x895e9bfe, SUC_lexicon_tags_11 },
    { 0xe7fe9bff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe9c02, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11e9c06, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x247e9c09, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf81e9c0b, SUC_lexicon_tags_13 },
    { 0x6b1e9c0b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x627e9c0f, SUC_lexicon_tags_43 },
    { 0xbb3e9c10, SUC_lexicon_tags_17 },
    { 0xa2fe9c11, SUC_lexicon_tags_1 },
    { 0x6ede9c11, SUC_lexicon_tags_103 },
    { 0xfb5e9c0f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x29be9c15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295e9c19, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x26be9c1c, SUC_lexicon_tags_47 },
    { 0x8a5e9c1c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x23be9c20, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xade9c22, SUC_lexicon_tags_12 },
    { 0x465e9c23, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75e9c2b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45e9c2f, SUC_lexicon_tags_19 },
    { 0xf45e9c30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9e9c37, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x43be9c3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1de9c40, SUC_lexicon_tags_63 },
    { 0x1f9e9c41, SUC_lexicon_tags_12 },
    { 0xa3de9c42, SUC_lexicon_tags_26 },
    { 0x629e9c43, SUC_lexicon_tags_55 },
    { 0xbc7e9c43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8de9c4e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975e9c56, SUC_lexicon_tags_15 },
    { 0xeabe9c57, SUC_lexicon_tags_37 },
    { 0xae9e9c58, SUC_lexicon_tags_47 },
    { 0xfc7e9c59, SUC_lexicon_tags_7 },
    { 0xb55e9c5a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69e9c5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8de9c65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5de9c6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abe9c75, SUC_lexicon_tags_0 },
    { 0x231e9c76, SUC_lexicon_tags_15 },
    { 0xa57e9c77, SUC_lexicon_tags_0 },
    { 0xe67e9c78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353e9c7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x215e9c80, SUC_lexicon_tags_66 },
    { 0xad9e9c81, SUC_lexicon_tags_77 },
    { 0x4b7e9c82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893e9c87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66de9c8e, SUC_lexicon_tags_1 },
    { 0x5d5e9c8f, SUC_lexicon_tags_24 },
    { 0x159e9c8f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45e9c94, SUC_lexicon_tags_8 },
    { 0x12fe9c95, SUC_lexicon_tags_983 },
    { 0xbfbe9c95, SUC_lexicon_tags_9 },
    { 0xcefe9c94, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x251e9c99, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4e9e9c9b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cde9ca0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe0de9ca2, SUC_lexicon_tags_1 },
    { 0x7a1e9ca3, SUC_lexicon_tags_26 },
    { 0xd43e9ca4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c9e9ca6, SUC_lexicon_tags_0 },
    { 0x11be9ca7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbe9cac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21e9caf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfe9cb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71e9cc0, SUC_lexicon_tags_17 },
    { 0x235e9cc0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2be9cca, SUC_lexicon_tags_22 },
    { 0xb87e9ccb, SUC_lexicon_tags_11 },
    { 0xd11e9ccc, SUC_lexicon_tags_7 },
    { 0x261e9ccd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1e9cd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645e9cd5, SUC_lexicon_tags_27 },
    { 0xd43e9cd5, SUC_lexicon_tags_994 },
    { 0, NULL },
    { 0x3b9e9cd8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xedde9cda, SUC_lexicon_tags_0 },
    { 0x53de9cda, SUC_lexicon_tags_37 },
    { 0x371e9cdc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x471e9cdf, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x141e9ce1, SUC_lexicon_tags_37 },
    { 0xf7e9ce2, SUC_lexicon_tags_15 },
    { 0xcfbe9ce3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x90be9ce5, SUC_lexicon_tags_1 },
    { 0xb0de9ce6, SUC_lexicon_tags_15 },
    { 0x317e9ce6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeade9ceb, SUC_lexicon_tags_301 },
    { 0x9dde9cec, SUC_lexicon_tags_37 },
    { 0xeebe9cec, SUC_lexicon_tags_26 },
    { 0xd21e9cee, SUC_lexicon_tags_23 },
    { 0xff5e9cef, SUC_lexicon_tags_55 },
    { 0xa99e9cf0, SUC_lexicon_tags_47 },
    { 0x7e9cf1, SUC_lexicon_tags_15 },
    { 0x309e9ced, SUC_lexicon_tags_11 },
    { 0x277e9cf3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x977e9cf6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fe9cf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7e9d06, SUC_lexicon_tags_11 },
    { 0x103e9d06, SUC_lexicon_tags_17 },
    { 0xd73e9d07, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdbbe9d0a, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9e9d10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaade9d12, SUC_lexicon_tags_0 },
    { 0xe7de9d12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc81e9d16, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xe5be9d18, SUC_lexicon_tags_43 },
    { 0xc73e9d19, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb53e9d1b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x139e9d1d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1e9d21, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xa27e9d23, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7e9d2a, SUC_lexicon_tags_7 },
    { 0x299e9d2b, SUC_lexicon_tags_68 },
    { 0x4b5e9d2b, SUC_lexicon_tags_25 },
    { 0xe17e9d2d, SUC_lexicon_tags_37 },
    { 0xad7e9d2d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3e9d31, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x98de9d33, SUC_lexicon_tags_9 },
    { 0x457e9d34, SUC_lexicon_tags_41 },
    { 0xb87e9d33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33e9d3a, SUC_lexicon_tags_33 },
    { 0x3e1e9d3b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45be9d40, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x917e9d42, SUC_lexicon_tags_60 },
    { 0x67fe9d42, SUC_lexicon_tags_49 },
    { 0xa39e9d44, SUC_lexicon_tags_68 },
    { 0x1a9e9d45, SUC_lexicon_tags_17 },
    { 0xacfe9d44, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x635e9d48, SUC_lexicon_tags_23 },
    { 0xa1be9d48, SUC_lexicon_tags_25 },
    { 0x89be9d4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5be9d4c, SUC_lexicon_tags_31 },
    { 0x5cbe9d4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x787e9d50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4dbe9d52, SUC_lexicon_tags_25 },
    { 0xb1be9d53, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x799e9d56, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841e9d5b, SUC_lexicon_tags_2 },
    { 0x111e9d5b, SUC_lexicon_tags_418 },
    { 0x579e9d5d, SUC_lexicon_tags_29 },
    { 0x24de9d5b, SUC_lexicon_tags_15 },
    { 0x7d3e9d5f, SUC_lexicon_tags_32 },
    { 0x9d1e9d5f, SUC_lexicon_tags_39 },
    { 0x5ade9d60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x317e9d64, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72be9d6a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5e9d72, SUC_lexicon_tags_7 },
    { 0x1e1e9d72, SUC_lexicon_tags_72 },
    { 0x4e5e9d74, SUC_lexicon_tags_21 },
    { 0x59be9d75, SUC_lexicon_tags_17 },
    { 0x90de9d72, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe47e9d78, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fe9d7c, SUC_lexicon_tags_29 },
    { 0x6f3e9d7d, SUC_lexicon_tags_28 },
    { 0x49be9d7d, SUC_lexicon_tags_28 },
    { 0x781e9d7f, SUC_lexicon_tags_15 },
    { 0x635e9d80, SUC_lexicon_tags_31 },
    { 0x163e9d80, SUC_lexicon_tags_27 },
    { 0x1bbe9d81, SUC_lexicon_tags_29 },
    { 0x427e9d83, SUC_lexicon_tags_11 },
    { 0xdcbe9d84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40de9d86, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x183e9d88, SUC_lexicon_tags_65 },
    { 0xf41e9d88, SUC_lexicon_tags_1 },
    { 0x899e9d88, SUC_lexicon_tags_60 },
    { 0x3b9e9d8b, SUC_lexicon_tags_33 },
    { 0x79e9d8b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x827e9d8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x883e9d90, SUC_lexicon_tags_14 },
    { 0xe21e9d90, SUC_lexicon_tags_6 },
    { 0xa1fe9d91, SUC_lexicon_tags_6 },
    { 0x84fe9d93, SUC_lexicon_tags_6 },
    { 0x9f3e9d94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943e9d99, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64be9d9d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x889e9d9f, SUC_lexicon_tags_17 },
    { 0x3b7e9d9f, SUC_lexicon_tags_9 },
    { 0xe7de9da0, SUC_lexicon_tags_17 },
    { 0xf45e9da2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899e9da7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fe9dae, SUC_lexicon_tags_16 },
    { 0xfb1e9daf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247e9db4, SUC_lexicon_tags_70 },
    { 0xc97e9db5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9e9db8, SUC_lexicon_tags_1 },
    { 0xe31e9db9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87e9dbd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x1e5e9dbf, SUC_lexicon_tags_15 },
    { 0xfe5e9dbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcfe9dc8, SUC_lexicon_tags_29 },
    { 0xa99e9dc8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8efe9dcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71e9dd7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9be9dde, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fe9de1, SUC_lexicon_tags_47 },
    { 0x795e9de2, SUC_lexicon_tags_1334 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8de9de5, SUC_lexicon_tags_60 },
    { 0x781e9de6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x61fe9de8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7e9dee, SUC_lexicon_tags_47 },
    { 0x2f7e9def, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9e9df3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11e9df6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97e9dfa, SUC_lexicon_tags_37 },
    { 0x135e9dfa, SUC_lexicon_tags_1 },
    { 0x94be9dfa, SUC_lexicon_tags_1 },
    { 0xa3fe9dfd, SUC_lexicon_tags_25 },
    { 0xde9e9dfa, SUC_lexicon_tags_17 },
    { 0x4bfe9dfe, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xee7e9e01, SUC_lexicon_tags_27 },
    { 0x97fe9e02, SUC_lexicon_tags_32 },
    { 0x6be9e02, SUC_lexicon_tags_35 },
    { 0x99be9e03, SUC_lexicon_tags_60 },
    { 0x875e9e05, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4abe9e07, SUC_lexicon_tags_7 },
    { 0x63e9e08, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3e9e0f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9de9e12, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe9e15, SUC_lexicon_tags_25 },
    { 0x3a9e9e16, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x271e9e18, SUC_lexicon_tags_27 },
    { 0x7c5e9e18, SUC_lexicon_tags_65 },
    { 0x117e9e18, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29e9e20, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fe9e25, SUC_lexicon_tags_43 },
    { 0xc8fe9e25, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x905e9e28, SUC_lexicon_tags_14 },
    { 0xd23e9e29, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x9cbe9e2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1e9e32, SUC_lexicon_tags_9 },
    { 0x7cde9e33, SUC_lexicon_tags_25 },
    { 0xd43e9e33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaede9e36, SUC_lexicon_tags_23 },
    { 0x9fe9e37, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd61e9e39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x89de9e3b, SUC_lexicon_tags_12 },
    { 0x1abe9e3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545e9e42, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x189e9e45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4de9e48, SUC_lexicon_tags_12 },
    { 0x2e9e9e49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe4de9e4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629e9e50, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5a5e9e52, SUC_lexicon_tags_2 },
    { 0x463e9e52, SUC_lexicon_tags_11 },
    { 0x521e9e54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x175e9e56, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3e9e5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231e9e62, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x803e9e64, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9e9e67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1e9e71, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5e9e77, SUC_lexicon_tags_7 },
    { 0xefde9e77, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657e9e7c, SUC_lexicon_tags_2 },
    { 0x7f3e9e7d, SUC_lexicon_tags_79 },
    { 0x77be9e7e, SUC_lexicon_tags_2 },
    { 0x8ffe9e7e, SUC_lexicon_tags_1 },
    { 0x89fe9e7e, SUC_lexicon_tags_14 },
    { 0x43de9e81, SUC_lexicon_tags_13 },
    { 0x527e9e7f, SUC_lexicon_tags_31 },
    { 0x757e9e83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefe9e88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221e9e8c, SUC_lexicon_tags_31 },
    { 0x793e9e8c, SUC_lexicon_tags_25 },
    { 0x6c5e9e8e, SUC_lexicon_tags_37 },
    { 0x2bbe9e8e, SUC_lexicon_tags_31 },
    { 0x743e9e90, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4fe9e95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdade9e97, SUC_lexicon_tags_49 },
    { 0x3dde9e98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ede9e9c, SUC_lexicon_tags_15 },
    { 0x9cfe9e9d, SUC_lexicon_tags_65 },
    { 0x11be9e9d, SUC_lexicon_tags_7 },
    { 0x265e9e9e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ade9ea2, SUC_lexicon_tags_17 },
    { 0xcf1e9ea3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5e9eab, SUC_lexicon_tags_43 },
    { 0x8c1e9eac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bde9eb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81e9eb4, SUC_lexicon_tags_10 },
    { 0x901e9eb5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9e9eb9, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6b7e9ebb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51e9ec2, SUC_lexicon_tags_7 },
    { 0x661e9ec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7afe9ec5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc59e9ec7, SUC_lexicon_tags_15 },
    { 0x491e9ec8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x961e9eca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fe9ecd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847e9ed4, SUC_lexicon_tags_18 },
    { 0x5a3e9ed5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x16be9ed8, SUC_lexicon_tags_13 },
    { 0x15be9ed9, SUC_lexicon_tags_17 },
    { 0xb5de9eda, SUC_lexicon_tags_38 },
    { 0x257e9edb, SUC_lexicon_tags_27 },
    { 0xce7e9edc, SUC_lexicon_tags_28 },
    { 0x779e9eda, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fe9ee2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5cde9ee4, SUC_lexicon_tags_23 },
    { 0x8d5e9ee5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57e9eea, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd7e9eec, SUC_lexicon_tags_37 },
    { 0xeffe9eed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83e9ef3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb5e9ef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69e9efd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1e9f01, SUC_lexicon_tags_37 },
    { 0x7e1e9f02, SUC_lexicon_tags_7 },
    { 0x1c3e9f03, SUC_lexicon_tags_28 },
    { 0x29fe9f03, SUC_lexicon_tags_1 },
    { 0x597e9f03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd39e9f07, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7de9f0a, SUC_lexicon_tags_17 },
    { 0xf03e9f0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9de9f10, SUC_lexicon_tags_17 },
    { 0x6d1e9f11, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1e9f14, SUC_lexicon_tags_60 },
    { 0xfc7e9f14, SUC_lexicon_tags_60 },
    { 0xaede9f16, SUC_lexicon_tags_2 },
    { 0xfc1e9f16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x929e9f1a, SUC_lexicon_tags_2 },
    { 0xb0de9f1a, SUC_lexicon_tags_19 },
    { 0x459e9f1b, SUC_lexicon_tags_11 },
    { 0xc9be9f1d, SUC_lexicon_tags_16 },
    { 0x6dde9f1a, SUC_lexicon_tags_39 },
    { 0x88de9f1f, SUC_lexicon_tags_11 },
    { 0xbf1e9f20, SUC_lexicon_tags_46 },
    { 0x329e9f20, SUC_lexicon_tags_2 },
    { 0xd9fe9f22, SUC_lexicon_tags_91 },
    { 0x95de9f20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1e9f26, SUC_lexicon_tags_39 },
    { 0xaabe9f27, SUC_lexicon_tags_63 },
    { 0x847e9f26, SUC_lexicon_tags_12 },
    { 0x38fe9f29, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15e9f2e, SUC_lexicon_tags_12 },
    { 0xc79e9f2f, SUC_lexicon_tags_37 },
    { 0x171e9f2f, SUC_lexicon_tags_28 },
    { 0xfabe9f30, SUC_lexicon_tags_29 },
    { 0x19e9f31, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x389e9f34, SUC_lexicon_tags_25 },
    { 0x529e9f35, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431e9f3a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffe9f3f, SUC_lexicon_tags_11 },
    { 0xff3e9f3f, SUC_lexicon_tags_31 },
    { 0x137e9f41, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x289e9f43, SUC_lexicon_tags_45 },
    { 0x237e9f43, SUC_lexicon_tags_66 },
    { 0x3c3e9f45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbde9f48, SUC_lexicon_tags_0 },
    { 0x9d7e9f48, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3e9f4e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3e9f52, SUC_lexicon_tags_25 },
    { 0x81de9f53, SUC_lexicon_tags_15 },
    { 0x91fe9f53, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769e9f59, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x46fe9f5c, SUC_lexicon_tags_63 },
    { 0x5bfe9f5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbe9f60, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1e9f64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1e9f68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5e9f70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5e9f77, SUC_lexicon_tags_17 },
    { 0xc43e9f77, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33de9f84, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7e9f88, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x315e9f8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4de9f8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb79e9f91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x79be9f94, SUC_lexicon_tags_56 },
    { 0x17e9f95, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3d1e9f97, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3e9fa0, SUC_lexicon_tags_60 },
    { 0x919e9fa0, SUC_lexicon_tags_13 },
    { 0x1c9e9fa2, SUC_lexicon_tags_23 },
    { 0x245e9fa2, SUC_lexicon_tags_30 },
    { 0x503e9fa4, SUC_lexicon_tags_26 },
    { 0x64fe9fa5, SUC_lexicon_tags_38 },
    { 0x28be9fa6, SUC_lexicon_tags_23 },
    { 0xf6be9fa1, SUC_lexicon_tags_19 },
    { 0xfde9fa8, SUC_lexicon_tags_30 },
    { 0x85de9fa9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86de9faf, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xac9e9fb2, SUC_lexicon_tags_37 },
    { 0x249e9fb3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbe9fb6, SUC_lexicon_tags_7 },
    { 0xb3be9fb7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe9fe9fb9, SUC_lexicon_tags_14 },
    { 0x329e9fba, SUC_lexicon_tags_1 },
    { 0xf1de9fba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x957e9fbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdde9fbf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17e9fc7, SUC_lexicon_tags_29 },
    { 0x1d9e9fc8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x30de9fca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9e9fce, SUC_lexicon_tags_32 },
    { 0x809e9fcf, SUC_lexicon_tags_14 },
    { 0x5e3e9fce, SUC_lexicon_tags_11 },
    { 0x58be9fd1, SUC_lexicon_tags_7 },
    { 0xcade9fd2, SUC_lexicon_tags_31 },
    { 0xfd3e9fd2, SUC_lexicon_tags_35 },
    { 0x47e9fd4, SUC_lexicon_tags_15 },
    { 0x923e9fd5, SUC_lexicon_tags_12 },
    { 0x831e9fd5, SUC_lexicon_tags_17 },
    { 0x9de9fd4, SUC_lexicon_tags_19 },
    { 0x231e9fd8, SUC_lexicon_tags_31 },
    { 0x8d5e9fd3, SUC_lexicon_tags_2 },
    { 0x721e9fd0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95be9fdf, SUC_lexicon_tags_2 },
    { 0x78de9fdf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461e9fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95e9fea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbe9fee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x18be9ff0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3e9ff4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fe9ff7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcde9ffb, SUC_lexicon_tags_12 },
    { 0x9cfe9ffc, SUC_lexicon_tags_0 },
    { 0xfa5e9ffc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ea003, SUC_lexicon_tags_28 },
    { 0xfb1ea004, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x537ea007, SUC_lexicon_tags_10 },
    { 0x93ea007, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea00b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8fea00d, SUC_lexicon_tags_38 },
    { 0x263ea00d, SUC_lexicon_tags_11 },
    { 0xacfea00f, SUC_lexicon_tags_14 },
    { 0x8e3ea00f, SUC_lexicon_tags_54 },
    { 0xb6dea00f, SUC_lexicon_tags_15 },
    { 0x661ea00e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd35ea014, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793ea019, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xf81ea01b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fea01e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1d9ea020, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc83ea022, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x325ea024, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759ea028, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3ea02e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1fea030, SUC_lexicon_tags_68 },
    { 0x313ea030, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfbfea033, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x21dea035, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x397ea037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x645ea039, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63ea03c, SUC_lexicon_tags_12 },
    { 0xa19ea03d, SUC_lexicon_tags_11 },
    { 0xf7bea03d, SUC_lexicon_tags_16 },
    { 0x82dea03f, SUC_lexicon_tags_12 },
    { 0x4b3ea03d, SUC_lexicon_tags_124 },
    { 0x9b5ea03f, SUC_lexicon_tags_14 },
    { 0xcb9ea042, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb2fea044, SUC_lexicon_tags_37 },
    { 0x56fea044, SUC_lexicon_tags_15 },
    { 0x94dea046, SUC_lexicon_tags_33 },
    { 0x51ea047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebea04f, SUC_lexicon_tags_38 },
    { 0x3ddea050, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x305ea053, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9ea056, SUC_lexicon_tags_7 },
    { 0x319ea057, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bea05c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa87ea05e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x8ffea060, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435ea064, SUC_lexicon_tags_37 },
    { 0x9f3ea065, SUC_lexicon_tags_3 },
    { 0x76dea065, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5edea069, SUC_lexicon_tags_24 },
    { 0x8c1ea06a, SUC_lexicon_tags_11 },
    { 0xc95ea06a, SUC_lexicon_tags_23 },
    { 0x145ea06c, SUC_lexicon_tags_66 },
    { 0xf6bea06d, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x425ea06f, SUC_lexicon_tags_226 },
    { 0x2c3ea070, SUC_lexicon_tags_7 },
    { 0xa43ea070, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459ea078, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ea081, SUC_lexicon_tags_11 },
    { 0x565ea082, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2cdea084, SUC_lexicon_tags_7 },
    { 0x5c9ea085, SUC_lexicon_tags_37 },
    { 0x52dea086, SUC_lexicon_tags_37 },
    { 0x7a3ea086, SUC_lexicon_tags_11 },
    { 0xa85ea086, SUC_lexicon_tags_107 },
    { 0x8a7ea086, SUC_lexicon_tags_1 },
    { 0x163ea088, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7ea096, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bea099, SUC_lexicon_tags_63 },
    { 0xd8dea099, SUC_lexicon_tags_30 },
    { 0xcefea09b, SUC_lexicon_tags_23 },
    { 0x557ea099, SUC_lexicon_tags_14 },
    { 0xdabea09d, SUC_lexicon_tags_27 },
    { 0x6c3ea09e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5ea0a1, SUC_lexicon_tags_7 },
    { 0x3fea0a1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x27bea0a5, SUC_lexicon_tags_11 },
    { 0xf9ea0a6, SUC_lexicon_tags_11 },
    { 0x44fea0a7, SUC_lexicon_tags_15 },
    { 0xac9ea0a8, SUC_lexicon_tags_25 },
    { 0xba3ea0a9, SUC_lexicon_tags_7 },
    { 0xfddea0a8, SUC_lexicon_tags_31 },
    { 0x1afea0ab, SUC_lexicon_tags_37 },
    { 0xe73ea0ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfea0b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedbea0b3, SUC_lexicon_tags_49 },
    { 0x685ea0b3, SUC_lexicon_tags_8 },
    { 0x179ea0b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x229ea0b8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05ea0c0, SUC_lexicon_tags_47 },
    { 0x1ea0c1, SUC_lexicon_tags_26 },
    { 0x8d5ea0c2, SUC_lexicon_tags_34 },
    { 0x7d7ea0c2, SUC_lexicon_tags_2 },
    { 0x8c5ea0c4, SUC_lexicon_tags_0 },
    { 0x43dea0c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xba3ea0c8, SUC_lexicon_tags_1 },
    { 0xae1ea0c9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847ea0ce, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbea0d4, SUC_lexicon_tags_43 },
    { 0x62dea0d5, SUC_lexicon_tags_49 },
    { 0xd11ea0d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x635ea0d9, SUC_lexicon_tags_49 },
    { 0x7f3ea0da, SUC_lexicon_tags_17 },
    { 0x491ea0d9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x88dea0dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x529ea0df, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83ea0e4, SUC_lexicon_tags_2 },
    { 0x593ea0e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581ea0eb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x479ea0ed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3ea0f3, SUC_lexicon_tags_54 },
    { 0xa45ea0f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x417ea0f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01ea0fa, SUC_lexicon_tags_36 },
    { 0xb1bea0fb, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1ea0ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3ea102, SUC_lexicon_tags_45 },
    { 0xa9fea103, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3ea106, SUC_lexicon_tags_1 },
    { 0x127ea106, SUC_lexicon_tags_37 },
    { 0xfe7ea107, SUC_lexicon_tags_17 },
    { 0xefdea109, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ea10e, SUC_lexicon_tags_39 },
    { 0x9fdea10f, SUC_lexicon_tags_47 },
    { 0xd6bea110, SUC_lexicon_tags_36 },
    { 0x7bfea110, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda9ea113, SUC_lexicon_tags_2 },
    { 0x965ea114, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x611ea116, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181ea120, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd31ea122, SUC_lexicon_tags_351 },
    { 0x937ea123, SUC_lexicon_tags_31 },
    { 0x519ea124, SUC_lexicon_tags_14 },
    { 0x849ea125, SUC_lexicon_tags_45 },
    { 0x30fea122, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bea12d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2f5ea12f, SUC_lexicon_tags_31 },
    { 0x83dea12f, SUC_lexicon_tags_32 },
    { 0xc77ea131, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddea13a, SUC_lexicon_tags_14 },
    { 0xb8dea13a, SUC_lexicon_tags_11 },
    { 0x621ea13b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7ea13f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9e5ea141, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xefdea144, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7edea146, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273ea14b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ea14e, SUC_lexicon_tags_1 },
    { 0x9b5ea14f, SUC_lexicon_tags_0 },
    { 0x623ea14e, SUC_lexicon_tags_2 },
    { 0x9afea150, SUC_lexicon_tags_2 },
    { 0xecbea14e, SUC_lexicon_tags_49 },
    { 0xfffea153, SUC_lexicon_tags_17 },
    { 0xe59ea151, SUC_lexicon_tags_37 },
    { 0xf17ea155, SUC_lexicon_tags_15 },
    { 0x9edea156, SUC_lexicon_tags_1 },
    { 0x951ea154, SUC_lexicon_tags_29 },
    { 0x4c9ea158, SUC_lexicon_tags_17 },
    { 0x11dea159, SUC_lexicon_tags_7 },
    { 0x55fea14f, SUC_lexicon_tags_33 },
    { 0x2e5ea15b, SUC_lexicon_tags_11 },
    { 0x1b3ea154, SUC_lexicon_tags_33 },
    { 0xf1ea158, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455ea162, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247ea168, SUC_lexicon_tags_167 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edea16c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x53dea16e, SUC_lexicon_tags_13 },
    { 0x3a9ea16e, SUC_lexicon_tags_19 },
    { 0x2a5ea170, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3ea173, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7ea178, SUC_lexicon_tags_17 },
    { 0xa59ea179, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fea17c, SUC_lexicon_tags_17 },
    { 0x12dea17d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dea185, SUC_lexicon_tags_46 },
    { 0x91ea186, SUC_lexicon_tags_30 },
    { 0xff9ea185, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6e3ea189, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x557ea18c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddea193, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9ea196, SUC_lexicon_tags_27 },
    { 0x7bdea197, SUC_lexicon_tags_37 },
    { 0xc57ea196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bea19c, SUC_lexicon_tags_15 },
    { 0xd7dea19d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bea1a2, SUC_lexicon_tags_2 },
    { 0x229ea1a3, SUC_lexicon_tags_12 },
    { 0x393ea1a3, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xf61ea1a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x621ea1a9, SUC_lexicon_tags_49 },
    { 0xedea1aa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x489ea1ad, SUC_lexicon_tags_12 },
    { 0xd91ea1ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x427ea1b0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x503ea1b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5ea1bd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5ea1c0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9ea1c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe09ea1c5, SUC_lexicon_tags_2 },
    { 0x7fdea1c5, SUC_lexicon_tags_37 },
    { 0x99ea1c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dea1ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa25ea1cd, SUC_lexicon_tags_37 },
    { 0x385ea1ce, SUC_lexicon_tags_1 },
    { 0x8d5ea1cf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95ea1d2, SUC_lexicon_tags_66 },
    { 0x9a9ea1d3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235ea1d8, SUC_lexicon_tags_46 },
    { 0xf73ea1d9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3ea1dc, SUC_lexicon_tags_7 },
    { 0x63bea1dc, SUC_lexicon_tags_37 },
    { 0x151ea1dd, SUC_lexicon_tags_7 },
    { 0x827ea1de, SUC_lexicon_tags_16 },
    { 0x8adea1de, SUC_lexicon_tags_9 },
    { 0x935ea1e1, SUC_lexicon_tags_2 },
    { 0xd35ea1e2, SUC_lexicon_tags_25 },
    { 0xb23ea1e2, SUC_lexicon_tags_2 },
    { 0x7c1ea1dd, SUC_lexicon_tags_1 },
    { 0xafea1e5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925ea1ea, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8efea1ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241ea1f5, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bea1f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1ea1fd, SUC_lexicon_tags_33 },
    { 0x29fea1fe, SUC_lexicon_tags_7 },
    { 0xf7ea1fe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x38fea201, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e9ea203, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7ea207, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273ea20c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbea20f, SUC_lexicon_tags_118 },
    { 0xac1ea210, SUC_lexicon_tags_11 },
    { 0x6a5ea210, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fea215, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e1ea217, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35ea21c, SUC_lexicon_tags_64 },
    { 0x17bea21d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3ea224, SUC_lexicon_tags_11 },
    { 0x9bbea224, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fea228, SUC_lexicon_tags_37 },
    { 0xa45ea229, SUC_lexicon_tags_62 },
    { 0x19fea22a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dea231, SUC_lexicon_tags_12 },
    { 0xc2dea231, SUC_lexicon_tags_25 },
    { 0xef1ea233, SUC_lexicon_tags_3 },
    { 0xff3ea233, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcfea236, SUC_lexicon_tags_19 },
    { 0xca3ea237, SUC_lexicon_tags_7 },
    { 0x639ea237, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc87ea23a, SUC_lexicon_tags_23 },
    { 0xed5ea23b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb6dea23d, SUC_lexicon_tags_7 },
    { 0x52bea23d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfea24f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fea256, SUC_lexicon_tags_11 },
    { 0xa9bea257, SUC_lexicon_tags_33 },
    { 0xa7bea257, SUC_lexicon_tags_44 },
    { 0xbedea259, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x84fea25c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5ea266, SUC_lexicon_tags_55 },
    { 0xf6bea267, SUC_lexicon_tags_7 },
    { 0x401ea267, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15fea26a, SUC_lexicon_tags_37 },
    { 0x90fea26a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x81bea26d, SUC_lexicon_tags_14 },
    { 0x301ea26e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99ea273, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb03ea276, SUC_lexicon_tags_49 },
    { 0xf0dea277, SUC_lexicon_tags_21 },
    { 0x3a1ea278, SUC_lexicon_tags_11 },
    { 0xd95ea279, SUC_lexicon_tags_29 },
    { 0xd95ea277, SUC_lexicon_tags_6 },
    { 0x871ea27b, SUC_lexicon_tags_14 },
    { 0x327ea27a, SUC_lexicon_tags_25 },
    { 0x7c7ea27d, SUC_lexicon_tags_37 },
    { 0x95fea27e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x35bea280, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc75ea282, SUC_lexicon_tags_12 },
    { 0xb9ea283, SUC_lexicon_tags_61 },
    { 0x8c1ea284, SUC_lexicon_tags_12 },
    { 0x997ea285, SUC_lexicon_tags_12 },
    { 0x1adea286, SUC_lexicon_tags_85 },
    { 0x327ea286, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805ea28d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dea295, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52fea297, SUC_lexicon_tags_7 },
    { 0xcb9ea298, SUC_lexicon_tags_13 },
    { 0x971ea299, SUC_lexicon_tags_15 },
    { 0xf1fea29a, SUC_lexicon_tags_29 },
    { 0x13fea29a, SUC_lexicon_tags_31 },
    { 0x1f1ea29b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2cbea29e, SUC_lexicon_tags_14 },
    { 0x45ea29e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3ea2a3, SUC_lexicon_tags_37 },
    { 0x237ea2a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1ea2a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727ea2ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x865ea2ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7efea2b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57ea2b6, SUC_lexicon_tags_38 },
    { 0xa8bea2b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6bdea2b9, SUC_lexicon_tags_2 },
    { 0x6a3ea2b9, SUC_lexicon_tags_15 },
    { 0xc37ea2b9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x641ea2bd, SUC_lexicon_tags_63 },
    { 0x4bea2be, SUC_lexicon_tags_47 },
    { 0x675ea2bf, SUC_lexicon_tags_26 },
    { 0x3dfea2bf, SUC_lexicon_tags_46 },
    { 0x1b5ea2c1, SUC_lexicon_tags_1 },
    { 0x1edea2bf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5ea2c5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x941ea2c7, SUC_lexicon_tags_17 },
    { 0x6edea2c8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x703ea2ca, SUC_lexicon_tags_43 },
    { 0xa1dea2ca, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713ea2d1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbefea2d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63dea2d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcd7ea2d9, SUC_lexicon_tags_63 },
    { 0xd3fea2d9, SUC_lexicon_tags_26 },
    { 0x4edea2db, SUC_lexicon_tags_7 },
    { 0xba5ea2d9, SUC_lexicon_tags_1 },
    { 0xf7bea2dd, SUC_lexicon_tags_63 },
    { 0x9dea2de, SUC_lexicon_tags_1198 },
    { 0x945ea2dd, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47ea2e2, SUC_lexicon_tags_62 },
    { 0xb7ea2e2, SUC_lexicon_tags_33 },
    { 0x55ea2e2, SUC_lexicon_tags_19 },
    { 0x679ea2e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31dea2e7, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x82dea2e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98bea2eb, SUC_lexicon_tags_23 },
    { 0xd03ea2ec, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6cdea2ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xd35ea2f0, SUC_lexicon_tags_3 },
    { 0x643ea2f1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675ea2f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5ea2fb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673ea300, SUC_lexicon_tags_0 },
    { 0x3b9ea300, SUC_lexicon_tags_21 },
    { 0x2b5ea302, SUC_lexicon_tags_25 },
    { 0x247ea302, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fea309, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc51ea30b, SUC_lexicon_tags_17 },
    { 0xa29ea30c, SUC_lexicon_tags_12 },
    { 0x7b3ea30d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc09ea30f, SUC_lexicon_tags_27 },
    { 0x67bea310, SUC_lexicon_tags_26 },
    { 0x95fea310, SUC_lexicon_tags_12 },
    { 0x4bfea312, SUC_lexicon_tags_66 },
    { 0xe83ea311, SUC_lexicon_tags_7 },
    { 0xfc5ea314, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x33ea316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bea31d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe0fea31f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdea326, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5edea328, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5ea330, SUC_lexicon_tags_46 },
    { 0xb45ea331, SUC_lexicon_tags_26 },
    { 0x781ea332, SUC_lexicon_tags_45 },
    { 0xb3bea330, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0x625ea336, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x7f3ea338, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x23dea33a, SUC_lexicon_tags_25 },
    { 0x5f9ea33a, SUC_lexicon_tags_7 },
    { 0x8ffea33a, SUC_lexicon_tags_11 },
    { 0x7c3ea33a, SUC_lexicon_tags_207 },
    { 0x51bea33e, SUC_lexicon_tags_17 },
    { 0x49bea33f, SUC_lexicon_tags_28 },
    { 0x221ea33f, SUC_lexicon_tags_49 },
    { 0x7a1ea33d, SUC_lexicon_tags_30 },
    { 0x3fbea342, SUC_lexicon_tags_8 },
    { 0xaa9ea343, SUC_lexicon_tags_37 },
    { 0x1bdea344, SUC_lexicon_tags_9 },
    { 0xbf1ea341, SUC_lexicon_tags_8 },
    { 0x87dea346, SUC_lexicon_tags_15 },
    { 0xcbfea33c, SUC_lexicon_tags_12 },
    { 0xe4dea347, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x673ea34a, SUC_lexicon_tags_57 },
    { 0xa23ea34b, SUC_lexicon_tags_27 },
    { 0x1f3ea34c, SUC_lexicon_tags_14 },
    { 0x439ea34a, SUC_lexicon_tags_17 },
    { 0xf37ea34e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6bea350, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbea353, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7ea356, SUC_lexicon_tags_1 },
    { 0xb8fea356, SUC_lexicon_tags_32 },
    { 0x6e1ea357, SUC_lexicon_tags_29 },
    { 0xa23ea359, SUC_lexicon_tags_62 },
    { 0x3d3ea35a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23ea35d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07ea364, SUC_lexicon_tags_15 },
    { 0x9c1ea365, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbf7ea367, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973ea36b, SUC_lexicon_tags_11 },
    { 0x759ea36c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31ea371, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x565ea373, SUC_lexicon_tags_17 },
    { 0x101ea373, SUC_lexicon_tags_9 },
    { 0x4e3ea375, SUC_lexicon_tags_11 },
    { 0x34bea375, SUC_lexicon_tags_16 },
    { 0x529ea376, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdea37a, SUC_lexicon_tags_66 },
    { 0x61ea37b, SUC_lexicon_tags_29 },
    { 0x647ea37a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x213ea37e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fea382, SUC_lexicon_tags_1 },
    { 0xb7ea383, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91ea387, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9ea38c, SUC_lexicon_tags_11 },
    { 0x15ea38c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb05ea38f, SUC_lexicon_tags_11 },
    { 0xf5dea390, SUC_lexicon_tags_12 },
    { 0xbf1ea391, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98bea393, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0xe09ea395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bea39d, SUC_lexicon_tags_33 },
    { 0x909ea39d, SUC_lexicon_tags_37 },
    { 0x2bbea39e, SUC_lexicon_tags_0 },
    { 0x1e5ea3a0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x90dea3a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7ea3a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dea3a8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbdbea3aa, SUC_lexicon_tags_0 },
    { 0x65ea3aa, SUC_lexicon_tags_60 },
    { 0xeb3ea3ac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdea3af, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x999ea3b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52dea3b7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935ea3bc, SUC_lexicon_tags_17 },
    { 0xb85ea3bc, SUC_lexicon_tags_21 },
    { 0xea7ea3be, SUC_lexicon_tags_11 },
    { 0x533ea3bd, SUC_lexicon_tags_1 },
    { 0xefdea3c0, SUC_lexicon_tags_15 },
    { 0x59dea3c1, SUC_lexicon_tags_29 },
    { 0x9bbea3c2, SUC_lexicon_tags_14 },
    { 0x45ea3bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x393ea3c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fea3ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xabdea3cd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbea3d0, SUC_lexicon_tags_9 },
    { 0x6d9ea3d1, SUC_lexicon_tags_60 },
    { 0xd35ea3d1, SUC_lexicon_tags_25 },
    { 0xc91ea3d2, SUC_lexicon_tags_2 },
    { 0x5cbea3d4, SUC_lexicon_tags_47 },
    { 0xcabea3d0, SUC_lexicon_tags_37 },
    { 0x7ebea3d4, SUC_lexicon_tags_38 },
    { 0x5b1ea3d0, SUC_lexicon_tags_30 },
    { 0xfabea3d4, SUC_lexicon_tags_17 },
    { 0xefbea3d9, SUC_lexicon_tags_15 },
    { 0xd97ea3d9, SUC_lexicon_tags_14 },
    { 0x19fea3da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efea3e0, SUC_lexicon_tags_32 },
    { 0x19bea3e0, SUC_lexicon_tags_12 },
    { 0x399ea3e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdea3e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfea3ea, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abea3ed, SUC_lexicon_tags_7 },
    { 0xa59ea3ee, SUC_lexicon_tags_25 },
    { 0xc07ea3ef, SUC_lexicon_tags_7 },
    { 0xfffea3f0, SUC_lexicon_tags_10 },
    { 0xfb7ea3f0, SUC_lexicon_tags_14 },
    { 0x753ea3ed, SUC_lexicon_tags_66 },
    { 0x87fea3ef, SUC_lexicon_tags_37 },
    { 0x55bea3f4, SUC_lexicon_tags_12 },
    { 0x667ea3f4, SUC_lexicon_tags_9 },
    { 0xc8dea3f4, SUC_lexicon_tags_37 },
    { 0x4e1ea3f7, SUC_lexicon_tags_38 },
    { 0x14dea3ed, SUC_lexicon_tags_46 },
    { 0x193ea3f9, SUC_lexicon_tags_1 },
    { 0x71ea3ed, SUC_lexicon_tags_0 },
    { 0x94dea3fb, SUC_lexicon_tags_44 },
    { 0x681ea3f8, SUC_lexicon_tags_12 },
    { 0x4b1ea3fd, SUC_lexicon_tags_38 },
    { 0xc3dea3fe, SUC_lexicon_tags_95 },
    { 0x769ea3fe, SUC_lexicon_tags_19 },
    { 0x101ea400, SUC_lexicon_tags_11 },
    { 0x7ea3fe, SUC_lexicon_tags_1 },
    { 0xe0fea402, SUC_lexicon_tags_12 },
    { 0xbc3ea401, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x481ea405, SUC_lexicon_tags_12 },
    { 0x321ea406, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfe3ea408, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ea40b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ea40e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbea411, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x769ea414, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x309ea417, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x223ea41a, SUC_lexicon_tags_17 },
    { 0x29fea41b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x335ea41e, SUC_lexicon_tags_30 },
    { 0x1ffea41f, SUC_lexicon_tags_25 },
    { 0xd57ea420, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x7fea422, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abea426, SUC_lexicon_tags_33 },
    { 0x71ea427, SUC_lexicon_tags_23 },
    { 0x289ea427, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9ea42f, SUC_lexicon_tags_37 },
    { 0x5dbea430, SUC_lexicon_tags_15 },
    { 0xb3bea430, SUC_lexicon_tags_25 },
    { 0xbadea432, SUC_lexicon_tags_45 },
    { 0xe39ea432, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bea437, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5ea43a, SUC_lexicon_tags_28 },
    { 0xf55ea43b, SUC_lexicon_tags_12 },
    { 0xe69ea43c, SUC_lexicon_tags_71 },
    { 0x505ea43c, SUC_lexicon_tags_31 },
    { 0x8bdea43c, SUC_lexicon_tags_17 },
    { 0x17dea43f, SUC_lexicon_tags_17 },
    { 0x76bea43f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x735ea443, SUC_lexicon_tags_15 },
    { 0xf91ea444, SUC_lexicon_tags_15 },
    { 0xc1fea444, SUC_lexicon_tags_1 },
    { 0x8adea446, SUC_lexicon_tags_15 },
    { 0xfa9ea444, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddea44b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x657ea44e, SUC_lexicon_tags_7 },
    { 0x5f7ea44f, SUC_lexicon_tags_25 },
    { 0x79fea450, SUC_lexicon_tags_17 },
    { 0xefea450, SUC_lexicon_tags_16 },
    { 0x57ea451, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129ea45d, SUC_lexicon_tags_25 },
    { 0x243ea45d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ea463, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45ea467, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bea46a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55ea46d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403ea471, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987ea477, SUC_lexicon_tags_46 },
    { 0x6a7ea478, SUC_lexicon_tags_1 },
    { 0x5ffea479, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x53fea47b, SUC_lexicon_tags_37 },
    { 0x121ea47c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1ea480, SUC_lexicon_tags_11 },
    { 0xa0fea481, SUC_lexicon_tags_12 },
    { 0xac9ea482, SUC_lexicon_tags_15 },
    { 0xfedea481, SUC_lexicon_tags_10 },
    { 0x43dea484, SUC_lexicon_tags_45 },
    { 0xf65ea484, SUC_lexicon_tags_10 },
    { 0x6dea481, SUC_lexicon_tags_6 },
    { 0xfb9ea484, SUC_lexicon_tags_23 },
    { 0xa87ea488, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bea497, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x67bea499, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9ea49c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfea49f, SUC_lexicon_tags_7 },
    { 0x29bea49f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dea4a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd7ea4a7, SUC_lexicon_tags_17 },
    { 0x173ea4a8, SUC_lexicon_tags_37 },
    { 0xbcdea4a9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941ea4ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd61ea4b0, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9ea4b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981ea4bf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa13ea4c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x15dea4c4, SUC_lexicon_tags_32 },
    { 0x8c1ea4c4, SUC_lexicon_tags_21 },
    { 0xdddea4c4, SUC_lexicon_tags_1 },
    { 0xd7bea4c6, SUC_lexicon_tags_2 },
    { 0xd0fea4c8, SUC_lexicon_tags_37 },
    { 0x10bea4c9, SUC_lexicon_tags_1 },
    { 0x995ea4c6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969ea4d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x29bea4d2, SUC_lexicon_tags_31 },
    { 0xf89ea4d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88bea4d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491ea4db, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf5bea4dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855ea4e1, SUC_lexicon_tags_66 },
    { 0x6bfea4e2, SUC_lexicon_tags_18 },
    { 0xca1ea4e1, SUC_lexicon_tags_46 },
    { 0x47fea4e4, SUC_lexicon_tags_2 },
    { 0xd57ea4e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7ea4e8, SUC_lexicon_tags_15 },
    { 0x9edea4e9, SUC_lexicon_tags_47 },
    { 0x783ea4e9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40fea4f3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x66fea4f6, SUC_lexicon_tags_25 },
    { 0x965ea4f6, SUC_lexicon_tags_46 },
    { 0xeb3ea4f8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7ea4ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c9ea501, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe91ea503, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa7bea505, SUC_lexicon_tags_45 },
    { 0xb37ea506, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153ea50b, SUC_lexicon_tags_11 },
    { 0x275ea50b, SUC_lexicon_tags_23 },
    { 0xf1dea50d, SUC_lexicon_tags_7 },
    { 0xa5dea50d, SUC_lexicon_tags_234 },
    { 0xebea50f, SUC_lexicon_tags_65 },
    { 0x13dea510, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x623ea513, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbea518, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ea51e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb7dea520, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x53fea522, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dea525, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9ea529, SUC_lexicon_tags_28 },
    { 0x309ea52a, SUC_lexicon_tags_37 },
    { 0x767ea52b, SUC_lexicon_tags_25 },
    { 0x891ea52c, SUC_lexicon_tags_11 },
    { 0xd2bea52a, SUC_lexicon_tags_0 },
    { 0x37ea52e, SUC_lexicon_tags_2 },
    { 0xdc5ea52f, SUC_lexicon_tags_17 },
    { 0x21ea52f, SUC_lexicon_tags_30 },
    { 0xa2bea52e, SUC_lexicon_tags_53 },
    { 0x361ea52c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9ea536, SUC_lexicon_tags_17 },
    { 0x52dea536, SUC_lexicon_tags_84 },
    { 0x93dea538, SUC_lexicon_tags_37 },
    { 0x38fea536, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fea53c, SUC_lexicon_tags_46 },
    { 0xf93ea53c, SUC_lexicon_tags_12 },
    { 0x94bea53e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715ea546, SUC_lexicon_tags_49 },
    { 0xd73ea546, SUC_lexicon_tags_49 },
    { 0x53fea546, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0fea54a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ea54d, SUC_lexicon_tags_11 },
    { 0xfc5ea54d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xda5ea550, SUC_lexicon_tags_11 },
    { 0x3f3ea551, SUC_lexicon_tags_45 },
    { 0xf65ea551, SUC_lexicon_tags_37 },
    { 0xf91ea553, SUC_lexicon_tags_17 },
    { 0x979ea553, SUC_lexicon_tags_0 },
    { 0xf07ea555, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xa43ea557, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfea560, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x59ea562, SUC_lexicon_tags_55 },
    { 0x933ea562, SUC_lexicon_tags_36 },
    { 0xe0fea564, SUC_lexicon_tags_19 },
    { 0x74bea565, SUC_lexicon_tags_25 },
    { 0x535ea566, SUC_lexicon_tags_45 },
    { 0xee1ea567, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfea56b, SUC_lexicon_tags_11 },
    { 0x943ea56c, SUC_lexicon_tags_2 },
    { 0x259ea56c, SUC_lexicon_tags_1 },
    { 0xac9ea56e, SUC_lexicon_tags_17 },
    { 0x567ea56b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ea572, SUC_lexicon_tags_66 },
    { 0xb3ea573, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7ea578, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ea57b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99ea57e, SUC_lexicon_tags_7 },
    { 0x859ea57e, SUC_lexicon_tags_12 },
    { 0xc25ea580, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353ea588, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1ffea58a, SUC_lexicon_tags_77 },
    { 0x681ea58a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dea58f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865ea596, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bea599, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3ea5a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bea5aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ea5b0, SUC_lexicon_tags_47 },
    { 0xb6dea5b1, SUC_lexicon_tags_23 },
    { 0x3a5ea5b0, SUC_lexicon_tags_45 },
    { 0xe19ea5b2, SUC_lexicon_tags_6 },
    { 0x9a9ea5b4, SUC_lexicon_tags_0 },
    { 0x69bea5b2, SUC_lexicon_tags_81 },
    { 0xd67ea5b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8f7ea5b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fea5c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89ea5c4, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9ea5c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x51dea5ca, SUC_lexicon_tags_33 },
    { 0x861ea5ca, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195ea5d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5ea5d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x949ea5d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1cdea5da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7bea5dc, SUC_lexicon_tags_30 },
    { 0x49fea5dd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1ea5e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407ea5e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x803ea5e7, SUC_lexicon_tags_0 },
    { 0x2bfea5e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7ea5ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23ea5ef, SUC_lexicon_tags_37 },
    { 0x753ea5ef, SUC_lexicon_tags_47 },
    { 0xdc7ea5f0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x34fea5f3, SUC_lexicon_tags_0 },
    { 0x48bea5f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dea5fa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x48dea5fc, SUC_lexicon_tags_28 },
    { 0xbfea5fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bea601, SUC_lexicon_tags_17 },
    { 0xab9ea601, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09ea60b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa3dea60d, SUC_lexicon_tags_1 },
    { 0x11ea60d, SUC_lexicon_tags_45 },
    { 0xe9dea60e, SUC_lexicon_tags_12 },
    { 0xb5bea60f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe2dea612, SUC_lexicon_tags_15 },
    { 0xbb5ea612, SUC_lexicon_tags_37 },
    { 0x2d5ea612, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9ea618, SUC_lexicon_tags_47 },
    { 0x2bbea619, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x825ea61b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbea622, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493ea626, SUC_lexicon_tags_11 },
    { 0xfea627, SUC_lexicon_tags_56 },
    { 0xac9ea628, SUC_lexicon_tags_39 },
    { 0x149ea629, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93ea62d, SUC_lexicon_tags_7 },
    { 0xcdea62e, SUC_lexicon_tags_82 },
    { 0x87ea62d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5ea635, SUC_lexicon_tags_7 },
    { 0x855ea636, SUC_lexicon_tags_15 },
    { 0x7fbea636, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dea641, SUC_lexicon_tags_11 },
    { 0x707ea642, SUC_lexicon_tags_0 },
    { 0x565ea643, SUC_lexicon_tags_17 },
    { 0x9cdea644, SUC_lexicon_tags_9 },
    { 0x31bea644, SUC_lexicon_tags_37 },
    { 0x93fea646, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x47bea648, SUC_lexicon_tags_1 },
    { 0x49ea649, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4a7ea64b, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0xa3ea64d, SUC_lexicon_tags_8 },
    { 0xab5ea64e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x819ea650, SUC_lexicon_tags_38 },
    { 0xba3ea650, SUC_lexicon_tags_5 },
    { 0x96dea651, SUC_lexicon_tags_59 },
    { 0x15bea653, SUC_lexicon_tags_15 },
    { 0xcc7ea654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a9ea656, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x13bea659, SUC_lexicon_tags_29 },
    { 0xd99ea65a, SUC_lexicon_tags_1 },
    { 0x955ea659, SUC_lexicon_tags_6 },
    { 0x97dea65a, SUC_lexicon_tags_37 },
    { 0xf6fea65c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bea666, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bea66a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2ebea66c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8c7ea66e, SUC_lexicon_tags_82 },
    { 0x22bea66f, SUC_lexicon_tags_25 },
    { 0xbd5ea66f, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ea676, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577ea67d, SUC_lexicon_tags_91 },
    { 0x76bea67e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x95dea680, SUC_lexicon_tags_38 },
    { 0x643ea680, SUC_lexicon_tags_37 },
    { 0xaf1ea681, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6dfea684, SUC_lexicon_tags_33 },
    { 0x71fea685, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2f3ea687, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dea68f, SUC_lexicon_tags_1 },
    { 0xc7ea690, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x267ea692, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9ea697, SUC_lexicon_tags_7 },
    { 0xc79ea698, SUC_lexicon_tags_4 },
    { 0xd09ea699, SUC_lexicon_tags_39 },
    { 0x755ea699, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bea69c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7ea6a0, SUC_lexicon_tags_2 },
    { 0x6a1ea6a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7ea6a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac1ea6a7, SUC_lexicon_tags_1 },
    { 0x387ea6a7, SUC_lexicon_tags_14 },
    { 0x31ea6a8, SUC_lexicon_tags_66 },
    { 0xf87ea6a9, SUC_lexicon_tags_1 },
    { 0xde5ea6a8, SUC_lexicon_tags_55 },
    { 0x43ea6a8, SUC_lexicon_tags_66 },
    { 0xa15ea6ad, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0xaebea6af, SUC_lexicon_tags_79 },
    { 0x9efea6af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa7fea6b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1ea6b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69dea6c3, SUC_lexicon_tags_47 },
    { 0x34bea6c4, SUC_lexicon_tags_29 },
    { 0x28dea6c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5ea6cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2ebea6cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49ea6cf, SUC_lexicon_tags_2 },
    { 0x1a9ea6d0, SUC_lexicon_tags_12 },
    { 0xf63ea6d0, SUC_lexicon_tags_37 },
    { 0x49dea6d0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9ea6d5, SUC_lexicon_tags_33 },
    { 0xb4bea6d6, SUC_lexicon_tags_87 },
    { 0x119ea6d7, SUC_lexicon_tags_82 },
    { 0x397ea6d8, SUC_lexicon_tags_28 },
    { 0x88bea6d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dea6df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547ea6e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29ea6e7, SUC_lexicon_tags_26 },
    { 0xe55ea6e7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x195ea6eb, SUC_lexicon_tags_70 },
    { 0xf47ea6eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x953ea6ef, SUC_lexicon_tags_46 },
    { 0x467ea6f0, SUC_lexicon_tags_26 },
    { 0x49fea6f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1ea6f6, SUC_lexicon_tags_45 },
    { 0x5a5ea6f7, SUC_lexicon_tags_47 },
    { 0x6adea6f8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x281ea6fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x32bea6fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fea700, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x269ea703, SUC_lexicon_tags_35 },
    { 0xca7ea703, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9ea70b, SUC_lexicon_tags_63 },
    { 0xf9dea70c, SUC_lexicon_tags_23 },
    { 0x221ea70b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x679ea70f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10fea716, SUC_lexicon_tags_44 },
    { 0xfb5ea716, SUC_lexicon_tags_12 },
    { 0x927ea718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaabea71a, SUC_lexicon_tags_11 },
    { 0x9e9ea71b, SUC_lexicon_tags_22 },
    { 0xb89ea71c, SUC_lexicon_tags_25 },
    { 0x19bea71c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddea720, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5d1ea722, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ea725, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667ea729, SUC_lexicon_tags_66 },
    { 0xaa5ea729, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5ea72e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabea732, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x833ea734, SUC_lexicon_tags_17 },
    { 0x193ea735, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2adea73f, SUC_lexicon_tags_6 },
    { 0xcbea73f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93fea744, SUC_lexicon_tags_343 },
    { 0x577ea744, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x18fea747, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4afea749, SUC_lexicon_tags_17 },
    { 0x21dea749, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bea74d, SUC_lexicon_tags_17 },
    { 0x201ea74e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5bbea750, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2e3ea752, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x24dea755, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x323ea758, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57ea75d, SUC_lexicon_tags_29 },
    { 0xf67ea75d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x26fea760, SUC_lexicon_tags_26 },
    { 0x84fea760, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbc9ea763, SUC_lexicon_tags_77 },
    { 0x3b1ea764, SUC_lexicon_tags_15 },
    { 0xdd3ea764, SUC_lexicon_tags_28 },
    { 0x74fea766, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc33ea768, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbc7ea76a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665ea76f, SUC_lexicon_tags_7 },
    { 0xae7ea76f, SUC_lexicon_tags_0 },
    { 0xa3dea770, SUC_lexicon_tags_12 },
    { 0x305ea76f, SUC_lexicon_tags_45 },
    { 0x943ea76f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x823ea776, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xda7ea778, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fea77c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317ea782, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc77ea784, SUC_lexicon_tags_11 },
    { 0x7bbea784, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6bbea787, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41ea78f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fea796, SUC_lexicon_tags_17 },
    { 0x51dea797, SUC_lexicon_tags_37 },
    { 0xcffea798, SUC_lexicon_tags_47 },
    { 0x5a3ea799, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x549ea79b, SUC_lexicon_tags_17 },
    { 0xb67ea79b, SUC_lexicon_tags_12 },
    { 0x387ea79d, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0x1bbea79f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603ea7a3, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbea7a7, SUC_lexicon_tags_32 },
    { 0x5d3ea7a8, SUC_lexicon_tags_25 },
    { 0x561ea7a8, SUC_lexicon_tags_26 },
    { 0x9afea7aa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadea7b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ea7b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fea7b6, SUC_lexicon_tags_46 },
    { 0xdfbea7b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ea7bb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4ebea7bd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fea7c4, SUC_lexicon_tags_17 },
    { 0xc15ea7c5, SUC_lexicon_tags_77 },
    { 0x6f5ea7c4, SUC_lexicon_tags_0 },
    { 0xd5fea7c7, SUC_lexicon_tags_2 },
    { 0x529ea7c8, SUC_lexicon_tags_12 },
    { 0xe41ea7c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869ea7ce, SUC_lexicon_tags_27 },
    { 0x961ea7cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f3ea7d1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x38dea7d3, SUC_lexicon_tags_15 },
    { 0x599ea7d4, SUC_lexicon_tags_11 },
    { 0x215ea7d3, SUC_lexicon_tags_46 },
    { 0x26bea7d6, SUC_lexicon_tags_7 },
    { 0xa41ea7d7, SUC_lexicon_tags_37 },
    { 0x86dea7d7, SUC_lexicon_tags_68 },
    { 0x779ea7d7, SUC_lexicon_tags_17 },
    { 0x547ea7da, SUC_lexicon_tags_1 },
    { 0x453ea7db, SUC_lexicon_tags_47 },
    { 0x3bbea7d9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x3b9ea7de, SUC_lexicon_tags_47 },
    { 0x727ea7de, SUC_lexicon_tags_19 },
    { 0x7ddea7e0, SUC_lexicon_tags_33 },
    { 0x46fea7e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793ea7ea, SUC_lexicon_tags_25 },
    { 0x19fea7eb, SUC_lexicon_tags_33 },
    { 0xe7dea7ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bea7f3, SUC_lexicon_tags_7 },
    { 0x1bbea7f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5ea7fb, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x495ea7fd, SUC_lexicon_tags_17 },
    { 0x13dea7fe, SUC_lexicon_tags_1 },
    { 0xc13ea7fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5afea801, SUC_lexicon_tags_170 },
    { 0x4adea802, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x747ea804, SUC_lexicon_tags_37 },
    { 0x917ea804, SUC_lexicon_tags_82 },
    { 0x6bea806, SUC_lexicon_tags_45 },
    { 0x3b5ea807, SUC_lexicon_tags_55 },
    { 0xa4fea808, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x713ea80b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x65fea80e, SUC_lexicon_tags_1 },
    { 0x115ea80f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ea812, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2bbea814, SUC_lexicon_tags_34 },
    { 0x67ea815, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x209ea817, SUC_lexicon_tags_7 },
    { 0x7abea818, SUC_lexicon_tags_17 },
    { 0x28bea819, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x415ea81b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799ea821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6f5ea823, SUC_lexicon_tags_25 },
    { 0x133ea823, SUC_lexicon_tags_17 },
    { 0xd99ea825, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc4bea827, SUC_lexicon_tags_31 },
    { 0xe0fea828, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fea82e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fea836, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefea83a, SUC_lexicon_tags_84 },
    { 0x883ea83a, SUC_lexicon_tags_16 },
    { 0xeafea83a, SUC_lexicon_tags_12 },
    { 0x361ea83b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3dea83f, SUC_lexicon_tags_1 },
    { 0x87ea83f, SUC_lexicon_tags_68 },
    { 0x155ea841, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbea844, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253ea84b, SUC_lexicon_tags_8 },
    { 0x3afea84b, SUC_lexicon_tags_23 },
    { 0x925ea84d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b3ea84f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c7ea851, SUC_lexicon_tags_1 },
    { 0x8ffea851, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ea85c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf79ea85e, SUC_lexicon_tags_14 },
    { 0x8ddea85e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x4ddea861, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bea864, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9ea869, SUC_lexicon_tags_11 },
    { 0xcd7ea869, SUC_lexicon_tags_19 },
    { 0x73ea86a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x73fea86d, SUC_lexicon_tags_6 },
    { 0x135ea86e, SUC_lexicon_tags_1 },
    { 0x4f5ea86e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5ea872, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x465ea875, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74dea87b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b1ea87d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2fea880, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0x4cfea882, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb4fea884, SUC_lexicon_tags_37 },
    { 0xba3ea884, SUC_lexicon_tags_47 },
    { 0x839ea886, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fea88c, SUC_lexicon_tags_28 },
    { 0x68dea88c, SUC_lexicon_tags_27 },
    { 0x227ea88e, SUC_lexicon_tags_15 },
    { 0x1c7ea88e, SUC_lexicon_tags_7 },
    { 0x49ea890, SUC_lexicon_tags_2 },
    { 0x147ea890, SUC_lexicon_tags_15 },
    { 0x167ea890, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd3ea894, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebea89b, SUC_lexicon_tags_0 },
    { 0x4ddea89b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edea8a0, SUC_lexicon_tags_0 },
    { 0x431ea8a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ea8a5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7ea8a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109ea8ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3fea8af, SUC_lexicon_tags_25 },
    { 0x95bea8b0, SUC_lexicon_tags_23 },
    { 0x5dbea8b0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9ea8b4, SUC_lexicon_tags_23 },
    { 0x5f5ea8b5, SUC_lexicon_tags_25 },
    { 0x807ea8b6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bea8bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73ea8be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffea8c3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965ea8c7, SUC_lexicon_tags_3 },
    { 0x279ea8c7, SUC_lexicon_tags_37 },
    { 0xa7fea8c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9ea8cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x27fea8cf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4cfea8d1, SUC_lexicon_tags_1 },
    { 0xbbdea8d2, SUC_lexicon_tags_37 },
    { 0x663ea8d3, SUC_lexicon_tags_37 },
    { 0xb6bea8d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x281ea8d7, SUC_lexicon_tags_25 },
    { 0xef1ea8d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xba5ea8da, SUC_lexicon_tags_0 },
    { 0x4fdea8db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3ea8de, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8fbea8e0, SUC_lexicon_tags_17 },
    { 0xb25ea8e1, SUC_lexicon_tags_47 },
    { 0xedfea8e2, SUC_lexicon_tags_49 },
    { 0x8ebea8e2, SUC_lexicon_tags_37 },
    { 0xa91ea8e2, SUC_lexicon_tags_1 },
    { 0xd1fea8e1, SUC_lexicon_tags_48 },
    { 0x6bbea8e6, SUC_lexicon_tags_2 },
    { 0x311ea8e6, SUC_lexicon_tags_1 },
    { 0x423ea8e8, SUC_lexicon_tags_49 },
    { 0x9d9ea8e7, SUC_lexicon_tags_15 },
    { 0xe7bea8ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7ea8f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe17ea8f5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bea8f9, SUC_lexicon_tags_45 },
    { 0xe9bea8fa, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfea8fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x919ea900, SUC_lexicon_tags_66 },
    { 0xd29ea901, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177ea90b, SUC_lexicon_tags_25 },
    { 0xce3ea90c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55ea911, SUC_lexicon_tags_25 },
    { 0x8e1ea912, SUC_lexicon_tags_11 },
    { 0x16dea912, SUC_lexicon_tags_17 },
    { 0x3e9ea914, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7ea918, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57ea91c, SUC_lexicon_tags_49 },
    { 0x6afea91c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb3fea91f, SUC_lexicon_tags_25 },
    { 0x1e3ea920, SUC_lexicon_tags_2 },
    { 0x6cdea920, SUC_lexicon_tags_16 },
    { 0xdf1ea921, SUC_lexicon_tags_6 },
    { 0x6c9ea923, SUC_lexicon_tags_17 },
    { 0xc27ea924, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3ea927, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x4c3ea929, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87ea92d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x531ea930, SUC_lexicon_tags_23 },
    { 0xf65ea931, SUC_lexicon_tags_25 },
    { 0xf43ea932, SUC_lexicon_tags_37 },
    { 0x7b7ea933, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x897ea936, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x233ea939, SUC_lexicon_tags_43 },
    { 0x44bea93a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x19bea93c, SUC_lexicon_tags_31 },
    { 0xe95ea93d, SUC_lexicon_tags_25 },
    { 0x55ea93e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275ea942, SUC_lexicon_tags_14 },
    { 0x75dea943, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8b5ea945, SUC_lexicon_tags_31 },
    { 0x9fbea945, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9ea94c, SUC_lexicon_tags_33 },
    { 0x915ea94d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xad3ea94f, SUC_lexicon_tags_28 },
    { 0xf1ea950, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f1ea952, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x543ea955, SUC_lexicon_tags_26 },
    { 0x835ea956, SUC_lexicon_tags_56 },
    { 0xf1bea957, SUC_lexicon_tags_68 },
    { 0x18fea958, SUC_lexicon_tags_7 },
    { 0xc5ea959, SUC_lexicon_tags_6 },
    { 0x5e5ea95a, SUC_lexicon_tags_31 },
    { 0x36dea95a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x4b7ea95d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x295ea95f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6a3ea961, SUC_lexicon_tags_2 },
    { 0xc7bea961, SUC_lexicon_tags_4 },
    { 0x2b3ea963, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe8dea965, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3ea969, SUC_lexicon_tags_37 },
    { 0xc8fea96a, SUC_lexicon_tags_47 },
    { 0x99ea96b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5ea96e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4cfea970, SUC_lexicon_tags_1 },
    { 0x961ea971, SUC_lexicon_tags_37 },
    { 0x19dea972, SUC_lexicon_tags_65 },
    { 0xb51ea971, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc2fea975, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1e7ea977, SUC_lexicon_tags_7 },
    { 0xafdea978, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb5ea97a, SUC_lexicon_tags_103 },
    { 0x157ea97a, SUC_lexicon_tags_37 },
    { 0xa23ea97c, SUC_lexicon_tags_33 },
    { 0xb5bea97c, SUC_lexicon_tags_25 },
    { 0x68bea97d, SUC_lexicon_tags_7 },
    { 0x65ea97e, SUC_lexicon_tags_17 },
    { 0xabbea980, SUC_lexicon_tags_2 },
    { 0xc83ea981, SUC_lexicon_tags_43 },
    { 0x2c1ea982, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x599ea985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x687ea987, SUC_lexicon_tags_47 },
    { 0x6a7ea987, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x739ea98b, SUC_lexicon_tags_49 },
    { 0xb31ea98b, SUC_lexicon_tags_17 },
    { 0x815ea98d, SUC_lexicon_tags_7 },
    { 0x3ffea98e, SUC_lexicon_tags_15 },
    { 0x655ea98e, SUC_lexicon_tags_1 },
    { 0x6e3ea98f, SUC_lexicon_tags_8 },
    { 0x969ea991, SUC_lexicon_tags_4 },
    { 0xba7ea991, SUC_lexicon_tags_17 },
    { 0x3f9ea990, SUC_lexicon_tags_49 },
    { 0x5adea98c, SUC_lexicon_tags_17 },
    { 0xad1ea995, SUC_lexicon_tags_12 },
    { 0x1f9ea995, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afea99f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbea9a6, SUC_lexicon_tags_25 },
    { 0xc69ea9a7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc1bea9a9, SUC_lexicon_tags_17 },
    { 0xbcdea9aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x525ea9ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abea9b3, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xa41ea9b5, SUC_lexicon_tags_0 },
    { 0x527ea9b6, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0, NULL },
    { 0x65bea9b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7ea9bf, SUC_lexicon_tags_38 },
    { 0x3e5ea9bf, SUC_lexicon_tags_43 },
    { 0x8ffea9c1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8fea9c3, SUC_lexicon_tags_30 },
    { 0xf87ea9c4, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xffbea9c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0fea9c9, SUC_lexicon_tags_37 },
    { 0xa2dea9c9, SUC_lexicon_tags_15 },
    { 0x1bdea9c9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbea9d4, SUC_lexicon_tags_11 },
    { 0x6bea9d5, SUC_lexicon_tags_17 },
    { 0x657ea9d5, SUC_lexicon_tags_12 },
    { 0xd23ea9d4, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x547ea9da, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50fea9e0, SUC_lexicon_tags_60 },
    { 0x191ea9e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6ddea9e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577ea9e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdea9ec, SUC_lexicon_tags_45 },
    { 0xedea9ed, SUC_lexicon_tags_49 },
    { 0x485ea9ee, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bea9f7, SUC_lexicon_tags_28 },
    { 0x2c7ea9f8, SUC_lexicon_tags_11 },
    { 0x6c3ea9f8, SUC_lexicon_tags_17 },
    { 0x9adea9f8, SUC_lexicon_tags_45 },
    { 0xd33ea9fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421eaa00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x60beaa02, SUC_lexicon_tags_17 },
    { 0x9eaa02, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1eaa08, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x357eaa0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x271eaa0c, SUC_lexicon_tags_1 },
    { 0xd3feaa0c, SUC_lexicon_tags_37 },
    { 0x1d3eaa0c, SUC_lexicon_tags_12 },
    { 0x4e9eaa0f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3eaa14, SUC_lexicon_tags_37 },
    { 0xc51eaa15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x19deaa18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4beaa1b, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9beaa1e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3eaa22, SUC_lexicon_tags_11 },
    { 0xf95eaa23, SUC_lexicon_tags_14 },
    { 0xdcbeaa23, SUC_lexicon_tags_33 },
    { 0x5edeaa24, SUC_lexicon_tags_15 },
    { 0x981eaa26, SUC_lexicon_tags_65 },
    { 0xf57eaa27, SUC_lexicon_tags_25 },
    { 0x48deaa28, SUC_lexicon_tags_82 },
    { 0x493eaa29, SUC_lexicon_tags_14 },
    { 0xf13eaa26, SUC_lexicon_tags_7 },
    { 0xd45eaa24, SUC_lexicon_tags_17 },
    { 0x2dbeaa2c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2feaa33, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693eaa37, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69deaa39, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38feaa43, SUC_lexicon_tags_37 },
    { 0x333eaa44, SUC_lexicon_tags_12 },
    { 0x72beaa45, SUC_lexicon_tags_30 },
    { 0xa7deaa46, SUC_lexicon_tags_39 },
    { 0xea5eaa44, SUC_lexicon_tags_30 },
    { 0x1a3eaa48, SUC_lexicon_tags_47 },
    { 0x98feaa49, SUC_lexicon_tags_6 },
    { 0x22beaa43, SUC_lexicon_tags_11 },
    { 0x1fdeaa4b, SUC_lexicon_tags_17 },
    { 0x363eaa4b, SUC_lexicon_tags_3 },
    { 0x401eaa4d, SUC_lexicon_tags_15 },
    { 0x9dfeaa4d, SUC_lexicon_tags_43 },
    { 0x891eaa4c, SUC_lexicon_tags_87 },
    { 0xa3beaa47, SUC_lexicon_tags_49 },
    { 0xf9eaa4b, SUC_lexicon_tags_1 },
    { 0x413eaa4f, SUC_lexicon_tags_33 },
    { 0x151eaa53, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd4feaa55, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33eaa64, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x72deaa66, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x145eaa68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23eaa6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871eaa76, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd03eaa78, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfeaa7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa67eaa7e, SUC_lexicon_tags_15 },
    { 0xffeaa7f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523eaa83, SUC_lexicon_tags_39 },
    { 0xed1eaa84, SUC_lexicon_tags_30 },
    { 0x9f7eaa85, SUC_lexicon_tags_19 },
    { 0x8d1eaa86, SUC_lexicon_tags_27 },
    { 0x3dfeaa87, SUC_lexicon_tags_82 },
    { 0xffdeaa83, SUC_lexicon_tags_19 },
    { 0xe3deaa87, SUC_lexicon_tags_47 },
    { 0x9b9eaa87, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdeaa90, SUC_lexicon_tags_31 },
    { 0xc1feaa90, SUC_lexicon_tags_12 },
    { 0x9ebeaa92, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe63eaa96, SUC_lexicon_tags_0 },
    { 0x8beaa97, SUC_lexicon_tags_7 },
    { 0x1c9eaa98, SUC_lexicon_tags_15 },
    { 0x8b9eaa97, SUC_lexicon_tags_496 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255eaaa3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15eaaa7, SUC_lexicon_tags_8 },
    { 0x419eaaa8, SUC_lexicon_tags_41 },
    { 0xd1beaaa9, SUC_lexicon_tags_25 },
    { 0x1e7eaaaa, SUC_lexicon_tags_5 },
    { 0xfe1eaaab, SUC_lexicon_tags_9 },
    { 0xa8beaaac, SUC_lexicon_tags_16 },
    { 0x587eaaac, SUC_lexicon_tags_63 },
    { 0x205eaaac, SUC_lexicon_tags_57 },
    { 0x2d1eaaaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaefeaab1, SUC_lexicon_tags_23 },
    { 0x6dbeaab2, SUC_lexicon_tags_1 },
    { 0x73eaab2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebeaab8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfeaabc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe81eaabe, SUC_lexicon_tags_6 },
    { 0xab9eaabf, SUC_lexicon_tags_3 },
    { 0xf1eaabe, SUC_lexicon_tags_2 },
    { 0x111eaac1, SUC_lexicon_tags_28 },
    { 0x903eaabe, SUC_lexicon_tags_0 },
    { 0x171eaac3, SUC_lexicon_tags_2 },
    { 0xc9eaabf, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x883eaac7, SUC_lexicon_tags_27 },
    { 0x371eaac8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3b5eaaca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2deaacd, SUC_lexicon_tags_25 },
    { 0x315eaace, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa05eaad0, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7eaad5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13eaad8, SUC_lexicon_tags_16 },
    { 0xb3eaad9, SUC_lexicon_tags_25 },
    { 0xc7feaad9, SUC_lexicon_tags_37 },
    { 0x911eaadb, SUC_lexicon_tags_27 },
    { 0xad7eaad9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x497eaade, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2feaae2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb3beaae4, SUC_lexicon_tags_49 },
    { 0xb9eaae5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x865eaae7, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89eaaea, SUC_lexicon_tags_68 },
    { 0xf8beaaeb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfeaaf1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf87eaaf3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf17eaaf6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdeaafb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabeaafe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbeab01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a3eab03, SUC_lexicon_tags_49 },
    { 0xd45eab04, SUC_lexicon_tags_25 },
    { 0xd7eab05, SUC_lexicon_tags_7 },
    { 0x7beab06, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7eab0d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0xdeab0f, SUC_lexicon_tags_7 },
    { 0xc03eab0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xba9eab12, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1eab16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc5beab18, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedeab1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd1beab1e, SUC_lexicon_tags_9 },
    { 0x3c1eab1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29deab23, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x46deab25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefeab29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3eab2c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7eab31, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xfcdeab33, SUC_lexicon_tags_37 },
    { 0x7bfeab34, SUC_lexicon_tags_15 },
    { 0xd4beab33, SUC_lexicon_tags_63 },
    { 0xcf3eab36, SUC_lexicon_tags_30 },
    { 0x1b1eab36, SUC_lexicon_tags_911 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3eab3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837eab41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x93beab44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44beab4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfb7eab4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409eab58, SUC_lexicon_tags_12 },
    { 0x8c9eab58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x39deab5c, SUC_lexicon_tags_23 },
    { 0x4a9eab5c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9eab60, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x79beab62, SUC_lexicon_tags_25 },
    { 0xe7feab63, SUC_lexicon_tags_17 },
    { 0xe8deab64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x49feab67, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919eab6c, SUC_lexicon_tags_2 },
    { 0xd9deab6d, SUC_lexicon_tags_1 },
    { 0x9eab6e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x579eab70, SUC_lexicon_tags_28 },
    { 0x42deab71, SUC_lexicon_tags_11 },
    { 0x231eab72, SUC_lexicon_tags_19 },
    { 0xeddeab73, SUC_lexicon_tags_2 },
    { 0xd65eab71, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55eab7a, SUC_lexicon_tags_25 },
    { 0xf03eab7b, SUC_lexicon_tags_10 },
    { 0xd77eab7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9beab82, SUC_lexicon_tags_15 },
    { 0xea3eab83, SUC_lexicon_tags_11 },
    { 0x217eab84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdeab87, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbafeab8a, SUC_lexicon_tags_1 },
    { 0x621eab8a, SUC_lexicon_tags_37 },
    { 0x5d3eab8a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa81eab8e, SUC_lexicon_tags_27 },
    { 0x329eab8e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eab92, SUC_lexicon_tags_15 },
    { 0x9efeab93, SUC_lexicon_tags_33 },
    { 0x27beab93, SUC_lexicon_tags_11 },
    { 0x9a1eab93, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x923eab97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68beab99, SUC_lexicon_tags_2 },
    { 0xbbeab99, SUC_lexicon_tags_6 },
    { 0x31feab9b, SUC_lexicon_tags_7 },
    { 0xb8beab9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1feab9f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35feaba4, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x54feaba7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebeabb2, SUC_lexicon_tags_34 },
    { 0x2d5eabb3, SUC_lexicon_tags_56 },
    { 0x5a1eabb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x629eabb6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xecdeabb9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1eabbc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfefeabbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1cfeabc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x729eabc3, SUC_lexicon_tags_79 },
    { 0x235eabc3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5eabc8, SUC_lexicon_tags_56 },
    { 0x989eabc8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x277eabcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x89beabce, SUC_lexicon_tags_2 },
    { 0xacfeabcf, SUC_lexicon_tags_7 },
    { 0xdedeabd0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x211eabd3, SUC_lexicon_tags_17 },
    { 0xccbeabd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbeabdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7deabde, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfeabe4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3eabe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x259eabe9, SUC_lexicon_tags_37 },
    { 0x263eabea, SUC_lexicon_tags_19 },
    { 0x8beabeb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9eabef, SUC_lexicon_tags_63 },
    { 0x311eabf0, SUC_lexicon_tags_38 },
    { 0xcafeabf1, SUC_lexicon_tags_12 },
    { 0xba3eabf1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xefeabf4, SUC_lexicon_tags_33 },
    { 0x283eabf4, SUC_lexicon_tags_25 },
    { 0xb7eabf6, SUC_lexicon_tags_35 },
    { 0x36deabf7, SUC_lexicon_tags_6 },
    { 0x6e9eabf8, SUC_lexicon_tags_65 },
    { 0xc37eabf8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66deabfb, SUC_lexicon_tags_17 },
    { 0x6f1eabfc, SUC_lexicon_tags_2 },
    { 0xcb3eabfd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31eac00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfeac05, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817eac0a, SUC_lexicon_tags_77 },
    { 0xd81eac0b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535eac10, SUC_lexicon_tags_158 },
    { 0x2dfeac11, SUC_lexicon_tags_27 },
    { 0x5ffeac12, SUC_lexicon_tags_47 },
    { 0xa41eac13, SUC_lexicon_tags_25 },
    { 0x895eac12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd83eac16, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbeac1a, SUC_lexicon_tags_627 },
    { 0x66deac1b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79deac1f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eac24, SUC_lexicon_tags_37 },
    { 0x64deac25, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4deac28, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87eac34, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x4a5eac36, SUC_lexicon_tags_77 },
    { 0xdeac37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959eac3b, SUC_lexicon_tags_82 },
    { 0xadbeac3c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781eac43, SUC_lexicon_tags_12 },
    { 0xa81eac44, SUC_lexicon_tags_7 },
    { 0xae3eac45, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01eac49, SUC_lexicon_tags_11 },
    { 0xb5deac4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99beac4d, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64deac55, SUC_lexicon_tags_2 },
    { 0xbf3eac56, SUC_lexicon_tags_37 },
    { 0xd71eac56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd55eac59, SUC_lexicon_tags_23 },
    { 0xd07eac59, SUC_lexicon_tags_30 },
    { 0x1dbeac5b, SUC_lexicon_tags_9 },
    { 0xb85eac5c, SUC_lexicon_tags_17 },
    { 0xd5beac5c, SUC_lexicon_tags_10 },
    { 0x5bfeac5e, SUC_lexicon_tags_37 },
    { 0x151eac5e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa53eac61, SUC_lexicon_tags_17 },
    { 0x497eac61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb17eac64, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3c1eac66, SUC_lexicon_tags_2 },
    { 0x5cdeac67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x27eac69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12deac6b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xff3eac6d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41feac72, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935eac77, SUC_lexicon_tags_38 },
    { 0xbd5eac77, SUC_lexicon_tags_7 },
    { 0x6edeac78, SUC_lexicon_tags_15 },
    { 0x55feac7a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x29beac7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89eac81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfeac84, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbeac91, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc43eac93, SUC_lexicon_tags_25 },
    { 0x215eac94, SUC_lexicon_tags_2 },
    { 0x493eac95, SUC_lexicon_tags_2 },
    { 0xe4deac96, SUC_lexicon_tags_45 },
    { 0xd87eac97, SUC_lexicon_tags_25 },
    { 0x6e1eac96, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749eac9c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83beaca0, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3beaca3, SUC_lexicon_tags_47 },
    { 0xd15eaca3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07eaca7, SUC_lexicon_tags_32 },
    { 0x26deaca8, SUC_lexicon_tags_79 },
    { 0xd3deaca8, SUC_lexicon_tags_11 },
    { 0xb19eacaa, SUC_lexicon_tags_38 },
    { 0xa9feaca8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfa3eacad, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291eacb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x557eacb4, SUC_lexicon_tags_136 },
    { 0x30deacb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5eacbb, SUC_lexicon_tags_26 },
    { 0x4c1eacbb, SUC_lexicon_tags_33 },
    { 0x53feacbd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1feacbf, SUC_lexicon_tags_27 },
    { 0x311eacbf, SUC_lexicon_tags_7 },
    { 0xad1eacc1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17beacc6, SUC_lexicon_tags_13 },
    { 0x61eacc7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81feacca, SUC_lexicon_tags_43 },
    { 0x231eacca, SUC_lexicon_tags_27 },
    { 0x1afeaccb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x245eaccf, SUC_lexicon_tags_13 },
    { 0x2cdeaccf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4cfeacd2, SUC_lexicon_tags_27 },
    { 0x39eacd3, SUC_lexicon_tags_0 },
    { 0x47beacd4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17feacd8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebeacdd, SUC_lexicon_tags_12 },
    { 0xe2beacde, SUC_lexicon_tags_8 },
    { 0x76beacdf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31eace5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xcebeace7, SUC_lexicon_tags_7 },
    { 0xa71eace7, SUC_lexicon_tags_23 },
    { 0x28feace7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873eaced, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359eacf1, SUC_lexicon_tags_63 },
    { 0x893eacf2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99eacf6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xab3eacf8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb75eacfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6adeacfc, SUC_lexicon_tags_31 },
    { 0x565eacfc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c5eacff, SUC_lexicon_tags_11 },
    { 0xa67ead00, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915ead06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8fead08, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbbead0a, SUC_lexicon_tags_49 },
    { 0x9abead0b, SUC_lexicon_tags_17 },
    { 0xf89ead0c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19ead12, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddead16, SUC_lexicon_tags_25 },
    { 0x52dead16, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf11ead19, SUC_lexicon_tags_15 },
    { 0xfddead1a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x821ead1d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf9bead1f, SUC_lexicon_tags_3 },
    { 0x327ead20, SUC_lexicon_tags_12 },
    { 0x60dead21, SUC_lexicon_tags_179 },
    { 0x95dead22, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62bead24, SUC_lexicon_tags_15 },
    { 0x5d9ead25, SUC_lexicon_tags_2 },
    { 0x8efead26, SUC_lexicon_tags_13 },
    { 0x537ead27, SUC_lexicon_tags_61 },
    { 0x57dead28, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329ead2e, SUC_lexicon_tags_5 },
    { 0x92dead2f, SUC_lexicon_tags_9 },
    { 0x68fead30, SUC_lexicon_tags_17 },
    { 0x90dead30, SUC_lexicon_tags_65 },
    { 0xe8fead31, SUC_lexicon_tags_61 },
    { 0xc0dead2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837ead41, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3fead45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc23ead47, SUC_lexicon_tags_55 },
    { 0xe29ead48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d1ead4a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbead52, SUC_lexicon_tags_37 },
    { 0x7abead52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd05ead55, SUC_lexicon_tags_25 },
    { 0x259ead55, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fead59, SUC_lexicon_tags_15 },
    { 0x303ead5a, SUC_lexicon_tags_26 },
    { 0x7a9ead59, SUC_lexicon_tags_0 },
    { 0x14dead5c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x55ead5e, SUC_lexicon_tags_11 },
    { 0x8e3ead5f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f7ead61, SUC_lexicon_tags_144 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65ead68, SUC_lexicon_tags_57 },
    { 0xda5ead69, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161ead70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dead74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dead77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x897ead7a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xaf3ead7c, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fead7f, SUC_lexicon_tags_0 },
    { 0x34fead80, SUC_lexicon_tags_17 },
    { 0xa39ead80, SUC_lexicon_tags_45 },
    { 0x345ead80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe9fead84, SUC_lexicon_tags_15 },
    { 0x3a5ead85, SUC_lexicon_tags_14 },
    { 0xfe9ead85, SUC_lexicon_tags_15 },
    { 0x12bead87, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3fead89, SUC_lexicon_tags_68 },
    { 0xddbead8a, SUC_lexicon_tags_17 },
    { 0xa5bead8b, SUC_lexicon_tags_14 },
    { 0x667ead8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83ead8e, SUC_lexicon_tags_17 },
    { 0xbf7ead8e, SUC_lexicon_tags_1 },
    { 0x4cfead8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd4bead92, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5ead96, SUC_lexicon_tags_38 },
    { 0xefbead96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc99ead99, SUC_lexicon_tags_97 },
    { 0xf23ead9a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9eada0, SUC_lexicon_tags_66 },
    { 0x2e9eada1, SUC_lexicon_tags_64 },
    { 0x167eada0, SUC_lexicon_tags_17 },
    { 0xe49eada3, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xe0deada5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebeada8, SUC_lexicon_tags_29 },
    { 0xc61eada9, SUC_lexicon_tags_15 },
    { 0xe7beada8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfeadb0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1efeadb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecfeadb5, SUC_lexicon_tags_66 },
    { 0x1f7eadb6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9eadbf, SUC_lexicon_tags_11 },
    { 0x167eadc0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x935eadc2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35beadc8, SUC_lexicon_tags_12 },
    { 0xaafeadc9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x539eadcb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x837eadcd, SUC_lexicon_tags_60 },
    { 0x297eadce, SUC_lexicon_tags_34 },
    { 0x8fbeadce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x961eadd1, SUC_lexicon_tags_21 },
    { 0x203eadd2, SUC_lexicon_tags_30 },
    { 0xd6beadd2, SUC_lexicon_tags_12 },
    { 0x975eadd1, SUC_lexicon_tags_34 },
    { 0xd19eadd2, SUC_lexicon_tags_27 },
    { 0xcf1eadd5, SUC_lexicon_tags_37 },
    { 0xeb3eadd7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x161eadda, SUC_lexicon_tags_17 },
    { 0x383eaddb, SUC_lexicon_tags_1 },
    { 0x38deaddb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5eade2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x543eade4, SUC_lexicon_tags_12 },
    { 0x2beade5, SUC_lexicon_tags_37 },
    { 0x877eade5, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83eadec, SUC_lexicon_tags_37 },
    { 0xfb5eaded, SUC_lexicon_tags_9 },
    { 0x381eadee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97feadf2, SUC_lexicon_tags_2 },
    { 0x499eadf3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3feadf5, SUC_lexicon_tags_15 },
    { 0x745eadf5, SUC_lexicon_tags_82 },
    { 0xe01eadf7, SUC_lexicon_tags_11 },
    { 0x479eadf7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x17beadfa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7eadfd, SUC_lexicon_tags_31 },
    { 0x4f7eadfe, SUC_lexicon_tags_1 },
    { 0x1c9eadff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x305eae01, SUC_lexicon_tags_13 },
    { 0xf55eae02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615eae09, SUC_lexicon_tags_45 },
    { 0x1eae0a, SUC_lexicon_tags_0 },
    { 0xfedeae0b, SUC_lexicon_tags_25 },
    { 0x143eae0c, SUC_lexicon_tags_27 },
    { 0xabeae0b, SUC_lexicon_tags_8 },
    { 0xb4beae0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a7eae10, SUC_lexicon_tags_37 },
    { 0xe71eae11, SUC_lexicon_tags_38 },
    { 0xf0deae12, SUC_lexicon_tags_17 },
    { 0x413eae13, SUC_lexicon_tags_43 },
    { 0x4c9eae13, SUC_lexicon_tags_1 },
    { 0xdb3eae15, SUC_lexicon_tags_16 },
    { 0x529eae16, SUC_lexicon_tags_60 },
    { 0x3cdeae17, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x253eae1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x22feae1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7c1eae1e, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239eae26, SUC_lexicon_tags_11 },
    { 0xafbeae27, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x73eae29, SUC_lexicon_tags_0 },
    { 0x673eae29, SUC_lexicon_tags_63 },
    { 0x93feae2b, SUC_lexicon_tags_282 },
    { 0x6d9eae29, SUC_lexicon_tags_49 },
    { 0x4beae29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7eae31, SUC_lexicon_tags_38 },
    { 0x729eae31, SUC_lexicon_tags_48 },
    { 0xc7feae33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfdbeae35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb1eae37, SUC_lexicon_tags_25 },
    { 0x74feae38, SUC_lexicon_tags_11 },
    { 0x65eae39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x985eae3b, SUC_lexicon_tags_17 },
    { 0xb69eae3c, SUC_lexicon_tags_33 },
    { 0xb63eae3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2abeae3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a1eae41, SUC_lexicon_tags_66 },
    { 0xb9eae41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xef9eae44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabeae48, SUC_lexicon_tags_2 },
    { 0x463eae48, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x51feae4b, SUC_lexicon_tags_38 },
    { 0x511eae4c, SUC_lexicon_tags_16 },
    { 0x9feae4b, SUC_lexicon_tags_63 },
    { 0xfebeae4e, SUC_lexicon_tags_15 },
    { 0xf59eae4f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbeae57, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56beae5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe83eae5d, SUC_lexicon_tags_17 },
    { 0xd95eae5e, SUC_lexicon_tags_84 },
    { 0xbd1eae5f, SUC_lexicon_tags_25 },
    { 0x51feae60, SUC_lexicon_tags_27 },
    { 0xc6deae5f, SUC_lexicon_tags_25 },
    { 0xc1beae5d, SUC_lexicon_tags_11 },
    { 0xf3eae63, SUC_lexicon_tags_30 },
    { 0xfcbeae64, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbeae68, SUC_lexicon_tags_25 },
    { 0xcd1eae69, SUC_lexicon_tags_26 },
    { 0xf2beae6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x193eae6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5eae6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7feae70, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5deae7b, SUC_lexicon_tags_82 },
    { 0xc3beae7c, SUC_lexicon_tags_7 },
    { 0x3f1eae7d, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9eae84, SUC_lexicon_tags_5 },
    { 0x36beae85, SUC_lexicon_tags_12 },
    { 0x7ffeae85, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x707eae88, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x125eae8b, SUC_lexicon_tags_28 },
    { 0x41feae8c, SUC_lexicon_tags_16 },
    { 0xfeae8d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425eae93, SUC_lexicon_tags_19 },
    { 0xcd9eae94, SUC_lexicon_tags_45 },
    { 0xdddeae95, SUC_lexicon_tags_1 },
    { 0x1d5eae95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98feae9b, SUC_lexicon_tags_61 },
    { 0x581eae9c, SUC_lexicon_tags_7 },
    { 0x97feae9d, SUC_lexicon_tags_31 },
    { 0x3ebeae9e, SUC_lexicon_tags_3 },
    { 0x21beae9f, SUC_lexicon_tags_45 },
    { 0xdbeae9e, SUC_lexicon_tags_12 },
    { 0xa5deaea1, SUC_lexicon_tags_11 },
    { 0xb5beae9b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x813eaea5, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa0deaea7, SUC_lexicon_tags_14 },
    { 0x667eaea8, SUC_lexicon_tags_37 },
    { 0xa3beaea9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa09eaeac, SUC_lexicon_tags_12 },
    { 0x38feaead, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70beaeb1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfeaeb7, SUC_lexicon_tags_2 },
    { 0x14deaeb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad1eaeba, SUC_lexicon_tags_1 },
    { 0x431eaebb, SUC_lexicon_tags_12 },
    { 0xe35eaeba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x80feaebf, SUC_lexicon_tags_13 },
    { 0x3f1eaec0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969eaec7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x247eaeca, SUC_lexicon_tags_27 },
    { 0xc21eaecb, SUC_lexicon_tags_3 },
    { 0xa51eaecb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1eaecf, SUC_lexicon_tags_17 },
    { 0x8dbeaecf, SUC_lexicon_tags_87 },
    { 0x10deaed1, SUC_lexicon_tags_25 },
    { 0x14beaed1, SUC_lexicon_tags_61 },
    { 0xdb1eaed1, SUC_lexicon_tags_33 },
    { 0x4b7eaed4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7a7eaed6, SUC_lexicon_tags_33 },
    { 0xb99eaed7, SUC_lexicon_tags_7 },
    { 0x79eaed7, SUC_lexicon_tags_1 },
    { 0x45deaed9, SUC_lexicon_tags_29 },
    { 0x2ebeaed9, SUC_lexicon_tags_8 },
    { 0x4e5eaedb, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45eaedf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559eaee4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x23feaee6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x84feaee8, SUC_lexicon_tags_17 },
    { 0x195eaee8, SUC_lexicon_tags_30 },
    { 0xa93eaeea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x889eaeec, SUC_lexicon_tags_116 },
    { 0x735eaeed, SUC_lexicon_tags_8 },
    { 0xc19eaeed, SUC_lexicon_tags_39 },
    { 0x18beaeec, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x66deaef1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x909eaef3, SUC_lexicon_tags_15 },
    { 0x4c9eaef4, SUC_lexicon_tags_8 },
    { 0x59eaef5, SUC_lexicon_tags_63 },
    { 0x7dbeaef6, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2abeaef8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xab3eaefa, SUC_lexicon_tags_3 },
    { 0x461eaefb, SUC_lexicon_tags_47 },
    { 0x70deaefc, SUC_lexicon_tags_112 },
    { 0xb51eaefd, SUC_lexicon_tags_25 },
    { 0x677eaefd, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xc3beaf00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x583eaf02, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xefdeaf04, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837eaf08, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7beaf0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35eaf0f, SUC_lexicon_tags_2 },
    { 0x53deaf0f, SUC_lexicon_tags_15 },
    { 0xd1feaf11, SUC_lexicon_tags_19 },
    { 0xc83eaf12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177eaf16, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbeaf1b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffeaf1e, SUC_lexicon_tags_14 },
    { 0x5fdeaf1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70deaf25, SUC_lexicon_tags_7 },
    { 0x37beaf26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbeaf29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdeaf32, SUC_lexicon_tags_116 },
    { 0xf57eaf32, SUC_lexicon_tags_9 },
    { 0xbe9eaf34, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x28beaf36, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x381eaf38, SUC_lexicon_tags_27 },
    { 0x79feaf39, SUC_lexicon_tags_1 },
    { 0x293eaf39, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3b7eaf3c, SUC_lexicon_tags_28 },
    { 0xd67eaf3c, SUC_lexicon_tags_2 },
    { 0xee9eaf3e, SUC_lexicon_tags_38 },
    { 0xe77eaf3f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x735eaf41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49eaf45, SUC_lexicon_tags_4 },
    { 0x7adeaf45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbeaf4c, SUC_lexicon_tags_43 },
    { 0x6a1eaf4c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x277eaf4f, SUC_lexicon_tags_63 },
    { 0xbcdeaf50, SUC_lexicon_tags_12 },
    { 0x2c5eaf4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9beaf54, SUC_lexicon_tags_46 },
    { 0x30beaf54, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9beaf57, SUC_lexicon_tags_7 },
    { 0x335eaf58, SUC_lexicon_tags_34 },
    { 0x21feaf59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x549eaf5c, SUC_lexicon_tags_1 },
    { 0x1dfeaf5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x797eaf60, SUC_lexicon_tags_16 },
    { 0xfe1eaf61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83deaf63, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13beaf67, SUC_lexicon_tags_39 },
    { 0x2e3eaf67, SUC_lexicon_tags_12 },
    { 0x14deaf69, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9afeaf6b, SUC_lexicon_tags_7 },
    { 0xb45eaf6c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf87eaf6e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1eaf71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe85eaf73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17beaf76, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb67eaf78, SUC_lexicon_tags_11 },
    { 0x2dbeaf78, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb81eaf7b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4fbeaf7d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x31feaf7f, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1eaf82, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf95eaf84, SUC_lexicon_tags_2 },
    { 0x5ffeaf85, SUC_lexicon_tags_97 },
    { 0x2ebeaf85, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2e3eaf88, SUC_lexicon_tags_2 },
    { 0x8ddeaf88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xaf7eaf8b, SUC_lexicon_tags_15 },
    { 0xdf9eaf8c, SUC_lexicon_tags_6 },
    { 0xf87eaf8c, SUC_lexicon_tags_17 },
    { 0xd11eaf8e, SUC_lexicon_tags_37 },
    { 0x34beaf8f, SUC_lexicon_tags_11 },
    { 0x549eaf90, SUC_lexicon_tags_77 },
    { 0xd8beaf91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c1eaf93, SUC_lexicon_tags_37 },
    { 0x9a1eaf94, SUC_lexicon_tags_9 },
    { 0x18beaf95, SUC_lexicon_tags_11 },
    { 0x7bfeaf93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5bfeaf98, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbebeaf9a, SUC_lexicon_tags_0 },
    { 0x65eaf9b, SUC_lexicon_tags_37 },
    { 0x86feaf9a, SUC_lexicon_tags_14 },
    { 0xfb1eaf9d, SUC_lexicon_tags_7 },
    { 0xc53eaf9b, SUC_lexicon_tags_631 },
    { 0xc75eaf9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x661eafa1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0deafa5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99eafaa, SUC_lexicon_tags_13 },
    { 0xb07eafab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x125eafae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x37deafb0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401eafb4, SUC_lexicon_tags_29 },
    { 0xf9deafb5, SUC_lexicon_tags_1 },
    { 0x92feafb6, SUC_lexicon_tags_1289 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867eafc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3abeafc4, SUC_lexicon_tags_37 },
    { 0x6abeafc5, SUC_lexicon_tags_45 },
    { 0x6d3eafc6, SUC_lexicon_tags_11 },
    { 0xfabeafc7, SUC_lexicon_tags_11 },
    { 0x3edeafc8, SUC_lexicon_tags_15 },
    { 0x5f7eafc9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633eafce, SUC_lexicon_tags_6 },
    { 0xb91eafce, SUC_lexicon_tags_14 },
    { 0x2cfeafd0, SUC_lexicon_tags_63 },
    { 0x675eafd1, SUC_lexicon_tags_25 },
    { 0x31deafd2, SUC_lexicon_tags_11 },
    { 0x605eafd3, SUC_lexicon_tags_15 },
    { 0xf71eafd4, SUC_lexicon_tags_15 },
    { 0xb15eafd5, SUC_lexicon_tags_17 },
    { 0x983eafd6, SUC_lexicon_tags_43 },
    { 0x3d7eafd7, SUC_lexicon_tags_19 },
    { 0xb65eafd6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xadeafdb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69deafe0, SUC_lexicon_tags_37 },
    { 0x3b1eafe1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1eafe5, SUC_lexicon_tags_33 },
    { 0xd9feafe6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xebeafe8, SUC_lexicon_tags_39 },
    { 0x147eafe9, SUC_lexicon_tags_30 },
    { 0x1afeafea, SUC_lexicon_tags_8 },
    { 0x805eafea, SUC_lexicon_tags_37 },
    { 0xa7deafe8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x543eafef, SUC_lexicon_tags_37 },
    { 0xb0deaff0, SUC_lexicon_tags_84 },
    { 0x705eafef, SUC_lexicon_tags_12 },
    { 0xc7beaff2, SUC_lexicon_tags_15 },
    { 0xc45eaff2, SUC_lexicon_tags_60 },
    { 0x7abeaff4, SUC_lexicon_tags_18 },
    { 0x26beaff4, SUC_lexicon_tags_1 },
    { 0x975eaff5, SUC_lexicon_tags_27 },
    { 0x8c7eafef, SUC_lexicon_tags_12 },
    { 0x1b1eaff8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7feaffb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdeaffe, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x5d7eb000, SUC_lexicon_tags_29 },
    { 0xa7eb001, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99deb005, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x675eb007, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x7deb009, SUC_lexicon_tags_45 },
    { 0xa61eb00a, SUC_lexicon_tags_27 },
    { 0xa6feb00b, SUC_lexicon_tags_17 },
    { 0xb15eb00c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4feb00e, SUC_lexicon_tags_37 },
    { 0x6bfeb00e, SUC_lexicon_tags_15 },
    { 0x22feb00f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x9bdeb012, SUC_lexicon_tags_33 },
    { 0x871eb012, SUC_lexicon_tags_15 },
    { 0xe6deb013, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x789eb017, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0feb01a, SUC_lexicon_tags_17 },
    { 0x775eb01b, SUC_lexicon_tags_1 },
    { 0x315eb01a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdeb01f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6feb021, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415eb02f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x71feb031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfefeb036, SUC_lexicon_tags_25 },
    { 0x21eb037, SUC_lexicon_tags_1 },
    { 0x3b5eb038, SUC_lexicon_tags_17 },
    { 0xa8beb039, SUC_lexicon_tags_0 },
    { 0x583eb039, SUC_lexicon_tags_171 },
    { 0x835eb03b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5adeb03d, SUC_lexicon_tags_14 },
    { 0x233eb03e, SUC_lexicon_tags_14 },
    { 0x589eb03f, SUC_lexicon_tags_27 },
    { 0x1d1eb03f, SUC_lexicon_tags_13 },
    { 0xe27eb041, SUC_lexicon_tags_2 },
    { 0x887eb042, SUC_lexicon_tags_3 },
    { 0x7d5eb03f, SUC_lexicon_tags_12 },
    { 0xfb1eb03f, SUC_lexicon_tags_37 },
    { 0xc63eb040, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01eb04d, SUC_lexicon_tags_18 },
    { 0xff1eb04e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901eb055, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42beb059, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3eb05e, SUC_lexicon_tags_32 },
    { 0xacdeb05e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6edeb061, SUC_lexicon_tags_15 },
    { 0x671eb062, SUC_lexicon_tags_29 },
    { 0x469eb062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfa1eb065, SUC_lexicon_tags_47 },
    { 0x255eb066, SUC_lexicon_tags_38 },
    { 0xad1eb067, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x67eb06a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76feb06e, SUC_lexicon_tags_29 },
    { 0xa85eb06e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x931eb071, SUC_lexicon_tags_19 },
    { 0xed1eb071, SUC_lexicon_tags_43 },
    { 0x76feb073, SUC_lexicon_tags_48 },
    { 0xa77eb073, SUC_lexicon_tags_49 },
    { 0x4b7eb075, SUC_lexicon_tags_17 },
    { 0x713eb075, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2feb078, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x937eb07b, SUC_lexicon_tags_37 },
    { 0x6bdeb07c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbeb085, SUC_lexicon_tags_33 },
    { 0xe4beb085, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31deb08e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x639eb090, SUC_lexicon_tags_9 },
    { 0x5b1eb091, SUC_lexicon_tags_26 },
    { 0x5ddeb090, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7deb098, SUC_lexicon_tags_14 },
    { 0xf01eb098, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5beb09c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8beb0a0, SUC_lexicon_tags_46 },
    { 0x33feb0a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcedeb0a3, SUC_lexicon_tags_23 },
    { 0x6a5eb0a4, SUC_lexicon_tags_82 },
    { 0x15deb0a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7eb0a8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa21eb0aa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xedeb0ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785eb0b2, SUC_lexicon_tags_1 },
    { 0x341eb0b3, SUC_lexicon_tags_25 },
    { 0x1d7eb0b2, SUC_lexicon_tags_1 },
    { 0x2afeb0b3, SUC_lexicon_tags_25 },
    { 0x7edeb0b6, SUC_lexicon_tags_38 },
    { 0x479eb0b6, SUC_lexicon_tags_31 },
    { 0x211eb0b8, SUC_lexicon_tags_49 },
    { 0x2a7eb0b8, SUC_lexicon_tags_14 },
    { 0x123eb0b8, SUC_lexicon_tags_92 },
    { 0xb2beb0bb, SUC_lexicon_tags_43 },
    { 0xb2beb0bc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x42deb0bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221eb0c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x339eb0c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd5eb0c7, SUC_lexicon_tags_7 },
    { 0x7b9eb0c7, SUC_lexicon_tags_43 },
    { 0x95deb0c9, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x4c3eb0cb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdbeb0cd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497eb0d3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdeb0d6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa7deb0d8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4c5eb0da, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbeb0df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15deb0e5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xd0feb0e7, SUC_lexicon_tags_1 },
    { 0x7c7eb0e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbeb0ef, SUC_lexicon_tags_16 },
    { 0x827eb0ef, SUC_lexicon_tags_23 },
    { 0x4cfeb0f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397eb0f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2beb0fb, SUC_lexicon_tags_45 },
    { 0x4e5eb0fc, SUC_lexicon_tags_63 },
    { 0x179eb0fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x14feb0ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907eb104, SUC_lexicon_tags_25 },
    { 0xab1eb104, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x605eb107, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x86feb109, SUC_lexicon_tags_9 },
    { 0xc1deb10a, SUC_lexicon_tags_1 },
    { 0x25eb10a, SUC_lexicon_tags_2 },
    { 0xef9eb10c, SUC_lexicon_tags_37 },
    { 0xb8beb10c, SUC_lexicon_tags_77 },
    { 0x1dfeb10e, SUC_lexicon_tags_25 },
    { 0xb3deb10f, SUC_lexicon_tags_33 },
    { 0x19feb10f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x981eb113, SUC_lexicon_tags_6 },
    { 0x765eb113, SUC_lexicon_tags_45 },
    { 0xbb5eb115, SUC_lexicon_tags_37 },
    { 0xec5eb116, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbcdeb118, SUC_lexicon_tags_17 },
    { 0x17eb119, SUC_lexicon_tags_2 },
    { 0xb73eb118, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54deb11e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf03eb120, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbeb124, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x881eb127, SUC_lexicon_tags_826 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01eb12a, SUC_lexicon_tags_27 },
    { 0xee3eb12a, SUC_lexicon_tags_476 },
    { 0x49feb12b, SUC_lexicon_tags_27 },
    { 0xa13eb12a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5eb132, SUC_lexicon_tags_6 },
    { 0xb5feb133, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa27eb135, SUC_lexicon_tags_43 },
    { 0x897eb135, SUC_lexicon_tags_16 },
    { 0xbb5eb137, SUC_lexicon_tags_25 },
    { 0x8f1eb137, SUC_lexicon_tags_38 },
    { 0x991eb138, SUC_lexicon_tags_6 },
    { 0x29feb138, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x39feb13c, SUC_lexicon_tags_10 },
    { 0xfc7eb13d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abeb141, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x421eb144, SUC_lexicon_tags_14 },
    { 0xb25eb145, SUC_lexicon_tags_31 },
    { 0xf67eb144, SUC_lexicon_tags_11 },
    { 0xd8beb147, SUC_lexicon_tags_19 },
    { 0x157eb148, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5eb14b, SUC_lexicon_tags_61 },
    { 0xf09eb14c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa2deb14e, SUC_lexicon_tags_0 },
    { 0x857eb14e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7beb153, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x661eb155, SUC_lexicon_tags_87 },
    { 0x3c3eb155, SUC_lexicon_tags_0 },
    { 0x977eb156, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x53beb159, SUC_lexicon_tags_27 },
    { 0x381eb15a, SUC_lexicon_tags_12 },
    { 0xb7feb15a, SUC_lexicon_tags_12 },
    { 0x6bfeb15b, SUC_lexicon_tags_1 },
    { 0x5dbeb15c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3eb161, SUC_lexicon_tags_25 },
    { 0x51feb162, SUC_lexicon_tags_8 },
    { 0xddfeb163, SUC_lexicon_tags_27 },
    { 0x9c1eb164, SUC_lexicon_tags_37 },
    { 0xb3feb162, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adeb16c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xac3eb16f, SUC_lexicon_tags_15 },
    { 0xd6deb170, SUC_lexicon_tags_47 },
    { 0xaa5eb171, SUC_lexicon_tags_45 },
    { 0xb67eb172, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3eb176, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3deb178, SUC_lexicon_tags_7 },
    { 0xc1beb179, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb37eb17b, SUC_lexicon_tags_13 },
    { 0xb17eb17c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc33eb17e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51deb185, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669eb18c, SUC_lexicon_tags_17 },
    { 0x11eb18d, SUC_lexicon_tags_31 },
    { 0xd1deb18d, SUC_lexicon_tags_33 },
    { 0xf63eb18f, SUC_lexicon_tags_12 },
    { 0x5e7eb190, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x743eb192, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5beb1a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4deb1aa, SUC_lexicon_tags_31 },
    { 0x6c7eb1ab, SUC_lexicon_tags_7 },
    { 0x76deb1ac, SUC_lexicon_tags_38 },
    { 0x285eb1ac, SUC_lexicon_tags_12 },
    { 0xd1eb1aa, SUC_lexicon_tags_39 },
    { 0xb91eb1af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeb1b3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9eb1b8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5eb1bd, SUC_lexicon_tags_25 },
    { 0xd79eb1be, SUC_lexicon_tags_45 },
    { 0x103eb1bd, SUC_lexicon_tags_17 },
    { 0x1b5eb1c0, SUC_lexicon_tags_1 },
    { 0x8a7eb1c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xecbeb1c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x49beb1c5, SUC_lexicon_tags_63 },
    { 0xb7feb1c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9adeb1c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35eb1cf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68feb1d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6cfeb1d8, SUC_lexicon_tags_45 },
    { 0x59deb1d8, SUC_lexicon_tags_13 },
    { 0x669eb1da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x935eb1dd, SUC_lexicon_tags_1 },
    { 0xa5beb1de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc09eb1e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3eb1e8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc5eb1ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07eb1ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01eb1f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10deb1fb, SUC_lexicon_tags_8 },
    { 0xb59eb1fb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadfeb200, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa63eb202, SUC_lexicon_tags_1 },
    { 0xfabeb203, SUC_lexicon_tags_36 },
    { 0x32feb203, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73eb207, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x445eb209, SUC_lexicon_tags_1 },
    { 0xad3eb20a, SUC_lexicon_tags_38 },
    { 0x4e3eb20a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4feb20e, SUC_lexicon_tags_31 },
    { 0xcb5eb20f, SUC_lexicon_tags_6 },
    { 0xbeb210, SUC_lexicon_tags_29 },
    { 0x82beb20f, SUC_lexicon_tags_27 },
    { 0x75feb20e, SUC_lexicon_tags_25 },
    { 0x907eb212, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3eb216, SUC_lexicon_tags_37 },
    { 0x87eb216, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x43beb21a, SUC_lexicon_tags_23 },
    { 0xdd9eb21b, SUC_lexicon_tags_7 },
    { 0xbf1eb21c, SUC_lexicon_tags_63 },
    { 0xdd9eb21c, SUC_lexicon_tags_2 },
    { 0x2b5eb21d, SUC_lexicon_tags_47 },
    { 0x5adeb21f, SUC_lexicon_tags_49 },
    { 0x375eb21b, SUC_lexicon_tags_22 },
    { 0xb31eb221, SUC_lexicon_tags_1 },
    { 0xe3feb222, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfbdeb224, SUC_lexicon_tags_17 },
    { 0xfeb225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23eb228, SUC_lexicon_tags_17 },
    { 0xec7eb229, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x939eb22b, SUC_lexicon_tags_2 },
    { 0xdddeb22b, SUC_lexicon_tags_49 },
    { 0x9ddeb22d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5eb230, SUC_lexicon_tags_17 },
    { 0x45deb231, SUC_lexicon_tags_46 },
    { 0x5d3eb231, SUC_lexicon_tags_29 },
    { 0xf3feb230, SUC_lexicon_tags_44 },
    { 0xc1eb234, SUC_lexicon_tags_2 },
    { 0xf1feb235, SUC_lexicon_tags_17 },
    { 0x333eb236, SUC_lexicon_tags_1 },
    { 0x50deb236, SUC_lexicon_tags_7 },
    { 0xa3eb238, SUC_lexicon_tags_7 },
    { 0x5fdeb234, SUC_lexicon_tags_2 },
    { 0x77deb234, SUC_lexicon_tags_11 },
    { 0x337eb23b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4fbeb23d, SUC_lexicon_tags_17 },
    { 0x555eb23e, SUC_lexicon_tags_49 },
    { 0xec5eb23f, SUC_lexicon_tags_31 },
    { 0x58deb23e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x175eb243, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46beb247, SUC_lexicon_tags_1 },
    { 0x6d9eb247, SUC_lexicon_tags_17 },
    { 0xfffeb249, SUC_lexicon_tags_12 },
    { 0x5ebeb24a, SUC_lexicon_tags_25 },
    { 0x6ddeb24b, SUC_lexicon_tags_14 },
    { 0xb1feb24c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55beb252, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09eb257, SUC_lexicon_tags_71 },
    { 0x201eb258, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43beb25e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94deb262, SUC_lexicon_tags_179 },
    { 0x6e9eb262, SUC_lexicon_tags_37 },
    { 0x7ffeb264, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x301eb266, SUC_lexicon_tags_10 },
    { 0xdebeb267, SUC_lexicon_tags_43 },
    { 0xf7feb267, SUC_lexicon_tags_7 },
    { 0xe89eb267, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x387eb26b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadeb26e, SUC_lexicon_tags_31 },
    { 0xcedeb26f, SUC_lexicon_tags_49 },
    { 0xeddeb26f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8deb273, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95eb276, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21eb279, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995eb282, SUC_lexicon_tags_47 },
    { 0xb07eb283, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50beb288, SUC_lexicon_tags_23 },
    { 0xaa1eb289, SUC_lexicon_tags_32 },
    { 0xa1beb28a, SUC_lexicon_tags_7 },
    { 0x417eb28a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adeb28e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x17deb290, SUC_lexicon_tags_15 },
    { 0x3edeb291, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x93deb294, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc05eb298, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70feb29f, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x769eb2a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741eb2ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8d3eb2ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1eb2b2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeb2b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x661eb2ba, SUC_lexicon_tags_60 },
    { 0x98feb2bb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cbeb2bd, SUC_lexicon_tags_37 },
    { 0x993eb2bd, SUC_lexicon_tags_9 },
    { 0x75beb2bd, SUC_lexicon_tags_107 },
    { 0x553eb2c0, SUC_lexicon_tags_37 },
    { 0x1c1eb2bd, SUC_lexicon_tags_38 },
    { 0x33beb2c1, SUC_lexicon_tags_21 },
    { 0xbafeb2c3, SUC_lexicon_tags_11 },
    { 0x34deb2c4, SUC_lexicon_tags_14 },
    { 0x17feb2c5, SUC_lexicon_tags_23 },
    { 0xb0feb2c4, SUC_lexicon_tags_40 },
    { 0xd79eb2c6, SUC_lexicon_tags_45 },
    { 0x24beb2c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea9eb2ca, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfd5eb2cc, SUC_lexicon_tags_0 },
    { 0xc7eb2cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x291eb2cf, SUC_lexicon_tags_32 },
    { 0xaf7eb2d0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x185eb2d2, SUC_lexicon_tags_37 },
    { 0xe4deb2d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3beb2da, SUC_lexicon_tags_70 },
    { 0xb65eb2db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3eb2de, SUC_lexicon_tags_26 },
    { 0x945eb2df, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985eb2e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe15eb2e9, SUC_lexicon_tags_37 },
    { 0x1ebeb2e9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15eb2ee, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3eb2f3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7eb2f6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x929eb2f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbeb2ff, SUC_lexicon_tags_0 },
    { 0x681eb2ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9eb303, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86feb30b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x709eb30e, SUC_lexicon_tags_1 },
    { 0xed3eb30f, SUC_lexicon_tags_60 },
    { 0xc2feb30f, SUC_lexicon_tags_11 },
    { 0xdb9eb311, SUC_lexicon_tags_7 },
    { 0x625eb30f, SUC_lexicon_tags_25 },
    { 0xc9eb311, SUC_lexicon_tags_38 },
    { 0xbd5eb314, SUC_lexicon_tags_30 },
    { 0x307eb315, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8deb317, SUC_lexicon_tags_12 },
    { 0x451eb318, SUC_lexicon_tags_68 },
    { 0x7bbeb318, SUC_lexicon_tags_25 },
    { 0x403eb31a, SUC_lexicon_tags_43 },
    { 0x301eb31a, SUC_lexicon_tags_46 },
    { 0xc0deb31c, SUC_lexicon_tags_11 },
    { 0x641eb31b, SUC_lexicon_tags_15 },
    { 0xf11eb31c, SUC_lexicon_tags_43 },
    { 0x5adeb31f, SUC_lexicon_tags_25 },
    { 0xf6beb31b, SUC_lexicon_tags_15 },
    { 0x8b1eb317, SUC_lexicon_tags_27 },
    { 0x78deb322, SUC_lexicon_tags_70 },
    { 0x6bfeb319, SUC_lexicon_tags_14 },
    { 0x387eb324, SUC_lexicon_tags_1 },
    { 0xaefeb325, SUC_lexicon_tags_1 },
    { 0xffdeb325, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99feb329, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1eb32c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6beb330, SUC_lexicon_tags_9 },
    { 0x6c7eb331, SUC_lexicon_tags_37 },
    { 0x1c7eb331, SUC_lexicon_tags_30 },
    { 0x357eb333, SUC_lexicon_tags_55 },
    { 0x9cdeb333, SUC_lexicon_tags_15 },
    { 0x919eb335, SUC_lexicon_tags_37 },
    { 0xe2deb335, SUC_lexicon_tags_23 },
    { 0x499eb335, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983eb33b, SUC_lexicon_tags_12 },
    { 0x5b3eb33b, SUC_lexicon_tags_17 },
    { 0xe1feb33d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95eb340, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x765eb342, SUC_lexicon_tags_35 },
    { 0xc53eb342, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1a3eb345, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7eb34d, SUC_lexicon_tags_11 },
    { 0xf3deb34d, SUC_lexicon_tags_92 },
    { 0x74feb34e, SUC_lexicon_tags_7 },
    { 0xe83eb34f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xac7eb352, SUC_lexicon_tags_35 },
    { 0x357eb352, SUC_lexicon_tags_79 },
    { 0x415eb354, SUC_lexicon_tags_38 },
    { 0xab7eb354, SUC_lexicon_tags_47 },
    { 0xef3eb356, SUC_lexicon_tags_84 },
    { 0xdffeb356, SUC_lexicon_tags_17 },
    { 0xee3eb354, SUC_lexicon_tags_23 },
    { 0xbefeb355, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x13beb35c, SUC_lexicon_tags_15 },
    { 0x257eb35d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x515eb360, SUC_lexicon_tags_2 },
    { 0x203eb360, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495eb367, SUC_lexicon_tags_25 },
    { 0x357eb368, SUC_lexicon_tags_6 },
    { 0xc49eb369, SUC_lexicon_tags_37 },
    { 0xc05eb36a, SUC_lexicon_tags_3 },
    { 0x12beb36a, SUC_lexicon_tags_43 },
    { 0x2deb36c, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149eb375, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb73eb377, SUC_lexicon_tags_29 },
    { 0x827eb378, SUC_lexicon_tags_8 },
    { 0xe21eb379, SUC_lexicon_tags_87 },
    { 0xaf3eb378, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9eb386, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4beb389, SUC_lexicon_tags_12 },
    { 0xd71eb38a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfadeb38c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9eb391, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfeb396, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5e5eb398, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc9deb39a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x181eb39d, SUC_lexicon_tags_0 },
    { 0xd67eb39e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x921eb3a1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xbefeb3a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63beb3a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73beb3ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23eb3b2, SUC_lexicon_tags_11 },
    { 0x691eb3b2, SUC_lexicon_tags_17 },
    { 0x1bfeb3b4, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b7eb3b8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9eb3bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x16deb3bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5eb3c4, SUC_lexicon_tags_25 },
    { 0x5feb3c5, SUC_lexicon_tags_30 },
    { 0x7d3eb3c6, SUC_lexicon_tags_27 },
    { 0x8e1eb3c6, SUC_lexicon_tags_25 },
    { 0x32deb3c6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xafeb3ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2b1eb3cc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x15deb3ce, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59eb3d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3feb3da, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x385eb3dd, SUC_lexicon_tags_15 },
    { 0x675eb3dd, SUC_lexicon_tags_4 },
    { 0x591eb3df, SUC_lexicon_tags_1 },
    { 0xaf9eb3de, SUC_lexicon_tags_7 },
    { 0xdf7eb3e1, SUC_lexicon_tags_17 },
    { 0x7d7eb3e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdbeb3e4, SUC_lexicon_tags_19 },
    { 0x945eb3e4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1feb3e8, SUC_lexicon_tags_87 },
    { 0x477eb3e9, SUC_lexicon_tags_11 },
    { 0xd6deb3ea, SUC_lexicon_tags_11 },
    { 0x78feb3eb, SUC_lexicon_tags_91 },
    { 0x1cbeb3e8, SUC_lexicon_tags_7 },
    { 0xbc3eb3ed, SUC_lexicon_tags_7 },
    { 0xe0beb3ee, SUC_lexicon_tags_49 },
    { 0x531eb3ef, SUC_lexicon_tags_25 },
    { 0xd3deb3f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adeb3f3, SUC_lexicon_tags_38 },
    { 0x2dbeb3f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3eb3fa, SUC_lexicon_tags_46 },
    { 0x521eb3fb, SUC_lexicon_tags_11 },
    { 0x353eb3fc, SUC_lexicon_tags_15 },
    { 0x9f5eb3fc, SUC_lexicon_tags_17 },
    { 0x1dbeb3fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64beb401, SUC_lexicon_tags_33 },
    { 0x3bbeb401, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc03eb404, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55deb406, SUC_lexicon_tags_37 },
    { 0x357eb407, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x13eb409, SUC_lexicon_tags_15 },
    { 0xd0feb409, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39eb40e, SUC_lexicon_tags_61 },
    { 0x49beb40e, SUC_lexicon_tags_17 },
    { 0x35feb410, SUC_lexicon_tags_63 },
    { 0xc69eb411, SUC_lexicon_tags_10 },
    { 0x499eb410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb5feb414, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbeb41d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x37feb41f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1eb423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6feb42c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395eb432, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef5eb434, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589eb438, SUC_lexicon_tags_14 },
    { 0x1b5eb439, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbeb43c, SUC_lexicon_tags_23 },
    { 0x1e1eb43c, SUC_lexicon_tags_63 },
    { 0x593eb43e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x557eb441, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x825eb443, SUC_lexicon_tags_37 },
    { 0x87deb443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbeb44a, SUC_lexicon_tags_26 },
    { 0x539eb44a, SUC_lexicon_tags_31 },
    { 0xcebeb44b, SUC_lexicon_tags_2 },
    { 0xb7deb44b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1eb450, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21eb453, SUC_lexicon_tags_7 },
    { 0xa3deb454, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7beb45c, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x183eb45f, SUC_lexicon_tags_2 },
    { 0x7e9eb45f, SUC_lexicon_tags_45 },
    { 0x1f3eb460, SUC_lexicon_tags_37 },
    { 0x29deb462, SUC_lexicon_tags_2 },
    { 0x8f5eb45f, SUC_lexicon_tags_68 },
    { 0xae3eb464, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3eb469, SUC_lexicon_tags_49 },
    { 0x863eb46a, SUC_lexicon_tags_45 },
    { 0xb33eb46b, SUC_lexicon_tags_669 },
    { 0xf0feb46c, SUC_lexicon_tags_40 },
    { 0x1f7eb46a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53deb473, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4feb479, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87eb47e, SUC_lexicon_tags_7 },
    { 0x805eb47f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6deb483, SUC_lexicon_tags_7 },
    { 0xb47eb484, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805eb48a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd7eb48c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73eb493, SUC_lexicon_tags_28 },
    { 0xe8deb493, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03eb498, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129eb49d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3eb4a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x45deb4a6, SUC_lexicon_tags_37 },
    { 0xa37eb4a7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97eb4aa, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27eb4ae, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x263eb4b0, SUC_lexicon_tags_17 },
    { 0xa2deb4b1, SUC_lexicon_tags_15 },
    { 0x29eb4b2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf0beb4b4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8feb4ba, SUC_lexicon_tags_17 },
    { 0x8bbeb4bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc1feb4bd, SUC_lexicon_tags_15 },
    { 0x3bbeb4be, SUC_lexicon_tags_48 },
    { 0xed5eb4bf, SUC_lexicon_tags_25 },
    { 0xee9eb4c0, SUC_lexicon_tags_15 },
    { 0x22beb4c1, SUC_lexicon_tags_15 },
    { 0xe35eb4be, SUC_lexicon_tags_60 },
    { 0xda9eb4c0, SUC_lexicon_tags_2 },
    { 0x505eb4c1, SUC_lexicon_tags_7 },
    { 0x139eb4c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73eb4ce, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfa5eb4d0, SUC_lexicon_tags_350 },
    { 0x59deb4d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7beb4d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabeb4db, SUC_lexicon_tags_11 },
    { 0x879eb4dc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8beb4e0, SUC_lexicon_tags_87 },
    { 0xe4beb4e1, SUC_lexicon_tags_17 },
    { 0xda5eb4e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8feb4e4, SUC_lexicon_tags_12 },
    { 0xb53eb4e5, SUC_lexicon_tags_1 },
    { 0xf5eb4e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631eb4ec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe39eb4ee, SUC_lexicon_tags_65 },
    { 0x1d3eb4ee, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcabeb4f1, SUC_lexicon_tags_45 },
    { 0xcfdeb4f2, SUC_lexicon_tags_0 },
    { 0x1ebeb4f3, SUC_lexicon_tags_7 },
    { 0x7a7eb4f4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13feb4f7, SUC_lexicon_tags_16 },
    { 0x9cbeb4f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1eb4fd, SUC_lexicon_tags_12 },
    { 0x917eb4fe, SUC_lexicon_tags_1 },
    { 0xf33eb4ff, SUC_lexicon_tags_11 },
    { 0xbb3eb4fe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xffeb502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfd1eb504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5eb508, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32deb50e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7eb51d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44feb529, SUC_lexicon_tags_4 },
    { 0x307eb529, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xee9eb52c, SUC_lexicon_tags_17 },
    { 0x407eb52d, SUC_lexicon_tags_30 },
    { 0x5f1eb52e, SUC_lexicon_tags_9 },
    { 0x20beb52f, SUC_lexicon_tags_27 },
    { 0x1bbeb530, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7eb536, SUC_lexicon_tags_14 },
    { 0x6f1eb536, SUC_lexicon_tags_19 },
    { 0xe41eb538, SUC_lexicon_tags_29 },
    { 0x329eb539, SUC_lexicon_tags_17 },
    { 0x479eb538, SUC_lexicon_tags_11 },
    { 0xfa3eb53b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf61eb53e, SUC_lexicon_tags_1 },
    { 0x413eb53f, SUC_lexicon_tags_4 },
    { 0x7dbeb53f, SUC_lexicon_tags_49 },
    { 0x26feb540, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7eb545, SUC_lexicon_tags_49 },
    { 0xaadeb545, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7eb549, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x607eb54b, SUC_lexicon_tags_198 },
    { 0x38beb54b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaabeb54e, SUC_lexicon_tags_15 },
    { 0xd09eb54f, SUC_lexicon_tags_12 },
    { 0xddbeb550, SUC_lexicon_tags_38 },
    { 0x329eb551, SUC_lexicon_tags_23 },
    { 0xb01eb551, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8deb555, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd0feb557, SUC_lexicon_tags_19 },
    { 0x6d7eb558, SUC_lexicon_tags_60 },
    { 0x491eb557, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5eb55d, SUC_lexicon_tags_43 },
    { 0x531eb55e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63feb564, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x905eb567, SUC_lexicon_tags_70 },
    { 0x415eb568, SUC_lexicon_tags_13 },
    { 0xe5beb569, SUC_lexicon_tags_25 },
    { 0xb6feb56a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54beb56f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5eb572, SUC_lexicon_tags_31 },
    { 0x14beb572, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471eb579, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf19eb57b, SUC_lexicon_tags_15 },
    { 0x19eb57b, SUC_lexicon_tags_11 },
    { 0xc55eb57d, SUC_lexicon_tags_31 },
    { 0x697eb57e, SUC_lexicon_tags_29 },
    { 0xf63eb57f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9f1eb581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc33eb584, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc21eb586, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9eb58d, SUC_lexicon_tags_11 },
    { 0x5f3eb58e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1eb592, SUC_lexicon_tags_37 },
    { 0xd5beb593, SUC_lexicon_tags_66 },
    { 0x789eb592, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4beb598, SUC_lexicon_tags_17 },
    { 0xf5feb599, SUC_lexicon_tags_11 },
    { 0x7ffeb598, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35eb59d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3eb5a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb11eb5a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145eb5aa, SUC_lexicon_tags_45 },
    { 0x73beb5aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x653eb5ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x387eb5af, SUC_lexicon_tags_11 },
    { 0x599eb5b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbeb5b9, SUC_lexicon_tags_37 },
    { 0x469eb5b9, SUC_lexicon_tags_37 },
    { 0x9f3eb5bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd45eb5bf, SUC_lexicon_tags_35 },
    { 0x6afeb5c0, SUC_lexicon_tags_33 },
    { 0x33feb5c1, SUC_lexicon_tags_8 },
    { 0xd8feb5c1, SUC_lexicon_tags_7 },
    { 0xe05eb5c2, SUC_lexicon_tags_48 },
    { 0xe4beb5c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x259eb5c7, SUC_lexicon_tags_17 },
    { 0xd09eb5c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301eb5d5, SUC_lexicon_tags_25 },
    { 0x807eb5d6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbeb5dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2beb5e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33eb5e9, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43eb5f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cfeb5f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x397eb5fb, SUC_lexicon_tags_37 },
    { 0x1c5eb5fb, SUC_lexicon_tags_1667 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfeb603, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27beb605, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07eb60a, SUC_lexicon_tags_7 },
    { 0xed7eb60a, SUC_lexicon_tags_66 },
    { 0xd8beb60c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9eb610, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x50deb613, SUC_lexicon_tags_11 },
    { 0x813eb614, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4feb61a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9eb623, SUC_lexicon_tags_275 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7deb628, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x57eb62a, SUC_lexicon_tags_11 },
    { 0x97deb62b, SUC_lexicon_tags_28 },
    { 0xdc5eb62c, SUC_lexicon_tags_25 },
    { 0x97deb62c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce3eb62f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5eb637, SUC_lexicon_tags_25 },
    { 0xed5eb638, SUC_lexicon_tags_15 },
    { 0x793eb639, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49eb63e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3eb641, SUC_lexicon_tags_11 },
    { 0x703eb641, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdeb649, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x46beb64c, SUC_lexicon_tags_7 },
    { 0xbedeb64c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581eb651, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x91deb654, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85eb65a, SUC_lexicon_tags_11 },
    { 0x98feb65b, SUC_lexicon_tags_30 },
    { 0x701eb65b, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3deb662, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edeb66a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb8feb66c, SUC_lexicon_tags_43 },
    { 0x88feb66d, SUC_lexicon_tags_2 },
    { 0xeabeb66e, SUC_lexicon_tags_48 },
    { 0x68feb66f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x387eb671, SUC_lexicon_tags_59 },
    { 0xb8deb672, SUC_lexicon_tags_14 },
    { 0x64feb673, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17beb67b, SUC_lexicon_tags_25 },
    { 0x8ebeb67c, SUC_lexicon_tags_792 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15feb680, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7eb683, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3eb68b, SUC_lexicon_tags_63 },
    { 0xa09eb68c, SUC_lexicon_tags_47 },
    { 0x4c1eb68c, SUC_lexicon_tags_17 },
    { 0x913eb68e, SUC_lexicon_tags_17 },
    { 0x6abeb68d, SUC_lexicon_tags_69 },
    { 0xba7eb68c, SUC_lexicon_tags_15 },
    { 0xf9eb691, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83deb695, SUC_lexicon_tags_8 },
    { 0xefdeb695, SUC_lexicon_tags_12 },
    { 0xc25eb696, SUC_lexicon_tags_4 },
    { 0x185eb698, SUC_lexicon_tags_46 },
    { 0x573eb699, SUC_lexicon_tags_148 },
    { 0x6d7eb695, SUC_lexicon_tags_15 },
    { 0xe43eb69b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5eb6a0, SUC_lexicon_tags_45 },
    { 0x793eb6a0, SUC_lexicon_tags_25 },
    { 0x1ffeb6a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929eb6a8, SUC_lexicon_tags_17 },
    { 0x4d9eb6a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9eb6ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x103eb6b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce5eb6b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77deb6b6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x425eb6b9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x101eb6bb, SUC_lexicon_tags_91 },
    { 0x1c5eb6bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedeb6bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8deb6c4, SUC_lexicon_tags_29 },
    { 0x17deb6c5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x26feb6c8, SUC_lexicon_tags_14 },
    { 0x8e5eb6c9, SUC_lexicon_tags_10 },
    { 0x487eb6c8, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x615eb6cc, SUC_lexicon_tags_2 },
    { 0xfcfeb6cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x77beb6d0, SUC_lexicon_tags_84 },
    { 0x8d3eb6d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83beb6d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18deb6dd, SUC_lexicon_tags_17 },
    { 0xef1eb6dd, SUC_lexicon_tags_17 },
    { 0xea1eb6df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdeb6e3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5eb6e7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0feb6ea, SUC_lexicon_tags_15 },
    { 0xaf3eb6ea, SUC_lexicon_tags_33 },
    { 0xafbeb6ec, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09eb6f0, SUC_lexicon_tags_7 },
    { 0x3a9eb6f1, SUC_lexicon_tags_25 },
    { 0x1d1eb6f1, SUC_lexicon_tags_15 },
    { 0x76beb6f0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xff1eb6f5, SUC_lexicon_tags_33 },
    { 0x1deb6f6, SUC_lexicon_tags_8 },
    { 0xd63eb6f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d1eb6f9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x34deb6fc, SUC_lexicon_tags_2 },
    { 0xbb3eb6fc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27eb702, SUC_lexicon_tags_41 },
    { 0x4feb702, SUC_lexicon_tags_60 },
    { 0xd9deb703, SUC_lexicon_tags_45 },
    { 0x323eb704, SUC_lexicon_tags_2 },
    { 0x197eb703, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1eb70b, SUC_lexicon_tags_644 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86feb70f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11eb714, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x419eb716, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25feb71b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa29eb71d, SUC_lexicon_tags_7 },
    { 0xf5eb71e, SUC_lexicon_tags_33 },
    { 0xbd7eb71e, SUC_lexicon_tags_1 },
    { 0xe75eb71f, SUC_lexicon_tags_35 },
    { 0x419eb71f, SUC_lexicon_tags_25 },
    { 0xb11eb71d, SUC_lexicon_tags_37 },
    { 0x9f9eb723, SUC_lexicon_tags_7 },
    { 0xe3feb71f, SUC_lexicon_tags_11 },
    { 0x61beb722, SUC_lexicon_tags_40 },
    { 0x3e9eb726, SUC_lexicon_tags_15 },
    { 0xcbeb727, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829eb72c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967eb732, SUC_lexicon_tags_17 },
    { 0x83eb733, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107eb739, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x26deb73c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9eb73f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cfeb742, SUC_lexicon_tags_33 },
    { 0xd0feb743, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37eb746, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadeb74c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x743eb74e, SUC_lexicon_tags_11 },
    { 0x8a1eb74f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x605eb752, SUC_lexicon_tags_15 },
    { 0x5eb753, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5eb75c, SUC_lexicon_tags_2 },
    { 0x191eb75d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x785eb75f, SUC_lexicon_tags_19 },
    { 0xf83eb760, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951eb76a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187eb770, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3deb773, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915eb777, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbeb77a, SUC_lexicon_tags_47 },
    { 0x7cfeb77b, SUC_lexicon_tags_48 },
    { 0xfc1eb77c, SUC_lexicon_tags_15 },
    { 0xd51eb77c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6feb781, SUC_lexicon_tags_2 },
    { 0xdb1eb781, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731eb786, SUC_lexicon_tags_9 },
    { 0x5deb787, SUC_lexicon_tags_28 },
    { 0xe3beb788, SUC_lexicon_tags_17 },
    { 0xd93eb789, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedeb78e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x379eb791, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7eb794, SUC_lexicon_tags_9 },
    { 0x36deb795, SUC_lexicon_tags_87 },
    { 0xab1eb795, SUC_lexicon_tags_12 },
    { 0x409eb797, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9eb79f, SUC_lexicon_tags_15 },
    { 0x531eb7a0, SUC_lexicon_tags_28 },
    { 0x63deb7a1, SUC_lexicon_tags_49 },
    { 0xce1eb79f, SUC_lexicon_tags_37 },
    { 0x725eb7a0, SUC_lexicon_tags_17 },
    { 0xb91eb79f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3eb7a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf19eb7ab, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8ddeb7ad, SUC_lexicon_tags_47 },
    { 0x349eb7ad, SUC_lexicon_tags_14 },
    { 0x2f5eb7af, SUC_lexicon_tags_38 },
    { 0x505eb7af, SUC_lexicon_tags_118 },
    { 0xb03eb7ad, SUC_lexicon_tags_25 },
    { 0xcbbeb7ad, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd7deb7b4, SUC_lexicon_tags_19 },
    { 0x571eb7b5, SUC_lexicon_tags_17 },
    { 0x85deb7b6, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5eb7b9, SUC_lexicon_tags_0 },
    { 0xcc7eb7ba, SUC_lexicon_tags_63 },
    { 0x86beb7bb, SUC_lexicon_tags_12 },
    { 0x58beb7bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535eb7c2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5bbeb7c4, SUC_lexicon_tags_12 },
    { 0xab3eb7c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x98feb7c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529eb7cd, SUC_lexicon_tags_11 },
    { 0xdedeb7cd, SUC_lexicon_tags_66 },
    { 0x809eb7cf, SUC_lexicon_tags_124 },
    { 0x959eb7ce, SUC_lexicon_tags_7 },
    { 0x293eb7d1, SUC_lexicon_tags_66 },
    { 0x45feb7d1, SUC_lexicon_tags_27 },
    { 0x851eb7d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe59eb7d5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x599eb7d7, SUC_lexicon_tags_32 },
    { 0xdb3eb7d8, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1beb7df, SUC_lexicon_tags_31 },
    { 0xf47eb7e0, SUC_lexicon_tags_15 },
    { 0x9b9eb7e1, SUC_lexicon_tags_19 },
    { 0xf6beb7e2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfeb7e8, SUC_lexicon_tags_12 },
    { 0x315eb7e8, SUC_lexicon_tags_15 },
    { 0x9fdeb7ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99eb7ef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7d1eb7f1, SUC_lexicon_tags_26 },
    { 0x14feb7f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffeb7f6, SUC_lexicon_tags_16 },
    { 0x869eb7f6, SUC_lexicon_tags_32 },
    { 0x5c1eb7f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb81eb7fb, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xa49eb7fd, SUC_lexicon_tags_17 },
    { 0x373eb7fe, SUC_lexicon_tags_63 },
    { 0x77feb7ff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbeb80c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93eb80f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5deb814, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383eb818, SUC_lexicon_tags_45 },
    { 0xbb9eb819, SUC_lexicon_tags_87 },
    { 0xfedeb81a, SUC_lexicon_tags_27 },
    { 0x57deb81b, SUC_lexicon_tags_43 },
    { 0xec9eb81c, SUC_lexicon_tags_37 },
    { 0x915eb81b, SUC_lexicon_tags_19 },
    { 0x347eb81e, SUC_lexicon_tags_31 },
    { 0xf41eb81f, SUC_lexicon_tags_1 },
    { 0x8cdeb81d, SUC_lexicon_tags_1 },
    { 0x8dbeb81f, SUC_lexicon_tags_7 },
    { 0x599eb822, SUC_lexicon_tags_66 },
    { 0xe6feb823, SUC_lexicon_tags_1 },
    { 0x97feb822, SUC_lexicon_tags_17 },
    { 0xdfbeb819, SUC_lexicon_tags_11 },
    { 0x73feb81c, SUC_lexicon_tags_19 },
    { 0x481eb827, SUC_lexicon_tags_65 },
    { 0x8f3eb828, SUC_lexicon_tags_15 },
    { 0x4f9eb829, SUC_lexicon_tags_27 },
    { 0xf5beb82a, SUC_lexicon_tags_15 },
    { 0x923eb82b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb6deb82d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x555eb82f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7eb832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61eb83d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdb3eb83f, SUC_lexicon_tags_1 },
    { 0xac9eb83f, SUC_lexicon_tags_27 },
    { 0x7c5eb841, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66beb84d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x23beb850, SUC_lexicon_tags_26 },
    { 0x881eb851, SUC_lexicon_tags_37 },
    { 0xce5eb851, SUC_lexicon_tags_63 },
    { 0x1c7eb851, SUC_lexicon_tags_63 },
    { 0x9b9eb854, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781eb859, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67eb85d, SUC_lexicon_tags_17 },
    { 0xfe7eb85d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9eb861, SUC_lexicon_tags_60 },
    { 0x881eb862, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24feb866, SUC_lexicon_tags_44 },
    { 0xbcbeb867, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5cdeb869, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xca1eb86c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xbefeb86e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x65eb870, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1eb874, SUC_lexicon_tags_34 },
    { 0xc5feb875, SUC_lexicon_tags_27 },
    { 0xdeb876, SUC_lexicon_tags_17 },
    { 0x90feb876, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x14feb87a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdeb87f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x275eb881, SUC_lexicon_tags_26 },
    { 0x21deb881, SUC_lexicon_tags_1 },
    { 0xba9eb883, SUC_lexicon_tags_31 },
    { 0xb51eb883, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x135eb886, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8c5eb888, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x33deb88b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7eb88e, SUC_lexicon_tags_16 },
    { 0x5efeb88e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7feb896, SUC_lexicon_tags_3 },
    { 0x14feb896, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdeb89a, SUC_lexicon_tags_43 },
    { 0x8dfeb89b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7e3eb89d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef7eb89f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3eb8a3, SUC_lexicon_tags_25 },
    { 0xec9eb8a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbadeb8a6, SUC_lexicon_tags_32 },
    { 0x4cbeb8a6, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x82deb8a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7eb8b4, SUC_lexicon_tags_2 },
    { 0xb09eb8b5, SUC_lexicon_tags_30 },
    { 0x835eb8b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71eb8ba, SUC_lexicon_tags_31 },
    { 0x245eb8bb, SUC_lexicon_tags_33 },
    { 0xe61eb8bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x94feb8bf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9eb8c2, SUC_lexicon_tags_14 },
    { 0x8a9eb8c3, SUC_lexicon_tags_29 },
    { 0x98beb8c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7eb8c8, SUC_lexicon_tags_14 },
    { 0x351eb8c9, SUC_lexicon_tags_37 },
    { 0x447eb8ca, SUC_lexicon_tags_68 },
    { 0xac5eb8cb, SUC_lexicon_tags_12 },
    { 0x271eb8c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xde9eb8ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427eb8d2, SUC_lexicon_tags_6 },
    { 0x2fdeb8d3, SUC_lexicon_tags_32 },
    { 0xb75eb8d3, SUC_lexicon_tags_23 },
    { 0x7d5eb8d2, SUC_lexicon_tags_1 },
    { 0xdd9eb8d6, SUC_lexicon_tags_12 },
    { 0x147eb8d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc4deb8d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x2b1eb8db, SUC_lexicon_tags_19 },
    { 0xdcbeb8dc, SUC_lexicon_tags_2 },
    { 0x16beb8db, SUC_lexicon_tags_15 },
    { 0x2dfeb8de, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddeb8e2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9eb8e5, SUC_lexicon_tags_32 },
    { 0xdc9eb8e6, SUC_lexicon_tags_10 },
    { 0xe3beb8e7, SUC_lexicon_tags_17 },
    { 0x1b3eb8e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999eb8ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9b1eb8ef, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25eb8f4, SUC_lexicon_tags_27 },
    { 0x2edeb8f4, SUC_lexicon_tags_1 },
    { 0x40feb8f6, SUC_lexicon_tags_28 },
    { 0x6adeb8f7, SUC_lexicon_tags_8 },
    { 0x767eb8f7, SUC_lexicon_tags_12 },
    { 0x999eb8f9, SUC_lexicon_tags_25 },
    { 0x3b5eb8f9, SUC_lexicon_tags_12 },
    { 0xa35eb8f7, SUC_lexicon_tags_37 },
    { 0x3a7eb8fc, SUC_lexicon_tags_22 },
    { 0x6b3eb8f4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9adeb8ff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7eb905, SUC_lexicon_tags_32 },
    { 0x84beb906, SUC_lexicon_tags_43 },
    { 0xffeb905, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55eb90a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85beb90e, SUC_lexicon_tags_87 },
    { 0xa25eb90e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbefeb911, SUC_lexicon_tags_7 },
    { 0x94beb912, SUC_lexicon_tags_12 },
    { 0x445eb913, SUC_lexicon_tags_46 },
    { 0xf7deb911, SUC_lexicon_tags_12 },
    { 0xd1feb915, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe3feb917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedfeb919, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3a3eb91b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x921eb91d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x239eb920, SUC_lexicon_tags_3 },
    { 0xe7feb921, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x473eb924, SUC_lexicon_tags_8 },
    { 0x631eb925, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x417eb927, SUC_lexicon_tags_33 },
    { 0xfb9eb928, SUC_lexicon_tags_15 },
    { 0x5d9eb929, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x679eb92b, SUC_lexicon_tags_37 },
    { 0x46beb92b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7eb930, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33eb934, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x24deb937, SUC_lexicon_tags_49 },
    { 0x267eb938, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdeb93b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5eb93e, SUC_lexicon_tags_28 },
    { 0xd31eb93f, SUC_lexicon_tags_17 },
    { 0xfbfeb940, SUC_lexicon_tags_19 },
    { 0x871eb940, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf1eb945, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x295eb947, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x293eb94c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afeb954, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1eb959, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13eb95f, SUC_lexicon_tags_6 },
    { 0x6bbeb960, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3deb965, SUC_lexicon_tags_16 },
    { 0xc45eb966, SUC_lexicon_tags_241 },
    { 0x981eb967, SUC_lexicon_tags_45 },
    { 0xe7feb968, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7eb977, SUC_lexicon_tags_5 },
    { 0xf99eb978, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac1eb97a, SUC_lexicon_tags_37 },
    { 0x8fbeb97b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3eb980, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48feb986, SUC_lexicon_tags_11 },
    { 0xa33eb987, SUC_lexicon_tags_11 },
    { 0x935eb986, SUC_lexicon_tags_10 },
    { 0x71eb986, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a9eb98b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525eb996, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x173eb998, SUC_lexicon_tags_47 },
    { 0xad9eb999, SUC_lexicon_tags_7 },
    { 0x9f1eb998, SUC_lexicon_tags_43 },
    { 0xf33eb99b, SUC_lexicon_tags_46 },
    { 0xefeb99a, SUC_lexicon_tags_11 },
    { 0xbebeb99d, SUC_lexicon_tags_23 },
    { 0x4c5eb99e, SUC_lexicon_tags_14 },
    { 0xa33eb99d, SUC_lexicon_tags_9 },
    { 0x145eb9a0, SUC_lexicon_tags_27 },
    { 0x9cfeb9a1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x1ddeb9a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfbfeb9a5, SUC_lexicon_tags_25 },
    { 0x733eb9a6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdeb9a9, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7feb9ad, SUC_lexicon_tags_25 },
    { 0xabbeb9ae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe6beb9b0, SUC_lexicon_tags_27 },
    { 0x809eb9b1, SUC_lexicon_tags_2 },
    { 0xe79eb9b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9bbeb9b4, SUC_lexicon_tags_57 },
    { 0x623eb9b5, SUC_lexicon_tags_2 },
    { 0xa97eb9b6, SUC_lexicon_tags_37 },
    { 0x931eb9b6, SUC_lexicon_tags_82 },
    { 0xb53eb9b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3eb9bb, SUC_lexicon_tags_82 },
    { 0x463eb9bb, SUC_lexicon_tags_33 },
    { 0x7e5eb9bb, SUC_lexicon_tags_21 },
    { 0x2d7eb9bc, SUC_lexicon_tags_17 },
    { 0x32beb9bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42feb9c1, SUC_lexicon_tags_11 },
    { 0x63deb9c1, SUC_lexicon_tags_8 },
    { 0x4cfeb9c3, SUC_lexicon_tags_0 },
    { 0x35beb9c3, SUC_lexicon_tags_25 },
    { 0x17beb9c5, SUC_lexicon_tags_9 },
    { 0x65deb9c2, SUC_lexicon_tags_17 },
    { 0xca1eb9c7, SUC_lexicon_tags_11 },
    { 0x7f9eb9c7, SUC_lexicon_tags_45 },
    { 0x8bfeb9c5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487eb9d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x683eb9d4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3e3eb9d6, SUC_lexicon_tags_29 },
    { 0x51eb9d7, SUC_lexicon_tags_30 },
    { 0x9ebeb9d7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x899eb9db, SUC_lexicon_tags_2 },
    { 0xd15eb9dc, SUC_lexicon_tags_45 },
    { 0x64deb9dd, SUC_lexicon_tags_11 },
    { 0x425eb9de, SUC_lexicon_tags_19 },
    { 0x90feb9df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e9eb9e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3eb9e4, SUC_lexicon_tags_44 },
    { 0x9f3eb9e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdeb9e8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3eb9eb, SUC_lexicon_tags_16 },
    { 0x4a9eb9ec, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99eb9f3, SUC_lexicon_tags_63 },
    { 0x445eb9f4, SUC_lexicon_tags_63 },
    { 0x7fbeb9f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3eb9fc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9cbeb9fe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x739eba00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xad3eba02, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5eba05, SUC_lexicon_tags_7 },
    { 0xc07eba05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76beba08, SUC_lexicon_tags_49 },
    { 0xdf1eba09, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x373eba0b, SUC_lexicon_tags_1 },
    { 0x667eba0c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf8beba0e, SUC_lexicon_tags_66 },
    { 0xa4beba0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28deba15, SUC_lexicon_tags_17 },
    { 0x41beba16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13eba1b, SUC_lexicon_tags_38 },
    { 0x61eba1b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbeba1f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf19eba22, SUC_lexicon_tags_33 },
    { 0xcf3eba23, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9eba29, SUC_lexicon_tags_17 },
    { 0xbfeba2a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa7beba2c, SUC_lexicon_tags_37 },
    { 0xb67eba2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85eba2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951eba33, SUC_lexicon_tags_6 },
    { 0xdc1eba34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159eba3d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebeba44, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25eba47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadeba4a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791eba4f, SUC_lexicon_tags_1389 },
    { 0xee9eba50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbb5eba52, SUC_lexicon_tags_38 },
    { 0x721eba52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa4deba55, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3f5eba57, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817eba5e, SUC_lexicon_tags_27 },
    { 0xfd5eba5f, SUC_lexicon_tags_25 },
    { 0x63beba60, SUC_lexicon_tags_104 },
    { 0x561eba60, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65eba65, SUC_lexicon_tags_29 },
    { 0x661eba66, SUC_lexicon_tags_14 },
    { 0x221eba67, SUC_lexicon_tags_25 },
    { 0xf69eba68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63eba6b, SUC_lexicon_tags_37 },
    { 0xfadeba6c, SUC_lexicon_tags_7 },
    { 0x6dbeba6b, SUC_lexicon_tags_37 },
    { 0x28feba6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85beba72, SUC_lexicon_tags_6 },
    { 0x81deba73, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6b9eba75, SUC_lexicon_tags_17 },
    { 0x53deba75, SUC_lexicon_tags_21 },
    { 0xdf7eba77, SUC_lexicon_tags_26 },
    { 0x557eba77, SUC_lexicon_tags_37 },
    { 0x64deba76, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4bdeba7b, SUC_lexicon_tags_19 },
    { 0xb19eba7c, SUC_lexicon_tags_12 },
    { 0xde1eba7d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48beba81, SUC_lexicon_tags_17 },
    { 0xfe7eba82, SUC_lexicon_tags_11 },
    { 0xa75eba81, SUC_lexicon_tags_40 },
    { 0xd7eba84, SUC_lexicon_tags_37 },
    { 0xbc3eba85, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381eba8b, SUC_lexicon_tags_25 },
    { 0xc75eba8c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dbeba8e, SUC_lexicon_tags_13 },
    { 0x9ebeba8f, SUC_lexicon_tags_64 },
    { 0x6bbeba90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x851eba93, SUC_lexicon_tags_61 },
    { 0x889eba94, SUC_lexicon_tags_9 },
    { 0xfb5eba95, SUC_lexicon_tags_15 },
    { 0x13beba95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x263eba98, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xca3eba9a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705eba9e, SUC_lexicon_tags_10 },
    { 0xaedeba9f, SUC_lexicon_tags_1 },
    { 0x6a3ebaa0, SUC_lexicon_tags_45 },
    { 0x937eba9f, SUC_lexicon_tags_33 },
    { 0x78debaa2, SUC_lexicon_tags_61 },
    { 0xfabebaa3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x92debaa5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bebaa8, SUC_lexicon_tags_27 },
    { 0x477ebaa8, SUC_lexicon_tags_15 },
    { 0x417ebaa9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195ebaae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7ebab6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xaebebab9, SUC_lexicon_tags_851 },
    { 0x909ebaba, SUC_lexicon_tags_63 },
    { 0xe13ebabb, SUC_lexicon_tags_30 },
    { 0x8f5ebabb, SUC_lexicon_tags_12 },
    { 0xd75ebabd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x215ebac0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68debac2, SUC_lexicon_tags_14 },
    { 0xcadebac2, SUC_lexicon_tags_11 },
    { 0x10bebac4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1febac8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc13ebaca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201ebacf, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5ebad4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x371ebad7, SUC_lexicon_tags_2 },
    { 0xfcfebad8, SUC_lexicon_tags_37 },
    { 0x99febad8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x855ebadb, SUC_lexicon_tags_114 },
    { 0xb37ebadc, SUC_lexicon_tags_63 },
    { 0x6d5ebadd, SUC_lexicon_tags_37 },
    { 0x29debade, SUC_lexicon_tags_2 },
    { 0xcf3ebadc, SUC_lexicon_tags_1 },
    { 0xbfebae0, SUC_lexicon_tags_25 },
    { 0x44debae1, SUC_lexicon_tags_32 },
    { 0x989ebae2, SUC_lexicon_tags_37 },
    { 0xcc5ebae3, SUC_lexicon_tags_11 },
    { 0x297ebae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5dbebae6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x771ebae8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21ebaed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499ebaf2, SUC_lexicon_tags_39 },
    { 0x4efebaf3, SUC_lexicon_tags_23 },
    { 0x6d3ebaf4, SUC_lexicon_tags_10 },
    { 0x7ebaf5, SUC_lexicon_tags_149 },
    { 0x103ebaf5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x59debaf8, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x89ebafa, SUC_lexicon_tags_2 },
    { 0x2fbebafb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73ebb04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x789ebb07, SUC_lexicon_tags_33 },
    { 0x37ebb08, SUC_lexicon_tags_7 },
    { 0xe51ebb09, SUC_lexicon_tags_0 },
    { 0x97bebb07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1febb14, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5c1ebb16, SUC_lexicon_tags_13 },
    { 0x1bfebb16, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xaf7ebb19, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73debb1f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11febb25, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x62debb27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75debb29, SUC_lexicon_tags_49 },
    { 0xcbfebb2a, SUC_lexicon_tags_12 },
    { 0xd87ebb2a, SUC_lexicon_tags_0 },
    { 0xaabebb2a, SUC_lexicon_tags_21 },
    { 0x43bebb2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb3ebb2f, SUC_lexicon_tags_63 },
    { 0xbf7ebb30, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135ebb35, SUC_lexicon_tags_382 },
    { 0, NULL },
    { 0xf01ebb37, SUC_lexicon_tags_0 },
    { 0x2e3ebb38, SUC_lexicon_tags_25 },
    { 0xa73ebb38, SUC_lexicon_tags_11 },
    { 0x8c5ebb39, SUC_lexicon_tags_17 },
    { 0xf97ebb3b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xa7debb3d, SUC_lexicon_tags_37 },
    { 0xac5ebb3e, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x7f7ebb40, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x41febb43, SUC_lexicon_tags_16 },
    { 0x4b7ebb44, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x807ebb46, SUC_lexicon_tags_37 },
    { 0xac7ebb47, SUC_lexicon_tags_11 },
    { 0x3ddebb47, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1ebb4e, SUC_lexicon_tags_43 },
    { 0xda1ebb4f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3efebb57, SUC_lexicon_tags_2 },
    { 0xaf1ebb58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58febb5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5ebb5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb5ebb61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x641ebb64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35ebb69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x691ebb6c, SUC_lexicon_tags_49 },
    { 0x971ebb6c, SUC_lexicon_tags_15 },
    { 0x945ebb6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23debb74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75ebb7a, SUC_lexicon_tags_12 },
    { 0x761ebb7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfebb80, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9ebb8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbebb90, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bebb93, SUC_lexicon_tags_15 },
    { 0xab9ebb94, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0x621ebb97, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67febb9c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x987ebb9e, SUC_lexicon_tags_26 },
    { 0x88bebb9e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1ebba4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93ebba8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x39febbaa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x139ebbad, SUC_lexicon_tags_27 },
    { 0xc0bebbae, SUC_lexicon_tags_57 },
    { 0x8a3ebbad, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x89debbb2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05ebbb5, SUC_lexicon_tags_29 },
    { 0x651ebbb5, SUC_lexicon_tags_23 },
    { 0xd77ebbb7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625ebbbe, SUC_lexicon_tags_0 },
    { 0x1d5ebbbf, SUC_lexicon_tags_11 },
    { 0xd91ebbbe, SUC_lexicon_tags_17 },
    { 0x36bebbbf, SUC_lexicon_tags_17 },
    { 0x371ebbc2, SUC_lexicon_tags_11 },
    { 0x5edebbc1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f7ebbc5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bebbca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x159ebbcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x621ebbce, SUC_lexicon_tags_28 },
    { 0xd2debbcf, SUC_lexicon_tags_12 },
    { 0x971ebbcf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17febbd5, SUC_lexicon_tags_11 },
    { 0xba3ebbd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415ebbda, SUC_lexicon_tags_55 },
    { 0x2afebbda, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x233ebbdd, SUC_lexicon_tags_66 },
    { 0xf8debbde, SUC_lexicon_tags_21 },
    { 0x775ebbdf, SUC_lexicon_tags_17 },
    { 0xa6bebbdf, SUC_lexicon_tags_12 },
    { 0x353ebbe1, SUC_lexicon_tags_12 },
    { 0x6b9ebbe1, SUC_lexicon_tags_28 },
    { 0x267ebbdd, SUC_lexicon_tags_46 },
    { 0x329ebbe4, SUC_lexicon_tags_47 },
    { 0x6c3ebbde, SUC_lexicon_tags_47 },
    { 0x39bebbe5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69ebbf1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1febbf4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x967ebbf6, SUC_lexicon_tags_47 },
    { 0x45febbf6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c1ebbf9, SUC_lexicon_tags_6 },
    { 0x553ebbfa, SUC_lexicon_tags_7 },
    { 0x805ebbfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ebc00, SUC_lexicon_tags_12 },
    { 0x9bfebc01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x377ebc04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3ebc07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957ebc10, SUC_lexicon_tags_15 },
    { 0x17febc11, SUC_lexicon_tags_15 },
    { 0xb6bebc12, SUC_lexicon_tags_248 },
    { 0x929ebc10, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bebc17, SUC_lexicon_tags_12 },
    { 0x9a9ebc18, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7debc1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bebc1f, SUC_lexicon_tags_19 },
    { 0xfb7ebc1f, SUC_lexicon_tags_25 },
    { 0x887ebc20, SUC_lexicon_tags_12 },
    { 0xdfebc22, SUC_lexicon_tags_27 },
    { 0xb4debc21, SUC_lexicon_tags_37 },
    { 0x823ebc24, SUC_lexicon_tags_19 },
    { 0xb7bebc25, SUC_lexicon_tags_37 },
    { 0xa95ebc26, SUC_lexicon_tags_23 },
    { 0x4d7ebc27, SUC_lexicon_tags_25 },
    { 0xa4bebc28, SUC_lexicon_tags_12 },
    { 0x7d7ebc21, SUC_lexicon_tags_6 },
    { 0x663ebc24, SUC_lexicon_tags_17 },
    { 0xbebebc2b, SUC_lexicon_tags_15 },
    { 0x29debc2c, SUC_lexicon_tags_7 },
    { 0xc7bebc28, SUC_lexicon_tags_43 },
    { 0x30bebc2d, SUC_lexicon_tags_63 },
    { 0x1f3ebc28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbebc32, SUC_lexicon_tags_17 },
    { 0x859ebc33, SUC_lexicon_tags_49 },
    { 0xd0febc34, SUC_lexicon_tags_31 },
    { 0xc4febc34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x359ebc37, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe63ebc39, SUC_lexicon_tags_31 },
    { 0xb87ebc3a, SUC_lexicon_tags_55 },
    { 0xfafebc3b, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99ebc3f, SUC_lexicon_tags_208 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481ebc43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x341ebc46, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1ebc49, SUC_lexicon_tags_4 },
    { 0x425ebc4a, SUC_lexicon_tags_4 },
    { 0x24debc4b, SUC_lexicon_tags_68 },
    { 0xc43ebc4a, SUC_lexicon_tags_2 },
    { 0xb4bebc4b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xebebc50, SUC_lexicon_tags_25 },
    { 0x93ebc51, SUC_lexicon_tags_34 },
    { 0x597ebc52, SUC_lexicon_tags_12 },
    { 0x7b5ebc51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337ebc58, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67ebc5b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff3ebc5d, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35ebc62, SUC_lexicon_tags_46 },
    { 0xe4bebc63, SUC_lexicon_tags_2 },
    { 0xd2bebc64, SUC_lexicon_tags_12 },
    { 0x6a3ebc65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921ebc6a, SUC_lexicon_tags_39 },
    { 0x941ebc6a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbebc6f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adebc73, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x649ebc75, SUC_lexicon_tags_11 },
    { 0xc7febc76, SUC_lexicon_tags_15 },
    { 0xb7bebc76, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x413ebc79, SUC_lexicon_tags_201 },
    { 0xc03ebc79, SUC_lexicon_tags_12 },
    { 0xb0bebc79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9ebc82, SUC_lexicon_tags_33 },
    { 0x879ebc83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bebc8b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bebc91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5ebc96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7ebc99, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573ebca0, SUC_lexicon_tags_27 },
    { 0x3afebca1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17ebca5, SUC_lexicon_tags_37 },
    { 0xe45ebca5, SUC_lexicon_tags_37 },
    { 0x9e1ebca5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaafebca9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebebcac, SUC_lexicon_tags_6 },
    { 0xe1febcac, SUC_lexicon_tags_23 },
    { 0xecdebcac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ebcb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa87ebcb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927ebcb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x695ebcbc, SUC_lexicon_tags_38 },
    { 0x37ebcbd, SUC_lexicon_tags_4 },
    { 0xc01ebcbe, SUC_lexicon_tags_6 },
    { 0x713ebcbd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9ebcc2, SUC_lexicon_tags_11 },
    { 0x3b3ebcc3, SUC_lexicon_tags_4 },
    { 0x327ebcc2, SUC_lexicon_tags_93 },
    { 0xaedebcc5, SUC_lexicon_tags_27 },
    { 0xfa3ebcc5, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87ebcca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7debcd4, SUC_lexicon_tags_33 },
    { 0xf8febcd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x84debcd7, SUC_lexicon_tags_11 },
    { 0x863ebcd7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x213ebcdb, SUC_lexicon_tags_28 },
    { 0x891ebcdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7abebcde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1ebce4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83ebce7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365ebceb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525ebcf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157ebcf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5febcf9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb89ebcfb, SUC_lexicon_tags_27 },
    { 0xa9ebcfc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3debcfe, SUC_lexicon_tags_77 },
    { 0x77febcff, SUC_lexicon_tags_27 },
    { 0x95debcff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xebbebd02, SUC_lexicon_tags_15 },
    { 0xc7ebd03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6febd06, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd3debd08, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1ebd0c, SUC_lexicon_tags_12 },
    { 0x7ebebd0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3ebd10, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x277ebd12, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5ebd19, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x63febd1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffebd1f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ebd22, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161ebd26, SUC_lexicon_tags_49 },
    { 0xb7bebd27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd4bebd29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1ebd2c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bebd32, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89ebd38, SUC_lexicon_tags_7 },
    { 0x8c9ebd39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x965ebd3c, SUC_lexicon_tags_0 },
    { 0xc21ebd3c, SUC_lexicon_tags_34 },
    { 0xd09ebd3e, SUC_lexicon_tags_11 },
    { 0x65debd3f, SUC_lexicon_tags_26 },
    { 0xd45ebd3c, SUC_lexicon_tags_19 },
    { 0x1ffebd40, SUC_lexicon_tags_27 },
    { 0xc17ebd40, SUC_lexicon_tags_82 },
    { 0xc17ebd43, SUC_lexicon_tags_19 },
    { 0x177ebd43, SUC_lexicon_tags_14 },
    { 0x3ebebd45, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3ebd4d, SUC_lexicon_tags_15 },
    { 0x16febd4e, SUC_lexicon_tags_9 },
    { 0x5c9ebd4f, SUC_lexicon_tags_46 },
    { 0x7dbebd50, SUC_lexicon_tags_16 },
    { 0xa33ebd50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5ebd55, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643ebd5d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ebd65, SUC_lexicon_tags_50 },
    { 0xb25ebd65, SUC_lexicon_tags_23 },
    { 0xb75ebd67, SUC_lexicon_tags_213 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabebd6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa07ebd6d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1dbebd6f, SUC_lexicon_tags_14 },
    { 0x887ebd70, SUC_lexicon_tags_17 },
    { 0xd9febd71, SUC_lexicon_tags_31 },
    { 0xca3ebd70, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bebd79, SUC_lexicon_tags_23 },
    { 0x2b3ebd7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537ebd7e, SUC_lexicon_tags_87 },
    { 0x5adebd7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2afebd85, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x773ebd87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51ebd91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5ebd94, SUC_lexicon_tags_11 },
    { 0x40bebd95, SUC_lexicon_tags_95 },
    { 0x9c7ebd96, SUC_lexicon_tags_47 },
    { 0xda9ebd95, SUC_lexicon_tags_7 },
    { 0xb99ebd95, SUC_lexicon_tags_238 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdebd9b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x53debd9d, SUC_lexicon_tags_17 },
    { 0xcc9ebd9e, SUC_lexicon_tags_11 },
    { 0x75debd9f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1ebda2, SUC_lexicon_tags_2 },
    { 0x925ebda3, SUC_lexicon_tags_63 },
    { 0xae1ebda4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133ebda8, SUC_lexicon_tags_17 },
    { 0x3d7ebda8, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x61bebdac, SUC_lexicon_tags_15 },
    { 0xcf3ebdad, SUC_lexicon_tags_25 },
    { 0xeadebdae, SUC_lexicon_tags_39 },
    { 0x9dfebdaf, SUC_lexicon_tags_7 },
    { 0x757ebdac, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ebdb5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x603ebdb7, SUC_lexicon_tags_17 },
    { 0xbbdebdb8, SUC_lexicon_tags_27 },
    { 0xe7debdb9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ebdc1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xae3ebdc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1febdc6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57debdca, SUC_lexicon_tags_47 },
    { 0xb4debdcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ebdd0, SUC_lexicon_tags_21 },
    { 0xbd5ebdd1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xebdebdd3, SUC_lexicon_tags_28 },
    { 0x371ebdd4, SUC_lexicon_tags_84 },
    { 0x123ebdd5, SUC_lexicon_tags_15 },
    { 0x349ebdd6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5ebddc, SUC_lexicon_tags_27 },
    { 0x7c1ebddd, SUC_lexicon_tags_37 },
    { 0xe8debdde, SUC_lexicon_tags_65 },
    { 0xb55ebddf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ebde7, SUC_lexicon_tags_23 },
    { 0x471ebde7, SUC_lexicon_tags_37 },
    { 0xbdfebde9, SUC_lexicon_tags_22 },
    { 0x861ebde8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xee1ebdec, SUC_lexicon_tags_16 },
    { 0x271ebdec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x603ebdf0, SUC_lexicon_tags_12 },
    { 0xc19ebdf0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x2b3ebdf3, SUC_lexicon_tags_27 },
    { 0x3bdebdf4, SUC_lexicon_tags_10 },
    { 0x31bebdf5, SUC_lexicon_tags_7 },
    { 0xb73ebdf6, SUC_lexicon_tags_11 },
    { 0x5ddebdf5, SUC_lexicon_tags_43 },
    { 0x25debdf3, SUC_lexicon_tags_1 },
    { 0xb17ebdf9, SUC_lexicon_tags_12 },
    { 0xd21ebdf9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbebe00, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8d1ebe02, SUC_lexicon_tags_37 },
    { 0x405ebe03, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483ebe07, SUC_lexicon_tags_11 },
    { 0x3ffebe08, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb29ebe0a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9ebe0d, SUC_lexicon_tags_12 },
    { 0xf9bebe0e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119ebe16, SUC_lexicon_tags_68 },
    { 0x58debe17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x815ebe19, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x741ebe1b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78bebe20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3febe24, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ebe28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5debe30, SUC_lexicon_tags_15 },
    { 0x3bebe31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31ebe35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bebe38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34bebe3a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7febe3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9ebe42, SUC_lexicon_tags_14 },
    { 0xee3ebe42, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdebe47, SUC_lexicon_tags_65 },
    { 0x1a9ebe48, SUC_lexicon_tags_0 },
    { 0x947ebe49, SUC_lexicon_tags_70 },
    { 0x623ebe47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x801ebe4d, SUC_lexicon_tags_37 },
    { 0x293ebe4e, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb87ebe50, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x507ebe52, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3ebe55, SUC_lexicon_tags_48 },
    { 0x579ebe55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x22debe58, SUC_lexicon_tags_30 },
    { 0x7b3ebe59, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56febe5e, SUC_lexicon_tags_47 },
    { 0x55bebe5f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753ebe63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x865ebe65, SUC_lexicon_tags_27 },
    { 0xe5ebe65, SUC_lexicon_tags_13 },
    { 0x68debe66, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211ebe71, SUC_lexicon_tags_23 },
    { 0x171ebe71, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7ebe77, SUC_lexicon_tags_17 },
    { 0xb3debe77, SUC_lexicon_tags_45 },
    { 0x5abebe78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x791ebe7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3debe7f, SUC_lexicon_tags_17 },
    { 0xaa7ebe80, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89ebe83, SUC_lexicon_tags_11 },
    { 0x29febe84, SUC_lexicon_tags_37 },
    { 0xbf1ebe84, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9ebe8d, SUC_lexicon_tags_7 },
    { 0x61debe8e, SUC_lexicon_tags_7 },
    { 0xd39ebe8f, SUC_lexicon_tags_2 },
    { 0x55bebe8f, SUC_lexicon_tags_11 },
    { 0x6febe8f, SUC_lexicon_tags_33 },
    { 0xc5febe91, SUC_lexicon_tags_30 },
    { 0x7c3ebe90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9ebe97, SUC_lexicon_tags_1 },
    { 0xc5febe98, SUC_lexicon_tags_28 },
    { 0x697ebe98, SUC_lexicon_tags_25 },
    { 0x445ebe9a, SUC_lexicon_tags_5 },
    { 0xb9ebe9a, SUC_lexicon_tags_29 },
    { 0x8f3ebe9b, SUC_lexicon_tags_2 },
    { 0xe61ebe9c, SUC_lexicon_tags_14 },
    { 0xdc3ebe9e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1afebea0, SUC_lexicon_tags_11 },
    { 0xc81ebea1, SUC_lexicon_tags_10 },
    { 0xdc1ebea2, SUC_lexicon_tags_1 },
    { 0xa5ebea3, SUC_lexicon_tags_1 },
    { 0xaf7ebea1, SUC_lexicon_tags_12 },
    { 0x17ebea5, SUC_lexicon_tags_33 },
    { 0xf23ebea6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e9ebeaa, SUC_lexicon_tags_12 },
    { 0x607ebeab, SUC_lexicon_tags_8 },
    { 0xc57ebeac, SUC_lexicon_tags_31 },
    { 0xa7debeaa, SUC_lexicon_tags_12 },
    { 0x917ebeae, SUC_lexicon_tags_13 },
    { 0x805ebeaf, SUC_lexicon_tags_31 },
    { 0xc6debeaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91ebeb3, SUC_lexicon_tags_46 },
    { 0x8dbebeb4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x111ebeb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7ebeb9, SUC_lexicon_tags_37 },
    { 0x341ebeb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9febebd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4f3ebebf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x20febec2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bebec8, SUC_lexicon_tags_37 },
    { 0xb09ebec8, SUC_lexicon_tags_37 },
    { 0x635ebec9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733ebecf, SUC_lexicon_tags_23 },
    { 0x799ebecf, SUC_lexicon_tags_27 },
    { 0x6afebecf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6afebed4, SUC_lexicon_tags_8 },
    { 0xacdebed5, SUC_lexicon_tags_25 },
    { 0xc4bebed5, SUC_lexicon_tags_11 },
    { 0xcd9ebed4, SUC_lexicon_tags_14 },
    { 0x48debed8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9ebedc, SUC_lexicon_tags_12 },
    { 0x3e7ebedd, SUC_lexicon_tags_23 },
    { 0xe99ebede, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317ebee2, SUC_lexicon_tags_17 },
    { 0xd23ebee3, SUC_lexicon_tags_1 },
    { 0xe7bebee3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9ebee7, SUC_lexicon_tags_12 },
    { 0xcbbebee7, SUC_lexicon_tags_37 },
    { 0x1d9ebee9, SUC_lexicon_tags_12 },
    { 0xf5ebee8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5f5ebeec, SUC_lexicon_tags_15 },
    { 0x941ebeed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedebef1, SUC_lexicon_tags_14 },
    { 0x485ebef2, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xea5ebef4, SUC_lexicon_tags_1 },
    { 0x771ebef4, SUC_lexicon_tags_25 },
    { 0x9ddebef4, SUC_lexicon_tags_47 },
    { 0xccbebef7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7ebefa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb11ebefc, SUC_lexicon_tags_25 },
    { 0xae9ebefd, SUC_lexicon_tags_77 },
    { 0x6e7ebefe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf31ebf00, SUC_lexicon_tags_60 },
    { 0x367ebf01, SUC_lexicon_tags_49 },
    { 0xaa9ebf02, SUC_lexicon_tags_23 },
    { 0xe67ebf03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33ebf09, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x997ebf0c, SUC_lexicon_tags_8 },
    { 0x98febf0d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35ebf11, SUC_lexicon_tags_37 },
    { 0xe09ebf12, SUC_lexicon_tags_49 },
    { 0x9e9ebf13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x94febf15, SUC_lexicon_tags_31 },
    { 0xc41ebf16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x779ebf20, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137ebf28, SUC_lexicon_tags_23 },
    { 0x20febf28, SUC_lexicon_tags_30 },
    { 0x88febf2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66febf33, SUC_lexicon_tags_14 },
    { 0xc6debf34, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9ebf37, SUC_lexicon_tags_7 },
    { 0x133ebf38, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x193ebf3b, SUC_lexicon_tags_12 },
    { 0x455ebf3b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87ebf40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1ebf45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5ebf4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665ebf4f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bebf52, SUC_lexicon_tags_60 },
    { 0x801ebf53, SUC_lexicon_tags_30 },
    { 0xe1febf53, SUC_lexicon_tags_28 },
    { 0x243ebf55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6a3ebf57, SUC_lexicon_tags_55 },
    { 0x181ebf58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7d9ebf5a, SUC_lexicon_tags_0 },
    { 0x599ebf5a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x92bebf5d, SUC_lexicon_tags_25 },
    { 0xe51ebf5e, SUC_lexicon_tags_7 },
    { 0x723ebf5f, SUC_lexicon_tags_62 },
    { 0x3debf60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ebf64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12febf69, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x421ebf6c, SUC_lexicon_tags_85 },
    { 0x9a1ebf6d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5ebf70, SUC_lexicon_tags_60 },
    { 0x535ebf71, SUC_lexicon_tags_43 },
    { 0xa9bebf70, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159ebf7c, SUC_lexicon_tags_15 },
    { 0xea9ebf7c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ebf81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1ebf85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f1ebf87, SUC_lexicon_tags_32 },
    { 0x7dfebf88, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x813ebf8a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529ebf94, SUC_lexicon_tags_8 },
    { 0x4adebf95, SUC_lexicon_tags_25 },
    { 0x5a5ebf96, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4debf99, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21ebf9c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x13debf9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499ebfab, SUC_lexicon_tags_1 },
    { 0x3b3ebfab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x481ebfae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6febfb6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe01ebfb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x221ebfbb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7ebfbf, SUC_lexicon_tags_12 },
    { 0xa95ebfc0, SUC_lexicon_tags_2 },
    { 0xb5febfbf, SUC_lexicon_tags_27 },
    { 0xd3ebfc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63febfc9, SUC_lexicon_tags_11 },
    { 0xbddebfc9, SUC_lexicon_tags_7 },
    { 0xc8bebfcb, SUC_lexicon_tags_47 },
    { 0xfd1ebfc9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfebfcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b5ebfd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x297ebfd4, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56debfd9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3debfdc, SUC_lexicon_tags_47 },
    { 0x2cbebfdd, SUC_lexicon_tags_29 },
    { 0x7cfebfde, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1ebfe4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfebfe8, SUC_lexicon_tags_33 },
    { 0x681ebfe9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37ebfed, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfebff3, SUC_lexicon_tags_43 },
    { 0xdd1ebff4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7ebffa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef7ebffc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9ebfff, SUC_lexicon_tags_0 },
    { 0x6a1ec000, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bec003, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffec006, SUC_lexicon_tags_11 },
    { 0xcd7ec006, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13ec00a, SUC_lexicon_tags_25 },
    { 0xd95ec00b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263ec013, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xda7ec015, SUC_lexicon_tags_17 },
    { 0x8b7ec015, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3ec019, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41ec01d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487ec023, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9ec026, SUC_lexicon_tags_27 },
    { 0x5ffec026, SUC_lexicon_tags_5 },
    { 0xfb1ec028, SUC_lexicon_tags_4 },
    { 0xa8dec029, SUC_lexicon_tags_7 },
    { 0xbf9ec028, SUC_lexicon_tags_62 },
    { 0x2abec026, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x989ec02d, SUC_lexicon_tags_15 },
    { 0x583ec02e, SUC_lexicon_tags_14 },
    { 0xa59ec02e, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x875ec033, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6d1ec035, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb45ec037, SUC_lexicon_tags_15 },
    { 0x7c9ec038, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc79ec03a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x78bec03c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb47ec03f, SUC_lexicon_tags_25 },
    { 0xd15ec040, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09ec048, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x39ec04a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83dec04c, SUC_lexicon_tags_15 },
    { 0xe69ec04d, SUC_lexicon_tags_0 },
    { 0x6f3ec04c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bec052, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519ec05b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x9cdec05d, SUC_lexicon_tags_75 },
    { 0x63fec05d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2b5ec060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe7ec062, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9ec065, SUC_lexicon_tags_1 },
    { 0x665ec065, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577ec06a, SUC_lexicon_tags_7 },
    { 0x4b7ec06a, SUC_lexicon_tags_25 },
    { 0x4c5ec06c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63ec06f, SUC_lexicon_tags_37 },
    { 0x3c5ec070, SUC_lexicon_tags_21 },
    { 0x37dec070, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0xbcbec073, SUC_lexicon_tags_45 },
    { 0xbcbec073, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27ec079, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5ec07f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x42fec082, SUC_lexicon_tags_15 },
    { 0x5fdec083, SUC_lexicon_tags_1 },
    { 0x92fec084, SUC_lexicon_tags_37 },
    { 0x36fec085, SUC_lexicon_tags_34 },
    { 0x473ec086, SUC_lexicon_tags_50 },
    { 0xaf5ec087, SUC_lexicon_tags_44 },
    { 0xad9ec088, SUC_lexicon_tags_33 },
    { 0x121ec084, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44dec091, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa3bec093, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdddec095, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xfddec097, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb1ec099, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93ec0a1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1ec0a7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4a9ec0a9, SUC_lexicon_tags_14 },
    { 0x2e3ec0aa, SUC_lexicon_tags_34 },
    { 0x559ec0ab, SUC_lexicon_tags_33 },
    { 0xd61ec0ab, SUC_lexicon_tags_18 },
    { 0x5b7ec0ad, SUC_lexicon_tags_11 },
    { 0xf25ec0ab, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbec0b2, SUC_lexicon_tags_7 },
    { 0xea5ec0b3, SUC_lexicon_tags_7 },
    { 0x6fbec0b2, SUC_lexicon_tags_57 },
    { 0xd07ec0b5, SUC_lexicon_tags_17 },
    { 0x4b3ec0b6, SUC_lexicon_tags_33 },
    { 0xd61ec0b6, SUC_lexicon_tags_23 },
    { 0xb09ec0b8, SUC_lexicon_tags_7 },
    { 0x2afec0b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453ec0bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1ec0c3, SUC_lexicon_tags_129 },
    { 0x323ec0c4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x11dec0c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x827ec0c8, SUC_lexicon_tags_15 },
    { 0xd59ec0c9, SUC_lexicon_tags_17 },
    { 0xe97ec0ca, SUC_lexicon_tags_7 },
    { 0x6afec0cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ec0d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9dec0d5, SUC_lexicon_tags_151 },
    { 0xfdec0d5, SUC_lexicon_tags_45 },
    { 0xb4dec0d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0fec0d9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fec0df, SUC_lexicon_tags_17 },
    { 0x4bec0e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47ec0e4, SUC_lexicon_tags_25 },
    { 0xb43ec0e4, SUC_lexicon_tags_9 },
    { 0xd23ec0e5, SUC_lexicon_tags_11 },
    { 0xfb1ec0e7, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83ec0ea, SUC_lexicon_tags_7 },
    { 0x63ec0ea, SUC_lexicon_tags_43 },
    { 0x15fec0ec, SUC_lexicon_tags_46 },
    { 0x383ec0ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d3ec0ef, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb33ec0f1, SUC_lexicon_tags_2 },
    { 0xdf7ec0f1, SUC_lexicon_tags_12 },
    { 0x6ebec0f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5edec0f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcddec0f7, SUC_lexicon_tags_6 },
    { 0x9ddec0f8, SUC_lexicon_tags_7 },
    { 0xb47ec0f7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8efec0fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa09ec0fe, SUC_lexicon_tags_25 },
    { 0x37bec0fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc3ec104, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fec107, SUC_lexicon_tags_1 },
    { 0x301ec108, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x569ec10a, SUC_lexicon_tags_31 },
    { 0x67fec10b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x34dec10d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91ec111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49ec114, SUC_lexicon_tags_7 },
    { 0xb0dec115, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfec118, SUC_lexicon_tags_9 },
    { 0xc19ec119, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf19ec11b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4e1ec11d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951ec121, SUC_lexicon_tags_14 },
    { 0x175ec122, SUC_lexicon_tags_64 },
    { 0x103ec123, SUC_lexicon_tags_16 },
    { 0x1e9ec124, SUC_lexicon_tags_3 },
    { 0xaffec125, SUC_lexicon_tags_11 },
    { 0x9cdec126, SUC_lexicon_tags_34 },
    { 0x505ec124, SUC_lexicon_tags_15 },
    { 0xbe5ec128, SUC_lexicon_tags_28 },
    { 0x71ec129, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf81ec12b, SUC_lexicon_tags_25 },
    { 0xf77ec12c, SUC_lexicon_tags_12 },
    { 0xf45ec12c, SUC_lexicon_tags_38 },
    { 0xf5ec12e, SUC_lexicon_tags_1 },
    { 0xf81ec12f, SUC_lexicon_tags_25 },
    { 0x831ec130, SUC_lexicon_tags_2 },
    { 0x529ec131, SUC_lexicon_tags_38 },
    { 0xb4dec12e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3d7ec134, SUC_lexicon_tags_17 },
    { 0x765ec135, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5a7ec137, SUC_lexicon_tags_15 },
    { 0xa71ec138, SUC_lexicon_tags_15 },
    { 0x5fec139, SUC_lexicon_tags_2 },
    { 0x5c1ec13a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebec13d, SUC_lexicon_tags_7 },
    { 0x29ec13e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ec142, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa2dec144, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x711ec146, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x371ec148, SUC_lexicon_tags_254 },
    { 0xe37ec149, SUC_lexicon_tags_15 },
    { 0x5d1ec149, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe0dec14c, SUC_lexicon_tags_2 },
    { 0x46dec14c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7ec150, SUC_lexicon_tags_12 },
    { 0xcd1ec150, SUC_lexicon_tags_12 },
    { 0x6abec152, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1ec155, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87ec159, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183ec15e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dec161, SUC_lexicon_tags_17 },
    { 0x21fec161, SUC_lexicon_tags_11 },
    { 0xe5fec163, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fec167, SUC_lexicon_tags_64 },
    { 0x52dec168, SUC_lexicon_tags_82 },
    { 0x775ec169, SUC_lexicon_tags_25 },
    { 0x1edec16a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0xe05ec16c, SUC_lexicon_tags_10 },
    { 0xb17ec16d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fec170, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbec173, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81ec178, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdec17b, SUC_lexicon_tags_1 },
    { 0xb4dec17c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfec17f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x53bec181, SUC_lexicon_tags_37 },
    { 0x949ec181, SUC_lexicon_tags_27 },
    { 0x541ec183, SUC_lexicon_tags_160 },
    { 0xbfdec184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x26fec187, SUC_lexicon_tags_19 },
    { 0x87dec187, SUC_lexicon_tags_30 },
    { 0x671ec189, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dec18e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x219ec190, SUC_lexicon_tags_63 },
    { 0x771ec191, SUC_lexicon_tags_7 },
    { 0x2a1ec192, SUC_lexicon_tags_27 },
    { 0xd77ec192, SUC_lexicon_tags_2 },
    { 0xbadec191, SUC_lexicon_tags_12 },
    { 0x13fec191, SUC_lexicon_tags_49 },
    { 0x64fec195, SUC_lexicon_tags_79 },
    { 0xebbec197, SUC_lexicon_tags_39 },
    { 0xe39ec198, SUC_lexicon_tags_17 },
    { 0x8c1ec194, SUC_lexicon_tags_14 },
    { 0x8cfec19a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf5dec19c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dec1a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfec1a6, SUC_lexicon_tags_47 },
    { 0x49fec1a6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x3e5ec1a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x48bec1ab, SUC_lexicon_tags_8 },
    { 0xd8dec1ac, SUC_lexicon_tags_15 },
    { 0x1bdec1ad, SUC_lexicon_tags_11 },
    { 0x4adec1ad, SUC_lexicon_tags_7 },
    { 0x7b1ec1ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7ec1b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379ec1ba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x777ec1bc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cfec1c9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9ec1cd, SUC_lexicon_tags_19 },
    { 0x5e7ec1cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb61ec1d0, SUC_lexicon_tags_2 },
    { 0xc65ec1d1, SUC_lexicon_tags_32 },
    { 0x7a3ec1d2, SUC_lexicon_tags_165 },
    { 0xe05ec1d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x481ec1d6, SUC_lexicon_tags_21 },
    { 0x33bec1d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb81ec1d9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7ec1dc, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bec1df, SUC_lexicon_tags_63 },
    { 0x979ec1e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35ec1e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x389ec1e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x807ec1e6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dec1ea, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x52bec1ec, SUC_lexicon_tags_17 },
    { 0x657ec1ed, SUC_lexicon_tags_16 },
    { 0x41dec1ee, SUC_lexicon_tags_33 },
    { 0xbe1ec1ec, SUC_lexicon_tags_10 },
    { 0x9ec1f0, SUC_lexicon_tags_17 },
    { 0x261ec1ec, SUC_lexicon_tags_47 },
    { 0x9f5ec1f2, SUC_lexicon_tags_12 },
    { 0xd8fec1f2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201ec1f9, SUC_lexicon_tags_29 },
    { 0x4fdec1fa, SUC_lexicon_tags_113 },
    { 0x349ec1fb, SUC_lexicon_tags_11 },
    { 0x55ec1fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfec1ff, SUC_lexicon_tags_19 },
    { 0x4b5ec200, SUC_lexicon_tags_17 },
    { 0xc6bec200, SUC_lexicon_tags_37 },
    { 0x3ffec202, SUC_lexicon_tags_11 },
    { 0x303ec203, SUC_lexicon_tags_12 },
    { 0xf43ec1ff, SUC_lexicon_tags_46 },
    { 0x6e1ec202, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105ec20b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x1c7ec20d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77ec210, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c9ec212, SUC_lexicon_tags_68 },
    { 0x357ec212, SUC_lexicon_tags_16 },
    { 0x2c7ec212, SUC_lexicon_tags_27 },
    { 0x591ec213, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45ec21a, SUC_lexicon_tags_17 },
    { 0xafbec21a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fec220, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbec224, SUC_lexicon_tags_38 },
    { 0xf9bec225, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe89ec227, SUC_lexicon_tags_87 },
    { 0x80bec228, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x939ec22a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x851ec22d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x49bec22f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367ec233, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdec23a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x171ec23d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x639ec240, SUC_lexicon_tags_1 },
    { 0x2d5ec240, SUC_lexicon_tags_35 },
    { 0x3edec242, SUC_lexicon_tags_9 },
    { 0xdc1ec242, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x3fbec245, SUC_lexicon_tags_10 },
    { 0x21bec245, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13ec249, SUC_lexicon_tags_25 },
    { 0x21bec24a, SUC_lexicon_tags_33 },
    { 0x617ec24b, SUC_lexicon_tags_1 },
    { 0x90fec24b, SUC_lexicon_tags_11 },
    { 0xcadec24c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xed7ec24f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc2dec251, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fec259, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45ec25c, SUC_lexicon_tags_19 },
    { 0xd47ec25d, SUC_lexicon_tags_2 },
    { 0xda5ec25e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3ec26c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18fec272, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcbbec274, SUC_lexicon_tags_14 },
    { 0x1b3ec274, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4efec27f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1a3ec281, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7ec287, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdec28d, SUC_lexicon_tags_28 },
    { 0x7d9ec28e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdbdec290, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbec293, SUC_lexicon_tags_8 },
    { 0x767ec294, SUC_lexicon_tags_2 },
    { 0xb6bec294, SUC_lexicon_tags_1 },
    { 0x77dec296, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6cdec298, SUC_lexicon_tags_15 },
    { 0x4b9ec299, SUC_lexicon_tags_7 },
    { 0xd51ec299, SUC_lexicon_tags_19 },
    { 0xbc1ec299, SUC_lexicon_tags_12 },
    { 0x48dec299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431ec2a0, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x28bec2a2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227ec2aa, SUC_lexicon_tags_8 },
    { 0xa65ec2ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9bec2ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75ec2b1, SUC_lexicon_tags_15 },
    { 0xf89ec2b2, SUC_lexicon_tags_124 },
    { 0xd91ec2b3, SUC_lexicon_tags_55 },
    { 0x255ec2b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf67ec2b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7ffec2b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddbec2ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525ec2c0, SUC_lexicon_tags_23 },
    { 0x241ec2c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3ffec2c3, SUC_lexicon_tags_15 },
    { 0xf31ec2c3, SUC_lexicon_tags_30 },
    { 0x83ec2c5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dec2ca, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3ec2ce, SUC_lexicon_tags_107 },
    { 0x15dec2cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67fec2d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe95ec2d6, SUC_lexicon_tags_27 },
    { 0xa23ec2d7, SUC_lexicon_tags_30 },
    { 0x53ec2d6, SUC_lexicon_tags_33 },
    { 0x86dec2d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3ec2dd, SUC_lexicon_tags_13 },
    { 0x921ec2de, SUC_lexicon_tags_25 },
    { 0x50dec2df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15fec2e4, SUC_lexicon_tags_12 },
    { 0x5b5ec2e5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffec2ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07ec2ef, SUC_lexicon_tags_15 },
    { 0x673ec2f0, SUC_lexicon_tags_1 },
    { 0x7a1ec2f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7ec2f6, SUC_lexicon_tags_27 },
    { 0xcbfec2f7, SUC_lexicon_tags_30 },
    { 0x8b7ec2f7, SUC_lexicon_tags_23 },
    { 0x393ec2f7, SUC_lexicon_tags_16 },
    { 0xdb5ec2fa, SUC_lexicon_tags_33 },
    { 0x129ec2fb, SUC_lexicon_tags_5 },
    { 0x7b1ec2fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7ec300, SUC_lexicon_tags_15 },
    { 0x785ec300, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fec307, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edec30d, SUC_lexicon_tags_37 },
    { 0xe7fec30e, SUC_lexicon_tags_37 },
    { 0x337ec30f, SUC_lexicon_tags_63 },
    { 0x469ec310, SUC_lexicon_tags_15 },
    { 0xd5dec30f, SUC_lexicon_tags_2 },
    { 0xa43ec312, SUC_lexicon_tags_15 },
    { 0xbdec313, SUC_lexicon_tags_37 },
    { 0x1fec313, SUC_lexicon_tags_37 },
    { 0x101ec310, SUC_lexicon_tags_2 },
    { 0x51bec30f, SUC_lexicon_tags_43 },
    { 0xa73ec317, SUC_lexicon_tags_80 },
    { 0x74bec318, SUC_lexicon_tags_37 },
    { 0xb3ec318, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3ec320, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5ec323, SUC_lexicon_tags_2 },
    { 0x4f5ec324, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc73ec326, SUC_lexicon_tags_15 },
    { 0x9b9ec327, SUC_lexicon_tags_35 },
    { 0x303ec328, SUC_lexicon_tags_7 },
    { 0xa59ec329, SUC_lexicon_tags_7 },
    { 0x7fbec32a, SUC_lexicon_tags_39 },
    { 0x725ec32b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x941ec32e, SUC_lexicon_tags_47 },
    { 0x145ec32f, SUC_lexicon_tags_37 },
    { 0xa87ec330, SUC_lexicon_tags_47 },
    { 0xcb9ec330, SUC_lexicon_tags_64 },
    { 0x599ec332, SUC_lexicon_tags_23 },
    { 0xf05ec330, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x497ec335, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc03ec337, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x33ec339, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf41ec33b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd39ec33d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fec342, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1ec345, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9ec349, SUC_lexicon_tags_23 },
    { 0x61bec349, SUC_lexicon_tags_1 },
    { 0x993ec349, SUC_lexicon_tags_19 },
    { 0x243ec34a, SUC_lexicon_tags_7 },
    { 0x6a7ec34d, SUC_lexicon_tags_1 },
    { 0x9a7ec34e, SUC_lexicon_tags_56 },
    { 0x257ec34e, SUC_lexicon_tags_17 },
    { 0xa5bec34f, SUC_lexicon_tags_19 },
    { 0xf59ec34f, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0xf2bec353, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9ec35a, SUC_lexicon_tags_49 },
    { 0x31bec35a, SUC_lexicon_tags_33 },
    { 0x9bfec35c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xdf9ec35e, SUC_lexicon_tags_12 },
    { 0xeefec35f, SUC_lexicon_tags_50 },
    { 0x577ec35e, SUC_lexicon_tags_7 },
    { 0x7ec35f, SUC_lexicon_tags_63 },
    { 0xafec362, SUC_lexicon_tags_2 },
    { 0x4ebec362, SUC_lexicon_tags_56 },
    { 0x319ec363, SUC_lexicon_tags_14 },
    { 0x885ec362, SUC_lexicon_tags_25 },
    { 0x98dec366, SUC_lexicon_tags_28 },
    { 0x489ec367, SUC_lexicon_tags_47 },
    { 0xd7bec362, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fec36b, SUC_lexicon_tags_17 },
    { 0xdfbec36c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffec36f, SUC_lexicon_tags_12 },
    { 0x96dec370, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8b3ec372, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x913ec374, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x5d9ec376, SUC_lexicon_tags_56 },
    { 0x69ec377, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dec37b, SUC_lexicon_tags_2 },
    { 0x3f1ec37b, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbec381, SUC_lexicon_tags_45 },
    { 0x34dec382, SUC_lexicon_tags_15 },
    { 0xc67ec382, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x525ec386, SUC_lexicon_tags_17 },
    { 0x28dec387, SUC_lexicon_tags_28 },
    { 0xdb5ec387, SUC_lexicon_tags_0 },
    { 0xc6fec389, SUC_lexicon_tags_49 },
    { 0x58dec38a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741ec38e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x99fec391, SUC_lexicon_tags_11 },
    { 0x80dec391, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1ec39c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7cdec39e, SUC_lexicon_tags_87 },
    { 0x9e5ec39e, SUC_lexicon_tags_1 },
    { 0xb7fec3a0, SUC_lexicon_tags_17 },
    { 0xde1ec3a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xec3ec3a3, SUC_lexicon_tags_16 },
    { 0x95dec3a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ec3a9, SUC_lexicon_tags_27 },
    { 0x6b9ec3aa, SUC_lexicon_tags_38 },
    { 0x647ec3ab, SUC_lexicon_tags_45 },
    { 0x269ec3ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde5ec3ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94dec3b2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dec3b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5ec3bd, SUC_lexicon_tags_16 },
    { 0x3a5ec3bd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb15ec3c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27ec3c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ec3c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48fec3d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x337ec3d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbec3d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453ec3e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5ec3e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1ec3e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645ec3ec, SUC_lexicon_tags_87 },
    { 0x615ec3ed, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87ec3f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409ec3f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bec3fa, SUC_lexicon_tags_15 },
    { 0x4d1ec3fb, SUC_lexicon_tags_17 },
    { 0xcc3ec3fb, SUC_lexicon_tags_19 },
    { 0x6a9ec3fb, SUC_lexicon_tags_2 },
    { 0xe17ec3fe, SUC_lexicon_tags_37 },
    { 0x9b7ec3fe, SUC_lexicon_tags_66 },
    { 0x5d3ec3fe, SUC_lexicon_tags_401 },
    { 0, NULL },
    { 0x469ec402, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x123ec405, SUC_lexicon_tags_2 },
    { 0x95ec406, SUC_lexicon_tags_31 },
    { 0x2f3ec407, SUC_lexicon_tags_28 },
    { 0xd27ec408, SUC_lexicon_tags_28 },
    { 0x363ec409, SUC_lexicon_tags_77 },
    { 0xdb5ec405, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41ec40d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x817ec40f, SUC_lexicon_tags_43 },
    { 0xab5ec40f, SUC_lexicon_tags_8 },
    { 0x383ec40f, SUC_lexicon_tags_56 },
    { 0x875ec412, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3ec414, SUC_lexicon_tags_7 },
    { 0x8e3ec414, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793ec419, SUC_lexicon_tags_12 },
    { 0x101ec41a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731ec422, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5ec425, SUC_lexicon_tags_29 },
    { 0x78fec426, SUC_lexicon_tags_11 },
    { 0x829ec425, SUC_lexicon_tags_60 },
    { 0x22dec428, SUC_lexicon_tags_15 },
    { 0x44fec428, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669ec42d, SUC_lexicon_tags_63 },
    { 0x1f1ec42e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31ec431, SUC_lexicon_tags_14 },
    { 0xb4fec432, SUC_lexicon_tags_18 },
    { 0xb17ec433, SUC_lexicon_tags_39 },
    { 0xb21ec431, SUC_lexicon_tags_1 },
    { 0xcd3ec435, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebec438, SUC_lexicon_tags_27 },
    { 0xacdec439, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89ec441, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x983ec443, SUC_lexicon_tags_33 },
    { 0xc6bec444, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17ec449, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdec44e, SUC_lexicon_tags_37 },
    { 0xb5dec44f, SUC_lexicon_tags_63 },
    { 0x855ec44e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753ec454, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x459ec456, SUC_lexicon_tags_25 },
    { 0xb7bec457, SUC_lexicon_tags_19 },
    { 0xf0dec457, SUC_lexicon_tags_25 },
    { 0xd8fec459, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfafec45b, SUC_lexicon_tags_7 },
    { 0x371ec45c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3cbec45e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2bec460, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x525ec462, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93ec465, SUC_lexicon_tags_46 },
    { 0xaa7ec466, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189ec46a, SUC_lexicon_tags_11 },
    { 0x299ec46b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9ec471, SUC_lexicon_tags_77 },
    { 0x9edec472, SUC_lexicon_tags_17 },
    { 0xd23ec473, SUC_lexicon_tags_17 },
    { 0x6d9ec472, SUC_lexicon_tags_2 },
    { 0x939ec473, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fec47b, SUC_lexicon_tags_9 },
    { 0xd1fec47c, SUC_lexicon_tags_17 },
    { 0xb3fec47b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575ec485, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa47ec487, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279ec48b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddec48e, SUC_lexicon_tags_15 },
    { 0x4dbec48f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5ec495, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73ec498, SUC_lexicon_tags_1 },
    { 0xd31ec499, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb1ec49b, SUC_lexicon_tags_1 },
    { 0xfbec49c, SUC_lexicon_tags_15 },
    { 0x805ec49d, SUC_lexicon_tags_15 },
    { 0x3b3ec49e, SUC_lexicon_tags_37 },
    { 0xa7dec49e, SUC_lexicon_tags_33 },
    { 0x953ec49d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb31ec4a2, SUC_lexicon_tags_7 },
    { 0xa13ec4a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8cbec4a5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dec4a9, SUC_lexicon_tags_2 },
    { 0x2abec4a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x561ec4ad, SUC_lexicon_tags_65 },
    { 0xb77ec4ae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x323ec4b0, SUC_lexicon_tags_17 },
    { 0x8bfec4b0, SUC_lexicon_tags_7 },
    { 0xcf9ec4b2, SUC_lexicon_tags_27 },
    { 0xebfec4b3, SUC_lexicon_tags_12 },
    { 0xd53ec4b4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fec4b9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ec4bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25fec4bf, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd0bec4c1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x163ec4c4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xbc3ec4c6, SUC_lexicon_tags_15 },
    { 0x66dec4c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31ec4cd, SUC_lexicon_tags_7 },
    { 0x9ebec4cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xecfec4d1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf29ec4d4, SUC_lexicon_tags_9 },
    { 0x9e1ec4d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdec4da, SUC_lexicon_tags_82 },
    { 0xd4bec4da, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfec4de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x707ec4e1, SUC_lexicon_tags_60 },
    { 0x643ec4e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bec4e7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bec4ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199ec4f1, SUC_lexicon_tags_16 },
    { 0xad1ec4f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x353ec4f5, SUC_lexicon_tags_1 },
    { 0x485ec4f6, SUC_lexicon_tags_0 },
    { 0xb99ec4f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377ec502, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fec504, SUC_lexicon_tags_15 },
    { 0x705ec505, SUC_lexicon_tags_11 },
    { 0x611ec505, SUC_lexicon_tags_21 },
    { 0xfb1ec505, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd3ec509, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25dec512, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89ec516, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xba5ec518, SUC_lexicon_tags_37 },
    { 0x99dec518, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x483ec51b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x123ec51d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8dec51f, SUC_lexicon_tags_10 },
    { 0xe61ec51f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x28bec522, SUC_lexicon_tags_14 },
    { 0xc91ec523, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683ec527, SUC_lexicon_tags_19 },
    { 0xadfec528, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5fec52a, SUC_lexicon_tags_30 },
    { 0x419ec52b, SUC_lexicon_tags_1 },
    { 0xf29ec52c, SUC_lexicon_tags_30 },
    { 0x7e5ec52d, SUC_lexicon_tags_12 },
    { 0x501ec52c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48dec532, SUC_lexicon_tags_19 },
    { 0x499ec532, SUC_lexicon_tags_160 },
    { 0x609ec532, SUC_lexicon_tags_8 },
    { 0x345ec535, SUC_lexicon_tags_23 },
    { 0x977ec535, SUC_lexicon_tags_0 },
    { 0x1efec535, SUC_lexicon_tags_12 },
    { 0x285ec534, SUC_lexicon_tags_1620 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7ec53d, SUC_lexicon_tags_15 },
    { 0xe1fec53d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49ec542, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bec546, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2e3ec548, SUC_lexicon_tags_65 },
    { 0x997ec549, SUC_lexicon_tags_32 },
    { 0xbe9ec54a, SUC_lexicon_tags_45 },
    { 0xa43ec54b, SUC_lexicon_tags_13 },
    { 0x627ec54c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x581ec54f, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fec553, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x57dec555, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb4dec557, SUC_lexicon_tags_55 },
    { 0x2c9ec558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5dec55b, SUC_lexicon_tags_68 },
    { 0x375ec55b, SUC_lexicon_tags_17 },
    { 0x781ec55d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dec560, SUC_lexicon_tags_33 },
    { 0x8abec560, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa33ec563, SUC_lexicon_tags_23 },
    { 0xc7bec564, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275ec569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37ec570, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba1ec572, SUC_lexicon_tags_12 },
    { 0x57fec573, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x133ec575, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93ec578, SUC_lexicon_tags_15 },
    { 0x283ec579, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35ec587, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb6dec589, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4f5ec58b, SUC_lexicon_tags_11 },
    { 0x1ffec58c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffec590, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x6a9ec592, SUC_lexicon_tags_15 },
    { 0xd7ec593, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x75bec595, SUC_lexicon_tags_19 },
    { 0xcc7ec596, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9ec599, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x2a3ec59b, SUC_lexicon_tags_1 },
    { 0x81ec59b, SUC_lexicon_tags_23 },
    { 0x4ddec59b, SUC_lexicon_tags_27 },
    { 0x79dec59d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x119ec5a0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9ec5a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf35ec5a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dec5ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abec5b1, SUC_lexicon_tags_55 },
    { 0x6c7ec5b2, SUC_lexicon_tags_39 },
    { 0x3f5ec5b1, SUC_lexicon_tags_8 },
    { 0x795ec5b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3ec5b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35ec5b9, SUC_lexicon_tags_17 },
    { 0xa01ec5ba, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x581ec5bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5ec5c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffec5ca, SUC_lexicon_tags_7 },
    { 0xb55ec5ca, SUC_lexicon_tags_1 },
    { 0xd6bec5cc, SUC_lexicon_tags_38 },
    { 0x853ec5cd, SUC_lexicon_tags_29 },
    { 0x411ec5ce, SUC_lexicon_tags_2 },
    { 0x9bdec5cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fec5d5, SUC_lexicon_tags_15 },
    { 0x60dec5d6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x677ec5d8, SUC_lexicon_tags_38 },
    { 0xc5ec5d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51ec5de, SUC_lexicon_tags_43 },
    { 0x757ec5df, SUC_lexicon_tags_15 },
    { 0xef7ec5df, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635ec5ec, SUC_lexicon_tags_25 },
    { 0xebfec5ed, SUC_lexicon_tags_8 },
    { 0xfffec5ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bec5f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fec5fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b9ec5fc, SUC_lexicon_tags_23 },
    { 0x8c5ec5fc, SUC_lexicon_tags_38 },
    { 0x3e5ec5fe, SUC_lexicon_tags_37 },
    { 0x815ec5fe, SUC_lexicon_tags_34 },
    { 0x233ec600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fec603, SUC_lexicon_tags_7 },
    { 0xd1bec603, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673ec60a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbec60d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfec611, SUC_lexicon_tags_25 },
    { 0xc3dec612, SUC_lexicon_tags_37 },
    { 0x575ec613, SUC_lexicon_tags_39 },
    { 0x29bec614, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695ec619, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623ec61e, SUC_lexicon_tags_0 },
    { 0xc5dec61f, SUC_lexicon_tags_49 },
    { 0x1fdec61f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa3ec622, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7ec625, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ec629, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bec62f, SUC_lexicon_tags_11 },
    { 0x7e7ec62f, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xfabec633, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dec636, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fec63c, SUC_lexicon_tags_2 },
    { 0x97dec63d, SUC_lexicon_tags_11 },
    { 0x437ec63e, SUC_lexicon_tags_1 },
    { 0x971ec63f, SUC_lexicon_tags_69 },
    { 0x6ddec640, SUC_lexicon_tags_78 },
    { 0xfbfec63f, SUC_lexicon_tags_14 },
    { 0x31ec642, SUC_lexicon_tags_11 },
    { 0x69ec643, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907ec649, SUC_lexicon_tags_32 },
    { 0x55bec649, SUC_lexicon_tags_46 },
    { 0x8afec64b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3ec651, SUC_lexicon_tags_8 },
    { 0x23ec651, SUC_lexicon_tags_19 },
    { 0x131ec653, SUC_lexicon_tags_37 },
    { 0xe5ec651, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dec65d, SUC_lexicon_tags_27 },
    { 0x151ec65e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbec661, SUC_lexicon_tags_68 },
    { 0xb0dec662, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbec667, SUC_lexicon_tags_38 },
    { 0xfdbec667, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x71fec66b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe5fec66d, SUC_lexicon_tags_6 },
    { 0x8a7ec66d, SUC_lexicon_tags_44 },
    { 0xfebec66e, SUC_lexicon_tags_12 },
    { 0xd31ec670, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6ddec672, SUC_lexicon_tags_16 },
    { 0x693ec673, SUC_lexicon_tags_32 },
    { 0xaffec674, SUC_lexicon_tags_11 },
    { 0xcf9ec675, SUC_lexicon_tags_37 },
    { 0x14dec676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd1fec678, SUC_lexicon_tags_25 },
    { 0x33bec679, SUC_lexicon_tags_47 },
    { 0xabec67a, SUC_lexicon_tags_11 },
    { 0x117ec67b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb23ec67d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5ec683, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9ec68b, SUC_lexicon_tags_15 },
    { 0xee3ec68c, SUC_lexicon_tags_28 },
    { 0x349ec68d, SUC_lexicon_tags_15 },
    { 0x129ec68d, SUC_lexicon_tags_17 },
    { 0x32fec68f, SUC_lexicon_tags_12 },
    { 0xca1ec690, SUC_lexicon_tags_7 },
    { 0xba1ec691, SUC_lexicon_tags_37 },
    { 0x20bec68e, SUC_lexicon_tags_37 },
    { 0xfe7ec68d, SUC_lexicon_tags_1 },
    { 0x711ec692, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5e5ec696, SUC_lexicon_tags_25 },
    { 0x69ec696, SUC_lexicon_tags_38 },
    { 0x35ec698, SUC_lexicon_tags_17 },
    { 0xa9dec699, SUC_lexicon_tags_63 },
    { 0xa5dec696, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x515ec69c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5ec6a0, SUC_lexicon_tags_11 },
    { 0x475ec6a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39ec6a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2bdec6aa, SUC_lexicon_tags_45 },
    { 0xa4fec6ab, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141ec6b3, SUC_lexicon_tags_31 },
    { 0x453ec6b3, SUC_lexicon_tags_10 },
    { 0x125ec6b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ec6bc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb3ec6be, SUC_lexicon_tags_17 },
    { 0xbcdec6be, SUC_lexicon_tags_16 },
    { 0xd71ec6c0, SUC_lexicon_tags_29 },
    { 0xbd7ec6c1, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117ec6c5, SUC_lexicon_tags_31 },
    { 0x74bec6c5, SUC_lexicon_tags_43 },
    { 0x2e7ec6c6, SUC_lexicon_tags_12 },
    { 0xa7fec6c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x169ec6cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4b9ec6cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577ec6d5, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ec6d8, SUC_lexicon_tags_7 },
    { 0xfe3ec6d9, SUC_lexicon_tags_37 },
    { 0x77bec6d8, SUC_lexicon_tags_19 },
    { 0xe05ec6d9, SUC_lexicon_tags_63 },
    { 0x3c5ec6d9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x641ec6de, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dec6e4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fec6e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30fec6ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451ec6f1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x25bec6f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fec6fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7ec6fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5ec703, SUC_lexicon_tags_23 },
    { 0x655ec704, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x447ec707, SUC_lexicon_tags_17 },
    { 0xb35ec708, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9ec70c, SUC_lexicon_tags_36 },
    { 0x493ec70d, SUC_lexicon_tags_7 },
    { 0x237ec70c, SUC_lexicon_tags_8 },
    { 0xa67ec70f, SUC_lexicon_tags_10 },
    { 0x23bec70f, SUC_lexicon_tags_38 },
    { 0x6a9ec710, SUC_lexicon_tags_7 },
    { 0x819ec710, SUC_lexicon_tags_74 },
    { 0x157ec70d, SUC_lexicon_tags_12 },
    { 0x387ec70d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fec717, SUC_lexicon_tags_11 },
    { 0x883ec718, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfec721, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xbcfec723, SUC_lexicon_tags_14 },
    { 0xefec724, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x711ec726, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcebec728, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3fec72b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15dec72d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62dec731, SUC_lexicon_tags_45 },
    { 0x561ec731, SUC_lexicon_tags_19 },
    { 0x7f3ec732, SUC_lexicon_tags_14 },
    { 0x5d5ec734, SUC_lexicon_tags_37 },
    { 0x24fec735, SUC_lexicon_tags_77 },
    { 0x57bec732, SUC_lexicon_tags_14 },
    { 0xffbec734, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60fec739, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe29ec73b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ec740, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x393ec742, SUC_lexicon_tags_3 },
    { 0x3f3ec743, SUC_lexicon_tags_199 },
    { 0x573ec743, SUC_lexicon_tags_19 },
    { 0x62fec745, SUC_lexicon_tags_23 },
    { 0xfebec745, SUC_lexicon_tags_47 },
    { 0x619ec746, SUC_lexicon_tags_9 },
    { 0x283ec748, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341ec74c, SUC_lexicon_tags_1 },
    { 0xb0fec74c, SUC_lexicon_tags_37 },
    { 0x263ec74c, SUC_lexicon_tags_15 },
    { 0xe3ec74f, SUC_lexicon_tags_11 },
    { 0x327ec74f, SUC_lexicon_tags_25 },
    { 0x44bec751, SUC_lexicon_tags_12 },
    { 0x8dec750, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ec75f, SUC_lexicon_tags_37 },
    { 0x61ec760, SUC_lexicon_tags_107 },
    { 0x33ec75f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503ec768, SUC_lexicon_tags_17 },
    { 0x1a5ec769, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705ec76e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207ec772, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x379ec775, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa2dec777, SUC_lexicon_tags_11 },
    { 0xc7bec777, SUC_lexicon_tags_19 },
    { 0x3f9ec777, SUC_lexicon_tags_91 },
    { 0x13bec77a, SUC_lexicon_tags_11 },
    { 0x3f1ec77a, SUC_lexicon_tags_17 },
    { 0x84fec77b, SUC_lexicon_tags_26 },
    { 0xd0bec77d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa31ec77f, SUC_lexicon_tags_45 },
    { 0x751ec77f, SUC_lexicon_tags_9 },
    { 0xf95ec780, SUC_lexicon_tags_11 },
    { 0xd0fec781, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7ec787, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4dec789, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x451ec78b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x175ec78d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x63bec78f, SUC_lexicon_tags_38 },
    { 0xcb5ec790, SUC_lexicon_tags_47 },
    { 0xf29ec791, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61ec794, SUC_lexicon_tags_21 },
    { 0x68fec795, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1ec798, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21ec79b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x461ec79d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47ec7a4, SUC_lexicon_tags_65 },
    { 0xdebec7a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61bec7a7, SUC_lexicon_tags_14 },
    { 0xebdec7a8, SUC_lexicon_tags_26 },
    { 0xeedec7a9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35ec7ac, SUC_lexicon_tags_7 },
    { 0x5c1ec7ac, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfec7b2, SUC_lexicon_tags_17 },
    { 0x34dec7b3, SUC_lexicon_tags_37 },
    { 0xef3ec7b4, SUC_lexicon_tags_11 },
    { 0xc8bec7b4, SUC_lexicon_tags_37 },
    { 0x60dec7b6, SUC_lexicon_tags_25 },
    { 0x939ec7b3, SUC_lexicon_tags_37 },
    { 0x83bec7b8, SUC_lexicon_tags_12 },
    { 0xd15ec7b9, SUC_lexicon_tags_14 },
    { 0xab5ec7b9, SUC_lexicon_tags_47 },
    { 0x70dec7bb, SUC_lexicon_tags_60 },
    { 0xf7ec7ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb85ec7bf, SUC_lexicon_tags_12 },
    { 0xd89ec7c0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699ec7c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x239ec7ca, SUC_lexicon_tags_2 },
    { 0x6a1ec7cb, SUC_lexicon_tags_38 },
    { 0x8bdec7cb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05ec7d0, SUC_lexicon_tags_14 },
    { 0x7d9ec7d1, SUC_lexicon_tags_46 },
    { 0xd0fec7d2, SUC_lexicon_tags_10 },
    { 0x703ec7d3, SUC_lexicon_tags_0 },
    { 0xff5ec7d4, SUC_lexicon_tags_12 },
    { 0xeb1ec7d2, SUC_lexicon_tags_12 },
    { 0x121ec7d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9ec7d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bec7dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923ec7e7, SUC_lexicon_tags_1 },
    { 0xbf7ec7e7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afec7f0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3a9ec7f2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xab3ec7f4, SUC_lexicon_tags_8 },
    { 0x63dec7f5, SUC_lexicon_tags_12 },
    { 0x83fec7f6, SUC_lexicon_tags_46 },
    { 0x995ec7f4, SUC_lexicon_tags_37 },
    { 0x84bec7f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x499ec7fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767ec806, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xd65ec808, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229ec812, SUC_lexicon_tags_60 },
    { 0x93dec813, SUC_lexicon_tags_47 },
    { 0xde3ec812, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dec818, SUC_lexicon_tags_6 },
    { 0xc4dec818, SUC_lexicon_tags_28 },
    { 0xb97ec81a, SUC_lexicon_tags_1 },
    { 0x491ec81b, SUC_lexicon_tags_40 },
    { 0xff7ec81b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dec81f, SUC_lexicon_tags_2 },
    { 0x491ec820, SUC_lexicon_tags_2 },
    { 0x6a5ec820, SUC_lexicon_tags_37 },
    { 0x51fec822, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x97ec824, SUC_lexicon_tags_55 },
    { 0x349ec825, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35ec828, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcedec82a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x615ec82c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1ec82f, SUC_lexicon_tags_37 },
    { 0x20fec82f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x39bec832, SUC_lexicon_tags_29 },
    { 0x977ec833, SUC_lexicon_tags_1 },
    { 0xf49ec834, SUC_lexicon_tags_11 },
    { 0x75dec834, SUC_lexicon_tags_23 },
    { 0x983ec836, SUC_lexicon_tags_25 },
    { 0x8d9ec837, SUC_lexicon_tags_27 },
    { 0xa3fec835, SUC_lexicon_tags_17 },
    { 0xa93ec838, SUC_lexicon_tags_39 },
    { 0x2d5ec832, SUC_lexicon_tags_7 },
    { 0xb1bec83b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121ec840, SUC_lexicon_tags_17 },
    { 0x891ec840, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb5fec843, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1ec846, SUC_lexicon_tags_2 },
    { 0x86dec847, SUC_lexicon_tags_2 },
    { 0xea3ec848, SUC_lexicon_tags_27 },
    { 0x81fec848, SUC_lexicon_tags_2 },
    { 0x439ec84a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc53ec84c, SUC_lexicon_tags_37 },
    { 0xbec84d, SUC_lexicon_tags_7 },
    { 0xa8dec84d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x3bbec850, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5fec855, SUC_lexicon_tags_37 },
    { 0xbc3ec855, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x4b3ec858, SUC_lexicon_tags_116 },
    { 0x859ec858, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe1ec85b, SUC_lexicon_tags_17 },
    { 0x23dec85b, SUC_lexicon_tags_46 },
    { 0x9b1ec85d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x66dec860, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x545ec862, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcd1ec864, SUC_lexicon_tags_1 },
    { 0xfffec864, SUC_lexicon_tags_38 },
    { 0xc75ec864, SUC_lexicon_tags_1 },
    { 0x291ec866, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5ec86c, SUC_lexicon_tags_31 },
    { 0xe0dec86d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bec872, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9b1ec874, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x721ec876, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1ec879, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48fec87b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dec87e, SUC_lexicon_tags_35 },
    { 0xc37ec87f, SUC_lexicon_tags_77 },
    { 0x17dec87f, SUC_lexicon_tags_85 },
    { 0xe31ec880, SUC_lexicon_tags_60 },
    { 0x147ec87f, SUC_lexicon_tags_35 },
    { 0xc9bec883, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc1ec885, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfec889, SUC_lexicon_tags_66 },
    { 0x533ec889, SUC_lexicon_tags_29 },
    { 0x409ec889, SUC_lexicon_tags_7 },
    { 0x36dec88c, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1ec890, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b9ec892, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x197ec894, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dec898, SUC_lexicon_tags_7 },
    { 0x7d9ec898, SUC_lexicon_tags_27 },
    { 0xd85ec89a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x357ec89c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231ec8a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcc7ec8a2, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dec8a7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x37fec8a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91ec8b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41ec8b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437ec8bb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x637ec8bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55ec8c7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593ec8cc, SUC_lexicon_tags_47 },
    { 0xbcbec8cd, SUC_lexicon_tags_11 },
    { 0xc8fec8cd, SUC_lexicon_tags_7 },
    { 0x5f9ec8cf, SUC_lexicon_tags_48 },
    { 0x6bbec8d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721ec8d9, SUC_lexicon_tags_26 },
    { 0x7f3ec8da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9ec8e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbec8e7, SUC_lexicon_tags_23 },
    { 0x7a3ec8e7, SUC_lexicon_tags_22 },
    { 0x311ec8e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83ec8ec, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7f1ec8ee, SUC_lexicon_tags_49 },
    { 0x491ec8ef, SUC_lexicon_tags_63 },
    { 0x6fec8f0, SUC_lexicon_tags_37 },
    { 0x555ec8ef, SUC_lexicon_tags_23 },
    { 0xe1ec8ef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ec8f7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x777ec8f9, SUC_lexicon_tags_21 },
    { 0x83ec8fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4d7ec8fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15ec8ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5ec904, SUC_lexicon_tags_293 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3ec908, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fec90b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcd3ec90d, SUC_lexicon_tags_37 },
    { 0x2b7ec90e, SUC_lexicon_tags_23 },
    { 0xcdbec90e, SUC_lexicon_tags_11 },
    { 0xb21ec90e, SUC_lexicon_tags_17 },
    { 0xe51ec911, SUC_lexicon_tags_31 },
    { 0xf51ec911, SUC_lexicon_tags_33 },
    { 0xd0fec913, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x86dec915, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c7ec917, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ec91a, SUC_lexicon_tags_31 },
    { 0x93fec91a, SUC_lexicon_tags_1 },
    { 0xdddec91c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fec91f, SUC_lexicon_tags_0 },
    { 0x3b5ec920, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x795ec922, SUC_lexicon_tags_25 },
    { 0x147ec923, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaebec926, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebec929, SUC_lexicon_tags_278 },
    { 0x92bec929, SUC_lexicon_tags_1 },
    { 0x4ffec929, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadec932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bec936, SUC_lexicon_tags_12 },
    { 0x26dec937, SUC_lexicon_tags_25 },
    { 0x1f3ec938, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21ec93c, SUC_lexicon_tags_12 },
    { 0xbf9ec93d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797ec943, SUC_lexicon_tags_23 },
    { 0xf7bec944, SUC_lexicon_tags_2 },
    { 0x6bdec944, SUC_lexicon_tags_33 },
    { 0xeddec946, SUC_lexicon_tags_2 },
    { 0x7f7ec945, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3ec94e, SUC_lexicon_tags_109 },
    { 0x6d5ec94f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9bec951, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91dec953, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221ec957, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fec95d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x12dec95f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x419ec961, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4f5ec963, SUC_lexicon_tags_782 },
    { 0, NULL },
    { 0xd91ec965, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1ec96a, SUC_lexicon_tags_34 },
    { 0xa75ec96b, SUC_lexicon_tags_23 },
    { 0xd2fec96c, SUC_lexicon_tags_11 },
    { 0x9d3ec96d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33ec971, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dec975, SUC_lexicon_tags_38 },
    { 0xf7ec975, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdec97d, SUC_lexicon_tags_66 },
    { 0x4a9ec97d, SUC_lexicon_tags_33 },
    { 0x261ec97e, SUC_lexicon_tags_28 },
    { 0x179ec980, SUC_lexicon_tags_63 },
    { 0x3e9ec981, SUC_lexicon_tags_1 },
    { 0x281ec982, SUC_lexicon_tags_57 },
    { 0x12bec983, SUC_lexicon_tags_37 },
    { 0x9abec984, SUC_lexicon_tags_5 },
    { 0x1c9ec984, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbec98a, SUC_lexicon_tags_14 },
    { 0x7bfec98b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x31fec98e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1ec991, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5ec998, SUC_lexicon_tags_2 },
    { 0x63fec999, SUC_lexicon_tags_30 },
    { 0xc7fec99a, SUC_lexicon_tags_25 },
    { 0xcebec99a, SUC_lexicon_tags_23 },
    { 0x993ec99c, SUC_lexicon_tags_27 },
    { 0xc27ec99d, SUC_lexicon_tags_45 },
    { 0xf63ec99e, SUC_lexicon_tags_82 },
    { 0x89ec99c, SUC_lexicon_tags_2 },
    { 0x697ec9a0, SUC_lexicon_tags_15 },
    { 0x12dec99a, SUC_lexicon_tags_25 },
    { 0x65ec99d, SUC_lexicon_tags_57 },
    { 0xddbec9a3, SUC_lexicon_tags_7 },
    { 0x6d9ec99a, SUC_lexicon_tags_7 },
    { 0x277ec9a1, SUC_lexicon_tags_33 },
    { 0x68fec9a6, SUC_lexicon_tags_12 },
    { 0x821ec9a7, SUC_lexicon_tags_27 },
    { 0x4b7ec9a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5ec9ab, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfe7ec9ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbec9b0, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf63ec9b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f7ec9b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bec9b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf53ec9ba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ec9be, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x985ec9c0, SUC_lexicon_tags_2 },
    { 0x619ec9c1, SUC_lexicon_tags_12 },
    { 0x381ec9c2, SUC_lexicon_tags_45 },
    { 0x429ec9c3, SUC_lexicon_tags_37 },
    { 0xd9ec9c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9ec9c8, SUC_lexicon_tags_49 },
    { 0x5fec9c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323ec9cd, SUC_lexicon_tags_31 },
    { 0x73dec9ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddec9d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x117ec9d5, SUC_lexicon_tags_33 },
    { 0x5d7ec9d5, SUC_lexicon_tags_47 },
    { 0x1fbec9d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95ec9de, SUC_lexicon_tags_43 },
    { 0xd59ec9df, SUC_lexicon_tags_37 },
    { 0x713ec9e0, SUC_lexicon_tags_66 },
    { 0xe1ec9df, SUC_lexicon_tags_16 },
    { 0x153ec9e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8d3ec9e4, SUC_lexicon_tags_37 },
    { 0xc93ec9e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439ec9ea, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dec9ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xb75ec9f1, SUC_lexicon_tags_16 },
    { 0x2e9ec9f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b7ec9f4, SUC_lexicon_tags_2 },
    { 0xb57ec9f5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5ec9f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbec9ff, SUC_lexicon_tags_15 },
    { 0x61deca00, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x2a3eca02, SUC_lexicon_tags_16 },
    { 0xef9eca02, SUC_lexicon_tags_33 },
    { 0x621eca04, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbeca08, SUC_lexicon_tags_17 },
    { 0x1b7eca09, SUC_lexicon_tags_17 },
    { 0xa67eca0a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x695eca0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551eca14, SUC_lexicon_tags_65 },
    { 0x615eca14, SUC_lexicon_tags_37 },
    { 0x6d7eca16, SUC_lexicon_tags_13 },
    { 0x663eca17, SUC_lexicon_tags_37 },
    { 0x307eca16, SUC_lexicon_tags_49 },
    { 0xef7eca15, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x247eca1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95eca21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x781eca24, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8beca2e, SUC_lexicon_tags_17 },
    { 0xec9eca2f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x17eca31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1eca34, SUC_lexicon_tags_2 },
    { 0xec7eca35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc81eca39, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393eca3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687eca41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42beca43, SUC_lexicon_tags_17 },
    { 0xe47eca44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x469eca47, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffeca4b, SUC_lexicon_tags_0 },
    { 0x28feca4c, SUC_lexicon_tags_151 },
    { 0xa81eca4d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163eca51, SUC_lexicon_tags_11 },
    { 0xfb5eca52, SUC_lexicon_tags_10 },
    { 0xe51eca53, SUC_lexicon_tags_14 },
    { 0x481eca54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c5eca56, SUC_lexicon_tags_13 },
    { 0xf19eca57, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3eca5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb85eca5d, SUC_lexicon_tags_17 },
    { 0x8b1eca5e, SUC_lexicon_tags_53 },
    { 0x4edeca5e, SUC_lexicon_tags_0 },
    { 0xe9beca5f, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x543eca62, SUC_lexicon_tags_11 },
    { 0x1bfeca63, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x27deca65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f3eca67, SUC_lexicon_tags_46 },
    { 0xff5eca67, SUC_lexicon_tags_14 },
    { 0xbfdeca69, SUC_lexicon_tags_9 },
    { 0x3eca68, SUC_lexicon_tags_32 },
    { 0xd73eca6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf11eca6d, SUC_lexicon_tags_17 },
    { 0x1c3eca6d, SUC_lexicon_tags_17 },
    { 0x805eca6d, SUC_lexicon_tags_25 },
    { 0x5bfeca70, SUC_lexicon_tags_37 },
    { 0xbdbeca70, SUC_lexicon_tags_37 },
    { 0xb69eca72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3d1eca74, SUC_lexicon_tags_12 },
    { 0x1e7eca74, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437eca7b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x87eca7e, SUC_lexicon_tags_37 },
    { 0x25deca7f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29eca83, SUC_lexicon_tags_31 },
    { 0x8a5eca84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbbfeca86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17feca89, SUC_lexicon_tags_7 },
    { 0x9f5eca8a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9beca8d, SUC_lexicon_tags_7 },
    { 0x3bfeca8e, SUC_lexicon_tags_17 },
    { 0xfa9eca8f, SUC_lexicon_tags_15 },
    { 0xdbfeca8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01eca93, SUC_lexicon_tags_12 },
    { 0x779eca94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95eca9a, SUC_lexicon_tags_30 },
    { 0x5e5eca9b, SUC_lexicon_tags_31 },
    { 0x19feca9c, SUC_lexicon_tags_23 },
    { 0xbf5eca9c, SUC_lexicon_tags_38 },
    { 0x2c1eca9d, SUC_lexicon_tags_2 },
    { 0x95feca9f, SUC_lexicon_tags_15 },
    { 0xf13eca9d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebecaa8, SUC_lexicon_tags_30 },
    { 0xea7ecaa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7ecaac, SUC_lexicon_tags_28 },
    { 0x7dbecaad, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x861ecab0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xae5ecab2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57decab7, SUC_lexicon_tags_201 },
    { 0xf85ecab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafecabc, SUC_lexicon_tags_4 },
    { 0xcdbecabd, SUC_lexicon_tags_17 },
    { 0x67ecabe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23ecac2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc71ecac8, SUC_lexicon_tags_11 },
    { 0x969ecac9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ecad0, SUC_lexicon_tags_29 },
    { 0x9b1ecad0, SUC_lexicon_tags_15 },
    { 0x811ecad2, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x91ecad4, SUC_lexicon_tags_55 },
    { 0xa67ecad4, SUC_lexicon_tags_25 },
    { 0xf45ecad5, SUC_lexicon_tags_19 },
    { 0x355ecad6, SUC_lexicon_tags_45 },
    { 0xb0fecad8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07ecae1, SUC_lexicon_tags_25 },
    { 0xf3fecae2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999ecae6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ecae9, SUC_lexicon_tags_47 },
    { 0x891ecaea, SUC_lexicon_tags_48 },
    { 0x73becaeb, SUC_lexicon_tags_74 },
    { 0xbb7ecaec, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadecaf0, SUC_lexicon_tags_12 },
    { 0x9efecaf1, SUC_lexicon_tags_1 },
    { 0x807ecaf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x199ecaf4, SUC_lexicon_tags_12 },
    { 0x3afecaf5, SUC_lexicon_tags_29 },
    { 0xd03ecaf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577ecafa, SUC_lexicon_tags_11 },
    { 0xb3decafb, SUC_lexicon_tags_63 },
    { 0xb1decafc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11ecaff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9ecb08, SUC_lexicon_tags_12 },
    { 0x935ecb09, SUC_lexicon_tags_31 },
    { 0x33ecb08, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x925ecb0d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37decb15, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5afecb17, SUC_lexicon_tags_9 },
    { 0xa9decb17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13ecb1e, SUC_lexicon_tags_1 },
    { 0x577ecb1f, SUC_lexicon_tags_17 },
    { 0x959ecb20, SUC_lexicon_tags_8 },
    { 0xae5ecb1e, SUC_lexicon_tags_31 },
    { 0x86fecb1e, SUC_lexicon_tags_420 },
    { 0, NULL },
    { 0x3f1ecb24, SUC_lexicon_tags_30 },
    { 0xd23ecb25, SUC_lexicon_tags_37 },
    { 0x121ecb26, SUC_lexicon_tags_7 },
    { 0x853ecb27, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7fdecb29, SUC_lexicon_tags_5 },
    { 0x883ecb2a, SUC_lexicon_tags_14 },
    { 0x1b7ecb29, SUC_lexicon_tags_2 },
    { 0x4b3ecb29, SUC_lexicon_tags_17 },
    { 0xf4fecb2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x347ecb2f, SUC_lexicon_tags_12 },
    { 0xcc1ecb30, SUC_lexicon_tags_15 },
    { 0xb3fecb31, SUC_lexicon_tags_12 },
    { 0x535ecb30, SUC_lexicon_tags_19 },
    { 0x627ecb2f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fecb38, SUC_lexicon_tags_17 },
    { 0x6fdecb39, SUC_lexicon_tags_61 },
    { 0x781ecb3a, SUC_lexicon_tags_4 },
    { 0x1c1ecb3a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7ecb43, SUC_lexicon_tags_11 },
    { 0xf89ecb43, SUC_lexicon_tags_1 },
    { 0x7b1ecb43, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x25decb47, SUC_lexicon_tags_31 },
    { 0x86fecb48, SUC_lexicon_tags_33 },
    { 0x395ecb49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf13ecb4b, SUC_lexicon_tags_55 },
    { 0xb57ecb4c, SUC_lexicon_tags_12 },
    { 0xad1ecb4d, SUC_lexicon_tags_70 },
    { 0xcfecb4e, SUC_lexicon_tags_43 },
    { 0xa45ecb4f, SUC_lexicon_tags_35 },
    { 0x8d3ecb50, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53ecb54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc97ecb57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb3becb59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe61ecb5b, SUC_lexicon_tags_11 },
    { 0x925ecb5b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42fecb5e, SUC_lexicon_tags_33 },
    { 0x255ecb5e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409ecb68, SUC_lexicon_tags_63 },
    { 0x265ecb69, SUC_lexicon_tags_26 },
    { 0x95fecb6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9becb70, SUC_lexicon_tags_7 },
    { 0xc2decb71, SUC_lexicon_tags_23 },
    { 0xda5ecb71, SUC_lexicon_tags_37 },
    { 0x601ecb71, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29ecb77, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5ecb7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3ecb7f, SUC_lexicon_tags_15 },
    { 0x171ecb80, SUC_lexicon_tags_29 },
    { 0xec3ecb80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601ecb87, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbecb8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fecb91, SUC_lexicon_tags_17 },
    { 0x38becb92, SUC_lexicon_tags_14 },
    { 0x997ecb92, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x115ecb95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ecb9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe1becb9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfecba1, SUC_lexicon_tags_48 },
    { 0xcc7ecba2, SUC_lexicon_tags_1 },
    { 0x947ecba3, SUC_lexicon_tags_37 },
    { 0x213ecba1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x249ecba7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03ecbac, SUC_lexicon_tags_23 },
    { 0x2afecbac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfecbb3, SUC_lexicon_tags_19 },
    { 0x509ecbb4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x107ecbb7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xcc1ecbb9, SUC_lexicon_tags_7 },
    { 0x137ecbb9, SUC_lexicon_tags_3 },
    { 0xdcbecbb9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ecbbf, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fecbc2, SUC_lexicon_tags_37 },
    { 0x573ecbc3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x177ecbc6, SUC_lexicon_tags_49 },
    { 0xe5becbc6, SUC_lexicon_tags_10 },
    { 0xdfdecbc8, SUC_lexicon_tags_63 },
    { 0x1ebecbc6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7cbecbcb, SUC_lexicon_tags_7 },
    { 0x25fecbcc, SUC_lexicon_tags_25 },
    { 0x363ecbcd, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadecbd3, SUC_lexicon_tags_37 },
    { 0xd27ecbd4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09ecbd7, SUC_lexicon_tags_31 },
    { 0xa79ecbd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffecbdb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fecbde, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x63fecbe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5ecbe4, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x263ecbe6, SUC_lexicon_tags_33 },
    { 0x989ecbe7, SUC_lexicon_tags_27 },
    { 0xf13ecbe6, SUC_lexicon_tags_60 },
    { 0x19decbe8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9ebecbeb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcefecbef, SUC_lexicon_tags_14 },
    { 0x981ecbf0, SUC_lexicon_tags_14 },
    { 0x1c7ecbf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671ecbf7, SUC_lexicon_tags_30 },
    { 0xc61ecbf8, SUC_lexicon_tags_43 },
    { 0xad1ecbf9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515ecbfd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e9ecbff, SUC_lexicon_tags_5 },
    { 0xbb1ecbff, SUC_lexicon_tags_25 },
    { 0x183ecc01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x12fecc03, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fecc12, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993ecc16, SUC_lexicon_tags_60 },
    { 0xd85ecc17, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5ddecc19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x55becc1c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ecc20, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fecc23, SUC_lexicon_tags_13 },
    { 0x923ecc24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3ecc27, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1ecc2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbecc2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x945ecc30, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b3ecc32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x283ecc34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe7ecc36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3ecc3d, SUC_lexicon_tags_15 },
    { 0x3dbecc3d, SUC_lexicon_tags_6 },
    { 0xbb7ecc3f, SUC_lexicon_tags_30 },
    { 0x39ecc40, SUC_lexicon_tags_11 },
    { 0x2a7ecc40, SUC_lexicon_tags_27 },
    { 0x7ddecc3d, SUC_lexicon_tags_2 },
    { 0xc23ecc3f, SUC_lexicon_tags_48 },
    { 0x3f5ecc44, SUC_lexicon_tags_6 },
    { 0x513ecc41, SUC_lexicon_tags_9 },
    { 0x21ecc46, SUC_lexicon_tags_25 },
    { 0xccbecc46, SUC_lexicon_tags_25 },
    { 0x6ffecc47, SUC_lexicon_tags_57 },
    { 0x27fecc49, SUC_lexicon_tags_65 },
    { 0x71becc4a, SUC_lexicon_tags_32 },
    { 0x8f1ecc3e, SUC_lexicon_tags_11 },
    { 0xe79ecc4c, SUC_lexicon_tags_19 },
    { 0x7d5ecc4c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59ecc50, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23ecc53, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x52fecc55, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19ecc59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fecc5d, SUC_lexicon_tags_46 },
    { 0x6becc5d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653ecc65, SUC_lexicon_tags_16 },
    { 0x1b7ecc66, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47ecc6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fecc72, SUC_lexicon_tags_37 },
    { 0x3b5ecc73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80fecc75, SUC_lexicon_tags_7 },
    { 0xf97ecc76, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x773ecc78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8decc7c, SUC_lexicon_tags_1 },
    { 0xa09ecc7d, SUC_lexicon_tags_37 },
    { 0xabfecc7d, SUC_lexicon_tags_243 },
    { 0x9c5ecc7f, SUC_lexicon_tags_31 },
    { 0x7ddecc7d, SUC_lexicon_tags_2 },
    { 0x9c3ecc81, SUC_lexicon_tags_12 },
    { 0xbd7ecc81, SUC_lexicon_tags_25 },
    { 0xb9fecc81, SUC_lexicon_tags_91 },
    { 0x3a9ecc82, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331ecc89, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14fecc94, SUC_lexicon_tags_11 },
    { 0x3bdecc95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769ecc9b, SUC_lexicon_tags_33 },
    { 0xab3ecc9c, SUC_lexicon_tags_92 },
    { 0x37becc9b, SUC_lexicon_tags_43 },
    { 0xea7ecc9c, SUC_lexicon_tags_25 },
    { 0x819ecc9f, SUC_lexicon_tags_44 },
    { 0x579ecc9f, SUC_lexicon_tags_92 },
    { 0xe85ecc9b, SUC_lexicon_tags_12 },
    { 0x267ecca2, SUC_lexicon_tags_26 },
    { 0x7cdecca3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x12becca5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x54becca8, SUC_lexicon_tags_7 },
    { 0x4c3ecca8, SUC_lexicon_tags_79 },
    { 0x4beccaa, SUC_lexicon_tags_29 },
    { 0xb5feccaa, SUC_lexicon_tags_27 },
    { 0xe5eccab, SUC_lexicon_tags_12 },
    { 0xa49eccad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9beccb1, SUC_lexicon_tags_38 },
    { 0xd79eccb2, SUC_lexicon_tags_12 },
    { 0x98beccb3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5eccb6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9eccb9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd39eccbb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb31eccbd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77deccc0, SUC_lexicon_tags_1 },
    { 0x48deccc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe6feccc3, SUC_lexicon_tags_8 },
    { 0xb47eccc4, SUC_lexicon_tags_6 },
    { 0x239eccc5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x441eccc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5eccd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21eccd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x41eccd6, SUC_lexicon_tags_37 },
    { 0x367eccd6, SUC_lexicon_tags_29 },
    { 0x66beccd8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55deccdc, SUC_lexicon_tags_43 },
    { 0x1d1eccdd, SUC_lexicon_tags_17 },
    { 0x313eccde, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x987ecce0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x339ecce2, SUC_lexicon_tags_35 },
    { 0x91fecce3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ecce6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81decceb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9bfecced, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xecdeccef, SUC_lexicon_tags_37 },
    { 0xb4deccf0, SUC_lexicon_tags_12 },
    { 0x4adeccf0, SUC_lexicon_tags_2 },
    { 0xe7deccf2, SUC_lexicon_tags_12 },
    { 0x377eccf1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27eccf6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3eccfb, SUC_lexicon_tags_26 },
    { 0x629eccfc, SUC_lexicon_tags_7 },
    { 0x123eccfc, SUC_lexicon_tags_37 },
    { 0xb27eccfc, SUC_lexicon_tags_14 },
    { 0x711eccff, SUC_lexicon_tags_15 },
    { 0xdb1eccfe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fecd03, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9becd08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65decd0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9ecd0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5efecd0f, SUC_lexicon_tags_45 },
    { 0x27decd10, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40becd14, SUC_lexicon_tags_37 },
    { 0x9cdecd15, SUC_lexicon_tags_25 },
    { 0xe93ecd15, SUC_lexicon_tags_29 },
    { 0x247ecd17, SUC_lexicon_tags_27 },
    { 0x81fecd18, SUC_lexicon_tags_25 },
    { 0xfdbecd19, SUC_lexicon_tags_17 },
    { 0x3e5ecd18, SUC_lexicon_tags_19 },
    { 0xa5fecd1b, SUC_lexicon_tags_15 },
    { 0x421ecd1b, SUC_lexicon_tags_75 },
    { 0x43decd17, SUC_lexicon_tags_25 },
    { 0x975ecd15, SUC_lexicon_tags_47 },
    { 0xc29ecd1c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb87ecd21, SUC_lexicon_tags_87 },
    { 0x23fecd21, SUC_lexicon_tags_17 },
    { 0xdf9ecd22, SUC_lexicon_tags_37 },
    { 0x38decd24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9decd27, SUC_lexicon_tags_65 },
    { 0xf43ecd28, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ecd2b, SUC_lexicon_tags_46 },
    { 0x383ecd2c, SUC_lexicon_tags_68 },
    { 0x699ecd2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17ecd31, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9becd37, SUC_lexicon_tags_21 },
    { 0xce7ecd38, SUC_lexicon_tags_77 },
    { 0x3ddecd39, SUC_lexicon_tags_0 },
    { 0x5a7ecd3a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1e1ecd3c, SUC_lexicon_tags_47 },
    { 0x6d1ecd3d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fecd40, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x60becd43, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999ecd48, SUC_lexicon_tags_21 },
    { 0xa3becd49, SUC_lexicon_tags_33 },
    { 0x691ecd4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x57becd4c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd89ecd4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x111ecd50, SUC_lexicon_tags_37 },
    { 0xf0becd51, SUC_lexicon_tags_13 },
    { 0x54becd51, SUC_lexicon_tags_15 },
    { 0xeb9ecd50, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9fdecd55, SUC_lexicon_tags_37 },
    { 0x595ecd55, SUC_lexicon_tags_23 },
    { 0x375ecd57, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57ecd5a, SUC_lexicon_tags_38 },
    { 0x135ecd5a, SUC_lexicon_tags_1 },
    { 0x629ecd5c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdecd5f, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0x519ecd61, SUC_lexicon_tags_270 },
    { 0, NULL },
    { 0x24fecd63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ecd66, SUC_lexicon_tags_17 },
    { 0x405ecd67, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21ecd70, SUC_lexicon_tags_26 },
    { 0xeb3ecd70, SUC_lexicon_tags_16 },
    { 0x625ecd72, SUC_lexicon_tags_11 },
    { 0xdf3ecd73, SUC_lexicon_tags_37 },
    { 0xbd1ecd74, SUC_lexicon_tags_7 },
    { 0x5a9ecd75, SUC_lexicon_tags_1 },
    { 0xb21ecd75, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fecd7b, SUC_lexicon_tags_2 },
    { 0x6e5ecd7c, SUC_lexicon_tags_47 },
    { 0x5e3ecd7d, SUC_lexicon_tags_17 },
    { 0x831ecd7e, SUC_lexicon_tags_14 },
    { 0xc67ecd7d, SUC_lexicon_tags_2 },
    { 0xdbdecd80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8cdecd82, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27ecd8a, SUC_lexicon_tags_47 },
    { 0x9c7ecd8a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x785ecd8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd95ecd8f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb2fecd91, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99ecd99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ecd9d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x859ecda0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abecda4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x96decda7, SUC_lexicon_tags_2 },
    { 0x929ecda8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1ecdaf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7ecdb6, SUC_lexicon_tags_28 },
    { 0x38becdb6, SUC_lexicon_tags_37 },
    { 0x39ecdb8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42decdbd, SUC_lexicon_tags_43 },
    { 0x155ecdbe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x431ecdc0, SUC_lexicon_tags_17 },
    { 0x631ecdc1, SUC_lexicon_tags_1 },
    { 0xa29ecdc1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1decdc4, SUC_lexicon_tags_57 },
    { 0x45fecdc5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6ebecdc7, SUC_lexicon_tags_7 },
    { 0x5adecdc8, SUC_lexicon_tags_3 },
    { 0xccbecdc9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739ecdcd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7becdcf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x6cbecdd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fecdd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x391ecdd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ecddd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429ecde3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7ecde7, SUC_lexicon_tags_6 },
    { 0xedfecde8, SUC_lexicon_tags_19 },
    { 0x305ecde9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x277ecdeb, SUC_lexicon_tags_2 },
    { 0xf7ecdeb, SUC_lexicon_tags_37 },
    { 0xf53ecdec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9ecdf6, SUC_lexicon_tags_109 },
    { 0xf4becdf7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89ecdfb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3ecdfe, SUC_lexicon_tags_11 },
    { 0xd73ecdff, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45ece03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fece06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641ece0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x98fece0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7ece13, SUC_lexicon_tags_17 },
    { 0x143ece14, SUC_lexicon_tags_11 },
    { 0x5c9ece14, SUC_lexicon_tags_11 },
    { 0xe3ece16, SUC_lexicon_tags_63 },
    { 0x82fece17, SUC_lexicon_tags_1 },
    { 0xdf3ece17, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x395ece1a, SUC_lexicon_tags_74 },
    { 0xbd7ece1a, SUC_lexicon_tags_11 },
    { 0x1ece1c, SUC_lexicon_tags_37 },
    { 0x217ece1d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ebece20, SUC_lexicon_tags_38 },
    { 0x647ece20, SUC_lexicon_tags_15 },
    { 0x6b5ece22, SUC_lexicon_tags_33 },
    { 0x13fece22, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x507ece25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f1ece27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bece2c, SUC_lexicon_tags_17 },
    { 0x4d1ece2d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19ece30, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7ece33, SUC_lexicon_tags_87 },
    { 0xf5ece34, SUC_lexicon_tags_19 },
    { 0xf13ece33, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1ece3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbefece42, SUC_lexicon_tags_60 },
    { 0xf25ece42, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ffece48, SUC_lexicon_tags_8 },
    { 0x239ece49, SUC_lexicon_tags_12 },
    { 0xf4dece4a, SUC_lexicon_tags_19 },
    { 0xc99ece4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1ece50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x683ece52, SUC_lexicon_tags_17 },
    { 0xf0bece53, SUC_lexicon_tags_14 },
    { 0x68fece54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dece58, SUC_lexicon_tags_87 },
    { 0x747ece59, SUC_lexicon_tags_57 },
    { 0xcefece5a, SUC_lexicon_tags_25 },
    { 0x3f5ece5b, SUC_lexicon_tags_17 },
    { 0xecdece5c, SUC_lexicon_tags_82 },
    { 0x76dece5d, SUC_lexicon_tags_15 },
    { 0xf7dece5e, SUC_lexicon_tags_14 },
    { 0xaefece5e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdc7ece61, SUC_lexicon_tags_84 },
    { 0xf41ece61, SUC_lexicon_tags_1 },
    { 0xcd7ece62, SUC_lexicon_tags_17 },
    { 0x447ece64, SUC_lexicon_tags_2 },
    { 0xa27ece65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133ece6d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9ece73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc25ece76, SUC_lexicon_tags_7 },
    { 0x373ece77, SUC_lexicon_tags_13 },
    { 0x2f1ece78, SUC_lexicon_tags_38 },
    { 0xd29ece79, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9ece7f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x877ece82, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6f9ece84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1e3ece86, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5ece89, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ece8e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8efece90, SUC_lexicon_tags_1 },
    { 0xea3ece91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ece95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799ece9c, SUC_lexicon_tags_19 },
    { 0x1bbece9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1ecea2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61becea7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77eceab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbeceb1, SUC_lexicon_tags_7 },
    { 0x3ffeceb1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1eceb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95eceb9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5ecebc, SUC_lexicon_tags_45 },
    { 0x871ecebd, SUC_lexicon_tags_11 },
    { 0x3dbecebc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x865ecec1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x739ecec4, SUC_lexicon_tags_25 },
    { 0xb3fecec5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe5ecec7, SUC_lexicon_tags_63 },
    { 0x8ffecec8, SUC_lexicon_tags_47 },
    { 0xc2decec8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe01ececb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x315ecece, SUC_lexicon_tags_68 },
    { 0x9cbececf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45feceda, SUC_lexicon_tags_35 },
    { 0xe0becedb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8c9ecedd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f1ecedf, SUC_lexicon_tags_52 },
    { 0x5b9ecee0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73becee2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xcb3ecee4, SUC_lexicon_tags_12 },
    { 0x495ecee5, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51eceef, SUC_lexicon_tags_12 },
    { 0x6edecef0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3ecef4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5fecef6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48becef8, SUC_lexicon_tags_49 },
    { 0xfc1ecef9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1feceff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0fecf07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85ecf0e, SUC_lexicon_tags_37 },
    { 0x1e5ecf0f, SUC_lexicon_tags_98 },
    { 0xb79ecf10, SUC_lexicon_tags_17 },
    { 0xfffecf0f, SUC_lexicon_tags_7 },
    { 0xf67ecf10, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409ecf17, SUC_lexicon_tags_55 },
    { 0x85decf17, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92becf1c, SUC_lexicon_tags_14 },
    { 0xb8fecf1d, SUC_lexicon_tags_87 },
    { 0x729ecf1c, SUC_lexicon_tags_37 },
    { 0x9cbecf1f, SUC_lexicon_tags_15 },
    { 0x54decf1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9ecf25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x767ecf28, SUC_lexicon_tags_14 },
    { 0x3fdecf29, SUC_lexicon_tags_8 },
    { 0x5b3ecf29, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf2fecf2c, SUC_lexicon_tags_17 },
    { 0x335ecf2d, SUC_lexicon_tags_33 },
    { 0x27becf2d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9ecf33, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03ecf36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7ecf4a, SUC_lexicon_tags_25 },
    { 0x3c3ecf4b, SUC_lexicon_tags_25 },
    { 0x5b1ecf4c, SUC_lexicon_tags_37 },
    { 0x7d5ecf4d, SUC_lexicon_tags_28 },
    { 0xc7decf4d, SUC_lexicon_tags_17 },
    { 0x7d5ecf4f, SUC_lexicon_tags_1 },
    { 0x68decf50, SUC_lexicon_tags_17 },
    { 0x677ecf4f, SUC_lexicon_tags_55 },
    { 0x7bdecf52, SUC_lexicon_tags_11 },
    { 0xe79ecf4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3ecf57, SUC_lexicon_tags_19 },
    { 0xdc9ecf58, SUC_lexicon_tags_2 },
    { 0xe9decf59, SUC_lexicon_tags_11 },
    { 0x225ecf5a, SUC_lexicon_tags_37 },
    { 0xb5becf5b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbecf60, SUC_lexicon_tags_9 },
    { 0xb31ecf61, SUC_lexicon_tags_14 },
    { 0xf01ecf60, SUC_lexicon_tags_7 },
    { 0x925ecf60, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fecf69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdecf73, SUC_lexicon_tags_27 },
    { 0x57ecf73, SUC_lexicon_tags_69 },
    { 0xfe5ecf75, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x243ecf77, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ecf7b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x67becf7e, SUC_lexicon_tags_11 },
    { 0x385ecf7f, SUC_lexicon_tags_7 },
    { 0x27fecf7f, SUC_lexicon_tags_387 },
    { 0xd2decf7f, SUC_lexicon_tags_82 },
    { 0xc41ecf82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47ecf85, SUC_lexicon_tags_105 },
    { 0x93becf86, SUC_lexicon_tags_17 },
    { 0xbfecf85, SUC_lexicon_tags_37 },
    { 0x795ecf88, SUC_lexicon_tags_37 },
    { 0xa1decf86, SUC_lexicon_tags_0 },
    { 0xafdecf88, SUC_lexicon_tags_37 },
    { 0x20fecf8b, SUC_lexicon_tags_65 },
    { 0x185ecf8c, SUC_lexicon_tags_11 },
    { 0xc7becf8b, SUC_lexicon_tags_25 },
    { 0x969ecf8e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xeffecf91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85ecf98, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43ecf9b, SUC_lexicon_tags_55 },
    { 0xebbecf9b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5decfa3, SUC_lexicon_tags_29 },
    { 0xc99ecfa3, SUC_lexicon_tags_11 },
    { 0xce1ecfa5, SUC_lexicon_tags_15 },
    { 0xac3ecfa5, SUC_lexicon_tags_25 },
    { 0x29ecfa7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ecfaf, SUC_lexicon_tags_44 },
    { 0x57ecfb0, SUC_lexicon_tags_1 },
    { 0x7efecfb1, SUC_lexicon_tags_33 },
    { 0xa03ecfb2, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0xfd3ecfb4, SUC_lexicon_tags_25 },
    { 0xac5ecfb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfecfb9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe29ecfbb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9ecfc0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5ecfc6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x523ecfc8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x843ecfca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afecfce, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdefecfd0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbdecfd5, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7becfd9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1ecfdc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x867ecfde, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x57ecfe0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9ecfe6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ecfeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa51ecfee, SUC_lexicon_tags_2 },
    { 0xf6decfef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fecff2, SUC_lexicon_tags_16 },
    { 0x78becff3, SUC_lexicon_tags_15 },
    { 0x58becff2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469ecffa, SUC_lexicon_tags_15 },
    { 0x919ecffa, SUC_lexicon_tags_74 },
    { 0xbbdecffb, SUC_lexicon_tags_47 },
    { 0xd69ecffd, SUC_lexicon_tags_15 },
    { 0x485ecffc, SUC_lexicon_tags_45 },
    { 0x443ecfff, SUC_lexicon_tags_7 },
    { 0x5c1ed000, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x327ed003, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x50ded006, SUC_lexicon_tags_27 },
    { 0xa1fed007, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9ed00a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837ed00e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa9ed010, SUC_lexicon_tags_27 },
    { 0xb9fed011, SUC_lexicon_tags_12 },
    { 0x509ed011, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa2ded014, SUC_lexicon_tags_1 },
    { 0xfa7ed014, SUC_lexicon_tags_32 },
    { 0xea7ed016, SUC_lexicon_tags_38 },
    { 0x1ded017, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd4fed019, SUC_lexicon_tags_14 },
    { 0xfa1ed01a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581ed01e, SUC_lexicon_tags_19 },
    { 0x113ed01e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fed022, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xa5fed024, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1ed029, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xbe3ed02b, SUC_lexicon_tags_11 },
    { 0x537ed02c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fed02f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cded032, SUC_lexicon_tags_31 },
    { 0x21fed033, SUC_lexicon_tags_13 },
    { 0xe4fed033, SUC_lexicon_tags_17 },
    { 0x14bed035, SUC_lexicon_tags_63 },
    { 0x9cbed034, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x661ed038, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafed03c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9ed040, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb55ed042, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xea1ed044, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x313ed046, SUC_lexicon_tags_25 },
    { 0x3c1ed046, SUC_lexicon_tags_11 },
    { 0x9fed048, SUC_lexicon_tags_12 },
    { 0xe9ed047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9abed04b, SUC_lexicon_tags_17 },
    { 0xf4bed04c, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bed04f, SUC_lexicon_tags_37 },
    { 0x461ed050, SUC_lexicon_tags_14 },
    { 0x323ed04f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5ed05a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701ed05f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7ed066, SUC_lexicon_tags_14 },
    { 0x59ded067, SUC_lexicon_tags_46 },
    { 0xef5ed068, SUC_lexicon_tags_38 },
    { 0x359ed068, SUC_lexicon_tags_38 },
    { 0x2d1ed069, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x20fed06c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x935ed06e, SUC_lexicon_tags_42 },
    { 0x487ed06e, SUC_lexicon_tags_57 },
    { 0x925ed070, SUC_lexicon_tags_27 },
    { 0x47bed071, SUC_lexicon_tags_14 },
    { 0x989ed072, SUC_lexicon_tags_15 },
    { 0x463ed073, SUC_lexicon_tags_30 },
    { 0x393ed072, SUC_lexicon_tags_45 },
    { 0xc81ed075, SUC_lexicon_tags_34 },
    { 0xbfed075, SUC_lexicon_tags_38 },
    { 0x1ded06e, SUC_lexicon_tags_1 },
    { 0xf01ed077, SUC_lexicon_tags_59 },
    { 0x7b5ed079, SUC_lexicon_tags_7 },
    { 0x483ed07a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa95ed07c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd57ed07e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bed085, SUC_lexicon_tags_47 },
    { 0xbd7ed086, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1ed08a, SUC_lexicon_tags_1 },
    { 0x47fed08b, SUC_lexicon_tags_17 },
    { 0x3cbed08a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8b1ed08e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bed091, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x845ed094, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa95ed096, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241ed09b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165ed09f, SUC_lexicon_tags_37 },
    { 0xc55ed0a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8a9ed0a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x43bed0a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ded0a7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bed0aa, SUC_lexicon_tags_55 },
    { 0xfe3ed0aa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x47ed0ad, SUC_lexicon_tags_14 },
    { 0x1afed0ae, SUC_lexicon_tags_15 },
    { 0xaa5ed0ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb6fed0b1, SUC_lexicon_tags_54 },
    { 0xd05ed0b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x853ed0b5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7ed0b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x53ded0bb, SUC_lexicon_tags_12 },
    { 0x50ded0bc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf4fed0be, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27ed0c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x945ed0c6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbed0ca, SUC_lexicon_tags_11 },
    { 0x797ed0ca, SUC_lexicon_tags_2 },
    { 0x90ded0cb, SUC_lexicon_tags_25 },
    { 0xd15ed0cb, SUC_lexicon_tags_8 },
    { 0x42ded0cc, SUC_lexicon_tags_17 },
    { 0xd05ed0cf, SUC_lexicon_tags_8 },
    { 0x889ed0d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983ed0d5, SUC_lexicon_tags_77 },
    { 0xd3fed0d6, SUC_lexicon_tags_44 },
    { 0xbf5ed0d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ded0da, SUC_lexicon_tags_1 },
    { 0x3f7ed0db, SUC_lexicon_tags_11 },
    { 0x439ed0dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce5ed0de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ded0e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9ed0e7, SUC_lexicon_tags_19 },
    { 0x8d7ed0e8, SUC_lexicon_tags_27 },
    { 0x625ed0e7, SUC_lexicon_tags_38 },
    { 0xee1ed0e9, SUC_lexicon_tags_63 },
    { 0x741ed0e7, SUC_lexicon_tags_4 },
    { 0x7f9ed0ec, SUC_lexicon_tags_56 },
    { 0x70ded0ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e3ed0ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323ed0f3, SUC_lexicon_tags_17 },
    { 0x8f7ed0f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd1ed0f6, SUC_lexicon_tags_37 },
    { 0xe6ded0f7, SUC_lexicon_tags_28 },
    { 0x8dfed0f8, SUC_lexicon_tags_37 },
    { 0x60ded0f9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x225ed0fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5ed101, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbded105, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x19ded108, SUC_lexicon_tags_60 },
    { 0xfc1ed108, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bed10d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x14bed10f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x781ed111, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x579ed113, SUC_lexicon_tags_47 },
    { 0xa35ed114, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ed117, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x62bed119, SUC_lexicon_tags_25 },
    { 0xfc7ed119, SUC_lexicon_tags_3 },
    { 0x32bed11b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7d5ed11d, SUC_lexicon_tags_23 },
    { 0x3f9ed11e, SUC_lexicon_tags_28 },
    { 0x731ed11e, SUC_lexicon_tags_48 },
    { 0x509ed120, SUC_lexicon_tags_7 },
    { 0xff3ed11d, SUC_lexicon_tags_37 },
    { 0xf21ed122, SUC_lexicon_tags_17 },
    { 0xc07ed123, SUC_lexicon_tags_4 },
    { 0x911ed123, SUC_lexicon_tags_117 },
    { 0xd2ded125, SUC_lexicon_tags_27 },
    { 0xee5ed126, SUC_lexicon_tags_15 },
    { 0x233ed123, SUC_lexicon_tags_14 },
    { 0x373ed122, SUC_lexicon_tags_60 },
    { 0x54fed129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8fed12e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58ded130, SUC_lexicon_tags_10 },
    { 0xaa3ed130, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdcded133, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9d7ed135, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ded140, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafed143, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4afed146, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ed149, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3ed14e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1ed151, SUC_lexicon_tags_204 },
    { 0x6a7ed151, SUC_lexicon_tags_15 },
    { 0xcffed152, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ed159, SUC_lexicon_tags_7 },
    { 0x535ed15a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fded15f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31ed163, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7ed169, SUC_lexicon_tags_31 },
    { 0xe1ed16a, SUC_lexicon_tags_11 },
    { 0x7f5ed16a, SUC_lexicon_tags_11 },
    { 0x665ed16c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb3ed16e, SUC_lexicon_tags_27 },
    { 0x855ed16f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x203ed172, SUC_lexicon_tags_11 },
    { 0x1d5ed173, SUC_lexicon_tags_33 },
    { 0x8dbed174, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3ed178, SUC_lexicon_tags_7 },
    { 0xeb7ed178, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xffbed17b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x675ed17d, SUC_lexicon_tags_31 },
    { 0x25ed17e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x279ed181, SUC_lexicon_tags_12 },
    { 0x5afed182, SUC_lexicon_tags_45 },
    { 0xecbed183, SUC_lexicon_tags_23 },
    { 0xa55ed184, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9ed187, SUC_lexicon_tags_68 },
    { 0xc5ded188, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebed18b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cded18e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95ed190, SUC_lexicon_tags_2 },
    { 0xdc7ed191, SUC_lexicon_tags_12 },
    { 0xbcfed192, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77ed19b, SUC_lexicon_tags_27 },
    { 0x35fed19c, SUC_lexicon_tags_14 },
    { 0xc4bed19d, SUC_lexicon_tags_49 },
    { 0xc65ed19d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf1ed1a0, SUC_lexicon_tags_26 },
    { 0x98bed1a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643ed1aa, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbed1ae, SUC_lexicon_tags_11 },
    { 0x371ed1af, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x12fed1b1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x5a7ed1b3, SUC_lexicon_tags_84 },
    { 0xdebed1b3, SUC_lexicon_tags_46 },
    { 0x741ed1b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdd3ed1b7, SUC_lexicon_tags_2 },
    { 0xb09ed1b8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53ed1bc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x7f9ed1be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d9ed1c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13ed1c3, SUC_lexicon_tags_37 },
    { 0xb5bed1c3, SUC_lexicon_tags_792 },
    { 0, NULL },
    { 0xd5bed1c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7ed1ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x64fed1cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3b3ed1cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe19ed1d1, SUC_lexicon_tags_60 },
    { 0x957ed1d1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x18bed1d4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcded1d6, SUC_lexicon_tags_1 },
    { 0x539ed1d7, SUC_lexicon_tags_17 },
    { 0x653ed1d7, SUC_lexicon_tags_31 },
    { 0xa39ed1d9, SUC_lexicon_tags_16 },
    { 0x121ed1d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aded1e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bed1e3, SUC_lexicon_tags_37 },
    { 0x405ed1e4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87ed1ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1ed1f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fed201, SUC_lexicon_tags_46 },
    { 0xac1ed201, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7ed207, SUC_lexicon_tags_37 },
    { 0x5f9ed208, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515ed20c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11ded20e, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afed21b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3ed21f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ed227, SUC_lexicon_tags_23 },
    { 0x889ed228, SUC_lexicon_tags_37 },
    { 0x7f3ed228, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fed22c, SUC_lexicon_tags_37 },
    { 0x637ed22d, SUC_lexicon_tags_74 },
    { 0x7fed22e, SUC_lexicon_tags_946 },
    { 0x673ed22f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8fed231, SUC_lexicon_tags_25 },
    { 0xf15ed231, SUC_lexicon_tags_29 },
    { 0x88ded231, SUC_lexicon_tags_37 },
    { 0x223ed233, SUC_lexicon_tags_11 },
    { 0x8aded231, SUC_lexicon_tags_1 },
    { 0x663ed236, SUC_lexicon_tags_30 },
    { 0xea1ed237, SUC_lexicon_tags_15 },
    { 0xcd5ed238, SUC_lexicon_tags_27 },
    { 0x189ed239, SUC_lexicon_tags_40 },
    { 0x7b5ed238, SUC_lexicon_tags_25 },
    { 0xd57ed238, SUC_lexicon_tags_7 },
    { 0x679ed23b, SUC_lexicon_tags_49 },
    { 0x6ffed23b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x725ed240, SUC_lexicon_tags_27 },
    { 0xf31ed241, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05ed244, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x453ed246, SUC_lexicon_tags_28 },
    { 0xc4fed247, SUC_lexicon_tags_45 },
    { 0x241ed248, SUC_lexicon_tags_11 },
    { 0x879ed247, SUC_lexicon_tags_30 },
    { 0x5f1ed24a, SUC_lexicon_tags_1114 },
    { 0xafbed24a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5eded252, SUC_lexicon_tags_66 },
    { 0x78fed252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fed256, SUC_lexicon_tags_1 },
    { 0x54ded256, SUC_lexicon_tags_27 },
    { 0x715ed256, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3ed25b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63ed25f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1ed263, SUC_lexicon_tags_17 },
    { 0xd21ed263, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819ed268, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x845ed26a, SUC_lexicon_tags_101 },
    { 0x137ed26b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9ed278, SUC_lexicon_tags_1 },
    { 0x2e9ed279, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ed27e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x457ed280, SUC_lexicon_tags_63 },
    { 0x9f3ed281, SUC_lexicon_tags_11 },
    { 0xeefed281, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327ed286, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x61ed289, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9ed28f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x84fed291, SUC_lexicon_tags_17 },
    { 0x231ed291, SUC_lexicon_tags_19 },
    { 0xb09ed293, SUC_lexicon_tags_30 },
    { 0x729ed294, SUC_lexicon_tags_27 },
    { 0x953ed293, SUC_lexicon_tags_37 },
    { 0x25bed296, SUC_lexicon_tags_7 },
    { 0x82bed297, SUC_lexicon_tags_47 },
    { 0xb7ed293, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787ed29e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fed2ad, SUC_lexicon_tags_56 },
    { 0x66bed2ae, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x91fed2b0, SUC_lexicon_tags_63 },
    { 0x1f9ed2b1, SUC_lexicon_tags_91 },
    { 0x45ed2b2, SUC_lexicon_tags_944 },
    { 0x69ded2b3, SUC_lexicon_tags_6 },
    { 0x5cded2b3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1ed2b7, SUC_lexicon_tags_65 },
    { 0xdc1ed2b8, SUC_lexicon_tags_45 },
    { 0xb3bed2b8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb91ed2bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ed2c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fed2c5, SUC_lexicon_tags_17 },
    { 0xb69ed2c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xddded2c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc7ed2ca, SUC_lexicon_tags_1 },
    { 0x911ed2ca, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xeefed2cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d3ed2cf, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8fed2d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfed2d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1ed2dd, SUC_lexicon_tags_37 },
    { 0xbf5ed2dd, SUC_lexicon_tags_136 },
    { 0xb83ed2df, SUC_lexicon_tags_60 },
    { 0x99ed2e0, SUC_lexicon_tags_49 },
    { 0x2e7ed2e1, SUC_lexicon_tags_7 },
    { 0x11ed2e1, SUC_lexicon_tags_7 },
    { 0x521ed2e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bed2e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbed2ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ed2f2, SUC_lexicon_tags_17 },
    { 0xe1bed2f3, SUC_lexicon_tags_82 },
    { 0xec1ed2f3, SUC_lexicon_tags_49 },
    { 0x939ed2f3, SUC_lexicon_tags_33 },
    { 0x8f9ed2f5, SUC_lexicon_tags_15 },
    { 0x4fed2f7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x403ed2f9, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1ed2fd, SUC_lexicon_tags_46 },
    { 0xd93ed2fe, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9ed301, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963ed307, SUC_lexicon_tags_11 },
    { 0x7bed307, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cded30b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa79ed30d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7ed310, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253ed315, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39ed31b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x50bed31e, SUC_lexicon_tags_11 },
    { 0xc6fed31f, SUC_lexicon_tags_40 },
    { 0x9bed320, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x41ded322, SUC_lexicon_tags_1 },
    { 0xae9ed323, SUC_lexicon_tags_11 },
    { 0x391ed323, SUC_lexicon_tags_1 },
    { 0x931ed325, SUC_lexicon_tags_42 },
    { 0x64fed326, SUC_lexicon_tags_43 },
    { 0x1e9ed325, SUC_lexicon_tags_7 },
    { 0x209ed326, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499ed32e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fed339, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3ed33d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x229ed340, SUC_lexicon_tags_7 },
    { 0x5e7ed341, SUC_lexicon_tags_54 },
    { 0x429ed342, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd07ed344, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807ed34e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45ed351, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05ed356, SUC_lexicon_tags_17 },
    { 0xadbed357, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2d3ed359, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x995ed35c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bed360, SUC_lexicon_tags_18 },
    { 0x557ed361, SUC_lexicon_tags_7 },
    { 0x189ed360, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddded367, SUC_lexicon_tags_17 },
    { 0x945ed368, SUC_lexicon_tags_26 },
    { 0x33fed368, SUC_lexicon_tags_63 },
    { 0x781ed369, SUC_lexicon_tags_2 },
    { 0x139ed36b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa27ed36d, SUC_lexicon_tags_151 },
    { 0x681ed36d, SUC_lexicon_tags_40 },
    { 0x47bed36f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x10ded371, SUC_lexicon_tags_106 },
    { 0xe95ed372, SUC_lexicon_tags_47 },
    { 0xec1ed373, SUC_lexicon_tags_2 },
    { 0xfaded374, SUC_lexicon_tags_15 },
    { 0x2c9ed374, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa5fed377, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18bed379, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3ed37d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953ed381, SUC_lexicon_tags_63 },
    { 0xbaded382, SUC_lexicon_tags_43 },
    { 0xba9ed382, SUC_lexicon_tags_11 },
    { 0xc07ed381, SUC_lexicon_tags_27 },
    { 0x6c3ed381, SUC_lexicon_tags_2 },
    { 0x7abed386, SUC_lexicon_tags_68 },
    { 0x57fed386, SUC_lexicon_tags_79 },
    { 0x89bed385, SUC_lexicon_tags_25 },
    { 0xca9ed388, SUC_lexicon_tags_33 },
    { 0x79bed381, SUC_lexicon_tags_38 },
    { 0xf41ed38b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc43ed38d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1ed390, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42fed394, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x881ed39a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93ed3a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1ed3a8, SUC_lexicon_tags_30 },
    { 0x151ed3a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9ed3ac, SUC_lexicon_tags_37 },
    { 0x7dded3ad, SUC_lexicon_tags_2 },
    { 0x8d7ed3ad, SUC_lexicon_tags_1 },
    { 0xaebed3af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5ed3b2, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51ed3b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65ed3b8, SUC_lexicon_tags_25 },
    { 0xe1ded3b9, SUC_lexicon_tags_7 },
    { 0x4c9ed3ba, SUC_lexicon_tags_26 },
    { 0xab1ed3b8, SUC_lexicon_tags_7 },
    { 0x75ed3bc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5fed3bf, SUC_lexicon_tags_1 },
    { 0x6a3ed3c0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x375ed3c3, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x317ed3c7, SUC_lexicon_tags_15 },
    { 0xf0ded3c7, SUC_lexicon_tags_37 },
    { 0x601ed3c9, SUC_lexicon_tags_11 },
    { 0xeeded3c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9ed3d0, SUC_lexicon_tags_47 },
    { 0xb7bed3d0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5ed3d5, SUC_lexicon_tags_11 },
    { 0x7a7ed3d6, SUC_lexicon_tags_43 },
    { 0xe4bed3d6, SUC_lexicon_tags_0 },
    { 0x639ed3d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8eded3da, SUC_lexicon_tags_60 },
    { 0x51fed3da, SUC_lexicon_tags_17 },
    { 0xb31ed3db, SUC_lexicon_tags_38 },
    { 0x7d9ed3dd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783ed3e3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x265ed3f1, SUC_lexicon_tags_12 },
    { 0xeded3f2, SUC_lexicon_tags_26 },
    { 0x41ded3f1, SUC_lexicon_tags_8 },
    { 0xaa3ed3f3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a9ed3f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7ed3f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba9ed3fb, SUC_lexicon_tags_2 },
    { 0x689ed3fb, SUC_lexicon_tags_11 },
    { 0x729ed3fd, SUC_lexicon_tags_34 },
    { 0x1abed3fe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19ed403, SUC_lexicon_tags_12 },
    { 0x269ed404, SUC_lexicon_tags_25 },
    { 0xbd3ed405, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa83ed407, SUC_lexicon_tags_60 },
    { 0xc0bed408, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7ed40b, SUC_lexicon_tags_17 },
    { 0xf89ed40c, SUC_lexicon_tags_13 },
    { 0x40fed40d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bded413, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5ed417, SUC_lexicon_tags_37 },
    { 0x785ed417, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5a7ed41a, SUC_lexicon_tags_1 },
    { 0xed7ed41b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbed420, SUC_lexicon_tags_30 },
    { 0x2e9ed421, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x13fed423, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa37ed425, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbed42d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07ed430, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bed433, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bed436, SUC_lexicon_tags_37 },
    { 0x831ed437, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ded43d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55ed440, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeabed442, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fed447, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x803ed44b, SUC_lexicon_tags_12 },
    { 0x36ded44c, SUC_lexicon_tags_2 },
    { 0x9bfed44d, SUC_lexicon_tags_2 },
    { 0x11ded44e, SUC_lexicon_tags_29 },
    { 0x9b7ed44e, SUC_lexicon_tags_44 },
    { 0x191ed44d, SUC_lexicon_tags_1 },
    { 0xe83ed451, SUC_lexicon_tags_13 },
    { 0xa85ed452, SUC_lexicon_tags_2 },
    { 0x8f5ed453, SUC_lexicon_tags_7 },
    { 0xa1ed450, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1ed458, SUC_lexicon_tags_15 },
    { 0x9ded459, SUC_lexicon_tags_11 },
    { 0x6c3ed45a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd35ed45c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1ed45f, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xffded461, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bed465, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x55ed467, SUC_lexicon_tags_60 },
    { 0x42bed468, SUC_lexicon_tags_45 },
    { 0x545ed468, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853ed46d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1ed472, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x357ed475, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5ed479, SUC_lexicon_tags_1 },
    { 0x229ed47a, SUC_lexicon_tags_27 },
    { 0x97ded47b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfed482, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x965ed484, SUC_lexicon_tags_38 },
    { 0xe23ed485, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3ed48f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ded494, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5ed49b, SUC_lexicon_tags_28 },
    { 0xe09ed49c, SUC_lexicon_tags_23 },
    { 0x789ed49b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bed4a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43ded4a4, SUC_lexicon_tags_68 },
    { 0xb0bed4a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9ed4ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ded4b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ded4b6, SUC_lexicon_tags_7 },
    { 0x831ed4b7, SUC_lexicon_tags_23 },
    { 0x1d1ed4b6, SUC_lexicon_tags_7 },
    { 0x1f5ed4b7, SUC_lexicon_tags_25 },
    { 0x1c1ed4b7, SUC_lexicon_tags_0 },
    { 0x4d7ed4bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x965ed4be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ded4c1, SUC_lexicon_tags_11 },
    { 0x253ed4c1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x445ed4c4, SUC_lexicon_tags_23 },
    { 0x2afed4c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fded4c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd7ed4ca, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5ed4cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5ed4d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x267ed4d5, SUC_lexicon_tags_140 },
    { 0x559ed4d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1ed4d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x133ed4db, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x661ed4dd, SUC_lexicon_tags_28 },
    { 0xaa3ed4de, SUC_lexicon_tags_61 },
    { 0x7bfed4df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d1ed4e1, SUC_lexicon_tags_50 },
    { 0x7ed4e1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x185ed4e5, SUC_lexicon_tags_37 },
    { 0x9fded4e6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc8ded4e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ded4f7, SUC_lexicon_tags_82 },
    { 0xdbfed4f7, SUC_lexicon_tags_7 },
    { 0xf5ed4f9, SUC_lexicon_tags_15 },
    { 0x689ed4fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7ed4fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99ed502, SUC_lexicon_tags_17 },
    { 0xac7ed503, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf6ded505, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc5ed507, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ded50a, SUC_lexicon_tags_87 },
    { 0xed3ed50a, SUC_lexicon_tags_2 },
    { 0xbd5ed50c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x70bed50e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691ed512, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49ed516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x229ed519, SUC_lexicon_tags_15 },
    { 0x9bbed519, SUC_lexicon_tags_4 },
    { 0x8b7ed519, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbc5ed51d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafed520, SUC_lexicon_tags_23 },
    { 0xd6ded521, SUC_lexicon_tags_402 },
    { 0x67ded520, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bed527, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xafbed529, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bed52e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x22fed531, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ded534, SUC_lexicon_tags_15 },
    { 0xe45ed534, SUC_lexicon_tags_37 },
    { 0xb89ed536, SUC_lexicon_tags_23 },
    { 0x2e9ed537, SUC_lexicon_tags_8 },
    { 0x72bed538, SUC_lexicon_tags_49 },
    { 0x6e1ed538, SUC_lexicon_tags_47 },
    { 0xc37ed534, SUC_lexicon_tags_17 },
    { 0x441ed536, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517ed543, SUC_lexicon_tags_37 },
    { 0xce1ed543, SUC_lexicon_tags_15 },
    { 0x93bed544, SUC_lexicon_tags_15 },
    { 0x327ed544, SUC_lexicon_tags_74 },
    { 0x267ed544, SUC_lexicon_tags_1 },
    { 0xcb9ed543, SUC_lexicon_tags_32 },
    { 0x85ed549, SUC_lexicon_tags_21 },
    { 0x2bed54a, SUC_lexicon_tags_87 },
    { 0xb3ed546, SUC_lexicon_tags_37 },
    { 0x6c7ed54c, SUC_lexicon_tags_12 },
    { 0x20ded54d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7aded550, SUC_lexicon_tags_7 },
    { 0x13fed551, SUC_lexicon_tags_1 },
    { 0x6cbed551, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9dded554, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffed558, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1ed55c, SUC_lexicon_tags_2 },
    { 0x1bfed55d, SUC_lexicon_tags_17 },
    { 0x287ed55e, SUC_lexicon_tags_38 },
    { 0x5a9ed55f, SUC_lexicon_tags_37 },
    { 0x803ed55d, SUC_lexicon_tags_9 },
    { 0x2c7ed561, SUC_lexicon_tags_0 },
    { 0x35bed561, SUC_lexicon_tags_23 },
    { 0xf0bed563, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ed567, SUC_lexicon_tags_15 },
    { 0x6a1ed568, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x443ed56a, SUC_lexicon_tags_31 },
    { 0x573ed56b, SUC_lexicon_tags_11 },
    { 0x21ded56b, SUC_lexicon_tags_14 },
    { 0x77bed56d, SUC_lexicon_tags_47 },
    { 0x215ed56d, SUC_lexicon_tags_6 },
    { 0xfbfed56f, SUC_lexicon_tags_37 },
    { 0x115ed570, SUC_lexicon_tags_12 },
    { 0xda9ed56c, SUC_lexicon_tags_17 },
    { 0xc79ed572, SUC_lexicon_tags_48 },
    { 0x4f5ed56a, SUC_lexicon_tags_17 },
    { 0xeafed574, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37ed578, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29ded57a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305ed57f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1ed584, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad5ed586, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4abed588, SUC_lexicon_tags_11 },
    { 0xbbbed588, SUC_lexicon_tags_23 },
    { 0x317ed58a, SUC_lexicon_tags_178 },
    { 0xb5bed58b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3ed590, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ded594, SUC_lexicon_tags_1 },
    { 0xfd5ed594, SUC_lexicon_tags_25 },
    { 0x327ed596, SUC_lexicon_tags_1151 },
    { 0x47ded597, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359ed59d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x287ed5a0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1ed5a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59ed5ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bed5ae, SUC_lexicon_tags_12 },
    { 0xe1ded5af, SUC_lexicon_tags_12 },
    { 0xe6fed5b0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47ed5b4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x675ed5ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409ed5c0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe1ed5c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7ed5cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbabed5ce, SUC_lexicon_tags_31 },
    { 0x4b3ed5cf, SUC_lexicon_tags_12 },
    { 0xd95ed5d0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6fed5d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965ed5e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8aded5e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x13fed5e5, SUC_lexicon_tags_31 },
    { 0xd97ed5e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x88ded5e8, SUC_lexicon_tags_37 },
    { 0x45ded5e8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c1ed5ee, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89ed5f2, SUC_lexicon_tags_25 },
    { 0x333ed5f2, SUC_lexicon_tags_33 },
    { 0x2fbed5f4, SUC_lexicon_tags_45 },
    { 0xe7ed5f4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5ed5f8, SUC_lexicon_tags_44 },
    { 0xc41ed5f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8d3ed5fb, SUC_lexicon_tags_27 },
    { 0x343ed5fc, SUC_lexicon_tags_43 },
    { 0x6a3ed5fd, SUC_lexicon_tags_129 },
    { 0x4c3ed5fe, SUC_lexicon_tags_14 },
    { 0xdd5ed5fe, SUC_lexicon_tags_29 },
    { 0xc7ded600, SUC_lexicon_tags_37 },
    { 0x63ed601, SUC_lexicon_tags_17 },
    { 0x417ed602, SUC_lexicon_tags_16 },
    { 0xa1bed602, SUC_lexicon_tags_79 },
    { 0xa41ed5fc, SUC_lexicon_tags_17 },
    { 0x35bed5fc, SUC_lexicon_tags_4 },
    { 0x3cbed604, SUC_lexicon_tags_15 },
    { 0x6cfed607, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x237ed60a, SUC_lexicon_tags_1 },
    { 0x36fed60b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969ed611, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x891ed613, SUC_lexicon_tags_49 },
    { 0xe05ed614, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfed61a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bded623, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9b9ed625, SUC_lexicon_tags_3 },
    { 0xce3ed625, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509ed62a, SUC_lexicon_tags_37 },
    { 0xa6bed62b, SUC_lexicon_tags_16 },
    { 0x117ed62b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x401ed62f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713ed634, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bed63a, SUC_lexicon_tags_60 },
    { 0x135ed63b, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x46ded63d, SUC_lexicon_tags_60 },
    { 0x387ed63d, SUC_lexicon_tags_1 },
    { 0xde1ed63d, SUC_lexicon_tags_2 },
    { 0x287ed63d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdd5ed642, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23ed64a, SUC_lexicon_tags_37 },
    { 0x731ed64a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73ed653, SUC_lexicon_tags_2 },
    { 0xa51ed654, SUC_lexicon_tags_26 },
    { 0x21ed654, SUC_lexicon_tags_38 },
    { 0xfe7ed656, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1ed65c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3ed662, SUC_lexicon_tags_37 },
    { 0x8ded662, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x191ed665, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5bed667, SUC_lexicon_tags_2 },
    { 0xd2bed667, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909ed66d, SUC_lexicon_tags_7 },
    { 0x5bed66d, SUC_lexicon_tags_12 },
    { 0xa63ed66f, SUC_lexicon_tags_28 },
    { 0x881ed66f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xed1ed672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1ed675, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9ed679, SUC_lexicon_tags_45 },
    { 0xeafed67a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x695ed67d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x289ed683, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdefed687, SUC_lexicon_tags_31 },
    { 0x87ed688, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf87ed68a, SUC_lexicon_tags_60 },
    { 0x547ed68a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x62ded68d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ed690, SUC_lexicon_tags_29 },
    { 0x94bed690, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2d5ed693, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9ed698, SUC_lexicon_tags_25 },
    { 0xf89ed699, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ed69f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537ed6a4, SUC_lexicon_tags_30 },
    { 0x73ded6a5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69ed6a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x15ded6ab, SUC_lexicon_tags_15 },
    { 0x5e9ed6ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1ed6b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fed6b8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9ed6bb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x395ed6bd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95ed6c0, SUC_lexicon_tags_30 },
    { 0x533ed6c1, SUC_lexicon_tags_28 },
    { 0x781ed6c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfefed6c4, SUC_lexicon_tags_17 },
    { 0x7eded6c4, SUC_lexicon_tags_1185 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107ed6ca, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfa9ed6cc, SUC_lexicon_tags_46 },
    { 0x22fed6cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9ed6d2, SUC_lexicon_tags_1260 },
    { 0xd59ed6d3, SUC_lexicon_tags_25 },
    { 0x9ebed6d4, SUC_lexicon_tags_11 },
    { 0x1b1ed6d5, SUC_lexicon_tags_25 },
    { 0x6e1ed6d4, SUC_lexicon_tags_32 },
    { 0xfefed6d7, SUC_lexicon_tags_1 },
    { 0xea7ed6d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56fed6dc, SUC_lexicon_tags_38 },
    { 0x2cfed6dc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4c9ed6df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119ed6e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xccfed6e6, SUC_lexicon_tags_15 },
    { 0x997ed6e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1ed6ed, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1ed6f2, SUC_lexicon_tags_25 },
    { 0xa01ed6f2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5cfed6f5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe5ed6f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa2ded6f9, SUC_lexicon_tags_27 },
    { 0x39bed6fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x189ed6fd, SUC_lexicon_tags_9 },
    { 0x813ed6fd, SUC_lexicon_tags_68 },
    { 0x999ed6ff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ed706, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1ed709, SUC_lexicon_tags_129 },
    { 0x1dbed70a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bed70d, SUC_lexicon_tags_39 },
    { 0x4efed70e, SUC_lexicon_tags_15 },
    { 0x7abed70f, SUC_lexicon_tags_30 },
    { 0x407ed710, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x551ed712, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fed715, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x417ed718, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe41ed71a, SUC_lexicon_tags_63 },
    { 0x90bed71a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fded71d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5ed720, SUC_lexicon_tags_0 },
    { 0x6c5ed720, SUC_lexicon_tags_11 },
    { 0x9dbed722, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cded726, SUC_lexicon_tags_65 },
    { 0x9cbed727, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1bed729, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ded72c, SUC_lexicon_tags_49 },
    { 0x1bed72d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fed731, SUC_lexicon_tags_12 },
    { 0x4f5ed732, SUC_lexicon_tags_12 },
    { 0xc63ed733, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x821ed736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ded73c, SUC_lexicon_tags_68 },
    { 0xaa1ed73d, SUC_lexicon_tags_65 },
    { 0x66fed73d, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0x5cbed740, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bed744, SUC_lexicon_tags_36 },
    { 0xb5ded745, SUC_lexicon_tags_95 },
    { 0xd0ded746, SUC_lexicon_tags_46 },
    { 0x293ed747, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847ed74d, SUC_lexicon_tags_88 },
    { 0x55fed74d, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0x23ed750, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377ed754, SUC_lexicon_tags_14 },
    { 0x975ed755, SUC_lexicon_tags_7 },
    { 0x783ed755, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xbabed758, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3ed75e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ded766, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661ed76a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3ed76f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xb27ed771, SUC_lexicon_tags_7 },
    { 0xba5ed771, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbed775, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81ed779, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf37ed77b, SUC_lexicon_tags_14 },
    { 0xd5ded77b, SUC_lexicon_tags_28 },
    { 0xa51ed77d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81ed780, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7ed785, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb29ed787, SUC_lexicon_tags_32 },
    { 0xed9ed787, SUC_lexicon_tags_48 },
    { 0x907ed787, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ded78c, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99ed78f, SUC_lexicon_tags_12 },
    { 0x4cbed790, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25ed799, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ed79c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3ed79f, SUC_lexicon_tags_57 },
    { 0x585ed7a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03ed7a5, SUC_lexicon_tags_15 },
    { 0x15bed7a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555ed7ab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x801ed7ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x335ed7af, SUC_lexicon_tags_46 },
    { 0x289ed7af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd8bed7b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x239ed7b5, SUC_lexicon_tags_7 },
    { 0x2fbed7b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x44fed7b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6bfed7bb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7ed7c0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x495ed7c2, SUC_lexicon_tags_37 },
    { 0xbe1ed7c2, SUC_lexicon_tags_15 },
    { 0xd09ed7c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbed7c7, SUC_lexicon_tags_12 },
    { 0xe51ed7c8, SUC_lexicon_tags_142 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfed7cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa67ed7d2, SUC_lexicon_tags_25 },
    { 0x7eded7d2, SUC_lexicon_tags_15 },
    { 0x199ed7d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x12fed7d7, SUC_lexicon_tags_7 },
    { 0x1f9ed7d8, SUC_lexicon_tags_60 },
    { 0xbabed7d8, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0x4d9ed7db, SUC_lexicon_tags_34 },
    { 0xae9ed7db, SUC_lexicon_tags_28 },
    { 0xf93ed7dd, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59ed7e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x41ed7e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafded7ea, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcb7ed7ec, SUC_lexicon_tags_28 },
    { 0xe5ded7ed, SUC_lexicon_tags_68 },
    { 0xa05ed7ed, SUC_lexicon_tags_8 },
    { 0x76ded7ef, SUC_lexicon_tags_15 },
    { 0xd41ed7ed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x695ed7f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x891ed7f4, SUC_lexicon_tags_81 },
    { 0xd7ed7f5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x69ed7f7, SUC_lexicon_tags_45 },
    { 0x8bfed7f8, SUC_lexicon_tags_15 },
    { 0xf91ed7f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfed7fe, SUC_lexicon_tags_7 },
    { 0x5c1ed7ff, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe99ed801, SUC_lexicon_tags_28 },
    { 0x615ed802, SUC_lexicon_tags_12 },
    { 0x9e7ed802, SUC_lexicon_tags_21 },
    { 0xd9ed803, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffed80b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26fed80f, SUC_lexicon_tags_45 },
    { 0x2d7ed80f, SUC_lexicon_tags_77 },
    { 0xfd5ed80f, SUC_lexicon_tags_2 },
    { 0x7c1ed812, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x939ed814, SUC_lexicon_tags_9 },
    { 0x849ed815, SUC_lexicon_tags_8 },
    { 0x32bed816, SUC_lexicon_tags_3 },
    { 0x59ed816, SUC_lexicon_tags_49 },
    { 0x623ed817, SUC_lexicon_tags_25 },
    { 0xfdfed816, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1a5ed81b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xea5ed81d, SUC_lexicon_tags_1 },
    { 0x79ded81d, SUC_lexicon_tags_7 },
    { 0xa99ed81d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cded824, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x329ed826, SUC_lexicon_tags_38 },
    { 0xa4bed827, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a9ed829, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5ed82d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3ed832, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1ed837, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x893ed841, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd15ed843, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ded848, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bed84b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x151ed84e, SUC_lexicon_tags_37 },
    { 0x3ed84f, SUC_lexicon_tags_25 },
    { 0x97ded850, SUC_lexicon_tags_87 },
    { 0x1a5ed84f, SUC_lexicon_tags_15 },
    { 0xfc3ed852, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bed855, SUC_lexicon_tags_63 },
    { 0xfebed856, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47ed85d, SUC_lexicon_tags_49 },
    { 0x2c1ed85e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57fed862, SUC_lexicon_tags_12 },
    { 0x311ed863, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fed869, SUC_lexicon_tags_23 },
    { 0xde9ed869, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9ed86e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115ed872, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1ed876, SUC_lexicon_tags_15 },
    { 0x743ed877, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x975ed87a, SUC_lexicon_tags_141 },
    { 0x4f3ed87b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ded87e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97ed881, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bed889, SUC_lexicon_tags_10 },
    { 0x8abed88a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7ed88d, SUC_lexicon_tags_17 },
    { 0xcc9ed88e, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0xfa1ed890, SUC_lexicon_tags_7 },
    { 0x5ded891, SUC_lexicon_tags_65 },
    { 0x45ded892, SUC_lexicon_tags_23 },
    { 0x203ed891, SUC_lexicon_tags_37 },
    { 0x46bed891, SUC_lexicon_tags_17 },
    { 0x6fded895, SUC_lexicon_tags_27 },
    { 0xa69ed896, SUC_lexicon_tags_0 },
    { 0x83bed897, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1d3ed899, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbed89c, SUC_lexicon_tags_2 },
    { 0x1c3ed89d, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7ed8a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbbed8a3, SUC_lexicon_tags_1 },
    { 0xc97ed8a3, SUC_lexicon_tags_20 },
    { 0xfc1ed8a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd39ed8a7, SUC_lexicon_tags_27 },
    { 0x67bed8a7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ded8ad, SUC_lexicon_tags_15 },
    { 0xd93ed8ae, SUC_lexicon_tags_82 },
    { 0x51fed8ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad7ed8b1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85ed8b5, SUC_lexicon_tags_12 },
    { 0xa07ed8b6, SUC_lexicon_tags_29 },
    { 0x491ed8b7, SUC_lexicon_tags_13 },
    { 0x251ed8b7, SUC_lexicon_tags_5 },
    { 0x391ed8b5, SUC_lexicon_tags_17 },
    { 0x2a7ed8ba, SUC_lexicon_tags_0 },
    { 0x6e9ed8bb, SUC_lexicon_tags_66 },
    { 0x873ed8bb, SUC_lexicon_tags_37 },
    { 0xb8bed8bc, SUC_lexicon_tags_25 },
    { 0x813ed8bd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643ed8c2, SUC_lexicon_tags_66 },
    { 0x647ed8c3, SUC_lexicon_tags_2 },
    { 0x1ebed8c4, SUC_lexicon_tags_39 },
    { 0xe9ded8c5, SUC_lexicon_tags_37 },
    { 0x39ded8c3, SUC_lexicon_tags_87 },
    { 0xa1fed8c7, SUC_lexicon_tags_49 },
    { 0xb63ed8c3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x5efed8cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x261ed8ce, SUC_lexicon_tags_16 },
    { 0xc4bed8cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa59ed8d1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7ed8d5, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ed8d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09ed8e0, SUC_lexicon_tags_12 },
    { 0x51bed8e0, SUC_lexicon_tags_9 },
    { 0xd57ed8e1, SUC_lexicon_tags_29 },
    { 0x8afed8e3, SUC_lexicon_tags_28 },
    { 0x74ded8e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1ed8ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7a5ed8ee, SUC_lexicon_tags_25 },
    { 0xab7ed8ef, SUC_lexicon_tags_2 },
    { 0x147ed8f0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x201ed8f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa55ed8f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1ed8f9, SUC_lexicon_tags_27 },
    { 0x451ed8f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe5fed8fc, SUC_lexicon_tags_43 },
    { 0x4ded8fd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ed903, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf49ed905, SUC_lexicon_tags_11 },
    { 0xbc7ed906, SUC_lexicon_tags_25 },
    { 0x555ed907, SUC_lexicon_tags_15 },
    { 0x557ed908, SUC_lexicon_tags_120 },
    { 0x859ed908, SUC_lexicon_tags_1 },
    { 0xcc5ed908, SUC_lexicon_tags_49 },
    { 0xb63ed907, SUC_lexicon_tags_16 },
    { 0x30bed90a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x7fbed90e, SUC_lexicon_tags_15 },
    { 0x953ed90e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf75ed911, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4fbed913, SUC_lexicon_tags_0 },
    { 0x145ed914, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ed917, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbed91a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xcf1ed91c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169ed920, SUC_lexicon_tags_27 },
    { 0xc37ed920, SUC_lexicon_tags_21 },
    { 0x35fed922, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ded92a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25ed92d, SUC_lexicon_tags_46 },
    { 0x343ed92d, SUC_lexicon_tags_7 },
    { 0xcb3ed92f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1aded932, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719ed936, SUC_lexicon_tags_25 },
    { 0x757ed937, SUC_lexicon_tags_33 },
    { 0x7dded938, SUC_lexicon_tags_11 },
    { 0xc1ded938, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xad5ed93b, SUC_lexicon_tags_30 },
    { 0x9abed93c, SUC_lexicon_tags_37 },
    { 0x3c7ed93c, SUC_lexicon_tags_11 },
    { 0x3a7ed93c, SUC_lexicon_tags_613 },
    { 0x15bed93f, SUC_lexicon_tags_12 },
    { 0x837ed940, SUC_lexicon_tags_84 },
    { 0x677ed93b, SUC_lexicon_tags_1 },
    { 0xa63ed942, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbed945, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x211ed947, SUC_lexicon_tags_45 },
    { 0x74bed948, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe7fed94a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7ed94e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf75ed950, SUC_lexicon_tags_35 },
    { 0xd6bed951, SUC_lexicon_tags_17 },
    { 0xac1ed951, SUC_lexicon_tags_3 },
    { 0x651ed951, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa9bed955, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57fed957, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbed95a, SUC_lexicon_tags_7 },
    { 0xd35ed95a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19bed961, SUC_lexicon_tags_14 },
    { 0xe97ed961, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8dded964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fded968, SUC_lexicon_tags_1 },
    { 0x9dfed969, SUC_lexicon_tags_15 },
    { 0x313ed96a, SUC_lexicon_tags_29 },
    { 0x5cbed96b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443ed96f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ded973, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xee1ed975, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ded978, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x465ed97a, SUC_lexicon_tags_21 },
    { 0xb9fed97b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf3ed97d, SUC_lexicon_tags_33 },
    { 0x63fed97d, SUC_lexicon_tags_6 },
    { 0x5e1ed97f, SUC_lexicon_tags_19 },
    { 0x77ded97e, SUC_lexicon_tags_17 },
    { 0xb6ded981, SUC_lexicon_tags_7 },
    { 0x1b1ed982, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bed984, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5ed988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17ed98e, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91ed992, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41ed999, SUC_lexicon_tags_17 },
    { 0x395ed99a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2fed99e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfabed9a0, SUC_lexicon_tags_37 },
    { 0x3cfed9a1, SUC_lexicon_tags_17 },
    { 0x7a9ed9a1, SUC_lexicon_tags_31 },
    { 0x7bbed9a1, SUC_lexicon_tags_47 },
    { 0x57fed9a2, SUC_lexicon_tags_17 },
    { 0x859ed9a2, SUC_lexicon_tags_15 },
    { 0xdd7ed9a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x439ed9a8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535ed9ae, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637ed9b5, SUC_lexicon_tags_12 },
    { 0x52bed9b6, SUC_lexicon_tags_30 },
    { 0x3c5ed9b5, SUC_lexicon_tags_151 },
    { 0x22fed9b6, SUC_lexicon_tags_106 },
    { 0xffed9b6, SUC_lexicon_tags_45 },
    { 0x17fed9ba, SUC_lexicon_tags_11 },
    { 0xa03ed9bb, SUC_lexicon_tags_60 },
    { 0x411ed9b7, SUC_lexicon_tags_30 },
    { 0x587ed9b7, SUC_lexicon_tags_46 },
    { 0x213ed9bd, SUC_lexicon_tags_28 },
    { 0x233ed9bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ded9c8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9afed9ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x901ed9d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xec3ed9d3, SUC_lexicon_tags_29 },
    { 0x25ed9d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb3ed9d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5ed9d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb6ded9db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7ed9e0, SUC_lexicon_tags_7 },
    { 0xe39ed9e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfed9e8, SUC_lexicon_tags_17 },
    { 0x255ed9e9, SUC_lexicon_tags_7 },
    { 0x2f7ed9e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd7fed9ec, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xc8ded9ee, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ded9f2, SUC_lexicon_tags_107 },
    { 0x967ed9f2, SUC_lexicon_tags_599 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9ed9f7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x66bed9f9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3ed9fc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfe3ed9fe, SUC_lexicon_tags_23 },
    { 0x89ed9ff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295eda03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe95eda05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x175eda0d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823eda11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x355eda14, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x54feda16, SUC_lexicon_tags_33 },
    { 0xed5eda16, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61feda1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525eda20, SUC_lexicon_tags_9 },
    { 0x33eda20, SUC_lexicon_tags_522 },
    { 0x307eda22, SUC_lexicon_tags_47 },
    { 0x8feda22, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29deda28, SUC_lexicon_tags_85 },
    { 0xb97eda28, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7eda2d, SUC_lexicon_tags_23 },
    { 0x65feda2e, SUC_lexicon_tags_17 },
    { 0x513eda2f, SUC_lexicon_tags_33 },
    { 0xc4beda2f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3eda35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21eda38, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5fbeda3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xededa3c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdeda41, SUC_lexicon_tags_26 },
    { 0x3ebeda41, SUC_lexicon_tags_60 },
    { 0xa3beda42, SUC_lexicon_tags_33 },
    { 0x3c1eda41, SUC_lexicon_tags_37 },
    { 0x465eda43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3deda48, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7eda4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1eda4f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5eda54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9eda58, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903eda5e, SUC_lexicon_tags_30 },
    { 0x3e3eda5f, SUC_lexicon_tags_30 },
    { 0x10feda60, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x695eda63, SUC_lexicon_tags_13 },
    { 0xe25eda64, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3eda67, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247eda6b, SUC_lexicon_tags_21 },
    { 0x59eda6c, SUC_lexicon_tags_17 },
    { 0x443eda6d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdeda70, SUC_lexicon_tags_49 },
    { 0x9bdeda71, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95eda7b, SUC_lexicon_tags_39 },
    { 0xc77eda7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3eda7f, SUC_lexicon_tags_0 },
    { 0xe1beda7f, SUC_lexicon_tags_2 },
    { 0xdfdeda80, SUC_lexicon_tags_184 },
    { 0, NULL },
    { 0x4c1eda83, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811eda87, SUC_lexicon_tags_11 },
    { 0x7b3eda87, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3b9eda8a, SUC_lexicon_tags_60 },
    { 0x527eda8a, SUC_lexicon_tags_14 },
    { 0x2ebeda8c, SUC_lexicon_tags_25 },
    { 0x2dfeda8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e5eda8f, SUC_lexicon_tags_6 },
    { 0x2abeda8f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9feda92, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279eda97, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5eda9f, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdedaa5, SUC_lexicon_tags_1 },
    { 0xefedaa5, SUC_lexicon_tags_1 },
    { 0x883edaa7, SUC_lexicon_tags_8 },
    { 0x8f7edaa8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35edaab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9edab0, SUC_lexicon_tags_63 },
    { 0x6a1edab1, SUC_lexicon_tags_37 },
    { 0xc15edab1, SUC_lexicon_tags_37 },
    { 0x4cfedab2, SUC_lexicon_tags_11 },
    { 0xf99edab4, SUC_lexicon_tags_17 },
    { 0xc77edab5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa13edab7, SUC_lexicon_tags_1 },
    { 0xcd5edab8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdc5edaba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485edac6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9edaca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbedad4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bedad9, SUC_lexicon_tags_37 },
    { 0x9a9edada, SUC_lexicon_tags_23 },
    { 0xec5edad9, SUC_lexicon_tags_7 },
    { 0xd0fedadc, SUC_lexicon_tags_15 },
    { 0x25fedadd, SUC_lexicon_tags_91 },
    { 0x2adedade, SUC_lexicon_tags_20 },
    { 0xaf3edadf, SUC_lexicon_tags_37 },
    { 0x937edae0, SUC_lexicon_tags_32 },
    { 0x33dedae0, SUC_lexicon_tags_35 },
    { 0x42fedadf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dedae6, SUC_lexicon_tags_1 },
    { 0x4c3edae7, SUC_lexicon_tags_33 },
    { 0x135edae8, SUC_lexicon_tags_37 },
    { 0x85edae7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6afedaeb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xebdedaed, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd21edaef, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465edaf6, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x85edaf8, SUC_lexicon_tags_38 },
    { 0x29bedaf9, SUC_lexicon_tags_14 },
    { 0x2cbedafa, SUC_lexicon_tags_25 },
    { 0x533edafb, SUC_lexicon_tags_39 },
    { 0xba1edaf8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbedb01, SUC_lexicon_tags_28 },
    { 0xcafedb02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1edb08, SUC_lexicon_tags_33 },
    { 0x13fedb08, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x751edb0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe9edb0d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bedb13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x797edb15, SUC_lexicon_tags_61 },
    { 0x3f5edb16, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bedb19, SUC_lexicon_tags_12 },
    { 0x5cdedb1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedb1f, SUC_lexicon_tags_2 },
    { 0x781edb20, SUC_lexicon_tags_25 },
    { 0x783edb21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93edb24, SUC_lexicon_tags_1 },
    { 0xef5edb24, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x13dedb28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49edb2b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fedb34, SUC_lexicon_tags_27 },
    { 0xfadedb34, SUC_lexicon_tags_12 },
    { 0x59dedb36, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfedb3a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x987edb3d, SUC_lexicon_tags_63 },
    { 0x7a1edb3e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x263edb40, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x397edb42, SUC_lexicon_tags_18 },
    { 0x8b7edb43, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9edb46, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33edb4b, SUC_lexicon_tags_179 },
    { 0x2f9edb4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813edb52, SUC_lexicon_tags_1 },
    { 0xbdbedb53, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x107edb56, SUC_lexicon_tags_33 },
    { 0x297edb56, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7afedb5e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dedb62, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0fedb66, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x555edb69, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x15fedb6b, SUC_lexicon_tags_25 },
    { 0x2fdedb6b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40dedb71, SUC_lexicon_tags_15 },
    { 0x751edb72, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xee3edb74, SUC_lexicon_tags_10 },
    { 0x603edb75, SUC_lexicon_tags_11 },
    { 0xa4fedb74, SUC_lexicon_tags_87 },
    { 0x565edb77, SUC_lexicon_tags_37 },
    { 0x5ededb77, SUC_lexicon_tags_25 },
    { 0xf3bedb77, SUC_lexicon_tags_15 },
    { 0x1d3edb79, SUC_lexicon_tags_10 },
    { 0xd7fedb7b, SUC_lexicon_tags_65 },
    { 0x7f3edb7c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5cfedb7e, SUC_lexicon_tags_25 },
    { 0xe21edb7f, SUC_lexicon_tags_12 },
    { 0xe79edb7e, SUC_lexicon_tags_1532 },
    { 0, NULL },
    { 0x53bedb82, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47edb86, SUC_lexicon_tags_7 },
    { 0xa21edb87, SUC_lexicon_tags_27 },
    { 0xde7edb88, SUC_lexicon_tags_17 },
    { 0xc4dedb89, SUC_lexicon_tags_87 },
    { 0x7b7edb8a, SUC_lexicon_tags_11 },
    { 0x9f5edb8a, SUC_lexicon_tags_23 },
    { 0x835edb8a, SUC_lexicon_tags_60 },
    { 0xd37edb8a, SUC_lexicon_tags_1 },
    { 0x30dedb87, SUC_lexicon_tags_14 },
    { 0xe7edb8d, SUC_lexicon_tags_63 },
    { 0xda5edb90, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39edb93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x855edb96, SUC_lexicon_tags_6 },
    { 0xe61edb96, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf87edb99, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb59edb9b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bedb9f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fedba2, SUC_lexicon_tags_17 },
    { 0xf89edba2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x5a7edba5, SUC_lexicon_tags_45 },
    { 0x605edba6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9edbab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939edbb0, SUC_lexicon_tags_12 },
    { 0xa0dedbb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd5edbb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfedbbb, SUC_lexicon_tags_12 },
    { 0x191edbbb, SUC_lexicon_tags_10 },
    { 0x625edbbc, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xc79edbbf, SUC_lexicon_tags_63 },
    { 0xe39edbc0, SUC_lexicon_tags_15 },
    { 0xcafedbc1, SUC_lexicon_tags_14 },
    { 0x3c3edbc2, SUC_lexicon_tags_47 },
    { 0xe1dedbc3, SUC_lexicon_tags_31 },
    { 0xaa9edbc2, SUC_lexicon_tags_7 },
    { 0xbf3edbc5, SUC_lexicon_tags_14 },
    { 0xb09edbc6, SUC_lexicon_tags_60 },
    { 0x19dedbc7, SUC_lexicon_tags_17 },
    { 0x5d9edbc4, SUC_lexicon_tags_6 },
    { 0xbfbedbc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x18fedbcb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dedbd0, SUC_lexicon_tags_1 },
    { 0xc89edbd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdedbd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddedbdc, SUC_lexicon_tags_3 },
    { 0xef5edbdd, SUC_lexicon_tags_14 },
    { 0x373edbde, SUC_lexicon_tags_55 },
    { 0x9d7edbde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bedbe3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3edbe7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fedbea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3edbef, SUC_lexicon_tags_47 },
    { 0xeabedbef, SUC_lexicon_tags_26 },
    { 0x5dbedbf1, SUC_lexicon_tags_17 },
    { 0xff5edbf2, SUC_lexicon_tags_10 },
    { 0x395edbf1, SUC_lexicon_tags_108 },
    { 0x563edbf1, SUC_lexicon_tags_25 },
    { 0x449edbf1, SUC_lexicon_tags_11 },
    { 0xb69edbf6, SUC_lexicon_tags_49 },
    { 0xa51edbf7, SUC_lexicon_tags_11 },
    { 0xfe5edbf8, SUC_lexicon_tags_29 },
    { 0x7f3edbf4, SUC_lexicon_tags_25 },
    { 0xdb9edbf8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x317edbfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12dedbff, SUC_lexicon_tags_2 },
    { 0x2efedbff, SUC_lexicon_tags_17 },
    { 0xa63edc01, SUC_lexicon_tags_2 },
    { 0x7d1edbff, SUC_lexicon_tags_11 },
    { 0x587edbff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7edc07, SUC_lexicon_tags_87 },
    { 0xf4fedc08, SUC_lexicon_tags_30 },
    { 0x56dedc09, SUC_lexicon_tags_43 },
    { 0xe21edc09, SUC_lexicon_tags_16 },
    { 0x1fdedc0b, SUC_lexicon_tags_27 },
    { 0xffdedc0b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5edc0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3d3edc11, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd6dedc13, SUC_lexicon_tags_32 },
    { 0x20bedc14, SUC_lexicon_tags_45 },
    { 0x3edc15, SUC_lexicon_tags_25 },
    { 0x91edc15, SUC_lexicon_tags_0 },
    { 0xbffedc17, SUC_lexicon_tags_9 },
    { 0x707edc13, SUC_lexicon_tags_4 },
    { 0x629edc17, SUC_lexicon_tags_40 },
    { 0xab3edc1a, SUC_lexicon_tags_38 },
    { 0xbf5edc1b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x34fedc1e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadedc21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53edc24, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75edc28, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93edc2e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbe1edc30, SUC_lexicon_tags_47 },
    { 0x51fedc30, SUC_lexicon_tags_10 },
    { 0xff7edc32, SUC_lexicon_tags_37 },
    { 0x9fedc33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdedc3a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19fedc3f, SUC_lexicon_tags_16 },
    { 0xeffedc40, SUC_lexicon_tags_1 },
    { 0xe0fedc41, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x247edc43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x10bedc45, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85edc48, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x799edc4b, SUC_lexicon_tags_17 },
    { 0x363edc4b, SUC_lexicon_tags_47 },
    { 0x589edc4d, SUC_lexicon_tags_7 },
    { 0x8f9edc4b, SUC_lexicon_tags_17 },
    { 0xe51edc4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fedc52, SUC_lexicon_tags_12 },
    { 0xb19edc53, SUC_lexicon_tags_46 },
    { 0xf61edc54, SUC_lexicon_tags_106 },
    { 0xc55edc52, SUC_lexicon_tags_47 },
    { 0xa21edc56, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x489edc58, SUC_lexicon_tags_25 },
    { 0x9bdedc58, SUC_lexicon_tags_61 },
    { 0x5bbedc5a, SUC_lexicon_tags_2 },
    { 0x76bedc5b, SUC_lexicon_tags_14 },
    { 0x821edc58, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd77edc5e, SUC_lexicon_tags_11 },
    { 0x533edc5f, SUC_lexicon_tags_25 },
    { 0xa7fedc60, SUC_lexicon_tags_12 },
    { 0xf9bedc61, SUC_lexicon_tags_8 },
    { 0xb97edc62, SUC_lexicon_tags_19 },
    { 0x4ededc5f, SUC_lexicon_tags_56 },
    { 0xe33edc64, SUC_lexicon_tags_38 },
    { 0xcbfedc64, SUC_lexicon_tags_7 },
    { 0x5a1edc66, SUC_lexicon_tags_0 },
    { 0xc77edc60, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x861edc6a, SUC_lexicon_tags_87 },
    { 0x307edc6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dedc71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47fedc76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeadedc79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x96dedc7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee3edc7e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdedc80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdd5edc82, SUC_lexicon_tags_43 },
    { 0x331edc82, SUC_lexicon_tags_2 },
    { 0x8afedc84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccdedc86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaa9edc88, SUC_lexicon_tags_11 },
    { 0xcf5edc88, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x763edc8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85edc91, SUC_lexicon_tags_30 },
    { 0x61edc91, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807edc97, SUC_lexicon_tags_17 },
    { 0x149edc98, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xcbbedc9a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe99edc9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69edca3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63edca7, SUC_lexicon_tags_49 },
    { 0xf6dedca8, SUC_lexicon_tags_12 },
    { 0x99bedca7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc89edcab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfedcb1, SUC_lexicon_tags_14 },
    { 0xd17edcb1, SUC_lexicon_tags_11 },
    { 0x1cbedcb1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfddedcb5, SUC_lexicon_tags_21 },
    { 0x581edcb6, SUC_lexicon_tags_1 },
    { 0xaadedcb6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fedcbb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x363edcbd, SUC_lexicon_tags_37 },
    { 0x24dedcbe, SUC_lexicon_tags_15 },
    { 0xe99edcbf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x841edcc1, SUC_lexicon_tags_17 },
    { 0x1edcc2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179edcc9, SUC_lexicon_tags_33 },
    { 0xd07edcca, SUC_lexicon_tags_27 },
    { 0xf97edccb, SUC_lexicon_tags_1 },
    { 0x457edcca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6fedcd2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xbd9edcd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfb9edcd6, SUC_lexicon_tags_77 },
    { 0x793edcd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf85edcd9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7edcdd, SUC_lexicon_tags_15 },
    { 0xc3fedcde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53dedce2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x49bedce4, SUC_lexicon_tags_7 },
    { 0xfefedce5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x967edce7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3edcee, SUC_lexicon_tags_35 },
    { 0x853edcef, SUC_lexicon_tags_12 },
    { 0x415edcef, SUC_lexicon_tags_60 },
    { 0x5bfedcf1, SUC_lexicon_tags_11 },
    { 0x2efedcf2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dedcfb, SUC_lexicon_tags_37 },
    { 0x953edcfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1edd03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dedd09, SUC_lexicon_tags_12 },
    { 0xdc3edd09, SUC_lexicon_tags_49 },
    { 0xdd1edd0b, SUC_lexicon_tags_37 },
    { 0x853edd09, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fedd10, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x93edd13, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dedd19, SUC_lexicon_tags_15 },
    { 0xfffedd19, SUC_lexicon_tags_0 },
    { 0xa37edd1b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45edd22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dedd26, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x505edd28, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419edd2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffedd32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bedd39, SUC_lexicon_tags_0 },
    { 0xd29edd39, SUC_lexicon_tags_25 },
    { 0xcfbedd3b, SUC_lexicon_tags_11 },
    { 0x3ebedd3a, SUC_lexicon_tags_29 },
    { 0x56bedd3c, SUC_lexicon_tags_30 },
    { 0x69edd3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997edd42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbfdedd44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc5edd46, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa07edd49, SUC_lexicon_tags_11 },
    { 0xebbedd4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x915edd4c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77edd50, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xde3edd52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233edd56, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x83dedd58, SUC_lexicon_tags_28 },
    { 0x661edd58, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd31edd5d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5edd66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63dedd6a, SUC_lexicon_tags_37 },
    { 0x5b7edd6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3b7edd6d, SUC_lexicon_tags_53 },
    { 0x193edd6e, SUC_lexicon_tags_63 },
    { 0xc97edd6f, SUC_lexicon_tags_27 },
    { 0xfe1edd70, SUC_lexicon_tags_63 },
    { 0xf09edd70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607edd78, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x381edd7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fedd7e, SUC_lexicon_tags_68 },
    { 0xb05edd7e, SUC_lexicon_tags_11 },
    { 0x29bedd80, SUC_lexicon_tags_47 },
    { 0x2d3edd7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdedd84, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9edd87, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efedd8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfedd8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x21fedd91, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5edd94, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffedd97, SUC_lexicon_tags_11 },
    { 0xc09edd97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x953edd9a, SUC_lexicon_tags_11 },
    { 0x5d9edd9a, SUC_lexicon_tags_34 },
    { 0x4fedd9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x163edd9f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xeededda2, SUC_lexicon_tags_15 },
    { 0xa47edda3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707edda7, SUC_lexicon_tags_37 },
    { 0x2dbedda8, SUC_lexicon_tags_16 },
    { 0xf49edda7, SUC_lexicon_tags_37 },
    { 0xc4bedda7, SUC_lexicon_tags_43 },
    { 0xf65edda7, SUC_lexicon_tags_25 },
    { 0xd27eddac, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0x75eddaf, SUC_lexicon_tags_34 },
    { 0x735eddaf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1eddba, SUC_lexicon_tags_23 },
    { 0x8efeddbb, SUC_lexicon_tags_29 },
    { 0x1bdeddba, SUC_lexicon_tags_1 },
    { 0x455eddbb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907eddc7, SUC_lexicon_tags_19 },
    { 0xa5beddc8, SUC_lexicon_tags_0 },
    { 0xfc9eddc9, SUC_lexicon_tags_11 },
    { 0x20feddc9, SUC_lexicon_tags_45 },
    { 0x4beddc8, SUC_lexicon_tags_7 },
    { 0xf1feddcc, SUC_lexicon_tags_12 },
    { 0x7ebeddcc, SUC_lexicon_tags_49 },
    { 0x5d7eddc9, SUC_lexicon_tags_0 },
    { 0x465eddcd, SUC_lexicon_tags_19 },
    { 0x3c1eddc9, SUC_lexicon_tags_57 },
    { 0x4b5eddce, SUC_lexicon_tags_17 },
    { 0x8beddd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x9d9eddd4, SUC_lexicon_tags_25 },
    { 0x217eddd4, SUC_lexicon_tags_0 },
    { 0x309eddd6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9eddd9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517edddf, SUC_lexicon_tags_26 },
    { 0x8a1edde0, SUC_lexicon_tags_65 },
    { 0xc25edde0, SUC_lexicon_tags_49 },
    { 0x523edde2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723edde9, SUC_lexicon_tags_17 },
    { 0x9f7edde9, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0xd43eddec, SUC_lexicon_tags_28 },
    { 0x901edded, SUC_lexicon_tags_31 },
    { 0x3f9eddee, SUC_lexicon_tags_1 },
    { 0x85feddec, SUC_lexicon_tags_116 },
    { 0xa6feddf0, SUC_lexicon_tags_10 },
    { 0x641eddf0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x721eddf3, SUC_lexicon_tags_27 },
    { 0x759eddf3, SUC_lexicon_tags_1 },
    { 0xef5eddf4, SUC_lexicon_tags_113 },
    { 0x119eddf6, SUC_lexicon_tags_8 },
    { 0x431eddf7, SUC_lexicon_tags_11 },
    { 0x9d3eddf8, SUC_lexicon_tags_12 },
    { 0x9afeddf9, SUC_lexicon_tags_79 },
    { 0xbdbeddfa, SUC_lexicon_tags_1 },
    { 0x291eddfa, SUC_lexicon_tags_17 },
    { 0x89eddfa, SUC_lexicon_tags_11 },
    { 0x957eddf7, SUC_lexicon_tags_37 },
    { 0x995eddfa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdede04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x67dede07, SUC_lexicon_tags_37 },
    { 0x5bfede08, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fede0e, SUC_lexicon_tags_0 },
    { 0x163ede0f, SUC_lexicon_tags_26 },
    { 0x3f1ede0f, SUC_lexicon_tags_47 },
    { 0xce7ede11, SUC_lexicon_tags_84 },
    { 0x859ede12, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07ede15, SUC_lexicon_tags_19 },
    { 0xd0dede16, SUC_lexicon_tags_15 },
    { 0x3ffede17, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0fede1a, SUC_lexicon_tags_23 },
    { 0xb2fede1a, SUC_lexicon_tags_0 },
    { 0x1d3ede1c, SUC_lexicon_tags_46 },
    { 0xaa7ede1d, SUC_lexicon_tags_33 },
    { 0xfdbede1e, SUC_lexicon_tags_7 },
    { 0x7bfede1e, SUC_lexicon_tags_14 },
    { 0xcabede1c, SUC_lexicon_tags_8 },
    { 0xfbbede1b, SUC_lexicon_tags_19 },
    { 0xd71ede1e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52fede26, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41ede2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6adede36, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4bbede38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10bede3c, SUC_lexicon_tags_37 },
    { 0xb07ede3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddede4d, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361ede51, SUC_lexicon_tags_47 },
    { 0xd4fede51, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddede59, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bede5d, SUC_lexicon_tags_27 },
    { 0x89bede5e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83ede61, SUC_lexicon_tags_12 },
    { 0xd0dede61, SUC_lexicon_tags_25 },
    { 0x245ede63, SUC_lexicon_tags_2 },
    { 0x4b7ede64, SUC_lexicon_tags_4 },
    { 0x829ede65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x16dede68, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1c7ede6a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7ede71, SUC_lexicon_tags_70 },
    { 0x6c9ede72, SUC_lexicon_tags_34 },
    { 0xe53ede71, SUC_lexicon_tags_80 },
    { 0xa11ede74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bede77, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x83fede79, SUC_lexicon_tags_28 },
    { 0x9dfede7a, SUC_lexicon_tags_11 },
    { 0xc61ede7b, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bede7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5ede81, SUC_lexicon_tags_29 },
    { 0xf13ede82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x293ede85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5ede88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc6fede8a, SUC_lexicon_tags_12 },
    { 0x68bede8b, SUC_lexicon_tags_25 },
    { 0x3ddede8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x773ede8f, SUC_lexicon_tags_7 },
    { 0xa37ede8f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953ede9b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdf1ede9d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bedea9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x627edeab, SUC_lexicon_tags_12 },
    { 0xf4fedeab, SUC_lexicon_tags_25 },
    { 0x20dedead, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31edeb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2d7edeb4, SUC_lexicon_tags_1 },
    { 0x607edeb5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfedebf, SUC_lexicon_tags_2 },
    { 0x99dedebf, SUC_lexicon_tags_15 },
    { 0x717edec1, SUC_lexicon_tags_4 },
    { 0xea7edec2, SUC_lexicon_tags_2 },
    { 0x785edec1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99edec7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bedecc, SUC_lexicon_tags_7 },
    { 0x16dedecd, SUC_lexicon_tags_607 },
    { 0xd77edecd, SUC_lexicon_tags_43 },
    { 0xfb1edece, SUC_lexicon_tags_31 },
    { 0x95fedece, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71eded3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adeded7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3ededa, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17dedede, SUC_lexicon_tags_28 },
    { 0xa09edede, SUC_lexicon_tags_49 },
    { 0xc63edede, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931edee7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0x20dedeea, SUC_lexicon_tags_12 },
    { 0xcb3edeeb, SUC_lexicon_tags_8 },
    { 0x2ddedeec, SUC_lexicon_tags_66 },
    { 0x51fedeeb, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9edef1, SUC_lexicon_tags_8 },
    { 0x8bfedef2, SUC_lexicon_tags_44 },
    { 0x281edef2, SUC_lexicon_tags_1 },
    { 0xdd3edef4, SUC_lexicon_tags_43 },
    { 0x585edef1, SUC_lexicon_tags_11 },
    { 0x12dedef4, SUC_lexicon_tags_12 },
    { 0x177edef7, SUC_lexicon_tags_1 },
    { 0x2d9edef8, SUC_lexicon_tags_8 },
    { 0x25dedef3, SUC_lexicon_tags_17 },
    { 0x267edefa, SUC_lexicon_tags_33 },
    { 0x9c9edefb, SUC_lexicon_tags_1 },
    { 0x269edef5, SUC_lexicon_tags_29 },
    { 0xd3edefd, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x691edf00, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36fedf04, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bedf11, SUC_lexicon_tags_0 },
    { 0xd95edf12, SUC_lexicon_tags_1 },
    { 0xacdedf13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3dedf18, SUC_lexicon_tags_65 },
    { 0xc3dedf19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9edf1c, SUC_lexicon_tags_48 },
    { 0x5fdedf1d, SUC_lexicon_tags_54 },
    { 0x5ebedf1d, SUC_lexicon_tags_39 },
    { 0xe5bedf1f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x74bedf21, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x625edf23, SUC_lexicon_tags_2 },
    { 0x155edf24, SUC_lexicon_tags_63 },
    { 0xc25edf25, SUC_lexicon_tags_4 },
    { 0xb81edf25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70fedf2a, SUC_lexicon_tags_17 },
    { 0x41dedf2b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd6fedf2d, SUC_lexicon_tags_8 },
    { 0x1a5edf2e, SUC_lexicon_tags_57 },
    { 0xafbedf2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4abedf32, SUC_lexicon_tags_11 },
    { 0x979edf33, SUC_lexicon_tags_37 },
    { 0xa0fedf34, SUC_lexicon_tags_16 },
    { 0x329edf34, SUC_lexicon_tags_19 },
    { 0x253edf36, SUC_lexicon_tags_11 },
    { 0x97bedf37, SUC_lexicon_tags_25 },
    { 0xaf3edf34, SUC_lexicon_tags_9 },
    { 0x397edf38, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1edf46, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1dedf48, SUC_lexicon_tags_26 },
    { 0x11dedf48, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323edf4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9edf50, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0x90fedf52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a5edf54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991edf5b, SUC_lexicon_tags_1 },
    { 0x677edf5c, SUC_lexicon_tags_1 },
    { 0x7a9edf5d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73edf68, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37edf6d, SUC_lexicon_tags_69 },
    { 0x971edf6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3edf77, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb81edf79, SUC_lexicon_tags_37 },
    { 0xeb5edf79, SUC_lexicon_tags_8 },
    { 0x17bedf79, SUC_lexicon_tags_23 },
    { 0x77bedf7c, SUC_lexicon_tags_37 },
    { 0x683edf7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x75fedf7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99edf85, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda9edf87, SUC_lexicon_tags_14 },
    { 0xff3edf88, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf83edf8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89edf8e, SUC_lexicon_tags_15 },
    { 0xaddedf8e, SUC_lexicon_tags_29 },
    { 0x8a3edf90, SUC_lexicon_tags_254 },
    { 0x47dedf91, SUC_lexicon_tags_47 },
    { 0x38fedf90, SUC_lexicon_tags_79 },
    { 0x693edf92, SUC_lexicon_tags_7 },
    { 0x47bedf90, SUC_lexicon_tags_25 },
    { 0x61bedf95, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8a3edf97, SUC_lexicon_tags_38 },
    { 0x6d3edf97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83bedf9d, SUC_lexicon_tags_68 },
    { 0x76bedf9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x923edfa0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x77dedfa3, SUC_lexicon_tags_17 },
    { 0x23edfa3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x965edfa6, SUC_lexicon_tags_23 },
    { 0xed7edfa7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebedfab, SUC_lexicon_tags_26 },
    { 0x39dedfac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3e5edfae, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddedfb2, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x749edfb4, SUC_lexicon_tags_63 },
    { 0xf25edfb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x775edfb8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x541edfba, SUC_lexicon_tags_48 },
    { 0xe7fedfbb, SUC_lexicon_tags_7 },
    { 0xff9edfbb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5edfc0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x44fedfc2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9edfc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585edfcb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fedfcf, SUC_lexicon_tags_37 },
    { 0x93bedfcf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x637edfd2, SUC_lexicon_tags_37 },
    { 0x781edfd3, SUC_lexicon_tags_25 },
    { 0xc95edfd3, SUC_lexicon_tags_25 },
    { 0x14dedfd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bedfdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559edfe1, SUC_lexicon_tags_42 },
    { 0x207edfe1, SUC_lexicon_tags_0 },
    { 0xac9edfe3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bedfe6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fedfed, SUC_lexicon_tags_38 },
    { 0x619edfee, SUC_lexicon_tags_16 },
    { 0x40bedfed, SUC_lexicon_tags_17 },
    { 0x635edfef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x441edff2, SUC_lexicon_tags_70 },
    { 0x575edff2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfedff7, SUC_lexicon_tags_26 },
    { 0x225edff8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1edffb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x249edffe, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25ee001, SUC_lexicon_tags_7 },
    { 0xc9dee002, SUC_lexicon_tags_27 },
    { 0x97ee002, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x93fee005, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4cdee007, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x699ee009, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x89ee00b, SUC_lexicon_tags_7 },
    { 0x181ee00c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab3ee00e, SUC_lexicon_tags_7 },
    { 0xa53ee00f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367ee013, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1ee016, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3ee019, SUC_lexicon_tags_37 },
    { 0xc9ee01a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbee01f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907ee025, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61ee02a, SUC_lexicon_tags_47 },
    { 0x665ee02b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x65ee02e, SUC_lexicon_tags_5 },
    { 0x68bee02e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x495ee031, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x6f3ee033, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bee03d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefee043, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3ee048, SUC_lexicon_tags_385 },
    { 0x687ee048, SUC_lexicon_tags_10 },
    { 0x7edee049, SUC_lexicon_tags_14 },
    { 0xba5ee049, SUC_lexicon_tags_9 },
    { 0x8c3ee048, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17ee050, SUC_lexicon_tags_17 },
    { 0xbe7ee051, SUC_lexicon_tags_38 },
    { 0xa9dee052, SUC_lexicon_tags_17 },
    { 0xbd5ee050, SUC_lexicon_tags_14 },
    { 0xb1ee054, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3ee059, SUC_lexicon_tags_1 },
    { 0x94fee059, SUC_lexicon_tags_12 },
    { 0xad1ee05b, SUC_lexicon_tags_7 },
    { 0x473ee05c, SUC_lexicon_tags_11 },
    { 0x3b3ee05d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ee063, SUC_lexicon_tags_48 },
    { 0x231ee063, SUC_lexicon_tags_6 },
    { 0x709ee064, SUC_lexicon_tags_47 },
    { 0xd8bee066, SUC_lexicon_tags_11 },
    { 0x151ee067, SUC_lexicon_tags_79 },
    { 0x257ee065, SUC_lexicon_tags_26 },
    { 0x575ee067, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fee06c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1ee06f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x94fee071, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea7ee073, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1ee076, SUC_lexicon_tags_79 },
    { 0xf29ee077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff7ee079, SUC_lexicon_tags_12 },
    { 0x60dee07a, SUC_lexicon_tags_33 },
    { 0x28bee079, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ee080, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbee083, SUC_lexicon_tags_88 },
    { 0x229ee083, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd21ee086, SUC_lexicon_tags_16 },
    { 0x5d7ee086, SUC_lexicon_tags_10 },
    { 0xf7ee088, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5dfee08a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd5dee08c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x801ee08e, SUC_lexicon_tags_39 },
    { 0x117ee08f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d1ee091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fee095, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3ee09f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bee0a3, SUC_lexicon_tags_33 },
    { 0x84dee0a4, SUC_lexicon_tags_11 },
    { 0xa5dee0a5, SUC_lexicon_tags_28 },
    { 0xc05ee0a6, SUC_lexicon_tags_11 },
    { 0xee5ee0a7, SUC_lexicon_tags_37 },
    { 0xa4fee0a8, SUC_lexicon_tags_38 },
    { 0x1cfee0a9, SUC_lexicon_tags_108 },
    { 0xfbee0a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fee0ae, SUC_lexicon_tags_37 },
    { 0x259ee0af, SUC_lexicon_tags_47 },
    { 0xf77ee0b0, SUC_lexicon_tags_15 },
    { 0x859ee0b1, SUC_lexicon_tags_8 },
    { 0x995ee0b0, SUC_lexicon_tags_0 },
    { 0xa3fee0af, SUC_lexicon_tags_23 },
    { 0xdcbee0b4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x24fee0b6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x76fee0b8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01ee0bb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bee0bf, SUC_lexicon_tags_0 },
    { 0x9bdee0c0, SUC_lexicon_tags_23 },
    { 0x45fee0c1, SUC_lexicon_tags_66 },
    { 0xa6dee0c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abee0c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279ee0cf, SUC_lexicon_tags_65 },
    { 0xd65ee0d0, SUC_lexicon_tags_37 },
    { 0xfc3ee0d1, SUC_lexicon_tags_11 },
    { 0xef7ee0d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67ee0d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25ee0d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x48bee0da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x769ee0dc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3ee0e3, SUC_lexicon_tags_14 },
    { 0x8afee0e3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85ee0ee, SUC_lexicon_tags_29 },
    { 0x8bee0ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xff7ee0f1, SUC_lexicon_tags_387 },
    { 0x1f9ee0f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afee0f9, SUC_lexicon_tags_12 },
    { 0xe9dee0fa, SUC_lexicon_tags_6 },
    { 0x6a5ee0fa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24bee100, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fee106, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x899ee108, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95ee10d, SUC_lexicon_tags_12 },
    { 0xecdee10e, SUC_lexicon_tags_11 },
    { 0x6bbee10f, SUC_lexicon_tags_12 },
    { 0x2d7ee110, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5ee113, SUC_lexicon_tags_11 },
    { 0x6e7ee114, SUC_lexicon_tags_60 },
    { 0x399ee115, SUC_lexicon_tags_7 },
    { 0x3c1ee116, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x843ee119, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bee11f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbee124, SUC_lexicon_tags_85 },
    { 0x7d1ee125, SUC_lexicon_tags_23 },
    { 0xc17ee126, SUC_lexicon_tags_17 },
    { 0x969ee127, SUC_lexicon_tags_1 },
    { 0x40bee128, SUC_lexicon_tags_2 },
    { 0xbbdee129, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x475ee12b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bee130, SUC_lexicon_tags_37 },
    { 0x7e9ee130, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfee134, SUC_lexicon_tags_45 },
    { 0xc89ee135, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x54bee138, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb41ee13a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9ee13c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3ee140, SUC_lexicon_tags_4 },
    { 0xda5ee141, SUC_lexicon_tags_7 },
    { 0x155ee142, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x6ffee144, SUC_lexicon_tags_31 },
    { 0xa95ee144, SUC_lexicon_tags_15 },
    { 0xbefee146, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1ee14a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5ee14e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121ee158, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3fee15c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dee15f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01ee16a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1ee16d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddee171, SUC_lexicon_tags_61 },
    { 0x347ee172, SUC_lexicon_tags_108 },
    { 0x795ee171, SUC_lexicon_tags_7 },
    { 0x903ee174, SUC_lexicon_tags_3 },
    { 0xf6bee173, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe57ee177, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f5ee179, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19ee17f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x11bee181, SUC_lexicon_tags_19 },
    { 0xf4fee181, SUC_lexicon_tags_21 },
    { 0x211ee181, SUC_lexicon_tags_17 },
    { 0x9ebee184, SUC_lexicon_tags_7 },
    { 0x7e5ee183, SUC_lexicon_tags_24 },
    { 0x8fbee183, SUC_lexicon_tags_57 },
    { 0xd67ee182, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74dee18c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe5ee18e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13dee194, SUC_lexicon_tags_14 },
    { 0x8dbee195, SUC_lexicon_tags_32 },
    { 0xd01ee195, SUC_lexicon_tags_17 },
    { 0xb71ee197, SUC_lexicon_tags_14 },
    { 0x4afee197, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7ee1a3, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75ee1a9, SUC_lexicon_tags_47 },
    { 0x9a7ee1a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x779ee1ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133ee1b2, SUC_lexicon_tags_33 },
    { 0x6e7ee1b3, SUC_lexicon_tags_68 },
    { 0x10fee1b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xda9ee1b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc75ee1b8, SUC_lexicon_tags_37 },
    { 0xc7fee1b9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7ee1bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231ee1c7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdb1ee1c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8b1ee1cb, SUC_lexicon_tags_26 },
    { 0xca5ee1cc, SUC_lexicon_tags_7 },
    { 0x5a1ee1cd, SUC_lexicon_tags_27 },
    { 0x5cbee1cd, SUC_lexicon_tags_8 },
    { 0x783ee1cf, SUC_lexicon_tags_38 },
    { 0x431ee1d0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dee1d4, SUC_lexicon_tags_32 },
    { 0x229ee1d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bee1d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf95ee1da, SUC_lexicon_tags_30 },
    { 0x82fee1da, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x33dee1dd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bee1e5, SUC_lexicon_tags_0 },
    { 0x371ee1e6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x481ee1e9, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x2a1ee1eb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131ee1ef, SUC_lexicon_tags_23 },
    { 0x82dee1ef, SUC_lexicon_tags_30 },
    { 0xf37ee1f1, SUC_lexicon_tags_49 },
    { 0x755ee1f2, SUC_lexicon_tags_28 },
    { 0x20bee1f3, SUC_lexicon_tags_43 },
    { 0x255ee1f2, SUC_lexicon_tags_19 },
    { 0x61ee1f0, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x7e3ee1f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x425ee1fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07ee1fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3ee205, SUC_lexicon_tags_8 },
    { 0xac5ee206, SUC_lexicon_tags_162 },
    { 0xd45ee205, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21ee20a, SUC_lexicon_tags_1 },
    { 0x859ee20b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xde9ee20d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xff3ee20f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9ee214, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe41ee216, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x191ee218, SUC_lexicon_tags_37 },
    { 0x533ee219, SUC_lexicon_tags_34 },
    { 0xfafee21a, SUC_lexicon_tags_1 },
    { 0xda3ee21b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bee222, SUC_lexicon_tags_27 },
    { 0x68bee222, SUC_lexicon_tags_141 },
    { 0x50dee224, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bfee226, SUC_lexicon_tags_14 },
    { 0x927ee226, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x457ee22a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe4bee22c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cbee22e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x563ee230, SUC_lexicon_tags_1 },
    { 0x82fee230, SUC_lexicon_tags_1 },
    { 0x73bee231, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bee238, SUC_lexicon_tags_15 },
    { 0x499ee238, SUC_lexicon_tags_39 },
    { 0x7ebee238, SUC_lexicon_tags_63 },
    { 0xf53ee23b, SUC_lexicon_tags_39 },
    { 0x2bee23c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x959ee23e, SUC_lexicon_tags_57 },
    { 0x76fee23f, SUC_lexicon_tags_37 },
    { 0x617ee240, SUC_lexicon_tags_33 },
    { 0x7d9ee241, SUC_lexicon_tags_23 },
    { 0x9e1ee242, SUC_lexicon_tags_25 },
    { 0x389ee242, SUC_lexicon_tags_11 },
    { 0xd6fee242, SUC_lexicon_tags_2 },
    { 0xadee23e, SUC_lexicon_tags_112 },
    { 0x66dee241, SUC_lexicon_tags_8 },
    { 0xd0fee244, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44dee24b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdee252, SUC_lexicon_tags_12 },
    { 0x1e1ee252, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf87ee255, SUC_lexicon_tags_25 },
    { 0xb5bee256, SUC_lexicon_tags_44 },
    { 0x5d9ee257, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8f1ee259, SUC_lexicon_tags_37 },
    { 0x8f1ee25a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x379ee25d, SUC_lexicon_tags_23 },
    { 0x4e3ee25e, SUC_lexicon_tags_23 },
    { 0x4e9ee25f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5ee267, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbee26b, SUC_lexicon_tags_11 },
    { 0xf7dee26c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971ee270, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x20bee272, SUC_lexicon_tags_46 },
    { 0xe2dee273, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777ee27c, SUC_lexicon_tags_14 },
    { 0x7dbee27c, SUC_lexicon_tags_7 },
    { 0x81ee27c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3ee281, SUC_lexicon_tags_115 },
    { 0x6a5ee281, SUC_lexicon_tags_31 },
    { 0x97dee283, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x92bee285, SUC_lexicon_tags_7 },
    { 0xf3fee286, SUC_lexicon_tags_35 },
    { 0xc43ee287, SUC_lexicon_tags_15 },
    { 0x82bee288, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x583ee28a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9ee28d, SUC_lexicon_tags_13 },
    { 0x2bdee28d, SUC_lexicon_tags_32 },
    { 0x709ee28f, SUC_lexicon_tags_25 },
    { 0xd51ee290, SUC_lexicon_tags_858 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77ee293, SUC_lexicon_tags_32 },
    { 0xdf9ee294, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bee297, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x753ee299, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x589ee29b, SUC_lexicon_tags_44 },
    { 0xfc9ee29b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6dee29e, SUC_lexicon_tags_17 },
    { 0x72dee29f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5ee2a4, SUC_lexicon_tags_2 },
    { 0x5b9ee2a4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb97ee2a7, SUC_lexicon_tags_7 },
    { 0x2b5ee2a8, SUC_lexicon_tags_1 },
    { 0xb27ee2a9, SUC_lexicon_tags_7 },
    { 0x9edee2a9, SUC_lexicon_tags_75 },
    { 0x9c5ee2ab, SUC_lexicon_tags_26 },
    { 0x38fee2a8, SUC_lexicon_tags_12 },
    { 0xebfee2a8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x557ee2b0, SUC_lexicon_tags_19 },
    { 0xfa7ee2b1, SUC_lexicon_tags_15 },
    { 0x449ee2b2, SUC_lexicon_tags_25 },
    { 0xa89ee2b0, SUC_lexicon_tags_12 },
    { 0x82bee2b1, SUC_lexicon_tags_37 },
    { 0xd1dee2b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x259ee2b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bbee2b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddee2bc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x6b9ee2be, SUC_lexicon_tags_63 },
    { 0xe1fee2be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xadbee2c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871ee2c5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddee2c8, SUC_lexicon_tags_8 },
    { 0xc8fee2c8, SUC_lexicon_tags_27 },
    { 0x2f5ee2c8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7fee2ce, SUC_lexicon_tags_7 },
    { 0xcbfee2cf, SUC_lexicon_tags_12 },
    { 0xf2dee2d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7ee2d6, SUC_lexicon_tags_2 },
    { 0x5fee2d7, SUC_lexicon_tags_12 },
    { 0x57ee2d8, SUC_lexicon_tags_37 },
    { 0xc4fee2d9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x597ee2dc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ee2e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01ee2e4, SUC_lexicon_tags_12 },
    { 0x59ee2e5, SUC_lexicon_tags_95 },
    { 0x7e1ee2e5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dee2eb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7ee2ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ee2f7, SUC_lexicon_tags_261 },
    { 0xe39ee2f8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdee2fb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9ee300, SUC_lexicon_tags_17 },
    { 0xbf1ee300, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3ee304, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4dbee306, SUC_lexicon_tags_30 },
    { 0xd0bee307, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x499ee309, SUC_lexicon_tags_12 },
    { 0x1fee30a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967ee311, SUC_lexicon_tags_30 },
    { 0xba5ee312, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bee31a, SUC_lexicon_tags_15 },
    { 0xcedee31b, SUC_lexicon_tags_7 },
    { 0x943ee31c, SUC_lexicon_tags_16 },
    { 0xab9ee31d, SUC_lexicon_tags_23 },
    { 0xc17ee31e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623ee325, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831ee32e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3ee331, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x305ee333, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x257ee335, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357ee33a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x9ebee33c, SUC_lexicon_tags_82 },
    { 0x31dee33d, SUC_lexicon_tags_17 },
    { 0x4edee33e, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687ee342, SUC_lexicon_tags_15 },
    { 0xf47ee342, SUC_lexicon_tags_47 },
    { 0x521ee343, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf3ee346, SUC_lexicon_tags_12 },
    { 0xfbbee347, SUC_lexicon_tags_46 },
    { 0xd27ee348, SUC_lexicon_tags_28 },
    { 0xb95ee349, SUC_lexicon_tags_29 },
    { 0xa59ee34a, SUC_lexicon_tags_38 },
    { 0x66fee34b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x483ee34d, SUC_lexicon_tags_35 },
    { 0x54bee34d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x981ee350, SUC_lexicon_tags_15 },
    { 0xac7ee351, SUC_lexicon_tags_47 },
    { 0xe43ee351, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dee357, SUC_lexicon_tags_36 },
    { 0x8c5ee357, SUC_lexicon_tags_25 },
    { 0xfbfee357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x105ee35b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441ee35f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bee362, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb33ee364, SUC_lexicon_tags_25 },
    { 0x89bee365, SUC_lexicon_tags_37 },
    { 0x785ee366, SUC_lexicon_tags_42 },
    { 0x40bee367, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255ee36b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adee36e, SUC_lexicon_tags_27 },
    { 0x155ee36f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x17bee371, SUC_lexicon_tags_257 },
    { 0xddfee372, SUC_lexicon_tags_15 },
    { 0x75ee373, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13ee378, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1ee37b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25ee37e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xef7ee381, SUC_lexicon_tags_25 },
    { 0x6afee382, SUC_lexicon_tags_39 },
    { 0x84fee381, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8e1ee385, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ee38c, SUC_lexicon_tags_12 },
    { 0xda5ee38d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edee391, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7ee394, SUC_lexicon_tags_19 },
    { 0xa9bee394, SUC_lexicon_tags_2 },
    { 0x1bee396, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59dee398, SUC_lexicon_tags_1 },
    { 0x995ee398, SUC_lexicon_tags_35 },
    { 0x6e5ee399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x38dee39c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7c5ee39e, SUC_lexicon_tags_85 },
    { 0x185ee39e, SUC_lexicon_tags_25 },
    { 0xb1ee3a0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x21fee3a2, SUC_lexicon_tags_59 },
    { 0xddfee3a3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa23ee3a5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5d3ee3a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4dee3a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf11ee3ab, SUC_lexicon_tags_15 },
    { 0xeadee3ab, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x25ee3ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xf83ee3b0, SUC_lexicon_tags_8 },
    { 0xdefee3b1, SUC_lexicon_tags_28 },
    { 0x8c5ee3b1, SUC_lexicon_tags_0 },
    { 0xde7ee3b3, SUC_lexicon_tags_12 },
    { 0x3b3ee3b1, SUC_lexicon_tags_7 },
    { 0xc5bee3b5, SUC_lexicon_tags_50 },
    { 0xe87ee3b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91ee3ba, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x28dee3bd, SUC_lexicon_tags_2 },
    { 0x9f9ee3be, SUC_lexicon_tags_79 },
    { 0x6a7ee3bf, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x6b9ee3c1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3ee3c7, SUC_lexicon_tags_26 },
    { 0xe41ee3c8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bee3cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91ee3d3, SUC_lexicon_tags_70 },
    { 0x77bee3d4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63ee3d7, SUC_lexicon_tags_12 },
    { 0x299ee3d8, SUC_lexicon_tags_29 },
    { 0xcf5ee3d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1ee3dc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95ee3e0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bee3e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34dee3e9, SUC_lexicon_tags_16 },
    { 0x32dee3ea, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bee3ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf13ee3f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf1bee3f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec9ee3f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3ee3f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bee3fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeb3ee400, SUC_lexicon_tags_37 },
    { 0xa69ee401, SUC_lexicon_tags_12 },
    { 0xe1ee401, SUC_lexicon_tags_25 },
    { 0x81bee400, SUC_lexicon_tags_25 },
    { 0x64bee404, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6a1ee406, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7ee40b, SUC_lexicon_tags_33 },
    { 0x1adee40c, SUC_lexicon_tags_9 },
    { 0xb69ee40b, SUC_lexicon_tags_7 },
    { 0xc1bee40d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dee413, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2fee417, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb39ee419, SUC_lexicon_tags_12 },
    { 0xe75ee41a, SUC_lexicon_tags_6 },
    { 0x8d1ee41b, SUC_lexicon_tags_7 },
    { 0xb25ee41c, SUC_lexicon_tags_17 },
    { 0x37ee41d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bee422, SUC_lexicon_tags_17 },
    { 0x2e5ee423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x117ee425, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d7ee427, SUC_lexicon_tags_25 },
    { 0x5f1ee427, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x491ee42a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xc27ee42c, SUC_lexicon_tags_2 },
    { 0x2f5ee42d, SUC_lexicon_tags_26 },
    { 0x959ee42d, SUC_lexicon_tags_1 },
    { 0x3bdee42f, SUC_lexicon_tags_28 },
    { 0x7fbee42f, SUC_lexicon_tags_19 },
    { 0x67dee42c, SUC_lexicon_tags_33 },
    { 0x25ee432, SUC_lexicon_tags_1 },
    { 0xaf9ee432, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x85dee435, SUC_lexicon_tags_17 },
    { 0xcd5ee435, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1ee43a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1ee442, SUC_lexicon_tags_37 },
    { 0xfffee442, SUC_lexicon_tags_1 },
    { 0x85bee444, SUC_lexicon_tags_14 },
    { 0x4c5ee445, SUC_lexicon_tags_15 },
    { 0x18dee446, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x249ee448, SUC_lexicon_tags_19 },
    { 0x8c3ee449, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f5ee44b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455ee453, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x417ee455, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd2bee457, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x695ee459, SUC_lexicon_tags_6 },
    { 0x8e5ee459, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9ee45f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dee462, SUC_lexicon_tags_55 },
    { 0x645ee463, SUC_lexicon_tags_26 },
    { 0x571ee464, SUC_lexicon_tags_16 },
    { 0xd8bee464, SUC_lexicon_tags_60 },
    { 0x52dee466, SUC_lexicon_tags_1 },
    { 0xabee467, SUC_lexicon_tags_37 },
    { 0xe6bee467, SUC_lexicon_tags_26 },
    { 0x3a1ee469, SUC_lexicon_tags_23 },
    { 0xa99ee46a, SUC_lexicon_tags_3 },
    { 0xdefee46b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301ee46f, SUC_lexicon_tags_52 },
    { 0xd25ee470, SUC_lexicon_tags_15 },
    { 0xc3fee470, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5ee475, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55ee47d, SUC_lexicon_tags_376 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5ee485, SUC_lexicon_tags_39 },
    { 0xe09ee485, SUC_lexicon_tags_19 },
    { 0xe7ee485, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x335ee48a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39ee48d, SUC_lexicon_tags_15 },
    { 0x5f7ee48d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xb65ee491, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1ee494, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x443ee497, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27ee49d, SUC_lexicon_tags_11 },
    { 0xd2bee49e, SUC_lexicon_tags_2 },
    { 0xfd1ee49f, SUC_lexicon_tags_10 },
    { 0xbb7ee49f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x79dee4a3, SUC_lexicon_tags_15 },
    { 0xbefee4a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1ee4a8, SUC_lexicon_tags_37 },
    { 0xdb1ee4a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x393ee4ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbee4af, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8e9ee4b1, SUC_lexicon_tags_23 },
    { 0xddee4b2, SUC_lexicon_tags_70 },
    { 0xcffee4b2, SUC_lexicon_tags_11 },
    { 0x395ee4b4, SUC_lexicon_tags_16 },
    { 0xf59ee4b4, SUC_lexicon_tags_11 },
    { 0x489ee4b5, SUC_lexicon_tags_4 },
    { 0x743ee4b7, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7ee4ba, SUC_lexicon_tags_28 },
    { 0x323ee4bb, SUC_lexicon_tags_7 },
    { 0xd69ee4bc, SUC_lexicon_tags_30 },
    { 0x41fee4bd, SUC_lexicon_tags_31 },
    { 0x97bee4be, SUC_lexicon_tags_82 },
    { 0xfb3ee4bf, SUC_lexicon_tags_39 },
    { 0x4edee4bf, SUC_lexicon_tags_1 },
    { 0xd3fee4bf, SUC_lexicon_tags_107 },
    { 0x60bee4c2, SUC_lexicon_tags_79 },
    { 0x4d3ee4c1, SUC_lexicon_tags_7 },
    { 0x171ee4bf, SUC_lexicon_tags_27 },
    { 0x25ee4c5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669ee4cb, SUC_lexicon_tags_38 },
    { 0xb13ee4cc, SUC_lexicon_tags_17 },
    { 0xd37ee4cd, SUC_lexicon_tags_19 },
    { 0x1bdee4ce, SUC_lexicon_tags_43 },
    { 0x3efee4ce, SUC_lexicon_tags_51 },
    { 0x1fdee4d0, SUC_lexicon_tags_23 },
    { 0x525ee4d0, SUC_lexicon_tags_14 },
    { 0x227ee4d2, SUC_lexicon_tags_33 },
    { 0xe65ee4d2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58fee4da, SUC_lexicon_tags_10 },
    { 0x1e3ee4db, SUC_lexicon_tags_12 },
    { 0x235ee4db, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dee4e1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dee4ec, SUC_lexicon_tags_60 },
    { 0x3b3ee4ed, SUC_lexicon_tags_17 },
    { 0x509ee4ee, SUC_lexicon_tags_70 },
    { 0x78bee4ee, SUC_lexicon_tags_23 },
    { 0x7b1ee4ee, SUC_lexicon_tags_7 },
    { 0x1dfee4f1, SUC_lexicon_tags_27 },
    { 0xfc3ee4f1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x993ee4f4, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf79ee4f6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85ee4fd, SUC_lexicon_tags_17 },
    { 0xaf3ee4fd, SUC_lexicon_tags_17 },
    { 0xc51ee4ff, SUC_lexicon_tags_37 },
    { 0xf5ee500, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xef3ee502, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdedee504, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47ee507, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x411ee509, SUC_lexicon_tags_68 },
    { 0xdedee50a, SUC_lexicon_tags_31 },
    { 0xf83ee50a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb63ee50d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x533ee510, SUC_lexicon_tags_16 },
    { 0x163ee510, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693ee516, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5ee51b, SUC_lexicon_tags_2 },
    { 0x8fbee51b, SUC_lexicon_tags_12 },
    { 0xfcbee51d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bee523, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x41fee526, SUC_lexicon_tags_25 },
    { 0xa2dee527, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abee52a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfee531, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bee534, SUC_lexicon_tags_23 },
    { 0xbfdee534, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627ee539, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7ee53c, SUC_lexicon_tags_47 },
    { 0x701ee53d, SUC_lexicon_tags_18 },
    { 0xa59ee53c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefee543, SUC_lexicon_tags_29 },
    { 0x3b7ee544, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7dfee546, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dee54b, SUC_lexicon_tags_1 },
    { 0x90bee54c, SUC_lexicon_tags_1 },
    { 0x9e9ee54c, SUC_lexicon_tags_26 },
    { 0x9e3ee54c, SUC_lexicon_tags_21 },
    { 0xb69ee54f, SUC_lexicon_tags_25 },
    { 0x9bfee550, SUC_lexicon_tags_21 },
    { 0x409ee550, SUC_lexicon_tags_30 },
    { 0x5e9ee552, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ee555, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x145ee558, SUC_lexicon_tags_15 },
    { 0xc03ee558, SUC_lexicon_tags_37 },
    { 0x219ee55a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8fee560, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfee563, SUC_lexicon_tags_1 },
    { 0xfcdee563, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x177ee567, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71ee56d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bee571, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xfa9ee573, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1ee576, SUC_lexicon_tags_25 },
    { 0x8cbee576, SUC_lexicon_tags_32 },
    { 0xa85ee576, SUC_lexicon_tags_25 },
    { 0xb69ee579, SUC_lexicon_tags_15 },
    { 0x169ee579, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e1ee57c, SUC_lexicon_tags_23 },
    { 0x511ee57c, SUC_lexicon_tags_3 },
    { 0xb4bee57c, SUC_lexicon_tags_60 },
    { 0x94dee57f, SUC_lexicon_tags_27 },
    { 0x20fee57d, SUC_lexicon_tags_33 },
    { 0x75ee57f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5a3ee583, SUC_lexicon_tags_49 },
    { 0xcc7ee584, SUC_lexicon_tags_12 },
    { 0x62fee585, SUC_lexicon_tags_31 },
    { 0x143ee585, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x137ee588, SUC_lexicon_tags_37 },
    { 0x179ee589, SUC_lexicon_tags_17 },
    { 0x8c1ee58a, SUC_lexicon_tags_17 },
    { 0xdc5ee588, SUC_lexicon_tags_30 },
    { 0xce1ee589, SUC_lexicon_tags_17 },
    { 0xfffee58c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x779ee590, SUC_lexicon_tags_49 },
    { 0x267ee591, SUC_lexicon_tags_23 },
    { 0x457ee590, SUC_lexicon_tags_10 },
    { 0xdb9ee593, SUC_lexicon_tags_31 },
    { 0x5b7ee594, SUC_lexicon_tags_15 },
    { 0x37ee594, SUC_lexicon_tags_8 },
    { 0x535ee596, SUC_lexicon_tags_96 },
    { 0x503ee597, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5ee59c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x6d7ee59e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x205ee5a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5b9ee5a2, SUC_lexicon_tags_28 },
    { 0xc69ee5a3, SUC_lexicon_tags_11 },
    { 0xeafee5a3, SUC_lexicon_tags_11 },
    { 0xdbee5a2, SUC_lexicon_tags_2 },
    { 0x2e7ee5a4, SUC_lexicon_tags_4 },
    { 0x75bee5a4, SUC_lexicon_tags_11 },
    { 0xddfee5a8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59bee5ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd87ee5af, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x959ee5b2, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x329ee5b5, SUC_lexicon_tags_12 },
    { 0xc6dee5b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37ee5c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee7ee5ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x419ee5cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdee5d0, SUC_lexicon_tags_60 },
    { 0xd49ee5d1, SUC_lexicon_tags_97 },
    { 0x5efee5d0, SUC_lexicon_tags_9 },
    { 0x289ee5d3, SUC_lexicon_tags_1 },
    { 0xf9dee5d4, SUC_lexicon_tags_21 },
    { 0xdfbee5d2, SUC_lexicon_tags_16 },
    { 0xfb3ee5d6, SUC_lexicon_tags_37 },
    { 0x9ffee5d7, SUC_lexicon_tags_7 },
    { 0x56fee5d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfee5db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767ee5e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x66bee5e2, SUC_lexicon_tags_0 },
    { 0xa0dee5e3, SUC_lexicon_tags_11 },
    { 0x1f9ee5e4, SUC_lexicon_tags_27 },
    { 0x663ee5e5, SUC_lexicon_tags_0 },
    { 0xcdee5e6, SUC_lexicon_tags_1 },
    { 0x965ee5e3, SUC_lexicon_tags_11 },
    { 0xeabee5e8, SUC_lexicon_tags_27 },
    { 0xabfee5e3, SUC_lexicon_tags_26 },
    { 0x1f7ee5e2, SUC_lexicon_tags_19 },
    { 0x213ee5e4, SUC_lexicon_tags_11 },
    { 0xc65ee5ec, SUC_lexicon_tags_4 },
    { 0xa6fee5ed, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45ee5f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1ee5fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ddee5ff, SUC_lexicon_tags_33 },
    { 0x273ee5ff, SUC_lexicon_tags_0 },
    { 0x24fee601, SUC_lexicon_tags_106 },
    { 0x43bee601, SUC_lexicon_tags_37 },
    { 0xd01ee603, SUC_lexicon_tags_37 },
    { 0x95dee603, SUC_lexicon_tags_38 },
    { 0x1fee604, SUC_lexicon_tags_23 },
    { 0x759ee605, SUC_lexicon_tags_14 },
    { 0xfcdee607, SUC_lexicon_tags_7 },
    { 0xca9ee607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87ee610, SUC_lexicon_tags_14 },
    { 0x30fee611, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x70fee613, SUC_lexicon_tags_81 },
    { 0xb65ee614, SUC_lexicon_tags_11 },
    { 0xa1dee615, SUC_lexicon_tags_17 },
    { 0x6bfee616, SUC_lexicon_tags_4 },
    { 0xfc3ee617, SUC_lexicon_tags_12 },
    { 0xb5dee618, SUC_lexicon_tags_40 },
    { 0xb0bee619, SUC_lexicon_tags_17 },
    { 0xf05ee61a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bee61d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22fee622, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x483ee624, SUC_lexicon_tags_70 },
    { 0x7e3ee625, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79ee62a, SUC_lexicon_tags_37 },
    { 0x69ee62a, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569ee62f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bee632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb55ee634, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47ee637, SUC_lexicon_tags_15 },
    { 0x4fbee638, SUC_lexicon_tags_19 },
    { 0x9b1ee639, SUC_lexicon_tags_11 },
    { 0x281ee639, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x64dee63d, SUC_lexicon_tags_351 },
    { 0x87bee63d, SUC_lexicon_tags_37 },
    { 0x8afee63f, SUC_lexicon_tags_33 },
    { 0x789ee640, SUC_lexicon_tags_43 },
    { 0x639ee641, SUC_lexicon_tags_63 },
    { 0xab3ee642, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3dee647, SUC_lexicon_tags_15 },
    { 0xfcfee648, SUC_lexicon_tags_2 },
    { 0x619ee648, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xacdee64b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd47ee64d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x86bee64f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x49dee651, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7ee655, SUC_lexicon_tags_23 },
    { 0xcc5ee655, SUC_lexicon_tags_25 },
    { 0x6e7ee655, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dee65d, SUC_lexicon_tags_1 },
    { 0xf3bee65e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7bee660, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x193ee663, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5ee66a, SUC_lexicon_tags_30 },
    { 0xa21ee66b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffee66f, SUC_lexicon_tags_47 },
    { 0x45fee66f, SUC_lexicon_tags_7 },
    { 0x399ee671, SUC_lexicon_tags_11 },
    { 0xb77ee671, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3ee676, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x24dee679, SUC_lexicon_tags_23 },
    { 0xba1ee67a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x281ee67c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fee684, SUC_lexicon_tags_23 },
    { 0x23bee685, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36bee687, SUC_lexicon_tags_26 },
    { 0xa17ee688, SUC_lexicon_tags_25 },
    { 0x841ee689, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbee68c, SUC_lexicon_tags_25 },
    { 0xbdee68d, SUC_lexicon_tags_14 },
    { 0x831ee68e, SUC_lexicon_tags_6 },
    { 0x9b7ee68f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7ee696, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x5a9ee698, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fee69f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2fee6a2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xacbee6a4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfcdee6a6, SUC_lexicon_tags_47 },
    { 0x5ebee6a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197ee6ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa11ee6ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8fee6b1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa2bee6b3, SUC_lexicon_tags_29 },
    { 0x469ee6b4, SUC_lexicon_tags_1 },
    { 0x9d5ee6b5, SUC_lexicon_tags_17 },
    { 0xb51ee6b6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fee6b9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3ee6bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbee6bf, SUC_lexicon_tags_8 },
    { 0x5e5ee6c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb19ee6c2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x823ee6c4, SUC_lexicon_tags_30 },
    { 0x3dbee6c4, SUC_lexicon_tags_27 },
    { 0xaf3ee6c6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x215ee6c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fee6cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47fee6ce, SUC_lexicon_tags_33 },
    { 0xc09ee6ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51ee6d2, SUC_lexicon_tags_45 },
    { 0xee9ee6d3, SUC_lexicon_tags_6 },
    { 0x16bee6d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x70bee6d7, SUC_lexicon_tags_31 },
    { 0xd53ee6d8, SUC_lexicon_tags_37 },
    { 0xefbee6d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913ee6e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd9fee6e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3ee6eb, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2dee6ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x80dee6f0, SUC_lexicon_tags_2 },
    { 0x4dfee6f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7ee6f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bee6f7, SUC_lexicon_tags_43 },
    { 0x69fee6f8, SUC_lexicon_tags_42 },
    { 0xeb1ee6f9, SUC_lexicon_tags_89 },
    { 0xa3ee6fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd1ee6fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9bee6fe, SUC_lexicon_tags_15 },
    { 0xf57ee6ff, SUC_lexicon_tags_53 },
    { 0x5b9ee6ff, SUC_lexicon_tags_2 },
    { 0xacdee701, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fee706, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb5ee708, SUC_lexicon_tags_2 },
    { 0xf59ee709, SUC_lexicon_tags_16 },
    { 0x367ee70a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x3b9ee70c, SUC_lexicon_tags_35 },
    { 0x235ee70d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9ee710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197ee716, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dee719, SUC_lexicon_tags_57 },
    { 0x48dee71a, SUC_lexicon_tags_64 },
    { 0xa3fee71b, SUC_lexicon_tags_37 },
    { 0xf99ee719, SUC_lexicon_tags_7 },
    { 0xde7ee71d, SUC_lexicon_tags_12 },
    { 0x429ee71d, SUC_lexicon_tags_7 },
    { 0x34dee71f, SUC_lexicon_tags_37 },
    { 0x939ee71f, SUC_lexicon_tags_25 },
    { 0xf21ee721, SUC_lexicon_tags_7 },
    { 0x3bee722, SUC_lexicon_tags_37 },
    { 0x911ee723, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fee729, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb7dee72b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1ee72f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53ee735, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5b3ee737, SUC_lexicon_tags_33 },
    { 0x3d1ee738, SUC_lexicon_tags_57 },
    { 0x2bee739, SUC_lexicon_tags_17 },
    { 0x30fee739, SUC_lexicon_tags_0 },
    { 0x4bfee739, SUC_lexicon_tags_17 },
    { 0xeadee73a, SUC_lexicon_tags_37 },
    { 0xf0fee737, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9ee743, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ee74b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5a5ee74d, SUC_lexicon_tags_11 },
    { 0x81bee74d, SUC_lexicon_tags_25 },
    { 0xbf5ee74f, SUC_lexicon_tags_27 },
    { 0xa45ee74f, SUC_lexicon_tags_14 },
    { 0xb3fee750, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x50fee753, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601ee75c, SUC_lexicon_tags_101 },
    { 0xef3ee75d, SUC_lexicon_tags_2 },
    { 0xa75ee75e, SUC_lexicon_tags_15 },
    { 0xb57ee75e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b1ee761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63dee763, SUC_lexicon_tags_7 },
    { 0xfe1ee763, SUC_lexicon_tags_1 },
    { 0x7e1ee763, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x621ee767, SUC_lexicon_tags_17 },
    { 0xf6bee768, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bee76b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5afee771, SUC_lexicon_tags_1 },
    { 0x329ee772, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbee776, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfee77b, SUC_lexicon_tags_236 },
    { 0, NULL },
    { 0, NULL },
    { 0x239ee77e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfe9ee780, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbee783, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f9ee785, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5ee78e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9ee792, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7ee795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dee79a, SUC_lexicon_tags_15 },
    { 0xf83ee79b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2dee79f, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0x645ee7a1, SUC_lexicon_tags_6 },
    { 0x6cfee7a2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaefee7ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87fee7b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1bfee7b2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x769ee7b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbee7b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf1ee7b9, SUC_lexicon_tags_43 },
    { 0x98bee7b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbc3ee7bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781ee7c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69ee7c6, SUC_lexicon_tags_47 },
    { 0x1cfee7c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93ee7ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63ee7cd, SUC_lexicon_tags_33 },
    { 0x81fee7cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bee7d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe9ee7d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x711ee7d5, SUC_lexicon_tags_17 },
    { 0xca3ee7d6, SUC_lexicon_tags_7 },
    { 0x1adee7d7, SUC_lexicon_tags_37 },
    { 0x46bee7d7, SUC_lexicon_tags_30 },
    { 0xd5fee7d8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc5dee7db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9e5ee7dd, SUC_lexicon_tags_87 },
    { 0x539ee7dd, SUC_lexicon_tags_15 },
    { 0xd9ee7df, SUC_lexicon_tags_19 },
    { 0xadfee7e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393ee7ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbe9ee7ee, SUC_lexicon_tags_82 },
    { 0x40dee7ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb85ee7f1, SUC_lexicon_tags_182 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589ee7f6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63dee7fb, SUC_lexicon_tags_38 },
    { 0xf1dee7fc, SUC_lexicon_tags_37 },
    { 0xd45ee7fc, SUC_lexicon_tags_49 },
    { 0x4adee7fc, SUC_lexicon_tags_11 },
    { 0xa95ee7ff, SUC_lexicon_tags_8 },
    { 0xc65ee800, SUC_lexicon_tags_28 },
    { 0x4a5ee800, SUC_lexicon_tags_7 },
    { 0xf35ee7fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abee808, SUC_lexicon_tags_25 },
    { 0xe07ee809, SUC_lexicon_tags_7 },
    { 0xa0bee80a, SUC_lexicon_tags_191 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71ee80f, SUC_lexicon_tags_6 },
    { 0x859ee810, SUC_lexicon_tags_47 },
    { 0xc2bee811, SUC_lexicon_tags_22 },
    { 0x9bee811, SUC_lexicon_tags_11 },
    { 0x423ee812, SUC_lexicon_tags_17 },
    { 0x477ee811, SUC_lexicon_tags_3 },
    { 0xc09ee815, SUC_lexicon_tags_31 },
    { 0x83bee816, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabee81a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909ee81e, SUC_lexicon_tags_23 },
    { 0x255ee81f, SUC_lexicon_tags_37 },
    { 0xf2dee81f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb33ee822, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x415ee828, SUC_lexicon_tags_2 },
    { 0x8afee828, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711ee82d, SUC_lexicon_tags_25 },
    { 0x6adee82e, SUC_lexicon_tags_11 },
    { 0x9dee82f, SUC_lexicon_tags_17 },
    { 0x289ee82f, SUC_lexicon_tags_27 },
    { 0x3d3ee830, SUC_lexicon_tags_37 },
    { 0x1cdee831, SUC_lexicon_tags_17 },
    { 0x1c3ee833, SUC_lexicon_tags_25 },
    { 0x719ee82f, SUC_lexicon_tags_0 },
    { 0xe7ee830, SUC_lexicon_tags_38 },
    { 0x41dee836, SUC_lexicon_tags_63 },
    { 0xc17ee833, SUC_lexicon_tags_12 },
    { 0x9a7ee838, SUC_lexicon_tags_7 },
    { 0xba7ee839, SUC_lexicon_tags_14 },
    { 0xf9fee839, SUC_lexicon_tags_25 },
    { 0xcabee83a, SUC_lexicon_tags_12 },
    { 0x4f1ee83a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83ee848, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5c1ee84a, SUC_lexicon_tags_2 },
    { 0x1bdee84b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b1ee84d, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0xf8bee84f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5bfee851, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fee855, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8c5ee857, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc5fee859, SUC_lexicon_tags_10 },
    { 0xb1bee85a, SUC_lexicon_tags_54 },
    { 0x14dee85a, SUC_lexicon_tags_30 },
    { 0xe03ee85c, SUC_lexicon_tags_15 },
    { 0x525ee85d, SUC_lexicon_tags_11 },
    { 0x61fee85c, SUC_lexicon_tags_49 },
    { 0xe1ee85f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dee864, SUC_lexicon_tags_15 },
    { 0xe13ee864, SUC_lexicon_tags_201 },
    { 0x803ee866, SUC_lexicon_tags_37 },
    { 0x9bdee864, SUC_lexicon_tags_33 },
    { 0xfadee865, SUC_lexicon_tags_9 },
    { 0x1f9ee869, SUC_lexicon_tags_32 },
    { 0x603ee86a, SUC_lexicon_tags_37 },
    { 0x295ee86b, SUC_lexicon_tags_26 },
    { 0x719ee868, SUC_lexicon_tags_21 },
    { 0xe87ee866, SUC_lexicon_tags_38 },
    { 0x269ee86e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5fee870, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66fee878, SUC_lexicon_tags_27 },
    { 0x1f1ee879, SUC_lexicon_tags_11 },
    { 0xd11ee87a, SUC_lexicon_tags_16 },
    { 0x7edee87a, SUC_lexicon_tags_27 },
    { 0x5b5ee879, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69ee87f, SUC_lexicon_tags_37 },
    { 0x94fee880, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73ee886, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5ee88f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181ee89a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x131ee89c, SUC_lexicon_tags_11 },
    { 0x567ee89c, SUC_lexicon_tags_37 },
    { 0xd91ee89e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3ee8a2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71dee8a7, SUC_lexicon_tags_39 },
    { 0x453ee8a7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5ee8ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6edee8ae, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x689ee8b0, SUC_lexicon_tags_33 },
    { 0xbc9ee8b1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77ee8b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd95ee8b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfee8bc, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5f9ee8be, SUC_lexicon_tags_12 },
    { 0x641ee8bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bee8c2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55ee8c7, SUC_lexicon_tags_0 },
    { 0xbdbee8c7, SUC_lexicon_tags_37 },
    { 0xe73ee8c7, SUC_lexicon_tags_11 },
    { 0x30fee8c8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x867ee8cc, SUC_lexicon_tags_10 },
    { 0x659ee8cd, SUC_lexicon_tags_11 },
    { 0xd5dee8ce, SUC_lexicon_tags_17 },
    { 0x97dee8cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4fee8d3, SUC_lexicon_tags_14 },
    { 0xa5ee8d4, SUC_lexicon_tags_17 },
    { 0xcbee8d3, SUC_lexicon_tags_38 },
    { 0x135ee8d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x223ee8d9, SUC_lexicon_tags_23 },
    { 0x3c7ee8da, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x21fee8dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x669ee8de, SUC_lexicon_tags_87 },
    { 0x71dee8df, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7ee8e6, SUC_lexicon_tags_19 },
    { 0x1bfee8e7, SUC_lexicon_tags_66 },
    { 0x51ee8e7, SUC_lexicon_tags_37 },
    { 0x6cfee8e9, SUC_lexicon_tags_15 },
    { 0xf15ee8ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83ee8ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x219ee8f0, SUC_lexicon_tags_17 },
    { 0x92fee8f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879ee8f6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65ee8fb, SUC_lexicon_tags_22 },
    { 0x6c3ee8fb, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ee902, SUC_lexicon_tags_7 },
    { 0xcd1ee902, SUC_lexicon_tags_47 },
    { 0x4e5ee904, SUC_lexicon_tags_15 },
    { 0x84fee903, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421ee90a, SUC_lexicon_tags_28 },
    { 0x659ee90a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x967ee916, SUC_lexicon_tags_68 },
    { 0xce1ee917, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21ee91c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x183ee91f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15dee924, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1b5ee926, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f1ee928, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49ee92b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97ee92f, SUC_lexicon_tags_37 },
    { 0xb41ee92f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6e1ee932, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3ee937, SUC_lexicon_tags_12 },
    { 0x46dee937, SUC_lexicon_tags_2 },
    { 0x499ee939, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb23ee93f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ee949, SUC_lexicon_tags_12 },
    { 0x497ee94a, SUC_lexicon_tags_21 },
    { 0xb55ee94b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x261ee94e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857ee95a, SUC_lexicon_tags_9 },
    { 0x63dee95b, SUC_lexicon_tags_47 },
    { 0xfdfee95c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39ee960, SUC_lexicon_tags_33 },
    { 0x2ddee961, SUC_lexicon_tags_9 },
    { 0xc17ee961, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xca5ee964, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xafee966, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bee96a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x601ee96d, SUC_lexicon_tags_28 },
    { 0x457ee96d, SUC_lexicon_tags_15 },
    { 0x479ee96d, SUC_lexicon_tags_84 },
    { 0xaafee970, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb01ee972, SUC_lexicon_tags_25 },
    { 0xef9ee972, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x637ee975, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x28dee977, SUC_lexicon_tags_30 },
    { 0x1bdee978, SUC_lexicon_tags_17 },
    { 0x73dee978, SUC_lexicon_tags_12 },
    { 0x635ee97a, SUC_lexicon_tags_25 },
    { 0x9a3ee97b, SUC_lexicon_tags_25 },
    { 0x92bee97c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75ee981, SUC_lexicon_tags_7 },
    { 0x22fee982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef7ee984, SUC_lexicon_tags_84 },
    { 0x92fee985, SUC_lexicon_tags_15 },
    { 0xc19ee986, SUC_lexicon_tags_7 },
    { 0x579ee987, SUC_lexicon_tags_21 },
    { 0x9b1ee987, SUC_lexicon_tags_11 },
    { 0xefee988, SUC_lexicon_tags_15 },
    { 0xf77ee985, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257ee9a1, SUC_lexicon_tags_37 },
    { 0x919ee9a2, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209ee9a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7ee9ac, SUC_lexicon_tags_33 },
    { 0x395ee9ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc1fee9af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x75dee9b2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdee9b7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17ee9be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x941ee9c1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x6c3ee9c3, SUC_lexicon_tags_7 },
    { 0xd5dee9c4, SUC_lexicon_tags_84 },
    { 0xe53ee9c5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ee9c8, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1ee9ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x793ee9d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3ee9d4, SUC_lexicon_tags_7 },
    { 0x6d5ee9d5, SUC_lexicon_tags_12 },
    { 0xb71ee9d4, SUC_lexicon_tags_28 },
    { 0x811ee9d7, SUC_lexicon_tags_68 },
    { 0x269ee9d8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbee9db, SUC_lexicon_tags_47 },
    { 0xbebee9db, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ee9e1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8e9ee9e3, SUC_lexicon_tags_87 },
    { 0x873ee9e3, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x75dee9e6, SUC_lexicon_tags_25 },
    { 0x493ee9e7, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xcbbee9e9, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x165ee9eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92dee9ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddee9f3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xeefee9f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6c1ee9f8, SUC_lexicon_tags_29 },
    { 0xd49ee9f9, SUC_lexicon_tags_63 },
    { 0x597ee9f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5eea00, SUC_lexicon_tags_77 },
    { 0x92feea01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb95eea03, SUC_lexicon_tags_26 },
    { 0xcdeea04, SUC_lexicon_tags_49 },
    { 0xce7eea05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75feea0c, SUC_lexicon_tags_17 },
    { 0xb49eea0d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xffbeea0f, SUC_lexicon_tags_11 },
    { 0x46feea10, SUC_lexicon_tags_9 },
    { 0xa03eea11, SUC_lexicon_tags_28 },
    { 0xf8feea11, SUC_lexicon_tags_47 },
    { 0x791eea13, SUC_lexicon_tags_17 },
    { 0x9ffeea10, SUC_lexicon_tags_33 },
    { 0x9d5eea15, SUC_lexicon_tags_44 },
    { 0x225eea15, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x241eea19, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc3feea1b, SUC_lexicon_tags_48 },
    { 0x48deea1c, SUC_lexicon_tags_37 },
    { 0xcf5eea1d, SUC_lexicon_tags_33 },
    { 0xcc5eea1e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59beea21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37eea27, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1eea30, SUC_lexicon_tags_1 },
    { 0x8efeea30, SUC_lexicon_tags_15 },
    { 0xcbeea31, SUC_lexicon_tags_15 },
    { 0x34deea31, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc9deea35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x803eea37, SUC_lexicon_tags_7 },
    { 0x78feea38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee3eea3a, SUC_lexicon_tags_15 },
    { 0x54beea3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdeea3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xca3eea41, SUC_lexicon_tags_1 },
    { 0x5dbeea42, SUC_lexicon_tags_25 },
    { 0x9e5eea41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0feea45, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9eea48, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x781eea4a, SUC_lexicon_tags_29 },
    { 0x25deea4b, SUC_lexicon_tags_1 },
    { 0x141eea4a, SUC_lexicon_tags_28 },
    { 0x393eea4c, SUC_lexicon_tags_12 },
    { 0x4cbeea4a, SUC_lexicon_tags_37 },
    { 0x6b7eea4f, SUC_lexicon_tags_38 },
    { 0x26beea50, SUC_lexicon_tags_19 },
    { 0x279eea51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x76beea53, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x679eea59, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff5eea60, SUC_lexicon_tags_12 },
    { 0x5f1eea61, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71eea67, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1deea6a, SUC_lexicon_tags_77 },
    { 0xbfbeea6b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01eea6f, SUC_lexicon_tags_2 },
    { 0x6ddeea70, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479eea76, SUC_lexicon_tags_18 },
    { 0xc1eea76, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07eea7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbeea7f, SUC_lexicon_tags_33 },
    { 0x245eea80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3feea86, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21eea8a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x73eea8d, SUC_lexicon_tags_12 },
    { 0xeadeea8e, SUC_lexicon_tags_8 },
    { 0x899eea8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487eea93, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9eea96, SUC_lexicon_tags_11 },
    { 0x55feea96, SUC_lexicon_tags_45 },
    { 0x34deea98, SUC_lexicon_tags_31 },
    { 0xda3eea99, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43eea9f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x907eeaa1, SUC_lexicon_tags_17 },
    { 0x77beeaa2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x843eeaa4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x799eeaa6, SUC_lexicon_tags_129 },
    { 0x341eeaa7, SUC_lexicon_tags_43 },
    { 0x5dfeeaa8, SUC_lexicon_tags_11 },
    { 0x399eeaa9, SUC_lexicon_tags_10 },
    { 0x805eeaa7, SUC_lexicon_tags_11 },
    { 0x1c1eeaab, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbeeaae, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe81eeab0, SUC_lexicon_tags_113 },
    { 0x81deeab1, SUC_lexicon_tags_57 },
    { 0xe7beeab2, SUC_lexicon_tags_37 },
    { 0x8fdeeab3, SUC_lexicon_tags_2 },
    { 0x185eeab1, SUC_lexicon_tags_1 },
    { 0xabeeab4, SUC_lexicon_tags_25 },
    { 0xaabeeab0, SUC_lexicon_tags_12 },
    { 0xbeeab0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edeeac0, SUC_lexicon_tags_11 },
    { 0x3feeac1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x163eeac3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e3eeac5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x10deeac7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51eeacb, SUC_lexicon_tags_37 },
    { 0xde1eeacb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x583eeace, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x433eead0, SUC_lexicon_tags_84 },
    { 0x803eead0, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0x323eead3, SUC_lexicon_tags_2 },
    { 0x8bfeead4, SUC_lexicon_tags_107 },
    { 0x4d5eead4, SUC_lexicon_tags_17 },
    { 0xa9feead6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331eeade, SUC_lexicon_tags_38 },
    { 0xd4deeadf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8deeae1, SUC_lexicon_tags_15 },
    { 0xf7eeae1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3efeeae4, SUC_lexicon_tags_25 },
    { 0x25eeae5, SUC_lexicon_tags_2 },
    { 0xd2deeae6, SUC_lexicon_tags_47 },
    { 0x6adeeae6, SUC_lexicon_tags_15 },
    { 0x781eeae4, SUC_lexicon_tags_38 },
    { 0x777eeae9, SUC_lexicon_tags_26 },
    { 0xd25eeae9, SUC_lexicon_tags_2 },
    { 0x7deeaea, SUC_lexicon_tags_11 },
    { 0x385eeaea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44deeaf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x89beeaf5, SUC_lexicon_tags_2 },
    { 0x96beeaf6, SUC_lexicon_tags_25 },
    { 0x857eeaf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x463eeafa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c9eeafc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x65feeafe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141eeb04, SUC_lexicon_tags_37 },
    { 0xf93eeb05, SUC_lexicon_tags_7 },
    { 0x7bfeeb06, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdeeb0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef1eeb0d, SUC_lexicon_tags_37 },
    { 0xbfdeeb0e, SUC_lexicon_tags_87 },
    { 0x8a9eeb0f, SUC_lexicon_tags_37 },
    { 0x94deeb10, SUC_lexicon_tags_11 },
    { 0x629eeb0f, SUC_lexicon_tags_37 },
    { 0x271eeb0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7eeb15, SUC_lexicon_tags_70 },
    { 0xcfbeeb16, SUC_lexicon_tags_21 },
    { 0x5a5eeb17, SUC_lexicon_tags_10 },
    { 0xc63eeb18, SUC_lexicon_tags_33 },
    { 0xaa9eeb17, SUC_lexicon_tags_19 },
    { 0xa79eeb19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x817eeb1c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5eeb1f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81feeb23, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9eeb28, SUC_lexicon_tags_16 },
    { 0xe8feeb29, SUC_lexicon_tags_12 },
    { 0xa81eeb28, SUC_lexicon_tags_29 },
    { 0xfc1eeb2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7eeb37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733eeb3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0beeb41, SUC_lexicon_tags_2 },
    { 0xaabeeb42, SUC_lexicon_tags_2 },
    { 0x9d5eeb42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5eeb46, SUC_lexicon_tags_87 },
    { 0xe1eeb47, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x181eeb49, SUC_lexicon_tags_11 },
    { 0xb41eeb4a, SUC_lexicon_tags_28 },
    { 0x2d1eeb4a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59beeb50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1deeb54, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3eeb5d, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7eeb62, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3beeb65, SUC_lexicon_tags_7 },
    { 0xca9eeb65, SUC_lexicon_tags_39 },
    { 0x893eeb67, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9eeb6a, SUC_lexicon_tags_17 },
    { 0xaa1eeb6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb81eeb6d, SUC_lexicon_tags_29 },
    { 0x5dfeeb6e, SUC_lexicon_tags_25 },
    { 0xccfeeb6f, SUC_lexicon_tags_11 },
    { 0x42beeb6f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb95eeb72, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x90beeb75, SUC_lexicon_tags_7 },
    { 0xc8beeb76, SUC_lexicon_tags_965 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179eeb7a, SUC_lexicon_tags_43 },
    { 0x871eeb7a, SUC_lexicon_tags_2 },
    { 0xaf1eeb7a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x203eeb7e, SUC_lexicon_tags_1 },
    { 0xc73eeb7f, SUC_lexicon_tags_12 },
    { 0xf29eeb7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621eeb84, SUC_lexicon_tags_174 },
    { 0xec3eeb85, SUC_lexicon_tags_11 },
    { 0xa2feeb86, SUC_lexicon_tags_6 },
    { 0xc23eeb84, SUC_lexicon_tags_15 },
    { 0xdadeeb88, SUC_lexicon_tags_2 },
    { 0xc21eeb84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd79eeb92, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdeeb97, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x377eeb9a, SUC_lexicon_tags_30 },
    { 0x451eeb9a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x83feeb9d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x99deeba0, SUC_lexicon_tags_25 },
    { 0x5f9eeba1, SUC_lexicon_tags_39 },
    { 0xd53eeba2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5eeba6, SUC_lexicon_tags_4 },
    { 0x599eeba7, SUC_lexicon_tags_13 },
    { 0x4dbeeba7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0feebae, SUC_lexicon_tags_12 },
    { 0xd71eebaf, SUC_lexicon_tags_0 },
    { 0x54beebaf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x725eebb3, SUC_lexicon_tags_12 },
    { 0x863eebb3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3feebbb, SUC_lexicon_tags_8 },
    { 0xe2beebbb, SUC_lexicon_tags_106 },
    { 0x65feebbc, SUC_lexicon_tags_32 },
    { 0x539eebbe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbeebc6, SUC_lexicon_tags_14 },
    { 0x851eebc7, SUC_lexicon_tags_32 },
    { 0x5e3eebc6, SUC_lexicon_tags_11 },
    { 0x9afeebc9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x4c7eebcb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abeebd0, SUC_lexicon_tags_15 },
    { 0xcb7eebd1, SUC_lexicon_tags_17 },
    { 0xc89eebd2, SUC_lexicon_tags_25 },
    { 0x48beebd3, SUC_lexicon_tags_25 },
    { 0xae5eebd4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x71eebd7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc3eebd9, SUC_lexicon_tags_25 },
    { 0x8b9eebd9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7adeebdc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bfeebdf, SUC_lexicon_tags_60 },
    { 0xfffeebdf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb1beebe2, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9feebe5, SUC_lexicon_tags_38 },
    { 0x7d1eebe6, SUC_lexicon_tags_11 },
    { 0x869eebe6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921eebeb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47deebef, SUC_lexicon_tags_27 },
    { 0x329eebf0, SUC_lexicon_tags_63 },
    { 0xce3eebf1, SUC_lexicon_tags_49 },
    { 0x6b7eebf1, SUC_lexicon_tags_2 },
    { 0x297eebef, SUC_lexicon_tags_15 },
    { 0xae1eebf4, SUC_lexicon_tags_25 },
    { 0xd1feebef, SUC_lexicon_tags_10 },
    { 0x531eebef, SUC_lexicon_tags_12 },
    { 0x4f1eebef, SUC_lexicon_tags_37 },
    { 0xde9eebf2, SUC_lexicon_tags_37 },
    { 0x181eebf9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26feebfe, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267eec04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf67eec06, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101eec0d, SUC_lexicon_tags_8 },
    { 0x19beec0e, SUC_lexicon_tags_37 },
    { 0xb4feec0f, SUC_lexicon_tags_114 },
    { 0xf99eec0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf5beec12, SUC_lexicon_tags_17 },
    { 0x38deec13, SUC_lexicon_tags_37 },
    { 0xbabeec14, SUC_lexicon_tags_7 },
    { 0x7bfeec15, SUC_lexicon_tags_13 },
    { 0xf65eec16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649eec1d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4c3eec1f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7c1eec21, SUC_lexicon_tags_170 },
    { 0x307eec22, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7eec25, SUC_lexicon_tags_7 },
    { 0x13eec25, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb37eec28, SUC_lexicon_tags_10 },
    { 0x531eec28, SUC_lexicon_tags_2 },
    { 0xb23eec2a, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x8f9eec2c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333eec30, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9eec34, SUC_lexicon_tags_2 },
    { 0x3f7eec35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195eec3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6deec3f, SUC_lexicon_tags_65 },
    { 0x4f5eec40, SUC_lexicon_tags_10 },
    { 0x9fbeec41, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89eec47, SUC_lexicon_tags_14 },
    { 0x241eec48, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9a5eec4a, SUC_lexicon_tags_17 },
    { 0xdc5eec4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c1eec4d, SUC_lexicon_tags_63 },
    { 0x963eec4e, SUC_lexicon_tags_60 },
    { 0x1f9eec4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5eec53, SUC_lexicon_tags_17 },
    { 0xa8beec54, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441eec58, SUC_lexicon_tags_12 },
    { 0x283eec59, SUC_lexicon_tags_2 },
    { 0x345eec5a, SUC_lexicon_tags_107 },
    { 0xba1eec5b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x56beec5e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadeec62, SUC_lexicon_tags_17 },
    { 0xe45eec62, SUC_lexicon_tags_6 },
    { 0x917eec64, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36deec6a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x539eec6c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4bbeec6e, SUC_lexicon_tags_47 },
    { 0x14beec6f, SUC_lexicon_tags_7 },
    { 0x9edeec6f, SUC_lexicon_tags_11 },
    { 0x8d3eec6f, SUC_lexicon_tags_11 },
    { 0x991eec6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1eec77, SUC_lexicon_tags_19 },
    { 0xf9beec78, SUC_lexicon_tags_1 },
    { 0x799eec79, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5eec7f, SUC_lexicon_tags_14 },
    { 0x6b3eec80, SUC_lexicon_tags_30 },
    { 0x3beec7f, SUC_lexicon_tags_26 },
    { 0x663eec80, SUC_lexicon_tags_25 },
    { 0x56feec83, SUC_lexicon_tags_37 },
    { 0xe77eec83, SUC_lexicon_tags_60 },
    { 0xa95eec81, SUC_lexicon_tags_28 },
    { 0x889eec7f, SUC_lexicon_tags_14 },
    { 0x279eec87, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83beec8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629eec99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9eec9c, SUC_lexicon_tags_15 },
    { 0x72beec9d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3eeca0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201eeca4, SUC_lexicon_tags_27 },
    { 0xcbdeeca4, SUC_lexicon_tags_45 },
    { 0x7d3eeca4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8beecab, SUC_lexicon_tags_11 },
    { 0x927eecac, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeecb8, SUC_lexicon_tags_49 },
    { 0x733eecb8, SUC_lexicon_tags_11 },
    { 0x869eecba, SUC_lexicon_tags_7 },
    { 0xefbeecba, SUC_lexicon_tags_11 },
    { 0xa09eecbb, SUC_lexicon_tags_29 },
    { 0xbf5eecbd, SUC_lexicon_tags_37 },
    { 0xfddeecbd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xb8deecc0, SUC_lexicon_tags_4 },
    { 0x7c1eecc1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdeecc4, SUC_lexicon_tags_35 },
    { 0x195eecc5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3eeccc, SUC_lexicon_tags_31 },
    { 0xbefeeccc, SUC_lexicon_tags_136 },
    { 0x337eecce, SUC_lexicon_tags_4 },
    { 0x53deeccd, SUC_lexicon_tags_45 },
    { 0x44beecd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07eecd7, SUC_lexicon_tags_12 },
    { 0x795eecd8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfbeecda, SUC_lexicon_tags_43 },
    { 0xdd1eecda, SUC_lexicon_tags_1297 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263eecdf, SUC_lexicon_tags_51 },
    { 0x751eecdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x821eece3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5eece7, SUC_lexicon_tags_38 },
    { 0x6d1eece8, SUC_lexicon_tags_7 },
    { 0xfefeece8, SUC_lexicon_tags_12 },
    { 0xaebeece7, SUC_lexicon_tags_10 },
    { 0x88beece8, SUC_lexicon_tags_14 },
    { 0x787eecea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1feecf0, SUC_lexicon_tags_2 },
    { 0x123eecf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7deecf5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x9fdeecf7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x165eecf9, SUC_lexicon_tags_37 },
    { 0x417eecfa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9deecfe, SUC_lexicon_tags_0 },
    { 0x39eecfe, SUC_lexicon_tags_17 },
    { 0xf7beed00, SUC_lexicon_tags_6 },
    { 0xd37eed01, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581eed05, SUC_lexicon_tags_0 },
    { 0x8afeed05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17eed0c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99eed12, SUC_lexicon_tags_21 },
    { 0x9e9eed12, SUC_lexicon_tags_29 },
    { 0x479eed14, SUC_lexicon_tags_11 },
    { 0x52beed15, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1eed18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3eed1b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdeed20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cfeed23, SUC_lexicon_tags_56 },
    { 0xf4beed24, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4deed2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xab3eed2f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5eed33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6adeed35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x663eed3a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfafeed3c, SUC_lexicon_tags_11 },
    { 0xb3feed3c, SUC_lexicon_tags_37 },
    { 0x497eed3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd27eed40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1eed44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x801eed46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x907eed48, SUC_lexicon_tags_17 },
    { 0xb19eed49, SUC_lexicon_tags_2 },
    { 0xb3feed4a, SUC_lexicon_tags_32 },
    { 0x5a5eed4b, SUC_lexicon_tags_11 },
    { 0x2dfeed4b, SUC_lexicon_tags_2 },
    { 0xf19eed49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x829eed50, SUC_lexicon_tags_85 },
    { 0xae3eed51, SUC_lexicon_tags_55 },
    { 0x779eed52, SUC_lexicon_tags_17 },
    { 0x82feed51, SUC_lexicon_tags_15 },
    { 0x90beed50, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1eed57, SUC_lexicon_tags_60 },
    { 0xbfdeed57, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x233eed5a, SUC_lexicon_tags_57 },
    { 0x2c3eed5b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa61eed5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3f5eed5f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9bfeed61, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfeed65, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5eed6c, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x22beed6f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6deed71, SUC_lexicon_tags_2 },
    { 0x1a5eed72, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x585eed75, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x4bbeed77, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb3eed79, SUC_lexicon_tags_25 },
    { 0x679eed7a, SUC_lexicon_tags_37 },
    { 0x25beed7b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4a7eed7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50deed81, SUC_lexicon_tags_15 },
    { 0xb17eed82, SUC_lexicon_tags_4 },
    { 0x417eed82, SUC_lexicon_tags_27 },
    { 0x251eed84, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7eed87, SUC_lexicon_tags_17 },
    { 0x925eed88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x555eed8a, SUC_lexicon_tags_38 },
    { 0x861eed8a, SUC_lexicon_tags_12 },
    { 0x95deed8a, SUC_lexicon_tags_15 },
    { 0x243eed8d, SUC_lexicon_tags_0 },
    { 0x639eed8e, SUC_lexicon_tags_7 },
    { 0xd25eed8e, SUC_lexicon_tags_15 },
    { 0xd71eed8f, SUC_lexicon_tags_11 },
    { 0xa43eed91, SUC_lexicon_tags_138 },
    { 0xaf9eed91, SUC_lexicon_tags_37 },
    { 0x701eed93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35eed97, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7feed9f, SUC_lexicon_tags_31 },
    { 0x43deed9f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3eeda5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xae1eeda8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3deedb1, SUC_lexicon_tags_1 },
    { 0xb4deedb1, SUC_lexicon_tags_101 },
    { 0x943eedb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99feedb8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedeedbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11eedc0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3eedc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x235eedc8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x151eedcb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x19eedcd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf8beedcf, SUC_lexicon_tags_1 },
    { 0x369eedd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17eedd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddeedd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x78deedd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeb3eedda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2bdeeddc, SUC_lexicon_tags_53 },
    { 0xfa5eeddc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5eede5, SUC_lexicon_tags_11 },
    { 0xdf3eede6, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xcefeede8, SUC_lexicon_tags_35 },
    { 0x9c3eede9, SUC_lexicon_tags_37 },
    { 0x8c7eede9, SUC_lexicon_tags_31 },
    { 0x669eedeb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x191eeded, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edeedf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629eedf4, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1eedf9, SUC_lexicon_tags_35 },
    { 0x4afeedf9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1deedfd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3eee00, SUC_lexicon_tags_23 },
    { 0x9c3eee01, SUC_lexicon_tags_2 },
    { 0x96feee01, SUC_lexicon_tags_49 },
    { 0x837eee02, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0x43beee05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67eee0b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6deee0e, SUC_lexicon_tags_25 },
    { 0x615eee0f, SUC_lexicon_tags_17 },
    { 0x997eee0f, SUC_lexicon_tags_17 },
    { 0xa47eee0e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5eee19, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91feee1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3abeee1f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9feee29, SUC_lexicon_tags_47 },
    { 0xa67eee2a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3eee30, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5eee36, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1f9eee38, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1eee3e, SUC_lexicon_tags_11 },
    { 0x32deee3e, SUC_lexicon_tags_37 },
    { 0x71beee3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3eee43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x60beee45, SUC_lexicon_tags_17 },
    { 0xaf5eee46, SUC_lexicon_tags_47 },
    { 0xbe3eee46, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3bfeee49, SUC_lexicon_tags_49 },
    { 0x797eee4a, SUC_lexicon_tags_37 },
    { 0xf3eee4b, SUC_lexicon_tags_2 },
    { 0x291eee49, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3a3eee4e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7eee51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43eee54, SUC_lexicon_tags_87 },
    { 0x39deee54, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9deee57, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x559eee5a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8d1eee5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849eee60, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd97eee62, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961eee66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4b7eee68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3eee6e, SUC_lexicon_tags_60 },
    { 0x6a3eee6f, SUC_lexicon_tags_30 },
    { 0x377eee6f, SUC_lexicon_tags_8 },
    { 0xeadeee70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17eee76, SUC_lexicon_tags_17 },
    { 0x63feee77, SUC_lexicon_tags_23 },
    { 0xeefeee78, SUC_lexicon_tags_2 },
    { 0xc85eee76, SUC_lexicon_tags_11 },
    { 0x42beee79, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6beee7d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9feee80, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18beee85, SUC_lexicon_tags_11 },
    { 0x34beee86, SUC_lexicon_tags_7 },
    { 0x4efeee87, SUC_lexicon_tags_15 },
    { 0xd9deee88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197eee9a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279eee9e, SUC_lexicon_tags_49 },
    { 0xea5eee9f, SUC_lexicon_tags_84 },
    { 0xe33eee9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb73eeea2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7eeea5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x63deeea7, SUC_lexicon_tags_37 },
    { 0x275eeea7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a5eeeaa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1eeeae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb6beeeb0, SUC_lexicon_tags_569 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965eeeb4, SUC_lexicon_tags_45 },
    { 0xac5eeeb5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x31deeeb8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75feeebd, SUC_lexicon_tags_37 },
    { 0xd45eeebe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9eeec5, SUC_lexicon_tags_14 },
    { 0xfd9eeec6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3eeecd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x861eeecf, SUC_lexicon_tags_1 },
    { 0xcdbeeed0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4bdeeed2, SUC_lexicon_tags_65 },
    { 0xca9eeed3, SUC_lexicon_tags_15 },
    { 0x545eeed4, SUC_lexicon_tags_49 },
    { 0x765eeed5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc5beeed7, SUC_lexicon_tags_57 },
    { 0x335eeed8, SUC_lexicon_tags_30 },
    { 0xb2beeed8, SUC_lexicon_tags_17 },
    { 0x4b7eeeda, SUC_lexicon_tags_47 },
    { 0x3ffeeed7, SUC_lexicon_tags_25 },
    { 0xfc5eeeda, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x32deeedf, SUC_lexicon_tags_12 },
    { 0x75feeee0, SUC_lexicon_tags_14 },
    { 0x817eeee1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7feeee3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x26beeee5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34beeeea, SUC_lexicon_tags_27 },
    { 0xaa5eeeeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdeeef0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85eeef3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbeeef9, SUC_lexicon_tags_7 },
    { 0x47deeefa, SUC_lexicon_tags_17 },
    { 0x41deeefb, SUC_lexicon_tags_2 },
    { 0x3b3eeef9, SUC_lexicon_tags_37 },
    { 0x19deeefd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefeef00, SUC_lexicon_tags_17 },
    { 0x4c5eef01, SUC_lexicon_tags_37 },
    { 0x973eef02, SUC_lexicon_tags_1 },
    { 0x671eef02, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe89eef05, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0deef08, SUC_lexicon_tags_23 },
    { 0x2c9eef08, SUC_lexicon_tags_32 },
    { 0xf9feef0a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdeef0d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19eef10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x55beef12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x97feef14, SUC_lexicon_tags_45 },
    { 0xff9eef15, SUC_lexicon_tags_11 },
    { 0xe67eef15, SUC_lexicon_tags_11 },
    { 0x511eef17, SUC_lexicon_tags_12 },
    { 0xa3deef18, SUC_lexicon_tags_19 },
    { 0x9efeef19, SUC_lexicon_tags_4 },
    { 0x615eef19, SUC_lexicon_tags_33 },
    { 0x489eef1b, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53eef1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1dbeef20, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885eef27, SUC_lexicon_tags_23 },
    { 0x68beef28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3eef2f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x549eef32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x307eef34, SUC_lexicon_tags_17 },
    { 0x721eef35, SUC_lexicon_tags_12 },
    { 0x42beef36, SUC_lexicon_tags_61 },
    { 0x1eef36, SUC_lexicon_tags_60 },
    { 0x8ebeef38, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe75eef3a, SUC_lexicon_tags_26 },
    { 0xbd3eef3b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x1a9eef3d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad1eef42, SUC_lexicon_tags_33 },
    { 0xd87eef43, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x689eef45, SUC_lexicon_tags_17 },
    { 0xa55eef46, SUC_lexicon_tags_116 },
    { 0x191eef45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x30feef49, SUC_lexicon_tags_49 },
    { 0x49feef49, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x885eef4c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227eef58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e7eef5a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3eef5e, SUC_lexicon_tags_39 },
    { 0xeb7eef5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20deef61, SUC_lexicon_tags_37 },
    { 0xa67eef62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437eef68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107eef6c, SUC_lexicon_tags_45 },
    { 0x89beef6d, SUC_lexicon_tags_4 },
    { 0xa27eef6e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffeef74, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf27eef76, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0beef7d, SUC_lexicon_tags_0 },
    { 0x43deef7d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7eef81, SUC_lexicon_tags_7 },
    { 0x125eef82, SUC_lexicon_tags_14 },
    { 0x2edeef83, SUC_lexicon_tags_55 },
    { 0x85beef82, SUC_lexicon_tags_47 },
    { 0x6feef84, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffeef88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf45eef8a, SUC_lexicon_tags_25 },
    { 0xec5eef8a, SUC_lexicon_tags_21 },
    { 0x25eef8c, SUC_lexicon_tags_91 },
    { 0x521eef8c, SUC_lexicon_tags_1 },
    { 0xff3eef8c, SUC_lexicon_tags_84 },
    { 0x5a5eef8b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5eef95, SUC_lexicon_tags_25 },
    { 0x169eef96, SUC_lexicon_tags_30 },
    { 0x75feef95, SUC_lexicon_tags_12 },
    { 0x323eef98, SUC_lexicon_tags_23 },
    { 0xf7eef99, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659eef9d, SUC_lexicon_tags_15 },
    { 0x93feef9e, SUC_lexicon_tags_17 },
    { 0xe19eef9f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7eefa2, SUC_lexicon_tags_7 },
    { 0x83deefa3, SUC_lexicon_tags_11 },
    { 0x587eefa4, SUC_lexicon_tags_17 },
    { 0xcfbeefa5, SUC_lexicon_tags_7 },
    { 0x311eefa3, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xf9beefa8, SUC_lexicon_tags_15 },
    { 0x26deefa9, SUC_lexicon_tags_18 },
    { 0xfe5eefa9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc81eefac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xec5eefaf, SUC_lexicon_tags_61 },
    { 0x69eefb0, SUC_lexicon_tags_1 },
    { 0x37deefb1, SUC_lexicon_tags_0 },
    { 0xd2beefb1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46deefb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d9eefb7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68beefb9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xde7eefbb, SUC_lexicon_tags_63 },
    { 0x989eefbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3eefc0, SUC_lexicon_tags_12 },
    { 0xfffeefc1, SUC_lexicon_tags_31 },
    { 0xde1eefc1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81eefc5, SUC_lexicon_tags_38 },
    { 0xafeefc6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb43eefc8, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xdb1eefca, SUC_lexicon_tags_11 },
    { 0x54feefcb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x977eefce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x985eefd1, SUC_lexicon_tags_60 },
    { 0x7adeefd2, SUC_lexicon_tags_12 },
    { 0x99deefd1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7eefd7, SUC_lexicon_tags_25 },
    { 0x7c3eefd8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdeefdc, SUC_lexicon_tags_63 },
    { 0xcf3eefdd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x935eefdf, SUC_lexicon_tags_2 },
    { 0xae5eefe0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97eefe3, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3eefe6, SUC_lexicon_tags_19 },
    { 0x579eefe7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989eefec, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01eeff2, SUC_lexicon_tags_11 },
    { 0xd61eeff2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbeeff7, SUC_lexicon_tags_87 },
    { 0x6b5eeff8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x6bfeeffa, SUC_lexicon_tags_2 },
    { 0x6ebeeffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3ef003, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5def00a, SUC_lexicon_tags_19 },
    { 0x837ef00a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb93ef00d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x107ef00f, SUC_lexicon_tags_68 },
    { 0x711ef010, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb9def012, SUC_lexicon_tags_39 },
    { 0xbefef013, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5ef018, SUC_lexicon_tags_11 },
    { 0xdc1ef019, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821ef01d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bef021, SUC_lexicon_tags_30 },
    { 0x497ef022, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x583ef024, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb8fef026, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7ef02d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3ef030, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb97ef032, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x945ef034, SUC_lexicon_tags_15 },
    { 0x88def035, SUC_lexicon_tags_8 },
    { 0xd51ef036, SUC_lexicon_tags_7 },
    { 0xb3ef034, SUC_lexicon_tags_16 },
    { 0xb8bef038, SUC_lexicon_tags_1 },
    { 0x9dbef035, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f1ef03b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdcdef03d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61ef043, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129ef049, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217ef04f, SUC_lexicon_tags_2 },
    { 0x2f3ef050, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273ef055, SUC_lexicon_tags_37 },
    { 0x48bef055, SUC_lexicon_tags_77 },
    { 0x1a9ef056, SUC_lexicon_tags_7 },
    { 0x6f3ef058, SUC_lexicon_tags_15 },
    { 0x7e7ef059, SUC_lexicon_tags_2 },
    { 0xb73ef059, SUC_lexicon_tags_17 },
    { 0x1fef059, SUC_lexicon_tags_6 },
    { 0xe37ef05c, SUC_lexicon_tags_16 },
    { 0xef3ef058, SUC_lexicon_tags_27 },
    { 0xc0def05e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x35def060, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6c7ef062, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13ef065, SUC_lexicon_tags_19 },
    { 0x6b7ef066, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x95def068, SUC_lexicon_tags_27 },
    { 0x327ef068, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x11def06b, SUC_lexicon_tags_25 },
    { 0x82def06c, SUC_lexicon_tags_47 },
    { 0x3bef06d, SUC_lexicon_tags_14 },
    { 0xb29ef06d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc21ef070, SUC_lexicon_tags_33 },
    { 0x157ef071, SUC_lexicon_tags_1 },
    { 0x1fdef072, SUC_lexicon_tags_7 },
    { 0xae9ef070, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51ef075, SUC_lexicon_tags_45 },
    { 0x36fef075, SUC_lexicon_tags_28 },
    { 0xbbfef077, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7ef082, SUC_lexicon_tags_60 },
    { 0xa21ef083, SUC_lexicon_tags_7 },
    { 0xc0fef084, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5ef089, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9afef08c, SUC_lexicon_tags_63 },
    { 0xf3ef08d, SUC_lexicon_tags_17 },
    { 0xbfdef08e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe47ef090, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x731ef092, SUC_lexicon_tags_47 },
    { 0x665ef093, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7ef09d, SUC_lexicon_tags_10 },
    { 0x37bef09e, SUC_lexicon_tags_11 },
    { 0xeefef09e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45def0a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439ef0a8, SUC_lexicon_tags_34 },
    { 0x885ef0a9, SUC_lexicon_tags_1 },
    { 0xe0def0aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3def0ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa7bef0ae, SUC_lexicon_tags_1 },
    { 0xc5bef0af, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337ef0b3, SUC_lexicon_tags_15 },
    { 0x599ef0b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3def0b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629ef0c0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3ef0c5, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x823ef0c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9ef0cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x49def0ce, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x30bef0d1, SUC_lexicon_tags_25 },
    { 0xe97ef0d1, SUC_lexicon_tags_25 },
    { 0x171ef0d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33ef0d9, SUC_lexicon_tags_12 },
    { 0xb25ef0d9, SUC_lexicon_tags_47 },
    { 0xe05ef0da, SUC_lexicon_tags_26 },
    { 0xde7ef0db, SUC_lexicon_tags_25 },
    { 0x8fef0dd, SUC_lexicon_tags_12 },
    { 0x6afef0dd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fef0e3, SUC_lexicon_tags_83 },
    { 0x661ef0e4, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129ef0e8, SUC_lexicon_tags_9 },
    { 0xf15ef0e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297ef0ee, SUC_lexicon_tags_28 },
    { 0x8b9ef0ee, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x809ef0f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1ef0f6, SUC_lexicon_tags_25 },
    { 0x26def0f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x15ef0f9, SUC_lexicon_tags_37 },
    { 0xbef0fa, SUC_lexicon_tags_23 },
    { 0xf47ef0f9, SUC_lexicon_tags_39 },
    { 0xac7ef0fc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x69ef0fe, SUC_lexicon_tags_50 },
    { 0xc73ef0ff, SUC_lexicon_tags_46 },
    { 0x4b9ef100, SUC_lexicon_tags_28 },
    { 0x73ef0fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x423ef103, SUC_lexicon_tags_11 },
    { 0x4edef103, SUC_lexicon_tags_0 },
    { 0x507ef103, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x43def107, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bef10e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa15ef111, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdef115, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59ef119, SUC_lexicon_tags_60 },
    { 0x78fef119, SUC_lexicon_tags_17 },
    { 0x371ef11b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe25ef11d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31ef121, SUC_lexicon_tags_34 },
    { 0x15fef122, SUC_lexicon_tags_14 },
    { 0x269ef122, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafef127, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x661ef12a, SUC_lexicon_tags_1 },
    { 0x393ef12a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x43bef12d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x847ef130, SUC_lexicon_tags_28 },
    { 0xe9def131, SUC_lexicon_tags_14 },
    { 0xb01ef131, SUC_lexicon_tags_30 },
    { 0xb45ef133, SUC_lexicon_tags_49 },
    { 0x625ef134, SUC_lexicon_tags_27 },
    { 0x3bef132, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x153ef137, SUC_lexicon_tags_19 },
    { 0xc7fef138, SUC_lexicon_tags_25 },
    { 0x3def139, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x391ef13c, SUC_lexicon_tags_27 },
    { 0xe1ef13d, SUC_lexicon_tags_47 },
    { 0x43ef13e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37ef141, SUC_lexicon_tags_19 },
    { 0xcc9ef141, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24fef146, SUC_lexicon_tags_14 },
    { 0xbfdef146, SUC_lexicon_tags_1 },
    { 0xad9ef148, SUC_lexicon_tags_37 },
    { 0x5cbef148, SUC_lexicon_tags_2 },
    { 0xc9bef14a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x387ef14c, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16fef151, SUC_lexicon_tags_11 },
    { 0xf69ef151, SUC_lexicon_tags_21 },
    { 0xb27ef151, SUC_lexicon_tags_2 },
    { 0x81bef154, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3ef159, SUC_lexicon_tags_2 },
    { 0x6fbef159, SUC_lexicon_tags_2 },
    { 0x43bef15a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ef15f, SUC_lexicon_tags_7 },
    { 0x813ef160, SUC_lexicon_tags_77 },
    { 0xdc7ef161, SUC_lexicon_tags_37 },
    { 0x53ef15f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc9ef164, SUC_lexicon_tags_329 },
    { 0x631ef164, SUC_lexicon_tags_45 },
    { 0xce7ef166, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbef16c, SUC_lexicon_tags_54 },
    { 0xf3bef16c, SUC_lexicon_tags_17 },
    { 0xde9ef16e, SUC_lexicon_tags_25 },
    { 0x45def16f, SUC_lexicon_tags_25 },
    { 0xb9def170, SUC_lexicon_tags_15 },
    { 0x485ef171, SUC_lexicon_tags_60 },
    { 0xfb3ef172, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3ef175, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc3ef177, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ef17a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x94bef17c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fef181, SUC_lexicon_tags_25 },
    { 0x6a1ef182, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x575ef184, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bef188, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657ef18e, SUC_lexicon_tags_1 },
    { 0x3a1ef18f, SUC_lexicon_tags_15 },
    { 0x1b5ef190, SUC_lexicon_tags_7 },
    { 0x31fef18f, SUC_lexicon_tags_12 },
    { 0x16bef192, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5e3ef194, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf9fef196, SUC_lexicon_tags_11 },
    { 0xee1ef197, SUC_lexicon_tags_6 },
    { 0x887ef198, SUC_lexicon_tags_11 },
    { 0xc9fef199, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x87def19b, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xe9fef19d, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53ef1aa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ef1b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bef1b6, SUC_lexicon_tags_23 },
    { 0xd3fef1b7, SUC_lexicon_tags_17 },
    { 0x97bef1b7, SUC_lexicon_tags_21 },
    { 0x6e3ef1b8, SUC_lexicon_tags_118 },
    { 0x3d7ef1ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc55ef1bc, SUC_lexicon_tags_12 },
    { 0x393ef1bc, SUC_lexicon_tags_31 },
    { 0x43bef1bd, SUC_lexicon_tags_11 },
    { 0xd1bef1bf, SUC_lexicon_tags_19 },
    { 0x97fef1bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611ef1c8, SUC_lexicon_tags_23 },
    { 0xaa1ef1c9, SUC_lexicon_tags_47 },
    { 0xaafef1c8, SUC_lexicon_tags_37 },
    { 0x97fef1cb, SUC_lexicon_tags_2 },
    { 0x71def1ca, SUC_lexicon_tags_6 },
    { 0x7d3ef1cd, SUC_lexicon_tags_2 },
    { 0xf77ef1ce, SUC_lexicon_tags_38 },
    { 0xeb1ef1cf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x121ef1d4, SUC_lexicon_tags_27 },
    { 0xfe7ef1d5, SUC_lexicon_tags_25 },
    { 0xda1ef1d6, SUC_lexicon_tags_25 },
    { 0x3bbef1d7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a7ef1d9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23ef1dc, SUC_lexicon_tags_6 },
    { 0xa6bef1dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295ef1e5, SUC_lexicon_tags_15 },
    { 0xfddef1e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bef1ea, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x463ef1ec, SUC_lexicon_tags_11 },
    { 0xb93ef1ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949ef1f1, SUC_lexicon_tags_56 },
    { 0xc99ef1f2, SUC_lexicon_tags_33 },
    { 0xba5ef1f3, SUC_lexicon_tags_75 },
    { 0xe53ef1f4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7ef1f8, SUC_lexicon_tags_37 },
    { 0x76def1f9, SUC_lexicon_tags_7 },
    { 0xa5ef1fa, SUC_lexicon_tags_12 },
    { 0x597ef1fa, SUC_lexicon_tags_65 },
    { 0xc63ef1f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd59ef1fe, SUC_lexicon_tags_37 },
    { 0x3c1ef1ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533ef205, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67ef208, SUC_lexicon_tags_13 },
    { 0x84fef209, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bef20c, SUC_lexicon_tags_37 },
    { 0x3bef20d, SUC_lexicon_tags_25 },
    { 0xb07ef20e, SUC_lexicon_tags_23 },
    { 0x9e3ef20e, SUC_lexicon_tags_7 },
    { 0x359ef20e, SUC_lexicon_tags_7 },
    { 0xb47ef20f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fef217, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fef21e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x219ef220, SUC_lexicon_tags_15 },
    { 0x8fdef220, SUC_lexicon_tags_2 },
    { 0xab3ef222, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89ef224, SUC_lexicon_tags_26 },
    { 0xe01ef225, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443ef229, SUC_lexicon_tags_25 },
    { 0x997ef22a, SUC_lexicon_tags_11 },
    { 0xafef22b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x43ef22e, SUC_lexicon_tags_14 },
    { 0x941ef22e, SUC_lexicon_tags_37 },
    { 0xc9bef230, SUC_lexicon_tags_65 },
    { 0x35ef230, SUC_lexicon_tags_15 },
    { 0xa25ef232, SUC_lexicon_tags_1 },
    { 0x993ef22e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8def237, SUC_lexicon_tags_11 },
    { 0xd87ef237, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x59def23b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x35ef23d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497ef241, SUC_lexicon_tags_6 },
    { 0x1e7ef242, SUC_lexicon_tags_31 },
    { 0x5d7ef242, SUC_lexicon_tags_87 },
    { 0xa67ef244, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fef248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x249ef24a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6def253, SUC_lexicon_tags_30 },
    { 0x2dbef254, SUC_lexicon_tags_37 },
    { 0x8dbef255, SUC_lexicon_tags_49 },
    { 0xb5fef256, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb27ef258, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc8bef25a, SUC_lexicon_tags_215 },
    { 0x23fef25b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bef261, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86fef267, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x70def26a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7d3ef26c, SUC_lexicon_tags_39 },
    { 0x209ef26c, SUC_lexicon_tags_14 },
    { 0xbd1ef26e, SUC_lexicon_tags_0 },
    { 0x5f9ef26e, SUC_lexicon_tags_2 },
    { 0xeadef270, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dfef273, SUC_lexicon_tags_65 },
    { 0x323ef274, SUC_lexicon_tags_9 },
    { 0x485ef275, SUC_lexicon_tags_7 },
    { 0x705ef276, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x73fef279, SUC_lexicon_tags_37 },
    { 0xcf1ef27a, SUC_lexicon_tags_9 },
    { 0xfdbef27b, SUC_lexicon_tags_27 },
    { 0xb03ef27c, SUC_lexicon_tags_43 },
    { 0x8abef27d, SUC_lexicon_tags_17 },
    { 0xd73ef27a, SUC_lexicon_tags_9 },
    { 0xb65ef27f, SUC_lexicon_tags_17 },
    { 0x4ebef27b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadef285, SUC_lexicon_tags_37 },
    { 0x157ef286, SUC_lexicon_tags_15 },
    { 0xc27ef287, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9c3ef289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943ef293, SUC_lexicon_tags_15 },
    { 0xf0fef293, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc35ef296, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55ef29a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9ef29d, SUC_lexicon_tags_23 },
    { 0xe6def29e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xec7ef2a1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59def2a3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x1f3ef2a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9ef2a8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bef2b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebef2ba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c3ef2bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x227ef2be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1ef2c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x88def2c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bef2cc, SUC_lexicon_tags_37 },
    { 0x45fef2cd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67ef2d7, SUC_lexicon_tags_45 },
    { 0xfcbef2d8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x501ef2db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbef2de, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb7ef2e0, SUC_lexicon_tags_17 },
    { 0x305ef2e1, SUC_lexicon_tags_23 },
    { 0xa7ef2e2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31ef2e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe87ef2ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x23fef2ec, SUC_lexicon_tags_15 },
    { 0x447ef2ed, SUC_lexicon_tags_7 },
    { 0x8dbef2ee, SUC_lexicon_tags_2 },
    { 0xa6fef2ee, SUC_lexicon_tags_31 },
    { 0xe33ef2ed, SUC_lexicon_tags_37 },
    { 0xa3bef2ef, SUC_lexicon_tags_2 },
    { 0x8fbef2f2, SUC_lexicon_tags_19 },
    { 0x391ef2ec, SUC_lexicon_tags_31 },
    { 0x5f3ef2f4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13ef2f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdffef2fb, SUC_lexicon_tags_1 },
    { 0xed3ef2fc, SUC_lexicon_tags_57 },
    { 0x3c9ef2fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x54def2ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x435ef302, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5ef305, SUC_lexicon_tags_28 },
    { 0xe2bef306, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf9ef308, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa97ef30a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9ef30e, SUC_lexicon_tags_27 },
    { 0xe23ef30f, SUC_lexicon_tags_7 },
    { 0xbb1ef310, SUC_lexicon_tags_27 },
    { 0x31fef311, SUC_lexicon_tags_1 },
    { 0x1afef312, SUC_lexicon_tags_29 },
    { 0xa1ef313, SUC_lexicon_tags_44 },
    { 0xc03ef312, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaffef318, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1afef31b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xeebef31d, SUC_lexicon_tags_45 },
    { 0x47fef31e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8def327, SUC_lexicon_tags_33 },
    { 0xb83ef328, SUC_lexicon_tags_17 },
    { 0xf57ef329, SUC_lexicon_tags_43 },
    { 0xeafef32a, SUC_lexicon_tags_2 },
    { 0x2cbef329, SUC_lexicon_tags_23 },
    { 0xfa1ef32b, SUC_lexicon_tags_17 },
    { 0x2fbef32d, SUC_lexicon_tags_12 },
    { 0xff3ef32c, SUC_lexicon_tags_63 },
    { 0x65fef32c, SUC_lexicon_tags_25 },
    { 0x56fef328, SUC_lexicon_tags_45 },
    { 0x439ef32f, SUC_lexicon_tags_7 },
    { 0xf83ef332, SUC_lexicon_tags_15 },
    { 0x469ef332, SUC_lexicon_tags_14 },
    { 0xcc5ef334, SUC_lexicon_tags_15 },
    { 0x4f7ef332, SUC_lexicon_tags_23 },
    { 0x973ef336, SUC_lexicon_tags_14 },
    { 0xf45ef337, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453ef33f, SUC_lexicon_tags_33 },
    { 0x365ef340, SUC_lexicon_tags_33 },
    { 0x6c5ef341, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adef348, SUC_lexicon_tags_17 },
    { 0x5afef349, SUC_lexicon_tags_37 },
    { 0x15fef349, SUC_lexicon_tags_77 },
    { 0x403ef348, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x187ef34d, SUC_lexicon_tags_63 },
    { 0x67def34e, SUC_lexicon_tags_38 },
    { 0xaf1ef34f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bef35b, SUC_lexicon_tags_11 },
    { 0x5c7ef35b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1def360, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361ef367, SUC_lexicon_tags_9 },
    { 0x2a3ef367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d5ef36a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbef36e, SUC_lexicon_tags_16 },
    { 0x459ef36f, SUC_lexicon_tags_37 },
    { 0xe7bef370, SUC_lexicon_tags_60 },
    { 0x597ef370, SUC_lexicon_tags_37 },
    { 0x157ef372, SUC_lexicon_tags_25 },
    { 0x8e9ef372, SUC_lexicon_tags_10 },
    { 0xe95ef374, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xec1ef376, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7bfef378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb21ef37b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x63ef37d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x881ef37f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbef383, SUC_lexicon_tags_1 },
    { 0x755ef383, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc37ef386, SUC_lexicon_tags_7 },
    { 0xd43ef387, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe43ef389, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83ef38e, SUC_lexicon_tags_12 },
    { 0xb13ef38f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x341ef391, SUC_lexicon_tags_12 },
    { 0x9bdef392, SUC_lexicon_tags_17 },
    { 0x9a3ef393, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8def398, SUC_lexicon_tags_17 },
    { 0x801ef399, SUC_lexicon_tags_2 },
    { 0x781ef399, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5ef3a2, SUC_lexicon_tags_82 },
    { 0x29ef3a3, SUC_lexicon_tags_12 },
    { 0x9abef3a2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x84bef3a6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9ef3ae, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfef3b4, SUC_lexicon_tags_37 },
    { 0x21fef3b5, SUC_lexicon_tags_37 },
    { 0x819ef3b6, SUC_lexicon_tags_27 },
    { 0x3a9ef3b7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc69ef3b9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5ef3be, SUC_lexicon_tags_17 },
    { 0xc3fef3be, SUC_lexicon_tags_37 },
    { 0xa4def3c0, SUC_lexicon_tags_6 },
    { 0x5cbef3c0, SUC_lexicon_tags_2 },
    { 0x24def3c0, SUC_lexicon_tags_25 },
    { 0xf6fef3c3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3def3c5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebef3c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x749ef3cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737ef3d2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3ef3d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x48bef3d8, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xdbfef3da, SUC_lexicon_tags_17 },
    { 0x7c7ef3db, SUC_lexicon_tags_37 },
    { 0x65ef3dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x393ef3e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9def3e8, SUC_lexicon_tags_19 },
    { 0xafef3e9, SUC_lexicon_tags_49 },
    { 0xac5ef3e8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9ef3ee, SUC_lexicon_tags_25 },
    { 0x965ef3ef, SUC_lexicon_tags_43 },
    { 0xa79ef3f0, SUC_lexicon_tags_28 },
    { 0x58def3f1, SUC_lexicon_tags_11 },
    { 0xe19ef3f2, SUC_lexicon_tags_37 },
    { 0xe01ef3ee, SUC_lexicon_tags_63 },
    { 0xb15ef3f2, SUC_lexicon_tags_30 },
    { 0xbcbef3f0, SUC_lexicon_tags_25 },
    { 0xf4fef3f6, SUC_lexicon_tags_32 },
    { 0x9dbef3f7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2ddef3f9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbef3ff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb93ef401, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abef406, SUC_lexicon_tags_17 },
    { 0x537ef407, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983ef40c, SUC_lexicon_tags_11 },
    { 0x909ef40d, SUC_lexicon_tags_37 },
    { 0xce3ef40c, SUC_lexicon_tags_11 },
    { 0x511ef40e, SUC_lexicon_tags_27 },
    { 0x213ef410, SUC_lexicon_tags_25 },
    { 0x61fef411, SUC_lexicon_tags_2 },
    { 0x471ef411, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99ef416, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdef41c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfafef41f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x39fef421, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5ef425, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0xf91ef427, SUC_lexicon_tags_57 },
    { 0x9dfef428, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bef42b, SUC_lexicon_tags_79 },
    { 0xd41ef42b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53def430, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6ddef432, SUC_lexicon_tags_8 },
    { 0x703ef433, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd3ef435, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddef439, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33ef442, SUC_lexicon_tags_65 },
    { 0x509ef443, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5a3ef445, SUC_lexicon_tags_46 },
    { 0x415ef446, SUC_lexicon_tags_37 },
    { 0x42bef445, SUC_lexicon_tags_39 },
    { 0x893ef448, SUC_lexicon_tags_23 },
    { 0x3b1ef449, SUC_lexicon_tags_19 },
    { 0xf33ef449, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05ef44d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x93bef44f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5fbef451, SUC_lexicon_tags_45 },
    { 0xe91ef451, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93ef455, SUC_lexicon_tags_212 },
    { 0x567ef456, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbef459, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ef45c, SUC_lexicon_tags_38 },
    { 0xa57ef45c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09ef464, SUC_lexicon_tags_17 },
    { 0x6b5ef464, SUC_lexicon_tags_28 },
    { 0xeddef465, SUC_lexicon_tags_17 },
    { 0xff7ef465, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9ef46c, SUC_lexicon_tags_17 },
    { 0xe33ef46c, SUC_lexicon_tags_2 },
    { 0xb4fef46c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf97ef470, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f5ef472, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53ef475, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc69ef477, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf6fef479, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9ef480, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ef483, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbf9ef485, SUC_lexicon_tags_9 },
    { 0x103ef486, SUC_lexicon_tags_27 },
    { 0x995ef486, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfef48a, SUC_lexicon_tags_3 },
    { 0x71ef48b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x455ef48d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563ef495, SUC_lexicon_tags_600 },
    { 0, NULL },
    { 0x851ef497, SUC_lexicon_tags_37 },
    { 0x669ef497, SUC_lexicon_tags_17 },
    { 0xe29ef499, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adef4a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6c7ef4a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9ef4a9, SUC_lexicon_tags_17 },
    { 0xa81ef4a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfef4af, SUC_lexicon_tags_39 },
    { 0x42def4b0, SUC_lexicon_tags_16 },
    { 0x8cfef4af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443ef4b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x90bef4b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc7bef4ba, SUC_lexicon_tags_68 },
    { 0x93fef4bb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8e5ef4bd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x40fef4bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbef4c6, SUC_lexicon_tags_45 },
    { 0x7c7ef4c7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc9ef4c9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71ef4cf, SUC_lexicon_tags_43 },
    { 0x623ef4cf, SUC_lexicon_tags_38 },
    { 0xcd9ef4d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x775ef4d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xecfef4d7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1ef4db, SUC_lexicon_tags_56 },
    { 0x133ef4dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33ef4e0, SUC_lexicon_tags_19 },
    { 0x655ef4e0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fbef4e3, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689ef4e8, SUC_lexicon_tags_15 },
    { 0x7def4e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7ef4f5, SUC_lexicon_tags_37 },
    { 0xa73ef4f6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfef4f9, SUC_lexicon_tags_17 },
    { 0x6c1ef4fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x22fef4fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x909ef4fe, SUC_lexicon_tags_61 },
    { 0x8b7ef4fe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653ef508, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7def50b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1ef517, SUC_lexicon_tags_27 },
    { 0x107ef518, SUC_lexicon_tags_49 },
    { 0xe3def519, SUC_lexicon_tags_43 },
    { 0x75fef51a, SUC_lexicon_tags_8 },
    { 0x1a1ef51b, SUC_lexicon_tags_22 },
    { 0x2f1ef51c, SUC_lexicon_tags_47 },
    { 0x8def51d, SUC_lexicon_tags_25 },
    { 0x2e9ef518, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17ef521, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe31ef523, SUC_lexicon_tags_61 },
    { 0x44fef524, SUC_lexicon_tags_8 },
    { 0xeb7ef525, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x219ef527, SUC_lexicon_tags_39 },
    { 0x925ef528, SUC_lexicon_tags_37 },
    { 0xa4bef529, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x493ef52b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x16def52e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7ef531, SUC_lexicon_tags_0 },
    { 0xdbdef532, SUC_lexicon_tags_2 },
    { 0x27ef533, SUC_lexicon_tags_15 },
    { 0xac7ef532, SUC_lexicon_tags_25 },
    { 0x84fef535, SUC_lexicon_tags_37 },
    { 0xc43ef534, SUC_lexicon_tags_47 },
    { 0x9a9ef533, SUC_lexicon_tags_10 },
    { 0xa03ef538, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfef53b, SUC_lexicon_tags_28 },
    { 0x26fef53b, SUC_lexicon_tags_2 },
    { 0x2a5ef53c, SUC_lexicon_tags_37 },
    { 0x9a1ef53b, SUC_lexicon_tags_17 },
    { 0xe53ef53f, SUC_lexicon_tags_25 },
    { 0x491ef540, SUC_lexicon_tags_15 },
    { 0xdb5ef540, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1ef544, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x177ef54c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7ef54f, SUC_lexicon_tags_25 },
    { 0x515ef550, SUC_lexicon_tags_6 },
    { 0x5cbef550, SUC_lexicon_tags_46 },
    { 0xbbbef54f, SUC_lexicon_tags_25 },
    { 0x6e7ef553, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83ef557, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625ef55d, SUC_lexicon_tags_3 },
    { 0xfbbef55d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdebef561, SUC_lexicon_tags_57 },
    { 0x85def561, SUC_lexicon_tags_15 },
    { 0xf23ef563, SUC_lexicon_tags_1 },
    { 0xf53ef564, SUC_lexicon_tags_1 },
    { 0x315ef565, SUC_lexicon_tags_2 },
    { 0x7fef565, SUC_lexicon_tags_7 },
    { 0xdd3ef567, SUC_lexicon_tags_15 },
    { 0xe3def567, SUC_lexicon_tags_37 },
    { 0x4ffef568, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe4def56b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x601ef56d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9ef570, SUC_lexicon_tags_33 },
    { 0x87def571, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc39ef573, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbef577, SUC_lexicon_tags_11 },
    { 0x7d5ef577, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebef57b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543ef583, SUC_lexicon_tags_8 },
    { 0x2fbef583, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5ef588, SUC_lexicon_tags_7 },
    { 0x853ef588, SUC_lexicon_tags_14 },
    { 0x2fdef58a, SUC_lexicon_tags_11 },
    { 0x8c5ef589, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbef58f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bef594, SUC_lexicon_tags_17 },
    { 0x36def595, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77ef59b, SUC_lexicon_tags_31 },
    { 0x451ef59c, SUC_lexicon_tags_29 },
    { 0x993ef59d, SUC_lexicon_tags_34 },
    { 0x7d1ef59d, SUC_lexicon_tags_45 },
    { 0x585ef59f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x143ef5a1, SUC_lexicon_tags_4 },
    { 0x1c5ef5a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebef5a6, SUC_lexicon_tags_45 },
    { 0x491ef5a7, SUC_lexicon_tags_30 },
    { 0xc35ef5a8, SUC_lexicon_tags_37 },
    { 0x807ef5a8, SUC_lexicon_tags_37 },
    { 0xe1ef5a8, SUC_lexicon_tags_17 },
    { 0x6a3ef5a7, SUC_lexicon_tags_15 },
    { 0x37fef5a7, SUC_lexicon_tags_47 },
    { 0x873ef5aa, SUC_lexicon_tags_39 },
    { 0x645ef5ae, SUC_lexicon_tags_17 },
    { 0xb7ef5af, SUC_lexicon_tags_35 },
    { 0x379ef5ae, SUC_lexicon_tags_19 },
    { 0x387ef5b1, SUC_lexicon_tags_17 },
    { 0x237ef5b2, SUC_lexicon_tags_62 },
    { 0xe4fef5b2, SUC_lexicon_tags_27 },
    { 0x6b5ef5ae, SUC_lexicon_tags_37 },
    { 0x93def5b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x991ef5b7, SUC_lexicon_tags_33 },
    { 0x681ef5b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55def5bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885ef5c0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3ef5c4, SUC_lexicon_tags_158 },
    { 0xe13ef5c5, SUC_lexicon_tags_21 },
    { 0x479ef5c6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93ef5ce, SUC_lexicon_tags_27 },
    { 0xb7def5ce, SUC_lexicon_tags_28 },
    { 0xe61ef5d0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x527ef5d3, SUC_lexicon_tags_45 },
    { 0xf43ef5d4, SUC_lexicon_tags_16 },
    { 0x57fef5d5, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x8fbef5d7, SUC_lexicon_tags_17 },
    { 0xbcbef5d7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebef5df, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0x30bef5e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677ef5e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3a3ef5e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5ef5ea, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593ef5ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb8def5f0, SUC_lexicon_tags_13 },
    { 0xf93ef5f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x757ef5f4, SUC_lexicon_tags_15 },
    { 0xc6bef5f5, SUC_lexicon_tags_38 },
    { 0xc1def5f4, SUC_lexicon_tags_69 },
    { 0xe3ef5f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xecfef5f9, SUC_lexicon_tags_7 },
    { 0x261ef5fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1ef5fd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1ef601, SUC_lexicon_tags_46 },
    { 0x869ef602, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1ef60d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205ef615, SUC_lexicon_tags_65 },
    { 0x70bef616, SUC_lexicon_tags_1 },
    { 0xef9ef616, SUC_lexicon_tags_25 },
    { 0x8d5ef617, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbef61c, SUC_lexicon_tags_1 },
    { 0x7ddef61d, SUC_lexicon_tags_11 },
    { 0x5b3ef61c, SUC_lexicon_tags_25 },
    { 0x533ef61f, SUC_lexicon_tags_8 },
    { 0x931ef620, SUC_lexicon_tags_1 },
    { 0xeb5ef620, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x40bef623, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xffef626, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2def62a, SUC_lexicon_tags_19 },
    { 0x551ef62a, SUC_lexicon_tags_38 },
    { 0xfafef62a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x18bef62e, SUC_lexicon_tags_45 },
    { 0xaa1ef62e, SUC_lexicon_tags_63 },
    { 0x42def630, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x851ef633, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625ef63f, SUC_lexicon_tags_0 },
    { 0xde5ef640, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817ef647, SUC_lexicon_tags_25 },
    { 0x323ef647, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27ef64c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35ef650, SUC_lexicon_tags_2 },
    { 0x5c1ef650, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x317ef653, SUC_lexicon_tags_65 },
    { 0x6b5ef653, SUC_lexicon_tags_46 },
    { 0x487ef655, SUC_lexicon_tags_47 },
    { 0xcfbef656, SUC_lexicon_tags_53 },
    { 0x711ef655, SUC_lexicon_tags_15 },
    { 0x957ef656, SUC_lexicon_tags_27 },
    { 0xd5fef659, SUC_lexicon_tags_65 },
    { 0x74bef65a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x6f3ef65c, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x393ef65e, SUC_lexicon_tags_17 },
    { 0x4cfef65f, SUC_lexicon_tags_33 },
    { 0xa91ef660, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03ef665, SUC_lexicon_tags_31 },
    { 0x235ef666, SUC_lexicon_tags_8 },
    { 0xbadef665, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x527ef669, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x355ef66c, SUC_lexicon_tags_14 },
    { 0xbebef66d, SUC_lexicon_tags_37 },
    { 0x65ef66d, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbef673, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x1a9ef675, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xab7ef678, SUC_lexicon_tags_2 },
    { 0xda3ef679, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bef67c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51ef684, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcafef68c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x595ef68e, SUC_lexicon_tags_7 },
    { 0xc3ef68f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x759ef691, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bef693, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbef697, SUC_lexicon_tags_55 },
    { 0xd89ef697, SUC_lexicon_tags_19 },
    { 0x20bef697, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53ef6a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x219ef6a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ef6aa, SUC_lexicon_tags_63 },
    { 0x8f1ef6ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19ef6af, SUC_lexicon_tags_1 },
    { 0x50def6b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x889ef6b2, SUC_lexicon_tags_1 },
    { 0xb39ef6b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bef6ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3def6bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x37bef6be, SUC_lexicon_tags_47 },
    { 0xc91ef6bf, SUC_lexicon_tags_37 },
    { 0xa8bef6c0, SUC_lexicon_tags_8 },
    { 0xdd1ef6c1, SUC_lexicon_tags_29 },
    { 0x669ef6c2, SUC_lexicon_tags_19 },
    { 0xf43ef6c1, SUC_lexicon_tags_63 },
    { 0x7bfef6c4, SUC_lexicon_tags_21 },
    { 0x8e5ef6c4, SUC_lexicon_tags_30 },
    { 0x32fef6c4, SUC_lexicon_tags_25 },
    { 0x4dbef6c7, SUC_lexicon_tags_8 },
    { 0x56bef6c3, SUC_lexicon_tags_2 },
    { 0x9efef6c9, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdef6cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3ffef6ce, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5b3ef6d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xad1ef6d2, SUC_lexicon_tags_21 },
    { 0xc49ef6d3, SUC_lexicon_tags_11 },
    { 0xe4bef6d3, SUC_lexicon_tags_15 },
    { 0xad1ef6d5, SUC_lexicon_tags_61 },
    { 0xb69ef6d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x525ef6d8, SUC_lexicon_tags_11 },
    { 0x1a5ef6d9, SUC_lexicon_tags_25 },
    { 0xa0bef6d8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1d7ef6dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x979ef6df, SUC_lexicon_tags_17 },
    { 0x755ef6e0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafef6e5, SUC_lexicon_tags_11 },
    { 0x7def6e5, SUC_lexicon_tags_45 },
    { 0x23def6e7, SUC_lexicon_tags_27 },
    { 0x721ef6e6, SUC_lexicon_tags_33 },
    { 0x9bfef6e8, SUC_lexicon_tags_2 },
    { 0x1e9ef6ea, SUC_lexicon_tags_27 },
    { 0x973ef6eb, SUC_lexicon_tags_29 },
    { 0x8c3ef6ea, SUC_lexicon_tags_15 },
    { 0x1bfef6ed, SUC_lexicon_tags_7 },
    { 0x62bef6eb, SUC_lexicon_tags_9 },
    { 0x99fef6ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85ef6f5, SUC_lexicon_tags_0 },
    { 0xd27ef6f6, SUC_lexicon_tags_0 },
    { 0x7fbef6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ef6fa, SUC_lexicon_tags_79 },
    { 0xffef6fb, SUC_lexicon_tags_17 },
    { 0x123ef6fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afef704, SUC_lexicon_tags_29 },
    { 0x781ef705, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x973ef707, SUC_lexicon_tags_10 },
    { 0x145ef708, SUC_lexicon_tags_2 },
    { 0x8cfef707, SUC_lexicon_tags_21 },
    { 0xb69ef70a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbef70e, SUC_lexicon_tags_17 },
    { 0xd77ef70f, SUC_lexicon_tags_44 },
    { 0x32bef710, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741ef714, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x773ef717, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf2fef719, SUC_lexicon_tags_11 },
    { 0x8c7ef71a, SUC_lexicon_tags_1 },
    { 0x4f1ef719, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x78bef71d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759ef721, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd1bef723, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5ef729, SUC_lexicon_tags_16 },
    { 0x9c1ef72a, SUC_lexicon_tags_27 },
    { 0x5e3ef72a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdef731, SUC_lexicon_tags_25 },
    { 0x465ef731, SUC_lexicon_tags_71 },
    { 0xde3ef731, SUC_lexicon_tags_60 },
    { 0xd25ef732, SUC_lexicon_tags_23 },
    { 0x833ef734, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbd9ef737, SUC_lexicon_tags_37 },
    { 0xa9bef738, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ef73c, SUC_lexicon_tags_15 },
    { 0x483ef73d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207ef746, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x10def749, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fef753, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x271ef755, SUC_lexicon_tags_11 },
    { 0x66def756, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3ef759, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa59ef75f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bef763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x55ef765, SUC_lexicon_tags_25 },
    { 0xb4def766, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bef76c, SUC_lexicon_tags_255 },
    { 0, NULL },
    { 0x5f3ef76e, SUC_lexicon_tags_8 },
    { 0x79bef76f, SUC_lexicon_tags_2 },
    { 0xfb3ef770, SUC_lexicon_tags_12 },
    { 0x75bef770, SUC_lexicon_tags_2 },
    { 0x297ef772, SUC_lexicon_tags_11 },
    { 0x7a9ef773, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0def778, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91ef77e, SUC_lexicon_tags_63 },
    { 0xebfef77f, SUC_lexicon_tags_15 },
    { 0x8d9ef780, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2e7ef782, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xc6fef784, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99fef786, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857ef78b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcadef78d, SUC_lexicon_tags_39 },
    { 0xd03ef78e, SUC_lexicon_tags_2 },
    { 0x3ffef78e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63ef792, SUC_lexicon_tags_38 },
    { 0x3ffef792, SUC_lexicon_tags_24 },
    { 0x855ef792, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9ef797, SUC_lexicon_tags_12 },
    { 0xadfef797, SUC_lexicon_tags_14 },
    { 0xa53ef799, SUC_lexicon_tags_14 },
    { 0xa1bef799, SUC_lexicon_tags_15 },
    { 0x7bdef79a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x527ef79e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddef7a2, SUC_lexicon_tags_44 },
    { 0x73def7a3, SUC_lexicon_tags_17 },
    { 0x665ef7a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143ef7aa, SUC_lexicon_tags_81 },
    { 0x69bef7aa, SUC_lexicon_tags_37 },
    { 0xd37ef7aa, SUC_lexicon_tags_1 },
    { 0x381ef7ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbef7b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391ef7b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bef7bd, SUC_lexicon_tags_7 },
    { 0xbb5ef7be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5ef7c0, SUC_lexicon_tags_39 },
    { 0x551ef7c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89ef7c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adef7cd, SUC_lexicon_tags_28 },
    { 0x95bef7cd, SUC_lexicon_tags_28 },
    { 0x7a9ef7cf, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xe0fef7d1, SUC_lexicon_tags_15 },
    { 0xdb1ef7d2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ef7df, SUC_lexicon_tags_37 },
    { 0xcb1ef7e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9fef7e4, SUC_lexicon_tags_39 },
    { 0x1ebef7e5, SUC_lexicon_tags_1 },
    { 0xcb5ef7e5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x805ef7e8, SUC_lexicon_tags_19 },
    { 0x459ef7e9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x385ef7eb, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61ef7ee, SUC_lexicon_tags_10 },
    { 0x36def7ef, SUC_lexicon_tags_17 },
    { 0x1e9ef7f0, SUC_lexicon_tags_38 },
    { 0xed9ef7f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x75def7f3, SUC_lexicon_tags_15 },
    { 0xb07ef7f3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x823ef7f6, SUC_lexicon_tags_33 },
    { 0x5b1ef7f7, SUC_lexicon_tags_2 },
    { 0xa71ef7f8, SUC_lexicon_tags_15 },
    { 0x3d7ef7f6, SUC_lexicon_tags_38 },
    { 0xe87ef7f9, SUC_lexicon_tags_11 },
    { 0xed1ef7f9, SUC_lexicon_tags_27 },
    { 0x249ef7f8, SUC_lexicon_tags_1 },
    { 0x8dfef7fd, SUC_lexicon_tags_2 },
    { 0xe5bef7f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7ef801, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9ef805, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xedbef808, SUC_lexicon_tags_33 },
    { 0xccbef809, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x121ef80b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491ef80f, SUC_lexicon_tags_33 },
    { 0xd3bef810, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879ef817, SUC_lexicon_tags_25 },
    { 0x9fbef817, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x65bef81a, SUC_lexicon_tags_37 },
    { 0x5c9ef81a, SUC_lexicon_tags_3 },
    { 0x6a9ef81a, SUC_lexicon_tags_1 },
    { 0x17bef81d, SUC_lexicon_tags_25 },
    { 0x6e1ef81d, SUC_lexicon_tags_11 },
    { 0xa5ef81f, SUC_lexicon_tags_37 },
    { 0x37def81c, SUC_lexicon_tags_2 },
    { 0xb3ef821, SUC_lexicon_tags_1 },
    { 0xcd5ef822, SUC_lexicon_tags_1 },
    { 0x7f5ef823, SUC_lexicon_tags_46 },
    { 0xd5ef820, SUC_lexicon_tags_34 },
    { 0x7e7ef825, SUC_lexicon_tags_7 },
    { 0xf1def826, SUC_lexicon_tags_38 },
    { 0xd2bef826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafef829, SUC_lexicon_tags_25 },
    { 0x1b5ef82a, SUC_lexicon_tags_32 },
    { 0x11bef82b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x685ef82d, SUC_lexicon_tags_37 },
    { 0x9b3ef82e, SUC_lexicon_tags_14 },
    { 0x51bef82f, SUC_lexicon_tags_7 },
    { 0xd1fef82e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5ef833, SUC_lexicon_tags_14 },
    { 0x625ef833, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7ef839, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x55bef83b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebef83f, SUC_lexicon_tags_27 },
    { 0x24bef840, SUC_lexicon_tags_87 },
    { 0x18bef841, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1ef845, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd41ef84a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa95ef84c, SUC_lexicon_tags_33 },
    { 0xc25ef84d, SUC_lexicon_tags_32 },
    { 0xf59ef84d, SUC_lexicon_tags_1 },
    { 0x40bef84f, SUC_lexicon_tags_33 },
    { 0xc45ef850, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x405ef852, SUC_lexicon_tags_45 },
    { 0x52fef852, SUC_lexicon_tags_15 },
    { 0x6dfef854, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf33ef856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1ef85d, SUC_lexicon_tags_48 },
    { 0x573ef85d, SUC_lexicon_tags_7 },
    { 0x7c7ef85e, SUC_lexicon_tags_19 },
    { 0x6dfef85d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x76bef862, SUC_lexicon_tags_65 },
    { 0x96bef863, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5ef866, SUC_lexicon_tags_10 },
    { 0xb25ef867, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8d9ef869, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7ef86d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdef874, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac1ef876, SUC_lexicon_tags_45 },
    { 0xe85ef877, SUC_lexicon_tags_63 },
    { 0xc81ef878, SUC_lexicon_tags_2 },
    { 0x141ef879, SUC_lexicon_tags_25 },
    { 0x6adef877, SUC_lexicon_tags_15 },
    { 0xadfef87b, SUC_lexicon_tags_37 },
    { 0x5adef879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185ef881, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ef885, SUC_lexicon_tags_17 },
    { 0x387ef886, SUC_lexicon_tags_10 },
    { 0x7d5ef886, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33ef88f, SUC_lexicon_tags_55 },
    { 0x45fef890, SUC_lexicon_tags_7 },
    { 0xcc7ef890, SUC_lexicon_tags_23 },
    { 0x9f7ef890, SUC_lexicon_tags_6 },
    { 0xcefef893, SUC_lexicon_tags_1 },
    { 0xaebef893, SUC_lexicon_tags_37 },
    { 0xee1ef895, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0x497ef898, SUC_lexicon_tags_32 },
    { 0x81def899, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa89ef89b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113ef8a1, SUC_lexicon_tags_17 },
    { 0x3e9ef8a2, SUC_lexicon_tags_37 },
    { 0xfef8a1, SUC_lexicon_tags_6 },
    { 0xc61ef8a4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa93ef8a6, SUC_lexicon_tags_33 },
    { 0x707ef8a7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205ef8ac, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91ef8b5, SUC_lexicon_tags_11 },
    { 0x7b1ef8b6, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1ef8bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8bdef8c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bef8c4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57ef8cc, SUC_lexicon_tags_11 },
    { 0x9e5ef8cc, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7ef8d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfef8d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5ef8db, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x88def8de, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ef8e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x20fef8e5, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9ef8e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37ef8eb, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xafbef8ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x301ef8f0, SUC_lexicon_tags_16 },
    { 0x643ef8f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x82def8f4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffef8f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9ef8f9, SUC_lexicon_tags_12 },
    { 0x5e3ef8f9, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135ef8fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0def902, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x47def905, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7def90a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf65ef90c, SUC_lexicon_tags_37 },
    { 0x6adef90d, SUC_lexicon_tags_27 },
    { 0x7a9ef90c, SUC_lexicon_tags_39 },
    { 0xbf5ef90c, SUC_lexicon_tags_43 },
    { 0x5d1ef910, SUC_lexicon_tags_77 },
    { 0x771ef910, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17ef915, SUC_lexicon_tags_8 },
    { 0x181ef916, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7ef91b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bef923, SUC_lexicon_tags_2 },
    { 0xb9fef924, SUC_lexicon_tags_7 },
    { 0x575ef925, SUC_lexicon_tags_14 },
    { 0x40fef923, SUC_lexicon_tags_84 },
    { 0x4abef927, SUC_lexicon_tags_1 },
    { 0x24fef924, SUC_lexicon_tags_1 },
    { 0x7b5ef929, SUC_lexicon_tags_45 },
    { 0xf9ef929, SUC_lexicon_tags_15 },
    { 0xaa1ef92a, SUC_lexicon_tags_30 },
    { 0xcf1ef92c, SUC_lexicon_tags_25 },
    { 0x80bef92c, SUC_lexicon_tags_37 },
    { 0xa9ef926, SUC_lexicon_tags_25 },
    { 0x86def925, SUC_lexicon_tags_37 },
    { 0xed5ef92c, SUC_lexicon_tags_33 },
    { 0x505ef92f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3def935, SUC_lexicon_tags_3 },
    { 0xdcfef935, SUC_lexicon_tags_17 },
    { 0x5f1ef936, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafef93f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xba9ef941, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8fef945, SUC_lexicon_tags_9 },
    { 0x1c1ef946, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5b9ef948, SUC_lexicon_tags_11 },
    { 0xc63ef948, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9ef94c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc4bef94e, SUC_lexicon_tags_11 },
    { 0x2ebef94f, SUC_lexicon_tags_33 },
    { 0xedef94f, SUC_lexicon_tags_1 },
    { 0x87def94f, SUC_lexicon_tags_7 },
    { 0x52def94f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf55ef954, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ef958, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaafef95e, SUC_lexicon_tags_25 },
    { 0x91fef95e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x939ef962, SUC_lexicon_tags_40 },
    { 0xecfef963, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5ef96c, SUC_lexicon_tags_37 },
    { 0xa1def96d, SUC_lexicon_tags_6 },
    { 0xc05ef96d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdafef971, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6bef973, SUC_lexicon_tags_11 },
    { 0xe5bef974, SUC_lexicon_tags_37 },
    { 0xdef973, SUC_lexicon_tags_906 },
    { 0x8bfef976, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4a7ef978, SUC_lexicon_tags_23 },
    { 0x69ef978, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11ef97f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x589ef981, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5ef986, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fef98a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bef98f, SUC_lexicon_tags_15 },
    { 0xbf1ef990, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1ef993, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85ef99a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddef99f, SUC_lexicon_tags_17 },
    { 0x4ffef9a0, SUC_lexicon_tags_47 },
    { 0xce1ef9a1, SUC_lexicon_tags_5 },
    { 0x3edef99f, SUC_lexicon_tags_12 },
    { 0x4bfef9a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x231ef9a6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb27ef9a9, SUC_lexicon_tags_27 },
    { 0xaedef9a9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309ef9ae, SUC_lexicon_tags_26 },
    { 0x8bdef9af, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x669ef9b1, SUC_lexicon_tags_17 },
    { 0x8d7ef9b1, SUC_lexicon_tags_2 },
    { 0xa23ef9b1, SUC_lexicon_tags_17 },
    { 0x86fef9b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5ef9bb, SUC_lexicon_tags_37 },
    { 0xb31ef9bb, SUC_lexicon_tags_66 },
    { 0xa11ef9bd, SUC_lexicon_tags_35 },
    { 0x959ef9be, SUC_lexicon_tags_31 },
    { 0x8d5ef9bf, SUC_lexicon_tags_33 },
    { 0x701ef9c0, SUC_lexicon_tags_43 },
    { 0x45bef9bf, SUC_lexicon_tags_82 },
    { 0x41def9c2, SUC_lexicon_tags_46 },
    { 0xcadef9c3, SUC_lexicon_tags_10 },
    { 0x383ef9bd, SUC_lexicon_tags_40 },
    { 0xf09ef9be, SUC_lexicon_tags_7 },
    { 0x9a1ef9bc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x699ef9c9, SUC_lexicon_tags_26 },
    { 0x47bef9ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ef9cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63ef9d3, SUC_lexicon_tags_7 },
    { 0x4c9ef9d4, SUC_lexicon_tags_0 },
    { 0x14bef9d5, SUC_lexicon_tags_8 },
    { 0x1bef9d6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x291ef9d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x473ef9da, SUC_lexicon_tags_49 },
    { 0xb57ef9da, SUC_lexicon_tags_31 },
    { 0x869ef9dc, SUC_lexicon_tags_7 },
    { 0x7f3ef9dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd07ef9df, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbef9e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7def9f1, SUC_lexicon_tags_39 },
    { 0xd99ef9f2, SUC_lexicon_tags_17 },
    { 0x85ef9f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5ef9f9, SUC_lexicon_tags_65 },
    { 0xc51ef9fa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x533ef9fc, SUC_lexicon_tags_2 },
    { 0x503ef9fd, SUC_lexicon_tags_28 },
    { 0xf2def9fe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09efa01, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x96fefa04, SUC_lexicon_tags_8 },
    { 0xa7befa05, SUC_lexicon_tags_37 },
    { 0xd49efa04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf09efa08, SUC_lexicon_tags_194 },
    { 0xc3befa09, SUC_lexicon_tags_38 },
    { 0x2e7efa09, SUC_lexicon_tags_29 },
    { 0x693efa08, SUC_lexicon_tags_17 },
    { 0x4d1efa0c, SUC_lexicon_tags_0 },
    { 0xe7efa09, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x913efa0f, SUC_lexicon_tags_15 },
    { 0x25fefa0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x75fefa12, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb29efa14, SUC_lexicon_tags_15 },
    { 0x5c5efa15, SUC_lexicon_tags_11 },
    { 0x2afefa16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xddefa18, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x853efa1b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09efa1e, SUC_lexicon_tags_10 },
    { 0x43defa1f, SUC_lexicon_tags_8 },
    { 0x8d5efa20, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3efa24, SUC_lexicon_tags_14 },
    { 0xc4defa24, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafefa2c, SUC_lexicon_tags_14 },
    { 0x699efa2d, SUC_lexicon_tags_14 },
    { 0x7f3efa2c, SUC_lexicon_tags_1 },
    { 0xf47efa2f, SUC_lexicon_tags_46 },
    { 0xca7efa2f, SUC_lexicon_tags_23 },
    { 0xe9befa31, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaadefa33, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x11befa36, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5afefa39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x525efa3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfefa3f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fefa47, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65efa4c, SUC_lexicon_tags_11 },
    { 0xd97efa4c, SUC_lexicon_tags_87 },
    { 0xed7efa4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fefa53, SUC_lexicon_tags_374 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3efa56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2fefa59, SUC_lexicon_tags_38 },
    { 0x20befa59, SUC_lexicon_tags_11 },
    { 0xf57efa5a, SUC_lexicon_tags_66 },
    { 0x1bbefa5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbf1efa5e, SUC_lexicon_tags_48 },
    { 0xc09efa5e, SUC_lexicon_tags_47 },
    { 0x831efa5f, SUC_lexicon_tags_87 },
    { 0x5adefa61, SUC_lexicon_tags_47 },
    { 0x13efa5e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86defa65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57efa6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdebefa6d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x503efa70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x28befa72, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7efa76, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb13efa78, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd4befa7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc0befa7c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7defa81, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5a9efa83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddefa8a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5efa97, SUC_lexicon_tags_65 },
    { 0xc2defa97, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb01efa9b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2cfefa9d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745efaa1, SUC_lexicon_tags_15 },
    { 0x8cbefaa1, SUC_lexicon_tags_17 },
    { 0xc09efaa1, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57efaa6, SUC_lexicon_tags_63 },
    { 0xba5efaa7, SUC_lexicon_tags_17 },
    { 0x359efaa6, SUC_lexicon_tags_2 },
    { 0x9d5efaa8, SUC_lexicon_tags_47 },
    { 0xd2befaa9, SUC_lexicon_tags_17 },
    { 0xd83efaa8, SUC_lexicon_tags_14 },
    { 0x26befaa6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5efab1, SUC_lexicon_tags_17 },
    { 0xed1efab1, SUC_lexicon_tags_2 },
    { 0xccfefab3, SUC_lexicon_tags_138 },
    { 0x13befab1, SUC_lexicon_tags_17 },
    { 0xc11efab3, SUC_lexicon_tags_45 },
    { 0xd5befab5, SUC_lexicon_tags_17 },
    { 0xbbbefab4, SUC_lexicon_tags_60 },
    { 0x8bdefab8, SUC_lexicon_tags_12 },
    { 0xa5defab4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x63efabb, SUC_lexicon_tags_27 },
    { 0x355efabc, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x429efabe, SUC_lexicon_tags_38 },
    { 0x373efabf, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x299efac1, SUC_lexicon_tags_8 },
    { 0x32befac1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1a5efac4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9efac9, SUC_lexicon_tags_12 },
    { 0x697efaca, SUC_lexicon_tags_25 },
    { 0xecbefacb, SUC_lexicon_tags_2 },
    { 0x6abefaca, SUC_lexicon_tags_0 },
    { 0xd45efacd, SUC_lexicon_tags_25 },
    { 0xfb1efacd, SUC_lexicon_tags_77 },
    { 0x49efacc, SUC_lexicon_tags_17 },
    { 0x687efad0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x871efad2, SUC_lexicon_tags_43 },
    { 0xccdefad3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x721efad7, SUC_lexicon_tags_7 },
    { 0xdffefad8, SUC_lexicon_tags_13 },
    { 0xadfefad9, SUC_lexicon_tags_13 },
    { 0xfa1efad7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x24fefadd, SUC_lexicon_tags_91 },
    { 0x9bfefadd, SUC_lexicon_tags_40 },
    { 0x611efadf, SUC_lexicon_tags_1 },
    { 0xae7efae0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5efae3, SUC_lexicon_tags_1 },
    { 0x2a1efae4, SUC_lexicon_tags_17 },
    { 0x28befae5, SUC_lexicon_tags_26 },
    { 0x6f7efae5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8ebefae8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe1defaea, SUC_lexicon_tags_15 },
    { 0xb25efaeb, SUC_lexicon_tags_33 },
    { 0x13fefaeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b7efaee, SUC_lexicon_tags_87 },
    { 0xa9fefaee, SUC_lexicon_tags_2 },
    { 0x515efaef, SUC_lexicon_tags_48 },
    { 0xe07efaee, SUC_lexicon_tags_12 },
    { 0xffbefaf2, SUC_lexicon_tags_19 },
    { 0xab7efaf2, SUC_lexicon_tags_732 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37efaf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705efafb, SUC_lexicon_tags_47 },
    { 0x6f5efafc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77efaff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8d3efb01, SUC_lexicon_tags_25 },
    { 0xbcfefb02, SUC_lexicon_tags_79 },
    { 0x4e9efb02, SUC_lexicon_tags_2 },
    { 0xbf1efb03, SUC_lexicon_tags_17 },
    { 0x3ebefb05, SUC_lexicon_tags_113 },
    { 0x625efb05, SUC_lexicon_tags_47 },
    { 0x8f1efb07, SUC_lexicon_tags_2 },
    { 0xd09efb04, SUC_lexicon_tags_8 },
    { 0xf7fefb09, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1efb0c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x49fefb0e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc35efb11, SUC_lexicon_tags_39 },
    { 0xd51efb12, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59efb15, SUC_lexicon_tags_32 },
    { 0x3a3efb16, SUC_lexicon_tags_74 },
    { 0xc71efb17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5efb1f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1efb23, SUC_lexicon_tags_26 },
    { 0x193efb23, SUC_lexicon_tags_17 },
    { 0x4bbefb25, SUC_lexicon_tags_45 },
    { 0x19befb26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97defb28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6befb2b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25efb31, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa97efb36, SUC_lexicon_tags_28 },
    { 0xa71efb37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81efb3a, SUC_lexicon_tags_25 },
    { 0x825efb3b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0xc2befb3d, SUC_lexicon_tags_26 },
    { 0x6e7efb3e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7efb41, SUC_lexicon_tags_299 },
    { 0, NULL },
    { 0xfffefb43, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7efb46, SUC_lexicon_tags_14 },
    { 0xca7efb47, SUC_lexicon_tags_6 },
    { 0xea1efb47, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943efb4f, SUC_lexicon_tags_25 },
    { 0x503efb50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29efb53, SUC_lexicon_tags_47 },
    { 0xaf5efb54, SUC_lexicon_tags_0 },
    { 0x8c5efb54, SUC_lexicon_tags_33 },
    { 0x18fefb56, SUC_lexicon_tags_17 },
    { 0xc47efb54, SUC_lexicon_tags_43 },
    { 0x993efb54, SUC_lexicon_tags_25 },
    { 0x8bbefb54, SUC_lexicon_tags_25 },
    { 0x899efb5a, SUC_lexicon_tags_14 },
    { 0xdd7efb5a, SUC_lexicon_tags_1 },
    { 0xe89efb55, SUC_lexicon_tags_38 },
    { 0x6b5efb5b, SUC_lexicon_tags_23 },
    { 0x5c5efb5d, SUC_lexicon_tags_32 },
    { 0xc21efb54, SUC_lexicon_tags_37 },
    { 0xf3efb5e, SUC_lexicon_tags_45 },
    { 0x11defb54, SUC_lexicon_tags_11 },
    { 0x659efb62, SUC_lexicon_tags_2 },
    { 0x753efb63, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xabefb66, SUC_lexicon_tags_197 },
    { 0x21befb66, SUC_lexicon_tags_16 },
    { 0x83fefb68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa93efb6a, SUC_lexicon_tags_17 },
    { 0x93befb6b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57efb73, SUC_lexicon_tags_30 },
    { 0xd37efb74, SUC_lexicon_tags_1 },
    { 0x689efb73, SUC_lexicon_tags_15 },
    { 0x6b3efb74, SUC_lexicon_tags_37 },
    { 0x1c5efb77, SUC_lexicon_tags_12 },
    { 0xc5efb78, SUC_lexicon_tags_25 },
    { 0x103efb79, SUC_lexicon_tags_42 },
    { 0xde5efb77, SUC_lexicon_tags_2 },
    { 0x9befb76, SUC_lexicon_tags_38 },
    { 0x6bfefb78, SUC_lexicon_tags_1 },
    { 0x64defb77, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0x843efb80, SUC_lexicon_tags_27 },
    { 0x311efb81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd9efb83, SUC_lexicon_tags_108 },
    { 0xe37efb84, SUC_lexicon_tags_6 },
    { 0x561efb85, SUC_lexicon_tags_21 },
    { 0x7f1efb86, SUC_lexicon_tags_47 },
    { 0xae9efb87, SUC_lexicon_tags_7 },
    { 0x20befb88, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419efb8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9efb8f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdefb92, SUC_lexicon_tags_29 },
    { 0xd65efb93, SUC_lexicon_tags_160 },
    { 0x989efb94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3efb98, SUC_lexicon_tags_28 },
    { 0x4afefb99, SUC_lexicon_tags_56 },
    { 0xf91efb9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc77efb9c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547efba2, SUC_lexicon_tags_17 },
    { 0x5fefba3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x321efba5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60fefbad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbefbb5, SUC_lexicon_tags_10 },
    { 0xd8befbb6, SUC_lexicon_tags_31 },
    { 0xad3efbb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1fefbba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0fefbbe, SUC_lexicon_tags_45 },
    { 0xd55efbbe, SUC_lexicon_tags_24 },
    { 0xd3fefbc0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21befbc5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4adefbc7, SUC_lexicon_tags_16 },
    { 0xeddefbc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x84befbcb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xa77efbce, SUC_lexicon_tags_12 },
    { 0xf83efbcf, SUC_lexicon_tags_21 },
    { 0xf85efbcf, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddefbd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4abefbd6, SUC_lexicon_tags_25 },
    { 0xb37efbd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567efbdb, SUC_lexicon_tags_39 },
    { 0x6a3efbdb, SUC_lexicon_tags_31 },
    { 0xcdefbdd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9efbe1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8c7efbe3, SUC_lexicon_tags_43 },
    { 0x9d3efbe4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69efbe7, SUC_lexicon_tags_0 },
    { 0x6dfefbe8, SUC_lexicon_tags_30 },
    { 0x579efbe9, SUC_lexicon_tags_47 },
    { 0x491efbea, SUC_lexicon_tags_14 },
    { 0x5efbe9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xf53efbed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1efbf3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fefbf6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdedefbf8, SUC_lexicon_tags_49 },
    { 0xc3efbf8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x279efbfc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4ddefbfe, SUC_lexicon_tags_45 },
    { 0xdb3efbff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5efc04, SUC_lexicon_tags_11 },
    { 0xda9efc04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627efc09, SUC_lexicon_tags_25 },
    { 0x6c7efc0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x633efc0d, SUC_lexicon_tags_7 },
    { 0xc01efc0e, SUC_lexicon_tags_19 },
    { 0xc71efc0f, SUC_lexicon_tags_27 },
    { 0x107efc10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40defc15, SUC_lexicon_tags_28 },
    { 0x2a7efc15, SUC_lexicon_tags_30 },
    { 0xad1efc17, SUC_lexicon_tags_0 },
    { 0x107efc18, SUC_lexicon_tags_17 },
    { 0xc4defc18, SUC_lexicon_tags_113 },
    { 0xedefc18, SUC_lexicon_tags_49 },
    { 0x6adefc1b, SUC_lexicon_tags_46 },
    { 0xf79efc1c, SUC_lexicon_tags_17 },
    { 0x367efc1d, SUC_lexicon_tags_7 },
    { 0x2defc1e, SUC_lexicon_tags_11 },
    { 0xbbefc1f, SUC_lexicon_tags_7 },
    { 0xbcfefc1b, SUC_lexicon_tags_14 },
    { 0x735efc19, SUC_lexicon_tags_0 },
    { 0xbb9efc1d, SUC_lexicon_tags_38 },
    { 0x419efc20, SUC_lexicon_tags_2 },
    { 0x633efc24, SUC_lexicon_tags_35 },
    { 0x9bdefc15, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901efc29, SUC_lexicon_tags_68 },
    { 0x2bfefc2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x515efc2d, SUC_lexicon_tags_2 },
    { 0x2c3efc2e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb27efc30, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddefc33, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85efc39, SUC_lexicon_tags_33 },
    { 0x915efc3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49defc3e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x47fefc40, SUC_lexicon_tags_0 },
    { 0xad7efc40, SUC_lexicon_tags_15 },
    { 0xaa5efc41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x173efc44, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbefc4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x36befc51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf1fefc53, SUC_lexicon_tags_24 },
    { 0x539efc54, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x613efc57, SUC_lexicon_tags_37 },
    { 0x845efc58, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281efc61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5efc64, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905efc6d, SUC_lexicon_tags_37 },
    { 0xa1befc6e, SUC_lexicon_tags_2 },
    { 0x739efc6f, SUC_lexicon_tags_12 },
    { 0x7c9efc70, SUC_lexicon_tags_2 },
    { 0x64defc71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb67efc73, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb6defc75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x511efc77, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x789efc7a, SUC_lexicon_tags_0 },
    { 0x1e3efc7b, SUC_lexicon_tags_39 },
    { 0xb41efc7c, SUC_lexicon_tags_37 },
    { 0x93defc7d, SUC_lexicon_tags_29 },
    { 0x4edefc7d, SUC_lexicon_tags_37 },
    { 0x781efc7f, SUC_lexicon_tags_29 },
    { 0x449efc80, SUC_lexicon_tags_7 },
    { 0x321efc81, SUC_lexicon_tags_1 },
    { 0xa65efc80, SUC_lexicon_tags_17 },
    { 0x717efc83, SUC_lexicon_tags_16 },
    { 0xcbdefc7c, SUC_lexicon_tags_17 },
    { 0xc55efc80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8efefc87, SUC_lexicon_tags_25 },
    { 0xa7efc88, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9befc8a, SUC_lexicon_tags_2 },
    { 0xa29efc8b, SUC_lexicon_tags_37 },
    { 0xc59efc8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6edefc8e, SUC_lexicon_tags_37 },
    { 0x29befc8e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddefc94, SUC_lexicon_tags_17 },
    { 0x699efc95, SUC_lexicon_tags_25 },
    { 0x15efc95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x29efc98, SUC_lexicon_tags_17 },
    { 0xc57efc99, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbefc9e, SUC_lexicon_tags_33 },
    { 0x92fefc9f, SUC_lexicon_tags_15 },
    { 0xee5efca0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93befca5, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1efca9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x70fefcab, SUC_lexicon_tags_15 },
    { 0x175efcac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76befcae, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbefcb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x62fefcb5, SUC_lexicon_tags_179 },
    { 0x6a5efcb6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xa1befcb8, SUC_lexicon_tags_62 },
    { 0xf93efcb9, SUC_lexicon_tags_34 },
    { 0xe87efcb9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9efcc0, SUC_lexicon_tags_12 },
    { 0x90befcc0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2befcc5, SUC_lexicon_tags_29 },
    { 0x173efcc6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5e1efcc8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbefccd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x343efcd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x27defcd3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6e9efcd5, SUC_lexicon_tags_17 },
    { 0x345efcd5, SUC_lexicon_tags_8 },
    { 0x58fefcd5, SUC_lexicon_tags_7 },
    { 0x915efcd8, SUC_lexicon_tags_37 },
    { 0x93efcd7, SUC_lexicon_tags_37 },
    { 0xc53efcd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcc5efcdc, SUC_lexicon_tags_2 },
    { 0xbcbefcdc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3efce0, SUC_lexicon_tags_31 },
    { 0x31fefce0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fefce6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20fefcf0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97efcf3, SUC_lexicon_tags_27 },
    { 0xa81efcf4, SUC_lexicon_tags_12 },
    { 0xcc3efcf3, SUC_lexicon_tags_2 },
    { 0x669efcf6, SUC_lexicon_tags_37 },
    { 0x781efcf7, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabefd00, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203efd05, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x16befd07, SUC_lexicon_tags_1 },
    { 0x3fbefd07, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5a7efd0a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x263efd0d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1efd10, SUC_lexicon_tags_23 },
    { 0x835efd10, SUC_lexicon_tags_68 },
    { 0x5bbefd12, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40befd1b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x293efd1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe35efd20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebefd23, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e7efd25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409efd2c, SUC_lexicon_tags_45 },
    { 0xd71efd2d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x711efd30, SUC_lexicon_tags_71 },
    { 0x1e1efd31, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ebefd33, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9efd36, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdefd39, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x339efd3b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb87efd3d, SUC_lexicon_tags_7 },
    { 0x343efd3e, SUC_lexicon_tags_23 },
    { 0x291efd3f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637efd43, SUC_lexicon_tags_31 },
    { 0x73befd44, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285efd48, SUC_lexicon_tags_16 },
    { 0xde3efd48, SUC_lexicon_tags_7 },
    { 0xb03efd49, SUC_lexicon_tags_1 },
    { 0xad3efd4b, SUC_lexicon_tags_7 },
    { 0xb29efd4b, SUC_lexicon_tags_11 },
    { 0x9c5efd4d, SUC_lexicon_tags_12 },
    { 0x725efd4e, SUC_lexicon_tags_0 },
    { 0x12befd4f, SUC_lexicon_tags_19 },
    { 0x4e1efd50, SUC_lexicon_tags_37 },
    { 0x721efd50, SUC_lexicon_tags_15 },
    { 0x2a7efd51, SUC_lexicon_tags_15 },
    { 0x63fefd53, SUC_lexicon_tags_15 },
    { 0x3a1efd53, SUC_lexicon_tags_2 },
    { 0xcf5efd55, SUC_lexicon_tags_27 },
    { 0x7adefd55, SUC_lexicon_tags_60 },
    { 0x943efd53, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1efd5f, SUC_lexicon_tags_30 },
    { 0x9f7efd60, SUC_lexicon_tags_30 },
    { 0x19defd5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xba9efd63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fefd66, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9efd69, SUC_lexicon_tags_72 },
    { 0x865efd6a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x21efd6d, SUC_lexicon_tags_17 },
    { 0xfc7efd6e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01efd72, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59efd75, SUC_lexicon_tags_47 },
    { 0x2adefd76, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25efd79, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54fefd7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3defd7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x7f1efd80, SUC_lexicon_tags_15 },
    { 0xee1efd81, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe81efd83, SUC_lexicon_tags_47 },
    { 0x7ebefd84, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbb5efd86, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x21defd88, SUC_lexicon_tags_19 },
    { 0xd37efd89, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7efd8d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x85fefd90, SUC_lexicon_tags_11 },
    { 0x4e3efd91, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x19befd94, SUC_lexicon_tags_25 },
    { 0x899efd95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbefd98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9d7efd9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adefda2, SUC_lexicon_tags_23 },
    { 0x233efda3, SUC_lexicon_tags_104 },
    { 0xbc3efda4, SUC_lexicon_tags_11 },
    { 0x179efda5, SUC_lexicon_tags_17 },
    { 0x383efda6, SUC_lexicon_tags_37 },
    { 0xbcfefda7, SUC_lexicon_tags_25 },
    { 0x633efda8, SUC_lexicon_tags_28 },
    { 0x2e9efda5, SUC_lexicon_tags_1 },
    { 0xfcbefda2, SUC_lexicon_tags_17 },
    { 0x9dfefda7, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x289efdae, SUC_lexicon_tags_11 },
    { 0x7c5efdaf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb93efdb1, SUC_lexicon_tags_7 },
    { 0x16defdb2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33efdbe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7efdc4, SUC_lexicon_tags_15 },
    { 0xe7defdc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6fefdca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fefdce, SUC_lexicon_tags_29 },
    { 0xcadefdcf, SUC_lexicon_tags_37 },
    { 0x793efdd0, SUC_lexicon_tags_32 },
    { 0xa53efdd1, SUC_lexicon_tags_17 },
    { 0xffbefdd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9efdd8, SUC_lexicon_tags_25 },
    { 0x5c3efdd9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbefdde, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x863efde1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301efde5, SUC_lexicon_tags_26 },
    { 0x36fefde5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x961efde8, SUC_lexicon_tags_11 },
    { 0x53defde9, SUC_lexicon_tags_7 },
    { 0xb1fefdea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599efdf0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x663efdf3, SUC_lexicon_tags_151 },
    { 0x8c1efdf4, SUC_lexicon_tags_25 },
    { 0x8b9efdf4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59efdfb, SUC_lexicon_tags_16 },
    { 0x79befdfc, SUC_lexicon_tags_27 },
    { 0x94befdfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x903efe00, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27defe05, SUC_lexicon_tags_11 },
    { 0x145efe06, SUC_lexicon_tags_14 },
    { 0x1d9efe07, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x61fefe09, SUC_lexicon_tags_15 },
    { 0xeebefe0a, SUC_lexicon_tags_25 },
    { 0x855efe0b, SUC_lexicon_tags_85 },
    { 0x81efe0a, SUC_lexicon_tags_38 },
    { 0x2edefe0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x479efe0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f9efe11, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x99defe13, SUC_lexicon_tags_49 },
    { 0x5bbefe14, SUC_lexicon_tags_56 },
    { 0x451efe15, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fefe1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3efe27, SUC_lexicon_tags_21 },
    { 0xff3efe28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12befe2f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3efe33, SUC_lexicon_tags_4 },
    { 0xb4defe34, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2fefe39, SUC_lexicon_tags_37 },
    { 0x7f9efe3a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adefe3f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1efe43, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x447efe45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa23efe47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951efe53, SUC_lexicon_tags_25 },
    { 0x25efe54, SUC_lexicon_tags_32 },
    { 0x259efe54, SUC_lexicon_tags_30 },
    { 0xa43efe56, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x43efe59, SUC_lexicon_tags_32 },
    { 0xe1efe5a, SUC_lexicon_tags_8 },
    { 0x621efe59, SUC_lexicon_tags_29 },
    { 0x3d5efe5c, SUC_lexicon_tags_2 },
    { 0xa9efe5d, SUC_lexicon_tags_11 },
    { 0x259efe5e, SUC_lexicon_tags_36 },
    { 0xfb7efe5c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651efe66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9befe69, SUC_lexicon_tags_12 },
    { 0xbb9efe69, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11efe6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb3efe71, SUC_lexicon_tags_32 },
    { 0x499efe71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7efe75, SUC_lexicon_tags_2 },
    { 0x47defe75, SUC_lexicon_tags_37 },
    { 0x941efe77, SUC_lexicon_tags_31 },
    { 0x2c5efe78, SUC_lexicon_tags_66 },
    { 0x2a3efe78, SUC_lexicon_tags_30 },
    { 0xb03efe7a, SUC_lexicon_tags_79 },
    { 0xdf9efe7a, SUC_lexicon_tags_1 },
    { 0xa17efe77, SUC_lexicon_tags_12 },
    { 0xf49efe7d, SUC_lexicon_tags_65 },
    { 0xa03efe7d, SUC_lexicon_tags_11 },
    { 0x56befe7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf91efe81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7efe86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeb5efe88, SUC_lexicon_tags_17 },
    { 0x3a7efe88, SUC_lexicon_tags_27 },
    { 0x6dbefe8a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38fefe8e, SUC_lexicon_tags_49 },
    { 0x81befe8e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe93efe91, SUC_lexicon_tags_6 },
    { 0xf2befe92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1efe96, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81efe99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5efe9f, SUC_lexicon_tags_34 },
    { 0x177efea0, SUC_lexicon_tags_43 },
    { 0x1afefea1, SUC_lexicon_tags_47 },
    { 0xf0defe9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x941efea8, SUC_lexicon_tags_63 },
    { 0xef7efea9, SUC_lexicon_tags_31 },
    { 0xfb5efea9, SUC_lexicon_tags_23 },
    { 0xe9fefeab, SUC_lexicon_tags_2 },
    { 0xb1fefeac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef5efeae, SUC_lexicon_tags_11 },
    { 0x58befeae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bfefeb5, SUC_lexicon_tags_8 },
    { 0x1dbefeb5, SUC_lexicon_tags_11 },
    { 0xfabefeb7, SUC_lexicon_tags_212 },
    { 0xde5efeb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafefebc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0defec4, SUC_lexicon_tags_46 },
    { 0x701efec4, SUC_lexicon_tags_1 },
    { 0xc9defec6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e3efec8, SUC_lexicon_tags_31 },
    { 0xf3defec9, SUC_lexicon_tags_33 },
    { 0xa5befeca, SUC_lexicon_tags_1 },
    { 0x31befec8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x971efece, SUC_lexicon_tags_12 },
    { 0xafefece, SUC_lexicon_tags_30 },
    { 0x265efece, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fefed5, SUC_lexicon_tags_82 },
    { 0xc07efed6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c7efed8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42defee1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffefeea, SUC_lexicon_tags_11 },
    { 0xc15efeeb, SUC_lexicon_tags_17 },
    { 0xbabefeeb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x86befeef, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95efef3, SUC_lexicon_tags_21 },
    { 0xab7efef4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79efefe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9eff01, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc8beff03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3deff09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7eff1a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3eff1e, SUC_lexicon_tags_31 },
    { 0xd71eff1f, SUC_lexicon_tags_7 },
    { 0x11deff20, SUC_lexicon_tags_25 },
    { 0xe89eff21, SUC_lexicon_tags_37 },
    { 0xe17eff21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51eff24, SUC_lexicon_tags_37 },
    { 0x4bdeff25, SUC_lexicon_tags_63 },
    { 0xff3eff25, SUC_lexicon_tags_7 },
    { 0xae1eff26, SUC_lexicon_tags_19 },
    { 0x687eff28, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe17eff2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x237eff2e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x875eff30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xeabeff32, SUC_lexicon_tags_25 },
    { 0xd67eff33, SUC_lexicon_tags_30 },
    { 0xdd9eff33, SUC_lexicon_tags_48 },
    { 0xfe9eff32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabeff3b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x95feff3d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfeff40, SUC_lexicon_tags_2 },
    { 0x525eff40, SUC_lexicon_tags_37 },
    { 0x10beff40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x23deff45, SUC_lexicon_tags_0 },
    { 0xb2feff46, SUC_lexicon_tags_17 },
    { 0x2b1eff47, SUC_lexicon_tags_0 },
    { 0x13deff48, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13eff4d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcb9eff4f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9eff52, SUC_lexicon_tags_12 },
    { 0x73eff53, SUC_lexicon_tags_2 },
    { 0x231eff53, SUC_lexicon_tags_15 },
    { 0x3f3eff54, SUC_lexicon_tags_11 },
    { 0x559eff56, SUC_lexicon_tags_40 },
    { 0x537eff55, SUC_lexicon_tags_45 },
    { 0xc8feff58, SUC_lexicon_tags_15 },
    { 0xb9deff58, SUC_lexicon_tags_17 },
    { 0xe25eff5a, SUC_lexicon_tags_53 },
    { 0x775eff5b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x7afeff5e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffeff6a, SUC_lexicon_tags_25 },
    { 0xb8feff6b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1eff70, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x30feff73, SUC_lexicon_tags_15 },
    { 0xa03eff74, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9eff7a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbeff7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e9eff80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbeff84, SUC_lexicon_tags_37 },
    { 0xc8feff85, SUC_lexicon_tags_28 },
    { 0xb91eff84, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2deff8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x953eff8d, SUC_lexicon_tags_7 },
    { 0x709eff8d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x18beff91, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xb17eff94, SUC_lexicon_tags_0 },
    { 0x345eff94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa77eff97, SUC_lexicon_tags_11 },
    { 0x98deff98, SUC_lexicon_tags_84 },
    { 0x947eff99, SUC_lexicon_tags_68 },
    { 0xd8deff9a, SUC_lexicon_tags_1 },
    { 0x22deff9b, SUC_lexicon_tags_17 },
    { 0x85eff9b, SUC_lexicon_tags_25 },
    { 0x3edeff9b, SUC_lexicon_tags_15 },
    { 0xabfeff9b, SUC_lexicon_tags_35 },
    { 0x661eff9f, SUC_lexicon_tags_10 },
    { 0x5deff9b, SUC_lexicon_tags_15 },
    { 0x961effa1, SUC_lexicon_tags_77 },
    { 0x5d3effa0, SUC_lexicon_tags_28 },
    { 0x671effa3, SUC_lexicon_tags_7 },
    { 0xb1feffa4, SUC_lexicon_tags_31 },
    { 0xb39effa5, SUC_lexicon_tags_2 },
    { 0xa3beffa5, SUC_lexicon_tags_37 },
    { 0xb81effa7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31effb2, SUC_lexicon_tags_25 },
    { 0x2bbeffb2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x837effb6, SUC_lexicon_tags_7 },
    { 0x465effb7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x463effb9, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcfeffbe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3effc1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5effc6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcc1effc8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x199effca, SUC_lexicon_tags_82 },
    { 0xf0beffcb, SUC_lexicon_tags_47 },
    { 0x27feffcb, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7effd0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9effd4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6efeffd8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479effe3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5dbeffe5, SUC_lexicon_tags_47 },
    { 0x6f3effe5, SUC_lexicon_tags_21 },
    { 0x291effe7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9beffeb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9effef, SUC_lexicon_tags_15 },
    { 0x1e7effef, SUC_lexicon_tags_6 },
    { 0x5b3efff1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7efff4, SUC_lexicon_tags_12 },
    { 0x825efff4, SUC_lexicon_tags_7 },
    { 0x3bdefff6, SUC_lexicon_tags_19 },
    { 0x92defff7, SUC_lexicon_tags_1 },
    { 0x8f9efff8, SUC_lexicon_tags_10 },
    { 0x15fefff9, SUC_lexicon_tags_8 },
    { 0x9efefffa, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xa73efffd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9f0002, SUC_lexicon_tags_6 },
    { 0xf83f0002, SUC_lexicon_tags_14 },
    { 0xcc3f0004, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebf0007, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bf000f, SUC_lexicon_tags_85 },
    { 0x979f0010, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6d9f0012, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211f0019, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x905f001b, SUC_lexicon_tags_49 },
    { 0x979f001c, SUC_lexicon_tags_7 },
    { 0xdb7f001d, SUC_lexicon_tags_33 },
    { 0xd93f001b, SUC_lexicon_tags_37 },
    { 0xc75f001f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5dff0021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899f0026, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe21f0028, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdf002c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77f0033, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60ff0039, SUC_lexicon_tags_49 },
    { 0x76ff003a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5e5f003c, SUC_lexicon_tags_69 },
    { 0x943f003d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa49f003f, SUC_lexicon_tags_11 },
    { 0x633f0040, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf19f0042, SUC_lexicon_tags_11 },
    { 0x761f0043, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9f0047, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6bbf0049, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6abf004b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ff004e, SUC_lexicon_tags_19 },
    { 0x6a1f004f, SUC_lexicon_tags_25 },
    { 0x775f004f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdf005e, SUC_lexicon_tags_11 },
    { 0x8e1f005f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x58df0062, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83f0068, SUC_lexicon_tags_25 },
    { 0xc71f0068, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521f006e, SUC_lexicon_tags_15 },
    { 0xbe9f006f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x765f0071, SUC_lexicon_tags_19 },
    { 0x897f0072, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc23f0075, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1f007a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x45bf007c, SUC_lexicon_tags_15 },
    { 0x9f7f007c, SUC_lexicon_tags_37 },
    { 0xcb7f007d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9f0082, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3f0088, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279f008c, SUC_lexicon_tags_6 },
    { 0x821f008c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddf0090, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553f0096, SUC_lexicon_tags_45 },
    { 0xcdff0097, SUC_lexicon_tags_25 },
    { 0x9fdf0097, SUC_lexicon_tags_33 },
    { 0x151f0099, SUC_lexicon_tags_47 },
    { 0x931f009a, SUC_lexicon_tags_17 },
    { 0xfd5f009a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebf009e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb85f00a0, SUC_lexicon_tags_1 },
    { 0xa21f00a0, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3f00a5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39f00a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x413f00aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x785f00ad, SUC_lexicon_tags_46 },
    { 0xabdf00ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5df00b3, SUC_lexicon_tags_6 },
    { 0xf7ff00b4, SUC_lexicon_tags_27 },
    { 0x3e5f00b4, SUC_lexicon_tags_113 },
    { 0x989f00b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1f00ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a5f00bc, SUC_lexicon_tags_4 },
    { 0x9df00bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc65f00bf, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x753f00c1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c3f00c3, SUC_lexicon_tags_21 },
    { 0xbf00c4, SUC_lexicon_tags_27 },
    { 0x4bdf00c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcbff00c7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x937f00ca, SUC_lexicon_tags_2 },
    { 0xde9f00cb, SUC_lexicon_tags_37 },
    { 0xffbf00cb, SUC_lexicon_tags_2 },
    { 0x41bf00cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31f00d4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7f00d7, SUC_lexicon_tags_15 },
    { 0x637f00d8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ff00db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5f00e2, SUC_lexicon_tags_31 },
    { 0xb31f00e2, SUC_lexicon_tags_17 },
    { 0x2abf00e4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xae9f00e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05f00ee, SUC_lexicon_tags_30 },
    { 0x789f00ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbf00f2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1f0100, SUC_lexicon_tags_17 },
    { 0x52df0100, SUC_lexicon_tags_60 },
    { 0xde7f0102, SUC_lexicon_tags_12 },
    { 0xc9df0102, SUC_lexicon_tags_27 },
    { 0x315f0101, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x631f0106, SUC_lexicon_tags_25 },
    { 0xc7f0106, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x983f010a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111f0111, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x8cbf0113, SUC_lexicon_tags_17 },
    { 0xf87f0114, SUC_lexicon_tags_14 },
    { 0xa5f0114, SUC_lexicon_tags_7 },
    { 0x3f3f0115, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6bbf0118, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ff011c, SUC_lexicon_tags_6 },
    { 0x1c9f011d, SUC_lexicon_tags_12 },
    { 0x857f011e, SUC_lexicon_tags_1 },
    { 0x5a3f011f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x357f0121, SUC_lexicon_tags_25 },
    { 0x7f1f0122, SUC_lexicon_tags_7 },
    { 0x1d1f0123, SUC_lexicon_tags_37 },
    { 0x4fdf0124, SUC_lexicon_tags_2 },
    { 0xa91f0121, SUC_lexicon_tags_79 },
    { 0x1f9f0125, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f012a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb33f012c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bf012f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x983f0135, SUC_lexicon_tags_82 },
    { 0x791f0135, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadf0139, SUC_lexicon_tags_7 },
    { 0x573f0139, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd4ff013c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7f013f, SUC_lexicon_tags_1 },
    { 0x761f0140, SUC_lexicon_tags_17 },
    { 0xfbff0141, SUC_lexicon_tags_15 },
    { 0x287f0142, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14df0148, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xe7f014a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0x481f014d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x749f0150, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7f0156, SUC_lexicon_tags_37 },
    { 0x1d3f0157, SUC_lexicon_tags_87 },
    { 0x83df0158, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96df015f, SUC_lexicon_tags_25 },
    { 0x63bf015f, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x157f0162, SUC_lexicon_tags_37 },
    { 0xe03f0163, SUC_lexicon_tags_19 },
    { 0xeebf0163, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x35f0166, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbf016c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5f016e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bf0171, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xaf5f0173, SUC_lexicon_tags_43 },
    { 0xbeff0173, SUC_lexicon_tags_17 },
    { 0x317f0175, SUC_lexicon_tags_11 },
    { 0xcf3f0176, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x631f0178, SUC_lexicon_tags_12 },
    { 0x60bf0179, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ff017c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53f0185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5f018a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e9f0190, SUC_lexicon_tags_27 },
    { 0xeb5f0190, SUC_lexicon_tags_0 },
    { 0x8f7f0192, SUC_lexicon_tags_7 },
    { 0xbd3f0192, SUC_lexicon_tags_87 },
    { 0xac7f0192, SUC_lexicon_tags_235 },
    { 0x4f3f0195, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3f019a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x849f019c, SUC_lexicon_tags_120 },
    { 0x3df019d, SUC_lexicon_tags_70 },
    { 0xe59f019e, SUC_lexicon_tags_7 },
    { 0xb55f019f, SUC_lexicon_tags_28 },
    { 0x5ddf019f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39f01a4, SUC_lexicon_tags_1 },
    { 0x531f01a5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xee9f01a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215f01b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9f01b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5f01ba, SUC_lexicon_tags_43 },
    { 0x9f7f01bb, SUC_lexicon_tags_64 },
    { 0x3adf01bc, SUC_lexicon_tags_26 },
    { 0x31bf01bc, SUC_lexicon_tags_37 },
    { 0x87bf01be, SUC_lexicon_tags_11 },
    { 0xf43f01bf, SUC_lexicon_tags_15 },
    { 0x5bff01bf, SUC_lexicon_tags_15 },
    { 0xc4bf01c1, SUC_lexicon_tags_114 },
    { 0xda7f01bd, SUC_lexicon_tags_16 },
    { 0xa5f01c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x815f01c6, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0, NULL },
    { 0xb09f01c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edf01d0, SUC_lexicon_tags_25 },
    { 0x8fdf01d1, SUC_lexicon_tags_24 },
    { 0x855f01d2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f01da, SUC_lexicon_tags_15 },
    { 0x17df01db, SUC_lexicon_tags_11 },
    { 0x477f01dc, SUC_lexicon_tags_17 },
    { 0xc69f01dd, SUC_lexicon_tags_30 },
    { 0x271f01db, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbddf01e0, SUC_lexicon_tags_2 },
    { 0x9d1f01e0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bff01e8, SUC_lexicon_tags_25 },
    { 0xb0ff01e9, SUC_lexicon_tags_26 },
    { 0xfd3f01e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f01f0, SUC_lexicon_tags_37 },
    { 0xd79f01f1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x617f01f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x2c9f01f6, SUC_lexicon_tags_10 },
    { 0xbaff01f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff1f01fc, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9f01ff, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951f0207, SUC_lexicon_tags_35 },
    { 0x751f0208, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc2bf020a, SUC_lexicon_tags_37 },
    { 0xfb1f020b, SUC_lexicon_tags_11 },
    { 0x9a9f020c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61f0210, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x95df0212, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4ff0216, SUC_lexicon_tags_25 },
    { 0x3b1f0216, SUC_lexicon_tags_47 },
    { 0xd81f0218, SUC_lexicon_tags_10 },
    { 0x5d1f0218, SUC_lexicon_tags_17 },
    { 0xd11f0219, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x45ff021c, SUC_lexicon_tags_77 },
    { 0xa7bf021d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfff0220, SUC_lexicon_tags_21 },
    { 0x239f0221, SUC_lexicon_tags_15 },
    { 0xaa9f0220, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdf022b, SUC_lexicon_tags_19 },
    { 0x369f022c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x77df022f, SUC_lexicon_tags_14 },
    { 0xbf7f0230, SUC_lexicon_tags_11 },
    { 0xb55f0230, SUC_lexicon_tags_17 },
    { 0xba1f0232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ff0237, SUC_lexicon_tags_2 },
    { 0x61ff0237, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bf023b, SUC_lexicon_tags_2 },
    { 0x72df023c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x639f023f, SUC_lexicon_tags_25 },
    { 0xe41f0240, SUC_lexicon_tags_9 },
    { 0xf77f023f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x80df0243, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x989f0246, SUC_lexicon_tags_31 },
    { 0x323f0247, SUC_lexicon_tags_25 },
    { 0x46ff0247, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x829f024b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa1bf024d, SUC_lexicon_tags_38 },
    { 0xce3f024d, SUC_lexicon_tags_27 },
    { 0x86bf024f, SUC_lexicon_tags_37 },
    { 0xc33f0250, SUC_lexicon_tags_15 },
    { 0xefbf0251, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xddf0253, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x511f0255, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1df0257, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x82bf0259, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313f025e, SUC_lexicon_tags_11 },
    { 0x169f025f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3f0272, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd5bf0274, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325f0278, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc27f027c, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871f0280, SUC_lexicon_tags_25 },
    { 0xac9f0281, SUC_lexicon_tags_45 },
    { 0x25df0282, SUC_lexicon_tags_38 },
    { 0xc1bf0281, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4adf0285, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc83f0287, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adf028c, SUC_lexicon_tags_17 },
    { 0x67ff028d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccff0291, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xeedf0294, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x361f0299, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x43df029c, SUC_lexicon_tags_25 },
    { 0x315f029d, SUC_lexicon_tags_46 },
    { 0xe63f029e, SUC_lexicon_tags_16 },
    { 0xa1ff029f, SUC_lexicon_tags_37 },
    { 0x29bf029d, SUC_lexicon_tags_38 },
    { 0x55df029d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903f02a7, SUC_lexicon_tags_12 },
    { 0xee1f02a8, SUC_lexicon_tags_378 },
    { 0, NULL },
    { 0xc3ff02aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x709f02ad, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x455f02af, SUC_lexicon_tags_11 },
    { 0xc69f02b0, SUC_lexicon_tags_46 },
    { 0x413f02b0, SUC_lexicon_tags_17 },
    { 0xd3f02b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5f02b5, SUC_lexicon_tags_12 },
    { 0x773f02b6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf35f02b8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1f02bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391f02c0, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47f02c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f02c9, SUC_lexicon_tags_2 },
    { 0xb59f02c9, SUC_lexicon_tags_30 },
    { 0xecdf02cb, SUC_lexicon_tags_46 },
    { 0xd9ff02cb, SUC_lexicon_tags_29 },
    { 0xb3f02cd, SUC_lexicon_tags_17 },
    { 0x993f02cd, SUC_lexicon_tags_39 },
    { 0xbdbf02c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x251f02d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x197f02d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9df02d8, SUC_lexicon_tags_16 },
    { 0x6f5f02d9, SUC_lexicon_tags_43 },
    { 0x479f02d8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3df02de, SUC_lexicon_tags_25 },
    { 0xdd3f02de, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0xfa9f02e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdf02e5, SUC_lexicon_tags_25 },
    { 0xac1f02e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85df02ea, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x133f02ed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bf02f1, SUC_lexicon_tags_47 },
    { 0x673f02f1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x343f02f4, SUC_lexicon_tags_7 },
    { 0xf57f02f5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45f02f9, SUC_lexicon_tags_23 },
    { 0x8e5f02f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x895f02fc, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7f0301, SUC_lexicon_tags_17 },
    { 0xa23f0301, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5f0306, SUC_lexicon_tags_25 },
    { 0xe47f0307, SUC_lexicon_tags_44 },
    { 0x82ff0307, SUC_lexicon_tags_30 },
    { 0xbe1f0306, SUC_lexicon_tags_10 },
    { 0xdf9f030a, SUC_lexicon_tags_19 },
    { 0xf7df030b, SUC_lexicon_tags_11 },
    { 0x445f030c, SUC_lexicon_tags_27 },
    { 0xb47f030d, SUC_lexicon_tags_15 },
    { 0x2fdf030e, SUC_lexicon_tags_15 },
    { 0x48df030e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbf0312, SUC_lexicon_tags_62 },
    { 0x2adf0313, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x807f0315, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ff031a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7f0320, SUC_lexicon_tags_33 },
    { 0x3bff0321, SUC_lexicon_tags_33 },
    { 0xa69f0320, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bf0326, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe85f0328, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11f032b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423f0331, SUC_lexicon_tags_49 },
    { 0x503f0331, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f3f0334, SUC_lexicon_tags_7 },
    { 0xfd1f0334, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07f0339, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ff033c, SUC_lexicon_tags_2 },
    { 0xfebf033c, SUC_lexicon_tags_17 },
    { 0x5ebf033e, SUC_lexicon_tags_17 },
    { 0x8f9f033c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd31f0341, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351f0345, SUC_lexicon_tags_15 },
    { 0x47f0346, SUC_lexicon_tags_87 },
    { 0x87bf0346, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x993f0349, SUC_lexicon_tags_17 },
    { 0xb81f034a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657f0357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43f035d, SUC_lexicon_tags_27 },
    { 0x315f035e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93f0360, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3f0364, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119f0368, SUC_lexicon_tags_68 },
    { 0xe7f0369, SUC_lexicon_tags_45 },
    { 0x7bff0369, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5f036d, SUC_lexicon_tags_106 },
    { 0x19f036e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3f0372, SUC_lexicon_tags_11 },
    { 0x903f0372, SUC_lexicon_tags_45 },
    { 0x9e7f0374, SUC_lexicon_tags_47 },
    { 0xcebf0375, SUC_lexicon_tags_17 },
    { 0x1bf0376, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf1f0378, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x227f037a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6bff037c, SUC_lexicon_tags_2 },
    { 0xd71f037c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc65f037f, SUC_lexicon_tags_25 },
    { 0xc31f037f, SUC_lexicon_tags_18 },
    { 0x8adf037f, SUC_lexicon_tags_27 },
    { 0xe9bf0382, SUC_lexicon_tags_2 },
    { 0x13ff0383, SUC_lexicon_tags_1 },
    { 0x28bf0382, SUC_lexicon_tags_0 },
    { 0x82ff0382, SUC_lexicon_tags_15 },
    { 0x6abf0382, SUC_lexicon_tags_136 },
    { 0x2e7f0387, SUC_lexicon_tags_12 },
    { 0xc49f0388, SUC_lexicon_tags_60 },
    { 0x49ff0389, SUC_lexicon_tags_45 },
    { 0x657f038a, SUC_lexicon_tags_37 },
    { 0x39df038b, SUC_lexicon_tags_30 },
    { 0xaff038c, SUC_lexicon_tags_17 },
    { 0x7b5f0388, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ff0393, SUC_lexicon_tags_65 },
    { 0xaddf0393, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207f039a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5f03a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07f03a9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa17f03ab, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf03af, SUC_lexicon_tags_17 },
    { 0x39ff03b0, SUC_lexicon_tags_53 },
    { 0x51ff03b0, SUC_lexicon_tags_52 },
    { 0xf6ff03b2, SUC_lexicon_tags_124 },
    { 0x57df03b3, SUC_lexicon_tags_7 },
    { 0xff1f03b3, SUC_lexicon_tags_47 },
    { 0xc29f03b3, SUC_lexicon_tags_8 },
    { 0xaebf03b6, SUC_lexicon_tags_12 },
    { 0xfdbf03b6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbff03ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbddf03bc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7cff03be, SUC_lexicon_tags_37 },
    { 0xa91f03bf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeff03c2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729f03c6, SUC_lexicon_tags_17 },
    { 0x6adf03c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bf03cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391f03d3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadf03d6, SUC_lexicon_tags_30 },
    { 0xda5f03d7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6a7f03d9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcb1f03db, SUC_lexicon_tags_10 },
    { 0xf77f03dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x749f03de, SUC_lexicon_tags_16 },
    { 0x7a7f03df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3f03e4, SUC_lexicon_tags_57 },
    { 0x497f03e5, SUC_lexicon_tags_47 },
    { 0xa03f03e6, SUC_lexicon_tags_87 },
    { 0xe8df03e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3f03ed, SUC_lexicon_tags_0 },
    { 0xb1df03ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edf03f1, SUC_lexicon_tags_2 },
    { 0x973f03f1, SUC_lexicon_tags_1 },
    { 0xb21f03f3, SUC_lexicon_tags_82 },
    { 0x8f3f03f1, SUC_lexicon_tags_17 },
    { 0xd3ff03f3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3f03f8, SUC_lexicon_tags_11 },
    { 0xd79f03f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443f03fd, SUC_lexicon_tags_17 },
    { 0x55df03fe, SUC_lexicon_tags_7 },
    { 0x7e3f03ff, SUC_lexicon_tags_1 },
    { 0x313f0400, SUC_lexicon_tags_1 },
    { 0xc33f03fe, SUC_lexicon_tags_15 },
    { 0xc01f0402, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3f0405, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bf040b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1f0410, SUC_lexicon_tags_31 },
    { 0xad1f0411, SUC_lexicon_tags_43 },
    { 0xc25f0412, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x531f0416, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79f0418, SUC_lexicon_tags_17 },
    { 0x329f0418, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x201f041b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb25f041d, SUC_lexicon_tags_4 },
    { 0xbc1f041e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e5f0420, SUC_lexicon_tags_43 },
    { 0x49df0421, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xebdf0423, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5f0428, SUC_lexicon_tags_44 },
    { 0xb87f0429, SUC_lexicon_tags_1 },
    { 0x711f042a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3c1f042c, SUC_lexicon_tags_47 },
    { 0xeb9f042d, SUC_lexicon_tags_47 },
    { 0x861f042d, SUC_lexicon_tags_17 },
    { 0xaa7f042d, SUC_lexicon_tags_23 },
    { 0xeb9f0430, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3b3f0432, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe11f0434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x101f0436, SUC_lexicon_tags_65 },
    { 0x6c7f0437, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x85ff0439, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdf043c, SUC_lexicon_tags_10 },
    { 0x6d9f043c, SUC_lexicon_tags_17 },
    { 0xeaff043d, SUC_lexicon_tags_25 },
    { 0x4dff043d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98df0445, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39f0448, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x519f044b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823f044f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6df0452, SUC_lexicon_tags_233 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645f0459, SUC_lexicon_tags_11 },
    { 0xcc5f0459, SUC_lexicon_tags_37 },
    { 0xe17f045b, SUC_lexicon_tags_26 },
    { 0xccbf045c, SUC_lexicon_tags_37 },
    { 0xf3f045d, SUC_lexicon_tags_23 },
    { 0xe2df045d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfb1f0460, SUC_lexicon_tags_0 },
    { 0x35df0460, SUC_lexicon_tags_14 },
    { 0x149f0462, SUC_lexicon_tags_25 },
    { 0xc51f0463, SUC_lexicon_tags_3 },
    { 0xe15f0461, SUC_lexicon_tags_15 },
    { 0xd33f0465, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x635f0467, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243f046b, SUC_lexicon_tags_16 },
    { 0xa11f046c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x701f046e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65f0473, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67f0476, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcadf0479, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x797f047c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ff047e, SUC_lexicon_tags_7 },
    { 0x217f047f, SUC_lexicon_tags_1 },
    { 0xeb9f0480, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15f0486, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669f048b, SUC_lexicon_tags_6 },
    { 0x54bf048c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf0492, SUC_lexicon_tags_1 },
    { 0x939f0493, SUC_lexicon_tags_66 },
    { 0xa33f0493, SUC_lexicon_tags_60 },
    { 0x88ff0494, SUC_lexicon_tags_33 },
    { 0x3d5f0496, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfbff0498, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x263f049a, SUC_lexicon_tags_2 },
    { 0xbd5f049b, SUC_lexicon_tags_17 },
    { 0xe6df049c, SUC_lexicon_tags_12 },
    { 0x497f049d, SUC_lexicon_tags_27 },
    { 0x939f049e, SUC_lexicon_tags_19 },
    { 0x28bf049f, SUC_lexicon_tags_2 },
    { 0x4fbf049f, SUC_lexicon_tags_12 },
    { 0x297f049c, SUC_lexicon_tags_1 },
    { 0xd9ff049b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x673f04a5, SUC_lexicon_tags_21 },
    { 0xcd9f04a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f04a9, SUC_lexicon_tags_25 },
    { 0x53ff04aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71f04ad, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7f04b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101f04b9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3b5f04bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89f04c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f04c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebf04c9, SUC_lexicon_tags_26 },
    { 0x52ff04ca, SUC_lexicon_tags_28 },
    { 0x3cff04cb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xaaff04cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x14df04cf, SUC_lexicon_tags_0 },
    { 0x6f9f04cf, SUC_lexicon_tags_17 },
    { 0xd7f04d0, SUC_lexicon_tags_83 },
    { 0xebbf04d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21f04d5, SUC_lexicon_tags_37 },
    { 0xe1ff04d6, SUC_lexicon_tags_2 },
    { 0xb3df04d6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29df04de, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ff04e4, SUC_lexicon_tags_43 },
    { 0x34df04e4, SUC_lexicon_tags_37 },
    { 0x261f04e6, SUC_lexicon_tags_17 },
    { 0xf09f04e7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3f04ef, SUC_lexicon_tags_11 },
    { 0x637f04f0, SUC_lexicon_tags_15 },
    { 0xb6bf04ef, SUC_lexicon_tags_30 },
    { 0xb19f04f0, SUC_lexicon_tags_30 },
    { 0xa87f04f2, SUC_lexicon_tags_47 },
    { 0xae9f04f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5f04f8, SUC_lexicon_tags_12 },
    { 0xc79f04f8, SUC_lexicon_tags_11 },
    { 0x93df04fa, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89bf0503, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693f0508, SUC_lexicon_tags_0 },
    { 0x2a3f0509, SUC_lexicon_tags_37 },
    { 0x34df050a, SUC_lexicon_tags_15 },
    { 0x7d9f0509, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x547f050d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xb77f050f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x623f0511, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xc59f0513, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x255f0516, SUC_lexicon_tags_17 },
    { 0x98bf0517, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4edf051a, SUC_lexicon_tags_12 },
    { 0x495f051b, SUC_lexicon_tags_26 },
    { 0x873f051c, SUC_lexicon_tags_45 },
    { 0x6edf051d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1f0520, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5df0524, SUC_lexicon_tags_15 },
    { 0x77df0524, SUC_lexicon_tags_2 },
    { 0x535f0526, SUC_lexicon_tags_39 },
    { 0x25f0524, SUC_lexicon_tags_25 },
    { 0x7bbf0528, SUC_lexicon_tags_17 },
    { 0xb8bf0529, SUC_lexicon_tags_10 },
    { 0x305f052a, SUC_lexicon_tags_14 },
    { 0x367f052a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf51f052d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57df0532, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bf0535, SUC_lexicon_tags_33 },
    { 0x4adf0536, SUC_lexicon_tags_55 },
    { 0x66ff0537, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x5b5f0539, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1f053e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fdf0540, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abf0543, SUC_lexicon_tags_26 },
    { 0xa19f0544, SUC_lexicon_tags_14 },
    { 0x38df0545, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30df0549, SUC_lexicon_tags_12 },
    { 0x1b3f054a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xed5f054c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcb9f054e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x387f0551, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd15f0553, SUC_lexicon_tags_38 },
    { 0x6e1f0554, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xfb9f0556, SUC_lexicon_tags_6 },
    { 0xca7f0557, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc83f055a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bf055d, SUC_lexicon_tags_30 },
    { 0x4b9f055e, SUC_lexicon_tags_7 },
    { 0x163f055e, SUC_lexicon_tags_39 },
    { 0xc8df0560, SUC_lexicon_tags_1 },
    { 0x98ff0560, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3df0566, SUC_lexicon_tags_11 },
    { 0x71df0566, SUC_lexicon_tags_28 },
    { 0x431f0566, SUC_lexicon_tags_12 },
    { 0x4adf0569, SUC_lexicon_tags_37 },
    { 0x277f0569, SUC_lexicon_tags_310 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7f056e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3f0572, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7f0575, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49f0579, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xf4ff057b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bf0580, SUC_lexicon_tags_27 },
    { 0x12df0580, SUC_lexicon_tags_60 },
    { 0xf73f0580, SUC_lexicon_tags_6 },
    { 0x2d3f0583, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25f0586, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6df0589, SUC_lexicon_tags_17 },
    { 0xdcdf058a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7f0599, SUC_lexicon_tags_16 },
    { 0x5ddf059a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7f059d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bf05a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f05a5, SUC_lexicon_tags_83 },
    { 0xbe3f05a6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xbcdf05a8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31ff05aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5f05ad, SUC_lexicon_tags_25 },
    { 0x6d9f05ad, SUC_lexicon_tags_45 },
    { 0xb4bf05af, SUC_lexicon_tags_19 },
    { 0xc93f05af, SUC_lexicon_tags_48 },
    { 0xf17f05b1, SUC_lexicon_tags_11 },
    { 0xef5f05b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbf05b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd6df05b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc57f05ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x527f05bc, SUC_lexicon_tags_28 },
    { 0x9ebf05bc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1fdf05bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31f05c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x67f05c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x287f05cc, SUC_lexicon_tags_26 },
    { 0x1a5f05cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749f05d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57f05d5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f05e0, SUC_lexicon_tags_25 },
    { 0xfd5f05e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x477f05e3, SUC_lexicon_tags_11 },
    { 0x637f05e4, SUC_lexicon_tags_15 },
    { 0x21df05e4, SUC_lexicon_tags_8 },
    { 0x417f05e6, SUC_lexicon_tags_46 },
    { 0xdc7f05e4, SUC_lexicon_tags_7 },
    { 0x2a3f05e8, SUC_lexicon_tags_30 },
    { 0x683f05e4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0df05ef, SUC_lexicon_tags_12 },
    { 0x90ff05ef, SUC_lexicon_tags_17 },
    { 0x35f05f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeabf05f3, SUC_lexicon_tags_38 },
    { 0xebbf05f4, SUC_lexicon_tags_8 },
    { 0xd65f05f5, SUC_lexicon_tags_68 },
    { 0x143f05f6, SUC_lexicon_tags_14 },
    { 0xd83f05f5, SUC_lexicon_tags_25 },
    { 0x109f05f5, SUC_lexicon_tags_25 },
    { 0x41bf05f9, SUC_lexicon_tags_17 },
    { 0xdddf05fa, SUC_lexicon_tags_23 },
    { 0x891f05fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ff0602, SUC_lexicon_tags_1 },
    { 0x413f0603, SUC_lexicon_tags_30 },
    { 0x935f0603, SUC_lexicon_tags_19 },
    { 0x653f0604, SUC_lexicon_tags_1 },
    { 0x6ff0602, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3f060f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xffdf0612, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d1f0614, SUC_lexicon_tags_33 },
    { 0xa05f0615, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4e7f0617, SUC_lexicon_tags_15 },
    { 0x423f0617, SUC_lexicon_tags_35 },
    { 0xed5f0619, SUC_lexicon_tags_17 },
    { 0xb6ff061a, SUC_lexicon_tags_25 },
    { 0xcddf0617, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1f061f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5f0623, SUC_lexicon_tags_14 },
    { 0x56df0623, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabf062a, SUC_lexicon_tags_45 },
    { 0xde5f062b, SUC_lexicon_tags_7 },
    { 0x239f062a, SUC_lexicon_tags_39 },
    { 0x921f062c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61f0631, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe3f0633, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85f0637, SUC_lexicon_tags_37 },
    { 0x531f0637, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45f0640, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bf0643, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x33df0646, SUC_lexicon_tags_1 },
    { 0x1a9f0646, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959f064d, SUC_lexicon_tags_235 },
    { 0x94ff064e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bf0652, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x569f0654, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x20ff0656, SUC_lexicon_tags_60 },
    { 0xea3f0657, SUC_lexicon_tags_1 },
    { 0xffbf0658, SUC_lexicon_tags_25 },
    { 0xc91f0659, SUC_lexicon_tags_0 },
    { 0x765f0658, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x355f065c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341f0661, SUC_lexicon_tags_65 },
    { 0x9aff0662, SUC_lexicon_tags_28 },
    { 0x67df0663, SUC_lexicon_tags_49 },
    { 0xa09f0664, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3aff066a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb01f066f, SUC_lexicon_tags_140 },
    { 0x60df0670, SUC_lexicon_tags_12 },
    { 0xa55f0670, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557f0675, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x953f0677, SUC_lexicon_tags_5 },
    { 0xffff0678, SUC_lexicon_tags_43 },
    { 0x857f0679, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf067d, SUC_lexicon_tags_17 },
    { 0xc6ff067e, SUC_lexicon_tags_19 },
    { 0x969f067e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc89f0681, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ff0688, SUC_lexicon_tags_65 },
    { 0xc9df0689, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf068d, SUC_lexicon_tags_25 },
    { 0x11ff068e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8c9f0690, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfc1f0692, SUC_lexicon_tags_12 },
    { 0x98bf0693, SUC_lexicon_tags_47 },
    { 0xab5f0694, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0x149f0697, SUC_lexicon_tags_7 },
    { 0xeb3f0698, SUC_lexicon_tags_7 },
    { 0xfff0699, SUC_lexicon_tags_0 },
    { 0xa6bf069a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25f069e, SUC_lexicon_tags_25 },
    { 0xb17f069f, SUC_lexicon_tags_15 },
    { 0x953f06a0, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff06a4, SUC_lexicon_tags_17 },
    { 0x4dff06a4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf99f06a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x881f06a9, SUC_lexicon_tags_15 },
    { 0x371f06aa, SUC_lexicon_tags_19 },
    { 0xcc9f06ab, SUC_lexicon_tags_79 },
    { 0x515f06aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3df06b1, SUC_lexicon_tags_12 },
    { 0x5bdf06b2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7f06bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f06c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5ff06c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7f06c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1b1f06c8, SUC_lexicon_tags_55 },
    { 0x49f06c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5f06cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4adf06cf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9f06d2, SUC_lexicon_tags_43 },
    { 0x939f06d3, SUC_lexicon_tags_28 },
    { 0xe11f06d3, SUC_lexicon_tags_14 },
    { 0xf81f06d4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x763f06d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdf06dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9f06e0, SUC_lexicon_tags_47 },
    { 0x827f06e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6bbf06e3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x641f06e6, SUC_lexicon_tags_7 },
    { 0x9aff06e6, SUC_lexicon_tags_747 },
    { 0, NULL },
    { 0, NULL },
    { 0x695f06ea, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f06ed, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487f06f2, SUC_lexicon_tags_47 },
    { 0xb2ff06f3, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5f06fa, SUC_lexicon_tags_18 },
    { 0x85f06fb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173f0703, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799f0707, SUC_lexicon_tags_8 },
    { 0x87ff0708, SUC_lexicon_tags_12 },
    { 0xe09f0709, SUC_lexicon_tags_33 },
    { 0x817f070a, SUC_lexicon_tags_29 },
    { 0x4bdf070b, SUC_lexicon_tags_37 },
    { 0x35ff070c, SUC_lexicon_tags_12 },
    { 0x399f070d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3ff070f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ff0713, SUC_lexicon_tags_68 },
    { 0x761f0713, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf5df0716, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773f071a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801f071e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35f0724, SUC_lexicon_tags_15 },
    { 0x499f0725, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x387f0727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ff072a, SUC_lexicon_tags_25 },
    { 0x9e7f072b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215f072f, SUC_lexicon_tags_11 },
    { 0x1c1f0730, SUC_lexicon_tags_16 },
    { 0x163f0731, SUC_lexicon_tags_25 },
    { 0x9d3f0732, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bf0736, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bf0740, SUC_lexicon_tags_17 },
    { 0x3fff0741, SUC_lexicon_tags_95 },
    { 0x2a3f0742, SUC_lexicon_tags_43 },
    { 0x167f0743, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x569f0746, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05f0749, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x957f074c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87f074e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99f0751, SUC_lexicon_tags_47 },
    { 0xbb7f0752, SUC_lexicon_tags_21 },
    { 0x5c5f0752, SUC_lexicon_tags_29 },
    { 0xeb3f0754, SUC_lexicon_tags_37 },
    { 0xa41f0755, SUC_lexicon_tags_39 },
    { 0x71f0755, SUC_lexicon_tags_2 },
    { 0xc43f0757, SUC_lexicon_tags_15 },
    { 0x421f0754, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x251f075a, SUC_lexicon_tags_17 },
    { 0xe4ff075b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7bbf075d, SUC_lexicon_tags_31 },
    { 0xf4ff075e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971f0764, SUC_lexicon_tags_58 },
    { 0x383f0765, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa9df0767, SUC_lexicon_tags_45 },
    { 0x9f3f0767, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcdff076a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1f076d, SUC_lexicon_tags_17 },
    { 0x6a5f076e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x177f0770, SUC_lexicon_tags_2 },
    { 0x25f0771, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23df0776, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x93df0779, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf5df077b, SUC_lexicon_tags_81 },
    { 0xf33f077c, SUC_lexicon_tags_25 },
    { 0x955f077b, SUC_lexicon_tags_29 },
    { 0xe75f077b, SUC_lexicon_tags_12 },
    { 0xd1df077f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x5c3f0781, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe71f0784, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335f0788, SUC_lexicon_tags_2 },
    { 0xc33f0789, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25f078c, SUC_lexicon_tags_11 },
    { 0xe7df078c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1f0792, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1f0797, SUC_lexicon_tags_1 },
    { 0x92ff0798, SUC_lexicon_tags_9 },
    { 0x8bbf0798, SUC_lexicon_tags_11 },
    { 0x901f079a, SUC_lexicon_tags_15 },
    { 0xb4bf079b, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3f079e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedff07a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5f07a6, SUC_lexicon_tags_28 },
    { 0x249f07a6, SUC_lexicon_tags_17 },
    { 0x10ff07a8, SUC_lexicon_tags_38 },
    { 0xd53f07a9, SUC_lexicon_tags_47 },
    { 0x795f07a9, SUC_lexicon_tags_47 },
    { 0xf9bf07ab, SUC_lexicon_tags_12 },
    { 0x5bff07a8, SUC_lexicon_tags_7 },
    { 0x757f07a6, SUC_lexicon_tags_33 },
    { 0x77f07ae, SUC_lexicon_tags_29 },
    { 0x55bf07ad, SUC_lexicon_tags_19 },
    { 0x9ddf07b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05f07b5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833f07ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadf07bd, SUC_lexicon_tags_10 },
    { 0x351f07be, SUC_lexicon_tags_17 },
    { 0x6abf07bf, SUC_lexicon_tags_11 },
    { 0x26bf07c0, SUC_lexicon_tags_7 },
    { 0x563f07be, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633f07c6, SUC_lexicon_tags_9 },
    { 0x107f07c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f07cb, SUC_lexicon_tags_15 },
    { 0x941f07cb, SUC_lexicon_tags_37 },
    { 0x211f07cd, SUC_lexicon_tags_37 },
    { 0x211f07cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x7e7f07d0, SUC_lexicon_tags_27 },
    { 0x4b3f07d1, SUC_lexicon_tags_21 },
    { 0x475f07d2, SUC_lexicon_tags_7 },
    { 0x917f07d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761f07dd, SUC_lexicon_tags_27 },
    { 0x757f07dd, SUC_lexicon_tags_63 },
    { 0xe75f07dd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3df07e6, SUC_lexicon_tags_37 },
    { 0xa87f07e6, SUC_lexicon_tags_6 },
    { 0x63df07e7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd37f07ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7f07f0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3c3f07f2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x65f07f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf1f07f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x113f07fa, SUC_lexicon_tags_15 },
    { 0x1cdf07fb, SUC_lexicon_tags_25 },
    { 0x949f07fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7df07ff, SUC_lexicon_tags_34 },
    { 0x145f0800, SUC_lexicon_tags_2 },
    { 0xe6ff0801, SUC_lexicon_tags_38 },
    { 0x129f0802, SUC_lexicon_tags_31 },
    { 0x513f0801, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83f0807, SUC_lexicon_tags_64 },
    { 0x5f0807, SUC_lexicon_tags_7 },
    { 0xefdf0807, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x975f080b, SUC_lexicon_tags_49 },
    { 0x12df080b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa9ff080e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x639f0810, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df0813, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3fff0815, SUC_lexicon_tags_12 },
    { 0xf63f0815, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bf081a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77df081e, SUC_lexicon_tags_8 },
    { 0xa93f081e, SUC_lexicon_tags_2 },
    { 0x75ff081e, SUC_lexicon_tags_18 },
    { 0xdbbf081f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bf0828, SUC_lexicon_tags_12 },
    { 0xdfff0829, SUC_lexicon_tags_8 },
    { 0x183f0828, SUC_lexicon_tags_63 },
    { 0x717f082b, SUC_lexicon_tags_26 },
    { 0xfcff0829, SUC_lexicon_tags_6 },
    { 0x191f082c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x663f0830, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7f0835, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f083c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xefff083e, SUC_lexicon_tags_12 },
    { 0x3bff083f, SUC_lexicon_tags_14 },
    { 0x9d5f083f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bf0843, SUC_lexicon_tags_1 },
    { 0xca9f0844, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x657f0846, SUC_lexicon_tags_45 },
    { 0xc79f0847, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf084a, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f084d, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98df0851, SUC_lexicon_tags_1 },
    { 0x63df0852, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f0857, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87f085c, SUC_lexicon_tags_14 },
    { 0x92df085c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5f0863, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5df0865, SUC_lexicon_tags_1 },
    { 0xd45f0866, SUC_lexicon_tags_12 },
    { 0x45ff0865, SUC_lexicon_tags_2 },
    { 0x7d5f0868, SUC_lexicon_tags_15 },
    { 0xca3f0869, SUC_lexicon_tags_32 },
    { 0x58ff0867, SUC_lexicon_tags_9 },
    { 0x2a5f086b, SUC_lexicon_tags_56 },
    { 0x423f0867, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x945f086e, SUC_lexicon_tags_11 },
    { 0x99ff086f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3f0873, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fff0877, SUC_lexicon_tags_5 },
    { 0x869f0878, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa2df087a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f087e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf0882, SUC_lexicon_tags_28 },
    { 0x369f0882, SUC_lexicon_tags_0 },
    { 0x8bff0884, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x17df0887, SUC_lexicon_tags_38 },
    { 0x95ff0888, SUC_lexicon_tags_7 },
    { 0x207f0889, SUC_lexicon_tags_60 },
    { 0xaf5f088a, SUC_lexicon_tags_23 },
    { 0xed5f088b, SUC_lexicon_tags_1 },
    { 0x30bf0887, SUC_lexicon_tags_61 },
    { 0xc67f0887, SUC_lexicon_tags_6 },
    { 0xa1f088e, SUC_lexicon_tags_95 },
    { 0x195f088f, SUC_lexicon_tags_1 },
    { 0x1cbf0890, SUC_lexicon_tags_1 },
    { 0xa55f088e, SUC_lexicon_tags_11 },
    { 0x48bf088b, SUC_lexicon_tags_11 },
    { 0x63bf0890, SUC_lexicon_tags_11 },
    { 0x383f0891, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1f089e, SUC_lexicon_tags_15 },
    { 0x2b7f089f, SUC_lexicon_tags_23 },
    { 0x207f08a0, SUC_lexicon_tags_47 },
    { 0x98ff08a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3f08a4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xed9f08a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ff08aa, SUC_lexicon_tags_19 },
    { 0xa57f08ab, SUC_lexicon_tags_28 },
    { 0x11df08ac, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5f08af, SUC_lexicon_tags_7 },
    { 0xb03f08b0, SUC_lexicon_tags_15 },
    { 0xb1f08b1, SUC_lexicon_tags_30 },
    { 0x63bf08b2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb55f08b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41f08b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeff08bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547f08c1, SUC_lexicon_tags_1 },
    { 0x783f08c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9cdf08c4, SUC_lexicon_tags_31 },
    { 0xd83f08c5, SUC_lexicon_tags_1 },
    { 0x9cbf08c5, SUC_lexicon_tags_25 },
    { 0x4c3f08c6, SUC_lexicon_tags_25 },
    { 0x4cbf08c8, SUC_lexicon_tags_61 },
    { 0xafdf08c8, SUC_lexicon_tags_7 },
    { 0x5dbf08c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x59df08cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3a9f08cf, SUC_lexicon_tags_65 },
    { 0xd03f08d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3f08d5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319f08dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9f08e1, SUC_lexicon_tags_17 },
    { 0x3df08e2, SUC_lexicon_tags_6 },
    { 0xabf08e2, SUC_lexicon_tags_23 },
    { 0x78df08e2, SUC_lexicon_tags_11 },
    { 0xe5df08e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45f08e9, SUC_lexicon_tags_9 },
    { 0x987f08ea, SUC_lexicon_tags_39 },
    { 0xff3f08ea, SUC_lexicon_tags_49 },
    { 0x413f08ec, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761f08f3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6df08f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f08f8, SUC_lexicon_tags_13 },
    { 0x16ff08f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45f08fc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x589f08ff, SUC_lexicon_tags_26 },
    { 0xbe5f0900, SUC_lexicon_tags_82 },
    { 0x8bf0901, SUC_lexicon_tags_95 },
    { 0x653f0902, SUC_lexicon_tags_38 },
    { 0xca5f0903, SUC_lexicon_tags_31 },
    { 0xd03f0904, SUC_lexicon_tags_138 },
    { 0xb21f0905, SUC_lexicon_tags_7 },
    { 0x33df0906, SUC_lexicon_tags_46 },
    { 0x8cbf0907, SUC_lexicon_tags_55 },
    { 0x361f0906, SUC_lexicon_tags_26 },
    { 0xaebf0909, SUC_lexicon_tags_47 },
    { 0xd39f090a, SUC_lexicon_tags_37 },
    { 0xa63f090a, SUC_lexicon_tags_45 },
    { 0x3b1f090a, SUC_lexicon_tags_0 },
    { 0x389f08ff, SUC_lexicon_tags_29 },
    { 0xe33f0905, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x30ff0911, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1f0916, SUC_lexicon_tags_9 },
    { 0xbb1f0917, SUC_lexicon_tags_11 },
    { 0x6b5f0917, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xec3f091b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2aff091d, SUC_lexicon_tags_15 },
    { 0xfc9f091d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1f0924, SUC_lexicon_tags_47 },
    { 0x311f0925, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59f0929, SUC_lexicon_tags_19 },
    { 0x735f092a, SUC_lexicon_tags_7 },
    { 0xe01f092b, SUC_lexicon_tags_45 },
    { 0xffdf092c, SUC_lexicon_tags_38 },
    { 0x46bf092b, SUC_lexicon_tags_30 },
    { 0xcbf092b, SUC_lexicon_tags_60 },
    { 0x1bdf092f, SUC_lexicon_tags_23 },
    { 0xb09f0930, SUC_lexicon_tags_19 },
    { 0x68bf0930, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9f0935, SUC_lexicon_tags_31 },
    { 0xaa7f0936, SUC_lexicon_tags_17 },
    { 0x189f0937, SUC_lexicon_tags_1 },
    { 0x9d9f0938, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd1f093a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bf093f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843f0943, SUC_lexicon_tags_17 },
    { 0x415f0944, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x787f0946, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8ff0948, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dff094e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685f0953, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f0956, SUC_lexicon_tags_37 },
    { 0x95f0956, SUC_lexicon_tags_11 },
    { 0x609f0958, SUC_lexicon_tags_7 },
    { 0xaadf0959, SUC_lexicon_tags_12 },
    { 0x935f0956, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x567f095c, SUC_lexicon_tags_37 },
    { 0x55ff095c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39f0963, SUC_lexicon_tags_12 },
    { 0xea5f0964, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x32bf0966, SUC_lexicon_tags_11 },
    { 0x42bf0966, SUC_lexicon_tags_25 },
    { 0x8eff0966, SUC_lexicon_tags_2 },
    { 0xf93f0969, SUC_lexicon_tags_12 },
    { 0xee5f096a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb61f096c, SUC_lexicon_tags_7 },
    { 0xa7df096d, SUC_lexicon_tags_25 },
    { 0x7bdf096c, SUC_lexicon_tags_39 },
    { 0xf59f096f, SUC_lexicon_tags_19 },
    { 0x70bf096f, SUC_lexicon_tags_15 },
    { 0x64ff096e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bf0974, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ff097a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd73f097c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x347f097e, SUC_lexicon_tags_19 },
    { 0xc8df097e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x855f0981, SUC_lexicon_tags_11 },
    { 0xeb9f0981, SUC_lexicon_tags_118 },
    { 0x4b7f0983, SUC_lexicon_tags_26 },
    { 0x99df0983, SUC_lexicon_tags_17 },
    { 0x5c1f0985, SUC_lexicon_tags_1 },
    { 0xb73f0986, SUC_lexicon_tags_3 },
    { 0x63bf0981, SUC_lexicon_tags_2 },
    { 0x3e7f0988, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x665f098a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bf098e, SUC_lexicon_tags_31 },
    { 0x247f098f, SUC_lexicon_tags_19 },
    { 0x46ff0990, SUC_lexicon_tags_25 },
    { 0xef5f0991, SUC_lexicon_tags_7 },
    { 0x4f5f098e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5f0995, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389f0999, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abf099d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdf09a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7eff09a2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7f09a5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edf09a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x19ff09ab, SUC_lexicon_tags_3 },
    { 0x181f09ac, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89f09b3, SUC_lexicon_tags_2 },
    { 0x5cff09b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a3f09b6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935f09bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x263f09bf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9df09c3, SUC_lexicon_tags_17 },
    { 0x8ff09c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f7f09c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x781f09c8, SUC_lexicon_tags_21 },
    { 0xa5ff09c9, SUC_lexicon_tags_55 },
    { 0x9c3f09ca, SUC_lexicon_tags_14 },
    { 0x119f09ca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dff09ce, SUC_lexicon_tags_7 },
    { 0x31bf09ce, SUC_lexicon_tags_1 },
    { 0xa43f09d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c3f09d2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1f09d5, SUC_lexicon_tags_12 },
    { 0x8a9f09d6, SUC_lexicon_tags_2 },
    { 0xf45f09d7, SUC_lexicon_tags_13 },
    { 0x50bf09d7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbff09dd, SUC_lexicon_tags_57 },
    { 0x919f09dd, SUC_lexicon_tags_11 },
    { 0x737f09df, SUC_lexicon_tags_3 },
    { 0x421f09dd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x4abf09e2, SUC_lexicon_tags_2 },
    { 0x4dff09e3, SUC_lexicon_tags_23 },
    { 0x2a3f09e3, SUC_lexicon_tags_25 },
    { 0x583f09e3, SUC_lexicon_tags_19 },
    { 0x1ddf09e4, SUC_lexicon_tags_11 },
    { 0xd71f09e7, SUC_lexicon_tags_25 },
    { 0xca3f09e7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475f09ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x87ff09ef, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555f09f3, SUC_lexicon_tags_23 },
    { 0xc6ff09f4, SUC_lexicon_tags_106 },
    { 0x4cdf09f5, SUC_lexicon_tags_25 },
    { 0x2aff09f4, SUC_lexicon_tags_26 },
    { 0x6a5f09f6, SUC_lexicon_tags_7 },
    { 0x59f09f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf09fc, SUC_lexicon_tags_38 },
    { 0x331f09fc, SUC_lexicon_tags_49 },
    { 0x7fff09fe, SUC_lexicon_tags_25 },
    { 0x3aff09ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x96bf0a01, SUC_lexicon_tags_63 },
    { 0x835f0a01, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83f0a08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29f0a16, SUC_lexicon_tags_25 },
    { 0xdc3f0a17, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xf29f0a19, SUC_lexicon_tags_7 },
    { 0xc67f0a1a, SUC_lexicon_tags_18 },
    { 0xfedf0a1b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x479f0a1d, SUC_lexicon_tags_2 },
    { 0x371f0a1d, SUC_lexicon_tags_27 },
    { 0xbb7f0a1f, SUC_lexicon_tags_23 },
    { 0xc09f0a1e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeff0a23, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599f0a28, SUC_lexicon_tags_11 },
    { 0x91f0a29, SUC_lexicon_tags_35 },
    { 0x373f0a2a, SUC_lexicon_tags_2 },
    { 0x41f0a2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3a1f0a2d, SUC_lexicon_tags_63 },
    { 0x221f0a2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bf0a34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9f0a37, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x22ff0a39, SUC_lexicon_tags_0 },
    { 0xc07f0a3a, SUC_lexicon_tags_33 },
    { 0x651f0a3a, SUC_lexicon_tags_11 },
    { 0x525f0a39, SUC_lexicon_tags_2 },
    { 0x405f0a3d, SUC_lexicon_tags_37 },
    { 0x52bf0a3c, SUC_lexicon_tags_13 },
    { 0x683f0a3d, SUC_lexicon_tags_35 },
    { 0xfcbf0a3d, SUC_lexicon_tags_62 },
    { 0xc9f0a3b, SUC_lexicon_tags_9 },
    { 0x8dbf0a42, SUC_lexicon_tags_27 },
    { 0x169f0a43, SUC_lexicon_tags_11 },
    { 0x5b5f0a44, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdf0a4e, SUC_lexicon_tags_1 },
    { 0x351f0a4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x815f0a51, SUC_lexicon_tags_7 },
    { 0x41f0a51, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29f0a56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95f0a5b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ff0a5f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f0a62, SUC_lexicon_tags_63 },
    { 0x7edf0a63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41f0a66, SUC_lexicon_tags_25 },
    { 0xbf1f0a67, SUC_lexicon_tags_15 },
    { 0x1f0a68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711f0a6e, SUC_lexicon_tags_43 },
    { 0xc11f0a6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc53f0a72, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x69ff0a74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c7f0a76, SUC_lexicon_tags_11 },
    { 0x659f0a77, SUC_lexicon_tags_71 },
    { 0x5dff0a78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abf0a7c, SUC_lexicon_tags_37 },
    { 0x525f0a7c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x949f0a81, SUC_lexicon_tags_2 },
    { 0xc73f0a82, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463f0a86, SUC_lexicon_tags_282 },
    { 0x9c7f0a87, SUC_lexicon_tags_19 },
    { 0xbd1f0a88, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0x843f0a8a, SUC_lexicon_tags_7 },
    { 0x6eff0a8b, SUC_lexicon_tags_17 },
    { 0xee5f0a8c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14df0a91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdf0a95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x78bf0a98, SUC_lexicon_tags_55 },
    { 0xbcdf0a99, SUC_lexicon_tags_11 },
    { 0x397f0a98, SUC_lexicon_tags_4 },
    { 0xf6df0a99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x23ff0a9d, SUC_lexicon_tags_6 },
    { 0x567f0a9d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7f0aa3, SUC_lexicon_tags_29 },
    { 0xcd5f0aa4, SUC_lexicon_tags_16 },
    { 0xe6ff0aa5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd03f0aa7, SUC_lexicon_tags_1499 },
    { 0x525f0aa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x175f0aaa, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96df0ab4, SUC_lexicon_tags_1 },
    { 0xb13f0ab5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20df0aba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35f0ac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x73ff0ac4, SUC_lexicon_tags_11 },
    { 0x9abf0ac5, SUC_lexicon_tags_23 },
    { 0x1a5f0ac5, SUC_lexicon_tags_7 },
    { 0x903f0ac6, SUC_lexicon_tags_11 },
    { 0x45bf0ac8, SUC_lexicon_tags_23 },
    { 0x88df0ac8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x413f0acb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91f0acf, SUC_lexicon_tags_25 },
    { 0xc01f0ad0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abf0ad5, SUC_lexicon_tags_417 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdf0ad9, SUC_lexicon_tags_45 },
    { 0x183f0ada, SUC_lexicon_tags_14 },
    { 0x10ff0adb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bf0ae0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x385f0ae3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadf0aea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbbf0aec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe87f0aee, SUC_lexicon_tags_31 },
    { 0x717f0aee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ff0af2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55f0af8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833f0afd, SUC_lexicon_tags_163 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5f0b00, SUC_lexicon_tags_14 },
    { 0x3adf0b01, SUC_lexicon_tags_23 },
    { 0x613f0b02, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ff0b0a, SUC_lexicon_tags_1 },
    { 0x3c1f0b0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03f0b10, SUC_lexicon_tags_19 },
    { 0x7bdf0b11, SUC_lexicon_tags_17 },
    { 0x473f0b12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f0b16, SUC_lexicon_tags_60 },
    { 0xbdf0b17, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xecdf0b19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd7f0b1b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8cdf0b1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe7df0b1f, SUC_lexicon_tags_17 },
    { 0xf7df0b20, SUC_lexicon_tags_25 },
    { 0xd57f0b1f, SUC_lexicon_tags_17 },
    { 0xdf7f0b22, SUC_lexicon_tags_60 },
    { 0x1a5f0b23, SUC_lexicon_tags_1 },
    { 0x631f0b23, SUC_lexicon_tags_17 },
    { 0xdc5f0b24, SUC_lexicon_tags_28 },
    { 0xf93f0b25, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x20df0b29, SUC_lexicon_tags_29 },
    { 0xdd7f0b2a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47f0b2e, SUC_lexicon_tags_16 },
    { 0x75f0b2e, SUC_lexicon_tags_84 },
    { 0x4f9f0b30, SUC_lexicon_tags_17 },
    { 0x71df0b31, SUC_lexicon_tags_11 },
    { 0xd0bf0b32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f0b37, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x57f0b3a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffbf0b3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x325f0b42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x763f0b44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123f0b48, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7f0b50, SUC_lexicon_tags_61 },
    { 0xb75f0b51, SUC_lexicon_tags_17 },
    { 0xc8ff0b51, SUC_lexicon_tags_11 },
    { 0x3a7f0b52, SUC_lexicon_tags_23 },
    { 0x789f0b54, SUC_lexicon_tags_37 },
    { 0x2d9f0b50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bf0b5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61f0b63, SUC_lexicon_tags_14 },
    { 0x20df0b64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x75df0b67, SUC_lexicon_tags_11 },
    { 0x83bf0b68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871f0b6e, SUC_lexicon_tags_15 },
    { 0xd5ff0b6f, SUC_lexicon_tags_7 },
    { 0xd7ff0b70, SUC_lexicon_tags_23 },
    { 0x63df0b71, SUC_lexicon_tags_37 },
    { 0xfb9f0b70, SUC_lexicon_tags_25 },
    { 0x465f0b71, SUC_lexicon_tags_14 },
    { 0x115f0b74, SUC_lexicon_tags_43 },
    { 0xa45f0b75, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x167f0b78, SUC_lexicon_tags_63 },
    { 0x873f0b78, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5f0b7e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df0b81, SUC_lexicon_tags_33 },
    { 0x54df0b81, SUC_lexicon_tags_4 },
    { 0x691f0b82, SUC_lexicon_tags_37 },
    { 0x45df0b84, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xac5f0b86, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd6df0b88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559f0b8c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dbf0b8e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75f0b91, SUC_lexicon_tags_2 },
    { 0x839f0b92, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69f0b95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0df0b99, SUC_lexicon_tags_0 },
    { 0x619f0b9a, SUC_lexicon_tags_25 },
    { 0x987f0b9b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589f0ba0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x18df0ba3, SUC_lexicon_tags_1 },
    { 0xec3f0ba4, SUC_lexicon_tags_7 },
    { 0xa7df0ba3, SUC_lexicon_tags_48 },
    { 0x1cdf0ba6, SUC_lexicon_tags_7 },
    { 0x357f0ba5, SUC_lexicon_tags_9 },
    { 0x47bf0ba8, SUC_lexicon_tags_25 },
    { 0x163f0ba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x681f0bac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981f0bb0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1f0bb6, SUC_lexicon_tags_11 },
    { 0x203f0bb7, SUC_lexicon_tags_7 },
    { 0x1eff0bb8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ff0bbd, SUC_lexicon_tags_11 },
    { 0x1aff0bbe, SUC_lexicon_tags_25 },
    { 0xc09f0bbd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8df0bc1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3f0bc7, SUC_lexicon_tags_3 },
    { 0xf71f0bc8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793f0bd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdf0bd7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195f0bdc, SUC_lexicon_tags_6 },
    { 0x2ff0bdd, SUC_lexicon_tags_70 },
    { 0xddf0bdd, SUC_lexicon_tags_7 },
    { 0xa3f0bdf, SUC_lexicon_tags_36 },
    { 0xc3ff0be0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33f0be3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645f0be9, SUC_lexicon_tags_30 },
    { 0x653f0bea, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5f0bf1, SUC_lexicon_tags_8 },
    { 0x847f0bf2, SUC_lexicon_tags_45 },
    { 0xd45f0bf2, SUC_lexicon_tags_14 },
    { 0xd37f0bf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbfdf0bf6, SUC_lexicon_tags_0 },
    { 0xd7f0bf6, SUC_lexicon_tags_15 },
    { 0xc29f0bf8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423f0bfc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe5f0bfe, SUC_lexicon_tags_15 },
    { 0x42df0bfe, SUC_lexicon_tags_47 },
    { 0x827f0bfe, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe5df0c02, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641f0c07, SUC_lexicon_tags_68 },
    { 0x959f0c07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33f0c0c, SUC_lexicon_tags_7 },
    { 0xd29f0c0c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45df0c18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe9ff0c1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b1f0c1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x56df0c1e, SUC_lexicon_tags_15 },
    { 0x2e1f0c1e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ff0c22, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f0c25, SUC_lexicon_tags_17 },
    { 0x4fdf0c26, SUC_lexicon_tags_61 },
    { 0xd05f0c26, SUC_lexicon_tags_32 },
    { 0x4f7f0c26, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf1bf0c2a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe47f0c2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439f0c33, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5f0c37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f0c3a, SUC_lexicon_tags_11 },
    { 0x5e9f0c3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1f0c3e, SUC_lexicon_tags_4 },
    { 0x671f0c3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5a5f0c41, SUC_lexicon_tags_47 },
    { 0xab9f0c42, SUC_lexicon_tags_3 },
    { 0xe91f0c43, SUC_lexicon_tags_68 },
    { 0xaa1f0c42, SUC_lexicon_tags_65 },
    { 0x3f5f0c45, SUC_lexicon_tags_12 },
    { 0x1abf0c46, SUC_lexicon_tags_7 },
    { 0xe17f0c47, SUC_lexicon_tags_16 },
    { 0xbaff0c43, SUC_lexicon_tags_59 },
    { 0x23df0c49, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf0c4e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x281f0c51, SUC_lexicon_tags_1 },
    { 0x73f0c52, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1f0c58, SUC_lexicon_tags_8 },
    { 0xeb7f0c58, SUC_lexicon_tags_25 },
    { 0xdcbf0c58, SUC_lexicon_tags_53 },
    { 0x43bf0c58, SUC_lexicon_tags_77 },
    { 0xba5f0c5c, SUC_lexicon_tags_45 },
    { 0xc23f0c5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8c7f0c5f, SUC_lexicon_tags_43 },
    { 0xdedf0c60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x923f0c62, SUC_lexicon_tags_17 },
    { 0xff1f0c63, SUC_lexicon_tags_37 },
    { 0x9b9f0c64, SUC_lexicon_tags_4 },
    { 0x5e3f0c65, SUC_lexicon_tags_62 },
    { 0x481f0c66, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x797f0c68, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55f0c6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495f0c74, SUC_lexicon_tags_33 },
    { 0x4adf0c75, SUC_lexicon_tags_38 },
    { 0x1ddf0c75, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x67df0c79, SUC_lexicon_tags_4 },
    { 0xb2df0c7a, SUC_lexicon_tags_37 },
    { 0x9bbf0c7a, SUC_lexicon_tags_15 },
    { 0xb99f0c7b, SUC_lexicon_tags_37 },
    { 0x965f0c7d, SUC_lexicon_tags_60 },
    { 0x4d1f0c7e, SUC_lexicon_tags_11 },
    { 0x6d5f0c7f, SUC_lexicon_tags_12 },
    { 0xbbf0c7f, SUC_lexicon_tags_1 },
    { 0x305f0c7a, SUC_lexicon_tags_14 },
    { 0x53ff0c7b, SUC_lexicon_tags_25 },
    { 0x6d7f0c83, SUC_lexicon_tags_43 },
    { 0x651f0c84, SUC_lexicon_tags_83 },
    { 0xcd7f0c84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87f0c8d, SUC_lexicon_tags_1 },
    { 0x4a9f0c8e, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x889f0c92, SUC_lexicon_tags_11 },
    { 0xc89f0c93, SUC_lexicon_tags_1 },
    { 0x4fbf0c93, SUC_lexicon_tags_17 },
    { 0xc69f0c94, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x969f0c99, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9f0c9c, SUC_lexicon_tags_11 },
    { 0xde7f0c9c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb2ff0c9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x715f0ca1, SUC_lexicon_tags_14 },
    { 0x845f0ca2, SUC_lexicon_tags_1 },
    { 0x31ff0ca3, SUC_lexicon_tags_17 },
    { 0xce3f0ca4, SUC_lexicon_tags_60 },
    { 0xdfbf0ca4, SUC_lexicon_tags_46 },
    { 0x8b7f0ca6, SUC_lexicon_tags_39 },
    { 0xc93f0ca5, SUC_lexicon_tags_37 },
    { 0x61f0ca8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abf0cac, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x54ff0cae, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdf0cb2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933f0cb6, SUC_lexicon_tags_6 },
    { 0x943f0cb7, SUC_lexicon_tags_15 },
    { 0xb69f0cb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427f0cbe, SUC_lexicon_tags_47 },
    { 0x9d7f0cbf, SUC_lexicon_tags_11 },
    { 0x13df0cc0, SUC_lexicon_tags_32 },
    { 0xa33f0cc1, SUC_lexicon_tags_11 },
    { 0x167f0cc2, SUC_lexicon_tags_26 },
    { 0x2c3f0cc3, SUC_lexicon_tags_1 },
    { 0xf1f0cbf, SUC_lexicon_tags_2 },
    { 0x531f0cc5, SUC_lexicon_tags_25 },
    { 0x39df0cc0, SUC_lexicon_tags_37 },
    { 0x231f0cc7, SUC_lexicon_tags_60 },
    { 0x8b9f0cbe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739f0ccf, SUC_lexicon_tags_37 },
    { 0xd6df0cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1f0cd4, SUC_lexicon_tags_8 },
    { 0xea7f0cd4, SUC_lexicon_tags_37 },
    { 0xd89f0cd4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf0bf0cd8, SUC_lexicon_tags_25 },
    { 0xdc9f0cd9, SUC_lexicon_tags_37 },
    { 0x461f0cd8, SUC_lexicon_tags_30 },
    { 0xd1df0cdb, SUC_lexicon_tags_12 },
    { 0xa23f0cda, SUC_lexicon_tags_60 },
    { 0x459f0cdb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3f0ce2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55f0ce7, SUC_lexicon_tags_37 },
    { 0xc99f0ce8, SUC_lexicon_tags_63 },
    { 0x6abf0ce9, SUC_lexicon_tags_34 },
    { 0x3f5f0cea, SUC_lexicon_tags_49 },
    { 0x9d5f0cea, SUC_lexicon_tags_60 },
    { 0x765f0ce9, SUC_lexicon_tags_12 },
    { 0x683f0ce9, SUC_lexicon_tags_17 },
    { 0xc5f0cee, SUC_lexicon_tags_85 },
    { 0xa41f0cee, SUC_lexicon_tags_63 },
    { 0xa69f0cef, SUC_lexicon_tags_0 },
    { 0xd5bf0cee, SUC_lexicon_tags_77 },
    { 0x395f0cf2, SUC_lexicon_tags_47 },
    { 0xbcdf0ce7, SUC_lexicon_tags_15 },
    { 0x815f0cf4, SUC_lexicon_tags_63 },
    { 0xe41f0cf5, SUC_lexicon_tags_11 },
    { 0x10ff0cf4, SUC_lexicon_tags_27 },
    { 0xdcbf0cf7, SUC_lexicon_tags_0 },
    { 0xde9f0cf5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x597f0cfb, SUC_lexicon_tags_7 },
    { 0x493f0cfc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bff0cff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x949f0d02, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf4df0d04, SUC_lexicon_tags_12 },
    { 0x6dff0d04, SUC_lexicon_tags_10 },
    { 0x6df0d06, SUC_lexicon_tags_39 },
    { 0x293f0d05, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x829f0d0f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541f0d13, SUC_lexicon_tags_17 },
    { 0x955f0d14, SUC_lexicon_tags_14 },
    { 0x845f0d15, SUC_lexicon_tags_38 },
    { 0x47df0d16, SUC_lexicon_tags_9 },
    { 0xcd3f0d17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1f0d1a, SUC_lexicon_tags_25 },
    { 0x9eff0d1b, SUC_lexicon_tags_13 },
    { 0xfddf0d1c, SUC_lexicon_tags_36 },
    { 0x507f0d1a, SUC_lexicon_tags_25 },
    { 0x70ff0d1e, SUC_lexicon_tags_49 },
    { 0x3e1f0d1f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe87f0d26, SUC_lexicon_tags_26 },
    { 0xa99f0d27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bf0d2f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xcbbf0d31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f0d34, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x67ff0d37, SUC_lexicon_tags_14 },
    { 0xe25f0d38, SUC_lexicon_tags_14 },
    { 0x715f0d39, SUC_lexicon_tags_62 },
    { 0xbbf0d39, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x739f0d3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a9f0d3e, SUC_lexicon_tags_1 },
    { 0x3f1f0d3f, SUC_lexicon_tags_9 },
    { 0xa07f0d40, SUC_lexicon_tags_1 },
    { 0xee7f0d40, SUC_lexicon_tags_27 },
    { 0xd05f0d41, SUC_lexicon_tags_2 },
    { 0xe7ff0d41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adf0d4a, SUC_lexicon_tags_25 },
    { 0x47f0d4b, SUC_lexicon_tags_47 },
    { 0x975f0d4a, SUC_lexicon_tags_12 },
    { 0xfc5f0d4d, SUC_lexicon_tags_17 },
    { 0xa8bf0d4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff0d52, SUC_lexicon_tags_94 },
    { 0x739f0d53, SUC_lexicon_tags_17 },
    { 0xc85f0d54, SUC_lexicon_tags_68 },
    { 0xda9f0d54, SUC_lexicon_tags_246 },
    { 0xa5bf0d55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731f0d5a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdf0d5f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa21f0d62, SUC_lexicon_tags_17 },
    { 0xd7ff0d63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c5f0d65, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x25f0d67, SUC_lexicon_tags_37 },
    { 0x49f0d67, SUC_lexicon_tags_112 },
    { 0x66df0d69, SUC_lexicon_tags_12 },
    { 0x957f0d69, SUC_lexicon_tags_47 },
    { 0xc1f0d69, SUC_lexicon_tags_33 },
    { 0x9f9f0d6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163f0d70, SUC_lexicon_tags_87 },
    { 0x25ff0d71, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abf0d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5f0d7b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x779f0d7e, SUC_lexicon_tags_16 },
    { 0x899f0d7e, SUC_lexicon_tags_30 },
    { 0x1fdf0d80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x333f0d82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947f0d86, SUC_lexicon_tags_17 },
    { 0x3a3f0d86, SUC_lexicon_tags_27 },
    { 0x4d3f0d88, SUC_lexicon_tags_17 },
    { 0xf09f0d87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xeb9f0d8b, SUC_lexicon_tags_1 },
    { 0x3fff0d8c, SUC_lexicon_tags_0 },
    { 0xe45f0d8b, SUC_lexicon_tags_2 },
    { 0x7c3f0d8e, SUC_lexicon_tags_47 },
    { 0xef7f0d8f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x809f0d91, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x887f0d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x149f0d96, SUC_lexicon_tags_23 },
    { 0x4d5f0d97, SUC_lexicon_tags_12 },
    { 0x689f0d98, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbd3f0d9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91f0da0, SUC_lexicon_tags_14 },
    { 0x309f0da0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9f0da8, SUC_lexicon_tags_2 },
    { 0xe4ff0da8, SUC_lexicon_tags_63 },
    { 0x5a5f0daa, SUC_lexicon_tags_82 },
    { 0x24ff0dab, SUC_lexicon_tags_17 },
    { 0x123f0dac, SUC_lexicon_tags_4 },
    { 0x951f0dad, SUC_lexicon_tags_11 },
    { 0x6bdf0dae, SUC_lexicon_tags_16 },
    { 0xd81f0daa, SUC_lexicon_tags_507 },
    { 0, NULL },
    { 0x3cff0db1, SUC_lexicon_tags_17 },
    { 0x137f0db2, SUC_lexicon_tags_26 },
    { 0x8df0db2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x3b5f0db5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf83f0db7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3f0dbc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf0dbf, SUC_lexicon_tags_15 },
    { 0x277f0dc0, SUC_lexicon_tags_6 },
    { 0x233f0dc1, SUC_lexicon_tags_0 },
    { 0xacdf0dc0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931f0dc7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f0dcb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc65f0dcd, SUC_lexicon_tags_77 },
    { 0x10df0dce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x983f0dd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93f0dd4, SUC_lexicon_tags_25 },
    { 0x6e1f0dd5, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5df0dda, SUC_lexicon_tags_2 },
    { 0xbebf0ddb, SUC_lexicon_tags_46 },
    { 0x10bf0ddc, SUC_lexicon_tags_8 },
    { 0x1b7f0ddd, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x22bf0ddf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aff0de3, SUC_lexicon_tags_1150 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0df0de7, SUC_lexicon_tags_9 },
    { 0xa37f0de8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x841f0dea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7f0ded, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x7bff0def, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb1f0df2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x357f0df5, SUC_lexicon_tags_7 },
    { 0x965f0df6, SUC_lexicon_tags_46 },
    { 0xa5bf0df7, SUC_lexicon_tags_26 },
    { 0x22ff0df7, SUC_lexicon_tags_15 },
    { 0x851f0df9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf0dfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565f0e00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997f0e08, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x4fdf0e0a, SUC_lexicon_tags_31 },
    { 0xbabf0e0b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e1f0e0d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5f0e11, SUC_lexicon_tags_11 },
    { 0xc1df0e11, SUC_lexicon_tags_12 },
    { 0xe8bf0e13, SUC_lexicon_tags_39 },
    { 0x309f0e13, SUC_lexicon_tags_19 },
    { 0xbb7f0e14, SUC_lexicon_tags_49 },
    { 0x58df0e16, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9f0e18, SUC_lexicon_tags_12 },
    { 0xf5f0e19, SUC_lexicon_tags_33 },
    { 0xe9bf0e1a, SUC_lexicon_tags_0 },
    { 0xc59f0e1b, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255f0e1f, SUC_lexicon_tags_28 },
    { 0x77bf0e20, SUC_lexicon_tags_27 },
    { 0xc43f0e21, SUC_lexicon_tags_151 },
    { 0xadff0e22, SUC_lexicon_tags_2 },
    { 0xaf9f0e20, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x697f0e26, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1f0e29, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8b3f0e2b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9f0e2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221f0e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40ff0e37, SUC_lexicon_tags_17 },
    { 0x719f0e38, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x835f0e3b, SUC_lexicon_tags_12 },
    { 0xc93f0e3b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdf0e3f, SUC_lexicon_tags_0 },
    { 0xbbf0e40, SUC_lexicon_tags_6 },
    { 0xd2bf0e41, SUC_lexicon_tags_31 },
    { 0xa93f0e40, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x143f0e45, SUC_lexicon_tags_47 },
    { 0x98ff0e45, SUC_lexicon_tags_12 },
    { 0x5dff0e47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1f0e4d, SUC_lexicon_tags_15 },
    { 0x9b9f0e4e, SUC_lexicon_tags_17 },
    { 0x315f0e4e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9f0e52, SUC_lexicon_tags_121 },
    { 0xbd9f0e52, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc79f0e55, SUC_lexicon_tags_19 },
    { 0xaa5f0e56, SUC_lexicon_tags_49 },
    { 0xd9f0e56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423f0e5c, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9df0e62, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x639f0e64, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbf3f0e66, SUC_lexicon_tags_25 },
    { 0xf3ff0e66, SUC_lexicon_tags_45 },
    { 0xfbbf0e68, SUC_lexicon_tags_49 },
    { 0x701f0e68, SUC_lexicon_tags_6 },
    { 0xe93f0e6a, SUC_lexicon_tags_48 },
    { 0x76ff0e68, SUC_lexicon_tags_25 },
    { 0x11f0e6c, SUC_lexicon_tags_7 },
    { 0xa5ff0e6a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf0e73, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bf0e76, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbf0e79, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1f0e7c, SUC_lexicon_tags_17 },
    { 0x7c7f0e7c, SUC_lexicon_tags_28 },
    { 0xe7ff0e7d, SUC_lexicon_tags_7 },
    { 0x54bf0e7f, SUC_lexicon_tags_60 },
    { 0x98bf0e80, SUC_lexicon_tags_49 },
    { 0x8cff0e81, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733f0e85, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadf0e89, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe03f0e8c, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0x615f0e8e, SUC_lexicon_tags_1 },
    { 0x793f0e8f, SUC_lexicon_tags_11 },
    { 0x7fdf0e8e, SUC_lexicon_tags_77 },
    { 0x9cbf0e91, SUC_lexicon_tags_17 },
    { 0xde9f0e92, SUC_lexicon_tags_31 },
    { 0xa7f0e8f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5f0e97, SUC_lexicon_tags_65 },
    { 0xa7df0e98, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22df0e9c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919f0ea0, SUC_lexicon_tags_17 },
    { 0x6b5f0ea1, SUC_lexicon_tags_4 },
    { 0x9b9f0ea1, SUC_lexicon_tags_37 },
    { 0x70bf0ea3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3f0eb1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9f0eb6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3df0eba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x567f0ebc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x461f0ebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47f0ec5, SUC_lexicon_tags_37 },
    { 0x4f1f0ec6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbf0ec9, SUC_lexicon_tags_15 },
    { 0xb2ff0eca, SUC_lexicon_tags_15 },
    { 0x323f0eca, SUC_lexicon_tags_55 },
    { 0xcd7f0ecc, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xd85f0ece, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb5f0ed0, SUC_lexicon_tags_68 },
    { 0x82df0ed0, SUC_lexicon_tags_10 },
    { 0x35f0ed0, SUC_lexicon_tags_7 },
    { 0x51bf0ed0, SUC_lexicon_tags_0 },
    { 0x149f0ed4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14bf0ed6, SUC_lexicon_tags_17 },
    { 0xa95f0ed7, SUC_lexicon_tags_19 },
    { 0x90bf0ed8, SUC_lexicon_tags_29 },
    { 0x5f9f0ed8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x633f0edc, SUC_lexicon_tags_17 },
    { 0x231f0edc, SUC_lexicon_tags_60 },
    { 0xc9ff0ede, SUC_lexicon_tags_33 },
    { 0x5f9f0ede, SUC_lexicon_tags_14 },
    { 0x759f0ede, SUC_lexicon_tags_11 },
    { 0x3f5f0ee1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x999f0ee3, SUC_lexicon_tags_2 },
    { 0x70ff0ee4, SUC_lexicon_tags_23 },
    { 0x4d5f0ee4, SUC_lexicon_tags_0 },
    { 0x251f0ee6, SUC_lexicon_tags_21 },
    { 0xd73f0ee7, SUC_lexicon_tags_19 },
    { 0x94ff0ee8, SUC_lexicon_tags_6 },
    { 0x13f0ee9, SUC_lexicon_tags_29 },
    { 0xe4bf0eea, SUC_lexicon_tags_15 },
    { 0x4eff0ee9, SUC_lexicon_tags_28 },
    { 0x7d5f0eec, SUC_lexicon_tags_37 },
    { 0x6fff0eea, SUC_lexicon_tags_0 },
    { 0xbabf0eee, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28df0ef4, SUC_lexicon_tags_47 },
    { 0xd3f0ef5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9df0efc, SUC_lexicon_tags_1 },
    { 0x235f0efc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaf7f0eff, SUC_lexicon_tags_31 },
    { 0xe1f0eff, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x947f0f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bf0f06, SUC_lexicon_tags_1 },
    { 0x163f0f07, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f0f0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7df0f0d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe11f0f0f, SUC_lexicon_tags_14 },
    { 0xcf7f0f10, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3adf0f13, SUC_lexicon_tags_12 },
    { 0x6c7f0f14, SUC_lexicon_tags_14 },
    { 0x5fbf0f13, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0xf31f0f17, SUC_lexicon_tags_27 },
    { 0x1d9f0f18, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb9ff0f1a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57f0f1d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x12bf0f1f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x533f0f21, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9f0f25, SUC_lexicon_tags_4 },
    { 0xdaff0f26, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x59ff0f28, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abf0f2d, SUC_lexicon_tags_25 },
    { 0x7df0f2e, SUC_lexicon_tags_37 },
    { 0x4aff0f2f, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x8f7f0f31, SUC_lexicon_tags_27 },
    { 0xfbdf0f31, SUC_lexicon_tags_104 },
    { 0x73f0f33, SUC_lexicon_tags_26 },
    { 0x8d9f0f34, SUC_lexicon_tags_4 },
    { 0x795f0f35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ff0f3c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb67f0f3e, SUC_lexicon_tags_17 },
    { 0x4c7f0f3f, SUC_lexicon_tags_4 },
    { 0x939f0f3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3f0f44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e9f0f49, SUC_lexicon_tags_2 },
    { 0x95ff0f4a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9a3f0f4c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbf0f50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7f0f53, SUC_lexicon_tags_11 },
    { 0x735f0f54, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x1c7f0f56, SUC_lexicon_tags_9 },
    { 0xb83f0f56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49f0f5e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7f0f62, SUC_lexicon_tags_10 },
    { 0xf19f0f63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535f0f68, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ff0f6f, SUC_lexicon_tags_15 },
    { 0x8fff0f70, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bf0f73, SUC_lexicon_tags_2 },
    { 0x419f0f74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x269f0f76, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x6f5f0f78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605f0f85, SUC_lexicon_tags_15 },
    { 0x66ff0f85, SUC_lexicon_tags_26 },
    { 0x49df0f85, SUC_lexicon_tags_1 },
    { 0x3e5f0f87, SUC_lexicon_tags_2 },
    { 0x15bf0f89, SUC_lexicon_tags_26 },
    { 0x5b3f0f8a, SUC_lexicon_tags_23 },
    { 0xd63f0f8b, SUC_lexicon_tags_25 },
    { 0x4c3f0f8b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9f0f91, SUC_lexicon_tags_7 },
    { 0x323f0f92, SUC_lexicon_tags_55 },
    { 0x7b1f0f92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20ff0f9c, SUC_lexicon_tags_6 },
    { 0xbe9f0f9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa3ff0f9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ff0fa5, SUC_lexicon_tags_9 },
    { 0x5cdf0fa6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe05f0fa8, SUC_lexicon_tags_25 },
    { 0x40ff0fa9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48df0fad, SUC_lexicon_tags_15 },
    { 0xed9f0fae, SUC_lexicon_tags_15 },
    { 0x20df0faf, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743f0fb7, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf5df0fb9, SUC_lexicon_tags_15 },
    { 0xdf9f0fba, SUC_lexicon_tags_43 },
    { 0xc59f0fbb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439f0fbf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x64df0fc2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdf0fc6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x13f0fc8, SUC_lexicon_tags_49 },
    { 0x7ddf0fc8, SUC_lexicon_tags_264 },
    { 0xf79f0fca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf17f0fcc, SUC_lexicon_tags_25 },
    { 0x7f9f0fcd, SUC_lexicon_tags_25 },
    { 0xeff0fcc, SUC_lexicon_tags_25 },
    { 0x235f0fcf, SUC_lexicon_tags_37 },
    { 0xb95f0fd0, SUC_lexicon_tags_37 },
    { 0xd81f0fd0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xe43f0fd3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425f0fd9, SUC_lexicon_tags_14 },
    { 0x3eff0fda, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbf0fde, SUC_lexicon_tags_7 },
    { 0x2b1f0fdf, SUC_lexicon_tags_46 },
    { 0x5bdf0fe0, SUC_lexicon_tags_27 },
    { 0xe23f0fe0, SUC_lexicon_tags_4 },
    { 0x525f0fe0, SUC_lexicon_tags_15 },
    { 0x267f0fe3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x3fff0fe5, SUC_lexicon_tags_37 },
    { 0x6b3f0fe6, SUC_lexicon_tags_25 },
    { 0x65ff0fe7, SUC_lexicon_tags_26 },
    { 0xac7f0fe8, SUC_lexicon_tags_11 },
    { 0x561f0fe8, SUC_lexicon_tags_12 },
    { 0xd1bf0fe5, SUC_lexicon_tags_12 },
    { 0x9fdf0fea, SUC_lexicon_tags_4 },
    { 0x521f0fec, SUC_lexicon_tags_7 },
    { 0x83df0fec, SUC_lexicon_tags_15 },
    { 0x62df0fee, SUC_lexicon_tags_38 },
    { 0xc5df0fee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5f0ff8, SUC_lexicon_tags_28 },
    { 0x16df0ff9, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253f0ffd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7f1001, SUC_lexicon_tags_1 },
    { 0x627f1001, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f5f1004, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x43df1006, SUC_lexicon_tags_63 },
    { 0x6edf1007, SUC_lexicon_tags_32 },
    { 0x429f1008, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x663f100a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbf100e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddf1011, SUC_lexicon_tags_45 },
    { 0x1bff1012, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1f1018, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9f1025, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x45f1027, SUC_lexicon_tags_66 },
    { 0xe37f1028, SUC_lexicon_tags_27 },
    { 0x583f1029, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc51f102b, SUC_lexicon_tags_21 },
    { 0xde1f102c, SUC_lexicon_tags_1 },
    { 0x9e1f102c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf1035, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdfdf1037, SUC_lexicon_tags_37 },
    { 0xab5f1037, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedf103e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x555f1041, SUC_lexicon_tags_2 },
    { 0x4f7f1041, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1f1048, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x97df104a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f1052, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x193f1055, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ff105b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1f1061, SUC_lexicon_tags_12 },
    { 0x7e7f1062, SUC_lexicon_tags_33 },
    { 0x91ff1063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1f1066, SUC_lexicon_tags_39 },
    { 0xe0ff1066, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x66bf106a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1f106e, SUC_lexicon_tags_23 },
    { 0x765f106f, SUC_lexicon_tags_0 },
    { 0x8f1f1070, SUC_lexicon_tags_7 },
    { 0x871f106f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa35f1073, SUC_lexicon_tags_7 },
    { 0x78bf1074, SUC_lexicon_tags_108 },
    { 0x289f1075, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x857f1077, SUC_lexicon_tags_95 },
    { 0xd93f1078, SUC_lexicon_tags_12 },
    { 0x2bff1079, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd9f107b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45f107e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3f1083, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603f1087, SUC_lexicon_tags_25 },
    { 0x67bf1088, SUC_lexicon_tags_11 },
    { 0x74df1089, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65df108e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9f1092, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa87f109c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0df10a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x613f10a6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x46df10a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x255f10aa, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xff3f10ac, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91f10b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ff10b8, SUC_lexicon_tags_68 },
    { 0x5f5f10b8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5df10bb, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89f10bf, SUC_lexicon_tags_1 },
    { 0xb0ff10c0, SUC_lexicon_tags_25 },
    { 0xd99f10c0, SUC_lexicon_tags_56 },
    { 0xbabf10c2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x663f10c4, SUC_lexicon_tags_31 },
    { 0x1d7f10c5, SUC_lexicon_tags_6 },
    { 0x86ff10c6, SUC_lexicon_tags_6 },
    { 0xed7f10c6, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1f10cb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1f10ce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x947f10d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa11f10d7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x161f10d9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xce9f10db, SUC_lexicon_tags_9 },
    { 0x3f3f10dc, SUC_lexicon_tags_16 },
    { 0xe7bf10dd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x54bf10e0, SUC_lexicon_tags_15 },
    { 0x3d5f10e1, SUC_lexicon_tags_9 },
    { 0x14bf10e2, SUC_lexicon_tags_7 },
    { 0xa3ff10e3, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x467f10e6, SUC_lexicon_tags_7 },
    { 0xbf1f10e6, SUC_lexicon_tags_6 },
    { 0xaf9f10e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1f10ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8bdf10ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf10f1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe81f10f3, SUC_lexicon_tags_1 },
    { 0x7cff10f4, SUC_lexicon_tags_43 },
    { 0x7df10f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x529f10f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x1aff10fa, SUC_lexicon_tags_29 },
    { 0x4e7f10fb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1f10fe, SUC_lexicon_tags_14 },
    { 0x5bff10ff, SUC_lexicon_tags_26 },
    { 0x641f10ff, SUC_lexicon_tags_1 },
    { 0xcebf10fe, SUC_lexicon_tags_30 },
    { 0x547f1100, SUC_lexicon_tags_55 },
    { 0x463f10fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb89f1106, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9f110a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x737f110c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x401f110e, SUC_lexicon_tags_37 },
    { 0x6d9f110f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295f1113, SUC_lexicon_tags_0 },
    { 0x7aff1113, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x181f1117, SUC_lexicon_tags_23 },
    { 0x3adf1117, SUC_lexicon_tags_60 },
    { 0x987f1119, SUC_lexicon_tags_45 },
    { 0x9cdf1118, SUC_lexicon_tags_1 },
    { 0xb2df1119, SUC_lexicon_tags_9 },
    { 0x5b5f111c, SUC_lexicon_tags_63 },
    { 0x567f111c, SUC_lexicon_tags_14 },
    { 0x615f111e, SUC_lexicon_tags_12 },
    { 0x4e1f111e, SUC_lexicon_tags_25 },
    { 0x90df111f, SUC_lexicon_tags_53 },
    { 0xb69f111f, SUC_lexicon_tags_27 },
    { 0xc53f1122, SUC_lexicon_tags_7 },
    { 0xa49f1122, SUC_lexicon_tags_37 },
    { 0x3eff111f, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0x999f1127, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38df112c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c5f1130, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbedf1132, SUC_lexicon_tags_37 },
    { 0xa7ff1133, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f113f, SUC_lexicon_tags_49 },
    { 0x221f1140, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bf1144, SUC_lexicon_tags_63 },
    { 0xec7f1144, SUC_lexicon_tags_46 },
    { 0x389f1144, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x351f1149, SUC_lexicon_tags_37 },
    { 0xb0bf114a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x355f114f, SUC_lexicon_tags_12 },
    { 0xb29f1150, SUC_lexicon_tags_7 },
    { 0x57bf114f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x7f5f1153, SUC_lexicon_tags_45 },
    { 0xfbff1153, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225f115a, SUC_lexicon_tags_33 },
    { 0x147f115a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61f1162, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01f1166, SUC_lexicon_tags_37 },
    { 0x671f1167, SUC_lexicon_tags_33 },
    { 0x31bf1167, SUC_lexicon_tags_23 },
    { 0x15ff1169, SUC_lexicon_tags_38 },
    { 0x419f1169, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf3df116c, SUC_lexicon_tags_11 },
    { 0x35df116d, SUC_lexicon_tags_17 },
    { 0x4ff116e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x10bf1170, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe15f1172, SUC_lexicon_tags_9 },
    { 0xb9ff1173, SUC_lexicon_tags_30 },
    { 0xee7f1173, SUC_lexicon_tags_63 },
    { 0xbfbf1175, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817f1181, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbf1185, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511f118a, SUC_lexicon_tags_1 },
    { 0x14ff118b, SUC_lexicon_tags_25 },
    { 0x72df118c, SUC_lexicon_tags_14 },
    { 0x4dbf118d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x643f118f, SUC_lexicon_tags_27 },
    { 0xc69f1190, SUC_lexicon_tags_7 },
    { 0xf95f1191, SUC_lexicon_tags_15 },
    { 0xe65f1192, SUC_lexicon_tags_25 },
    { 0x877f1193, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4e9f1195, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bf1199, SUC_lexicon_tags_17 },
    { 0xe71f1199, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcbdf119c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee7f119e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883f11a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa3ff11a6, SUC_lexicon_tags_30 },
    { 0xf6bf11a7, SUC_lexicon_tags_1 },
    { 0xf7f11a8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f11ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xd89f11af, SUC_lexicon_tags_9 },
    { 0x1d3f11af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253f11b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x59df11b6, SUC_lexicon_tags_29 },
    { 0xa4bf11b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21f11bb, SUC_lexicon_tags_47 },
    { 0xda1f11bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5f11c3, SUC_lexicon_tags_33 },
    { 0x81df11c4, SUC_lexicon_tags_7 },
    { 0x437f11c4, SUC_lexicon_tags_7 },
    { 0x683f11c6, SUC_lexicon_tags_15 },
    { 0xa2df11c3, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fff11cd, SUC_lexicon_tags_254 },
    { 0xa3ff11ce, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xa61f11d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edf11d6, SUC_lexicon_tags_2 },
    { 0x663f11d7, SUC_lexicon_tags_1 },
    { 0xb5f11d8, SUC_lexicon_tags_30 },
    { 0x79bf11d9, SUC_lexicon_tags_17 },
    { 0xec5f11d7, SUC_lexicon_tags_17 },
    { 0x487f11db, SUC_lexicon_tags_16 },
    { 0xa7df11db, SUC_lexicon_tags_17 },
    { 0xb4bf11dd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x729f11e0, SUC_lexicon_tags_27 },
    { 0x94ff11e1, SUC_lexicon_tags_12 },
    { 0xcb3f11e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x1b9f11e4, SUC_lexicon_tags_1 },
    { 0x145f11e4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5c5f11e7, SUC_lexicon_tags_156 },
    { 0xbdbf11e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x849f11eb, SUC_lexicon_tags_1 },
    { 0x2b7f11eb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abf11f2, SUC_lexicon_tags_11 },
    { 0x865f11f3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507f1200, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523f1208, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f1210, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x417f1213, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7f1216, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631f121d, SUC_lexicon_tags_65 },
    { 0x299f121e, SUC_lexicon_tags_15 },
    { 0x1bbf121f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20df1223, SUC_lexicon_tags_17 },
    { 0xbedf1224, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f122a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7f122f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ff1237, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09f123b, SUC_lexicon_tags_37 },
    { 0x8e5f123b, SUC_lexicon_tags_79 },
    { 0xe7ff123d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xed1f123f, SUC_lexicon_tags_48 },
    { 0xd3df123f, SUC_lexicon_tags_21 },
    { 0xdd5f123f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa49f1244, SUC_lexicon_tags_30 },
    { 0x627f1245, SUC_lexicon_tags_25 },
    { 0x80bf1246, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0xdb1f1248, SUC_lexicon_tags_37 },
    { 0xf19f1249, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x263f1250, SUC_lexicon_tags_27 },
    { 0x75f1251, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x659f1254, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325f1259, SUC_lexicon_tags_15 },
    { 0x757f125a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x58bf125d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ff1261, SUC_lexicon_tags_107 },
    { 0x8cff1262, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbf1267, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69bf1269, SUC_lexicon_tags_17 },
    { 0x779f126a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae7f1276, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf03f127a, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29f127e, SUC_lexicon_tags_235 },
    { 0xc7ff127f, SUC_lexicon_tags_39 },
    { 0xdf5f127f, SUC_lexicon_tags_29 },
    { 0xbb7f1281, SUC_lexicon_tags_15 },
    { 0x52bf1282, SUC_lexicon_tags_11 },
    { 0xea1f1282, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bff1286, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd07f1288, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753f1290, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2c1f1292, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x24df1294, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8ff1296, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x72df1299, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb61f129b, SUC_lexicon_tags_37 },
    { 0x46df129c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe51f129e, SUC_lexicon_tags_27 },
    { 0x94df129f, SUC_lexicon_tags_38 },
    { 0x20df129e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bbf12a5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eff12a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb35f12ab, SUC_lexicon_tags_1 },
    { 0xb1f12ac, SUC_lexicon_tags_12 },
    { 0xe49f12ad, SUC_lexicon_tags_63 },
    { 0x5d1f12ad, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x581f12b1, SUC_lexicon_tags_2 },
    { 0xd8df12b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07f12b8, SUC_lexicon_tags_9 },
    { 0x6edf12b9, SUC_lexicon_tags_1 },
    { 0x189f12ba, SUC_lexicon_tags_2 },
    { 0x8a5f12b9, SUC_lexicon_tags_29 },
    { 0x543f12bc, SUC_lexicon_tags_15 },
    { 0xeb1f12b9, SUC_lexicon_tags_21 },
    { 0x8edf12ba, SUC_lexicon_tags_12 },
    { 0x649f12ba, SUC_lexicon_tags_1 },
    { 0x81bf12c0, SUC_lexicon_tags_39 },
    { 0x2c5f12c1, SUC_lexicon_tags_4 },
    { 0x7a9f12bc, SUC_lexicon_tags_13 },
    { 0x3a5f12c3, SUC_lexicon_tags_31 },
    { 0x815f12c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9f12c7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb99f12c9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x807f12cb, SUC_lexicon_tags_15 },
    { 0xb51f12cc, SUC_lexicon_tags_47 },
    { 0xf65f12cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x963f12cf, SUC_lexicon_tags_27 },
    { 0xfd3f12d0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8d7f12d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x707f12d4, SUC_lexicon_tags_69 },
    { 0x81bf12d4, SUC_lexicon_tags_7 },
    { 0x685f12d4, SUC_lexicon_tags_0 },
    { 0x5e5f12d6, SUC_lexicon_tags_33 },
    { 0x57f12d8, SUC_lexicon_tags_38 },
    { 0x95df12d6, SUC_lexicon_tags_12 },
    { 0x949f12da, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13df12e0, SUC_lexicon_tags_65 },
    { 0x769f12e1, SUC_lexicon_tags_17 },
    { 0xdc7f12e2, SUC_lexicon_tags_23 },
    { 0x55f12e2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf12e8, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x839f12eb, SUC_lexicon_tags_1 },
    { 0x47bf12eb, SUC_lexicon_tags_14 },
    { 0x37f12ec, SUC_lexicon_tags_27 },
    { 0x9ebf12eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe7ff12f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1f12f4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bf12fd, SUC_lexicon_tags_12 },
    { 0x3aff12fe, SUC_lexicon_tags_14 },
    { 0xfa9f12ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543f1303, SUC_lexicon_tags_37 },
    { 0x1d7f1304, SUC_lexicon_tags_2 },
    { 0x1d7f1305, SUC_lexicon_tags_2 },
    { 0xc4df1305, SUC_lexicon_tags_35 },
    { 0x503f1304, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ff130b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9f130e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe4df1310, SUC_lexicon_tags_45 },
    { 0x695f1310, SUC_lexicon_tags_6 },
    { 0xc47f1312, SUC_lexicon_tags_11 },
    { 0xf25f1313, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9df1315, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bf1318, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8a7f131a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa9f131c, SUC_lexicon_tags_49 },
    { 0xf65f131d, SUC_lexicon_tags_12 },
    { 0x3f5f131e, SUC_lexicon_tags_82 },
    { 0x96ff131f, SUC_lexicon_tags_11 },
    { 0x5b9f1320, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09f1324, SUC_lexicon_tags_2 },
    { 0x93ff1325, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233f132a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7f1333, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x953f1335, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9f1338, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c3f133c, SUC_lexicon_tags_17 },
    { 0x51df133d, SUC_lexicon_tags_33 },
    { 0xc95f133d, SUC_lexicon_tags_17 },
    { 0x323f133c, SUC_lexicon_tags_37 },
    { 0x745f1340, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bf1347, SUC_lexicon_tags_31 },
    { 0x9b7f1347, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3d3f134a, SUC_lexicon_tags_16 },
    { 0x83ff134b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x3adf134d, SUC_lexicon_tags_32 },
    { 0xa6df134e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bf1352, SUC_lexicon_tags_15 },
    { 0x613f1352, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc5df1355, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x939f1358, SUC_lexicon_tags_7 },
    { 0x79bf1359, SUC_lexicon_tags_0 },
    { 0xdf7f1359, SUC_lexicon_tags_12 },
    { 0xa53f1359, SUC_lexicon_tags_1 },
    { 0x7f135a, SUC_lexicon_tags_62 },
    { 0x3e7f135d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eff1363, SUC_lexicon_tags_15 },
    { 0x5a3f1364, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13f136d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563f1372, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x49f1374, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7adf1376, SUC_lexicon_tags_2 },
    { 0xff9f1377, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51f1382, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3f1385, SUC_lexicon_tags_6 },
    { 0x565f1386, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5f1389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x495f138b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x51bf138d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x95ff138f, SUC_lexicon_tags_15 },
    { 0x56bf1390, SUC_lexicon_tags_37 },
    { 0x777f138f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3f1397, SUC_lexicon_tags_25 },
    { 0x5ebf1398, SUC_lexicon_tags_14 },
    { 0x659f1398, SUC_lexicon_tags_15 },
    { 0x81df1398, SUC_lexicon_tags_66 },
    { 0x92ff139b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23f139e, SUC_lexicon_tags_25 },
    { 0xd83f139f, SUC_lexicon_tags_45 },
    { 0x927f139e, SUC_lexicon_tags_87 },
    { 0xa1bf13a1, SUC_lexicon_tags_23 },
    { 0xcfbf13a1, SUC_lexicon_tags_15 },
    { 0xc87f13a3, SUC_lexicon_tags_48 },
    { 0x101f13a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xecbf13a6, SUC_lexicon_tags_112 },
    { 0x431f13a7, SUC_lexicon_tags_45 },
    { 0x79df13a7, SUC_lexicon_tags_25 },
    { 0xa21f13a9, SUC_lexicon_tags_9 },
    { 0xce1f13a9, SUC_lexicon_tags_25 },
    { 0x411f13ab, SUC_lexicon_tags_31 },
    { 0xba3f13ac, SUC_lexicon_tags_15 },
    { 0xb29f13ad, SUC_lexicon_tags_2 },
    { 0x321f13ad, SUC_lexicon_tags_27 },
    { 0xc9f13ae, SUC_lexicon_tags_27 },
    { 0x80bf13ab, SUC_lexicon_tags_11 },
    { 0xcfbf13aa, SUC_lexicon_tags_9 },
    { 0x2abf13b2, SUC_lexicon_tags_1 },
    { 0x22ff13b3, SUC_lexicon_tags_14 },
    { 0xc7df13b4, SUC_lexicon_tags_4 },
    { 0xbb3f13ae, SUC_lexicon_tags_11 },
    { 0xd87f13b6, SUC_lexicon_tags_37 },
    { 0x9f1f13b7, SUC_lexicon_tags_0 },
    { 0x6cff13ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd8df13ba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc43f13bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359f13c4, SUC_lexicon_tags_1 },
    { 0x927f13c5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff13cc, SUC_lexicon_tags_25 },
    { 0xf2bf13cd, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ff13d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9f13d5, SUC_lexicon_tags_15 },
    { 0x9a9f13d6, SUC_lexicon_tags_0 },
    { 0x9c7f13d7, SUC_lexicon_tags_29 },
    { 0x2b3f13d8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ff13db, SUC_lexicon_tags_87 },
    { 0x327f13dc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bf13e3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x671f13e5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbc5f13e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x15ff13e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1df13ec, SUC_lexicon_tags_32 },
    { 0x71ff13ed, SUC_lexicon_tags_15 },
    { 0x319f13ee, SUC_lexicon_tags_37 },
    { 0x237f13ec, SUC_lexicon_tags_37 },
    { 0x5bdf13f0, SUC_lexicon_tags_1 },
    { 0x1c1f13ee, SUC_lexicon_tags_31 },
    { 0x75bf13ed, SUC_lexicon_tags_11 },
    { 0x455f13ec, SUC_lexicon_tags_17 },
    { 0x99ff13ef, SUC_lexicon_tags_15 },
    { 0xa25f13f5, SUC_lexicon_tags_15 },
    { 0xda7f13f6, SUC_lexicon_tags_49 },
    { 0x9dff13f7, SUC_lexicon_tags_57 },
    { 0x621f13f8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x11df13fb, SUC_lexicon_tags_28 },
    { 0x21ff13fc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9f1408, SUC_lexicon_tags_15 },
    { 0x45df1408, SUC_lexicon_tags_21 },
    { 0xc0bf140a, SUC_lexicon_tags_29 },
    { 0x5dbf140b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x1a1f140d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5a3f140f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f1417, SUC_lexicon_tags_30 },
    { 0xf41f1417, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc15f141b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xccdf141d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5f1420, SUC_lexicon_tags_17 },
    { 0xfdff1421, SUC_lexicon_tags_25 },
    { 0xb77f1421, SUC_lexicon_tags_12 },
    { 0xea5f1423, SUC_lexicon_tags_37 },
    { 0x477f1424, SUC_lexicon_tags_37 },
    { 0xe8bf1424, SUC_lexicon_tags_12 },
    { 0x121f1426, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f1429, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203f142e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x93ff1430, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39f1433, SUC_lexicon_tags_43 },
    { 0xbf9f1434, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x865f1436, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x309f1438, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x407f143a, SUC_lexicon_tags_25 },
    { 0xb03f143b, SUC_lexicon_tags_26 },
    { 0xc35f143c, SUC_lexicon_tags_1 },
    { 0xb29f143d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bf1441, SUC_lexicon_tags_37 },
    { 0x885f1442, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeabf1444, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76bf1449, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf144c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583f1453, SUC_lexicon_tags_79 },
    { 0x5adf1454, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddf145f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xedf1462, SUC_lexicon_tags_8 },
    { 0x74df1463, SUC_lexicon_tags_29 },
    { 0xd17f1463, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37f1468, SUC_lexicon_tags_37 },
    { 0x4e5f1469, SUC_lexicon_tags_17 },
    { 0x949f1469, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe63f146c, SUC_lexicon_tags_2 },
    { 0x9e1f146d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15f1471, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1c3f1473, SUC_lexicon_tags_38 },
    { 0xf29f1474, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x843f1479, SUC_lexicon_tags_63 },
    { 0x2bff1479, SUC_lexicon_tags_12 },
    { 0x80df147a, SUC_lexicon_tags_11 },
    { 0xad9f147b, SUC_lexicon_tags_11 },
    { 0xb89f147d, SUC_lexicon_tags_0 },
    { 0x15f147e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1482, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbf1485, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe53f1487, SUC_lexicon_tags_11 },
    { 0xfdf1488, SUC_lexicon_tags_38 },
    { 0xa21f1489, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x355f148c, SUC_lexicon_tags_81 },
    { 0x273f148c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bf1492, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ff1496, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xccff1498, SUC_lexicon_tags_17 },
    { 0x6a3f1499, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x729f149c, SUC_lexicon_tags_29 },
    { 0x8cff149d, SUC_lexicon_tags_87 },
    { 0xe83f149e, SUC_lexicon_tags_33 },
    { 0x1a3f149d, SUC_lexicon_tags_32 },
    { 0xbdff149c, SUC_lexicon_tags_15 },
    { 0xe0df14a1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xad1f14a3, SUC_lexicon_tags_8 },
    { 0x81df14a3, SUC_lexicon_tags_1 },
    { 0x1d1f14a5, SUC_lexicon_tags_84 },
    { 0x293f14a6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x829f14a9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07f14af, SUC_lexicon_tags_2 },
    { 0x93ff14b0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37f14b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb55f14b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x195f14b6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x81ff14b8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xccdf14ba, SUC_lexicon_tags_12 },
    { 0x6eff14bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcaff14bd, SUC_lexicon_tags_25 },
    { 0xce1f14be, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x195f14c0, SUC_lexicon_tags_35 },
    { 0x871f14c1, SUC_lexicon_tags_118 },
    { 0xb67f14c1, SUC_lexicon_tags_15 },
    { 0x61f14c2, SUC_lexicon_tags_27 },
    { 0xf87f14c4, SUC_lexicon_tags_27 },
    { 0xacbf14c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1fdf14c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x325f14c9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465f14d5, SUC_lexicon_tags_26 },
    { 0xba1f14d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd11f14d8, SUC_lexicon_tags_25 },
    { 0x75f14d9, SUC_lexicon_tags_0 },
    { 0x93ff14d9, SUC_lexicon_tags_33 },
    { 0x169f14db, SUC_lexicon_tags_11 },
    { 0x2e7f14dc, SUC_lexicon_tags_11 },
    { 0x645f14d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x981f14e0, SUC_lexicon_tags_49 },
    { 0x7c3f14e1, SUC_lexicon_tags_26 },
    { 0x6bf14e1, SUC_lexicon_tags_25 },
    { 0x55ff14e1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6edf14f1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddf14f4, SUC_lexicon_tags_30 },
    { 0x927f14f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf4ff14f7, SUC_lexicon_tags_55 },
    { 0x337f14f8, SUC_lexicon_tags_61 },
    { 0xd4ff14f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x929f14fb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39f14fe, SUC_lexicon_tags_65 },
    { 0xcb9f14fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91f1507, SUC_lexicon_tags_7 },
    { 0x7c5f1508, SUC_lexicon_tags_27 },
    { 0xef1f1507, SUC_lexicon_tags_31 },
    { 0x1bff1509, SUC_lexicon_tags_41 },
    { 0x1a9f1508, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x275f150e, SUC_lexicon_tags_10 },
    { 0x1e9f150f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbff1512, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x829f1515, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adf1518, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x971f151a, SUC_lexicon_tags_49 },
    { 0xeb5f151a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xab7f151d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa35f151f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81f1524, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf9f1526, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdbf1528, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07f1534, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bf1537, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x601f153a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5f153e, SUC_lexicon_tags_11 },
    { 0xdcbf153f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x52ff1541, SUC_lexicon_tags_25 },
    { 0x405f1542, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9abf1544, SUC_lexicon_tags_25 },
    { 0x6bff1544, SUC_lexicon_tags_47 },
    { 0xa4ff1546, SUC_lexicon_tags_12 },
    { 0x179f1547, SUC_lexicon_tags_79 },
    { 0x48ff1548, SUC_lexicon_tags_37 },
    { 0x5ff1548, SUC_lexicon_tags_17 },
    { 0xcb1f1546, SUC_lexicon_tags_16 },
    { 0xd51f1547, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603f154f, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x653f1551, SUC_lexicon_tags_25 },
    { 0xfe7f1552, SUC_lexicon_tags_11 },
    { 0x361f1552, SUC_lexicon_tags_7 },
    { 0xcf7f1554, SUC_lexicon_tags_37 },
    { 0x90ff1552, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x411f1557, SUC_lexicon_tags_25 },
    { 0xfebf1557, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f155b, SUC_lexicon_tags_45 },
    { 0x523f155c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf1561, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5f156c, SUC_lexicon_tags_47 },
    { 0x4a9f156d, SUC_lexicon_tags_45 },
    { 0xffbf156c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7f1571, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75f1575, SUC_lexicon_tags_77 },
    { 0x611f1576, SUC_lexicon_tags_37 },
    { 0xee1f1576, SUC_lexicon_tags_25 },
    { 0x629f1578, SUC_lexicon_tags_21 },
    { 0x685f1576, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x351f157b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3f157f, SUC_lexicon_tags_1 },
    { 0x99df1580, SUC_lexicon_tags_69 },
    { 0x273f1580, SUC_lexicon_tags_2 },
    { 0xdf9f1580, SUC_lexicon_tags_18 },
    { 0x825f1580, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x533f1586, SUC_lexicon_tags_23 },
    { 0xef3f1587, SUC_lexicon_tags_29 },
    { 0x8e1f1588, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543f1591, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9f1594, SUC_lexicon_tags_16 },
    { 0x193f1594, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ff1598, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3f159c, SUC_lexicon_tags_63 },
    { 0x929f159c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdf15a0, SUC_lexicon_tags_14 },
    { 0x6e7f15a0, SUC_lexicon_tags_43 },
    { 0xf0df15a2, SUC_lexicon_tags_15 },
    { 0xabff15a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf49f15ac, SUC_lexicon_tags_7 },
    { 0x741f15ad, SUC_lexicon_tags_17 },
    { 0x18bf15ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xddf15b0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5df15ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbdf15bc, SUC_lexicon_tags_8 },
    { 0x855f15bc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43df15c2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ff15c6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1f15cb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbf15d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3f15d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf15d8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x181f15da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbf15de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f15e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdff15e5, SUC_lexicon_tags_40 },
    { 0x527f15e5, SUC_lexicon_tags_47 },
    { 0x9ff15e7, SUC_lexicon_tags_46 },
    { 0xad1f15e8, SUC_lexicon_tags_39 },
    { 0xe5df15e9, SUC_lexicon_tags_1 },
    { 0x6f7f15e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443f15f2, SUC_lexicon_tags_17 },
    { 0x781f15f2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x3bff15f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfc5f15f7, SUC_lexicon_tags_56 },
    { 0x4eff15f8, SUC_lexicon_tags_11 },
    { 0xc35f15f8, SUC_lexicon_tags_47 },
    { 0x28df15fa, SUC_lexicon_tags_45 },
    { 0xac5f15f7, SUC_lexicon_tags_2 },
    { 0xe0df15fa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9f15ff, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495f1603, SUC_lexicon_tags_49 },
    { 0x3fdf1604, SUC_lexicon_tags_118 },
    { 0xbcdf1603, SUC_lexicon_tags_11 },
    { 0x6edf1603, SUC_lexicon_tags_63 },
    { 0x729f1607, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdf160b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbf160e, SUC_lexicon_tags_60 },
    { 0x3b7f160f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f1616, SUC_lexicon_tags_12 },
    { 0xd21f1616, SUC_lexicon_tags_15 },
    { 0x5f5f1618, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xddbf161a, SUC_lexicon_tags_19 },
    { 0x413f161a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bf161e, SUC_lexicon_tags_66 },
    { 0xd2bf161f, SUC_lexicon_tags_49 },
    { 0xfb1f1620, SUC_lexicon_tags_25 },
    { 0xca7f161f, SUC_lexicon_tags_196 },
    { 0xc5df161e, SUC_lexicon_tags_25 },
    { 0x9f3f1621, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4bf1625, SUC_lexicon_tags_14 },
    { 0x5a9f1626, SUC_lexicon_tags_19 },
    { 0x807f1626, SUC_lexicon_tags_34 },
    { 0x1a9f1628, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571f1636, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf3bf1638, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x523f163a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xef7f163c, SUC_lexicon_tags_14 },
    { 0xf41f163c, SUC_lexicon_tags_43 },
    { 0x913f163d, SUC_lexicon_tags_33 },
    { 0xe29f163e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xdabf1641, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619f1647, SUC_lexicon_tags_87 },
    { 0xfe7f1647, SUC_lexicon_tags_8 },
    { 0xccdf1649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7f164d, SUC_lexicon_tags_79 },
    { 0x37f164e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09f1659, SUC_lexicon_tags_37 },
    { 0x639f165a, SUC_lexicon_tags_11 },
    { 0x60ff165a, SUC_lexicon_tags_4 },
    { 0xb91f165c, SUC_lexicon_tags_1 },
    { 0x7c3f165d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ff1660, SUC_lexicon_tags_33 },
    { 0x9f1f1661, SUC_lexicon_tags_38 },
    { 0xc91f1660, SUC_lexicon_tags_46 },
    { 0xb23f1660, SUC_lexicon_tags_9 },
    { 0xfe7f1664, SUC_lexicon_tags_10 },
    { 0xb4df1665, SUC_lexicon_tags_2 },
    { 0x25ff1666, SUC_lexicon_tags_57 },
    { 0x8ebf1666, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ff166a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bff166d, SUC_lexicon_tags_37 },
    { 0x26bf166e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x91bf1670, SUC_lexicon_tags_33 },
    { 0xb6df1670, SUC_lexicon_tags_11 },
    { 0x481f1672, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x95bf1674, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaddf1679, SUC_lexicon_tags_37 },
    { 0x153f167a, SUC_lexicon_tags_32 },
    { 0x565f167a, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997f167f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837f1684, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdf168b, SUC_lexicon_tags_12 },
    { 0x4bf168c, SUC_lexicon_tags_129 },
    { 0xf7bf168c, SUC_lexicon_tags_17 },
    { 0xb77f168e, SUC_lexicon_tags_7 },
    { 0x477f168f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25f1693, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x805f1695, SUC_lexicon_tags_47 },
    { 0x297f1696, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71f169c, SUC_lexicon_tags_25 },
    { 0xdebf169c, SUC_lexicon_tags_1 },
    { 0xa2bf169d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x7dbf16a0, SUC_lexicon_tags_224 },
    { 0x20bf16a0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3df16a4, SUC_lexicon_tags_27 },
    { 0x16df16a5, SUC_lexicon_tags_1 },
    { 0xebdf16a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5f16a9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa07f16ab, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945f16b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3f16b5, SUC_lexicon_tags_1 },
    { 0x8c1f16b6, SUC_lexicon_tags_1 },
    { 0xab5f16b7, SUC_lexicon_tags_23 },
    { 0x2f5f16b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7e9f16ba, SUC_lexicon_tags_1 },
    { 0xf4df16ba, SUC_lexicon_tags_25 },
    { 0x2fff16bc, SUC_lexicon_tags_38 },
    { 0x909f16ba, SUC_lexicon_tags_106 },
    { 0x287f16bd, SUC_lexicon_tags_11 },
    { 0x5fbf16be, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51f16c7, SUC_lexicon_tags_0 },
    { 0xe81f16c8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f3f16ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7f16ce, SUC_lexicon_tags_28 },
    { 0xd7df16cf, SUC_lexicon_tags_25 },
    { 0x725f16d0, SUC_lexicon_tags_1 },
    { 0xc49f16cf, SUC_lexicon_tags_0 },
    { 0xe3df16cf, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x579f16d4, SUC_lexicon_tags_17 },
    { 0xb11f16d5, SUC_lexicon_tags_25 },
    { 0xe53f16d6, SUC_lexicon_tags_37 },
    { 0x503f16d7, SUC_lexicon_tags_101 },
    { 0x975f16d8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbf16dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18df16e2, SUC_lexicon_tags_27 },
    { 0xde5f16e3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfff16e6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfdf16e8, SUC_lexicon_tags_1331 },
    { 0, NULL },
    { 0x563f16ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdf16ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bf16f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bf16f8, SUC_lexicon_tags_15 },
    { 0xcd5f16f8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xec1f16fb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10df16ff, SUC_lexicon_tags_37 },
    { 0xbe7f1700, SUC_lexicon_tags_47 },
    { 0x76bf1701, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05f1704, SUC_lexicon_tags_47 },
    { 0x2abf1705, SUC_lexicon_tags_27 },
    { 0x5e7f1706, SUC_lexicon_tags_0 },
    { 0xd73f1707, SUC_lexicon_tags_21 },
    { 0xe5df1708, SUC_lexicon_tags_37 },
    { 0xd1df1709, SUC_lexicon_tags_1 },
    { 0xe3f1704, SUC_lexicon_tags_12 },
    { 0x3e9f170a, SUC_lexicon_tags_37 },
    { 0xd85f170c, SUC_lexicon_tags_1 },
    { 0xc63f170d, SUC_lexicon_tags_37 },
    { 0x5c7f170d, SUC_lexicon_tags_45 },
    { 0x7cdf170d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671f1716, SUC_lexicon_tags_17 },
    { 0x445f1717, SUC_lexicon_tags_25 },
    { 0xc63f1717, SUC_lexicon_tags_7 },
    { 0xbfff1716, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbf171f, SUC_lexicon_tags_28 },
    { 0x243f1720, SUC_lexicon_tags_25 },
    { 0x89df1721, SUC_lexicon_tags_0 },
    { 0x2a5f171f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631f1727, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51f172b, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29f1732, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x10bf1734, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xd77f1736, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f173c, SUC_lexicon_tags_25 },
    { 0xd51f173c, SUC_lexicon_tags_47 },
    { 0x8e3f173c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xba5f1741, SUC_lexicon_tags_49 },
    { 0x48ff1742, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xde5f1744, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d5f1746, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3f174a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7f1755, SUC_lexicon_tags_66 },
    { 0xc5bf1755, SUC_lexicon_tags_12 },
    { 0xe65f1756, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7bff1759, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x543f175c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbf1765, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x795f176a, SUC_lexicon_tags_14 },
    { 0xef9f176a, SUC_lexicon_tags_33 },
    { 0xe89f176c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bf1771, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedf177a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x2aff177c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x477f177f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ff1781, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bf1788, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdf178b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3f1794, SUC_lexicon_tags_40 },
    { 0x311f1795, SUC_lexicon_tags_11 },
    { 0x8b9f1795, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137f179a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xa33f179c, SUC_lexicon_tags_33 },
    { 0x859f179d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x93df17a0, SUC_lexicon_tags_2 },
    { 0xaa7f17a1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf37f17a3, SUC_lexicon_tags_21 },
    { 0x113f17a4, SUC_lexicon_tags_82 },
    { 0xb61f17a5, SUC_lexicon_tags_264 },
    { 0xd03f17a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfb7f17a8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11f17b1, SUC_lexicon_tags_25 },
    { 0x3bf17b1, SUC_lexicon_tags_37 },
    { 0x1a7f17b3, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bf17b9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x971f17bf, SUC_lexicon_tags_12 },
    { 0x13df17c0, SUC_lexicon_tags_63 },
    { 0xb4df17c1, SUC_lexicon_tags_17 },
    { 0x67f17c1, SUC_lexicon_tags_37 },
    { 0xcbdf17c3, SUC_lexicon_tags_15 },
    { 0x75bf17bf, SUC_lexicon_tags_37 },
    { 0xd1df17c4, SUC_lexicon_tags_44 },
    { 0x4a3f17c6, SUC_lexicon_tags_2 },
    { 0x609f17c6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49f17ce, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x78df17d0, SUC_lexicon_tags_33 },
    { 0xa27f17d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9adf17d4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7f17d8, SUC_lexicon_tags_8 },
    { 0x493f17d9, SUC_lexicon_tags_15 },
    { 0x8bdf17d9, SUC_lexicon_tags_5 },
    { 0xf35f17da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdf17de, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2a1f17e0, SUC_lexicon_tags_25 },
    { 0x305f17e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x411f17e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ff17e6, SUC_lexicon_tags_77 },
    { 0x3fbf17e7, SUC_lexicon_tags_7 },
    { 0x5e5f17e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3f17f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bf17f6, SUC_lexicon_tags_1 },
    { 0xd8bf17f6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x29bf17f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9f17fb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcb9f17fd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff1801, SUC_lexicon_tags_25 },
    { 0x41f1801, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x91df1804, SUC_lexicon_tags_49 },
    { 0x8b7f1805, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x439f1807, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9f180c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x47df180f, SUC_lexicon_tags_2 },
    { 0x833f1810, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe7df1812, SUC_lexicon_tags_21 },
    { 0xcebf1812, SUC_lexicon_tags_17 },
    { 0x97ff1813, SUC_lexicon_tags_51 },
    { 0x4f5f1813, SUC_lexicon_tags_13 },
    { 0x38df1813, SUC_lexicon_tags_13 },
    { 0x9e5f1814, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x503f1819, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2d7f181b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1f1821, SUC_lexicon_tags_45 },
    { 0x9e3f1822, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7f1827, SUC_lexicon_tags_12 },
    { 0x53bf1828, SUC_lexicon_tags_15 },
    { 0x713f1829, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdf182c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5f1830, SUC_lexicon_tags_15 },
    { 0xfe7f1831, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ff1836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbc7f1838, SUC_lexicon_tags_60 },
    { 0x705f1838, SUC_lexicon_tags_46 },
    { 0xccdf183a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x471f183c, SUC_lexicon_tags_38 },
    { 0xa71f183c, SUC_lexicon_tags_28 },
    { 0xf57f183e, SUC_lexicon_tags_1 },
    { 0xbabf183e, SUC_lexicon_tags_2 },
    { 0xaeff1840, SUC_lexicon_tags_37 },
    { 0xd23f1840, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749f1847, SUC_lexicon_tags_17 },
    { 0xabff1847, SUC_lexicon_tags_1 },
    { 0x917f1849, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ff184d, SUC_lexicon_tags_14 },
    { 0x867f184e, SUC_lexicon_tags_12 },
    { 0xfbdf184e, SUC_lexicon_tags_7 },
    { 0x8c3f184f, SUC_lexicon_tags_37 },
    { 0x94df1851, SUC_lexicon_tags_23 },
    { 0x3f9f1850, SUC_lexicon_tags_39 },
    { 0x33df1853, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7f1856, SUC_lexicon_tags_7 },
    { 0xa1df1857, SUC_lexicon_tags_77 },
    { 0x717f1857, SUC_lexicon_tags_37 },
    { 0xaa1f1858, SUC_lexicon_tags_57 },
    { 0x271f1858, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x451f185c, SUC_lexicon_tags_7 },
    { 0x219f185d, SUC_lexicon_tags_16 },
    { 0xdb5f185e, SUC_lexicon_tags_37 },
    { 0xba5f185f, SUC_lexicon_tags_7 },
    { 0xb09f1860, SUC_lexicon_tags_65 },
    { 0x6a1f1861, SUC_lexicon_tags_1 },
    { 0xaedf1861, SUC_lexicon_tags_17 },
    { 0x567f1863, SUC_lexicon_tags_7 },
    { 0xd89f1863, SUC_lexicon_tags_37 },
    { 0x7fff1865, SUC_lexicon_tags_15 },
    { 0x27ff1865, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x1e9f1868, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x117f186a, SUC_lexicon_tags_32 },
    { 0xf1ff186a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535f1870, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x701f1873, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f187b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167f1883, SUC_lexicon_tags_63 },
    { 0xb1df1883, SUC_lexicon_tags_37 },
    { 0xbddf1883, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697f188a, SUC_lexicon_tags_18 },
    { 0xbf1f188b, SUC_lexicon_tags_27 },
    { 0xbe1f188c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981f1892, SUC_lexicon_tags_77 },
    { 0xc07f1893, SUC_lexicon_tags_19 },
    { 0x8ff1894, SUC_lexicon_tags_12 },
    { 0xaadf1893, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf1ff1897, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9cff1899, SUC_lexicon_tags_1 },
    { 0xf95f189a, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aff189f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x81df18a1, SUC_lexicon_tags_4 },
    { 0x231f18a2, SUC_lexicon_tags_11 },
    { 0xfe1f18a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ff18a8, SUC_lexicon_tags_39 },
    { 0x971f18a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239f18ae, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0x10bf18b0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbdff18b2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ff18b9, SUC_lexicon_tags_1 },
    { 0x6cdf18b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x297f18bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x605f18be, SUC_lexicon_tags_19 },
    { 0x487f18be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7f18c3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3f18c7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5f18ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf18cd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49f18d1, SUC_lexicon_tags_25 },
    { 0x6f5f18d1, SUC_lexicon_tags_29 },
    { 0x25df18d3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdf18d6, SUC_lexicon_tags_19 },
    { 0xf75f18d7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xab1f18da, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x47bf18dc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd49f18de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907f18e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bf18e8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbaff18ea, SUC_lexicon_tags_7 },
    { 0x6c3f18eb, SUC_lexicon_tags_23 },
    { 0x1e5f18ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215f18f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41f18f5, SUC_lexicon_tags_26 },
    { 0x501f18f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677f18fa, SUC_lexicon_tags_37 },
    { 0x30df18fa, SUC_lexicon_tags_65 },
    { 0x49ff18fc, SUC_lexicon_tags_15 },
    { 0xafdf18fa, SUC_lexicon_tags_1 },
    { 0xb0bf18fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff1901, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x34bf1903, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2ddf1905, SUC_lexicon_tags_27 },
    { 0x9e7f1906, SUC_lexicon_tags_47 },
    { 0x313f1905, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x829f190a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x823f190d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7e3f190f, SUC_lexicon_tags_29 },
    { 0x6bbf1910, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4e9f1912, SUC_lexicon_tags_15 },
    { 0x3fff1913, SUC_lexicon_tags_43 },
    { 0xc89f1914, SUC_lexicon_tags_46 },
    { 0x4b5f1915, SUC_lexicon_tags_15 },
    { 0x7b7f1913, SUC_lexicon_tags_1 },
    { 0xb29f1912, SUC_lexicon_tags_19 },
    { 0x547f1912, SUC_lexicon_tags_42 },
    { 0xab5f1919, SUC_lexicon_tags_49 },
    { 0xa0df1914, SUC_lexicon_tags_12 },
    { 0x3c3f1919, SUC_lexicon_tags_7 },
    { 0xce3f191c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xdaff191e, SUC_lexicon_tags_15 },
    { 0xe4ff191f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x29ff1921, SUC_lexicon_tags_37 },
    { 0x9e5f1922, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153f1929, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x7df192b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdfbf192d, SUC_lexicon_tags_160 },
    { 0x741f192e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5b3f1930, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbf1934, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x481f1936, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6ff1938, SUC_lexicon_tags_65 },
    { 0x48ff1939, SUC_lexicon_tags_39 },
    { 0xf93f1938, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9f193e, SUC_lexicon_tags_17 },
    { 0x859f193f, SUC_lexicon_tags_37 },
    { 0x87bf1940, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x497f1942, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37f1948, SUC_lexicon_tags_12 },
    { 0x321f1949, SUC_lexicon_tags_7 },
    { 0x1f9f194a, SUC_lexicon_tags_27 },
    { 0xd5f194a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f194e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9a9f1950, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1edf1952, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdf1956, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x22bf1959, SUC_lexicon_tags_17 },
    { 0xb13f1959, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf09f195c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbf1960, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825f1965, SUC_lexicon_tags_32 },
    { 0xdedf1966, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5d7f1968, SUC_lexicon_tags_37 },
    { 0x761f1969, SUC_lexicon_tags_2 },
    { 0xe85f196a, SUC_lexicon_tags_35 },
    { 0xe43f196b, SUC_lexicon_tags_30 },
    { 0xe5df1969, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xf19f196e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1a9f1970, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179f1974, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbb5f1976, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x213f1978, SUC_lexicon_tags_60 },
    { 0x4d7f1979, SUC_lexicon_tags_25 },
    { 0x581f197a, SUC_lexicon_tags_46 },
    { 0xa99f197a, SUC_lexicon_tags_23 },
    { 0xfc3f1979, SUC_lexicon_tags_33 },
    { 0x9ddf197a, SUC_lexicon_tags_17 },
    { 0x1c1f197b, SUC_lexicon_tags_7 },
    { 0x4e5f197f, SUC_lexicon_tags_60 },
    { 0xa8bf1980, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc49f1984, SUC_lexicon_tags_2 },
    { 0xe93f1985, SUC_lexicon_tags_60 },
    { 0xd11f1986, SUC_lexicon_tags_26 },
    { 0x9e1f1987, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5f198a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x203f198c, SUC_lexicon_tags_37 },
    { 0xa0ff198d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x385f198f, SUC_lexicon_tags_12 },
    { 0x175f1990, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01f1994, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835f1999, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54df199e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd41f19a1, SUC_lexicon_tags_1 },
    { 0xa39f19a1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4df19a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dff19ad, SUC_lexicon_tags_2 },
    { 0xe6df19ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf99f19b0, SUC_lexicon_tags_17 },
    { 0x72df19b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x69bf19b3, SUC_lexicon_tags_16 },
    { 0x74bf19b4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7f19b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1f19bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dff19c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x465f19c8, SUC_lexicon_tags_57 },
    { 0xcabf19c8, SUC_lexicon_tags_11 },
    { 0xff1f19c9, SUC_lexicon_tags_2 },
    { 0x773f19cb, SUC_lexicon_tags_9 },
    { 0x1ff19cc, SUC_lexicon_tags_37 },
    { 0x1b7f19ca, SUC_lexicon_tags_44 },
    { 0x78df19ca, SUC_lexicon_tags_2 },
    { 0x5e9f19cf, SUC_lexicon_tags_8 },
    { 0x827f19d0, SUC_lexicon_tags_10 },
    { 0x89ff19c8, SUC_lexicon_tags_38 },
    { 0x9dbf19ca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x919f19d5, SUC_lexicon_tags_30 },
    { 0xde5f19d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27f19d9, SUC_lexicon_tags_25 },
    { 0x67f19da, SUC_lexicon_tags_31 },
    { 0x6c1f19db, SUC_lexicon_tags_7 },
    { 0x275f19d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253f19e2, SUC_lexicon_tags_4 },
    { 0x67f19e3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x537f19e5, SUC_lexicon_tags_10 },
    { 0xdddf19e5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259f19ea, SUC_lexicon_tags_201 },
    { 0x8b7f19eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x95ff19ed, SUC_lexicon_tags_49 },
    { 0xc69f19ee, SUC_lexicon_tags_8 },
    { 0x287f19ee, SUC_lexicon_tags_25 },
    { 0x145f19f0, SUC_lexicon_tags_33 },
    { 0x51f19f1, SUC_lexicon_tags_31 },
    { 0xdfbf19f1, SUC_lexicon_tags_2 },
    { 0x3a7f19f2, SUC_lexicon_tags_37 },
    { 0x461f19f0, SUC_lexicon_tags_33 },
    { 0xddbf19f5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa65f19f7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9f19ff, SUC_lexicon_tags_15 },
    { 0x4fbf1a00, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707f1a06, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x69bf1a09, SUC_lexicon_tags_29 },
    { 0xc6df1a0a, SUC_lexicon_tags_17 },
    { 0x8e9f1a0b, SUC_lexicon_tags_17 },
    { 0x8f7f1a0b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1f1a10, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xb21f1a12, SUC_lexicon_tags_45 },
    { 0xf57f1a12, SUC_lexicon_tags_29 },
    { 0x4fff1a14, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcedf1a1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x89f1a1d, SUC_lexicon_tags_8 },
    { 0x25ff1a1e, SUC_lexicon_tags_11 },
    { 0x46bf1a1f, SUC_lexicon_tags_2 },
    { 0x27f1a1f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81f1a24, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7f1a2b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc1f1a2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5a3f1a2f, SUC_lexicon_tags_951 },
    { 0xfcff1a30, SUC_lexicon_tags_26 },
    { 0x9c3f1a2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69f1a37, SUC_lexicon_tags_37 },
    { 0x279f1a38, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x7df1a3a, SUC_lexicon_tags_28 },
    { 0x8fbf1a3b, SUC_lexicon_tags_0 },
    { 0x8f1f1a3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ff1a3f, SUC_lexicon_tags_11 },
    { 0xcedf1a40, SUC_lexicon_tags_47 },
    { 0x2a5f1a3f, SUC_lexicon_tags_45 },
    { 0x5a1f1a42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7f1a45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357f1a4a, SUC_lexicon_tags_47 },
    { 0xa1ff1a4a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeff1a4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x467f1a51, SUC_lexicon_tags_0 },
    { 0x215f1a52, SUC_lexicon_tags_55 },
    { 0x2df1a52, SUC_lexicon_tags_24 },
    { 0x137f1a54, SUC_lexicon_tags_19 },
    { 0x84ff1a55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7f1a57, SUC_lexicon_tags_39 },
    { 0x19df1a58, SUC_lexicon_tags_65 },
    { 0x2c1f1a59, SUC_lexicon_tags_11 },
    { 0x133f1a5a, SUC_lexicon_tags_19 },
    { 0xda1f1a5b, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0xcaff1a5d, SUC_lexicon_tags_25 },
    { 0x595f1a5d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x291f1a61, SUC_lexicon_tags_8 },
    { 0x9dbf1a61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ff1a66, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7f1a6a, SUC_lexicon_tags_11 },
    { 0x71df1a6a, SUC_lexicon_tags_21 },
    { 0xf47f1a6a, SUC_lexicon_tags_7 },
    { 0xd7f1a6b, SUC_lexicon_tags_40 },
    { 0xfb7f1a6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9f1a77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345f1a7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xbf3f1a7f, SUC_lexicon_tags_25 },
    { 0x173f1a80, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x763f1a82, SUC_lexicon_tags_26 },
    { 0x409f1a83, SUC_lexicon_tags_29 },
    { 0xc11f1a84, SUC_lexicon_tags_37 },
    { 0x7adf1a85, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9f1a8e, SUC_lexicon_tags_32 },
    { 0x569f1a8e, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xd43f1a91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x241f1a93, SUC_lexicon_tags_66 },
    { 0x4c9f1a94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bf1a9a, SUC_lexicon_tags_14 },
    { 0xc2df1a9b, SUC_lexicon_tags_30 },
    { 0xb13f1a9a, SUC_lexicon_tags_33 },
    { 0xfcdf1a9b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x667f1aa0, SUC_lexicon_tags_11 },
    { 0x7d1f1aa1, SUC_lexicon_tags_37 },
    { 0xdbbf1aa2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93f1aac, SUC_lexicon_tags_23 },
    { 0x5eff1aad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddbf1ab1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7bbf1ab3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa3f1ab5, SUC_lexicon_tags_17 },
    { 0x1bdf1ab6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x793f1ab9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x85ff1abc, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267f1ac1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bf1ac4, SUC_lexicon_tags_39 },
    { 0x74bf1ac4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbf1ad0, SUC_lexicon_tags_34 },
    { 0xaf1f1ad1, SUC_lexicon_tags_138 },
    { 0xb1f1ad2, SUC_lexicon_tags_29 },
    { 0xa6ff1ad3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79f1ad6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd6bf1ad8, SUC_lexicon_tags_1 },
    { 0x33df1ad8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27f1ade, SUC_lexicon_tags_61 },
    { 0x20df1adf, SUC_lexicon_tags_11 },
    { 0xf67f1ae0, SUC_lexicon_tags_63 },
    { 0xd89f1adf, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703f1ae7, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbf1aed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21f1af0, SUC_lexicon_tags_47 },
    { 0x2bf1af1, SUC_lexicon_tags_483 },
    { 0x647f1af2, SUC_lexicon_tags_40 },
    { 0x90bf1af3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35df1af7, SUC_lexicon_tags_43 },
    { 0xb5bf1af8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x511f1afb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbf1afe, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfd7f1b00, SUC_lexicon_tags_11 },
    { 0xc21f1b01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9f1b03, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x259f1b06, SUC_lexicon_tags_63 },
    { 0x379f1b06, SUC_lexicon_tags_0 },
    { 0x597f1b06, SUC_lexicon_tags_121 },
    { 0x2a7f1b09, SUC_lexicon_tags_60 },
    { 0x52bf1b0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81df1b0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x69df1b11, SUC_lexicon_tags_38 },
    { 0xf57f1b12, SUC_lexicon_tags_8 },
    { 0x557f1b11, SUC_lexicon_tags_2 },
    { 0x617f1b13, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5f1b1c, SUC_lexicon_tags_25 },
    { 0xc39f1b1d, SUC_lexicon_tags_43 },
    { 0x737f1b1e, SUC_lexicon_tags_17 },
    { 0x7a3f1b1e, SUC_lexicon_tags_27 },
    { 0x3cbf1b20, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3bf1b22, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5f1b25, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3a5f1b27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb7f1b29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467f1b2f, SUC_lexicon_tags_37 },
    { 0x4c7f1b30, SUC_lexicon_tags_17 },
    { 0xfbff1b2f, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0xbaff1b34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x557f1b36, SUC_lexicon_tags_15 },
    { 0x21ff1b36, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd01f1b39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3f1b3c, SUC_lexicon_tags_37 },
    { 0xe3df1b3d, SUC_lexicon_tags_31 },
    { 0x613f1b3e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9f1b45, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1f1b48, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87f1b51, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x521f1b54, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x60df1b57, SUC_lexicon_tags_27 },
    { 0x2e9f1b58, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5f1b5b, SUC_lexicon_tags_15 },
    { 0x869f1b5c, SUC_lexicon_tags_7 },
    { 0x401f1b5d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x341f1b60, SUC_lexicon_tags_47 },
    { 0x381f1b61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ff1b64, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x909f1b67, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9c3f1b69, SUC_lexicon_tags_66 },
    { 0xd9ff1b6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bf1b6d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53f1b79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x715f1b7b, SUC_lexicon_tags_11 },
    { 0x347f1b7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437f1b89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x50bf1b8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11df1b90, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbf1b93, SUC_lexicon_tags_37 },
    { 0x441f1b94, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1a7f1b96, SUC_lexicon_tags_6 },
    { 0xb37f1b97, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7f1b9b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x4b7f1b9d, SUC_lexicon_tags_1 },
    { 0x50df1b9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbd5f1ba0, SUC_lexicon_tags_23 },
    { 0xf63f1ba0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9eff1ba3, SUC_lexicon_tags_54 },
    { 0x86df1ba3, SUC_lexicon_tags_37 },
    { 0x1cbf1ba5, SUC_lexicon_tags_12 },
    { 0xf11f1ba5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f1bb2, SUC_lexicon_tags_17 },
    { 0x371f1bb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x615f1bb5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x899f1bb7, SUC_lexicon_tags_27 },
    { 0x281f1bb8, SUC_lexicon_tags_6 },
    { 0x6ddf1bb9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9f1bbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe95f1bbe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edf1bc2, SUC_lexicon_tags_37 },
    { 0xd71f1bc3, SUC_lexicon_tags_14 },
    { 0xb1f1bc3, SUC_lexicon_tags_15 },
    { 0x18df1bc4, SUC_lexicon_tags_16 },
    { 0x483f1bc2, SUC_lexicon_tags_17 },
    { 0x13bf1bc7, SUC_lexicon_tags_37 },
    { 0x7e9f1bc8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbf1bd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f1bd7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f1bde, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8f7f1be0, SUC_lexicon_tags_87 },
    { 0xf8ff1be0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5df1be6, SUC_lexicon_tags_14 },
    { 0xb1bf1be7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bf1bea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f1bf0, SUC_lexicon_tags_17 },
    { 0xc41f1bf1, SUC_lexicon_tags_25 },
    { 0x59bf1bf2, SUC_lexicon_tags_18 },
    { 0xf07f1bf1, SUC_lexicon_tags_1 },
    { 0xb3f1bf4, SUC_lexicon_tags_16 },
    { 0xc27f1bf3, SUC_lexicon_tags_7 },
    { 0x933f1bf0, SUC_lexicon_tags_25 },
    { 0x693f1bf7, SUC_lexicon_tags_23 },
    { 0x37f1bf8, SUC_lexicon_tags_5 },
    { 0x5f7f1bf7, SUC_lexicon_tags_25 },
    { 0xb73f1bfa, SUC_lexicon_tags_2 },
    { 0x313f1bf9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9abf1c00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9f1c06, SUC_lexicon_tags_14 },
    { 0xe8ff1c07, SUC_lexicon_tags_29 },
    { 0x83f1c08, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf75f1c0a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73f1c0e, SUC_lexicon_tags_0 },
    { 0x573f1c0f, SUC_lexicon_tags_2 },
    { 0x5aff1c0f, SUC_lexicon_tags_17 },
    { 0x585f1c11, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x919f1c13, SUC_lexicon_tags_45 },
    { 0x62bf1c14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6df1c17, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f7f1c19, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xaa3f1c1b, SUC_lexicon_tags_15 },
    { 0x8e5f1c1b, SUC_lexicon_tags_21 },
    { 0xad9f1c1c, SUC_lexicon_tags_61 },
    { 0x17f1c1d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x61df1c21, SUC_lexicon_tags_47 },
    { 0x1c7f1c22, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7f1c26, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9f1c29, SUC_lexicon_tags_39 },
    { 0x431f1c2a, SUC_lexicon_tags_25 },
    { 0x45f1c2a, SUC_lexicon_tags_37 },
    { 0x8f5f1c2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x193f1c2e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbf1c31, SUC_lexicon_tags_23 },
    { 0xf37f1c32, SUC_lexicon_tags_25 },
    { 0xd0ff1c33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x939f1c36, SUC_lexicon_tags_11 },
    { 0xd25f1c37, SUC_lexicon_tags_32 },
    { 0x309f1c36, SUC_lexicon_tags_17 },
    { 0x65bf1c38, SUC_lexicon_tags_47 },
    { 0xd21f1c3a, SUC_lexicon_tags_7 },
    { 0xfa3f1c3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657f1c40, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x971f1c42, SUC_lexicon_tags_11 },
    { 0x897f1c43, SUC_lexicon_tags_31 },
    { 0x643f1c43, SUC_lexicon_tags_57 },
    { 0x82bf1c42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9f1c4d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x8cdf1c4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5f1c52, SUC_lexicon_tags_27 },
    { 0x619f1c53, SUC_lexicon_tags_0 },
    { 0x249f1c54, SUC_lexicon_tags_9 },
    { 0x91f1c52, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9df1c58, SUC_lexicon_tags_33 },
    { 0x273f1c59, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5f1c5e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x2cdf1c60, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3f1c66, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bf1c6c, SUC_lexicon_tags_37 },
    { 0x759f1c6d, SUC_lexicon_tags_25 },
    { 0xa35f1c6e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf1c71, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe01f1c74, SUC_lexicon_tags_12 },
    { 0xfcff1c75, SUC_lexicon_tags_37 },
    { 0x37df1c75, SUC_lexicon_tags_13 },
    { 0x8cbf1c76, SUC_lexicon_tags_11 },
    { 0x759f1c78, SUC_lexicon_tags_1 },
    { 0x91f1c78, SUC_lexicon_tags_3 },
    { 0x255f1c7a, SUC_lexicon_tags_82 },
    { 0x575f1c7b, SUC_lexicon_tags_77 },
    { 0x9bff1c7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf95f1c7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f5f1c80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc21f1c82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ff1c87, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7f1c8f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x869f1c92, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3f1c95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd83f1c97, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687f1c9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27f1ca0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141f1ca4, SUC_lexicon_tags_23 },
    { 0xb5ff1ca5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0df1ca9, SUC_lexicon_tags_15 },
    { 0x81df1caa, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9f1cad, SUC_lexicon_tags_1 },
    { 0x5dbf1cad, SUC_lexicon_tags_0 },
    { 0x5a9f1caf, SUC_lexicon_tags_17 },
    { 0x46df1cb0, SUC_lexicon_tags_25 },
    { 0x9e5f1caf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x36bf1cb3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9f1cb6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbddf1cb8, SUC_lexicon_tags_14 },
    { 0x781f1cb9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ff1cbc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9f1cbf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11f1cc2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xa29f1cc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bf1cca, SUC_lexicon_tags_63 },
    { 0xc7df1ccb, SUC_lexicon_tags_1 },
    { 0xd3f1ccc, SUC_lexicon_tags_66 },
    { 0x989f1ccd, SUC_lexicon_tags_33 },
    { 0xfaff1ccc, SUC_lexicon_tags_27 },
    { 0x3c1f1cce, SUC_lexicon_tags_23 },
    { 0x747f1cd0, SUC_lexicon_tags_14 },
    { 0xcbdf1cd1, SUC_lexicon_tags_1 },
    { 0x1c9f1cd2, SUC_lexicon_tags_63 },
    { 0xb93f1cd0, SUC_lexicon_tags_21 },
    { 0x405f1cd4, SUC_lexicon_tags_6 },
    { 0x95bf1cd5, SUC_lexicon_tags_37 },
    { 0xab1f1cd6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd67f1cd8, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adf1cdb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817f1ce0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2bf1ce2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb9ff1ce4, SUC_lexicon_tags_26 },
    { 0x10ff1ce5, SUC_lexicon_tags_38 },
    { 0x179f1ce5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3c3f1ce8, SUC_lexicon_tags_60 },
    { 0x4cbf1ce8, SUC_lexicon_tags_2 },
    { 0x627f1ce8, SUC_lexicon_tags_1 },
    { 0x67ff1cea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37f1cee, SUC_lexicon_tags_9 },
    { 0x65bf1cef, SUC_lexicon_tags_43 },
    { 0x13df1cef, SUC_lexicon_tags_23 },
    { 0xf67f1cf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171f1cf5, SUC_lexicon_tags_7 },
    { 0x9edf1cf5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd61f1cf9, SUC_lexicon_tags_37 },
    { 0x171f1cf9, SUC_lexicon_tags_11 },
    { 0x13ff1cf9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x101f1cfe, SUC_lexicon_tags_29 },
    { 0xfa3f1cff, SUC_lexicon_tags_1 },
    { 0x77df1d00, SUC_lexicon_tags_25 },
    { 0xe31f1d01, SUC_lexicon_tags_2 },
    { 0x41f1d02, SUC_lexicon_tags_26 },
    { 0x575f1cff, SUC_lexicon_tags_17 },
    { 0x653f1d00, SUC_lexicon_tags_16 },
    { 0x33bf1d02, SUC_lexicon_tags_84 },
    { 0x5e7f1d06, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbf1d0c, SUC_lexicon_tags_37 },
    { 0x7adf1d0d, SUC_lexicon_tags_15 },
    { 0x68ff1d0d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbf1d11, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x90df1d14, SUC_lexicon_tags_43 },
    { 0xcd1f1d15, SUC_lexicon_tags_9 },
    { 0x68df1d14, SUC_lexicon_tags_26 },
    { 0xf53f1d16, SUC_lexicon_tags_71 },
    { 0xfd7f1d18, SUC_lexicon_tags_2 },
    { 0x71df1d19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbf1d23, SUC_lexicon_tags_204 },
    { 0x67df1d23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7f1d29, SUC_lexicon_tags_15 },
    { 0x123f1d2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe19f1d2c, SUC_lexicon_tags_12 },
    { 0x947f1d2d, SUC_lexicon_tags_6 },
    { 0xbd1f1d2e, SUC_lexicon_tags_15 },
    { 0xd73f1d2c, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181f1d3d, SUC_lexicon_tags_17 },
    { 0x507f1d3e, SUC_lexicon_tags_33 },
    { 0xa8bf1d3f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1e9f1d41, SUC_lexicon_tags_65 },
    { 0x805f1d42, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1f1d45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8bf1d47, SUC_lexicon_tags_15 },
    { 0xe75f1d48, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa3df1d4a, SUC_lexicon_tags_17 },
    { 0x4f7f1d4a, SUC_lexicon_tags_37 },
    { 0x775f1d4a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfff1d4e, SUC_lexicon_tags_49 },
    { 0x35ff1d4f, SUC_lexicon_tags_15 },
    { 0x4fdf1d50, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7f1d57, SUC_lexicon_tags_28 },
    { 0x53ff1d58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec5f1d5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ff1d5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3f1d63, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f1d66, SUC_lexicon_tags_37 },
    { 0xdf7f1d67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747f1d71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9f1d75, SUC_lexicon_tags_47 },
    { 0x2bbf1d76, SUC_lexicon_tags_46 },
    { 0x6a9f1d77, SUC_lexicon_tags_37 },
    { 0x1b1f1d76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873f1d86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bf1d8e, SUC_lexicon_tags_9 },
    { 0x95ff1d8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5ff1d91, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x85ff1d93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27f1d96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xce5f1d98, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ff1d9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x811f1d9d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xebf1da0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x723f1da2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ff1da5, SUC_lexicon_tags_35 },
    { 0x7d1f1da6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2bf1da8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877f1dae, SUC_lexicon_tags_15 },
    { 0x5edf1daf, SUC_lexicon_tags_28 },
    { 0xe59f1dae, SUC_lexicon_tags_37 },
    { 0xb0ff1dae, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2b3f1db3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x42df1db6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ff1db9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x955f1dbb, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811f1dbf, SUC_lexicon_tags_158 },
    { 0x189f1dc0, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xad7f1dc2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x3dff1dc4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e5f1dcc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfb7f1dce, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x25f1dd0, SUC_lexicon_tags_98 },
    { 0x4cdf1dd0, SUC_lexicon_tags_27 },
    { 0x9ff1dd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacff1ddf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5f1de6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9adf1de8, SUC_lexicon_tags_23 },
    { 0xa0df1de9, SUC_lexicon_tags_25 },
    { 0x4a5f1de9, SUC_lexicon_tags_17 },
    { 0xac1f1de8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5f1def, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3f1df2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdabf1df4, SUC_lexicon_tags_2 },
    { 0x71f1df5, SUC_lexicon_tags_25 },
    { 0x5c3f1df5, SUC_lexicon_tags_17 },
    { 0xcfdf1df5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61f1e00, SUC_lexicon_tags_23 },
    { 0xdfff1e01, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xa5df1e03, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x317f1e05, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x94df1e07, SUC_lexicon_tags_35 },
    { 0xb2bf1e08, SUC_lexicon_tags_32 },
    { 0xc6ff1e09, SUC_lexicon_tags_31 },
    { 0x915f1e09, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xae1f1e0d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x925f1e0f, SUC_lexicon_tags_19 },
    { 0x3d1f1e10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x763f1e12, SUC_lexicon_tags_12 },
    { 0x3b7f1e13, SUC_lexicon_tags_17 },
    { 0x33ff1e12, SUC_lexicon_tags_43 },
    { 0xbd3f1e15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ff1e1b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x287f1e1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xed3f1e20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xddbf1e22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bf1e26, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0xbf9f1e28, SUC_lexicon_tags_27 },
    { 0x943f1e28, SUC_lexicon_tags_37 },
    { 0xfcff1e2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb93f1e2c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ff1e32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37f1e36, SUC_lexicon_tags_17 },
    { 0x2a9f1e37, SUC_lexicon_tags_23 },
    { 0xf29f1e36, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbf1e3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1fbf1e3d, SUC_lexicon_tags_7 },
    { 0x60bf1e3e, SUC_lexicon_tags_43 },
    { 0xae5f1e3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f1e48, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837f1e4d, SUC_lexicon_tags_37 },
    { 0x585f1e4e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adf1e56, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c3f1e58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3f1e63, SUC_lexicon_tags_19 },
    { 0x6fdf1e63, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbff1e6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x707f1e6f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313f1e74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbf1e77, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7f1e7a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1df1e7e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bf1e82, SUC_lexicon_tags_7 },
    { 0xd9df1e83, SUC_lexicon_tags_25 },
    { 0xb77f1e84, SUC_lexicon_tags_60 },
    { 0x403f1e85, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81df1e8d, SUC_lexicon_tags_45 },
    { 0xd81f1e8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695f1e93, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe7bf1e95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7f1e98, SUC_lexicon_tags_11 },
    { 0x571f1e98, SUC_lexicon_tags_17 },
    { 0x3c5f1e9a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33f1ea2, SUC_lexicon_tags_1 },
    { 0x6c3f1ea2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70ff1ea5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f1ea8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x247f1eab, SUC_lexicon_tags_41 },
    { 0xa07f1eac, SUC_lexicon_tags_26 },
    { 0x99bf1eac, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25f1eb1, SUC_lexicon_tags_1 },
    { 0x7e9f1eb2, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbf1eb5, SUC_lexicon_tags_12 },
    { 0x951f1eb6, SUC_lexicon_tags_1 },
    { 0xad7f1eb7, SUC_lexicon_tags_5 },
    { 0xa5f1eb8, SUC_lexicon_tags_8 },
    { 0xb2ff1eb8, SUC_lexicon_tags_37 },
    { 0x1c7f1eba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb89f1ebc, SUC_lexicon_tags_49 },
    { 0x59f1ebc, SUC_lexicon_tags_46 },
    { 0xf8df1ebe, SUC_lexicon_tags_15 },
    { 0x4a7f1ebe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc45f1ec1, SUC_lexicon_tags_10 },
    { 0x601f1ec2, SUC_lexicon_tags_82 },
    { 0x683f1ec2, SUC_lexicon_tags_23 },
    { 0x90ff1ec2, SUC_lexicon_tags_15 },
    { 0xa5df1ec5, SUC_lexicon_tags_57 },
    { 0xcd3f1ec6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5f1ecb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75f1ed5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x68ff1ed7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191f1edb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x103f1ede, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x5e7f1ee0, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cff1ee3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x1ddf1ee5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x43f1ee8, SUC_lexicon_tags_12 },
    { 0xcfbf1ee8, SUC_lexicon_tags_79 },
    { 0x6adf1eea, SUC_lexicon_tags_19 },
    { 0x131f1eeb, SUC_lexicon_tags_3 },
    { 0x25bf1eeb, SUC_lexicon_tags_3 },
    { 0x87df1eed, SUC_lexicon_tags_11 },
    { 0x81bf1ee8, SUC_lexicon_tags_46 },
    { 0x697f1ee9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2aff1ef1, SUC_lexicon_tags_11 },
    { 0x35f1ef2, SUC_lexicon_tags_25 },
    { 0x669f1ef3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x25bf1ef6, SUC_lexicon_tags_2 },
    { 0xd5f1ef6, SUC_lexicon_tags_1 },
    { 0x29bf1ef8, SUC_lexicon_tags_7 },
    { 0x47df1ef9, SUC_lexicon_tags_7 },
    { 0x2a5f1efa, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9f1f03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1f09, SUC_lexicon_tags_37 },
    { 0x5bbf1f0a, SUC_lexicon_tags_12 },
    { 0xec7f1f0b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x20bf1f0f, SUC_lexicon_tags_23 },
    { 0xc7f1f0f, SUC_lexicon_tags_82 },
    { 0xe45f1f11, SUC_lexicon_tags_12 },
    { 0x923f1f11, SUC_lexicon_tags_5 },
    { 0xcc5f1f13, SUC_lexicon_tags_30 },
    { 0xb7bf1f12, SUC_lexicon_tags_0 },
    { 0x9e1f1f13, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6df1f22, SUC_lexicon_tags_33 },
    { 0xae1f1f23, SUC_lexicon_tags_485 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ff1f28, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63f1f2b, SUC_lexicon_tags_37 },
    { 0x97f1f2c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x133f1f30, SUC_lexicon_tags_25 },
    { 0x127f1f31, SUC_lexicon_tags_46 },
    { 0xdc5f1f30, SUC_lexicon_tags_17 },
    { 0x8d7f1f33, SUC_lexicon_tags_2 },
    { 0xd0ff1f34, SUC_lexicon_tags_45 },
    { 0xaa5f1f35, SUC_lexicon_tags_1 },
    { 0x99ff1f36, SUC_lexicon_tags_11 },
    { 0xdb7f1f37, SUC_lexicon_tags_37 },
    { 0xc41f1f34, SUC_lexicon_tags_27 },
    { 0xbb9f1f30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bf1f3d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x15df1f3f, SUC_lexicon_tags_274 },
    { 0xd4ff1f3f, SUC_lexicon_tags_29 },
    { 0xeadf1f41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x939f1f43, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa8bf1f45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359f1f4a, SUC_lexicon_tags_31 },
    { 0x8c1f1f4a, SUC_lexicon_tags_2 },
    { 0xcdf1f4c, SUC_lexicon_tags_113 },
    { 0xc7ff1f4b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cff1f51, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89f1f5a, SUC_lexicon_tags_12 },
    { 0xf1f1f5b, SUC_lexicon_tags_82 },
    { 0xc1ff1f5c, SUC_lexicon_tags_37 },
    { 0xae9f1f5b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adf1f61, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0x1abf1f63, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4b1f1f65, SUC_lexicon_tags_17 },
    { 0x711f1f66, SUC_lexicon_tags_7 },
    { 0xe75f1f66, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eff1f6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf95f1f6d, SUC_lexicon_tags_6 },
    { 0x2adf1f6e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cff1f71, SUC_lexicon_tags_12 },
    { 0x401f1f72, SUC_lexicon_tags_11 },
    { 0x4eff1f73, SUC_lexicon_tags_3 },
    { 0x475f1f74, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1fff1f76, SUC_lexicon_tags_8 },
    { 0xa3bf1f77, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97df1f82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xed9f1f84, SUC_lexicon_tags_14 },
    { 0x329f1f85, SUC_lexicon_tags_25 },
    { 0x337f1f85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbf1f8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5f1f90, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765f1f96, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7f1f9b, SUC_lexicon_tags_33 },
    { 0x637f1f9b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241f1fa3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8c5f1fa5, SUC_lexicon_tags_2 },
    { 0x2cff1fa5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd21f1fa9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x923f1fb0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11bf1fb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78df1fb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdf1fbe, SUC_lexicon_tags_0 },
    { 0xfb5f1fbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7f1fc2, SUC_lexicon_tags_14 },
    { 0xb89f1fc3, SUC_lexicon_tags_17 },
    { 0x9f3f1fc4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x431f1fc7, SUC_lexicon_tags_11 },
    { 0x8abf1fc7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7f1fcc, SUC_lexicon_tags_23 },
    { 0xe29f1fcd, SUC_lexicon_tags_82 },
    { 0x465f1fce, SUC_lexicon_tags_18 },
    { 0x3adf1fcd, SUC_lexicon_tags_8 },
    { 0x719f1fce, SUC_lexicon_tags_25 },
    { 0x8f3f1fcc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879f1fd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8ddf1fd8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbf1fdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3edf1fdf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf27f1fe1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x417f1fe3, SUC_lexicon_tags_27 },
    { 0x88df1fe4, SUC_lexicon_tags_14 },
    { 0xba1f1fe4, SUC_lexicon_tags_12 },
    { 0x73df1fe5, SUC_lexicon_tags_37 },
    { 0x585f1fe3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bf1fed, SUC_lexicon_tags_14 },
    { 0x77f1fed, SUC_lexicon_tags_9 },
    { 0x89f1fed, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x541f1ff1, SUC_lexicon_tags_15 },
    { 0xe3df1ff2, SUC_lexicon_tags_11 },
    { 0x1bf1ff2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cff1ffb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47df2009, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1df200d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3f2013, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f2019, SUC_lexicon_tags_107 },
    { 0xcfdf201a, SUC_lexicon_tags_47 },
    { 0xe5ff201b, SUC_lexicon_tags_938 },
    { 0, NULL },
    { 0xfd1f201d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ff2020, SUC_lexicon_tags_15 },
    { 0x4fdf2020, SUC_lexicon_tags_37 },
    { 0x495f2022, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1f2029, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x827f202b, SUC_lexicon_tags_2 },
    { 0x9ff202b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191f2030, SUC_lexicon_tags_17 },
    { 0x25f2030, SUC_lexicon_tags_11 },
    { 0x6cdf2030, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9f2035, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51f203d, SUC_lexicon_tags_0 },
    { 0xa75f203e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ff2042, SUC_lexicon_tags_29 },
    { 0x3ddf2043, SUC_lexicon_tags_55 },
    { 0xc97f2044, SUC_lexicon_tags_37 },
    { 0xce1f2045, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc21f2047, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1f204b, SUC_lexicon_tags_33 },
    { 0x109f204c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc77f204e, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x977f2051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd97f2053, SUC_lexicon_tags_11 },
    { 0x7bdf2054, SUC_lexicon_tags_19 },
    { 0x4b9f2054, SUC_lexicon_tags_1 },
    { 0x169f2053, SUC_lexicon_tags_12 },
    { 0xc15f2057, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3f205c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ff2060, SUC_lexicon_tags_7 },
    { 0x4f1f2061, SUC_lexicon_tags_47 },
    { 0xcc3f2060, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ff2065, SUC_lexicon_tags_12 },
    { 0xa61f2066, SUC_lexicon_tags_23 },
    { 0xdebf2065, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc85f206d, SUC_lexicon_tags_37 },
    { 0x393f206e, SUC_lexicon_tags_37 },
    { 0xc57f206f, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xa93f2071, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1df2076, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455f207a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x95f207c, SUC_lexicon_tags_43 },
    { 0xc95f207d, SUC_lexicon_tags_33 },
    { 0xc59f207e, SUC_lexicon_tags_27 },
    { 0xaf3f207d, SUC_lexicon_tags_45 },
    { 0x74bf207f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x211f2082, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc8df2084, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x83bf2086, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5f208a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7f208d, SUC_lexicon_tags_15 },
    { 0xc1f208d, SUC_lexicon_tags_25 },
    { 0x8d9f208f, SUC_lexicon_tags_37 },
    { 0xb15f208f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203f2094, SUC_lexicon_tags_17 },
    { 0xefdf2095, SUC_lexicon_tags_45 },
    { 0xa85f2095, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf2099, SUC_lexicon_tags_37 },
    { 0x46df209a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc6df209c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7f209f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333f20a4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3f20aa, SUC_lexicon_tags_37 },
    { 0xf7ff20ab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f20ae, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbf20b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945f20b7, SUC_lexicon_tags_25 },
    { 0xbc7f20b8, SUC_lexicon_tags_46 },
    { 0x3aff20b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7f20be, SUC_lexicon_tags_17 },
    { 0x7f3f20bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x4b5f20c1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x11df20c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xbedf20c5, SUC_lexicon_tags_23 },
    { 0xb45f20c6, SUC_lexicon_tags_8 },
    { 0xae5f20c7, SUC_lexicon_tags_27 },
    { 0xe07f20c8, SUC_lexicon_tags_38 },
    { 0x2a1f20c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x805f20cb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171f20d6, SUC_lexicon_tags_7 },
    { 0x215f20d7, SUC_lexicon_tags_30 },
    { 0x985f20d8, SUC_lexicon_tags_28 },
    { 0xe81f20d7, SUC_lexicon_tags_19 },
    { 0xfbff20da, SUC_lexicon_tags_15 },
    { 0x583f20d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15f20df, SUC_lexicon_tags_37 },
    { 0xb2df20e0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21f20e5, SUC_lexicon_tags_17 },
    { 0x2a1f20e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc65f20e8, SUC_lexicon_tags_77 },
    { 0x5e5f20e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7df20ed, SUC_lexicon_tags_19 },
    { 0xc45f20ed, SUC_lexicon_tags_17 },
    { 0x6a3f20ef, SUC_lexicon_tags_63 },
    { 0x7d1f20ed, SUC_lexicon_tags_82 },
    { 0xd1bf20f1, SUC_lexicon_tags_37 },
    { 0xd15f20ed, SUC_lexicon_tags_1 },
    { 0x51df20f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5f20f6, SUC_lexicon_tags_25 },
    { 0xda3f20f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf20fa, SUC_lexicon_tags_25 },
    { 0x48bf20fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29f20fe, SUC_lexicon_tags_29 },
    { 0xf21f20fe, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x2dbf2101, SUC_lexicon_tags_11 },
    { 0xe87f2102, SUC_lexicon_tags_27 },
    { 0xf89f2102, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2df2105, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9f2109, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3f210e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551f2115, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437f211a, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x88df211c, SUC_lexicon_tags_29 },
    { 0xd19f211d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x337f211f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1f2122, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x135f2124, SUC_lexicon_tags_45 },
    { 0xf3df2125, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf97f2128, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x957f212a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x63ff212c, SUC_lexicon_tags_27 },
    { 0x441f212d, SUC_lexicon_tags_30 },
    { 0xb93f212d, SUC_lexicon_tags_25 },
    { 0xda9f212e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x845f2131, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x443f2134, SUC_lexicon_tags_290 },
    { 0, NULL },
    { 0x8d1f2136, SUC_lexicon_tags_28 },
    { 0xa93f2136, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x373f213a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdf2141, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x83f2143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ff2146, SUC_lexicon_tags_17 },
    { 0x221f2146, SUC_lexicon_tags_1 },
    { 0x8bff2148, SUC_lexicon_tags_11 },
    { 0xb0bf2148, SUC_lexicon_tags_63 },
    { 0x2b7f2148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ff214d, SUC_lexicon_tags_12 },
    { 0xa53f214d, SUC_lexicon_tags_25 },
    { 0x53ff214f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf2156, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927f215b, SUC_lexicon_tags_8 },
    { 0x27ff215b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4b7f215e, SUC_lexicon_tags_43 },
    { 0xc73f215f, SUC_lexicon_tags_37 },
    { 0xff3f2160, SUC_lexicon_tags_31 },
    { 0x403f215f, SUC_lexicon_tags_1 },
    { 0x9a5f215f, SUC_lexicon_tags_33 },
    { 0x9bff2161, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0xd11f2165, SUC_lexicon_tags_45 },
    { 0x84df2166, SUC_lexicon_tags_37 },
    { 0xe5f2166, SUC_lexicon_tags_47 },
    { 0x58df2166, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x909f216a, SUC_lexicon_tags_39 },
    { 0x8d3f216b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xce7f216d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9f2178, SUC_lexicon_tags_25 },
    { 0xe11f2178, SUC_lexicon_tags_37 },
    { 0x65f2178, SUC_lexicon_tags_25 },
    { 0x6b5f2178, SUC_lexicon_tags_25 },
    { 0x5d1f217b, SUC_lexicon_tags_55 },
    { 0xb79f217a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdf2180, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3edf2182, SUC_lexicon_tags_7 },
    { 0x693f2183, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x3ebf2185, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x767f2188, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x705f218b, SUC_lexicon_tags_12 },
    { 0x46ff218c, SUC_lexicon_tags_30 },
    { 0x353f218d, SUC_lexicon_tags_13 },
    { 0xb37f218d, SUC_lexicon_tags_2 },
    { 0x9bbf218c, SUC_lexicon_tags_17 },
    { 0x52bf218b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x755f2192, SUC_lexicon_tags_17 },
    { 0xe99f2192, SUC_lexicon_tags_15 },
    { 0x347f2193, SUC_lexicon_tags_15 },
    { 0xbdff2195, SUC_lexicon_tags_12 },
    { 0xae9f2196, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adf219b, SUC_lexicon_tags_37 },
    { 0xa89f219b, SUC_lexicon_tags_82 },
    { 0x94df219d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf21a5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31f21a8, SUC_lexicon_tags_26 },
    { 0xac5f21a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x955f21ac, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79f21b4, SUC_lexicon_tags_24 },
    { 0xd3f21b5, SUC_lexicon_tags_48 },
    { 0x52ff21b6, SUC_lexicon_tags_84 },
    { 0x70bf21b6, SUC_lexicon_tags_14 },
    { 0xd4ff21b7, SUC_lexicon_tags_19 },
    { 0x6b1f21b6, SUC_lexicon_tags_8 },
    { 0x453f21ba, SUC_lexicon_tags_12 },
    { 0x507f21ba, SUC_lexicon_tags_11 },
    { 0xf97f21bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559f21c0, SUC_lexicon_tags_10 },
    { 0xe9df21c1, SUC_lexicon_tags_25 },
    { 0x8df21c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7f21c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5f21c8, SUC_lexicon_tags_39 },
    { 0x7edf21c9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x315f21cb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xebbf21cd, SUC_lexicon_tags_298 },
    { 0xeebf21cd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4a9f21d0, SUC_lexicon_tags_47 },
    { 0x353f21d1, SUC_lexicon_tags_6 },
    { 0x439f21d2, SUC_lexicon_tags_12 },
    { 0x7bff21d2, SUC_lexicon_tags_33 },
    { 0xf95f21d2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc63f21d6, SUC_lexicon_tags_38 },
    { 0xb97f21d7, SUC_lexicon_tags_66 },
    { 0xae5f21d8, SUC_lexicon_tags_9 },
    { 0xea5f21d8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ff21de, SUC_lexicon_tags_23 },
    { 0x727f21df, SUC_lexicon_tags_14 },
    { 0x9d3f21e0, SUC_lexicon_tags_11 },
    { 0xb5df21de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5f21e4, SUC_lexicon_tags_27 },
    { 0xabdf21e5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27f21ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f3f21ec, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x573f21ee, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cff21f4, SUC_lexicon_tags_39 },
    { 0xc27f21f5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x319f21f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9f2202, SUC_lexicon_tags_25 },
    { 0x44ff2203, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x495f2206, SUC_lexicon_tags_38 },
    { 0xfa9f2206, SUC_lexicon_tags_39 },
    { 0x7e5f2207, SUC_lexicon_tags_17 },
    { 0x15ff2209, SUC_lexicon_tags_12 },
    { 0x8f7f220a, SUC_lexicon_tags_19 },
    { 0x1e9f220b, SUC_lexicon_tags_23 },
    { 0x96ff220b, SUC_lexicon_tags_34 },
    { 0x267f2206, SUC_lexicon_tags_11 },
    { 0xf6df220e, SUC_lexicon_tags_32 },
    { 0x3d5f220b, SUC_lexicon_tags_0 },
    { 0xc61f220d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f2213, SUC_lexicon_tags_341 },
    { 0, NULL },
    { 0x35ff2215, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb17f221b, SUC_lexicon_tags_2 },
    { 0x983f221c, SUC_lexicon_tags_12 },
    { 0x5c9f221d, SUC_lexicon_tags_158 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7f2220, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625f2227, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xb13f222a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x869f222d, SUC_lexicon_tags_1 },
    { 0x38ff222d, SUC_lexicon_tags_23 },
    { 0x259f222f, SUC_lexicon_tags_13 },
    { 0xa75f2230, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe1bf2232, SUC_lexicon_tags_37 },
    { 0x515f2233, SUC_lexicon_tags_25 },
    { 0x1b3f2234, SUC_lexicon_tags_47 },
    { 0xd7f2234, SUC_lexicon_tags_7 },
    { 0x943f2236, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xac9f2238, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb85f223a, SUC_lexicon_tags_12 },
    { 0x1a5f223a, SUC_lexicon_tags_11 },
    { 0x5e1f223c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x785f223e, SUC_lexicon_tags_26 },
    { 0x641f223f, SUC_lexicon_tags_15 },
    { 0xca3f2240, SUC_lexicon_tags_2 },
    { 0xa1ff2241, SUC_lexicon_tags_17 },
    { 0x2e1f2242, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x79f2245, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19f2249, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0x417f224c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767f2251, SUC_lexicon_tags_49 },
    { 0x405f2251, SUC_lexicon_tags_32 },
    { 0xa07f2253, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73f2257, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x447f225c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x145f225e, SUC_lexicon_tags_2 },
    { 0xc1f225f, SUC_lexicon_tags_17 },
    { 0x7ff2260, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa8df2262, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5f2268, SUC_lexicon_tags_118 },
    { 0x6adf2269, SUC_lexicon_tags_15 },
    { 0xb27f226a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc25f2276, SUC_lexicon_tags_11 },
    { 0x223f2277, SUC_lexicon_tags_31 },
    { 0xe91f2278, SUC_lexicon_tags_1 },
    { 0xbcdf2279, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x281f227c, SUC_lexicon_tags_60 },
    { 0x84df227d, SUC_lexicon_tags_48 },
    { 0x61f227d, SUC_lexicon_tags_11 },
    { 0x611f227c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x139f2282, SUC_lexicon_tags_25 },
    { 0x973f2283, SUC_lexicon_tags_23 },
    { 0x8eff2284, SUC_lexicon_tags_34 },
    { 0x179f2285, SUC_lexicon_tags_20 },
    { 0x6a1f2285, SUC_lexicon_tags_38 },
    { 0x6adf2286, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57f228c, SUC_lexicon_tags_37 },
    { 0xb55f228d, SUC_lexicon_tags_47 },
    { 0x3f9f228d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8df2290, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x705f2294, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871f2298, SUC_lexicon_tags_37 },
    { 0x81df2299, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x721f229c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbf22aa, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33f22ad, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179f22b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5f22b9, SUC_lexicon_tags_83 },
    { 0x2f7f22ba, SUC_lexicon_tags_55 },
    { 0x1f7f22bb, SUC_lexicon_tags_0 },
    { 0xf79f22bc, SUC_lexicon_tags_23 },
    { 0xa9ff22bd, SUC_lexicon_tags_26 },
    { 0xc1ff22bc, SUC_lexicon_tags_43 },
    { 0x3d5f22be, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7eff22c1, SUC_lexicon_tags_7 },
    { 0x6d5f22c2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x655f22c4, SUC_lexicon_tags_129 },
    { 0xf3ff22c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5f22c7, SUC_lexicon_tags_43 },
    { 0x43df22c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bf22cb, SUC_lexicon_tags_23 },
    { 0x3d7f22cc, SUC_lexicon_tags_7 },
    { 0x7cff22cc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x783f22d0, SUC_lexicon_tags_15 },
    { 0xa47f22d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x72df22d3, SUC_lexicon_tags_2 },
    { 0xf49f22d3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bf22db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37df22e5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6e9f22e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bf22ea, SUC_lexicon_tags_2 },
    { 0x2cff22eb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15f22ef, SUC_lexicon_tags_23 },
    { 0x8e5f22f0, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0x909f22f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5f22f7, SUC_lexicon_tags_0 },
    { 0xef5f22f8, SUC_lexicon_tags_12 },
    { 0x6f3f22f7, SUC_lexicon_tags_14 },
    { 0x75f22f9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3f2300, SUC_lexicon_tags_15 },
    { 0x3d9f2301, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1f2305, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429f230b, SUC_lexicon_tags_30 },
    { 0x84ff230b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f2310, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479f231b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2f9f231d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd2ff231f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bf2324, SUC_lexicon_tags_8 },
    { 0x527f2324, SUC_lexicon_tags_27 },
    { 0xe61f2326, SUC_lexicon_tags_107 },
    { 0x807f2325, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83df232b, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x27f232d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff233b, SUC_lexicon_tags_60 },
    { 0xcc7f233c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c3f233e, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x1b9f2340, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34df2344, SUC_lexicon_tags_25 },
    { 0x40bf2345, SUC_lexicon_tags_47 },
    { 0x205f2346, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x323f2348, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f234d, SUC_lexicon_tags_8 },
    { 0xd97f234e, SUC_lexicon_tags_37 },
    { 0x64ff234d, SUC_lexicon_tags_17 },
    { 0x98df234f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9b7f2352, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x603f2355, SUC_lexicon_tags_28 },
    { 0x8e1f2355, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9f235c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f2362, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f2365, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bf2368, SUC_lexicon_tags_2 },
    { 0x11ff2368, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7f236c, SUC_lexicon_tags_7 },
    { 0x31ff236c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3f2371, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237f237b, SUC_lexicon_tags_11 },
    { 0x6ddf237b, SUC_lexicon_tags_15 },
    { 0x7c7f237d, SUC_lexicon_tags_37 },
    { 0xe45f237e, SUC_lexicon_tags_25 },
    { 0x215f237f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x691f2382, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5f2387, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xea5f2389, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7f238c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2e5f238e, SUC_lexicon_tags_11 },
    { 0xe85f238e, SUC_lexicon_tags_63 },
    { 0x41ff2390, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xeedf2392, SUC_lexicon_tags_33 },
    { 0xde1f2393, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa33f2395, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd81f2397, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xea9f2399, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedff23a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bf23a4, SUC_lexicon_tags_48 },
    { 0x3cdf23a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3f23aa, SUC_lexicon_tags_32 },
    { 0xf31f23ab, SUC_lexicon_tags_7 },
    { 0x5ff23ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5f23b6, SUC_lexicon_tags_17 },
    { 0x363f23b7, SUC_lexicon_tags_243 },
    { 0xeaff23b8, SUC_lexicon_tags_2 },
    { 0xf47f23b9, SUC_lexicon_tags_13 },
    { 0x2dff23ba, SUC_lexicon_tags_30 },
    { 0xae7f23ba, SUC_lexicon_tags_25 },
    { 0xb93f23b6, SUC_lexicon_tags_14 },
    { 0x61f23bd, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807f23c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed7f23c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x749f23ca, SUC_lexicon_tags_15 },
    { 0x183f23cb, SUC_lexicon_tags_31 },
    { 0xae3f23cc, SUC_lexicon_tags_23 },
    { 0x8edf23cd, SUC_lexicon_tags_11 },
    { 0x7c9f23ce, SUC_lexicon_tags_25 },
    { 0xf2df23cb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x273f23d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdadf23d5, SUC_lexicon_tags_44 },
    { 0x1b3f23d6, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x705f23d8, SUC_lexicon_tags_60 },
    { 0xc3df23d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd7df23db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x32ff23dd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xcdf23df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd8df23e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3c9f23e3, SUC_lexicon_tags_26 },
    { 0x39f23e4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xa5ff23e6, SUC_lexicon_tags_9 },
    { 0xaadf23e7, SUC_lexicon_tags_92 },
    { 0x1f5f23e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb85f23ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ff23f1, SUC_lexicon_tags_17 },
    { 0x5ddf23f1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1f23f6, SUC_lexicon_tags_33 },
    { 0x689f23f6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x31df23f9, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0xcc3f23fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70df2401, SUC_lexicon_tags_11 },
    { 0x7d7f2402, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743f2407, SUC_lexicon_tags_23 },
    { 0xd49f2407, SUC_lexicon_tags_46 },
    { 0x855f2409, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x651f240c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1f240f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bf2413, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51f2417, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61f241a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x487f241c, SUC_lexicon_tags_12 },
    { 0x13f241c, SUC_lexicon_tags_0 },
    { 0x4a5f241e, SUC_lexicon_tags_23 },
    { 0x54ff241f, SUC_lexicon_tags_37 },
    { 0x923f241c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x151f2422, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xee7f2424, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715f242a, SUC_lexicon_tags_45 },
    { 0x7a9f242b, SUC_lexicon_tags_23 },
    { 0x8cff242c, SUC_lexicon_tags_118 },
    { 0x5d1f242d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c9f242f, SUC_lexicon_tags_63 },
    { 0xeedf242f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x74df2432, SUC_lexicon_tags_11 },
    { 0xfa5f2433, SUC_lexicon_tags_7 },
    { 0xaa5f2433, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9f2437, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1f243e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3dff2440, SUC_lexicon_tags_47 },
    { 0x473f2440, SUC_lexicon_tags_12 },
    { 0x523f2442, SUC_lexicon_tags_11 },
    { 0x6bff2443, SUC_lexicon_tags_8 },
    { 0xfbdf2443, SUC_lexicon_tags_47 },
    { 0xc7bf2445, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53f244b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3f244e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3c3f2450, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7f2453, SUC_lexicon_tags_77 },
    { 0xecff2454, SUC_lexicon_tags_8 },
    { 0x2ff2455, SUC_lexicon_tags_4 },
    { 0xa47f2455, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x12bf2459, SUC_lexicon_tags_27 },
    { 0x501f245a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x25f245d, SUC_lexicon_tags_1 },
    { 0x38bf245e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdf2461, SUC_lexicon_tags_11 },
    { 0xfb7f2461, SUC_lexicon_tags_37 },
    { 0xd11f2463, SUC_lexicon_tags_19 },
    { 0x295f2464, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfb3f2466, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xee7f2469, SUC_lexicon_tags_37 },
    { 0x3f9f246a, SUC_lexicon_tags_7 },
    { 0xa3f2469, SUC_lexicon_tags_15 },
    { 0x677f246c, SUC_lexicon_tags_23 },
    { 0x62ff246b, SUC_lexicon_tags_12 },
    { 0xf1df246e, SUC_lexicon_tags_2 },
    { 0xd9df246f, SUC_lexicon_tags_17 },
    { 0xb87f246f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7f2473, SUC_lexicon_tags_15 },
    { 0x47f2473, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdf2478, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31f2480, SUC_lexicon_tags_84 },
    { 0x605f2481, SUC_lexicon_tags_1 },
    { 0x25ff2480, SUC_lexicon_tags_68 },
    { 0x385f2482, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1ebf2485, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957f2489, SUC_lexicon_tags_37 },
    { 0xb87f248a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x899f248d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ff2493, SUC_lexicon_tags_25 },
    { 0x4fbf2493, SUC_lexicon_tags_19 },
    { 0x793f2493, SUC_lexicon_tags_11 },
    { 0x4b7f2495, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bf249b, SUC_lexicon_tags_0 },
    { 0xc6ff249c, SUC_lexicon_tags_46 },
    { 0x405f249d, SUC_lexicon_tags_31 },
    { 0x487f249e, SUC_lexicon_tags_17 },
    { 0xf79f249f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371f24a7, SUC_lexicon_tags_1 },
    { 0x215f24a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xef3f24aa, SUC_lexicon_tags_45 },
    { 0x29bf24aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x245f24ae, SUC_lexicon_tags_46 },
    { 0x42bf24ae, SUC_lexicon_tags_12 },
    { 0x26ff24b0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ff24b5, SUC_lexicon_tags_60 },
    { 0x6bdf24b6, SUC_lexicon_tags_107 },
    { 0x827f24b6, SUC_lexicon_tags_192 },
    { 0xd4ff24b8, SUC_lexicon_tags_9 },
    { 0x919f24b7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38df24bd, SUC_lexicon_tags_2 },
    { 0x4b3f24be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfd3f24c0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673f24c4, SUC_lexicon_tags_47 },
    { 0x4ebf24c5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfe9f24c7, SUC_lexicon_tags_0 },
    { 0x617f24c8, SUC_lexicon_tags_12 },
    { 0xd8bf24c9, SUC_lexicon_tags_11 },
    { 0x11ff24ca, SUC_lexicon_tags_9 },
    { 0x5fdf24cb, SUC_lexicon_tags_7 },
    { 0xcf9f24cc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bf24d2, SUC_lexicon_tags_5 },
    { 0x43ff24d3, SUC_lexicon_tags_60 },
    { 0xac5f24d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe63f24d6, SUC_lexicon_tags_22 },
    { 0xe61f24d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261f24e0, SUC_lexicon_tags_2 },
    { 0x48bf24e1, SUC_lexicon_tags_37 },
    { 0x157f24e0, SUC_lexicon_tags_29 },
    { 0xe7df24e3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa57f24e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221f24ec, SUC_lexicon_tags_28 },
    { 0xd51f24ed, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31f24f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf24f7, SUC_lexicon_tags_12 },
    { 0x203f24f7, SUC_lexicon_tags_82 },
    { 0xf69f24f8, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13f2501, SUC_lexicon_tags_17 },
    { 0x9ebf2502, SUC_lexicon_tags_8 },
    { 0x2dbf2503, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1f2507, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x37ff250a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9f2510, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xde5f2512, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x551f2514, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9ff2516, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5f2519, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3f251e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2aff2522, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905f2526, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x219f252b, SUC_lexicon_tags_7 },
    { 0x89df252c, SUC_lexicon_tags_12 },
    { 0x925f252d, SUC_lexicon_tags_8 },
    { 0x3f5f252e, SUC_lexicon_tags_47 },
    { 0xdb3f252f, SUC_lexicon_tags_25 },
    { 0x541f2530, SUC_lexicon_tags_47 },
    { 0x4c3f2530, SUC_lexicon_tags_33 },
    { 0xfdf2531, SUC_lexicon_tags_7 },
    { 0xc99f2533, SUC_lexicon_tags_11 },
    { 0x1b5f2533, SUC_lexicon_tags_262 },
    { 0x9f5f2535, SUC_lexicon_tags_9 },
    { 0xe53f2530, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbf253c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb7f253e, SUC_lexicon_tags_37 },
    { 0x4e1f253f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdf2545, SUC_lexicon_tags_25 },
    { 0x185f2546, SUC_lexicon_tags_87 },
    { 0xd69f2547, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbf254a, SUC_lexicon_tags_30 },
    { 0xefbf254a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd31f254d, SUC_lexicon_tags_11 },
    { 0x15ff254e, SUC_lexicon_tags_1 },
    { 0xcadf254f, SUC_lexicon_tags_19 },
    { 0x3eff254f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe29f2552, SUC_lexicon_tags_12 },
    { 0x921f2553, SUC_lexicon_tags_39 },
    { 0x4edf2553, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441f2559, SUC_lexicon_tags_39 },
    { 0x82df2559, SUC_lexicon_tags_23 },
    { 0xf95f255b, SUC_lexicon_tags_61 },
    { 0xedbf255c, SUC_lexicon_tags_2 },
    { 0xbe5f255d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ff2564, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1f2567, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ff256a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bff256f, SUC_lexicon_tags_37 },
    { 0x3a9f256f, SUC_lexicon_tags_27 },
    { 0x9b3f2570, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb5f2573, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c7f2575, SUC_lexicon_tags_12 },
    { 0x945f2576, SUC_lexicon_tags_7 },
    { 0x57ff2577, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x211f257a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523f2582, SUC_lexicon_tags_15 },
    { 0xb65f2583, SUC_lexicon_tags_17 },
    { 0x32ff2583, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3f2589, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x45ff258c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24bf2590, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbabf2592, SUC_lexicon_tags_31 },
    { 0x34ff2593, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x889f2595, SUC_lexicon_tags_17 },
    { 0x435f2596, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9f259b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x72df259e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bf25a1, SUC_lexicon_tags_85 },
    { 0xa63f25a1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x20df25a5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb9df25a7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf25ad, SUC_lexicon_tags_12 },
    { 0x6a1f25ad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf99f25b0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59f25b4, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b9f25b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35f25bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333f25c2, SUC_lexicon_tags_30 },
    { 0x713f25c2, SUC_lexicon_tags_28 },
    { 0xe51f25c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7f25c9, SUC_lexicon_tags_1 },
    { 0x43df25ca, SUC_lexicon_tags_48 },
    { 0x209f25ca, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x569f25cd, SUC_lexicon_tags_7 },
    { 0x3f3f25ce, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bff25d2, SUC_lexicon_tags_27 },
    { 0x533f25d3, SUC_lexicon_tags_38 },
    { 0x39bf25d2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe07f25d6, SUC_lexicon_tags_8 },
    { 0xa05f25d7, SUC_lexicon_tags_31 },
    { 0x8df25d8, SUC_lexicon_tags_77 },
    { 0x543f25d9, SUC_lexicon_tags_17 },
    { 0x72bf25d8, SUC_lexicon_tags_19 },
    { 0x515f25d7, SUC_lexicon_tags_15 },
    { 0xb27f25d6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437f25e3, SUC_lexicon_tags_37 },
    { 0xf77f25e4, SUC_lexicon_tags_30 },
    { 0xc3f25e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7f25e9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x169f25eb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445f25f1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf1f25f3, SUC_lexicon_tags_29 },
    { 0x757f25f4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bf25fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb57f25fd, SUC_lexicon_tags_19 },
    { 0x36df25fe, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993f2606, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23f2609, SUC_lexicon_tags_25 },
    { 0x52df2609, SUC_lexicon_tags_11 },
    { 0x87bf260a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9f2614, SUC_lexicon_tags_106 },
    { 0x2c7f2615, SUC_lexicon_tags_19 },
    { 0x741f2614, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93df261b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7dbf261d, SUC_lexicon_tags_15 },
    { 0x7fbf261d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x883f2620, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x689f2623, SUC_lexicon_tags_3 },
    { 0xabff2623, SUC_lexicon_tags_11 },
    { 0x1cbf2625, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25f262a, SUC_lexicon_tags_40 },
    { 0xe65f262a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb1bf262d, SUC_lexicon_tags_0 },
    { 0x873f262e, SUC_lexicon_tags_46 },
    { 0x9d7f262f, SUC_lexicon_tags_11 },
    { 0x6ddf2630, SUC_lexicon_tags_19 },
    { 0x4fdf2631, SUC_lexicon_tags_49 },
    { 0x7adf2630, SUC_lexicon_tags_15 },
    { 0x9a5f2633, SUC_lexicon_tags_0 },
    { 0xd0bf2634, SUC_lexicon_tags_12 },
    { 0xf57f2634, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x171f2638, SUC_lexicon_tags_23 },
    { 0x489f2639, SUC_lexicon_tags_45 },
    { 0xd2ff263a, SUC_lexicon_tags_23 },
    { 0x88ff263b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbff263e, SUC_lexicon_tags_17 },
    { 0xb7bf263f, SUC_lexicon_tags_39 },
    { 0x23ff263f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa9f2642, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x31f2644, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf1ff2646, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x377f2649, SUC_lexicon_tags_6 },
    { 0xa73f264a, SUC_lexicon_tags_14 },
    { 0xd89f2649, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x633f2650, SUC_lexicon_tags_47 },
    { 0xf61f2650, SUC_lexicon_tags_12 },
    { 0xb57f2652, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ff2655, SUC_lexicon_tags_113 },
    { 0x671f2656, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4d7f2658, SUC_lexicon_tags_15 },
    { 0x419f2659, SUC_lexicon_tags_77 },
    { 0x9dbf265a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54ff265c, SUC_lexicon_tags_1 },
    { 0xcc5f265d, SUC_lexicon_tags_12 },
    { 0x761f265e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x181f2661, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3b7f2663, SUC_lexicon_tags_7 },
    { 0xfc9f2663, SUC_lexicon_tags_2 },
    { 0xcf7f2663, SUC_lexicon_tags_70 },
    { 0x75df2663, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43f2669, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e5f266b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x873f266d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x747f2671, SUC_lexicon_tags_14 },
    { 0xf77f2672, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5f2676, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ff267a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f3f267c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf267f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77ff2681, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5f2684, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ff2687, SUC_lexicon_tags_87 },
    { 0x4df2688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x88df268b, SUC_lexicon_tags_11 },
    { 0x8f1f268c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x523f268e, SUC_lexicon_tags_17 },
    { 0x17bf268e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75f2695, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eff2698, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x635f269c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f26a0, SUC_lexicon_tags_11 },
    { 0xe05f26a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9f26a5, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x447f26a7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863f26ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3f26b1, SUC_lexicon_tags_14 },
    { 0x8b7f26b2, SUC_lexicon_tags_7 },
    { 0xc61f26b3, SUC_lexicon_tags_17 },
    { 0x999f26b2, SUC_lexicon_tags_31 },
    { 0x1f26b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19f26bd, SUC_lexicon_tags_17 },
    { 0x77ff26be, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x76df26c1, SUC_lexicon_tags_37 },
    { 0x25bf26c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbd9f26c4, SUC_lexicon_tags_17 },
    { 0xd4bf26c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9f26c8, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09f26cc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdf26d9, SUC_lexicon_tags_46 },
    { 0xce7f26d9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb03f26dc, SUC_lexicon_tags_7 },
    { 0xef7f26dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86df26e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ff26ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3f26f0, SUC_lexicon_tags_8 },
    { 0xb19f26f0, SUC_lexicon_tags_37 },
    { 0xd59f26f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31f26f9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x177f26fc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71f26ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe1f2701, SUC_lexicon_tags_123 },
    { 0x751f2701, SUC_lexicon_tags_17 },
    { 0xecff2703, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfff270c, SUC_lexicon_tags_9 },
    { 0x3a3f270c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xee3f2710, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb27f2712, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67f2715, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbff271e, SUC_lexicon_tags_15 },
    { 0x311f271f, SUC_lexicon_tags_27 },
    { 0xe6ff2720, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdf7f2722, SUC_lexicon_tags_1 },
    { 0x735f2723, SUC_lexicon_tags_49 },
    { 0x8fdf2724, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xd61f2726, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7f272b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6cdf272d, SUC_lexicon_tags_2 },
    { 0xf89f272e, SUC_lexicon_tags_49 },
    { 0x9fdf272f, SUC_lexicon_tags_2 },
    { 0x9cbf2730, SUC_lexicon_tags_1 },
    { 0xcd1f2730, SUC_lexicon_tags_1 },
    { 0x89f2731, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bf273c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdf2740, SUC_lexicon_tags_82 },
    { 0xaeff2741, SUC_lexicon_tags_15 },
    { 0x63ff2742, SUC_lexicon_tags_56 },
    { 0xbc9f2743, SUC_lexicon_tags_17 },
    { 0xd0ff2744, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfb5f2746, SUC_lexicon_tags_6 },
    { 0x561f2747, SUC_lexicon_tags_11 },
    { 0x207f2748, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25f274b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba5f274d, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0x2df274f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaff2758, SUC_lexicon_tags_11 },
    { 0x15ff2759, SUC_lexicon_tags_2 },
    { 0xa21f275a, SUC_lexicon_tags_1 },
    { 0x8fbf275b, SUC_lexicon_tags_47 },
    { 0xd9f275a, SUC_lexicon_tags_2 },
    { 0xbb1f275b, SUC_lexicon_tags_38 },
    { 0x9bff275e, SUC_lexicon_tags_25 },
    { 0xeeff275e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xb7f2761, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7f276d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549f2771, SUC_lexicon_tags_7 },
    { 0x663f2772, SUC_lexicon_tags_10 },
    { 0xfcff2773, SUC_lexicon_tags_19 },
    { 0x2ddf2774, SUC_lexicon_tags_7 },
    { 0xf0bf2775, SUC_lexicon_tags_60 },
    { 0x8c5f2774, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf3bf2778, SUC_lexicon_tags_791 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7df277c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9f2784, SUC_lexicon_tags_17 },
    { 0xf1f2785, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x447f2787, SUC_lexicon_tags_9 },
    { 0x635f2788, SUC_lexicon_tags_49 },
    { 0xa31f2789, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463f278f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x87ff2791, SUC_lexicon_tags_25 },
    { 0x667f2791, SUC_lexicon_tags_3 },
    { 0x21df2792, SUC_lexicon_tags_140 },
    { 0xde3f2794, SUC_lexicon_tags_25 },
    { 0x29ff2794, SUC_lexicon_tags_220 },
    { 0x8cbf2795, SUC_lexicon_tags_53 },
    { 0xea3f2792, SUC_lexicon_tags_11 },
    { 0x405f2797, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559f279e, SUC_lexicon_tags_70 },
    { 0x281f279f, SUC_lexicon_tags_25 },
    { 0x2f9f27a0, SUC_lexicon_tags_11 },
    { 0xfcff27a1, SUC_lexicon_tags_38 },
    { 0xa1bf27a2, SUC_lexicon_tags_29 },
    { 0xdaff27a2, SUC_lexicon_tags_12 },
    { 0xccff27a4, SUC_lexicon_tags_4 },
    { 0x5e9f27a4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd53f27a7, SUC_lexicon_tags_47 },
    { 0xb0df27a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbbf27aa, SUC_lexicon_tags_65 },
    { 0x395f27ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5f27af, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5f27b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x137f27b5, SUC_lexicon_tags_3 },
    { 0xc63f27b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23bf27b8, SUC_lexicon_tags_9 },
    { 0xe8bf27b9, SUC_lexicon_tags_17 },
    { 0x5a9f27b9, SUC_lexicon_tags_1 },
    { 0x123f27b8, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ff27be, SUC_lexicon_tags_97 },
    { 0x3d3f27be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb67f27c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bf27c7, SUC_lexicon_tags_5 },
    { 0x2d1f27c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x69bf27cb, SUC_lexicon_tags_7 },
    { 0xd07f27cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x397f27ce, SUC_lexicon_tags_0 },
    { 0x6e1f27cf, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdff27d3, SUC_lexicon_tags_38 },
    { 0x9b9f27d4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ff27d8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755f27dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa1bf27de, SUC_lexicon_tags_26 },
    { 0x801f27df, SUC_lexicon_tags_49 },
    { 0x357f27df, SUC_lexicon_tags_17 },
    { 0x73f27e1, SUC_lexicon_tags_37 },
    { 0x4fdf27e0, SUC_lexicon_tags_32 },
    { 0x303f27df, SUC_lexicon_tags_70 },
    { 0xdd7f27e4, SUC_lexicon_tags_17 },
    { 0x775f27e5, SUC_lexicon_tags_30 },
    { 0xc47f27e6, SUC_lexicon_tags_17 },
    { 0x283f27e0, SUC_lexicon_tags_14 },
    { 0xc1ff27e8, SUC_lexicon_tags_6 },
    { 0xb63f27df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbf27ef, SUC_lexicon_tags_4 },
    { 0x865f27f0, SUC_lexicon_tags_37 },
    { 0x13f27f1, SUC_lexicon_tags_28 },
    { 0x989f27f2, SUC_lexicon_tags_7 },
    { 0x4bdf27f2, SUC_lexicon_tags_17 },
    { 0xf57f27f4, SUC_lexicon_tags_15 },
    { 0x65f27f5, SUC_lexicon_tags_74 },
    { 0xe4df27f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813f27fc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89df2801, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xf63f2803, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ff2806, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207f280b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bf280f, SUC_lexicon_tags_27 },
    { 0xfdf2810, SUC_lexicon_tags_124 },
    { 0xe6bf2811, SUC_lexicon_tags_26 },
    { 0xdb7f2812, SUC_lexicon_tags_37 },
    { 0x16ff2813, SUC_lexicon_tags_33 },
    { 0xb37f2810, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc5ff2816, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x433f2819, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f281e, SUC_lexicon_tags_11 },
    { 0x48df281e, SUC_lexicon_tags_26 },
    { 0x321f281f, SUC_lexicon_tags_12 },
    { 0x11f2821, SUC_lexicon_tags_27 },
    { 0xf79f2822, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99f282a, SUC_lexicon_tags_10 },
    { 0x22ff282b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xde1f282e, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bf2832, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x465f2834, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993f2838, SUC_lexicon_tags_39 },
    { 0xb11f2838, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44bf2840, SUC_lexicon_tags_43 },
    { 0x375f2840, SUC_lexicon_tags_34 },
    { 0x845f2840, SUC_lexicon_tags_33 },
    { 0xac1f2843, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa17f2845, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481f284d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a7f284f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaff2852, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa87f2854, SUC_lexicon_tags_207 },
    { 0xd75f2855, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85f2858, SUC_lexicon_tags_12 },
    { 0xaa9f2858, SUC_lexicon_tags_25 },
    { 0xe11f2858, SUC_lexicon_tags_1 },
    { 0x579f2858, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdf285f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25f2865, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc79f2867, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f286a, SUC_lexicon_tags_23 },
    { 0xd2ff286b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5f3f286d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcc5f286f, SUC_lexicon_tags_29 },
    { 0x84bf2870, SUC_lexicon_tags_13 },
    { 0xef1f2871, SUC_lexicon_tags_2 },
    { 0x7e5f2871, SUC_lexicon_tags_82 },
    { 0x5c1f286f, SUC_lexicon_tags_34 },
    { 0xb73f2872, SUC_lexicon_tags_25 },
    { 0x36bf2875, SUC_lexicon_tags_32 },
    { 0xc73f2876, SUC_lexicon_tags_12 },
    { 0x249f2876, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7df287e, SUC_lexicon_tags_11 },
    { 0xa41f287f, SUC_lexicon_tags_37 },
    { 0xc7f2880, SUC_lexicon_tags_37 },
    { 0x91ff287f, SUC_lexicon_tags_37 },
    { 0xe9bf287f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x603f2885, SUC_lexicon_tags_56 },
    { 0x493f2886, SUC_lexicon_tags_84 },
    { 0x87df2887, SUC_lexicon_tags_11 },
    { 0x359f2888, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5f2891, SUC_lexicon_tags_15 },
    { 0xeeff2892, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9f2895, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7f289d, SUC_lexicon_tags_29 },
    { 0x8e1f289e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbf28a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ff28a7, SUC_lexicon_tags_478 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69f28b0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ff28b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abf28b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5f28b9, SUC_lexicon_tags_6 },
    { 0xfdf28ba, SUC_lexicon_tags_17 },
    { 0x6d1f28bb, SUC_lexicon_tags_65 },
    { 0xf39f28b9, SUC_lexicon_tags_29 },
    { 0x5d9f28bd, SUC_lexicon_tags_12 },
    { 0x94bf28b9, SUC_lexicon_tags_12 },
    { 0x33ff28bf, SUC_lexicon_tags_31 },
    { 0xbb9f28c0, SUC_lexicon_tags_7 },
    { 0xc5f28c0, SUC_lexicon_tags_37 },
    { 0xd77f28c2, SUC_lexicon_tags_1 },
    { 0x14df28c2, SUC_lexicon_tags_11 },
    { 0x817f28bc, SUC_lexicon_tags_62 },
    { 0x2f7f28c5, SUC_lexicon_tags_12 },
    { 0x77df28bf, SUC_lexicon_tags_47 },
    { 0x8cdf28c1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbf28cb, SUC_lexicon_tags_13 },
    { 0x4b9f28cc, SUC_lexicon_tags_26 },
    { 0x25f28cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x225f28cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfa1f28d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3f28d5, SUC_lexicon_tags_17 },
    { 0xfc1f28d6, SUC_lexicon_tags_16 },
    { 0xa79f28d7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xac1f28d9, SUC_lexicon_tags_85 },
    { 0x14ff28da, SUC_lexicon_tags_17 },
    { 0xdf5f28db, SUC_lexicon_tags_8 },
    { 0xefdf28dc, SUC_lexicon_tags_92 },
    { 0x24bf28da, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x401f28e0, SUC_lexicon_tags_13 },
    { 0xed3f28e1, SUC_lexicon_tags_15 },
    { 0x5fdf28e2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1f28e7, SUC_lexicon_tags_30 },
    { 0x8b5f28e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4df28ed, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86df28f4, SUC_lexicon_tags_45 },
    { 0x8b9f28f5, SUC_lexicon_tags_45 },
    { 0x849f28f6, SUC_lexicon_tags_8 },
    { 0x725f28f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff28fb, SUC_lexicon_tags_16 },
    { 0x52ff28fc, SUC_lexicon_tags_7 },
    { 0x8b7f28fd, SUC_lexicon_tags_32 },
    { 0x8c5f28fe, SUC_lexicon_tags_37 },
    { 0x9a9f28fb, SUC_lexicon_tags_17 },
    { 0x537f28ff, SUC_lexicon_tags_261 },
    { 0x3d7f2901, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bf2904, SUC_lexicon_tags_49 },
    { 0x17bf2905, SUC_lexicon_tags_38 },
    { 0xcfbf2906, SUC_lexicon_tags_30 },
    { 0xb19f2905, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681f290d, SUC_lexicon_tags_43 },
    { 0xcaff290e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x707f2910, SUC_lexicon_tags_15 },
    { 0x243f2911, SUC_lexicon_tags_25 },
    { 0xa55f2911, SUC_lexicon_tags_66 },
    { 0x527f2913, SUC_lexicon_tags_45 },
    { 0x67bf2910, SUC_lexicon_tags_7 },
    { 0x739f2915, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf9f291a, SUC_lexicon_tags_30 },
    { 0xf5f291b, SUC_lexicon_tags_25 },
    { 0xe17f291c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37f2923, SUC_lexicon_tags_14 },
    { 0xbc1f2924, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87f2929, SUC_lexicon_tags_19 },
    { 0x4d7f292a, SUC_lexicon_tags_17 },
    { 0x261f292b, SUC_lexicon_tags_7 },
    { 0xc9df292b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ff2932, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd69f2934, SUC_lexicon_tags_17 },
    { 0x211f2935, SUC_lexicon_tags_2 },
    { 0x65ff2935, SUC_lexicon_tags_25 },
    { 0x271f2937, SUC_lexicon_tags_60 },
    { 0x857f2938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc61f293a, SUC_lexicon_tags_1 },
    { 0x891f293b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1f2940, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b7f2942, SUC_lexicon_tags_7 },
    { 0xce1f2943, SUC_lexicon_tags_4 },
    { 0xcdbf2944, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfd5f2946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x767f2948, SUC_lexicon_tags_38 },
    { 0x53ff2949, SUC_lexicon_tags_27 },
    { 0x727f2949, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407f294f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57f2956, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xd15f2958, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3f295d, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x307f295f, SUC_lexicon_tags_38 },
    { 0x4f5f295f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc1bf2962, SUC_lexicon_tags_17 },
    { 0x337f2963, SUC_lexicon_tags_1 },
    { 0x631f2963, SUC_lexicon_tags_17 },
    { 0xcdf2965, SUC_lexicon_tags_1 },
    { 0x3ddf2966, SUC_lexicon_tags_17 },
    { 0xe83f2967, SUC_lexicon_tags_65 },
    { 0x301f2967, SUC_lexicon_tags_11 },
    { 0x6a1f2969, SUC_lexicon_tags_17 },
    { 0xabf296a, SUC_lexicon_tags_7 },
    { 0x4c9f2963, SUC_lexicon_tags_33 },
    { 0xcdbf296c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5cdf296e, SUC_lexicon_tags_1 },
    { 0xa5f296f, SUC_lexicon_tags_45 },
    { 0x95df2970, SUC_lexicon_tags_56 },
    { 0xd05f2971, SUC_lexicon_tags_43 },
    { 0x5d9f296f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11df2975, SUC_lexicon_tags_47 },
    { 0xae7f2976, SUC_lexicon_tags_14 },
    { 0x751f2976, SUC_lexicon_tags_37 },
    { 0x4e3f2976, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x41ff297b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x341f297d, SUC_lexicon_tags_7 },
    { 0x929f297d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x739f2980, SUC_lexicon_tags_17 },
    { 0x365f2981, SUC_lexicon_tags_12 },
    { 0xcfff2981, SUC_lexicon_tags_31 },
    { 0x7b9f2981, SUC_lexicon_tags_27 },
    { 0xd23f2984, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41f2989, SUC_lexicon_tags_15 },
    { 0xd29f298a, SUC_lexicon_tags_30 },
    { 0xd35f298b, SUC_lexicon_tags_49 },
    { 0x87f2989, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d3f298e, SUC_lexicon_tags_1 },
    { 0x41df298f, SUC_lexicon_tags_37 },
    { 0x3c3f2990, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x191f299b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xebbf299d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbd1f299f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x52bf29a1, SUC_lexicon_tags_7 },
    { 0xc3bf29a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3f29a5, SUC_lexicon_tags_7 },
    { 0xbebf29a6, SUC_lexicon_tags_15 },
    { 0x5f5f29a7, SUC_lexicon_tags_21 },
    { 0x175f29a7, SUC_lexicon_tags_6 },
    { 0x7a1f29a6, SUC_lexicon_tags_11 },
    { 0x7d9f29a5, SUC_lexicon_tags_17 },
    { 0xdb9f29a6, SUC_lexicon_tags_91 },
    { 0xb3f29a8, SUC_lexicon_tags_17 },
    { 0x69df29ad, SUC_lexicon_tags_41 },
    { 0x28df29ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639f29b2, SUC_lexicon_tags_2 },
    { 0xb25f29b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5f29b8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f29be, SUC_lexicon_tags_7 },
    { 0x8ddf29bf, SUC_lexicon_tags_11 },
    { 0xc81f29bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515f29c4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedf29cd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9f29d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ff29d7, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7f29db, SUC_lexicon_tags_1 },
    { 0x533f29dc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x645f29df, SUC_lexicon_tags_11 },
    { 0x85f29e0, SUC_lexicon_tags_25 },
    { 0x839f29e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x607f29e3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeadf29e5, SUC_lexicon_tags_45 },
    { 0x51f29e6, SUC_lexicon_tags_40 },
    { 0x64ff29e7, SUC_lexicon_tags_112 },
    { 0xbebf29e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309f29ed, SUC_lexicon_tags_1 },
    { 0x343f29ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x553f29f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x115f29f7, SUC_lexicon_tags_0 },
    { 0xcd3f29f7, SUC_lexicon_tags_27 },
    { 0xae9f29f9, SUC_lexicon_tags_0 },
    { 0x863f29f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9f29fd, SUC_lexicon_tags_23 },
    { 0x327f29fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf2a03, SUC_lexicon_tags_59 },
    { 0x63ff2a03, SUC_lexicon_tags_26 },
    { 0x3e9f2a05, SUC_lexicon_tags_23 },
    { 0x841f2a06, SUC_lexicon_tags_17 },
    { 0x535f2a07, SUC_lexicon_tags_87 },
    { 0x90ff2a08, SUC_lexicon_tags_25 },
    { 0xdd3f2a09, SUC_lexicon_tags_25 },
    { 0x53df2a09, SUC_lexicon_tags_7 },
    { 0xb95f2a05, SUC_lexicon_tags_15 },
    { 0xe7bf2a0c, SUC_lexicon_tags_15 },
    { 0x47df2a06, SUC_lexicon_tags_9 },
    { 0x5adf2a05, SUC_lexicon_tags_46 },
    { 0x13df2a0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddf2a18, SUC_lexicon_tags_12 },
    { 0xcd7f2a19, SUC_lexicon_tags_25 },
    { 0x34ff2a19, SUC_lexicon_tags_23 },
    { 0x843f2a1b, SUC_lexicon_tags_14 },
    { 0x4df2a18, SUC_lexicon_tags_12 },
    { 0x9fbf2a1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc85f2a1f, SUC_lexicon_tags_36 },
    { 0x79df2a20, SUC_lexicon_tags_37 },
    { 0x79bf2a21, SUC_lexicon_tags_87 },
    { 0xfcbf2a22, SUC_lexicon_tags_7 },
    { 0x1d1f2a20, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7b5f2a25, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x861f2a28, SUC_lexicon_tags_77 },
    { 0x1f7f2a29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe73f2a2b, SUC_lexicon_tags_17 },
    { 0x8b7f2a2c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30df2a35, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897f2a3c, SUC_lexicon_tags_9 },
    { 0xbff2a3d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x771f2a3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3f2a43, SUC_lexicon_tags_49 },
    { 0x341f2a43, SUC_lexicon_tags_72 },
    { 0x1fff2a43, SUC_lexicon_tags_9 },
    { 0xbfbf2a46, SUC_lexicon_tags_17 },
    { 0xa21f2a45, SUC_lexicon_tags_1 },
    { 0x889f2a48, SUC_lexicon_tags_87 },
    { 0x5dff2a49, SUC_lexicon_tags_33 },
    { 0x613f2a48, SUC_lexicon_tags_11 },
    { 0x43f2a4b, SUC_lexicon_tags_43 },
    { 0x241f2a4c, SUC_lexicon_tags_12 },
    { 0xd81f2a43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1df2a54, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33f2a58, SUC_lexicon_tags_23 },
    { 0xaff2a58, SUC_lexicon_tags_47 },
    { 0x11bf2a5a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1f2a5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x317f2a61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x247f2a63, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf2a6c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bf2a78, SUC_lexicon_tags_49 },
    { 0x8f9f2a78, SUC_lexicon_tags_45 },
    { 0xa03f2a79, SUC_lexicon_tags_37 },
    { 0xb19f2a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x289f2a7d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9df2a80, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6d1f2a82, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3f2a87, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x663f2a89, SUC_lexicon_tags_11 },
    { 0x21f2a8a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x637f2a8f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959f2a94, SUC_lexicon_tags_0 },
    { 0xe09f2a95, SUC_lexicon_tags_25 },
    { 0x107f2a96, SUC_lexicon_tags_16 },
    { 0xe7ff2a96, SUC_lexicon_tags_5 },
    { 0x1a7f2a97, SUC_lexicon_tags_25 },
    { 0xdddf2a94, SUC_lexicon_tags_46 },
    { 0xbf5f2a9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1d3f2a9c, SUC_lexicon_tags_19 },
    { 0xb59f2a9c, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65df2aa2, SUC_lexicon_tags_29 },
    { 0x521f2aa2, SUC_lexicon_tags_91 },
    { 0x67df2aa2, SUC_lexicon_tags_26 },
    { 0x5c9f2aa5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ff2ab0, SUC_lexicon_tags_15 },
    { 0x679f2ab1, SUC_lexicon_tags_7 },
    { 0xc7df2ab1, SUC_lexicon_tags_79 },
    { 0xf51f2ab3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebf2ab6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff2ac0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403f2ac6, SUC_lexicon_tags_37 },
    { 0x299f2ac7, SUC_lexicon_tags_15 },
    { 0x9e7f2ac7, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0x8bdf2aca, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x28df2acc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ff2ad3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xef3f2ad6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdff2ad9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343f2ae3, SUC_lexicon_tags_35 },
    { 0xbe9f2ae4, SUC_lexicon_tags_37 },
    { 0x159f2ae5, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x913f2aea, SUC_lexicon_tags_38 },
    { 0x741f2aeb, SUC_lexicon_tags_11 },
    { 0xc8bf2aeb, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1f2aef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x42df2af1, SUC_lexicon_tags_55 },
    { 0xf63f2af2, SUC_lexicon_tags_43 },
    { 0x97ff2af3, SUC_lexicon_tags_43 },
    { 0xc6ff2af4, SUC_lexicon_tags_25 },
    { 0x9bbf2af4, SUC_lexicon_tags_6 },
    { 0xd0df2af6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb9bf2af8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7f2afc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f2b06, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ff2b0b, SUC_lexicon_tags_2 },
    { 0x723f2b0c, SUC_lexicon_tags_7 },
    { 0xb31f2b0b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91f2b11, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xd13f2b13, SUC_lexicon_tags_33 },
    { 0x95bf2b14, SUC_lexicon_tags_9 },
    { 0xb55f2b15, SUC_lexicon_tags_23 },
    { 0x2d1f2b16, SUC_lexicon_tags_2 },
    { 0xb2df2b13, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xfc3f2b19, SUC_lexicon_tags_15 },
    { 0x1e7f2b19, SUC_lexicon_tags_13 },
    { 0x3cdf2b1b, SUC_lexicon_tags_60 },
    { 0x291f2b1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x263f2b1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbfff2b20, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0xdc1f2b22, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43f2b2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd97f2b2c, SUC_lexicon_tags_216 },
    { 0, NULL },
    { 0x135f2b2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0ff2b30, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7f2b34, SUC_lexicon_tags_17 },
    { 0x15bf2b35, SUC_lexicon_tags_15 },
    { 0xf2df2b35, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7f2b38, SUC_lexicon_tags_69 },
    { 0x89f2b39, SUC_lexicon_tags_17 },
    { 0x343f2b3a, SUC_lexicon_tags_7 },
    { 0x395f2b3b, SUC_lexicon_tags_11 },
    { 0xbc3f2b3c, SUC_lexicon_tags_1 },
    { 0xe65f2b3a, SUC_lexicon_tags_37 },
    { 0xbd3f2b3c, SUC_lexicon_tags_69 },
    { 0x5aff2b3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9bdf2b41, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x327f2b43, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65f2b46, SUC_lexicon_tags_25 },
    { 0x357f2b47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403f2b57, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdf2b5b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2a9f2b5d, SUC_lexicon_tags_47 },
    { 0x5e1f2b5e, SUC_lexicon_tags_12 },
    { 0x275f2b5f, SUC_lexicon_tags_17 },
    { 0xe19f2b60, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaabf2b62, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf2b66, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x105f2b68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f2b6e, SUC_lexicon_tags_17 },
    { 0x329f2b6f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x821f2b72, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63f2b77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x785f2b7a, SUC_lexicon_tags_30 },
    { 0xaa3f2b7b, SUC_lexicon_tags_7 },
    { 0x447f2b7c, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801f2b80, SUC_lexicon_tags_2 },
    { 0x23f2b81, SUC_lexicon_tags_31 },
    { 0xa9ff2b82, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5f2b85, SUC_lexicon_tags_97 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bf2b89, SUC_lexicon_tags_12 },
    { 0x47ff2b89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433f2b8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x997f2b91, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfadf2b93, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861f2b97, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x117f2b99, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79bf2b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ff2ba1, SUC_lexicon_tags_19 },
    { 0xaf5f2ba2, SUC_lexicon_tags_27 },
    { 0x313f2ba3, SUC_lexicon_tags_11 },
    { 0xbadf2ba4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xba7f2ba7, SUC_lexicon_tags_26 },
    { 0xf7f2ba8, SUC_lexicon_tags_43 },
    { 0x1c3f2ba9, SUC_lexicon_tags_7 },
    { 0xb21f2ba8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x297f2bac, SUC_lexicon_tags_6 },
    { 0x23f2bad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe9df2baf, SUC_lexicon_tags_8 },
    { 0xff7f2bb0, SUC_lexicon_tags_23 },
    { 0x4b9f2bb1, SUC_lexicon_tags_12 },
    { 0x949f2bb2, SUC_lexicon_tags_149 },
    { 0x787f2bb3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe7ff2bb5, SUC_lexicon_tags_55 },
    { 0xfb7f2bb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe61f2bb8, SUC_lexicon_tags_37 },
    { 0x545f2bb8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f5f2bc0, SUC_lexicon_tags_25 },
    { 0xea7f2bc1, SUC_lexicon_tags_19 },
    { 0xa75f2bc2, SUC_lexicon_tags_17 },
    { 0x165f2bc2, SUC_lexicon_tags_1 },
    { 0x811f2bc2, SUC_lexicon_tags_43 },
    { 0xad7f2bc5, SUC_lexicon_tags_47 },
    { 0x637f2bc6, SUC_lexicon_tags_49 },
    { 0x125f2bc2, SUC_lexicon_tags_25 },
    { 0x981f2bc5, SUC_lexicon_tags_27 },
    { 0xb59f2bc9, SUC_lexicon_tags_32 },
    { 0xe71f2bc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x463f2bcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ff2bd2, SUC_lexicon_tags_7 },
    { 0x115f2bd3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47ff2bd8, SUC_lexicon_tags_18 },
    { 0x9df2bd8, SUC_lexicon_tags_37 },
    { 0xd49f2bda, SUC_lexicon_tags_46 },
    { 0xa39f2bdb, SUC_lexicon_tags_19 },
    { 0x64ff2bda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac7f2be4, SUC_lexicon_tags_20 },
    { 0xc0ff2be4, SUC_lexicon_tags_2 },
    { 0xde5f2be6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13f2bf2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171f2bfa, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1eff2bfc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7f2c00, SUC_lexicon_tags_27 },
    { 0xc8bf2c01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f2c04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x773f2c06, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x38ff2c09, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ff2c0c, SUC_lexicon_tags_66 },
    { 0x79df2c0d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f2c11, SUC_lexicon_tags_33 },
    { 0xc0bf2c12, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2c9f2c14, SUC_lexicon_tags_27 },
    { 0x823f2c14, SUC_lexicon_tags_38 },
    { 0xf99f2c16, SUC_lexicon_tags_1 },
    { 0x883f2c15, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdadf2c19, SUC_lexicon_tags_17 },
    { 0x6c7f2c19, SUC_lexicon_tags_46 },
    { 0xa55f2c1b, SUC_lexicon_tags_6 },
    { 0x86ff2c1b, SUC_lexicon_tags_129 },
    { 0x585f2c1d, SUC_lexicon_tags_17 },
    { 0x971f2c1d, SUC_lexicon_tags_16 },
    { 0xd9df2c19, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77f2c24, SUC_lexicon_tags_11 },
    { 0xffff2c25, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221f2c29, SUC_lexicon_tags_1 },
    { 0x9d5f2c2a, SUC_lexicon_tags_11 },
    { 0x37ff2c2a, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xdf7f2c2d, SUC_lexicon_tags_11 },
    { 0x205f2c2e, SUC_lexicon_tags_33 },
    { 0x35df2c2f, SUC_lexicon_tags_37 },
    { 0x513f2c2d, SUC_lexicon_tags_21 },
    { 0x669f2c31, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f2c34, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3f2c39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03f2c3e, SUC_lexicon_tags_37 },
    { 0x5cdf2c3f, SUC_lexicon_tags_49 },
    { 0x325f2c3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507f2c45, SUC_lexicon_tags_38 },
    { 0xb31f2c45, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbf2c49, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9f2c4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1f2c5a, SUC_lexicon_tags_48 },
    { 0xc1df2c5b, SUC_lexicon_tags_15 },
    { 0xcb5f2c5c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb2df2c5e, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0x773f2c61, SUC_lexicon_tags_6 },
    { 0x931f2c62, SUC_lexicon_tags_38 },
    { 0xb0bf2c63, SUC_lexicon_tags_17 },
    { 0x3b1f2c64, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43f2c67, SUC_lexicon_tags_7 },
    { 0x215f2c68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73f2c6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed1f2c6e, SUC_lexicon_tags_2 },
    { 0xc1f2c6f, SUC_lexicon_tags_17 },
    { 0xea9f2c6f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97f2c73, SUC_lexicon_tags_17 },
    { 0x73df2c74, SUC_lexicon_tags_2 },
    { 0x371f2c75, SUC_lexicon_tags_15 },
    { 0x35bf2c76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535f2c7a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1f2c85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f2c88, SUC_lexicon_tags_29 },
    { 0x63df2c88, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0x3e1f2c8b, SUC_lexicon_tags_33 },
    { 0xcb5f2c8c, SUC_lexicon_tags_37 },
    { 0x165f2c8d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5f2c92, SUC_lexicon_tags_7 },
    { 0xd0bf2c92, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdbff2c95, SUC_lexicon_tags_30 },
    { 0xfbff2c95, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337f2c9a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df2c9d, SUC_lexicon_tags_17 },
    { 0xbdbf2c9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9f2ca1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0df2ca5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f2cad, SUC_lexicon_tags_37 },
    { 0x487f2cae, SUC_lexicon_tags_17 },
    { 0xad5f2caf, SUC_lexicon_tags_28 },
    { 0x66df2cae, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0x46ff2cb2, SUC_lexicon_tags_63 },
    { 0x4b9f2cb3, SUC_lexicon_tags_38 },
    { 0xaadf2cb4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3f2cb8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5bbf2cba, SUC_lexicon_tags_61 },
    { 0x4fff2cbb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa57f2cc2, SUC_lexicon_tags_8 },
    { 0xb1f2cc3, SUC_lexicon_tags_15 },
    { 0xf5f2cc4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf3ff2cc6, SUC_lexicon_tags_0 },
    { 0xf29f2cc7, SUC_lexicon_tags_234 },
    { 0xd01f2cc8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75f2ccb, SUC_lexicon_tags_0 },
    { 0x3a1f2ccb, SUC_lexicon_tags_23 },
    { 0xeedf2ccc, SUC_lexicon_tags_19 },
    { 0xd99f2cce, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f2cd3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x553f2cd6, SUC_lexicon_tags_27 },
    { 0xf57f2cd6, SUC_lexicon_tags_0 },
    { 0x82ff2cd7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd03f2cdb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7f2cde, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xedff2ce0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f2ce8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3f2cf3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x189f2cf5, SUC_lexicon_tags_43 },
    { 0xabf2cf6, SUC_lexicon_tags_23 },
    { 0xc7f2cf7, SUC_lexicon_tags_95 },
    { 0x709f2cf8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5f2cfb, SUC_lexicon_tags_21 },
    { 0xf9df2cfb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddf2cff, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xfff2d02, SUC_lexicon_tags_47 },
    { 0xbd9f2d03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7f2d06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3f2d0a, SUC_lexicon_tags_35 },
    { 0x49ff2d0a, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdf2d10, SUC_lexicon_tags_12 },
    { 0x4bbf2d11, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bf2d14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xcb7f2d16, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xbbf2d18, SUC_lexicon_tags_10 },
    { 0x7bbf2d18, SUC_lexicon_tags_2 },
    { 0x38df2d1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc9f2d1c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bf2d21, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x777f2d25, SUC_lexicon_tags_1 },
    { 0x815f2d26, SUC_lexicon_tags_2 },
    { 0x7f2d27, SUC_lexicon_tags_11 },
    { 0x5b7f2d26, SUC_lexicon_tags_25 },
    { 0x1bdf2d26, SUC_lexicon_tags_37 },
    { 0x57df2d2a, SUC_lexicon_tags_1 },
    { 0x7bf2d2b, SUC_lexicon_tags_9 },
    { 0x531f2d27, SUC_lexicon_tags_38 },
    { 0x631f2d2a, SUC_lexicon_tags_36 },
    { 0xe4df2d2e, SUC_lexicon_tags_49 },
    { 0x11f2d29, SUC_lexicon_tags_0 },
    { 0xcbdf2d25, SUC_lexicon_tags_43 },
    { 0x5f3f2d29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbb1f2d33, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879f2d37, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x959f2d3a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbf2d40, SUC_lexicon_tags_11 },
    { 0xbf3f2d40, SUC_lexicon_tags_45 },
    { 0xe53f2d42, SUC_lexicon_tags_11 },
    { 0xedbf2d43, SUC_lexicon_tags_2 },
    { 0x773f2d42, SUC_lexicon_tags_31 },
    { 0xc1bf2d41, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ff2d48, SUC_lexicon_tags_40 },
    { 0xcbf2d48, SUC_lexicon_tags_15 },
    { 0x4dbf2d4a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff2d4d, SUC_lexicon_tags_3 },
    { 0x8e5f2d4d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ff2d56, SUC_lexicon_tags_12 },
    { 0xc15f2d56, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75f2d5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89df2d60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bf2d69, SUC_lexicon_tags_11 },
    { 0x6c3f2d6a, SUC_lexicon_tags_12 },
    { 0xa53f2d6b, SUC_lexicon_tags_12 },
    { 0xa3ff2d6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187f2d70, SUC_lexicon_tags_17 },
    { 0xfabf2d71, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x6e9f2d73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x929f2d76, SUC_lexicon_tags_17 },
    { 0x5a3f2d76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8adf2d86, SUC_lexicon_tags_14 },
    { 0x341f2d86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5f2d8a, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x10ff2d8c, SUC_lexicon_tags_18 },
    { 0x3f3f2d8c, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0x475f2d8f, SUC_lexicon_tags_9 },
    { 0x3dff2d90, SUC_lexicon_tags_25 },
    { 0xecff2d91, SUC_lexicon_tags_17 },
    { 0xdc1f2d90, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ff2d97, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85f2d9b, SUC_lexicon_tags_14 },
    { 0xddf2d9c, SUC_lexicon_tags_1 },
    { 0xff3f2d9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45f2da5, SUC_lexicon_tags_14 },
    { 0xdcff2da6, SUC_lexicon_tags_6 },
    { 0x76df2da5, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bf2dad, SUC_lexicon_tags_39 },
    { 0x6df2dae, SUC_lexicon_tags_28 },
    { 0x44ff2daf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88df2db6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717f2dbb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bf2dc6, SUC_lexicon_tags_49 },
    { 0x177f2dc6, SUC_lexicon_tags_25 },
    { 0xe0df2dc6, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xda3f2dca, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9f2dce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1f2dd1, SUC_lexicon_tags_37 },
    { 0xe65f2dd1, SUC_lexicon_tags_150 },
    { 0, NULL },
    { 0x8d9f2dd4, SUC_lexicon_tags_99 },
    { 0xd99f2dd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd59f2dd7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7f2de0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x723f2de3, SUC_lexicon_tags_25 },
    { 0x407f2de4, SUC_lexicon_tags_8 },
    { 0x71df2de4, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbf2dea, SUC_lexicon_tags_48 },
    { 0x507f2deb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x697f2ded, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x553f2df0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf2df6, SUC_lexicon_tags_7 },
    { 0xc61f2df7, SUC_lexicon_tags_27 },
    { 0x559f2df8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77df2dfd, SUC_lexicon_tags_44 },
    { 0xcb7f2dfe, SUC_lexicon_tags_14 },
    { 0x879f2dfe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455f2e08, SUC_lexicon_tags_18 },
    { 0xe1f2e08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111f2e0d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95f2e11, SUC_lexicon_tags_11 },
    { 0xc25f2e11, SUC_lexicon_tags_7 },
    { 0x53bf2e13, SUC_lexicon_tags_37 },
    { 0xe0df2e13, SUC_lexicon_tags_27 },
    { 0x1f5f2e15, SUC_lexicon_tags_7 },
    { 0xf5bf2e16, SUC_lexicon_tags_19 },
    { 0xf8df2e17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2bbf2e19, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61ff2e1d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc63f2e1f, SUC_lexicon_tags_18 },
    { 0xa9bf2e20, SUC_lexicon_tags_47 },
    { 0x409f2e21, SUC_lexicon_tags_27 },
    { 0x363f2e20, SUC_lexicon_tags_15 },
    { 0x70bf2e21, SUC_lexicon_tags_17 },
    { 0x6d1f2e24, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9df2e27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7f2e2b, SUC_lexicon_tags_2 },
    { 0x4d5f2e2b, SUC_lexicon_tags_44 },
    { 0x90ff2e2b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x359f2e2f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167f2e34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221f2e39, SUC_lexicon_tags_47 },
    { 0x9df2e3a, SUC_lexicon_tags_1 },
    { 0x9f3f2e3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc1f2e3d, SUC_lexicon_tags_13 },
    { 0xfb9f2e3d, SUC_lexicon_tags_27 },
    { 0xa3df2e3e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xfbff2e41, SUC_lexicon_tags_123 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7f2e4b, SUC_lexicon_tags_1 },
    { 0x2cdf2e4c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57df2e50, SUC_lexicon_tags_23 },
    { 0xaa1f2e50, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x867f2e54, SUC_lexicon_tags_25 },
    { 0x3f5f2e54, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbf2e5a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3f2e60, SUC_lexicon_tags_84 },
    { 0xad3f2e61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x741f2e64, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xb17f2e66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50df2e6a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x609f2e6d, SUC_lexicon_tags_34 },
    { 0x57df2e6e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x789f2e70, SUC_lexicon_tags_29 },
    { 0xecdf2e71, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53df2e78, SUC_lexicon_tags_17 },
    { 0x817f2e79, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x837f2e7c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991f2e80, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11f2e84, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0df2e8b, SUC_lexicon_tags_37 },
    { 0x64ff2e8c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ff2ea0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35f2ea3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7bf2ea5, SUC_lexicon_tags_23 },
    { 0xc75f2ea6, SUC_lexicon_tags_21 },
    { 0xc1ff2ea5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe85f2eaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49f2eb3, SUC_lexicon_tags_17 },
    { 0x541f2eb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0df2eb7, SUC_lexicon_tags_0 },
    { 0x16bf2eb8, SUC_lexicon_tags_33 },
    { 0x10ff2eb9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929f2ebd, SUC_lexicon_tags_7 },
    { 0x5e3f2ebe, SUC_lexicon_tags_2 },
    { 0xd63f2ebf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7f2ec4, SUC_lexicon_tags_46 },
    { 0x3b1f2ec4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xd85f2ec7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dff2ecb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab5f2ecd, SUC_lexicon_tags_8 },
    { 0x531f2ece, SUC_lexicon_tags_19 },
    { 0x5a1f2ecf, SUC_lexicon_tags_47 },
    { 0x7f3f2ece, SUC_lexicon_tags_2 },
    { 0x555f2ecf, SUC_lexicon_tags_16 },
    { 0x35f2ed0, SUC_lexicon_tags_66 },
    { 0x5b5f2ecd, SUC_lexicon_tags_25 },
    { 0xd95f2ed0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3bf2ed6, SUC_lexicon_tags_145 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9f2edc, SUC_lexicon_tags_29 },
    { 0xd07f2edd, SUC_lexicon_tags_11 },
    { 0x58bf2edc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xefff2ee0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1f2ee5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3f2ef0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9f2ef3, SUC_lexicon_tags_23 },
    { 0xa8bf2ef3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x657f2ef6, SUC_lexicon_tags_17 },
    { 0x363f2ef7, SUC_lexicon_tags_17 },
    { 0xa13f2ef8, SUC_lexicon_tags_17 },
    { 0x339f2ef9, SUC_lexicon_tags_14 },
    { 0x87df2ef7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ff2f00, SUC_lexicon_tags_11 },
    { 0x155f2f00, SUC_lexicon_tags_9 },
    { 0x93df2f02, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ff2f07, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9f2f0b, SUC_lexicon_tags_26 },
    { 0x1e9f2f0b, SUC_lexicon_tags_43 },
    { 0x1bdf2f0d, SUC_lexicon_tags_2 },
    { 0xad5f2f0e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7f2f18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x63f2f1a, SUC_lexicon_tags_12 },
    { 0x8e9f2f1b, SUC_lexicon_tags_37 },
    { 0xfb9f2f1b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x66df2f1f, SUC_lexicon_tags_60 },
    { 0x3fff2f20, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x527f2f22, SUC_lexicon_tags_17 },
    { 0xb29f2f23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5f2f27, SUC_lexicon_tags_17 },
    { 0x395f2f28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x387f2f2a, SUC_lexicon_tags_11 },
    { 0x83f2f2b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5f2f2f, SUC_lexicon_tags_0 },
    { 0xbd9f2f2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbff2f36, SUC_lexicon_tags_12 },
    { 0xe91f2f36, SUC_lexicon_tags_2 },
    { 0xd37f2f37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7f2f3b, SUC_lexicon_tags_25 },
    { 0x655f2f3c, SUC_lexicon_tags_47 },
    { 0x10df2f3d, SUC_lexicon_tags_11 },
    { 0x16bf2f3e, SUC_lexicon_tags_7 },
    { 0x169f2f3e, SUC_lexicon_tags_30 },
    { 0xcabf2f3d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdf2f47, SUC_lexicon_tags_1 },
    { 0x36bf2f47, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdf1f2f4a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03f2f4e, SUC_lexicon_tags_21 },
    { 0xf4df2f4f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3edf2f51, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x24bf2f53, SUC_lexicon_tags_29 },
    { 0xbfff2f54, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195f2f59, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7df2f5c, SUC_lexicon_tags_37 },
    { 0x505f2f5d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5f2f60, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x353f2f62, SUC_lexicon_tags_37 },
    { 0x463f2f63, SUC_lexicon_tags_11 },
    { 0x5eff2f64, SUC_lexicon_tags_8 },
    { 0xbd7f2f65, SUC_lexicon_tags_8 },
    { 0x71f2f65, SUC_lexicon_tags_25 },
    { 0x76df2f67, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63df2f6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43df2f73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311f2f77, SUC_lexicon_tags_47 },
    { 0x55ff2f78, SUC_lexicon_tags_25 },
    { 0xf19f2f77, SUC_lexicon_tags_42 },
    { 0x5a3f2f77, SUC_lexicon_tags_2 },
    { 0xafbf2f7b, SUC_lexicon_tags_18 },
    { 0x323f2f7a, SUC_lexicon_tags_61 },
    { 0x85bf2f7d, SUC_lexicon_tags_1 },
    { 0x6edf2f7d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ff2f8e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xaa7f2f90, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f2f94, SUC_lexicon_tags_1 },
    { 0x16bf2f94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x617f2f97, SUC_lexicon_tags_63 },
    { 0xe37f2f97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdbff2f9a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xac7f2f9c, SUC_lexicon_tags_27 },
    { 0x2ebf2f9d, SUC_lexicon_tags_17 },
    { 0x5e5f2f9e, SUC_lexicon_tags_37 },
    { 0xeebf2f9d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f2fa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adf2fa6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55df2fb4, SUC_lexicon_tags_64 },
    { 0x2d1f2fb5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x95f2fb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cbf2fb9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x197f2fbc, SUC_lexicon_tags_1 },
    { 0x56bf2fbd, SUC_lexicon_tags_23 },
    { 0xc61f2fbd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc13f2fc0, SUC_lexicon_tags_1 },
    { 0xadbf2fc1, SUC_lexicon_tags_106 },
    { 0xa95f2fc2, SUC_lexicon_tags_37 },
    { 0xfc9f2fc3, SUC_lexicon_tags_33 },
    { 0xa49f2fc2, SUC_lexicon_tags_15 },
    { 0x18ff2fc2, SUC_lexicon_tags_17 },
    { 0x6e9f2fc4, SUC_lexicon_tags_45 },
    { 0x21df2fc7, SUC_lexicon_tags_4 },
    { 0x245f2fc8, SUC_lexicon_tags_3 },
    { 0x261f2fc2, SUC_lexicon_tags_7 },
    { 0x19ff2fc8, SUC_lexicon_tags_81 },
    { 0x8f1f2fc2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x391f2fcd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229f2fd3, SUC_lexicon_tags_11 },
    { 0xb59f2fd3, SUC_lexicon_tags_14 },
    { 0xec1f2fd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x703f2fd7, SUC_lexicon_tags_17 },
    { 0x461f2fd8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x52df2fda, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bf2fdd, SUC_lexicon_tags_17 },
    { 0xb6ff2fdd, SUC_lexicon_tags_2 },
    { 0x607f2fde, SUC_lexicon_tags_17 },
    { 0xeadf2fe0, SUC_lexicon_tags_49 },
    { 0x973f2fdf, SUC_lexicon_tags_34 },
    { 0xd0df2fe2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6df2fe4, SUC_lexicon_tags_65 },
    { 0x1a9f2fe5, SUC_lexicon_tags_38 },
    { 0x827f2fe5, SUC_lexicon_tags_12 },
    { 0xe13f2fe5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x18bf2fe9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f2fef, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421f2ff5, SUC_lexicon_tags_2 },
    { 0xbe7f2ff5, SUC_lexicon_tags_83 },
    { 0x6a3f2ff7, SUC_lexicon_tags_38 },
    { 0x5bf2ff5, SUC_lexicon_tags_32 },
    { 0x89f2ff5, SUC_lexicon_tags_17 },
    { 0xca7f2ff5, SUC_lexicon_tags_32 },
    { 0x67bf2ffb, SUC_lexicon_tags_43 },
    { 0xcb7f2ffc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b5f2ffe, SUC_lexicon_tags_37 },
    { 0x99ff2ffe, SUC_lexicon_tags_15 },
    { 0x625f3000, SUC_lexicon_tags_45 },
    { 0xd53f2ffe, SUC_lexicon_tags_0 },
    { 0x72ff3000, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xbe5f3004, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x79df3006, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1f300b, SUC_lexicon_tags_23 },
    { 0x9abf300c, SUC_lexicon_tags_8 },
    { 0xe0ff300d, SUC_lexicon_tags_25 },
    { 0xe4df300d, SUC_lexicon_tags_11 },
    { 0x3c5f300e, SUC_lexicon_tags_6 },
    { 0x72ff300e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabf3013, SUC_lexicon_tags_47 },
    { 0x8d9f3014, SUC_lexicon_tags_27 },
    { 0x937f3013, SUC_lexicon_tags_252 },
    { 0x2d5f3015, SUC_lexicon_tags_31 },
    { 0xd6ff3017, SUC_lexicon_tags_29 },
    { 0xa5ff3017, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3f301b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9f3024, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3f3027, SUC_lexicon_tags_11 },
    { 0xff5f3028, SUC_lexicon_tags_9 },
    { 0x425f3028, SUC_lexicon_tags_11 },
    { 0x27ff3028, SUC_lexicon_tags_37 },
    { 0x8f9f302b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xafdf302d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x209f3030, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4df3033, SUC_lexicon_tags_37 },
    { 0xcd7f3034, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55f3037, SUC_lexicon_tags_57 },
    { 0x761f3037, SUC_lexicon_tags_19 },
    { 0x591f3039, SUC_lexicon_tags_37 },
    { 0x661f303a, SUC_lexicon_tags_17 },
    { 0x3bbf3038, SUC_lexicon_tags_45 },
    { 0xd6ff303b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819f3041, SUC_lexicon_tags_17 },
    { 0x909f3042, SUC_lexicon_tags_11 },
    { 0x645f3041, SUC_lexicon_tags_46 },
    { 0xce5f3042, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd7f3046, SUC_lexicon_tags_17 },
    { 0xeaff3047, SUC_lexicon_tags_23 },
    { 0x6a5f3048, SUC_lexicon_tags_1 },
    { 0xdc7f3048, SUC_lexicon_tags_47 },
    { 0xa9bf3049, SUC_lexicon_tags_17 },
    { 0x741f304b, SUC_lexicon_tags_17 },
    { 0x6fdf304a, SUC_lexicon_tags_64 },
    { 0xde7f304d, SUC_lexicon_tags_14 },
    { 0x789f304e, SUC_lexicon_tags_25 },
    { 0xaabf304f, SUC_lexicon_tags_33 },
    { 0xb2df3050, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973f3055, SUC_lexicon_tags_6 },
    { 0xf53f3056, SUC_lexicon_tags_15 },
    { 0xbdf3057, SUC_lexicon_tags_11 },
    { 0x25df3057, SUC_lexicon_tags_26 },
    { 0x7e3f3059, SUC_lexicon_tags_17 },
    { 0x46df305a, SUC_lexicon_tags_38 },
    { 0x233f305b, SUC_lexicon_tags_0 },
    { 0x94ff305c, SUC_lexicon_tags_12 },
    { 0x26df305d, SUC_lexicon_tags_27 },
    { 0x1fdf305e, SUC_lexicon_tags_1 },
    { 0xb55f3059, SUC_lexicon_tags_113 },
    { 0x3b3f305c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d9f3062, SUC_lexicon_tags_43 },
    { 0xd5bf3062, SUC_lexicon_tags_29 },
    { 0x38bf3062, SUC_lexicon_tags_38 },
    { 0xd49f3064, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9eff3067, SUC_lexicon_tags_8 },
    { 0x7b3f3068, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1f306f, SUC_lexicon_tags_10 },
    { 0xc9f306f, SUC_lexicon_tags_0 },
    { 0x6b7f3071, SUC_lexicon_tags_77 },
    { 0x94df3072, SUC_lexicon_tags_25 },
    { 0x62df3073, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x30bf3075, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ff3079, SUC_lexicon_tags_37 },
    { 0x899f3079, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc17f307c, SUC_lexicon_tags_29 },
    { 0xafdf307c, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bf3084, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88bf308a, SUC_lexicon_tags_15 },
    { 0xb09f308b, SUC_lexicon_tags_37 },
    { 0xcaff308c, SUC_lexicon_tags_38 },
    { 0xc9df308d, SUC_lexicon_tags_17 },
    { 0x1e3f308a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x1e5f3090, SUC_lexicon_tags_15 },
    { 0x945f3090, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67f3094, SUC_lexicon_tags_14 },
    { 0x39bf3095, SUC_lexicon_tags_15 },
    { 0x3ddf3096, SUC_lexicon_tags_15 },
    { 0xaf5f3097, SUC_lexicon_tags_47 },
    { 0x2f5f3098, SUC_lexicon_tags_14 },
    { 0x389f3097, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x15bf309b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ff30a1, SUC_lexicon_tags_17 },
    { 0xef1f30a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85f30a8, SUC_lexicon_tags_56 },
    { 0x7a9f30a8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3f30ae, SUC_lexicon_tags_14 },
    { 0xbff30af, SUC_lexicon_tags_32 },
    { 0xdff30b0, SUC_lexicon_tags_1 },
    { 0xb23f30b1, SUC_lexicon_tags_4 },
    { 0xe23f30b1, SUC_lexicon_tags_37 },
    { 0x71ff30b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x511f30b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307f30ba, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57df30c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19f30c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9f30d0, SUC_lexicon_tags_45 },
    { 0x83f30d1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x5bff30d3, SUC_lexicon_tags_11 },
    { 0x3cdf30d4, SUC_lexicon_tags_0 },
    { 0x437f30d4, SUC_lexicon_tags_17 },
    { 0x6bff30d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29f30d9, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1f30dc, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xebf30de, SUC_lexicon_tags_7 },
    { 0x731f30df, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd1bf30e1, SUC_lexicon_tags_1 },
    { 0xd65f30e2, SUC_lexicon_tags_82 },
    { 0xd21f30e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edf30eb, SUC_lexicon_tags_12 },
    { 0x495f30ec, SUC_lexicon_tags_30 },
    { 0xe7df30eb, SUC_lexicon_tags_11 },
    { 0x52df30ee, SUC_lexicon_tags_10 },
    { 0xa5ff30ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x71bf30f1, SUC_lexicon_tags_10 },
    { 0x66bf30f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9fbf30f4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xebdf30f7, SUC_lexicon_tags_2 },
    { 0xeabf30f8, SUC_lexicon_tags_14 },
    { 0x179f30f9, SUC_lexicon_tags_12 },
    { 0x7abf30fa, SUC_lexicon_tags_23 },
    { 0x92ff30f8, SUC_lexicon_tags_16 },
    { 0x19ff30fc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe79f30ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x901f3101, SUC_lexicon_tags_1 },
    { 0x5e9f3102, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf3106, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdb5f3108, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec3f310d, SUC_lexicon_tags_46 },
    { 0x439f310d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35f3112, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ff3115, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01f3120, SUC_lexicon_tags_37 },
    { 0xb1df3121, SUC_lexicon_tags_11 },
    { 0xb7f3122, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9f3128, SUC_lexicon_tags_12 },
    { 0xaebf3129, SUC_lexicon_tags_15 },
    { 0xb1bf3128, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51f312f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf07f3131, SUC_lexicon_tags_7 },
    { 0x98df3132, SUC_lexicon_tags_17 },
    { 0xbb1f3133, SUC_lexicon_tags_11 },
    { 0x961f3134, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295f313a, SUC_lexicon_tags_0 },
    { 0x4f1f313b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9eff3141, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x887f3143, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247f314a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13df3150, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x697f3152, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5f3154, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a7f3156, SUC_lexicon_tags_31 },
    { 0xccdf3157, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751f315b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5f315e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241f3163, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3f3166, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x853f3168, SUC_lexicon_tags_82 },
    { 0xf3f3169, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddf316e, SUC_lexicon_tags_7 },
    { 0x721f316e, SUC_lexicon_tags_35 },
    { 0x14df3170, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd47f3172, SUC_lexicon_tags_82 },
    { 0xf25f3173, SUC_lexicon_tags_82 },
    { 0x60ff3174, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8b9f3176, SUC_lexicon_tags_33 },
    { 0xb1df3177, SUC_lexicon_tags_11 },
    { 0xf3ff3178, SUC_lexicon_tags_7 },
    { 0x113f3178, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9c1f317b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcc3f317d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x325f3180, SUC_lexicon_tags_477 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3f3183, SUC_lexicon_tags_37 },
    { 0x795f3184, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xad9f3186, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ff318b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fff3194, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909f3198, SUC_lexicon_tags_47 },
    { 0xb93f3198, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x663f319b, SUC_lexicon_tags_60 },
    { 0x30df319c, SUC_lexicon_tags_15 },
    { 0x2b5f319c, SUC_lexicon_tags_12 },
    { 0x417f319d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97f31a6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3f31a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x7df31ab, SUC_lexicon_tags_25 },
    { 0x3abf31ac, SUC_lexicon_tags_37 },
    { 0xb33f31ac, SUC_lexicon_tags_48 },
    { 0x319f31ae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bf31b3, SUC_lexicon_tags_12 },
    { 0x639f31b4, SUC_lexicon_tags_7 },
    { 0x653f31b3, SUC_lexicon_tags_15 },
    { 0xf5ff31b6, SUC_lexicon_tags_60 },
    { 0x7bf31b4, SUC_lexicon_tags_21 },
    { 0xa55f31b4, SUC_lexicon_tags_28 },
    { 0x7b3f31b3, SUC_lexicon_tags_45 },
    { 0xf4df31ba, SUC_lexicon_tags_11 },
    { 0x8bbf31b9, SUC_lexicon_tags_37 },
    { 0x1aff31b8, SUC_lexicon_tags_68 },
    { 0xe1ff31ba, SUC_lexicon_tags_37 },
    { 0x981f31be, SUC_lexicon_tags_34 },
    { 0xc7ff31bf, SUC_lexicon_tags_9 },
    { 0xa9f31c0, SUC_lexicon_tags_26 },
    { 0xe01f31c1, SUC_lexicon_tags_48 },
    { 0x487f31c1, SUC_lexicon_tags_7 },
    { 0x1d3f31c3, SUC_lexicon_tags_37 },
    { 0xbff31c4, SUC_lexicon_tags_65 },
    { 0x6bf31c4, SUC_lexicon_tags_1 },
    { 0x3df31c6, SUC_lexicon_tags_12 },
    { 0x8d7f31c6, SUC_lexicon_tags_2 },
    { 0xd1ff31c8, SUC_lexicon_tags_28 },
    { 0x515f31bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xef5f31cb, SUC_lexicon_tags_37 },
    { 0xc27f31cb, SUC_lexicon_tags_25 },
    { 0x3bdf31cd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bf31d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bf31d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df31db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf65f31dd, SUC_lexicon_tags_17 },
    { 0xe2bf31de, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa91f31e2, SUC_lexicon_tags_91 },
    { 0xf01f31e3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215f31e7, SUC_lexicon_tags_17 },
    { 0x8d5f31e7, SUC_lexicon_tags_80 },
    { 0xf1f31e9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfd9f31eb, SUC_lexicon_tags_2 },
    { 0x45f31ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ff31ef, SUC_lexicon_tags_25 },
    { 0xd6bf31f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x965f31f2, SUC_lexicon_tags_30 },
    { 0x64bf31f3, SUC_lexicon_tags_37 },
    { 0xe63f31f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xec3f31f6, SUC_lexicon_tags_45 },
    { 0xa05f31f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7df31fe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc01f3201, SUC_lexicon_tags_37 },
    { 0xb1bf3202, SUC_lexicon_tags_23 },
    { 0x4fbf3202, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ff3206, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f320a, SUC_lexicon_tags_37 },
    { 0xc2df320b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd07f320d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bf3212, SUC_lexicon_tags_34 },
    { 0xa9bf3213, SUC_lexicon_tags_12 },
    { 0xfbbf3213, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ff321a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5b1f321c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22df3220, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x46bf3222, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567f3228, SUC_lexicon_tags_2 },
    { 0xce1f3229, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3b3f322b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x72ff322d, SUC_lexicon_tags_37 },
    { 0xbabf322e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1d7f3230, SUC_lexicon_tags_25 },
    { 0x4a9f3231, SUC_lexicon_tags_45 },
    { 0x913f3232, SUC_lexicon_tags_19 },
    { 0x533f3232, SUC_lexicon_tags_37 },
    { 0xa95f3233, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x269f3236, SUC_lexicon_tags_23 },
    { 0x3abf3236, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddf323a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fff323d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5f324a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa79f324c, SUC_lexicon_tags_39 },
    { 0x74df324c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abf3250, SUC_lexicon_tags_66 },
    { 0xaddf3250, SUC_lexicon_tags_12 },
    { 0xc95f3252, SUC_lexicon_tags_30 },
    { 0xfa3f3252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6df325b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ff325d, SUC_lexicon_tags_177 },
    { 0x3b9f325d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407f3263, SUC_lexicon_tags_11 },
    { 0x7b3f3263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37f3280, SUC_lexicon_tags_31 },
    { 0xdeff3280, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7d7f3283, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adf3286, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9f3289, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687f3292, SUC_lexicon_tags_84 },
    { 0x531f3292, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd95f3295, SUC_lexicon_tags_25 },
    { 0x8dbf3296, SUC_lexicon_tags_17 },
    { 0x687f3296, SUC_lexicon_tags_174 },
    { 0xa81f3296, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x24df329a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bff329d, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07f32a0, SUC_lexicon_tags_49 },
    { 0x399f32a1, SUC_lexicon_tags_9 },
    { 0xa6df32a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xef5f32a4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63f32a8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe8ff32aa, SUC_lexicon_tags_2 },
    { 0x629f32ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53f32ae, SUC_lexicon_tags_7 },
    { 0xcd1f32af, SUC_lexicon_tags_65 },
    { 0x83f32b0, SUC_lexicon_tags_49 },
    { 0x485f32b1, SUC_lexicon_tags_17 },
    { 0x3bff32b2, SUC_lexicon_tags_17 },
    { 0x7bff32b2, SUC_lexicon_tags_47 },
    { 0x55ff32b4, SUC_lexicon_tags_7 },
    { 0x7f3f32b3, SUC_lexicon_tags_65 },
    { 0x47f32b6, SUC_lexicon_tags_12 },
    { 0xb3bf32b4, SUC_lexicon_tags_11 },
    { 0x7f1f32b0, SUC_lexicon_tags_29 },
    { 0x751f32b9, SUC_lexicon_tags_26 },
    { 0x9cff32ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x341f32bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537f32c4, SUC_lexicon_tags_55 },
    { 0x95f32c5, SUC_lexicon_tags_25 },
    { 0x4cbf32c6, SUC_lexicon_tags_15 },
    { 0xb9df32c7, SUC_lexicon_tags_43 },
    { 0x43df32c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525f32ce, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x52ff32d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3e5f32d2, SUC_lexicon_tags_1 },
    { 0xbc9f32d2, SUC_lexicon_tags_37 },
    { 0xeb7f32d4, SUC_lexicon_tags_11 },
    { 0xe15f32d4, SUC_lexicon_tags_57 },
    { 0xfbdf32d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129f32dd, SUC_lexicon_tags_2 },
    { 0x755f32dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bf32e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaff32e7, SUC_lexicon_tags_7 },
    { 0x165f32e8, SUC_lexicon_tags_15 },
    { 0x385f32e9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ff32f0, SUC_lexicon_tags_55 },
    { 0xcbdf32f1, SUC_lexicon_tags_2 },
    { 0x687f32f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x77df32f4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bf32f9, SUC_lexicon_tags_10 },
    { 0x9bbf32fa, SUC_lexicon_tags_17 },
    { 0xe37f32fa, SUC_lexicon_tags_1 },
    { 0x715f32fb, SUC_lexicon_tags_178 },
    { 0x3f32fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdf3301, SUC_lexicon_tags_91 },
    { 0x817f3302, SUC_lexicon_tags_12 },
    { 0xdeff3303, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaff3307, SUC_lexicon_tags_47 },
    { 0x4a9f3308, SUC_lexicon_tags_3 },
    { 0xd57f3308, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf49f330b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3f3312, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449f3320, SUC_lexicon_tags_77 },
    { 0x9c7f3321, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865f3325, SUC_lexicon_tags_65 },
    { 0xa31f3325, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617f332b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61ff332d, SUC_lexicon_tags_45 },
    { 0x915f332e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x80df3331, SUC_lexicon_tags_17 },
    { 0x1cdf3331, SUC_lexicon_tags_6 },
    { 0x76bf3333, SUC_lexicon_tags_63 },
    { 0xdd5f3334, SUC_lexicon_tags_7 },
    { 0xff3f3335, SUC_lexicon_tags_47 },
    { 0xcc1f3336, SUC_lexicon_tags_11 },
    { 0xedbf3336, SUC_lexicon_tags_17 },
    { 0x109f3338, SUC_lexicon_tags_37 },
    { 0xde9f3339, SUC_lexicon_tags_12 },
    { 0x193f3336, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x47f333c, SUC_lexicon_tags_37 },
    { 0x8e3f333c, SUC_lexicon_tags_14 },
    { 0x657f333c, SUC_lexicon_tags_15 },
    { 0xeebf333c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x5edf3342, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1f3345, SUC_lexicon_tags_17 },
    { 0x301f3346, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bf3349, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc23f334b, SUC_lexicon_tags_17 },
    { 0xdc1f334c, SUC_lexicon_tags_12 },
    { 0x165f334d, SUC_lexicon_tags_23 },
    { 0x7eff334e, SUC_lexicon_tags_11 },
    { 0x789f334f, SUC_lexicon_tags_37 },
    { 0x769f334f, SUC_lexicon_tags_17 },
    { 0x7cff3350, SUC_lexicon_tags_30 },
    { 0x54bf334d, SUC_lexicon_tags_23 },
    { 0xcddf3353, SUC_lexicon_tags_28 },
    { 0x889f3353, SUC_lexicon_tags_17 },
    { 0xdb9f3355, SUC_lexicon_tags_47 },
    { 0x7a5f334d, SUC_lexicon_tags_1 },
    { 0x8d1f3357, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43f335a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x963f335c, SUC_lexicon_tags_26 },
    { 0x5e1f335c, SUC_lexicon_tags_49 },
    { 0x82bf335d, SUC_lexicon_tags_2 },
    { 0x79f335f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95f3362, SUC_lexicon_tags_13 },
    { 0x551f3363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21f3369, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01f336e, SUC_lexicon_tags_17 },
    { 0xa21f336f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x193f3372, SUC_lexicon_tags_1 },
    { 0x7fff3373, SUC_lexicon_tags_65 },
    { 0x38bf3374, SUC_lexicon_tags_11 },
    { 0xf85f3374, SUC_lexicon_tags_1 },
    { 0x155f3373, SUC_lexicon_tags_2 },
    { 0x6fbf3377, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09f337b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abf338d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517f3393, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa73f3395, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x3d9f3397, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1f339b, SUC_lexicon_tags_83 },
    { 0xb51f339c, SUC_lexicon_tags_11 },
    { 0xd53f339c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14df33a1, SUC_lexicon_tags_2 },
    { 0x81f33a2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x135f33a4, SUC_lexicon_tags_1 },
    { 0xab5f33a4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbf33aa, SUC_lexicon_tags_1 },
    { 0x419f33aa, SUC_lexicon_tags_66 },
    { 0x319f33ab, SUC_lexicon_tags_8 },
    { 0x3bbf33aa, SUC_lexicon_tags_2 },
    { 0xc57f33aa, SUC_lexicon_tags_49 },
    { 0x149f33af, SUC_lexicon_tags_59 },
    { 0x2dbf33af, SUC_lexicon_tags_1 },
    { 0x667f33ae, SUC_lexicon_tags_48 },
    { 0x311f33ae, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbff33bd, SUC_lexicon_tags_60 },
    { 0x3f5f33be, SUC_lexicon_tags_30 },
    { 0xba9f33bf, SUC_lexicon_tags_12 },
    { 0x83ff33bd, SUC_lexicon_tags_77 },
    { 0x84ff33bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcbf33c4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddf33c7, SUC_lexicon_tags_45 },
    { 0xe13f33c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685f33cd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xde1f33cf, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadf33d4, SUC_lexicon_tags_2 },
    { 0x1ebf33d5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x265f33d7, SUC_lexicon_tags_38 },
    { 0x47bf33d7, SUC_lexicon_tags_37 },
    { 0xc21f33d8, SUC_lexicon_tags_1 },
    { 0x4bdf33da, SUC_lexicon_tags_46 },
    { 0x37f33d8, SUC_lexicon_tags_11 },
    { 0x137f33db, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xb2ff33de, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x493f33e1, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831f33e5, SUC_lexicon_tags_37 },
    { 0xf61f33e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5f33e9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceff33ef, SUC_lexicon_tags_12 },
    { 0xb23f33f0, SUC_lexicon_tags_1 },
    { 0x881f33ef, SUC_lexicon_tags_3 },
    { 0xfc5f33f2, SUC_lexicon_tags_39 },
    { 0x547f33f3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669f33f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbf33fa, SUC_lexicon_tags_47 },
    { 0x3a3f33fb, SUC_lexicon_tags_25 },
    { 0x743f33fb, SUC_lexicon_tags_17 },
    { 0x58ff33fc, SUC_lexicon_tags_19 },
    { 0x6fdf33fe, SUC_lexicon_tags_15 },
    { 0x125f33fc, SUC_lexicon_tags_31 },
    { 0x9f1f33fe, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15f3404, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe27f3406, SUC_lexicon_tags_25 },
    { 0x7aff3406, SUC_lexicon_tags_15 },
    { 0xb27f3406, SUC_lexicon_tags_11 },
    { 0xdf1f3409, SUC_lexicon_tags_44 },
    { 0x9e1f3408, SUC_lexicon_tags_25 },
    { 0xf75f3407, SUC_lexicon_tags_28 },
    { 0x31ff340c, SUC_lexicon_tags_7 },
    { 0xe37f340d, SUC_lexicon_tags_26 },
    { 0x55ff340d, SUC_lexicon_tags_56 },
    { 0xc89f340c, SUC_lexicon_tags_21 },
    { 0xc83f3410, SUC_lexicon_tags_1 },
    { 0xe79f3410, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x761f3414, SUC_lexicon_tags_32 },
    { 0xaabf3414, SUC_lexicon_tags_11 },
    { 0x9e1f3416, SUC_lexicon_tags_11 },
    { 0x82bf3416, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1cff3419, SUC_lexicon_tags_17 },
    { 0xf65f341a, SUC_lexicon_tags_27 },
    { 0x157f341b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49df341f, SUC_lexicon_tags_7 },
    { 0x2edf3420, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x965f3423, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9f342c, SUC_lexicon_tags_0 },
    { 0x943f342d, SUC_lexicon_tags_1 },
    { 0x529f342d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7f3431, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x54bf3433, SUC_lexicon_tags_11 },
    { 0x81f3433, SUC_lexicon_tags_12 },
    { 0x2c7f3435, SUC_lexicon_tags_23 },
    { 0xd85f3434, SUC_lexicon_tags_25 },
    { 0xdedf3434, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37f343a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x589f343c, SUC_lexicon_tags_11 },
    { 0x87bf343d, SUC_lexicon_tags_15 },
    { 0x323f343e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x92bf3440, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8adf3443, SUC_lexicon_tags_37 },
    { 0x839f3444, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21f3447, SUC_lexicon_tags_15 },
    { 0x6df3448, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x113f344a, SUC_lexicon_tags_17 },
    { 0xcddf344a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x687f3452, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x821f3454, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ff3457, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38bf3459, SUC_lexicon_tags_60 },
    { 0x6a5f3459, SUC_lexicon_tags_49 },
    { 0x5e9f345b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1ff345d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bf3465, SUC_lexicon_tags_2 },
    { 0xd7df3466, SUC_lexicon_tags_37 },
    { 0xe51f3467, SUC_lexicon_tags_15 },
    { 0xe5bf3465, SUC_lexicon_tags_37 },
    { 0xf39f3469, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2df346e, SUC_lexicon_tags_12 },
    { 0xb8ff346e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x159f3471, SUC_lexicon_tags_37 },
    { 0x34bf3472, SUC_lexicon_tags_60 },
    { 0x29f3473, SUC_lexicon_tags_17 },
    { 0xa9bf3472, SUC_lexicon_tags_30 },
    { 0xdc3f3475, SUC_lexicon_tags_37 },
    { 0x6c7f3476, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xabff3478, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2d7f347a, SUC_lexicon_tags_47 },
    { 0x5f1f347b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbf3483, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x14ff3486, SUC_lexicon_tags_26 },
    { 0x7bbf3487, SUC_lexicon_tags_38 },
    { 0x4fff3488, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957f348f, SUC_lexicon_tags_53 },
    { 0x4df348f, SUC_lexicon_tags_37 },
    { 0x4fff3491, SUC_lexicon_tags_2 },
    { 0x1c5f3492, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd27f3494, SUC_lexicon_tags_15 },
    { 0x115f3495, SUC_lexicon_tags_37 },
    { 0x319f3496, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ff349a, SUC_lexicon_tags_27 },
    { 0x36ff349b, SUC_lexicon_tags_2 },
    { 0x127f349c, SUC_lexicon_tags_17 },
    { 0x3b9f349c, SUC_lexicon_tags_9 },
    { 0xfe7f349d, SUC_lexicon_tags_15 },
    { 0x24bf349a, SUC_lexicon_tags_193 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69f34a3, SUC_lexicon_tags_14 },
    { 0x43ff34a3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5f34a9, SUC_lexicon_tags_25 },
    { 0xe93f34aa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x459f34ac, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33f34b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x445f34b4, SUC_lexicon_tags_2 },
    { 0x15f34b5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6d5f34b7, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0x175f34b9, SUC_lexicon_tags_49 },
    { 0x31ff34ba, SUC_lexicon_tags_14 },
    { 0x91ff34bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbf34be, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xb41f34c1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbff34c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe8bf34c6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x179f34ca, SUC_lexicon_tags_45 },
    { 0x4dff34cb, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xce5f34cd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x51ff34cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5f34db, SUC_lexicon_tags_60 },
    { 0xb7ff34db, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x343f34de, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x407f34e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6c5f34e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x963f34e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9f34e7, SUC_lexicon_tags_11 },
    { 0x5d7f34e8, SUC_lexicon_tags_2 },
    { 0xd43f34e8, SUC_lexicon_tags_27 },
    { 0xe29f34ea, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ff34ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x685f34f0, SUC_lexicon_tags_15 },
    { 0x57f34f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9f34f4, SUC_lexicon_tags_25 },
    { 0x3f34f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331f34fa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1a1f34fc, SUC_lexicon_tags_17 },
    { 0x51bf34fd, SUC_lexicon_tags_15 },
    { 0x6d7f34fe, SUC_lexicon_tags_37 },
    { 0x17df34fe, SUC_lexicon_tags_17 },
    { 0x5ebf34ff, SUC_lexicon_tags_1 },
    { 0x129f3501, SUC_lexicon_tags_27 },
    { 0xe05f3502, SUC_lexicon_tags_43 },
    { 0xd1ff34fc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaf5f3505, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa8df3507, SUC_lexicon_tags_27 },
    { 0x22ff3507, SUC_lexicon_tags_87 },
    { 0xa8ff3509, SUC_lexicon_tags_2 },
    { 0xb65f3509, SUC_lexicon_tags_11 },
    { 0x82df3507, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5f3511, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf3514, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3f3518, SUC_lexicon_tags_7 },
    { 0x267f3519, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ff351d, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63f3520, SUC_lexicon_tags_17 },
    { 0xba3f3521, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29f3526, SUC_lexicon_tags_17 },
    { 0x4dff3527, SUC_lexicon_tags_68 },
    { 0xfedf3527, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x65df352a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x23bf352c, SUC_lexicon_tags_15 },
    { 0x9f1f352d, SUC_lexicon_tags_35 },
    { 0x265f352e, SUC_lexicon_tags_72 },
    { 0x647f352f, SUC_lexicon_tags_17 },
    { 0xe87f3530, SUC_lexicon_tags_24 },
    { 0xb7bf3531, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xad9f3533, SUC_lexicon_tags_14 },
    { 0xfa9f3533, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119f3538, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x827f353a, SUC_lexicon_tags_60 },
    { 0x5f353a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdf3541, SUC_lexicon_tags_37 },
    { 0xf05f3542, SUC_lexicon_tags_28 },
    { 0xb35f3543, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xd0df3545, SUC_lexicon_tags_15 },
    { 0x743f3546, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x30bf3548, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583f354c, SUC_lexicon_tags_11 },
    { 0x70ff354d, SUC_lexicon_tags_37 },
    { 0xc6ff354d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763f3552, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x35df3554, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x293f3557, SUC_lexicon_tags_69 },
    { 0x12df3558, SUC_lexicon_tags_48 },
    { 0x691f3559, SUC_lexicon_tags_31 },
    { 0xb79f3557, SUC_lexicon_tags_7 },
    { 0x34ff355b, SUC_lexicon_tags_35 },
    { 0xcb5f355c, SUC_lexicon_tags_7 },
    { 0x255f3559, SUC_lexicon_tags_7 },
    { 0x469f355e, SUC_lexicon_tags_17 },
    { 0xb5bf3558, SUC_lexicon_tags_14 },
    { 0xa5df355e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x793f3562, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x203f3564, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe31f3566, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17f356b, SUC_lexicon_tags_49 },
    { 0x6d9f356c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeddf356f, SUC_lexicon_tags_60 },
    { 0x613f3570, SUC_lexicon_tags_25 },
    { 0xe0df3570, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373f3576, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x20ff3578, SUC_lexicon_tags_60 },
    { 0x229f3578, SUC_lexicon_tags_55 },
    { 0x1f9f3578, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72df357e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41f3584, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f3588, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f358b, SUC_lexicon_tags_11 },
    { 0x761f358c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43f3592, SUC_lexicon_tags_15 },
    { 0xdd7f3593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x455f3595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddf3598, SUC_lexicon_tags_25 },
    { 0xf39f3599, SUC_lexicon_tags_14 },
    { 0x3a1f3599, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb57f359c, SUC_lexicon_tags_11 },
    { 0x8e7f359d, SUC_lexicon_tags_12 },
    { 0xd0ff359e, SUC_lexicon_tags_2 },
    { 0x203f359f, SUC_lexicon_tags_23 },
    { 0xcb3f35a0, SUC_lexicon_tags_26 },
    { 0x83bf35a1, SUC_lexicon_tags_23 },
    { 0xd4bf35a1, SUC_lexicon_tags_1 },
    { 0xc29f35a3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37f35a7, SUC_lexicon_tags_47 },
    { 0x525f35a8, SUC_lexicon_tags_95 },
    { 0x491f35a7, SUC_lexicon_tags_84 },
    { 0x817f35aa, SUC_lexicon_tags_37 },
    { 0x919f35aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb51f35ad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493f35b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31f35be, SUC_lexicon_tags_39 },
    { 0xc5bf35bf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbf35c9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x541f35cc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x52bf35ce, SUC_lexicon_tags_49 },
    { 0x915f35ce, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6a5f35d1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x66ff35d3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x52bf35d5, SUC_lexicon_tags_31 },
    { 0xbebf35d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377f35da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31f35dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xae7f35df, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x981f35e2, SUC_lexicon_tags_45 },
    { 0xe7f35e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5bdf35e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4a5f35e7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899f35ee, SUC_lexicon_tags_49 },
    { 0x4abf35ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x747f35f1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4df35f3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xe63f35f5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x497f35f7, SUC_lexicon_tags_37 },
    { 0xbf35f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16df35fa, SUC_lexicon_tags_27 },
    { 0xebbf35fb, SUC_lexicon_tags_2 },
    { 0xa0df35fb, SUC_lexicon_tags_37 },
    { 0x4f3f35fa, SUC_lexicon_tags_29 },
    { 0xea5f35fc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x219f3600, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9f3604, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89f3608, SUC_lexicon_tags_30 },
    { 0x203f3609, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f360e, SUC_lexicon_tags_12 },
    { 0xc6bf360f, SUC_lexicon_tags_49 },
    { 0x549f360f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdf3616, SUC_lexicon_tags_17 },
    { 0xb9ff3616, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcb1f3619, SUC_lexicon_tags_38 },
    { 0xd9bf3619, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4df361c, SUC_lexicon_tags_55 },
    { 0xa8ff361c, SUC_lexicon_tags_42 },
    { 0x4b3f361c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171f3624, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7cbf3626, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5f3629, SUC_lexicon_tags_2 },
    { 0x755f3629, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0bf362c, SUC_lexicon_tags_216 },
    { 0xaf1f362d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x69df3630, SUC_lexicon_tags_11 },
    { 0xb8ff3631, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x793f363a, SUC_lexicon_tags_0 },
    { 0x18ff363b, SUC_lexicon_tags_11 },
    { 0x73ff363c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7d3f363e, SUC_lexicon_tags_25 },
    { 0xd09f363f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x145f3641, SUC_lexicon_tags_37 },
    { 0x7b9f3642, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755f364b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95f364f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ff3653, SUC_lexicon_tags_25 },
    { 0x775f3654, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5f3657, SUC_lexicon_tags_15 },
    { 0x4e9f3658, SUC_lexicon_tags_45 },
    { 0x347f3659, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf366d, SUC_lexicon_tags_40 },
    { 0xfd9f366e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5f3671, SUC_lexicon_tags_37 },
    { 0xdbdf3672, SUC_lexicon_tags_33 },
    { 0xe39f3673, SUC_lexicon_tags_1 },
    { 0xf73f3674, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc57f3676, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x753f3678, SUC_lexicon_tags_23 },
    { 0xa09f3679, SUC_lexicon_tags_47 },
    { 0xb7bf367a, SUC_lexicon_tags_14 },
    { 0x7d5f367b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9f3680, SUC_lexicon_tags_11 },
    { 0x8f9f3680, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x22df3684, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x737f3687, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ff368b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbf3692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x417f3694, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629f369b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5f36a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x347f36a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x32ff36a4, SUC_lexicon_tags_17 },
    { 0xa69f36a4, SUC_lexicon_tags_28 },
    { 0x725f36a6, SUC_lexicon_tags_25 },
    { 0xe21f36a7, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa29f36a9, SUC_lexicon_tags_27 },
    { 0x441f36a9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6df36ac, SUC_lexicon_tags_46 },
    { 0xa3f36ac, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebf36b1, SUC_lexicon_tags_68 },
    { 0x48df36b2, SUC_lexicon_tags_25 },
    { 0xcc1f36b3, SUC_lexicon_tags_60 },
    { 0x3f1f36b4, SUC_lexicon_tags_32 },
    { 0x927f36b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x597f36b7, SUC_lexicon_tags_47 },
    { 0xd0bf36b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21f36bc, SUC_lexicon_tags_9 },
    { 0x143f36bd, SUC_lexicon_tags_17 },
    { 0xd11f36be, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc07f36c0, SUC_lexicon_tags_32 },
    { 0x98df36c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbff36c4, SUC_lexicon_tags_64 },
    { 0x167f36c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd9ff36c7, SUC_lexicon_tags_12 },
    { 0xf49f36c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1f36cd, SUC_lexicon_tags_37 },
    { 0x787f36ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x88df36d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fff36d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f36d9, SUC_lexicon_tags_18 },
    { 0x295f36da, SUC_lexicon_tags_38 },
    { 0xfadf36db, SUC_lexicon_tags_49 },
    { 0xec3f36dc, SUC_lexicon_tags_301 },
    { 0x54ff36dd, SUC_lexicon_tags_30 },
    { 0xbd9f36db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x283f36e0, SUC_lexicon_tags_2 },
    { 0xa45f36e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x733f36e4, SUC_lexicon_tags_33 },
    { 0xa5df36e4, SUC_lexicon_tags_15 },
    { 0x931f36e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bf36e9, SUC_lexicon_tags_79 },
    { 0x7aff36ea, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x71bf36ec, SUC_lexicon_tags_1 },
    { 0xdc5f36ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7f36f0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f36f4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9f36f8, SUC_lexicon_tags_63 },
    { 0x4adf36f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adf3701, SUC_lexicon_tags_1 },
    { 0x85ff3702, SUC_lexicon_tags_12 },
    { 0xe49f3702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x933f3705, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bff370c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x90ff370f, SUC_lexicon_tags_63 },
    { 0xbd1f370f, SUC_lexicon_tags_30 },
    { 0x637f3711, SUC_lexicon_tags_82 },
    { 0xbf3710, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xaabf3714, SUC_lexicon_tags_37 },
    { 0x54df3715, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb5ff3717, SUC_lexicon_tags_11 },
    { 0xd4ff3717, SUC_lexicon_tags_17 },
    { 0x5b1f3718, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43f371d, SUC_lexicon_tags_68 },
    { 0xe63f371e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe19f3721, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3f3724, SUC_lexicon_tags_68 },
    { 0xdbdf3724, SUC_lexicon_tags_11 },
    { 0x8f9f3726, SUC_lexicon_tags_7 },
    { 0x913f3727, SUC_lexicon_tags_37 },
    { 0x90ff3727, SUC_lexicon_tags_25 },
    { 0x2f3f3726, SUC_lexicon_tags_43 },
    { 0xccbf372a, SUC_lexicon_tags_14 },
    { 0xaebf3728, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453f372f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x95ff3731, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1a1f3733, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdf3736, SUC_lexicon_tags_15 },
    { 0xa5bf3737, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xf7bf3739, SUC_lexicon_tags_12 },
    { 0xaddf3739, SUC_lexicon_tags_2 },
    { 0xccbf3739, SUC_lexicon_tags_23 },
    { 0xe53f373a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x357f373e, SUC_lexicon_tags_17 },
    { 0x7df373f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eff3743, SUC_lexicon_tags_37 },
    { 0x311f3744, SUC_lexicon_tags_69 },
    { 0x101f3745, SUC_lexicon_tags_31 },
    { 0x889f3746, SUC_lexicon_tags_45 },
    { 0x333f3746, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dff374d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fff3753, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x50bf3755, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03f3758, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x267f375c, SUC_lexicon_tags_8 },
    { 0xb27f375c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x409f3760, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x39f3763, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x62df3765, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x7f5f3767, SUC_lexicon_tags_6 },
    { 0xc71f3768, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x35df376a, SUC_lexicon_tags_7 },
    { 0x9e9f376b, SUC_lexicon_tags_1 },
    { 0x889f376c, SUC_lexicon_tags_37 },
    { 0x1b9f376b, SUC_lexicon_tags_26 },
    { 0x313f376b, SUC_lexicon_tags_27 },
    { 0xc67f376f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf67f3771, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x213f3779, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7f3780, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343f3786, SUC_lexicon_tags_38 },
    { 0xbe3f3787, SUC_lexicon_tags_38 },
    { 0x3dbf3787, SUC_lexicon_tags_17 },
    { 0x827f3789, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddf378f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bf3795, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457f379a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ff379e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfa5f37a0, SUC_lexicon_tags_2 },
    { 0x5d9f37a1, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x9c5f37a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe31f37a5, SUC_lexicon_tags_4 },
    { 0x7f7f37a6, SUC_lexicon_tags_0 },
    { 0xbedf37a7, SUC_lexicon_tags_45 },
    { 0x9a1f37a8, SUC_lexicon_tags_2 },
    { 0x575f37a8, SUC_lexicon_tags_25 },
    { 0xabdf37a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c9f37ac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307f37b3, SUC_lexicon_tags_38 },
    { 0xb3df37b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bf37b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3f37bb, SUC_lexicon_tags_37 },
    { 0x361f37bc, SUC_lexicon_tags_0 },
    { 0x93ff37bc, SUC_lexicon_tags_11 },
    { 0xf47f37bd, SUC_lexicon_tags_61 },
    { 0x563f37bf, SUC_lexicon_tags_1 },
    { 0xdfff37bb, SUC_lexicon_tags_79 },
    { 0xf95f37c1, SUC_lexicon_tags_1 },
    { 0x389f37be, SUC_lexicon_tags_0 },
    { 0xd6bf37c3, SUC_lexicon_tags_2 },
    { 0x6c7f37c4, SUC_lexicon_tags_33 },
    { 0x9b1f37c3, SUC_lexicon_tags_25 },
    { 0x797f37c6, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf7f37c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3f37cc, SUC_lexicon_tags_14 },
    { 0xd7df37cc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xafff37cf, SUC_lexicon_tags_16 },
    { 0x2d3f37cf, SUC_lexicon_tags_7 },
    { 0x36df37d1, SUC_lexicon_tags_17 },
    { 0x23f37cf, SUC_lexicon_tags_0 },
    { 0x693f37cf, SUC_lexicon_tags_0 },
    { 0xccbf37cf, SUC_lexicon_tags_17 },
    { 0xf45f37d5, SUC_lexicon_tags_11 },
    { 0x515f37d3, SUC_lexicon_tags_1 },
    { 0x825f37d7, SUC_lexicon_tags_47 },
    { 0xacbf37d8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75df37dc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x225f37de, SUC_lexicon_tags_27 },
    { 0xb6df37df, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07f37e3, SUC_lexicon_tags_31 },
    { 0xa13f37e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf37e9, SUC_lexicon_tags_65 },
    { 0x271f37ea, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe55f37ec, SUC_lexicon_tags_33 },
    { 0x313f37ec, SUC_lexicon_tags_17 },
    { 0x1d9f37ec, SUC_lexicon_tags_29 },
    { 0x62df37ef, SUC_lexicon_tags_25 },
    { 0xca3f37f0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb39f37f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6aff37f6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x74ff37f8, SUC_lexicon_tags_60 },
    { 0x78ff37f9, SUC_lexicon_tags_27 },
    { 0x559f37f8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9df37fd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x133f3800, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bdf3802, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bff3804, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa5bf3806, SUC_lexicon_tags_48 },
    { 0x1cbf3807, SUC_lexicon_tags_2 },
    { 0xe73f3808, SUC_lexicon_tags_33 },
    { 0x7c9f3809, SUC_lexicon_tags_25 },
    { 0x8d1f3806, SUC_lexicon_tags_7 },
    { 0xabff3809, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb99f380d, SUC_lexicon_tags_12 },
    { 0x38bf380e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5f7f3810, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xde7f3813, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb05f3815, SUC_lexicon_tags_25 },
    { 0xa21f3816, SUC_lexicon_tags_63 },
    { 0xa19f3817, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbbf3819, SUC_lexicon_tags_7 },
    { 0xabbf381a, SUC_lexicon_tags_2 },
    { 0x927f381a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xd99f381d, SUC_lexicon_tags_17 },
    { 0x567f381e, SUC_lexicon_tags_19 },
    { 0x149f381f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73f3824, SUC_lexicon_tags_129 },
    { 0xaf9f3825, SUC_lexicon_tags_63 },
    { 0xcbff3826, SUC_lexicon_tags_9 },
    { 0xcb7f3826, SUC_lexicon_tags_7 },
    { 0xaa5f3827, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13f382d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe3bf382f, SUC_lexicon_tags_25 },
    { 0x3f9f3830, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe6bf3832, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdf3835, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2df383a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1f383f, SUC_lexicon_tags_1 },
    { 0x5d9f3840, SUC_lexicon_tags_14 },
    { 0xbbf3840, SUC_lexicon_tags_11 },
    { 0x507f3842, SUC_lexicon_tags_48 },
    { 0x315f3843, SUC_lexicon_tags_12 },
    { 0x3a5f383f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5cbf3846, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649f384d, SUC_lexicon_tags_7 },
    { 0x73ff384e, SUC_lexicon_tags_25 },
    { 0xe6df384e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367f3856, SUC_lexicon_tags_43 },
    { 0x92df3856, SUC_lexicon_tags_2 },
    { 0x73f3856, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5f385b, SUC_lexicon_tags_15 },
    { 0x489f385c, SUC_lexicon_tags_30 },
    { 0xc65f385d, SUC_lexicon_tags_178 },
    { 0x4ebf385d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x1f3f3860, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83bf3863, SUC_lexicon_tags_11 },
    { 0x883f3864, SUC_lexicon_tags_15 },
    { 0xbc9f3864, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2c9f3867, SUC_lexicon_tags_65 },
    { 0x30bf3868, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x967f386a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2dbf386c, SUC_lexicon_tags_38 },
    { 0x3f5f386d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bf3873, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd75f387b, SUC_lexicon_tags_15 },
    { 0x1aff387b, SUC_lexicon_tags_10 },
    { 0xad9f387d, SUC_lexicon_tags_74 },
    { 0x5c3f387e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5f3883, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93f3887, SUC_lexicon_tags_47 },
    { 0x835f3888, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5f388f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5f3892, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf0ff3894, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xbe9f3896, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13f389a, SUC_lexicon_tags_12 },
    { 0x2d7f389b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x405f389d, SUC_lexicon_tags_37 },
    { 0x8d5f389d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x8d7f38a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x68bf38a2, SUC_lexicon_tags_62 },
    { 0x9f1f38a3, SUC_lexicon_tags_37 },
    { 0x99bf38a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbf38a7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33df38ad, SUC_lexicon_tags_7 },
    { 0xabbf38ad, SUC_lexicon_tags_2 },
    { 0xbbdf38af, SUC_lexicon_tags_2 },
    { 0x7adf38b0, SUC_lexicon_tags_37 },
    { 0xd5f38b1, SUC_lexicon_tags_17 },
    { 0x183f38b2, SUC_lexicon_tags_12 },
    { 0x60bf38b2, SUC_lexicon_tags_27 },
    { 0x7e1f38ae, SUC_lexicon_tags_15 },
    { 0xe49f38b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d3f38b7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31f38bb, SUC_lexicon_tags_136 },
    { 0xc4bf38bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261f38c0, SUC_lexicon_tags_11 },
    { 0x42bf38c0, SUC_lexicon_tags_29 },
    { 0xc25f38c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4d3f38c4, SUC_lexicon_tags_46 },
    { 0xf01f38c4, SUC_lexicon_tags_1 },
    { 0x8abf38c6, SUC_lexicon_tags_17 },
    { 0xb55f38c6, SUC_lexicon_tags_28 },
    { 0xba7f38c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39df38cc, SUC_lexicon_tags_29 },
    { 0x521f38cc, SUC_lexicon_tags_23 },
    { 0xe6bf38cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ff38d8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x455f38da, SUC_lexicon_tags_37 },
    { 0x1ff38db, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69f38de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1f38e3, SUC_lexicon_tags_37 },
    { 0x33ff38e4, SUC_lexicon_tags_25 },
    { 0x7eff38e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf38ea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77f38f2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x831f38f4, SUC_lexicon_tags_6 },
    { 0x6f5f38f5, SUC_lexicon_tags_37 },
    { 0x6fbf38f6, SUC_lexicon_tags_27 },
    { 0xbe3f38f7, SUC_lexicon_tags_31 },
    { 0xeabf38f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf38fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ff3900, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ff3906, SUC_lexicon_tags_15 },
    { 0x555f3907, SUC_lexicon_tags_15 },
    { 0x581f3907, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47ff390a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd1f390d, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2df3912, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x6f9f3914, SUC_lexicon_tags_3 },
    { 0xc69f3915, SUC_lexicon_tags_37 },
    { 0x1b1f3916, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1f3919, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbf391c, SUC_lexicon_tags_1 },
    { 0x98ff391d, SUC_lexicon_tags_12 },
    { 0xf53f391c, SUC_lexicon_tags_37 },
    { 0xaabf391f, SUC_lexicon_tags_7 },
    { 0x19df391c, SUC_lexicon_tags_55 },
    { 0x865f3921, SUC_lexicon_tags_15 },
    { 0xca3f3922, SUC_lexicon_tags_15 },
    { 0x671f3923, SUC_lexicon_tags_32 },
    { 0xe59f3924, SUC_lexicon_tags_11 },
    { 0x8c7f3924, SUC_lexicon_tags_23 },
    { 0xa2bf391d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611f392a, SUC_lexicon_tags_15 },
    { 0x513f392b, SUC_lexicon_tags_26 },
    { 0x315f392b, SUC_lexicon_tags_1 },
    { 0x81df392d, SUC_lexicon_tags_77 },
    { 0xcd3f392e, SUC_lexicon_tags_6 },
    { 0xc1ff392e, SUC_lexicon_tags_18 },
    { 0x63ff3930, SUC_lexicon_tags_57 },
    { 0x60bf3931, SUC_lexicon_tags_27 },
    { 0x65f3932, SUC_lexicon_tags_9 },
    { 0x50df3933, SUC_lexicon_tags_12 },
    { 0x4a5f3933, SUC_lexicon_tags_65 },
    { 0x30bf3935, SUC_lexicon_tags_27 },
    { 0xb9df3935, SUC_lexicon_tags_0 },
    { 0x90ff3937, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xca1f3939, SUC_lexicon_tags_2 },
    { 0x413f393a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95f393e, SUC_lexicon_tags_25 },
    { 0x7cbf393f, SUC_lexicon_tags_49 },
    { 0x8b9f3940, SUC_lexicon_tags_38 },
    { 0xf35f3940, SUC_lexicon_tags_2 },
    { 0xee5f393e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3f3945, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf6df3947, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f3952, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3f3958, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x203f395a, SUC_lexicon_tags_49 },
    { 0x61f395a, SUC_lexicon_tags_11 },
    { 0xb9ff395a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd97f395f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc07f3961, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd4df3963, SUC_lexicon_tags_310 },
    { 0x137f3964, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabf3967, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x907f3969, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bf396c, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f396f, SUC_lexicon_tags_43 },
    { 0xb55f396f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55f3978, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bdf397a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x959f397c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xcb1f397e, SUC_lexicon_tags_49 },
    { 0xe2df397f, SUC_lexicon_tags_37 },
    { 0x4c1f397f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x839f3984, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ff3988, SUC_lexicon_tags_32 },
    { 0x323f3989, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x33bf398b, SUC_lexicon_tags_38 },
    { 0x4bff398c, SUC_lexicon_tags_7 },
    { 0xc21f398d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65f3991, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x455f3993, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ff3996, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973f399c, SUC_lexicon_tags_17 },
    { 0xff9f399d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95bf399f, SUC_lexicon_tags_57 },
    { 0x11f39a0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ff39ab, SUC_lexicon_tags_46 },
    { 0x1ddf39ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49df39b6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x49df39b8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1f39bb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xc6bf39bd, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x347f39bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bf39c8, SUC_lexicon_tags_45 },
    { 0xc11f39c9, SUC_lexicon_tags_11 },
    { 0x691f39ca, SUC_lexicon_tags_12 },
    { 0x563f39cb, SUC_lexicon_tags_2 },
    { 0x29bf39c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa9f39ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15f39d1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x157f39d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aff39d6, SUC_lexicon_tags_55 },
    { 0x387f39d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd69f39d9, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x921f39db, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeff39df, SUC_lexicon_tags_47 },
    { 0xdc3f39df, SUC_lexicon_tags_11 },
    { 0xc53f39e1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1bff39e3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xce7f39e5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb0df39e7, SUC_lexicon_tags_17 },
    { 0x5d7f39e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61f39eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4f3f39ed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9f39f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3df39f5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x21ff39f7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63f39fa, SUC_lexicon_tags_25 },
    { 0xdf5f39fb, SUC_lexicon_tags_3 },
    { 0xef1f39fb, SUC_lexicon_tags_6 },
    { 0x5cff39fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03f3a00, SUC_lexicon_tags_11 },
    { 0x977f3a01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9f3a05, SUC_lexicon_tags_31 },
    { 0x395f3a06, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1f3a12, SUC_lexicon_tags_14 },
    { 0x9dbf3a12, SUC_lexicon_tags_13 },
    { 0x4bdf3a12, SUC_lexicon_tags_25 },
    { 0x2bdf3a15, SUC_lexicon_tags_2 },
    { 0x525f3a16, SUC_lexicon_tags_15 },
    { 0xff3f3a14, SUC_lexicon_tags_65 },
    { 0x17df3a13, SUC_lexicon_tags_43 },
    { 0xa1ff3a19, SUC_lexicon_tags_65 },
    { 0xdd5f3a19, SUC_lexicon_tags_37 },
    { 0x543f3a19, SUC_lexicon_tags_29 },
    { 0x14bf3a12, SUC_lexicon_tags_38 },
    { 0x7d9f3a19, SUC_lexicon_tags_45 },
    { 0xd87f3a1d, SUC_lexicon_tags_25 },
    { 0x3c9f3a1f, SUC_lexicon_tags_27 },
    { 0x831f3a20, SUC_lexicon_tags_91 },
    { 0x391f3a1a, SUC_lexicon_tags_9 },
    { 0x6e3f3a22, SUC_lexicon_tags_33 },
    { 0x135f3a22, SUC_lexicon_tags_31 },
    { 0xec5f3a24, SUC_lexicon_tags_28 },
    { 0x7eff3a24, SUC_lexicon_tags_37 },
    { 0xd3f3a26, SUC_lexicon_tags_25 },
    { 0x893f3a27, SUC_lexicon_tags_37 },
    { 0x85f3a24, SUC_lexicon_tags_11 },
    { 0x54ff3a25, SUC_lexicon_tags_0 },
    { 0x319f3a29, SUC_lexicon_tags_30 },
    { 0x9dff3a2b, SUC_lexicon_tags_15 },
    { 0x35bf3a2b, SUC_lexicon_tags_17 },
    { 0x45df3a2b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abf3a30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb39f3a32, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607f3a39, SUC_lexicon_tags_1 },
    { 0xe9ff3a3a, SUC_lexicon_tags_7 },
    { 0xd3ff3a3a, SUC_lexicon_tags_0 },
    { 0x98bf3a3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x145f3a3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ff3a41, SUC_lexicon_tags_37 },
    { 0xa85f3a41, SUC_lexicon_tags_25 },
    { 0x443f3a43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5f3a48, SUC_lexicon_tags_37 },
    { 0x4fbf3a49, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bf3a4c, SUC_lexicon_tags_6 },
    { 0xfe7f3a4d, SUC_lexicon_tags_31 },
    { 0x96ff3a4d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbf3a51, SUC_lexicon_tags_64 },
    { 0x2c9f3a51, SUC_lexicon_tags_16 },
    { 0xe79f3a53, SUC_lexicon_tags_47 },
    { 0x85ff3a53, SUC_lexicon_tags_0 },
    { 0x209f3a51, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xfc5f3a57, SUC_lexicon_tags_47 },
    { 0x153f3a57, SUC_lexicon_tags_37 },
    { 0x5c7f3a57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17f3a5e, SUC_lexicon_tags_27 },
    { 0x217f3a5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3edf3a61, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe43f3a65, SUC_lexicon_tags_23 },
    { 0xa97f3a66, SUC_lexicon_tags_2 },
    { 0xe5df3a67, SUC_lexicon_tags_16 },
    { 0x46df3a68, SUC_lexicon_tags_141 },
    { 0x833f3a66, SUC_lexicon_tags_38 },
    { 0x7b5f3a6a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb23f3a6c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3df3a70, SUC_lexicon_tags_2 },
    { 0xb7df3a71, SUC_lexicon_tags_11 },
    { 0x255f3a72, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7f3a76, SUC_lexicon_tags_103 },
    { 0x91df3a77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x789f3a79, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafff3a7b, SUC_lexicon_tags_1 },
    { 0xc09f3a7b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xd6df3a7e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf83f3a80, SUC_lexicon_tags_103 },
    { 0x97ff3a81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8c7f3a83, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x39bf3a86, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1f3a89, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc41f3a90, SUC_lexicon_tags_37 },
    { 0x82bf3a90, SUC_lexicon_tags_2 },
    { 0xbcbf3a92, SUC_lexicon_tags_87 },
    { 0x3b9f3a93, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfeff3a95, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd3f3a97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x597f3a99, SUC_lexicon_tags_11 },
    { 0x4bff3a9a, SUC_lexicon_tags_7 },
    { 0x52bf3a9b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ff3aa0, SUC_lexicon_tags_25 },
    { 0xd59f3aa1, SUC_lexicon_tags_37 },
    { 0x753f3aa2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7f3aa6, SUC_lexicon_tags_38 },
    { 0x703f3aa7, SUC_lexicon_tags_87 },
    { 0x2dff3aa8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23df3aad, SUC_lexicon_tags_63 },
    { 0x375f3aae, SUC_lexicon_tags_47 },
    { 0x41f3aaf, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3f3ab4, SUC_lexicon_tags_33 },
    { 0x659f3ab4, SUC_lexicon_tags_7 },
    { 0xc09f3ab6, SUC_lexicon_tags_11 },
    { 0x85ff3ab6, SUC_lexicon_tags_7 },
    { 0xda9f3ab8, SUC_lexicon_tags_2 },
    { 0x211f3ab9, SUC_lexicon_tags_7 },
    { 0x479f3aba, SUC_lexicon_tags_2 },
    { 0xd7bf3abb, SUC_lexicon_tags_47 },
    { 0x309f3abb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa11f3abf, SUC_lexicon_tags_55 },
    { 0x889f3abf, SUC_lexicon_tags_11 },
    { 0xd5df3ac1, SUC_lexicon_tags_12 },
    { 0x4b3f3ac2, SUC_lexicon_tags_6 },
    { 0x4b5f3ac3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd07f3ac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8df3ac8, SUC_lexicon_tags_15 },
    { 0x82df3ac9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdf3acc, SUC_lexicon_tags_38 },
    { 0xf7bf3acd, SUC_lexicon_tags_38 },
    { 0xb7f3ace, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x883f3ad0, SUC_lexicon_tags_11 },
    { 0x891f3ad1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165f3ad6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19df3adc, SUC_lexicon_tags_0 },
    { 0x6f7f3adc, SUC_lexicon_tags_47 },
    { 0x4fbf3ade, SUC_lexicon_tags_141 },
    { 0x72ff3ade, SUC_lexicon_tags_12 },
    { 0x6b9f3ae0, SUC_lexicon_tags_38 },
    { 0xc0bf3ae1, SUC_lexicon_tags_25 },
    { 0xf6ff3adf, SUC_lexicon_tags_1 },
    { 0xbbdf3ae1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x885f3ae5, SUC_lexicon_tags_11 },
    { 0xbbdf3ae5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x761f3ae8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbe7f3aea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71f3aed, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1f3af4, SUC_lexicon_tags_17 },
    { 0x8adf3af5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287f3afb, SUC_lexicon_tags_17 },
    { 0xb35f3afb, SUC_lexicon_tags_29 },
    { 0xb09f3afd, SUC_lexicon_tags_21 },
    { 0xf11f3afe, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ff3b03, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x33ff3b06, SUC_lexicon_tags_65 },
    { 0xe3ff3b06, SUC_lexicon_tags_41 },
    { 0x81f3b07, SUC_lexicon_tags_27 },
    { 0xcdf3b09, SUC_lexicon_tags_11 },
    { 0xa53f3b06, SUC_lexicon_tags_12 },
    { 0xbd9f3b08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x439f3b0d, SUC_lexicon_tags_2 },
    { 0x77bf3b0d, SUC_lexicon_tags_42 },
    { 0x515f3b0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd17f3b11, SUC_lexicon_tags_17 },
    { 0x11df3b12, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9f3b15, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb79f3b1f, SUC_lexicon_tags_29 },
    { 0xf81f3b1f, SUC_lexicon_tags_27 },
    { 0xf1ff3b20, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459f3b25, SUC_lexicon_tags_17 },
    { 0x39df3b26, SUC_lexicon_tags_21 },
    { 0x6adf3b26, SUC_lexicon_tags_80 },
    { 0x73f3b28, SUC_lexicon_tags_15 },
    { 0xd63f3b29, SUC_lexicon_tags_11 },
    { 0x271f3b29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x83bf3b2c, SUC_lexicon_tags_45 },
    { 0x5a7f3b2d, SUC_lexicon_tags_2 },
    { 0xe89f3b2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x14df3b30, SUC_lexicon_tags_13 },
    { 0x59ff3b31, SUC_lexicon_tags_33 },
    { 0x791f3b30, SUC_lexicon_tags_7 },
    { 0x623f3b32, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79f3b38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xca5f3b3a, SUC_lexicon_tags_47 },
    { 0x8fff3b3b, SUC_lexicon_tags_11 },
    { 0xfb3f3b3a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd47f3b3e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65f3b42, SUC_lexicon_tags_11 },
    { 0xf41f3b42, SUC_lexicon_tags_0 },
    { 0x23bf3b44, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc81f3b46, SUC_lexicon_tags_15 },
    { 0x6bff3b47, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8fbf3b49, SUC_lexicon_tags_38 },
    { 0x553f3b4a, SUC_lexicon_tags_8 },
    { 0x8df3b49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeff3b51, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9f3b55, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59f3b5e, SUC_lexicon_tags_79 },
    { 0xb9bf3b5f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5f3b64, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ff3b6e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x605f3b70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdf3b73, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f3b78, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ff3b88, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3d1f3b8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbd5f3b8c, SUC_lexicon_tags_25 },
    { 0xc13f3b8d, SUC_lexicon_tags_9 },
    { 0x12ff3b8d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13f3b91, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f3b95, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc69f3b98, SUC_lexicon_tags_25 },
    { 0xb43f3b99, SUC_lexicon_tags_39 },
    { 0xc41f3b99, SUC_lexicon_tags_11 },
    { 0x96ff3b98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xb67f3b9d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x961f3ba0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x473f3ba5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5f3bb0, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f3bb5, SUC_lexicon_tags_30 },
    { 0xb41f3bb6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb23f3bb8, SUC_lexicon_tags_19 },
    { 0x585f3bb8, SUC_lexicon_tags_28 },
    { 0x37df3bba, SUC_lexicon_tags_23 },
    { 0x805f3bbb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x66bf3bbd, SUC_lexicon_tags_2 },
    { 0xbf3f3bbe, SUC_lexicon_tags_27 },
    { 0xc63f3bbf, SUC_lexicon_tags_28 },
    { 0xc81f3bbf, SUC_lexicon_tags_7 },
    { 0x2e3f3bc1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd7df3bc3, SUC_lexicon_tags_8 },
    { 0x845f3bc4, SUC_lexicon_tags_2 },
    { 0x8b7f3bc5, SUC_lexicon_tags_33 },
    { 0x6d1f3bc6, SUC_lexicon_tags_17 },
    { 0x18df3bc7, SUC_lexicon_tags_87 },
    { 0x76df3bc7, SUC_lexicon_tags_37 },
    { 0x98ff3bc3, SUC_lexicon_tags_37 },
    { 0xe35f3bca, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x269f3bcc, SUC_lexicon_tags_32 },
    { 0xd97f3bcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x763f3bcf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x913f3bd1, SUC_lexicon_tags_1 },
    { 0xde7f3bd2, SUC_lexicon_tags_27 },
    { 0x97df3bd1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1f3bdb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409f3bdf, SUC_lexicon_tags_2 },
    { 0x6abf3bdf, SUC_lexicon_tags_26 },
    { 0xc4ff3bdf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x24ff3be4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf6bf3be6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x399f3be8, SUC_lexicon_tags_14 },
    { 0x35f3be9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adf3bec, SUC_lexicon_tags_87 },
    { 0x6a7f3bed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb95f3bf2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x77bf3bf5, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17f3bfc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbf3bff, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe63f3c01, SUC_lexicon_tags_11 },
    { 0xcfbf3c02, SUC_lexicon_tags_9 },
    { 0x997f3c02, SUC_lexicon_tags_101 },
    { 0x879f3c02, SUC_lexicon_tags_47 },
    { 0x4d3f3c05, SUC_lexicon_tags_80 },
    { 0x7f5f3c06, SUC_lexicon_tags_26 },
    { 0x235f3c07, SUC_lexicon_tags_47 },
    { 0x7bf3c06, SUC_lexicon_tags_27 },
    { 0xb6df3c02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7f3f3c0b, SUC_lexicon_tags_15 },
    { 0x849f3c0b, SUC_lexicon_tags_2 },
    { 0x96bf3c0b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68ff3c14, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ff3c19, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2d1f3c1b, SUC_lexicon_tags_19 },
    { 0x95f3c1c, SUC_lexicon_tags_22 },
    { 0x63f3c1c, SUC_lexicon_tags_25 },
    { 0x16df3c1e, SUC_lexicon_tags_10 },
    { 0x137f3c1d, SUC_lexicon_tags_31 },
    { 0x8d5f3c1c, SUC_lexicon_tags_27 },
    { 0x931f3c1e, SUC_lexicon_tags_12 },
    { 0x769f3c22, SUC_lexicon_tags_27 },
    { 0x2e7f3c22, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbf3c27, SUC_lexicon_tags_38 },
    { 0x569f3c28, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb49f3c2b, SUC_lexicon_tags_14 },
    { 0x537f3c2c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45f3c34, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475f3c38, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7f3c3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7f3c45, SUC_lexicon_tags_81 },
    { 0x3a7f3c46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb43f3c48, SUC_lexicon_tags_14 },
    { 0x2cdf3c49, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73f3c4c, SUC_lexicon_tags_16 },
    { 0xa39f3c4c, SUC_lexicon_tags_12 },
    { 0x15f3c4e, SUC_lexicon_tags_1 },
    { 0x8bf3c4f, SUC_lexicon_tags_15 },
    { 0x7b5f3c4f, SUC_lexicon_tags_23 },
    { 0xcd7f3c51, SUC_lexicon_tags_37 },
    { 0x3fdf3c52, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bf3c57, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x839f3c5a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa23f3c5c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xf77f3c5e, SUC_lexicon_tags_27 },
    { 0x1fff3c5f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x8c3f3c61, SUC_lexicon_tags_23 },
    { 0xed7f3c62, SUC_lexicon_tags_34 },
    { 0x45df3c62, SUC_lexicon_tags_27 },
    { 0x45bf3c63, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x43df3c67, SUC_lexicon_tags_49 },
    { 0x935f3c68, SUC_lexicon_tags_11 },
    { 0xfb5f3c68, SUC_lexicon_tags_31 },
    { 0x3bdf3c67, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bf3c6f, SUC_lexicon_tags_49 },
    { 0x64bf3c70, SUC_lexicon_tags_12 },
    { 0xbebf3c70, SUC_lexicon_tags_45 },
    { 0xfddf3c71, SUC_lexicon_tags_27 },
    { 0x7abf3c73, SUC_lexicon_tags_32 },
    { 0x721f3c72, SUC_lexicon_tags_49 },
    { 0x371f3c75, SUC_lexicon_tags_29 },
    { 0xd81f3c76, SUC_lexicon_tags_2 },
    { 0x603f3c76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ff3c7a, SUC_lexicon_tags_82 },
    { 0xcddf3c7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3f3c83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bf3c86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc09f3c88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73f3c91, SUC_lexicon_tags_65 },
    { 0xcdf3c92, SUC_lexicon_tags_25 },
    { 0xe0ff3c91, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x96ff3c95, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27f3c9a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x4ddf3c9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xe13f3c9e, SUC_lexicon_tags_19 },
    { 0x429f3c9e, SUC_lexicon_tags_8 },
    { 0xb2ff3c9e, SUC_lexicon_tags_16 },
    { 0xd8bf3ca1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb05f3ca3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x179f3ca6, SUC_lexicon_tags_37 },
    { 0x5d7f3ca7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x489f3ca9, SUC_lexicon_tags_1 },
    { 0x465f3ca9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x51df3cac, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1f3caf, SUC_lexicon_tags_4 },
    { 0x1fbf3cb0, SUC_lexicon_tags_83 },
    { 0x36bf3caf, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf3cb7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dff3cbe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa9df3cc0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d1f3cc2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x315f3cc4, SUC_lexicon_tags_47 },
    { 0xd0df3cc4, SUC_lexicon_tags_116 },
    { 0x49bf3cc6, SUC_lexicon_tags_2 },
    { 0x3e9f3cc7, SUC_lexicon_tags_32 },
    { 0x359f3cc5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ff3cd0, SUC_lexicon_tags_10 },
    { 0x5cbf3cd1, SUC_lexicon_tags_15 },
    { 0xecdf3cd0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59ff3cd4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f3cda, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf57f3cdc, SUC_lexicon_tags_30 },
    { 0x75bf3cdd, SUC_lexicon_tags_25 },
    { 0xfadf3cde, SUC_lexicon_tags_25 },
    { 0x441f3cdd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bf3ce2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x651f3ce4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe73f3ce7, SUC_lexicon_tags_13 },
    { 0x527f3ce8, SUC_lexicon_tags_17 },
    { 0xb7df3ce9, SUC_lexicon_tags_12 },
    { 0x339f3ce7, SUC_lexicon_tags_47 },
    { 0x34df3ceb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x4b3f3ced, SUC_lexicon_tags_15 },
    { 0x4bdf3cee, SUC_lexicon_tags_11 },
    { 0xd15f3ced, SUC_lexicon_tags_37 },
    { 0x853f3cf0, SUC_lexicon_tags_12 },
    { 0x297f3cf1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339f3cfd, SUC_lexicon_tags_37 },
    { 0x801f3cfe, SUC_lexicon_tags_221 },
    { 0x1d9f3cff, SUC_lexicon_tags_49 },
    { 0xedf3d00, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607f3d05, SUC_lexicon_tags_4 },
    { 0x78df3d06, SUC_lexicon_tags_49 },
    { 0x905f3d07, SUC_lexicon_tags_17 },
    { 0xebf3d08, SUC_lexicon_tags_29 },
    { 0x457f3d09, SUC_lexicon_tags_7 },
    { 0x859f3d0a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4fbf3d0c, SUC_lexicon_tags_8 },
    { 0xd73f3d0d, SUC_lexicon_tags_12 },
    { 0xe5f3d0c, SUC_lexicon_tags_37 },
    { 0xbc1f3d0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423f3d13, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bf3d18, SUC_lexicon_tags_1 },
    { 0x87f3d19, SUC_lexicon_tags_1 },
    { 0x1e5f3d1a, SUC_lexicon_tags_8 },
    { 0x921f3d19, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xefff3d1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47f3d22, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11df3d26, SUC_lexicon_tags_26 },
    { 0xb79f3d27, SUC_lexicon_tags_35 },
    { 0x2b1f3d28, SUC_lexicon_tags_15 },
    { 0x6dbf3d28, SUC_lexicon_tags_1 },
    { 0x3bf3d2a, SUC_lexicon_tags_25 },
    { 0xa3bf3d2b, SUC_lexicon_tags_25 },
    { 0x2c1f3d2b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff3d31, SUC_lexicon_tags_26 },
    { 0xe2ff3d32, SUC_lexicon_tags_19 },
    { 0x341f3d33, SUC_lexicon_tags_56 },
    { 0xe07f3d33, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07f3d37, SUC_lexicon_tags_8 },
    { 0xd1bf3d38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39f3d44, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc1ff3d46, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbf3d4a, SUC_lexicon_tags_11 },
    { 0xc35f3d4b, SUC_lexicon_tags_27 },
    { 0xfa3f3d4c, SUC_lexicon_tags_48 },
    { 0x261f3d4d, SUC_lexicon_tags_37 },
    { 0xbc3f3d4c, SUC_lexicon_tags_2 },
    { 0x537f3d4f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x681f3d52, SUC_lexicon_tags_120 },
    { 0x461f3d53, SUC_lexicon_tags_26 },
    { 0x109f3d54, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdf3d59, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5f3d5e, SUC_lexicon_tags_7 },
    { 0xfb5f3d5f, SUC_lexicon_tags_12 },
    { 0x7e7f3d5f, SUC_lexicon_tags_49 },
    { 0xda1f3d61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd0df3d63, SUC_lexicon_tags_25 },
    { 0x4dff3d64, SUC_lexicon_tags_25 },
    { 0x249f3d63, SUC_lexicon_tags_55 },
    { 0x863f3d66, SUC_lexicon_tags_47 },
    { 0x5e5f3d66, SUC_lexicon_tags_65 },
    { 0x4adf3d68, SUC_lexicon_tags_25 },
    { 0xbf3f3d69, SUC_lexicon_tags_1 },
    { 0x2d7f3d6a, SUC_lexicon_tags_27 },
    { 0xbedf3d69, SUC_lexicon_tags_17 },
    { 0x76df3d6c, SUC_lexicon_tags_0 },
    { 0xa55f3d66, SUC_lexicon_tags_35 },
    { 0x5e9f3d6e, SUC_lexicon_tags_25 },
    { 0xe37f3d6f, SUC_lexicon_tags_2 },
    { 0xf89f3d70, SUC_lexicon_tags_11 },
    { 0x579f3d6e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x50df3d73, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45f3d7d, SUC_lexicon_tags_9 },
    { 0x9a3f3d7e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe35f3d80, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dff3d8a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1f3d8d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4fdf3d8f, SUC_lexicon_tags_0 },
    { 0x34ff3d90, SUC_lexicon_tags_23 },
    { 0x615f3d91, SUC_lexicon_tags_17 },
    { 0xd4ff3d92, SUC_lexicon_tags_60 },
    { 0x691f3d93, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1f3d97, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bf3d9a, SUC_lexicon_tags_37 },
    { 0x4cdf3d9a, SUC_lexicon_tags_17 },
    { 0xbadf3d9c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37f3da3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7f3da6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc4df3da8, SUC_lexicon_tags_30 },
    { 0xc91f3da9, SUC_lexicon_tags_17 },
    { 0xcd9f3da8, SUC_lexicon_tags_2 },
    { 0x7adf3dab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3d3f3dad, SUC_lexicon_tags_48 },
    { 0x9df3dad, SUC_lexicon_tags_29 },
    { 0xa53f3dad, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9f3db2, SUC_lexicon_tags_11 },
    { 0x3bdf3db3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x125f3db5, SUC_lexicon_tags_12 },
    { 0xfa3f3db5, SUC_lexicon_tags_1 },
    { 0x36bf3db6, SUC_lexicon_tags_1 },
    { 0xe23f3db8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb2bf3dba, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x813f3dbc, SUC_lexicon_tags_57 },
    { 0x3bbf3dbc, SUC_lexicon_tags_14 },
    { 0x7f9f3dbe, SUC_lexicon_tags_37 },
    { 0xce1f3dbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1f3dca, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195f3dce, SUC_lexicon_tags_55 },
    { 0xf7bf3dce, SUC_lexicon_tags_33 },
    { 0xfcbf3dd0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x337f3dd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x115f3dd5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bf3dd8, SUC_lexicon_tags_49 },
    { 0x495f3dd9, SUC_lexicon_tags_19 },
    { 0x225f3dd8, SUC_lexicon_tags_21 },
    { 0xb37f3dd8, SUC_lexicon_tags_26 },
    { 0x871f3ddc, SUC_lexicon_tags_16 },
    { 0x49ff3ddd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5f3de0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5e1f3de2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585f3dea, SUC_lexicon_tags_17 },
    { 0x839f3deb, SUC_lexicon_tags_22 },
    { 0xf97f3dec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd0df3dee, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bf3df1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x757f3df4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21f3dfb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1df3dff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27f3e02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc19f3e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3f3e0e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9edf3e10, SUC_lexicon_tags_1 },
    { 0x901f3e10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51bf3e13, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5f3e19, SUC_lexicon_tags_15 },
    { 0x177f3e19, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x465f3e1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4eff3e1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3f3e23, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd41f3e25, SUC_lexicon_tags_8 },
    { 0x9e1f3e26, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4cdf3e28, SUC_lexicon_tags_12 },
    { 0xfcff3e28, SUC_lexicon_tags_14 },
    { 0x1a3f3e2a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bf3e2f, SUC_lexicon_tags_38 },
    { 0xcdbf3e30, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47f3e32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9f3e34, SUC_lexicon_tags_28 },
    { 0xcc9f3e35, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ff3e39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1f3e3e, SUC_lexicon_tags_105 },
    { 0xb39f3e3f, SUC_lexicon_tags_33 },
    { 0xb43f3e3e, SUC_lexicon_tags_49 },
    { 0x955f3e41, SUC_lexicon_tags_7 },
    { 0x581f3e42, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x367f3e44, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe0df3e46, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3f3e4a, SUC_lexicon_tags_43 },
    { 0x4dff3e4a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8dff3e4d, SUC_lexicon_tags_37 },
    { 0x39ff3e4e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x40bf3e51, SUC_lexicon_tags_47 },
    { 0x21df3e52, SUC_lexicon_tags_1 },
    { 0xa25f3e53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe79f3e58, SUC_lexicon_tags_45 },
    { 0xfbff3e59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x783f3e5c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf89f3e5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x791f3e60, SUC_lexicon_tags_45 },
    { 0x7b7f3e61, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5f3e64, SUC_lexicon_tags_11 },
    { 0x127f3e65, SUC_lexicon_tags_15 },
    { 0xe45f3e64, SUC_lexicon_tags_0 },
    { 0x539f3e67, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99f3e74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f3e78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x301f3e7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c3f3e7c, SUC_lexicon_tags_65 },
    { 0x207f3e7d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x45bf3e7f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c9f3e81, SUC_lexicon_tags_2 },
    { 0x153f3e81, SUC_lexicon_tags_27 },
    { 0x391f3e81, SUC_lexicon_tags_30 },
    { 0x15df3e84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9f3e89, SUC_lexicon_tags_43 },
    { 0x2adf3e8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48ff3e92, SUC_lexicon_tags_2 },
    { 0xb5ff3e93, SUC_lexicon_tags_11 },
    { 0x267f3e94, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x875f3e96, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7f3e9c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x381f3e9e, SUC_lexicon_tags_27 },
    { 0xa95f3e9f, SUC_lexicon_tags_25 },
    { 0x983f3e9e, SUC_lexicon_tags_15 },
    { 0x529f3ea0, SUC_lexicon_tags_17 },
    { 0x151f3e9f, SUC_lexicon_tags_33 },
    { 0x67ff3e9e, SUC_lexicon_tags_6 },
    { 0x4e3f3e9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81f3ea8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x609f3eaa, SUC_lexicon_tags_29 },
    { 0x2adf3eab, SUC_lexicon_tags_25 },
    { 0xa5f3eab, SUC_lexicon_tags_25 },
    { 0xa9bf3ead, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbf9f3eaf, SUC_lexicon_tags_56 },
    { 0x6fff3eb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9f3eb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf91f3eb6, SUC_lexicon_tags_37 },
    { 0x25bf3eb7, SUC_lexicon_tags_11 },
    { 0x617f3eb6, SUC_lexicon_tags_11 },
    { 0x689f3eb7, SUC_lexicon_tags_37 },
    { 0xe0bf3eb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3f3ebd, SUC_lexicon_tags_60 },
    { 0x8c7f3ebe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x691f3ec1, SUC_lexicon_tags_25 },
    { 0xdabf3ec2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abf3eca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3f3ecf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabf3ed6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x453f3ed8, SUC_lexicon_tags_29 },
    { 0x351f3ed8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15f3edd, SUC_lexicon_tags_25 },
    { 0x845f3ede, SUC_lexicon_tags_14 },
    { 0xfd7f3ede, SUC_lexicon_tags_1 },
    { 0xcbff3ee0, SUC_lexicon_tags_7 },
    { 0x7eff3ee0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5f3ee4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x881f3ee6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ff3ee9, SUC_lexicon_tags_25 },
    { 0x77bf3ee9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9f3eed, SUC_lexicon_tags_12 },
    { 0x7e5f3eee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x625f3ef1, SUC_lexicon_tags_9 },
    { 0x11df3ef1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207f3eff, SUC_lexicon_tags_49 },
    { 0xc73f3f00, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x669f3f03, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3df3f05, SUC_lexicon_tags_12 },
    { 0x76bf3f06, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353f3f0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73f3f0c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedf3f10, SUC_lexicon_tags_14 },
    { 0xd7df3f10, SUC_lexicon_tags_15 },
    { 0x85f3f12, SUC_lexicon_tags_43 },
    { 0x7f3f3f12, SUC_lexicon_tags_12 },
    { 0xc77f3f13, SUC_lexicon_tags_49 },
    { 0x173f3f11, SUC_lexicon_tags_33 },
    { 0xcebf3f10, SUC_lexicon_tags_12 },
    { 0x523f3f17, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7f3f1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ff3f29, SUC_lexicon_tags_11 },
    { 0x61f3f2a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fff3f36, SUC_lexicon_tags_47 },
    { 0x3e7f3f36, SUC_lexicon_tags_201 },
    { 0x8a9f3f37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1bff3f3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89f3f40, SUC_lexicon_tags_17 },
    { 0x8e3f3f41, SUC_lexicon_tags_37 },
    { 0xf6ff3f42, SUC_lexicon_tags_33 },
    { 0xc87f3f42, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bf3f47, SUC_lexicon_tags_23 },
    { 0xe7df3f47, SUC_lexicon_tags_7 },
    { 0xa03f3f49, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2d1f3f4b, SUC_lexicon_tags_12 },
    { 0x8dff3f4c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x48df3f4e, SUC_lexicon_tags_31 },
    { 0x92df3f4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5abf3f51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57df3f65, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3f3f6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53f3f71, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x9bf3f73, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeedf3f75, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x577f3f77, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x60bf3f79, SUC_lexicon_tags_30 },
    { 0x80df3f7a, SUC_lexicon_tags_17 },
    { 0x6dff3f79, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1cbf3f7d, SUC_lexicon_tags_47 },
    { 0x1d5f3f7e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f3f82, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909f3f86, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xbadf3f88, SUC_lexicon_tags_27 },
    { 0x35f3f89, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1f7f3f8b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853f3f8f, SUC_lexicon_tags_37 },
    { 0x3a5f3f90, SUC_lexicon_tags_47 },
    { 0x47ff3f90, SUC_lexicon_tags_33 },
    { 0x40df3f92, SUC_lexicon_tags_1 },
    { 0x83df3f92, SUC_lexicon_tags_25 },
    { 0xe1f3f90, SUC_lexicon_tags_43 },
    { 0xeb1f3f95, SUC_lexicon_tags_29 },
    { 0xb1df3f96, SUC_lexicon_tags_33 },
    { 0x719f3f96, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa99f3f99, SUC_lexicon_tags_27 },
    { 0x597f3f9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5f3f9f, SUC_lexicon_tags_31 },
    { 0x3fff3f9f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523f3fa4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493f3fa9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31f3faf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75f3fb2, SUC_lexicon_tags_9 },
    { 0x259f3fb3, SUC_lexicon_tags_1 },
    { 0x50df3fb4, SUC_lexicon_tags_63 },
    { 0xfe7f3fb4, SUC_lexicon_tags_26 },
    { 0x56ff3fb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x607f3fb9, SUC_lexicon_tags_0 },
    { 0x39f3fba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeabf3fbd, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93f3fc1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbf3fc7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb9ff3fc9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11f3fcd, SUC_lexicon_tags_45 },
    { 0x4f7f3fce, SUC_lexicon_tags_17 },
    { 0x5adf3fcf, SUC_lexicon_tags_17 },
    { 0xf2bf3fd0, SUC_lexicon_tags_30 },
    { 0x833f3fd1, SUC_lexicon_tags_43 },
    { 0x86bf3fd2, SUC_lexicon_tags_79 },
    { 0xb9df3fd0, SUC_lexicon_tags_29 },
    { 0x363f3fd4, SUC_lexicon_tags_19 },
    { 0xdb3f3fd4, SUC_lexicon_tags_220 },
    { 0x8a5f3fd5, SUC_lexicon_tags_14 },
    { 0xd9df3fd7, SUC_lexicon_tags_45 },
    { 0x5d5f3fd5, SUC_lexicon_tags_37 },
    { 0x67bf3fd1, SUC_lexicon_tags_39 },
    { 0x40df3fda, SUC_lexicon_tags_37 },
    { 0xd2df3fd3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x183f3fde, SUC_lexicon_tags_25 },
    { 0x949f3fdf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xea3f3fe2, SUC_lexicon_tags_16 },
    { 0x793f3fe3, SUC_lexicon_tags_29 },
    { 0x457f3fe4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf49f3fe6, SUC_lexicon_tags_27 },
    { 0xf8bf3fe7, SUC_lexicon_tags_198 },
    { 0xf97f3fe8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x933f3fea, SUC_lexicon_tags_27 },
    { 0xbeff3feb, SUC_lexicon_tags_37 },
    { 0xa6df3fec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff3ff7, SUC_lexicon_tags_151 },
    { 0x189f3ff8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9f3ffb, SUC_lexicon_tags_25 },
    { 0xad5f3ffb, SUC_lexicon_tags_11 },
    { 0xa61f3ffd, SUC_lexicon_tags_12 },
    { 0x1d3f3ffe, SUC_lexicon_tags_13 },
    { 0x48bf3ffe, SUC_lexicon_tags_27 },
    { 0x595f4000, SUC_lexicon_tags_14 },
    { 0x823f4000, SUC_lexicon_tags_10 },
    { 0x19df4002, SUC_lexicon_tags_1 },
    { 0x7abf4000, SUC_lexicon_tags_68 },
    { 0xcff4004, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x361f4006, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x315f4009, SUC_lexicon_tags_15 },
    { 0xb9df4009, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27f4011, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf65f4013, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf2bf4015, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x135f4017, SUC_lexicon_tags_7 },
    { 0x645f4018, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5b1f401a, SUC_lexicon_tags_46 },
    { 0x90ff401a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x19f401e, SUC_lexicon_tags_60 },
    { 0x645f401f, SUC_lexicon_tags_25 },
    { 0x217f401e, SUC_lexicon_tags_79 },
    { 0x983f4020, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf37f4023, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd63f4025, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ff4028, SUC_lexicon_tags_62 },
    { 0x955f4028, SUC_lexicon_tags_2 },
    { 0x4e7f402a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x591f402c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bf4033, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x865f4035, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x357f4037, SUC_lexicon_tags_6 },
    { 0x299f4037, SUC_lexicon_tags_37 },
    { 0xd1f4037, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x189f403d, SUC_lexicon_tags_49 },
    { 0xbaff403e, SUC_lexicon_tags_33 },
    { 0xeeff403f, SUC_lexicon_tags_49 },
    { 0xfe1f4040, SUC_lexicon_tags_47 },
    { 0xb3ff4040, SUC_lexicon_tags_108 },
    { 0x563f4040, SUC_lexicon_tags_26 },
    { 0x699f4043, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xadbf4046, SUC_lexicon_tags_2 },
    { 0x703f4047, SUC_lexicon_tags_27 },
    { 0xa99f4048, SUC_lexicon_tags_47 },
    { 0xfc5f4046, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x695f404b, SUC_lexicon_tags_25 },
    { 0xc83f404c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x647f404f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebf4053, SUC_lexicon_tags_71 },
    { 0xd5ff4054, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bf4057, SUC_lexicon_tags_19 },
    { 0x2f3f4058, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5f405f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83f4065, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe03f4067, SUC_lexicon_tags_25 },
    { 0x11bf4068, SUC_lexicon_tags_17 },
    { 0x46bf4068, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6df406c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x597f406f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75f4072, SUC_lexicon_tags_6 },
    { 0xe2df4072, SUC_lexicon_tags_23 },
    { 0x4d7f4073, SUC_lexicon_tags_25 },
    { 0x5bf4074, SUC_lexicon_tags_12 },
    { 0xf5f4076, SUC_lexicon_tags_11 },
    { 0x549f4075, SUC_lexicon_tags_178 },
    { 0x559f4078, SUC_lexicon_tags_37 },
    { 0xf15f4078, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5bf407b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5df407d, SUC_lexicon_tags_33 },
    { 0xabdf407e, SUC_lexicon_tags_37 },
    { 0x947f407f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f4082, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x149f4085, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3adf4087, SUC_lexicon_tags_17 },
    { 0x521f4087, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe25f408a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141f408e, SUC_lexicon_tags_37 },
    { 0x94ff408e, SUC_lexicon_tags_108 },
    { 0x707f4090, SUC_lexicon_tags_37 },
    { 0xc93f4091, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfdf4094, SUC_lexicon_tags_47 },
    { 0x24df4094, SUC_lexicon_tags_47 },
    { 0x337f4096, SUC_lexicon_tags_38 },
    { 0x2d5f4097, SUC_lexicon_tags_17 },
    { 0x311f4098, SUC_lexicon_tags_17 },
    { 0x65df4099, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x6cdf409b, SUC_lexicon_tags_2 },
    { 0xa27f409c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x993f409e, SUC_lexicon_tags_19 },
    { 0x11ff409f, SUC_lexicon_tags_23 },
    { 0x1eff40a0, SUC_lexicon_tags_25 },
    { 0xe7bf409e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x723f40a4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5f40aa, SUC_lexicon_tags_31 },
    { 0x781f40aa, SUC_lexicon_tags_17 },
    { 0x4c1f40ac, SUC_lexicon_tags_12 },
    { 0xc4bf40ad, SUC_lexicon_tags_15 },
    { 0xc51f40ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x867f40b1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3f40b6, SUC_lexicon_tags_11 },
    { 0x73f40b7, SUC_lexicon_tags_37 },
    { 0x71bf40b6, SUC_lexicon_tags_48 },
    { 0x607f40b6, SUC_lexicon_tags_11 },
    { 0x9f5f40ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x67df40bc, SUC_lexicon_tags_170 },
    { 0xf9df40bd, SUC_lexicon_tags_25 },
    { 0xdbff40be, SUC_lexicon_tags_17 },
    { 0xba7f40bf, SUC_lexicon_tags_19 },
    { 0xc65f40bc, SUC_lexicon_tags_28 },
    { 0x727f40be, SUC_lexicon_tags_16 },
    { 0x8d1f40bf, SUC_lexicon_tags_7 },
    { 0xe81f40bc, SUC_lexicon_tags_6 },
    { 0xfa1f40bc, SUC_lexicon_tags_84 },
    { 0xa2ff40c5, SUC_lexicon_tags_11 },
    { 0x775f40c6, SUC_lexicon_tags_37 },
    { 0x513f40c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33bf40cb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7df40cf, SUC_lexicon_tags_49 },
    { 0x1abf40d0, SUC_lexicon_tags_12 },
    { 0x1b3f40d1, SUC_lexicon_tags_12 },
    { 0xa5bf40cf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x771f40d4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2bdf40d6, SUC_lexicon_tags_23 },
    { 0xe29f40d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x261f40d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x181f40dc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7bf40de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19f40e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x921f40e5, SUC_lexicon_tags_17 },
    { 0x407f40e5, SUC_lexicon_tags_49 },
    { 0xafbf40e6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7f40eb, SUC_lexicon_tags_11 },
    { 0xbc9f40ec, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xc29f40ee, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9f40f6, SUC_lexicon_tags_15 },
    { 0x2abf40f7, SUC_lexicon_tags_2 },
    { 0xcb5f40f8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xf8ff40fa, SUC_lexicon_tags_2 },
    { 0x493f40fb, SUC_lexicon_tags_2 },
    { 0x105f40fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb35f40ff, SUC_lexicon_tags_49 },
    { 0x215f4100, SUC_lexicon_tags_1 },
    { 0xd9ff4100, SUC_lexicon_tags_15 },
    { 0xc17f4102, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4a9f4104, SUC_lexicon_tags_140 },
    { 0xe77f4104, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb65f4107, SUC_lexicon_tags_11 },
    { 0xc43f4108, SUC_lexicon_tags_10 },
    { 0x6f9f4107, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183f410d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddf4110, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xf09f4112, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xd07f4114, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb53f411a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceff4120, SUC_lexicon_tags_18 },
    { 0x74df4121, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377f4125, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x573f4128, SUC_lexicon_tags_23 },
    { 0xf5df4129, SUC_lexicon_tags_16 },
    { 0xff3f412a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5f412d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x71bf4130, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x5c3f4132, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211f413b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x79ff413e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79f4142, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa81f4144, SUC_lexicon_tags_31 },
    { 0x5d5f4145, SUC_lexicon_tags_17 },
    { 0xa5f4144, SUC_lexicon_tags_45 },
    { 0xe57f4146, SUC_lexicon_tags_7 },
    { 0x9a7f4146, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd7f414a, SUC_lexicon_tags_82 },
    { 0x635f414a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63df4150, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105f4155, SUC_lexicon_tags_45 },
    { 0x2bdf4155, SUC_lexicon_tags_79 },
    { 0x2cbf4155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab3f4159, SUC_lexicon_tags_15 },
    { 0xb41f4159, SUC_lexicon_tags_14 },
    { 0x7bdf415b, SUC_lexicon_tags_2 },
    { 0x9fff415b, SUC_lexicon_tags_27 },
    { 0x571f415d, SUC_lexicon_tags_33 },
    { 0xb53f4159, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x273f4161, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9f4164, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5f4169, SUC_lexicon_tags_37 },
    { 0x853f416a, SUC_lexicon_tags_9 },
    { 0x831f416b, SUC_lexicon_tags_27 },
    { 0xdd3f416b, SUC_lexicon_tags_11 },
    { 0xd67f416d, SUC_lexicon_tags_40 },
    { 0x7c3f416a, SUC_lexicon_tags_45 },
    { 0xa17f416f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783f4177, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f417b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bf4180, SUC_lexicon_tags_25 },
    { 0x8a7f4180, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff3f4186, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5f418a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2b7f418c, SUC_lexicon_tags_32 },
    { 0xa15f418d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xc6bf418f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x575f4192, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x63f4194, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0df4197, SUC_lexicon_tags_9 },
    { 0x8d3f4198, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bff419c, SUC_lexicon_tags_137 },
    { 0x915f419d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x573f419f, SUC_lexicon_tags_2 },
    { 0x7ff419f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb33f41a4, SUC_lexicon_tags_1 },
    { 0x2b1f41a5, SUC_lexicon_tags_37 },
    { 0x53bf41a6, SUC_lexicon_tags_0 },
    { 0xfa1f41a5, SUC_lexicon_tags_10 },
    { 0x759f41a8, SUC_lexicon_tags_4 },
    { 0xcabf41a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5f41ae, SUC_lexicon_tags_17 },
    { 0x507f41af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bf41b4, SUC_lexicon_tags_39 },
    { 0xda5f41b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70df41b9, SUC_lexicon_tags_66 },
    { 0xa9ff41b9, SUC_lexicon_tags_61 },
    { 0xcc9f41bb, SUC_lexicon_tags_1 },
    { 0xc7bf41bb, SUC_lexicon_tags_267 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42df41c1, SUC_lexicon_tags_17 },
    { 0x527f41c1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329f41c6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241f41ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fff41d0, SUC_lexicon_tags_49 },
    { 0xef1f41d1, SUC_lexicon_tags_15 },
    { 0x9b5f41d2, SUC_lexicon_tags_14 },
    { 0x125f41d3, SUC_lexicon_tags_37 },
    { 0x94bf41d3, SUC_lexicon_tags_82 },
    { 0xef5f41d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x291f41d7, SUC_lexicon_tags_14 },
    { 0xc31f41d7, SUC_lexicon_tags_12 },
    { 0x991f41d9, SUC_lexicon_tags_2 },
    { 0x4c1f41da, SUC_lexicon_tags_29 },
    { 0x971f41db, SUC_lexicon_tags_14 },
    { 0xb27f41db, SUC_lexicon_tags_2 },
    { 0x2ddf41db, SUC_lexicon_tags_1 },
    { 0x423f41dd, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0x51f41e0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbf41e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x167f41e8, SUC_lexicon_tags_37 },
    { 0x72ff41e9, SUC_lexicon_tags_16 },
    { 0x371f41e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x739f41ec, SUC_lexicon_tags_26 },
    { 0xec7f41ed, SUC_lexicon_tags_12 },
    { 0x679f41ed, SUC_lexicon_tags_1 },
    { 0x605f41ec, SUC_lexicon_tags_13 },
    { 0x61f41ed, SUC_lexicon_tags_18 },
    { 0x3b5f41f0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7f41f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29bf4206, SUC_lexicon_tags_17 },
    { 0xedf4206, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x887f4209, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a3f420b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b9f420d, SUC_lexicon_tags_62 },
    { 0xa8bf420e, SUC_lexicon_tags_7 },
    { 0x539f420f, SUC_lexicon_tags_9 },
    { 0x8aff4210, SUC_lexicon_tags_47 },
    { 0xb93f4211, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f4216, SUC_lexicon_tags_32 },
    { 0xdf4216, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x329f4219, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1f422a, SUC_lexicon_tags_37 },
    { 0xbb5f422a, SUC_lexicon_tags_65 },
    { 0x689f422c, SUC_lexicon_tags_55 },
    { 0xf43f422d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb9f4232, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853f4238, SUC_lexicon_tags_7 },
    { 0x297f4239, SUC_lexicon_tags_17 },
    { 0xc93f4239, SUC_lexicon_tags_12 },
    { 0xe57f423b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x813f423d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x89df423f, SUC_lexicon_tags_19 },
    { 0xa1f423f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15df4245, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x57df4248, SUC_lexicon_tags_4 },
    { 0x7aff4249, SUC_lexicon_tags_37 },
    { 0xa6df424a, SUC_lexicon_tags_7 },
    { 0xe37f4248, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf9df424d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd53f4253, SUC_lexicon_tags_12 },
    { 0x99f4254, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xef7f4256, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103f425a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xad7f425c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x405f425e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163f4263, SUC_lexicon_tags_37 },
    { 0xdf7f4263, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81f4268, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x409f426a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5cdf426c, SUC_lexicon_tags_25 },
    { 0xa6df426d, SUC_lexicon_tags_63 },
    { 0x41bf426e, SUC_lexicon_tags_17 },
    { 0x363f426f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bff4272, SUC_lexicon_tags_68 },
    { 0x571f4273, SUC_lexicon_tags_4 },
    { 0x31bf4272, SUC_lexicon_tags_8 },
    { 0x5ebf4275, SUC_lexicon_tags_11 },
    { 0x6a9f4273, SUC_lexicon_tags_12 },
    { 0x33bf4277, SUC_lexicon_tags_1 },
    { 0xdd5f4277, SUC_lexicon_tags_63 },
    { 0xfbbf4279, SUC_lexicon_tags_9 },
    { 0x8cdf4279, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c1f427c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbf427f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871f4284, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b1f4286, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xee9f4288, SUC_lexicon_tags_7 },
    { 0x5cdf4289, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7f428c, SUC_lexicon_tags_34 },
    { 0xf51f428c, SUC_lexicon_tags_25 },
    { 0x1e5f428e, SUC_lexicon_tags_1 },
    { 0xb65f428e, SUC_lexicon_tags_17 },
    { 0xdcdf4290, SUC_lexicon_tags_21 },
    { 0xa03f428e, SUC_lexicon_tags_0 },
    { 0xf81f428e, SUC_lexicon_tags_17 },
    { 0x593f4293, SUC_lexicon_tags_14 },
    { 0x9c7f4294, SUC_lexicon_tags_7 },
    { 0x991f4295, SUC_lexicon_tags_63 },
    { 0x1ff4296, SUC_lexicon_tags_56 },
    { 0x4d7f4290, SUC_lexicon_tags_7 },
    { 0x6dff4292, SUC_lexicon_tags_5 },
    { 0x5b1f4291, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdff429d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x629f42a0, SUC_lexicon_tags_2 },
    { 0x2b9f42a1, SUC_lexicon_tags_25 },
    { 0x64ff42a1, SUC_lexicon_tags_1 },
    { 0x3fbf42a3, SUC_lexicon_tags_37 },
    { 0x4a7f42a4, SUC_lexicon_tags_14 },
    { 0xf4bf42a5, SUC_lexicon_tags_15 },
    { 0xb37f42a5, SUC_lexicon_tags_15 },
    { 0x2eff42a4, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x379f42aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x977f42ad, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x25f42af, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479f42b4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x139f42b7, SUC_lexicon_tags_4 },
    { 0x377f42b8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc2df42ba, SUC_lexicon_tags_2 },
    { 0x33f42ba, SUC_lexicon_tags_48 },
    { 0x83f42ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc55f42be, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaadf42c2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc2bf42c4, SUC_lexicon_tags_55 },
    { 0x3a5f42c4, SUC_lexicon_tags_15 },
    { 0x4c9f42c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedf42ca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x98ff42cd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x11f42d0, SUC_lexicon_tags_2 },
    { 0x29f42d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f42d4, SUC_lexicon_tags_21 },
    { 0x2adf42d5, SUC_lexicon_tags_38 },
    { 0x669f42d4, SUC_lexicon_tags_37 },
    { 0xda5f42d7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0df42da, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0x729f42dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745f42e3, SUC_lexicon_tags_7 },
    { 0x7fdf42e4, SUC_lexicon_tags_7 },
    { 0xde3f42e3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcaff42e7, SUC_lexicon_tags_1 },
    { 0x2adf42e7, SUC_lexicon_tags_23 },
    { 0x2f1f42e9, SUC_lexicon_tags_27 },
    { 0xe9df42e9, SUC_lexicon_tags_7 },
    { 0xcbbf42ea, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddf42f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1df42f5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9df42fa, SUC_lexicon_tags_43 },
    { 0x7edf42fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc99f42fd, SUC_lexicon_tags_19 },
    { 0x30bf42fd, SUC_lexicon_tags_47 },
    { 0x955f42ff, SUC_lexicon_tags_26 },
    { 0x301f4300, SUC_lexicon_tags_7 },
    { 0x701f4301, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aff4305, SUC_lexicon_tags_25 },
    { 0x583f4306, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3f4309, SUC_lexicon_tags_25 },
    { 0xee9f430a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9f430d, SUC_lexicon_tags_94 },
    { 0, NULL },
    { 0x14bf430f, SUC_lexicon_tags_282 },
    { 0x2a3f4310, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1f4316, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf93f4318, SUC_lexicon_tags_29 },
    { 0x943f4319, SUC_lexicon_tags_11 },
    { 0x7fbf4318, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9f4320, SUC_lexicon_tags_56 },
    { 0x8bf4321, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x98bf4323, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbcbf4325, SUC_lexicon_tags_49 },
    { 0xf7bf4325, SUC_lexicon_tags_7 },
    { 0x63df4327, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x711f432a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adf432f, SUC_lexicon_tags_28 },
    { 0xd59f432f, SUC_lexicon_tags_11 },
    { 0x353f4331, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x435f4333, SUC_lexicon_tags_60 },
    { 0xe81f4334, SUC_lexicon_tags_25 },
    { 0xb7df4333, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841f4339, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x923f433c, SUC_lexicon_tags_33 },
    { 0x71bf433c, SUC_lexicon_tags_19 },
    { 0x863f433c, SUC_lexicon_tags_45 },
    { 0xd29f433f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf4345, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x451f4348, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827f434e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x779f4350, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbf4353, SUC_lexicon_tags_7 },
    { 0x3cff4354, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc97f4359, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51f435c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3df435e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa09f4360, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57f4365, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x55bf4367, SUC_lexicon_tags_2 },
    { 0xdc1f4367, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x9ddf436a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff436d, SUC_lexicon_tags_7 },
    { 0xd31f436e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6df4371, SUC_lexicon_tags_37 },
    { 0xef9f4371, SUC_lexicon_tags_11 },
    { 0x115f4372, SUC_lexicon_tags_299 },
    { 0xcd9f4374, SUC_lexicon_tags_31 },
    { 0x86df4375, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92ff4379, SUC_lexicon_tags_45 },
    { 0x7d7f437a, SUC_lexicon_tags_6 },
    { 0x9e9f437b, SUC_lexicon_tags_37 },
    { 0x739f437b, SUC_lexicon_tags_45 },
    { 0x1d5f437d, SUC_lexicon_tags_8 },
    { 0x899f437c, SUC_lexicon_tags_25 },
    { 0x633f437f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129f4384, SUC_lexicon_tags_21 },
    { 0xe7bf4385, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdf4389, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3e7f438b, SUC_lexicon_tags_31 },
    { 0x191f438c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbf4390, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x16bf4392, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe3df4394, SUC_lexicon_tags_30 },
    { 0x34ff4395, SUC_lexicon_tags_1 },
    { 0x3c9f4395, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc13f4398, SUC_lexicon_tags_9 },
    { 0xd9df4398, SUC_lexicon_tags_39 },
    { 0x5d7f439a, SUC_lexicon_tags_25 },
    { 0xc37f439b, SUC_lexicon_tags_47 },
    { 0x10ff439c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ff439f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x609f43a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x829f43a4, SUC_lexicon_tags_12 },
    { 0x35df43a5, SUC_lexicon_tags_56 },
    { 0x493f43a4, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x59bf43a8, SUC_lexicon_tags_1 },
    { 0xbd3f43a9, SUC_lexicon_tags_2 },
    { 0x4e5f43aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6f1f43ac, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3ddf43ae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55f43b4, SUC_lexicon_tags_38 },
    { 0xc99f43b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x641f43b7, SUC_lexicon_tags_33 },
    { 0x9a5f43b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1f43bb, SUC_lexicon_tags_72 },
    { 0x15df43bb, SUC_lexicon_tags_6 },
    { 0xd67f43bd, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f43c3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf43c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xad1f43ca, SUC_lexicon_tags_14 },
    { 0xee7f43cb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bff43cf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x26bf43d1, SUC_lexicon_tags_37 },
    { 0x98ff43d2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x971f43d5, SUC_lexicon_tags_14 },
    { 0xafbf43d6, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc3df43d8, SUC_lexicon_tags_47 },
    { 0x707f43d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbf43dd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f43e3, SUC_lexicon_tags_15 },
    { 0x11bf43e4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559f43eb, SUC_lexicon_tags_43 },
    { 0xe3f43ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3eff43ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ff43f1, SUC_lexicon_tags_6 },
    { 0x7a1f43f2, SUC_lexicon_tags_37 },
    { 0xa43f43f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aff43f6, SUC_lexicon_tags_27 },
    { 0xf63f43f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x699f43ff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ff4406, SUC_lexicon_tags_54 },
    { 0xc05f4406, SUC_lexicon_tags_9 },
    { 0xf5ff4408, SUC_lexicon_tags_27 },
    { 0xa13f4409, SUC_lexicon_tags_12 },
    { 0x13bf440a, SUC_lexicon_tags_57 },
    { 0xcc1f4408, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x613f440e, SUC_lexicon_tags_39 },
    { 0x48bf440f, SUC_lexicon_tags_79 },
    { 0x567f4410, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbf4418, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901f441e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51bf4420, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f4426, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x829f4428, SUC_lexicon_tags_7 },
    { 0x96bf4429, SUC_lexicon_tags_27 },
    { 0xd25f442a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1f442d, SUC_lexicon_tags_7 },
    { 0x897f442e, SUC_lexicon_tags_39 },
    { 0x301f442e, SUC_lexicon_tags_47 },
    { 0x28df4430, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9f4435, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bf4438, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x55df443a, SUC_lexicon_tags_157 },
    { 0x113f443a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729f4440, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xd4df4442, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ff4446, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdf444a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x965f444c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc8ff444e, SUC_lexicon_tags_47 },
    { 0xe7ff444f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xde9f4451, SUC_lexicon_tags_65 },
    { 0xa13f4451, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x99f4454, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x1f4456, SUC_lexicon_tags_38 },
    { 0x465f4456, SUC_lexicon_tags_26 },
    { 0xc59f4456, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x957f445a, SUC_lexicon_tags_7 },
    { 0x7e3f445a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x689f445d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x857f4461, SUC_lexicon_tags_14 },
    { 0x645f4461, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xeebf4464, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8eff446a, SUC_lexicon_tags_61 },
    { 0x6cff446a, SUC_lexicon_tags_33 },
    { 0xad3f446a, SUC_lexicon_tags_33 },
    { 0x769f446d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x81f446f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x503f4472, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf87f4477, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf93f4479, SUC_lexicon_tags_15 },
    { 0x52df447a, SUC_lexicon_tags_1 },
    { 0x18ff447b, SUC_lexicon_tags_2 },
    { 0x25bf4479, SUC_lexicon_tags_82 },
    { 0x4d7f447d, SUC_lexicon_tags_60 },
    { 0xfcbf447a, SUC_lexicon_tags_17 },
    { 0xf93f447f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xeb1f4481, SUC_lexicon_tags_24 },
    { 0x751f4482, SUC_lexicon_tags_12 },
    { 0x783f4483, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5ff4485, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77df4487, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0df448c, SUC_lexicon_tags_2 },
    { 0xc9f448d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51f4490, SUC_lexicon_tags_45 },
    { 0x477f4491, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7f4495, SUC_lexicon_tags_13 },
    { 0x9ebf4495, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8b1f4498, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11f449e, SUC_lexicon_tags_15 },
    { 0xf71f449f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59f44a2, SUC_lexicon_tags_37 },
    { 0x9e1f44a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdfff44a5, SUC_lexicon_tags_7 },
    { 0x221f44a5, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xe63f44a8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911f44ad, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x943f44b0, SUC_lexicon_tags_7 },
    { 0xc51f44b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813f44b5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xea9f44b7, SUC_lexicon_tags_16 },
    { 0xaaff44b8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37f44c0, SUC_lexicon_tags_37 },
    { 0xc03f44c0, SUC_lexicon_tags_12 },
    { 0x7c3f44c1, SUC_lexicon_tags_66 },
    { 0x6d7f44c0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x60df44c5, SUC_lexicon_tags_27 },
    { 0x225f44c6, SUC_lexicon_tags_40 },
    { 0x80df44c6, SUC_lexicon_tags_17 },
    { 0x409f44c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc1f44cd, SUC_lexicon_tags_17 },
    { 0xe97f44ce, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a9f44d1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67df44d5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ff44da, SUC_lexicon_tags_15 },
    { 0x901f44db, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabf44df, SUC_lexicon_tags_14 },
    { 0xe35f44e0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965f44e4, SUC_lexicon_tags_27 },
    { 0xe0df44e5, SUC_lexicon_tags_15 },
    { 0xf05f44e6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xffdf44e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x98bf44eb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d5f44ed, SUC_lexicon_tags_17 },
    { 0xfa9f44ee, SUC_lexicon_tags_28 },
    { 0xbcdf44ef, SUC_lexicon_tags_46 },
    { 0x82bf44f0, SUC_lexicon_tags_29 },
    { 0xb91f44f1, SUC_lexicon_tags_15 },
    { 0x7a3f44f2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaff44f6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb3ff44f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x295f44fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243f44fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5b1f4500, SUC_lexicon_tags_11 },
    { 0x87bf4501, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x587f4504, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf4508, SUC_lexicon_tags_7 },
    { 0xdabf4509, SUC_lexicon_tags_37 },
    { 0xf9df450a, SUC_lexicon_tags_60 },
    { 0x3c7f450b, SUC_lexicon_tags_638 },
    { 0, NULL },
    { 0, NULL },
    { 0x40df450e, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x2b1f4510, SUC_lexicon_tags_2 },
    { 0x31bf4510, SUC_lexicon_tags_7 },
    { 0xfd5f4512, SUC_lexicon_tags_47 },
    { 0x731f4512, SUC_lexicon_tags_17 },
    { 0xaa1f4513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x41df4517, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe75f4519, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x121f451b, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38df4527, SUC_lexicon_tags_12 },
    { 0x11bf4528, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3f452d, SUC_lexicon_tags_2 },
    { 0xdedf452d, SUC_lexicon_tags_60 },
    { 0x17ff452f, SUC_lexicon_tags_45 },
    { 0xdcdf452f, SUC_lexicon_tags_27 },
    { 0xa69f452e, SUC_lexicon_tags_9 },
    { 0x29ff4532, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ff4538, SUC_lexicon_tags_2 },
    { 0x8bdf4539, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75f453c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e3f453e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x917f4540, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9f4544, SUC_lexicon_tags_37 },
    { 0x69df4545, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7edf4547, SUC_lexicon_tags_45 },
    { 0x547f4547, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443f4553, SUC_lexicon_tags_49 },
    { 0x1d7f4554, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9f455a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x29f455c, SUC_lexicon_tags_11 },
    { 0xacbf455d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x393f4560, SUC_lexicon_tags_7 },
    { 0xbb7f4561, SUC_lexicon_tags_11 },
    { 0x455f4562, SUC_lexicon_tags_11 },
    { 0xaf1f4563, SUC_lexicon_tags_15 },
    { 0xf9f4562, SUC_lexicon_tags_23 },
    { 0xacdf4565, SUC_lexicon_tags_43 },
    { 0x8d1f4565, SUC_lexicon_tags_33 },
    { 0x3b9f4566, SUC_lexicon_tags_25 },
    { 0x38bf4568, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6adf456a, SUC_lexicon_tags_25 },
    { 0xdbbf456b, SUC_lexicon_tags_25 },
    { 0x659f456c, SUC_lexicon_tags_10 },
    { 0xe83f456c, SUC_lexicon_tags_28 },
    { 0xa17f456c, SUC_lexicon_tags_37 },
    { 0xf9ff456c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x451f4572, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x95bf4574, SUC_lexicon_tags_12 },
    { 0x1e7f4574, SUC_lexicon_tags_60 },
    { 0xe6bf4574, SUC_lexicon_tags_12 },
    { 0xac9f4576, SUC_lexicon_tags_14 },
    { 0xba7f4578, SUC_lexicon_tags_82 },
    { 0x4bdf4575, SUC_lexicon_tags_19 },
    { 0xc61f457a, SUC_lexicon_tags_15 },
    { 0x769f457b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1f457f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7f4583, SUC_lexicon_tags_37 },
    { 0xb9ff4583, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf39f4586, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x155f4588, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7cff458a, SUC_lexicon_tags_37 },
    { 0x11bf458b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x877f458d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07f4590, SUC_lexicon_tags_45 },
    { 0x3abf4590, SUC_lexicon_tags_17 },
    { 0xa59f4592, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x449f4596, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xab5f4598, SUC_lexicon_tags_55 },
    { 0x183f4599, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bf459f, SUC_lexicon_tags_8 },
    { 0xc3df45a0, SUC_lexicon_tags_6 },
    { 0x47f45a1, SUC_lexicon_tags_2 },
    { 0xcd3f459f, SUC_lexicon_tags_25 },
    { 0x28df45a0, SUC_lexicon_tags_33 },
    { 0xd99f45a4, SUC_lexicon_tags_17 },
    { 0xa15f45a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5bf45a9, SUC_lexicon_tags_17 },
    { 0xa59f45a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf45ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479f45b7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x57df45b9, SUC_lexicon_tags_2 },
    { 0x55df45ba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9e1f45bc, SUC_lexicon_tags_12 },
    { 0x8c3f45bd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7f45c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x5e9f45c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x731f45c8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbff45ca, SUC_lexicon_tags_45 },
    { 0x38bf45cb, SUC_lexicon_tags_37 },
    { 0x39bf45cb, SUC_lexicon_tags_11 },
    { 0x8e1f45cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9ebf45cf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67f45d3, SUC_lexicon_tags_29 },
    { 0x6cff45d4, SUC_lexicon_tags_7 },
    { 0x115f45d5, SUC_lexicon_tags_33 },
    { 0xdebf45d3, SUC_lexicon_tags_25 },
    { 0x97ff45d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abf45df, SUC_lexicon_tags_17 },
    { 0x859f45e0, SUC_lexicon_tags_25 },
    { 0x2a5f45e1, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09f45e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7f45ea, SUC_lexicon_tags_63 },
    { 0x991f45ea, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bf45f0, SUC_lexicon_tags_14 },
    { 0x873f45f1, SUC_lexicon_tags_17 },
    { 0x6e7f45f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x795f45f5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769f45f9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f45fc, SUC_lexicon_tags_82 },
    { 0x5bdf45fc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xe6ff45ff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51f4602, SUC_lexicon_tags_12 },
    { 0xc7bf4603, SUC_lexicon_tags_43 },
    { 0xaf1f4604, SUC_lexicon_tags_11 },
    { 0x997f4605, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edf4608, SUC_lexicon_tags_37 },
    { 0x91f4609, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09f460d, SUC_lexicon_tags_63 },
    { 0xe1bf460d, SUC_lexicon_tags_25 },
    { 0x6c5f460d, SUC_lexicon_tags_11 },
    { 0x709f4610, SUC_lexicon_tags_25 },
    { 0xe13f4611, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411f4616, SUC_lexicon_tags_34 },
    { 0x86bf4617, SUC_lexicon_tags_4 },
    { 0x4dff4618, SUC_lexicon_tags_19 },
    { 0xb39f4619, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1df461e, SUC_lexicon_tags_11 },
    { 0xe91f461f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7f4623, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x219f4626, SUC_lexicon_tags_25 },
    { 0xb4bf4627, SUC_lexicon_tags_35 },
    { 0xa83f4627, SUC_lexicon_tags_49 },
    { 0x551f4629, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x229f462b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1f462f, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc81f4631, SUC_lexicon_tags_65 },
    { 0x7f1f4632, SUC_lexicon_tags_17 },
    { 0x1e1f4631, SUC_lexicon_tags_38 },
    { 0x19bf4634, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x49bf4636, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x203f4638, SUC_lexicon_tags_25 },
    { 0xcdbf4639, SUC_lexicon_tags_2 },
    { 0xe4df463a, SUC_lexicon_tags_17 },
    { 0xc7f463a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xb3bf463d, SUC_lexicon_tags_37 },
    { 0xd15f463e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dff4643, SUC_lexicon_tags_29 },
    { 0x9f1f4644, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x303f4646, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ff464a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09f464e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbf4655, SUC_lexicon_tags_0 },
    { 0x267f4656, SUC_lexicon_tags_9 },
    { 0xe0df4655, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x13df465a, SUC_lexicon_tags_15 },
    { 0x7a3f465b, SUC_lexicon_tags_37 },
    { 0x16ff465c, SUC_lexicon_tags_25 },
    { 0x1eff465c, SUC_lexicon_tags_15 },
    { 0x55ff465a, SUC_lexicon_tags_43 },
    { 0x143f465f, SUC_lexicon_tags_27 },
    { 0xbbff4660, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9f4663, SUC_lexicon_tags_12 },
    { 0x5ff4664, SUC_lexicon_tags_9 },
    { 0x8bff4664, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb07f4668, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bf466b, SUC_lexicon_tags_69 },
    { 0x98bf466c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x263f466f, SUC_lexicon_tags_12 },
    { 0xc97f4670, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1f4673, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a9f4675, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3f467b, SUC_lexicon_tags_11 },
    { 0x781f467c, SUC_lexicon_tags_46 },
    { 0x789f467d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe7bf467f, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eff4687, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaff468a, SUC_lexicon_tags_24 },
    { 0xc25f468b, SUC_lexicon_tags_15 },
    { 0x2cbf468a, SUC_lexicon_tags_3 },
    { 0x285f468d, SUC_lexicon_tags_63 },
    { 0x2cdf468d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x461f4691, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe5f4693, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2fff4695, SUC_lexicon_tags_10 },
    { 0x8c3f4696, SUC_lexicon_tags_1 },
    { 0xe63f4697, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33f469f, SUC_lexicon_tags_14 },
    { 0x6a9f46a0, SUC_lexicon_tags_60 },
    { 0xef7f46a0, SUC_lexicon_tags_2 },
    { 0x135f46a1, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x23df46a4, SUC_lexicon_tags_11 },
    { 0x8f1f46a4, SUC_lexicon_tags_64 },
    { 0x101f46a6, SUC_lexicon_tags_19 },
    { 0xc17f46a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5f46aa, SUC_lexicon_tags_17 },
    { 0x89f46ab, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbaff46ad, SUC_lexicon_tags_55 },
    { 0x7a1f46ae, SUC_lexicon_tags_61 },
    { 0xf21f46af, SUC_lexicon_tags_40 },
    { 0xb0bf46af, SUC_lexicon_tags_9 },
    { 0x479f46af, SUC_lexicon_tags_11 },
    { 0x8a7f46ad, SUC_lexicon_tags_47 },
    { 0x78bf46b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3f46b7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1f46bd, SUC_lexicon_tags_6 },
    { 0x145f46bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b1f46c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x653f46c5, SUC_lexicon_tags_15 },
    { 0xb09f46c6, SUC_lexicon_tags_12 },
    { 0x577f46c7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xe05f46c9, SUC_lexicon_tags_47 },
    { 0x6c1f46ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695f46d4, SUC_lexicon_tags_8 },
    { 0x673f46d5, SUC_lexicon_tags_43 },
    { 0xeb3f46d4, SUC_lexicon_tags_19 },
    { 0xe3bf46d7, SUC_lexicon_tags_17 },
    { 0xfc3f46d8, SUC_lexicon_tags_37 },
    { 0x6b9f46d5, SUC_lexicon_tags_12 },
    { 0xc75f46d9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375f46e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7f46e3, SUC_lexicon_tags_25 },
    { 0x2fff46e3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x809f46e6, SUC_lexicon_tags_68 },
    { 0xde5f46e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945f46ec, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbf46f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bf46f4, SUC_lexicon_tags_29 },
    { 0x51df46f5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x48ff46f7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8aff46fa, SUC_lexicon_tags_158 },
    { 0xe73f46fb, SUC_lexicon_tags_7 },
    { 0x35ff46fb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x82ff46fe, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35f4701, SUC_lexicon_tags_16 },
    { 0xbfff4702, SUC_lexicon_tags_12 },
    { 0x3bff4703, SUC_lexicon_tags_2 },
    { 0xe95f4702, SUC_lexicon_tags_1 },
    { 0xf27f4705, SUC_lexicon_tags_15 },
    { 0x87ff4705, SUC_lexicon_tags_60 },
    { 0xe53f4707, SUC_lexicon_tags_38 },
    { 0xdcbf4708, SUC_lexicon_tags_14 },
    { 0x7c7f4706, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbf4712, SUC_lexicon_tags_723 },
    { 0, NULL },
    { 0xe05f4714, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917f471a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x693f471c, SUC_lexicon_tags_11 },
    { 0x15ff471d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751f472d, SUC_lexicon_tags_1 },
    { 0x505f472d, SUC_lexicon_tags_1 },
    { 0x1d3f472f, SUC_lexicon_tags_1 },
    { 0xb2ff472f, SUC_lexicon_tags_2 },
    { 0x7c9f4730, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657f4735, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4a1f4737, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f473f, SUC_lexicon_tags_6 },
    { 0x19ff4740, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ff4744, SUC_lexicon_tags_10 },
    { 0xa11f4744, SUC_lexicon_tags_45 },
    { 0xae1f4746, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54ff474e, SUC_lexicon_tags_14 },
    { 0x48df474f, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9f4753, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaff4757, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x18ff4759, SUC_lexicon_tags_11 },
    { 0x9fdf4759, SUC_lexicon_tags_33 },
    { 0x557f475b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x817f475d, SUC_lexicon_tags_7 },
    { 0x705f475e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37f4762, SUC_lexicon_tags_32 },
    { 0x6c9f4763, SUC_lexicon_tags_34 },
    { 0x91bf4764, SUC_lexicon_tags_37 },
    { 0x781f4765, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9f476a, SUC_lexicon_tags_17 },
    { 0x921f476b, SUC_lexicon_tags_25 },
    { 0x413f476c, SUC_lexicon_tags_17 },
    { 0xc93f476d, SUC_lexicon_tags_37 },
    { 0x4e7f476e, SUC_lexicon_tags_15 },
    { 0x12ff476f, SUC_lexicon_tags_47 },
    { 0x575f4770, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x18bf4773, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xaaff4775, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xffff4777, SUC_lexicon_tags_15 },
    { 0x9e7f4778, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa15f477e, SUC_lexicon_tags_8 },
    { 0x427f477f, SUC_lexicon_tags_27 },
    { 0xb0bf477e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27f478a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5f4792, SUC_lexicon_tags_12 },
    { 0x1b1f4793, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bff4799, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xab5f479b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x795f479d, SUC_lexicon_tags_37 },
    { 0x583f479e, SUC_lexicon_tags_33 },
    { 0xc4df479f, SUC_lexicon_tags_48 },
    { 0x75ff47a0, SUC_lexicon_tags_38 },
    { 0x119f479f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x38df47a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5f47a8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe27f47aa, SUC_lexicon_tags_62 },
    { 0x92bf47ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x409f47b1, SUC_lexicon_tags_37 },
    { 0xf05f47b2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bf47b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4aff47b7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdcdf47b9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb7f47bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x925f47bd, SUC_lexicon_tags_58 },
    { 0xf5f47bd, SUC_lexicon_tags_23 },
    { 0x499f47bf, SUC_lexicon_tags_17 },
    { 0xf39f47c0, SUC_lexicon_tags_33 },
    { 0x377f47bd, SUC_lexicon_tags_1 },
    { 0x565f47be, SUC_lexicon_tags_17 },
    { 0xddf47be, SUC_lexicon_tags_129 },
    { 0xcbff47c4, SUC_lexicon_tags_37 },
    { 0x331f47bf, SUC_lexicon_tags_7 },
    { 0x8d1f47c1, SUC_lexicon_tags_17 },
    { 0xf3bf47c7, SUC_lexicon_tags_30 },
    { 0x29f47bd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x835f47ca, SUC_lexicon_tags_6 },
    { 0x15bf47ca, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xd2ff47cd, SUC_lexicon_tags_17 },
    { 0x491f47cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bf47d5, SUC_lexicon_tags_27 },
    { 0x675f47d6, SUC_lexicon_tags_55 },
    { 0xdc7f47d7, SUC_lexicon_tags_1 },
    { 0x481f47d7, SUC_lexicon_tags_31 },
    { 0x4edf47d7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53f47e2, SUC_lexicon_tags_32 },
    { 0xa5ff47e3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcc7f47e5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9f47f4, SUC_lexicon_tags_11 },
    { 0x965f47f5, SUC_lexicon_tags_51 },
    { 0xcb7f47f6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1c3f47f8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x271f47fb, SUC_lexicon_tags_37 },
    { 0x3e1f47fc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169f4800, SUC_lexicon_tags_27 },
    { 0x3bdf4801, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0xa65f4803, SUC_lexicon_tags_12 },
    { 0xfd1f4804, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6eff4808, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60bf480c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xabbf480f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ff4812, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x87ff4814, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bf4818, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05f481c, SUC_lexicon_tags_68 },
    { 0x731f481d, SUC_lexicon_tags_2 },
    { 0xce3f481e, SUC_lexicon_tags_55 },
    { 0x6bbf481f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x7b7f4821, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdf4825, SUC_lexicon_tags_12 },
    { 0xc1f4826, SUC_lexicon_tags_17 },
    { 0xc87f4827, SUC_lexicon_tags_37 },
    { 0x725f4828, SUC_lexicon_tags_60 },
    { 0xc93f4827, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96df482e, SUC_lexicon_tags_87 },
    { 0x24bf482e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591f4838, SUC_lexicon_tags_4 },
    { 0x5d5f4839, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ff483e, SUC_lexicon_tags_40 },
    { 0xf9bf483e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x767f4841, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x725f4844, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x561f484a, SUC_lexicon_tags_37 },
    { 0xc3ff484b, SUC_lexicon_tags_29 },
    { 0x88bf484b, SUC_lexicon_tags_25 },
    { 0xc1bf484d, SUC_lexicon_tags_10 },
    { 0x73ff484e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abf4852, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x34ff4856, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1b1f4858, SUC_lexicon_tags_15 },
    { 0xc1ff4859, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ff485d, SUC_lexicon_tags_17 },
    { 0xf57f485d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfcff4860, SUC_lexicon_tags_25 },
    { 0x54ff4861, SUC_lexicon_tags_37 },
    { 0xd6ff4861, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3f4868, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24df486e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77f4871, SUC_lexicon_tags_38 },
    { 0x983f4871, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471f4876, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d3f487b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adf4886, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309f488b, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xa91f488d, SUC_lexicon_tags_61 },
    { 0x999f488d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x59f4890, SUC_lexicon_tags_15 },
    { 0x1f9f4891, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xde9f4893, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9f4897, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x89bf4899, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbf489d, SUC_lexicon_tags_11 },
    { 0x617f489e, SUC_lexicon_tags_1 },
    { 0xf0bf489f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x985f48a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x187f48a7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251f48af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f48b3, SUC_lexicon_tags_11 },
    { 0x259f48b4, SUC_lexicon_tags_43 },
    { 0x5bdf48b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfebf48b9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1f48bf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xc51f48c1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x75ff48c3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x511f48c5, SUC_lexicon_tags_12 },
    { 0x74df48c5, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99f48cb, SUC_lexicon_tags_32 },
    { 0x1c5f48cc, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5b1f48ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25f48da, SUC_lexicon_tags_60 },
    { 0x9d7f48db, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90df48df, SUC_lexicon_tags_37 },
    { 0x513f48e0, SUC_lexicon_tags_0 },
    { 0xf77f48e1, SUC_lexicon_tags_0 },
    { 0x33f48e2, SUC_lexicon_tags_25 },
    { 0xfabf48e1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x66df48e5, SUC_lexicon_tags_1 },
    { 0x8fbf48e5, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0xffbf48e8, SUC_lexicon_tags_26 },
    { 0xe6df48e9, SUC_lexicon_tags_26 },
    { 0x24ff48ea, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xefff48ec, SUC_lexicon_tags_6 },
    { 0xf53f48ec, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x9abf48f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabf48fb, SUC_lexicon_tags_45 },
    { 0xffdf48fc, SUC_lexicon_tags_8 },
    { 0x72df48fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xaf1f48ff, SUC_lexicon_tags_32 },
    { 0x2b1f48ff, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4df4904, SUC_lexicon_tags_31 },
    { 0xc79f4905, SUC_lexicon_tags_12 },
    { 0x465f4905, SUC_lexicon_tags_1 },
    { 0x79ff4907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485f490d, SUC_lexicon_tags_2 },
    { 0xaa3f490e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5f4913, SUC_lexicon_tags_27 },
    { 0x245f4913, SUC_lexicon_tags_12 },
    { 0xa95f4915, SUC_lexicon_tags_8 },
    { 0x3c9f4915, SUC_lexicon_tags_12 },
    { 0x761f4916, SUC_lexicon_tags_25 },
    { 0x597f4918, SUC_lexicon_tags_11 },
    { 0x6a3f4913, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x88df491b, SUC_lexicon_tags_25 },
    { 0x161f491c, SUC_lexicon_tags_33 },
    { 0x87bf491c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4f9f491f, SUC_lexicon_tags_37 },
    { 0x93ff4920, SUC_lexicon_tags_14 },
    { 0x41ff491f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbf4924, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddf492f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503f4936, SUC_lexicon_tags_87 },
    { 0x505f4937, SUC_lexicon_tags_13 },
    { 0xbddf4937, SUC_lexicon_tags_11 },
    { 0xa5ff4936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x655f493c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x539f493f, SUC_lexicon_tags_0 },
    { 0x8c5f493f, SUC_lexicon_tags_1 },
    { 0x7b5f493f, SUC_lexicon_tags_8 },
    { 0x69df4941, SUC_lexicon_tags_1 },
    { 0xe1df4942, SUC_lexicon_tags_23 },
    { 0xc8ff4944, SUC_lexicon_tags_10 },
    { 0xbc3f4943, SUC_lexicon_tags_7 },
    { 0xbc1f4946, SUC_lexicon_tags_14 },
    { 0x79ff4947, SUC_lexicon_tags_17 },
    { 0xd31f4947, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8df494b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b9f494d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdeff494f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x99ff4951, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeff4958, SUC_lexicon_tags_262 },
    { 0, NULL },
    { 0xf79f495a, SUC_lexicon_tags_15 },
    { 0x47ff495b, SUC_lexicon_tags_4 },
    { 0x5d7f495a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f7f495e, SUC_lexicon_tags_25 },
    { 0x495f495e, SUC_lexicon_tags_37 },
    { 0x2b1f495f, SUC_lexicon_tags_7 },
    { 0x257f4961, SUC_lexicon_tags_33 },
    { 0x511f4962, SUC_lexicon_tags_6 },
    { 0xe53f4963, SUC_lexicon_tags_25 },
    { 0x761f4964, SUC_lexicon_tags_7 },
    { 0x6c7f4965, SUC_lexicon_tags_17 },
    { 0x227f4964, SUC_lexicon_tags_25 },
    { 0x221f4967, SUC_lexicon_tags_14 },
    { 0x565f4963, SUC_lexicon_tags_54 },
    { 0xf3ff4969, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43f496f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1f4979, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ff497d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5bf497f, SUC_lexicon_tags_10 },
    { 0x205f4980, SUC_lexicon_tags_4 },
    { 0x281f4981, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3f4984, SUC_lexicon_tags_15 },
    { 0x8a1f4985, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb81f4987, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4fff4989, SUC_lexicon_tags_30 },
    { 0x79df4989, SUC_lexicon_tags_17 },
    { 0x3a3f498b, SUC_lexicon_tags_37 },
    { 0x337f498b, SUC_lexicon_tags_31 },
    { 0x78ff498d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddf4994, SUC_lexicon_tags_11 },
    { 0xb93f4995, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9f49a0, SUC_lexicon_tags_17 },
    { 0x34bf49a1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3f49a4, SUC_lexicon_tags_25 },
    { 0x3d5f49a5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3f49aa, SUC_lexicon_tags_12 },
    { 0xe11f49ab, SUC_lexicon_tags_56 },
    { 0xaa5f49aa, SUC_lexicon_tags_31 },
    { 0x42bf49ad, SUC_lexicon_tags_7 },
    { 0xfe1f49ae, SUC_lexicon_tags_17 },
    { 0x69bf49ae, SUC_lexicon_tags_2 },
    { 0x1aff49af, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3f49b6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9f49bb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f49be, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5f49c1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf49c5, SUC_lexicon_tags_38 },
    { 0xe6df49c6, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bf49c9, SUC_lexicon_tags_47 },
    { 0x3fff49ca, SUC_lexicon_tags_14 },
    { 0xf99f49c9, SUC_lexicon_tags_43 },
    { 0x935f49cc, SUC_lexicon_tags_48 },
    { 0xcc5f49cd, SUC_lexicon_tags_2 },
    { 0xbc7f49cb, SUC_lexicon_tags_7 },
    { 0xd11f49ca, SUC_lexicon_tags_123 },
    { 0x3df49cb, SUC_lexicon_tags_25 },
    { 0xc2bf49d1, SUC_lexicon_tags_37 },
    { 0xc69f49d0, SUC_lexicon_tags_7 },
    { 0x697f49d3, SUC_lexicon_tags_70 },
    { 0xb1bf49d3, SUC_lexicon_tags_26 },
    { 0xeddf49d4, SUC_lexicon_tags_45 },
    { 0x933f49c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf81f49d8, SUC_lexicon_tags_17 },
    { 0xd79f49d9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dff49dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3f49e1, SUC_lexicon_tags_8 },
    { 0x1df49e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129f49ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb6df49ec, SUC_lexicon_tags_27 },
    { 0x6f1f49ed, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5f49f0, SUC_lexicon_tags_23 },
    { 0x61bf49f1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xaa1f49f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f49f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbf49fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27f49fe, SUC_lexicon_tags_2 },
    { 0x695f49ff, SUC_lexicon_tags_19 },
    { 0xb8ff4a00, SUC_lexicon_tags_148 },
    { 0x3d1f4a00, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73f4a09, SUC_lexicon_tags_1 },
    { 0x16ff4a09, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2a3f4a0c, SUC_lexicon_tags_47 },
    { 0xea5f4a0d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa17f4a11, SUC_lexicon_tags_151 },
    { 0xc31f4a12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe11f4a14, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x645f4a16, SUC_lexicon_tags_14 },
    { 0xb75f4a17, SUC_lexicon_tags_3 },
    { 0x3e9f4a18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3f3f4a1a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x343f4a1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed3f4a1e, SUC_lexicon_tags_60 },
    { 0xdfff4a1f, SUC_lexicon_tags_4 },
    { 0xd09f4a20, SUC_lexicon_tags_9 },
    { 0xdc7f4a21, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f4a24, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3f4a27, SUC_lexicon_tags_15 },
    { 0x241f4a28, SUC_lexicon_tags_37 },
    { 0x341f4a27, SUC_lexicon_tags_60 },
    { 0x5e7f4a2a, SUC_lexicon_tags_12 },
    { 0x4e7f4a2a, SUC_lexicon_tags_87 },
    { 0x9c3f4a2c, SUC_lexicon_tags_7 },
    { 0x7edf4a2c, SUC_lexicon_tags_39 },
    { 0x89df4a2d, SUC_lexicon_tags_15 },
    { 0x2d5f4a2f, SUC_lexicon_tags_10 },
    { 0x55df4a2f, SUC_lexicon_tags_15 },
    { 0x505f4a2b, SUC_lexicon_tags_14 },
    { 0x3c3f4a30, SUC_lexicon_tags_45 },
    { 0x865f4a33, SUC_lexicon_tags_0 },
    { 0xa9f4a34, SUC_lexicon_tags_43 },
    { 0xbf1f4a33, SUC_lexicon_tags_33 },
    { 0xee5f4a36, SUC_lexicon_tags_37 },
    { 0xb0ff4a37, SUC_lexicon_tags_15 },
    { 0x85bf4a2a, SUC_lexicon_tags_25 },
    { 0xd39f4a39, SUC_lexicon_tags_25 },
    { 0x20ff4a39, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x26bf4a3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5fbf4a3f, SUC_lexicon_tags_37 },
    { 0xeeff4a40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fff4a46, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f4a49, SUC_lexicon_tags_63 },
    { 0x5cbf4a4a, SUC_lexicon_tags_45 },
    { 0x4fbf4a4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf8df4a4d, SUC_lexicon_tags_11 },
    { 0xf39f4a4e, SUC_lexicon_tags_39 },
    { 0x4bbf4a4f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3f4a56, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x667f4a5a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbf4a5e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31df4a60, SUC_lexicon_tags_15 },
    { 0x3fdf4a61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x295f4a64, SUC_lexicon_tags_15 },
    { 0x57df4a65, SUC_lexicon_tags_49 },
    { 0xfc3f4a65, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7f4a6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9f4a6e, SUC_lexicon_tags_11 },
    { 0x451f4a6f, SUC_lexicon_tags_47 },
    { 0x359f4a70, SUC_lexicon_tags_39 },
    { 0x837f4a70, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x14df4a73, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x915f4a75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x817f4a77, SUC_lexicon_tags_14 },
    { 0x8e9f4a78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce7f4a7c, SUC_lexicon_tags_12 },
    { 0x74df4a7d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895f4a81, SUC_lexicon_tags_7 },
    { 0xcc7f4a81, SUC_lexicon_tags_49 },
    { 0x529f4a82, SUC_lexicon_tags_30 },
    { 0x72df4a81, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x187f4a86, SUC_lexicon_tags_17 },
    { 0x24df4a87, SUC_lexicon_tags_2 },
    { 0xcc7f4a87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15f4a8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xae5f4a8d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x8b7f4a8f, SUC_lexicon_tags_1 },
    { 0xb53f4a8f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc63f4a96, SUC_lexicon_tags_7 },
    { 0xfd9f4a97, SUC_lexicon_tags_8 },
    { 0x1ebf4a98, SUC_lexicon_tags_37 },
    { 0xa8ff4a97, SUC_lexicon_tags_17 },
    { 0xb5ff4a9a, SUC_lexicon_tags_92 },
    { 0xc2ff4a9b, SUC_lexicon_tags_14 },
    { 0x729f4a9b, SUC_lexicon_tags_15 },
    { 0xbe9f4a9c, SUC_lexicon_tags_1 },
    { 0x283f4a9e, SUC_lexicon_tags_37 },
    { 0xe89f4a9f, SUC_lexicon_tags_17 },
    { 0xd43f4a9d, SUC_lexicon_tags_17 },
    { 0x5d5f4aa0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11f4aa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185f4aac, SUC_lexicon_tags_25 },
    { 0xb17f4aac, SUC_lexicon_tags_47 },
    { 0x379f4aae, SUC_lexicon_tags_15 },
    { 0xb1df4aaf, SUC_lexicon_tags_12 },
    { 0x179f4aad, SUC_lexicon_tags_43 },
    { 0xc47f4ab0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4bdf4ab3, SUC_lexicon_tags_17 },
    { 0x41df4ab4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x525f4ab7, SUC_lexicon_tags_30 },
    { 0x167f4ab8, SUC_lexicon_tags_10 },
    { 0x875f4ab9, SUC_lexicon_tags_23 },
    { 0x29f4ab9, SUC_lexicon_tags_33 },
    { 0x78df4abb, SUC_lexicon_tags_9 },
    { 0xde7f4abb, SUC_lexicon_tags_15 },
    { 0x889f4abd, SUC_lexicon_tags_37 },
    { 0x32df4abe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f4ac1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bf4ac6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddf4acb, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1f4ad1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f4ad5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe3f4ad7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d3f4ada, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf4add, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7f4ae0, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bf4ae4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4ddf4ae6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9f4aea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x893f4aec, SUC_lexicon_tags_63 },
    { 0xb51f4aed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7f4af1, SUC_lexicon_tags_231 },
    { 0xd3bf4af2, SUC_lexicon_tags_12 },
    { 0x78df4af2, SUC_lexicon_tags_37 },
    { 0x82ff4af1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1f4af8, SUC_lexicon_tags_26 },
    { 0x91f4af9, SUC_lexicon_tags_14 },
    { 0xb83f4afa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f4afe, SUC_lexicon_tags_45 },
    { 0x401f4aff, SUC_lexicon_tags_12 },
    { 0x13df4aff, SUC_lexicon_tags_2 },
    { 0x197f4b00, SUC_lexicon_tags_10 },
    { 0xef3f4afe, SUC_lexicon_tags_7 },
    { 0x42ff4b03, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657f4b0a, SUC_lexicon_tags_37 },
    { 0xc41f4b0a, SUC_lexicon_tags_37 },
    { 0x72bf4b0a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339f4b10, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb3f4b12, SUC_lexicon_tags_1 },
    { 0x51df4b13, SUC_lexicon_tags_5 },
    { 0x755f4b14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1f4b18, SUC_lexicon_tags_8 },
    { 0xc9f4b19, SUC_lexicon_tags_12 },
    { 0x7abf4b1a, SUC_lexicon_tags_45 },
    { 0xa85f4b19, SUC_lexicon_tags_17 },
    { 0x5a7f4b1c, SUC_lexicon_tags_1 },
    { 0xffdf4b1d, SUC_lexicon_tags_39 },
    { 0x5fbf4b1e, SUC_lexicon_tags_10 },
    { 0xa03f4b1f, SUC_lexicon_tags_17 },
    { 0x4a5f4b20, SUC_lexicon_tags_47 },
    { 0x679f4b1c, SUC_lexicon_tags_25 },
    { 0x7a7f4b22, SUC_lexicon_tags_17 },
    { 0x239f4b22, SUC_lexicon_tags_81 },
    { 0x7b1f4b22, SUC_lexicon_tags_37 },
    { 0xe5df4b25, SUC_lexicon_tags_37 },
    { 0x421f4b25, SUC_lexicon_tags_27 },
    { 0x76bf4b27, SUC_lexicon_tags_6 },
    { 0xba5f4b22, SUC_lexicon_tags_14 },
    { 0xeb5f4b26, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90df4b2e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47f4b33, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc83f4b35, SUC_lexicon_tags_19 },
    { 0x9c3f4b36, SUC_lexicon_tags_50 },
    { 0xe0df4b37, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa71f4b39, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6b3f4b3b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe85f4b3f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb97f4b42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7f4b45, SUC_lexicon_tags_37 },
    { 0x763f4b46, SUC_lexicon_tags_63 },
    { 0x4c7f4b47, SUC_lexicon_tags_84 },
    { 0xbbff4b48, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27f4b4d, SUC_lexicon_tags_37 },
    { 0x78df4b4e, SUC_lexicon_tags_28 },
    { 0x147f4b4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x813f4b51, SUC_lexicon_tags_37 },
    { 0x35ff4b52, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bf4b57, SUC_lexicon_tags_38 },
    { 0xabbf4b58, SUC_lexicon_tags_1 },
    { 0x2f7f4b59, SUC_lexicon_tags_17 },
    { 0xef1f4b57, SUC_lexicon_tags_14 },
    { 0x45bf4b5b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91df4b63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x49f4b65, SUC_lexicon_tags_30 },
    { 0x713f4b66, SUC_lexicon_tags_1 },
    { 0x94bf4b66, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadff4b6b, SUC_lexicon_tags_307 },
    { 0x337f4b6c, SUC_lexicon_tags_1 },
    { 0x5ff4b6c, SUC_lexicon_tags_49 },
    { 0xcb7f4b6c, SUC_lexicon_tags_34 },
    { 0x8bf4b6e, SUC_lexicon_tags_25 },
    { 0xb47f4b70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1f4b74, SUC_lexicon_tags_25 },
    { 0x19f4b75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33df4b77, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73f4b7e, SUC_lexicon_tags_4 },
    { 0xbd7f4b7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3f7f4b81, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xa57f4b83, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x59df4b85, SUC_lexicon_tags_25 },
    { 0xd6df4b86, SUC_lexicon_tags_15 },
    { 0x6aff4b87, SUC_lexicon_tags_4 },
    { 0x611f4b88, SUC_lexicon_tags_45 },
    { 0xe5df4b89, SUC_lexicon_tags_11 },
    { 0xa43f4b89, SUC_lexicon_tags_3 },
    { 0x531f4b86, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ff4b8e, SUC_lexicon_tags_6 },
    { 0xbc7f4b8f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49f4b92, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1f4b98, SUC_lexicon_tags_30 },
    { 0x1c3f4b99, SUC_lexicon_tags_37 },
    { 0x427f4b99, SUC_lexicon_tags_14 },
    { 0x283f4b9b, SUC_lexicon_tags_37 },
    { 0x9cff4b9c, SUC_lexicon_tags_49 },
    { 0xc01f4b9d, SUC_lexicon_tags_27 },
    { 0x881f4b9e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bf4ba3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ff4ba8, SUC_lexicon_tags_118 },
    { 0x713f4ba8, SUC_lexicon_tags_38 },
    { 0x2abf4ba8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75f4bb1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xccdf4bb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fff4bb8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47f4bbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeaff4bbd, SUC_lexicon_tags_13 },
    { 0x49f4bbd, SUC_lexicon_tags_39 },
    { 0x86bf4bbf, SUC_lexicon_tags_47 },
    { 0x4b3f4bc0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6a7f4bc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dff4bc5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ff4bcc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ff4bd1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c3f4bd5, SUC_lexicon_tags_1 },
    { 0x257f4bd6, SUC_lexicon_tags_60 },
    { 0x77bf4bd6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749f4be4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23f4be8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x585f4bea, SUC_lexicon_tags_5 },
    { 0x713f4beb, SUC_lexicon_tags_45 },
    { 0x453f4beb, SUC_lexicon_tags_11 },
    { 0xbfdf4bed, SUC_lexicon_tags_7 },
    { 0x509f4bed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb85f4bf0, SUC_lexicon_tags_38 },
    { 0xe49f4bf1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bf4bf4, SUC_lexicon_tags_582 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7f4bf8, SUC_lexicon_tags_7 },
    { 0xabbf4bf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f4bfe, SUC_lexicon_tags_15 },
    { 0x6ff4bff, SUC_lexicon_tags_25 },
    { 0x5e3f4c00, SUC_lexicon_tags_12 },
    { 0x815f4bfe, SUC_lexicon_tags_37 },
    { 0x70ff4c02, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28df4c04, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49bf4c0b, SUC_lexicon_tags_43 },
    { 0x375f4c0b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd0df4c0e, SUC_lexicon_tags_45 },
    { 0xc2df4c0e, SUC_lexicon_tags_9 },
    { 0x78bf4c0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d9f4c12, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f4c1e, SUC_lexicon_tags_25 },
    { 0x93df4c1f, SUC_lexicon_tags_14 },
    { 0xee5f4c20, SUC_lexicon_tags_1 },
    { 0xff3f4c21, SUC_lexicon_tags_57 },
    { 0xe35f4c22, SUC_lexicon_tags_15 },
    { 0xd75f4c22, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69f4c2e, SUC_lexicon_tags_61 },
    { 0x279f4c2f, SUC_lexicon_tags_17 },
    { 0xffff4c30, SUC_lexicon_tags_88 },
    { 0x37bf4c2f, SUC_lexicon_tags_23 },
    { 0x1b7f4c32, SUC_lexicon_tags_15 },
    { 0xe5df4c32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x937f4c35, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x857f4c38, SUC_lexicon_tags_29 },
    { 0x811f4c39, SUC_lexicon_tags_23 },
    { 0xe9ff4c39, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7df4c3e, SUC_lexicon_tags_37 },
    { 0xa79f4c3f, SUC_lexicon_tags_37 },
    { 0xd29f4c3e, SUC_lexicon_tags_63 },
    { 0x1e9f4c41, SUC_lexicon_tags_47 },
    { 0x339f4c42, SUC_lexicon_tags_19 },
    { 0xbe1f4c43, SUC_lexicon_tags_37 },
    { 0x4fff4c43, SUC_lexicon_tags_37 },
    { 0x5bff4c45, SUC_lexicon_tags_27 },
    { 0x253f4c45, SUC_lexicon_tags_23 },
    { 0xbe3f4c40, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff4c4e, SUC_lexicon_tags_26 },
    { 0x331f4c4e, SUC_lexicon_tags_15 },
    { 0x613f4c4f, SUC_lexicon_tags_63 },
    { 0x35f4c51, SUC_lexicon_tags_7 },
    { 0xe5f4c52, SUC_lexicon_tags_25 },
    { 0x5b9f4c52, SUC_lexicon_tags_3 },
    { 0xdd7f4c51, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bf4c57, SUC_lexicon_tags_47 },
    { 0x27bf4c58, SUC_lexicon_tags_17 },
    { 0xb21f4c57, SUC_lexicon_tags_14 },
    { 0xe17f4c59, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9f4c5f, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ff4c63, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdf4c67, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12df4c78, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f4c7c, SUC_lexicon_tags_60 },
    { 0x831f4c7c, SUC_lexicon_tags_4 },
    { 0x4fdf4c7e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb5f4c80, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfe9f4c82, SUC_lexicon_tags_17 },
    { 0xa21f4c83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39f4c8a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x57bf4c8c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ff4c8f, SUC_lexicon_tags_113 },
    { 0x9b1f4c90, SUC_lexicon_tags_42 },
    { 0xa13f4c91, SUC_lexicon_tags_44 },
    { 0xcaff4c92, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96df4c96, SUC_lexicon_tags_63 },
    { 0x4bbf4c96, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xac9f4c99, SUC_lexicon_tags_2 },
    { 0xae5f4c9a, SUC_lexicon_tags_7 },
    { 0xa81f4c9b, SUC_lexicon_tags_12 },
    { 0x841f4c9a, SUC_lexicon_tags_46 },
    { 0x62ff4c9b, SUC_lexicon_tags_8 },
    { 0xf69f4c9e, SUC_lexicon_tags_47 },
    { 0xaddf4c99, SUC_lexicon_tags_25 },
    { 0xc61f4c9b, SUC_lexicon_tags_12 },
    { 0x179f4ca1, SUC_lexicon_tags_56 },
    { 0xd7ff4c9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6ff4ca4, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ff4ca9, SUC_lexicon_tags_7 },
    { 0xcc3f4ca9, SUC_lexicon_tags_39 },
    { 0xc7f4ca9, SUC_lexicon_tags_29 },
    { 0x71f4ca9, SUC_lexicon_tags_23 },
    { 0x345f4cac, SUC_lexicon_tags_27 },
    { 0xac9f4cac, SUC_lexicon_tags_12 },
    { 0x7edf4caf, SUC_lexicon_tags_23 },
    { 0x7f5f4caf, SUC_lexicon_tags_60 },
    { 0x6adf4cb1, SUC_lexicon_tags_27 },
    { 0x251f4cb2, SUC_lexicon_tags_15 },
    { 0x895f4cb2, SUC_lexicon_tags_46 },
    { 0xcb7f4cad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3f4cb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3f4cbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1f4cc5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41f4ccf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x187f4cd1, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa47f4cd4, SUC_lexicon_tags_12 },
    { 0xb4df4cd4, SUC_lexicon_tags_12 },
    { 0x125f4cd4, SUC_lexicon_tags_7 },
    { 0x41f4cd7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9b1f4cd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c7f4cdb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbf4cdf, SUC_lexicon_tags_26 },
    { 0x6b1f4cdf, SUC_lexicon_tags_37 },
    { 0xaf7f4ce1, SUC_lexicon_tags_34 },
    { 0x3d3f4ce1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x361f4ce4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53f4ce7, SUC_lexicon_tags_37 },
    { 0x857f4ce8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed9f4cea, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bf4ced, SUC_lexicon_tags_60 },
    { 0x73f4cee, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7abf4cf0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf4cfb, SUC_lexicon_tags_12 },
    { 0xa31f4cfc, SUC_lexicon_tags_19 },
    { 0x99ff4cfd, SUC_lexicon_tags_19 },
    { 0x757f4cfd, SUC_lexicon_tags_0 },
    { 0x4ebf4cfd, SUC_lexicon_tags_7 },
    { 0x623f4d00, SUC_lexicon_tags_12 },
    { 0xc29f4d01, SUC_lexicon_tags_61 },
    { 0x90ff4d00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513f4d07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9f4d16, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xcc9f4d18, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615f4d1f, SUC_lexicon_tags_4 },
    { 0x3d9f4d20, SUC_lexicon_tags_8 },
    { 0xee3f4d21, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x73df4d24, SUC_lexicon_tags_1 },
    { 0x825f4d25, SUC_lexicon_tags_49 },
    { 0xfd5f4d26, SUC_lexicon_tags_37 },
    { 0xd4ff4d24, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xf61f4d29, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x11f4d2b, SUC_lexicon_tags_43 },
    { 0x31bf4d2c, SUC_lexicon_tags_25 },
    { 0xe8bf4d2d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xedff4d2f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x45bf4d32, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe3f4d34, SUC_lexicon_tags_15 },
    { 0xb9df4d35, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x717f4d38, SUC_lexicon_tags_317 },
    { 0, NULL },
    { 0, NULL },
    { 0x287f4d3b, SUC_lexicon_tags_7 },
    { 0xba3f4d3c, SUC_lexicon_tags_26 },
    { 0x7f5f4d3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ff4d40, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7f4d44, SUC_lexicon_tags_14 },
    { 0xdfdf4d44, SUC_lexicon_tags_63 },
    { 0xc73f4d46, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5f4d49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f4d4c, SUC_lexicon_tags_32 },
    { 0x913f4d4c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x413f4d51, SUC_lexicon_tags_62 },
    { 0xb69f4d52, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bdf4d55, SUC_lexicon_tags_28 },
    { 0x889f4d56, SUC_lexicon_tags_11 },
    { 0x51f4d56, SUC_lexicon_tags_18 },
    { 0xf87f4d58, SUC_lexicon_tags_38 },
    { 0xfd9f4d59, SUC_lexicon_tags_43 },
    { 0xec5f4d5a, SUC_lexicon_tags_17 },
    { 0xb31f4d59, SUC_lexicon_tags_15 },
    { 0x933f4d5b, SUC_lexicon_tags_23 },
    { 0x341f4d5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x58df4d5f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x643f4d62, SUC_lexicon_tags_25 },
    { 0x59f4d63, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ff4d66, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23f4d6a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65f4d71, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbf4d74, SUC_lexicon_tags_82 },
    { 0x671f4d75, SUC_lexicon_tags_17 },
    { 0xdb3f4d76, SUC_lexicon_tags_17 },
    { 0x4e9f4d77, SUC_lexicon_tags_37 },
    { 0xe4ff4d78, SUC_lexicon_tags_2 },
    { 0xa41f4d79, SUC_lexicon_tags_2 },
    { 0x949f4d7a, SUC_lexicon_tags_1 },
    { 0x9d1f4d7b, SUC_lexicon_tags_37 },
    { 0x9adf4d78, SUC_lexicon_tags_116 },
    { 0x4b3f4d7c, SUC_lexicon_tags_18 },
    { 0x83ff4d7e, SUC_lexicon_tags_7 },
    { 0xdd7f4d78, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1df4d83, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x817f4d86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x901f4d89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1f4d8d, SUC_lexicon_tags_9 },
    { 0x67ff4d8e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9f4d91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2df4d9d, SUC_lexicon_tags_17 },
    { 0x753f4d9e, SUC_lexicon_tags_37 },
    { 0xbbdf4d9f, SUC_lexicon_tags_2 },
    { 0x2b3f4da0, SUC_lexicon_tags_87 },
    { 0xa7f4da1, SUC_lexicon_tags_30 },
    { 0xd03f4da0, SUC_lexicon_tags_9 },
    { 0x795f4da3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xad9f4da6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bff4da9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdf4dae, SUC_lexicon_tags_43 },
    { 0xb65f4dae, SUC_lexicon_tags_14 },
    { 0xeeff4db0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bf4db4, SUC_lexicon_tags_12 },
    { 0xd31f4db5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x275f4db8, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bff4dbd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bf4dc1, SUC_lexicon_tags_79 },
    { 0xba1f4dc1, SUC_lexicon_tags_23 },
    { 0x555f4dc2, SUC_lexicon_tags_46 },
    { 0x799f4dc4, SUC_lexicon_tags_12 },
    { 0x8d9f4dc5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf9ff4dc7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe11f4dc9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc0bf4dcb, SUC_lexicon_tags_16 },
    { 0x74bf4dcc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbf4dd3, SUC_lexicon_tags_47 },
    { 0x28ff4dd3, SUC_lexicon_tags_11 },
    { 0xcf1f4dd5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x20ff4dd7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x171f4dda, SUC_lexicon_tags_12 },
    { 0xce1f4ddb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x43bf4ddd, SUC_lexicon_tags_65 },
    { 0x2a9f4ddd, SUC_lexicon_tags_25 },
    { 0x363f4ddf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7f4de3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x999f4dea, SUC_lexicon_tags_77 },
    { 0x507f4dea, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbf4ded, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x587f4def, SUC_lexicon_tags_15 },
    { 0xa13f4df0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5f4df6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x89bf4df9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x31ff4dfb, SUC_lexicon_tags_59 },
    { 0x4e9f4dfb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdf4dff, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x759f4e01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1f4e04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9f4e07, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x477f4e09, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xebbf4e0b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf25f4e0d, SUC_lexicon_tags_2 },
    { 0x489f4e0e, SUC_lexicon_tags_3 },
    { 0xff5f4e0f, SUC_lexicon_tags_33 },
    { 0x97ff4e10, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xba7f4e12, SUC_lexicon_tags_0 },
    { 0x66df4e12, SUC_lexicon_tags_11 },
    { 0x821f4e12, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643f4e1c, SUC_lexicon_tags_12 },
    { 0xdcdf4e1d, SUC_lexicon_tags_25 },
    { 0xa29f4e1d, SUC_lexicon_tags_37 },
    { 0x1c9f4e1f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7f4e25, SUC_lexicon_tags_0 },
    { 0x955f4e25, SUC_lexicon_tags_65 },
    { 0x78df4e27, SUC_lexicon_tags_60 },
    { 0xdadf4e28, SUC_lexicon_tags_33 },
    { 0xfa1f4e28, SUC_lexicon_tags_37 },
    { 0x6eff4e29, SUC_lexicon_tags_31 },
    { 0x103f4e2b, SUC_lexicon_tags_11 },
    { 0xb8ff4e2c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6ff4e33, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x92ff4e36, SUC_lexicon_tags_66 },
    { 0x447f4e37, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0x2e5f4e39, SUC_lexicon_tags_19 },
    { 0xce1f4e3a, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeff4e42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x321f4e44, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1f4e48, SUC_lexicon_tags_4 },
    { 0x2ddf4e49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bf4e4c, SUC_lexicon_tags_13 },
    { 0x5abf4e4c, SUC_lexicon_tags_48 },
    { 0x7e1f4e4e, SUC_lexicon_tags_12 },
    { 0x5e9f4e4f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x753f4e52, SUC_lexicon_tags_7 },
    { 0x3fff4e53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5f4e58, SUC_lexicon_tags_60 },
    { 0x193f4e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5f4e5f, SUC_lexicon_tags_15 },
    { 0xff3f4e5f, SUC_lexicon_tags_25 },
    { 0x48bf4e5f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bf4e66, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7e1f4e68, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3f4e6b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf4e6e, SUC_lexicon_tags_1467 },
    { 0xca7f4e6e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3aff4e72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3f4e77, SUC_lexicon_tags_11 },
    { 0x91bf4e78, SUC_lexicon_tags_17 },
    { 0xdcbf4e78, SUC_lexicon_tags_17 },
    { 0x6adf4e7a, SUC_lexicon_tags_17 },
    { 0xc05f4e7b, SUC_lexicon_tags_6 },
    { 0x505f4e7a, SUC_lexicon_tags_11 },
    { 0x6c3f4e7d, SUC_lexicon_tags_19 },
    { 0xb99f4e7c, SUC_lexicon_tags_107 },
    { 0xfff4e79, SUC_lexicon_tags_17 },
    { 0xd0df4e80, SUC_lexicon_tags_11 },
    { 0x41df4e80, SUC_lexicon_tags_25 },
    { 0x9f4e82, SUC_lexicon_tags_16 },
    { 0x6e3f4e80, SUC_lexicon_tags_8 },
    { 0x329f4e7c, SUC_lexicon_tags_66 },
    { 0xa25f4e85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7f4e89, SUC_lexicon_tags_37 },
    { 0x9dff4e8a, SUC_lexicon_tags_33 },
    { 0x2bf4e8b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45df4e91, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5f4e97, SUC_lexicon_tags_25 },
    { 0xff5f4e98, SUC_lexicon_tags_11 },
    { 0xf93f4e99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813f4e9e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ff4ea7, SUC_lexicon_tags_7 },
    { 0xb5ff4ea8, SUC_lexicon_tags_49 },
    { 0x87ff4ea9, SUC_lexicon_tags_33 },
    { 0x70bf4ea9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539f4eaf, SUC_lexicon_tags_37 },
    { 0x6ff4eb0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x909f4eb2, SUC_lexicon_tags_37 },
    { 0xf6bf4eb2, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1df4eb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ff4ebe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfbf4ec0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbf4ec5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82df4ecb, SUC_lexicon_tags_28 },
    { 0x513f4ecb, SUC_lexicon_tags_190 },
    { 0xf77f4ecb, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9f4ed0, SUC_lexicon_tags_33 },
    { 0xde3f4ed1, SUC_lexicon_tags_37 },
    { 0x195f4ed0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebff4ed8, SUC_lexicon_tags_23 },
    { 0x9a9f4ed8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5e5f4edb, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bf4ee0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xbabf4ee2, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xf9ff4ee4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51ff4ee6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25bf4eea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x57ff4eec, SUC_lexicon_tags_25 },
    { 0xc0df4eed, SUC_lexicon_tags_17 },
    { 0x3bf4eed, SUC_lexicon_tags_65 },
    { 0x87ff4eef, SUC_lexicon_tags_17 },
    { 0x613f4ef0, SUC_lexicon_tags_1 },
    { 0x8cdf4ef0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebf4ef6, SUC_lexicon_tags_47 },
    { 0x4a1f4ef7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x101f4ef9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f3f4efb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29df4eff, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6e7f4f01, SUC_lexicon_tags_12 },
    { 0x721f4f01, SUC_lexicon_tags_31 },
    { 0x35df4f01, SUC_lexicon_tags_32 },
    { 0x53ff4f04, SUC_lexicon_tags_15 },
    { 0x2bf4f05, SUC_lexicon_tags_23 },
    { 0x795f4f04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bff4f0b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991f4f12, SUC_lexicon_tags_60 },
    { 0xa27f4f13, SUC_lexicon_tags_31 },
    { 0x57ff4f14, SUC_lexicon_tags_27 },
    { 0xbebf4f15, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebbf4f1a, SUC_lexicon_tags_66 },
    { 0x361f4f1b, SUC_lexicon_tags_25 },
    { 0xacdf4f1c, SUC_lexicon_tags_2 },
    { 0x9b3f4f1a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xb37f4f1f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x999f4f21, SUC_lexicon_tags_9 },
    { 0xd83f4f22, SUC_lexicon_tags_37 },
    { 0xb33f4f23, SUC_lexicon_tags_33 },
    { 0x2ff4f24, SUC_lexicon_tags_43 },
    { 0x395f4f25, SUC_lexicon_tags_8 },
    { 0xa03f4f26, SUC_lexicon_tags_43 },
    { 0xdadf4f25, SUC_lexicon_tags_72 },
    { 0xdbff4f28, SUC_lexicon_tags_17 },
    { 0x93ff4f27, SUC_lexicon_tags_15 },
    { 0x66df4f27, SUC_lexicon_tags_37 },
    { 0x3ff4f2a, SUC_lexicon_tags_17 },
    { 0x3aff4f22, SUC_lexicon_tags_60 },
    { 0x9b1f4f2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x165f4f30, SUC_lexicon_tags_11 },
    { 0xee9f4f31, SUC_lexicon_tags_30 },
    { 0x3f1f4f32, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x24df4f34, SUC_lexicon_tags_33 },
    { 0xbb9f4f35, SUC_lexicon_tags_1 },
    { 0xed3f4f36, SUC_lexicon_tags_11 },
    { 0x6d1f4f36, SUC_lexicon_tags_11 },
    { 0xcdff4f38, SUC_lexicon_tags_65 },
    { 0x761f4f39, SUC_lexicon_tags_4 },
    { 0x8b1f4f3a, SUC_lexicon_tags_243 },
    { 0xa15f4f3b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8df4f3e, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45f4f42, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe79f4f44, SUC_lexicon_tags_37 },
    { 0x71f4f44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f9f4f47, SUC_lexicon_tags_44 },
    { 0x917f4f47, SUC_lexicon_tags_60 },
    { 0xda5f4f49, SUC_lexicon_tags_15 },
    { 0xff4f48, SUC_lexicon_tags_66 },
    { 0x313f4f4b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60df4f54, SUC_lexicon_tags_11 },
    { 0xeb1f4f55, SUC_lexicon_tags_140 },
    { 0xc35f4f55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827f4f5b, SUC_lexicon_tags_17 },
    { 0x16df4f5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83f4f61, SUC_lexicon_tags_2 },
    { 0x6dff4f62, SUC_lexicon_tags_17 },
    { 0x165f4f61, SUC_lexicon_tags_224 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ff4f68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb2ff4f6a, SUC_lexicon_tags_15 },
    { 0x605f4f6b, SUC_lexicon_tags_33 },
    { 0x323f4f6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19f4f70, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xe6ff4f72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67f4f75, SUC_lexicon_tags_11 },
    { 0xb19f4f75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5f4f7b, SUC_lexicon_tags_47 },
    { 0x565f4f7c, SUC_lexicon_tags_38 },
    { 0x37df4f7d, SUC_lexicon_tags_37 },
    { 0xa13f4f7e, SUC_lexicon_tags_28 },
    { 0x2fff4f7f, SUC_lexicon_tags_218 },
    { 0x969f4f7f, SUC_lexicon_tags_55 },
    { 0xbe1f4f7d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abf4f84, SUC_lexicon_tags_12 },
    { 0x5edf4f84, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x92ff4f87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bf4f8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x65df4f8c, SUC_lexicon_tags_15 },
    { 0xf63f4f8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaf9f4f8f, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9f4f94, SUC_lexicon_tags_8 },
    { 0x11ff4f95, SUC_lexicon_tags_37 },
    { 0x6df4f96, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9f4f9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x623f4f9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901f4fa0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x131f4fa3, SUC_lexicon_tags_2 },
    { 0xb63f4fa3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bf4fa8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547f4fac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7f4fb6, SUC_lexicon_tags_57 },
    { 0x355f4fb7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbf4fbd, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x967f4fbf, SUC_lexicon_tags_23 },
    { 0x60bf4fc0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb63f4fc2, SUC_lexicon_tags_1 },
    { 0xb73f4fc3, SUC_lexicon_tags_16 },
    { 0xefff4fc4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97df4fc8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21df4fcc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x349f4fcf, SUC_lexicon_tags_30 },
    { 0x9d5f4fcf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xddbf4fd2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3f4fd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x987f4fd7, SUC_lexicon_tags_11 },
    { 0x42ff4fd8, SUC_lexicon_tags_46 },
    { 0x315f4fd9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51df4fde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb07f4fe0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfa9f4fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655f4fe9, SUC_lexicon_tags_19 },
    { 0x5a1f4fea, SUC_lexicon_tags_15 },
    { 0xa59f4fea, SUC_lexicon_tags_37 },
    { 0x237f4fe9, SUC_lexicon_tags_30 },
    { 0xe89f4fed, SUC_lexicon_tags_9 },
    { 0x729f4fea, SUC_lexicon_tags_35 },
    { 0x721f4fef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x441f4ff2, SUC_lexicon_tags_1 },
    { 0xd9ff4ff2, SUC_lexicon_tags_23 },
    { 0xf3f4ff4, SUC_lexicon_tags_60 },
    { 0x87df4ff5, SUC_lexicon_tags_37 },
    { 0x331f4ff4, SUC_lexicon_tags_11 },
    { 0x475f4ff7, SUC_lexicon_tags_7 },
    { 0xe2df4ff2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12df5004, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe59f5006, SUC_lexicon_tags_215 },
    { 0xf2df5007, SUC_lexicon_tags_38 },
    { 0x67df5008, SUC_lexicon_tags_26 },
    { 0x565f5009, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff500f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9f5015, SUC_lexicon_tags_38 },
    { 0x437f5015, SUC_lexicon_tags_7 },
    { 0xcb5f5017, SUC_lexicon_tags_11 },
    { 0x66bf5016, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa79f501a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf27f501c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f5022, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9f5027, SUC_lexicon_tags_14 },
    { 0x1f5028, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x995f502b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x80bf502e, SUC_lexicon_tags_37 },
    { 0x341f502e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3f5032, SUC_lexicon_tags_9 },
    { 0xdddf5033, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa93f5035, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1f5039, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbf503e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ff5043, SUC_lexicon_tags_19 },
    { 0xa7bf5044, SUC_lexicon_tags_37 },
    { 0xa6bf5045, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x57f5047, SUC_lexicon_tags_38 },
    { 0xe3df5048, SUC_lexicon_tags_63 },
    { 0xc41f5049, SUC_lexicon_tags_11 },
    { 0xe57f5048, SUC_lexicon_tags_23 },
    { 0x37ff504b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xcd5f504d, SUC_lexicon_tags_26 },
    { 0x113f504d, SUC_lexicon_tags_25 },
    { 0x3fdf504f, SUC_lexicon_tags_11 },
    { 0xfff504e, SUC_lexicon_tags_16 },
    { 0x1cdf504f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26df5055, SUC_lexicon_tags_17 },
    { 0x813f5056, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99f505f, SUC_lexicon_tags_77 },
    { 0xb8df505f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9f5063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3df5068, SUC_lexicon_tags_48 },
    { 0xb0bf5069, SUC_lexicon_tags_7 },
    { 0x29bf5069, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471f5070, SUC_lexicon_tags_9 },
    { 0x7bbf5071, SUC_lexicon_tags_14 },
    { 0x523f5072, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x953f5074, SUC_lexicon_tags_12 },
    { 0x8c1f5075, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfbdf5077, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adf507c, SUC_lexicon_tags_30 },
    { 0x235f507c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7df5081, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xf9df5083, SUC_lexicon_tags_11 },
    { 0x159f5083, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x673f5086, SUC_lexicon_tags_26 },
    { 0x4e9f5087, SUC_lexicon_tags_2 },
    { 0xb15f5088, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fff508b, SUC_lexicon_tags_84 },
    { 0x6bff508b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe09f508f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9f5092, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2abf5094, SUC_lexicon_tags_17 },
    { 0x795f5094, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcd7f5097, SUC_lexicon_tags_4 },
    { 0x753f5098, SUC_lexicon_tags_17 },
    { 0xb91f5099, SUC_lexicon_tags_11 },
    { 0xc5f5098, SUC_lexicon_tags_16 },
    { 0x931f509a, SUC_lexicon_tags_249 },
    { 0x533f509c, SUC_lexicon_tags_37 },
    { 0xafbf509c, SUC_lexicon_tags_47 },
    { 0x4f3f5097, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc15f50a0, SUC_lexicon_tags_15 },
    { 0x921f50a0, SUC_lexicon_tags_17 },
    { 0x2f7f50a0, SUC_lexicon_tags_47 },
    { 0x6cdf50a3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x453f50a5, SUC_lexicon_tags_23 },
    { 0x53ff50a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bf50aa, SUC_lexicon_tags_61 },
    { 0xd41f50ab, SUC_lexicon_tags_17 },
    { 0xf9f50ab, SUC_lexicon_tags_29 },
    { 0x62ff50ac, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5f50b1, SUC_lexicon_tags_11 },
    { 0x4d3f50b2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaff50b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57df50b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd4df50bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bf50bf, SUC_lexicon_tags_8 },
    { 0xbebf50c0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd97f50c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec9f50c4, SUC_lexicon_tags_7 },
    { 0xb4df50c5, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdf50c8, SUC_lexicon_tags_37 },
    { 0xadbf50c9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5f50cd, SUC_lexicon_tags_6 },
    { 0x8d3f50ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x361f50d1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5f50d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515f50d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x975f50dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bf50e3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adf50ea, SUC_lexicon_tags_1 },
    { 0x457f50ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f50f0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15f50f7, SUC_lexicon_tags_263 },
    { 0x4c1f50f7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7f50fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x66ff50fe, SUC_lexicon_tags_1 },
    { 0x79df50ff, SUC_lexicon_tags_30 },
    { 0xf3f5100, SUC_lexicon_tags_27 },
    { 0x9d5f5101, SUC_lexicon_tags_7 },
    { 0x52bf5102, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbf5105, SUC_lexicon_tags_38 },
    { 0xf8df5106, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69df510a, SUC_lexicon_tags_9 },
    { 0x6f9f510b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf7f510d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xad7f510f, SUC_lexicon_tags_13 },
    { 0x3bbf5110, SUC_lexicon_tags_11 },
    { 0x513f5110, SUC_lexicon_tags_12 },
    { 0x6d5f5110, SUC_lexicon_tags_16 },
    { 0x853f5111, SUC_lexicon_tags_5 },
    { 0xcb3f5110, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bf5118, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411f511c, SUC_lexicon_tags_32 },
    { 0xe6bf511d, SUC_lexicon_tags_299 },
    { 0x8c5f511e, SUC_lexicon_tags_37 },
    { 0x2f1f511f, SUC_lexicon_tags_38 },
    { 0x93df511e, SUC_lexicon_tags_12 },
    { 0xcf7f5121, SUC_lexicon_tags_25 },
    { 0x84bf5122, SUC_lexicon_tags_17 },
    { 0x3edf5123, SUC_lexicon_tags_2 },
    { 0xa37f5124, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x24df5126, SUC_lexicon_tags_5 },
    { 0x20bf5127, SUC_lexicon_tags_47 },
    { 0xbe9f5128, SUC_lexicon_tags_8 },
    { 0xc79f5127, SUC_lexicon_tags_38 },
    { 0xea3f512a, SUC_lexicon_tags_15 },
    { 0x283f512b, SUC_lexicon_tags_15 },
    { 0x31ff512c, SUC_lexicon_tags_11 },
    { 0xecff512d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x595f512f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdf5134, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7aff5136, SUC_lexicon_tags_37 },
    { 0x38bf5136, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbb1f5139, SUC_lexicon_tags_11 },
    { 0x365f5139, SUC_lexicon_tags_11 },
    { 0x885f513b, SUC_lexicon_tags_9 },
    { 0x33f513c, SUC_lexicon_tags_15 },
    { 0x9d1f513d, SUC_lexicon_tags_26 },
    { 0xe79f513e, SUC_lexicon_tags_27 },
    { 0xff3f513f, SUC_lexicon_tags_38 },
    { 0x739f5140, SUC_lexicon_tags_15 },
    { 0x657f5140, SUC_lexicon_tags_23 },
    { 0x9bff5142, SUC_lexicon_tags_37 },
    { 0xacff5143, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1df5145, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bf514b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7f5150, SUC_lexicon_tags_0 },
    { 0x2cff5151, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29f5155, SUC_lexicon_tags_19 },
    { 0xf37f5155, SUC_lexicon_tags_37 },
    { 0x75f5157, SUC_lexicon_tags_7 },
    { 0x961f5158, SUC_lexicon_tags_45 },
    { 0x8f9f5156, SUC_lexicon_tags_31 },
    { 0xa59f515a, SUC_lexicon_tags_38 },
    { 0xed5f515b, SUC_lexicon_tags_11 },
    { 0xc25f515a, SUC_lexicon_tags_37 },
    { 0xe43f515c, SUC_lexicon_tags_0 },
    { 0xce7f515a, SUC_lexicon_tags_37 },
    { 0xadf515a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x743f5162, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde3f5164, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xa45f5167, SUC_lexicon_tags_43 },
    { 0x2b3f5167, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf73f516a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe99f516c, SUC_lexicon_tags_37 },
    { 0x5a1f516c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bf5171, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xcebf5173, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f5f5177, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb81f5179, SUC_lexicon_tags_29 },
    { 0xef1f517a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdf517d, SUC_lexicon_tags_11 },
    { 0x155f517e, SUC_lexicon_tags_9 },
    { 0x9b7f517f, SUC_lexicon_tags_38 },
    { 0xd17f517e, SUC_lexicon_tags_1 },
    { 0xdabf5181, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241f5185, SUC_lexicon_tags_21 },
    { 0xd57f5186, SUC_lexicon_tags_21 },
    { 0xb95f5186, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x9adf5189, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa4df518b, SUC_lexicon_tags_37 },
    { 0xe11f518c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcff5190, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x8bff5192, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bf5197, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373f519c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe9ff519e, SUC_lexicon_tags_0 },
    { 0x25bf519f, SUC_lexicon_tags_38 },
    { 0xc3ff51a0, SUC_lexicon_tags_17 },
    { 0x399f519f, SUC_lexicon_tags_16 },
    { 0xc25f519f, SUC_lexicon_tags_12 },
    { 0x71ff51a2, SUC_lexicon_tags_32 },
    { 0x635f51a0, SUC_lexicon_tags_43 },
    { 0xfc5f51a5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x925f51a8, SUC_lexicon_tags_11 },
    { 0x6c5f51a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583f51b0, SUC_lexicon_tags_20 },
    { 0xfe5f51b1, SUC_lexicon_tags_27 },
    { 0x705f51b1, SUC_lexicon_tags_37 },
    { 0xb01f51b2, SUC_lexicon_tags_63 },
    { 0x5f3f51b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x851f51b6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xf07f51b8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21f51bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7f51c0, SUC_lexicon_tags_27 },
    { 0xb2ff51c1, SUC_lexicon_tags_25 },
    { 0x5d3f51c1, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9f51c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x423f51cb, SUC_lexicon_tags_1495 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbf51da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafff51e4, SUC_lexicon_tags_38 },
    { 0xa27f51e5, SUC_lexicon_tags_7 },
    { 0xa11f51e6, SUC_lexicon_tags_11 },
    { 0x117f51e6, SUC_lexicon_tags_7 },
    { 0x963f51e8, SUC_lexicon_tags_34 },
    { 0x11bf51e9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59f51f2, SUC_lexicon_tags_25 },
    { 0x6b9f51f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77f51f6, SUC_lexicon_tags_7 },
    { 0xb07f51f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x29df51f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3cdf51fb, SUC_lexicon_tags_11 },
    { 0xe09f51fc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc85f51ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73f5201, SUC_lexicon_tags_97 },
    { 0x9a3f5202, SUC_lexicon_tags_11 },
    { 0x17ff5203, SUC_lexicon_tags_43 },
    { 0x22df5204, SUC_lexicon_tags_97 },
    { 0x7b7f5205, SUC_lexicon_tags_37 },
    { 0x1cff5204, SUC_lexicon_tags_38 },
    { 0xedbf5203, SUC_lexicon_tags_21 },
    { 0x765f5204, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613f520d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbb5f520f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1f5212, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73f5215, SUC_lexicon_tags_39 },
    { 0xecff5215, SUC_lexicon_tags_19 },
    { 0x7b5f5217, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bf521c, SUC_lexicon_tags_3 },
    { 0x669f521d, SUC_lexicon_tags_1 },
    { 0x35f521e, SUC_lexicon_tags_60 },
    { 0x815f521e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b7f5221, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7f5228, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdf522c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ff5238, SUC_lexicon_tags_6 },
    { 0x31ff5239, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75f523d, SUC_lexicon_tags_37 },
    { 0x80bf523e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbcbf5240, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ff5248, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7f524b, SUC_lexicon_tags_11 },
    { 0x37bf524c, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0x2d7f524e, SUC_lexicon_tags_18 },
    { 0x65f524f, SUC_lexicon_tags_12 },
    { 0x677f524f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9f5254, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x20bf5256, SUC_lexicon_tags_1 },
    { 0xa8df5257, SUC_lexicon_tags_17 },
    { 0x8e9f5258, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5f525b, SUC_lexicon_tags_72 },
    { 0xdbf525c, SUC_lexicon_tags_17 },
    { 0xb65f525d, SUC_lexicon_tags_198 },
    { 0x3c7f525e, SUC_lexicon_tags_15 },
    { 0x763f525f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xee3f5261, SUC_lexicon_tags_1 },
    { 0xf15f5262, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xd53f5264, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xef5f5266, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa35f5268, SUC_lexicon_tags_47 },
    { 0x635f5268, SUC_lexicon_tags_1 },
    { 0x8b1f526a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3f5272, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9f5277, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x567f5279, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7f527e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ff5281, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xef9f5283, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x46df5286, SUC_lexicon_tags_25 },
    { 0x5c1f5286, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ff528a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabf528e, SUC_lexicon_tags_12 },
    { 0x32bf528f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebf5294, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3e9f5296, SUC_lexicon_tags_29 },
    { 0x14bf5296, SUC_lexicon_tags_25 },
    { 0x95bf5298, SUC_lexicon_tags_47 },
    { 0xe67f5299, SUC_lexicon_tags_70 },
    { 0x3f9f529a, SUC_lexicon_tags_11 },
    { 0xafff529b, SUC_lexicon_tags_4 },
    { 0x86bf529c, SUC_lexicon_tags_21 },
    { 0x1fdf529b, SUC_lexicon_tags_25 },
    { 0x4cdf529e, SUC_lexicon_tags_25 },
    { 0x4d9f529e, SUC_lexicon_tags_37 },
    { 0x2b7f52a0, SUC_lexicon_tags_61 },
    { 0x69df529d, SUC_lexicon_tags_25 },
    { 0x551f529f, SUC_lexicon_tags_2 },
    { 0xffbf529d, SUC_lexicon_tags_30 },
    { 0xb11f52a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bf52a7, SUC_lexicon_tags_17 },
    { 0xa09f52a8, SUC_lexicon_tags_12 },
    { 0xc73f52a9, SUC_lexicon_tags_45 },
    { 0xda7f52a9, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1f52af, SUC_lexicon_tags_38 },
    { 0x6aff52b0, SUC_lexicon_tags_47 },
    { 0x503f52b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3f52b7, SUC_lexicon_tags_43 },
    { 0x59bf52b8, SUC_lexicon_tags_15 },
    { 0x9a5f52b9, SUC_lexicon_tags_11 },
    { 0xf77f52b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29df52c0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1bf52c2, SUC_lexicon_tags_15 },
    { 0xb7f52c3, SUC_lexicon_tags_47 },
    { 0xab1f52c4, SUC_lexicon_tags_14 },
    { 0x7c9f52c4, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x381f52c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x767f52c9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9f52cd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdff52d0, SUC_lexicon_tags_12 },
    { 0xa45f52d1, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7f52d4, SUC_lexicon_tags_38 },
    { 0x749f52d5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x4dff52d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x11bf52d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bf52e0, SUC_lexicon_tags_25 },
    { 0xc25f52e1, SUC_lexicon_tags_37 },
    { 0xe39f52e2, SUC_lexicon_tags_25 },
    { 0x339f52e3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ff52e6, SUC_lexicon_tags_15 },
    { 0xc35f52e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87f52ea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759f52f1, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1f52f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x887f52fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1f52ff, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9f5306, SUC_lexicon_tags_15 },
    { 0xed1f5307, SUC_lexicon_tags_17 },
    { 0x411f5306, SUC_lexicon_tags_19 },
    { 0x5e9f5309, SUC_lexicon_tags_48 },
    { 0xbfff530a, SUC_lexicon_tags_43 },
    { 0x437f530b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91f530f, SUC_lexicon_tags_34 },
    { 0xbf1f5310, SUC_lexicon_tags_45 },
    { 0x8c1f5310, SUC_lexicon_tags_49 },
    { 0x617f5312, SUC_lexicon_tags_30 },
    { 0x7cff5313, SUC_lexicon_tags_1 },
    { 0xb07f5314, SUC_lexicon_tags_15 },
    { 0x959f5315, SUC_lexicon_tags_140 },
    { 0x887f5314, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x633f5318, SUC_lexicon_tags_25 },
    { 0x47df5318, SUC_lexicon_tags_2 },
    { 0x441f5318, SUC_lexicon_tags_11 },
    { 0xf11f531b, SUC_lexicon_tags_25 },
    { 0xb2df5319, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19bf5321, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadf532a, SUC_lexicon_tags_15 },
    { 0x283f532b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x53bf532e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x959f5330, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbff5332, SUC_lexicon_tags_25 },
    { 0xdadf5333, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f533b, SUC_lexicon_tags_11 },
    { 0x227f533c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1f533f, SUC_lexicon_tags_17 },
    { 0x15df5340, SUC_lexicon_tags_37 },
    { 0xc27f5341, SUC_lexicon_tags_17 },
    { 0x357f5341, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3f5f5344, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5f534d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83f5350, SUC_lexicon_tags_697 },
    { 0, NULL },
    { 0xb5f5352, SUC_lexicon_tags_25 },
    { 0x857f5353, SUC_lexicon_tags_65 },
    { 0x785f5354, SUC_lexicon_tags_68 },
    { 0xd3ff5355, SUC_lexicon_tags_43 },
    { 0xf99f5356, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb31f5358, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x81bf535a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3f5361, SUC_lexicon_tags_49 },
    { 0x653f5362, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799f5366, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bf536c, SUC_lexicon_tags_77 },
    { 0xefbf536c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x775f536f, SUC_lexicon_tags_37 },
    { 0x511f5370, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x615f5373, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x1bff5375, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edf537a, SUC_lexicon_tags_54 },
    { 0x38bf537b, SUC_lexicon_tags_25 },
    { 0xaabf537c, SUC_lexicon_tags_13 },
    { 0x3d1f537c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xd03f537f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903f5387, SUC_lexicon_tags_25 },
    { 0xe4ff5387, SUC_lexicon_tags_17 },
    { 0x57f5388, SUC_lexicon_tags_38 },
    { 0xaedf538a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55f5390, SUC_lexicon_tags_19 },
    { 0x589f5391, SUC_lexicon_tags_19 },
    { 0x1f1f5392, SUC_lexicon_tags_83 },
    { 0xbbbf5393, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x697f5395, SUC_lexicon_tags_9 },
    { 0xf99f5396, SUC_lexicon_tags_7 },
    { 0xde5f5396, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7f539a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1f539d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x273f53a0, SUC_lexicon_tags_63 },
    { 0x7aff53a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61f53a5, SUC_lexicon_tags_21 },
    { 0x921f53a6, SUC_lexicon_tags_10 },
    { 0xf97f53a7, SUC_lexicon_tags_33 },
    { 0xe63f53a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623f53b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85f53b9, SUC_lexicon_tags_25 },
    { 0x5c7f53b9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8bbf53bc, SUC_lexicon_tags_14 },
    { 0x1cff53bc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bf53c6, SUC_lexicon_tags_56 },
    { 0xe3bf53c7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5df53cc, SUC_lexicon_tags_69 },
    { 0xe25f53cd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x393f53cf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917f53d6, SUC_lexicon_tags_1 },
    { 0xcfff53d7, SUC_lexicon_tags_13 },
    { 0x253f53d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1f53db, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5f53e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5f53e5, SUC_lexicon_tags_47 },
    { 0x8fdf53e5, SUC_lexicon_tags_45 },
    { 0x8c5f53e6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x527f53ea, SUC_lexicon_tags_45 },
    { 0x83ff53eb, SUC_lexicon_tags_27 },
    { 0x6d5f53ec, SUC_lexicon_tags_10 },
    { 0xebf53eb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3df53f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xff3f53f3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ff53f8, SUC_lexicon_tags_11 },
    { 0x825f53f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8eff53fb, SUC_lexicon_tags_46 },
    { 0xef5f53fc, SUC_lexicon_tags_60 },
    { 0xb5df53fd, SUC_lexicon_tags_9 },
    { 0x6f7f53fe, SUC_lexicon_tags_11 },
    { 0x4ff53ff, SUC_lexicon_tags_19 },
    { 0x27ff5400, SUC_lexicon_tags_32 },
    { 0xddf5400, SUC_lexicon_tags_1 },
    { 0x1c5f53fe, SUC_lexicon_tags_11 },
    { 0xb01f5403, SUC_lexicon_tags_30 },
    { 0xa59f5403, SUC_lexicon_tags_25 },
    { 0x343f5401, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdf5408, SUC_lexicon_tags_2 },
    { 0x20ff5408, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833f540f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403f5414, SUC_lexicon_tags_15 },
    { 0xf81f5415, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd71f5417, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba5f541b, SUC_lexicon_tags_17 },
    { 0xb93f541b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe41f541e, SUC_lexicon_tags_1 },
    { 0x1e5f541e, SUC_lexicon_tags_32 },
    { 0x429f541e, SUC_lexicon_tags_6 },
    { 0x5bdf5420, SUC_lexicon_tags_17 },
    { 0x2cdf5420, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x307f5425, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3f542b, SUC_lexicon_tags_33 },
    { 0x1fbf542b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975f5431, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x60df5434, SUC_lexicon_tags_1 },
    { 0xb81f5435, SUC_lexicon_tags_33 },
    { 0xb25f5436, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861f543c, SUC_lexicon_tags_39 },
    { 0x995f543c, SUC_lexicon_tags_12 },
    { 0x615f543e, SUC_lexicon_tags_8 },
    { 0x4e3f543f, SUC_lexicon_tags_7 },
    { 0x225f543e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2df5446, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83f5449, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5f544e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375f5453, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c3f5455, SUC_lexicon_tags_36 },
    { 0x2fbf5456, SUC_lexicon_tags_28 },
    { 0x627f5455, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9f545d, SUC_lexicon_tags_37 },
    { 0x1a7f545d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x153f5461, SUC_lexicon_tags_37 },
    { 0x8edf5462, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x83f5464, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883f546b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f5473, SUC_lexicon_tags_15 },
    { 0xaedf5474, SUC_lexicon_tags_14 },
    { 0x805f5474, SUC_lexicon_tags_1 },
    { 0xe09f5476, SUC_lexicon_tags_63 },
    { 0xb65f5476, SUC_lexicon_tags_23 },
    { 0x7b5f5475, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x215f5483, SUC_lexicon_tags_12 },
    { 0x585f5484, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30bf548e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97ff5490, SUC_lexicon_tags_7 },
    { 0x45ff5491, SUC_lexicon_tags_27 },
    { 0xd85f5492, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff5495, SUC_lexicon_tags_27 },
    { 0xcddf5496, SUC_lexicon_tags_11 },
    { 0x925f5497, SUC_lexicon_tags_45 },
    { 0x2f9f5497, SUC_lexicon_tags_7 },
    { 0x841f5496, SUC_lexicon_tags_19 },
    { 0xe61f549a, SUC_lexicon_tags_15 },
    { 0x221f549a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xd75f549e, SUC_lexicon_tags_1 },
    { 0x733f549f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x519f54a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5a9f54a4, SUC_lexicon_tags_49 },
    { 0x679f54a5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5aff54a8, SUC_lexicon_tags_19 },
    { 0xcf5f54a9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabf54ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x603f54b0, SUC_lexicon_tags_2 },
    { 0x95df54b1, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x769f54b4, SUC_lexicon_tags_49 },
    { 0x7d3f54b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665f54bd, SUC_lexicon_tags_25 },
    { 0xc59f54be, SUC_lexicon_tags_63 },
    { 0xf9ff54be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edf54c3, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f54c7, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5f54cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3f54d1, SUC_lexicon_tags_45 },
    { 0x621f54d2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1df54d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2df54dd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9f54e0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49ff54e4, SUC_lexicon_tags_17 },
    { 0xb51f54e5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07f54e9, SUC_lexicon_tags_17 },
    { 0x8dbf54ea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40df54f4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f54fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22df5506, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9f550e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd0bf5510, SUC_lexicon_tags_27 },
    { 0x42ff5511, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x1fbf5513, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa3ff5515, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfc5f5517, SUC_lexicon_tags_14 },
    { 0xac5f5518, SUC_lexicon_tags_8 },
    { 0x293f5519, SUC_lexicon_tags_43 },
    { 0x2fdf551a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2edf551c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9fff551e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xee1f5520, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93f5524, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbf5529, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac1f552e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x13df5530, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3f5534, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77f5537, SUC_lexicon_tags_37 },
    { 0x359f5538, SUC_lexicon_tags_9 },
    { 0xae1f5539, SUC_lexicon_tags_9 },
    { 0x37f5539, SUC_lexicon_tags_45 },
    { 0x207f553b, SUC_lexicon_tags_7 },
    { 0xdd3f5538, SUC_lexicon_tags_30 },
    { 0x445f553d, SUC_lexicon_tags_25 },
    { 0xffdf553e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9f5542, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacdf5544, SUC_lexicon_tags_37 },
    { 0xa8ff5545, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x86bf5547, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9f554a, SUC_lexicon_tags_7 },
    { 0xd39f554b, SUC_lexicon_tags_2 },
    { 0x57bf554c, SUC_lexicon_tags_10 },
    { 0x36ff554a, SUC_lexicon_tags_17 },
    { 0xf43f554e, SUC_lexicon_tags_17 },
    { 0x3adf554e, SUC_lexicon_tags_14 },
    { 0xf81f5550, SUC_lexicon_tags_25 },
    { 0xf0ff5551, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x705f5554, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5f555d, SUC_lexicon_tags_38 },
    { 0xdcdf555e, SUC_lexicon_tags_47 },
    { 0x785f555f, SUC_lexicon_tags_15 },
    { 0x6c9f555e, SUC_lexicon_tags_28 },
    { 0xb87f5561, SUC_lexicon_tags_17 },
    { 0x8b9f5562, SUC_lexicon_tags_11 },
    { 0x977f555f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x22bf5565, SUC_lexicon_tags_17 },
    { 0x3c3f5566, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf53f5568, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9df556b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x187f556d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf53f5570, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x65df5573, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193f5579, SUC_lexicon_tags_55 },
    { 0xcf9f5579, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35f557f, SUC_lexicon_tags_15 },
    { 0x2fdf5580, SUC_lexicon_tags_55 },
    { 0x209f5581, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x823f5583, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf7bf5585, SUC_lexicon_tags_48 },
    { 0x49df5586, SUC_lexicon_tags_33 },
    { 0x63f5586, SUC_lexicon_tags_1 },
    { 0x8cbf5586, SUC_lexicon_tags_1 },
    { 0xee3f5589, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa09f558b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b3f558f, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1f5596, SUC_lexicon_tags_2 },
    { 0x84bf5597, SUC_lexicon_tags_14 },
    { 0x521f5597, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285f559e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23f55a3, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xb4bf55a5, SUC_lexicon_tags_27 },
    { 0xf4ff55a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7fdf55a8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3f55ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7f55b2, SUC_lexicon_tags_2 },
    { 0x31ff55b3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x189f55b6, SUC_lexicon_tags_13 },
    { 0x957f55b7, SUC_lexicon_tags_0 },
    { 0x86df55b6, SUC_lexicon_tags_45 },
    { 0x175f55b9, SUC_lexicon_tags_77 },
    { 0xb1df55b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x95bf55bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9f55bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bf55c3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x667f55c5, SUC_lexicon_tags_25 },
    { 0x615f55c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa55f55c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1cdf55ca, SUC_lexicon_tags_3 },
    { 0x8e3f55ca, SUC_lexicon_tags_15 },
    { 0xc71f55cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ff55d3, SUC_lexicon_tags_1 },
    { 0xad5f55d3, SUC_lexicon_tags_30 },
    { 0xa81f55d4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x231f55d7, SUC_lexicon_tags_23 },
    { 0x155f55d7, SUC_lexicon_tags_7 },
    { 0x125f55d7, SUC_lexicon_tags_15 },
    { 0x2edf55d9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255f55de, SUC_lexicon_tags_12 },
    { 0x693f55df, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f55e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc59f55e9, SUC_lexicon_tags_27 },
    { 0xdaff55e9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61f55ed, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe2ff55ef, SUC_lexicon_tags_15 },
    { 0xffdf55f0, SUC_lexicon_tags_1 },
    { 0x94bf55f1, SUC_lexicon_tags_21 },
    { 0xc97f55f2, SUC_lexicon_tags_60 },
    { 0x18df55f0, SUC_lexicon_tags_10 },
    { 0xfb3f55f4, SUC_lexicon_tags_25 },
    { 0xce1f55f5, SUC_lexicon_tags_1 },
    { 0x75df55f6, SUC_lexicon_tags_38 },
    { 0xda1f55f7, SUC_lexicon_tags_13 },
    { 0xb45f55f8, SUC_lexicon_tags_127 },
    { 0x4fbf55f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259f55ff, SUC_lexicon_tags_2 },
    { 0x7e5f5600, SUC_lexicon_tags_24 },
    { 0x67f5600, SUC_lexicon_tags_2 },
    { 0xc09f55ff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ff5605, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7f5608, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe2bf560a, SUC_lexicon_tags_48 },
    { 0x6c9f560a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x2aff560d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x697f560f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd0ff5611, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651f5615, SUC_lexicon_tags_38 },
    { 0xd79f5616, SUC_lexicon_tags_14 },
    { 0xda1f5617, SUC_lexicon_tags_15 },
    { 0x1d7f5618, SUC_lexicon_tags_343 },
    { 0xc95f5619, SUC_lexicon_tags_69 },
    { 0x7d7f561a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x87ff561c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2a9f561e, SUC_lexicon_tags_38 },
    { 0x8b3f561f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x30bf5621, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9f5624, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5f562a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73f562e, SUC_lexicon_tags_71 },
    { 0xfa5f562f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72ff5633, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23f5636, SUC_lexicon_tags_87 },
    { 0x4b5f5637, SUC_lexicon_tags_7 },
    { 0xf89f5638, SUC_lexicon_tags_93 },
    { 0, NULL },
    { 0x7fff563a, SUC_lexicon_tags_7 },
    { 0x15df563b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc7f563d, SUC_lexicon_tags_63 },
    { 0xb25f563e, SUC_lexicon_tags_13 },
    { 0xa13f563d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbf5644, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83f5647, SUC_lexicon_tags_2 },
    { 0x2fdf5648, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbebf564a, SUC_lexicon_tags_7 },
    { 0x5a9f564b, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xd89f564d, SUC_lexicon_tags_10 },
    { 0x58df564e, SUC_lexicon_tags_7 },
    { 0x51bf564f, SUC_lexicon_tags_37 },
    { 0x595f5650, SUC_lexicon_tags_29 },
    { 0xbbdf564e, SUC_lexicon_tags_11 },
    { 0xa37f5650, SUC_lexicon_tags_9 },
    { 0xe59f564e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01f5658, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd2df565a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bf565e, SUC_lexicon_tags_2 },
    { 0x971f565f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcd9f5661, SUC_lexicon_tags_65 },
    { 0x165f5662, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea3f5668, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf566b, SUC_lexicon_tags_9 },
    { 0xeadf566c, SUC_lexicon_tags_1 },
    { 0xd1f566d, SUC_lexicon_tags_1 },
    { 0x6a3f566b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7f5671, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf59f5673, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01f5679, SUC_lexicon_tags_11 },
    { 0xb45f567a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x313f567c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f5681, SUC_lexicon_tags_7 },
    { 0xebbf5682, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5f5685, SUC_lexicon_tags_37 },
    { 0x58ff5686, SUC_lexicon_tags_15 },
    { 0xfeff5686, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x191f5689, SUC_lexicon_tags_33 },
    { 0x1a5f5689, SUC_lexicon_tags_12 },
    { 0x369f568b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbf568f, SUC_lexicon_tags_19 },
    { 0x6a7f5690, SUC_lexicon_tags_19 },
    { 0xde7f5691, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1f5698, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b1f569a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c3f569c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ff56a1, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235f56a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2cbf56aa, SUC_lexicon_tags_17 },
    { 0x90ff56ab, SUC_lexicon_tags_12 },
    { 0xf91f56ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x425f56ae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xcaff56b1, SUC_lexicon_tags_836 },
    { 0, NULL },
    { 0xb07f56b3, SUC_lexicon_tags_11 },
    { 0xddff56b4, SUC_lexicon_tags_15 },
    { 0x511f56b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x293f56b7, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5a1f56b9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xdb1f56bb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9f56bf, SUC_lexicon_tags_19 },
    { 0xf4df56c0, SUC_lexicon_tags_179 },
    { 0xe97f56c1, SUC_lexicon_tags_16 },
    { 0x2b5f56c1, SUC_lexicon_tags_33 },
    { 0x7fdf56c0, SUC_lexicon_tags_27 },
    { 0xeabf56bf, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcdff56c6, SUC_lexicon_tags_25 },
    { 0x591f56c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe13f56c9, SUC_lexicon_tags_27 },
    { 0xf5ff56ca, SUC_lexicon_tags_2 },
    { 0xa75f56ca, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebf56d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1f56db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb99f56e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7f56e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445f56e9, SUC_lexicon_tags_17 },
    { 0xacdf56ea, SUC_lexicon_tags_192 },
    { 0x23ff56eb, SUC_lexicon_tags_8 },
    { 0xf7df56eb, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9f56f6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd69f56f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x8ddf56fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74bf5702, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff5707, SUC_lexicon_tags_1 },
    { 0xe01f5707, SUC_lexicon_tags_12 },
    { 0xe03f5709, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43f5710, SUC_lexicon_tags_23 },
    { 0x8a5f5710, SUC_lexicon_tags_158 },
    { 0xbe7f5712, SUC_lexicon_tags_24 },
    { 0x7c3f5712, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1f5716, SUC_lexicon_tags_49 },
    { 0x48df5717, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83f571b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x97bf571e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ff5725, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xcbff5727, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491f572c, SUC_lexicon_tags_62 },
    { 0x7e7f572d, SUC_lexicon_tags_1 },
    { 0xb27f572e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159f5735, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5cff5737, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7edf5739, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adf5741, SUC_lexicon_tags_19 },
    { 0x251f5742, SUC_lexicon_tags_13 },
    { 0x6cbf5743, SUC_lexicon_tags_1 },
    { 0xdebf5742, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe8bf5746, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0xdf9f5748, SUC_lexicon_tags_0 },
    { 0xed5f5749, SUC_lexicon_tags_23 },
    { 0x79f574a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73f5750, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4c1f5752, SUC_lexicon_tags_481 },
    { 0x2c3f5753, SUC_lexicon_tags_7 },
    { 0x361f5753, SUC_lexicon_tags_11 },
    { 0x15f5752, SUC_lexicon_tags_25 },
    { 0xb8bf5756, SUC_lexicon_tags_0 },
    { 0x929f5757, SUC_lexicon_tags_126 },
    { 0x8d1f5756, SUC_lexicon_tags_1 },
    { 0x755f5757, SUC_lexicon_tags_33 },
    { 0x153f5759, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7f5762, SUC_lexicon_tags_49 },
    { 0x57ff5762, SUC_lexicon_tags_56 },
    { 0x8b7f5764, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597f5768, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977f576c, SUC_lexicon_tags_26 },
    { 0x21ff576c, SUC_lexicon_tags_38 },
    { 0x52df576d, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x10bf5770, SUC_lexicon_tags_17 },
    { 0x899f5770, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xafff5773, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1f5778, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73f577d, SUC_lexicon_tags_7 },
    { 0xff1f577e, SUC_lexicon_tags_45 },
    { 0x7c1f577f, SUC_lexicon_tags_7 },
    { 0x5f9f5780, SUC_lexicon_tags_10 },
    { 0x401f5781, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe1f5785, SUC_lexicon_tags_27 },
    { 0xf39f5786, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9e5f5788, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x399f578a, SUC_lexicon_tags_15 },
    { 0xde9f578b, SUC_lexicon_tags_37 },
    { 0x87f578b, SUC_lexicon_tags_19 },
    { 0x699f578d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x119f578f, SUC_lexicon_tags_221 },
    { 0xe77f5790, SUC_lexicon_tags_27 },
    { 0x129f5790, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x203f5794, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903f5798, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153f579d, SUC_lexicon_tags_15 },
    { 0x2b5f579e, SUC_lexicon_tags_15 },
    { 0x7fdf579f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x957f57a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811f57a8, SUC_lexicon_tags_25 },
    { 0x6a5f57a9, SUC_lexicon_tags_1 },
    { 0x86df57a9, SUC_lexicon_tags_8 },
    { 0xa39f57ab, SUC_lexicon_tags_11 },
    { 0xc95f57a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67f57b0, SUC_lexicon_tags_19 },
    { 0x21ff57b1, SUC_lexicon_tags_38 },
    { 0x1c7f57b1, SUC_lexicon_tags_17 },
    { 0x55bf57b3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f57b6, SUC_lexicon_tags_17 },
    { 0x787f57b6, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5f57c2, SUC_lexicon_tags_27 },
    { 0x451f57c3, SUC_lexicon_tags_11 },
    { 0x13f57c2, SUC_lexicon_tags_47 },
    { 0xdf7f57c4, SUC_lexicon_tags_70 },
    { 0xe7bf57c6, SUC_lexicon_tags_37 },
    { 0x4aff57c5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ff57d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdf57da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd3ff57dc, SUC_lexicon_tags_25 },
    { 0xbc1f57dd, SUC_lexicon_tags_12 },
    { 0xd6df57de, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bf57e1, SUC_lexicon_tags_17 },
    { 0x8bf57e1, SUC_lexicon_tags_27 },
    { 0x4f7f57e3, SUC_lexicon_tags_49 },
    { 0x50df57e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x715f57e6, SUC_lexicon_tags_25 },
    { 0x37ff57e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3f57eb, SUC_lexicon_tags_35 },
    { 0x20ff57ec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33f57f6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7f57fb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa9f57fd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3f5807, SUC_lexicon_tags_9 },
    { 0x85f5808, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf25f580a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bf580e, SUC_lexicon_tags_37 },
    { 0xb11f580f, SUC_lexicon_tags_1 },
    { 0x95ff580f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x56ff5812, SUC_lexicon_tags_43 },
    { 0x3d3f5812, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f5819, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bf581c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf65f5823, SUC_lexicon_tags_45 },
    { 0x507f5824, SUC_lexicon_tags_823 },
    { 0x1bdf5824, SUC_lexicon_tags_29 },
    { 0xbb9f5825, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93bf582f, SUC_lexicon_tags_49 },
    { 0xd23f582f, SUC_lexicon_tags_27 },
    { 0x7e1f5831, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x603f5834, SUC_lexicon_tags_33 },
    { 0xe51f5835, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145f583f, SUC_lexicon_tags_26 },
    { 0xfbbf5840, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489f5846, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x24ff5848, SUC_lexicon_tags_26 },
    { 0xdc7f5849, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x273f584b, SUC_lexicon_tags_14 },
    { 0x8a3f584c, SUC_lexicon_tags_27 },
    { 0x321f584d, SUC_lexicon_tags_63 },
    { 0xfe9f584e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x191f5851, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x31df5854, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89f5857, SUC_lexicon_tags_11 },
    { 0xeddf5858, SUC_lexicon_tags_19 },
    { 0x523f5859, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x253f585c, SUC_lexicon_tags_49 },
    { 0x645f585d, SUC_lexicon_tags_11 },
    { 0xbd3f585d, SUC_lexicon_tags_43 },
    { 0xbedf585e, SUC_lexicon_tags_77 },
    { 0x7d7f585d, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1df5863, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x42df5865, SUC_lexicon_tags_554 },
    { 0, NULL },
    { 0x3c1f5867, SUC_lexicon_tags_25 },
    { 0x48df5868, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf2ff586a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x585f586c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bf586e, SUC_lexicon_tags_30 },
    { 0x1df586e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x799f5871, SUC_lexicon_tags_35 },
    { 0xa5df5872, SUC_lexicon_tags_48 },
    { 0x3cff5872, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd19f5875, SUC_lexicon_tags_28 },
    { 0xc17f5876, SUC_lexicon_tags_1 },
    { 0x931f5877, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f587c, SUC_lexicon_tags_160 },
    { 0x65df587d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71ff5884, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd99f588c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x85df588e, SUC_lexicon_tags_17 },
    { 0x60ff588f, SUC_lexicon_tags_38 },
    { 0xf5f588e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x97f5892, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf93f5894, SUC_lexicon_tags_17 },
    { 0x5df5895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x21bf5897, SUC_lexicon_tags_35 },
    { 0xe5ff5897, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xed3f589b, SUC_lexicon_tags_16 },
    { 0xdf3f589c, SUC_lexicon_tags_25 },
    { 0x1d9f589d, SUC_lexicon_tags_68 },
    { 0xbbff589d, SUC_lexicon_tags_15 },
    { 0x7f9f589b, SUC_lexicon_tags_1 },
    { 0xf6bf58a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2eff58a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199f58aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xffbf58ad, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7f58b0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x909f58b2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1f58b5, SUC_lexicon_tags_43 },
    { 0x235f58b6, SUC_lexicon_tags_33 },
    { 0xb91f58b5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29f58bb, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0x3a9f58bd, SUC_lexicon_tags_11 },
    { 0xf9df58be, SUC_lexicon_tags_2 },
    { 0xd53f58bf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9f58c3, SUC_lexicon_tags_15 },
    { 0xe1f58c3, SUC_lexicon_tags_12 },
    { 0x6fbf58c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdf58cb, SUC_lexicon_tags_2 },
    { 0x687f58cb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2c5f58ce, SUC_lexicon_tags_12 },
    { 0xcc1f58cf, SUC_lexicon_tags_25 },
    { 0x711f58d0, SUC_lexicon_tags_31 },
    { 0xe01f58d0, SUC_lexicon_tags_1 },
    { 0xa63f58d2, SUC_lexicon_tags_25 },
    { 0x65bf58d3, SUC_lexicon_tags_17 },
    { 0x83df58d3, SUC_lexicon_tags_27 },
    { 0xe5bf58d5, SUC_lexicon_tags_2 },
    { 0x645f58ce, SUC_lexicon_tags_44 },
    { 0x8cdf58d7, SUC_lexicon_tags_47 },
    { 0x1f3f58d7, SUC_lexicon_tags_17 },
    { 0x83ff58d7, SUC_lexicon_tags_55 },
    { 0xe0ff58da, SUC_lexicon_tags_12 },
    { 0xfa5f58d7, SUC_lexicon_tags_17 },
    { 0x247f58dc, SUC_lexicon_tags_2 },
    { 0xb5bf58dd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x79bf58df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ff58e3, SUC_lexicon_tags_14 },
    { 0xc3bf58e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbf58e7, SUC_lexicon_tags_2 },
    { 0x763f58e8, SUC_lexicon_tags_47 },
    { 0x3e7f58e8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99f58ec, SUC_lexicon_tags_9 },
    { 0x55f58ec, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbf58f5, SUC_lexicon_tags_17 },
    { 0x9dff58f6, SUC_lexicon_tags_162 },
    { 0, NULL },
    { 0x9edf58f8, SUC_lexicon_tags_47 },
    { 0x647f58f9, SUC_lexicon_tags_27 },
    { 0xdabf58fa, SUC_lexicon_tags_11 },
    { 0x3c9f58fb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17f5903, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25df5907, SUC_lexicon_tags_45 },
    { 0x783f5907, SUC_lexicon_tags_27 },
    { 0x99df5909, SUC_lexicon_tags_45 },
    { 0xf89f590a, SUC_lexicon_tags_17 },
    { 0x39f590b, SUC_lexicon_tags_25 },
    { 0xc31f590c, SUC_lexicon_tags_11 },
    { 0x183f590d, SUC_lexicon_tags_12 },
    { 0x77ff590a, SUC_lexicon_tags_37 },
    { 0x291f590f, SUC_lexicon_tags_32 },
    { 0xa75f5910, SUC_lexicon_tags_6 },
    { 0xbdf5911, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x6adf5914, SUC_lexicon_tags_0 },
    { 0x7b9f5914, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161f591d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x159f591f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3c1f5921, SUC_lexicon_tags_12 },
    { 0xc7bf5922, SUC_lexicon_tags_49 },
    { 0x817f5922, SUC_lexicon_tags_19 },
    { 0x2dbf5924, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4df5927, SUC_lexicon_tags_38 },
    { 0x30ff5927, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bf592b, SUC_lexicon_tags_39 },
    { 0x983f592b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe0bf592e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9f5931, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95f5936, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05f5939, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x741f593c, SUC_lexicon_tags_541 },
    { 0, NULL },
    { 0, NULL },
    { 0x53f593f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47f5943, SUC_lexicon_tags_23 },
    { 0xe6df5943, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f5948, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ff594b, SUC_lexicon_tags_33 },
    { 0xb69f594c, SUC_lexicon_tags_17 },
    { 0x947f594d, SUC_lexicon_tags_12 },
    { 0x609f594e, SUC_lexicon_tags_15 },
    { 0x993f594f, SUC_lexicon_tags_17 },
    { 0xa27f594f, SUC_lexicon_tags_0 },
    { 0x269f5950, SUC_lexicon_tags_11 },
    { 0x983f5952, SUC_lexicon_tags_0 },
    { 0xfebf594b, SUC_lexicon_tags_1 },
    { 0xf99f594d, SUC_lexicon_tags_1419 },
    { 0x8d7f594d, SUC_lexicon_tags_23 },
    { 0x275f5956, SUC_lexicon_tags_17 },
    { 0xd3bf5957, SUC_lexicon_tags_17 },
    { 0x325f5956, SUC_lexicon_tags_23 },
    { 0x11bf5959, SUC_lexicon_tags_38 },
    { 0xd39f5958, SUC_lexicon_tags_93 },
    { 0x129f594e, SUC_lexicon_tags_7 },
    { 0x511f595c, SUC_lexicon_tags_17 },
    { 0xc41f595d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13df5962, SUC_lexicon_tags_21 },
    { 0x3f7f5963, SUC_lexicon_tags_15 },
    { 0x89f5963, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x681f5967, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x533f596c, SUC_lexicon_tags_19 },
    { 0xeeff596c, SUC_lexicon_tags_12 },
    { 0x9f596e, SUC_lexicon_tags_60 },
    { 0x4ebf596f, SUC_lexicon_tags_13 },
    { 0xedff5970, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907f5975, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423f5989, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bf598f, SUC_lexicon_tags_57 },
    { 0x503f5990, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5f5993, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe73f599a, SUC_lexicon_tags_1 },
    { 0x587f599b, SUC_lexicon_tags_3 },
    { 0xd3ff599b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa8ff599e, SUC_lexicon_tags_39 },
    { 0xb3bf599e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ff59a2, SUC_lexicon_tags_47 },
    { 0xe03f59a3, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907f59ab, SUC_lexicon_tags_77 },
    { 0x341f59ab, SUC_lexicon_tags_23 },
    { 0xf8bf59ab, SUC_lexicon_tags_17 },
    { 0xd21f59ae, SUC_lexicon_tags_38 },
    { 0xc0bf59af, SUC_lexicon_tags_65 },
    { 0x277f59ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891f59b6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbf59b9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x317f59bb, SUC_lexicon_tags_45 },
    { 0x1b3f59bb, SUC_lexicon_tags_2 },
    { 0x727f59bd, SUC_lexicon_tags_11 },
    { 0x29ff59be, SUC_lexicon_tags_45 },
    { 0xae3f59be, SUC_lexicon_tags_31 },
    { 0x1c1f59c0, SUC_lexicon_tags_27 },
    { 0x8fff59c1, SUC_lexicon_tags_43 },
    { 0x3ddf59c1, SUC_lexicon_tags_60 },
    { 0x3ddf59c0, SUC_lexicon_tags_10 },
    { 0x973f59c4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e5f59c6, SUC_lexicon_tags_30 },
    { 0xc8bf59c7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x369f59c9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1f59d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfa1f59d3, SUC_lexicon_tags_25 },
    { 0xb75f59d4, SUC_lexicon_tags_37 },
    { 0x1f5f59d3, SUC_lexicon_tags_17 },
    { 0xabbf59d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x74df59d9, SUC_lexicon_tags_17 },
    { 0x283f59da, SUC_lexicon_tags_37 },
    { 0xc9f59da, SUC_lexicon_tags_15 },
    { 0x513f59dc, SUC_lexicon_tags_37 },
    { 0x7bdf59dd, SUC_lexicon_tags_65 },
    { 0x50df59dc, SUC_lexicon_tags_7 },
    { 0x71bf59d9, SUC_lexicon_tags_49 },
    { 0x9d7f59dd, SUC_lexicon_tags_37 },
    { 0xfe7f59e1, SUC_lexicon_tags_32 },
    { 0x5d5f59e2, SUC_lexicon_tags_47 },
    { 0x461f59e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67bf59ec, SUC_lexicon_tags_1 },
    { 0xcfff59ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ff59f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x623f59f8, SUC_lexicon_tags_14 },
    { 0x64ff59f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3f59fc, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69f59ff, SUC_lexicon_tags_33 },
    { 0x97f5a00, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f5a03, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f5a08, SUC_lexicon_tags_25 },
    { 0x2cff5a08, SUC_lexicon_tags_15 },
    { 0x92bf5a0a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ff5a11, SUC_lexicon_tags_12 },
    { 0x4ebf5a12, SUC_lexicon_tags_57 },
    { 0x17f5a12, SUC_lexicon_tags_15 },
    { 0x643f5a11, SUC_lexicon_tags_26 },
    { 0x3d3f5a15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbc3f5a17, SUC_lexicon_tags_38 },
    { 0xb3bf5a17, SUC_lexicon_tags_12 },
    { 0x5bf5a19, SUC_lexicon_tags_23 },
    { 0x26bf5a19, SUC_lexicon_tags_62 },
    { 0xa87f5a17, SUC_lexicon_tags_25 },
    { 0x2adf5a1b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bf5a24, SUC_lexicon_tags_23 },
    { 0xe59f5a25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x773f5a28, SUC_lexicon_tags_0 },
    { 0x609f5a29, SUC_lexicon_tags_11 },
    { 0x7abf5a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ff5a2f, SUC_lexicon_tags_25 },
    { 0x325f5a2f, SUC_lexicon_tags_63 },
    { 0xbeff5a31, SUC_lexicon_tags_7 },
    { 0x1c5f5a32, SUC_lexicon_tags_31 },
    { 0x603f5a32, SUC_lexicon_tags_15 },
    { 0x2d9f5a33, SUC_lexicon_tags_11 },
    { 0x8f3f5a32, SUC_lexicon_tags_43 },
    { 0x32bf5a36, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ff5a39, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627f5a49, SUC_lexicon_tags_7 },
    { 0xde3f5a49, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c7f5a4c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x885f5a4f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911f5a54, SUC_lexicon_tags_45 },
    { 0xaf3f5a55, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x4edf5a58, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961f5a5d, SUC_lexicon_tags_29 },
    { 0xf65f5a5e, SUC_lexicon_tags_56 },
    { 0xf25f5a5f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdf5a68, SUC_lexicon_tags_8 },
    { 0xe69f5a69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x607f5a6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ff5a71, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9adf5a73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8f5f5a75, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45f5a7d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x661f5a7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5f5a85, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x853f5a88, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdaff5a8d, SUC_lexicon_tags_26 },
    { 0x5f5f5a8e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35f5a91, SUC_lexicon_tags_11 },
    { 0x7c5f5a92, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xacff5a94, SUC_lexicon_tags_37 },
    { 0x24df5a95, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x7a1f5a97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3f5a9d, SUC_lexicon_tags_17 },
    { 0x6a3f5a9e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739f5aa8, SUC_lexicon_tags_49 },
    { 0x301f5aa8, SUC_lexicon_tags_2 },
    { 0xeabf5aa8, SUC_lexicon_tags_63 },
    { 0x9ff5aab, SUC_lexicon_tags_37 },
    { 0xba9f5aab, SUC_lexicon_tags_11 },
    { 0x9d3f5aac, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1d7f5aaf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adf5ab6, SUC_lexicon_tags_26 },
    { 0xc9f5ab6, SUC_lexicon_tags_23 },
    { 0xb9df5ab7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2bf5aba, SUC_lexicon_tags_8 },
    { 0xd71f5abb, SUC_lexicon_tags_12 },
    { 0xc55f5abc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x469f5abf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bff5ac3, SUC_lexicon_tags_11 },
    { 0xae9f5ac4, SUC_lexicon_tags_287 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83f5ac7, SUC_lexicon_tags_17 },
    { 0x3dff5ac7, SUC_lexicon_tags_255 },
    { 0x275f5ac9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5dff5acb, SUC_lexicon_tags_7 },
    { 0xec1f5acb, SUC_lexicon_tags_2 },
    { 0x9f7f5acd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ff5ad5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc3f5ad7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501f5adc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bf5adf, SUC_lexicon_tags_39 },
    { 0x921f5ae0, SUC_lexicon_tags_43 },
    { 0x8b5f5ae1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5ae6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xd4bf5ae8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaabf5aea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x461f5aed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x317f5af0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcfff5af2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333f5af6, SUC_lexicon_tags_49 },
    { 0xc4bf5af6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x791f5b04, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91f5b07, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66df5b0d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5f5b12, SUC_lexicon_tags_182 },
    { 0xa3ff5b12, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x57df5b15, SUC_lexicon_tags_27 },
    { 0x9f3f5b15, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x487f5b19, SUC_lexicon_tags_1 },
    { 0xd87f5b19, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbf5b20, SUC_lexicon_tags_15 },
    { 0xc67f5b21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x917f5b23, SUC_lexicon_tags_120 },
    { 0, NULL },
    { 0x6ebf5b25, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5f5b28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddf5b2d, SUC_lexicon_tags_2 },
    { 0xfebf5b2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1f5b31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x269f5b33, SUC_lexicon_tags_17 },
    { 0xa09f5b34, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b1f5b38, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x2cbf5b3a, SUC_lexicon_tags_2 },
    { 0x5f1f5b3a, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa4df5b3d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x537f5b3f, SUC_lexicon_tags_12 },
    { 0x7f3f5b40, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xeabf5b42, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9f5b45, SUC_lexicon_tags_12 },
    { 0x609f5b46, SUC_lexicon_tags_25 },
    { 0x725f5b47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33f5b4b, SUC_lexicon_tags_37 },
    { 0x5fbf5b4c, SUC_lexicon_tags_19 },
    { 0x2d9f5b4b, SUC_lexicon_tags_25 },
    { 0x167f5b4d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aff5b55, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbf5b5a, SUC_lexicon_tags_25 },
    { 0x91df5b5b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x789f5b5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x217f5b61, SUC_lexicon_tags_60 },
    { 0x45bf5b62, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x125f5b64, SUC_lexicon_tags_37 },
    { 0x69df5b65, SUC_lexicon_tags_30 },
    { 0x2a1f5b66, SUC_lexicon_tags_47 },
    { 0x67f5b67, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1f5b6c, SUC_lexicon_tags_37 },
    { 0x65f5b6d, SUC_lexicon_tags_26 },
    { 0x267f5b6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1ebf5b70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6d7f5b72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbf5b7b, SUC_lexicon_tags_46 },
    { 0xf13f5b7c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749f5b82, SUC_lexicon_tags_9 },
    { 0x35f5b82, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3edf5b85, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x70df5b87, SUC_lexicon_tags_16 },
    { 0xba7f5b88, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591f5b8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3dff5b90, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3cdf5b92, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x647f5b94, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x297f5b96, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bf5b9b, SUC_lexicon_tags_11 },
    { 0xc57f5b9c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461f5bac, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5f5bb7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81f5bbe, SUC_lexicon_tags_16 },
    { 0x1dff5bbe, SUC_lexicon_tags_12 },
    { 0xbc7f5bbe, SUC_lexicon_tags_14 },
    { 0x29df5bc1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883f5bc6, SUC_lexicon_tags_37 },
    { 0x29df5bc7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xc47f5bc9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x2bf5bcb, SUC_lexicon_tags_5 },
    { 0x817f5bcc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4d7f5bce, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43f5bd1, SUC_lexicon_tags_11 },
    { 0xb95f5bd2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bf5bd5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb45f5bd8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7f1f5bda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7f5be1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xaddf5be4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5f5be7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227f5bed, SUC_lexicon_tags_12 },
    { 0x38bf5bee, SUC_lexicon_tags_26 },
    { 0xc2ff5bef, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x76df5bf1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xee7f5bf3, SUC_lexicon_tags_171 },
    { 0, NULL },
    { 0, NULL },
    { 0x377f5bf6, SUC_lexicon_tags_8 },
    { 0x5e7f5bf7, SUC_lexicon_tags_23 },
    { 0x47f5bf6, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07f5bfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x311f5c00, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6f1f5c02, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf5c06, SUC_lexicon_tags_15 },
    { 0x28ff5c07, SUC_lexicon_tags_10 },
    { 0x1fdf5c07, SUC_lexicon_tags_47 },
    { 0xed9f5c08, SUC_lexicon_tags_11 },
    { 0x8d5f5c0a, SUC_lexicon_tags_7 },
    { 0x961f5c0b, SUC_lexicon_tags_7 },
    { 0x77df5c0c, SUC_lexicon_tags_6 },
    { 0x605f5c0d, SUC_lexicon_tags_19 },
    { 0x545f5c0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9f5c15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe91f5c17, SUC_lexicon_tags_39 },
    { 0xaff5c18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x431f5c1a, SUC_lexicon_tags_49 },
    { 0xdbbf5c1a, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0xb8bf5c1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525f5c24, SUC_lexicon_tags_47 },
    { 0x909f5c25, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33f5c29, SUC_lexicon_tags_37 },
    { 0x61f5c2a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x107f5c2c, SUC_lexicon_tags_11 },
    { 0xab3f5c2d, SUC_lexicon_tags_82 },
    { 0x8f9f5c2e, SUC_lexicon_tags_25 },
    { 0xd85f5c2c, SUC_lexicon_tags_7 },
    { 0x37bf5c30, SUC_lexicon_tags_15 },
    { 0x303f5c30, SUC_lexicon_tags_23 },
    { 0x56bf5c32, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xca7f5c35, SUC_lexicon_tags_2 },
    { 0x8e7f5c36, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f9f5c38, SUC_lexicon_tags_25 },
    { 0x62bf5c39, SUC_lexicon_tags_30 },
    { 0xa85f5c3a, SUC_lexicon_tags_12 },
    { 0xb1df5c3a, SUC_lexicon_tags_2 },
    { 0x3df5c38, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x577f5c3e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x62df5c40, SUC_lexicon_tags_38 },
    { 0x1ff5c41, SUC_lexicon_tags_38 },
    { 0x91df5c40, SUC_lexicon_tags_1 },
    { 0xc63f5c43, SUC_lexicon_tags_47 },
    { 0x85df5c44, SUC_lexicon_tags_45 },
    { 0xccdf5c45, SUC_lexicon_tags_29 },
    { 0x51f5c44, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe01f5c48, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x567f5c4a, SUC_lexicon_tags_17 },
    { 0x27df5c4b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x287f5c4d, SUC_lexicon_tags_11 },
    { 0x14df5c4d, SUC_lexicon_tags_12 },
    { 0xbb9f5c4f, SUC_lexicon_tags_204 },
    { 0x7cdf5c50, SUC_lexicon_tags_19 },
    { 0x371f5c51, SUC_lexicon_tags_2 },
    { 0x207f5c52, SUC_lexicon_tags_49 },
    { 0x715f5c53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5f5c57, SUC_lexicon_tags_134 },
    { 0x2a5f5c57, SUC_lexicon_tags_2 },
    { 0x303f5c57, SUC_lexicon_tags_14 },
    { 0x25bf5c5a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ff5c5e, SUC_lexicon_tags_4 },
    { 0x353f5c5f, SUC_lexicon_tags_8 },
    { 0x993f5c60, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9f5c64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbd7f5c66, SUC_lexicon_tags_15 },
    { 0xfd1f5c67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41ff5c6b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbf5c71, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84df5c76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25f5c79, SUC_lexicon_tags_17 },
    { 0x72df5c7a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x875f5c7c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddf5c82, SUC_lexicon_tags_17 },
    { 0x9a5f5c82, SUC_lexicon_tags_54 },
    { 0x6b7f5c84, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb7bf5c86, SUC_lexicon_tags_37 },
    { 0x3b7f5c86, SUC_lexicon_tags_17 },
    { 0xd41f5c88, SUC_lexicon_tags_37 },
    { 0xcaff5c88, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x703f5c8b, SUC_lexicon_tags_17 },
    { 0x3b7f5c8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401f5c90, SUC_lexicon_tags_29 },
    { 0x501f5c90, SUC_lexicon_tags_49 },
    { 0x519f5c90, SUC_lexicon_tags_15 },
    { 0xf73f5c93, SUC_lexicon_tags_87 },
    { 0xc2ff5c93, SUC_lexicon_tags_46 },
    { 0xc5ff5c95, SUC_lexicon_tags_7 },
    { 0xbebf5c90, SUC_lexicon_tags_47 },
    { 0xad1f5c97, SUC_lexicon_tags_45 },
    { 0x845f5c94, SUC_lexicon_tags_11 },
    { 0x8fdf5c99, SUC_lexicon_tags_37 },
    { 0x429f5c92, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33df5c9e, SUC_lexicon_tags_84 },
    { 0xf15f5c9f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb35f5ca1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x387f5ca4, SUC_lexicon_tags_11 },
    { 0x27df5ca5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bf5ca9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63f5cad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5f5cb0, SUC_lexicon_tags_27 },
    { 0xa79f5cb1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77f5cb9, SUC_lexicon_tags_13 },
    { 0xd0df5cba, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2dbf5cbc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5f5cbf, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05f5cc3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5f5cc6, SUC_lexicon_tags_27 },
    { 0x2d3f5cc7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x973f5cc9, SUC_lexicon_tags_87 },
    { 0x573f5cca, SUC_lexicon_tags_2 },
    { 0xc8df5cc9, SUC_lexicon_tags_15 },
    { 0x321f5ccc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcff5cd1, SUC_lexicon_tags_11 },
    { 0x7d5f5cd2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x277f5cd5, SUC_lexicon_tags_35 },
    { 0xbfbf5cd6, SUC_lexicon_tags_47 },
    { 0xdc3f5cd6, SUC_lexicon_tags_31 },
    { 0x343f5cd6, SUC_lexicon_tags_46 },
    { 0x4e5f5cd9, SUC_lexicon_tags_243 },
    { 0xc29f5cda, SUC_lexicon_tags_11 },
    { 0x34df5cda, SUC_lexicon_tags_2 },
    { 0x17bf5cdb, SUC_lexicon_tags_16 },
    { 0x891f5cd9, SUC_lexicon_tags_60 },
    { 0x14bf5cde, SUC_lexicon_tags_7 },
    { 0x29f5cd7, SUC_lexicon_tags_11 },
    { 0x8e5f5ce0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9f5ce6, SUC_lexicon_tags_2 },
    { 0x51ff5ce7, SUC_lexicon_tags_43 },
    { 0x803f5ce8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89f5cef, SUC_lexicon_tags_33 },
    { 0x789f5cf0, SUC_lexicon_tags_33 },
    { 0x49df5cf1, SUC_lexicon_tags_25 },
    { 0x703f5cf2, SUC_lexicon_tags_37 },
    { 0x49ff5cf2, SUC_lexicon_tags_158 },
    { 0x683f5cf4, SUC_lexicon_tags_8 },
    { 0xdfff5cf5, SUC_lexicon_tags_12 },
    { 0xb8ff5cf4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33f5cfa, SUC_lexicon_tags_37 },
    { 0xcb5f5cfa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5fbf5cfd, SUC_lexicon_tags_26 },
    { 0x64df5cfe, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01f5d01, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x58ff5d04, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627f5d0a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x90ff5d0c, SUC_lexicon_tags_11 },
    { 0x7edf5d0d, SUC_lexicon_tags_63 },
    { 0xab9f5d0e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9df5d12, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe21f5d19, SUC_lexicon_tags_37 },
    { 0xd35f5d1a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6e7f5d1c, SUC_lexicon_tags_13 },
    { 0x369f5d1c, SUC_lexicon_tags_30 },
    { 0x4a7f5d1c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdf5d21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x983f5d23, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5f5d29, SUC_lexicon_tags_33 },
    { 0xd67f5d2a, SUC_lexicon_tags_7 },
    { 0xd3bf5d2b, SUC_lexicon_tags_11 },
    { 0xe43f5d29, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83f5d30, SUC_lexicon_tags_21 },
    { 0x383f5d31, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x63f5d34, SUC_lexicon_tags_27 },
    { 0x51df5d35, SUC_lexicon_tags_37 },
    { 0x73f5d35, SUC_lexicon_tags_30 },
    { 0x68bf5d34, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273f5d3b, SUC_lexicon_tags_27 },
    { 0x551f5d3c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe15f5d3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3f5d43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x3c9f5d45, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdf5d4a, SUC_lexicon_tags_0 },
    { 0xa33f5d4b, SUC_lexicon_tags_15 },
    { 0x73ff5d4c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf69f5d4e, SUC_lexicon_tags_24 },
    { 0xe31f5d4f, SUC_lexicon_tags_11 },
    { 0xd3ff5d4f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ff5d58, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff5d5b, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bf5d5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87df5d61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3f5d64, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845f5d68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xb89f5d6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7f5d6e, SUC_lexicon_tags_14 },
    { 0x415f5d6f, SUC_lexicon_tags_0 },
    { 0x8edf5d6f, SUC_lexicon_tags_0 },
    { 0x36bf5d71, SUC_lexicon_tags_17 },
    { 0xd4ff5d6f, SUC_lexicon_tags_15 },
    { 0xa97f5d6e, SUC_lexicon_tags_29 },
    { 0xfddf5d72, SUC_lexicon_tags_49 },
    { 0xd8ff5d75, SUC_lexicon_tags_11 },
    { 0xc61f5d75, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7f5d79, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x699f5d7b, SUC_lexicon_tags_27 },
    { 0x727f5d7c, SUC_lexicon_tags_0 },
    { 0xcbff5d7c, SUC_lexicon_tags_48 },
    { 0x1b9f5d7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe61f5d81, SUC_lexicon_tags_38 },
    { 0x85bf5d81, SUC_lexicon_tags_1 },
    { 0x101f5d83, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x723f5d85, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdf5d8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x151f5d92, SUC_lexicon_tags_27 },
    { 0x863f5d93, SUC_lexicon_tags_17 },
    { 0xdc5f5d94, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xe89f5d96, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331f5d9a, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x19bf5d9c, SUC_lexicon_tags_38 },
    { 0x8e9f5d9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf5da2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42bf5da4, SUC_lexicon_tags_25 },
    { 0x6a3f5da5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3f5da8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1f5dab, SUC_lexicon_tags_11 },
    { 0x519f5dac, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x54bf5dae, SUC_lexicon_tags_33 },
    { 0x94bf5daf, SUC_lexicon_tags_28 },
    { 0x8c1f5db0, SUC_lexicon_tags_17 },
    { 0x3f1f5daf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb07f5db3, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53f5db6, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x21bf5db8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xea1f5dba, SUC_lexicon_tags_15 },
    { 0x63bf5dbb, SUC_lexicon_tags_7 },
    { 0x2e9f5dbc, SUC_lexicon_tags_170 },
    { 0xfaff5dbc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedf5dc0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xf03f5dc3, SUC_lexicon_tags_28 },
    { 0xa47f5dc4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x37ff5dc6, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5d9f5dc8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe69f5dca, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x17df5dcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765f5dd2, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xef9f5dd4, SUC_lexicon_tags_30 },
    { 0x9b1f5dd5, SUC_lexicon_tags_2 },
    { 0xa31f5dd6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x651f5dd9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1f5ddd, SUC_lexicon_tags_48 },
    { 0xc77f5dde, SUC_lexicon_tags_35 },
    { 0xedff5ddf, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddf5de4, SUC_lexicon_tags_47 },
    { 0x1edf5de4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bff5de9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xb53f5deb, SUC_lexicon_tags_37 },
    { 0x579f5dec, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbb5f5dee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101f5df4, SUC_lexicon_tags_37 },
    { 0xc3ff5df5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ff5dfa, SUC_lexicon_tags_31 },
    { 0xaeff5dfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36df5e04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555f5e09, SUC_lexicon_tags_3 },
    { 0xef9f5e0a, SUC_lexicon_tags_17 },
    { 0xbb7f5e0a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa81f5e10, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7f5e13, SUC_lexicon_tags_0 },
    { 0x6d3f5e13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997f5e18, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x379f5e1a, SUC_lexicon_tags_25 },
    { 0xd17f5e1b, SUC_lexicon_tags_62 },
    { 0xd0ff5e1c, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xd31f5e1f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xbd1f5e21, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x543f5e23, SUC_lexicon_tags_1 },
    { 0xc37f5e24, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x983f5e27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1f5e2a, SUC_lexicon_tags_14 },
    { 0xdaff5e2a, SUC_lexicon_tags_10 },
    { 0xa6bf5e2b, SUC_lexicon_tags_37 },
    { 0x87ff5e2d, SUC_lexicon_tags_37 },
    { 0x653f5e2e, SUC_lexicon_tags_31 },
    { 0x5e5f5e2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5f5e34, SUC_lexicon_tags_36 },
    { 0x165f5e34, SUC_lexicon_tags_12 },
    { 0x4b3f5e36, SUC_lexicon_tags_33 },
    { 0x505f5e37, SUC_lexicon_tags_25 },
    { 0x577f5e35, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d1f5e3a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1f5e40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61f5e44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73f5e47, SUC_lexicon_tags_23 },
    { 0x819f5e47, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0bf5e4b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ff5e4e, SUC_lexicon_tags_1 },
    { 0xd47f5e4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7f5e57, SUC_lexicon_tags_0 },
    { 0xbdf5e57, SUC_lexicon_tags_25 },
    { 0xdf5f5e57, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7f5e5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467f5e65, SUC_lexicon_tags_45 },
    { 0x365f5e65, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5f5e6d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7f5e71, SUC_lexicon_tags_11 },
    { 0x2a5f5e71, SUC_lexicon_tags_37 },
    { 0xc79f5e73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5edf5e75, SUC_lexicon_tags_37 },
    { 0xaeff5e75, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x735f5e78, SUC_lexicon_tags_11 },
    { 0x723f5e79, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f1f5e7d, SUC_lexicon_tags_37 },
    { 0x92ff5e7e, SUC_lexicon_tags_7 },
    { 0x391f5e7d, SUC_lexicon_tags_37 },
    { 0x79df5e80, SUC_lexicon_tags_8 },
    { 0x6fff5e81, SUC_lexicon_tags_15 },
    { 0x165f5e82, SUC_lexicon_tags_23 },
    { 0x1bf5e83, SUC_lexicon_tags_2 },
    { 0xad3f5e80, SUC_lexicon_tags_17 },
    { 0xea1f5e7f, SUC_lexicon_tags_64 },
    { 0x147f5e7d, SUC_lexicon_tags_11 },
    { 0x769f5e87, SUC_lexicon_tags_4 },
    { 0xe8ff5e88, SUC_lexicon_tags_15 },
    { 0x445f5e86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x45f5e8b, SUC_lexicon_tags_69 },
    { 0x61bf5e8c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61df5e90, SUC_lexicon_tags_54 },
    { 0xcf5f5e91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3f5e94, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x52bf5e96, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x75bf5e99, SUC_lexicon_tags_1 },
    { 0x4ebf5e9a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ff5e9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0df5ea1, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ff5ea6, SUC_lexicon_tags_2 },
    { 0x157f5ea7, SUC_lexicon_tags_27 },
    { 0x3f7f5ea7, SUC_lexicon_tags_17 },
    { 0xf27f5ea9, SUC_lexicon_tags_3 },
    { 0x62ff5eaa, SUC_lexicon_tags_25 },
    { 0xb61f5eaa, SUC_lexicon_tags_37 },
    { 0xa0df5eac, SUC_lexicon_tags_25 },
    { 0x11f5ead, SUC_lexicon_tags_55 },
    { 0xec5f5ea8, SUC_lexicon_tags_184 },
    { 0x45f5eaa, SUC_lexicon_tags_37 },
    { 0x69f5ea6, SUC_lexicon_tags_37 },
    { 0xf53f5eb0, SUC_lexicon_tags_15 },
    { 0xc57f5eb2, SUC_lexicon_tags_53 },
    { 0x3a1f5eb3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cff5ebc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddf5ec3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb19f5ec6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x353f5ec8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1f5ecb, SUC_lexicon_tags_27 },
    { 0x62df5ecc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849f5ed1, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc01f5ed5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337f5ed9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1cff5edb, SUC_lexicon_tags_30 },
    { 0x617f5edb, SUC_lexicon_tags_23 },
    { 0x901f5edd, SUC_lexicon_tags_2 },
    { 0xa8bf5ede, SUC_lexicon_tags_48 },
    { 0xc63f5edf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1f5ee3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa07f5ee5, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1f5eec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367f5ef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc1f5ef2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x513f5ef4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15df5ef8, SUC_lexicon_tags_37 },
    { 0xf61f5ef9, SUC_lexicon_tags_17 },
    { 0x943f5efa, SUC_lexicon_tags_1 },
    { 0x7fff5efb, SUC_lexicon_tags_31 },
    { 0xd33f5efb, SUC_lexicon_tags_53 },
    { 0xf91f5efd, SUC_lexicon_tags_55 },
    { 0x1a1f5efe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc5ff5f00, SUC_lexicon_tags_85 },
    { 0xcc9f5f01, SUC_lexicon_tags_12 },
    { 0xa7bf5f02, SUC_lexicon_tags_31 },
    { 0xef1f5f03, SUC_lexicon_tags_11 },
    { 0xf85f5f00, SUC_lexicon_tags_79 },
    { 0x271f5f03, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x507f5f0a, SUC_lexicon_tags_52 },
    { 0x7aff5f0b, SUC_lexicon_tags_65 },
    { 0x401f5f0b, SUC_lexicon_tags_21 },
    { 0x9dbf5f0c, SUC_lexicon_tags_170 },
    { 0x845f5f0e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ff5f12, SUC_lexicon_tags_28 },
    { 0x973f5f12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc73f5f15, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9f5f18, SUC_lexicon_tags_60 },
    { 0x67f5f19, SUC_lexicon_tags_1 },
    { 0xb4ff5f18, SUC_lexicon_tags_47 },
    { 0x8df5f1b, SUC_lexicon_tags_15 },
    { 0x6d1f5f19, SUC_lexicon_tags_11 },
    { 0xbdf5f1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x141f5f1f, SUC_lexicon_tags_87 },
    { 0x2b5f5f20, SUC_lexicon_tags_17 },
    { 0x15ff5f21, SUC_lexicon_tags_19 },
    { 0x557f5f22, SUC_lexicon_tags_25 },
    { 0x4eff5f23, SUC_lexicon_tags_2 },
    { 0xe9f5f22, SUC_lexicon_tags_14 },
    { 0x4eff5f25, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xb67f5f27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x651f5f29, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfff5f2c, SUC_lexicon_tags_2 },
    { 0xce1f5f2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1f5f38, SUC_lexicon_tags_49 },
    { 0x4cbf5f38, SUC_lexicon_tags_14 },
    { 0x5b1f5f38, SUC_lexicon_tags_7 },
    { 0x795f5f3b, SUC_lexicon_tags_18 },
    { 0x3f1f5f3b, SUC_lexicon_tags_1 },
    { 0x86bf5f3c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527f5f43, SUC_lexicon_tags_32 },
    { 0xe89f5f43, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597f5f4a, SUC_lexicon_tags_17 },
    { 0x54bf5f4a, SUC_lexicon_tags_38 },
    { 0xd17f5f4c, SUC_lexicon_tags_37 },
    { 0xb05f5f4d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07f5f51, SUC_lexicon_tags_37 },
    { 0xec1f5f51, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x79bf5f54, SUC_lexicon_tags_14 },
    { 0xed1f5f55, SUC_lexicon_tags_43 },
    { 0xad9f5f56, SUC_lexicon_tags_7 },
    { 0xf07f5f57, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ff5f5b, SUC_lexicon_tags_46 },
    { 0x209f5f5c, SUC_lexicon_tags_7 },
    { 0x94df5f5c, SUC_lexicon_tags_43 },
    { 0xb35f5f5e, SUC_lexicon_tags_55 },
    { 0x2a9f5f5e, SUC_lexicon_tags_7 },
    { 0xa0bf5f5c, SUC_lexicon_tags_19 },
    { 0xf93f5f5d, SUC_lexicon_tags_26 },
    { 0x8f7f5f62, SUC_lexicon_tags_2 },
    { 0x463f5f62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495f5f6d, SUC_lexicon_tags_261 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63f5f70, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xc61f5f72, SUC_lexicon_tags_19 },
    { 0x4dff5f73, SUC_lexicon_tags_8 },
    { 0xf65f5f74, SUC_lexicon_tags_47 },
    { 0x869f5f74, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x33bf5f77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf5f7c, SUC_lexicon_tags_37 },
    { 0x3d3f5f7d, SUC_lexicon_tags_27 },
    { 0xf95f5f7c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607f5f83, SUC_lexicon_tags_23 },
    { 0x981f5f84, SUC_lexicon_tags_23 },
    { 0xa2df5f84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x31bf5f87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ff5f8b, SUC_lexicon_tags_49 },
    { 0xff5f8b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93f5f8f, SUC_lexicon_tags_37 },
    { 0xe4df5f90, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1f5f97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23f5f9d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71f5fa0, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdf5fa7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97f5fac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd7f5fae, SUC_lexicon_tags_187 },
    { 0x521f5faf, SUC_lexicon_tags_14 },
    { 0x395f5fb0, SUC_lexicon_tags_17 },
    { 0x193f5fb1, SUC_lexicon_tags_81 },
    { 0x5bf5fb0, SUC_lexicon_tags_15 },
    { 0x671f5fb0, SUC_lexicon_tags_15 },
    { 0x959f5fb4, SUC_lexicon_tags_43 },
    { 0x76df5fb2, SUC_lexicon_tags_45 },
    { 0xd51f5fae, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26df5fbd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcfff5fbf, SUC_lexicon_tags_3 },
    { 0xe2ff5fc0, SUC_lexicon_tags_37 },
    { 0x72bf5fc0, SUC_lexicon_tags_11 },
    { 0xb59f5fc2, SUC_lexicon_tags_17 },
    { 0x4e1f5fc2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x55f5fc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9f5fca, SUC_lexicon_tags_17 },
    { 0xcabf5fcb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddf5fcf, SUC_lexicon_tags_1 },
    { 0x8fbf5fd0, SUC_lexicon_tags_11 },
    { 0x1cdf5fd1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9f5fd4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37f5fd8, SUC_lexicon_tags_12 },
    { 0x98df5fd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9c7f5fdb, SUC_lexicon_tags_71 },
    { 0xb09f5fdc, SUC_lexicon_tags_9 },
    { 0xbe1f5fdc, SUC_lexicon_tags_37 },
    { 0x8eff5fde, SUC_lexicon_tags_2 },
    { 0x981f5fdf, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27f5fe2, SUC_lexicon_tags_29 },
    { 0x337f5fe3, SUC_lexicon_tags_8 },
    { 0xd25f5fe3, SUC_lexicon_tags_33 },
    { 0x24bf5fe5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71f5fe8, SUC_lexicon_tags_37 },
    { 0x791f5fe8, SUC_lexicon_tags_47 },
    { 0x15f5fe9, SUC_lexicon_tags_7 },
    { 0x65df5feb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x585f5fed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x563f5ff2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x32bf5ff4, SUC_lexicon_tags_37 },
    { 0x8f3f5ff5, SUC_lexicon_tags_15 },
    { 0xce5f5ff6, SUC_lexicon_tags_6 },
    { 0x971f5ff7, SUC_lexicon_tags_874 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ff5ffe, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x8f3f6000, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x729f6002, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbf6006, SUC_lexicon_tags_11 },
    { 0xb8ff6006, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19f6010, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2d5f6012, SUC_lexicon_tags_61 },
    { 0x349f6013, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbf6016, SUC_lexicon_tags_63 },
    { 0x2b7f6017, SUC_lexicon_tags_12 },
    { 0xe59f6018, SUC_lexicon_tags_19 },
    { 0xd31f6019, SUC_lexicon_tags_45 },
    { 0xdbbf6019, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39f601e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bf6021, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8cbf6023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1f6027, SUC_lexicon_tags_63 },
    { 0x1d1f6028, SUC_lexicon_tags_18 },
    { 0xe25f6028, SUC_lexicon_tags_25 },
    { 0x2c7f6029, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf6df602c, SUC_lexicon_tags_12 },
    { 0x7f9f602c, SUC_lexicon_tags_7 },
    { 0x69bf602e, SUC_lexicon_tags_47 },
    { 0x597f602e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x801f6031, SUC_lexicon_tags_7 },
    { 0x18df6032, SUC_lexicon_tags_37 },
    { 0xe3df6033, SUC_lexicon_tags_14 },
    { 0x645f6034, SUC_lexicon_tags_23 },
    { 0x6e9f6035, SUC_lexicon_tags_12 },
    { 0x231f6036, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x651f6039, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x833f603c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f6041, SUC_lexicon_tags_37 },
    { 0xd55f6042, SUC_lexicon_tags_11 },
    { 0x25ff6043, SUC_lexicon_tags_25 },
    { 0x5bff6043, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x69df6046, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x865f604c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x401f604f, SUC_lexicon_tags_12 },
    { 0x5bbf6050, SUC_lexicon_tags_14 },
    { 0x735f6051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x927f6057, SUC_lexicon_tags_37 },
    { 0xb9df6057, SUC_lexicon_tags_19 },
    { 0x641f6057, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeff605c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1a5f605e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1f6065, SUC_lexicon_tags_7 },
    { 0x42df6066, SUC_lexicon_tags_17 },
    { 0x22ff6065, SUC_lexicon_tags_36 },
    { 0x903f6066, SUC_lexicon_tags_17 },
    { 0x433f6069, SUC_lexicon_tags_37 },
    { 0x4f5f606a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdf606d, SUC_lexicon_tags_82 },
    { 0xbe5f606e, SUC_lexicon_tags_25 },
    { 0xea7f606f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bf6075, SUC_lexicon_tags_63 },
    { 0xf5df6076, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953f607b, SUC_lexicon_tags_63 },
    { 0x173f607c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb8ff607e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb81f6080, SUC_lexicon_tags_29 },
    { 0x241f6080, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x80ff6083, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd33f6085, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eff6089, SUC_lexicon_tags_43 },
    { 0x93df608a, SUC_lexicon_tags_28 },
    { 0x2e9f608a, SUC_lexicon_tags_11 },
    { 0x58df608c, SUC_lexicon_tags_68 },
    { 0x885f608c, SUC_lexicon_tags_47 },
    { 0x9a7f608b, SUC_lexicon_tags_7 },
    { 0xeb5f608f, SUC_lexicon_tags_25 },
    { 0xcf9f608d, SUC_lexicon_tags_1 },
    { 0x15f6091, SUC_lexicon_tags_45 },
    { 0x47f6091, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d1f6094, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123f609c, SUC_lexicon_tags_103 },
    { 0xee7f609c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7f60a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x933f60a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9f60a5, SUC_lexicon_tags_33 },
    { 0x707f60a6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4c1f60a8, SUC_lexicon_tags_30 },
    { 0x91ff60a9, SUC_lexicon_tags_29 },
    { 0x295f60aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21f60ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849f60b3, SUC_lexicon_tags_2 },
    { 0x433f60b3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7f60b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df60bc, SUC_lexicon_tags_21 },
    { 0x665f60bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa37f60bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65f60c2, SUC_lexicon_tags_11 },
    { 0x8adf60c3, SUC_lexicon_tags_16 },
    { 0x8fbf60c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497f60c9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fff60d3, SUC_lexicon_tags_25 },
    { 0x53ff60d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb87f60d6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab7f60da, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3f60e1, SUC_lexicon_tags_37 },
    { 0xfe5f60e2, SUC_lexicon_tags_26 },
    { 0xfb5f60e3, SUC_lexicon_tags_17 },
    { 0x88ff60e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ff60e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x437f60ea, SUC_lexicon_tags_2 },
    { 0x7ddf60ea, SUC_lexicon_tags_9 },
    { 0x345f60ec, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67f60f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d7f60f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783f60f7, SUC_lexicon_tags_23 },
    { 0x98bf60f8, SUC_lexicon_tags_17 },
    { 0x1b3f60f7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4f9f60fb, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x251f60fd, SUC_lexicon_tags_9 },
    { 0xe63f60fd, SUC_lexicon_tags_37 },
    { 0xd87f60fe, SUC_lexicon_tags_17 },
    { 0x6fdf6100, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x653f6102, SUC_lexicon_tags_56 },
    { 0xcdff6103, SUC_lexicon_tags_17 },
    { 0x27df6102, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9f6107, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fff6112, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f611a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x9bf611c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x50bf611e, SUC_lexicon_tags_8 },
    { 0x313f611e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x25df6122, SUC_lexicon_tags_43 },
    { 0x1d3f6123, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bf6129, SUC_lexicon_tags_25 },
    { 0x849f612a, SUC_lexicon_tags_46 },
    { 0x58bf612b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f612f, SUC_lexicon_tags_37 },
    { 0x2c3f612f, SUC_lexicon_tags_34 },
    { 0x51f6130, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbf6134, SUC_lexicon_tags_56 },
    { 0xac3f6135, SUC_lexicon_tags_63 },
    { 0xc25f6136, SUC_lexicon_tags_7 },
    { 0x73f6137, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3f613d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f6141, SUC_lexicon_tags_25 },
    { 0x707f6142, SUC_lexicon_tags_15 },
    { 0x42df6143, SUC_lexicon_tags_25 },
    { 0x4d7f6143, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ff614c, SUC_lexicon_tags_15 },
    { 0x7c9f614c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x4d3f614f, SUC_lexicon_tags_48 },
    { 0x115f6150, SUC_lexicon_tags_45 },
    { 0x9d7f6151, SUC_lexicon_tags_19 },
    { 0x2eff614f, SUC_lexicon_tags_14 },
    { 0x11bf6153, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x407f6158, SUC_lexicon_tags_8 },
    { 0x569f6159, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bf615c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7f615f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffff6165, SUC_lexicon_tags_49 },
    { 0xc49f6166, SUC_lexicon_tags_30 },
    { 0xf63f6167, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd27f6169, SUC_lexicon_tags_37 },
    { 0x34bf6169, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ff6172, SUC_lexicon_tags_36 },
    { 0xad9f6173, SUC_lexicon_tags_15 },
    { 0xe25f6173, SUC_lexicon_tags_106 },
    { 0xc5bf6175, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23f6179, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f617c, SUC_lexicon_tags_23 },
    { 0x66df617c, SUC_lexicon_tags_1 },
    { 0x141f617e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x907f6181, SUC_lexicon_tags_11 },
    { 0x1d5f6182, SUC_lexicon_tags_15 },
    { 0x7cbf6183, SUC_lexicon_tags_69 },
    { 0xae7f6184, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fff6187, SUC_lexicon_tags_19 },
    { 0xb71f6188, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f618f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb1bf6191, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bf619e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xf15f61a1, SUC_lexicon_tags_38 },
    { 0xc73f61a2, SUC_lexicon_tags_26 },
    { 0xa5ff61a2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x571f61a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9f61ad, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x945f61b1, SUC_lexicon_tags_17 },
    { 0xae9f61b1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf61b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ff61b9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x961f61bb, SUC_lexicon_tags_11 },
    { 0x4d3f61bc, SUC_lexicon_tags_23 },
    { 0xb57f61bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x499f61c0, SUC_lexicon_tags_33 },
    { 0x4dbf61c1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdebf61c3, SUC_lexicon_tags_2 },
    { 0x5e5f61c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc75f61cd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f61d0, SUC_lexicon_tags_36 },
    { 0x22bf61d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcc9f61d3, SUC_lexicon_tags_37 },
    { 0xc3df61d4, SUC_lexicon_tags_17 },
    { 0x187f61d3, SUC_lexicon_tags_2 },
    { 0x801f61d6, SUC_lexicon_tags_2 },
    { 0x11ff61d7, SUC_lexicon_tags_7 },
    { 0x1aff61d8, SUC_lexicon_tags_49 },
    { 0xb11f61d9, SUC_lexicon_tags_12 },
    { 0x47bf61da, SUC_lexicon_tags_2 },
    { 0x6fdf61d4, SUC_lexicon_tags_8 },
    { 0x1fbf61dc, SUC_lexicon_tags_35 },
    { 0x8f1f61dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf3ff61df, SUC_lexicon_tags_15 },
    { 0x6adf61df, SUC_lexicon_tags_2 },
    { 0x2f5f61e0, SUC_lexicon_tags_1 },
    { 0x5d3f61df, SUC_lexicon_tags_7 },
    { 0x4bbf61df, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x617f61e7, SUC_lexicon_tags_82 },
    { 0x6c3f61e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f61ec, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0xbd1f61ee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85f61f2, SUC_lexicon_tags_70 },
    { 0xc57f61f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x31ff61f5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4ff61f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09f61fe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff6201, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x3adf6204, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x19bf6206, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141f620d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x27ff620f, SUC_lexicon_tags_35 },
    { 0xd11f620f, SUC_lexicon_tags_47 },
    { 0x66bf6211, SUC_lexicon_tags_7 },
    { 0xfc3f6211, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1f6217, SUC_lexicon_tags_27 },
    { 0x733f6218, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x769f621a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xea1f621c, SUC_lexicon_tags_43 },
    { 0x7bf621d, SUC_lexicon_tags_62 },
    { 0xc5bf621e, SUC_lexicon_tags_26 },
    { 0xffff621d, SUC_lexicon_tags_11 },
    { 0x47df621d, SUC_lexicon_tags_37 },
    { 0x191f6221, SUC_lexicon_tags_6 },
    { 0x3f5f6222, SUC_lexicon_tags_17 },
    { 0xa0df6223, SUC_lexicon_tags_26 },
    { 0x763f6223, SUC_lexicon_tags_79 },
    { 0xbc7f6225, SUC_lexicon_tags_47 },
    { 0xbaff621c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x669f6229, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9f622c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x183f622e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bf6233, SUC_lexicon_tags_49 },
    { 0xd97f6233, SUC_lexicon_tags_19 },
    { 0x639f6233, SUC_lexicon_tags_25 },
    { 0x7ebf6236, SUC_lexicon_tags_26 },
    { 0xb61f6236, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x823f623a, SUC_lexicon_tags_7 },
    { 0xf8df623a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xfe3f623d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525f6241, SUC_lexicon_tags_1 },
    { 0x1c7f6241, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5df6247, SUC_lexicon_tags_17 },
    { 0x97bf6247, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x315f624a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9f624d, SUC_lexicon_tags_1 },
    { 0x205f624e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f6252, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x89f6254, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x319f6256, SUC_lexicon_tags_47 },
    { 0xcedf6257, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xe41f6259, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dff625b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc25f625d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x963f625f, SUC_lexicon_tags_11 },
    { 0x94ff625f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x397f6263, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761f626a, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0xb01f626c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bf6274, SUC_lexicon_tags_0 },
    { 0xa3f6275, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41df6283, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffff6285, SUC_lexicon_tags_60 },
    { 0x333f6285, SUC_lexicon_tags_47 },
    { 0xf5f6287, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251f628c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadff6293, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89f6298, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3f629b, SUC_lexicon_tags_48 },
    { 0x7bf629b, SUC_lexicon_tags_27 },
    { 0x4d9f629b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x521f62a2, SUC_lexicon_tags_11 },
    { 0x60df62a3, SUC_lexicon_tags_15 },
    { 0xb87f62a4, SUC_lexicon_tags_11 },
    { 0x46df62a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5f62b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12ff62b4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x1ebf62b6, SUC_lexicon_tags_33 },
    { 0x58df62b6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9f62ba, SUC_lexicon_tags_15 },
    { 0x9e7f62ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x231f62bd, SUC_lexicon_tags_11 },
    { 0x1fbf62bd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07f62c3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x717f62c5, SUC_lexicon_tags_25 },
    { 0x951f62c6, SUC_lexicon_tags_12 },
    { 0x45df62c7, SUC_lexicon_tags_396 },
    { 0x8edf62c6, SUC_lexicon_tags_23 },
    { 0xb83f62c9, SUC_lexicon_tags_11 },
    { 0xb23f62ca, SUC_lexicon_tags_15 },
    { 0x895f62c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa2bf62cd, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadf62d4, SUC_lexicon_tags_529 },
    { 0xff1f62d5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45f62db, SUC_lexicon_tags_7 },
    { 0x5eff62dc, SUC_lexicon_tags_8 },
    { 0x57bf62db, SUC_lexicon_tags_10 },
    { 0xafff62dd, SUC_lexicon_tags_1 },
    { 0x63bf62df, SUC_lexicon_tags_10 },
    { 0xc27f62e0, SUC_lexicon_tags_25 },
    { 0x9d3f62db, SUC_lexicon_tags_30 },
    { 0xb23f62db, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x22df62e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3f62eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3f62f3, SUC_lexicon_tags_17 },
    { 0x53ff62f4, SUC_lexicon_tags_25 },
    { 0xb81f62f4, SUC_lexicon_tags_10 },
    { 0xba5f62f6, SUC_lexicon_tags_63 },
    { 0x9d3f62f4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bf62fb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3f62ff, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9f6302, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x81bf6304, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x767f6306, SUC_lexicon_tags_79 },
    { 0xac1f6307, SUC_lexicon_tags_32 },
    { 0x92ff6308, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67df630c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b1f6315, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x757f6317, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91f631d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ff6320, SUC_lexicon_tags_246 },
    { 0x289f6320, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bf6324, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x75f6327, SUC_lexicon_tags_1 },
    { 0xb71f6328, SUC_lexicon_tags_7 },
    { 0x101f6329, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x653f632b, SUC_lexicon_tags_15 },
    { 0x55df632c, SUC_lexicon_tags_33 },
    { 0x9e5f632d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23f6330, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd49f6332, SUC_lexicon_tags_7 },
    { 0xca5f6333, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27f6339, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bf633c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x501f633f, SUC_lexicon_tags_31 },
    { 0x6cbf6340, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x1d9f6342, SUC_lexicon_tags_5 },
    { 0x56ff6343, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98df6347, SUC_lexicon_tags_2 },
    { 0xdf9f6348, SUC_lexicon_tags_11 },
    { 0x963f6349, SUC_lexicon_tags_8 },
    { 0x6b9f6347, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ff634e, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x591f6351, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f6354, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb51f6358, SUC_lexicon_tags_28 },
    { 0x98ff6359, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf635d, SUC_lexicon_tags_14 },
    { 0x85ff635e, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23f6363, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x5b3f6365, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91f6368, SUC_lexicon_tags_47 },
    { 0x33f6368, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x215f636b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x663f636d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67f6373, SUC_lexicon_tags_19 },
    { 0x19bf6373, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebf6378, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc5f6381, SUC_lexicon_tags_29 },
    { 0x343f6381, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95f6387, SUC_lexicon_tags_25 },
    { 0xd89f6387, SUC_lexicon_tags_15 },
    { 0x601f6389, SUC_lexicon_tags_0 },
    { 0x50df6387, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ff6391, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7f6395, SUC_lexicon_tags_37 },
    { 0x41bf6396, SUC_lexicon_tags_34 },
    { 0x89f6397, SUC_lexicon_tags_15 },
    { 0xa7df6397, SUC_lexicon_tags_35 },
    { 0xb5df6399, SUC_lexicon_tags_37 },
    { 0xf69f6399, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdb9f639c, SUC_lexicon_tags_12 },
    { 0x165f639d, SUC_lexicon_tags_108 },
    { 0xf51f639c, SUC_lexicon_tags_47 },
    { 0x4f5f639c, SUC_lexicon_tags_11 },
    { 0x68ff63a0, SUC_lexicon_tags_37 },
    { 0xb6ff63a0, SUC_lexicon_tags_19 },
    { 0x835f63a2, SUC_lexicon_tags_37 },
    { 0x7c7f63a1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8df63a7, SUC_lexicon_tags_17 },
    { 0xe1bf63a7, SUC_lexicon_tags_25 },
    { 0xdd7f63a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x81df63ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07f63b0, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19f63b9, SUC_lexicon_tags_12 },
    { 0x339f63ba, SUC_lexicon_tags_27 },
    { 0xfadf63bb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ff63bf, SUC_lexicon_tags_11 },
    { 0x9e1f63c0, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221f63c4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aff63cb, SUC_lexicon_tags_0 },
    { 0x497f63cc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95f63d0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xff5f63d2, SUC_lexicon_tags_38 },
    { 0xf05f63d3, SUC_lexicon_tags_37 },
    { 0x17bf63d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x507f63d7, SUC_lexicon_tags_47 },
    { 0x66ff63d8, SUC_lexicon_tags_25 },
    { 0x477f63d8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x32ff63db, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdf63e2, SUC_lexicon_tags_1 },
    { 0xf63f63e3, SUC_lexicon_tags_61 },
    { 0x883f63e2, SUC_lexicon_tags_37 },
    { 0x219f63e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa37f63e8, SUC_lexicon_tags_16 },
    { 0x8a7f63e8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bf63ed, SUC_lexicon_tags_17 },
    { 0x5ddf63ee, SUC_lexicon_tags_63 },
    { 0x4eff63ee, SUC_lexicon_tags_10 },
    { 0x5bff63f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f63f4, SUC_lexicon_tags_17 },
    { 0x275f63f4, SUC_lexicon_tags_45 },
    { 0xf39f63f6, SUC_lexicon_tags_84 },
    { 0x503f63f7, SUC_lexicon_tags_65 },
    { 0xa13f63f8, SUC_lexicon_tags_29 },
    { 0x219f63f9, SUC_lexicon_tags_101 },
    { 0x711f63fa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1df6402, SUC_lexicon_tags_29 },
    { 0xfbf6402, SUC_lexicon_tags_113 },
    { 0x15f6404, SUC_lexicon_tags_11 },
    { 0xcc5f6405, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f640a, SUC_lexicon_tags_11 },
    { 0x3f7f640b, SUC_lexicon_tags_14 },
    { 0xb09f640c, SUC_lexicon_tags_47 },
    { 0x635f640d, SUC_lexicon_tags_253 },
    { 0x6d1f640e, SUC_lexicon_tags_26 },
    { 0x8c9f640f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc99f6411, SUC_lexicon_tags_27 },
    { 0x285f6412, SUC_lexicon_tags_38 },
    { 0x3c5f6413, SUC_lexicon_tags_80 },
    { 0x66df6414, SUC_lexicon_tags_25 },
    { 0x9ddf6414, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933f641a, SUC_lexicon_tags_48 },
    { 0xabdf641b, SUC_lexicon_tags_11 },
    { 0x689f641a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf6420, SUC_lexicon_tags_25 },
    { 0x689f6421, SUC_lexicon_tags_230 },
    { 0, NULL },
    { 0, NULL },
    { 0x59bf6424, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeff6427, SUC_lexicon_tags_5 },
    { 0x639f6428, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16bf642a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bf642e, SUC_lexicon_tags_48 },
    { 0x3d7f642f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ff6435, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403f643b, SUC_lexicon_tags_10 },
    { 0xb9df643b, SUC_lexicon_tags_19 },
    { 0xe13f643d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b5f6442, SUC_lexicon_tags_2 },
    { 0x395f6442, SUC_lexicon_tags_11 },
    { 0x76df6443, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5adf6446, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f644a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7f644e, SUC_lexicon_tags_30 },
    { 0xb59f644f, SUC_lexicon_tags_11 },
    { 0xd05f644f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12ff6452, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x35f6454, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bff645d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f6466, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbeff646b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1f646f, SUC_lexicon_tags_14 },
    { 0xdd1f6470, SUC_lexicon_tags_60 },
    { 0x279f6471, SUC_lexicon_tags_25 },
    { 0xb33f6471, SUC_lexicon_tags_49 },
    { 0x1c7f6473, SUC_lexicon_tags_25 },
    { 0xa1ff6471, SUC_lexicon_tags_14 },
    { 0x2c3f6475, SUC_lexicon_tags_31 },
    { 0xd29f6476, SUC_lexicon_tags_23 },
    { 0x13f6470, SUC_lexicon_tags_49 },
    { 0xe13f6478, SUC_lexicon_tags_31 },
    { 0xe25f6474, SUC_lexicon_tags_25 },
    { 0xf0bf647a, SUC_lexicon_tags_39 },
    { 0xb29f6471, SUC_lexicon_tags_17 },
    { 0x125f6479, SUC_lexicon_tags_17 },
    { 0x681f647d, SUC_lexicon_tags_11 },
    { 0x719f647d, SUC_lexicon_tags_10 },
    { 0x78ff647f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x741f6481, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x5d1f6483, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x50ff6485, SUC_lexicon_tags_19 },
    { 0x6b7f6486, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbf648d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc95f6490, SUC_lexicon_tags_7 },
    { 0xca1f6491, SUC_lexicon_tags_11 },
    { 0x4fff6492, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabf6498, SUC_lexicon_tags_32 },
    { 0xc89f6499, SUC_lexicon_tags_3 },
    { 0xbaff6499, SUC_lexicon_tags_55 },
    { 0x8f7f649a, SUC_lexicon_tags_19 },
    { 0x81df649c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5f64a4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5f64a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x679f64ab, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405f64b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43f64bd, SUC_lexicon_tags_17 },
    { 0x623f64be, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165f64c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x771f64c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1f64cc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63bf64ce, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0x47f64d0, SUC_lexicon_tags_56 },
    { 0x1fbf64d0, SUC_lexicon_tags_30 },
    { 0xca7f64d2, SUC_lexicon_tags_17 },
    { 0x60ff64d3, SUC_lexicon_tags_81 },
    { 0xa3f64d1, SUC_lexicon_tags_43 },
    { 0x803f64d5, SUC_lexicon_tags_14 },
    { 0x323f64d1, SUC_lexicon_tags_25 },
    { 0x5eff64d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa47f64d9, SUC_lexicon_tags_7 },
    { 0x4adf64da, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135f64de, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x955f64e0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xafdf64e3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x645f64e5, SUC_lexicon_tags_32 },
    { 0x563f64e6, SUC_lexicon_tags_26 },
    { 0xca1f64e7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5f64eb, SUC_lexicon_tags_32 },
    { 0x789f64ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa39f64ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdf64f1, SUC_lexicon_tags_25 },
    { 0xaa7f64f1, SUC_lexicon_tags_43 },
    { 0xcabf64f2, SUC_lexicon_tags_33 },
    { 0xc53f64f1, SUC_lexicon_tags_37 },
    { 0xfc9f64f4, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bff6501, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7a7f6503, SUC_lexicon_tags_17 },
    { 0x7fff6504, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bff6507, SUC_lexicon_tags_17 },
    { 0xb07f6508, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa01f650a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dff650d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd63f6510, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9f6513, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7f651a, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x6bff651c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x875f651e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x61df6520, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x1adf6522, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x16bf6524, SUC_lexicon_tags_0 },
    { 0x633f6525, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa4ff6527, SUC_lexicon_tags_43 },
    { 0xc49f6528, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131f652f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x79df6532, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x635f6534, SUC_lexicon_tags_14 },
    { 0xfdff6535, SUC_lexicon_tags_23 },
    { 0x55f6536, SUC_lexicon_tags_35 },
    { 0x9e5f6537, SUC_lexicon_tags_15 },
    { 0x93f6538, SUC_lexicon_tags_85 },
    { 0xb05f6539, SUC_lexicon_tags_38 },
    { 0x64ff653a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8f5f653c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9f6541, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x717f6546, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7f6549, SUC_lexicon_tags_14 },
    { 0xf99f654a, SUC_lexicon_tags_0 },
    { 0x49bf654a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x217f6551, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x877f6554, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdf6559, SUC_lexicon_tags_12 },
    { 0x9e5f655a, SUC_lexicon_tags_38 },
    { 0x4fff655b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd95f655e, SUC_lexicon_tags_43 },
    { 0x8a5f655f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f6561, SUC_lexicon_tags_7 },
    { 0x9c1f6562, SUC_lexicon_tags_3 },
    { 0xdb7f6563, SUC_lexicon_tags_1 },
    { 0xfbf6563, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bff6567, SUC_lexicon_tags_43 },
    { 0x32ff6567, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bf6570, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385f6574, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9f6580, SUC_lexicon_tags_31 },
    { 0xcedf6581, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x63bf6583, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2dff6586, SUC_lexicon_tags_15 },
    { 0x989f6586, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391f658b, SUC_lexicon_tags_87 },
    { 0x34bf658c, SUC_lexicon_tags_11 },
    { 0x14ff658b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff6593, SUC_lexicon_tags_41 },
    { 0xce9f6593, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3a3f6596, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b1f6598, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df659b, SUC_lexicon_tags_63 },
    { 0xc97f659c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33f65a1, SUC_lexicon_tags_14 },
    { 0xd7f65a1, SUC_lexicon_tags_7 },
    { 0xccdf65a3, SUC_lexicon_tags_17 },
    { 0xd67f65a4, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x893f65a7, SUC_lexicon_tags_19 },
    { 0x2e3f65a8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xfcdf65aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x72ff65ac, SUC_lexicon_tags_48 },
    { 0xfb3f65ad, SUC_lexicon_tags_14 },
    { 0x859f65ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483f65b2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bf65b7, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff65ba, SUC_lexicon_tags_37 },
    { 0xd59f65bb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569f65c2, SUC_lexicon_tags_37 },
    { 0xc5f65c3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bf65cc, SUC_lexicon_tags_60 },
    { 0xff9f65cd, SUC_lexicon_tags_47 },
    { 0xd79f65ce, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x693f65d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb8ff65d2, SUC_lexicon_tags_31 },
    { 0x8ddf65d3, SUC_lexicon_tags_23 },
    { 0xfcbf65d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57f65db, SUC_lexicon_tags_11 },
    { 0x5d9f65dc, SUC_lexicon_tags_29 },
    { 0x57f65dd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x85bf65df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x145f65e1, SUC_lexicon_tags_1 },
    { 0xf71f65e2, SUC_lexicon_tags_11 },
    { 0xb07f65e1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1f65e7, SUC_lexicon_tags_29 },
    { 0x59bf65e8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9f65ef, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1bbf65f1, SUC_lexicon_tags_25 },
    { 0x7eff65f2, SUC_lexicon_tags_23 },
    { 0x271f65f1, SUC_lexicon_tags_11 },
    { 0x9e7f65f4, SUC_lexicon_tags_46 },
    { 0xb7df65f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdbbf65f7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x591f65fb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23df65fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825f6602, SUC_lexicon_tags_11 },
    { 0xef7f6602, SUC_lexicon_tags_57 },
    { 0x2edf6602, SUC_lexicon_tags_160 },
    { 0x4c5f6602, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f660a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x863f660d, SUC_lexicon_tags_14 },
    { 0xb03f660e, SUC_lexicon_tags_0 },
    { 0x57ff660f, SUC_lexicon_tags_8 },
    { 0xe51f660f, SUC_lexicon_tags_77 },
    { 0xb2df6611, SUC_lexicon_tags_48 },
    { 0x7f5f6610, SUC_lexicon_tags_46 },
    { 0x451f6610, SUC_lexicon_tags_12 },
    { 0xdff6614, SUC_lexicon_tags_48 },
    { 0xe0bf6614, SUC_lexicon_tags_72 },
    { 0x4bf6616, SUC_lexicon_tags_2 },
    { 0xda5f6611, SUC_lexicon_tags_7 },
    { 0x7d1f6618, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0x82df661a, SUC_lexicon_tags_12 },
    { 0xaf5f661b, SUC_lexicon_tags_91 },
    { 0x4b9f661c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adf6621, SUC_lexicon_tags_11 },
    { 0x7c3f6622, SUC_lexicon_tags_109 },
    { 0xe6ff6623, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xf93f6625, SUC_lexicon_tags_40 },
    { 0x18ff6626, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd63f662d, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7f6631, SUC_lexicon_tags_25 },
    { 0xee1f6632, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77f6636, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x635f6638, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe49f663a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x171f663c, SUC_lexicon_tags_1 },
    { 0xa67f663c, SUC_lexicon_tags_17 },
    { 0xfd1f663c, SUC_lexicon_tags_7 },
    { 0xaebf663c, SUC_lexicon_tags_25 },
    { 0xc3df6640, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabf6643, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519f6648, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe21f664a, SUC_lexicon_tags_17 },
    { 0xb5ff664b, SUC_lexicon_tags_11 },
    { 0x85ff664a, SUC_lexicon_tags_19 },
    { 0xfd9f664d, SUC_lexicon_tags_60 },
    { 0xd8bf664d, SUC_lexicon_tags_55 },
    { 0xe5f664b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9f6655, SUC_lexicon_tags_23 },
    { 0x75ff6656, SUC_lexicon_tags_15 },
    { 0x4b7f6655, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05f6660, SUC_lexicon_tags_13 },
    { 0x295f6661, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x93f6663, SUC_lexicon_tags_2 },
    { 0x2c9f6664, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1cff6666, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38ff6668, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1f666c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bff6671, SUC_lexicon_tags_17 },
    { 0xbb9f6672, SUC_lexicon_tags_0 },
    { 0xca5f6671, SUC_lexicon_tags_68 },
    { 0x80ff6672, SUC_lexicon_tags_1 },
    { 0xec7f6675, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdf6677, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xadff6679, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3f667c, SUC_lexicon_tags_60 },
    { 0x73f667d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91f6681, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x289f6683, SUC_lexicon_tags_37 },
    { 0x369f6684, SUC_lexicon_tags_35 },
    { 0xeb7f6685, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd7bf6687, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xcddf6689, SUC_lexicon_tags_25 },
    { 0x235f668a, SUC_lexicon_tags_17 },
    { 0xf55f668b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf21f668d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x103f668f, SUC_lexicon_tags_0 },
    { 0x861f6690, SUC_lexicon_tags_103 },
    { 0x93f6691, SUC_lexicon_tags_80 },
    { 0xce5f6692, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x433f6695, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d9f669e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe19f66a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc9f66a2, SUC_lexicon_tags_12 },
    { 0x105f66a3, SUC_lexicon_tags_7 },
    { 0xb21f66a3, SUC_lexicon_tags_12 },
    { 0x523f66a4, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x19ff66a7, SUC_lexicon_tags_31 },
    { 0x303f66a8, SUC_lexicon_tags_25 },
    { 0x485f66a9, SUC_lexicon_tags_39 },
    { 0x145f66aa, SUC_lexicon_tags_104 },
    { 0xf89f66ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71f66b1, SUC_lexicon_tags_17 },
    { 0xbe1f66b1, SUC_lexicon_tags_35 },
    { 0x8e7f66b3, SUC_lexicon_tags_2 },
    { 0xd7df66b3, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x119f66b7, SUC_lexicon_tags_92 },
    { 0x5bdf66b8, SUC_lexicon_tags_11 },
    { 0x19bf66b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50ff66c5, SUC_lexicon_tags_1 },
    { 0x97f66c6, SUC_lexicon_tags_17 },
    { 0x2f7f66c5, SUC_lexicon_tags_15 },
    { 0x183f66c8, SUC_lexicon_tags_1 },
    { 0x2cdf66c9, SUC_lexicon_tags_1 },
    { 0xea9f66ca, SUC_lexicon_tags_16 },
    { 0xed5f66c7, SUC_lexicon_tags_23 },
    { 0x997f66c5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987f66d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3f66d9, SUC_lexicon_tags_63 },
    { 0xeebf66da, SUC_lexicon_tags_17 },
    { 0x6b9f66db, SUC_lexicon_tags_0 },
    { 0x5d5f66dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5f66e1, SUC_lexicon_tags_25 },
    { 0x6ff66e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x307f66e5, SUC_lexicon_tags_55 },
    { 0x8c9f66e6, SUC_lexicon_tags_0 },
    { 0x5b9f66e7, SUC_lexicon_tags_11 },
    { 0x8dff66e8, SUC_lexicon_tags_37 },
    { 0xcdff66e5, SUC_lexicon_tags_1 },
    { 0x385f66ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x599f66ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bf66ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c5f66f1, SUC_lexicon_tags_1 },
    { 0x5a1f66f2, SUC_lexicon_tags_15 },
    { 0x5dff66f3, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83f66f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f66fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17f6704, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9f6708, SUC_lexicon_tags_14 },
    { 0x525f6709, SUC_lexicon_tags_56 },
    { 0xce9f6709, SUC_lexicon_tags_17 },
    { 0xe7ff670a, SUC_lexicon_tags_1 },
    { 0x1a7f6708, SUC_lexicon_tags_29 },
    { 0x1e9f670d, SUC_lexicon_tags_29 },
    { 0x66bf670e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ff6713, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495f671b, SUC_lexicon_tags_62 },
    { 0x329f671c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93f6720, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ff6723, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61f6728, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9df672b, SUC_lexicon_tags_2 },
    { 0x6e7f672c, SUC_lexicon_tags_14 },
    { 0x78bf672d, SUC_lexicon_tags_37 },
    { 0x75bf672e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6cbf6730, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5f9f6732, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6d5f6734, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x88bf6736, SUC_lexicon_tags_9 },
    { 0x7f9f6737, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe41f6739, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f6741, SUC_lexicon_tags_40 },
    { 0x957f6742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92df6746, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x427f6748, SUC_lexicon_tags_29 },
    { 0xa0bf6748, SUC_lexicon_tags_26 },
    { 0x9e1f6749, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3fbf674c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x34df674f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7f6755, SUC_lexicon_tags_25 },
    { 0x25ff6755, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xdf3f6758, SUC_lexicon_tags_11 },
    { 0xc25f6758, SUC_lexicon_tags_7 },
    { 0x17f6759, SUC_lexicon_tags_37 },
    { 0x1b7f675b, SUC_lexicon_tags_60 },
    { 0xa7bf6758, SUC_lexicon_tags_32 },
    { 0x371f675c, SUC_lexicon_tags_26 },
    { 0x1b5f675e, SUC_lexicon_tags_87 },
    { 0xf85f675f, SUC_lexicon_tags_21 },
    { 0x669f6760, SUC_lexicon_tags_1 },
    { 0xf8bf675f, SUC_lexicon_tags_11 },
    { 0x133f6762, SUC_lexicon_tags_63 },
    { 0xa39f6763, SUC_lexicon_tags_47 },
    { 0x4dbf6760, SUC_lexicon_tags_11 },
    { 0xe21f675f, SUC_lexicon_tags_19 },
    { 0xa65f6765, SUC_lexicon_tags_19 },
    { 0xa3ff6767, SUC_lexicon_tags_7 },
    { 0x88ff675f, SUC_lexicon_tags_37 },
    { 0x335f675f, SUC_lexicon_tags_33 },
    { 0x411f6766, SUC_lexicon_tags_16 },
    { 0x7e9f676b, SUC_lexicon_tags_15 },
    { 0x159f676c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x50df676e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc63f6770, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddf6775, SUC_lexicon_tags_0 },
    { 0xebdf6775, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85bf6780, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdf6783, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117f6789, SUC_lexicon_tags_19 },
    { 0xdcbf6789, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebff678c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x247f678e, SUC_lexicon_tags_27 },
    { 0x1d1f678f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfeff6796, SUC_lexicon_tags_37 },
    { 0x67bf6797, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x445f679a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x299f679d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653f67a1, SUC_lexicon_tags_1 },
    { 0x645f67a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd9ff67a4, SUC_lexicon_tags_11 },
    { 0x81ff67a5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbf67a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39f67ac, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa25f67ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f67b1, SUC_lexicon_tags_11 },
    { 0xe51f67b1, SUC_lexicon_tags_37 },
    { 0xa9df67b3, SUC_lexicon_tags_15 },
    { 0xfd5f67b4, SUC_lexicon_tags_2 },
    { 0x74df67b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41f67bf, SUC_lexicon_tags_49 },
    { 0x9f5f67c0, SUC_lexicon_tags_6 },
    { 0x62df67bf, SUC_lexicon_tags_28 },
    { 0x5b3f67c2, SUC_lexicon_tags_41 },
    { 0xb53f67c2, SUC_lexicon_tags_47 },
    { 0xe29f67c4, SUC_lexicon_tags_65 },
    { 0xc5ff67c4, SUC_lexicon_tags_71 },
    { 0xfb9f67c4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaf1f67c8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x64bf67cb, SUC_lexicon_tags_6 },
    { 0xde1f67cc, SUC_lexicon_tags_12 },
    { 0x5f5f67cb, SUC_lexicon_tags_25 },
    { 0x1d7f67ce, SUC_lexicon_tags_32 },
    { 0xbd7f67ce, SUC_lexicon_tags_14 },
    { 0xd41f67d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x215f67d2, SUC_lexicon_tags_12 },
    { 0xdbdf67d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xccbf67d6, SUC_lexicon_tags_49 },
    { 0xa5ff67d6, SUC_lexicon_tags_17 },
    { 0xb55f67d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa43f67db, SUC_lexicon_tags_11 },
    { 0x889f67dc, SUC_lexicon_tags_47 },
    { 0x687f67dc, SUC_lexicon_tags_14 },
    { 0x12bf67de, SUC_lexicon_tags_49 },
    { 0x6d1f67df, SUC_lexicon_tags_43 },
    { 0x11f67e0, SUC_lexicon_tags_15 },
    { 0x6b3f67de, SUC_lexicon_tags_23 },
    { 0x20bf67e2, SUC_lexicon_tags_37 },
    { 0x56bf67e3, SUC_lexicon_tags_47 },
    { 0x679f67dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc1bf67e6, SUC_lexicon_tags_223 },
    { 0x60bf67e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x349f67e9, SUC_lexicon_tags_7 },
    { 0x683f67e9, SUC_lexicon_tags_45 },
    { 0x203f67eb, SUC_lexicon_tags_2 },
    { 0x881f67ec, SUC_lexicon_tags_29 },
    { 0xa4df67eb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3abf67ef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb07f67f3, SUC_lexicon_tags_15 },
    { 0x59df67f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d3f67f7, SUC_lexicon_tags_15 },
    { 0xaa7f67f8, SUC_lexicon_tags_0 },
    { 0x5b9f67f9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x159f67fc, SUC_lexicon_tags_16 },
    { 0xfcff67fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x227f67ff, SUC_lexicon_tags_33 },
    { 0x5ddf67ff, SUC_lexicon_tags_0 },
    { 0x765f6801, SUC_lexicon_tags_3 },
    { 0xc7df6801, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9f6805, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bf6808, SUC_lexicon_tags_19 },
    { 0xd95f6809, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x14bf680c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x167f680e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x505f6810, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd7f6812, SUC_lexicon_tags_2 },
    { 0x4cff6813, SUC_lexicon_tags_43 },
    { 0x1a5f6814, SUC_lexicon_tags_14 },
    { 0xf9ff6815, SUC_lexicon_tags_5 },
    { 0xc1bf6814, SUC_lexicon_tags_11 },
    { 0xb8df6814, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddf681a, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bf6828, SUC_lexicon_tags_16 },
    { 0x3fff6829, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x86ff682b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x58bf682d, SUC_lexicon_tags_33 },
    { 0xfa7f682d, SUC_lexicon_tags_17 },
    { 0x993f682e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbe1f6831, SUC_lexicon_tags_9 },
    { 0x7e9f6831, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23f6835, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1bf6838, SUC_lexicon_tags_63 },
    { 0x52bf6839, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bf683f, SUC_lexicon_tags_15 },
    { 0x6a9f6840, SUC_lexicon_tags_49 },
    { 0x661f6841, SUC_lexicon_tags_37 },
    { 0x6d7f6840, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9f684c, SUC_lexicon_tags_2 },
    { 0xe23f684d, SUC_lexicon_tags_47 },
    { 0xf55f684e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbbf6850, SUC_lexicon_tags_8 },
    { 0x829f6851, SUC_lexicon_tags_47 },
    { 0x2abf6852, SUC_lexicon_tags_39 },
    { 0x88bf6853, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25f6857, SUC_lexicon_tags_2 },
    { 0x4eff6858, SUC_lexicon_tags_25 },
    { 0xcf9f6859, SUC_lexicon_tags_37 },
    { 0xc6df6859, SUC_lexicon_tags_0 },
    { 0xfb7f685b, SUC_lexicon_tags_47 },
    { 0x9e3f6858, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57f6862, SUC_lexicon_tags_47 },
    { 0x28ff6862, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5f6866, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x675f6869, SUC_lexicon_tags_63 },
    { 0xd99f686a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dff686e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85f687a, SUC_lexicon_tags_1 },
    { 0xf21f687b, SUC_lexicon_tags_12 },
    { 0x41f687c, SUC_lexicon_tags_14 },
    { 0x8f1f687a, SUC_lexicon_tags_33 },
    { 0x3b1f687e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x32df6881, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x65df6883, SUC_lexicon_tags_148 },
    { 0xd49f6884, SUC_lexicon_tags_28 },
    { 0x273f6884, SUC_lexicon_tags_11 },
    { 0xbf9f6886, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xef5f6889, SUC_lexicon_tags_11 },
    { 0xda1f688a, SUC_lexicon_tags_31 },
    { 0x497f688b, SUC_lexicon_tags_60 },
    { 0x93f688c, SUC_lexicon_tags_17 },
    { 0xed9f688c, SUC_lexicon_tags_2 },
    { 0x251f688b, SUC_lexicon_tags_45 },
    { 0x41bf688f, SUC_lexicon_tags_11 },
    { 0x5bf688f, SUC_lexicon_tags_2 },
    { 0x4a5f6891, SUC_lexicon_tags_30 },
    { 0x8e3f6892, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x279f6896, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x15ff6898, SUC_lexicon_tags_11 },
    { 0xd87f6899, SUC_lexicon_tags_1 },
    { 0xe55f689a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9df68a2, SUC_lexicon_tags_17 },
    { 0xd77f68a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x445f68a6, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x853f68a9, SUC_lexicon_tags_37 },
    { 0x94bf68a9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb0ff68ac, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf7f68b3, SUC_lexicon_tags_19 },
    { 0x1cbf68b4, SUC_lexicon_tags_7 },
    { 0xc31f68b3, SUC_lexicon_tags_32 },
    { 0x5c7f68b6, SUC_lexicon_tags_47 },
    { 0x241f68b7, SUC_lexicon_tags_1 },
    { 0xbadf68b4, SUC_lexicon_tags_13 },
    { 0x85bf68b9, SUC_lexicon_tags_25 },
    { 0x60ff68b6, SUC_lexicon_tags_9 },
    { 0x58df68bb, SUC_lexicon_tags_7 },
    { 0x21df68b7, SUC_lexicon_tags_7 },
    { 0x1f68bc, SUC_lexicon_tags_38 },
    { 0xcebf68bc, SUC_lexicon_tags_12 },
    { 0x697f68b4, SUC_lexicon_tags_17 },
    { 0x52ff68bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27f68c7, SUC_lexicon_tags_336 },
    { 0xe5f68c7, SUC_lexicon_tags_37 },
    { 0x9f7f68c7, SUC_lexicon_tags_25 },
    { 0xd2df68ca, SUC_lexicon_tags_26 },
    { 0x52df68c8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x72df68ce, SUC_lexicon_tags_15 },
    { 0x781f68ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x1f5f68d1, SUC_lexicon_tags_31 },
    { 0x875f68d2, SUC_lexicon_tags_12 },
    { 0xbc1f68d2, SUC_lexicon_tags_11 },
    { 0x4e1f68d3, SUC_lexicon_tags_37 },
    { 0x73df68d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbf68d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x22df68da, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305f68e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9f68ec, SUC_lexicon_tags_12 },
    { 0xb5ff68ed, SUC_lexicon_tags_15 },
    { 0xcb9f68ed, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbff68f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35f68f7, SUC_lexicon_tags_14 },
    { 0x577f68f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83ff68fb, SUC_lexicon_tags_38 },
    { 0x9c5f68fb, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x769f68ff, SUC_lexicon_tags_17 },
    { 0x455f68ff, SUC_lexicon_tags_37 },
    { 0xae7f68ff, SUC_lexicon_tags_5 },
    { 0xc23f68ff, SUC_lexicon_tags_1 },
    { 0x31f6903, SUC_lexicon_tags_61 },
    { 0x111f68ff, SUC_lexicon_tags_1 },
    { 0xaa3f6904, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4ff6909, SUC_lexicon_tags_7 },
    { 0x42ff6909, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf4ff690c, SUC_lexicon_tags_27 },
    { 0x5bbf690d, SUC_lexicon_tags_836 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fff6910, SUC_lexicon_tags_62 },
    { 0x545f6911, SUC_lexicon_tags_47 },
    { 0xdb7f6911, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5dbf6914, SUC_lexicon_tags_17 },
    { 0x283f6915, SUC_lexicon_tags_37 },
    { 0x7f7f6915, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x509f6919, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f691e, SUC_lexicon_tags_2 },
    { 0xe9f691e, SUC_lexicon_tags_11 },
    { 0x7fdf691f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa4ff6922, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x795f6924, SUC_lexicon_tags_11 },
    { 0x5ddf6925, SUC_lexicon_tags_71 },
    { 0xa79f6926, SUC_lexicon_tags_92 },
    { 0x9b3f6924, SUC_lexicon_tags_15 },
    { 0xd75f6928, SUC_lexicon_tags_2 },
    { 0x3d5f6925, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71bf692b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ff692e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77f6932, SUC_lexicon_tags_0 },
    { 0xbc9f6933, SUC_lexicon_tags_77 },
    { 0xa39f6934, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x109f6936, SUC_lexicon_tags_2 },
    { 0x82bf6937, SUC_lexicon_tags_1 },
    { 0xe3f6937, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5f693d, SUC_lexicon_tags_25 },
    { 0xce9f693e, SUC_lexicon_tags_43 },
    { 0xf6bf693f, SUC_lexicon_tags_19 },
    { 0xf5df693e, SUC_lexicon_tags_61 },
    { 0x533f6941, SUC_lexicon_tags_37 },
    { 0x139f6942, SUC_lexicon_tags_8 },
    { 0xec5f693d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837f6947, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7b9f6949, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbf694c, SUC_lexicon_tags_16 },
    { 0x3f3f694d, SUC_lexicon_tags_60 },
    { 0xf31f694e, SUC_lexicon_tags_37 },
    { 0x4a7f694d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x49df6952, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979f6956, SUC_lexicon_tags_17 },
    { 0xfd5f6956, SUC_lexicon_tags_8 },
    { 0x909f6958, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bf695f, SUC_lexicon_tags_37 },
    { 0x4b5f6960, SUC_lexicon_tags_47 },
    { 0xd3f6961, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50df6968, SUC_lexicon_tags_23 },
    { 0xf9df6969, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddf696d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bf6977, SUC_lexicon_tags_25 },
    { 0x7e3f6978, SUC_lexicon_tags_1 },
    { 0x447f6978, SUC_lexicon_tags_6 },
    { 0x983f697a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ff697d, SUC_lexicon_tags_27 },
    { 0x59f697e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4df6983, SUC_lexicon_tags_25 },
    { 0xadbf6984, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68df6988, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95f6996, SUC_lexicon_tags_26 },
    { 0x38ff6996, SUC_lexicon_tags_43 },
    { 0x9e1f6998, SUC_lexicon_tags_5 },
    { 0xf95f6999, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc39f699b, SUC_lexicon_tags_1 },
    { 0x545f699c, SUC_lexicon_tags_25 },
    { 0x617f699d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddf69a0, SUC_lexicon_tags_25 },
    { 0xea5f69a1, SUC_lexicon_tags_7 },
    { 0x503f69a1, SUC_lexicon_tags_63 },
    { 0x451f69a3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1f69a6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137f69ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ff69ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x885f69b1, SUC_lexicon_tags_56 },
    { 0xce9f69b2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493f69ba, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x62bf69bc, SUC_lexicon_tags_17 },
    { 0xce5f69bc, SUC_lexicon_tags_27 },
    { 0x6c1f69be, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5f69c3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2df69c6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb65f69c8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xeabf69ca, SUC_lexicon_tags_57 },
    { 0xdc5f69cb, SUC_lexicon_tags_8 },
    { 0x80bf69cb, SUC_lexicon_tags_33 },
    { 0x907f69cc, SUC_lexicon_tags_11 },
    { 0xaa7f69cd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x299f69d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35df69d6, SUC_lexicon_tags_37 },
    { 0x77df69d7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9f69df, SUC_lexicon_tags_25 },
    { 0x341f69df, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe41f69e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe11f69e9, SUC_lexicon_tags_35 },
    { 0xfc3f69ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bf69ee, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x123f69f2, SUC_lexicon_tags_12 },
    { 0x8a5f69f2, SUC_lexicon_tags_55 },
    { 0xe75f69f3, SUC_lexicon_tags_25 },
    { 0xe65f69f5, SUC_lexicon_tags_30 },
    { 0x589f69f6, SUC_lexicon_tags_45 },
    { 0x275f69f3, SUC_lexicon_tags_33 },
    { 0x27df69f3, SUC_lexicon_tags_16 },
    { 0x913f69f9, SUC_lexicon_tags_17 },
    { 0x33df69f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9f69fd, SUC_lexicon_tags_32 },
    { 0x695f69fe, SUC_lexicon_tags_1 },
    { 0xa69f69fd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e1f6a01, SUC_lexicon_tags_2 },
    { 0xc7ff6a01, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5f6a08, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75f6a0f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdf6a15, SUC_lexicon_tags_7 },
    { 0x757f6a15, SUC_lexicon_tags_16 },
    { 0x7e5f6a17, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851f6a1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9f6a21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x287f6a24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567f6a32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe65f6a34, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffff6a3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbd7f6a3d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1f6a41, SUC_lexicon_tags_11 },
    { 0xab5f6a41, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99f6a45, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdc9f6a47, SUC_lexicon_tags_11 },
    { 0xfb9f6a48, SUC_lexicon_tags_2 },
    { 0xad7f6a49, SUC_lexicon_tags_10 },
    { 0x56bf6a4a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfff6a52, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37df6a57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x54df6a5a, SUC_lexicon_tags_12 },
    { 0x80ff6a5b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93df6a60, SUC_lexicon_tags_56 },
    { 0xe57f6a60, SUC_lexicon_tags_6 },
    { 0x837f6a62, SUC_lexicon_tags_9 },
    { 0x369f6a63, SUC_lexicon_tags_49 },
    { 0xe9ff6a62, SUC_lexicon_tags_38 },
    { 0x431f6a65, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xc89f6a67, SUC_lexicon_tags_7 },
    { 0xf17f6a67, SUC_lexicon_tags_25 },
    { 0x34ff6a68, SUC_lexicon_tags_7 },
    { 0x5ebf6a6a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72bf6a70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x617f6a73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc2df6a76, SUC_lexicon_tags_27 },
    { 0xe97f6a77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bf6a7b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdf6a81, SUC_lexicon_tags_9 },
    { 0x249f6a81, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3f6a85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x949f6a88, SUC_lexicon_tags_63 },
    { 0x881f6a88, SUC_lexicon_tags_84 },
    { 0xb7ff6a8a, SUC_lexicon_tags_17 },
    { 0x7ebf6a89, SUC_lexicon_tags_38 },
    { 0xe07f6a8b, SUC_lexicon_tags_37 },
    { 0x9dbf6a88, SUC_lexicon_tags_9 },
    { 0xd1f6a8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x27f6a90, SUC_lexicon_tags_2 },
    { 0x627f6a91, SUC_lexicon_tags_63 },
    { 0xfc5f6a92, SUC_lexicon_tags_33 },
    { 0xf4bf6a91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x60df6a9e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7f6aa2, SUC_lexicon_tags_51 },
    { 0x2fbf6aa3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f5f6aa5, SUC_lexicon_tags_0 },
    { 0xc79f6aa6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23f6aab, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e7f6aae, SUC_lexicon_tags_33 },
    { 0xc8df6aaf, SUC_lexicon_tags_63 },
    { 0x85df6ab0, SUC_lexicon_tags_19 },
    { 0x1c7f6ab0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfff6ab7, SUC_lexicon_tags_11 },
    { 0x75ff6ab7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bf6abc, SUC_lexicon_tags_17 },
    { 0x1f5f6abd, SUC_lexicon_tags_70 },
    { 0x36ff6abc, SUC_lexicon_tags_31 },
    { 0x5c1f6abf, SUC_lexicon_tags_63 },
    { 0x4c9f6ac0, SUC_lexicon_tags_1 },
    { 0xa47f6abf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7f6ac4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91f6acb, SUC_lexicon_tags_15 },
    { 0x7c9f6acc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbf6ad2, SUC_lexicon_tags_26 },
    { 0x475f6ad2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8edf6ad7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xeb9f6ad9, SUC_lexicon_tags_62 },
    { 0x5eff6ada, SUC_lexicon_tags_17 },
    { 0x79ff6ada, SUC_lexicon_tags_12 },
    { 0xdc1f6adc, SUC_lexicon_tags_32 },
    { 0x895f6add, SUC_lexicon_tags_48 },
    { 0xea3f6adb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcb9f6ae0, SUC_lexicon_tags_17 },
    { 0x171f6ae1, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xda5f6ae3, SUC_lexicon_tags_19 },
    { 0x75f6ae4, SUC_lexicon_tags_2 },
    { 0x637f6ae4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1dbf6ae7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7df6aeb, SUC_lexicon_tags_17 },
    { 0x325f6aec, SUC_lexicon_tags_2 },
    { 0x48bf6aec, SUC_lexicon_tags_8 },
    { 0xf6ff6aec, SUC_lexicon_tags_1 },
    { 0x75ff6aed, SUC_lexicon_tags_12 },
    { 0x4bf6af0, SUC_lexicon_tags_65 },
    { 0x99bf6af0, SUC_lexicon_tags_12 },
    { 0xfddf6af2, SUC_lexicon_tags_9 },
    { 0x89bf6af3, SUC_lexicon_tags_31 },
    { 0x351f6aed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf93f6af6, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x83bf6af9, SUC_lexicon_tags_68 },
    { 0x209f6afa, SUC_lexicon_tags_112 },
    { 0xd4df6afb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x279f6afd, SUC_lexicon_tags_0 },
    { 0x2df6afd, SUC_lexicon_tags_45 },
    { 0x45df6aff, SUC_lexicon_tags_2 },
    { 0x72bf6afe, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c7f6b05, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1aff6b09, SUC_lexicon_tags_16 },
    { 0x733f6b09, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bff6b0d, SUC_lexicon_tags_46 },
    { 0x7b7f6b0d, SUC_lexicon_tags_37 },
    { 0x757f6b0f, SUC_lexicon_tags_10 },
    { 0x931f6b10, SUC_lexicon_tags_45 },
    { 0xd9df6b11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647f6b17, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115f6b22, SUC_lexicon_tags_0 },
    { 0x6f9f6b23, SUC_lexicon_tags_55 },
    { 0x91ff6b23, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0xb01f6b26, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3f6b2a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x69ff6b2d, SUC_lexicon_tags_55 },
    { 0x12df6b2d, SUC_lexicon_tags_17 },
    { 0xdd7f6b2f, SUC_lexicon_tags_17 },
    { 0x847f6b2e, SUC_lexicon_tags_7 },
    { 0x1c1f6b31, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7f6b35, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55f6b39, SUC_lexicon_tags_2 },
    { 0x5d7f6b3a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281f6b41, SUC_lexicon_tags_33 },
    { 0xa49f6b41, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x65f6b44, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bf6b48, SUC_lexicon_tags_37 },
    { 0x32ff6b49, SUC_lexicon_tags_63 },
    { 0x4cff6b49, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7df6b4d, SUC_lexicon_tags_7 },
    { 0xcc5f6b4e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59f6b52, SUC_lexicon_tags_8 },
    { 0xf69f6b53, SUC_lexicon_tags_66 },
    { 0xe3ff6b53, SUC_lexicon_tags_48 },
    { 0x65f6b55, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b5f6b58, SUC_lexicon_tags_33 },
    { 0x853f6b59, SUC_lexicon_tags_47 },
    { 0x931f6b5a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2dbf6b5c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21f6b61, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25f6b65, SUC_lexicon_tags_12 },
    { 0x1e7f6b66, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1f6b69, SUC_lexicon_tags_33 },
    { 0x2f1f6b69, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd5f6b6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89f6b72, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bf6b76, SUC_lexicon_tags_2 },
    { 0x1c9f6b77, SUC_lexicon_tags_70 },
    { 0x8e7f6b77, SUC_lexicon_tags_27 },
    { 0x69f6b78, SUC_lexicon_tags_16 },
    { 0x56ff6b7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbf6b7f, SUC_lexicon_tags_50 },
    { 0x455f6b7f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x35ff6b83, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x64df6b85, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bf6b8b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23f6b91, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x691f6b95, SUC_lexicon_tags_29 },
    { 0x459f6b96, SUC_lexicon_tags_11 },
    { 0x84bf6b97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321f6b9e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bf6ba2, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95f6ba7, SUC_lexicon_tags_38 },
    { 0x2a1f6ba8, SUC_lexicon_tags_198 },
    { 0x5a3f6ba8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05f6bb0, SUC_lexicon_tags_31 },
    { 0x663f6bb1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3f6bb7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ff6bbb, SUC_lexicon_tags_37 },
    { 0x71df6bbb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x579f6bbe, SUC_lexicon_tags_87 },
    { 0xd21f6bbf, SUC_lexicon_tags_60 },
    { 0xc3f6bc0, SUC_lexicon_tags_11 },
    { 0xf57f6bbe, SUC_lexicon_tags_49 },
    { 0xec5f6bc2, SUC_lexicon_tags_30 },
    { 0x3f9f6bc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5f6bc8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf0ff6bca, SUC_lexicon_tags_37 },
    { 0x8b5f6bca, SUC_lexicon_tags_23 },
    { 0x59f6bca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83f6bd1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919f6bd6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdf6bde, SUC_lexicon_tags_15 },
    { 0x5b7f6bdf, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69f6be4, SUC_lexicon_tags_201 },
    { 0x659f6be4, SUC_lexicon_tags_0 },
    { 0x803f6be6, SUC_lexicon_tags_6 },
    { 0xb99f6be6, SUC_lexicon_tags_6 },
    { 0xdc9f6be8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x753f6bea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc6df6bec, SUC_lexicon_tags_13 },
    { 0x99bf6bec, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fff6bfc, SUC_lexicon_tags_12 },
    { 0xc6bf6bfd, SUC_lexicon_tags_2 },
    { 0x1eff6bfd, SUC_lexicon_tags_33 },
    { 0x4a5f6bff, SUC_lexicon_tags_15 },
    { 0x153f6c00, SUC_lexicon_tags_103 },
    { 0x19df6c00, SUC_lexicon_tags_6 },
    { 0x293f6c02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x299f6c04, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddf6c0b, SUC_lexicon_tags_15 },
    { 0x9a3f6c0b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdf6c13, SUC_lexicon_tags_45 },
    { 0x88df6c13, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa35f6c16, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xf23f6c18, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea9f6c1a, SUC_lexicon_tags_2 },
    { 0x43ff6c1b, SUC_lexicon_tags_30 },
    { 0x29df6c1b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bf6c28, SUC_lexicon_tags_25 },
    { 0xf05f6c29, SUC_lexicon_tags_46 },
    { 0xcc5f6c2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325f6c31, SUC_lexicon_tags_32 },
    { 0x399f6c31, SUC_lexicon_tags_26 },
    { 0xa79f6c33, SUC_lexicon_tags_8 },
    { 0xc1df6c34, SUC_lexicon_tags_25 },
    { 0x505f6c35, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1f6c39, SUC_lexicon_tags_9 },
    { 0xfb7f6c3a, SUC_lexicon_tags_15 },
    { 0x30bf6c3b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405f6c40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddff6c44, SUC_lexicon_tags_17 },
    { 0xfe3f6c45, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4abf6c4b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bf6c4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc69f6c51, SUC_lexicon_tags_25 },
    { 0x601f6c51, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469f6c56, SUC_lexicon_tags_25 },
    { 0x8dbf6c56, SUC_lexicon_tags_11 },
    { 0xaa3f6c57, SUC_lexicon_tags_52 },
    { 0x155f6c57, SUC_lexicon_tags_49 },
    { 0xf87f6c56, SUC_lexicon_tags_10 },
    { 0x3f7f6c5b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bf6c5e, SUC_lexicon_tags_6 },
    { 0x58ff6c5e, SUC_lexicon_tags_33 },
    { 0x7c3f6c60, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe37f6c63, SUC_lexicon_tags_17 },
    { 0xb61f6c64, SUC_lexicon_tags_40 },
    { 0x15f6c65, SUC_lexicon_tags_10 },
    { 0x539f6c66, SUC_lexicon_tags_12 },
    { 0x537f6c67, SUC_lexicon_tags_43 },
    { 0xb31f6c66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6bdf6c6a, SUC_lexicon_tags_35 },
    { 0xfd1f6c6a, SUC_lexicon_tags_17 },
    { 0x771f6c6c, SUC_lexicon_tags_7 },
    { 0xbf6c6a, SUC_lexicon_tags_2 },
    { 0x2f3f6c6e, SUC_lexicon_tags_6 },
    { 0x25bf6c6d, SUC_lexicon_tags_12 },
    { 0x17ff6c70, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49f6c73, SUC_lexicon_tags_38 },
    { 0x19f6c74, SUC_lexicon_tags_1 },
    { 0xf79f6c73, SUC_lexicon_tags_68 },
    { 0xb2df6c76, SUC_lexicon_tags_30 },
    { 0xe59f6c77, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x683f6c7c, SUC_lexicon_tags_57 },
    { 0x3fdf6c7d, SUC_lexicon_tags_49 },
    { 0x22ff6c7d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511f6c82, SUC_lexicon_tags_11 },
    { 0x4d1f6c83, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161f6c88, SUC_lexicon_tags_60 },
    { 0x775f6c89, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577f6c8d, SUC_lexicon_tags_23 },
    { 0x9b9f6c8d, SUC_lexicon_tags_43 },
    { 0xffbf6c8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39f6c96, SUC_lexicon_tags_2 },
    { 0x855f6c96, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd7f6c9d, SUC_lexicon_tags_7 },
    { 0xe3f6c9d, SUC_lexicon_tags_25 },
    { 0x1bdf6c9f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59f6ca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa0bf6ca3, SUC_lexicon_tags_14 },
    { 0x60df6ca4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9fff6ca6, SUC_lexicon_tags_387 },
    { 0xf31f6ca7, SUC_lexicon_tags_2 },
    { 0x29df6ca6, SUC_lexicon_tags_34 },
    { 0x757f6ca9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x73bf6cab, SUC_lexicon_tags_7 },
    { 0x681f6cac, SUC_lexicon_tags_7 },
    { 0xda3f6cab, SUC_lexicon_tags_37 },
    { 0xd0ff6cae, SUC_lexicon_tags_26 },
    { 0x6a3f6cae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcff6cb3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6d9f6cb5, SUC_lexicon_tags_23 },
    { 0x8c5f6cb6, SUC_lexicon_tags_26 },
    { 0x387f6cb5, SUC_lexicon_tags_66 },
    { 0xc7bf6cb8, SUC_lexicon_tags_31 },
    { 0x6dbf6cb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2bff6cbb, SUC_lexicon_tags_11 },
    { 0x77ff6cbc, SUC_lexicon_tags_29 },
    { 0x5e9f6cbd, SUC_lexicon_tags_8 },
    { 0x2f9f6cbe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb45f6cc2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bf6cc5, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951f6cca, SUC_lexicon_tags_30 },
    { 0x537f6cca, SUC_lexicon_tags_47 },
    { 0xb2df6cca, SUC_lexicon_tags_1635 },
    { 0, NULL },
    { 0x70df6cce, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67f6cd2, SUC_lexicon_tags_84 },
    { 0x33ff6cd3, SUC_lexicon_tags_391 },
    { 0x359f6cd3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7f6cdb, SUC_lexicon_tags_11 },
    { 0x44df6cdc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x945f6cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221f6ce3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9f6ce8, SUC_lexicon_tags_1 },
    { 0x717f6ce9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ff6cef, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681f6cf3, SUC_lexicon_tags_8 },
    { 0x499f6cf4, SUC_lexicon_tags_5 },
    { 0x6f7f6cf5, SUC_lexicon_tags_14 },
    { 0x44df6cf3, SUC_lexicon_tags_7 },
    { 0x697f6cf6, SUC_lexicon_tags_27 },
    { 0xe57f6cf8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x14df6cfa, SUC_lexicon_tags_37 },
    { 0x943f6cfb, SUC_lexicon_tags_14 },
    { 0xccdf6cfc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde3f6cfe, SUC_lexicon_tags_37 },
    { 0x7a1f6cff, SUC_lexicon_tags_23 },
    { 0x653f6cff, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x415f6d02, SUC_lexicon_tags_2 },
    { 0x911f6d02, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x61f6d05, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ff6d0a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f6d0d, SUC_lexicon_tags_60 },
    { 0x193f6d0e, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x985f6d11, SUC_lexicon_tags_25 },
    { 0x469f6d11, SUC_lexicon_tags_0 },
    { 0x975f6d13, SUC_lexicon_tags_23 },
    { 0xee7f6d14, SUC_lexicon_tags_198 },
    { 0x33bf6d11, SUC_lexicon_tags_23 },
    { 0x699f6d16, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbc5f6d18, SUC_lexicon_tags_1 },
    { 0x145f6d18, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391f6d1e, SUC_lexicon_tags_28 },
    { 0xe79f6d1e, SUC_lexicon_tags_37 },
    { 0x24bf6d1e, SUC_lexicon_tags_1223 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423f6d24, SUC_lexicon_tags_12 },
    { 0xe19f6d25, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f6d28, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe73f6d2a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ff6d2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x483f6d2f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427f6d33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x59f6d36, SUC_lexicon_tags_11 },
    { 0xdc1f6d37, SUC_lexicon_tags_1 },
    { 0x157f6d37, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9f6d3b, SUC_lexicon_tags_25 },
    { 0x6d7f6d3b, SUC_lexicon_tags_17 },
    { 0x5e5f6d3d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x55f6d3f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bf6d44, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7f6d49, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x64ff6d4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22df6d51, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ff6d57, SUC_lexicon_tags_30 },
    { 0x74ff6d58, SUC_lexicon_tags_37 },
    { 0xc1ff6d59, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbc9f6d5b, SUC_lexicon_tags_17 },
    { 0x113f6d5c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf6d61, SUC_lexicon_tags_30 },
    { 0xcd7f6d61, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99ff6d66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc49f6d69, SUC_lexicon_tags_2 },
    { 0x557f6d69, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117f6d6f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadf6d75, SUC_lexicon_tags_7 },
    { 0x5aff6d76, SUC_lexicon_tags_7 },
    { 0x29ff6d76, SUC_lexicon_tags_0 },
    { 0xa8bf6d77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x20bf6d7a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41f6d7d, SUC_lexicon_tags_15 },
    { 0x293f6d7e, SUC_lexicon_tags_138 },
    { 0x833f6d7f, SUC_lexicon_tags_7 },
    { 0xa33f6d80, SUC_lexicon_tags_33 },
    { 0xc7df6d81, SUC_lexicon_tags_1 },
    { 0x8f5f6d81, SUC_lexicon_tags_11 },
    { 0xd1df6d83, SUC_lexicon_tags_1 },
    { 0x1d1f6d84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa21f6d86, SUC_lexicon_tags_1 },
    { 0x421f6d87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x46df6d89, SUC_lexicon_tags_55 },
    { 0x48ff6d89, SUC_lexicon_tags_8 },
    { 0x6f3f6d8b, SUC_lexicon_tags_39 },
    { 0x379f6d8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f6d90, SUC_lexicon_tags_37 },
    { 0x679f6d91, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3adf6d95, SUC_lexicon_tags_17 },
    { 0xacbf6d96, SUC_lexicon_tags_17 },
    { 0xae3f6d97, SUC_lexicon_tags_15 },
    { 0x3b5f6d98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5f6d9f, SUC_lexicon_tags_7 },
    { 0x1b3f6da0, SUC_lexicon_tags_3 },
    { 0xb4df6da1, SUC_lexicon_tags_14 },
    { 0xdeff6d9f, SUC_lexicon_tags_12 },
    { 0x751f6d9f, SUC_lexicon_tags_37 },
    { 0x1e5f6da4, SUC_lexicon_tags_17 },
    { 0x365f6da5, SUC_lexicon_tags_12 },
    { 0xa31f6da6, SUC_lexicon_tags_68 },
    { 0x7b9f6da7, SUC_lexicon_tags_1 },
    { 0xe97f6da6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fff6dad, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6b7f6daf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8fdf6db1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9f6db4, SUC_lexicon_tags_7 },
    { 0x7abf6db4, SUC_lexicon_tags_7 },
    { 0xe45f6db6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xf25f6db9, SUC_lexicon_tags_47 },
    { 0x77df6db9, SUC_lexicon_tags_15 },
    { 0x9cff6dbb, SUC_lexicon_tags_37 },
    { 0x2adf6dbb, SUC_lexicon_tags_7 },
    { 0xb3ff6dbd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0ff6dc7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x189f6dc9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa23f6dcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x337f6dcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143f6dd3, SUC_lexicon_tags_25 },
    { 0x3d5f6dd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27f6dd7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb67f6dd9, SUC_lexicon_tags_11 },
    { 0x11f6dd9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7df6dde, SUC_lexicon_tags_0 },
    { 0xd6ff6ddf, SUC_lexicon_tags_37 },
    { 0x79df6de0, SUC_lexicon_tags_49 },
    { 0x83df6de0, SUC_lexicon_tags_15 },
    { 0xa11f6de2, SUC_lexicon_tags_25 },
    { 0xdb9f6de3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767f6de7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x40ff6dea, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x7f3f6dec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x629f6df2, SUC_lexicon_tags_45 },
    { 0x171f6df3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x219f6df5, SUC_lexicon_tags_9 },
    { 0x8b1f6df6, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7f6dfb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83f6dfe, SUC_lexicon_tags_359 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1f6e01, SUC_lexicon_tags_11 },
    { 0x815f6e02, SUC_lexicon_tags_64 },
    { 0xf01f6e03, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdf6e06, SUC_lexicon_tags_19 },
    { 0xcc5f6e07, SUC_lexicon_tags_63 },
    { 0xc1ff6e06, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9f6e10, SUC_lexicon_tags_82 },
    { 0x7fbf6e10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3f6e18, SUC_lexicon_tags_7 },
    { 0xb2df6e19, SUC_lexicon_tags_25 },
    { 0x5f3f6e1a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x22bf6e1d, SUC_lexicon_tags_39 },
    { 0xd2ff6e1e, SUC_lexicon_tags_15 },
    { 0xc83f6e1d, SUC_lexicon_tags_69 },
    { 0x191f6e20, SUC_lexicon_tags_61 },
    { 0x451f6e21, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5df6e26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91f6e2a, SUC_lexicon_tags_2 },
    { 0x93ff6e2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5f6e30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59f6e35, SUC_lexicon_tags_49 },
    { 0xf65f6e35, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b3f6e38, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x103f6e3b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xab3f6e3e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bf6e42, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3a5f6e44, SUC_lexicon_tags_37 },
    { 0x423f6e45, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7df6e4b, SUC_lexicon_tags_47 },
    { 0x3cdf6e4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x50bf6e4f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67f6e51, SUC_lexicon_tags_66 },
    { 0x291f6e52, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bf6e59, SUC_lexicon_tags_63 },
    { 0xba7f6e5a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x703f6e5c, SUC_lexicon_tags_25 },
    { 0x2bff6e5d, SUC_lexicon_tags_111 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9f6e60, SUC_lexicon_tags_15 },
    { 0xd95f6e61, SUC_lexicon_tags_114 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbf6e65, SUC_lexicon_tags_33 },
    { 0xa35f6e65, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xebf6e68, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x109f6e6a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdf6e71, SUC_lexicon_tags_18 },
    { 0xfedf6e72, SUC_lexicon_tags_31 },
    { 0x3c3f6e73, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2c9f6e75, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15f6e7a, SUC_lexicon_tags_19 },
    { 0x4aff6e7b, SUC_lexicon_tags_37 },
    { 0x407f6e7b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221f6e80, SUC_lexicon_tags_25 },
    { 0x5df6e81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17f6e86, SUC_lexicon_tags_12 },
    { 0xba1f6e86, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4f7f6e89, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x283f6e90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ff6e95, SUC_lexicon_tags_63 },
    { 0x93f6e96, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65bf6ea2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdf6ea6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf6ea9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5f6eac, SUC_lexicon_tags_17 },
    { 0xfc7f6ead, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873f6eb1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb5f6eb3, SUC_lexicon_tags_26 },
    { 0x3ebf6eb4, SUC_lexicon_tags_23 },
    { 0xb6ff6eb4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x185f6eb8, SUC_lexicon_tags_17 },
    { 0x6aff6eb9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07f6ebe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x51f6ec1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445f6ec8, SUC_lexicon_tags_38 },
    { 0x59ff6ec9, SUC_lexicon_tags_30 },
    { 0xcc9f6eca, SUC_lexicon_tags_47 },
    { 0xd13f6ecb, SUC_lexicon_tags_17 },
    { 0x5d3f6ecc, SUC_lexicon_tags_37 },
    { 0xa03f6ecd, SUC_lexicon_tags_12 },
    { 0xfd3f6ecb, SUC_lexicon_tags_37 },
    { 0xeabf6ecf, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x63f6ed1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ff6ed4, SUC_lexicon_tags_15 },
    { 0x43bf6ed4, SUC_lexicon_tags_17 },
    { 0xccdf6ed6, SUC_lexicon_tags_57 },
    { 0x225f6ed7, SUC_lexicon_tags_46 },
    { 0x401f6ed5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7dff6eda, SUC_lexicon_tags_37 },
    { 0x18bf6eda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xfadf6ede, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4df6ee3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ff6ee8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417f6eec, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5f6ef0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x22bf6ef3, SUC_lexicon_tags_9 },
    { 0x91f6ef4, SUC_lexicon_tags_23 },
    { 0xbdf6ef4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1f6ef9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x44df6efb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x72df6efe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfedf6f00, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6df6f05, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcf3f6f07, SUC_lexicon_tags_28 },
    { 0x877f6f08, SUC_lexicon_tags_37 },
    { 0x73ff6f07, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6e1f6f0b, SUC_lexicon_tags_6 },
    { 0x4edf6f0c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x16df6f0f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x41ff6f11, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x195f6f13, SUC_lexicon_tags_1 },
    { 0xb3f6f14, SUC_lexicon_tags_16 },
    { 0x669f6f15, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77f6f22, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3f6f2a, SUC_lexicon_tags_7 },
    { 0x177f6f2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x66ff6f2d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa61f6f2f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadf6f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc79f6f36, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xaaff6f38, SUC_lexicon_tags_47 },
    { 0x149f6f39, SUC_lexicon_tags_61 },
    { 0x3ebf6f3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x631f6f3d, SUC_lexicon_tags_4 },
    { 0x9fdf6f3e, SUC_lexicon_tags_29 },
    { 0x60df6f3e, SUC_lexicon_tags_17 },
    { 0xc2bf6f3d, SUC_lexicon_tags_108 },
    { 0x6eff6f41, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ff6f49, SUC_lexicon_tags_89 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5f6f4e, SUC_lexicon_tags_37 },
    { 0xcb7f6f4f, SUC_lexicon_tags_11 },
    { 0xfd9f6f50, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3f6f55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x145f6f57, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xc39f6f5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x44ff6f5c, SUC_lexicon_tags_37 },
    { 0x615f6f5d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd7f6f5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9f6f64, SUC_lexicon_tags_11 },
    { 0x893f6f64, SUC_lexicon_tags_47 },
    { 0x37ff6f64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x173f6f68, SUC_lexicon_tags_3 },
    { 0x975f6f69, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x919f6f6c, SUC_lexicon_tags_25 },
    { 0x713f6f6d, SUC_lexicon_tags_23 },
    { 0x3eff6f6e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xca1f6f70, SUC_lexicon_tags_17 },
    { 0xaff6f71, SUC_lexicon_tags_24 },
    { 0xd09f6f72, SUC_lexicon_tags_21 },
    { 0xbb7f6f73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323f6f79, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x747f6f7b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297f6f81, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7f6f84, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xc81f6f86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe83f6f88, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacff6f8c, SUC_lexicon_tags_85 },
    { 0xadff6f8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x487f6f8f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1f6f94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ff6f99, SUC_lexicon_tags_18 },
    { 0x645f6f9a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cdf6f9c, SUC_lexicon_tags_12 },
    { 0x8fdf6f9d, SUC_lexicon_tags_18 },
    { 0x635f6f9c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bf6fa7, SUC_lexicon_tags_1 },
    { 0xfa3f6fa8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f6fae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbf6fb5, SUC_lexicon_tags_37 },
    { 0x9edf6fb6, SUC_lexicon_tags_37 },
    { 0x8ebf6fb7, SUC_lexicon_tags_7 },
    { 0xfd7f6fb6, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x14bf6fba, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbc3f6fbc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbf6fc0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df6fc4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103f6fcd, SUC_lexicon_tags_26 },
    { 0x1eff6fce, SUC_lexicon_tags_11 },
    { 0x735f6fcd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbf6fda, SUC_lexicon_tags_15 },
    { 0x5cbf6fdb, SUC_lexicon_tags_8 },
    { 0x619f6fdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1f7f6fde, SUC_lexicon_tags_39 },
    { 0xca3f6fdf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9f6fe2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55f6fe7, SUC_lexicon_tags_17 },
    { 0xfb5f6fe8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227f6fec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35f6ff0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x199f6ff3, SUC_lexicon_tags_61 },
    { 0x465f6ff4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339f6ffc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a3f6fff, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01f700b, SUC_lexicon_tags_49 },
    { 0xcfbf700c, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x19df700e, SUC_lexicon_tags_1 },
    { 0xaa5f700f, SUC_lexicon_tags_47 },
    { 0x8b3f7010, SUC_lexicon_tags_27 },
    { 0x2ebf700f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6e3f7013, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849f7019, SUC_lexicon_tags_38 },
    { 0x537f7019, SUC_lexicon_tags_38 },
    { 0x8bf7019, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x859f701d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf7020, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x319f7024, SUC_lexicon_tags_11 },
    { 0xe39f7025, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x5a3f7027, SUC_lexicon_tags_65 },
    { 0xa1f7028, SUC_lexicon_tags_8 },
    { 0xfaff7029, SUC_lexicon_tags_60 },
    { 0x239f7028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49f702e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x361f7031, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613f7035, SUC_lexicon_tags_11 },
    { 0x6fff7036, SUC_lexicon_tags_13 },
    { 0x449f7037, SUC_lexicon_tags_7 },
    { 0x6d7f7036, SUC_lexicon_tags_14 },
    { 0x41df7039, SUC_lexicon_tags_11 },
    { 0x16bf7037, SUC_lexicon_tags_33 },
    { 0x8b1f703b, SUC_lexicon_tags_16 },
    { 0xd85f703b, SUC_lexicon_tags_15 },
    { 0xc4df703d, SUC_lexicon_tags_38 },
    { 0x4a9f703b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695f7045, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x615f7047, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75f704a, SUC_lexicon_tags_49 },
    { 0xcedf704b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ff704e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5f7055, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x277f7057, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc1f7059, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3f705e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xeadf7060, SUC_lexicon_tags_14 },
    { 0xb85f7061, SUC_lexicon_tags_65 },
    { 0x55f7062, SUC_lexicon_tags_26 },
    { 0x567f7063, SUC_lexicon_tags_12 },
    { 0xa1df7064, SUC_lexicon_tags_32 },
    { 0x573f7061, SUC_lexicon_tags_27 },
    { 0x527f7064, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bf706a, SUC_lexicon_tags_264 },
    { 0x5bff706b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdf7072, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845f7079, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x321f707b, SUC_lexicon_tags_16 },
    { 0x33df707b, SUC_lexicon_tags_1 },
    { 0x223f707d, SUC_lexicon_tags_7 },
    { 0xfff707b, SUC_lexicon_tags_29 },
    { 0xcd5f707f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787f7086, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdd5f7088, SUC_lexicon_tags_66 },
    { 0x273f7089, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0xb05f708c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1f7092, SUC_lexicon_tags_63 },
    { 0x7b9f7093, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x42ff7095, SUC_lexicon_tags_25 },
    { 0xe0df7096, SUC_lexicon_tags_15 },
    { 0x10bf7097, SUC_lexicon_tags_12 },
    { 0x6d5f7097, SUC_lexicon_tags_37 },
    { 0x759f7098, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07f709e, SUC_lexicon_tags_15 },
    { 0x7dbf709f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x437f70a3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55f70a9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x925f70ab, SUC_lexicon_tags_7 },
    { 0xa49f70ac, SUC_lexicon_tags_108 },
    { 0x217f70ad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x325f70af, SUC_lexicon_tags_16 },
    { 0xceff70b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3f70b4, SUC_lexicon_tags_17 },
    { 0x371f70b5, SUC_lexicon_tags_1 },
    { 0x66ff70b5, SUC_lexicon_tags_15 },
    { 0x34bf70b6, SUC_lexicon_tags_16 },
    { 0xaebf70b7, SUC_lexicon_tags_7 },
    { 0x6bbf70b9, SUC_lexicon_tags_14 },
    { 0x7df70ba, SUC_lexicon_tags_15 },
    { 0xd65f70bb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x7ff70bd, SUC_lexicon_tags_11 },
    { 0xe5ff70be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5f70c2, SUC_lexicon_tags_1 },
    { 0x505f70c3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x5d1f70c5, SUC_lexicon_tags_12 },
    { 0x837f70c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f70ca, SUC_lexicon_tags_63 },
    { 0x707f70cb, SUC_lexicon_tags_11 },
    { 0xd5bf70cb, SUC_lexicon_tags_37 },
    { 0x183f70cd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21f70d2, SUC_lexicon_tags_7 },
    { 0x28df70d3, SUC_lexicon_tags_2 },
    { 0xa8df70d2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe9f70da, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x371f70dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41f70e0, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xd65f70e2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x145f70e4, SUC_lexicon_tags_82 },
    { 0x915f70e5, SUC_lexicon_tags_49 },
    { 0x7dbf70e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x381f70e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7f70f0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdff70f2, SUC_lexicon_tags_17 },
    { 0xedbf70f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe77f70f5, SUC_lexicon_tags_17 },
    { 0x91ff70f5, SUC_lexicon_tags_11 },
    { 0x981f70f6, SUC_lexicon_tags_43 },
    { 0x79ff70f8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bf7103, SUC_lexicon_tags_25 },
    { 0x739f7103, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3f7109, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3f710d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5f7110, SUC_lexicon_tags_17 },
    { 0xbc7f7111, SUC_lexicon_tags_12 },
    { 0xdfff7111, SUC_lexicon_tags_0 },
    { 0xef5f7113, SUC_lexicon_tags_33 },
    { 0xb33f7114, SUC_lexicon_tags_7 },
    { 0xe37f7115, SUC_lexicon_tags_15 },
    { 0x2e3f7115, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x10bf7118, SUC_lexicon_tags_12 },
    { 0x7dff7119, SUC_lexicon_tags_37 },
    { 0xf27f7119, SUC_lexicon_tags_12 },
    { 0x1dbf7118, SUC_lexicon_tags_17 },
    { 0x37ff7118, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e7f7122, SUC_lexicon_tags_12 },
    { 0xdb1f7123, SUC_lexicon_tags_26 },
    { 0x8b9f7123, SUC_lexicon_tags_12 },
    { 0x97df7125, SUC_lexicon_tags_2 },
    { 0x5df7126, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1f712b, SUC_lexicon_tags_17 },
    { 0x809f712c, SUC_lexicon_tags_27 },
    { 0xc09f712d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x717f712f, SUC_lexicon_tags_1 },
    { 0x65bf712f, SUC_lexicon_tags_18 },
    { 0x8b1f7131, SUC_lexicon_tags_25 },
    { 0x99ff7132, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa77f7134, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ff7139, SUC_lexicon_tags_26 },
    { 0x9e5f7139, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x651f713d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21df7142, SUC_lexicon_tags_12 },
    { 0x6c3f7143, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655f714a, SUC_lexicon_tags_25 },
    { 0xc29f714b, SUC_lexicon_tags_30 },
    { 0x4edf714b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x393f714f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29f7154, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9bf7156, SUC_lexicon_tags_31 },
    { 0xf0bf7156, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3f7160, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07f7166, SUC_lexicon_tags_47 },
    { 0xa2ff7166, SUC_lexicon_tags_2 },
    { 0xe1bf7168, SUC_lexicon_tags_1 },
    { 0x761f7167, SUC_lexicon_tags_17 },
    { 0xae9f716a, SUC_lexicon_tags_63 },
    { 0xf3df716b, SUC_lexicon_tags_12 },
    { 0x8f1f716c, SUC_lexicon_tags_29 },
    { 0x121f7168, SUC_lexicon_tags_29 },
    { 0x6edf716e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957f7172, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x767f7175, SUC_lexicon_tags_62 },
    { 0xd87f7175, SUC_lexicon_tags_10 },
    { 0xbb7f7176, SUC_lexicon_tags_7 },
    { 0x9d1f7176, SUC_lexicon_tags_11 },
    { 0xb3f7179, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa09f717b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xe6bf717d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x47f7180, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1df7186, SUC_lexicon_tags_55 },
    { 0xedbf7187, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dff7197, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde1f719b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x351f719d, SUC_lexicon_tags_38 },
    { 0x637f719e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9f71a2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf71a7, SUC_lexicon_tags_25 },
    { 0x7c1f71a7, SUC_lexicon_tags_25 },
    { 0xcbf71a9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31df71ab, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x56bf71ad, SUC_lexicon_tags_12 },
    { 0x3a3f71ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5f71b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95f71b6, SUC_lexicon_tags_49 },
    { 0xfcff71b6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ff71be, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7d7f71c0, SUC_lexicon_tags_66 },
    { 0x7f3f71c0, SUC_lexicon_tags_91 },
    { 0x353f71c2, SUC_lexicon_tags_18 },
    { 0x187f71c3, SUC_lexicon_tags_25 },
    { 0x3aff71c2, SUC_lexicon_tags_11 },
    { 0x251f71c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92df71cc, SUC_lexicon_tags_80 },
    { 0xb05f71cd, SUC_lexicon_tags_38 },
    { 0xbbf71ce, SUC_lexicon_tags_15 },
    { 0xe53f71ce, SUC_lexicon_tags_25 },
    { 0xa49f71cd, SUC_lexicon_tags_37 },
    { 0xe9bf71d1, SUC_lexicon_tags_14 },
    { 0xfbdf71d0, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ff71dd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xf6bf71df, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93f71e4, SUC_lexicon_tags_7 },
    { 0x633f71e5, SUC_lexicon_tags_37 },
    { 0xa2df71e6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebf71ea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311f71f0, SUC_lexicon_tags_77 },
    { 0xb5ff71f0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc05f71f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf4df71f5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5f71f9, SUC_lexicon_tags_25 },
    { 0x8c7f71fa, SUC_lexicon_tags_37 },
    { 0x767f71fa, SUC_lexicon_tags_49 },
    { 0x1abf71f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x51ff71fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe05f7200, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cff7204, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xe11f7207, SUC_lexicon_tags_48 },
    { 0xbe9f7208, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x8f9f720a, SUC_lexicon_tags_2 },
    { 0xe7df720a, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x5dbf720d, SUC_lexicon_tags_11 },
    { 0xb67f720e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x631f7210, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf7215, SUC_lexicon_tags_14 },
    { 0xa35f7216, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa51f7218, SUC_lexicon_tags_7 },
    { 0xdf3f7218, SUC_lexicon_tags_51 },
    { 0xdf1f7219, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93f721e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa89f7221, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x781f7228, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbf722d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbf7230, SUC_lexicon_tags_1 },
    { 0x3e1f7231, SUC_lexicon_tags_17 },
    { 0xe9ff7230, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb65f7236, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0xebf7238, SUC_lexicon_tags_45 },
    { 0x3a5f7239, SUC_lexicon_tags_19 },
    { 0xc83f7238, SUC_lexicon_tags_27 },
    { 0x1b1f723b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x647f723d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x439f723f, SUC_lexicon_tags_17 },
    { 0xf17f7240, SUC_lexicon_tags_25 },
    { 0xab7f7241, SUC_lexicon_tags_17 },
    { 0x2c1f7242, SUC_lexicon_tags_87 },
    { 0xa1bf7242, SUC_lexicon_tags_11 },
    { 0x867f723f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd7ff7246, SUC_lexicon_tags_1 },
    { 0xa23f7247, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf13f7249, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x857f724c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bff7252, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x407f7254, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bf725b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x491f725d, SUC_lexicon_tags_14 },
    { 0x331f725e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88df7268, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x339f726a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd87f726d, SUC_lexicon_tags_16 },
    { 0x173f726d, SUC_lexicon_tags_7 },
    { 0x9a1f726e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1e1f7271, SUC_lexicon_tags_12 },
    { 0x329f7272, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x3dbf7274, SUC_lexicon_tags_2 },
    { 0x4e3f7275, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1df727b, SUC_lexicon_tags_31 },
    { 0x86bf727b, SUC_lexicon_tags_1 },
    { 0x567f727d, SUC_lexicon_tags_34 },
    { 0xadf727e, SUC_lexicon_tags_27 },
    { 0x23ff727c, SUC_lexicon_tags_12 },
    { 0x9ff727b, SUC_lexicon_tags_91 },
    { 0xcf9f7281, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807f7286, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0xa93f7289, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x85f728b, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7f728e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x635f7290, SUC_lexicon_tags_31 },
    { 0x2a9f7290, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1df729b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x745f729e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7f72a1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc8ff72a3, SUC_lexicon_tags_29 },
    { 0xabbf72a3, SUC_lexicon_tags_12 },
    { 0x10bf72a5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x851f72a8, SUC_lexicon_tags_6 },
    { 0xbd9f72a9, SUC_lexicon_tags_1 },
    { 0xffdf72aa, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbf72ad, SUC_lexicon_tags_37 },
    { 0x459f72ae, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7f72b2, SUC_lexicon_tags_120 },
    { 0xcf7f72b3, SUC_lexicon_tags_10 },
    { 0x3eff72b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x997f72b6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1f72b9, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x94bf72bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f72c2, SUC_lexicon_tags_68 },
    { 0xa97f72c3, SUC_lexicon_tags_29 },
    { 0x8abf72c4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77df72ca, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfe9f72cc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809f72d0, SUC_lexicon_tags_7 },
    { 0x8fdf72d1, SUC_lexicon_tags_17 },
    { 0x2cff72d2, SUC_lexicon_tags_11 },
    { 0xf17f72d3, SUC_lexicon_tags_17 },
    { 0xc35f72d4, SUC_lexicon_tags_37 },
    { 0x2abf72d5, SUC_lexicon_tags_64 },
    { 0xcebf72d6, SUC_lexicon_tags_11 },
    { 0xc05f72d7, SUC_lexicon_tags_120 },
    { 0x189f72d8, SUC_lexicon_tags_27 },
    { 0xaaff72d7, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x855f72dd, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf8ff72df, SUC_lexicon_tags_6 },
    { 0xdb1f72e0, SUC_lexicon_tags_17 },
    { 0xaf3f72e1, SUC_lexicon_tags_33 },
    { 0x5c5f72e0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fff72e6, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x69ff72e8, SUC_lexicon_tags_37 },
    { 0x5dff72e8, SUC_lexicon_tags_3 },
    { 0xe3ff72ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x717f72ec, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x155f72ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf41f72f0, SUC_lexicon_tags_91 },
    { 0xa55f72f0, SUC_lexicon_tags_23 },
    { 0xcd1f72f1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x817f72f5, SUC_lexicon_tags_12 },
    { 0xeb5f72f6, SUC_lexicon_tags_29 },
    { 0x335f72f6, SUC_lexicon_tags_32 },
    { 0x907f72f5, SUC_lexicon_tags_17 },
    { 0xd35f72f9, SUC_lexicon_tags_26 },
    { 0xaa3f72fa, SUC_lexicon_tags_7 },
    { 0xe23f72f9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbf7304, SUC_lexicon_tags_63 },
    { 0x6f9f7305, SUC_lexicon_tags_45 },
    { 0x297f7306, SUC_lexicon_tags_25 },
    { 0x67df7306, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9f730b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7f7310, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429f7315, SUC_lexicon_tags_33 },
    { 0xc37f7316, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3f7319, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x103f731b, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x4e5f731d, SUC_lexicon_tags_15 },
    { 0xdc1f731e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c1f7329, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x303f732b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd77f732e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11f7332, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdf7337, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395f733b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x135f733e, SUC_lexicon_tags_63 },
    { 0xadff733f, SUC_lexicon_tags_25 },
    { 0xa41f7340, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53f7344, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x841f7347, SUC_lexicon_tags_7 },
    { 0xe63f7348, SUC_lexicon_tags_7 },
    { 0x4c1f7348, SUC_lexicon_tags_4 },
    { 0xd3bf7349, SUC_lexicon_tags_82 },
    { 0x9cff734b, SUC_lexicon_tags_25 },
    { 0x8adf734b, SUC_lexicon_tags_45 },
    { 0x645f734b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22df7355, SUC_lexicon_tags_87 },
    { 0x693f7356, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3f735e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x861f7361, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5f7364, SUC_lexicon_tags_77 },
    { 0xa59f7365, SUC_lexicon_tags_37 },
    { 0x521f7366, SUC_lexicon_tags_25 },
    { 0xd7f7365, SUC_lexicon_tags_12 },
    { 0x403f7368, SUC_lexicon_tags_151 },
    { 0xa27f7365, SUC_lexicon_tags_1 },
    { 0xc11f736a, SUC_lexicon_tags_49 },
    { 0x479f736b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdf7371, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x631f7373, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63f7376, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf43f7379, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x831f737b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597f7380, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ddf7386, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5fdf7388, SUC_lexicon_tags_24 },
    { 0xc6df7388, SUC_lexicon_tags_25 },
    { 0xbb7f738a, SUC_lexicon_tags_32 },
    { 0x795f738b, SUC_lexicon_tags_30 },
    { 0xd39f738b, SUC_lexicon_tags_12 },
    { 0x169f738b, SUC_lexicon_tags_37 },
    { 0xe8bf738a, SUC_lexicon_tags_17 },
    { 0x393f738e, SUC_lexicon_tags_12 },
    { 0xa93f7390, SUC_lexicon_tags_13 },
    { 0xb7f738b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x24df7394, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x189f7397, SUC_lexicon_tags_79 },
    { 0x92df7398, SUC_lexicon_tags_25 },
    { 0x89df7399, SUC_lexicon_tags_1 },
    { 0x8d9f739a, SUC_lexicon_tags_34 },
    { 0x98bf739b, SUC_lexicon_tags_44 },
    { 0x657f739c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf79f739e, SUC_lexicon_tags_26 },
    { 0xfb9f739f, SUC_lexicon_tags_25 },
    { 0xfb5f739f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa29f73a2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde9f73a6, SUC_lexicon_tags_30 },
    { 0x83ff73a7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36df73ac, SUC_lexicon_tags_0 },
    { 0xdd7f73ac, SUC_lexicon_tags_12 },
    { 0xbc5f73ae, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x205f73b1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x769f73b3, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbf73b7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdb7f73b9, SUC_lexicon_tags_15 },
    { 0xb45f73ba, SUC_lexicon_tags_15 },
    { 0x733f73bb, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x977f73bf, SUC_lexicon_tags_29 },
    { 0x45ff73bf, SUC_lexicon_tags_15 },
    { 0x5b7f73c1, SUC_lexicon_tags_47 },
    { 0xbd5f73c0, SUC_lexicon_tags_7 },
    { 0xc99f73c3, SUC_lexicon_tags_37 },
    { 0xfcdf73c3, SUC_lexicon_tags_65 },
    { 0x6a9f73c4, SUC_lexicon_tags_63 },
    { 0x7abf73c6, SUC_lexicon_tags_17 },
    { 0xe9df73c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43f73cb, SUC_lexicon_tags_65 },
    { 0xe1ff73cb, SUC_lexicon_tags_12 },
    { 0x3d7f73cc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdf73d0, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xeadf73d3, SUC_lexicon_tags_62 },
    { 0x82bf73d3, SUC_lexicon_tags_1 },
    { 0xfb7f73d5, SUC_lexicon_tags_9 },
    { 0xaebf73d4, SUC_lexicon_tags_61 },
    { 0x39df73d7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xec3f73da, SUC_lexicon_tags_30 },
    { 0xc7bf73db, SUC_lexicon_tags_27 },
    { 0xabdf73dc, SUC_lexicon_tags_14 },
    { 0x801f73da, SUC_lexicon_tags_14 },
    { 0x885f73de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd8bf73e0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa9bf73e2, SUC_lexicon_tags_2 },
    { 0xdc5f73e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bf73e7, SUC_lexicon_tags_11 },
    { 0xb6bf73e8, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7f73ef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbf73f2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513f73f6, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9f73fb, SUC_lexicon_tags_37 },
    { 0x4b3f73fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1fbf73fe, SUC_lexicon_tags_66 },
    { 0x8d9f73ff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x617f7402, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15f7406, SUC_lexicon_tags_79 },
    { 0xf23f7407, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1b9f7409, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681f7411, SUC_lexicon_tags_3 },
    { 0x1c9f7412, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xf01f7414, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbf7417, SUC_lexicon_tags_17 },
    { 0x249f7418, SUC_lexicon_tags_1 },
    { 0x5ff7418, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x985f741c, SUC_lexicon_tags_60 },
    { 0x6abf741d, SUC_lexicon_tags_2 },
    { 0x19bf741e, SUC_lexicon_tags_1 },
    { 0xd79f741c, SUC_lexicon_tags_27 },
    { 0x975f7420, SUC_lexicon_tags_17 },
    { 0x7ebf741f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3f7429, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1f742c, SUC_lexicon_tags_11 },
    { 0xe8bf742d, SUC_lexicon_tags_12 },
    { 0x4ff742c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x8d9f7430, SUC_lexicon_tags_87 },
    { 0xddff7431, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeabf7433, SUC_lexicon_tags_37 },
    { 0x70bf7433, SUC_lexicon_tags_2 },
    { 0x8fbf7435, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39f7445, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x619f7447, SUC_lexicon_tags_26 },
    { 0xcc3f7448, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x979f744b, SUC_lexicon_tags_15 },
    { 0x481f744c, SUC_lexicon_tags_79 },
    { 0xdf1f744c, SUC_lexicon_tags_25 },
    { 0xc91f744d, SUC_lexicon_tags_107 },
    { 0xabff744d, SUC_lexicon_tags_79 },
    { 0x969f7450, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d3f7454, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93f7456, SUC_lexicon_tags_19 },
    { 0x67f7457, SUC_lexicon_tags_61 },
    { 0x663f7458, SUC_lexicon_tags_0 },
    { 0x7fdf7459, SUC_lexicon_tags_38 },
    { 0xa85f745a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3bf745d, SUC_lexicon_tags_10 },
    { 0x523f745d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f7462, SUC_lexicon_tags_17 },
    { 0xc27f7463, SUC_lexicon_tags_65 },
    { 0x415f7463, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x489f7467, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf95f7469, SUC_lexicon_tags_60 },
    { 0xff3f746a, SUC_lexicon_tags_9 },
    { 0xe61f746b, SUC_lexicon_tags_2 },
    { 0x237f746c, SUC_lexicon_tags_10 },
    { 0x719f7469, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f7470, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x49f7473, SUC_lexicon_tags_38 },
    { 0xfb9f7474, SUC_lexicon_tags_10 },
    { 0xf3bf7473, SUC_lexicon_tags_37 },
    { 0x687f7476, SUC_lexicon_tags_17 },
    { 0xa7df7476, SUC_lexicon_tags_15 },
    { 0x6f1f7477, SUC_lexicon_tags_57 },
    { 0x6e9f7479, SUC_lexicon_tags_15 },
    { 0x52df747a, SUC_lexicon_tags_31 },
    { 0xb99f747b, SUC_lexicon_tags_9 },
    { 0xb07f747b, SUC_lexicon_tags_82 },
    { 0x6bdf747c, SUC_lexicon_tags_60 },
    { 0xd7f7477, SUC_lexicon_tags_14 },
    { 0x383f747f, SUC_lexicon_tags_25 },
    { 0x103f7480, SUC_lexicon_tags_63 },
    { 0x661f7479, SUC_lexicon_tags_29 },
    { 0x9c5f7482, SUC_lexicon_tags_14 },
    { 0x9bdf747c, SUC_lexicon_tags_11 },
    { 0x9b5f7477, SUC_lexicon_tags_44 },
    { 0xb3bf7485, SUC_lexicon_tags_168 },
    { 0xddff7479, SUC_lexicon_tags_87 },
    { 0xa31f747c, SUC_lexicon_tags_12 },
    { 0x535f7488, SUC_lexicon_tags_12 },
    { 0xcbff7489, SUC_lexicon_tags_1 },
    { 0x117f7489, SUC_lexicon_tags_0 },
    { 0x4bbf747c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcff7490, SUC_lexicon_tags_26 },
    { 0x40bf7491, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9d3f7493, SUC_lexicon_tags_38 },
    { 0x875f7494, SUC_lexicon_tags_31 },
    { 0x443f7495, SUC_lexicon_tags_17 },
    { 0x93f7496, SUC_lexicon_tags_0 },
    { 0x1ebf7495, SUC_lexicon_tags_55 },
    { 0x783f7497, SUC_lexicon_tags_2 },
    { 0xa71f7499, SUC_lexicon_tags_17 },
    { 0x673f7497, SUC_lexicon_tags_17 },
    { 0x2a3f749b, SUC_lexicon_tags_37 },
    { 0xfadf7495, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8e9f749e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdff74a3, SUC_lexicon_tags_12 },
    { 0x98ff74a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887f74a8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfbbf74aa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x43df74ac, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5f74ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfb5f74bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x285f74bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6df74c5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9f74ca, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x809f74cc, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x4cdf74ce, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1f74d2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdff74d5, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1df74da, SUC_lexicon_tags_10 },
    { 0xc33f74da, SUC_lexicon_tags_63 },
    { 0x347f74dc, SUC_lexicon_tags_17 },
    { 0x8fbf74da, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa3bf74df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbf74e3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27f74e7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ff74eb, SUC_lexicon_tags_60 },
    { 0xf03f74ec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44df74f0, SUC_lexicon_tags_85 },
    { 0x823f74f1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ff74f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x40df74f7, SUC_lexicon_tags_15 },
    { 0xb7df74f8, SUC_lexicon_tags_17 },
    { 0xd15f74f8, SUC_lexicon_tags_18 },
    { 0xf5bf74fa, SUC_lexicon_tags_3 },
    { 0x36ff74fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5f7503, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x345f7505, SUC_lexicon_tags_37 },
    { 0x471f7506, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdf750a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xb09f750c, SUC_lexicon_tags_11 },
    { 0xa7f750d, SUC_lexicon_tags_19 },
    { 0xef5f750e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc79f7510, SUC_lexicon_tags_19 },
    { 0x653f7511, SUC_lexicon_tags_28 },
    { 0xcc9f7512, SUC_lexicon_tags_19 },
    { 0x177f7513, SUC_lexicon_tags_7 },
    { 0x597f7512, SUC_lexicon_tags_19 },
    { 0x393f7515, SUC_lexicon_tags_9 },
    { 0xe7df7516, SUC_lexicon_tags_30 },
    { 0x8f7f7513, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81f7523, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x44ff7525, SUC_lexicon_tags_17 },
    { 0xacdf7526, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9f752c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f7530, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ff7535, SUC_lexicon_tags_43 },
    { 0x139f7536, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb49f753a, SUC_lexicon_tags_28 },
    { 0x173f753a, SUC_lexicon_tags_1 },
    { 0xc87f753b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ff7541, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45f7544, SUC_lexicon_tags_12 },
    { 0xfb9f7544, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5f7549, SUC_lexicon_tags_193 },
    { 0x311f754a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77df7550, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1f755d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x763f755f, SUC_lexicon_tags_61 },
    { 0xc7f755f, SUC_lexicon_tags_25 },
    { 0x491f755f, SUC_lexicon_tags_65 },
    { 0x479f7562, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ff756c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5f7573, SUC_lexicon_tags_27 },
    { 0x257f7574, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x26bf7577, SUC_lexicon_tags_17 },
    { 0xca7f7577, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33bf757a, SUC_lexicon_tags_0 },
    { 0x4e1f757b, SUC_lexicon_tags_10 },
    { 0x49df757a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997f7583, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf81f7585, SUC_lexicon_tags_38 },
    { 0xd43f7586, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x43bf7589, SUC_lexicon_tags_82 },
    { 0xa11f758a, SUC_lexicon_tags_17 },
    { 0xf57f758b, SUC_lexicon_tags_47 },
    { 0x9d3f758c, SUC_lexicon_tags_2 },
    { 0xff9f758b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x77f758f, SUC_lexicon_tags_42 },
    { 0xc41f7590, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7edf7593, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457f7598, SUC_lexicon_tags_23 },
    { 0xc83f7599, SUC_lexicon_tags_0 },
    { 0xd39f7599, SUC_lexicon_tags_11 },
    { 0x823f7598, SUC_lexicon_tags_1 },
    { 0xedff759a, SUC_lexicon_tags_49 },
    { 0xc6bf759d, SUC_lexicon_tags_11 },
    { 0xfedf759b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc1f75a0, SUC_lexicon_tags_60 },
    { 0x4cbf75a1, SUC_lexicon_tags_30 },
    { 0x295f75a2, SUC_lexicon_tags_4 },
    { 0xfebf75a3, SUC_lexicon_tags_23 },
    { 0x28df75a3, SUC_lexicon_tags_2 },
    { 0x63f75a2, SUC_lexicon_tags_69 },
    { 0xedbf75a4, SUC_lexicon_tags_11 },
    { 0x7b5f75a2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaff75af, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x233f75b1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97f75b7, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0x96df75b9, SUC_lexicon_tags_27 },
    { 0x597f75b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fdf75bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6b7f75be, SUC_lexicon_tags_7 },
    { 0x495f75bf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7f75c4, SUC_lexicon_tags_27 },
    { 0x171f75c4, SUC_lexicon_tags_39 },
    { 0x2eff75c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcff75c9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a5f75cb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129f75d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x355f75d3, SUC_lexicon_tags_25 },
    { 0xed7f75d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28df75d6, SUC_lexicon_tags_17 },
    { 0x147f75d7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd5f75d9, SUC_lexicon_tags_48 },
    { 0xd59f75d9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x915f75dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf73f75de, SUC_lexicon_tags_17 },
    { 0xae9f75de, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f75e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x541f75e5, SUC_lexicon_tags_37 },
    { 0xa11f75e6, SUC_lexicon_tags_6 },
    { 0x957f75e7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33f75ea, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81f75ed, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdc7f75ef, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5f75f2, SUC_lexicon_tags_8 },
    { 0xcd1f75f2, SUC_lexicon_tags_45 },
    { 0xb2ff75f2, SUC_lexicon_tags_11 },
    { 0xb33f75f4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcbff75f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bf75fc, SUC_lexicon_tags_37 },
    { 0x78bf75fd, SUC_lexicon_tags_38 },
    { 0x471f75fe, SUC_lexicon_tags_15 },
    { 0xebdf75ff, SUC_lexicon_tags_63 },
    { 0xa29f7600, SUC_lexicon_tags_87 },
    { 0xfb3f7600, SUC_lexicon_tags_17 },
    { 0x7df7602, SUC_lexicon_tags_45 },
    { 0x253f7603, SUC_lexicon_tags_1 },
    { 0xe7bf7602, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x53f7606, SUC_lexicon_tags_10 },
    { 0x14df7607, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xda9f7609, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21ff760d, SUC_lexicon_tags_0 },
    { 0x4e3f760d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf7611, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x169f7614, SUC_lexicon_tags_8 },
    { 0x3c1f7615, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1f761b, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x689f761e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759f7622, SUC_lexicon_tags_34 },
    { 0xc01f7623, SUC_lexicon_tags_69 },
    { 0xae1f7623, SUC_lexicon_tags_26 },
    { 0x325f7622, SUC_lexicon_tags_15 },
    { 0xce7f7624, SUC_lexicon_tags_49 },
    { 0x5aff7627, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x301f7629, SUC_lexicon_tags_26 },
    { 0x65f762a, SUC_lexicon_tags_37 },
    { 0x623f762b, SUC_lexicon_tags_37 },
    { 0x989f762c, SUC_lexicon_tags_32 },
    { 0x883f762c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7f7631, SUC_lexicon_tags_57 },
    { 0x86ff7632, SUC_lexicon_tags_25 },
    { 0x7d1f7633, SUC_lexicon_tags_27 },
    { 0xce7f7631, SUC_lexicon_tags_7 },
    { 0x2c3f7635, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd15f763e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9f7641, SUC_lexicon_tags_48 },
    { 0xf43f7641, SUC_lexicon_tags_12 },
    { 0x569f7643, SUC_lexicon_tags_17 },
    { 0xbd9f7643, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37ff7646, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdf7649, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe07f764b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe45f764d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x339f7652, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x14bf7654, SUC_lexicon_tags_1 },
    { 0x2c3f7654, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57bf7659, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bf765e, SUC_lexicon_tags_49 },
    { 0x265f765f, SUC_lexicon_tags_113 },
    { 0xd4ff765f, SUC_lexicon_tags_38 },
    { 0x211f7661, SUC_lexicon_tags_29 },
    { 0x94df7662, SUC_lexicon_tags_66 },
    { 0xbdbf7663, SUC_lexicon_tags_48 },
    { 0xc47f7660, SUC_lexicon_tags_25 },
    { 0x8adf7665, SUC_lexicon_tags_19 },
    { 0xa0bf7663, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x31ff7669, SUC_lexicon_tags_33 },
    { 0xadf7669, SUC_lexicon_tags_68 },
    { 0x311f7669, SUC_lexicon_tags_17 },
    { 0xd3bf7669, SUC_lexicon_tags_31 },
    { 0x5cbf766b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcabf7670, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe69f7672, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdb7f7674, SUC_lexicon_tags_63 },
    { 0xc55f7675, SUC_lexicon_tags_15 },
    { 0x1f9f7676, SUC_lexicon_tags_27 },
    { 0x619f7677, SUC_lexicon_tags_45 },
    { 0x4d9f7678, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ff767b, SUC_lexicon_tags_5 },
    { 0xfb1f767c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43f7682, SUC_lexicon_tags_17 },
    { 0xb45f7683, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7f7686, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43f768f, SUC_lexicon_tags_34 },
    { 0x759f7690, SUC_lexicon_tags_38 },
    { 0x7d1f768f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xdcdf7693, SUC_lexicon_tags_37 },
    { 0x3c7f7694, SUC_lexicon_tags_17 },
    { 0x789f7695, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569f7699, SUC_lexicon_tags_11 },
    { 0xd95f7699, SUC_lexicon_tags_7 },
    { 0x281f7699, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3f7f769d, SUC_lexicon_tags_4 },
    { 0xbcdf769d, SUC_lexicon_tags_12 },
    { 0x8c9f769d, SUC_lexicon_tags_77 },
    { 0x4dbf76a0, SUC_lexicon_tags_7 },
    { 0xd57f769e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xfabf76a4, SUC_lexicon_tags_25 },
    { 0xb63f76a5, SUC_lexicon_tags_61 },
    { 0x663f76a5, SUC_lexicon_tags_17 },
    { 0x8e7f76a6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa93f76a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x259f76ab, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4c3f76ad, SUC_lexicon_tags_65 },
    { 0xd13f76ae, SUC_lexicon_tags_15 },
    { 0x3aff76ad, SUC_lexicon_tags_19 },
    { 0x941f76b0, SUC_lexicon_tags_85 },
    { 0xa9df76b1, SUC_lexicon_tags_7 },
    { 0x8c5f76b0, SUC_lexicon_tags_31 },
    { 0xda3f76b1, SUC_lexicon_tags_15 },
    { 0x9a9f76b4, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09f76b8, SUC_lexicon_tags_96 },
    { 0x359f76b9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x12df76bc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ff76bf, SUC_lexicon_tags_17 },
    { 0x19ff76bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ddf76c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5df76c5, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83f76cb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x54df76cd, SUC_lexicon_tags_7 },
    { 0xe85f76cd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xe23f76d0, SUC_lexicon_tags_7 },
    { 0xe01f76d1, SUC_lexicon_tags_51 },
    { 0xe71f76d2, SUC_lexicon_tags_26 },
    { 0xa9ff76d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x179f76d6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383f76e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8ebf76e8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137f76ed, SUC_lexicon_tags_4 },
    { 0x2e9f76ed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe83f76f0, SUC_lexicon_tags_12 },
    { 0xff3f76f1, SUC_lexicon_tags_37 },
    { 0xf89f76f2, SUC_lexicon_tags_14 },
    { 0xc1f76f3, SUC_lexicon_tags_7 },
    { 0x3e9f76f4, SUC_lexicon_tags_12 },
    { 0x481f76f4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfd5f76f7, SUC_lexicon_tags_7 },
    { 0x5b1f76f7, SUC_lexicon_tags_11 },
    { 0x8dbf76f9, SUC_lexicon_tags_7 },
    { 0xe4bf76f8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5f76fd, SUC_lexicon_tags_49 },
    { 0x783f76fe, SUC_lexicon_tags_25 },
    { 0xcadf76ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3f7703, SUC_lexicon_tags_25 },
    { 0x3bf7704, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfc5f7706, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xae1f7708, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0xe9df770a, SUC_lexicon_tags_7 },
    { 0xf29f770a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x5ff770d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5f7711, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343f7715, SUC_lexicon_tags_17 },
    { 0x33ff7716, SUC_lexicon_tags_178 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf91f771a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x3bff771c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69f7720, SUC_lexicon_tags_17 },
    { 0x8ebf7720, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f7727, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343f772d, SUC_lexicon_tags_1 },
    { 0x579f772e, SUC_lexicon_tags_25 },
    { 0x1e9f772e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5f7733, SUC_lexicon_tags_1 },
    { 0x1cbf7734, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x59ff7736, SUC_lexicon_tags_21 },
    { 0x7a9f7737, SUC_lexicon_tags_12 },
    { 0x22ff7738, SUC_lexicon_tags_61 },
    { 0x6bdf7737, SUC_lexicon_tags_38 },
    { 0x48df7739, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655f773e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4c1f7740, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bf7746, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9f774b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6b1f774d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xd11f774f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239f7753, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5fbf7755, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3eff7757, SUC_lexicon_tags_732 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25f775a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5f775d, SUC_lexicon_tags_45 },
    { 0x7c5f775e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x821f7760, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53f7763, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65df7769, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1f776c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb03f776e, SUC_lexicon_tags_1 },
    { 0x9cdf776f, SUC_lexicon_tags_82 },
    { 0x85bf776e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7f7773, SUC_lexicon_tags_47 },
    { 0x239f7773, SUC_lexicon_tags_28 },
    { 0x91bf7775, SUC_lexicon_tags_37 },
    { 0x793f7774, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349f777f, SUC_lexicon_tags_82 },
    { 0x37ff777f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3f7783, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x54ff7786, SUC_lexicon_tags_77 },
    { 0x9c9f7787, SUC_lexicon_tags_47 },
    { 0x5f5f7788, SUC_lexicon_tags_15 },
    { 0x253f7789, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96bf778f, SUC_lexicon_tags_17 },
    { 0x365f7790, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07f7796, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x915f7799, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aff779e, SUC_lexicon_tags_38 },
    { 0x333f779f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5f77a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbbdf77a7, SUC_lexicon_tags_49 },
    { 0x1cff77a7, SUC_lexicon_tags_87 },
    { 0xe9df77a8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe7df77ab, SUC_lexicon_tags_11 },
    { 0x109f77ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96df77b0, SUC_lexicon_tags_38 },
    { 0xacbf77b1, SUC_lexicon_tags_19 },
    { 0x7b9f77b2, SUC_lexicon_tags_15 },
    { 0x6cbf77b1, SUC_lexicon_tags_37 },
    { 0xc83f77b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51f77b9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9f77bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9e5f77bf, SUC_lexicon_tags_70 },
    { 0xbb7f77c0, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x95ff77c2, SUC_lexicon_tags_12 },
    { 0x421f77c3, SUC_lexicon_tags_1 },
    { 0x4a1f77c4, SUC_lexicon_tags_29 },
    { 0x1edf77c5, SUC_lexicon_tags_55 },
    { 0x57f77c5, SUC_lexicon_tags_19 },
    { 0xd1ff77c4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7f77cf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32df77d3, SUC_lexicon_tags_12 },
    { 0x613f77d3, SUC_lexicon_tags_92 },
    { 0x8abf77d5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bf77dc, SUC_lexicon_tags_11 },
    { 0xd2df77dd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37f77e3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5f77e8, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x721f77ea, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9df77ed, SUC_lexicon_tags_7 },
    { 0x1d3f77ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdf77f1, SUC_lexicon_tags_15 },
    { 0xe49f77f2, SUC_lexicon_tags_27 },
    { 0xdb5f77f3, SUC_lexicon_tags_7 },
    { 0x815f77f4, SUC_lexicon_tags_48 },
    { 0xe81f77f5, SUC_lexicon_tags_14 },
    { 0x273f77f2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdf77fa, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7f77fd, SUC_lexicon_tags_7 },
    { 0x261f77fd, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23f7802, SUC_lexicon_tags_37 },
    { 0x47df7802, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x951f7805, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d5f780c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x421f7814, SUC_lexicon_tags_25 },
    { 0x47f7815, SUC_lexicon_tags_37 },
    { 0x2bbf7816, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbbf7819, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9df781b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x953f781f, SUC_lexicon_tags_2 },
    { 0x489f7820, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xefdf7823, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x409f7825, SUC_lexicon_tags_17 },
    { 0xfa9f7826, SUC_lexicon_tags_60 },
    { 0x7b9f7825, SUC_lexicon_tags_54 },
    { 0x8e7f7826, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bf782c, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb5f7835, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xea9f7837, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d1f7839, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x78df783c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7f783f, SUC_lexicon_tags_30 },
    { 0x3ebf783f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xefff7842, SUC_lexicon_tags_49 },
    { 0xcf3f7843, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc37f7846, SUC_lexicon_tags_37 },
    { 0xb17f7847, SUC_lexicon_tags_12 },
    { 0x7df7847, SUC_lexicon_tags_106 },
    { 0x7bbf7849, SUC_lexicon_tags_25 },
    { 0x701f784a, SUC_lexicon_tags_11 },
    { 0xed3f784b, SUC_lexicon_tags_6 },
    { 0xd4ff784c, SUC_lexicon_tags_12 },
    { 0xb87f784d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73df7852, SUC_lexicon_tags_37 },
    { 0xd83f7853, SUC_lexicon_tags_60 },
    { 0xea5f7854, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x213f7856, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7f785b, SUC_lexicon_tags_17 },
    { 0x989f785b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5f7862, SUC_lexicon_tags_65 },
    { 0xda9f7862, SUC_lexicon_tags_62 },
    { 0x35df7863, SUC_lexicon_tags_27 },
    { 0xb3ff7865, SUC_lexicon_tags_26 },
    { 0x4df7866, SUC_lexicon_tags_8 },
    { 0xfbff7865, SUC_lexicon_tags_43 },
    { 0x965f7868, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x94ff786b, SUC_lexicon_tags_1 },
    { 0x861f786c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdf7870, SUC_lexicon_tags_57 },
    { 0x2bf7871, SUC_lexicon_tags_82 },
    { 0x49f7871, SUC_lexicon_tags_2 },
    { 0xb4ff7871, SUC_lexicon_tags_11 },
    { 0x12df7871, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bf7877, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x913f7879, SUC_lexicon_tags_17 },
    { 0x885f7879, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x559f787c, SUC_lexicon_tags_12 },
    { 0xcbbf787d, SUC_lexicon_tags_63 },
    { 0xe1f787e, SUC_lexicon_tags_45 },
    { 0xfb5f787d, SUC_lexicon_tags_2 },
    { 0x289f787d, SUC_lexicon_tags_54 },
    { 0xe7bf7881, SUC_lexicon_tags_47 },
    { 0x94df787e, SUC_lexicon_tags_0 },
    { 0xd4df7883, SUC_lexicon_tags_12 },
    { 0x70ff7883, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbf7887, SUC_lexicon_tags_8 },
    { 0x321f7888, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddf788b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdf788d, SUC_lexicon_tags_70 },
    { 0xf3f788e, SUC_lexicon_tags_15 },
    { 0x55ff788e, SUC_lexicon_tags_39 },
    { 0xce5f788d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97f7896, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19f789d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xafff78a0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681f78a4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x793f78a6, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bf78ac, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x299f78ae, SUC_lexicon_tags_74 },
    { 0x67f78af, SUC_lexicon_tags_30 },
    { 0x42ff78b0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x92ff78b2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751f78b9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5f78bd, SUC_lexicon_tags_160 },
    { 0xba5f78be, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x253f78c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7f78c5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x483f78c7, SUC_lexicon_tags_37 },
    { 0xe95f78c7, SUC_lexicon_tags_33 },
    { 0x5b7f78c7, SUC_lexicon_tags_1 },
    { 0x61f78c8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbf78d2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf35f78d4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19f78da, SUC_lexicon_tags_37 },
    { 0x151f78db, SUC_lexicon_tags_27 },
    { 0x759f78dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x3fdf78de, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9a9f78e0, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7f78e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x467f78e7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xdcff78e9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x6e3f78eb, SUC_lexicon_tags_14 },
    { 0x83bf78ec, SUC_lexicon_tags_42 },
    { 0x715f78ec, SUC_lexicon_tags_22 },
    { 0x533f78ee, SUC_lexicon_tags_63 },
    { 0x919f78ed, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df78f8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937f78fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf7900, SUC_lexicon_tags_37 },
    { 0x35ff7900, SUC_lexicon_tags_26 },
    { 0xd33f7900, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dff7906, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f1f790e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf7916, SUC_lexicon_tags_23 },
    { 0xcfdf7917, SUC_lexicon_tags_14 },
    { 0xee3f7918, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xa67f791a, SUC_lexicon_tags_17 },
    { 0x283f791b, SUC_lexicon_tags_31 },
    { 0xf75f791b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3f791f, SUC_lexicon_tags_1 },
    { 0x875f7920, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x149f7923, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdf7928, SUC_lexicon_tags_17 },
    { 0x8d3f7928, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x439f792b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59f792f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeedf7931, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x449f7933, SUC_lexicon_tags_7 },
    { 0x327f7934, SUC_lexicon_tags_17 },
    { 0xb5ff7934, SUC_lexicon_tags_12 },
    { 0x333f7936, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x357f7938, SUC_lexicon_tags_38 },
    { 0x205f7939, SUC_lexicon_tags_0 },
    { 0xa35f793a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5b5f793c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bf7944, SUC_lexicon_tags_51 },
    { 0xf5f7945, SUC_lexicon_tags_25 },
    { 0xb01f7946, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x747f7948, SUC_lexicon_tags_165 },
    { 0xfe9f7949, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53f7952, SUC_lexicon_tags_7 },
    { 0xd4bf7953, SUC_lexicon_tags_37 },
    { 0x973f7954, SUC_lexicon_tags_68 },
    { 0xaf7f7954, SUC_lexicon_tags_1089 },
    { 0x8a5f7956, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x389f7958, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe67f795b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a9f795d, SUC_lexicon_tags_32 },
    { 0x5dbf795d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x951f7961, SUC_lexicon_tags_17 },
    { 0xb69f7961, SUC_lexicon_tags_37 },
    { 0x437f7962, SUC_lexicon_tags_2 },
    { 0xd33f7963, SUC_lexicon_tags_17 },
    { 0x919f7965, SUC_lexicon_tags_931 },
    { 0x99f7966, SUC_lexicon_tags_35 },
    { 0xc45f7966, SUC_lexicon_tags_17 },
    { 0x24ff7968, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9f796e, SUC_lexicon_tags_47 },
    { 0xc4ff796f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9f7977, SUC_lexicon_tags_63 },
    { 0x4dbf7977, SUC_lexicon_tags_7 },
    { 0xf5df7979, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7f797d, SUC_lexicon_tags_262 },
    { 0x57bf797e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x341f7980, SUC_lexicon_tags_25 },
    { 0xd13f7980, SUC_lexicon_tags_35 },
    { 0x32bf7982, SUC_lexicon_tags_2 },
    { 0x23ff7983, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5f7986, SUC_lexicon_tags_11 },
    { 0xe1f7987, SUC_lexicon_tags_28 },
    { 0xacbf7988, SUC_lexicon_tags_63 },
    { 0xc15f7988, SUC_lexicon_tags_37 },
    { 0xcddf798a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95f798f, SUC_lexicon_tags_2 },
    { 0xb5f7990, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309f7998, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb9f799a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89f799d, SUC_lexicon_tags_3 },
    { 0xbcff799e, SUC_lexicon_tags_15 },
    { 0x673f799e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc35f79a3, SUC_lexicon_tags_15 },
    { 0x415f79a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x341f79a6, SUC_lexicon_tags_23 },
    { 0x57df79a7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x19df79a9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4f3f79ab, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf63f79ad, SUC_lexicon_tags_2 },
    { 0x6a1f79ad, SUC_lexicon_tags_11 },
    { 0x91df79ae, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55df79b4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa59f79b7, SUC_lexicon_tags_60 },
    { 0xec5f79b8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89f79c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765f79c5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21f79ca, SUC_lexicon_tags_262 },
    { 0x4d9f79ca, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6f1f79cd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3f79d1, SUC_lexicon_tags_11 },
    { 0x233f79d1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75f79dc, SUC_lexicon_tags_7 },
    { 0x33bf79dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bf79e1, SUC_lexicon_tags_12 },
    { 0x415f79e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f79e6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x311f79e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc65f79ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x733f79ee, SUC_lexicon_tags_2 },
    { 0x687f79ef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf79f2, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1df79f6, SUC_lexicon_tags_17 },
    { 0x703f79f7, SUC_lexicon_tags_43 },
    { 0x96df79f8, SUC_lexicon_tags_47 },
    { 0x78df79f7, SUC_lexicon_tags_27 },
    { 0x3bdf79f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdc5f79fc, SUC_lexicon_tags_7 },
    { 0x7fdf79fd, SUC_lexicon_tags_37 },
    { 0x4f3f79fe, SUC_lexicon_tags_1 },
    { 0x685f79fe, SUC_lexicon_tags_11 },
    { 0x681f7a00, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3f7a07, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd9bf7a09, SUC_lexicon_tags_235 },
    { 0x7b5f7a0a, SUC_lexicon_tags_37 },
    { 0xd63f7a0a, SUC_lexicon_tags_19 },
    { 0x90bf7a0b, SUC_lexicon_tags_23 },
    { 0x721f7a0c, SUC_lexicon_tags_13 },
    { 0xc9bf7a0a, SUC_lexicon_tags_12 },
    { 0x279f7a0f, SUC_lexicon_tags_5 },
    { 0xc09f7a10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45f7a16, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609f7a1a, SUC_lexicon_tags_17 },
    { 0x7cbf7a1a, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x453f7a1d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd47f7a1f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9f7a23, SUC_lexicon_tags_0 },
    { 0x873f7a24, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aff7a27, SUC_lexicon_tags_39 },
    { 0x4f7f7a28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfff7a2b, SUC_lexicon_tags_0 },
    { 0x73f7a2c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55f7a2f, SUC_lexicon_tags_48 },
    { 0xeddf7a30, SUC_lexicon_tags_39 },
    { 0x283f7a31, SUC_lexicon_tags_43 },
    { 0x84df7a32, SUC_lexicon_tags_17 },
    { 0xdf1f7a33, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457f7a37, SUC_lexicon_tags_60 },
    { 0xfb7f7a38, SUC_lexicon_tags_49 },
    { 0xf5f7a38, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd87f7a3b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x379f7a3e, SUC_lexicon_tags_7 },
    { 0xccbf7a3f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac3f7a43, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddf7a4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbe7f7a4f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22df7a57, SUC_lexicon_tags_37 },
    { 0x1e7f7a57, SUC_lexicon_tags_9 },
    { 0xbd5f7a59, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf17f7a5b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x737f7a5f, SUC_lexicon_tags_38 },
    { 0x29ff7a60, SUC_lexicon_tags_27 },
    { 0xec5f7a61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7f7a68, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x417f7a6d, SUC_lexicon_tags_23 },
    { 0x305f7a6d, SUC_lexicon_tags_141 },
    { 0, NULL },
    { 0, NULL },
    { 0x47df7a71, SUC_lexicon_tags_33 },
    { 0xc67f7a71, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x48bf7a75, SUC_lexicon_tags_65 },
    { 0xac1f7a76, SUC_lexicon_tags_79 },
    { 0xbd9f7a77, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x351f7a79, SUC_lexicon_tags_61 },
    { 0x361f7a7a, SUC_lexicon_tags_11 },
    { 0x49f7a7b, SUC_lexicon_tags_2 },
    { 0xe95f7a7b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5f7a7f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7f7a82, SUC_lexicon_tags_15 },
    { 0xf31f7a83, SUC_lexicon_tags_37 },
    { 0x8f5f7a83, SUC_lexicon_tags_30 },
    { 0xdadf7a85, SUC_lexicon_tags_2 },
    { 0x12bf7a86, SUC_lexicon_tags_15 },
    { 0x5fdf7a87, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x643f7a89, SUC_lexicon_tags_68 },
    { 0x959f7a89, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f7a8d, SUC_lexicon_tags_29 },
    { 0x391f7a8e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x449f7a90, SUC_lexicon_tags_46 },
    { 0xcedf7a90, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x661f7a93, SUC_lexicon_tags_12 },
    { 0xba5f7a94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf7a98, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4f1f7a9a, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xbe9f7a9c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebf7aa5, SUC_lexicon_tags_2 },
    { 0x48ff7aa5, SUC_lexicon_tags_19 },
    { 0xe19f7aa7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc25f7aa9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7f7aad, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x9b1f7aaf, SUC_lexicon_tags_12 },
    { 0xb65f7ab0, SUC_lexicon_tags_48 },
    { 0x229f7ab1, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xad3f7ab3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa85f7ab5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37ff7ab9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x7f9f7abb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859f7ac5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ff7aca, SUC_lexicon_tags_37 },
    { 0x937f7acb, SUC_lexicon_tags_55 },
    { 0xe3df7acc, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13f7ad2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x945f7ad5, SUC_lexicon_tags_107 },
    { 0xb3bf7ad6, SUC_lexicon_tags_37 },
    { 0x4ebf7ad7, SUC_lexicon_tags_61 },
    { 0xa6bf7ad8, SUC_lexicon_tags_25 },
    { 0x8ebf7ad9, SUC_lexicon_tags_37 },
    { 0x803f7ad9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x503f7adc, SUC_lexicon_tags_15 },
    { 0x793f7add, SUC_lexicon_tags_13 },
    { 0xbf3f7ade, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe3f7ae4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bf7aea, SUC_lexicon_tags_1 },
    { 0xce7f7aeb, SUC_lexicon_tags_37 },
    { 0xbf9f7aec, SUC_lexicon_tags_39 },
    { 0xc25f7aed, SUC_lexicon_tags_17 },
    { 0x5df7aee, SUC_lexicon_tags_56 },
    { 0x5e7f7aee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31f7af3, SUC_lexicon_tags_47 },
    { 0x6df7af4, SUC_lexicon_tags_30 },
    { 0xa6bf7af5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x48ff7af8, SUC_lexicon_tags_2 },
    { 0x2f3f7af8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf15f7afb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5abf7afe, SUC_lexicon_tags_31 },
    { 0xf35f7aff, SUC_lexicon_tags_7 },
    { 0x631f7aff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461f7b07, SUC_lexicon_tags_7 },
    { 0xd6ff7b08, SUC_lexicon_tags_47 },
    { 0x13ff7b08, SUC_lexicon_tags_21 },
    { 0xb69f7b08, SUC_lexicon_tags_11 },
    { 0x1fbf7b0b, SUC_lexicon_tags_47 },
    { 0xc63f7b08, SUC_lexicon_tags_27 },
    { 0x39df7b0d, SUC_lexicon_tags_12 },
    { 0xe8bf7b0e, SUC_lexicon_tags_7 },
    { 0xa47f7b09, SUC_lexicon_tags_65 },
    { 0x8cdf7b10, SUC_lexicon_tags_15 },
    { 0x391f7b10, SUC_lexicon_tags_3 },
    { 0xb63f7b0d, SUC_lexicon_tags_29 },
    { 0x91f7b13, SUC_lexicon_tags_25 },
    { 0x52df7b14, SUC_lexicon_tags_27 },
    { 0x6ebf7b13, SUC_lexicon_tags_62 },
    { 0xc61f7b16, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x207f7b19, SUC_lexicon_tags_1 },
    { 0x6c5f7b19, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7ebf7b1c, SUC_lexicon_tags_2 },
    { 0x6adf7b1c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfe5f7b1f, SUC_lexicon_tags_4 },
    { 0x64df7b20, SUC_lexicon_tags_17 },
    { 0x5ddf7b20, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe89f7b26, SUC_lexicon_tags_31 },
    { 0x21f7b26, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdf7b2a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbeff7b2c, SUC_lexicon_tags_25 },
    { 0x4a7f7b2d, SUC_lexicon_tags_61 },
    { 0xe5bf7b2e, SUC_lexicon_tags_11 },
    { 0xe2bf7b2f, SUC_lexicon_tags_17 },
    { 0x59df7b30, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59df7b39, SUC_lexicon_tags_25 },
    { 0x8b3f7b3a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5c5f7b3c, SUC_lexicon_tags_17 },
    { 0x2edf7b3d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x655f7b3f, SUC_lexicon_tags_12 },
    { 0x867f7b40, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb87f7b42, SUC_lexicon_tags_165 },
    { 0x4f1f7b42, SUC_lexicon_tags_8 },
    { 0x5a5f7b44, SUC_lexicon_tags_1 },
    { 0xe91f7b45, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd07f7b49, SUC_lexicon_tags_2 },
    { 0x743f7b4a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1c9f7b4c, SUC_lexicon_tags_11 },
    { 0x993f7b4d, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75f7b52, SUC_lexicon_tags_26 },
    { 0x299f7b53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e1f7b55, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807f7b59, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadf7b61, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb25f7b64, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x475f7b66, SUC_lexicon_tags_37 },
    { 0x1d3f7b67, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9f7b6b, SUC_lexicon_tags_11 },
    { 0x193f7b6c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x9ff7b6e, SUC_lexicon_tags_25 },
    { 0xafff7b6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x429f7b71, SUC_lexicon_tags_37 },
    { 0x58df7b71, SUC_lexicon_tags_30 },
    { 0x2ebf7b72, SUC_lexicon_tags_17 },
    { 0x491f7b74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x555f7b77, SUC_lexicon_tags_28 },
    { 0x14df7b77, SUC_lexicon_tags_1 },
    { 0x1b5f7b79, SUC_lexicon_tags_65 },
    { 0x37ff7b79, SUC_lexicon_tags_25 },
    { 0xf6df7b7a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xbdf7b7d, SUC_lexicon_tags_11 },
    { 0xa95f7b7e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7dbf7b80, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599f7b85, SUC_lexicon_tags_27 },
    { 0xa89f7b85, SUC_lexicon_tags_11 },
    { 0xdd1f7b87, SUC_lexicon_tags_47 },
    { 0xa4ff7b88, SUC_lexicon_tags_23 },
    { 0x13f7b89, SUC_lexicon_tags_17 },
    { 0x1a9f7b8a, SUC_lexicon_tags_11 },
    { 0xeddf7b85, SUC_lexicon_tags_0 },
    { 0x4d3f7b8c, SUC_lexicon_tags_47 },
    { 0xc1f7b8d, SUC_lexicon_tags_34 },
    { 0xb6df7b8e, SUC_lexicon_tags_11 },
    { 0xb2ff7b8e, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9f7b99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdf7b9c, SUC_lexicon_tags_7 },
    { 0x5d3f7b9d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33f7ba0, SUC_lexicon_tags_28 },
    { 0xb8bf7ba0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3f7ba4, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x479f7ba7, SUC_lexicon_tags_0 },
    { 0xd2df7ba8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdbf7baa, SUC_lexicon_tags_33 },
    { 0x6d3f7bab, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03f7baf, SUC_lexicon_tags_21 },
    { 0x1c3f7baf, SUC_lexicon_tags_55 },
    { 0x889f7bb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb3df7bb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21f7bb7, SUC_lexicon_tags_31 },
    { 0xef3f7bb8, SUC_lexicon_tags_11 },
    { 0xeb7f7bb9, SUC_lexicon_tags_25 },
    { 0xa5f7bb8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89f7bbe, SUC_lexicon_tags_87 },
    { 0xe69f7bbe, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5f7bc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ff7bce, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x25ff7bd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x383f7bd2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x475f7bd4, SUC_lexicon_tags_11 },
    { 0xb7df7bd4, SUC_lexicon_tags_2 },
    { 0x2dff7bd4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xb61f7bd9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x455f7bde, SUC_lexicon_tags_1 },
    { 0x205f7bdf, SUC_lexicon_tags_12 },
    { 0x85df7bdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5f7be5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8bdf7be7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcff7beb, SUC_lexicon_tags_37 },
    { 0xd89f7bec, SUC_lexicon_tags_68 },
    { 0x847f7bed, SUC_lexicon_tags_8 },
    { 0x299f7bee, SUC_lexicon_tags_17 },
    { 0x2f1f7bed, SUC_lexicon_tags_25 },
    { 0xf23f7bed, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xcbf7bf2, SUC_lexicon_tags_64 },
    { 0xb31f7bf3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x64df7bf5, SUC_lexicon_tags_7 },
    { 0x30ff7bf6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3f7bf9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91df7bfd, SUC_lexicon_tags_38 },
    { 0x2e7f7bfd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ff7c01, SUC_lexicon_tags_23 },
    { 0x473f7c02, SUC_lexicon_tags_37 },
    { 0x7edf7c01, SUC_lexicon_tags_13 },
    { 0xc37f7c03, SUC_lexicon_tags_23 },
    { 0xee5f7c05, SUC_lexicon_tags_4 },
    { 0xaa5f7c04, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa99f7c08, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa57f7c0b, SUC_lexicon_tags_15 },
    { 0xdf9f7c0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xcc1f7c0e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2d9f7c10, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ff7c13, SUC_lexicon_tags_1 },
    { 0x617f7c13, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0x291f7c16, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ff7c19, SUC_lexicon_tags_60 },
    { 0xfa1f7c1a, SUC_lexicon_tags_60 },
    { 0xb45f7c1a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x83df7c1d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181f7c28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45f7c2b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f7c2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x55df7c32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xc01f7c34, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x141f7c36, SUC_lexicon_tags_37 },
    { 0xd5f7c37, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127f7c3b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8ff7c47, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebf7c4c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21f7c50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f7c53, SUC_lexicon_tags_7 },
    { 0x4e9f7c54, SUC_lexicon_tags_11 },
    { 0x367f7c53, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973f7c5d, SUC_lexicon_tags_37 },
    { 0x2f1f7c5d, SUC_lexicon_tags_43 },
    { 0xcf1f7c5e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xdd9f7c61, SUC_lexicon_tags_27 },
    { 0x1fdf7c62, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8ff7c66, SUC_lexicon_tags_43 },
    { 0x2adf7c67, SUC_lexicon_tags_33 },
    { 0xc63f7c68, SUC_lexicon_tags_61 },
    { 0x2eff7c69, SUC_lexicon_tags_82 },
    { 0xf61f7c69, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x957f7c6d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43f7c71, SUC_lexicon_tags_148 },
    { 0x475f7c72, SUC_lexicon_tags_9 },
    { 0x261f7c72, SUC_lexicon_tags_37 },
    { 0xb4ff7c73, SUC_lexicon_tags_46 },
    { 0x9e5f7c75, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x1f7c77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x98df7c79, SUC_lexicon_tags_15 },
    { 0xa7f7c79, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x34df7c7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823f7c81, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa65f7c83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc33f7c89, SUC_lexicon_tags_8 },
    { 0x51df7c8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf7c8d, SUC_lexicon_tags_45 },
    { 0xa7bf7c8e, SUC_lexicon_tags_14 },
    { 0xf7ff7c8d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x9e5f7c91, SUC_lexicon_tags_17 },
    { 0xfbdf7c92, SUC_lexicon_tags_65 },
    { 0xbe5f7c91, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x91df7c95, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1f7c98, SUC_lexicon_tags_25 },
    { 0x1d3f7c99, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15f7c9c, SUC_lexicon_tags_108 },
    { 0xfd7f7c9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9f7ca0, SUC_lexicon_tags_7 },
    { 0xbdbf7ca0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x687f7ca4, SUC_lexicon_tags_25 },
    { 0x1c7f7ca4, SUC_lexicon_tags_38 },
    { 0xcadf7ca6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x719f7caf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f7cb4, SUC_lexicon_tags_46 },
    { 0x42ff7cb5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61df7cba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf47f7cbc, SUC_lexicon_tags_17 },
    { 0x543f7cbd, SUC_lexicon_tags_60 },
    { 0xd03f7cbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f7cc4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1f7cc8, SUC_lexicon_tags_43 },
    { 0x3e9f7cc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x739f7ccc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711f7cd2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bf7cd5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707f7cdc, SUC_lexicon_tags_15 },
    { 0xd5df7cdd, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7f7ce7, SUC_lexicon_tags_26 },
    { 0x3d1f7ce7, SUC_lexicon_tags_7 },
    { 0xfb1f7ce8, SUC_lexicon_tags_25 },
    { 0x4e3f7ce7, SUC_lexicon_tags_7 },
    { 0x1dbf7ceb, SUC_lexicon_tags_30 },
    { 0x929f7cec, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x28bf7cee, SUC_lexicon_tags_0 },
    { 0x35df7cef, SUC_lexicon_tags_65 },
    { 0xec1f7cef, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ff7cf5, SUC_lexicon_tags_37 },
    { 0x13df7cf5, SUC_lexicon_tags_17 },
    { 0x5fff7cf5, SUC_lexicon_tags_15 },
    { 0xeff7cf8, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7c1f7cfa, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xa65f7cfc, SUC_lexicon_tags_19 },
    { 0xd23f7cfd, SUC_lexicon_tags_15 },
    { 0xa2df7cfd, SUC_lexicon_tags_15 },
    { 0x239f7cff, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe49f7d01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4aff7d05, SUC_lexicon_tags_30 },
    { 0xd1df7d06, SUC_lexicon_tags_68 },
    { 0xac1f7d05, SUC_lexicon_tags_37 },
    { 0x54ff7d06, SUC_lexicon_tags_10 },
    { 0x175f7d07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x32bf7d0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1f7d11, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733f7d15, SUC_lexicon_tags_17 },
    { 0x3f7f7d16, SUC_lexicon_tags_37 },
    { 0x3d7f7d15, SUC_lexicon_tags_2 },
    { 0xc5f7d16, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe3df7d1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4c9f7d1c, SUC_lexicon_tags_45 },
    { 0x1c9f7d1c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcff7d20, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431f7d27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd3bf7d29, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x967f7d2b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ff7d32, SUC_lexicon_tags_11 },
    { 0x36bf7d33, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x61df7d35, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1df7d38, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfff7d3d, SUC_lexicon_tags_42 },
    { 0x925f7d3e, SUC_lexicon_tags_49 },
    { 0xab3f7d3e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x487f7d41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xccbf7d43, SUC_lexicon_tags_31 },
    { 0xf1df7d44, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69f7d4e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bf7d51, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bf7d56, SUC_lexicon_tags_23 },
    { 0xed9f7d57, SUC_lexicon_tags_11 },
    { 0xa49f7d56, SUC_lexicon_tags_63 },
    { 0x3adf7d59, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xa17f7d5b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbf7d5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe17f7d60, SUC_lexicon_tags_31 },
    { 0x255f7d60, SUC_lexicon_tags_68 },
    { 0x881f7d61, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38ff7d70, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x293f7d72, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc6df7d74, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x381f7d7a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x675f7d7c, SUC_lexicon_tags_25 },
    { 0xf79f7d7d, SUC_lexicon_tags_13 },
    { 0xf3f7d7e, SUC_lexicon_tags_15 },
    { 0x8f5f7d7f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x785f7d81, SUC_lexicon_tags_27 },
    { 0x4edf7d82, SUC_lexicon_tags_25 },
    { 0x4fff7d83, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14bf7d89, SUC_lexicon_tags_11 },
    { 0x147f7d8a, SUC_lexicon_tags_12 },
    { 0x279f7d8a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5e5f7d8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5cdf7d8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x865f7d92, SUC_lexicon_tags_18 },
    { 0x851f7d93, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf33f7d95, SUC_lexicon_tags_12 },
    { 0xcfff7d96, SUC_lexicon_tags_19 },
    { 0x483f7d97, SUC_lexicon_tags_25 },
    { 0xbb9f7d98, SUC_lexicon_tags_26 },
    { 0xb9df7d98, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42df7d9e, SUC_lexicon_tags_13 },
    { 0x2aff7d9f, SUC_lexicon_tags_14 },
    { 0x869f7d9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ff7da3, SUC_lexicon_tags_7 },
    { 0xdf9f7da4, SUC_lexicon_tags_37 },
    { 0x19bf7da3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x33ff7da7, SUC_lexicon_tags_0 },
    { 0x67f7da8, SUC_lexicon_tags_26 },
    { 0xc4df7da9, SUC_lexicon_tags_43 },
    { 0xc23f7daa, SUC_lexicon_tags_27 },
    { 0xdb7f7da8, SUC_lexicon_tags_30 },
    { 0xda3f7dac, SUC_lexicon_tags_23 },
    { 0xfc3f7dac, SUC_lexicon_tags_25 },
    { 0x77bf7dae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51f7db2, SUC_lexicon_tags_211 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9f7db7, SUC_lexicon_tags_25 },
    { 0x15df7db7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeadf7dbf, SUC_lexicon_tags_11 },
    { 0x533f7dbf, SUC_lexicon_tags_7 },
    { 0x1dbf7dc0, SUC_lexicon_tags_17 },
    { 0xb5bf7dc2, SUC_lexicon_tags_11 },
    { 0xc8bf7dc3, SUC_lexicon_tags_17 },
    { 0x119f7dc3, SUC_lexicon_tags_25 },
    { 0xcd5f7dc5, SUC_lexicon_tags_34 },
    { 0xff1f7dc6, SUC_lexicon_tags_1 },
    { 0xa09f7dc7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x9ff7dc9, SUC_lexicon_tags_45 },
    { 0xd51f7dca, SUC_lexicon_tags_0 },
    { 0xde7f7dcb, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0xe7ff7dcd, SUC_lexicon_tags_14 },
    { 0x8cff7dce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x473f7dd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcff7dd6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9f7dd9, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x4bf7ddb, SUC_lexicon_tags_37 },
    { 0x7bf7ddb, SUC_lexicon_tags_11 },
    { 0x4f3f7ddb, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1f7de3, SUC_lexicon_tags_16 },
    { 0x141f7de3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13f7dea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x5c5f7dec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf9f7dee, SUC_lexicon_tags_11 },
    { 0xc6df7def, SUC_lexicon_tags_10 },
    { 0x55bf7df0, SUC_lexicon_tags_54 },
    { 0x42df7df1, SUC_lexicon_tags_10 },
    { 0x4bdf7df0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8ff7df4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7adf7df8, SUC_lexicon_tags_31 },
    { 0x479f7df9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255f7e00, SUC_lexicon_tags_30 },
    { 0x377f7e01, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5f7e05, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x933f7e07, SUC_lexicon_tags_29 },
    { 0x913f7e08, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51ff7e12, SUC_lexicon_tags_12 },
    { 0x78ff7e13, SUC_lexicon_tags_12 },
    { 0x59df7e14, SUC_lexicon_tags_60 },
    { 0x4e7f7e13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6b9f7e17, SUC_lexicon_tags_23 },
    { 0xf1f7e18, SUC_lexicon_tags_11 },
    { 0x133f7e18, SUC_lexicon_tags_0 },
    { 0xd3f7e1a, SUC_lexicon_tags_30 },
    { 0xb1f7e17, SUC_lexicon_tags_37 },
    { 0x62df7e1c, SUC_lexicon_tags_19 },
    { 0x495f7e19, SUC_lexicon_tags_12 },
    { 0x205f7e1e, SUC_lexicon_tags_11 },
    { 0x3e7f7e1f, SUC_lexicon_tags_65 },
    { 0xf7df7e1f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadf7e2a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cff7e2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67f7e34, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x447f7e37, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545f7e3c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xea1f7e3f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a7f7e45, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa7ff7e47, SUC_lexicon_tags_11 },
    { 0x5dbf7e47, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xc91f7e4b, SUC_lexicon_tags_37 },
    { 0xad1f7e4b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe01f7e52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abf7e55, SUC_lexicon_tags_11 },
    { 0x205f7e56, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x275f7e58, SUC_lexicon_tags_37 },
    { 0x87ff7e59, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bf7e5c, SUC_lexicon_tags_37 },
    { 0x349f7e5d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2aff7e60, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xefbf7e63, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd87f7e68, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x725f7e6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3f7e72, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3f7e75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x139f7e77, SUC_lexicon_tags_74 },
    { 0xbedf7e77, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x533f7e7b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abf7e88, SUC_lexicon_tags_18 },
    { 0x8a5f7e89, SUC_lexicon_tags_19 },
    { 0x39df7e8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f7e90, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdff7e99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ff7e9c, SUC_lexicon_tags_4 },
    { 0x5f3f7e9c, SUC_lexicon_tags_60 },
    { 0xa9bf7e9d, SUC_lexicon_tags_15 },
    { 0x217f7e9e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f7ea4, SUC_lexicon_tags_45 },
    { 0x4a9f7ea5, SUC_lexicon_tags_30 },
    { 0xcadf7ea6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f7eaa, SUC_lexicon_tags_1 },
    { 0x407f7eaa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x273f7ead, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x41bf7eaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1f7eb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ff7eb8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ff7ebb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7f7ebe, SUC_lexicon_tags_17 },
    { 0x7b1f7ebf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2f7f7ec1, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x55ff7ec3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7f7ec9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1f7ecc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6d5f7ece, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9adf7ed1, SUC_lexicon_tags_70 },
    { 0x127f7ed2, SUC_lexicon_tags_15 },
    { 0xc67f7ed3, SUC_lexicon_tags_57 },
    { 0xd67f7ed3, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbabf7ed6, SUC_lexicon_tags_14 },
    { 0xf8bf7ed7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef7f7ee0, SUC_lexicon_tags_7 },
    { 0xd15f7ee1, SUC_lexicon_tags_25 },
    { 0xdf7ee2, SUC_lexicon_tags_16 },
    { 0x8e7f7ee3, SUC_lexicon_tags_68 },
    { 0x631f7ee1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x983f7ee6, SUC_lexicon_tags_25 },
    { 0x449f7ee6, SUC_lexicon_tags_37 },
    { 0x109f7ee6, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93f7eeb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7f7ef5, SUC_lexicon_tags_47 },
    { 0xb19f7ef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185f7efd, SUC_lexicon_tags_0 },
    { 0x4abf7efd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bff7f02, SUC_lexicon_tags_37 },
    { 0x1cdf7f03, SUC_lexicon_tags_61 },
    { 0x9a1f7f03, SUC_lexicon_tags_11 },
    { 0x87bf7f05, SUC_lexicon_tags_15 },
    { 0x21f7f06, SUC_lexicon_tags_29 },
    { 0x95df7f07, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bf7f0e, SUC_lexicon_tags_49 },
    { 0x523f7f0f, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf31f7f11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bf7f16, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bf7f1c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ff7f27, SUC_lexicon_tags_17 },
    { 0x70ff7f28, SUC_lexicon_tags_7 },
    { 0xf67f7f27, SUC_lexicon_tags_7 },
    { 0x599f7f2a, SUC_lexicon_tags_68 },
    { 0xd0bf7f2b, SUC_lexicon_tags_12 },
    { 0xdf1f7f2c, SUC_lexicon_tags_16 },
    { 0x603f7f2d, SUC_lexicon_tags_12 },
    { 0x477f7f2e, SUC_lexicon_tags_25 },
    { 0x17bf7f2a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7fdf7f31, SUC_lexicon_tags_49 },
    { 0x93f7f32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19f7f36, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cff7f39, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13f7f3c, SUC_lexicon_tags_11 },
    { 0x339f7f3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd19f7f42, SUC_lexicon_tags_24 },
    { 0xe35f7f42, SUC_lexicon_tags_37 },
    { 0x273f7f44, SUC_lexicon_tags_47 },
    { 0xb1df7f44, SUC_lexicon_tags_25 },
    { 0x3bbf7f45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7f7f49, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xc73f7f4c, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xd99f7f4e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd45f7f50, SUC_lexicon_tags_0 },
    { 0x195f7f51, SUC_lexicon_tags_47 },
    { 0x14ff7f52, SUC_lexicon_tags_25 },
    { 0xf3ff7f50, SUC_lexicon_tags_100 },
    { 0xdcbf7f53, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedf7f5a, SUC_lexicon_tags_64 },
    { 0xfe7f7f5b, SUC_lexicon_tags_19 },
    { 0xb1ff7f5c, SUC_lexicon_tags_30 },
    { 0xdadf7f5a, SUC_lexicon_tags_37 },
    { 0x859f7f5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec7f7f60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x935f7f63, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xd37f7f66, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbdf7f68, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x99ff7f6b, SUC_lexicon_tags_4 },
    { 0xc91f7f6b, SUC_lexicon_tags_38 },
    { 0xc95f7f6c, SUC_lexicon_tags_49 },
    { 0x23bf7f6c, SUC_lexicon_tags_0 },
    { 0x3f9f7f6f, SUC_lexicon_tags_2 },
    { 0x76bf7f70, SUC_lexicon_tags_23 },
    { 0xaf7f7f71, SUC_lexicon_tags_23 },
    { 0x78bf7f6b, SUC_lexicon_tags_16 },
    { 0x65bf7f73, SUC_lexicon_tags_19 },
    { 0x6ddf7f6e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x26bf7f76, SUC_lexicon_tags_17 },
    { 0xbbf7f76, SUC_lexicon_tags_46 },
    { 0x71bf7f78, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f5f7f7a, SUC_lexicon_tags_29 },
    { 0xd91f7f7a, SUC_lexicon_tags_49 },
    { 0x9a3f7f7c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x509f7f7e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb57f7f86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65f7f8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb6ff7f8c, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4fff7f8e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787f7f93, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x535f7f95, SUC_lexicon_tags_15 },
    { 0x4dff7f96, SUC_lexicon_tags_12 },
    { 0x1bdf7f97, SUC_lexicon_tags_13 },
    { 0x67df7f95, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x599f7f9a, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x20ff7f9c, SUC_lexicon_tags_11 },
    { 0x7c7f7f9c, SUC_lexicon_tags_47 },
    { 0x19bf7f9e, SUC_lexicon_tags_25 },
    { 0x269f7f9d, SUC_lexicon_tags_74 },
    { 0x681f7f9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5f7fa8, SUC_lexicon_tags_47 },
    { 0x213f7fa9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x651f7fab, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943f7faf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x403f7fb2, SUC_lexicon_tags_15 },
    { 0xfbf7fb2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bf7fb7, SUC_lexicon_tags_12 },
    { 0xff7f7fb8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe9f7fba, SUC_lexicon_tags_17 },
    { 0x323f7fba, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecdf7fc6, SUC_lexicon_tags_37 },
    { 0x7a3f7fc7, SUC_lexicon_tags_35 },
    { 0xb9f7fc7, SUC_lexicon_tags_11 },
    { 0xf19f7fc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x553f7fcb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x433f7fd4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695f7fdb, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567f7fe3, SUC_lexicon_tags_37 },
    { 0x887f7fe4, SUC_lexicon_tags_11 },
    { 0xbe3f7fe4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddf7fe9, SUC_lexicon_tags_11 },
    { 0xe67f7fea, SUC_lexicon_tags_15 },
    { 0x73bf7fe9, SUC_lexicon_tags_17 },
    { 0x549f7feb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5f7ff0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x12ff7ff2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fbf7ff4, SUC_lexicon_tags_70 },
    { 0x54df7ff5, SUC_lexicon_tags_7 },
    { 0xfd1f7ff4, SUC_lexicon_tags_46 },
    { 0xb79f7ff7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x907f7ffa, SUC_lexicon_tags_17 },
    { 0x35ff7ffb, SUC_lexicon_tags_2 },
    { 0xa5df7ffc, SUC_lexicon_tags_0 },
    { 0x651f7ffc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ff8000, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x9fbf8002, SUC_lexicon_tags_0 },
    { 0x10ff8003, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763f8011, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11f8016, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0df801e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x987f8020, SUC_lexicon_tags_26 },
    { 0x865f8021, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd07f8023, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9f8028, SUC_lexicon_tags_47 },
    { 0x267f8029, SUC_lexicon_tags_15 },
    { 0x49df8028, SUC_lexicon_tags_23 },
    { 0x871f802b, SUC_lexicon_tags_0 },
    { 0x61ff802c, SUC_lexicon_tags_35 },
    { 0x27df802d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x41bf802f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x211f8031, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x871f8033, SUC_lexicon_tags_15 },
    { 0x509f8034, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x60ff8036, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x857f8039, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x99bf803c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe85f803e, SUC_lexicon_tags_39 },
    { 0xf41f803f, SUC_lexicon_tags_15 },
    { 0x2f7f8040, SUC_lexicon_tags_7 },
    { 0x6d5f803f, SUC_lexicon_tags_56 },
    { 0x23df8041, SUC_lexicon_tags_2 },
    { 0x959f8043, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x40df8046, SUC_lexicon_tags_29 },
    { 0x701f8047, SUC_lexicon_tags_1 },
    { 0x159f8048, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9f8054, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x169f8059, SUC_lexicon_tags_49 },
    { 0x3aff805a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x947f805d, SUC_lexicon_tags_4 },
    { 0x9eff805d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa7f8060, SUC_lexicon_tags_37 },
    { 0x7c3f8061, SUC_lexicon_tags_36 },
    { 0x127f8062, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29f8065, SUC_lexicon_tags_0 },
    { 0x93ff8065, SUC_lexicon_tags_37 },
    { 0x7a3f8067, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcf3f8069, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7ff806f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x501f8071, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9f8075, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe97f807a, SUC_lexicon_tags_0 },
    { 0x283f807b, SUC_lexicon_tags_17 },
    { 0x5b1f807b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x5fbf807e, SUC_lexicon_tags_15 },
    { 0x17bf807f, SUC_lexicon_tags_25 },
    { 0x863f807f, SUC_lexicon_tags_63 },
    { 0xd61f8081, SUC_lexicon_tags_37 },
    { 0xe1bf8080, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x831f8085, SUC_lexicon_tags_15 },
    { 0x6b5f8086, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711f808a, SUC_lexicon_tags_1 },
    { 0x93df808b, SUC_lexicon_tags_15 },
    { 0x701f808a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x98bf808e, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbf8091, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7f8095, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa31f8098, SUC_lexicon_tags_51 },
    { 0x59df8099, SUC_lexicon_tags_11 },
    { 0x689f8099, SUC_lexicon_tags_39 },
    { 0x98df8099, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9f809e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdf80a5, SUC_lexicon_tags_65 },
    { 0xc71f80a6, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x999f80a8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xdebf80aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11f80b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x44bf80b7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39f80ba, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7b5f80bc, SUC_lexicon_tags_16 },
    { 0x493f80bc, SUC_lexicon_tags_48 },
    { 0xa43f80be, SUC_lexicon_tags_61 },
    { 0x5fdf80bf, SUC_lexicon_tags_25 },
    { 0x3cbf80c0, SUC_lexicon_tags_27 },
    { 0xb11f80c0, SUC_lexicon_tags_17 },
    { 0xfff80c2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9f80c5, SUC_lexicon_tags_11 },
    { 0x675f80c6, SUC_lexicon_tags_45 },
    { 0x249f80c7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeeff80cc, SUC_lexicon_tags_11 },
    { 0xa2bf80cd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb31f80d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf47f80d5, SUC_lexicon_tags_12 },
    { 0x3edf80d6, SUC_lexicon_tags_1 },
    { 0x797f80d7, SUC_lexicon_tags_1 },
    { 0xb89f80d6, SUC_lexicon_tags_1 },
    { 0x52bf80d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x569f80db, SUC_lexicon_tags_65 },
    { 0x8fbf80db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x94df80de, SUC_lexicon_tags_0 },
    { 0xd33f80df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd9f80e1, SUC_lexicon_tags_31 },
    { 0x99ff80e1, SUC_lexicon_tags_33 },
    { 0xaeff80e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x109f80e5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x70df80e7, SUC_lexicon_tags_1 },
    { 0x201f80e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x873f80ea, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7f80f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebf80f9, SUC_lexicon_tags_63 },
    { 0xb07f80fa, SUC_lexicon_tags_22 },
    { 0x995f80fb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x477f80fe, SUC_lexicon_tags_27 },
    { 0xfaff80ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9c7f8101, SUC_lexicon_tags_27 },
    { 0x8bf8101, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xaff8105, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bf8109, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4cdf810b, SUC_lexicon_tags_49 },
    { 0x551f810c, SUC_lexicon_tags_25 },
    { 0x973f810d, SUC_lexicon_tags_37 },
    { 0xb09f810d, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x229f8110, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5fdf8112, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71f8117, SUC_lexicon_tags_7 },
    { 0x3edf8118, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63ff8122, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb8ff8124, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89f8128, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3f8130, SUC_lexicon_tags_27 },
    { 0xaeff8131, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03f8134, SUC_lexicon_tags_0 },
    { 0x8cff8134, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1f813a, SUC_lexicon_tags_1 },
    { 0x603f813a, SUC_lexicon_tags_17 },
    { 0x99df813c, SUC_lexicon_tags_17 },
    { 0xcd1f813d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x89f8140, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adf8145, SUC_lexicon_tags_1 },
    { 0x9bff8146, SUC_lexicon_tags_31 },
    { 0xb7df8147, SUC_lexicon_tags_1 },
    { 0x89df8146, SUC_lexicon_tags_21 },
    { 0x3fdf8149, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9e9f814b, SUC_lexicon_tags_1 },
    { 0xb75f814c, SUC_lexicon_tags_31 },
    { 0xee1f814d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x109f814f, SUC_lexicon_tags_12 },
    { 0xb47f8150, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x55bf8152, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xac7f8154, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xe55f8156, SUC_lexicon_tags_37 },
    { 0xc47f8157, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe3df8159, SUC_lexicon_tags_33 },
    { 0x145f8159, SUC_lexicon_tags_27 },
    { 0x4df815a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ff8162, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285f8167, SUC_lexicon_tags_25 },
    { 0x2cff8168, SUC_lexicon_tags_8 },
    { 0x2d5f8169, SUC_lexicon_tags_1 },
    { 0xab9f816a, SUC_lexicon_tags_11 },
    { 0xf9df8169, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x86df816e, SUC_lexicon_tags_7 },
    { 0xdabf816f, SUC_lexicon_tags_49 },
    { 0x465f8170, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6aff8172, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35f8175, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7f8178, SUC_lexicon_tags_22 },
    { 0x669f8178, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bf8182, SUC_lexicon_tags_9 },
    { 0xe59f8183, SUC_lexicon_tags_11 },
    { 0x837f8184, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdf818c, SUC_lexicon_tags_11 },
    { 0x537f818d, SUC_lexicon_tags_25 },
    { 0x133f818d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7df819c, SUC_lexicon_tags_65 },
    { 0x415f819d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x253f819f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761f81a6, SUC_lexicon_tags_2 },
    { 0xed3f81a7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41f81aa, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9aff81ad, SUC_lexicon_tags_19 },
    { 0xaff81ad, SUC_lexicon_tags_17 },
    { 0x779f81ad, SUC_lexicon_tags_2 },
    { 0xd5f81b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80df81b2, SUC_lexicon_tags_37 },
    { 0x27ff81b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0df81b7, SUC_lexicon_tags_1 },
    { 0xeadf81b8, SUC_lexicon_tags_84 },
    { 0x5f1f81b9, SUC_lexicon_tags_15 },
    { 0xa8ff81b9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xecdf81bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bf81c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49f81c4, SUC_lexicon_tags_7 },
    { 0xa6ff81c5, SUC_lexicon_tags_37 },
    { 0x535f81c5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bf81ca, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dff81cf, SUC_lexicon_tags_2 },
    { 0xee3f81d0, SUC_lexicon_tags_25 },
    { 0x269f81d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf83f81d6, SUC_lexicon_tags_23 },
    { 0x959f81d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x93ff81d9, SUC_lexicon_tags_17 },
    { 0xe05f81d9, SUC_lexicon_tags_0 },
    { 0xa81f81db, SUC_lexicon_tags_15 },
    { 0x57f81db, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139f81e0, SUC_lexicon_tags_8 },
    { 0xe43f81e1, SUC_lexicon_tags_31 },
    { 0xf1df81e2, SUC_lexicon_tags_47 },
    { 0xa4bf81e3, SUC_lexicon_tags_37 },
    { 0x271f81e3, SUC_lexicon_tags_17 },
    { 0x11f81e4, SUC_lexicon_tags_87 },
    { 0x491f81e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x687f81e9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7f81f3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e9f81fa, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xa79f81fc, SUC_lexicon_tags_29 },
    { 0x9e1f81fc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73f8206, SUC_lexicon_tags_27 },
    { 0xc8bf8207, SUC_lexicon_tags_30 },
    { 0x4e5f8207, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545f820d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x535f8210, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x867f8213, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x6bff8215, SUC_lexicon_tags_11 },
    { 0x25ff8216, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bf8219, SUC_lexicon_tags_37 },
    { 0x8dff821a, SUC_lexicon_tags_1 },
    { 0x593f821a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x6eff821d, SUC_lexicon_tags_17 },
    { 0x98ff821e, SUC_lexicon_tags_30 },
    { 0xed9f821f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe75f8223, SUC_lexicon_tags_7 },
    { 0xd79f8224, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bf822d, SUC_lexicon_tags_15 },
    { 0x16bf822e, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x2a1f8230, SUC_lexicon_tags_6 },
    { 0x4cbf8230, SUC_lexicon_tags_43 },
    { 0x393f8230, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eff823a, SUC_lexicon_tags_37 },
    { 0x333f823a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x68ff823d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd6df823f, SUC_lexicon_tags_4 },
    { 0x405f8240, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1f8245, SUC_lexicon_tags_12 },
    { 0xb9ff8246, SUC_lexicon_tags_37 },
    { 0x72bf8246, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddf824a, SUC_lexicon_tags_91 },
    { 0x841f824b, SUC_lexicon_tags_65 },
    { 0x7e3f824b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdf8256, SUC_lexicon_tags_7 },
    { 0xdfff8256, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ff825a, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3f825e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x753f8261, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x905f8263, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xccbf8265, SUC_lexicon_tags_37 },
    { 0x3fdf8266, SUC_lexicon_tags_12 },
    { 0xd41f8266, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5f826a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57df8274, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x397f8277, SUC_lexicon_tags_7 },
    { 0x30bf8278, SUC_lexicon_tags_7 },
    { 0x29bf8279, SUC_lexicon_tags_33 },
    { 0xc3ff8277, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe31f827c, SUC_lexicon_tags_91 },
    { 0x519f827d, SUC_lexicon_tags_1 },
    { 0xa67f827e, SUC_lexicon_tags_12 },
    { 0xf31f827f, SUC_lexicon_tags_549 },
    { 0xd8ff827f, SUC_lexicon_tags_10 },
    { 0xdd1f8280, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709f8285, SUC_lexicon_tags_63 },
    { 0x577f8285, SUC_lexicon_tags_11 },
    { 0xd43f8287, SUC_lexicon_tags_60 },
    { 0x463f8288, SUC_lexicon_tags_11 },
    { 0x695f8286, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebf828f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5f8292, SUC_lexicon_tags_12 },
    { 0xccbf8292, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe6bf8295, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ff829d, SUC_lexicon_tags_25 },
    { 0x877f829d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x849f82a0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5f82a4, SUC_lexicon_tags_45 },
    { 0x2d5f82a5, SUC_lexicon_tags_15 },
    { 0xe47f82a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdb1f82a8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdf82ac, SUC_lexicon_tags_34 },
    { 0x16df82ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x939f82b0, SUC_lexicon_tags_9 },
    { 0xbc3f82b1, SUC_lexicon_tags_37 },
    { 0xd41f82b2, SUC_lexicon_tags_6 },
    { 0x663f82b3, SUC_lexicon_tags_17 },
    { 0x631f82b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3f82bb, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf82c0, SUC_lexicon_tags_10 },
    { 0xddf82c1, SUC_lexicon_tags_7 },
    { 0xd51f82c2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb15f82c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa09f82c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79f82c9, SUC_lexicon_tags_0 },
    { 0x16ff82c9, SUC_lexicon_tags_37 },
    { 0x943f82cb, SUC_lexicon_tags_12 },
    { 0x9e9f82cc, SUC_lexicon_tags_11 },
    { 0xc7ff82cc, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xae7f82cf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x487f82d2, SUC_lexicon_tags_9 },
    { 0x511f82d2, SUC_lexicon_tags_11 },
    { 0x6ddf82d4, SUC_lexicon_tags_28 },
    { 0xcfff82d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831f82d9, SUC_lexicon_tags_48 },
    { 0x6d7f82da, SUC_lexicon_tags_12 },
    { 0xd81f82db, SUC_lexicon_tags_37 },
    { 0xec5f82da, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9f82e1, SUC_lexicon_tags_17 },
    { 0x56df82e1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01f82e6, SUC_lexicon_tags_13 },
    { 0x5e5f82e7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9f82eb, SUC_lexicon_tags_17 },
    { 0x759f82ec, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e9f82ee, SUC_lexicon_tags_17 },
    { 0x9d1f82ef, SUC_lexicon_tags_12 },
    { 0xed3f82f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x63df82f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdff82f6, SUC_lexicon_tags_15 },
    { 0x871f82f6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x543f82fc, SUC_lexicon_tags_48 },
    { 0x613f82fd, SUC_lexicon_tags_27 },
    { 0xbe7f82fe, SUC_lexicon_tags_25 },
    { 0x23df82ff, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa5ff8301, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57df8308, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x6aff830b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565f8310, SUC_lexicon_tags_7 },
    { 0xf17f8311, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bf8316, SUC_lexicon_tags_48 },
    { 0x64bf8316, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849f831f, SUC_lexicon_tags_17 },
    { 0x4ddf8320, SUC_lexicon_tags_2 },
    { 0x441f8321, SUC_lexicon_tags_0 },
    { 0x3ddf8322, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887f8329, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3f1f832b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bf8330, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4abf8332, SUC_lexicon_tags_25 },
    { 0xfaff8332, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xde7f8335, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x987f8337, SUC_lexicon_tags_45 },
    { 0x2bdf8338, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31f833d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x673f8340, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90df8344, SUC_lexicon_tags_23 },
    { 0xcb9f8345, SUC_lexicon_tags_47 },
    { 0x973f8345, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3f834a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee7f834f, SUC_lexicon_tags_72 },
    { 0x94bf834f, SUC_lexicon_tags_25 },
    { 0x7cdf8351, SUC_lexicon_tags_10 },
    { 0x31bf834f, SUC_lexicon_tags_15 },
    { 0x689f8353, SUC_lexicon_tags_0 },
    { 0x6fff8353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f8357, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3f835a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5f8360, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0xdbbf8362, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b1f8364, SUC_lexicon_tags_17 },
    { 0xf61f8364, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xa1ff8367, SUC_lexicon_tags_17 },
    { 0x929f8368, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8ff836a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc77f836c, SUC_lexicon_tags_28 },
    { 0x27ff836c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9b7f836f, SUC_lexicon_tags_21 },
    { 0x9b3f8370, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x893f8372, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dff8377, SUC_lexicon_tags_30 },
    { 0xdbff8378, SUC_lexicon_tags_2 },
    { 0x983f8378, SUC_lexicon_tags_11 },
    { 0x827f8379, SUC_lexicon_tags_25 },
    { 0x841f8377, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb47f8380, SUC_lexicon_tags_7 },
    { 0x5f3f8381, SUC_lexicon_tags_0 },
    { 0x8edf8381, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x551f8385, SUC_lexicon_tags_60 },
    { 0x639f8385, SUC_lexicon_tags_7 },
    { 0x139f8387, SUC_lexicon_tags_49 },
    { 0xbdff8388, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bf8391, SUC_lexicon_tags_17 },
    { 0xc3bf8392, SUC_lexicon_tags_17 },
    { 0xec7f8391, SUC_lexicon_tags_68 },
    { 0xdedf8393, SUC_lexicon_tags_17 },
    { 0xcfff8395, SUC_lexicon_tags_15 },
    { 0x7e9f8396, SUC_lexicon_tags_19 },
    { 0x1abf8397, SUC_lexicon_tags_7 },
    { 0xc83f8398, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3eff839b, SUC_lexicon_tags_84 },
    { 0xcb1f839c, SUC_lexicon_tags_37 },
    { 0x1e3f839b, SUC_lexicon_tags_80 },
    { 0xc23f839c, SUC_lexicon_tags_37 },
    { 0x5e9f839f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf6bf83a1, SUC_lexicon_tags_15 },
    { 0x2e5f83a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb31f83a4, SUC_lexicon_tags_15 },
    { 0x6b5f83a4, SUC_lexicon_tags_11 },
    { 0x37df83a4, SUC_lexicon_tags_70 },
    { 0x53ff83a7, SUC_lexicon_tags_12 },
    { 0x8ddf83a7, SUC_lexicon_tags_56 },
    { 0x647f83a6, SUC_lexicon_tags_15 },
    { 0x21bf83aa, SUC_lexicon_tags_37 },
    { 0x217f83ab, SUC_lexicon_tags_12 },
    { 0x7a9f83ab, SUC_lexicon_tags_14 },
    { 0x927f83ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x259f83af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdf83b2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x423f83b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbd7f83b6, SUC_lexicon_tags_11 },
    { 0xd31f83b6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdf83ba, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x997f83bc, SUC_lexicon_tags_0 },
    { 0xcb5f83bd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77f83c2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xf5df83c4, SUC_lexicon_tags_25 },
    { 0xc8df83c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5f83ca, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcbbf83cc, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xc9df83ce, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ff83d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13f83d8, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fff83db, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb85f83dd, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1f83e3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf57f83e7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ff83ec, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xd29f83ee, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc35f83f0, SUC_lexicon_tags_1 },
    { 0xafdf83f0, SUC_lexicon_tags_1 },
    { 0x6e1f83f1, SUC_lexicon_tags_37 },
    { 0x2e7f83f3, SUC_lexicon_tags_0 },
    { 0xf3f83f4, SUC_lexicon_tags_30 },
    { 0xa45f83f5, SUC_lexicon_tags_12 },
    { 0x739f83f0, SUC_lexicon_tags_17 },
    { 0x683f83f3, SUC_lexicon_tags_19 },
    { 0x68bf83f8, SUC_lexicon_tags_12 },
    { 0xc61f83f8, SUC_lexicon_tags_11 },
    { 0xc0ff83f3, SUC_lexicon_tags_88 },
    { 0xd25f83f8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x51bf83fe, SUC_lexicon_tags_47 },
    { 0x50bf83fe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57f8407, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9edf8409, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aff840d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x451f840f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb55f8412, SUC_lexicon_tags_25 },
    { 0x211f8413, SUC_lexicon_tags_14 },
    { 0x999f8414, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c9f8416, SUC_lexicon_tags_56 },
    { 0xbc9f8417, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x633f841a, SUC_lexicon_tags_16 },
    { 0x5c5f841a, SUC_lexicon_tags_12 },
    { 0x2a3f841c, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bf8422, SUC_lexicon_tags_37 },
    { 0x4b7f8423, SUC_lexicon_tags_23 },
    { 0xaabf8424, SUC_lexicon_tags_2 },
    { 0x607f8425, SUC_lexicon_tags_48 },
    { 0x135f8424, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x995f8429, SUC_lexicon_tags_77 },
    { 0x28df8429, SUC_lexicon_tags_46 },
    { 0xf8bf842a, SUC_lexicon_tags_32 },
    { 0xe85f842c, SUC_lexicon_tags_25 },
    { 0x44df842d, SUC_lexicon_tags_30 },
    { 0x11f842e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67f8433, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07f8438, SUC_lexicon_tags_60 },
    { 0xa87f8438, SUC_lexicon_tags_7 },
    { 0x913f843a, SUC_lexicon_tags_68 },
    { 0x719f843b, SUC_lexicon_tags_23 },
    { 0x901f843a, SUC_lexicon_tags_7 },
    { 0x12ff843d, SUC_lexicon_tags_15 },
    { 0x9cff843a, SUC_lexicon_tags_6 },
    { 0x82df843c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345f8445, SUC_lexicon_tags_7 },
    { 0xadf8446, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11ff844a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x65df844d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bf8455, SUC_lexicon_tags_27 },
    { 0x383f8456, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x891f845a, SUC_lexicon_tags_37 },
    { 0xd25f845b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x165f845f, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41f8463, SUC_lexicon_tags_7 },
    { 0xc27f8464, SUC_lexicon_tags_63 },
    { 0x6e9f8464, SUC_lexicon_tags_60 },
    { 0x4adf8463, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bf846a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeebf846d, SUC_lexicon_tags_100 },
    { 0x209f846e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9f8472, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x759f8474, SUC_lexicon_tags_25 },
    { 0x24bf8475, SUC_lexicon_tags_14 },
    { 0xbb3f8475, SUC_lexicon_tags_12 },
    { 0x265f8475, SUC_lexicon_tags_17 },
    { 0x603f8478, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3df847b, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ff8485, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4fdf8487, SUC_lexicon_tags_17 },
    { 0xa2ff8488, SUC_lexicon_tags_8 },
    { 0xd69f8488, SUC_lexicon_tags_66 },
    { 0x715f848a, SUC_lexicon_tags_107 },
    { 0xaa9f848b, SUC_lexicon_tags_7 },
    { 0x785f848c, SUC_lexicon_tags_37 },
    { 0x59ff848d, SUC_lexicon_tags_87 },
    { 0x8f3f848c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8a9f8490, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f8496, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x91df8499, SUC_lexicon_tags_15 },
    { 0xaff849a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xb25f849c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f84a2, SUC_lexicon_tags_30 },
    { 0x27f84a3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75f84a6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x97bf84a8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9f84af, SUC_lexicon_tags_11 },
    { 0x4abf84af, SUC_lexicon_tags_45 },
    { 0x99df84b1, SUC_lexicon_tags_11 },
    { 0x2aff84b1, SUC_lexicon_tags_46 },
    { 0x6eff84b2, SUC_lexicon_tags_25 },
    { 0x257f84b4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeff84b7, SUC_lexicon_tags_11 },
    { 0xd85f84b8, SUC_lexicon_tags_17 },
    { 0x9b1f84b9, SUC_lexicon_tags_4 },
    { 0x3b7f84b9, SUC_lexicon_tags_30 },
    { 0x1c5f84ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5f84bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2edf84c1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9f84c7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2df84cc, SUC_lexicon_tags_25 },
    { 0xee9f84cd, SUC_lexicon_tags_7 },
    { 0x537f84ce, SUC_lexicon_tags_15 },
    { 0xa9f84cd, SUC_lexicon_tags_16 },
    { 0xdabf84ce, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0x74bf84d2, SUC_lexicon_tags_5 },
    { 0x649f84d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xda9f84d5, SUC_lexicon_tags_17 },
    { 0xd49f84d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff84d9, SUC_lexicon_tags_17 },
    { 0x185f84d9, SUC_lexicon_tags_11 },
    { 0x1dff84d9, SUC_lexicon_tags_79 },
    { 0x349f84dc, SUC_lexicon_tags_47 },
    { 0xc37f84dd, SUC_lexicon_tags_11 },
    { 0xb1df84de, SUC_lexicon_tags_7 },
    { 0xbbf84d9, SUC_lexicon_tags_45 },
    { 0x93df84e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xda3f84e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf31f84e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x801f84e8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x799f84eb, SUC_lexicon_tags_1 },
    { 0x371f84ec, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xecff84ee, SUC_lexicon_tags_19 },
    { 0xbbbf84ef, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0x4d3f84f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3f84f8, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18df84fe, SUC_lexicon_tags_84 },
    { 0xbcbf84ff, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bf8504, SUC_lexicon_tags_17 },
    { 0x7a1f8505, SUC_lexicon_tags_11 },
    { 0x911f8506, SUC_lexicon_tags_31 },
    { 0xd0ff8506, SUC_lexicon_tags_40 },
    { 0x929f8504, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8df850b, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xc7ff850d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedf8511, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7f8514, SUC_lexicon_tags_25 },
    { 0xa5ff8514, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x469f8518, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x42df851a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87f851e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdf8525, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd6bf8527, SUC_lexicon_tags_19 },
    { 0xd31f8528, SUC_lexicon_tags_9 },
    { 0x551f8529, SUC_lexicon_tags_15 },
    { 0x83bf8527, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf0bf852c, SUC_lexicon_tags_47 },
    { 0x647f852c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xfedf852f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dff8535, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x955f853c, SUC_lexicon_tags_39 },
    { 0xe9f853d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff8542, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7f8545, SUC_lexicon_tags_38 },
    { 0x631f8545, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x77ff8548, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe87f854a, SUC_lexicon_tags_55 },
    { 0xaadf854b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f854e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95f8552, SUC_lexicon_tags_2 },
    { 0xd9f8553, SUC_lexicon_tags_25 },
    { 0xf83f8553, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xceff8558, SUC_lexicon_tags_49 },
    { 0x56df8559, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xab9f855c, SUC_lexicon_tags_43 },
    { 0x893f855d, SUC_lexicon_tags_40 },
    { 0x51df855e, SUC_lexicon_tags_22 },
    { 0xbadf855e, SUC_lexicon_tags_2 },
    { 0x2ff855e, SUC_lexicon_tags_30 },
    { 0x6fdf8561, SUC_lexicon_tags_97 },
    { 0x989f8562, SUC_lexicon_tags_27 },
    { 0x843f8563, SUC_lexicon_tags_64 },
    { 0x1c9f8564, SUC_lexicon_tags_2 },
    { 0xc6ff8565, SUC_lexicon_tags_37 },
    { 0x953f8565, SUC_lexicon_tags_17 },
    { 0x9edf8561, SUC_lexicon_tags_1 },
    { 0x27ff8568, SUC_lexicon_tags_14 },
    { 0xb59f8569, SUC_lexicon_tags_17 },
    { 0x577f8565, SUC_lexicon_tags_46 },
    { 0x34df856b, SUC_lexicon_tags_25 },
    { 0xc3df8566, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89f8570, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x48bf8572, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xc53f8574, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x62df8577, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x175f857a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d1f857d, SUC_lexicon_tags_12 },
    { 0x505f857d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ff8583, SUC_lexicon_tags_47 },
    { 0x5e3f8584, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7f8589, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x9c7f858b, SUC_lexicon_tags_126 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa53f858f, SUC_lexicon_tags_17 },
    { 0x72ff8590, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25f8598, SUC_lexicon_tags_23 },
    { 0x8c3f8599, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadf859f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa9df85a1, SUC_lexicon_tags_54 },
    { 0x993f85a2, SUC_lexicon_tags_23 },
    { 0xc57f85a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6f7f85a5, SUC_lexicon_tags_46 },
    { 0xc57f85a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x923f85a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5f85af, SUC_lexicon_tags_10 },
    { 0x903f85b0, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe23f85b2, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xb17f85b4, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9f85b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dff85c0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9f85c9, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ddf85cb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3f85cf, SUC_lexicon_tags_17 },
    { 0x787f85d0, SUC_lexicon_tags_11 },
    { 0xbe3f85d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf69f85d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf9f85d6, SUC_lexicon_tags_29 },
    { 0xeaff85d7, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bf85db, SUC_lexicon_tags_37 },
    { 0xfc5f85dc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17f85e0, SUC_lexicon_tags_8 },
    { 0x85ff85e1, SUC_lexicon_tags_35 },
    { 0xe0bf85e1, SUC_lexicon_tags_39 },
    { 0xfb3f85e3, SUC_lexicon_tags_15 },
    { 0xd69f85e3, SUC_lexicon_tags_14 },
    { 0x633f85e5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1f85e9, SUC_lexicon_tags_47 },
    { 0x90bf85ea, SUC_lexicon_tags_9 },
    { 0x5abf85eb, SUC_lexicon_tags_8 },
    { 0x90df85ea, SUC_lexicon_tags_15 },
    { 0x285f85ea, SUC_lexicon_tags_11 },
    { 0xe61f85ee, SUC_lexicon_tags_1 },
    { 0x433f85ef, SUC_lexicon_tags_37 },
    { 0x2a5f85e9, SUC_lexicon_tags_11 },
    { 0x7bff85f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26ff85f6, SUC_lexicon_tags_12 },
    { 0xa6ff85f7, SUC_lexicon_tags_27 },
    { 0x75df85f6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x717f85fa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83f85fd, SUC_lexicon_tags_23 },
    { 0x8aff85fe, SUC_lexicon_tags_17 },
    { 0xeff85ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x83bf8601, SUC_lexicon_tags_27 },
    { 0xc97f8602, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bf8605, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303f8609, SUC_lexicon_tags_2 },
    { 0x6d3f860a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd3ff860c, SUC_lexicon_tags_66 },
    { 0xcb3f860d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859f8614, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459f8619, SUC_lexicon_tags_21 },
    { 0x7fdf861a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97ff8622, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe41f8624, SUC_lexicon_tags_25 },
    { 0xe5bf8625, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x557f8627, SUC_lexicon_tags_13 },
    { 0xb8bf8628, SUC_lexicon_tags_37 },
    { 0x3f1f8629, SUC_lexicon_tags_45 },
    { 0x6e7f862a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa77f8631, SUC_lexicon_tags_38 },
    { 0xe59f8631, SUC_lexicon_tags_12 },
    { 0x957f8633, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4ddf8635, SUC_lexicon_tags_27 },
    { 0xa0bf8636, SUC_lexicon_tags_49 },
    { 0xeb5f8636, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcff8639, SUC_lexicon_tags_2 },
    { 0xd09f8639, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x89bf863c, SUC_lexicon_tags_35 },
    { 0xd97f863d, SUC_lexicon_tags_66 },
    { 0x353f863d, SUC_lexicon_tags_63 },
    { 0x7f7f863f, SUC_lexicon_tags_17 },
    { 0x66ff8640, SUC_lexicon_tags_15 },
    { 0xcf7f8640, SUC_lexicon_tags_11 },
    { 0xba7f8642, SUC_lexicon_tags_15 },
    { 0xe4df863f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x411f8645, SUC_lexicon_tags_63 },
    { 0xec7f8646, SUC_lexicon_tags_12 },
    { 0x801f8647, SUC_lexicon_tags_33 },
    { 0x4edf8648, SUC_lexicon_tags_7 },
    { 0x95bf8649, SUC_lexicon_tags_1 },
    { 0x8a1f8649, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bff864d, SUC_lexicon_tags_17 },
    { 0x73bf864e, SUC_lexicon_tags_11 },
    { 0x1e1f864f, SUC_lexicon_tags_2 },
    { 0x4abf8650, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d5f8652, SUC_lexicon_tags_37 },
    { 0x2adf8653, SUC_lexicon_tags_17 },
    { 0x5d9f8654, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159f8659, SUC_lexicon_tags_17 },
    { 0xf17f865a, SUC_lexicon_tags_56 },
    { 0x8dbf865a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa59f865d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x979f8660, SUC_lexicon_tags_8 },
    { 0xf51f8660, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73ff8665, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859f866b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599f8671, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x277f8675, SUC_lexicon_tags_43 },
    { 0xc0df8676, SUC_lexicon_tags_31 },
    { 0xbfdf8676, SUC_lexicon_tags_84 },
    { 0x375f8677, SUC_lexicon_tags_29 },
    { 0x30df8679, SUC_lexicon_tags_49 },
    { 0x209f8678, SUC_lexicon_tags_18 },
    { 0xc1ff867a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8e3f867d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x697f867f, SUC_lexicon_tags_11 },
    { 0xa1ff8680, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39f8685, SUC_lexicon_tags_1 },
    { 0xb41f8685, SUC_lexicon_tags_23 },
    { 0x9cdf8687, SUC_lexicon_tags_37 },
    { 0xffdf8685, SUC_lexicon_tags_12 },
    { 0x5f9f8689, SUC_lexicon_tags_65 },
    { 0x7e7f8689, SUC_lexicon_tags_21 },
    { 0xa5df8689, SUC_lexicon_tags_25 },
    { 0x929f8689, SUC_lexicon_tags_19 },
    { 0x1c1f868d, SUC_lexicon_tags_5 },
    { 0x8b3f868e, SUC_lexicon_tags_23 },
    { 0x101f868c, SUC_lexicon_tags_27 },
    { 0x675f868d, SUC_lexicon_tags_30 },
    { 0xc21f8685, SUC_lexicon_tags_207 },
    { 0x24bf8692, SUC_lexicon_tags_85 },
    { 0x83f8693, SUC_lexicon_tags_27 },
    { 0xa2bf8692, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7f8699, SUC_lexicon_tags_25 },
    { 0x7a9f869a, SUC_lexicon_tags_223 },
    { 0xa1ff8699, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0x317f869e, SUC_lexicon_tags_17 },
    { 0x425f869f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7f86a1, SUC_lexicon_tags_15 },
    { 0x99f86a1, SUC_lexicon_tags_17 },
    { 0xeadf86a3, SUC_lexicon_tags_37 },
    { 0x4bdf86a4, SUC_lexicon_tags_25 },
    { 0x45df86a2, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf97f86a9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebf86ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe17f86ae, SUC_lexicon_tags_33 },
    { 0x6f3f86af, SUC_lexicon_tags_25 },
    { 0x529f86b0, SUC_lexicon_tags_2 },
    { 0xdb5f86af, SUC_lexicon_tags_17 },
    { 0xf89f86af, SUC_lexicon_tags_2 },
    { 0x4eff86b3, SUC_lexicon_tags_17 },
    { 0x907f86b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67f86b6, SUC_lexicon_tags_63 },
    { 0x109f86b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x539f86ba, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83f86be, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931f86c2, SUC_lexicon_tags_37 },
    { 0x5abf86c3, SUC_lexicon_tags_26 },
    { 0x311f86c3, SUC_lexicon_tags_15 },
    { 0xc7ff86c5, SUC_lexicon_tags_30 },
    { 0x1b3f86c6, SUC_lexicon_tags_2 },
    { 0xa99f86c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9aff86cb, SUC_lexicon_tags_1 },
    { 0x659f86cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xae5f86ce, SUC_lexicon_tags_12 },
    { 0xdebf86cf, SUC_lexicon_tags_7 },
    { 0x561f86cf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bf86d5, SUC_lexicon_tags_17 },
    { 0xefdf86d5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x909f86da, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ff86dd, SUC_lexicon_tags_5 },
    { 0xa77f86de, SUC_lexicon_tags_17 },
    { 0x3b5f86df, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ff86e3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13f86eb, SUC_lexicon_tags_30 },
    { 0x575f86ec, SUC_lexicon_tags_13 },
    { 0xfcbf86eb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x959f86f3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x685f86f5, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x9bdf86f7, SUC_lexicon_tags_19 },
    { 0x855f86f8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91f86fc, SUC_lexicon_tags_17 },
    { 0xe5df86fc, SUC_lexicon_tags_29 },
    { 0x1f86fe, SUC_lexicon_tags_25 },
    { 0xeb1f86fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7f8705, SUC_lexicon_tags_63 },
    { 0x579f8705, SUC_lexicon_tags_14 },
    { 0x995f8707, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xedf8709, SUC_lexicon_tags_15 },
    { 0x81f870a, SUC_lexicon_tags_6 },
    { 0xd53f870a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfe9f870d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x467f870f, SUC_lexicon_tags_17 },
    { 0x52df8710, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x28ff8712, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x18ff8715, SUC_lexicon_tags_63 },
    { 0xcdbf8716, SUC_lexicon_tags_12 },
    { 0x86bf8715, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3f871a, SUC_lexicon_tags_15 },
    { 0x30bf871b, SUC_lexicon_tags_12 },
    { 0x837f871c, SUC_lexicon_tags_35 },
    { 0xa9f871c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f8720, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3f3f8722, SUC_lexicon_tags_21 },
    { 0x2d7f8723, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x863f8725, SUC_lexicon_tags_7 },
    { 0x211f8725, SUC_lexicon_tags_16 },
    { 0x301f8727, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf23f872a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa37f872c, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2df872f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x47f8731, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x921f8733, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x70bf8735, SUC_lexicon_tags_31 },
    { 0xdb9f8736, SUC_lexicon_tags_45 },
    { 0xb2ff8735, SUC_lexicon_tags_17 },
    { 0x14bf8735, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399f873c, SUC_lexicon_tags_30 },
    { 0x233f873c, SUC_lexicon_tags_17 },
    { 0x8fdf873e, SUC_lexicon_tags_11 },
    { 0x487f873d, SUC_lexicon_tags_37 },
    { 0x37bf873e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bf8744, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x833f8748, SUC_lexicon_tags_2 },
    { 0x839f8749, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3f8756, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa55f875b, SUC_lexicon_tags_43 },
    { 0x931f875c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f875f, SUC_lexicon_tags_44 },
    { 0xe03f875f, SUC_lexicon_tags_112 },
    { 0xf9ff8761, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6ff8764, SUC_lexicon_tags_4 },
    { 0xedff8765, SUC_lexicon_tags_9 },
    { 0x799f8765, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xadff8769, SUC_lexicon_tags_49 },
    { 0xd77f876a, SUC_lexicon_tags_66 },
    { 0xd97f876a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x93f876e, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05f877c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9f8780, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07f8784, SUC_lexicon_tags_1 },
    { 0x5ebf8785, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741f8789, SUC_lexicon_tags_14 },
    { 0x775f8789, SUC_lexicon_tags_45 },
    { 0x189f878b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1b5f878d, SUC_lexicon_tags_15 },
    { 0xaf9f878d, SUC_lexicon_tags_25 },
    { 0x6ff878f, SUC_lexicon_tags_47 },
    { 0xedff8790, SUC_lexicon_tags_47 },
    { 0xd11f8791, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823f8796, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x47ff8798, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdf879c, SUC_lexicon_tags_11 },
    { 0xfd7f879d, SUC_lexicon_tags_57 },
    { 0x76ff879c, SUC_lexicon_tags_55 },
    { 0xe57f879f, SUC_lexicon_tags_13 },
    { 0xb21f879e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddf87a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bf87ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x81ff87af, SUC_lexicon_tags_2 },
    { 0x59bf87b0, SUC_lexicon_tags_48 },
    { 0xc83f87b0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99df87bc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc3ff87be, SUC_lexicon_tags_7 },
    { 0xde7f87bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x95bf87c1, SUC_lexicon_tags_15 },
    { 0x17df87c2, SUC_lexicon_tags_31 },
    { 0xcbf87c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8b1f87c5, SUC_lexicon_tags_46 },
    { 0x46bf87c5, SUC_lexicon_tags_5 },
    { 0xb63f87c7, SUC_lexicon_tags_88 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269f87cc, SUC_lexicon_tags_43 },
    { 0x559f87cc, SUC_lexicon_tags_2 },
    { 0x949f87cc, SUC_lexicon_tags_48 },
    { 0xba1f87ce, SUC_lexicon_tags_14 },
    { 0xe83f87d0, SUC_lexicon_tags_2 },
    { 0x6d7f87d1, SUC_lexicon_tags_37 },
    { 0x1c3f87d1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x311f87d4, SUC_lexicon_tags_15 },
    { 0x995f87d5, SUC_lexicon_tags_30 },
    { 0x5b9f87d6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x50bf87d8, SUC_lexicon_tags_61 },
    { 0x137f87d9, SUC_lexicon_tags_11 },
    { 0xa67f87d8, SUC_lexicon_tags_23 },
    { 0x52df87db, SUC_lexicon_tags_92 },
    { 0xa43f87db, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xbb5f87de, SUC_lexicon_tags_1 },
    { 0x20ff87df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xec3f87e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7f87e4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605f87e9, SUC_lexicon_tags_63 },
    { 0x443f87ea, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f87ee, SUC_lexicon_tags_14 },
    { 0xa5ff87ef, SUC_lexicon_tags_15 },
    { 0xe5f87ef, SUC_lexicon_tags_45 },
    { 0x80df87f1, SUC_lexicon_tags_60 },
    { 0xc77f87f0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71f87f6, SUC_lexicon_tags_12 },
    { 0xbf87f6, SUC_lexicon_tags_26 },
    { 0xaa1f87f7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef1f87fc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2ff8801, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x863f8803, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579f8809, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xee9f880b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x6e3f880d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5f8810, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3f8814, SUC_lexicon_tags_83 },
    { 0xea5f8814, SUC_lexicon_tags_7 },
    { 0xb27f8816, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbf881a, SUC_lexicon_tags_65 },
    { 0x167f881b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc4df881d, SUC_lexicon_tags_10 },
    { 0xafff881e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x485f8826, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9f882f, SUC_lexicon_tags_56 },
    { 0xf65f8830, SUC_lexicon_tags_14 },
    { 0x5c3f8831, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ff8837, SUC_lexicon_tags_8 },
    { 0xafff8837, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb7ff883a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7f883d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77f8844, SUC_lexicon_tags_17 },
    { 0xfa9f8845, SUC_lexicon_tags_37 },
    { 0xdff8846, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbf884a, SUC_lexicon_tags_17 },
    { 0xd39f884a, SUC_lexicon_tags_0 },
    { 0x457f884c, SUC_lexicon_tags_21 },
    { 0x5b9f884d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x7b9f884f, SUC_lexicon_tags_64 },
    { 0xeb3f8850, SUC_lexicon_tags_57 },
    { 0xbbdf8851, SUC_lexicon_tags_3 },
    { 0x733f8852, SUC_lexicon_tags_33 },
    { 0x34bf8851, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f8858, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1c5f885a, SUC_lexicon_tags_7 },
    { 0xc1f885b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255f885f, SUC_lexicon_tags_26 },
    { 0x137f885f, SUC_lexicon_tags_25 },
    { 0xf9f885f, SUC_lexicon_tags_21 },
    { 0x165f8862, SUC_lexicon_tags_15 },
    { 0x957f8860, SUC_lexicon_tags_2 },
    { 0x9c7f8864, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81f8867, SUC_lexicon_tags_43 },
    { 0x325f8867, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x991f886a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb5f886c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1f886f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe3df8871, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe3f8873, SUC_lexicon_tags_2 },
    { 0xdbff8873, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101f8878, SUC_lexicon_tags_47 },
    { 0xd8df8879, SUC_lexicon_tags_26 },
    { 0x44ff887a, SUC_lexicon_tags_17 },
    { 0x849f887b, SUC_lexicon_tags_37 },
    { 0x781f887b, SUC_lexicon_tags_11 },
    { 0x2dbf887d, SUC_lexicon_tags_8 },
    { 0x70ff887e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe41f8880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ff8885, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb1df8887, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x10ff888a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x66df888c, SUC_lexicon_tags_80 },
    { 0xc57f888d, SUC_lexicon_tags_15 },
    { 0x21ff888e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe73f8890, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x85df8892, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26df8897, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59f889d, SUC_lexicon_tags_43 },
    { 0xd59f889e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfc1f88a0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x177f88a3, SUC_lexicon_tags_12 },
    { 0xebbf88a3, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a5f88a8, SUC_lexicon_tags_17 },
    { 0x7fbf88a9, SUC_lexicon_tags_17 },
    { 0x4bff88aa, SUC_lexicon_tags_11 },
    { 0x803f88aa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61f88b0, SUC_lexicon_tags_2 },
    { 0xf55f88b1, SUC_lexicon_tags_17 },
    { 0x3cdf88b2, SUC_lexicon_tags_1 },
    { 0x287f88b3, SUC_lexicon_tags_49 },
    { 0xc4df88b4, SUC_lexicon_tags_17 },
    { 0x237f88b5, SUC_lexicon_tags_11 },
    { 0x455f88b5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdf88b9, SUC_lexicon_tags_15 },
    { 0x105f88ba, SUC_lexicon_tags_56 },
    { 0x27ff88bb, SUC_lexicon_tags_43 },
    { 0xf3df88bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbebf88be, SUC_lexicon_tags_12 },
    { 0x83df88bf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa91f88c1, SUC_lexicon_tags_40 },
    { 0xa35f88c2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fff88c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb7f88c8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x809f88ca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bf88d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0df88d9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0df88df, SUC_lexicon_tags_49 },
    { 0xbb7f88e0, SUC_lexicon_tags_10 },
    { 0x79f88e1, SUC_lexicon_tags_55 },
    { 0x679f88e2, SUC_lexicon_tags_12 },
    { 0x2ebf88e3, SUC_lexicon_tags_1 },
    { 0xd5bf88e0, SUC_lexicon_tags_7 },
    { 0x545f88df, SUC_lexicon_tags_10 },
    { 0xcf5f88e2, SUC_lexicon_tags_37 },
    { 0xecdf88e6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xec3f88e9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1f88ed, SUC_lexicon_tags_37 },
    { 0x14bf88ee, SUC_lexicon_tags_2 },
    { 0x127f88ef, SUC_lexicon_tags_28 },
    { 0xa13f88f0, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe73f88f2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadf88f6, SUC_lexicon_tags_47 },
    { 0x19f88f7, SUC_lexicon_tags_17 },
    { 0xe7bf88f8, SUC_lexicon_tags_2 },
    { 0xebff88f9, SUC_lexicon_tags_79 },
    { 0x70ff88f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3df88fd, SUC_lexicon_tags_48 },
    { 0x6eff88fd, SUC_lexicon_tags_1 },
    { 0x82bf88ff, SUC_lexicon_tags_0 },
    { 0x76ff88ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x263f8902, SUC_lexicon_tags_12 },
    { 0x35bf8903, SUC_lexicon_tags_11 },
    { 0xcbf8904, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5f8909, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcaff890b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b9f8913, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9f8916, SUC_lexicon_tags_31 },
    { 0xaddf8917, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x143f891d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ff8921, SUC_lexicon_tags_1 },
    { 0x7f8922, SUC_lexicon_tags_55 },
    { 0xfb1f8923, SUC_lexicon_tags_106 },
    { 0xd85f8923, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5f8927, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd3bf8929, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadf8931, SUC_lexicon_tags_8 },
    { 0xad9f8932, SUC_lexicon_tags_31 },
    { 0xe39f8932, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8c9f8935, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8d9f8937, SUC_lexicon_tags_11 },
    { 0xa5bf8938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x115f893c, SUC_lexicon_tags_28 },
    { 0x84bf893d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351f894c, SUC_lexicon_tags_11 },
    { 0x2a1f894c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63f8950, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdf8955, SUC_lexicon_tags_26 },
    { 0xe71f8956, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509f895d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x24bf8960, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a9f8962, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x501f8964, SUC_lexicon_tags_179 },
    { 0xef9f8965, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x9fdf8967, SUC_lexicon_tags_19 },
    { 0x67ff8968, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x53bf896a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x149f896d, SUC_lexicon_tags_40 },
    { 0x98df896e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27f8972, SUC_lexicon_tags_30 },
    { 0x16df8973, SUC_lexicon_tags_25 },
    { 0x1bdf8974, SUC_lexicon_tags_23 },
    { 0x533f8973, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x755f8978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc05f897b, SUC_lexicon_tags_26 },
    { 0x839f897b, SUC_lexicon_tags_8 },
    { 0x4f3f897b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdaff8980, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe4bf8984, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf49f8987, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25f898e, SUC_lexicon_tags_7 },
    { 0xd13f898f, SUC_lexicon_tags_37 },
    { 0x71f8990, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbf8993, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ff8998, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43df899c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91f89a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdf89a5, SUC_lexicon_tags_30 },
    { 0x495f89a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3f89a9, SUC_lexicon_tags_3 },
    { 0xd59f89aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ff89b0, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181f89bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbf89c0, SUC_lexicon_tags_7 },
    { 0x10bf89c1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f89c5, SUC_lexicon_tags_68 },
    { 0x15ff89c6, SUC_lexicon_tags_47 },
    { 0xd07f89c5, SUC_lexicon_tags_25 },
    { 0x93ff89c6, SUC_lexicon_tags_298 },
    { 0, NULL },
    { 0x5d1f89ca, SUC_lexicon_tags_9 },
    { 0xfff89cb, SUC_lexicon_tags_11 },
    { 0x227f89cc, SUC_lexicon_tags_45 },
    { 0xdc9f89cc, SUC_lexicon_tags_15 },
    { 0x591f89ce, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ff89d6, SUC_lexicon_tags_1 },
    { 0xda3f89d7, SUC_lexicon_tags_37 },
    { 0xcaff89d8, SUC_lexicon_tags_872 },
    { 0, NULL },
    { 0x95f89da, SUC_lexicon_tags_19 },
    { 0xb03f89da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x633f89dd, SUC_lexicon_tags_17 },
    { 0x819f89de, SUC_lexicon_tags_7 },
    { 0xf1bf89df, SUC_lexicon_tags_2 },
    { 0xcbbf89e0, SUC_lexicon_tags_7 },
    { 0x893f89e0, SUC_lexicon_tags_1 },
    { 0x9ebf89e2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x76bf89e4, SUC_lexicon_tags_46 },
    { 0xceff89e4, SUC_lexicon_tags_27 },
    { 0xa69f89e6, SUC_lexicon_tags_14 },
    { 0x5f5f89e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9edf89eb, SUC_lexicon_tags_2 },
    { 0x211f89ec, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31f89f0, SUC_lexicon_tags_1 },
    { 0xf95f89f1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ff89f4, SUC_lexicon_tags_28 },
    { 0x427f89f5, SUC_lexicon_tags_8 },
    { 0x5dbf89f6, SUC_lexicon_tags_1 },
    { 0x3d3f89f6, SUC_lexicon_tags_1 },
    { 0xef9f89f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edf8a01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cff8a04, SUC_lexicon_tags_37 },
    { 0x2a5f8a04, SUC_lexicon_tags_11 },
    { 0x2cbf8a04, SUC_lexicon_tags_33 },
    { 0xe5df8a07, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x64df8a0a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87f8a0d, SUC_lexicon_tags_1 },
    { 0x691f8a0e, SUC_lexicon_tags_10 },
    { 0xad7f8a0f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ff8a13, SUC_lexicon_tags_12 },
    { 0xb01f8a14, SUC_lexicon_tags_7 },
    { 0xbf7f8a15, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3f8a18, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x10bf8a1a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1eff8a1e, SUC_lexicon_tags_15 },
    { 0xd89f8a1e, SUC_lexicon_tags_1 },
    { 0x579f8a20, SUC_lexicon_tags_37 },
    { 0x7abf8a1f, SUC_lexicon_tags_26 },
    { 0xedf8a22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597f8a2a, SUC_lexicon_tags_14 },
    { 0xacbf8a2b, SUC_lexicon_tags_9 },
    { 0xe47f8a2b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x371f8a30, SUC_lexicon_tags_15 },
    { 0x14ff8a30, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9adf8a33, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed5f8a38, SUC_lexicon_tags_11 },
    { 0xce1f8a38, SUC_lexicon_tags_43 },
    { 0xc3ff8a38, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3f8a44, SUC_lexicon_tags_4 },
    { 0x7df8a45, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3f8a48, SUC_lexicon_tags_19 },
    { 0x48bf8a49, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887f8a4f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x11bf8a51, SUC_lexicon_tags_12 },
    { 0xa7ff8a52, SUC_lexicon_tags_25 },
    { 0xdc1f8a53, SUC_lexicon_tags_37 },
    { 0x4f5f8a52, SUC_lexicon_tags_3 },
    { 0x553f8a55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9df8a57, SUC_lexicon_tags_8 },
    { 0x57bf8a58, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd29f8a5e, SUC_lexicon_tags_55 },
    { 0x849f8a5f, SUC_lexicon_tags_1 },
    { 0x557f8a60, SUC_lexicon_tags_14 },
    { 0xa95f8a61, SUC_lexicon_tags_14 },
    { 0xe3ff8a62, SUC_lexicon_tags_336 },
    { 0x9d5f8a63, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57ff8a68, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbdf8a6b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x65df8a6e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x3ddf8a70, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaff8a75, SUC_lexicon_tags_19 },
    { 0x2b9f8a76, SUC_lexicon_tags_79 },
    { 0x27ff8a77, SUC_lexicon_tags_63 },
    { 0xb9ff8a78, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x84df8a7a, SUC_lexicon_tags_17 },
    { 0x69ff8a7a, SUC_lexicon_tags_16 },
    { 0x3e7f8a7c, SUC_lexicon_tags_68 },
    { 0x6c9f8a7a, SUC_lexicon_tags_7 },
    { 0x12bf8a7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x451f8a80, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297f8a85, SUC_lexicon_tags_27 },
    { 0x2b7f8a85, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd95f8a88, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x2f7f8a8a, SUC_lexicon_tags_2 },
    { 0xa77f8a8a, SUC_lexicon_tags_1 },
    { 0xd0ff8a8b, SUC_lexicon_tags_43 },
    { 0xd21f8a8c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2bdf8a8f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xf77f8a92, SUC_lexicon_tags_25 },
    { 0xd9ff8a92, SUC_lexicon_tags_31 },
    { 0x24ff8a93, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x245f8a96, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbf8a99, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf71f8a9c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54df8aa2, SUC_lexicon_tags_2 },
    { 0xa35f8aa2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5f8aaf, SUC_lexicon_tags_15 },
    { 0x1f3f8aaf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7f8ab8, SUC_lexicon_tags_17 },
    { 0xcdf8ab8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa3df8abb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf01f8abd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49f8ac0, SUC_lexicon_tags_25 },
    { 0x3b7f8ac1, SUC_lexicon_tags_25 },
    { 0x859f8ac2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9f5f8ac4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe53f8ac8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f8acf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x329f8ad1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9f8ad4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33f8adb, SUC_lexicon_tags_25 },
    { 0x8f3f8adc, SUC_lexicon_tags_2 },
    { 0x7d9f8add, SUC_lexicon_tags_11 },
    { 0xf2ff8adc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bf8ae1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x83df8ae3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479f8ae7, SUC_lexicon_tags_39 },
    { 0x833f8ae8, SUC_lexicon_tags_2 },
    { 0xe3df8ae8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bf8aed, SUC_lexicon_tags_2 },
    { 0x53ff8aed, SUC_lexicon_tags_2 },
    { 0x78ff8aef, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc1bf8af5, SUC_lexicon_tags_0 },
    { 0xed7f8af6, SUC_lexicon_tags_19 },
    { 0xbe5f8af7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x86df8af9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x839f8afc, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd63f8afe, SUC_lexicon_tags_87 },
    { 0x93bf8aff, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa9bf8b01, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x92ff8b03, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1f8b08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3f8b0e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa0ff8b10, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadff8b14, SUC_lexicon_tags_17 },
    { 0xb17f8b14, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0x6d5f8b17, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x1eff8b1a, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fff8b21, SUC_lexicon_tags_63 },
    { 0xa8bf8b22, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xb51f8b25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa2bf8b27, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe61f8b29, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7f8b2e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa33f8b30, SUC_lexicon_tags_1 },
    { 0x9c9f8b30, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9f8b36, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x65ff8b38, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41f8b3f, SUC_lexicon_tags_37 },
    { 0xdf5f8b40, SUC_lexicon_tags_38 },
    { 0xeebf8b41, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xfeff8b43, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127f8b4a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x7adf8b4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x60df8b4f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61f8b56, SUC_lexicon_tags_37 },
    { 0x9eff8b57, SUC_lexicon_tags_63 },
    { 0x281f8b58, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7f8b5e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x297f8b60, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5f8b69, SUC_lexicon_tags_66 },
    { 0xa57f8b6a, SUC_lexicon_tags_27 },
    { 0xfa7f8b6b, SUC_lexicon_tags_33 },
    { 0xc31f8b6b, SUC_lexicon_tags_2 },
    { 0x67bf8b6d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf8b78, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x723f8b7a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f7f8b7c, SUC_lexicon_tags_5 },
    { 0xa6bf8b7c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383f8b82, SUC_lexicon_tags_15 },
    { 0xf17f8b83, SUC_lexicon_tags_57 },
    { 0x207f8b83, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf8b89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49f8b8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x58df8b8f, SUC_lexicon_tags_29 },
    { 0x359f8b90, SUC_lexicon_tags_33 },
    { 0x589f8b91, SUC_lexicon_tags_25 },
    { 0x429f8b92, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa39f8b99, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0x5c3f8b9b, SUC_lexicon_tags_61 },
    { 0x1f5f8b9c, SUC_lexicon_tags_43 },
    { 0x623f8b9d, SUC_lexicon_tags_2 },
    { 0x97df8b9c, SUC_lexicon_tags_7 },
    { 0x781f8b9d, SUC_lexicon_tags_14 },
    { 0x621f8b9c, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61df8ba5, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bf8bab, SUC_lexicon_tags_30 },
    { 0xf83f8bab, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961f8bb1, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ff8bbb, SUC_lexicon_tags_47 },
    { 0xdebf8bbb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8df8bc3, SUC_lexicon_tags_16 },
    { 0x4cdf8bc4, SUC_lexicon_tags_17 },
    { 0x241f8bc5, SUC_lexicon_tags_15 },
    { 0x30ff8bc6, SUC_lexicon_tags_38 },
    { 0x725f8bc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5df8bcb, SUC_lexicon_tags_11 },
    { 0xdfdf8bcc, SUC_lexicon_tags_29 },
    { 0x1abf8bcd, SUC_lexicon_tags_43 },
    { 0x781f8bce, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x99f8bd1, SUC_lexicon_tags_37 },
    { 0x505f8bd1, SUC_lexicon_tags_425 },
    { 0, NULL },
    { 0x5a9f8bd4, SUC_lexicon_tags_23 },
    { 0x577f8bd5, SUC_lexicon_tags_15 },
    { 0xf41f8bd6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b7f8bda, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xce3f8bdc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x6c7f8bde, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12ff8be3, SUC_lexicon_tags_62 },
    { 0x529f8be4, SUC_lexicon_tags_25 },
    { 0x5c3f8be5, SUC_lexicon_tags_1 },
    { 0xa29f8be6, SUC_lexicon_tags_2 },
    { 0xe0ff8be6, SUC_lexicon_tags_23 },
    { 0xba3f8be8, SUC_lexicon_tags_64 },
    { 0xf35f8be6, SUC_lexicon_tags_87 },
    { 0x261f8be4, SUC_lexicon_tags_33 },
    { 0x845f8be5, SUC_lexicon_tags_47 },
    { 0xf33f8bec, SUC_lexicon_tags_11 },
    { 0x545f8bed, SUC_lexicon_tags_45 },
    { 0x673f8bea, SUC_lexicon_tags_1 },
    { 0xcdff8bef, SUC_lexicon_tags_11 },
    { 0x749f8bf0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbf8bf8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bff8bfd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x873f8c00, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc8df8c02, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6df8c06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57f8c09, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x6bff8c0b, SUC_lexicon_tags_7 },
    { 0x595f8c0c, SUC_lexicon_tags_17 },
    { 0xde9f8c0b, SUC_lexicon_tags_37 },
    { 0xe55f8c0e, SUC_lexicon_tags_25 },
    { 0xca7f8c0f, SUC_lexicon_tags_2 },
    { 0x9c5f8c0e, SUC_lexicon_tags_26 },
    { 0xf2ff8c0e, SUC_lexicon_tags_2 },
    { 0xa51f8c11, SUC_lexicon_tags_27 },
    { 0xed5f8c13, SUC_lexicon_tags_2 },
    { 0xbf5f8c14, SUC_lexicon_tags_43 },
    { 0x3adf8c13, SUC_lexicon_tags_7 },
    { 0x553f8c16, SUC_lexicon_tags_7 },
    { 0xe47f8c17, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xe13f8c1a, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43f8c22, SUC_lexicon_tags_25 },
    { 0x433f8c22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10df8c27, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59f8c2a, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ff8c30, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6b1f8c32, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda1f8c34, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb6ff8c36, SUC_lexicon_tags_2 },
    { 0xbdbf8c36, SUC_lexicon_tags_79 },
    { 0xfa1f8c38, SUC_lexicon_tags_1 },
    { 0x27bf8c39, SUC_lexicon_tags_38 },
    { 0x90bf8c39, SUC_lexicon_tags_1 },
    { 0x75f8c3b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xea7f8c3d, SUC_lexicon_tags_10 },
    { 0xea7f8c3e, SUC_lexicon_tags_0 },
    { 0x565f8c3f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xaebf8c41, SUC_lexicon_tags_49 },
    { 0xe41f8c42, SUC_lexicon_tags_2 },
    { 0x171f8c43, SUC_lexicon_tags_15 },
    { 0xef9f8c44, SUC_lexicon_tags_15 },
    { 0x2e1f8c43, SUC_lexicon_tags_47 },
    { 0x7aff8c43, SUC_lexicon_tags_2 },
    { 0xaa9f8c45, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebf8c4c, SUC_lexicon_tags_23 },
    { 0xaabf8c4d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77f8c5d, SUC_lexicon_tags_2 },
    { 0x271f8c5e, SUC_lexicon_tags_79 },
    { 0xd87f8c5e, SUC_lexicon_tags_21 },
    { 0x479f8c60, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8a9f8c62, SUC_lexicon_tags_4 },
    { 0x675f8c62, SUC_lexicon_tags_16 },
    { 0xba7f8c62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdf8c67, SUC_lexicon_tags_68 },
    { 0x77ff8c68, SUC_lexicon_tags_66 },
    { 0x453f8c67, SUC_lexicon_tags_45 },
    { 0x537f8c6a, SUC_lexicon_tags_27 },
    { 0x321f8c6b, SUC_lexicon_tags_30 },
    { 0xfb7f8c6c, SUC_lexicon_tags_14 },
    { 0x8e7f8c6c, SUC_lexicon_tags_188 },
    { 0, NULL },
    { 0xbf5f8c6f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1f8c74, SUC_lexicon_tags_65 },
    { 0x7ebf8c75, SUC_lexicon_tags_37 },
    { 0x3b1f8c74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x847f8c79, SUC_lexicon_tags_7 },
    { 0x673f8c79, SUC_lexicon_tags_33 },
    { 0x573f8c7b, SUC_lexicon_tags_213 },
    { 0xbe7f8c7a, SUC_lexicon_tags_40 },
    { 0x2bff8c7d, SUC_lexicon_tags_10 },
    { 0x29bf8c7e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x61ff8c80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3f8c87, SUC_lexicon_tags_29 },
    { 0xcfbf8c88, SUC_lexicon_tags_21 },
    { 0xcbf8c89, SUC_lexicon_tags_12 },
    { 0x521f8c8a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdf8c8d, SUC_lexicon_tags_1 },
    { 0x9d3f8c8d, SUC_lexicon_tags_50 },
    { 0xe0df8c8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2ff8c94, SUC_lexicon_tags_3 },
    { 0xd25f8c94, SUC_lexicon_tags_11 },
    { 0xda9f8c96, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ff8c99, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd85f8c9b, SUC_lexicon_tags_0 },
    { 0xfff8c9b, SUC_lexicon_tags_7 },
    { 0xd7df8c9c, SUC_lexicon_tags_0 },
    { 0x147f8c9d, SUC_lexicon_tags_37 },
    { 0x465f8c9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x739f8ca3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29f8cae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5f8cb5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x623f8cb8, SUC_lexicon_tags_17 },
    { 0xf9bf8cb9, SUC_lexicon_tags_15 },
    { 0x4e1f8cb9, SUC_lexicon_tags_33 },
    { 0x459f8cba, SUC_lexicon_tags_25 },
    { 0xbb7f8cbc, SUC_lexicon_tags_37 },
    { 0x7ebf8cbc, SUC_lexicon_tags_12 },
    { 0x3b7f8cbb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x70ff8cc0, SUC_lexicon_tags_624 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87f8cc3, SUC_lexicon_tags_16 },
    { 0x367f8cc3, SUC_lexicon_tags_15 },
    { 0xabdf8cc5, SUC_lexicon_tags_9 },
    { 0x6e7f8cc6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2c3f8cc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x939f8ccd, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x391f8cd3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481f8cd9, SUC_lexicon_tags_17 },
    { 0x791f8cda, SUC_lexicon_tags_47 },
    { 0x559f8cdb, SUC_lexicon_tags_15 },
    { 0x973f8cdb, SUC_lexicon_tags_34 },
    { 0xb3ff8cd9, SUC_lexicon_tags_37 },
    { 0x113f8cde, SUC_lexicon_tags_1 },
    { 0xcf1f8cd9, SUC_lexicon_tags_19 },
    { 0xd2df8ce0, SUC_lexicon_tags_29 },
    { 0x757f8ce1, SUC_lexicon_tags_25 },
    { 0x44df8ce2, SUC_lexicon_tags_38 },
    { 0xb4ff8cde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91f8cea, SUC_lexicon_tags_14 },
    { 0xe85f8cea, SUC_lexicon_tags_33 },
    { 0xe93f8cec, SUC_lexicon_tags_60 },
    { 0x673f8ced, SUC_lexicon_tags_47 },
    { 0x1a3f8cec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x297f8cf0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c3f8cf3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37f8cf7, SUC_lexicon_tags_2 },
    { 0xda3f8cf7, SUC_lexicon_tags_28 },
    { 0x81bf8cf8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105f8d00, SUC_lexicon_tags_7 },
    { 0xa01f8d01, SUC_lexicon_tags_70 },
    { 0x2f7f8d00, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xbd3f8d04, SUC_lexicon_tags_27 },
    { 0x449f8d05, SUC_lexicon_tags_25 },
    { 0x773f8d05, SUC_lexicon_tags_12 },
    { 0x559f8d04, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961f8d0b, SUC_lexicon_tags_47 },
    { 0x7e5f8d0c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7df8d12, SUC_lexicon_tags_37 },
    { 0x32df8d13, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9bf8d15, SUC_lexicon_tags_15 },
    { 0xee3f8d16, SUC_lexicon_tags_17 },
    { 0xa19f8d17, SUC_lexicon_tags_25 },
    { 0x1a5f8d17, SUC_lexicon_tags_9 },
    { 0xc61f8d19, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93f8d1d, SUC_lexicon_tags_33 },
    { 0x797f8d1e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x29df8d20, SUC_lexicon_tags_1 },
    { 0x515f8d21, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb8df8d23, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3f8d29, SUC_lexicon_tags_25 },
    { 0x46ff8d2a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd15f8d2c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x325f8d30, SUC_lexicon_tags_19 },
    { 0xc69f8d30, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145f8d3d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x229f8d3f, SUC_lexicon_tags_25 },
    { 0x125f8d3f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xaebf8d42, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd2df8d44, SUC_lexicon_tags_1 },
    { 0xfbdf8d45, SUC_lexicon_tags_25 },
    { 0xa3ff8d45, SUC_lexicon_tags_0 },
    { 0x957f8d45, SUC_lexicon_tags_9 },
    { 0xa39f8d48, SUC_lexicon_tags_28 },
    { 0x71f8d49, SUC_lexicon_tags_63 },
    { 0x31f8d4a, SUC_lexicon_tags_14 },
    { 0x4e1f8d4a, SUC_lexicon_tags_37 },
    { 0x11df8d4a, SUC_lexicon_tags_39 },
    { 0x54df8d4d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x38df8d4f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x569f8d54, SUC_lexicon_tags_33 },
    { 0xefdf8d55, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedf8d59, SUC_lexicon_tags_33 },
    { 0xdfff8d5a, SUC_lexicon_tags_660 },
    { 0x159f8d5b, SUC_lexicon_tags_11 },
    { 0x35bf8d5c, SUC_lexicon_tags_25 },
    { 0xfdbf8d5b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5f8d61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa23f8d63, SUC_lexicon_tags_17 },
    { 0x6f1f8d64, SUC_lexicon_tags_37 },
    { 0x20ff8d65, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23f8d69, SUC_lexicon_tags_9 },
    { 0xc63f8d6a, SUC_lexicon_tags_11 },
    { 0x82df8d6b, SUC_lexicon_tags_36 },
    { 0xeb3f8d69, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf8d76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x237f8d78, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xd85f8d7b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31f8d82, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x983f8d84, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd25f8d86, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x149f8d88, SUC_lexicon_tags_17 },
    { 0xf65f8d88, SUC_lexicon_tags_1 },
    { 0x6e3f8d8a, SUC_lexicon_tags_25 },
    { 0x889f8d88, SUC_lexicon_tags_12 },
    { 0x58df8d89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc7f8d8e, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275f8d94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7f8d98, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9f8d9b, SUC_lexicon_tags_37 },
    { 0x377f8d9c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01f8d9f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53f8da6, SUC_lexicon_tags_30 },
    { 0xe21f8da7, SUC_lexicon_tags_15 },
    { 0xe97f8da8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x135f8daa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3df8db0, SUC_lexicon_tags_33 },
    { 0x11df8db1, SUC_lexicon_tags_28 },
    { 0x165f8db2, SUC_lexicon_tags_10 },
    { 0x219f8db2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bf8db7, SUC_lexicon_tags_7 },
    { 0xf73f8db7, SUC_lexicon_tags_25 },
    { 0x79df8db9, SUC_lexicon_tags_10 },
    { 0x41f8dba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3e1f8dbc, SUC_lexicon_tags_29 },
    { 0xe43f8dbc, SUC_lexicon_tags_65 },
    { 0xc1df8dbe, SUC_lexicon_tags_30 },
    { 0xe31f8dbd, SUC_lexicon_tags_4 },
    { 0x7b9f8dc0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x28ff8dc2, SUC_lexicon_tags_0 },
    { 0xaf7f8dc3, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0xa71f8dc5, SUC_lexicon_tags_25 },
    { 0x8b5f8dc6, SUC_lexicon_tags_25 },
    { 0xb55f8dc5, SUC_lexicon_tags_11 },
    { 0xa9df8dc7, SUC_lexicon_tags_26 },
    { 0xa47f8dc9, SUC_lexicon_tags_12 },
    { 0xf25f8dc9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9e3f8dcc, SUC_lexicon_tags_37 },
    { 0xf4df8dcc, SUC_lexicon_tags_60 },
    { 0xfc1f8dcd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49f8dd2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x805f8dd4, SUC_lexicon_tags_27 },
    { 0xdb5f8dd4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x367f8dd7, SUC_lexicon_tags_19 },
    { 0xa27f8dd8, SUC_lexicon_tags_87 },
    { 0x60bf8dd8, SUC_lexicon_tags_60 },
    { 0x399f8dd7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x975f8de1, SUC_lexicon_tags_60 },
    { 0x54df8de1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bf8ded, SUC_lexicon_tags_38 },
    { 0x513f8ded, SUC_lexicon_tags_14 },
    { 0x6c7f8dee, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xbaff8df1, SUC_lexicon_tags_0 },
    { 0xe3ff8df2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcff8df5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47f8df9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bf8e01, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6df8e06, SUC_lexicon_tags_28 },
    { 0x425f8e07, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb9ff8e09, SUC_lexicon_tags_8 },
    { 0xc67f8e0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5f8e0c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd53f8e0e, SUC_lexicon_tags_60 },
    { 0xccdf8e0f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ff8e14, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7f8e18, SUC_lexicon_tags_11 },
    { 0xb9df8e19, SUC_lexicon_tags_2 },
    { 0x26ff8e1a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e1f8e1e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ff8e21, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1f8e25, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4e1f8e27, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa75f8e2b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x493f8e2e, SUC_lexicon_tags_17 },
    { 0x173f8e2f, SUC_lexicon_tags_12 },
    { 0xc19f8e2f, SUC_lexicon_tags_49 },
    { 0x98ff8e2f, SUC_lexicon_tags_27 },
    { 0xbd9f8e32, SUC_lexicon_tags_31 },
    { 0x145f8e33, SUC_lexicon_tags_0 },
    { 0xc9df8e33, SUC_lexicon_tags_15 },
    { 0xb67f8e35, SUC_lexicon_tags_9 },
    { 0x485f8e36, SUC_lexicon_tags_25 },
    { 0xafbf8e33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x39df8e39, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf3f8e3c, SUC_lexicon_tags_63 },
    { 0x5f8e3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x72df8e3f, SUC_lexicon_tags_1 },
    { 0xc77f8e40, SUC_lexicon_tags_12 },
    { 0x48ff8e3f, SUC_lexicon_tags_17 },
    { 0x1bbf8e40, SUC_lexicon_tags_37 },
    { 0xaa7f8e43, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf03f8e45, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xa75f8e48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb37f8e4a, SUC_lexicon_tags_2 },
    { 0xc43f8e4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdf8e4e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x3aff8e50, SUC_lexicon_tags_63 },
    { 0x991f8e50, SUC_lexicon_tags_49 },
    { 0x333f8e52, SUC_lexicon_tags_3 },
    { 0x8cbf8e53, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8c5f8e55, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x97bf8e5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01f8e61, SUC_lexicon_tags_2 },
    { 0xfff8e62, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1f8e66, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x443f8e69, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bf8e6d, SUC_lexicon_tags_11 },
    { 0xa57f8e6e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbf8e72, SUC_lexicon_tags_37 },
    { 0xbb5f8e73, SUC_lexicon_tags_1 },
    { 0x9bbf8e72, SUC_lexicon_tags_15 },
    { 0x6c1f8e75, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x735f8e77, SUC_lexicon_tags_7 },
    { 0x25f8e77, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe43f8e7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735f8e7e, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0xa2df8e81, SUC_lexicon_tags_46 },
    { 0xd3f8e82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73f8e86, SUC_lexicon_tags_63 },
    { 0xeff8e87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25f8e8b, SUC_lexicon_tags_10 },
    { 0xfd5f8e8c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x431f8e8e, SUC_lexicon_tags_148 },
    { 0x13bf8e8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145f8e95, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abf8e99, SUC_lexicon_tags_14 },
    { 0x635f8e9a, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ff8ea4, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93f8eaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65f8eb1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x139f8eb4, SUC_lexicon_tags_47 },
    { 0x989f8eb5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x30df8eb8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979f8ebf, SUC_lexicon_tags_8 },
    { 0x651f8ec0, SUC_lexicon_tags_33 },
    { 0x809f8ec1, SUC_lexicon_tags_12 },
    { 0x9f7f8ec2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb75f8ec4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11f8ecb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x911f8ecd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x297f8ed0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x87ff8ed2, SUC_lexicon_tags_15 },
    { 0x625f8ed3, SUC_lexicon_tags_25 },
    { 0x7b9f8ed2, SUC_lexicon_tags_30 },
    { 0xc5ff8ed5, SUC_lexicon_tags_12 },
    { 0x133f8ed3, SUC_lexicon_tags_38 },
    { 0xbf3f8ed7, SUC_lexicon_tags_25 },
    { 0x4d5f8ed8, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc37f8eda, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1f8edf, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65f8ee4, SUC_lexicon_tags_9 },
    { 0xf59f8ee5, SUC_lexicon_tags_17 },
    { 0xb73f8ee6, SUC_lexicon_tags_29 },
    { 0x15df8ee4, SUC_lexicon_tags_21 },
    { 0xda5f8ee8, SUC_lexicon_tags_2 },
    { 0x23bf8ee8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x271f8eeb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x50bf8eed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611f8ef1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x299f8ef3, SUC_lexicon_tags_49 },
    { 0x2a3f8ef3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb45f8ef6, SUC_lexicon_tags_7 },
    { 0x77f8ef6, SUC_lexicon_tags_33 },
    { 0xaf1f8ef6, SUC_lexicon_tags_60 },
    { 0xc3ff8ef9, SUC_lexicon_tags_39 },
    { 0xee5f8efa, SUC_lexicon_tags_11 },
    { 0xc4df8ef9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x613f8efd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bf8f04, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d9f8f08, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x22df8f0a, SUC_lexicon_tags_17 },
    { 0x9f1f8f0a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x147f8f0e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3f8f11, SUC_lexicon_tags_72 },
    { 0xa9ff8f12, SUC_lexicon_tags_49 },
    { 0xce5f8f11, SUC_lexicon_tags_22 },
    { 0xa03f8f11, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x813f8f17, SUC_lexicon_tags_33 },
    { 0xed9f8f18, SUC_lexicon_tags_10 },
    { 0xc21f8f18, SUC_lexicon_tags_23 },
    { 0xbdbf8f18, SUC_lexicon_tags_48 },
    { 0x425f8f1b, SUC_lexicon_tags_2 },
    { 0x73df8f18, SUC_lexicon_tags_15 },
    { 0xff3f8f1d, SUC_lexicon_tags_107 },
    { 0xb27f8f18, SUC_lexicon_tags_19 },
    { 0xd59f8f18, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x45bf8f21, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3f8f24, SUC_lexicon_tags_52 },
    { 0, NULL },
    { 0, NULL },
    { 0xe69f8f27, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4ddf8f29, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0xcdf8f2b, SUC_lexicon_tags_38 },
    { 0x7f1f8f2c, SUC_lexicon_tags_29 },
    { 0x8ff8f2d, SUC_lexicon_tags_9 },
    { 0x35bf8f2e, SUC_lexicon_tags_25 },
    { 0xff5f8f2f, SUC_lexicon_tags_15 },
    { 0x6c9f8f30, SUC_lexicon_tags_2 },
    { 0xe2ff8f2f, SUC_lexicon_tags_30 },
    { 0x95df8f32, SUC_lexicon_tags_65 },
    { 0xd21f8f2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2edf8f38, SUC_lexicon_tags_45 },
    { 0xf45f8f38, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcd5f8f3b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357f8f41, SUC_lexicon_tags_10 },
    { 0x777f8f42, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xbabf8f44, SUC_lexicon_tags_65 },
    { 0x68df8f45, SUC_lexicon_tags_14 },
    { 0x5e9f8f44, SUC_lexicon_tags_29 },
    { 0xfd9f8f47, SUC_lexicon_tags_23 },
    { 0xe57f8f48, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x773f8f4d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7aff8f51, SUC_lexicon_tags_45 },
    { 0x31f8f52, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe75f8f54, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7f8f61, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x613f8f63, SUC_lexicon_tags_39 },
    { 0xacbf8f63, SUC_lexicon_tags_32 },
    { 0x15df8f63, SUC_lexicon_tags_63 },
    { 0x23ff8f66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x649f8f68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879f8f6c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1f8f72, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x3eff8f74, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebf8f78, SUC_lexicon_tags_19 },
    { 0xe7ff8f78, SUC_lexicon_tags_6 },
    { 0x855f8f79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x37ff8f7c, SUC_lexicon_tags_65 },
    { 0xbd7f8f7d, SUC_lexicon_tags_12 },
    { 0xc79f8f7e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7f8f88, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcff8f8c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x56df8f8e, SUC_lexicon_tags_2 },
    { 0x31f8f8e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79bf8f91, SUC_lexicon_tags_37 },
    { 0xc71f8f91, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x903f8f95, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5f8f9a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdf8f9f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9df8fa2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a9f8fa5, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55f8fac, SUC_lexicon_tags_23 },
    { 0xf83f8fac, SUC_lexicon_tags_7 },
    { 0x965f8fae, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63f8fb1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x25bf8fb3, SUC_lexicon_tags_37 },
    { 0x425f8fb4, SUC_lexicon_tags_10 },
    { 0x265f8fb3, SUC_lexicon_tags_7 },
    { 0xdf8fb6, SUC_lexicon_tags_8 },
    { 0x3adf8fb7, SUC_lexicon_tags_15 },
    { 0x9b1f8fb8, SUC_lexicon_tags_17 },
    { 0x215f8fb3, SUC_lexicon_tags_11 },
    { 0x953f8fba, SUC_lexicon_tags_17 },
    { 0x773f8fbb, SUC_lexicon_tags_7 },
    { 0x351f8fb7, SUC_lexicon_tags_23 },
    { 0xa7ff8fbd, SUC_lexicon_tags_37 },
    { 0x361f8fbe, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bf8fc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfbff8fc3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe11f8fc5, SUC_lexicon_tags_17 },
    { 0x2a9f8fc6, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66bf8fcf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x861f8fd1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55f8fe1, SUC_lexicon_tags_19 },
    { 0xf4ff8fe2, SUC_lexicon_tags_17 },
    { 0x82df8fe3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x839f8fe6, SUC_lexicon_tags_25 },
    { 0x63ff8fe6, SUC_lexicon_tags_15 },
    { 0x143f8fe8, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735f8ff0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9f8ff5, SUC_lexicon_tags_21 },
    { 0x5aff8ff6, SUC_lexicon_tags_584 },
    { 0x7c3f8ff5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ff8ffc, SUC_lexicon_tags_106 },
    { 0x625f8ffd, SUC_lexicon_tags_7 },
    { 0x725f8ffe, SUC_lexicon_tags_25 },
    { 0xe0bf8ffe, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xb6ff9001, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb91f9003, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f9f9009, SUC_lexicon_tags_7 },
    { 0xfc7f900a, SUC_lexicon_tags_530 },
    { 0x979f900a, SUC_lexicon_tags_4 },
    { 0xaebf900c, SUC_lexicon_tags_57 },
    { 0x751f9009, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc81f900f, SUC_lexicon_tags_493 },
    { 0x3b7f900f, SUC_lexicon_tags_31 },
    { 0x61f9011, SUC_lexicon_tags_27 },
    { 0xfa3f900f, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x611f901f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa67f9021, SUC_lexicon_tags_85 },
    { 0x37f9021, SUC_lexicon_tags_38 },
    { 0x469f9023, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7df9027, SUC_lexicon_tags_2 },
    { 0x68df9028, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497f902d, SUC_lexicon_tags_47 },
    { 0xb21f902e, SUC_lexicon_tags_14 },
    { 0xb9f902f, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddf9032, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xec9f9034, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb5f9036, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x827f9039, SUC_lexicon_tags_37 },
    { 0x63ff903a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315f903f, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x593f9042, SUC_lexicon_tags_25 },
    { 0x41bf9043, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x513f9047, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7b5f9049, SUC_lexicon_tags_33 },
    { 0x61bf904a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bf9052, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87f9056, SUC_lexicon_tags_27 },
    { 0x5a3f9057, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe17f9059, SUC_lexicon_tags_2 },
    { 0xd67f905a, SUC_lexicon_tags_17 },
    { 0x771f905b, SUC_lexicon_tags_7 },
    { 0x9b1f905c, SUC_lexicon_tags_1 },
    { 0xef9f9059, SUC_lexicon_tags_11 },
    { 0x563f905d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbfdf9060, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x367f9062, SUC_lexicon_tags_15 },
    { 0x115f9062, SUC_lexicon_tags_47 },
    { 0xd85f9064, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x943f9069, SUC_lexicon_tags_1 },
    { 0xf27f9069, SUC_lexicon_tags_47 },
    { 0xb19f9069, SUC_lexicon_tags_23 },
    { 0xfa3f906c, SUC_lexicon_tags_12 },
    { 0xebbf9069, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x89ff9070, SUC_lexicon_tags_17 },
    { 0x903f9070, SUC_lexicon_tags_25 },
    { 0x72bf9070, SUC_lexicon_tags_0 },
    { 0x93f9073, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf31f9075, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xb9f9077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1f907b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bff9082, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f7f9087, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5f908a, SUC_lexicon_tags_38 },
    { 0x853f908b, SUC_lexicon_tags_1 },
    { 0xff7f908c, SUC_lexicon_tags_40 },
    { 0xc27f908c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27f9091, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5f9097, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76df909c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x759f909e, SUC_lexicon_tags_12 },
    { 0x5c7f909f, SUC_lexicon_tags_23 },
    { 0xe2bf90a0, SUC_lexicon_tags_7 },
    { 0xcedf90a1, SUC_lexicon_tags_6 },
    { 0x43df90a1, SUC_lexicon_tags_37 },
    { 0x321f90a1, SUC_lexicon_tags_27 },
    { 0x9dff90a4, SUC_lexicon_tags_39 },
    { 0x565f90a4, SUC_lexicon_tags_29 },
    { 0x38df90a0, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2a7f90a8, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7f90af, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0x63ff90b1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7f90b5, SUC_lexicon_tags_17 },
    { 0xbe3f90b6, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85df90bb, SUC_lexicon_tags_9 },
    { 0x9dff90bc, SUC_lexicon_tags_154 },
    { 0x2f5f90bd, SUC_lexicon_tags_25 },
    { 0x50bf90bc, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x853f90c2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631f90c7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x379f90c9, SUC_lexicon_tags_4 },
    { 0x8bf90ca, SUC_lexicon_tags_25 },
    { 0x6e7f90cb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e7f90d1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93f90d8, SUC_lexicon_tags_11 },
    { 0x695f90d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9df90e0, SUC_lexicon_tags_2 },
    { 0x83ff90e1, SUC_lexicon_tags_25 },
    { 0x333f90e1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x443f90e8, SUC_lexicon_tags_38 },
    { 0xbfff90e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9f90ed, SUC_lexicon_tags_29 },
    { 0x9f5f90ee, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9f90f3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x929f90f6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xd17f90f9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb21f90fb, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2b1f90fd, SUC_lexicon_tags_19 },
    { 0xeff90fd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ff9103, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe37f9105, SUC_lexicon_tags_2 },
    { 0x163f9105, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x837f9108, SUC_lexicon_tags_66 },
    { 0xeddf9108, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1f910d, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1f9112, SUC_lexicon_tags_1 },
    { 0xcbbf9113, SUC_lexicon_tags_27 },
    { 0xad3f9113, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fff911d, SUC_lexicon_tags_11 },
    { 0xfd5f911e, SUC_lexicon_tags_63 },
    { 0xb53f911e, SUC_lexicon_tags_11 },
    { 0xe95f9120, SUC_lexicon_tags_12 },
    { 0xac5f911f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f3f9123, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74df9127, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5f912c, SUC_lexicon_tags_2 },
    { 0xe3ff912d, SUC_lexicon_tags_4 },
    { 0x10bf912e, SUC_lexicon_tags_11 },
    { 0x139f912f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bf9133, SUC_lexicon_tags_11 },
    { 0xc11f9134, SUC_lexicon_tags_63 },
    { 0xb9ff9135, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5f9139, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87f913d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7d9f913f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ff9143, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x38ff9145, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x44ff9147, SUC_lexicon_tags_2 },
    { 0x3b5f9148, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2eff914a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x65ff914c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfadf9150, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21f9159, SUC_lexicon_tags_17 },
    { 0x7bf915a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfedf915c, SUC_lexicon_tags_17 },
    { 0x20ff915d, SUC_lexicon_tags_17 },
    { 0x62df915e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f7f9162, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xceff9164, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x305f9166, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x933f9169, SUC_lexicon_tags_12 },
    { 0x7f916a, SUC_lexicon_tags_14 },
    { 0xb77f916b, SUC_lexicon_tags_44 },
    { 0xaa1f916c, SUC_lexicon_tags_12 },
    { 0x557f9169, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xf0bf916f, SUC_lexicon_tags_2 },
    { 0x4f9f916f, SUC_lexicon_tags_12 },
    { 0x40ff9171, SUC_lexicon_tags_305 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bf9177, SUC_lexicon_tags_8 },
    { 0x695f9178, SUC_lexicon_tags_7 },
    { 0xf89f9178, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ff917d, SUC_lexicon_tags_12 },
    { 0x7b9f917d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebf9181, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe97f9184, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5cff9186, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1f918b, SUC_lexicon_tags_12 },
    { 0x55df918b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1f918f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f9197, SUC_lexicon_tags_37 },
    { 0xeabf9198, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdff919f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbf91a4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6ff91ac, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7f91af, SUC_lexicon_tags_23 },
    { 0xcfdf91b0, SUC_lexicon_tags_17 },
    { 0x6d5f91b0, SUC_lexicon_tags_17 },
    { 0x86bf91b2, SUC_lexicon_tags_33 },
    { 0x40ff91b3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x581f91b6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd01f91b9, SUC_lexicon_tags_17 },
    { 0x2b5f91ba, SUC_lexicon_tags_21 },
    { 0xa7df91bb, SUC_lexicon_tags_7 },
    { 0xb75f91bb, SUC_lexicon_tags_1 },
    { 0x2ebf91bb, SUC_lexicon_tags_70 },
    { 0x5eff91be, SUC_lexicon_tags_17 },
    { 0xb61f91bf, SUC_lexicon_tags_7 },
    { 0x615f91bf, SUC_lexicon_tags_66 },
    { 0x15f91c1, SUC_lexicon_tags_15 },
    { 0x35bf91c2, SUC_lexicon_tags_33 },
    { 0xe59f91c2, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57f91c6, SUC_lexicon_tags_30 },
    { 0x90ff91c7, SUC_lexicon_tags_2 },
    { 0xab3f91c7, SUC_lexicon_tags_65 },
    { 0x2ddf91c7, SUC_lexicon_tags_27 },
    { 0xe77f91c8, SUC_lexicon_tags_29 },
    { 0xc27f91ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x311f91cf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x395f91d1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ff91de, SUC_lexicon_tags_23 },
    { 0x31ff91df, SUC_lexicon_tags_23 },
    { 0xfc9f91e0, SUC_lexicon_tags_38 },
    { 0x93ff91e1, SUC_lexicon_tags_12 },
    { 0x721f91e1, SUC_lexicon_tags_43 },
    { 0x657f91e1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x77f91e6, SUC_lexicon_tags_26 },
    { 0x4ebf91e7, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3f91ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261f91f4, SUC_lexicon_tags_9 },
    { 0xe71f91f5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17df91f7, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427f91fd, SUC_lexicon_tags_26 },
    { 0x3b3f91fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf47f9201, SUC_lexicon_tags_19 },
    { 0x883f9201, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68df9206, SUC_lexicon_tags_17 },
    { 0x37ff9207, SUC_lexicon_tags_60 },
    { 0x41f9208, SUC_lexicon_tags_1 },
    { 0xf3f9208, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607f920e, SUC_lexicon_tags_87 },
    { 0xb8bf920e, SUC_lexicon_tags_7 },
    { 0x47ff920f, SUC_lexicon_tags_37 },
    { 0x757f920e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63f9214, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x52bf9216, SUC_lexicon_tags_19 },
    { 0x6eff9217, SUC_lexicon_tags_37 },
    { 0x19df9217, SUC_lexicon_tags_25 },
    { 0xa21f9217, SUC_lexicon_tags_37 },
    { 0x92df9219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09f9223, SUC_lexicon_tags_17 },
    { 0x4cdf9224, SUC_lexicon_tags_37 },
    { 0xec5f9223, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf922a, SUC_lexicon_tags_100 },
    { 0x551f922b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x639f922e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35f9235, SUC_lexicon_tags_25 },
    { 0x785f9236, SUC_lexicon_tags_46 },
    { 0x373f9237, SUC_lexicon_tags_12 },
    { 0xc4df9238, SUC_lexicon_tags_11 },
    { 0x66df9235, SUC_lexicon_tags_37 },
    { 0x355f9238, SUC_lexicon_tags_17 },
    { 0x709f923b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x8adf923d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f9f9240, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdc1f9242, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cff9246, SUC_lexicon_tags_17 },
    { 0x15df9247, SUC_lexicon_tags_40 },
    { 0x7e1f9247, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9f924c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x4bdf924e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xa0bf9250, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109f9255, SUC_lexicon_tags_15 },
    { 0xf2bf9256, SUC_lexicon_tags_25 },
    { 0x789f9257, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x8b7f9259, SUC_lexicon_tags_11 },
    { 0x263f925a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x72bf925c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3b5f925e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0x207f9260, SUC_lexicon_tags_7 },
    { 0x92ff9260, SUC_lexicon_tags_11 },
    { 0x5eff9260, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x21df9265, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff926b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13f926f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x625f9272, SUC_lexicon_tags_86 },
    { 0, NULL },
    { 0x175f9274, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe51f9277, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5e5f9279, SUC_lexicon_tags_18 },
    { 0x893f9279, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df9284, SUC_lexicon_tags_38 },
    { 0x7e1f9284, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x18df9288, SUC_lexicon_tags_12 },
    { 0xb5bf9288, SUC_lexicon_tags_7 },
    { 0xc7ff9289, SUC_lexicon_tags_37 },
    { 0x781f928b, SUC_lexicon_tags_11 },
    { 0xd21f9289, SUC_lexicon_tags_11 },
    { 0x4c5f928a, SUC_lexicon_tags_10 },
    { 0xebbf928d, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111f9294, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x285f9297, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb15f929f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x617f92a2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x133f92a5, SUC_lexicon_tags_37 },
    { 0x7cdf92a6, SUC_lexicon_tags_118 },
    { 0x603f92a6, SUC_lexicon_tags_7 },
    { 0xb91f92a5, SUC_lexicon_tags_37 },
    { 0xe79f92a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1f92ac, SUC_lexicon_tags_31 },
    { 0x93df92ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1b7f92af, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x8a3f92b1, SUC_lexicon_tags_45 },
    { 0x26bf92b2, SUC_lexicon_tags_30 },
    { 0x413f92b3, SUC_lexicon_tags_43 },
    { 0x973f92b4, SUC_lexicon_tags_1 },
    { 0x571f92b5, SUC_lexicon_tags_47 },
    { 0x11bf92b6, SUC_lexicon_tags_26 },
    { 0x83bf92b4, SUC_lexicon_tags_14 },
    { 0x1f9f92b1, SUC_lexicon_tags_62 },
    { 0x2d7f92b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeebf92bb, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1dbf92bd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257f92c4, SUC_lexicon_tags_11 },
    { 0xb25f92c5, SUC_lexicon_tags_11 },
    { 0x1e3f92c5, SUC_lexicon_tags_38 },
    { 0x253f92c5, SUC_lexicon_tags_9 },
    { 0xec5f92c8, SUC_lexicon_tags_7 },
    { 0x53bf92c9, SUC_lexicon_tags_37 },
    { 0x50ff92ca, SUC_lexicon_tags_14 },
    { 0xccff92cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9a1f92cd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821f92d3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x805f92d5, SUC_lexicon_tags_69 },
    { 0xd71f92d6, SUC_lexicon_tags_15 },
    { 0xcebf92d6, SUC_lexicon_tags_17 },
    { 0x681f92d5, SUC_lexicon_tags_11 },
    { 0xbedf92d9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245f92dd, SUC_lexicon_tags_47 },
    { 0xa79f92de, SUC_lexicon_tags_49 },
    { 0xa4bf92de, SUC_lexicon_tags_11 },
    { 0xfd9f92df, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xaa1f92e2, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe3df92e4, SUC_lexicon_tags_63 },
    { 0xebdf92e5, SUC_lexicon_tags_0 },
    { 0x3d3f92e5, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7f92ed, SUC_lexicon_tags_15 },
    { 0x803f92ee, SUC_lexicon_tags_43 },
    { 0x4f3f92ef, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3cff92f1, SUC_lexicon_tags_14 },
    { 0x60ff92f2, SUC_lexicon_tags_7 },
    { 0x593f92f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x139f92f5, SUC_lexicon_tags_68 },
    { 0x259f92f6, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf93f92f9, SUC_lexicon_tags_19 },
    { 0xdbdf92fa, SUC_lexicon_tags_32 },
    { 0xfe5f92fa, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xc4df92fd, SUC_lexicon_tags_17 },
    { 0xf6ff92fe, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1f9301, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe29f9304, SUC_lexicon_tags_37 },
    { 0xe95f9305, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8c7f9307, SUC_lexicon_tags_14 },
    { 0xa9ff9308, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe59f930f, SUC_lexicon_tags_38 },
    { 0x7c1f930f, SUC_lexicon_tags_33 },
    { 0x5d7f9310, SUC_lexicon_tags_11 },
    { 0xaaff9312, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40bf9319, SUC_lexicon_tags_77 },
    { 0xb1ff931a, SUC_lexicon_tags_5 },
    { 0x22df931b, SUC_lexicon_tags_17 },
    { 0x1f5f931c, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe7ff931f, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9f9325, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc3ff9327, SUC_lexicon_tags_0 },
    { 0x3c7f9327, SUC_lexicon_tags_25 },
    { 0xb49f9329, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x403f9330, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x3a3f9332, SUC_lexicon_tags_1 },
    { 0x6fff9332, SUC_lexicon_tags_11 },
    { 0x23f9332, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xf4ff9336, SUC_lexicon_tags_25 },
    { 0x94ff9337, SUC_lexicon_tags_17 },
    { 0x125f9338, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9bf933a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x217f933c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3f933f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787f9343, SUC_lexicon_tags_2 },
    { 0x87df9343, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x39bf9346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5f9349, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbf9352, SUC_lexicon_tags_55 },
    { 0x815f9353, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9a5f9355, SUC_lexicon_tags_17 },
    { 0xdb9f9356, SUC_lexicon_tags_69 },
    { 0x6f5f9355, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5f935b, SUC_lexicon_tags_7 },
    { 0x95f935b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41f9360, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d1f9366, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503f936b, SUC_lexicon_tags_14 },
    { 0x405f936b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491f9370, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9f9376, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769f937b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7f9381, SUC_lexicon_tags_47 },
    { 0xaf3f9382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xef3f9384, SUC_lexicon_tags_49 },
    { 0xa81f9385, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5f938b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x86df938d, SUC_lexicon_tags_7 },
    { 0xbebf938e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x461f9392, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb33f9394, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5c1f9396, SUC_lexicon_tags_17 },
    { 0x453f9396, SUC_lexicon_tags_13 },
    { 0xcd7f9396, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf21f939a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7f939e, SUC_lexicon_tags_33 },
    { 0x4f7f939f, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3f93a4, SUC_lexicon_tags_6 },
    { 0x827f93a4, SUC_lexicon_tags_62 },
    { 0x9df93a6, SUC_lexicon_tags_107 },
    { 0xf1f93a7, SUC_lexicon_tags_61 },
    { 0x697f93a8, SUC_lexicon_tags_65 },
    { 0x5ddf93a9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabf93b1, SUC_lexicon_tags_2 },
    { 0xa9ff93b2, SUC_lexicon_tags_91 },
    { 0x453f93b3, SUC_lexicon_tags_12 },
    { 0x8fff93b4, SUC_lexicon_tags_14 },
    { 0x4b7f93b5, SUC_lexicon_tags_11 },
    { 0x8bff93b6, SUC_lexicon_tags_2 },
    { 0xf17f93b6, SUC_lexicon_tags_28 },
    { 0x285f93b8, SUC_lexicon_tags_17 },
    { 0x7a9f93b8, SUC_lexicon_tags_77 },
    { 0x371f93b7, SUC_lexicon_tags_68 },
    { 0xfe1f93b7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81f93bf, SUC_lexicon_tags_87 },
    { 0x9cbf93bf, SUC_lexicon_tags_14 },
    { 0x7bdf93c1, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0xb35f93c3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4df93c6, SUC_lexicon_tags_77 },
    { 0x803f93c7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x107f93cb, SUC_lexicon_tags_57 },
    { 0x4bff93cb, SUC_lexicon_tags_259 },
    { 0, NULL },
    { 0x467f93ce, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bf93d3, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965f93d7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbf93df, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd67f93e1, SUC_lexicon_tags_30 },
    { 0x9a3f93e2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc9bf93e4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaabf93e7, SUC_lexicon_tags_38 },
    { 0xdddf93e8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bf93ec, SUC_lexicon_tags_188 },
    { 0x1a3f93ed, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01f93f2, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdf93f7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x601f93f9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18df9404, SUC_lexicon_tags_25 },
    { 0x431f9405, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f7f9408, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9f940b, SUC_lexicon_tags_32 },
    { 0x3b1f940b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3f940f, SUC_lexicon_tags_11 },
    { 0x535f9410, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe73f9412, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x673f9414, SUC_lexicon_tags_25 },
    { 0x235f9415, SUC_lexicon_tags_4 },
    { 0xc93f9416, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbf9421, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfcbf9423, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x391f9425, SUC_lexicon_tags_33 },
    { 0x47bf9425, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe07f9428, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95f942b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x799f942e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ff9432, SUC_lexicon_tags_0 },
    { 0xc87f9433, SUC_lexicon_tags_14 },
    { 0x737f9434, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65df9438, SUC_lexicon_tags_1 },
    { 0xca1f9439, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xdff943b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5fff943d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xd33f943f, SUC_lexicon_tags_60 },
    { 0x6b5f943f, SUC_lexicon_tags_19 },
    { 0x41bf9441, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe7f9443, SUC_lexicon_tags_12 },
    { 0x163f9444, SUC_lexicon_tags_2 },
    { 0x549f9445, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x649f9447, SUC_lexicon_tags_138 },
    { 0x52ff9448, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x429f944a, SUC_lexicon_tags_8 },
    { 0x8bbf944b, SUC_lexicon_tags_30 },
    { 0xaaff944b, SUC_lexicon_tags_60 },
    { 0x329f944d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde3f944f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x86bf9452, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x6d5f9454, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe25f9458, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x98bf945a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x799f9460, SUC_lexicon_tags_37 },
    { 0xee3f9461, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xd51f9464, SUC_lexicon_tags_1 },
    { 0xcedf9465, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e7f9467, SUC_lexicon_tags_37 },
    { 0x375f9468, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03f946f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7f9472, SUC_lexicon_tags_38 },
    { 0x129f9473, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11f9481, SUC_lexicon_tags_37 },
    { 0xeabf9482, SUC_lexicon_tags_17 },
    { 0x5f9482, SUC_lexicon_tags_2 },
    { 0xf3df9482, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9f9488, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47f9493, SUC_lexicon_tags_11 },
    { 0xe1ff9494, SUC_lexicon_tags_182 },
    { 0x629f9493, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669f9499, SUC_lexicon_tags_15 },
    { 0xe3df949a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41f949f, SUC_lexicon_tags_23 },
    { 0xa99f94a0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5df94a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x879f94a6, SUC_lexicon_tags_25 },
    { 0xbd1f94a7, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebf94b4, SUC_lexicon_tags_32 },
    { 0x79ff94b4, SUC_lexicon_tags_38 },
    { 0x3c5f94b6, SUC_lexicon_tags_26 },
    { 0xca9f94b7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x831f94bb, SUC_lexicon_tags_25 },
    { 0x593f94bc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x709f94bf, SUC_lexicon_tags_15 },
    { 0x56bf94c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x761f94c2, SUC_lexicon_tags_11 },
    { 0xdbff94c3, SUC_lexicon_tags_17 },
    { 0xc09f94c4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763f94c8, SUC_lexicon_tags_26 },
    { 0x82df94c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x613f94cc, SUC_lexicon_tags_38 },
    { 0x6c5f94cd, SUC_lexicon_tags_75 },
    { 0x9c5f94cc, SUC_lexicon_tags_47 },
    { 0x86df94cf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657f94d4, SUC_lexicon_tags_33 },
    { 0x8a7f94d5, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77f94da, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xacff94dc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bf94e2, SUC_lexicon_tags_42 },
    { 0x669f94e3, SUC_lexicon_tags_15 },
    { 0x3ff94e2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6df94e7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x425f94e9, SUC_lexicon_tags_904 },
    { 0x3c1f94e9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x13ff94ec, SUC_lexicon_tags_14 },
    { 0xf9f94ec, SUC_lexicon_tags_37 },
    { 0xd6bf94ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x53df94f0, SUC_lexicon_tags_8 },
    { 0xb37f94f1, SUC_lexicon_tags_8 },
    { 0x27bf94f2, SUC_lexicon_tags_26 },
    { 0x137f94f3, SUC_lexicon_tags_87 },
    { 0xd83f94f4, SUC_lexicon_tags_17 },
    { 0xea5f94f5, SUC_lexicon_tags_47 },
    { 0xa21f94f6, SUC_lexicon_tags_7 },
    { 0xd27f94f6, SUC_lexicon_tags_17 },
    { 0x679f94f6, SUC_lexicon_tags_37 },
    { 0x201f94f9, SUC_lexicon_tags_7 },
    { 0xd07f94f9, SUC_lexicon_tags_0 },
    { 0x541f94f2, SUC_lexicon_tags_14 },
    { 0x3d3f94fc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ff9501, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c3f9504, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x189f9507, SUC_lexicon_tags_37 },
    { 0xa03f9508, SUC_lexicon_tags_68 },
    { 0x98ff9508, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x691f950c, SUC_lexicon_tags_45 },
    { 0x317f950d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ff9516, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15f9519, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x809f951b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5f9521, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1f9525, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x70ff9528, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2abf952a, SUC_lexicon_tags_17 },
    { 0x88df952a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc5f952d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bff9530, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed1f9538, SUC_lexicon_tags_32 },
    { 0xa93f9539, SUC_lexicon_tags_31 },
    { 0xe97f9538, SUC_lexicon_tags_15 },
    { 0x919f953a, SUC_lexicon_tags_11 },
    { 0x179f953c, SUC_lexicon_tags_12 },
    { 0xcb1f953c, SUC_lexicon_tags_37 },
    { 0x63ff953e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x567f9541, SUC_lexicon_tags_12 },
    { 0x5bdf9541, SUC_lexicon_tags_14 },
    { 0x557f9541, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7f3f9545, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb1f9548, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21f954b, SUC_lexicon_tags_17 },
    { 0xcff954c, SUC_lexicon_tags_121 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71f9552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x587f9556, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x225f9558, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f1f955b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285f9561, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15ff9565, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcb5f9567, SUC_lexicon_tags_79 },
    { 0xcbff9568, SUC_lexicon_tags_15 },
    { 0x80bf9569, SUC_lexicon_tags_25 },
    { 0x1f5f9568, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73f956e, SUC_lexicon_tags_38 },
    { 0x41df956e, SUC_lexicon_tags_12 },
    { 0xc4ff956f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7f9576, SUC_lexicon_tags_11 },
    { 0x62ff9577, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xb3df9579, SUC_lexicon_tags_25 },
    { 0x3f9f957a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x885f957d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815f9581, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0x555f9583, SUC_lexicon_tags_28 },
    { 0x197f9584, SUC_lexicon_tags_15 },
    { 0x469f9585, SUC_lexicon_tags_15 },
    { 0x9c7f9586, SUC_lexicon_tags_955 },
    { 0, NULL },
    { 0x9cff9588, SUC_lexicon_tags_47 },
    { 0x1f9589, SUC_lexicon_tags_5 },
    { 0x3b5f958a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1f958e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ff9593, SUC_lexicon_tags_4 },
    { 0xd47f9593, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x693f959a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x645f959e, SUC_lexicon_tags_45 },
    { 0x75f959f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b3f95a3, SUC_lexicon_tags_14 },
    { 0x4a7f95a3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ff95a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x895f95ac, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7f95af, SUC_lexicon_tags_17 },
    { 0xe9bf95af, SUC_lexicon_tags_14 },
    { 0xd05f95b1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ff95b4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7f95ba, SUC_lexicon_tags_37 },
    { 0x649f95bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x389f95c1, SUC_lexicon_tags_25 },
    { 0x41f95c1, SUC_lexicon_tags_12 },
    { 0xa23f95c3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7edf95c9, SUC_lexicon_tags_27 },
    { 0xa1f95ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x42bf95cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x143f95ce, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1f95d1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xffff95d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xef1f95d5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd03f95db, SUC_lexicon_tags_27 },
    { 0x5bf95dc, SUC_lexicon_tags_19 },
    { 0xc3ff95dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8df95e0, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0x519f95e2, SUC_lexicon_tags_25 },
    { 0x11df95e3, SUC_lexicon_tags_25 },
    { 0x237f95e3, SUC_lexicon_tags_31 },
    { 0xbc5f95e2, SUC_lexicon_tags_33 },
    { 0x5dbf95e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9f95eb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x61f95ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8aff95f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ff95fc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94df9603, SUC_lexicon_tags_14 },
    { 0x575f9604, SUC_lexicon_tags_31 },
    { 0xd8df9604, SUC_lexicon_tags_11 },
    { 0xbdff9606, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ebf960c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x47bf960e, SUC_lexicon_tags_37 },
    { 0xc27f960f, SUC_lexicon_tags_57 },
    { 0x845f9610, SUC_lexicon_tags_29 },
    { 0x883f9611, SUC_lexicon_tags_5 },
    { 0x857f9611, SUC_lexicon_tags_14 },
    { 0x107f9611, SUC_lexicon_tags_61 },
    { 0x60df9614, SUC_lexicon_tags_31 },
    { 0x281f9612, SUC_lexicon_tags_11 },
    { 0xeebf9612, SUC_lexicon_tags_12 },
    { 0x6ff9617, SUC_lexicon_tags_17 },
    { 0xe83f9612, SUC_lexicon_tags_43 },
    { 0x11bf9614, SUC_lexicon_tags_17 },
    { 0x9a3f961a, SUC_lexicon_tags_26 },
    { 0x745f961b, SUC_lexicon_tags_9 },
    { 0x249f961c, SUC_lexicon_tags_11 },
    { 0xa3df9611, SUC_lexicon_tags_2 },
    { 0x9a5f9612, SUC_lexicon_tags_66 },
    { 0x4b7f9616, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8bf9621, SUC_lexicon_tags_29 },
    { 0xcbbf9622, SUC_lexicon_tags_31 },
    { 0x3e7f9622, SUC_lexicon_tags_11 },
    { 0x8abf9624, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x745f9628, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda7f962c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xfabf962e, SUC_lexicon_tags_25 },
    { 0x70ff962e, SUC_lexicon_tags_1 },
    { 0x497f9630, SUC_lexicon_tags_12 },
    { 0xbb5f9631, SUC_lexicon_tags_17 },
    { 0x3f9f9632, SUC_lexicon_tags_8 },
    { 0x49f9633, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2adf9635, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71f963a, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59f9641, SUC_lexicon_tags_7 },
    { 0xabf9642, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x815f9647, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4f9f9649, SUC_lexicon_tags_17 },
    { 0x9ebf9649, SUC_lexicon_tags_15 },
    { 0xa2ff964a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbf9650, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f9653, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x41df9656, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ff965b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911f965f, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf47f9666, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1f966f, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ff9672, SUC_lexicon_tags_12 },
    { 0xf75f9672, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ff9677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xefbf9679, SUC_lexicon_tags_33 },
    { 0x6fff967a, SUC_lexicon_tags_3 },
    { 0x979f967b, SUC_lexicon_tags_60 },
    { 0x459f967c, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddf9684, SUC_lexicon_tags_15 },
    { 0xd43f9685, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x297f9688, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36ff9692, SUC_lexicon_tags_7 },
    { 0x941f9693, SUC_lexicon_tags_45 },
    { 0xa69f9693, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdf9699, SUC_lexicon_tags_33 },
    { 0x17bf969a, SUC_lexicon_tags_2 },
    { 0xf59f969b, SUC_lexicon_tags_0 },
    { 0xbddf969c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1f969f, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7f96a2, SUC_lexicon_tags_45 },
    { 0x8f3f96a3, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bf96a7, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21f96af, SUC_lexicon_tags_27 },
    { 0x45df96b0, SUC_lexicon_tags_17 },
    { 0x6bff96b1, SUC_lexicon_tags_17 },
    { 0x5dff96b0, SUC_lexicon_tags_11 },
    { 0x935f96b2, SUC_lexicon_tags_23 },
    { 0x1aff96b3, SUC_lexicon_tags_45 },
    { 0x8e5f96b5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bf96bc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc6df96be, SUC_lexicon_tags_15 },
    { 0x877f96bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d1f96c9, SUC_lexicon_tags_68 },
    { 0x83df96ca, SUC_lexicon_tags_7 },
    { 0x3fff96cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95f96d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357f96dc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x975f96de, SUC_lexicon_tags_25 },
    { 0xd1df96df, SUC_lexicon_tags_2 },
    { 0xd61f96df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295f96e4, SUC_lexicon_tags_15 },
    { 0x45f96e4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9dbf96e7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x27bf96e9, SUC_lexicon_tags_12 },
    { 0xc6df96e9, SUC_lexicon_tags_37 },
    { 0x53bf96eb, SUC_lexicon_tags_496 },
    { 0x343f96eb, SUC_lexicon_tags_27 },
    { 0x6f1f96ed, SUC_lexicon_tags_33 },
    { 0x5f5f96ee, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe53f96f0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x99df96f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x557f96f5, SUC_lexicon_tags_12 },
    { 0xd4ff96f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xdadf96f9, SUC_lexicon_tags_25 },
    { 0xe47f96f9, SUC_lexicon_tags_28 },
    { 0xb0ff96fb, SUC_lexicon_tags_29 },
    { 0x3fdf96fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabf9700, SUC_lexicon_tags_7 },
    { 0x31df9701, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abf9704, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5df9706, SUC_lexicon_tags_21 },
    { 0x6d5f9707, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x6c9f9709, SUC_lexicon_tags_33 },
    { 0xd45f970a, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x31f970d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bf9710, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91f9719, SUC_lexicon_tags_60 },
    { 0x8a9f971a, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bf971e, SUC_lexicon_tags_11 },
    { 0xdebf971f, SUC_lexicon_tags_7 },
    { 0xee7f9720, SUC_lexicon_tags_31 },
    { 0xbcbf9721, SUC_lexicon_tags_45 },
    { 0x765f9722, SUC_lexicon_tags_9 },
    { 0x81bf9723, SUC_lexicon_tags_25 },
    { 0x185f9724, SUC_lexicon_tags_19 },
    { 0xd51f9725, SUC_lexicon_tags_0 },
    { 0xda3f971e, SUC_lexicon_tags_8 },
    { 0xcabf9727, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x333f9729, SUC_lexicon_tags_34 },
    { 0x17f972a, SUC_lexicon_tags_14 },
    { 0x119f9729, SUC_lexicon_tags_37 },
    { 0x73df9729, SUC_lexicon_tags_37 },
    { 0x557f972d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x725f972f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x79df9732, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4b5f9734, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdc1f9736, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc15f973a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5f973d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ff9747, SUC_lexicon_tags_6 },
    { 0x905f9748, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493f974e, SUC_lexicon_tags_11 },
    { 0x5adf974f, SUC_lexicon_tags_17 },
    { 0xeb3f974f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x141f9752, SUC_lexicon_tags_48 },
    { 0x8dbf9753, SUC_lexicon_tags_11 },
    { 0xaeff9754, SUC_lexicon_tags_27 },
    { 0xd43f9755, SUC_lexicon_tags_11 },
    { 0xdc5f9754, SUC_lexicon_tags_23 },
    { 0x6aff9757, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee1f975c, SUC_lexicon_tags_39 },
    { 0x75f975d, SUC_lexicon_tags_15 },
    { 0x9df975d, SUC_lexicon_tags_2 },
    { 0x791f975c, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0x16df9761, SUC_lexicon_tags_2 },
    { 0xa9bf9761, SUC_lexicon_tags_4 },
    { 0x611f9763, SUC_lexicon_tags_62 },
    { 0xacdf9763, SUC_lexicon_tags_11 },
    { 0xf87f9763, SUC_lexicon_tags_27 },
    { 0xff3f9764, SUC_lexicon_tags_17 },
    { 0x359f9761, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x32bf9769, SUC_lexicon_tags_9 },
    { 0xd1bf9769, SUC_lexicon_tags_45 },
    { 0x2fdf976b, SUC_lexicon_tags_2 },
    { 0x95ff976c, SUC_lexicon_tags_19 },
    { 0x1bff976d, SUC_lexicon_tags_68 },
    { 0x497f976e, SUC_lexicon_tags_7 },
    { 0xc9bf976c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9772, SUC_lexicon_tags_29 },
    { 0x62ff9772, SUC_lexicon_tags_1 },
    { 0xc35f9772, SUC_lexicon_tags_1 },
    { 0xa07f9775, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3f977a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4bf9784, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x739f9787, SUC_lexicon_tags_2 },
    { 0x245f9787, SUC_lexicon_tags_21 },
    { 0x57bf9789, SUC_lexicon_tags_7 },
    { 0x9b7f9789, SUC_lexicon_tags_7 },
    { 0x393f978b, SUC_lexicon_tags_11 },
    { 0xbd1f978c, SUC_lexicon_tags_7 },
    { 0xa75f978c, SUC_lexicon_tags_65 },
    { 0x48ff978e, SUC_lexicon_tags_25 },
    { 0x851f978f, SUC_lexicon_tags_11 },
    { 0x97bf978f, SUC_lexicon_tags_15 },
    { 0xa83f9790, SUC_lexicon_tags_15 },
    { 0x2edf978f, SUC_lexicon_tags_15 },
    { 0xeadf9793, SUC_lexicon_tags_82 },
    { 0x671f978c, SUC_lexicon_tags_30 },
    { 0xc4bf9787, SUC_lexicon_tags_23 },
    { 0x61df9796, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x55ff9798, SUC_lexicon_tags_11 },
    { 0x43f9798, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x609f979d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40df97a1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x50df97a3, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdf97a8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bf97ab, SUC_lexicon_tags_17 },
    { 0x9c3f97ab, SUC_lexicon_tags_11 },
    { 0x96bf97ab, SUC_lexicon_tags_2 },
    { 0x62df97ae, SUC_lexicon_tags_37 },
    { 0x7c9f97af, SUC_lexicon_tags_47 },
    { 0xd23f97b0, SUC_lexicon_tags_7 },
    { 0x2bf97b1, SUC_lexicon_tags_14 },
    { 0xce7f97b0, SUC_lexicon_tags_25 },
    { 0x6b7f97b3, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ff97b9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9df97c0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb5f97c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bf97c5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdf97c8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1f97cb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f97ce, SUC_lexicon_tags_37 },
    { 0x29f97ce, SUC_lexicon_tags_23 },
    { 0xa5df97ce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ff97d3, SUC_lexicon_tags_47 },
    { 0x4e5f97d4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x813f97d7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bf97dc, SUC_lexicon_tags_6 },
    { 0x28df97dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaff97e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x241f97e3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf13f97e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe49f97e7, SUC_lexicon_tags_254 },
    { 0x4b7f97e8, SUC_lexicon_tags_12 },
    { 0x409f97e8, SUC_lexicon_tags_259 },
    { 0x5dff97ea, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ff97ed, SUC_lexicon_tags_25 },
    { 0xce1f97ed, SUC_lexicon_tags_5 },
    { 0x2a1f97ef, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ff97f2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bf97f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xee5f97fc, SUC_lexicon_tags_37 },
    { 0xbd1f97fd, SUC_lexicon_tags_11 },
    { 0x147f97fe, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xedff9800, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ff9804, SUC_lexicon_tags_12 },
    { 0x231f9805, SUC_lexicon_tags_12 },
    { 0x841f9806, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdf980f, SUC_lexicon_tags_8 },
    { 0x809f980f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7f9814, SUC_lexicon_tags_38 },
    { 0x8f1f9815, SUC_lexicon_tags_48 },
    { 0xac3f9815, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x78df9818, SUC_lexicon_tags_37 },
    { 0xdc5f9819, SUC_lexicon_tags_84 },
    { 0x1a1f981a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87f981d, SUC_lexicon_tags_26 },
    { 0xa8ff981e, SUC_lexicon_tags_12 },
    { 0x337f981f, SUC_lexicon_tags_11 },
    { 0x2fbf9820, SUC_lexicon_tags_49 },
    { 0x4eff9821, SUC_lexicon_tags_63 },
    { 0xa17f9822, SUC_lexicon_tags_37 },
    { 0x9d7f9820, SUC_lexicon_tags_39 },
    { 0x263f9823, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd71f9828, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa85f982b, SUC_lexicon_tags_17 },
    { 0x51bf982b, SUC_lexicon_tags_45 },
    { 0x1a7f982b, SUC_lexicon_tags_7 },
    { 0x497f982e, SUC_lexicon_tags_15 },
    { 0x30df982d, SUC_lexicon_tags_29 },
    { 0xc99f9830, SUC_lexicon_tags_25 },
    { 0x50df9831, SUC_lexicon_tags_31 },
    { 0x55ff9832, SUC_lexicon_tags_7 },
    { 0x2a5f982b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x199f9835, SUC_lexicon_tags_37 },
    { 0xdb9f9836, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ff983b, SUC_lexicon_tags_2 },
    { 0x10bf983b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea1f9844, SUC_lexicon_tags_38 },
    { 0xc9ff9845, SUC_lexicon_tags_6 },
    { 0x5c5f9846, SUC_lexicon_tags_571 },
    { 0x20df9845, SUC_lexicon_tags_12 },
    { 0x34df9846, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9f9851, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23bf9855, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf25f9859, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x933f985b, SUC_lexicon_tags_37 },
    { 0xaadf985b, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x741f985e, SUC_lexicon_tags_14 },
    { 0xbb3f985f, SUC_lexicon_tags_37 },
    { 0xe95f985e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bf9864, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb37f9869, SUC_lexicon_tags_1 },
    { 0x8e7f9869, SUC_lexicon_tags_37 },
    { 0x291f986a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcabf9870, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x45df9872, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x661f9877, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa55f9879, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xff9f987b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x729f987d, SUC_lexicon_tags_26 },
    { 0x64bf987e, SUC_lexicon_tags_11 },
    { 0xa19f987f, SUC_lexicon_tags_25 },
    { 0x37bf9880, SUC_lexicon_tags_7 },
    { 0xe3bf987f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe45f9883, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x10df9885, SUC_lexicon_tags_3 },
    { 0xafff9886, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3f988f, SUC_lexicon_tags_27 },
    { 0x82ff9890, SUC_lexicon_tags_48 },
    { 0x6e5f9891, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x991f9893, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x365f9895, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x24bf9897, SUC_lexicon_tags_12 },
    { 0xab9f9898, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17f989e, SUC_lexicon_tags_11 },
    { 0xb79f989f, SUC_lexicon_tags_38 },
    { 0xfa7f989f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xf5f98a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd41f98a4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfddf98a7, SUC_lexicon_tags_17 },
    { 0xf93f98a8, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff7f98ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc25f98af, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x267f98b1, SUC_lexicon_tags_0 },
    { 0x1fff98b1, SUC_lexicon_tags_1 },
    { 0x4cff98b2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7cdf98b5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x95f98b8, SUC_lexicon_tags_7 },
    { 0x579f98b9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xa83f98bb, SUC_lexicon_tags_65 },
    { 0x199f98bc, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x137f98c0, SUC_lexicon_tags_7 },
    { 0xf5df98c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05f98c5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x895f98c8, SUC_lexicon_tags_49 },
    { 0x8a1f98c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x461f98cb, SUC_lexicon_tags_15 },
    { 0x567f98cb, SUC_lexicon_tags_37 },
    { 0xf0ff98cd, SUC_lexicon_tags_17 },
    { 0x95bf98ce, SUC_lexicon_tags_17 },
    { 0x4cbf98ce, SUC_lexicon_tags_68 },
    { 0x293f98d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adf98db, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59f98e2, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x2ebf98e4, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7df98eb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227f98ef, SUC_lexicon_tags_38 },
    { 0xaf1f98f0, SUC_lexicon_tags_11 },
    { 0x3d9f98f1, SUC_lexicon_tags_0 },
    { 0xdd3f98f2, SUC_lexicon_tags_15 },
    { 0x471f98f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63f98f8, SUC_lexicon_tags_25 },
    { 0xa51f98f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa51f98fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6dff98fd, SUC_lexicon_tags_37 },
    { 0xf75f98fd, SUC_lexicon_tags_47 },
    { 0xc57f98fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xda1f9901, SUC_lexicon_tags_37 },
    { 0x2c7f9902, SUC_lexicon_tags_0 },
    { 0x2df9903, SUC_lexicon_tags_16 },
    { 0xb4df9901, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x43bf9907, SUC_lexicon_tags_25 },
    { 0xfe5f9908, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1f9912, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xeeff9914, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x2d9f9916, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x5d5f9918, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bf991c, SUC_lexicon_tags_26 },
    { 0xc35f991d, SUC_lexicon_tags_17 },
    { 0xaf3f991e, SUC_lexicon_tags_17 },
    { 0xf1bf991f, SUC_lexicon_tags_28 },
    { 0xdddf991e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2abf9922, SUC_lexicon_tags_49 },
    { 0x379f9923, SUC_lexicon_tags_12 },
    { 0x7bf9922, SUC_lexicon_tags_11 },
    { 0x393f9922, SUC_lexicon_tags_17 },
    { 0x339f9922, SUC_lexicon_tags_26 },
    { 0x919f9927, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb6ff9929, SUC_lexicon_tags_55 },
    { 0xda3f992a, SUC_lexicon_tags_57 },
    { 0x489f992b, SUC_lexicon_tags_16 },
    { 0x95f992b, SUC_lexicon_tags_15 },
    { 0x3bbf992b, SUC_lexicon_tags_48 },
    { 0x769f992e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf9ff9930, SUC_lexicon_tags_1 },
    { 0xea3f9931, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37df9935, SUC_lexicon_tags_1 },
    { 0xd6ff9936, SUC_lexicon_tags_11 },
    { 0x3f1f9935, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x31ff9939, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x613f993b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x885f993f, SUC_lexicon_tags_25 },
    { 0xdb3f9940, SUC_lexicon_tags_87 },
    { 0x9a9f9941, SUC_lexicon_tags_11 },
    { 0x281f9942, SUC_lexicon_tags_25 },
    { 0x5f9f9943, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1f9946, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27f994a, SUC_lexicon_tags_30 },
    { 0xe11f994a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823f9952, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5f9956, SUC_lexicon_tags_11 },
    { 0x499f9957, SUC_lexicon_tags_15 },
    { 0xc53f9958, SUC_lexicon_tags_17 },
    { 0x991f9958, SUC_lexicon_tags_55 },
    { 0x5df995a, SUC_lexicon_tags_17 },
    { 0xd3f9958, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5e9f995d, SUC_lexicon_tags_11 },
    { 0xcb3f995e, SUC_lexicon_tags_8 },
    { 0x36bf995e, SUC_lexicon_tags_37 },
    { 0x353f9960, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd75f9962, SUC_lexicon_tags_14 },
    { 0x167f9963, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x613f9965, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99f996f, SUC_lexicon_tags_38 },
    { 0x71df996f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x543f9973, SUC_lexicon_tags_7 },
    { 0x89ff9974, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x893f9976, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x10bf9978, SUC_lexicon_tags_1 },
    { 0x41f9978, SUC_lexicon_tags_15 },
    { 0x49ff997a, SUC_lexicon_tags_2 },
    { 0x7ddf9979, SUC_lexicon_tags_37 },
    { 0xabdf997c, SUC_lexicon_tags_31 },
    { 0x8d5f997d, SUC_lexicon_tags_38 },
    { 0x4d1f997e, SUC_lexicon_tags_23 },
    { 0x125f997f, SUC_lexicon_tags_25 },
    { 0xa4ff997f, SUC_lexicon_tags_33 },
    { 0xe3ff997d, SUC_lexicon_tags_48 },
    { 0xff3f997a, SUC_lexicon_tags_47 },
    { 0x7fbf9983, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xb77f9986, SUC_lexicon_tags_23 },
    { 0xd2ff9987, SUC_lexicon_tags_15 },
    { 0xc89f9987, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321f998f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd69f9991, SUC_lexicon_tags_19 },
    { 0xfcdf9992, SUC_lexicon_tags_1 },
    { 0xc4bf9992, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x625f9996, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1f999a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x495f999f, SUC_lexicon_tags_3 },
    { 0xb7f99a0, SUC_lexicon_tags_6 },
    { 0xefff99a0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07f99a5, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed3f99ae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x537f99b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343f99b8, SUC_lexicon_tags_37 },
    { 0x16ff99b8, SUC_lexicon_tags_11 },
    { 0x37bf99b9, SUC_lexicon_tags_15 },
    { 0x6cff99bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3f99c2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8ff99c5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x185f99c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaff99d3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5f99d6, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x69ff99d8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237f99dc, SUC_lexicon_tags_37 },
    { 0x845f99dd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673f99e1, SUC_lexicon_tags_37 },
    { 0x7a5f99e1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b3f99e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1e1f99e8, SUC_lexicon_tags_9 },
    { 0x82ff99e8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x377f99ec, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x6cff99ee, SUC_lexicon_tags_39 },
    { 0x791f99ef, SUC_lexicon_tags_37 },
    { 0x497f99f0, SUC_lexicon_tags_11 },
    { 0xdf7f99f1, SUC_lexicon_tags_20 },
    { 0x43ff99f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f1f99f5, SUC_lexicon_tags_32 },
    { 0x2cff99f6, SUC_lexicon_tags_17 },
    { 0xd11f99f7, SUC_lexicon_tags_25 },
    { 0x3c9f99f8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccff99fc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xe83f99fe, SUC_lexicon_tags_6 },
    { 0x90bf99ff, SUC_lexicon_tags_2 },
    { 0xdbbf99fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9df9a05, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x387f9a08, SUC_lexicon_tags_27 },
    { 0x947f9a09, SUC_lexicon_tags_11 },
    { 0xe0df9a09, SUC_lexicon_tags_49 },
    { 0xfb5f9a0b, SUC_lexicon_tags_60 },
    { 0x4e7f9a0b, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d1f9a0f, SUC_lexicon_tags_7 },
    { 0x43df9a10, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7f9a14, SUC_lexicon_tags_1 },
    { 0x92df9a15, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdc5f9a17, SUC_lexicon_tags_197 },
    { 0, NULL },
    { 0x31ff9a19, SUC_lexicon_tags_46 },
    { 0xa73f9a19, SUC_lexicon_tags_99 },
    { 0x863f9a19, SUC_lexicon_tags_37 },
    { 0x115f9a1c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x275f9a1f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce1f9a21, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x165f9a24, SUC_lexicon_tags_10 },
    { 0x17ff9a24, SUC_lexicon_tags_16 },
    { 0x5bdf9a26, SUC_lexicon_tags_37 },
    { 0x28ff9a27, SUC_lexicon_tags_17 },
    { 0xb95f9a24, SUC_lexicon_tags_67 },
    { 0x389f9a29, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd0df9a2c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9f9a2f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xccdf9a31, SUC_lexicon_tags_87 },
    { 0xd89f9a31, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ff9a37, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91df9a3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43f9a43, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x66df9a45, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545f9a4e, SUC_lexicon_tags_54 },
    { 0x8c1f9a4f, SUC_lexicon_tags_45 },
    { 0xec9f9a4e, SUC_lexicon_tags_14 },
    { 0x49df9a50, SUC_lexicon_tags_46 },
    { 0xfd9f9a52, SUC_lexicon_tags_26 },
    { 0x377f9a52, SUC_lexicon_tags_11 },
    { 0xa29f9a4f, SUC_lexicon_tags_43 },
    { 0xc5ff9a55, SUC_lexicon_tags_25 },
    { 0xa2bf9a4e, SUC_lexicon_tags_0 },
    { 0xd1bf9a51, SUC_lexicon_tags_40 },
    { 0x255f9a58, SUC_lexicon_tags_38 },
    { 0x8c3f9a54, SUC_lexicon_tags_15 },
    { 0x461f9a5a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xcdbf9a5c, SUC_lexicon_tags_25 },
    { 0x6e9f9a5d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x995f9a63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35f9a67, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e1f9a6c, SUC_lexicon_tags_37 },
    { 0x401f9a6d, SUC_lexicon_tags_12 },
    { 0x509f9a6e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x683f9a71, SUC_lexicon_tags_11 },
    { 0xec1f9a72, SUC_lexicon_tags_2 },
    { 0xebdf9a71, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xf9f9a76, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139f9a7a, SUC_lexicon_tags_2 },
    { 0x163f9a7b, SUC_lexicon_tags_34 },
    { 0x96df9a7c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x977f9a7f, SUC_lexicon_tags_4 },
    { 0xc73f9a80, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0x333f9a82, SUC_lexicon_tags_1 },
    { 0x7cdf9a82, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1f9a86, SUC_lexicon_tags_158 },
    { 0xa29f9a87, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43f9a8b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x6b3f9a8d, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0xccdf9a90, SUC_lexicon_tags_25 },
    { 0x8adf9a90, SUC_lexicon_tags_15 },
    { 0xd1df9a92, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55bf9a9b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x985f9a9f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebff9aa3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x535f9aa6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x591f9aa8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05f9aae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x845f9ab5, SUC_lexicon_tags_1 },
    { 0xe63f9ab6, SUC_lexicon_tags_27 },
    { 0xfbff9ab7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49f9aba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x337f9abc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9f9ac0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x55df9ac3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf31f9ac5, SUC_lexicon_tags_49 },
    { 0x467f9ac6, SUC_lexicon_tags_17 },
    { 0xe81f9ac7, SUC_lexicon_tags_13 },
    { 0x703f9ac5, SUC_lexicon_tags_23 },
    { 0xc27f9ac9, SUC_lexicon_tags_28 },
    { 0xc21f9aca, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3f9ad1, SUC_lexicon_tags_46 },
    { 0xf59f9ad2, SUC_lexicon_tags_16 },
    { 0xbf7f9ad2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x131f9ad6, SUC_lexicon_tags_7 },
    { 0xc33f9ad6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1f9adc, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9df9ae2, SUC_lexicon_tags_33 },
    { 0xfd1f9ae3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167f9aec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45f9af0, SUC_lexicon_tags_17 },
    { 0x8d5f9af1, SUC_lexicon_tags_37 },
    { 0x811f9af2, SUC_lexicon_tags_53 },
    { 0xc0df9af0, SUC_lexicon_tags_17 },
    { 0x97bf9af1, SUC_lexicon_tags_25 },
    { 0xe4bf9af5, SUC_lexicon_tags_1 },
    { 0x2c5f9af6, SUC_lexicon_tags_0 },
    { 0xb3df9af7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe83f9af9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x465f9afe, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xb53f9b01, SUC_lexicon_tags_26 },
    { 0x9f3f9b02, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x307f9b04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbf9b09, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe77f9b0e, SUC_lexicon_tags_25 },
    { 0xd0df9b0f, SUC_lexicon_tags_14 },
    { 0xad5f9b10, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57f9b13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfc9f9b15, SUC_lexicon_tags_60 },
    { 0x1e7f9b16, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x103f9b19, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd51f9b1b, SUC_lexicon_tags_13 },
    { 0x71bf9b1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x619f9b1e, SUC_lexicon_tags_12 },
    { 0x331f9b1f, SUC_lexicon_tags_14 },
    { 0x3f5f9b20, SUC_lexicon_tags_37 },
    { 0x67df9b20, SUC_lexicon_tags_85 },
    { 0x8e7f9b21, SUC_lexicon_tags_7 },
    { 0xf85f9b23, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5f9b27, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9f9b2c, SUC_lexicon_tags_11 },
    { 0xd37f9b2c, SUC_lexicon_tags_9 },
    { 0xb21f9b2e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x415f9b31, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xbeff9b33, SUC_lexicon_tags_23 },
    { 0xb89f9b33, SUC_lexicon_tags_47 },
    { 0xb43f9b35, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851f9b3c, SUC_lexicon_tags_11 },
    { 0x38bf9b3d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x635f9b40, SUC_lexicon_tags_10 },
    { 0x595f9b41, SUC_lexicon_tags_15 },
    { 0xb31f9b42, SUC_lexicon_tags_25 },
    { 0xbfdf9b43, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddf9b47, SUC_lexicon_tags_87 },
    { 0x311f9b47, SUC_lexicon_tags_11 },
    { 0x9c1f9b47, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3f9b4c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa1ff9b4e, SUC_lexicon_tags_12 },
    { 0x5b3f9b4f, SUC_lexicon_tags_124 },
    { 0xe7df9b50, SUC_lexicon_tags_1 },
    { 0x357f9b51, SUC_lexicon_tags_7 },
    { 0xef7f9b52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7f9b56, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7f9b64, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x72bf9b67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf35f9b6a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x463f9b6d, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7f9b74, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3f9b78, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x385f9b7b, SUC_lexicon_tags_15 },
    { 0xc99f9b7c, SUC_lexicon_tags_1 },
    { 0x28df9b7d, SUC_lexicon_tags_23 },
    { 0x61bf9b7b, SUC_lexicon_tags_11 },
    { 0xdb1f9b7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x275f9b81, SUC_lexicon_tags_66 },
    { 0x7c1f9b81, SUC_lexicon_tags_26 },
    { 0x5bbf9b83, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90df9b87, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125f9b8f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3f9b93, SUC_lexicon_tags_6 },
    { 0x985f9b94, SUC_lexicon_tags_68 },
    { 0x8ff9b95, SUC_lexicon_tags_0 },
    { 0x775f9b94, SUC_lexicon_tags_15 },
    { 0xb77f9b97, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75f9b9b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf95f9b9f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24df9ba3, SUC_lexicon_tags_11 },
    { 0xed9f9ba4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb2bf9ba6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24bf9baa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7f9baf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x869f9bb4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x869f9bb6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0df9bbc, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbf9bc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc01f9bc6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddf9bca, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xacff9bcd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x82bf9bcf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf57f9bd1, SUC_lexicon_tags_47 },
    { 0x697f9bd2, SUC_lexicon_tags_0 },
    { 0x3a7f9bd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xbaff9bd5, SUC_lexicon_tags_15 },
    { 0x461f9bd6, SUC_lexicon_tags_21 },
    { 0xbb1f9bd5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85f9bdc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x173f9bde, SUC_lexicon_tags_8 },
    { 0x179f9bdf, SUC_lexicon_tags_16 },
    { 0x5fbf9bde, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x437f9be3, SUC_lexicon_tags_17 },
    { 0x29f9be4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf47f9be6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bf9be9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5f9bf1, SUC_lexicon_tags_37 },
    { 0xfb1f9bf2, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27f9bf6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb37f9bf9, SUC_lexicon_tags_35 },
    { 0xe09f9bf9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295f9bff, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x42bf9c01, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x65bf9c04, SUC_lexicon_tags_0 },
    { 0xe0bf9c05, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x671f9c07, SUC_lexicon_tags_17 },
    { 0xc13f9c08, SUC_lexicon_tags_1 },
    { 0xb07f9c09, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x159f9c0b, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0xb05f9c0d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd47f9c14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc1ff9c16, SUC_lexicon_tags_29 },
    { 0x31f9c16, SUC_lexicon_tags_15 },
    { 0xe85f9c18, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x989f9c1a, SUC_lexicon_tags_11 },
    { 0x6ff9c1a, SUC_lexicon_tags_30 },
    { 0x3e7f9c1a, SUC_lexicon_tags_28 },
    { 0x6e5f9c1d, SUC_lexicon_tags_58 },
    { 0xcd7f9c1d, SUC_lexicon_tags_25 },
    { 0x6edf9c1f, SUC_lexicon_tags_15 },
    { 0xa39f9c20, SUC_lexicon_tags_1 },
    { 0x443f9c1b, SUC_lexicon_tags_47 },
    { 0x91ff9c22, SUC_lexicon_tags_17 },
    { 0xc51f9c1a, SUC_lexicon_tags_87 },
    { 0xb59f9c24, SUC_lexicon_tags_11 },
    { 0xe8bf9c25, SUC_lexicon_tags_40 },
    { 0xa5df9c24, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797f9c2c, SUC_lexicon_tags_7 },
    { 0x693f9c2d, SUC_lexicon_tags_11 },
    { 0xd57f9c2e, SUC_lexicon_tags_44 },
    { 0xcf5f9c2f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9cdf9c31, SUC_lexicon_tags_33 },
    { 0x745f9c32, SUC_lexicon_tags_37 },
    { 0x9c1f9c32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bf9c38, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29f9c41, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2a1f9c43, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x405f9c45, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x131f9c47, SUC_lexicon_tags_60 },
    { 0xfe1f9c47, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23ff9c4a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6bf9c4d, SUC_lexicon_tags_15 },
    { 0xf91f9c4d, SUC_lexicon_tags_79 },
    { 0x511f9c4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4ebf9c51, SUC_lexicon_tags_9 },
    { 0xf49f9c52, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x681f9c5a, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0x957f9c5d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xfeff9c60, SUC_lexicon_tags_12 },
    { 0x43df9c61, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69f9c66, SUC_lexicon_tags_46 },
    { 0xb03f9c67, SUC_lexicon_tags_17 },
    { 0xd8df9c68, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7f9c6e, SUC_lexicon_tags_11 },
    { 0x8bf9c6f, SUC_lexicon_tags_2 },
    { 0x29f9c70, SUC_lexicon_tags_43 },
    { 0xcedf9c71, SUC_lexicon_tags_47 },
    { 0xc1df9c71, SUC_lexicon_tags_29 },
    { 0xaabf9c72, SUC_lexicon_tags_19 },
    { 0x311f9c6f, SUC_lexicon_tags_11 },
    { 0x687f9c71, SUC_lexicon_tags_14 },
    { 0x2cdf9c70, SUC_lexicon_tags_11 },
    { 0x815f9c77, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x947f9c79, SUC_lexicon_tags_37 },
    { 0xad3f9c79, SUC_lexicon_tags_25 },
    { 0x3aff9c7b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x419f9c81, SUC_lexicon_tags_26 },
    { 0x23ff9c82, SUC_lexicon_tags_253 },
    { 0x179f9c83, SUC_lexicon_tags_6 },
    { 0x5ddf9c84, SUC_lexicon_tags_92 },
    { 0x153f9c85, SUC_lexicon_tags_636 },
    { 0x5a7f9c86, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x675f9c88, SUC_lexicon_tags_37 },
    { 0x5c3f9c88, SUC_lexicon_tags_49 },
    { 0x78ff9c8a, SUC_lexicon_tags_9 },
    { 0x737f9c8a, SUC_lexicon_tags_17 },
    { 0x641f9c8c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe8ff9c8e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ff9c94, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3ff9c99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x11df9c9b, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cff9ca0, SUC_lexicon_tags_57 },
    { 0x6d5f9ca1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85f9ca9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3f7f9cab, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x8ff9cae, SUC_lexicon_tags_61 },
    { 0xd23f9cae, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x40ff9cb4, SUC_lexicon_tags_16 },
    { 0xfa3f9cb4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2ebf9cb7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bf9cc0, SUC_lexicon_tags_87 },
    { 0x99ff9cc0, SUC_lexicon_tags_7 },
    { 0xf2df9cc2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x96bf9cc4, SUC_lexicon_tags_25 },
    { 0x10bf9cc5, SUC_lexicon_tags_56 },
    { 0x6e1f9cc6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287f9cca, SUC_lexicon_tags_61 },
    { 0xf75f9ccb, SUC_lexicon_tags_37 },
    { 0x83df9cca, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebf9cd3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ff9cd6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731f9cda, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6d9f9cdc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xdb5f9cde, SUC_lexicon_tags_25 },
    { 0xf1bf9cdf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa8bf9ce1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x635f9ce4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bf9ce7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x505f9cec, SUC_lexicon_tags_170 },
    { 0xc71f9cec, SUC_lexicon_tags_25 },
    { 0xc27f9cee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa5df9cf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x83f9cf3, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43f9cf7, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41f9cfc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec1f9d00, SUC_lexicon_tags_12 },
    { 0x337f9d01, SUC_lexicon_tags_2 },
    { 0xecbf9d01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x28df9d04, SUC_lexicon_tags_46 },
    { 0x82ff9d04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2b3f9d07, SUC_lexicon_tags_29 },
    { 0xf57f9d08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7f9d0e, SUC_lexicon_tags_15 },
    { 0xd09f9d0e, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x7adf9d11, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbf9d14, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x2fdf9d16, SUC_lexicon_tags_12 },
    { 0x659f9d17, SUC_lexicon_tags_17 },
    { 0xd5df9d18, SUC_lexicon_tags_39 },
    { 0x61ff9d17, SUC_lexicon_tags_8 },
    { 0xd43f9d19, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x129f9d1c, SUC_lexicon_tags_2 },
    { 0x133f9d1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4df9d22, SUC_lexicon_tags_37 },
    { 0x203f9d23, SUC_lexicon_tags_99 },
    { 0, NULL },
    { 0xaf7f9d25, SUC_lexicon_tags_15 },
    { 0x4d3f9d25, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x453f9d29, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5f9d31, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x861f9d33, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x45df9d35, SUC_lexicon_tags_12 },
    { 0xfadf9d36, SUC_lexicon_tags_19 },
    { 0x511f9d37, SUC_lexicon_tags_1 },
    { 0x9bff9d36, SUC_lexicon_tags_8 },
    { 0x5e7f9d38, SUC_lexicon_tags_29 },
    { 0x565f9d39, SUC_lexicon_tags_9 },
    { 0xecdf9d3b, SUC_lexicon_tags_37 },
    { 0x147f9d3c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfff9d42, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddf9d46, SUC_lexicon_tags_33 },
    { 0x4cbf9d47, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887f9d4c, SUC_lexicon_tags_0 },
    { 0x429f9d4d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d5f9d52, SUC_lexicon_tags_37 },
    { 0x4e1f9d53, SUC_lexicon_tags_1 },
    { 0xe61f9d52, SUC_lexicon_tags_17 },
    { 0x4f3f9d55, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17f9d5e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddf9d64, SUC_lexicon_tags_30 },
    { 0x96bf9d65, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x21df9d68, SUC_lexicon_tags_2 },
    { 0x3fbf9d68, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe33f9d6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1bf9d6f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5f9d72, SUC_lexicon_tags_7 },
    { 0x5d1f9d72, SUC_lexicon_tags_46 },
    { 0xc6df9d74, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4edf9d7d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xec3f9d7f, SUC_lexicon_tags_19 },
    { 0x3edf9d80, SUC_lexicon_tags_11 },
    { 0x3b1f9d81, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf29f9d86, SUC_lexicon_tags_27 },
    { 0xc7df9d87, SUC_lexicon_tags_39 },
    { 0x565f9d86, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3f9d8c, SUC_lexicon_tags_37 },
    { 0xa6bf9d8c, SUC_lexicon_tags_23 },
    { 0x6b3f9d8c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa7f9d90, SUC_lexicon_tags_25 },
    { 0x4abf9d91, SUC_lexicon_tags_17 },
    { 0x7fbf9d90, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1f9d98, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x699f9d9a, SUC_lexicon_tags_25 },
    { 0x27f9d9b, SUC_lexicon_tags_38 },
    { 0x7bdf9d9c, SUC_lexicon_tags_2 },
    { 0x531f9d9c, SUC_lexicon_tags_11 },
    { 0x717f9d9d, SUC_lexicon_tags_55 },
    { 0xcbf9d9e, SUC_lexicon_tags_11 },
    { 0x691f9d9d, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbf9dab, SUC_lexicon_tags_12 },
    { 0x6b1f9dac, SUC_lexicon_tags_53 },
    { 0x845f9dad, SUC_lexicon_tags_11 },
    { 0x23f9dae, SUC_lexicon_tags_37 },
    { 0x9eff9dae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ff9db2, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdff9db5, SUC_lexicon_tags_12 },
    { 0x589f9db5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ff9db9, SUC_lexicon_tags_1 },
    { 0xad7f9dba, SUC_lexicon_tags_27 },
    { 0xf27f9dbb, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4d3f9dbd, SUC_lexicon_tags_25 },
    { 0x34ff9dbe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x2bff9dc0, SUC_lexicon_tags_40 },
    { 0x7e5f9dc1, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x133f9dc4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ff9dc7, SUC_lexicon_tags_45 },
    { 0x1f9f9dc8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebf9dce, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733f9dd4, SUC_lexicon_tags_11 },
    { 0x7cff9dd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6df9dd9, SUC_lexicon_tags_17 },
    { 0x3f1f9dd9, SUC_lexicon_tags_11 },
    { 0x2a7f9ddb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05f9de0, SUC_lexicon_tags_25 },
    { 0x6abf9de0, SUC_lexicon_tags_15 },
    { 0xf1ff9de2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07f9de5, SUC_lexicon_tags_25 },
    { 0x45ff9de5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x877f9de9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x46bf9deb, SUC_lexicon_tags_14 },
    { 0xb99f9dec, SUC_lexicon_tags_33 },
    { 0xa29f9dec, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9f9df0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269f9df5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x99ff9df7, SUC_lexicon_tags_25 },
    { 0xa01f9df8, SUC_lexicon_tags_17 },
    { 0x54ff9df9, SUC_lexicon_tags_56 },
    { 0xab1f9df7, SUC_lexicon_tags_13 },
    { 0xa1df9dfb, SUC_lexicon_tags_7 },
    { 0xb4df9dfc, SUC_lexicon_tags_7 },
    { 0xb51f9dfc, SUC_lexicon_tags_82 },
    { 0xd41f9dfe, SUC_lexicon_tags_31 },
    { 0xb45f9dfe, SUC_lexicon_tags_9 },
    { 0xc7ff9e00, SUC_lexicon_tags_2 },
    { 0x7d5f9dfe, SUC_lexicon_tags_23 },
    { 0x87f9e02, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259f9e08, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfff9e0c, SUC_lexicon_tags_277 },
    { 0x94df9e0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb63f9e0f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x215f9e12, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb69f9e14, SUC_lexicon_tags_0 },
    { 0xad1f9e15, SUC_lexicon_tags_0 },
    { 0xc57f9e16, SUC_lexicon_tags_17 },
    { 0x23f9e17, SUC_lexicon_tags_47 },
    { 0x92bf9e18, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1f9e1d, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0xe1bf9e1f, SUC_lexicon_tags_48 },
    { 0x41f9e20, SUC_lexicon_tags_1 },
    { 0x97f9e21, SUC_lexicon_tags_12 },
    { 0x67bf9e22, SUC_lexicon_tags_91 },
    { 0x691f9e22, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9f9e2a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x139f9e2e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x265f9e31, SUC_lexicon_tags_27 },
    { 0xf4df9e32, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf75f9e34, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f7f9e37, SUC_lexicon_tags_11 },
    { 0x5f3f9e38, SUC_lexicon_tags_17 },
    { 0x64bf9e37, SUC_lexicon_tags_55 },
    { 0xccdf9e3a, SUC_lexicon_tags_62 },
    { 0xe83f9e3b, SUC_lexicon_tags_0 },
    { 0xd09f9e3c, SUC_lexicon_tags_26 },
    { 0xb59f9e3d, SUC_lexicon_tags_70 },
    { 0x805f9e3e, SUC_lexicon_tags_57 },
    { 0xf09f9e39, SUC_lexicon_tags_7 },
    { 0x825f9e40, SUC_lexicon_tags_38 },
    { 0x473f9e40, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09f9e46, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7b3f9e48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ff9e4e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x95df9e51, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x34ff9e54, SUC_lexicon_tags_2 },
    { 0xc91f9e54, SUC_lexicon_tags_45 },
    { 0x31ff9e56, SUC_lexicon_tags_7 },
    { 0xc1df9e56, SUC_lexicon_tags_11 },
    { 0x6d5f9e58, SUC_lexicon_tags_63 },
    { 0x50bf9e57, SUC_lexicon_tags_62 },
    { 0x5b3f9e56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x657f9e5d, SUC_lexicon_tags_27 },
    { 0x9a7f9e5d, SUC_lexicon_tags_7 },
    { 0x1fdf9e5f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ff9e64, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xecff9e67, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x7eff9e6a, SUC_lexicon_tags_30 },
    { 0xeebf9e6a, SUC_lexicon_tags_37 },
    { 0x313f9e6c, SUC_lexicon_tags_17 },
    { 0x40bf9e6d, SUC_lexicon_tags_7 },
    { 0x109f9e6b, SUC_lexicon_tags_40 },
    { 0x69f9e6f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3df9e72, SUC_lexicon_tags_17 },
    { 0xbc9f9e73, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64bf9e78, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xcd7f9e7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6edf9e7d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x199f9e81, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x693f9e84, SUC_lexicon_tags_12 },
    { 0xdeff9e84, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1f9e8a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa27f9e8c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x657f9e8f, SUC_lexicon_tags_14 },
    { 0x7ff9e90, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fbf9e93, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7f9e97, SUC_lexicon_tags_1 },
    { 0xf6df9e98, SUC_lexicon_tags_63 },
    { 0x101f9e99, SUC_lexicon_tags_0 },
    { 0xa7df9e9a, SUC_lexicon_tags_43 },
    { 0x537f9e99, SUC_lexicon_tags_11 },
    { 0x4bff9e9c, SUC_lexicon_tags_12 },
    { 0x127f9e9c, SUC_lexicon_tags_46 },
    { 0x7c3f9e9e, SUC_lexicon_tags_25 },
    { 0xb8bf9e9f, SUC_lexicon_tags_37 },
    { 0x9f9ea0, SUC_lexicon_tags_33 },
    { 0x8e7f9e9d, SUC_lexicon_tags_26 },
    { 0xb65f9ea1, SUC_lexicon_tags_37 },
    { 0xa91f9ea3, SUC_lexicon_tags_27 },
    { 0xc4bf9ea2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe61f9ea6, SUC_lexicon_tags_2 },
    { 0x657f9ea7, SUC_lexicon_tags_11 },
    { 0x53ff9ea8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xee7f9eaa, SUC_lexicon_tags_17 },
    { 0x69f9eab, SUC_lexicon_tags_2 },
    { 0xedff9eac, SUC_lexicon_tags_38 },
    { 0xbfdf9ead, SUC_lexicon_tags_2 },
    { 0xecbf9ead, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x671f9eb3, SUC_lexicon_tags_0 },
    { 0x6c1f9eb4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17f9eb8, SUC_lexicon_tags_1 },
    { 0xf4df9eb9, SUC_lexicon_tags_43 },
    { 0x3e1f9eba, SUC_lexicon_tags_1 },
    { 0x353f9ebb, SUC_lexicon_tags_65 },
    { 0x813f9ebc, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddf9ec0, SUC_lexicon_tags_31 },
    { 0xf3f9ec1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3f9ec4, SUC_lexicon_tags_11 },
    { 0x5a1f9ec4, SUC_lexicon_tags_1 },
    { 0xd7df9ec4, SUC_lexicon_tags_55 },
    { 0xb7df9ec7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbf9ecf, SUC_lexicon_tags_19 },
    { 0xe75f9ed0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xadbf9ed2, SUC_lexicon_tags_37 },
    { 0x8bff9ed3, SUC_lexicon_tags_25 },
    { 0x7e3f9ed2, SUC_lexicon_tags_54 },
    { 0x513f9ed2, SUC_lexicon_tags_33 },
    { 0x9cdf9ed2, SUC_lexicon_tags_17 },
    { 0x62ff9ed4, SUC_lexicon_tags_7 },
    { 0xe8ff9ed8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ff9edd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edf9ee8, SUC_lexicon_tags_5 },
    { 0xfc1f9ee9, SUC_lexicon_tags_25 },
    { 0x3e7f9eea, SUC_lexicon_tags_17 },
    { 0x21ff9eeb, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xf41f9eee, SUC_lexicon_tags_25 },
    { 0x64df9eef, SUC_lexicon_tags_43 },
    { 0x2df9eef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb69f9ef4, SUC_lexicon_tags_15 },
    { 0x8eff9ef5, SUC_lexicon_tags_6 },
    { 0x719f9ef5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebf9ef9, SUC_lexicon_tags_7 },
    { 0x56ff9efa, SUC_lexicon_tags_79 },
    { 0x821f9efb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xef9f9efe, SUC_lexicon_tags_63 },
    { 0xc3df9efe, SUC_lexicon_tags_341 },
    { 0x5e5f9eff, SUC_lexicon_tags_1 },
    { 0x2dff9f00, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x769f9f0a, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7f9f0e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41f9f14, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x141f9f18, SUC_lexicon_tags_7 },
    { 0xddff9f19, SUC_lexicon_tags_133 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9f9f1e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb2ff9f20, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x78ff9f22, SUC_lexicon_tags_17 },
    { 0x48ff9f22, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf01f9f27, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bbf9f2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e5f9f31, SUC_lexicon_tags_7 },
    { 0x87df9f31, SUC_lexicon_tags_5 },
    { 0x8a9f9f33, SUC_lexicon_tags_2 },
    { 0x42df9f34, SUC_lexicon_tags_55 },
    { 0xa93f9f34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abf9f38, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0xdc7f9f3a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a5f9f3c, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887f9f43, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xabf9f46, SUC_lexicon_tags_60 },
    { 0xd7f9f46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x553f9f4a, SUC_lexicon_tags_27 },
    { 0xbc7f9f4b, SUC_lexicon_tags_25 },
    { 0x7bdf9f4c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd51f9f52, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905f9f56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xae9f9f59, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc61f9f5f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x331f9f65, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9f9f68, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19f9f6b, SUC_lexicon_tags_12 },
    { 0x6b7f9f6c, SUC_lexicon_tags_13 },
    { 0x22ff9f6b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x587f9f6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817f9f78, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xf11f9f7b, SUC_lexicon_tags_25 },
    { 0x23bf9f7b, SUC_lexicon_tags_19 },
    { 0x8b7f9f7b, SUC_lexicon_tags_61 },
    { 0x3d5f9f7e, SUC_lexicon_tags_0 },
    { 0x4e1f9f7f, SUC_lexicon_tags_37 },
    { 0x45f9f80, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x81f9f83, SUC_lexicon_tags_0 },
    { 0x19f9f84, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7f9f88, SUC_lexicon_tags_15 },
    { 0x273f9f89, SUC_lexicon_tags_35 },
    { 0xda7f9f88, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc11f9f8c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x9ddf9f8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3fdf9f90, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bf9f95, SUC_lexicon_tags_0 },
    { 0xb25f9f96, SUC_lexicon_tags_15 },
    { 0xd3ff9f97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c1f9f9c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5aff9f9e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x145f9fa1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203f9fab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509f9faf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd13f9fb2, SUC_lexicon_tags_17 },
    { 0x589f9fb3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9f9fb9, SUC_lexicon_tags_12 },
    { 0x7dff9fb9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x915f9fbd, SUC_lexicon_tags_1 },
    { 0x10bf9fbd, SUC_lexicon_tags_7 },
    { 0xaa9f9fbe, SUC_lexicon_tags_31 },
    { 0x85df9fbd, SUC_lexicon_tags_18 },
    { 0xeabf9fbf, SUC_lexicon_tags_38 },
    { 0x27bf9fc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xeff9fc5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd5df9fc7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeadf9fc9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89f9fd4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x3b9f9fd6, SUC_lexicon_tags_27 },
    { 0xf2df9fd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x63f9fd9, SUC_lexicon_tags_25 },
    { 0x853f9fda, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xecbf9fdc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5f9fe7, SUC_lexicon_tags_43 },
    { 0xe65f9fe7, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0x1ebf9fea, SUC_lexicon_tags_45 },
    { 0xfb7f9feb, SUC_lexicon_tags_25 },
    { 0xbf9feb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x23f9fee, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3f9ff1, SUC_lexicon_tags_6 },
    { 0x723f9ff2, SUC_lexicon_tags_1 },
    { 0x629f9ff3, SUC_lexicon_tags_17 },
    { 0x4f3f9ff1, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x41df9ff6, SUC_lexicon_tags_14 },
    { 0x4a1f9ff7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x997f9ffb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd29f9ffd, SUC_lexicon_tags_19 },
    { 0x1f9ffe, SUC_lexicon_tags_82 },
    { 0xd71f9fff, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe3fa002, SUC_lexicon_tags_2 },
    { 0xd17fa002, SUC_lexicon_tags_17 },
    { 0x4bbfa004, SUC_lexicon_tags_218 },
    { 0xa3fa004, SUC_lexicon_tags_37 },
    { 0x78dfa004, SUC_lexicon_tags_17 },
    { 0xcc3fa007, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bffa00a, SUC_lexicon_tags_14 },
    { 0xd69fa00b, SUC_lexicon_tags_4 },
    { 0xaf9fa00b, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3fa013, SUC_lexicon_tags_8 },
    { 0x30ffa014, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x583fa017, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7f9fa019, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5fa01d, SUC_lexicon_tags_49 },
    { 0xf2ffa01e, SUC_lexicon_tags_43 },
    { 0xc13fa01f, SUC_lexicon_tags_39 },
    { 0xeffa020, SUC_lexicon_tags_11 },
    { 0x77fa01d, SUC_lexicon_tags_21 },
    { 0xc4bfa01f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d5fa025, SUC_lexicon_tags_11 },
    { 0x82dfa026, SUC_lexicon_tags_37 },
    { 0x895fa027, SUC_lexicon_tags_80 },
    { 0x801fa025, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0xbcffa02b, SUC_lexicon_tags_17 },
    { 0x3d9fa02c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xceffa02f, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9fa037, SUC_lexicon_tags_61 },
    { 0xcb9fa038, SUC_lexicon_tags_2 },
    { 0xdb7fa039, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x53ffa03b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd53fa03d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fa042, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf09fa045, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x345fa047, SUC_lexicon_tags_37 },
    { 0xd9bfa048, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ffa04d, SUC_lexicon_tags_21 },
    { 0x9c1fa04e, SUC_lexicon_tags_29 },
    { 0x2f1fa04f, SUC_lexicon_tags_1 },
    { 0x1bffa04d, SUC_lexicon_tags_29 },
    { 0x73ffa04e, SUC_lexicon_tags_14 },
    { 0x413fa052, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafbfa056, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xdedfa058, SUC_lexicon_tags_16 },
    { 0x8a1fa059, SUC_lexicon_tags_25 },
    { 0x177fa059, SUC_lexicon_tags_17 },
    { 0x62bfa05b, SUC_lexicon_tags_27 },
    { 0x1f7fa05c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bffa062, SUC_lexicon_tags_45 },
    { 0xf59fa063, SUC_lexicon_tags_54 },
    { 0x891fa064, SUC_lexicon_tags_37 },
    { 0x8b1fa063, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecffa06d, SUC_lexicon_tags_7 },
    { 0xa39fa06e, SUC_lexicon_tags_26 },
    { 0xbd5fa06e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5fa072, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fa076, SUC_lexicon_tags_0 },
    { 0x2bffa077, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd45fa079, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xffdfa07b, SUC_lexicon_tags_10 },
    { 0x8c5fa07c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39fa086, SUC_lexicon_tags_2 },
    { 0x9bbfa087, SUC_lexicon_tags_15 },
    { 0x59fa088, SUC_lexicon_tags_8 },
    { 0x2f1fa089, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xe3fa08b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fa098, SUC_lexicon_tags_15 },
    { 0x133fa099, SUC_lexicon_tags_2 },
    { 0xccbfa09a, SUC_lexicon_tags_33 },
    { 0x5f5fa099, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621fa0a0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x259fa0a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedfa0a7, SUC_lexicon_tags_6 },
    { 0x779fa0a8, SUC_lexicon_tags_9 },
    { 0x841fa0a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65fa0ae, SUC_lexicon_tags_10 },
    { 0x2b3fa0af, SUC_lexicon_tags_15 },
    { 0x30dfa0b0, SUC_lexicon_tags_27 },
    { 0x1cffa0ae, SUC_lexicon_tags_65 },
    { 0xb4dfa0b2, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x87ffa0b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb83fa0b9, SUC_lexicon_tags_0 },
    { 0x25ffa0b9, SUC_lexicon_tags_37 },
    { 0xcabfa0bb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3fa0be, SUC_lexicon_tags_30 },
    { 0xeffa0bf, SUC_lexicon_tags_49 },
    { 0xe2dfa0c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x58bfa0c2, SUC_lexicon_tags_7 },
    { 0xc6ffa0c3, SUC_lexicon_tags_9 },
    { 0x50dfa0c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x961fa0c7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9effa0c9, SUC_lexicon_tags_15 },
    { 0x7c7fa0ca, SUC_lexicon_tags_15 },
    { 0x9f3fa0c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557fa0d7, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5fa0de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb49fa0e0, SUC_lexicon_tags_2 },
    { 0xa7bfa0e1, SUC_lexicon_tags_7 },
    { 0x419fa0e2, SUC_lexicon_tags_118 },
    { 0xb3bfa0e1, SUC_lexicon_tags_18 },
    { 0x377fa0e1, SUC_lexicon_tags_11 },
    { 0x687fa0e3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcd7fa0e7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bfa0eb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83fa0ee, SUC_lexicon_tags_1 },
    { 0xa05fa0ee, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedbfa0f3, SUC_lexicon_tags_1 },
    { 0x717fa0f4, SUC_lexicon_tags_2 },
    { 0x697fa0f4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1effa0fc, SUC_lexicon_tags_1 },
    { 0x367fa0fd, SUC_lexicon_tags_37 },
    { 0xe3ffa0fe, SUC_lexicon_tags_43 },
    { 0xe47fa0ff, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdfa102, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dffa107, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf7ffa109, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe95fa10d, SUC_lexicon_tags_12 },
    { 0xc5bfa10e, SUC_lexicon_tags_25 },
    { 0xcc7fa10f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x927fa111, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x973fa113, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x107fa115, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3fa11c, SUC_lexicon_tags_1 },
    { 0x5e5fa11c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685fa121, SUC_lexicon_tags_47 },
    { 0x805fa122, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc69fa129, SUC_lexicon_tags_14 },
    { 0xd73fa129, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5f3fa12c, SUC_lexicon_tags_29 },
    { 0xcddfa12c, SUC_lexicon_tags_15 },
    { 0x467fa12c, SUC_lexicon_tags_14 },
    { 0xb5dfa12f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9fa133, SUC_lexicon_tags_10 },
    { 0x6bfa133, SUC_lexicon_tags_7 },
    { 0x981fa135, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e9fa13b, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x411fa13e, SUC_lexicon_tags_27 },
    { 0xa71fa13e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7fa145, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd57fa14c, SUC_lexicon_tags_0 },
    { 0x91dfa14d, SUC_lexicon_tags_38 },
    { 0x957fa14d, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa07fa153, SUC_lexicon_tags_15 },
    { 0x963fa153, SUC_lexicon_tags_25 },
    { 0x24dfa155, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x783fa15d, SUC_lexicon_tags_69 },
    { 0x8fdfa15e, SUC_lexicon_tags_15 },
    { 0xe21fa15d, SUC_lexicon_tags_66 },
    { 0x949fa15d, SUC_lexicon_tags_14 },
    { 0x4effa15e, SUC_lexicon_tags_37 },
    { 0x1d5fa15e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ffa165, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37fa16b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dfa16f, SUC_lexicon_tags_63 },
    { 0x79ffa16f, SUC_lexicon_tags_4 },
    { 0x261fa16f, SUC_lexicon_tags_37 },
    { 0xe85fa172, SUC_lexicon_tags_47 },
    { 0x99fa173, SUC_lexicon_tags_6 },
    { 0x5ffa174, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xfd1fa176, SUC_lexicon_tags_62 },
    { 0xf73fa176, SUC_lexicon_tags_2 },
    { 0xa75fa177, SUC_lexicon_tags_38 },
    { 0x3c7fa179, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235fa17d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adfa181, SUC_lexicon_tags_11 },
    { 0x88ffa182, SUC_lexicon_tags_17 },
    { 0x289fa181, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1fa188, SUC_lexicon_tags_4 },
    { 0xda1fa189, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x41fa18b, SUC_lexicon_tags_47 },
    { 0x1ebfa18b, SUC_lexicon_tags_19 },
    { 0x6a9fa18b, SUC_lexicon_tags_11 },
    { 0x495fa18e, SUC_lexicon_tags_17 },
    { 0x98ffa18e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9fa193, SUC_lexicon_tags_7 },
    { 0x5f5fa194, SUC_lexicon_tags_2 },
    { 0xab5fa194, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x657fa197, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9e5fa199, SUC_lexicon_tags_2 },
    { 0xf33fa19a, SUC_lexicon_tags_37 },
    { 0xf17fa199, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1fa19d, SUC_lexicon_tags_47 },
    { 0x59fa19e, SUC_lexicon_tags_24 },
    { 0x6ffa19f, SUC_lexicon_tags_17 },
    { 0xdcdfa19e, SUC_lexicon_tags_2 },
    { 0x56bfa1a1, SUC_lexicon_tags_30 },
    { 0x545fa1a2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91fa1a7, SUC_lexicon_tags_37 },
    { 0x6affa1a8, SUC_lexicon_tags_14 },
    { 0xad5fa1a7, SUC_lexicon_tags_8 },
    { 0xebfa1a9, SUC_lexicon_tags_11 },
    { 0x9c1fa1a9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xc51fa1ae, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x2fffa1b0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x4a1fa1b2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xdebfa1b4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21fa1b7, SUC_lexicon_tags_10 },
    { 0x853fa1b8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dfa1bc, SUC_lexicon_tags_23 },
    { 0xa55fa1bc, SUC_lexicon_tags_25 },
    { 0x54dfa1bd, SUC_lexicon_tags_16 },
    { 0x2bdfa1bf, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb63fa1c5, SUC_lexicon_tags_43 },
    { 0xbfa1c6, SUC_lexicon_tags_31 },
    { 0xf65fa1c7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x51dfa1c9, SUC_lexicon_tags_62 },
    { 0xba5fa1ca, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41fa1ce, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655fa1d2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x283fa1d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe59fa1d7, SUC_lexicon_tags_7 },
    { 0x65fa1d8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x135fa1da, SUC_lexicon_tags_19 },
    { 0xeeffa1db, SUC_lexicon_tags_25 },
    { 0x321fa1db, SUC_lexicon_tags_2 },
    { 0xb61fa1da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd57fa1df, SUC_lexicon_tags_17 },
    { 0x651fa1e0, SUC_lexicon_tags_63 },
    { 0x9d7fa1e1, SUC_lexicon_tags_14 },
    { 0xd5ffa1e2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119fa1e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x347fa1e8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfa1f2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x183fa1f6, SUC_lexicon_tags_1 },
    { 0x879fa1f7, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c3fa1fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc67fa1fc, SUC_lexicon_tags_38 },
    { 0xd59fa1fc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x11dfa1ff, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97fa203, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbdfa205, SUC_lexicon_tags_60 },
    { 0x66ffa205, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddfa20a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63dfa20c, SUC_lexicon_tags_30 },
    { 0x94bfa20d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xdadfa20f, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557fa218, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xcf9fa21a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07fa21d, SUC_lexicon_tags_37 },
    { 0x77ffa21e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x3ebfa220, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bfa226, SUC_lexicon_tags_25 },
    { 0x441fa226, SUC_lexicon_tags_14 },
    { 0x19fa226, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x579fa22a, SUC_lexicon_tags_39 },
    { 0x64dfa22b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81dfa22d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb25fa23b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb23fa23d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1b9fa23f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x39ffa242, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bfa249, SUC_lexicon_tags_37 },
    { 0xaaffa24a, SUC_lexicon_tags_34 },
    { 0x62bfa24b, SUC_lexicon_tags_7 },
    { 0x635fa24c, SUC_lexicon_tags_25 },
    { 0x773fa24c, SUC_lexicon_tags_11 },
    { 0xbf7fa24e, SUC_lexicon_tags_11 },
    { 0x15ffa24f, SUC_lexicon_tags_0 },
    { 0x1b3fa250, SUC_lexicon_tags_25 },
    { 0xe6bfa24b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebfa255, SUC_lexicon_tags_27 },
    { 0x1f1fa256, SUC_lexicon_tags_12 },
    { 0x469fa257, SUC_lexicon_tags_27 },
    { 0x587fa258, SUC_lexicon_tags_63 },
    { 0xb41fa257, SUC_lexicon_tags_17 },
    { 0xa8dfa25a, SUC_lexicon_tags_10 },
    { 0x381fa25b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xb2ffa25d, SUC_lexicon_tags_2 },
    { 0xd69fa25d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xc4ffa260, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd79fa262, SUC_lexicon_tags_60 },
    { 0xeabfa263, SUC_lexicon_tags_2 },
    { 0x44ffa262, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bfa26c, SUC_lexicon_tags_14 },
    { 0x775fa26d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bfa276, SUC_lexicon_tags_1298 },
    { 0, NULL },
    { 0x3fffa278, SUC_lexicon_tags_17 },
    { 0x8a5fa278, SUC_lexicon_tags_63 },
    { 0xb99fa27a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ffa27e, SUC_lexicon_tags_17 },
    { 0xbf5fa27f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c9fa283, SUC_lexicon_tags_19 },
    { 0xfc7fa283, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa15fa286, SUC_lexicon_tags_25 },
    { 0x5cbfa287, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5fa28b, SUC_lexicon_tags_27 },
    { 0xb8bfa28b, SUC_lexicon_tags_37 },
    { 0x667fa28c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ffa292, SUC_lexicon_tags_47 },
    { 0x5a7fa293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf3bfa296, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbfa299, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x867fa29b, SUC_lexicon_tags_14 },
    { 0x10bfa29b, SUC_lexicon_tags_27 },
    { 0x6edfa29c, SUC_lexicon_tags_22 },
    { 0x735fa29c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdfa2a1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d7fa2a3, SUC_lexicon_tags_13 },
    { 0x9ffa2a3, SUC_lexicon_tags_37 },
    { 0xe79fa2a5, SUC_lexicon_tags_72 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67fa2c1, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29fa2cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb49fa2cd, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3fa2d2, SUC_lexicon_tags_34 },
    { 0x983fa2d3, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0x129fa2d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fa2dc, SUC_lexicon_tags_3 },
    { 0x4cbfa2dc, SUC_lexicon_tags_27 },
    { 0x771fa2de, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8a7fa2e0, SUC_lexicon_tags_17 },
    { 0xd7dfa2e0, SUC_lexicon_tags_9 },
    { 0xb85fa2e1, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb7fa2e8, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x209fa2ea, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bfa2ef, SUC_lexicon_tags_61 },
    { 0xe6ffa2f0, SUC_lexicon_tags_2 },
    { 0x533fa2f0, SUC_lexicon_tags_18 },
    { 0xcb7fa2f1, SUC_lexicon_tags_38 },
    { 0x5d1fa2f3, SUC_lexicon_tags_31 },
    { 0x1affa2f3, SUC_lexicon_tags_88 },
    { 0x80ffa2f4, SUC_lexicon_tags_66 },
    { 0x9cdfa2f2, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xd79fa2f9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x30dfa2fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbfa300, SUC_lexicon_tags_87 },
    { 0x5c3fa301, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86dfa306, SUC_lexicon_tags_959 },
    { 0, NULL },
    { 0x559fa308, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dfa30b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4bfa30e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9fa313, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x485fa315, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xed5fa318, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9a7fa31a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x89dfa31c, SUC_lexicon_tags_11 },
    { 0xae7fa31d, SUC_lexicon_tags_37 },
    { 0x867fa31e, SUC_lexicon_tags_11 },
    { 0xd89fa31d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1bfa322, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3fa328, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3fa32b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x83dfa32d, SUC_lexicon_tags_2 },
    { 0x1fbfa32e, SUC_lexicon_tags_1012 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fbfa335, SUC_lexicon_tags_17 },
    { 0x573fa335, SUC_lexicon_tags_9 },
    { 0x577fa336, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0xe59fa339, SUC_lexicon_tags_15 },
    { 0xfd9fa339, SUC_lexicon_tags_11 },
    { 0xc59fa33b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe5bfa33d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x94dfa340, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x42dfa343, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9fa346, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9fa35f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde3fa361, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0x375fa363, SUC_lexicon_tags_26 },
    { 0xfc1fa364, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x325fa367, SUC_lexicon_tags_55 },
    { 0xd33fa368, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1fa371, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6d9fa373, SUC_lexicon_tags_2 },
    { 0xb83fa373, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x587fa376, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb9ffa378, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dffa37c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3fa380, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdfa384, SUC_lexicon_tags_16 },
    { 0x405fa385, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x761fa387, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599fa38c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe99fa390, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x261fa392, SUC_lexicon_tags_8 },
    { 0x7adfa393, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc7ffa395, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105fa399, SUC_lexicon_tags_32 },
    { 0xd83fa399, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb5dfa39c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399fa3a0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x63ffa3a2, SUC_lexicon_tags_47 },
    { 0x9f1fa3a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebfa3a6, SUC_lexicon_tags_37 },
    { 0x3e9fa3a7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffa3aa, SUC_lexicon_tags_10 },
    { 0x3fbfa3ab, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x207fa3ad, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc13fa3af, SUC_lexicon_tags_60 },
    { 0x609fa3b0, SUC_lexicon_tags_45 },
    { 0xb93fa3b1, SUC_lexicon_tags_27 },
    { 0x2c9fa3b2, SUC_lexicon_tags_43 },
    { 0xd1dfa3b1, SUC_lexicon_tags_1 },
    { 0x1e7fa3b4, SUC_lexicon_tags_25 },
    { 0x587fa3b5, SUC_lexicon_tags_12 },
    { 0x557fa3b5, SUC_lexicon_tags_11 },
    { 0x4ebfa3b7, SUC_lexicon_tags_2 },
    { 0x731fa3b7, SUC_lexicon_tags_1 },
    { 0xb35fa3b9, SUC_lexicon_tags_84 },
    { 0x377fa3b5, SUC_lexicon_tags_57 },
    { 0x787fa3b3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741fa3c0, SUC_lexicon_tags_151 },
    { 0xe3dfa3c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x5a7fa3c3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56dfa3c8, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x869fa3cb, SUC_lexicon_tags_63 },
    { 0xe61fa3cb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101fa3d0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb61fa3da, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc37fa3dc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207fa3e9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203fa3ee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429fa3f3, SUC_lexicon_tags_61 },
    { 0x19fa3f4, SUC_lexicon_tags_25 },
    { 0x6cffa3f3, SUC_lexicon_tags_37 },
    { 0xc75fa3f5, SUC_lexicon_tags_23 },
    { 0x63dfa3f7, SUC_lexicon_tags_12 },
    { 0x461fa3f8, SUC_lexicon_tags_2 },
    { 0x6f3fa3f3, SUC_lexicon_tags_116 },
    { 0xcf9fa3fa, SUC_lexicon_tags_39 },
    { 0xa35fa3f4, SUC_lexicon_tags_60 },
    { 0x4e3fa3fc, SUC_lexicon_tags_60 },
    { 0xaddfa3fc, SUC_lexicon_tags_11 },
    { 0x409fa3fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9fa404, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xacbfa407, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d9fa40e, SUC_lexicon_tags_15 },
    { 0xcc7fa40f, SUC_lexicon_tags_141 },
    { 0x643fa410, SUC_lexicon_tags_11 },
    { 0x597fa410, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ffa415, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfa419, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dfa420, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dfa425, SUC_lexicon_tags_12 },
    { 0x99ffa425, SUC_lexicon_tags_57 },
    { 0x913fa427, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x331fa42a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd89fa43b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xf3bfa43d, SUC_lexicon_tags_55 },
    { 0x695fa43e, SUC_lexicon_tags_33 },
    { 0x763fa43f, SUC_lexicon_tags_12 },
    { 0xdcdfa440, SUC_lexicon_tags_11 },
    { 0xd3dfa441, SUC_lexicon_tags_40 },
    { 0xbadfa442, SUC_lexicon_tags_42 },
    { 0xc5dfa43e, SUC_lexicon_tags_15 },
    { 0x8e1fa43f, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd9fa446, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x39ffa448, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x959fa44a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3fa44d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x285fa44f, SUC_lexicon_tags_37 },
    { 0x819fa450, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ffa453, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3fa457, SUC_lexicon_tags_29 },
    { 0x29ffa458, SUC_lexicon_tags_11 },
    { 0x279fa458, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bfa461, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x281fa463, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x437fa465, SUC_lexicon_tags_12 },
    { 0x29dfa466, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5dfa468, SUC_lexicon_tags_19 },
    { 0x6effa468, SUC_lexicon_tags_17 },
    { 0x781fa46a, SUC_lexicon_tags_17 },
    { 0x9fdfa46b, SUC_lexicon_tags_12 },
    { 0x76bfa46c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x59bfa46e, SUC_lexicon_tags_26 },
    { 0x9a1fa46f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdfa47a, SUC_lexicon_tags_30 },
    { 0x145fa47b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6d1fa47d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fa481, SUC_lexicon_tags_7 },
    { 0xbeffa482, SUC_lexicon_tags_45 },
    { 0x52dfa483, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xedffa485, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x185fa487, SUC_lexicon_tags_23 },
    { 0x253fa487, SUC_lexicon_tags_37 },
    { 0x725fa489, SUC_lexicon_tags_47 },
    { 0x60ffa48a, SUC_lexicon_tags_33 },
    { 0xeb9fa48b, SUC_lexicon_tags_37 },
    { 0x195fa48c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x205fa492, SUC_lexicon_tags_70 },
    { 0xca7fa492, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe79fa495, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc55fa49d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x14ffa49f, SUC_lexicon_tags_37 },
    { 0x553fa4a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26dfa4a2, SUC_lexicon_tags_1 },
    { 0x83bfa4a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc89fa4a9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fa4af, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xe27fa4b2, SUC_lexicon_tags_70 },
    { 0x6cffa4b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb3fa4b5, SUC_lexicon_tags_15 },
    { 0xfb1fa4b6, SUC_lexicon_tags_27 },
    { 0xfcbfa4b7, SUC_lexicon_tags_31 },
    { 0x759fa4b8, SUC_lexicon_tags_37 },
    { 0x137fa4b8, SUC_lexicon_tags_37 },
    { 0x8fffa4ba, SUC_lexicon_tags_23 },
    { 0x719fa4ba, SUC_lexicon_tags_37 },
    { 0x2a5fa4bc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5fa4bf, SUC_lexicon_tags_17 },
    { 0xd8bfa4bf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3fa4c5, SUC_lexicon_tags_37 },
    { 0xec9fa4c5, SUC_lexicon_tags_17 },
    { 0x6a9fa4c7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7fa4cb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xfb9fa4cd, SUC_lexicon_tags_38 },
    { 0x69dfa4ce, SUC_lexicon_tags_25 },
    { 0x7dffa4ce, SUC_lexicon_tags_30 },
    { 0xdddfa4d0, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x54bfa4d2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4c7fa4d4, SUC_lexicon_tags_31 },
    { 0x637fa4d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07fa4d9, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dfa4dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x299fa4de, SUC_lexicon_tags_37 },
    { 0xc83fa4df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401fa4e6, SUC_lexicon_tags_1 },
    { 0x667fa4e6, SUC_lexicon_tags_7 },
    { 0x8a5fa4e8, SUC_lexicon_tags_12 },
    { 0xa53fa4e9, SUC_lexicon_tags_0 },
    { 0xa0dfa4e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x8c5fa4ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2affa4f3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x393fa4f6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x827fa4f8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35fa4fb, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957fa501, SUC_lexicon_tags_26 },
    { 0x9effa502, SUC_lexicon_tags_27 },
    { 0xc0ffa503, SUC_lexicon_tags_60 },
    { 0xaa9fa503, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfa508, SUC_lexicon_tags_25 },
    { 0xbc7fa509, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1fa50e, SUC_lexicon_tags_127 },
    { 0x83bfa50f, SUC_lexicon_tags_51 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9fa523, SUC_lexicon_tags_16 },
    { 0x5d7fa524, SUC_lexicon_tags_16 },
    { 0x4b7fa523, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xf99fa527, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xee5fa529, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5fa52e, SUC_lexicon_tags_37 },
    { 0x9ffa52f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x49ffa532, SUC_lexicon_tags_70 },
    { 0xe9ffa533, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb87fa537, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x773fa53a, SUC_lexicon_tags_77 },
    { 0x56dfa53a, SUC_lexicon_tags_37 },
    { 0x543fa53c, SUC_lexicon_tags_11 },
    { 0xe5dfa53d, SUC_lexicon_tags_11 },
    { 0xc4bfa53e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cbfa543, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83fa54a, SUC_lexicon_tags_26 },
    { 0xd55fa54a, SUC_lexicon_tags_27 },
    { 0xf7ffa54c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ffa550, SUC_lexicon_tags_16 },
    { 0xc2ffa551, SUC_lexicon_tags_33 },
    { 0x46bfa552, SUC_lexicon_tags_47 },
    { 0x3bdfa552, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbddfa559, SUC_lexicon_tags_27 },
    { 0xb01fa55a, SUC_lexicon_tags_15 },
    { 0x643fa55b, SUC_lexicon_tags_37 },
    { 0xcf1fa55c, SUC_lexicon_tags_33 },
    { 0xb6dfa55d, SUC_lexicon_tags_25 },
    { 0x42ffa55e, SUC_lexicon_tags_12 },
    { 0xbeffa55f, SUC_lexicon_tags_33 },
    { 0x13fa55f, SUC_lexicon_tags_108 },
    { 0xacbfa561, SUC_lexicon_tags_98 },
    { 0xa6bfa562, SUC_lexicon_tags_12 },
    { 0xa4ffa563, SUC_lexicon_tags_12 },
    { 0x5c1fa55b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd57fa567, SUC_lexicon_tags_45 },
    { 0xc75fa567, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbfa56d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x37bfa56f, SUC_lexicon_tags_43 },
    { 0x7dbfa56f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fa573, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951fa579, SUC_lexicon_tags_68 },
    { 0x55dfa57a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbfa57d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f1fa585, SUC_lexicon_tags_12 },
    { 0xae1fa585, SUC_lexicon_tags_14 },
    { 0x83dfa585, SUC_lexicon_tags_27 },
    { 0xb6ffa588, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb2bfa58a, SUC_lexicon_tags_37 },
    { 0x9f9fa58a, SUC_lexicon_tags_1 },
    { 0xb15fa58c, SUC_lexicon_tags_87 },
    { 0xeabfa58d, SUC_lexicon_tags_46 },
    { 0x839fa58d, SUC_lexicon_tags_17 },
    { 0x639fa58f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41fa596, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xea5fa598, SUC_lexicon_tags_11 },
    { 0x5fdfa599, SUC_lexicon_tags_25 },
    { 0x1a1fa598, SUC_lexicon_tags_7 },
    { 0x645fa59b, SUC_lexicon_tags_8 },
    { 0x6e1fa59b, SUC_lexicon_tags_301 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7ffa5ad, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb49fa5af, SUC_lexicon_tags_49 },
    { 0x60ffa5b0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbfa5c1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bfa5cd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a5fa5d1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xdf3fa5d3, SUC_lexicon_tags_25 },
    { 0x807fa5d4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x267fa5d6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfd1fa5d8, SUC_lexicon_tags_26 },
    { 0xa77fa5d9, SUC_lexicon_tags_8 },
    { 0x9fdfa5da, SUC_lexicon_tags_31 },
    { 0x6d7fa5da, SUC_lexicon_tags_39 },
    { 0x7cbfa5dc, SUC_lexicon_tags_17 },
    { 0xb71fa5dd, SUC_lexicon_tags_38 },
    { 0x5e1fa5dd, SUC_lexicon_tags_2 },
    { 0xc45fa5df, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2bdfa5e1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x285fa5e3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd83fa5e5, SUC_lexicon_tags_11 },
    { 0x333fa5e6, SUC_lexicon_tags_11 },
    { 0xc83fa5e7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x373fa5e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383fa5ef, SUC_lexicon_tags_2 },
    { 0x4d5fa5ef, SUC_lexicon_tags_38 },
    { 0x52ffa5f1, SUC_lexicon_tags_21 },
    { 0x8e1fa5f0, SUC_lexicon_tags_38 },
    { 0xa6ffa5f3, SUC_lexicon_tags_15 },
    { 0x97fa5f4, SUC_lexicon_tags_12 },
    { 0xe53fa5f1, SUC_lexicon_tags_38 },
    { 0xc7ffa5ef, SUC_lexicon_tags_23 },
    { 0xbf1fa5f0, SUC_lexicon_tags_45 },
    { 0x75fa5f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xb23fa5fb, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f5fa5ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21fa602, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfa606, SUC_lexicon_tags_34 },
    { 0x74bfa607, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x3cffa609, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdfa60e, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x5cdfa610, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9fa614, SUC_lexicon_tags_25 },
    { 0x765fa615, SUC_lexicon_tags_7 },
    { 0x3d3fa614, SUC_lexicon_tags_1 },
    { 0x3c3fa614, SUC_lexicon_tags_11 },
    { 0x317fa617, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3c7fa61a, SUC_lexicon_tags_19 },
    { 0x681fa61b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc07fa61e, SUC_lexicon_tags_12 },
    { 0xce9fa61e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x935fa623, SUC_lexicon_tags_11 },
    { 0x375fa624, SUC_lexicon_tags_63 },
    { 0xbe1fa625, SUC_lexicon_tags_47 },
    { 0x76ffa626, SUC_lexicon_tags_12 },
    { 0x573fa624, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5fa62c, SUC_lexicon_tags_63 },
    { 0x801fa62c, SUC_lexicon_tags_15 },
    { 0x78dfa62e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x247fa631, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b3fa636, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dfa63b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xedffa63d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9fa642, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1fa64a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5fa64f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd0ffa651, SUC_lexicon_tags_1 },
    { 0xa57fa652, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3fa656, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bfa65f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9fa663, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc61fa665, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x289fa667, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x615fa66d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xad1fa66f, SUC_lexicon_tags_11 },
    { 0x47ffa670, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467fa674, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195fa67e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07fa681, SUC_lexicon_tags_17 },
    { 0x425fa681, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1fa68c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5fa690, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bfa693, SUC_lexicon_tags_11 },
    { 0xc5dfa693, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xf49fa696, SUC_lexicon_tags_2 },
    { 0xb3bfa697, SUC_lexicon_tags_32 },
    { 0x84ffa697, SUC_lexicon_tags_1542 },
    { 0x27bfa699, SUC_lexicon_tags_151 },
    { 0xde3fa69a, SUC_lexicon_tags_60 },
    { 0x1bdfa69a, SUC_lexicon_tags_11 },
    { 0x743fa697, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fdfa6a6, SUC_lexicon_tags_14 },
    { 0xdd5fa6a6, SUC_lexicon_tags_17 },
    { 0x85ffa6a7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9fa6af, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x17dfa6b2, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6edfa6b4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7fa6b8, SUC_lexicon_tags_7 },
    { 0xe61fa6b8, SUC_lexicon_tags_7 },
    { 0x625fa6ba, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x73dfa6bc, SUC_lexicon_tags_38 },
    { 0x67ffa6bd, SUC_lexicon_tags_25 },
    { 0x385fa6bd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf7fa6c0, SUC_lexicon_tags_83 },
    { 0x769fa6c1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xee5fa6c3, SUC_lexicon_tags_19 },
    { 0x707fa6c4, SUC_lexicon_tags_7 },
    { 0x841fa6c3, SUC_lexicon_tags_92 },
    { 0x931fa6c5, SUC_lexicon_tags_37 },
    { 0xedffa6c7, SUC_lexicon_tags_14 },
    { 0x159fa6c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7fa6cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5fa6d3, SUC_lexicon_tags_45 },
    { 0xa2ffa6d3, SUC_lexicon_tags_7 },
    { 0x9c5fa6d4, SUC_lexicon_tags_17 },
    { 0x253fa6d6, SUC_lexicon_tags_53 },
    { 0x277fa6d7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x989fa6d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bfa6df, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3d9fa6e1, SUC_lexicon_tags_12 },
    { 0xf93fa6e2, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43fa6e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7fa6ea, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x499fa6f1, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbfa6f8, SUC_lexicon_tags_19 },
    { 0x323fa6f8, SUC_lexicon_tags_82 },
    { 0xc8dfa6fa, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa39fa6fd, SUC_lexicon_tags_27 },
    { 0xcf5fa6fe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dfa705, SUC_lexicon_tags_894 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb93fa70a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723fa70f, SUC_lexicon_tags_9 },
    { 0x707fa710, SUC_lexicon_tags_43 },
    { 0xeb1fa711, SUC_lexicon_tags_14 },
    { 0x623fa712, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71fa715, SUC_lexicon_tags_25 },
    { 0x5f5fa715, SUC_lexicon_tags_35 },
    { 0x7b1fa717, SUC_lexicon_tags_12 },
    { 0xd3dfa717, SUC_lexicon_tags_39 },
    { 0x82ffa715, SUC_lexicon_tags_25 },
    { 0x799fa71a, SUC_lexicon_tags_35 },
    { 0x301fa71b, SUC_lexicon_tags_0 },
    { 0x33ffa71b, SUC_lexicon_tags_11 },
    { 0xdc7fa71d, SUC_lexicon_tags_0 },
    { 0x2ffa71e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbfa727, SUC_lexicon_tags_37 },
    { 0x54bfa728, SUC_lexicon_tags_28 },
    { 0x159fa728, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477fa72f, SUC_lexicon_tags_170 },
    { 0xf07fa730, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd7dfa732, SUC_lexicon_tags_31 },
    { 0xddffa733, SUC_lexicon_tags_37 },
    { 0xd51fa733, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebfa738, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1fa73b, SUC_lexicon_tags_39 },
    { 0x6bdfa73c, SUC_lexicon_tags_2 },
    { 0xb87fa73c, SUC_lexicon_tags_25 },
    { 0x833fa73c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x62ffa741, SUC_lexicon_tags_40 },
    { 0xc9ffa742, SUC_lexicon_tags_44 },
    { 0xcadfa741, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3fa746, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x809fa74b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaeffa74e, SUC_lexicon_tags_7 },
    { 0xeb1fa74e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe1fa752, SUC_lexicon_tags_43 },
    { 0xc0dfa752, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7fa756, SUC_lexicon_tags_15 },
    { 0x363fa756, SUC_lexicon_tags_11 },
    { 0xb99fa756, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x359fa75a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x78ffa75c, SUC_lexicon_tags_1 },
    { 0x4d9fa75d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5fa760, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x985fa762, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x903fa766, SUC_lexicon_tags_28 },
    { 0x769fa767, SUC_lexicon_tags_21 },
    { 0x597fa768, SUC_lexicon_tags_23 },
    { 0xb47fa769, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7c1fa76b, SUC_lexicon_tags_2 },
    { 0x4affa76c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73bfa76e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfa774, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbfa779, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbfa77c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x171fa77f, SUC_lexicon_tags_11 },
    { 0x6fdfa77f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdfa783, SUC_lexicon_tags_60 },
    { 0x113fa783, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dfa789, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bffa78d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x897fa790, SUC_lexicon_tags_25 },
    { 0xa2bfa791, SUC_lexicon_tags_37 },
    { 0xe85fa792, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x335fa796, SUC_lexicon_tags_151 },
    { 0xfe5fa797, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3fa79a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x277fa79c, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x30dfa79e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe45fa7a0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xffa7a2, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xfedfa7a5, SUC_lexicon_tags_244 },
    { 0x87ffa7a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x13ffa7a8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7fa7ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcffa7ad, SUC_lexicon_tags_253 },
    { 0x285fa7ae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa63fa7b1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xf83fa7b4, SUC_lexicon_tags_27 },
    { 0x2d7fa7b4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7fa7b9, SUC_lexicon_tags_49 },
    { 0x729fa7b9, SUC_lexicon_tags_19 },
    { 0x101fa7bb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bfa7be, SUC_lexicon_tags_38 },
    { 0x295fa7bf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbfa7c2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd47fa7c5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x163fa7c8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99fa7d0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xce9fa7d3, SUC_lexicon_tags_37 },
    { 0xee1fa7d3, SUC_lexicon_tags_12 },
    { 0x981fa7d5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1fa7da, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x875fa7dc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64ffa7e5, SUC_lexicon_tags_7 },
    { 0xb47fa7e5, SUC_lexicon_tags_37 },
    { 0xc6ffa7e7, SUC_lexicon_tags_19 },
    { 0x9b3fa7e6, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0xcfdfa7ea, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9fa7ee, SUC_lexicon_tags_25 },
    { 0x841fa7ee, SUC_lexicon_tags_57 },
    { 0xb33fa7f0, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xc3bfa7f2, SUC_lexicon_tags_25 },
    { 0xe75fa7f3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79fa7f6, SUC_lexicon_tags_87 },
    { 0x1bdfa7f6, SUC_lexicon_tags_14 },
    { 0x443fa7f6, SUC_lexicon_tags_64 },
    { 0x7a5fa7f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbfa7ff, SUC_lexicon_tags_11 },
    { 0xd0dfa800, SUC_lexicon_tags_35 },
    { 0x841fa801, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x797fa804, SUC_lexicon_tags_28 },
    { 0xffdfa805, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x883fa809, SUC_lexicon_tags_14 },
    { 0x901fa80a, SUC_lexicon_tags_12 },
    { 0xdc1fa80a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77fa80e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x343fa810, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x9f3fa812, SUC_lexicon_tags_53 },
    { 0xe9dfa813, SUC_lexicon_tags_15 },
    { 0xd49fa814, SUC_lexicon_tags_10 },
    { 0x7c9fa815, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf6bfa819, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa81fa81b, SUC_lexicon_tags_26 },
    { 0x88bfa81c, SUC_lexicon_tags_1 },
    { 0xda1fa81b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x203fa81f, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x565fa821, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x327fa826, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85dfa82a, SUC_lexicon_tags_23 },
    { 0xf8bfa82b, SUC_lexicon_tags_45 },
    { 0xda1fa82b, SUC_lexicon_tags_49 },
    { 0x7c9fa82c, SUC_lexicon_tags_17 },
    { 0xc7dfa82e, SUC_lexicon_tags_17 },
    { 0x21dfa82e, SUC_lexicon_tags_3 },
    { 0x18ffa830, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5b5fa832, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bbfa837, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88ffa83b, SUC_lexicon_tags_25 },
    { 0xc3bfa83c, SUC_lexicon_tags_7 },
    { 0xbedfa83c, SUC_lexicon_tags_9 },
    { 0x6a1fa83c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xdc5fa840, SUC_lexicon_tags_1 },
    { 0x8b3fa840, SUC_lexicon_tags_15 },
    { 0x6e3fa842, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4ffa849, SUC_lexicon_tags_37 },
    { 0x4f1fa84a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3fa84e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6f9fa850, SUC_lexicon_tags_15 },
    { 0x651fa850, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe67fa853, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91dfa859, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x58bfa85b, SUC_lexicon_tags_6 },
    { 0x64ffa85c, SUC_lexicon_tags_39 },
    { 0x51bfa85d, SUC_lexicon_tags_17 },
    { 0x2d3fa85e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x153fa864, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bfa868, SUC_lexicon_tags_15 },
    { 0x263fa868, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x5effa86c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x711fa870, SUC_lexicon_tags_8 },
    { 0x5bfa871, SUC_lexicon_tags_60 },
    { 0x52bfa872, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5effa878, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753fa87e, SUC_lexicon_tags_46 },
    { 0xecbfa87f, SUC_lexicon_tags_29 },
    { 0x145fa880, SUC_lexicon_tags_14 },
    { 0x699fa881, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49fa884, SUC_lexicon_tags_51 },
    { 0x46ffa885, SUC_lexicon_tags_41 },
    { 0x853fa885, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb7fa889, SUC_lexicon_tags_17 },
    { 0x71ffa88a, SUC_lexicon_tags_23 },
    { 0xa65fa88b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfa88e, SUC_lexicon_tags_27 },
    { 0x967fa88f, SUC_lexicon_tags_85 },
    { 0x9dbfa88f, SUC_lexicon_tags_25 },
    { 0x577fa88e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589fa896, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x59fa898, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3fa89d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4effa8a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7fa8a5, SUC_lexicon_tags_34 },
    { 0xe8ffa8a5, SUC_lexicon_tags_12 },
    { 0x293fa8a5, SUC_lexicon_tags_15 },
    { 0xb93fa8a8, SUC_lexicon_tags_17 },
    { 0xd5dfa8a8, SUC_lexicon_tags_11 },
    { 0xa03fa8aa, SUC_lexicon_tags_25 },
    { 0x82bfa8ab, SUC_lexicon_tags_28 },
    { 0x813fa8ab, SUC_lexicon_tags_0 },
    { 0xf0ffa8aa, SUC_lexicon_tags_16 },
    { 0xcf3fa8ab, SUC_lexicon_tags_12 },
    { 0xeb7fa8af, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa57fa8b1, SUC_lexicon_tags_2 },
    { 0x4abfa8b2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x5d9fa8b4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589fa8ba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2c3fa8bc, SUC_lexicon_tags_25 },
    { 0xc3ffa8bd, SUC_lexicon_tags_25 },
    { 0xbf1fa8be, SUC_lexicon_tags_27 },
    { 0xab7fa8bf, SUC_lexicon_tags_68 },
    { 0x4edfa8bf, SUC_lexicon_tags_27 },
    { 0xca3fa8bf, SUC_lexicon_tags_7 },
    { 0xdbfa8c1, SUC_lexicon_tags_37 },
    { 0xdb7fa8bc, SUC_lexicon_tags_25 },
    { 0x3fffa8c4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x735fa8c7, SUC_lexicon_tags_48 },
    { 0xcfbfa8c7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8dffa8ca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x947fa8cd, SUC_lexicon_tags_1 },
    { 0x873fa8cd, SUC_lexicon_tags_63 },
    { 0x649fa8cf, SUC_lexicon_tags_68 },
    { 0x8bbfa8d0, SUC_lexicon_tags_11 },
    { 0x189fa8d1, SUC_lexicon_tags_11 },
    { 0x6a3fa8d2, SUC_lexicon_tags_2 },
    { 0xa31fa8d3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1cffa8d6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5effa8da, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x921fa8dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x449fa8de, SUC_lexicon_tags_27 },
    { 0x29dfa8df, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffa8e5, SUC_lexicon_tags_55 },
    { 0x619fa8e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cbfa8eb, SUC_lexicon_tags_15 },
    { 0x897fa8ec, SUC_lexicon_tags_25 },
    { 0xcadfa8ed, SUC_lexicon_tags_77 },
    { 0x96dfa8ee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2ffa8f0, SUC_lexicon_tags_26 },
    { 0xb87fa8f0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xfc3fa8f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x503fa8f5, SUC_lexicon_tags_20 },
    { 0xd09fa8f6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dfa8fa, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7d1fa8fc, SUC_lexicon_tags_17 },
    { 0x161fa8fc, SUC_lexicon_tags_0 },
    { 0x269fa8fe, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbfa901, SUC_lexicon_tags_63 },
    { 0x4ebfa902, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfa906, SUC_lexicon_tags_17 },
    { 0xfadfa906, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe45fa909, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbb5fa90c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6a9fa90e, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bfa91a, SUC_lexicon_tags_64 },
    { 0x8c1fa91b, SUC_lexicon_tags_23 },
    { 0xb15fa91c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73bfa91e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23fa921, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x2b5fa923, SUC_lexicon_tags_2 },
    { 0x953fa923, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abfa927, SUC_lexicon_tags_27 },
    { 0x6c9fa928, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x523fa92a, SUC_lexicon_tags_88 },
    { 0xf11fa92b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6fffa92d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe07fa930, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedfa936, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x95fa939, SUC_lexicon_tags_49 },
    { 0xc4ffa939, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x7a7fa93c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2f3fa93e, SUC_lexicon_tags_11 },
    { 0x6ddfa93e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35fa942, SUC_lexicon_tags_11 },
    { 0x557fa942, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x451fa945, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x76dfa947, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd0bfa949, SUC_lexicon_tags_1 },
    { 0x9e3fa949, SUC_lexicon_tags_68 },
    { 0x75fa94b, SUC_lexicon_tags_17 },
    { 0x513fa94a, SUC_lexicon_tags_51 },
    { 0xe37fa94d, SUC_lexicon_tags_65 },
    { 0x6cbfa94e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x233fa952, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27fa958, SUC_lexicon_tags_1 },
    { 0x89dfa958, SUC_lexicon_tags_40 },
    { 0x2c3fa959, SUC_lexicon_tags_11 },
    { 0xbfdfa958, SUC_lexicon_tags_7 },
    { 0x877fa95a, SUC_lexicon_tags_2 },
    { 0xffdfa95d, SUC_lexicon_tags_17 },
    { 0xd3bfa95e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xbabfa960, SUC_lexicon_tags_43 },
    { 0xb7bfa960, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5fa964, SUC_lexicon_tags_47 },
    { 0xcb5fa965, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3e5fa967, SUC_lexicon_tags_31 },
    { 0x9ebfa968, SUC_lexicon_tags_23 },
    { 0x37fa968, SUC_lexicon_tags_25 },
    { 0xd09fa96a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9fa978, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe6dfa97a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd45fa97d, SUC_lexicon_tags_37 },
    { 0xd49fa97e, SUC_lexicon_tags_11 },
    { 0x245fa97d, SUC_lexicon_tags_19 },
    { 0x7bdfa97d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbc3fa982, SUC_lexicon_tags_5 },
    { 0xd0dfa982, SUC_lexicon_tags_2 },
    { 0x825fa984, SUC_lexicon_tags_12 },
    { 0x803fa985, SUC_lexicon_tags_11 },
    { 0x387fa983, SUC_lexicon_tags_7 },
    { 0x84bfa987, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe37fa989, SUC_lexicon_tags_2 },
    { 0x56ffa98a, SUC_lexicon_tags_27 },
    { 0x3b1fa98b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x32bfa98d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36bfa994, SUC_lexicon_tags_1049 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x309fa999, SUC_lexicon_tags_37 },
    { 0x71fa999, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93fa99e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5b1fa9a0, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929fa9a4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe49fa9a6, SUC_lexicon_tags_38 },
    { 0xf0ffa9a7, SUC_lexicon_tags_654 },
    { 0, NULL },
    { 0xa81fa9a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf87fa9ab, SUC_lexicon_tags_14 },
    { 0xdf9fa9ab, SUC_lexicon_tags_11 },
    { 0x3e5fa9ad, SUC_lexicon_tags_16 },
    { 0x80ffa9ad, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05fa9b6, SUC_lexicon_tags_28 },
    { 0x54ffa9b7, SUC_lexicon_tags_43 },
    { 0xf5fa9b8, SUC_lexicon_tags_7 },
    { 0x7f1fa9b9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9fa9bd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3fa9c0, SUC_lexicon_tags_1 },
    { 0xde9fa9c1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ffa9c6, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x349fa9cb, SUC_lexicon_tags_44 },
    { 0x283fa9cc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549fa9d5, SUC_lexicon_tags_12 },
    { 0x839fa9d5, SUC_lexicon_tags_47 },
    { 0xff1fa9d5, SUC_lexicon_tags_28 },
    { 0xacdfa9d7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bfa9dc, SUC_lexicon_tags_38 },
    { 0xa3ffa9dc, SUC_lexicon_tags_49 },
    { 0x179fa9dd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x4d5fa9e0, SUC_lexicon_tags_9 },
    { 0x899fa9e1, SUC_lexicon_tags_12 },
    { 0x9f5fa9e2, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacdfa9e6, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x29fa9e8, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdfa9eb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x657fa9f2, SUC_lexicon_tags_114 },
    { 0x507fa9f2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x319fa9f5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa19fa9f7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3fa9fa, SUC_lexicon_tags_83 },
    { 0x239fa9fb, SUC_lexicon_tags_38 },
    { 0xbe5fa9fb, SUC_lexicon_tags_7 },
    { 0xd9fa9fd, SUC_lexicon_tags_11 },
    { 0x55dfa9fe, SUC_lexicon_tags_17 },
    { 0xc3ffa9ff, SUC_lexicon_tags_1 },
    { 0xa53faa00, SUC_lexicon_tags_2 },
    { 0x49faa01, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1c3faa03, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x87bfaa06, SUC_lexicon_tags_7 },
    { 0xe21faa07, SUC_lexicon_tags_84 },
    { 0xc73faa07, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xffffaa0a, SUC_lexicon_tags_68 },
    { 0x75ffaa0b, SUC_lexicon_tags_17 },
    { 0xf3ffaa0a, SUC_lexicon_tags_17 },
    { 0xe09faa0d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x943faa0f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x763faa12, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbfaa18, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5faa1d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67faa27, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29faa2c, SUC_lexicon_tags_15 },
    { 0x479faa2d, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x117faa30, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0xf75faa33, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x915faa3a, SUC_lexicon_tags_15 },
    { 0xcebfaa3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dfaa3e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39faa43, SUC_lexicon_tags_16 },
    { 0x5e1faa43, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5faa48, SUC_lexicon_tags_31 },
    { 0x675faa49, SUC_lexicon_tags_87 },
    { 0xf99faa49, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb13faa4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x17dfaa4f, SUC_lexicon_tags_28 },
    { 0xd85faa50, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7faa53, SUC_lexicon_tags_38 },
    { 0xf69faa54, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ffaa57, SUC_lexicon_tags_0 },
    { 0x677faa58, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b3faa5a, SUC_lexicon_tags_60 },
    { 0x859faa5b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x243faa5e, SUC_lexicon_tags_27 },
    { 0xa1ffaa5f, SUC_lexicon_tags_17 },
    { 0x359faa60, SUC_lexicon_tags_15 },
    { 0x789faa61, SUC_lexicon_tags_7 },
    { 0xe5faa61, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x937faa66, SUC_lexicon_tags_11 },
    { 0xf87faa66, SUC_lexicon_tags_14 },
    { 0xc11faa67, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491faa70, SUC_lexicon_tags_88 },
    { 0x883faa71, SUC_lexicon_tags_17 },
    { 0x975faa72, SUC_lexicon_tags_34 },
    { 0xebbfaa72, SUC_lexicon_tags_7 },
    { 0x73bfaa74, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcc7faa76, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x83bfaa79, SUC_lexicon_tags_49 },
    { 0x77faa79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bfaa80, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddfaa83, SUC_lexicon_tags_48 },
    { 0x625faa84, SUC_lexicon_tags_3 },
    { 0x723faa85, SUC_lexicon_tags_15 },
    { 0x773faa85, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x783faa89, SUC_lexicon_tags_17 },
    { 0xae5faa8a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9faa8e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bfaa97, SUC_lexicon_tags_179 },
    { 0xb4dfaa98, SUC_lexicon_tags_106 },
    { 0xfc1faa97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x463faa9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5faaa0, SUC_lexicon_tags_63 },
    { 0x1a7faaa1, SUC_lexicon_tags_7 },
    { 0xff3faaa1, SUC_lexicon_tags_1 },
    { 0x6c5faaa2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9faaa7, SUC_lexicon_tags_37 },
    { 0x7a9faaa7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc83faab1, SUC_lexicon_tags_65 },
    { 0x48dfaab1, SUC_lexicon_tags_30 },
    { 0xa5dfaab3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1faaba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd13faabc, SUC_lexicon_tags_7 },
    { 0x6c7faabd, SUC_lexicon_tags_26 },
    { 0x8a1faabe, SUC_lexicon_tags_68 },
    { 0x9a3faabe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x47faac1, SUC_lexicon_tags_0 },
    { 0x7dffaac2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde7faacc, SUC_lexicon_tags_17 },
    { 0x61bfaacd, SUC_lexicon_tags_43 },
    { 0x3f1faacd, SUC_lexicon_tags_25 },
    { 0xf4ffaacf, SUC_lexicon_tags_68 },
    { 0x719faad0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xad1faad3, SUC_lexicon_tags_12 },
    { 0xdd9faad4, SUC_lexicon_tags_25 },
    { 0x507faad5, SUC_lexicon_tags_1 },
    { 0x9b3faad6, SUC_lexicon_tags_11 },
    { 0xe5dfaad7, SUC_lexicon_tags_1 },
    { 0x417faad6, SUC_lexicon_tags_15 },
    { 0xcf7faad9, SUC_lexicon_tags_2 },
    { 0x91faada, SUC_lexicon_tags_9 },
    { 0xf2bfaada, SUC_lexicon_tags_6 },
    { 0x537faadc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x317faadf, SUC_lexicon_tags_0 },
    { 0x593faadf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcbfaae6, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x55ffaae9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x30dfaaeb, SUC_lexicon_tags_2 },
    { 0xbdffaaeb, SUC_lexicon_tags_12 },
    { 0x3c7faaed, SUC_lexicon_tags_49 },
    { 0xf69faaee, SUC_lexicon_tags_6 },
    { 0x481faaef, SUC_lexicon_tags_61 },
    { 0x44dfaaf0, SUC_lexicon_tags_7 },
    { 0xe4dfaaee, SUC_lexicon_tags_17 },
    { 0xe6ffaaeb, SUC_lexicon_tags_25 },
    { 0x933faaed, SUC_lexicon_tags_7 },
    { 0x957faaf4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd65faaf9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf8ffaafb, SUC_lexicon_tags_2 },
    { 0x1a7faafc, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519fab00, SUC_lexicon_tags_56 },
    { 0x70dfab00, SUC_lexicon_tags_6 },
    { 0x417fab02, SUC_lexicon_tags_25 },
    { 0x477fab01, SUC_lexicon_tags_25 },
    { 0x97fab04, SUC_lexicon_tags_47 },
    { 0x71fab04, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99fab0c, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0xce5fab0e, SUC_lexicon_tags_0 },
    { 0x2f1fab0f, SUC_lexicon_tags_9 },
    { 0xc43fab0e, SUC_lexicon_tags_38 },
    { 0xe41fab11, SUC_lexicon_tags_17 },
    { 0x355fab0e, SUC_lexicon_tags_47 },
    { 0x6e3fab13, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6ffab18, SUC_lexicon_tags_37 },
    { 0x7bdfab19, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe83fab1d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363fab21, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323fab25, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ffab28, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dfab2e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7bfab32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x82dfab35, SUC_lexicon_tags_11 },
    { 0xf41fab35, SUC_lexicon_tags_25 },
    { 0xec9fab37, SUC_lexicon_tags_37 },
    { 0xcedfab37, SUC_lexicon_tags_1 },
    { 0x27fab37, SUC_lexicon_tags_27 },
    { 0x9abfab37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcbdfab3c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xceffab3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x515fab43, SUC_lexicon_tags_27 },
    { 0xae3fab44, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7fab49, SUC_lexicon_tags_7 },
    { 0xea3fab49, SUC_lexicon_tags_11 },
    { 0x73dfab4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9fab4e, SUC_lexicon_tags_4 },
    { 0x3ddfab4f, SUC_lexicon_tags_43 },
    { 0xaabfab4f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x89fab53, SUC_lexicon_tags_129 },
    { 0xa89fab53, SUC_lexicon_tags_28 },
    { 0x3f3fab55, SUC_lexicon_tags_2 },
    { 0x303fab56, SUC_lexicon_tags_15 },
    { 0x3cffab57, SUC_lexicon_tags_3 },
    { 0x27dfab58, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf17fab5a, SUC_lexicon_tags_19 },
    { 0x541fab5b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd67fab5d, SUC_lexicon_tags_12 },
    { 0x613fab5e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabfab63, SUC_lexicon_tags_16 },
    { 0x233fab63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ffab67, SUC_lexicon_tags_7 },
    { 0xdf7fab67, SUC_lexicon_tags_10 },
    { 0x3e1fab67, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5fab6d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1fab70, SUC_lexicon_tags_1 },
    { 0xc85fab71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3a1fab73, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf45fab75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643fab7a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x309fab7d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x733fab81, SUC_lexicon_tags_38 },
    { 0x1abfab82, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9fab85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x549fab87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffab8c, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93fab97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12dfab99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41fab9c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bfaba1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1bfaba3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73faba9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33fabb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x67bfabb5, SUC_lexicon_tags_17 },
    { 0xdfdfabb6, SUC_lexicon_tags_15 },
    { 0x549fabb7, SUC_lexicon_tags_15 },
    { 0x2b1fabb7, SUC_lexicon_tags_25 },
    { 0x2adfabb9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dfabbd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc59fabbf, SUC_lexicon_tags_30 },
    { 0xb2dfabbf, SUC_lexicon_tags_0 },
    { 0x21ffabc1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x221fabc3, SUC_lexicon_tags_31 },
    { 0xe89fabc3, SUC_lexicon_tags_11 },
    { 0xb37fabc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3d1fabc7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xde3fabca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x37dfabcc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11fabd0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bffabd6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbdfabdc, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d1fabe0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xba7fabe2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bfabe5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68bfabed, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x907fabf0, SUC_lexicon_tags_11 },
    { 0x70bfabf0, SUC_lexicon_tags_33 },
    { 0x891fabf1, SUC_lexicon_tags_60 },
    { 0x7bfabf3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x979fabf7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb9dfabf9, SUC_lexicon_tags_19 },
    { 0x351fabf9, SUC_lexicon_tags_45 },
    { 0x55fabfb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77fabff, SUC_lexicon_tags_1 },
    { 0x2dfac00, SUC_lexicon_tags_17 },
    { 0xdd9fac01, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd37fac05, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb3fac09, SUC_lexicon_tags_7 },
    { 0x3ebfac09, SUC_lexicon_tags_2 },
    { 0xa03fac0b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd97fac0d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb8dfac0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe57fac12, SUC_lexicon_tags_4 },
    { 0xd65fac13, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bfac15, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4c1fac17, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa47fac1c, SUC_lexicon_tags_17 },
    { 0x4c7fac1d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9fac23, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x20ffac25, SUC_lexicon_tags_28 },
    { 0x6f5fac26, SUC_lexicon_tags_28 },
    { 0xd77fac27, SUC_lexicon_tags_39 },
    { 0xa97fac28, SUC_lexicon_tags_63 },
    { 0xad3fac26, SUC_lexicon_tags_37 },
    { 0x535fac2a, SUC_lexicon_tags_38 },
    { 0x73ffac2b, SUC_lexicon_tags_17 },
    { 0x495fac2c, SUC_lexicon_tags_37 },
    { 0x2a5fac2a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4f5fac2f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9fac32, SUC_lexicon_tags_17 },
    { 0xfcbfac32, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x437fac36, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ffac3a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ffac3d, SUC_lexicon_tags_25 },
    { 0xb39fac3e, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x235fac40, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc43fac42, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3fac49, SUC_lexicon_tags_14 },
    { 0x37bfac4a, SUC_lexicon_tags_2 },
    { 0x369fac4b, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1fac4e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xa63fac50, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xed9fac52, SUC_lexicon_tags_61 },
    { 0x79bfac52, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fac58, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fac5d, SUC_lexicon_tags_27 },
    { 0xf9dfac5d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x271fac60, SUC_lexicon_tags_4 },
    { 0x16bfac61, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd55fac63, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1abfac69, SUC_lexicon_tags_25 },
    { 0x7c9fac6a, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x573fac6d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdfac71, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x989fac74, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5fac77, SUC_lexicon_tags_7 },
    { 0xcdffac78, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaadfac7b, SUC_lexicon_tags_31 },
    { 0x8adfac7c, SUC_lexicon_tags_32 },
    { 0x717fac7b, SUC_lexicon_tags_47 },
    { 0x31bfac7c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x745fac80, SUC_lexicon_tags_17 },
    { 0x915fac81, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x3b5fac83, SUC_lexicon_tags_8 },
    { 0xca7fac84, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x781fac86, SUC_lexicon_tags_14 },
    { 0x3b5fac86, SUC_lexicon_tags_27 },
    { 0x10ffac87, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bfac8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x193fac91, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2e9fac93, SUC_lexicon_tags_117 },
    { 0x691fac94, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbfac98, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x73dfac9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33fac9f, SUC_lexicon_tags_156 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529faca6, SUC_lexicon_tags_63 },
    { 0xa13faca7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedffacab, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x999facad, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x23bfacb0, SUC_lexicon_tags_13 },
    { 0x707facb1, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3facb6, SUC_lexicon_tags_29 },
    { 0x3f1facb7, SUC_lexicon_tags_2 },
    { 0x379facb7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x5d7facba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa79facbf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5dfacc5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bfaccc, SUC_lexicon_tags_25 },
    { 0x58ffaccc, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x911facd4, SUC_lexicon_tags_995 },
    { 0, NULL },
    { 0, NULL },
    { 0x413facd7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x663facda, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x207face0, SUC_lexicon_tags_1 },
    { 0xea5face0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x2d7face3, SUC_lexicon_tags_60 },
    { 0x9bface4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5facea, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571facf0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe4bfacf2, SUC_lexicon_tags_12 },
    { 0xdfdfacf3, SUC_lexicon_tags_32 },
    { 0x9d5facf4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbfacf7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9dfacfd, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xb0ffacff, SUC_lexicon_tags_2 },
    { 0xf3facff, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873fad09, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x79fad0b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x125fad0f, SUC_lexicon_tags_2 },
    { 0x2edfad10, SUC_lexicon_tags_17 },
    { 0x79dfad10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7fad17, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x747fad19, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbfad1c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5fad1f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5fad22, SUC_lexicon_tags_23 },
    { 0x28bfad23, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8dffad25, SUC_lexicon_tags_10 },
    { 0x1c7fad26, SUC_lexicon_tags_1 },
    { 0x287fad27, SUC_lexicon_tags_5 },
    { 0x413fad27, SUC_lexicon_tags_14 },
    { 0xa7bfad29, SUC_lexicon_tags_31 },
    { 0x7d1fad29, SUC_lexicon_tags_1 },
    { 0x647fad2b, SUC_lexicon_tags_66 },
    { 0x45bfad28, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf89fad30, SUC_lexicon_tags_4 },
    { 0x7edfad30, SUC_lexicon_tags_25 },
    { 0xc3dfad30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa79fad34, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957fad39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dfad3d, SUC_lexicon_tags_112 },
    { 0xdd5fad3e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb03fad42, SUC_lexicon_tags_49 },
    { 0x5d1fad42, SUC_lexicon_tags_37 },
    { 0xb87fad43, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x21fad46, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x323fad48, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x98dfad4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dfad50, SUC_lexicon_tags_4 },
    { 0xb51fad51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xda1fad54, SUC_lexicon_tags_16 },
    { 0x807fad55, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9fad63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d5fad6d, SUC_lexicon_tags_37 },
    { 0xf81fad6e, SUC_lexicon_tags_25 },
    { 0x75bfad6f, SUC_lexicon_tags_3 },
    { 0x865fad70, SUC_lexicon_tags_43 },
    { 0xe07fad71, SUC_lexicon_tags_9 },
    { 0xb05fad70, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x105fad74, SUC_lexicon_tags_39 },
    { 0x723fad74, SUC_lexicon_tags_16 },
    { 0x783fad75, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8d9fad78, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbc1fad7a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51fad80, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31fad8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x273fad94, SUC_lexicon_tags_14 },
    { 0x351fad94, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bfad98, SUC_lexicon_tags_12 },
    { 0x8cbfad99, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x2abfad9b, SUC_lexicon_tags_30 },
    { 0x20dfad9b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9fada1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf67fada4, SUC_lexicon_tags_21 },
    { 0xb8bfada4, SUC_lexicon_tags_37 },
    { 0x577fada6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x529fadaa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x543fadac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x719fadae, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x6dbfadb0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x5effadb2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd1ffadb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5fadb7, SUC_lexicon_tags_32 },
    { 0x1edfadb8, SUC_lexicon_tags_17 },
    { 0x9c3fadb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4f9fadbb, SUC_lexicon_tags_7 },
    { 0x99bfadbc, SUC_lexicon_tags_1 },
    { 0x1c3fadbd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f3fadc0, SUC_lexicon_tags_15 },
    { 0x6d7fadc1, SUC_lexicon_tags_79 },
    { 0xf1ffadc1, SUC_lexicon_tags_31 },
    { 0x70bfadc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfadc9, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x765fadcb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc67fadcd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xb73fadd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c1fadd3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x743fadd5, SUC_lexicon_tags_19 },
    { 0x1dbfadd6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281fadde, SUC_lexicon_tags_19 },
    { 0xbfffadde, SUC_lexicon_tags_23 },
    { 0xa49fade0, SUC_lexicon_tags_13 },
    { 0xb81fade1, SUC_lexicon_tags_37 },
    { 0xea1fade2, SUC_lexicon_tags_31 },
    { 0x4bbfade3, SUC_lexicon_tags_25 },
    { 0x7abfade4, SUC_lexicon_tags_37 },
    { 0x799fadde, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x567fade7, SUC_lexicon_tags_56 },
    { 0xf23fade8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adfadec, SUC_lexicon_tags_91 },
    { 0x73ffaded, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62dfadf1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf8ffadf3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763fadf7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7adfadf9, SUC_lexicon_tags_11 },
    { 0x45ffadfa, SUC_lexicon_tags_85 },
    { 0x22bfadfa, SUC_lexicon_tags_29 },
    { 0x52ffadf9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe29fadfe, SUC_lexicon_tags_5 },
    { 0x873fadff, SUC_lexicon_tags_60 },
    { 0x269fadfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6ffae03, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x197fae05, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdfae0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf91fae0c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x711fae0e, SUC_lexicon_tags_31 },
    { 0x6adfae0f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x237fae17, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b7fae1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddfae20, SUC_lexicon_tags_2 },
    { 0x51dfae21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9fae28, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5fae2e, SUC_lexicon_tags_17 },
    { 0x7fdfae2f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x481fae34, SUC_lexicon_tags_12 },
    { 0xa2ffae35, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527fae3d, SUC_lexicon_tags_12 },
    { 0xe4bfae3e, SUC_lexicon_tags_49 },
    { 0xcdffae3f, SUC_lexicon_tags_26 },
    { 0xc1bfae3f, SUC_lexicon_tags_17 },
    { 0x75bfae3e, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0x6b1fae44, SUC_lexicon_tags_11 },
    { 0x3a7fae45, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd9dfae47, SUC_lexicon_tags_15 },
    { 0xd1dfae48, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77fae4d, SUC_lexicon_tags_55 },
    { 0xb29fae4e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x5dffae50, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x603fae52, SUC_lexicon_tags_0 },
    { 0xcd1fae52, SUC_lexicon_tags_25 },
    { 0xcedfae52, SUC_lexicon_tags_15 },
    { 0x3fdfae55, SUC_lexicon_tags_27 },
    { 0x5f7fae52, SUC_lexicon_tags_25 },
    { 0x66bfae57, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf59fae5c, SUC_lexicon_tags_9 },
    { 0x4ffae5c, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x34ffae5f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfae62, SUC_lexicon_tags_26 },
    { 0x94ffae62, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb29fae67, SUC_lexicon_tags_1 },
    { 0x26bfae68, SUC_lexicon_tags_47 },
    { 0x8bdfae67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xf9dfae6b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d9fae6d, SUC_lexicon_tags_14 },
    { 0x611fae6e, SUC_lexicon_tags_25 },
    { 0xb27fae6d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3fae72, SUC_lexicon_tags_37 },
    { 0xeb3fae73, SUC_lexicon_tags_18 },
    { 0x8edfae74, SUC_lexicon_tags_23 },
    { 0xe19fae75, SUC_lexicon_tags_62 },
    { 0xf1ffae75, SUC_lexicon_tags_56 },
    { 0xbebfae72, SUC_lexicon_tags_38 },
    { 0xb81fae78, SUC_lexicon_tags_68 },
    { 0x817fae73, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x261fae7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7fae81, SUC_lexicon_tags_49 },
    { 0x193fae81, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69fae8b, SUC_lexicon_tags_12 },
    { 0xb93fae8b, SUC_lexicon_tags_7 },
    { 0x96bfae8b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdfae92, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157fae96, SUC_lexicon_tags_25 },
    { 0xca9fae96, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dfae9b, SUC_lexicon_tags_0 },
    { 0xc4ffae9b, SUC_lexicon_tags_114 },
    { 0x769fae9d, SUC_lexicon_tags_17 },
    { 0x167fae9d, SUC_lexicon_tags_8 },
    { 0x519fae9f, SUC_lexicon_tags_4 },
    { 0x22ffaea0, SUC_lexicon_tags_10 },
    { 0x4dfaea1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2a7faea3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xeedfaea5, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x141faea8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3faeac, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397faeb3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a1faeb8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4adfaeba, SUC_lexicon_tags_27 },
    { 0x30dfaeba, SUC_lexicon_tags_59 },
    { 0x41ffaebc, SUC_lexicon_tags_11 },
    { 0xbf1faebd, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225faec7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb5faecb, SUC_lexicon_tags_32 },
    { 0xb1dfaecc, SUC_lexicon_tags_129 },
    { 0xe45faecc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7faed3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bfaed9, SUC_lexicon_tags_25 },
    { 0x32ffaeda, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bffaedd, SUC_lexicon_tags_28 },
    { 0xe13faede, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x77dfaee1, SUC_lexicon_tags_165 },
    { 0x16ffaee2, SUC_lexicon_tags_38 },
    { 0xd3bfaee3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb2ffaee5, SUC_lexicon_tags_37 },
    { 0xf69faee6, SUC_lexicon_tags_6 },
    { 0xfc7faee7, SUC_lexicon_tags_29 },
    { 0x3ebfaee5, SUC_lexicon_tags_9 },
    { 0x95dfaee9, SUC_lexicon_tags_7 },
    { 0xcdbfaeea, SUC_lexicon_tags_21 },
    { 0x46bfaeea, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xaa9faeed, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x195faeef, SUC_lexicon_tags_34 },
    { 0x5e1faef0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x429faef3, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xa9bfaef5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1faef8, SUC_lexicon_tags_19 },
    { 0x909faef9, SUC_lexicon_tags_27 },
    { 0xc11faefa, SUC_lexicon_tags_14 },
    { 0x84ffaef9, SUC_lexicon_tags_28 },
    { 0x6d7faef8, SUC_lexicon_tags_7 },
    { 0x7c1faefb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb19faf02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe9faf06, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x665faf0e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb4ffaf10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x67ffaf12, SUC_lexicon_tags_17 },
    { 0xc8bfaf13, SUC_lexicon_tags_11 },
    { 0x2f1faf14, SUC_lexicon_tags_43 },
    { 0x94ffaf15, SUC_lexicon_tags_26 },
    { 0xd0bfaf14, SUC_lexicon_tags_7 },
    { 0x3fbfaf16, SUC_lexicon_tags_17 },
    { 0xcb5faf18, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb7faf1b, SUC_lexicon_tags_26 },
    { 0x16bfaf1c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95bfaf21, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xeeffaf24, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31faf28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x76ffaf2a, SUC_lexicon_tags_17 },
    { 0xcbfaf2b, SUC_lexicon_tags_32 },
    { 0x4a9faf2c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdeffaf2e, SUC_lexicon_tags_11 },
    { 0x1c1faf2f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fbfaf34, SUC_lexicon_tags_45 },
    { 0x4c3faf35, SUC_lexicon_tags_25 },
    { 0x519faf35, SUC_lexicon_tags_12 },
    { 0x43ffaf37, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf41faf3b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xae3faf3d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7faf42, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x647faf45, SUC_lexicon_tags_43 },
    { 0x209faf46, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x757faf4a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25faf51, SUC_lexicon_tags_25 },
    { 0x139faf52, SUC_lexicon_tags_11 },
    { 0x8cbfaf53, SUC_lexicon_tags_12 },
    { 0xef1faf54, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3faf57, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59faf5a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x183faf5c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbd3faf5e, SUC_lexicon_tags_25 },
    { 0xe0bfaf5f, SUC_lexicon_tags_37 },
    { 0x58bfaf5f, SUC_lexicon_tags_7 },
    { 0x521faf61, SUC_lexicon_tags_25 },
    { 0x4f7faf62, SUC_lexicon_tags_33 },
    { 0x1c1faf63, SUC_lexicon_tags_27 },
    { 0x6edfaf61, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xc01faf66, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523faf6e, SUC_lexicon_tags_2 },
    { 0xdb7faf6f, SUC_lexicon_tags_10 },
    { 0xf75faf6f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7faf73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dfaf76, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebfaf79, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7faf80, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x383faf86, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bffaf8a, SUC_lexicon_tags_1 },
    { 0xcdffaf8b, SUC_lexicon_tags_17 },
    { 0x64ffaf8c, SUC_lexicon_tags_18 },
    { 0x6f9faf8d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x427faf90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5bdfaf94, SUC_lexicon_tags_1 },
    { 0x3c7faf95, SUC_lexicon_tags_23 },
    { 0xfd9faf95, SUC_lexicon_tags_0 },
    { 0xc15faf97, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb3ffaf9a, SUC_lexicon_tags_17 },
    { 0xb97faf9a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x7f1faf9e, SUC_lexicon_tags_1 },
    { 0x40dfaf9e, SUC_lexicon_tags_30 },
    { 0xbc5fafa0, SUC_lexicon_tags_0 },
    { 0x229fafa1, SUC_lexicon_tags_0 },
    { 0x2dbfafa2, SUC_lexicon_tags_17 },
    { 0xab1fafa2, SUC_lexicon_tags_25 },
    { 0x5d3fafa3, SUC_lexicon_tags_68 },
    { 0x78bfafa0, SUC_lexicon_tags_17 },
    { 0x36bfafa6, SUC_lexicon_tags_6 },
    { 0x495fafa5, SUC_lexicon_tags_23 },
    { 0x393faf9f, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdfafad, SUC_lexicon_tags_47 },
    { 0xc91fafad, SUC_lexicon_tags_7 },
    { 0x8ddfafaf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x42ffafb2, SUC_lexicon_tags_14 },
    { 0x3e5fafb2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x967fafb5, SUC_lexicon_tags_17 },
    { 0x93fafb6, SUC_lexicon_tags_25 },
    { 0x2ffafb5, SUC_lexicon_tags_22 },
    { 0xc8dfafb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d3fafba, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x10ffafc6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ffafcc, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7ffafd0, SUC_lexicon_tags_15 },
    { 0x703fafd0, SUC_lexicon_tags_15 },
    { 0x1c1fafd2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bfafd7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593fafdd, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7fafe0, SUC_lexicon_tags_37 },
    { 0x885fafe0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x94ffafe3, SUC_lexicon_tags_15 },
    { 0x45fafe4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x869fafe7, SUC_lexicon_tags_15 },
    { 0x963fafe8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79fafeb, SUC_lexicon_tags_45 },
    { 0xbb5fafec, SUC_lexicon_tags_15 },
    { 0xa97fafed, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3faff0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xbebfaff2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xd97faff4, SUC_lexicon_tags_15 },
    { 0x7abfaff4, SUC_lexicon_tags_12 },
    { 0xb6dfaff6, SUC_lexicon_tags_38 },
    { 0x495faff5, SUC_lexicon_tags_11 },
    { 0x52dfaff8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x491faffa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5faffd, SUC_lexicon_tags_30 },
    { 0xe3dfaffe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbfb006, SUC_lexicon_tags_4 },
    { 0x123fb006, SUC_lexicon_tags_37 },
    { 0xddbfb008, SUC_lexicon_tags_12 },
    { 0x945fb009, SUC_lexicon_tags_37 },
    { 0x5e7fb00a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ffb00e, SUC_lexicon_tags_17 },
    { 0xfdbfb00f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe65fb011, SUC_lexicon_tags_47 },
    { 0x651fb011, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57fb017, SUC_lexicon_tags_12 },
    { 0x1cdfb017, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1bfb01a, SUC_lexicon_tags_19 },
    { 0xf31fb01a, SUC_lexicon_tags_19 },
    { 0xbe9fb01c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x227fb026, SUC_lexicon_tags_48 },
    { 0x9b1fb027, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3fb02b, SUC_lexicon_tags_49 },
    { 0x529fb02b, SUC_lexicon_tags_15 },
    { 0x929fb02d, SUC_lexicon_tags_85 },
    { 0xa1fb02e, SUC_lexicon_tags_1 },
    { 0x157fb02e, SUC_lexicon_tags_15 },
    { 0x377fb030, SUC_lexicon_tags_37 },
    { 0xc83fb02f, SUC_lexicon_tags_47 },
    { 0x271fb032, SUC_lexicon_tags_7 },
    { 0xa4dfb030, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x113fb037, SUC_lexicon_tags_21 },
    { 0x803fb037, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xe0dfb03a, SUC_lexicon_tags_67 },
    { 0x8b5fb03a, SUC_lexicon_tags_38 },
    { 0x873fb03a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761fb042, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc67fb045, SUC_lexicon_tags_37 },
    { 0x515fb046, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fb04a, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcaffb04e, SUC_lexicon_tags_0 },
    { 0x2c5fb04f, SUC_lexicon_tags_45 },
    { 0x9e3fb050, SUC_lexicon_tags_17 },
    { 0xaa5fb051, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x3d9fb053, SUC_lexicon_tags_45 },
    { 0x4e3fb053, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159fb060, SUC_lexicon_tags_11 },
    { 0xcbdfb060, SUC_lexicon_tags_37 },
    { 0x2d9fb062, SUC_lexicon_tags_16 },
    { 0xffdfb061, SUC_lexicon_tags_12 },
    { 0xf67fb064, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83fb069, SUC_lexicon_tags_34 },
    { 0xbb5fb069, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7fb06f, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365fb073, SUC_lexicon_tags_2 },
    { 0x3d3fb074, SUC_lexicon_tags_434 },
    { 0xfe5fb075, SUC_lexicon_tags_37 },
    { 0x86bfb076, SUC_lexicon_tags_6 },
    { 0x2affb074, SUC_lexicon_tags_63 },
    { 0xccffb077, SUC_lexicon_tags_21 },
    { 0xb4bfb079, SUC_lexicon_tags_2 },
    { 0xe21fb073, SUC_lexicon_tags_10 },
    { 0xb29fb07b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdfb080, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf23fb082, SUC_lexicon_tags_23 },
    { 0xa5fb083, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbfb092, SUC_lexicon_tags_15 },
    { 0xc75fb093, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1fb09b, SUC_lexicon_tags_25 },
    { 0x1c7fb09c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367fb0a4, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ffb0a8, SUC_lexicon_tags_47 },
    { 0xebffb0a9, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x501fb0b0, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x6a7fb0b2, SUC_lexicon_tags_17 },
    { 0x625fb0b3, SUC_lexicon_tags_25 },
    { 0xc3ffb0b4, SUC_lexicon_tags_36 },
    { 0x253fb0b3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3fb0ba, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3fb0bd, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2dfb0c1, SUC_lexicon_tags_15 },
    { 0x839fb0c2, SUC_lexicon_tags_25 },
    { 0x155fb0c1, SUC_lexicon_tags_17 },
    { 0x685fb0c4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x17ffb0c6, SUC_lexicon_tags_31 },
    { 0xb37fb0c6, SUC_lexicon_tags_12 },
    { 0xde5fb0c8, SUC_lexicon_tags_69 },
    { 0xa7bfb0c8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x369fb0cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe17fb0ce, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05fb0d3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x573fb0d5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dfb0d9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x21ffb0db, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4effb0dd, SUC_lexicon_tags_12 },
    { 0xe29fb0de, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3fb0e7, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fb0ed, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x9cbfb0ef, SUC_lexicon_tags_11 },
    { 0xc2bfb0f0, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dfb0f5, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9fb0f8, SUC_lexicon_tags_37 },
    { 0xa03fb0f9, SUC_lexicon_tags_37 },
    { 0x2bfb0fa, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x3b1fb0fc, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f3fb0ff, SUC_lexicon_tags_27 },
    { 0x9a1fb100, SUC_lexicon_tags_109 },
    { 0x6d7fb100, SUC_lexicon_tags_37 },
    { 0x44dfb102, SUC_lexicon_tags_7 },
    { 0xa8ffb103, SUC_lexicon_tags_65 },
    { 0xdb7fb102, SUC_lexicon_tags_2 },
    { 0x577fb100, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x64ffb108, SUC_lexicon_tags_15 },
    { 0xa4ffb109, SUC_lexicon_tags_63 },
    { 0xab7fb10a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x33fb10d, SUC_lexicon_tags_7 },
    { 0xe07fb10e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3fb116, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xdf7fb118, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccffb11f, SUC_lexicon_tags_27 },
    { 0x6b1fb120, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x15ffb122, SUC_lexicon_tags_63 },
    { 0x82bfb122, SUC_lexicon_tags_8 },
    { 0x4c1fb123, SUC_lexicon_tags_19 },
    { 0xdb1fb123, SUC_lexicon_tags_47 },
    { 0xbcdfb122, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x841fb128, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebfb12c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5effb133, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xecbfb135, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3fb139, SUC_lexicon_tags_2 },
    { 0x2f7fb13a, SUC_lexicon_tags_23 },
    { 0x33bfb13b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fb13f, SUC_lexicon_tags_14 },
    { 0xe0dfb140, SUC_lexicon_tags_2 },
    { 0xc7bfb13f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39ffb145, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5fb148, SUC_lexicon_tags_37 },
    { 0x19ffb149, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xa77fb14b, SUC_lexicon_tags_27 },
    { 0x5f9fb14c, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fb151, SUC_lexicon_tags_60 },
    { 0x215fb152, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbfb158, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x711fb15b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x10dfb15d, SUC_lexicon_tags_607 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235fb161, SUC_lexicon_tags_15 },
    { 0x677fb161, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491fb168, SUC_lexicon_tags_45 },
    { 0x369fb169, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa17fb16c, SUC_lexicon_tags_2 },
    { 0x1f5fb16d, SUC_lexicon_tags_12 },
    { 0x7c1fb16e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7fb176, SUC_lexicon_tags_2 },
    { 0x453fb176, SUC_lexicon_tags_31 },
    { 0x9b7fb178, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93fb17c, SUC_lexicon_tags_38 },
    { 0xcdfb17d, SUC_lexicon_tags_60 },
    { 0xdf5fb17e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535fb182, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x345fb184, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d9fb18a, SUC_lexicon_tags_14 },
    { 0x7bffb18a, SUC_lexicon_tags_37 },
    { 0x5abfb18a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c3fb18f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x48bfb196, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ffb199, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7fb19e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441fb1a9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5fb1ae, SUC_lexicon_tags_25 },
    { 0xd41fb1af, SUC_lexicon_tags_33 },
    { 0x993fb1ae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x541fb1b2, SUC_lexicon_tags_25 },
    { 0xf7fb1b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e1fb1b5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x439fb1bd, SUC_lexicon_tags_15 },
    { 0x541fb1be, SUC_lexicon_tags_11 },
    { 0x16ffb1bf, SUC_lexicon_tags_25 },
    { 0xe33fb1bf, SUC_lexicon_tags_29 },
    { 0xb87fb1c0, SUC_lexicon_tags_12 },
    { 0x33dfb1c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xdfdfb1c4, SUC_lexicon_tags_6 },
    { 0x9d3fb1c5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8b3fb1c7, SUC_lexicon_tags_17 },
    { 0x4f1fb1c8, SUC_lexicon_tags_16 },
    { 0x40dfb1c9, SUC_lexicon_tags_17 },
    { 0x56bfb1ca, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd25fb1cc, SUC_lexicon_tags_98 },
    { 0x401fb1cd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xb59fb1d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc5ffb1d2, SUC_lexicon_tags_37 },
    { 0xd33fb1d3, SUC_lexicon_tags_23 },
    { 0x725fb1d4, SUC_lexicon_tags_29 },
    { 0x447fb1d2, SUC_lexicon_tags_49 },
    { 0x3a3fb1d6, SUC_lexicon_tags_4 },
    { 0x83bfb1d7, SUC_lexicon_tags_85 },
    { 0x7b9fb1d6, SUC_lexicon_tags_2 },
    { 0xfc5fb1d8, SUC_lexicon_tags_11 },
    { 0xf7dfb1d8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x69ffb1dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ffb1e0, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315fb1e4, SUC_lexicon_tags_1 },
    { 0x201fb1e5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69bfb1ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa5fb1ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b3fb1f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb17fb1f5, SUC_lexicon_tags_10 },
    { 0x97bfb1f6, SUC_lexicon_tags_17 },
    { 0x1c7fb1f7, SUC_lexicon_tags_7 },
    { 0x627fb1f8, SUC_lexicon_tags_15 },
    { 0xf2bfb1f9, SUC_lexicon_tags_38 },
    { 0x989fb1fa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x25ffb200, SUC_lexicon_tags_100 },
    { 0xa61fb201, SUC_lexicon_tags_11 },
    { 0xe6bfb202, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd5fb206, SUC_lexicon_tags_75 },
    { 0x77bfb206, SUC_lexicon_tags_23 },
    { 0x155fb207, SUC_lexicon_tags_2 },
    { 0xf87fb208, SUC_lexicon_tags_39 },
    { 0x885fb208, SUC_lexicon_tags_85 },
    { 0x5d1fb20b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa93fb20d, SUC_lexicon_tags_12 },
    { 0xefffb20d, SUC_lexicon_tags_12 },
    { 0x397fb20e, SUC_lexicon_tags_44 },
    { 0xd49fb210, SUC_lexicon_tags_8 },
    { 0x3c3fb211, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e3fb215, SUC_lexicon_tags_15 },
    { 0xcdfb216, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x595fb219, SUC_lexicon_tags_29 },
    { 0xc1fb21a, SUC_lexicon_tags_170 },
    { 0xa13fb21b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3cffb21e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21dfb222, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ffb227, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x56ffb22a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5ffb22c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517fb231, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x4cdfb233, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5fb23a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84dfb23d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x929fb240, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11fb248, SUC_lexicon_tags_60 },
    { 0x775fb249, SUC_lexicon_tags_25 },
    { 0xd83fb24a, SUC_lexicon_tags_15 },
    { 0x389fb24b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55fb24f, SUC_lexicon_tags_48 },
    { 0xb91fb250, SUC_lexicon_tags_0 },
    { 0x40bfb251, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9fb254, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77fb259, SUC_lexicon_tags_17 },
    { 0x1edfb259, SUC_lexicon_tags_7 },
    { 0x19fb25b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x215fb25e, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7fb261, SUC_lexicon_tags_15 },
    { 0xcf7fb262, SUC_lexicon_tags_39 },
    { 0xb6ffb261, SUC_lexicon_tags_37 },
    { 0xb71fb262, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xadffb267, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83ffb26d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ffb276, SUC_lexicon_tags_17 },
    { 0xf9bfb276, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x379fb279, SUC_lexicon_tags_25 },
    { 0x15bfb27a, SUC_lexicon_tags_17 },
    { 0x21ffb27b, SUC_lexicon_tags_17 },
    { 0x65dfb27c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xad3fb27f, SUC_lexicon_tags_20 },
    { 0xb2ffb27f, SUC_lexicon_tags_25 },
    { 0x8e7fb280, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96dfb288, SUC_lexicon_tags_17 },
    { 0xa2bfb289, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79ffb28d, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47dfb291, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb2bfb293, SUC_lexicon_tags_17 },
    { 0x331fb293, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201fb29a, SUC_lexicon_tags_25 },
    { 0x4f1fb29b, SUC_lexicon_tags_17 },
    { 0x4bbfb29c, SUC_lexicon_tags_1 },
    { 0x10bfb29b, SUC_lexicon_tags_30 },
    { 0x625fb29e, SUC_lexicon_tags_8 },
    { 0x1a3fb29d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x71fb2a1, SUC_lexicon_tags_60 },
    { 0x321fb2a2, SUC_lexicon_tags_1 },
    { 0x83ffb2a1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf4bfb2a5, SUC_lexicon_tags_29 },
    { 0xf3bfb2a6, SUC_lexicon_tags_25 },
    { 0xfbffb2a6, SUC_lexicon_tags_49 },
    { 0x3a9fb2a5, SUC_lexicon_tags_1 },
    { 0xc15fb2a9, SUC_lexicon_tags_25 },
    { 0x45dfb2a7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x395fb2ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523fb2b2, SUC_lexicon_tags_33 },
    { 0x329fb2b3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xba1fb2b6, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x6d3fb2b8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc3fb2bd, SUC_lexicon_tags_2 },
    { 0xe33fb2bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09fb2c6, SUC_lexicon_tags_17 },
    { 0xa89fb2c6, SUC_lexicon_tags_25 },
    { 0xc7bfb2c8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9c7fb2ca, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x377fb2cc, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111fb2d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2ffb2d9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cdfb2e2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7fb2e7, SUC_lexicon_tags_8 },
    { 0xe29fb2e8, SUC_lexicon_tags_47 },
    { 0x57ffb2e7, SUC_lexicon_tags_37 },
    { 0x971fb2e7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf63fb2ee, SUC_lexicon_tags_18 },
    { 0x541fb2ef, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x223fb2f1, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9fb2f5, SUC_lexicon_tags_23 },
    { 0x355fb2f5, SUC_lexicon_tags_23 },
    { 0x71dfb2f5, SUC_lexicon_tags_82 },
    { 0xce5fb2f8, SUC_lexicon_tags_12 },
    { 0xd11fb2f6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01fb2fd, SUC_lexicon_tags_14 },
    { 0xfedfb2fe, SUC_lexicon_tags_12 },
    { 0x107fb2ff, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x9f3fb301, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdd5fb303, SUC_lexicon_tags_27 },
    { 0x5d7fb304, SUC_lexicon_tags_60 },
    { 0x645fb305, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfffb30a, SUC_lexicon_tags_7 },
    { 0x687fb30a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1fb310, SUC_lexicon_tags_77 },
    { 0x45fb311, SUC_lexicon_tags_23 },
    { 0x9cbfb311, SUC_lexicon_tags_38 },
    { 0x1bdfb313, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65ffb31d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ffb323, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcadfb329, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x923fb32c, SUC_lexicon_tags_19 },
    { 0x261fb32d, SUC_lexicon_tags_47 },
    { 0x865fb32d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ffb333, SUC_lexicon_tags_15 },
    { 0xe1bfb334, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe57fb338, SUC_lexicon_tags_37 },
    { 0xad7fb339, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7fb33e, SUC_lexicon_tags_0 },
    { 0x681fb33f, SUC_lexicon_tags_1 },
    { 0x993fb340, SUC_lexicon_tags_33 },
    { 0xeaffb341, SUC_lexicon_tags_47 },
    { 0x755fb341, SUC_lexicon_tags_37 },
    { 0x56ffb33f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd21fb345, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65dfb34a, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x421fb34d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301fb351, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9effb35b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1c7fb35d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x86dfb360, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x521fb363, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xc29fb366, SUC_lexicon_tags_13 },
    { 0x1f7fb367, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x91ffb36a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x88bfb36c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x99dfb36f, SUC_lexicon_tags_37 },
    { 0xb9dfb370, SUC_lexicon_tags_12 },
    { 0x51dfb371, SUC_lexicon_tags_43 },
    { 0x675fb372, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x357fb374, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x427fb376, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x415fb379, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb7fb37b, SUC_lexicon_tags_37 },
    { 0xe57fb37c, SUC_lexicon_tags_10 },
    { 0x20ffb37d, SUC_lexicon_tags_12 },
    { 0x417fb37d, SUC_lexicon_tags_12 },
    { 0xad9fb37e, SUC_lexicon_tags_0 },
    { 0x6effb37d, SUC_lexicon_tags_27 },
    { 0x57bfb381, SUC_lexicon_tags_38 },
    { 0x26ffb382, SUC_lexicon_tags_19 },
    { 0xfb1fb383, SUC_lexicon_tags_19 },
    { 0xf27fb37b, SUC_lexicon_tags_0 },
    { 0xdb9fb384, SUC_lexicon_tags_29 },
    { 0xe23fb386, SUC_lexicon_tags_15 },
    { 0xa9fb386, SUC_lexicon_tags_47 },
    { 0x2f3fb388, SUC_lexicon_tags_43 },
    { 0xe03fb383, SUC_lexicon_tags_6 },
    { 0x227fb38a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1fb392, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xdb1fb395, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf59fb397, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdfb39f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9fb3a3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bffb3a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fdfb3af, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xfaffb3b1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xac1fb3b3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfffb3bf, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0x51bfb3c1, SUC_lexicon_tags_17 },
    { 0x789fb3c1, SUC_lexicon_tags_38 },
    { 0x49bfb3c2, SUC_lexicon_tags_7 },
    { 0x1ddfb3c1, SUC_lexicon_tags_55 },
    { 0x715fb3c1, SUC_lexicon_tags_39 },
    { 0xb03fb3c6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51fb3ca, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7fb3cd, SUC_lexicon_tags_27 },
    { 0x273fb3cd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x593fb3d2, SUC_lexicon_tags_17 },
    { 0x447fb3d2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xfb5fb3d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6dfb3d7, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4affb3df, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x5c5fb3e1, SUC_lexicon_tags_7 },
    { 0xcc9fb3e2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa6ffb3e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x45dfb3e6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf83fb3e8, SUC_lexicon_tags_15 },
    { 0x82dfb3e8, SUC_lexicon_tags_25 },
    { 0x4ddfb3ea, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0xecffb3ec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa35fb3f4, SUC_lexicon_tags_13 },
    { 0x73bfb3f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x93dfb3f7, SUC_lexicon_tags_12 },
    { 0x2cffb3f7, SUC_lexicon_tags_33 },
    { 0x121fb3f7, SUC_lexicon_tags_17 },
    { 0xe33fb3fa, SUC_lexicon_tags_15 },
    { 0xacbfb3fb, SUC_lexicon_tags_49 },
    { 0xb6bfb3fb, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ffb403, SUC_lexicon_tags_0 },
    { 0x55bfb404, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3fb40a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf07fb40e, SUC_lexicon_tags_32 },
    { 0xbbffb40e, SUC_lexicon_tags_25 },
    { 0x3ddfb410, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x2f3fb412, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe27fb414, SUC_lexicon_tags_33 },
    { 0xc89fb415, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c9fb41d, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x4b3fb41f, SUC_lexicon_tags_7 },
    { 0x445fb420, SUC_lexicon_tags_29 },
    { 0xfeffb41f, SUC_lexicon_tags_549 },
    { 0, NULL },
    { 0xef5fb423, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x91dfb425, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cffb42a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x65fb42e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdfb432, SUC_lexicon_tags_12 },
    { 0x769fb433, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xa05fb436, SUC_lexicon_tags_17 },
    { 0x957fb437, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x66ffb43c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41fb442, SUC_lexicon_tags_23 },
    { 0x7f9fb442, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cdfb445, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x401fb448, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x3dffb44a, SUC_lexicon_tags_28 },
    { 0x48ffb44a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ffb455, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11fb458, SUC_lexicon_tags_55 },
    { 0x6effb459, SUC_lexicon_tags_12 },
    { 0x365fb459, SUC_lexicon_tags_17 },
    { 0xab3fb45b, SUC_lexicon_tags_14 },
    { 0x45dfb45c, SUC_lexicon_tags_37 },
    { 0x19bfb45b, SUC_lexicon_tags_31 },
    { 0x89dfb45e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b9fb461, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299fb466, SUC_lexicon_tags_9 },
    { 0x767fb466, SUC_lexicon_tags_92 },
    { 0x613fb466, SUC_lexicon_tags_11 },
    { 0x961fb469, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69fb46d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaddfb46f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8b1fb471, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0x987fb473, SUC_lexicon_tags_16 },
    { 0xf23fb474, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc9dfb476, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1fb47b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x761fb47f, SUC_lexicon_tags_49 },
    { 0xfebfb480, SUC_lexicon_tags_33 },
    { 0x9abfb481, SUC_lexicon_tags_12 },
    { 0x235fb482, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x8d1fb484, SUC_lexicon_tags_27 },
    { 0xbb3fb484, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x891fb487, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf11fb489, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8ddfb48b, SUC_lexicon_tags_12 },
    { 0x109fb48c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6e1fb48e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5fb491, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x229fb49b, SUC_lexicon_tags_11 },
    { 0x16ffb49c, SUC_lexicon_tags_14 },
    { 0x2d5fb49b, SUC_lexicon_tags_30 },
    { 0xab5fb49e, SUC_lexicon_tags_37 },
    { 0x59bfb49f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x767fb4a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xab3fb4a5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x965fb4a7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd73fb4aa, SUC_lexicon_tags_46 },
    { 0x6f5fb4aa, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc83fb4ad, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x343fb4b1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdfb4b4, SUC_lexicon_tags_17 },
    { 0xc1dfb4b5, SUC_lexicon_tags_41 },
    { 0xe0dfb4b6, SUC_lexicon_tags_15 },
    { 0x89dfb4b5, SUC_lexicon_tags_11 },
    { 0xc2ffb4b7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf69fb4bc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5fb4c0, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x77bfb4c2, SUC_lexicon_tags_26 },
    { 0xc7ffb4c3, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xa97fb4c6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x963fb4c9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4b1fb4cb, SUC_lexicon_tags_37 },
    { 0x857fb4cc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7fb4d0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5fb4d5, SUC_lexicon_tags_15 },
    { 0xf67fb4d6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x541fb4e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31fb4e4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xbffb4e6, SUC_lexicon_tags_92 },
    { 0x319fb4e6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abfb4eb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dffb4f0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x857fb4f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7fb4f9, SUC_lexicon_tags_65 },
    { 0xc33fb4fa, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575fb500, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x3e5fb502, SUC_lexicon_tags_42 },
    { 0x167fb503, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x613fb506, SUC_lexicon_tags_12 },
    { 0xdf1fb506, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ffb512, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9fb516, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x545fb518, SUC_lexicon_tags_43 },
    { 0x47fb519, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbfb51d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x605fb520, SUC_lexicon_tags_1 },
    { 0x915fb521, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dfb524, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x323fb52e, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bfb532, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x983fb535, SUC_lexicon_tags_37 },
    { 0xebfb536, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xd39fb539, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc43fb53d, SUC_lexicon_tags_10 },
    { 0x9c5fb53e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacdfb540, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29dfb545, SUC_lexicon_tags_7 },
    { 0x27ffb546, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x565fb54c, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x14ffb54e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x265fb550, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6b7fb552, SUC_lexicon_tags_22 },
    { 0xd33fb552, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x459fb555, SUC_lexicon_tags_6 },
    { 0x1fbfb556, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xacffb559, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dfb55d, SUC_lexicon_tags_3 },
    { 0x869fb55e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbfb563, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bfb567, SUC_lexicon_tags_45 },
    { 0x3cffb568, SUC_lexicon_tags_25 },
    { 0xa73fb567, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x4effb56c, SUC_lexicon_tags_39 },
    { 0xaa7fb56d, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x6e9fb56f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa87fb571, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3fb576, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x751fb581, SUC_lexicon_tags_87 },
    { 0x621fb582, SUC_lexicon_tags_17 },
    { 0x263fb582, SUC_lexicon_tags_11 },
    { 0xc9dfb581, SUC_lexicon_tags_47 },
    { 0x625fb585, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xaabfb587, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcd9fb589, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x585fb58d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xebbfb58f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bfb592, SUC_lexicon_tags_0 },
    { 0x8cbfb592, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613fb597, SUC_lexicon_tags_2 },
    { 0x7c3fb598, SUC_lexicon_tags_77 },
    { 0x349fb599, SUC_lexicon_tags_25 },
    { 0xcc9fb599, SUC_lexicon_tags_15 },
    { 0x6adfb597, SUC_lexicon_tags_13 },
    { 0x73fb59c, SUC_lexicon_tags_43 },
    { 0xd7bfb59a, SUC_lexicon_tags_15 },
    { 0xb77fb59c, SUC_lexicon_tags_12 },
    { 0x4bffb59f, SUC_lexicon_tags_30 },
    { 0x293fb5a0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963fb5a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe49fb5a6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x101fb5a8, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0x751fb5aa, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9fb5ae, SUC_lexicon_tags_45 },
    { 0x237fb5af, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x521fb5b1, SUC_lexicon_tags_11 },
    { 0x1a9fb5b1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x811fb5b4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1ffb5b8, SUC_lexicon_tags_17 },
    { 0x5ddfb5b9, SUC_lexicon_tags_38 },
    { 0x737fb5b9, SUC_lexicon_tags_87 },
    { 0x6dffb5bb, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x77fb5be, SUC_lexicon_tags_12 },
    { 0x6abfb5bf, SUC_lexicon_tags_9 },
    { 0x825fb5bf, SUC_lexicon_tags_16 },
    { 0xf51fb5c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1fb5c6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x2f1fb5c8, SUC_lexicon_tags_15 },
    { 0x6a9fb5c9, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x453fb5cd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1bfb5d7, SUC_lexicon_tags_25 },
    { 0x6bdfb5d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cbfb5dc, SUC_lexicon_tags_65 },
    { 0x923fb5dc, SUC_lexicon_tags_2 },
    { 0xaf9fb5de, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbfb5e1, SUC_lexicon_tags_11 },
    { 0x407fb5e1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bfb5e5, SUC_lexicon_tags_154 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x377fb5ea, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bfb5f2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdffb5f5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a1fb5f8, SUC_lexicon_tags_7 },
    { 0x19ffb5f8, SUC_lexicon_tags_38 },
    { 0xde3fb5fa, SUC_lexicon_tags_12 },
    { 0x443fb5fa, SUC_lexicon_tags_68 },
    { 0xc55fb5fc, SUC_lexicon_tags_37 },
    { 0xaebfb5fd, SUC_lexicon_tags_2 },
    { 0x4c7fb5fd, SUC_lexicon_tags_16 },
    { 0x123fb5ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc17fb602, SUC_lexicon_tags_11 },
    { 0xfd7fb603, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x409fb605, SUC_lexicon_tags_2 },
    { 0x439fb605, SUC_lexicon_tags_14 },
    { 0x13fb607, SUC_lexicon_tags_26 },
    { 0x9e9fb608, SUC_lexicon_tags_14 },
    { 0x505fb609, SUC_lexicon_tags_7 },
    { 0xf11fb609, SUC_lexicon_tags_37 },
    { 0x3e7fb60b, SUC_lexicon_tags_2 },
    { 0x459fb609, SUC_lexicon_tags_47 },
    { 0x4b1fb60c, SUC_lexicon_tags_11 },
    { 0x1e9fb609, SUC_lexicon_tags_9 },
    { 0xcdbfb60c, SUC_lexicon_tags_21 },
    { 0x54ffb605, SUC_lexicon_tags_0 },
    { 0xf31fb60d, SUC_lexicon_tags_31 },
    { 0xa47fb607, SUC_lexicon_tags_12 },
    { 0x8b1fb613, SUC_lexicon_tags_38 },
    { 0xa3bfb613, SUC_lexicon_tags_23 },
    { 0xb1dfb615, SUC_lexicon_tags_37 },
    { 0xe03fb616, SUC_lexicon_tags_11 },
    { 0x7d3fb614, SUC_lexicon_tags_63 },
    { 0xe67fb618, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5fb61b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bffb620, SUC_lexicon_tags_11 },
    { 0xf6dfb621, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dfb624, SUC_lexicon_tags_70 },
    { 0x111fb625, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0xe23fb627, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2ebfb629, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x179fb62b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58bfb631, SUC_lexicon_tags_165 },
    { 0xb87fb632, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb0dfb634, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x105fb636, SUC_lexicon_tags_17 },
    { 0xb55fb637, SUC_lexicon_tags_32 },
    { 0x11ffb636, SUC_lexicon_tags_25 },
    { 0x199fb639, SUC_lexicon_tags_17 },
    { 0x595fb63a, SUC_lexicon_tags_37 },
    { 0xdd1fb637, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21fb63f, SUC_lexicon_tags_12 },
    { 0x6edfb640, SUC_lexicon_tags_40 },
    { 0xef9fb63f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753fb64e, SUC_lexicon_tags_63 },
    { 0x491fb64e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x449fb651, SUC_lexicon_tags_7 },
    { 0x88ffb651, SUC_lexicon_tags_23 },
    { 0x1f5fb653, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x293fb655, SUC_lexicon_tags_17 },
    { 0x12bfb656, SUC_lexicon_tags_8 },
    { 0xd89fb656, SUC_lexicon_tags_7 },
    { 0x1fffb658, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x97dfb65a, SUC_lexicon_tags_27 },
    { 0xfb9fb65b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771fb664, SUC_lexicon_tags_15 },
    { 0x513fb665, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xde3fb667, SUC_lexicon_tags_11 },
    { 0x8ddfb668, SUC_lexicon_tags_9 },
    { 0xf09fb668, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31fb672, SUC_lexicon_tags_2 },
    { 0x2d3fb673, SUC_lexicon_tags_1 },
    { 0x537fb674, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5fb677, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19fb67b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfb67e, SUC_lexicon_tags_25 },
    { 0xca9fb67f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1fb682, SUC_lexicon_tags_16 },
    { 0xa31fb683, SUC_lexicon_tags_2 },
    { 0x23fb683, SUC_lexicon_tags_63 },
    { 0x815fb683, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x609fb688, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x281fb68d, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dfb690, SUC_lexicon_tags_37 },
    { 0xd9fb691, SUC_lexicon_tags_136 },
    { 0x64ffb692, SUC_lexicon_tags_27 },
    { 0xaeffb692, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xd9ffb695, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fb697, SUC_lexicon_tags_15 },
    { 0xcf5fb697, SUC_lexicon_tags_7 },
    { 0x935fb699, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2b5fb69b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf63fb69d, SUC_lexicon_tags_68 },
    { 0x39ffb69e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x215fb6a0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567fb6a5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bfb6a8, SUC_lexicon_tags_47 },
    { 0x7f9fb6a9, SUC_lexicon_tags_36 },
    { 0x565fb6a9, SUC_lexicon_tags_55 },
    { 0xce3fb6ab, SUC_lexicon_tags_15 },
    { 0x1f1fb6ab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbc1fb6ae, SUC_lexicon_tags_7 },
    { 0xc05fb6af, SUC_lexicon_tags_1 },
    { 0x22ffb6b0, SUC_lexicon_tags_11 },
    { 0x465fb6b0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ffb6b4, SUC_lexicon_tags_82 },
    { 0xe8ffb6b4, SUC_lexicon_tags_33 },
    { 0xfb7fb6b6, SUC_lexicon_tags_11 },
    { 0xd41fb6b4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x619fb6bd, SUC_lexicon_tags_47 },
    { 0xb79fb6bd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x17bfb6c1, SUC_lexicon_tags_40 },
    { 0x8dbfb6c1, SUC_lexicon_tags_11 },
    { 0xc43fb6c3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81ffb6cc, SUC_lexicon_tags_7 },
    { 0xf3bfb6cc, SUC_lexicon_tags_15 },
    { 0xaebfb6ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5dfb6d2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76ffb6d9, SUC_lexicon_tags_37 },
    { 0xb71fb6d9, SUC_lexicon_tags_25 },
    { 0x4b5fb6db, SUC_lexicon_tags_7 },
    { 0xd65fb6dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x283fb6de, SUC_lexicon_tags_87 },
    { 0xccffb6df, SUC_lexicon_tags_12 },
    { 0x1a3fb6e0, SUC_lexicon_tags_37 },
    { 0xde9fb6e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fb6e5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x157fb6e8, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fffb6eb, SUC_lexicon_tags_38 },
    { 0x52bfb6ec, SUC_lexicon_tags_37 },
    { 0x42dfb6ed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29fb6f1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81fb6f4, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x7fb6f6, SUC_lexicon_tags_15 },
    { 0xa0dfb6f7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7fb6fc, SUC_lexicon_tags_12 },
    { 0x8e3fb6fd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x831fb700, SUC_lexicon_tags_17 },
    { 0xc25fb701, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x9d9fb703, SUC_lexicon_tags_7 },
    { 0x8f7fb704, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4edfb706, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9fb70a, SUC_lexicon_tags_182 },
    { 0x5fdfb70a, SUC_lexicon_tags_11 },
    { 0x279fb70c, SUC_lexicon_tags_17 },
    { 0xc79fb70b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfbbfb70f, SUC_lexicon_tags_37 },
    { 0x2b5fb710, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x42bfb713, SUC_lexicon_tags_17 },
    { 0xe67fb714, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65fb718, SUC_lexicon_tags_25 },
    { 0x159fb719, SUC_lexicon_tags_45 },
    { 0x83dfb71a, SUC_lexicon_tags_7 },
    { 0xf0dfb719, SUC_lexicon_tags_11 },
    { 0xa25fb71a, SUC_lexicon_tags_27 },
    { 0xbadfb718, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ffb720, SUC_lexicon_tags_15 },
    { 0xb89fb721, SUC_lexicon_tags_913 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bfb724, SUC_lexicon_tags_38 },
    { 0xbc3fb725, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc9dfb727, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5fb72e, SUC_lexicon_tags_27 },
    { 0xcc1fb72f, SUC_lexicon_tags_37 },
    { 0x6affb730, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1cdfb732, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bfb735, SUC_lexicon_tags_1 },
    { 0xbb9fb735, SUC_lexicon_tags_2 },
    { 0x525fb736, SUC_lexicon_tags_1420 },
    { 0x9f7fb736, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc55fb73a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc11fb73c, SUC_lexicon_tags_33 },
    { 0x815fb73d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x4bdfb73f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x887fb743, SUC_lexicon_tags_2 },
    { 0xdebfb744, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x69fb746, SUC_lexicon_tags_12 },
    { 0x48dfb747, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d7fb74e, SUC_lexicon_tags_56 },
    { 0x84ffb74f, SUC_lexicon_tags_45 },
    { 0x595fb750, SUC_lexicon_tags_37 },
    { 0xee1fb751, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf1fb757, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877fb75b, SUC_lexicon_tags_49 },
    { 0xe89fb75c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd75fb75e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x5a3fb760, SUC_lexicon_tags_11 },
    { 0xeb1fb761, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc19fb763, SUC_lexicon_tags_15 },
    { 0x485fb764, SUC_lexicon_tags_7 },
    { 0xa09fb763, SUC_lexicon_tags_0 },
    { 0x915fb765, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf83fb768, SUC_lexicon_tags_17 },
    { 0x353fb769, SUC_lexicon_tags_45 },
    { 0x52dfb769, SUC_lexicon_tags_39 },
    { 0x96ffb76b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59fb770, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x557fb772, SUC_lexicon_tags_28 },
    { 0x1cbfb772, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x599fb77f, SUC_lexicon_tags_179 },
    { 0x473fb77f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d1fb782, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe29fb784, SUC_lexicon_tags_8 },
    { 0x657fb785, SUC_lexicon_tags_7 },
    { 0x401fb786, SUC_lexicon_tags_33 },
    { 0x4c1fb785, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x595fb789, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x9ebfb78c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x67ffb78e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3fb793, SUC_lexicon_tags_108 },
    { 0xf2dfb793, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3cbfb796, SUC_lexicon_tags_32 },
    { 0x69ffb797, SUC_lexicon_tags_11 },
    { 0x97fb798, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x28dfb79b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x17fb79e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x599fb7a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3fb7a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11fb7ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x595fb7ad, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x863fb7af, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fb7b4, SUC_lexicon_tags_17 },
    { 0x5edfb7b5, SUC_lexicon_tags_28 },
    { 0xa3fb7b6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbfb7be, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901fb7c4, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xdcbfb7c6, SUC_lexicon_tags_15 },
    { 0xa39fb7c7, SUC_lexicon_tags_17 },
    { 0xf03fb7c8, SUC_lexicon_tags_11 },
    { 0x17ffb7c8, SUC_lexicon_tags_37 },
    { 0xbddfb7ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fffb7cf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x443fb7d1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dfb7da, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9fb7de, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1fb7e1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f5fb7eb, SUC_lexicon_tags_3 },
    { 0x2b7fb7eb, SUC_lexicon_tags_14 },
    { 0xb43fb7ed, SUC_lexicon_tags_25 },
    { 0x953fb7ee, SUC_lexicon_tags_87 },
    { 0x71bfb7ef, SUC_lexicon_tags_27 },
    { 0xcb3fb7ed, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfb7f6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x129fb7f8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ffb7fe, SUC_lexicon_tags_46 },
    { 0x99fb7fe, SUC_lexicon_tags_27 },
    { 0x8fffb7fe, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73bfb804, SUC_lexicon_tags_890 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcffb808, SUC_lexicon_tags_26 },
    { 0xd15fb809, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x859fb80b, SUC_lexicon_tags_87 },
    { 0xebffb80c, SUC_lexicon_tags_12 },
    { 0x8b7fb80d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x78ffb80f, SUC_lexicon_tags_38 },
    { 0xdd9fb810, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xe35fb813, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x801fb816, SUC_lexicon_tags_25 },
    { 0x12bfb816, SUC_lexicon_tags_14 },
    { 0x56bfb818, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abfb81c, SUC_lexicon_tags_10 },
    { 0x71fb81d, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x82bfb81f, SUC_lexicon_tags_23 },
    { 0xccbfb81f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bfb824, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca1fb82a, SUC_lexicon_tags_57 },
    { 0x187fb82b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59dfb82f, SUC_lexicon_tags_37 },
    { 0x9cffb830, SUC_lexicon_tags_18 },
    { 0xab9fb830, SUC_lexicon_tags_15 },
    { 0xbf9fb832, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x64dfb834, SUC_lexicon_tags_28 },
    { 0x84bfb835, SUC_lexicon_tags_25 },
    { 0x529fb835, SUC_lexicon_tags_2 },
    { 0xf81fb837, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xe2dfb839, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb31fb83b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa89fb83d, SUC_lexicon_tags_1 },
    { 0xac3fb83d, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31fb845, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb5fb849, SUC_lexicon_tags_2 },
    { 0x1bfb84a, SUC_lexicon_tags_11 },
    { 0x25bfb84a, SUC_lexicon_tags_63 },
    { 0x745fb84c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x88dfb850, SUC_lexicon_tags_17 },
    { 0xc6dfb851, SUC_lexicon_tags_17 },
    { 0xdddfb852, SUC_lexicon_tags_5 },
    { 0xf57fb853, SUC_lexicon_tags_17 },
    { 0xe7dfb850, SUC_lexicon_tags_17 },
    { 0x92dfb854, SUC_lexicon_tags_1 },
    { 0x557fb856, SUC_lexicon_tags_43 },
    { 0xe61fb852, SUC_lexicon_tags_11 },
    { 0xb3bfb856, SUC_lexicon_tags_15 },
    { 0xb0bfb856, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a7fb85b, SUC_lexicon_tags_13 },
    { 0xaf5fb85b, SUC_lexicon_tags_11 },
    { 0x527fb85c, SUC_lexicon_tags_21 },
    { 0x8c9fb85b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xfc3fb860, SUC_lexicon_tags_1 },
    { 0x493fb861, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xf8bfb863, SUC_lexicon_tags_15 },
    { 0xebffb864, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x365fb871, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf8bfb873, SUC_lexicon_tags_7 },
    { 0xf43fb874, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xce5fb876, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x773fb879, SUC_lexicon_tags_25 },
    { 0x6f7fb87a, SUC_lexicon_tags_48 },
    { 0x8e5fb87b, SUC_lexicon_tags_12 },
    { 0xfd3fb87c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3fb881, SUC_lexicon_tags_87 },
    { 0xf63fb882, SUC_lexicon_tags_17 },
    { 0x85fb881, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x14dfb885, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2f7fb887, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe33fb890, SUC_lexicon_tags_11 },
    { 0xf89fb891, SUC_lexicon_tags_224 },
    { 0x8b1fb892, SUC_lexicon_tags_45 },
    { 0xcf9fb893, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1fb896, SUC_lexicon_tags_17 },
    { 0x70dfb896, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45fb89b, SUC_lexicon_tags_3 },
    { 0x831fb89c, SUC_lexicon_tags_25 },
    { 0x557fb89c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc9fb89f, SUC_lexicon_tags_15 },
    { 0xaa1fb89f, SUC_lexicon_tags_29 },
    { 0x1a5fb8a0, SUC_lexicon_tags_15 },
    { 0xd5dfb8a2, SUC_lexicon_tags_25 },
    { 0xf93fb8a1, SUC_lexicon_tags_1 },
    { 0x373fb8a4, SUC_lexicon_tags_10 },
    { 0x29fb8a5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x991fb8a9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x3a5fb8ab, SUC_lexicon_tags_118 },
    { 0xb7dfb8ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3fb8b1, SUC_lexicon_tags_11 },
    { 0xe1ffb8b1, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa69fb8b7, SUC_lexicon_tags_2 },
    { 0x501fb8b8, SUC_lexicon_tags_49 },
    { 0xb8bfb8b9, SUC_lexicon_tags_9 },
    { 0x2bbfb8ba, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ffb8bf, SUC_lexicon_tags_25 },
    { 0xeedfb8c0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x66ffb8c3, SUC_lexicon_tags_33 },
    { 0x383fb8c4, SUC_lexicon_tags_9 },
    { 0x383fb8c3, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x81fb8c8, SUC_lexicon_tags_289 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbfb8cc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7fb8cf, SUC_lexicon_tags_11 },
    { 0x343fb8d0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601fb8d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x123fb8d7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd1ffb8d9, SUC_lexicon_tags_33 },
    { 0x2f5fb8da, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5ffb8de, SUC_lexicon_tags_11 },
    { 0xbb9fb8df, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1fb8e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x677fb8e5, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5fb8ea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf37fb8ee, SUC_lexicon_tags_11 },
    { 0xc8dfb8ef, SUC_lexicon_tags_23 },
    { 0x9bffb8ef, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51fb8f8, SUC_lexicon_tags_15 },
    { 0x7e3fb8f9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707fb8fe, SUC_lexicon_tags_22 },
    { 0x911fb8fe, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dfb907, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbfb90a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x915fb90c, SUC_lexicon_tags_12 },
    { 0x601fb90c, SUC_lexicon_tags_7 },
    { 0xb5dfb90e, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dfb912, SUC_lexicon_tags_23 },
    { 0xebfb913, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd61fb915, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x255fb917, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc37fb919, SUC_lexicon_tags_152 },
    { 0xc9dfb91a, SUC_lexicon_tags_12 },
    { 0xea7fb919, SUC_lexicon_tags_6 },
    { 0xcb7fb91a, SUC_lexicon_tags_56 },
    { 0x96dfb919, SUC_lexicon_tags_27 },
    { 0xc4dfb91c, SUC_lexicon_tags_15 },
    { 0xa19fb91f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x71bfb922, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731fb928, SUC_lexicon_tags_11 },
    { 0xf53fb929, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9b5fb92b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11dfb932, SUC_lexicon_tags_25 },
    { 0xe8ffb933, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2dbfb935, SUC_lexicon_tags_1 },
    { 0x76dfb936, SUC_lexicon_tags_19 },
    { 0xcebfb935, SUC_lexicon_tags_16 },
    { 0x147fb938, SUC_lexicon_tags_47 },
    { 0xa21fb936, SUC_lexicon_tags_63 },
    { 0xd33fb935, SUC_lexicon_tags_26 },
    { 0x8f5fb93b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xdf5fb93d, SUC_lexicon_tags_63 },
    { 0x369fb93e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7fb946, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1ffb94a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dfb94d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x10bfb950, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3fb954, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd9dfb956, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x471fb959, SUC_lexicon_tags_8 },
    { 0xa9fb95a, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0xc3bfb95c, SUC_lexicon_tags_198 },
    { 0, NULL },
    { 0xc85fb95e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x81ffb961, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x341fb964, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ffb96b, SUC_lexicon_tags_49 },
    { 0xcf9fb96c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x2c3fb96e, SUC_lexicon_tags_207 },
    { 0, NULL },
    { 0x371fb970, SUC_lexicon_tags_2 },
    { 0x81fb970, SUC_lexicon_tags_1 },
    { 0xa83fb971, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3fb977, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7cffb979, SUC_lexicon_tags_37 },
    { 0x3dbfb979, SUC_lexicon_tags_37 },
    { 0xae3fb979, SUC_lexicon_tags_46 },
    { 0xed7fb97c, SUC_lexicon_tags_65 },
    { 0x527fb97d, SUC_lexicon_tags_10 },
    { 0xd33fb97a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd3fb982, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9bfb985, SUC_lexicon_tags_104 },
    { 0xf0ffb986, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xb69fb989, SUC_lexicon_tags_38 },
    { 0x707fb98a, SUC_lexicon_tags_25 },
    { 0x40ffb98b, SUC_lexicon_tags_25 },
    { 0x955fb98c, SUC_lexicon_tags_63 },
    { 0x133fb98d, SUC_lexicon_tags_31 },
    { 0xdadfb98e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdb3fb990, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dfb993, SUC_lexicon_tags_25 },
    { 0x91bfb993, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x725fb996, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x66dfb999, SUC_lexicon_tags_13 },
    { 0x9a9fb999, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdfb99e, SUC_lexicon_tags_5 },
    { 0xc79fb99f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x371fb9a1, SUC_lexicon_tags_14 },
    { 0x2abfb9a1, SUC_lexicon_tags_26 },
    { 0x52ffb9a2, SUC_lexicon_tags_45 },
    { 0xbc1fb9a3, SUC_lexicon_tags_10 },
    { 0x235fb9a2, SUC_lexicon_tags_30 },
    { 0xdf3fb9a6, SUC_lexicon_tags_11 },
    { 0xb43fb9a7, SUC_lexicon_tags_14 },
    { 0x95fb9a8, SUC_lexicon_tags_9 },
    { 0x69ffb9a7, SUC_lexicon_tags_33 },
    { 0x829fb9aa, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x57dfb9ad, SUC_lexicon_tags_11 },
    { 0xabfb9ae, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x7fdfb9b0, SUC_lexicon_tags_26 },
    { 0xffdfb9b0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b5fb9b7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x383fb9b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x673fb9bd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaaffb9c5, SUC_lexicon_tags_30 },
    { 0x99bfb9c6, SUC_lexicon_tags_27 },
    { 0x477fb9c7, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x785fb9cb, SUC_lexicon_tags_1 },
    { 0x321fb9cc, SUC_lexicon_tags_19 },
    { 0xa33fb9cb, SUC_lexicon_tags_29 },
    { 0xdb5fb9ce, SUC_lexicon_tags_25 },
    { 0xc4dfb9ce, SUC_lexicon_tags_80 },
    { 0x52bfb9d0, SUC_lexicon_tags_27 },
    { 0x77bfb9d1, SUC_lexicon_tags_29 },
    { 0x483fb9d2, SUC_lexicon_tags_37 },
    { 0xda1fb9d2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x9bfb9d5, SUC_lexicon_tags_30 },
    { 0x753fb9d6, SUC_lexicon_tags_49 },
    { 0x29bfb9d5, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x619fb9d9, SUC_lexicon_tags_31 },
    { 0xf8ffb9da, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc15fb9dc, SUC_lexicon_tags_32 },
    { 0xc15fb9dd, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0bfb9e3, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x40bfb9e5, SUC_lexicon_tags_7 },
    { 0x63bfb9e6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbe7fb9e8, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xe45fb9ea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x861fb9ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfb9ef, SUC_lexicon_tags_242 },
    { 0xb69fb9ef, SUC_lexicon_tags_47 },
    { 0xc57fb9f0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x647fb9fb, SUC_lexicon_tags_45 },
    { 0xc45fb9fc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b1fba00, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2fdfba02, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5fba08, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ffba0f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ffba13, SUC_lexicon_tags_1 },
    { 0x567fba13, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x601fba1d, SUC_lexicon_tags_44 },
    { 0x953fba1d, SUC_lexicon_tags_17 },
    { 0xe3dfba1f, SUC_lexicon_tags_21 },
    { 0xe11fba20, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebfba24, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb97fba26, SUC_lexicon_tags_7 },
    { 0xf09fba26, SUC_lexicon_tags_49 },
    { 0x311fba28, SUC_lexicon_tags_43 },
    { 0x77ffba28, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x925fba2b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbbfba2d, SUC_lexicon_tags_1 },
    { 0xf57fba2d, SUC_lexicon_tags_25 },
    { 0x865fba2f, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7fba32, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7fba35, SUC_lexicon_tags_26 },
    { 0xf5fba36, SUC_lexicon_tags_12 },
    { 0x94bfba37, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe61fba3b, SUC_lexicon_tags_37 },
    { 0xcbfba3c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xc1ffba3e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x875fba41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ffba47, SUC_lexicon_tags_85 },
    { 0xabdfba48, SUC_lexicon_tags_15 },
    { 0xfdfba49, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xdffba4b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe93fba4d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85fba52, SUC_lexicon_tags_282 },
    { 0xf45fba52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe25fba55, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401fba59, SUC_lexicon_tags_16 },
    { 0x5e7fba5a, SUC_lexicon_tags_27 },
    { 0x35fba5b, SUC_lexicon_tags_40 },
    { 0x4f7fba5c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae1fba61, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeabfba64, SUC_lexicon_tags_4 },
    { 0x48dfba64, SUC_lexicon_tags_29 },
    { 0x5a7fba66, SUC_lexicon_tags_77 },
    { 0x8bbfba66, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ffba6a, SUC_lexicon_tags_11 },
    { 0x7dffba6b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa7fba73, SUC_lexicon_tags_29 },
    { 0x94bfba74, SUC_lexicon_tags_39 },
    { 0x94ffba73, SUC_lexicon_tags_32 },
    { 0x3dfba73, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5ffba79, SUC_lexicon_tags_26 },
    { 0x90dfba7a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dfba80, SUC_lexicon_tags_9 },
    { 0xe1dfba81, SUC_lexicon_tags_161 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x621fba86, SUC_lexicon_tags_39 },
    { 0x35bfba87, SUC_lexicon_tags_37 },
    { 0x5a9fba88, SUC_lexicon_tags_2 },
    { 0xb9fba89, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fba8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43dfba97, SUC_lexicon_tags_30 },
    { 0x2e7fba98, SUC_lexicon_tags_23 },
    { 0x37bfba98, SUC_lexicon_tags_11 },
    { 0xe57fba9a, SUC_lexicon_tags_26 },
    { 0xe2dfba9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4d1fba9d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe21fbaa0, SUC_lexicon_tags_63 },
    { 0x149fbaa1, SUC_lexicon_tags_26 },
    { 0xd61fbaa2, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0xa47fbaa4, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ffbaaa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6adfbaac, SUC_lexicon_tags_2 },
    { 0xfb1fbaad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589fbab6, SUC_lexicon_tags_16 },
    { 0x187fbab7, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x455fbab9, SUC_lexicon_tags_30 },
    { 0x2cbfbaba, SUC_lexicon_tags_7 },
    { 0x987fbab9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x12dfbabd, SUC_lexicon_tags_8 },
    { 0x5c1fbabd, SUC_lexicon_tags_50 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7fbac1, SUC_lexicon_tags_14 },
    { 0x9a9fbac1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xbbbfbac4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x175fbac6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa8dfbac8, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x245fbad0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edfbad4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731fbad8, SUC_lexicon_tags_45 },
    { 0xd77fbad9, SUC_lexicon_tags_28 },
    { 0x601fbada, SUC_lexicon_tags_11 },
    { 0x2c5fbad8, SUC_lexicon_tags_15 },
    { 0x553fbadc, SUC_lexicon_tags_60 },
    { 0xb55fbadc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5fbae3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90bfbae7, SUC_lexicon_tags_9 },
    { 0x923fbae8, SUC_lexicon_tags_84 },
    { 0x18bfbae7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc19fbaeb, SUC_lexicon_tags_39 },
    { 0xcc3fbaec, SUC_lexicon_tags_33 },
    { 0xd67fbaec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc7ffbaef, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdfbaf4, SUC_lexicon_tags_19 },
    { 0xbd3fbaf5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddfbaf9, SUC_lexicon_tags_23 },
    { 0x229fbafa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e3fbafd, SUC_lexicon_tags_660 },
    { 0xf15fbafe, SUC_lexicon_tags_47 },
    { 0xa2dfbaff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x883fbb01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcffbb05, SUC_lexicon_tags_19 },
    { 0xf4ffbb06, SUC_lexicon_tags_8 },
    { 0x569fbb05, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x419fbb0a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47bfbb0d, SUC_lexicon_tags_12 },
    { 0xf9fbb0d, SUC_lexicon_tags_2 },
    { 0x6a9fbb0d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x675fbb12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x58bfbb14, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01fbb19, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ffbb1f, SUC_lexicon_tags_63 },
    { 0x293fbb20, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x865fbb23, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1fbb26, SUC_lexicon_tags_15 },
    { 0xcf5fbb27, SUC_lexicon_tags_138 },
    { 0x70ffbb28, SUC_lexicon_tags_11 },
    { 0xd4dfbb29, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x583fbb33, SUC_lexicon_tags_9 },
    { 0xd75fbb33, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x473fbb36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce9fbb3c, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bdfbb4b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dffbb4e, SUC_lexicon_tags_60 },
    { 0x3ddfbb4f, SUC_lexicon_tags_17 },
    { 0x309fbb50, SUC_lexicon_tags_27 },
    { 0x8b5fbb51, SUC_lexicon_tags_17 },
    { 0xc85fbb50, SUC_lexicon_tags_45 },
    { 0x63ffbb50, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bfbb56, SUC_lexicon_tags_28 },
    { 0xf85fbb56, SUC_lexicon_tags_45 },
    { 0x9abfbb57, SUC_lexicon_tags_25 },
    { 0x8c1fbb57, SUC_lexicon_tags_46 },
    { 0x5fbfbb5a, SUC_lexicon_tags_11 },
    { 0xf37fbb5a, SUC_lexicon_tags_16 },
    { 0xb69fbb5a, SUC_lexicon_tags_215 },
    { 0x237fbb57, SUC_lexicon_tags_25 },
    { 0xd1bfbb5e, SUC_lexicon_tags_25 },
    { 0xba7fbb5e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x135fbb69, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x63ffbb6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd01fbb6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbfbb73, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xba7fbb76, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb1fbb78, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1edfbb7f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x92dfbb81, SUC_lexicon_tags_28 },
    { 0x157fbb82, SUC_lexicon_tags_1 },
    { 0xfbbfbb83, SUC_lexicon_tags_29 },
    { 0xbadfbb81, SUC_lexicon_tags_25 },
    { 0x591fbb85, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xff9fbb8c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x46bfbb8e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ffbb92, SUC_lexicon_tags_33 },
    { 0x82dfbb92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f3fbb9d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523fbba2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbfbba9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xda5fbbab, SUC_lexicon_tags_2 },
    { 0x9c7fbbac, SUC_lexicon_tags_48 },
    { 0x4d9fbbac, SUC_lexicon_tags_15 },
    { 0x4a7fbbac, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863fbbb2, SUC_lexicon_tags_1 },
    { 0xdb3fbbb3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29fbbb7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfbbbc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d7fbbc0, SUC_lexicon_tags_45 },
    { 0xc33fbbc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bfbbc5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dfbbcd, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x653fbbcf, SUC_lexicon_tags_22 },
    { 0xd7ffbbd0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x43bfbbd3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xb91fbbd5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x101fbbd9, SUC_lexicon_tags_37 },
    { 0x2d5fbbda, SUC_lexicon_tags_2 },
    { 0x227fbbdb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x457fbbdd, SUC_lexicon_tags_37 },
    { 0xbe5fbbde, SUC_lexicon_tags_5 },
    { 0x14dfbbdf, SUC_lexicon_tags_198 },
    { 0x501fbbe0, SUC_lexicon_tags_47 },
    { 0x17dfbbe1, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fbbec, SUC_lexicon_tags_57 },
    { 0x1bbfbbec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x28bfbbf2, SUC_lexicon_tags_46 },
    { 0x295fbbf3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bfbbf9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x329fbbfb, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x7a1fbbfd, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x159fbc02, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc07fbc04, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9fbc0b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd33fbc10, SUC_lexicon_tags_17 },
    { 0x4a5fbc10, SUC_lexicon_tags_3 },
    { 0xa6ffbc12, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c7fbc1b, SUC_lexicon_tags_12 },
    { 0x6e1fbc1c, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85fbc20, SUC_lexicon_tags_47 },
    { 0xe49fbc20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e9fbc24, SUC_lexicon_tags_48 },
    { 0x521fbc24, SUC_lexicon_tags_7 },
    { 0x7ddfbc24, SUC_lexicon_tags_47 },
    { 0x8abfbc27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9adfbc2b, SUC_lexicon_tags_15 },
    { 0x5e9fbc2c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddfbc2f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcf5fbc31, SUC_lexicon_tags_17 },
    { 0x251fbc32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5fbc35, SUC_lexicon_tags_45 },
    { 0x6c9fbc36, SUC_lexicon_tags_16 },
    { 0xcc3fbc35, SUC_lexicon_tags_23 },
    { 0xa59fbc37, SUC_lexicon_tags_33 },
    { 0xf45fbc39, SUC_lexicon_tags_37 },
    { 0x8dbfbc3a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5dfbc3c, SUC_lexicon_tags_7 },
    { 0x9dbfbc3d, SUC_lexicon_tags_257 },
    { 0x6a3fbc3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5dfbc42, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x4cdfbc45, SUC_lexicon_tags_64 },
    { 0x7e5fbc46, SUC_lexicon_tags_49 },
    { 0x4affbc47, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ffbc4a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf3fbc54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3d3fbc56, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dffbc5b, SUC_lexicon_tags_19 },
    { 0x2ddfbc5c, SUC_lexicon_tags_19 },
    { 0x63ffbc5c, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xaf9fbc5f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a7fbc62, SUC_lexicon_tags_1 },
    { 0xff9fbc62, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdfbc68, SUC_lexicon_tags_26 },
    { 0x987fbc69, SUC_lexicon_tags_12 },
    { 0x43fbc6a, SUC_lexicon_tags_11 },
    { 0x2c9fbc6b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa45fbc70, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905fbc75, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb95fbc77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145fbc7c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x1a9fbc7e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3fbc81, SUC_lexicon_tags_47 },
    { 0x3cbfbc82, SUC_lexicon_tags_55 },
    { 0xc9ffbc81, SUC_lexicon_tags_48 },
    { 0xabbfbc82, SUC_lexicon_tags_7 },
    { 0xf1dfbc82, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1ffbc88, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc1fbc8b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b3fbc91, SUC_lexicon_tags_7 },
    { 0x55fbc92, SUC_lexicon_tags_49 },
    { 0x6e3fbc93, SUC_lexicon_tags_37 },
    { 0x7a5fbc94, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7fbc97, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x78dfbc9c, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43fbc9f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ffbca5, SUC_lexicon_tags_37 },
    { 0x5cdfbca6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3dbfbca8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb27fbcb0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x585fbcb2, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0x7d5fbcb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xef5fbcb6, SUC_lexicon_tags_39 },
    { 0x753fbcb7, SUC_lexicon_tags_25 },
    { 0xb1ffbcb8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf5ffbcba, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41dfbcbf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x639fbcc2, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x5fdfbcc4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x307fbcca, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7fbccd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91fbcd0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x46ffbcd2, SUC_lexicon_tags_60 },
    { 0x34bfbcd3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71fbcd6, SUC_lexicon_tags_13 },
    { 0xf6bfbcd7, SUC_lexicon_tags_55 },
    { 0xdcdfbcd8, SUC_lexicon_tags_43 },
    { 0x21bfbcd8, SUC_lexicon_tags_11 },
    { 0xa35fbcd8, SUC_lexicon_tags_12 },
    { 0xe0bfbcdb, SUC_lexicon_tags_2 },
    { 0xf71fbcd8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b1fbcdf, SUC_lexicon_tags_1 },
    { 0x247fbcdf, SUC_lexicon_tags_25 },
    { 0xc61fbce1, SUC_lexicon_tags_11 },
    { 0x173fbce1, SUC_lexicon_tags_14 },
    { 0x763fbce3, SUC_lexicon_tags_37 },
    { 0xcd9fbce4, SUC_lexicon_tags_25 },
    { 0xf9bfbcdf, SUC_lexicon_tags_43 },
    { 0xb21fbce6, SUC_lexicon_tags_0 },
    { 0xe53fbce2, SUC_lexicon_tags_15 },
    { 0xcbdfbce8, SUC_lexicon_tags_15 },
    { 0x4e5fbce8, SUC_lexicon_tags_37 },
    { 0x48bfbcea, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x601fbcec, SUC_lexicon_tags_46 },
    { 0xc63fbcec, SUC_lexicon_tags_47 },
    { 0x943fbcee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdffbcf1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dfbcf5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83bfbcf7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73dfbcfb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x753fbcfd, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x641fbcff, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x149fbd03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05fbd0a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd25fbd0f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84dfbd13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67fbd1a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb13fbd20, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x869fbd22, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x757fbd25, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa6bfbd27, SUC_lexicon_tags_32 },
    { 0x3f5fbd28, SUC_lexicon_tags_46 },
    { 0xf6bfbd29, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bfbd2c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffffbd31, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc1dfbd33, SUC_lexicon_tags_15 },
    { 0xabbfbd34, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bfbd37, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf4ffbd3a, SUC_lexicon_tags_17 },
    { 0x1f9fbd3b, SUC_lexicon_tags_82 },
    { 0x2a5fbd3b, SUC_lexicon_tags_2 },
    { 0x63bfbd3d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x361fbd3f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dfbd48, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xcdbfbd4a, SUC_lexicon_tags_63 },
    { 0x459fbd4b, SUC_lexicon_tags_15 },
    { 0x42dfbd4c, SUC_lexicon_tags_27 },
    { 0xca9fbd4c, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab9fbd51, SUC_lexicon_tags_47 },
    { 0x903fbd52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd93fbd5a, SUC_lexicon_tags_87 },
    { 0x903fbd5b, SUC_lexicon_tags_53 },
    { 0x849fbd5c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x1bffbd5e, SUC_lexicon_tags_77 },
    { 0x511fbd5f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x33bfbd62, SUC_lexicon_tags_2 },
    { 0x1d3fbd63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd87fbd65, SUC_lexicon_tags_1 },
    { 0x895fbd65, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa75fbd68, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x331fbd6b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x707fbd6d, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9fbd77, SUC_lexicon_tags_37 },
    { 0x6adfbd78, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xeb7fbd7a, SUC_lexicon_tags_17 },
    { 0x88ffbd7a, SUC_lexicon_tags_17 },
    { 0x197fbd7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7fbd7f, SUC_lexicon_tags_15 },
    { 0x96ffbd80, SUC_lexicon_tags_0 },
    { 0xb55fbd81, SUC_lexicon_tags_48 },
    { 0x941fbd82, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fbd85, SUC_lexicon_tags_6 },
    { 0xdaffbd85, SUC_lexicon_tags_15 },
    { 0x503fbd87, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xb5ffbd89, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb43fbd8c, SUC_lexicon_tags_37 },
    { 0xcaffbd8d, SUC_lexicon_tags_14 },
    { 0x6e9fbd8e, SUC_lexicon_tags_15 },
    { 0x495fbd8f, SUC_lexicon_tags_15 },
    { 0x15fbd8c, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x849fbd97, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9fbd9a, SUC_lexicon_tags_113 },
    { 0x2f1fbd9b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3fbd9e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475fbda3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9fbdaa, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xa4ffbdac, SUC_lexicon_tags_29 },
    { 0x49ffbdac, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a1fbdb5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xf23fbdb7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9fbdbc, SUC_lexicon_tags_7 },
    { 0x73ffbdbc, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85fbdc3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x801fbdc6, SUC_lexicon_tags_2 },
    { 0x2bdfbdc7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x429fbdc9, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f3fbdd0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xaffbdd3, SUC_lexicon_tags_12 },
    { 0x169fbdd4, SUC_lexicon_tags_17 },
    { 0x905fbdd5, SUC_lexicon_tags_11 },
    { 0x3effbdd4, SUC_lexicon_tags_38 },
    { 0xd23fbdd7, SUC_lexicon_tags_25 },
    { 0x2d3fbdd7, SUC_lexicon_tags_33 },
    { 0x6f1fbdd9, SUC_lexicon_tags_37 },
    { 0x999fbdda, SUC_lexicon_tags_117 },
    { 0xd89fbdd5, SUC_lexicon_tags_14 },
    { 0xad7fbdd5, SUC_lexicon_tags_26 },
    { 0xb27fbdd5, SUC_lexicon_tags_2 },
    { 0x7bfbdda, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5fbde2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x411fbde7, SUC_lexicon_tags_45 },
    { 0xb25fbde7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebfbdec, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xa9dfbdee, SUC_lexicon_tags_33 },
    { 0x4a7fbdef, SUC_lexicon_tags_38 },
    { 0xabbfbdee, SUC_lexicon_tags_12 },
    { 0x8d3fbdef, SUC_lexicon_tags_15 },
    { 0xe71fbdf2, SUC_lexicon_tags_17 },
    { 0x241fbdf3, SUC_lexicon_tags_670 },
    { 0x491fbdf4, SUC_lexicon_tags_18 },
    { 0xd6dfbdf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebffbdf9, SUC_lexicon_tags_63 },
    { 0xc0ffbdfa, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xc1ffbdfc, SUC_lexicon_tags_48 },
    { 0xcdffbdfd, SUC_lexicon_tags_26 },
    { 0xae1fbdfd, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x787fbe03, SUC_lexicon_tags_23 },
    { 0x289fbe04, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x243fbe06, SUC_lexicon_tags_7 },
    { 0xfadfbe07, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcbffbe09, SUC_lexicon_tags_15 },
    { 0x91ffbe0a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ffbe0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x481fbe10, SUC_lexicon_tags_37 },
    { 0x4d1fbe11, SUC_lexicon_tags_2 },
    { 0xbddfbe12, SUC_lexicon_tags_14 },
    { 0x8e5fbe12, SUC_lexicon_tags_83 },
    { 0x8c9fbe12, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda5fbe1a, SUC_lexicon_tags_25 },
    { 0x389fbe1b, SUC_lexicon_tags_17 },
    { 0xdd1fbe1c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x91dfbe1f, SUC_lexicon_tags_42 },
    { 0x4a9fbe20, SUC_lexicon_tags_14 },
    { 0xe5bfbe1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xc3fbe23, SUC_lexicon_tags_11 },
    { 0xe87fbe24, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75ffbe28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5fbe2d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x487fbe31, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x88ffbe34, SUC_lexicon_tags_37 },
    { 0x221fbe34, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379fbe39, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xefdfbe3b, SUC_lexicon_tags_37 },
    { 0x257fbe3c, SUC_lexicon_tags_49 },
    { 0x7a5fbe3d, SUC_lexicon_tags_17 },
    { 0x4d5fbe3e, SUC_lexicon_tags_12 },
    { 0x17bfbe3f, SUC_lexicon_tags_17 },
    { 0x711fbe3c, SUC_lexicon_tags_2 },
    { 0x651fbe3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5fbe44, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x14dfbe47, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38bfbe4f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bfbe54, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x541fbe56, SUC_lexicon_tags_25 },
    { 0xf05fbe57, SUC_lexicon_tags_19 },
    { 0xe8dfbe58, SUC_lexicon_tags_1 },
    { 0x941fbe58, SUC_lexicon_tags_29 },
    { 0x893fbe58, SUC_lexicon_tags_17 },
    { 0xb83fbe59, SUC_lexicon_tags_860 },
    { 0xf2ffbe5c, SUC_lexicon_tags_204 },
    { 0, NULL },
    { 0xadffbe5e, SUC_lexicon_tags_37 },
    { 0x73ffbe5e, SUC_lexicon_tags_0 },
    { 0xffbfbe60, SUC_lexicon_tags_0 },
    { 0x2c1fbe5f, SUC_lexicon_tags_12 },
    { 0xfc3fbe62, SUC_lexicon_tags_37 },
    { 0x8adfbe63, SUC_lexicon_tags_26 },
    { 0x45fbe64, SUC_lexicon_tags_7 },
    { 0x3f5fbe61, SUC_lexicon_tags_12 },
    { 0x7e3fbe61, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x965fbe68, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5fbe6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf27fbe6d, SUC_lexicon_tags_7 },
    { 0x731fbe6e, SUC_lexicon_tags_13 },
    { 0x689fbe6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x28bfbe71, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103fbe75, SUC_lexicon_tags_1 },
    { 0x9c9fbe76, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9fbe79, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8c3fbe7b, SUC_lexicon_tags_18 },
    { 0xe13fbe7c, SUC_lexicon_tags_47 },
    { 0xcdffbe7d, SUC_lexicon_tags_2 },
    { 0x209fbe7e, SUC_lexicon_tags_46 },
    { 0x331fbe7f, SUC_lexicon_tags_17 },
    { 0x26dfbe7e, SUC_lexicon_tags_32 },
    { 0xb35fbe80, SUC_lexicon_tags_19 },
    { 0xd5dfbe82, SUC_lexicon_tags_22 },
    { 0x28bfbe83, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bfbe88, SUC_lexicon_tags_37 },
    { 0x455fbe88, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3bfbe8d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13fbe93, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x98bfbe96, SUC_lexicon_tags_12 },
    { 0x3dbfbe97, SUC_lexicon_tags_27 },
    { 0xbfdfbe98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa1fbe9c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbfbe9f, SUC_lexicon_tags_12 },
    { 0xed7fbea0, SUC_lexicon_tags_11 },
    { 0x8a3fbea1, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x501fbea3, SUC_lexicon_tags_21 },
    { 0xf09fbea4, SUC_lexicon_tags_28 },
    { 0x109fbea5, SUC_lexicon_tags_37 },
    { 0xd0dfbea3, SUC_lexicon_tags_47 },
    { 0x5f9fbea3, SUC_lexicon_tags_2 },
    { 0x409fbea8, SUC_lexicon_tags_24 },
    { 0x10dfbea8, SUC_lexicon_tags_17 },
    { 0x31bfbea8, SUC_lexicon_tags_17 },
    { 0xfadfbeab, SUC_lexicon_tags_60 },
    { 0xeaffbeaa, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8adfbeae, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfbeb1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5fbeb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x73fbeb7, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x835fbebb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e5fbebf, SUC_lexicon_tags_37 },
    { 0xab3fbebf, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x463fbec2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7fbec5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2a9fbec7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7fbecd, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5fbed1, SUC_lexicon_tags_6 },
    { 0x96dfbed2, SUC_lexicon_tags_31 },
    { 0x8b7fbed2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3effbedb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbfbee1, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0xf9fbee3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dfbee7, SUC_lexicon_tags_21 },
    { 0x57bfbee8, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0x743fbeea, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb8dfbeec, SUC_lexicon_tags_2 },
    { 0x3b3fbeed, SUC_lexicon_tags_10 },
    { 0x54dfbeee, SUC_lexicon_tags_2 },
    { 0xe59fbeef, SUC_lexicon_tags_11 },
    { 0x319fbef0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddfbef4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x445fbf02, SUC_lexicon_tags_8 },
    { 0x309fbf03, SUC_lexicon_tags_402 },
    { 0xf5bfbf04, SUC_lexicon_tags_11 },
    { 0xc1bfbf04, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5fbf09, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bffbf0d, SUC_lexicon_tags_72 },
    { 0xb3fbf0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1fbf11, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3fbf1a, SUC_lexicon_tags_26 },
    { 0xe27fbf1a, SUC_lexicon_tags_11 },
    { 0xcabfbf1b, SUC_lexicon_tags_12 },
    { 0xfc1fbf1b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1fbf21, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c1fbf24, SUC_lexicon_tags_47 },
    { 0xf53fbf25, SUC_lexicon_tags_53 },
    { 0xf87fbf26, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdfbf29, SUC_lexicon_tags_12 },
    { 0xf2bfbf2a, SUC_lexicon_tags_15 },
    { 0xf37fbf2a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa5fbf2d, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbfbf32, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e3fbf34, SUC_lexicon_tags_25 },
    { 0xb35fbf35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c1fbf38, SUC_lexicon_tags_0 },
    { 0xd27fbf39, SUC_lexicon_tags_28 },
    { 0x6bffbf3a, SUC_lexicon_tags_254 },
    { 0xb9ffbf3b, SUC_lexicon_tags_1 },
    { 0xbddfbf3c, SUC_lexicon_tags_17 },
    { 0x5a3fbf3d, SUC_lexicon_tags_25 },
    { 0xe49fbf3e, SUC_lexicon_tags_113 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82bfbf42, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dfbf49, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ffbf4f, SUC_lexicon_tags_17 },
    { 0x2e3fbf50, SUC_lexicon_tags_15 },
    { 0x505fbf51, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x9ebfbf53, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x76dfbf55, SUC_lexicon_tags_37 },
    { 0xb29fbf55, SUC_lexicon_tags_29 },
    { 0xd1fbf57, SUC_lexicon_tags_35 },
    { 0x3e7fbf58, SUC_lexicon_tags_25 },
    { 0xa09fbf58, SUC_lexicon_tags_19 },
    { 0x2b9fbf58, SUC_lexicon_tags_25 },
    { 0xca1fbf5b, SUC_lexicon_tags_6 },
    { 0x33ffbf5c, SUC_lexicon_tags_2 },
    { 0x45ffbf5b, SUC_lexicon_tags_28 },
    { 0xc5bfbf57, SUC_lexicon_tags_2 },
    { 0x9bdfbf5f, SUC_lexicon_tags_88 },
    { 0x37ffbf60, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d7fbf64, SUC_lexicon_tags_11 },
    { 0x615fbf65, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x511fbf67, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3fbf6f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe81fbf75, SUC_lexicon_tags_14 },
    { 0x3bffbf75, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbf7b, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ffbf80, SUC_lexicon_tags_12 },
    { 0xf79fbf80, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x5bdfbf83, SUC_lexicon_tags_4 },
    { 0xb2dfbf84, SUC_lexicon_tags_63 },
    { 0x9effbf83, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2f5fbf87, SUC_lexicon_tags_48 },
    { 0xa9dfbf87, SUC_lexicon_tags_37 },
    { 0x72dfbf89, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x7cbfbf8b, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54dfbf8f, SUC_lexicon_tags_30 },
    { 0x607fbf90, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ffbf95, SUC_lexicon_tags_27 },
    { 0x9c3fbf95, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x33bfbf98, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x60bfbf9b, SUC_lexicon_tags_60 },
    { 0x299fbf9b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x379fbfa0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dffbfa9, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x493fbfad, SUC_lexicon_tags_25 },
    { 0x69ffbfae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b9fbfb1, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xb99fbfb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x847fbfb7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x283fbfb9, SUC_lexicon_tags_19 },
    { 0x859fbfba, SUC_lexicon_tags_12 },
    { 0x375fbfb9, SUC_lexicon_tags_42 },
    { 0x8c5fbfbc, SUC_lexicon_tags_1 },
    { 0xd05fbfbd, SUC_lexicon_tags_29 },
    { 0x4fdfbfbe, SUC_lexicon_tags_28 },
    { 0xd67fbfbf, SUC_lexicon_tags_47 },
    { 0x283fbfbd, SUC_lexicon_tags_11 },
    { 0x4dbfbfc1, SUC_lexicon_tags_33 },
    { 0x2bdfbfbe, SUC_lexicon_tags_46 },
    { 0xdc3fbfc3, SUC_lexicon_tags_14 },
    { 0x38dfbfc4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x355fbfc7, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77fbfcb, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xd4bfbfcd, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2dbfbfd3, SUC_lexicon_tags_1 },
    { 0x4d1fbfd4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ffbfd7, SUC_lexicon_tags_60 },
    { 0x445fbfd8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7fbfdb, SUC_lexicon_tags_14 },
    { 0x5e9fbfdc, SUC_lexicon_tags_2 },
    { 0x1b7fbfdd, SUC_lexicon_tags_32 },
    { 0x69ffbfde, SUC_lexicon_tags_7 },
    { 0x419fbfde, SUC_lexicon_tags_9 },
    { 0x8b3fbfdf, SUC_lexicon_tags_8 },
    { 0xa59fbfdb, SUC_lexicon_tags_35 },
    { 0x3e3fbfdc, SUC_lexicon_tags_2 },
    { 0x19bfbfe3, SUC_lexicon_tags_48 },
    { 0x67bfbfe3, SUC_lexicon_tags_9 },
    { 0x3f5fbfe5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x895fbfe7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65fbfed, SUC_lexicon_tags_37 },
    { 0x9d5fbfee, SUC_lexicon_tags_230 },
    { 0xcebfbfef, SUC_lexicon_tags_11 },
    { 0xa69fbfee, SUC_lexicon_tags_66 },
    { 0x78ffbfee, SUC_lexicon_tags_23 },
    { 0xfd1fbff1, SUC_lexicon_tags_13 },
    { 0xc37fbff3, SUC_lexicon_tags_37 },
    { 0xa8dfbff3, SUC_lexicon_tags_37 },
    { 0x241fbff5, SUC_lexicon_tags_43 },
    { 0x187fbff6, SUC_lexicon_tags_49 },
    { 0x699fbff7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9e9fbff9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbfbffe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0ffc000, SUC_lexicon_tags_12 },
    { 0xcd9fc000, SUC_lexicon_tags_37 },
    { 0xc93fc002, SUC_lexicon_tags_2 },
    { 0xfd1fc002, SUC_lexicon_tags_17 },
    { 0x247fc004, SUC_lexicon_tags_6 },
    { 0x17ffc004, SUC_lexicon_tags_7 },
    { 0xf29fc004, SUC_lexicon_tags_30 },
    { 0xcfbfc003, SUC_lexicon_tags_42 },
    { 0x46bfc005, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x40ffc00a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca5fc010, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x607fc013, SUC_lexicon_tags_47 },
    { 0xac5fc013, SUC_lexicon_tags_17 },
    { 0xac1fc014, SUC_lexicon_tags_25 },
    { 0x88ffc016, SUC_lexicon_tags_43 },
    { 0x833fc017, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeffc01d, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467fc021, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xacdfc023, SUC_lexicon_tags_37 },
    { 0xd5ffc024, SUC_lexicon_tags_7 },
    { 0x8e1fc025, SUC_lexicon_tags_65 },
    { 0xb5ffc026, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x495fc028, SUC_lexicon_tags_35 },
    { 0x95bfc029, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x1dffc02b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3effc038, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x82dfc03b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddfc044, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xf45fc047, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x127fc04d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x8edfc04f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ffc053, SUC_lexicon_tags_16 },
    { 0xa77fc054, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc3fc056, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xa99fc059, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacffc05e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x76dfc062, SUC_lexicon_tags_91 },
    { 0x55ffc063, SUC_lexicon_tags_2 },
    { 0x3c3fc062, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcffc069, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x469fc077, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xde9fc07a, SUC_lexicon_tags_45 },
    { 0xdebfc07b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd7fc081, SUC_lexicon_tags_1546 },
    { 0, NULL },
    { 0xf57fc083, SUC_lexicon_tags_7 },
    { 0x129fc084, SUC_lexicon_tags_55 },
    { 0x195fc085, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x425fc088, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ffc08f, SUC_lexicon_tags_47 },
    { 0x5cdfc090, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3fc096, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x385fc09a, SUC_lexicon_tags_43 },
    { 0x295fc09b, SUC_lexicon_tags_42 },
    { 0xfcbfc09c, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0x8ffc09e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed7fc0a3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5fc0a6, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbfc0aa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x193fc0ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5dbfc0ae, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bfc0b3, SUC_lexicon_tags_29 },
    { 0x259fc0b4, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa01fc0b6, SUC_lexicon_tags_60 },
    { 0x71dfc0b6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x11fc0b9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd13fc0bd, SUC_lexicon_tags_320 },
    { 0, NULL },
    { 0xc1dfc0bf, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61bfc0c3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313fc0c8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb73fc0d0, SUC_lexicon_tags_1 },
    { 0xe69fc0d1, SUC_lexicon_tags_1 },
    { 0x459fc0d2, SUC_lexicon_tags_7 },
    { 0xa43fc0d1, SUC_lexicon_tags_14 },
    { 0xbf1fc0d1, SUC_lexicon_tags_9 },
    { 0x7adfc0d5, SUC_lexicon_tags_2 },
    { 0x33ffc0d4, SUC_lexicon_tags_27 },
    { 0x8d5fc0d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181fc0e0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x65dfc0e3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xd67fc0e5, SUC_lexicon_tags_29 },
    { 0xba7fc0e6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571fc0ea, SUC_lexicon_tags_15 },
    { 0x6f1fc0ea, SUC_lexicon_tags_4 },
    { 0x451fc0ea, SUC_lexicon_tags_58 },
    { 0xe91fc0ed, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x44ffc0ef, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaebfc0f4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xb0dfc0f6, SUC_lexicon_tags_0 },
    { 0x1bffc0f7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f9fc0fa, SUC_lexicon_tags_33 },
    { 0x85ffc0fa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35bfc101, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1fc105, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5fc10c, SUC_lexicon_tags_47 },
    { 0x6c7fc10c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd27fc112, SUC_lexicon_tags_1228 },
    { 0, NULL },
    { 0, NULL },
    { 0xc79fc115, SUC_lexicon_tags_28 },
    { 0x551fc116, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x355fc118, SUC_lexicon_tags_21 },
    { 0x557fc119, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc37fc11f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x375fc124, SUC_lexicon_tags_15 },
    { 0xb5ffc124, SUC_lexicon_tags_39 },
    { 0xb7dfc125, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9a1fc128, SUC_lexicon_tags_7 },
    { 0x617fc129, SUC_lexicon_tags_19 },
    { 0x12bfc12a, SUC_lexicon_tags_45 },
    { 0xae7fc128, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895fc135, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x383fc137, SUC_lexicon_tags_7 },
    { 0xc87fc138, SUC_lexicon_tags_17 },
    { 0x473fc138, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3fdfc13b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x79ffc13d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe99fc13f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebfc143, SUC_lexicon_tags_2 },
    { 0x385fc144, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a3fc148, SUC_lexicon_tags_298 },
    { 0x251fc148, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4d1fc14b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x189fc14d, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xfe9fc14f, SUC_lexicon_tags_14 },
    { 0xe8ffc150, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5fc156, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba1fc15a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x47bfc15c, SUC_lexicon_tags_33 },
    { 0xc05fc15d, SUC_lexicon_tags_37 },
    { 0x1e1fc15e, SUC_lexicon_tags_1 },
    { 0x347fc15d, SUC_lexicon_tags_68 },
    { 0xcfbfc15e, SUC_lexicon_tags_4 },
    { 0x61ffc160, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x577fc167, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7e7fc169, SUC_lexicon_tags_14 },
    { 0xdeffc16a, SUC_lexicon_tags_87 },
    { 0x15ffc16b, SUC_lexicon_tags_12 },
    { 0x893fc16b, SUC_lexicon_tags_8 },
    { 0x397fc16a, SUC_lexicon_tags_7 },
    { 0x96dfc16b, SUC_lexicon_tags_66 },
    { 0x79dfc16f, SUC_lexicon_tags_116 },
    { 0xf01fc16a, SUC_lexicon_tags_12 },
    { 0xba7fc171, SUC_lexicon_tags_45 },
    { 0x451fc171, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc7ffc174, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603fc17a, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0xf63fc17c, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbfc180, SUC_lexicon_tags_9 },
    { 0x47fc181, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ffc184, SUC_lexicon_tags_31 },
    { 0x9adfc185, SUC_lexicon_tags_38 },
    { 0x8bfc186, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbfc18c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a1fc190, SUC_lexicon_tags_14 },
    { 0xc07fc190, SUC_lexicon_tags_25 },
    { 0xf89fc190, SUC_lexicon_tags_17 },
    { 0x92dfc192, SUC_lexicon_tags_7 },
    { 0xaa9fc191, SUC_lexicon_tags_25 },
    { 0x665fc192, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdfc198, SUC_lexicon_tags_11 },
    { 0x901fc199, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39fc1a1, SUC_lexicon_tags_38 },
    { 0x257fc1a2, SUC_lexicon_tags_23 },
    { 0x363fc1a1, SUC_lexicon_tags_12 },
    { 0x323fc1a4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x257fc1b0, SUC_lexicon_tags_19 },
    { 0x435fc1b1, SUC_lexicon_tags_2 },
    { 0x30bfc1b1, SUC_lexicon_tags_17 },
    { 0xffbfc1b3, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf31fc1b5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xcc5fc1b7, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0xb0dfc1b9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x233fc1bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211fc1bf, SUC_lexicon_tags_43 },
    { 0xdb3fc1bf, SUC_lexicon_tags_38 },
    { 0x3f9fc1c1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdfc1c4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb3fc1cc, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7fc1d1, SUC_lexicon_tags_7 },
    { 0x1a7fc1d2, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb35fc1d9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x8f1fc1db, SUC_lexicon_tags_25 },
    { 0x835fc1db, SUC_lexicon_tags_47 },
    { 0x969fc1dd, SUC_lexicon_tags_11 },
    { 0x825fc1de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd81fc1e5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5fc1e8, SUC_lexicon_tags_25 },
    { 0x9dffc1e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x83bfc1eb, SUC_lexicon_tags_2 },
    { 0xf27fc1eb, SUC_lexicon_tags_7 },
    { 0x235fc1eb, SUC_lexicon_tags_37 },
    { 0x6fffc1ee, SUC_lexicon_tags_45 },
    { 0xed1fc1ef, SUC_lexicon_tags_32 },
    { 0x29bfc1ef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa5bfc1f2, SUC_lexicon_tags_37 },
    { 0x887fc1f3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x12dfc1f5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3fc1f8, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x8bfc1fa, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9fc201, SUC_lexicon_tags_15 },
    { 0x935fc201, SUC_lexicon_tags_1 },
    { 0xe7dfc203, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x31fc206, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fc20e, SUC_lexicon_tags_37 },
    { 0xa25fc20f, SUC_lexicon_tags_11 },
    { 0x19fc20e, SUC_lexicon_tags_47 },
    { 0x221fc211, SUC_lexicon_tags_9 },
    { 0xbf5fc212, SUC_lexicon_tags_68 },
    { 0x325fc20e, SUC_lexicon_tags_237 },
    { 0x943fc20e, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbffc219, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa83fc21b, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x519fc225, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dfc22c, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89fc232, SUC_lexicon_tags_30 },
    { 0x269fc233, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xec1fc236, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdfc239, SUC_lexicon_tags_11 },
    { 0xe2ffc23a, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x211fc23c, SUC_lexicon_tags_1 },
    { 0x457fc23d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8c9fc23f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x497fc241, SUC_lexicon_tags_65 },
    { 0x713fc242, SUC_lexicon_tags_15 },
    { 0x103fc243, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x479fc249, SUC_lexicon_tags_47 },
    { 0x4b5fc24a, SUC_lexicon_tags_14 },
    { 0x4cffc24b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6dffc24d, SUC_lexicon_tags_7 },
    { 0xfd1fc24e, SUC_lexicon_tags_350 },
    { 0xf23fc24e, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeabfc253, SUC_lexicon_tags_21 },
    { 0x2bffc254, SUC_lexicon_tags_49 },
    { 0xdfc254, SUC_lexicon_tags_7 },
    { 0xfeffc254, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97fc25b, SUC_lexicon_tags_2 },
    { 0x6b1fc25c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x73bfc25e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf09fc263, SUC_lexicon_tags_77 },
    { 0x32bfc264, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9fc267, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x895fc26a, SUC_lexicon_tags_12 },
    { 0x7f3fc26b, SUC_lexicon_tags_37 },
    { 0x74ffc26c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x97dfc26e, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x881fc271, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x461fc274, SUC_lexicon_tags_12 },
    { 0x843fc275, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x453fc277, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24ffc27b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd89fc27d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebfc284, SUC_lexicon_tags_1 },
    { 0x6e7fc285, SUC_lexicon_tags_45 },
    { 0xa47fc285, SUC_lexicon_tags_31 },
    { 0xccbfc285, SUC_lexicon_tags_235 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6ffc28a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3d7fc28c, SUC_lexicon_tags_28 },
    { 0x953fc28c, SUC_lexicon_tags_15 },
    { 0x2b1fc28d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x9f3fc291, SUC_lexicon_tags_25 },
    { 0x7cbfc291, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x15fc294, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3fc298, SUC_lexicon_tags_12 },
    { 0xfffc299, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ffc29d, SUC_lexicon_tags_191 },
    { 0x2effc29e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x581fc2a4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe91fc2a6, SUC_lexicon_tags_0 },
    { 0xcbffc2a7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4adfc2ab, SUC_lexicon_tags_7 },
    { 0xb9dfc2ac, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989fc2b4, SUC_lexicon_tags_17 },
    { 0x2ffc2b5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823fc2bc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb87fc2bf, SUC_lexicon_tags_11 },
    { 0x907fc2c0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35fc2c3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xb89fc2c5, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x563fc2c7, SUC_lexicon_tags_12 },
    { 0x8f7fc2c8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffffc2cc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ffc2cf, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9fc2d2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9fc2d5, SUC_lexicon_tags_14 },
    { 0x945fc2d5, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0x365fc2d9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759fc2de, SUC_lexicon_tags_37 },
    { 0xb89fc2de, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5dffc2e1, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ffc2e4, SUC_lexicon_tags_2 },
    { 0xea3fc2e5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x631fc2e9, SUC_lexicon_tags_66 },
    { 0x5c7fc2ea, SUC_lexicon_tags_3 },
    { 0xd35fc2ea, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd93fc2ed, SUC_lexicon_tags_39 },
    { 0x23fc2ee, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x841fc2f6, SUC_lexicon_tags_37 },
    { 0xa77fc2f7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x8effc2fa, SUC_lexicon_tags_15 },
    { 0xb13fc2fa, SUC_lexicon_tags_37 },
    { 0x28dfc2fc, SUC_lexicon_tags_17 },
    { 0xb09fc2fb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfc301, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59fc30b, SUC_lexicon_tags_45 },
    { 0x5e1fc30b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x539fc30e, SUC_lexicon_tags_2 },
    { 0xaebfc30f, SUC_lexicon_tags_3 },
    { 0xf11fc310, SUC_lexicon_tags_8 },
    { 0xfd3fc30e, SUC_lexicon_tags_0 },
    { 0x6fbfc312, SUC_lexicon_tags_11 },
    { 0x2d5fc312, SUC_lexicon_tags_8 },
    { 0xec3fc312, SUC_lexicon_tags_11 },
    { 0x9a7fc30f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3d1fc317, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x8adfc319, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa6ffc31b, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dfc31e, SUC_lexicon_tags_17 },
    { 0xf3bfc31f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbabfc325, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc9fc32f, SUC_lexicon_tags_65 },
    { 0x5f9fc330, SUC_lexicon_tags_8 },
    { 0x4d5fc330, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x747fc334, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x941fc336, SUC_lexicon_tags_17 },
    { 0x645fc337, SUC_lexicon_tags_25 },
    { 0x8b1fc337, SUC_lexicon_tags_23 },
    { 0x55ffc338, SUC_lexicon_tags_23 },
    { 0xac3fc337, SUC_lexicon_tags_27 },
    { 0xcd7fc336, SUC_lexicon_tags_11 },
    { 0x5d5fc33c, SUC_lexicon_tags_15 },
    { 0xcfbfc33c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x129fc342, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb4bfc344, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9fc34a, SUC_lexicon_tags_14 },
    { 0x80bfc34b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39fc34f, SUC_lexicon_tags_7 },
    { 0x357fc350, SUC_lexicon_tags_15 },
    { 0x5bffc351, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8e3fc353, SUC_lexicon_tags_0 },
    { 0xfffc354, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x33ffc356, SUC_lexicon_tags_31 },
    { 0x9dffc356, SUC_lexicon_tags_45 },
    { 0xc19fc358, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3fc35b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9a5fc35d, SUC_lexicon_tags_55 },
    { 0x403fc35d, SUC_lexicon_tags_37 },
    { 0x2fdfc35e, SUC_lexicon_tags_1 },
    { 0x76dfc360, SUC_lexicon_tags_14 },
    { 0xc2ffc361, SUC_lexicon_tags_38 },
    { 0x529fc362, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x775fc367, SUC_lexicon_tags_37 },
    { 0xf63fc368, SUC_lexicon_tags_25 },
    { 0xd0bfc369, SUC_lexicon_tags_33 },
    { 0x465fc36a, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfdfc36e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x535fc370, SUC_lexicon_tags_39 },
    { 0xafbfc371, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xe97fc373, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19fc37d, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dfc382, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb9fc385, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa71fc387, SUC_lexicon_tags_9 },
    { 0x163fc388, SUC_lexicon_tags_69 },
    { 0x493fc387, SUC_lexicon_tags_25 },
    { 0xe61fc38a, SUC_lexicon_tags_14 },
    { 0xd0bfc38b, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd15fc38e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc7bfc390, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xffdfc39b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15fc39e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x7dfc3a0, SUC_lexicon_tags_15 },
    { 0xd9fc3a1, SUC_lexicon_tags_7 },
    { 0x8abfc3a2, SUC_lexicon_tags_11 },
    { 0xf03fc3a3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163fc3ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfe5fc3ae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9fc3b1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5fc3b5, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bfc3b9, SUC_lexicon_tags_37 },
    { 0xe67fc3ba, SUC_lexicon_tags_107 },
    { 0xbd5fc3bb, SUC_lexicon_tags_26 },
    { 0xd15fc3bc, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87ffc3c0, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fffc3c4, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9ddfc3c9, SUC_lexicon_tags_48 },
    { 0x10bfc3ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9fc3ce, SUC_lexicon_tags_19 },
    { 0x707fc3cf, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xe93fc3d1, SUC_lexicon_tags_20 },
    { 0x1c3fc3d1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbfc3d6, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fffc3df, SUC_lexicon_tags_26 },
    { 0xa15fc3e0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3fc3e3, SUC_lexicon_tags_37 },
    { 0x253fc3e4, SUC_lexicon_tags_84 },
    { 0x413fc3e4, SUC_lexicon_tags_47 },
    { 0x91fc3e3, SUC_lexicon_tags_43 },
    { 0x7fc3e3, SUC_lexicon_tags_7 },
    { 0x9fdfc3e8, SUC_lexicon_tags_47 },
    { 0x3b3fc3e9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68dfc3f0, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xfedfc3f2, SUC_lexicon_tags_40 },
    { 0x149fc3f2, SUC_lexicon_tags_17 },
    { 0x945fc3f3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccffc3fb, SUC_lexicon_tags_60 },
    { 0xc21fc3fc, SUC_lexicon_tags_47 },
    { 0x3adfc3fc, SUC_lexicon_tags_49 },
    { 0xd21fc3fc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabfc403, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd9dfc405, SUC_lexicon_tags_17 },
    { 0x73bfc406, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddfc40d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11fc412, SUC_lexicon_tags_7 },
    { 0x145fc412, SUC_lexicon_tags_37 },
    { 0x381fc413, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89fc41f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x783fc421, SUC_lexicon_tags_33 },
    { 0x7fbfc422, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x349fc424, SUC_lexicon_tags_42 },
    { 0x55fc425, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x103fc42a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfbdfc42c, SUC_lexicon_tags_11 },
    { 0x5d7fc42c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x6fbfc42f, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3fbfc431, SUC_lexicon_tags_37 },
    { 0xfebfc432, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe3fc434, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d7fc438, SUC_lexicon_tags_7 },
    { 0xd7fc439, SUC_lexicon_tags_46 },
    { 0x7cffc43a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x823fc43c, SUC_lexicon_tags_49 },
    { 0xe25fc43d, SUC_lexicon_tags_37 },
    { 0x7f1fc43d, SUC_lexicon_tags_31 },
    { 0x1a9fc43f, SUC_lexicon_tags_31 },
    { 0x10ffc440, SUC_lexicon_tags_15 },
    { 0x2e1fc441, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ffc444, SUC_lexicon_tags_15 },
    { 0xd25fc444, SUC_lexicon_tags_17 },
    { 0x735fc446, SUC_lexicon_tags_54 },
    { 0xab5fc447, SUC_lexicon_tags_11 },
    { 0xe3bfc448, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xf91fc44a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x729fc44f, SUC_lexicon_tags_37 },
    { 0x415fc450, SUC_lexicon_tags_27 },
    { 0xb13fc450, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1fc457, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xe1bfc459, SUC_lexicon_tags_17 },
    { 0xfbffc45a, SUC_lexicon_tags_55 },
    { 0xfcbfc45a, SUC_lexicon_tags_3 },
    { 0x26dfc45c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5abfc461, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ffc464, SUC_lexicon_tags_0 },
    { 0xbb3fc465, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x559fc46e, SUC_lexicon_tags_65 },
    { 0xb29fc46f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763fc474, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7fc479, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xff9fc47c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf03fc47e, SUC_lexicon_tags_30 },
    { 0x55fc47f, SUC_lexicon_tags_43 },
    { 0x211fc47e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x43bfc482, SUC_lexicon_tags_11 },
    { 0xe27fc483, SUC_lexicon_tags_10 },
    { 0x211fc484, SUC_lexicon_tags_2 },
    { 0x473fc485, SUC_lexicon_tags_7 },
    { 0x19fc486, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb97fc48e, SUC_lexicon_tags_28 },
    { 0x48ffc48f, SUC_lexicon_tags_31 },
    { 0x85bfc490, SUC_lexicon_tags_66 },
    { 0xdebfc490, SUC_lexicon_tags_43 },
    { 0x64ffc492, SUC_lexicon_tags_17 },
    { 0xca3fc491, SUC_lexicon_tags_9 },
    { 0x3a7fc493, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa23fc496, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1e5fc498, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1fc49f, SUC_lexicon_tags_30 },
    { 0x7c7fc49f, SUC_lexicon_tags_27 },
    { 0x871fc4a1, SUC_lexicon_tags_25 },
    { 0xafffc4a0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x901fc4a6, SUC_lexicon_tags_25 },
    { 0x81fc4a7, SUC_lexicon_tags_28 },
    { 0x305fc4a7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xd91fc4ab, SUC_lexicon_tags_31 },
    { 0xfd5fc4ac, SUC_lexicon_tags_62 },
    { 0xe37fc4ab, SUC_lexicon_tags_27 },
    { 0x96dfc4ad, SUC_lexicon_tags_39 },
    { 0x251fc4af, SUC_lexicon_tags_1 },
    { 0x5f9fc4b0, SUC_lexicon_tags_14 },
    { 0xb9bfc4af, SUC_lexicon_tags_66 },
    { 0x36dfc4b2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677fc4b6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x12bfc4b8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5fc4bc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2c7fc4be, SUC_lexicon_tags_30 },
    { 0x4dfc4bf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x731fc4c3, SUC_lexicon_tags_57 },
    { 0x7b5fc4c4, SUC_lexicon_tags_33 },
    { 0x3c5fc4c5, SUC_lexicon_tags_25 },
    { 0xb69fc4c3, SUC_lexicon_tags_1 },
    { 0xff1fc4c6, SUC_lexicon_tags_25 },
    { 0x289fc4c8, SUC_lexicon_tags_16 },
    { 0x425fc4c8, SUC_lexicon_tags_19 },
    { 0xb71fc4ca, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddfc4cf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x1c5fc4d1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x367fc4d5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49fc4d8, SUC_lexicon_tags_45 },
    { 0x739fc4d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x489fc4dc, SUC_lexicon_tags_37 },
    { 0x2e5fc4dd, SUC_lexicon_tags_37 },
    { 0xb6ffc4de, SUC_lexicon_tags_32 },
    { 0xc4bfc4df, SUC_lexicon_tags_14 },
    { 0x8ebfc4e0, SUC_lexicon_tags_25 },
    { 0x2a3fc4e1, SUC_lexicon_tags_25 },
    { 0xb7bfc4e0, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc09fc4e4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ddfc4e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x9e3fc4e8, SUC_lexicon_tags_28 },
    { 0xdcdfc4e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ffc4ec, SUC_lexicon_tags_17 },
    { 0x391fc4ed, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7fc4f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xc0bfc4f4, SUC_lexicon_tags_2 },
    { 0x91dfc4f4, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9fc4fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe0ffc4fe, SUC_lexicon_tags_0 },
    { 0x873fc4ff, SUC_lexicon_tags_60 },
    { 0xc23fc500, SUC_lexicon_tags_15 },
    { 0x275fc500, SUC_lexicon_tags_32 },
    { 0x467fc500, SUC_lexicon_tags_17 },
    { 0xeffc503, SUC_lexicon_tags_19 },
    { 0x21dfc503, SUC_lexicon_tags_17 },
    { 0xea7fc505, SUC_lexicon_tags_58 },
    { 0xeb7fc506, SUC_lexicon_tags_12 },
    { 0x615fc507, SUC_lexicon_tags_19 },
    { 0x787fc504, SUC_lexicon_tags_7 },
    { 0x89fc509, SUC_lexicon_tags_23 },
    { 0x293fc50a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2d9fc50c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ffc510, SUC_lexicon_tags_37 },
    { 0xe1bfc511, SUC_lexicon_tags_37 },
    { 0x743fc512, SUC_lexicon_tags_117 },
    { 0x9b1fc513, SUC_lexicon_tags_14 },
    { 0x359fc510, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1affc518, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fbfc51b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5fc51f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xfbbfc521, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9bfc526, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525fc52a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf5ffc52c, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ffc52f, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bbfc534, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x47dfc536, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dfc53a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x187fc53d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1fc541, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd5bfc543, SUC_lexicon_tags_59 },
    { 0x6a9fc544, SUC_lexicon_tags_23 },
    { 0x87fc545, SUC_lexicon_tags_15 },
    { 0x4c9fc543, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fc54d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xfd1fc54f, SUC_lexicon_tags_80 },
    { 0xeedfc550, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3fc555, SUC_lexicon_tags_1079 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bffc558, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x155fc55a, SUC_lexicon_tags_19 },
    { 0x8bbfc55b, SUC_lexicon_tags_38 },
    { 0xe83fc55a, SUC_lexicon_tags_7 },
    { 0x88dfc55c, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x15dfc560, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdfc565, SUC_lexicon_tags_31 },
    { 0xea7fc566, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05fc569, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bfc56c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xde7fc56e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23fc572, SUC_lexicon_tags_12 },
    { 0x997fc573, SUC_lexicon_tags_82 },
    { 0xaaffc573, SUC_lexicon_tags_27 },
    { 0x7dfc574, SUC_lexicon_tags_1 },
    { 0xb19fc575, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe79fc578, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc6bfc583, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31bfc587, SUC_lexicon_tags_33 },
    { 0x7adfc587, SUC_lexicon_tags_15 },
    { 0x849fc589, SUC_lexicon_tags_12 },
    { 0x95dfc58a, SUC_lexicon_tags_8 },
    { 0x2f5fc58a, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0xd19fc58d, SUC_lexicon_tags_35 },
    { 0x419fc58d, SUC_lexicon_tags_23 },
    { 0x5bdfc58e, SUC_lexicon_tags_47 },
    { 0xd89fc590, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ffc594, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe31fc596, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa2dfc598, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x8bfc59b, SUC_lexicon_tags_25 },
    { 0x5c1fc59b, SUC_lexicon_tags_11 },
    { 0x4bffc59d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf15fc5a3, SUC_lexicon_tags_43 },
    { 0xbedfc5a4, SUC_lexicon_tags_15 },
    { 0x889fc5a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd85fc5a7, SUC_lexicon_tags_48 },
    { 0x845fc5a8, SUC_lexicon_tags_3 },
    { 0x4c5fc5a8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x65dfc5ab, SUC_lexicon_tags_11 },
    { 0x37fc5ac, SUC_lexicon_tags_15 },
    { 0xd1ffc5ac, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7fc5b4, SUC_lexicon_tags_31 },
    { 0x8b1fc5b5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x4b5fc5b7, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x17dfc5ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x201fc5bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb3fc5c7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x7effc5ca, SUC_lexicon_tags_30 },
    { 0x5dfc5ca, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x27dfc5cd, SUC_lexicon_tags_7 },
    { 0x67fc5cd, SUC_lexicon_tags_2 },
    { 0xd7fc5ce, SUC_lexicon_tags_2 },
    { 0x171fc5d0, SUC_lexicon_tags_15 },
    { 0xaddfc5d1, SUC_lexicon_tags_47 },
    { 0x341fc5d2, SUC_lexicon_tags_1 },
    { 0xdb5fc5d0, SUC_lexicon_tags_23 },
    { 0x80dfc5d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x131fc5d7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d5fc5db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98dfc5e1, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x4d1fc5e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb77fc5e5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2a1fc5e7, SUC_lexicon_tags_16 },
    { 0x73ffc5e7, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xaa5fc5ea, SUC_lexicon_tags_7 },
    { 0x33ffc5ea, SUC_lexicon_tags_11 },
    { 0x155fc5ec, SUC_lexicon_tags_114 },
    { 0xa85fc5ed, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0xa4bfc5ef, SUC_lexicon_tags_16 },
    { 0xc23fc5f0, SUC_lexicon_tags_17 },
    { 0x9d9fc5f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5e7fc5f3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xaa7fc5f5, SUC_lexicon_tags_84 },
    { 0x95ffc5f5, SUC_lexicon_tags_31 },
    { 0xd6dfc5f6, SUC_lexicon_tags_12 },
    { 0x645fc5f7, SUC_lexicon_tags_21 },
    { 0xd59fc5f6, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f3fc5fc, SUC_lexicon_tags_12 },
    { 0x88ffc5fd, SUC_lexicon_tags_14 },
    { 0x4d1fc5fe, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867fc603, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x883fc605, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe41fc60b, SUC_lexicon_tags_25 },
    { 0x24bfc60c, SUC_lexicon_tags_26 },
    { 0x699fc60d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3ddfc60f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43ffc616, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x755fc61a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa91fc61d, SUC_lexicon_tags_30 },
    { 0xf3ffc61d, SUC_lexicon_tags_0 },
    { 0xf99fc61f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x495fc622, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0xc87fc624, SUC_lexicon_tags_12 },
    { 0xeaffc625, SUC_lexicon_tags_10 },
    { 0xbbbfc626, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x145fc629, SUC_lexicon_tags_27 },
    { 0x37fc629, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8effc62f, SUC_lexicon_tags_31 },
    { 0xb5ffc630, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fc636, SUC_lexicon_tags_12 },
    { 0x611fc636, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf43fc63e, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x42dfc640, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4bbfc648, SUC_lexicon_tags_43 },
    { 0xe17fc649, SUC_lexicon_tags_7 },
    { 0x34ffc64a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239fc64f, SUC_lexicon_tags_30 },
    { 0x36bfc650, SUC_lexicon_tags_12 },
    { 0xafffc651, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905fc655, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77fc65b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd59fc660, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5fc664, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a5fc668, SUC_lexicon_tags_84 },
    { 0x12bfc669, SUC_lexicon_tags_11 },
    { 0xd17fc669, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dffc66e, SUC_lexicon_tags_11 },
    { 0xc51fc66e, SUC_lexicon_tags_1 },
    { 0x35dfc670, SUC_lexicon_tags_1 },
    { 0xa1fc670, SUC_lexicon_tags_9 },
    { 0x2bfc670, SUC_lexicon_tags_1 },
    { 0xb7bfc673, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcd9fc675, SUC_lexicon_tags_7 },
    { 0x4b7fc675, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0xc9fc678, SUC_lexicon_tags_17 },
    { 0x61fc678, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x6effc67c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8fdfc67f, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bfc686, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11fc68a, SUC_lexicon_tags_2 },
    { 0x4d5fc68a, SUC_lexicon_tags_23 },
    { 0xb49fc68c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4fdfc68e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x34dfc690, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xaf7fc692, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9fc698, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xadffc69a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddfc69d, SUC_lexicon_tags_30 },
    { 0x9a5fc69d, SUC_lexicon_tags_30 },
    { 0xc63fc69f, SUC_lexicon_tags_25 },
    { 0xb45fc69e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x493fc6a2, SUC_lexicon_tags_37 },
    { 0x4c3fc6a2, SUC_lexicon_tags_87 },
    { 0x5d7fc6a4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe55fc6a9, SUC_lexicon_tags_1 },
    { 0x77bfc6a9, SUC_lexicon_tags_87 },
    { 0x731fc6aa, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269fc6af, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x93ffc6b1, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x661fc6b3, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dfc6be, SUC_lexicon_tags_26 },
    { 0xb51fc6bf, SUC_lexicon_tags_66 },
    { 0x61bfc6bf, SUC_lexicon_tags_2 },
    { 0xd33fc6c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x811fc6c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2ffc6c9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfebfc6cb, SUC_lexicon_tags_15 },
    { 0x5e5fc6cc, SUC_lexicon_tags_37 },
    { 0x20bfc6cd, SUC_lexicon_tags_37 },
    { 0x65bfc6cd, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xb9ffc6d0, SUC_lexicon_tags_1 },
    { 0xd75fc6d1, SUC_lexicon_tags_11 },
    { 0x2adfc6d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xacbfc6d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x5d9fc6d6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ffc6da, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x651fc6dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x63ffc6df, SUC_lexicon_tags_398 },
    { 0x779fc6e0, SUC_lexicon_tags_17 },
    { 0x5f1fc6e0, SUC_lexicon_tags_3 },
    { 0x475fc6e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x609fc6e4, SUC_lexicon_tags_15 },
    { 0x5b3fc6e4, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9fc6e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17fc6ee, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb5dfc6f0, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x269fc6f7, SUC_lexicon_tags_71 },
    { 0x375fc6f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83fc6ff, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d5fc703, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xa09fc705, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73fc71a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfb5fc71c, SUC_lexicon_tags_29 },
    { 0xf6ffc71d, SUC_lexicon_tags_9 },
    { 0xc7fc71d, SUC_lexicon_tags_43 },
    { 0x435fc71e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fc725, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973fc72a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xa67fc72d, SUC_lexicon_tags_15 },
    { 0xb79fc72e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6dbfc730, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32ffc736, SUC_lexicon_tags_9 },
    { 0xc7ffc737, SUC_lexicon_tags_16 },
    { 0xfaffc737, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1fc73c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x197fc73e, SUC_lexicon_tags_37 },
    { 0x39ffc73f, SUC_lexicon_tags_83 },
    { 0x895fc73f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fffc746, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x179fc748, SUC_lexicon_tags_25 },
    { 0x5a9fc749, SUC_lexicon_tags_13 },
    { 0x15fc748, SUC_lexicon_tags_33 },
    { 0x5cffc748, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f7fc74e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x979fc750, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1fc754, SUC_lexicon_tags_7 },
    { 0x9f5fc754, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d9fc75a, SUC_lexicon_tags_7 },
    { 0x391fc75b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a3fc75e, SUC_lexicon_tags_2 },
    { 0x517fc75f, SUC_lexicon_tags_1 },
    { 0x36dfc760, SUC_lexicon_tags_43 },
    { 0x98ffc761, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933fc767, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xccdfc769, SUC_lexicon_tags_1 },
    { 0xd87fc769, SUC_lexicon_tags_45 },
    { 0x1c3fc76b, SUC_lexicon_tags_49 },
    { 0xc31fc76b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x8a3fc76e, SUC_lexicon_tags_30 },
    { 0xff9fc76f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52bfc774, SUC_lexicon_tags_28 },
    { 0xa63fc775, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x79bfc777, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x605fc77b, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcffc780, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf93fc785, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x81dfc787, SUC_lexicon_tags_59 },
    { 0x183fc788, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe91fc793, SUC_lexicon_tags_17 },
    { 0xc2ffc793, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0xeaffc796, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fffc79d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcebfc7a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43bfc7af, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1fc7b3, SUC_lexicon_tags_49 },
    { 0x53dfc7b3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x227fc7b6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ffc7ba, SUC_lexicon_tags_14 },
    { 0x387fc7ba, SUC_lexicon_tags_12 },
    { 0xd07fc7bc, SUC_lexicon_tags_25 },
    { 0x587fc7bd, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23fc7c1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bfc7c4, SUC_lexicon_tags_15 },
    { 0xf5bfc7c4, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd35fc7c9, SUC_lexicon_tags_32 },
    { 0x83ffc7ca, SUC_lexicon_tags_61 },
    { 0x273fc7ca, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919fc7d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xba9fc7d3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fbfc7d9, SUC_lexicon_tags_7 },
    { 0x661fc7d9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc37fc7dc, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x787fc7df, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x539fc7e2, SUC_lexicon_tags_84 },
    { 0x715fc7e3, SUC_lexicon_tags_49 },
    { 0xa1dfc7e4, SUC_lexicon_tags_37 },
    { 0x1dbfc7e3, SUC_lexicon_tags_17 },
    { 0xe97fc7e2, SUC_lexicon_tags_1 },
    { 0xaeffc7e7, SUC_lexicon_tags_16 },
    { 0x945fc7e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbffc7ee, SUC_lexicon_tags_11 },
    { 0xc51fc7ef, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bfc7f7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x91dfc7f9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bfc7fc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x49bfc7ff, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2abfc802, SUC_lexicon_tags_19 },
    { 0x58ffc802, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1fc809, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9fc80e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddfc811, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd55fc813, SUC_lexicon_tags_15 },
    { 0xc45fc813, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa23fc81a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x35fc81c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x37bfc81e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xedffc820, SUC_lexicon_tags_23 },
    { 0xa53fc821, SUC_lexicon_tags_8 },
    { 0x3d1fc822, SUC_lexicon_tags_14 },
    { 0x95bfc823, SUC_lexicon_tags_2 },
    { 0xc17fc823, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x549fc829, SUC_lexicon_tags_60 },
    { 0xf5ffc82a, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x945fc82d, SUC_lexicon_tags_63 },
    { 0x711fc82e, SUC_lexicon_tags_33 },
    { 0x54dfc82e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bfc835, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427fc839, SUC_lexicon_tags_6 },
    { 0x349fc83a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3fc83d, SUC_lexicon_tags_49 },
    { 0x15bfc83e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435fc84f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa0ffc854, SUC_lexicon_tags_17 },
    { 0x40bfc855, SUC_lexicon_tags_60 },
    { 0x22dfc854, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfc862, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x747fc864, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e5fc867, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x4e3fc869, SUC_lexicon_tags_116 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bfc86c, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x917fc870, SUC_lexicon_tags_11 },
    { 0xf87fc871, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07fc876, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd91fc87a, SUC_lexicon_tags_15 },
    { 0x89dfc87a, SUC_lexicon_tags_25 },
    { 0x995fc87c, SUC_lexicon_tags_1 },
    { 0xcf7fc87d, SUC_lexicon_tags_4 },
    { 0xb63fc87d, SUC_lexicon_tags_21 },
    { 0xf75fc87c, SUC_lexicon_tags_60 },
    { 0x85ffc87e, SUC_lexicon_tags_31 },
    { 0xe5ffc87e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a7fc889, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bdfc88c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1affc88e, SUC_lexicon_tags_43 },
    { 0xcffc88f, SUC_lexicon_tags_37 },
    { 0x7edfc890, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xc41fc893, SUC_lexicon_tags_768 },
    { 0x9bfc894, SUC_lexicon_tags_11 },
    { 0x9d7fc895, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7fc89d, SUC_lexicon_tags_14 },
    { 0xa47fc89e, SUC_lexicon_tags_9 },
    { 0x9d1fc89f, SUC_lexicon_tags_82 },
    { 0xb1bfc89d, SUC_lexicon_tags_10 },
    { 0x21bfc8a1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x6bffc8a4, SUC_lexicon_tags_47 },
    { 0x66bfc8a5, SUC_lexicon_tags_1 },
    { 0x74dfc8a4, SUC_lexicon_tags_2 },
    { 0x913fc8a7, SUC_lexicon_tags_19 },
    { 0x137fc8a8, SUC_lexicon_tags_21 },
    { 0xc8bfc8a9, SUC_lexicon_tags_15 },
    { 0xd0dfc8a9, SUC_lexicon_tags_1 },
    { 0xccffc8a6, SUC_lexicon_tags_265 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdfc8ae, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf07fc8b1, SUC_lexicon_tags_27 },
    { 0xd03fc8b2, SUC_lexicon_tags_4 },
    { 0x655fc8b3, SUC_lexicon_tags_23 },
    { 0x69bfc8b4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47fc8ba, SUC_lexicon_tags_34 },
    { 0x273fc8bb, SUC_lexicon_tags_32 },
    { 0x603fc8bc, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9fc8c1, SUC_lexicon_tags_17 },
    { 0xd6dfc8c2, SUC_lexicon_tags_30 },
    { 0x377fc8c3, SUC_lexicon_tags_0 },
    { 0x837fc8c3, SUC_lexicon_tags_29 },
    { 0xee7fc8c5, SUC_lexicon_tags_30 },
    { 0x889fc8c6, SUC_lexicon_tags_7 },
    { 0xbcdfc8c6, SUC_lexicon_tags_451 },
    { 0x41fc8c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dfc8cc, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x17bfc8cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x235fc8d3, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xa1ffc8d5, SUC_lexicon_tags_16 },
    { 0xe43fc8d6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7fc8dc, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21fc8e1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdfc8e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x673fc8e7, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb43fc8e9, SUC_lexicon_tags_61 },
    { 0x61ffc8ea, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7d7fc8ec, SUC_lexicon_tags_47 },
    { 0xdbbfc8ed, SUC_lexicon_tags_21 },
    { 0x5f7fc8ec, SUC_lexicon_tags_11 },
    { 0x14bfc8ef, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeffc8f4, SUC_lexicon_tags_63 },
    { 0xc41fc8f5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcbfc8fb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5fc8ff, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x443fc901, SUC_lexicon_tags_63 },
    { 0x17bfc902, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39fc906, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x5c7fc908, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd81fc90a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bfc90e, SUC_lexicon_tags_2 },
    { 0x7a7fc90f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x59fc911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5fc916, SUC_lexicon_tags_49 },
    { 0x46ffc917, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa07fc919, SUC_lexicon_tags_12 },
    { 0xdd1fc91a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x38bfc91d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a3fc921, SUC_lexicon_tags_37 },
    { 0xc41fc922, SUC_lexicon_tags_12 },
    { 0x871fc921, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x67fc925, SUC_lexicon_tags_68 },
    { 0xc25fc925, SUC_lexicon_tags_37 },
    { 0x257fc926, SUC_lexicon_tags_11 },
    { 0x985fc928, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bfc92d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x201fc92f, SUC_lexicon_tags_66 },
    { 0x8abfc930, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc17fc936, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xcd1fc938, SUC_lexicon_tags_39 },
    { 0xbf7fc938, SUC_lexicon_tags_10 },
    { 0x8f1fc939, SUC_lexicon_tags_79 },
    { 0xbbdfc93b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x401fc93d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13fc943, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x265fc946, SUC_lexicon_tags_15 },
    { 0xec1fc947, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3fc94b, SUC_lexicon_tags_49 },
    { 0x935fc94c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa71fc955, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf1fc95c, SUC_lexicon_tags_60 },
    { 0xb0ffc95d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x46ffc960, SUC_lexicon_tags_637 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x603fc965, SUC_lexicon_tags_70 },
    { 0x81fc965, SUC_lexicon_tags_14 },
    { 0xe37fc965, SUC_lexicon_tags_68 },
    { 0xd9bfc968, SUC_lexicon_tags_69 },
    { 0x77ffc965, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbaffc970, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x351fc974, SUC_lexicon_tags_47 },
    { 0x6fbfc975, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7fc980, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xc3ffc982, SUC_lexicon_tags_26 },
    { 0xd3dfc982, SUC_lexicon_tags_12 },
    { 0xbcbfc983, SUC_lexicon_tags_1 },
    { 0x79dfc985, SUC_lexicon_tags_25 },
    { 0xc2ffc986, SUC_lexicon_tags_19 },
    { 0x8c1fc987, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8f5fc989, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa95fc98e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dfc994, SUC_lexicon_tags_37 },
    { 0x251fc994, SUC_lexicon_tags_47 },
    { 0xe21fc996, SUC_lexicon_tags_82 },
    { 0xbddfc997, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5b7fc999, SUC_lexicon_tags_45 },
    { 0x39ffc99a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x813fc9a0, SUC_lexicon_tags_37 },
    { 0x5e1fc9a1, SUC_lexicon_tags_22 },
    { 0xa5dfc9a2, SUC_lexicon_tags_32 },
    { 0x6dfc9a3, SUC_lexicon_tags_6 },
    { 0x67dfc9a3, SUC_lexicon_tags_37 },
    { 0xf15fc9a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x695fc9a7, SUC_lexicon_tags_4 },
    { 0x5bffc9a8, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0x679fc9ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9a1fc9ad, SUC_lexicon_tags_21 },
    { 0xd83fc9ae, SUC_lexicon_tags_220 },
    { 0x28ffc9ae, SUC_lexicon_tags_11 },
    { 0xb6bfc9b0, SUC_lexicon_tags_23 },
    { 0x46ffc9b1, SUC_lexicon_tags_47 },
    { 0xd35fc9b2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x155fc9b6, SUC_lexicon_tags_37 },
    { 0x101fc9b7, SUC_lexicon_tags_26 },
    { 0x95fc9b8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xbbffc9bb, SUC_lexicon_tags_25 },
    { 0x789fc9bc, SUC_lexicon_tags_19 },
    { 0x273fc9bd, SUC_lexicon_tags_48 },
    { 0x591fc9be, SUC_lexicon_tags_27 },
    { 0xdaffc9bf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56ffc9c3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca7fc9c8, SUC_lexicon_tags_12 },
    { 0xc6dfc9c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c7fc9cd, SUC_lexicon_tags_37 },
    { 0xfd5fc9ce, SUC_lexicon_tags_60 },
    { 0x847fc9cf, SUC_lexicon_tags_43 },
    { 0x3ebfc9d0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x8dbfc9d2, SUC_lexicon_tags_17 },
    { 0xa73fc9d2, SUC_lexicon_tags_77 },
    { 0x50bfc9d4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cffc9d8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xacdfc9db, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x645fc9de, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xadffc9e4, SUC_lexicon_tags_351 },
    { 0xe9bfc9e5, SUC_lexicon_tags_0 },
    { 0x65bfc9e4, SUC_lexicon_tags_259 },
    { 0xd0dfc9e7, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xc9bfc9e9, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0xc09fc9ec, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x277fc9ee, SUC_lexicon_tags_11 },
    { 0x51fc9ef, SUC_lexicon_tags_49 },
    { 0x15fc9f0, SUC_lexicon_tags_29 },
    { 0xf99fc9f0, SUC_lexicon_tags_69 },
    { 0x773fc9f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1dfc9f6, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x423fc9ff, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x43bfca01, SUC_lexicon_tags_57 },
    { 0x55fca02, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd1fca07, SUC_lexicon_tags_17 },
    { 0xcd7fca07, SUC_lexicon_tags_37 },
    { 0xd55fca07, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xa31fca0b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bfca0f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bfca14, SUC_lexicon_tags_44 },
    { 0xcdbfca14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd21fca1b, SUC_lexicon_tags_15 },
    { 0xcb9fca1c, SUC_lexicon_tags_11 },
    { 0x415fca1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x729fca20, SUC_lexicon_tags_37 },
    { 0x5d1fca21, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc99fca28, SUC_lexicon_tags_10 },
    { 0xb6bfca29, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc7fca2b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67fca2f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x43bfca31, SUC_lexicon_tags_2 },
    { 0x195fca32, SUC_lexicon_tags_26 },
    { 0x309fca33, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6f5fca35, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x931fca37, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x525fca39, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0xcdbfca3b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x58dfca3f, SUC_lexicon_tags_11 },
    { 0xd5ffca3f, SUC_lexicon_tags_31 },
    { 0x17ffca3f, SUC_lexicon_tags_0 },
    { 0xfdbfca42, SUC_lexicon_tags_11 },
    { 0x699fca42, SUC_lexicon_tags_25 },
    { 0xcaffca41, SUC_lexicon_tags_32 },
    { 0xe1bfca45, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbebfca47, SUC_lexicon_tags_7 },
    { 0x4f3fca48, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa19fca4b, SUC_lexicon_tags_45 },
    { 0x41ffca4c, SUC_lexicon_tags_124 },
    { 0x68ffca4d, SUC_lexicon_tags_25 },
    { 0x88ffca4d, SUC_lexicon_tags_46 },
    { 0x6fffca4f, SUC_lexicon_tags_23 },
    { 0x32bfca50, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xcfffca52, SUC_lexicon_tags_35 },
    { 0x949fca53, SUC_lexicon_tags_14 },
    { 0x1affca52, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bfca58, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dfca5d, SUC_lexicon_tags_146 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x867fca61, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5fca69, SUC_lexicon_tags_23 },
    { 0x493fca6a, SUC_lexicon_tags_38 },
    { 0x217fca6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x62ffca6d, SUC_lexicon_tags_0 },
    { 0x4b9fca6e, SUC_lexicon_tags_66 },
    { 0xdb5fca6f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xe0ffca71, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf67fca79, SUC_lexicon_tags_37 },
    { 0x887fca79, SUC_lexicon_tags_2 },
    { 0x595fca7b, SUC_lexicon_tags_41 },
    { 0x1cdfca7c, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0x5dbfca7e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf55fca84, SUC_lexicon_tags_7 },
    { 0x4bdfca85, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa6bfca87, SUC_lexicon_tags_2 },
    { 0x5fca88, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x2e9fca8a, SUC_lexicon_tags_12 },
    { 0x1a3fca8b, SUC_lexicon_tags_15 },
    { 0x83fca8a, SUC_lexicon_tags_7 },
    { 0x37ffca8d, SUC_lexicon_tags_37 },
    { 0x483fca8e, SUC_lexicon_tags_11 },
    { 0x70bfca8f, SUC_lexicon_tags_26 },
    { 0xa27fca90, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x645fca92, SUC_lexicon_tags_27 },
    { 0x46bfca93, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa37fca95, SUC_lexicon_tags_7 },
    { 0x7fca96, SUC_lexicon_tags_17 },
    { 0x6d7fca97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x173fca9a, SUC_lexicon_tags_25 },
    { 0x631fca9a, SUC_lexicon_tags_37 },
    { 0x41bfca9c, SUC_lexicon_tags_38 },
    { 0x441fca9d, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xc8ffca9f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3fcaa2, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x325fcaa4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931fcaab, SUC_lexicon_tags_23 },
    { 0xc4dfcaab, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edfcab0, SUC_lexicon_tags_7 },
    { 0x3dbfcab1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dfcab4, SUC_lexicon_tags_17 },
    { 0xae1fcab5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62ffcaba, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0x423fcabd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905fcac1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x873fcac9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09fcacf, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6fffcad3, SUC_lexicon_tags_63 },
    { 0x277fcad4, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadfcade, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xb15fcae0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfaffcae2, SUC_lexicon_tags_25 },
    { 0x479fcae3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37fcae8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x59ffcaeb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741fcaef, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x92bfcaf2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dfcaf7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xf9bfcaf9, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x25fcafb, SUC_lexicon_tags_113 },
    { 0xbb3fcafc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x58bfcafe, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x7f9fcb00, SUC_lexicon_tags_55 },
    { 0xbddfcb01, SUC_lexicon_tags_1 },
    { 0x5c1fcb02, SUC_lexicon_tags_28 },
    { 0x41ffcb03, SUC_lexicon_tags_2 },
    { 0xf25fcb04, SUC_lexicon_tags_70 },
    { 0xafdfcb02, SUC_lexicon_tags_37 },
    { 0x207fcb06, SUC_lexicon_tags_12 },
    { 0x789fcb06, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3fcb0d, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x885fcb0f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4a5fcb11, SUC_lexicon_tags_14 },
    { 0x45bfcb12, SUC_lexicon_tags_6 },
    { 0x1cbfcb12, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9fcb16, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b1fcb18, SUC_lexicon_tags_49 },
    { 0x777fcb18, SUC_lexicon_tags_15 },
    { 0x51ffcb1a, SUC_lexicon_tags_23 },
    { 0x39fcb1a, SUC_lexicon_tags_30 },
    { 0x8cffcb1c, SUC_lexicon_tags_65 },
    { 0x545fcb18, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x123fcb20, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x40dfcb22, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fffcb2b, SUC_lexicon_tags_40 },
    { 0x96ffcb2c, SUC_lexicon_tags_10 },
    { 0x61fcb2d, SUC_lexicon_tags_14 },
    { 0xabbfcb2e, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x837fcb30, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fcb34, SUC_lexicon_tags_25 },
    { 0xe3fcb34, SUC_lexicon_tags_47 },
    { 0x74bfcb34, SUC_lexicon_tags_48 },
    { 0xcb3fcb34, SUC_lexicon_tags_17 },
    { 0x6cdfcb38, SUC_lexicon_tags_2 },
    { 0xbc9fcb38, SUC_lexicon_tags_27 },
    { 0x2fdfcb38, SUC_lexicon_tags_15 },
    { 0x1b9fcb3a, SUC_lexicon_tags_15 },
    { 0xe71fcb36, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bfcb3f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x731fcb42, SUC_lexicon_tags_49 },
    { 0xde3fcb42, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xebffcb45, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e1fcb4a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbfcb4f, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adfcb52, SUC_lexicon_tags_14 },
    { 0x4bfcb52, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33dfcb55, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xe7dfcb57, SUC_lexicon_tags_15 },
    { 0x357fcb57, SUC_lexicon_tags_6 },
    { 0xb65fcb57, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x451fcb5d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x94bfcb62, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x985fcb64, SUC_lexicon_tags_37 },
    { 0xedffcb64, SUC_lexicon_tags_12 },
    { 0x463fcb66, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf31fcb6c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeddfcb74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bdfcb7b, SUC_lexicon_tags_23 },
    { 0x11bfcb7b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x4cdfcb7e, SUC_lexicon_tags_65 },
    { 0x28dfcb7f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xe67fcb81, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf89fcb84, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x55bfcb86, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5fcb8f, SUC_lexicon_tags_33 },
    { 0x5d9fcb90, SUC_lexicon_tags_14 },
    { 0x163fcb91, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x1d1fcb93, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fcb97, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdfcb9d, SUC_lexicon_tags_44 },
    { 0xa09fcb9e, SUC_lexicon_tags_134 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1fcba1, SUC_lexicon_tags_45 },
    { 0xcedfcba2, SUC_lexicon_tags_15 },
    { 0xc37fcba3, SUC_lexicon_tags_19 },
    { 0x137fcba2, SUC_lexicon_tags_7 },
    { 0x7b9fcba5, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x697fcbab, SUC_lexicon_tags_37 },
    { 0xf1dfcbab, SUC_lexicon_tags_25 },
    { 0xc85fcbac, SUC_lexicon_tags_37 },
    { 0x21dfcbae, SUC_lexicon_tags_31 },
    { 0x6adfcbaf, SUC_lexicon_tags_2 },
    { 0x5bdfcbac, SUC_lexicon_tags_15 },
    { 0x769fcbb1, SUC_lexicon_tags_3 },
    { 0xbe7fcbb1, SUC_lexicon_tags_47 },
    { 0xeedfcbb3, SUC_lexicon_tags_37 },
    { 0x325fcbb0, SUC_lexicon_tags_25 },
    { 0x9b9fcbb5, SUC_lexicon_tags_92 },
    { 0x75dfcbb6, SUC_lexicon_tags_25 },
    { 0x637fcbb6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xabfcbba, SUC_lexicon_tags_15 },
    { 0x389fcbbb, SUC_lexicon_tags_37 },
    { 0x6a3fcbbc, SUC_lexicon_tags_17 },
    { 0x45fcbbd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fcbc0, SUC_lexicon_tags_12 },
    { 0x417fcbc1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdfcbc5, SUC_lexicon_tags_5 },
    { 0xd0dfcbc5, SUC_lexicon_tags_1 },
    { 0x38ffcbc5, SUC_lexicon_tags_11 },
    { 0x171fcbc8, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x471fcbce, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589fcbd3, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963fcbd7, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x473fcbda, SUC_lexicon_tags_17 },
    { 0x1bdfcbda, SUC_lexicon_tags_2 },
    { 0x1d7fcbdc, SUC_lexicon_tags_43 },
    { 0xa3dfcbdc, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdfcbe7, SUC_lexicon_tags_25 },
    { 0x23dfcbe8, SUC_lexicon_tags_5 },
    { 0x123fcbe9, SUC_lexicon_tags_15 },
    { 0x5f1fcbe8, SUC_lexicon_tags_1 },
    { 0x447fcbeb, SUC_lexicon_tags_49 },
    { 0xa5bfcbe9, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x16dfcbee, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa41fcbf6, SUC_lexicon_tags_55 },
    { 0x487fcbf7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edfcbfd, SUC_lexicon_tags_37 },
    { 0x403fcbfe, SUC_lexicon_tags_66 },
    { 0x893fcbfe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc43fcc02, SUC_lexicon_tags_1 },
    { 0xe55fcc03, SUC_lexicon_tags_11 },
    { 0x3e5fcc04, SUC_lexicon_tags_8 },
    { 0x77ffcc04, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x49fcc08, SUC_lexicon_tags_6 },
    { 0x6dfcc09, SUC_lexicon_tags_17 },
    { 0x45fcc0a, SUC_lexicon_tags_6 },
    { 0x35bfcc0b, SUC_lexicon_tags_19 },
    { 0xa03fcc0c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x925fcc14, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9fcc18, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e1fcc1e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe65fcc20, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa71fcc22, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203fcc26, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bfcc29, SUC_lexicon_tags_17 },
    { 0x64ffcc2a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23fcc2f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x791fcc31, SUC_lexicon_tags_14 },
    { 0xfb7fcc32, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9bfcc3b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x765fcc41, SUC_lexicon_tags_31 },
    { 0x48ffcc42, SUC_lexicon_tags_25 },
    { 0x851fcc43, SUC_lexicon_tags_15 },
    { 0xd83fcc44, SUC_lexicon_tags_65 },
    { 0xf63fcc42, SUC_lexicon_tags_7 },
    { 0xe63fcc42, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5fcc4a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xfbbfcc4c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5dbfcc4e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6dffcc50, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x8f7fcc52, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba3fcc59, SUC_lexicon_tags_56 },
    { 0xc35fcc5a, SUC_lexicon_tags_31 },
    { 0xa47fcc5a, SUC_lexicon_tags_92 },
    { 0x2affcc59, SUC_lexicon_tags_37 },
    { 0x883fcc5a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x821fcc60, SUC_lexicon_tags_11 },
    { 0xa89fcc60, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7effcc66, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9fcc6b, SUC_lexicon_tags_46 },
    { 0xdc5fcc6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x175fcc6f, SUC_lexicon_tags_31 },
    { 0xe59fcc6f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb91fcc73, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfedfcc7a, SUC_lexicon_tags_1 },
    { 0xb59fcc7a, SUC_lexicon_tags_48 },
    { 0xf89fcc7c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x3e7fcc7f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa93fcc81, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3fcc8d, SUC_lexicon_tags_7 },
    { 0x721fcc8e, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03fcc94, SUC_lexicon_tags_0 },
    { 0xbdfcc94, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x897fcc9b, SUC_lexicon_tags_19 },
    { 0xa37fcc9c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x743fcc9f, SUC_lexicon_tags_45 },
    { 0xd79fcca0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xe63fcca3, SUC_lexicon_tags_12 },
    { 0x665fcca3, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xd33fcca6, SUC_lexicon_tags_17 },
    { 0x709fcca6, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x79dfccaa, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x90dfccb0, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x313fccb3, SUC_lexicon_tags_11 },
    { 0x46ffccb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf0dfccb6, SUC_lexicon_tags_2 },
    { 0xcedfccb7, SUC_lexicon_tags_17 },
    { 0x167fccb7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb1fccbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bfccc0, SUC_lexicon_tags_11 },
    { 0xc9ffccc1, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cffccc5, SUC_lexicon_tags_30 },
    { 0xb4bfccc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x337fccc8, SUC_lexicon_tags_106 },
    { 0x82bfccc9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbbbfcccb, SUC_lexicon_tags_80 },
    { 0x935fcccc, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fccd2, SUC_lexicon_tags_19 },
    { 0xde7fccd3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x24dfccd9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x197fccdb, SUC_lexicon_tags_27 },
    { 0x11dfccdc, SUC_lexicon_tags_37 },
    { 0xa6dfccdd, SUC_lexicon_tags_26 },
    { 0xfeffccde, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda9fcce5, SUC_lexicon_tags_16 },
    { 0xd73fcce5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x4f3fcce8, SUC_lexicon_tags_9 },
    { 0x925fcce9, SUC_lexicon_tags_37 },
    { 0x6e1fccea, SUC_lexicon_tags_17 },
    { 0x657fcce8, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x4cdfcced, SUC_lexicon_tags_11 },
    { 0xcbbfccee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa0ffccf0, SUC_lexicon_tags_26 },
    { 0xb1dfccf1, SUC_lexicon_tags_16 },
    { 0x403fccf2, SUC_lexicon_tags_68 },
    { 0x5fdfccf0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x44ffccf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x571fccf8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfffcd00, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d9fcd03, SUC_lexicon_tags_27 },
    { 0x913fcd03, SUC_lexicon_tags_16 },
    { 0x5d7fcd03, SUC_lexicon_tags_35 },
    { 0xfb1fcd04, SUC_lexicon_tags_11 },
    { 0x889fcd07, SUC_lexicon_tags_31 },
    { 0xa37fcd06, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xf6dfcd0b, SUC_lexicon_tags_56 },
    { 0x3cffcd0b, SUC_lexicon_tags_2 },
    { 0x3bbfcd0d, SUC_lexicon_tags_27 },
    { 0x909fcd0c, SUC_lexicon_tags_30 },
    { 0x8e9fcd0f, SUC_lexicon_tags_17 },
    { 0xa81fcd10, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x625fcd14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dfcd18, SUC_lexicon_tags_30 },
    { 0xc65fcd19, SUC_lexicon_tags_6 },
    { 0xee9fcd1a, SUC_lexicon_tags_17 },
    { 0x73ffcd1b, SUC_lexicon_tags_6 },
    { 0x18ffcd1b, SUC_lexicon_tags_11 },
    { 0x291fcd1d, SUC_lexicon_tags_65 },
    { 0x29bfcd1e, SUC_lexicon_tags_27 },
    { 0x83fcd1d, SUC_lexicon_tags_14 },
    { 0xbeffcd1f, SUC_lexicon_tags_23 },
    { 0xe9fcd21, SUC_lexicon_tags_65 },
    { 0x70dfcd22, SUC_lexicon_tags_19 },
    { 0xee9fcd22, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xe2bfcd26, SUC_lexicon_tags_1428 },
    { 0, NULL },
    { 0xf31fcd28, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x5c1fcd2a, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x48dfcd2c, SUC_lexicon_tags_2 },
    { 0x599fcd2d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821fcd31, SUC_lexicon_tags_25 },
    { 0x365fcd32, SUC_lexicon_tags_14 },
    { 0x22dfcd33, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xbbffcd35, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcffcd37, SUC_lexicon_tags_37 },
    { 0xc47fcd38, SUC_lexicon_tags_12 },
    { 0x663fcd38, SUC_lexicon_tags_0 },
    { 0xd6bfcd3a, SUC_lexicon_tags_26 },
    { 0xe63fcd3a, SUC_lexicon_tags_12 },
    { 0xf01fcd38, SUC_lexicon_tags_25 },
    { 0x831fcd3b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c5fcd41, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x651fcd49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x689fcd4f, SUC_lexicon_tags_15 },
    { 0x579fcd4f, SUC_lexicon_tags_27 },
    { 0x815fcd51, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdfcd54, SUC_lexicon_tags_25 },
    { 0xb1dfcd55, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ffcd5a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x87fcd5c, SUC_lexicon_tags_1197 },
    { 0, NULL },
    { 0, NULL },
    { 0x387fcd5f, SUC_lexicon_tags_14 },
    { 0x7c1fcd5f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xbe7fcd62, SUC_lexicon_tags_11 },
    { 0x839fcd63, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77fcd67, SUC_lexicon_tags_31 },
    { 0x2a7fcd68, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd79fcd6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fcd6d, SUC_lexicon_tags_11 },
    { 0x929fcd6e, SUC_lexicon_tags_15 },
    { 0x7b1fcd6d, SUC_lexicon_tags_17 },
    { 0xecdfcd6f, SUC_lexicon_tags_32 },
    { 0x561fcd71, SUC_lexicon_tags_27 },
    { 0xf5dfcd72, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1fcd75, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6edfcd77, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd4dfcd79, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x545fcd7d, SUC_lexicon_tags_33 },
    { 0x485fcd7d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb23fcd80, SUC_lexicon_tags_55 },
    { 0xd2bfcd80, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92bfcd86, SUC_lexicon_tags_43 },
    { 0xc65fcd87, SUC_lexicon_tags_17 },
    { 0xe55fcd88, SUC_lexicon_tags_37 },
    { 0x9e9fcd89, SUC_lexicon_tags_3 },
    { 0xd15fcd88, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abfcd91, SUC_lexicon_tags_77 },
    { 0x9f3fcd92, SUC_lexicon_tags_6 },
    { 0xd45fcd93, SUC_lexicon_tags_34 },
    { 0x69bfcd94, SUC_lexicon_tags_11 },
    { 0xac3fcd92, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x90bfcd97, SUC_lexicon_tags_7 },
    { 0x661fcd98, SUC_lexicon_tags_45 },
    { 0x4d5fcd97, SUC_lexicon_tags_25 },
    { 0x687fcd9a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dfcda7, SUC_lexicon_tags_33 },
    { 0x44bfcda8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x319fcdab, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x307fcdae, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x223fcdb1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ffcdb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf23fcdb7, SUC_lexicon_tags_1 },
    { 0xa95fcdb8, SUC_lexicon_tags_27 },
    { 0x519fcdb9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83fcdbd, SUC_lexicon_tags_49 },
    { 0xea3fcdbe, SUC_lexicon_tags_12 },
    { 0x227fcdbf, SUC_lexicon_tags_46 },
    { 0x69dfcdc0, SUC_lexicon_tags_7 },
    { 0x75fcdc1, SUC_lexicon_tags_9 },
    { 0xb43fcdc2, SUC_lexicon_tags_17 },
    { 0xb8bfcdbd, SUC_lexicon_tags_17 },
    { 0xce1fcdc0, SUC_lexicon_tags_37 },
    { 0x497fcdbe, SUC_lexicon_tags_17 },
    { 0x39ffcdc6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5fcdcb, SUC_lexicon_tags_17 },
    { 0xf2bfcdcb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d7fcdd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x39fcdd9, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x277fcddb, SUC_lexicon_tags_11 },
    { 0x9dffcddb, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173fcde5, SUC_lexicon_tags_32 },
    { 0xe15fcde6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255fcdea, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xbbffcdec, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x505fcdee, SUC_lexicon_tags_48 },
    { 0xa63fcdef, SUC_lexicon_tags_12 },
    { 0x957fcdf0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb4bfcdf2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa5ffcdf4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x635fcdf6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd29fcdf9, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7ffcdfd, SUC_lexicon_tags_1 },
    { 0xb3ffcdfe, SUC_lexicon_tags_63 },
    { 0x72bfcdff, SUC_lexicon_tags_2 },
    { 0xfcbfce00, SUC_lexicon_tags_255 },
    { 0x2e3fce00, SUC_lexicon_tags_17 },
    { 0x7a3fce00, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51fce07, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x78dfce0a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09fce0d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x51ffce0f, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7fce14, SUC_lexicon_tags_15 },
    { 0x3b1fce14, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ffce1b, SUC_lexicon_tags_98 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc7fce22, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86bfce26, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8dfce2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7fce2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x301fce34, SUC_lexicon_tags_12 },
    { 0xc85fce34, SUC_lexicon_tags_14 },
    { 0xedbfce35, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x8e1fce38, SUC_lexicon_tags_11 },
    { 0x4bbfce39, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x6d3fce3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xf23fce3d, SUC_lexicon_tags_15 },
    { 0xac3fce3e, SUC_lexicon_tags_11 },
    { 0x5f5fce3d, SUC_lexicon_tags_11 },
    { 0xdf7fce40, SUC_lexicon_tags_30 },
    { 0x3dbfce41, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bfce45, SUC_lexicon_tags_12 },
    { 0x8e7fce46, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x40dfce48, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe7fce4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8b7fce50, SUC_lexicon_tags_44 },
    { 0x73fce51, SUC_lexicon_tags_56 },
    { 0x753fce52, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc21fce59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x2b1fce5c, SUC_lexicon_tags_49 },
    { 0xbd9fce5d, SUC_lexicon_tags_26 },
    { 0x339fce5e, SUC_lexicon_tags_14 },
    { 0xe31fce5f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd53fce61, SUC_lexicon_tags_1 },
    { 0x797fce62, SUC_lexicon_tags_12 },
    { 0x857fce63, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ffce67, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x117fce69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7fce6e, SUC_lexicon_tags_12 },
    { 0x4abfce6e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x3f1fce71, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x1a1fce73, SUC_lexicon_tags_2 },
    { 0x533fce74, SUC_lexicon_tags_7 },
    { 0xc91fce74, SUC_lexicon_tags_28 },
    { 0x6bffce76, SUC_lexicon_tags_12 },
    { 0xabffce77, SUC_lexicon_tags_47 },
    { 0x8f7fce78, SUC_lexicon_tags_25 },
    { 0x999fce79, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4ddfce7c, SUC_lexicon_tags_15 },
    { 0x3d9fce7d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2ffce82, SUC_lexicon_tags_41 },
    { 0x319fce83, SUC_lexicon_tags_39 },
    { 0xd61fce82, SUC_lexicon_tags_33 },
    { 0x159fce83, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb9fce88, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xfddfce8a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xa83fce8d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x817fce91, SUC_lexicon_tags_26 },
    { 0x1bffce92, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x3dbfce94, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x945fce96, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7fce9a, SUC_lexicon_tags_1 },
    { 0x3bdfce9a, SUC_lexicon_tags_37 },
    { 0x9c5fce9c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3bfcea0, SUC_lexicon_tags_27 },
    { 0x499fcea0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77fcea5, SUC_lexicon_tags_79 },
    { 0x137fcea6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3fceac, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x6abfceae, SUC_lexicon_tags_7 },
    { 0x27ffceae, SUC_lexicon_tags_17 },
    { 0x775fceb0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2f1fceb4, SUC_lexicon_tags_57 },
    { 0xc8bfceb5, SUC_lexicon_tags_61 },
    { 0xaddfceb6, SUC_lexicon_tags_37 },
    { 0x4affceb7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f5fceba, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cffcebf, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bfcec4, SUC_lexicon_tags_17 },
    { 0x1fffcec5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf45fcecb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x56ffcecd, SUC_lexicon_tags_2 },
    { 0xa8bfcece, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xd39fced0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc07fced4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15dfceda, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1fcedf, SUC_lexicon_tags_11 },
    { 0xd25fcee0, SUC_lexicon_tags_8 },
    { 0x7f5fcee0, SUC_lexicon_tags_68 },
    { 0x4dffcee0, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfcee6, SUC_lexicon_tags_92 },
    { 0x781fcee7, SUC_lexicon_tags_17 },
    { 0xa2dfcee8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf6bfceea, SUC_lexicon_tags_1 },
    { 0x441fceeb, SUC_lexicon_tags_14 },
    { 0xce1fceeb, SUC_lexicon_tags_4 },
    { 0x6bbfceed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bfcef2, SUC_lexicon_tags_7 },
    { 0x2dbfcef3, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xbbfcef5, SUC_lexicon_tags_17 },
    { 0x807fcef6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9fcefa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xd23fcefd, SUC_lexicon_tags_10 },
    { 0x45dfcefe, SUC_lexicon_tags_15 },
    { 0x8a3fcefe, SUC_lexicon_tags_15 },
    { 0x245fcf00, SUC_lexicon_tags_85 },
    { 0xde5fcf00, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x851fcf03, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ffcf06, SUC_lexicon_tags_151 },
    { 0x475fcf07, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7fcf0a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x73fcf0e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4ffcf12, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbfcf15, SUC_lexicon_tags_8 },
    { 0xbd7fcf15, SUC_lexicon_tags_11 },
    { 0x7c5fcf16, SUC_lexicon_tags_15 },
    { 0x4f1fcf17, SUC_lexicon_tags_38 },
    { 0x3d3fcf15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16bfcf1f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x55bfcf22, SUC_lexicon_tags_10 },
    { 0x2fffcf23, SUC_lexicon_tags_46 },
    { 0xcdbfcf23, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x885fcf26, SUC_lexicon_tags_29 },
    { 0x6fbfcf27, SUC_lexicon_tags_1 },
    { 0x6bdfcf28, SUC_lexicon_tags_138 },
    { 0x7b1fcf29, SUC_lexicon_tags_11 },
    { 0xaf5fcf29, SUC_lexicon_tags_31 },
    { 0x263fcf27, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x2f9fcf2d, SUC_lexicon_tags_0 },
    { 0x9e5fcf2d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x267fcf31, SUC_lexicon_tags_25 },
    { 0xff3fcf32, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bfcf3b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x7f9fcf3d, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x643fcf3f, SUC_lexicon_tags_25 },
    { 0x3e7fcf3f, SUC_lexicon_tags_31 },
    { 0x547fcf41, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x857fcf43, SUC_lexicon_tags_7 },
    { 0x43ffcf43, SUC_lexicon_tags_45 },
    { 0xe53fcf45, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xbf9fcf48, SUC_lexicon_tags_30 },
    { 0x89dfcf49, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb05fcf4f, SUC_lexicon_tags_19 },
    { 0x597fcf4f, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5fcf53, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37fcf58, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfaffcf5d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1f1fcf5f, SUC_lexicon_tags_15 },
    { 0xc8ffcf60, SUC_lexicon_tags_37 },
    { 0xbf9fcf61, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x733fcf63, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcabfcf65, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc9fcf68, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x51fcf6a, SUC_lexicon_tags_63 },
    { 0x529fcf6b, SUC_lexicon_tags_47 },
    { 0xcdffcf6a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb31fcf6f, SUC_lexicon_tags_0 },
    { 0x941fcf70, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11fcf7b, SUC_lexicon_tags_56 },
    { 0x347fcf7b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xd83fcf7f, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d5fcf84, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9fcf87, SUC_lexicon_tags_7 },
    { 0x6d5fcf88, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243fcf8e, SUC_lexicon_tags_11 },
    { 0x51ffcf8e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2d1fcf91, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe51fcf96, SUC_lexicon_tags_45 },
    { 0xb03fcf97, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x667fcf99, SUC_lexicon_tags_23 },
    { 0x2bbfcf9a, SUC_lexicon_tags_17 },
    { 0x9b1fcf9b, SUC_lexicon_tags_57 },
    { 0x479fcf9c, SUC_lexicon_tags_31 },
    { 0x707fcf9d, SUC_lexicon_tags_6 },
    { 0x1c9fcf9e, SUC_lexicon_tags_25 },
    { 0xc6ffcf9f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6cbfcfa7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x28bfcfaa, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x52ffcfad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x449fcfaf, SUC_lexicon_tags_55 },
    { 0x367fcfb0, SUC_lexicon_tags_0 },
    { 0x903fcfb1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc39fcfb6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x33dfcfb9, SUC_lexicon_tags_15 },
    { 0x253fcfba, SUC_lexicon_tags_17 },
    { 0x889fcfba, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0, NULL },
    { 0x715fcfbe, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x659fcfc0, SUC_lexicon_tags_25 },
    { 0x27ffcfc0, SUC_lexicon_tags_47 },
    { 0x9cbfcfc1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb75fcfc7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe35fcfc9, SUC_lexicon_tags_0 },
    { 0xb2ffcfca, SUC_lexicon_tags_25 },
    { 0x725fcfca, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xbffcfcd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x631fcfcf, SUC_lexicon_tags_37 },
    { 0xc83fcfd0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59fcfd8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a5fcfdb, SUC_lexicon_tags_7 },
    { 0x77fcfdc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x303fcfe2, SUC_lexicon_tags_1 },
    { 0xeebfcfe2, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cdfcfe7, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45dfcfec, SUC_lexicon_tags_28 },
    { 0x4f3fcfec, SUC_lexicon_tags_1 },
    { 0x1bdfcfed, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabdfcff2, SUC_lexicon_tags_31 },
    { 0x49bfcff3, SUC_lexicon_tags_15 },
    { 0x2c7fcff4, SUC_lexicon_tags_45 },
    { 0x81bfcff4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcb9fcff7, SUC_lexicon_tags_204 },
    { 0xcadfcff8, SUC_lexicon_tags_63 },
    { 0xf6dfcff8, SUC_lexicon_tags_6 },
    { 0xe3fcff7, SUC_lexicon_tags_48 },
    { 0x1d1fcff9, SUC_lexicon_tags_37 },
    { 0x315fcffc, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x979fcffe, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4b5fd000, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xdfbfd003, SUC_lexicon_tags_43 },
    { 0xa35fd004, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xda7fd006, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46ffd00d, SUC_lexicon_tags_25 },
    { 0xfc9fd00e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8dbfd011, SUC_lexicon_tags_60 },
    { 0xe3fd011, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xce5fd015, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc87fd018, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7fd021, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb81fd027, SUC_lexicon_tags_2 },
    { 0x693fd027, SUC_lexicon_tags_37 },
    { 0x67bfd029, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x183fd02b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xed9fd02d, SUC_lexicon_tags_33 },
    { 0x5f1fd02d, SUC_lexicon_tags_11 },
    { 0x107fd02f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3fd033, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc9fd035, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fd038, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x79bfd03b, SUC_lexicon_tags_8 },
    { 0xb6ffd03c, SUC_lexicon_tags_11 },
    { 0x90dfd03c, SUC_lexicon_tags_9 },
    { 0xf55fd03d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x2ddfd040, SUC_lexicon_tags_21 },
    { 0xa37fd041, SUC_lexicon_tags_37 },
    { 0x2b3fd042, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x595fd044, SUC_lexicon_tags_20 },
    { 0xcc5fd044, SUC_lexicon_tags_38 },
    { 0x815fd046, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x52bfd049, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc09fd04b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x4b3fd04d, SUC_lexicon_tags_17 },
    { 0x35dfd04e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdfd052, SUC_lexicon_tags_33 },
    { 0xd5fd053, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd6dfd055, SUC_lexicon_tags_25 },
    { 0x51ffd056, SUC_lexicon_tags_30 },
    { 0xab5fd056, SUC_lexicon_tags_11 },
    { 0x16bfd058, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xe89fd05a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9fd060, SUC_lexicon_tags_11 },
    { 0x7c9fd061, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x311fd063, SUC_lexicon_tags_23 },
    { 0x7cffd063, SUC_lexicon_tags_323 },
    { 0, NULL },
    { 0x75ffd066, SUC_lexicon_tags_17 },
    { 0xc2ffd067, SUC_lexicon_tags_12 },
    { 0xddbfd068, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x8f1fd06a, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xb39fd06c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xde5fd072, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x2bdfd074, SUC_lexicon_tags_36 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ffd077, SUC_lexicon_tags_47 },
    { 0x897fd078, SUC_lexicon_tags_95 },
    { 0, NULL },
    { 0x757fd07a, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cdfd07f, SUC_lexicon_tags_25 },
    { 0x31fd080, SUC_lexicon_tags_31 },
    { 0xd57fd07f, SUC_lexicon_tags_37 },
    { 0x551fd081, SUC_lexicon_tags_37 },
    { 0x613fd083, SUC_lexicon_tags_12 },
    { 0x323fd083, SUC_lexicon_tags_1 },
    { 0x1c5fd085, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13fd08d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xef1fd090, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bfd097, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27dfd09c, SUC_lexicon_tags_33 },
    { 0x5ffd09d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12bfd0a2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x677fd0a4, SUC_lexicon_tags_27 },
    { 0x371fd0a5, SUC_lexicon_tags_14 },
    { 0x9cbfd0a6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x96bfd0a8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa01fd0aa, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xe55fd0ad, SUC_lexicon_tags_148 },
    { 0xf25fd0ae, SUC_lexicon_tags_25 },
    { 0x4ffd0ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf17fd0b3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x163fd0b5, SUC_lexicon_tags_72 },
    { 0x34dfd0b6, SUC_lexicon_tags_7 },
    { 0x229fd0b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84ffd0bd, SUC_lexicon_tags_7 },
    { 0x3d3fd0bd, SUC_lexicon_tags_37 },
    { 0xaf3fd0bf, SUC_lexicon_tags_33 },
    { 0x6ffd0bf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x42bfd0c2, SUC_lexicon_tags_43 },
    { 0xb1bfd0c2, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5fd0c6, SUC_lexicon_tags_79 },
    { 0x33ffd0c7, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x397fd0c9, SUC_lexicon_tags_11 },
    { 0x31fd0c9, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdeffd0d3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x10bfd0d5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4dffd0d7, SUC_lexicon_tags_39 },
    { 0x27dfd0d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x381fd0da, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1fd0de, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1e5fd0e0, SUC_lexicon_tags_17 },
    { 0xcf5fd0e1, SUC_lexicon_tags_11 },
    { 0x2edfd0e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80dfd0e9, SUC_lexicon_tags_2 },
    { 0x925fd0ea, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239fd0f3, SUC_lexicon_tags_2 },
    { 0x967fd0f4, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ffd0fa, SUC_lexicon_tags_7 },
    { 0x7dbfd0fb, SUC_lexicon_tags_19 },
    { 0x481fd0fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3fd101, SUC_lexicon_tags_1 },
    { 0x6ddfd102, SUC_lexicon_tags_37 },
    { 0xd9bfd103, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x299fd107, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x295fd10b, SUC_lexicon_tags_19 },
    { 0xee7fd10b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x5e3fd10e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0xb33fd111, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x4d9fd113, SUC_lexicon_tags_17 },
    { 0xbd9fd113, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d5fd118, SUC_lexicon_tags_49 },
    { 0x73bfd119, SUC_lexicon_tags_43 },
    { 0xc83fd11a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x467fd11f, SUC_lexicon_tags_28 },
    { 0x18ffd120, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55ffd125, SUC_lexicon_tags_33 },
    { 0xb77fd126, SUC_lexicon_tags_11 },
    { 0xfa3fd127, SUC_lexicon_tags_17 },
    { 0x13fd127, SUC_lexicon_tags_108 },
    { 0x1affd129, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x653fd12c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fd132, SUC_lexicon_tags_17 },
    { 0x5fbfd132, SUC_lexicon_tags_8 },
    { 0x5a1fd133, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27fd137, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x257fd13a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x649fd140, SUC_lexicon_tags_14 },
    { 0xf57fd141, SUC_lexicon_tags_19 },
    { 0x7e9fd142, SUC_lexicon_tags_21 },
    { 0x15bfd143, SUC_lexicon_tags_11 },
    { 0xa4dfd144, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9fd14a, SUC_lexicon_tags_27 },
    { 0x321fd14b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xb0dfd14d, SUC_lexicon_tags_17 },
    { 0x5cbfd14d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bfd154, SUC_lexicon_tags_2 },
    { 0x19bfd155, SUC_lexicon_tags_19 },
    { 0xd41fd156, SUC_lexicon_tags_11 },
    { 0x717fd156, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe7bfd159, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3fd15f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7fd162, SUC_lexicon_tags_30 },
    { 0xf5ffd163, SUC_lexicon_tags_49 },
    { 0x773fd164, SUC_lexicon_tags_45 },
    { 0xd99fd163, SUC_lexicon_tags_15 },
    { 0xe0bfd162, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xda9fd169, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x427fd171, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5fd174, SUC_lexicon_tags_7 },
    { 0x4c7fd175, SUC_lexicon_tags_37 },
    { 0x4cdfd176, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dfd17a, SUC_lexicon_tags_6 },
    { 0x931fd17b, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3fd17e, SUC_lexicon_tags_37 },
    { 0x6d7fd17f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315fd184, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x12ffd187, SUC_lexicon_tags_25 },
    { 0x1c5fd188, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xacbfd18a, SUC_lexicon_tags_54 },
    { 0x6bbfd18b, SUC_lexicon_tags_2 },
    { 0xe19fd18c, SUC_lexicon_tags_49 },
    { 0x60bfd18d, SUC_lexicon_tags_0 },
    { 0x19fd18d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0xda5fd190, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x27ffd193, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd85fd198, SUC_lexicon_tags_60 },
    { 0x861fd199, SUC_lexicon_tags_53 },
    { 0x5b9fd198, SUC_lexicon_tags_26 },
    { 0x641fd19b, SUC_lexicon_tags_14 },
    { 0x30dfd19b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdfd1a1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861fd1a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55fd1a9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1a1fd1ab, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0x56bfd1ad, SUC_lexicon_tags_23 },
    { 0x2b5fd1ae, SUC_lexicon_tags_2 },
    { 0x2e5fd1ae, SUC_lexicon_tags_60 },
    { 0x29dfd1ad, SUC_lexicon_tags_23 },
    { 0xb3ffd1b0, SUC_lexicon_tags_87 },
    { 0xcfbfd1af, SUC_lexicon_tags_23 },
    { 0x5ffd1b3, SUC_lexicon_tags_17 },
    { 0x63bfd1b4, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa4bfd1ba, SUC_lexicon_tags_17 },
    { 0x697fd1bb, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8b1fd1bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x549fd1bf, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863fd1c3, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa37fd1c7, SUC_lexicon_tags_1 },
    { 0xcedfd1c8, SUC_lexicon_tags_35 },
    { 0x2fdfd1c9, SUC_lexicon_tags_41 },
    { 0x53fd1c9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb5fd1d0, SUC_lexicon_tags_9 },
    { 0x93ffd1d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb33fd1d3, SUC_lexicon_tags_26 },
    { 0x57bfd1d4, SUC_lexicon_tags_46 },
    { 0x6e9fd1d3, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc91fd1d9, SUC_lexicon_tags_12 },
    { 0x603fd1da, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0x8d1fd1dc, SUC_lexicon_tags_38 },
    { 0xc33fd1dd, SUC_lexicon_tags_38 },
    { 0x3f3fd1dc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x87ffd1e0, SUC_lexicon_tags_1 },
    { 0x89bfd1e1, SUC_lexicon_tags_37 },
    { 0x19fd1e0, SUC_lexicon_tags_15 },
    { 0x5c7fd1e3, SUC_lexicon_tags_29 },
    { 0xb49fd1e0, SUC_lexicon_tags_0 },
    { 0x95fd1e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639fd1ee, SUC_lexicon_tags_12 },
    { 0x54bfd1ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23dfd1f8, SUC_lexicon_tags_92 },
    { 0x753fd1f9, SUC_lexicon_tags_23 },
    { 0x839fd1fa, SUC_lexicon_tags_44 },
    { 0x8ffd1f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x227fd1fd, SUC_lexicon_tags_19 },
    { 0xe15fd1fe, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb3fd203, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x299fd205, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfd20c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x23dfd20e, SUC_lexicon_tags_2 },
    { 0x743fd20f, SUC_lexicon_tags_37 },
    { 0x49ffd20f, SUC_lexicon_tags_29 },
    { 0xec1fd20e, SUC_lexicon_tags_38 },
    { 0xebfd212, SUC_lexicon_tags_2 },
    { 0x62ffd213, SUC_lexicon_tags_0 },
    { 0xba1fd214, SUC_lexicon_tags_29 },
    { 0xbcbfd212, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e7fd219, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b5fd221, SUC_lexicon_tags_47 },
    { 0x37bfd222, SUC_lexicon_tags_12 },
    { 0x1e7fd221, SUC_lexicon_tags_1 },
    { 0x777fd221, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec9fd228, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa09fd22e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x8effd231, SUC_lexicon_tags_378 },
    { 0x82dfd231, SUC_lexicon_tags_49 },
    { 0x25ffd233, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xc9dfd236, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd97fd23c, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xbfd23e, SUC_lexicon_tags_47 },
    { 0xdc1fd23e, SUC_lexicon_tags_38 },
    { 0xbabfd240, SUC_lexicon_tags_35 },
    { 0x345fd241, SUC_lexicon_tags_31 },
    { 0x99dfd242, SUC_lexicon_tags_8 },
    { 0xe7fd242, SUC_lexicon_tags_1 },
    { 0xdbfd243, SUC_lexicon_tags_23 },
    { 0xf23fd245, SUC_lexicon_tags_45 },
    { 0xa41fd245, SUC_lexicon_tags_23 },
    { 0xb7dfd247, SUC_lexicon_tags_43 },
    { 0x8adfd243, SUC_lexicon_tags_37 },
    { 0xcf1fd249, SUC_lexicon_tags_37 },
    { 0x27fd246, SUC_lexicon_tags_65 },
    { 0x4c1fd244, SUC_lexicon_tags_37 },
    { 0x17ffd243, SUC_lexicon_tags_23 },
    { 0xf85fd241, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x7bffd24f, SUC_lexicon_tags_2 },
    { 0x85ffd250, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509fd254, SUC_lexicon_tags_25 },
    { 0xebbfd255, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4fbfd257, SUC_lexicon_tags_11 },
    { 0x4dbfd257, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823fd25c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbfd262, SUC_lexicon_tags_25 },
    { 0x583fd263, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xba9fd265, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbe3fd267, SUC_lexicon_tags_1 },
    { 0x7d1fd267, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x551fd26b, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x77fd26d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5fd273, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x3fbfd276, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80ffd27c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xed3fd27e, SUC_lexicon_tags_6 },
    { 0x34ffd27e, SUC_lexicon_tags_25 },
    { 0xbb3fd280, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x1e3fd283, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x797fd287, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcddfd28b, SUC_lexicon_tags_54 },
    { 0xef9fd28b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x89dfd28e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285fd296, SUC_lexicon_tags_812 },
    { 0x445fd296, SUC_lexicon_tags_11 },
    { 0x125fd298, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x6ffd29a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdfd29f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcffd2a3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee5fd2a9, SUC_lexicon_tags_27 },
    { 0xaf7fd2a9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x485fd2ac, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeebfd2b0, SUC_lexicon_tags_15 },
    { 0xaa7fd2b1, SUC_lexicon_tags_65 },
    { 0x5d7fd2b2, SUC_lexicon_tags_25 },
    { 0xe9fd2b2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x789fd2b6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dffd2ba, SUC_lexicon_tags_43 },
    { 0x2c1fd2bb, SUC_lexicon_tags_63 },
    { 0xfe1fd2bc, SUC_lexicon_tags_7 },
    { 0x8d5fd2bd, SUC_lexicon_tags_29 },
    { 0xc3dfd2be, SUC_lexicon_tags_7 },
    { 0x3e7fd2bc, SUC_lexicon_tags_45 },
    { 0xebffd2bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x44bfd2c2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x903fd2c4, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dfd2ca, SUC_lexicon_tags_63 },
    { 0xd23fd2ca, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fd2d0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x243fd2d9, SUC_lexicon_tags_47 },
    { 0x28ffd2da, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xfeffd2dc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x2affd2df, SUC_lexicon_tags_15 },
    { 0xf53fd2e0, SUC_lexicon_tags_29 },
    { 0x8d1fd2e1, SUC_lexicon_tags_23 },
    { 0x62dfd2e2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827fd2e6, SUC_lexicon_tags_25 },
    { 0xebdfd2e7, SUC_lexicon_tags_87 },
    { 0x6dfd2e6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xb61fd2ea, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfd2ed, SUC_lexicon_tags_47 },
    { 0x571fd2ee, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb01fd2f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x557fd2f9, SUC_lexicon_tags_7 },
    { 0xbbdfd2f9, SUC_lexicon_tags_38 },
    { 0x6c7fd2f9, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xa17fd2fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x60ffd300, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x339fd303, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x335fd305, SUC_lexicon_tags_32 },
    { 0x177fd305, SUC_lexicon_tags_10 },
    { 0x3c7fd306, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1ffd30a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdfd312, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0x173fd314, SUC_lexicon_tags_26 },
    { 0x933fd314, SUC_lexicon_tags_390 },
    { 0x8dffd314, SUC_lexicon_tags_87 },
    { 0x7e9fd317, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabfd31d, SUC_lexicon_tags_37 },
    { 0x941fd31e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf9fd322, SUC_lexicon_tags_25 },
    { 0xa9fd322, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ffd32a, SUC_lexicon_tags_42 },
    { 0x5d1fd32b, SUC_lexicon_tags_1 },
    { 0xff1fd32b, SUC_lexicon_tags_25 },
    { 0x75dfd32c, SUC_lexicon_tags_6 },
    { 0x1a5fd32e, SUC_lexicon_tags_9 },
    { 0x93dfd32c, SUC_lexicon_tags_37 },
    { 0x889fd32c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bfd333, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bffd336, SUC_lexicon_tags_2 },
    { 0x2d5fd337, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fbfd33b, SUC_lexicon_tags_28 },
    { 0xe4bfd33b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x62dfd33f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x715fd345, SUC_lexicon_tags_1 },
    { 0x97fd345, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5bfd349, SUC_lexicon_tags_47 },
    { 0x20bfd34a, SUC_lexicon_tags_49 },
    { 0xeabfd349, SUC_lexicon_tags_17 },
    { 0x2f5fd34c, SUC_lexicon_tags_11 },
    { 0xfedfd34d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c1fd352, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1effd35b, SUC_lexicon_tags_79 },
    { 0x337fd35c, SUC_lexicon_tags_1 },
    { 0xd29fd35d, SUC_lexicon_tags_38 },
    { 0xbb7fd35c, SUC_lexicon_tags_63 },
    { 0x48dfd35f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dfd363, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3dfd367, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc6bfd369, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3bfd36b, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x5fffd36d, SUC_lexicon_tags_37 },
    { 0x9f9fd36e, SUC_lexicon_tags_11 },
    { 0xbb7fd36d, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0xb17fd371, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bfd377, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22bfd37e, SUC_lexicon_tags_43 },
    { 0xe6ffd37e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xeb9fd382, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9fd38a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x26bfd38d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x247fd396, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abfd39a, SUC_lexicon_tags_83 },
    { 0xdd9fd39b, SUC_lexicon_tags_25 },
    { 0xe79fd39c, SUC_lexicon_tags_28 },
    { 0x60ffd39a, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xa3ffd39f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13fd3a4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xeaffd3a7, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703fd3ad, SUC_lexicon_tags_34 },
    { 0x529fd3ae, SUC_lexicon_tags_27 },
    { 0x1fbfd3ae, SUC_lexicon_tags_46 },
    { 0xc33fd3b0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7b1fd3b2, SUC_lexicon_tags_57 },
    { 0xeaffd3b3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7ddfd3b5, SUC_lexicon_tags_63 },
    { 0xb11fd3b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xabfd3b9, SUC_lexicon_tags_60 },
    { 0xfdffd3ba, SUC_lexicon_tags_13 },
    { 0x509fd3bb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc4bfd3be, SUC_lexicon_tags_38 },
    { 0xce9fd3bf, SUC_lexicon_tags_15 },
    { 0x2d9fd3c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd9fd3c5, SUC_lexicon_tags_25 },
    { 0x31ffd3c6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x167fd3ca, SUC_lexicon_tags_11 },
    { 0x3b9fd3cb, SUC_lexicon_tags_15 },
    { 0x471fd3cc, SUC_lexicon_tags_9 },
    { 0x62dfd3ca, SUC_lexicon_tags_35 },
    { 0x877fd3cb, SUC_lexicon_tags_12 },
    { 0x657fd3cf, SUC_lexicon_tags_87 },
    { 0xb4ffd3cc, SUC_lexicon_tags_25 },
    { 0x111fd3d1, SUC_lexicon_tags_7 },
    { 0x857fd3d2, SUC_lexicon_tags_65 },
    { 0x65ffd3d3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bfd3d6, SUC_lexicon_tags_15 },
    { 0x62bfd3d7, SUC_lexicon_tags_65 },
    { 0x3b7fd3d7, SUC_lexicon_tags_6 },
    { 0xaedfd3d8, SUC_lexicon_tags_48 },
    { 0xcebfd3da, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x51dfd3dd, SUC_lexicon_tags_17 },
    { 0xb5ffd3de, SUC_lexicon_tags_80 },
    { 0x1abfd3dd, SUC_lexicon_tags_17 },
    { 0xf77fd3e0, SUC_lexicon_tags_7 },
    { 0x3c7fd3df, SUC_lexicon_tags_45 },
    { 0x1b1fd3e2, SUC_lexicon_tags_14 },
    { 0x7cffd3e1, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab3fd3e9, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39bfd3ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x339fd3ef, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x8adfd3f2, SUC_lexicon_tags_33 },
    { 0x407fd3f3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe09fd3f7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bbfd3fc, SUC_lexicon_tags_9 },
    { 0xc09fd3fd, SUC_lexicon_tags_37 },
    { 0x167fd3fd, SUC_lexicon_tags_33 },
    { 0x417fd3ff, SUC_lexicon_tags_14 },
    { 0x441fd3ff, SUC_lexicon_tags_42 },
    { 0x245fd3fe, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x6f5fd404, SUC_lexicon_tags_11 },
    { 0xe73fd404, SUC_lexicon_tags_30 },
    { 0x8bbfd404, SUC_lexicon_tags_37 },
    { 0xba7fd407, SUC_lexicon_tags_7 },
    { 0x223fd408, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x8bffd40a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa59fd40c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbbdfd40e, SUC_lexicon_tags_29 },
    { 0x769fd40f, SUC_lexicon_tags_15 },
    { 0xf05fd410, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6adfd412, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2a9fd414, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x34bfd417, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x1c7fd419, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8affd41e, SUC_lexicon_tags_21 },
    { 0x58ffd41f, SUC_lexicon_tags_23 },
    { 0xf3dfd420, SUC_lexicon_tags_9 },
    { 0x225fd421, SUC_lexicon_tags_11 },
    { 0x99ffd422, SUC_lexicon_tags_15 },
    { 0xdc1fd423, SUC_lexicon_tags_29 },
    { 0xe6ffd424, SUC_lexicon_tags_11 },
    { 0xd1ffd425, SUC_lexicon_tags_37 },
    { 0x31fd426, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9fd429, SUC_lexicon_tags_65 },
    { 0xc93fd42a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbedfd430, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bffd433, SUC_lexicon_tags_12 },
    { 0xf4bfd434, SUC_lexicon_tags_80 },
    { 0x4dffd433, SUC_lexicon_tags_11 },
    { 0x8cbfd436, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x821fd43b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cffd440, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x3d7fd442, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x29fd445, SUC_lexicon_tags_25 },
    { 0x40bfd446, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x74ffd448, SUC_lexicon_tags_25 },
    { 0x8a7fd448, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fd453, SUC_lexicon_tags_17 },
    { 0xc4ffd454, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63fd457, SUC_lexicon_tags_167 },
    { 0x4b3fd457, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x70ffd45a, SUC_lexicon_tags_29 },
    { 0x3f1fd45b, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0x789fd45d, SUC_lexicon_tags_27 },
    { 0xe69fd45e, SUC_lexicon_tags_1 },
    { 0xfcbfd45f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xa45fd461, SUC_lexicon_tags_38 },
    { 0x95fd461, SUC_lexicon_tags_7 },
    { 0xe8bfd462, SUC_lexicon_tags_7 },
    { 0x27ffd464, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2d1fd466, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5fd46b, SUC_lexicon_tags_15 },
    { 0xd8ffd46c, SUC_lexicon_tags_37 },
    { 0x493fd46d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x951fd46f, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bfd476, SUC_lexicon_tags_11 },
    { 0x897fd476, SUC_lexicon_tags_31 },
    { 0x9fd477, SUC_lexicon_tags_47 },
    { 0x403fd479, SUC_lexicon_tags_4 },
    { 0x6b3fd47a, SUC_lexicon_tags_11 },
    { 0xa8ffd47b, SUC_lexicon_tags_56 },
    { 0xca9fd47c, SUC_lexicon_tags_2 },
    { 0x15ffd478, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xc31fd480, SUC_lexicon_tags_15 },
    { 0xacffd481, SUC_lexicon_tags_15 },
    { 0x93ffd482, SUC_lexicon_tags_45 },
    { 0xe15fd483, SUC_lexicon_tags_15 },
    { 0xf3fd484, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xaebfd486, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc9fd48a, SUC_lexicon_tags_10 },
    { 0x1bbfd48b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x4dbfd48d, SUC_lexicon_tags_37 },
    { 0x93bfd48d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x557fd490, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xd17fd492, SUC_lexicon_tags_254 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bfd496, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd7fd49b, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x457fd49d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7fd4a1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x187fd4a3, SUC_lexicon_tags_29 },
    { 0xaf7fd4a3, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd9dfd4a6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x3bffd4a8, SUC_lexicon_tags_12 },
    { 0x15dfd4a9, SUC_lexicon_tags_37 },
    { 0x301fd4aa, SUC_lexicon_tags_0 },
    { 0x6c1fd4aa, SUC_lexicon_tags_38 },
    { 0x189fd4ac, SUC_lexicon_tags_17 },
    { 0x94bfd4ad, SUC_lexicon_tags_27 },
    { 0x52ffd4ae, SUC_lexicon_tags_33 },
    { 0xa15fd4af, SUC_lexicon_tags_17 },
    { 0x3ebfd4b0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbedfd4b2, SUC_lexicon_tags_31 },
    { 0x689fd4b3, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe29fd4b5, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc21fd4b8, SUC_lexicon_tags_23 },
    { 0x39fd4b9, SUC_lexicon_tags_19 },
    { 0xe3ffd4b9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa03fd4be, SUC_lexicon_tags_28 },
    { 0xcc7fd4bf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2fffd4c1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c9fd4c5, SUC_lexicon_tags_12 },
    { 0xf43fd4c5, SUC_lexicon_tags_15 },
    { 0xb7bfd4c7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x125fd4ca, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337fd4d5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bfd4d8, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdfd4dc, SUC_lexicon_tags_26 },
    { 0xc4bfd4dd, SUC_lexicon_tags_25 },
    { 0x2a7fd4de, SUC_lexicon_tags_23 },
    { 0x60bfd4dd, SUC_lexicon_tags_105 },
    { 0x9e1fd4dd, SUC_lexicon_tags_12 },
    { 0xf13fd4de, SUC_lexicon_tags_36 },
    { 0x8e3fd4dd, SUC_lexicon_tags_25 },
    { 0xde1fd4e0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0ffd4e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xe51fd4e8, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xb63fd4eb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x745fd4ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x347fd4f4, SUC_lexicon_tags_31 },
    { 0xb0bfd4f5, SUC_lexicon_tags_1 },
    { 0x601fd4f4, SUC_lexicon_tags_27 },
    { 0xda5fd4f7, SUC_lexicon_tags_21 },
    { 0x6c5fd4f8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd1fd4fa, SUC_lexicon_tags_27 },
    { 0x371fd4fb, SUC_lexicon_tags_14 },
    { 0x7a5fd4fa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc7ffd4fe, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd3fd501, SUC_lexicon_tags_12 },
    { 0x1b7fd502, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x39fd504, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd55fd509, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9f3fd50b, SUC_lexicon_tags_0 },
    { 0x4d3fd50c, SUC_lexicon_tags_11 },
    { 0xb03fd50d, SUC_lexicon_tags_55 },
    { 0xf95fd50b, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0xb47fd510, SUC_lexicon_tags_1 },
    { 0xb67fd511, SUC_lexicon_tags_1543 },
    { 0x6c5fd512, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd8dfd514, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x929fd51b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7fd526, SUC_lexicon_tags_1 },
    { 0x893fd527, SUC_lexicon_tags_2 },
    { 0x17bfd528, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdffd52b, SUC_lexicon_tags_27 },
    { 0x70ffd52c, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x981fd535, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x177fd537, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xcc3fd53a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe47fd53e, SUC_lexicon_tags_1 },
    { 0x2a7fd53e, SUC_lexicon_tags_49 },
    { 0x21fd53e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xe79fd542, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xf81fd545, SUC_lexicon_tags_7 },
    { 0x663fd546, SUC_lexicon_tags_10 },
    { 0xa5bfd547, SUC_lexicon_tags_17 },
    { 0xd1fd545, SUC_lexicon_tags_27 },
    { 0x959fd545, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0x515fd54b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1c1fd54d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cdfd551, SUC_lexicon_tags_0 },
    { 0xcdbfd551, SUC_lexicon_tags_37 },
    { 0x7f1fd551, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4ffd557, SUC_lexicon_tags_37 },
    { 0xe3bfd558, SUC_lexicon_tags_19 },
    { 0x263fd559, SUC_lexicon_tags_1 },
    { 0x6e3fd557, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc6dfd55d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2bfd562, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf85fd569, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bfd573, SUC_lexicon_tags_1 },
    { 0x9c9fd574, SUC_lexicon_tags_12 },
    { 0x57bfd575, SUC_lexicon_tags_129 },
    { 0x3ddfd576, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bfd579, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x707fd57e, SUC_lexicon_tags_15 },
    { 0xbfdfd57e, SUC_lexicon_tags_6 },
    { 0x6effd57f, SUC_lexicon_tags_37 },
    { 0x3c9fd580, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x3abfd583, SUC_lexicon_tags_16 },
    { 0x4c9fd584, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x6fbfd586, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bbfd592, SUC_lexicon_tags_47 },
    { 0x3adfd593, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x137fd595, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbfd59a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa4dfd59c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x457fd59f, SUC_lexicon_tags_55 },
    { 0x8adfd5a0, SUC_lexicon_tags_15 },
    { 0x207fd5a0, SUC_lexicon_tags_14 },
    { 0xd25fd5a2, SUC_lexicon_tags_25 },
    { 0xe8dfd5a3, SUC_lexicon_tags_7 },
    { 0xcedfd5a4, SUC_lexicon_tags_17 },
    { 0xc87fd5a2, SUC_lexicon_tags_37 },
    { 0x1adfd5a5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71fd5a9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc1dfd5ab, SUC_lexicon_tags_0 },
    { 0xe41fd5ab, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x28ffd5af, SUC_lexicon_tags_60 },
    { 0x217fd5b0, SUC_lexicon_tags_11 },
    { 0x9e1fd5b1, SUC_lexicon_tags_14 },
    { 0x507fd5b1, SUC_lexicon_tags_37 },
    { 0xa7fd5b1, SUC_lexicon_tags_49 },
    { 0x737fd5af, SUC_lexicon_tags_47 },
    { 0x5f9fd5b5, SUC_lexicon_tags_6 },
    { 0xbd3fd5b5, SUC_lexicon_tags_12 },
    { 0x615fd5b5, SUC_lexicon_tags_17 },
    { 0x411fd5b5, SUC_lexicon_tags_25 },
    { 0xdfdfd5b8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x84bfd5bc, SUC_lexicon_tags_7 },
    { 0x847fd5bd, SUC_lexicon_tags_69 },
    { 0xae9fd5bd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3fd5c2, SUC_lexicon_tags_12 },
    { 0x497fd5c3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7fd5c8, SUC_lexicon_tags_25 },
    { 0xe1ffd5c8, SUC_lexicon_tags_37 },
    { 0x9b9fd5ca, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x6e5fd5cc, SUC_lexicon_tags_47 },
    { 0x465fd5cc, SUC_lexicon_tags_25 },
    { 0xdd3fd5ce, SUC_lexicon_tags_16 },
    { 0x291fd5cf, SUC_lexicon_tags_2 },
    { 0xd8dfd5cf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x897fd5d3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x249fd5d7, SUC_lexicon_tags_2 },
    { 0xe61fd5d8, SUC_lexicon_tags_16 },
    { 0x813fd5d8, SUC_lexicon_tags_27 },
    { 0x159fd5da, SUC_lexicon_tags_238 },
    { 0x217fd5d7, SUC_lexicon_tags_11 },
    { 0xa25fd5db, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x5c9fd5de, SUC_lexicon_tags_15 },
    { 0x211fd5df, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95fd5e3, SUC_lexicon_tags_47 },
    { 0xed5fd5e4, SUC_lexicon_tags_38 },
    { 0x267fd5e3, SUC_lexicon_tags_27 },
    { 0x9bffd5e6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc75fd5e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc3dfd5eb, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8fffd5f1, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9c5fd5f3, SUC_lexicon_tags_30 },
    { 0xb71fd5f3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd73fd5f6, SUC_lexicon_tags_37 },
    { 0x31fd5f7, SUC_lexicon_tags_43 },
    { 0x959fd5f6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fd5fc, SUC_lexicon_tags_14 },
    { 0xe5ffd5fd, SUC_lexicon_tags_11 },
    { 0x487fd5fe, SUC_lexicon_tags_67 },
    { 0, NULL },
    { 0xcadfd600, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc03fd603, SUC_lexicon_tags_16 },
    { 0x3abfd604, SUC_lexicon_tags_9 },
    { 0x7a7fd605, SUC_lexicon_tags_19 },
    { 0xd0dfd606, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395fd60a, SUC_lexicon_tags_38 },
    { 0x5adfd60b, SUC_lexicon_tags_6 },
    { 0x1effd60c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xa81fd60e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xbcdfd610, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x62bfd617, SUC_lexicon_tags_7 },
    { 0x13ffd617, SUC_lexicon_tags_7 },
    { 0x1dffd617, SUC_lexicon_tags_12 },
    { 0x7f3fd618, SUC_lexicon_tags_7 },
    { 0x157fd61a, SUC_lexicon_tags_44 },
    { 0xebdfd617, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x3fd61e, SUC_lexicon_tags_32 },
    { 0x4c5fd61f, SUC_lexicon_tags_12 },
    { 0xc3ffd61e, SUC_lexicon_tags_80 },
    { 0xf4ffd621, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33dfd629, SUC_lexicon_tags_17 },
    { 0x223fd629, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe03fd62c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0x709fd62f, SUC_lexicon_tags_15 },
    { 0x4d1fd630, SUC_lexicon_tags_7 },
    { 0x441fd631, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xec1fd633, SUC_lexicon_tags_11 },
    { 0x2abfd633, SUC_lexicon_tags_11 },
    { 0xc6ffd634, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x62bfd638, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xf7fd63a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x71ffd63d, SUC_lexicon_tags_17 },
    { 0x2a5fd63e, SUC_lexicon_tags_43 },
    { 0xc7fd63f, SUC_lexicon_tags_29 },
    { 0x273fd63d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xfcffd642, SUC_lexicon_tags_13 },
    { 0x665fd643, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ffd646, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebfd649, SUC_lexicon_tags_13 },
    { 0x507fd64a, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x75dfd64d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x435fd652, SUC_lexicon_tags_37 },
    { 0x785fd652, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea7fd659, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x583fd65b, SUC_lexicon_tags_28 },
    { 0xeadfd65c, SUC_lexicon_tags_25 },
    { 0x3b1fd65b, SUC_lexicon_tags_40 },
    { 0xe2dfd65d, SUC_lexicon_tags_38 },
    { 0x649fd65d, SUC_lexicon_tags_2 },
    { 0x7a5fd660, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xecffd662, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7ebfd664, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bbfd66b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb33fd66d, SUC_lexicon_tags_17 },
    { 0x487fd66e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95fd673, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3ffd676, SUC_lexicon_tags_9 },
    { 0x3a7fd677, SUC_lexicon_tags_45 },
    { 0x997fd677, SUC_lexicon_tags_32 },
    { 0x821fd679, SUC_lexicon_tags_2 },
    { 0x549fd677, SUC_lexicon_tags_37 },
    { 0x81ffd679, SUC_lexicon_tags_61 },
    { 0xd3fd67c, SUC_lexicon_tags_7 },
    { 0xab3fd67d, SUC_lexicon_tags_16 },
    { 0x129fd67c, SUC_lexicon_tags_7 },
    { 0xc63fd67f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x7b7fd681, SUC_lexicon_tags_4 },
    { 0xd81fd682, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc13fd685, SUC_lexicon_tags_7 },
    { 0xf69fd686, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x353fd68b, SUC_lexicon_tags_17 },
    { 0xc37fd68b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xdc9fd68e, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xd5ffd690, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x735fd693, SUC_lexicon_tags_33 },
    { 0xd1dfd693, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4affd699, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc77fd6a3, SUC_lexicon_tags_15 },
    { 0x599fd6a4, SUC_lexicon_tags_17 },
    { 0x7e7fd6a4, SUC_lexicon_tags_12 },
    { 0x48dfd6a6, SUC_lexicon_tags_4 },
    { 0x58ffd6a6, SUC_lexicon_tags_1 },
    { 0xd01fd6a6, SUC_lexicon_tags_17 },
    { 0xe29fd6a6, SUC_lexicon_tags_8 },
    { 0x86bfd6a8, SUC_lexicon_tags_32 },
    { 0x4c9fd6aa, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x723fd6b4, SUC_lexicon_tags_17 },
    { 0x28dfd6b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33fd6ba, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x829fd6bc, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0bfd6c0, SUC_lexicon_tags_12 },
    { 0xca1fd6c0, SUC_lexicon_tags_25 },
    { 0x9c7fd6c2, SUC_lexicon_tags_0 },
    { 0xf0bfd6c0, SUC_lexicon_tags_69 },
    { 0x43dfd6c4, SUC_lexicon_tags_8 },
    { 0x7d1fd6c5, SUC_lexicon_tags_3 },
    { 0xb99fd6c3, SUC_lexicon_tags_17 },
    { 0xdddfd6c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x2e1fd6c9, SUC_lexicon_tags_17 },
    { 0x3e3fd6ca, SUC_lexicon_tags_14 },
    { 0x733fd6cb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27bfd6d4, SUC_lexicon_tags_47 },
    { 0xb85fd6d5, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x107fd6d7, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81dfd6dc, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ddfd6e0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xed1fd6e3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ffd6e8, SUC_lexicon_tags_11 },
    { 0x33ffd6e8, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x837fd6ef, SUC_lexicon_tags_170 },
    { 0x4fdfd6f0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefdfd6f6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5dfd6f8, SUC_lexicon_tags_64 },
    { 0xdf7fd6f9, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadfd6fe, SUC_lexicon_tags_13 },
    { 0xf05fd6ff, SUC_lexicon_tags_49 },
    { 0xe2dfd6fe, SUC_lexicon_tags_16 },
    { 0x15bfd701, SUC_lexicon_tags_7 },
    { 0xe05fd702, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xaa7fd704, SUC_lexicon_tags_11 },
    { 0x5e5fd704, SUC_lexicon_tags_11 },
    { 0x737fd706, SUC_lexicon_tags_11 },
    { 0x1e5fd706, SUC_lexicon_tags_1 },
    { 0x7f9fd708, SUC_lexicon_tags_12 },
    { 0x527fd709, SUC_lexicon_tags_27 },
    { 0x28ffd709, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xd4ffd70d, SUC_lexicon_tags_13 },
    { 0x51dfd70d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x683fd710, SUC_lexicon_tags_25 },
    { 0x1e3fd710, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d7fd716, SUC_lexicon_tags_10 },
    { 0x7dfd717, SUC_lexicon_tags_56 },
    { 0xe15fd718, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa73fd721, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429fd72e, SUC_lexicon_tags_1 },
    { 0x4bffd72e, SUC_lexicon_tags_1 },
    { 0x5fdfd72f, SUC_lexicon_tags_37 },
    { 0x325fd731, SUC_lexicon_tags_15 },
    { 0xacffd732, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x23bfd734, SUC_lexicon_tags_12 },
    { 0x595fd734, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5fd73c, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xe27fd73e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbadfd742, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429fd746, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x227fd748, SUC_lexicon_tags_15 },
    { 0x27ffd749, SUC_lexicon_tags_17 },
    { 0xe23fd74a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x85ffd74f, SUC_lexicon_tags_31 },
    { 0xbadfd74f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x89dfd753, SUC_lexicon_tags_36 },
    { 0x103fd754, SUC_lexicon_tags_13 },
    { 0x115fd755, SUC_lexicon_tags_2 },
    { 0xf6ffd754, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc33fd758, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dfd75c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x111fd763, SUC_lexicon_tags_17 },
    { 0xc1ffd764, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fd768, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xe83fd76b, SUC_lexicon_tags_17 },
    { 0x567fd76b, SUC_lexicon_tags_1 },
    { 0x339fd76d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7fd774, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xa17fd776, SUC_lexicon_tags_15 },
    { 0x5f9fd777, SUC_lexicon_tags_1 },
    { 0xd77fd777, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93fd77d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xfa7fd77f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc87fd783, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bffd786, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x639fd78b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c9fd78e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a9fd792, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5fd798, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcbdfd79a, SUC_lexicon_tags_37 },
    { 0x971fd79b, SUC_lexicon_tags_19 },
    { 0x5d5fd79b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x41dfd79f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf8dfd7a1, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xbd1fd7a3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x291fd7a5, SUC_lexicon_tags_29 },
    { 0xbbfd7a6, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbadfd7a8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2abfd7b0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x545fd7b3, SUC_lexicon_tags_4 },
    { 0x2dbfd7b3, SUC_lexicon_tags_37 },
    { 0x25ffd7b3, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x4a9fd7b7, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6dbfd7bd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfbbfd7bf, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dffd7c5, SUC_lexicon_tags_7 },
    { 0x17fd7c6, SUC_lexicon_tags_37 },
    { 0xc5dfd7c5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xeb9fd7c9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e9fd7cc, SUC_lexicon_tags_19 },
    { 0xc17fd7cd, SUC_lexicon_tags_11 },
    { 0x59ffd7ce, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x507fd7d0, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7fd7d3, SUC_lexicon_tags_25 },
    { 0x943fd7d3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa21fd7d6, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5fd7da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xff1fd7dd, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x93bfd7e0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xce9fd7e2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe1ffd7e4, SUC_lexicon_tags_15 },
    { 0x89fd7e5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bfd7ee, SUC_lexicon_tags_25 },
    { 0xe8ffd7ef, SUC_lexicon_tags_15 },
    { 0x2bdfd7ef, SUC_lexicon_tags_25 },
    { 0xdebfd7f1, SUC_lexicon_tags_2 },
    { 0xf5dfd7f2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4bfd7f6, SUC_lexicon_tags_28 },
    { 0xb15fd7f6, SUC_lexicon_tags_1 },
    { 0xb9ffd7f8, SUC_lexicon_tags_56 },
    { 0x4dbfd7f9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x89dfd7fc, SUC_lexicon_tags_26 },
    { 0xf6bfd7fd, SUC_lexicon_tags_2 },
    { 0x1dffd7fd, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbbfd801, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d1fd805, SUC_lexicon_tags_5 },
    { 0x681fd806, SUC_lexicon_tags_29 },
    { 0x7fbfd807, SUC_lexicon_tags_35 },
    { 0xb1ffd808, SUC_lexicon_tags_31 },
    { 0x755fd809, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x631fd80c, SUC_lexicon_tags_17 },
    { 0xb23fd80d, SUC_lexicon_tags_149 },
    { 0xfbbfd80e, SUC_lexicon_tags_47 },
    { 0x3a5fd80f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539fd814, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5fd818, SUC_lexicon_tags_2 },
    { 0x48dfd819, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x4adfd81b, SUC_lexicon_tags_92 },
    { 0x423fd81c, SUC_lexicon_tags_60 },
    { 0x9e1fd81d, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x7affd820, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x981fd823, SUC_lexicon_tags_18 },
    { 0x1e1fd824, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bfd828, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xdcbfd82a, SUC_lexicon_tags_20 },
    { 0xe2dfd82b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc51fd82f, SUC_lexicon_tags_12 },
    { 0xe03fd830, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93ffd834, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe01fd836, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dfd83a, SUC_lexicon_tags_138 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x41bfd841, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321fd84b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62ffd84d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x17ffd850, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x57dfd855, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcdbfd857, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x55fd85a, SUC_lexicon_tags_48 },
    { 0xb6dfd85b, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5fd85f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x871fd864, SUC_lexicon_tags_2 },
    { 0x2f9fd864, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xa5bfd867, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109fd86c, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203fd871, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4cbfd875, SUC_lexicon_tags_11 },
    { 0x949fd876, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dfd87a, SUC_lexicon_tags_26 },
    { 0xc01fd87b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xc79fd87d, SUC_lexicon_tags_25 },
    { 0x2c3fd87e, SUC_lexicon_tags_15 },
    { 0x8dbfd87f, SUC_lexicon_tags_11 },
    { 0x253fd87f, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ffd883, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x9d1fd885, SUC_lexicon_tags_19 },
    { 0x42dfd886, SUC_lexicon_tags_36 },
    { 0x963fd887, SUC_lexicon_tags_7 },
    { 0x1a9fd886, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0xe03fd88a, SUC_lexicon_tags_60 },
    { 0x1a9fd88a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xb17fd88d, SUC_lexicon_tags_70 },
    { 0x92bfd88e, SUC_lexicon_tags_87 },
    { 0x31ffd88d, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7a5fd894, SUC_lexicon_tags_65 },
    { 0x883fd894, SUC_lexicon_tags_61 },
    { 0x233fd896, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xa6dfd899, SUC_lexicon_tags_46 },
    { 0x9c3fd89a, SUC_lexicon_tags_68 },
    { 0xf13fd89b, SUC_lexicon_tags_7 },
    { 0xb6bfd899, SUC_lexicon_tags_11 },
    { 0xce7fd89d, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x357fd89f, SUC_lexicon_tags_12 },
    { 0xbadfd89f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x1f5fd8a2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xc1dfd8a4, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe17fd8a8, SUC_lexicon_tags_30 },
    { 0xb4bfd8a9, SUC_lexicon_tags_11 },
    { 0x72dfd8a9, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd1fd8ad, SUC_lexicon_tags_12 },
    { 0xc23fd8ae, SUC_lexicon_tags_7 },
    { 0xcdbfd8af, SUC_lexicon_tags_37 },
    { 0x125fd8b0, SUC_lexicon_tags_11 },
    { 0xf65fd8b1, SUC_lexicon_tags_8 },
    { 0x4ddfd8b2, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad7fd8b6, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x827fd8b8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1effd8bd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xbb5fd8bf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x253fd8c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1fd8d1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x217fd8d3, SUC_lexicon_tags_70 },
    { 0x2c5fd8d4, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677fd8da, SUC_lexicon_tags_11 },
    { 0x629fd8db, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xf01fd8dd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1fd8e0, SUC_lexicon_tags_1 },
    { 0x239fd8e1, SUC_lexicon_tags_10 },
    { 0x639fd8e2, SUC_lexicon_tags_19 },
    { 0xbd3fd8e3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fffd8e6, SUC_lexicon_tags_51 },
    { 0x38bfd8e7, SUC_lexicon_tags_31 },
    { 0x135fd8e7, SUC_lexicon_tags_37 },
    { 0x6d9fd8e9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5fd8ed, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xcbfd8ef, SUC_lexicon_tags_4 },
    { 0x867fd8ef, SUC_lexicon_tags_17 },
    { 0x35bfd8ef, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ffd8f4, SUC_lexicon_tags_27 },
    { 0xe71fd8f4, SUC_lexicon_tags_15 },
    { 0xd39fd8f4, SUC_lexicon_tags_20 },
    { 0, NULL },
    { 0xf87fd8f8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b7fd8fb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x305fd900, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30ffd905, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5fd90b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc75fd90d, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x131fd90f, SUC_lexicon_tags_45 },
    { 0xb01fd910, SUC_lexicon_tags_43 },
    { 0x13fd911, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x669fd920, SUC_lexicon_tags_33 },
    { 0xf3fd921, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdedfd925, SUC_lexicon_tags_15 },
    { 0x56dfd926, SUC_lexicon_tags_2 },
    { 0x6f3fd925, SUC_lexicon_tags_20 },
    { 0x945fd928, SUC_lexicon_tags_23 },
    { 0xef1fd928, SUC_lexicon_tags_37 },
    { 0xc49fd925, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x415fd92c, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0xc77fd92e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x3abfd930, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x11dfd933, SUC_lexicon_tags_37 },
    { 0xf1dfd934, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dfd937, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xadffd93a, SUC_lexicon_tags_23 },
    { 0x795fd93b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe05fd942, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x741fd945, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5fd948, SUC_lexicon_tags_70 },
    { 0x2fbfd949, SUC_lexicon_tags_35 },
    { 0xce9fd949, SUC_lexicon_tags_17 },
    { 0x293fd948, SUC_lexicon_tags_23 },
    { 0x23dfd949, SUC_lexicon_tags_25 },
    { 0x36bfd94d, SUC_lexicon_tags_26 },
    { 0x70bfd94e, SUC_lexicon_tags_56 },
    { 0xe7fd94d, SUC_lexicon_tags_66 },
    { 0xfb5fd949, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x903fd953, SUC_lexicon_tags_63 },
    { 0x5dfd953, SUC_lexicon_tags_12 },
    { 0x20dfd955, SUC_lexicon_tags_25 },
    { 0xa9dfd956, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0bfd959, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x22ffd960, SUC_lexicon_tags_84 },
    { 0x2a7fd961, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x5d1fd963, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a7fd969, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b9fd96f, SUC_lexicon_tags_12 },
    { 0x229fd970, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5effd972, SUC_lexicon_tags_27 },
    { 0xa4bfd973, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1fd977, SUC_lexicon_tags_37 },
    { 0xcd3fd978, SUC_lexicon_tags_11 },
    { 0x99fd979, SUC_lexicon_tags_19 },
    { 0x25ffd979, SUC_lexicon_tags_11 },
    { 0xd1fd97b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa49fd981, SUC_lexicon_tags_2 },
    { 0xd5bfd982, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0x96ffd984, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe87fd986, SUC_lexicon_tags_170 },
    { 0x891fd987, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bfd98b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6abfd990, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xd93fd992, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x523fd994, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f5fd998, SUC_lexicon_tags_29 },
    { 0xdeffd998, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7fd99c, SUC_lexicon_tags_46 },
    { 0x80ffd99d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x293fd99f, SUC_lexicon_tags_1 },
    { 0xca3fd9a0, SUC_lexicon_tags_11 },
    { 0x789fd99f, SUC_lexicon_tags_13 },
    { 0x4affd99f, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cbfd9a7, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x1b5fd9a9, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83fd9af, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5fd9b2, SUC_lexicon_tags_38 },
    { 0x615fd9b3, SUC_lexicon_tags_17 },
    { 0x48ffd9b4, SUC_lexicon_tags_12 },
    { 0xb3ffd9b3, SUC_lexicon_tags_25 },
    { 0x45dfd9b3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5d3fd9b9, SUC_lexicon_tags_129 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dfd9bd, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x56dfd9bf, SUC_lexicon_tags_25 },
    { 0xbcffd9bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7bffd9c2, SUC_lexicon_tags_123 },
    { 0x129fd9c2, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x4edfd9c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957fd9c9, SUC_lexicon_tags_2 },
    { 0x849fd9c9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1adfd9cf, SUC_lexicon_tags_46 },
    { 0xb71fd9cf, SUC_lexicon_tags_1 },
    { 0xab1fd9d0, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95dfd9d5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x85fd9d7, SUC_lexicon_tags_25 },
    { 0xe6bfd9d8, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x75fd9da, SUC_lexicon_tags_17 },
    { 0xe27fd9da, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2dfd9e0, SUC_lexicon_tags_39 },
    { 0xafbfd9e0, SUC_lexicon_tags_37 },
    { 0x945fd9e0, SUC_lexicon_tags_27 },
    { 0xf13fd9e1, SUC_lexicon_tags_2 },
    { 0x8ebfd9e0, SUC_lexicon_tags_146 },
    { 0xca1fd9e5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f7fd9e9, SUC_lexicon_tags_11 },
    { 0x291fd9ea, SUC_lexicon_tags_65 },
    { 0x47ffd9eb, SUC_lexicon_tags_28 },
    { 0x3effd9ec, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19dfd9f4, SUC_lexicon_tags_17 },
    { 0xab9fd9f5, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9cffd9fc, SUC_lexicon_tags_17 },
    { 0x2fbfd9fd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e1fda00, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xc9fda02, SUC_lexicon_tags_17 },
    { 0xf45fda03, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7dfda06, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989fda0a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0xcd9fda0c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x67ffda11, SUC_lexicon_tags_25 },
    { 0xbe7fda12, SUC_lexicon_tags_2 },
    { 0x3b5fda12, SUC_lexicon_tags_15 },
    { 0x2ebfda14, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x61fda17, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8dfda1b, SUC_lexicon_tags_6 },
    { 0x267fda1b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb71fda20, SUC_lexicon_tags_37 },
    { 0x2bdfda21, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x733fda23, SUC_lexicon_tags_11 },
    { 0x785fda24, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc1fda2a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x7e9fda2c, SUC_lexicon_tags_32 },
    { 0xb35fda2d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebfda30, SUC_lexicon_tags_27 },
    { 0x1fda31, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861fda39, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fda40, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1dfda47, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x255fda4e, SUC_lexicon_tags_7 },
    { 0xdc1fda4f, SUC_lexicon_tags_19 },
    { 0xb19fda50, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x41bfda53, SUC_lexicon_tags_17 },
    { 0x1c5fda54, SUC_lexicon_tags_49 },
    { 0x713fda55, SUC_lexicon_tags_14 },
    { 0xa93fda56, SUC_lexicon_tags_351 },
    { 0, NULL },
    { 0, NULL },
    { 0x629fda59, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xee1fda5b, SUC_lexicon_tags_47 },
    { 0x643fda5b, SUC_lexicon_tags_7 },
    { 0x831fda5c, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x51fda60, SUC_lexicon_tags_1 },
    { 0x50bfda61, SUC_lexicon_tags_19 },
    { 0x291fda62, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x191fda64, SUC_lexicon_tags_39 },
    { 0xfa5fda64, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1fda68, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x45dfda6a, SUC_lexicon_tags_376 },
    { 0xf7bfda6b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x907fda70, SUC_lexicon_tags_43 },
    { 0xa65fda71, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x61dfda74, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4effda78, SUC_lexicon_tags_12 },
    { 0x357fda78, SUC_lexicon_tags_33 },
    { 0x42ffda7a, SUC_lexicon_tags_12 },
    { 0xf7bfda7b, SUC_lexicon_tags_37 },
    { 0x9b7fda7a, SUC_lexicon_tags_15 },
    { 0xc83fda7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x405fda7f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xc79fda81, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdd9fda83, SUC_lexicon_tags_37 },
    { 0xf41fda83, SUC_lexicon_tags_42 },
    { 0x97bfda85, SUC_lexicon_tags_19 },
    { 0x719fda86, SUC_lexicon_tags_15 },
    { 0xfb7fda84, SUC_lexicon_tags_14 },
    { 0x85ffda88, SUC_lexicon_tags_11 },
    { 0x7fbfda89, SUC_lexicon_tags_19 },
    { 0x451fda89, SUC_lexicon_tags_14 },
    { 0x7bbfda8b, SUC_lexicon_tags_14 },
    { 0xe11fda8b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb65fda8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd43fda95, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xacbfda9d, SUC_lexicon_tags_87 },
    { 0x289fda9e, SUC_lexicon_tags_7 },
    { 0x33bfda9f, SUC_lexicon_tags_894 },
    { 0x8a7fda9f, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x421fdaa2, SUC_lexicon_tags_23 },
    { 0x2bffdaa2, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0x6e5fdaa5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77dfdaaa, SUC_lexicon_tags_38 },
    { 0xa5dfdaaa, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xa89fdaad, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x2e5fdaaf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5fdab5, SUC_lexicon_tags_25 },
    { 0xb49fdab6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1a3fdab9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bfdabb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc7fdabd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xd59fdabf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9fdac4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42ffdac8, SUC_lexicon_tags_48 },
    { 0x3adfdac8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ffdacd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bfdad2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb85fdad7, SUC_lexicon_tags_151 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf77fdadd, SUC_lexicon_tags_25 },
    { 0x523fdadd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x57bfdae0, SUC_lexicon_tags_79 },
    { 0x48ffdae0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xb15fdae4, SUC_lexicon_tags_29 },
    { 0xd9fdae5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bfdaec, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x4dbfdaef, SUC_lexicon_tags_62 },
    { 0xcfdfdaef, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x457fdaf5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xebffdaf7, SUC_lexicon_tags_62 },
    { 0x73dfdaf8, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x11bfdafc, SUC_lexicon_tags_60 },
    { 0xa55fdafd, SUC_lexicon_tags_26 },
    { 0xd25fdafe, SUC_lexicon_tags_100 },
    { 0xee1fdafe, SUC_lexicon_tags_33 },
    { 0xa67fdafc, SUC_lexicon_tags_32 },
    { 0x205fdb01, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0xdcbfdb03, SUC_lexicon_tags_15 },
    { 0xad7fdb03, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x8dfdb06, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x209fdb08, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x119fdb0e, SUC_lexicon_tags_45 },
    { 0x951fdb0e, SUC_lexicon_tags_1 },
    { 0x269fdb10, SUC_lexicon_tags_11 },
    { 0xff1fdb10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb91fdb15, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe5bfdb1b, SUC_lexicon_tags_15 },
    { 0x319fdb1c, SUC_lexicon_tags_14 },
    { 0x23bfdb1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x483fdb21, SUC_lexicon_tags_25 },
    { 0xf5bfdb22, SUC_lexicon_tags_25 },
    { 0x467fdb23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x9bfdb25, SUC_lexicon_tags_63 },
    { 0xf8bfdb26, SUC_lexicon_tags_17 },
    { 0xd0bfdb26, SUC_lexicon_tags_14 },
    { 0xca7fdb26, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x259fdb2f, SUC_lexicon_tags_38 },
    { 0xf7bfdb30, SUC_lexicon_tags_198 },
    { 0x557fdb31, SUC_lexicon_tags_43 },
    { 0x99bfdb32, SUC_lexicon_tags_25 },
    { 0x715fdb32, SUC_lexicon_tags_37 },
    { 0x125fdb34, SUC_lexicon_tags_37 },
    { 0x67ffdb34, SUC_lexicon_tags_30 },
    { 0xa43fdb36, SUC_lexicon_tags_17 },
    { 0x4a3fdb30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9fdb3a, SUC_lexicon_tags_37 },
    { 0xb89fdb3b, SUC_lexicon_tags_25 },
    { 0xe3bfdb3c, SUC_lexicon_tags_2 },
    { 0x409fdb3d, SUC_lexicon_tags_37 },
    { 0x95dfdb3d, SUC_lexicon_tags_17 },
    { 0x643fdb3f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x165fdb41, SUC_lexicon_tags_2 },
    { 0xadbfdb42, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa5fdb47, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x2e3fdb4a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f7fdb50, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x78dfdb52, SUC_lexicon_tags_77 },
    { 0xff3fdb53, SUC_lexicon_tags_7 },
    { 0x2e5fdb54, SUC_lexicon_tags_27 },
    { 0xc97fdb54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dfdb5e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd19fdb60, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x243fdb63, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ffdb68, SUC_lexicon_tags_2 },
    { 0x15fdb69, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb2bfdb6b, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbdbfdb71, SUC_lexicon_tags_10 },
    { 0x897fdb72, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd75fdb74, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e3fdb7a, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x891fdb7c, SUC_lexicon_tags_26 },
    { 0x879fdb7d, SUC_lexicon_tags_19 },
    { 0x8bdfdb7d, SUC_lexicon_tags_43 },
    { 0x30ffdb7e, SUC_lexicon_tags_7 },
    { 0x145fdb80, SUC_lexicon_tags_37 },
    { 0x867fdb7e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x211fdb83, SUC_lexicon_tags_14 },
    { 0x68bfdb83, SUC_lexicon_tags_8 },
    { 0xadbfdb85, SUC_lexicon_tags_12 },
    { 0x73bfdb86, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeb7fdb8a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99dfdb8f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3dfdb97, SUC_lexicon_tags_52 },
    { 0xa31fdb97, SUC_lexicon_tags_25 },
    { 0x62bfdb99, SUC_lexicon_tags_15 },
    { 0x359fdb99, SUC_lexicon_tags_37 },
    { 0xa41fdb9b, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0x6b3fdb9d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11fdba2, SUC_lexicon_tags_63 },
    { 0x44ffdba3, SUC_lexicon_tags_25 },
    { 0x5d5fdba4, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfdba7, SUC_lexicon_tags_12 },
    { 0xfa1fdba8, SUC_lexicon_tags_29 },
    { 0xc3bfdba8, SUC_lexicon_tags_198 },
    { 0x607fdbaa, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37bfdbb1, SUC_lexicon_tags_9 },
    { 0xf2dfdbb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ffdbb5, SUC_lexicon_tags_17 },
    { 0xb8bfdbb6, SUC_lexicon_tags_17 },
    { 0x8dfdbb7, SUC_lexicon_tags_17 },
    { 0x843fdbb7, SUC_lexicon_tags_0 },
    { 0xc1ffdbb6, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xd99fdbbc, SUC_lexicon_tags_33 },
    { 0x193fdbbc, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x50bfdbbf, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5affdbc4, SUC_lexicon_tags_25 },
    { 0x88ffdbc5, SUC_lexicon_tags_8 },
    { 0xc6dfdbc5, SUC_lexicon_tags_11 },
    { 0xea7fdbc7, SUC_lexicon_tags_15 },
    { 0x5fdfdbc7, SUC_lexicon_tags_6 },
    { 0xa3ffdbc9, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dffdbcc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xe4dfdbcf, SUC_lexicon_tags_17 },
    { 0x575fdbcf, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x997fdbd2, SUC_lexicon_tags_45 },
    { 0x17dfdbd2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc4dfdbd7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41fdbdc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xf3fdbde, SUC_lexicon_tags_49 },
    { 0x9e3fdbde, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x877fdbe2, SUC_lexicon_tags_15 },
    { 0x54bfdbe3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x57bfdbe5, SUC_lexicon_tags_43 },
    { 0xba9fdbe6, SUC_lexicon_tags_2 },
    { 0x4e1fdbe7, SUC_lexicon_tags_47 },
    { 0x27bfdbe8, SUC_lexicon_tags_15 },
    { 0x77bfdbe9, SUC_lexicon_tags_15 },
    { 0x13dfdbea, SUC_lexicon_tags_7 },
    { 0x7cbfdbeb, SUC_lexicon_tags_25 },
    { 0xa49fdbe7, SUC_lexicon_tags_49 },
    { 0x9a3fdbe9, SUC_lexicon_tags_27 },
    { 0xae1fdbee, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x579fdbf1, SUC_lexicon_tags_6 },
    { 0x47fdbf2, SUC_lexicon_tags_63 },
    { 0xe69fdbf3, SUC_lexicon_tags_1 },
    { 0x85fdbf1, SUC_lexicon_tags_39 },
    { 0x42ffdbf5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe93fdbf7, SUC_lexicon_tags_63 },
    { 0x517fdbf7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xa69fdbfb, SUC_lexicon_tags_141 },
    { 0x9e3fdbfc, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7d3fdbfe, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ebfdc00, SUC_lexicon_tags_8 },
    { 0x46dfdc01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xa7ffdc03, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3fdc0e, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x197fdc10, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bfdc14, SUC_lexicon_tags_25 },
    { 0xb6ffdc14, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x12dfdc1a, SUC_lexicon_tags_12 },
    { 0xe07fdc1a, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7fdc20, SUC_lexicon_tags_7 },
    { 0xafdfdc21, SUC_lexicon_tags_47 },
    { 0x45ffdc21, SUC_lexicon_tags_37 },
    { 0xe97fdc23, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xf45fdc25, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x613fdc28, SUC_lexicon_tags_12 },
    { 0xde3fdc28, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31fdc2d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x691fdc2f, SUC_lexicon_tags_6 },
    { 0xd79fdc30, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67fdc36, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dfdc39, SUC_lexicon_tags_60 },
    { 0xa93fdc39, SUC_lexicon_tags_31 },
    { 0xb59fdc3b, SUC_lexicon_tags_19 },
    { 0xbf1fdc3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x3a3fdc3f, SUC_lexicon_tags_4 },
    { 0x3e9fdc3f, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59fdc49, SUC_lexicon_tags_47 },
    { 0x51bfdc49, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x735fdc4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd69fdc51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x697fdc54, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x7bbfdc56, SUC_lexicon_tags_17 },
    { 0x4b7fdc57, SUC_lexicon_tags_8 },
    { 0xfcdfdc57, SUC_lexicon_tags_15 },
    { 0xf0bfdc58, SUC_lexicon_tags_2 },
    { 0x991fdc5a, SUC_lexicon_tags_11 },
    { 0x27bfdc5b, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb3dfdc64, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39fdc68, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dfdc6b, SUC_lexicon_tags_25 },
    { 0x5ddfdc6b, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbfdc70, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9fdc7a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x1adfdc7d, SUC_lexicon_tags_92 },
    { 0x41dfdc7d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x111fdc80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x26bfdc86, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fdfdc8a, SUC_lexicon_tags_74 },
    { 0xfb5fdc8b, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xe2bfdc8d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9fdc91, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x37bfdc93, SUC_lexicon_tags_65 },
    { 0x791fdc93, SUC_lexicon_tags_17 },
    { 0xba5fdc93, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb1fdc97, SUC_lexicon_tags_47 },
    { 0x88ffdc98, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xfdffdc9a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x2cbfdc9c, SUC_lexicon_tags_47 },
    { 0xd27fdc9d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa19fdca3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x907fdca5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x71ffdca7, SUC_lexicon_tags_64 },
    { 0xfa5fdca7, SUC_lexicon_tags_12 },
    { 0x63ffdca8, SUC_lexicon_tags_63 },
    { 0x639fdca7, SUC_lexicon_tags_22 },
    { 0xf77fdca9, SUC_lexicon_tags_234 },
    { 0x863fdca8, SUC_lexicon_tags_0 },
    { 0x2bdfdcad, SUC_lexicon_tags_34 },
    { 0x8bbfdcae, SUC_lexicon_tags_33 },
    { 0xd33fdca9, SUC_lexicon_tags_45 },
    { 0xf4dfdcb0, SUC_lexicon_tags_59 },
    { 0x5a1fdcb1, SUC_lexicon_tags_25 },
    { 0x55ffdcb2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8b5fdcb7, SUC_lexicon_tags_6 },
    { 0x49bfdcb7, SUC_lexicon_tags_2 },
    { 0xb17fdcb8, SUC_lexicon_tags_96 },
    { 0x785fdcba, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x8dfdcbc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1fdcbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x333fdcc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32bfdcc5, SUC_lexicon_tags_7 },
    { 0x3ffdcc6, SUC_lexicon_tags_46 },
    { 0xdd7fdcc7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc37fdcc9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475fdcd1, SUC_lexicon_tags_389 },
    { 0, NULL },
    { 0xbc7fdcd3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xba7fdcd5, SUC_lexicon_tags_37 },
    { 0x841fdcd6, SUC_lexicon_tags_61 },
    { 0xd69fdcd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2a3fdcdb, SUC_lexicon_tags_62 },
    { 0x237fdcdb, SUC_lexicon_tags_43 },
    { 0xa05fdcdb, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebfdce1, SUC_lexicon_tags_15 },
    { 0xfd7fdce1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x767fdce4, SUC_lexicon_tags_16 },
    { 0x4ffdce4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa61fdceb, SUC_lexicon_tags_85 },
    { 0x85ffdceb, SUC_lexicon_tags_15 },
    { 0x567fdced, SUC_lexicon_tags_17 },
    { 0xd11fdced, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x539fdcf0, SUC_lexicon_tags_14 },
    { 0xca1fdcf1, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xac3fdcf3, SUC_lexicon_tags_0 },
    { 0x725fdcf3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa5dfdcf6, SUC_lexicon_tags_28 },
    { 0x95ffdcf6, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9fdcfa, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x283fdcfc, SUC_lexicon_tags_15 },
    { 0x733fdcfc, SUC_lexicon_tags_1 },
    { 0xb75fdcfe, SUC_lexicon_tags_82 },
    { 0x551fdcff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe3dfdd04, SUC_lexicon_tags_10 },
    { 0x5ddfdd04, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x239fdd09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb3fdd0b, SUC_lexicon_tags_25 },
    { 0xdc5fdd0b, SUC_lexicon_tags_82 },
    { 0xd51fdd0d, SUC_lexicon_tags_25 },
    { 0xbe1fdd0c, SUC_lexicon_tags_11 },
    { 0xcb3fdd0f, SUC_lexicon_tags_25 },
    { 0xd93fdd10, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x46bfdd12, SUC_lexicon_tags_7 },
    { 0x5c9fdd13, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bfdd1a, SUC_lexicon_tags_16 },
    { 0xd9dfdd1b, SUC_lexicon_tags_12 },
    { 0x983fdd1c, SUC_lexicon_tags_23 },
    { 0x893fdd1c, SUC_lexicon_tags_13 },
    { 0x9b7fdd1d, SUC_lexicon_tags_37 },
    { 0xf5bfdd1d, SUC_lexicon_tags_27 },
    { 0xb4bfdd20, SUC_lexicon_tags_11 },
    { 0x7abfdd21, SUC_lexicon_tags_38 },
    { 0x16ffdd1b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ddfdd24, SUC_lexicon_tags_15 },
    { 0xd6dfdd24, SUC_lexicon_tags_39 },
    { 0xe5fdd24, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xafdfdd29, SUC_lexicon_tags_47 },
    { 0xaf9fdd2a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x471fdd2c, SUC_lexicon_tags_208 },
    { 0xb55fdd2d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x33ffdd35, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb4ffdd37, SUC_lexicon_tags_7 },
    { 0x3dfdd38, SUC_lexicon_tags_71 },
    { 0x27dfdd38, SUC_lexicon_tags_33 },
    { 0xa07fdd3a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x9ffdd3c, SUC_lexicon_tags_28 },
    { 0xaffdd3d, SUC_lexicon_tags_17 },
    { 0x5ddfdd3c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46bfdd43, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xe59fdd46, SUC_lexicon_tags_37 },
    { 0x4fffdd47, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0x7abfdd4a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1fdd50, SUC_lexicon_tags_38 },
    { 0xe3bfdd50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ffdd56, SUC_lexicon_tags_19 },
    { 0x11bfdd57, SUC_lexicon_tags_25 },
    { 0x929fdd56, SUC_lexicon_tags_19 },
    { 0x625fdd59, SUC_lexicon_tags_8 },
    { 0x533fdd58, SUC_lexicon_tags_60 },
    { 0xb57fdd57, SUC_lexicon_tags_9 },
    { 0xa99fdd5c, SUC_lexicon_tags_47 },
    { 0x18bfdd5d, SUC_lexicon_tags_48 },
    { 0x393fdd5e, SUC_lexicon_tags_35 },
    { 0x6f3fdd5c, SUC_lexicon_tags_1 },
    { 0x54ffdd60, SUC_lexicon_tags_28 },
    { 0x813fdd61, SUC_lexicon_tags_0 },
    { 0xc73fdd5f, SUC_lexicon_tags_70 },
    { 0xec5fdd59, SUC_lexicon_tags_17 },
    { 0x95bfdd5f, SUC_lexicon_tags_59 },
    { 0x21fdd65, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x69fdd67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727fdd6b, SUC_lexicon_tags_23 },
    { 0xbfdfdd6c, SUC_lexicon_tags_33 },
    { 0x759fdd6c, SUC_lexicon_tags_47 },
    { 0xea3fdd6e, SUC_lexicon_tags_46 },
    { 0xcd9fdd6e, SUC_lexicon_tags_15 },
    { 0x17ffdd70, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x587fdd72, SUC_lexicon_tags_1 },
    { 0xf2bfdd73, SUC_lexicon_tags_37 },
    { 0x359fdd73, SUC_lexicon_tags_43 },
    { 0x3fbfdd75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x8fbfdd77, SUC_lexicon_tags_26 },
    { 0x68bfdd77, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x4a7fdd7a, SUC_lexicon_tags_10 },
    { 0x28ffdd7b, SUC_lexicon_tags_30 },
    { 0x7d7fdd7a, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe13fdd81, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x73ffdd83, SUC_lexicon_tags_17 },
    { 0xf19fdd84, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbddfdd86, SUC_lexicon_tags_7 },
    { 0x89fdd87, SUC_lexicon_tags_21 },
    { 0x693fdd88, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xc71fdd8b, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd61fdd98, SUC_lexicon_tags_38 },
    { 0xe07fdd99, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4f5fdd9e, SUC_lexicon_tags_12 },
    { 0xaf7fdd9e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x381fdda1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x321fdda4, SUC_lexicon_tags_31 },
    { 0xb6dfdda4, SUC_lexicon_tags_37 },
    { 0x5f1fdda6, SUC_lexicon_tags_29 },
    { 0xa15fdda6, SUC_lexicon_tags_12 },
    { 0xd4bfdda7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfdfddab, SUC_lexicon_tags_80 },
    { 0x5a7fddac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x75ffddaf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x6dffddb1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf99fddb6, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9fddbb, SUC_lexicon_tags_2 },
    { 0xe93fddbc, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafdfddc0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bfddc3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x829fddc5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91dfddd0, SUC_lexicon_tags_83 },
    { 0xdabfddd1, SUC_lexicon_tags_1 },
    { 0x9d1fddd2, SUC_lexicon_tags_7 },
    { 0x43bfddd2, SUC_lexicon_tags_53 },
    { 0xfa9fddd2, SUC_lexicon_tags_39 },
    { 0xc31fddd5, SUC_lexicon_tags_12 },
    { 0x89bfddd6, SUC_lexicon_tags_19 },
    { 0xc59fddd1, SUC_lexicon_tags_14 },
    { 0x3ddfddd8, SUC_lexicon_tags_39 },
    { 0xd2ffddd8, SUC_lexicon_tags_27 },
    { 0x15ffddd0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7fddde, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x151fdde1, SUC_lexicon_tags_17 },
    { 0xb39fdde2, SUC_lexicon_tags_37 },
    { 0x281fdde2, SUC_lexicon_tags_1 },
    { 0xa5fdde3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edfddeb, SUC_lexicon_tags_806 },
    { 0xe27fddec, SUC_lexicon_tags_11 },
    { 0x7d1fddec, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x231fddf3, SUC_lexicon_tags_330 },
    { 0x305fddf3, SUC_lexicon_tags_25 },
    { 0xc45fddf3, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x59ffddf9, SUC_lexicon_tags_7 },
    { 0x2e3fddf9, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xe31fddfd, SUC_lexicon_tags_57 },
    { 0x9f1fddfe, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bfde04, SUC_lexicon_tags_11 },
    { 0xabbfde04, SUC_lexicon_tags_38 },
    { 0xe9ffde06, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe1fde08, SUC_lexicon_tags_25 },
    { 0x52bfde09, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc19fde0e, SUC_lexicon_tags_29 },
    { 0xbb1fde0e, SUC_lexicon_tags_6 },
    { 0x66bfde0e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x163fde12, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8a9fde14, SUC_lexicon_tags_17 },
    { 0xce1fde15, SUC_lexicon_tags_14 },
    { 0x699fde16, SUC_lexicon_tags_65 },
    { 0xb7dfde17, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xc97fde19, SUC_lexicon_tags_15 },
    { 0x60bfde1a, SUC_lexicon_tags_8 },
    { 0x329fde19, SUC_lexicon_tags_17 },
    { 0x881fde1c, SUC_lexicon_tags_22 },
    { 0x757fde19, SUC_lexicon_tags_2 },
    { 0x425fde19, SUC_lexicon_tags_141 },
    { 0x5b5fde1f, SUC_lexicon_tags_55 },
    { 0x373fde20, SUC_lexicon_tags_19 },
    { 0xa27fde1f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5fde24, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xe39fde27, SUC_lexicon_tags_46 },
    { 0xd99fde28, SUC_lexicon_tags_33 },
    { 0xd5ffde29, SUC_lexicon_tags_21 },
    { 0xd49fde2a, SUC_lexicon_tags_12 },
    { 0xcb1fde27, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3fde2f, SUC_lexicon_tags_6 },
    { 0x465fde30, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363fde36, SUC_lexicon_tags_81 },
    { 0, NULL },
    { 0x1e9fde38, SUC_lexicon_tags_1 },
    { 0xbc1fde39, SUC_lexicon_tags_47 },
    { 0xa05fde38, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1edfde3c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa53fde3f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc5fde42, SUC_lexicon_tags_30 },
    { 0x52dfde42, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x617fde46, SUC_lexicon_tags_8 },
    { 0x773fde47, SUC_lexicon_tags_15 },
    { 0x803fde48, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe23fde4d, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x875fde50, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x773fde52, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x475fde54, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bfde57, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x33ffde59, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe1dfde5e, SUC_lexicon_tags_37 },
    { 0xae5fde5e, SUC_lexicon_tags_11 },
    { 0x887fde60, SUC_lexicon_tags_60 },
    { 0x95bfde5f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x57bfde63, SUC_lexicon_tags_27 },
    { 0x7a7fde64, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xda9fde66, SUC_lexicon_tags_7 },
    { 0xd97fde67, SUC_lexicon_tags_6 },
    { 0xeeffde68, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd2dfde6b, SUC_lexicon_tags_2 },
    { 0x43ffde6b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e7fde72, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9ffde7c, SUC_lexicon_tags_53 },
    { 0x155fde7d, SUC_lexicon_tags_0 },
    { 0xe4ffde7e, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x971fde80, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf61fde89, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bfde94, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb55fde9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbfdea0, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x285fdea7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dbfdeab, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbfdeaf, SUC_lexicon_tags_15 },
    { 0x19fdeaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bfdeb5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x57ffdeb8, SUC_lexicon_tags_15 },
    { 0xc91fdeb8, SUC_lexicon_tags_1 },
    { 0x8d7fdeb8, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xafdfdebc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd3fdebf, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdfdec2, SUC_lexicon_tags_0 },
    { 0xc49fdec2, SUC_lexicon_tags_30 },
    { 0x931fdec4, SUC_lexicon_tags_17 },
    { 0x8b1fdec5, SUC_lexicon_tags_71 },
    { 0xc0ffdec6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc93fdece, SUC_lexicon_tags_15 },
    { 0xca1fdecf, SUC_lexicon_tags_2 },
    { 0xec7fdecf, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc79fdede, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7fdfdee6, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c7fdef1, SUC_lexicon_tags_21 },
    { 0x829fdef2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad9fdef6, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdfdef9, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fffdefc, SUC_lexicon_tags_2 },
    { 0x8a5fdefc, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35fdf01, SUC_lexicon_tags_23 },
    { 0x4cdfdf02, SUC_lexicon_tags_87 },
    { 0xfb9fdf01, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe37fdf12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x55dfdf16, SUC_lexicon_tags_17 },
    { 0xb27fdf17, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dfdf1a, SUC_lexicon_tags_26 },
    { 0xa8bfdf1b, SUC_lexicon_tags_8 },
    { 0x1bffdf1b, SUC_lexicon_tags_37 },
    { 0x669fdf1d, SUC_lexicon_tags_30 },
    { 0xc35fdf1b, SUC_lexicon_tags_19 },
    { 0x3d1fdf1f, SUC_lexicon_tags_77 },
    { 0xf11fdf20, SUC_lexicon_tags_38 },
    { 0x159fdf21, SUC_lexicon_tags_27 },
    { 0x98dfdf22, SUC_lexicon_tags_47 },
    { 0x1a7fdf23, SUC_lexicon_tags_27 },
    { 0x171fdf24, SUC_lexicon_tags_17 },
    { 0xd5bfdf24, SUC_lexicon_tags_45 },
    { 0x2dffdf23, SUC_lexicon_tags_15 },
    { 0xd07fdf23, SUC_lexicon_tags_15 },
    { 0x289fdf28, SUC_lexicon_tags_43 },
    { 0xf9ffdf26, SUC_lexicon_tags_15 },
    { 0x47ffdf21, SUC_lexicon_tags_2 },
    { 0x6f3fdf1c, SUC_lexicon_tags_129 },
    { 0x813fdf22, SUC_lexicon_tags_63 },
    { 0x95bfdf2b, SUC_lexicon_tags_62 },
    { 0xcbfdf1e, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x9fffdf30, SUC_lexicon_tags_243 },
    { 0, NULL },
    { 0, NULL },
    { 0xb0dfdf33, SUC_lexicon_tags_69 },
    { 0x533fdf34, SUC_lexicon_tags_25 },
    { 0x515fdf35, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb11fdf39, SUC_lexicon_tags_104 },
    { 0, NULL },
    { 0xa4bfdf3b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ffdf40, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xff1fdf42, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf2bfdf46, SUC_lexicon_tags_17 },
    { 0xdbbfdf47, SUC_lexicon_tags_43 },
    { 0x85ffdf47, SUC_lexicon_tags_12 },
    { 0x693fdf49, SUC_lexicon_tags_25 },
    { 0xe5fdf4a, SUC_lexicon_tags_0 },
    { 0x86ffdf4a, SUC_lexicon_tags_17 },
    { 0x209fdf4c, SUC_lexicon_tags_25 },
    { 0x65dfdf4c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x459fdf51, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe47fdf54, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1fdf57, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfc7fdf5c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x145fdf6b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6bfdf73, SUC_lexicon_tags_15 },
    { 0x917fdf74, SUC_lexicon_tags_25 },
    { 0x81fdf75, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x50bfdf77, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7fdf7b, SUC_lexicon_tags_7 },
    { 0x2bfdf7c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa4dfdf7e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xcdbfdf80, SUC_lexicon_tags_60 },
    { 0x94ffdf80, SUC_lexicon_tags_25 },
    { 0xe79fdf82, SUC_lexicon_tags_23 },
    { 0xa23fdf83, SUC_lexicon_tags_28 },
    { 0x1dffdf84, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x16dfdf87, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc5fdf8c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xe77fdf8f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xf57fdf92, SUC_lexicon_tags_37 },
    { 0xec7fdf92, SUC_lexicon_tags_61 },
    { 0xfbbfdf92, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0xd27fdf97, SUC_lexicon_tags_2 },
    { 0xa39fdf97, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe65fdf9c, SUC_lexicon_tags_7 },
    { 0xc27fdf9d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x605fdfa0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xebffdfa2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf11fdfa9, SUC_lexicon_tags_70 },
    { 0x1affdfaa, SUC_lexicon_tags_17 },
    { 0x9fbfdfa9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9fdfb1, SUC_lexicon_tags_2 },
    { 0xae1fdfb2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xdb7fdfb4, SUC_lexicon_tags_15 },
    { 0xfcffdfb5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9bfdfbd, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3fdfc0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x2e5fdfc2, SUC_lexicon_tags_79 },
    { 0x1affdfc3, SUC_lexicon_tags_26 },
    { 0x561fdfc3, SUC_lexicon_tags_15 },
    { 0x223fdfc5, SUC_lexicon_tags_15 },
    { 0xe0dfdfc6, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe5bfdfc8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xbb9fdfca, SUC_lexicon_tags_37 },
    { 0x783fdfcb, SUC_lexicon_tags_13 },
    { 0xc7ffdfca, SUC_lexicon_tags_3 },
    { 0xb7ffdfcc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xf13fdfd0, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0xcf5fdfd2, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x75fdfd5, SUC_lexicon_tags_1 },
    { 0x331fdfd6, SUC_lexicon_tags_7 },
    { 0xb83fdfd5, SUC_lexicon_tags_68 },
    { 0xf17fdfd5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x849fdfda, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa09fdfdc, SUC_lexicon_tags_7 },
    { 0x133fdfdd, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x74dfdfdf, SUC_lexicon_tags_15 },
    { 0xfcbfdfe0, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x5fffdfe3, SUC_lexicon_tags_2 },
    { 0x81fdfe4, SUC_lexicon_tags_7 },
    { 0x6b3fdfe4, SUC_lexicon_tags_17 },
    { 0x1b1fdfe5, SUC_lexicon_tags_31 },
    { 0x339fdfe6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe45fdfed, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf51fdff4, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x58dfdff7, SUC_lexicon_tags_1 },
    { 0x689fdff7, SUC_lexicon_tags_13 },
    { 0xe37fdff9, SUC_lexicon_tags_15 },
    { 0xc69fdff8, SUC_lexicon_tags_17 },
    { 0xea7fdff7, SUC_lexicon_tags_31 },
    { 0xcfffdff8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7cdfdfff, SUC_lexicon_tags_7 },
    { 0x89bfdfff, SUC_lexicon_tags_25 },
    { 0x343fdfff, SUC_lexicon_tags_37 },
    { 0xa8ffe001, SUC_lexicon_tags_0 },
    { 0x183fe001, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xdd3fe005, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x499fe007, SUC_lexicon_tags_34 },
    { 0xa65fe007, SUC_lexicon_tags_15 },
    { 0xccbfe008, SUC_lexicon_tags_31 },
    { 0xb79fe00a, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0xbd5fe00c, SUC_lexicon_tags_68 },
    { 0x631fe00d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x31ffe012, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcb9fe01b, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98dfe028, SUC_lexicon_tags_0 },
    { 0x491fe029, SUC_lexicon_tags_45 },
    { 0x8c1fe029, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x675fe02c, SUC_lexicon_tags_37 },
    { 0x719fe02c, SUC_lexicon_tags_49 },
    { 0x35dfe02e, SUC_lexicon_tags_37 },
    { 0xb9ffe02f, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x97fe031, SUC_lexicon_tags_25 },
    { 0x73fe031, SUC_lexicon_tags_178 },
    { 0x351fe031, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847fe03b, SUC_lexicon_tags_1 },
    { 0x937fe03b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x80ffe03e, SUC_lexicon_tags_57 },
    { 0xde1fe03f, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe31fe043, SUC_lexicon_tags_25 },
    { 0x51fe044, SUC_lexicon_tags_9 },
    { 0x5d1fe045, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x391fe047, SUC_lexicon_tags_9 },
    { 0x48dfe048, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0ffe04b, SUC_lexicon_tags_17 },
    { 0x357fe04c, SUC_lexicon_tags_60 },
    { 0xae9fe04c, SUC_lexicon_tags_47 },
    { 0xc2dfe04e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x637fe050, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c5fe056, SUC_lexicon_tags_49 },
    { 0x3ebfe057, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x565fe059, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x829fe05c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf7ffe05e, SUC_lexicon_tags_17 },
    { 0x75ffe05f, SUC_lexicon_tags_31 },
    { 0x24ffe060, SUC_lexicon_tags_17 },
    { 0xf99fe061, SUC_lexicon_tags_26 },
    { 0x6c1fe05f, SUC_lexicon_tags_63 },
    { 0x7f5fe05e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa05fe067, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xce9fe069, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6ffe06b, SUC_lexicon_tags_17 },
    { 0x761fe06c, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7fe070, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x243fe072, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bfe075, SUC_lexicon_tags_37 },
    { 0x743fe076, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe07fe07a, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x6ddfe07c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a7fe084, SUC_lexicon_tags_39 },
    { 0x5a1fe085, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xc0dfe088, SUC_lexicon_tags_8 },
    { 0x6dffe089, SUC_lexicon_tags_56 },
    { 0x9effe08a, SUC_lexicon_tags_37 },
    { 0xa43fe08a, SUC_lexicon_tags_0 },
    { 0xf5ffe08c, SUC_lexicon_tags_148 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x807fe091, SUC_lexicon_tags_84 },
    { 0xcc1fe092, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xcd9fe095, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedffe0a0, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xebfe0a2, SUC_lexicon_tags_24 },
    { 0x417fe0a2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7bfe0a5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x513fe0a7, SUC_lexicon_tags_31 },
    { 0x5fe0a8, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f9fe0ad, SUC_lexicon_tags_9 },
    { 0x883fe0ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf33fe0b4, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xedffe0c1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7c7fe0c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a7fe0c9, SUC_lexicon_tags_27 },
    { 0x54dfe0ca, SUC_lexicon_tags_57 },
    { 0x27ffe0cb, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdfe0ce, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5fe0d2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x5b7fe0d4, SUC_lexicon_tags_37 },
    { 0xbdffe0d5, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x1f9fe0d7, SUC_lexicon_tags_35 },
    { 0xe43fe0d8, SUC_lexicon_tags_27 },
    { 0xedffe0d9, SUC_lexicon_tags_37 },
    { 0x511fe0d9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5fe0dd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x429fe0e7, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa89fe0eb, SUC_lexicon_tags_98 },
    { 0xf9ffe0ec, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6effe0f0, SUC_lexicon_tags_16 },
    { 0x151fe0f1, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xe2dfe0f3, SUC_lexicon_tags_6 },
    { 0xb9fe0f4, SUC_lexicon_tags_11 },
    { 0x149fe0f5, SUC_lexicon_tags_7 },
    { 0xccbfe0f5, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x19fe0f8, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x4d5fe0fa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa17fe0fc, SUC_lexicon_tags_37 },
    { 0x379fe0fc, SUC_lexicon_tags_25 },
    { 0xf09fe0fe, SUC_lexicon_tags_7 },
    { 0xa9bfe0ff, SUC_lexicon_tags_13 },
    { 0xba1fe0ff, SUC_lexicon_tags_37 },
    { 0xf49fe0fc, SUC_lexicon_tags_82 },
    { 0x6c1fe102, SUC_lexicon_tags_2 },
    { 0xb7dfe103, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x8e5fe106, SUC_lexicon_tags_44 },
    { 0xac5fe107, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1fe10b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xcc3fe10d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9fe114, SUC_lexicon_tags_77 },
    { 0x585fe115, SUC_lexicon_tags_7 },
    { 0x395fe115, SUC_lexicon_tags_37 },
    { 0xa95fe115, SUC_lexicon_tags_38 },
    { 0x2bffe114, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xa9dfe11a, SUC_lexicon_tags_37 },
    { 0x975fe11b, SUC_lexicon_tags_11 },
    { 0x787fe11c, SUC_lexicon_tags_12 },
    { 0xbd3fe11d, SUC_lexicon_tags_2 },
    { 0x3a7fe11c, SUC_lexicon_tags_13 },
    { 0x27bfe11d, SUC_lexicon_tags_11 },
    { 0xb05fe11d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dfe124, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xafbfe126, SUC_lexicon_tags_11 },
    { 0x535fe126, SUC_lexicon_tags_12 },
    { 0x953fe126, SUC_lexicon_tags_17 },
    { 0xa47fe126, SUC_lexicon_tags_25 },
    { 0xe1bfe128, SUC_lexicon_tags_15 },
    { 0xce3fe12b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x293fe12e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3fe131, SUC_lexicon_tags_1 },
    { 0x635fe131, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xbe3fe134, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x727fe139, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xdedfe13c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905fe142, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa51fe146, SUC_lexicon_tags_37 },
    { 0xe8dfe147, SUC_lexicon_tags_23 },
    { 0x811fe148, SUC_lexicon_tags_31 },
    { 0x9dbfe149, SUC_lexicon_tags_2 },
    { 0xe9fe14a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xd1ffe14c, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x9d3fe14f, SUC_lexicon_tags_10 },
    { 0x51ffe150, SUC_lexicon_tags_15 },
    { 0xef1fe151, SUC_lexicon_tags_46 },
    { 0x599fe150, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x58ffe154, SUC_lexicon_tags_11 },
    { 0xd1dfe155, SUC_lexicon_tags_37 },
    { 0xc17fe156, SUC_lexicon_tags_22 },
    { 0x58dfe155, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x32dfe15a, SUC_lexicon_tags_14 },
    { 0x929fe15a, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9fe162, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99fe166, SUC_lexicon_tags_14 },
    { 0x55bfe166, SUC_lexicon_tags_15 },
    { 0xfa1fe168, SUC_lexicon_tags_1 },
    { 0x2a1fe168, SUC_lexicon_tags_65 },
    { 0xb39fe168, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x8b7fe16d, SUC_lexicon_tags_68 },
    { 0x82bfe16e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x5b1fe171, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xeadfe173, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdfe176, SUC_lexicon_tags_48 },
    { 0x735fe177, SUC_lexicon_tags_45 },
    { 0x12ffe177, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x81ffe17a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa27fe187, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xa0dfe18a, SUC_lexicon_tags_87 },
    { 0x715fe18a, SUC_lexicon_tags_15 },
    { 0xb4bfe18a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd67fe191, SUC_lexicon_tags_37 },
    { 0xc9ffe192, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x12dfe194, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0xe6bfe196, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6e5fe198, SUC_lexicon_tags_83 },
    { 0, NULL },
    { 0x733fe19a, SUC_lexicon_tags_47 },
    { 0x56ffe19b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x8a1fe19d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf1fe1a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bffe1a5, SUC_lexicon_tags_11 },
    { 0xdadfe1a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x23ffe1b0, SUC_lexicon_tags_30 },
    { 0x58bfe1b1, SUC_lexicon_tags_7 },
    { 0x963fe1b1, SUC_lexicon_tags_2 },
    { 0x733fe1b3, SUC_lexicon_tags_44 },
    { 0xcd7fe1b4, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5c5fe1b7, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x723fe1b9, SUC_lexicon_tags_25 },
    { 0xee5fe1ba, SUC_lexicon_tags_1 },
    { 0x3a7fe1bb, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x94bfe1be, SUC_lexicon_tags_127 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebfe1c6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcc1fe1cc, SUC_lexicon_tags_7 },
    { 0x377fe1cd, SUC_lexicon_tags_37 },
    { 0x537fe1ce, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4fdfe1d2, SUC_lexicon_tags_9 },
    { 0x233fe1d3, SUC_lexicon_tags_1 },
    { 0x31dfe1d3, SUC_lexicon_tags_108 },
    { 0x57bfe1d3, SUC_lexicon_tags_27 },
    { 0xe2bfe1d6, SUC_lexicon_tags_1 },
    { 0xd71fe1d5, SUC_lexicon_tags_64 },
    { 0x4effe1d8, SUC_lexicon_tags_15 },
    { 0xae3fe1d3, SUC_lexicon_tags_12 },
    { 0x2f9fe1da, SUC_lexicon_tags_12 },
    { 0x181fe1db, SUC_lexicon_tags_71 },
    { 0x29fe1db, SUC_lexicon_tags_23 },
    { 0x2dbfe1dd, SUC_lexicon_tags_0 },
    { 0x4ebfe1de, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0x1f1fe1e0, SUC_lexicon_tags_15 },
    { 0x85dfe1e1, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x517fe1e6, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x549fe1e8, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1fe1ed, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2ebfe1f0, SUC_lexicon_tags_59 },
    { 0xe99fe1f0, SUC_lexicon_tags_28 },
    { 0x1bfe1f2, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45bfe1f6, SUC_lexicon_tags_31 },
    { 0xf37fe1f7, SUC_lexicon_tags_30 },
    { 0x575fe1f7, SUC_lexicon_tags_7 },
    { 0xc63fe1f6, SUC_lexicon_tags_170 },
    { 0xe9ffe1f8, SUC_lexicon_tags_17 },
    { 0x82dfe1f8, SUC_lexicon_tags_51 },
    { 0xfcbfe1fc, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x857fe1ff, SUC_lexicon_tags_17 },
    { 0x429fe200, SUC_lexicon_tags_15 },
    { 0x3a3fe200, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x5e7fe204, SUC_lexicon_tags_77 },
    { 0x77bfe205, SUC_lexicon_tags_14 },
    { 0x5b5fe205, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x237fe208, SUC_lexicon_tags_37 },
    { 0x85ffe209, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeaffe20e, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x89ffe215, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3d3fe21c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a9fe220, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x597fe225, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdfe229, SUC_lexicon_tags_7 },
    { 0x523fe22a, SUC_lexicon_tags_29 },
    { 0x6fffe229, SUC_lexicon_tags_69 },
    { 0x8f3fe22b, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa3ffe22e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9f1fe230, SUC_lexicon_tags_3 },
    { 0x8e5fe231, SUC_lexicon_tags_29 },
    { 0x3a3fe232, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa23fe234, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbb3fe239, SUC_lexicon_tags_11 },
    { 0x8adfe239, SUC_lexicon_tags_37 },
    { 0x1a3fe23a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xbdfe23e, SUC_lexicon_tags_1 },
    { 0xca7fe23e, SUC_lexicon_tags_7 },
    { 0xbc9fe23e, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x48dfe243, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x3fffe245, SUC_lexicon_tags_2 },
    { 0x381fe245, SUC_lexicon_tags_6 },
    { 0x27bfe245, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xf8bfe249, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x893fe24c, SUC_lexicon_tags_17 },
    { 0xea1fe24d, SUC_lexicon_tags_15 },
    { 0x22dfe24e, SUC_lexicon_tags_12 },
    { 0xe09fe24f, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x339fe251, SUC_lexicon_tags_60 },
    { 0x51fe252, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x2bbfe254, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xfabfe256, SUC_lexicon_tags_29 },
    { 0x283fe256, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x359fe25b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe69fe25f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xe6dfe261, SUC_lexicon_tags_25 },
    { 0xfb3fe262, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x81ffe264, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x65fe266, SUC_lexicon_tags_37 },
    { 0xb25fe266, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x931fe26f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdfbfe271, SUC_lexicon_tags_68 },
    { 0x667fe272, SUC_lexicon_tags_19 },
    { 0xa77fe273, SUC_lexicon_tags_17 },
    { 0xcb9fe271, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa29fe277, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xfc1fe279, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x8cbfe27b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xeb5fe27d, SUC_lexicon_tags_15 },
    { 0xb4bfe27e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70ffe282, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7bdfe286, SUC_lexicon_tags_14 },
    { 0xce3fe287, SUC_lexicon_tags_15 },
    { 0xfdfe288, SUC_lexicon_tags_17 },
    { 0x473fe288, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x8fffe28b, SUC_lexicon_tags_11 },
    { 0x817fe28c, SUC_lexicon_tags_77 },
    { 0x5b1fe28d, SUC_lexicon_tags_25 },
    { 0xdabfe28e, SUC_lexicon_tags_33 },
    { 0x8e9fe28f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x497fe293, SUC_lexicon_tags_1 },
    { 0xd4dfe293, SUC_lexicon_tags_70 },
    { 0xafdfe295, SUC_lexicon_tags_11 },
    { 0xc49fe296, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x48dfe299, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c5fe29d, SUC_lexicon_tags_66 },
    { 0xe8ffe29d, SUC_lexicon_tags_127 },
    { 0xc05fe29f, SUC_lexicon_tags_53 },
    { 0x255fe29f, SUC_lexicon_tags_23 },
    { 0x4edfe2a1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x40ffe2a3, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adfe2a6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xbe1fe2a8, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x551fe2aa, SUC_lexicon_tags_63 },
    { 0x845fe2ab, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xee1fe2ad, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x9dfe2af, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x101fe2b1, SUC_lexicon_tags_47 },
    { 0x27bfe2b2, SUC_lexicon_tags_1 },
    { 0xb89fe2b2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5fe2b7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x425fe2bb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7fe2c0, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x92dfe2c3, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x1f7fe2c5, SUC_lexicon_tags_45 },
    { 0xa5fe2c6, SUC_lexicon_tags_37 },
    { 0x8c5fe2c7, SUC_lexicon_tags_23 },
    { 0xe85fe2c5, SUC_lexicon_tags_47 },
    { 0xd35fe2c9, SUC_lexicon_tags_11 },
    { 0xd39fe2c9, SUC_lexicon_tags_55 },
    { 0x4e1fe2c6, SUC_lexicon_tags_15 },
    { 0x12ffe2c5, SUC_lexicon_tags_37 },
    { 0x543fe2c9, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x345fe2d2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21bfe2d6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475fe2db, SUC_lexicon_tags_14 },
    { 0x1b1fe2db, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x73bfe2de, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcbfe2e1, SUC_lexicon_tags_96 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ffe2e6, SUC_lexicon_tags_1 },
    { 0x575fe2e7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x773fe2ea, SUC_lexicon_tags_8 },
    { 0xf43fe2ea, SUC_lexicon_tags_28 },
    { 0x955fe2ec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa93fe2f0, SUC_lexicon_tags_68 },
    { 0x7d7fe2f0, SUC_lexicon_tags_7 },
    { 0x967fe2f2, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x911fe2f4, SUC_lexicon_tags_4 },
    { 0x58ffe2f4, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xee9fe2f9, SUC_lexicon_tags_35 },
    { 0xfc9fe2fa, SUC_lexicon_tags_15 },
    { 0xc79fe2fb, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0x4e5fe2fd, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xcf3fe2ff, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa2bfe304, SUC_lexicon_tags_873 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a9fe30a, SUC_lexicon_tags_29 },
    { 0xe77fe30b, SUC_lexicon_tags_87 },
    { 0x2bffe30a, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x1ddfe30e, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x397fe316, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x729fe319, SUC_lexicon_tags_31 },
    { 0x651fe31a, SUC_lexicon_tags_37 },
    { 0x21bfe31b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5fe31e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8a5fe323, SUC_lexicon_tags_30 },
    { 0x41bfe324, SUC_lexicon_tags_23 },
    { 0xaa3fe325, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a3fe32a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x885fe32d, SUC_lexicon_tags_55 },
    { 0xbf3fe32e, SUC_lexicon_tags_8 },
    { 0x6d7fe32f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x989fe333, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x76ffe336, SUC_lexicon_tags_11 },
    { 0x9bfe337, SUC_lexicon_tags_37 },
    { 0xc1dfe338, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49fe341, SUC_lexicon_tags_11 },
    { 0x665fe342, SUC_lexicon_tags_38 },
    { 0x2a1fe343, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc67fe34d, SUC_lexicon_tags_7 },
    { 0xe2dfe34e, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bfe353, SUC_lexicon_tags_2 },
    { 0xc6bfe354, SUC_lexicon_tags_11 },
    { 0x255fe353, SUC_lexicon_tags_12 },
    { 0x4c5fe356, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7dfe35b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5fe35f, SUC_lexicon_tags_23 },
    { 0xe07fe360, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x12ffe362, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17ffe369, SUC_lexicon_tags_23 },
    { 0xd4bfe36a, SUC_lexicon_tags_25 },
    { 0x80ffe36a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc59fe36f, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x20bfe372, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7c5fe37a, SUC_lexicon_tags_87 },
    { 0x853fe37a, SUC_lexicon_tags_47 },
    { 0x6cffe37c, SUC_lexicon_tags_27 },
    { 0xa1bfe37d, SUC_lexicon_tags_63 },
    { 0x727fe37e, SUC_lexicon_tags_11 },
    { 0xe35fe37d, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd63fe381, SUC_lexicon_tags_87 },
    { 0x59ffe382, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x735fe384, SUC_lexicon_tags_45 },
    { 0xadbfe385, SUC_lexicon_tags_7 },
    { 0x1dbfe385, SUC_lexicon_tags_343 },
    { 0xa9fe387, SUC_lexicon_tags_23 },
    { 0x1bbfe387, SUC_lexicon_tags_68 },
    { 0xd9bfe387, SUC_lexicon_tags_0 },
    { 0x549fe38a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3effe38e, SUC_lexicon_tags_13 },
    { 0xefbfe38f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xae9fe391, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09fe396, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ffe39b, SUC_lexicon_tags_1 },
    { 0xeebfe39c, SUC_lexicon_tags_12 },
    { 0x709fe39c, SUC_lexicon_tags_47 },
    { 0x187fe39d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa1ffe3a0, SUC_lexicon_tags_30 },
    { 0xa7bfe3a1, SUC_lexicon_tags_55 },
    { 0xa43fe3a2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x349fe3a5, SUC_lexicon_tags_35 },
    { 0x4d1fe3a6, SUC_lexicon_tags_27 },
    { 0x2d7fe3a7, SUC_lexicon_tags_0 },
    { 0xfc5fe3a8, SUC_lexicon_tags_87 },
    { 0x99dfe3a6, SUC_lexicon_tags_11 },
    { 0x5dbfe3a6, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xc5dfe3ac, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16ffe3b1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x373fe3b5, SUC_lexicon_tags_15 },
    { 0x827fe3b5, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x98bfe3b8, SUC_lexicon_tags_23 },
    { 0x8fffe3b8, SUC_lexicon_tags_16 },
    { 0x723fe3ba, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x399fe3bc, SUC_lexicon_tags_25 },
    { 0xc07fe3bd, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x14ffe3c1, SUC_lexicon_tags_47 },
    { 0xd33fe3c1, SUC_lexicon_tags_37 },
    { 0x99dfe3c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd95fe3c7, SUC_lexicon_tags_35 },
    { 0x143fe3c8, SUC_lexicon_tags_25 },
    { 0xc37fe3c9, SUC_lexicon_tags_15 },
    { 0xe05fe3ca, SUC_lexicon_tags_25 },
    { 0xa87fe3c9, SUC_lexicon_tags_26 },
    { 0x9e9fe3cb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xc11fe3cf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xba9fe3d8, SUC_lexicon_tags_6 },
    { 0x75ffe3d9, SUC_lexicon_tags_1264 },
    { 0, NULL },
    { 0x6c9fe3db, SUC_lexicon_tags_201 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb0ffe3df, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x4bbfe3e1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dffe3e9, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xe81fe3ec, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1bdfe3f0, SUC_lexicon_tags_65 },
    { 0x1a7fe3f1, SUC_lexicon_tags_2 },
    { 0x25dfe3f0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe29fe3fa, SUC_lexicon_tags_31 },
    { 0x3effe3fb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x813fe3fd, SUC_lexicon_tags_10 },
    { 0x545fe3fe, SUC_lexicon_tags_27 },
    { 0x6dbfe3ff, SUC_lexicon_tags_0 },
    { 0xcc1fe400, SUC_lexicon_tags_1 },
    { 0x8e5fe401, SUC_lexicon_tags_37 },
    { 0x67fe402, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe65fe404, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec7fe409, SUC_lexicon_tags_12 },
    { 0x955fe40a, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1fe40d, SUC_lexicon_tags_47 },
    { 0x18bfe40e, SUC_lexicon_tags_31 },
    { 0xfdfe40f, SUC_lexicon_tags_66 },
    { 0xab5fe410, SUC_lexicon_tags_17 },
    { 0x7a3fe411, SUC_lexicon_tags_14 },
    { 0x47ffe412, SUC_lexicon_tags_25 },
    { 0x725fe413, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7fe418, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x589fe41b, SUC_lexicon_tags_0 },
    { 0xc5fe41c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x489fe420, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x1c7fe423, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd09fe426, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91bfe42a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x769fe42c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xae3fe42f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xee1fe431, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xf05fe433, SUC_lexicon_tags_17 },
    { 0xd55fe433, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xf93fe436, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8ffe43a, SUC_lexicon_tags_27 },
    { 0x2c5fe43a, SUC_lexicon_tags_15 },
    { 0x7c9fe43c, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x2cbfe43e, SUC_lexicon_tags_11 },
    { 0x779fe43e, SUC_lexicon_tags_39 },
    { 0xed5fe43e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x771fe44b, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xb0bfe44d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18ffe453, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x833fe456, SUC_lexicon_tags_25 },
    { 0x8a9fe456, SUC_lexicon_tags_12 },
    { 0xee1fe458, SUC_lexicon_tags_7 },
    { 0x239fe459, SUC_lexicon_tags_17 },
    { 0x97bfe459, SUC_lexicon_tags_106 },
    { 0x11bfe456, SUC_lexicon_tags_39 },
    { 0x959fe459, SUC_lexicon_tags_32 },
    { 0x753fe456, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x7ebfe45f, SUC_lexicon_tags_4 },
    { 0x587fe45f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xf51fe463, SUC_lexicon_tags_30 },
    { 0x8cdfe464, SUC_lexicon_tags_6 },
    { 0xc27fe465, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x701fe46a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd77fe474, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xabbfe476, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x403fe479, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9fe47b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xa81fe47e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x165fe480, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc3fe485, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xb81fe487, SUC_lexicon_tags_47 },
    { 0x6edfe487, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xa63fe48a, SUC_lexicon_tags_17 },
    { 0xf59fe48b, SUC_lexicon_tags_15 },
    { 0xf55fe48a, SUC_lexicon_tags_15 },
    { 0xd8bfe48b, SUC_lexicon_tags_48 },
    { 0x66ffe48b, SUC_lexicon_tags_11 },
    { 0xd13fe48d, SUC_lexicon_tags_7 },
    { 0x4f7fe48a, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca9fe496, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8bfe4a0, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xde9fe4a2, SUC_lexicon_tags_0 },
    { 0x91bfe4a3, SUC_lexicon_tags_26 },
    { 0x47ffe4a4, SUC_lexicon_tags_37 },
    { 0x731fe4a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a7fe4a9, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bfe4b0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb3fe4b2, SUC_lexicon_tags_49 },
    { 0x2b9fe4b2, SUC_lexicon_tags_11 },
    { 0xb3ffe4b4, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbf5fe4bc, SUC_lexicon_tags_26 },
    { 0xa99fe4bd, SUC_lexicon_tags_28 },
    { 0x9a9fe4bc, SUC_lexicon_tags_17 },
    { 0x957fe4bc, SUC_lexicon_tags_43 },
    { 0xb13fe4c0, SUC_lexicon_tags_14 },
    { 0x433fe4c1, SUC_lexicon_tags_7 },
    { 0x489fe4be, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xd25fe4c5, SUC_lexicon_tags_14 },
    { 0xa7ffe4c6, SUC_lexicon_tags_61 },
    { 0x56dfe4c5, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xf5bfe4ca, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x2b3fe4cc, SUC_lexicon_tags_34 },
    { 0x1f9fe4cd, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xb8dfe4cf, SUC_lexicon_tags_25 },
    { 0xe83fe4cf, SUC_lexicon_tags_25 },
    { 0xdddfe4d1, SUC_lexicon_tags_1 },
    { 0xdedfe4d2, SUC_lexicon_tags_11 },
    { 0x80dfe4d2, SUC_lexicon_tags_220 },
    { 0x61dfe4d2, SUC_lexicon_tags_64 },
    { 0x521fe4d5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbfe4d9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xac5fe4dc, SUC_lexicon_tags_60 },
    { 0xb17fe4dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x573fe4e3, SUC_lexicon_tags_37 },
    { 0xb77fe4e4, SUC_lexicon_tags_15 },
    { 0xd97fe4e5, SUC_lexicon_tags_7 },
    { 0x35fe4e4, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x759fe4ea, SUC_lexicon_tags_25 },
    { 0xffbfe4eb, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5fe4f3, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0ffe4f8, SUC_lexicon_tags_336 },
    { 0xd2dfe4f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x755fe4fb, SUC_lexicon_tags_9 },
    { 0xa19fe4fc, SUC_lexicon_tags_17 },
    { 0xe63fe4fd, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x6dffe4ff, SUC_lexicon_tags_11 },
    { 0x4d3fe500, SUC_lexicon_tags_25 },
    { 0xafffe4ff, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x681fe503, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe67fe505, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x4f9fe508, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb81fe50a, SUC_lexicon_tags_29 },
    { 0x44bfe50a, SUC_lexicon_tags_25 },
    { 0x1bfe50b, SUC_lexicon_tags_9 },
    { 0x99bfe50b, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209fe511, SUC_lexicon_tags_7 },
    { 0x719fe512, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe2dfe524, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x579fe528, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dfe532, SUC_lexicon_tags_64 },
    { 0x985fe532, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfd9fe535, SUC_lexicon_tags_11 },
    { 0xb97fe535, SUC_lexicon_tags_47 },
    { 0x91bfe536, SUC_lexicon_tags_37 },
    { 0xbbfe536, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dfe53b, SUC_lexicon_tags_7 },
    { 0x809fe53b, SUC_lexicon_tags_83 },
    { 0x899fe53d, SUC_lexicon_tags_9 },
    { 0x629fe53e, SUC_lexicon_tags_25 },
    { 0xfddfe53d, SUC_lexicon_tags_47 },
    { 0xc67fe540, SUC_lexicon_tags_27 },
    { 0xf75fe541, SUC_lexicon_tags_11 },
    { 0x351fe542, SUC_lexicon_tags_11 },
    { 0x1d3fe543, SUC_lexicon_tags_9 },
    { 0xd61fe542, SUC_lexicon_tags_11 },
    { 0x13dfe53e, SUC_lexicon_tags_1 },
    { 0x8e1fe545, SUC_lexicon_tags_19 },
    { 0x9fbfe53f, SUC_lexicon_tags_29 },
    { 0x885fe548, SUC_lexicon_tags_33 },
    { 0x86dfe548, SUC_lexicon_tags_47 },
    { 0x6dfe54a, SUC_lexicon_tags_25 },
    { 0xe63fe549, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x51ffe54d, SUC_lexicon_tags_25 },
    { 0xfdffe54e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2bfe553, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x8d1fe555, SUC_lexicon_tags_15 },
    { 0xa1dfe555, SUC_lexicon_tags_14 },
    { 0xa3ffe557, SUC_lexicon_tags_17 },
    { 0x593fe557, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7ddfe55c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbd3fe55f, SUC_lexicon_tags_2 },
    { 0xf1dfe55f, SUC_lexicon_tags_31 },
    { 0x883fe560, SUC_lexicon_tags_7 },
    { 0x7cffe55f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x797fe565, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa95fe567, SUC_lexicon_tags_55 },
    { 0xb25fe567, SUC_lexicon_tags_14 },
    { 0xb61fe569, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x9d3fe56b, SUC_lexicon_tags_49 },
    { 0x825fe56c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x527fe571, SUC_lexicon_tags_112 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb9ffe575, SUC_lexicon_tags_11 },
    { 0xefdfe575, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xd7fe578, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x551fe57d, SUC_lexicon_tags_18 },
    { 0x6e1fe57e, SUC_lexicon_tags_23 },
    { 0x137fe57e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x45ffe581, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x223fe583, SUC_lexicon_tags_2 },
    { 0xe71fe584, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc55fe586, SUC_lexicon_tags_11 },
    { 0x6dfe587, SUC_lexicon_tags_2 },
    { 0xe17fe588, SUC_lexicon_tags_17 },
    { 0x22bfe587, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0xcdbfe58b, SUC_lexicon_tags_15 },
    { 0xf8bfe58c, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2b7fe590, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5fe599, SUC_lexicon_tags_30 },
    { 0x8effe59a, SUC_lexicon_tags_1 },
    { 0xceffe59b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9ffe59d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2fffe59f, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcfbfe5a3, SUC_lexicon_tags_25 },
    { 0x433fe5a3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x60bfe5a6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x239fe5a8, SUC_lexicon_tags_30 },
    { 0xfd7fe5a8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x56bfe5b3, SUC_lexicon_tags_43 },
    { 0x543fe5b4, SUC_lexicon_tags_6 },
    { 0x9ffe5b5, SUC_lexicon_tags_11 },
    { 0x1c5fe5b3, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf3ffe5ba, SUC_lexicon_tags_19 },
    { 0x785fe5bb, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x20dfe5be, SUC_lexicon_tags_48 },
    { 0xfd1fe5bf, SUC_lexicon_tags_23 },
    { 0x389fe5c0, SUC_lexicon_tags_25 },
    { 0xef9fe5c0, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x793fe5c3, SUC_lexicon_tags_9 },
    { 0xdb3fe5c4, SUC_lexicon_tags_37 },
    { 0x26ffe5c4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbfe5c9, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30dfe5cd, SUC_lexicon_tags_2 },
    { 0xaa9fe5ce, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x30dfe5d1, SUC_lexicon_tags_46 },
    { 0x21bfe5d2, SUC_lexicon_tags_2 },
    { 0xd9fe5d3, SUC_lexicon_tags_48 },
    { 0x8a7fe5d3, SUC_lexicon_tags_0 },
    { 0xa93fe5d5, SUC_lexicon_tags_15 },
    { 0x351fe5d5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a9fe5db, SUC_lexicon_tags_11 },
    { 0x6b9fe5dc, SUC_lexicon_tags_49 },
    { 0xd39fe5db, SUC_lexicon_tags_37 },
    { 0xa79fe5de, SUC_lexicon_tags_8 },
    { 0x701fe5df, SUC_lexicon_tags_31 },
    { 0x21dfe5db, SUC_lexicon_tags_26 },
    { 0x9c9fe5db, SUC_lexicon_tags_34 },
    { 0x33bfe5dd, SUC_lexicon_tags_21 },
    { 0x9dfe5df, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0x677fe5e6, SUC_lexicon_tags_32 },
    { 0x943fe5e6, SUC_lexicon_tags_37 },
    { 0xd4ffe5e6, SUC_lexicon_tags_15 },
    { 0xdc3fe5e8, SUC_lexicon_tags_1 },
    { 0x5d9fe5e6, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xe23fe5ed, SUC_lexicon_tags_31 },
    { 0x73dfe5ed, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7fe5f1, SUC_lexicon_tags_25 },
    { 0x9fe5f2, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa07fe5f4, SUC_lexicon_tags_25 },
    { 0xd6dfe5f5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9fe5f8, SUC_lexicon_tags_45 },
    { 0xcd3fe5f9, SUC_lexicon_tags_37 },
    { 0xacbfe5f9, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9fe5fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x743fe604, SUC_lexicon_tags_37 },
    { 0xfa9fe605, SUC_lexicon_tags_28 },
    { 0x51dfe605, SUC_lexicon_tags_17 },
    { 0x74dfe607, SUC_lexicon_tags_9 },
    { 0x719fe608, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x9e3fe60a, SUC_lexicon_tags_15 },
    { 0xa2dfe60a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf3fe60e, SUC_lexicon_tags_17 },
    { 0x3dffe60f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bfe613, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8abfe617, SUC_lexicon_tags_63 },
    { 0x1d1fe618, SUC_lexicon_tags_130 },
    { 0x313fe619, SUC_lexicon_tags_30 },
    { 0x9e1fe619, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa7fe622, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bfe625, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf55fe628, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x713fe62c, SUC_lexicon_tags_17 },
    { 0xea1fe62d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4dffe631, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3dfe633, SUC_lexicon_tags_7 },
    { 0xdb5fe634, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x669fe637, SUC_lexicon_tags_35 },
    { 0x9a9fe638, SUC_lexicon_tags_60 },
    { 0xd69fe639, SUC_lexicon_tags_113 },
    { 0x42ffe638, SUC_lexicon_tags_7 },
    { 0x563fe63b, SUC_lexicon_tags_46 },
    { 0x873fe63b, SUC_lexicon_tags_37 },
    { 0x271fe63a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xa41fe640, SUC_lexicon_tags_7 },
    { 0xc7dfe641, SUC_lexicon_tags_37 },
    { 0xc17fe640, SUC_lexicon_tags_1 },
    { 0x623fe643, SUC_lexicon_tags_25 },
    { 0xa69fe644, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbffe650, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477fe654, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x409fe657, SUC_lexicon_tags_47 },
    { 0xb19fe658, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7fe65b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xf01fe65e, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x859fe660, SUC_lexicon_tags_37 },
    { 0x8fdfe660, SUC_lexicon_tags_17 },
    { 0xa6bfe662, SUC_lexicon_tags_47 },
    { 0xa4ffe660, SUC_lexicon_tags_1 },
    { 0x5bdfe662, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xfbbfe666, SUC_lexicon_tags_29 },
    { 0x3d7fe666, SUC_lexicon_tags_14 },
    { 0xb93fe668, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x4dbfe66a, SUC_lexicon_tags_1 },
    { 0x5f7fe66b, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebdfe670, SUC_lexicon_tags_17 },
    { 0x29dfe671, SUC_lexicon_tags_8 },
    { 0x7f7fe672, SUC_lexicon_tags_141 },
    { 0x4d5fe673, SUC_lexicon_tags_45 },
    { 0x623fe672, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8cbfe678, SUC_lexicon_tags_487 },
    { 0x153fe678, SUC_lexicon_tags_63 },
    { 0xd5dfe679, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcffe67e, SUC_lexicon_tags_9 },
    { 0x6fbfe67f, SUC_lexicon_tags_29 },
    { 0x585fe680, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1fe685, SUC_lexicon_tags_17 },
    { 0xb4bfe686, SUC_lexicon_tags_35 },
    { 0xb9dfe685, SUC_lexicon_tags_0 },
    { 0x693fe688, SUC_lexicon_tags_48 },
    { 0xd69fe688, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xdbdfe68b, SUC_lexicon_tags_32 },
    { 0x961fe68b, SUC_lexicon_tags_1 },
    { 0xfa9fe68b, SUC_lexicon_tags_47 },
    { 0xd13fe68e, SUC_lexicon_tags_25 },
    { 0xfe3fe68d, SUC_lexicon_tags_11 },
    { 0xb4dfe68c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3bdfe692, SUC_lexicon_tags_25 },
    { 0xbbfe692, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x393fe695, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf35fe6a1, SUC_lexicon_tags_7 },
    { 0xcfdfe6a1, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xec5fe6a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc03fe6aa, SUC_lexicon_tags_33 },
    { 0x361fe6ab, SUC_lexicon_tags_11 },
    { 0xcdfe6ac, SUC_lexicon_tags_4 },
    { 0x407fe6ab, SUC_lexicon_tags_23 },
    { 0xd5bfe6ae, SUC_lexicon_tags_25 },
    { 0x96ffe6ad, SUC_lexicon_tags_15 },
    { 0x48ffe6ac, SUC_lexicon_tags_60 },
    { 0xdc1fe6b1, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bfe6b4, SUC_lexicon_tags_3 },
    { 0x8affe6b5, SUC_lexicon_tags_47 },
    { 0x8e9fe6b5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bffe6ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x653fe6bd, SUC_lexicon_tags_12 },
    { 0xf9fe6be, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1effe6c6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb6dfe6ce, SUC_lexicon_tags_63 },
    { 0x621fe6ce, SUC_lexicon_tags_37 },
    { 0x66bfe6ce, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe5ffe6d2, SUC_lexicon_tags_27 },
    { 0xb97fe6d3, SUC_lexicon_tags_14 },
    { 0x447fe6d3, SUC_lexicon_tags_43 },
    { 0x67ffe6d5, SUC_lexicon_tags_25 },
    { 0xc0ffe6d6, SUC_lexicon_tags_37 },
    { 0x4bbfe6d7, SUC_lexicon_tags_11 },
    { 0x76ffe6d8, SUC_lexicon_tags_33 },
    { 0xa97fe6d6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xa35fe6db, SUC_lexicon_tags_14 },
    { 0xb39fe6db, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x98bfe6e0, SUC_lexicon_tags_15 },
    { 0x2fdfe6e0, SUC_lexicon_tags_47 },
    { 0x13fe6e2, SUC_lexicon_tags_19 },
    { 0x6cdfe6e2, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fe6e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbebfe6ee, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x35dfe6f0, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x14dfe6f2, SUC_lexicon_tags_23 },
    { 0xbe1fe6f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1fe6f6, SUC_lexicon_tags_364 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749fe6fa, SUC_lexicon_tags_38 },
    { 0xebffe6fb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xee3fe6fd, SUC_lexicon_tags_17 },
    { 0xf15fe6fe, SUC_lexicon_tags_23 },
    { 0xe39fe6ff, SUC_lexicon_tags_30 },
    { 0x409fe6ff, SUC_lexicon_tags_27 },
    { 0x8c7fe6fe, SUC_lexicon_tags_0 },
    { 0x35dfe702, SUC_lexicon_tags_2 },
    { 0x8d9fe6fd, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x399fe705, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd9ffe707, SUC_lexicon_tags_21 },
    { 0xb83fe708, SUC_lexicon_tags_4 },
    { 0xb9ffe709, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dfe70e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5a5fe715, SUC_lexicon_tags_25 },
    { 0x943fe715, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x73fe718, SUC_lexicon_tags_25 },
    { 0x905fe719, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce1fe71f, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x441fe726, SUC_lexicon_tags_1 },
    { 0x453fe727, SUC_lexicon_tags_37 },
    { 0x1b3fe727, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x945fe72b, SUC_lexicon_tags_28 },
    { 0xc7dfe72b, SUC_lexicon_tags_15 },
    { 0xbe3fe72d, SUC_lexicon_tags_15 },
    { 0xb85fe72d, SUC_lexicon_tags_30 },
    { 0x2f3fe72f, SUC_lexicon_tags_37 },
    { 0x2f7fe730, SUC_lexicon_tags_87 },
    { 0x45bfe72e, SUC_lexicon_tags_71 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdb9fe735, SUC_lexicon_tags_47 },
    { 0x91bfe735, SUC_lexicon_tags_38 },
    { 0x375fe737, SUC_lexicon_tags_33 },
    { 0x207fe736, SUC_lexicon_tags_2 },
    { 0x98bfe737, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xd59fe73b, SUC_lexicon_tags_11 },
    { 0x70ffe73c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x17bfe73e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847fe746, SUC_lexicon_tags_2 },
    { 0x825fe747, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd09fe74d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc2ffe756, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0xffffe759, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5fe75f, SUC_lexicon_tags_7 },
    { 0x8bbfe760, SUC_lexicon_tags_1 },
    { 0x4bfe761, SUC_lexicon_tags_37 },
    { 0x337fe75f, SUC_lexicon_tags_23 },
    { 0x5fe762, SUC_lexicon_tags_48 },
    { 0x61bfe764, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd33fe767, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811fe76b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0x287fe76e, SUC_lexicon_tags_75 },
    { 0xc11fe76f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xd99fe771, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275fe778, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x2affe77a, SUC_lexicon_tags_49 },
    { 0x6edfe77a, SUC_lexicon_tags_1 },
    { 0x2f9fe77c, SUC_lexicon_tags_34 },
    { 0x175fe77d, SUC_lexicon_tags_12 },
    { 0x4b9fe77e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x9c1fe780, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8dfe785, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xe07fe787, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e5fe78d, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa13fe795, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d5fe79a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xddbfe79d, SUC_lexicon_tags_14 },
    { 0xd45fe79d, SUC_lexicon_tags_7 },
    { 0x159fe79e, SUC_lexicon_tags_37 },
    { 0xcdfe7a0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x44dfe7a2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x21ffe7a5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x317fe7a8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x3c9fe7ab, SUC_lexicon_tags_12 },
    { 0x4fbfe7ac, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x23dfe7ae, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcdbfe7b0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x455fe7b2, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65fe7b5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x537fe7b7, SUC_lexicon_tags_8 },
    { 0x53dfe7b7, SUC_lexicon_tags_19 },
    { 0xefffe7b7, SUC_lexicon_tags_11 },
    { 0x81ffe7b9, SUC_lexicon_tags_1 },
    { 0x103fe7bb, SUC_lexicon_tags_87 },
    { 0x101fe7bc, SUC_lexicon_tags_7 },
    { 0x6adfe7b9, SUC_lexicon_tags_65 },
    { 0x1bfe7be, SUC_lexicon_tags_10 },
    { 0x1e9fe7bf, SUC_lexicon_tags_15 },
    { 0x751fe7c0, SUC_lexicon_tags_17 },
    { 0x48dfe7c1, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5fe7c7, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x8c9fe7c9, SUC_lexicon_tags_4 },
    { 0x61bfe7c9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bfe7cf, SUC_lexicon_tags_12 },
    { 0x58ffe7d0, SUC_lexicon_tags_11 },
    { 0x103fe7d1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1ebfe7d3, SUC_lexicon_tags_329 },
    { 0xc6ffe7d3, SUC_lexicon_tags_33 },
    { 0x67bfe7d4, SUC_lexicon_tags_25 },
    { 0x1b3fe7d3, SUC_lexicon_tags_46 },
    { 0x101fe7d6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x913fe7d9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x881fe7dc, SUC_lexicon_tags_17 },
    { 0x83fe7dd, SUC_lexicon_tags_100 },
    { 0x565fe7de, SUC_lexicon_tags_12 },
    { 0xbb9fe7df, SUC_lexicon_tags_17 },
    { 0xf33fe7df, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0x205fe7e2, SUC_lexicon_tags_65 },
    { 0xb8dfe7e3, SUC_lexicon_tags_26 },
    { 0x68ffe7e4, SUC_lexicon_tags_27 },
    { 0x4ebfe7e3, SUC_lexicon_tags_17 },
    { 0x885fe7e6, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x55ffe7e8, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7bfe7ed, SUC_lexicon_tags_12 },
    { 0x1e9fe7ee, SUC_lexicon_tags_35 },
    { 0xaa7fe7ee, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x721fe7f1, SUC_lexicon_tags_37 },
    { 0xbfe7f2, SUC_lexicon_tags_47 },
    { 0xddfe7f3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7dfe7f6, SUC_lexicon_tags_15 },
    { 0x77fe7f7, SUC_lexicon_tags_12 },
    { 0x66bfe7f8, SUC_lexicon_tags_87 },
    { 0xe2dfe7f8, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6dfe7fd, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x197fe800, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8effe807, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xfc3fe809, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xeddfe80c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xfa9fe80f, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x399fe815, SUC_lexicon_tags_1 },
    { 0xe1fe816, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46dfe81f, SUC_lexicon_tags_6 },
    { 0x9effe820, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x587fe823, SUC_lexicon_tags_920 },
    { 0xc23fe824, SUC_lexicon_tags_7 },
    { 0x79dfe825, SUC_lexicon_tags_9 },
    { 0xa6dfe825, SUC_lexicon_tags_17 },
    { 0x15ffe826, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x92dfe82c, SUC_lexicon_tags_1 },
    { 0xc4dfe82d, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xd49fe830, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x373fe832, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x661fe834, SUC_lexicon_tags_2 },
    { 0x1c9fe835, SUC_lexicon_tags_2 },
    { 0x8f5fe836, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b1fe839, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f7fe83d, SUC_lexicon_tags_63 },
    { 0x34bfe83e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xde5fe841, SUC_lexicon_tags_0 },
    { 0x601fe842, SUC_lexicon_tags_1 },
    { 0xbdfe843, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedfe848, SUC_lexicon_tags_229 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x80bfe84d, SUC_lexicon_tags_2 },
    { 0xf1bfe84e, SUC_lexicon_tags_8 },
    { 0x677fe84e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e9fe854, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x183fe857, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa29fe85f, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0x563fe862, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9bffe86c, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x9a1fe86f, SUC_lexicon_tags_26 },
    { 0x623fe870, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x86ffe873, SUC_lexicon_tags_14 },
    { 0xbadfe874, SUC_lexicon_tags_12 },
    { 0xfa3fe874, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa21fe880, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0x689fe883, SUC_lexicon_tags_37 },
    { 0xf9ffe884, SUC_lexicon_tags_15 },
    { 0xc0ffe885, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xc55fe887, SUC_lexicon_tags_12 },
    { 0xd0bfe887, SUC_lexicon_tags_38 },
    { 0x27fe889, SUC_lexicon_tags_8 },
    { 0xa61fe88a, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9fe893, SUC_lexicon_tags_55 },
    { 0x4ffe894, SUC_lexicon_tags_7 },
    { 0x2bbfe895, SUC_lexicon_tags_11 },
    { 0x439fe896, SUC_lexicon_tags_25 },
    { 0x15ffe897, SUC_lexicon_tags_40 },
    { 0x4cffe897, SUC_lexicon_tags_24 },
    { 0, NULL },
    { 0, NULL },
    { 0x8edfe89b, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa99fe89f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61fe8a3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x207fe8a5, SUC_lexicon_tags_39 },
    { 0xa85fe8a6, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xcf7fe8a8, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x4a5fe8aa, SUC_lexicon_tags_47 },
    { 0x865fe8ab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf49fe8ad, SUC_lexicon_tags_2 },
    { 0x48bfe8ae, SUC_lexicon_tags_8 },
    { 0x593fe8af, SUC_lexicon_tags_77 },
    { 0x54ffe8b0, SUC_lexicon_tags_11 },
    { 0x809fe8ae, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x6f9fe8b3, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbbfe8b8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf27fe8bc, SUC_lexicon_tags_2 },
    { 0x287fe8bd, SUC_lexicon_tags_11 },
    { 0x293fe8bc, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bfe8c1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x32ffe8c4, SUC_lexicon_tags_87 },
    { 0x311fe8c4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaedfe8cb, SUC_lexicon_tags_33 },
    { 0x549fe8cc, SUC_lexicon_tags_301 },
    { 0x4dfe8cb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x36dfe8cf, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x753fe8d2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511fe8de, SUC_lexicon_tags_28 },
    { 0xa55fe8de, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x6dffe8e1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x71bfe8e5, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x2b1fe8e7, SUC_lexicon_tags_14 },
    { 0x1ebfe8e8, SUC_lexicon_tags_17 },
    { 0x87fe8e9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3fe8f1, SUC_lexicon_tags_31 },
    { 0x59ffe8f1, SUC_lexicon_tags_12 },
    { 0x159fe8f1, SUC_lexicon_tags_131 },
    { 0, NULL },
    { 0xd01fe8f5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xc77fe8f7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x8e7fe8f9, SUC_lexicon_tags_31 },
    { 0xf9ffe8fa, SUC_lexicon_tags_23 },
    { 0x3d3fe8fa, SUC_lexicon_tags_17 },
    { 0x4dffe8fc, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbfe8ff, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x16dfe904, SUC_lexicon_tags_100 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdadfe909, SUC_lexicon_tags_29 },
    { 0x523fe90a, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3e5fe90f, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xa51fe911, SUC_lexicon_tags_47 },
    { 0xea5fe912, SUC_lexicon_tags_11 },
    { 0x173fe911, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x665fe915, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x45dfe918, SUC_lexicon_tags_17 },
    { 0x4e3fe919, SUC_lexicon_tags_437 },
    { 0, NULL },
    { 0, NULL },
    { 0xdabfe91c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa4ffe91e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18bfe922, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4e3fe928, SUC_lexicon_tags_16 },
    { 0x5bdfe929, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x171fe92f, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb2bfe936, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x55ffe938, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x68bfe93b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd25fe93d, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xc37fe93f, SUC_lexicon_tags_14 },
    { 0xb0ffe940, SUC_lexicon_tags_65 },
    { 0xbc5fe941, SUC_lexicon_tags_45 },
    { 0x1fdfe940, SUC_lexicon_tags_182 },
    { 0x10bfe943, SUC_lexicon_tags_11 },
    { 0x765fe941, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x4c3fe947, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x3b9fe94a, SUC_lexicon_tags_1 },
    { 0x9cdfe94a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x15ffe94e, SUC_lexicon_tags_12 },
    { 0x161fe94e, SUC_lexicon_tags_68 },
    { 0x67fe950, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfebfe952, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xe93fe955, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x71bfe958, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x91bfe95b, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dfe95f, SUC_lexicon_tags_68 },
    { 0x455fe960, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x125fe962, SUC_lexicon_tags_12 },
    { 0x89dfe962, SUC_lexicon_tags_1 },
    { 0x36bfe964, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0xdcdfe967, SUC_lexicon_tags_2 },
    { 0x9dfe968, SUC_lexicon_tags_15 },
    { 0x805fe969, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73fe96d, SUC_lexicon_tags_30 },
    { 0xfbdfe96e, SUC_lexicon_tags_31 },
    { 0xe79fe96f, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x91fe975, SUC_lexicon_tags_2 },
    { 0x5f9fe976, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xcdbfe978, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53bfe97c, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x2ddfe97e, SUC_lexicon_tags_1 },
    { 0xafffe97f, SUC_lexicon_tags_477 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x315fe985, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x1cffe987, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x959fe989, SUC_lexicon_tags_38 },
    { 0x505fe989, SUC_lexicon_tags_33 },
    { 0x487fe98b, SUC_lexicon_tags_47 },
    { 0x295fe98c, SUC_lexicon_tags_1 },
    { 0xafdfe98d, SUC_lexicon_tags_25 },
    { 0xad3fe98b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x5a5fe990, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xcddfe993, SUC_lexicon_tags_19 },
    { 0x4effe993, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x873fe997, SUC_lexicon_tags_1 },
    { 0x35ffe998, SUC_lexicon_tags_17 },
    { 0xfedfe998, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xb71fe99b, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x37fe99e, SUC_lexicon_tags_21 },
    { 0xac5fe99f, SUC_lexicon_tags_14 },
    { 0xe7bfe99e, SUC_lexicon_tags_4 },
    { 0xe6ffe9a1, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x927fe9a3, SUC_lexicon_tags_15 },
    { 0xc3bfe9a3, SUC_lexicon_tags_12 },
    { 0x4d3fe9a4, SUC_lexicon_tags_26 },
    { 0xfaffe9a5, SUC_lexicon_tags_3 },
    { 0x405fe9a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16bfe9a9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc73fe9ad, SUC_lexicon_tags_37 },
    { 0x49ffe9ad, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7fe9b3, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xd9bfe9b5, SUC_lexicon_tags_23 },
    { 0xdf9fe9b5, SUC_lexicon_tags_39 },
    { 0xc25fe9b5, SUC_lexicon_tags_62 },
    { 0xdfe9b6, SUC_lexicon_tags_12 },
    { 0xa03fe9b5, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe8bfe9bb, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa43fe9bf, SUC_lexicon_tags_11 },
    { 0x92bfe9c0, SUC_lexicon_tags_2 },
    { 0x76bfe9c1, SUC_lexicon_tags_25 },
    { 0xbdbfe9c1, SUC_lexicon_tags_25 },
    { 0xb1dfe9c2, SUC_lexicon_tags_11 },
    { 0x639fe9c1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc89fe9c6, SUC_lexicon_tags_17 },
    { 0xeabfe9c6, SUC_lexicon_tags_12 },
    { 0x911fe9c8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1fe9cb, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0xf5dfe9cd, SUC_lexicon_tags_192 },
    { 0, NULL },
    { 0x2f1fe9cf, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x6e3fe9d2, SUC_lexicon_tags_17 },
    { 0x561fe9d2, SUC_lexicon_tags_68 },
    { 0xf89fe9d4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x167fe9d6, SUC_lexicon_tags_23 },
    { 0x615fe9d7, SUC_lexicon_tags_25 },
    { 0x1f3fe9d8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x341fe9dd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x1bffe9df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xf15fe9e1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8dfe9e3, SUC_lexicon_tags_7 },
    { 0xa35fe9e3, SUC_lexicon_tags_53 },
    { 0xba9fe9e3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dfe9e8, SUC_lexicon_tags_9 },
    { 0x9ebfe9e9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x5ebfe9eb, SUC_lexicon_tags_39 },
    { 0xeb5fe9ec, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0x15bfe9ef, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x287fe9f7, SUC_lexicon_tags_0 },
    { 0x85bfe9f7, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x30ffe9fa, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xd89fe9fc, SUC_lexicon_tags_17 },
    { 0x731fe9fd, SUC_lexicon_tags_27 },
    { 0xa01fe9fd, SUC_lexicon_tags_7 },
    { 0x2abfe9ff, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xdf3fea01, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x29bfea04, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x52bfea06, SUC_lexicon_tags_15 },
    { 0x90ffea07, SUC_lexicon_tags_1 },
    { 0x8d5fea08, SUC_lexicon_tags_37 },
    { 0x4c5fea07, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9effea11, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8dfea15, SUC_lexicon_tags_1 },
    { 0xe5fea15, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2d1fea1a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bfea1e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xeb1fea20, SUC_lexicon_tags_15 },
    { 0x1a9fea20, SUC_lexicon_tags_46 },
    { 0xc3ffea20, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x511fea27, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6affea2b, SUC_lexicon_tags_26 },
    { 0xc35fea2c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x199fea2e, SUC_lexicon_tags_17 },
    { 0xb85fea2e, SUC_lexicon_tags_1 },
    { 0x573fea2e, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x73bfea32, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b5fea39, SUC_lexicon_tags_38 },
    { 0x25fea3a, SUC_lexicon_tags_6 },
    { 0xaedfea3b, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0x30bfea3d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc31fea42, SUC_lexicon_tags_1 },
    { 0x775fea42, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653fea48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x503fea4a, SUC_lexicon_tags_9 },
    { 0x153fea4b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad3fea4f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe93fea54, SUC_lexicon_tags_12 },
    { 0xc71fea55, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0x927fea58, SUC_lexicon_tags_12 },
    { 0x7ffea59, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13ffea5d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b7fea61, SUC_lexicon_tags_34 },
    { 0xfadfea61, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x153fea65, SUC_lexicon_tags_43 },
    { 0x107fea66, SUC_lexicon_tags_47 },
    { 0x475fea65, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x44ffea6e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0x283fea70, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x4dbfea72, SUC_lexicon_tags_7 },
    { 0x3f7fea73, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb43fea77, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa9fea7a, SUC_lexicon_tags_25 },
    { 0xbe3fea7b, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x431fea82, SUC_lexicon_tags_2 },
    { 0xf85fea83, SUC_lexicon_tags_17 },
    { 0x77ffea84, SUC_lexicon_tags_7 },
    { 0xacdfea85, SUC_lexicon_tags_1 },
    { 0xd25fea85, SUC_lexicon_tags_47 },
    { 0xe71fea85, SUC_lexicon_tags_7 },
    { 0xf11fea88, SUC_lexicon_tags_3 },
    { 0x87ffea89, SUC_lexicon_tags_60 },
    { 0xf1bfea89, SUC_lexicon_tags_1 },
    { 0x23fea82, SUC_lexicon_tags_1 },
    { 0x4e7fea8c, SUC_lexicon_tags_15 },
    { 0x8e3fea85, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbb3fea8f, SUC_lexicon_tags_47 },
    { 0x76ffea90, SUC_lexicon_tags_46 },
    { 0xab7fea91, SUC_lexicon_tags_49 },
    { 0x6e5fea92, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x543fea94, SUC_lexicon_tags_7 },
    { 0x1bfea94, SUC_lexicon_tags_11 },
    { 0xf9bfea96, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25bfea98, SUC_lexicon_tags_19 },
    { 0xb5ffea98, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x6dffea9c, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x243fea9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a3feaa3, SUC_lexicon_tags_82 },
    { 0xfe7feaa4, SUC_lexicon_tags_11 },
    { 0xc89feaa5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xc85feaa7, SUC_lexicon_tags_15 },
    { 0xeb9feaa8, SUC_lexicon_tags_46 },
    { 0xa49feaa9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c5feaaf, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x369feab2, SUC_lexicon_tags_56 },
    { 0x229feab3, SUC_lexicon_tags_10 },
    { 0x4fffeab3, SUC_lexicon_tags_66 },
    { 0x2c9feab4, SUC_lexicon_tags_21 },
    { 0xf7bfeab6, SUC_lexicon_tags_38 },
    { 0x6ddfeab6, SUC_lexicon_tags_37 },
    { 0xc49feab2, SUC_lexicon_tags_4 },
    { 0xacdfeab9, SUC_lexicon_tags_6 },
    { 0xe8ffeaba, SUC_lexicon_tags_68 },
    { 0xf89feabb, SUC_lexicon_tags_11 },
    { 0x931feab6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x26ffeabe, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xda1feac0, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xe01feac2, SUC_lexicon_tags_0 },
    { 0x5b1feac2, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd2ffeac8, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcddfeaca, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf75fead3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x3ddfead6, SUC_lexicon_tags_17 },
    { 0xf13fead6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9feae1, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0xe53feae4, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8affeaea, SUC_lexicon_tags_1644 },
    { 0, NULL },
    { 0x737feaec, SUC_lexicon_tags_47 },
    { 0xd63feaed, SUC_lexicon_tags_26 },
    { 0x525feaee, SUC_lexicon_tags_88 },
    { 0x8a1feaee, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xffeaf1, SUC_lexicon_tags_159 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc23feaf6, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x77ffeaf9, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xedbfeafb, SUC_lexicon_tags_7 },
    { 0x74dfeafc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc1dfeaff, SUC_lexicon_tags_37 },
    { 0x515feb00, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x401feb04, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xe45feb06, SUC_lexicon_tags_9 },
    { 0xf2dfeb07, SUC_lexicon_tags_47 },
    { 0xaf9feb06, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd17feb0d, SUC_lexicon_tags_37 },
    { 0xed1feb0e, SUC_lexicon_tags_7 },
    { 0x639feb0f, SUC_lexicon_tags_38 },
    { 0xb95feb10, SUC_lexicon_tags_38 },
    { 0xd2dfeb0f, SUC_lexicon_tags_23 },
    { 0xecdfeb12, SUC_lexicon_tags_2 },
    { 0x997feb0e, SUC_lexicon_tags_156 },
    { 0x329feb0f, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x701feb17, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xda5feb1a, SUC_lexicon_tags_39 },
    { 0xcb5feb1a, SUC_lexicon_tags_28 },
    { 0xd21feb1b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dfeb27, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x4fbfeb29, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb5ffeb2e, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa27feb30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x7c7feb32, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x6e3feb34, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x7dfeb36, SUC_lexicon_tags_2 },
    { 0xa67feb37, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ffeb3a, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9b3feb40, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x515feb42, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xa07feb44, SUC_lexicon_tags_37 },
    { 0xc3feb44, SUC_lexicon_tags_25 },
    { 0x80dfeb44, SUC_lexicon_tags_19 },
    { 0x3cffeb47, SUC_lexicon_tags_179 },
    { 0, NULL },
    { 0, NULL },
    { 0x37dfeb4a, SUC_lexicon_tags_6 },
    { 0xb5bfeb4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x52ffeb4f, SUC_lexicon_tags_17 },
    { 0x6a3feb4f, SUC_lexicon_tags_107 },
    { 0, NULL },
    { 0, NULL },
    { 0x851feb53, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97dfeb55, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bdfeb5b, SUC_lexicon_tags_63 },
    { 0x501feb5c, SUC_lexicon_tags_18 },
    { 0x9a5feb5d, SUC_lexicon_tags_66 },
    { 0x18bfeb5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xda7feb61, SUC_lexicon_tags_26 },
    { 0xa7ffeb61, SUC_lexicon_tags_80 },
    { 0xfbdfeb62, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa31feb6a, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xa95feb6d, SUC_lexicon_tags_10 },
    { 0x9dfeb6e, SUC_lexicon_tags_32 },
    { 0xc97feb6f, SUC_lexicon_tags_68 },
    { 0x4f7feb6f, SUC_lexicon_tags_7 },
    { 0x2c9feb71, SUC_lexicon_tags_11 },
    { 0xaabfeb6d, SUC_lexicon_tags_32 },
    { 0xd9dfeb73, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x25ffeb76, SUC_lexicon_tags_7 },
    { 0xff7feb77, SUC_lexicon_tags_79 },
    { 0x50dfeb78, SUC_lexicon_tags_1 },
    { 0xecdfeb79, SUC_lexicon_tags_11 },
    { 0x559feb77, SUC_lexicon_tags_19 },
    { 0xef7feb7b, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x105feb80, SUC_lexicon_tags_12 },
    { 0x7c1feb81, SUC_lexicon_tags_67 },
    { 0xfeffeb82, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc93feb85, SUC_lexicon_tags_2 },
    { 0xa43feb86, SUC_lexicon_tags_12 },
    { 0xea1feb87, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x817feb89, SUC_lexicon_tags_38 },
    { 0x2f7feb8a, SUC_lexicon_tags_14 },
    { 0x745feb89, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fdfeb8e, SUC_lexicon_tags_48 },
    { 0x9fdfeb8e, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x1ddfeb91, SUC_lexicon_tags_15 },
    { 0x9dffeb91, SUC_lexicon_tags_37 },
    { 0xcaffeb92, SUC_lexicon_tags_39 },
    { 0x80bfeb94, SUC_lexicon_tags_27 },
    { 0x275feb93, SUC_lexicon_tags_17 },
    { 0x88ffeb92, SUC_lexicon_tags_17 },
    { 0x87bfeb97, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd5feb9d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdfeba3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dbfeba6, SUC_lexicon_tags_15 },
    { 0xa81feba6, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x759febaa, SUC_lexicon_tags_17 },
    { 0xb8dfebab, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbfebaf, SUC_lexicon_tags_12 },
    { 0x3e5febb0, SUC_lexicon_tags_2 },
    { 0xfa9febaf, SUC_lexicon_tags_170 },
    { 0, NULL },
    { 0, NULL },
    { 0xf7bfebb4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xa9febb6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685febba, SUC_lexicon_tags_7 },
    { 0x487febba, SUC_lexicon_tags_19 },
    { 0xdfebbb, SUC_lexicon_tags_17 },
    { 0xae9febbb, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfebc0, SUC_lexicon_tags_11 },
    { 0x5ddfebc1, SUC_lexicon_tags_17 },
    { 0xca1febc2, SUC_lexicon_tags_47 },
    { 0x2e7febc2, SUC_lexicon_tags_65 },
    { 0x849febc4, SUC_lexicon_tags_37 },
    { 0x42bfebc0, SUC_lexicon_tags_33 },
    { 0x92dfebc6, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab5febcf, SUC_lexicon_tags_7 },
    { 0x3f1febcf, SUC_lexicon_tags_23 },
    { 0x4f9febd1, SUC_lexicon_tags_91 },
    { 0x5abfebd2, SUC_lexicon_tags_7 },
    { 0xea3febd1, SUC_lexicon_tags_88 },
    { 0x651febd1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3e3febd6, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x539febda, SUC_lexicon_tags_174 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x993febde, SUC_lexicon_tags_25 },
    { 0xfe7febdf, SUC_lexicon_tags_37 },
    { 0x191febdf, SUC_lexicon_tags_31 },
    { 0x671febde, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x193febe3, SUC_lexicon_tags_62 },
    { 0xcaffebe3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x283febe7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x9c5febe9, SUC_lexicon_tags_1 },
    { 0xba3febe9, SUC_lexicon_tags_11 },
    { 0xa43febe9, SUC_lexicon_tags_23 },
    { 0x3e5febec, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x8e7febee, SUC_lexicon_tags_15 },
    { 0x94ffebee, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbdfebfb, SUC_lexicon_tags_60 },
    { 0x92bfebfc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161fec04, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xb39fec06, SUC_lexicon_tags_27 },
    { 0x391fec07, SUC_lexicon_tags_15 },
    { 0xe3dfec08, SUC_lexicon_tags_7 },
    { 0x685fec06, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1fffec0d, SUC_lexicon_tags_62 },
    { 0x88ffec0e, SUC_lexicon_tags_33 },
    { 0x763fec0f, SUC_lexicon_tags_14 },
    { 0xc41fec0f, SUC_lexicon_tags_37 },
    { 0x74ffec11, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5fec14, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa63fec1a, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x5adfec1c, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96ffec25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xb9ffec27, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0x73ffec29, SUC_lexicon_tags_37 },
    { 0x4dbfec29, SUC_lexicon_tags_15 },
    { 0x7bfec2b, SUC_lexicon_tags_301 },
    { 0xed7fec2c, SUC_lexicon_tags_15 },
    { 0x8e9fec2d, SUC_lexicon_tags_37 },
    { 0x9e9fec2d, SUC_lexicon_tags_15 },
    { 0xc3fec2e, SUC_lexicon_tags_8 },
    { 0x515fec30, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xd1fec33, SUC_lexicon_tags_37 },
    { 0xe27fec34, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa1fec39, SUC_lexicon_tags_47 },
    { 0x29ffec3a, SUC_lexicon_tags_0 },
    { 0x6adfec39, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2fdfec42, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0x215fec45, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x3dbfec48, SUC_lexicon_tags_47 },
    { 0xaa9fec49, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4c7fec4b, SUC_lexicon_tags_19 },
    { 0x75fec4c, SUC_lexicon_tags_87 },
    { 0xb59fec4d, SUC_lexicon_tags_4 },
    { 0xdadfec4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x95ffec51, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xecbfec56, SUC_lexicon_tags_4 },
    { 0x817fec56, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf3fec5a, SUC_lexicon_tags_7 },
    { 0x745fec5a, SUC_lexicon_tags_23 },
    { 0x8e3fec5a, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x589fec63, SUC_lexicon_tags_17 },
    { 0xe11fec63, SUC_lexicon_tags_7 },
    { 0xc33fec65, SUC_lexicon_tags_32 },
    { 0x6ffec66, SUC_lexicon_tags_19 },
    { 0xd5fec66, SUC_lexicon_tags_37 },
    { 0x3ffec65, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbfec6e, SUC_lexicon_tags_12 },
    { 0x96ffec6f, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xd49fec71, SUC_lexicon_tags_33 },
    { 0x983fec72, SUC_lexicon_tags_37 },
    { 0x2e9fec72, SUC_lexicon_tags_33 },
    { 0x20dfec71, SUC_lexicon_tags_21 },
    { 0x9adfec75, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x841fec78, SUC_lexicon_tags_6 },
    { 0xebffec79, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xd07fec7c, SUC_lexicon_tags_27 },
    { 0x519fec7d, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0xd05fec80, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x31dfec83, SUC_lexicon_tags_31 },
    { 0x159fec84, SUC_lexicon_tags_11 },
    { 0xc8dfec84, SUC_lexicon_tags_15 },
    { 0xa07fec86, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x3c5fec88, SUC_lexicon_tags_11 },
    { 0x5c3fec89, SUC_lexicon_tags_43 },
    { 0x5d9fec89, SUC_lexicon_tags_11 },
    { 0x7c9fec8b, SUC_lexicon_tags_336 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fec93, SUC_lexicon_tags_37 },
    { 0x865fec94, SUC_lexicon_tags_63 },
    { 0xab1fec93, SUC_lexicon_tags_14 },
    { 0x91ffec96, SUC_lexicon_tags_12 },
    { 0xf4bfec97, SUC_lexicon_tags_29 },
    { 0x887fec98, SUC_lexicon_tags_29 },
    { 0x66bfec99, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0x65dfec9b, SUC_lexicon_tags_54 },
    { 0, NULL },
    { 0x135fec9d, SUC_lexicon_tags_82 },
    { 0xebbfec9e, SUC_lexicon_tags_17 },
    { 0xec3fec9d, SUC_lexicon_tags_25 },
    { 0x619feca0, SUC_lexicon_tags_43 },
    { 0x38dfec9d, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x321feca4, SUC_lexicon_tags_44 },
    { 0xf11feca4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x1dbfeca7, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xf47feca9, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xd3dfecab, SUC_lexicon_tags_12 },
    { 0xcb7fecac, SUC_lexicon_tags_17 },
    { 0xe01fecad, SUC_lexicon_tags_29 },
    { 0xa59fecab, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3f9fecb4, SUC_lexicon_tags_25 },
    { 0xb31fecb5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x395fecb8, SUC_lexicon_tags_45 },
    { 0x30ffecb9, SUC_lexicon_tags_25 },
    { 0x6c1fecb8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xf63fecbd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0x503fecc0, SUC_lexicon_tags_32 },
    { 0xec3fecc1, SUC_lexicon_tags_37 },
    { 0xefdfecc1, SUC_lexicon_tags_15 },
    { 0xbb9fecc3, SUC_lexicon_tags_23 },
    { 0x247fecc4, SUC_lexicon_tags_48 },
    { 0x1bbfecc3, SUC_lexicon_tags_14 },
    { 0x7b1fecc0, SUC_lexicon_tags_63 },
    { 0x46ffecc7, SUC_lexicon_tags_1 },
    { 0x4bffecc7, SUC_lexicon_tags_8 },
    { 0xa99fecc9, SUC_lexicon_tags_25 },
    { 0x54ffecca, SUC_lexicon_tags_36 },
    { 0xa95feccb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbfbfeccd, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x6dffeccf, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0xbb3fecd1, SUC_lexicon_tags_15 },
    { 0x99bfecd2, SUC_lexicon_tags_1 },
    { 0xb9bfecd3, SUC_lexicon_tags_15 },
    { 0xad9fecd4, SUC_lexicon_tags_34 },
    { 0x651fecd4, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xc35fecd7, SUC_lexicon_tags_15 },
    { 0x9c7fecd8, SUC_lexicon_tags_65 },
    { 0xc5bfecd7, SUC_lexicon_tags_15 },
    { 0xefffecd8, SUC_lexicon_tags_7 },
    { 0xb7dfecda, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b7fecde, SUC_lexicon_tags_101 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5fece2, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb09feceb, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9edfecee, SUC_lexicon_tags_23 },
    { 0x9dfecee, SUC_lexicon_tags_49 },
    { 0x5f3fecf0, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4a7fecf2, SUC_lexicon_tags_12 },
    { 0x7f7fecf3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xaf5fecf6, SUC_lexicon_tags_2 },
    { 0x3b9fecf6, SUC_lexicon_tags_15 },
    { 0xdbffecf6, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x83dfecfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x613fed00, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x363fed04, SUC_lexicon_tags_7 },
    { 0xaddfed05, SUC_lexicon_tags_27 },
    { 0x6ffed05, SUC_lexicon_tags_39 },
    { 0xc5bfed07, SUC_lexicon_tags_45 },
    { 0x61dfed08, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbabfed0b, SUC_lexicon_tags_45 },
    { 0xa99fed0b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe57fed0e, SUC_lexicon_tags_43 },
    { 0x60dfed0f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x811fed13, SUC_lexicon_tags_17 },
    { 0x6e5fed14, SUC_lexicon_tags_23 },
    { 0x5f3fed15, SUC_lexicon_tags_69 },
    { 0x487fed15, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0x491fed19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x257fed1b, SUC_lexicon_tags_15 },
    { 0x2a9fed1c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9ddfed1e, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7fed25, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xb6bfed28, SUC_lexicon_tags_15 },
    { 0x8ddfed29, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x37dfed2d, SUC_lexicon_tags_63 },
    { 0x6fbfed2e, SUC_lexicon_tags_19 },
    { 0x325fed2f, SUC_lexicon_tags_107 },
    { 0xf5bfed2f, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0xa65fed33, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc47fed3e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xd89fed41, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x74ffed45, SUC_lexicon_tags_7 },
    { 0x775fed46, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0xe8ffed49, SUC_lexicon_tags_2 },
    { 0x571fed4a, SUC_lexicon_tags_27 },
    { 0x21ffed4b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbcdfed4f, SUC_lexicon_tags_37 },
    { 0xf2dfed50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0x8f1fed53, SUC_lexicon_tags_2 },
    { 0x6dfed54, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9fed57, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb4dfed5f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa9bfed61, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xa2ffed63, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x86ffed67, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x31bfed6a, SUC_lexicon_tags_148 },
    { 0xe73fed6b, SUC_lexicon_tags_10 },
    { 0x227fed6c, SUC_lexicon_tags_11 },
    { 0x5bbfed6a, SUC_lexicon_tags_8 },
    { 0xdfffed6e, SUC_lexicon_tags_12 },
    { 0x17bfed6f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x465fed72, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x3bffed74, SUC_lexicon_tags_7 },
    { 0x795fed74, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ffed79, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x297fed7b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef3fed7f, SUC_lexicon_tags_15 },
    { 0xd6ffed80, SUC_lexicon_tags_3 },
    { 0x3ddfed81, SUC_lexicon_tags_25 },
    { 0x349fed80, SUC_lexicon_tags_28 },
    { 0xdfbfed83, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x555fed87, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb67fed8b, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbfed90, SUC_lexicon_tags_39 },
    { 0x2fbfed90, SUC_lexicon_tags_25 },
    { 0x2e5fed92, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xac3fed94, SUC_lexicon_tags_63 },
    { 0x57fed95, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x491fed97, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x16ffed99, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe37fed9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09fed9f, SUC_lexicon_tags_1 },
    { 0x273fed9f, SUC_lexicon_tags_60 },
    { 0xc75feda1, SUC_lexicon_tags_1 },
    { 0x2b7feda2, SUC_lexicon_tags_1 },
    { 0x15ffeda3, SUC_lexicon_tags_17 },
    { 0x2a7feda4, SUC_lexicon_tags_39 },
    { 0x9feda5, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc57feda9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957fedae, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0xf5bfedb0, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x255fedb2, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xb59fedb4, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd83fedb8, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0x853fedbb, SUC_lexicon_tags_17 },
    { 0x627fedbb, SUC_lexicon_tags_37 },
    { 0x69fedbb, SUC_lexicon_tags_35 },
    { 0xe77fedbe, SUC_lexicon_tags_25 },
    { 0xee9fedbe, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749fedc9, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ebfedce, SUC_lexicon_tags_25 },
    { 0xe17fedcf, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x87bfedd6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xda3feddb, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x14bfeddd, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x23ffeddf, SUC_lexicon_tags_12 },
    { 0x69bfeddf, SUC_lexicon_tags_63 },
    { 0x2ebfede1, SUC_lexicon_tags_153 },
    { 0x36ffede0, SUC_lexicon_tags_17 },
    { 0x40bfede0, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x84bfede9, SUC_lexicon_tags_23 },
    { 0x567fedea, SUC_lexicon_tags_92 },
    { 0xfeffedeb, SUC_lexicon_tags_13 },
    { 0x45fedec, SUC_lexicon_tags_74 },
    { 0x885fedec, SUC_lexicon_tags_37 },
    { 0x2e1fedee, SUC_lexicon_tags_668 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x173fedf2, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8ebfedf6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x9b5fedf9, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0x575fedfb, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x983fedfe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fee02, SUC_lexicon_tags_46 },
    { 0x6f5fee02, SUC_lexicon_tags_30 },
    { 0xc69fee04, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x2dffee06, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8e1fee0b, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x149fee0d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3a7fee15, SUC_lexicon_tags_18 },
    { 0, NULL },
    { 0, NULL },
    { 0xe99fee18, SUC_lexicon_tags_60 },
    { 0x105fee19, SUC_lexicon_tags_17 },
    { 0xbeffee19, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x389fee1c, SUC_lexicon_tags_37 },
    { 0xb67fee1d, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8dfee23, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0xb09fee25, SUC_lexicon_tags_25 },
    { 0xe3dfee26, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0xe41fee29, SUC_lexicon_tags_17 },
    { 0xcfbfee29, SUC_lexicon_tags_77 },
    { 0x19dfee2b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x32dfee30, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x153fee32, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf4dfee36, SUC_lexicon_tags_37 },
    { 0x5b1fee37, SUC_lexicon_tags_49 },
    { 0xfeffee38, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0x813fee3b, SUC_lexicon_tags_28 },
    { 0x2fdfee3b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7e7fee44, SUC_lexicon_tags_2 },
    { 0x495fee44, SUC_lexicon_tags_0 },
    { 0x35ffee46, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcedfee48, SUC_lexicon_tags_14 },
    { 0xbf1fee49, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x7a1fee4c, SUC_lexicon_tags_273 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3edfee53, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0, NULL },
    { 0x7bffee56, SUC_lexicon_tags_44 },
    { 0x9f1fee56, SUC_lexicon_tags_37 },
    { 0xaebfee57, SUC_lexicon_tags_1 },
    { 0xe19fee57, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0xc25fee5b, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933fee64, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x72dfee66, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xebbfee69, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbe7fee74, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdfee77, SUC_lexicon_tags_60 },
    { 0x6c7fee77, SUC_lexicon_tags_15 },
    { 0x947fee79, SUC_lexicon_tags_1 },
    { 0x15ffee7a, SUC_lexicon_tags_25 },
    { 0xb65fee7b, SUC_lexicon_tags_31 },
    { 0x91ffee7b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd5ffee82, SUC_lexicon_tags_30 },
    { 0x2fdfee83, SUC_lexicon_tags_25 },
    { 0x979fee84, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c7fee8a, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9a9fee91, SUC_lexicon_tags_138 },
    { 0x4d7fee91, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0x6c9fee94, SUC_lexicon_tags_1 },
    { 0xafbfee94, SUC_lexicon_tags_2 },
    { 0x431fee94, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3fee9e, SUC_lexicon_tags_37 },
    { 0xaf7fee9f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd39feea3, SUC_lexicon_tags_17 },
    { 0x7fdfeea4, SUC_lexicon_tags_109 },
    { 0, NULL },
    { 0xf45feea6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe15feead, SUC_lexicon_tags_37 },
    { 0x959feeae, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x323feeb1, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x695feebb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x887feebe, SUC_lexicon_tags_61 },
    { 0xd43feebf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xc19feec1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9ffeec6, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x8edfeec9, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0x231feecb, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xfcffeecd, SUC_lexicon_tags_25 },
    { 0x233feece, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x345feed0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xacffeed3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x475feede, SUC_lexicon_tags_16 },
    { 0x86bfeedf, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc8ffeee5, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f1feee8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x1dfeeea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xa77feeec, SUC_lexicon_tags_17 },
    { 0x749feeed, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82ffeef2, SUC_lexicon_tags_7 },
    { 0x629feef2, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0x481feef6, SUC_lexicon_tags_39 },
    { 0xf3dfeef7, SUC_lexicon_tags_19 },
    { 0x3edfeef7, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x5d3feefa, SUC_lexicon_tags_26 },
    { 0x493feefb, SUC_lexicon_tags_38 },
    { 0x773feefa, SUC_lexicon_tags_7 },
    { 0x81feefd, SUC_lexicon_tags_17 },
    { 0x3ebfeefe, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7cbfef04, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x4dffef06, SUC_lexicon_tags_17 },
    { 0xd53fef07, SUC_lexicon_tags_12 },
    { 0xbc7fef08, SUC_lexicon_tags_25 },
    { 0xb25fef08, SUC_lexicon_tags_11 },
    { 0xd91fef0a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd89fef0c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc6dfef0e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x23dfef10, SUC_lexicon_tags_230 },
    { 0x9d9fef11, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc9fef15, SUC_lexicon_tags_40 },
    { 0, NULL },
    { 0x5a7fef17, SUC_lexicon_tags_33 },
    { 0x6b1fef18, SUC_lexicon_tags_49 },
    { 0xb83fef18, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3fef1e, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0xcebfef20, SUC_lexicon_tags_11 },
    { 0xa83fef21, SUC_lexicon_tags_12 },
    { 0x23bfef22, SUC_lexicon_tags_4 },
    { 0xffdfef23, SUC_lexicon_tags_40 },
    { 0xb85fef22, SUC_lexicon_tags_47 },
    { 0x2bbfef23, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x695fef27, SUC_lexicon_tags_4 },
    { 0xb4bfef28, SUC_lexicon_tags_1 },
    { 0xb7bfef28, SUC_lexicon_tags_19 },
    { 0xe51fef2a, SUC_lexicon_tags_14 },
    { 0x31ffef2b, SUC_lexicon_tags_11 },
    { 0x2e9fef2b, SUC_lexicon_tags_19 },
    { 0x109fef28, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xcfbfef2f, SUC_lexicon_tags_37 },
    { 0xb49fef30, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x35bfef32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x61dfef35, SUC_lexicon_tags_5 },
    { 0x7adfef36, SUC_lexicon_tags_17 },
    { 0x3d9fef36, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677fef40, SUC_lexicon_tags_8 },
    { 0x92dfef40, SUC_lexicon_tags_16 },
    { 0x48dfef40, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac9fef46, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xd1bfef48, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc45fef53, SUC_lexicon_tags_23 },
    { 0x2affef53, SUC_lexicon_tags_175 },
    { 0, NULL },
    { 0x7d5fef56, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89fef61, SUC_lexicon_tags_34 },
    { 0xa3fef62, SUC_lexicon_tags_17 },
    { 0xf81fef63, SUC_lexicon_tags_68 },
    { 0xddffef64, SUC_lexicon_tags_25 },
    { 0xa4dfef65, SUC_lexicon_tags_25 },
    { 0x37bfef66, SUC_lexicon_tags_27 },
    { 0xf07fef67, SUC_lexicon_tags_65 },
    { 0xd89fef68, SUC_lexicon_tags_0 },
    { 0x629fef68, SUC_lexicon_tags_66 },
    { 0x271fef68, SUC_lexicon_tags_37 },
    { 0x79ffef6b, SUC_lexicon_tags_12 },
    { 0x8a5fef6c, SUC_lexicon_tags_7 },
    { 0x65ffef6d, SUC_lexicon_tags_25 },
    { 0x9b5fef6e, SUC_lexicon_tags_17 },
    { 0x6f9fef6f, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xab7fef71, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x8b3fef73, SUC_lexicon_tags_68 },
    { 0x423fef74, SUC_lexicon_tags_28 },
    { 0x6d5fef73, SUC_lexicon_tags_17 },
    { 0x44bfef76, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1dfef7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea9fef7f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa79fef81, SUC_lexicon_tags_118 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe03fef85, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x64bfef87, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x34dfef89, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x753fef8e, SUC_lexicon_tags_7 },
    { 0xa67fef8e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd4dfef97, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x9f1fef99, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x413fef9c, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xc3fef9e, SUC_lexicon_tags_26 },
    { 0xb57fef9f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3fefa6, SUC_lexicon_tags_14 },
    { 0x15dfefa7, SUC_lexicon_tags_11 },
    { 0xcb7fefa6, SUC_lexicon_tags_56 },
    { 0x293fefa9, SUC_lexicon_tags_25 },
    { 0x195fefa9, SUC_lexicon_tags_11 },
    { 0xeaffefaa, SUC_lexicon_tags_12 },
    { 0xd15fefac, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6a5fefb2, SUC_lexicon_tags_15 },
    { 0x77fefb3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x237fefb5, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x907fefb7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x547fefb9, SUC_lexicon_tags_37 },
    { 0xa25fefba, SUC_lexicon_tags_37 },
    { 0x561fefb9, SUC_lexicon_tags_7 },
    { 0x37fefbc, SUC_lexicon_tags_19 },
    { 0x44bfefbd, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0x427fefbf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x643fefc5, SUC_lexicon_tags_43 },
    { 0x939fefc6, SUC_lexicon_tags_46 },
    { 0x6bbfefc7, SUC_lexicon_tags_1 },
    { 0x96bfefc8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fefcd, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfabfefd1, SUC_lexicon_tags_0 },
    { 0x87dfefd2, SUC_lexicon_tags_15 },
    { 0xdcffefd3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x265fefd5, SUC_lexicon_tags_7 },
    { 0x69dfefd6, SUC_lexicon_tags_1 },
    { 0x7c1fefd5, SUC_lexicon_tags_1 },
    { 0x2bbfefd7, SUC_lexicon_tags_27 },
    { 0x3e7fefd9, SUC_lexicon_tags_2 },
    { 0x737fefd6, SUC_lexicon_tags_37 },
    { 0x961fefdb, SUC_lexicon_tags_25 },
    { 0x789fefdc, SUC_lexicon_tags_7 },
    { 0xb9fefdd, SUC_lexicon_tags_12 },
    { 0xa77fefdd, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xea5fefe2, SUC_lexicon_tags_66 },
    { 0x3dffefe2, SUC_lexicon_tags_25 },
    { 0x2a5fefe4, SUC_lexicon_tags_12 },
    { 0xda3fefe5, SUC_lexicon_tags_21 },
    { 0x84bfefe6, SUC_lexicon_tags_15 },
    { 0x79fefe3, SUC_lexicon_tags_124 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3fefeb, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x763fefef, SUC_lexicon_tags_262 },
    { 0xa33fefef, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x477feff4, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x623feff7, SUC_lexicon_tags_40 },
    { 0x2cdfeff7, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xddffeffa, SUC_lexicon_tags_11 },
    { 0x81ffeffb, SUC_lexicon_tags_63 },
    { 0xc8ffeffc, SUC_lexicon_tags_2 },
    { 0x231feffd, SUC_lexicon_tags_70 },
    { 0, NULL },
    { 0, NULL },
    { 0x51fff000, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5f9ff003, SUC_lexicon_tags_2 },
    { 0xb85ff004, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x62fff006, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x305ff008, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6f9ff00c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3dfff010, SUC_lexicon_tags_27 },
    { 0xbffff011, SUC_lexicon_tags_1 },
    { 0x555ff012, SUC_lexicon_tags_4 },
    { 0xbefff010, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x895ff015, SUC_lexicon_tags_27 },
    { 0x413ff016, SUC_lexicon_tags_47 },
    { 0xcd9ff016, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25ff01a, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccbff01f, SUC_lexicon_tags_8 },
    { 0xad3ff01f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bff02b, SUC_lexicon_tags_25 },
    { 0x459ff02b, SUC_lexicon_tags_31 },
    { 0x8cfff02c, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3fff030, SUC_lexicon_tags_15 },
    { 0x993ff031, SUC_lexicon_tags_4 },
    { 0xe91ff032, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x109ff036, SUC_lexicon_tags_11 },
    { 0x86dff037, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x807ff03a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x143ff03c, SUC_lexicon_tags_33 },
    { 0xcf7ff03d, SUC_lexicon_tags_69 },
    { 0x8c7ff03e, SUC_lexicon_tags_1 },
    { 0x6d1ff03f, SUC_lexicon_tags_37 },
    { 0xa35ff040, SUC_lexicon_tags_37 },
    { 0xddff040, SUC_lexicon_tags_30 },
    { 0x21bff042, SUC_lexicon_tags_37 },
    { 0x94fff042, SUC_lexicon_tags_14 },
    { 0x8f7ff044, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xde3ff046, SUC_lexicon_tags_16 },
    { 0xa9fff047, SUC_lexicon_tags_49 },
    { 0xc25ff047, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x97bff04a, SUC_lexicon_tags_19 },
    { 0xde9ff04a, SUC_lexicon_tags_60 },
    { 0x7ffff04c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x4b5ff04f, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x805ff051, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe5ff05a, SUC_lexicon_tags_17 },
    { 0xf47ff05a, SUC_lexicon_tags_38 },
    { 0x425ff05b, SUC_lexicon_tags_15 },
    { 0x36fff05c, SUC_lexicon_tags_17 },
    { 0xcd3ff05e, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe39ff062, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x61ff064, SUC_lexicon_tags_25 },
    { 0xa49ff065, SUC_lexicon_tags_19 },
    { 0x5ffff066, SUC_lexicon_tags_8 },
    { 0x74fff066, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0xcc7ff069, SUC_lexicon_tags_38 },
    { 0xcd5ff06a, SUC_lexicon_tags_15 },
    { 0x7edff06a, SUC_lexicon_tags_70 },
    { 0x825ff06a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x895ff071, SUC_lexicon_tags_9 },
    { 0xee7ff072, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfdbff07a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8c1ff080, SUC_lexicon_tags_37 },
    { 0x275ff080, SUC_lexicon_tags_61 },
    { 0xe0dff082, SUC_lexicon_tags_48 },
    { 0x3f7ff083, SUC_lexicon_tags_65 },
    { 0xbdff084, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0x3afff087, SUC_lexicon_tags_25 },
    { 0xeafff088, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x677ff08c, SUC_lexicon_tags_35 },
    { 0x1a7ff08d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x36dff097, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0xb1ff09a, SUC_lexicon_tags_7 },
    { 0x37bff09b, SUC_lexicon_tags_37 },
    { 0x605ff09c, SUC_lexicon_tags_19 },
    { 0x9a1ff09d, SUC_lexicon_tags_6 },
    { 0x5b5ff09e, SUC_lexicon_tags_2 },
    { 0xd83ff09f, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7ff0a3, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x289ff0a6, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd31ff0a8, SUC_lexicon_tags_38 },
    { 0x191ff0a9, SUC_lexicon_tags_49 },
    { 0x8fdff0aa, SUC_lexicon_tags_12 },
    { 0x61fff0ab, SUC_lexicon_tags_7 },
    { 0x86fff0ac, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf13ff0ae, SUC_lexicon_tags_17 },
    { 0xc59ff0ae, SUC_lexicon_tags_379 },
    { 0x267ff0af, SUC_lexicon_tags_11 },
    { 0xae5ff0b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x94fff0b3, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5bff0bf, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe65ff0c2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xebfff0c6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ff0ca, SUC_lexicon_tags_11 },
    { 0xf87ff0cb, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xad5ff0d0, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653ff0d5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebff0db, SUC_lexicon_tags_10 },
    { 0x563ff0db, SUC_lexicon_tags_81 },
    { 0x29bff0dc, SUC_lexicon_tags_25 },
    { 0xc73ff0de, SUC_lexicon_tags_37 },
    { 0x6a7ff0df, SUC_lexicon_tags_23 },
    { 0xe6bff0e0, SUC_lexicon_tags_11 },
    { 0x353ff0e1, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xef9ff0e5, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x97ff0e7, SUC_lexicon_tags_27 },
    { 0xe37ff0e8, SUC_lexicon_tags_13 },
    { 0xb5ff0e8, SUC_lexicon_tags_7 },
    { 0x359ff0ea, SUC_lexicon_tags_38 },
    { 0x82fff0ea, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x879ff0f0, SUC_lexicon_tags_28 },
    { 0x2b1ff0f1, SUC_lexicon_tags_7 },
    { 0x49ff0f2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x225ff0f6, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x86bff0f8, SUC_lexicon_tags_6 },
    { 0x7c9ff0f8, SUC_lexicon_tags_33 },
    { 0x71ff0fa, SUC_lexicon_tags_60 },
    { 0x481ff0fb, SUC_lexicon_tags_2 },
    { 0xc9ff0fc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3ff106, SUC_lexicon_tags_13 },
    { 0xee7ff107, SUC_lexicon_tags_17 },
    { 0x6cfff108, SUC_lexicon_tags_47 },
    { 0xd9bff107, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x973ff110, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa8bff116, SUC_lexicon_tags_85 },
    { 0x2b1ff117, SUC_lexicon_tags_2 },
    { 0x9e9ff118, SUC_lexicon_tags_17 },
    { 0xbf5ff118, SUC_lexicon_tags_3 },
    { 0xc19ff117, SUC_lexicon_tags_9 },
    { 0xce3ff11b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0xe4fff11d, SUC_lexicon_tags_23 },
    { 0x52fff11e, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0xe5ff120, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x329ff125, SUC_lexicon_tags_26 },
    { 0x27bff125, SUC_lexicon_tags_2 },
    { 0x4abff127, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0xe03ff129, SUC_lexicon_tags_39 },
    { 0x5ff129, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xc05ff12c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ff12f, SUC_lexicon_tags_14 },
    { 0x6d1ff130, SUC_lexicon_tags_15 },
    { 0xd91ff12f, SUC_lexicon_tags_43 },
    { 0xe81ff132, SUC_lexicon_tags_10 },
    { 0x8f5ff130, SUC_lexicon_tags_1294 },
    { 0x965ff12f, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x965ff138, SUC_lexicon_tags_7 },
    { 0xec7ff139, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7ebff13c, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6abff13e, SUC_lexicon_tags_11 },
    { 0x4bff13f, SUC_lexicon_tags_42 },
    { 0xcdbff13e, SUC_lexicon_tags_2 },
    { 0x84fff13f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x53fff148, SUC_lexicon_tags_2 },
    { 0x705ff148, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedff14e, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163ff152, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xb55ff154, SUC_lexicon_tags_0 },
    { 0xe55ff155, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4b9ff15e, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa33ff162, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7abff16b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x901ff16d, SUC_lexicon_tags_33 },
    { 0x405ff16e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a1ff171, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275ff175, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x46fff17b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa3ff17d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdefff17f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe19ff183, SUC_lexicon_tags_11 },
    { 0xadfff184, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd69ff18d, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0x11bff190, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5ff193, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x42bff19c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc11ff1a2, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1dbff1a7, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69ff1ad, SUC_lexicon_tags_15 },
    { 0x1b1ff1ae, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x64fff1b1, SUC_lexicon_tags_45 },
    { 0x291ff1b2, SUC_lexicon_tags_6 },
    { 0x657ff1b2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x8e7ff1b5, SUC_lexicon_tags_33 },
    { 0xa1fff1b5, SUC_lexicon_tags_49 },
    { 0xda3ff1b5, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xeddff1b9, SUC_lexicon_tags_25 },
    { 0x607ff1ba, SUC_lexicon_tags_30 },
    { 0xb8bff1b9, SUC_lexicon_tags_4 },
    { 0x717ff1b9, SUC_lexicon_tags_17 },
    { 0xc31ff1bd, SUC_lexicon_tags_17 },
    { 0x9e7ff1be, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb39ff1c2, SUC_lexicon_tags_25 },
    { 0x4b1ff1c3, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x3f5ff1c5, SUC_lexicon_tags_12 },
    { 0x727ff1c6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x525ff1cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9d9ff1d6, SUC_lexicon_tags_25 },
    { 0x4f3ff1d7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd65ff1da, SUC_lexicon_tags_12 },
    { 0xa1fff1db, SUC_lexicon_tags_30 },
    { 0xa85ff1da, SUC_lexicon_tags_63 },
    { 0x8bbff1dc, SUC_lexicon_tags_63 },
    { 0xe61ff1dc, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe35ff1e3, SUC_lexicon_tags_11 },
    { 0x679ff1e4, SUC_lexicon_tags_11 },
    { 0x52dff1e5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x3d9ff1e7, SUC_lexicon_tags_11 },
    { 0x5a1ff1e8, SUC_lexicon_tags_16 },
    { 0xe05ff1e8, SUC_lexicon_tags_262 },
    { 0xf69ff1e9, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x273ff1ec, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5ff1ef, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x203ff1f4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa6fff1f8, SUC_lexicon_tags_25 },
    { 0x37bff1f9, SUC_lexicon_tags_35 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e1ff1fd, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x211ff1ff, SUC_lexicon_tags_17 },
    { 0xa75ff200, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x69ff202, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0xf39ff205, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xa1dff207, SUC_lexicon_tags_45 },
    { 0x55fff207, SUC_lexicon_tags_55 },
    { 0xe9bff207, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x3dfff20b, SUC_lexicon_tags_26 },
    { 0x95bff20c, SUC_lexicon_tags_1 },
    { 0x9ddff20d, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x575ff212, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x33bff214, SUC_lexicon_tags_17 },
    { 0x5abff214, SUC_lexicon_tags_2 },
    { 0x969ff216, SUC_lexicon_tags_16 },
    { 0xa2fff217, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x26bff219, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x957ff21d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x9cbff21f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x877ff224, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x491ff22a, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd0dff230, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x709ff236, SUC_lexicon_tags_38 },
    { 0x88dff237, SUC_lexicon_tags_23 },
    { 0x983ff238, SUC_lexicon_tags_19 },
    { 0x4c7ff238, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dff240, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5adff245, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xdf9ff248, SUC_lexicon_tags_17 },
    { 0xf03ff249, SUC_lexicon_tags_61 },
    { 0x9abff24a, SUC_lexicon_tags_17 },
    { 0xfdfff24a, SUC_lexicon_tags_60 },
    { 0xc41ff24b, SUC_lexicon_tags_60 },
    { 0xd8bff24d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x949ff24f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x395ff254, SUC_lexicon_tags_45 },
    { 0xac7ff254, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x817ff257, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb21ff25e, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27ff263, SUC_lexicon_tags_25 },
    { 0xabdff264, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x77bff266, SUC_lexicon_tags_25 },
    { 0x767ff267, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ff26c, SUC_lexicon_tags_55 },
    { 0x85fff26d, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d7ff273, SUC_lexicon_tags_152 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddff277, SUC_lexicon_tags_151 },
    { 0x7d9ff278, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xffdff27b, SUC_lexicon_tags_25 },
    { 0x79bff27b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x7d1ff27e, SUC_lexicon_tags_11 },
    { 0x51dff27e, SUC_lexicon_tags_29 },
    { 0x7e9ff27e, SUC_lexicon_tags_17 },
    { 0xb9ff281, SUC_lexicon_tags_7 },
    { 0x547ff282, SUC_lexicon_tags_47 },
    { 0x46bff27e, SUC_lexicon_tags_2 },
    { 0xae9ff284, SUC_lexicon_tags_21 },
    { 0x9b1ff27f, SUC_lexicon_tags_23 },
    { 0xab5ff286, SUC_lexicon_tags_6 },
    { 0x6ffff281, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0x87dff28a, SUC_lexicon_tags_7 },
    { 0x145ff28a, SUC_lexicon_tags_25 },
    { 0xd95ff28c, SUC_lexicon_tags_47 },
    { 0x4b7ff28d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x535ff293, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe27ff29b, SUC_lexicon_tags_61 },
    { 0xb9dff29c, SUC_lexicon_tags_5 },
    { 0x87ff29d, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe5ff2a0, SUC_lexicon_tags_11 },
    { 0x2ddff2a0, SUC_lexicon_tags_11 },
    { 0xc27ff2a1, SUC_lexicon_tags_151 },
    { 0x82fff2a3, SUC_lexicon_tags_31 },
    { 0xf53ff2a4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xb8fff2a6, SUC_lexicon_tags_21 },
    { 0x275ff2a7, SUC_lexicon_tags_17 },
    { 0xfd9ff2a8, SUC_lexicon_tags_17 },
    { 0x731ff2a9, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x83dff2ae, SUC_lexicon_tags_7 },
    { 0x6afff2ae, SUC_lexicon_tags_80 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x211ff2b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x39fff2bc, SUC_lexicon_tags_2 },
    { 0x413ff2bd, SUC_lexicon_tags_87 },
    { 0x429ff2bd, SUC_lexicon_tags_1 },
    { 0x7dff2bf, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdcfff2c8, SUC_lexicon_tags_17 },
    { 0x143ff2c9, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd05ff2d0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfff2d3, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff2d8, SUC_lexicon_tags_23 },
    { 0x79dff2d8, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x449ff2db, SUC_lexicon_tags_17 },
    { 0xae5ff2dc, SUC_lexicon_tags_11 },
    { 0x7f9ff2dd, SUC_lexicon_tags_25 },
    { 0x589ff2de, SUC_lexicon_tags_48 },
    { 0xd49ff2de, SUC_lexicon_tags_30 },
    { 0x3b3ff2e0, SUC_lexicon_tags_18 },
    { 0x407ff2e0, SUC_lexicon_tags_15 },
    { 0x47ff2e2, SUC_lexicon_tags_25 },
    { 0xa03ff2e3, SUC_lexicon_tags_12 },
    { 0x2efff2e0, SUC_lexicon_tags_57 },
    { 0x76bff2e5, SUC_lexicon_tags_2 },
    { 0x301ff2e1, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xb8fff2e8, SUC_lexicon_tags_63 },
    { 0x45ff2e9, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xc55ff2ec, SUC_lexicon_tags_12 },
    { 0xe81ff2ed, SUC_lexicon_tags_21 },
    { 0x32bff2ee, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x31dff2f0, SUC_lexicon_tags_11 },
    { 0x6ffff2f1, SUC_lexicon_tags_25 },
    { 0x489ff2f1, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb41ff2f7, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0x7e7ff2f9, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a5ff2fd, SUC_lexicon_tags_43 },
    { 0x63bff2fe, SUC_lexicon_tags_1130 },
    { 0x8d9ff2ff, SUC_lexicon_tags_140 },
    { 0, NULL },
    { 0x397ff301, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9dff304, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd1fff306, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0x4ddff308, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0, NULL },
    { 0xa3bff30b, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x827ff310, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x19ff320, SUC_lexicon_tags_8 },
    { 0x12dff321, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x5afff323, SUC_lexicon_tags_19 },
    { 0x1f7ff324, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x9d1ff326, SUC_lexicon_tags_37 },
    { 0xf33ff326, SUC_lexicon_tags_25 },
    { 0x21bff328, SUC_lexicon_tags_7 },
    { 0x61dff329, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c7ff332, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbff336, SUC_lexicon_tags_66 },
    { 0xf2dff336, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x92fff33a, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x861ff33f, SUC_lexicon_tags_103 },
    { 0xfbff33f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x741ff344, SUC_lexicon_tags_60 },
    { 0x2fff345, SUC_lexicon_tags_7 },
    { 0xc7bff346, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xd25ff348, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x675ff34b, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xbfbff34d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x5dff350, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xe43ff353, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe6bff358, SUC_lexicon_tags_0 },
    { 0x8cbff359, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x7ddff35b, SUC_lexicon_tags_12 },
    { 0x3d7ff35b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae9ff369, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x7a7ff36b, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x7dfff36e, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x5b3ff370, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ff373, SUC_lexicon_tags_48 },
    { 0xc69ff374, SUC_lexicon_tags_17 },
    { 0xa1ff374, SUC_lexicon_tags_35 },
    { 0x15fff375, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x6bdff378, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0x3abff37b, SUC_lexicon_tags_12 },
    { 0xf1ff37c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x64dff37e, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xc3dff381, SUC_lexicon_tags_30 },
    { 0x7f1ff382, SUC_lexicon_tags_49 },
    { 0x81fff382, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xbc5ff385, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfe1ff38a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x511ff38c, SUC_lexicon_tags_8 },
    { 0x269ff38d, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x161ff391, SUC_lexicon_tags_23 },
    { 0x79ff392, SUC_lexicon_tags_74 },
    { 0, NULL },
    { 0, NULL },
    { 0x51dff395, SUC_lexicon_tags_57 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9ff399, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xec5ff39b, SUC_lexicon_tags_38 },
    { 0x3efff39c, SUC_lexicon_tags_31 },
    { 0x215ff39c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x549ff39f, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x6cbff3a1, SUC_lexicon_tags_12 },
    { 0x6b7ff3a2, SUC_lexicon_tags_38 },
    { 0xb81ff3a3, SUC_lexicon_tags_25 },
    { 0xecfff3a3, SUC_lexicon_tags_46 },
    { 0x4cdff3a5, SUC_lexicon_tags_15 },
    { 0x305ff3a5, SUC_lexicon_tags_7 },
    { 0x985ff3a1, SUC_lexicon_tags_26 },
    { 0xca9ff3a2, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bff3ad, SUC_lexicon_tags_23 },
    { 0x911ff3ad, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x313ff3b2, SUC_lexicon_tags_12 },
    { 0x387ff3b2, SUC_lexicon_tags_45 },
    { 0xfcbff3b3, SUC_lexicon_tags_7 },
    { 0xe93ff3b3, SUC_lexicon_tags_60 },
    { 0x355ff3b6, SUC_lexicon_tags_1 },
    { 0xda1ff3b6, SUC_lexicon_tags_1 },
    { 0x7c7ff3b6, SUC_lexicon_tags_1 },
    { 0x8bdff3b9, SUC_lexicon_tags_41 },
    { 0, NULL },
    { 0x3fbff3bb, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x96fff3c8, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6afff3d2, SUC_lexicon_tags_4 },
    { 0xa53ff3d3, SUC_lexicon_tags_172 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xafff3da, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xb27ff3dc, SUC_lexicon_tags_31 },
    { 0x8c1ff3dc, SUC_lexicon_tags_21 },
    { 0x47ff3dd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x767ff3e1, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x127ff3e3, SUC_lexicon_tags_26 },
    { 0x64bff3e4, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ff3f4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x291ff3fb, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x919ff402, SUC_lexicon_tags_19 },
    { 0x825ff403, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb89ff407, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75bff40c, SUC_lexicon_tags_87 },
    { 0xc05ff40d, SUC_lexicon_tags_23 },
    { 0x5d9ff40e, SUC_lexicon_tags_37 },
    { 0x4f9ff40f, SUC_lexicon_tags_26 },
    { 0xc81ff40c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x501ff412, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x46bff415, SUC_lexicon_tags_17 },
    { 0x21ff415, SUC_lexicon_tags_29 },
    { 0xb2dff416, SUC_lexicon_tags_0 },
    { 0xb6dff418, SUC_lexicon_tags_11 },
    { 0x113ff416, SUC_lexicon_tags_14 },
    { 0x7bdff415, SUC_lexicon_tags_12 },
    { 0xc11ff41b, SUC_lexicon_tags_63 },
    { 0xb03ff41b, SUC_lexicon_tags_31 },
    { 0x31ff41d, SUC_lexicon_tags_17 },
    { 0xba1ff41e, SUC_lexicon_tags_0 },
    { 0x49dff41f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd55ff422, SUC_lexicon_tags_1 },
    { 0x84fff423, SUC_lexicon_tags_37 },
    { 0x9e5ff424, SUC_lexicon_tags_67 },
    { 0xdff423, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd05ff427, SUC_lexicon_tags_11 },
    { 0x8bbff428, SUC_lexicon_tags_2 },
    { 0x647ff428, SUC_lexicon_tags_81 },
    { 0x913ff427, SUC_lexicon_tags_15 },
    { 0xa41ff42b, SUC_lexicon_tags_12 },
    { 0xa31ff427, SUC_lexicon_tags_25 },
    { 0x645ff42c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xa03ff42f, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x25dff431, SUC_lexicon_tags_60 },
    { 0x3ddff431, SUC_lexicon_tags_12 },
    { 0xa07ff433, SUC_lexicon_tags_1 },
    { 0x2d5ff433, SUC_lexicon_tags_109 },
    { 0x39ff433, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x1e1ff437, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x181ff43d, SUC_lexicon_tags_0 },
    { 0xd6fff43e, SUC_lexicon_tags_19 },
    { 0xbf5ff43f, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0xc9ff441, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae5ff448, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0x961ff44a, SUC_lexicon_tags_299 },
    { 0x4d1ff44b, SUC_lexicon_tags_1 },
    { 0xa35ff44c, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0x2ffff44e, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1b7ff453, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1ff457, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x369ff459, SUC_lexicon_tags_26 },
    { 0x665ff45a, SUC_lexicon_tags_37 },
    { 0x523ff45b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x3d9ff45e, SUC_lexicon_tags_30 },
    { 0x84bff45e, SUC_lexicon_tags_28 },
    { 0xa33ff460, SUC_lexicon_tags_48 },
    { 0xe6dff461, SUC_lexicon_tags_38 },
    { 0xa29ff461, SUC_lexicon_tags_168 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8bff468, SUC_lexicon_tags_103 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x47bff470, SUC_lexicon_tags_4 },
    { 0x333ff471, SUC_lexicon_tags_47 },
    { 0x3e7ff471, SUC_lexicon_tags_27 },
    { 0xe89ff471, SUC_lexicon_tags_1 },
    { 0x803ff474, SUC_lexicon_tags_23 },
    { 0xc61ff474, SUC_lexicon_tags_85 },
    { 0, NULL },
    { 0, NULL },
    { 0x1bff478, SUC_lexicon_tags_6 },
    { 0x8a3ff479, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0x741ff47c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x571ff483, SUC_lexicon_tags_32 },
    { 0x12bff484, SUC_lexicon_tags_0 },
    { 0x691ff485, SUC_lexicon_tags_68 },
    { 0x783ff486, SUC_lexicon_tags_14 },
    { 0xe91ff486, SUC_lexicon_tags_27 },
    { 0xc91ff488, SUC_lexicon_tags_48 },
    { 0xb3ff486, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x9d7ff48b, SUC_lexicon_tags_17 },
    { 0x96dff48c, SUC_lexicon_tags_7 },
    { 0xe2bff48c, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x131ff48f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3ff492, SUC_lexicon_tags_6 },
    { 0xbe7ff493, SUC_lexicon_tags_1 },
    { 0xfdfff493, SUC_lexicon_tags_60 },
    { 0x7fdff494, SUC_lexicon_tags_3 },
    { 0x6e5ff493, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x7d5ff498, SUC_lexicon_tags_37 },
    { 0x769ff498, SUC_lexicon_tags_1339 },
    { 0, NULL },
    { 0, NULL },
    { 0xc27ff49c, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc53ff4a3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x76bff4a5, SUC_lexicon_tags_12 },
    { 0xbd7ff4a5, SUC_lexicon_tags_2 },
    { 0x991ff4a7, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xae3ff4ad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xc4bff4af, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe8bff4b3, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29fff4b5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa3ff4ba, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x327ff4bc, SUC_lexicon_tags_26 },
    { 0xb8fff4bc, SUC_lexicon_tags_15 },
    { 0xd8fff4be, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xb71ff4c0, SUC_lexicon_tags_61 },
    { 0xad1ff4c0, SUC_lexicon_tags_1 },
    { 0x38fff4c2, SUC_lexicon_tags_17 },
    { 0x24bff4c3, SUC_lexicon_tags_79 },
    { 0x303ff4c3, SUC_lexicon_tags_13 },
    { 0x477ff4c2, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xab1ff4c7, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf21ff4ce, SUC_lexicon_tags_27 },
    { 0xa77ff4ce, SUC_lexicon_tags_27 },
    { 0x3efff4d0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x547ff4d2, SUC_lexicon_tags_21 },
    { 0xc2fff4d3, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e3ff4d8, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bff4db, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x755ff4de, SUC_lexicon_tags_35 },
    { 0x991ff4de, SUC_lexicon_tags_11 },
    { 0x413ff4e0, SUC_lexicon_tags_23 },
    { 0x2dbff4e0, SUC_lexicon_tags_6 },
    { 0x457ff4e2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d1ff4e6, SUC_lexicon_tags_47 },
    { 0x9c9ff4e7, SUC_lexicon_tags_12 },
    { 0x52dff4e8, SUC_lexicon_tags_17 },
    { 0x371ff4e7, SUC_lexicon_tags_25 },
    { 0x76fff4e7, SUC_lexicon_tags_664 },
    { 0x43ff4eb, SUC_lexicon_tags_13 },
    { 0x11ff4e9, SUC_lexicon_tags_15 },
    { 0xc9fff4ed, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf9ff4f4, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7f3ff4fa, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3ff4fd, SUC_lexicon_tags_12 },
    { 0xd53ff4fd, SUC_lexicon_tags_63 },
    { 0xc69ff4fe, SUC_lexicon_tags_49 },
    { 0x5bfff500, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x405ff505, SUC_lexicon_tags_31 },
    { 0x795ff506, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4a1ff50b, SUC_lexicon_tags_136 },
    { 0, NULL },
    { 0x555ff50d, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3cbff512, SUC_lexicon_tags_74 },
    { 0x71dff513, SUC_lexicon_tags_69 },
    { 0, NULL },
    { 0, NULL },
    { 0x1b3ff516, SUC_lexicon_tags_38 },
    { 0x927ff517, SUC_lexicon_tags_25 },
    { 0x6b1ff516, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xc3ff51a, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x875ff51c, SUC_lexicon_tags_25 },
    { 0x771ff51d, SUC_lexicon_tags_15 },
    { 0x649ff51e, SUC_lexicon_tags_38 },
    { 0x99dff51d, SUC_lexicon_tags_30 },
    { 0x4b9ff520, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x653ff52a, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd11ff52e, SUC_lexicon_tags_29 },
    { 0xd41ff52f, SUC_lexicon_tags_17 },
    { 0xee5ff530, SUC_lexicon_tags_8 },
    { 0x617ff531, SUC_lexicon_tags_2 },
    { 0xc09ff530, SUC_lexicon_tags_37 },
    { 0xf03ff530, SUC_lexicon_tags_2 },
    { 0xe4dff52f, SUC_lexicon_tags_2 },
    { 0x241ff531, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357ff539, SUC_lexicon_tags_39 },
    { 0x157ff53a, SUC_lexicon_tags_15 },
    { 0xbd5ff53a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x847ff540, SUC_lexicon_tags_136 },
    { 0x45fff541, SUC_lexicon_tags_17 },
    { 0x65dff542, SUC_lexicon_tags_11 },
    { 0xb3ff542, SUC_lexicon_tags_37 },
    { 0x9fff544, SUC_lexicon_tags_14 },
    { 0x145ff541, SUC_lexicon_tags_1 },
    { 0x95fff542, SUC_lexicon_tags_14 },
    { 0x457ff544, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xebbff549, SUC_lexicon_tags_37 },
    { 0x795ff54a, SUC_lexicon_tags_1 },
    { 0x1afff54b, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0x831ff54d, SUC_lexicon_tags_11 },
    { 0xf83ff54e, SUC_lexicon_tags_17 },
    { 0xa65ff54f, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x819ff553, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x28fff556, SUC_lexicon_tags_61 },
    { 0xbb7ff557, SUC_lexicon_tags_42 },
    { 0xfd5ff558, SUC_lexicon_tags_26 },
    { 0xf49ff559, SUC_lexicon_tags_8 },
    { 0x2cdff55a, SUC_lexicon_tags_40 },
    { 0x9b9ff55a, SUC_lexicon_tags_11 },
    { 0x3b1ff559, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd9ff560, SUC_lexicon_tags_15 },
    { 0x17fff560, SUC_lexicon_tags_58 },
    { 0, NULL },
    { 0, NULL },
    { 0x621ff564, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0, NULL },
    { 0xc8bff567, SUC_lexicon_tags_42 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x655ff56c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd49ff571, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xbadff574, SUC_lexicon_tags_33 },
    { 0xd41ff574, SUC_lexicon_tags_7 },
    { 0xb7ff576, SUC_lexicon_tags_25 },
    { 0x3cfff577, SUC_lexicon_tags_71 },
    { 0xa3fff575, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75fff57e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c9ff583, SUC_lexicon_tags_1 },
    { 0xd45ff583, SUC_lexicon_tags_19 },
    { 0x953ff585, SUC_lexicon_tags_533 },
    { 0x515ff583, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xac5ff58c, SUC_lexicon_tags_607 },
    { 0xfafff58d, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x749ff591, SUC_lexicon_tags_28 },
    { 0xa91ff592, SUC_lexicon_tags_39 },
    { 0xaadff591, SUC_lexicon_tags_45 },
    { 0x75dff593, SUC_lexicon_tags_87 },
    { 0x8cbff595, SUC_lexicon_tags_61 },
    { 0x6b3ff595, SUC_lexicon_tags_27 },
    { 0x81ff597, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963ff59e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xecfff5a0, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x9b7ff5a2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x97ff5a5, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51bff5a9, SUC_lexicon_tags_17 },
    { 0x6bfff5aa, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x5a1ff5ad, SUC_lexicon_tags_17 },
    { 0x227ff5ae, SUC_lexicon_tags_63 },
    { 0xe1ff5af, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0xc33ff5b1, SUC_lexicon_tags_21 },
    { 0x457ff5b1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x163ff5b6, SUC_lexicon_tags_15 },
    { 0xd45ff5b7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xbdfff5b9, SUC_lexicon_tags_34 },
    { 0xc8bff5ba, SUC_lexicon_tags_8 },
    { 0xfb7ff5bb, SUC_lexicon_tags_60 },
    { 0xc8bff5bc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xefbff5c3, SUC_lexicon_tags_11 },
    { 0xaa5ff5c3, SUC_lexicon_tags_12 },
    { 0xcb9ff5c5, SUC_lexicon_tags_23 },
    { 0x82fff5c6, SUC_lexicon_tags_12 },
    { 0x51ff5c7, SUC_lexicon_tags_49 },
    { 0xdd1ff5c8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x8fff5ca, SUC_lexicon_tags_23 },
    { 0xd6bff5cb, SUC_lexicon_tags_47 },
    { 0xf8bff5cb, SUC_lexicon_tags_21 },
    { 0x79bff5cc, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x27fff5cf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x2a9ff5d2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x54bff5d6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xd0dff5d8, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbc7ff5dc, SUC_lexicon_tags_12 },
    { 0x6d7ff5dc, SUC_lexicon_tags_28 },
    { 0xcadff5de, SUC_lexicon_tags_0 },
    { 0x341ff5df, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x2bbff5e2, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0x395ff5e4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf9fff5e6, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa9fff5eb, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x251ff5ef, SUC_lexicon_tags_17 },
    { 0x1bbff5ef, SUC_lexicon_tags_15 },
    { 0x429ff5f1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdf5ff5f5, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x51dff5f9, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x6b9ff5fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xd11ff5fe, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x81ff600, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd3ff606, SUC_lexicon_tags_40 },
    { 0x2cdff607, SUC_lexicon_tags_9 },
    { 0xe49ff607, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xb57ff60a, SUC_lexicon_tags_6 },
    { 0x861ff60b, SUC_lexicon_tags_15 },
    { 0xacbff60b, SUC_lexicon_tags_1 },
    { 0x56dff60c, SUC_lexicon_tags_17 },
    { 0x67bff60e, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xfb7ff611, SUC_lexicon_tags_25 },
    { 0x33fff611, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x98fff614, SUC_lexicon_tags_12 },
    { 0x617ff615, SUC_lexicon_tags_14 },
    { 0x6f3ff614, SUC_lexicon_tags_17 },
    { 0x773ff617, SUC_lexicon_tags_1 },
    { 0x631ff618, SUC_lexicon_tags_7 },
    { 0x45ff618, SUC_lexicon_tags_1 },
    { 0x9c1ff616, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x68dff61f, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd6dff625, SUC_lexicon_tags_12 },
    { 0x2dff626, SUC_lexicon_tags_15 },
    { 0xdc5ff627, SUC_lexicon_tags_35 },
    { 0x4ddff625, SUC_lexicon_tags_15 },
    { 0xaedff627, SUC_lexicon_tags_65 },
    { 0xde1ff628, SUC_lexicon_tags_37 },
    { 0x7ff62a, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0xfcdff62e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7b5ff633, SUC_lexicon_tags_229 },
    { 0xb1fff633, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x2d7ff636, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbff639, SUC_lexicon_tags_11 },
    { 0xc09ff639, SUC_lexicon_tags_87 },
    { 0x4cbff63b, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3ff641, SUC_lexicon_tags_53 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa65ff647, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321ff64b, SUC_lexicon_tags_12 },
    { 0x211ff64c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x99fff64e, SUC_lexicon_tags_47 },
    { 0xdabff64f, SUC_lexicon_tags_27 },
    { 0xcf1ff650, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x20fff652, SUC_lexicon_tags_1 },
    { 0x25dff653, SUC_lexicon_tags_4 },
    { 0xe49ff652, SUC_lexicon_tags_25 },
    { 0x329ff655, SUC_lexicon_tags_48 },
    { 0x87bff656, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x3ffff658, SUC_lexicon_tags_10 },
    { 0x9ebff659, SUC_lexicon_tags_21 },
    { 0xd01ff65a, SUC_lexicon_tags_2 },
    { 0x613ff65a, SUC_lexicon_tags_351 },
    { 0xd97ff659, SUC_lexicon_tags_6 },
    { 0xd8dff659, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabbff667, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xca3ff66c, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x703ff670, SUC_lexicon_tags_108 },
    { 0x671ff671, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x297ff67a, SUC_lexicon_tags_31 },
    { 0xfadff67b, SUC_lexicon_tags_65 },
    { 0x2adff67b, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0x6a1ff67e, SUC_lexicon_tags_1 },
    { 0x20fff67f, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9dff682, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x697ff684, SUC_lexicon_tags_79 },
    { 0, NULL },
    { 0xb95ff686, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0x123ff689, SUC_lexicon_tags_104 },
    { 0x567ff689, SUC_lexicon_tags_11 },
    { 0xbefff68b, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x25ff68d, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x30fff695, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x59fff697, SUC_lexicon_tags_25 },
    { 0xf7bff697, SUC_lexicon_tags_27 },
    { 0xbf1ff698, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x183ff69c, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x357ff6a3, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x9c9ff6a5, SUC_lexicon_tags_23 },
    { 0xcc3ff6a6, SUC_lexicon_tags_27 },
    { 0x18dff6a7, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x969ff6a9, SUC_lexicon_tags_15 },
    { 0xe1dff6aa, SUC_lexicon_tags_19 },
    { 0x76dff6ab, SUC_lexicon_tags_37 },
    { 0x8f9ff6aa, SUC_lexicon_tags_31 },
    { 0xc85ff6ab, SUC_lexicon_tags_25 },
    { 0x8e3ff6ae, SUC_lexicon_tags_37 },
    { 0x82fff6af, SUC_lexicon_tags_17 },
    { 0x21bff6b0, SUC_lexicon_tags_29 },
    { 0xbabff6b0, SUC_lexicon_tags_17 },
    { 0x6b7ff6b2, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xae5ff6b4, SUC_lexicon_tags_37 },
    { 0x929ff6b4, SUC_lexicon_tags_45 },
    { 0x13bff6b5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1ff6ba, SUC_lexicon_tags_15 },
    { 0xed1ff6bb, SUC_lexicon_tags_4 },
    { 0xa05ff6bb, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0x5d5ff6be, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xbe3ff6c1, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3c7ff6c3, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0xa0bff6c5, SUC_lexicon_tags_25 },
    { 0xfafff6c6, SUC_lexicon_tags_1 },
    { 0xe41ff6c5, SUC_lexicon_tags_19 },
    { 0xd65ff6c7, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdafff6cc, SUC_lexicon_tags_105 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb77ff6d0, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xcbff6d2, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x927ff6d4, SUC_lexicon_tags_7 },
    { 0x75dff6d5, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1e1ff6d9, SUC_lexicon_tags_49 },
    { 0x4bbff6da, SUC_lexicon_tags_37 },
    { 0x98fff6db, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc09ff6df, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x659ff6e6, SUC_lexicon_tags_15 },
    { 0x281ff6e6, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c9ff6eb, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x71ff6ed, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce3ff6f1, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaf7ff6f7, SUC_lexicon_tags_84 },
    { 0xa47ff6f8, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xcb1ff6fb, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x367ff6fd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc95ff702, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x627ff707, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x69fff709, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xb23ff70b, SUC_lexicon_tags_0 },
    { 0xbf1ff70b, SUC_lexicon_tags_15 },
    { 0x1d7ff70d, SUC_lexicon_tags_25 },
    { 0x19fff70d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3ff718, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0x449ff71a, SUC_lexicon_tags_2 },
    { 0x7fdff71b, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x29ff720, SUC_lexicon_tags_2 },
    { 0x8bff721, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb1fff729, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x15bff72d, SUC_lexicon_tags_1 },
    { 0xf3bff72e, SUC_lexicon_tags_2 },
    { 0x6b1ff72f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4d3ff734, SUC_lexicon_tags_65 },
    { 0xc89ff735, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cfff739, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x567ff73e, SUC_lexicon_tags_27 },
    { 0x823ff73e, SUC_lexicon_tags_33 },
    { 0x77dff740, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xa95ff742, SUC_lexicon_tags_91 },
    { 0x847ff742, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x72dff746, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa25ff74a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7ff74e, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd7ff750, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x455ff752, SUC_lexicon_tags_25 },
    { 0x887ff752, SUC_lexicon_tags_37 },
    { 0xd09ff754, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0x633ff756, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe87ff758, SUC_lexicon_tags_7 },
    { 0x229ff759, SUC_lexicon_tags_12 },
    { 0x1e5ff75a, SUC_lexicon_tags_12 },
    { 0xbf3ff758, SUC_lexicon_tags_17 },
    { 0x6a9ff75b, SUC_lexicon_tags_63 },
    { 0x149ff75d, SUC_lexicon_tags_114 },
    { 0x331ff75d, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x69dff760, SUC_lexicon_tags_1 },
    { 0x593ff761, SUC_lexicon_tags_1 },
    { 0xe79ff761, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x35bff765, SUC_lexicon_tags_1 },
    { 0xd29ff765, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x61ff768, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x685ff76f, SUC_lexicon_tags_11 },
    { 0x76dff770, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0x39fff773, SUC_lexicon_tags_17 },
    { 0xb2bff773, SUC_lexicon_tags_43 },
    { 0xf0dff774, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x50bff77f, SUC_lexicon_tags_12 },
    { 0xaa5ff780, SUC_lexicon_tags_8 },
    { 0xb41ff781, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdabff787, SUC_lexicon_tags_29 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3ff792, SUC_lexicon_tags_61 },
    { 0x583ff793, SUC_lexicon_tags_13 },
    { 0xa27ff794, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x79dff798, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xad7ff79b, SUC_lexicon_tags_15 },
    { 0x33bff79b, SUC_lexicon_tags_19 },
    { 0xa7fff79d, SUC_lexicon_tags_7 },
    { 0xfb5ff79c, SUC_lexicon_tags_25 },
    { 0xe07ff79c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf23ff7a4, SUC_lexicon_tags_15 },
    { 0xcdfff7a5, SUC_lexicon_tags_37 },
    { 0xd85ff7a4, SUC_lexicon_tags_7 },
    { 0x7fff7a7, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0xab5ff7aa, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x395ff7ac, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ff7b0, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb8dff7b4, SUC_lexicon_tags_23 },
    { 0xcd3ff7b4, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaa3ff7b7, SUC_lexicon_tags_34 },
    { 0x2e1ff7b7, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x6b1ff7ba, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfff7c2, SUC_lexicon_tags_4 },
    { 0xa8dff7c3, SUC_lexicon_tags_0 },
    { 0x68bff7c4, SUC_lexicon_tags_3 },
    { 0xf67ff7c4, SUC_lexicon_tags_12 },
    { 0x4c3ff7c4, SUC_lexicon_tags_11 },
    { 0x24dff7c7, SUC_lexicon_tags_46 },
    { 0x48fff7c8, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x35ff7d1, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xed1ff7d3, SUC_lexicon_tags_30 },
    { 0x719ff7d3, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x921ff7d7, SUC_lexicon_tags_7 },
    { 0x2c3ff7d7, SUC_lexicon_tags_23 },
    { 0xe61ff7d9, SUC_lexicon_tags_53 },
    { 0xe57ff7da, SUC_lexicon_tags_12 },
    { 0x2edff7da, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0xe73ff7dd, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xaafff7df, SUC_lexicon_tags_14 },
    { 0x9ff7df, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaa3ff7e7, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x61dff7e9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x651ff7ec, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bff7ef, SUC_lexicon_tags_0 },
    { 0x9a7ff7ef, SUC_lexicon_tags_68 },
    { 0x76dff7f0, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd5dff7f4, SUC_lexicon_tags_16 },
    { 0x501ff7f5, SUC_lexicon_tags_47 },
    { 0x163ff7f6, SUC_lexicon_tags_664 },
    { 0, NULL },
    { 0x4cbff7f8, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa3ff7fa, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x55ff7fc, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x205ff7ff, SUC_lexicon_tags_10 },
    { 0x367ff800, SUC_lexicon_tags_1 },
    { 0xd57ff801, SUC_lexicon_tags_43 },
    { 0xc41ff802, SUC_lexicon_tags_12 },
    { 0xd2bff803, SUC_lexicon_tags_59 },
    { 0, NULL },
    { 0xc67ff805, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x757ff807, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0xfa7ff809, SUC_lexicon_tags_11 },
    { 0xc9ff80a, SUC_lexicon_tags_12 },
    { 0x467ff80b, SUC_lexicon_tags_8 },
    { 0x8b1ff809, SUC_lexicon_tags_25 },
    { 0xfefff80d, SUC_lexicon_tags_30 },
    { 0x5b1ff80d, SUC_lexicon_tags_37 },
    { 0xeddff80f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x60fff812, SUC_lexicon_tags_16 },
    { 0xaa7ff813, SUC_lexicon_tags_1 },
    { 0x47fff814, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x627ff817, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x391ff819, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xaabff81d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x117ff827, SUC_lexicon_tags_63 },
    { 0x87ff828, SUC_lexicon_tags_15 },
    { 0x9a5ff829, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0xd79ff82b, SUC_lexicon_tags_37 },
    { 0x129ff82c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xb5bff82f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xd9bff832, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd1bff838, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xf7bff83a, SUC_lexicon_tags_37 },
    { 0xcebff83a, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x95fff843, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd8bff848, SUC_lexicon_tags_22 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9dbff853, SUC_lexicon_tags_21 },
    { 0xe41ff853, SUC_lexicon_tags_38 },
    { 0xa85ff854, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x17dff857, SUC_lexicon_tags_17 },
    { 0xe23ff857, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9c5ff85f, SUC_lexicon_tags_10 },
    { 0xf01ff860, SUC_lexicon_tags_7 },
    { 0x545ff861, SUC_lexicon_tags_1 },
    { 0xe63ff862, SUC_lexicon_tags_35 },
    { 0xb53ff862, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xc59ff866, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf53ff86b, SUC_lexicon_tags_7 },
    { 0x383ff86c, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x6fff86f, SUC_lexicon_tags_17 },
    { 0x479ff870, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x98bff872, SUC_lexicon_tags_31 },
    { 0x175ff873, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xcc3ff875, SUC_lexicon_tags_44 },
    { 0xe4dff876, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f3ff87b, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc65ff883, SUC_lexicon_tags_49 },
    { 0x783ff883, SUC_lexicon_tags_12 },
    { 0x505ff885, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899ff88c, SUC_lexicon_tags_82 },
    { 0xeebff88d, SUC_lexicon_tags_31 },
    { 0x177ff88e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf73ff896, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xc47ff899, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xe7dff89b, SUC_lexicon_tags_17 },
    { 0x2a9ff89b, SUC_lexicon_tags_29 },
    { 0xa8dff89b, SUC_lexicon_tags_38 },
    { 0x973ff89b, SUC_lexicon_tags_37 },
    { 0x46fff89e, SUC_lexicon_tags_10 },
    { 0x545ff89f, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x219ff8a3, SUC_lexicon_tags_31 },
    { 0x565ff8a3, SUC_lexicon_tags_60 },
    { 0xb57ff8a5, SUC_lexicon_tags_38 },
    { 0x419ff8a6, SUC_lexicon_tags_15 },
    { 0xdf5ff8a6, SUC_lexicon_tags_11 },
    { 0xeafff8a5, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x221ff8b1, SUC_lexicon_tags_11 },
    { 0x8bff8b2, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe19ff8b4, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0x631ff8b6, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7ff8b9, SUC_lexicon_tags_7 },
    { 0x1c1ff8ba, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0xb83ff8bd, SUC_lexicon_tags_45 },
    { 0x6dfff8be, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x5ebff8c1, SUC_lexicon_tags_37 },
    { 0x75dff8c1, SUC_lexicon_tags_100 },
    { 0x5bfff8c2, SUC_lexicon_tags_15 },
    { 0xe91ff8c3, SUC_lexicon_tags_21 },
    { 0x701ff8c5, SUC_lexicon_tags_23 },
    { 0x9cbff8c6, SUC_lexicon_tags_11 },
    { 0x543ff8c5, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd73ff8d4, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xe9ff8d6, SUC_lexicon_tags_0 },
    { 0xc4fff8d7, SUC_lexicon_tags_37 },
    { 0x65bff8d8, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0xe1ff8da, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x29bff8dc, SUC_lexicon_tags_106 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5ff8e4, SUC_lexicon_tags_19 },
    { 0, NULL },
    { 0xc55ff8e6, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xbebff8e8, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd7ff8ec, SUC_lexicon_tags_9 },
    { 0x675ff8ed, SUC_lexicon_tags_29 },
    { 0x483ff8ed, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bff8f1, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0xe69ff8f3, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0xf01ff8f5, SUC_lexicon_tags_30 },
    { 0x361ff8f5, SUC_lexicon_tags_14 },
    { 0x349ff8f7, SUC_lexicon_tags_16 },
    { 0x877ff8f8, SUC_lexicon_tags_12 },
    { 0x51bff8f9, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeedff8fe, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa01ff907, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x513ff90a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x43fff90e, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c9ff912, SUC_lexicon_tags_56 },
    { 0, NULL },
    { 0x889ff914, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd23ff91f, SUC_lexicon_tags_37 },
    { 0x9abff91f, SUC_lexicon_tags_37 },
    { 0x257ff921, SUC_lexicon_tags_12 },
    { 0xb7fff922, SUC_lexicon_tags_11 },
    { 0x89dff922, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x199ff925, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0xcefff928, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63bff92f, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x637ff931, SUC_lexicon_tags_66 },
    { 0, NULL },
    { 0, NULL },
    { 0xa87ff934, SUC_lexicon_tags_27 },
    { 0x297ff934, SUC_lexicon_tags_2 },
    { 0xc0bff936, SUC_lexicon_tags_37 },
    { 0x43ff935, SUC_lexicon_tags_761 },
    { 0xc43ff938, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x17bff93d, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dff944, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x4c9ff94d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x503ff951, SUC_lexicon_tags_37 },
    { 0x20dff952, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0xf05ff955, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8bff958, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x209ff95e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x19bff960, SUC_lexicon_tags_26 },
    { 0x1abff960, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xa9dff964, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xcdbff967, SUC_lexicon_tags_0 },
    { 0x169ff968, SUC_lexicon_tags_39 },
    { 0xf33ff968, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x333ff96e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa83ff975, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xdadff978, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x367ff97b, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x47fff97e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xefdff981, SUC_lexicon_tags_19 },
    { 0x847ff982, SUC_lexicon_tags_28 },
    { 0x8cdff983, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa1dff98e, SUC_lexicon_tags_15 },
    { 0x9fdff98e, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x823ff994, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xaa1ff996, SUC_lexicon_tags_21 },
    { 0x5e9ff997, SUC_lexicon_tags_47 },
    { 0xccfff997, SUC_lexicon_tags_7 },
    { 0xe51ff998, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x625ff99b, SUC_lexicon_tags_13 },
    { 0x191ff99c, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x613ff99e, SUC_lexicon_tags_15 },
    { 0xd33ff99f, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x6e5ff9a1, SUC_lexicon_tags_25 },
    { 0xddff9a1, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf39ff9a7, SUC_lexicon_tags_31 },
    { 0xc85ff9a8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x469ff9ab, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x3fdff9ad, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x257ff9af, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951ff9b3, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x9c3ff9b5, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x157ff9bd, SUC_lexicon_tags_1 },
    { 0x8e7ff9bd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xa9fff9c0, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x979ff9c2, SUC_lexicon_tags_144 },
    { 0xcdff9c3, SUC_lexicon_tags_19 },
    { 0x8abff9c2, SUC_lexicon_tags_1 },
    { 0x6abff9c4, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb73ff9c7, SUC_lexicon_tags_33 },
    { 0x877ff9c8, SUC_lexicon_tags_11 },
    { 0x917ff9c9, SUC_lexicon_tags_19 },
    { 0xf11ff9ca, SUC_lexicon_tags_2 },
    { 0x701ff9c9, SUC_lexicon_tags_17 },
    { 0xbe9ff9cc, SUC_lexicon_tags_4 },
    { 0xc41ff9c7, SUC_lexicon_tags_28 },
    { 0xdc1ff9ce, SUC_lexicon_tags_47 },
    { 0x19bff9cf, SUC_lexicon_tags_68 },
    { 0xc23ff9cf, SUC_lexicon_tags_12 },
    { 0x5dfff9d1, SUC_lexicon_tags_29 },
    { 0x955ff9c8, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0xbddff9d5, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x2d9ff9d8, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbff9e0, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xe71ff9e2, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x7fbff9e5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0x7bff9e7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1f9ff9ed, SUC_lexicon_tags_38 },
    { 0x6bbff9ee, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0x6cfff9f1, SUC_lexicon_tags_63 },
    { 0x83bff9f2, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xfffff9f4, SUC_lexicon_tags_15 },
    { 0xa15ff9f5, SUC_lexicon_tags_28 },
    { 0x7a9ff9f6, SUC_lexicon_tags_12 },
    { 0xbfdff9f7, SUC_lexicon_tags_1 },
    { 0xddfff9f8, SUC_lexicon_tags_17 },
    { 0xb93ff9f9, SUC_lexicon_tags_43 },
    { 0xb7dff9f5, SUC_lexicon_tags_9 },
    { 0x875ff9fb, SUC_lexicon_tags_45 },
    { 0x7b9ff9fb, SUC_lexicon_tags_7 },
    { 0xaffff9fd, SUC_lexicon_tags_38 },
    { 0xc4bff9fd, SUC_lexicon_tags_84 },
    { 0xa63ff9ff, SUC_lexicon_tags_17 },
    { 0x177ff9fd, SUC_lexicon_tags_2 },
    { 0x2d5ffa01, SUC_lexicon_tags_25 },
    { 0xa6fffa01, SUC_lexicon_tags_108 },
    { 0, NULL },
    { 0, NULL },
    { 0xf21ffa05, SUC_lexicon_tags_60 },
    { 0x157ffa06, SUC_lexicon_tags_8 },
    { 0x701ffa06, SUC_lexicon_tags_46 },
    { 0x171ffa06, SUC_lexicon_tags_16 },
    { 0xa5dffa09, SUC_lexicon_tags_48 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x77bffa0e, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0xcf7ffa11, SUC_lexicon_tags_7 },
    { 0x2f3ffa11, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x743ffa14, SUC_lexicon_tags_12 },
    { 0x413ffa14, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0x6ebffa18, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0x907ffa1b, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0x81bffa1e, SUC_lexicon_tags_29 },
    { 0x67bffa1f, SUC_lexicon_tags_43 },
    { 0x2afffa20, SUC_lexicon_tags_30 },
    { 0x2a9ffa1e, SUC_lexicon_tags_37 },
    { 0x5e5ffa21, SUC_lexicon_tags_12 },
    { 0x709ffa23, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xa4dffa25, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc29ffa2a, SUC_lexicon_tags_160 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xeafffa31, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x64bffa33, SUC_lexicon_tags_10 },
    { 0x5c3ffa34, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0xdffa36, SUC_lexicon_tags_92 },
    { 0, NULL },
    { 0x455ffa38, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x3ffffa3a, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc3bffa3f, SUC_lexicon_tags_31 },
    { 0xdfffa40, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1a7ffa47, SUC_lexicon_tags_14 },
    { 0xb2fffa47, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x4cbffa4a, SUC_lexicon_tags_11 },
    { 0x63bffa4a, SUC_lexicon_tags_17 },
    { 0x1f9ffa4a, SUC_lexicon_tags_12 },
    { 0xa7fffa4a, SUC_lexicon_tags_37 },
    { 0xf0fffa4e, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x3ebffa50, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfffa56, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x851ffa5a, SUC_lexicon_tags_12 },
    { 0xc5bffa5b, SUC_lexicon_tags_49 },
    { 0x7c5ffa5c, SUC_lexicon_tags_28 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x275ffa68, SUC_lexicon_tags_38 },
    { 0x601ffa69, SUC_lexicon_tags_66 },
    { 0xb37ffa6a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1cdffa6f, SUC_lexicon_tags_25 },
    { 0x831ffa70, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x99bffa76, SUC_lexicon_tags_35 },
    { 0xc63ffa77, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x443ffa79, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x72dffa82, SUC_lexicon_tags_165 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf1fffa86, SUC_lexicon_tags_11 },
    { 0x8d1ffa87, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfbfffa8b, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x211ffa8d, SUC_lexicon_tags_14 },
    { 0xaf3ffa8d, SUC_lexicon_tags_43 },
    { 0xe47ffa8f, SUC_lexicon_tags_62 },
    { 0xe2fffa8e, SUC_lexicon_tags_35 },
    { 0x4fffa91, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x523ffa96, SUC_lexicon_tags_12 },
    { 0x239ffa96, SUC_lexicon_tags_7 },
    { 0x9fffa98, SUC_lexicon_tags_10 },
    { 0x559ffa99, SUC_lexicon_tags_9 },
    { 0x51bffa9a, SUC_lexicon_tags_15 },
    { 0x7b9ffa9a, SUC_lexicon_tags_2 },
    { 0xf73ffa9c, SUC_lexicon_tags_1142 },
    { 0x8bffa9b, SUC_lexicon_tags_39 },
    { 0x5f3ffa9e, SUC_lexicon_tags_35 },
    { 0x34bffa9e, SUC_lexicon_tags_33 },
    { 0xf5fffaa0, SUC_lexicon_tags_2 },
    { 0x37bffaa0, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0, NULL },
    { 0xed7ffaa4, SUC_lexicon_tags_35 },
    { 0x56fffaa5, SUC_lexicon_tags_62 },
    { 0, NULL },
    { 0x42fffaa7, SUC_lexicon_tags_27 },
    { 0xfb3ffaa8, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0xbd5ffaaa, SUC_lexicon_tags_37 },
    { 0x8cbffaab, SUC_lexicon_tags_55 },
    { 0x797ffaab, SUC_lexicon_tags_21 },
    { 0x279ffaaa, SUC_lexicon_tags_23 },
    { 0x4cdffaad, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x933ffab4, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x509ffab8, SUC_lexicon_tags_45 },
    { 0x76dffab8, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x665ffabb, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xdfbffabd, SUC_lexicon_tags_70 },
    { 0xf59ffabd, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dffac1, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x7c7ffac4, SUC_lexicon_tags_60 },
    { 0xba3ffac5, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0x225ffac8, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0x521ffacb, SUC_lexicon_tags_49 },
    { 0x3d3ffacc, SUC_lexicon_tags_37 },
    { 0x2ffffacd, SUC_lexicon_tags_15 },
    { 0x4ddfface, SUC_lexicon_tags_11 },
    { 0xfe3ffacc, SUC_lexicon_tags_82 },
    { 0xc5bffad0, SUC_lexicon_tags_13 },
    { 0xad7ffad1, SUC_lexicon_tags_1 },
    { 0x163ffad2, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x69bffad4, SUC_lexicon_tags_0 },
    { 0xff5ffad5, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xddfffad9, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x4d5ffadb, SUC_lexicon_tags_47 },
    { 0x77bffadc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x9e9ffade, SUC_lexicon_tags_75 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x63fffae3, SUC_lexicon_tags_17 },
    { 0x61bffae4, SUC_lexicon_tags_45 },
    { 0x9fffae5, SUC_lexicon_tags_15 },
    { 0x923ffae5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xce5ffaea, SUC_lexicon_tags_11 },
    { 0x227ffaea, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e7ffaf7, SUC_lexicon_tags_44 },
    { 0, NULL },
    { 0x80fffaf9, SUC_lexicon_tags_8 },
    { 0x65fffaf9, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x825ffaff, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdddffb06, SUC_lexicon_tags_37 },
    { 0xf5ffb06, SUC_lexicon_tags_19 },
    { 0xcd3ffb06, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x557ffb0b, SUC_lexicon_tags_23 },
    { 0x895ffb0c, SUC_lexicon_tags_12 },
    { 0xf7ffb0d, SUC_lexicon_tags_17 },
    { 0x4bffb0d, SUC_lexicon_tags_11 },
    { 0x99fffb0f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xabfffb13, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0xfd3ffb15, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5e5ffb1c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x905ffb23, SUC_lexicon_tags_71 },
    { 0xc13ffb23, SUC_lexicon_tags_43 },
    { 0x6a9ffb25, SUC_lexicon_tags_28 },
    { 0x28fffb26, SUC_lexicon_tags_11 },
    { 0x26fffb26, SUC_lexicon_tags_46 },
    { 0x611ffb28, SUC_lexicon_tags_14 },
    { 0x8a7ffb29, SUC_lexicon_tags_126 },
    { 0x53ffb2a, SUC_lexicon_tags_1 },
    { 0x61dffb2b, SUC_lexicon_tags_47 },
    { 0x437ffb2c, SUC_lexicon_tags_0 },
    { 0x601ffb23, SUC_lexicon_tags_0 },
    { 0x8dfffb2e, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1c1ffb33, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0xc99ffb36, SUC_lexicon_tags_37 },
    { 0x4fffb37, SUC_lexicon_tags_17 },
    { 0xa1bffb38, SUC_lexicon_tags_1 },
    { 0xbb7ffb37, SUC_lexicon_tags_66 },
    { 0x40fffb3a, SUC_lexicon_tags_9 },
    { 0x207ffb3b, SUC_lexicon_tags_47 },
    { 0x3cfffb3c, SUC_lexicon_tags_172 },
    { 0x641ffb38, SUC_lexicon_tags_26 },
    { 0x4efffb3e, SUC_lexicon_tags_17 },
    { 0x2e1ffb3e, SUC_lexicon_tags_12 },
    { 0xc33ffb40, SUC_lexicon_tags_37 },
    { 0x8dfffb41, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xc63ffb44, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x61dffb4a, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x5fffb4c, SUC_lexicon_tags_25 },
    { 0x9bbffb4d, SUC_lexicon_tags_37 },
    { 0xb57ffb4e, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x649ffb51, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x5a7ffb53, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0xb9ffb55, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x13bffb5f, SUC_lexicon_tags_7 },
    { 0x70bffb60, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x631ffb63, SUC_lexicon_tags_15 },
    { 0x4f5ffb63, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0xbe1ffb66, SUC_lexicon_tags_37 },
    { 0x24fffb66, SUC_lexicon_tags_11 },
    { 0x167ffb68, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xf07ffb6a, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x7e1ffb6c, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x1d1ffb6f, SUC_lexicon_tags_66 },
    { 0x4c5ffb70, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x50dffb73, SUC_lexicon_tags_27 },
    { 0x32fffb74, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbd5ffb78, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe9bffb7c, SUC_lexicon_tags_1 },
    { 0x171ffb7d, SUC_lexicon_tags_14 },
    { 0x311ffb7d, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0xcebffb81, SUC_lexicon_tags_65 },
    { 0, NULL },
    { 0, NULL },
    { 0xe25ffb84, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x987ffb8b, SUC_lexicon_tags_43 },
    { 0xd09ffb8c, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c7ffb93, SUC_lexicon_tags_8 },
    { 0xc7fffb93, SUC_lexicon_tags_60 },
    { 0x6adffb95, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfffb99, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0xef7ffb9b, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x2cdffb9d, SUC_lexicon_tags_37 },
    { 0xe33ffb9e, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x179ffba1, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0x80fffba4, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdffba7, SUC_lexicon_tags_2 },
    { 0x84bffba8, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xc45ffbab, SUC_lexicon_tags_17 },
    { 0x17fffbac, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0x37ffbae, SUC_lexicon_tags_60 },
    { 0x963ffbae, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xac3ffbb1, SUC_lexicon_tags_25 },
    { 0x785ffbb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8f7ffbb8, SUC_lexicon_tags_7 },
    { 0x263ffbb9, SUC_lexicon_tags_62 },
    { 0xd79ffbba, SUC_lexicon_tags_0 },
    { 0xacdffbba, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x2bdffbbd, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x963ffbc1, SUC_lexicon_tags_17 },
    { 0x1abffbc1, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x61ffbc4, SUC_lexicon_tags_2 },
    { 0x3d3ffbc5, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2c3ffbd0, SUC_lexicon_tags_87 },
    { 0x429ffbd0, SUC_lexicon_tags_56 },
    { 0xf79ffbd0, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xcd7ffbd4, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xa71ffbd7, SUC_lexicon_tags_26 },
    { 0x6f1ffbd7, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x75dffbdc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xf05ffbde, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7dffbe2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x481ffbe4, SUC_lexicon_tags_5 },
    { 0xb4fffbe5, SUC_lexicon_tags_37 },
    { 0x5c5ffbe6, SUC_lexicon_tags_33 },
    { 0x911ffbe6, SUC_lexicon_tags_15 },
    { 0xeefffbe8, SUC_lexicon_tags_14 },
    { 0xc25ffbe7, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x70bffbef, SUC_lexicon_tags_25 },
    { 0xaa7ffbef, SUC_lexicon_tags_15 },
    { 0x6d3ffbf1, SUC_lexicon_tags_60 },
    { 0xeedffbf2, SUC_lexicon_tags_13 },
    { 0, NULL },
    { 0x14dffbf4, SUC_lexicon_tags_59 },
    { 0x7ffbf5, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe71ffbf9, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x305ffbfc, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xe87ffbfe, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7bffc01, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0xe43ffc03, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x549ffc05, SUC_lexicon_tags_19 },
    { 0xd23ffc05, SUC_lexicon_tags_42 },
    { 0x6f3ffc07, SUC_lexicon_tags_19 },
    { 0xa0fffc08, SUC_lexicon_tags_11 },
    { 0x8c9ffc09, SUC_lexicon_tags_9 },
    { 0x4ebffc0a, SUC_lexicon_tags_7 },
    { 0xf65ffc0b, SUC_lexicon_tags_12 },
    { 0x8bbffc0b, SUC_lexicon_tags_2 },
    { 0xc3bffc0d, SUC_lexicon_tags_2 },
    { 0xad7ffc0a, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x18dffc14, SUC_lexicon_tags_11 },
    { 0xd83ffc15, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x2cdffc18, SUC_lexicon_tags_7 },
    { 0x71bffc19, SUC_lexicon_tags_4 },
    { 0x113ffc1a, SUC_lexicon_tags_1 },
    { 0x6cdffc1a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xfa1ffc1d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3ffc22, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x52dffc25, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x961ffc29, SUC_lexicon_tags_23 },
    { 0xe47ffc29, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xf0bffc2d, SUC_lexicon_tags_11 },
    { 0xd5ffc2e, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x45ffc32, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa7bffc3a, SUC_lexicon_tags_28 },
    { 0x3a5ffc3a, SUC_lexicon_tags_6 },
    { 0xe49ffc3c, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xccdffc42, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0x221ffc44, SUC_lexicon_tags_43 },
    { 0xf45ffc45, SUC_lexicon_tags_42 },
    { 0xd83ffc46, SUC_lexicon_tags_45 },
    { 0xd5ffc46, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x7d7ffc4c, SUC_lexicon_tags_46 },
    { 0x535ffc4d, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xf07ffc4f, SUC_lexicon_tags_40 },
    { 0x9d5ffc50, SUC_lexicon_tags_77 },
    { 0xe13ffc51, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x82fffc54, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x6f7ffc56, SUC_lexicon_tags_19 },
    { 0x77fffc56, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0xa5ffc59, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x863ffc5e, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdd5ffc62, SUC_lexicon_tags_46 },
    { 0xe37ffc63, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa3ffc67, SUC_lexicon_tags_12 },
    { 0xe61ffc68, SUC_lexicon_tags_60 },
    { 0, NULL },
    { 0x375ffc6a, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x51ffc6c, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x6a9ffc6f, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x42fffc71, SUC_lexicon_tags_64 },
    { 0, NULL },
    { 0, NULL },
    { 0x72dffc74, SUC_lexicon_tags_63 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa85ffc78, SUC_lexicon_tags_15 },
    { 0x6cbffc78, SUC_lexicon_tags_14 },
    { 0xd89ffc7a, SUC_lexicon_tags_36 },
    { 0x8dbffc7a, SUC_lexicon_tags_11 },
    { 0xe69ffc7c, SUC_lexicon_tags_1 },
    { 0x29ffc7d, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0xcd1ffc7f, SUC_lexicon_tags_38 },
    { 0xc6bffc80, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfd9ffc87, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb7dffc8c, SUC_lexicon_tags_21 },
    { 0x1a7ffc8c, SUC_lexicon_tags_37 },
    { 0xf3ffc8e, SUC_lexicon_tags_37 },
    { 0xd93ffc8f, SUC_lexicon_tags_20 },
    { 0x481ffc8f, SUC_lexicon_tags_38 },
    { 0x3fdffc8f, SUC_lexicon_tags_7 },
    { 0x87ffc92, SUC_lexicon_tags_47 },
    { 0xf0bffc8d, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0xd71ffc95, SUC_lexicon_tags_7 },
    { 0x8f5ffc96, SUC_lexicon_tags_25 },
    { 0xd3bffc97, SUC_lexicon_tags_82 },
    { 0x111ffc97, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x805ffc9b, SUC_lexicon_tags_10 },
    { 0xc8bffc9c, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0x73bffc9f, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xdb1ffca1, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x48dffca3, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbbbffca9, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0x3ffffcab, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x1a9ffcad, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x527ffcaf, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0xdefffcb2, SUC_lexicon_tags_60 },
    { 0x4c1ffcb3, SUC_lexicon_tags_23 },
    { 0x471ffcb4, SUC_lexicon_tags_23 },
    { 0x693ffcb5, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xd11ffcb7, SUC_lexicon_tags_34 },
    { 0, NULL },
    { 0x635ffcb9, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0xf8fffcbc, SUC_lexicon_tags_17 },
    { 0x217ffcbd, SUC_lexicon_tags_47 },
    { 0xc6fffcbd, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5edffcc2, SUC_lexicon_tags_45 },
    { 0xefbffcc3, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x805ffccc, SUC_lexicon_tags_15 },
    { 0xc95ffccd, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xe69ffccf, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x757ffcd1, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5f5ffcdb, SUC_lexicon_tags_37 },
    { 0xa7fffcdc, SUC_lexicon_tags_91 },
    { 0, NULL },
    { 0x319ffcde, SUC_lexicon_tags_61 },
    { 0xdfdffcde, SUC_lexicon_tags_12 },
    { 0x6ebffce0, SUC_lexicon_tags_1 },
    { 0x717ffcde, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x547ffce9, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0xfb9ffceb, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9b1ffced, SUC_lexicon_tags_12 },
    { 0xe03ffced, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x485ffcf1, SUC_lexicon_tags_66 },
    { 0x619ffcf2, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0x1fdffcf5, SUC_lexicon_tags_3 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3c5ffcfb, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xd7bffcff, SUC_lexicon_tags_77 },
    { 0xc1ffd00, SUC_lexicon_tags_25 },
    { 0x4a1ffd01, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x811ffd03, SUC_lexicon_tags_17 },
    { 0x25ffd03, SUC_lexicon_tags_47 },
    { 0x3e5ffd03, SUC_lexicon_tags_38 },
    { 0xdf1ffd06, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xa5ffd0f, SUC_lexicon_tags_2 },
    { 0x8fdffd10, SUC_lexicon_tags_12 },
    { 0x715ffd11, SUC_lexicon_tags_37 },
    { 0x20fffd12, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x241ffd16, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe67ffd1a, SUC_lexicon_tags_11 },
    { 0x34fffd1b, SUC_lexicon_tags_28 },
    { 0xf3bffd1a, SUC_lexicon_tags_11 },
    { 0xefdffd1d, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x951ffd24, SUC_lexicon_tags_9 },
    { 0xba7ffd25, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0x9a1ffd27, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xe13ffd29, SUC_lexicon_tags_11 },
    { 0xdb7ffd29, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x337ffd31, SUC_lexicon_tags_2 },
    { 0x353ffd32, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0xa35ffd35, SUC_lexicon_tags_31 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x8d7ffd40, SUC_lexicon_tags_25 },
    { 0x1b9ffd41, SUC_lexicon_tags_7 },
    { 0xff3ffd41, SUC_lexicon_tags_38 },
    { 0xfe5ffd40, SUC_lexicon_tags_26 },
    { 0x95dffd44, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6ddffd4a, SUC_lexicon_tags_7 },
    { 0xf8bffd4a, SUC_lexicon_tags_55 },
    { 0, NULL },
    { 0x60fffd4d, SUC_lexicon_tags_63 },
    { 0x4d9ffd4e, SUC_lexicon_tags_15 },
    { 0x5a3ffd4d, SUC_lexicon_tags_26 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5cdffd5b, SUC_lexicon_tags_65 },
    { 0xe1ffd5c, SUC_lexicon_tags_37 },
    { 0xd49ffd5c, SUC_lexicon_tags_10 },
    { 0x5fffd5e, SUC_lexicon_tags_49 },
    { 0xccbffd5f, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0xd35ffd61, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0x9fbffd64, SUC_lexicon_tags_104 },
    { 0xde9ffd65, SUC_lexicon_tags_11 },
    { 0xd2dffd66, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82fffd72, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0xc7ffd75, SUC_lexicon_tags_68 },
    { 0xb55ffd76, SUC_lexicon_tags_25 },
    { 0x9fbffd77, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x623ffd7a, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf5fffd82, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0x16bffd85, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x917ffd87, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0x55dffd89, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xe15ffd8b, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5b3ffd90, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x711ffd93, SUC_lexicon_tags_1 },
    { 0x575ffd94, SUC_lexicon_tags_15 },
    { 0xf7fffd95, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0x3a3ffd97, SUC_lexicon_tags_127 },
    { 0x8d5ffd97, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0x281ffd9b, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0xedfffd9e, SUC_lexicon_tags_33 },
    { 0x4cdffd9f, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x341ffda2, SUC_lexicon_tags_28 },
    { 0x681ffda2, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x47ffda5, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa75ffda7, SUC_lexicon_tags_25 },
    { 0x60bffda8, SUC_lexicon_tags_11 },
    { 0x6cfffda9, SUC_lexicon_tags_70 },
    { 0x77bffdaa, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0x581ffdac, SUC_lexicon_tags_63 },
    { 0xddffdad, SUC_lexicon_tags_198 },
    { 0x697ffdae, SUC_lexicon_tags_46 },
    { 0x25dffdae, SUC_lexicon_tags_0 },
    { 0xd15ffdb0, SUC_lexicon_tags_2 },
    { 0xeabffdb1, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xed9ffdbc, SUC_lexicon_tags_12 },
    { 0x841ffdbc, SUC_lexicon_tags_1 },
    { 0xee1ffdbe, SUC_lexicon_tags_70 },
    { 0x4cdffdbf, SUC_lexicon_tags_1 },
    { 0xf81ffdbc, SUC_lexicon_tags_43 },
    { 0x2afffdc1, SUC_lexicon_tags_23 },
    { 0x2ebffdbe, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xce1ffdc5, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0xd81ffdc8, SUC_lexicon_tags_68 },
    { 0, NULL },
    { 0, NULL },
    { 0xfd1ffdcb, SUC_lexicon_tags_28 },
    { 0x7c3ffdcc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x81fffdd0, SUC_lexicon_tags_11 },
    { 0x1b5ffdd0, SUC_lexicon_tags_60 },
    { 0x981ffdd2, SUC_lexicon_tags_14 },
    { 0xcefffdd2, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xbffffdd7, SUC_lexicon_tags_4 },
    { 0x2b5ffdd8, SUC_lexicon_tags_1 },
    { 0xe37ffdd7, SUC_lexicon_tags_60 },
    { 0xef5ffdda, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0xe45ffddc, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0x443ffddf, SUC_lexicon_tags_1 },
    { 0x253ffddf, SUC_lexicon_tags_108 },
    { 0xe91ffddf, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0x211ffde3, SUC_lexicon_tags_8 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6c3ffde9, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0xfc5ffdec, SUC_lexicon_tags_17 },
    { 0x88bffded, SUC_lexicon_tags_34 },
    { 0xc1dffdee, SUC_lexicon_tags_11 },
    { 0x875ffdef, SUC_lexicon_tags_27 },
    { 0xcb9ffdef, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0x6cbffdf2, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0xa51ffdf4, SUC_lexicon_tags_61 },
    { 0, NULL },
    { 0x835ffdf6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xeb5ffdf8, SUC_lexicon_tags_31 },
    { 0x56bffdf9, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x3bffdfd, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0xd43ffe00, SUC_lexicon_tags_929 },
    { 0xf35ffe01, SUC_lexicon_tags_37 },
    { 0x3cfffe02, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb91ffe04, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x2cfffe0a, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x607ffe0e, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0x289ffe10, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x1f3ffe13, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdbffe19, SUC_lexicon_tags_45 },
    { 0, NULL },
    { 0xb85ffe1b, SUC_lexicon_tags_0 },
    { 0xcfdffe1b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0xa1fffe1f, SUC_lexicon_tags_2 },
    { 0x7a5ffe20, SUC_lexicon_tags_11 },
    { 0x383ffe20, SUC_lexicon_tags_14 },
    { 0xecdffe21, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc5dffe26, SUC_lexicon_tags_15 },
    { 0x573ffe27, SUC_lexicon_tags_68 },
    { 0x5ddffe28, SUC_lexicon_tags_15 },
    { 0x8bfffe29, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0x50bffe2b, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcd3ffe30, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x82dffe34, SUC_lexicon_tags_31 },
    { 0x7d3ffe35, SUC_lexicon_tags_15 },
    { 0x26fffe35, SUC_lexicon_tags_15 },
    { 0x79ffe37, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x321ffe3b, SUC_lexicon_tags_45 },
    { 0x28bffe3c, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x195ffe41, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9efffe45, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0x2fffe47, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcf5ffe4d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0x85fffe4f, SUC_lexicon_tags_31 },
    { 0xf81ffe50, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x11fffe53, SUC_lexicon_tags_26 },
    { 0xdfdffe54, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x899ffe5d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0x263ffe5f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x64fffe64, SUC_lexicon_tags_12 },
    { 0x3bbffe64, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x93dffe69, SUC_lexicon_tags_0 },
    { 0xc63ffe6a, SUC_lexicon_tags_25 },
    { 0xf3ffe6b, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0x3fdffe6d, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0x661ffe70, SUC_lexicon_tags_47 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x641ffe75, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0xdb1ffe77, SUC_lexicon_tags_11 },
    { 0xf1fffe77, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0xa5dffe7a, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x5adffe7d, SUC_lexicon_tags_12 },
    { 0xd4bffe7e, SUC_lexicon_tags_2 },
    { 0xf7fffe7f, SUC_lexicon_tags_32 },
    { 0xcfffe80, SUC_lexicon_tags_84 },
    { 0, NULL },
    { 0x51bffe82, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x859ffe86, SUC_lexicon_tags_1 },
    { 0xe79ffe87, SUC_lexicon_tags_46 },
    { 0x7bbffe86, SUC_lexicon_tags_4 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x69fffe8e, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0x6a3ffe90, SUC_lexicon_tags_33 },
    { 0xa65ffe91, SUC_lexicon_tags_82 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x49fffe9a, SUC_lexicon_tags_49 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e9ffe9f, SUC_lexicon_tags_14 },
    { 0x767ffe9f, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0, NULL },
    { 0x50fffea3, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0xa8dffea6, SUC_lexicon_tags_19 },
    { 0x56dffea7, SUC_lexicon_tags_32 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x27ffeae, SUC_lexicon_tags_57 },
    { 0xbc3ffeaf, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe0dffeb4, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6d3ffeb9, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6b9ffebf, SUC_lexicon_tags_0 },
    { 0x1a1ffec0, SUC_lexicon_tags_12 },
    { 0x677ffec1, SUC_lexicon_tags_81 },
    { 0x41dffebf, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0xb3bffec4, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x1ddffeca, SUC_lexicon_tags_3 },
    { 0x4d9ffecb, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5ddffed1, SUC_lexicon_tags_29 },
    { 0x89dffed2, SUC_lexicon_tags_19 },
    { 0x859ffed2, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0xe49ffed6, SUC_lexicon_tags_11 },
    { 0xf21ffed6, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x131ffee7, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc13ffeed, SUC_lexicon_tags_2 },
    { 0xf8dffeee, SUC_lexicon_tags_2 },
    { 0x8adffeef, SUC_lexicon_tags_8 },
    { 0x999ffef0, SUC_lexicon_tags_26 },
    { 0x247ffeef, SUC_lexicon_tags_12 },
    { 0xd51ffef0, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfddffef6, SUC_lexicon_tags_2 },
    { 0x3d3ffef7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0xdd9ffefa, SUC_lexicon_tags_27 },
    { 0x8a7ffefa, SUC_lexicon_tags_2 },
    { 0x539ffefc, SUC_lexicon_tags_17 },
    { 0xb9fffefd, SUC_lexicon_tags_38 },
    { 0x3cfffefb, SUC_lexicon_tags_33 },
    { 0x7c3ffeff, SUC_lexicon_tags_23 },
    { 0xf5ffefb, SUC_lexicon_tags_6 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x6bdfff04, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xa6ffff06, SUC_lexicon_tags_7 },
    { 0x2ffff07, SUC_lexicon_tags_46 },
    { 0x15ffff08, SUC_lexicon_tags_77 },
    { 0xe2dfff09, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81fff12, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0, NULL },
    { 0x4e1fff15, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0xf5ffff17, SUC_lexicon_tags_12 },
    { 0x633fff18, SUC_lexicon_tags_9 },
    { 0, NULL },
    { 0xfd9fff1a, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xf65fff1c, SUC_lexicon_tags_38 },
    { 0x31ffff1d, SUC_lexicon_tags_25 },
    { 0x3b1fff1d, SUC_lexicon_tags_14 },
    { 0, NULL },
    { 0, NULL },
    { 0x8adfff21, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0x393fff23, SUC_lexicon_tags_17 },
    { 0x3b5fff24, SUC_lexicon_tags_7 },
    { 0x813fff24, SUC_lexicon_tags_23 },
    { 0x9edfff26, SUC_lexicon_tags_17 },
    { 0xdc5fff27, SUC_lexicon_tags_25 },
    { 0x303fff25, SUC_lexicon_tags_1070 },
    { 0x25ffff23, SUC_lexicon_tags_31 },
    { 0x417fff2a, SUC_lexicon_tags_34 },
    { 0x2effff2a, SUC_lexicon_tags_77 },
    { 0x237fff2c, SUC_lexicon_tags_15 },
    { 0x4d1fff2b, SUC_lexicon_tags_15 },
    { 0xaf7fff2e, SUC_lexicon_tags_12 },
    { 0x15dfff27, SUC_lexicon_tags_23 },
    { 0x879fff23, SUC_lexicon_tags_17 },
    { 0x58bfff31, SUC_lexicon_tags_7 },
    { 0xd4dfff31, SUC_lexicon_tags_1 },
    { 0xb71fff33, SUC_lexicon_tags_17 },
    { 0xacdfff2e, SUC_lexicon_tags_15 },
    { 0x859fff35, SUC_lexicon_tags_3 },
    { 0xef9fff2b, SUC_lexicon_tags_23 },
    { 0x511fff37, SUC_lexicon_tags_1 },
    { 0x8fdfff38, SUC_lexicon_tags_11 },
    { 0x85fff36, SUC_lexicon_tags_23 },
    { 0, NULL },
    { 0, NULL },
    { 0xae5fff3c, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbeffff3f, SUC_lexicon_tags_66 },
    { 0x22dfff40, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0xf85fff43, SUC_lexicon_tags_15 },
    { 0xdb5fff44, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0xc3ffff46, SUC_lexicon_tags_23 },
    { 0xd77fff47, SUC_lexicon_tags_37 },
    { 0x891fff48, SUC_lexicon_tags_72 },
    { 0xc3bfff47, SUC_lexicon_tags_33 },
    { 0x26dfff49, SUC_lexicon_tags_12 },
    { 0xfabfff4b, SUC_lexicon_tags_37 },
    { 0xfc3fff4c, SUC_lexicon_tags_51 },
    { 0x8c9fff4d, SUC_lexicon_tags_7 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x5dffff56, SUC_lexicon_tags_23 },
    { 0x377fff57, SUC_lexicon_tags_43 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xab1fff5b, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xeb3fff5d, SUC_lexicon_tags_25 },
    { 0x739fff5e, SUC_lexicon_tags_17 },
    { 0xb7fff5f, SUC_lexicon_tags_9 },
    { 0x36dfff60, SUC_lexicon_tags_12 },
    { 0xbb7fff61, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x395fff63, SUC_lexicon_tags_33 },
    { 0xf09fff64, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x499fff67, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0x597fff6a, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0x8bfff6c, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xb61fff6e, SUC_lexicon_tags_16 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf81fff72, SUC_lexicon_tags_37 },
    { 0x9cffff73, SUC_lexicon_tags_25 },
    { 0x3adfff73, SUC_lexicon_tags_30 },
    { 0xe73fff73, SUC_lexicon_tags_49 },
    { 0xcaffff76, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0xd9dfff78, SUC_lexicon_tags_12 },
    { 0x79dfff79, SUC_lexicon_tags_63 },
    { 0x7bffff7a, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xf71fff80, SUC_lexicon_tags_37 },
    { 0x257fff81, SUC_lexicon_tags_46 },
    { 0, NULL },
    { 0x4cdfff83, SUC_lexicon_tags_31 },
    { 0x953fff83, SUC_lexicon_tags_25 },
    { 0xc5bfff85, SUC_lexicon_tags_11 },
    { 0xedffff85, SUC_lexicon_tags_87 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x38dfff8a, SUC_lexicon_tags_17 },
    { 0x765fff8a, SUC_lexicon_tags_77 },
    { 0, NULL },
    { 0, NULL },
    { 0x5bfff8e, SUC_lexicon_tags_0 },
    { 0x3bfff8f, SUC_lexicon_tags_21 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xb59fff94, SUC_lexicon_tags_37 },
    { 0xc11fff95, SUC_lexicon_tags_17 },
    { 0xd0bfff95, SUC_lexicon_tags_68 },
    { 0xadbfff97, SUC_lexicon_tags_6 },
    { 0x6bffff94, SUC_lexicon_tags_30 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x9e5fffa0, SUC_lexicon_tags_38 },
    { 0, NULL },
    { 0, NULL },
    { 0xc19fffa3, SUC_lexicon_tags_15 },
    { 0, NULL },
    { 0x65fffa5, SUC_lexicon_tags_15 },
    { 0xb37fffa5, SUC_lexicon_tags_68 },
    { 0x3fdfffa5, SUC_lexicon_tags_17 },
    { 0xfdffffa8, SUC_lexicon_tags_26 },
    { 0x3d1fffa9, SUC_lexicon_tags_8 },
    { 0xbc7fffa9, SUC_lexicon_tags_7 },
    { 0x783fffa8, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xe7fffaf, SUC_lexicon_tags_11 },
    { 0x431fffb0, SUC_lexicon_tags_27 },
    { 0, NULL },
    { 0x321fffb2, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xbfdfffb5, SUC_lexicon_tags_12 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0x21fffbb, SUC_lexicon_tags_28 },
    { 0x3a7fffbc, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0x29ffffbf, SUC_lexicon_tags_2 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xdc7fffc3, SUC_lexicon_tags_14 },
    { 0x1bbfffc4, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0xd35fffc7, SUC_lexicon_tags_11 },
    { 0, NULL },
    { 0x8a5fffc9, SUC_lexicon_tags_14 },
    { 0x73dfffca, SUC_lexicon_tags_5 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xcffffd5, SUC_lexicon_tags_38 },
    { 0x385fffd6, SUC_lexicon_tags_10 },
    { 0, NULL },
    { 0, NULL },
    { 0xdddfffd9, SUC_lexicon_tags_117 },
    { 0, NULL },
    { 0, NULL },
    { 0x14dfffdc, SUC_lexicon_tags_25 },
    { 0x68dfffdd, SUC_lexicon_tags_17 },
    { 0, NULL },
    { 0, NULL },
    { 0x9e3fffe0, SUC_lexicon_tags_2 },
    { 0x955fffe0, SUC_lexicon_tags_0 },
    { 0, NULL },
    { 0xb7bfffe3, SUC_lexicon_tags_38 },
    { 0x391fffe4, SUC_lexicon_tags_39 },
    { 0, NULL },
    { 0, NULL },
    { 0x973fffe7, SUC_lexicon_tags_37 },
    { 0xc53fffe7, SUC_lexicon_tags_33 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xfa5fffec, SUC_lexicon_tags_37 },
    { 0, NULL },
    { 0, NULL },
    { 0xea9fffef, SUC_lexicon_tags_25 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0xc9fffff3, SUC_lexicon_tags_23 },
    { 0x2c3ffff4, SUC_lexicon_tags_38 },
    { 0xb9bffff5, SUC_lexicon_tags_9 },
    { 0x615ffff6, SUC_lexicon_tags_11 },
    { 0x7bffff6, SUC_lexicon_tags_1 },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL },
    { 0, NULL }
};


static inline const label *SUC_lexicon_get_tags(uint32_t key) {
    size_t i = key & 0x1fffff;
    for (;;) {
        if (SUC_lexicon[i].hash == key) return (const label*)SUC_lexicon[i].value;
        if (SUC_lexicon[i].value == NULL) return SUC_lexicon_tags_open;
        i = (i + 1) & 0x1fffff;
    }
}

#define get_tags SUC_lexicon_get_tags
#define brown_size 0x100000

static const hash32_kv_label brown[brown_size] = {
    { 0x439fffff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7500004, 65 },
    { 0x86f00004, 33 },
    { 0x6c300006, 40 },
    { 0, 0 },
    { 0xc2300008, 90 },
    { 0xb5e00009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a800010, 61 },
    { 0, 0 },
    { 0x35300012, 78 },
    { 0x4ed00012, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b300018, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0001b, 90 },
    { 0xf2a0001c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d0002c, 33 },
    { 0, 0 },
    { 0xaa20002e, 55 },
    { 0x3d90002e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a300033, 90 },
    { 0x6be00034, 43 },
    { 0, 0 },
    { 0x62000036, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d0003a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6720003d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa600040, 33 },
    { 0xa6700041, 78 },
    { 0x9a000042, 91 },
    { 0, 0 },
    { 0xe6400044, 33 },
    { 0xb2000045, 33 },
    { 0x30600044, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0004a, 90 },
    { 0x2e60004b, 33 },
    { 0, 0 },
    { 0x7740004d, 68 },
    { 0, 0 },
    { 0xf590004f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a600052, 61 },
    { 0, 0 },
    { 0x2a900054, 23 },
    { 0x73d00054, 90 },
    { 0x7c900056, 33 },
    { 0x8d700056, 95 },
    { 0xd3300058, 90 },
    { 0x7bd00057, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140005d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4600060, 12 },
    { 0, 0 },
    { 0x73900062, 19 },
    { 0xee600063, 33 },
    { 0x8200064, 61 },
    { 0x70800065, 90 },
    { 0x2db00064, 55 },
    { 0x39400067, 95 },
    { 0xb5c00064, 93 },
    { 0xde00069, 23 },
    { 0xd0200063, 66 },
    { 0x5000006a, 84 },
    { 0xf400006c, 55 },
    { 0xdcb0006c, 95 },
    { 0xdef00062, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e200073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9120007a, 95 },
    { 0xf5a0007b, 76 },
    { 0xaf90007a, 61 },
    { 0x9d90007d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x63200080, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec200087, 55 },
    { 0x28600088, 61 },
    { 0x9b700089, 95 },
    { 0x9a800089, 66 },
    { 0xec60008b, 33 },
    { 0xb900008c, 33 },
    { 0x5980008d, 90 },
    { 0x68a0008e, 99 },
    { 0x8ca0008b, 68 },
    { 0x5c500090, 33 },
    { 0x1d300091, 25 },
    { 0x98600092, 91 },
    { 0x8dd00093, 95 },
    { 0x43400094, 33 },
    { 0xaac00090, 100 },
    { 0x9e800095, 93 },
    { 0, 0 },
    { 0x17400098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9000a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c000a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1000ac, 33 },
    { 0xe5d000ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795000b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263000bd, 98 },
    { 0x82f000be, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4000c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39000c8, 91 },
    { 0x276000c9, 75 },
    { 0x1aa000c9, 33 },
    { 0, 0 },
    { 0x1ca000cc, 67 },
    { 0x599000cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff000d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6000e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493000ed, 95 },
    { 0xbb000ee, 61 },
    { 0x976000ef, 83 },
    { 0, 0 },
    { 0x448000f1, 90 },
    { 0x748000f2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54000f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d000fd, 98 },
    { 0x49c000fe, 61 },
    { 0, 0 },
    { 0x2ab00100, 66 },
    { 0, 0 },
    { 0x97900102, 95 },
    { 0xf4700103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4100108, 46 },
    { 0xa900109, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f00110, 82 },
    { 0xb5e00111, 18 },
    { 0x28000111, 46 },
    { 0xb2a00113, 99 },
    { 0, 0 },
    { 0xde900115, 78 },
    { 0xba100116, 55 },
    { 0xab100117, 33 },
    { 0, 0 },
    { 0xc5e00119, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2070011e, 90 },
    { 0x1c10011f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30400123, 81 },
    { 0x3b300124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba00127, 98 },
    { 0xe4000128, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b00137, 95 },
    { 0, 0 },
    { 0x58f00139, 43 },
    { 0xdb60013a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0300141, 52 },
    { 0xbb00141, 95 },
    { 0, 0 },
    { 0x73500144, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3100148, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0014c, 40 },
    { 0, 0 },
    { 0xf390014e, 34 },
    { 0xcf0014f, 68 },
    { 0xff20014f, 95 },
    { 0xec600150, 93 },
    { 0x2800152, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e00155, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c0015d, 23 },
    { 0x5840015d, 61 },
    { 0xeaf0015f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85900163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b00168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca0016b, 55 },
    { 0x8e50016c, 78 },
    { 0x5250016d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33200170, 36 },
    { 0xdfd00171, 98 },
    { 0x25000172, 61 },
    { 0x66700173, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b00177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa00183, 61 },
    { 0xd600183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce00187, 33 },
    { 0, 0 },
    { 0x95300189, 23 },
    { 0, 0 },
    { 0x49b0018b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e00196, 88 },
    { 0x46400196, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940019c, 23 },
    { 0, 0 },
    { 0xc340019e, 46 },
    { 0x5630019f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb001a4, 61 },
    { 0xa18001a5, 61 },
    { 0, 0 },
    { 0x119001a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2001ae, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0x479001b1, 61 },
    { 0x1a4001b2, 43 },
    { 0, 0 },
    { 0xd2c001b4, 46 },
    { 0xf8a001b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24001bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575001c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384001c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78001cf, 17 },
    { 0xcd1001d0, 98 },
    { 0x578001cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185001d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705001d9, 96 },
    { 0xa68001da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25001e9, 23 },
    { 0xf60001ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d001f0, 23 },
    { 0xb91001f1, 55 },
    { 0xb32001f2, 76 },
    { 0xc3f001f3, 23 },
    { 0x1b1001f4, 33 },
    { 0x5e7001f5, 46 },
    { 0, 0 },
    { 0x118001f7, 90 },
    { 0, 0 },
    { 0x5c001f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5001fc, 36 },
    { 0, 0 },
    { 0xfc7001fe, 100 },
    { 0x1a0001ff, 76 },
    { 0x58200200, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab00208, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72100216, 95 },
    { 0xd7000217, 52 },
    { 0xa8700218, 33 },
    { 0xc6000219, 23 },
    { 0x71b0021a, 76 },
    { 0x1f40021a, 76 },
    { 0, 0 },
    { 0x570021d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b700220, 88 },
    { 0, 0 },
    { 0xeff00222, 98 },
    { 0xa2500222, 98 },
    { 0x6200224, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3400227, 55 },
    { 0x26900228, 95 },
    { 0xd6f00229, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf70022f, 34 },
    { 0, 0 },
    { 0x9f700231, 23 },
    { 0xa9500231, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb00239, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b00240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e000247, 55 },
    { 0x12a00248, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf00251, 61 },
    { 0, 0 },
    { 0xb0500253, 75 },
    { 0, 0 },
    { 0xac000255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060025a, 98 },
    { 0, 0 },
    { 0xfe60025c, 60 },
    { 0, 0 },
    { 0x18b0025e, 43 },
    { 0, 0 },
    { 0x92300260, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b700263, 61 },
    { 0, 0 },
    { 0x43600265, 81 },
    { 0xa2400266, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380026b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d10026e, 95 },
    { 0x6a90026f, 28 },
    { 0x39f00270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77500278, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50027c, 23 },
    { 0, 0 },
    { 0x4b50027e, 67 },
    { 0xd0027f, 61 },
    { 0x53d0027f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad700283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98600287, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60028d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b00290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b00298, 69 },
    { 0x9300299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f0029e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e002a2, 98 },
    { 0, 0 },
    { 0xd9f002a4, 59 },
    { 0x2ca002a5, 33 },
    { 0x4e002a6, 98 },
    { 0x6ea002a5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f002ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133002b5, 68 },
    { 0xe39002b6, 52 },
    { 0x285002b7, 77 },
    { 0xf22002b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a002bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519002c1, 90 },
    { 0x80002c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02002c5, 78 },
    { 0xcf6002c6, 76 },
    { 0, 0 },
    { 0x9cb002c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3002ce, 87 },
    { 0, 0 },
    { 0xf83002d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1002d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358002df, 69 },
    { 0x126002e0, 46 },
    { 0, 0 },
    { 0x4bd002e2, 33 },
    { 0x339002e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859002e7, 43 },
    { 0, 0 },
    { 0xba1002e9, 95 },
    { 0x13e002ea, 66 },
    { 0x919002eb, 95 },
    { 0, 0 },
    { 0xbdb002ed, 33 },
    { 0, 0 },
    { 0x562002ef, 88 },
    { 0xaeb002ef, 95 },
    { 0xd60002f1, 52 },
    { 0, 0 },
    { 0x9e7002f3, 19 },
    { 0, 0 },
    { 0x28f002f5, 33 },
    { 0xfe6002f6, 76 },
    { 0x570002f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a002fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8002ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5000306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd610030d, 33 },
    { 0xa0030e, 84 },
    { 0x34a0030f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b400313, 43 },
    { 0xe7e00313, 43 },
    { 0x8400313, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb260031a, 23 },
    { 0xf790031a, 99 },
    { 0, 0 },
    { 0x8600031d, 90 },
    { 0x8060031d, 34 },
    { 0, 0 },
    { 0xd7400320, 23 },
    { 0xc2f00321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a00324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45900328, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe900332, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60033a, 34 },
    { 0x2820033b, 98 },
    { 0xb500033c, 61 },
    { 0xc620033c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44900351, 87 },
    { 0x5cd00352, 90 },
    { 0x11b00353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d00356, 95 },
    { 0, 0 },
    { 0xd2300358, 77 },
    { 0xa6b00359, 66 },
    { 0xa1400358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a70035d, 33 },
    { 0x4f40035e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae00364, 88 },
    { 0x2d700364, 61 },
    { 0xd9400366, 90 },
    { 0x3dc00367, 61 },
    { 0xba600368, 75 },
    { 0xbf600369, 78 },
    { 0x9eb0036a, 43 },
    { 0xf7d0036b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0036f, 12 },
    { 0x9cb00370, 33 },
    { 0x7b60036f, 33 },
    { 0xa4600372, 90 },
    { 0x86000372, 33 },
    { 0x60000374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe200379, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2020037e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71800384, 98 },
    { 0xedf00385, 95 },
    { 0, 0 },
    { 0xed400387, 98 },
    { 0x1de00388, 43 },
    { 0xc500389, 93 },
    { 0, 0 },
    { 0xaf70038b, 91 },
    { 0, 0 },
    { 0x9d20038d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda600391, 52 },
    { 0x16600392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84600395, 85 },
    { 0, 0 },
    { 0x30300397, 100 },
    { 0xff400397, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1520039b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd820039e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x723003a1, 23 },
    { 0xc9b003a2, 23 },
    { 0xe62003a1, 34 },
    { 0, 0 },
    { 0xb4a003a5, 23 },
    { 0x240003a6, 95 },
    { 0, 0 },
    { 0x838003a8, 66 },
    { 0, 0 },
    { 0x6ef003aa, 33 },
    { 0, 0 },
    { 0x83d003ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a003b1, 56 },
    { 0xd76003b1, 33 },
    { 0x141003b1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813003c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de003c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e003c6, 85 },
    { 0, 0 },
    { 0xb51003c8, 43 },
    { 0x2f2003c9, 96 },
    { 0xab9003c9, 34 },
    { 0xb2f003ca, 60 },
    { 0x3fe003cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51003cf, 33 },
    { 0xde7003d0, 93 },
    { 0x6a4003d1, 56 },
    { 0, 0 },
    { 0x3d9003d3, 61 },
    { 0xa3a003d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14003d9, 90 },
    { 0, 0 },
    { 0x191003db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5003df, 34 },
    { 0xe6e003e0, 61 },
    { 0xf1003e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8003e5, 98 },
    { 0, 0 },
    { 0x8e7003e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb003ee, 23 },
    { 0, 0 },
    { 0x8f1003f0, 66 },
    { 0x7f9003f0, 77 },
    { 0, 0 },
    { 0xe70003f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08003f8, 23 },
    { 0xbd5003f8, 95 },
    { 0xe83003fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x750003fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a500408, 48 },
    { 0xc1500409, 17 },
    { 0x7ba0040a, 39 },
    { 0, 0 },
    { 0x2b40040c, 55 },
    { 0x9760040d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece00410, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47800419, 83 },
    { 0, 0 },
    { 0x2ac0041b, 93 },
    { 0, 0 },
    { 0x9400041d, 77 },
    { 0x72f0041e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2500421, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78400426, 17 },
    { 0xda700427, 43 },
    { 0, 0 },
    { 0x48e00429, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe540042c, 90 },
    { 0x7f60042d, 98 },
    { 0xae50042e, 46 },
    { 0x4ae0042f, 34 },
    { 0xfde00430, 90 },
    { 0xd70042f, 61 },
    { 0x5e400432, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda000436, 34 },
    { 0x9eb00437, 95 },
    { 0, 0 },
    { 0xdf100439, 27 },
    { 0, 0 },
    { 0x8730043b, 33 },
    { 0xd050043b, 95 },
    { 0xf0f0043d, 78 },
    { 0x3460043e, 61 },
    { 0xdc40043f, 80 },
    { 0xfde0043f, 36 },
    { 0x7e600441, 33 },
    { 0xea800442, 90 },
    { 0x1bb00443, 34 },
    { 0, 0 },
    { 0x53b00445, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0100449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d50044c, 90 },
    { 0x74d0044d, 95 },
    { 0xeaa0044e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47700454, 90 },
    { 0x30c00455, 61 },
    { 0, 0 },
    { 0xf1400457, 66 },
    { 0x59600458, 75 },
    { 0xd2e00459, 8 },
    { 0, 0 },
    { 0x82f0045b, 93 },
    { 0x3350045c, 34 },
    { 0, 0 },
    { 0xc7f0045e, 33 },
    { 0, 0 },
    { 0xeaf00460, 90 },
    { 0, 0 },
    { 0xf9000462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2700466, 90 },
    { 0xf3b00467, 66 },
    { 0, 0 },
    { 0xeb000469, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2880046c, 90 },
    { 0x14b0046d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e900471, 77 },
    { 0, 0 },
    { 0x6b100473, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff00479, 34 },
    { 0, 0 },
    { 0xc80047b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f700481, 33 },
    { 0, 0 },
    { 0xd7000483, 95 },
    { 0x24400484, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b20048b, 80 },
    { 0xafb0048b, 98 },
    { 0x90e0048c, 87 },
    { 0x16a0048e, 76 },
    { 0, 0 },
    { 0x64400490, 98 },
    { 0x2e900491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4600496, 33 },
    { 0x84100497, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40049a, 23 },
    { 0x7ba0049b, 55 },
    { 0x2090049c, 52 },
    { 0x1430049d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6004a7, 80 },
    { 0xbe2004a8, 33 },
    { 0xbe3004a7, 90 },
    { 0x23e004a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4004ae, 23 },
    { 0x67b004af, 90 },
    { 0x564004af, 90 },
    { 0, 0 },
    { 0x4f004b2, 98 },
    { 0, 0 },
    { 0xc4d004b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x924004b7, 90 },
    { 0, 0 },
    { 0x5ee004b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc004c1, 90 },
    { 0x979004c2, 59 },
    { 0x71d004c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce004c7, 81 },
    { 0xd32004c8, 90 },
    { 0x67b004c9, 43 },
    { 0xb70004ca, 78 },
    { 0x16a004cb, 88 },
    { 0xfe0004ca, 52 },
    { 0xe4d004cd, 17 },
    { 0xec2004c9, 23 },
    { 0xe28004cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4004db, 33 },
    { 0x882004db, 87 },
    { 0, 0 },
    { 0x477004de, 98 },
    { 0, 0 },
    { 0x52d004e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119004e7, 91 },
    { 0, 0 },
    { 0xc5b004e9, 90 },
    { 0x2e9004ea, 88 },
    { 0, 0 },
    { 0x118004ec, 23 },
    { 0x7b2004ec, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af004f1, 87 },
    { 0xd3b004f2, 52 },
    { 0xb1e004f2, 90 },
    { 0x4d2004f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9004fa, 75 },
    { 0xa7a004fb, 75 },
    { 0, 0 },
    { 0x4ad004fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc00502, 17 },
    { 0xc8200503, 40 },
    { 0x22600502, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b0050c, 55 },
    { 0, 0 },
    { 0x9f0050e, 12 },
    { 0x69f0050f, 78 },
    { 0x30800510, 33 },
    { 0x4ea0050f, 68 },
    { 0x2f00512, 90 },
    { 0x38800513, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f00518, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb0051b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c800520, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d00524, 40 },
    { 0x9ac00525, 76 },
    { 0x7a700526, 85 },
    { 0, 0 },
    { 0x86000528, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0052b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf200533, 43 },
    { 0xf8600534, 91 },
    { 0xdf400535, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b0053a, 68 },
    { 0x2620053b, 98 },
    { 0, 0 },
    { 0x54e0053d, 78 },
    { 0, 0 },
    { 0xfc00053f, 98 },
    { 0x7900540, 34 },
    { 0x7250053f, 99 },
    { 0, 0 },
    { 0x8b000543, 90 },
    { 0x5ee00543, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d700547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7160054e, 95 },
    { 0xdad0054e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c00553, 87 },
    { 0xbea00554, 90 },
    { 0xcbd00553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf740055c, 43 },
    { 0xe8f0055d, 87 },
    { 0x58d0055d, 43 },
    { 0, 0 },
    { 0x76b00560, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6700566, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e800569, 90 },
    { 0x8650056a, 98 },
    { 0x3d70056a, 81 },
    { 0x1c80056c, 84 },
    { 0xbde0056b, 95 },
    { 0x37e0056c, 23 },
    { 0x9ac0056f, 91 },
    { 0x86f0056b, 17 },
    { 0x6c60056a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f00579, 66 },
    { 0xd600057a, 90 },
    { 0x9450057b, 76 },
    { 0xcae0057c, 65 },
    { 0x1030057d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea00582, 81 },
    { 0, 0 },
    { 0x3a300584, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a00588, 23 },
    { 0, 0 },
    { 0xd660058a, 61 },
    { 0xd1f0058b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b0058e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab00592, 90 },
    { 0x65500593, 66 },
    { 0x24d00593, 43 },
    { 0xadd00595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef00598, 84 },
    { 0xd2800598, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66005a3, 46 },
    { 0xfc9005a4, 98 },
    { 0, 0 },
    { 0x282005a6, 61 },
    { 0xd92005a7, 95 },
    { 0x738005a8, 43 },
    { 0, 0 },
    { 0x171005aa, 98 },
    { 0x9b2005ab, 90 },
    { 0x2fb005ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09005b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96005ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3005bd, 33 },
    { 0x631005be, 100 },
    { 0, 0 },
    { 0x55005c0, 98 },
    { 0xf45005c1, 33 },
    { 0, 0 },
    { 0xa98005c3, 90 },
    { 0x799005c4, 100 },
    { 0xaa2005c5, 33 },
    { 0x570005c6, 90 },
    { 0, 0 },
    { 0x852005c8, 99 },
    { 0xe0c005c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x504005cc, 90 },
    { 0x9ca005cd, 23 },
    { 0, 0 },
    { 0x498005cf, 90 },
    { 0x840005cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06005da, 84 },
    { 0x9e6005db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f005e0, 95 },
    { 0, 0 },
    { 0xbcd005e2, 46 },
    { 0xb19005e2, 98 },
    { 0, 0 },
    { 0xbf005e5, 90 },
    { 0x28a005e6, 87 },
    { 0xe7a005e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3005eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f005ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90005f2, 61 },
    { 0, 0 },
    { 0xd3b005f4, 23 },
    { 0x13005f5, 33 },
    { 0, 0 },
    { 0xc92005f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7005fa, 91 },
    { 0xe53005fa, 90 },
    { 0x960005fc, 81 },
    { 0x117005fc, 55 },
    { 0, 0 },
    { 0xc6c005ff, 76 },
    { 0x7df00600, 46 },
    { 0xc9e00600, 66 },
    { 0x7f200602, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa00608, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d500610, 91 },
    { 0, 0 },
    { 0x2d100612, 33 },
    { 0x9da00613, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55400616, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c00619, 97 },
    { 0xccf00619, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e10061d, 90 },
    { 0xfb70061e, 91 },
    { 0x8060061f, 17 },
    { 0x75e00620, 34 },
    { 0, 0 },
    { 0x6ca00622, 93 },
    { 0x31000623, 75 },
    { 0x85000624, 90 },
    { 0xff500625, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d0062a, 61 },
    { 0, 0 },
    { 0x5c70062c, 90 },
    { 0xc0e0062d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2400631, 33 },
    { 0xcc500632, 97 },
    { 0xe5600633, 23 },
    { 0x2ef00633, 76 },
    { 0, 0 },
    { 0x71f00636, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8600639, 23 },
    { 0xcfa00639, 61 },
    { 0x72e0063b, 76 },
    { 0xda0063c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf330063f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2100642, 46 },
    { 0xc9300643, 34 },
    { 0, 0 },
    { 0x73a00645, 23 },
    { 0x28b00646, 90 },
    { 0, 0 },
    { 0x68300648, 33 },
    { 0, 0 },
    { 0xff90064a, 33 },
    { 0x2fe0064a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4630064e, 77 },
    { 0, 0 },
    { 0x66a00650, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef00653, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5500656, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97500662, 77 },
    { 0x5ea00663, 76 },
    { 0xb5e00662, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0200669, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x610066c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4100672, 23 },
    { 0, 0 },
    { 0x8a100674, 90 },
    { 0x53a00675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db0067a, 55 },
    { 0xf480067a, 34 },
    { 0xe620067c, 100 },
    { 0, 0 },
    { 0x35c0067e, 23 },
    { 0xd650067f, 95 },
    { 0x1b70067f, 98 },
    { 0x8ec00681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d00686, 99 },
    { 0x9fe00687, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b50068c, 100 },
    { 0, 0 },
    { 0xe080068e, 84 },
    { 0xa9e0068f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d00698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23006a1, 55 },
    { 0, 0 },
    { 0x821006a3, 43 },
    { 0x4a2006a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52006a9, 98 },
    { 0x875006aa, 90 },
    { 0x629006a9, 66 },
    { 0x885006ac, 33 },
    { 0x511006ad, 93 },
    { 0x71a006a9, 12 },
    { 0x195006af, 61 },
    { 0x98a006b0, 61 },
    { 0x42a006b1, 34 },
    { 0x5c9006b0, 23 },
    { 0xc0d006b3, 61 },
    { 0x854006b1, 95 },
    { 0x443006b1, 76 },
    { 0, 0 },
    { 0x265006b7, 33 },
    { 0x3b0006b7, 46 },
    { 0x42d006b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650006c3, 100 },
    { 0x388006c3, 61 },
    { 0x749006c5, 43 },
    { 0, 0 },
    { 0x7c9006c7, 93 },
    { 0x6bb006c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb006d0, 67 },
    { 0, 0 },
    { 0x2f0006d2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1006d5, 90 },
    { 0x66a006d6, 100 },
    { 0, 0 },
    { 0x31b006d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911006e9, 43 },
    { 0xcde006ea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd006ee, 90 },
    { 0x489006ef, 61 },
    { 0x5d006ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253006f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe006fa, 95 },
    { 0x8c4006fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4006fe, 90 },
    { 0x83d006ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b00703, 61 },
    { 0, 0 },
    { 0x3e200705, 52 },
    { 0xa6d00706, 33 },
    { 0x6f300707, 75 },
    { 0x4a100708, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b0070c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0700717, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde60071b, 78 },
    { 0xbde0071c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e80071f, 99 },
    { 0, 0 },
    { 0x43400721, 100 },
    { 0xcd600722, 23 },
    { 0, 0 },
    { 0x3c00724, 33 },
    { 0x45a00725, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0072a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc870072e, 90 },
    { 0, 0 },
    { 0x17100730, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1500733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db0073a, 33 },
    { 0x9070073b, 90 },
    { 0x34e0073b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e100741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7600745, 17 },
    { 0, 0 },
    { 0xbeb00747, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a0074e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f00759, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c0075c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b200762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a600765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c10076c, 90 },
    { 0xb650076c, 77 },
    { 0x6e30076e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc00772, 93 },
    { 0xada00773, 95 },
    { 0x1e700774, 23 },
    { 0x14000775, 76 },
    { 0xeb300774, 90 },
    { 0, 0 },
    { 0xd8700778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf0077c, 90 },
    { 0, 0 },
    { 0x4910077e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65900782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc800785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60800788, 78 },
    { 0x49a00789, 91 },
    { 0, 0 },
    { 0x9780078b, 99 },
    { 0, 0 },
    { 0x7dc0078d, 46 },
    { 0x6ec0078d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b400794, 76 },
    { 0x26e00795, 23 },
    { 0xa300795, 90 },
    { 0, 0 },
    { 0xfef00798, 90 },
    { 0x6bc00798, 66 },
    { 0x80e0079a, 69 },
    { 0, 0 },
    { 0x1e60079c, 33 },
    { 0xe790079d, 85 },
    { 0x1f70079e, 90 },
    { 0x8b60079d, 96 },
    { 0x6140079f, 55 },
    { 0x6140079e, 33 },
    { 0x7c20079e, 66 },
    { 0x53e007a3, 98 },
    { 0xa04007a3, 75 },
    { 0x195007a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41007ac, 83 },
    { 0x786007ad, 33 },
    { 0xd72007ad, 77 },
    { 0xe3a007af, 90 },
    { 0x28c007b0, 90 },
    { 0x635007b1, 23 },
    { 0x679007b2, 100 },
    { 0, 0 },
    { 0x4ef007b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x898007b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57007bf, 43 },
    { 0, 0 },
    { 0xcb5007c1, 69 },
    { 0, 0 },
    { 0x77a007c3, 46 },
    { 0x2b9007c4, 66 },
    { 0, 0 },
    { 0x226007c6, 23 },
    { 0xf43007c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f007cf, 66 },
    { 0x20e007cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae007d4, 87 },
    { 0, 0 },
    { 0x383007d6, 98 },
    { 0, 0 },
    { 0x23a007d8, 61 },
    { 0x9f2007d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac007e2, 98 },
    { 0xf53007e2, 90 },
    { 0x5b5007e2, 84 },
    { 0xe73007e5, 78 },
    { 0x153007e6, 98 },
    { 0x172007e7, 46 },
    { 0xcb6007e8, 98 },
    { 0x500007e9, 90 },
    { 0xde5007e2, 75 },
    { 0, 0 },
    { 0x807007ec, 46 },
    { 0x572007ed, 76 },
    { 0xece007ed, 96 },
    { 0xc24007ef, 98 },
    { 0x370007f0, 95 },
    { 0xeaa007ed, 90 },
    { 0, 0 },
    { 0xafc007f3, 55 },
    { 0xd9b007f4, 98 },
    { 0x55f007f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1007fa, 76 },
    { 0x328007fa, 43 },
    { 0, 0 },
    { 0x3b7007fd, 95 },
    { 0xe3c007fe, 23 },
    { 0xf38007fe, 33 },
    { 0x8fa00800, 90 },
    { 0, 0 },
    { 0xe7300802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d00807, 90 },
    { 0, 0 },
    { 0xfb600809, 78 },
    { 0, 0 },
    { 0x68f0080b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680080e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee00814, 90 },
    { 0x3ff00815, 55 },
    { 0x56600816, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700081b, 33 },
    { 0x7f90081b, 61 },
    { 0, 0 },
    { 0x3230081e, 90 },
    { 0x6900081f, 43 },
    { 0x9740081e, 98 },
    { 0x86600821, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e00827, 61 },
    { 0xc9c00828, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3970082c, 90 },
    { 0xeac0082d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8600830, 98 },
    { 0xd6b00830, 33 },
    { 0x2400831, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10700836, 55 },
    { 0xa4400837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xff30083a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca0083d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82900842, 61 },
    { 0x34600842, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c0084e, 95 },
    { 0xdfa0084f, 97 },
    { 0xe600850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2900853, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf00856, 81 },
    { 0, 0 },
    { 0xa5c00858, 90 },
    { 0xfbe00859, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c0085e, 77 },
    { 0xa430085f, 98 },
    { 0x8f10085f, 33 },
    { 0x66200861, 61 },
    { 0x16f00862, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb340086d, 52 },
    { 0x7ad0086d, 33 },
    { 0x9280086d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64200874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc00877, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc30087b, 33 },
    { 0, 0 },
    { 0xfa0087d, 66 },
    { 0x3a20087d, 33 },
    { 0, 0 },
    { 0x7b00880, 98 },
    { 0xd00880, 17 },
    { 0x40b00880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d00886, 84 },
    { 0xa500886, 78 },
    { 0x29100887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb000088c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d0088f, 43 },
    { 0, 0 },
    { 0x8d400891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d00896, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf70089d, 90 },
    { 0xa1b0089e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3008a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552008a5, 90 },
    { 0xfc6008a5, 34 },
    { 0xcb6008a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14008ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5008b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807008b7, 66 },
    { 0, 0 },
    { 0xb7d008b9, 84 },
    { 0x3ae008ba, 61 },
    { 0xc82008bb, 90 },
    { 0x495008bc, 46 },
    { 0xb2c008bc, 34 },
    { 0, 0 },
    { 0x3ed008bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1008c2, 61 },
    { 0, 0 },
    { 0x83008c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62008ca, 90 },
    { 0x593008cb, 20 },
    { 0, 0 },
    { 0x325008cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e008d0, 91 },
    { 0xffb008d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e008d6, 34 },
    { 0, 0 },
    { 0x39e008d8, 34 },
    { 0x286008d9, 90 },
    { 0xbbb008d9, 78 },
    { 0x5fa008db, 43 },
    { 0xf52008dc, 40 },
    { 0, 0 },
    { 0xe4c008de, 46 },
    { 0, 0 },
    { 0x11b008e0, 98 },
    { 0xae5008e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3008f2, 56 },
    { 0x783008f2, 52 },
    { 0x5c2008f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e008f7, 33 },
    { 0xabe008f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a008fb, 61 },
    { 0x1df008fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db00900, 68 },
    { 0xe3300900, 90 },
    { 0, 0 },
    { 0x99c00903, 46 },
    { 0xd5600904, 95 },
    { 0xa6f00905, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c00908, 98 },
    { 0xf7400909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0090f, 23 },
    { 0x19c00910, 99 },
    { 0x93200911, 90 },
    { 0x42600912, 23 },
    { 0x87800913, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb800917, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6460091b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6100091e, 23 },
    { 0x8690091f, 61 },
    { 0x290091f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f00927, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca20092c, 90 },
    { 0x30b0092d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd00931, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e00934, 61 },
    { 0x46800935, 68 },
    { 0, 0 },
    { 0x26200937, 33 },
    { 0, 0 },
    { 0x3da00939, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe00943, 95 },
    { 0, 0 },
    { 0x34200945, 40 },
    { 0x89b00946, 33 },
    { 0x92400947, 84 },
    { 0, 0 },
    { 0x3db00949, 23 },
    { 0, 0 },
    { 0x5b10094b, 90 },
    { 0xe940094c, 76 },
    { 0x7f90094d, 61 },
    { 0x3ee0094e, 90 },
    { 0x6b50094c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db00955, 90 },
    { 0, 0 },
    { 0xeb100957, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea10095d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e900961, 33 },
    { 0x2ad00962, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa600967, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f0096c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5500971, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d00975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1200978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d0097e, 87 },
    { 0x4fd0097f, 90 },
    { 0x19b00980, 90 },
    { 0x98c0097e, 55 },
    { 0xb5600982, 23 },
    { 0x3a500983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f00986, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c00989, 46 },
    { 0xe010098a, 90 },
    { 0xc360098a, 33 },
    { 0x48a0098c, 90 },
    { 0x2610098c, 78 },
    { 0x8cd0098c, 90 },
    { 0x20b0098f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c600998, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6009a0, 34 },
    { 0xe79009a0, 61 },
    { 0x3b8009a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc009aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4009ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3009b3, 33 },
    { 0xd65009b3, 93 },
    { 0xb4d009b5, 66 },
    { 0, 0 },
    { 0x227009b7, 91 },
    { 0xac6009b8, 23 },
    { 0x5b0009b9, 90 },
    { 0x5ac009ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799009be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175009c6, 78 },
    { 0x46f009c7, 66 },
    { 0x206009c8, 46 },
    { 0, 0 },
    { 0x345009ca, 90 },
    { 0xeb009cb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd009d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce009d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c009d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8009db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab009e1, 23 },
    { 0x90f009e1, 23 },
    { 0x44a009e3, 43 },
    { 0, 0 },
    { 0x49a009e5, 85 },
    { 0x5da009e6, 90 },
    { 0x485009e7, 43 },
    { 0, 0 },
    { 0xe9d009e9, 84 },
    { 0, 0 },
    { 0xf4d009eb, 77 },
    { 0x956009ec, 33 },
    { 0x8ad009ed, 78 },
    { 0x155009ec, 80 },
    { 0xb5009ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1009f9, 90 },
    { 0x9a5009fa, 61 },
    { 0xb7d009fb, 90 },
    { 0x9c4009fc, 97 },
    { 0, 0 },
    { 0x1fd009fe, 29 },
    { 0, 0 },
    { 0x5b900a00, 61 },
    { 0, 0 },
    { 0x67d00a02, 61 },
    { 0xfd900a02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39000a08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a00a0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a00a10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9700a16, 19 },
    { 0xb4900a16, 61 },
    { 0, 0 },
    { 0x5ee00a19, 47 },
    { 0xdfc00a1a, 34 },
    { 0x76100a1a, 93 },
    { 0, 0 },
    { 0xb5800a1d, 76 },
    { 0xadb00a1e, 76 },
    { 0xc1600a1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa00a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b00a29, 23 },
    { 0x94b00a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d400a33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd00a37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e00a3c, 23 },
    { 0x85100a3d, 80 },
    { 0xd7200a3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d00a42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e100a4c, 98 },
    { 0xc800a4c, 90 },
    { 0x5f00a4e, 98 },
    { 0x7b900a4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4200a53, 95 },
    { 0x8b600a53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f00a58, 33 },
    { 0xed800a58, 33 },
    { 0x52900a5a, 96 },
    { 0, 0 },
    { 0x60600a5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef400a67, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50500a6c, 84 },
    { 0, 0 },
    { 0xbe100a6e, 33 },
    { 0xb2b00a6f, 34 },
    { 0x7f600a70, 83 },
    { 0x19f00a6f, 95 },
    { 0xebc00a72, 46 },
    { 0, 0 },
    { 0x41f00a74, 90 },
    { 0, 0 },
    { 0x63a00a76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39600a79, 98 },
    { 0xd6500a7a, 33 },
    { 0, 0 },
    { 0x49300a7c, 93 },
    { 0xd1f00a7c, 34 },
    { 0xf1900a7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb200a85, 90 },
    { 0xa8800a85, 61 },
    { 0, 0 },
    { 0x66000a88, 38 },
    { 0, 0 },
    { 0xd4100a8a, 98 },
    { 0xb1a00a8b, 100 },
    { 0, 0 },
    { 0xcdb00a8d, 97 },
    { 0xe1c00a8e, 87 },
    { 0x7cb00a8f, 33 },
    { 0x5ca00a8f, 61 },
    { 0x4a900a91, 33 },
    { 0, 0 },
    { 0xd6c00a93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f00a99, 88 },
    { 0x3e900a99, 33 },
    { 0x8c00a9b, 66 },
    { 0x88b00a9b, 61 },
    { 0, 0 },
    { 0xd1c00a9e, 55 },
    { 0, 0 },
    { 0x30b00aa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b100aa5, 61 },
    { 0xa0a00aa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x68900aa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e00aad, 34 },
    { 0xb3900aae, 90 },
    { 0x7c800aaf, 99 },
    { 0xa800ab0, 66 },
    { 0xd0600aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe200ab4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e00ab7, 98 },
    { 0xcc100ab8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd00abf, 43 },
    { 0, 0 },
    { 0x62600ac1, 76 },
    { 0x7b300ac2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee00ac6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d00ac9, 77 },
    { 0x31300ac9, 34 },
    { 0x9fd00aca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde00ad5, 91 },
    { 0, 0 },
    { 0x2e00ad7, 23 },
    { 0, 0 },
    { 0x9100ad9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd500adc, 93 },
    { 0x5400add, 34 },
    { 0x6a300ade, 77 },
    { 0x8e800adf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca500ae2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a00ae6, 61 },
    { 0, 0 },
    { 0x48f00ae8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4500af3, 77 },
    { 0, 0 },
    { 0x55b00af5, 98 },
    { 0x8b900af6, 90 },
    { 0x9b00af5, 78 },
    { 0x96700af8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b500afb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d00b04, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb00b07, 33 },
    { 0, 0 },
    { 0x74d00b09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab000b0e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e700b15, 95 },
    { 0, 0 },
    { 0x97300b17, 61 },
    { 0xdea00b18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d200b1d, 23 },
    { 0x4fa00b1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b000b21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe00b24, 77 },
    { 0x76a00b25, 88 },
    { 0x42200b25, 95 },
    { 0x1f400b24, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6000b2c, 98 },
    { 0xb1c00b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57100b35, 76 },
    { 0x53000b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f000b3b, 61 },
    { 0, 0 },
    { 0x83d00b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd00b40, 23 },
    { 0x7cd00b41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19000b45, 33 },
    { 0, 0 },
    { 0xfd700b47, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41100b4c, 98 },
    { 0, 0 },
    { 0x58e00b4e, 34 },
    { 0xd9600b4f, 33 },
    { 0xcf900b50, 33 },
    { 0x98c00b51, 96 },
    { 0x28600b52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c00b5b, 61 },
    { 0x19200b5c, 95 },
    { 0xa600b5d, 90 },
    { 0x49400b5e, 33 },
    { 0x7ad00b5b, 90 },
    { 0, 0 },
    { 0x8e600b61, 90 },
    { 0xfea00b62, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24000b66, 52 },
    { 0, 0 },
    { 0x6a400b68, 52 },
    { 0x76d00b69, 98 },
    { 0, 0 },
    { 0xb3f00b6b, 90 },
    { 0, 0 },
    { 0x71d00b6d, 90 },
    { 0, 0 },
    { 0x1f600b6f, 76 },
    { 0x3f400b70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76200b74, 61 },
    { 0, 0 },
    { 0x6d900b76, 87 },
    { 0xd2100b76, 80 },
    { 0x72700b77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe00b7d, 17 },
    { 0x88300b7d, 33 },
    { 0xc3200b7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed800b84, 33 },
    { 0x6ad00b85, 61 },
    { 0, 0 },
    { 0xc000b87, 90 },
    { 0x6d900b88, 76 },
    { 0, 0 },
    { 0x6ca00b8a, 55 },
    { 0x44900b8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c00b8f, 23 },
    { 0x90d00b90, 90 },
    { 0xb2400b91, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff00b96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5800b99, 61 },
    { 0, 0 },
    { 0x26d00b9b, 61 },
    { 0x1b200b9b, 95 },
    { 0x66700b9d, 77 },
    { 0x62700b9e, 61 },
    { 0x12000b9e, 76 },
    { 0x93200b9e, 43 },
    { 0x1f200ba1, 90 },
    { 0xbf700ba2, 76 },
    { 0xc000ba1, 43 },
    { 0x8fd00ba4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb800ba7, 91 },
    { 0xa8200ba8, 34 },
    { 0xb5c00ba9, 90 },
    { 0x22000ba9, 76 },
    { 0xd5100bab, 87 },
    { 0x6600bac, 66 },
    { 0x72700bab, 95 },
    { 0, 0 },
    { 0xd2c00baf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5200bb2, 55 },
    { 0, 0 },
    { 0xfb300bb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd00bb8, 23 },
    { 0x56b00bb8, 87 },
    { 0xfde00bba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf100bbd, 81 },
    { 0xc5000bbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4800bcd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66800bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a900bd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef00bdd, 98 },
    { 0x1f700bdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6300be4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0600be8, 90 },
    { 0x4f600be9, 33 },
    { 0, 0 },
    { 0x30700beb, 95 },
    { 0xf1200bec, 98 },
    { 0, 0 },
    { 0xeeb00bee, 61 },
    { 0xdf600bef, 78 },
    { 0x5bc00bf0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e300bf4, 95 },
    { 0x7f00bf4, 23 },
    { 0, 0 },
    { 0xe7d00bf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd00bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47200c02, 4 },
    { 0, 0 },
    { 0x4b400c04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac000c0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b00c10, 93 },
    { 0, 0 },
    { 0x18a00c12, 52 },
    { 0x5c600c12, 55 },
    { 0xb3100c14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b000c1b, 98 },
    { 0x6b000c1b, 100 },
    { 0xa9900c1d, 88 },
    { 0x24700c1d, 90 },
    { 0x5da00c1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9400c24, 98 },
    { 0xf700c24, 68 },
    { 0x6f300c26, 98 },
    { 0x8e200c27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a00c2a, 98 },
    { 0, 0 },
    { 0x5c00c2c, 61 },
    { 0xb7b00c2d, 93 },
    { 0xc5f00c2e, 61 },
    { 0x4bc00c2e, 17 },
    { 0xea400c2f, 23 },
    { 0xa2200c31, 61 },
    { 0x53f00c32, 34 },
    { 0xfba00c33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c00c3a, 78 },
    { 0xe5100c3a, 43 },
    { 0, 0 },
    { 0xc8700c3d, 87 },
    { 0x52200c3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82300c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc400c45, 91 },
    { 0, 0 },
    { 0x82000c47, 95 },
    { 0x4800c48, 95 },
    { 0, 0 },
    { 0xe6900c4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff00c50, 33 },
    { 0xd0f00c51, 61 },
    { 0x39e00c50, 52 },
    { 0, 0 },
    { 0xde600c54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d300c5a, 93 },
    { 0, 0 },
    { 0x7aa00c5c, 61 },
    { 0x85400c5d, 43 },
    { 0x27000c5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc400c61, 98 },
    { 0xcfa00c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a00c66, 23 },
    { 0x19700c66, 93 },
    { 0, 0 },
    { 0xeff00c69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99700c6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c800c6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f00c74, 98 },
    { 0x5c300c75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26700c79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2900c7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38400c84, 76 },
    { 0, 0 },
    { 0xcfb00c86, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af00c8a, 90 },
    { 0xf0f00c8b, 67 },
    { 0xd500c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea00c92, 67 },
    { 0, 0 },
    { 0xb8600c94, 95 },
    { 0x55d00c95, 33 },
    { 0xebb00c94, 19 },
    { 0xc6e00c97, 78 },
    { 0x8000c98, 96 },
    { 0x92800c99, 98 },
    { 0xe7f00c9a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd00ca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72700ca9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e100cb1, 95 },
    { 0x4c400cb1, 33 },
    { 0, 0 },
    { 0x72b00cb4, 96 },
    { 0x44500cb4, 46 },
    { 0xa3b00cb6, 55 },
    { 0xc4000cb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d00cbc, 98 },
    { 0x4b900cbd, 61 },
    { 0x58800cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee00cc3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d00cc8, 100 },
    { 0, 0 },
    { 0xfd500cca, 17 },
    { 0x1de00cca, 98 },
    { 0, 0 },
    { 0x4700ccd, 98 },
    { 0, 0 },
    { 0x32700ccf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d900cd2, 95 },
    { 0x96b00cd3, 23 },
    { 0xef700cd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c900cda, 90 },
    { 0, 0 },
    { 0xe8c00cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec00ce5, 34 },
    { 0x3000ce6, 92 },
    { 0xb0c00ce7, 98 },
    { 0x52300ce8, 95 },
    { 0xa0c00ce9, 61 },
    { 0xa6300ce5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b00ced, 23 },
    { 0, 0 },
    { 0x9e100cef, 76 },
    { 0x4c100cef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x89300cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46700cf7, 23 },
    { 0x90b00cf8, 66 },
    { 0xb6f00cf9, 95 },
    { 0xd4d00cfa, 17 },
    { 0, 0 },
    { 0x76500cfc, 43 },
    { 0, 0 },
    { 0xb3f00cfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f00d02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e100d08, 61 },
    { 0xe4600d09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10300d0c, 61 },
    { 0x5000d0d, 90 },
    { 0x83700d0d, 61 },
    { 0x75000d0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41500d13, 67 },
    { 0xb9700d14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9600d1c, 67 },
    { 0x8a200d1d, 23 },
    { 0, 0 },
    { 0x7d600d1f, 76 },
    { 0, 0 },
    { 0x41e00d21, 98 },
    { 0x55b00d21, 23 },
    { 0, 0 },
    { 0x75e00d24, 46 },
    { 0, 0 },
    { 0xc3a00d26, 55 },
    { 0x8d700d27, 23 },
    { 0x5400d28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f900d30, 33 },
    { 0xb8500d30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c00d36, 34 },
    { 0xea000d37, 95 },
    { 0xa6900d38, 98 },
    { 0x4a100d38, 95 },
    { 0x34b00d36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f00d3f, 33 },
    { 0, 0 },
    { 0xc0900d41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff00d46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb00d49, 87 },
    { 0x3f000d49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b00d4e, 78 },
    { 0, 0 },
    { 0x6ab00d50, 55 },
    { 0, 0 },
    { 0xc5600d52, 90 },
    { 0x9200d53, 99 },
    { 0xead00d54, 56 },
    { 0x21e00d55, 33 },
    { 0x9cd00d53, 98 },
    { 0x10400d57, 76 },
    { 0x46400d58, 61 },
    { 0x97500d58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a00d5d, 61 },
    { 0xea000d5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc00d61, 97 },
    { 0x28500d62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b700d68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f00d6d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xac800d70, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7400d73, 95 },
    { 0x1e200d74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb00d7b, 61 },
    { 0x3c300d7b, 33 },
    { 0xa9900d7d, 98 },
    { 0xc7400d7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87100d86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3300d8e, 76 },
    { 0, 0 },
    { 0x13600d90, 55 },
    { 0xda00d91, 90 },
    { 0x6e00d91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b600d95, 61 },
    { 0, 0 },
    { 0x99100d97, 43 },
    { 0, 0 },
    { 0x34e00d99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d400d9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a00da1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee300da9, 76 },
    { 0, 0 },
    { 0xd1b00dab, 46 },
    { 0x13c00dac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f00db0, 61 },
    { 0x3de00db1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b400dba, 98 },
    { 0x67700dbb, 46 },
    { 0x54d00dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb00dbf, 91 },
    { 0, 0 },
    { 0x7000dc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c00dc4, 90 },
    { 0x80700dc5, 98 },
    { 0, 0 },
    { 0x87500dc7, 33 },
    { 0x7d00dc8, 76 },
    { 0xa7d00dc9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f00dcd, 33 },
    { 0, 0 },
    { 0x46700dcf, 23 },
    { 0x38c00dd0, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b400dd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56200ddb, 90 },
    { 0xb4700ddb, 61 },
    { 0x85f00ddc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7500de5, 61 },
    { 0, 0 },
    { 0x85b00de7, 33 },
    { 0xa3600de8, 98 },
    { 0, 0 },
    { 0x11900dea, 34 },
    { 0x56400deb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae00def, 34 },
    { 0, 0 },
    { 0xef200df1, 43 },
    { 0x4d900df1, 61 },
    { 0, 0 },
    { 0x72700df4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea00df8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a00dfb, 23 },
    { 0xcb700dfc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x40300dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16900e03, 90 },
    { 0x1c400e03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28700e09, 95 },
    { 0x67800e09, 90 },
    { 0, 0 },
    { 0xe1e00e0c, 55 },
    { 0x82500e0c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a100e13, 61 },
    { 0, 0 },
    { 0x7af00e15, 88 },
    { 0x7a200e15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f100e1b, 61 },
    { 0x9ae00e1c, 98 },
    { 0x23c00e1d, 33 },
    { 0, 0 },
    { 0x58600e1f, 90 },
    { 0x85a00e20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0700e23, 66 },
    { 0, 0 },
    { 0x48f00e25, 23 },
    { 0, 0 },
    { 0xfe400e27, 93 },
    { 0xc7300e28, 52 },
    { 0xb8100e29, 33 },
    { 0, 0 },
    { 0x41900e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e00e2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x45000e31, 55 },
    { 0x95c00e32, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b00e35, 46 },
    { 0, 0 },
    { 0xeae00e37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc900e3a, 55 },
    { 0xa9b00e3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24400e41, 23 },
    { 0x1e700e42, 97 },
    { 0x99100e43, 98 },
    { 0xc8000e44, 90 },
    { 0, 0 },
    { 0x76c00e46, 99 },
    { 0x1a000e47, 23 },
    { 0xadd00e48, 17 },
    { 0x92000e46, 52 },
    { 0x26a00e49, 33 },
    { 0x99b00e4a, 23 },
    { 0, 0 },
    { 0x54b00e4d, 33 },
    { 0, 0 },
    { 0x2a000e4f, 95 },
    { 0xabc00e4f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d00e5c, 33 },
    { 0x18000e5d, 46 },
    { 0x8b300e5e, 65 },
    { 0x10f00e5f, 34 },
    { 0xb0600e5f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd00e64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32700e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a00e6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50300e79, 33 },
    { 0x76200e7a, 65 },
    { 0, 0 },
    { 0x1bc00e7c, 45 },
    { 0xd1500e7c, 33 },
    { 0, 0 },
    { 0x43c00e7f, 65 },
    { 0x84a00e80, 61 },
    { 0xd2d00e81, 69 },
    { 0x7e00e7f, 98 },
    { 0xb3c00e83, 40 },
    { 0xbde00e84, 76 },
    { 0x97300e84, 34 },
    { 0x57600e86, 33 },
    { 0x99700e86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16500e8e, 43 },
    { 0, 0 },
    { 0x8cf00e90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde500e94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb00e99, 43 },
    { 0, 0 },
    { 0xd6300e9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e00ea0, 23 },
    { 0, 0 },
    { 0x46300ea2, 98 },
    { 0xb2c00ea3, 90 },
    { 0x900ea4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d200ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3000eab, 34 },
    { 0xbc400eac, 95 },
    { 0x78a00ead, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0000eb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64600eb9, 34 },
    { 0, 0 },
    { 0xd9900ebb, 33 },
    { 0xe3200ebc, 68 },
    { 0, 0 },
    { 0xf5100ebe, 95 },
    { 0x7b000ebf, 95 },
    { 0xb7a00ebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47800ec6, 90 },
    { 0, 0 },
    { 0x7fb00ec8, 61 },
    { 0, 0 },
    { 0xd3e00eca, 52 },
    { 0, 0 },
    { 0x14700ecc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1000ed7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc500eda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b200edf, 33 },
    { 0x15100edf, 16 },
    { 0, 0 },
    { 0xcd700ee2, 36 },
    { 0, 0 },
    { 0x17500ee4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4500ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f00eec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d00ef0, 61 },
    { 0xd6c00ef1, 91 },
    { 0, 0 },
    { 0xf1200ef3, 34 },
    { 0xcdf00ef3, 99 },
    { 0x7f700ef5, 61 },
    { 0xbc700ef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68700efa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd800f01, 95 },
    { 0x72000f02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00f07, 23 },
    { 0xbd400f08, 78 },
    { 0x46200f09, 43 },
    { 0x38c00f09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21000f0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8300f11, 33 },
    { 0, 0 },
    { 0x2f000f13, 34 },
    { 0x9ab00f14, 19 },
    { 0, 0 },
    { 0x29000f16, 90 },
    { 0, 0 },
    { 0x17e00f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5300f1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe00f23, 90 },
    { 0xd6000f24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af00f29, 90 },
    { 0, 0 },
    { 0xbd600f2b, 95 },
    { 0, 0 },
    { 0xa0800f2d, 23 },
    { 0, 0 },
    { 0xc4500f2f, 76 },
    { 0xc9200f2f, 98 },
    { 0, 0 },
    { 0xf8200f32, 34 },
    { 0xcfb00f33, 98 },
    { 0x85300f33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa500f3d, 52 },
    { 0xad000f3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14300f42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31800f46, 90 },
    { 0x34300f47, 33 },
    { 0x8e000f48, 23 },
    { 0, 0 },
    { 0xe000f4a, 91 },
    { 0x28300f4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e00f53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef00f56, 33 },
    { 0x66000f57, 90 },
    { 0x79a00f56, 91 },
    { 0xdb00f59, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa600f5f, 98 },
    { 0, 0 },
    { 0xcf500f61, 46 },
    { 0xf7d00f62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5400f66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d00f69, 33 },
    { 0x17100f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a00f6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a00f71, 76 },
    { 0x55700f71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10400f7a, 34 },
    { 0xdae00f7a, 98 },
    { 0x1ee00f7c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a00f80, 97 },
    { 0x6c900f80, 33 },
    { 0xabe00f82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56700f89, 98 },
    { 0x79a00f8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6000f91, 98 },
    { 0, 0 },
    { 0xf5100f93, 33 },
    { 0, 0 },
    { 0x59a00f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88500f98, 65 },
    { 0xa6500f99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e00fa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef00fa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4300fa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7600fb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde00fbd, 55 },
    { 0xe2300fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0700fc5, 61 },
    { 0xa2900fc6, 85 },
    { 0xf4500fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca000fca, 87 },
    { 0x15600fcb, 66 },
    { 0x7c000fca, 33 },
    { 0x77400fcd, 87 },
    { 0, 0 },
    { 0x5ea00fcf, 84 },
    { 0xf8c00fcf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5600fd5, 33 },
    { 0x3b300fd6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e00fde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac400fe1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b00fea, 95 },
    { 0, 0 },
    { 0x78900fec, 34 },
    { 0x1a00fec, 33 },
    { 0, 0 },
    { 0x74400fef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7800ff2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd00ff6, 90 },
    { 0xea900ff6, 100 },
    { 0xbd700ff8, 81 },
    { 0xbb600ff9, 95 },
    { 0x61500ff9, 84 },
    { 0, 0 },
    { 0xcd500ffc, 33 },
    { 0x22700ffc, 76 },
    { 0x5b800ffe, 40 },
    { 0xb9300ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba01003, 61 },
    { 0, 0 },
    { 0xc9f01005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3340100a, 33 },
    { 0xf3d0100b, 90 },
    { 0x26a0100b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570100f, 33 },
    { 0x37201010, 33 },
    { 0xdaf01011, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a01014, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d0101b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2610101f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2201025, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc01028, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde80102d, 76 },
    { 0xc910102d, 88 },
    { 0, 0 },
    { 0xe3401030, 78 },
    { 0, 0 },
    { 0x5da01032, 95 },
    { 0, 0 },
    { 0x86001034, 33 },
    { 0xef101035, 77 },
    { 0xcb201036, 89 },
    { 0xe8801037, 99 },
    { 0x34601037, 91 },
    { 0x9eb01036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b0103c, 23 },
    { 0, 0 },
    { 0x5a40103e, 61 },
    { 0, 0 },
    { 0xee901040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef301043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d01049, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4520104d, 33 },
    { 0x4390104d, 76 },
    { 0x52f0104f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4701058, 95 },
    { 0xb2601059, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8510105e, 95 },
    { 0, 0 },
    { 0x24301060, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa501064, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa101067, 85 },
    { 0x4a501068, 33 },
    { 0x31d01069, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2010106c, 61 },
    { 0x2200106d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1401070, 53 },
    { 0xa9001071, 78 },
    { 0x6501070, 95 },
    { 0x23b01071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4101078, 46 },
    { 0x20c01079, 52 },
    { 0x98301079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d0107e, 80 },
    { 0xc050107f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1370108e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be01091, 90 },
    { 0, 0 },
    { 0x62c01093, 98 },
    { 0x71f01093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe01098, 87 },
    { 0xef501098, 85 },
    { 0, 0 },
    { 0xf230109b, 85 },
    { 0x73f0109c, 23 },
    { 0x9c30109d, 66 },
    { 0, 0 },
    { 0xdc90109f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27010a4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a010a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71010ab, 34 },
    { 0x128010ac, 61 },
    { 0xc19010ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6010b0, 38 },
    { 0x24d010b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66010b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78010b8, 84 },
    { 0, 0 },
    { 0x869010ba, 61 },
    { 0x1da010bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0010be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8010c5, 61 },
    { 0x4de010c6, 51 },
    { 0x245010c7, 90 },
    { 0xfec010c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957010cc, 33 },
    { 0xb97010cd, 61 },
    { 0xcb4010ce, 61 },
    { 0x233010ce, 61 },
    { 0, 0 },
    { 0x215010d1, 90 },
    { 0x49f010d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x417010d5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68010da, 98 },
    { 0x82d010da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150010e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409010e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f010ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6010f0, 53 },
    { 0, 0 },
    { 0xb6c010f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467010f8, 91 },
    { 0xf33010f8, 66 },
    { 0, 0 },
    { 0xe61010fb, 91 },
    { 0x36e010fc, 33 },
    { 0x3cd010fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa301100, 95 },
    { 0x67e01101, 56 },
    { 0xba201101, 90 },
    { 0xb8701103, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c801106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77101109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac50110c, 87 },
    { 0x81a0110c, 90 },
    { 0x3920110e, 85 },
    { 0xdcb0110f, 98 },
    { 0, 0 },
    { 0x58c01111, 99 },
    { 0x48c01112, 65 },
    { 0x8fc01113, 66 },
    { 0x66901113, 33 },
    { 0xdd501115, 90 },
    { 0xcfe01116, 47 },
    { 0x55101117, 87 },
    { 0, 0 },
    { 0x4f601119, 78 },
    { 0x33d0111a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13201120, 87 },
    { 0xd3401121, 95 },
    { 0x64e01121, 55 },
    { 0xf8701123, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b01129, 90 },
    { 0x28001129, 93 },
    { 0x3300112b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae0112f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1101132, 93 },
    { 0x43d01133, 100 },
    { 0xc7101134, 52 },
    { 0x55a01133, 34 },
    { 0x92501134, 55 },
    { 0, 0 },
    { 0x3bc01138, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda00113c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b01141, 90 },
    { 0xe2501142, 23 },
    { 0x25401143, 90 },
    { 0xf3d01143, 61 },
    { 0, 0 },
    { 0x7c001146, 56 },
    { 0x82101147, 34 },
    { 0xf3301146, 33 },
    { 0x80201149, 76 },
    { 0xc0a0114a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc580114e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b01151, 34 },
    { 0xafd01152, 100 },
    { 0xc7401152, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d01157, 33 },
    { 0xe4f01157, 61 },
    { 0x31201157, 90 },
    { 0, 0 },
    { 0x5880115b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d01161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24701167, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c50116b, 23 },
    { 0x66f0116c, 85 },
    { 0x9a10116b, 46 },
    { 0xf900116b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9701174, 76 },
    { 0x6c301174, 77 },
    { 0x3e101175, 55 },
    { 0x20f01175, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d0117b, 33 },
    { 0, 0 },
    { 0x4410117d, 65 },
    { 0x45c0117e, 85 },
    { 0, 0 },
    { 0x7a501180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb01188, 76 },
    { 0x15301189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9800118d, 68 },
    { 0x8e00118e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a01197, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc210119c, 90 },
    { 0x3320119c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7011a0, 66 },
    { 0xf92011a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2011a7, 38 },
    { 0xe34011a7, 91 },
    { 0xa1b011a8, 69 },
    { 0xba2011aa, 52 },
    { 0xad1011ab, 88 },
    { 0, 0 },
    { 0xe8011ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a011b4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed011b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f011bb, 78 },
    { 0x9bb011bc, 99 },
    { 0xd3d011bd, 90 },
    { 0x3f2011bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24011c1, 77 },
    { 0x99b011c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398011c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43011cb, 55 },
    { 0, 0 },
    { 0xfc7011cd, 78 },
    { 0, 0 },
    { 0xc58011cf, 76 },
    { 0xbfa011cf, 61 },
    { 0, 0 },
    { 0x30d011d2, 77 },
    { 0, 0 },
    { 0x7af011d4, 95 },
    { 0xc4f011d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd011d9, 99 },
    { 0x141011d9, 90 },
    { 0xb9a011d9, 76 },
    { 0xacf011dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120011e1, 61 },
    { 0, 0 },
    { 0x139011e3, 34 },
    { 0x2b8011e4, 33 },
    { 0, 0 },
    { 0xd02011e6, 40 },
    { 0x108011e7, 90 },
    { 0xd63011e7, 23 },
    { 0x416011e9, 43 },
    { 0, 0 },
    { 0xbd2011eb, 34 },
    { 0xb68011ec, 90 },
    { 0, 0 },
    { 0x73c011ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f011f2, 90 },
    { 0, 0 },
    { 0x80f011f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952011f8, 61 },
    { 0, 0 },
    { 0x8a6011fa, 98 },
    { 0x979011fb, 91 },
    { 0x878011fc, 61 },
    { 0x56011fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b01201, 61 },
    { 0xb2e01202, 34 },
    { 0x8f301203, 90 },
    { 0x64001202, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe701207, 68 },
    { 0x3d001207, 46 },
    { 0, 0 },
    { 0x5a40120a, 23 },
    { 0x4fa0120b, 90 },
    { 0x15d0120c, 34 },
    { 0x9110120d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd501210, 98 },
    { 0, 0 },
    { 0x25701212, 76 },
    { 0x57701213, 23 },
    { 0xb1e01214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db01217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0121c, 43 },
    { 0x5b70121d, 68 },
    { 0x9cc0121d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee01221, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70122b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f01232, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f201235, 84 },
    { 0x73a01235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc01239, 90 },
    { 0xeee01239, 95 },
    { 0xeb001239, 65 },
    { 0, 0 },
    { 0x5960123d, 98 },
    { 0, 0 },
    { 0xae80123f, 76 },
    { 0, 0 },
    { 0x99301241, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c01244, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0101247, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2350124c, 98 },
    { 0, 0 },
    { 0x6d30124e, 17 },
    { 0, 0 },
    { 0x5401250, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47301253, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9610125e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6101263, 90 },
    { 0xa0201264, 90 },
    { 0xbcb01263, 78 },
    { 0x35601266, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b501269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e80126d, 34 },
    { 0x9aa0126e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb01277, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4740127f, 43 },
    { 0x71701280, 23 },
    { 0xfbe01281, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e01285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde90128a, 66 },
    { 0x6ed0128b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa290128e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b701291, 78 },
    { 0, 0 },
    { 0xf9501293, 46 },
    { 0x23401294, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b0129b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4012a1, 17 },
    { 0xa07012a2, 98 },
    { 0xda4012a1, 98 },
    { 0, 0 },
    { 0xd03012a5, 98 },
    { 0xec8012a5, 90 },
    { 0xcfd012a5, 91 },
    { 0, 0 },
    { 0x5e012a9, 33 },
    { 0, 0 },
    { 0xf89012ab, 52 },
    { 0xae2012ac, 93 },
    { 0x68d012ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9012b0, 52 },
    { 0xa7b012b0, 46 },
    { 0, 0 },
    { 0xc12012b3, 43 },
    { 0x355012b4, 90 },
    { 0x571012b5, 67 },
    { 0x94d012b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c012c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef012c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334012c8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x140012cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e012cf, 61 },
    { 0x7ad012d0, 66 },
    { 0x686012cf, 76 },
    { 0xf96012d2, 68 },
    { 0xee8012d3, 98 },
    { 0x42a012d2, 100 },
    { 0x390012d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb012db, 90 },
    { 0x239012dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663012e2, 67 },
    { 0x9bd012e3, 67 },
    { 0x234012e4, 95 },
    { 0xc93012e5, 33 },
    { 0x61b012e6, 76 },
    { 0xe5a012e6, 68 },
    { 0x136012e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c012ee, 34 },
    { 0xd47012ef, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03012f5, 84 },
    { 0xd1c012f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af012fb, 34 },
    { 0, 0 },
    { 0xbed012fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26401301, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff01305, 83 },
    { 0xd9c01305, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43801310, 33 },
    { 0x6b501310, 90 },
    { 0xcae01312, 40 },
    { 0xa0001310, 95 },
    { 0x51a01312, 23 },
    { 0xd4701312, 90 },
    { 0x5ea01311, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c10131b, 55 },
    { 0xf070131b, 43 },
    { 0x8ba0131d, 33 },
    { 0xbee0131e, 61 },
    { 0x5dd0131c, 61 },
    { 0, 0 },
    { 0x82801321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb901324, 95 },
    { 0, 0 },
    { 0x87301326, 36 },
    { 0xaaa01327, 61 },
    { 0, 0 },
    { 0x88501329, 33 },
    { 0xd3e0132a, 90 },
    { 0xcc50132b, 33 },
    { 0x36e0132c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef30132f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e01333, 23 },
    { 0x8bd01334, 95 },
    { 0, 0 },
    { 0x5af01336, 55 },
    { 0x4b301337, 23 },
    { 0, 0 },
    { 0xbe001339, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0133d, 85 },
    { 0xcf10133d, 55 },
    { 0, 0 },
    { 0x6301340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e101343, 90 },
    { 0, 0 },
    { 0x11801345, 90 },
    { 0, 0 },
    { 0x1a401347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b0134b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa90134f, 33 },
    { 0xdfc01350, 90 },
    { 0xc0f01351, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98101354, 61 },
    { 0x2af01355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db01359, 84 },
    { 0, 0 },
    { 0xac40135b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8801366, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6080136a, 55 },
    { 0xdc90136b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe840136e, 61 },
    { 0xe080136f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b101375, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e401379, 55 },
    { 0x1ec0137a, 23 },
    { 0, 0 },
    { 0xcf20137c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b80137f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5901384, 66 },
    { 0x71301385, 91 },
    { 0, 0 },
    { 0xbab01387, 98 },
    { 0x1a501387, 24 },
    { 0, 0 },
    { 0xf290138a, 81 },
    { 0x1470138b, 95 },
    { 0xc00138a, 90 },
    { 0xd260138d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e01394, 34 },
    { 0x2ec01394, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf501398, 87 },
    { 0xf2601399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b0139e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946013a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff013a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24013aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4013ae, 8 },
    { 0xf34013af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d013b4, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0013b7, 61 },
    { 0, 0 },
    { 0x5d3013b9, 33 },
    { 0x12a013ba, 55 },
    { 0xbdc013bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4013be, 95 },
    { 0x9cc013be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4013c2, 61 },
    { 0x614013c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3013ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e013cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99013d4, 61 },
    { 0x49b013d5, 88 },
    { 0xea7013d5, 55 },
    { 0xe1013d7, 65 },
    { 0, 0 },
    { 0x250013d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe013dd, 23 },
    { 0xc84013dd, 33 },
    { 0x71a013de, 43 },
    { 0xdce013e0, 98 },
    { 0x210013e0, 33 },
    { 0x5c6013e1, 33 },
    { 0, 0 },
    { 0xad6013e4, 98 },
    { 0xb79013e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5013e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440013ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757013f0, 90 },
    { 0x1c0013f1, 87 },
    { 0, 0 },
    { 0xd72013f3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5013fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26701404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd001408, 36 },
    { 0xe1f01408, 95 },
    { 0x90f0140a, 23 },
    { 0, 0 },
    { 0x1aa0140c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8210140f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d01412, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5901415, 46 },
    { 0xda01416, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a00141f, 90 },
    { 0x7a70141f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da01426, 95 },
    { 0xa3901426, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa40142c, 61 },
    { 0x61a0142c, 87 },
    { 0xd90142e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42301433, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e01438, 33 },
    { 0, 0 },
    { 0x4c60143a, 61 },
    { 0x5e80143a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb80143e, 66 },
    { 0xbda0143f, 76 },
    { 0xfb901440, 90 },
    { 0xb340143e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8901445, 90 },
    { 0x22b01446, 76 },
    { 0x59701445, 88 },
    { 0xab001448, 91 },
    { 0xa4101449, 33 },
    { 0xae301447, 95 },
    { 0x97a01446, 90 },
    { 0xdfe0144c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a701452, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d01458, 75 },
    { 0xebe01459, 95 },
    { 0, 0 },
    { 0xe6d0145b, 90 },
    { 0x9260145c, 61 },
    { 0x7bd0145c, 95 },
    { 0xe350145c, 90 },
    { 0xcbe0145c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68701463, 93 },
    { 0, 0 },
    { 0xce501465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b01469, 34 },
    { 0xba01469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d90146f, 34 },
    { 0xf7f01470, 61 },
    { 0xbc401471, 33 },
    { 0x47501470, 95 },
    { 0x5ea01471, 43 },
    { 0x33101472, 43 },
    { 0x9a101470, 33 },
    { 0x77701472, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd90147b, 90 },
    { 0, 0 },
    { 0xebf0147d, 95 },
    { 0, 0 },
    { 0x6a0147f, 98 },
    { 0xce01480, 43 },
    { 0, 0 },
    { 0x2d901482, 59 },
    { 0x4e101483, 98 },
    { 0x3e01484, 88 },
    { 0x1c701483, 46 },
    { 0x87201486, 91 },
    { 0xd3401487, 95 },
    { 0x9c001488, 99 },
    { 0xaec01489, 93 },
    { 0x8b01488, 76 },
    { 0x9a70148b, 33 },
    { 0xdfc0148c, 43 },
    { 0x2b50148d, 61 },
    { 0xaa20148d, 55 },
    { 0x7d201487, 90 },
    { 0xf9c01490, 61 },
    { 0x8310148a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50301497, 34 },
    { 0, 0 },
    { 0x99d01499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d0149e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938014a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb014aa, 43 },
    { 0, 0 },
    { 0x33b014ac, 87 },
    { 0x2f6014ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98014b3, 98 },
    { 0, 0 },
    { 0x738014b5, 98 },
    { 0x2bc014b5, 91 },
    { 0, 0 },
    { 0x615014b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5014bd, 90 },
    { 0x883014be, 52 },
    { 0, 0 },
    { 0x133014c0, 61 },
    { 0x407014c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b014c6, 33 },
    { 0, 0 },
    { 0x1ef014c8, 23 },
    { 0x100014c9, 80 },
    { 0x93d014ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40014ce, 90 },
    { 0, 0 },
    { 0x5c1014d0, 55 },
    { 0x649014d1, 52 },
    { 0x332014d2, 67 },
    { 0xffa014d0, 33 },
    { 0x540014d2, 85 },
    { 0, 0 },
    { 0xdbb014d6, 91 },
    { 0xe2f014d6, 93 },
    { 0x855014d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234014df, 34 },
    { 0xea2014df, 61 },
    { 0x333014df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b014e4, 23 },
    { 0, 0 },
    { 0x438014e6, 33 },
    { 0xc92014e6, 98 },
    { 0x680014e8, 95 },
    { 0x8d7014e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22014ec, 90 },
    { 0, 0 },
    { 0xea2014ee, 78 },
    { 0xfe7014ef, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89014f3, 90 },
    { 0x204014f4, 68 },
    { 0xa5f014f5, 55 },
    { 0xebb014f6, 34 },
    { 0, 0 },
    { 0xd90014f8, 90 },
    { 0x4a0014f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da014fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3001501, 77 },
    { 0, 0 },
    { 0x35c01503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c0150e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc01514, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3801517, 61 },
    { 0, 0 },
    { 0x26601519, 98 },
    { 0, 0 },
    { 0xf060151b, 80 },
    { 0, 0 },
    { 0x71c0151d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43801522, 23 },
    { 0x58701523, 34 },
    { 0xd1901524, 34 },
    { 0x92401524, 90 },
    { 0x59101526, 98 },
    { 0xbd001524, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf00152b, 90 },
    { 0, 0 },
    { 0x2400152d, 91 },
    { 0xa9c0152e, 77 },
    { 0x1720152e, 83 },
    { 0x3a0152d, 98 },
    { 0, 0 },
    { 0x1ff01532, 88 },
    { 0x18201533, 33 },
    { 0, 0 },
    { 0x4c001535, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x56601538, 33 },
    { 0, 0 },
    { 0x81d0153a, 61 },
    { 0, 0 },
    { 0x3140153c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2701540, 95 },
    { 0, 0 },
    { 0xe6201542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d01548, 90 },
    { 0x21d01549, 100 },
    { 0x2d001549, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed90154e, 98 },
    { 0x8d20154e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d01552, 52 },
    { 0x26c01553, 93 },
    { 0xecf01554, 90 },
    { 0x3ce01554, 95 },
    { 0x87201556, 99 },
    { 0xd5201557, 34 },
    { 0xbbf01555, 33 },
    { 0xfe501555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff701560, 61 },
    { 0x2101561, 33 },
    { 0x41301562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40801566, 76 },
    { 0xb2e01567, 33 },
    { 0x19d01568, 98 },
    { 0, 0 },
    { 0x15a0156a, 85 },
    { 0, 0 },
    { 0xead0156c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3201570, 67 },
    { 0, 0 },
    { 0xbab01572, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c01576, 67 },
    { 0x1fb01577, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa680157a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b0157e, 91 },
    { 0x6c60157e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af01583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f01586, 33 },
    { 0xc2201587, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de0158a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d0158e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4101597, 90 },
    { 0, 0 },
    { 0x50701599, 99 },
    { 0x8320159a, 61 },
    { 0x88d0159a, 23 },
    { 0x1dd0159b, 34 },
    { 0xc7e0159d, 98 },
    { 0x8ee0159d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b015a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f015a6, 23 },
    { 0xbdd015a6, 23 },
    { 0x5015a6, 90 },
    { 0x71c015a9, 90 },
    { 0, 0 },
    { 0x538015ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480015b1, 95 },
    { 0xb1015b2, 93 },
    { 0, 0 },
    { 0xa2b015b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565015be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x587015c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea015c4, 90 },
    { 0xa06015c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434015c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b015cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315015d2, 76 },
    { 0, 0 },
    { 0xd90015d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67015d7, 98 },
    { 0x9b0015d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829015df, 76 },
    { 0x9ed015e0, 98 },
    { 0xbb3015df, 90 },
    { 0xa24015e2, 93 },
    { 0x84d015e3, 98 },
    { 0, 0 },
    { 0x94c015e5, 90 },
    { 0x40f015e6, 93 },
    { 0xb7f015e7, 95 },
    { 0xb8c015e5, 61 },
    { 0xe91015e9, 84 },
    { 0xa5f015ea, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82015ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc015f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95015fc, 91 },
    { 0, 0 },
    { 0xa2015fe, 33 },
    { 0, 0 },
    { 0xc7701600, 43 },
    { 0, 0 },
    { 0xd6801602, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c01606, 61 },
    { 0x1e501607, 66 },
    { 0xeaa01608, 33 },
    { 0xa5e01606, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e0160f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89801613, 46 },
    { 0xe3901613, 75 },
    { 0x8bc01615, 61 },
    { 0x4c01615, 78 },
    { 0x46c01617, 61 },
    { 0x5c101616, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d0161b, 17 },
    { 0x4550161c, 34 },
    { 0x6860161d, 85 },
    { 0, 0 },
    { 0x8590161f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72301624, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b01627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb10162b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1350162e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42201632, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2001637, 33 },
    { 0, 0 },
    { 0xc6d01639, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c00163c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22101640, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4601645, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9150164d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c101650, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f701658, 98 },
    { 0xfc801659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1060165c, 66 },
    { 0, 0 },
    { 0x1440165e, 33 },
    { 0xb360165e, 33 },
    { 0xf2101660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8501666, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1001669, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d0166d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75801677, 93 },
    { 0, 0 },
    { 0x2e001679, 23 },
    { 0xb9c0167a, 80 },
    { 0, 0 },
    { 0x1e80167c, 65 },
    { 0x810167d, 90 },
    { 0x52a0167c, 95 },
    { 0x4d90167c, 75 },
    { 0, 0 },
    { 0xddc01681, 66 },
    { 0x17f01682, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a601685, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b01690, 61 },
    { 0, 0 },
    { 0x5c01692, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a01695, 90 },
    { 0x5f501696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7450169e, 94 },
    { 0x7a50169f, 61 },
    { 0x807016a0, 95 },
    { 0x6ea016a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a016a7, 61 },
    { 0x9e6016a8, 92 },
    { 0xc4e016a9, 90 },
    { 0x1ee016aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a016ae, 61 },
    { 0xb15016af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a016b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b016b7, 95 },
    { 0x45d016b8, 55 },
    { 0xa00016b9, 95 },
    { 0x9bc016b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca016c1, 55 },
    { 0xf44016c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9016c5, 91 },
    { 0x6a2016c6, 55 },
    { 0xcb8016c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae016cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96016cf, 34 },
    { 0x8ae016cf, 95 },
    { 0, 0 },
    { 0x4ff016d2, 66 },
    { 0, 0 },
    { 0xff9016d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8016d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6016da, 33 },
    { 0xd11016da, 93 },
    { 0, 0 },
    { 0xe6b016dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b016e1, 61 },
    { 0, 0 },
    { 0x2d016e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95016e6, 34 },
    { 0xf1e016e7, 43 },
    { 0, 0 },
    { 0x563016e9, 23 },
    { 0x1c9016ea, 33 },
    { 0, 0 },
    { 0xfe5016ec, 95 },
    { 0, 0 },
    { 0xad4016ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8016f1, 90 },
    { 0x3ea016f1, 98 },
    { 0x7a0016f3, 23 },
    { 0, 0 },
    { 0xecf016f5, 33 },
    { 0xa45016f5, 56 },
    { 0x132016f7, 93 },
    { 0x6ea016f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82016fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e301705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f0170b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed40170f, 95 },
    { 0xd5c01710, 68 },
    { 0x61701711, 34 },
    { 0xf8d01712, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec801716, 43 },
    { 0x95c01717, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e01720, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x94201723, 98 },
    { 0xca801724, 36 },
    { 0x2d701724, 95 },
    { 0x43101726, 85 },
    { 0x24601727, 100 },
    { 0x15f01728, 76 },
    { 0xd9401723, 61 },
    { 0xf5c0172a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2670172e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd01733, 33 },
    { 0xae501733, 23 },
    { 0x8a401735, 76 },
    { 0x44501735, 100 },
    { 0x81001737, 98 },
    { 0x79701734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3970173d, 88 },
    { 0xbbe0173e, 61 },
    { 0xbb90173f, 43 },
    { 0xa0d0173e, 90 },
    { 0x3420173d, 90 },
    { 0x98101742, 61 },
    { 0x49901742, 23 },
    { 0x6101744, 33 },
    { 0x8bc01744, 65 },
    { 0x10901745, 87 },
    { 0x36001747, 98 },
    { 0xbbd01748, 90 },
    { 0xc6501747, 95 },
    { 0x8ee01743, 34 },
    { 0x78d01748, 64 },
    { 0xeba0173e, 90 },
    { 0, 0 },
    { 0xef00174e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c01751, 53 },
    { 0, 0 },
    { 0x34301753, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180175c, 95 },
    { 0x68e0175d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86501761, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c01765, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf380176d, 55 },
    { 0x7390176e, 23 },
    { 0x83c0176f, 100 },
    { 0x5d201770, 95 },
    { 0x4d60176d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c01777, 90 },
    { 0xe6e01777, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa00177b, 43 },
    { 0x1630177c, 90 },
    { 0x6910177d, 93 },
    { 0, 0 },
    { 0xcaa0177f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce901784, 77 },
    { 0, 0 },
    { 0x60801786, 95 },
    { 0xfd901786, 90 },
    { 0, 0 },
    { 0x42b01789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda10178c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb070178f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32701794, 61 },
    { 0x54201795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1801798, 77 },
    { 0xb3d01799, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d0179d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8017a3, 61 },
    { 0, 0 },
    { 0xf32017a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb017a9, 97 },
    { 0x507017aa, 84 },
    { 0x2a4017a9, 98 },
    { 0, 0 },
    { 0x6ff017ad, 95 },
    { 0xeb017ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8017b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32017ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48017bd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52017c4, 33 },
    { 0xfd017c5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f017cc, 61 },
    { 0x29e017cd, 90 },
    { 0x67017cd, 93 },
    { 0xad9017cd, 55 },
    { 0xa37017cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e017d3, 34 },
    { 0, 0 },
    { 0x91c017d5, 98 },
    { 0xa53017d5, 90 },
    { 0x298017d5, 92 },
    { 0x407017d8, 75 },
    { 0, 0 },
    { 0x36e017da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a017e1, 33 },
    { 0x5cc017e2, 84 },
    { 0x576017e3, 78 },
    { 0x13d017e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870017ea, 99 },
    { 0, 0 },
    { 0x9bb017ec, 76 },
    { 0, 0 },
    { 0xeb7017ee, 78 },
    { 0, 0 },
    { 0xfe1017f0, 90 },
    { 0x88e017f1, 90 },
    { 0x3e8017f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd017f8, 85 },
    { 0, 0 },
    { 0xeae017fa, 76 },
    { 0xfab017fb, 52 },
    { 0x975017fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7501800, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b01804, 69 },
    { 0, 0 },
    { 0xf3e01806, 78 },
    { 0, 0 },
    { 0xd1301808, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc0180c, 46 },
    { 0, 0 },
    { 0x9e90180e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f901815, 52 },
    { 0xa2101816, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0181b, 80 },
    { 0, 0 },
    { 0xcd80181d, 23 },
    { 0x52f0181e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2701821, 91 },
    { 0xca601822, 76 },
    { 0xdb101823, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf01828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c0182b, 23 },
    { 0x41f0182c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b01830, 91 },
    { 0xf1b01831, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae01835, 98 },
    { 0, 0 },
    { 0x4a101837, 46 },
    { 0, 0 },
    { 0x32101839, 98 },
    { 0x7420183a, 98 },
    { 0, 0 },
    { 0x20183c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd840183f, 34 },
    { 0xc9601840, 61 },
    { 0xb4c01840, 93 },
    { 0xf1101842, 33 },
    { 0x28c01843, 98 },
    { 0xd2001842, 33 },
    { 0x68101841, 61 },
    { 0xa4601846, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb940184a, 93 },
    { 0xa050184a, 61 },
    { 0xbd40184c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd50184f, 90 },
    { 0xa4301850, 95 },
    { 0x4b901851, 95 },
    { 0x85901852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd701855, 98 },
    { 0x33801855, 61 },
    { 0xa1e01857, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0185b, 33 },
    { 0, 0 },
    { 0x7830185d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43401863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1510186b, 61 },
    { 0x6fe0186c, 90 },
    { 0x2f70186c, 76 },
    { 0, 0 },
    { 0x6830186f, 43 },
    { 0xc7501870, 34 },
    { 0x60101871, 17 },
    { 0xe0001871, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6201879, 76 },
    { 0xdaf01879, 98 },
    { 0x6be01879, 98 },
    { 0x4140187b, 67 },
    { 0x7560187d, 8 },
    { 0, 0 },
    { 0xfc50187f, 55 },
    { 0xf601880, 61 },
    { 0x65c01880, 55 },
    { 0xefb01882, 33 },
    { 0xd1801883, 78 },
    { 0xfe301881, 33 },
    { 0x69501881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc0188c, 33 },
    { 0xde50188c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a01892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46601896, 91 },
    { 0x1dc01897, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f0189d, 33 },
    { 0, 0 },
    { 0xc620189f, 87 },
    { 0, 0 },
    { 0x967018a1, 76 },
    { 0x50d018a1, 90 },
    { 0, 0 },
    { 0x511018a4, 98 },
    { 0x566018a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a018a8, 23 },
    { 0xbbf018a8, 43 },
    { 0xdb8018aa, 76 },
    { 0x36b018ab, 23 },
    { 0x410018ac, 99 },
    { 0x63c018ab, 94 },
    { 0xf4018ac, 34 },
    { 0x697018aa, 23 },
    { 0x8d1018b0, 85 },
    { 0x91b018b0, 61 },
    { 0x49b018b2, 61 },
    { 0xd95018b3, 95 },
    { 0x37e018b0, 95 },
    { 0, 0 },
    { 0x7f7018b6, 95 },
    { 0xcbb018b6, 90 },
    { 0x17b018b8, 33 },
    { 0, 0 },
    { 0x432018ba, 33 },
    { 0xd04018bb, 90 },
    { 0x92c018bc, 33 },
    { 0, 0 },
    { 0x7d8018be, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57018c1, 61 },
    { 0x16f018c2, 77 },
    { 0xd29018c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632018c8, 76 },
    { 0x348018c8, 33 },
    { 0, 0 },
    { 0x1f9018cb, 91 },
    { 0x5c5018cc, 55 },
    { 0xb9c018cc, 84 },
    { 0x1e7018ce, 98 },
    { 0x38f018cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1018d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44018d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0018dc, 90 },
    { 0x67a018dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x419018e0, 46 },
    { 0x627018e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x428018e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70018e7, 95 },
    { 0x941018e8, 93 },
    { 0x581018e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886018ed, 61 },
    { 0x67d018ee, 77 },
    { 0, 0 },
    { 0x460018f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7018f6, 43 },
    { 0xfe2018f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b018fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2018fe, 95 },
    { 0xc62018ff, 33 },
    { 0x5cf018fe, 90 },
    { 0xafe018fe, 90 },
    { 0x55d01900, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d201905, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c01908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc470190c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d601912, 90 },
    { 0x65801913, 95 },
    { 0xe4f01912, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3890191a, 46 },
    { 0xfcf0191b, 33 },
    { 0x8aa0191c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d0191f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18101924, 61 },
    { 0, 0 },
    { 0x56201926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c0192c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc01931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c201934, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9301939, 90 },
    { 0xb670193a, 90 },
    { 0x5270193a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70601940, 84 },
    { 0, 0 },
    { 0x9001942, 46 },
    { 0x6f801943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a0194a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a0194e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e301953, 67 },
    { 0x8ba01954, 88 },
    { 0xbee01955, 91 },
    { 0, 0 },
    { 0x34801957, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd0195b, 100 },
    { 0x89f0195c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3201962, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c601968, 90 },
    { 0x5d301968, 91 },
    { 0, 0 },
    { 0x6f20196b, 55 },
    { 0x63b0196b, 76 },
    { 0, 0 },
    { 0xad80196e, 90 },
    { 0, 0 },
    { 0x8ee01970, 43 },
    { 0x33801971, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1101974, 66 },
    { 0, 0 },
    { 0xa4001976, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4730197a, 96 },
    { 0, 0 },
    { 0xbfa0197c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7150197f, 23 },
    { 0xb8201980, 99 },
    { 0x39f01981, 33 },
    { 0x2801982, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c01985, 61 },
    { 0x8ab01986, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5990198d, 67 },
    { 0xf100198e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93301991, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9710199e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a019a1, 61 },
    { 0x790019a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f019a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc019a8, 95 },
    { 0x173019a8, 61 },
    { 0, 0 },
    { 0x5cf019ab, 98 },
    { 0, 0 },
    { 0xde8019ad, 90 },
    { 0xfd2019ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8019b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f019b8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58019bb, 100 },
    { 0x317019bc, 100 },
    { 0x418019bc, 93 },
    { 0x492019be, 69 },
    { 0x5ed019bf, 92 },
    { 0x65b019bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456019c4, 55 },
    { 0x920019c4, 95 },
    { 0xc35019c6, 98 },
    { 0xfe8019c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1019ca, 90 },
    { 0, 0 },
    { 0x358019cc, 46 },
    { 0x4fe019cd, 33 },
    { 0xb3b019ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2019d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2019d6, 95 },
    { 0x9e9019d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d019db, 81 },
    { 0x6ce019dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1019e1, 55 },
    { 0x29019e2, 66 },
    { 0, 0 },
    { 0x20a019e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b019e7, 76 },
    { 0, 0 },
    { 0x1a4019e9, 66 },
    { 0, 0 },
    { 0xec6019eb, 33 },
    { 0x915019ec, 33 },
    { 0x4ac019ec, 23 },
    { 0xd99019ee, 99 },
    { 0, 0 },
    { 0xce2019f0, 38 },
    { 0, 0 },
    { 0x4b019f2, 80 },
    { 0x8fa019f3, 85 },
    { 0xfa2019f4, 75 },
    { 0x265019f4, 76 },
    { 0x37c019f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab019f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76401a00, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca01a05, 76 },
    { 0, 0 },
    { 0x32d01a07, 90 },
    { 0, 0 },
    { 0xb3501a09, 43 },
    { 0, 0 },
    { 0x81601a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1101a11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb01a18, 76 },
    { 0, 0 },
    { 0xbfb01a1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66601a20, 61 },
    { 0x61b01a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3801a26, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a001a2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77401a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a01a35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73801a3a, 23 },
    { 0x73d01a3b, 46 },
    { 0, 0 },
    { 0xcaa01a3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50901a42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d01a48, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff01a4e, 33 },
    { 0x48e01a4f, 87 },
    { 0, 0 },
    { 0xb7f01a51, 37 },
    { 0, 0 },
    { 0x1bb01a53, 87 },
    { 0x98201a54, 34 },
    { 0x32801a53, 98 },
    { 0xd9e01a56, 33 },
    { 0x60301a53, 90 },
    { 0x6f101a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c701a5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb01a5e, 34 },
    { 0xaa101a5f, 98 },
    { 0x92901a5f, 33 },
    { 0xd7001a60, 61 },
    { 0x54501a60, 90 },
    { 0x1f001a63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63301a66, 87 },
    { 0xc0401a67, 34 },
    { 0xb0301a68, 61 },
    { 0x4df01a69, 43 },
    { 0xeec01a68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c01a70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c301a75, 80 },
    { 0, 0 },
    { 0x3b501a77, 98 },
    { 0, 0 },
    { 0xb0801a79, 61 },
    { 0x55601a7a, 76 },
    { 0x97a01a7b, 88 },
    { 0x14601a7c, 90 },
    { 0xee501a7b, 90 },
    { 0x95d01a7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28701a88, 98 },
    { 0, 0 },
    { 0xa1a01a8a, 80 },
    { 0xb6a01a8a, 95 },
    { 0xc1701a8b, 23 },
    { 0, 0 },
    { 0x9001a8e, 4 },
    { 0xb4101a8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb01a93, 93 },
    { 0x4701a94, 99 },
    { 0xffd01a95, 95 },
    { 0x29201a96, 98 },
    { 0x4c201a97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e101a9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7301aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47501aa3, 95 },
    { 0, 0 },
    { 0x7e301aa5, 55 },
    { 0xb7001aa6, 95 },
    { 0xc0001aa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e01aac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63501ab0, 33 },
    { 0x58001ab0, 76 },
    { 0x3301ab0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61001aba, 88 },
    { 0xfa401abb, 43 },
    { 0x2b901abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99701abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68201ac6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b01acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1501ad1, 98 },
    { 0x6801ad1, 90 },
    { 0x26101ad3, 98 },
    { 0xf3701ad1, 93 },
    { 0x46501ad1, 61 },
    { 0, 0 },
    { 0x87401ad7, 61 },
    { 0, 0 },
    { 0x3b901ad9, 87 },
    { 0x37d01ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d01ade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1801ae1, 23 },
    { 0xfcd01ae2, 43 },
    { 0x87b01ae3, 61 },
    { 0xaa801ae1, 98 },
    { 0, 0 },
    { 0xe401ae6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e101aef, 90 },
    { 0x73201af0, 100 },
    { 0xc2501af1, 52 },
    { 0x44b01af0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf401af5, 90 },
    { 0, 0 },
    { 0xce701af7, 100 },
    { 0xc1501af8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b01afc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5401aff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec01b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35901b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58901b16, 34 },
    { 0x37201b17, 23 },
    { 0x95a01b18, 95 },
    { 0xcd01b19, 33 },
    { 0x3e401b1a, 76 },
    { 0x9de01b16, 95 },
    { 0, 0 },
    { 0xc6e01b1d, 33 },
    { 0x42701b1d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a801b21, 90 },
    { 0, 0 },
    { 0x14801b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd01b29, 87 },
    { 0x3ad01b29, 84 },
    { 0, 0 },
    { 0x51901b2c, 90 },
    { 0xa5e01b2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46901b30, 61 },
    { 0, 0 },
    { 0xc7701b32, 76 },
    { 0, 0 },
    { 0x52e01b34, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60801b40, 33 },
    { 0, 0 },
    { 0x96801b42, 61 },
    { 0xfd001b43, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b01b4f, 90 },
    { 0xcc101b4f, 40 },
    { 0xfaa01b4f, 93 },
    { 0xd7301b52, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2501b57, 90 },
    { 0, 0 },
    { 0xabc01b59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2401b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b01b68, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb601b6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e01b71, 52 },
    { 0x7eb01b71, 90 },
    { 0xafa01b71, 95 },
    { 0x2ad01b74, 61 },
    { 0x3a401b73, 61 },
    { 0xe4401b76, 84 },
    { 0x10601b77, 91 },
    { 0xe9e01b75, 98 },
    { 0x1a801b78, 66 },
    { 0x83f01b77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e01b7d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48001b82, 61 },
    { 0, 0 },
    { 0xe1001b84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41601b8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c01b92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb01b97, 85 },
    { 0, 0 },
    { 0xcb601b99, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce01b9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca01b9f, 77 },
    { 0, 0 },
    { 0xa4001ba1, 23 },
    { 0xca201ba2, 61 },
    { 0xd2301ba3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6501ba8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b01bac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c301bb0, 93 },
    { 0x45301bb1, 33 },
    { 0xc9f01bb1, 87 },
    { 0x93001bb3, 55 },
    { 0xddf01bb4, 61 },
    { 0x95401bb5, 61 },
    { 0x47401bb6, 91 },
    { 0x6f901bb1, 91 },
    { 0, 0 },
    { 0xee301bb9, 34 },
    { 0xefc01bba, 91 },
    { 0xe0001bba, 61 },
    { 0, 0 },
    { 0xdd801bbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45901bc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a301bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb01bd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b701bd5, 33 },
    { 0x46d01bd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9001bda, 95 },
    { 0x35d01bda, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1901bdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82001be7, 55 },
    { 0x8d601be7, 90 },
    { 0x9ed01be9, 90 },
    { 0xfba01bea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c501bf1, 95 },
    { 0, 0 },
    { 0x68801bf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad01bf7, 55 },
    { 0xdc901bf8, 93 },
    { 0x4a601bf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e01bfd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a01c01, 98 },
    { 0, 0 },
    { 0x67201c03, 98 },
    { 0xc8e01c04, 56 },
    { 0x46e01c03, 78 },
    { 0x68801c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf601c0a, 46 },
    { 0xe8801c0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xced01c0e, 61 },
    { 0xe1801c0f, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd801c12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c01c15, 85 },
    { 0xec501c15, 98 },
    { 0x37d01c17, 87 },
    { 0x61101c17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0101c1e, 23 },
    { 0xcb801c1f, 33 },
    { 0xead01c1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79801c23, 23 },
    { 0x96301c24, 61 },
    { 0x24501c23, 67 },
    { 0, 0 },
    { 0x11801c27, 61 },
    { 0xdc101c28, 75 },
    { 0xa5101c29, 99 },
    { 0x94d01c27, 23 },
    { 0x99b01c28, 95 },
    { 0, 0 },
    { 0x5601c2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e101c37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc701c3f, 90 },
    { 0, 0 },
    { 0x46e01c41, 68 },
    { 0, 0 },
    { 0x59201c43, 52 },
    { 0x23001c44, 52 },
    { 0x39a01c45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46401c4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5001c4d, 52 },
    { 0, 0 },
    { 0xdfd01c4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f01c52, 91 },
    { 0x96301c53, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c01c57, 90 },
    { 0x4e501c57, 61 },
    { 0x11801c57, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x57301c5c, 99 },
    { 0x9e701c5d, 61 },
    { 0x6e901c5d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xef501c61, 93 },
    { 0xb6201c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b01c66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c01c6d, 87 },
    { 0xb0d01c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a01c71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc01c74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28201c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d701c7f, 65 },
    { 0, 0 },
    { 0x25901c81, 90 },
    { 0, 0 },
    { 0x1aa01c83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5801c87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6001c8b, 61 },
    { 0x3e801c8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd01c90, 93 },
    { 0xc2001c90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f801c96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c01c99, 23 },
    { 0x86201c99, 80 },
    { 0xb1501c9b, 23 },
    { 0x29601c9c, 40 },
    { 0xa9901c9b, 55 },
    { 0x8b701c9e, 98 },
    { 0x2d601c9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30801ca3, 67 },
    { 0, 0 },
    { 0x4501ca5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d01ca8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9501cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a501cb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e01cb6, 61 },
    { 0x68101cb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde601cbb, 91 },
    { 0, 0 },
    { 0x41e01cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83101cc0, 23 },
    { 0, 0 },
    { 0xe1801cc2, 90 },
    { 0, 0 },
    { 0xd9101cc4, 93 },
    { 0xfcb01cc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa001cc8, 53 },
    { 0xf5901cc9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad01ccc, 34 },
    { 0xfee01ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b701cd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6601cdc, 61 },
    { 0xc5701cdc, 43 },
    { 0, 0 },
    { 0xbd101cdf, 33 },
    { 0, 0 },
    { 0x8e001ce1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97001ce4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a01ce9, 87 },
    { 0xd8401cea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1601cf1, 33 },
    { 0x43501cf2, 91 },
    { 0, 0 },
    { 0xcab01cf4, 33 },
    { 0x87601cf4, 61 },
    { 0xcd701cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24701cfb, 76 },
    { 0xc6c01cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc401cff, 61 },
    { 0, 0 },
    { 0x99801d01, 77 },
    { 0x3b101d01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41701d08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca01d11, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32801d18, 98 },
    { 0, 0 },
    { 0xc7b01d1a, 34 },
    { 0xd0f01d1a, 47 },
    { 0xf4001d1c, 76 },
    { 0x68201d1c, 23 },
    { 0x8f001d1e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df01d21, 95 },
    { 0x9b501d22, 76 },
    { 0x27d01d23, 98 },
    { 0x17601d23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa901d27, 90 },
    { 0, 0 },
    { 0xc7e01d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01d2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af01d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd01d35, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63101d38, 98 },
    { 0x34a01d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b01d40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35001d44, 33 },
    { 0xae01d44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea01d4b, 38 },
    { 0xe1b01d4c, 66 },
    { 0x1a001d4b, 38 },
    { 0x9b401d4e, 48 },
    { 0xddb01d4f, 95 },
    { 0x1c101d4f, 90 },
    { 0xc2d01d50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df01d54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c901d58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0001d5b, 61 },
    { 0, 0 },
    { 0x92501d5d, 93 },
    { 0, 0 },
    { 0x41001d5f, 61 },
    { 0, 0 },
    { 0x47e01d61, 90 },
    { 0, 0 },
    { 0x50501d63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80801d6c, 46 },
    { 0xcdb01d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf801d71, 90 },
    { 0, 0 },
    { 0x1ba01d73, 33 },
    { 0, 0 },
    { 0x4a801d75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff101d78, 100 },
    { 0x2ed01d78, 55 },
    { 0xe7901d7a, 23 },
    { 0x7d901d7b, 78 },
    { 0x79f01d7b, 61 },
    { 0x73001d7a, 95 },
    { 0x7fa01d7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5001d84, 23 },
    { 0x3e601d84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd901d89, 46 },
    { 0x2b901d8a, 96 },
    { 0xea201d8a, 46 },
    { 0x61101d8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f01d8f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34001d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd01d98, 33 },
    { 0xef301d98, 55 },
    { 0xa001d9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6201d9d, 98 },
    { 0, 0 },
    { 0x2b301d9f, 93 },
    { 0xbaf01da0, 90 },
    { 0x63001d9f, 93 },
    { 0xa5601da2, 33 },
    { 0x34901da2, 90 },
    { 0xb1b01da4, 52 },
    { 0x32301da3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc01daa, 99 },
    { 0x2c801daa, 85 },
    { 0x97701dac, 61 },
    { 0xaa001dab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91401db7, 76 },
    { 0x59701db8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x98201dbb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e501dbf, 75 },
    { 0x7b501dbf, 33 },
    { 0, 0 },
    { 0xc5b01dc2, 87 },
    { 0xbd301dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12901dc7, 95 },
    { 0, 0 },
    { 0x83101dc9, 40 },
    { 0, 0 },
    { 0x4a801dcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65801dce, 95 },
    { 0x13801dcf, 61 },
    { 0xa3301dd0, 93 },
    { 0x87301dd1, 90 },
    { 0x6ed01dd2, 77 },
    { 0x4b501dd2, 85 },
    { 0x94001dd4, 85 },
    { 0x80d01dd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d01ddd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c101de5, 61 },
    { 0x79701de6, 67 },
    { 0x87201de7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0301df3, 76 },
    { 0, 0 },
    { 0x3ab01df5, 76 },
    { 0xa2b01df6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41401dfa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54701e00, 59 },
    { 0, 0 },
    { 0xc1f01e02, 33 },
    { 0xe3701e03, 67 },
    { 0, 0 },
    { 0x9bf01e05, 87 },
    { 0, 0 },
    { 0x77201e07, 81 },
    { 0xfd01e08, 96 },
    { 0x46701e09, 55 },
    { 0x39501e08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e701e0d, 34 },
    { 0x47c01e0d, 99 },
    { 0, 0 },
    { 0xd801e10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb01e14, 100 },
    { 0x52f01e15, 23 },
    { 0x44d01e14, 95 },
    { 0x80601e14, 85 },
    { 0x34e01e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c01e21, 98 },
    { 0xbb301e22, 34 },
    { 0xeeb01e23, 55 },
    { 0x8401e24, 55 },
    { 0x6201e23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13201e28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf901e39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea601e3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3701e41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c01e4b, 95 },
    { 0x91601e4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a01e54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0801e57, 66 },
    { 0, 0 },
    { 0xeb801e59, 61 },
    { 0x3f401e5a, 76 },
    { 0xdf901e5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe01e60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb01e65, 23 },
    { 0, 0 },
    { 0x43101e67, 90 },
    { 0xca401e68, 46 },
    { 0x58e01e68, 98 },
    { 0xf1f01e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc501e6e, 61 },
    { 0x28201e6e, 95 },
    { 0x67201e6e, 59 },
    { 0xabc01e71, 61 },
    { 0xa0301e72, 40 },
    { 0xaa801e73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5601e78, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d01e7f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11201e86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d01e89, 76 },
    { 0xaaa01e89, 43 },
    { 0xe3e01e8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd01e8e, 78 },
    { 0, 0 },
    { 0xbdf01e90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d801e93, 61 },
    { 0xa3801e94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da01e97, 84 },
    { 0xbe201e97, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb01e9e, 34 },
    { 0x29501e9f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a01ea3, 42 },
    { 0x6c101ea3, 61 },
    { 0, 0 },
    { 0x32501ea6, 78 },
    { 0, 0 },
    { 0xebf01ea8, 78 },
    { 0x1d501ea9, 55 },
    { 0x32601ea8, 68 },
    { 0x63b01ea8, 55 },
    { 0x30c01ea8, 23 },
    { 0x32a01ead, 66 },
    { 0xa901ead, 52 },
    { 0x40801ea9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b101eb2, 90 },
    { 0x83e01eb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b01eb6, 43 },
    { 0x89b01eb7, 61 },
    { 0xcf001eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67001ebb, 77 },
    { 0x87201ebb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70601ebf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf01ec2, 90 },
    { 0xa3301ec2, 78 },
    { 0, 0 },
    { 0x55201ec5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9601ecb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d201ece, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b01ed3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f01ede, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa01ee2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7801ee5, 61 },
    { 0x3c601ee6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df01eeb, 82 },
    { 0x79b01eec, 100 },
    { 0, 0 },
    { 0xe9101eee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda101ef2, 67 },
    { 0xbe301ef3, 55 },
    { 0x2dd01ef4, 77 },
    { 0xa3a01ef4, 90 },
    { 0xcd001ef5, 98 },
    { 0xb0001ef4, 90 },
    { 0x1cd01ef7, 93 },
    { 0x59f01ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901efd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b01f04, 61 },
    { 0x34701f05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf801f0c, 33 },
    { 0xc3501f0c, 23 },
    { 0xb7401f0e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed01f15, 78 },
    { 0, 0 },
    { 0x4d01f17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d701f1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c01f22, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4501f27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec01f2a, 95 },
    { 0, 0 },
    { 0x2a601f2c, 90 },
    { 0x3a001f2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd101f32, 98 },
    { 0x2b601f33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf501f3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd501f3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e501f41, 61 },
    { 0x36d01f41, 95 },
    { 0, 0 },
    { 0xc0701f44, 90 },
    { 0, 0 },
    { 0xddb01f46, 55 },
    { 0, 0 },
    { 0x69401f48, 95 },
    { 0xe4601f48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c501f50, 94 },
    { 0, 0 },
    { 0xab001f52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1501f55, 33 },
    { 0x89901f56, 91 },
    { 0x6e701f55, 75 },
    { 0, 0 },
    { 0xd6b01f59, 34 },
    { 0, 0 },
    { 0xacc01f5b, 61 },
    { 0xc8b01f5c, 33 },
    { 0x6b01f5c, 90 },
    { 0x87801f5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f01f61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb501f6e, 98 },
    { 0x84f01f6f, 34 },
    { 0x9ab01f70, 78 },
    { 0, 0 },
    { 0x9a101f72, 95 },
    { 0, 0 },
    { 0x72501f74, 95 },
    { 0, 0 },
    { 0x71b01f76, 90 },
    { 0, 0 },
    { 0x21e01f78, 33 },
    { 0x3cb01f79, 55 },
    { 0xc0901f78, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4701f7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf801f84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f301f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79001f8f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb501f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e01f98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda01f9b, 90 },
    { 0x7c401f9c, 52 },
    { 0x94901f9c, 87 },
    { 0xc4601f9e, 34 },
    { 0x7f401f9f, 33 },
    { 0x6e001f9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f01faa, 33 },
    { 0, 0 },
    { 0xc001fac, 98 },
    { 0, 0 },
    { 0xad001fae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2001fb8, 96 },
    { 0xf4d01fb8, 95 },
    { 0x9001fb8, 33 },
    { 0x6df01fbb, 65 },
    { 0xf8701fbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec01fc0, 99 },
    { 0x58901fc1, 76 },
    { 0xfad01fc2, 76 },
    { 0xcc401fc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f501fc8, 55 },
    { 0, 0 },
    { 0x15901fca, 90 },
    { 0x9f201fcb, 67 },
    { 0xc6201fcc, 33 },
    { 0x64901fcd, 91 },
    { 0xc3101fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab01fd1, 90 },
    { 0x19701fd2, 33 },
    { 0xc1801fd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50801fd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31101fdf, 90 },
    { 0x7d01fe0, 23 },
    { 0xec701fe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c01feb, 95 },
    { 0x37601fec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98501ff2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59601ff8, 95 },
    { 0, 0 },
    { 0x12401ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd01fff, 95 },
    { 0x2f902000, 98 },
    { 0x81c02000, 33 },
    { 0xfe602000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd302005, 78 },
    { 0x8102005, 95 },
    { 0x73e02007, 76 },
    { 0, 0 },
    { 0xda002009, 99 },
    { 0x7ed02009, 67 },
    { 0xe470200b, 77 },
    { 0xa060200c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x150200f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92402012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330201d, 68 },
    { 0xd0c0201e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d02023, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a0203b, 98 },
    { 0xc750203b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3430203f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb802044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c02047, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb200204a, 84 },
    { 0, 0 },
    { 0x8ee0204c, 33 },
    { 0xc2a0204c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xff002050, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0802055, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa60205b, 33 },
    { 0xd6b0205b, 95 },
    { 0x1f0205d, 90 },
    { 0x85c0205d, 98 },
    { 0xdb0205e, 83 },
    { 0xf4e02060, 23 },
    { 0x4f702061, 33 },
    { 0, 0 },
    { 0xc8002063, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec20206c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde102072, 23 },
    { 0, 0 },
    { 0x38b02074, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5302077, 98 },
    { 0x6fa02077, 61 },
    { 0xa1802079, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a40207c, 76 },
    { 0xe7e0207d, 84 },
    { 0x9c40207e, 81 },
    { 0xbe20207e, 66 },
    { 0xff902080, 55 },
    { 0x4100207f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d02089, 76 },
    { 0, 0 },
    { 0xec0208b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd690208f, 84 },
    { 0xd5f02090, 43 },
    { 0xa102091, 23 },
    { 0x3bd02091, 23 },
    { 0x63a02091, 33 },
    { 0x98b02094, 65 },
    { 0x21e02094, 61 },
    { 0x6d402096, 61 },
    { 0x12602092, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b0209b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1020a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7020a4, 43 },
    { 0, 0 },
    { 0x3f1020a6, 75 },
    { 0xf5e020a7, 90 },
    { 0xf83020a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661020b0, 98 },
    { 0x740020b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a020b6, 33 },
    { 0x81020b6, 99 },
    { 0x633020b6, 90 },
    { 0xc60020b9, 61 },
    { 0xea4020b9, 33 },
    { 0xf48020ba, 61 },
    { 0, 0 },
    { 0x598020bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c020c0, 90 },
    { 0xee9020c0, 51 },
    { 0xc65020c1, 23 },
    { 0xc17020c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x243020c6, 84 },
    { 0xb3b020c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed020cb, 95 },
    { 0x7da020cc, 91 },
    { 0x33e020cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff020d0, 95 },
    { 0, 0 },
    { 0x4f3020d2, 88 },
    { 0x1c1020d3, 95 },
    { 0x6d4020d2, 23 },
    { 0, 0 },
    { 0x38d020d6, 90 },
    { 0xb39020d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xca020da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee020df, 33 },
    { 0, 0 },
    { 0xfe020e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1020e4, 76 },
    { 0xbb8020e5, 98 },
    { 0xd9a020e5, 34 },
    { 0, 0 },
    { 0xdfb020e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e020eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962020ef, 43 },
    { 0xcd1020f0, 55 },
    { 0, 0 },
    { 0xff8020f2, 61 },
    { 0x409020f3, 34 },
    { 0x9d9020f4, 61 },
    { 0x31a020f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c020f8, 61 },
    { 0, 0 },
    { 0x3d0020fa, 93 },
    { 0x397020fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d02100, 67 },
    { 0, 0 },
    { 0x1f702102, 24 },
    { 0, 0 },
    { 0x23b02104, 77 },
    { 0x36d02105, 64 },
    { 0x94702106, 95 },
    { 0, 0 },
    { 0xd3b02108, 90 },
    { 0, 0 },
    { 0x93e0210a, 23 },
    { 0, 0 },
    { 0x2ba0210c, 85 },
    { 0xbb80210c, 98 },
    { 0xdf70210c, 33 },
    { 0xaba0210c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a02112, 55 },
    { 0, 0 },
    { 0xd8b02114, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd550211d, 34 },
    { 0, 0 },
    { 0x4370211f, 95 },
    { 0, 0 },
    { 0x60102121, 43 },
    { 0x68602122, 23 },
    { 0xc6f02123, 43 },
    { 0x4cb02123, 23 },
    { 0xf9f02125, 34 },
    { 0x79302125, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced0212a, 33 },
    { 0, 0 },
    { 0x68d0212c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af0212f, 43 },
    { 0xe2402130, 95 },
    { 0xfc602131, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9702134, 33 },
    { 0, 0 },
    { 0xed402136, 33 },
    { 0xe9502136, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e60213c, 66 },
    { 0, 0 },
    { 0x39f0213e, 61 },
    { 0, 0 },
    { 0x2fe02140, 34 },
    { 0xd3902140, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa02145, 99 },
    { 0x10402146, 61 },
    { 0x2b802147, 33 },
    { 0x42602145, 95 },
    { 0, 0 },
    { 0x9b30214a, 66 },
    { 0x77b0214b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f02150, 93 },
    { 0x4c602150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd902156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45202159, 33 },
    { 0xb3a0215a, 23 },
    { 0, 0 },
    { 0xd090215c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d702160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b02166, 61 },
    { 0, 0 },
    { 0xfab02168, 61 },
    { 0xea302169, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a50216c, 34 },
    { 0x2410216d, 61 },
    { 0x5d0216d, 84 },
    { 0x3220216f, 40 },
    { 0, 0 },
    { 0xd7f02171, 43 },
    { 0x2ca02172, 23 },
    { 0xd2002172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d0217b, 33 },
    { 0, 0 },
    { 0x8e30217d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d02185, 76 },
    { 0, 0 },
    { 0xf3502187, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6440218d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92202192, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f10219a, 90 },
    { 0, 0 },
    { 0x1be0219c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x110219f, 23 },
    { 0, 0 },
    { 0x99b021a1, 55 },
    { 0xe0c021a2, 76 },
    { 0, 0 },
    { 0x2e021a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca021a9, 95 },
    { 0x87021aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122021ae, 61 },
    { 0x6f8021af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b021b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc021b8, 46 },
    { 0x246021b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22021bd, 61 },
    { 0xcb1021be, 98 },
    { 0, 0 },
    { 0x903021c0, 98 },
    { 0, 0 },
    { 0x715021c2, 84 },
    { 0xad0021c2, 48 },
    { 0x409021c4, 43 },
    { 0x20d021c2, 23 },
    { 0x8cd021c6, 90 },
    { 0, 0 },
    { 0x663021c8, 95 },
    { 0xb84021c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9021db, 55 },
    { 0, 0 },
    { 0x355021dd, 90 },
    { 0, 0 },
    { 0xd94021df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69021e7, 52 },
    { 0xd38021e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92021eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76021f0, 90 },
    { 0, 0 },
    { 0x2e021f2, 91 },
    { 0xa6e021f3, 81 },
    { 0x798021f3, 95 },
    { 0x384021f5, 75 },
    { 0, 0 },
    { 0x565021f7, 76 },
    { 0x2a1021f8, 90 },
    { 0xfc0021f9, 99 },
    { 0x8ab021fa, 98 },
    { 0x5eb021f7, 59 },
    { 0, 0 },
    { 0x5b3021fd, 23 },
    { 0x71a021fe, 76 },
    { 0x644021fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02204, 98 },
    { 0xf7602205, 23 },
    { 0xae802206, 95 },
    { 0xdf702207, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd870220a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb0220f, 65 },
    { 0x5c02210, 23 },
    { 0xbd602211, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74202215, 69 },
    { 0x3fd02216, 33 },
    { 0xa9402216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee60221d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d02220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d02224, 100 },
    { 0x7bf02225, 46 },
    { 0xd2e02225, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe050222a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe02231, 95 },
    { 0, 0 },
    { 0x60f02233, 90 },
    { 0x9b502234, 33 },
    { 0, 0 },
    { 0xb5002236, 87 },
    { 0xb3b02236, 47 },
    { 0xf4402238, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6700223b, 34 },
    { 0, 0 },
    { 0xa3c0223d, 61 },
    { 0x30c0223d, 95 },
    { 0x92f0223f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1902245, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9930224e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87702256, 76 },
    { 0x91c02257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9480225e, 90 },
    { 0, 0 },
    { 0x52f02260, 85 },
    { 0xc1102261, 90 },
    { 0x1cd02262, 78 },
    { 0xb5b02262, 52 },
    { 0xb0202264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940226a, 90 },
    { 0, 0 },
    { 0x6fb0226c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59502270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d02274, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d02277, 90 },
    { 0x4bd02278, 87 },
    { 0, 0 },
    { 0x3ea0227a, 76 },
    { 0, 0 },
    { 0x9c10227c, 23 },
    { 0xcc0227d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c702280, 76 },
    { 0xd5402281, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a02285, 34 },
    { 0x78102285, 33 },
    { 0x33402287, 77 },
    { 0x10602287, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9240228b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8102292, 98 },
    { 0, 0 },
    { 0x8c402294, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a80229a, 95 },
    { 0x4f30229b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6022a1, 78 },
    { 0x53c022a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54022a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67022aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd022af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d022b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7022b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb022ba, 61 },
    { 0xb05022ba, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec022c3, 55 },
    { 0x1a0022c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b022d0, 98 },
    { 0xe9b022d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4022d5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x411022d8, 91 },
    { 0xc3d022d9, 61 },
    { 0x371022da, 90 },
    { 0, 0 },
    { 0xbf4022dc, 34 },
    { 0xc7022dc, 98 },
    { 0x9cb022de, 43 },
    { 0xc6e022de, 76 },
    { 0xd23022df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586022e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932022ed, 33 },
    { 0x461022ed, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x433022f1, 43 },
    { 0xc5a022f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b022f6, 66 },
    { 0x3fe022f7, 23 },
    { 0x444022f8, 23 },
    { 0x739022f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4502302, 90 },
    { 0, 0 },
    { 0x66802304, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a02307, 95 },
    { 0x2df02308, 23 },
    { 0xc7602309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0230c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54602311, 90 },
    { 0, 0 },
    { 0x17302313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55402316, 97 },
    { 0xe4f02317, 61 },
    { 0x77702316, 61 },
    { 0, 0 },
    { 0x5c80231a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd80231d, 98 },
    { 0xf010231e, 34 },
    { 0xdcd0231f, 34 },
    { 0xb8a0231d, 90 },
    { 0x26b02321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d802324, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80202329, 80 },
    { 0x86502329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a90232e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf002331, 91 },
    { 0, 0 },
    { 0x65202333, 85 },
    { 0x80e02334, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf802339, 98 },
    { 0, 0 },
    { 0xffe0233b, 36 },
    { 0x67b0233c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0233f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1602347, 90 },
    { 0, 0 },
    { 0x2be02349, 98 },
    { 0x7fa02349, 33 },
    { 0, 0 },
    { 0x2730234c, 23 },
    { 0x1730234d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a02356, 61 },
    { 0xd4402357, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6050235a, 95 },
    { 0, 0 },
    { 0xbd70235c, 90 },
    { 0, 0 },
    { 0xbbc0235e, 90 },
    { 0x7510235f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96202366, 81 },
    { 0x3ac02367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140236a, 55 },
    { 0xd710236a, 33 },
    { 0x8ce0236c, 98 },
    { 0x6ba0236c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7802372, 87 },
    { 0x76902372, 43 },
    { 0x7dd02374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb902377, 93 },
    { 0x51d02378, 61 },
    { 0xaa902379, 61 },
    { 0, 0 },
    { 0xad70237b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a02381, 93 },
    { 0x6bb02382, 43 },
    { 0x3f402383, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b02387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa0238c, 95 },
    { 0, 0 },
    { 0x5fa0238e, 77 },
    { 0xed30238e, 69 },
    { 0xc950238e, 61 },
    { 0x16c02391, 90 },
    { 0x63c02390, 61 },
    { 0x45402393, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c02398, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f80239f, 98 },
    { 0xb3e023a0, 87 },
    { 0xfe4023a0, 90 },
    { 0xe5e023a1, 98 },
    { 0x6d1023a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b023a6, 40 },
    { 0x14b023a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72023ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75023b3, 90 },
    { 0, 0 },
    { 0x492023b5, 66 },
    { 0, 0 },
    { 0x686023b7, 98 },
    { 0, 0 },
    { 0x677023b9, 95 },
    { 0, 0 },
    { 0x17023bb, 90 },
    { 0, 0 },
    { 0x749023bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec023c0, 98 },
    { 0x593023c1, 91 },
    { 0, 0 },
    { 0xcd7023c3, 55 },
    { 0x85e023c4, 69 },
    { 0x91a023c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62023c8, 46 },
    { 0, 0 },
    { 0xec4023ca, 76 },
    { 0xd44023cb, 76 },
    { 0xf54023cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42023de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436023e2, 90 },
    { 0x855023e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1023e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x357023e9, 61 },
    { 0x10c023ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d023ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39023f0, 61 },
    { 0x555023f1, 98 },
    { 0xac7023f2, 23 },
    { 0, 0 },
    { 0x73a023f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb023f9, 98 },
    { 0x3c2023fa, 33 },
    { 0x32b023f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23602403, 98 },
    { 0xf3e02404, 76 },
    { 0x4d902405, 90 },
    { 0, 0 },
    { 0x53902407, 61 },
    { 0, 0 },
    { 0x67702409, 23 },
    { 0, 0 },
    { 0x8c40240b, 85 },
    { 0xd720240c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1602412, 99 },
    { 0x9bb02413, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f402418, 90 },
    { 0x3ba02419, 95 },
    { 0x90e0241a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a02421, 23 },
    { 0, 0 },
    { 0xae502423, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e02427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa0242c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a02432, 90 },
    { 0, 0 },
    { 0x5bc02434, 90 },
    { 0xc0202435, 100 },
    { 0x42002436, 33 },
    { 0xa1602436, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0243a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7902441, 90 },
    { 0xcbf02442, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f02445, 61 },
    { 0xad02446, 33 },
    { 0, 0 },
    { 0x97e02448, 55 },
    { 0x4cc02448, 95 },
    { 0x7680244a, 23 },
    { 0xab70244a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670244f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1880245b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f02463, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b102469, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f0246e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19202472, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca02475, 98 },
    { 0x2902475, 76 },
    { 0xde202477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2002480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa802485, 33 },
    { 0x4eb02486, 23 },
    { 0, 0 },
    { 0xa1302488, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc880248c, 23 },
    { 0xbd40248d, 95 },
    { 0, 0 },
    { 0x7120248f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d02495, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce02498, 34 },
    { 0, 0 },
    { 0xe3f0249a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9970249e, 43 },
    { 0x2eb0249f, 88 },
    { 0xa450249f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8024a3, 9 },
    { 0, 0 },
    { 0xf9f024a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x445024a8, 34 },
    { 0, 0 },
    { 0x2d9024aa, 100 },
    { 0, 0 },
    { 0x59b024ac, 55 },
    { 0xfd4024ad, 90 },
    { 0x267024ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd024b3, 61 },
    { 0x17e024b4, 90 },
    { 0x77b024b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4024b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b024bc, 33 },
    { 0x7fd024bd, 85 },
    { 0xa01024bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85024c2, 25 },
    { 0xeb8024c3, 61 },
    { 0, 0 },
    { 0xbe6024c5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9024c8, 87 },
    { 0xb62024c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329024ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2024d1, 88 },
    { 0, 0 },
    { 0x20c024d3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x342024d6, 23 },
    { 0, 0 },
    { 0x16f024d8, 33 },
    { 0x333024d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e024dc, 66 },
    { 0x40f024dd, 52 },
    { 0xf31024de, 76 },
    { 0x667024de, 80 },
    { 0x2f3024e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c024e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce024e8, 23 },
    { 0x633024e8, 55 },
    { 0x40a024ea, 95 },
    { 0x37a024eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85024ee, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57024f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b024f6, 98 },
    { 0, 0 },
    { 0xb84024f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8024fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12602503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec602506, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee0250c, 91 },
    { 0, 0 },
    { 0xa540250e, 61 },
    { 0xe2e0250f, 61 },
    { 0, 0 },
    { 0x2a202511, 33 },
    { 0x8d902511, 34 },
    { 0, 0 },
    { 0xc5602514, 55 },
    { 0xbd602515, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45102519, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a0251f, 33 },
    { 0xdd402520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b0252d, 90 },
    { 0x8b70252e, 95 },
    { 0xcb00252d, 82 },
    { 0x5aa02530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10253a, 34 },
    { 0, 0 },
    { 0xb960253c, 61 },
    { 0xf460253c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa02541, 90 },
    { 0x4eb02542, 34 },
    { 0x1a302542, 95 },
    { 0x5ab02544, 93 },
    { 0x7ca02545, 33 },
    { 0x38402546, 55 },
    { 0xb1e02544, 97 },
    { 0, 0 },
    { 0x9c102549, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a0254e, 93 },
    { 0x95d0254f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a902552, 43 },
    { 0x45802553, 90 },
    { 0xd3f02554, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac02557, 76 },
    { 0x33902558, 33 },
    { 0, 0 },
    { 0x6a10255a, 91 },
    { 0xdd90255b, 76 },
    { 0, 0 },
    { 0x1260255d, 33 },
    { 0, 0 },
    { 0x2580255f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd602563, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc0256a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8210256e, 61 },
    { 0xc160256e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91502572, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54402576, 91 },
    { 0x79502576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c0257b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e0257e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xed802581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc02585, 66 },
    { 0, 0 },
    { 0x7a002587, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x790258a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60258e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32402591, 23 },
    { 0xac202591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a02595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2502598, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf10259f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57025a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3025a9, 34 },
    { 0x271025a9, 58 },
    { 0xc5c025ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93025b0, 69 },
    { 0, 0 },
    { 0x7b3025b2, 98 },
    { 0x1bb025b3, 46 },
    { 0x92f025b4, 55 },
    { 0x161025b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446025bb, 61 },
    { 0x744025bb, 34 },
    { 0, 0 },
    { 0x56c025be, 61 },
    { 0xde0025bf, 98 },
    { 0x8d1025bf, 91 },
    { 0, 0 },
    { 0x787025c2, 61 },
    { 0, 0 },
    { 0xdbb025c4, 55 },
    { 0xf62025c5, 98 },
    { 0xa67025c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda025cc, 23 },
    { 0xbf8025cc, 67 },
    { 0x957025cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc025d1, 99 },
    { 0xd68025d2, 23 },
    { 0, 0 },
    { 0x12a025d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91025d7, 26 },
    { 0x7af025d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1025db, 34 },
    { 0x7d0025dc, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c025e1, 77 },
    { 0, 0 },
    { 0x7da025e3, 67 },
    { 0xfd8025e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d025e9, 91 },
    { 0x78e025e9, 43 },
    { 0x736025eb, 76 },
    { 0xd3025ec, 90 },
    { 0xa1e025ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f025f0, 43 },
    { 0, 0 },
    { 0xe80025f2, 70 },
    { 0x8d9025f2, 55 },
    { 0x863025f4, 83 },
    { 0xbe8025f5, 98 },
    { 0, 0 },
    { 0x3fc025f7, 33 },
    { 0xbd6025f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9025fe, 93 },
    { 0xf58025fe, 81 },
    { 0x1dd02600, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e02605, 95 },
    { 0x6502606, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380260c, 98 },
    { 0x6fe0260c, 91 },
    { 0, 0 },
    { 0xbc20260f, 33 },
    { 0xeca02610, 23 },
    { 0x6580260f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88402619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a50261d, 33 },
    { 0x9920261e, 46 },
    { 0x6920261f, 23 },
    { 0x8730261e, 17 },
    { 0x35002620, 33 },
    { 0xc720261e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62702626, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c202629, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650262d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc602631, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a502639, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae0263d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8602640, 43 },
    { 0, 0 },
    { 0x78702642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98102645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce02648, 99 },
    { 0xe4002649, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9202651, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe402659, 90 },
    { 0xd3d02659, 33 },
    { 0x4f70265b, 95 },
    { 0x9b50265b, 84 },
    { 0xa010265c, 98 },
    { 0, 0 },
    { 0xd530265f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb902665, 76 },
    { 0xd0d02666, 90 },
    { 0x90f02665, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f00266a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85002671, 68 },
    { 0xc2e02671, 76 },
    { 0x28202672, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53502678, 61 },
    { 0xfda02679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d0267e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6902681, 90 },
    { 0xdb802682, 90 },
    { 0, 0 },
    { 0x9602684, 23 },
    { 0, 0 },
    { 0xdbc02686, 95 },
    { 0xa7e02687, 93 },
    { 0x52802688, 66 },
    { 0x39e02688, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa280268d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e02691, 98 },
    { 0x45302692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d0269c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c0269f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x661026a2, 23 },
    { 0x45c026a3, 52 },
    { 0x159026a3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d026a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8026ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4026af, 76 },
    { 0, 0 },
    { 0xa1e026b1, 76 },
    { 0x670026b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c026bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4026c3, 36 },
    { 0x7c2026c4, 33 },
    { 0xbcf026c3, 85 },
    { 0x7fc026c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8026c9, 84 },
    { 0, 0 },
    { 0xa4c026cb, 61 },
    { 0x9dc026cb, 97 },
    { 0xc1026cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111026d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e026db, 76 },
    { 0x37f026dc, 46 },
    { 0xd84026dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df026e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72026e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0026ee, 90 },
    { 0x633026ee, 95 },
    { 0, 0 },
    { 0x500026f1, 98 },
    { 0xe54026f2, 90 },
    { 0xaa8026f3, 95 },
    { 0xda8026f4, 61 },
    { 0xab0026f4, 91 },
    { 0x4f1026f2, 34 },
    { 0x62026f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d026fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed002703, 77 },
    { 0xeb602703, 61 },
    { 0, 0 },
    { 0xc4502706, 55 },
    { 0, 0 },
    { 0xe7302708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2720270b, 65 },
    { 0, 0 },
    { 0x750270d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27302713, 76 },
    { 0, 0 },
    { 0xcea02715, 84 },
    { 0x55e02716, 90 },
    { 0, 0 },
    { 0x68802718, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c30271b, 33 },
    { 0, 0 },
    { 0x62b0271d, 52 },
    { 0x9f30271d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x38702721, 81 },
    { 0x8d002722, 90 },
    { 0, 0 },
    { 0x4e902724, 55 },
    { 0xa3802725, 90 },
    { 0, 0 },
    { 0xe0e02727, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d0272c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da02730, 55 },
    { 0xaa702730, 33 },
    { 0x5b802730, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db02737, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d0273b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d02741, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83102745, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0274d, 23 },
    { 0, 0 },
    { 0x8020274f, 96 },
    { 0xf6502750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0802753, 61 },
    { 0x8eb02753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f20275e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c02766, 23 },
    { 0x10602767, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf330276b, 65 },
    { 0, 0 },
    { 0xe7c0276d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd02771, 95 },
    { 0xca602772, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc02775, 43 },
    { 0x26e02775, 33 },
    { 0, 0 },
    { 0x51e02778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a00277b, 61 },
    { 0xe3b0277c, 90 },
    { 0xd180277c, 61 },
    { 0xe810277e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37702782, 78 },
    { 0x73d02782, 33 },
    { 0x1cb02783, 43 },
    { 0x75802785, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x55102788, 98 },
    { 0x56f02789, 91 },
    { 0xfb002788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7302794, 61 },
    { 0x78702795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e90279a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef0279d, 91 },
    { 0xe2f0279e, 55 },
    { 0x6d0279e, 23 },
    { 0xd01027a0, 46 },
    { 0, 0 },
    { 0x246027a2, 93 },
    { 0, 0 },
    { 0xb36027a4, 95 },
    { 0x862027a5, 95 },
    { 0x879027a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4027a9, 100 },
    { 0, 0 },
    { 0x1a027ab, 23 },
    { 0, 0 },
    { 0xf9a027ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8027b1, 81 },
    { 0, 0 },
    { 0xefe027b3, 90 },
    { 0x43a027b4, 90 },
    { 0xa8027b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20027b8, 90 },
    { 0, 0 },
    { 0xb78027ba, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34027be, 33 },
    { 0, 0 },
    { 0x593027c0, 34 },
    { 0x102027c1, 90 },
    { 0xc11027c2, 23 },
    { 0, 0 },
    { 0x159027c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2027c8, 66 },
    { 0x791027c9, 61 },
    { 0x26a027c9, 52 },
    { 0xc47027cb, 95 },
    { 0, 0 },
    { 0x39b027cd, 52 },
    { 0, 0 },
    { 0x572027cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0027d2, 61 },
    { 0, 0 },
    { 0xbcb027d4, 43 },
    { 0x886027d5, 55 },
    { 0x537027d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95027d9, 56 },
    { 0x3fe027da, 61 },
    { 0x800027db, 61 },
    { 0, 0 },
    { 0x222027dd, 61 },
    { 0, 0 },
    { 0x5fb027df, 33 },
    { 0, 0 },
    { 0x2b1027e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe027ee, 43 },
    { 0x9c7027ef, 43 },
    { 0, 0 },
    { 0xb9c027f1, 38 },
    { 0xce4027f2, 98 },
    { 0x883027f2, 33 },
    { 0, 0 },
    { 0xabb027f5, 33 },
    { 0xc45027f5, 78 },
    { 0, 0 },
    { 0x200027f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541027ff, 98 },
    { 0, 0 },
    { 0xedd02801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b02807, 61 },
    { 0, 0 },
    { 0x62902809, 92 },
    { 0xb3a0280a, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a0280d, 90 },
    { 0, 0 },
    { 0xfd00280f, 33 },
    { 0xdd002810, 87 },
    { 0, 0 },
    { 0xf1902812, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69302817, 90 },
    { 0, 0 },
    { 0x9dc02819, 77 },
    { 0x6a902819, 90 },
    { 0xc300281b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda0281e, 75 },
    { 0, 0 },
    { 0x53002820, 91 },
    { 0x95102820, 100 },
    { 0, 0 },
    { 0x49302823, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77702827, 95 },
    { 0x62f02827, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2630282c, 8 },
    { 0, 0 },
    { 0x2020282e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xea502831, 98 },
    { 0, 0 },
    { 0xaee02833, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b20283b, 33 },
    { 0, 0 },
    { 0xa150283d, 98 },
    { 0x6900283e, 85 },
    { 0x6b30283e, 95 },
    { 0x1ca02840, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc02843, 34 },
    { 0x47202844, 90 },
    { 0x18102845, 52 },
    { 0, 0 },
    { 0xad102847, 33 },
    { 0xfd402848, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb0284b, 98 },
    { 0x9850284b, 61 },
    { 0x6470284d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a02855, 55 },
    { 0x21d02856, 90 },
    { 0xff002857, 95 },
    { 0x8eb02855, 34 },
    { 0xf3902859, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f0285d, 33 },
    { 0, 0 },
    { 0xd840285f, 33 },
    { 0xb202860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53802864, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd520286a, 96 },
    { 0x9570286b, 46 },
    { 0x7230286b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba0286f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe02872, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2802876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c0287c, 84 },
    { 0x5cc0287d, 34 },
    { 0xb310287c, 33 },
    { 0x5680287f, 91 },
    { 0xd2b02880, 95 },
    { 0xeb02880, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45602885, 61 },
    { 0x61502886, 61 },
    { 0, 0 },
    { 0xc0902888, 78 },
    { 0, 0 },
    { 0x2160288a, 34 },
    { 0, 0 },
    { 0xaac0288c, 61 },
    { 0x15f0288c, 61 },
    { 0x9260288c, 90 },
    { 0x1c50288f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a02892, 17 },
    { 0xeca02893, 98 },
    { 0x56202894, 97 },
    { 0xe4b02894, 23 },
    { 0, 0 },
    { 0x23102897, 61 },
    { 0, 0 },
    { 0x28502899, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4040289d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d028a2, 76 },
    { 0, 0 },
    { 0x434028a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57028a9, 90 },
    { 0x19c028a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd028ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202028b3, 98 },
    { 0xb78028b4, 90 },
    { 0x52d028b5, 90 },
    { 0xbdc028b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b028b9, 95 },
    { 0x9df028b9, 40 },
    { 0x1fe028bb, 33 },
    { 0x685028b9, 64 },
    { 0x650028bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd028c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818028c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626028cd, 66 },
    { 0x702028ce, 43 },
    { 0xcc7028ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1028d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76028d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x679028db, 33 },
    { 0x4d9028db, 61 },
    { 0xf17028dd, 84 },
    { 0xefe028de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29028e1, 95 },
    { 0x5b3028e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692028ed, 80 },
    { 0, 0 },
    { 0xcd0028ef, 98 },
    { 0x1d1028f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c028f3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b028f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09028fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x895028fe, 95 },
    { 0, 0 },
    { 0x88202900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c302904, 99 },
    { 0x80802904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa002908, 90 },
    { 0xfe302909, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd020290f, 95 },
    { 0, 0 },
    { 0xe3c02911, 67 },
    { 0xc6502912, 61 },
    { 0xa7d02913, 77 },
    { 0x7a002914, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa710291e, 33 },
    { 0xf970291e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70102922, 66 },
    { 0xe3302923, 85 },
    { 0x7ba02924, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32402928, 23 },
    { 0, 0 },
    { 0xce80292a, 90 },
    { 0xd220292b, 33 },
    { 0x1a0292c, 65 },
    { 0xc4f0292d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58902932, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc02937, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd0293b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df0293e, 87 },
    { 0xfe20293e, 33 },
    { 0x4fa0293f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba02943, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce02948, 99 },
    { 0, 0 },
    { 0x1ca0294a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e0294f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef602952, 48 },
    { 0x2b202952, 61 },
    { 0x95002954, 98 },
    { 0x93402955, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e02958, 77 },
    { 0, 0 },
    { 0x7a80295a, 76 },
    { 0, 0 },
    { 0xec0295c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa02961, 78 },
    { 0x41402962, 100 },
    { 0xbb502962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa02966, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c90296a, 76 },
    { 0xcd30296b, 95 },
    { 0xfb30296c, 61 },
    { 0xb800296d, 98 },
    { 0x320296d, 66 },
    { 0, 0 },
    { 0xc0502970, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57902975, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b90297a, 33 },
    { 0, 0 },
    { 0x4f40297c, 34 },
    { 0x53b0297d, 75 },
    { 0, 0 },
    { 0xe320297f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e802984, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59102988, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7302990, 85 },
    { 0x6b02991, 67 },
    { 0x8b102992, 40 },
    { 0x5e802993, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67802999, 77 },
    { 0, 0 },
    { 0x2cd0299b, 100 },
    { 0x3000299b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c0299f, 76 },
    { 0x798029a0, 95 },
    { 0xce3029a1, 90 },
    { 0, 0 },
    { 0x804029a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f029a6, 100 },
    { 0xd1e029a7, 98 },
    { 0x4b029a8, 98 },
    { 0, 0 },
    { 0xebb029aa, 91 },
    { 0xbec029ab, 90 },
    { 0x477029ac, 33 },
    { 0x588029ad, 84 },
    { 0x309029ae, 76 },
    { 0, 0 },
    { 0x753029b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a029b3, 40 },
    { 0x1ba029b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae029b7, 84 },
    { 0, 0 },
    { 0xa16029b9, 33 },
    { 0x831029ba, 34 },
    { 0, 0 },
    { 0x16b029bc, 33 },
    { 0, 0 },
    { 0xb91029be, 23 },
    { 0x40b029bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7029c2, 23 },
    { 0xd58029c2, 75 },
    { 0x7a6029c4, 98 },
    { 0x840029c5, 90 },
    { 0x243029c6, 52 },
    { 0xa1b029c7, 95 },
    { 0x2c2029c8, 33 },
    { 0x295029c9, 33 },
    { 0x916029c8, 55 },
    { 0x1f6029cb, 55 },
    { 0xef7029c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd029cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e029d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118029da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0029e4, 95 },
    { 0x1b4029e4, 76 },
    { 0x43a029e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873029ea, 76 },
    { 0, 0 },
    { 0xae0029ec, 87 },
    { 0, 0 },
    { 0x76b029ee, 33 },
    { 0xdb0029ef, 33 },
    { 0x75b029f0, 98 },
    { 0xe29029ef, 93 },
    { 0xd25029ef, 90 },
    { 0x974029f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c029f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7202a03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9402a06, 95 },
    { 0, 0 },
    { 0x83602a08, 34 },
    { 0, 0 },
    { 0x8d902a0a, 61 },
    { 0x1ff02a0b, 76 },
    { 0x8b402a0c, 88 },
    { 0xa102a0a, 95 },
    { 0xb6b02a0e, 95 },
    { 0x99202a0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb02a13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d02a16, 66 },
    { 0xad002a17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68902a22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a002a27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a702a2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f02a30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e02a36, 90 },
    { 0x5b402a37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae02a3c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e02a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c02a4b, 33 },
    { 0x42202a4c, 33 },
    { 0xe3702a4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b02a63, 100 },
    { 0x2a702a64, 61 },
    { 0x42302a65, 38 },
    { 0x5302a65, 90 },
    { 0xd1402a67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90802a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde002a6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad02a73, 95 },
    { 0x57402a74, 90 },
    { 0, 0 },
    { 0x60602a76, 81 },
    { 0x72102a77, 98 },
    { 0xc6102a77, 69 },
    { 0, 0 },
    { 0x69102a7a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13902a7e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7902a87, 76 },
    { 0xac02a88, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8802a8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee502a93, 69 },
    { 0, 0 },
    { 0x2be02a95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x24902a98, 33 },
    { 0x8b702a98, 34 },
    { 0x29c02a9a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af02a9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1802aa1, 23 },
    { 0xcc102aa1, 23 },
    { 0x1f502aa3, 91 },
    { 0x88a02aa4, 61 },
    { 0xe9f02aa5, 34 },
    { 0xf9c02aa6, 98 },
    { 0x51502aa3, 98 },
    { 0xdf02aa7, 90 },
    { 0, 0 },
    { 0xb3c02aaa, 65 },
    { 0x2202aab, 90 },
    { 0, 0 },
    { 0x2dc02aad, 76 },
    { 0xc4b02aae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c02ab2, 90 },
    { 0xbee02ab3, 34 },
    { 0, 0 },
    { 0x64402ab5, 23 },
    { 0x4fa02ab6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf602abc, 95 },
    { 0x29502abd, 61 },
    { 0xd7202abe, 87 },
    { 0, 0 },
    { 0x22902ac0, 61 },
    { 0x83402ac1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f102ac6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f02acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e02ad7, 23 },
    { 0x13902ad7, 34 },
    { 0, 0 },
    { 0x1e302ada, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf02ade, 90 },
    { 0, 0 },
    { 0x74702ae0, 91 },
    { 0, 0 },
    { 0x99c02ae2, 93 },
    { 0xfa902ae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d02ae7, 52 },
    { 0x2e102ae8, 94 },
    { 0xedc02ae8, 34 },
    { 0, 0 },
    { 0xd6702aeb, 23 },
    { 0xe7702aec, 95 },
    { 0, 0 },
    { 0x3d402aee, 33 },
    { 0x3d202aef, 61 },
    { 0, 0 },
    { 0xe2e02af1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa02af4, 33 },
    { 0, 0 },
    { 0xc9602af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d02afb, 98 },
    { 0, 0 },
    { 0x69602afd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4402b01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf02b09, 87 },
    { 0, 0 },
    { 0xf102b0b, 23 },
    { 0x67b02b0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc02b12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37402b18, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63402b1c, 81 },
    { 0xdce02b1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d02b24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e02b29, 33 },
    { 0xf7302b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea02b2d, 33 },
    { 0, 0 },
    { 0x29402b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71002b32, 67 },
    { 0x74102b33, 33 },
    { 0x97a02b32, 61 },
    { 0xc2802b34, 100 },
    { 0, 0 },
    { 0x4b502b37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d02b3b, 90 },
    { 0xba302b3b, 23 },
    { 0, 0 },
    { 0x69102b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50002b41, 33 },
    { 0, 0 },
    { 0xdbf02b43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea02b46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca02b4a, 61 },
    { 0x65d02b4a, 66 },
    { 0x16502b4c, 98 },
    { 0, 0 },
    { 0xb0902b4e, 95 },
    { 0x5f802b4f, 76 },
    { 0xa0d02b50, 90 },
    { 0x27a02b51, 61 },
    { 0xedd02b50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b702b55, 55 },
    { 0, 0 },
    { 0x75502b57, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52002b5d, 98 },
    { 0x33402b5d, 55 },
    { 0, 0 },
    { 0xd9f02b60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf02b67, 33 },
    { 0x76e02b68, 84 },
    { 0x97902b68, 99 },
    { 0x41802b6a, 90 },
    { 0, 0 },
    { 0xebc02b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f02b70, 67 },
    { 0, 0 },
    { 0xd5802b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab02b79, 33 },
    { 0, 0 },
    { 0x65602b7b, 97 },
    { 0x4a502b7c, 98 },
    { 0, 0 },
    { 0x27402b7e, 95 },
    { 0xb6e02b7e, 55 },
    { 0x20102b7e, 34 },
    { 0xa2902b7f, 36 },
    { 0, 0 },
    { 0x6a702b83, 34 },
    { 0, 0 },
    { 0x2002b85, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0102b88, 23 },
    { 0, 0 },
    { 0x90f02b8a, 33 },
    { 0x2ee02b8b, 90 },
    { 0xa9102b8b, 55 },
    { 0, 0 },
    { 0xe8a02b8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79402b91, 61 },
    { 0x51702b92, 90 },
    { 0x54402b93, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a02b96, 23 },
    { 0xf1802b96, 23 },
    { 0, 0 },
    { 0xe8602b99, 69 },
    { 0xab902b9a, 98 },
    { 0x65602b9a, 33 },
    { 0x9b102b9c, 61 },
    { 0x30a02b9d, 76 },
    { 0xdeb02b9e, 61 },
    { 0x54802b9f, 77 },
    { 0xec402b9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47102ba3, 81 },
    { 0x49c02ba4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7202ba9, 66 },
    { 0, 0 },
    { 0x6e502bab, 55 },
    { 0, 0 },
    { 0x8f202bad, 52 },
    { 0x40102bae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb002bb9, 90 },
    { 0x36002bba, 95 },
    { 0xc4c02bbb, 95 },
    { 0x62a02bbc, 85 },
    { 0x1202bbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74402bc1, 77 },
    { 0x33e02bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12202bca, 46 },
    { 0, 0 },
    { 0x63902bcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f02bd1, 76 },
    { 0, 0 },
    { 0xe4b02bd3, 84 },
    { 0xbba02bd3, 76 },
    { 0x73302bd5, 90 },
    { 0, 0 },
    { 0x82002bd7, 91 },
    { 0, 0 },
    { 0xee402bd9, 95 },
    { 0x6b502bda, 33 },
    { 0x49802bdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c702be8, 36 },
    { 0x67802be9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf02bef, 76 },
    { 0x7b402bf0, 90 },
    { 0x85a02bf1, 61 },
    { 0, 0 },
    { 0x4dd02bf3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb02bf7, 78 },
    { 0, 0 },
    { 0x75202bf9, 76 },
    { 0, 0 },
    { 0x2002bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd02c02, 55 },
    { 0xc1d02c03, 78 },
    { 0x70b02c04, 23 },
    { 0x6db02c05, 76 },
    { 0x56702c06, 77 },
    { 0, 0 },
    { 0x9a802c08, 83 },
    { 0x2ec02c09, 43 },
    { 0x49002c09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a02c0e, 85 },
    { 0xf3702c0f, 34 },
    { 0x56a02c10, 90 },
    { 0x5e802c10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c402c14, 95 },
    { 0, 0 },
    { 0xc6802c16, 33 },
    { 0x4bc02c17, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd02c1a, 87 },
    { 0x45f02c1b, 95 },
    { 0x59e02c1c, 61 },
    { 0x7c802c1d, 61 },
    { 0x71802c1c, 33 },
    { 0x4b202c1d, 90 },
    { 0x19702c1c, 75 },
    { 0x66602c21, 90 },
    { 0, 0 },
    { 0x30c02c23, 33 },
    { 0, 0 },
    { 0x66902c25, 70 },
    { 0x3302c26, 76 },
    { 0x8e02c27, 98 },
    { 0, 0 },
    { 0x40002c29, 90 },
    { 0x14302c29, 55 },
    { 0, 0 },
    { 0xb8c02c2c, 87 },
    { 0x7b102c2c, 90 },
    { 0xabf02c2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91402c33, 98 },
    { 0x9dc02c34, 64 },
    { 0x59c02c34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee02c48, 95 },
    { 0xcac02c49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c502c4e, 88 },
    { 0x3fe02c4f, 95 },
    { 0xfcf02c50, 78 },
    { 0x34f02c51, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e02c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac02c5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81302c63, 97 },
    { 0xe8702c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff302c68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1102c6d, 81 },
    { 0, 0 },
    { 0x8fd02c6f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a302c73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x70502c76, 88 },
    { 0, 0 },
    { 0x80902c78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed902c80, 81 },
    { 0x53102c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26202c8d, 98 },
    { 0, 0 },
    { 0x20c02c8f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f02c98, 76 },
    { 0x21e02c99, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee02c9c, 38 },
    { 0x54302c9d, 90 },
    { 0xc1302c9c, 95 },
    { 0xc7502c9f, 43 },
    { 0, 0 },
    { 0x7802ca1, 90 },
    { 0xa4002ca2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39802ca5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3302caa, 43 },
    { 0, 0 },
    { 0xed602cac, 33 },
    { 0xeb02cad, 90 },
    { 0xdeb02cad, 61 },
    { 0x3f002caf, 93 },
    { 0xbdd02cb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc02cb4, 90 },
    { 0x9ee02cb5, 33 },
    { 0x60c02cb4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db02cbb, 55 },
    { 0, 0 },
    { 0xd6802cbd, 61 },
    { 0x36002cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e02cc1, 52 },
    { 0x29502cc1, 85 },
    { 0xf7d02cc3, 76 },
    { 0x83102cc3, 33 },
    { 0xc6702cc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2102cc9, 90 },
    { 0, 0 },
    { 0x31802ccb, 98 },
    { 0x1702ccc, 39 },
    { 0x91802ccb, 23 },
    { 0, 0 },
    { 0xf3902ccf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91502cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1402cda, 90 },
    { 0xe6c02cda, 23 },
    { 0xcf02cdc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87902ce0, 90 },
    { 0, 0 },
    { 0xc7902ce2, 81 },
    { 0x43602ce2, 90 },
    { 0xafa02ce3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff902ceb, 85 },
    { 0xb3202cec, 33 },
    { 0x23202ced, 98 },
    { 0, 0 },
    { 0xd8402cef, 23 },
    { 0, 0 },
    { 0xd0102cf1, 90 },
    { 0x68302cf2, 55 },
    { 0, 0 },
    { 0x52702cf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad02cfa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3802cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb902d04, 78 },
    { 0xab302d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0202d09, 76 },
    { 0, 0 },
    { 0xdc102d0b, 46 },
    { 0x86002d0b, 61 },
    { 0xfea02d0c, 23 },
    { 0, 0 },
    { 0x9f402d0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91102d14, 34 },
    { 0x7d002d14, 55 },
    { 0, 0 },
    { 0x55402d17, 98 },
    { 0, 0 },
    { 0x8d002d19, 33 },
    { 0, 0 },
    { 0xd0702d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f02d1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc802d22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a02d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f02d30, 61 },
    { 0xe3e02d31, 98 },
    { 0x62b02d32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87602d37, 90 },
    { 0xded02d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d02d3e, 61 },
    { 0x8e702d3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4102d43, 34 },
    { 0x57302d43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f502d4e, 88 },
    { 0x89002d4f, 76 },
    { 0xbdc02d50, 77 },
    { 0x59a02d51, 90 },
    { 0x9a302d52, 48 },
    { 0xe9602d53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d802d56, 61 },
    { 0x6ae02d57, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc702d5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c02d61, 43 },
    { 0x75702d62, 17 },
    { 0, 0 },
    { 0xee902d64, 90 },
    { 0xc5f02d65, 91 },
    { 0xe5a02d65, 23 },
    { 0x4f902d67, 90 },
    { 0x4e302d68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe602d76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57302d7b, 98 },
    { 0x96302d7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0702d83, 90 },
    { 0x1b402d84, 43 },
    { 0x5be02d85, 93 },
    { 0x28102d86, 46 },
    { 0xf2d02d87, 61 },
    { 0, 0 },
    { 0x3de02d89, 61 },
    { 0xbcc02d8a, 99 },
    { 0xebf02d89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a602d8e, 90 },
    { 0x7ec02d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1102d92, 61 },
    { 0, 0 },
    { 0xac502d94, 90 },
    { 0x15702d95, 33 },
    { 0, 0 },
    { 0xe3c02d97, 93 },
    { 0, 0 },
    { 0xd3902d99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60202d9e, 36 },
    { 0xac902d9f, 55 },
    { 0, 0 },
    { 0xa6a02da1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7202da6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc02dae, 33 },
    { 0x8bb02daf, 65 },
    { 0xd3e02daf, 23 },
    { 0xcc602db1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11802db8, 55 },
    { 0xe0b02db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6802dbc, 61 },
    { 0xe0202dbd, 76 },
    { 0xc3d02dbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3202dc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b02dc8, 90 },
    { 0x30b02dc8, 95 },
    { 0x3a402dca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a02dd2, 33 },
    { 0, 0 },
    { 0x9a402dd4, 76 },
    { 0, 0 },
    { 0x9df02dd6, 61 },
    { 0x92b02dd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72102ddb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a02de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db02de7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba102deb, 52 },
    { 0x6ad02dec, 95 },
    { 0x7ac02dec, 66 },
    { 0x81e02dee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b02df1, 90 },
    { 0xb7c02df1, 94 },
    { 0xa7102df3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae02df8, 76 },
    { 0x2d002df9, 95 },
    { 0x4a902df9, 55 },
    { 0xf2802df9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88702dfe, 95 },
    { 0x1c902dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4602e06, 23 },
    { 0, 0 },
    { 0x75e02e08, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd802e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e502e11, 61 },
    { 0x93702e11, 98 },
    { 0, 0 },
    { 0xad402e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4302e17, 34 },
    { 0x64802e17, 33 },
    { 0, 0 },
    { 0xa9502e1a, 33 },
    { 0, 0 },
    { 0xdc202e1c, 33 },
    { 0xf0802e1d, 64 },
    { 0x93602e1c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c02e21, 98 },
    { 0, 0 },
    { 0x5c202e23, 100 },
    { 0x46a02e23, 33 },
    { 0x9c502e25, 95 },
    { 0xf3902e26, 89 },
    { 0xb0c02e27, 78 },
    { 0xab302e28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99402e2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3802e37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b602e3b, 55 },
    { 0x23c02e3b, 61 },
    { 0xdce02e3d, 67 },
    { 0, 0 },
    { 0xe402e3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b602e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f02e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c602e58, 33 },
    { 0, 0 },
    { 0x38702e5a, 43 },
    { 0xcc702e5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d02e60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c702e68, 33 },
    { 0x14f02e69, 78 },
    { 0, 0 },
    { 0x1fd02e6b, 90 },
    { 0x39402e6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf02e6f, 55 },
    { 0x9ab02e70, 23 },
    { 0, 0 },
    { 0xf8902e72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d02e76, 52 },
    { 0x65a02e76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab02e7d, 23 },
    { 0xb5a02e7e, 87 },
    { 0xf9902e7f, 93 },
    { 0, 0 },
    { 0x2fc02e81, 33 },
    { 0xaad02e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a802e85, 43 },
    { 0xe7402e85, 33 },
    { 0xe7e02e85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa802e90, 43 },
    { 0x9b302e91, 84 },
    { 0x46502e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c02e95, 99 },
    { 0xbb802e96, 34 },
    { 0, 0 },
    { 0x1b002e98, 80 },
    { 0xc6102e99, 75 },
    { 0x56c02e9a, 43 },
    { 0xfc102e98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba02e9f, 33 },
    { 0x60202ea0, 43 },
    { 0, 0 },
    { 0x5a102ea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e202ea7, 98 },
    { 0x14202ea7, 76 },
    { 0x7a302ea7, 34 },
    { 0, 0 },
    { 0x52502eab, 33 },
    { 0xba802eac, 87 },
    { 0x59902eac, 95 },
    { 0xdf102eae, 61 },
    { 0x16002eaf, 23 },
    { 0xc1702eb0, 17 },
    { 0xd4702eae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c902eb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c02eba, 23 },
    { 0xbe002ebb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d02ebe, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9902ec1, 67 },
    { 0xc4f02ec2, 87 },
    { 0xc3102ec3, 98 },
    { 0, 0 },
    { 0xccf02ec5, 76 },
    { 0x54202ec5, 61 },
    { 0xdd402ec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76002ecf, 48 },
    { 0x2af02ed0, 95 },
    { 0, 0 },
    { 0x98a02ed2, 61 },
    { 0xcdb02ed2, 33 },
    { 0x84402ed2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c402ed7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf02eda, 34 },
    { 0x6dc02edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61302edf, 100 },
    { 0, 0 },
    { 0xee402ee1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5002ee4, 98 },
    { 0, 0 },
    { 0x4e702ee6, 90 },
    { 0xf6e02ee7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40002ef3, 87 },
    { 0x3d502ef4, 90 },
    { 0x63802ef4, 33 },
    { 0xb7d02ef6, 86 },
    { 0, 0 },
    { 0xd3b02ef8, 61 },
    { 0x65902ef9, 95 },
    { 0, 0 },
    { 0x87002efb, 61 },
    { 0, 0 },
    { 0x5d402efd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74902f03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca02f06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c02f0c, 95 },
    { 0xb4102f0d, 94 },
    { 0, 0 },
    { 0x46602f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94202f19, 23 },
    { 0, 0 },
    { 0x59a02f1b, 90 },
    { 0xaf902f1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93702f23, 23 },
    { 0xe1c02f23, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5302f27, 33 },
    { 0x97b02f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b02f33, 95 },
    { 0xe1702f34, 77 },
    { 0x68f02f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2002f3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c02f45, 76 },
    { 0x41602f46, 33 },
    { 0, 0 },
    { 0x12802f48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d02f4c, 76 },
    { 0xe4902f4d, 61 },
    { 0xc6802f4d, 38 },
    { 0x40402f4f, 61 },
    { 0, 0 },
    { 0xd9702f51, 55 },
    { 0, 0 },
    { 0xb4602f53, 23 },
    { 0, 0 },
    { 0x64802f55, 56 },
    { 0x1ad02f55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15602f5c, 99 },
    { 0x9d702f5d, 61 },
    { 0, 0 },
    { 0x50802f5f, 95 },
    { 0xe2702f60, 61 },
    { 0x4ef02f61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea502f67, 61 },
    { 0x58002f68, 61 },
    { 0, 0 },
    { 0xaff02f6a, 98 },
    { 0xe8602f6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6702f70, 46 },
    { 0x19f02f70, 76 },
    { 0xbf302f72, 33 },
    { 0xfa502f73, 91 },
    { 0, 0 },
    { 0x1b302f75, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b02f79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd002f7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86902f81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba002f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46802f8f, 93 },
    { 0, 0 },
    { 0xcf602f91, 95 },
    { 0x59702f92, 33 },
    { 0xe9902f91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95502f96, 90 },
    { 0x5b02f96, 95 },
    { 0xf4e02f98, 98 },
    { 0x3d202f99, 34 },
    { 0x4b202f96, 95 },
    { 0xe3902f9b, 90 },
    { 0x85402f9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84202fa0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b202fa3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f02faa, 40 },
    { 0xdcc02fab, 33 },
    { 0xefa02fac, 95 },
    { 0xd7802fad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x19502fb0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f02fb4, 77 },
    { 0x84502fb5, 33 },
    { 0, 0 },
    { 0x402fb7, 90 },
    { 0xbff02fb8, 33 },
    { 0x5f802fb9, 98 },
    { 0xb4802fba, 61 },
    { 0xe3b02fba, 61 },
    { 0, 0 },
    { 0xdc102fbd, 90 },
    { 0x32b02fbe, 76 },
    { 0xd1002fbe, 34 },
    { 0x47202fc0, 88 },
    { 0x5ef02fc0, 61 },
    { 0x73002fc2, 55 },
    { 0xb4a02fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3002fc6, 85 },
    { 0, 0 },
    { 0x5b502fc8, 90 },
    { 0, 0 },
    { 0x37a02fca, 65 },
    { 0xe5502fcb, 33 },
    { 0xa2f02fcb, 34 },
    { 0xc5a02fcd, 33 },
    { 0x25602fcd, 66 },
    { 0xff902fce, 90 },
    { 0xbdc02fcd, 84 },
    { 0x96302fcf, 66 },
    { 0x4d402fcc, 98 },
    { 0xf7d02fd3, 61 },
    { 0, 0 },
    { 0x9702fd5, 33 },
    { 0xb5402fd5, 33 },
    { 0, 0 },
    { 0x90802fd8, 93 },
    { 0xc7402fd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb002fdd, 95 },
    { 0x55402fde, 61 },
    { 0xada02fdf, 90 },
    { 0x11602fe0, 46 },
    { 0xdcb02fde, 90 },
    { 0x2b202fe2, 61 },
    { 0xe4802fe2, 90 },
    { 0x26d02fe4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa02fe8, 98 },
    { 0xa2402fe9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb02fec, 90 },
    { 0x96b02fed, 38 },
    { 0x29c02fee, 76 },
    { 0x5db02fef, 76 },
    { 0, 0 },
    { 0x18402ff1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa02ff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd302ffb, 61 },
    { 0, 0 },
    { 0x26b02ffd, 75 },
    { 0x2a802ffe, 90 },
    { 0, 0 },
    { 0x10003000, 61 },
    { 0xc5203000, 70 },
    { 0x67503000, 53 },
    { 0xdc403003, 98 },
    { 0x5cd03004, 93 },
    { 0x7c103003, 90 },
    { 0x30b03006, 55 },
    { 0x27203006, 55 },
    { 0x47103008, 87 },
    { 0x91303009, 61 },
    { 0x61a0300a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d0300d, 90 },
    { 0x49e0300d, 43 },
    { 0xa1c0300d, 61 },
    { 0xbb10300f, 23 },
    { 0x55d03011, 95 },
    { 0, 0 },
    { 0x60e03013, 43 },
    { 0x2cd03014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0301d, 100 },
    { 0, 0 },
    { 0x94a0301f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa03022, 33 },
    { 0xa8803022, 93 },
    { 0xd7303022, 43 },
    { 0x80403024, 81 },
    { 0x4da03026, 90 },
    { 0, 0 },
    { 0xbb703028, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc500302b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf0302f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd03032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31703035, 61 },
    { 0x5d203036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb03039, 90 },
    { 0, 0 },
    { 0x2d30303b, 66 },
    { 0x5880303c, 61 },
    { 0, 0 },
    { 0x80c0303e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c703041, 98 },
    { 0xcc003042, 52 },
    { 0x91203043, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00304a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd03050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90003054, 68 },
    { 0, 0 },
    { 0xafc03056, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe0305c, 33 },
    { 0x4900305c, 46 },
    { 0, 0 },
    { 0xa730305f, 33 },
    { 0, 0 },
    { 0x53803061, 23 },
    { 0x54e03062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe330306d, 76 },
    { 0xed60306d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39503072, 61 },
    { 0x85e03073, 55 },
    { 0xa2d03072, 23 },
    { 0xe603073, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2303079, 55 },
    { 0xda503079, 76 },
    { 0xa720307b, 61 },
    { 0, 0 },
    { 0xc460307d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e03082, 95 },
    { 0x4f903082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9380308a, 95 },
    { 0xccb0308b, 95 },
    { 0x4910308b, 38 },
    { 0, 0 },
    { 0xf140308e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c03093, 52 },
    { 0x4a303094, 69 },
    { 0xc0603095, 55 },
    { 0x8f03096, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3390309b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f030a3, 78 },
    { 0x7d1030a4, 53 },
    { 0xf42030a5, 76 },
    { 0x7d3030a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x962030a9, 76 },
    { 0, 0 },
    { 0x172030ab, 23 },
    { 0xa52030ab, 33 },
    { 0xf7b030ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e030b1, 33 },
    { 0x2b3030b2, 23 },
    { 0xc4b030b2, 90 },
    { 0x998030b4, 90 },
    { 0x2d5030b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455030bb, 95 },
    { 0, 0 },
    { 0x68d030bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da030c2, 95 },
    { 0x3c2030c3, 43 },
    { 0x4c4030c4, 33 },
    { 0x4ac030c5, 75 },
    { 0, 0 },
    { 0x950030c7, 90 },
    { 0x623030c8, 34 },
    { 0, 0 },
    { 0x420030ca, 34 },
    { 0, 0 },
    { 0xfa2030cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5030d0, 95 },
    { 0, 0 },
    { 0xe49030d2, 23 },
    { 0xeea030d3, 98 },
    { 0, 0 },
    { 0xf21030d5, 43 },
    { 0, 0 },
    { 0xc03030d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785030e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f030e8, 46 },
    { 0xa74030e9, 55 },
    { 0, 0 },
    { 0xbec030eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc030f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x757030f3, 81 },
    { 0xd19030f3, 90 },
    { 0xa25030f5, 55 },
    { 0, 0 },
    { 0x2b8030f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899030fe, 61 },
    { 0x331030fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef03103, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75903108, 95 },
    { 0x26903109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf70310e, 90 },
    { 0x96e0310f, 95 },
    { 0xb7403110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab03114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a30311c, 46 },
    { 0, 0 },
    { 0xbda0311e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd603121, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6303125, 23 },
    { 0x3ed03126, 98 },
    { 0, 0 },
    { 0x83b03128, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b0312b, 90 },
    { 0x7a70312c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd990312f, 43 },
    { 0xd0f03130, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56203134, 76 },
    { 0x24403135, 61 },
    { 0x4a803135, 55 },
    { 0x6a103137, 61 },
    { 0x60003138, 55 },
    { 0x12603139, 33 },
    { 0, 0 },
    { 0x7d0313b, 75 },
    { 0xb810313c, 93 },
    { 0x7550313d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5903143, 95 },
    { 0xbda03144, 65 },
    { 0x42b03143, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc03150, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79703153, 84 },
    { 0x13803154, 46 },
    { 0x20d03154, 81 },
    { 0x7e03155, 95 },
    { 0x99903157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9040315f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f03162, 75 },
    { 0xa4c03163, 55 },
    { 0x7b903163, 68 },
    { 0, 0 },
    { 0x30a03166, 55 },
    { 0, 0 },
    { 0x2ba03168, 34 },
    { 0, 0 },
    { 0xc070316a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb403170, 61 },
    { 0x95c03171, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d03174, 33 },
    { 0xa4603174, 33 },
    { 0xf5203176, 100 },
    { 0x59903177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e20317d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c03183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe803189, 76 },
    { 0, 0 },
    { 0xf40318b, 95 },
    { 0xa640318c, 87 },
    { 0xafc0318b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3503190, 76 },
    { 0x51e03191, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede03199, 98 },
    { 0, 0 },
    { 0x9b30319b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee031a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4031a4, 33 },
    { 0, 0 },
    { 0x4cc031a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531031ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c031ae, 84 },
    { 0xeb0031af, 93 },
    { 0xa6f031af, 95 },
    { 0xcd8031b1, 98 },
    { 0x808031b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6031b6, 95 },
    { 0x953031b6, 95 },
    { 0x6e2031b8, 55 },
    { 0x16f031b7, 93 },
    { 0x679031ba, 98 },
    { 0x705031ba, 52 },
    { 0x61c031bb, 100 },
    { 0x39a031bd, 91 },
    { 0x7f1031be, 98 },
    { 0x18f031bd, 87 },
    { 0x19b031bd, 61 },
    { 0x1b3031c1, 43 },
    { 0x7c1031c2, 52 },
    { 0xf11031bb, 23 },
    { 0, 0 },
    { 0x19d031c5, 95 },
    { 0x79c031c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd031cb, 34 },
    { 0xa7a031cb, 33 },
    { 0x2b7031cd, 80 },
    { 0xf14031cd, 85 },
    { 0xa5c031cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x223031d2, 55 },
    { 0, 0 },
    { 0x2f0031d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9031d8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c031dd, 61 },
    { 0, 0 },
    { 0x800031df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f031e3, 33 },
    { 0xe8f031e3, 65 },
    { 0, 0 },
    { 0x218031e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252031eb, 33 },
    { 0x306031eb, 66 },
    { 0x1e1031ec, 33 },
    { 0x4a0031ee, 95 },
    { 0x2a1031ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6031f4, 61 },
    { 0x85d031f4, 93 },
    { 0x393031f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x793031f9, 77 },
    { 0x685031fa, 46 },
    { 0, 0 },
    { 0x28e031fc, 33 },
    { 0x305031fd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41803204, 55 },
    { 0xe4903205, 61 },
    { 0xa703204, 43 },
    { 0x26703207, 33 },
    { 0x13903207, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570320e, 90 },
    { 0x8e60320f, 33 },
    { 0, 0 },
    { 0x58703211, 95 },
    { 0, 0 },
    { 0x65f03213, 61 },
    { 0x4eb03213, 90 },
    { 0, 0 },
    { 0xd8603216, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45103219, 85 },
    { 0xf5303219, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf600321f, 46 },
    { 0, 0 },
    { 0x2d903221, 99 },
    { 0xb2e03222, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae03227, 23 },
    { 0, 0 },
    { 0xaa703229, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf490322c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e0322f, 33 },
    { 0x7490322f, 89 },
    { 0xd3003231, 33 },
    { 0xad0322f, 34 },
    { 0xbe903232, 52 },
    { 0xba403234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f803237, 95 },
    { 0x87403238, 55 },
    { 0xb1103239, 90 },
    { 0x160323a, 91 },
    { 0, 0 },
    { 0xcde0323c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a0323f, 77 },
    { 0xd2803240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b03244, 43 },
    { 0, 0 },
    { 0x8f103246, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b0324a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe520324e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4903253, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff03258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4103263, 98 },
    { 0xc0e03264, 55 },
    { 0x91d03264, 61 },
    { 0x5a103266, 61 },
    { 0x6903267, 90 },
    { 0x5dd03266, 43 },
    { 0x35203269, 33 },
    { 0x5e703266, 33 },
    { 0x9cb03265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70326e, 61 },
    { 0x7b30326f, 66 },
    { 0, 0 },
    { 0xf503271, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3274, 85 },
    { 0x62403274, 90 },
    { 0, 0 },
    { 0x68503277, 33 },
    { 0x3fb03278, 95 },
    { 0xa8d03279, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb100327c, 90 },
    { 0xf10327d, 76 },
    { 0xe130327c, 78 },
    { 0xbf10327d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e03282, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81303290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d503293, 23 },
    { 0x74003293, 76 },
    { 0, 0 },
    { 0x55a03296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa0329e, 61 },
    { 0x6fb0329e, 33 },
    { 0x6f30329f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf032a5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc032a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc032ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c032b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2032b7, 90 },
    { 0xf82032b7, 33 },
    { 0xa43032b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97032bc, 61 },
    { 0xf30032bd, 90 },
    { 0x60b032bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f032c1, 98 },
    { 0xac1032c2, 90 },
    { 0xd98032c3, 76 },
    { 0x992032c3, 61 },
    { 0xd80032c3, 17 },
    { 0xa46032c1, 61 },
    { 0, 0 },
    { 0xbf6032c8, 90 },
    { 0, 0 },
    { 0x934032ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae032cf, 55 },
    { 0xc5b032d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef032d7, 61 },
    { 0, 0 },
    { 0xb95032d9, 33 },
    { 0, 0 },
    { 0x385032db, 95 },
    { 0xa85032dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d032df, 76 },
    { 0xf0032df, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8032e4, 76 },
    { 0x209032e5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327032ec, 61 },
    { 0, 0 },
    { 0xd8c032ee, 90 },
    { 0xb98032ef, 76 },
    { 0, 0 },
    { 0x72e032f1, 55 },
    { 0x74c032f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b032f7, 76 },
    { 0xf02032f7, 90 },
    { 0xc8c032f7, 23 },
    { 0x52c032f7, 23 },
    { 0xd35032f8, 98 },
    { 0xec8032fc, 65 },
    { 0xa6032fd, 77 },
    { 0xa1f032fd, 61 },
    { 0x867032ff, 98 },
    { 0xe0e03300, 88 },
    { 0x1a503300, 33 },
    { 0xe2c03300, 23 },
    { 0xd3e03303, 93 },
    { 0xc09032ff, 23 },
    { 0, 0 },
    { 0xbc003306, 76 },
    { 0, 0 },
    { 0x85703308, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd60330e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac03314, 87 },
    { 0xf8c03314, 34 },
    { 0x5d603316, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5203319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9390331c, 34 },
    { 0, 0 },
    { 0x43d0331e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00332a, 69 },
    { 0, 0 },
    { 0xfa60332c, 23 },
    { 0x7c30332d, 19 },
    { 0, 0 },
    { 0x8250332f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb03334, 33 },
    { 0x75d03335, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9460333a, 90 },
    { 0x7c10333a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12503340, 34 },
    { 0x1a203341, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xece03344, 78 },
    { 0xf0303345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e90334b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab503352, 90 },
    { 0xafe03352, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e303357, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf310335b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df0335e, 33 },
    { 0xbb30335f, 90 },
    { 0, 0 },
    { 0xcb603361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8080336b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e03371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a03376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83103379, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2790337e, 33 },
    { 0x9820337f, 33 },
    { 0xd170337f, 43 },
    { 0xd103381, 66 },
    { 0x7eb0337f, 90 },
    { 0x5600337f, 90 },
    { 0x1370337e, 98 },
    { 0xe1403385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960338f, 95 },
    { 0x81a03390, 82 },
    { 0, 0 },
    { 0x7a303392, 61 },
    { 0, 0 },
    { 0xe303394, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8340339d, 90 },
    { 0x3c80339e, 85 },
    { 0xe510339d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70033a4, 76 },
    { 0x264033a5, 34 },
    { 0xd57033a4, 34 },
    { 0xe45033a7, 90 },
    { 0x8bc033a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0033ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6033af, 76 },
    { 0, 0 },
    { 0x7f9033b1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c033b4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x791033b7, 95 },
    { 0x4d7033b8, 61 },
    { 0x2c4033b9, 23 },
    { 0x2fd033ba, 95 },
    { 0x584033ba, 33 },
    { 0xdbd033b8, 34 },
    { 0x324033ba, 76 },
    { 0xfe7033be, 98 },
    { 0x60a033be, 91 },
    { 0xe52033bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe033c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556033c9, 90 },
    { 0x26e033c9, 23 },
    { 0x1af033cb, 58 },
    { 0xd59033ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06033d1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f033d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922033dd, 76 },
    { 0xaf1033dd, 61 },
    { 0x48033df, 90 },
    { 0x5ae033de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193033e5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d033e8, 23 },
    { 0x6b0033e9, 23 },
    { 0x5a033ea, 33 },
    { 0x8e5033ea, 90 },
    { 0, 0 },
    { 0x4b6033ed, 88 },
    { 0x49b033ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6033f1, 90 },
    { 0, 0 },
    { 0xaa5033f3, 66 },
    { 0x9e4033f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597033ff, 100 },
    { 0, 0 },
    { 0x6e303401, 99 },
    { 0, 0 },
    { 0xec103403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe03407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd0340b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b0340e, 90 },
    { 0x4a70340f, 90 },
    { 0x37603410, 93 },
    { 0xcfd03410, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x79503414, 61 },
    { 0xca503415, 98 },
    { 0, 0 },
    { 0x98003417, 76 },
    { 0xe8703417, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f0341c, 87 },
    { 0x2f50341d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74703420, 52 },
    { 0xda203420, 95 },
    { 0x10403421, 33 },
    { 0x2b03420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92403427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9990342a, 76 },
    { 0xe170342a, 98 },
    { 0xc5e0342c, 98 },
    { 0x8870342a, 90 },
    { 0xfb40342e, 76 },
    { 0, 0 },
    { 0x30403430, 76 },
    { 0x5e603430, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d03439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1080343c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf503441, 61 },
    { 0x24503441, 23 },
    { 0x8c003442, 33 },
    { 0x1f03443, 75 },
    { 0x84c03445, 23 },
    { 0x9b303445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b90344d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb03450, 93 },
    { 0x70803450, 33 },
    { 0xaea03451, 95 },
    { 0, 0 },
    { 0x92d03454, 91 },
    { 0x6fb03454, 33 },
    { 0xf4103456, 61 },
    { 0x10403455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc0345a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba30345d, 33 },
    { 0x2ae0345d, 17 },
    { 0, 0 },
    { 0xa3603460, 95 },
    { 0x74503461, 40 },
    { 0x72c03461, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8300346d, 23 },
    { 0x3280346d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d03471, 90 },
    { 0x65603472, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb03477, 76 },
    { 0x7fc03477, 93 },
    { 0x19603477, 95 },
    { 0x5ad03478, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7710347d, 33 },
    { 0xe150347e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42503485, 61 },
    { 0x1d503485, 23 },
    { 0x5a703487, 98 },
    { 0, 0 },
    { 0xd8103489, 88 },
    { 0xac603489, 87 },
    { 0, 0 },
    { 0x7e50348c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61603495, 95 },
    { 0xeb203495, 90 },
    { 0xb6903497, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220349d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9034a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43034a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62034a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54034ac, 61 },
    { 0x5b2034ac, 95 },
    { 0xc3d034ae, 33 },
    { 0, 0 },
    { 0xa6034b0, 43 },
    { 0, 0 },
    { 0xa12034b2, 34 },
    { 0, 0 },
    { 0xa6c034b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97034b7, 98 },
    { 0x103034b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14034c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca034c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b034cc, 33 },
    { 0, 0 },
    { 0x337034ce, 76 },
    { 0x284034ce, 61 },
    { 0, 0 },
    { 0xbca034d1, 90 },
    { 0xee034d1, 90 },
    { 0xe36034d1, 99 },
    { 0xea7034d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e034d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd034dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b034e5, 52 },
    { 0xbc4034e6, 76 },
    { 0x605034e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x174034ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12034ee, 95 },
    { 0xd33034ee, 61 },
    { 0x251034ee, 84 },
    { 0x845034f0, 55 },
    { 0x8d3034ef, 31 },
    { 0xb39034f1, 99 },
    { 0x15d034f1, 53 },
    { 0xf5e034f5, 84 },
    { 0, 0 },
    { 0xbeb034f7, 61 },
    { 0xa41034f8, 33 },
    { 0xf32034f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd703500, 33 },
    { 0, 0 },
    { 0xb9603502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98403507, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a303515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4830351c, 90 },
    { 0x7a50351d, 34 },
    { 0, 0 },
    { 0xece0351f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50703522, 95 },
    { 0xa2003523, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e60352a, 61 },
    { 0xdff0352b, 81 },
    { 0, 0 },
    { 0x4050352d, 34 },
    { 0, 0 },
    { 0xcd60352f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28803534, 76 },
    { 0, 0 },
    { 0x7cf03536, 34 },
    { 0x4e403536, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c0353b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a80353e, 33 },
    { 0xd030353e, 33 },
    { 0, 0 },
    { 0x7a203541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa0354a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c0354f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4580355c, 90 },
    { 0x8180355d, 90 },
    { 0, 0 },
    { 0xb9f0355f, 98 },
    { 0x3aa03560, 61 },
    { 0xc9803561, 64 },
    { 0xd6c03560, 95 },
    { 0x8b80355f, 98 },
    { 0x69c03564, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade03568, 55 },
    { 0x60f03569, 61 },
    { 0, 0 },
    { 0xbd20356b, 33 },
    { 0, 0 },
    { 0x8c00356d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d03572, 33 },
    { 0x80a03573, 90 },
    { 0x7503574, 23 },
    { 0xf2303575, 99 },
    { 0xd2f03574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f0357e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f03582, 34 },
    { 0x6203583, 66 },
    { 0xb4103582, 33 },
    { 0x9403585, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51103589, 23 },
    { 0x85e0358a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a03592, 33 },
    { 0x9503592, 61 },
    { 0x61f03594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30503597, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa680359c, 55 },
    { 0x6d90359d, 52 },
    { 0x3530359e, 23 },
    { 0xc6e0359f, 61 },
    { 0x4480359c, 67 },
    { 0, 0 },
    { 0x61035a2, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78035a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1035ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363035b2, 76 },
    { 0x3d6035b3, 22 },
    { 0x18e035b3, 90 },
    { 0xa47035b2, 23 },
    { 0x753035b6, 34 },
    { 0xa0c035b5, 33 },
    { 0xa4a035b6, 17 },
    { 0xd03035b9, 98 },
    { 0xde5035b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9035bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x880035c0, 95 },
    { 0, 0 },
    { 0x951035c2, 23 },
    { 0x17c035c2, 76 },
    { 0xe8e035c2, 43 },
    { 0, 0 },
    { 0x91b035c6, 61 },
    { 0, 0 },
    { 0x943035c8, 66 },
    { 0x907035c9, 99 },
    { 0xb67035c9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27035ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685035d2, 95 },
    { 0, 0 },
    { 0xcda035d4, 23 },
    { 0x119035d5, 55 },
    { 0x3b3035d4, 98 },
    { 0x491035d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132035db, 43 },
    { 0xb07035dc, 98 },
    { 0x2c7035dd, 90 },
    { 0x5b035de, 34 },
    { 0, 0 },
    { 0x5a035e0, 43 },
    { 0x55d035e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be035e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3035ec, 95 },
    { 0x55b035ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7035f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1035f7, 43 },
    { 0, 0 },
    { 0xdd035f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0035fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x747035ff, 95 },
    { 0x59303600, 98 },
    { 0, 0 },
    { 0x1e603602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea03605, 90 },
    { 0, 0 },
    { 0xa1403607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590360c, 98 },
    { 0x620360d, 61 },
    { 0x1570360e, 33 },
    { 0x8060360f, 90 },
    { 0, 0 },
    { 0xa3903611, 98 },
    { 0x16e03612, 61 },
    { 0xc6e03613, 33 },
    { 0x93903614, 33 },
    { 0xc1f03612, 33 },
    { 0, 0 },
    { 0x3e03617, 34 },
    { 0x3a103618, 87 },
    { 0xd4503619, 23 },
    { 0x3df03617, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c0361d, 90 },
    { 0x9990361e, 65 },
    { 0xf710361e, 33 },
    { 0x3750361e, 33 },
    { 0xb0d03621, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61003628, 61 },
    { 0, 0 },
    { 0x3790362a, 95 },
    { 0, 0 },
    { 0xd620362c, 90 },
    { 0xeb20362d, 53 },
    { 0x9900362e, 80 },
    { 0xf150362e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e03632, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f03637, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc0363a, 95 },
    { 0, 0 },
    { 0x4b10363c, 98 },
    { 0xc6a0363d, 81 },
    { 0xb340363e, 46 },
    { 0, 0 },
    { 0x9fe03640, 67 },
    { 0x99803641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46303645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd090364a, 91 },
    { 0x97c0364a, 98 },
    { 0, 0 },
    { 0xe7f0364d, 90 },
    { 0xe430364e, 78 },
    { 0xc30364f, 65 },
    { 0xb4403650, 65 },
    { 0xa4403650, 33 },
    { 0x76903652, 43 },
    { 0x32803652, 61 },
    { 0x3d403652, 66 },
    { 0xfbc03650, 77 },
    { 0x72a03655, 46 },
    { 0x7d803650, 90 },
    { 0xcbb0364d, 100 },
    { 0x58303654, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f60365d, 34 },
    { 0, 0 },
    { 0xae90365f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f03665, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc03669, 80 },
    { 0, 0 },
    { 0xcc30366b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5803670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa503674, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf03679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64303685, 90 },
    { 0, 0 },
    { 0x82303687, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8020368a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be03690, 98 },
    { 0xd7b03691, 76 },
    { 0x8dc03692, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1803695, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410369f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414036a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8036b0, 76 },
    { 0x90c036b1, 93 },
    { 0x760036b2, 90 },
    { 0, 0 },
    { 0x632036b4, 33 },
    { 0, 0 },
    { 0x4d3036b6, 65 },
    { 0xacf036b7, 95 },
    { 0x4e7036b8, 90 },
    { 0x260036b8, 78 },
    { 0, 0 },
    { 0xb8a036bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1036c0, 76 },
    { 0x7f9036c1, 58 },
    { 0x5ab036c2, 91 },
    { 0, 0 },
    { 0xe01036c4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5036c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be036cb, 78 },
    { 0x2e1036cb, 91 },
    { 0xcb9036cb, 94 },
    { 0, 0 },
    { 0x12e036cf, 69 },
    { 0x5d8036cf, 33 },
    { 0, 0 },
    { 0x2aa036d2, 90 },
    { 0xbe7036d3, 97 },
    { 0x7b1036d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380036dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9036e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6036e5, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536036e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc036ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0036f2, 17 },
    { 0x111036f3, 43 },
    { 0, 0 },
    { 0xe16036f5, 34 },
    { 0, 0 },
    { 0xcd1036f7, 34 },
    { 0, 0 },
    { 0xbae036f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b03702, 78 },
    { 0, 0 },
    { 0x5bf03704, 98 },
    { 0xc2f03705, 77 },
    { 0x36a03704, 95 },
    { 0x55a03707, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e10370b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7700370e, 46 },
    { 0, 0 },
    { 0x7f303710, 95 },
    { 0x22703710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb703715, 76 },
    { 0xa7203716, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x22903719, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b0371c, 90 },
    { 0, 0 },
    { 0x5bc0371e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66903729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840372d, 98 },
    { 0x7290372e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a03732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f03735, 61 },
    { 0, 0 },
    { 0x96003737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db0373c, 95 },
    { 0x23b0373d, 46 },
    { 0, 0 },
    { 0x8490373f, 90 },
    { 0, 0 },
    { 0x57703741, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c03747, 55 },
    { 0xb3e03748, 34 },
    { 0xaa303749, 98 },
    { 0x42a03749, 95 },
    { 0x52f0374b, 95 },
    { 0x28f03747, 33 },
    { 0, 0 },
    { 0x3910374e, 78 },
    { 0, 0 },
    { 0xb2a03750, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae03758, 55 },
    { 0x3ab03758, 76 },
    { 0x1503758, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b303760, 90 },
    { 0, 0 },
    { 0xbef03762, 33 },
    { 0x95103763, 46 },
    { 0x79903763, 77 },
    { 0xa5e03764, 99 },
    { 0, 0 },
    { 0xe1e03767, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d0376b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6703770, 76 },
    { 0x42903771, 98 },
    { 0, 0 },
    { 0x2a303773, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670377b, 46 },
    { 0x11e0377c, 75 },
    { 0x5fd0377d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f03781, 93 },
    { 0xa903782, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0403786, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c10378b, 90 },
    { 0xd090378b, 34 },
    { 0, 0 },
    { 0x1f90378e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92403795, 61 },
    { 0x3ac03796, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e03799, 33 },
    { 0, 0 },
    { 0x450379b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231037a2, 66 },
    { 0, 0 },
    { 0xfff037a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb037ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a037b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38037b6, 55 },
    { 0, 0 },
    { 0x82f037b8, 95 },
    { 0, 0 },
    { 0x442037ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a037c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611037c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7037c7, 55 },
    { 0x1e9037c8, 90 },
    { 0, 0 },
    { 0xefe037ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc037ce, 76 },
    { 0xa037cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a037d2, 95 },
    { 0, 0 },
    { 0x550037d4, 88 },
    { 0x9fe037d4, 61 },
    { 0x85c037d5, 98 },
    { 0x1c8037d7, 61 },
    { 0xcd9037d8, 80 },
    { 0x946037d9, 61 },
    { 0x9c0037d9, 34 },
    { 0x94e037d5, 95 },
    { 0, 0 },
    { 0xe38037dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732037e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e037ec, 90 },
    { 0x2f4037ed, 90 },
    { 0x8f037ec, 68 },
    { 0x9c0037ef, 94 },
    { 0x871037ef, 34 },
    { 0x823037ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51037f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x331037f9, 98 },
    { 0x9f1037f9, 78 },
    { 0xf3a037fa, 55 },
    { 0xdec037fc, 61 },
    { 0x6e3037fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f003803, 33 },
    { 0, 0 },
    { 0xf1303805, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b0380d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb103811, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d03815, 95 },
    { 0x92b03816, 33 },
    { 0x3e203817, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a0381a, 33 },
    { 0, 0 },
    { 0xbf50381c, 61 },
    { 0x5950381d, 100 },
    { 0xe0d0381e, 46 },
    { 0, 0 },
    { 0x6fe03820, 65 },
    { 0, 0 },
    { 0xc8803822, 61 },
    { 0, 0 },
    { 0x3a303824, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15503829, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4470382d, 90 },
    { 0, 0 },
    { 0xafd0382f, 23 },
    { 0xb2b0382f, 43 },
    { 0xcc303831, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30403834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c03837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c03840, 95 },
    { 0, 0 },
    { 0x3b303842, 90 },
    { 0x8a703843, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61203849, 23 },
    { 0xbe703849, 90 },
    { 0x8bd0384b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93403857, 90 },
    { 0xb0d03858, 99 },
    { 0x3fd03859, 23 },
    { 0x1d403858, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9350385d, 76 },
    { 0, 0 },
    { 0x3540385f, 23 },
    { 0xf9803860, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0003868, 33 },
    { 0, 0 },
    { 0x640386a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd0386e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e03874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xef303877, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e0387a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a00387d, 93 },
    { 0x4600387d, 95 },
    { 0, 0 },
    { 0x30703880, 90 },
    { 0, 0 },
    { 0xf5603882, 33 },
    { 0xc3c03883, 98 },
    { 0x82603884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00388b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc03890, 40 },
    { 0xdb603891, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd03894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20389a, 90 },
    { 0xe0b0389b, 90 },
    { 0xd400389c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c038a0, 99 },
    { 0xa59038a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f038a7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8038ab, 90 },
    { 0x4aa038ab, 99 },
    { 0, 0 },
    { 0x24038ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40038b2, 52 },
    { 0, 0 },
    { 0x38b038b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9038b8, 34 },
    { 0x3f3038b9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc038c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40038cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d038cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb038d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3038da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef038dd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452038e8, 23 },
    { 0x4fc038e9, 95 },
    { 0x911038ea, 90 },
    { 0, 0 },
    { 0x9f4038ec, 95 },
    { 0x794038ec, 33 },
    { 0, 0 },
    { 0x171038ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c038f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87038f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c038fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da038ff, 77 },
    { 0, 0 },
    { 0x57803901, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d03907, 40 },
    { 0, 0 },
    { 0x8e803909, 90 },
    { 0, 0 },
    { 0x9d20390b, 98 },
    { 0x7030390b, 36 },
    { 0x56a0390d, 100 },
    { 0x7200390d, 87 },
    { 0x3c70390f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x49703912, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c403915, 55 },
    { 0, 0 },
    { 0x77103917, 23 },
    { 0x5c803917, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2240391d, 23 },
    { 0x7d00391e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d503922, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0392c, 75 },
    { 0xb1b0392d, 95 },
    { 0x92b0392d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2703933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff03938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d0393d, 87 },
    { 0xdda0393e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f03945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7420394a, 61 },
    { 0x6460394b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0394e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98503952, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d03958, 95 },
    { 0xea403959, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc260395d, 34 },
    { 0, 0 },
    { 0x7cf0395f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80403962, 27 },
    { 0xcd03962, 33 },
    { 0xb7a03962, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e00396a, 33 },
    { 0x72f0396b, 84 },
    { 0x490396b, 90 },
    { 0xdff0396b, 69 },
    { 0xe940396e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8903971, 81 },
    { 0x64603971, 91 },
    { 0x5d303972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39803978, 98 },
    { 0x1503979, 85 },
    { 0, 0 },
    { 0x35d0397b, 78 },
    { 0xa6d0397c, 33 },
    { 0, 0 },
    { 0x5ae0397e, 95 },
    { 0x35e0397e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f03984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f03988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f0398b, 56 },
    { 0x2ad0398b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c0398f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5460399b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f00399f, 46 },
    { 0xec60399f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b039a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6039ae, 90 },
    { 0, 0 },
    { 0xafc039b0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19039b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87039bb, 90 },
    { 0xb78039bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3039c1, 76 },
    { 0xcb5039c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f039ce, 61 },
    { 0x35f039cf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0039d3, 81 },
    { 0, 0 },
    { 0xc76039d5, 98 },
    { 0x143039d6, 43 },
    { 0, 0 },
    { 0xec039d8, 90 },
    { 0, 0 },
    { 0x798039da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de039de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e039e1, 90 },
    { 0xd07039e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6039e6, 61 },
    { 0, 0 },
    { 0x379039e8, 90 },
    { 0x4c4039e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6039ee, 55 },
    { 0x414039ef, 33 },
    { 0xbc8039f0, 76 },
    { 0xf18039f1, 76 },
    { 0xdc2039f1, 55 },
    { 0x914039f2, 23 },
    { 0xca3039f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85039f7, 88 },
    { 0xae3039f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61039ff, 99 },
    { 0xcc603a00, 84 },
    { 0x2e703a01, 69 },
    { 0, 0 },
    { 0x40703a03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903a09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf03a0d, 23 },
    { 0x36f03a0e, 90 },
    { 0xb2003a0f, 95 },
    { 0xcd203a10, 91 },
    { 0x38e03a11, 76 },
    { 0xa4403a12, 65 },
    { 0x3ac03a13, 35 },
    { 0xe4f03a14, 40 },
    { 0x97703a0f, 95 },
    { 0, 0 },
    { 0x4803a17, 23 },
    { 0, 0 },
    { 0x68103a19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c03a20, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3503a23, 84 },
    { 0x3c803a24, 93 },
    { 0x70903a25, 23 },
    { 0, 0 },
    { 0xad603a27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c403a2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd03a30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f03a34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e303a37, 76 },
    { 0x1b503a38, 33 },
    { 0, 0 },
    { 0xc0a03a3a, 90 },
    { 0, 0 },
    { 0x75603a3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49203a3f, 98 },
    { 0xc3a03a40, 98 },
    { 0x1803a41, 80 },
    { 0, 0 },
    { 0xd3c03a43, 94 },
    { 0, 0 },
    { 0xfe003a45, 61 },
    { 0xc6903a46, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad03a4c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd03a4f, 81 },
    { 0, 0 },
    { 0x41103a51, 76 },
    { 0x8b603a52, 34 },
    { 0x7f703a53, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61703a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52703a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54003a61, 43 },
    { 0x66f03a62, 61 },
    { 0x4203a63, 87 },
    { 0x66503a62, 61 },
    { 0x34403a65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36903a69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x89803a6c, 23 },
    { 0x23b03a6d, 95 },
    { 0, 0 },
    { 0xd5003a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c503a73, 76 },
    { 0xb2503a74, 90 },
    { 0xfbb03a74, 34 },
    { 0x8e703a76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6603a7f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da03a85, 84 },
    { 0, 0 },
    { 0x3d503a87, 98 },
    { 0x78903a87, 43 },
    { 0x23003a89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57403a8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a03a90, 33 },
    { 0xdd03a90, 61 },
    { 0xa803a92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e103a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72603a9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff03aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23403aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc03aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a203ab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85403aba, 61 },
    { 0, 0 },
    { 0xd9903abc, 95 },
    { 0x8bf03abd, 90 },
    { 0x8d403abe, 93 },
    { 0, 0 },
    { 0x49e03ac0, 95 },
    { 0x24603ac1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d403ac8, 43 },
    { 0xe8e03ac9, 98 },
    { 0, 0 },
    { 0x1b603acb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f803ad0, 46 },
    { 0x30c03ad0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce03ad5, 90 },
    { 0xf6303ad6, 61 },
    { 0xe7803ad7, 33 },
    { 0x36603ad8, 87 },
    { 0x7ed03ad7, 34 },
    { 0x8e503ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a703add, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7503ae2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c03ae6, 88 },
    { 0xb1203ae7, 80 },
    { 0x26903ae6, 78 },
    { 0xe4803ae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d03aec, 98 },
    { 0x17e03aed, 33 },
    { 0, 0 },
    { 0xa5103aef, 90 },
    { 0xa7c03af0, 33 },
    { 0, 0 },
    { 0x24603af2, 33 },
    { 0xc8003af3, 95 },
    { 0x8bb03af4, 98 },
    { 0x63c03af5, 95 },
    { 0xc0b03af6, 61 },
    { 0xef103af6, 99 },
    { 0xe7003af8, 61 },
    { 0x59c03af9, 84 },
    { 0x79b03af9, 55 },
    { 0, 0 },
    { 0x7f503afc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30703b00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a03b03, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce603b0a, 91 },
    { 0xf9403b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f03b14, 99 },
    { 0xad503b14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c03b19, 98 },
    { 0, 0 },
    { 0xf8e03b1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b03b1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a03b21, 78 },
    { 0, 0 },
    { 0x8b803b23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f03b26, 61 },
    { 0, 0 },
    { 0xff03b28, 98 },
    { 0xf7d03b28, 98 },
    { 0x6d903b2a, 98 },
    { 0x72903b29, 34 },
    { 0, 0 },
    { 0x4af03b2d, 33 },
    { 0x1303b2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab03b31, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a703b36, 55 },
    { 0x85203b37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x31203b3a, 33 },
    { 0x39703b3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36403b40, 43 },
    { 0xcaa03b41, 23 },
    { 0xcbb03b41, 46 },
    { 0x5103b43, 83 },
    { 0xdd103b43, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52103b49, 61 },
    { 0x5c703b4a, 98 },
    { 0x64103b4a, 61 },
    { 0x7103b4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9903b50, 98 },
    { 0x13803b50, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f703b57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e03b5d, 90 },
    { 0, 0 },
    { 0xa8d03b5f, 90 },
    { 0xa0f03b60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56803b65, 61 },
    { 0xa5003b66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc803b6a, 90 },
    { 0x20903b6b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb703b6e, 76 },
    { 0xfc03b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e03b77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f03b7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c03b7d, 76 },
    { 0xdbb03b7e, 23 },
    { 0xaa203b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f203b83, 46 },
    { 0xbe503b84, 90 },
    { 0xbfe03b83, 33 },
    { 0x67903b86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc03b89, 98 },
    { 0, 0 },
    { 0xc2903b8b, 90 },
    { 0x1b603b8c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f03b91, 93 },
    { 0, 0 },
    { 0xfc203b93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb03b99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77503b9e, 77 },
    { 0, 0 },
    { 0x95503ba0, 76 },
    { 0x22b03ba1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c03ba5, 98 },
    { 0x71003ba6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8003bae, 99 },
    { 0xbec03baf, 46 },
    { 0x8b403bb0, 91 },
    { 0xee003bae, 23 },
    { 0x4f703bb0, 90 },
    { 0, 0 },
    { 0x52c03bb4, 34 },
    { 0xb6203bb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e03bbe, 95 },
    { 0xe2c03bbf, 90 },
    { 0x56c03bbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40303bc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae03bcc, 34 },
    { 0xf9403bcd, 61 },
    { 0, 0 },
    { 0x9e103bcf, 33 },
    { 0x3f403bd0, 61 },
    { 0xb0903bd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd003bd4, 55 },
    { 0x22a03bd5, 33 },
    { 0x98e03bd6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a603bda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b103be2, 81 },
    { 0xf2103be2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d303be7, 52 },
    { 0xc2d03be8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0303bef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b303bf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9703bfd, 55 },
    { 0x13503bfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40003c02, 90 },
    { 0x70e03c02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73203c0c, 33 },
    { 0x8503c0d, 43 },
    { 0x10603c0d, 43 },
    { 0xffe03c0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8803c18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84803c1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f03c22, 69 },
    { 0, 0 },
    { 0xa2403c24, 98 },
    { 0, 0 },
    { 0xfb203c26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac03c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e03c2c, 81 },
    { 0x87103c2c, 90 },
    { 0x26503c2e, 23 },
    { 0xf2103c2c, 75 },
    { 0, 0 },
    { 0x7d103c31, 33 },
    { 0x3fa03c32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c03c38, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d03c3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e03c40, 95 },
    { 0x6603c40, 76 },
    { 0x90c03c41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32603c48, 95 },
    { 0, 0 },
    { 0x12a03c4a, 55 },
    { 0x45403c4a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f603c4e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x12403c51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad03c54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c03c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4703c5b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x45203c5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7303c61, 85 },
    { 0x9cf03c61, 33 },
    { 0xd9d03c63, 43 },
    { 0x7503c63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c03c68, 95 },
    { 0x41703c69, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x26703c6c, 34 },
    { 0xb0103c6c, 95 },
    { 0, 0 },
    { 0x8fe03c6f, 93 },
    { 0x6d803c70, 33 },
    { 0xcd603c71, 43 },
    { 0x9bb03c72, 23 },
    { 0x28f03c71, 61 },
    { 0, 0 },
    { 0x9b603c75, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd03c7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1003c7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39503c83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e03c87, 34 },
    { 0x50703c88, 85 },
    { 0x33c03c87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1103c8e, 69 },
    { 0x31a03c8f, 75 },
    { 0, 0 },
    { 0x76303c91, 67 },
    { 0xd8d03c92, 61 },
    { 0x87f03c93, 52 },
    { 0, 0 },
    { 0x75503c95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e03c9c, 55 },
    { 0, 0 },
    { 0xd0403c9e, 23 },
    { 0x8db03c9e, 100 },
    { 0x94f03ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5803ca6, 91 },
    { 0x23703ca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e103cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d803cb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17103cb7, 33 },
    { 0, 0 },
    { 0x34b03cb9, 90 },
    { 0xeca03cba, 69 },
    { 0x9c203cb9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8503cbe, 93 },
    { 0x76903cbf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf803cc3, 91 },
    { 0, 0 },
    { 0xc6703cc5, 68 },
    { 0x21003cc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f903cca, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac903cce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa303cd5, 43 },
    { 0, 0 },
    { 0xc0403cd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32703cda, 100 },
    { 0xc1603cda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7603cdf, 33 },
    { 0, 0 },
    { 0xcf503ce1, 33 },
    { 0xdf603ce2, 99 },
    { 0x44903ce3, 90 },
    { 0xd8d03ce1, 98 },
    { 0x54303ce4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e503ce9, 46 },
    { 0x74603cea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7703cef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca303cf3, 98 },
    { 0, 0 },
    { 0x31803cf5, 43 },
    { 0x57c03cf6, 23 },
    { 0xcfc03cf6, 98 },
    { 0xc4303cf5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff03d00, 96 },
    { 0xbc403d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70203d07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca03d0b, 90 },
    { 0x1c103d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d03d0f, 61 },
    { 0xd8a03d10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98003d13, 95 },
    { 0xc2203d14, 90 },
    { 0xca03d15, 91 },
    { 0x29d03d16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57403d1a, 90 },
    { 0, 0 },
    { 0x1e203d1c, 34 },
    { 0xe2103d1d, 61 },
    { 0x6b003d1d, 61 },
    { 0x77403d1f, 83 },
    { 0xe4703d20, 23 },
    { 0x4c703d1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a03d28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b03d2b, 99 },
    { 0x2a703d2c, 95 },
    { 0, 0 },
    { 0x1a803d2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef903d34, 85 },
    { 0, 0 },
    { 0xd5403d36, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd03d39, 93 },
    { 0, 0 },
    { 0xb2c03d3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43203d41, 95 },
    { 0x3b803d42, 90 },
    { 0xdc503d43, 99 },
    { 0xa0b03d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94803d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88803d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca503d59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1603d5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b03d5f, 46 },
    { 0x22203d5f, 61 },
    { 0x21503d60, 89 },
    { 0, 0 },
    { 0x30d03d63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e03d66, 90 },
    { 0xb3203d67, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb903d70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6203d74, 23 },
    { 0x79d03d75, 98 },
    { 0x29903d75, 69 },
    { 0, 0 },
    { 0x15103d78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef03d7c, 43 },
    { 0xe7703d7d, 95 },
    { 0x1b603d7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5003d86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f03d8c, 75 },
    { 0, 0 },
    { 0x77903d8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c03d91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b03d94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d03d97, 34 },
    { 0, 0 },
    { 0xeb103d99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca03d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2403da1, 55 },
    { 0x20403da2, 98 },
    { 0x10c03da3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff03dab, 46 },
    { 0xe1803dab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa103db6, 90 },
    { 0, 0 },
    { 0x65203db8, 98 },
    { 0x66103db9, 98 },
    { 0xfbd03db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4303dbf, 33 },
    { 0x76f03dc0, 46 },
    { 0x96f03dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b03dc5, 23 },
    { 0xf5703dc6, 33 },
    { 0x5e003dc7, 46 },
    { 0x2bc03dc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da03dcb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c103dd4, 87 },
    { 0xdc003dd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a03dd8, 90 },
    { 0x35203dd8, 75 },
    { 0, 0 },
    { 0x31103ddb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6303de4, 93 },
    { 0, 0 },
    { 0xe4403de6, 33 },
    { 0x56303de7, 99 },
    { 0, 0 },
    { 0xa7b03de9, 46 },
    { 0x10403de9, 78 },
    { 0xe3503dea, 33 },
    { 0xa4d03deb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe903df3, 33 },
    { 0xe603df3, 43 },
    { 0, 0 },
    { 0xada03df6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e003dfa, 76 },
    { 0x7c903dfa, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0703dff, 61 },
    { 0x43903e00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a03e03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68403e07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1603e0a, 45 },
    { 0x3fe03e0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2603e0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f403e12, 91 },
    { 0, 0 },
    { 0x6a203e14, 100 },
    { 0, 0 },
    { 0xba703e16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf03e1c, 98 },
    { 0, 0 },
    { 0x70103e1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81103e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec703e2d, 33 },
    { 0, 0 },
    { 0x9df03e2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2303e36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d203e3a, 87 },
    { 0xa9d03e3a, 87 },
    { 0, 0 },
    { 0xf5f03e3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8803e42, 95 },
    { 0x10003e43, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb003e48, 90 },
    { 0, 0 },
    { 0x12b03e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee03e4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf903e51, 66 },
    { 0x9b403e52, 100 },
    { 0x64a03e53, 52 },
    { 0xf0d03e52, 98 },
    { 0xe8703e52, 61 },
    { 0xe3503e54, 55 },
    { 0xdb203e57, 46 },
    { 0x16403e58, 23 },
    { 0xbf703e58, 34 },
    { 0xbfd03e5a, 76 },
    { 0xa4b03e57, 81 },
    { 0, 0 },
    { 0x90a03e5d, 33 },
    { 0xaf203e5e, 90 },
    { 0x54b03e5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b03e65, 33 },
    { 0x8f003e66, 66 },
    { 0x65c03e65, 33 },
    { 0x16103e68, 90 },
    { 0x53503e66, 69 },
    { 0x60903e6a, 52 },
    { 0x73303e67, 95 },
    { 0xf8103e65, 61 },
    { 0xe2603e6c, 61 },
    { 0x31203e66, 61 },
    { 0x34003e6f, 97 },
    { 0x26b03e70, 34 },
    { 0x6a703e71, 34 },
    { 0x4b403e71, 95 },
    { 0x69303e6c, 77 },
    { 0xe4903e74, 46 },
    { 0, 0 },
    { 0x99003e76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8503e7a, 61 },
    { 0, 0 },
    { 0xcba03e7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c03e87, 45 },
    { 0, 0 },
    { 0xc6303e89, 74 },
    { 0xdca03e8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d03e8e, 34 },
    { 0, 0 },
    { 0xa6b03e90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3203e97, 91 },
    { 0x6b03e97, 88 },
    { 0x54903e98, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b03e9d, 8 },
    { 0x8cb03e9e, 61 },
    { 0, 0 },
    { 0x4603ea0, 17 },
    { 0, 0 },
    { 0x29003ea2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc903ea5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b03eaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4203ead, 34 },
    { 0x7f903eae, 43 },
    { 0, 0 },
    { 0x54703eb0, 84 },
    { 0, 0 },
    { 0xb0203eb2, 61 },
    { 0x1b303eb2, 66 },
    { 0, 0 },
    { 0x4f003eb5, 77 },
    { 0xeb903eb5, 95 },
    { 0x86a03eb7, 34 },
    { 0x31b03eb7, 78 },
    { 0x69103eb9, 34 },
    { 0x1a603eb8, 95 },
    { 0xf7303ebb, 33 },
    { 0xa7a03eb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65003ec3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd903ed2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b03edb, 95 },
    { 0x55d03edc, 23 },
    { 0x92e03edd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad03ee1, 93 },
    { 0, 0 },
    { 0x7ee03ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b03ee9, 84 },
    { 0xe4203ee9, 91 },
    { 0xb5803eeb, 95 },
    { 0xd9d03eec, 33 },
    { 0x86f03eed, 34 },
    { 0x1b03eec, 100 },
    { 0x9d603eef, 76 },
    { 0x77d03eee, 98 },
    { 0xb5503eec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe203ef5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e003ef8, 99 },
    { 0x12403ef9, 88 },
    { 0xc4103efa, 23 },
    { 0xcd603efa, 23 },
    { 0x5f403efc, 98 },
    { 0xe0003efc, 90 },
    { 0x70f03efc, 90 },
    { 0x5f903efe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de03f07, 43 },
    { 0x5ed03f08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f03f11, 52 },
    { 0xf2203f12, 33 },
    { 0x32403f13, 55 },
    { 0x3f13, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce03f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c03f22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba03f25, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da03f28, 90 },
    { 0xe1903f29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0803f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb03f34, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a603f37, 90 },
    { 0, 0 },
    { 0xf0403f39, 33 },
    { 0xfeb03f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea903f41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab03f45, 61 },
    { 0x67903f46, 91 },
    { 0xb2c03f46, 33 },
    { 0, 0 },
    { 0x75603f49, 33 },
    { 0xb7c03f4a, 23 },
    { 0xdb203f49, 27 },
    { 0xc5803f4c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e303f4f, 33 },
    { 0x5cb03f50, 43 },
    { 0xd9803f50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa903f55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed03f58, 84 },
    { 0xecb03f59, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f03f62, 95 },
    { 0, 0 },
    { 0x4d003f64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b803f68, 66 },
    { 0x58803f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55903f6c, 88 },
    { 0xc3403f6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b03f72, 76 },
    { 0xe7f03f73, 90 },
    { 0x1b603f74, 90 },
    { 0, 0 },
    { 0xa4703f76, 90 },
    { 0xae103f77, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa03f7d, 23 },
    { 0x68b03f7e, 85 },
    { 0x4d003f7f, 90 },
    { 0xc3b03f80, 67 },
    { 0xbc003f81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7803f84, 23 },
    { 0x54903f85, 55 },
    { 0x42b03f86, 43 },
    { 0xff003f85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1003f8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0803f8d, 67 },
    { 0, 0 },
    { 0x62903f8f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c803f92, 99 },
    { 0, 0 },
    { 0xd8803f94, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f703f99, 90 },
    { 0, 0 },
    { 0x57003f9b, 61 },
    { 0, 0 },
    { 0xa2303f9d, 95 },
    { 0xd6403f9e, 98 },
    { 0x50f03f9d, 99 },
    { 0x14303fa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9303fa4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8903fab, 33 },
    { 0xc7703fab, 43 },
    { 0x97803fac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x54403fb0, 98 },
    { 0x82703fb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc03fbb, 33 },
    { 0, 0 },
    { 0xa4e03fbd, 61 },
    { 0x3cb03fbe, 90 },
    { 0x9b403fbe, 33 },
    { 0x69f03fc0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef03fcb, 61 },
    { 0xd0d03fcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f03fd0, 76 },
    { 0xad103fd0, 90 },
    { 0, 0 },
    { 0xdf103fd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e03fdf, 98 },
    { 0x48a03fe0, 87 },
    { 0x18c03fe1, 93 },
    { 0x3c603fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac503fe7, 61 },
    { 0x4b203fe8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a03feb, 90 },
    { 0, 0 },
    { 0x3a703fed, 84 },
    { 0x8ae03fee, 95 },
    { 0xae403fee, 90 },
    { 0xcb603fed, 95 },
    { 0xe0203fef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27103ff8, 95 },
    { 0xec303ff9, 33 },
    { 0, 0 },
    { 0x64103ffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89103fff, 98 },
    { 0, 0 },
    { 0x31904001, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3504008, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd310400d, 23 },
    { 0, 0 },
    { 0x8d20400f, 33 },
    { 0, 0 },
    { 0x27b04011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c704015, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c04018, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd90401f, 90 },
    { 0xd1804020, 61 },
    { 0x6ac04021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe904024, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50304029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa730402e, 33 },
    { 0x1df0402f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf104032, 55 },
    { 0xa4004033, 61 },
    { 0xed704034, 46 },
    { 0, 0 },
    { 0x57304036, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69704039, 40 },
    { 0x1850403a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd0403e, 61 },
    { 0, 0 },
    { 0xc3b04040, 80 },
    { 0x1fc04041, 33 },
    { 0x9f604041, 90 },
    { 0xf4204043, 90 },
    { 0xf0904044, 91 },
    { 0, 0 },
    { 0xfa204046, 55 },
    { 0xd1304047, 95 },
    { 0xd9004048, 23 },
    { 0x95404047, 84 },
    { 0xe604048, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8010404f, 98 },
    { 0, 0 },
    { 0xb004051, 81 },
    { 0xeff04052, 76 },
    { 0, 0 },
    { 0x2f104054, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f20405d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2304068, 33 },
    { 0x54504069, 73 },
    { 0xcbc04069, 90 },
    { 0x1dc04069, 95 },
    { 0x1fe04068, 76 },
    { 0x4900406a, 76 },
    { 0x7d70406e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59004071, 90 },
    { 0xb1e04071, 95 },
    { 0, 0 },
    { 0x11304074, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd150407a, 88 },
    { 0, 0 },
    { 0xacb0407c, 24 },
    { 0x7f0407d, 53 },
    { 0x6d50407c, 90 },
    { 0xa460407f, 90 },
    { 0xa5104080, 90 },
    { 0xf0804080, 95 },
    { 0xbf104082, 52 },
    { 0x60504081, 78 },
    { 0x1f20407f, 76 },
    { 0x65204085, 98 },
    { 0x6e04086, 55 },
    { 0xbef04087, 46 },
    { 0x34104088, 95 },
    { 0x19004088, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf020408d, 61 },
    { 0, 0 },
    { 0x7d60408f, 33 },
    { 0, 0 },
    { 0xa5604091, 91 },
    { 0, 0 },
    { 0x92804093, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d04097, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0409a, 99 },
    { 0x32b0409b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb040a5, 91 },
    { 0xb7d040a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x467040a9, 65 },
    { 0x5f8040aa, 33 },
    { 0xda9040ab, 33 },
    { 0x6d6040ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec040af, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38040b4, 61 },
    { 0x695040b4, 61 },
    { 0xba8040b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c040b9, 53 },
    { 0xc39040b9, 55 },
    { 0, 0 },
    { 0xd4a040bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b040c2, 90 },
    { 0, 0 },
    { 0x754040c4, 101 },
    { 0x452040c5, 98 },
    { 0x4db040c6, 33 },
    { 0x137040c4, 95 },
    { 0x149040c7, 61 },
    { 0x5b5040c9, 33 },
    { 0xafa040ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489040d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3040d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x166040d7, 34 },
    { 0x3ce040d7, 75 },
    { 0, 0 },
    { 0x252040da, 33 },
    { 0xefb040db, 33 },
    { 0x90d040db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58040e1, 61 },
    { 0x950040e2, 93 },
    { 0xb17040e3, 66 },
    { 0x8fb040e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf040e7, 55 },
    { 0x9c8040e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227040ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891040f0, 61 },
    { 0, 0 },
    { 0x6ab040f2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f040fc, 66 },
    { 0x42d040fd, 23 },
    { 0xba9040fe, 78 },
    { 0x58b040fe, 33 },
    { 0, 0 },
    { 0x17404101, 17 },
    { 0xf0304102, 91 },
    { 0xf5104102, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf804106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27504109, 95 },
    { 0xc750410a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e80410e, 87 },
    { 0xf350410f, 93 },
    { 0, 0 },
    { 0x4f704111, 78 },
    { 0xde904112, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f04118, 84 },
    { 0x23704119, 98 },
    { 0xba304118, 90 },
    { 0x2f20411b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb04120, 33 },
    { 0x3d004121, 75 },
    { 0x8c104122, 23 },
    { 0xe0904122, 95 },
    { 0x14e04123, 76 },
    { 0, 0 },
    { 0xce404126, 66 },
    { 0x55804127, 90 },
    { 0x66104127, 33 },
    { 0, 0 },
    { 0x9440412a, 61 },
    { 0x9d10412b, 96 },
    { 0x4760412c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc504130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82304133, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9904136, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c04139, 33 },
    { 0xa130413a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf0413d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f604141, 77 },
    { 0x56004141, 64 },
    { 0xa004141, 90 },
    { 0x2c504141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc0414a, 90 },
    { 0, 0 },
    { 0xa2f0414c, 85 },
    { 0x55a0414d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf04153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb104157, 99 },
    { 0x34204157, 81 },
    { 0, 0 },
    { 0xaaf0415a, 85 },
    { 0, 0 },
    { 0xbcf0415c, 90 },
    { 0, 0 },
    { 0xe130415e, 99 },
    { 0xdcd0415e, 33 },
    { 0x4380415f, 90 },
    { 0, 0 },
    { 0x25f04162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f304165, 85 },
    { 0xcb304166, 95 },
    { 0xdaa04167, 90 },
    { 0x69104165, 100 },
    { 0x94a04167, 43 },
    { 0xb7f0416a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf890417b, 55 },
    { 0, 0 },
    { 0xe0417d, 76 },
    { 0, 0 },
    { 0x73a0417f, 100 },
    { 0x29404180, 91 },
    { 0xc2b0417f, 61 },
    { 0x2a30417f, 61 },
    { 0x82504183, 98 },
    { 0xfa504180, 33 },
    { 0xcba04185, 33 },
    { 0, 0 },
    { 0xa2004187, 34 },
    { 0, 0 },
    { 0x3e304189, 23 },
    { 0x8ab0418a, 84 },
    { 0, 0 },
    { 0x8090418c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43004192, 66 },
    { 0x49204193, 68 },
    { 0x93604193, 34 },
    { 0xc5a04192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e04199, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5390419c, 90 },
    { 0x51d0419d, 98 },
    { 0, 0 },
    { 0x2b60419f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70041a2, 23 },
    { 0x55041a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8041a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b041af, 76 },
    { 0, 0 },
    { 0x717041b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45041b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d041b9, 61 },
    { 0xc93041ba, 90 },
    { 0, 0 },
    { 0x8cd041bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8041c0, 55 },
    { 0, 0 },
    { 0x11041c2, 87 },
    { 0xab2041c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62041c6, 100 },
    { 0x6dc041c7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb041ce, 68 },
    { 0xb6e041cf, 61 },
    { 0, 0 },
    { 0x82f041d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7041d5, 61 },
    { 0xcb7041d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa041e0, 33 },
    { 0xdf9041e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d041e4, 33 },
    { 0x20f041e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631041ee, 61 },
    { 0, 0 },
    { 0x6c3041f0, 84 },
    { 0x985041f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640041fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b504203, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe04208, 76 },
    { 0xb3804209, 33 },
    { 0, 0 },
    { 0x14c0420b, 98 },
    { 0, 0 },
    { 0x6790420d, 17 },
    { 0x9d70420e, 77 },
    { 0, 0 },
    { 0x5ff04210, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e204217, 90 },
    { 0x13504218, 33 },
    { 0, 0 },
    { 0xcc20421a, 33 },
    { 0, 0 },
    { 0x9740421c, 23 },
    { 0xea30421d, 52 },
    { 0x8050421e, 100 },
    { 0, 0 },
    { 0x39804220, 95 },
    { 0, 0 },
    { 0x91f04222, 89 },
    { 0, 0 },
    { 0x22b04224, 95 },
    { 0x8cb04225, 23 },
    { 0, 0 },
    { 0x67d04227, 88 },
    { 0x14d04228, 95 },
    { 0, 0 },
    { 0x5150422a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6004233, 34 },
    { 0x4e404234, 93 },
    { 0, 0 },
    { 0x60a04236, 90 },
    { 0x8f604237, 95 },
    { 0xe2304236, 76 },
    { 0x22404237, 55 },
    { 0x3d70423a, 90 },
    { 0x3470423a, 61 },
    { 0x83e04238, 24 },
    { 0, 0 },
    { 0x9020423e, 88 },
    { 0x1c40423e, 90 },
    { 0, 0 },
    { 0x58d04241, 43 },
    { 0x4bf04241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45104245, 90 },
    { 0x8e904245, 98 },
    { 0, 0 },
    { 0x42c04248, 90 },
    { 0x16904249, 76 },
    { 0x35c04249, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd20424f, 98 },
    { 0, 0 },
    { 0xf504251, 52 },
    { 0, 0 },
    { 0x35204253, 61 },
    { 0x61f04254, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca0425b, 76 },
    { 0x75a0425c, 98 },
    { 0xbb10425c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc04260, 98 },
    { 0, 0 },
    { 0xd1004262, 43 },
    { 0, 0 },
    { 0xf3304264, 23 },
    { 0xbfc04265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1810426d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26904272, 62 },
    { 0x32104273, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de04278, 17 },
    { 0x75d04279, 96 },
    { 0x18d0427a, 55 },
    { 0x47c0427b, 61 },
    { 0xfd70427a, 90 },
    { 0, 0 },
    { 0xbea0427e, 91 },
    { 0xd2e0427e, 98 },
    { 0x5bd0427e, 33 },
    { 0xb800427f, 90 },
    { 0x89a0427e, 95 },
    { 0xc8004283, 66 },
    { 0x78b0427f, 75 },
    { 0x10504280, 90 },
    { 0x8ba04286, 23 },
    { 0, 0 },
    { 0xe7804288, 52 },
    { 0x69a04289, 61 },
    { 0x4d80428a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3950428d, 90 },
    { 0xf620428d, 61 },
    { 0, 0 },
    { 0x68304290, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2404295, 100 },
    { 0x92204295, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db04299, 34 },
    { 0x3ab0429a, 66 },
    { 0, 0 },
    { 0xdf70429c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb042a3, 76 },
    { 0, 0 },
    { 0x339042a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e042ac, 95 },
    { 0xc7d042ad, 93 },
    { 0x79d042ad, 61 },
    { 0xc75042ac, 66 },
    { 0, 0 },
    { 0xbcf042b1, 46 },
    { 0, 0 },
    { 0x24a042b3, 17 },
    { 0xf0b042b4, 33 },
    { 0xe27042b3, 95 },
    { 0, 0 },
    { 0xade042b7, 90 },
    { 0xe6e042b8, 46 },
    { 0, 0 },
    { 0x928042ba, 34 },
    { 0xb60042ba, 99 },
    { 0, 0 },
    { 0x655042bd, 98 },
    { 0x6be042bd, 17 },
    { 0, 0 },
    { 0x4c6042c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785042c5, 81 },
    { 0x52b042c5, 76 },
    { 0xf45042c7, 84 },
    { 0x12c042c8, 98 },
    { 0x1ec042c7, 99 },
    { 0, 0 },
    { 0x2df042cb, 34 },
    { 0xfb6042cb, 55 },
    { 0xf75042cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d042d2, 90 },
    { 0x782042d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f042d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8042e0, 61 },
    { 0x6fa042e0, 98 },
    { 0xd22042e2, 95 },
    { 0x166042e2, 55 },
    { 0xcfe042e4, 17 },
    { 0x1a6042e5, 33 },
    { 0x959042e4, 76 },
    { 0x59c042e7, 90 },
    { 0xe3c042e8, 33 },
    { 0x7b2042e5, 61 },
    { 0x462042e4, 55 },
    { 0, 0 },
    { 0xb96042ec, 68 },
    { 0, 0 },
    { 0xf47042ee, 52 },
    { 0xa95042ef, 91 },
    { 0x2a0042ef, 23 },
    { 0x733042f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f042f7, 23 },
    { 0, 0 },
    { 0xafe042f9, 61 },
    { 0x5b5042fa, 80 },
    { 0x1c3042f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8042ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb04304, 52 },
    { 0xfc604304, 61 },
    { 0xe0504306, 84 },
    { 0x48b04304, 33 },
    { 0xf7d04306, 46 },
    { 0xa4904309, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540430c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb04311, 100 },
    { 0xdb104312, 90 },
    { 0xadb04313, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda40431d, 66 },
    { 0x3250431e, 23 },
    { 0xca90431f, 23 },
    { 0, 0 },
    { 0x4a704321, 34 },
    { 0, 0 },
    { 0x2e404323, 95 },
    { 0xba404323, 23 },
    { 0, 0 },
    { 0x81804326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290432a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90804330, 33 },
    { 0xc6504331, 56 },
    { 0xe3a04331, 68 },
    { 0xbdb04331, 33 },
    { 0x1e404334, 61 },
    { 0x44704334, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1004338, 61 },
    { 0x1a04339, 95 },
    { 0xeb304338, 96 },
    { 0x24c04339, 78 },
    { 0x91a0433c, 61 },
    { 0xaba0433d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7504343, 17 },
    { 0xac704344, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5304347, 91 },
    { 0xa8d04348, 90 },
    { 0x9bb04348, 76 },
    { 0, 0 },
    { 0x4c40434b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a04352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac60435e, 23 },
    { 0x7580435f, 23 },
    { 0x53d0435e, 99 },
    { 0xfbf04361, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a04366, 40 },
    { 0, 0 },
    { 0x25604368, 33 },
    { 0xe8f04369, 23 },
    { 0x42304368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba20436d, 34 },
    { 0xba40436e, 76 },
    { 0xe150436f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3904373, 65 },
    { 0, 0 },
    { 0x26e04375, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7900437b, 33 },
    { 0x570437c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b104387, 95 },
    { 0x58604388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4030438e, 61 },
    { 0xacd0438f, 95 },
    { 0xbf004390, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b04394, 95 },
    { 0xfca04395, 46 },
    { 0, 0 },
    { 0xb2404397, 98 },
    { 0xe0c04398, 75 },
    { 0xa9904399, 91 },
    { 0x62d04399, 32 },
    { 0x3b40439b, 52 },
    { 0, 0 },
    { 0xf2d0439d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1043a2, 90 },
    { 0x8c2043a3, 61 },
    { 0, 0 },
    { 0x7b1043a5, 59 },
    { 0xaf6043a5, 65 },
    { 0xacb043a7, 61 },
    { 0, 0 },
    { 0x244043a9, 43 },
    { 0x4b6043aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a043af, 98 },
    { 0x68b043b0, 75 },
    { 0xeee043b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92043b6, 23 },
    { 0x78a043b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26043be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860043c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce043cf, 76 },
    { 0, 0 },
    { 0x9c8043d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4043d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80043da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2043df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914043e3, 90 },
    { 0xbb043e4, 99 },
    { 0xc38043e3, 90 },
    { 0xd16043e4, 76 },
    { 0x7c8043e6, 85 },
    { 0, 0 },
    { 0xc8a043e9, 33 },
    { 0xdf7043e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512043ef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f043f4, 17 },
    { 0x781043f5, 98 },
    { 0, 0 },
    { 0x109043f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2043fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51104403, 40 },
    { 0xece04404, 99 },
    { 0xd9204405, 98 },
    { 0xab204406, 61 },
    { 0x30304403, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c70440a, 91 },
    { 0x4a90440a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba04414, 76 },
    { 0xf8904415, 61 },
    { 0x9fe04415, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2750441b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc04422, 33 },
    { 0x34904423, 61 },
    { 0xdab04424, 90 },
    { 0x31404422, 90 },
    { 0, 0 },
    { 0xe6404427, 61 },
    { 0x7d404427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8770442b, 98 },
    { 0x4400442c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57204432, 98 },
    { 0x52b04433, 61 },
    { 0x4af04434, 66 },
    { 0xf9304432, 90 },
    { 0x41d04432, 90 },
    { 0x75504437, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a0443a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa020443d, 61 },
    { 0xe2e0443d, 95 },
    { 0x7e20443f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f04445, 23 },
    { 0xc5c04446, 23 },
    { 0x27304447, 81 },
    { 0x5ee04448, 46 },
    { 0xb3d04449, 66 },
    { 0xc900444a, 90 },
    { 0x5e504447, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b804458, 46 },
    { 0x90d04459, 33 },
    { 0x9150445a, 33 },
    { 0, 0 },
    { 0xc560445c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d04464, 90 },
    { 0x29504465, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6004468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba40446c, 46 },
    { 0x6fa0446d, 87 },
    { 0, 0 },
    { 0xbdf0446f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39204474, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c404477, 76 },
    { 0, 0 },
    { 0xce204479, 90 },
    { 0xa390447a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a0447e, 90 },
    { 0x79a0447f, 95 },
    { 0x59604480, 93 },
    { 0xd5c04481, 61 },
    { 0x3a004482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5504487, 93 },
    { 0, 0 },
    { 0x32904489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e0448c, 90 },
    { 0, 0 },
    { 0x7940448e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e04491, 33 },
    { 0xc1e04492, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16504496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0449c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40449f, 84 },
    { 0xc2a0449f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6044a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0044a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389044ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x933044af, 46 },
    { 0xc51044b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8044b3, 77 },
    { 0, 0 },
    { 0x43f044b5, 33 },
    { 0, 0 },
    { 0xff7044b7, 95 },
    { 0xe74044b7, 98 },
    { 0xe58044b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47044bc, 90 },
    { 0xc38044bc, 33 },
    { 0, 0 },
    { 0x3d6044bf, 61 },
    { 0, 0 },
    { 0x387044c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24044c4, 43 },
    { 0x682044c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310044cc, 36 },
    { 0x93e044cc, 36 },
    { 0x289044ce, 23 },
    { 0, 0 },
    { 0xe46044d0, 75 },
    { 0, 0 },
    { 0x4c1044d2, 61 },
    { 0, 0 },
    { 0xb2e044d4, 23 },
    { 0x499044d5, 90 },
    { 0xc6d044d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b044dd, 55 },
    { 0x415044de, 90 },
    { 0x359044dd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1044e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2044e6, 95 },
    { 0x7f044e7, 33 },
    { 0xdc9044e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6044ee, 61 },
    { 0x8c7044ef, 23 },
    { 0x2e2044ee, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68044f4, 95 },
    { 0xfbe044f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69044fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b404507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4900450c, 33 },
    { 0, 0 },
    { 0x8000450e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43204511, 81 },
    { 0x17504511, 90 },
    { 0xd5704511, 34 },
    { 0x97604514, 61 },
    { 0xa4f04515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c00451f, 33 },
    { 0, 0 },
    { 0x35604521, 77 },
    { 0, 0 },
    { 0x604523, 100 },
    { 0xf5d04523, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c0452b, 97 },
    { 0x1440452c, 46 },
    { 0x1c0452c, 78 },
    { 0, 0 },
    { 0x2b20452f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c04532, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f0453a, 76 },
    { 0, 0 },
    { 0xfaa0453c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35604543, 46 },
    { 0xddc04544, 87 },
    { 0x39804545, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a10454c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57904550, 98 },
    { 0x82e04551, 95 },
    { 0x72304552, 76 },
    { 0x46c04551, 90 },
    { 0x51404551, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460455b, 90 },
    { 0, 0 },
    { 0x4ce0455d, 95 },
    { 0x1640455e, 40 },
    { 0, 0 },
    { 0xfe104560, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96404563, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81104566, 81 },
    { 0xf5b04567, 77 },
    { 0xcd404568, 66 },
    { 0xbc204568, 90 },
    { 0, 0 },
    { 0xec90456b, 61 },
    { 0x7370456c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff0456f, 90 },
    { 0x68d04570, 61 },
    { 0xdfb04571, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9704575, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c04578, 84 },
    { 0, 0 },
    { 0x28d0457a, 84 },
    { 0, 0 },
    { 0x1750457c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a04584, 43 },
    { 0xc9a04584, 61 },
    { 0x41004584, 76 },
    { 0xbc304586, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc00458a, 61 },
    { 0xf3a0458b, 53 },
    { 0xd0e0458c, 61 },
    { 0x3820458c, 46 },
    { 0x8f70458d, 61 },
    { 0x8380458d, 23 },
    { 0x7d30458c, 23 },
    { 0xa8404591, 98 },
    { 0x4540458c, 98 },
    { 0x2d30458e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0704597, 76 },
    { 0x92204598, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80459c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f0459f, 61 },
    { 0x8d7045a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a045a4, 67 },
    { 0xf8f045a5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66045a9, 33 },
    { 0, 0 },
    { 0x17b045ab, 43 },
    { 0, 0 },
    { 0x950045ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1045b1, 93 },
    { 0x924045b1, 33 },
    { 0x3e2045b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1045bc, 48 },
    { 0, 0 },
    { 0x3e045be, 33 },
    { 0, 0 },
    { 0xc8045c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120045c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6045ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386045d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9045d3, 61 },
    { 0, 0 },
    { 0x269045d5, 97 },
    { 0x975045d6, 90 },
    { 0x22a045d7, 91 },
    { 0, 0 },
    { 0xeda045d9, 91 },
    { 0xcfd045da, 33 },
    { 0x468045da, 93 },
    { 0, 0 },
    { 0xb7e045dd, 43 },
    { 0x688045dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3045e5, 23 },
    { 0x7b0045e6, 95 },
    { 0, 0 },
    { 0x71c045e8, 76 },
    { 0xaec045e8, 61 },
    { 0xbef045ea, 93 },
    { 0x87c045ea, 55 },
    { 0xd9a045eb, 23 },
    { 0, 0 },
    { 0x854045ee, 95 },
    { 0xdfd045ef, 43 },
    { 0xf6d045f0, 61 },
    { 0xe10045ef, 23 },
    { 0, 0 },
    { 0xb60045f3, 33 },
    { 0xf9e045f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x315045f7, 61 },
    { 0x32b045f8, 23 },
    { 0x2e5045f9, 46 },
    { 0x37e045f9, 90 },
    { 0xad7045fb, 87 },
    { 0x1fd045fc, 52 },
    { 0xcee045fc, 66 },
    { 0xe2b045fc, 90 },
    { 0, 0 },
    { 0x94304600, 95 },
    { 0, 0 },
    { 0xd504602, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc904605, 34 },
    { 0, 0 },
    { 0x11704607, 90 },
    { 0xa2e04608, 98 },
    { 0xe2604609, 36 },
    { 0x12804607, 33 },
    { 0x6e0460b, 33 },
    { 0, 0 },
    { 0x4420460d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca104610, 90 },
    { 0xb3e04611, 55 },
    { 0x53b04612, 61 },
    { 0, 0 },
    { 0x86204614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76904617, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f20461d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c04620, 87 },
    { 0x7404621, 61 },
    { 0xa8804620, 55 },
    { 0x40804623, 66 },
    { 0, 0 },
    { 0xfc04625, 23 },
    { 0xe7c04625, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d0462b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe880462f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46504636, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c0463b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6860463e, 91 },
    { 0x7040463f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1604644, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0704648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a0464b, 90 },
    { 0x84c0464b, 61 },
    { 0xf890464d, 55 },
    { 0, 0 },
    { 0x3840464f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7504656, 90 },
    { 0, 0 },
    { 0x1d304658, 34 },
    { 0, 0 },
    { 0x6280465a, 96 },
    { 0x5480465b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c0465e, 33 },
    { 0x1240465e, 100 },
    { 0xc2b04660, 52 },
    { 0, 0 },
    { 0x8da04662, 31 },
    { 0xa1d04663, 61 },
    { 0, 0 },
    { 0x22904665, 92 },
    { 0x1ac04665, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4504669, 61 },
    { 0x19304669, 61 },
    { 0xff40466a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xef50466e, 76 },
    { 0x3900466f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff04673, 82 },
    { 0xd6e04674, 33 },
    { 0xd7304674, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8304678, 23 },
    { 0x38604679, 46 },
    { 0x2904679, 61 },
    { 0xe90467b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db04683, 33 },
    { 0xbe504683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb0468a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b0468e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a04695, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb04698, 23 },
    { 0x45204699, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09046a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19046a6, 23 },
    { 0, 0 },
    { 0xb2046a8, 75 },
    { 0x495046a9, 27 },
    { 0xf21046aa, 63 },
    { 0x272046ab, 33 },
    { 0x7c9046aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8046b0, 65 },
    { 0x77f046b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17046b5, 61 },
    { 0xb02046b6, 38 },
    { 0x394046b5, 95 },
    { 0xe60046b8, 93 },
    { 0, 0 },
    { 0x8d8046ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x316046be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8046c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531046c6, 67 },
    { 0x683046c7, 90 },
    { 0xa5d046c8, 76 },
    { 0x566046c9, 33 },
    { 0x96d046c9, 90 },
    { 0x314046c6, 87 },
    { 0xa3d046cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e046d5, 61 },
    { 0x88b046d6, 91 },
    { 0x968046d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34046dc, 61 },
    { 0, 0 },
    { 0x4f046de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f046e4, 93 },
    { 0x394046e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc046e9, 33 },
    { 0x528046ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d046ee, 76 },
    { 0x182046ef, 23 },
    { 0x9b5046f0, 61 },
    { 0xda0046ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b046f7, 23 },
    { 0, 0 },
    { 0x4a046f9, 66 },
    { 0x349046fa, 98 },
    { 0x4f9046fb, 46 },
    { 0, 0 },
    { 0xfcb046fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d104708, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de0470c, 34 },
    { 0x35e0470c, 95 },
    { 0x57c0470e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5804715, 98 },
    { 0xa6704716, 55 },
    { 0x2304717, 81 },
    { 0x76904718, 76 },
    { 0xa4504719, 90 },
    { 0x5ee04717, 100 },
    { 0x85604716, 55 },
    { 0, 0 },
    { 0x6410471d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55804720, 66 },
    { 0x22e04721, 72 },
    { 0x40a04721, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5104725, 88 },
    { 0, 0 },
    { 0xc3a04727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc430472b, 98 },
    { 0x4d90472c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5020472f, 90 },
    { 0x86f04730, 55 },
    { 0x33b04730, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc04737, 98 },
    { 0x1c904738, 98 },
    { 0x2cd04739, 34 },
    { 0x43c04737, 95 },
    { 0, 0 },
    { 0xa9a0473c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b04740, 90 },
    { 0x8bf04740, 99 },
    { 0xebc04741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89304746, 98 },
    { 0xd5004747, 55 },
    { 0xde004747, 90 },
    { 0xf3204749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe750474f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9004753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf04757, 61 },
    { 0, 0 },
    { 0xf9804759, 90 },
    { 0x67404759, 99 },
    { 0xbd30475b, 76 },
    { 0x6e00475c, 33 },
    { 0x1600475a, 55 },
    { 0xbe30475e, 66 },
    { 0x42b0475e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d04767, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe030476a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a0476e, 95 },
    { 0x3ee0476e, 93 },
    { 0, 0 },
    { 0x1bb04771, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x78504774, 98 },
    { 0xfa804774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96704779, 98 },
    { 0, 0 },
    { 0x8120477b, 87 },
    { 0xd8e0477c, 23 },
    { 0x3b90477d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da04782, 17 },
    { 0xe9404782, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b0478d, 90 },
    { 0xa800478e, 23 },
    { 0x5e80478f, 76 },
    { 0xa704790, 34 },
    { 0x38c04791, 33 },
    { 0x18504792, 95 },
    { 0, 0 },
    { 0xcd804794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e04798, 43 },
    { 0xc4704799, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9430479e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a047a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5047a9, 33 },
    { 0x22d047a9, 99 },
    { 0xbd9047ab, 100 },
    { 0xc23047ac, 67 },
    { 0x11047ad, 100 },
    { 0xfc9047ae, 88 },
    { 0xa1b047ad, 90 },
    { 0x7cc047b0, 98 },
    { 0x9dd047ae, 90 },
    { 0x37f047b2, 33 },
    { 0x595047b3, 90 },
    { 0x302047b0, 34 },
    { 0x967047b3, 90 },
    { 0xd23047ae, 76 },
    { 0x4f0047b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9047bf, 66 },
    { 0, 0 },
    { 0xbb1047c1, 90 },
    { 0, 0 },
    { 0x195047c3, 52 },
    { 0x56047c4, 83 },
    { 0x4b0047c4, 95 },
    { 0x774047c6, 61 },
    { 0x889047c7, 61 },
    { 0xfc5047c8, 61 },
    { 0x34c047c8, 100 },
    { 0xe0a047c8, 90 },
    { 0, 0 },
    { 0x8c6047cc, 77 },
    { 0, 0 },
    { 0xa19047ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1047d2, 34 },
    { 0x356047d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b047d7, 66 },
    { 0xc7b047d8, 77 },
    { 0x50f047d9, 90 },
    { 0xcf4047d9, 43 },
    { 0, 0 },
    { 0x18c047dc, 49 },
    { 0xa1047dd, 95 },
    { 0x3e9047de, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x500047e1, 33 },
    { 0x17c047e2, 98 },
    { 0x4a0047e3, 61 },
    { 0, 0 },
    { 0xc45047e5, 43 },
    { 0, 0 },
    { 0xd68047e7, 33 },
    { 0x6fa047e8, 90 },
    { 0x31f047e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b047ec, 67 },
    { 0xa6f047ed, 23 },
    { 0xfd0047ee, 98 },
    { 0x28047ef, 60 },
    { 0x827047f0, 91 },
    { 0, 0 },
    { 0xf8a047f2, 33 },
    { 0, 0 },
    { 0x41a047f4, 90 },
    { 0x2e9047f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3047f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b04801, 80 },
    { 0x84204801, 95 },
    { 0x33c04803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af0480b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1500480f, 81 },
    { 0, 0 },
    { 0xbb904811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a04815, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14204818, 98 },
    { 0, 0 },
    { 0xf4d0481a, 91 },
    { 0, 0 },
    { 0xeca0481c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd04823, 95 },
    { 0xdfc04823, 85 },
    { 0xd9804824, 34 },
    { 0, 0 },
    { 0x3a604827, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b50482e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e04832, 33 },
    { 0x9e604832, 61 },
    { 0, 0 },
    { 0x2bc04835, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea04838, 95 },
    { 0, 0 },
    { 0x6cd0483a, 93 },
    { 0x43b0483b, 95 },
    { 0, 0 },
    { 0xad0483d, 55 },
    { 0xd450483e, 76 },
    { 0x3720483f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904847, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb604850, 40 },
    { 0x17f04851, 87 },
    { 0xdc504852, 61 },
    { 0x33e04852, 33 },
    { 0x45904853, 90 },
    { 0, 0 },
    { 0xec404856, 90 },
    { 0x59d04857, 91 },
    { 0x24904856, 61 },
    { 0, 0 },
    { 0xdf0485a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0485e, 53 },
    { 0, 0 },
    { 0x48304860, 23 },
    { 0xaf104860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a04866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9804869, 52 },
    { 0x7c90486a, 90 },
    { 0x84f0486b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640486e, 40 },
    { 0, 0 },
    { 0x58b04870, 100 },
    { 0x34004871, 90 },
    { 0x99c04871, 23 },
    { 0xe6704873, 61 },
    { 0, 0 },
    { 0x40304875, 84 },
    { 0, 0 },
    { 0xf5404877, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5650487b, 61 },
    { 0x6bf0487c, 23 },
    { 0xf2f0487d, 23 },
    { 0xb280487e, 69 },
    { 0x4700487f, 99 },
    { 0xebc04880, 90 },
    { 0x2820487e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc104884, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93104887, 34 },
    { 0, 0 },
    { 0x53804889, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b0488c, 88 },
    { 0x7d20488d, 37 },
    { 0, 0 },
    { 0x1e20488f, 90 },
    { 0x82e0488f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd504895, 56 },
    { 0x71c04896, 46 },
    { 0x40704897, 93 },
    { 0x35e04898, 55 },
    { 0xdd404896, 95 },
    { 0, 0 },
    { 0x92a0489b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb430489e, 23 },
    { 0, 0 },
    { 0xdd048a0, 98 },
    { 0xa07048a1, 55 },
    { 0xf08048a2, 61 },
    { 0x27d048a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3048a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb048ad, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f048b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca048b4, 61 },
    { 0xbd5048b5, 65 },
    { 0x99a048b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82048bd, 34 },
    { 0x928048bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x784048c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc048c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121048d3, 96 },
    { 0xe18048d3, 96 },
    { 0xfce048d5, 90 },
    { 0xe45048d3, 23 },
    { 0, 0 },
    { 0x189048d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a048e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40048e3, 76 },
    { 0x6ce048e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5048e7, 40 },
    { 0, 0 },
    { 0x7b0048e9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f048ef, 93 },
    { 0, 0 },
    { 0x209048f1, 61 },
    { 0xaa4048f2, 23 },
    { 0, 0 },
    { 0xa81048f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e048f9, 68 },
    { 0, 0 },
    { 0xe83048fb, 90 },
    { 0xc7e048fc, 95 },
    { 0, 0 },
    { 0x4b6048fe, 94 },
    { 0, 0 },
    { 0x6c604900, 93 },
    { 0x3104900, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31804906, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9680490b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70490f, 78 },
    { 0xef704910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45104914, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25304918, 33 },
    { 0xed804919, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57304924, 90 },
    { 0, 0 },
    { 0x45c04926, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e30492b, 33 },
    { 0xd100492c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f60492f, 98 },
    { 0xcca04930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18204934, 84 },
    { 0xa5304935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d30493a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e04942, 33 },
    { 0x65e04943, 66 },
    { 0xc5704944, 90 },
    { 0x5e904945, 76 },
    { 0xe6b04944, 52 },
    { 0xa1804947, 90 },
    { 0x7604944, 33 },
    { 0x8404942, 23 },
    { 0, 0 },
    { 0x5ba0494b, 34 },
    { 0x30d0494c, 48 },
    { 0x58f0494c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e504951, 61 },
    { 0xb9e04951, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70204956, 43 },
    { 0, 0 },
    { 0xf4404958, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d04965, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f04969, 33 },
    { 0x9940496a, 33 },
    { 0x62d0496b, 33 },
    { 0xf5c0496b, 23 },
    { 0, 0 },
    { 0x7400496e, 77 },
    { 0xd390496f, 98 },
    { 0xb490496e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c104977, 33 },
    { 0x73e04978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e0497b, 46 },
    { 0x74d0497b, 95 },
    { 0x72c0497c, 90 },
    { 0xa2a0497e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27204985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb00498c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e04993, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0804998, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b0499e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x738049a1, 33 },
    { 0x36a049a2, 95 },
    { 0x412049a3, 61 },
    { 0xe3c049a4, 33 },
    { 0xa2a049a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12049a9, 34 },
    { 0, 0 },
    { 0xcb4049ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a049b2, 36 },
    { 0xb6d049b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4049b8, 33 },
    { 0x946049b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f049bd, 90 },
    { 0xadf049be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59049c1, 23 },
    { 0xa86049c2, 33 },
    { 0, 0 },
    { 0x45d049c4, 46 },
    { 0xff5049c5, 98 },
    { 0x6b7049c4, 76 },
    { 0, 0 },
    { 0x264049c8, 91 },
    { 0, 0 },
    { 0xbb7049ca, 99 },
    { 0, 0 },
    { 0x33c049cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73049d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x504049d5, 61 },
    { 0, 0 },
    { 0x604049d7, 23 },
    { 0, 0 },
    { 0xf83049d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8049de, 98 },
    { 0x1d5049df, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2049e4, 90 },
    { 0x48d049e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592049ea, 76 },
    { 0x183049eb, 90 },
    { 0x96e049eb, 95 },
    { 0x56d049ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689049f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35049f7, 43 },
    { 0xdd7049f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7049fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e104a02, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf04a05, 90 },
    { 0x14904a06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f204a09, 95 },
    { 0x3e204a09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae04a10, 100 },
    { 0xced04a11, 93 },
    { 0x4af04a11, 34 },
    { 0xa1f04a13, 93 },
    { 0xc0904a13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e104a18, 66 },
    { 0, 0 },
    { 0xd104a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89904a1d, 61 },
    { 0, 0 },
    { 0xf4904a1f, 85 },
    { 0x36204a1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd504a28, 87 },
    { 0, 0 },
    { 0xdc504a2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d104a32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x56104a35, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c04a3a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd04a3d, 98 },
    { 0x4db04a3e, 98 },
    { 0xf804a3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf04a47, 33 },
    { 0xa1504a47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e04a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3404a50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75604a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8404a5d, 100 },
    { 0x48e04a5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34004a61, 90 },
    { 0x3d504a61, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35004a67, 33 },
    { 0xadd04a68, 95 },
    { 0x38404a68, 33 },
    { 0x6a04a68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69904a6d, 87 },
    { 0x32304a6e, 61 },
    { 0x16604a6f, 91 },
    { 0x5d004a70, 66 },
    { 0, 0 },
    { 0xa7004a72, 43 },
    { 0, 0 },
    { 0x46b04a74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd04a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5804a7d, 90 },
    { 0x7e04a7d, 95 },
    { 0xccc04a7f, 91 },
    { 0, 0 },
    { 0xef904a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0304a86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d204a8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b504a92, 88 },
    { 0x55704a92, 88 },
    { 0xd2204a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa404a98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20104a9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xed304aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd04aaa, 43 },
    { 0, 0 },
    { 0x2a704aac, 75 },
    { 0x66e04aad, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde404ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89004ac2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82204acd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8304ad0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e04ad4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e04ad7, 98 },
    { 0xded04ad8, 76 },
    { 0xa7d04ad7, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a04ae0, 77 },
    { 0, 0 },
    { 0xd7b04ae2, 76 },
    { 0x1f804ae3, 52 },
    { 0xb0c04ae2, 69 },
    { 0xe904ae4, 61 },
    { 0, 0 },
    { 0x9a404ae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9204aec, 95 },
    { 0, 0 },
    { 0x36204aee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b04af2, 85 },
    { 0x8504af2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c04af6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd04af9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a404afc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88104aff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e04b04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8004b07, 99 },
    { 0, 0 },
    { 0xb7e04b09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25204b0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad404b12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f04b16, 55 },
    { 0x63104b17, 90 },
    { 0xb2104b17, 23 },
    { 0xa7204b16, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb604b1c, 61 },
    { 0x5ea04b1c, 91 },
    { 0, 0 },
    { 0x43604b1f, 90 },
    { 0, 0 },
    { 0xeb504b21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82404b26, 37 },
    { 0x87504b27, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x25904b2a, 37 },
    { 0x67304b2b, 43 },
    { 0x6a804b2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90304b32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe404b36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa04b39, 33 },
    { 0xf2a04b3a, 33 },
    { 0xb3504b3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e04b3e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e04b42, 61 },
    { 0x27204b42, 90 },
    { 0x33604b44, 90 },
    { 0, 0 },
    { 0x2b204b46, 34 },
    { 0xa1204b47, 98 },
    { 0xa2404b48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca04b4b, 78 },
    { 0x5ee04b4b, 55 },
    { 0xb2504b4c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f04b53, 99 },
    { 0x7c204b54, 43 },
    { 0x2ca04b53, 61 },
    { 0xf8d04b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb004b59, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a504b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d04b5f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2304b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2504b69, 95 },
    { 0, 0 },
    { 0xe2404b6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4304b70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc104b75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a04b7c, 64 },
    { 0x1b704b7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf04b83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b04b8a, 90 },
    { 0xe7404b8b, 23 },
    { 0xfb604b8c, 61 },
    { 0x64c04b8c, 55 },
    { 0, 0 },
    { 0x54d04b8f, 93 },
    { 0x28304b8f, 40 },
    { 0, 0 },
    { 0x42a04b92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c104b95, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x15804b98, 61 },
    { 0x34904b98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c04b9d, 95 },
    { 0, 0 },
    { 0x3f704b9f, 33 },
    { 0x90904ba0, 76 },
    { 0, 0 },
    { 0x39d04ba2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b204ba5, 93 },
    { 0x27f04ba6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b04baa, 90 },
    { 0xe9c04bab, 95 },
    { 0x30604baa, 98 },
    { 0x5e04bab, 34 },
    { 0xb7d04bae, 33 },
    { 0, 0 },
    { 0x14904bb0, 90 },
    { 0x95504bb0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xde804bb4, 98 },
    { 0, 0 },
    { 0x5e804bb6, 23 },
    { 0xc5004bb6, 33 },
    { 0, 0 },
    { 0x75104bb9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24104bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52104bc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97804bc7, 90 },
    { 0x42d04bc8, 98 },
    { 0, 0 },
    { 0x25a04bca, 34 },
    { 0xa6c04bcb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb404bd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb804bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73904bda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f04be1, 61 },
    { 0xa8104be2, 98 },
    { 0x4904be3, 99 },
    { 0x28304be2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67204be9, 98 },
    { 0x6204bea, 98 },
    { 0x1e104beb, 76 },
    { 0x5f704bec, 90 },
    { 0xa5804bed, 88 },
    { 0, 0 },
    { 0x73d04bef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f04bfa, 98 },
    { 0, 0 },
    { 0x9af04bfc, 46 },
    { 0xf8f04bfd, 33 },
    { 0x9bd04bfd, 95 },
    { 0, 0 },
    { 0xade04c00, 92 },
    { 0, 0 },
    { 0x9fa04c02, 90 },
    { 0xce204c02, 43 },
    { 0, 0 },
    { 0xf5e04c05, 23 },
    { 0x3f504c06, 39 },
    { 0, 0 },
    { 0x35404c08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82504c0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95204c11, 55 },
    { 0xb5304c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28204c15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6004c19, 33 },
    { 0x91f04c1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4304c22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4104c26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6504c2f, 76 },
    { 0x67704c2f, 33 },
    { 0xb0204c31, 76 },
    { 0xc0104c30, 34 },
    { 0x67304c2f, 33 },
    { 0, 0 },
    { 0xe1004c35, 33 },
    { 0x84a04c35, 33 },
    { 0, 0 },
    { 0x28404c38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b104c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74104c40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3904c47, 33 },
    { 0xd8e04c48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51604c4d, 76 },
    { 0xbf904c4e, 90 },
    { 0x2ef04c4f, 52 },
    { 0xb2b04c50, 61 },
    { 0x15904c4d, 93 },
    { 0xcc904c4f, 95 },
    { 0xc1a04c53, 88 },
    { 0x44604c53, 55 },
    { 0x9be04c55, 90 },
    { 0x84804c4f, 55 },
    { 0x97204c57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8604c5b, 98 },
    { 0, 0 },
    { 0x93404c5d, 67 },
    { 0x44b04c5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b04c62, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec04c66, 55 },
    { 0, 0 },
    { 0x2e704c68, 61 },
    { 0x69704c69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59804c6c, 95 },
    { 0x8d604c6d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a04c70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6704c77, 46 },
    { 0xe2904c77, 75 },
    { 0, 0 },
    { 0x74f04c7a, 33 },
    { 0x8004c7a, 76 },
    { 0x2e504c7a, 67 },
    { 0xef604c7c, 17 },
    { 0x5ba04c7b, 46 },
    { 0x15004c7a, 95 },
    { 0x82404c80, 33 },
    { 0x9fd04c80, 33 },
    { 0xa4b04c82, 98 },
    { 0x6b504c7d, 95 },
    { 0x9a004c84, 85 },
    { 0, 0 },
    { 0x52304c86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f04c89, 98 },
    { 0xf204c8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1304c8d, 98 },
    { 0, 0 },
    { 0x7e704c8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5804c93, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f204c96, 98 },
    { 0, 0 },
    { 0x10804c98, 23 },
    { 0x7c604c98, 95 },
    { 0xef704c9a, 90 },
    { 0x49e04c9b, 65 },
    { 0, 0 },
    { 0xfa704c9d, 23 },
    { 0x66f04c9e, 76 },
    { 0, 0 },
    { 0x2c004ca0, 23 },
    { 0xef104ca0, 23 },
    { 0x70f04ca1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8204caa, 61 },
    { 0xa3304caa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec04cae, 23 },
    { 0, 0 },
    { 0xfa004cb0, 23 },
    { 0x17304cb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d04cb4, 61 },
    { 0xad904cb5, 88 },
    { 0, 0 },
    { 0xe5104cb7, 93 },
    { 0xc6e04cb8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b04cbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee04cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5204cc5, 52 },
    { 0, 0 },
    { 0xad104cc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef04cca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b04ccf, 61 },
    { 0, 0 },
    { 0x21704cd1, 98 },
    { 0, 0 },
    { 0xcd104cd3, 65 },
    { 0x77804cd4, 23 },
    { 0x89e04cd3, 90 },
    { 0xd0c04cd6, 66 },
    { 0xb4704cd6, 23 },
    { 0, 0 },
    { 0xaff04cd9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x77204cdc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe04ce1, 77 },
    { 0, 0 },
    { 0x3f604ce3, 49 },
    { 0x2ed04ce3, 23 },
    { 0, 0 },
    { 0x23a04ce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea04cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d04cf7, 43 },
    { 0xddd04cf7, 95 },
    { 0x28604cf9, 98 },
    { 0xbe504cf9, 34 },
    { 0x45f04cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e704cfe, 34 },
    { 0x2ea04cff, 99 },
    { 0xf1704cfe, 98 },
    { 0x5504d01, 23 },
    { 0, 0 },
    { 0xb7004d03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e04d09, 76 },
    { 0x23904d09, 98 },
    { 0x9ab04d09, 34 },
    { 0x67704d0c, 90 },
    { 0x16e04d0c, 52 },
    { 0xfd604d0b, 95 },
    { 0x66404d0f, 33 },
    { 0x7cb04d0b, 90 },
    { 0xc4404d11, 95 },
    { 0x25f04d11, 23 },
    { 0x86804d11, 65 },
    { 0xad504d0c, 85 },
    { 0, 0 },
    { 0x56c04d16, 95 },
    { 0xf5e04d17, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a04d1c, 90 },
    { 0xe5504d1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a04d20, 98 },
    { 0x6ac04d21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de04d29, 93 },
    { 0, 0 },
    { 0x3004d2b, 61 },
    { 0xcb804d2c, 90 },
    { 0, 0 },
    { 0x32e04d2e, 95 },
    { 0, 0 },
    { 0x4ad04d30, 23 },
    { 0x1e004d31, 23 },
    { 0x85e04d32, 55 },
    { 0, 0 },
    { 0xdcb04d34, 33 },
    { 0xb3104d35, 90 },
    { 0x40104d36, 75 },
    { 0x12404d35, 90 },
    { 0, 0 },
    { 0x75b04d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e04d3c, 98 },
    { 0xc804d3d, 33 },
    { 0xc7604d3d, 98 },
    { 0x26b04d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7904d43, 95 },
    { 0, 0 },
    { 0xfcb04d45, 33 },
    { 0x74804d46, 61 },
    { 0x5c604d47, 66 },
    { 0x3f204d47, 61 },
    { 0, 0 },
    { 0x1fc04d4a, 23 },
    { 0xc7904d4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7604d51, 33 },
    { 0xa5204d51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc04d57, 61 },
    { 0x28004d58, 34 },
    { 0, 0 },
    { 0x90004d5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa04d5e, 100 },
    { 0xce304d5f, 91 },
    { 0, 0 },
    { 0xa5a04d61, 46 },
    { 0x18f04d62, 98 },
    { 0, 0 },
    { 0xeda04d64, 98 },
    { 0x98a04d65, 90 },
    { 0x4604d66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b004d69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee04d6c, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x11104d6f, 90 },
    { 0, 0 },
    { 0xc7604d71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x67204d74, 100 },
    { 0x14904d75, 81 },
    { 0x72904d74, 95 },
    { 0xe7204d77, 100 },
    { 0xa8704d75, 95 },
    { 0x67b04d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d404d7f, 34 },
    { 0x46904d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33604d83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a04d86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b04d8b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x10804d8e, 23 },
    { 0xd0d04d8e, 90 },
    { 0x46e04d90, 90 },
    { 0xd5b04d91, 68 },
    { 0x93304d91, 69 },
    { 0xe1304d92, 52 },
    { 0xc0204d94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5704d9b, 95 },
    { 0x74d04d9c, 95 },
    { 0, 0 },
    { 0x2b804d9e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb04da3, 91 },
    { 0x12204da4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9904da8, 90 },
    { 0x58304da8, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef04daf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf04db2, 61 },
    { 0xdbe04db3, 33 },
    { 0xa0504db4, 17 },
    { 0x81304db2, 90 },
    { 0x40004db2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10004db9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57904dbc, 85 },
    { 0, 0 },
    { 0xc9904dbe, 95 },
    { 0xad904dbf, 33 },
    { 0x3a504dc0, 98 },
    { 0xeb204dc1, 65 },
    { 0, 0 },
    { 0x41904dc3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7704dc7, 90 },
    { 0, 0 },
    { 0x2ea04dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a04dce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac04dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d04dde, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f04de1, 33 },
    { 0x9a804de1, 98 },
    { 0xa2904de3, 23 },
    { 0x1cf04de4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda04dea, 61 },
    { 0x5c304dea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c604df0, 95 },
    { 0x92104df0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28504df9, 95 },
    { 0, 0 },
    { 0xd1c04dfb, 66 },
    { 0x4e604dfc, 85 },
    { 0xd3c04dfd, 90 },
    { 0x82a04dfe, 91 },
    { 0x7ff04dff, 23 },
    { 0x33a04e00, 61 },
    { 0x9604e01, 61 },
    { 0x6db04e02, 76 },
    { 0x95304dfe, 34 },
    { 0xcd104e04, 76 },
    { 0xc2704e05, 76 },
    { 0xfca04e06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9204e0c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c04e13, 55 },
    { 0x1be04e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d04e17, 85 },
    { 0xc2e04e18, 90 },
    { 0x32704e19, 55 },
    { 0x5904e1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50104e1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92004e24, 33 },
    { 0x94504e24, 61 },
    { 0x6ca04e26, 33 },
    { 0x24604e27, 88 },
    { 0xac304e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f204e2d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e04e30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a04e36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93504e40, 90 },
    { 0x51504e41, 33 },
    { 0x24804e42, 90 },
    { 0xeb904e42, 33 },
    { 0xede04e44, 76 },
    { 0x5a504e44, 55 },
    { 0x5f304e45, 65 },
    { 0x68704e46, 90 },
    { 0xef304e48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e104e4c, 33 },
    { 0xebe04e4d, 88 },
    { 0x2a904e4c, 90 },
    { 0x71f04e4f, 34 },
    { 0xa8204e4c, 81 },
    { 0xaa104e50, 87 },
    { 0x94c04e4e, 46 },
    { 0x84804e53, 23 },
    { 0x2cd04e4e, 43 },
    { 0xfed04e55, 61 },
    { 0x19e04e4c, 23 },
    { 0x70604e4e, 65 },
    { 0, 0 },
    { 0x30c04e59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf04e5d, 46 },
    { 0x7304e5e, 99 },
    { 0xb7d04e5d, 23 },
    { 0x17404e60, 33 },
    { 0x3df04e61, 34 },
    { 0x67b04e62, 33 },
    { 0xb2d04e62, 61 },
    { 0xea04e63, 84 },
    { 0x54f04e63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce204e6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec104e71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d04e77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd404e7e, 77 },
    { 0xe8e04e7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce04e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a04e89, 94 },
    { 0xb1504e89, 46 },
    { 0xfd004e89, 98 },
    { 0, 0 },
    { 0x8a704e8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d204e94, 34 },
    { 0, 0 },
    { 0x21304e96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80204e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78204ea0, 95 },
    { 0xaa804ea0, 77 },
    { 0xe6f04ea0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef004eaa, 40 },
    { 0x84c04eaa, 76 },
    { 0xa5704eac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8004eaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef304eb3, 23 },
    { 0xae804eb4, 61 },
    { 0xa1504eb4, 41 },
    { 0, 0 },
    { 0x72f04eb7, 46 },
    { 0, 0 },
    { 0x86504eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c04ec1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15904ec7, 74 },
    { 0, 0 },
    { 0xcde04ec9, 33 },
    { 0, 0 },
    { 0x7fb04ecb, 78 },
    { 0, 0 },
    { 0xab204ecd, 52 },
    { 0, 0 },
    { 0x8c504ecf, 33 },
    { 0, 0 },
    { 0x73a04ed1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8804ed8, 84 },
    { 0x5604ed9, 84 },
    { 0x32304eda, 53 },
    { 0, 0 },
    { 0x89a04edc, 23 },
    { 0xc1004edd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba004ee3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8604ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c04ef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d04efc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78304f00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a04f05, 61 },
    { 0x15104f06, 81 },
    { 0x6c04f07, 67 },
    { 0x5e204f08, 43 },
    { 0xa2604f07, 34 },
    { 0x2c04f05, 23 },
    { 0, 0 },
    { 0x77e04f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84904f0f, 90 },
    { 0, 0 },
    { 0xb1904f11, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc04f1b, 61 },
    { 0, 0 },
    { 0x92104f1d, 61 },
    { 0x72904f1e, 61 },
    { 0xeea04f1f, 52 },
    { 0x11004f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af04f25, 76 },
    { 0, 0 },
    { 0x22104f27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62404f2c, 90 },
    { 0x83f04f2d, 55 },
    { 0x41f04f2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99204f32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59404f35, 34 },
    { 0x4e404f35, 52 },
    { 0xe04f36, 33 },
    { 0, 0 },
    { 0x49504f39, 95 },
    { 0x31204f39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6004f3f, 17 },
    { 0x4df04f40, 33 },
    { 0x88d04f41, 66 },
    { 0xb8c04f42, 90 },
    { 0xc8804f42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74804f46, 43 },
    { 0x45304f47, 90 },
    { 0, 0 },
    { 0xba904f49, 61 },
    { 0, 0 },
    { 0xafc04f4b, 77 },
    { 0, 0 },
    { 0xf6c04f4d, 98 },
    { 0xf0104f4e, 93 },
    { 0x14f04f4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef04f53, 23 },
    { 0xf4a04f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa04f60, 78 },
    { 0x3c104f61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5604f66, 92 },
    { 0, 0 },
    { 0xb9204f68, 61 },
    { 0, 0 },
    { 0x8aa04f6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804f6f, 76 },
    { 0xeef04f6f, 91 },
    { 0x79204f71, 95 },
    { 0x42304f72, 43 },
    { 0x64d04f73, 95 },
    { 0x30a04f74, 66 },
    { 0x17b04f75, 95 },
    { 0xffe04f75, 23 },
    { 0xfe204f73, 87 },
    { 0x23404f70, 35 },
    { 0xe6b04f79, 33 },
    { 0xa9b04f7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b04f7e, 75 },
    { 0xc2b04f7f, 61 },
    { 0x56004f7f, 90 },
    { 0x81204f81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe04f84, 43 },
    { 0x3cb04f84, 33 },
    { 0x5cb04f86, 93 },
    { 0, 0 },
    { 0xfd004f88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe04f8f, 93 },
    { 0x75f04f90, 23 },
    { 0, 0 },
    { 0x8e204f92, 95 },
    { 0x2de04f93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f004f96, 43 },
    { 0, 0 },
    { 0xafd04f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc104f9b, 33 },
    { 0xcec04f9c, 34 },
    { 0xd2604f9d, 91 },
    { 0, 0 },
    { 0xa9b04f9f, 34 },
    { 0xc7f04fa0, 33 },
    { 0x2ff04fa1, 23 },
    { 0xf5c04fa2, 90 },
    { 0x10204fa3, 43 },
    { 0xc9504fa0, 80 },
    { 0x83e04fa5, 69 },
    { 0, 0 },
    { 0xe5204fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a04faa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x61904fad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda404fb0, 33 },
    { 0, 0 },
    { 0xf04fb2, 90 },
    { 0x8aa04fb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7604fb7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9404fbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d04fc0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e04fc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45304fca, 95 },
    { 0, 0 },
    { 0x72f04fcc, 100 },
    { 0x12504fcc, 85 },
    { 0, 0 },
    { 0x6c304fcf, 61 },
    { 0, 0 },
    { 0xf7104fd1, 90 },
    { 0xa4a04fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da04fda, 61 },
    { 0x3cf04fdb, 81 },
    { 0x20504fdc, 55 },
    { 0x48704fdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf04fe1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3704fe4, 23 },
    { 0x5dc04fe4, 98 },
    { 0x5a404fe6, 33 },
    { 0x10504fe7, 90 },
    { 0xe8204fe6, 95 },
    { 0, 0 },
    { 0x1e204fea, 95 },
    { 0xa2d04feb, 43 },
    { 0x4fb04feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36104fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae04ff3, 90 },
    { 0x9fc04ff4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e04ff8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16904ffb, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x63604ffe, 84 },
    { 0, 0 },
    { 0xddf05000, 66 },
    { 0x72605000, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9505004, 33 },
    { 0xd7b05005, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc05009, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b0500d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a505012, 43 },
    { 0x98f05013, 90 },
    { 0xd1705014, 55 },
    { 0xfbf05012, 43 },
    { 0xa7105016, 80 },
    { 0xc8e05014, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb00501c, 61 },
    { 0x9be0501d, 23 },
    { 0, 0 },
    { 0x38f0501f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d05024, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x605027, 87 },
    { 0x47005027, 52 },
    { 0xed105029, 91 },
    { 0xb4e05027, 23 },
    { 0xeea0502b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7860502f, 90 },
    { 0x40605030, 95 },
    { 0, 0 },
    { 0xc3b05032, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc905035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd05041, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38105048, 76 },
    { 0xe7e05049, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0504d, 65 },
    { 0x7060504d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33705052, 23 },
    { 0xd4405053, 76 },
    { 0x7d005054, 95 },
    { 0xcd305053, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f0505a, 61 },
    { 0x1970505b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e0505f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x83905062, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2520506a, 5 },
    { 0, 0 },
    { 0x1640506c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c905071, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57305076, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a90507c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf730507f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c05082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a05085, 93 },
    { 0xe6f05085, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec05089, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad0508d, 90 },
    { 0x670508e, 90 },
    { 0x2ae0508f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d705093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd805096, 76 },
    { 0xadd05097, 90 },
    { 0, 0 },
    { 0x4b405099, 95 },
    { 0, 0 },
    { 0xeac0509b, 90 },
    { 0, 0 },
    { 0xe3a0509d, 98 },
    { 0x44c0509e, 61 },
    { 0xc830509f, 95 },
    { 0x3670509f, 33 },
    { 0x63f0509f, 33 },
    { 0xfcf050a2, 33 },
    { 0x4f6050a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2050ab, 98 },
    { 0, 0 },
    { 0x15e050ad, 55 },
    { 0xcd6050ad, 95 },
    { 0x690050af, 81 },
    { 0x349050b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17050b3, 55 },
    { 0xcb050b4, 90 },
    { 0x3d6050b4, 65 },
    { 0xc5f050b6, 55 },
    { 0xf57050b7, 55 },
    { 0x3fb050b8, 76 },
    { 0x73a050b6, 100 },
    { 0x141050ba, 61 },
    { 0x369050bb, 90 },
    { 0xade050ba, 34 },
    { 0xd3d050b8, 33 },
    { 0xf7050b3, 33 },
    { 0xd80050bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3050c3, 98 },
    { 0x583050c4, 36 },
    { 0, 0 },
    { 0xba1050c6, 61 },
    { 0x54b050c7, 93 },
    { 0xf18050c8, 61 },
    { 0x23d050c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe050d0, 33 },
    { 0xd6a050d1, 95 },
    { 0xbb6050d1, 95 },
    { 0x3f2050d3, 95 },
    { 0x1d0050d3, 90 },
    { 0x8f7050d4, 99 },
    { 0x762050d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6050de, 76 },
    { 0x51a050df, 76 },
    { 0x32050df, 46 },
    { 0x1ed050e1, 23 },
    { 0x11e050e1, 90 },
    { 0x775050e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b050e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9050e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801050ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc050f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x424050f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98050f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65050ff, 23 },
    { 0x95105100, 76 },
    { 0x42a05101, 33 },
    { 0xb8305102, 61 },
    { 0xc0005102, 61 },
    { 0x4c405102, 94 },
    { 0xd4f05105, 33 },
    { 0xe3505106, 55 },
    { 0xb9b05103, 55 },
    { 0, 0 },
    { 0xcf505109, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd170510c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef0510f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62705112, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2605116, 33 },
    { 0, 0 },
    { 0x9d005118, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe390511b, 84 },
    { 0x5250511c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6005120, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf05123, 66 },
    { 0xcc705124, 61 },
    { 0x29c05124, 90 },
    { 0xeb705125, 17 },
    { 0x14c05124, 96 },
    { 0x89505128, 98 },
    { 0xc4205128, 23 },
    { 0, 0 },
    { 0xe5d0512b, 66 },
    { 0xfa30512c, 61 },
    { 0x8ce0512d, 81 },
    { 0, 0 },
    { 0xfc30512f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32205135, 95 },
    { 0xdd705136, 34 },
    { 0xe9e05137, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b0513b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b805145, 61 },
    { 0xa2b05146, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa0514a, 23 },
    { 0x29e0514b, 43 },
    { 0x9d00514c, 90 },
    { 0x4b50514d, 43 },
    { 0, 0 },
    { 0x3a80514f, 33 },
    { 0xf5305150, 76 },
    { 0, 0 },
    { 0xcbc05152, 23 },
    { 0x5d305153, 76 },
    { 0, 0 },
    { 0xd9605155, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef20515b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3550515e, 61 },
    { 0xf5f0515f, 33 },
    { 0xf530515f, 90 },
    { 0xaa005161, 61 },
    { 0xafa05161, 95 },
    { 0x34305161, 33 },
    { 0x7fb0515e, 100 },
    { 0x2f905165, 95 },
    { 0x3100515e, 95 },
    { 0, 0 },
    { 0xad005168, 55 },
    { 0x8e805169, 77 },
    { 0, 0 },
    { 0x9630516b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5405172, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad05175, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9230517a, 61 },
    { 0x51f0517b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0605180, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b905184, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc805187, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf90518c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a0519b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0051a2, 98 },
    { 0x80b051a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a051a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188051af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8051b5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf051be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f051c3, 34 },
    { 0x31c051c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8051cb, 46 },
    { 0x901051cb, 76 },
    { 0x3ec051cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9051d0, 46 },
    { 0xb18051d1, 90 },
    { 0, 0 },
    { 0x43e051d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb051d7, 90 },
    { 0x502051d8, 55 },
    { 0xfda051d8, 61 },
    { 0x6bb051d9, 85 },
    { 0x8ba051db, 34 },
    { 0x7c4051dc, 33 },
    { 0, 0 },
    { 0x914051de, 33 },
    { 0, 0 },
    { 0x7b8051e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d051ed, 55 },
    { 0, 0 },
    { 0x264051ef, 76 },
    { 0x325051ef, 23 },
    { 0xb1f051f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a051f4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af051f7, 23 },
    { 0, 0 },
    { 0x611051f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c05200, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7305203, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36905208, 17 },
    { 0x86e05208, 17 },
    { 0, 0 },
    { 0xc310520b, 90 },
    { 0x9c70520c, 33 },
    { 0xde90520d, 90 },
    { 0xec90520d, 17 },
    { 0x9060520d, 33 },
    { 0x2280520e, 55 },
    { 0xae05211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f05214, 61 },
    { 0, 0 },
    { 0x81705216, 61 },
    { 0xf3905217, 33 },
    { 0x60505218, 34 },
    { 0x45405216, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a005220, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7505225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe0522c, 72 },
    { 0xc5f0522c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5805237, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0523b, 95 },
    { 0, 0 },
    { 0x6df0523d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c005240, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f05243, 61 },
    { 0xdb05244, 33 },
    { 0, 0 },
    { 0xdbe05246, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57405249, 98 },
    { 0x8360524a, 61 },
    { 0xe7a0524a, 61 },
    { 0xe880524c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a05251, 76 },
    { 0xf5105252, 95 },
    { 0, 0 },
    { 0x12e05254, 74 },
    { 0, 0 },
    { 0x58e05256, 33 },
    { 0xead05257, 81 },
    { 0x9a005257, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe440525b, 90 },
    { 0x8480525c, 33 },
    { 0, 0 },
    { 0x5c10525e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5305264, 61 },
    { 0xf3405264, 33 },
    { 0, 0 },
    { 0x3a205267, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e10526f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f05273, 95 },
    { 0x77905274, 34 },
    { 0x34005275, 98 },
    { 0xc5805275, 95 },
    { 0x85705277, 43 },
    { 0x21905278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5405281, 46 },
    { 0, 0 },
    { 0x29305283, 95 },
    { 0x7e205284, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed05288, 99 },
    { 0xc0f05289, 76 },
    { 0x47c0528a, 98 },
    { 0x3040528b, 46 },
    { 0xf360528c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab00528f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc05294, 95 },
    { 0, 0 },
    { 0x1f405296, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db0529a, 81 },
    { 0xb8a0529b, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2052a3, 33 },
    { 0x794052a4, 33 },
    { 0x8ce052a4, 90 },
    { 0x1e8052a3, 33 },
    { 0xbe8052a5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6052aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67052b3, 68 },
    { 0x71c052b4, 43 },
    { 0xd40052b4, 98 },
    { 0, 0 },
    { 0xefc052b7, 55 },
    { 0x99c052b8, 90 },
    { 0, 0 },
    { 0xab9052ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150052bf, 34 },
    { 0xd61052c0, 23 },
    { 0x106052c1, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb052ca, 76 },
    { 0x970052cb, 23 },
    { 0x409052cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8052d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b052dc, 43 },
    { 0x7fa052dd, 23 },
    { 0x554052de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49052e3, 46 },
    { 0xfb4052e4, 76 },
    { 0xe9d052e5, 85 },
    { 0x79b052e3, 99 },
    { 0, 0 },
    { 0x4d6052e8, 61 },
    { 0x1ac052e9, 84 },
    { 0xe9f052e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x535052ed, 61 },
    { 0, 0 },
    { 0xe64052ef, 68 },
    { 0, 0 },
    { 0x932052f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80052fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5205305, 96 },
    { 0, 0 },
    { 0x81e05307, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3360530a, 61 },
    { 0xe520530a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7780530f, 61 },
    { 0x70f05310, 95 },
    { 0xf420530f, 90 },
    { 0x82205312, 23 },
    { 0x55105312, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2805316, 76 },
    { 0x6aa05317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4780531a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb0531d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14605321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26805324, 90 },
    { 0, 0 },
    { 0xf0505326, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280532a, 67 },
    { 0xf960532b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73405332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc205335, 78 },
    { 0, 0 },
    { 0xf6705337, 89 },
    { 0xa4005338, 96 },
    { 0xab405339, 61 },
    { 0x74d05338, 66 },
    { 0xc1305337, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1090533f, 67 },
    { 0x1e205340, 91 },
    { 0xc3105341, 33 },
    { 0x15705342, 77 },
    { 0x7e70533f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b05346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d20534b, 90 },
    { 0, 0 },
    { 0x470534d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a05352, 90 },
    { 0x59e05353, 75 },
    { 0x35205354, 61 },
    { 0xe6205354, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc10535c, 53 },
    { 0, 0 },
    { 0x3700535e, 66 },
    { 0xed80535f, 95 },
    { 0x8105360, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b05364, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6405367, 76 },
    { 0, 0 },
    { 0x4ca05369, 23 },
    { 0xe110536a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1120536e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea05373, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7605378, 67 },
    { 0xa6805379, 61 },
    { 0xa4f05379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360537f, 23 },
    { 0x1dd05380, 46 },
    { 0x30805380, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b05384, 33 },
    { 0x48c05385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a05389, 95 },
    { 0x58505389, 98 },
    { 0, 0 },
    { 0xefd0538c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f05396, 98 },
    { 0x9b005396, 46 },
    { 0xc0e05396, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a0539d, 61 },
    { 0x1720539d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb053a3, 38 },
    { 0x7bc053a4, 61 },
    { 0xf13053a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1053a9, 33 },
    { 0, 0 },
    { 0xea5053ab, 95 },
    { 0x187053ab, 43 },
    { 0x370053ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a053b1, 93 },
    { 0x1c3053b2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00053b5, 90 },
    { 0, 0 },
    { 0x5fa053b7, 98 },
    { 0xf2053b8, 55 },
    { 0x423053b9, 43 },
    { 0x9c4053ba, 58 },
    { 0x5fd053bb, 33 },
    { 0x950053bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x747053bf, 43 },
    { 0x38b053c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae053c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x339053c8, 98 },
    { 0x1da053c8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de053cc, 43 },
    { 0xafa053cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51053d1, 83 },
    { 0, 0 },
    { 0xb0d053d3, 33 },
    { 0x1d2053d3, 99 },
    { 0x325053d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94053de, 46 },
    { 0xc4b053df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76053ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8053ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e053f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc053f7, 23 },
    { 0xe05053f8, 77 },
    { 0x975053f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce05401, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c05406, 99 },
    { 0, 0 },
    { 0x97a05408, 95 },
    { 0x94705409, 95 },
    { 0xcef05409, 33 },
    { 0x83e0540a, 23 },
    { 0, 0 },
    { 0xfc80540d, 95 },
    { 0, 0 },
    { 0xb270540f, 93 },
    { 0x3cf05410, 84 },
    { 0xdd605410, 91 },
    { 0x2fa05411, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b05419, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950541f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35205422, 43 },
    { 0xde905423, 34 },
    { 0x31a05424, 76 },
    { 0x8a805424, 78 },
    { 0x21505424, 75 },
    { 0x64d05427, 65 },
    { 0xc4105428, 67 },
    { 0xb5805423, 90 },
    { 0x8a405426, 33 },
    { 0xd9b05422, 76 },
    { 0x4c005425, 23 },
    { 0x70e05427, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b805435, 90 },
    { 0xd2505436, 95 },
    { 0xc7c05436, 23 },
    { 0x68d05436, 76 },
    { 0x72d05438, 98 },
    { 0xa280543a, 33 },
    { 0xc3505438, 61 },
    { 0xc40543c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d05440, 98 },
    { 0, 0 },
    { 0x55605442, 90 },
    { 0xc9805443, 34 },
    { 0, 0 },
    { 0xfd605445, 46 },
    { 0x7d605445, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd20544a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d0544e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c05453, 87 },
    { 0x33205453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a0545a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb105466, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec0546a, 23 },
    { 0x1060546a, 52 },
    { 0xf610546a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b10546f, 33 },
    { 0x79f05470, 91 },
    { 0x50605471, 55 },
    { 0xee705472, 95 },
    { 0xed505473, 43 },
    { 0x8205473, 93 },
    { 0xc6805475, 85 },
    { 0x83805471, 95 },
    { 0, 0 },
    { 0x56605478, 95 },
    { 0x3ac05479, 87 },
    { 0x41205478, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e005481, 87 },
    { 0, 0 },
    { 0x4e105483, 23 },
    { 0x52005483, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70548a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe990548d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b05490, 95 },
    { 0xf7f05491, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd0549a, 90 },
    { 0, 0 },
    { 0xad30549c, 23 },
    { 0x6d80549c, 95 },
    { 0, 0 },
    { 0xb0e0549f, 95 },
    { 0x68c0549f, 90 },
    { 0x60d054a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638054a7, 61 },
    { 0xf58054a7, 33 },
    { 0xf9054a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1054ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef054b4, 95 },
    { 0x52a054b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc054bc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71054bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82054c2, 76 },
    { 0xa05054c2, 90 },
    { 0x58b054c2, 66 },
    { 0, 0 },
    { 0x198054c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0054cb, 55 },
    { 0xa60054cb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88054cf, 93 },
    { 0, 0 },
    { 0xe8d054d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f054d6, 34 },
    { 0xa29054d6, 99 },
    { 0, 0 },
    { 0x386054d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc054de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e054e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e054e6, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308054ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x700054ef, 23 },
    { 0x547054ef, 95 },
    { 0x32a054f1, 38 },
    { 0x91c054f1, 43 },
    { 0, 0 },
    { 0x121054f4, 43 },
    { 0xcd3054f5, 98 },
    { 0x944054f4, 87 },
    { 0x2dc054f6, 81 },
    { 0, 0 },
    { 0x763054f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05054fc, 66 },
    { 0x74b054fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa05504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe105509, 90 },
    { 0xb540550a, 95 },
    { 0x8f50550b, 99 },
    { 0x4ea0550c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19905511, 78 },
    { 0xfcd05512, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c05518, 33 },
    { 0xa6205518, 33 },
    { 0x6c90551a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b60552b, 33 },
    { 0x5c0552c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570552f, 61 },
    { 0x3e10552f, 84 },
    { 0x30305531, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba0553c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1505540, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d05543, 98 },
    { 0, 0 },
    { 0xf0d05545, 95 },
    { 0x17005546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f30554a, 98 },
    { 0, 0 },
    { 0xb000554c, 33 },
    { 0x2020554d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0105553, 90 },
    { 0x46605554, 78 },
    { 0, 0 },
    { 0x8ac05556, 76 },
    { 0x14105557, 95 },
    { 0xccb05558, 23 },
    { 0x8cb05559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4330555c, 90 },
    { 0x72e0555c, 90 },
    { 0x6a00555e, 97 },
    { 0xcd20555d, 98 },
    { 0x38b0555c, 75 },
    { 0xa9605561, 53 },
    { 0xf5805562, 96 },
    { 0x2d405561, 46 },
    { 0x61805561, 33 },
    { 0, 0 },
    { 0x43d05566, 95 },
    { 0xafa05567, 33 },
    { 0x3ea05568, 23 },
    { 0xa8805569, 33 },
    { 0xe30556a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d0556e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e605574, 95 },
    { 0x4eb05574, 17 },
    { 0x39305576, 61 },
    { 0x8ef05577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd720557c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c105580, 78 },
    { 0, 0 },
    { 0x4cb05582, 33 },
    { 0xaa305583, 90 },
    { 0x70005584, 90 },
    { 0x9705585, 85 },
    { 0xb0e05585, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe680558a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c10558e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab05592, 90 },
    { 0, 0 },
    { 0x99705594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b705597, 52 },
    { 0xd005598, 43 },
    { 0x67f05598, 98 },
    { 0x5820559a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c20559f, 61 },
    { 0xa19055a0, 90 },
    { 0xa6e055a0, 100 },
    { 0xde1055a0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807055a6, 96 },
    { 0x981055a7, 95 },
    { 0x9cc055a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb055ac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb055b4, 78 },
    { 0xc56055b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9055b8, 43 },
    { 0, 0 },
    { 0xb7e055ba, 76 },
    { 0xc3055bb, 90 },
    { 0xfea055bb, 98 },
    { 0x541055bb, 55 },
    { 0xea5055bd, 23 },
    { 0, 0 },
    { 0x5b6055c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x776055c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836055c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe055cc, 33 },
    { 0, 0 },
    { 0xbff055ce, 33 },
    { 0xa44055ce, 61 },
    { 0xf38055d0, 33 },
    { 0xa1f055d0, 90 },
    { 0x4c6055ce, 17 },
    { 0x4e2055d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47055d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7055dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362055e0, 95 },
    { 0x386055e0, 61 },
    { 0, 0 },
    { 0x852055e3, 46 },
    { 0xb6b055e4, 95 },
    { 0xa52055e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x903055e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5055ed, 90 },
    { 0, 0 },
    { 0xca5055ef, 81 },
    { 0, 0 },
    { 0xd69055f1, 55 },
    { 0, 0 },
    { 0x7e055f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e055f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05055fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad505602, 90 },
    { 0x24f05603, 78 },
    { 0, 0 },
    { 0xfbe05605, 33 },
    { 0, 0 },
    { 0xa4305607, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410560b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e505619, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d0561c, 95 },
    { 0x3c10561c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26405620, 55 },
    { 0x68f05621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8805629, 23 },
    { 0x45d0562a, 55 },
    { 0xeda0562a, 84 },
    { 0x6180562b, 98 },
    { 0xb040562d, 90 },
    { 0x33f0562d, 93 },
    { 0x640562e, 90 },
    { 0xf0c0562c, 98 },
    { 0, 0 },
    { 0xdaa05632, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe905637, 93 },
    { 0x99d05638, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1770563b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff505643, 97 },
    { 0, 0 },
    { 0x8d005645, 84 },
    { 0x21105646, 76 },
    { 0xdf905646, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x430564a, 98 },
    { 0x23d0564b, 87 },
    { 0xbcf0564c, 97 },
    { 0x410564d, 69 },
    { 0xd5b0564d, 33 },
    { 0xaa50564d, 17 },
    { 0x56005650, 61 },
    { 0xe0105651, 90 },
    { 0x5ee0564d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab05655, 90 },
    { 0x83d05656, 98 },
    { 0xa1005657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e80565d, 64 },
    { 0xe5e0565e, 98 },
    { 0xdca0565f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83605662, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42605667, 55 },
    { 0x95e05667, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b05672, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a05677, 77 },
    { 0, 0 },
    { 0x4fd05679, 34 },
    { 0x41a0567a, 61 },
    { 0, 0 },
    { 0x80b0567c, 87 },
    { 0x7a10567c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3305683, 100 },
    { 0x8ce05684, 81 },
    { 0xf2b05684, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x90905688, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba0568b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1570568e, 61 },
    { 0x1f70568e, 46 },
    { 0xdff05690, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70405694, 93 },
    { 0x4d205695, 17 },
    { 0xb7905696, 91 },
    { 0x8ad05697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5390569f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x732056a2, 46 },
    { 0xd4f056a2, 33 },
    { 0xfa6056a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5056a7, 61 },
    { 0xfb7056a8, 23 },
    { 0x11c056a7, 65 },
    { 0x8b5056aa, 90 },
    { 0x736056ab, 90 },
    { 0, 0 },
    { 0x8f5056ad, 76 },
    { 0x52a056ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77056b1, 84 },
    { 0xe16056b2, 33 },
    { 0xead056b3, 36 },
    { 0x297056b4, 76 },
    { 0x50056b5, 52 },
    { 0x148056b1, 90 },
    { 0, 0 },
    { 0xad8056b8, 95 },
    { 0, 0 },
    { 0xb64056ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671056c0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1056cb, 98 },
    { 0, 0 },
    { 0xff7056cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb056d4, 68 },
    { 0, 0 },
    { 0x8d2056d6, 98 },
    { 0, 0 },
    { 0xc7056d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4056dd, 34 },
    { 0xb6056de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95056e5, 23 },
    { 0xd8c056e5, 90 },
    { 0xe85056e7, 33 },
    { 0xfa5056e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60056ec, 61 },
    { 0xaa8056ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x646056f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569056f6, 36 },
    { 0x933056f7, 95 },
    { 0xb65056f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c056fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x438056fe, 87 },
    { 0x6cb056ff, 66 },
    { 0xff905700, 76 },
    { 0x8e305701, 98 },
    { 0, 0 },
    { 0x87405703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e05706, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x19805709, 98 },
    { 0x1c90570a, 90 },
    { 0, 0 },
    { 0x99f0570c, 40 },
    { 0x6d60570d, 92 },
    { 0x8ad0570e, 33 },
    { 0x6990570d, 90 },
    { 0x6df05710, 76 },
    { 0x9bf05711, 90 },
    { 0, 0 },
    { 0xe5505713, 99 },
    { 0, 0 },
    { 0x78b05715, 68 },
    { 0, 0 },
    { 0x6ba05717, 90 },
    { 0, 0 },
    { 0xe6e05719, 90 },
    { 0xa8a0571a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10571d, 42 },
    { 0xdb0571d, 67 },
    { 0x2a20571f, 66 },
    { 0, 0 },
    { 0x95505721, 98 },
    { 0x11d05722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d305727, 34 },
    { 0, 0 },
    { 0xd4505729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df0572d, 76 },
    { 0, 0 },
    { 0xcd40572f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43705732, 55 },
    { 0x27a05733, 68 },
    { 0x60f05734, 91 },
    { 0xf3505734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f05738, 23 },
    { 0, 0 },
    { 0x9340573a, 66 },
    { 0x5d00573b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2490573f, 53 },
    { 0x4e105740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70305747, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20574a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90574f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f05754, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf305757, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0575c, 33 },
    { 0xe990575d, 23 },
    { 0xa630575e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe005764, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb05767, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8270576a, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf290576e, 85 },
    { 0x2b0576e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c05773, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf05776, 90 },
    { 0x3ae05776, 98 },
    { 0, 0 },
    { 0x9205779, 90 },
    { 0x1120577a, 69 },
    { 0x4c80577a, 98 },
    { 0xb510577c, 61 },
    { 0x9380577a, 98 },
    { 0xf230577e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e705783, 52 },
    { 0x74705784, 33 },
    { 0x33205783, 90 },
    { 0, 0 },
    { 0x58e05787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c80578a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b05792, 46 },
    { 0xb6805792, 33 },
    { 0x92a05794, 61 },
    { 0xb6305792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d605798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b0579b, 98 },
    { 0x4cf0579c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a057a4, 90 },
    { 0x73a057a5, 81 },
    { 0, 0 },
    { 0x224057a7, 98 },
    { 0x360057a8, 76 },
    { 0xaf0057a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285057bc, 90 },
    { 0x8e8057bd, 34 },
    { 0xc5a057bd, 81 },
    { 0, 0 },
    { 0xc37057c0, 33 },
    { 0, 0 },
    { 0x99b057c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40057c5, 52 },
    { 0x96c057c6, 33 },
    { 0x961057c5, 90 },
    { 0x568057c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84057cc, 23 },
    { 0, 0 },
    { 0xd96057ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66057d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc057d5, 33 },
    { 0x80e057d6, 23 },
    { 0x758057d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3057dc, 95 },
    { 0xca2057dd, 61 },
    { 0x1aa057de, 90 },
    { 0x347057df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a057e6, 95 },
    { 0xa9f057e7, 76 },
    { 0x411057e8, 99 },
    { 0x7f7057e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628057ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26057f5, 98 },
    { 0x116057f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a057fe, 90 },
    { 0, 0 },
    { 0x66a05800, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a505803, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df05806, 52 },
    { 0x86505807, 61 },
    { 0xba05808, 33 },
    { 0xfdc05809, 93 },
    { 0x2d60580a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70580f, 61 },
    { 0x5a80580f, 77 },
    { 0x3d305810, 98 },
    { 0xf7905812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0405817, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8170581c, 61 },
    { 0, 0 },
    { 0x1810581e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c05822, 23 },
    { 0xd4005823, 66 },
    { 0xdc405824, 76 },
    { 0x47105822, 33 },
    { 0xf1105826, 76 },
    { 0xb8805824, 91 },
    { 0, 0 },
    { 0x8e705829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4470582d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd705831, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e05840, 52 },
    { 0x13905840, 17 },
    { 0xce105842, 33 },
    { 0x95205840, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c05849, 85 },
    { 0, 0 },
    { 0x2100584b, 87 },
    { 0x2c70584c, 23 },
    { 0x3670584d, 100 },
    { 0x7260584c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f05852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1405859, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0585c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0405870, 90 },
    { 0xc0c05870, 33 },
    { 0x22705871, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d805878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990587c, 55 },
    { 0xcd70587d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f05880, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b505883, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f205887, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc930588d, 96 },
    { 0xa230588d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f05893, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x86105896, 68 },
    { 0xa6805897, 33 },
    { 0xa4505896, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65058a1, 34 },
    { 0, 0 },
    { 0x4c058a3, 93 },
    { 0x9a0058a4, 90 },
    { 0, 0 },
    { 0x257058a6, 59 },
    { 0xa5e058a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b058aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809058ae, 88 },
    { 0xd51058af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba058b3, 97 },
    { 0, 0 },
    { 0x23c058b5, 52 },
    { 0x809058b5, 33 },
    { 0, 0 },
    { 0xdae058b8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403058bc, 90 },
    { 0xae7058bd, 90 },
    { 0xa1e058be, 84 },
    { 0x44d058bc, 61 },
    { 0x674058c0, 33 },
    { 0, 0 },
    { 0xdaf058c2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e058c6, 95 },
    { 0xea8058c7, 98 },
    { 0x22e058c8, 43 },
    { 0x60058c8, 84 },
    { 0x373058ca, 90 },
    { 0x9bb058ca, 17 },
    { 0xbdb058cc, 40 },
    { 0x886058cc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08058d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e058d9, 76 },
    { 0x9e8058da, 33 },
    { 0xe19058da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433058df, 43 },
    { 0xb3a058df, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26058e5, 61 },
    { 0xb80058e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x226058e9, 78 },
    { 0, 0 },
    { 0x9fe058eb, 90 },
    { 0, 0 },
    { 0x2ec058ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83058f4, 90 },
    { 0x8fb058f5, 91 },
    { 0, 0 },
    { 0xf66058f7, 23 },
    { 0, 0 },
    { 0xe0058f9, 33 },
    { 0, 0 },
    { 0xd09058fb, 23 },
    { 0xdeb058fc, 78 },
    { 0, 0 },
    { 0x511058fe, 95 },
    { 0xa6d058ff, 46 },
    { 0x1ed058ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf240590a, 75 },
    { 0x10d0590b, 34 },
    { 0xd0e0590c, 61 },
    { 0x4ff0590c, 17 },
    { 0, 0 },
    { 0x79e0590f, 55 },
    { 0xc220590f, 76 },
    { 0x4da05911, 92 },
    { 0, 0 },
    { 0xdbe05913, 76 },
    { 0x49705914, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95405917, 100 },
    { 0, 0 },
    { 0xef605919, 76 },
    { 0xc770591a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e0591d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c505929, 95 },
    { 0xe8c0592a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe700592d, 23 },
    { 0x2b30592e, 90 },
    { 0x1610592d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16805933, 55 },
    { 0, 0 },
    { 0x7f505935, 78 },
    { 0x97c05935, 23 },
    { 0xcb405936, 91 },
    { 0x18005935, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf60593c, 61 },
    { 0x8900593c, 98 },
    { 0xff00593e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a505945, 23 },
    { 0, 0 },
    { 0xdf805947, 93 },
    { 0, 0 },
    { 0x20b05949, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd00594d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd105959, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4580595e, 61 },
    { 0xf8e0595e, 40 },
    { 0x46d05960, 66 },
    { 0x91205961, 95 },
    { 0x5c105961, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de05969, 23 },
    { 0x7ce05969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x596d, 87 },
    { 0x27c0596e, 90 },
    { 0x3130596f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65805972, 98 },
    { 0x36e05973, 23 },
    { 0, 0 },
    { 0xbcb05975, 97 },
    { 0x9c05976, 99 },
    { 0x4ca05977, 76 },
    { 0x69805975, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe780597b, 90 },
    { 0x3e10597c, 68 },
    { 0x43d0597c, 90 },
    { 0x7360597b, 33 },
    { 0x6a0597f, 98 },
    { 0xdbe0597d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70605983, 95 },
    { 0, 0 },
    { 0x4dd05985, 98 },
    { 0xa9105986, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a05989, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef0598f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb405992, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d05997, 33 },
    { 0, 0 },
    { 0xe6e05999, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7020599c, 78 },
    { 0x9690599d, 23 },
    { 0x16b0599d, 88 },
    { 0xd3e0599f, 23 },
    { 0xd05059a0, 55 },
    { 0xe65059a1, 84 },
    { 0x140059a2, 91 },
    { 0x14c0599d, 95 },
    { 0x11b059a1, 55 },
    { 0xd7f059a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0059a8, 85 },
    { 0x654059a8, 23 },
    { 0x990059aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4059ae, 95 },
    { 0, 0 },
    { 0x61d059b0, 33 },
    { 0x5e6059b1, 61 },
    { 0xedd059b2, 85 },
    { 0x267059b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e059b9, 93 },
    { 0, 0 },
    { 0xdb059bb, 90 },
    { 0x5c4059bc, 90 },
    { 0, 0 },
    { 0x7bd059be, 56 },
    { 0x32d059bf, 90 },
    { 0xa46059c0, 81 },
    { 0x627059c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d059c6, 84 },
    { 0, 0 },
    { 0x7d6059c8, 99 },
    { 0xe55059c9, 34 },
    { 0x7dd059ca, 93 },
    { 0x726059ca, 23 },
    { 0xe0e059c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922059d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb059d6, 33 },
    { 0, 0 },
    { 0xf0c059d8, 76 },
    { 0xd27059d9, 81 },
    { 0xe97059d9, 98 },
    { 0xe40059d8, 34 },
    { 0xf2a059db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc059e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30059e6, 98 },
    { 0x4ac059e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e059ea, 81 },
    { 0xfae059eb, 55 },
    { 0x46d059ec, 55 },
    { 0x8c059ed, 23 },
    { 0x74059ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7059f2, 90 },
    { 0xd1d059f3, 90 },
    { 0xee7059f2, 96 },
    { 0xf84059f3, 33 },
    { 0xe63059f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60059f9, 53 },
    { 0x2a3059f9, 98 },
    { 0x553059fb, 61 },
    { 0xbb059f9, 33 },
    { 0xc8a059f9, 33 },
    { 0x8ef059fe, 90 },
    { 0x6b059ff, 33 },
    { 0x293059fe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2905a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d705a06, 100 },
    { 0xcb505a07, 98 },
    { 0x84f05a07, 68 },
    { 0x5be05a08, 98 },
    { 0xf5a05a07, 43 },
    { 0x97205a07, 76 },
    { 0x7d205a08, 23 },
    { 0, 0 },
    { 0xd9105a0e, 67 },
    { 0x7c005a0f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e05a16, 99 },
    { 0x4c405a16, 95 },
    { 0x33605a18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef05a1c, 33 },
    { 0, 0 },
    { 0x37705a1e, 98 },
    { 0xd4f05a1f, 98 },
    { 0xb4c05a20, 37 },
    { 0, 0 },
    { 0x39205a22, 76 },
    { 0xd7605a23, 76 },
    { 0xf4905a24, 90 },
    { 0xc8405a24, 33 },
    { 0xdcc05a23, 84 },
    { 0x66305a24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c05a2d, 33 },
    { 0xfb105a2e, 85 },
    { 0, 0 },
    { 0xdec05a30, 84 },
    { 0, 0 },
    { 0xe6505a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0605a35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee805a3a, 23 },
    { 0, 0 },
    { 0xe4905a3c, 95 },
    { 0xb7705a3c, 34 },
    { 0x64905a3e, 90 },
    { 0xa8305a3f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93705a48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45605a4c, 66 },
    { 0x2c505a4c, 46 },
    { 0x7505a4e, 19 },
    { 0, 0 },
    { 0x25a05a50, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85205a54, 23 },
    { 0, 0 },
    { 0x5a56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f105a59, 98 },
    { 0x5df05a5a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e05a5d, 95 },
    { 0xad705a5e, 81 },
    { 0x83e05a5d, 90 },
    { 0x81905a60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb805a64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d805a6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0105a6e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b05a72, 52 },
    { 0, 0 },
    { 0xf9405a74, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3705a77, 23 },
    { 0xfc805a78, 40 },
    { 0x6c05a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd405a7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a05a80, 100 },
    { 0xd9b05a81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87205a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5205a87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe05a8a, 61 },
    { 0, 0 },
    { 0x75805a8c, 61 },
    { 0x6b05a8d, 52 },
    { 0x12c05a8d, 33 },
    { 0x98005a8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8105a93, 78 },
    { 0x5d805a94, 34 },
    { 0x2f05a94, 87 },
    { 0xd4e05a96, 33 },
    { 0, 0 },
    { 0x1c805a98, 55 },
    { 0, 0 },
    { 0xb3305a9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xab705a9d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9305aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9005aa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb105aa9, 43 },
    { 0x77305aaa, 76 },
    { 0xb7005aab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf05ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9205ab6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e205ab9, 61 },
    { 0x56f05aba, 95 },
    { 0, 0 },
    { 0x2ad05abc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db05abf, 88 },
    { 0x22205ac0, 33 },
    { 0, 0 },
    { 0xaae05ac2, 90 },
    { 0, 0 },
    { 0x9e605ac4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3305aca, 95 },
    { 0, 0 },
    { 0xd4c05acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c05ad0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd205ad4, 90 },
    { 0, 0 },
    { 0xb4505ad6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8605adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f05ae0, 86 },
    { 0x49505ae1, 78 },
    { 0x36505ae2, 33 },
    { 0xaca05ae3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b05aec, 90 },
    { 0x91405aec, 90 },
    { 0xc1005aee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a05af3, 90 },
    { 0, 0 },
    { 0x5ca05af5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58305afa, 85 },
    { 0x5c605afb, 93 },
    { 0, 0 },
    { 0x54605afd, 23 },
    { 0, 0 },
    { 0x9e405aff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b05b0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb05b12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d05b17, 90 },
    { 0x13b05b18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x98905b1b, 76 },
    { 0x37f05b1b, 61 },
    { 0xa8e05b1d, 95 },
    { 0xb0005b1e, 87 },
    { 0xb2c05b1e, 33 },
    { 0x74b05b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a05b30, 33 },
    { 0, 0 },
    { 0x44105b32, 33 },
    { 0x7d905b33, 55 },
    { 0, 0 },
    { 0x805b35, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c805b38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4705b3b, 90 },
    { 0xda305b3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5305b3f, 85 },
    { 0xea105b40, 95 },
    { 0, 0 },
    { 0x59e05b42, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7505b4a, 91 },
    { 0x8ac05b4b, 61 },
    { 0x6205b4c, 75 },
    { 0xa9205b4d, 85 },
    { 0x33205b4e, 90 },
    { 0x2d805b4e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad05b52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d005b56, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c305b59, 90 },
    { 0, 0 },
    { 0x6fd05b5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c405b5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6705b63, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a05b66, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee05b6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71805b72, 100 },
    { 0, 0 },
    { 0x70105b74, 91 },
    { 0, 0 },
    { 0x39a05b76, 90 },
    { 0, 0 },
    { 0xd1805b78, 61 },
    { 0, 0 },
    { 0x57e05b7a, 33 },
    { 0x58805b7b, 90 },
    { 0x2d105b7c, 33 },
    { 0xd505b7d, 90 },
    { 0, 0 },
    { 0xdc705b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22205b82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef05b87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e05b8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce05b8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5505b92, 61 },
    { 0xb105b93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b05b96, 43 },
    { 0xab05b97, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905b9b, 68 },
    { 0x19205b9c, 90 },
    { 0xb8905b9d, 95 },
    { 0xca905b9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26205ba5, 17 },
    { 0x59305ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da05ba9, 33 },
    { 0, 0 },
    { 0x40a05bab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96805bae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc05bb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86805bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d05bc2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d05bc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f05bc9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10805bce, 67 },
    { 0x97a05bcf, 33 },
    { 0x41c05bcf, 80 },
    { 0xe705bd1, 69 },
    { 0x24405bd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d105bd7, 53 },
    { 0x4ed05bd8, 90 },
    { 0x4ec05bd9, 34 },
    { 0x1e205bda, 90 },
    { 0xc2b05bdb, 98 },
    { 0x61505bdc, 99 },
    { 0x48205bdc, 95 },
    { 0, 0 },
    { 0xd705bdf, 34 },
    { 0, 0 },
    { 0x40005be1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a405be4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62805bea, 87 },
    { 0x3dd05bea, 33 },
    { 0x4e105bea, 66 },
    { 0x5be05bed, 61 },
    { 0x63005bee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x86805bf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51405bf4, 95 },
    { 0x87905bf5, 90 },
    { 0xb5405bf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c05bf9, 78 },
    { 0x9e305bfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de05bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c705c00, 76 },
    { 0x60805c01, 61 },
    { 0xccd05c02, 33 },
    { 0, 0 },
    { 0x1f505c04, 33 },
    { 0x9c05c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe905c09, 51 },
    { 0x7fa05c0a, 95 },
    { 0, 0 },
    { 0x10e05c0c, 64 },
    { 0, 0 },
    { 0x9d105c0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad05c1a, 66 },
    { 0x605c1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a05c24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d05c2e, 76 },
    { 0, 0 },
    { 0x6a305c30, 85 },
    { 0xea005c31, 61 },
    { 0x31e05c32, 34 },
    { 0xe4f05c30, 23 },
    { 0xa9205c32, 90 },
    { 0x49005c35, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac05c3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f05c3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5805c47, 87 },
    { 0xf6005c47, 91 },
    { 0, 0 },
    { 0x87805c4a, 43 },
    { 0, 0 },
    { 0x5d805c4c, 67 },
    { 0xe7905c4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde05c53, 98 },
    { 0xe8705c54, 90 },
    { 0xfe005c55, 77 },
    { 0, 0 },
    { 0xd305c57, 43 },
    { 0, 0 },
    { 0xd7905c59, 90 },
    { 0xb2805c5a, 98 },
    { 0xa3d05c5b, 61 },
    { 0x4b505c5c, 95 },
    { 0, 0 },
    { 0x5d605c5e, 98 },
    { 0xce405c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f05c66, 81 },
    { 0, 0 },
    { 0x28305c68, 33 },
    { 0, 0 },
    { 0x50d05c6a, 43 },
    { 0xb1205c6b, 33 },
    { 0x51205c6b, 23 },
    { 0xf0f05c6d, 43 },
    { 0x94705c6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb05c74, 61 },
    { 0x54705c74, 33 },
    { 0, 0 },
    { 0x38705c77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc05c7a, 95 },
    { 0xa9705c7b, 93 },
    { 0x96d05c7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c05c80, 46 },
    { 0xccf05c81, 90 },
    { 0x6b05c82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff05c85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9705c8a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c05c8d, 55 },
    { 0x55005c8e, 78 },
    { 0x86f05c8f, 77 },
    { 0x10205c8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac05c99, 61 },
    { 0x6cf05c9a, 78 },
    { 0xb7a05c9a, 95 },
    { 0x91e05c99, 53 },
    { 0, 0 },
    { 0x35805c9e, 17 },
    { 0x2d05c9f, 61 },
    { 0x84505ca0, 33 },
    { 0x8c905ca1, 34 },
    { 0x90905ca2, 61 },
    { 0, 0 },
    { 0x43205ca4, 85 },
    { 0x2a905ca5, 33 },
    { 0xf5b05ca6, 95 },
    { 0x46c05ca5, 55 },
    { 0x22a05ca8, 61 },
    { 0, 0 },
    { 0xa1e05caa, 93 },
    { 0, 0 },
    { 0x63405cac, 55 },
    { 0x7ab05cad, 76 },
    { 0xb0005cae, 99 },
    { 0x94b05cad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e05cb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae05cb8, 61 },
    { 0x9f005cb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb05cbe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16305cc2, 93 },
    { 0, 0 },
    { 0x13b05cc4, 95 },
    { 0, 0 },
    { 0x25a05cc6, 33 },
    { 0x4dc05cc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75705cca, 90 },
    { 0xff405ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61005cce, 90 },
    { 0xfc405ccf, 55 },
    { 0xf1705ccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e05cd3, 23 },
    { 0x28105cd3, 100 },
    { 0xdb105cd5, 90 },
    { 0x11c05cd3, 98 },
    { 0x3c705cd6, 95 },
    { 0xb2b05cd7, 76 },
    { 0xaa205cd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed05cdf, 33 },
    { 0x26905cdf, 95 },
    { 0x8cf05ce0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16905ce5, 95 },
    { 0xbe905ce5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d05cee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e105cf4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a05cf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d05cfe, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9305d04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78705d08, 78 },
    { 0xb205d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6205d0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd05d12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a305d17, 43 },
    { 0xb6005d18, 100 },
    { 0, 0 },
    { 0xfc005d1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd05d1f, 52 },
    { 0x30205d20, 55 },
    { 0x73205d1f, 95 },
    { 0x89905d22, 90 },
    { 0xca105d20, 98 },
    { 0x41c05d20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x85405d27, 85 },
    { 0, 0 },
    { 0xdf005d29, 43 },
    { 0x69c05d2a, 33 },
    { 0xdb005d2b, 90 },
    { 0, 0 },
    { 0x89f05d2d, 61 },
    { 0xd6205d2e, 61 },
    { 0, 0 },
    { 0x49c05d30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54605d34, 93 },
    { 0x32a05d35, 23 },
    { 0xe9405d36, 43 },
    { 0xd8e05d37, 90 },
    { 0x88305d38, 99 },
    { 0x4b005d39, 46 },
    { 0xa8805d3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14505d3e, 95 },
    { 0, 0 },
    { 0xa7105d40, 84 },
    { 0xa6505d40, 90 },
    { 0, 0 },
    { 0x8bf05d43, 85 },
    { 0, 0 },
    { 0xc705d45, 76 },
    { 0, 0 },
    { 0x7ba05d47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb605d4a, 78 },
    { 0, 0 },
    { 0xf0005d4c, 23 },
    { 0, 0 },
    { 0xb9705d4e, 66 },
    { 0, 0 },
    { 0x86605d50, 90 },
    { 0, 0 },
    { 0xe4a05d52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51705d55, 61 },
    { 0x1c405d56, 23 },
    { 0x81005d57, 43 },
    { 0x26705d58, 93 },
    { 0x86c05d58, 55 },
    { 0, 0 },
    { 0xa7205d5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e05d61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e05d68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95805d6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a805d71, 95 },
    { 0, 0 },
    { 0x59105d73, 61 },
    { 0, 0 },
    { 0xd05d75, 68 },
    { 0x69205d76, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49005d7a, 90 },
    { 0, 0 },
    { 0x1005d7c, 98 },
    { 0x28005d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f05d80, 88 },
    { 0x7ee05d81, 91 },
    { 0x21205d82, 98 },
    { 0xe6d05d83, 33 },
    { 0, 0 },
    { 0xe5d05d85, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50505d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c05d8c, 33 },
    { 0xacb05d8d, 98 },
    { 0xbe805d8e, 55 },
    { 0xb6b05d8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d05d93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x43605d96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d05d9b, 90 },
    { 0, 0 },
    { 0x43805d9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a05da5, 90 },
    { 0x53a05da5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2705dab, 77 },
    { 0, 0 },
    { 0x1c405dad, 34 },
    { 0xd1f05dad, 33 },
    { 0x4fc05daf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce05db6, 33 },
    { 0xc0e05db7, 87 },
    { 0x91c05db8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb05dc2, 55 },
    { 0x71b05dc3, 33 },
    { 0xb8505dc4, 85 },
    { 0x6f905dc3, 94 },
    { 0x60205dc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b905dcc, 90 },
    { 0x7cf05dcd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab05dd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf905dd6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13405dde, 23 },
    { 0x76405dde, 90 },
    { 0x4f405de0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb305de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b605ded, 98 },
    { 0xf6605ded, 95 },
    { 0, 0 },
    { 0xe3905df0, 97 },
    { 0x7f905df0, 98 },
    { 0, 0 },
    { 0x18105df3, 61 },
    { 0x9cd05df3, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd05df8, 90 },
    { 0x82205df9, 99 },
    { 0xe9005df8, 90 },
    { 0, 0 },
    { 0x51c05dfc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba05e03, 98 },
    { 0, 0 },
    { 0x2105e05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f05e09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa05e11, 98 },
    { 0xa5005e12, 95 },
    { 0x3f05e13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e405e1b, 85 },
    { 0x99205e1c, 52 },
    { 0x13305e1c, 81 },
    { 0x48b05e1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2605e26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac105e29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe605e2d, 33 },
    { 0xbaa05e2e, 77 },
    { 0xb7705e2d, 52 },
    { 0xd1805e30, 76 },
    { 0x69405e2e, 33 },
    { 0x8c505e2f, 87 },
    { 0x24a05e30, 33 },
    { 0x705e32, 66 },
    { 0x56e05e30, 97 },
    { 0xb2305e33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64105e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5105e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb105e47, 46 },
    { 0xc1605e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd05e4b, 88 },
    { 0x99d05e4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xee505e4f, 61 },
    { 0x95d05e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb05e54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca05e57, 99 },
    { 0x82905e58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17105e5c, 33 },
    { 0x1db05e5d, 95 },
    { 0xedf05e5e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c05e61, 91 },
    { 0, 0 },
    { 0x33e05e63, 93 },
    { 0x47905e64, 85 },
    { 0x4ff05e65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea05e6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf205e6d, 33 },
    { 0x3a305e6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1905e73, 52 },
    { 0xc1b05e74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c605e7a, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0x83205e7d, 23 },
    { 0, 0 },
    { 0xbfc05e7f, 95 },
    { 0, 0 },
    { 0x92005e81, 33 },
    { 0, 0 },
    { 0x56805e83, 90 },
    { 0x99105e84, 61 },
    { 0x84c05e83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe05e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de05e8c, 61 },
    { 0x45305e8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9505e91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab05e95, 61 },
    { 0x5f505e95, 55 },
    { 0, 0 },
    { 0xdc005e98, 34 },
    { 0, 0 },
    { 0xa8805e9a, 90 },
    { 0x85c05e9b, 23 },
    { 0, 0 },
    { 0xd6b05e9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81305ea4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e605eaa, 33 },
    { 0, 0 },
    { 0x4c605eac, 90 },
    { 0x97f05ead, 97 },
    { 0xde605eae, 33 },
    { 0xb8305eae, 88 },
    { 0xa1305eac, 43 },
    { 0xe1105eb1, 55 },
    { 0x4c005eb2, 33 },
    { 0x55c05eae, 61 },
    { 0xd8105eb4, 61 },
    { 0x47105eb5, 95 },
    { 0, 0 },
    { 0x6dc05eb7, 61 },
    { 0x3d205eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b05ebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d05ec0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba05ec4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff05ec8, 100 },
    { 0xf505ec8, 23 },
    { 0, 0 },
    { 0x4e405ecb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b805ecf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa005ed2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c605ed8, 61 },
    { 0xa1a05ed9, 33 },
    { 0x3c605eda, 23 },
    { 0x65105edb, 78 },
    { 0xe8f05edc, 45 },
    { 0x5c905edd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c05ee5, 99 },
    { 0xd5005ee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2405ee9, 90 },
    { 0x98205ee9, 75 },
    { 0, 0 },
    { 0x70005eec, 66 },
    { 0xfcf05eec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e005ef0, 61 },
    { 0, 0 },
    { 0x76e05ef2, 90 },
    { 0x1005ef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e05f04, 90 },
    { 0x7b205f05, 33 },
    { 0xc6b05f06, 65 },
    { 0x7e05f06, 33 },
    { 0x2c605f08, 98 },
    { 0xa7a05f08, 90 },
    { 0xf9f05f0a, 76 },
    { 0x89d05f0b, 90 },
    { 0x7a005f0b, 23 },
    { 0, 0 },
    { 0xef105f0e, 23 },
    { 0x85405f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf705f12, 95 },
    { 0xb9a05f12, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac05f19, 76 },
    { 0xabe05f1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35105f1d, 23 },
    { 0x5a905f1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d705f26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec05f35, 56 },
    { 0, 0 },
    { 0xcdb05f37, 33 },
    { 0x7e405f38, 90 },
    { 0xc0d05f39, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f05f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4205f48, 61 },
    { 0xc7405f49, 34 },
    { 0x40d05f4a, 33 },
    { 0, 0 },
    { 0xf0905f4c, 36 },
    { 0x1005f4c, 96 },
    { 0x6bb05f4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd005f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab05f55, 87 },
    { 0, 0 },
    { 0x82005f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4205f5c, 52 },
    { 0xa3305f5d, 61 },
    { 0x47305f5e, 52 },
    { 0, 0 },
    { 0x5be05f60, 33 },
    { 0xb9f05f60, 33 },
    { 0x7f605f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e05f66, 61 },
    { 0x95005f67, 55 },
    { 0, 0 },
    { 0x33705f69, 56 },
    { 0x8de05f69, 91 },
    { 0x93105f6b, 65 },
    { 0xfc05f69, 23 },
    { 0, 0 },
    { 0x2eb05f6e, 90 },
    { 0, 0 },
    { 0x2d705f70, 33 },
    { 0x13a05f70, 61 },
    { 0x19b05f72, 95 },
    { 0x7b305f73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad205f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe305f7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3405f8a, 100 },
    { 0xa4705f8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f05f8f, 61 },
    { 0x34605f8f, 61 },
    { 0, 0 },
    { 0xefa05f92, 67 },
    { 0, 0 },
    { 0xa8405f94, 91 },
    { 0xe6b05f95, 34 },
    { 0, 0 },
    { 0x7ec05f97, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2005f9c, 33 },
    { 0, 0 },
    { 0x7eb05f9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4805fa5, 61 },
    { 0, 0 },
    { 0xca805fa7, 81 },
    { 0x3da05fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4105fac, 43 },
    { 0xb6e05fad, 90 },
    { 0, 0 },
    { 0x9fe05faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde805fb2, 93 },
    { 0xaf105fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc605fb6, 23 },
    { 0xef505fb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d05fba, 66 },
    { 0x14e05fbb, 95 },
    { 0xd0805fbc, 99 },
    { 0x86105fbd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a05fc1, 61 },
    { 0, 0 },
    { 0x705fc3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8605fc6, 52 },
    { 0, 0 },
    { 0xa7905fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b205fcb, 61 },
    { 0x15005fcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d05fdc, 33 },
    { 0x8cc05fdd, 23 },
    { 0x19c05fdd, 34 },
    { 0x7bd05fdf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda05fe3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c05fe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db05fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5805ff2, 67 },
    { 0, 0 },
    { 0x79105ff4, 90 },
    { 0, 0 },
    { 0x7c205ff6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12206001, 90 },
    { 0, 0 },
    { 0x4b706003, 61 },
    { 0x35506004, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0600a, 66 },
    { 0, 0 },
    { 0x3770600c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda06010, 98 },
    { 0xcf006010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d06014, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d06017, 61 },
    { 0xbed06018, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f60601c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b006021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1906025, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa670602c, 33 },
    { 0xfee0602d, 23 },
    { 0, 0 },
    { 0x5330602f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc06035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e06039, 90 },
    { 0, 0 },
    { 0x22d0603b, 33 },
    { 0xc7b0603c, 34 },
    { 0x5930603c, 33 },
    { 0x5760603e, 88 },
    { 0, 0 },
    { 0x10e06040, 78 },
    { 0x82506041, 33 },
    { 0x66906042, 64 },
    { 0xf6806042, 23 },
    { 0, 0 },
    { 0xf8906045, 23 },
    { 0, 0 },
    { 0xc9c06047, 23 },
    { 0xfd06047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110604f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79406058, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0605c, 52 },
    { 0, 0 },
    { 0xf530605e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f06061, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc06067, 43 },
    { 0x22b06067, 23 },
    { 0x32a06069, 95 },
    { 0x18c06069, 90 },
    { 0xc4e0606b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c0606f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d06072, 66 },
    { 0x93706073, 76 },
    { 0xe7c06074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c406079, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc160607d, 87 },
    { 0xa530607e, 28 },
    { 0x8700607d, 95 },
    { 0x67506080, 55 },
    { 0, 0 },
    { 0x60306082, 33 },
    { 0, 0 },
    { 0x90d06084, 76 },
    { 0xe5206085, 76 },
    { 0, 0 },
    { 0x45306087, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71006090, 91 },
    { 0, 0 },
    { 0x84b06092, 99 },
    { 0, 0 },
    { 0x76306094, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b00609d, 95 },
    { 0x7c20609d, 98 },
    { 0x8f00609f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d060a3, 61 },
    { 0xfea060a3, 61 },
    { 0x45e060a5, 98 },
    { 0xae7060a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc060a9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51060ae, 94 },
    { 0x448060af, 43 },
    { 0, 0 },
    { 0x2cf060b1, 90 },
    { 0x845060b2, 43 },
    { 0xf96060b3, 34 },
    { 0xf1e060b1, 61 },
    { 0x18b060b5, 93 },
    { 0xfac060b6, 84 },
    { 0x83e060b6, 75 },
    { 0x82b060b7, 98 },
    { 0x9dc060b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628060be, 87 },
    { 0, 0 },
    { 0xfbf060c0, 84 },
    { 0x5d1060c0, 90 },
    { 0xd40060c2, 90 },
    { 0, 0 },
    { 0xb1a060c4, 61 },
    { 0x359060c5, 33 },
    { 0xced060c5, 34 },
    { 0xe3e060c6, 90 },
    { 0x2c5060c8, 95 },
    { 0, 0 },
    { 0x10c060ca, 38 },
    { 0, 0 },
    { 0xdfd060cc, 61 },
    { 0xd39060cd, 61 },
    { 0xac7060ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x793060d1, 34 },
    { 0x77e060d2, 93 },
    { 0x94c060d3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1060d6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5060db, 76 },
    { 0xc4f060dc, 98 },
    { 0xed1060dd, 34 },
    { 0, 0 },
    { 0xfed060df, 98 },
    { 0x616060e0, 90 },
    { 0xaa4060e1, 55 },
    { 0x420060e1, 33 },
    { 0xe32060e3, 61 },
    { 0x532060e4, 95 },
    { 0, 0 },
    { 0xfa9060e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab060ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d060ed, 33 },
    { 0x266060ee, 91 },
    { 0, 0 },
    { 0x61e060f0, 52 },
    { 0xe1060f1, 81 },
    { 0, 0 },
    { 0x6f7060f3, 61 },
    { 0, 0 },
    { 0xda4060f5, 95 },
    { 0x7f2060f6, 93 },
    { 0, 0 },
    { 0xed060f8, 90 },
    { 0x889060f8, 98 },
    { 0xfe7060fa, 61 },
    { 0xc5b060f8, 90 },
    { 0x43d060f8, 33 },
    { 0xa55060f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb06101, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b06105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58106108, 61 },
    { 0x1e006109, 33 },
    { 0x4f50610a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8206110, 61 },
    { 0, 0 },
    { 0xb8906112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8906118, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad0611c, 85 },
    { 0, 0 },
    { 0x5b80611e, 66 },
    { 0, 0 },
    { 0x71806120, 84 },
    { 0xeae06121, 98 },
    { 0xd8206121, 23 },
    { 0x33c06122, 90 },
    { 0x9cb06123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e906127, 88 },
    { 0xdee06128, 49 },
    { 0xf4106129, 78 },
    { 0xe7f0612a, 34 },
    { 0x2f00612b, 90 },
    { 0xc1f0612c, 92 },
    { 0x26c0612d, 90 },
    { 0x43a0612d, 33 },
    { 0x340612f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95406133, 95 },
    { 0, 0 },
    { 0x1ff06135, 33 },
    { 0xf3006135, 64 },
    { 0x59606137, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0613d, 91 },
    { 0x59f0613e, 76 },
    { 0, 0 },
    { 0x5c406140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65806145, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f0614a, 34 },
    { 0, 0 },
    { 0xfcb0614c, 34 },
    { 0xfd00614d, 44 },
    { 0, 0 },
    { 0xac00614f, 90 },
    { 0xdab06150, 61 },
    { 0xdce06150, 64 },
    { 0xfa00614f, 87 },
    { 0x46306153, 59 },
    { 0xa506154, 33 },
    { 0x82106155, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7780615e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d06161, 90 },
    { 0xb9206161, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7130616a, 61 },
    { 0xedf0616b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17006174, 75 },
    { 0x11b06175, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad0617f, 95 },
    { 0x4c806180, 76 },
    { 0x76706181, 43 },
    { 0xe3706182, 34 },
    { 0x57a06182, 98 },
    { 0xc5e06183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a06187, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc0618c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f70618f, 40 },
    { 0x9a00618f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4406194, 33 },
    { 0x4e006195, 84 },
    { 0xbb06195, 98 },
    { 0x40306197, 33 },
    { 0, 0 },
    { 0xc1406199, 77 },
    { 0x18a06199, 95 },
    { 0, 0 },
    { 0xa990619c, 97 },
    { 0x5960619c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0061a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e061a5, 95 },
    { 0xb29061a6, 61 },
    { 0xbb6061a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x270061aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7061ae, 33 },
    { 0x695061af, 68 },
    { 0x9da061b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43061b4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac061b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef061bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x265061c0, 76 },
    { 0x507061c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17061c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9061ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee061d5, 43 },
    { 0xace061d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a061da, 90 },
    { 0xdf4061da, 90 },
    { 0x60a061dc, 85 },
    { 0x830061dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58061e2, 34 },
    { 0x61f061e3, 52 },
    { 0xea7061e4, 98 },
    { 0, 0 },
    { 0x997061e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6061ea, 93 },
    { 0, 0 },
    { 0xbe1061ec, 87 },
    { 0xf9a061ec, 33 },
    { 0x4f5061ec, 42 },
    { 0x60061ef, 34 },
    { 0xc61061f0, 78 },
    { 0x9f9061f1, 90 },
    { 0xcef061f2, 34 },
    { 0x4ba061f1, 90 },
    { 0, 0 },
    { 0xe03061f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886061ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4306203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79406206, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36706209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9906210, 90 },
    { 0x4ab06211, 98 },
    { 0x31b06212, 23 },
    { 0x9bb06212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f06216, 81 },
    { 0x51406217, 90 },
    { 0xe7f06217, 23 },
    { 0, 0 },
    { 0xf670621a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4980621f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f906223, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d06228, 23 },
    { 0xe0706229, 77 },
    { 0x7160622a, 98 },
    { 0xb1d0622a, 98 },
    { 0x1060622a, 43 },
    { 0x420622c, 23 },
    { 0x2600622e, 90 },
    { 0x83d0622b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f06232, 61 },
    { 0, 0 },
    { 0xfd506234, 99 },
    { 0xae806234, 61 },
    { 0xab206234, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c106239, 23 },
    { 0x11506239, 23 },
    { 0xe6e0623b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a06240, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e506243, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d006246, 93 },
    { 0, 0 },
    { 0x89306248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5680624d, 76 },
    { 0xad50624e, 95 },
    { 0xfa10624f, 66 },
    { 0x50f0624e, 55 },
    { 0x7ef06250, 43 },
    { 0x1eb06251, 90 },
    { 0x5e20624e, 66 },
    { 0xfe60624d, 81 },
    { 0xd1506255, 81 },
    { 0xdea06250, 55 },
    { 0xb1d06251, 23 },
    { 0, 0 },
    { 0x79806259, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe320625c, 99 },
    { 0x5270625d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c06260, 90 },
    { 0, 0 },
    { 0xe2606262, 90 },
    { 0xd7706263, 61 },
    { 0xd4806264, 85 },
    { 0xbf606265, 85 },
    { 0x8f06266, 34 },
    { 0xc8b06267, 96 },
    { 0x17a06265, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0626d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d06271, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9706274, 33 },
    { 0, 0 },
    { 0x4ed06276, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f80627d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7806283, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f606286, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff0628a, 77 },
    { 0, 0 },
    { 0x3060628c, 90 },
    { 0xa2a0628d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c06293, 90 },
    { 0, 0 },
    { 0xf8806295, 55 },
    { 0, 0 },
    { 0x96d06297, 77 },
    { 0, 0 },
    { 0x9ff06299, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f0629c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8790629f, 100 },
    { 0x9b6062a0, 33 },
    { 0x41c062a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52062a5, 99 },
    { 0, 0 },
    { 0xcc6062a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321062ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1062b5, 98 },
    { 0x77d062b5, 81 },
    { 0x5b0062b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee062bd, 61 },
    { 0xd98062bd, 98 },
    { 0x1d7062bf, 98 },
    { 0, 0 },
    { 0x363062c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8062c4, 95 },
    { 0x6f7062c4, 33 },
    { 0, 0 },
    { 0x15a062c7, 78 },
    { 0x6e6062c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27062d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4062d6, 76 },
    { 0x2dc062d7, 52 },
    { 0xb4a062d7, 76 },
    { 0, 0 },
    { 0x1a0062da, 36 },
    { 0x942062da, 46 },
    { 0xe0d062dc, 98 },
    { 0xef6062dd, 90 },
    { 0x1da062dd, 55 },
    { 0x3f9062df, 98 },
    { 0x5bc062e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45062e3, 84 },
    { 0x572062e4, 90 },
    { 0xe0062e5, 38 },
    { 0xe34062e5, 66 },
    { 0x8a8062e6, 87 },
    { 0x598062e5, 90 },
    { 0x873062e9, 61 },
    { 0x665062e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6062f0, 76 },
    { 0xcc9062f0, 98 },
    { 0xa7e062f2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e062f6, 23 },
    { 0x7f4062f7, 93 },
    { 0xb9d062f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x758062fb, 61 },
    { 0x3b8062fc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37606300, 95 },
    { 0x6e106301, 90 },
    { 0, 0 },
    { 0xaef06303, 87 },
    { 0x18206304, 88 },
    { 0x3ea06304, 87 },
    { 0, 0 },
    { 0xd1d06307, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb0630e, 90 },
    { 0x26a0630f, 90 },
    { 0xa490630e, 33 },
    { 0x8706310, 98 },
    { 0xd0206312, 98 },
    { 0x12806313, 95 },
    { 0xbf406313, 98 },
    { 0xdeb0630e, 33 },
    { 0, 0 },
    { 0xa3006317, 90 },
    { 0, 0 },
    { 0xc8906319, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5760631f, 95 },
    { 0, 0 },
    { 0x99706321, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c0632a, 33 },
    { 0xf5f0632b, 61 },
    { 0x8400632c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c06332, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa06336, 77 },
    { 0x3f606337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d0633a, 66 },
    { 0, 0 },
    { 0x11d0633c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc06340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76906344, 38 },
    { 0xcb906344, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b06348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb050634b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb660634f, 52 },
    { 0, 0 },
    { 0x9b406351, 23 },
    { 0, 0 },
    { 0xf4f06353, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f06357, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e0635a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b06362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1606365, 34 },
    { 0xf8d06365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec906374, 40 },
    { 0x89206375, 100 },
    { 0x57106376, 49 },
    { 0xf9806377, 23 },
    { 0xff806378, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a0637c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b06381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5706388, 90 },
    { 0x6b706389, 94 },
    { 0, 0 },
    { 0xbac0638b, 98 },
    { 0, 0 },
    { 0x7240638d, 90 },
    { 0, 0 },
    { 0xa570638f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d106398, 99 },
    { 0x55a06399, 76 },
    { 0, 0 },
    { 0xd830639b, 84 },
    { 0, 0 },
    { 0xbee0639d, 98 },
    { 0x7de0639e, 97 },
    { 0, 0 },
    { 0xe063a0, 69 },
    { 0x2e8063a0, 33 },
    { 0xdcc063a0, 67 },
    { 0x187063a3, 43 },
    { 0x654063a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x381063a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50063aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4063af, 23 },
    { 0, 0 },
    { 0xf37063b1, 93 },
    { 0xf29063b1, 90 },
    { 0, 0 },
    { 0xb64063b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77063b7, 46 },
    { 0, 0 },
    { 0x792063b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c063bc, 34 },
    { 0, 0 },
    { 0xb23063be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4063c5, 55 },
    { 0, 0 },
    { 0x11f063c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17063cd, 33 },
    { 0x272063cd, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x751063d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28063d4, 95 },
    { 0x9ca063d4, 61 },
    { 0xfca063d6, 90 },
    { 0xc4a063d6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e063dc, 90 },
    { 0, 0 },
    { 0x229063de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634063e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6063e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd063ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802063f3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675063f7, 33 },
    { 0xf67063f8, 95 },
    { 0xbc063f9, 33 },
    { 0xa5063fa, 33 },
    { 0, 0 },
    { 0x8fe063fc, 46 },
    { 0xd04063fd, 23 },
    { 0xf83063fd, 80 },
    { 0xb2c063ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9006406, 68 },
    { 0x3ba06407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540640a, 33 },
    { 0x4900640b, 61 },
    { 0x5150640c, 76 },
    { 0xd4d0640b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b206416, 83 },
    { 0xf7306416, 33 },
    { 0x22306416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe980641c, 33 },
    { 0x9230641d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac606422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac06425, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d206428, 61 },
    { 0x2bd06428, 33 },
    { 0x1b10642a, 76 },
    { 0x89a0642a, 95 },
    { 0xe360642b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd410642f, 95 },
    { 0xf9c06430, 66 },
    { 0x21e06430, 98 },
    { 0xc1906432, 55 },
    { 0xc4906432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75706438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b06443, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06446, 91 },
    { 0x7af06446, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3650644d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa006452, 91 },
    { 0, 0 },
    { 0xa0606454, 77 },
    { 0xd1d06455, 90 },
    { 0x6be06456, 55 },
    { 0, 0 },
    { 0xad906458, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b0645e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9006462, 66 },
    { 0x23906463, 90 },
    { 0, 0 },
    { 0x5b006465, 56 },
    { 0x6c006466, 91 },
    { 0x33f06465, 90 },
    { 0x5006468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f0646b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46006471, 76 },
    { 0x89206471, 33 },
    { 0x3ef06473, 95 },
    { 0x71d06474, 61 },
    { 0x4c106474, 46 },
    { 0x13d06475, 90 },
    { 0x44b06473, 33 },
    { 0xa5006478, 95 },
    { 0xa6006478, 67 },
    { 0x59a0647a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0647e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x24706481, 33 },
    { 0x48a06481, 83 },
    { 0x2cb06483, 90 },
    { 0x4ea06483, 46 },
    { 0x37706485, 61 },
    { 0x5a906486, 98 },
    { 0xaf106486, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a06491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba06495, 99 },
    { 0xa7306496, 95 },
    { 0, 0 },
    { 0x83f06498, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb0649b, 87 },
    { 0x3120649c, 76 },
    { 0, 0 },
    { 0xd7d0649e, 23 },
    { 0xeaf0649e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d064a5, 34 },
    { 0x6d3064a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48064a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a064b0, 90 },
    { 0xeaf064b1, 61 },
    { 0xd6d064b0, 83 },
    { 0, 0 },
    { 0x8f1064b4, 90 },
    { 0x4c5064b5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc064bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee064c1, 33 },
    { 0x8e9064c2, 93 },
    { 0xc2d064c2, 87 },
    { 0x166064c1, 76 },
    { 0x26a064c5, 34 },
    { 0xc064c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea064cb, 84 },
    { 0x6db064cc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631064d3, 61 },
    { 0xa65064d4, 61 },
    { 0x60a064d3, 43 },
    { 0x870064d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288064dc, 87 },
    { 0x279064dc, 95 },
    { 0, 0 },
    { 0xafa064df, 80 },
    { 0xc95064df, 90 },
    { 0, 0 },
    { 0x17d064e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6064e7, 34 },
    { 0x961064e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904064ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39064f2, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73064f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b064fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x343064fe, 76 },
    { 0x265064ff, 43 },
    { 0x89c06500, 77 },
    { 0, 0 },
    { 0x28906502, 78 },
    { 0x4da06503, 95 },
    { 0x2cd06504, 98 },
    { 0xe0806504, 55 },
    { 0, 0 },
    { 0xf0606507, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b40650b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1310650f, 87 },
    { 0x3706510, 23 },
    { 0, 0 },
    { 0x8d606512, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3906515, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d306519, 33 },
    { 0xfbb0651a, 87 },
    { 0xcb40651b, 66 },
    { 0xdd80651c, 91 },
    { 0xd100651a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e506520, 33 },
    { 0xd2e06521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32306524, 100 },
    { 0, 0 },
    { 0xb1706526, 95 },
    { 0xfd606527, 33 },
    { 0x30006527, 61 },
    { 0, 0 },
    { 0xc230652a, 98 },
    { 0, 0 },
    { 0xae60652c, 90 },
    { 0x8a00652c, 33 },
    { 0xa5e0652e, 61 },
    { 0, 0 },
    { 0xb7006530, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15906535, 99 },
    { 0, 0 },
    { 0xa0706537, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb440653a, 75 },
    { 0xff40653b, 67 },
    { 0, 0 },
    { 0x9180653d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c06540, 33 },
    { 0, 0 },
    { 0x91c06542, 93 },
    { 0, 0 },
    { 0x7d206544, 40 },
    { 0xe8506545, 33 },
    { 0x50606545, 90 },
    { 0x62f06547, 98 },
    { 0xefb06548, 34 },
    { 0xf1006549, 87 },
    { 0xa5c06549, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff0654d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b106550, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf0655a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb90655d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd106561, 61 },
    { 0x69e06562, 66 },
    { 0xf5b06563, 23 },
    { 0xc6806564, 61 },
    { 0x6c006565, 23 },
    { 0x3bf06565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f06569, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb0656c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5240656f, 61 },
    { 0, 0 },
    { 0x60206571, 95 },
    { 0xf2b06572, 95 },
    { 0x86306573, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9006576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c0657b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca06581, 55 },
    { 0xc5106582, 32 },
    { 0, 0 },
    { 0xc4006584, 34 },
    { 0xbae06585, 52 },
    { 0xb0906585, 76 },
    { 0x8be06587, 76 },
    { 0x80a06587, 34 },
    { 0, 0 },
    { 0x2d60658a, 52 },
    { 0, 0 },
    { 0x7d0658c, 61 },
    { 0xa1f0658c, 33 },
    { 0x9130658e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b306594, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2606598, 85 },
    { 0x66206598, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0659e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba065a2, 98 },
    { 0, 0 },
    { 0x506065a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166065a9, 33 },
    { 0xc4b065aa, 46 },
    { 0, 0 },
    { 0x1a065ac, 76 },
    { 0, 0 },
    { 0xabf065ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2065b3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b065b6, 95 },
    { 0x7f8065b7, 86 },
    { 0x968065b6, 95 },
    { 0, 0 },
    { 0x6c1065ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e065be, 33 },
    { 0, 0 },
    { 0xef9065c0, 90 },
    { 0x335065c1, 98 },
    { 0x327065c1, 43 },
    { 0xb1c065c1, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799065ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378065d0, 33 },
    { 0, 0 },
    { 0x29a065d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43065d6, 90 },
    { 0, 0 },
    { 0x8bf065d8, 66 },
    { 0, 0 },
    { 0xd55065da, 56 },
    { 0xd9e065db, 81 },
    { 0, 0 },
    { 0x354065dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac065e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7065e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28065e6, 23 },
    { 0xf4b065e6, 80 },
    { 0x328065e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47065ee, 33 },
    { 0xbde065ee, 76 },
    { 0x964065f0, 23 },
    { 0x3f2065ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6065f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777065fa, 87 },
    { 0, 0 },
    { 0xe6f065fc, 61 },
    { 0x565065fd, 43 },
    { 0xf76065fe, 66 },
    { 0, 0 },
    { 0x3b006600, 75 },
    { 0, 0 },
    { 0x8706602, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88906607, 95 },
    { 0, 0 },
    { 0x86206609, 90 },
    { 0x5170660a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f006611, 33 },
    { 0xb7306612, 33 },
    { 0, 0 },
    { 0xe9a06614, 90 },
    { 0xfa906615, 92 },
    { 0x19906616, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d906619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4710661c, 90 },
    { 0xed60661c, 90 },
    { 0x1220661c, 51 },
    { 0, 0 },
    { 0x98d06620, 91 },
    { 0xef706621, 33 },
    { 0xd7906622, 90 },
    { 0x9e06623, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d006627, 66 },
    { 0xb4c06628, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc0662c, 98 },
    { 0, 0 },
    { 0x2ab0662e, 52 },
    { 0x60b0662f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f006634, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x54506637, 66 },
    { 0x62406637, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb90663b, 33 },
    { 0xd0c0663c, 33 },
    { 0x9310663d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f0664b, 95 },
    { 0x1970664c, 66 },
    { 0xf840664d, 97 },
    { 0, 0 },
    { 0x4910664f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc806654, 61 },
    { 0xb2c06655, 78 },
    { 0x56606655, 34 },
    { 0xc8206654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5970665c, 98 },
    { 0xcde0665d, 65 },
    { 0, 0 },
    { 0x4050665f, 76 },
    { 0, 0 },
    { 0x10e06661, 23 },
    { 0xca906662, 61 },
    { 0x50c06661, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d06666, 33 },
    { 0x34106667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d40666a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d506676, 91 },
    { 0x2af06677, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd0667b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa840667e, 78 },
    { 0, 0 },
    { 0xd7906680, 95 },
    { 0, 0 },
    { 0xcf06682, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2506688, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c50668b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e06693, 91 },
    { 0x9d106693, 33 },
    { 0, 0 },
    { 0xceb06696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c60669a, 61 },
    { 0, 0 },
    { 0xe120669c, 90 },
    { 0, 0 },
    { 0x48c0669e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33066a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47066aa, 85 },
    { 0xe5b066aa, 61 },
    { 0x865066ac, 61 },
    { 0xba4066ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6066b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28066b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa066ba, 67 },
    { 0x6b8066bb, 33 },
    { 0x3e0066ba, 33 },
    { 0x9ea066bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0066c9, 33 },
    { 0x7d9066ca, 95 },
    { 0x33a066ca, 52 },
    { 0x2f7066c9, 90 },
    { 0xe47066cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea066d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38066d6, 98 },
    { 0x260066d7, 93 },
    { 0x56f066d7, 33 },
    { 0, 0 },
    { 0x19d066da, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0066e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d066ec, 52 },
    { 0xffc066ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24066f0, 98 },
    { 0x27b066f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a066f6, 23 },
    { 0x1ce066f7, 34 },
    { 0x557066f8, 23 },
    { 0x45f066f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8066ff, 25 },
    { 0x28006700, 33 },
    { 0xa4d06700, 91 },
    { 0xae306702, 84 },
    { 0x4406702, 98 },
    { 0x70406700, 61 },
    { 0xd9a06702, 61 },
    { 0, 0 },
    { 0x32806707, 23 },
    { 0, 0 },
    { 0x48806709, 61 },
    { 0xc9506709, 90 },
    { 0x5330670b, 61 },
    { 0x77606709, 91 },
    { 0, 0 },
    { 0x5490670e, 90 },
    { 0x69e0670f, 75 },
    { 0x42806710, 33 },
    { 0x9f006711, 40 },
    { 0, 0 },
    { 0xddd06713, 91 },
    { 0xb8006714, 23 },
    { 0x34206714, 93 },
    { 0xbd206715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640671a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d06722, 52 },
    { 0x71306723, 61 },
    { 0x5b006724, 43 },
    { 0x92106725, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3520672d, 33 },
    { 0, 0 },
    { 0x6c80672f, 33 },
    { 0x2850672f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6706733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f106736, 33 },
    { 0x52906737, 17 },
    { 0xb6d06737, 61 },
    { 0xe8906739, 90 },
    { 0xefe06737, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b306741, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f106744, 98 },
    { 0x68906745, 90 },
    { 0xeb306745, 61 },
    { 0x7eb06747, 46 },
    { 0x3706748, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15006753, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49906757, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac0675a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df0675d, 90 },
    { 0xe100675e, 90 },
    { 0x8de0675f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b06762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe206768, 23 },
    { 0, 0 },
    { 0xca50676a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe360676f, 90 },
    { 0x8980676f, 56 },
    { 0x5d706771, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae806777, 76 },
    { 0x6dd06778, 90 },
    { 0x30306778, 76 },
    { 0xb4306779, 61 },
    { 0xf6d0677b, 77 },
    { 0xabf0677c, 61 },
    { 0x45c0677b, 90 },
    { 0, 0 },
    { 0x2e40677f, 84 },
    { 0, 0 },
    { 0x3bb06781, 98 },
    { 0, 0 },
    { 0x1e006783, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a0678d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a106791, 76 },
    { 0, 0 },
    { 0x7d206793, 33 },
    { 0xec906794, 91 },
    { 0xc4106795, 66 },
    { 0xd6406795, 90 },
    { 0xd406795, 100 },
    { 0, 0 },
    { 0xad706799, 61 },
    { 0, 0 },
    { 0x2360679b, 33 },
    { 0, 0 },
    { 0x1800679d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f067a1, 76 },
    { 0, 0 },
    { 0x17f067a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x541067a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27067a9, 33 },
    { 0x4067aa, 33 },
    { 0x8f6067aa, 85 },
    { 0x10c067ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x164067af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2067b6, 33 },
    { 0xd8f067b7, 61 },
    { 0x187067b7, 99 },
    { 0, 0 },
    { 0x622067ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82067bd, 90 },
    { 0x158067bd, 33 },
    { 0x3e2067bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80067c3, 33 },
    { 0, 0 },
    { 0xa61067c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x901067c8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2067ce, 33 },
    { 0x138067cf, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef067d2, 33 },
    { 0x868067d3, 52 },
    { 0, 0 },
    { 0xe7067d5, 90 },
    { 0x353067d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x341067d9, 52 },
    { 0, 0 },
    { 0xf5b067db, 90 },
    { 0, 0 },
    { 0xa71067dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9067e4, 98 },
    { 0x214067e5, 23 },
    { 0x4fc067e6, 61 },
    { 0, 0 },
    { 0x4f5067e8, 76 },
    { 0xaf7067e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e067ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c067f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c067f7, 76 },
    { 0xff6067f8, 95 },
    { 0x2ef067f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c706800, 33 },
    { 0x8b506800, 43 },
    { 0xd8006802, 91 },
    { 0, 0 },
    { 0x42806804, 76 },
    { 0, 0 },
    { 0x5ea06806, 23 },
    { 0x71006807, 81 },
    { 0xe8c06808, 80 },
    { 0, 0 },
    { 0xc640680a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb80680d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f06814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9406818, 91 },
    { 0x2c306818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac0681c, 75 },
    { 0, 0 },
    { 0xf2f0681e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf306822, 98 },
    { 0xea206823, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42606827, 34 },
    { 0x98606827, 98 },
    { 0x1206827, 98 },
    { 0, 0 },
    { 0x7d70682b, 43 },
    { 0, 0 },
    { 0x39d0682d, 78 },
    { 0xb8f0682e, 78 },
    { 0x52a0682e, 65 },
    { 0x16c06830, 76 },
    { 0xd3e06831, 100 },
    { 0x43b06832, 66 },
    { 0xd506832, 33 },
    { 0x88a06834, 91 },
    { 0x88706835, 95 },
    { 0x6b806836, 61 },
    { 0x4c10682e, 89 },
    { 0, 0 },
    { 0xcfe06839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b0683f, 98 },
    { 0x47506840, 76 },
    { 0x13006841, 84 },
    { 0xa4706840, 90 },
    { 0x2fb06843, 61 },
    { 0x5406842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf0684b, 33 },
    { 0, 0 },
    { 0xb6c0684d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f306852, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a06858, 55 },
    { 0x96606859, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c706861, 61 },
    { 0xf3606861, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83906867, 90 },
    { 0xcbd06868, 87 },
    { 0xa6906868, 52 },
    { 0xc550686a, 33 },
    { 0x9810686b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd730686e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80106871, 93 },
    { 0, 0 },
    { 0x40106873, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff806876, 78 },
    { 0x8aa06877, 61 },
    { 0x49e06876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3460687f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a06884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93806888, 46 },
    { 0, 0 },
    { 0x56d0688a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe706892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4106896, 76 },
    { 0x17806896, 66 },
    { 0x1f06896, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d0689c, 90 },
    { 0x3930689c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35068a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53068a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5068ae, 23 },
    { 0x103068ae, 90 },
    { 0xa56068ae, 55 },
    { 0x995068b1, 100 },
    { 0x2a5068b2, 61 },
    { 0x72d068b3, 99 },
    { 0xdc7068b3, 52 },
    { 0, 0 },
    { 0x34e068b6, 36 },
    { 0, 0 },
    { 0x827068b8, 43 },
    { 0x444068b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f068bc, 33 },
    { 0xa58068bd, 90 },
    { 0x641068be, 84 },
    { 0xda9068bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58068c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab068c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf068ca, 23 },
    { 0, 0 },
    { 0x19c068cc, 33 },
    { 0x389068cd, 55 },
    { 0, 0 },
    { 0xba4068cf, 99 },
    { 0x995068d0, 75 },
    { 0xef3068d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6068d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4068d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x227068da, 46 },
    { 0x3f2068da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1068df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86068e2, 76 },
    { 0xd71068e2, 33 },
    { 0xbfe068e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4068e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81068eb, 61 },
    { 0xf23068ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x453068ef, 90 },
    { 0, 0 },
    { 0x672068f1, 90 },
    { 0x16068f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60068f5, 55 },
    { 0xbcf068f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617068fc, 91 },
    { 0x15a068fc, 33 },
    { 0x15e068fe, 61 },
    { 0xdd068ff, 52 },
    { 0x60006900, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51106907, 61 },
    { 0xf506908, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc60690d, 91 },
    { 0xab10690e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b606915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9860691a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf06921, 99 },
    { 0, 0 },
    { 0x27306923, 52 },
    { 0, 0 },
    { 0x4e306925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9406928, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9040692f, 95 },
    { 0x7a06930, 23 },
    { 0xd2306931, 33 },
    { 0xaf306932, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef206936, 98 },
    { 0xac706936, 23 },
    { 0x4d906938, 23 },
    { 0x24506939, 85 },
    { 0, 0 },
    { 0x99a0693b, 33 },
    { 0, 0 },
    { 0x8010693d, 95 },
    { 0x5ad0693e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea06942, 55 },
    { 0, 0 },
    { 0x19206944, 90 },
    { 0x20206944, 90 },
    { 0, 0 },
    { 0x87a06947, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf10694a, 33 },
    { 0xbca0694a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13906954, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c806958, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc290695c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b06965, 33 },
    { 0, 0 },
    { 0xbcd06967, 76 },
    { 0, 0 },
    { 0xa1006969, 76 },
    { 0x1190696a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb06973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d50697a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb220697f, 64 },
    { 0, 0 },
    { 0x3906981, 34 },
    { 0x87806982, 81 },
    { 0xa2306983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe06987, 55 },
    { 0x4e006988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe0698b, 76 },
    { 0, 0 },
    { 0x4600698d, 99 },
    { 0, 0 },
    { 0xe040698f, 98 },
    { 0x4260698f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d06998, 33 },
    { 0x47d06999, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd700699e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541069a2, 23 },
    { 0, 0 },
    { 0x5e8069a4, 33 },
    { 0, 0 },
    { 0x7c8069a6, 61 },
    { 0x7c6069a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658069ab, 68 },
    { 0xb28069ab, 98 },
    { 0x201069ad, 61 },
    { 0x753069ac, 78 },
    { 0x798069ac, 61 },
    { 0x16d069b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be069b4, 43 },
    { 0, 0 },
    { 0x8af069b6, 90 },
    { 0x688069b7, 46 },
    { 0, 0 },
    { 0x6b9069b9, 95 },
    { 0x42f069ba, 98 },
    { 0, 0 },
    { 0xa2c069bc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15069c2, 55 },
    { 0, 0 },
    { 0xa74069c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa069cb, 90 },
    { 0x226069cc, 34 },
    { 0x9fd069cc, 33 },
    { 0xe2d069ce, 90 },
    { 0xf49069cf, 76 },
    { 0xaf1069d0, 77 },
    { 0xba8069ce, 99 },
    { 0x625069cb, 61 },
    { 0x621069cc, 98 },
    { 0x226069d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc069de, 95 },
    { 0xb16069de, 99 },
    { 0, 0 },
    { 0xfdb069e1, 61 },
    { 0x3a0069e2, 26 },
    { 0, 0 },
    { 0xb70069e4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d069e8, 33 },
    { 0x2b0069e9, 46 },
    { 0xe06069ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760069ee, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56069fb, 98 },
    { 0xaae069fb, 33 },
    { 0x9f6069fd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed06a06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa06a09, 98 },
    { 0x91f06a0a, 99 },
    { 0xd0306a0b, 98 },
    { 0x44706a0a, 43 },
    { 0, 0 },
    { 0x35f06a0e, 98 },
    { 0x80e06a0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d06a13, 55 },
    { 0x16f06a13, 33 },
    { 0x5a006a15, 17 },
    { 0xf5d06a14, 100 },
    { 0, 0 },
    { 0xa5e06a18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d06a1c, 90 },
    { 0, 0 },
    { 0x92606a1e, 90 },
    { 0xff806a1f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd06a24, 66 },
    { 0x14306a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa106a28, 78 },
    { 0xc3506a28, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a06a2d, 55 },
    { 0, 0 },
    { 0x2e706a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63606a32, 95 },
    { 0, 0 },
    { 0xbd706a34, 95 },
    { 0, 0 },
    { 0xeee06a36, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d06a3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb06a41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c806a4b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88806a4f, 91 },
    { 0, 0 },
    { 0x1bc06a51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe006a54, 33 },
    { 0, 0 },
    { 0x20f06a56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8206a5a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe306a5d, 33 },
    { 0, 0 },
    { 0x2c606a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f06a63, 23 },
    { 0x1f606a63, 43 },
    { 0x61906a65, 23 },
    { 0x85306a65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96606a6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae406a72, 98 },
    { 0, 0 },
    { 0x61506a74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f606a77, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b06a7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75206a80, 90 },
    { 0x12906a80, 55 },
    { 0xfbd06a81, 34 },
    { 0xeb906a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43206a88, 100 },
    { 0, 0 },
    { 0x1b306a8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46606a9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3006aa5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99506aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd306ab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1306ab5, 75 },
    { 0, 0 },
    { 0x88806ab7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1806aba, 96 },
    { 0x9c206abb, 33 },
    { 0xe4e06abc, 76 },
    { 0x65806abc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82506ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a06ac8, 95 },
    { 0x48606ac9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c906acc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d06acf, 77 },
    { 0x18706acf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de06ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b06add, 23 },
    { 0xa8106ade, 38 },
    { 0x4be06adf, 33 },
    { 0x80006add, 98 },
    { 0, 0 },
    { 0xf5506ae2, 46 },
    { 0, 0 },
    { 0x52c06ae4, 42 },
    { 0x6e06ae5, 90 },
    { 0xea906ae4, 75 },
    { 0xd6806ae7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3906aec, 68 },
    { 0xc06aed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d406af4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f06afc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f06b01, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0606b05, 23 },
    { 0x5ec06b06, 23 },
    { 0xea006b07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e06b0c, 95 },
    { 0x1f006b0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f06b14, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d06b1c, 85 },
    { 0xe7506b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f106b20, 93 },
    { 0xd7d06b20, 34 },
    { 0x38e06b22, 65 },
    { 0, 0 },
    { 0xaea06b24, 90 },
    { 0x1ef06b24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e06b29, 98 },
    { 0x62b06b2a, 98 },
    { 0x7a506b2b, 90 },
    { 0, 0 },
    { 0x65106b2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73206b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f06b35, 33 },
    { 0xa5d06b36, 95 },
    { 0, 0 },
    { 0x72606b38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c06b41, 33 },
    { 0xe5d06b42, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4706b47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe406b4a, 61 },
    { 0x9ce06b4b, 78 },
    { 0xd3906b4c, 91 },
    { 0x1a906b4d, 76 },
    { 0xd0a06b4a, 61 },
    { 0, 0 },
    { 0x54106b50, 43 },
    { 0, 0 },
    { 0xde706b52, 61 },
    { 0x49206b53, 68 },
    { 0x8306b54, 23 },
    { 0x4a106b53, 100 },
    { 0, 0 },
    { 0x34906b57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e406b5a, 23 },
    { 0x46f06b5b, 55 },
    { 0xe0606b5c, 61 },
    { 0x93906b5d, 23 },
    { 0x7e06b5d, 95 },
    { 0x65b06b5c, 90 },
    { 0, 0 },
    { 0xc0106b61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af06b6e, 46 },
    { 0x6d06b6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1006b76, 93 },
    { 0x80c06b77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb906b7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f706b7f, 95 },
    { 0xa3906b80, 61 },
    { 0xe1d06b81, 98 },
    { 0xf5a06b81, 87 },
    { 0x7be06b81, 98 },
    { 0xa5e06b81, 33 },
    { 0x38b06b82, 87 },
    { 0x77c06b83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf606b8b, 61 },
    { 0, 0 },
    { 0x9e906b8d, 61 },
    { 0, 0 },
    { 0x14106b8f, 33 },
    { 0, 0 },
    { 0x60806b91, 61 },
    { 0, 0 },
    { 0x3c306b93, 90 },
    { 0, 0 },
    { 0x1206b95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa06b98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f06b9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9506ba8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3006bab, 99 },
    { 0, 0 },
    { 0x50d06bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d06bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe406bb4, 66 },
    { 0xade06bb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d06bb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17206bbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff06bbf, 95 },
    { 0x2506bc0, 76 },
    { 0, 0 },
    { 0x4fb06bc2, 98 },
    { 0, 0 },
    { 0xae506bc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54906bc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc06bcb, 33 },
    { 0xbfe06bcb, 23 },
    { 0xf9606bcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf06bd9, 33 },
    { 0x4506bd9, 91 },
    { 0x45006bdb, 98 },
    { 0x39b06bda, 55 },
    { 0x64e06bdd, 95 },
    { 0xc1106bde, 23 },
    { 0x80a06bde, 23 },
    { 0, 0 },
    { 0x36706be1, 23 },
    { 0x27506be1, 90 },
    { 0xff006be3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4106be7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29706bec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6506bf9, 61 },
    { 0x7db06bfa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b06c01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b06c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f06c0c, 90 },
    { 0xeba06c0d, 91 },
    { 0xd3606c0d, 91 },
    { 0, 0 },
    { 0x89a06c10, 33 },
    { 0xea606c11, 84 },
    { 0x20e06c10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60906c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf806c1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c306c1f, 100 },
    { 0x5fc06c1f, 98 },
    { 0xa2806c21, 43 },
    { 0x51d06c20, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97306c27, 22 },
    { 0x2e506c27, 23 },
    { 0xbf506c29, 76 },
    { 0x8ce06c29, 77 },
    { 0xa3306c2a, 56 },
    { 0xaa806c27, 43 },
    { 0x43b06c2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc06c32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb206c35, 91 },
    { 0x91e06c35, 67 },
    { 0x47c06c36, 34 },
    { 0xbf606c35, 61 },
    { 0x24f06c35, 61 },
    { 0xb6d06c3a, 95 },
    { 0x2d206c36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac06c42, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c906c49, 33 },
    { 0x8a006c4a, 76 },
    { 0x10006c4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92506c4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad906c54, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de06c58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f406c5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac806c5e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8506c61, 76 },
    { 0xb6106c62, 23 },
    { 0x16506c63, 34 },
    { 0x6dd06c64, 23 },
    { 0x3f06c65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda406c69, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99706c6f, 52 },
    { 0, 0 },
    { 0xd06c71, 78 },
    { 0x94f06c72, 33 },
    { 0xf0606c73, 85 },
    { 0, 0 },
    { 0x5a306c75, 61 },
    { 0x20806c76, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b606c79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec006c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40706c85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb06c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a206c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b106c92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2606c95, 98 },
    { 0x45506c95, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a606c99, 90 },
    { 0x40a06c99, 52 },
    { 0, 0 },
    { 0x6e506c9c, 98 },
    { 0x74206c9d, 43 },
    { 0x2cd06c9e, 91 },
    { 0xf7506c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80406ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b506ca5, 90 },
    { 0x5c706ca6, 95 },
    { 0xd8806ca6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f906cb2, 61 },
    { 0x85306cb3, 52 },
    { 0xa2b06cb4, 23 },
    { 0xd606cb4, 43 },
    { 0x2bc06cb6, 55 },
    { 0, 0 },
    { 0xd8706cb8, 95 },
    { 0xe8a06cb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45406cbf, 80 },
    { 0x63006cbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f506cc6, 61 },
    { 0x8e506cc6, 33 },
    { 0x25d06cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6806cd6, 33 },
    { 0xb7206cd7, 33 },
    { 0, 0 },
    { 0x93106cd9, 98 },
    { 0x71e06cda, 23 },
    { 0x54a06cdb, 46 },
    { 0, 0 },
    { 0xd4f06cdd, 77 },
    { 0x2ae06cde, 91 },
    { 0x1ac06cdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9106ce2, 33 },
    { 0x13c06ce3, 52 },
    { 0, 0 },
    { 0x64c06ce5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa506ce9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf06cec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9406cf0, 10 },
    { 0x1d206cf0, 66 },
    { 0x42006cf2, 95 },
    { 0, 0 },
    { 0x58006cf4, 33 },
    { 0, 0 },
    { 0x13006cf6, 76 },
    { 0x27d06cf7, 95 },
    { 0x4ef06cf7, 33 },
    { 0x88606cf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d06cfc, 33 },
    { 0, 0 },
    { 0xae506cfe, 93 },
    { 0, 0 },
    { 0x8606d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d06d03, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x69506d06, 61 },
    { 0, 0 },
    { 0x13106d08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5806d0c, 23 },
    { 0xb6d06d0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2906d11, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35506d18, 23 },
    { 0, 0 },
    { 0x69606d1a, 81 },
    { 0x87306d1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd706d24, 90 },
    { 0xfb606d25, 98 },
    { 0xf1906d26, 24 },
    { 0x63f06d27, 33 },
    { 0x74b06d28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e606d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d806d31, 55 },
    { 0xc8906d32, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d706d39, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x37706d3c, 23 },
    { 0x86206d3d, 44 },
    { 0xa1206d3e, 33 },
    { 0xaff06d3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39006d44, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c06d47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56406d4c, 43 },
    { 0x29206d4d, 33 },
    { 0, 0 },
    { 0x96006d4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93206d56, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a906d5d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c06d61, 17 },
    { 0, 0 },
    { 0xde806d63, 84 },
    { 0xb8906d64, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3206d6a, 87 },
    { 0, 0 },
    { 0x75106d6c, 95 },
    { 0xe8b06d6d, 52 },
    { 0xfb806d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a06d71, 23 },
    { 0x53b06d71, 95 },
    { 0x3bb06d73, 23 },
    { 0x14e06d72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a106d78, 34 },
    { 0x5cd06d79, 99 },
    { 0x9f106d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b806d7e, 43 },
    { 0, 0 },
    { 0x9c806d80, 33 },
    { 0xc0806d81, 88 },
    { 0, 0 },
    { 0x4d06d83, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c06d8a, 23 },
    { 0x1e706d8a, 77 },
    { 0, 0 },
    { 0x5b806d8d, 95 },
    { 0x35b06d8e, 66 },
    { 0, 0 },
    { 0x69d06d90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26506d96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff06d9a, 23 },
    { 0xeac06d9a, 33 },
    { 0x55206d9c, 98 },
    { 0x9ba06d9d, 55 },
    { 0xa8d06d9e, 84 },
    { 0xc2406d9e, 61 },
    { 0xcef06da0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc06da3, 93 },
    { 0x4aa06da3, 90 },
    { 0xe3a06da4, 98 },
    { 0, 0 },
    { 0x94f06da7, 95 },
    { 0x91906da8, 43 },
    { 0x90906da8, 95 },
    { 0xa8a06da8, 33 },
    { 0, 0 },
    { 0x68506dac, 33 },
    { 0x4f506dac, 51 },
    { 0xf906dae, 95 },
    { 0x2a506dae, 65 },
    { 0x29f06dae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a06db6, 98 },
    { 0x47306db7, 33 },
    { 0x21a06db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35406dbb, 90 },
    { 0, 0 },
    { 0x37706dbd, 90 },
    { 0xafb06dbd, 68 },
    { 0, 0 },
    { 0x78806dc0, 55 },
    { 0x68806dc0, 95 },
    { 0, 0 },
    { 0x73406dc3, 93 },
    { 0x28106dc3, 23 },
    { 0, 0 },
    { 0xcad06dc6, 90 },
    { 0, 0 },
    { 0x42006dc8, 66 },
    { 0x4bc06dc8, 61 },
    { 0x55b06dca, 40 },
    { 0x8b206dcb, 33 },
    { 0x87106dcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c806dd0, 98 },
    { 0x3fc06dd0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d606dd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1706dd7, 91 },
    { 0, 0 },
    { 0x64406dd9, 75 },
    { 0x88e06dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4506ddd, 76 },
    { 0xece06dde, 61 },
    { 0x8b506ddf, 95 },
    { 0, 0 },
    { 0xd5c06de1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac806de4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4006de9, 90 },
    { 0, 0 },
    { 0x7aa06deb, 17 },
    { 0, 0 },
    { 0xf4906ded, 95 },
    { 0xd5806ded, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa06df2, 100 },
    { 0x44b06df3, 98 },
    { 0, 0 },
    { 0x1b06df5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa06df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e06dfd, 55 },
    { 0, 0 },
    { 0xf7306dff, 76 },
    { 0xbcd06dff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b606e05, 90 },
    { 0x5a306e06, 96 },
    { 0xe0306e06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c606e0d, 83 },
    { 0xa2506e0e, 95 },
    { 0, 0 },
    { 0xb6f06e10, 61 },
    { 0, 0 },
    { 0x69906e12, 23 },
    { 0x3cf06e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a06e16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e06e1a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45106e23, 100 },
    { 0, 0 },
    { 0x85906e25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7306e28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1806e2b, 77 },
    { 0xec06e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81906e33, 61 },
    { 0, 0 },
    { 0x45106e35, 61 },
    { 0xe1b06e36, 98 },
    { 0xf5d06e35, 75 },
    { 0xa2b06e38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5406e3d, 65 },
    { 0x2b106e3e, 33 },
    { 0x49306e3f, 67 },
    { 0xce106e40, 34 },
    { 0xde406e3f, 43 },
    { 0, 0 },
    { 0xfeb06e43, 76 },
    { 0xaa06e44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe06e47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab06e4d, 55 },
    { 0x77906e4d, 94 },
    { 0x9aa06e4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e906e52, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xda606e55, 76 },
    { 0xc6406e55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9706e5d, 76 },
    { 0xe7906e5d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb06e62, 91 },
    { 0x2c506e62, 33 },
    { 0x51306e63, 38 },
    { 0x85506e65, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7706e69, 90 },
    { 0xe1c06e6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21506e73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee06e77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6206e82, 68 },
    { 0xbb106e83, 76 },
    { 0xfdc06e83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb906e87, 91 },
    { 0x25f06e87, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a06e93, 43 },
    { 0xb0a06e94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd606e98, 33 },
    { 0xa1606e98, 95 },
    { 0, 0 },
    { 0xa3706e9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c406e9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36106ea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a706ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2706ead, 23 },
    { 0xdbf06eae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x59106eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23006eb4, 61 },
    { 0, 0 },
    { 0x5c406eb6, 61 },
    { 0, 0 },
    { 0xd1906eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda206ebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a06ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2706ec9, 40 },
    { 0xe9c06eca, 46 },
    { 0, 0 },
    { 0x3af06ecc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06ed2, 43 },
    { 0, 0 },
    { 0x45306ed4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe06ed7, 85 },
    { 0x54106ed7, 17 },
    { 0, 0 },
    { 0x4eb06eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd206edf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x17506ee2, 65 },
    { 0xdd906ee2, 53 },
    { 0x3906ee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d006ee8, 61 },
    { 0x3c06ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99506eec, 90 },
    { 0, 0 },
    { 0xf3b06eee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe106ef8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56906efe, 33 },
    { 0x38c06efe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f06f03, 55 },
    { 0xee206f04, 91 },
    { 0, 0 },
    { 0x9d06f06, 67 },
    { 0, 0 },
    { 0x93606f08, 76 },
    { 0, 0 },
    { 0x14906f0a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x56706f0d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f06f11, 90 },
    { 0, 0 },
    { 0x43806f13, 90 },
    { 0x5e606f13, 33 },
    { 0, 0 },
    { 0xa0106f16, 76 },
    { 0, 0 },
    { 0x76706f18, 76 },
    { 0xeaf06f19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d06f1c, 90 },
    { 0x47106f1d, 33 },
    { 0x45b06f1e, 95 },
    { 0xbe406f1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17006f26, 88 },
    { 0, 0 },
    { 0x1a606f28, 33 },
    { 0, 0 },
    { 0x3fd06f2a, 61 },
    { 0x9ad06f2b, 90 },
    { 0x4ce06f2c, 61 },
    { 0xe3406f2d, 99 },
    { 0x5fb06f2d, 43 },
    { 0xd4206f2b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d06f34, 92 },
    { 0, 0 },
    { 0x6ba06f36, 33 },
    { 0x57806f37, 84 },
    { 0x5e506f38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc306f3b, 95 },
    { 0x2a506f3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a06f41, 76 },
    { 0x1dc06f42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39206f47, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df06f4a, 95 },
    { 0xec06f4a, 33 },
    { 0x46c06f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e06f4f, 85 },
    { 0x10506f50, 90 },
    { 0x5b06f51, 61 },
    { 0xa5a06f51, 93 },
    { 0xaa106f53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6906f56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d506f5b, 46 },
    { 0x53a06f5c, 98 },
    { 0xbe206f5b, 33 },
    { 0x9bf06f5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b06f6a, 77 },
    { 0x6c406f6b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73306f73, 48 },
    { 0x65006f74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c06f79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce906f7c, 76 },
    { 0xbcf06f7d, 98 },
    { 0xc4506f7c, 99 },
    { 0x81106f7f, 61 },
    { 0x5cf06f80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7106f84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec06f87, 77 },
    { 0, 0 },
    { 0x31906f89, 98 },
    { 0, 0 },
    { 0x49406f8b, 95 },
    { 0, 0 },
    { 0x8ef06f8d, 90 },
    { 0, 0 },
    { 0xb7e06f8f, 76 },
    { 0xdce06f90, 91 },
    { 0x8106f91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46906f99, 77 },
    { 0x75206f9a, 98 },
    { 0xfa106f99, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3406f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c06fa6, 34 },
    { 0x98a06fa6, 95 },
    { 0, 0 },
    { 0x7b906fa9, 80 },
    { 0x76006fa9, 91 },
    { 0, 0 },
    { 0xdda06fac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee606fb1, 87 },
    { 0x27806fb2, 46 },
    { 0, 0 },
    { 0x47e06fb4, 59 },
    { 0xde406fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b406fbb, 83 },
    { 0x75e06fbc, 61 },
    { 0, 0 },
    { 0xdfe06fbe, 26 },
    { 0x4a06fbe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35006fc5, 83 },
    { 0x92606fc5, 98 },
    { 0, 0 },
    { 0x30e06fc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24306fd0, 100 },
    { 0xc7b06fd0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd06fd5, 95 },
    { 0, 0 },
    { 0xa3906fd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5106fdb, 33 },
    { 0x23606fdc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x43406fdf, 68 },
    { 0, 0 },
    { 0x5f806fe1, 33 },
    { 0xc9e06fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a06fe5, 98 },
    { 0, 0 },
    { 0xc6206fe7, 90 },
    { 0x59406fe8, 98 },
    { 0x67106fe8, 93 },
    { 0x4d206fea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e06fee, 61 },
    { 0, 0 },
    { 0x6ba06ff0, 34 },
    { 0, 0 },
    { 0x48f06ff2, 90 },
    { 0, 0 },
    { 0x30606ff4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d606ff7, 91 },
    { 0, 0 },
    { 0xb2006ff9, 91 },
    { 0xc8c06ff9, 61 },
    { 0, 0 },
    { 0xa9406ffc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd406fff, 90 },
    { 0x82707000, 90 },
    { 0, 0 },
    { 0xd5507002, 33 },
    { 0x12b07003, 99 },
    { 0x2c807004, 67 },
    { 0, 0 },
    { 0x83c07006, 87 },
    { 0, 0 },
    { 0x84507008, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3810700b, 34 },
    { 0x7eb0700b, 95 },
    { 0xb390700b, 61 },
    { 0x1b30700b, 95 },
    { 0x19f0700f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b0701d, 33 },
    { 0, 0 },
    { 0xe590701f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7807024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0702b, 84 },
    { 0, 0 },
    { 0xd660702d, 61 },
    { 0x1df0702e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77507036, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf0703a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7380703d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75307040, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2107044, 76 },
    { 0x84f07045, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20704a, 91 },
    { 0xc600704b, 43 },
    { 0xe7c0704c, 23 },
    { 0xcc30704d, 90 },
    { 0x42a0704e, 61 },
    { 0x7920704e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd07052, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f07055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00705d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4207060, 50 },
    { 0, 0 },
    { 0x21f07062, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19307068, 55 },
    { 0x6b807069, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf30706c, 90 },
    { 0, 0 },
    { 0x47c0706e, 33 },
    { 0x8890706f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e07074, 84 },
    { 0x1c807075, 95 },
    { 0xad607076, 90 },
    { 0x20b07077, 98 },
    { 0x6e207074, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a0707b, 92 },
    { 0xed80707c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83507080, 76 },
    { 0xf1007080, 95 },
    { 0x27b07080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e007086, 95 },
    { 0xfe507087, 55 },
    { 0x4807087, 23 },
    { 0xebf07088, 76 },
    { 0xcec0708a, 90 },
    { 0x55a07087, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2290708e, 61 },
    { 0, 0 },
    { 0x55207090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3607094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca607099, 55 },
    { 0x8260709a, 98 },
    { 0x8920709a, 90 },
    { 0x2bb0709a, 23 },
    { 0x3c60709d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806070a8, 95 },
    { 0x8b1070a9, 98 },
    { 0x7d4070aa, 65 },
    { 0x874070ab, 61 },
    { 0xc37070ab, 23 },
    { 0x135070ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x661070b0, 33 },
    { 0, 0 },
    { 0x379070b2, 98 },
    { 0, 0 },
    { 0x1d2070b4, 34 },
    { 0xb70070b5, 98 },
    { 0x89d070b5, 33 },
    { 0x6fc070b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798070bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84070c4, 99 },
    { 0x797070c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956070ce, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af070d1, 87 },
    { 0x963070d2, 95 },
    { 0, 0 },
    { 0xbb5070d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0070d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd070dd, 33 },
    { 0xa8070dd, 61 },
    { 0, 0 },
    { 0x823070e0, 43 },
    { 0, 0 },
    { 0x529070e2, 33 },
    { 0x190070e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0070e7, 53 },
    { 0x3ea070e8, 33 },
    { 0xb28070e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624070ef, 99 },
    { 0xa07070ef, 91 },
    { 0xddf070ef, 95 },
    { 0xe070f1, 90 },
    { 0x2b3070f3, 46 },
    { 0x174070f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79070f8, 98 },
    { 0, 0 },
    { 0x150070fa, 100 },
    { 0xc61070fa, 23 },
    { 0xcc3070fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x544070ff, 23 },
    { 0xa0307100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba07104, 95 },
    { 0, 0 },
    { 0xf1607106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90710d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56807119, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc440711e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x24807121, 33 },
    { 0x22a07122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c07126, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2007131, 78 },
    { 0, 0 },
    { 0xaa407133, 95 },
    { 0, 0 },
    { 0xdb307135, 23 },
    { 0x34007136, 52 },
    { 0x46d07135, 61 },
    { 0x67e07138, 43 },
    { 0, 0 },
    { 0xc460713a, 90 },
    { 0, 0 },
    { 0xb110713c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc40713f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d07142, 84 },
    { 0xa1007143, 99 },
    { 0xd9207144, 61 },
    { 0x2a507143, 33 },
    { 0x76407143, 90 },
    { 0x99d07147, 61 },
    { 0x8c207143, 90 },
    { 0xf7c07144, 90 },
    { 0xb6c0714a, 55 },
    { 0xcfc0714a, 93 },
    { 0xbfe0714b, 33 },
    { 0x2590714d, 78 },
    { 0xb1107143, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f907152, 87 },
    { 0, 0 },
    { 0x54a07154, 33 },
    { 0x16107155, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0107158, 52 },
    { 0xdfa07159, 43 },
    { 0x90a07159, 76 },
    { 0xdbd07159, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0715e, 98 },
    { 0x2fc0715f, 97 },
    { 0xd0507160, 61 },
    { 0x4ea0715e, 61 },
    { 0x8440715e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c07169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2500716e, 17 },
    { 0x3100716f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7407172, 66 },
    { 0xda407172, 34 },
    { 0x1307173, 98 },
    { 0x13107175, 52 },
    { 0x20207176, 90 },
    { 0x95a07177, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7030717a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12207181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0907185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e0718a, 95 },
    { 0x1500718a, 36 },
    { 0xca30718b, 95 },
    { 0xe0a0718d, 98 },
    { 0xe5a0718c, 90 },
    { 0x8000718d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78707192, 90 },
    { 0xdc007193, 90 },
    { 0xe4b07193, 34 },
    { 0xd3407195, 23 },
    { 0x9b907195, 55 },
    { 0x3ec07197, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5980719d, 39 },
    { 0x4400719e, 87 },
    { 0xe1c0719f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00071a4, 61 },
    { 0x380071a5, 37 },
    { 0xd63071a6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1071ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142071b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df071b6, 93 },
    { 0, 0 },
    { 0xec1071b8, 33 },
    { 0x63d071b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2071bc, 23 },
    { 0x6c071bc, 76 },
    { 0x9a1071bc, 61 },
    { 0x57e071bf, 95 },
    { 0, 0 },
    { 0x7d3071c1, 61 },
    { 0x929071c2, 34 },
    { 0xc5f071c3, 61 },
    { 0, 0 },
    { 0x75b071c5, 55 },
    { 0x77071c5, 76 },
    { 0xc03071c7, 33 },
    { 0xdfe071c5, 91 },
    { 0xc51071c9, 55 },
    { 0, 0 },
    { 0x73071cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8071d2, 95 },
    { 0xb1d071d2, 33 },
    { 0x536071d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96071d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c071dd, 98 },
    { 0xec4071dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932071e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528071e6, 61 },
    { 0xbd071e7, 43 },
    { 0, 0 },
    { 0x2f5071e9, 98 },
    { 0, 0 },
    { 0x3af071eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5071ee, 88 },
    { 0, 0 },
    { 0x695071f0, 61 },
    { 0x53a071f1, 90 },
    { 0, 0 },
    { 0x58e071f3, 46 },
    { 0x2e7071f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16071f7, 61 },
    { 0x762071f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27071fb, 33 },
    { 0xaf2071fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b07200, 90 },
    { 0, 0 },
    { 0x4ab07202, 17 },
    { 0x4607203, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c807208, 87 },
    { 0xa607209, 76 },
    { 0xbc70720a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67607210, 33 },
    { 0x8e007210, 64 },
    { 0x7fc07212, 55 },
    { 0xa4007213, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b07216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c07219, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa07221, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a07224, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1000722c, 78 },
    { 0xaa00722d, 43 },
    { 0xf1f0722c, 34 },
    { 0x8a70722f, 90 },
    { 0x11107230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef07233, 33 },
    { 0x92107234, 76 },
    { 0x4fd07235, 90 },
    { 0, 0 },
    { 0x61607237, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd480723d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75807242, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c07245, 61 },
    { 0xee507246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e0724b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc207250, 95 },
    { 0x43c07251, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6307254, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f0725b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3960725e, 66 },
    { 0xa010725f, 99 },
    { 0xa80725f, 33 },
    { 0, 0 },
    { 0xb1307262, 23 },
    { 0xfd907263, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc807268, 23 },
    { 0, 0 },
    { 0x4670726a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810726f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1407273, 90 },
    { 0x7d007274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe530727c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e07286, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8590728e, 95 },
    { 0x3020728f, 34 },
    { 0, 0 },
    { 0x18507291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7307297, 100 },
    { 0, 0 },
    { 0x7bc07299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b072a1, 33 },
    { 0, 0 },
    { 0x598072a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156072ac, 43 },
    { 0xcbb072ad, 75 },
    { 0x111072ae, 33 },
    { 0x36d072af, 99 },
    { 0xd4d072af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b072b3, 33 },
    { 0xd27072b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79072b7, 19 },
    { 0, 0 },
    { 0xa19072b9, 96 },
    { 0xc6072b9, 33 },
    { 0x7c3072bb, 43 },
    { 0xa44072bc, 90 },
    { 0xb15072bd, 23 },
    { 0xd27072bd, 33 },
    { 0x808072b9, 98 },
    { 0, 0 },
    { 0x7bf072c1, 34 },
    { 0, 0 },
    { 0xc5c072c3, 90 },
    { 0xc81072c4, 78 },
    { 0xc72072c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56072cc, 90 },
    { 0x914072cd, 90 },
    { 0x279072cc, 43 },
    { 0x98072cf, 95 },
    { 0xf14072d0, 98 },
    { 0, 0 },
    { 0xb64072d2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110072d7, 23 },
    { 0xc072d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x588072db, 61 },
    { 0x1bf072dc, 61 },
    { 0xd8f072dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572072e1, 34 },
    { 0x871072e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70072e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170072ef, 90 },
    { 0, 0 },
    { 0xaa5072f1, 90 },
    { 0xb22072f2, 98 },
    { 0x670072f3, 67 },
    { 0x6f6072f4, 90 },
    { 0xfdf072f2, 43 },
    { 0x784072f6, 33 },
    { 0, 0 },
    { 0x488072f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e072fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b907301, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19807309, 90 },
    { 0, 0 },
    { 0x4da0730b, 97 },
    { 0, 0 },
    { 0xafa0730d, 65 },
    { 0x90c0730d, 40 },
    { 0xc650730f, 84 },
    { 0xee407310, 33 },
    { 0, 0 },
    { 0xa4e07312, 61 },
    { 0xa4807312, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x80607316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be07319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc90731c, 95 },
    { 0x47a0731c, 67 },
    { 0x6410731c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46007322, 78 },
    { 0x79a07323, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe507326, 68 },
    { 0, 0 },
    { 0x6507328, 23 },
    { 0xb9b07329, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe580732f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e70733b, 2 },
    { 0x3930733c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e0733f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f07342, 95 },
    { 0x9ee07343, 87 },
    { 0x4f507344, 80 },
    { 0, 0 },
    { 0x71a07346, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c507349, 98 },
    { 0xf907349, 95 },
    { 0x8270734b, 23 },
    { 0x1b30734c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e07351, 90 },
    { 0, 0 },
    { 0xaec07353, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a07357, 76 },
    { 0, 0 },
    { 0xb8c07359, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920735c, 23 },
    { 0, 0 },
    { 0x8e10735e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b07362, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d07367, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e0736a, 95 },
    { 0x9280736b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6107371, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e07377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc330737b, 33 },
    { 0xece0737b, 34 },
    { 0x26f0737b, 99 },
    { 0, 0 },
    { 0x16e0737f, 43 },
    { 0x17d07380, 75 },
    { 0x7fa07381, 67 },
    { 0, 0 },
    { 0xb8d07383, 78 },
    { 0x8ed07383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26007389, 100 },
    { 0x6640738a, 43 },
    { 0x2650738b, 90 },
    { 0x66c0738a, 61 },
    { 0x6360738c, 34 },
    { 0, 0 },
    { 0x8100738f, 46 },
    { 0x4d80738f, 90 },
    { 0x6c007391, 98 },
    { 0, 0 },
    { 0x53f07393, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8040739c, 80 },
    { 0, 0 },
    { 0x7de0739e, 46 },
    { 0xdc50739f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea073a5, 95 },
    { 0x3af073a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87073ab, 100 },
    { 0x3b6073ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b073af, 80 },
    { 0, 0 },
    { 0x160073b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd073ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64073bf, 95 },
    { 0, 0 },
    { 0x957073c1, 55 },
    { 0, 0 },
    { 0x74073c3, 90 },
    { 0xd86073c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1073cb, 76 },
    { 0x989073cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2073cf, 98 },
    { 0x68b073d0, 98 },
    { 0, 0 },
    { 0xbfe073d2, 90 },
    { 0xc80073d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5073d7, 67 },
    { 0x55073d7, 78 },
    { 0, 0 },
    { 0xa2b073da, 95 },
    { 0xb64073da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1073e3, 33 },
    { 0x883073e3, 34 },
    { 0x22e073e5, 77 },
    { 0, 0 },
    { 0xa08073e7, 44 },
    { 0x766073e8, 61 },
    { 0, 0 },
    { 0x6fe073ea, 55 },
    { 0xd0073eb, 66 },
    { 0xb50073ec, 90 },
    { 0xfc2073eb, 23 },
    { 0xd18073ea, 88 },
    { 0x56c073ef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x504073f2, 46 },
    { 0, 0 },
    { 0x494073f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a073f7, 87 },
    { 0x722073f8, 95 },
    { 0x729073f8, 98 },
    { 0xd7e073f8, 76 },
    { 0xccf073f8, 55 },
    { 0, 0 },
    { 0x940073fd, 90 },
    { 0x4c5073fe, 61 },
    { 0x6a8073ff, 61 },
    { 0x704073ff, 61 },
    { 0xbc707401, 77 },
    { 0xd3207402, 93 },
    { 0, 0 },
    { 0x42807404, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f0740a, 90 },
    { 0, 0 },
    { 0xa5c0740c, 66 },
    { 0x2330740d, 90 },
    { 0xb80740d, 34 },
    { 0x9890740c, 23 },
    { 0x5150740f, 90 },
    { 0xc3907411, 98 },
    { 0x11107411, 95 },
    { 0xd707411, 95 },
    { 0x2300740e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5707417, 90 },
    { 0xab807418, 34 },
    { 0, 0 },
    { 0x1790741a, 33 },
    { 0, 0 },
    { 0x4070741c, 43 },
    { 0x2520741d, 43 },
    { 0x6a30741e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa07422, 85 },
    { 0x33807422, 90 },
    { 0x9a807424, 33 },
    { 0xa207425, 43 },
    { 0xa2407426, 78 },
    { 0x74907425, 95 },
    { 0xbd207428, 61 },
    { 0xf0107428, 95 },
    { 0xbd707422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c07430, 34 },
    { 0xbc807431, 33 },
    { 0xb0907431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db07440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79307445, 98 },
    { 0x24c07445, 90 },
    { 0, 0 },
    { 0xb8507448, 61 },
    { 0x89907449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae07451, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3107456, 99 },
    { 0, 0 },
    { 0x65d07458, 76 },
    { 0xa1407459, 23 },
    { 0x60207458, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d0745f, 85 },
    { 0, 0 },
    { 0xb4c07461, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f507466, 90 },
    { 0x5ad07466, 95 },
    { 0x2a607468, 23 },
    { 0, 0 },
    { 0x4910746a, 33 },
    { 0x36d0746a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7507472, 99 },
    { 0xdfe07473, 81 },
    { 0xc5307473, 90 },
    { 0x4d907473, 23 },
    { 0, 0 },
    { 0xe2a07477, 91 },
    { 0xda407477, 78 },
    { 0x2cc07479, 88 },
    { 0x19c0747a, 92 },
    { 0x94107477, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf07488, 91 },
    { 0xbe107489, 33 },
    { 0xb5d07489, 90 },
    { 0x54b07489, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b07491, 99 },
    { 0xe1d07492, 55 },
    { 0xe2507493, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9707496, 61 },
    { 0x6b907497, 23 },
    { 0, 0 },
    { 0xf5207499, 52 },
    { 0xc970749a, 90 },
    { 0, 0 },
    { 0x64d0749c, 23 },
    { 0x6640749d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f074a2, 33 },
    { 0x9c9074a3, 76 },
    { 0x11f074a4, 46 },
    { 0x5c0074a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996074a9, 90 },
    { 0xead074aa, 33 },
    { 0xbab074aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb074b1, 34 },
    { 0x6bd074b1, 99 },
    { 0x641074b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b074b8, 98 },
    { 0xf91074b8, 83 },
    { 0xce074ba, 90 },
    { 0x5f2074bb, 95 },
    { 0xc42074bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a074c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6074c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4074cd, 33 },
    { 0x69d074ce, 76 },
    { 0xa3b074cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x495074d2, 99 },
    { 0x1b0074d3, 52 },
    { 0xf6a074d3, 61 },
    { 0x18a074d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4074da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x607074dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5074e0, 23 },
    { 0xe82074e1, 33 },
    { 0x412074e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c074e8, 98 },
    { 0, 0 },
    { 0x84b074ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9074ed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430074f1, 90 },
    { 0x34c074f2, 91 },
    { 0, 0 },
    { 0xd6c074f4, 95 },
    { 0x816074f5, 90 },
    { 0xc4a074f6, 52 },
    { 0xddc074f7, 67 },
    { 0x7dc074f4, 90 },
    { 0xf35074f4, 55 },
    { 0x59b074f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73074ff, 90 },
    { 0x318074ff, 55 },
    { 0, 0 },
    { 0xa7507502, 98 },
    { 0x8b007502, 68 },
    { 0x2b007504, 43 },
    { 0x6bd07502, 33 },
    { 0x39107506, 76 },
    { 0, 0 },
    { 0xd6507508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e0750e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a07512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7507515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6920751a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7907529, 91 },
    { 0, 0 },
    { 0xb380752b, 90 },
    { 0x9fe0752c, 43 },
    { 0x8e60752d, 67 },
    { 0xf20752d, 34 },
    { 0, 0 },
    { 0xd7107530, 34 },
    { 0xb6207531, 82 },
    { 0x5df07532, 95 },
    { 0xed407533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0753f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9007542, 61 },
    { 0, 0 },
    { 0x9a407544, 17 },
    { 0x82b07545, 67 },
    { 0x46307546, 91 },
    { 0x1807547, 95 },
    { 0xe4707547, 93 },
    { 0x3c207549, 23 },
    { 0, 0 },
    { 0x4dd0754b, 98 },
    { 0x1510754b, 95 },
    { 0x8a20754b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb907559, 81 },
    { 0x3950755a, 34 },
    { 0x6290755b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee0755f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0707563, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6007566, 98 },
    { 0x7707567, 46 },
    { 0x73d07568, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f07570, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6507573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63307576, 66 },
    { 0xc4607576, 84 },
    { 0xe8a07578, 90 },
    { 0xa5a07579, 33 },
    { 0x12707579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a707580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9107583, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e007588, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc610758d, 61 },
    { 0, 0 },
    { 0x47f0758f, 69 },
    { 0, 0 },
    { 0xa2207591, 98 },
    { 0xf5907591, 17 },
    { 0x59407593, 95 },
    { 0xcc707594, 90 },
    { 0x2ed07595, 43 },
    { 0xaf807596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0759c, 93 },
    { 0, 0 },
    { 0x65d0759e, 46 },
    { 0x77e0759e, 40 },
    { 0x7d90759f, 90 },
    { 0, 0 },
    { 0x69a075a2, 46 },
    { 0x932075a2, 23 },
    { 0xf9d075a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d075a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2075ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x134075af, 95 },
    { 0x853075b0, 60 },
    { 0xc58075b1, 90 },
    { 0, 0 },
    { 0x385075b3, 66 },
    { 0, 0 },
    { 0xe2075b5, 87 },
    { 0, 0 },
    { 0x253075b7, 43 },
    { 0x3d1075b8, 76 },
    { 0x9fb075b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858075be, 99 },
    { 0x42b075bf, 91 },
    { 0x6e075c0, 78 },
    { 0x998075c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x725075c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f075c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b075ce, 68 },
    { 0, 0 },
    { 0x562075d0, 34 },
    { 0x8de075d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d075d4, 90 },
    { 0xf3b075d4, 88 },
    { 0xdfb075d5, 87 },
    { 0xc49075d7, 90 },
    { 0x8f8075d7, 98 },
    { 0x3b2075d9, 59 },
    { 0xc8a075da, 90 },
    { 0x515075da, 61 },
    { 0xb0a075d9, 65 },
    { 0x663075d8, 88 },
    { 0xef4075de, 88 },
    { 0x34c075df, 90 },
    { 0xdaf075e0, 66 },
    { 0xf25075e0, 91 },
    { 0x553075e2, 55 },
    { 0, 0 },
    { 0xc8b075e4, 43 },
    { 0, 0 },
    { 0xfa8075e6, 90 },
    { 0x398075e7, 34 },
    { 0xff7075e8, 90 },
    { 0xfc3075e8, 90 },
    { 0, 0 },
    { 0xfbd075eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80075ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac075f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98075fc, 33 },
    { 0x1ff075fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a07603, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c907607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b07610, 95 },
    { 0x2c107610, 55 },
    { 0xf3807611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22607615, 34 },
    { 0x5ac07616, 33 },
    { 0xef407617, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa720761c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f07620, 95 },
    { 0, 0 },
    { 0x20d07622, 56 },
    { 0, 0 },
    { 0x24a07624, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6407629, 46 },
    { 0, 0 },
    { 0x1e40762b, 17 },
    { 0, 0 },
    { 0x762d, 69 },
    { 0xbd90762e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b207632, 90 },
    { 0, 0 },
    { 0x65807634, 81 },
    { 0, 0 },
    { 0x3bb07636, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc330763c, 95 },
    { 0x5850763d, 61 },
    { 0, 0 },
    { 0x76d0763f, 34 },
    { 0xbfa07640, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd807643, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c07647, 80 },
    { 0, 0 },
    { 0xdc207649, 46 },
    { 0, 0 },
    { 0xf8d0764b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b70764f, 98 },
    { 0xe5207650, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f07654, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b0765a, 34 },
    { 0xf700765b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f80765e, 43 },
    { 0, 0 },
    { 0x7407660, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8707663, 43 },
    { 0, 0 },
    { 0xef107665, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x207668, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d107670, 52 },
    { 0x9f407671, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23607679, 33 },
    { 0xe1b0767a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d407680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be07683, 65 },
    { 0x85707684, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee07689, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b00768f, 76 },
    { 0xaa607690, 99 },
    { 0, 0 },
    { 0xf0c07692, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa07696, 23 },
    { 0x97407696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3076a1, 76 },
    { 0, 0 },
    { 0x338076a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904076aa, 91 },
    { 0xac0076ab, 91 },
    { 0x3ee076ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f076b0, 88 },
    { 0x248076b0, 95 },
    { 0xc4076b0, 90 },
    { 0x9cf076b3, 33 },
    { 0, 0 },
    { 0x3ed076b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105076bd, 34 },
    { 0x426076be, 23 },
    { 0x3d7076bd, 95 },
    { 0, 0 },
    { 0x6a6076c1, 46 },
    { 0x68e076c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae076c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102076cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba076d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1076d9, 33 },
    { 0, 0 },
    { 0x95a076db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x434076de, 61 },
    { 0xbc1076df, 99 },
    { 0x6a2076e0, 98 },
    { 0x21f076e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb076e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d076e8, 34 },
    { 0x819076e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128076ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3076f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772076f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28207705, 43 },
    { 0xb3d07706, 100 },
    { 0xcc707706, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a0770c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30907716, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7007722, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6807725, 23 },
    { 0x1fe07726, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e307730, 90 },
    { 0x6d507731, 76 },
    { 0, 0 },
    { 0x81807733, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7107736, 23 },
    { 0x78307737, 95 },
    { 0x6e207738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc950773b, 90 },
    { 0x63c0773c, 46 },
    { 0xfc20773d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2907742, 33 },
    { 0x29507743, 61 },
    { 0, 0 },
    { 0xaca07745, 34 },
    { 0, 0 },
    { 0x3f407747, 61 },
    { 0xd7107748, 67 },
    { 0xa8007748, 66 },
    { 0x7140774a, 43 },
    { 0x5480774a, 76 },
    { 0xc130774b, 98 },
    { 0xa470774d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf07750, 95 },
    { 0x21707751, 90 },
    { 0x2cb07752, 95 },
    { 0xc5307753, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68407756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1230775c, 95 },
    { 0x7b30775c, 34 },
    { 0, 0 },
    { 0xd8f0775f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a607769, 55 },
    { 0, 0 },
    { 0x1fe0776b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce07771, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb107777, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f0777a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x530777d, 33 },
    { 0, 0 },
    { 0x3830777f, 66 },
    { 0x73407780, 23 },
    { 0x8e207781, 98 },
    { 0, 0 },
    { 0x69e07783, 61 },
    { 0x79407783, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d07787, 76 },
    { 0xb8307788, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b60778e, 90 },
    { 0x6580778f, 90 },
    { 0x98a0778f, 34 },
    { 0x52b07791, 76 },
    { 0xaf407792, 43 },
    { 0xa4a07792, 85 },
    { 0, 0 },
    { 0xea007795, 46 },
    { 0x1807796, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690779a, 33 },
    { 0x8eb0779b, 55 },
    { 0, 0 },
    { 0x190779d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3077a4, 66 },
    { 0x2ec077a5, 90 },
    { 0xe97077a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9077a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e077ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7077b4, 84 },
    { 0x342077b5, 18 },
    { 0, 0 },
    { 0x57b077b7, 96 },
    { 0xae0077b8, 90 },
    { 0xf10077b9, 55 },
    { 0x764077b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b077bd, 90 },
    { 0x58d077be, 91 },
    { 0xa68077bd, 25 },
    { 0x346077c0, 33 },
    { 0xb44077c1, 90 },
    { 0, 0 },
    { 0xcff077c3, 65 },
    { 0, 0 },
    { 0x741077c5, 34 },
    { 0x3d3077c6, 68 },
    { 0x32e077c5, 97 },
    { 0, 0 },
    { 0x4c8077c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5077cc, 98 },
    { 0x437077cd, 23 },
    { 0xd4077ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7077d3, 95 },
    { 0x4ac077d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1077d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba077dd, 91 },
    { 0, 0 },
    { 0x625077df, 90 },
    { 0x292077df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af077e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x202077e7, 95 },
    { 0, 0 },
    { 0x2c4077e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116077f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503077f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca077f9, 61 },
    { 0x8a3077fa, 91 },
    { 0x581077fa, 78 },
    { 0x867077fc, 61 },
    { 0xf93077fd, 61 },
    { 0x51f077fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95307805, 95 },
    { 0xd5e07806, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b10780a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca80780d, 90 },
    { 0x93d0780e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1807813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57607816, 76 },
    { 0x76e07816, 90 },
    { 0x78a07817, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0707823, 98 },
    { 0xb9207824, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14607829, 98 },
    { 0xf8707829, 98 },
    { 0xc690782b, 23 },
    { 0xbce0782c, 55 },
    { 0, 0 },
    { 0xf90782e, 36 },
    { 0x59b0782f, 23 },
    { 0, 0 },
    { 0x9fa07831, 81 },
    { 0x68f07831, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae07839, 23 },
    { 0xbb0783a, 33 },
    { 0x6df0783b, 90 },
    { 0x42e0783c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb880783f, 61 },
    { 0, 0 },
    { 0x70f07841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a007846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a0784d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde07854, 38 },
    { 0x2fa07855, 96 },
    { 0x97007856, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d00785f, 95 },
    { 0, 0 },
    { 0x4a107861, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf407864, 90 },
    { 0x1ba07865, 76 },
    { 0x35f07864, 87 },
    { 0, 0 },
    { 0xfe207868, 90 },
    { 0, 0 },
    { 0xd6c0786a, 23 },
    { 0x9900786b, 90 },
    { 0x6f90786b, 80 },
    { 0x5470786d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6907870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd07873, 43 },
    { 0x75507874, 93 },
    { 0x5ca07873, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95207878, 23 },
    { 0x32407879, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30787c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca07884, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5407887, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb30788c, 43 },
    { 0, 0 },
    { 0x2b50788e, 95 },
    { 0xf260788f, 90 },
    { 0xee0788e, 93 },
    { 0, 0 },
    { 0x7c907892, 90 },
    { 0xe9607893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc770789a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf60789e, 90 },
    { 0, 0 },
    { 0x7e6078a0, 66 },
    { 0, 0 },
    { 0x7cf078a2, 66 },
    { 0x70e078a2, 23 },
    { 0x54a078a4, 90 },
    { 0x8a4078a3, 93 },
    { 0xe6a078a6, 91 },
    { 0x4a5078a7, 99 },
    { 0, 0 },
    { 0xda7078a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483078ad, 90 },
    { 0, 0 },
    { 0x566078af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4078b6, 95 },
    { 0, 0 },
    { 0xf31078b8, 66 },
    { 0, 0 },
    { 0x93d078ba, 34 },
    { 0, 0 },
    { 0x33f078bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f078c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766078ca, 77 },
    { 0xec8078cb, 61 },
    { 0x89d078cb, 55 },
    { 0xf93078cd, 91 },
    { 0x4df078ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350078d2, 83 },
    { 0x7e9078d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7078da, 94 },
    { 0x22c078db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173078e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21078e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92078e8, 23 },
    { 0, 0 },
    { 0xbae078ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8078ed, 46 },
    { 0, 0 },
    { 0xb90078ef, 90 },
    { 0x4ca078f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5078f7, 95 },
    { 0x14c078f7, 33 },
    { 0x4a0078f9, 85 },
    { 0x2b8078f9, 76 },
    { 0xbb4078fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d807902, 33 },
    { 0xebf07902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd0790f, 95 },
    { 0, 0 },
    { 0xe6907911, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43007918, 46 },
    { 0x72a07918, 23 },
    { 0x9890791a, 90 },
    { 0x1ec0791b, 95 },
    { 0x41e0791c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc807920, 76 },
    { 0, 0 },
    { 0xe9207922, 98 },
    { 0x38107923, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b07926, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0792e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20907934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b0793a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c07944, 84 },
    { 0x72007945, 92 },
    { 0x50e07945, 23 },
    { 0xe8007947, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf620794c, 55 },
    { 0, 0 },
    { 0xa630794e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe507952, 88 },
    { 0, 0 },
    { 0xbf307954, 66 },
    { 0x63107955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b00795f, 46 },
    { 0x6fd0795f, 34 },
    { 0x59e0795f, 95 },
    { 0x71907962, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2807965, 95 },
    { 0xcc007966, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87407969, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8530796c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71707972, 33 },
    { 0x61807973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d07976, 95 },
    { 0x7b007977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed0797b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac0797f, 93 },
    { 0xff407980, 90 },
    { 0x9fc07981, 84 },
    { 0, 0 },
    { 0x96407983, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7707986, 90 },
    { 0, 0 },
    { 0x50207988, 99 },
    { 0, 0 },
    { 0x3a30798a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360798e, 77 },
    { 0xc6c0798f, 34 },
    { 0xe400798e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d507996, 17 },
    { 0x21b07997, 52 },
    { 0x2fc07998, 76 },
    { 0x45907998, 55 },
    { 0x31007998, 43 },
    { 0, 0 },
    { 0x6d80799c, 61 },
    { 0, 0 },
    { 0x9570799e, 55 },
    { 0xc330799f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x495079a2, 33 },
    { 0, 0 },
    { 0x92a079a4, 98 },
    { 0xa0a079a4, 95 },
    { 0, 0 },
    { 0xa4d079a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93079ad, 46 },
    { 0x2d0079ad, 52 },
    { 0, 0 },
    { 0x1a1079b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159079b4, 93 },
    { 0xbe7079b5, 34 },
    { 0xf079b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37079c3, 93 },
    { 0, 0 },
    { 0x8ff079c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea079cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x535079ce, 33 },
    { 0xb84079cf, 98 },
    { 0x8a0079cf, 99 },
    { 0xa36079d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03079d8, 61 },
    { 0x49b079d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40079dc, 23 },
    { 0xd28079dd, 33 },
    { 0, 0 },
    { 0xe2f079df, 23 },
    { 0, 0 },
    { 0x336079e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41079e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef079eb, 33 },
    { 0xa47079ec, 43 },
    { 0xfda079ed, 93 },
    { 0x7be079ed, 61 },
    { 0x9b079ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e079f2, 61 },
    { 0x8f8079f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f079f7, 34 },
    { 0x896079f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b079fc, 23 },
    { 0x356079fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f707a01, 67 },
    { 0xac07a02, 43 },
    { 0, 0 },
    { 0xb8207a04, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95507a0c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6607a14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c07a17, 83 },
    { 0x83b07a18, 98 },
    { 0x37e07a19, 51 },
    { 0xaf207a1a, 33 },
    { 0x9507a1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2207a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6907a21, 33 },
    { 0xcb307a21, 93 },
    { 0xc2907a23, 55 },
    { 0x42207a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b607a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5207a2d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b907a35, 90 },
    { 0xe307a36, 93 },
    { 0x7db07a37, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b07a3b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8007a41, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb07a44, 95 },
    { 0x48f07a44, 43 },
    { 0xfcf07a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2107a4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3807a51, 33 },
    { 0, 0 },
    { 0xe907a53, 55 },
    { 0xcc807a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc07a57, 69 },
    { 0, 0 },
    { 0xb0907a59, 23 },
    { 0xe3e07a5a, 55 },
    { 0xeeb07a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93907a5e, 33 },
    { 0, 0 },
    { 0xa4807a60, 95 },
    { 0x37407a60, 61 },
    { 0xfc207a62, 23 },
    { 0, 0 },
    { 0xfff07a64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62407a68, 33 },
    { 0xf2c07a69, 95 },
    { 0x52207a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a07a6d, 24 },
    { 0x97807a6e, 90 },
    { 0x74507a6f, 84 },
    { 0x65307a6f, 90 },
    { 0xa9d07a71, 90 },
    { 0xdc007a72, 90 },
    { 0xdf107a73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c507a77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c307a7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f307a7e, 43 },
    { 0, 0 },
    { 0x5b207a80, 87 },
    { 0xd3007a81, 23 },
    { 0, 0 },
    { 0xef407a83, 93 },
    { 0x28207a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83607a8d, 23 },
    { 0xeea07a8e, 34 },
    { 0xa7807a8f, 33 },
    { 0x87f07a90, 61 },
    { 0x96c07a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66407a94, 99 },
    { 0xf6c07a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97307a98, 23 },
    { 0, 0 },
    { 0xb9307a9a, 84 },
    { 0, 0 },
    { 0xbe907a9c, 33 },
    { 0x9ef07a9d, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x7807aa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab007aa3, 98 },
    { 0x7c207aa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5907aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c507aac, 90 },
    { 0xd2e07aad, 77 },
    { 0x7f407aae, 82 },
    { 0x77507aae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4107ab2, 33 },
    { 0xcd007ab3, 43 },
    { 0xbd807ab3, 69 },
    { 0xb9507ab4, 76 },
    { 0xf8907ab6, 85 },
    { 0, 0 },
    { 0x81207ab8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c107abc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c07ac4, 90 },
    { 0xba907ac4, 46 },
    { 0xa8e07ac5, 77 },
    { 0xddf07ac5, 61 },
    { 0x36207ac8, 43 },
    { 0x54207ac9, 23 },
    { 0x3d107ac7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5007acd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b07ad1, 65 },
    { 0x17a07ad1, 33 },
    { 0x9607ad3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b307adb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3207ae5, 98 },
    { 0, 0 },
    { 0xaa607ae7, 93 },
    { 0x56007ae8, 65 },
    { 0xdfe07ae9, 98 },
    { 0, 0 },
    { 0x29f07aeb, 78 },
    { 0x8a907aec, 97 },
    { 0xb207aed, 23 },
    { 0, 0 },
    { 0xa3a07aef, 33 },
    { 0, 0 },
    { 0x16207af1, 55 },
    { 0x84407af2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f07af9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a07b00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3207b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e507b0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a07b11, 43 },
    { 0, 0 },
    { 0xe4207b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a07b16, 61 },
    { 0, 0 },
    { 0xc7507b18, 95 },
    { 0, 0 },
    { 0xac307b1a, 55 },
    { 0x2d907b1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc07b1e, 90 },
    { 0xddf07b1f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8307b25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac607b2d, 52 },
    { 0xf2707b2e, 85 },
    { 0x97d07b2d, 90 },
    { 0x80907b2d, 78 },
    { 0xbd307b2f, 93 },
    { 0xbed07b32, 17 },
    { 0x23907b32, 61 },
    { 0x1c007b32, 23 },
    { 0x4cf07b35, 93 },
    { 0x5e107b36, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a07b3a, 33 },
    { 0xec607b3a, 55 },
    { 0xa5407b3b, 43 },
    { 0x82507b3a, 44 },
    { 0x5ec07b3c, 65 },
    { 0x51307b3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf307b43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20507b4a, 61 },
    { 0x5a707b4b, 51 },
    { 0xafc07b4c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c07b4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62007b55, 75 },
    { 0, 0 },
    { 0x3d207b57, 55 },
    { 0x76107b58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c07b5b, 23 },
    { 0xe3407b5b, 85 },
    { 0, 0 },
    { 0x90f07b5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0107b61, 33 },
    { 0, 0 },
    { 0x5f107b63, 46 },
    { 0, 0 },
    { 0xe007b65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f07b68, 61 },
    { 0x97e07b69, 43 },
    { 0x8607b6a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e607b6d, 91 },
    { 0, 0 },
    { 0xc6f07b6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3907b74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf107b7e, 55 },
    { 0, 0 },
    { 0x51307b80, 55 },
    { 0x30007b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f07b84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e407b8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe507b8e, 43 },
    { 0x7e007b8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f807b92, 93 },
    { 0x1307b93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c07b97, 98 },
    { 0xf0907b97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed07b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a07ba4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25107ba8, 95 },
    { 0, 0 },
    { 0xc8107baa, 51 },
    { 0, 0 },
    { 0xcfa07bac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40307bb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb07bb5, 100 },
    { 0x2c007bb6, 52 },
    { 0x25b07bb6, 61 },
    { 0xfca07bb8, 98 },
    { 0xce07bb9, 46 },
    { 0xf2607bba, 23 },
    { 0x40c07bb8, 67 },
    { 0, 0 },
    { 0x73307bbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed07bc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x25707bc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce07bc8, 34 },
    { 0, 0 },
    { 0xe0d07bca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54007bce, 55 },
    { 0xc0a07bcf, 23 },
    { 0x32907bcf, 95 },
    { 0x91407bd1, 98 },
    { 0x25707bce, 55 },
    { 0, 0 },
    { 0x4e507bd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a07bd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6607bda, 33 },
    { 0, 0 },
    { 0xece07bdc, 84 },
    { 0xe0707bdd, 95 },
    { 0x99007bde, 98 },
    { 0xd3907bdf, 61 },
    { 0xc9007be0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89707be6, 90 },
    { 0, 0 },
    { 0xe6007be8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b07bec, 98 },
    { 0, 0 },
    { 0x99307bee, 88 },
    { 0x91407bef, 98 },
    { 0x77d07bf0, 100 },
    { 0x54f07bf1, 56 },
    { 0xf5207bf1, 33 },
    { 0xb5807bf3, 23 },
    { 0x75c07bf4, 76 },
    { 0, 0 },
    { 0x47607bf6, 75 },
    { 0x9be07bf7, 80 },
    { 0xf3607bf8, 61 },
    { 0x8be07bf9, 76 },
    { 0xa4e07bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb07c01, 38 },
    { 0x3907c02, 90 },
    { 0x78207c02, 33 },
    { 0x47907c03, 76 },
    { 0x57107c05, 95 },
    { 0x91807c06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa907c0f, 100 },
    { 0x80e07c10, 23 },
    { 0x8a707c10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf807c14, 90 },
    { 0x7e607c14, 76 },
    { 0, 0 },
    { 0xb6507c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de07c1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b07c20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6107c23, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de07c28, 46 },
    { 0x20c07c29, 33 },
    { 0, 0 },
    { 0xfe007c2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2207c2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3107c31, 61 },
    { 0xcf507c32, 68 },
    { 0x15d07c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1007c36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2307c3a, 84 },
    { 0, 0 },
    { 0x34607c3c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d07c43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77007c47, 98 },
    { 0xbc307c48, 61 },
    { 0x92807c48, 61 },
    { 0xe4f07c4a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa07c53, 55 },
    { 0x20e07c54, 68 },
    { 0xf0c07c54, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe07c59, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f07c5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e107c60, 77 },
    { 0xdf307c60, 43 },
    { 0x92c07c62, 33 },
    { 0xa5e07c63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c07c68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd07c6d, 98 },
    { 0x86607c6e, 88 },
    { 0x59c07c6f, 99 },
    { 0x5a07c70, 23 },
    { 0xa5a07c71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f07c75, 90 },
    { 0xd2b07c75, 98 },
    { 0xd3307c76, 46 },
    { 0xed407c78, 67 },
    { 0, 0 },
    { 0xadc07c7a, 61 },
    { 0x4c007c7b, 76 },
    { 0x86b07c7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3907c7f, 90 },
    { 0xd2a07c7f, 46 },
    { 0xe8f07c81, 51 },
    { 0xfaf07c82, 23 },
    { 0x2f207c83, 87 },
    { 0, 0 },
    { 0xa3907c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c07c89, 61 },
    { 0x6e107c89, 90 },
    { 0x99a07c89, 70 },
    { 0, 0 },
    { 0x1c207c8d, 78 },
    { 0xb2f07c8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42707c91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b07c97, 99 },
    { 0x5f07c98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a407c9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a107ca6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e407caa, 61 },
    { 0, 0 },
    { 0x2af07cac, 98 },
    { 0xdb307cac, 55 },
    { 0x7a907cad, 34 },
    { 0x1da07caf, 34 },
    { 0xd1207caf, 90 },
    { 0x6d107cb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe007cb6, 43 },
    { 0x67507cb7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad07cbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa07cbf, 67 },
    { 0, 0 },
    { 0xbed07cc1, 98 },
    { 0, 0 },
    { 0xeac07cc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1307cc7, 95 },
    { 0, 0 },
    { 0x58707cc9, 61 },
    { 0x89907cc9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28407ccf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a07cd4, 76 },
    { 0, 0 },
    { 0xfea07cd6, 90 },
    { 0x12407cd7, 23 },
    { 0, 0 },
    { 0xd5907cd9, 23 },
    { 0x45e07cda, 61 },
    { 0x33d07cdb, 98 },
    { 0x28e07cd9, 90 },
    { 0xcf507cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69907ce3, 91 },
    { 0, 0 },
    { 0xd4a07ce5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf307ce8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d07cee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de07cf4, 80 },
    { 0, 0 },
    { 0x73f07cf6, 61 },
    { 0xf8407cf6, 33 },
    { 0x79707cf6, 66 },
    { 0x79507cf9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7007cfc, 75 },
    { 0, 0 },
    { 0x2d507cfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87807d01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd907d04, 33 },
    { 0, 0 },
    { 0x8dc07d06, 33 },
    { 0x1e807d06, 98 },
    { 0xf2a07d08, 90 },
    { 0, 0 },
    { 0xd6a07d0a, 33 },
    { 0xd7707d0b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f07d0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f07d12, 43 },
    { 0, 0 },
    { 0x9d207d14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73607d1c, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f07d22, 75 },
    { 0x98407d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef107d2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x93707d2d, 34 },
    { 0x50e07d2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a07d33, 95 },
    { 0, 0 },
    { 0x62507d35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df07d3e, 98 },
    { 0, 0 },
    { 0x6a807d40, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce07d44, 61 },
    { 0xd3c07d45, 43 },
    { 0, 0 },
    { 0x9c907d47, 100 },
    { 0x42d07d48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62607d4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xff07d4e, 33 },
    { 0x61607d4f, 33 },
    { 0x7b607d50, 81 },
    { 0x80507d4e, 95 },
    { 0x74607d52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58107d55, 22 },
    { 0, 0 },
    { 0x72207d57, 77 },
    { 0, 0 },
    { 0x38207d59, 55 },
    { 0xa8e07d59, 33 },
    { 0xbfe07d5b, 95 },
    { 0xb9c07d5c, 55 },
    { 0, 0 },
    { 0x28207d5e, 55 },
    { 0x86607d5f, 98 },
    { 0x63f07d60, 23 },
    { 0x29207d60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1807d67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b07d6c, 100 },
    { 0x19107d6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28207d71, 23 },
    { 0xd3407d72, 75 },
    { 0xc4207d73, 90 },
    { 0xa2a07d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43007d78, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007d7f, 93 },
    { 0x50107d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4607d84, 98 },
    { 0xdda07d85, 65 },
    { 0xdf907d85, 77 },
    { 0, 0 },
    { 0x52007d88, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5907d8d, 33 },
    { 0xa4207d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94407d91, 43 },
    { 0x91107d92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e07d9c, 100 },
    { 0, 0 },
    { 0xad507d9e, 23 },
    { 0x3bd07d9f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc07da5, 95 },
    { 0xa7607da6, 76 },
    { 0x91607da6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f07daa, 23 },
    { 0, 0 },
    { 0x5ab07dac, 68 },
    { 0, 0 },
    { 0x11507dae, 33 },
    { 0, 0 },
    { 0x5f507db0, 69 },
    { 0x94e07db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e707dbd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f507dc0, 87 },
    { 0xc6b07dc1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca07dc6, 23 },
    { 0, 0 },
    { 0xde107dc8, 95 },
    { 0, 0 },
    { 0xed407dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7207dd6, 90 },
    { 0, 0 },
    { 0x6f407dd8, 67 },
    { 0xdd607dd8, 95 },
    { 0x91807dda, 23 },
    { 0, 0 },
    { 0xc0b07ddc, 95 },
    { 0x1c007ddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36107de2, 95 },
    { 0, 0 },
    { 0xfe707de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007dea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6107ded, 98 },
    { 0, 0 },
    { 0x3a307def, 76 },
    { 0, 0 },
    { 0xcf007df1, 33 },
    { 0x14d07df2, 78 },
    { 0, 0 },
    { 0x9f407df4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef07dfb, 98 },
    { 0, 0 },
    { 0x2d407dfd, 90 },
    { 0xd7307dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d307e04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee07e07, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf607e0c, 76 },
    { 0x16307e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df07e13, 66 },
    { 0x63307e14, 61 },
    { 0xbfb07e14, 90 },
    { 0x1e107e16, 43 },
    { 0x59e07e17, 61 },
    { 0x1c407e18, 84 },
    { 0x79407e18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d07e22, 33 },
    { 0xd9f07e23, 99 },
    { 0xa8807e24, 98 },
    { 0xe2a07e24, 33 },
    { 0xf8a07e26, 98 },
    { 0xfde07e27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae07e2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc07e2e, 33 },
    { 0x66907e2f, 61 },
    { 0xce007e30, 23 },
    { 0x4bc07e30, 93 },
    { 0xa2107e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe307e38, 46 },
    { 0xa5607e39, 61 },
    { 0xd5907e3a, 61 },
    { 0x4e607e3b, 23 },
    { 0xc9c07e3a, 61 },
    { 0x38107e3a, 34 },
    { 0x8fc07e3e, 61 },
    { 0xe9507e3f, 17 },
    { 0x35707e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10707e43, 91 },
    { 0x81d07e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe07e4d, 33 },
    { 0xb3c07e4e, 84 },
    { 0xe9607e4f, 33 },
    { 0xec507e4d, 95 },
    { 0, 0 },
    { 0x9b707e52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0307e5d, 90 },
    { 0, 0 },
    { 0x9aa07e5f, 65 },
    { 0xd3d07e60, 61 },
    { 0xed107e61, 46 },
    { 0x96607e5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f407e69, 61 },
    { 0x9ed07e6a, 33 },
    { 0xd8e07e6b, 23 },
    { 0x84807e6a, 98 },
    { 0xb607e69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b07e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f07e74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c107e79, 34 },
    { 0x3e207e7a, 34 },
    { 0, 0 },
    { 0xf7e07e7c, 34 },
    { 0x91407e7d, 93 },
    { 0, 0 },
    { 0xb407e7f, 78 },
    { 0x95c07e80, 61 },
    { 0xbb507e80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61907e84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf07e88, 46 },
    { 0x5a907e88, 83 },
    { 0x92007e8a, 47 },
    { 0, 0 },
    { 0xefa07e8c, 85 },
    { 0x1d07e8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45607e91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb07e95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b707e9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11807ea1, 98 },
    { 0x94407ea2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d07eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f707eae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95607eba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f07ebe, 90 },
    { 0, 0 },
    { 0x35407ec0, 55 },
    { 0x41507ec1, 33 },
    { 0x71d07ec2, 100 },
    { 0, 0 },
    { 0xde607ec4, 100 },
    { 0xb4f07ec5, 90 },
    { 0, 0 },
    { 0xd0907ec7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f307ecc, 90 },
    { 0xb1807ecc, 95 },
    { 0x2b07ece, 90 },
    { 0x67707ecf, 33 },
    { 0xe907ece, 33 },
    { 0, 0 },
    { 0xb9307ed2, 95 },
    { 0, 0 },
    { 0xedf07ed4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa807edb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93807ede, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa07ee1, 69 },
    { 0x42307ee2, 33 },
    { 0x61a07ee3, 90 },
    { 0x6b507ee4, 90 },
    { 0x6d307ee3, 66 },
    { 0x53707ee6, 43 },
    { 0x15707ee3, 33 },
    { 0xafb07ee8, 34 },
    { 0x9b007ee5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8207eed, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9007ef2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a07eff, 52 },
    { 0x54507f00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9807f07, 34 },
    { 0x88007f08, 95 },
    { 0x7e107f07, 23 },
    { 0, 0 },
    { 0xc3407f0b, 43 },
    { 0xd8c07f0c, 90 },
    { 0xd7407f0c, 78 },
    { 0x25907f0e, 52 },
    { 0, 0 },
    { 0xec807f10, 97 },
    { 0, 0 },
    { 0x7ed07f12, 33 },
    { 0xc0a07f12, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb07f17, 84 },
    { 0xfad07f18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e07f1b, 76 },
    { 0xc4a07f1c, 85 },
    { 0xde407f1c, 61 },
    { 0x31507f1e, 64 },
    { 0x59b07f1f, 76 },
    { 0x4a007f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e07f25, 61 },
    { 0x93307f25, 90 },
    { 0xf7407f27, 99 },
    { 0x11b07f27, 34 },
    { 0xf1507f27, 95 },
    { 0xd0907f28, 43 },
    { 0x20507f28, 100 },
    { 0x6fc07f27, 99 },
    { 0x49607f2d, 95 },
    { 0xffb07f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b07f31, 85 },
    { 0, 0 },
    { 0xf1407f33, 93 },
    { 0x74507f34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea607f3f, 76 },
    { 0, 0 },
    { 0x7a707f41, 61 },
    { 0, 0 },
    { 0x96507f43, 43 },
    { 0, 0 },
    { 0x47407f45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd507f48, 76 },
    { 0x6be07f48, 33 },
    { 0xdc007f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3807f4f, 33 },
    { 0, 0 },
    { 0xc0d07f51, 33 },
    { 0xc6507f51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c907f55, 91 },
    { 0x36907f56, 83 },
    { 0x17407f57, 95 },
    { 0x7107f58, 23 },
    { 0xff907f59, 23 },
    { 0x83707f5a, 98 },
    { 0x82707f5b, 55 },
    { 0, 0 },
    { 0x27a07f5d, 48 },
    { 0, 0 },
    { 0xa6f07f5f, 93 },
    { 0x78a07f60, 76 },
    { 0, 0 },
    { 0xdbe07f62, 33 },
    { 0xd8607f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce807f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44207f6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f07f70, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe007f75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c07f79, 81 },
    { 0, 0 },
    { 0x85907f7b, 61 },
    { 0, 0 },
    { 0xd4107f7d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf07f86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca07f8a, 87 },
    { 0x56b07f8b, 90 },
    { 0x9d007f8c, 76 },
    { 0, 0 },
    { 0x71a07f8e, 90 },
    { 0x40307f8f, 61 },
    { 0xa9207f90, 66 },
    { 0x1db07f91, 90 },
    { 0xb6407f8f, 80 },
    { 0, 0 },
    { 0x3ef07f94, 67 },
    { 0xc2b07f95, 23 },
    { 0x1a907f95, 85 },
    { 0x9fc07f97, 23 },
    { 0x66b07f98, 90 },
    { 0xcea07f99, 98 },
    { 0x9e607f9a, 34 },
    { 0xf9a07f98, 23 },
    { 0x8c707f96, 91 },
    { 0x59d07f9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3807fa6, 61 },
    { 0xdf107fa6, 90 },
    { 0, 0 },
    { 0x10d07fa9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b407fac, 61 },
    { 0x60307fac, 97 },
    { 0x6c07fac, 91 },
    { 0x73f07fac, 67 },
    { 0x91007fb0, 49 },
    { 0, 0 },
    { 0xf7907fb2, 33 },
    { 0x30d07fb2, 23 },
    { 0, 0 },
    { 0x2da07fb5, 34 },
    { 0x3fa07fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca07fb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62307fbd, 23 },
    { 0x25707fbe, 15 },
    { 0x10707fbe, 33 },
    { 0xd3007fbd, 91 },
    { 0x6e707fc1, 99 },
    { 0x9fb07fc2, 86 },
    { 0xe0807fc0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac07fc6, 90 },
    { 0xadf07fc7, 98 },
    { 0x6be07fc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d07fce, 98 },
    { 0xade07fcf, 43 },
    { 0xe4607fd0, 33 },
    { 0x5dc07fd1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e207fd4, 34 },
    { 0, 0 },
    { 0xcd807fd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd107fdb, 90 },
    { 0x29d07fdc, 33 },
    { 0, 0 },
    { 0x3d207fde, 34 },
    { 0, 0 },
    { 0x5fd07fe0, 66 },
    { 0x2ce07fe1, 76 },
    { 0, 0 },
    { 0x4c207fe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1807fe6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d07fed, 95 },
    { 0, 0 },
    { 0xabd07fef, 23 },
    { 0, 0 },
    { 0xc5c07ff1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5007ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c07fff, 80 },
    { 0, 0 },
    { 0x8e008001, 33 },
    { 0x8e608001, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa708005, 90 },
    { 0x29d08006, 85 },
    { 0, 0 },
    { 0xb6f08008, 95 },
    { 0, 0 },
    { 0xb6f0800a, 33 },
    { 0, 0 },
    { 0x4fb0800c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5620800f, 55 },
    { 0, 0 },
    { 0x5c708011, 23 },
    { 0, 0 },
    { 0x37d08013, 90 },
    { 0xc3d08014, 95 },
    { 0, 0 },
    { 0x42e08016, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c508019, 61 },
    { 0x16e08019, 66 },
    { 0x1f00801b, 23 },
    { 0x5d50801c, 46 },
    { 0x8f70801c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7308025, 99 },
    { 0x2dc08026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8008029, 95 },
    { 0x65b0802a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a0802d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27608034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64108037, 33 },
    { 0x4d908037, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x180803b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77108041, 33 },
    { 0xf7208041, 87 },
    { 0xdfe08043, 61 },
    { 0x3b208043, 33 },
    { 0xae708045, 84 },
    { 0x3f108043, 95 },
    { 0x67508047, 78 },
    { 0x5c08048, 34 },
    { 0x41b08045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87808050, 46 },
    { 0, 0 },
    { 0xfb708052, 93 },
    { 0x7308053, 91 },
    { 0xc8708053, 98 },
    { 0x50308054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e08058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc0805b, 33 },
    { 0x95e0805b, 78 },
    { 0xfb90805c, 61 },
    { 0, 0 },
    { 0xd830805f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba08063, 100 },
    { 0x8b808064, 90 },
    { 0, 0 },
    { 0x1c108066, 23 },
    { 0x26808066, 76 },
    { 0, 0 },
    { 0xa1c08069, 61 },
    { 0x2c60806a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf0806f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c208072, 76 },
    { 0x6e908073, 43 },
    { 0, 0 },
    { 0x5dd08075, 55 },
    { 0x3fe08075, 90 },
    { 0, 0 },
    { 0x6eb08078, 46 },
    { 0x6c408078, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b20807d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af08082, 46 },
    { 0xf2808083, 33 },
    { 0x82908084, 34 },
    { 0x7f308082, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e08088, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f90808e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4108094, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d0809b, 87 },
    { 0, 0 },
    { 0x990809d, 55 },
    { 0x7b0809d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc080a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66080a7, 85 },
    { 0xdc2080a7, 98 },
    { 0x8ac080a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51080ac, 87 },
    { 0, 0 },
    { 0x1c080ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x173080b1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43080b4, 95 },
    { 0xfab080b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04080bb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4080be, 61 },
    { 0, 0 },
    { 0x80e080c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592080cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b080d0, 61 },
    { 0, 0 },
    { 0x960080d2, 91 },
    { 0x92080d3, 68 },
    { 0xc15080d3, 43 },
    { 0xfd3080d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8080d8, 76 },
    { 0, 0 },
    { 0x9c9080da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332080df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd080e2, 17 },
    { 0, 0 },
    { 0x69080e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3080e7, 73 },
    { 0xa03080e7, 23 },
    { 0x21b080e9, 23 },
    { 0, 0 },
    { 0x4ac080eb, 98 },
    { 0, 0 },
    { 0xcd4080ed, 55 },
    { 0xaa080ee, 100 },
    { 0, 0 },
    { 0xa02080f0, 90 },
    { 0, 0 },
    { 0x20a080f2, 64 },
    { 0x46e080f3, 80 },
    { 0xeb4080f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d080f8, 36 },
    { 0, 0 },
    { 0x6c080fa, 55 },
    { 0x5b1080fa, 33 },
    { 0xfcc080fb, 23 },
    { 0, 0 },
    { 0xb59080fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f08102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb08109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710810f, 78 },
    { 0xbb00810f, 95 },
    { 0xa4c08111, 80 },
    { 0xd1508110, 100 },
    { 0x1b408113, 33 },
    { 0xeea08110, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c008118, 95 },
    { 0, 0 },
    { 0x8390811a, 90 },
    { 0, 0 },
    { 0x9ac0811c, 77 },
    { 0x6430811d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d08120, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42908127, 52 },
    { 0, 0 },
    { 0x53c08129, 90 },
    { 0xe7b08129, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df08130, 90 },
    { 0x31e08131, 81 },
    { 0x78808132, 76 },
    { 0x29d08132, 43 },
    { 0x1ee08134, 64 },
    { 0x52b08135, 98 },
    { 0x85908136, 91 },
    { 0x70608135, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a0813b, 23 },
    { 0xe390813c, 52 },
    { 0x7890813d, 98 },
    { 0x2f10813d, 90 },
    { 0, 0 },
    { 0x1e108140, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c408143, 46 },
    { 0x73508144, 97 },
    { 0xdc608145, 61 },
    { 0x40b08146, 61 },
    { 0, 0 },
    { 0x49b08148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f0814c, 95 },
    { 0x8f00814d, 61 },
    { 0, 0 },
    { 0xb410814f, 76 },
    { 0xda908150, 52 },
    { 0x9fd0814f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7108155, 34 },
    { 0, 0 },
    { 0x1db08157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b0815e, 98 },
    { 0, 0 },
    { 0xfe708160, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e508163, 90 },
    { 0xdfb08164, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a408167, 99 },
    { 0xc308168, 99 },
    { 0xfc708168, 55 },
    { 0x74b0816a, 68 },
    { 0x87008168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c08170, 23 },
    { 0x46908171, 33 },
    { 0x8b008172, 88 },
    { 0xfa108172, 33 },
    { 0x41508173, 87 },
    { 0, 0 },
    { 0x6da08176, 98 },
    { 0x1a808176, 90 },
    { 0, 0 },
    { 0x1db08179, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe0817d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a108180, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x56408183, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f50818e, 33 },
    { 0, 0 },
    { 0xdac08190, 95 },
    { 0x64308191, 95 },
    { 0xac108192, 95 },
    { 0x4d408192, 90 },
    { 0xa5708192, 76 },
    { 0x52008190, 84 },
    { 0x6b808196, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa700819d, 33 },
    { 0xf7f0819d, 51 },
    { 0, 0 },
    { 0x9d2081a0, 23 },
    { 0x453081a1, 98 },
    { 0x8fa081a2, 34 },
    { 0, 0 },
    { 0x8d9081a4, 69 },
    { 0, 0 },
    { 0x27a081a6, 23 },
    { 0, 0 },
    { 0xdea081a8, 61 },
    { 0x9c4081a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x233081ac, 52 },
    { 0xb8b081ad, 95 },
    { 0, 0 },
    { 0x8e081af, 78 },
    { 0, 0 },
    { 0x39081b1, 61 },
    { 0x943081b1, 76 },
    { 0, 0 },
    { 0x461081b4, 33 },
    { 0xe4d081b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41081b8, 48 },
    { 0xf21081b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb081bd, 43 },
    { 0xdd7081be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae081c5, 23 },
    { 0x120081c5, 17 },
    { 0, 0 },
    { 0xca4081c8, 53 },
    { 0, 0 },
    { 0x8eb081ca, 90 },
    { 0x8fb081cb, 61 },
    { 0x656081cc, 61 },
    { 0x569081cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb081d0, 35 },
    { 0x9ee081d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d081d6, 90 },
    { 0, 0 },
    { 0xa44081d8, 34 },
    { 0xc38081d8, 93 },
    { 0x6ba081d9, 95 },
    { 0, 0 },
    { 0x822081dc, 33 },
    { 0x1c8081dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x519081e0, 90 },
    { 0xb6d081e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616081e8, 61 },
    { 0x971081e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4081ec, 68 },
    { 0, 0 },
    { 0xbf081ee, 90 },
    { 0xb09081ef, 61 },
    { 0xf3a081f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209081f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a081fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc008203, 61 },
    { 0, 0 },
    { 0x31908205, 90 },
    { 0xe7508206, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a30820d, 33 },
    { 0, 0 },
    { 0x3c40820f, 61 },
    { 0x9c80820f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e308214, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73108218, 77 },
    { 0, 0 },
    { 0x1bd0821a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc508220, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f708228, 61 },
    { 0xf7d08229, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a0822c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61908230, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc680823e, 98 },
    { 0x88d0823f, 61 },
    { 0x70e08240, 23 },
    { 0x6f00823f, 23 },
    { 0, 0 },
    { 0x20908243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2708246, 81 },
    { 0x16808246, 23 },
    { 0, 0 },
    { 0xa0208249, 43 },
    { 0, 0 },
    { 0xcba0824b, 98 },
    { 0x6e0824b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58508252, 53 },
    { 0x8e608253, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xca508256, 87 },
    { 0xce108257, 77 },
    { 0x40d08258, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60825d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c08263, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c208268, 99 },
    { 0x66908269, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb470826f, 95 },
    { 0, 0 },
    { 0x69708271, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed608275, 90 },
    { 0, 0 },
    { 0x9f808277, 23 },
    { 0x9f708278, 52 },
    { 0xf6208278, 61 },
    { 0, 0 },
    { 0xb030827b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2390827e, 90 },
    { 0xed90827f, 78 },
    { 0x64208280, 90 },
    { 0xab708281, 85 },
    { 0x54c08282, 76 },
    { 0, 0 },
    { 0xc7408284, 90 },
    { 0xc3708285, 88 },
    { 0xf4a08286, 90 },
    { 0xcf608287, 33 },
    { 0x76608285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d0828d, 98 },
    { 0, 0 },
    { 0x310828f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10008292, 43 },
    { 0x76308292, 69 },
    { 0xbcb08293, 33 },
    { 0x69408292, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c90829f, 98 },
    { 0, 0 },
    { 0xc2a082a1, 46 },
    { 0x19c082a2, 52 },
    { 0x4ae082a3, 33 },
    { 0x3f2082a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82082aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b082b0, 91 },
    { 0x8b9082b0, 93 },
    { 0x888082b0, 55 },
    { 0, 0 },
    { 0xb26082b4, 95 },
    { 0, 0 },
    { 0x8c6082b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2082ba, 32 },
    { 0, 0 },
    { 0x6fe082bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d082c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b082c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7082c9, 61 },
    { 0x7cf082ca, 90 },
    { 0x914082cb, 55 },
    { 0, 0 },
    { 0xde7082cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b082d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b082d9, 46 },
    { 0xb69082d9, 61 },
    { 0xa9082db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x522082de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b082e1, 33 },
    { 0x415082e2, 52 },
    { 0xd9082e3, 33 },
    { 0, 0 },
    { 0x5082e5, 90 },
    { 0, 0 },
    { 0x662082e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718082ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2082f5, 96 },
    { 0x3d082f6, 91 },
    { 0x745082f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859082fb, 91 },
    { 0, 0 },
    { 0xf38082fd, 95 },
    { 0x13e082fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af08306, 76 },
    { 0x6cd08307, 66 },
    { 0, 0 },
    { 0xb5e08309, 99 },
    { 0x6d30830a, 88 },
    { 0xeaf0830a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d0830e, 61 },
    { 0xc440830f, 93 },
    { 0x9c008310, 81 },
    { 0xa8f08311, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd108314, 61 },
    { 0, 0 },
    { 0x85108316, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21208319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3120831c, 81 },
    { 0x4e40831d, 46 },
    { 0xf620831c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e08323, 46 },
    { 0xbcd08324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0808329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe110832c, 95 },
    { 0, 0 },
    { 0x4a80832e, 91 },
    { 0, 0 },
    { 0xef308330, 61 },
    { 0xaca08331, 95 },
    { 0x7f208332, 75 },
    { 0x17608333, 76 },
    { 0xa5b08334, 95 },
    { 0x3d008335, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3740833c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff308343, 33 },
    { 0x6be08344, 95 },
    { 0x5a408345, 33 },
    { 0xe2908346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d0834a, 33 },
    { 0, 0 },
    { 0x4400834c, 90 },
    { 0xcd70834c, 61 },
    { 0, 0 },
    { 0x9110834f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x52808352, 98 },
    { 0x34a08353, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f08358, 95 },
    { 0xa008359, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda0835e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7508361, 95 },
    { 0xece08362, 98 },
    { 0, 0 },
    { 0x8ff08364, 33 },
    { 0xda708364, 43 },
    { 0x54908366, 53 },
    { 0x1ad08367, 19 },
    { 0x1bd08367, 46 },
    { 0xa0508368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb650836d, 33 },
    { 0xcbe0836e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e08371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f708375, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f0837b, 90 },
    { 0x8bb0837c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d08382, 76 },
    { 0x8b208382, 46 },
    { 0x27008384, 90 },
    { 0, 0 },
    { 0xa8508386, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d108392, 93 },
    { 0xdf508392, 67 },
    { 0xf7208394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a08397, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb0839f, 90 },
    { 0, 0 },
    { 0x1a083a1, 61 },
    { 0xa4e083a1, 76 },
    { 0x2b5083a3, 76 },
    { 0, 0 },
    { 0xef8083a5, 64 },
    { 0x639083a5, 95 },
    { 0x1d7083a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc083aa, 90 },
    { 0x11e083aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8083ae, 33 },
    { 0x569083af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27083b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb083b5, 77 },
    { 0x152083b6, 90 },
    { 0, 0 },
    { 0x760083b8, 23 },
    { 0, 0 },
    { 0xaac083ba, 23 },
    { 0x9be083ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda083be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b083c2, 98 },
    { 0xa58083c3, 52 },
    { 0xf86083c4, 90 },
    { 0x87d083c3, 68 },
    { 0x204083c6, 38 },
    { 0x3c1083c3, 66 },
    { 0x5a5083c3, 76 },
    { 0xb41083c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30083ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb083d6, 43 },
    { 0x21b083d7, 80 },
    { 0xecf083d8, 33 },
    { 0x9db083d7, 99 },
    { 0xfa7083da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6083df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a083e2, 95 },
    { 0x88d083e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c083e6, 94 },
    { 0, 0 },
    { 0xc2b083e8, 100 },
    { 0, 0 },
    { 0x489083ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129083f3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41083f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28308400, 55 },
    { 0, 0 },
    { 0x66808402, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f08406, 33 },
    { 0x9f608407, 52 },
    { 0, 0 },
    { 0x61708409, 17 },
    { 0, 0 },
    { 0x14e0840b, 34 },
    { 0xed80840b, 52 },
    { 0x1770840c, 90 },
    { 0x7e00840e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb308411, 98 },
    { 0, 0 },
    { 0x38408413, 90 },
    { 0x81b08414, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91908418, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb320841c, 61 },
    { 0x4f50841c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c08420, 90 },
    { 0x46708421, 34 },
    { 0xffb08421, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e08427, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0842b, 76 },
    { 0, 0 },
    { 0x9d30842d, 90 },
    { 0xaa90842d, 33 },
    { 0x9440842d, 67 },
    { 0x6708430, 61 },
    { 0, 0 },
    { 0xeb008432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2508435, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59108438, 68 },
    { 0, 0 },
    { 0x2920843a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4740843e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x12708441, 98 },
    { 0xcbf08441, 46 },
    { 0x7ff08443, 93 },
    { 0x6d308444, 100 },
    { 0xa8308445, 33 },
    { 0x84d08446, 90 },
    { 0xa2208447, 61 },
    { 0xeab08448, 76 },
    { 0, 0 },
    { 0x8be0844a, 90 },
    { 0, 0 },
    { 0xc40844c, 61 },
    { 0xcd40844d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0108452, 68 },
    { 0x21308453, 98 },
    { 0x88908453, 90 },
    { 0, 0 },
    { 0xab608456, 61 },
    { 0x1c08457, 61 },
    { 0xc3908458, 98 },
    { 0x73008457, 84 },
    { 0x1a808457, 55 },
    { 0x94a0845b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f0845f, 90 },
    { 0, 0 },
    { 0xcb708461, 17 },
    { 0, 0 },
    { 0x5cd08463, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96108469, 46 },
    { 0x68108469, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f08470, 33 },
    { 0x55808471, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a08474, 33 },
    { 0xfad08475, 23 },
    { 0xe1b08475, 66 },
    { 0, 0 },
    { 0x9c808478, 23 },
    { 0, 0 },
    { 0xcf00847a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b0847e, 84 },
    { 0, 0 },
    { 0x99708480, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c08487, 34 },
    { 0xbd108487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5300848f, 23 },
    { 0x3f008490, 35 },
    { 0, 0 },
    { 0xf6308492, 33 },
    { 0xc5a08493, 23 },
    { 0xabb08494, 90 },
    { 0x64108494, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5740849b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6470849e, 98 },
    { 0xb10849e, 90 },
    { 0xd980849e, 90 },
    { 0x343084a1, 95 },
    { 0x712084a2, 78 },
    { 0xa0b084a3, 66 },
    { 0xe02084a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23084a9, 33 },
    { 0x217084a9, 75 },
    { 0xb45084ab, 90 },
    { 0, 0 },
    { 0x843084ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36084b3, 95 },
    { 0xbef084b4, 95 },
    { 0, 0 },
    { 0x8aa084b6, 76 },
    { 0, 0 },
    { 0x989084b8, 95 },
    { 0xbb9084b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18084bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x737084bf, 90 },
    { 0x3c7084bf, 61 },
    { 0xd35084c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x104084c4, 99 },
    { 0, 0 },
    { 0x79c084c6, 85 },
    { 0x9ec084c7, 23 },
    { 0x3d5084c8, 23 },
    { 0x2f1084c8, 61 },
    { 0xe95084c7, 61 },
    { 0xf74084cb, 67 },
    { 0xb7f084cc, 34 },
    { 0x25e084cc, 76 },
    { 0xcac084cb, 98 },
    { 0, 0 },
    { 0xb48084d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57084d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11084d7, 33 },
    { 0xfa3084d7, 76 },
    { 0, 0 },
    { 0x8af084da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4084dd, 77 },
    { 0x7e4084de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d084e1, 35 },
    { 0x75f084e2, 33 },
    { 0x5b7084e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577084ea, 34 },
    { 0xa36084eb, 85 },
    { 0x5b7084ea, 95 },
    { 0, 0 },
    { 0x474084ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x535084f1, 61 },
    { 0x7b084f1, 34 },
    { 0xcad084f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90084f7, 33 },
    { 0x186084f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45084fe, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b08503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c0850e, 61 },
    { 0xa220850e, 90 },
    { 0x84e0850f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100851a, 90 },
    { 0xac90851b, 33 },
    { 0x57d0851b, 23 },
    { 0, 0 },
    { 0x700851e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed08524, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d08528, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60852b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f00852e, 66 },
    { 0x92e0852f, 98 },
    { 0x9360852f, 67 },
    { 0x8008531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46908534, 99 },
    { 0x6a908535, 52 },
    { 0, 0 },
    { 0x3bb08537, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf10853e, 91 },
    { 0x8d90853f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a608543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59608547, 61 },
    { 0xd1908548, 66 },
    { 0x23a08547, 85 },
    { 0xaee0854a, 98 },
    { 0, 0 },
    { 0x3d40854c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7570854f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b708553, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa450855a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1320855f, 76 },
    { 0xf6d08560, 43 },
    { 0xe4508560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf08566, 90 },
    { 0x42f08567, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e008571, 65 },
    { 0xbcf08572, 90 },
    { 0xff308572, 55 },
    { 0, 0 },
    { 0x1bf08575, 61 },
    { 0xd9408576, 69 },
    { 0, 0 },
    { 0x8d708578, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1750857b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31308584, 33 },
    { 0, 0 },
    { 0x37d08586, 90 },
    { 0x6dd08587, 77 },
    { 0, 0 },
    { 0x99108589, 95 },
    { 0x4508589, 23 },
    { 0x3f00858b, 75 },
    { 0x4d008589, 34 },
    { 0x2ce0858d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12708590, 90 },
    { 0xf3908591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e08594, 61 },
    { 0x3e408595, 43 },
    { 0xace08595, 69 },
    { 0xc1708597, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e0859a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd900859d, 90 },
    { 0, 0 },
    { 0xc00859f, 23 },
    { 0x3d70859f, 98 },
    { 0x21085a0, 99 },
    { 0xe49085a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236085a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76085ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f085b4, 55 },
    { 0, 0 },
    { 0xdc0085b6, 99 },
    { 0x583085b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52085c0, 84 },
    { 0, 0 },
    { 0xbee085c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded085c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af085cc, 66 },
    { 0x263085cd, 46 },
    { 0x19d085ce, 55 },
    { 0x9b3085cc, 66 },
    { 0xd2a085d0, 95 },
    { 0, 0 },
    { 0xbcb085d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105085d8, 61 },
    { 0x76f085d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd085dd, 99 },
    { 0xd2d085de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db085e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba085e8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78085eb, 61 },
    { 0x968085ec, 36 },
    { 0x412085ed, 61 },
    { 0xd9c085ed, 66 },
    { 0xd05085ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60085f2, 65 },
    { 0xc00085f2, 55 },
    { 0xe5b085f4, 52 },
    { 0xf1a085f4, 43 },
    { 0xaab085f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x895085f9, 61 },
    { 0, 0 },
    { 0x6c8085fb, 33 },
    { 0xf0d085fb, 90 },
    { 0xded085fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f08602, 61 },
    { 0xfb808603, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c908606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a908610, 68 },
    { 0x69108610, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f90861a, 75 },
    { 0xb910861a, 78 },
    { 0xe670861b, 93 },
    { 0x6660861d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d608621, 98 },
    { 0x12a08622, 61 },
    { 0x8cb08623, 93 },
    { 0x68b08623, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a508627, 55 },
    { 0x30d08627, 23 },
    { 0, 0 },
    { 0x3ab0862a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b0862d, 90 },
    { 0x4de0862d, 95 },
    { 0x9da0862f, 23 },
    { 0x27608630, 76 },
    { 0xb2f0862f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d08634, 23 },
    { 0x22308635, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910863d, 88 },
    { 0x4f70863e, 67 },
    { 0x8870863e, 90 },
    { 0x2080863f, 95 },
    { 0x7cc0863e, 91 },
    { 0x4be08642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0208645, 33 },
    { 0xcbe08646, 98 },
    { 0, 0 },
    { 0x46e08648, 90 },
    { 0x98608648, 90 },
    { 0x5b508649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb20864e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b108652, 75 },
    { 0x79508652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e08656, 99 },
    { 0, 0 },
    { 0x6e08658, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0865b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7360865f, 33 },
    { 0xdca0865f, 98 },
    { 0x98f08661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee08664, 90 },
    { 0x71108665, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee208669, 69 },
    { 0x3210866a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e608671, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb08674, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e08677, 90 },
    { 0, 0 },
    { 0x65408679, 23 },
    { 0, 0 },
    { 0x3d60867b, 93 },
    { 0, 0 },
    { 0x3010867d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80208683, 100 },
    { 0x44508684, 52 },
    { 0x5cb08684, 95 },
    { 0x97d08684, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5600868f, 98 },
    { 0xb3a0868f, 100 },
    { 0xb7408691, 43 },
    { 0xcd308691, 4 },
    { 0x70708693, 33 },
    { 0x52108694, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff708697, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694086a2, 46 },
    { 0, 0 },
    { 0x69086a4, 33 },
    { 0xb23086a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a086aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45086ae, 96 },
    { 0xf6086af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405086bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32086c3, 61 },
    { 0xff9086c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e086c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478086ce, 33 },
    { 0x153086ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7086d5, 61 },
    { 0xeb5086d6, 66 },
    { 0xc04086d7, 78 },
    { 0x5fc086d5, 67 },
    { 0x8fb086d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686086e2, 61 },
    { 0x233086e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701086e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5086ea, 46 },
    { 0xf78086ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d086ee, 78 },
    { 0x593086ef, 61 },
    { 0xad4086f0, 90 },
    { 0, 0 },
    { 0x7d9086f2, 98 },
    { 0, 0 },
    { 0x39f086f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8086f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9086ff, 61 },
    { 0, 0 },
    { 0xe308701, 33 },
    { 0xb7808702, 33 },
    { 0x4f08703, 34 },
    { 0x37d08704, 33 },
    { 0xd3708705, 98 },
    { 0, 0 },
    { 0xd3008707, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b50870e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed08712, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f08715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9770871b, 81 },
    { 0xe120871c, 33 },
    { 0x720871d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e08725, 66 },
    { 0xa4208726, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea0872b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a0872e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b208731, 23 },
    { 0x7ca08731, 99 },
    { 0xd2e08733, 61 },
    { 0x43208734, 69 },
    { 0x1fd08733, 61 },
    { 0xbe808734, 23 },
    { 0xb7208737, 98 },
    { 0x25b08733, 20 },
    { 0x2bc08739, 90 },
    { 0x4e0873a, 90 },
    { 0xa430873b, 85 },
    { 0xea708739, 55 },
    { 0, 0 },
    { 0xc650873e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74808747, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x820874a, 98 },
    { 0x8bf0874b, 55 },
    { 0xf6a0874c, 90 },
    { 0x300874d, 84 },
    { 0x5710874e, 64 },
    { 0x29d0874f, 91 },
    { 0x80b08750, 95 },
    { 0x8530874d, 95 },
    { 0x6b308752, 100 },
    { 0x2050874d, 98 },
    { 0x1b308754, 76 },
    { 0, 0 },
    { 0xd9908756, 85 },
    { 0x78708756, 34 },
    { 0x9e808758, 98 },
    { 0x4ad08758, 69 },
    { 0x6310875a, 55 },
    { 0x8040875b, 61 },
    { 0, 0 },
    { 0x8d60875d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b708760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c08766, 33 },
    { 0x87208767, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c50876a, 76 },
    { 0xe690876b, 99 },
    { 0x20e0876b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e10876f, 33 },
    { 0x94708770, 33 },
    { 0xee608771, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0108774, 78 },
    { 0xcf208775, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e0877b, 50 },
    { 0, 0 },
    { 0x1770877d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34408780, 33 },
    { 0xce608781, 61 },
    { 0x7b708780, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e008785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9910878b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c08792, 95 },
    { 0x90f08792, 76 },
    { 0x90508794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca08798, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1860879b, 98 },
    { 0x30879b, 23 },
    { 0xa810879d, 67 },
    { 0, 0 },
    { 0x1730879f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4087a3, 36 },
    { 0x47d087a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35087a7, 77 },
    { 0, 0 },
    { 0xdbf087a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d087ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d087af, 46 },
    { 0, 0 },
    { 0xc53087b1, 46 },
    { 0xc3f087b2, 75 },
    { 0xf20087b3, 84 },
    { 0xb33087b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4087b7, 90 },
    { 0xff5087b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x219087bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad087c0, 99 },
    { 0, 0 },
    { 0x590087c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a087c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x912087cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7087ce, 95 },
    { 0x74d087cf, 61 },
    { 0xf07087d0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4087d6, 33 },
    { 0, 0 },
    { 0xc6c087d8, 23 },
    { 0xce5087d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5087dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec087e4, 61 },
    { 0xd46087e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82087ef, 33 },
    { 0, 0 },
    { 0xf8a087f1, 74 },
    { 0, 0 },
    { 0x5fe087f3, 23 },
    { 0x64f087f3, 100 },
    { 0, 0 },
    { 0xe3b087f6, 43 },
    { 0, 0 },
    { 0x904087f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb087fb, 98 },
    { 0x28c087fc, 33 },
    { 0xf54087fc, 90 },
    { 0x5e3087fe, 84 },
    { 0x1e6087ff, 55 },
    { 0xb0208800, 90 },
    { 0x1fd087fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4508809, 98 },
    { 0, 0 },
    { 0x9450880b, 95 },
    { 0x3fc0880c, 33 },
    { 0, 0 },
    { 0x8710880e, 95 },
    { 0xb680880f, 61 },
    { 0x3b80880f, 34 },
    { 0xd5208811, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c08816, 87 },
    { 0x7a508817, 55 },
    { 0xec08818, 61 },
    { 0x19408819, 66 },
    { 0xdb108819, 90 },
    { 0, 0 },
    { 0x6f70881c, 23 },
    { 0x41a0881d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee08820, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a508823, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3880882c, 33 },
    { 0x510882d, 33 },
    { 0, 0 },
    { 0x72d0882f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d508833, 98 },
    { 0xa1108834, 93 },
    { 0, 0 },
    { 0x4f708836, 76 },
    { 0x1c08837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd20883c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51808843, 99 },
    { 0xdae08844, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c08847, 90 },
    { 0, 0 },
    { 0xa3708849, 90 },
    { 0x5ff0884a, 46 },
    { 0, 0 },
    { 0xd840884c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c40884f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec908852, 88 },
    { 0x3f608853, 91 },
    { 0x50e08852, 33 },
    { 0x57c08855, 93 },
    { 0xa0f08856, 34 },
    { 0xff508856, 88 },
    { 0x5508858, 33 },
    { 0xd708855, 23 },
    { 0xb5408855, 34 },
    { 0x14508858, 95 },
    { 0x52c08858, 90 },
    { 0xa0408857, 77 },
    { 0, 0 },
    { 0x340885f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e08863, 98 },
    { 0xf5308864, 61 },
    { 0x42b08864, 34 },
    { 0x28a08865, 55 },
    { 0xa7508867, 85 },
    { 0x27208865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c40886b, 61 },
    { 0x88e0886c, 61 },
    { 0x4460886d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed08871, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8508876, 95 },
    { 0, 0 },
    { 0xa708878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f0887b, 34 },
    { 0x95b0887b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a508889, 91 },
    { 0, 0 },
    { 0x3500888b, 77 },
    { 0xb980888b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60888f, 90 },
    { 0xbfb08890, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33208895, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0889b, 61 },
    { 0x8b80889c, 84 },
    { 0x21e0889c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91088a1, 61 },
    { 0xbf0088a2, 78 },
    { 0x9ee088a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3088a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25088ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d088b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e088b5, 95 },
    { 0xe32088b6, 90 },
    { 0x8da088b5, 98 },
    { 0xb3c088b8, 23 },
    { 0xde9088b8, 33 },
    { 0xe38088ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5088bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x943088c0, 90 },
    { 0x3ff088c1, 33 },
    { 0x958088c1, 95 },
    { 0x679088c1, 55 },
    { 0, 0 },
    { 0xffc088c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c088c8, 78 },
    { 0x4c1088c9, 90 },
    { 0xf30088c9, 76 },
    { 0xbb2088cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x883088ce, 55 },
    { 0, 0 },
    { 0xf3b088d0, 33 },
    { 0, 0 },
    { 0x1ea088d2, 75 },
    { 0, 0 },
    { 0x420088d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e088d8, 90 },
    { 0, 0 },
    { 0x5db088da, 95 },
    { 0, 0 },
    { 0x4c3088dc, 33 },
    { 0x261088dd, 61 },
    { 0x84c088de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f088e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a088e7, 76 },
    { 0x139088e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17088eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x246088ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e088f1, 93 },
    { 0xd9d088f2, 61 },
    { 0, 0 },
    { 0xa0c088f4, 90 },
    { 0, 0 },
    { 0x464088f6, 85 },
    { 0, 0 },
    { 0x650088f8, 83 },
    { 0, 0 },
    { 0xe7088fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806088ff, 33 },
    { 0, 0 },
    { 0x76508901, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b08904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d608909, 99 },
    { 0xca20890a, 61 },
    { 0xfa40890b, 33 },
    { 0x6cd08909, 55 },
    { 0xaf10890d, 90 },
    { 0x20f0890d, 98 },
    { 0x8950890a, 99 },
    { 0xb9d08910, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a08914, 61 },
    { 0x85c08915, 85 },
    { 0xfcb08914, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d40891c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15808920, 95 },
    { 0x5ef08921, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf08924, 68 },
    { 0xb4c08925, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6108929, 33 },
    { 0xf790892a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6208931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45008936, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b708939, 78 },
    { 0, 0 },
    { 0xdaa0893b, 91 },
    { 0xa6f0893c, 90 },
    { 0, 0 },
    { 0xa380893e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e08949, 90 },
    { 0, 0 },
    { 0x4480894b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c0894e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf08951, 84 },
    { 0xe1908952, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43008959, 91 },
    { 0xa5f08959, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d0895e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d08962, 95 },
    { 0x81e08962, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6508966, 90 },
    { 0xdd808967, 95 },
    { 0x6a208968, 66 },
    { 0x6c508969, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1080896c, 66 },
    { 0xc120896c, 76 },
    { 0x3a40896c, 90 },
    { 0x1c90896f, 90 },
    { 0xbfa08970, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa08979, 61 },
    { 0, 0 },
    { 0xec30897b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd0897f, 98 },
    { 0xb8f0897f, 33 },
    { 0, 0 },
    { 0x76208982, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9708988, 90 },
    { 0x82c08988, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1830898e, 90 },
    { 0xdd80898f, 77 },
    { 0x68c08990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a808998, 55 },
    { 0xeb608999, 87 },
    { 0, 0 },
    { 0x9950899b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf0899e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651089a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a089a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4089ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e089b2, 61 },
    { 0xab3089b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5089bd, 96 },
    { 0, 0 },
    { 0x795089bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f089c2, 95 },
    { 0xcf8089c3, 90 },
    { 0, 0 },
    { 0x598089c5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43089cb, 84 },
    { 0xe3b089cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1089d0, 33 },
    { 0xb24089d1, 91 },
    { 0, 0 },
    { 0xf13089d3, 33 },
    { 0x6c089d3, 48 },
    { 0, 0 },
    { 0xb4a089d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6089da, 23 },
    { 0xb5f089da, 95 },
    { 0x6df089dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b089e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df089e4, 55 },
    { 0xe8c089e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8089eb, 90 },
    { 0xc2089ec, 65 },
    { 0xc20089ec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d089f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3089f5, 98 },
    { 0, 0 },
    { 0x75a089f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a089fa, 42 },
    { 0x594089fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd089ff, 90 },
    { 0xd5508a00, 84 },
    { 0x9f508a01, 78 },
    { 0xef708a01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb508a05, 23 },
    { 0x51608a05, 33 },
    { 0xf808a07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4408a0a, 98 },
    { 0x90408a0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb08a15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f108a18, 23 },
    { 0x1f908a19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e08a1d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc608a21, 98 },
    { 0, 0 },
    { 0xe4508a23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb008a26, 93 },
    { 0xd5208a27, 34 },
    { 0x4bf08a26, 33 },
    { 0x1e908a29, 33 },
    { 0xb2b08a28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb08a30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b208a36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b08a3b, 75 },
    { 0xc1808a3c, 22 },
    { 0x49c08a3d, 77 },
    { 0x98608a3d, 99 },
    { 0x99c08a3f, 90 },
    { 0xc708a40, 33 },
    { 0x44b08a3e, 98 },
    { 0x73f08a42, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc508a47, 61 },
    { 0x22b08a48, 66 },
    { 0, 0 },
    { 0x14e08a4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d08a4f, 34 },
    { 0, 0 },
    { 0xed808a51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59208a58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25708a5c, 90 },
    { 0x2bf08a5d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8708a64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2408a6a, 95 },
    { 0, 0 },
    { 0x1c608a6c, 90 },
    { 0x84308a6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb508a70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad708a74, 78 },
    { 0xafc08a75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b08a78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde08a7d, 67 },
    { 0, 0 },
    { 0x99108a7f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82908a89, 90 },
    { 0, 0 },
    { 0x91408a8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a408a8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96008a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d408a96, 55 },
    { 0xdc308a97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae708a9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e08aa8, 90 },
    { 0xb3c08aa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa08aae, 90 },
    { 0, 0 },
    { 0x60008ab0, 76 },
    { 0x56608ab1, 98 },
    { 0xe8008ab1, 98 },
    { 0x30208ab3, 61 },
    { 0x42708ab4, 23 },
    { 0xb0308ab3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f08ab9, 95 },
    { 0x67508aba, 85 },
    { 0x14208ab9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f208abe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b908ac2, 87 },
    { 0xdb008ac3, 23 },
    { 0x3f208ac2, 95 },
    { 0x45a08ac5, 77 },
    { 0x3bf08ac6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44108ac9, 34 },
    { 0, 0 },
    { 0x70708acb, 33 },
    { 0xdde08acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2108acf, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x23508ad2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6208ad5, 85 },
    { 0xcca08ad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27608ad9, 95 },
    { 0x83608ada, 87 },
    { 0xb7808adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e08ae0, 46 },
    { 0, 0 },
    { 0xe1908ae2, 33 },
    { 0x6d108ae3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f08ae7, 90 },
    { 0, 0 },
    { 0x8fd08ae9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f08aec, 95 },
    { 0x26008aed, 61 },
    { 0xda008aed, 42 },
    { 0xa0208aed, 90 },
    { 0, 0 },
    { 0x25e08af1, 61 },
    { 0xa3308af2, 95 },
    { 0, 0 },
    { 0x95908af4, 90 },
    { 0x6b408af4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6208afa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc08b01, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e08b04, 91 },
    { 0, 0 },
    { 0x64408b06, 78 },
    { 0x41a08b06, 95 },
    { 0x8b308b08, 88 },
    { 0xdac08b09, 23 },
    { 0x75b08b0a, 84 },
    { 0xdcf08b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd208b15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83508b19, 95 },
    { 0, 0 },
    { 0xb7d08b1b, 90 },
    { 0x42d08b1b, 93 },
    { 0x3f608b1b, 55 },
    { 0, 0 },
    { 0xcea08b1f, 34 },
    { 0x37108b1f, 55 },
    { 0, 0 },
    { 0x66a08b22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97308b27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34808b2f, 40 },
    { 0, 0 },
    { 0x50508b31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60008b37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b08b3c, 23 },
    { 0, 0 },
    { 0x70a08b3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d08b44, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c08b4c, 91 },
    { 0x28208b4d, 33 },
    { 0x15f08b4c, 23 },
    { 0x23508b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66108b56, 61 },
    { 0x9e108b56, 95 },
    { 0xb4e08b57, 90 },
    { 0xebc08b58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60808b5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10408b64, 99 },
    { 0, 0 },
    { 0x1e308b66, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf508b6a, 77 },
    { 0xb4308b6b, 33 },
    { 0x45408b6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x24408b6f, 66 },
    { 0, 0 },
    { 0x68008b71, 23 },
    { 0, 0 },
    { 0x16308b73, 78 },
    { 0x5e708b74, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c08b78, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e08b80, 46 },
    { 0xf7b08b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80208b89, 77 },
    { 0x71e08b89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8608b95, 95 },
    { 0xff108b95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa408b99, 76 },
    { 0x6a008b99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba008b9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1708ba5, 95 },
    { 0x81808ba5, 101 },
    { 0x4c308ba7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9408bb8, 88 },
    { 0, 0 },
    { 0xf6408bba, 23 },
    { 0x54308bba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d08bbe, 74 },
    { 0xd9708bbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98708bc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90308bc8, 98 },
    { 0x1dd08bc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30908bd1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f08bd6, 90 },
    { 0x6b908bd7, 33 },
    { 0x9908bd7, 23 },
    { 0x16208bd8, 34 },
    { 0x80e08bd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26808bde, 90 },
    { 0x2ab08bdf, 95 },
    { 0, 0 },
    { 0xd308be1, 95 },
    { 0, 0 },
    { 0xb5d08be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2308be6, 98 },
    { 0xa108be7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82408beb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab008bee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c08bf1, 33 },
    { 0, 0 },
    { 0xda508bf3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a08bf8, 33 },
    { 0, 0 },
    { 0x81e08bfa, 90 },
    { 0x6f408bfb, 33 },
    { 0x78208bfc, 90 },
    { 0x2ff08bfb, 33 },
    { 0x99508bfe, 55 },
    { 0, 0 },
    { 0x7b808c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b108c08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d08c0c, 23 },
    { 0xc8908c0d, 55 },
    { 0xa5808c0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb08c17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c608c1c, 95 },
    { 0, 0 },
    { 0x9c608c1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe08c22, 33 },
    { 0xbc08c22, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b08c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10008c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d08c32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc08c37, 61 },
    { 0x69e08c37, 76 },
    { 0, 0 },
    { 0xd4808c3a, 43 },
    { 0, 0 },
    { 0xcbf08c3c, 23 },
    { 0x78508c3d, 33 },
    { 0xc8908c3e, 99 },
    { 0x32f08c3f, 90 },
    { 0x81b08c40, 61 },
    { 0xd2d08c41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55808c47, 33 },
    { 0x8a308c48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c108c4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99308c51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d08c58, 77 },
    { 0x44208c59, 95 },
    { 0x30d08c5a, 98 },
    { 0x96108c58, 77 },
    { 0x31708c5c, 40 },
    { 0x35c08c5a, 90 },
    { 0, 0 },
    { 0xa5708c5f, 66 },
    { 0x66608c5f, 61 },
    { 0xdf308c61, 23 },
    { 0xaeb08c5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd08c65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b08c69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5308c6e, 34 },
    { 0xa8508c6e, 61 },
    { 0, 0 },
    { 0x5d508c71, 61 },
    { 0x68708c71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8408c75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d208c7a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5108c7d, 95 },
    { 0xf3008c7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a08c88, 33 },
    { 0xea708c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa808c8c, 95 },
    { 0, 0 },
    { 0xe0908c8e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e08c92, 55 },
    { 0xd3d08c92, 93 },
    { 0x2e808c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1408c97, 33 },
    { 0, 0 },
    { 0xf2408c99, 95 },
    { 0xa6108c99, 90 },
    { 0x8f208c9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b08c9e, 23 },
    { 0, 0 },
    { 0x4208ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23008ca4, 100 },
    { 0xbc708ca4, 55 },
    { 0xfa008ca6, 52 },
    { 0xe9b08ca7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92308cae, 95 },
    { 0, 0 },
    { 0x2b408cb0, 23 },
    { 0xb8208cb1, 76 },
    { 0xa8808cb1, 99 },
    { 0x4e708cb1, 88 },
    { 0xb3508cb2, 87 },
    { 0, 0 },
    { 0x4c308cb6, 46 },
    { 0x14208cb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de08cbd, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb08cc0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9008cc3, 95 },
    { 0xc2608cc4, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd08cc7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b08ccf, 78 },
    { 0x53d08ccf, 90 },
    { 0x4f108ccf, 61 },
    { 0xd7d08ccf, 100 },
    { 0x42808cd3, 95 },
    { 0xd4c08cd4, 68 },
    { 0x27208cd4, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef08cd9, 23 },
    { 0xd0b08cda, 69 },
    { 0xcdf08cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e08ce1, 98 },
    { 0x43f08ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df08ce7, 33 },
    { 0xe1808ce7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7008cee, 40 },
    { 0, 0 },
    { 0x1e008cf0, 99 },
    { 0x7d208cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18508cf4, 88 },
    { 0, 0 },
    { 0xb7e08cf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64708cf9, 33 },
    { 0xaa08cfa, 33 },
    { 0xf1f08cfa, 55 },
    { 0x1b08cfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b08d02, 46 },
    { 0, 0 },
    { 0x98308d04, 95 },
    { 0x56508d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d08d0c, 33 },
    { 0x55308d0d, 90 },
    { 0xd5e08d0e, 95 },
    { 0xa0208d0e, 33 },
    { 0x40808d10, 34 },
    { 0xbc108d0f, 90 },
    { 0, 0 },
    { 0xb0908d13, 17 },
    { 0x15a08d14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb508d1a, 23 },
    { 0xfd008d1b, 78 },
    { 0x50408d1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f08d20, 61 },
    { 0, 0 },
    { 0xaa408d22, 61 },
    { 0xb3108d23, 75 },
    { 0, 0 },
    { 0x42508d25, 98 },
    { 0x20508d26, 33 },
    { 0xa9b08d27, 61 },
    { 0, 0 },
    { 0x72e08d29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d08d2c, 98 },
    { 0x35608d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92408d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8708d34, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde08d38, 98 },
    { 0, 0 },
    { 0x7e008d3a, 95 },
    { 0x13d08d3b, 23 },
    { 0x90808d3a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x69108d3f, 65 },
    { 0x3e708d3f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa08d45, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf208d49, 61 },
    { 0xdd708d4a, 67 },
    { 0x90308d4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c608d4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf08d56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c08d59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70908d5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0608d65, 95 },
    { 0, 0 },
    { 0x95808d67, 24 },
    { 0x4b108d68, 43 },
    { 0x72708d67, 90 },
    { 0xdb708d6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e808d6e, 98 },
    { 0xc2308d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a08d72, 84 },
    { 0x1ac08d73, 76 },
    { 0xd7b08d72, 98 },
    { 0xb6208d75, 61 },
    { 0xa1308d76, 52 },
    { 0x1ed08d74, 61 },
    { 0xbf808d78, 98 },
    { 0x19c08d76, 61 },
    { 0, 0 },
    { 0x4408d7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd108d7e, 55 },
    { 0xce108d7e, 33 },
    { 0x2fe08d80, 99 },
    { 0xe3708d81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba508d84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc208d88, 61 },
    { 0xce708d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42208d8c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed208d90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f08d9f, 33 },
    { 0, 0 },
    { 0x92f08da1, 95 },
    { 0, 0 },
    { 0xc3008da3, 33 },
    { 0, 0 },
    { 0x7b208da5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb08dad, 61 },
    { 0xc9108dae, 33 },
    { 0x5a708daf, 90 },
    { 0x14608db0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b08db6, 46 },
    { 0, 0 },
    { 0x22108db8, 69 },
    { 0, 0 },
    { 0x8108dba, 90 },
    { 0x35c08dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf308dbe, 52 },
    { 0x108dbe, 90 },
    { 0, 0 },
    { 0x57f08dc1, 95 },
    { 0, 0 },
    { 0x7df08dc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc08dc7, 22 },
    { 0x8e108dc8, 91 },
    { 0x20408dc9, 90 },
    { 0xd6708dc7, 76 },
    { 0x69f08dcb, 98 },
    { 0xf6908dcc, 90 },
    { 0xfa008dcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2608dd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e08dd9, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d08ddc, 61 },
    { 0xb3b08ddd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e08de0, 34 },
    { 0xc2d08de1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f08de5, 69 },
    { 0x8fb08de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15208deb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b08df3, 61 },
    { 0, 0 },
    { 0x14608df5, 90 },
    { 0x14208df6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf08df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78108dfc, 23 },
    { 0, 0 },
    { 0xdb808dfe, 33 },
    { 0x7ac08dff, 28 },
    { 0xc1d08dfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73808e09, 95 },
    { 0x9a108e0a, 77 },
    { 0xbf408e0b, 98 },
    { 0x7a908e0c, 98 },
    { 0xfed08e0d, 56 },
    { 0x89f08e0c, 33 },
    { 0, 0 },
    { 0xe3608e10, 61 },
    { 0x55508e11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb08e15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0408e18, 33 },
    { 0, 0 },
    { 0x49e08e1a, 66 },
    { 0, 0 },
    { 0x44208e1c, 33 },
    { 0xfde08e1d, 61 },
    { 0xf0808e1e, 68 },
    { 0x9d208e1d, 61 },
    { 0xf2d08e20, 95 },
    { 0xc9f08e20, 93 },
    { 0x3c408e1d, 31 },
    { 0xf5e08e23, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd08e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c08e2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e108e31, 98 },
    { 0, 0 },
    { 0x92608e33, 88 },
    { 0xc4308e34, 43 },
    { 0, 0 },
    { 0x11208e36, 23 },
    { 0xbdf08e37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x18208e3a, 55 },
    { 0xeb708e3a, 33 },
    { 0x32408e3c, 93 },
    { 0xb3708e3d, 66 },
    { 0xd408e3e, 98 },
    { 0xd3a08e3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7008e43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85108e47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2808e4b, 33 },
    { 0, 0 },
    { 0xe2808e4d, 33 },
    { 0x6c008e4e, 98 },
    { 0, 0 },
    { 0x29a08e50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff08e58, 66 },
    { 0xe708e59, 55 },
    { 0xba708e58, 95 },
    { 0, 0 },
    { 0x3c708e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd08e61, 46 },
    { 0x8a008e61, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c08e65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf08e6b, 65 },
    { 0x48108e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9908e71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6308e78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66208e7b, 76 },
    { 0xc9408e7c, 52 },
    { 0xf9408e7c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b608e82, 88 },
    { 0x95908e82, 61 },
    { 0xa7b08e84, 55 },
    { 0x6ae08e85, 33 },
    { 0x61908e86, 76 },
    { 0, 0 },
    { 0x32d08e88, 90 },
    { 0, 0 },
    { 0x8bd08e8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2808e8d, 43 },
    { 0xea508e8e, 95 },
    { 0xf6f08e8e, 91 },
    { 0, 0 },
    { 0x3d108e91, 93 },
    { 0x53108e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a08e96, 95 },
    { 0xcba08e97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98908ea5, 90 },
    { 0xcc608ea6, 65 },
    { 0x38e08ea7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e08ead, 98 },
    { 0xc1408eae, 43 },
    { 0x2dd08eaf, 91 },
    { 0, 0 },
    { 0x3b08eb1, 89 },
    { 0x54e08eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee808eb6, 95 },
    { 0, 0 },
    { 0x3ea08eb8, 76 },
    { 0xd7308eb9, 76 },
    { 0xdd508eb8, 61 },
    { 0, 0 },
    { 0x2ba08ebc, 93 },
    { 0x38108ebd, 88 },
    { 0xbf608ebd, 67 },
    { 0, 0 },
    { 0xeb108ec0, 66 },
    { 0x2c608ec1, 93 },
    { 0, 0 },
    { 0x82b08ec3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9308ec6, 91 },
    { 0, 0 },
    { 0x40808ec8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a508ecb, 99 },
    { 0x32008ecc, 90 },
    { 0xdd208ecd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29808ed0, 84 },
    { 0, 0 },
    { 0x68108ed2, 98 },
    { 0, 0 },
    { 0xbaf08ed4, 95 },
    { 0x7db08ed4, 61 },
    { 0, 0 },
    { 0xb1208ed7, 23 },
    { 0, 0 },
    { 0x52f08ed9, 55 },
    { 0x56308ed9, 98 },
    { 0xadf08ed9, 93 },
    { 0, 0 },
    { 0xb4c08edd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2408ee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b08ee3, 33 },
    { 0, 0 },
    { 0x8608ee5, 95 },
    { 0, 0 },
    { 0x27b08ee7, 34 },
    { 0x2e608ee8, 78 },
    { 0xa7008ee7, 34 },
    { 0xe3c08eea, 55 },
    { 0x28908eeb, 61 },
    { 0x12908eec, 61 },
    { 0xf2e08eec, 43 },
    { 0xd8808eec, 98 },
    { 0x9f708eef, 52 },
    { 0x12008ef0, 66 },
    { 0x47408ef0, 43 },
    { 0x93508ef1, 34 },
    { 0xa9d08ef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d708ef6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f908efa, 66 },
    { 0x7808efb, 85 },
    { 0xfe908efa, 78 },
    { 0xbc908efc, 76 },
    { 0x5da08efb, 95 },
    { 0xf1208eff, 61 },
    { 0, 0 },
    { 0xca08f01, 61 },
    { 0, 0 },
    { 0xb3808f03, 90 },
    { 0x62808f03, 66 },
    { 0x14108f05, 78 },
    { 0xea108f05, 43 },
    { 0, 0 },
    { 0x31f08f08, 97 },
    { 0x1e908f09, 93 },
    { 0x97208f0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2108f0f, 98 },
    { 0, 0 },
    { 0x46108f11, 98 },
    { 0xca008f12, 76 },
    { 0x12e08f13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc608f18, 23 },
    { 0x8e408f18, 84 },
    { 0x52308f19, 33 },
    { 0x7f408f1b, 94 },
    { 0, 0 },
    { 0xb7a08f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb808f22, 93 },
    { 0x71508f23, 95 },
    { 0x48008f23, 67 },
    { 0x40108f25, 90 },
    { 0x12608f26, 23 },
    { 0x44608f27, 33 },
    { 0xd9808f22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b08f2b, 17 },
    { 0x9bd08f2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad08f30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b08f33, 34 },
    { 0xec608f33, 61 },
    { 0, 0 },
    { 0xe3308f36, 55 },
    { 0x6b808f36, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x57608f3a, 64 },
    { 0x8ff08f3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd08f44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x97208f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f608f4a, 43 },
    { 0xc6808f4b, 76 },
    { 0x54908f4b, 98 },
    { 0, 0 },
    { 0xdaa08f4e, 95 },
    { 0, 0 },
    { 0xdee08f50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42708f54, 33 },
    { 0xd4a08f55, 55 },
    { 0xc7708f56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de08f59, 95 },
    { 0xfab08f5a, 90 },
    { 0xb4608f5a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb08f63, 99 },
    { 0, 0 },
    { 0xdbb08f65, 84 },
    { 0xc4208f65, 98 },
    { 0, 0 },
    { 0x35d08f68, 67 },
    { 0xc0308f68, 33 },
    { 0, 0 },
    { 0x46f08f6b, 61 },
    { 0, 0 },
    { 0xd3408f6d, 98 },
    { 0x36508f6d, 76 },
    { 0xb8f08f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf08f73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f08f81, 95 },
    { 0x61408f82, 8 },
    { 0x64d08f81, 98 },
    { 0x88908f84, 76 },
    { 0xfde08f85, 95 },
    { 0x42308f85, 33 },
    { 0xde008f87, 46 },
    { 0x88708f85, 33 },
    { 0x97208f89, 95 },
    { 0xf9c08f88, 76 },
    { 0x12308f87, 87 },
    { 0xacf08f84, 61 },
    { 0xc9b08f8a, 61 },
    { 0xe1d08f8e, 65 },
    { 0x81508f8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d008f93, 98 },
    { 0x43008f94, 17 },
    { 0xccd08f95, 98 },
    { 0xcf408f95, 98 },
    { 0, 0 },
    { 0x1bd08f98, 34 },
    { 0xf8108f98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f08f9c, 34 },
    { 0xc8108f9d, 98 },
    { 0x28008f9e, 23 },
    { 0x83908f9f, 61 },
    { 0x12008f9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a08fa6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62708fb6, 98 },
    { 0, 0 },
    { 0xd8b08fb8, 33 },
    { 0x13808fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef208fbd, 78 },
    { 0x24408fbd, 85 },
    { 0xc3b08fbe, 95 },
    { 0, 0 },
    { 0x82a08fc1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a08fc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd408fc9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e08fcc, 33 },
    { 0xa5b08fcd, 33 },
    { 0x6a408fce, 90 },
    { 0x8fe08fcf, 61 },
    { 0x79308fd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93908fd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b008fd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24808fe0, 100 },
    { 0x35f08fe0, 31 },
    { 0xc7108fe2, 55 },
    { 0x6de08fe1, 75 },
    { 0xaa308fe4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d08fe8, 66 },
    { 0xaf208fe9, 95 },
    { 0, 0 },
    { 0x63c08feb, 76 },
    { 0xcc608fec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d408ff3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d08ffc, 95 },
    { 0x25d08ffc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a09000, 70 },
    { 0, 0 },
    { 0xe1209002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc930900a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc109012, 33 },
    { 0x37209012, 98 },
    { 0, 0 },
    { 0xe1509015, 90 },
    { 0x2b009016, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26409021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be09027, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf370902b, 66 },
    { 0x7260902c, 98 },
    { 0x5de0902b, 80 },
    { 0x65b0902e, 46 },
    { 0x95b0902e, 78 },
    { 0x89e09030, 95 },
    { 0x3e20902e, 36 },
    { 0x5f09032, 90 },
    { 0, 0 },
    { 0x6d09034, 95 },
    { 0xfee09034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed509039, 61 },
    { 0, 0 },
    { 0x43d0903b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee009040, 61 },
    { 0x4ac09041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34609045, 88 },
    { 0xfe609045, 66 },
    { 0, 0 },
    { 0x7fc09048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4140904c, 33 },
    { 0x4550904d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a09054, 76 },
    { 0x88609055, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d0905f, 66 },
    { 0, 0 },
    { 0x97909061, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7409064, 100 },
    { 0x45b09065, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc830906a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a0906f, 95 },
    { 0x38409070, 98 },
    { 0x25509071, 23 },
    { 0xcdb09072, 98 },
    { 0, 0 },
    { 0x3b309074, 75 },
    { 0, 0 },
    { 0x23309076, 52 },
    { 0, 0 },
    { 0xd5b09078, 90 },
    { 0, 0 },
    { 0x1fb0907a, 90 },
    { 0, 0 },
    { 0xfac0907c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c0907f, 68 },
    { 0x1b0907f, 90 },
    { 0x97209081, 90 },
    { 0xd5509082, 90 },
    { 0xbf209083, 52 },
    { 0x38609083, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f09088, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0908c, 55 },
    { 0xfdf0908d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48409095, 90 },
    { 0xd5509096, 90 },
    { 0xacb09095, 95 },
    { 0x38b09095, 90 },
    { 0xca309099, 23 },
    { 0x5e80909a, 55 },
    { 0x9850909b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54090a0, 90 },
    { 0xb30090a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157090a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644090ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8090ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e090b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad090b9, 91 },
    { 0x1d3090ba, 36 },
    { 0x337090b9, 76 },
    { 0xae3090b9, 76 },
    { 0, 0 },
    { 0x15090be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d090c2, 61 },
    { 0, 0 },
    { 0xbe3090c4, 95 },
    { 0xbc5090c4, 75 },
    { 0xe53090c6, 90 },
    { 0, 0 },
    { 0x645090c8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57090cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1090d0, 76 },
    { 0, 0 },
    { 0xfe0090d2, 52 },
    { 0xf16090d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c090e0, 100 },
    { 0, 0 },
    { 0xfc6090e2, 43 },
    { 0x2b5090e3, 33 },
    { 0xeb1090e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b090e7, 33 },
    { 0, 0 },
    { 0x52090e9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c090f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4090f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde090fa, 61 },
    { 0x4d2090fa, 61 },
    { 0, 0 },
    { 0xa42090fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be09100, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x28609103, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc709109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b10910c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95709110, 34 },
    { 0x50509111, 66 },
    { 0, 0 },
    { 0x6da09113, 95 },
    { 0, 0 },
    { 0xf1a09115, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e09119, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44709120, 55 },
    { 0x2aa09121, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x75209124, 90 },
    { 0, 0 },
    { 0x80209126, 69 },
    { 0x61409127, 23 },
    { 0, 0 },
    { 0x32309129, 67 },
    { 0, 0 },
    { 0xd2c0912b, 90 },
    { 0x1610912c, 95 },
    { 0x1350912d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf09131, 53 },
    { 0x1ad09131, 76 },
    { 0x10b09131, 23 },
    { 0x74c09132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa900913a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f0913d, 33 },
    { 0xc510913e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba309145, 65 },
    { 0x41309146, 17 },
    { 0xe5f09147, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d0914a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1370914d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f309150, 55 },
    { 0xa8709151, 23 },
    { 0, 0 },
    { 0xedd09153, 33 },
    { 0, 0 },
    { 0xef109155, 87 },
    { 0x12c09155, 33 },
    { 0x70e09156, 76 },
    { 0xc2d09158, 68 },
    { 0xe9809159, 90 },
    { 0, 0 },
    { 0xd070915b, 76 },
    { 0, 0 },
    { 0x56e0915d, 68 },
    { 0x6a60915e, 98 },
    { 0x2f50915d, 77 },
    { 0xef709160, 34 },
    { 0xd290915f, 93 },
    { 0xdad09162, 90 },
    { 0x45c09162, 65 },
    { 0x8bf09164, 78 },
    { 0xcac09161, 34 },
    { 0x60709166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd09170, 90 },
    { 0x91d09171, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf609177, 46 },
    { 0, 0 },
    { 0x2a909179, 55 },
    { 0, 0 },
    { 0x860917b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f09183, 46 },
    { 0x30c09184, 90 },
    { 0x6e309184, 96 },
    { 0xedb09186, 65 },
    { 0x99909185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f0918b, 61 },
    { 0, 0 },
    { 0xf290918d, 40 },
    { 0xd0d0918e, 33 },
    { 0, 0 },
    { 0x23909190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63809193, 93 },
    { 0, 0 },
    { 0xb1a09195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7820919d, 76 },
    { 0x630919e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1091a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b091aa, 69 },
    { 0xc14091ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2091af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d091b3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b091b6, 76 },
    { 0x3d4091b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5091ba, 66 },
    { 0xfce091ba, 46 },
    { 0xe6091bc, 61 },
    { 0xe84091bd, 95 },
    { 0, 0 },
    { 0x63a091bf, 33 },
    { 0x435091c0, 61 },
    { 0xc30091c1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4091c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce091c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879091cf, 78 },
    { 0x24b091d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b091d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311091db, 55 },
    { 0xc8091dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520091e1, 76 },
    { 0x7ca091e2, 33 },
    { 0xb57091e2, 25 },
    { 0xccc091e3, 34 },
    { 0x11d091e5, 55 },
    { 0, 0 },
    { 0xd96091e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387091ec, 46 },
    { 0, 0 },
    { 0x76a091ee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a091f1, 98 },
    { 0x49c091f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b091f5, 34 },
    { 0xb53091f6, 84 },
    { 0, 0 },
    { 0x5cc091f8, 98 },
    { 0xd28091f9, 61 },
    { 0x13a091fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e09200, 98 },
    { 0x90d09200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9209207, 33 },
    { 0, 0 },
    { 0x27909209, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9309211, 49 },
    { 0xe7609212, 90 },
    { 0x7be09212, 90 },
    { 0, 0 },
    { 0x77f09215, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc0921b, 75 },
    { 0, 0 },
    { 0x850921d, 95 },
    { 0xe280921e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x44409221, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae909225, 98 },
    { 0xf8c09226, 43 },
    { 0xba509226, 85 },
    { 0x1f309228, 46 },
    { 0xf8909228, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb609233, 90 },
    { 0xb0409234, 23 },
    { 0x5a609235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d709239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d90923d, 81 },
    { 0x2310923d, 84 },
    { 0x3c90923e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59609242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42309249, 87 },
    { 0xe340924a, 61 },
    { 0x8700924b, 33 },
    { 0x9130924b, 90 },
    { 0x25b09249, 33 },
    { 0x7bb0924e, 33 },
    { 0xd700924c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb509252, 90 },
    { 0x63f09253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd309256, 43 },
    { 0x95309257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c0925b, 23 },
    { 0, 0 },
    { 0x2eb0925d, 98 },
    { 0xc0b0925e, 76 },
    { 0, 0 },
    { 0xfab09260, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a09268, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa780926c, 23 },
    { 0, 0 },
    { 0x3920926e, 90 },
    { 0, 0 },
    { 0x27b09270, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b09274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea0927b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f0927e, 40 },
    { 0, 0 },
    { 0xddb09280, 33 },
    { 0, 0 },
    { 0x10609282, 95 },
    { 0, 0 },
    { 0x1309284, 43 },
    { 0xab509285, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d0928b, 61 },
    { 0x2b50928c, 90 },
    { 0x8230928d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a09293, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c09296, 23 },
    { 0x8e409297, 61 },
    { 0x32209297, 33 },
    { 0x1d209297, 90 },
    { 0x56f09297, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb0929d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20092a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6092a7, 43 },
    { 0, 0 },
    { 0x432092a9, 91 },
    { 0xa01092aa, 99 },
    { 0x8ad092aa, 90 },
    { 0, 0 },
    { 0xeeb092ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f092b1, 33 },
    { 0, 0 },
    { 0x35b092b3, 23 },
    { 0xd40092b4, 61 },
    { 0x112092b3, 90 },
    { 0x16c092b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x978092b9, 43 },
    { 0, 0 },
    { 0x8c4092bb, 95 },
    { 0, 0 },
    { 0xa08092bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c092c0, 23 },
    { 0xb76092c1, 46 },
    { 0x4d7092c2, 43 },
    { 0x651092c3, 46 },
    { 0x555092c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68092c7, 93 },
    { 0, 0 },
    { 0x2f8092c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x302092cc, 66 },
    { 0x7c6092cd, 78 },
    { 0x615092cd, 98 },
    { 0xf95092cf, 36 },
    { 0x6b4092d0, 95 },
    { 0xf8e092d1, 90 },
    { 0, 0 },
    { 0x499092d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7092d7, 45 },
    { 0, 0 },
    { 0xdf6092d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934092e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4092e4, 90 },
    { 0, 0 },
    { 0x4f1092e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x113092e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c092ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d092ef, 95 },
    { 0, 0 },
    { 0x260092f1, 23 },
    { 0x4ae092f2, 61 },
    { 0, 0 },
    { 0x156092f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7092f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50092fc, 90 },
    { 0x3f1092fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72909300, 95 },
    { 0xea109301, 61 },
    { 0xa3709302, 61 },
    { 0x6ba09302, 78 },
    { 0xddf09303, 61 },
    { 0x54c09305, 91 },
    { 0x2d109302, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb60930b, 90 },
    { 0xa6f0930c, 90 },
    { 0x1970930d, 98 },
    { 0x4cc0930c, 95 },
    { 0xeeb0930b, 55 },
    { 0x3ed09310, 80 },
    { 0xfb509310, 32 },
    { 0xc20930d, 55 },
    { 0x9120930f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf030931a, 43 },
    { 0, 0 },
    { 0xa530931c, 33 },
    { 0xdac0931c, 77 },
    { 0, 0 },
    { 0x60a0931f, 33 },
    { 0, 0 },
    { 0x42a09321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47809325, 90 },
    { 0, 0 },
    { 0x45e09327, 95 },
    { 0xc9e09328, 77 },
    { 0, 0 },
    { 0x30a0932a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc480932d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2809330, 61 },
    { 0xec709331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0809336, 90 },
    { 0xead09336, 98 },
    { 0x86209338, 33 },
    { 0xf9b09336, 90 },
    { 0, 0 },
    { 0x3980933b, 23 },
    { 0, 0 },
    { 0xb300933d, 90 },
    { 0x18c0933d, 90 },
    { 0xafd0933f, 67 },
    { 0xbdf09340, 43 },
    { 0, 0 },
    { 0x88309342, 33 },
    { 0x41309342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf709347, 90 },
    { 0, 0 },
    { 0x1a509349, 95 },
    { 0, 0 },
    { 0x8d80934b, 98 },
    { 0x2570934c, 66 },
    { 0xdbc0934d, 83 },
    { 0xbdb0934e, 23 },
    { 0x45e0934f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99309355, 34 },
    { 0x33009356, 52 },
    { 0xb5109356, 95 },
    { 0, 0 },
    { 0x74109359, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb09361, 33 },
    { 0x95909362, 90 },
    { 0x49e09363, 90 },
    { 0x64509363, 23 },
    { 0x1ef09365, 76 },
    { 0xc6909365, 36 },
    { 0x20309364, 33 },
    { 0xfb409367, 23 },
    { 0xb2709369, 23 },
    { 0x5b60936a, 53 },
    { 0x6ff0936a, 98 },
    { 0xe5509361, 76 },
    { 0x2300936d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e09370, 90 },
    { 0, 0 },
    { 0x9409372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a209378, 76 },
    { 0xe2209378, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e0937c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd140937f, 76 },
    { 0x6aa09380, 61 },
    { 0x54809381, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55009386, 61 },
    { 0xab09387, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab809399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e30939e, 97 },
    { 0, 0 },
    { 0xf1093a0, 76 },
    { 0x808093a1, 17 },
    { 0x7a4093a2, 55 },
    { 0, 0 },
    { 0xec5093a4, 23 },
    { 0xe26093a4, 65 },
    { 0, 0 },
    { 0x585093a7, 23 },
    { 0, 0 },
    { 0x497093a9, 34 },
    { 0x159093aa, 76 },
    { 0, 0 },
    { 0xc8a093ac, 98 },
    { 0x79f093ad, 90 },
    { 0x29e093ae, 76 },
    { 0, 0 },
    { 0xb3a093b0, 61 },
    { 0x5ce093b0, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8093b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3093c2, 55 },
    { 0, 0 },
    { 0xb98093c4, 43 },
    { 0, 0 },
    { 0x251093c6, 23 },
    { 0xeb2093c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2093cb, 90 },
    { 0xed4093cc, 68 },
    { 0x312093cd, 34 },
    { 0x40e093cd, 61 },
    { 0, 0 },
    { 0xa43093d0, 33 },
    { 0x793093d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b093d6, 90 },
    { 0xfc6093d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550093dc, 17 },
    { 0, 0 },
    { 0xcb4093de, 87 },
    { 0x54d093de, 34 },
    { 0xa57093df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b093e5, 75 },
    { 0, 0 },
    { 0xfda093e7, 23 },
    { 0x46f093e8, 23 },
    { 0x316093e9, 87 },
    { 0x2fd093ea, 87 },
    { 0xde7093e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b093f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768093f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb093fa, 95 },
    { 0, 0 },
    { 0x7df093fc, 61 },
    { 0x2e8093fc, 90 },
    { 0, 0 },
    { 0xd0c093ff, 95 },
    { 0x10209400, 95 },
    { 0, 0 },
    { 0x3d09402, 95 },
    { 0x38d09403, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c309408, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4380940d, 95 },
    { 0xfa60940e, 78 },
    { 0xbbe0940f, 90 },
    { 0x2910940e, 33 },
    { 0, 0 },
    { 0xe6909412, 34 },
    { 0, 0 },
    { 0xf8009414, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74809417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a60941d, 33 },
    { 0x1d60941d, 90 },
    { 0x3d10941f, 34 },
    { 0x9d20941e, 33 },
    { 0xeb809421, 90 },
    { 0x7470941e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f09426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a0942a, 61 },
    { 0, 0 },
    { 0x7060942c, 84 },
    { 0x9870942c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b09430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a09434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d0943b, 76 },
    { 0xe4f0943c, 52 },
    { 0x930943c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73409440, 76 },
    { 0xa3f09441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d509444, 55 },
    { 0x52a09444, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e0944a, 33 },
    { 0x23b0944a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d0944e, 34 },
    { 0, 0 },
    { 0x46809450, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65409454, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f009458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c20945b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd50945f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5609465, 95 },
    { 0, 0 },
    { 0x99709467, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e50946a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4260946d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43709476, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd0947c, 43 },
    { 0x49a0947d, 98 },
    { 0xdf70947e, 90 },
    { 0x6af0947f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8009483, 91 },
    { 0x82609483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98109488, 33 },
    { 0x9b809489, 61 },
    { 0xf7a0948a, 55 },
    { 0, 0 },
    { 0xe860948c, 95 },
    { 0x8ac0948d, 61 },
    { 0xc8d0948e, 95 },
    { 0xd8d0948c, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0109494, 65 },
    { 0xf3709494, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4709499, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a094a4, 98 },
    { 0x76e094a4, 43 },
    { 0x861094a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156094be, 8 },
    { 0x795094be, 34 },
    { 0x33b094bf, 75 },
    { 0x1f3094bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x496094c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce094c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6094cb, 98 },
    { 0x5df094cc, 77 },
    { 0x357094cc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a094d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c094d9, 23 },
    { 0xe5b094da, 33 },
    { 0x5fe094db, 88 },
    { 0xe9f094db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442094e5, 43 },
    { 0x64c094e5, 43 },
    { 0, 0 },
    { 0xf6f094e8, 52 },
    { 0x10094e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06094ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0094f1, 98 },
    { 0x99c094f2, 78 },
    { 0xda5094f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e094f8, 90 },
    { 0xa68094f8, 98 },
    { 0, 0 },
    { 0x853094fb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735094ff, 55 },
    { 0, 0 },
    { 0x4af09501, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad609504, 93 },
    { 0x60409505, 33 },
    { 0xf9609506, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e0950b, 61 },
    { 0xc050950c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b0950f, 54 },
    { 0x12109510, 33 },
    { 0xe3b0950f, 29 },
    { 0, 0 },
    { 0xe0109513, 33 },
    { 0xb5009514, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb40951c, 23 },
    { 0, 0 },
    { 0x9b90951e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e09527, 93 },
    { 0, 0 },
    { 0x5c209529, 90 },
    { 0, 0 },
    { 0x3390952b, 99 },
    { 0, 0 },
    { 0xedb0952d, 95 },
    { 0x3b10952e, 23 },
    { 0xd20952f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3309533, 95 },
    { 0x57609533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc09537, 93 },
    { 0x3309538, 34 },
    { 0xb9609537, 76 },
    { 0x7330953a, 87 },
    { 0x46209538, 78 },
    { 0xe680953c, 99 },
    { 0, 0 },
    { 0xd2f0953e, 95 },
    { 0x4930953e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d09547, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680954a, 33 },
    { 0x9320954b, 33 },
    { 0x2390954c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e0954f, 98 },
    { 0x24509550, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe440955a, 67 },
    { 0xe3f0955a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b209561, 46 },
    { 0x8f209561, 90 },
    { 0, 0 },
    { 0xf1109564, 76 },
    { 0x3e009565, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa140956d, 52 },
    { 0x24f0956d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc709571, 84 },
    { 0x49c09572, 98 },
    { 0, 0 },
    { 0x49909574, 33 },
    { 0x3f909574, 65 },
    { 0xc4309576, 99 },
    { 0, 0 },
    { 0xd609578, 90 },
    { 0x6c809579, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7850957c, 33 },
    { 0xc4d0957d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70909581, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a09584, 55 },
    { 0, 0 },
    { 0x16009586, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7109591, 98 },
    { 0, 0 },
    { 0xf009593, 33 },
    { 0xe0709594, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf709599, 91 },
    { 0x5b60959a, 33 },
    { 0x6050959b, 76 },
    { 0x5080959b, 87 },
    { 0x5ec0959a, 61 },
    { 0x8eb0959e, 95 },
    { 0xd70959d, 90 },
    { 0x7ea0959b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c095a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd095a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc095ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c095b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2095b4, 33 },
    { 0x149095b4, 46 },
    { 0x3f9095b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8095ba, 93 },
    { 0, 0 },
    { 0x8c8095bc, 23 },
    { 0x8c8095bd, 34 },
    { 0xf54095be, 76 },
    { 0xd4c095bf, 88 },
    { 0, 0 },
    { 0xdd5095c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b095c7, 68 },
    { 0x21c095c7, 33 },
    { 0x2fb095c9, 66 },
    { 0x4bf095ca, 95 },
    { 0x3b9095cb, 95 },
    { 0xfc9095cb, 95 },
    { 0, 0 },
    { 0x824095ce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b095d1, 65 },
    { 0, 0 },
    { 0xff7095d3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0095d6, 55 },
    { 0, 0 },
    { 0xda1095d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794095dc, 92 },
    { 0x2b9095dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12095e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5095e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8095ea, 76 },
    { 0, 0 },
    { 0x737095ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897095f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b095f4, 90 },
    { 0, 0 },
    { 0x441095f6, 100 },
    { 0, 0 },
    { 0x94095f8, 34 },
    { 0x252095f9, 90 },
    { 0, 0 },
    { 0xb5f095fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd095ff, 36 },
    { 0, 0 },
    { 0x99309601, 34 },
    { 0x94909602, 46 },
    { 0xbf309603, 76 },
    { 0xcf09603, 59 },
    { 0xa1709601, 95 },
    { 0x56209606, 23 },
    { 0x92b09607, 61 },
    { 0, 0 },
    { 0xf6d09609, 68 },
    { 0xaf80960a, 98 },
    { 0xbde0960b, 95 },
    { 0, 0 },
    { 0xaff0960d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf609611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1309616, 99 },
    { 0, 0 },
    { 0xc5109618, 81 },
    { 0x19409618, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4430961c, 45 },
    { 0xc4e0961d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c609621, 23 },
    { 0xd9609621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c509626, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d20962f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b0963e, 23 },
    { 0, 0 },
    { 0x70f09640, 61 },
    { 0, 0 },
    { 0x1bb09642, 51 },
    { 0x51c09643, 95 },
    { 0xa2c09644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7509648, 33 },
    { 0xf5509649, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x830964c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480964f, 23 },
    { 0, 0 },
    { 0x45609651, 93 },
    { 0xf4e09651, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e09656, 90 },
    { 0, 0 },
    { 0x78909658, 90 },
    { 0, 0 },
    { 0x7610965a, 55 },
    { 0, 0 },
    { 0xe0b0965c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3320966c, 55 },
    { 0xd800966d, 88 },
    { 0xa850966e, 43 },
    { 0xb9c0966f, 95 },
    { 0xd6509670, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe509675, 66 },
    { 0xc4b09675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd900967a, 61 },
    { 0x1270967a, 76 },
    { 0, 0 },
    { 0xa710967d, 95 },
    { 0x3930967e, 95 },
    { 0xa550967f, 34 },
    { 0xdc60967f, 23 },
    { 0x7a209681, 88 },
    { 0x96809682, 61 },
    { 0, 0 },
    { 0xa6009684, 95 },
    { 0x27b09685, 43 },
    { 0x52909686, 91 },
    { 0xa0e09686, 46 },
    { 0x4cd09686, 61 },
    { 0, 0 },
    { 0x9670968a, 52 },
    { 0xe050968b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a60968f, 55 },
    { 0x7da09690, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef60969a, 61 },
    { 0, 0 },
    { 0x6c00969c, 55 },
    { 0x7200969c, 90 },
    { 0, 0 },
    { 0xce20969f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651096a5, 33 },
    { 0xbfc096a6, 61 },
    { 0x879096a6, 33 },
    { 0x182096a7, 95 },
    { 0xae1096a9, 33 },
    { 0xac2096a9, 43 },
    { 0x571096ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59096af, 93 },
    { 0xaec096af, 90 },
    { 0xfd5096b1, 76 },
    { 0x357096b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4096b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6096be, 98 },
    { 0xbfa096bf, 43 },
    { 0, 0 },
    { 0xb6d096c1, 84 },
    { 0x4fe096c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf096c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x559096c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488096cd, 76 },
    { 0x7a7096ce, 88 },
    { 0xd17096cf, 95 },
    { 0x3e096d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e096d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad096d9, 19 },
    { 0x6ff096da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a096e0, 61 },
    { 0xceb096e0, 61 },
    { 0x856096e2, 33 },
    { 0x526096e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c096e7, 87 },
    { 0x8c6096e8, 34 },
    { 0, 0 },
    { 0x744096ea, 99 },
    { 0xcf2096eb, 95 },
    { 0, 0 },
    { 0x8fa096ed, 61 },
    { 0x56c096ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29096f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58096fb, 36 },
    { 0x660096fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc09700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3809704, 61 },
    { 0x4cd09704, 95 },
    { 0, 0 },
    { 0x1f009707, 98 },
    { 0, 0 },
    { 0x98b09709, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d80970c, 55 },
    { 0x1ce0970c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d309710, 55 },
    { 0, 0 },
    { 0xd3209712, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36709717, 33 },
    { 0, 0 },
    { 0xe6009719, 76 },
    { 0x71e0971a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6870971d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad709723, 33 },
    { 0x8e909723, 98 },
    { 0, 0 },
    { 0x4db09726, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0972a, 90 },
    { 0x8110972b, 23 },
    { 0xecf0972c, 23 },
    { 0xfa20972b, 76 },
    { 0x1880972e, 33 },
    { 0xd220972f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b09736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec09739, 95 },
    { 0x72709739, 43 },
    { 0x3000973a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5470973e, 76 },
    { 0xbbd0973f, 85 },
    { 0xb3709740, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77609743, 90 },
    { 0xf2909743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf0974f, 99 },
    { 0, 0 },
    { 0x96609751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f09756, 90 },
    { 0, 0 },
    { 0x31f09758, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6430975b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50975e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db09761, 42 },
    { 0x77609761, 95 },
    { 0xcb409762, 61 },
    { 0, 0 },
    { 0xa7809765, 61 },
    { 0xc5b09766, 61 },
    { 0, 0 },
    { 0x12509768, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab80976b, 76 },
    { 0x2640976c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4300976f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d09774, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c709777, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a0977a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87409781, 90 },
    { 0x94a09781, 95 },
    { 0x63309781, 91 },
    { 0xaef09784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec0978c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc009793, 60 },
    { 0x29209794, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a409798, 95 },
    { 0, 0 },
    { 0x18f0979a, 90 },
    { 0xcb0979b, 61 },
    { 0xd3f0979b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac097a2, 98 },
    { 0, 0 },
    { 0xafd097a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844097a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d097b1, 95 },
    { 0, 0 },
    { 0xd79097b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c097b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d097b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c097be, 81 },
    { 0, 0 },
    { 0xea4097c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e097c7, 29 },
    { 0xdf2097c7, 33 },
    { 0xc94097c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1097cc, 90 },
    { 0xaab097cc, 100 },
    { 0x3e097ce, 33 },
    { 0xf31097cd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832097d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8097d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb097df, 81 },
    { 0x1c1097df, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4097e3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30097e6, 34 },
    { 0x552097e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7097ed, 90 },
    { 0xafc097ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311097f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f097f5, 90 },
    { 0x985097f6, 23 },
    { 0x6e9097f7, 90 },
    { 0x568097f7, 61 },
    { 0x29b097f7, 66 },
    { 0x8da097fa, 61 },
    { 0xf9d097f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b097fe, 40 },
    { 0x2f8097ff, 76 },
    { 0, 0 },
    { 0x33009801, 76 },
    { 0, 0 },
    { 0x1ca09803, 61 },
    { 0xd4a09804, 46 },
    { 0x35109805, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a0980a, 98 },
    { 0x1560980b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f09811, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f609815, 67 },
    { 0x22b09816, 80 },
    { 0xebe09817, 100 },
    { 0x30709815, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e10981e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b09822, 90 },
    { 0, 0 },
    { 0x309824, 33 },
    { 0, 0 },
    { 0xea509826, 85 },
    { 0, 0 },
    { 0x69309828, 43 },
    { 0x4f909828, 33 },
    { 0x2cc0982a, 23 },
    { 0x9ba0982b, 23 },
    { 0x9a409829, 55 },
    { 0x37b0982d, 90 },
    { 0x6f80982e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66909834, 99 },
    { 0xd2109834, 77 },
    { 0x49409835, 90 },
    { 0xdf309834, 52 },
    { 0x3ba09837, 33 },
    { 0xfe509838, 90 },
    { 0, 0 },
    { 0x6640983b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f10983f, 98 },
    { 0xccc09840, 34 },
    { 0, 0 },
    { 0xd9709842, 46 },
    { 0xedd09843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d09848, 65 },
    { 0x5e109848, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b0984d, 55 },
    { 0xab70984d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce609851, 90 },
    { 0, 0 },
    { 0xd3409853, 90 },
    { 0x93509854, 100 },
    { 0, 0 },
    { 0xd4109856, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f09859, 40 },
    { 0x72f0985a, 33 },
    { 0xc9209859, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92409860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9009869, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8550986d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d09870, 43 },
    { 0x31309871, 68 },
    { 0xd9a09872, 33 },
    { 0, 0 },
    { 0x4e009874, 55 },
    { 0, 0 },
    { 0xd4909876, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9180987a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0987d, 66 },
    { 0x3e70987e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9009884, 64 },
    { 0x68a09885, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a80988a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa990988f, 90 },
    { 0xb6e0988f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae909893, 46 },
    { 0xa6a09894, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b209897, 12 },
    { 0x6d09898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8330989f, 23 },
    { 0, 0 },
    { 0xe28098a1, 61 },
    { 0x697098a2, 55 },
    { 0, 0 },
    { 0x7e5098a4, 23 },
    { 0x4ae098a5, 98 },
    { 0xf7098a6, 95 },
    { 0x45098a4, 61 },
    { 0x725098a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720098ac, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249098b0, 43 },
    { 0x25098b0, 43 },
    { 0x67c098b2, 55 },
    { 0x9dd098b3, 95 },
    { 0, 0 },
    { 0x40098b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b098b9, 69 },
    { 0x133098ba, 33 },
    { 0xa8e098ba, 61 },
    { 0xabc098bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f098c0, 98 },
    { 0x719098c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e098c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01098cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d098d0, 34 },
    { 0x5b5098d1, 98 },
    { 0xa66098d0, 98 },
    { 0x869098d3, 84 },
    { 0, 0 },
    { 0xd13098d5, 55 },
    { 0, 0 },
    { 0x55d098d7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2098db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc098df, 34 },
    { 0x5b8098df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x238098e3, 61 },
    { 0, 0 },
    { 0x8a1098e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e098e9, 75 },
    { 0, 0 },
    { 0x2b8098eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd098f0, 93 },
    { 0xf6a098f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69098f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e098fa, 64 },
    { 0x343098fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf098ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb09904, 33 },
    { 0xca609905, 33 },
    { 0xc7409904, 67 },
    { 0x9de09904, 98 },
    { 0x9cf09908, 55 },
    { 0x9e09906, 98 },
    { 0x37309908, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10990d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6009913, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9309918, 61 },
    { 0x41509918, 95 },
    { 0x63709919, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d40991d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e609922, 61 },
    { 0x2e909923, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b409937, 90 },
    { 0xefb09938, 61 },
    { 0xd909939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe40993e, 76 },
    { 0x45c0993e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae09944, 91 },
    { 0xbf809945, 61 },
    { 0xd8209945, 76 },
    { 0x9e909947, 90 },
    { 0xd2e09948, 76 },
    { 0x4ea09949, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e0994f, 43 },
    { 0, 0 },
    { 0x6cc09951, 75 },
    { 0x4ee09952, 95 },
    { 0, 0 },
    { 0xf509954, 61 },
    { 0, 0 },
    { 0x36609956, 80 },
    { 0xaf009957, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f0995c, 17 },
    { 0, 0 },
    { 0x4920995e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7809962, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe09967, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe10996a, 99 },
    { 0x20c0996b, 90 },
    { 0x5c70996c, 76 },
    { 0xee50996d, 61 },
    { 0x8260996e, 98 },
    { 0x5f20996c, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca09972, 95 },
    { 0, 0 },
    { 0xa9609974, 46 },
    { 0xc8209975, 61 },
    { 0xce409976, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef00997c, 99 },
    { 0, 0 },
    { 0xdba0997e, 43 },
    { 0xec40997f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d09982, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33509988, 82 },
    { 0x8af09988, 61 },
    { 0xa3f0998a, 33 },
    { 0x4db0998a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9480998e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f09991, 61 },
    { 0, 0 },
    { 0xbd709993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6309999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d0999e, 78 },
    { 0x3ef0999f, 98 },
    { 0x3ea0999f, 61 },
    { 0x643099a0, 23 },
    { 0x7f9099a1, 46 },
    { 0, 0 },
    { 0x982099a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c099a7, 90 },
    { 0xf76099a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f099af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6099b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66099b9, 52 },
    { 0, 0 },
    { 0xfc8099bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e099c8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e099cd, 98 },
    { 0xf7c099cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1099df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e099e2, 33 },
    { 0x2f4099e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6099eb, 23 },
    { 0xb7e099ec, 76 },
    { 0x632099eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477099f1, 61 },
    { 0, 0 },
    { 0xe71099f3, 33 },
    { 0x106099f3, 23 },
    { 0x71e099f5, 17 },
    { 0x761099f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e099f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db099fd, 84 },
    { 0xc3e099fe, 88 },
    { 0xa66099fe, 95 },
    { 0x329099ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61609a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4409a08, 69 },
    { 0xea809a09, 33 },
    { 0x6f309a08, 76 },
    { 0x3c209a0b, 98 },
    { 0xa7e09a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5309a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf109a12, 43 },
    { 0xd3409a13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3009a16, 99 },
    { 0x2d409a17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b709a1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed209a1d, 93 },
    { 0x64609a1e, 76 },
    { 0xd6609a1f, 55 },
    { 0x27e09a20, 61 },
    { 0, 0 },
    { 0x1f09a22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35609a2b, 77 },
    { 0x56109a2c, 98 },
    { 0, 0 },
    { 0x1209a2e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5009a35, 90 },
    { 0x67609a36, 61 },
    { 0x6cf09a35, 95 },
    { 0xc3c09a38, 55 },
    { 0x2fb09a37, 67 },
    { 0xce009a39, 64 },
    { 0xce309a3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80309a40, 62 },
    { 0x59109a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79909a44, 98 },
    { 0, 0 },
    { 0xcd109a46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10209a4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c09a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b09a62, 34 },
    { 0xba209a62, 99 },
    { 0xd1509a63, 52 },
    { 0, 0 },
    { 0xe6109a66, 61 },
    { 0x5d009a67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a09a6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9509a6d, 98 },
    { 0xa7709a6e, 78 },
    { 0x96b09a6f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b09a72, 61 },
    { 0xd0f09a73, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b09a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20409a7d, 65 },
    { 0x38509a7e, 23 },
    { 0x88209a7e, 46 },
    { 0x5d709a80, 90 },
    { 0x4e009a81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae09a85, 33 },
    { 0x24f09a85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4709a89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6909a8f, 95 },
    { 0x7bc09a90, 91 },
    { 0, 0 },
    { 0x19009a92, 33 },
    { 0x68d09a92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f09a96, 78 },
    { 0, 0 },
    { 0xc109a98, 23 },
    { 0x27909a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16309a9e, 61 },
    { 0xe1309a9e, 90 },
    { 0xfa09aa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c209aa3, 78 },
    { 0xbcc09aa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b09aaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c909aad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25309ab1, 51 },
    { 0xeb609ab2, 61 },
    { 0x41809ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42309ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d09abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d09ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa109ac4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x28809ac7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95509aca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68009acf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be09ad3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4309ad6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c309add, 55 },
    { 0xf1509ade, 88 },
    { 0xa7509add, 95 },
    { 0, 0 },
    { 0xd909ae1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75309ae4, 95 },
    { 0x1ed09ae5, 23 },
    { 0x59909ae6, 99 },
    { 0, 0 },
    { 0x90909ae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47609aec, 23 },
    { 0xdd009aec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38109af0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92109af4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9109afc, 95 },
    { 0, 0 },
    { 0x59e09afe, 61 },
    { 0xe8309aff, 43 },
    { 0, 0 },
    { 0xb6d09b01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba09b05, 69 },
    { 0, 0 },
    { 0x63009b07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e09b0d, 56 },
    { 0x5c509b0e, 33 },
    { 0, 0 },
    { 0x9509b10, 46 },
    { 0, 0 },
    { 0xb2b09b12, 61 },
    { 0x76709b13, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a09b18, 76 },
    { 0, 0 },
    { 0x3d509b1a, 77 },
    { 0x3b409b1a, 98 },
    { 0x7d109b1b, 61 },
    { 0x8cb09b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db09b22, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40109b28, 46 },
    { 0xeee09b29, 68 },
    { 0, 0 },
    { 0xff09b2b, 52 },
    { 0x29a09b2c, 33 },
    { 0xb9e09b2d, 23 },
    { 0xbf609b2e, 23 },
    { 0x4b409b2b, 55 },
    { 0x91509b30, 61 },
    { 0x6909b30, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e09b38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf709b3d, 55 },
    { 0xced09b3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c09b44, 52 },
    { 0xda009b45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57309b48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53309b50, 98 },
    { 0xba309b51, 85 },
    { 0x59d09b51, 90 },
    { 0, 0 },
    { 0x74d09b54, 52 },
    { 0x40c09b55, 76 },
    { 0x71e09b55, 64 },
    { 0, 0 },
    { 0x3df09b58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e09b60, 76 },
    { 0xe8609b61, 46 },
    { 0xc2209b62, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d09b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2909b6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf09b70, 34 },
    { 0x56109b71, 98 },
    { 0x7bc09b72, 100 },
    { 0xfe409b73, 34 },
    { 0xdf309b70, 55 },
    { 0x8cd09b74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69909b78, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78709b81, 23 },
    { 0x63809b82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb409b86, 55 },
    { 0, 0 },
    { 0xa7809b88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29609b91, 98 },
    { 0, 0 },
    { 0x98109b93, 23 },
    { 0x8ea09b94, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa909b98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64509b9d, 56 },
    { 0x3ea09b9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a309ba1, 91 },
    { 0, 0 },
    { 0xa6e09ba3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c09ba7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d09bae, 90 },
    { 0x2f09baf, 76 },
    { 0x9bc09baf, 33 },
    { 0x89709baf, 61 },
    { 0x83309bb1, 76 },
    { 0x6d909bb3, 100 },
    { 0xc6a09bb4, 75 },
    { 0, 0 },
    { 0xc3609bb6, 84 },
    { 0, 0 },
    { 0x6a809bb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0309bbd, 85 },
    { 0x45509bbd, 47 },
    { 0x36709bbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10509bc3, 93 },
    { 0xfc09bc3, 66 },
    { 0x8d509bc5, 90 },
    { 0x23609bc3, 77 },
    { 0x77309bc3, 77 },
    { 0xb209bc8, 36 },
    { 0x12909bc9, 33 },
    { 0, 0 },
    { 0x3df09bcb, 90 },
    { 0xbb109bcc, 43 },
    { 0xa5009bcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d09bd4, 55 },
    { 0, 0 },
    { 0xb3509bd6, 95 },
    { 0, 0 },
    { 0x89509bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d09bdc, 78 },
    { 0, 0 },
    { 0x67009bde, 95 },
    { 0, 0 },
    { 0x3e409be0, 33 },
    { 0x25909be0, 55 },
    { 0x4d09be2, 91 },
    { 0xa4309be3, 61 },
    { 0x6f009be0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x11709be7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c09beb, 81 },
    { 0x77209beb, 100 },
    { 0xf409beb, 34 },
    { 0xe1f09bee, 98 },
    { 0xdbb09beb, 19 },
    { 0x3a209bee, 76 },
    { 0, 0 },
    { 0x9909bf2, 98 },
    { 0xa7a09bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31009bf8, 62 },
    { 0x97709bf9, 34 },
    { 0xa5009bf9, 99 },
    { 0x52909bfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3409bff, 78 },
    { 0x73c09c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a09c08, 85 },
    { 0x44509c09, 52 },
    { 0, 0 },
    { 0x71e09c0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc09c16, 33 },
    { 0x86509c16, 93 },
    { 0, 0 },
    { 0x68209c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67309c1e, 76 },
    { 0, 0 },
    { 0x21a09c20, 76 },
    { 0x3dd09c21, 90 },
    { 0, 0 },
    { 0xfd309c23, 90 },
    { 0x7e209c24, 38 },
    { 0x7a809c24, 95 },
    { 0, 0 },
    { 0x68f09c27, 61 },
    { 0xb8d09c27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b09c2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x47209c32, 76 },
    { 0x18e09c33, 90 },
    { 0xb7809c34, 55 },
    { 0x45209c32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b09c38, 61 },
    { 0x8de09c38, 34 },
    { 0xa1909c39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e09c45, 33 },
    { 0x4c09c46, 98 },
    { 0x6e609c46, 67 },
    { 0x36e09c48, 46 },
    { 0xe2809c49, 69 },
    { 0xfa909c4a, 61 },
    { 0xf2b09c4a, 61 },
    { 0xead09c4c, 99 },
    { 0xed809c48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b09c50, 87 },
    { 0xd3509c51, 33 },
    { 0x66609c51, 90 },
    { 0xf9f09c50, 23 },
    { 0x73509c54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d209c57, 78 },
    { 0x27609c58, 77 },
    { 0xc5909c57, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa09c5e, 23 },
    { 0x1a409c5e, 95 },
    { 0x2ff09c60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0109c63, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef09c6d, 52 },
    { 0x94b09c6e, 90 },
    { 0xc6909c6d, 90 },
    { 0xc8009c6d, 90 },
    { 0x42409c71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d209c74, 40 },
    { 0, 0 },
    { 0x58b09c76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3609c7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a09c87, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e09c8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e609c91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b09c97, 55 },
    { 0xae09c98, 87 },
    { 0, 0 },
    { 0x76b09c9a, 98 },
    { 0xf0409c9b, 61 },
    { 0xbf709c9c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf909ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d09ca4, 90 },
    { 0x50e09ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e09ca9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98209caf, 61 },
    { 0x8f09caf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc09cb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5409cba, 33 },
    { 0, 0 },
    { 0x37109cbc, 91 },
    { 0x91909cbc, 33 },
    { 0x7f209cbe, 61 },
    { 0, 0 },
    { 0x92d09cc0, 53 },
    { 0xffa09cc1, 95 },
    { 0xeb409cc2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xca309cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e09ccb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d09ccf, 90 },
    { 0xf6709ccf, 23 },
    { 0x36309cd1, 88 },
    { 0x5cd09cd2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88409ce1, 23 },
    { 0xfc009ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb09ce5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x54609ce8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a09ceb, 23 },
    { 0x87d09cec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e509cf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97109cf9, 61 },
    { 0xb5b09cfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a09cfd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd09d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa09d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24909d0d, 23 },
    { 0x3f009d0d, 77 },
    { 0xa7209d0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47409d13, 90 },
    { 0x4b309d13, 61 },
    { 0x4c109d15, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e09d18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d09d1b, 40 },
    { 0x46a09d1c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde109d20, 76 },
    { 0x5d109d21, 34 },
    { 0xc3809d22, 65 },
    { 0x6bb09d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a909d28, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x73509d2b, 23 },
    { 0x2af09d2c, 55 },
    { 0x75009d2c, 78 },
    { 0xc5d09d2e, 91 },
    { 0x9a509d2f, 76 },
    { 0xa6d09d2f, 93 },
    { 0x19609d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e09d35, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x79209d38, 34 },
    { 0, 0 },
    { 0x21809d3a, 92 },
    { 0x57d09d3b, 61 },
    { 0, 0 },
    { 0xffc09d3d, 33 },
    { 0xf6209d3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14709d43, 33 },
    { 0x4de09d43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8109d49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3709d53, 90 },
    { 0, 0 },
    { 0xbb409d55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d09d5c, 93 },
    { 0x1b709d5d, 66 },
    { 0, 0 },
    { 0x82409d5f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1409d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f809d68, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e09d6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8009d73, 91 },
    { 0x28c09d74, 101 },
    { 0xb5c09d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b09d7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b09d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da09d88, 65 },
    { 0, 0 },
    { 0xcf809d8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15709d90, 90 },
    { 0x23e09d91, 77 },
    { 0, 0 },
    { 0x66309d93, 93 },
    { 0xff509d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d09da0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc709da4, 90 },
    { 0x64c09da5, 23 },
    { 0x44009da5, 34 },
    { 0x8ea09da7, 33 },
    { 0x28b09da8, 95 },
    { 0xd2d09da4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e09dad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a09db0, 91 },
    { 0x28c09db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a909db8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a309dbc, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709dc3, 52 },
    { 0, 0 },
    { 0x8ed09dc5, 46 },
    { 0x5a609dc5, 23 },
    { 0x36f09dc7, 95 },
    { 0, 0 },
    { 0xaea09dc9, 33 },
    { 0x609dca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95109dce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90609dd7, 85 },
    { 0xafa09dd8, 77 },
    { 0x37009dd8, 66 },
    { 0, 0 },
    { 0x23709ddb, 91 },
    { 0, 0 },
    { 0x5ba09ddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23609de3, 90 },
    { 0, 0 },
    { 0x7c609de5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17609df0, 33 },
    { 0x7ae09df1, 90 },
    { 0x56e09df2, 61 },
    { 0xe2309df2, 61 },
    { 0x2209df4, 61 },
    { 0xe209df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e09dfa, 53 },
    { 0x37e09dfa, 93 },
    { 0x4809dfb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e09e00, 66 },
    { 0x3aa09e01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d09e0e, 76 },
    { 0, 0 },
    { 0xc1d09e10, 52 },
    { 0, 0 },
    { 0xcba09e12, 91 },
    { 0xcb609e13, 88 },
    { 0x47309e14, 46 },
    { 0xbdb09e15, 58 },
    { 0x24609e16, 88 },
    { 0x11b09e16, 84 },
    { 0x80409e18, 66 },
    { 0x1f209e16, 98 },
    { 0x66209e19, 98 },
    { 0, 0 },
    { 0x75409e1c, 69 },
    { 0x8cb09e1c, 23 },
    { 0x34009e1e, 93 },
    { 0, 0 },
    { 0xbcd09e20, 55 },
    { 0xbe109e20, 33 },
    { 0x98809e22, 93 },
    { 0x1c209e23, 98 },
    { 0x2a009e23, 33 },
    { 0, 0 },
    { 0x62c09e26, 75 },
    { 0x74b09e27, 23 },
    { 0x4d409e28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef909e2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec009e33, 95 },
    { 0x82809e34, 64 },
    { 0, 0 },
    { 0x4ae09e36, 33 },
    { 0, 0 },
    { 0xe0909e38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa409e3b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x42709e3e, 33 },
    { 0xc9709e3f, 67 },
    { 0x1609e3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6309e44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc09e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8809e53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56509e57, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9409e5b, 23 },
    { 0x16509e5c, 88 },
    { 0x80d09e5d, 95 },
    { 0x52e09e5c, 101 },
    { 0xd4b09e5f, 90 },
    { 0x11809e60, 43 },
    { 0x26c09e61, 90 },
    { 0xac609e5d, 95 },
    { 0xb9609e5f, 47 },
    { 0xcc709e5f, 84 },
    { 0x1d509e5e, 95 },
    { 0x15a09e66, 33 },
    { 0, 0 },
    { 0xc0909e68, 33 },
    { 0x95709e69, 76 },
    { 0x22809e69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a09e6d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d09e74, 23 },
    { 0, 0 },
    { 0x2f09e76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7409e7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a709e81, 46 },
    { 0, 0 },
    { 0xe5e09e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4209e8b, 43 },
    { 0x9e009e8b, 33 },
    { 0, 0 },
    { 0xb809e8e, 23 },
    { 0, 0 },
    { 0xa2b09e90, 90 },
    { 0xce509e91, 91 },
    { 0, 0 },
    { 0x61709e93, 46 },
    { 0x2509e94, 34 },
    { 0xfc109e93, 78 },
    { 0xa0009e96, 43 },
    { 0x20409e94, 17 },
    { 0x38309e98, 87 },
    { 0x9d209e93, 91 },
    { 0x70f09e9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa09e9f, 95 },
    { 0x3f609ea0, 90 },
    { 0, 0 },
    { 0x2a909ea2, 98 },
    { 0xbb909ea3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda409ea8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a09eae, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d09eb1, 61 },
    { 0, 0 },
    { 0x52309eb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5409eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9109ebb, 61 },
    { 0xafd09ebc, 33 },
    { 0, 0 },
    { 0xcb409ebe, 90 },
    { 0xbfb09ebe, 95 },
    { 0, 0 },
    { 0x3c409ec1, 96 },
    { 0xf4f09ec1, 87 },
    { 0xfde09ec3, 33 },
    { 0xcc809ec2, 33 },
    { 0x48509ec3, 98 },
    { 0xdc209ec6, 90 },
    { 0xc0409ec7, 33 },
    { 0xd2909ec4, 33 },
    { 0xc7109ec9, 61 },
    { 0x48009eca, 33 },
    { 0x36e09ec6, 100 },
    { 0, 0 },
    { 0xce909ecd, 98 },
    { 0xb6109ece, 23 },
    { 0xe0709ece, 55 },
    { 0x2b409ecd, 43 },
    { 0x8fc09ed1, 90 },
    { 0x8c409ed2, 90 },
    { 0x20109ed3, 66 },
    { 0x48109ed0, 90 },
    { 0x8d709ed5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d909ed9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xca209edc, 61 },
    { 0xc6409edd, 90 },
    { 0x86809ede, 61 },
    { 0, 0 },
    { 0xae409ee0, 80 },
    { 0xc3e09ee0, 61 },
    { 0x53609ee0, 33 },
    { 0x2ae09ee3, 95 },
    { 0, 0 },
    { 0x76109ee5, 91 },
    { 0, 0 },
    { 0xea09ee7, 46 },
    { 0, 0 },
    { 0xacf09ee9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa09eed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda09ef7, 95 },
    { 0, 0 },
    { 0x96309ef9, 78 },
    { 0x55b09ef9, 99 },
    { 0x4a309efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6209eff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b09f03, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd409f0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8509f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac09f11, 85 },
    { 0x45909f11, 95 },
    { 0x32f09f13, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a209f16, 95 },
    { 0, 0 },
    { 0x4a609f18, 90 },
    { 0, 0 },
    { 0xa1109f1a, 33 },
    { 0x20e09f1b, 52 },
    { 0x50c09f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e09f21, 84 },
    { 0, 0 },
    { 0x7a009f23, 98 },
    { 0x7f809f24, 98 },
    { 0xcb509f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc09f28, 55 },
    { 0xea409f29, 40 },
    { 0x70009f29, 33 },
    { 0, 0 },
    { 0xd2009f2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4509f2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c09f38, 99 },
    { 0x2b109f38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e309f43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf09f46, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a609f4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c09f4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e09f55, 95 },
    { 0xa1809f56, 61 },
    { 0xc0a09f56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f909f5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17409f60, 34 },
    { 0, 0 },
    { 0x77a09f62, 90 },
    { 0xeaa09f63, 99 },
    { 0, 0 },
    { 0x41109f65, 91 },
    { 0x7ea09f66, 23 },
    { 0xbc709f65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd509f6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b809f6d, 84 },
    { 0xe1b09f6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee09f72, 90 },
    { 0, 0 },
    { 0xf6409f74, 96 },
    { 0xd0509f75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af09f7e, 100 },
    { 0x23709f7f, 33 },
    { 0, 0 },
    { 0xad109f81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96109f87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81009f8f, 33 },
    { 0x47c09f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f09f93, 66 },
    { 0xd5c09f93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae09f98, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84609f9c, 61 },
    { 0x10309f9c, 95 },
    { 0x85a09f9c, 93 },
    { 0x43209f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0509fa2, 90 },
    { 0x3b109fa3, 61 },
    { 0xd8709fa4, 55 },
    { 0xd5a09fa3, 96 },
    { 0, 0 },
    { 0xa3909fa7, 95 },
    { 0xef909fa8, 98 },
    { 0x5ea09fa9, 98 },
    { 0xeb409fa8, 90 },
    { 0xea709fab, 43 },
    { 0x23909fac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6109fb6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd09fbf, 90 },
    { 0, 0 },
    { 0xc3309fc1, 36 },
    { 0x3f909fc2, 33 },
    { 0x74009fc3, 46 },
    { 0xa4309fc4, 84 },
    { 0x25b09fc5, 90 },
    { 0x40709fc6, 61 },
    { 0, 0 },
    { 0x93609fc8, 90 },
    { 0x70409fc8, 76 },
    { 0xfd609fc8, 34 },
    { 0xda09fcb, 76 },
    { 0, 0 },
    { 0xdb009fcd, 46 },
    { 0x96c09fcd, 90 },
    { 0xd7709fcf, 61 },
    { 0x1509fd0, 66 },
    { 0xa8609fcd, 95 },
    { 0xd7609fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0609fd5, 90 },
    { 0, 0 },
    { 0x36f09fd7, 98 },
    { 0x5309fd8, 33 },
    { 0x3e309fd8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee409fde, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8109fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62409fe5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf909fed, 90 },
    { 0, 0 },
    { 0xba709fef, 90 },
    { 0x91209ff0, 61 },
    { 0xec909ff0, 43 },
    { 0, 0 },
    { 0x309ff3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6609ff6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c409ffa, 33 },
    { 0x72f09ffb, 46 },
    { 0xc2c09ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b0a004, 85 },
    { 0xb5b0a004, 52 },
    { 0x4c50a005, 23 },
    { 0x8330a007, 90 },
    { 0, 0 },
    { 0xfb90a009, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a0a00c, 61 },
    { 0x9890a00c, 33 },
    { 0xe4c0a00e, 23 },
    { 0xc1d0a00f, 33 },
    { 0xd220a00e, 61 },
    { 0, 0 },
    { 0xb550a012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb60a019, 61 },
    { 0x380a01a, 34 },
    { 0xf550a01a, 90 },
    { 0xb1a0a01b, 33 },
    { 0x61e0a01d, 76 },
    { 0, 0 },
    { 0xab10a01f, 93 },
    { 0, 0 },
    { 0x8b80a021, 98 },
    { 0x50e0a022, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa0a028, 76 },
    { 0x70f0a029, 90 },
    { 0, 0 },
    { 0xb590a02b, 95 },
    { 0x1330a02b, 95 },
    { 0x5170a02d, 61 },
    { 0, 0 },
    { 0xf150a02f, 90 },
    { 0xec40a030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef0a037, 55 },
    { 0x7e80a038, 76 },
    { 0xd380a039, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8300a03d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b30a043, 33 },
    { 0xb3e0a044, 90 },
    { 0xc290a043, 98 },
    { 0xb510a046, 33 },
    { 0x40f0a045, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a00a04a, 76 },
    { 0xdfa0a04b, 17 },
    { 0xbe50a04c, 33 },
    { 0x9010a04d, 61 },
    { 0xc610a04e, 94 },
    { 0xde50a04a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf320a054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea50a05c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc60a05f, 90 },
    { 0, 0 },
    { 0xc620a061, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8760a065, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd870a068, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d0a06b, 76 },
    { 0xd3e0a06c, 23 },
    { 0, 0 },
    { 0xdcd0a06e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1200a077, 55 },
    { 0x1760a077, 91 },
    { 0x2890a079, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd540a07e, 65 },
    { 0, 0 },
    { 0x6c00a080, 43 },
    { 0x1020a081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce60a086, 33 },
    { 0, 0 },
    { 0x29c0a088, 77 },
    { 0xb170a089, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e0a094, 61 },
    { 0x90a095, 33 },
    { 0xac20a095, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c0a09b, 95 },
    { 0xdb0a09c, 61 },
    { 0x4530a09d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc50a0a2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b60a0ab, 61 },
    { 0x3450a0ac, 46 },
    { 0xac60a0ad, 77 },
    { 0x5900a0ae, 77 },
    { 0x6f50a0ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d0a0b4, 99 },
    { 0x8a60a0b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b70a0b8, 67 },
    { 0xded0a0b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec0a0bc, 33 },
    { 0, 0 },
    { 0x4cd0a0be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c0a0c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e20a0c5, 43 },
    { 0, 0 },
    { 0x22d0a0c7, 61 },
    { 0x4ef0a0c8, 61 },
    { 0, 0 },
    { 0xc160a0ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0a0ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3350a0d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90a0dd, 43 },
    { 0x5470a0dd, 90 },
    { 0, 0 },
    { 0x8a50a0e0, 61 },
    { 0xa550a0e1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf40a0e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a0a0ef, 55 },
    { 0xa7b0a0f0, 17 },
    { 0, 0 },
    { 0xe20a0f2, 43 },
    { 0xaef0a0f3, 23 },
    { 0x3c50a0f4, 55 },
    { 0x2a10a0f5, 38 },
    { 0x8830a0f6, 43 },
    { 0, 0 },
    { 0x1e10a0f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2360a0fb, 23 },
    { 0x7700a0fc, 33 },
    { 0xc8a0a0fd, 76 },
    { 0x87d0a0fb, 61 },
    { 0x12f0a0ff, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf180a103, 55 },
    { 0x7fe0a104, 95 },
    { 0xc630a105, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7240a108, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1590a10b, 98 },
    { 0x560a10c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf220a111, 90 },
    { 0xf450a112, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f60a116, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd480a11b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a0a124, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe450a128, 23 },
    { 0, 0 },
    { 0x6a10a12a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0a130, 95 },
    { 0xfb30a131, 99 },
    { 0xef90a130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd0a135, 76 },
    { 0x62f0a135, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a139, 90 },
    { 0x2700a13a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7890a13d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570a143, 61 },
    { 0, 0 },
    { 0x2e00a145, 61 },
    { 0, 0 },
    { 0x1ba0a147, 61 },
    { 0x2560a147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120a14c, 90 },
    { 0x5ce0a14d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd880a153, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a0a158, 87 },
    { 0xc990a158, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20a15e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3040a162, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7050a165, 23 },
    { 0x4670a166, 49 },
    { 0x6e20a167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e50a16e, 33 },
    { 0x5870a16f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c0a172, 46 },
    { 0xf680a172, 61 },
    { 0, 0 },
    { 0x6990a175, 66 },
    { 0xb820a175, 100 },
    { 0, 0 },
    { 0x4c40a178, 83 },
    { 0x2220a178, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b40a17d, 23 },
    { 0x7600a17e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1740a181, 76 },
    { 0x3f60a182, 90 },
    { 0xc8d0a183, 46 },
    { 0, 0 },
    { 0xa80a185, 90 },
    { 0x5330a186, 76 },
    { 0x2520a185, 90 },
    { 0x7880a188, 33 },
    { 0xab80a189, 33 },
    { 0x8a50a18a, 98 },
    { 0xead0a18b, 85 },
    { 0x1010a18a, 33 },
    { 0x2440a18b, 65 },
    { 0x69d0a18e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7790a195, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf110a198, 61 },
    { 0x7cf0a199, 90 },
    { 0x4900a19a, 23 },
    { 0xfff0a199, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5840a19e, 61 },
    { 0x3830a19f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd0a1a3, 86 },
    { 0x3890a1a3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a1ab, 33 },
    { 0, 0 },
    { 0x4930a1ad, 61 },
    { 0x7ed0a1ae, 61 },
    { 0, 0 },
    { 0x4910a1b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e0a1b3, 70 },
    { 0x9bc0a1b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1570a1b8, 23 },
    { 0, 0 },
    { 0xf930a1ba, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9110a1be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680a1c7, 76 },
    { 0x130a1c8, 61 },
    { 0xe260a1c8, 95 },
    { 0, 0 },
    { 0x8ef0a1cb, 23 },
    { 0, 0 },
    { 0x9880a1cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a1d4, 23 },
    { 0x55a0a1d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc90a1d8, 55 },
    { 0x8660a1d9, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a0a1de, 64 },
    { 0x95c0a1de, 98 },
    { 0, 0 },
    { 0xd990a1e1, 76 },
    { 0x3680a1e1, 33 },
    { 0xc4c0a1e3, 33 },
    { 0xa840a1e4, 23 },
    { 0xcc0a1e5, 95 },
    { 0x28c0a1e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7860a1ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c20a1f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe160a1f8, 88 },
    { 0x85f0a1f9, 47 },
    { 0x9bb0a1fa, 90 },
    { 0, 0 },
    { 0xc080a1fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e30a1ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6720a203, 99 },
    { 0x82f0a204, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f0a207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a20a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc0a20d, 78 },
    { 0x6510a20e, 34 },
    { 0, 0 },
    { 0x460a210, 34 },
    { 0xcac0a211, 34 },
    { 0xbbf0a212, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f0a215, 93 },
    { 0x6cb0a216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800a21d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b70a220, 34 },
    { 0xd050a220, 55 },
    { 0x54b0a222, 55 },
    { 0x9a90a223, 85 },
    { 0x9b70a222, 33 },
    { 0xeab0a224, 100 },
    { 0x2ac0a226, 95 },
    { 0xe180a225, 23 },
    { 0x2a50a228, 98 },
    { 0x13c0a228, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd010a22e, 55 },
    { 0xde40a22f, 88 },
    { 0xbc10a230, 95 },
    { 0, 0 },
    { 0x8c0a232, 98 },
    { 0, 0 },
    { 0x5510a234, 59 },
    { 0x850a235, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9600a239, 95 },
    { 0x9c50a239, 23 },
    { 0x77c0a23a, 33 },
    { 0, 0 },
    { 0x3bd0a23d, 55 },
    { 0x7fb0a23e, 91 },
    { 0xf9d0a23e, 23 },
    { 0, 0 },
    { 0x540a241, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d0a244, 84 },
    { 0xa780a245, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d0a249, 61 },
    { 0, 0 },
    { 0xb2d0a24b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40a24e, 90 },
    { 0xd8a0a24e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d60a255, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f30a25b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40a25e, 76 },
    { 0, 0 },
    { 0x26d0a260, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a70a263, 93 },
    { 0x91d0a264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe410a267, 95 },
    { 0x3900a268, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590a26f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4770a278, 61 },
    { 0x1aa0a278, 23 },
    { 0x5b10a27a, 87 },
    { 0xbba0a27b, 67 },
    { 0x4140a27c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa10a280, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c0a283, 90 },
    { 0, 0 },
    { 0x2e00a285, 95 },
    { 0xae70a286, 55 },
    { 0x6c00a287, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c0a292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0a29a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa0a29e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0a2a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0a2a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e30a2a9, 23 },
    { 0x3ce0a2aa, 23 },
    { 0xb470a2ab, 76 },
    { 0x1ae0a2ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0a2b0, 61 },
    { 0xb500a2b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc10a2b4, 55 },
    { 0xcb50a2b5, 95 },
    { 0x6770a2b6, 33 },
    { 0, 0 },
    { 0xe6e0a2b8, 78 },
    { 0, 0 },
    { 0xd8c0a2ba, 98 },
    { 0xe410a2bb, 90 },
    { 0x4140a2bb, 33 },
    { 0, 0 },
    { 0x7620a2be, 80 },
    { 0x26a0a2bf, 46 },
    { 0x890a2c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f0a2c3, 76 },
    { 0x7940a2c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d80a2c8, 23 },
    { 0, 0 },
    { 0x1c10a2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb110a2cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda0a2d0, 93 },
    { 0x61d0a2d1, 59 },
    { 0, 0 },
    { 0x2770a2d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d0a2d6, 90 },
    { 0x5510a2d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe650a2e0, 76 },
    { 0, 0 },
    { 0xe940a2e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9840a2e9, 69 },
    { 0, 0 },
    { 0xebc0a2eb, 65 },
    { 0x9450a2eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630a2f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b0a2f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c0a2fb, 33 },
    { 0x8730a2fb, 87 },
    { 0x57b0a2fc, 75 },
    { 0xb1e0a2fb, 87 },
    { 0x3530a2fb, 93 },
    { 0xd680a2fb, 93 },
    { 0xb1d0a301, 33 },
    { 0xb080a302, 43 },
    { 0xac00a303, 95 },
    { 0x95f0a303, 98 },
    { 0, 0 },
    { 0xcfe0a306, 90 },
    { 0x9b80a306, 76 },
    { 0xa360a308, 43 },
    { 0, 0 },
    { 0x780a30a, 95 },
    { 0xa720a30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7480a30e, 98 },
    { 0x36f0a30f, 84 },
    { 0x38f0a310, 90 },
    { 0x4470a310, 76 },
    { 0x76b0a312, 59 },
    { 0x33e0a311, 61 },
    { 0, 0 },
    { 0xa280a315, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9630a31b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3610a31f, 55 },
    { 0, 0 },
    { 0x86c0a321, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e80a324, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae0a327, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e0a32d, 66 },
    { 0, 0 },
    { 0x3c90a32f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7920a335, 61 },
    { 0, 0 },
    { 0x67f0a337, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf440a33f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d0a343, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d0a346, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd10a34b, 90 },
    { 0, 0 },
    { 0xafe0a34d, 61 },
    { 0, 0 },
    { 0x3ad0a34f, 52 },
    { 0xba10a34f, 98 },
    { 0x38d0a351, 43 },
    { 0x84d0a352, 76 },
    { 0, 0 },
    { 0xfae0a354, 100 },
    { 0xbf30a355, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a0a35a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe190a35e, 91 },
    { 0, 0 },
    { 0xa830a360, 33 },
    { 0x8dc0a361, 61 },
    { 0, 0 },
    { 0x2e40a363, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f0a369, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f00a36c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20a36f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d0a37f, 55 },
    { 0xa1b0a37f, 66 },
    { 0x5ab0a380, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5400a38d, 90 },
    { 0, 0 },
    { 0xfd80a38f, 91 },
    { 0, 0 },
    { 0x4f00a391, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae0a397, 33 },
    { 0xc460a397, 61 },
    { 0x2fe0a399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd910a39d, 95 },
    { 0, 0 },
    { 0xf5f0a39f, 64 },
    { 0x9350a3a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e0a3a4, 18 },
    { 0x4230a3a5, 52 },
    { 0x7aa0a3a5, 84 },
    { 0xede0a3a7, 34 },
    { 0xfeb0a3a5, 23 },
    { 0xbb70a3a9, 33 },
    { 0x1740a3aa, 98 },
    { 0, 0 },
    { 0x8510a3ac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8580a3b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0a3b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad30a3b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490a3bf, 87 },
    { 0x6530a3c0, 23 },
    { 0xbdc0a3c1, 75 },
    { 0x4820a3bf, 96 },
    { 0xbdd0a3c3, 43 },
    { 0xd030a3c3, 95 },
    { 0xc340a3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f0a3c8, 33 },
    { 0, 0 },
    { 0x5b30a3ca, 95 },
    { 0xeb70a3cb, 23 },
    { 0xde90a3cc, 98 },
    { 0xf420a3cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff80a3d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b0a3d7, 23 },
    { 0xc720a3d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc0a3dc, 23 },
    { 0x5b00a3dd, 87 },
    { 0x7850a3de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d0a3e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90a3e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd0a3e7, 34 },
    { 0, 0 },
    { 0x2ed0a3e9, 43 },
    { 0, 0 },
    { 0x8020a3eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d0a3ee, 33 },
    { 0xee20a3ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f0a3f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e40a3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc90a3fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa280a403, 36 },
    { 0x4f60a403, 98 },
    { 0xa1d0a405, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0a411, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1690a415, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f90a41a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be0a424, 34 },
    { 0x23e0a425, 33 },
    { 0, 0 },
    { 0xeab0a427, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790a42c, 41 },
    { 0x4210a42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4740a431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10a434, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae0a438, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef0a43e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a0a443, 88 },
    { 0xd950a444, 95 },
    { 0, 0 },
    { 0xdf60a446, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa590a449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1390a44c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f0a450, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a0a455, 85 },
    { 0x2bb0a455, 55 },
    { 0xa950a456, 43 },
    { 0xe380a458, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f0a45d, 98 },
    { 0, 0 },
    { 0xc970a45f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b0a462, 85 },
    { 0, 0 },
    { 0x4150a464, 90 },
    { 0xc80a465, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b70a46b, 52 },
    { 0, 0 },
    { 0xb3b0a46d, 77 },
    { 0, 0 },
    { 0x18f0a46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e0a478, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe20a47b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc60a47f, 34 },
    { 0, 0 },
    { 0xf4b0a481, 78 },
    { 0xc610a481, 27 },
    { 0x4ff0a483, 66 },
    { 0xbdf0a484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6180a48c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0a490, 95 },
    { 0xf2d0a490, 23 },
    { 0x4f30a492, 95 },
    { 0xf1a0a493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a0a49c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4310a4a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8910a4a5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea0a4ad, 36 },
    { 0, 0 },
    { 0x8000a4af, 100 },
    { 0x7e70a4b0, 77 },
    { 0x89f0a4af, 87 },
    { 0x2b60a4b2, 34 },
    { 0xa310a4b3, 66 },
    { 0x17c0a4b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a90a4bb, 33 },
    { 0x6f0a4bc, 43 },
    { 0xc260a4bc, 33 },
    { 0x5f80a4be, 61 },
    { 0x61a0a4be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2880a4ca, 23 },
    { 0x3160a4cb, 67 },
    { 0x9b50a4cc, 52 },
    { 0x2240a4cc, 34 },
    { 0x2700a4cb, 98 },
    { 0xddd0a4ce, 33 },
    { 0x5650a4d0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d30a4d6, 90 },
    { 0, 0 },
    { 0xa0c0a4d8, 90 },
    { 0xcd00a4d8, 98 },
    { 0xbb90a4d8, 95 },
    { 0x8330a4db, 23 },
    { 0x2ba0a4dc, 77 },
    { 0xdc40a4db, 98 },
    { 0x6990a4d8, 98 },
    { 0x8e0a4da, 55 },
    { 0, 0 },
    { 0xed00a4e1, 78 },
    { 0xb540a4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c0a4e6, 93 },
    { 0, 0 },
    { 0x7360a4e8, 91 },
    { 0xc360a4e9, 23 },
    { 0, 0 },
    { 0x8770a4eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4110a4ee, 34 },
    { 0x1830a4ef, 100 },
    { 0x4240a4f0, 55 },
    { 0x6880a4f0, 43 },
    { 0x5690a4f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0a4f7, 90 },
    { 0x6420a4f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0a4fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c30a4ff, 99 },
    { 0xdb70a500, 98 },
    { 0, 0 },
    { 0x8740a502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0a506, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b0a50b, 98 },
    { 0x63e0a50b, 90 },
    { 0x550a50d, 23 },
    { 0x2100a50d, 43 },
    { 0xc340a50e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf0a515, 33 },
    { 0x5b00a516, 95 },
    { 0x2490a516, 98 },
    { 0x5820a518, 90 },
    { 0xeba0a519, 90 },
    { 0xb3d0a519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5120a521, 90 },
    { 0, 0 },
    { 0xa2f0a523, 95 },
    { 0x35e0a524, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb240a52b, 23 },
    { 0, 0 },
    { 0x40b0a52d, 66 },
    { 0x1090a52e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd0a531, 92 },
    { 0, 0 },
    { 0xa070a533, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b0a538, 23 },
    { 0xa5f0a538, 24 },
    { 0, 0 },
    { 0x24f0a53b, 43 },
    { 0x2d40a53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a00a540, 95 },
    { 0, 0 },
    { 0x2a70a542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb700a545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab0a54c, 33 },
    { 0x3a30a54d, 99 },
    { 0xa660a54e, 23 },
    { 0xb250a54f, 100 },
    { 0, 0 },
    { 0x1920a551, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd00a555, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb290a559, 33 },
    { 0, 0 },
    { 0x8ba0a55b, 76 },
    { 0xc0e0a55b, 97 },
    { 0xce50a55d, 87 },
    { 0xf3b0a55d, 61 },
    { 0x7b50a55f, 17 },
    { 0xab10a560, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9450a565, 66 },
    { 0x61d0a566, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c0a56a, 78 },
    { 0xadc0a56b, 33 },
    { 0, 0 },
    { 0x8b30a56d, 93 },
    { 0x8be0a56e, 76 },
    { 0x3e30a56d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa560a576, 76 },
    { 0x3080a577, 46 },
    { 0xb770a577, 52 },
    { 0, 0 },
    { 0x75e0a57a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0a57e, 46 },
    { 0x7570a57f, 95 },
    { 0x34a0a580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40a584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa070a587, 95 },
    { 0x79a0a588, 76 },
    { 0x40c0a588, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1740a58e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b0a592, 100 },
    { 0, 0 },
    { 0xc210a594, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2930a59a, 33 },
    { 0x8880a59b, 91 },
    { 0, 0 },
    { 0x56d0a59d, 66 },
    { 0x5850a59e, 90 },
    { 0xcd0a59e, 33 },
    { 0x9440a59f, 43 },
    { 0xae50a5a0, 35 },
    { 0xf410a5a0, 76 },
    { 0xa700a5a3, 98 },
    { 0x1b00a5a4, 33 },
    { 0, 0 },
    { 0x510a5a6, 34 },
    { 0xfe10a5a7, 33 },
    { 0x5a60a5a7, 61 },
    { 0x3480a5a9, 23 },
    { 0xf4f0a5aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f0a5b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b0a5b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca0a5bc, 98 },
    { 0x2620a5bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d0a5c0, 40 },
    { 0xc1f0a5c1, 90 },
    { 0x4450a5c0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e0a5c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f80a5ca, 46 },
    { 0, 0 },
    { 0x1d60a5cc, 88 },
    { 0xdee0a5cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc700a5d2, 55 },
    { 0x740a5d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250a5d8, 77 },
    { 0x9150a5d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e0a5df, 95 },
    { 0, 0 },
    { 0xa770a5e1, 76 },
    { 0xda0a5e2, 23 },
    { 0x7280a5e3, 43 },
    { 0, 0 },
    { 0xa630a5e5, 78 },
    { 0x4100a5e6, 78 },
    { 0, 0 },
    { 0xdd70a5e8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e0a5ee, 61 },
    { 0xdff0a5ee, 52 },
    { 0x3e60a5f0, 95 },
    { 0, 0 },
    { 0x3a40a5f2, 76 },
    { 0xe1a0a5f2, 95 },
    { 0xe210a5f2, 61 },
    { 0, 0 },
    { 0x9db0a5f6, 40 },
    { 0, 0 },
    { 0x6f20a5f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa00a603, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac0a60a, 78 },
    { 0xb510a60b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd80a60e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca80a612, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650a616, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530a61a, 76 },
    { 0xb30a61b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c50a627, 66 },
    { 0x8250a628, 23 },
    { 0x2ad0a629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d00a62f, 46 },
    { 0xa150a630, 43 },
    { 0xec10a630, 95 },
    { 0, 0 },
    { 0xf70a633, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d20a636, 95 },
    { 0, 0 },
    { 0x46d0a638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d0a63b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5090a643, 17 },
    { 0x4670a643, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a0a64b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f0a650, 95 },
    { 0, 0 },
    { 0x3060a652, 33 },
    { 0x7c50a653, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a50a662, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc0a666, 36 },
    { 0x8420a666, 93 },
    { 0x5300a668, 23 },
    { 0x5230a669, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff0a66c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030a673, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920a678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e0a67b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0a67e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa0a681, 90 },
    { 0xadd0a682, 91 },
    { 0x3700a682, 55 },
    { 0, 0 },
    { 0xd0d0a685, 43 },
    { 0x94c0a686, 33 },
    { 0x5840a686, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120a68a, 67 },
    { 0, 0 },
    { 0xcde0a68c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b70a68f, 76 },
    { 0, 0 },
    { 0x4e00a691, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c0a695, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c0a699, 23 },
    { 0x3960a69a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a0a69f, 61 },
    { 0, 0 },
    { 0x61c0a6a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d40a6a4, 87 },
    { 0x5380a6a5, 78 },
    { 0, 0 },
    { 0x1980a6a7, 76 },
    { 0x3b90a6a7, 98 },
    { 0x4330a6a9, 67 },
    { 0x2b70a6aa, 23 },
    { 0x2eb0a6ab, 61 },
    { 0, 0 },
    { 0xfc40a6ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c0a6b3, 43 },
    { 0xa8c0a6b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c0a6bb, 61 },
    { 0x9510a6bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0a6bf, 61 },
    { 0x5dc0a6c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00a6c6, 90 },
    { 0x2c80a6c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00a6cb, 23 },
    { 0xe1d0a6cc, 75 },
    { 0x88a0a6cd, 61 },
    { 0xf490a6ce, 95 },
    { 0xff20a6cf, 55 },
    { 0x5530a6d0, 34 },
    { 0x5610a6d0, 90 },
    { 0x8880a6d2, 77 },
    { 0xa990a6d0, 59 },
    { 0xc5f0a6d2, 95 },
    { 0x7830a6d4, 65 },
    { 0x8100a6d6, 23 },
    { 0x4c60a6d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde80a6db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710a6df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6010a6e6, 17 },
    { 0xd890a6e6, 98 },
    { 0xbec0a6e7, 78 },
    { 0, 0 },
    { 0x6120a6ea, 91 },
    { 0, 0 },
    { 0xdbf0a6ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda40a6f0, 27 },
    { 0x6130a6f1, 33 },
    { 0xde60a6f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa510a6f7, 85 },
    { 0x9500a6f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f0a6fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690a700, 61 },
    { 0xe3c0a701, 61 },
    { 0xd260a702, 90 },
    { 0, 0 },
    { 0x2700a704, 85 },
    { 0, 0 },
    { 0x1250a706, 33 },
    { 0x8480a707, 23 },
    { 0x4a10a708, 66 },
    { 0, 0 },
    { 0x6330a70a, 33 },
    { 0, 0 },
    { 0xa440a70c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc850a711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf0a714, 33 },
    { 0x95d0a715, 55 },
    { 0, 0 },
    { 0x8200a717, 95 },
    { 0, 0 },
    { 0x4690a719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d0a71c, 91 },
    { 0x9e90a71d, 90 },
    { 0xba00a71e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb620a721, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6160a72f, 85 },
    { 0x3a00a730, 100 },
    { 0x6250a731, 76 },
    { 0, 0 },
    { 0xe4a0a733, 34 },
    { 0xd0c0a734, 90 },
    { 0, 0 },
    { 0x8680a736, 98 },
    { 0x6d40a737, 55 },
    { 0, 0 },
    { 0xdb40a739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd860a73c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe690a740, 23 },
    { 0, 0 },
    { 0xf3b0a742, 77 },
    { 0x970a743, 90 },
    { 0x6740a742, 33 },
    { 0x8ad0a745, 46 },
    { 0, 0 },
    { 0x1b90a747, 95 },
    { 0, 0 },
    { 0xe850a749, 76 },
    { 0xd0d0a74a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f0a74d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0a755, 95 },
    { 0x30b0a756, 55 },
    { 0, 0 },
    { 0x3b90a758, 98 },
    { 0, 0 },
    { 0x6640a75a, 100 },
    { 0x8390a75b, 23 },
    { 0xe1e0a75c, 90 },
    { 0xea30a75d, 89 },
    { 0xe8f0a75e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc970a762, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a0a766, 95 },
    { 0x7f0a767, 98 },
    { 0x83e0a768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970a76c, 95 },
    { 0x9280a76d, 61 },
    { 0xef0a76d, 61 },
    { 0x16e0a76c, 95 },
    { 0xa240a770, 76 },
    { 0x59d0a76e, 43 },
    { 0, 0 },
    { 0xb740a773, 55 },
    { 0xdef0a774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6970a778, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1190a77c, 55 },
    { 0xf030a77d, 76 },
    { 0x93c0a77d, 85 },
    { 0x480a77f, 23 },
    { 0x8790a780, 98 },
    { 0xf940a781, 95 },
    { 0xb7a0a782, 90 },
    { 0, 0 },
    { 0x9820a784, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e40a794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220a79a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d0a79f, 33 },
    { 0x7340a79f, 66 },
    { 0xce80a7a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d50a7a5, 33 },
    { 0, 0 },
    { 0x3f60a7a7, 93 },
    { 0x51a0a7a7, 93 },
    { 0xcf00a7a9, 95 },
    { 0, 0 },
    { 0xdc90a7ab, 61 },
    { 0, 0 },
    { 0xae00a7ad, 23 },
    { 0xf100a7ad, 23 },
    { 0, 0 },
    { 0xc410a7b0, 99 },
    { 0x68b0a7b1, 97 },
    { 0x8e00a7b2, 68 },
    { 0x6560a7b1, 33 },
    { 0xa490a7b3, 23 },
    { 0, 0 },
    { 0x2060a7b6, 33 },
    { 0, 0 },
    { 0x2310a7b8, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd0a7bc, 99 },
    { 0x4720a7bd, 90 },
    { 0xaa30a7bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2510a7cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c0a7d1, 91 },
    { 0xd0a0a7d2, 55 },
    { 0x3650a7d1, 67 },
    { 0xb170a7d4, 43 },
    { 0, 0 },
    { 0x3350a7d6, 34 },
    { 0xf0e0a7d7, 87 },
    { 0x70a7d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf0a7df, 93 },
    { 0x7070a7df, 55 },
    { 0, 0 },
    { 0x7a50a7e2, 66 },
    { 0x530a7e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb210a7e6, 46 },
    { 0x4d00a7e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e0a7ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0a7f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2130a7f5, 98 },
    { 0x1370a7f6, 23 },
    { 0x9530a7f7, 33 },
    { 0xb890a7f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb50a7fc, 98 },
    { 0, 0 },
    { 0xb5c0a7fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b0a803, 43 },
    { 0, 0 },
    { 0x7c70a805, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb40a80a, 34 },
    { 0x13c0a80b, 76 },
    { 0, 0 },
    { 0xdfb0a80d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3720a811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd0a818, 83 },
    { 0x1b80a819, 87 },
    { 0xb4c0a819, 23 },
    { 0x67d0a81b, 98 },
    { 0x8940a81a, 33 },
    { 0xe4d0a81d, 90 },
    { 0x46a0a81d, 23 },
    { 0, 0 },
    { 0x4f00a820, 43 },
    { 0x4f10a821, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40a824, 23 },
    { 0x6060a825, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900a829, 55 },
    { 0, 0 },
    { 0xfb80a82b, 51 },
    { 0x5090a82b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4530a831, 95 },
    { 0xf650a832, 66 },
    { 0xb790a831, 61 },
    { 0x5370a834, 55 },
    { 0, 0 },
    { 0x6c0a836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc0a83c, 44 },
    { 0x6ad0a83d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b0a846, 43 },
    { 0x3040a846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c80a84b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8900a84f, 33 },
    { 0, 0 },
    { 0xd350a851, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7300a854, 76 },
    { 0x9510a854, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b80a858, 33 },
    { 0xb760a859, 90 },
    { 0xd640a859, 90 },
    { 0xb9b0a85b, 81 },
    { 0xebb0a859, 95 },
    { 0xf340a85d, 61 },
    { 0, 0 },
    { 0xb340a85f, 85 },
    { 0x3890a860, 23 },
    { 0xeca0a860, 90 },
    { 0x9a0a862, 33 },
    { 0xaf70a85f, 33 },
    { 0x71a0a861, 98 },
    { 0x5be0a865, 62 },
    { 0xe4d0a866, 33 },
    { 0x3810a865, 87 },
    { 0x5310a868, 95 },
    { 0xc6c0a869, 98 },
    { 0x1680a86a, 99 },
    { 0xa290a86a, 61 },
    { 0x8090a86c, 66 },
    { 0x9180a86d, 91 },
    { 0xaa50a86b, 61 },
    { 0x8e60a861, 100 },
    { 0xeeb0a870, 90 },
    { 0xd380a871, 23 },
    { 0xbfc0a870, 36 },
    { 0x53f0a873, 90 },
    { 0, 0 },
    { 0xaa60a875, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff0a878, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0a87b, 98 },
    { 0, 0 },
    { 0xe860a87d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7240a881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd40a884, 23 },
    { 0, 0 },
    { 0xc640a886, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4060a889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60a88c, 89 },
    { 0x1020a88d, 61 },
    { 0x5910a88e, 33 },
    { 0, 0 },
    { 0xe0c0a890, 61 },
    { 0x1d90a891, 95 },
    { 0x6290a891, 23 },
    { 0x360a893, 53 },
    { 0x3920a894, 56 },
    { 0x63f0a895, 95 },
    { 0xd1e0a891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a0a899, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da0a89c, 95 },
    { 0xf770a89d, 34 },
    { 0x7c50a89e, 34 },
    { 0x8650a89f, 93 },
    { 0xe580a8a0, 98 },
    { 0xe5f0a8a1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4840a8a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd0a8a9, 32 },
    { 0x4fe0a8aa, 33 },
    { 0, 0 },
    { 0x4a0a8ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1190a8b4, 43 },
    { 0xbfe0a8b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1980a8b8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b0a8c6, 91 },
    { 0x6190a8c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0a8cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0a8ce, 91 },
    { 0x5250a8ce, 98 },
    { 0xa100a8d0, 80 },
    { 0x620a8d1, 43 },
    { 0x40f0a8cf, 23 },
    { 0, 0 },
    { 0x8f40a8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3450a8d9, 98 },
    { 0, 0 },
    { 0x3750a8db, 55 },
    { 0, 0 },
    { 0xfb20a8dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60a8e0, 99 },
    { 0x1ca0a8e1, 80 },
    { 0x5780a8e1, 95 },
    { 0xf90a8e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b0a8e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e70a8e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d0a8ec, 45 },
    { 0x1720a8ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe540a8f0, 43 },
    { 0xb440a8f1, 95 },
    { 0x5590a8f2, 90 },
    { 0xf0c0a8f3, 33 },
    { 0x50a8f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc520a8f7, 90 },
    { 0x3840a8f8, 80 },
    { 0xcdc0a8f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6400a8fd, 95 },
    { 0xb0c0a8fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a901, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd20a904, 88 },
    { 0x34a0a905, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c0a90f, 75 },
    { 0x8f20a90f, 93 },
    { 0xdff0a911, 90 },
    { 0xfe90a911, 98 },
    { 0xe5a0a90f, 76 },
    { 0x7f50a911, 67 },
    { 0, 0 },
    { 0xb990a916, 87 },
    { 0x5950a917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a90a91a, 87 },
    { 0, 0 },
    { 0xd9f0a91c, 52 },
    { 0, 0 },
    { 0xacd0a91e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e70a92f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c0a933, 93 },
    { 0xa2c0a934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc0a938, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e70a93e, 17 },
    { 0x8860a93f, 98 },
    { 0xdd80a93f, 76 },
    { 0xfee0a941, 61 },
    { 0x22d0a941, 75 },
    { 0xf690a943, 69 },
    { 0xac0a944, 33 },
    { 0xa0b0a940, 55 },
    { 0xf990a93e, 88 },
    { 0, 0 },
    { 0x6290a948, 46 },
    { 0xbd80a949, 90 },
    { 0xb3c0a948, 23 },
    { 0, 0 },
    { 0xa2b0a94c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e0a94f, 87 },
    { 0xb0e0a950, 98 },
    { 0x1220a94f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3920a954, 98 },
    { 0xab40a955, 98 },
    { 0, 0 },
    { 0x3d50a957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b0a964, 33 },
    { 0, 0 },
    { 0x4ef0a966, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac20a96d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f0a970, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3370a973, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f0a978, 34 },
    { 0, 0 },
    { 0x7aa0a97a, 33 },
    { 0x9f70a97b, 61 },
    { 0xe870a97c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac00a983, 23 },
    { 0, 0 },
    { 0x46a0a985, 33 },
    { 0x6b10a986, 86 },
    { 0x3c10a985, 78 },
    { 0x8430a988, 66 },
    { 0xb410a985, 98 },
    { 0x4b50a985, 23 },
    { 0, 0 },
    { 0xd690a98c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f60a993, 75 },
    { 0x1fe0a994, 95 },
    { 0x5dd0a995, 55 },
    { 0, 0 },
    { 0xd3d0a997, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1790a99b, 43 },
    { 0, 0 },
    { 0xb960a99d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f0a9a2, 61 },
    { 0x12f0a9a3, 98 },
    { 0x5c50a9a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b0a9a9, 81 },
    { 0x3100a9a9, 90 },
    { 0xf0a9aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0a9b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4460a9ba, 76 },
    { 0x1910a9bb, 61 },
    { 0x2510a9bc, 33 },
    { 0x54e0a9bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe680a9c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e0a9c5, 76 },
    { 0, 0 },
    { 0xf5f0a9c7, 77 },
    { 0x48a0a9c8, 33 },
    { 0x2530a9c9, 43 },
    { 0xdd90a9c8, 90 },
    { 0, 0 },
    { 0xc850a9cc, 61 },
    { 0, 0 },
    { 0x47f0a9ce, 64 },
    { 0, 0 },
    { 0xb980a9d0, 34 },
    { 0, 0 },
    { 0xd70a9d2, 55 },
    { 0, 0 },
    { 0x8260a9d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7730a9d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0a9db, 69 },
    { 0x22f0a9dc, 46 },
    { 0, 0 },
    { 0x6a60a9de, 55 },
    { 0x6930a9de, 78 },
    { 0, 0 },
    { 0x6d90a9e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe810a9e4, 33 },
    { 0xab80a9e5, 95 },
    { 0, 0 },
    { 0xf0c0a9e7, 90 },
    { 0, 0 },
    { 0xcd40a9e9, 61 },
    { 0, 0 },
    { 0x7a40a9eb, 85 },
    { 0xaf00a9ec, 85 },
    { 0xd070a9ed, 93 },
    { 0x78a0a9ee, 55 },
    { 0x7d0a9ee, 61 },
    { 0xf380a9ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a40a9fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0aa05, 33 },
    { 0, 0 },
    { 0x500aa07, 33 },
    { 0, 0 },
    { 0xd4d0aa09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe040aa0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae10aa12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c0aa15, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f0aa19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c60aa1c, 98 },
    { 0, 0 },
    { 0x4850aa1e, 99 },
    { 0x6330aa1f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680aa24, 76 },
    { 0xb870aa24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2820aa2c, 34 },
    { 0xe9f0aa2d, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0aa31, 23 },
    { 0x1dc0aa31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1420aa37, 33 },
    { 0, 0 },
    { 0xe6b0aa39, 95 },
    { 0x8580aa3a, 88 },
    { 0, 0 },
    { 0x1cc0aa3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0aa41, 61 },
    { 0, 0 },
    { 0x5d00aa43, 98 },
    { 0, 0 },
    { 0x7a30aa45, 78 },
    { 0x2ec0aa45, 84 },
    { 0x2900aa47, 93 },
    { 0xc580aa48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa080aa4d, 17 },
    { 0, 0 },
    { 0x3a0aa4f, 95 },
    { 0xed40aa4f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c0aa56, 61 },
    { 0, 0 },
    { 0xd800aa58, 84 },
    { 0xd0f0aa58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed70aa5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8380aa61, 95 },
    { 0, 0 },
    { 0x7fc0aa63, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410aa66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7610aa6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc630aa70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd90aa74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9390aa77, 98 },
    { 0, 0 },
    { 0x9080aa79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a70aa87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b70aa8c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f0aa96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e0aa9a, 90 },
    { 0x5590aa9b, 61 },
    { 0xbd30aa9c, 93 },
    { 0x9ec0aa9d, 53 },
    { 0xc190aa9d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a0aaa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3880aaa8, 84 },
    { 0x1490aaa9, 91 },
    { 0x6300aaaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c0aaaf, 78 },
    { 0, 0 },
    { 0xca70aab1, 33 },
    { 0xf750aab2, 67 },
    { 0xb1f0aab3, 38 },
    { 0x3880aab3, 61 },
    { 0, 0 },
    { 0x77f0aab6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7760aabb, 43 },
    { 0x3050aabb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xce20aabf, 75 },
    { 0xd50aabf, 55 },
    { 0, 0 },
    { 0xd6e0aac2, 38 },
    { 0, 0 },
    { 0x3690aac4, 75 },
    { 0x1470aac5, 34 },
    { 0x3e00aac6, 23 },
    { 0x80aac6, 90 },
    { 0x97c0aac4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e50aacf, 52 },
    { 0x5f30aacf, 76 },
    { 0x1790aad1, 23 },
    { 0x2360aad1, 55 },
    { 0, 0 },
    { 0x30c0aad4, 95 },
    { 0x8b60aad5, 98 },
    { 0x9340aad6, 33 },
    { 0x7bd0aad6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0aada, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f80aadd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd0aae1, 23 },
    { 0x6160aae2, 78 },
    { 0xa500aae3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc550aae7, 55 },
    { 0x3270aae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d0aaee, 51 },
    { 0xfdf0aaef, 61 },
    { 0x6fd0aaef, 76 },
    { 0xa100aaee, 52 },
    { 0xc5e0aaee, 90 },
    { 0x3cd0aaf3, 66 },
    { 0x4000aaf4, 76 },
    { 0x5080aaf3, 93 },
    { 0x8050aaf3, 98 },
    { 0xd4a0aaf7, 78 },
    { 0xad60aaf7, 90 },
    { 0x8fa0aaf9, 77 },
    { 0x8780aaf9, 91 },
    { 0xc040aaf7, 38 },
    { 0x4b70aafc, 33 },
    { 0xb400aafd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5110ab03, 95 },
    { 0x8f80ab03, 34 },
    { 0xf5a0ab04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf60ab0b, 43 },
    { 0x370ab0c, 98 },
    { 0x7dd0ab0c, 65 },
    { 0x4950ab0e, 76 },
    { 0xbe20ab0e, 61 },
    { 0x3b0ab0f, 40 },
    { 0, 0 },
    { 0xa850ab12, 61 },
    { 0xa950ab13, 95 },
    { 0xf9e0ab13, 33 },
    { 0, 0 },
    { 0x9810ab16, 46 },
    { 0xbec0ab16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9980ab1a, 66 },
    { 0, 0 },
    { 0x7860ab1c, 33 },
    { 0xea60ab1c, 52 },
    { 0xa5c0ab1e, 33 },
    { 0, 0 },
    { 0x8c20ab20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2510ab25, 98 },
    { 0xc9d0ab25, 46 },
    { 0xf360ab26, 46 },
    { 0, 0 },
    { 0x1ff0ab29, 95 },
    { 0xf7a0ab2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2880ab31, 90 },
    { 0x9b10ab31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x820ab35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa0ab38, 84 },
    { 0x72d0ab38, 61 },
    { 0xfca0ab3a, 90 },
    { 0x2990ab3a, 90 },
    { 0x52c0ab3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae30ab3f, 23 },
    { 0xed50ab40, 93 },
    { 0x3110ab40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd70ab44, 78 },
    { 0x64b0ab44, 90 },
    { 0xf130ab45, 99 },
    { 0, 0 },
    { 0xaa70ab48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa90ab4e, 90 },
    { 0x1f20ab4e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c0ab5b, 34 },
    { 0xd6b0ab5c, 95 },
    { 0, 0 },
    { 0x12c0ab5e, 87 },
    { 0, 0 },
    { 0x4f60ab60, 98 },
    { 0x4df0ab60, 34 },
    { 0, 0 },
    { 0xa450ab63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8950ab6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c40ab71, 55 },
    { 0xe9c0ab71, 44 },
    { 0, 0 },
    { 0x7510ab74, 61 },
    { 0xc90ab75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180ab79, 55 },
    { 0xf780ab7a, 95 },
    { 0x44c0ab7b, 33 },
    { 0x2750ab7a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00ab7f, 23 },
    { 0, 0 },
    { 0x7ff0ab81, 88 },
    { 0x19d0ab81, 98 },
    { 0xb1b0ab83, 95 },
    { 0xf3a0ab83, 43 },
    { 0x5910ab85, 33 },
    { 0x8010ab82, 100 },
    { 0x8750ab84, 61 },
    { 0x9ff0ab88, 78 },
    { 0xd780ab81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf0ab8d, 100 },
    { 0xc650ab8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc10ab91, 90 },
    { 0, 0 },
    { 0x3c90ab93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a0ab98, 98 },
    { 0x11e0ab99, 90 },
    { 0, 0 },
    { 0xcad0ab9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c0ab9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7480aba1, 90 },
    { 0x5150aba2, 55 },
    { 0xede0aba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930abb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc330abb6, 61 },
    { 0x27c0abb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490abbc, 93 },
    { 0x33c0abbd, 98 },
    { 0x4be0abbe, 43 },
    { 0x4620abbe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8140abc4, 43 },
    { 0xe540abc5, 33 },
    { 0xaad0abc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e0abc9, 33 },
    { 0xbca0abca, 95 },
    { 0x37f0abcb, 33 },
    { 0xc0c0abcc, 98 },
    { 0x14e0abcd, 86 },
    { 0x3320abcd, 99 },
    { 0, 0 },
    { 0xddf0abd0, 95 },
    { 0x62c0abd1, 78 },
    { 0xd610abd1, 68 },
    { 0, 0 },
    { 0xdb60abd4, 95 },
    { 0, 0 },
    { 0x8c90abd6, 33 },
    { 0xa490abd6, 93 },
    { 0x30d0abd7, 61 },
    { 0, 0 },
    { 0x8410abda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b0abdf, 90 },
    { 0, 0 },
    { 0x6a10abe1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b0abe6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa0abea, 95 },
    { 0x1c30abea, 43 },
    { 0xd2f0abec, 66 },
    { 0x7a20abec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa350abf2, 93 },
    { 0xf2d0abf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c0abf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2640abfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c30ac00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ac04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b00ac09, 48 },
    { 0xf640ac0a, 98 },
    { 0xfcd0ac09, 95 },
    { 0x2d90ac0c, 77 },
    { 0xe9a0ac0d, 87 },
    { 0x4e40ac0e, 90 },
    { 0x8b50ac0f, 93 },
    { 0x1110ac0e, 84 },
    { 0, 0 },
    { 0x49c0ac12, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa040ac15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60ac1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe360ac1d, 46 },
    { 0x8e0ac1e, 55 },
    { 0x6a00ac1f, 23 },
    { 0, 0 },
    { 0x860ac21, 65 },
    { 0x3d50ac22, 61 },
    { 0xd350ac23, 46 },
    { 0x40d0ac24, 98 },
    { 0x5c60ac23, 23 },
    { 0x31e0ac24, 33 },
    { 0x1e80ac22, 94 },
    { 0x1760ac28, 98 },
    { 0x8010ac27, 95 },
    { 0xd760ac2a, 33 },
    { 0xd770ac2a, 95 },
    { 0x59f0ac2b, 67 },
    { 0x2830ac23, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c0ac33, 99 },
    { 0x94e0ac34, 88 },
    { 0x26f0ac34, 61 },
    { 0, 0 },
    { 0x5ba0ac37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d50ac3b, 76 },
    { 0, 0 },
    { 0x86d0ac3d, 46 },
    { 0x8a60ac3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b40ac45, 98 },
    { 0, 0 },
    { 0x3230ac47, 33 },
    { 0x3b60ac48, 76 },
    { 0, 0 },
    { 0x4b0ac4a, 99 },
    { 0x6950ac4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20ac4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb30ac51, 98 },
    { 0x3190ac52, 90 },
    { 0, 0 },
    { 0x8370ac54, 76 },
    { 0, 0 },
    { 0xe730ac56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab0ac61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd0ac6a, 84 },
    { 0xc880ac6b, 95 },
    { 0, 0 },
    { 0x2f10ac6d, 90 },
    { 0, 0 },
    { 0x2320ac6f, 67 },
    { 0x2130ac70, 99 },
    { 0xe5e0ac71, 95 },
    { 0xd520ac71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6550ac75, 93 },
    { 0xd970ac76, 95 },
    { 0x7c70ac76, 65 },
    { 0xef40ac78, 90 },
    { 0x5bd0ac79, 76 },
    { 0x4dd0ac7a, 33 },
    { 0x60b0ac7b, 23 },
    { 0x5220ac7c, 92 },
    { 0x2d50ac7d, 52 },
    { 0x7f40ac7d, 96 },
    { 0, 0 },
    { 0xee30ac80, 55 },
    { 0xefb0ac80, 67 },
    { 0, 0 },
    { 0x4330ac83, 98 },
    { 0, 0 },
    { 0xa080ac85, 61 },
    { 0x5470ac85, 46 },
    { 0, 0 },
    { 0x31b0ac88, 61 },
    { 0xd690ac88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff50ac8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c30ac95, 76 },
    { 0xcfc0ac96, 95 },
    { 0xe710ac96, 66 },
    { 0xd7e0ac98, 33 },
    { 0xbbf0ac99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0ac9d, 23 },
    { 0x8700ac9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee0aca2, 34 },
    { 0xfff0aca2, 19 },
    { 0x3e0aca4, 75 },
    { 0x1f60aca5, 34 },
    { 0x52b0aca4, 95 },
    { 0x85b0aca7, 46 },
    { 0, 0 },
    { 0xf220aca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc60acad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d0acb5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x830acb8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0acbb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8590acbf, 61 },
    { 0, 0 },
    { 0xf0acc1, 46 },
    { 0x2730acc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de0acc5, 94 },
    { 0xd8b0acc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50accd, 93 },
    { 0x7770acce, 25 },
    { 0xc720accd, 90 },
    { 0xf6c0accd, 36 },
    { 0xa8a0acd1, 91 },
    { 0, 0 },
    { 0x5930acd3, 55 },
    { 0, 0 },
    { 0xaa20acd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00ace0, 95 },
    { 0xe810ace0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7870ace9, 61 },
    { 0xea00acea, 33 },
    { 0x3e70aceb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de0acf0, 95 },
    { 0xd010acf0, 91 },
    { 0xc590acf0, 46 },
    { 0x57f0acf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb30acf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490acfe, 55 },
    { 0xde10acfe, 98 },
    { 0x3860ad00, 67 },
    { 0, 0 },
    { 0xa7b0ad02, 52 },
    { 0x1290ad03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7960ad07, 67 },
    { 0x6f0ad08, 90 },
    { 0xbda0ad09, 23 },
    { 0x4460ad08, 90 },
    { 0, 0 },
    { 0xba80ad0c, 95 },
    { 0xb250ad0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f0ad10, 43 },
    { 0, 0 },
    { 0x8020ad12, 98 },
    { 0x6790ad13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490ad16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0ad1c, 43 },
    { 0, 0 },
    { 0xfd60ad1e, 61 },
    { 0x1100ad1f, 98 },
    { 0, 0 },
    { 0x3cd0ad21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e80ad26, 95 },
    { 0, 0 },
    { 0xae40ad28, 90 },
    { 0xe230ad29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea0ad2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30ad30, 23 },
    { 0xc520ad30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ad35, 33 },
    { 0x60a0ad36, 67 },
    { 0x27a0ad37, 66 },
    { 0x6d10ad38, 61 },
    { 0, 0 },
    { 0xec0ad3a, 23 },
    { 0x4350ad3a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd950ad42, 85 },
    { 0, 0 },
    { 0xfd00ad44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a90ad49, 23 },
    { 0x2f70ad4a, 95 },
    { 0x9e20ad4b, 61 },
    { 0, 0 },
    { 0x3ea0ad4d, 95 },
    { 0x1c20ad4e, 95 },
    { 0x6ed0ad4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5330ad52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ad55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6470ad5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba70ad60, 76 },
    { 0xfbd0ad61, 90 },
    { 0x8c80ad62, 17 },
    { 0x3ea0ad63, 90 },
    { 0x92e0ad64, 98 },
    { 0x3b80ad65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d0ad68, 67 },
    { 0x3260ad68, 90 },
    { 0xb6c0ad68, 66 },
    { 0x53e0ad68, 99 },
    { 0, 0 },
    { 0x91a0ad6d, 98 },
    { 0, 0 },
    { 0x7b30ad6f, 87 },
    { 0xfdb0ad6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710ad79, 95 },
    { 0x2b50ad79, 78 },
    { 0x8470ad7b, 76 },
    { 0xf020ad7a, 34 },
    { 0x1130ad7d, 93 },
    { 0x85b0ad7a, 33 },
    { 0xc8d0ad7f, 91 },
    { 0, 0 },
    { 0x7d50ad81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c0ad88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6370ad8d, 87 },
    { 0x72a0ad8e, 76 },
    { 0x83e0ad8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f60ad99, 98 },
    { 0xbfe0ad99, 61 },
    { 0x62b0ad99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1650ad9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce90ada2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220ada7, 67 },
    { 0xe7e0ada8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa140adab, 99 },
    { 0x8fc0adac, 23 },
    { 0xcc00adad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60adb0, 81 },
    { 0x53a0adb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c0adb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa770adb9, 99 },
    { 0, 0 },
    { 0xeb90adbb, 33 },
    { 0xe090adbc, 98 },
    { 0x320adbb, 61 },
    { 0x2600adbe, 99 },
    { 0x2b60adbe, 90 },
    { 0x8bd0adc0, 66 },
    { 0xf8f0adbe, 33 },
    { 0, 0 },
    { 0x9290adc3, 90 },
    { 0x7f20adc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1650adc7, 90 },
    { 0, 0 },
    { 0x9bf0adc9, 88 },
    { 0x4510adca, 91 },
    { 0xa600adca, 33 },
    { 0x6990adcc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c80adcf, 98 },
    { 0xed0add0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2100add5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f0addb, 85 },
    { 0, 0 },
    { 0xf530addd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9300ade3, 100 },
    { 0x3710ade4, 81 },
    { 0, 0 },
    { 0xe8c0ade6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe150ade9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b90adee, 23 },
    { 0xfc90adef, 46 },
    { 0x6c0adf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae0adf7, 78 },
    { 0x8b20adf7, 90 },
    { 0xf120adf9, 33 },
    { 0x6850adfa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee50ae01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f0ae05, 23 },
    { 0x2680ae06, 33 },
    { 0, 0 },
    { 0x17e0ae08, 90 },
    { 0xe010ae09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef0ae0f, 68 },
    { 0, 0 },
    { 0xaee0ae11, 76 },
    { 0, 0 },
    { 0x6680ae13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a10ae17, 87 },
    { 0xc560ae18, 33 },
    { 0xc810ae18, 75 },
    { 0, 0 },
    { 0x2a10ae1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20ae20, 100 },
    { 0, 0 },
    { 0x5c00ae22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc0ae25, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc110ae28, 52 },
    { 0xa3c0ae28, 76 },
    { 0x8b0ae2a, 23 },
    { 0, 0 },
    { 0x55b0ae2c, 55 },
    { 0x2640ae2d, 76 },
    { 0xa6d0ae2e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6130ae33, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc350ae39, 61 },
    { 0, 0 },
    { 0x46f0ae3b, 33 },
    { 0x63c0ae3c, 98 },
    { 0x30f0ae3d, 78 },
    { 0x70d0ae3c, 91 },
    { 0x90ae3c, 23 },
    { 0xce60ae40, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc130ae45, 93 },
    { 0xd7c0ae46, 33 },
    { 0x6070ae47, 93 },
    { 0, 0 },
    { 0x7190ae49, 43 },
    { 0x14c0ae4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe840ae52, 33 },
    { 0, 0 },
    { 0x1fb0ae54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce0ae57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ae5a, 91 },
    { 0, 0 },
    { 0x1680ae5c, 98 },
    { 0x1030ae5d, 55 },
    { 0xcdc0ae5e, 99 },
    { 0xd430ae5e, 23 },
    { 0, 0 },
    { 0xcff0ae61, 76 },
    { 0x27f0ae61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e50ae66, 55 },
    { 0x9400ae66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e40ae6a, 90 },
    { 0x4030ae6b, 34 },
    { 0xcbd0ae6a, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a0ae71, 95 },
    { 0x6b70ae72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa0ae7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d80ae81, 76 },
    { 0x6f70ae82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1780ae87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6430ae8c, 67 },
    { 0, 0 },
    { 0xc460ae8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc400ae92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790ae95, 98 },
    { 0x7d50ae95, 43 },
    { 0x37f0ae96, 43 },
    { 0, 0 },
    { 0x1d10ae99, 61 },
    { 0x4940ae99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc0aea3, 61 },
    { 0xb530aea4, 33 },
    { 0, 0 },
    { 0x5ef0aea6, 78 },
    { 0x660aea7, 90 },
    { 0x39b0aea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10aeb1, 61 },
    { 0, 0 },
    { 0xd810aeb3, 33 },
    { 0x33c0aeb4, 76 },
    { 0x2130aeb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d0aeb8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1180aebf, 55 },
    { 0, 0 },
    { 0x9220aec1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf80aec7, 61 },
    { 0x68a0aec7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c60aecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd60aed2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2230aed9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f90aedc, 52 },
    { 0xea70aedd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2770aee0, 76 },
    { 0x9110aee0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6280aee7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd0aeef, 17 },
    { 0xaee0aeef, 85 },
    { 0, 0 },
    { 0xbaa0aef2, 81 },
    { 0x2850aef3, 78 },
    { 0xb6a0aef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4020aef9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d0aefc, 90 },
    { 0, 0 },
    { 0x7dc0aefe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a0af0b, 91 },
    { 0x360af0c, 55 },
    { 0, 0 },
    { 0xb0c0af0e, 23 },
    { 0x8de0af0e, 66 },
    { 0xaeb0af10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4540af14, 99 },
    { 0, 0 },
    { 0xf680af16, 55 },
    { 0, 0 },
    { 0x5fa0af18, 76 },
    { 0, 0 },
    { 0xc3f0af1a, 68 },
    { 0, 0 },
    { 0xf3c0af1c, 34 },
    { 0xb580af1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd0af22, 33 },
    { 0, 0 },
    { 0x8980af24, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd0af2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb550af2f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2610af32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920af40, 66 },
    { 0xb980af40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8820af45, 43 },
    { 0xbc40af46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e0af49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf180af4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf70af53, 23 },
    { 0xb830af53, 98 },
    { 0x8820af54, 64 },
    { 0x71c0af56, 34 },
    { 0, 0 },
    { 0x8250af58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd0af5b, 90 },
    { 0, 0 },
    { 0x8370af5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe0af62, 61 },
    { 0, 0 },
    { 0x7a0af64, 33 },
    { 0x5040af65, 23 },
    { 0x6530af65, 61 },
    { 0x9fa0af67, 55 },
    { 0xb430af68, 43 },
    { 0x4eb0af69, 23 },
    { 0xff0af69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf060af76, 80 },
    { 0xa3e0af76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5530af7b, 55 },
    { 0x4b20af7b, 40 },
    { 0, 0 },
    { 0xabc0af7e, 98 },
    { 0xa460af7f, 61 },
    { 0x1e70af80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60af83, 90 },
    { 0x7470af84, 96 },
    { 0x1fd0af85, 95 },
    { 0x6500af85, 87 },
    { 0x83e0af87, 68 },
    { 0, 0 },
    { 0xa940af89, 33 },
    { 0, 0 },
    { 0x67b0af8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c00af8e, 43 },
    { 0x9680af8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2220af94, 100 },
    { 0xd480af94, 98 },
    { 0xd830af94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe390af9d, 78 },
    { 0x4210af9e, 55 },
    { 0x9130af9e, 76 },
    { 0x40e0afa0, 61 },
    { 0x3590afa1, 43 },
    { 0x47f0afa2, 98 },
    { 0x9a40afa0, 34 },
    { 0x8970afa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0afa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf0afaf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e0afb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6750afbd, 83 },
    { 0xf0d0afbe, 33 },
    { 0xd4c0afbf, 61 },
    { 0, 0 },
    { 0xf5f0afc1, 36 },
    { 0, 0 },
    { 0x3360afc3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6100afc6, 33 },
    { 0, 0 },
    { 0x4ea0afc8, 95 },
    { 0xd640afc8, 61 },
    { 0xc340afca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf90afd0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d0afd4, 61 },
    { 0xe4f0afd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba0afdc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9250afe5, 95 },
    { 0, 0 },
    { 0x65c0afe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d0afeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a0afef, 95 },
    { 0xc80afef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2220affd, 98 },
    { 0x8a70affe, 76 },
    { 0x28a0afff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3920b005, 23 },
    { 0x7db0b006, 98 },
    { 0x64d0b006, 23 },
    { 0, 0 },
    { 0x52a0b009, 90 },
    { 0, 0 },
    { 0xf6e0b00b, 38 },
    { 0xed00b00c, 76 },
    { 0xee50b00d, 43 },
    { 0, 0 },
    { 0xc020b00f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e80b015, 98 },
    { 0xfbc0b016, 46 },
    { 0x5180b016, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c50b01a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030b01d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3000b023, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2030b02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f0b031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d0b035, 87 },
    { 0, 0 },
    { 0x5890b037, 68 },
    { 0xc9a0b038, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc0b042, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4170b045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8010b048, 55 },
    { 0, 0 },
    { 0xecb0b04a, 98 },
    { 0, 0 },
    { 0xa50b04c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a50b050, 23 },
    { 0xa580b051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680b054, 61 },
    { 0xb410b055, 98 },
    { 0x6400b056, 98 },
    { 0xd7e0b056, 61 },
    { 0xed00b054, 33 },
    { 0x2d80b059, 69 },
    { 0, 0 },
    { 0x9150b05b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410b060, 93 },
    { 0x4030b061, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb170b065, 77 },
    { 0xc620b066, 61 },
    { 0x9850b066, 99 },
    { 0, 0 },
    { 0xa150b069, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa50b072, 95 },
    { 0, 0 },
    { 0x2120b074, 95 },
    { 0, 0 },
    { 0x3b10b076, 78 },
    { 0xc900b077, 90 },
    { 0, 0 },
    { 0x8290b079, 38 },
    { 0xbd90b07a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6690b07e, 76 },
    { 0xbcb0b07e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5210b082, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e0b086, 90 },
    { 0x7f40b087, 55 },
    { 0x28e0b088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb0b08b, 43 },
    { 0, 0 },
    { 0xd600b08d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800b091, 76 },
    { 0xb9d0b091, 90 },
    { 0xd760b091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0b096, 61 },
    { 0xfd50b097, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df0b09c, 23 },
    { 0x2d20b09d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa720b0a5, 17 },
    { 0xac10b0a6, 93 },
    { 0x44a0b0a7, 43 },
    { 0x2c40b0a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7420b0ac, 90 },
    { 0x23b0b0ad, 90 },
    { 0x9430b0ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be0b0b1, 33 },
    { 0x6f0b0b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede0b0b6, 61 },
    { 0x22b0b0b7, 95 },
    { 0x5af0b0b7, 43 },
    { 0xa470b0b6, 46 },
    { 0x8b80b0b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb050b0be, 61 },
    { 0, 0 },
    { 0xfea0b0c0, 61 },
    { 0xcd00b0c1, 33 },
    { 0xc800b0c2, 33 },
    { 0x1c10b0c3, 75 },
    { 0x4d80b0c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c60b0ca, 55 },
    { 0, 0 },
    { 0x1a60b0cc, 100 },
    { 0x6c10b0cc, 46 },
    { 0xfeb0b0ce, 55 },
    { 0xc3d0b0cf, 23 },
    { 0x7d00b0d0, 91 },
    { 0xbae0b0cf, 95 },
    { 0xd950b0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0b0d5, 55 },
    { 0x3fd0b0d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4420b0d9, 76 },
    { 0x5200b0da, 78 },
    { 0xde80b0da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b0df, 76 },
    { 0xd4d0b0df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d0b0e3, 90 },
    { 0xd0b0b0e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7300b0ea, 61 },
    { 0x8910b0eb, 90 },
    { 0xc70b0ec, 23 },
    { 0, 0 },
    { 0xcce0b0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf970b0f3, 55 },
    { 0, 0 },
    { 0x19f0b0f5, 87 },
    { 0x6a10b0f6, 78 },
    { 0x49c0b0f7, 38 },
    { 0x6640b0f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac80b0fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b0b102, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8050b107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a90b10c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde60b10f, 99 },
    { 0x2bf0b10f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5020b115, 55 },
    { 0xc050b116, 61 },
    { 0x270b116, 78 },
    { 0x7350b118, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf50b11c, 99 },
    { 0, 0 },
    { 0x9f10b11e, 95 },
    { 0xdf10b11f, 78 },
    { 0xd280b120, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9900b126, 23 },
    { 0x4220b126, 33 },
    { 0x8f10b128, 66 },
    { 0x6c80b127, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9600b12e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb0b131, 43 },
    { 0x9670b131, 98 },
    { 0xc700b133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0b13b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0b13f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff0b143, 76 },
    { 0x4590b144, 98 },
    { 0xbc80b143, 93 },
    { 0x1c10b146, 85 },
    { 0x7040b144, 87 },
    { 0x59b0b148, 23 },
    { 0x4440b143, 23 },
    { 0, 0 },
    { 0x3fc0b14b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b50b14e, 90 },
    { 0xda00b14f, 93 },
    { 0, 0 },
    { 0x7ae0b151, 87 },
    { 0x3900b152, 94 },
    { 0x9c0b152, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70b157, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc700b15d, 95 },
    { 0x310b15e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef0b164, 95 },
    { 0xd670b165, 95 },
    { 0xb590b164, 90 },
    { 0x5290b167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec0b16a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf400b16d, 98 },
    { 0xbd0b16d, 33 },
    { 0xed10b16d, 93 },
    { 0xf3e0b16f, 33 },
    { 0, 0 },
    { 0x6630b172, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100b17a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b0b182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0b189, 43 },
    { 0, 0 },
    { 0xd760b18b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e0b18f, 75 },
    { 0, 0 },
    { 0x1430b191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c20b194, 33 },
    { 0, 0 },
    { 0xfed0b196, 43 },
    { 0x3010b197, 98 },
    { 0xede0b198, 76 },
    { 0x2860b199, 90 },
    { 0x520b198, 90 },
    { 0xddd0b197, 33 },
    { 0x1c50b19c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8240b19f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7540b1a2, 95 },
    { 0x4be0b1a3, 34 },
    { 0x7a90b1a4, 98 },
    { 0xeb0b1a4, 23 },
    { 0x8850b1a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa60b1a9, 90 },
    { 0, 0 },
    { 0x5630b1ab, 23 },
    { 0xdc0b1ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe700b1b3, 90 },
    { 0, 0 },
    { 0x2c00b1b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4240b1bb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a0b1c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3930b1c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6800b1d1, 95 },
    { 0xbcd0b1d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce0b1d5, 23 },
    { 0xe140b1d6, 95 },
    { 0x2ba0b1d6, 23 },
    { 0x2f70b1d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0b1df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650b1e4, 55 },
    { 0x8190b1e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe20b1eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd600b1ee, 23 },
    { 0, 0 },
    { 0xf760b1f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210b1f6, 55 },
    { 0xbe60b1f7, 69 },
    { 0x7d40b1f6, 90 },
    { 0x26a0b1f6, 61 },
    { 0x9730b1f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5070b1ff, 65 },
    { 0x8450b1ff, 98 },
    { 0xf9d0b1ff, 46 },
    { 0x84e0b202, 98 },
    { 0xb1ff, 84 },
    { 0, 0 },
    { 0x6b80b205, 65 },
    { 0xf3f0b206, 90 },
    { 0x1720b207, 61 },
    { 0x66a0b207, 95 },
    { 0x1470b209, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba0b20d, 43 },
    { 0x84c0b20e, 78 },
    { 0x5730b20e, 95 },
    { 0xf660b210, 46 },
    { 0x4fe0b211, 84 },
    { 0, 0 },
    { 0xf5a0b213, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1350b217, 34 },
    { 0x2720b218, 23 },
    { 0xf7c0b218, 61 },
    { 0x35c0b218, 34 },
    { 0xbb00b21b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb720b21f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7480b22c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf40b22f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc240b234, 23 },
    { 0x78a0b234, 95 },
    { 0x2060b236, 43 },
    { 0, 0 },
    { 0x58b0b238, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe70b23b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a0b23e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc90b243, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd750b24b, 41 },
    { 0, 0 },
    { 0xe190b24d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac60b250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4930b255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8180b25a, 33 },
    { 0x52d0b25b, 55 },
    { 0, 0 },
    { 0xf4a0b25d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810b260, 90 },
    { 0xca90b261, 33 },
    { 0x4270b261, 95 },
    { 0x9920b263, 69 },
    { 0, 0 },
    { 0x8a80b265, 90 },
    { 0, 0 },
    { 0x7710b267, 52 },
    { 0, 0 },
    { 0x2d30b269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc0b26c, 90 },
    { 0x9d30b26d, 98 },
    { 0, 0 },
    { 0x7f90b26f, 33 },
    { 0x47d0b270, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb550b274, 76 },
    { 0xbc30b275, 61 },
    { 0x1110b274, 90 },
    { 0, 0 },
    { 0x97d0b278, 76 },
    { 0x2430b279, 98 },
    { 0, 0 },
    { 0x5580b27b, 23 },
    { 0x36c0b27b, 95 },
    { 0xd800b27d, 95 },
    { 0xa320b27e, 98 },
    { 0x6590b27f, 93 },
    { 0xf380b27f, 76 },
    { 0, 0 },
    { 0x3db0b282, 76 },
    { 0, 0 },
    { 0xcfc0b284, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d30b28b, 66 },
    { 0xde60b28c, 77 },
    { 0x6440b28c, 34 },
    { 0x5250b28e, 90 },
    { 0x30c0b28f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe440b292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb360b295, 36 },
    { 0xd1b0b296, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef0b2a1, 90 },
    { 0x64d0b2a2, 90 },
    { 0xd280b2a3, 95 },
    { 0xfff0b2a4, 76 },
    { 0, 0 },
    { 0xd1c0b2a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb10b2ad, 55 },
    { 0, 0 },
    { 0x2ad0b2af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c10b2b4, 91 },
    { 0x12b0b2b5, 55 },
    { 0xe8e0b2b5, 61 },
    { 0, 0 },
    { 0x99e0b2b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f0b2be, 66 },
    { 0xd460b2be, 66 },
    { 0, 0 },
    { 0x2b60b2c1, 90 },
    { 0, 0 },
    { 0xd320b2c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb160b2c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe780b2cb, 52 },
    { 0x8d70b2cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd0b2d2, 98 },
    { 0xda70b2d2, 67 },
    { 0xea20b2d4, 90 },
    { 0x7f50b2d5, 95 },
    { 0x2b50b2d6, 33 },
    { 0xe610b2d7, 90 },
    { 0xdaf0b2d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f0b2db, 98 },
    { 0xb040b2dc, 61 },
    { 0x8340b2db, 95 },
    { 0x89c0b2de, 61 },
    { 0, 0 },
    { 0xd0c0b2e0, 66 },
    { 0xe030b2e0, 98 },
    { 0xff60b2e2, 33 },
    { 0, 0 },
    { 0xc470b2e4, 75 },
    { 0, 0 },
    { 0x36d0b2e6, 91 },
    { 0, 0 },
    { 0xf970b2e8, 34 },
    { 0, 0 },
    { 0x35f0b2ea, 67 },
    { 0, 0 },
    { 0xd510b2ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff10b2f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1410b2f6, 34 },
    { 0x2ae0b2f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f0b2fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a20b2fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890b304, 76 },
    { 0, 0 },
    { 0xbb70b306, 90 },
    { 0, 0 },
    { 0xdf10b308, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d30b30c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf0b313, 90 },
    { 0xdd0b313, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb0b31e, 98 },
    { 0xac10b31f, 78 },
    { 0xfc30b31f, 33 },
    { 0x4b90b321, 17 },
    { 0x240b321, 34 },
    { 0x7f20b320, 33 },
    { 0xab80b31f, 44 },
    { 0x6d00b31e, 66 },
    { 0x2930b326, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb610b331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50b33a, 23 },
    { 0, 0 },
    { 0x6d0b33c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c0b33f, 90 },
    { 0, 0 },
    { 0xe1b0b341, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d50b344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7450b348, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf50b34d, 78 },
    { 0xbe30b34d, 90 },
    { 0xb950b34f, 46 },
    { 0x2b90b34e, 46 },
    { 0xa0c0b351, 93 },
    { 0x6fd0b352, 99 },
    { 0xa330b352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd0b35b, 34 },
    { 0x3800b35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d00b364, 94 },
    { 0x54d0b365, 91 },
    { 0xd730b365, 98 },
    { 0xce30b367, 95 },
    { 0x1ec0b368, 43 },
    { 0, 0 },
    { 0xbcd0b36a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a00b36e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1500b371, 40 },
    { 0, 0 },
    { 0x2d50b373, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970b377, 61 },
    { 0x5530b378, 98 },
    { 0x6510b379, 33 },
    { 0xe920b37a, 46 },
    { 0xb2d0b379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c50b37f, 23 },
    { 0x2e20b380, 90 },
    { 0x5c30b37f, 33 },
    { 0xf0b382, 61 },
    { 0, 0 },
    { 0x9100b384, 33 },
    { 0, 0 },
    { 0x3c40b386, 75 },
    { 0x6990b387, 95 },
    { 0x1310b388, 33 },
    { 0xf040b388, 92 },
    { 0xe250b389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a0b38d, 90 },
    { 0xde30b38d, 81 },
    { 0x6a00b38f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a70b397, 33 },
    { 0x5020b398, 98 },
    { 0x87f0b399, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x270b39c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd940b3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f0b3a4, 17 },
    { 0x3a80b3a4, 33 },
    { 0, 0 },
    { 0x6230b3a7, 66 },
    { 0xdc20b3a8, 90 },
    { 0x4430b3a9, 34 },
    { 0x3970b3aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8950b3ad, 85 },
    { 0xf940b3ae, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d0b3b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6570b3b5, 78 },
    { 0x3e30b3b5, 33 },
    { 0, 0 },
    { 0x2d10b3b8, 100 },
    { 0xbdd0b3b9, 33 },
    { 0, 0 },
    { 0x7000b3bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8170b3c0, 90 },
    { 0x3af0b3c1, 84 },
    { 0, 0 },
    { 0x2c60b3c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e50b3c9, 90 },
    { 0x8e90b3ca, 23 },
    { 0xcf90b3c9, 92 },
    { 0, 0 },
    { 0x370b3cd, 85 },
    { 0x9ac0b3ce, 85 },
    { 0xb330b3ce, 98 },
    { 0x1f10b3d0, 34 },
    { 0x37b0b3cf, 23 },
    { 0, 0 },
    { 0xfec0b3d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d0b3d8, 76 },
    { 0, 0 },
    { 0x9a80b3da, 77 },
    { 0xb4d0b3db, 98 },
    { 0, 0 },
    { 0x1ed0b3dd, 85 },
    { 0xe0d0b3de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e80b3e1, 66 },
    { 0x3c40b3e2, 98 },
    { 0x3be0b3e2, 90 },
    { 0x9550b3e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3440b3e7, 95 },
    { 0x6700b3e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1300b3ed, 43 },
    { 0, 0 },
    { 0xc3f0b3ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde20b3f2, 36 },
    { 0xbc40b3f3, 80 },
    { 0x56b0b3f4, 61 },
    { 0x3e00b3f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe560b3f8, 76 },
    { 0, 0 },
    { 0xd0a0b3fa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba0b3fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570b403, 61 },
    { 0x14e0b404, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b70b40c, 100 },
    { 0, 0 },
    { 0x88d0b40e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60b411, 91 },
    { 0xbef0b411, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf0b416, 33 },
    { 0xa3d0b417, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa0b41e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410b422, 23 },
    { 0xc80b423, 76 },
    { 0xc210b424, 76 },
    { 0xbb00b425, 58 },
    { 0x5ed0b426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4110b42a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b0b42e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a70b431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e0b435, 90 },
    { 0, 0 },
    { 0x99b0b437, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e0b442, 43 },
    { 0x970b443, 33 },
    { 0x7ab0b444, 23 },
    { 0x4540b444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f90b449, 51 },
    { 0, 0 },
    { 0x5e50b44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3770b44f, 76 },
    { 0x3b60b450, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a0b455, 99 },
    { 0x71a0b455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe30b45b, 90 },
    { 0, 0 },
    { 0x6290b45d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d00b464, 52 },
    { 0x5820b465, 98 },
    { 0x9040b465, 61 },
    { 0xd5d0b465, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d0b46a, 23 },
    { 0, 0 },
    { 0xfb0b46c, 23 },
    { 0xf180b46d, 98 },
    { 0, 0 },
    { 0x8390b46f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8120b472, 34 },
    { 0x4320b473, 55 },
    { 0xe730b474, 99 },
    { 0x3ee0b472, 46 },
    { 0, 0 },
    { 0x25f0b477, 90 },
    { 0, 0 },
    { 0xdf20b479, 55 },
    { 0x2f70b47a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf50b47d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c0b483, 66 },
    { 0, 0 },
    { 0x5bb0b485, 55 },
    { 0xd90b485, 23 },
    { 0x7110b486, 52 },
    { 0x9d20b488, 95 },
    { 0xa400b487, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b48d, 33 },
    { 0xcb50b48d, 43 },
    { 0x3d40b48f, 75 },
    { 0x9de0b48e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b0b497, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f0b49b, 58 },
    { 0, 0 },
    { 0x8e60b49d, 33 },
    { 0xaf30b49d, 53 },
    { 0xbd80b49d, 98 },
    { 0x12c0b4a0, 96 },
    { 0xe810b49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c0b4ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c0b4b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7400b4b6, 61 },
    { 0x6800b4b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe960b4ba, 23 },
    { 0xb740b4bb, 33 },
    { 0x81f0b4bc, 95 },
    { 0x4a0b4bc, 33 },
    { 0, 0 },
    { 0x86c0b4bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9050b4cb, 23 },
    { 0xb8c0b4cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0b4d6, 33 },
    { 0x1600b4d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c80b4dd, 33 },
    { 0, 0 },
    { 0x8da0b4df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d0b4e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0b4ea, 95 },
    { 0x8440b4eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c0b4ee, 55 },
    { 0xb360b4ee, 43 },
    { 0xa8f0b4f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7160b4f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb580b4f9, 23 },
    { 0x6630b4f9, 90 },
    { 0x1180b4fa, 93 },
    { 0x7480b4fa, 23 },
    { 0xb170b4fd, 76 },
    { 0x9150b4fe, 33 },
    { 0x1300b4ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8810b509, 76 },
    { 0, 0 },
    { 0x200b50b, 90 },
    { 0x1c90b50c, 98 },
    { 0, 0 },
    { 0xd610b50e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b0b516, 23 },
    { 0xede0b517, 66 },
    { 0x7ab0b516, 98 },
    { 0, 0 },
    { 0xb350b51a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f0b520, 17 },
    { 0, 0 },
    { 0x10d0b522, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5990b525, 61 },
    { 0x6550b526, 85 },
    { 0xc890b527, 43 },
    { 0xc2e0b528, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b80b52f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8920b532, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f0b537, 90 },
    { 0xc690b537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a90b53b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7260b545, 61 },
    { 0xb380b546, 33 },
    { 0x9420b547, 76 },
    { 0x83e0b548, 90 },
    { 0, 0 },
    { 0xdc20b54a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb580b54e, 61 },
    { 0x2bc0b54e, 98 },
    { 0, 0 },
    { 0x2860b551, 95 },
    { 0, 0 },
    { 0xdef0b553, 46 },
    { 0xcc10b554, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8850b55b, 96 },
    { 0x2960b55b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f0b55f, 96 },
    { 0xac00b560, 98 },
    { 0x5fd0b561, 23 },
    { 0x1ac0b562, 91 },
    { 0x7500b562, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae90b568, 64 },
    { 0xf920b569, 80 },
    { 0x9f50b569, 90 },
    { 0xd120b56b, 76 },
    { 0x9be0b56b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0b570, 65 },
    { 0xd3c0b571, 84 },
    { 0xc330b570, 87 },
    { 0xd980b573, 98 },
    { 0, 0 },
    { 0xb180b575, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540b579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990b57d, 75 },
    { 0x13e0b57d, 34 },
    { 0, 0 },
    { 0xc0f0b580, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e90b587, 90 },
    { 0x8190b588, 98 },
    { 0x33f0b588, 98 },
    { 0xd1e0b58a, 46 },
    { 0, 0 },
    { 0xd490b58c, 55 },
    { 0x96a0b58c, 42 },
    { 0x66f0b58e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9260b594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c0b5a0, 84 },
    { 0, 0 },
    { 0xb9f0b5a2, 90 },
    { 0, 0 },
    { 0x2e70b5a4, 46 },
    { 0, 0 },
    { 0x2300b5a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb290b5a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4950b5ac, 98 },
    { 0xa570b5ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef0b5b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe490b5b7, 90 },
    { 0x18f0b5b8, 23 },
    { 0x6a10b5b9, 98 },
    { 0x7940b5b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf0b5be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc440b5c2, 99 },
    { 0, 0 },
    { 0xab0b5c4, 43 },
    { 0x7580b5c4, 95 },
    { 0xd820b5c6, 95 },
    { 0xea70b5c7, 96 },
    { 0, 0 },
    { 0x5400b5c9, 99 },
    { 0x43d0b5ca, 66 },
    { 0x8410b5ca, 95 },
    { 0, 0 },
    { 0x6630b5cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd40b5d1, 88 },
    { 0x8670b5d2, 52 },
    { 0x15c0b5d2, 68 },
    { 0x3e50b5d3, 61 },
    { 0x3460b5d1, 34 },
    { 0, 0 },
    { 0xc830b5d7, 46 },
    { 0, 0 },
    { 0xdae0b5d9, 95 },
    { 0, 0 },
    { 0x8cc0b5db, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60b5e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec0b5e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810b5ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x440b5f0, 98 },
    { 0xf20b5f0, 64 },
    { 0x7870b5f0, 55 },
    { 0xe40b5f2, 34 },
    { 0, 0 },
    { 0xaa0b5f5, 33 },
    { 0x53d0b5f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e0b5f9, 98 },
    { 0, 0 },
    { 0xa6f0b5fb, 23 },
    { 0x4710b5fc, 43 },
    { 0xeaf0b5fd, 69 },
    { 0x8d90b5fd, 23 },
    { 0, 0 },
    { 0xfb30b600, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570b605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db0b60b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe90b610, 90 },
    { 0, 0 },
    { 0x60d0b612, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b617, 23 },
    { 0, 0 },
    { 0x9170b619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc160b61e, 55 },
    { 0xb540b61e, 95 },
    { 0x79b0b61e, 91 },
    { 0x4430b621, 61 },
    { 0xda50b622, 76 },
    { 0x76f0b61f, 34 },
    { 0xe110b624, 98 },
    { 0, 0 },
    { 0xb030b626, 33 },
    { 0xa8b0b627, 78 },
    { 0x47c0b628, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a80b62c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f0b630, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d50b633, 22 },
    { 0x4f0b634, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00b637, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6860b63c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c0b641, 61 },
    { 0, 0 },
    { 0x68f0b643, 90 },
    { 0xc410b644, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4380b64b, 90 },
    { 0x8560b64c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba0b650, 90 },
    { 0xfc30b650, 91 },
    { 0x6390b652, 61 },
    { 0xfdf0b652, 34 },
    { 0x5990b653, 61 },
    { 0x3cf0b655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7180b659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc0b65c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710b672, 93 },
    { 0x2c50b673, 98 },
    { 0xd80b674, 33 },
    { 0x4c90b672, 75 },
    { 0x8ca0b676, 61 },
    { 0, 0 },
    { 0x5e20b678, 61 },
    { 0, 0 },
    { 0xe8e0b67a, 76 },
    { 0x28b0b67a, 98 },
    { 0xd7b0b67c, 34 },
    { 0, 0 },
    { 0x4150b67e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c0b682, 43 },
    { 0xba20b683, 98 },
    { 0xbd90b682, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0b687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b690, 33 },
    { 0xe130b690, 93 },
    { 0x6ed0b692, 85 },
    { 0x2980b691, 67 },
    { 0, 0 },
    { 0xf6b0b695, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00b699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0b69d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f60b6a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0b6b1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f0b6b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0b6ba, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7640b6bf, 98 },
    { 0, 0 },
    { 0x6fc0b6c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da0b6c4, 100 },
    { 0, 0 },
    { 0x7e60b6c6, 55 },
    { 0xf8c0b6c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0b6ca, 61 },
    { 0x6670b6ca, 67 },
    { 0x8840b6cb, 68 },
    { 0, 0 },
    { 0x580b6ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa600b6e0, 95 },
    { 0x1550b6e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d70b6e7, 76 },
    { 0x710b6e8, 23 },
    { 0, 0 },
    { 0x9f30b6ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd620b6ed, 98 },
    { 0, 0 },
    { 0x1070b6ef, 28 },
    { 0x7a10b6ef, 97 },
    { 0x50d0b6f1, 61 },
    { 0x3840b6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a60b6f5, 88 },
    { 0xba30b6f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b6fa, 23 },
    { 0x6530b6fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c90b6fe, 90 },
    { 0xbed0b6ff, 87 },
    { 0x5110b6ff, 100 },
    { 0xbc00b701, 95 },
    { 0xfe0b701, 61 },
    { 0x6520b703, 33 },
    { 0xcb60b700, 55 },
    { 0x1640b705, 33 },
    { 0x84e0b706, 90 },
    { 0xe3e0b706, 23 },
    { 0x4810b700, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd470b70b, 77 },
    { 0, 0 },
    { 0x8230b70d, 91 },
    { 0x5290b70e, 33 },
    { 0x27d0b70d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7510b716, 77 },
    { 0x1cc0b717, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8540b71a, 95 },
    { 0, 0 },
    { 0xecb0b71c, 23 },
    { 0xe720b71d, 95 },
    { 0x6500b71e, 81 },
    { 0xc1f0b71f, 95 },
    { 0x9ed0b720, 84 },
    { 0x6650b720, 43 },
    { 0x58c0b721, 78 },
    { 0xd780b722, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0b727, 33 },
    { 0xcca0b728, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc0b72b, 76 },
    { 0xd930b72b, 34 },
    { 0, 0 },
    { 0x77f0b72e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b70b732, 33 },
    { 0xc180b733, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa370b736, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f20b739, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe0b73d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde40b741, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9310b746, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb60b749, 61 },
    { 0x4b10b74a, 90 },
    { 0x50a0b749, 84 },
    { 0, 0 },
    { 0x11f0b74d, 52 },
    { 0xf840b74d, 33 },
    { 0x79e0b74d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f50b752, 90 },
    { 0, 0 },
    { 0x2640b754, 43 },
    { 0x3590b755, 33 },
    { 0xf5e0b756, 98 },
    { 0x2dd0b757, 52 },
    { 0xb1b0b758, 87 },
    { 0x9d50b756, 66 },
    { 0, 0 },
    { 0xa220b75b, 33 },
    { 0, 0 },
    { 0x1b20b75d, 23 },
    { 0xe810b75d, 78 },
    { 0x27d0b75e, 90 },
    { 0, 0 },
    { 0x1bf0b761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b767, 23 },
    { 0xbe10b768, 86 },
    { 0, 0 },
    { 0xd8b0b76a, 66 },
    { 0x1b60b76b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed0b770, 90 },
    { 0xba00b771, 98 },
    { 0x4260b771, 34 },
    { 0x9000b770, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0b776, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf0b779, 95 },
    { 0x8f60b779, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e0b780, 46 },
    { 0xbeb0b781, 76 },
    { 0xf980b780, 90 },
    { 0, 0 },
    { 0xa490b784, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d0b788, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0b78e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f0b794, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4050b797, 55 },
    { 0x910b798, 61 },
    { 0xcd80b798, 91 },
    { 0, 0 },
    { 0xc40b79b, 34 },
    { 0x4c0b79c, 95 },
    { 0, 0 },
    { 0xd5c0b79e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0b7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af0b7a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0b7a9, 61 },
    { 0x5c70b7aa, 91 },
    { 0xb200b7ab, 99 },
    { 0x51e0b7ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2500b7b1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e0b7b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f30b7bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20b7be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa240b7c2, 90 },
    { 0x7840b7c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b0b7ca, 43 },
    { 0, 0 },
    { 0x4f60b7cc, 76 },
    { 0xa9d0b7cd, 95 },
    { 0xc660b7ce, 90 },
    { 0xa140b7ce, 90 },
    { 0xa700b7ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1990b7d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc0b7d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9200b7dd, 61 },
    { 0xda50b7dd, 93 },
    { 0, 0 },
    { 0xb2b0b7e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe30b7e3, 90 },
    { 0x9b90b7e4, 66 },
    { 0xcd00b7e3, 46 },
    { 0, 0 },
    { 0x4850b7e7, 51 },
    { 0xf450b7e8, 90 },
    { 0x7f90b7e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4930b7f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe630b7f7, 58 },
    { 0x2d20b7f8, 93 },
    { 0, 0 },
    { 0x7a20b7fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650b7fd, 99 },
    { 0xe610b7fe, 90 },
    { 0, 0 },
    { 0x92a0b800, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e20b806, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600b810, 55 },
    { 0xe4c0b811, 55 },
    { 0x5860b810, 33 },
    { 0xdb20b813, 90 },
    { 0xec50b813, 90 },
    { 0x22b0b813, 76 },
    { 0x2410b816, 61 },
    { 0xab70b816, 68 },
    { 0xfb20b818, 76 },
    { 0xfa0b814, 98 },
    { 0, 0 },
    { 0xe9f0b81b, 90 },
    { 0x28e0b81b, 93 },
    { 0x3520b81b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9020b822, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8900b827, 98 },
    { 0xf9f0b828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac0b82c, 61 },
    { 0x88f0b82c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf0b831, 93 },
    { 0, 0 },
    { 0x7860b833, 91 },
    { 0x3f10b834, 95 },
    { 0xb420b835, 93 },
    { 0, 0 },
    { 0x9b0b837, 98 },
    { 0, 0 },
    { 0x4fd0b839, 55 },
    { 0xf600b839, 33 },
    { 0, 0 },
    { 0x76f0b83c, 40 },
    { 0x6d40b83d, 23 },
    { 0x1830b83d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b0b841, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d0b846, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2120b84b, 61 },
    { 0x56d0b84c, 66 },
    { 0, 0 },
    { 0xe520b84e, 100 },
    { 0x71f0b84e, 95 },
    { 0x3670b850, 46 },
    { 0x2560b850, 34 },
    { 0x2140b84e, 98 },
    { 0xb850b853, 55 },
    { 0x3080b853, 95 },
    { 0x2ef0b855, 33 },
    { 0, 0 },
    { 0x5330b857, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2400b85a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e0b85e, 93 },
    { 0x27b0b85f, 55 },
    { 0x5060b860, 76 },
    { 0x99f0b861, 61 },
    { 0x9a80b85e, 40 },
    { 0x4b90b863, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb0b869, 80 },
    { 0, 0 },
    { 0xa290b86b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee0b86e, 46 },
    { 0xa1b0b86f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2010b876, 33 },
    { 0, 0 },
    { 0x4d50b878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6880b87b, 90 },
    { 0, 0 },
    { 0xc3c0b87d, 67 },
    { 0x22b0b87d, 81 },
    { 0x63e0b87f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8450b884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7510b888, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae00b88d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e70b890, 100 },
    { 0x7770b891, 23 },
    { 0x6670b892, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f30b895, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4530b898, 76 },
    { 0x75d0b899, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b8a0, 84 },
    { 0, 0 },
    { 0x78f0b8a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b8a6, 90 },
    { 0x52e0b8a7, 95 },
    { 0x9940b8a8, 76 },
    { 0, 0 },
    { 0x79f0b8aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b80b8b0, 55 },
    { 0, 0 },
    { 0xec80b8b2, 17 },
    { 0x9370b8b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf200b8b7, 84 },
    { 0xd0b8b7, 98 },
    { 0, 0 },
    { 0xa0a0b8ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc900b8bf, 23 },
    { 0xd930b8c0, 98 },
    { 0, 0 },
    { 0x2690b8c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd290b8c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb0b8c9, 33 },
    { 0, 0 },
    { 0x4f20b8cb, 55 },
    { 0, 0 },
    { 0xc650b8cd, 52 },
    { 0, 0 },
    { 0x9d50b8cf, 74 },
    { 0x1220b8cf, 23 },
    { 0x4660b8d1, 55 },
    { 0xa730b8d2, 76 },
    { 0x3b70b8d2, 61 },
    { 0, 0 },
    { 0xa6b0b8d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c0b8d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc090b8db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c0b8df, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6940b8e3, 93 },
    { 0xaa30b8e3, 75 },
    { 0xb640b8e5, 61 },
    { 0xfe00b8e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70b8ed, 61 },
    { 0x5220b8ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db0b8f3, 43 },
    { 0, 0 },
    { 0xb5c0b8f5, 15 },
    { 0xd740b8f6, 33 },
    { 0, 0 },
    { 0x22f0b8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0b8fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac0b900, 95 },
    { 0x83e0b901, 90 },
    { 0xd190b902, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad0b906, 23 },
    { 0xab50b907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2720b90d, 33 },
    { 0x90b0b90e, 90 },
    { 0xd600b90e, 60 },
    { 0, 0 },
    { 0xcbd0b911, 95 },
    { 0, 0 },
    { 0x4e00b913, 61 },
    { 0xb420b914, 61 },
    { 0x7fd0b915, 19 },
    { 0x7c90b916, 33 },
    { 0xaff0b917, 33 },
    { 0, 0 },
    { 0x9bc0b919, 95 },
    { 0xe460b91a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9980b91d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8260b920, 68 },
    { 0x48c0b920, 61 },
    { 0x3800b922, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc480b929, 33 },
    { 0x26f0b92a, 23 },
    { 0xce10b92a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90b92e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b933, 98 },
    { 0, 0 },
    { 0x7c70b935, 88 },
    { 0x1510b935, 66 },
    { 0x3c70b935, 34 },
    { 0, 0 },
    { 0x21d0b939, 81 },
    { 0x5590b93a, 99 },
    { 0x39b0b93b, 55 },
    { 0x3690b93c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d0b940, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xb560b943, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a0b949, 90 },
    { 0, 0 },
    { 0x3800b94b, 34 },
    { 0x1020b94c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d70b94f, 90 },
    { 0x5560b950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f0b954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060b959, 52 },
    { 0xe240b959, 88 },
    { 0, 0 },
    { 0x21c0b95c, 33 },
    { 0xda60b95d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50b965, 84 },
    { 0xf600b965, 90 },
    { 0x4580b967, 95 },
    { 0xccd0b968, 90 },
    { 0xf3d0b969, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8750b96d, 67 },
    { 0x8eb0b96e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f0b972, 52 },
    { 0xba10b973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4610b976, 99 },
    { 0, 0 },
    { 0x29b0b978, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7920b97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5960b983, 61 },
    { 0, 0 },
    { 0xc0b985, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc0b993, 64 },
    { 0xe0c0b994, 38 },
    { 0xe120b995, 43 },
    { 0xad70b996, 55 },
    { 0x4390b996, 33 },
    { 0x35d0b996, 33 },
    { 0x6300b997, 85 },
    { 0x8a50b99a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2230b99d, 34 },
    { 0, 0 },
    { 0xa110b99f, 33 },
    { 0x6700b9a0, 90 },
    { 0, 0 },
    { 0x8510b9a2, 90 },
    { 0xd910b9a2, 46 },
    { 0xdd0b9a4, 68 },
    { 0x6510b9a2, 44 },
    { 0xe0d0b9a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a60b9aa, 52 },
    { 0xfa10b9ab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4030b9b1, 77 },
    { 0, 0 },
    { 0x7fc0b9b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2780b9b8, 88 },
    { 0, 0 },
    { 0x8d00b9ba, 90 },
    { 0xa90b9ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e40b9c1, 77 },
    { 0x6290b9c2, 55 },
    { 0xd0f0b9c3, 61 },
    { 0xc250b9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c0b9c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5020b9cb, 33 },
    { 0x7910b9cc, 95 },
    { 0xdef0b9cd, 43 },
    { 0xd250b9ce, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b00b9d1, 76 },
    { 0x9040b9d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b9d5, 23 },
    { 0xd740b9d6, 95 },
    { 0x8fd0b9d6, 90 },
    { 0x3a00b9d6, 40 },
    { 0, 0 },
    { 0xd8f0b9da, 49 },
    { 0xd860b9db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a0b9df, 23 },
    { 0x55c0b9e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7270b9e9, 61 },
    { 0x3270b9ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0b9f3, 98 },
    { 0x6ed0b9f4, 91 },
    { 0xc870b9f5, 67 },
    { 0, 0 },
    { 0xfe40b9f7, 100 },
    { 0, 0 },
    { 0x3c20b9f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8930b9fd, 90 },
    { 0xdff0b9fe, 43 },
    { 0x9d60b9ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a80ba02, 23 },
    { 0, 0 },
    { 0x67d0ba04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da0ba08, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3410ba0b, 90 },
    { 0x6450ba0b, 90 },
    { 0x28f0ba0b, 43 },
    { 0x1180ba0d, 34 },
    { 0x9960ba0f, 23 },
    { 0x3100ba10, 87 },
    { 0, 0 },
    { 0x23d0ba12, 33 },
    { 0x2360ba13, 33 },
    { 0xd850ba14, 61 },
    { 0xc820ba15, 90 },
    { 0x9ea0ba15, 96 },
    { 0, 0 },
    { 0x410ba18, 90 },
    { 0x3dc0ba19, 46 },
    { 0x880ba1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda10ba1f, 76 },
    { 0xa890ba1f, 88 },
    { 0x4b20ba21, 4 },
    { 0x69d0ba1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340ba26, 67 },
    { 0x1a30ba27, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5870ba2b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5190ba2e, 98 },
    { 0, 0 },
    { 0x2ec0ba30, 40 },
    { 0x7370ba31, 33 },
    { 0x8fc0ba31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba60ba38, 76 },
    { 0, 0 },
    { 0xa530ba3a, 87 },
    { 0x79a0ba3a, 90 },
    { 0x9650ba3c, 23 },
    { 0x2320ba3c, 90 },
    { 0x6fd0ba3a, 62 },
    { 0x7600ba3f, 98 },
    { 0xd380ba3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0ba43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda50ba48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0ba4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a30ba4e, 33 },
    { 0, 0 },
    { 0xc220ba50, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb70ba55, 23 },
    { 0x49c0ba56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0ba5b, 98 },
    { 0xb6c0ba5b, 33 },
    { 0xa620ba5d, 98 },
    { 0, 0 },
    { 0x58c0ba5f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9520ba66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae80ba6a, 90 },
    { 0, 0 },
    { 0xf640ba6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d0ba6f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa0ba77, 90 },
    { 0x91c0ba78, 90 },
    { 0xdaa0ba79, 23 },
    { 0x85d0ba77, 17 },
    { 0xba80ba7b, 33 },
    { 0xc3d0ba7c, 93 },
    { 0x8240ba7d, 90 },
    { 0x7ba0ba7c, 98 },
    { 0xf620ba7f, 34 },
    { 0x9cf0ba80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe800ba84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d0ba87, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1550ba8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b0ba91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce20ba96, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0ba99, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca0ba9c, 33 },
    { 0, 0 },
    { 0xc0b0ba9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc390baab, 90 },
    { 0x5c70baac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a0baaf, 90 },
    { 0, 0 },
    { 0x1610bab1, 98 },
    { 0, 0 },
    { 0x5b20bab3, 46 },
    { 0, 0 },
    { 0x8b00bab5, 66 },
    { 0x59c0bab6, 87 },
    { 0xe390bab6, 33 },
    { 0, 0 },
    { 0x41a0bab9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef90bac6, 34 },
    { 0, 0 },
    { 0xd40bac8, 90 },
    { 0xd860bac9, 90 },
    { 0x2590bac9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b0bad1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc00badf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa0bae3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70bae6, 42 },
    { 0x2bb0bae7, 87 },
    { 0, 0 },
    { 0x77e0bae9, 61 },
    { 0xb860bae9, 90 },
    { 0, 0 },
    { 0xd100baec, 61 },
    { 0, 0 },
    { 0x3310baee, 90 },
    { 0xed00baef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce0baf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0bb02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e30bb05, 98 },
    { 0xb10bb05, 90 },
    { 0x7990bb07, 95 },
    { 0, 0 },
    { 0x4900bb09, 95 },
    { 0x6a60bb0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e0bb0d, 75 },
    { 0xe8e0bb0e, 93 },
    { 0x3dd0bb0d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c00bb12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc0bb15, 23 },
    { 0xaa80bb16, 90 },
    { 0, 0 },
    { 0x5270bb18, 33 },
    { 0x93b0bb18, 33 },
    { 0x8ed0bb18, 77 },
    { 0x4e30bb1b, 55 },
    { 0xfa70bb1c, 88 },
    { 0, 0 },
    { 0xd040bb1e, 52 },
    { 0x1d90bb1e, 33 },
    { 0x6c70bb1f, 61 },
    { 0x54f0bb21, 23 },
    { 0xefc0bb1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0bb26, 100 },
    { 0, 0 },
    { 0x7630bb28, 66 },
    { 0, 0 },
    { 0x2740bb2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df0bb2e, 98 },
    { 0, 0 },
    { 0xeba0bb30, 81 },
    { 0x36f0bb31, 77 },
    { 0x7f50bb31, 81 },
    { 0x3cf0bb30, 98 },
    { 0x34c0bb30, 93 },
    { 0xd520bb34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6110bb39, 95 },
    { 0, 0 },
    { 0x3020bb3b, 61 },
    { 0x2450bb3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10bb3f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140bb42, 90 },
    { 0xe40bb43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d0bb46, 33 },
    { 0, 0 },
    { 0xeba0bb48, 95 },
    { 0x2400bb49, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb0bb4e, 23 },
    { 0xb230bb4f, 90 },
    { 0x7e50bb50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec20bb56, 98 },
    { 0, 0 },
    { 0x78d0bb58, 88 },
    { 0xd220bb58, 33 },
    { 0xac10bb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca0bb5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00bb6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1970bb72, 61 },
    { 0, 0 },
    { 0xce60bb74, 33 },
    { 0xa5a0bb75, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb0bb78, 61 },
    { 0xf4d0bb79, 83 },
    { 0x4ad0bb7a, 61 },
    { 0x59b0bb79, 76 },
    { 0, 0 },
    { 0x3800bb7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3840bb80, 100 },
    { 0, 0 },
    { 0xa960bb82, 90 },
    { 0, 0 },
    { 0xa9e0bb84, 33 },
    { 0, 0 },
    { 0xc790bb86, 40 },
    { 0xfd10bb86, 46 },
    { 0x26c0bb88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7580bb8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d0bb91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e0bb94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540bb97, 76 },
    { 0x860bb97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e0bba0, 99 },
    { 0x2160bba1, 23 },
    { 0xd8a0bba2, 23 },
    { 0x3590bba2, 95 },
    { 0xb520bba4, 33 },
    { 0, 0 },
    { 0x7d50bba6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d0bba9, 87 },
    { 0x74c0bbaa, 90 },
    { 0x3220bbab, 33 },
    { 0x1760bbac, 95 },
    { 0x9050bbac, 77 },
    { 0xcd10bbae, 78 },
    { 0x60c0bbaf, 95 },
    { 0x3320bbb0, 91 },
    { 0xcce0bbb1, 93 },
    { 0x7c50bbae, 46 },
    { 0x710bbae, 98 },
    { 0x9d70bbb4, 67 },
    { 0x2b20bbae, 98 },
    { 0, 0 },
    { 0x47e0bbb7, 98 },
    { 0x4160bbb8, 55 },
    { 0x6710bbb8, 61 },
    { 0x3400bbb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba0bbc2, 98 },
    { 0xe000bbc3, 93 },
    { 0xb6e0bbc4, 23 },
    { 0xc3c0bbc5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e0bbca, 76 },
    { 0xae0bbcb, 61 },
    { 0x8b60bbcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e60bbcf, 90 },
    { 0xc640bbd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba0bbd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b0bbde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4170bbe1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bbe5, 17 },
    { 0x8c0bbe5, 52 },
    { 0x95d0bbe7, 23 },
    { 0xac60bbe8, 78 },
    { 0x7b20bbe9, 23 },
    { 0x5300bbe6, 61 },
    { 0xeb20bbeb, 33 },
    { 0, 0 },
    { 0x6680bbed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8740bbf2, 55 },
    { 0, 0 },
    { 0xe860bbf4, 95 },
    { 0, 0 },
    { 0x4d10bbf6, 95 },
    { 0x7750bbf7, 33 },
    { 0x910bbf8, 43 },
    { 0x3f30bbf7, 98 },
    { 0xb2c0bbf9, 90 },
    { 0x5e0bbf7, 90 },
    { 0x39e0bbfc, 91 },
    { 0xfb50bbfd, 23 },
    { 0, 0 },
    { 0x7dc0bbff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f80bc02, 34 },
    { 0, 0 },
    { 0xb960bc04, 23 },
    { 0x55c0bc05, 66 },
    { 0, 0 },
    { 0xd2f0bc07, 61 },
    { 0x8890bc08, 33 },
    { 0x9ab0bc08, 98 },
    { 0xd430bc0a, 76 },
    { 0x4180bc0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf00bc11, 61 },
    { 0x2a20bc11, 23 },
    { 0x3c10bc13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9660bc18, 95 },
    { 0, 0 },
    { 0x5c10bc1a, 52 },
    { 0, 0 },
    { 0x4fa0bc1c, 43 },
    { 0x4490bc1d, 95 },
    { 0, 0 },
    { 0xf520bc1f, 66 },
    { 0xb710bc1f, 90 },
    { 0x5b80bc21, 55 },
    { 0, 0 },
    { 0x49c0bc23, 95 },
    { 0, 0 },
    { 0x640bc25, 33 },
    { 0xde10bc26, 61 },
    { 0x1f60bc26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100bc2d, 23 },
    { 0, 0 },
    { 0xd290bc2f, 90 },
    { 0, 0 },
    { 0xfe50bc31, 95 },
    { 0x2410bc31, 46 },
    { 0, 0 },
    { 0x3620bc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4920bc37, 23 },
    { 0x13e0bc38, 23 },
    { 0x65f0bc39, 85 },
    { 0xe570bc3a, 80 },
    { 0x720bc38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac0bc3e, 33 },
    { 0xcdd0bc3f, 33 },
    { 0x1d0bc3f, 33 },
    { 0xf410bc41, 33 },
    { 0xfcf0bc3e, 91 },
    { 0, 0 },
    { 0x1930bc44, 43 },
    { 0x2270bc45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f60bc51, 98 },
    { 0, 0 },
    { 0xd350bc53, 90 },
    { 0xf20bc54, 33 },
    { 0x9ba0bc55, 33 },
    { 0x2500bc56, 46 },
    { 0x7e70bc54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa980bc5b, 87 },
    { 0xf9e0bc5b, 95 },
    { 0xaca0bc5b, 68 },
    { 0xdfc0bc5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a0bc63, 23 },
    { 0, 0 },
    { 0x6420bc65, 23 },
    { 0, 0 },
    { 0x7f40bc67, 61 },
    { 0x2f10bc68, 33 },
    { 0xd1d0bc69, 17 },
    { 0xd240bc6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc880bc6d, 61 },
    { 0, 0 },
    { 0xa30bc6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1030bc73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d50bc7c, 61 },
    { 0xa5a0bc7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e80bc83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d0bc86, 98 },
    { 0xcb30bc87, 98 },
    { 0x4c40bc87, 61 },
    { 0xea60bc87, 33 },
    { 0x5a80bc88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc0bc8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1340bc93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5460bc99, 96 },
    { 0, 0 },
    { 0x4590bc9b, 43 },
    { 0x1370bc9c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b90bc9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0bca2, 98 },
    { 0x7830bca3, 23 },
    { 0x6f10bca2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd780bcaa, 61 },
    { 0x2680bcaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3880bcae, 61 },
    { 0, 0 },
    { 0x530bcb0, 46 },
    { 0xd570bcb1, 33 },
    { 0xe6b0bcb1, 34 },
    { 0x1500bcb3, 46 },
    { 0x65c0bcb3, 94 },
    { 0, 0 },
    { 0x85a0bcb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb0bcb9, 88 },
    { 0xb630bcba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f0bcbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa010bcc3, 17 },
    { 0x9090bcc4, 34 },
    { 0xcc80bcc4, 34 },
    { 0x9450bcc6, 34 },
    { 0x3030bcc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0bcd3, 55 },
    { 0, 0 },
    { 0xe7b0bcd5, 95 },
    { 0xfee0bcd5, 93 },
    { 0x8220bcd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750bcda, 78 },
    { 0xe620bcdb, 33 },
    { 0, 0 },
    { 0x2880bcdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e0bce2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f0bce6, 75 },
    { 0x6fb0bce7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a0bcf1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba60bcf5, 76 },
    { 0, 0 },
    { 0xfe90bcf7, 98 },
    { 0xc250bcf8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad80bcff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f0bd11, 52 },
    { 0, 0 },
    { 0x3360bd13, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af0bd18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d0bd1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c0bd1f, 78 },
    { 0x9fa0bd1f, 90 },
    { 0x3740bd21, 93 },
    { 0, 0 },
    { 0x6df0bd23, 66 },
    { 0x540bd24, 61 },
    { 0x75e0bd25, 67 },
    { 0, 0 },
    { 0x78f0bd27, 84 },
    { 0, 0 },
    { 0xff10bd29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d0bd2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb650bd30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa40bd33, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd680bd37, 81 },
    { 0xaf0bd38, 98 },
    { 0xa980bd39, 95 },
    { 0xdeb0bd37, 42 },
    { 0, 0 },
    { 0xcf80bd3c, 90 },
    { 0, 0 },
    { 0x74c0bd3e, 91 },
    { 0xa20bd3f, 46 },
    { 0x5ae0bd3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd190bd44, 91 },
    { 0, 0 },
    { 0x5740bd46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a80bd4d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c60bd50, 96 },
    { 0, 0 },
    { 0xade0bd52, 33 },
    { 0, 0 },
    { 0xae50bd54, 43 },
    { 0xe560bd55, 80 },
    { 0, 0 },
    { 0x4d10bd57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd40bd5e, 90 },
    { 0, 0 },
    { 0x8d20bd60, 76 },
    { 0x9690bd61, 76 },
    { 0xda60bd60, 33 },
    { 0x4870bd63, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a0bd66, 55 },
    { 0x99b0bd66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e0bd6b, 88 },
    { 0xb5f0bd6b, 95 },
    { 0x3dd0bd6b, 61 },
    { 0, 0 },
    { 0x150bd6f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd800bd72, 33 },
    { 0x8f90bd73, 34 },
    { 0xb9e0bd74, 52 },
    { 0xc4a0bd75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6260bd7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae0bd80, 84 },
    { 0xb5d0bd81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9030bd84, 33 },
    { 0x4640bd84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa760bd88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780bd8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3370bd90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60bd96, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf670bd9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0bda4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f20bda7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd370bdab, 34 },
    { 0xa7d0bdac, 43 },
    { 0xee30bdad, 33 },
    { 0xfd10bdae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a20bdb2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb410bdb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e0bdc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b00bdc3, 34 },
    { 0, 0 },
    { 0x5d50bdc5, 98 },
    { 0xa9a0bdc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf290bdcc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac0bdd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e0bdda, 90 },
    { 0xf7e0bdda, 61 },
    { 0, 0 },
    { 0x88d0bddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a0bde0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6650bde4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e90bde7, 98 },
    { 0xc0bde8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20bdec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b0bdf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3820bdf3, 55 },
    { 0, 0 },
    { 0x8af0bdf5, 61 },
    { 0x9340bdf6, 76 },
    { 0x3ea0bdf7, 43 },
    { 0, 0 },
    { 0x37e0bdf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c90bdfc, 23 },
    { 0xb080bdfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3950be01, 61 },
    { 0x23c0be02, 90 },
    { 0, 0 },
    { 0x94d0be04, 98 },
    { 0x3500be05, 88 },
    { 0xff10be04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0be09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb0be12, 90 },
    { 0x38c0be13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4130be18, 76 },
    { 0, 0 },
    { 0xc120be1a, 33 },
    { 0x7770be1b, 81 },
    { 0x9d30be1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc60be20, 59 },
    { 0x9e10be20, 93 },
    { 0x34d0be22, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60be26, 34 },
    { 0x1a90be26, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a0be2d, 17 },
    { 0x9540be2e, 34 },
    { 0xf630be2d, 34 },
    { 0, 0 },
    { 0x30b0be31, 90 },
    { 0x24f0be31, 55 },
    { 0x8820be33, 98 },
    { 0xce90be32, 90 },
    { 0x7630be31, 90 },
    { 0x8310be36, 55 },
    { 0xce10be36, 61 },
    { 0xacc0be35, 98 },
    { 0x6db0be39, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf940be3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0be43, 46 },
    { 0xfb70be44, 34 },
    { 0xbcc0be43, 90 },
    { 0x20a0be45, 23 },
    { 0xa210be46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c40be4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb920be54, 52 },
    { 0x8fa0be54, 76 },
    { 0xd8f0be56, 23 },
    { 0xc5d0be57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a0be5a, 98 },
    { 0xef30be5b, 68 },
    { 0xd750be5b, 61 },
    { 0x8fd0be5a, 76 },
    { 0xcd30be5d, 33 },
    { 0x64d0be5b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5610be62, 95 },
    { 0x4c30be63, 99 },
    { 0xf340be64, 68 },
    { 0x7970be65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad0be6a, 33 },
    { 0x9d0be6a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd750be70, 66 },
    { 0xf670be71, 77 },
    { 0x3830be72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d0be7a, 90 },
    { 0x120be7b, 15 },
    { 0x28e0be7a, 43 },
    { 0, 0 },
    { 0xb9c0be7e, 91 },
    { 0x8d20be7f, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e0be84, 98 },
    { 0, 0 },
    { 0xf080be86, 90 },
    { 0xe780be87, 85 },
    { 0xbbf0be88, 76 },
    { 0, 0 },
    { 0xa550be8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f0be8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d0be91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x340be94, 98 },
    { 0, 0 },
    { 0x3a80be96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c0bea0, 61 },
    { 0, 0 },
    { 0xc010bea2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2780bea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d0bead, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d40beb0, 33 },
    { 0xc530beb1, 80 },
    { 0, 0 },
    { 0xc0c0beb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf0beb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd0beba, 76 },
    { 0x9e50bebb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd0bec1, 80 },
    { 0, 0 },
    { 0x7600bec3, 93 },
    { 0, 0 },
    { 0xde70bec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f40bec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd0becc, 77 },
    { 0x14e0becd, 76 },
    { 0x8d10bece, 87 },
    { 0x8da0becc, 91 },
    { 0x5b70bed0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd890bed5, 61 },
    { 0x18d0bed5, 33 },
    { 0x1e80bed6, 77 },
    { 0xc5a0bed8, 95 },
    { 0x89d0bed9, 95 },
    { 0x5a50beda, 91 },
    { 0x7300bed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940bedf, 43 },
    { 0xcaf0bee0, 33 },
    { 0x96d0bee1, 95 },
    { 0x8b10bee2, 55 },
    { 0, 0 },
    { 0xa9c0bee4, 98 },
    { 0xe5c0bee5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd0bee8, 98 },
    { 0, 0 },
    { 0x7100beea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec0beee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xba30bef1, 61 },
    { 0, 0 },
    { 0xf480bef3, 52 },
    { 0x5470bef3, 88 },
    { 0x4800bef5, 95 },
    { 0xf60bef5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b80beff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140bf02, 99 },
    { 0xe0b0bf03, 90 },
    { 0xd8c0bf04, 78 },
    { 0x3140bf05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed0bf08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390bf0d, 34 },
    { 0xb320bf0d, 100 },
    { 0xa40bf0f, 66 },
    { 0x51c0bf10, 78 },
    { 0, 0 },
    { 0x9d60bf12, 78 },
    { 0xb290bf12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e30bf1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a50bf20, 88 },
    { 0xdfe0bf21, 34 },
    { 0x8f10bf21, 99 },
    { 0xb370bf20, 33 },
    { 0xefa0bf24, 98 },
    { 0x32c0bf24, 66 },
    { 0, 0 },
    { 0x86f0bf27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8070bf2a, 84 },
    { 0, 0 },
    { 0x84a0bf2c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6540bf32, 84 },
    { 0x9070bf33, 90 },
    { 0, 0 },
    { 0x35e0bf35, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea0bf38, 75 },
    { 0xda10bf38, 93 },
    { 0x8d50bf39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b0bf3e, 65 },
    { 0, 0 },
    { 0xe5b0bf40, 90 },
    { 0xf1e0bf41, 90 },
    { 0xb1a0bf42, 34 },
    { 0x10b0bf41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e0bf47, 69 },
    { 0x50a0bf47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00bf4b, 90 },
    { 0, 0 },
    { 0x7590bf4d, 23 },
    { 0x1590bf4d, 76 },
    { 0xb2f0bf4d, 90 },
    { 0xddc0bf4d, 35 },
    { 0, 0 },
    { 0xac60bf52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad0bf58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640bf60, 98 },
    { 0x6460bf61, 33 },
    { 0, 0 },
    { 0x6810bf63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef0bf68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d30bf6b, 55 },
    { 0xc6e0bf6c, 90 },
    { 0x8a10bf6d, 34 },
    { 0xf380bf6e, 61 },
    { 0xf370bf6f, 95 },
    { 0x9000bf70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c0bf74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970bf78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1770bf7d, 33 },
    { 0x1d70bf7e, 66 },
    { 0x9d20bf7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f40bf88, 55 },
    { 0, 0 },
    { 0x85c0bf8a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9410bf8e, 90 },
    { 0xb3f0bf8f, 81 },
    { 0x4f20bf8f, 95 },
    { 0xd920bf90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e0bf94, 23 },
    { 0, 0 },
    { 0xf130bf96, 95 },
    { 0xcd40bf96, 23 },
    { 0xfb20bf97, 33 },
    { 0, 0 },
    { 0x6b30bf9a, 34 },
    { 0x52e0bf9b, 46 },
    { 0xb6f0bf9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e30bf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6880bfa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c0bfa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f0bfaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f0bfb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab10bfbc, 61 },
    { 0, 0 },
    { 0x1b80bfbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba0bfc1, 34 },
    { 0x6820bfc2, 95 },
    { 0x4440bfc3, 23 },
    { 0x83b0bfc4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc570bfcd, 52 },
    { 0xbf30bfcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b0bfd1, 94 },
    { 0, 0 },
    { 0x3860bfd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc60bfdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf990bfe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d0bfe3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab0bfe7, 46 },
    { 0xb630bfe8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c0bfeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed10bff1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8860bff4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9430bff7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d0bffc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2350bfff, 55 },
    { 0x5d80c000, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c0c009, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0c00c, 33 },
    { 0x81c0c00d, 97 },
    { 0x79a0c00e, 33 },
    { 0, 0 },
    { 0x8800c010, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5650c013, 61 },
    { 0xe3a0c014, 69 },
    { 0xa40c014, 95 },
    { 0x56f0c015, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca50c019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea40c01c, 90 },
    { 0xa570c01d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2370c023, 78 },
    { 0x30b0c023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e0c027, 98 },
    { 0x1030c027, 61 },
    { 0xa70c029, 100 },
    { 0xb560c02a, 55 },
    { 0x33a0c02a, 98 },
    { 0xe930c02b, 93 },
    { 0, 0 },
    { 0x870c02e, 34 },
    { 0x5550c02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5800c032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0c035, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa0c039, 23 },
    { 0x46f0c03a, 66 },
    { 0x8c30c03a, 78 },
    { 0, 0 },
    { 0x44b0c03d, 90 },
    { 0, 0 },
    { 0xdda0c03f, 61 },
    { 0x76b0c040, 98 },
    { 0, 0 },
    { 0x42d0c042, 95 },
    { 0x9cf0c043, 33 },
    { 0xee90c044, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8270c052, 43 },
    { 0x2d00c053, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8670c056, 76 },
    { 0x4960c057, 61 },
    { 0x6480c058, 98 },
    { 0, 0 },
    { 0x11f0c05a, 94 },
    { 0, 0 },
    { 0xf80c05c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a0c05f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a0c068, 95 },
    { 0x7c30c068, 95 },
    { 0x8ec0c06a, 90 },
    { 0x2530c06a, 68 },
    { 0x1c80c06c, 90 },
    { 0xecb0c06d, 100 },
    { 0x2fe0c06c, 61 },
    { 0, 0 },
    { 0x99a0c070, 43 },
    { 0x5c60c071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7130c07c, 55 },
    { 0xd720c07c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe950c080, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8670c087, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc540c08b, 90 },
    { 0x9940c08b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0c08f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7310c094, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2410c099, 90 },
    { 0x2590c09a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa300c09e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a80c0a1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a70c0a7, 55 },
    { 0x2c20c0a8, 66 },
    { 0x5000c0a9, 90 },
    { 0x9090c0aa, 95 },
    { 0xf9c0c0ab, 98 },
    { 0x3120c0ac, 87 },
    { 0x2580c0a9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4340c0b0, 17 },
    { 0, 0 },
    { 0x9e30c0b2, 95 },
    { 0x1e10c0b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce0c0b9, 43 },
    { 0x2030c0ba, 66 },
    { 0, 0 },
    { 0x7780c0bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad50c0bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe140c0c3, 46 },
    { 0xe8a0c0c4, 90 },
    { 0x7a50c0c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad80c0c9, 98 },
    { 0x7b50c0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc670c0d1, 90 },
    { 0x8620c0d2, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4950c0da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd770c0dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2430c0e2, 99 },
    { 0xf940c0e3, 70 },
    { 0xc080c0e4, 99 },
    { 0xad30c0e4, 33 },
    { 0x5810c0e6, 23 },
    { 0x7660c0e6, 99 },
    { 0x3070c0e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2990c0eb, 17 },
    { 0, 0 },
    { 0x8840c0ed, 55 },
    { 0, 0 },
    { 0x6f30c0ef, 76 },
    { 0, 0 },
    { 0x36b0c0f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7670c0f4, 55 },
    { 0xe610c0f5, 99 },
    { 0, 0 },
    { 0xae00c0f7, 61 },
    { 0, 0 },
    { 0x3c10c0f9, 95 },
    { 0x9ca0c0f9, 85 },
    { 0x85a0c0fb, 76 },
    { 0xfc70c0fc, 61 },
    { 0x7940c0f9, 90 },
    { 0xbb90c0fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd450c10e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf930c113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b0c118, 98 },
    { 0xf480c119, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e0c120, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c40c126, 95 },
    { 0x4b30c126, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20c12d, 99 },
    { 0x5890c12e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a0c131, 68 },
    { 0, 0 },
    { 0xb8b0c133, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d0c138, 98 },
    { 0, 0 },
    { 0xd450c13a, 90 },
    { 0x6cc0c13a, 75 },
    { 0, 0 },
    { 0xc8f0c13d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf0c147, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8230c14c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50c14f, 61 },
    { 0, 0 },
    { 0x8560c151, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60c157, 23 },
    { 0x2960c158, 66 },
    { 0x88f0c159, 61 },
    { 0xffa0c159, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670c15e, 46 },
    { 0xed00c15e, 90 },
    { 0x97e0c160, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80c163, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd0c16a, 78 },
    { 0, 0 },
    { 0x2020c16c, 65 },
    { 0xc980c16d, 66 },
    { 0x9760c16e, 87 },
    { 0x7a90c16e, 96 },
    { 0, 0 },
    { 0xf960c171, 23 },
    { 0x2e20c172, 99 },
    { 0x30c0c173, 61 },
    { 0xc280c174, 87 },
    { 0x2ea0c172, 93 },
    { 0xd9a0c174, 43 },
    { 0xe30c175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2390c17d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0c18e, 33 },
    { 0xe490c18e, 95 },
    { 0x5ce0c190, 91 },
    { 0xaa20c18f, 95 },
    { 0x8ba0c190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf760c195, 23 },
    { 0xd940c196, 46 },
    { 0, 0 },
    { 0xe340c198, 34 },
    { 0x1af0c199, 34 },
    { 0xc060c19a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a0c19d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d40c1aa, 61 },
    { 0, 0 },
    { 0xe00c1ac, 99 },
    { 0xf480c1ad, 90 },
    { 0x4e90c1ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf0c1b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250c1be, 33 },
    { 0x6e90c1bf, 23 },
    { 0x93d0c1bf, 90 },
    { 0xac30c1be, 61 },
    { 0, 0 },
    { 0x9ef0c1c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd630c1c6, 87 },
    { 0, 0 },
    { 0x70d0c1c8, 61 },
    { 0x9010c1c9, 33 },
    { 0x2330c1c8, 95 },
    { 0xcf00c1ca, 93 },
    { 0x4a40c1cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda20c1cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc10c1d2, 33 },
    { 0, 0 },
    { 0xd4e0c1d4, 33 },
    { 0, 0 },
    { 0x1260c1d6, 43 },
    { 0x96d0c1d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe520c1dc, 101 },
    { 0xd810c1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd100c1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a70c1e6, 46 },
    { 0, 0 },
    { 0xc0c0c1e8, 95 },
    { 0x9450c1e9, 17 },
    { 0, 0 },
    { 0x4a30c1eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df0c1ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a0c1f2, 55 },
    { 0x880c1f2, 23 },
    { 0xf8d0c1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf590c1f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f0c1fd, 55 },
    { 0, 0 },
    { 0x9f60c1ff, 61 },
    { 0, 0 },
    { 0x2360c201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1710c205, 90 },
    { 0x65d0c206, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b0c214, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa0c217, 95 },
    { 0x31c0c218, 61 },
    { 0, 0 },
    { 0x2aa0c21a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd0c21d, 33 },
    { 0x5c80c21e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f0c223, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c0c22b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550c22f, 43 },
    { 0xe5e0c22f, 99 },
    { 0x6bd0c231, 23 },
    { 0xf3f0c231, 61 },
    { 0x5560c233, 90 },
    { 0xf7f0c233, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7530c23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e0c23f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810c245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf610c249, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d90c24c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60c250, 23 },
    { 0, 0 },
    { 0xb000c252, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc20c255, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7810c258, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0c25c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a90c261, 98 },
    { 0, 0 },
    { 0x4960c263, 84 },
    { 0x1e20c264, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db0c268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2790c26f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a30c273, 90 },
    { 0x6a0c273, 78 },
    { 0x8380c275, 61 },
    { 0x7df0c276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c0c280, 76 },
    { 0xa5d0c281, 23 },
    { 0, 0 },
    { 0x85f0c283, 33 },
    { 0xd4e0c283, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa560c288, 23 },
    { 0xcbe0c289, 84 },
    { 0xbdd0c28a, 95 },
    { 0x1d0c28a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c0c28e, 77 },
    { 0x8a00c28f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd690c293, 66 },
    { 0x27b0c294, 33 },
    { 0, 0 },
    { 0x4940c296, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf870c299, 46 },
    { 0x6790c29a, 61 },
    { 0x3620c29a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb70c29e, 99 },
    { 0xcbe0c29e, 33 },
    { 0x1b60c29f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8840c2a4, 76 },
    { 0x6260c2a4, 33 },
    { 0x72c0c2a6, 76 },
    { 0xd10c2a6, 95 },
    { 0xcee0c2a8, 98 },
    { 0x66a0c2a9, 61 },
    { 0x8c10c2a6, 95 },
    { 0, 0 },
    { 0x8190c2ac, 95 },
    { 0x6cd0c2ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba80c2b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2660c2b5, 55 },
    { 0x7620c2b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b0c2bc, 33 },
    { 0x4a00c2bd, 81 },
    { 0x80d0c2be, 98 },
    { 0xebc0c2bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe880c2c7, 90 },
    { 0xc0d0c2c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa230c2d2, 95 },
    { 0, 0 },
    { 0x8160c2d4, 100 },
    { 0, 0 },
    { 0x9e80c2d6, 61 },
    { 0x6180c2d7, 61 },
    { 0xb0b0c2d8, 93 },
    { 0x8680c2d8, 95 },
    { 0x5310c2da, 78 },
    { 0x7cb0c2d9, 95 },
    { 0xa70c2dc, 95 },
    { 0x700c2dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4620c2e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa780c2e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b40c2e8, 23 },
    { 0xcdf0c2e9, 61 },
    { 0, 0 },
    { 0x5f70c2eb, 90 },
    { 0xc5d0c2ec, 65 },
    { 0xb7b0c2ed, 61 },
    { 0xe8d0c2ec, 76 },
    { 0, 0 },
    { 0x3a50c2f0, 77 },
    { 0xfad0c2f1, 34 },
    { 0, 0 },
    { 0xe440c2f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc680c2f7, 55 },
    { 0xa070c2f8, 33 },
    { 0xac50c2f8, 61 },
    { 0xeb40c2fa, 9 },
    { 0x70f0c2f8, 61 },
    { 0x280c2fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb40c303, 61 },
    { 0xb600c303, 90 },
    { 0x4760c305, 33 },
    { 0xbc50c306, 98 },
    { 0x9870c307, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7280c30a, 90 },
    { 0xdd0c30b, 46 },
    { 0xbe00c30c, 98 },
    { 0x4800c30d, 43 },
    { 0x8ba0c30c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a0c316, 95 },
    { 0x3280c317, 100 },
    { 0, 0 },
    { 0x3a70c319, 67 },
    { 0x2bd0c31a, 61 },
    { 0, 0 },
    { 0xde20c31c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e20c31f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba0c322, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d0c329, 90 },
    { 0x610c32a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0c32d, 36 },
    { 0, 0 },
    { 0x1640c32f, 90 },
    { 0, 0 },
    { 0xee20c331, 90 },
    { 0xb7d0c332, 98 },
    { 0, 0 },
    { 0x29a0c334, 71 },
    { 0, 0 },
    { 0x6f70c336, 98 },
    { 0xa860c337, 60 },
    { 0xff80c338, 93 },
    { 0xba80c339, 23 },
    { 0xa8b0c339, 23 },
    { 0xac90c336, 90 },
    { 0xea90c33c, 23 },
    { 0, 0 },
    { 0x5960c33e, 46 },
    { 0, 0 },
    { 0x2b90c340, 23 },
    { 0xed50c341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9060c348, 23 },
    { 0, 0 },
    { 0x9190c34a, 33 },
    { 0, 0 },
    { 0x4e80c34c, 85 },
    { 0, 0 },
    { 0x88e0c34e, 33 },
    { 0x6bf0c34f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd520c357, 52 },
    { 0, 0 },
    { 0x87d0c359, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d10c35c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7750c35f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9110c362, 55 },
    { 0xc150c363, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc660c366, 88 },
    { 0xb730c366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b0c36a, 90 },
    { 0xf360c36b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc0c373, 33 },
    { 0x90d0c374, 61 },
    { 0x60c374, 98 },
    { 0xb7c0c376, 95 },
    { 0x6110c376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e0c37b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800c388, 90 },
    { 0x15e0c389, 66 },
    { 0x8560c38a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa0c38e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b0c391, 23 },
    { 0x5080c392, 76 },
    { 0x8740c391, 87 },
    { 0x1850c394, 95 },
    { 0x5a50c394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10c39c, 51 },
    { 0x45e0c39c, 61 },
    { 0, 0 },
    { 0xc7e0c39f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4010c3a5, 98 },
    { 0xcb40c3a5, 87 },
    { 0x3a50c3a7, 87 },
    { 0x8840c3a8, 17 },
    { 0xa970c3a8, 66 },
    { 0xace0c3a8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9120c3ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd90c3b1, 90 },
    { 0x560c3b2, 70 },
    { 0xd590c3b3, 95 },
    { 0, 0 },
    { 0x3c30c3b5, 98 },
    { 0, 0 },
    { 0x6140c3b7, 90 },
    { 0x7240c3b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a0c3bb, 98 },
    { 0x88b0c3bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc0c3c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5170c3c3, 91 },
    { 0, 0 },
    { 0xcb20c3c5, 90 },
    { 0x65f0c3c6, 61 },
    { 0xa860c3c7, 98 },
    { 0xef10c3c7, 76 },
    { 0xca80c3c9, 40 },
    { 0xbea0c3c9, 90 },
    { 0x2ca0c3cb, 99 },
    { 0xf0e0c3cb, 98 },
    { 0x6f80c3cd, 61 },
    { 0xab90c3cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f0c3d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e50c3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5840c3e1, 95 },
    { 0, 0 },
    { 0x8cf0c3e3, 43 },
    { 0, 0 },
    { 0xa950c3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d0c3e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0c3ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc530c3f6, 65 },
    { 0, 0 },
    { 0xbc00c3f8, 78 },
    { 0xe9a0c3f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf70c401, 45 },
    { 0x3550c402, 93 },
    { 0xd980c402, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b0c406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2560c40b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0c410, 33 },
    { 0xf220c411, 90 },
    { 0, 0 },
    { 0x5aa0c413, 68 },
    { 0x330c413, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f60c41f, 66 },
    { 0x7e20c420, 23 },
    { 0x9460c420, 98 },
    { 0xba30c422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b0c428, 33 },
    { 0x9780c429, 55 },
    { 0xff0c428, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa0c436, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0c439, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6150c43e, 61 },
    { 0, 0 },
    { 0xa720c440, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xf840c443, 34 },
    { 0, 0 },
    { 0xb900c445, 43 },
    { 0x1d70c446, 23 },
    { 0, 0 },
    { 0x88b0c448, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0c44f, 90 },
    { 0, 0 },
    { 0x8f60c451, 33 },
    { 0x5350c452, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c0c455, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a20c458, 90 },
    { 0, 0 },
    { 0x9190c45a, 87 },
    { 0x8010c45b, 23 },
    { 0, 0 },
    { 0x6c90c45d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b10c463, 46 },
    { 0xeaf0c464, 33 },
    { 0, 0 },
    { 0xd350c466, 90 },
    { 0, 0 },
    { 0x5310c468, 78 },
    { 0xa130c469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc0c46e, 61 },
    { 0x35e0c46f, 95 },
    { 0xe510c470, 95 },
    { 0x8cf0c470, 91 },
    { 0x5810c470, 55 },
    { 0x8ef0c473, 33 },
    { 0x6490c474, 76 },
    { 0x8720c471, 55 },
    { 0xad50c476, 100 },
    { 0xf970c477, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4420c47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd70c480, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5310c486, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3210c48b, 84 },
    { 0, 0 },
    { 0x3620c48d, 91 },
    { 0, 0 },
    { 0x8370c48f, 34 },
    { 0, 0 },
    { 0x90d0c491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6340c495, 23 },
    { 0xe30c496, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5550c499, 90 },
    { 0, 0 },
    { 0x7980c49b, 33 },
    { 0x1000c49c, 61 },
    { 0x9540c49d, 76 },
    { 0, 0 },
    { 0xfe60c49f, 91 },
    { 0xd840c49f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60c4a6, 49 },
    { 0xa880c4a7, 61 },
    { 0x99c0c4a8, 23 },
    { 0x3680c4a9, 23 },
    { 0x5cf0c4aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4440c4b2, 61 },
    { 0x2660c4b2, 93 },
    { 0xd250c4b4, 78 },
    { 0x23c0c4b5, 55 },
    { 0, 0 },
    { 0x2800c4b7, 33 },
    { 0xd6c0c4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a90c4bd, 90 },
    { 0x6220c4be, 67 },
    { 0x6260c4bf, 95 },
    { 0xc1f0c4bd, 40 },
    { 0x5f0c4c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f0c4cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa0c4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b10c4db, 61 },
    { 0xc190c4dc, 61 },
    { 0, 0 },
    { 0xec40c4de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd190c4e1, 36 },
    { 0x99d0c4e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c0c4e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f0c4ea, 61 },
    { 0x9b00c4eb, 78 },
    { 0xaf00c4ec, 95 },
    { 0xaf80c4ed, 33 },
    { 0x2720c4ec, 98 },
    { 0xa320c4ef, 33 },
    { 0x66f0c4f0, 99 },
    { 0xa3e0c4f1, 78 },
    { 0x2af0c4f0, 23 },
    { 0xbc80c4f3, 23 },
    { 0, 0 },
    { 0xfc60c4f5, 33 },
    { 0xc380c4f6, 55 },
    { 0xcf50c4f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d0c4fb, 90 },
    { 0x3ef0c4fb, 33 },
    { 0x3d30c4fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b0c505, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee0c508, 90 },
    { 0, 0 },
    { 0xfb70c50a, 90 },
    { 0xd7c0c50b, 90 },
    { 0x82d0c50c, 95 },
    { 0x36d0c50d, 90 },
    { 0, 0 },
    { 0x8740c50f, 98 },
    { 0, 0 },
    { 0x1420c511, 27 },
    { 0xdbf0c512, 55 },
    { 0xa760c511, 61 },
    { 0x51d0c513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f0c521, 33 },
    { 0x55c0c521, 99 },
    { 0x1420c523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf40c526, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a00c529, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1850c538, 97 },
    { 0, 0 },
    { 0x2620c53a, 23 },
    { 0, 0 },
    { 0x3460c53c, 76 },
    { 0xf3e0c53d, 61 },
    { 0x8c00c53e, 33 },
    { 0xbb80c53c, 98 },
    { 0, 0 },
    { 0xd180c541, 93 },
    { 0, 0 },
    { 0x3650c543, 66 },
    { 0, 0 },
    { 0xaaf0c545, 33 },
    { 0x6ba0c546, 87 },
    { 0x2da0c547, 61 },
    { 0x2a20c548, 90 },
    { 0x2810c545, 34 },
    { 0x8a60c54a, 90 },
    { 0x8990c54b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4110c54e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d0c557, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd040c55b, 68 },
    { 0x3e90c55b, 84 },
    { 0, 0 },
    { 0xd50c55e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d0c561, 66 },
    { 0xa750c562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1050c56b, 90 },
    { 0, 0 },
    { 0xdd90c56d, 61 },
    { 0xb0c0c56e, 93 },
    { 0x2900c56d, 33 },
    { 0xae00c56e, 90 },
    { 0x910c571, 23 },
    { 0xa550c56d, 90 },
    { 0x960c573, 61 },
    { 0x46d0c573, 98 },
    { 0x8f30c575, 65 },
    { 0x89e0c576, 55 },
    { 0xf2d0c577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc0c580, 90 },
    { 0xbac0c580, 46 },
    { 0x9420c581, 46 },
    { 0x4fb0c583, 23 },
    { 0x7820c581, 96 },
    { 0xb850c585, 43 },
    { 0x8610c585, 61 },
    { 0xd250c580, 98 },
    { 0x3140c588, 90 },
    { 0x3ed0c586, 93 },
    { 0x9510c587, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f0c58e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb890c591, 87 },
    { 0x1770c591, 95 },
    { 0xca00c593, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b70c59a, 90 },
    { 0xdc80c59a, 76 },
    { 0x6ae0c59c, 61 },
    { 0xede0c59c, 98 },
    { 0x2a90c59d, 100 },
    { 0xf150c59c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd770c5a3, 76 },
    { 0xa30c5a3, 23 },
    { 0xa9e0c5a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0c5a8, 81 },
    { 0x7080c5a8, 90 },
    { 0x2810c5aa, 23 },
    { 0x99d0c5ab, 95 },
    { 0x3430c5ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2280c5af, 98 },
    { 0xc9f0c5b0, 87 },
    { 0xea30c5af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf0c5b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f0c5ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa230c5c1, 34 },
    { 0, 0 },
    { 0x17c0c5c3, 66 },
    { 0xf7e0c5c4, 43 },
    { 0xc690c5c5, 94 },
    { 0xff20c5c6, 68 },
    { 0x7d40c5c6, 66 },
    { 0xb10c5c7, 61 },
    { 0xe720c5c6, 95 },
    { 0xa900c5c6, 55 },
    { 0x99d0c5c4, 85 },
    { 0x1eb0c5cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc520c5d2, 43 },
    { 0xd7d0c5d2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca0c5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf30c5e2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a0c5e8, 75 },
    { 0x2d80c5e9, 76 },
    { 0, 0 },
    { 0xc800c5eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa890c5f0, 98 },
    { 0x8720c5f1, 33 },
    { 0xefb0c5f2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d50c5f5, 98 },
    { 0xe000c5f6, 95 },
    { 0, 0 },
    { 0xa8a0c5f8, 33 },
    { 0, 0 },
    { 0xfa60c5fa, 33 },
    { 0xbbe0c5fb, 61 },
    { 0x52d0c5fc, 99 },
    { 0x4890c5fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded0c602, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe000c605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb60c609, 61 },
    { 0x37c0c609, 87 },
    { 0, 0 },
    { 0x3390c60c, 23 },
    { 0x1570c60d, 43 },
    { 0x4700c60c, 90 },
    { 0xe570c60f, 33 },
    { 0x6460c610, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120c614, 43 },
    { 0x9700c615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b0c619, 90 },
    { 0, 0 },
    { 0x48c0c61b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670c621, 95 },
    { 0x7180c622, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8910c625, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630c629, 19 },
    { 0xd760c62a, 34 },
    { 0x1890c629, 90 },
    { 0x5ec0c62c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0c62f, 23 },
    { 0xbb20c62f, 61 },
    { 0xf780c62f, 61 },
    { 0xbad0c631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d20c635, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7150c639, 98 },
    { 0x2850c63a, 61 },
    { 0, 0 },
    { 0x5ee0c63c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0c640, 43 },
    { 0, 0 },
    { 0xd910c642, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d0c645, 61 },
    { 0, 0 },
    { 0x8720c647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60c64c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3710c650, 66 },
    { 0x2c30c651, 46 },
    { 0, 0 },
    { 0x9470c653, 98 },
    { 0x8290c654, 98 },
    { 0xf280c655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c65b, 23 },
    { 0x4050c65b, 87 },
    { 0xd6d0c65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf080c660, 64 },
    { 0x8fd0c660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b10c665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710c669, 61 },
    { 0x1900c66a, 99 },
    { 0x2b90c669, 33 },
    { 0x4f50c66c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e80c670, 33 },
    { 0, 0 },
    { 0x63b0c672, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b0c679, 90 },
    { 0x44c0c67a, 85 },
    { 0xc600c679, 17 },
    { 0, 0 },
    { 0xfe80c67d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc240c682, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5190c685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d0c68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d0c690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c0c697, 43 },
    { 0x55c0c698, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd920c69c, 88 },
    { 0x8c20c69d, 52 },
    { 0xc110c69d, 23 },
    { 0, 0 },
    { 0xd320c6a0, 23 },
    { 0x94f0c6a1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9200c6a6, 90 },
    { 0x9de0c6a7, 85 },
    { 0xfd0c6a8, 90 },
    { 0x1bc0c6a6, 75 },
    { 0x770c6aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc320c6ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b0c6b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a0c6b6, 37 },
    { 0, 0 },
    { 0x5a50c6b8, 53 },
    { 0xe330c6b8, 23 },
    { 0x7490c6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4910c6be, 33 },
    { 0xc60c6bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd0c6c9, 96 },
    { 0xc8b0c6c9, 84 },
    { 0, 0 },
    { 0x9010c6cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1720c6cf, 34 },
    { 0x9e90c6cf, 85 },
    { 0x19f0c6d1, 23 },
    { 0x4550c6d1, 98 },
    { 0xc340c6d1, 43 },
    { 0xc320c6d2, 33 },
    { 0x8030c6d5, 61 },
    { 0x57b0c6d6, 90 },
    { 0x5440c6d6, 66 },
    { 0xe990c6d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6100c6db, 90 },
    { 0, 0 },
    { 0x9de0c6dd, 68 },
    { 0, 0 },
    { 0x2b20c6df, 90 },
    { 0xed80c6e0, 97 },
    { 0, 0 },
    { 0x250c6e2, 95 },
    { 0x23c0c6e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6440c6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0c6ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b0c6ed, 76 },
    { 0x7db0c6ed, 34 },
    { 0x110c6ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d0c6f5, 88 },
    { 0x2920c6f5, 95 },
    { 0, 0 },
    { 0x9e10c6f8, 46 },
    { 0x6630c6f9, 23 },
    { 0xd800c6fa, 61 },
    { 0x6f40c6fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc670c700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5640c703, 95 },
    { 0x19b0c704, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe0c70a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e0c70e, 85 },
    { 0x2540c70e, 90 },
    { 0, 0 },
    { 0xba90c711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a40c714, 34 },
    { 0, 0 },
    { 0xe4a0c716, 46 },
    { 0xdb40c716, 95 },
    { 0xe220c718, 61 },
    { 0x1760c719, 61 },
    { 0, 0 },
    { 0x1670c71b, 61 },
    { 0xe9a0c71c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490c722, 101 },
    { 0x44c0c722, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2830c728, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd0c72c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f30c72f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0c733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea0c736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9010c73f, 95 },
    { 0x1920c740, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410c743, 17 },
    { 0xcd0c744, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac40c74a, 33 },
    { 0xf6b0c74a, 95 },
    { 0, 0 },
    { 0xaea0c74d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220c757, 33 },
    { 0x1830c758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c30c75c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1020c75f, 61 },
    { 0xab50c760, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf650c764, 95 },
    { 0xefc0c765, 98 },
    { 0x3e70c764, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b0c76a, 55 },
    { 0xec90c76b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4210c771, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da0c776, 55 },
    { 0xd5a0c777, 98 },
    { 0xd520c778, 84 },
    { 0x9490c776, 93 },
    { 0x1b80c776, 76 },
    { 0x7060c77b, 85 },
    { 0x1a20c77c, 67 },
    { 0xbc20c77b, 95 },
    { 0xc420c77b, 95 },
    { 0xe960c77f, 23 },
    { 0x9dc0c77c, 44 },
    { 0, 0 },
    { 0x7990c782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c78c, 66 },
    { 0x8c30c78c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f0c790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f70c794, 90 },
    { 0, 0 },
    { 0x4ca0c796, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4860c799, 90 },
    { 0, 0 },
    { 0x5350c79b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a0c79f, 61 },
    { 0xa130c7a0, 90 },
    { 0xd750c7a1, 43 },
    { 0x9a60c7a2, 43 },
    { 0xd340c7a3, 76 },
    { 0x9d20c7a4, 84 },
    { 0xc9d0c7a5, 55 },
    { 0xea50c7a6, 98 },
    { 0x5b80c7a5, 33 },
    { 0x75e0c7a4, 61 },
    { 0xbe0c7a9, 23 },
    { 0xe00c7a9, 74 },
    { 0, 0 },
    { 0x2ff0c7ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d0c7af, 98 },
    { 0x640c7af, 33 },
    { 0, 0 },
    { 0x2b10c7b2, 99 },
    { 0x7750c7b3, 33 },
    { 0x3800c7b3, 66 },
    { 0x9610c7b2, 95 },
    { 0, 0 },
    { 0x1150c7b7, 67 },
    { 0x770c7b8, 92 },
    { 0, 0 },
    { 0xe380c7ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7540c7c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb90c7c3, 98 },
    { 0, 0 },
    { 0x1930c7c5, 34 },
    { 0xac20c7c6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280c7c9, 61 },
    { 0xe2f0c7ca, 23 },
    { 0x7d40c7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6920c7d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1160c7da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee0c7e1, 61 },
    { 0x6410c7e1, 78 },
    { 0x5670c7e3, 90 },
    { 0xa80c7e4, 85 },
    { 0xcb70c7e3, 67 },
    { 0x6f40c7e2, 33 },
    { 0x2e50c7e7, 78 },
    { 0, 0 },
    { 0xfd00c7e9, 76 },
    { 0x9f10c7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6350c7ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5480c7f1, 34 },
    { 0, 0 },
    { 0x9f50c7f3, 69 },
    { 0x3fb0c7f4, 93 },
    { 0x9ea0c7f5, 78 },
    { 0xd230c7f4, 65 },
    { 0x94a0c7f7, 90 },
    { 0x8e60c7f8, 17 },
    { 0x17c0c7f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff10c7fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc0c802, 98 },
    { 0, 0 },
    { 0x72e0c804, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a0c80e, 78 },
    { 0x58c0c80e, 23 },
    { 0x5bd0c80f, 98 },
    { 0x48c0c811, 69 },
    { 0, 0 },
    { 0xa3c0c813, 98 },
    { 0xd1e0c814, 95 },
    { 0, 0 },
    { 0xe110c816, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a0c81c, 67 },
    { 0x2340c81d, 33 },
    { 0xdc10c81e, 23 },
    { 0x7d80c81f, 43 },
    { 0x6c50c820, 90 },
    { 0xe040c81c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4570c824, 61 },
    { 0x5280c824, 33 },
    { 0x73d0c824, 90 },
    { 0, 0 },
    { 0xa200c828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4330c82c, 90 },
    { 0x1f50c82d, 95 },
    { 0x2d70c82d, 98 },
    { 0xccc0c82f, 98 },
    { 0xb330c82f, 98 },
    { 0x7940c831, 90 },
    { 0xd090c832, 33 },
    { 0xcc0c82c, 98 },
    { 0x75f0c831, 46 },
    { 0x3610c82e, 33 },
    { 0, 0 },
    { 0x490c837, 61 },
    { 0, 0 },
    { 0x9c20c839, 34 },
    { 0, 0 },
    { 0xd9c0c83b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7520c83f, 33 },
    { 0, 0 },
    { 0xc8d0c841, 52 },
    { 0x79e0c842, 90 },
    { 0x630c843, 85 },
    { 0, 0 },
    { 0xd800c845, 33 },
    { 0x7020c845, 81 },
    { 0x26a0c846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf260c84a, 90 },
    { 0xbb10c84b, 34 },
    { 0x8bc0c84b, 23 },
    { 0, 0 },
    { 0x8260c84e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8920c855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270c858, 33 },
    { 0x5630c859, 55 },
    { 0x5290c85a, 34 },
    { 0x8610c85b, 19 },
    { 0, 0 },
    { 0x9e60c85d, 98 },
    { 0xe630c85e, 90 },
    { 0x3960c85d, 90 },
    { 0xaeb0c85f, 23 },
    { 0xcc70c85e, 53 },
    { 0, 0 },
    { 0xa8c0c863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf30c867, 66 },
    { 0xe240c867, 34 },
    { 0x8e50c869, 88 },
    { 0, 0 },
    { 0x9760c86b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd310c874, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5330c877, 61 },
    { 0, 0 },
    { 0x1320c879, 53 },
    { 0, 0 },
    { 0x3680c87b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4610c880, 33 },
    { 0xa5a0c880, 95 },
    { 0, 0 },
    { 0xeb30c883, 77 },
    { 0, 0 },
    { 0xd760c885, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd620c888, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9320c88c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1580c88f, 69 },
    { 0x7a60c890, 91 },
    { 0xf6e0c891, 23 },
    { 0x6120c88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c0c895, 46 },
    { 0, 0 },
    { 0x2a70c897, 100 },
    { 0, 0 },
    { 0x9f40c899, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d00c89e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf0c8a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a10c8a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0c8a7, 98 },
    { 0, 0 },
    { 0x66d0c8a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde00c8b0, 23 },
    { 0, 0 },
    { 0x5c30c8b2, 91 },
    { 0, 0 },
    { 0x14d0c8b4, 61 },
    { 0x42f0c8b4, 66 },
    { 0xd250c8b6, 90 },
    { 0xb5f0c8b7, 95 },
    { 0x2820c8b8, 36 },
    { 0xe320c8b8, 76 },
    { 0x58e0c8b7, 95 },
    { 0x15d0c8b7, 90 },
    { 0x2820c8bc, 95 },
    { 0x2bf0c8b9, 90 },
    { 0xe1d0c8b5, 66 },
    { 0xbcf0c8b9, 80 },
    { 0x8a00c8c0, 33 },
    { 0x62e0c8ba, 93 },
    { 0x8f90c8be, 75 },
    { 0xf940c8c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb0c8c7, 43 },
    { 0, 0 },
    { 0x2d0c8c9, 33 },
    { 0, 0 },
    { 0xcc50c8cb, 33 },
    { 0, 0 },
    { 0x1100c8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae0c8d1, 81 },
    { 0x1020c8d2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf310c8d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa0c8d9, 76 },
    { 0x6e80c8d9, 95 },
    { 0, 0 },
    { 0xd190c8dc, 43 },
    { 0xcef0c8dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc0c8e1, 55 },
    { 0xc410c8e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60c8e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd010c8eb, 100 },
    { 0, 0 },
    { 0xfd50c8ed, 98 },
    { 0xbcd0c8ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe0c8f3, 61 },
    { 0, 0 },
    { 0x22e0c8f5, 61 },
    { 0x7d20c8f6, 33 },
    { 0x4910c8f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c0c8fd, 61 },
    { 0x8f10c8fd, 33 },
    { 0, 0 },
    { 0x5f70c900, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7970c904, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb10c907, 81 },
    { 0xf390c907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80c90b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10c910, 76 },
    { 0x97b0c911, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f0c916, 36 },
    { 0x36f0c917, 34 },
    { 0xf8b0c916, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670c91c, 76 },
    { 0x9390c91c, 55 },
    { 0x52e0c91e, 33 },
    { 0xafb0c91d, 95 },
    { 0, 0 },
    { 0xe580c921, 46 },
    { 0x61b0c922, 93 },
    { 0xb750c923, 76 },
    { 0x2d00c924, 90 },
    { 0, 0 },
    { 0xccc0c926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c20c92e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a60c931, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2490c93a, 95 },
    { 0, 0 },
    { 0xfe30c93c, 61 },
    { 0, 0 },
    { 0xc130c93e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd510c942, 94 },
    { 0x9aa0c943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe000c94d, 76 },
    { 0x88a0c94e, 93 },
    { 0xe1a0c94f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca00c954, 61 },
    { 0x3db0c955, 76 },
    { 0x300c956, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f40c959, 61 },
    { 0xead0c95a, 52 },
    { 0xb4e0c95b, 81 },
    { 0, 0 },
    { 0x800c95d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e0c960, 95 },
    { 0xcf80c961, 81 },
    { 0xcb20c961, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc80c967, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb0c96d, 46 },
    { 0x6a30c96e, 55 },
    { 0xdd40c96f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd0c975, 33 },
    { 0x9550c976, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f0c97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd0c980, 90 },
    { 0, 0 },
    { 0x2bb0c982, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de0c98a, 95 },
    { 0x3b50c98b, 33 },
    { 0xcef0c98a, 98 },
    { 0xc000c98b, 98 },
    { 0xbce0c98e, 55 },
    { 0, 0 },
    { 0xdba0c990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe290c993, 43 },
    { 0, 0 },
    { 0x92e0c995, 33 },
    { 0x2970c996, 61 },
    { 0xbcf0c997, 33 },
    { 0xdb60c997, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0c99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c0c99f, 93 },
    { 0, 0 },
    { 0x4fc0c9a1, 33 },
    { 0, 0 },
    { 0x3f40c9a3, 61 },
    { 0xeb10c9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb0c9a8, 23 },
    { 0, 0 },
    { 0x3ac0c9aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae40c9af, 98 },
    { 0x9ae0c9af, 90 },
    { 0x4900c9b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d0c9b8, 91 },
    { 0xda30c9b9, 90 },
    { 0, 0 },
    { 0x5650c9bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb0c9bf, 17 },
    { 0xeb00c9c0, 61 },
    { 0xff10c9bf, 36 },
    { 0x22c0c9bf, 33 },
    { 0x6810c9c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a0c9c6, 90 },
    { 0x4880c9c7, 23 },
    { 0, 0 },
    { 0x9d80c9c9, 17 },
    { 0x39f0c9c9, 52 },
    { 0, 0 },
    { 0xb7f0c9cc, 98 },
    { 0xca80c9cd, 95 },
    { 0, 0 },
    { 0x41e0c9cf, 98 },
    { 0x8350c9d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100c9d5, 76 },
    { 0, 0 },
    { 0x34b0c9d7, 55 },
    { 0xcf20c9d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1210c9dc, 76 },
    { 0xd8b0c9dd, 66 },
    { 0, 0 },
    { 0xffa0c9df, 95 },
    { 0x2e50c9e0, 52 },
    { 0, 0 },
    { 0xc50c9e2, 55 },
    { 0x31c0c9e3, 90 },
    { 0xd070c9e4, 55 },
    { 0x95e0c9e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe720c9eb, 98 },
    { 0, 0 },
    { 0xb0c0c9ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4050c9f1, 66 },
    { 0xe040c9f2, 33 },
    { 0x8480c9f3, 90 },
    { 0xfc00c9f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40c9f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee60c9fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b0c9ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa0ca02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce0ca05, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d0ca0a, 90 },
    { 0xf080ca0a, 99 },
    { 0x1220ca0c, 33 },
    { 0xb1f0ca0d, 98 },
    { 0xed10ca0e, 33 },
    { 0x7bc0ca0f, 34 },
    { 0, 0 },
    { 0x6420ca11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1050ca19, 61 },
    { 0x8190ca1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf300ca25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd0ca28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f0ca2c, 95 },
    { 0, 0 },
    { 0x8830ca2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f00ca31, 61 },
    { 0x10b0ca31, 17 },
    { 0xd3e0ca32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf0ca36, 84 },
    { 0, 0 },
    { 0x23c0ca38, 90 },
    { 0, 0 },
    { 0x8440ca3a, 84 },
    { 0x1bc0ca3a, 61 },
    { 0x1a90ca3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4550ca41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf90ca50, 78 },
    { 0xd540ca51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x830ca54, 76 },
    { 0, 0 },
    { 0xb280ca56, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b0ca5a, 90 },
    { 0x85c0ca5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690ca60, 84 },
    { 0x42e0ca61, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0ca64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5790ca67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc0ca6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f0ca6d, 98 },
    { 0x6ab0ca6e, 80 },
    { 0, 0 },
    { 0xc30ca70, 33 },
    { 0, 0 },
    { 0x8480ca72, 95 },
    { 0, 0 },
    { 0xc750ca74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060ca79, 97 },
    { 0xf650ca7a, 34 },
    { 0x6db0ca7b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1980ca80, 88 },
    { 0, 0 },
    { 0x4040ca82, 67 },
    { 0x1950ca83, 90 },
    { 0x5730ca84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8640ca8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b0ca93, 61 },
    { 0x28e0ca93, 87 },
    { 0xc50ca94, 87 },
    { 0, 0 },
    { 0x4de0ca97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20ca9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc240ca9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e0caa4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa580caa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb320caab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0cab1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b30cab6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f0cab9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae0cabd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f0cac0, 93 },
    { 0, 0 },
    { 0xeb00cac2, 68 },
    { 0xc3b0cac3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9890cac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80caca, 52 },
    { 0xd780cacb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce60cad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc420cad9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c0cade, 95 },
    { 0, 0 },
    { 0x7140cae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb70cae4, 90 },
    { 0xb980cae5, 66 },
    { 0xb310cae5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70caea, 33 },
    { 0xdf0caeb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a0caee, 78 },
    { 0, 0 },
    { 0x7c80caf0, 23 },
    { 0, 0 },
    { 0x14b0caf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d50caf5, 23 },
    { 0x4440caf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060cafb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6110caff, 61 },
    { 0x5220cb00, 95 },
    { 0x1a20cb01, 34 },
    { 0x1340cb02, 90 },
    { 0x31b0cb03, 90 },
    { 0x68c0cb02, 33 },
    { 0x8fe0cb05, 23 },
    { 0x1870cb06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e0cb0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e60cb10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f0cb13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da0cb16, 99 },
    { 0, 0 },
    { 0x3a30cb18, 33 },
    { 0, 0 },
    { 0x1af0cb1a, 98 },
    { 0xb840cb1b, 95 },
    { 0xbb60cb1c, 61 },
    { 0x32e0cb1d, 76 },
    { 0, 0 },
    { 0x4690cb1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d0cb22, 87 },
    { 0x6910cb23, 85 },
    { 0x3cd0cb24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810cb29, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a10cb2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe330cb32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0cb36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e0cb3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe590cb3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c0cb41, 33 },
    { 0, 0 },
    { 0x8840cb43, 61 },
    { 0xf0b0cb43, 55 },
    { 0xf30cb45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6900cb48, 98 },
    { 0, 0 },
    { 0xd760cb4a, 61 },
    { 0, 0 },
    { 0x16c0cb4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2640cb52, 95 },
    { 0xd360cb53, 90 },
    { 0x6350cb53, 95 },
    { 0, 0 },
    { 0x1150cb56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf0cb5a, 65 },
    { 0, 0 },
    { 0xc7f0cb5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1330cb60, 55 },
    { 0x4290cb61, 34 },
    { 0x4080cb60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4860cb67, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb890cb6b, 95 },
    { 0xf4e0cb6b, 33 },
    { 0xf230cb6c, 98 },
    { 0x87e0cb6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570cb71, 45 },
    { 0x9ca0cb72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960cb76, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9870cb79, 23 },
    { 0, 0 },
    { 0x9590cb7b, 90 },
    { 0xb610cb7c, 78 },
    { 0x2470cb7d, 76 },
    { 0x4140cb7b, 95 },
    { 0x4070cb7f, 61 },
    { 0x20f0cb7c, 96 },
    { 0x2d60cb81, 23 },
    { 0, 0 },
    { 0x59c0cb83, 90 },
    { 0x2410cb83, 91 },
    { 0xb990cb83, 22 },
    { 0xe8a0cb86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e0cb8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb150cb93, 88 },
    { 0x5130cb93, 33 },
    { 0x4e10cb95, 93 },
    { 0x2d40cb96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df0cb9c, 61 },
    { 0xaa30cb9d, 95 },
    { 0, 0 },
    { 0xe450cb9f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d70cba3, 90 },
    { 0x7e80cba3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e0cba7, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db0cbab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x300cbae, 98 },
    { 0x49c0cbaf, 75 },
    { 0, 0 },
    { 0x72b0cbb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d0cbb4, 61 },
    { 0x4a00cbb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3270cbba, 98 },
    { 0x7c0cbbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad80cbbf, 61 },
    { 0x5170cbbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b0cbce, 52 },
    { 0xa290cbcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc0cbd3, 93 },
    { 0x5b40cbd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c0cbdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a0cbe1, 85 },
    { 0xb9f0cbe2, 47 },
    { 0x23c0cbe3, 95 },
    { 0x31c0cbe2, 33 },
    { 0x7b60cbe1, 81 },
    { 0xcc20cbe6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7150cbeb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e50cbf2, 98 },
    { 0, 0 },
    { 0x76a0cbf4, 95 },
    { 0, 0 },
    { 0xd5a0cbf6, 61 },
    { 0x6340cbf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb0cbfc, 76 },
    { 0x26c0cbfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f80cc00, 98 },
    { 0, 0 },
    { 0xdef0cc02, 33 },
    { 0xe4e0cc03, 88 },
    { 0x5890cc04, 98 },
    { 0xae50cc03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb0cc10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b80cc19, 33 },
    { 0xc670cc19, 46 },
    { 0x18f0cc1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e50cc1e, 98 },
    { 0xdc70cc1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b20cc24, 90 },
    { 0, 0 },
    { 0x5890cc26, 46 },
    { 0x7670cc27, 95 },
    { 0x4be0cc26, 68 },
    { 0xc820cc29, 33 },
    { 0, 0 },
    { 0x5a10cc2b, 77 },
    { 0xe3d0cc2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e50cc2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc290cc33, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3970cc36, 90 },
    { 0x4a40cc37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f70cc3c, 81 },
    { 0x7b60cc3c, 23 },
    { 0x3900cc3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0cc41, 43 },
    { 0x6ed0cc41, 84 },
    { 0, 0 },
    { 0xdc00cc44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b0cc47, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630cc4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410cc52, 98 },
    { 0x9100cc53, 90 },
    { 0x7f00cc53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf000cc59, 98 },
    { 0x5f00cc59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc90cc5e, 33 },
    { 0x8bd0cc5e, 52 },
    { 0xdf40cc60, 87 },
    { 0xd030cc61, 23 },
    { 0x320cc62, 33 },
    { 0, 0 },
    { 0x1fd0cc64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b0cc68, 90 },
    { 0xdb00cc69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa80cc77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250cc7b, 46 },
    { 0xe320cc7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a40cc85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e0cc88, 36 },
    { 0xb640cc89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa640cc8c, 34 },
    { 0, 0 },
    { 0x2970cc8e, 33 },
    { 0xe00cc8f, 33 },
    { 0, 0 },
    { 0xdc30cc91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c0cc94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e0cc97, 33 },
    { 0, 0 },
    { 0x4b00cc99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9330cc9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d0cca1, 61 },
    { 0, 0 },
    { 0x7010cca3, 61 },
    { 0xd100cca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2090cca9, 81 },
    { 0xa910cca9, 61 },
    { 0xb0b0ccab, 90 },
    { 0, 0 },
    { 0xc240ccad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0ccb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8660ccb7, 95 },
    { 0x7590ccb8, 95 },
    { 0, 0 },
    { 0x9fe0ccba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe60ccc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d0ccc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f0ccc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2680ccd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2740ccd8, 84 },
    { 0x5a60ccd9, 23 },
    { 0, 0 },
    { 0xea10ccdb, 34 },
    { 0x83b0ccdc, 90 },
    { 0, 0 },
    { 0x8ac0ccde, 98 },
    { 0, 0 },
    { 0xc2f0cce0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3240cce3, 87 },
    { 0, 0 },
    { 0xaab0cce5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1700ccf2, 23 },
    { 0xf9d0ccf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff0ccff, 69 },
    { 0xd9c0ccff, 87 },
    { 0, 0 },
    { 0x4e0cd02, 98 },
    { 0x8030cd02, 52 },
    { 0xc540cd04, 61 },
    { 0xa0cd04, 90 },
    { 0xb0e0cd06, 95 },
    { 0x9a30cd07, 98 },
    { 0, 0 },
    { 0xb510cd09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc830cd0c, 23 },
    { 0xd7a0cd0d, 61 },
    { 0x9eb0cd0e, 33 },
    { 0x3480cd0f, 90 },
    { 0x39a0cd10, 33 },
    { 0xc3f0cd10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c0cd18, 53 },
    { 0x62f0cd18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b0cd1e, 43 },
    { 0xc40cd1f, 23 },
    { 0x3810cd1f, 81 },
    { 0x99d0cd21, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd0cd24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e10cd29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa0cd31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d30cd34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4930cd39, 68 },
    { 0x5430cd39, 55 },
    { 0, 0 },
    { 0xb0a0cd3c, 90 },
    { 0, 0 },
    { 0xf470cd3e, 33 },
    { 0x40f0cd3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae10cd4a, 91 },
    { 0x71a0cd4a, 90 },
    { 0x6020cd4c, 95 },
    { 0x9ae0cd4d, 93 },
    { 0xc710cd4e, 90 },
    { 0xe2b0cd4f, 61 },
    { 0x41a0cd4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb20cd53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe320cd57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee0cd5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0cd5e, 90 },
    { 0, 0 },
    { 0x5410cd60, 93 },
    { 0x6b0cd60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8070cd64, 33 },
    { 0xc010cd65, 61 },
    { 0xd030cd66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d0cd6b, 90 },
    { 0x7180cd6c, 98 },
    { 0x9a20cd6c, 98 },
    { 0xeb40cd6c, 61 },
    { 0xdfe0cd6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce30cd79, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c30cd7d, 99 },
    { 0xab0cd7e, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd0cd82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4970cd86, 66 },
    { 0, 0 },
    { 0x1a20cd88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7700cd8b, 46 },
    { 0x1a10cd8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb260cd8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef70cd93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd40cd96, 99 },
    { 0x1d10cd97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8850cd9c, 99 },
    { 0x9740cd9c, 23 },
    { 0x89e0cd9e, 76 },
    { 0x51b0cd9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330cda3, 76 },
    { 0xcd20cda4, 33 },
    { 0x1150cda4, 98 },
    { 0x3ec0cda3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5740cda9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6000cdac, 90 },
    { 0xb960cdad, 61 },
    { 0xce00cdae, 90 },
    { 0xd3d0cdaf, 93 },
    { 0, 0 },
    { 0x9bd0cdb1, 61 },
    { 0x1520cdb2, 80 },
    { 0xace0cdb3, 90 },
    { 0xa410cdb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0cdb9, 90 },
    { 0x3190cdba, 61 },
    { 0xf350cdbb, 98 },
    { 0, 0 },
    { 0xbdd0cdbd, 95 },
    { 0x87b0cdbd, 61 },
    { 0, 0 },
    { 0xddd0cdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd990cdc4, 76 },
    { 0x4600cdc4, 77 },
    { 0xd930cdc5, 33 },
    { 0x1b70cdc6, 43 },
    { 0, 0 },
    { 0xc460cdc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd120cdce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c0cdd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3120cdd9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c80cddd, 87 },
    { 0xcae0cdde, 91 },
    { 0x6420cddd, 47 },
    { 0x4b0cddd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4750cde5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd0cdee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e0cdf3, 75 },
    { 0, 0 },
    { 0x1a60cdf5, 84 },
    { 0, 0 },
    { 0xe720cdf7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa950cdfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e0cdfe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3850ce04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c0ce0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0ce13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9690ce1b, 61 },
    { 0x5f30ce1c, 81 },
    { 0xad90ce1d, 93 },
    { 0x9e60ce1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b0ce21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e0ce27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db0ce2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e0ce32, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b70ce3a, 90 },
    { 0xa0b0ce3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280ce40, 55 },
    { 0x1970ce41, 43 },
    { 0x850ce41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc0ce45, 23 },
    { 0xeb20ce46, 33 },
    { 0, 0 },
    { 0x52e0ce48, 98 },
    { 0x7e0ce48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c90ce50, 33 },
    { 0, 0 },
    { 0xa430ce52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad0ce56, 23 },
    { 0, 0 },
    { 0x7430ce58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8710ce5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc0ce60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c00ce63, 98 },
    { 0, 0 },
    { 0x1f20ce65, 61 },
    { 0x3100ce66, 23 },
    { 0xb940ce66, 34 },
    { 0, 0 },
    { 0x74e0ce69, 22 },
    { 0x4ae0ce69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad90ce6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a30ce71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f0ce77, 98 },
    { 0x1350ce78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a70ce7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a0ce81, 95 },
    { 0x2a50ce82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd990ce86, 98 },
    { 0xedb0ce87, 61 },
    { 0, 0 },
    { 0x54d0ce89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5190ce8d, 77 },
    { 0x8f0ce8e, 46 },
    { 0, 0 },
    { 0x3170ce90, 68 },
    { 0x2490ce91, 90 },
    { 0xa2a0ce90, 76 },
    { 0, 0 },
    { 0x20a0ce94, 61 },
    { 0x9350ce94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba0ce9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff90cea3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6930cea6, 33 },
    { 0xb270cea6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ceac, 23 },
    { 0xfae0cead, 67 },
    { 0x4f20cead, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6440ceb6, 78 },
    { 0, 0 },
    { 0x9c80ceb8, 99 },
    { 0, 0 },
    { 0xa900ceba, 33 },
    { 0xcfa0cebb, 23 },
    { 0xb940ceba, 95 },
    { 0x70cebd, 91 },
    { 0, 0 },
    { 0xc8c0cebf, 52 },
    { 0xfb90cec0, 43 },
    { 0xc980cebf, 55 },
    { 0xae70cec2, 90 },
    { 0, 0 },
    { 0xab00cec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a70cecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e0ced4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe460cedb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe390cee2, 23 },
    { 0, 0 },
    { 0x43e0cee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf020ceed, 76 },
    { 0, 0 },
    { 0x5970ceef, 33 },
    { 0, 0 },
    { 0xa170cef1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd0cef7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf650cefe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca0cf03, 23 },
    { 0xbde0cf04, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a0cf08, 90 },
    { 0x8fb0cf08, 91 },
    { 0xdbf0cf08, 90 },
    { 0xbde0cf0b, 55 },
    { 0x5ad0cf0a, 93 },
    { 0xab20cf0d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f00cf11, 98 },
    { 0, 0 },
    { 0x5150cf13, 66 },
    { 0x9860cf14, 46 },
    { 0xcba0cf15, 98 },
    { 0xe360cf15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f40cf1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb0cf24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc0cf28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde10cf2f, 87 },
    { 0, 0 },
    { 0x93a0cf31, 67 },
    { 0, 0 },
    { 0xdaf0cf33, 90 },
    { 0xe8c0cf34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d40cf37, 95 },
    { 0x9c80cf38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d0cf3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef0cf40, 87 },
    { 0, 0 },
    { 0x8de0cf42, 87 },
    { 0x39d0cf42, 23 },
    { 0, 0 },
    { 0xc080cf45, 33 },
    { 0, 0 },
    { 0x6070cf47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630cf4a, 95 },
    { 0x7760cf4b, 90 },
    { 0, 0 },
    { 0x3c50cf4d, 76 },
    { 0xad70cf4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f0cf52, 33 },
    { 0, 0 },
    { 0x8460cf54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa840cf59, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b20cf5d, 98 },
    { 0x45a0cf5e, 90 },
    { 0x9a40cf5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad0cf62, 85 },
    { 0xf950cf63, 80 },
    { 0x50cf64, 95 },
    { 0, 0 },
    { 0x75d0cf66, 75 },
    { 0x9a10cf66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0cf6a, 90 },
    { 0xef30cf6a, 55 },
    { 0, 0 },
    { 0x6ec0cf6d, 81 },
    { 0xf610cf6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20cf71, 23 },
    { 0x9140cf72, 23 },
    { 0x47f0cf71, 55 },
    { 0x8a30cf74, 95 },
    { 0, 0 },
    { 0xc8c0cf76, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8810cf7a, 33 },
    { 0, 0 },
    { 0x1f0cf7c, 99 },
    { 0xdf0cf7d, 90 },
    { 0x92e0cf7e, 95 },
    { 0xe820cf7e, 76 },
    { 0x9840cf80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8600cf83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc10cf88, 91 },
    { 0xcef0cf88, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5450cf8c, 33 },
    { 0xb4a0cf8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b0cf93, 55 },
    { 0x8cb0cf94, 77 },
    { 0x61a0cf94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6140cf9c, 99 },
    { 0x80cf9d, 33 },
    { 0x45e0cf9d, 33 },
    { 0x5e10cf9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b0cfa4, 93 },
    { 0x3e20cfa4, 33 },
    { 0xf2b0cfa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc50cfad, 33 },
    { 0x2dd0cfae, 46 },
    { 0x3a0cfad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20cfb2, 52 },
    { 0xcb90cfb3, 61 },
    { 0xb6c0cfb4, 61 },
    { 0, 0 },
    { 0x9f30cfb6, 87 },
    { 0x52d0cfb7, 100 },
    { 0xaae0cfb8, 52 },
    { 0xc170cfb9, 82 },
    { 0xcf20cfb7, 90 },
    { 0, 0 },
    { 0xb3c0cfbc, 95 },
    { 0x6c90cfbc, 23 },
    { 0, 0 },
    { 0x20d0cfbf, 99 },
    { 0xa820cfc0, 90 },
    { 0xec20cfc1, 33 },
    { 0x2380cfc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6500cfc5, 55 },
    { 0, 0 },
    { 0x8220cfc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50cfcf, 66 },
    { 0x4530cfd0, 46 },
    { 0, 0 },
    { 0xdd40cfd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f50cfd5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda10cfd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f0cfdd, 98 },
    { 0, 0 },
    { 0x8c10cfdf, 76 },
    { 0x16a0cfe0, 46 },
    { 0xca10cfe1, 46 },
    { 0, 0 },
    { 0x25f0cfe3, 65 },
    { 0xfeb0cfe4, 98 },
    { 0x7bf0cfe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f80cfea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb70cfed, 34 },
    { 0x51e0cfed, 33 },
    { 0, 0 },
    { 0x3cc0cff0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b0cff5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710cff8, 76 },
    { 0x5230cff9, 95 },
    { 0xd410cffa, 85 },
    { 0x2da0cffb, 90 },
    { 0x5d00cffc, 23 },
    { 0x3ce0cffb, 73 },
    { 0x1950cff8, 76 },
    { 0xda50cffb, 66 },
    { 0x78d0cffa, 67 },
    { 0x8db0d001, 33 },
    { 0x66b0cffb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a20d007, 100 },
    { 0, 0 },
    { 0x2180d009, 61 },
    { 0, 0 },
    { 0x1560d00b, 90 },
    { 0x1600d00c, 66 },
    { 0x4510d00d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c0d01b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc0d01f, 61 },
    { 0, 0 },
    { 0x76e0d021, 33 },
    { 0xf4e0d022, 68 },
    { 0xb1d0d021, 43 },
    { 0, 0 },
    { 0x2670d025, 88 },
    { 0x8df0d026, 100 },
    { 0xd2c0d025, 95 },
    { 0xc5e0d025, 20 },
    { 0xb200d029, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c50d030, 33 },
    { 0x9360d031, 69 },
    { 0xc1b0d030, 90 },
    { 0x2150d033, 46 },
    { 0xce30d032, 34 },
    { 0, 0 },
    { 0x9420d036, 43 },
    { 0x4c30d036, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd20d03b, 33 },
    { 0x95b0d03c, 34 },
    { 0x74f0d03b, 33 },
    { 0xa150d03e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca30d042, 61 },
    { 0xa610d043, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9540d048, 99 },
    { 0x7630d049, 34 },
    { 0, 0 },
    { 0x4be0d04b, 61 },
    { 0, 0 },
    { 0xa440d04d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b0d052, 61 },
    { 0xba40d052, 34 },
    { 0xeea0d054, 75 },
    { 0, 0 },
    { 0x5830d056, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c0d05c, 61 },
    { 0x3e50d05d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60d060, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1730d065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf30d068, 98 },
    { 0, 0 },
    { 0xd8d0d06a, 90 },
    { 0xf620d06a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6490d070, 95 },
    { 0, 0 },
    { 0x6f60d072, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0d080, 95 },
    { 0x3080d081, 95 },
    { 0x7d60d082, 98 },
    { 0, 0 },
    { 0x9f20d084, 23 },
    { 0x9210d084, 40 },
    { 0, 0 },
    { 0x8290d087, 23 },
    { 0xbab0d088, 90 },
    { 0x38e0d087, 61 },
    { 0x2310d08a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d08d, 33 },
    { 0, 0 },
    { 0x9270d08f, 23 },
    { 0xd310d08f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30d093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000d097, 95 },
    { 0x8fc0d098, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe80d09b, 23 },
    { 0x530d09b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde30d0a0, 23 },
    { 0x3980d0a0, 61 },
    { 0, 0 },
    { 0x6f10d0a3, 46 },
    { 0x23b0d0a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6390d0a8, 33 },
    { 0x17c0d0a9, 91 },
    { 0, 0 },
    { 0xe6c0d0ab, 76 },
    { 0, 0 },
    { 0x4b70d0ad, 90 },
    { 0, 0 },
    { 0x9490d0af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e40d0b2, 98 },
    { 0, 0 },
    { 0x9840d0b4, 87 },
    { 0x8fd0d0b5, 52 },
    { 0, 0 },
    { 0xcfb0d0b7, 90 },
    { 0xcec0d0b8, 33 },
    { 0, 0 },
    { 0x86c0d0ba, 61 },
    { 0, 0 },
    { 0xb800d0bc, 90 },
    { 0x7870d0bd, 78 },
    { 0xfcb0d0be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0d0c1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf050d0c4, 77 },
    { 0xf2d0d0c5, 61 },
    { 0xdad0d0c6, 98 },
    { 0x4b90d0c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5400d0cf, 98 },
    { 0x5260d0d0, 61 },
    { 0xafc0d0d1, 85 },
    { 0x6c90d0d2, 88 },
    { 0xeb40d0d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe540d0d9, 43 },
    { 0x5ad0d0d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec0d0e0, 34 },
    { 0x1430d0e1, 61 },
    { 0xcd90d0e1, 64 },
    { 0, 0 },
    { 0x97a0d0e4, 87 },
    { 0x4ff0d0e5, 90 },
    { 0xa440d0e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d00d0e9, 34 },
    { 0xaff0d0e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da0d0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc0d0fa, 95 },
    { 0xcbf0d0fb, 33 },
    { 0x1b80d0fc, 23 },
    { 0xd9c0d0fc, 90 },
    { 0, 0 },
    { 0x890d0ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e70d108, 23 },
    { 0xb640d109, 98 },
    { 0xd200d10a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b20d10d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3570d117, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c0d11a, 95 },
    { 0x13c0d11b, 90 },
    { 0x4aa0d11c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd550d11f, 66 },
    { 0x98c0d120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc20d127, 100 },
    { 0x9580d128, 34 },
    { 0, 0 },
    { 0x4e00d12a, 61 },
    { 0xfb0d12b, 90 },
    { 0xcf20d12c, 61 },
    { 0x2510d12b, 98 },
    { 0x4ba0d12e, 83 },
    { 0, 0 },
    { 0xd6e0d130, 76 },
    { 0x1d40d131, 98 },
    { 0x2320d132, 55 },
    { 0x3890d133, 95 },
    { 0x19a0d133, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5790d138, 95 },
    { 0, 0 },
    { 0x7ab0d13a, 61 },
    { 0x8ed0d13b, 90 },
    { 0x9020d13b, 33 },
    { 0x5080d13d, 67 },
    { 0x2b10d13e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe180d143, 36 },
    { 0x41d0d144, 38 },
    { 0xce0d145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00d148, 52 },
    { 0xf330d149, 33 },
    { 0, 0 },
    { 0xf9d0d14b, 77 },
    { 0, 0 },
    { 0x61d0d14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1820d151, 43 },
    { 0xe3b0d152, 23 },
    { 0, 0 },
    { 0x1830d154, 90 },
    { 0xd690d155, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c20d158, 34 },
    { 0xf4a0d159, 55 },
    { 0x1cb0d158, 90 },
    { 0, 0 },
    { 0xee40d15c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0d163, 84 },
    { 0xae40d164, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc0d16c, 100 },
    { 0, 0 },
    { 0xef50d16e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df0d176, 23 },
    { 0xfa90d176, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8220d17c, 90 },
    { 0xcae0d17d, 98 },
    { 0xd870d17c, 100 },
    { 0xa350d17f, 46 },
    { 0, 0 },
    { 0xf350d181, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf20d184, 90 },
    { 0xcea0d184, 34 },
    { 0, 0 },
    { 0xda40d187, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e0d18c, 56 },
    { 0x4610d18c, 53 },
    { 0x4ca0d18e, 33 },
    { 0x4bb0d18f, 90 },
    { 0xccc0d18c, 46 },
    { 0xad90d18f, 33 },
    { 0x2250d192, 61 },
    { 0x1ac0d190, 98 },
    { 0x1350d18d, 61 },
    { 0, 0 },
    { 0x4810d196, 67 },
    { 0x8780d197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd0d19a, 90 },
    { 0xd0a0d19a, 23 },
    { 0xd60d19a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750d1a5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0d1aa, 99 },
    { 0x6d20d1aa, 98 },
    { 0xec20d1ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a0d1af, 95 },
    { 0x1970d1b0, 61 },
    { 0x3350d1b0, 33 },
    { 0x6020d1b1, 85 },
    { 0xe310d1b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c0d1b6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb220d1bb, 34 },
    { 0, 0 },
    { 0x86a0d1bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b0d1c2, 98 },
    { 0, 0 },
    { 0xd040d1c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3070d1c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6990d1cb, 90 },
    { 0, 0 },
    { 0xe3e0d1cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a0d1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2830d1df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff60d1e3, 78 },
    { 0, 0 },
    { 0xc1b0d1e5, 84 },
    { 0x82b0d1e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece0d1e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c40d1ec, 95 },
    { 0x1ef0d1ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4290d1f1, 84 },
    { 0x6a60d1f2, 90 },
    { 0xbf10d1f3, 61 },
    { 0x22f0d1f2, 61 },
    { 0xe600d1f5, 84 },
    { 0x94c0d1f6, 33 },
    { 0, 0 },
    { 0x4880d1f8, 98 },
    { 0x78d0d1f8, 33 },
    { 0, 0 },
    { 0x6700d1fb, 89 },
    { 0x1090d1fb, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c10d200, 95 },
    { 0x3140d201, 33 },
    { 0xfc70d202, 98 },
    { 0, 0 },
    { 0x4890d204, 33 },
    { 0xf0c0d204, 87 },
    { 0x92e0d206, 43 },
    { 0xf480d204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70d20c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10d214, 78 },
    { 0x8bc0d215, 23 },
    { 0xcb70d216, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f0d21a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2320d223, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3650d226, 33 },
    { 0x9ed0d227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e0d22a, 93 },
    { 0x38f0d22b, 90 },
    { 0xb7a0d22b, 55 },
    { 0xbb00d22d, 61 },
    { 0xaba0d22e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa710d238, 76 },
    { 0x3070d239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b0d23d, 33 },
    { 0x3710d23e, 90 },
    { 0x70d23f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c0d243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0d248, 90 },
    { 0, 0 },
    { 0xf5b0d24a, 76 },
    { 0x77d0d24b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb850d250, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f0d259, 90 },
    { 0x6100d259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be0d260, 67 },
    { 0, 0 },
    { 0x5500d262, 100 },
    { 0xf3d0d263, 38 },
    { 0x5760d264, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0d268, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c50d26b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6140d26f, 95 },
    { 0x15d0d270, 66 },
    { 0xb1a0d270, 69 },
    { 0x8e70d270, 95 },
    { 0x6470d270, 61 },
    { 0x71c0d274, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d0d277, 93 },
    { 0xacd0d278, 88 },
    { 0, 0 },
    { 0x7300d27a, 84 },
    { 0, 0 },
    { 0x200d27c, 98 },
    { 0xee60d27c, 75 },
    { 0x5870d27e, 23 },
    { 0, 0 },
    { 0x5840d280, 81 },
    { 0x9830d280, 96 },
    { 0xf9e0d282, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130d289, 90 },
    { 0xc000d289, 23 },
    { 0x3f20d28b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe400d291, 33 },
    { 0, 0 },
    { 0x76a0d293, 100 },
    { 0xdd00d294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70d297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc0d2a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9210d2ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6500d2b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0d2ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe770d2bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba0d2c2, 34 },
    { 0x3980d2c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e0d2c6, 90 },
    { 0x8e70d2c6, 95 },
    { 0, 0 },
    { 0xc00d2c9, 87 },
    { 0, 0 },
    { 0xfc30d2cb, 98 },
    { 0xab20d2cc, 80 },
    { 0x3130d2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e30d2d6, 43 },
    { 0xf940d2d7, 55 },
    { 0x5f60d2d7, 90 },
    { 0x6ff0d2d9, 95 },
    { 0, 0 },
    { 0x4a50d2db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c60d2de, 100 },
    { 0, 0 },
    { 0xf2e0d2e0, 99 },
    { 0x1dd0d2e0, 99 },
    { 0xb9f0d2e2, 61 },
    { 0xdb90d2e0, 75 },
    { 0x4f80d2e4, 55 },
    { 0xa280d2e4, 98 },
    { 0x8200d2e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7750d2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0d2f4, 81 },
    { 0xabc0d2f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0d2f9, 98 },
    { 0, 0 },
    { 0x45f0d2fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f0d300, 77 },
    { 0x5170d301, 61 },
    { 0x7530d301, 61 },
    { 0x9e30d302, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a00d306, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6930d30a, 98 },
    { 0, 0 },
    { 0x4310d30c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b0d312, 76 },
    { 0xa6f0d313, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c0d318, 69 },
    { 0x9ce0d318, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2500d31e, 84 },
    { 0x9dd0d31f, 68 },
    { 0x9a20d31e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8400d324, 95 },
    { 0, 0 },
    { 0xabb0d326, 98 },
    { 0, 0 },
    { 0x1440d328, 95 },
    { 0, 0 },
    { 0x1e60d32a, 93 },
    { 0xa130d32a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3350d32e, 98 },
    { 0x1440d32f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0d334, 33 },
    { 0, 0 },
    { 0xba90d336, 98 },
    { 0xc120d336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe0d340, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780d343, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf380d348, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec0d34c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d80d350, 23 },
    { 0, 0 },
    { 0x6a70d352, 33 },
    { 0x4150d352, 23 },
    { 0, 0 },
    { 0x7fc0d355, 90 },
    { 0, 0 },
    { 0xb6f0d357, 93 },
    { 0, 0 },
    { 0xd60d359, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920d360, 90 },
    { 0, 0 },
    { 0xcd40d362, 76 },
    { 0xd540d362, 34 },
    { 0x4b50d364, 65 },
    { 0xe860d363, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b0d368, 99 },
    { 0x1bc0d369, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4050d36d, 98 },
    { 0x4030d36e, 23 },
    { 0x5100d36e, 90 },
    { 0x6600d36f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7180d373, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f10d378, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa820d37d, 98 },
    { 0x100d37d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4580d381, 55 },
    { 0, 0 },
    { 0x3360d383, 55 },
    { 0x1820d384, 33 },
    { 0, 0 },
    { 0x9370d386, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd740d389, 90 },
    { 0x93f0d38a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc060d38d, 93 },
    { 0x6be0d38d, 55 },
    { 0x7830d38f, 90 },
    { 0xe8b0d390, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9570d394, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3820d397, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970d39b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1020d3a2, 40 },
    { 0x5aa0d3a2, 98 },
    { 0xdfe0d3a4, 46 },
    { 0, 0 },
    { 0x77b0d3a6, 90 },
    { 0xcc30d3a7, 98 },
    { 0xb3c0d3a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a0d3ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1730d3b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570d3b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3620d3b7, 90 },
    { 0x2e00d3b8, 61 },
    { 0, 0 },
    { 0x3a60d3ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de0d3bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e0d3c4, 98 },
    { 0x77b0d3c4, 67 },
    { 0x9ae0d3c6, 17 },
    { 0xd840d3c7, 76 },
    { 0xb590d3c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc420d3cf, 23 },
    { 0xf3b0d3cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480d3d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0d3d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf660d3dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a0d3e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf0d3e3, 64 },
    { 0, 0 },
    { 0x3c80d3e5, 89 },
    { 0x8ac0d3e6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7370d3ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf510d3ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f90d3f3, 95 },
    { 0, 0 },
    { 0xec10d3f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af0d3f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5490d3fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc120d3ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0d403, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5640d407, 66 },
    { 0xc0e0d407, 33 },
    { 0x68e0d409, 52 },
    { 0xeff0d409, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab40d40e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3230d412, 40 },
    { 0xb860d412, 61 },
    { 0x1a90d413, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b0d41e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf880d424, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f0d427, 61 },
    { 0, 0 },
    { 0x6900d429, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xef30d42c, 23 },
    { 0x9d20d42d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3000d435, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc80d440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db0d445, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b50d448, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4840d44d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb00d455, 55 },
    { 0x5370d456, 46 },
    { 0xc3e0d456, 61 },
    { 0x1630d456, 77 },
    { 0x3ee0d457, 100 },
    { 0, 0 },
    { 0x8a10d45b, 43 },
    { 0x3100d45c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b0d463, 68 },
    { 0, 0 },
    { 0x89b0d465, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc430d471, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e40d477, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f0d47a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4220d480, 55 },
    { 0, 0 },
    { 0x8190d482, 23 },
    { 0x5b80d483, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0d486, 23 },
    { 0, 0 },
    { 0x72e0d488, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d50d48c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb0d48f, 33 },
    { 0x8410d490, 55 },
    { 0, 0 },
    { 0x7730d492, 46 },
    { 0, 0 },
    { 0xca30d494, 61 },
    { 0x9c50d495, 33 },
    { 0x82a0d495, 43 },
    { 0x1650d497, 77 },
    { 0, 0 },
    { 0xed00d499, 61 },
    { 0x6c50d49a, 23 },
    { 0xbfa0d49b, 78 },
    { 0xb180d49a, 61 },
    { 0, 0 },
    { 0x4d30d49e, 91 },
    { 0x1aa0d49f, 52 },
    { 0, 0 },
    { 0x8700d4a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce0d4a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9220d4a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c0d4ac, 98 },
    { 0x6390d4ac, 95 },
    { 0, 0 },
    { 0x16b0d4af, 61 },
    { 0xaf40d4af, 53 },
    { 0x5a50d4b0, 34 },
    { 0xb4b0d4b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff0d4b7, 53 },
    { 0, 0 },
    { 0x8e00d4b9, 65 },
    { 0x3000d4b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc380d4bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae0d4c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6320d4c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc0d4ca, 88 },
    { 0, 0 },
    { 0x39d0d4cc, 46 },
    { 0, 0 },
    { 0xc30d4ce, 61 },
    { 0x8500d4cf, 98 },
    { 0, 0 },
    { 0x3040d4d1, 90 },
    { 0, 0 },
    { 0x21d0d4d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b80d4d6, 23 },
    { 0x15b0d4d7, 76 },
    { 0xf3f0d4d8, 33 },
    { 0x9440d4d9, 100 },
    { 0x7440d4d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb0d4e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b0d4e6, 61 },
    { 0, 0 },
    { 0x3930d4e8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d50d4eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0d4ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d0d4f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960d4fa, 46 },
    { 0x9680d4fb, 23 },
    { 0xf410d4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f60d501, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe000d504, 88 },
    { 0x78e0d505, 90 },
    { 0x86b0d506, 23 },
    { 0x4e60d507, 90 },
    { 0, 0 },
    { 0xe230d509, 87 },
    { 0x1350d50a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50d50f, 76 },
    { 0xde90d50f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb340d515, 61 },
    { 0xa100d516, 61 },
    { 0, 0 },
    { 0xa6f0d518, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9850d51b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e0d51e, 64 },
    { 0xc760d51e, 68 },
    { 0x7f00d520, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9890d52d, 33 },
    { 0x6670d52e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570d532, 33 },
    { 0xcb10d533, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f0d537, 67 },
    { 0, 0 },
    { 0xc690d539, 23 },
    { 0x5550d53a, 52 },
    { 0x2600d53b, 60 },
    { 0, 0 },
    { 0x9430d53d, 90 },
    { 0x7950d53d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d0d543, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa60d546, 61 },
    { 0x2850d547, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe230d54d, 95 },
    { 0x3a00d54e, 90 },
    { 0x23a0d54f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d552, 84 },
    { 0xcab0d553, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef0d556, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50d559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8790d55e, 23 },
    { 0x26b0d55f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7340d564, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d0d569, 90 },
    { 0, 0 },
    { 0x4db0d56b, 98 },
    { 0x400d56b, 95 },
    { 0, 0 },
    { 0x9500d56e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb400d572, 76 },
    { 0x5460d573, 90 },
    { 0, 0 },
    { 0x8670d575, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e0d579, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d0d57d, 33 },
    { 0x86f0d57e, 61 },
    { 0x77a0d57f, 43 },
    { 0xb970d580, 55 },
    { 0, 0 },
    { 0x3f20d582, 98 },
    { 0xf10d582, 33 },
    { 0xd00d584, 85 },
    { 0x9650d583, 93 },
    { 0, 0 },
    { 0x6b20d587, 90 },
    { 0x9300d588, 91 },
    { 0, 0 },
    { 0xee60d58a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b0d58e, 81 },
    { 0, 0 },
    { 0x6360d590, 55 },
    { 0xa3b0d591, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb460d594, 23 },
    { 0, 0 },
    { 0xbb00d596, 76 },
    { 0x770d597, 91 },
    { 0xca80d598, 98 },
    { 0, 0 },
    { 0x3440d59a, 76 },
    { 0xa160d59b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1510d59e, 43 },
    { 0x30a0d59e, 95 },
    { 0, 0 },
    { 0xc0f0d5a1, 55 },
    { 0, 0 },
    { 0xf5b0d5a3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa470d5a6, 55 },
    { 0, 0 },
    { 0xa4e0d5a8, 90 },
    { 0xf4b0d5a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed30d5ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f20d5b6, 90 },
    { 0xd750d5b7, 33 },
    { 0, 0 },
    { 0x6680d5b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2780d5bc, 78 },
    { 0x7090d5bd, 78 },
    { 0x7200d5bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40d5c1, 61 },
    { 0x59f0d5c2, 68 },
    { 0x27d0d5c2, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e0d5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c0d5c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0d5d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3520d5d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9140d5db, 95 },
    { 0xb8f0d5dc, 90 },
    { 0, 0 },
    { 0x2e80d5de, 61 },
    { 0x8f50d5de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7940d5e3, 100 },
    { 0x2850d5e4, 33 },
    { 0, 0 },
    { 0x8a80d5e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a10d5ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c0d5f1, 95 },
    { 0xaf20d5f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e0d5fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0d5fd, 69 },
    { 0x8f40d5fe, 61 },
    { 0, 0 },
    { 0xacd0d600, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f0d603, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b30d607, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0d60a, 98 },
    { 0x67a0d60b, 99 },
    { 0x380d60c, 81 },
    { 0x8e80d60b, 95 },
    { 0x9030d60e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa70d611, 98 },
    { 0x8b10d612, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d0d617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf00d61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f0d61d, 96 },
    { 0xe7e0d61e, 61 },
    { 0x1d80d61f, 40 },
    { 0x7ae0d620, 33 },
    { 0x2ae0d621, 23 },
    { 0xeb0d621, 98 },
    { 0x7870d621, 76 },
    { 0xab70d623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1510d628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710d62b, 33 },
    { 0xee50d62c, 96 },
    { 0xb4d0d62b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec20d631, 76 },
    { 0x73e0d632, 61 },
    { 0xce70d633, 61 },
    { 0x8560d634, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d0d639, 90 },
    { 0x2f80d639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0d640, 95 },
    { 0x5ed0d641, 90 },
    { 0, 0 },
    { 0x84c0d643, 67 },
    { 0x90d643, 55 },
    { 0x370d643, 55 },
    { 0x6c20d645, 61 },
    { 0xfbe0d647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda0d650, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9560d654, 93 },
    { 0xadb0d655, 91 },
    { 0xc5e0d655, 84 },
    { 0, 0 },
    { 0x3b90d658, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab0d661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f20d672, 95 },
    { 0x7840d672, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00d677, 95 },
    { 0x1560d678, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca0d67b, 61 },
    { 0, 0 },
    { 0x53f0d67d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c0d682, 55 },
    { 0, 0 },
    { 0xd0e0d684, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6580d687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c0d68a, 33 },
    { 0xa7f0d68b, 90 },
    { 0xa310d68c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc890d691, 60 },
    { 0x6730d691, 42 },
    { 0x580d692, 23 },
    { 0x2710d694, 23 },
    { 0, 0 },
    { 0x2de0d696, 91 },
    { 0x4ff0d696, 33 },
    { 0xb0d0d698, 34 },
    { 0, 0 },
    { 0x9790d69a, 95 },
    { 0, 0 },
    { 0x3190d69c, 33 },
    { 0xcd20d69d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1040d6a3, 95 },
    { 0, 0 },
    { 0x5f00d6a5, 17 },
    { 0x68c0d6a6, 43 },
    { 0xabf0d6a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70d6aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0d6ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b0d6b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc0d6b3, 95 },
    { 0, 0 },
    { 0x370d6b5, 98 },
    { 0, 0 },
    { 0x90d6b7, 52 },
    { 0x1300d6b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc510d6bb, 43 },
    { 0xef40d6bc, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f0d6c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd970d6cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe060d6cf, 95 },
    { 0, 0 },
    { 0x9640d6d1, 66 },
    { 0x8ff0d6d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc170d6db, 95 },
    { 0xfc70d6dc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f90d6e0, 23 },
    { 0xcb20d6e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x590d6e4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5990d6e7, 23 },
    { 0, 0 },
    { 0x7de0d6e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a10d6ec, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6730d6f1, 77 },
    { 0xdde0d6f2, 76 },
    { 0x78c0d6f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad0d6f6, 33 },
    { 0, 0 },
    { 0x55e0d6f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c0d6fd, 33 },
    { 0x6960d6fe, 100 },
    { 0, 0 },
    { 0x23d0d700, 61 },
    { 0x9820d700, 84 },
    { 0xfa40d702, 90 },
    { 0x4ff0d703, 46 },
    { 0x8140d700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc450d708, 23 },
    { 0x20f0d708, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe370d70e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc0d717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7090d71a, 33 },
    { 0xf800d71b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e60d71e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa700d723, 76 },
    { 0, 0 },
    { 0xd760d725, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f40d729, 61 },
    { 0xf0a0d729, 61 },
    { 0x56b0d72a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a0d732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b60d736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a0d739, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd0d742, 34 },
    { 0xd2c0d742, 98 },
    { 0, 0 },
    { 0xd120d745, 84 },
    { 0xdf40d746, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf0d74b, 87 },
    { 0xc5d0d74b, 78 },
    { 0xc90d74d, 93 },
    { 0, 0 },
    { 0x5b80d74f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f00d753, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xad00d756, 61 },
    { 0xca30d756, 90 },
    { 0x5a30d758, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b0d75b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a60d75e, 33 },
    { 0, 0 },
    { 0x9b80d760, 87 },
    { 0xcc0d761, 76 },
    { 0x9450d761, 100 },
    { 0xb530d763, 46 },
    { 0, 0 },
    { 0x43c0d765, 81 },
    { 0x6670d766, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1450d769, 24 },
    { 0x9690d76a, 76 },
    { 0, 0 },
    { 0x49d0d76c, 61 },
    { 0xa1e0d76d, 95 },
    { 0x84b0d76d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2790d772, 68 },
    { 0, 0 },
    { 0xa590d774, 99 },
    { 0, 0 },
    { 0xcd20d776, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf850d77f, 90 },
    { 0x2970d77f, 33 },
    { 0x2d60d781, 23 },
    { 0x82a0d782, 55 },
    { 0x5a0d782, 98 },
    { 0xef00d781, 55 },
    { 0x1a0d785, 95 },
    { 0x7450d786, 77 },
    { 0xa270d785, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa0d78c, 88 },
    { 0xb5f0d78d, 23 },
    { 0, 0 },
    { 0x3a80d78f, 93 },
    { 0xd650d78f, 33 },
    { 0, 0 },
    { 0xfd80d792, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a0d796, 34 },
    { 0x1b20d796, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d80d79c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f0d79f, 90 },
    { 0, 0 },
    { 0x30c0d7a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb90d7a6, 95 },
    { 0, 0 },
    { 0x76a0d7a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6260d7ac, 100 },
    { 0xd8d0d7ac, 23 },
    { 0xc6b0d7ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f0d7b2, 23 },
    { 0xa10d7b2, 43 },
    { 0x7a30d7b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee0d7b8, 23 },
    { 0x6480d7b9, 33 },
    { 0x5c10d7ba, 33 },
    { 0xab90d7b8, 99 },
    { 0xa810d7bc, 34 },
    { 0, 0 },
    { 0xc30d7be, 33 },
    { 0, 0 },
    { 0x67c0d7c0, 68 },
    { 0xc340d7c1, 33 },
    { 0x66a0d7c1, 38 },
    { 0xa740d7c3, 61 },
    { 0, 0 },
    { 0x3a30d7c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae80d7cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd870d7d1, 76 },
    { 0x7b70d7d2, 95 },
    { 0xfe70d7d3, 92 },
    { 0x7bb0d7d3, 33 },
    { 0xce40d7d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c0d7d8, 34 },
    { 0xde60d7d9, 33 },
    { 0x3210d7d9, 95 },
    { 0xa380d7d8, 76 },
    { 0x1f50d7dc, 33 },
    { 0xfc20d7da, 34 },
    { 0xa050d7d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a0d7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd350d7e8, 61 },
    { 0, 0 },
    { 0xe230d7ea, 23 },
    { 0, 0 },
    { 0xf130d7ec, 90 },
    { 0xaa50d7ed, 46 },
    { 0x6680d7ed, 55 },
    { 0x6b40d7ef, 84 },
    { 0xee20d7ef, 23 },
    { 0x3740d7f0, 95 },
    { 0x72f0d7f0, 53 },
    { 0x7530d7f3, 95 },
    { 0xe710d7ef, 95 },
    { 0, 0 },
    { 0x7090d7f6, 99 },
    { 0x950d7f7, 90 },
    { 0x1ba0d7f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb950d7fc, 90 },
    { 0x23c0d7fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7940d802, 23 },
    { 0x33c0d802, 33 },
    { 0x41a0d802, 61 },
    { 0xa440d805, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe30d808, 33 },
    { 0xf280d808, 66 },
    { 0xe200d80a, 34 },
    { 0x8650d80b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340d80e, 34 },
    { 0x56d0d80e, 33 },
    { 0xbfd0d80f, 95 },
    { 0x6db0d80e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f0d814, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9920d817, 90 },
    { 0x4ef0d817, 65 },
    { 0x8b10d818, 90 },
    { 0, 0 },
    { 0x4950d81b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc800d81f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd890d822, 73 },
    { 0x4c80d823, 98 },
    { 0x5920d822, 95 },
    { 0xaf90d822, 87 },
    { 0x2060d826, 76 },
    { 0xa460d822, 95 },
    { 0, 0 },
    { 0x3290d829, 23 },
    { 0xbe70d82a, 52 },
    { 0xa1d0d82b, 34 },
    { 0xab70d82c, 90 },
    { 0xf170d82d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc0d830, 98 },
    { 0xdd10d830, 98 },
    { 0x7a70d831, 90 },
    { 0, 0 },
    { 0x6160d834, 100 },
    { 0xbed0d835, 33 },
    { 0x1910d836, 61 },
    { 0x7aa0d836, 55 },
    { 0xbc60d838, 90 },
    { 0xb850d836, 53 },
    { 0, 0 },
    { 0x25d0d83b, 98 },
    { 0x56d0d83b, 98 },
    { 0x4e10d83d, 43 },
    { 0xc060d83b, 93 },
    { 0xbbb0d83e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7230d844, 93 },
    { 0, 0 },
    { 0xac90d846, 87 },
    { 0x1090d847, 43 },
    { 0, 0 },
    { 0xf470d849, 95 },
    { 0x1d50d849, 76 },
    { 0xa240d84b, 91 },
    { 0xbee0d84c, 83 },
    { 0xe50d84c, 88 },
    { 0x180d84e, 90 },
    { 0xa340d84e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7630d854, 90 },
    { 0x5490d855, 43 },
    { 0x73b0d856, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xd320d859, 83 },
    { 0xf8c0d85a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff80d85f, 98 },
    { 0x1ad0d85f, 95 },
    { 0x1990d861, 33 },
    { 0xab40d862, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc80d865, 43 },
    { 0xaf50d865, 90 },
    { 0, 0 },
    { 0x5a30d868, 98 },
    { 0x99b0d868, 95 },
    { 0xb3a0d869, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5840d86e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0d875, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a0d87e, 78 },
    { 0xc570d87f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370d884, 76 },
    { 0xa260d885, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f0d889, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef0d88c, 61 },
    { 0x4a70d88c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b0d894, 65 },
    { 0x1270d895, 55 },
    { 0x91c0d895, 90 },
    { 0, 0 },
    { 0x2100d898, 43 },
    { 0xea40d899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0d89f, 98 },
    { 0, 0 },
    { 0xcae0d8a1, 55 },
    { 0x78d0d8a2, 64 },
    { 0xcaf0d8a3, 61 },
    { 0, 0 },
    { 0x4680d8a5, 91 },
    { 0, 0 },
    { 0x2290d8a7, 63 },
    { 0, 0 },
    { 0xbc30d8a9, 61 },
    { 0xe4a0d8aa, 90 },
    { 0x8d80d8ab, 33 },
    { 0, 0 },
    { 0xa250d8ad, 95 },
    { 0, 0 },
    { 0xa060d8af, 95 },
    { 0, 0 },
    { 0x8390d8b1, 33 },
    { 0, 0 },
    { 0xa890d8b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc60d8bb, 43 },
    { 0xa310d8bc, 84 },
    { 0xcb40d8bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd760d8c0, 76 },
    { 0, 0 },
    { 0xca20d8c2, 90 },
    { 0x6c70d8c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa480d8c6, 61 },
    { 0x5020d8c7, 87 },
    { 0xf790d8c7, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe0d8cb, 90 },
    { 0x1c50d8cc, 23 },
    { 0x5f20d8cc, 100 },
    { 0xb3c0d8cd, 90 },
    { 0x4e90d8cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf910d8d2, 61 },
    { 0, 0 },
    { 0xe7f0d8d4, 90 },
    { 0, 0 },
    { 0xe970d8d6, 95 },
    { 0xfc60d8d7, 76 },
    { 0xfca0d8d7, 100 },
    { 0, 0 },
    { 0xd610d8da, 90 },
    { 0x61d0d8db, 61 },
    { 0xaea0d8dc, 34 },
    { 0xa7e0d8dd, 33 },
    { 0, 0 },
    { 0xe50d8df, 95 },
    { 0x3550d8e0, 23 },
    { 0xa000d8e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d0d8e5, 61 },
    { 0, 0 },
    { 0x8180d8e7, 52 },
    { 0, 0 },
    { 0xfff0d8e9, 33 },
    { 0x7a70d8e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3500d8ed, 76 },
    { 0, 0 },
    { 0xf6f0d8ef, 90 },
    { 0, 0 },
    { 0x580d8f1, 61 },
    { 0xbf0d8f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1700d8f9, 90 },
    { 0xc7b0d8fa, 98 },
    { 0, 0 },
    { 0x5780d8fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6300d902, 66 },
    { 0xfe50d902, 90 },
    { 0xf3a0d903, 95 },
    { 0, 0 },
    { 0x36b0d906, 55 },
    { 0x17d0d906, 33 },
    { 0xf2b0d907, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e0d90d, 69 },
    { 0x7120d90e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f0d912, 90 },
    { 0, 0 },
    { 0x5600d914, 46 },
    { 0x25d0d915, 90 },
    { 0xa9b0d916, 46 },
    { 0x9230d917, 90 },
    { 0x56a0d918, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750d91c, 23 },
    { 0xc0a0d91c, 98 },
    { 0x45e0d91e, 61 },
    { 0x1e80d91d, 46 },
    { 0x7e50d91c, 33 },
    { 0x5d30d921, 95 },
    { 0xe170d921, 33 },
    { 0, 0 },
    { 0x6f40d924, 39 },
    { 0xe250d925, 95 },
    { 0xe60d926, 98 },
    { 0, 0 },
    { 0x4ff0d928, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1980d92b, 33 },
    { 0, 0 },
    { 0x1ed0d92d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a0d932, 84 },
    { 0x90b0d933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa270d937, 33 },
    { 0, 0 },
    { 0x7840d939, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f0d93e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe00d943, 61 },
    { 0xf8a0d943, 23 },
    { 0, 0 },
    { 0x6750d946, 90 },
    { 0x1b20d947, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0d94a, 84 },
    { 0, 0 },
    { 0x1980d94c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800d951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3950d954, 95 },
    { 0, 0 },
    { 0xd960d956, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6970d959, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe50d95e, 61 },
    { 0x8ba0d95f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf00d962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc0d967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a70d96a, 99 },
    { 0x9a40d96a, 34 },
    { 0x4150d96c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc00d970, 33 },
    { 0xba30d971, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5670d97d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9030d982, 33 },
    { 0xadc0d983, 61 },
    { 0x9fb0d984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000d987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4790d98b, 98 },
    { 0x5fd0d98b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5550d990, 78 },
    { 0x1df0d991, 90 },
    { 0x8500d991, 46 },
    { 0xb590d992, 61 },
    { 0xc0a0d994, 91 },
    { 0, 0 },
    { 0x23e0d996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba50d99e, 90 },
    { 0xc50d99f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa950d9a5, 55 },
    { 0x43e0d9a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe100d9ad, 33 },
    { 0x57f0d9ae, 55 },
    { 0x15c0d9ae, 43 },
    { 0xfee0d9b0, 88 },
    { 0x9a0d9ad, 23 },
    { 0x22a0d9b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef0d9b5, 33 },
    { 0x760d9b6, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20d9bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360d9c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4080d9c4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe040d9c7, 90 },
    { 0, 0 },
    { 0xc430d9c9, 90 },
    { 0, 0 },
    { 0xd130d9cb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1070d9cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e0d9d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed10d9d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb370d9da, 46 },
    { 0xc810d9db, 78 },
    { 0xa210d9dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b20d9df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f00d9e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca0d9e6, 98 },
    { 0x40a0d9e7, 78 },
    { 0x66f0d9e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5340d9eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea30d9ef, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b0d9f2, 61 },
    { 0x7ca0d9f3, 95 },
    { 0, 0 },
    { 0x9250d9f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d9fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd420d9fd, 68 },
    { 0, 0 },
    { 0xdcb0d9ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10da04, 23 },
    { 0x640da04, 100 },
    { 0, 0 },
    { 0xd060da07, 90 },
    { 0, 0 },
    { 0xc520da09, 95 },
    { 0xf700da0a, 33 },
    { 0xc190da0b, 17 },
    { 0x15d0da0a, 95 },
    { 0xe480da0b, 90 },
    { 0x15a0da0e, 98 },
    { 0x4bb0da0f, 33 },
    { 0, 0 },
    { 0xac90da11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620da15, 91 },
    { 0, 0 },
    { 0x8290da17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00da1a, 95 },
    { 0xa8e0da1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1850da22, 90 },
    { 0xd90da23, 61 },
    { 0x8f20da22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0da2b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d30da31, 75 },
    { 0x5c10da31, 33 },
    { 0x9ad0da33, 46 },
    { 0xe110da31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf280da37, 76 },
    { 0xdc90da38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb50da3d, 33 },
    { 0, 0 },
    { 0x4940da3f, 23 },
    { 0xa790da3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790da44, 34 },
    { 0, 0 },
    { 0x6ca0da46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea0da49, 33 },
    { 0x3430da49, 93 },
    { 0x66d0da4b, 34 },
    { 0xf420da4c, 52 },
    { 0xfb30da4c, 81 },
    { 0xa200da4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe490da52, 34 },
    { 0, 0 },
    { 0xb070da54, 61 },
    { 0x5fe0da54, 95 },
    { 0xe270da56, 61 },
    { 0xc4c0da57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2270da5c, 66 },
    { 0, 0 },
    { 0x7270da5e, 23 },
    { 0xb3d0da5f, 95 },
    { 0x6780da60, 33 },
    { 0, 0 },
    { 0x6120da62, 61 },
    { 0xe7b0da63, 76 },
    { 0, 0 },
    { 0xa40da65, 23 },
    { 0x2560da66, 46 },
    { 0, 0 },
    { 0xca00da68, 95 },
    { 0x6580da69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a0da6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb10da7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c0da82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00da89, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d0da90, 43 },
    { 0, 0 },
    { 0xec0da92, 98 },
    { 0xe590da93, 98 },
    { 0xa0d0da93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc190da99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c0da9c, 49 },
    { 0x6320da9c, 55 },
    { 0xfa00da9c, 55 },
    { 0xd9e0da9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c0daa2, 61 },
    { 0, 0 },
    { 0xaf40daa4, 95 },
    { 0, 0 },
    { 0xdd20daa6, 95 },
    { 0, 0 },
    { 0x1a90daa8, 95 },
    { 0, 0 },
    { 0xa290daaa, 23 },
    { 0, 0 },
    { 0xd280daac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb0dabb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f30dabe, 95 },
    { 0, 0 },
    { 0x8470dac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba60dac3, 90 },
    { 0, 0 },
    { 0xe680dac5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef40daca, 55 },
    { 0xe7c0dacb, 61 },
    { 0x70b0dacc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0dadb, 100 },
    { 0x2f70dadc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe070dae1, 80 },
    { 0xd230dae2, 75 },
    { 0x7f00dae3, 33 },
    { 0, 0 },
    { 0x90dae5, 23 },
    { 0x5c30dae5, 79 },
    { 0x31e0dae7, 90 },
    { 0x83a0dae8, 91 },
    { 0, 0 },
    { 0x2f00daea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f0daef, 33 },
    { 0x5b80daf0, 80 },
    { 0x4770daf1, 76 },
    { 0x12a0daf2, 76 },
    { 0, 0 },
    { 0x32c0daf4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a0dafc, 95 },
    { 0xb120dafd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7840db02, 46 },
    { 0xfaf0db03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8640db08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f0db0b, 23 },
    { 0x5fe0db0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e0db0f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c0db15, 76 },
    { 0, 0 },
    { 0xa970db17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b0db1c, 34 },
    { 0x57a0db1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de0db20, 90 },
    { 0, 0 },
    { 0xc730db22, 76 },
    { 0, 0 },
    { 0x1530db24, 55 },
    { 0, 0 },
    { 0xe5a0db26, 55 },
    { 0x8d10db27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9240db2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660db2e, 33 },
    { 0x2820db2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e0db32, 76 },
    { 0, 0 },
    { 0xf560db34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2000db3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4390db3f, 77 },
    { 0x4580db40, 78 },
    { 0xc6a0db41, 55 },
    { 0xd0a0db40, 99 },
    { 0x7aa0db42, 23 },
    { 0x33b0db44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b0db47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd720db4d, 98 },
    { 0xbfa0db4e, 33 },
    { 0x99c0db4f, 44 },
    { 0x3bc0db4e, 98 },
    { 0, 0 },
    { 0xd5f0db52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f80db56, 55 },
    { 0xe0db56, 55 },
    { 0x8080db58, 46 },
    { 0x3970db59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e20db5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30db61, 78 },
    { 0x4420db62, 23 },
    { 0x16f0db63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0db66, 46 },
    { 0x50e0db66, 98 },
    { 0x2300db67, 61 },
    { 0x1140db69, 43 },
    { 0x3c0db6a, 61 },
    { 0, 0 },
    { 0x5b70db6c, 43 },
    { 0xb560db6d, 33 },
    { 0xf780db6e, 34 },
    { 0xa510db6e, 95 },
    { 0xf4c0db70, 64 },
    { 0x960db71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c20db75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd410db7d, 55 },
    { 0x8c10db7e, 61 },
    { 0x8120db7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd070db8a, 23 },
    { 0xc460db8b, 55 },
    { 0x8c30db8b, 34 },
    { 0x5810db8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa0db92, 61 },
    { 0x7c40db93, 33 },
    { 0x1400db94, 33 },
    { 0, 0 },
    { 0xcae0db96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4370db9a, 43 },
    { 0xa210db9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c60dba5, 98 },
    { 0xd200dba5, 61 },
    { 0x88e0dba6, 61 },
    { 0x3690dba8, 66 },
    { 0x5ea0dba9, 46 },
    { 0x3db0dba6, 23 },
    { 0xca30dbaa, 90 },
    { 0xdc90dbac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e0dbaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d60dbb2, 36 },
    { 0x5760dbb3, 69 },
    { 0, 0 },
    { 0x7990dbb5, 61 },
    { 0x4030dbb5, 27 },
    { 0, 0 },
    { 0x58c0dbb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe690dbbf, 100 },
    { 0x4ed0dbbf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9220dbc7, 67 },
    { 0x85e0dbc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe890dbcb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad0dbcf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab0dbd3, 98 },
    { 0xaaf0dbd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9530dbd8, 95 },
    { 0xbc0dbd9, 90 },
    { 0, 0 },
    { 0x12f0dbdb, 33 },
    { 0x2710dbdc, 61 },
    { 0x15e0dbdd, 99 },
    { 0, 0 },
    { 0x7e00dbdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f70dbe3, 66 },
    { 0xd550dbe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb0dbe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6390dbf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d20dbf6, 80 },
    { 0, 0 },
    { 0x4840dbf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b0dbfb, 34 },
    { 0xdae0dbfc, 98 },
    { 0xad00dbfc, 33 },
    { 0xaf30dbfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5590dc02, 90 },
    { 0, 0 },
    { 0xe320dc04, 90 },
    { 0xac80dc04, 40 },
    { 0x91e0dc04, 90 },
    { 0x8f90dc07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf900dc0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70dc0d, 98 },
    { 0, 0 },
    { 0x64c0dc0f, 55 },
    { 0x560dc10, 83 },
    { 0xd3a0dc0f, 61 },
    { 0x6ad0dc12, 61 },
    { 0x5b60dc13, 43 },
    { 0x4cf0dc14, 33 },
    { 0x39d0dc10, 33 },
    { 0xe470dc16, 91 },
    { 0x4fc0dc10, 61 },
    { 0x440dc18, 34 },
    { 0xcd80dc19, 91 },
    { 0, 0 },
    { 0x1390dc1b, 66 },
    { 0, 0 },
    { 0x3e20dc1d, 76 },
    { 0x5b80dc1e, 61 },
    { 0xfc30dc1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa790dc24, 46 },
    { 0, 0 },
    { 0xf3f0dc26, 97 },
    { 0x290dc26, 94 },
    { 0x95a0dc27, 99 },
    { 0x1dd0dc26, 84 },
    { 0x7e70dc26, 93 },
    { 0x5380dc28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d90dc2f, 55 },
    { 0x5f40dc30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e30dc33, 43 },
    { 0xf40dc34, 76 },
    { 0x72d0dc35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd980dc3c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc000dc3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc0dc42, 33 },
    { 0x4690dc43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa0dc46, 23 },
    { 0, 0 },
    { 0xe6e0dc48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x750dc4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c0dc50, 90 },
    { 0xd1b0dc50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990dc57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa780dc5a, 33 },
    { 0x9860dc5b, 61 },
    { 0, 0 },
    { 0xa5b0dc5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e30dc62, 33 },
    { 0x7820dc62, 84 },
    { 0xd660dc64, 33 },
    { 0xdb50dc62, 95 },
    { 0xb000dc66, 78 },
    { 0, 0 },
    { 0xc60dc68, 90 },
    { 0, 0 },
    { 0x87e0dc6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d50dc73, 55 },
    { 0x14c0dc74, 81 },
    { 0x3ed0dc75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9930dc79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb0dc82, 98 },
    { 0x7b00dc83, 87 },
    { 0xec90dc83, 52 },
    { 0, 0 },
    { 0xf670dc86, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3530dc8a, 23 },
    { 0x5f00dc8b, 95 },
    { 0xb30dc8a, 90 },
    { 0x2330dc8d, 23 },
    { 0xf140dc8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3840dc93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe690dc96, 34 },
    { 0x9720dc97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc100dc9a, 91 },
    { 0xf60dc9a, 90 },
    { 0, 0 },
    { 0xf380dc9d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e70dca2, 61 },
    { 0xe000dca2, 55 },
    { 0, 0 },
    { 0x5d40dca5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c40dca8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f0dcad, 95 },
    { 0xec30dcae, 81 },
    { 0xe840dcae, 76 },
    { 0xfec0dcb0, 46 },
    { 0x68d0dcaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc850dcb6, 98 },
    { 0, 0 },
    { 0x6d10dcb8, 33 },
    { 0, 0 },
    { 0x1640dcba, 95 },
    { 0xa200dcbb, 90 },
    { 0x5a10dcbc, 33 },
    { 0xbb20dcbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd0dcc3, 66 },
    { 0, 0 },
    { 0x3b70dcc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00dcca, 33 },
    { 0xf6c0dcca, 23 },
    { 0xf8f0dccc, 17 },
    { 0xffb0dccd, 90 },
    { 0x88c0dcce, 77 },
    { 0xda60dccd, 87 },
    { 0x10dcce, 55 },
    { 0x7730dcce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3130dcd5, 33 },
    { 0x1880dcd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b0dcd9, 90 },
    { 0xa3a0dcd9, 33 },
    { 0, 0 },
    { 0x1cb0dcdc, 43 },
    { 0, 0 },
    { 0xc3a0dcde, 88 },
    { 0xd5f0dcde, 93 },
    { 0x9810dce0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc400dce9, 55 },
    { 0, 0 },
    { 0x7950dceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a0dcef, 19 },
    { 0x8590dcf0, 88 },
    { 0, 0 },
    { 0xe390dcf2, 23 },
    { 0x5580dcf3, 91 },
    { 0x9bf0dcf4, 100 },
    { 0x4cf0dcf3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d0dcfa, 61 },
    { 0x3950dcfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5930dd00, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac20dd07, 55 },
    { 0, 0 },
    { 0x6390dd09, 81 },
    { 0x2ee0dd09, 100 },
    { 0x2090dd0b, 75 },
    { 0, 0 },
    { 0xd800dd0d, 78 },
    { 0x4f0dd0e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a0dd18, 98 },
    { 0, 0 },
    { 0x83e0dd1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6390dd1d, 90 },
    { 0x9560dd1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1310dd22, 66 },
    { 0, 0 },
    { 0x6120dd24, 95 },
    { 0x80b0dd25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb770dd2b, 61 },
    { 0, 0 },
    { 0xc3a0dd2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690dd37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dd3b, 61 },
    { 0x52a0dd3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d0dd41, 23 },
    { 0, 0 },
    { 0x2de0dd43, 67 },
    { 0, 0 },
    { 0x7a90dd45, 33 },
    { 0xb810dd46, 95 },
    { 0, 0 },
    { 0xa380dd48, 90 },
    { 0xbe40dd48, 61 },
    { 0x6550dd49, 76 },
    { 0x27e0dd4b, 76 },
    { 0xfe70dd4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd0dd50, 55 },
    { 0x2300dd51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1040dd54, 61 },
    { 0x1b10dd54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b0dd59, 90 },
    { 0xd8b0dd5a, 98 },
    { 0xb8e0dd59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb140dd5f, 46 },
    { 0, 0 },
    { 0x8480dd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7720dd64, 90 },
    { 0, 0 },
    { 0x3ff0dd66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be0dd69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30dd6d, 23 },
    { 0x2400dd6e, 98 },
    { 0xa7e0dd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60dd74, 93 },
    { 0xd010dd75, 77 },
    { 0xa850dd76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7550dd7a, 52 },
    { 0x4770dd7b, 95 },
    { 0xd4b0dd7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf70dd81, 98 },
    { 0x4200dd82, 61 },
    { 0xa880dd81, 83 },
    { 0x1b80dd84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340dd8a, 98 },
    { 0x9130dd8b, 90 },
    { 0x5650dd8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b0dd91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710dd96, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480dd99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd0dd9f, 93 },
    { 0xb3d0dda0, 84 },
    { 0, 0 },
    { 0xeb50dda2, 33 },
    { 0x33e0dda3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d70dda6, 97 },
    { 0xc8d0dda6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b40ddb2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b0ddb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb0ddbd, 66 },
    { 0xfdd0ddbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca20ddc1, 99 },
    { 0x5740ddc2, 98 },
    { 0xea90ddc3, 66 },
    { 0x6c30ddc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d0ddc7, 96 },
    { 0x7640ddc8, 33 },
    { 0, 0 },
    { 0xfbb0ddca, 34 },
    { 0xabf0ddcb, 33 },
    { 0x3ff0ddca, 87 },
    { 0x5e20ddcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8710ddd0, 34 },
    { 0xade0ddd1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e0ddd6, 87 },
    { 0, 0 },
    { 0x6530ddd8, 43 },
    { 0x8060ddd9, 84 },
    { 0x84d0ddda, 100 },
    { 0xb900dddb, 93 },
    { 0x46b0ddd8, 33 },
    { 0xa550dddd, 68 },
    { 0x83b0dddd, 90 },
    { 0, 0 },
    { 0xe730dde0, 90 },
    { 0, 0 },
    { 0xbf00dde2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e0dde7, 90 },
    { 0xf860dde7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d60ddef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b0ddf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb0ddf9, 34 },
    { 0xa330ddfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3400ddfe, 98 },
    { 0x7e10ddff, 33 },
    { 0xa990de00, 47 },
    { 0xa520ddff, 95 },
    { 0, 0 },
    { 0x9070de03, 98 },
    { 0x33b0de03, 98 },
    { 0x5780de05, 23 },
    { 0x38a0de05, 90 },
    { 0, 0 },
    { 0x8670de08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf360de0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f00de10, 33 },
    { 0, 0 },
    { 0xc020de12, 23 },
    { 0, 0 },
    { 0x1660de14, 100 },
    { 0xf460de15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb70de1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae50de21, 68 },
    { 0x16d0de22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e0de27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb0de2a, 98 },
    { 0x3670de2a, 23 },
    { 0x8e50de2b, 90 },
    { 0xe750de2b, 90 },
    { 0, 0 },
    { 0x1970de2f, 91 },
    { 0xb8c0de2f, 95 },
    { 0x9de0de31, 90 },
    { 0xb130de30, 51 },
    { 0x69b0de2f, 55 },
    { 0xcf0de34, 61 },
    { 0x2270de34, 33 },
    { 0x92a0de36, 88 },
    { 0x1640de37, 23 },
    { 0, 0 },
    { 0x9200de39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540de3d, 90 },
    { 0, 0 },
    { 0x6ab0de3f, 46 },
    { 0x3aa0de40, 61 },
    { 0xae30de3f, 95 },
    { 0xe990de41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9490de45, 95 },
    { 0x34a0de45, 98 },
    { 0, 0 },
    { 0xbcb0de48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d0de4b, 93 },
    { 0, 0 },
    { 0x16c0de4d, 90 },
    { 0x8a10de4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b0de52, 61 },
    { 0x5d70de53, 78 },
    { 0x7810de52, 95 },
    { 0xd2e0de55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f0de5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d90de61, 93 },
    { 0x8360de62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6850de65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0de68, 98 },
    { 0, 0 },
    { 0x750de6a, 34 },
    { 0, 0 },
    { 0x5b30de6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5350de70, 69 },
    { 0x39e0de71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410de75, 23 },
    { 0x9e10de76, 33 },
    { 0, 0 },
    { 0x8700de78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9750de7c, 66 },
    { 0, 0 },
    { 0x5f60de7e, 98 },
    { 0, 0 },
    { 0xfd40de80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb0de84, 76 },
    { 0x61a0de84, 99 },
    { 0xd0e0de85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac40de8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa0de90, 9 },
    { 0x2750de90, 95 },
    { 0xbcf0de90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a0de95, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c0de98, 90 },
    { 0xe640de99, 90 },
    { 0x59d0de9a, 87 },
    { 0, 0 },
    { 0x50f0de9c, 33 },
    { 0x75b0de9d, 83 },
    { 0x8900de9e, 66 },
    { 0, 0 },
    { 0x5790dea0, 52 },
    { 0xc2e0dea1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60dea4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe0dea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd00deb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b10deb9, 98 },
    { 0, 0 },
    { 0x3700debb, 34 },
    { 0x33e0debc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d0dec0, 61 },
    { 0x5ec0dec0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c0dec4, 66 },
    { 0x1e80dec4, 61 },
    { 0, 0 },
    { 0xfdc0dec7, 76 },
    { 0xe220dec8, 33 },
    { 0xa5d0dec9, 99 },
    { 0x94f0deca, 33 },
    { 0x5150decb, 84 },
    { 0x86b0deca, 92 },
    { 0, 0 },
    { 0x3f50dece, 52 },
    { 0, 0 },
    { 0x7680ded0, 90 },
    { 0, 0 },
    { 0x8720ded2, 91 },
    { 0x30ded2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3840ded9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc510dedd, 90 },
    { 0x1b30dede, 83 },
    { 0xa8d0dedf, 61 },
    { 0x900dee0, 84 },
    { 0, 0 },
    { 0x79a0dee2, 23 },
    { 0x7df0dee3, 33 },
    { 0xf930dee2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad50dee9, 53 },
    { 0xac50deea, 61 },
    { 0x6ba0dee9, 99 },
    { 0x3900deec, 90 },
    { 0xa930deed, 69 },
    { 0x8b20deec, 98 },
    { 0x5530deef, 77 },
    { 0x69a0def0, 83 },
    { 0x37f0deef, 91 },
    { 0x10f0def2, 23 },
    { 0x9390def1, 98 },
    { 0xca0def3, 90 },
    { 0x81d0def5, 98 },
    { 0x9130def4, 95 },
    { 0x7500deef, 69 },
    { 0x12d0def3, 98 },
    { 0, 0 },
    { 0x80f0defa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd0df03, 76 },
    { 0x7f70df03, 90 },
    { 0xec30df05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0df08, 98 },
    { 0, 0 },
    { 0xbed0df0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e0df0e, 91 },
    { 0xd6e0df0f, 67 },
    { 0x4480df10, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f0df15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f0df1b, 61 },
    { 0, 0 },
    { 0xf8a0df1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba0df22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f80df2a, 85 },
    { 0x99b0df2b, 77 },
    { 0x1f80df2b, 46 },
    { 0x77d0df2d, 61 },
    { 0x2bb0df2e, 65 },
    { 0x7aa0df2f, 88 },
    { 0xb360df2d, 23 },
    { 0, 0 },
    { 0x3600df32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b0df37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940df3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a50df43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8790df47, 33 },
    { 0xfd10df48, 23 },
    { 0x3c00df48, 61 },
    { 0, 0 },
    { 0xc950df4b, 62 },
    { 0xade0df4b, 98 },
    { 0xc7b0df4d, 23 },
    { 0, 0 },
    { 0xb130df4f, 76 },
    { 0, 0 },
    { 0xbd90df51, 23 },
    { 0, 0 },
    { 0x2560df53, 90 },
    { 0x1080df54, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a0df57, 90 },
    { 0xe760df58, 90 },
    { 0xd430df57, 61 },
    { 0x3650df57, 90 },
    { 0, 0 },
    { 0x1980df5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf200df5f, 33 },
    { 0x4ab0df5f, 33 },
    { 0, 0 },
    { 0xdc30df62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd10df65, 74 },
    { 0xc3f0df66, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb790df69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd890df71, 67 },
    { 0xee90df71, 61 },
    { 0x5520df73, 98 },
    { 0x35f0df71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2860df79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d10df80, 90 },
    { 0, 0 },
    { 0xef20df82, 95 },
    { 0x31b0df83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe950df8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c0df8f, 90 },
    { 0, 0 },
    { 0x78f0df91, 61 },
    { 0x15f0df91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0df96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f0df9b, 19 },
    { 0xc370df9c, 61 },
    { 0x1260df9d, 98 },
    { 0, 0 },
    { 0x93e0df9f, 55 },
    { 0xe7b0dfa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3210dfa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d0dfa8, 91 },
    { 0xb5e0dfa9, 95 },
    { 0x1df0dfa9, 95 },
    { 0, 0 },
    { 0x6af0dfac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d0dfb0, 76 },
    { 0, 0 },
    { 0xc140dfb2, 77 },
    { 0xd2b0dfb3, 76 },
    { 0xe660dfb4, 98 },
    { 0x9ad0dfb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db0dfc3, 33 },
    { 0, 0 },
    { 0x45c0dfc5, 23 },
    { 0, 0 },
    { 0x60dfc7, 33 },
    { 0x9ee0dfc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e20dfcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2890dfce, 91 },
    { 0xb870dfce, 99 },
    { 0x51c0dfce, 23 },
    { 0, 0 },
    { 0xa520dfd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf0dfda, 90 },
    { 0x6ba0dfdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e0dfe5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd00dfe8, 84 },
    { 0, 0 },
    { 0xd2f0dfea, 98 },
    { 0xba50dfea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa830dfef, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f80dff4, 90 },
    { 0x31e0dff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2070dffa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc410dffd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7340e001, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3130e008, 34 },
    { 0xd980e009, 23 },
    { 0xdb60e00a, 61 },
    { 0xb980e00b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1410e014, 90 },
    { 0x6ac0e014, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa00e01c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca0e024, 95 },
    { 0xd3c0e024, 75 },
    { 0, 0 },
    { 0x14d0e027, 98 },
    { 0xd1c0e028, 61 },
    { 0x4e30e027, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1810e031, 98 },
    { 0x1460e031, 77 },
    { 0, 0 },
    { 0x2d60e034, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb0e039, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9910e03c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b0e040, 95 },
    { 0x400e041, 43 },
    { 0xa060e040, 55 },
    { 0x8a60e042, 33 },
    { 0xfc80e040, 33 },
    { 0x1fb0e042, 23 },
    { 0x1440e044, 23 },
    { 0x640e047, 76 },
    { 0xa700e047, 61 },
    { 0x42e0e049, 34 },
    { 0xb20e044, 33 },
    { 0xd4f0e04b, 95 },
    { 0xad80e043, 90 },
    { 0xcf70e04d, 95 },
    { 0x81f0e046, 68 },
    { 0xdbc0e046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e0e054, 23 },
    { 0x9f30e054, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640e062, 33 },
    { 0, 0 },
    { 0x6ba0e064, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8550e06d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3020e070, 95 },
    { 0x1ad0e071, 33 },
    { 0x9910e072, 98 },
    { 0x7750e073, 60 },
    { 0x1380e074, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc80e077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd630e07b, 90 },
    { 0, 0 },
    { 0x7b60e07d, 33 },
    { 0xfb10e07e, 61 },
    { 0x8d30e07d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8880e087, 95 },
    { 0, 0 },
    { 0x4610e089, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb070e08c, 67 },
    { 0, 0 },
    { 0x7c80e08e, 88 },
    { 0, 0 },
    { 0x9e0e090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0e09a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc640e09d, 33 },
    { 0xd860e09d, 76 },
    { 0, 0 },
    { 0xfcd0e0a0, 61 },
    { 0x8110e0a1, 33 },
    { 0, 0 },
    { 0x1cb0e0a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f0e0a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2100e0ab, 98 },
    { 0x4fb0e0ac, 90 },
    { 0, 0 },
    { 0xc0e0e0ae, 33 },
    { 0x7a30e0af, 23 },
    { 0x51e0e0b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf60e0b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5610e0b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f0e0c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf890e0cd, 98 },
    { 0x4610e0ce, 33 },
    { 0xed90e0cd, 46 },
    { 0x7da0e0cf, 96 },
    { 0x18c0e0ce, 33 },
    { 0, 0 },
    { 0x2ec0e0d3, 98 },
    { 0x9fe0e0d4, 67 },
    { 0, 0 },
    { 0xcbb0e0d6, 90 },
    { 0x4760e0d7, 93 },
    { 0x4720e0d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc0e0de, 55 },
    { 0, 0 },
    { 0x6740e0e0, 90 },
    { 0xbf50e0e1, 52 },
    { 0xa030e0e2, 87 },
    { 0x5ce0e0e3, 8 },
    { 0x2db0e0e1, 55 },
    { 0x4430e0e4, 55 },
    { 0x30b0e0e6, 93 },
    { 0xdd50e0e6, 17 },
    { 0x4640e0e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d50e0f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f0e0f3, 55 },
    { 0xe440e0f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a60e0f7, 59 },
    { 0xcee0e0f8, 90 },
    { 0, 0 },
    { 0x3be0e0fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa750e0fe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e0e109, 24 },
    { 0x6880e10a, 90 },
    { 0x2d30e109, 76 },
    { 0xc8b0e10c, 34 },
    { 0x4770e10d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690e111, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb150e117, 61 },
    { 0x9320e118, 95 },
    { 0x1d80e117, 33 },
    { 0xd570e11a, 98 },
    { 0xc210e11a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe330e11e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f0e123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d0e126, 23 },
    { 0x9280e127, 90 },
    { 0x9bf0e127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db0e12b, 46 },
    { 0xd4d0e12b, 77 },
    { 0x8c60e12d, 55 },
    { 0xd300e12d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf00e131, 46 },
    { 0xb3a0e131, 36 },
    { 0, 0 },
    { 0x1e90e134, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa910e137, 76 },
    { 0x2460e138, 35 },
    { 0, 0 },
    { 0x3340e13a, 95 },
    { 0, 0 },
    { 0x2e0e13c, 90 },
    { 0x86c0e13d, 95 },
    { 0x2930e13e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e30e148, 66 },
    { 0, 0 },
    { 0x4770e14a, 90 },
    { 0, 0 },
    { 0xfc00e14c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d90e153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad0e159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f0e15c, 33 },
    { 0x86b0e15d, 98 },
    { 0, 0 },
    { 0x6080e15f, 33 },
    { 0, 0 },
    { 0x19c0e161, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250e166, 61 },
    { 0xcc50e166, 52 },
    { 0xee20e168, 77 },
    { 0, 0 },
    { 0x2c90e16a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x890e16d, 23 },
    { 0xae10e16d, 23 },
    { 0xb850e16f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca0e174, 78 },
    { 0xa830e174, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba0e17a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c20e17d, 90 },
    { 0x99f0e17d, 34 },
    { 0xaa90e17f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30e185, 83 },
    { 0x3320e186, 46 },
    { 0x89b0e187, 84 },
    { 0xc640e186, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910e18d, 90 },
    { 0x83e0e18d, 38 },
    { 0xfa40e18f, 55 },
    { 0x5750e190, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b0e195, 93 },
    { 0xae00e196, 61 },
    { 0x60e197, 66 },
    { 0, 0 },
    { 0x92b0e199, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6830e19f, 48 },
    { 0x5ad0e19f, 75 },
    { 0xc3a0e19f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d40e1a4, 66 },
    { 0x7bf0e1a5, 90 },
    { 0x3a00e1a4, 97 },
    { 0x8ef0e1a7, 61 },
    { 0xcb10e1a8, 98 },
    { 0xc9a0e1a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e0e1b2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc40e1b6, 100 },
    { 0xf6e0e1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf870e1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf40e1ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e1d1, 85 },
    { 0, 0 },
    { 0x4bc0e1d3, 95 },
    { 0, 0 },
    { 0xf680e1d5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8550e1da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e0e1e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa640e1e3, 90 },
    { 0, 0 },
    { 0xd710e1e5, 43 },
    { 0x89d0e1e6, 33 },
    { 0x6ed0e1e7, 43 },
    { 0x2ed0e1e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd60e1eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc010e1f0, 77 },
    { 0, 0 },
    { 0xfcd0e1f2, 98 },
    { 0x6d50e1f3, 34 },
    { 0xd680e1f4, 55 },
    { 0, 0 },
    { 0x9830e1f6, 95 },
    { 0x2400e1f7, 55 },
    { 0x5e90e1f8, 76 },
    { 0x4200e1f9, 33 },
    { 0xd880e1fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650e20a, 51 },
    { 0, 0 },
    { 0x20d0e20c, 78 },
    { 0, 0 },
    { 0x8c90e20e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad0e214, 84 },
    { 0xfe70e214, 87 },
    { 0x2fa0e216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a0e21c, 67 },
    { 0x790e21d, 98 },
    { 0x5eb0e21d, 52 },
    { 0x80d0e21d, 98 },
    { 0xb160e220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30e223, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb0e227, 95 },
    { 0xb400e228, 86 },
    { 0x8db0e227, 44 },
    { 0, 0 },
    { 0x45e0e22b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b40e22f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a30e232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe660e239, 90 },
    { 0, 0 },
    { 0x3a50e23b, 76 },
    { 0x5f0e23b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea0e240, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b90e245, 33 },
    { 0xa80e246, 98 },
    { 0xea60e247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf20e24b, 34 },
    { 0x9bb0e24b, 43 },
    { 0x2e90e24c, 61 },
    { 0x540e24b, 69 },
    { 0x5ab0e24f, 55 },
    { 0x50f0e250, 33 },
    { 0x1470e24e, 90 },
    { 0, 0 },
    { 0x61f0e253, 96 },
    { 0, 0 },
    { 0x710e255, 32 },
    { 0, 0 },
    { 0xa4e0e257, 66 },
    { 0, 0 },
    { 0x20c0e259, 34 },
    { 0, 0 },
    { 0x5f30e25b, 98 },
    { 0, 0 },
    { 0x8ca0e25d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250e261, 61 },
    { 0x5b00e262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c40e265, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd0e269, 99 },
    { 0x410e26a, 61 },
    { 0, 0 },
    { 0xce30e26c, 99 },
    { 0, 0 },
    { 0xff90e26e, 61 },
    { 0x7b40e26e, 84 },
    { 0x91e0e270, 90 },
    { 0x25e0e270, 77 },
    { 0x180e272, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0e275, 100 },
    { 0xd790e276, 93 },
    { 0xc890e277, 61 },
    { 0xc990e278, 76 },
    { 0x7a50e277, 36 },
    { 0x58c0e279, 46 },
    { 0x1110e27a, 88 },
    { 0, 0 },
    { 0x4ed0e27d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb660e280, 91 },
    { 0xd9e0e281, 81 },
    { 0x1ce0e281, 84 },
    { 0x720e282, 23 },
    { 0x91a0e282, 34 },
    { 0, 0 },
    { 0xab20e286, 34 },
    { 0x5090e287, 33 },
    { 0, 0 },
    { 0x5880e289, 99 },
    { 0, 0 },
    { 0x3840e28b, 61 },
    { 0x73b0e28b, 91 },
    { 0, 0 },
    { 0x2a10e28e, 95 },
    { 0x1a90e28f, 100 },
    { 0xcbc0e28e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb680e295, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc70e298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa200e2a4, 43 },
    { 0x9220e2a5, 98 },
    { 0xf7f0e2a6, 43 },
    { 0xe7a0e2a7, 95 },
    { 0, 0 },
    { 0xd800e2a9, 95 },
    { 0xda70e2aa, 69 },
    { 0x5c80e2ab, 95 },
    { 0x2bf0e2a9, 93 },
    { 0x5920e2ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c0e2b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe560e2b9, 90 },
    { 0x3540e2ba, 98 },
    { 0, 0 },
    { 0xb240e2bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f0e2bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f0e2c5, 90 },
    { 0x5ca0e2c6, 33 },
    { 0x3460e2c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab0e2cd, 55 },
    { 0, 0 },
    { 0x6600e2cf, 52 },
    { 0x1fa0e2cf, 33 },
    { 0, 0 },
    { 0x7d90e2d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf270e2db, 53 },
    { 0, 0 },
    { 0xe5e0e2dd, 95 },
    { 0x5880e2de, 94 },
    { 0xbaa0e2de, 90 },
    { 0xf6b0e2e0, 33 },
    { 0, 0 },
    { 0xffa0e2e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3350e2e7, 95 },
    { 0x7ed0e2e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500e2ec, 78 },
    { 0xa3c0e2ec, 66 },
    { 0x62e0e2ed, 61 },
    { 0, 0 },
    { 0xcf40e2f0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef0e2f4, 61 },
    { 0x3ce0e2f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a0e2f9, 76 },
    { 0x4260e2fa, 53 },
    { 0, 0 },
    { 0xc150e2fc, 23 },
    { 0, 0 },
    { 0xcb0e2fe, 90 },
    { 0xafe0e2fe, 34 },
    { 0xbba0e300, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b0e307, 23 },
    { 0xd0f0e307, 33 },
    { 0x1680e309, 95 },
    { 0xe880e30a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5130e312, 98 },
    { 0x9c60e312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef80e316, 43 },
    { 0x5d30e317, 80 },
    { 0x2f90e318, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca0e31b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9170e322, 33 },
    { 0x6da0e322, 98 },
    { 0, 0 },
    { 0xc680e325, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3490e329, 99 },
    { 0xd490e32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd080e32e, 34 },
    { 0xfce0e32f, 90 },
    { 0x1970e32e, 23 },
    { 0x6250e331, 66 },
    { 0xe8e0e331, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060e336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa10e342, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8050e345, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab0e34a, 61 },
    { 0x52c0e34b, 90 },
    { 0x6730e34b, 95 },
    { 0x6b0e34c, 95 },
    { 0xed80e34c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa510e352, 68 },
    { 0, 0 },
    { 0xae00e354, 77 },
    { 0x72b0e355, 61 },
    { 0, 0 },
    { 0xb450e357, 17 },
    { 0x47b0e357, 76 },
    { 0, 0 },
    { 0xa380e35a, 85 },
    { 0, 0 },
    { 0xb860e35c, 95 },
    { 0xbd00e35d, 61 },
    { 0, 0 },
    { 0x7ac0e35f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb090e362, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee70e36e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf100e371, 78 },
    { 0, 0 },
    { 0x58e0e373, 4 },
    { 0, 0 },
    { 0x1370e375, 90 },
    { 0, 0 },
    { 0x3420e377, 61 },
    { 0, 0 },
    { 0x610e379, 90 },
    { 0, 0 },
    { 0x21f0e37b, 52 },
    { 0xc6c0e37c, 90 },
    { 0xe020e37d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac0e380, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a0e384, 61 },
    { 0xdd0e384, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8610e38a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd0e393, 90 },
    { 0xa770e394, 23 },
    { 0xacf0e395, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf530e399, 61 },
    { 0, 0 },
    { 0x5ab0e39b, 23 },
    { 0x6080e39c, 33 },
    { 0xf890e39d, 43 },
    { 0, 0 },
    { 0x40d0e39f, 95 },
    { 0x4c50e3a0, 66 },
    { 0x5c70e3a1, 23 },
    { 0, 0 },
    { 0x7e90e3a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120e3a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb00e3b0, 34 },
    { 0, 0 },
    { 0xf800e3b2, 33 },
    { 0x5f0e3b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc500e3b7, 33 },
    { 0xcbe0e3b8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd730e3bc, 33 },
    { 0x8c00e3bd, 33 },
    { 0x5bb0e3bd, 33 },
    { 0, 0 },
    { 0x5e70e3c0, 33 },
    { 0, 0 },
    { 0x87f0e3c2, 46 },
    { 0xfc0e3c3, 65 },
    { 0xf160e3c2, 95 },
    { 0xdc80e3c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf270e3cc, 98 },
    { 0, 0 },
    { 0x690e3ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb0e3d2, 95 },
    { 0x61d0e3d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe140e3d9, 76 },
    { 0x70e3d9, 52 },
    { 0xcb90e3db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4020e3df, 88 },
    { 0x8650e3df, 99 },
    { 0, 0 },
    { 0x3f50e3e2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c80e3e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf110e3ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f0e3f2, 34 },
    { 0xac20e3f3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e0e3f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe20e3fe, 84 },
    { 0xa850e3ff, 76 },
    { 0x29c0e400, 33 },
    { 0, 0 },
    { 0x3b0e402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae0e407, 43 },
    { 0xad30e408, 68 },
    { 0x50a0e409, 88 },
    { 0xcb0e409, 95 },
    { 0x85c0e408, 33 },
    { 0x4510e40a, 61 },
    { 0x5380e40b, 90 },
    { 0x1d20e40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf150e411, 100 },
    { 0x3ed0e411, 61 },
    { 0, 0 },
    { 0xa7c0e414, 23 },
    { 0x3230e414, 76 },
    { 0x4dc0e415, 33 },
    { 0x4170e415, 66 },
    { 0x6440e418, 23 },
    { 0x5f80e415, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f0e41c, 81 },
    { 0x480e41d, 95 },
    { 0x67d0e41d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb0e423, 40 },
    { 0, 0 },
    { 0x69d0e425, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c0e42b, 78 },
    { 0, 0 },
    { 0x9f30e42d, 23 },
    { 0x5a40e42d, 34 },
    { 0, 0 },
    { 0xc9d0e430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf710e433, 61 },
    { 0xd130e434, 87 },
    { 0x2a80e435, 98 },
    { 0x5ce0e433, 55 },
    { 0xee20e437, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8600e43b, 90 },
    { 0, 0 },
    { 0x7770e43d, 23 },
    { 0, 0 },
    { 0xc3a0e43f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa10e444, 91 },
    { 0x66b0e445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d00e449, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b0e44c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d30e452, 33 },
    { 0xd330e453, 76 },
    { 0, 0 },
    { 0x9100e455, 23 },
    { 0x9fc0e455, 52 },
    { 0xd040e455, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f10e45b, 87 },
    { 0xc590e45c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90e461, 33 },
    { 0x2d60e461, 90 },
    { 0, 0 },
    { 0x2af0e464, 33 },
    { 0x84b0e464, 55 },
    { 0xed00e466, 53 },
    { 0xe4a0e465, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0e46c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2800e47c, 90 },
    { 0x4a10e47d, 90 },
    { 0, 0 },
    { 0xd3c0e47f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef40e483, 90 },
    { 0xbb90e484, 33 },
    { 0xdee0e485, 76 },
    { 0xbc00e486, 78 },
    { 0xc5d0e487, 95 },
    { 0, 0 },
    { 0x9e00e489, 33 },
    { 0xe1e0e48a, 52 },
    { 0xc820e48a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa410e490, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce0e497, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea40e4a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x910e4a4, 95 },
    { 0x5160e4a4, 90 },
    { 0x9c90e4a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d0e4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a10e4ac, 100 },
    { 0x4570e4ac, 90 },
    { 0, 0 },
    { 0xde0e4af, 33 },
    { 0x2480e4b0, 23 },
    { 0x7e40e4b1, 33 },
    { 0xe610e4b1, 33 },
    { 0x3b20e4b1, 55 },
    { 0, 0 },
    { 0xa3e0e4b5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c70e4c0, 95 },
    { 0, 0 },
    { 0xd6b0e4c2, 33 },
    { 0, 0 },
    { 0xe2e0e4c4, 98 },
    { 0x8930e4c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060e4c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8000e4cc, 71 },
    { 0x8280e4cc, 98 },
    { 0x4bd0e4cc, 90 },
    { 0, 0 },
    { 0x8a80e4d0, 88 },
    { 0xf6c0e4d0, 61 },
    { 0, 0 },
    { 0xaf20e4d3, 61 },
    { 0x68b0e4d3, 98 },
    { 0x3e80e4d3, 95 },
    { 0x440e4d6, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea90e4da, 55 },
    { 0x8830e4db, 90 },
    { 0x2030e4da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd170e4e0, 95 },
    { 0x69f0e4e0, 98 },
    { 0x87f0e4e2, 66 },
    { 0xc90e4e3, 76 },
    { 0xb330e4e2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c50e4ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630e4f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc010e4f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690e4f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c0e500, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c60e50a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d90e50f, 61 },
    { 0x5d80e510, 90 },
    { 0x2c40e511, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650e514, 77 },
    { 0x4ca0e515, 33 },
    { 0x630e515, 23 },
    { 0x8df0e516, 89 },
    { 0xcfb0e518, 23 },
    { 0xe4a0e519, 98 },
    { 0x2e0e51a, 98 },
    { 0x70a0e517, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc0e523, 66 },
    { 0xb2a0e524, 52 },
    { 0, 0 },
    { 0xf470e526, 95 },
    { 0x3eb0e527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc910e52c, 34 },
    { 0x8170e52d, 33 },
    { 0, 0 },
    { 0xf330e52f, 99 },
    { 0x86f0e530, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb0e536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2800e53e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab0e545, 99 },
    { 0, 0 },
    { 0x9e20e547, 64 },
    { 0, 0 },
    { 0x1c50e549, 98 },
    { 0, 0 },
    { 0x4bc0e54b, 33 },
    { 0xdda0e54b, 61 },
    { 0, 0 },
    { 0x1490e54e, 90 },
    { 0x7970e54f, 23 },
    { 0x2470e550, 90 },
    { 0xa720e550, 61 },
    { 0x9e90e552, 80 },
    { 0x16c0e552, 98 },
    { 0xd290e552, 90 },
    { 0, 0 },
    { 0xca60e556, 61 },
    { 0xc2f0e556, 95 },
    { 0xdc70e558, 23 },
    { 0, 0 },
    { 0x5080e55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7180e55f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f90e569, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d50e570, 61 },
    { 0xa4f0e571, 43 },
    { 0xad90e572, 33 },
    { 0x7110e570, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540e576, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5690e579, 93 },
    { 0x1f80e57a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce00e57d, 98 },
    { 0x4b60e57e, 61 },
    { 0xdb90e57f, 38 },
    { 0, 0 },
    { 0xabf0e581, 95 },
    { 0x8b10e581, 61 },
    { 0, 0 },
    { 0xa2f0e584, 94 },
    { 0xd050e585, 98 },
    { 0xa740e586, 43 },
    { 0x2b70e585, 61 },
    { 0x3520e588, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9990e58b, 98 },
    { 0, 0 },
    { 0xc6b0e58d, 61 },
    { 0xf890e58e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f60e594, 77 },
    { 0, 0 },
    { 0xfd30e596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc90e599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70e59f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd590e5a2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed0e5a5, 98 },
    { 0, 0 },
    { 0x2f10e5a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b50e5ab, 78 },
    { 0x6ef0e5ab, 48 },
    { 0, 0 },
    { 0x9f20e5ae, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa350e5b1, 93 },
    { 0x8fa0e5b1, 100 },
    { 0x49f0e5b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc110e5b6, 17 },
    { 0, 0 },
    { 0xc970e5b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e0e5bc, 91 },
    { 0x9690e5bd, 46 },
    { 0xd010e5be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea0e5c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba40e5c6, 95 },
    { 0x7d80e5c6, 23 },
    { 0x260e5c6, 99 },
    { 0x9120e5c9, 95 },
    { 0x16a0e5ca, 76 },
    { 0xba90e5c8, 36 },
    { 0x3480e5cc, 55 },
    { 0x4680e5c9, 33 },
    { 0, 0 },
    { 0xb8f0e5cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be0e5d4, 85 },
    { 0x60a0e5d5, 90 },
    { 0, 0 },
    { 0x5010e5d7, 93 },
    { 0x930e5d8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a0e5db, 96 },
    { 0, 0 },
    { 0x2d00e5dd, 34 },
    { 0xcb20e5de, 23 },
    { 0x3ac0e5df, 100 },
    { 0xe530e5e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b00e5e4, 43 },
    { 0xa6a0e5e5, 43 },
    { 0x4950e5e5, 61 },
    { 0xbbc0e5e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9290e5ea, 95 },
    { 0x59a0e5eb, 90 },
    { 0x7f10e5ec, 43 },
    { 0x74a0e5ed, 76 },
    { 0x58a0e5ee, 90 },
    { 0x1660e5ea, 61 },
    { 0xafb0e5f0, 90 },
    { 0x3920e5f1, 87 },
    { 0xe270e5f1, 61 },
    { 0xd110e5f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x200e5f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd560e5fa, 90 },
    { 0x7c40e5fb, 93 },
    { 0x65a0e5fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de0e600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6630e603, 90 },
    { 0x99a0e604, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d50e607, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b40e60a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910e60f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf030e616, 61 },
    { 0x59b0e617, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2250e61a, 98 },
    { 0xac00e61b, 61 },
    { 0, 0 },
    { 0x6b40e61d, 55 },
    { 0x2200e61e, 33 },
    { 0, 0 },
    { 0x2bd0e620, 99 },
    { 0x3cc0e621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee80e624, 84 },
    { 0, 0 },
    { 0x1410e626, 52 },
    { 0x6de0e627, 55 },
    { 0x8b80e626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f70e62b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c0e62f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6300e636, 23 },
    { 0xb2e0e637, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe00e63a, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40e63d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e0e641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea0e646, 77 },
    { 0x2060e646, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3980e654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba0e659, 98 },
    { 0, 0 },
    { 0xe620e65b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf970e669, 95 },
    { 0, 0 },
    { 0xd520e66b, 84 },
    { 0x1b50e66b, 95 },
    { 0xf6f0e66b, 61 },
    { 0x5310e66e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d0e671, 69 },
    { 0x50f0e672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7450e678, 76 },
    { 0x8230e679, 88 },
    { 0xc310e67a, 90 },
    { 0x5c80e67b, 34 },
    { 0x48c0e679, 90 },
    { 0, 0 },
    { 0xdbe0e67e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba20e682, 91 },
    { 0x4fa0e683, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb0e687, 33 },
    { 0x32d0e687, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e0e68b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff0e68f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3600e694, 76 },
    { 0, 0 },
    { 0xc6a0e696, 55 },
    { 0, 0 },
    { 0x250e698, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f40e69b, 84 },
    { 0xdf60e69c, 90 },
    { 0xf250e69d, 23 },
    { 0, 0 },
    { 0xf7b0e69f, 23 },
    { 0x76a0e6a0, 83 },
    { 0xf950e6a1, 55 },
    { 0, 0 },
    { 0x35d0e6a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa370e6a6, 43 },
    { 0xa690e6a7, 55 },
    { 0xa210e6a8, 98 },
    { 0xf0e0e6a9, 95 },
    { 0, 0 },
    { 0x8f70e6ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4960e6b0, 84 },
    { 0x77d0e6b1, 95 },
    { 0x97e0e6b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e6ba, 90 },
    { 0, 0 },
    { 0x7560e6bc, 61 },
    { 0x18d0e6bd, 52 },
    { 0xc1b0e6bd, 90 },
    { 0x2500e6bd, 87 },
    { 0xd850e6bd, 23 },
    { 0, 0 },
    { 0x8220e6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f70e6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b20e6c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa290e6d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb150e6d4, 98 },
    { 0x5150e6d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780e6dd, 61 },
    { 0xcb40e6de, 61 },
    { 0, 0 },
    { 0x65e0e6e0, 84 },
    { 0x7870e6e1, 90 },
    { 0, 0 },
    { 0x3a10e6e3, 23 },
    { 0x2480e6e4, 23 },
    { 0x6370e6e4, 33 },
    { 0xd80e6e5, 98 },
    { 0x30e0e6e5, 66 },
    { 0xc380e6e7, 95 },
    { 0xf450e6e9, 9 },
    { 0xc100e6e7, 90 },
    { 0xb8e0e6ea, 90 },
    { 0, 0 },
    { 0x92e0e6ed, 95 },
    { 0x65e0e6ee, 55 },
    { 0xdf40e6ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3360e6f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf0e6fa, 61 },
    { 0x8930e6fb, 76 },
    { 0x4730e6fc, 23 },
    { 0x9940e6fd, 23 },
    { 0x2120e6fe, 100 },
    { 0x5da0e6fd, 90 },
    { 0x6130e6fd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e706, 76 },
    { 0, 0 },
    { 0xa070e708, 98 },
    { 0, 0 },
    { 0x87d0e70a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330e70d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed0e710, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e60e713, 90 },
    { 0x2c40e714, 67 },
    { 0xeff0e715, 85 },
    { 0, 0 },
    { 0xf2a0e717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4320e71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6260e71e, 95 },
    { 0xfbf0e71f, 33 },
    { 0, 0 },
    { 0x7c70e721, 23 },
    { 0x3620e722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e0e725, 90 },
    { 0x2580e725, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d0e729, 33 },
    { 0, 0 },
    { 0xdf50e72b, 61 },
    { 0, 0 },
    { 0xd820e72d, 55 },
    { 0x25d0e72d, 93 },
    { 0xed50e72f, 61 },
    { 0, 0 },
    { 0x3360e731, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a60e734, 61 },
    { 0x2390e735, 66 },
    { 0xf800e734, 76 },
    { 0, 0 },
    { 0x22e0e738, 77 },
    { 0, 0 },
    { 0x3230e73a, 95 },
    { 0xa310e73b, 46 },
    { 0x9f60e73b, 61 },
    { 0x9fc0e73d, 17 },
    { 0, 0 },
    { 0xf080e73f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb300e745, 23 },
    { 0x5870e746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe00e74a, 83 },
    { 0xb870e74b, 90 },
    { 0xc090e74c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8120e752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e757, 99 },
    { 0x4ca0e758, 66 },
    { 0x1df0e758, 90 },
    { 0x8fa0e757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff0e75d, 90 },
    { 0xbc20e75e, 77 },
    { 0xf130e75d, 93 },
    { 0xd9a0e75f, 93 },
    { 0xa9a0e761, 90 },
    { 0xc090e761, 61 },
    { 0xeba0e763, 33 },
    { 0x9850e764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6610e767, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd0e76d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920e770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f60e773, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d0e779, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f0e77c, 33 },
    { 0x90a0e77d, 85 },
    { 0x3630e77e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf720e781, 81 },
    { 0x4e0e782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8640e785, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9160e78b, 99 },
    { 0, 0 },
    { 0x5ae0e78d, 55 },
    { 0x1600e78e, 92 },
    { 0xa850e78f, 95 },
    { 0xed00e790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5810e793, 100 },
    { 0xca60e793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff0e799, 58 },
    { 0, 0 },
    { 0xe510e79b, 98 },
    { 0, 0 },
    { 0xa730e79d, 34 },
    { 0, 0 },
    { 0x7340e79f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630e7a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0e7a8, 61 },
    { 0x6ae0e7a8, 61 },
    { 0x5b30e7a8, 90 },
    { 0x93d0e7a9, 33 },
    { 0xa960e7ac, 61 },
    { 0x38b0e7ab, 85 },
    { 0x4dc0e7ae, 99 },
    { 0x6300e7ad, 33 },
    { 0x6cc0e7a8, 65 },
    { 0x73b0e7b1, 84 },
    { 0, 0 },
    { 0xbf80e7b3, 76 },
    { 0xecd0e7b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0e7b8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e0e7bd, 99 },
    { 0x7ad0e7be, 91 },
    { 0xc9a0e7bf, 90 },
    { 0xfaa0e7bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80e7c7, 91 },
    { 0xf410e7c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf260e7cf, 23 },
    { 0, 0 },
    { 0xfc60e7d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630e7d8, 90 },
    { 0x7dd0e7d9, 78 },
    { 0x30e7d8, 43 },
    { 0xc680e7da, 34 },
    { 0x7190e7d8, 99 },
    { 0x4f90e7dd, 98 },
    { 0xf440e7de, 90 },
    { 0, 0 },
    { 0x7a00e7e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa430e7e5, 90 },
    { 0xd210e7e5, 100 },
    { 0xd660e7e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff00e7ed, 93 },
    { 0xde90e7ee, 23 },
    { 0, 0 },
    { 0x66d0e7f0, 23 },
    { 0, 0 },
    { 0x790e7f2, 90 },
    { 0x1360e7f3, 76 },
    { 0xb2a0e7f4, 93 },
    { 0x7df0e7f5, 90 },
    { 0x1300e7f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e30e7fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b0e801, 75 },
    { 0xa0f0e802, 84 },
    { 0x5e0e802, 33 },
    { 0xca70e804, 98 },
    { 0xfd50e805, 61 },
    { 0x270e805, 55 },
    { 0xb5e0e807, 90 },
    { 0xc0f0e808, 90 },
    { 0, 0 },
    { 0xbd00e80a, 33 },
    { 0x4c40e80b, 81 },
    { 0x6950e80a, 33 },
    { 0x6f90e80c, 95 },
    { 0xdd70e80c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540e818, 40 },
    { 0x4520e819, 100 },
    { 0, 0 },
    { 0xcb00e81b, 76 },
    { 0x3880e81c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5270e821, 91 },
    { 0, 0 },
    { 0x22e0e823, 76 },
    { 0x76a0e823, 33 },
    { 0xfc60e825, 90 },
    { 0xa000e826, 23 },
    { 0x41c0e825, 95 },
    { 0x7ef0e828, 95 },
    { 0x1640e828, 34 },
    { 0x1ed0e82a, 98 },
    { 0xd330e82b, 90 },
    { 0, 0 },
    { 0x7b60e82d, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e0e832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd0e838, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9880e83b, 90 },
    { 0, 0 },
    { 0xce20e83d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0e84a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea0e84f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c0e853, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de0e85b, 23 },
    { 0x810e85b, 90 },
    { 0, 0 },
    { 0x5cb0e85e, 65 },
    { 0xfc0e85e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0e865, 90 },
    { 0xd20e866, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8970e86b, 95 },
    { 0xaf20e86b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a0e872, 98 },
    { 0xdbe0e873, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70e87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc320e87f, 61 },
    { 0x6300e880, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa760e885, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2910e888, 61 },
    { 0xea60e889, 99 },
    { 0x2f70e888, 61 },
    { 0, 0 },
    { 0x1c20e88c, 84 },
    { 0xcc00e88c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f70e892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e0e895, 90 },
    { 0x4880e896, 61 },
    { 0xb0e0e896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d0e89a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed0e89f, 91 },
    { 0xd6b0e8a0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa80e8a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc890e8ac, 100 },
    { 0xbc0e8ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690e8b0, 33 },
    { 0x39c0e8b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d90e8b6, 61 },
    { 0, 0 },
    { 0x15a0e8b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530e8bb, 91 },
    { 0x1ac0e8bc, 33 },
    { 0, 0 },
    { 0x2f70e8be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc0e8c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8400e8c8, 90 },
    { 0, 0 },
    { 0x1760e8ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa0e8ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40e8d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd80e8d4, 23 },
    { 0x4870e8d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2070e8d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b0e8e0, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f30e8e5, 34 },
    { 0, 0 },
    { 0xdca0e8e7, 33 },
    { 0, 0 },
    { 0x8e60e8e9, 23 },
    { 0, 0 },
    { 0xde80e8eb, 95 },
    { 0xc4a0e8ec, 46 },
    { 0x6f00e8ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe790e8f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5300e8fb, 99 },
    { 0x3bc0e8fc, 55 },
    { 0xbaf0e8fb, 76 },
    { 0x7dd0e8fb, 91 },
    { 0x7f10e8ff, 90 },
    { 0x3cf0e8ff, 100 },
    { 0x68c0e901, 55 },
    { 0xdb30e901, 84 },
    { 0x66b0e902, 99 },
    { 0, 0 },
    { 0xc210e905, 97 },
    { 0x5d70e905, 67 },
    { 0xe580e907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9860e90c, 61 },
    { 0x5ed0e90d, 87 },
    { 0xc690e90e, 97 },
    { 0x8960e90f, 61 },
    { 0xf630e910, 53 },
    { 0xe150e911, 23 },
    { 0xf720e912, 87 },
    { 0xcb70e913, 77 },
    { 0xd520e914, 40 },
    { 0xefe0e914, 46 },
    { 0x6080e913, 76 },
    { 0xbf30e90c, 43 },
    { 0xb3c0e90e, 33 },
    { 0xd8b0e919, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf0e920, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f40e929, 90 },
    { 0xa690e929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d0e92e, 90 },
    { 0x3e60e92f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde80e933, 90 },
    { 0xb170e934, 46 },
    { 0, 0 },
    { 0x8160e936, 90 },
    { 0x48a0e937, 91 },
    { 0x41b0e938, 85 },
    { 0x8110e939, 33 },
    { 0x3c10e93a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a0e93d, 95 },
    { 0x4e00e93e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a60e941, 61 },
    { 0x6dd0e942, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db0e946, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70e951, 93 },
    { 0, 0 },
    { 0xa840e953, 90 },
    { 0xefd0e954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea0e95a, 100 },
    { 0x7f10e95b, 76 },
    { 0, 0 },
    { 0xc00e95d, 34 },
    { 0xcaf0e95e, 33 },
    { 0xd70e95e, 91 },
    { 0xe840e960, 90 },
    { 0x3da0e960, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7360e967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0e96a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd0e96e, 33 },
    { 0xb3b0e96f, 90 },
    { 0, 0 },
    { 0xfbb0e971, 33 },
    { 0xcdb0e971, 43 },
    { 0xb850e972, 90 },
    { 0x2d0e974, 95 },
    { 0xb1a0e974, 88 },
    { 0x1860e976, 93 },
    { 0xb5d0e977, 61 },
    { 0xd0e978, 55 },
    { 0xfe40e979, 90 },
    { 0xdb30e978, 90 },
    { 0x36b0e97b, 97 },
    { 0, 0 },
    { 0xdd30e97d, 95 },
    { 0, 0 },
    { 0x6170e97f, 33 },
    { 0xe5e0e980, 33 },
    { 0xb200e97f, 95 },
    { 0xa1b0e981, 90 },
    { 0, 0 },
    { 0xcfa0e984, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7120e990, 90 },
    { 0xe4b0e991, 33 },
    { 0, 0 },
    { 0x8740e993, 33 },
    { 0x46b0e993, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570e999, 91 },
    { 0x910e999, 23 },
    { 0x6040e99b, 78 },
    { 0xc2f0e99c, 91 },
    { 0x71c0e999, 95 },
    { 0x2760e99e, 23 },
    { 0xb490e99f, 33 },
    { 0x9e40e9a0, 91 },
    { 0x15e0e99f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c0e9a5, 90 },
    { 0xbfa0e9a5, 65 },
    { 0xaba0e9a5, 23 },
    { 0x6fd0e9a7, 90 },
    { 0, 0 },
    { 0xfc80e9aa, 87 },
    { 0x36e0e9ab, 23 },
    { 0x9920e9aa, 34 },
    { 0, 0 },
    { 0xb730e9ae, 76 },
    { 0x74a0e9ae, 90 },
    { 0, 0 },
    { 0x44c0e9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f0e9b4, 23 },
    { 0x4ea0e9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380e9b8, 55 },
    { 0x37a0e9b9, 77 },
    { 0xb8b0e9ba, 23 },
    { 0x7750e9bb, 91 },
    { 0x9000e9bb, 33 },
    { 0x4f10e9bd, 40 },
    { 0, 0 },
    { 0x6700e9bf, 61 },
    { 0, 0 },
    { 0xc4e0e9c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6340e9c4, 88 },
    { 0xff80e9c5, 98 },
    { 0xb140e9c6, 98 },
    { 0x9930e9c6, 98 },
    { 0xb100e9c6, 93 },
    { 0, 0 },
    { 0xdf20e9ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3060e9ce, 33 },
    { 0xc2f0e9ce, 90 },
    { 0xb730e9cf, 33 },
    { 0xea0e9d1, 95 },
    { 0x4760e9d0, 98 },
    { 0x6490e9d1, 43 },
    { 0x6670e9d4, 87 },
    { 0x6050e9d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d50e9d9, 46 },
    { 0xce10e9da, 48 },
    { 0x59c0e9db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b70e9e1, 34 },
    { 0, 0 },
    { 0x15a0e9e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4890e9e9, 95 },
    { 0xd5b0e9ea, 95 },
    { 0x4a30e9eb, 99 },
    { 0x9960e9ec, 33 },
    { 0x2610e9ed, 52 },
    { 0, 0 },
    { 0x3650e9ef, 98 },
    { 0x6830e9f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb0e9fa, 95 },
    { 0x35a0e9fb, 90 },
    { 0, 0 },
    { 0x8ce0e9fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c0ea05, 92 },
    { 0, 0 },
    { 0xab90ea07, 23 },
    { 0, 0 },
    { 0x5880ea09, 98 },
    { 0, 0 },
    { 0x9a60ea0b, 98 },
    { 0xbf10ea0b, 90 },
    { 0x5b0ea0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb0ea10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x210ea13, 98 },
    { 0xa430ea13, 55 },
    { 0x7310ea15, 90 },
    { 0xe180ea16, 98 },
    { 0x4c00ea14, 55 },
    { 0x7ae0ea18, 19 },
    { 0xcc60ea19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb780ea1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380ea20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c10ea23, 61 },
    { 0, 0 },
    { 0x8e90ea25, 90 },
    { 0xe620ea25, 90 },
    { 0xac80ea26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea60ea2a, 98 },
    { 0, 0 },
    { 0x1040ea2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7920ea36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb540ea3d, 61 },
    { 0x8f40ea3e, 91 },
    { 0xae90ea3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2570ea42, 81 },
    { 0x3d50ea42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe990ea4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5780ea4f, 61 },
    { 0x56a0ea50, 98 },
    { 0x23c0ea50, 52 },
    { 0xb710ea52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac0ea57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d0ea60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ea65, 98 },
    { 0x23b0ea66, 33 },
    { 0, 0 },
    { 0xe1f0ea68, 98 },
    { 0x67a0ea68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e40ea6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f60ea71, 90 },
    { 0xb4e0ea72, 90 },
    { 0x1c80ea72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b0ea7b, 61 },
    { 0x3470ea7c, 55 },
    { 0xbd30ea7d, 33 },
    { 0, 0 },
    { 0x6510ea7f, 61 },
    { 0xe070ea7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7220ea84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0ea88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf430ea8d, 98 },
    { 0, 0 },
    { 0x2720ea8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f0ea92, 34 },
    { 0, 0 },
    { 0x4ac0ea94, 23 },
    { 0xe460ea95, 52 },
    { 0xb000ea96, 61 },
    { 0, 0 },
    { 0x4520ea98, 93 },
    { 0x6fc0ea99, 95 },
    { 0xde90ea9a, 90 },
    { 0, 0 },
    { 0x6460ea9c, 43 },
    { 0, 0 },
    { 0x6950ea9e, 33 },
    { 0x61a0ea9f, 23 },
    { 0xf6a0eaa0, 98 },
    { 0x9d20eaa0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc0eaa4, 90 },
    { 0x2e40eaa5, 98 },
    { 0, 0 },
    { 0x73d0eaa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf10eaab, 77 },
    { 0x2990eaac, 95 },
    { 0xcae0eaac, 98 },
    { 0x7030eaab, 61 },
    { 0x72a0eaaf, 36 },
    { 0x4640eaaf, 90 },
    { 0x2cf0eaad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5240eab4, 90 },
    { 0, 0 },
    { 0x8c20eab6, 90 },
    { 0x6a70eab7, 61 },
    { 0, 0 },
    { 0x2e50eab9, 34 },
    { 0, 0 },
    { 0x8b30eabb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6700eac0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1620eac3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1550eaca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb650eace, 61 },
    { 0xb2e0eacf, 61 },
    { 0x4340eace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc150ead9, 23 },
    { 0x99e0eada, 95 },
    { 0x7aa0eadb, 98 },
    { 0x5140eadc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120eae0, 93 },
    { 0x5790eae1, 99 },
    { 0x5650eae0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6080eae9, 67 },
    { 0xadc0eaea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b0eaed, 90 },
    { 0xad80eaed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e0eaf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f0eaf8, 21 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7850eafe, 61 },
    { 0xc340eafe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1150eb02, 33 },
    { 0xbb0eb02, 96 },
    { 0xcaf0eb04, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa0eb09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc620eb0e, 61 },
    { 0, 0 },
    { 0xd320eb10, 93 },
    { 0xee10eb11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd380eb18, 17 },
    { 0xb830eb19, 95 },
    { 0xcd80eb19, 90 },
    { 0xc0eb1b, 77 },
    { 0, 0 },
    { 0xe6a0eb1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf00eb26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x710eb29, 33 },
    { 0xb030eb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0eb2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a0eb33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc0eb37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b0eb3a, 33 },
    { 0x5a10eb3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc800eb40, 69 },
    { 0xaa0eb41, 77 },
    { 0, 0 },
    { 0xab90eb43, 99 },
    { 0xcd10eb44, 34 },
    { 0, 0 },
    { 0x4c70eb46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1710eb4d, 90 },
    { 0xcad0eb4e, 90 },
    { 0xdeb0eb4e, 95 },
    { 0, 0 },
    { 0xff80eb51, 90 },
    { 0, 0 },
    { 0xa9b0eb53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6490eb56, 33 },
    { 0xcec0eb57, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x710eb5a, 90 },
    { 0, 0 },
    { 0x5380eb5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2170eb60, 85 },
    { 0xf460eb61, 33 },
    { 0xde30eb61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c50eb71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac0eb7a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb610eb7d, 90 },
    { 0x2230eb7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810eb84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d0eb87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f90eb8a, 61 },
    { 0, 0 },
    { 0xc720eb8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f0eb8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd0eb99, 33 },
    { 0, 0 },
    { 0x6020eb9b, 61 },
    { 0xd350eb9c, 61 },
    { 0xc4b0eb9c, 34 },
    { 0x66a0eb9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0eba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e0ebaa, 33 },
    { 0xca50ebab, 85 },
    { 0x6b80ebab, 99 },
    { 0x2100ebad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7010ebb0, 98 },
    { 0x7fe0ebb0, 43 },
    { 0xac80ebb2, 98 },
    { 0x14a0ebb0, 23 },
    { 0xce50ebb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f0ebb7, 93 },
    { 0xb800ebb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff80ebbc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc870ebc5, 33 },
    { 0, 0 },
    { 0x180ebc7, 61 },
    { 0, 0 },
    { 0x72b0ebc9, 78 },
    { 0x9970ebca, 61 },
    { 0x58e0ebcb, 33 },
    { 0x4d30ebcc, 96 },
    { 0x30e0ebcd, 98 },
    { 0x400ebce, 34 },
    { 0xa730ebce, 98 },
    { 0xc610ebcb, 76 },
    { 0xfce0ebd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca0ebd6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc0ebda, 90 },
    { 0xf150ebdb, 33 },
    { 0x7620ebdc, 99 },
    { 0xa7e0ebda, 33 },
    { 0xd030ebdd, 33 },
    { 0x15a0ebdf, 98 },
    { 0x38d0ebe0, 76 },
    { 0xb810ebe1, 90 },
    { 0xfba0ebdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb0ebe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d30ebeb, 80 },
    { 0x3570ebec, 90 },
    { 0x7ee0ebed, 55 },
    { 0x2590ebed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf160ebf1, 55 },
    { 0x2890ebf2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0ebfc, 95 },
    { 0, 0 },
    { 0x7bb0ebfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c30ec07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed0ec0e, 46 },
    { 0x7250ec0f, 80 },
    { 0, 0 },
    { 0x8340ec11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e80ec14, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc210ec1c, 46 },
    { 0, 0 },
    { 0x9c0ec1e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2140ec21, 33 },
    { 0x9f50ec22, 55 },
    { 0x92b0ec23, 61 },
    { 0xbeb0ec24, 93 },
    { 0x73f0ec25, 61 },
    { 0, 0 },
    { 0xdc50ec27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7330ec2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc320ec31, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd0ec35, 33 },
    { 0, 0 },
    { 0xcbe0ec37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ec3a, 98 },
    { 0xa230ec3b, 98 },
    { 0x3c30ec3c, 52 },
    { 0x1aa0ec3c, 33 },
    { 0xffb0ec3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d0ec4a, 23 },
    { 0, 0 },
    { 0x9280ec4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b90ec54, 66 },
    { 0, 0 },
    { 0x78a0ec56, 33 },
    { 0x2650ec57, 98 },
    { 0x6fa0ec57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec5b, 76 },
    { 0, 0 },
    { 0x4270ec5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf50ec62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd00ec67, 76 },
    { 0x8560ec68, 33 },
    { 0x1280ec67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3150ec6c, 84 },
    { 0, 0 },
    { 0x83d0ec6e, 90 },
    { 0x8980ec6f, 23 },
    { 0x2a30ec70, 100 },
    { 0xa4a0ec6f, 93 },
    { 0xb160ec6f, 23 },
    { 0xbea0ec73, 55 },
    { 0x3870ec73, 61 },
    { 0xdd20ec71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4350ec7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790ec7f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110ec87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb60ec8b, 98 },
    { 0x1330ec8c, 46 },
    { 0, 0 },
    { 0x8750ec8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b60ec96, 78 },
    { 0x24c0ec96, 81 },
    { 0x9a10ec98, 90 },
    { 0x2000ec98, 93 },
    { 0x3380ec9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb10ec9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b50eca0, 90 },
    { 0xe4d0eca1, 90 },
    { 0xb400eca0, 33 },
    { 0, 0 },
    { 0x97c0eca4, 23 },
    { 0x6930eca5, 76 },
    { 0x970eca5, 77 },
    { 0, 0 },
    { 0xf0b0eca8, 9 },
    { 0x76d0eca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb900ecae, 90 },
    { 0, 0 },
    { 0xa1d0ecb0, 94 },
    { 0xb4d0ecb1, 46 },
    { 0x38f0ecb0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1790ecba, 77 },
    { 0x41f0ecbb, 46 },
    { 0, 0 },
    { 0xcc0ecbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a0ecc0, 34 },
    { 0xfb40ecc1, 95 },
    { 0x4580ecc2, 80 },
    { 0x2af0ecc3, 33 },
    { 0, 0 },
    { 0x8130ecc5, 98 },
    { 0x410ecc5, 61 },
    { 0x8d40ecc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf040eccb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd150ecce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d0ecd1, 43 },
    { 0, 0 },
    { 0x6a90ecd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac0ecd8, 68 },
    { 0xf3c0ecd8, 66 },
    { 0, 0 },
    { 0xd0a0ecdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9070ecde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa0ece7, 43 },
    { 0xcd40ece8, 90 },
    { 0x9650ece9, 95 },
    { 0x6f50ece8, 61 },
    { 0x6890eceb, 61 },
    { 0x1780eceb, 84 },
    { 0xe60eceb, 33 },
    { 0xa170ecee, 61 },
    { 0xe870ecef, 90 },
    { 0x2b90ecf0, 61 },
    { 0x7810ecf1, 95 },
    { 0xc5a0ecf2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d0ecf6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6220ecfa, 93 },
    { 0xab50ecfb, 76 },
    { 0x88e0ecfa, 93 },
    { 0x9f10ecfd, 46 },
    { 0x6c10ecfd, 61 },
    { 0xf550ecff, 98 },
    { 0xbec0ed00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b00ed04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c0ed07, 17 },
    { 0, 0 },
    { 0xab90ed09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9550ed10, 23 },
    { 0, 0 },
    { 0x8700ed12, 87 },
    { 0x9830ed13, 95 },
    { 0x4f0ed13, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb100ed18, 87 },
    { 0x69c0ed19, 46 },
    { 0xfa60ed19, 33 },
    { 0xa270ed1b, 46 },
    { 0xfa10ed19, 46 },
    { 0x24b0ed1d, 23 },
    { 0xfc90ed1e, 43 },
    { 0xb190ed1e, 95 },
    { 0, 0 },
    { 0x32e0ed21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd90ed28, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ed2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3690ed3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d50ed3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00ed44, 55 },
    { 0xfef0ed44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9560ed49, 66 },
    { 0xb7b0ed4a, 95 },
    { 0x49f0ed4b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7060ed4e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1240ed53, 61 },
    { 0x4cd0ed53, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x9620ed57, 33 },
    { 0xb970ed58, 23 },
    { 0, 0 },
    { 0xfcb0ed5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc590ed5d, 64 },
    { 0x4b90ed5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x560ed61, 77 },
    { 0x9dc0ed62, 88 },
    { 0, 0 },
    { 0x8570ed64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef0ed68, 93 },
    { 0xce20ed68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb800ed6c, 93 },
    { 0x6660ed6d, 33 },
    { 0x2e00ed6e, 59 },
    { 0x1250ed6f, 95 },
    { 0x7270ed70, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2540ed73, 56 },
    { 0xc4e0ed74, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac0ed78, 90 },
    { 0x9cf0ed78, 91 },
    { 0xe730ed7a, 98 },
    { 0xb430ed7b, 23 },
    { 0, 0 },
    { 0x960ed7d, 61 },
    { 0, 0 },
    { 0xcc30ed7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc980ed83, 95 },
    { 0x2dd0ed84, 98 },
    { 0xc9f0ed85, 33 },
    { 0x8ce0ed86, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca60ed8f, 33 },
    { 0xe690ed90, 73 },
    { 0xe130ed91, 33 },
    { 0, 0 },
    { 0x6c80ed93, 90 },
    { 0x14f0ed93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce40ed98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ed9e, 98 },
    { 0x4450ed9f, 61 },
    { 0xc050ed9f, 75 },
    { 0x4510ed9f, 91 },
    { 0, 0 },
    { 0x56c0eda3, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a0eda6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc0eda9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe210edaf, 33 },
    { 0, 0 },
    { 0x4270edb1, 87 },
    { 0xaf00edb2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c0edb6, 46 },
    { 0, 0 },
    { 0xeb00edb8, 66 },
    { 0x3a0edb9, 23 },
    { 0x7e20edba, 61 },
    { 0x8ed0edba, 23 },
    { 0x9ef0edbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1640edc1, 23 },
    { 0xd250edc1, 23 },
    { 0x1080edc1, 61 },
    { 0, 0 },
    { 0x5d50edc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff0edc8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae00edcb, 92 },
    { 0x7b30edcc, 81 },
    { 0, 0 },
    { 0xd6d0edce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b30edd3, 66 },
    { 0x17f0edd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d0edd9, 90 },
    { 0x66a0edda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a80eddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd260ede2, 95 },
    { 0x3fc0ede3, 95 },
    { 0, 0 },
    { 0x8dc0ede5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc0ede9, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xd630edec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac0edf0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff30edf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb850edfd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5820ee03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f50ee09, 86 },
    { 0, 0 },
    { 0x3d0ee0b, 90 },
    { 0x4c90ee0b, 61 },
    { 0x1930ee0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9940ee10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf810ee15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd10ee18, 61 },
    { 0xb400ee19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe00ee1c, 55 },
    { 0x39a0ee1d, 66 },
    { 0xa820ee1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b00ee21, 95 },
    { 0x3150ee21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc0ee26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d0ee2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c0ee39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0ee3e, 40 },
    { 0x16b0ee3f, 61 },
    { 0x1360ee3f, 33 },
    { 0x3c80ee41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd740ee46, 17 },
    { 0xf4b0ee47, 95 },
    { 0xbb30ee47, 61 },
    { 0xbfe0ee46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac20ee4c, 95 },
    { 0xca90ee4d, 24 },
    { 0xc120ee4e, 98 },
    { 0x4b70ee4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8880ee53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c00ee57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c10ee5a, 55 },
    { 0xe0d0ee5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d30ee5e, 95 },
    { 0x7de0ee5f, 60 },
    { 0x7990ee60, 33 },
    { 0x2190ee60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba50ee6b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0ee6e, 55 },
    { 0x9e10ee6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd60ee76, 95 },
    { 0x3f10ee76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb90ee7a, 76 },
    { 0, 0 },
    { 0x5430ee7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe0ee84, 90 },
    { 0xef50ee84, 91 },
    { 0x6450ee84, 61 },
    { 0x1e60ee87, 61 },
    { 0x5af0ee87, 61 },
    { 0xda20ee88, 98 },
    { 0xd0b0ee89, 98 },
    { 0xff30ee8b, 76 },
    { 0x6190ee8c, 33 },
    { 0xdef0ee8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc560ee90, 46 },
    { 0xd170ee91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4830ee97, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f0ee9a, 61 },
    { 0, 0 },
    { 0x3b60ee9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad0ee9f, 101 },
    { 0x1430ee9f, 52 },
    { 0x2120eea0, 61 },
    { 0xf2c0eea0, 85 },
    { 0xaaf0eea3, 99 },
    { 0x73f0eea3, 80 },
    { 0xa6f0eea3, 87 },
    { 0x5850eea5, 98 },
    { 0, 0 },
    { 0x3980eea8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd0eeae, 61 },
    { 0x5fb0eeaf, 61 },
    { 0x27f0eeb0, 61 },
    { 0x2720eeb1, 98 },
    { 0xba80eeb2, 90 },
    { 0, 0 },
    { 0x7900eeb4, 55 },
    { 0xbc90eeb4, 61 },
    { 0x8e80eeb6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad00eeba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b60eec1, 46 },
    { 0x2d00eec1, 61 },
    { 0, 0 },
    { 0x310eec4, 98 },
    { 0x40b0eec4, 84 },
    { 0x4250eec4, 34 },
    { 0, 0 },
    { 0x91a0eec8, 55 },
    { 0, 0 },
    { 0x7c80eeca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4790eed4, 40 },
    { 0x850eed5, 98 },
    { 0x3890eed5, 98 },
    { 0x76c0eed4, 92 },
    { 0, 0 },
    { 0x9530eed9, 52 },
    { 0x53f0eed9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f0eede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec10eee3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc0eee9, 84 },
    { 0xa1f0eeea, 95 },
    { 0x89b0eeeb, 90 },
    { 0x9220eeeb, 81 },
    { 0, 0 },
    { 0xb670eeee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6720eef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0eef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa540eefe, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e60ef05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9700ef08, 61 },
    { 0x5fa0ef09, 100 },
    { 0x51b0ef0a, 98 },
    { 0xa7d0ef0a, 69 },
    { 0x1990ef0c, 61 },
    { 0xa00ef0c, 90 },
    { 0x47f0ef0e, 77 },
    { 0xa000ef0f, 90 },
    { 0x7d40ef10, 100 },
    { 0x4ab0ef0a, 33 },
    { 0x1e10ef12, 55 },
    { 0x93a0ef12, 64 },
    { 0x5560ef0b, 33 },
    { 0x1810ef15, 95 },
    { 0xef90ef10, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xf770ef19, 98 },
    { 0x3520ef1a, 34 },
    { 0xb130ef19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0ef1e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1450ef21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8240ef2a, 87 },
    { 0xef60ef2b, 38 },
    { 0xebf0ef2c, 95 },
    { 0, 0 },
    { 0x2c0ef2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7910ef31, 23 },
    { 0, 0 },
    { 0xe540ef33, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a40ef36, 33 },
    { 0x66e0ef37, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f10ef3a, 55 },
    { 0xb600ef3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b0ef3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7820ef46, 99 },
    { 0, 0 },
    { 0xfa60ef48, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6820ef4f, 75 },
    { 0, 0 },
    { 0xd5b0ef51, 95 },
    { 0x81c0ef52, 90 },
    { 0xf400ef52, 95 },
    { 0x270ef52, 90 },
    { 0x81d0ef52, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb0ef59, 95 },
    { 0, 0 },
    { 0xa630ef5b, 90 },
    { 0x7190ef5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0ef65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa030ef68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x910ef6b, 87 },
    { 0, 0 },
    { 0x200ef6d, 34 },
    { 0, 0 },
    { 0xc6e0ef6f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e0ef72, 90 },
    { 0xdb00ef73, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae0ef79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac0ef7d, 23 },
    { 0, 0 },
    { 0xb6e0ef7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1950ef83, 53 },
    { 0, 0 },
    { 0xee80ef85, 55 },
    { 0xd730ef86, 23 },
    { 0, 0 },
    { 0xb300ef88, 93 },
    { 0xb410ef88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1680ef8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df0ef94, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c0ef98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc0ef9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb90ef9e, 98 },
    { 0xf620ef9e, 40 },
    { 0xf290ef9f, 34 },
    { 0, 0 },
    { 0x700efa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4940efa6, 23 },
    { 0, 0 },
    { 0xe790efa8, 98 },
    { 0x9b50efa9, 23 },
    { 0x5d10efa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60efad, 95 },
    { 0x1c90efad, 81 },
    { 0x680efae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda0efb4, 51 },
    { 0, 0 },
    { 0x9590efb6, 45 },
    { 0xd330efb7, 61 },
    { 0xd7f0efb7, 61 },
    { 0x14a0efb9, 78 },
    { 0x8fc0efb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf70efbe, 100 },
    { 0x770efbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab0efc2, 99 },
    { 0x6440efc3, 98 },
    { 0x3b30efc4, 88 },
    { 0, 0 },
    { 0x66b0efc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb00efc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff0efcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf640efd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee0efd7, 98 },
    { 0x6a80efd8, 95 },
    { 0, 0 },
    { 0x8980efda, 61 },
    { 0x7780efda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3230efde, 55 },
    { 0xe870efde, 61 },
    { 0xc080efe0, 65 },
    { 0x5d20efe1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b0efe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380efea, 61 },
    { 0x60c0efeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f0efef, 46 },
    { 0, 0 },
    { 0x2030eff1, 61 },
    { 0x170eff2, 98 },
    { 0x8760eff3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5390eff6, 43 },
    { 0x7d70eff7, 77 },
    { 0xbe70eff6, 23 },
    { 0x1390eff9, 65 },
    { 0x3190eff8, 55 },
    { 0x3c70eff6, 33 },
    { 0, 0 },
    { 0x1420effd, 95 },
    { 0x6ac0effe, 88 },
    { 0x6870effe, 33 },
    { 0x5f40f000, 98 },
    { 0, 0 },
    { 0x9b40f002, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f70f007, 55 },
    { 0xe700f007, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb0f00c, 33 },
    { 0x4470f00d, 76 },
    { 0xdde0f00e, 33 },
    { 0x7e60f00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f30f013, 95 },
    { 0xb1b0f013, 52 },
    { 0xcf50f015, 66 },
    { 0xc6d0f014, 96 },
    { 0xf990f017, 90 },
    { 0xb7e0f014, 99 },
    { 0, 0 },
    { 0x2c0f01a, 55 },
    { 0x2ba0f01a, 46 },
    { 0x2e30f01c, 81 },
    { 0, 0 },
    { 0xebe0f01e, 53 },
    { 0, 0 },
    { 0xa50f020, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e00f023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe760f026, 90 },
    { 0x86a0f026, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a0f037, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3470f03c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2330f042, 34 },
    { 0x5230f043, 90 },
    { 0x70b0f043, 61 },
    { 0xbb30f045, 78 },
    { 0x400f043, 33 },
    { 0x8860f047, 95 },
    { 0x4840f047, 61 },
    { 0xc6f0f048, 95 },
    { 0x80c0f047, 90 },
    { 0xdce0f04b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a0f055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd0f058, 91 },
    { 0x3cf0f058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd220f05c, 85 },
    { 0, 0 },
    { 0x84a0f05e, 95 },
    { 0x7a40f05f, 36 },
    { 0x690f060, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d30f064, 76 },
    { 0, 0 },
    { 0x5580f066, 77 },
    { 0xe9c0f067, 91 },
    { 0, 0 },
    { 0x8840f069, 77 },
    { 0, 0 },
    { 0xfd90f06b, 90 },
    { 0, 0 },
    { 0x5750f06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba80f076, 33 },
    { 0xb450f077, 61 },
    { 0x97e0f077, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2450f07e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d0f082, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa980f085, 95 },
    { 0xee20f085, 91 },
    { 0x4a70f085, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e0f091, 23 },
    { 0, 0 },
    { 0x3940f093, 33 },
    { 0, 0 },
    { 0xcc80f095, 33 },
    { 0x5ff0f096, 84 },
    { 0x1e90f095, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec0f09b, 93 },
    { 0x5f70f09c, 66 },
    { 0, 0 },
    { 0x6a30f09e, 95 },
    { 0xb550f09e, 61 },
    { 0x6820f0a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00f0a4, 61 },
    { 0x6f20f0a5, 33 },
    { 0xfc50f0a5, 43 },
    { 0xd3c0f0a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6850f0aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a0f0ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d0f0b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1020f0b9, 99 },
    { 0xb7d0f0b9, 78 },
    { 0xe240f0ba, 34 },
    { 0x2790f0bc, 61 },
    { 0x23f0f0bd, 93 },
    { 0x740f0be, 77 },
    { 0x4130f0bf, 93 },
    { 0, 0 },
    { 0x6d20f0c1, 90 },
    { 0, 0 },
    { 0xe8c0f0c3, 90 },
    { 0x6a0f0c3, 67 },
    { 0xf650f0c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e0f0c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0f0ce, 33 },
    { 0xe140f0cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e30f0d7, 46 },
    { 0, 0 },
    { 0x3a20f0d9, 85 },
    { 0, 0 },
    { 0x100f0db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d60f0de, 98 },
    { 0x4980f0df, 43 },
    { 0x8390f0de, 61 },
    { 0, 0 },
    { 0x4820f0e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c50f0e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6800f0ed, 17 },
    { 0, 0 },
    { 0x2170f0ef, 91 },
    { 0xd610f0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe470f0f3, 61 },
    { 0x8060f0f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c60f0f9, 34 },
    { 0x2380f0fa, 99 },
    { 0x6a20f0f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a0f102, 55 },
    { 0x9440f103, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf550f106, 95 },
    { 0xc40f106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0f10c, 78 },
    { 0x91f0f10d, 55 },
    { 0x1300f10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad10f119, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570f11c, 61 },
    { 0, 0 },
    { 0x31f0f11e, 33 },
    { 0x53e0f11f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d0f123, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a0f12a, 90 },
    { 0x7d70f12b, 68 },
    { 0x6eb0f12c, 90 },
    { 0xf3b0f12b, 98 },
    { 0xe390f12e, 33 },
    { 0xa030f12e, 67 },
    { 0x3d20f130, 98 },
    { 0xf400f12d, 33 },
    { 0xe2c0f12c, 34 },
    { 0x140f12a, 90 },
    { 0x4680f131, 99 },
    { 0xef90f135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c0f13f, 33 },
    { 0xef70f140, 95 },
    { 0x7a60f140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe650f145, 43 },
    { 0x7840f146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee80f14e, 23 },
    { 0, 0 },
    { 0x3790f150, 98 },
    { 0x99f0f151, 52 },
    { 0xe430f152, 34 },
    { 0xa7d0f153, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c30f15a, 48 },
    { 0, 0 },
    { 0x8890f15c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c0f15f, 52 },
    { 0xe8a0f160, 91 },
    { 0, 0 },
    { 0x1740f162, 87 },
    { 0x97a0f162, 17 },
    { 0x21e0f164, 33 },
    { 0x5b50f165, 90 },
    { 0, 0 },
    { 0x1430f167, 90 },
    { 0xaa20f168, 52 },
    { 0x3170f169, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee0f170, 61 },
    { 0, 0 },
    { 0x6190f172, 55 },
    { 0, 0 },
    { 0x4350f174, 40 },
    { 0xd550f174, 33 },
    { 0, 0 },
    { 0xc070f177, 93 },
    { 0x6cb0f177, 95 },
    { 0x9520f177, 34 },
    { 0x13a0f17a, 77 },
    { 0, 0 },
    { 0x94d0f17c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe280f180, 23 },
    { 0, 0 },
    { 0x40b0f182, 76 },
    { 0x1a90f182, 98 },
    { 0xc530f184, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8470f189, 33 },
    { 0xa740f18a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae50f18e, 91 },
    { 0, 0 },
    { 0x85b0f190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf010f196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd0f19c, 46 },
    { 0x6a20f19d, 33 },
    { 0xe50f19e, 69 },
    { 0, 0 },
    { 0x700f1a0, 34 },
    { 0x8640f1a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b0f1a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7770f1a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c0f1ac, 90 },
    { 0x4ab0f1ad, 90 },
    { 0x3dc0f1ad, 90 },
    { 0x2300f1af, 95 },
    { 0x8780f1ae, 23 },
    { 0, 0 },
    { 0xa250f1b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3400f1b5, 23 },
    { 0, 0 },
    { 0xc460f1b7, 100 },
    { 0xd460f1b7, 34 },
    { 0xea30f1b9, 61 },
    { 0x9b60f1b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e0f1c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa0f1c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f90f1c9, 33 },
    { 0x9440f1ca, 99 },
    { 0x9cd0f1cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe080f1cf, 66 },
    { 0x4f0f1cf, 84 },
    { 0x4660f1d1, 61 },
    { 0x3cc0f1cf, 76 },
    { 0x4520f1d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f20f1db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb490f1e0, 90 },
    { 0x1770f1e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6330f1e5, 46 },
    { 0, 0 },
    { 0xae70f1e7, 76 },
    { 0, 0 },
    { 0x9750f1e9, 90 },
    { 0x9cc0f1e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860f1f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a10f1f5, 33 },
    { 0x53a0f1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530f1f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f40f1ff, 23 },
    { 0x8c90f200, 66 },
    { 0, 0 },
    { 0x9c50f202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab0f206, 93 },
    { 0x5900f207, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8050f212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d0f215, 55 },
    { 0xd4e0f216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f0f219, 55 },
    { 0x22f0f21a, 61 },
    { 0x6530f21b, 87 },
    { 0x460f21c, 90 },
    { 0x6a30f21c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca0f223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb520f227, 61 },
    { 0xff0f228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed0f22b, 61 },
    { 0xecc0f22c, 90 },
    { 0xae20f22d, 98 },
    { 0xd20f22d, 98 },
    { 0xa640f22f, 33 },
    { 0x59a0f230, 34 },
    { 0, 0 },
    { 0xefc0f232, 33 },
    { 0x6970f233, 66 },
    { 0x61f0f233, 61 },
    { 0x91f0f234, 88 },
    { 0x700f234, 34 },
    { 0x16d0f237, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d60f23b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c70f23f, 90 },
    { 0xc560f240, 84 },
    { 0x89b0f240, 23 },
    { 0x9020f242, 2 },
    { 0x92e0f243, 72 },
    { 0x7c70f244, 61 },
    { 0x59f0f245, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2300f248, 46 },
    { 0, 0 },
    { 0xcd90f24a, 68 },
    { 0xa3c0f24b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b0f250, 80 },
    { 0x6540f250, 64 },
    { 0x5fa0f252, 61 },
    { 0, 0 },
    { 0xed30f254, 98 },
    { 0x4a0f254, 90 },
    { 0, 0 },
    { 0xee80f257, 95 },
    { 0x3c90f258, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8910f25b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc0f25f, 85 },
    { 0x7500f260, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f26f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4010f275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c80f279, 90 },
    { 0x1f00f279, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd080f282, 33 },
    { 0x30e0f283, 90 },
    { 0x3980f283, 61 },
    { 0xb190f285, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b60f288, 33 },
    { 0, 0 },
    { 0x1f70f28a, 90 },
    { 0, 0 },
    { 0x5560f28c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd0f294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4380f297, 90 },
    { 0xc710f298, 95 },
    { 0, 0 },
    { 0x6f80f29a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2690f29f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b0f2a2, 98 },
    { 0xbfc0f2a3, 43 },
    { 0x7860f2a4, 78 },
    { 0, 0 },
    { 0x8330f2a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6780f2ab, 95 },
    { 0x2be0f2ac, 90 },
    { 0xf7f0f2ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf520f2b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0f2b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280f2bb, 95 },
    { 0x78f0f2bc, 55 },
    { 0x2450f2bb, 80 },
    { 0x4cf0f2be, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc0f2c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc50f2ce, 43 },
    { 0xa5a0f2cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8170f2d2, 55 },
    { 0, 0 },
    { 0xfc30f2d4, 78 },
    { 0xb500f2d5, 80 },
    { 0xfcf0f2d6, 52 },
    { 0x6550f2d7, 55 },
    { 0xaf80f2d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9710f2dd, 26 },
    { 0, 0 },
    { 0x8fa0f2df, 98 },
    { 0, 0 },
    { 0x1fb0f2e1, 68 },
    { 0xf700f2e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7960f2e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e70f2ec, 90 },
    { 0x9f50f2ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5590f2fa, 95 },
    { 0, 0 },
    { 0xd0f2fc, 98 },
    { 0x2a10f2fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650f300, 23 },
    { 0x6b70f300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d0f304, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940f30b, 61 },
    { 0xf350f30c, 61 },
    { 0x1130f30d, 43 },
    { 0xfc90f30e, 76 },
    { 0x7180f30e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7100f313, 84 },
    { 0xaa70f314, 33 },
    { 0xe340f315, 90 },
    { 0xf4b0f316, 61 },
    { 0xb560f316, 98 },
    { 0xb060f318, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a0f31d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed80f321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f30f324, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d40f329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed80f32e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd000f338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d50f33d, 90 },
    { 0xa50f33e, 83 },
    { 0x7180f33f, 33 },
    { 0, 0 },
    { 0xf7b0f341, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5410f345, 23 },
    { 0xb100f346, 95 },
    { 0x8370f347, 45 },
    { 0x3540f346, 90 },
    { 0xd120f349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c20f34d, 41 },
    { 0x16e0f34d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0f354, 36 },
    { 0xc1e0f354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5570f359, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a0f35d, 66 },
    { 0x4550f35e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1380f362, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7610f365, 90 },
    { 0, 0 },
    { 0xaeb0f367, 90 },
    { 0, 0 },
    { 0x7d40f369, 78 },
    { 0, 0 },
    { 0x7980f36b, 61 },
    { 0xec80f36c, 61 },
    { 0x8560f36d, 23 },
    { 0x74c0f36e, 90 },
    { 0xe710f36c, 59 },
    { 0xf540f370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e40f374, 65 },
    { 0xc7d0f374, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6960f378, 98 },
    { 0, 0 },
    { 0xd550f37a, 95 },
    { 0, 0 },
    { 0x7470f37c, 90 },
    { 0x9450f37c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e00f380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c0f384, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc750f387, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf0f38a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa0f38f, 93 },
    { 0xa60f390, 56 },
    { 0x3180f391, 61 },
    { 0x22f0f392, 98 },
    { 0x3f70f392, 94 },
    { 0xdec0f390, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0f399, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9360f3a5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd810f3a9, 34 },
    { 0x4e80f3a9, 34 },
    { 0, 0 },
    { 0xab30f3ac, 76 },
    { 0, 0 },
    { 0x7cb0f3ae, 76 },
    { 0xe990f3af, 61 },
    { 0xa260f3af, 98 },
    { 0xabd0f3af, 76 },
    { 0x7f90f3b2, 23 },
    { 0, 0 },
    { 0x8670f3b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b80f3b9, 85 },
    { 0xa710f3ba, 61 },
    { 0xe8a0f3ba, 96 },
    { 0xab0f3bb, 34 },
    { 0x38c0f3bd, 84 },
    { 0, 0 },
    { 0xf9c0f3bf, 98 },
    { 0xfef0f3c0, 90 },
    { 0, 0 },
    { 0x9f80f3c2, 95 },
    { 0x32f0f3c3, 78 },
    { 0xf1b0f3c4, 36 },
    { 0x6820f3c5, 52 },
    { 0xf740f3c5, 93 },
    { 0x8470f3c7, 52 },
    { 0, 0 },
    { 0xbad0f3c9, 46 },
    { 0xce90f3c9, 61 },
    { 0x2f50f3ca, 61 },
    { 0xd050f3cc, 90 },
    { 0xcb50f3c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e50f3d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b50f3d7, 55 },
    { 0x3290f3d7, 23 },
    { 0x6190f3d7, 66 },
    { 0x1860f3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb50f3dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c40f3e5, 88 },
    { 0xe160f3e6, 90 },
    { 0xb650f3e7, 17 },
    { 0xe860f3e8, 94 },
    { 0, 0 },
    { 0x4270f3ea, 23 },
    { 0, 0 },
    { 0x3590f3ec, 90 },
    { 0, 0 },
    { 0x3ef0f3ee, 76 },
    { 0x6380f3ef, 66 },
    { 0xe080f3ee, 34 },
    { 0, 0 },
    { 0x19e0f3f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2120f3fc, 33 },
    { 0, 0 },
    { 0x1120f3fe, 34 },
    { 0, 0 },
    { 0xf610f400, 33 },
    { 0x3750f401, 33 },
    { 0, 0 },
    { 0x3c50f403, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1000f408, 93 },
    { 0, 0 },
    { 0x4cf0f40a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1820f40e, 34 },
    { 0, 0 },
    { 0x8810f410, 61 },
    { 0, 0 },
    { 0x9be0f412, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8250f415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e0f41b, 35 },
    { 0, 0 },
    { 0xfd80f41d, 55 },
    { 0, 0 },
    { 0x6ca0f41f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c70f423, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120f426, 34 },
    { 0xab20f427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4940f42a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e80f430, 52 },
    { 0xf10f431, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b0f437, 95 },
    { 0, 0 },
    { 0xdf60f439, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620f43f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b0f442, 99 },
    { 0, 0 },
    { 0xe070f444, 23 },
    { 0xa7a0f445, 76 },
    { 0xc260f446, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640f44b, 90 },
    { 0x5ac0f44c, 66 },
    { 0xaf80f44d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe010f450, 98 },
    { 0x6570f450, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2840f456, 84 },
    { 0xe4b0f457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec0f45b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f0f45e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8280f462, 90 },
    { 0, 0 },
    { 0xa820f464, 87 },
    { 0x9d20f464, 23 },
    { 0xf160f466, 90 },
    { 0x8c80f467, 90 },
    { 0, 0 },
    { 0x3520f469, 61 },
    { 0x980f46a, 61 },
    { 0, 0 },
    { 0xc630f46c, 61 },
    { 0xf1e0f46c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e0f473, 55 },
    { 0xab90f474, 23 },
    { 0xb0b0f475, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f0f479, 99 },
    { 0, 0 },
    { 0x5950f47b, 95 },
    { 0, 0 },
    { 0x7a10f47d, 84 },
    { 0x1490f47e, 33 },
    { 0x78a0f47e, 33 },
    { 0x2780f47f, 34 },
    { 0xa640f481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f50f487, 87 },
    { 0x7270f488, 33 },
    { 0, 0 },
    { 0x5290f48a, 61 },
    { 0xb300f48b, 55 },
    { 0, 0 },
    { 0x3670f48d, 33 },
    { 0, 0 },
    { 0xee20f48f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc340f493, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa50f496, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b0f499, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc260f4a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a0f4ac, 90 },
    { 0xb130f4ad, 43 },
    { 0xb8f0f4ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f50f4b5, 23 },
    { 0xf100f4b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a0f4ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae0f4c0, 23 },
    { 0xcaa0f4c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba10f4c5, 90 },
    { 0xe780f4c6, 90 },
    { 0, 0 },
    { 0xede0f4c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda20f4ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa630f4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd710f4e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd730f4e5, 33 },
    { 0, 0 },
    { 0xefc0f4e7, 34 },
    { 0xc6c0f4e8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd240f4ef, 37 },
    { 0x73e0f4f0, 90 },
    { 0x6640f4f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b0f4f4, 90 },
    { 0xaf00f4f5, 93 },
    { 0xabe0f4f5, 61 },
    { 0xf710f4f5, 23 },
    { 0x2370f4f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb70f4fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc820f502, 95 },
    { 0x5d0f502, 90 },
    { 0xca70f504, 52 },
    { 0, 0 },
    { 0xbaf0f506, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e0f50e, 34 },
    { 0x5970f50f, 68 },
    { 0x53d0f510, 77 },
    { 0xaf70f510, 66 },
    { 0xa6b0f512, 65 },
    { 0x32b0f513, 76 },
    { 0x39a0f514, 85 },
    { 0x1be0f515, 98 },
    { 0x2230f516, 23 },
    { 0xbe90f517, 17 },
    { 0x8700f50f, 61 },
    { 0, 0 },
    { 0x8d90f51a, 59 },
    { 0, 0 },
    { 0x7370f51c, 61 },
    { 0, 0 },
    { 0xb5f0f51e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb760f521, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd0f528, 42 },
    { 0x6810f529, 33 },
    { 0x3ca0f528, 61 },
    { 0, 0 },
    { 0x91e0f52c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab00f531, 94 },
    { 0xea70f531, 90 },
    { 0x3ef0f533, 90 },
    { 0, 0 },
    { 0x50a0f535, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd70f538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca80f53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb700f542, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed0f546, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5530f54b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4120f552, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe470f55a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa560f55f, 23 },
    { 0xfc80f560, 43 },
    { 0x1c40f55f, 61 },
    { 0x7b80f562, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe90f569, 98 },
    { 0xbb40f56a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe370f56d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0f570, 90 },
    { 0xe4d0f570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec0f574, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba80f579, 90 },
    { 0x6d60f579, 61 },
    { 0xa40f57a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f00f57e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f0f584, 93 },
    { 0x8230f584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b0f588, 23 },
    { 0x3640f589, 90 },
    { 0, 0 },
    { 0xfbd0f58b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f0f590, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe420f593, 88 },
    { 0, 0 },
    { 0x54f0f595, 98 },
    { 0x7f10f596, 98 },
    { 0xc1e0f596, 64 },
    { 0x6c30f595, 46 },
    { 0xfff0f597, 95 },
    { 0x6a40f59a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed70f5a0, 100 },
    { 0x1240f5a0, 95 },
    { 0x6f20f5a1, 90 },
    { 0x8c0f5a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf080f5a8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0f5af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7320f5b2, 61 },
    { 0xa8a0f5b2, 33 },
    { 0x2300f5b4, 98 },
    { 0, 0 },
    { 0xd940f5b6, 43 },
    { 0x35d0f5b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0f5be, 81 },
    { 0, 0 },
    { 0x7930f5c0, 77 },
    { 0, 0 },
    { 0xb2d0f5c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf40f5c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d0f5c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b90f5cb, 90 },
    { 0, 0 },
    { 0x54f0f5cd, 90 },
    { 0x9190f5ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec0f5d2, 23 },
    { 0xa400f5d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab0f5d7, 43 },
    { 0, 0 },
    { 0x9f10f5d9, 33 },
    { 0x69c0f5da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af0f5de, 85 },
    { 0xb6b0f5df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1070f5e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc80f5ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a00f5f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc550f5f8, 55 },
    { 0xb5c0f5f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe350f5fd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf0f603, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3040f612, 61 },
    { 0x4750f612, 55 },
    { 0, 0 },
    { 0x56e0f615, 37 },
    { 0xfe70f616, 61 },
    { 0xf910f616, 95 },
    { 0, 0 },
    { 0xb40f619, 90 },
    { 0x6700f619, 23 },
    { 0xdc90f61b, 33 },
    { 0xa20f61b, 55 },
    { 0xf940f61b, 93 },
    { 0x48f0f619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe120f622, 87 },
    { 0x9d30f623, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae60f628, 76 },
    { 0x31d0f629, 98 },
    { 0, 0 },
    { 0x3d90f62b, 34 },
    { 0xe040f62b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb0f630, 33 },
    { 0, 0 },
    { 0xddf0f632, 90 },
    { 0, 0 },
    { 0xc020f634, 91 },
    { 0, 0 },
    { 0xd870f636, 95 },
    { 0xf540f636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc20f63e, 46 },
    { 0xb270f63f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380f643, 23 },
    { 0, 0 },
    { 0xb000f645, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8610f64a, 76 },
    { 0xdcc0f64b, 76 },
    { 0, 0 },
    { 0xbc0f64d, 61 },
    { 0xb2c0f64d, 23 },
    { 0x7e80f64e, 77 },
    { 0, 0 },
    { 0x5bb0f651, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa250f654, 95 },
    { 0xe450f655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f0f658, 34 },
    { 0xb60f658, 66 },
    { 0xd4f0f658, 95 },
    { 0x3cc0f659, 38 },
    { 0, 0 },
    { 0x3840f65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda0f661, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce0f668, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0f66b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa850f670, 66 },
    { 0xc150f671, 33 },
    { 0xcd70f672, 55 },
    { 0x3300f673, 23 },
    { 0xb6c0f673, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1410f67f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f60f684, 34 },
    { 0x5170f685, 61 },
    { 0x1200f686, 61 },
    { 0xcac0f686, 55 },
    { 0x2ea0f688, 90 },
    { 0, 0 },
    { 0xed50f68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e0f68d, 61 },
    { 0x8d0f68e, 87 },
    { 0x27c0f68f, 77 },
    { 0xfa40f68f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f0f694, 100 },
    { 0xfbc0f695, 40 },
    { 0xc2f0f696, 68 },
    { 0x6370f694, 98 },
    { 0x4640f697, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7200f69b, 33 },
    { 0xf4d0f69b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd300f6a3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa230f6aa, 61 },
    { 0xfa0f6aa, 43 },
    { 0x9070f6ac, 93 },
    { 0x7d90f6ab, 52 },
    { 0xa810f6ae, 95 },
    { 0x2f30f6af, 33 },
    { 0x4f30f6ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d0f6b5, 90 },
    { 0x12e0f6b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa870f6b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa0f6bf, 100 },
    { 0xc390f6c0, 61 },
    { 0x4420f6c1, 35 },
    { 0xdf30f6bf, 33 },
    { 0x9d20f6c1, 34 },
    { 0xdc80f6c4, 61 },
    { 0xfcd0f6c1, 24 },
    { 0, 0 },
    { 0x67f0f6c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba0f6cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf670f6d2, 43 },
    { 0x4ea0f6d3, 98 },
    { 0, 0 },
    { 0xf1f0f6d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b00f6da, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b0f6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb0f6e0, 84 },
    { 0, 0 },
    { 0x7390f6e2, 88 },
    { 0xce20f6e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a0f6e7, 55 },
    { 0, 0 },
    { 0xb260f6e9, 23 },
    { 0x4360f6ea, 23 },
    { 0x1850f6eb, 77 },
    { 0, 0 },
    { 0x2500f6ed, 33 },
    { 0, 0 },
    { 0x4f00f6ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2680f6f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f6fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0f6fe, 34 },
    { 0xc150f6ff, 52 },
    { 0xb1e0f700, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb030f704, 34 },
    { 0, 0 },
    { 0x6330f706, 90 },
    { 0, 0 },
    { 0xc00f708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1780f70b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e20f70f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8810f714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa730f71b, 33 },
    { 0xb140f71c, 83 },
    { 0xfbb0f71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a0f720, 93 },
    { 0x3060f720, 90 },
    { 0x2c10f720, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc700f72a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7310f72d, 52 },
    { 0x3980f72d, 23 },
    { 0xaf30f72d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd510f736, 43 },
    { 0, 0 },
    { 0xb690f738, 43 },
    { 0, 0 },
    { 0xc7a0f73a, 61 },
    { 0x2b10f73a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b0f743, 98 },
    { 0xdf90f744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb0f74c, 98 },
    { 0x96d0f74d, 90 },
    { 0x4660f74d, 29 },
    { 0x1b50f74d, 66 },
    { 0x2bf0f74e, 93 },
    { 0x7260f74f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c0f754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb20f759, 88 },
    { 0x24c0f75a, 69 },
    { 0x8f90f75a, 34 },
    { 0, 0 },
    { 0x57f0f75d, 90 },
    { 0xdee0f75e, 98 },
    { 0x8300f75f, 61 },
    { 0xfae0f760, 55 },
    { 0x8360f760, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0f766, 33 },
    { 0xc740f766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa40f76a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b80f76e, 61 },
    { 0x4e00f76e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e0f775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb470f778, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6270f784, 90 },
    { 0, 0 },
    { 0x3c10f786, 33 },
    { 0x6de0f787, 90 },
    { 0x3b40f787, 33 },
    { 0xf170f787, 90 },
    { 0x38c0f788, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa790f792, 85 },
    { 0x4720f793, 98 },
    { 0xceb0f794, 76 },
    { 0xa4d0f794, 53 },
    { 0xb970f795, 33 },
    { 0x2ac0f797, 95 },
    { 0x80b0f793, 61 },
    { 0x9f10f797, 96 },
    { 0x7470f79a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4680f79d, 90 },
    { 0xf030f79d, 90 },
    { 0x51c0f79e, 98 },
    { 0x5ec0f7a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea0f7a7, 61 },
    { 0, 0 },
    { 0x2260f7a9, 90 },
    { 0xa7a0f7aa, 28 },
    { 0x9330f7aa, 84 },
    { 0x3af0f7ac, 23 },
    { 0xa5f0f7ad, 77 },
    { 0x8df0f7ad, 33 },
    { 0xf120f7aa, 76 },
    { 0x23c0f7ae, 90 },
    { 0x89a0f7b1, 88 },
    { 0xf280f7b1, 96 },
    { 0x1ca0f7b3, 61 },
    { 0x2640f7ac, 93 },
    { 0, 0 },
    { 0x8c60f7b6, 90 },
    { 0, 0 },
    { 0x42b0f7b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790f7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e0f7be, 55 },
    { 0, 0 },
    { 0x3340f7c0, 43 },
    { 0x83a0f7c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b30f7c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a0f7c7, 75 },
    { 0x5770f7c8, 95 },
    { 0xe720f7c9, 91 },
    { 0xb680f7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc40f7d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3850f7d4, 33 },
    { 0x6e20f7d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f0f7da, 40 },
    { 0x3bb0f7db, 12 },
    { 0x6060f7dc, 90 },
    { 0x12a0f7dc, 99 },
    { 0x3fa0f7de, 90 },
    { 0x8270f7df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8180f7e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6760f7e7, 33 },
    { 0xc250f7e8, 39 },
    { 0x6530f7e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00f7ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0f7f4, 61 },
    { 0x3230f7f5, 91 },
    { 0x9550f7f6, 95 },
    { 0xd8c0f7f5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b0f7fc, 99 },
    { 0x5220f7fd, 68 },
    { 0xc510f7fe, 23 },
    { 0xfb20f7fd, 95 },
    { 0x45c0f800, 52 },
    { 0x6450f7ff, 75 },
    { 0x2ad0f7ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e80f807, 76 },
    { 0, 0 },
    { 0x33a0f809, 98 },
    { 0xd950f809, 33 },
    { 0x5070f80b, 75 },
    { 0xd880f80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c0f810, 33 },
    { 0, 0 },
    { 0x2190f812, 90 },
    { 0x8410f813, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8570f818, 23 },
    { 0xdb20f819, 90 },
    { 0xb350f81a, 75 },
    { 0x7580f81a, 95 },
    { 0x7b90f81c, 23 },
    { 0x43d0f81d, 33 },
    { 0, 0 },
    { 0x42c0f81f, 87 },
    { 0x9600f81f, 93 },
    { 0xa020f81f, 33 },
    { 0, 0 },
    { 0xa7f0f823, 61 },
    { 0xc8d0f824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7310f828, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610f82f, 34 },
    { 0, 0 },
    { 0x5250f831, 55 },
    { 0xde20f831, 95 },
    { 0, 0 },
    { 0xff20f834, 55 },
    { 0x22e0f835, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a0f83c, 34 },
    { 0xd160f83d, 52 },
    { 0, 0 },
    { 0xf9f0f83f, 90 },
    { 0x76e0f840, 95 },
    { 0, 0 },
    { 0x59c0f842, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc0f848, 95 },
    { 0, 0 },
    { 0xb630f84a, 95 },
    { 0x6e70f84a, 84 },
    { 0, 0 },
    { 0x6680f84d, 33 },
    { 0xe670f84d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f0f852, 53 },
    { 0x3990f853, 33 },
    { 0xf90f853, 23 },
    { 0x1790f855, 55 },
    { 0x6120f856, 90 },
    { 0xd580f853, 23 },
    { 0xe4d0f858, 98 },
    { 0xc8e0f859, 98 },
    { 0x36d0f857, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd00f85d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe0f862, 90 },
    { 0x4680f862, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b10f86a, 98 },
    { 0, 0 },
    { 0x6a20f86c, 61 },
    { 0, 0 },
    { 0xfa00f86e, 76 },
    { 0, 0 },
    { 0x31a0f870, 95 },
    { 0xb3b0f871, 90 },
    { 0xd7d0f872, 87 },
    { 0x86f0f872, 84 },
    { 0x7c80f871, 43 },
    { 0xf80f875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e0f87d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70f880, 33 },
    { 0x96d0f881, 52 },
    { 0, 0 },
    { 0xf70f883, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8800f888, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef70f891, 98 },
    { 0x8a10f892, 90 },
    { 0xe4f0f892, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6480f89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0f8a4, 98 },
    { 0x9740f8a4, 90 },
    { 0xf50f8a6, 43 },
    { 0x2670f8a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6590f8af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b0f8b3, 99 },
    { 0x8460f8b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d30f8b8, 33 },
    { 0xeab0f8b8, 76 },
    { 0xe830f8ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f0f8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0f8c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f0f8c5, 93 },
    { 0, 0 },
    { 0x7da0f8c7, 33 },
    { 0x21e0f8c7, 95 },
    { 0x1100f8c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5580f8d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c40f8d9, 23 },
    { 0xbdd0f8da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b0f8dd, 76 },
    { 0x3700f8de, 98 },
    { 0x1600f8dd, 82 },
    { 0x6b20f8e0, 46 },
    { 0xe50f8e0, 34 },
    { 0xbdd0f8e0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a60f8eb, 33 },
    { 0x9c70f8ec, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe570f8f2, 90 },
    { 0, 0 },
    { 0x6580f8f4, 93 },
    { 0, 0 },
    { 0xddd0f8f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6920f8fa, 33 },
    { 0xfc60f8fa, 55 },
    { 0x99b0f8fa, 90 },
    { 0x3900f8fd, 52 },
    { 0x3350f8fc, 76 },
    { 0xd670f8ff, 98 },
    { 0xa530f900, 95 },
    { 0x4dc0f901, 78 },
    { 0xf740f901, 98 },
    { 0x3d80f903, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f909, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b0f916, 100 },
    { 0, 0 },
    { 0x2020f918, 23 },
    { 0xd230f919, 95 },
    { 0, 0 },
    { 0x9a00f91b, 55 },
    { 0x9be0f91b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d0f91f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b0f927, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1250f92a, 76 },
    { 0, 0 },
    { 0x8a0f92c, 20 },
    { 0x5fd0f92d, 76 },
    { 0x31f0f92c, 33 },
    { 0, 0 },
    { 0xee0f930, 90 },
    { 0, 0 },
    { 0xa160f932, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0f936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310f93b, 40 },
    { 0, 0 },
    { 0xd460f93d, 61 },
    { 0x31f0f93e, 43 },
    { 0x1060f93f, 90 },
    { 0xee30f940, 96 },
    { 0xc010f940, 98 },
    { 0xa80f942, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1750f947, 98 },
    { 0xaa50f948, 68 },
    { 0xca40f949, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd0f94e, 61 },
    { 0x21d0f94f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c0f953, 55 },
    { 0x2e20f954, 99 },
    { 0, 0 },
    { 0xc140f956, 33 },
    { 0, 0 },
    { 0x7100f958, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef0f95f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0f963, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f90f966, 76 },
    { 0, 0 },
    { 0xb9a0f968, 33 },
    { 0xb620f968, 75 },
    { 0, 0 },
    { 0x5f40f96b, 90 },
    { 0x4b10f96c, 40 },
    { 0, 0 },
    { 0xd4f0f96e, 90 },
    { 0xebb0f96f, 23 },
    { 0xfbc0f970, 99 },
    { 0x4880f96f, 98 },
    { 0xece0f96e, 80 },
    { 0, 0 },
    { 0x47c0f974, 95 },
    { 0, 0 },
    { 0xd840f976, 33 },
    { 0x6980f976, 90 },
    { 0x3d90f978, 33 },
    { 0, 0 },
    { 0xea90f97a, 52 },
    { 0x92d0f97a, 90 },
    { 0x4ad0f97c, 61 },
    { 0, 0 },
    { 0xf0e0f97e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc50f985, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2410f98b, 23 },
    { 0xe220f98c, 85 },
    { 0, 0 },
    { 0xc420f98e, 91 },
    { 0x43d0f98f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc0f992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0f997, 91 },
    { 0, 0 },
    { 0x4870f999, 95 },
    { 0x1310f99a, 34 },
    { 0, 0 },
    { 0xc6d0f99c, 33 },
    { 0x64a0f99d, 66 },
    { 0x90d0f99c, 33 },
    { 0, 0 },
    { 0x4780f9a0, 98 },
    { 0x98d0f9a1, 88 },
    { 0x5010f9a2, 40 },
    { 0xce50f9a1, 61 },
    { 0, 0 },
    { 0x7db0f9a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0f9a8, 90 },
    { 0xd6a0f9a8, 95 },
    { 0xeb10f9a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580f9b0, 61 },
    { 0x80f9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa0f9b5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40f9be, 100 },
    { 0, 0 },
    { 0x380f9c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7650f9c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc0f9cb, 99 },
    { 0, 0 },
    { 0x8e90f9cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a70f9d4, 98 },
    { 0x9040f9d5, 95 },
    { 0, 0 },
    { 0x87c0f9d7, 98 },
    { 0x75c0f9d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f0f9dd, 61 },
    { 0xe940f9de, 66 },
    { 0x7580f9df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f9e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00f9e8, 93 },
    { 0x3450f9e9, 90 },
    { 0xdbc0f9e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb40f9ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf070f9f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d80f9fc, 90 },
    { 0x3560f9fd, 55 },
    { 0xe1d0f9fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6180fa01, 90 },
    { 0, 0 },
    { 0x5650fa03, 100 },
    { 0, 0 },
    { 0x23a0fa05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2840fa0c, 33 },
    { 0, 0 },
    { 0xc920fa0e, 95 },
    { 0x6770fa0e, 98 },
    { 0xedb0fa0f, 90 },
    { 0xa160fa11, 93 },
    { 0x69d0fa12, 98 },
    { 0xd250fa13, 90 },
    { 0, 0 },
    { 0xf50fa15, 78 },
    { 0xc3d0fa16, 34 },
    { 0, 0 },
    { 0xfa10fa18, 83 },
    { 0xba0fa19, 53 },
    { 0x2a90fa1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac0fa1f, 91 },
    { 0, 0 },
    { 0x7c50fa21, 78 },
    { 0xc1c0fa22, 91 },
    { 0x10e0fa22, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d0fa26, 33 },
    { 0, 0 },
    { 0x6eb0fa28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5950fa2f, 55 },
    { 0xd6f0fa2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b50fa3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0fa42, 85 },
    { 0, 0 },
    { 0x77b0fa44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1790fa49, 80 },
    { 0x2460fa4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb0fa4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f80fa51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb0fa56, 34 },
    { 0, 0 },
    { 0x8780fa58, 95 },
    { 0xb70fa59, 90 },
    { 0x8870fa58, 87 },
    { 0x7f00fa58, 93 },
    { 0x1920fa58, 33 },
    { 0x4810fa5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3090fa60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa0fa66, 98 },
    { 0x6590fa67, 91 },
    { 0xf4f0fa68, 90 },
    { 0x9d50fa69, 90 },
    { 0x5310fa69, 100 },
    { 0, 0 },
    { 0xaf20fa6c, 90 },
    { 0x30b0fa6d, 98 },
    { 0xf600fa6e, 23 },
    { 0xde60fa6f, 99 },
    { 0x5ae0fa70, 43 },
    { 0xf5e0fa71, 90 },
    { 0x8e30fa6e, 61 },
    { 0x8d0fa73, 98 },
    { 0x79f0fa72, 23 },
    { 0xeef0fa75, 98 },
    { 0x6d20fa71, 46 },
    { 0x8cf0fa6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8970fa7a, 61 },
    { 0x22d0fa7b, 33 },
    { 0x8990fa7a, 46 },
    { 0xe4d0fa7c, 66 },
    { 0x5a40fa7e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee0fa82, 76 },
    { 0xea20fa83, 95 },
    { 0x7870fa84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4540fa89, 98 },
    { 0xa9c0fa89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b0fa8d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0fa91, 55 },
    { 0x3eb0fa91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1730fa9a, 90 },
    { 0xb200fa9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6690faad, 98 },
    { 0x1400faae, 77 },
    { 0xc5e0faaf, 90 },
    { 0xc4f0faae, 23 },
    { 0, 0 },
    { 0x3510fab2, 95 },
    { 0, 0 },
    { 0xe070fab4, 98 },
    { 0x2cb0fab5, 53 },
    { 0x65f0fab5, 90 },
    { 0, 0 },
    { 0x5d90fab8, 75 },
    { 0, 0 },
    { 0xe0c0faba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf620fabe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf180fac5, 46 },
    { 0x7650fac6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc0faca, 23 },
    { 0xda30facb, 33 },
    { 0x1210facb, 67 },
    { 0x3790facb, 77 },
    { 0x5920facc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef0fad2, 90 },
    { 0x35d0fad3, 98 },
    { 0xf00fad4, 90 },
    { 0xb60fad5, 91 },
    { 0x4a90fad4, 17 },
    { 0, 0 },
    { 0x2660fad8, 68 },
    { 0x53a0fad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb290fade, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b10fae4, 61 },
    { 0x7b10fae5, 33 },
    { 0x2ce0fae4, 98 },
    { 0, 0 },
    { 0x6910fae8, 55 },
    { 0, 0 },
    { 0x70c0faea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b0faed, 85 },
    { 0xaf30faee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f0faf1, 40 },
    { 0x14a0faf2, 100 },
    { 0xa820faf2, 95 },
    { 0x6f70faf4, 93 },
    { 0xf210faf3, 33 },
    { 0x2e10faf1, 68 },
    { 0x8b0faf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0fafa, 77 },
    { 0x14c0fafa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c0fafe, 23 },
    { 0, 0 },
    { 0xcb80fb00, 98 },
    { 0, 0 },
    { 0x3f40fb02, 98 },
    { 0xab60fb02, 90 },
    { 0x88f0fb04, 78 },
    { 0xfcd0fb05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d0fb0b, 33 },
    { 0x1800fb0c, 80 },
    { 0x14d0fb0d, 51 },
    { 0xd450fb0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c0fb11, 91 },
    { 0x1540fb12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c0fb18, 95 },
    { 0, 0 },
    { 0xcb70fb1a, 95 },
    { 0xec10fb1b, 76 },
    { 0xd240fb1c, 55 },
    { 0, 0 },
    { 0x320fb1e, 76 },
    { 0x83e0fb1e, 55 },
    { 0x9ee0fb20, 95 },
    { 0x8a20fb20, 90 },
    { 0, 0 },
    { 0x16b0fb23, 91 },
    { 0, 0 },
    { 0x8a60fb25, 33 },
    { 0xc7f0fb26, 66 },
    { 0xcdc0fb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b60fb2c, 61 },
    { 0xe840fb2c, 55 },
    { 0x9630fb2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990fb35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5630fb38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1850fb3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f0fb3e, 61 },
    { 0xa720fb3f, 68 },
    { 0x2930fb40, 95 },
    { 0x13b0fb3e, 77 },
    { 0, 0 },
    { 0xece0fb43, 23 },
    { 0xe080fb44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd0fb47, 23 },
    { 0x4d60fb47, 77 },
    { 0xb800fb49, 90 },
    { 0xbcb0fb4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9210fb54, 33 },
    { 0, 0 },
    { 0x87e0fb56, 23 },
    { 0x68d0fb57, 52 },
    { 0xac50fb56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc00fb5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe600fb66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d20fb69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d0fb6d, 90 },
    { 0x7260fb6e, 46 },
    { 0x81f0fb6f, 61 },
    { 0x2360fb6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c20fb7e, 34 },
    { 0x25f0fb7f, 95 },
    { 0xa830fb80, 65 },
    { 0, 0 },
    { 0x9870fb82, 95 },
    { 0, 0 },
    { 0xc20fb84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0fb91, 93 },
    { 0xe0e0fb92, 66 },
    { 0xb4c0fb93, 88 },
    { 0xcca0fb94, 76 },
    { 0x5250fb93, 34 },
    { 0, 0 },
    { 0x9800fb97, 33 },
    { 0x17b0fb98, 34 },
    { 0x8700fb97, 33 },
    { 0x78a0fb9a, 33 },
    { 0x3010fb98, 61 },
    { 0x2390fb9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8200fba0, 43 },
    { 0, 0 },
    { 0x3430fba2, 43 },
    { 0, 0 },
    { 0xe560fba4, 61 },
    { 0xc780fba5, 61 },
    { 0, 0 },
    { 0x9b00fba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd560fbab, 23 },
    { 0x380fbac, 33 },
    { 0xde80fbac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc300fbb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2460fbb5, 98 },
    { 0x3190fbb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf230fbb9, 98 },
    { 0x51b0fbb9, 34 },
    { 0xd5e0fbbb, 33 },
    { 0, 0 },
    { 0x6df0fbbd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0fbc0, 90 },
    { 0x54d0fbc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc0fbc8, 64 },
    { 0, 0 },
    { 0xd6c0fbca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b0fbce, 99 },
    { 0xa050fbcf, 52 },
    { 0xbcb0fbd0, 95 },
    { 0xfb20fbd1, 88 },
    { 0xa630fbd0, 93 },
    { 0xe2a0fbd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e0fbd9, 95 },
    { 0xaba0fbda, 68 },
    { 0xd050fbd9, 98 },
    { 0xc10fbdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb10fbe2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb20fbe6, 33 },
    { 0x22e0fbe7, 78 },
    { 0xc5e0fbe7, 98 },
    { 0x9890fbe7, 33 },
    { 0x8a40fbe9, 95 },
    { 0xd260fbe9, 23 },
    { 0xc300fbe8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa650fbf4, 61 },
    { 0x6090fbf4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd40fbf8, 76 },
    { 0x7d0fbf9, 98 },
    { 0x43f0fbf9, 34 },
    { 0x8b40fbfb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8400fc01, 95 },
    { 0, 0 },
    { 0xca70fc03, 90 },
    { 0xbac0fc04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4710fc08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c0fc13, 23 },
    { 0, 0 },
    { 0x9e40fc15, 95 },
    { 0xdd90fc16, 85 },
    { 0x4310fc16, 61 },
    { 0, 0 },
    { 0xd950fc19, 33 },
    { 0x12d0fc1a, 90 },
    { 0xd410fc1b, 34 },
    { 0x7a10fc1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8990fc20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c0fc2a, 43 },
    { 0x4b70fc2b, 90 },
    { 0, 0 },
    { 0x37d0fc2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d0fc3a, 90 },
    { 0, 0 },
    { 0x6a00fc3c, 52 },
    { 0x5670fc3d, 33 },
    { 0x29a0fc3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe0fc41, 70 },
    { 0, 0 },
    { 0xc480fc43, 33 },
    { 0x2930fc44, 55 },
    { 0xd8e0fc45, 93 },
    { 0xd8b0fc45, 98 },
    { 0xd2d0fc47, 23 },
    { 0x850fc48, 98 },
    { 0x6c00fc49, 76 },
    { 0x6d40fc49, 90 },
    { 0, 0 },
    { 0x2b20fc4c, 46 },
    { 0, 0 },
    { 0xd4c0fc4e, 98 },
    { 0, 0 },
    { 0xb3e0fc50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f0fc54, 55 },
    { 0x6510fc54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f40fc58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa50fc5b, 95 },
    { 0xa580fc5c, 76 },
    { 0x17f0fc5d, 90 },
    { 0, 0 },
    { 0xbe50fc5f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c0fc65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd90fc6b, 90 },
    { 0x9e40fc6b, 78 },
    { 0xc1a0fc6b, 61 },
    { 0x1e20fc6e, 99 },
    { 0xe20fc6f, 61 },
    { 0, 0 },
    { 0xd700fc71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2910fc76, 91 },
    { 0, 0 },
    { 0xb910fc78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e0fc7d, 90 },
    { 0x3470fc7e, 95 },
    { 0x640fc7e, 61 },
    { 0x6720fc7f, 95 },
    { 0, 0 },
    { 0xa170fc82, 90 },
    { 0xa600fc83, 73 },
    { 0x6550fc84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4860fc89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc80fc8d, 23 },
    { 0xedb0fc8d, 33 },
    { 0x91b0fc8d, 90 },
    { 0x90d0fc90, 23 },
    { 0x61d0fc8e, 33 },
    { 0xe860fc92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0fc97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be0fc9a, 90 },
    { 0, 0 },
    { 0xa170fc9c, 98 },
    { 0x8dc0fc9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f80fca7, 33 },
    { 0xce70fca8, 90 },
    { 0x8d10fca9, 76 },
    { 0x5790fca8, 90 },
    { 0xe070fcaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa40fcae, 98 },
    { 0xb8c0fcaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7420fcb6, 94 },
    { 0xdea0fcb7, 23 },
    { 0x1260fcb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd640fcbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf570fcc1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc200fcc4, 76 },
    { 0x8cb0fcc5, 61 },
    { 0xead0fcc6, 61 },
    { 0xeeb0fcc7, 77 },
    { 0x9160fcc8, 90 },
    { 0x2500fcc8, 23 },
    { 0x5a50fcc9, 65 },
    { 0x3820fcc7, 23 },
    { 0x1bb0fccc, 87 },
    { 0, 0 },
    { 0x3a40fcce, 90 },
    { 0xc210fccf, 96 },
    { 0x5f60fcd0, 100 },
    { 0x40a0fccf, 34 },
    { 0x6cb0fcd2, 93 },
    { 0xed70fccf, 23 },
    { 0x4040fcd4, 23 },
    { 0xb550fcd4, 90 },
    { 0xebc0fcd5, 61 },
    { 0x5570fcd4, 33 },
    { 0, 0 },
    { 0xce50fcd9, 76 },
    { 0x9d0fcda, 19 },
    { 0xf0d0fcdb, 68 },
    { 0x7030fcdc, 17 },
    { 0xbce0fcdd, 95 },
    { 0x85b0fcdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f00fce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae0fce5, 91 },
    { 0x7f40fce6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad0fce9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb0fcee, 65 },
    { 0xb040fcef, 52 },
    { 0x5600fcf0, 90 },
    { 0xd280fcf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b00fcf5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf720fcf8, 22 },
    { 0, 0 },
    { 0x940fcfa, 95 },
    { 0x2070fcfb, 61 },
    { 0x4290fcfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6710fd01, 33 },
    { 0, 0 },
    { 0xb00fd03, 55 },
    { 0x6e40fd04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce0fd07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e0fd0c, 33 },
    { 0, 0 },
    { 0x2d90fd0e, 61 },
    { 0, 0 },
    { 0x9c40fd10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe0fd17, 33 },
    { 0xa380fd17, 76 },
    { 0, 0 },
    { 0xebc0fd1a, 23 },
    { 0xd230fd1b, 34 },
    { 0xcee0fd1b, 95 },
    { 0xb90fd1b, 2 },
    { 0x3bb0fd1b, 46 },
    { 0, 0 },
    { 0x5820fd20, 76 },
    { 0x3bc0fd20, 34 },
    { 0x8890fd20, 90 },
    { 0xe660fd23, 61 },
    { 0xc4f0fd23, 23 },
    { 0x56c0fd25, 33 },
    { 0xf500fd26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4750fd29, 90 },
    { 0xf3d0fd2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7900fd32, 90 },
    { 0x5a50fd33, 90 },
    { 0x2610fd33, 91 },
    { 0x3d0fd35, 95 },
    { 0x1690fd36, 98 },
    { 0x4420fd37, 61 },
    { 0xf6c0fd37, 33 },
    { 0xa270fd32, 97 },
    { 0x3160fd3a, 23 },
    { 0x5a70fd3a, 61 },
    { 0x50e0fd3c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b40fd3f, 98 },
    { 0, 0 },
    { 0x6da0fd41, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1470fd45, 100 },
    { 0xe780fd45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9160fd49, 4 },
    { 0x59e0fd4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b0fd52, 61 },
    { 0x8ae0fd53, 23 },
    { 0xece0fd52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b60fd5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7970fd60, 98 },
    { 0x4030fd61, 98 },
    { 0, 0 },
    { 0x2710fd63, 33 },
    { 0, 0 },
    { 0xb40fd65, 99 },
    { 0x5fb0fd66, 90 },
    { 0, 0 },
    { 0x1330fd68, 98 },
    { 0x2c60fd69, 17 },
    { 0x6860fd69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa350fd70, 53 },
    { 0xbce0fd70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3960fd75, 33 },
    { 0x1630fd76, 90 },
    { 0x7af0fd77, 88 },
    { 0x3490fd78, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4850fd81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b0fd84, 93 },
    { 0xcc0fd85, 61 },
    { 0x1d90fd85, 17 },
    { 0xfab0fd85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe940fd8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af0fd8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e50fd92, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f0fd96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00fd9a, 78 },
    { 0x6dc0fd9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd00fda1, 61 },
    { 0, 0 },
    { 0x2340fda3, 100 },
    { 0x7530fda4, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe950fdaa, 90 },
    { 0x3010fdaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f40fdae, 55 },
    { 0x5bc0fdae, 90 },
    { 0xd60fdaf, 67 },
    { 0xa5e0fdb1, 95 },
    { 0x7740fdb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d0fdb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e70fdbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70fdc4, 33 },
    { 0, 0 },
    { 0xd3a0fdc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba0fdca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4440fdd1, 66 },
    { 0, 0 },
    { 0x8e30fdd3, 72 },
    { 0, 0 },
    { 0xaac0fdd5, 90 },
    { 0xf0b0fdd5, 95 },
    { 0, 0 },
    { 0x51d0fdd8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1280fddb, 90 },
    { 0, 0 },
    { 0xfb10fddd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b0fde0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8870fde3, 46 },
    { 0, 0 },
    { 0x7370fde5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3770fdee, 44 },
    { 0x54c0fdef, 52 },
    { 0x4cc0fdf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc940fdfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80fe00, 33 },
    { 0x4c50fe01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7050fe0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae0fe11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf0fe14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9410fe17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1550fe20, 40 },
    { 0x20d0fe20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a0fe27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e0fe2c, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xed00fe2f, 76 },
    { 0xe780fe2f, 23 },
    { 0x9280fe31, 81 },
    { 0x7580fe32, 61 },
    { 0xa0fe33, 23 },
    { 0xf90fe32, 43 },
    { 0x63d0fe2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d90fe3a, 46 },
    { 0x8830fe3b, 97 },
    { 0xbb00fe3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f0fe3f, 43 },
    { 0x1890fe3f, 55 },
    { 0, 0 },
    { 0xc1e0fe42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d0fe45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6580fe48, 36 },
    { 0, 0 },
    { 0x34d0fe4a, 25 },
    { 0x47d0fe4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe210fe4e, 33 },
    { 0, 0 },
    { 0x6fa0fe50, 90 },
    { 0x13e0fe51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5940fe55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b0fe59, 23 },
    { 0x3c0fe5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300fe5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90fe65, 98 },
    { 0xd8d0fe66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40fe6e, 34 },
    { 0x8870fe6e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b0fe72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e20fe77, 23 },
    { 0xd320fe78, 61 },
    { 0, 0 },
    { 0x4740fe7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80fe7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a0fe80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3290fe83, 98 },
    { 0, 0 },
    { 0xcbf0fe85, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e0fe88, 23 },
    { 0x41a0fe89, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e90fe8c, 95 },
    { 0x1410fe8d, 76 },
    { 0x14b0fe8c, 90 },
    { 0x8eb0fe8f, 33 },
    { 0x50b0fe90, 33 },
    { 0xa490fe8f, 33 },
    { 0x8210fe92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb780fe9c, 23 },
    { 0, 0 },
    { 0xd910fe9e, 85 },
    { 0, 0 },
    { 0x3350fea0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc770fea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0fea9, 33 },
    { 0, 0 },
    { 0x6a80feab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d0feb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1530feb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1270febc, 69 },
    { 0, 0 },
    { 0x2f0febe, 61 },
    { 0xdd70febe, 98 },
    { 0, 0 },
    { 0x1fa0fec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d0fec8, 23 },
    { 0xe990fec8, 55 },
    { 0x9d50fec9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2820fecf, 23 },
    { 0xfbe0fed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd340fed4, 61 },
    { 0, 0 },
    { 0x850fed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf230fed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde0fedc, 61 },
    { 0xbca0fedd, 91 },
    { 0x5c40fedc, 34 },
    { 0, 0 },
    { 0xfbc0fee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d50fee3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef0fee9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa830feef, 93 },
    { 0, 0 },
    { 0x5e30fef1, 46 },
    { 0x8da0fef2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd00fef6, 90 },
    { 0, 0 },
    { 0x5640fef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b50ff01, 99 },
    { 0x8aa0ff01, 87 },
    { 0xea20ff02, 95 },
    { 0x1170ff04, 90 },
    { 0x9d70ff04, 95 },
    { 0x3080ff04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a0ff0b, 43 },
    { 0, 0 },
    { 0xe00ff0d, 90 },
    { 0x3480ff0e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc710ff16, 17 },
    { 0xfda0ff17, 90 },
    { 0, 0 },
    { 0xe20ff19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd0ff1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9960ff23, 43 },
    { 0xd7c0ff24, 98 },
    { 0x1730ff25, 95 },
    { 0x2f50ff25, 34 },
    { 0, 0 },
    { 0x8e40ff28, 56 },
    { 0, 0 },
    { 0x60b0ff2a, 33 },
    { 0xf420ff2b, 98 },
    { 0, 0 },
    { 0xdb20ff2d, 43 },
    { 0x8560ff2e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab40ff32, 78 },
    { 0xd650ff33, 88 },
    { 0x6550ff34, 33 },
    { 0xd6f0ff34, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b90ff39, 99 },
    { 0, 0 },
    { 0x74e0ff3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8040ff3e, 95 },
    { 0xb630ff3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de0ff48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d0ff4d, 61 },
    { 0, 0 },
    { 0x8a90ff4f, 34 },
    { 0, 0 },
    { 0xc5f0ff51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c0ff57, 61 },
    { 0x8ac0ff57, 61 },
    { 0, 0 },
    { 0x3050ff5a, 95 },
    { 0, 0 },
    { 0xffb0ff5c, 98 },
    { 0x2c10ff5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c10ff61, 90 },
    { 0xb4e0ff61, 75 },
    { 0, 0 },
    { 0x4a70ff64, 91 },
    { 0xc810ff65, 23 },
    { 0x94f0ff66, 61 },
    { 0, 0 },
    { 0x1220ff68, 55 },
    { 0xe770ff69, 93 },
    { 0xd840ff6a, 95 },
    { 0xbad0ff6a, 23 },
    { 0x5150ff6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc0ff6f, 97 },
    { 0xc400ff70, 61 },
    { 0x8cd0ff71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc870ff74, 55 },
    { 0, 0 },
    { 0x6bf0ff76, 98 },
    { 0xf3f0ff77, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c0ff7d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa200ff84, 61 },
    { 0, 0 },
    { 0xd810ff86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda80ff8e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b80ff93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb70ff97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1930ff9a, 84 },
    { 0xb6c0ff9b, 81 },
    { 0, 0 },
    { 0x89b0ff9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd0ffa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac0ffaf, 75 },
    { 0xbb70ffb0, 33 },
    { 0x7990ffaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa700ffb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0ffbf, 23 },
    { 0, 0 },
    { 0xf150ffc1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca0ffc8, 23 },
    { 0xc5e0ffc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a0ffcc, 33 },
    { 0, 0 },
    { 0xdc50ffce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e0ffd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea0ffd7, 76 },
    { 0x5c40ffd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e0ffde, 75 },
    { 0xc370ffdf, 61 },
    { 0, 0 },
    { 0x17b0ffe1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2050ffe9, 23 },
    { 0, 0 },
    { 0x7410ffeb, 46 },
    { 0xca50ffec, 91 },
    { 0xc3c0ffed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a00fff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8200fff3, 88 },
    { 0x52b0fff4, 81 },
    { 0x5dd0fff5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa0fffb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5320ffff, 55 },
    { 0, 0 },
    { 0xab10001, 95 },
    { 0xb4310002, 61 },
    { 0, 0 },
    { 0x5eb10004, 44 },
    { 0, 0 },
    { 0xf8d10006, 98 },
    { 0x62a10007, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b51000a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d31000e, 90 },
    { 0, 0 },
    { 0x10010, 87 },
    { 0x2ad10010, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1110016, 90 },
    { 0xfee10016, 23 },
    { 0x7b310017, 90 },
    { 0xf110019, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b1001e, 75 },
    { 0, 0 },
    { 0xb2110020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94710024, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9421002d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4610031, 90 },
    { 0x39810031, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff310036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31610045, 95 },
    { 0, 0 },
    { 0xae410047, 98 },
    { 0, 0 },
    { 0x53110049, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be1004f, 33 },
    { 0, 0 },
    { 0xc0b10051, 61 },
    { 0, 0 },
    { 0xd5910053, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b10059, 95 },
    { 0, 0 },
    { 0xfc21005b, 46 },
    { 0xc771005c, 33 },
    { 0x8071005c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1710062, 33 },
    { 0x2c610063, 95 },
    { 0x26410064, 98 },
    { 0x59410064, 43 },
    { 0x20710063, 90 },
    { 0xc5010066, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa531006c, 90 },
    { 0xde61006c, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c10070, 23 },
    { 0x14210071, 46 },
    { 0xbc010072, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa110075, 90 },
    { 0, 0 },
    { 0xb3510077, 90 },
    { 0, 0 },
    { 0x47810079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f1007c, 46 },
    { 0xba01007d, 33 },
    { 0xa601007d, 100 },
    { 0x96f1007e, 33 },
    { 0xe0f1007e, 90 },
    { 0xce21007d, 46 },
    { 0x44610082, 90 },
    { 0x54c10082, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99610088, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d1008c, 90 },
    { 0, 0 },
    { 0xba51008e, 46 },
    { 0x101008f, 75 },
    { 0x3d810090, 55 },
    { 0x54010091, 76 },
    { 0x7a210090, 52 },
    { 0xed1008e, 90 },
    { 0x92610094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf10097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b51009a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b1009e, 66 },
    { 0xa901009f, 17 },
    { 0xff4100a0, 95 },
    { 0, 0 },
    { 0x7a7100a2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d100a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca100aa, 90 },
    { 0, 0 },
    { 0x2ea100ac, 93 },
    { 0xeb100ad, 95 },
    { 0xd73100ae, 34 },
    { 0x553100af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0100b8, 23 },
    { 0, 0 },
    { 0x1e7100ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368100be, 43 },
    { 0x3a7100bf, 52 },
    { 0xe82100c0, 95 },
    { 0xfb5100c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f100c5, 61 },
    { 0x705100c5, 98 },
    { 0, 0 },
    { 0x724100c8, 90 },
    { 0xaf3100c9, 33 },
    { 0x3c1100ca, 23 },
    { 0, 0 },
    { 0xcc4100cc, 26 },
    { 0, 0 },
    { 0x1e8100ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b100d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe100da, 46 },
    { 0xd00100db, 61 },
    { 0x718100dc, 58 },
    { 0x453100dc, 34 },
    { 0x95e100de, 34 },
    { 0x2ec100dd, 33 },
    { 0xa96100e0, 23 },
    { 0xd1a100dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d100e7, 23 },
    { 0x817100e7, 75 },
    { 0, 0 },
    { 0x835100ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0100ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0100f2, 40 },
    { 0x314100f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1100f6, 34 },
    { 0xc5d100f7, 23 },
    { 0x352100f6, 63 },
    { 0xbb3100f9, 80 },
    { 0x801100fa, 61 },
    { 0xd21100fb, 90 },
    { 0xd7b100f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5100ff, 52 },
    { 0x88110100, 90 },
    { 0x83a10101, 33 },
    { 0xd9810101, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8510107, 90 },
    { 0x2e610108, 98 },
    { 0x1f610109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b61010d, 90 },
    { 0, 0 },
    { 0xacd1010f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee610116, 64 },
    { 0, 0 },
    { 0x47310118, 84 },
    { 0x3ba10118, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb261011c, 52 },
    { 0x32a1011d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42610122, 77 },
    { 0, 0 },
    { 0xf1a10124, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe10127, 23 },
    { 0x26b10127, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd61012b, 34 },
    { 0, 0 },
    { 0x95b1012d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8210130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a10139, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f1013c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd810140, 77 },
    { 0x90a10141, 55 },
    { 0xd5d10141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80010147, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f1014b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d1014e, 93 },
    { 0xa091014f, 52 },
    { 0xa871014f, 33 },
    { 0, 0 },
    { 0xaf610152, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b310157, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8441015f, 46 },
    { 0, 0 },
    { 0xccb10161, 61 },
    { 0, 0 },
    { 0x33810163, 66 },
    { 0xee710163, 43 },
    { 0, 0 },
    { 0x77110166, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4501016a, 67 },
    { 0, 0 },
    { 0x5631016c, 33 },
    { 0x2be1016d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c10171, 61 },
    { 0xec310172, 34 },
    { 0, 0 },
    { 0xdfd10174, 95 },
    { 0xb3310174, 77 },
    { 0x33e10176, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8471017a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4401017d, 61 },
    { 0xb4a1017e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6810181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe210184, 33 },
    { 0, 0 },
    { 0x86b10186, 90 },
    { 0xa3810187, 90 },
    { 0x8a410188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce1018c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee1018f, 33 },
    { 0, 0 },
    { 0x7af10191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5910194, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d10198, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00101a3, 64 },
    { 0x879101a4, 77 },
    { 0x7ea101a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b101a9, 61 },
    { 0x746101a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x165101ad, 95 },
    { 0x5f101ae, 95 },
    { 0x7e1101af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d101b5, 90 },
    { 0, 0 },
    { 0x50e101b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e101bc, 98 },
    { 0x2de101bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4101c9, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x212101cc, 61 },
    { 0x10c101cd, 33 },
    { 0, 0 },
    { 0xe2a101cf, 35 },
    { 0xb3f101d0, 23 },
    { 0x217101d0, 85 },
    { 0xcf101d2, 81 },
    { 0x9ae101d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2101d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2101da, 43 },
    { 0x629101db, 61 },
    { 0x332101db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8101e0, 76 },
    { 0xa09101e1, 61 },
    { 0x5a6101e2, 91 },
    { 0xe93101e3, 35 },
    { 0x7e0101e3, 76 },
    { 0, 0 },
    { 0xc94101e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25101eb, 61 },
    { 0xe0d101eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6101ef, 66 },
    { 0x2c0101f0, 33 },
    { 0xd22101f1, 61 },
    { 0, 0 },
    { 0xb6e101f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5101f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc101fc, 23 },
    { 0x34a101fd, 33 },
    { 0x38b101fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61910207, 95 },
    { 0x9a110208, 61 },
    { 0xcf210208, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b1020d, 98 },
    { 0x3331020d, 61 },
    { 0xc731020e, 61 },
    { 0x8b31020d, 4 },
    { 0x4f810211, 34 },
    { 0xe091020f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91021d, 43 },
    { 0, 0 },
    { 0x35a1021f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d10222, 33 },
    { 0, 0 },
    { 0x7210224, 46 },
    { 0, 0 },
    { 0x6c010226, 23 },
    { 0xa2b10226, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6911022c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce10232, 76 },
    { 0x6e410232, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde810237, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e1023b, 61 },
    { 0x3441023c, 43 },
    { 0, 0 },
    { 0x2741023e, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2910242, 99 },
    { 0xdbc10243, 33 },
    { 0, 0 },
    { 0x67610245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4381024a, 33 },
    { 0, 0 },
    { 0xbbd1024c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf410257, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b10260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca10263, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29910268, 88 },
    { 0, 0 },
    { 0x7da1026a, 55 },
    { 0, 0 },
    { 0xb481026c, 51 },
    { 0x89b1026d, 67 },
    { 0x7821026d, 33 },
    { 0x6661026f, 88 },
    { 0x30d10270, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf010274, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27910279, 23 },
    { 0x8cd1027a, 98 },
    { 0x9cc1027b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1027f, 33 },
    { 0x98f10280, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe110285, 33 },
    { 0, 0 },
    { 0x7b910287, 84 },
    { 0, 0 },
    { 0xd7210289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f1028d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a310292, 43 },
    { 0xc2010293, 23 },
    { 0x89310293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe561029a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad102a0, 90 },
    { 0, 0 },
    { 0xda7102a2, 88 },
    { 0x2bb102a2, 55 },
    { 0, 0 },
    { 0x89102a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927102ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b102b0, 61 },
    { 0xf58102b1, 95 },
    { 0xe82102b2, 61 },
    { 0x420102b3, 23 },
    { 0, 0 },
    { 0x401102b5, 34 },
    { 0x650102b6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea102bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e102c2, 95 },
    { 0, 0 },
    { 0xc3d102c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85102cd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976102d1, 95 },
    { 0, 0 },
    { 0xb9d102d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57102d6, 99 },
    { 0x71c102d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d102dc, 33 },
    { 0, 0 },
    { 0x2bb102de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6102e1, 78 },
    { 0, 0 },
    { 0xbd3102e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a102e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b102e9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be102ec, 76 },
    { 0, 0 },
    { 0xb97102ee, 78 },
    { 0xc1a102ef, 88 },
    { 0x8b8102f0, 90 },
    { 0xe76102f0, 95 },
    { 0x44f102ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4102f6, 33 },
    { 0xd50102f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967102fc, 74 },
    { 0x8ab102fd, 78 },
    { 0x3c5102fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe510307, 33 },
    { 0x2c410308, 76 },
    { 0x51810309, 67 },
    { 0x7aa10309, 23 },
    { 0xeb71030a, 91 },
    { 0x8d71030a, 76 },
    { 0, 0 },
    { 0xed71030e, 95 },
    { 0x9421030f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d10312, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41610316, 40 },
    { 0x21210316, 55 },
    { 0x71210318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f010322, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3310325, 52 },
    { 0x60110326, 95 },
    { 0, 0 },
    { 0x8e910328, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb010330, 64 },
    { 0x11710330, 61 },
    { 0x89610332, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea710335, 34 },
    { 0xf110335, 94 },
    { 0xfa810337, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc71033c, 66 },
    { 0xaa91033d, 52 },
    { 0x2ee1033c, 81 },
    { 0, 0 },
    { 0x1cd10340, 61 },
    { 0x6d110341, 49 },
    { 0x4fd10340, 95 },
    { 0xb1310343, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2710348, 88 },
    { 0x2f710349, 81 },
    { 0x88e1034a, 33 },
    { 0x2541034b, 23 },
    { 0x7f21034b, 55 },
    { 0x6771034b, 98 },
    { 0x74d10348, 90 },
    { 0xa251034f, 90 },
    { 0x5fd10350, 98 },
    { 0x1eb1034f, 55 },
    { 0xc9910352, 81 },
    { 0xd2d10353, 33 },
    { 0xd071034f, 90 },
    { 0x1210355, 61 },
    { 0x8c210356, 61 },
    { 0, 0 },
    { 0xb6910358, 23 },
    { 0, 0 },
    { 0x8491035a, 90 },
    { 0x3031035b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa731035e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35710366, 84 },
    { 0, 0 },
    { 0xa2110368, 98 },
    { 0xf0510369, 90 },
    { 0xd121036a, 76 },
    { 0x2210369, 55 },
    { 0, 0 },
    { 0xbfa1036d, 23 },
    { 0x5f61036e, 61 },
    { 0xc681036f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0610377, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de1037a, 98 },
    { 0xe911037b, 81 },
    { 0xa671037c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69710380, 85 },
    { 0xdb110381, 93 },
    { 0, 0 },
    { 0x81910383, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa331038a, 95 },
    { 0x5881038b, 76 },
    { 0xccc1038c, 81 },
    { 0xb821038d, 93 },
    { 0xc131038e, 55 },
    { 0x40d1038f, 90 },
    { 0xd1610390, 61 },
    { 0x5c110391, 23 },
    { 0x4d310391, 93 },
    { 0xf2e10391, 95 },
    { 0xccd10394, 55 },
    { 0x6a710395, 76 },
    { 0x48810396, 55 },
    { 0, 0 },
    { 0xc1f10398, 90 },
    { 0x99110398, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed1039d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a103a2, 95 },
    { 0x37d103a3, 37 },
    { 0x2d0103a2, 55 },
    { 0, 0 },
    { 0x228103a6, 85 },
    { 0x5a4103a7, 33 },
    { 0x101103a7, 99 },
    { 0xea8103a6, 55 },
    { 0xead103aa, 95 },
    { 0x292103aa, 68 },
    { 0x825103ac, 88 },
    { 0x25103ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3103b0, 33 },
    { 0x7b4103b0, 91 },
    { 0, 0 },
    { 0x589103b3, 67 },
    { 0xd28103b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7103b7, 33 },
    { 0x530103b8, 90 },
    { 0xcd0103b9, 61 },
    { 0xf6a103ba, 90 },
    { 0x29b103ba, 33 },
    { 0xd8b103bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf103bf, 38 },
    { 0xe08103bf, 33 },
    { 0, 0 },
    { 0xf67103c2, 61 },
    { 0xb17103c3, 85 },
    { 0x1cd103c4, 66 },
    { 0xb5d103c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b103cb, 75 },
    { 0x1c8103cc, 75 },
    { 0x337103cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5103d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237103d8, 33 },
    { 0x7b3103d9, 23 },
    { 0xf14103da, 90 },
    { 0x8f2103db, 77 },
    { 0xdf103dc, 50 },
    { 0x65d103dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e103e6, 40 },
    { 0, 0 },
    { 0xd52103e8, 86 },
    { 0xb90103e9, 56 },
    { 0x9c103e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19103ed, 61 },
    { 0xead103ed, 33 },
    { 0x849103ef, 23 },
    { 0x1a6103ef, 94 },
    { 0, 0 },
    { 0x276103f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba103f7, 56 },
    { 0, 0 },
    { 0xd95103f9, 84 },
    { 0x517103f9, 61 },
    { 0x160103fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa510402, 46 },
    { 0, 0 },
    { 0xc4010404, 100 },
    { 0xed810405, 23 },
    { 0, 0 },
    { 0xdec10407, 100 },
    { 0xe610408, 98 },
    { 0, 0 },
    { 0xf3e1040a, 34 },
    { 0, 0 },
    { 0x5a51040c, 61 },
    { 0x9341040c, 100 },
    { 0x3541040e, 33 },
    { 0xfbc1040f, 55 },
    { 0x50610410, 23 },
    { 0xd9a10411, 90 },
    { 0x95010412, 55 },
    { 0x851040f, 95 },
    { 0x2a210414, 69 },
    { 0x88d10410, 23 },
    { 0xdf10416, 98 },
    { 0xe1110417, 76 },
    { 0xdbd10418, 98 },
    { 0x40910412, 95 },
    { 0x5dc10417, 34 },
    { 0xad71040d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca1041e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb510421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa10424, 95 },
    { 0x44d10424, 61 },
    { 0xf6f10426, 34 },
    { 0xc8610426, 98 },
    { 0x7f810425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef11042d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f10430, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e710433, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea510445, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e10448, 95 },
    { 0x67610448, 23 },
    { 0, 0 },
    { 0x9cc1044b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5491044e, 95 },
    { 0, 0 },
    { 0xc3910450, 93 },
    { 0x78610451, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae10457, 90 },
    { 0x90610457, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e10468, 43 },
    { 0xba310468, 33 },
    { 0xb141046a, 98 },
    { 0, 0 },
    { 0x2b91046c, 90 },
    { 0xa6f1046d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2810472, 46 },
    { 0x89d10473, 33 },
    { 0x71d10474, 52 },
    { 0x8fb10474, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa10479, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1047d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1110482, 66 },
    { 0x76810482, 98 },
    { 0x43d10482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1910488, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x641048b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e21048e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83910491, 34 },
    { 0, 0 },
    { 0xe7c10493, 61 },
    { 0x55c10493, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b10499, 99 },
    { 0x66a1049a, 33 },
    { 0xf5b1049b, 33 },
    { 0, 0 },
    { 0xc811049d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c104a0, 38 },
    { 0, 0 },
    { 0xb45104a2, 33 },
    { 0x929104a3, 95 },
    { 0, 0 },
    { 0x481104a5, 61 },
    { 0x561104a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0104ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46104ae, 33 },
    { 0x47b104af, 93 },
    { 0, 0 },
    { 0x30f104b1, 61 },
    { 0xa67104b2, 78 },
    { 0x3d2104b2, 69 },
    { 0x287104b4, 85 },
    { 0xeef104b5, 87 },
    { 0x8e7104b5, 17 },
    { 0x86d104b2, 93 },
    { 0x496104b8, 66 },
    { 0x97a104b9, 91 },
    { 0x20e104ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c104be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce104c3, 61 },
    { 0xaec104c4, 33 },
    { 0xdf9104c5, 46 },
    { 0x849104c6, 78 },
    { 0, 0 },
    { 0x928104c8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7104cb, 23 },
    { 0, 0 },
    { 0x590104cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5104d1, 52 },
    { 0, 0 },
    { 0xca4104d3, 76 },
    { 0xab0104d4, 77 },
    { 0, 0 },
    { 0xfca104d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x216104d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e104dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5104e3, 33 },
    { 0xd71104e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539104e9, 89 },
    { 0x134104e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf104ef, 98 },
    { 0xd26104ef, 90 },
    { 0, 0 },
    { 0xcbb104f2, 52 },
    { 0x194104f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58104f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07104fb, 33 },
    { 0, 0 },
    { 0xa26104fd, 33 },
    { 0, 0 },
    { 0x3e1104ff, 95 },
    { 0x11510500, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee710506, 55 },
    { 0x4d510506, 23 },
    { 0x11e10507, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2841050d, 90 },
    { 0xe111050e, 61 },
    { 0x51d1050f, 76 },
    { 0xd1d10510, 99 },
    { 0x14910511, 55 },
    { 0, 0 },
    { 0x32510513, 55 },
    { 0xe2610514, 55 },
    { 0, 0 },
    { 0xb2610516, 93 },
    { 0, 0 },
    { 0x2be10518, 17 },
    { 0x9bf10519, 33 },
    { 0x60c10519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c91051d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef10529, 98 },
    { 0x7291052a, 75 },
    { 0, 0 },
    { 0x6561052c, 61 },
    { 0xf7a1052c, 76 },
    { 0x5b11052c, 24 },
    { 0x1141052f, 33 },
    { 0x751052f, 90 },
    { 0x22910531, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6710535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94010538, 23 },
    { 0, 0 },
    { 0x4e1053a, 55 },
    { 0xb951053b, 33 },
    { 0xe621053c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c10543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef10549, 55 },
    { 0x8cb1054a, 85 },
    { 0x6cb1054b, 75 },
    { 0xdee1054a, 90 },
    { 0, 0 },
    { 0x5dd1054e, 46 },
    { 0x89e1054f, 76 },
    { 0x4a310550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74610556, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b10559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd410560, 95 },
    { 0, 0 },
    { 0x8a210562, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2601056f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f10572, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac410576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c10579, 90 },
    { 0xbe81057a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e41057f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39610584, 87 },
    { 0x30710585, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c91058d, 66 },
    { 0x4761058e, 79 },
    { 0, 0 },
    { 0xd1910590, 55 },
    { 0, 0 },
    { 0xb8510592, 67 },
    { 0xc3110593, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c10597, 67 },
    { 0, 0 },
    { 0xf9d10599, 23 },
    { 0xe881059a, 61 },
    { 0xd4710599, 90 },
    { 0x96610599, 98 },
    { 0x5b81059d, 67 },
    { 0x47f1059b, 61 },
    { 0xffe1059c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a105a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855105b7, 33 },
    { 0x82105b8, 90 },
    { 0x404105b8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33105bd, 93 },
    { 0, 0 },
    { 0x1a9105bf, 93 },
    { 0, 0 },
    { 0xc3b105c1, 33 },
    { 0, 0 },
    { 0xb10105c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189105c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x192105ca, 90 },
    { 0x444105cb, 98 },
    { 0xd8f105ca, 90 },
    { 0x5c0105cd, 95 },
    { 0x49c105ce, 90 },
    { 0, 0 },
    { 0xe9e105d0, 95 },
    { 0x588105d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b105d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958105dd, 76 },
    { 0, 0 },
    { 0x860105df, 91 },
    { 0x280105df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76105e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d105e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e105eb, 76 },
    { 0xc105ec, 78 },
    { 0xb88105ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5105f5, 55 },
    { 0x13c105f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f10600, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6110606, 52 },
    { 0x63610607, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81710610, 76 },
    { 0, 0 },
    { 0x62f10612, 68 },
    { 0, 0 },
    { 0xa7710614, 95 },
    { 0x32f10615, 91 },
    { 0xb1710615, 55 },
    { 0xa0710617, 93 },
    { 0, 0 },
    { 0x59c10619, 89 },
    { 0, 0 },
    { 0x3001061b, 95 },
    { 0x49d1061b, 90 },
    { 0xd891061c, 67 },
    { 0x77d1061b, 98 },
    { 0xbfc1061f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce010624, 90 },
    { 0, 0 },
    { 0xc3410626, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c10629, 23 },
    { 0x6101062a, 52 },
    { 0x41410629, 40 },
    { 0x661062c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26310633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b10637, 33 },
    { 0, 0 },
    { 0x22c10639, 52 },
    { 0, 0 },
    { 0x3611063b, 76 },
    { 0xbb21063c, 81 },
    { 0x6c21063c, 99 },
    { 0x9931063c, 34 },
    { 0x4e81063c, 80 },
    { 0x6b010640, 56 },
    { 0x2610641, 77 },
    { 0xd3d10642, 61 },
    { 0x86910641, 98 },
    { 0xe6f10640, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e10648, 90 },
    { 0xfad10648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b1064d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a10652, 34 },
    { 0x3f010652, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c10657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d11065a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa1065d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea10668, 53 },
    { 0, 0 },
    { 0xfc21066a, 76 },
    { 0x7471066a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6071066e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54810671, 61 },
    { 0xf6910672, 90 },
    { 0x3f10671, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf510676, 33 },
    { 0xbf510676, 33 },
    { 0xac510678, 98 },
    { 0x4ad10679, 61 },
    { 0, 0 },
    { 0xe651067b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52010681, 61 },
    { 0x40410682, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6210685, 81 },
    { 0x52f10685, 95 },
    { 0xde510687, 23 },
    { 0, 0 },
    { 0x3f910689, 55 },
    { 0x7861068a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc971068d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc210694, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf310697, 55 },
    { 0x67c10697, 61 },
    { 0, 0 },
    { 0x3071069a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8691069e, 55 },
    { 0x9121069f, 52 },
    { 0x41d1069f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5106a4, 91 },
    { 0x955106a4, 23 },
    { 0, 0 },
    { 0xfb4106a7, 55 },
    { 0x505106a8, 61 },
    { 0xbd3106a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d106ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f106b9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x575106bc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46106c0, 90 },
    { 0, 0 },
    { 0x555106c2, 43 },
    { 0x6ad106c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39106c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d106cd, 81 },
    { 0xb95106ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9106d1, 33 },
    { 0, 0 },
    { 0x15d106d3, 76 },
    { 0x62c106d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b106d7, 98 },
    { 0xf9f106d8, 33 },
    { 0x442106d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46106dd, 81 },
    { 0xf01106dd, 33 },
    { 0x52106df, 61 },
    { 0, 0 },
    { 0x454106e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357106ea, 34 },
    { 0, 0 },
    { 0x75106ec, 93 },
    { 0xc1a106ed, 36 },
    { 0, 0 },
    { 0x12106ef, 98 },
    { 0xf15106f0, 61 },
    { 0xe17106f1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5106fd, 76 },
    { 0x612106fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb10703, 88 },
    { 0xf7810704, 34 },
    { 0xb4a10705, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa051070a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf10712, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a10716, 65 },
    { 0x4b410716, 84 },
    { 0x44410718, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901071b, 69 },
    { 0xd391071c, 95 },
    { 0, 0 },
    { 0x8ea1071e, 90 },
    { 0, 0 },
    { 0x3d410720, 43 },
    { 0xe2010721, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b10724, 98 },
    { 0, 0 },
    { 0x2c810726, 90 },
    { 0, 0 },
    { 0x3e310728, 98 },
    { 0, 0 },
    { 0x18b1072a, 81 },
    { 0x9811072b, 46 },
    { 0xd831072b, 98 },
    { 0xdd1072b, 61 },
    { 0xb191072e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac10734, 64 },
    { 0xec310734, 98 },
    { 0x73f10735, 43 },
    { 0, 0 },
    { 0xd610738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd251073d, 23 },
    { 0xe711073d, 46 },
    { 0x7491073f, 17 },
    { 0, 0 },
    { 0x28410741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed110746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5021074a, 67 },
    { 0x7e1074b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1661074e, 23 },
    { 0, 0 },
    { 0xf8910750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f310757, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91075d, 98 },
    { 0xfb71075d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67810761, 90 },
    { 0xd3f10761, 56 },
    { 0x5610763, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45510768, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2721076d, 33 },
    { 0x1601076d, 98 },
    { 0x43c1076f, 78 },
    { 0x9141076e, 95 },
    { 0x2d110771, 95 },
    { 0x7bd1076d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8261077b, 23 },
    { 0x2691077b, 84 },
    { 0x6ef1077c, 33 },
    { 0x3a71077e, 33 },
    { 0x3921077d, 91 },
    { 0x8d51077d, 100 },
    { 0xc7c10781, 34 },
    { 0xd5a10782, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d1078b, 88 },
    { 0xbcb1078b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c10790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae10793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df1079b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b107a3, 90 },
    { 0x933107a4, 90 },
    { 0x3f9107a5, 60 },
    { 0x789107a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28107aa, 23 },
    { 0x6e7107aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286107af, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x940107b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c107b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef107bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34107c0, 95 },
    { 0xe93107c1, 75 },
    { 0x10f107c2, 100 },
    { 0x1ce107c2, 90 },
    { 0x5d4107c4, 93 },
    { 0, 0 },
    { 0x6fc107c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662107ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172107cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0107d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb107d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x341107da, 90 },
    { 0, 0 },
    { 0x301107dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd107e0, 33 },
    { 0x2c9107e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b107e4, 34 },
    { 0x73b107e5, 98 },
    { 0xff3107e6, 23 },
    { 0, 0 },
    { 0xd54107e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6107ed, 33 },
    { 0xafa107ee, 56 },
    { 0xd94107ef, 98 },
    { 0xa1107f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c107f3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3107fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x905107ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c10804, 98 },
    { 0x20a10805, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b10808, 33 },
    { 0x64a10809, 23 },
    { 0xbcc1080a, 90 },
    { 0, 0 },
    { 0x41d1080c, 34 },
    { 0, 0 },
    { 0x7751080e, 55 },
    { 0x6fa1080f, 75 },
    { 0xc331080e, 34 },
    { 0xb6b10811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd10816, 76 },
    { 0, 0 },
    { 0x4a310818, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f1081c, 98 },
    { 0x75b1081d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3510820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d10824, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c1082a, 33 },
    { 0x1171082b, 46 },
    { 0xb661082b, 33 },
    { 0xec51082a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1110835, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde71083b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab1083e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9461084b, 2 },
    { 0, 0 },
    { 0x6a21084d, 93 },
    { 0, 0 },
    { 0x64a1084f, 61 },
    { 0x1181084f, 68 },
    { 0x23510851, 78 },
    { 0, 0 },
    { 0x61e10853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73910858, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a1085d, 61 },
    { 0x6331085e, 55 },
    { 0x1181085f, 23 },
    { 0xed010860, 65 },
    { 0, 0 },
    { 0xa6410862, 80 },
    { 0x9f110863, 99 },
    { 0, 0 },
    { 0x59c10865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3861086a, 95 },
    { 0, 0 },
    { 0xf571086c, 95 },
    { 0, 0 },
    { 0xd881086e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d310871, 33 },
    { 0x6b910871, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f10875, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8010879, 23 },
    { 0xb6c1087a, 95 },
    { 0, 0 },
    { 0x6ec1087c, 87 },
    { 0xe981087c, 90 },
    { 0x2001087e, 46 },
    { 0xcf71087f, 77 },
    { 0x6df1087f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee1088a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1210892, 33 },
    { 0x3dd10893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f510897, 52 },
    { 0x37f10897, 91 },
    { 0xf9810899, 23 },
    { 0x35210899, 61 },
    { 0xa0c1089a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4801089f, 37 },
    { 0xba81089f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106108a4, 47 },
    { 0xdb3108a4, 55 },
    { 0x755108a4, 43 },
    { 0xf92108a7, 88 },
    { 0x8fd108a7, 95 },
    { 0x15c108a9, 98 },
    { 0x3ee108aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf108af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0108b2, 38 },
    { 0xa2f108b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e108b6, 78 },
    { 0x7c8108b7, 76 },
    { 0, 0 },
    { 0xfab108b9, 33 },
    { 0x434108ba, 76 },
    { 0x4aa108b9, 78 },
    { 0xa0c108b9, 55 },
    { 0xf9d108ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69108c7, 90 },
    { 0xb37108c8, 98 },
    { 0x845108c9, 76 },
    { 0x9f1108ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8108d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04108d9, 33 },
    { 0xb64108d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3108df, 93 },
    { 0x99e108df, 61 },
    { 0x136108df, 43 },
    { 0xa49108e2, 90 },
    { 0x958108e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df108e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f108ea, 25 },
    { 0x58a108eb, 23 },
    { 0, 0 },
    { 0x73d108ed, 23 },
    { 0x73e108ee, 90 },
    { 0xff0108ee, 99 },
    { 0x77108f0, 33 },
    { 0, 0 },
    { 0x4a3108f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124108fa, 59 },
    { 0xee5108fb, 23 },
    { 0, 0 },
    { 0x25a108fd, 23 },
    { 0, 0 },
    { 0xe1e108ff, 78 },
    { 0, 0 },
    { 0x53610901, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x56810904, 33 },
    { 0xbf610905, 98 },
    { 0, 0 },
    { 0xe0910907, 34 },
    { 0x14410908, 23 },
    { 0, 0 },
    { 0xb571090a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5491090f, 99 },
    { 0x1ca1090f, 95 },
    { 0xfbb10911, 87 },
    { 0x7b410912, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd10915, 33 },
    { 0xd0c10916, 90 },
    { 0x13e10917, 90 },
    { 0xe2910916, 33 },
    { 0xab410917, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa1091d, 52 },
    { 0x57a1091d, 55 },
    { 0, 0 },
    { 0x18e10920, 98 },
    { 0x48e10921, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc10924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a10928, 33 },
    { 0x1fd10928, 95 },
    { 0x8601092a, 43 },
    { 0xf2c1092b, 61 },
    { 0x8ee1092b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc911092f, 61 },
    { 0x4b710930, 33 },
    { 0xc6d10931, 46 },
    { 0xfe11092f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b10936, 75 },
    { 0, 0 },
    { 0xc2410938, 61 },
    { 0xc3210939, 98 },
    { 0, 0 },
    { 0xd021093b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x651093e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76910945, 33 },
    { 0x6d110946, 72 },
    { 0xff510946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a10956, 95 },
    { 0xc0810957, 55 },
    { 0, 0 },
    { 0x51210959, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091095c, 98 },
    { 0xa801095c, 52 },
    { 0x1a21095c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17710961, 98 },
    { 0, 0 },
    { 0xff710963, 43 },
    { 0x79610964, 95 },
    { 0x5d810965, 90 },
    { 0xa510966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b1096a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5231096e, 66 },
    { 0xb81096f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b10974, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a110979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba1097c, 85 },
    { 0x6c61097d, 76 },
    { 0xefe1097d, 90 },
    { 0x1ec1097f, 67 },
    { 0x6c11097c, 95 },
    { 0, 0 },
    { 0x97310982, 76 },
    { 0xa9b10983, 34 },
    { 0x7e910984, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd10987, 55 },
    { 0xcb910988, 61 },
    { 0x99110988, 76 },
    { 0x12d1098a, 85 },
    { 0x4671098b, 61 },
    { 0x6be1098b, 34 },
    { 0x20110987, 23 },
    { 0x52a10988, 87 },
    { 0x2e81098a, 23 },
    { 0, 0 },
    { 0x96f10991, 43 },
    { 0, 0 },
    { 0x92910993, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c10997, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a1099b, 23 },
    { 0, 0 },
    { 0x51a1099d, 90 },
    { 0x1d21099e, 33 },
    { 0xf371099f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3109a4, 95 },
    { 0xfc8109a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125109aa, 91 },
    { 0x221109aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3109bc, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7109c1, 40 },
    { 0, 0 },
    { 0xc48109c3, 100 },
    { 0x726109c3, 76 },
    { 0x209109c4, 90 },
    { 0xab1109c6, 40 },
    { 0, 0 },
    { 0xe29109c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x510109cb, 90 },
    { 0x381109cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e109d0, 90 },
    { 0x873109d1, 99 },
    { 0x388109d2, 84 },
    { 0xbe9109d3, 77 },
    { 0xa4e109d2, 93 },
    { 0, 0 },
    { 0xe77109d6, 40 },
    { 0x213109d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57109dc, 46 },
    { 0, 0 },
    { 0x87f109de, 40 },
    { 0x5ec109de, 87 },
    { 0, 0 },
    { 0x623109e1, 33 },
    { 0, 0 },
    { 0x665109e3, 78 },
    { 0, 0 },
    { 0xfa1109e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4109ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda109f6, 33 },
    { 0, 0 },
    { 0x57e109f8, 76 },
    { 0, 0 },
    { 0x400109fa, 85 },
    { 0xcb2109fb, 84 },
    { 0, 0 },
    { 0x79109fd, 61 },
    { 0, 0 },
    { 0xe6109ff, 90 },
    { 0, 0 },
    { 0x32010a01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf10a05, 95 },
    { 0xfe810a06, 90 },
    { 0, 0 },
    { 0x62a10a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa610a12, 23 },
    { 0x96910a13, 55 },
    { 0, 0 },
    { 0xecc10a15, 61 },
    { 0xe0c10a15, 95 },
    { 0x18d10a17, 95 },
    { 0xf410a18, 47 },
    { 0, 0 },
    { 0xa3810a1a, 55 },
    { 0x37010a1a, 98 },
    { 0x4010a1c, 23 },
    { 0, 0 },
    { 0x2af10a1e, 55 },
    { 0xc9110a1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a210a26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd710a2e, 88 },
    { 0xae110a2f, 66 },
    { 0xd0910a2f, 98 },
    { 0, 0 },
    { 0x74710a32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e10a40, 90 },
    { 0x18510a41, 46 },
    { 0x3f10a40, 98 },
    { 0x3c410a40, 43 },
    { 0xeba10a43, 90 },
    { 0x31210a45, 91 },
    { 0xd3010a45, 61 },
    { 0, 0 },
    { 0x1d010a48, 33 },
    { 0x23b10a48, 98 },
    { 0x53e10a4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23110a4d, 90 },
    { 0xc0310a4e, 61 },
    { 0x11610a4f, 43 },
    { 0x7d210a50, 72 },
    { 0xc8710a51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe210a57, 33 },
    { 0xdb110a58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed610a5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a310a61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e10a64, 95 },
    { 0x7ce10a65, 23 },
    { 0xeed10a66, 23 },
    { 0xa2f10a67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d10a6f, 61 },
    { 0, 0 },
    { 0x49710a71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd10a74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a10a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88310a7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6410a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410a85, 23 },
    { 0, 0 },
    { 0xcb610a87, 90 },
    { 0x4e910a88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de10a92, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f10a95, 17 },
    { 0xc3510a96, 98 },
    { 0x2c410a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e10a9b, 99 },
    { 0, 0 },
    { 0xf8510a9d, 95 },
    { 0x42510a9e, 90 },
    { 0, 0 },
    { 0x86410aa0, 46 },
    { 0xeac10aa0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5310aa5, 90 },
    { 0x7ff10aa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0110aaa, 95 },
    { 0x2c710aab, 91 },
    { 0xbd410aac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db10aaf, 100 },
    { 0, 0 },
    { 0x6e210ab1, 95 },
    { 0, 0 },
    { 0xbc10ab3, 98 },
    { 0, 0 },
    { 0x2f010ab5, 36 },
    { 0x9bc10ab5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83910abe, 43 },
    { 0, 0 },
    { 0x15f10ac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34510ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c610ac8, 55 },
    { 0, 0 },
    { 0xd2b10aca, 98 },
    { 0x51a10acb, 90 },
    { 0, 0 },
    { 0xb0a10acd, 55 },
    { 0, 0 },
    { 0x74b10acf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3110ad2, 93 },
    { 0, 0 },
    { 0x11810ad4, 33 },
    { 0xe3d10ad5, 99 },
    { 0x9d010ad6, 95 },
    { 0xa0810ad7, 98 },
    { 0xa3710ad8, 23 },
    { 0, 0 },
    { 0xed310ada, 78 },
    { 0x2c210adb, 69 },
    { 0x9f010ada, 90 },
    { 0, 0 },
    { 0xd6f10ade, 61 },
    { 0xb5810adf, 90 },
    { 0xb8710ade, 88 },
    { 0x45310ae0, 61 },
    { 0, 0 },
    { 0x40d10ae3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2810ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91510af2, 23 },
    { 0xbad10af3, 61 },
    { 0x3ff10af4, 33 },
    { 0xc6010af5, 87 },
    { 0x8e810af6, 33 },
    { 0x31910af7, 90 },
    { 0xd7110af7, 91 },
    { 0xd2210af9, 85 },
    { 0x7b710af2, 34 },
    { 0x8d910afb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c10b01, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e710b04, 61 },
    { 0xe410b05, 98 },
    { 0xbb410b04, 76 },
    { 0, 0 },
    { 0x41010b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e10b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb410b0f, 61 },
    { 0x13d10b10, 84 },
    { 0, 0 },
    { 0x18b10b12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8010b16, 43 },
    { 0, 0 },
    { 0xdd710b18, 100 },
    { 0x8a410b19, 93 },
    { 0x67b10b1a, 43 },
    { 0x56710b1b, 34 },
    { 0x47910b1b, 96 },
    { 0xe0c10b1d, 33 },
    { 0x11b10b1e, 61 },
    { 0x83210b1d, 88 },
    { 0x66110b1d, 95 },
    { 0x3d810b1b, 33 },
    { 0x9f110b1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34610b26, 33 },
    { 0, 0 },
    { 0x92d10b28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8710b2e, 93 },
    { 0xab710b2f, 90 },
    { 0, 0 },
    { 0xb4510b31, 93 },
    { 0x22710b32, 85 },
    { 0x79a10b33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5810b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96210b3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde110b42, 86 },
    { 0, 0 },
    { 0xd8e10b44, 61 },
    { 0x1b010b45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb10b48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb10b4b, 95 },
    { 0, 0 },
    { 0x21510b4d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d610b52, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd10b55, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb710b5b, 90 },
    { 0x5d210b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a10b60, 100 },
    { 0, 0 },
    { 0x16310b62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53010b6e, 33 },
    { 0x14310b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4910b73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3810b7c, 34 },
    { 0x7ef10b7d, 101 },
    { 0xaac10b7e, 61 },
    { 0x43310b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a510b84, 76 },
    { 0, 0 },
    { 0xf2c10b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7010b8a, 93 },
    { 0x4e110b8b, 76 },
    { 0xa7310b8a, 87 },
    { 0x95710b8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3210b91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69110b9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab710ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63110ba4, 43 },
    { 0x99a10ba4, 34 },
    { 0x97b10ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c910baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55610bae, 85 },
    { 0x34510bae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3410bb9, 76 },
    { 0xc4010bba, 85 },
    { 0xac810bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10810bc2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde510bc7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x90910bca, 23 },
    { 0x3b010bca, 46 },
    { 0x8c910bca, 99 },
    { 0x94b10bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1910bd1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f10bd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1910bdb, 95 },
    { 0, 0 },
    { 0x48610bdd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2310be4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a410be7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc310bef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67710bf5, 66 },
    { 0x210bf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32110bf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99710bfc, 43 },
    { 0, 0 },
    { 0x9ae10bfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce10c01, 98 },
    { 0, 0 },
    { 0xa3910c03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93210c08, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47810c0e, 34 },
    { 0xca610c0e, 61 },
    { 0x2210c10, 96 },
    { 0, 0 },
    { 0xe4610c12, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d010c15, 98 },
    { 0, 0 },
    { 0x99810c17, 66 },
    { 0xcf410c17, 34 },
    { 0xbd010c19, 17 },
    { 0x88310c1a, 76 },
    { 0, 0 },
    { 0x94210c1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75410c1f, 78 },
    { 0, 0 },
    { 0xe5310c21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a10c24, 98 },
    { 0x8df10c24, 90 },
    { 0x77110c24, 67 },
    { 0xd8210c27, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2010c2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5310c34, 76 },
    { 0x91710c34, 34 },
    { 0, 0 },
    { 0xf3210c37, 90 },
    { 0xe2910c37, 87 },
    { 0, 0 },
    { 0x88610c3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81110c40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc710c43, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d810c46, 95 },
    { 0x31b10c47, 77 },
    { 0xb9a10c46, 95 },
    { 0, 0 },
    { 0x3a510c4a, 33 },
    { 0, 0 },
    { 0x91810c4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70510c50, 33 },
    { 0x33210c51, 23 },
    { 0x45d10c52, 33 },
    { 0, 0 },
    { 0xb3a10c54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63010c57, 34 },
    { 0x39810c57, 93 },
    { 0x58310c58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23210c5c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d210c5f, 95 },
    { 0xdb10c60, 33 },
    { 0, 0 },
    { 0xfae10c62, 56 },
    { 0x40e10c63, 78 },
    { 0xdfe10c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e810c68, 98 },
    { 0, 0 },
    { 0xc8d10c6a, 83 },
    { 0x20d10c6a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x57210c6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8410c71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29010c75, 98 },
    { 0, 0 },
    { 0x80410c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf10c7c, 98 },
    { 0xc910c7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca10c81, 95 },
    { 0xde410c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45610c87, 99 },
    { 0xd1c10c88, 33 },
    { 0x9de10c87, 33 },
    { 0x2bd10c8a, 90 },
    { 0x73410c88, 23 },
    { 0xe4610c8c, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77610c93, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69110c99, 78 },
    { 0, 0 },
    { 0x8de10c9b, 61 },
    { 0xb4110c9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9610c9f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d110ca5, 90 },
    { 0x6b810ca6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec010ca9, 33 },
    { 0, 0 },
    { 0x4db10cab, 95 },
    { 0x4aa10cac, 69 },
    { 0x9aa10cad, 90 },
    { 0xf8010cae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1910cb2, 93 },
    { 0xd1f10cb3, 23 },
    { 0xc110cb3, 96 },
    { 0xf7510cb3, 33 },
    { 0x2da10cb6, 95 },
    { 0, 0 },
    { 0x9ce10cb8, 76 },
    { 0x9df10cb9, 43 },
    { 0x50b10cba, 91 },
    { 0x88510cb8, 95 },
    { 0xee710cbc, 33 },
    { 0x62710cbc, 84 },
    { 0x4bc10cb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d210cc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c10ccc, 61 },
    { 0, 0 },
    { 0x6110cce, 75 },
    { 0x58810ccf, 33 },
    { 0x2f010cd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8210cd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd10cdc, 93 },
    { 0, 0 },
    { 0xa6110cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf10ce2, 33 },
    { 0xcc910ce3, 23 },
    { 0xb1d10ce4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23710ce7, 100 },
    { 0, 0 },
    { 0xd110ce9, 61 },
    { 0x8c210cea, 98 },
    { 0x6fb10ceb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e610cee, 38 },
    { 0x96410cee, 78 },
    { 0xeea10cee, 55 },
    { 0x19410cef, 55 },
    { 0x50910cf1, 33 },
    { 0xec510cf2, 66 },
    { 0x6510cf4, 61 },
    { 0xf3010cf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb010cf9, 100 },
    { 0x1010cfa, 34 },
    { 0, 0 },
    { 0xb5010cfc, 95 },
    { 0xe7410cfd, 87 },
    { 0, 0 },
    { 0xffb10cff, 99 },
    { 0x92010d00, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a110d06, 42 },
    { 0xb010d06, 67 },
    { 0x23910d08, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f010d10, 67 },
    { 0xa0110d11, 43 },
    { 0x31c10d11, 61 },
    { 0xd8310d11, 90 },
    { 0x13d10d11, 95 },
    { 0xc1310d15, 76 },
    { 0x15f10d15, 33 },
    { 0, 0 },
    { 0x17710d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82110d1b, 23 },
    { 0, 0 },
    { 0xa5910d1d, 67 },
    { 0, 0 },
    { 0xbec10d1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6010d2d, 90 },
    { 0x66110d2d, 34 },
    { 0xefe10d2f, 76 },
    { 0, 0 },
    { 0x81f10d31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e210d34, 76 },
    { 0x14810d34, 66 },
    { 0x9eb10d34, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c10d3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32210d41, 61 },
    { 0xd6010d42, 34 },
    { 0x12c10d43, 33 },
    { 0x9aa10d44, 55 },
    { 0xac610d44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e10d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab10d4c, 90 },
    { 0xb2710d4d, 51 },
    { 0xcd510d4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2910d59, 33 },
    { 0, 0 },
    { 0xb6c10d5b, 98 },
    { 0xca010d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c710d63, 87 },
    { 0x12110d64, 98 },
    { 0xc0510d65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6410d6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d610d6d, 19 },
    { 0x46210d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b10d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7010d79, 61 },
    { 0x23110d7a, 100 },
    { 0, 0 },
    { 0xf3410d7c, 90 },
    { 0x66910d7d, 76 },
    { 0xfec10d7e, 61 },
    { 0xce910d7f, 76 },
    { 0xeba10d80, 98 },
    { 0xbf310d80, 88 },
    { 0x68210d82, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf10d86, 38 },
    { 0x8a910d86, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xba710d8a, 90 },
    { 0xccf10d8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d10d96, 61 },
    { 0, 0 },
    { 0x92f10d98, 75 },
    { 0x610d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49810da2, 99 },
    { 0, 0 },
    { 0x6b510da4, 43 },
    { 0, 0 },
    { 0xbd110da6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69210db0, 99 },
    { 0x2d210db0, 76 },
    { 0, 0 },
    { 0x5a110db3, 90 },
    { 0x5c310db4, 55 },
    { 0xce410db5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b10dbb, 81 },
    { 0, 0 },
    { 0x12410dbd, 55 },
    { 0xd5910dbd, 55 },
    { 0x5eb10dbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d10dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32010dd0, 90 },
    { 0, 0 },
    { 0x64510dd2, 17 },
    { 0xa1810dd3, 75 },
    { 0x2f210dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba10dd9, 95 },
    { 0, 0 },
    { 0xf6b10ddb, 84 },
    { 0x2d410ddc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37610de1, 75 },
    { 0, 0 },
    { 0x25c10de3, 90 },
    { 0xab310de4, 23 },
    { 0x75c10de4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba410dea, 90 },
    { 0, 0 },
    { 0xca810dec, 52 },
    { 0, 0 },
    { 0xc9310dee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91610df1, 95 },
    { 0, 0 },
    { 0x3d010df3, 98 },
    { 0x7fe10df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4310dfc, 95 },
    { 0x38610dfd, 76 },
    { 0x18f10dfc, 23 },
    { 0, 0 },
    { 0xaee10e00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c10e03, 77 },
    { 0, 0 },
    { 0x67210e05, 34 },
    { 0x6fa10e05, 90 },
    { 0x6a410e07, 23 },
    { 0, 0 },
    { 0xc6810e09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1810e0c, 95 },
    { 0xa3510e0d, 80 },
    { 0x6e610e0e, 90 },
    { 0xd6c10e0e, 90 },
    { 0x5e110e0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed910e14, 61 },
    { 0xd8210e15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb610e1a, 43 },
    { 0x93110e1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79810e20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e10e26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f10e2d, 98 },
    { 0xcd410e2d, 33 },
    { 0x16710e2d, 33 },
    { 0xec410e30, 87 },
    { 0, 0 },
    { 0x27210e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e10e39, 55 },
    { 0x39210e39, 95 },
    { 0xe710e3a, 81 },
    { 0x6ca10e39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d10e3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x99410e42, 61 },
    { 0xf5f10e43, 33 },
    { 0x82310e44, 90 },
    { 0x6010e44, 66 },
    { 0, 0 },
    { 0xce810e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77310e4b, 76 },
    { 0, 0 },
    { 0x24010e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9810e50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d10e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e810e56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce310e59, 90 },
    { 0x4cc10e5a, 95 },
    { 0, 0 },
    { 0x61310e5c, 61 },
    { 0xd5110e5c, 98 },
    { 0x2f210e5d, 33 },
    { 0xbfb10e5c, 61 },
    { 0, 0 },
    { 0xe5e10e61, 46 },
    { 0, 0 },
    { 0x63f10e63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f010e66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f610e6a, 23 },
    { 0xca910e6b, 46 },
    { 0xfd610e6b, 33 },
    { 0xfe310e6d, 90 },
    { 0x6f610e6c, 55 },
    { 0xd9210e6a, 34 },
    { 0x1cf10e70, 66 },
    { 0x80110e70, 98 },
    { 0x46610e72, 83 },
    { 0x85510e6b, 33 },
    { 0x62510e72, 52 },
    { 0, 0 },
    { 0x3c610e76, 100 },
    { 0, 0 },
    { 0xdda10e78, 93 },
    { 0, 0 },
    { 0x3ef10e7a, 23 },
    { 0, 0 },
    { 0xb9d10e7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32210e81, 61 },
    { 0xe310e81, 66 },
    { 0x9d310e81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d910e87, 87 },
    { 0, 0 },
    { 0xfdf10e89, 95 },
    { 0xb2510e8a, 61 },
    { 0x9e410e8b, 95 },
    { 0x20510e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f10e8f, 61 },
    { 0, 0 },
    { 0x1a510e91, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84210e96, 98 },
    { 0x50f10e97, 61 },
    { 0x76d10e96, 90 },
    { 0xe4610e98, 98 },
    { 0xfde10e99, 23 },
    { 0, 0 },
    { 0x2cd10e9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71510e9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb110ea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde510eac, 90 },
    { 0, 0 },
    { 0x43110eae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7110eb1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7510eb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef910eb9, 33 },
    { 0xfc10eba, 91 },
    { 0xf6910eba, 4 },
    { 0xfd810ebc, 34 },
    { 0xcd510ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51410ec3, 61 },
    { 0, 0 },
    { 0xb3610ec5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f10ec9, 17 },
    { 0x6c310eca, 36 },
    { 0, 0 },
    { 0xa5910ecc, 61 },
    { 0x30310ecd, 87 },
    { 0, 0 },
    { 0x39810ecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe10ed2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae10ed5, 23 },
    { 0x3eb10ed5, 33 },
    { 0x4a110ed7, 97 },
    { 0x17910ed7, 98 },
    { 0x41410ed9, 52 },
    { 0x87d10ed8, 61 },
    { 0x8bb10edb, 95 },
    { 0xdb410edc, 98 },
    { 0x9de10edd, 95 },
    { 0, 0 },
    { 0xbc710edf, 99 },
    { 0xf6210ee0, 61 },
    { 0x23710ee0, 34 },
    { 0xf3410ee2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa10ee8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94610eeb, 33 },
    { 0x8eb10eec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab10eef, 99 },
    { 0x6af10ef0, 98 },
    { 0x50c10eef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20410ef4, 94 },
    { 0x85710ef5, 98 },
    { 0, 0 },
    { 0x6ba10ef7, 61 },
    { 0xfd410ef8, 33 },
    { 0x24610ef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf10f01, 90 },
    { 0x79310f02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b10f05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb710f09, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e010f0e, 66 },
    { 0, 0 },
    { 0x2aa10f10, 40 },
    { 0xc3210f11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9710f14, 55 },
    { 0x5cc10f15, 95 },
    { 0x9f910f16, 95 },
    { 0xf4910f16, 98 },
    { 0x6d310f18, 84 },
    { 0xfa910f19, 22 },
    { 0, 0 },
    { 0xdb910f1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef610f1e, 61 },
    { 0x3b410f1f, 90 },
    { 0x5ea10f1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45910f23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c10f26, 80 },
    { 0x7b710f26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96210f2d, 33 },
    { 0x72110f2e, 33 },
    { 0x3a710f2e, 90 },
    { 0xe5d10f30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f10f33, 46 },
    { 0x41310f33, 98 },
    { 0x57110f35, 90 },
    { 0x5a710f36, 33 },
    { 0, 0 },
    { 0x93810f38, 90 },
    { 0x64110f39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79510f40, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea410f44, 66 },
    { 0, 0 },
    { 0x44c10f46, 99 },
    { 0xc9f10f46, 76 },
    { 0xa7d10f47, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e610f4b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d10f4e, 61 },
    { 0x14f10f4f, 46 },
    { 0xce810f50, 46 },
    { 0, 0 },
    { 0xff810f52, 23 },
    { 0x6c610f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a10f57, 90 },
    { 0xd2510f57, 34 },
    { 0xb1810f59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f10f5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40010f61, 61 },
    { 0xd2d10f62, 61 },
    { 0x83610f63, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a10f66, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e10f69, 23 },
    { 0, 0 },
    { 0x2bf10f6b, 98 },
    { 0x83910f6c, 17 },
    { 0, 0 },
    { 0xb5710f6e, 61 },
    { 0, 0 },
    { 0x78d10f70, 55 },
    { 0xb1e10f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4210f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d710f78, 20 },
    { 0xa8510f78, 100 },
    { 0x5b210f7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a810f80, 55 },
    { 0xda510f80, 82 },
    { 0x98d10f82, 33 },
    { 0xdc810f83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90210f87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e10f91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92810f9a, 76 },
    { 0, 0 },
    { 0x3e010f9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd10fa0, 61 },
    { 0, 0 },
    { 0x67810fa2, 43 },
    { 0, 0 },
    { 0xd410fa4, 93 },
    { 0xfd910fa5, 33 },
    { 0, 0 },
    { 0x2a710fa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0410fad, 91 },
    { 0x7f810fad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21610fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb10fb8, 90 },
    { 0xad110fb9, 43 },
    { 0x6fb10fb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d910fbd, 76 },
    { 0xea710fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6310fc5, 91 },
    { 0xa5810fc6, 33 },
    { 0x9710fc7, 87 },
    { 0xa3310fc5, 77 },
    { 0x91010fc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f910fcf, 33 },
    { 0, 0 },
    { 0xf7c10fd1, 33 },
    { 0xac10fd2, 23 },
    { 0x9e710fd3, 53 },
    { 0xd0510fd3, 93 },
    { 0xda910fd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad710fda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6210fde, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb10fe1, 95 },
    { 0x47810fe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c910fe6, 33 },
    { 0xdf010fe6, 43 },
    { 0x1c910fe7, 90 },
    { 0, 0 },
    { 0xa0d10fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b10fed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb10ff2, 34 },
    { 0x33710ff3, 33 },
    { 0xb8b10ff4, 61 },
    { 0xcde10ff5, 46 },
    { 0x5b10ff6, 95 },
    { 0x6d010ff6, 34 },
    { 0xc4b10ff5, 84 },
    { 0x3ea10ff9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e10ffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45810fff, 34 },
    { 0x40611000, 48 },
    { 0x69811001, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4611007, 93 },
    { 0x9f611008, 95 },
    { 0, 0 },
    { 0x78b1100a, 95 },
    { 0x5d01100b, 77 },
    { 0x5421100c, 95 },
    { 0x47b1100d, 23 },
    { 0, 0 },
    { 0x56c1100f, 12 },
    { 0, 0 },
    { 0x90011011, 100 },
    { 0, 0 },
    { 0xc3f11013, 23 },
    { 0x86711014, 90 },
    { 0, 0 },
    { 0x63d11016, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251101a, 36 },
    { 0, 0 },
    { 0xaa91101c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1681101f, 34 },
    { 0x72811020, 46 },
    { 0x69d11021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4211102a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa11031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5411041, 55 },
    { 0xaa611042, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d11045, 43 },
    { 0xfa111046, 46 },
    { 0, 0 },
    { 0x60911048, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf561104c, 33 },
    { 0xb2a1104c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15411051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d11055, 76 },
    { 0, 0 },
    { 0x4d911057, 51 },
    { 0x53e11058, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc711105e, 98 },
    { 0xb3d1105e, 23 },
    { 0x1641105f, 33 },
    { 0x8e11061, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a611066, 90 },
    { 0x39d11066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad11070, 61 },
    { 0, 0 },
    { 0x96e11072, 23 },
    { 0x6b311073, 23 },
    { 0x11911074, 78 },
    { 0, 0 },
    { 0x90011076, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e11079, 95 },
    { 0, 0 },
    { 0xed21107b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12211082, 91 },
    { 0xab11083, 61 },
    { 0, 0 },
    { 0x9a611085, 55 },
    { 0x89e11085, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3761108b, 78 },
    { 0x5fe1108b, 55 },
    { 0x90f1108d, 91 },
    { 0x1c61108e, 95 },
    { 0x6191108f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50111093, 76 },
    { 0, 0 },
    { 0x10911095, 95 },
    { 0x85e11096, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553110a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f110ab, 76 },
    { 0, 0 },
    { 0x323110ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x337110b0, 93 },
    { 0, 0 },
    { 0xc39110b2, 23 },
    { 0xcff110b3, 90 },
    { 0x129110b4, 91 },
    { 0, 0 },
    { 0xcdf110b6, 90 },
    { 0xde0110b6, 76 },
    { 0, 0 },
    { 0xfc4110b9, 67 },
    { 0xfcf110ba, 23 },
    { 0x2e3110b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9110be, 55 },
    { 0x249110bf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693110c5, 34 },
    { 0x53b110c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2110ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2110d1, 90 },
    { 0x780110d1, 90 },
    { 0xd27110d3, 84 },
    { 0x69110d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402110d9, 76 },
    { 0, 0 },
    { 0xd80110db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x264110de, 55 },
    { 0x150110df, 90 },
    { 0x2c8110de, 34 },
    { 0x62110e1, 90 },
    { 0x810110e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d110e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24110eb, 40 },
    { 0xe99110eb, 85 },
    { 0, 0 },
    { 0x26f110ee, 33 },
    { 0xe15110ee, 100 },
    { 0, 0 },
    { 0x6fb110f1, 90 },
    { 0, 0 },
    { 0x1ae110f3, 84 },
    { 0xeda110f3, 90 },
    { 0x9aa110f5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d110f8, 75 },
    { 0, 0 },
    { 0x9e8110fa, 93 },
    { 0xa7f110fb, 95 },
    { 0x4a2110fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc11100, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11103, 67 },
    { 0, 0 },
    { 0x85111105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c211108, 98 },
    { 0, 0 },
    { 0x2771110a, 61 },
    { 0x8e91110b, 98 },
    { 0x6f41110a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd1110f, 98 },
    { 0x5461110f, 98 },
    { 0x17a11111, 34 },
    { 0x9dd11112, 93 },
    { 0x5f311113, 23 },
    { 0, 0 },
    { 0x67911115, 61 },
    { 0x22711116, 93 },
    { 0, 0 },
    { 0x29011118, 23 },
    { 0xe8411119, 23 },
    { 0, 0 },
    { 0x2c61111b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e11121, 34 },
    { 0x56711122, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4121112a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1112d, 33 },
    { 0xdfd1112d, 95 },
    { 0xacf1112d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e11133, 69 },
    { 0x5c511134, 23 },
    { 0xddb11135, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad1113b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9561113e, 23 },
    { 0x6c11113f, 78 },
    { 0x1861113f, 23 },
    { 0x44a11141, 61 },
    { 0, 0 },
    { 0x32c11143, 58 },
    { 0, 0 },
    { 0x44f11145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad1114b, 90 },
    { 0, 0 },
    { 0x6681114d, 91 },
    { 0x3c61114e, 99 },
    { 0, 0 },
    { 0x28011150, 23 },
    { 0x81a11150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb11154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd811115b, 84 },
    { 0xadc1115b, 55 },
    { 0, 0 },
    { 0x7a91115e, 33 },
    { 0, 0 },
    { 0x8511160, 76 },
    { 0xfe811161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab11165, 90 },
    { 0, 0 },
    { 0x48111167, 61 },
    { 0x32511168, 61 },
    { 0x7d911168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd01116f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53611176, 55 },
    { 0xb5c11176, 69 },
    { 0, 0 },
    { 0x7b711179, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b21117f, 88 },
    { 0x2a211180, 87 },
    { 0xb0111181, 90 },
    { 0, 0 },
    { 0xcd711183, 65 },
    { 0, 0 },
    { 0x6111185, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01118a, 61 },
    { 0x2951118b, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f211191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5111194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5721119c, 61 },
    { 0xfad1119c, 90 },
    { 0x1091119e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd111a3, 33 },
    { 0xbad111a3, 98 },
    { 0xf50111a5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a111aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4111b3, 61 },
    { 0xde4111b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407111ba, 90 },
    { 0xf2b111ba, 55 },
    { 0x8f111bc, 95 },
    { 0xd4a111bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5111c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f111c9, 90 },
    { 0, 0 },
    { 0x205111cb, 38 },
    { 0xe38111cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f111d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f111d6, 76 },
    { 0x111d7, 55 },
    { 0, 0 },
    { 0x8cc111d9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb111de, 90 },
    { 0, 0 },
    { 0x5bd111e0, 23 },
    { 0xb40111e1, 53 },
    { 0xd57111e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c111e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2111e8, 34 },
    { 0, 0 },
    { 0x4b5111ea, 23 },
    { 0, 0 },
    { 0x3c0111ec, 93 },
    { 0x3e0111ed, 68 },
    { 0x3b5111ee, 85 },
    { 0x8f7111ee, 33 },
    { 0x5d6111ee, 55 },
    { 0xa22111f1, 68 },
    { 0xca5111f2, 52 },
    { 0, 0 },
    { 0x8b5111f4, 40 },
    { 0, 0 },
    { 0x156111f6, 90 },
    { 0, 0 },
    { 0xc24111f8, 90 },
    { 0xc63111f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35111fe, 98 },
    { 0, 0 },
    { 0xe9211200, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d11205, 99 },
    { 0xffd11205, 90 },
    { 0x8f11206, 76 },
    { 0xef711206, 33 },
    { 0xae511209, 61 },
    { 0xc4f11209, 33 },
    { 0xd3d1120b, 75 },
    { 0x51e11206, 34 },
    { 0x2ae1120d, 34 },
    { 0x6cb1120d, 100 },
    { 0xc001120f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f11213, 90 },
    { 0, 0 },
    { 0xcc811215, 61 },
    { 0xfe211216, 61 },
    { 0xe7d11215, 23 },
    { 0, 0 },
    { 0x86e11219, 61 },
    { 0, 0 },
    { 0x44c1121b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75811220, 55 },
    { 0x58311221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e911224, 33 },
    { 0x4c211224, 33 },
    { 0x45711225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1122a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c111230, 17 },
    { 0xaaf11231, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b11238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1531123c, 23 },
    { 0xe9a1123d, 90 },
    { 0x83c1123e, 81 },
    { 0xfd51123e, 90 },
    { 0x87e11240, 95 },
    { 0xa141123e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68011244, 55 },
    { 0x31911244, 43 },
    { 0x34b11246, 76 },
    { 0x8f111247, 98 },
    { 0, 0 },
    { 0xdd711249, 98 },
    { 0xc6511249, 43 },
    { 0xac51124b, 46 },
    { 0xcc911249, 66 },
    { 0xa1f1124d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae11250, 95 },
    { 0xf5e11250, 90 },
    { 0xbcc11252, 90 },
    { 0xec211252, 95 },
    { 0, 0 },
    { 0xca11255, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9511267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc31126e, 23 },
    { 0x411126f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e11272, 99 },
    { 0xc7311273, 43 },
    { 0xe6811274, 84 },
    { 0x7d811275, 90 },
    { 0x92111276, 34 },
    { 0, 0 },
    { 0x11111278, 43 },
    { 0x7e111278, 23 },
    { 0xc2811278, 34 },
    { 0, 0 },
    { 0x88e1127c, 23 },
    { 0, 0 },
    { 0xc511127e, 95 },
    { 0x5d91127f, 87 },
    { 0xeb411280, 23 },
    { 0xf6311280, 81 },
    { 0, 0 },
    { 0x55611283, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b411286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f71128a, 61 },
    { 0, 0 },
    { 0xb781128c, 65 },
    { 0, 0 },
    { 0x9c11128e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc211293, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30011298, 46 },
    { 0, 0 },
    { 0xc891129a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1129e, 90 },
    { 0x53f1129f, 98 },
    { 0xbf81129f, 85 },
    { 0x8ea112a1, 23 },
    { 0xb8f112a0, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2112aa, 90 },
    { 0xac0112aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a112b2, 90 },
    { 0x1e7112b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32112b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733112be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3112c3, 84 },
    { 0x222112c4, 34 },
    { 0x8e8112c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9112c9, 95 },
    { 0x786112ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d112cf, 33 },
    { 0, 0 },
    { 0x895112d1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892112d5, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359112d9, 99 },
    { 0x52b112da, 23 },
    { 0x2b7112da, 98 },
    { 0x7d8112d9, 65 },
    { 0, 0 },
    { 0x54e112de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c112e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9112e9, 98 },
    { 0x434112ea, 61 },
    { 0x983112eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef112ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635112f3, 81 },
    { 0xee1112f3, 78 },
    { 0, 0 },
    { 0xe64112f6, 61 },
    { 0xd04112f7, 33 },
    { 0x4ff112f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd112fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac811301, 90 },
    { 0, 0 },
    { 0xebb11303, 85 },
    { 0xd0811303, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52111308, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c11313, 85 },
    { 0x36911313, 95 },
    { 0, 0 },
    { 0xf7711316, 87 },
    { 0xfc311317, 90 },
    { 0, 0 },
    { 0x57511319, 52 },
    { 0xb41131a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f41131d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8111323, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0111329, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1132d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d511334, 76 },
    { 0, 0 },
    { 0xba11336, 95 },
    { 0xff611336, 90 },
    { 0x6fb11338, 98 },
    { 0xfa811337, 90 },
    { 0xb1d1133a, 78 },
    { 0x65f1133a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a1133e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xef811341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45611344, 23 },
    { 0, 0 },
    { 0xe9311346, 61 },
    { 0x3db11347, 90 },
    { 0, 0 },
    { 0xd1d11349, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c1134c, 76 },
    { 0, 0 },
    { 0x7491134e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6611352, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab11356, 61 },
    { 0x2e911356, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb1135a, 76 },
    { 0xac71135a, 51 },
    { 0x4241135c, 78 },
    { 0, 0 },
    { 0x4a11135e, 98 },
    { 0xd2c1135f, 95 },
    { 0, 0 },
    { 0x26511361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81511369, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7621136c, 90 },
    { 0x9031136c, 37 },
    { 0x7e31136e, 34 },
    { 0x4ec1136f, 56 },
    { 0, 0 },
    { 0x39311371, 95 },
    { 0x93511372, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99511376, 98 },
    { 0xfe011376, 98 },
    { 0, 0 },
    { 0xec111379, 90 },
    { 0x86e1137a, 88 },
    { 0x9591137a, 66 },
    { 0x28211379, 90 },
    { 0x25b1137d, 33 },
    { 0x8f81137e, 95 },
    { 0x2c01137f, 33 },
    { 0x711380, 46 },
    { 0x84a11380, 84 },
    { 0x4b11137a, 33 },
    { 0x63a1137e, 76 },
    { 0xafb11384, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be11387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2411138d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xede11390, 61 },
    { 0, 0 },
    { 0xa3f11392, 34 },
    { 0xa1c11393, 55 },
    { 0, 0 },
    { 0x57e11395, 33 },
    { 0x3bd11395, 99 },
    { 0x24311397, 61 },
    { 0x27611396, 95 },
    { 0, 0 },
    { 0xf3d1139a, 100 },
    { 0x6af1139b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276113a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0113a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838113b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39113b9, 61 },
    { 0x7f0113ba, 95 },
    { 0xea0113bb, 95 },
    { 0xf7d113bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f113c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1113c6, 43 },
    { 0x95f113c7, 85 },
    { 0x703113c8, 90 },
    { 0, 0 },
    { 0x1ae113ca, 33 },
    { 0, 0 },
    { 0x567113cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a113d0, 90 },
    { 0, 0 },
    { 0x437113d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b113d7, 65 },
    { 0, 0 },
    { 0xab7113d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676113de, 95 },
    { 0, 0 },
    { 0x872113e0, 95 },
    { 0, 0 },
    { 0x7b3113e2, 34 },
    { 0xcdb113e3, 33 },
    { 0, 0 },
    { 0x189113e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c113e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04113ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6113f3, 93 },
    { 0x2d5113f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7113fa, 55 },
    { 0, 0 },
    { 0xb89113fc, 23 },
    { 0x9e0113fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a11400, 61 },
    { 0xf7311401, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc51140b, 61 },
    { 0x1091140b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f51140f, 40 },
    { 0, 0 },
    { 0x40b11411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38711414, 78 },
    { 0, 0 },
    { 0x19d11416, 90 },
    { 0xd0811417, 98 },
    { 0xd911418, 36 },
    { 0xeeb11419, 98 },
    { 0x13c1141a, 46 },
    { 0xadc1141b, 23 },
    { 0x62e1141c, 91 },
    { 0x15a1141b, 96 },
    { 0x90b1141c, 33 },
    { 0x1ef1141a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc11427, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c1142b, 23 },
    { 0x6541142c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e61142f, 84 },
    { 0x81611430, 33 },
    { 0x58111430, 33 },
    { 0xc0c11430, 76 },
    { 0x50811433, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811437, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e1143a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a11447, 78 },
    { 0xc4011448, 98 },
    { 0, 0 },
    { 0x9361144a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b61144f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef11454, 33 },
    { 0xe8e11455, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4901145b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a111460, 17 },
    { 0xdf011461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80111465, 61 },
    { 0xf1e11466, 85 },
    { 0xf3011467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11146c, 23 },
    { 0x25d1146c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67111470, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c811473, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3811476, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa221147b, 33 },
    { 0, 0 },
    { 0xbb11147d, 76 },
    { 0x7c41147d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37811487, 23 },
    { 0xc2411488, 23 },
    { 0, 0 },
    { 0x2f71148a, 90 },
    { 0x4e11148b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12711490, 61 },
    { 0x64011491, 91 },
    { 0xe2411491, 95 },
    { 0x76f11491, 76 },
    { 0xd9911493, 90 },
    { 0x96911495, 76 },
    { 0xdb611491, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca11499, 55 },
    { 0xdb1149a, 98 },
    { 0x8ce1149b, 66 },
    { 0xf271149c, 98 },
    { 0x2bf1149d, 84 },
    { 0xd821149d, 90 },
    { 0x3951149e, 98 },
    { 0, 0 },
    { 0x100114a1, 90 },
    { 0x748114a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50114ad, 85 },
    { 0x40f114ae, 43 },
    { 0xb7b114af, 76 },
    { 0x58c114ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22114b5, 46 },
    { 0, 0 },
    { 0xdc7114b7, 81 },
    { 0xc8d114b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40114c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95114c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459114c8, 61 },
    { 0xd63114c9, 93 },
    { 0, 0 },
    { 0xa84114cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2114cf, 33 },
    { 0, 0 },
    { 0x6a2114d1, 93 },
    { 0, 0 },
    { 0xb44114d3, 23 },
    { 0xff6114d4, 61 },
    { 0x7f4114d5, 94 },
    { 0xb53114d6, 76 },
    { 0xbd2114d5, 90 },
    { 0x8f2114d6, 61 },
    { 0x7f1114d9, 55 },
    { 0, 0 },
    { 0x3f0114db, 44 },
    { 0xf77114dc, 98 },
    { 0x64e114dd, 91 },
    { 0, 0 },
    { 0xd3114df, 55 },
    { 0x46114df, 67 },
    { 0x4d8114e0, 95 },
    { 0x2a5114e2, 61 },
    { 0, 0 },
    { 0xeb7114e4, 23 },
    { 0x865114e4, 93 },
    { 0, 0 },
    { 0xa30114e7, 78 },
    { 0, 0 },
    { 0x356114e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade114ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf114f1, 85 },
    { 0x1ef114f2, 95 },
    { 0, 0 },
    { 0xe3c114f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a114f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584114fe, 69 },
    { 0, 0 },
    { 0xb3011500, 98 },
    { 0x37311500, 43 },
    { 0, 0 },
    { 0xcdc11503, 90 },
    { 0, 0 },
    { 0x5ea11505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641150a, 36 },
    { 0xc451150b, 33 },
    { 0, 0 },
    { 0xd631150d, 55 },
    { 0xb361150e, 90 },
    { 0x5c91150e, 68 },
    { 0x79311510, 76 },
    { 0x25911511, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99311516, 87 },
    { 0xb9a11517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d81151b, 33 },
    { 0xef91151c, 33 },
    { 0xe181151d, 23 },
    { 0x6441151e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b11521, 61 },
    { 0xe6411522, 77 },
    { 0x74711522, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c61152b, 33 },
    { 0, 0 },
    { 0xd831152d, 61 },
    { 0, 0 },
    { 0x5281152f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c511538, 33 },
    { 0xedb11539, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6221153c, 91 },
    { 0x1211153d, 90 },
    { 0, 0 },
    { 0xdaa1153f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9811542, 95 },
    { 0, 0 },
    { 0x24611544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc11549, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6741154c, 55 },
    { 0xad1154d, 61 },
    { 0xde01154e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb1155a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3361155d, 99 },
    { 0x9401155e, 96 },
    { 0xcfa1155d, 96 },
    { 0x97a11560, 99 },
    { 0xc0711560, 95 },
    { 0, 0 },
    { 0x95311563, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d11568, 42 },
    { 0x40f11569, 33 },
    { 0xdd1156a, 61 },
    { 0x1481156b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b1156e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad711574, 90 },
    { 0x23911575, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd71157c, 76 },
    { 0xb61157d, 76 },
    { 0xa151157e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53111586, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2401158a, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4951158f, 95 },
    { 0x46e11590, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091159a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd115a1, 95 },
    { 0x8b2115a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342115ad, 34 },
    { 0, 0 },
    { 0x271115af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e115b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5115b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2115bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0115c8, 98 },
    { 0xb4a115c8, 98 },
    { 0, 0 },
    { 0xf0115cb, 55 },
    { 0xafe115cc, 33 },
    { 0, 0 },
    { 0x526115ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4115d5, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd115da, 98 },
    { 0xd30115db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547115e3, 95 },
    { 0x5ea115e4, 52 },
    { 0x11d115e5, 90 },
    { 0xd22115e6, 34 },
    { 0x422115e3, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96115ed, 61 },
    { 0xabb115ee, 87 },
    { 0x1ea115ee, 39 },
    { 0x588115ef, 55 },
    { 0xdf4115f1, 55 },
    { 0xd83115f2, 61 },
    { 0xfe115f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101115f7, 33 },
    { 0x6e2115f8, 90 },
    { 0xd3a115f9, 61 },
    { 0xb81115fa, 87 },
    { 0x6d2115fa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86411605, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e11609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb801160f, 23 },
    { 0, 0 },
    { 0xbfe11611, 81 },
    { 0xd3f11612, 23 },
    { 0xea211611, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a11619, 95 },
    { 0xc801161a, 99 },
    { 0x36e11619, 76 },
    { 0x7071161c, 77 },
    { 0x561161d, 92 },
    { 0x291161c, 90 },
    { 0, 0 },
    { 0x90711620, 87 },
    { 0xb2511621, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a11624, 77 },
    { 0xbf011624, 98 },
    { 0x1b411624, 90 },
    { 0xdb11627, 66 },
    { 0xb1111627, 68 },
    { 0x8fd11629, 76 },
    { 0x9c611624, 36 },
    { 0, 0 },
    { 0xbf91162c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe411631, 43 },
    { 0x3bc11632, 98 },
    { 0x38711633, 76 },
    { 0x7a11633, 100 },
    { 0x10a11633, 95 },
    { 0x80911636, 66 },
    { 0x81211635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb11163a, 95 },
    { 0xe521163b, 91 },
    { 0x6521163c, 61 },
    { 0x4061163b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c11644, 67 },
    { 0, 0 },
    { 0xfd211646, 33 },
    { 0x58211647, 23 },
    { 0xefc11648, 34 },
    { 0x9b211649, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf91164f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15311653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc11657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d1165a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f1165d, 61 },
    { 0x22e1165d, 93 },
    { 0, 0 },
    { 0xeec11660, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81166a, 95 },
    { 0x6c71166b, 34 },
    { 0x4d71166c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36111674, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61167e, 76 },
    { 0xc0e1167e, 75 },
    { 0, 0 },
    { 0x16211681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f11686, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e1168c, 33 },
    { 0x6c31168d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a311693, 33 },
    { 0xa5811693, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11511698, 61 },
    { 0xb911698, 65 },
    { 0, 0 },
    { 0x231169b, 61 },
    { 0x96c1169c, 90 },
    { 0, 0 },
    { 0x2111169e, 55 },
    { 0, 0 },
    { 0x27d116a0, 33 },
    { 0, 0 },
    { 0x89a116a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca116a7, 90 },
    { 0, 0 },
    { 0x31c116a9, 68 },
    { 0xc8116a9, 61 },
    { 0, 0 },
    { 0x472116ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481116b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38116b5, 98 },
    { 0xba7116b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8116b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8116be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa116ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd116cd, 23 },
    { 0xd08116ce, 66 },
    { 0x56c116ce, 95 },
    { 0x3fe116d0, 33 },
    { 0xaca116d1, 23 },
    { 0x13f116d2, 84 },
    { 0x3a7116d2, 33 },
    { 0, 0 },
    { 0x5cb116d5, 23 },
    { 0x4a4116d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e116db, 99 },
    { 0, 0 },
    { 0xff2116dd, 95 },
    { 0x615116de, 97 },
    { 0, 0 },
    { 0xd04116e0, 23 },
    { 0x939116e1, 90 },
    { 0xf23116e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7116e6, 90 },
    { 0, 0 },
    { 0xca4116e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952116ec, 69 },
    { 0, 0 },
    { 0x457116ee, 33 },
    { 0x9a9116ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5116f3, 95 },
    { 0xb5116f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400116f8, 78 },
    { 0x844116f9, 55 },
    { 0x4a116fa, 46 },
    { 0x3a3116fb, 23 },
    { 0xe1a116fc, 43 },
    { 0xe8d116fd, 90 },
    { 0x5da116fe, 98 },
    { 0x72f116fd, 52 },
    { 0x15611700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86311706, 78 },
    { 0, 0 },
    { 0xe0911708, 33 },
    { 0x6fb11709, 67 },
    { 0xfbd1170a, 61 },
    { 0x6e211709, 95 },
    { 0x54c1170c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b11713, 76 },
    { 0xca711714, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3311171d, 2 },
    { 0x3be1171d, 33 },
    { 0x6d31171f, 76 },
    { 0x27b1171d, 90 },
    { 0x21711720, 33 },
    { 0, 0 },
    { 0x4ca11723, 34 },
    { 0xdbc11724, 95 },
    { 0xacf11725, 88 },
    { 0xfdf11723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c1172a, 61 },
    { 0, 0 },
    { 0x50e1172c, 90 },
    { 0, 0 },
    { 0xbcf1172e, 53 },
    { 0x57c1172e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6711732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a11736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf1173e, 77 },
    { 0xb3f1173f, 84 },
    { 0xd1b11740, 90 },
    { 0xe3e11741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc11747, 99 },
    { 0xe4511747, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e1174e, 66 },
    { 0x2991174f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde111754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a11759, 95 },
    { 0x76311759, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e11175d, 55 },
    { 0xae61175d, 75 },
    { 0, 0 },
    { 0x88b11760, 61 },
    { 0x70611761, 23 },
    { 0, 0 },
    { 0xc2011763, 98 },
    { 0x3e611764, 33 },
    { 0, 0 },
    { 0x29c11766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7961176c, 85 },
    { 0, 0 },
    { 0xb4c1176e, 43 },
    { 0x1671176e, 58 },
    { 0xdce11770, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57811776, 95 },
    { 0, 0 },
    { 0x8111778, 78 },
    { 0x95411779, 90 },
    { 0, 0 },
    { 0x4291177b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd01177f, 90 },
    { 0, 0 },
    { 0xcc711781, 97 },
    { 0, 0 },
    { 0xc6d11783, 67 },
    { 0xa8911784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e11789, 100 },
    { 0, 0 },
    { 0xb001178b, 61 },
    { 0x2901178c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e1178f, 90 },
    { 0x65e1178f, 55 },
    { 0, 0 },
    { 0x43411792, 98 },
    { 0x43b11792, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f611798, 61 },
    { 0xfc211799, 87 },
    { 0x3711179a, 23 },
    { 0x4b51179a, 23 },
    { 0xc591179b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b117a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53117ad, 95 },
    { 0, 0 },
    { 0x199117af, 95 },
    { 0x1ff117b0, 98 },
    { 0xe76117b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x590117b4, 34 },
    { 0, 0 },
    { 0xe0a117b6, 61 },
    { 0x913117b7, 95 },
    { 0x39117b8, 33 },
    { 0x582117b6, 33 },
    { 0x34d117b9, 31 },
    { 0x40d117b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f117be, 98 },
    { 0, 0 },
    { 0x9ee117c0, 91 },
    { 0xdb7117c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d117c8, 98 },
    { 0x58e117c8, 90 },
    { 0, 0 },
    { 0x79b117cb, 38 },
    { 0x763117cc, 52 },
    { 0xed1117cc, 52 },
    { 0xaeb117ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc117d2, 84 },
    { 0x713117d3, 91 },
    { 0, 0 },
    { 0x305117d5, 33 },
    { 0x9bb117d6, 100 },
    { 0xe06117d6, 22 },
    { 0xd8b117d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63117db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0117e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b117e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x505117e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6117ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5117f2, 33 },
    { 0, 0 },
    { 0x375117f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f117fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a11800, 61 },
    { 0, 0 },
    { 0x74911802, 23 },
    { 0x92711803, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf11806, 69 },
    { 0xd0911807, 23 },
    { 0x59e11808, 61 },
    { 0x22e11806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11180f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c611817, 90 },
    { 0, 0 },
    { 0x6d311819, 66 },
    { 0x5be1181a, 76 },
    { 0x3451181b, 68 },
    { 0xcef1181c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6881181f, 68 },
    { 0xae611820, 68 },
    { 0, 0 },
    { 0xa611822, 69 },
    { 0xd9611823, 68 },
    { 0x18011823, 55 },
    { 0xd0611824, 66 },
    { 0xb5011826, 95 },
    { 0x6d11825, 23 },
    { 0x73c11824, 34 },
    { 0x4be11829, 98 },
    { 0xd8611826, 98 },
    { 0xb2911826, 61 },
    { 0xfb01182c, 90 },
    { 0, 0 },
    { 0xf381182e, 61 },
    { 0x7731182f, 77 },
    { 0xf1d1182f, 43 },
    { 0x25a11830, 87 },
    { 0xe3311832, 34 },
    { 0x57911833, 33 },
    { 0xe6e11833, 95 },
    { 0xe2a11830, 98 },
    { 0, 0 },
    { 0x25c11837, 100 },
    { 0xa1011837, 93 },
    { 0x41611839, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8611840, 93 },
    { 0xd4211841, 34 },
    { 0x49e11842, 33 },
    { 0, 0 },
    { 0x7c411844, 33 },
    { 0xcf511845, 90 },
    { 0x1f711846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921184a, 98 },
    { 0x4101184b, 33 },
    { 0x4291184a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d1184f, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe611856, 33 },
    { 0xbef11857, 90 },
    { 0xad311858, 95 },
    { 0x60111858, 34 },
    { 0, 0 },
    { 0x2541185b, 23 },
    { 0, 0 },
    { 0x8a71185d, 85 },
    { 0x3db1185e, 98 },
    { 0xa601185f, 33 },
    { 0, 0 },
    { 0xd8011861, 76 },
    { 0xc2111862, 66 },
    { 0x28411863, 90 },
    { 0xb8411862, 99 },
    { 0x2c511865, 33 },
    { 0xa5a11866, 99 },
    { 0xff11864, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b1186b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda1186f, 33 },
    { 0, 0 },
    { 0x5a711871, 38 },
    { 0xeac11872, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b11875, 90 },
    { 0xa2a11876, 61 },
    { 0xecb11876, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe021187c, 61 },
    { 0xfe51187d, 90 },
    { 0x2f51187d, 76 },
    { 0xfa51187d, 98 },
    { 0, 0 },
    { 0xad011881, 43 },
    { 0xaba11882, 90 },
    { 0xb2611882, 58 },
    { 0xedb11884, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf911887, 68 },
    { 0xdd011888, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a11894, 17 },
    { 0x24e11894, 52 },
    { 0x2b111894, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab1189a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391189d, 34 },
    { 0, 0 },
    { 0x2be1189f, 61 },
    { 0x27c118a0, 23 },
    { 0x97b118a0, 98 },
    { 0, 0 },
    { 0xf2a118a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec118a7, 85 },
    { 0, 0 },
    { 0xbb1118a9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d118ac, 90 },
    { 0, 0 },
    { 0x3f2118ae, 61 },
    { 0x10e118ae, 84 },
    { 0x308118b0, 33 },
    { 0xc48118b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6118b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f118bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e118c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c118c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d118c9, 88 },
    { 0x2ed118c9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e118cd, 98 },
    { 0x32f118ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533118da, 78 },
    { 0x697118da, 98 },
    { 0x159118dc, 93 },
    { 0x796118dc, 33 },
    { 0xd69118de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279118e5, 55 },
    { 0xbf6118e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c118f9, 38 },
    { 0x889118f9, 52 },
    { 0xd28118f9, 61 },
    { 0xb86118fa, 33 },
    { 0, 0 },
    { 0xdad118fe, 34 },
    { 0, 0 },
    { 0x1cd11900, 46 },
    { 0, 0 },
    { 0x9b211902, 46 },
    { 0, 0 },
    { 0xecb11904, 95 },
    { 0xc6811904, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e11908, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b1190d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7151191c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58011920, 98 },
    { 0, 0 },
    { 0x4f811922, 90 },
    { 0x1d611923, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d11926, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2911929, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b11192c, 33 },
    { 0xe251192d, 66 },
    { 0x6b71192e, 55 },
    { 0xdff1192e, 17 },
    { 0x6161192f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee611934, 43 },
    { 0xee111935, 55 },
    { 0x56411936, 93 },
    { 0, 0 },
    { 0x60311938, 61 },
    { 0, 0 },
    { 0x7df1193a, 34 },
    { 0, 0 },
    { 0x2491193c, 33 },
    { 0x4d61193d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f11940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5311943, 61 },
    { 0xc3b11944, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c1194a, 95 },
    { 0xdb71194a, 78 },
    { 0xac61194a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5581194f, 90 },
    { 0xac511950, 95 },
    { 0x77211950, 95 },
    { 0x7d611950, 52 },
    { 0x71411953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1671195e, 95 },
    { 0, 0 },
    { 0xa4d11960, 33 },
    { 0x6e811961, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f11969, 61 },
    { 0, 0 },
    { 0xd211196b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc1196e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f211971, 95 },
    { 0x71d11972, 52 },
    { 0xcc811973, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe911976, 93 },
    { 0x1c311977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf521197b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4011984, 56 },
    { 0x14111984, 61 },
    { 0xb9811984, 95 },
    { 0xfe211984, 23 },
    { 0, 0 },
    { 0xe2111989, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb831198c, 100 },
    { 0xd8f1198d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b911991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33311994, 43 },
    { 0xb4311995, 95 },
    { 0x70811995, 33 },
    { 0xaa111997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3651199b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6021199f, 93 },
    { 0, 0 },
    { 0xc86119a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8119a6, 23 },
    { 0xbfd119a6, 46 },
    { 0xf66119a7, 33 },
    { 0xb7a119a9, 61 },
    { 0x4ec119aa, 90 },
    { 0xdd119ab, 90 },
    { 0x355119ac, 34 },
    { 0xb41119ad, 33 },
    { 0xb33119aa, 76 },
    { 0xde119a9, 34 },
    { 0x6c7119b0, 100 },
    { 0x90b119b1, 61 },
    { 0, 0 },
    { 0xc74119b3, 87 },
    { 0x5e7119b4, 90 },
    { 0, 0 },
    { 0xd6b119b6, 80 },
    { 0xa45119b6, 34 },
    { 0x50f119b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12119bb, 33 },
    { 0, 0 },
    { 0xd1c119bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51119c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56119cb, 90 },
    { 0x468119cc, 90 },
    { 0, 0 },
    { 0x923119ce, 33 },
    { 0x78f119cf, 68 },
    { 0x9fc119ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x340119d3, 90 },
    { 0, 0 },
    { 0x757119d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2119d8, 88 },
    { 0xc4e119d9, 76 },
    { 0x9f5119da, 61 },
    { 0xdca119db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c119de, 40 },
    { 0, 0 },
    { 0x5fc119e0, 61 },
    { 0, 0 },
    { 0x43d119e2, 95 },
    { 0x162119e3, 90 },
    { 0, 0 },
    { 0x8d5119e5, 61 },
    { 0, 0 },
    { 0xc0c119e7, 48 },
    { 0x6fc119e8, 96 },
    { 0x80c119e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b119f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63119f4, 87 },
    { 0x94e119f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644119fa, 33 },
    { 0x357119fa, 67 },
    { 0x7a119fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0111a13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a11a16, 43 },
    { 0x2d511a17, 61 },
    { 0xc4b11a18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf11a1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e11a21, 95 },
    { 0xada11a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9611a26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94211a2d, 84 },
    { 0, 0 },
    { 0x2bc11a2f, 61 },
    { 0xcb411a30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6111a33, 43 },
    { 0x94e11a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae11a37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a111a3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c11a49, 34 },
    { 0x3f711a4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28611a55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89911a62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29811a66, 76 },
    { 0xd811a67, 23 },
    { 0x26811a67, 78 },
    { 0x9c411a69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c11a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5911a71, 95 },
    { 0x3d811a72, 23 },
    { 0x93911a73, 98 },
    { 0, 0 },
    { 0xbb411a75, 55 },
    { 0x9cb11a75, 34 },
    { 0, 0 },
    { 0x2da11a78, 34 },
    { 0x71e11a79, 52 },
    { 0xfe211a7a, 81 },
    { 0x74111a79, 98 },
    { 0x16111a7c, 61 },
    { 0xac211a79, 95 },
    { 0x42c11a7e, 95 },
    { 0xaa511a7a, 55 },
    { 0x3b311a7f, 61 },
    { 0xcf311a7d, 95 },
    { 0xee511a7e, 68 },
    { 0xa4011a83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89311a8c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac11a8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85411a95, 90 },
    { 0, 0 },
    { 0xa7011a97, 43 },
    { 0xf6711a97, 99 },
    { 0xe6e11a99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e611aa5, 33 },
    { 0xa1a11aa6, 52 },
    { 0xfd611aa7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87911aac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe11ab1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18711ab8, 33 },
    { 0, 0 },
    { 0xaea11aba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c11abe, 100 },
    { 0xdda11abe, 61 },
    { 0x9f311ac0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb11ac4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76211ac7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89411acb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca11ad6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce811ade, 55 },
    { 0xc011adf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e11ae6, 85 },
    { 0x75c11ae7, 100 },
    { 0x83b11ae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde711aeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4111aef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d011af2, 95 },
    { 0, 0 },
    { 0x79a11af4, 33 },
    { 0x39611af5, 93 },
    { 0x4ce11af4, 34 },
    { 0xc0e11af7, 69 },
    { 0xcae11af8, 49 },
    { 0xec511af5, 98 },
    { 0x80e11af4, 23 },
    { 0, 0 },
    { 0x77411afc, 81 },
    { 0x33a11afc, 75 },
    { 0x77a11afd, 23 },
    { 0x56911aff, 55 },
    { 0, 0 },
    { 0x63611b01, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11711b07, 95 },
    { 0, 0 },
    { 0x5bb11b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9211b0e, 98 },
    { 0x2dc11b0e, 33 },
    { 0, 0 },
    { 0xc3211b11, 61 },
    { 0x39911b11, 61 },
    { 0x8b411b13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d11b16, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b011b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a11b20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c11b24, 40 },
    { 0, 0 },
    { 0xfdd11b26, 98 },
    { 0x30211b27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc11b2a, 43 },
    { 0, 0 },
    { 0x79911b2c, 100 },
    { 0, 0 },
    { 0xcb311b2e, 90 },
    { 0xae711b2e, 78 },
    { 0xb2911b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef11b33, 90 },
    { 0x49611b34, 90 },
    { 0x3ce11b35, 98 },
    { 0x87d11b34, 23 },
    { 0xf1a11b37, 52 },
    { 0x7d611b36, 85 },
    { 0xd5411b36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a211b3d, 88 },
    { 0x3411b3e, 93 },
    { 0x95211b3e, 33 },
    { 0x8ee11b3d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65311b44, 100 },
    { 0, 0 },
    { 0x16311b46, 98 },
    { 0, 0 },
    { 0xe4c11b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f511b52, 66 },
    { 0xcfb11b52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a11b58, 34 },
    { 0, 0 },
    { 0x8e111b5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e11b65, 33 },
    { 0, 0 },
    { 0x6c111b67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8311b6b, 55 },
    { 0x1fb11b6c, 33 },
    { 0x9db11b6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50911b71, 75 },
    { 0, 0 },
    { 0x2d411b73, 33 },
    { 0, 0 },
    { 0xcd711b75, 34 },
    { 0xff11b76, 90 },
    { 0x6de11b76, 34 },
    { 0x49011b78, 61 },
    { 0xc9f11b77, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89711b7d, 46 },
    { 0x99511b7e, 99 },
    { 0, 0 },
    { 0xff311b80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf611b83, 65 },
    { 0xdc811b84, 93 },
    { 0x2a611b85, 61 },
    { 0x5c611b86, 58 },
    { 0xcc611b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee411b8a, 62 },
    { 0, 0 },
    { 0xc0d11b8c, 45 },
    { 0, 0 },
    { 0x4f511b8e, 95 },
    { 0xf4611b8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d011b92, 98 },
    { 0xcf511b92, 91 },
    { 0xd8311b94, 77 },
    { 0xa8511b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a11b9e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c911ba1, 46 },
    { 0, 0 },
    { 0xd711ba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66711ba8, 90 },
    { 0x5211ba8, 33 },
    { 0, 0 },
    { 0xc7111bab, 78 },
    { 0x38511bab, 95 },
    { 0, 0 },
    { 0xae211bae, 76 },
    { 0x17e11baf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5311bb3, 93 },
    { 0x15611bb4, 67 },
    { 0xdd611bb5, 93 },
    { 0xbd011bb6, 66 },
    { 0x8a111bb7, 90 },
    { 0x20e11bb4, 33 },
    { 0x68a11bb9, 99 },
    { 0xfbc11bba, 95 },
    { 0xbf511bbb, 95 },
    { 0x7111bbc, 33 },
    { 0x5e311bbc, 95 },
    { 0x40f11bbe, 90 },
    { 0xf9511bbf, 55 },
    { 0xab711bbb, 33 },
    { 0xb1711bc1, 23 },
    { 0x65111bc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe311bca, 40 },
    { 0xc3811bcb, 23 },
    { 0x9db11bcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d611bcf, 34 },
    { 0x4311bd0, 99 },
    { 0x41611bd0, 76 },
    { 0xbef11bd2, 33 },
    { 0x9e411bd2, 78 },
    { 0xc8311bd4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30611bd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19911bdd, 90 },
    { 0x6a411bde, 55 },
    { 0x9ef11bdf, 33 },
    { 0x7b811bde, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac11be5, 23 },
    { 0x36811be6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c311beb, 87 },
    { 0x82e11bec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b11bef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e11bf2, 98 },
    { 0x12711bf3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9911bfa, 40 },
    { 0xdf011bfb, 87 },
    { 0, 0 },
    { 0xf6611bfd, 34 },
    { 0xb3a11bfe, 33 },
    { 0xebc11bff, 75 },
    { 0xfc411bfd, 98 },
    { 0xc5811bff, 100 },
    { 0xdb711bfe, 90 },
    { 0, 0 },
    { 0xe6111c04, 76 },
    { 0x3311c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1411c08, 98 },
    { 0x81b11c08, 90 },
    { 0x82b11c0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59111c0d, 90 },
    { 0xa1311c0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6811c11, 95 },
    { 0xfc111c12, 74 },
    { 0xb7811c13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54111c16, 99 },
    { 0, 0 },
    { 0x4f611c18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce411c1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f11c20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0511c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e11c32, 33 },
    { 0xa5c11c32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e11c38, 61 },
    { 0xc7611c38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c11c3d, 43 },
    { 0xdb111c3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff911c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7011c47, 65 },
    { 0x5d411c48, 23 },
    { 0x45411c49, 90 },
    { 0xfb911c4a, 23 },
    { 0, 0 },
    { 0xb7511c4c, 98 },
    { 0x8df11c4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe11c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d11c59, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc711c5c, 90 },
    { 0xd8511c5d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e11c60, 76 },
    { 0x80e11c61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5811c67, 95 },
    { 0, 0 },
    { 0x41a11c69, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a611c6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f211c74, 55 },
    { 0xf4211c75, 33 },
    { 0xe411c76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2211c79, 76 },
    { 0x29711c7a, 91 },
    { 0xb2611c7b, 76 },
    { 0x3cb11c7b, 33 },
    { 0xc6511c7a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60811c8c, 43 },
    { 0x82911c8d, 90 },
    { 0x2f911c8e, 61 },
    { 0x7b11c8f, 100 },
    { 0x2ec11c90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a811c93, 61 },
    { 0, 0 },
    { 0x911c95, 33 },
    { 0xd7511c96, 61 },
    { 0x38e11c96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93811c9b, 99 },
    { 0xa7111c9c, 61 },
    { 0x4911c9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe11ca6, 87 },
    { 0x51911ca7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd11caa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e111cae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63711cc2, 90 },
    { 0xa0c11cc3, 78 },
    { 0x92711cc2, 98 },
    { 0xbcb11cc3, 33 },
    { 0, 0 },
    { 0x4c011cc7, 76 },
    { 0xb9211cc8, 76 },
    { 0xed111cc7, 55 },
    { 0xfc211cc9, 33 },
    { 0xc1011cc7, 78 },
    { 0xd2811ccc, 90 },
    { 0xee711ccd, 76 },
    { 0xf0411ccb, 55 },
    { 0xbdb11cc7, 33 },
    { 0x17e11ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38411cd7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7511cda, 98 },
    { 0xa1611cda, 52 },
    { 0, 0 },
    { 0x30511cdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7911ce0, 33 },
    { 0, 0 },
    { 0x18811ce2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0411ce9, 61 },
    { 0x49f11cea, 76 },
    { 0x4f11ce9, 61 },
    { 0xd0b11cec, 33 },
    { 0xb0311ced, 61 },
    { 0xd8611cec, 90 },
    { 0xd2e11cec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a611cf2, 90 },
    { 0x68c11cf3, 90 },
    { 0x34711cf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8511cf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f11cfa, 23 },
    { 0x22411cfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99011d03, 61 },
    { 0, 0 },
    { 0x5db11d05, 42 },
    { 0xa8711d06, 52 },
    { 0, 0 },
    { 0xbd611d08, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1511d12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23811d15, 33 },
    { 0, 0 },
    { 0xce511d17, 33 },
    { 0, 0 },
    { 0xe0a11d19, 23 },
    { 0x5d011d1a, 87 },
    { 0xae411d1b, 46 },
    { 0x34211d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53611d25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e11d2c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe11d34, 55 },
    { 0xa4911d35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb11d3b, 95 },
    { 0x2e11d3c, 55 },
    { 0x69811d3c, 90 },
    { 0, 0 },
    { 0xd8411d3f, 98 },
    { 0, 0 },
    { 0xcf211d41, 98 },
    { 0x14811d42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28711d4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea11d52, 41 },
    { 0xf0411d53, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f11d57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6811d5a, 77 },
    { 0x7a011d5b, 23 },
    { 0, 0 },
    { 0xd5c11d5d, 23 },
    { 0x6c311d5e, 90 },
    { 0x47011d5f, 84 },
    { 0xdb711d60, 76 },
    { 0xcdc11d61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22911d64, 43 },
    { 0, 0 },
    { 0xe1911d66, 78 },
    { 0x9d911d66, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd11d6b, 91 },
    { 0x43511d6c, 46 },
    { 0x77211d6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b11d70, 95 },
    { 0x79a11d71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e11d74, 98 },
    { 0x98011d75, 33 },
    { 0xe7811d76, 84 },
    { 0, 0 },
    { 0xf2011d78, 61 },
    { 0, 0 },
    { 0x33f11d7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa611d7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf11d88, 23 },
    { 0x50111d89, 33 },
    { 0, 0 },
    { 0xba611d8b, 23 },
    { 0x80811d8c, 61 },
    { 0x2311d8d, 90 },
    { 0x50411d8e, 55 },
    { 0x91111d8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a11d92, 99 },
    { 0xc2711d93, 76 },
    { 0x7dd11d92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7611d9a, 90 },
    { 0x4a011d9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb711da2, 95 },
    { 0xc3011da2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9011da7, 66 },
    { 0, 0 },
    { 0x42b11da9, 90 },
    { 0x91411da9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e211dae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d11db2, 76 },
    { 0x42e11db3, 100 },
    { 0, 0 },
    { 0xef811db5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33411db8, 33 },
    { 0xe1f11db9, 98 },
    { 0x44c11dba, 34 },
    { 0x9d911dbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f11dbe, 43 },
    { 0x95511dbf, 33 },
    { 0xb7111dc0, 66 },
    { 0, 0 },
    { 0xffb11dc2, 95 },
    { 0, 0 },
    { 0xe7311dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e11dc7, 84 },
    { 0x5bb11dc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b11dce, 98 },
    { 0x34d11dcf, 52 },
    { 0xad811dcf, 95 },
    { 0xfc711dce, 87 },
    { 0xe5a11dcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3411dd5, 90 },
    { 0xa6c11dd6, 90 },
    { 0xa5111dd5, 95 },
    { 0xee011dd8, 90 },
    { 0x93311dd5, 95 },
    { 0, 0 },
    { 0x71f11ddb, 75 },
    { 0, 0 },
    { 0x2a11ddd, 90 },
    { 0xbcd11dde, 23 },
    { 0, 0 },
    { 0x36711de0, 33 },
    { 0x3dc11de1, 95 },
    { 0xed211de2, 95 },
    { 0x2a011de3, 93 },
    { 0, 0 },
    { 0xe9311de5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe911de9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b11dec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x34811def, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e11dfa, 90 },
    { 0x93b11dfb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73411e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22811e09, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb911e0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78211e11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d311e16, 88 },
    { 0x73b11e17, 35 },
    { 0xf8111e17, 23 },
    { 0x52611e18, 33 },
    { 0x24011e19, 61 },
    { 0x6211e17, 22 },
    { 0xbc11e18, 95 },
    { 0xca111e1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3611e21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86811e26, 98 },
    { 0xa4911e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18511e2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f11e30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16011e33, 55 },
    { 0xf6411e34, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c711e38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87611e3b, 93 },
    { 0, 0 },
    { 0xb9111e3d, 33 },
    { 0xb7711e3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e511e42, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd11e4a, 56 },
    { 0xacc11e4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d11e4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc311e52, 46 },
    { 0, 0 },
    { 0xd5a11e54, 93 },
    { 0xca411e55, 61 },
    { 0, 0 },
    { 0xc4f11e57, 19 },
    { 0, 0 },
    { 0xd6611e59, 90 },
    { 0, 0 },
    { 0x7ff11e5b, 95 },
    { 0, 0 },
    { 0x94a11e5d, 33 },
    { 0xc6e11e5e, 66 },
    { 0, 0 },
    { 0x1f111e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a11e67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a611e6f, 33 },
    { 0, 0 },
    { 0x93811e71, 33 },
    { 0, 0 },
    { 0xce711e73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9211e77, 76 },
    { 0xacb11e77, 98 },
    { 0x9fd11e78, 46 },
    { 0x4a411e7a, 33 },
    { 0x82011e7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16511e7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5211e82, 55 },
    { 0, 0 },
    { 0x94e11e84, 46 },
    { 0x5b311e85, 100 },
    { 0x2711e86, 98 },
    { 0x19611e87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3411e8b, 90 },
    { 0x4ee11e8b, 43 },
    { 0x71a11e8d, 23 },
    { 0x1ea11e8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c11e91, 61 },
    { 0xb1111e91, 61 },
    { 0xaa611e93, 90 },
    { 0x57911e94, 43 },
    { 0xc1111e94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19811e9b, 90 },
    { 0, 0 },
    { 0xb4611e9d, 85 },
    { 0xb2511e9e, 76 },
    { 0xfea11e9f, 81 },
    { 0x2a11ea0, 34 },
    { 0x3bd11ea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44411ea5, 34 },
    { 0x1ef11ea5, 55 },
    { 0x41111ea6, 76 },
    { 0xc411ea6, 90 },
    { 0x6f211ea7, 66 },
    { 0xa5811ea9, 88 },
    { 0x8d411eab, 43 },
    { 0xd9011eab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac611eaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6111eb3, 76 },
    { 0, 0 },
    { 0xd6b11eb5, 33 },
    { 0x4d111eb5, 90 },
    { 0x15011eb7, 90 },
    { 0xf0811eb6, 98 },
    { 0, 0 },
    { 0x44111eba, 61 },
    { 0xf1311eba, 76 },
    { 0x1de11ebc, 90 },
    { 0x18011ebd, 34 },
    { 0x89411ebe, 84 },
    { 0xad311ebe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd611ec7, 74 },
    { 0, 0 },
    { 0x92a11ec9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd911ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce11ed8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b11edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3011ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c711ee2, 33 },
    { 0x3e611ee3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e611ee9, 75 },
    { 0x4c411eea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c11eee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63211ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83011ef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc11efb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f11efe, 55 },
    { 0x93c11efe, 95 },
    { 0x46611f00, 76 },
    { 0, 0 },
    { 0x84d11f02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff11f05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54611f0b, 34 },
    { 0x64211f0c, 98 },
    { 0x9ab11f0d, 61 },
    { 0, 0 },
    { 0xb3611f0f, 95 },
    { 0, 0 },
    { 0xe4711f11, 53 },
    { 0x10b11f12, 43 },
    { 0x77711f13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13311f16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e511f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c711f26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63211f2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7111f31, 90 },
    { 0, 0 },
    { 0xdaa11f33, 61 },
    { 0x82311f34, 77 },
    { 0x3c11f33, 99 },
    { 0, 0 },
    { 0xbf711f37, 93 },
    { 0x6011f37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a211f3f, 25 },
    { 0, 0 },
    { 0x7d711f41, 90 },
    { 0x66811f41, 66 },
    { 0, 0 },
    { 0x51711f44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf111f47, 90 },
    { 0x60e11f48, 61 },
    { 0x73c11f49, 100 },
    { 0xe0f11f49, 46 },
    { 0, 0 },
    { 0x11911f4c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a11f4f, 90 },
    { 0, 0 },
    { 0x35911f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27211f54, 34 },
    { 0x25211f55, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x80311f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94811f5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a11f5f, 90 },
    { 0x64b11f60, 95 },
    { 0xd1911f60, 90 },
    { 0xf6b11f62, 33 },
    { 0x4ed11f62, 90 },
    { 0xf7f11f64, 23 },
    { 0x45011f65, 76 },
    { 0x22711f65, 59 },
    { 0xc7511f65, 95 },
    { 0x7cc11f67, 55 },
    { 0xfb111f69, 76 },
    { 0x5e611f69, 61 },
    { 0x19711f6b, 61 },
    { 0xa8811f5f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78311f71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7211f75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f11f79, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d11f7f, 90 },
    { 0, 0 },
    { 0x9f411f81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a211f87, 33 },
    { 0, 0 },
    { 0x27f11f89, 64 },
    { 0x46f11f8a, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3311f8e, 76 },
    { 0xebc11f8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb11f94, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xff811f97, 90 },
    { 0xad011f98, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48811f9e, 61 },
    { 0xece11f9f, 100 },
    { 0x6f011f9f, 46 },
    { 0x78811fa1, 33 },
    { 0xbee11fa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b711fa7, 98 },
    { 0x50e11fa8, 33 },
    { 0, 0 },
    { 0xce511faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab11fb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e411fb5, 76 },
    { 0, 0 },
    { 0x20211fb7, 93 },
    { 0x8c911fb8, 33 },
    { 0x38011fb9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88011fbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6211fc6, 78 },
    { 0, 0 },
    { 0x89a11fc8, 33 },
    { 0, 0 },
    { 0x94111fca, 23 },
    { 0x4bf11fcb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d411fd0, 61 },
    { 0x30611fd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd11fd4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35211fdd, 33 },
    { 0, 0 },
    { 0xda211fdf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5611fe2, 61 },
    { 0x52a11fe3, 33 },
    { 0x84811fe4, 93 },
    { 0x8311fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b211fe9, 23 },
    { 0x2e411fea, 64 },
    { 0x68e11fe9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52411fef, 95 },
    { 0, 0 },
    { 0xbe211ff1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7311ff6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c12002, 98 },
    { 0xa7b12002, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81512007, 40 },
    { 0x42912008, 99 },
    { 0, 0 },
    { 0xb061200a, 99 },
    { 0, 0 },
    { 0x85a1200c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd171200f, 23 },
    { 0x6b312010, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d12013, 76 },
    { 0xe812014, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba12017, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6931201a, 34 },
    { 0x53f1201a, 84 },
    { 0x1f61201c, 8 },
    { 0, 0 },
    { 0x1de1201e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2712026, 88 },
    { 0xc9412027, 23 },
    { 0x96912028, 46 },
    { 0x5d112029, 43 },
    { 0xcc61202a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad1202e, 93 },
    { 0x1f1202f, 23 },
    { 0x4b512030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa712034, 90 },
    { 0, 0 },
    { 0xa2112036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc512041, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b812046, 33 },
    { 0x7be12047, 95 },
    { 0x4cd12046, 84 },
    { 0x6b112048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21204f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc612052, 90 },
    { 0, 0 },
    { 0xfaf12054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d01205d, 61 },
    { 0xd801205e, 23 },
    { 0, 0 },
    { 0x8aa12060, 55 },
    { 0xe5412061, 33 },
    { 0x26012060, 33 },
    { 0, 0 },
    { 0xaa512064, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8051206a, 100 },
    { 0x94d1206b, 98 },
    { 0x3311206a, 61 },
    { 0, 0 },
    { 0xafa1206e, 46 },
    { 0x4af1206e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c212073, 33 },
    { 0xced12073, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e12079, 43 },
    { 0x6fa1207a, 34 },
    { 0xbbc1207b, 90 },
    { 0xd451207b, 98 },
    { 0xeab1207c, 67 },
    { 0x2161207b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b712081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e912084, 90 },
    { 0xfc12084, 46 },
    { 0x3d712085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d21208b, 52 },
    { 0x5981208c, 61 },
    { 0x5281208d, 98 },
    { 0x20d1208d, 24 },
    { 0x1051208d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a12093, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a12098, 90 },
    { 0xf3012098, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5371209d, 23 },
    { 0x2591209e, 93 },
    { 0, 0 },
    { 0x428120a0, 52 },
    { 0, 0 },
    { 0x60d120a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e120a7, 61 },
    { 0, 0 },
    { 0xfc3120a9, 61 },
    { 0, 0 },
    { 0x411120ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185120b1, 78 },
    { 0xa62120b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839120b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983120bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63120c1, 23 },
    { 0, 0 },
    { 0x7b0120c3, 90 },
    { 0x6f1120c4, 85 },
    { 0xf3f120c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d120c8, 88 },
    { 0xf49120c9, 87 },
    { 0x237120c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d120d5, 33 },
    { 0x83c120d5, 33 },
    { 0xa1d120d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94120da, 33 },
    { 0, 0 },
    { 0x60a120dc, 91 },
    { 0x557120dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290120e1, 46 },
    { 0x2e2120e2, 76 },
    { 0, 0 },
    { 0x1d1120e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x301120e7, 75 },
    { 0, 0 },
    { 0x3cc120e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9120ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf120ef, 23 },
    { 0x39c120f0, 19 },
    { 0x645120f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf120f8, 34 },
    { 0x710120f9, 100 },
    { 0x32d120fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4120ff, 53 },
    { 0xc8512100, 33 },
    { 0xb54120ff, 87 },
    { 0, 0 },
    { 0x26a12103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f61210b, 34 },
    { 0x9dd1210c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df12110, 23 },
    { 0, 0 },
    { 0x3f512112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b12117, 23 },
    { 0xef512118, 61 },
    { 0x7712118, 34 },
    { 0x8bb1211a, 33 },
    { 0x38612119, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8851211e, 55 },
    { 0xb601211f, 93 },
    { 0, 0 },
    { 0x71d12121, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc1212d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6812132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7612137, 34 },
    { 0, 0 },
    { 0xf9f12139, 34 },
    { 0, 0 },
    { 0xd401213b, 95 },
    { 0xe8a1213b, 90 },
    { 0x9fa1213b, 90 },
    { 0xb4a1213b, 84 },
    { 0, 0 },
    { 0x7db12140, 61 },
    { 0xc3712141, 52 },
    { 0x46312141, 46 },
    { 0x7d812143, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e12146, 85 },
    { 0x5c912146, 93 },
    { 0x20d12147, 101 },
    { 0, 0 },
    { 0xf7c1214a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f1214e, 93 },
    { 0xc281214f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c12152, 95 },
    { 0xca712153, 93 },
    { 0x4db12153, 87 },
    { 0, 0 },
    { 0x20a12156, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e1215c, 76 },
    { 0x3291215d, 23 },
    { 0, 0 },
    { 0xe121215f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7212164, 90 },
    { 0, 0 },
    { 0xc9112166, 33 },
    { 0, 0 },
    { 0xb2512168, 100 },
    { 0x30012168, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b1216d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33612173, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88512177, 77 },
    { 0, 0 },
    { 0x65412179, 95 },
    { 0, 0 },
    { 0x2331217b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5921217e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe012181, 95 },
    { 0, 0 },
    { 0xfc12183, 84 },
    { 0x81012184, 85 },
    { 0, 0 },
    { 0x73e12186, 24 },
    { 0x20612187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1218c, 76 },
    { 0x3671218d, 90 },
    { 0x95f1218e, 33 },
    { 0x23b1218f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1712195, 43 },
    { 0, 0 },
    { 0x87c12197, 43 },
    { 0x5d012197, 23 },
    { 0x9f212197, 55 },
    { 0x9111219a, 55 },
    { 0x9811219b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20121a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991121a6, 34 },
    { 0x98a121a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4121ab, 90 },
    { 0, 0 },
    { 0x95f121ad, 23 },
    { 0x9cb121ad, 90 },
    { 0x2ab121ad, 75 },
    { 0, 0 },
    { 0xdfa121b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd121b7, 34 },
    { 0x72f121b8, 95 },
    { 0x138121b8, 46 },
    { 0x6c5121ba, 28 },
    { 0xe55121b9, 93 },
    { 0xe0a121bc, 34 },
    { 0x641121bd, 23 },
    { 0x249121b9, 95 },
    { 0x533121bb, 90 },
    { 0x268121c0, 23 },
    { 0x17121c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x601121c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d121c7, 99 },
    { 0x392121c7, 76 },
    { 0, 0 },
    { 0x201121ca, 95 },
    { 0, 0 },
    { 0x149121cc, 34 },
    { 0x51f121cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e121d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3121dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b121e3, 90 },
    { 0, 0 },
    { 0xa92121e5, 61 },
    { 0, 0 },
    { 0xaa7121e7, 2 },
    { 0x886121e7, 55 },
    { 0xf1c121e9, 91 },
    { 0xf27121ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824121ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c121f2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1121f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f121fc, 93 },
    { 0, 0 },
    { 0xc72121fe, 61 },
    { 0xf9c121ff, 90 },
    { 0x397121ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae612205, 61 },
    { 0x26212206, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b1220a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4801220e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56312212, 90 },
    { 0x16c12213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87112219, 90 },
    { 0xaee1221a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6912220, 33 },
    { 0xe4412220, 76 },
    { 0xf1b12221, 87 },
    { 0xf9112223, 84 },
    { 0, 0 },
    { 0xab312225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b12228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83612231, 55 },
    { 0xbf412231, 93 },
    { 0x58812233, 23 },
    { 0xb6612234, 90 },
    { 0x27312235, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe971223a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5061223d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23912240, 52 },
    { 0x8c312241, 75 },
    { 0xd4912242, 98 },
    { 0xff412243, 34 },
    { 0x4b412244, 61 },
    { 0x5f612244, 76 },
    { 0xf3f12246, 90 },
    { 0, 0 },
    { 0x7b612248, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c1224b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75212250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa012257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e012262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b12265, 66 },
    { 0xa6012265, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea1226c, 90 },
    { 0xdc11226d, 90 },
    { 0x7271226c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e12273, 33 },
    { 0, 0 },
    { 0xfd912275, 90 },
    { 0, 0 },
    { 0x62412277, 52 },
    { 0, 0 },
    { 0xa1e12279, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd91227d, 61 },
    { 0, 0 },
    { 0x2541227f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b812283, 98 },
    { 0, 0 },
    { 0x78d12285, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af1228b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af12294, 66 },
    { 0x7d412295, 94 },
    { 0xbb212296, 33 },
    { 0x9b412297, 64 },
    { 0x10f12298, 61 },
    { 0x45712297, 83 },
    { 0x4821229a, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f1229e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8122a2, 90 },
    { 0xe30122a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a122a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d122a9, 23 },
    { 0x4e8122a9, 90 },
    { 0, 0 },
    { 0x550122ac, 64 },
    { 0, 0 },
    { 0x2f9122ae, 81 },
    { 0x2fa122af, 55 },
    { 0x5e7122b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891122b5, 23 },
    { 0x822122b6, 61 },
    { 0x43b122b7, 23 },
    { 0xc85122b8, 98 },
    { 0, 0 },
    { 0xc3e122ba, 95 },
    { 0xcf1122bb, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f122bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5122c3, 98 },
    { 0xd87122c4, 78 },
    { 0x444122c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f122c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66122cc, 95 },
    { 0x688122cd, 87 },
    { 0x330122ce, 65 },
    { 0xb68122ce, 93 },
    { 0x4d9122d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c122d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f122e2, 95 },
    { 0x6aa122e3, 23 },
    { 0, 0 },
    { 0x993122e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28122ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb122ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72122f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af122f5, 95 },
    { 0, 0 },
    { 0x6cb122f7, 85 },
    { 0, 0 },
    { 0xc14122f9, 61 },
    { 0, 0 },
    { 0xe1c122fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a712304, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75312307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2451230a, 34 },
    { 0xf611230a, 95 },
    { 0x31e1230c, 43 },
    { 0, 0 },
    { 0x5311230e, 61 },
    { 0x92b1230e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b12314, 76 },
    { 0x81a12315, 88 },
    { 0, 0 },
    { 0x93f12317, 61 },
    { 0, 0 },
    { 0xbee12319, 61 },
    { 0xc941231a, 76 },
    { 0x4cc1231b, 33 },
    { 0xdf512319, 34 },
    { 0x3451231d, 90 },
    { 0x311231a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35412321, 61 },
    { 0xf9112322, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1232a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5431232d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e12331, 38 },
    { 0, 0 },
    { 0xcd512333, 95 },
    { 0x26612334, 80 },
    { 0xf5d12335, 98 },
    { 0xbc112335, 69 },
    { 0x2ad12333, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c1233b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb12345, 43 },
    { 0, 0 },
    { 0x7d412347, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b1234a, 23 },
    { 0xda41234b, 76 },
    { 0xafa1234a, 75 },
    { 0, 0 },
    { 0xefd1234e, 23 },
    { 0xf951234e, 61 },
    { 0x1234e, 34 },
    { 0xcb312351, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2512354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e812357, 34 },
    { 0, 0 },
    { 0x85f12359, 85 },
    { 0, 0 },
    { 0xf51235b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8312361, 90 },
    { 0, 0 },
    { 0xbd12363, 23 },
    { 0xdac12363, 61 },
    { 0x7f712363, 55 },
    { 0xdcd12366, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7812369, 91 },
    { 0, 0 },
    { 0xa641236b, 91 },
    { 0x7b21236c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21812370, 61 },
    { 0, 0 },
    { 0xa7812372, 76 },
    { 0, 0 },
    { 0x9f112374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2991237a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6141237d, 61 },
    { 0, 0 },
    { 0x2e51237f, 95 },
    { 0, 0 },
    { 0xf2112381, 43 },
    { 0, 0 },
    { 0xe7912383, 76 },
    { 0x8ff12384, 95 },
    { 0xcac12385, 90 },
    { 0x54512386, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e1238c, 98 },
    { 0x2e1238c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67112393, 19 },
    { 0, 0 },
    { 0xe2d12395, 52 },
    { 0, 0 },
    { 0x24412397, 91 },
    { 0, 0 },
    { 0x23d12399, 100 },
    { 0xde81239a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb41239f, 55 },
    { 0xd8a123a0, 85 },
    { 0x637123a1, 100 },
    { 0xe7b123a0, 23 },
    { 0xadb123a0, 66 },
    { 0x8d9123a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92123a7, 23 },
    { 0xc62123a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1123ab, 48 },
    { 0x354123ab, 61 },
    { 0x2d3123ad, 67 },
    { 0xc77123ad, 76 },
    { 0x5b4123ac, 98 },
    { 0x1f0123af, 34 },
    { 0, 0 },
    { 0x397123b2, 90 },
    { 0xd38123b3, 90 },
    { 0xf8f123b4, 87 },
    { 0, 0 },
    { 0xb3a123b6, 52 },
    { 0xfeb123b7, 85 },
    { 0xa07123b8, 84 },
    { 0xaa123b7, 78 },
    { 0x577123ba, 61 },
    { 0xe4123bb, 90 },
    { 0, 0 },
    { 0x2fa123bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1123c7, 76 },
    { 0, 0 },
    { 0x415123c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87123cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a123d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0123d4, 67 },
    { 0x71a123d5, 55 },
    { 0x914123d6, 61 },
    { 0xb95123d5, 33 },
    { 0xb06123d4, 76 },
    { 0, 0 },
    { 0x296123da, 23 },
    { 0xd96123db, 33 },
    { 0x9ef123dc, 61 },
    { 0, 0 },
    { 0x1c3123de, 66 },
    { 0, 0 },
    { 0x870123e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd123e3, 85 },
    { 0x352123e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa123e7, 90 },
    { 0xe9b123e7, 43 },
    { 0, 0 },
    { 0x75123ea, 87 },
    { 0x5f0123eb, 66 },
    { 0xe26123eb, 90 },
    { 0, 0 },
    { 0xb6e123ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb123f1, 90 },
    { 0x146123f2, 90 },
    { 0xb25123f1, 26 },
    { 0xec1123f4, 75 },
    { 0x17a123f1, 61 },
    { 0xf81123f6, 93 },
    { 0x945123f7, 95 },
    { 0x32d123f3, 66 },
    { 0, 0 },
    { 0x54123fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xca123fd, 23 },
    { 0x71c123fd, 46 },
    { 0x1f1123ff, 98 },
    { 0x9ca12400, 26 },
    { 0, 0 },
    { 0xa0e12402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f12405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e12409, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c1240c, 95 },
    { 0, 0 },
    { 0xf5c1240e, 88 },
    { 0, 0 },
    { 0xc1812410, 33 },
    { 0xa4e12411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2412417, 90 },
    { 0, 0 },
    { 0x29812419, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a1241c, 61 },
    { 0x89b1241c, 93 },
    { 0x2551241c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce12424, 97 },
    { 0, 0 },
    { 0xf6212426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a91242a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee91242d, 23 },
    { 0xaec1242d, 33 },
    { 0xf2d1242e, 43 },
    { 0x5d012430, 77 },
    { 0x92c12430, 98 },
    { 0x7eb12432, 90 },
    { 0x2c012432, 23 },
    { 0x62912434, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d1243a, 85 },
    { 0xdba1243b, 90 },
    { 0x9cf1243c, 76 },
    { 0x3b71243b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb412440, 87 },
    { 0x2d412441, 61 },
    { 0, 0 },
    { 0xce612443, 90 },
    { 0xbb512444, 77 },
    { 0x6bf12445, 98 },
    { 0x2a812446, 61 },
    { 0x48312447, 46 },
    { 0, 0 },
    { 0x72612449, 61 },
    { 0x7b512449, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b41244e, 76 },
    { 0xd5f1244e, 90 },
    { 0, 0 },
    { 0xf9112451, 93 },
    { 0x91f12452, 43 },
    { 0xbd312453, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c12459, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c12466, 84 },
    { 0x4ae12467, 59 },
    { 0x91f12466, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xede1246b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35112475, 91 },
    { 0, 0 },
    { 0x27212477, 61 },
    { 0, 0 },
    { 0xb0512479, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce71248b, 95 },
    { 0, 0 },
    { 0xe621248d, 98 },
    { 0xebd1248d, 23 },
    { 0x8381248d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e12495, 33 },
    { 0xc4012496, 33 },
    { 0x7d512497, 33 },
    { 0x31312498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a1249b, 99 },
    { 0, 0 },
    { 0x7c41249d, 33 },
    { 0x7e61249e, 61 },
    { 0xae21249f, 33 },
    { 0xa9c1249d, 34 },
    { 0x3df124a1, 33 },
    { 0x92e124a2, 90 },
    { 0xb42124a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde124ab, 23 },
    { 0x426124ac, 33 },
    { 0xb14124ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204124b8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a124bb, 46 },
    { 0xc15124bc, 61 },
    { 0, 0 },
    { 0x64a124be, 33 },
    { 0x33f124bf, 90 },
    { 0xeb7124c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x708124c3, 95 },
    { 0xc2f124c4, 75 },
    { 0xa87124c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c124c9, 98 },
    { 0xfe3124ca, 33 },
    { 0, 0 },
    { 0xb04124cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a124d2, 17 },
    { 0x996124d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f124d6, 78 },
    { 0x3db124d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739124de, 100 },
    { 0, 0 },
    { 0x8b2124e0, 33 },
    { 0xe90124e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551124e5, 34 },
    { 0xf70124e6, 55 },
    { 0x6fa124e5, 23 },
    { 0x7e3124e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770124ec, 23 },
    { 0x80d124ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394124f5, 90 },
    { 0xd7e124f6, 95 },
    { 0x82d124f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f412501, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x57012504, 55 },
    { 0, 0 },
    { 0xcc12506, 33 },
    { 0xbf912506, 23 },
    { 0xbf012506, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2661250b, 93 },
    { 0x1081250c, 55 },
    { 0xfe51250b, 90 },
    { 0xc8a1250e, 55 },
    { 0xe1a1250d, 96 },
    { 0x6151250b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3412513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7212517, 61 },
    { 0xa7412517, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a1251c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf751251f, 34 },
    { 0x4ce12520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c21252a, 61 },
    { 0x49a1252b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d12530, 43 },
    { 0x81812530, 94 },
    { 0x51d12530, 23 },
    { 0x62312532, 23 },
    { 0x8ea12530, 98 },
    { 0x71512533, 34 },
    { 0xcc012530, 90 },
    { 0x2d912537, 66 },
    { 0x39512531, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8181253b, 83 },
    { 0, 0 },
    { 0x1361253d, 92 },
    { 0x23c1253e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b12546, 48 },
    { 0x2e012547, 93 },
    { 0, 0 },
    { 0x36212549, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9961254c, 95 },
    { 0, 0 },
    { 0x64c1254e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7312551, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd712558, 17 },
    { 0x2f112559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1255f, 81 },
    { 0x17f1255f, 43 },
    { 0, 0 },
    { 0xca912562, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9471256a, 34 },
    { 0x1d21256b, 66 },
    { 0, 0 },
    { 0xbb21256d, 23 },
    { 0, 0 },
    { 0x38f1256f, 55 },
    { 0x28812570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7212576, 90 },
    { 0, 0 },
    { 0x31412578, 23 },
    { 0, 0 },
    { 0x4a11257a, 90 },
    { 0xe4e1257b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af1257e, 61 },
    { 0, 0 },
    { 0x75e12580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c12586, 98 },
    { 0, 0 },
    { 0x8a212588, 98 },
    { 0x6bd12588, 76 },
    { 0x30a1258a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa811258f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81312592, 23 },
    { 0x37212593, 23 },
    { 0x5ec12594, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c512597, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461259d, 3 },
    { 0x8931259e, 77 },
    { 0, 0 },
    { 0x95b125a0, 95 },
    { 0, 0 },
    { 0xe8d125a2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d125a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b125aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432125ae, 61 },
    { 0x230125af, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99125b8, 43 },
    { 0x1a3125b9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x135125bc, 33 },
    { 0x51b125bc, 68 },
    { 0xab0125be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2125c4, 98 },
    { 0xf125c5, 33 },
    { 0xade125c6, 23 },
    { 0x6d6125c6, 43 },
    { 0xfd6125c8, 61 },
    { 0xecd125c9, 43 },
    { 0x3ea125c5, 76 },
    { 0x5bc125c4, 66 },
    { 0x69a125c8, 66 },
    { 0xfa1125cd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836125d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd125d8, 50 },
    { 0xb0e125d9, 87 },
    { 0xe12125da, 34 },
    { 0x579125da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee125de, 52 },
    { 0, 0 },
    { 0x35e125e0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f125e4, 98 },
    { 0x153125e4, 85 },
    { 0xc73125e4, 23 },
    { 0, 0 },
    { 0x1c1125e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef125eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2125f5, 95 },
    { 0x221125f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1125fd, 34 },
    { 0x1db125fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3812608, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4141260b, 99 },
    { 0xbff1260c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9212616, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x77912619, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071261d, 90 },
    { 0xc601261e, 33 },
    { 0x9011261e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27412625, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311262d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11112633, 90 },
    { 0x21212634, 61 },
    { 0x4dc12633, 90 },
    { 0xafa12635, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e1263b, 91 },
    { 0x99d1263c, 33 },
    { 0xcd21263d, 46 },
    { 0xb021263e, 91 },
    { 0xb1f1263e, 87 },
    { 0xc8012640, 61 },
    { 0x4fa1263d, 84 },
    { 0xcdc12642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca12649, 33 },
    { 0x3e712649, 98 },
    { 0xd6d1264b, 55 },
    { 0, 0 },
    { 0x1691264d, 61 },
    { 0, 0 },
    { 0xdd41264f, 40 },
    { 0, 0 },
    { 0x89312651, 33 },
    { 0, 0 },
    { 0xd312653, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d012657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf1265b, 23 },
    { 0x7931265b, 23 },
    { 0, 0 },
    { 0x1891265e, 100 },
    { 0x7a31265f, 65 },
    { 0, 0 },
    { 0xdd312661, 55 },
    { 0x9fa12661, 98 },
    { 0, 0 },
    { 0x8b812664, 61 },
    { 0, 0 },
    { 0x35412666, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb1266a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a41266e, 95 },
    { 0, 0 },
    { 0x66012670, 98 },
    { 0x7dd12670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c012679, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa271267d, 55 },
    { 0x9ad1267e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb012681, 34 },
    { 0xeb912682, 98 },
    { 0xa9f12683, 88 },
    { 0x7ff12684, 52 },
    { 0x86212684, 34 },
    { 0x3f712686, 76 },
    { 0x31d12684, 66 },
    { 0xde512688, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9141268c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b1268f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4612692, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e12695, 76 },
    { 0x18912696, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b12699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa221269e, 98 },
    { 0xf311269f, 23 },
    { 0xa42126a0, 91 },
    { 0x4cb126a1, 61 },
    { 0x241126a2, 98 },
    { 0x8dc126a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419126a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc126ad, 90 },
    { 0xec9126ae, 93 },
    { 0xed3126af, 77 },
    { 0, 0 },
    { 0x40a126b1, 61 },
    { 0, 0 },
    { 0xca7126b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6126b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd126b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba126bf, 76 },
    { 0xf27126bf, 61 },
    { 0xb87126bf, 55 },
    { 0xdb9126c2, 52 },
    { 0, 0 },
    { 0x75f126c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd126c8, 85 },
    { 0, 0 },
    { 0x87f126ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0126d4, 76 },
    { 0xd4126d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4126e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f126ea, 95 },
    { 0xd4b126ea, 98 },
    { 0x188126ec, 23 },
    { 0x2a2126ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1126f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b126f5, 33 },
    { 0, 0 },
    { 0x8ee126f7, 23 },
    { 0, 0 },
    { 0xe80126f9, 98 },
    { 0x34f126fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214126fe, 43 },
    { 0x62c126ff, 33 },
    { 0xfc9126fe, 61 },
    { 0x9f112701, 90 },
    { 0x83f12702, 53 },
    { 0x1b112701, 90 },
    { 0x79e12702, 95 },
    { 0xc7012705, 78 },
    { 0xe2112705, 33 },
    { 0x7d812707, 61 },
    { 0xcd6126fe, 23 },
    { 0, 0 },
    { 0x63d1270a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8121270e, 90 },
    { 0xbea1270e, 43 },
    { 0xc651270e, 95 },
    { 0x9ad12711, 67 },
    { 0x87612711, 37 },
    { 0, 0 },
    { 0x84612714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f12717, 67 },
    { 0xfb612717, 55 },
    { 0xe012719, 81 },
    { 0xbe31271a, 95 },
    { 0xf821271b, 76 },
    { 0x25a1271c, 33 },
    { 0x8591271a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d12720, 61 },
    { 0xa0012721, 33 },
    { 0x61212722, 25 },
    { 0x22b12723, 95 },
    { 0x73912724, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe721272d, 23 },
    { 0x9491272e, 61 },
    { 0xef01272f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1512738, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e1273d, 33 },
    { 0xe471273e, 90 },
    { 0x89f1273e, 77 },
    { 0, 0 },
    { 0xa9712741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d12746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc241274c, 34 },
    { 0, 0 },
    { 0x7a91274e, 55 },
    { 0x99a1274f, 47 },
    { 0xf7d12750, 23 },
    { 0x4551274f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3612757, 90 },
    { 0x71612758, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8891275c, 78 },
    { 0x981275c, 81 },
    { 0xdeb1275e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f12762, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5812765, 61 },
    { 0x2112766, 55 },
    { 0, 0 },
    { 0xd4f12768, 91 },
    { 0x47412769, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe191276e, 34 },
    { 0xae41276f, 90 },
    { 0xae012770, 91 },
    { 0xaab12770, 91 },
    { 0xbb912770, 98 },
    { 0x60712773, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f12779, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f12787, 55 },
    { 0x15312788, 95 },
    { 0x42d12789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd812791, 55 },
    { 0, 0 },
    { 0xe7a12793, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20212799, 90 },
    { 0x6dc1279a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2127a0, 43 },
    { 0, 0 },
    { 0x499127a2, 33 },
    { 0x230127a3, 98 },
    { 0, 0 },
    { 0x3d127a5, 65 },
    { 0x5f3127a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf127ab, 46 },
    { 0x4d9127ac, 88 },
    { 0x5a7127ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf127b1, 53 },
    { 0xf43127b2, 38 },
    { 0x477127b2, 84 },
    { 0xb44127b2, 23 },
    { 0xc8c127b2, 76 },
    { 0xc4b127b2, 100 },
    { 0, 0 },
    { 0xf05127b8, 23 },
    { 0, 0 },
    { 0x27c127ba, 23 },
    { 0, 0 },
    { 0xfcd127bc, 76 },
    { 0x423127bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15127c0, 49 },
    { 0xa54127c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2127cb, 76 },
    { 0x9c8127cc, 100 },
    { 0xac9127cd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0127d1, 61 },
    { 0x655127d1, 23 },
    { 0xaa1127d2, 84 },
    { 0xa10127d2, 95 },
    { 0x867127d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc127d9, 65 },
    { 0, 0 },
    { 0x4a7127db, 23 },
    { 0, 0 },
    { 0x51d127dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45127e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15127e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6127ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2127f1, 23 },
    { 0x651127f2, 61 },
    { 0xebb127f1, 80 },
    { 0xd2b127f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79127f7, 90 },
    { 0x3da127f8, 90 },
    { 0xcf0127f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b127fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee012803, 81 },
    { 0, 0 },
    { 0x10512805, 61 },
    { 0x3de12806, 33 },
    { 0x8c512807, 99 },
    { 0x1e212808, 61 },
    { 0x6c812808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3821280d, 90 },
    { 0, 0 },
    { 0x54e1280f, 61 },
    { 0, 0 },
    { 0x5c012811, 61 },
    { 0x3c12811, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa581281a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b12825, 90 },
    { 0x17112826, 81 },
    { 0x5812827, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa891282a, 101 },
    { 0, 0 },
    { 0xec1282c, 56 },
    { 0x3891282d, 98 },
    { 0x8db1282c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c12834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee12837, 93 },
    { 0x96512838, 98 },
    { 0xf3312837, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4291283e, 90 },
    { 0, 0 },
    { 0xdbb12840, 34 },
    { 0x13012840, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812844, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd312850, 95 },
    { 0x82112851, 33 },
    { 0x4f912851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6812857, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e312862, 61 },
    { 0x42312863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc971286a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1791286e, 23 },
    { 0xd801286f, 43 },
    { 0xcd712870, 23 },
    { 0xbaa12871, 61 },
    { 0xec312872, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96612876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc1287e, 90 },
    { 0xe801287f, 67 },
    { 0x6d812880, 97 },
    { 0x3cc12880, 75 },
    { 0, 0 },
    { 0x6d512883, 61 },
    { 0x55012883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d12888, 23 },
    { 0, 0 },
    { 0x571288a, 55 },
    { 0, 0 },
    { 0x8f81288c, 98 },
    { 0xf41288d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x13612890, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x61412893, 52 },
    { 0xf4612893, 90 },
    { 0x57412895, 98 },
    { 0x96312896, 95 },
    { 0, 0 },
    { 0xec12898, 100 },
    { 0, 0 },
    { 0xf6c1289a, 23 },
    { 0xcfa1289b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc71289e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x891128a1, 95 },
    { 0x1dd128a1, 34 },
    { 0x1aa128a2, 90 },
    { 0x492128a1, 91 },
    { 0xdfb128a5, 90 },
    { 0x916128a1, 78 },
    { 0x563128a7, 43 },
    { 0x9ce128a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4128ab, 33 },
    { 0, 0 },
    { 0xe89128ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x623128b0, 78 },
    { 0xe65128b1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3128b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b128b9, 65 },
    { 0, 0 },
    { 0x414128bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be128be, 99 },
    { 0x85128bf, 98 },
    { 0x384128c0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a128c3, 66 },
    { 0xab9128c4, 33 },
    { 0xef128c3, 52 },
    { 0x9d4128c3, 34 },
    { 0, 0 },
    { 0x87b128c8, 90 },
    { 0xe1c128c9, 82 },
    { 0, 0 },
    { 0x58f128cb, 69 },
    { 0x9d128cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd128d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0128d8, 98 },
    { 0, 0 },
    { 0x9a7128da, 90 },
    { 0x9df128db, 95 },
    { 0x69c128dc, 76 },
    { 0x366128db, 98 },
    { 0x196128de, 67 },
    { 0x895128df, 61 },
    { 0x6bc128dd, 52 },
    { 0x720128e1, 33 },
    { 0xcd1128e2, 43 },
    { 0x67d128e3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649128ea, 55 },
    { 0, 0 },
    { 0x95b128ec, 61 },
    { 0x76a128ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1128f0, 80 },
    { 0, 0 },
    { 0x77e128f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230128f8, 76 },
    { 0, 0 },
    { 0xccf128fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe712903, 90 },
    { 0x7c012903, 90 },
    { 0x8cc12905, 66 },
    { 0x45312906, 61 },
    { 0x48112907, 23 },
    { 0x3a812904, 95 },
    { 0, 0 },
    { 0x5dc1290a, 95 },
    { 0, 0 },
    { 0x6fc1290c, 90 },
    { 0x6481290d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32012913, 93 },
    { 0x5ba12913, 52 },
    { 0, 0 },
    { 0x69012916, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb12919, 33 },
    { 0x6c12919, 61 },
    { 0, 0 },
    { 0x3f61291c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac12925, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1012928, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a41292d, 85 },
    { 0, 0 },
    { 0x6e21292f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1212933, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f412937, 85 },
    { 0x6ef12938, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a1293c, 43 },
    { 0x181293c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912941, 90 },
    { 0x18312942, 61 },
    { 0xb0712942, 75 },
    { 0, 0 },
    { 0x20c12945, 43 },
    { 0xe7b12945, 51 },
    { 0x8c912947, 65 },
    { 0xb2b12948, 88 },
    { 0xb712949, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21012954, 90 },
    { 0x7fb12954, 37 },
    { 0x20412956, 95 },
    { 0x73912955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca1295a, 17 },
    { 0x39e1295b, 90 },
    { 0x6751295b, 46 },
    { 0, 0 },
    { 0xdb21295e, 67 },
    { 0x8e51295e, 99 },
    { 0xb11295e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9112964, 61 },
    { 0xcdf12965, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b712968, 76 },
    { 0xeb312969, 46 },
    { 0x3f91296a, 33 },
    { 0xe9b12968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36012974, 90 },
    { 0x6fa12975, 61 },
    { 0xa8412976, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4271297b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca12981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f12984, 90 },
    { 0, 0 },
    { 0x4b312986, 75 },
    { 0, 0 },
    { 0x39912988, 95 },
    { 0xc4712989, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7031298e, 17 },
    { 0x4b1298f, 33 },
    { 0x24a1298e, 61 },
    { 0xb8c12991, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe12994, 90 },
    { 0x84512995, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5241299d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa129a0, 52 },
    { 0x8d1129a1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c129a5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a129aa, 55 },
    { 0xa11129aa, 55 },
    { 0, 0 },
    { 0x20f129ad, 33 },
    { 0x5b6129ae, 90 },
    { 0xab4129ae, 87 },
    { 0xa60129ae, 23 },
    { 0x601129af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e129b6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db129ba, 61 },
    { 0, 0 },
    { 0xa9a129bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d129bf, 56 },
    { 0xea3129c0, 76 },
    { 0x466129c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae129cb, 61 },
    { 0x48c129cb, 61 },
    { 0x232129cd, 98 },
    { 0, 0 },
    { 0x663129cf, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5129d2, 23 },
    { 0x96f129d3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7129e2, 34 },
    { 0x13c129e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a129e8, 87 },
    { 0, 0 },
    { 0xeb6129ea, 34 },
    { 0xe78129eb, 98 },
    { 0xe2b129ec, 23 },
    { 0, 0 },
    { 0xb4d129ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc129f5, 83 },
    { 0xeb6129f6, 69 },
    { 0xa62129f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5129fb, 23 },
    { 0x258129fb, 34 },
    { 0x25d129fd, 61 },
    { 0xe5129fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba12a01, 88 },
    { 0x88312a01, 99 },
    { 0x65312a03, 33 },
    { 0, 0 },
    { 0xda412a05, 93 },
    { 0x14212a05, 88 },
    { 0xd2b12a07, 81 },
    { 0x18312a08, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f12a12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17912a15, 93 },
    { 0, 0 },
    { 0xed12a17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc12a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c412a1f, 93 },
    { 0xc7d12a20, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0812a23, 61 },
    { 0xf4f12a23, 23 },
    { 0xefd12a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5612a29, 95 },
    { 0x6c712a2a, 90 },
    { 0, 0 },
    { 0xa4912a2c, 61 },
    { 0x8b212a2d, 98 },
    { 0, 0 },
    { 0x60312a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe012a33, 17 },
    { 0, 0 },
    { 0xff012a35, 67 },
    { 0x96312a36, 95 },
    { 0xde212a37, 98 },
    { 0xafd12a35, 76 },
    { 0x1fd12a39, 98 },
    { 0x3f312a36, 52 },
    { 0x23a12a3b, 33 },
    { 0x87a12a3c, 90 },
    { 0, 0 },
    { 0x11412a3e, 98 },
    { 0x94a12a3f, 33 },
    { 0, 0 },
    { 0x90512a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf12a44, 90 },
    { 0, 0 },
    { 0xc9312a46, 76 },
    { 0x14212a46, 90 },
    { 0xb9812a48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed12a4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d12a4f, 23 },
    { 0xebb12a50, 33 },
    { 0xda612a4f, 33 },
    { 0x92312a52, 61 },
    { 0x51712a53, 33 },
    { 0, 0 },
    { 0x96212a55, 78 },
    { 0, 0 },
    { 0xd3f12a57, 95 },
    { 0, 0 },
    { 0xef812a59, 98 },
    { 0xd9e12a59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3412a60, 33 },
    { 0x74812a60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef12a68, 68 },
    { 0x95912a69, 23 },
    { 0, 0 },
    { 0x21e12a6b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f12a6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f12a80, 61 },
    { 0, 0 },
    { 0x3d512a82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a12a8c, 85 },
    { 0x95512a8d, 85 },
    { 0x6b512a8d, 43 },
    { 0x4712a8e, 33 },
    { 0xd6312a8f, 76 },
    { 0xa7a12a8d, 93 },
    { 0, 0 },
    { 0x4b212a93, 95 },
    { 0, 0 },
    { 0x4cb12a95, 76 },
    { 0, 0 },
    { 0x42a12a97, 46 },
    { 0, 0 },
    { 0x94112a99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d812a9d, 98 },
    { 0x1e12a9e, 95 },
    { 0xfbf12a9e, 55 },
    { 0x10512aa0, 95 },
    { 0xaeb12aa0, 55 },
    { 0, 0 },
    { 0x3212aa3, 99 },
    { 0xabc12aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59412aab, 61 },
    { 0x36e12aac, 55 },
    { 0xa1112aab, 43 },
    { 0x19712aae, 53 },
    { 0x4f112aaf, 95 },
    { 0x80012ab0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14812ab5, 52 },
    { 0x58a12ab6, 99 },
    { 0xffa12ab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a512ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88412ac4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6012ac7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6412acc, 90 },
    { 0xfce12acc, 33 },
    { 0, 0 },
    { 0xeb912acf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f12ad3, 76 },
    { 0xf7712ad4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b212ad8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d12adc, 90 },
    { 0, 0 },
    { 0xb0212ade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66712ae4, 84 },
    { 0xa2412ae5, 90 },
    { 0, 0 },
    { 0xac912ae7, 33 },
    { 0x1fa12ae8, 91 },
    { 0xe4e12ae7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb12aec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c12aef, 34 },
    { 0x20e12af0, 90 },
    { 0, 0 },
    { 0xdbc12af2, 90 },
    { 0x76c12af2, 93 },
    { 0x65612af4, 90 },
    { 0x1d712af5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44112af8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14112afb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb12b02, 98 },
    { 0, 0 },
    { 0x98e12b04, 55 },
    { 0x65312b05, 75 },
    { 0x35712b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86212b0c, 98 },
    { 0xeb812b0d, 61 },
    { 0xe3512b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0112b12, 40 },
    { 0x83512b13, 95 },
    { 0, 0 },
    { 0x52912b15, 33 },
    { 0xc0512b16, 48 },
    { 0x12512b17, 90 },
    { 0, 0 },
    { 0x37012b19, 55 },
    { 0x46e12b19, 98 },
    { 0x5d412b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4912b1e, 95 },
    { 0, 0 },
    { 0xf4312b20, 95 },
    { 0, 0 },
    { 0x33812b22, 90 },
    { 0xe012b23, 23 },
    { 0x47812b23, 61 },
    { 0x69212b25, 90 },
    { 0xd5f12b26, 90 },
    { 0xa0d12b27, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88112b2b, 78 },
    { 0, 0 },
    { 0xc5a12b2d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9112b30, 61 },
    { 0, 0 },
    { 0x7ed12b32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a612b36, 88 },
    { 0x97212b36, 23 },
    { 0x22912b36, 90 },
    { 0x9ae12b39, 61 },
    { 0x2d512b39, 33 },
    { 0xe3c12b3b, 98 },
    { 0x73212b3a, 90 },
    { 0, 0 },
    { 0x82512b3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e12b43, 91 },
    { 0xdb12b43, 98 },
    { 0x53d12b45, 23 },
    { 0x8f412b45, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a12b57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd12b5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4212b61, 55 },
    { 0xffd12b62, 44 },
    { 0x43212b63, 23 },
    { 0xf6a12b64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c12b68, 67 },
    { 0, 0 },
    { 0xc0112b6a, 95 },
    { 0, 0 },
    { 0x8b012b6c, 61 },
    { 0x1ed12b6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9812b75, 33 },
    { 0xf4412b75, 100 },
    { 0x50312b77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b712b7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b12b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16612b80, 44 },
    { 0xeff12b81, 17 },
    { 0x9fd12b82, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x51312b85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c12b8a, 95 },
    { 0x8f612b8b, 33 },
    { 0x46a12b8b, 61 },
    { 0x54a12b8d, 17 },
    { 0xcbd12b8c, 76 },
    { 0, 0 },
    { 0xf9c12b90, 100 },
    { 0, 0 },
    { 0x35112b92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d12b99, 90 },
    { 0, 0 },
    { 0x8da12b9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b12ba2, 46 },
    { 0xcb312ba2, 95 },
    { 0x60412ba4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe12ba9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35012bb1, 68 },
    { 0x94e12bb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1012bbb, 90 },
    { 0, 0 },
    { 0xf8912bbd, 90 },
    { 0x3212bbe, 98 },
    { 0x79f12bbe, 67 },
    { 0x64c12bc0, 80 },
    { 0xdd312bc1, 66 },
    { 0xa2412bc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c12bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3212bce, 43 },
    { 0x49812bcf, 76 },
    { 0xf9f12bce, 90 },
    { 0xa412bd1, 78 },
    { 0x46312bd2, 98 },
    { 0xadb12bd1, 98 },
    { 0, 0 },
    { 0x73812bd5, 55 },
    { 0x89612bd5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1212bd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4712be0, 85 },
    { 0x84112be1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb912be7, 43 },
    { 0, 0 },
    { 0xea112be9, 90 },
    { 0x3d712bea, 99 },
    { 0xa5212bea, 91 },
    { 0x8912bec, 95 },
    { 0xa6b12bec, 81 },
    { 0xd9012bec, 33 },
    { 0x47212bef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12012bf8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90812bfc, 23 },
    { 0xdcc12bfd, 23 },
    { 0, 0 },
    { 0x9e512bff, 93 },
    { 0, 0 },
    { 0xfdf12c01, 90 },
    { 0, 0 },
    { 0x67812c03, 33 },
    { 0, 0 },
    { 0x23712c05, 23 },
    { 0, 0 },
    { 0xb0912c07, 98 },
    { 0xeb412c08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12912c12, 90 },
    { 0, 0 },
    { 0xe7612c14, 91 },
    { 0x29e12c14, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc712c19, 90 },
    { 0xb5912c1a, 33 },
    { 0, 0 },
    { 0xdd412c1c, 23 },
    { 0xba12c1d, 95 },
    { 0xbf612c1e, 43 },
    { 0x7ad12c1f, 76 },
    { 0xb2f12c1e, 90 },
    { 0x71912c1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96612c2b, 95 },
    { 0x5c312c2c, 90 },
    { 0x65d12c2d, 34 },
    { 0, 0 },
    { 0x5c712c2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2212c36, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb312c3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb912c44, 95 },
    { 0, 0 },
    { 0x8a12c46, 76 },
    { 0x77d12c47, 23 },
    { 0, 0 },
    { 0x2ca12c49, 81 },
    { 0x8d12c49, 61 },
    { 0xf1a12c4b, 93 },
    { 0xa2a12c4c, 40 },
    { 0x29d12c4d, 23 },
    { 0x78012c4a, 76 },
    { 0x19612c4a, 23 },
    { 0x79812c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb412c55, 61 },
    { 0x19912c56, 90 },
    { 0x9c412c57, 61 },
    { 0x26912c56, 55 },
    { 0x9c512c59, 99 },
    { 0x3312c59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1712c5d, 97 },
    { 0, 0 },
    { 0x98c12c5f, 23 },
    { 0xfaf12c5f, 98 },
    { 0xb412c61, 52 },
    { 0xa1d12c61, 90 },
    { 0xb1c12c60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e12c68, 33 },
    { 0x7fa12c69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e512c6c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d12c72, 76 },
    { 0, 0 },
    { 0x13612c74, 33 },
    { 0xd4012c74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e12c78, 90 },
    { 0x49412c79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1412c7f, 90 },
    { 0xea012c80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d812c83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b12c87, 33 },
    { 0x54a12c88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30012c91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e612c94, 90 },
    { 0xd5c12c95, 65 },
    { 0, 0 },
    { 0x7d812c97, 76 },
    { 0xe2412c97, 95 },
    { 0x1712c98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a412c9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc12c9f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34512ca6, 93 },
    { 0, 0 },
    { 0x43312ca8, 90 },
    { 0x4e712ca9, 76 },
    { 0xce312ca8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e912cad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d12cb1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x96312cb4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba12cb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6112cbb, 81 },
    { 0xe2c12cbc, 61 },
    { 0xb2412cbd, 75 },
    { 0x15312cbd, 23 },
    { 0xf2a12cbb, 98 },
    { 0xda12cbd, 55 },
    { 0x64d12cbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212cc8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d12ccc, 52 },
    { 0x72212ccd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d12cd3, 99 },
    { 0x24b12cd4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8912cd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf12cda, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12cde, 95 },
    { 0xecf12cdf, 93 },
    { 0x93412ce0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d12ce3, 35 },
    { 0, 0 },
    { 0x2f812ce5, 85 },
    { 0, 0 },
    { 0x92d12ce7, 45 },
    { 0, 0 },
    { 0x63212ce9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f12ced, 55 },
    { 0x3d512ced, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70812cf2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e712cf8, 33 },
    { 0x3e312cf8, 90 },
    { 0xd7a12cfa, 76 },
    { 0x68e12cfb, 90 },
    { 0xbf212cfb, 34 },
    { 0x5c112cfd, 90 },
    { 0x27712cfd, 33 },
    { 0xc8412cff, 46 },
    { 0x20d12cf9, 90 },
    { 0xeaf12d01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c12d08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5612d16, 95 },
    { 0x2c112d17, 98 },
    { 0x67e12d17, 33 },
    { 0x18312d19, 55 },
    { 0x2d512d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a12d1d, 100 },
    { 0, 0 },
    { 0xedd12d1f, 66 },
    { 0x36412d20, 23 },
    { 0x25f12d20, 55 },
    { 0x28212d22, 61 },
    { 0xb8512d23, 68 },
    { 0xc1612d23, 55 },
    { 0xfc612d21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b12d29, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x90212d2c, 97 },
    { 0, 0 },
    { 0x79f12d2e, 76 },
    { 0, 0 },
    { 0x7fe12d30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6412d35, 76 },
    { 0, 0 },
    { 0x52512d37, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe712d3a, 56 },
    { 0x73012d3b, 36 },
    { 0, 0 },
    { 0xe0112d3d, 98 },
    { 0x53b12d3e, 23 },
    { 0x44f12d3f, 90 },
    { 0xcbb12d3d, 96 },
    { 0x39712d3e, 61 },
    { 0xe4a12d3e, 61 },
    { 0, 0 },
    { 0x26c12d44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b112d48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b12d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98612d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b12d55, 91 },
    { 0xf4512d55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f112d5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c012d5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3712d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d512d6b, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f212d75, 61 },
    { 0x32012d76, 23 },
    { 0xac812d77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e12d7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a12d81, 46 },
    { 0, 0 },
    { 0x3f412d83, 99 },
    { 0x23612d84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c12d8a, 43 },
    { 0, 0 },
    { 0x6aa12d8c, 75 },
    { 0x65712d8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28912d9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44712da0, 43 },
    { 0xe2f12da0, 76 },
    { 0x5a012da2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f312da5, 23 },
    { 0x4912da6, 90 },
    { 0x22812da7, 23 },
    { 0xfd512da8, 43 },
    { 0x5e612da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c12dae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d12dbc, 95 },
    { 0x50512dbd, 17 },
    { 0xdf12dbd, 100 },
    { 0, 0 },
    { 0xd912dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e12dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a112dcb, 23 },
    { 0x97b12dcc, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xe712dcf, 33 },
    { 0x5b512dd0, 55 },
    { 0, 0 },
    { 0xe6612dd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7312dda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c212ddd, 98 },
    { 0x10612dde, 33 },
    { 0, 0 },
    { 0xc7312de0, 78 },
    { 0, 0 },
    { 0x9c712de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf912de8, 40 },
    { 0x30912de9, 33 },
    { 0x7f212de8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e512ded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb612df0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2312df3, 23 },
    { 0x13312df4, 95 },
    { 0xea512df4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d12df8, 33 },
    { 0xd0412df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d712dfc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc112dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db12e03, 80 },
    { 0xe9f12e04, 61 },
    { 0x4e512e03, 90 },
    { 0xdaf12e05, 61 },
    { 0x7d912e04, 99 },
    { 0xaf212e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff12e0b, 33 },
    { 0x8bb12e0c, 95 },
    { 0x9e712e0c, 93 },
    { 0x25512e0c, 93 },
    { 0xd4c12e0f, 23 },
    { 0, 0 },
    { 0x45812e11, 93 },
    { 0xd1c12e11, 98 },
    { 0, 0 },
    { 0x52812e14, 23 },
    { 0x52712e15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf412e1a, 33 },
    { 0xc2312e1b, 90 },
    { 0, 0 },
    { 0xbc212e1d, 61 },
    { 0, 0 },
    { 0xbda12e1f, 91 },
    { 0, 0 },
    { 0x8db12e21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d912e25, 55 },
    { 0x2d12e26, 61 },
    { 0x27812e27, 98 },
    { 0xaf412e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad12e2c, 33 },
    { 0x5c412e2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f512e37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d12e3c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59712e42, 55 },
    { 0x66812e43, 23 },
    { 0x8112e44, 55 },
    { 0, 0 },
    { 0x3d412e46, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3112e49, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b12e4e, 90 },
    { 0, 0 },
    { 0x2b512e50, 33 },
    { 0x1db12e51, 78 },
    { 0x40412e51, 23 },
    { 0x39e12e50, 61 },
    { 0x7dc12e51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69612e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad12e5a, 34 },
    { 0, 0 },
    { 0x7d212e5c, 23 },
    { 0x5be12e5d, 23 },
    { 0xfe912e5e, 61 },
    { 0x4af12e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa212e67, 77 },
    { 0xf1f12e67, 23 },
    { 0x25412e69, 61 },
    { 0xbb312e69, 95 },
    { 0x6712e67, 90 },
    { 0, 0 },
    { 0x46e12e6d, 90 },
    { 0, 0 },
    { 0xea812e6f, 90 },
    { 0xdd512e70, 23 },
    { 0x52b12e71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37812e7f, 95 },
    { 0xdb312e80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb12e87, 33 },
    { 0xc2c12e88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95512e8c, 23 },
    { 0x23812e8d, 19 },
    { 0xcdf12e8e, 66 },
    { 0x21d12e8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e012e9c, 95 },
    { 0, 0 },
    { 0x68c12e9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60012ea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90212ea7, 76 },
    { 0xea212ea8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba512eae, 66 },
    { 0x11c12eaf, 90 },
    { 0, 0 },
    { 0x2ad12eb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5312eb4, 90 },
    { 0, 0 },
    { 0x5e912eb6, 43 },
    { 0xfd712eb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4112eba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f812ebe, 55 },
    { 0x4dd12ebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80312ec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1912ec8, 61 },
    { 0x7c512ec8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc512ed1, 33 },
    { 0xb7912ed1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d12ed6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3512edc, 2 },
    { 0xd3112edd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2012ee2, 98 },
    { 0x7fd12ee3, 68 },
    { 0xdc912ee3, 43 },
    { 0, 0 },
    { 0x30712ee6, 33 },
    { 0xed312ee7, 98 },
    { 0xdf612ee7, 55 },
    { 0x7a012ee7, 91 },
    { 0xdb812ee8, 90 },
    { 0, 0 },
    { 0xf6812eec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30912ef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf112ef4, 43 },
    { 0x62c12ef5, 33 },
    { 0x79412ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d312ef9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c912efd, 43 },
    { 0, 0 },
    { 0xb7412eff, 55 },
    { 0x33a12f00, 52 },
    { 0x19012f01, 61 },
    { 0xd8712f01, 46 },
    { 0xd4312f01, 95 },
    { 0x90612f03, 90 },
    { 0x58012f05, 61 },
    { 0x82512f06, 90 },
    { 0xbea12f03, 76 },
    { 0xbbd12f08, 46 },
    { 0x97a12f04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79912f11, 61 },
    { 0x49e12f12, 91 },
    { 0x6e112f11, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a012f16, 90 },
    { 0x93612f17, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf12f1a, 66 },
    { 0, 0 },
    { 0xa3d12f1c, 91 },
    { 0, 0 },
    { 0xd5412f1e, 81 },
    { 0xf7512f1e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f12f24, 33 },
    { 0xb4b12f25, 43 },
    { 0xd7912f25, 23 },
    { 0x19112f26, 99 },
    { 0xc7212f27, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2212f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34712f34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca12f3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc112f42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d12f46, 91 },
    { 0x77c12f47, 95 },
    { 0xa4912f47, 75 },
    { 0xa7d12f47, 55 },
    { 0, 0 },
    { 0xa8b12f4b, 88 },
    { 0, 0 },
    { 0xeb112f4d, 93 },
    { 0x7d512f4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65512f55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9212f58, 90 },
    { 0x9ed12f59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e12f63, 90 },
    { 0x5a512f64, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55012f69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb12f6c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf12f73, 66 },
    { 0x85512f73, 33 },
    { 0, 0 },
    { 0xa2f12f76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41112f7c, 33 },
    { 0, 0 },
    { 0x3bf12f7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12f83, 77 },
    { 0, 0 },
    { 0xb4212f85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13712f88, 95 },
    { 0x95012f89, 98 },
    { 0x2d412f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a12f8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc12f91, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac12f95, 55 },
    { 0xadc12f96, 33 },
    { 0, 0 },
    { 0xf1d12f98, 95 },
    { 0xcb712f99, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f12fa0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39812fa4, 48 },
    { 0xcab12fa5, 61 },
    { 0, 0 },
    { 0xb2212fa7, 98 },
    { 0, 0 },
    { 0x71b12fa9, 80 },
    { 0x24012faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88312fae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81312fb1, 91 },
    { 0x46c12fb2, 43 },
    { 0x23012fb3, 90 },
    { 0, 0 },
    { 0xc1c12fb5, 83 },
    { 0, 0 },
    { 0x4de12fb7, 33 },
    { 0x5db12fb8, 97 },
    { 0xc7b12fb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c12fbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec12fc3, 99 },
    { 0xd2d12fc3, 95 },
    { 0, 0 },
    { 0x3612fc6, 93 },
    { 0x83012fc7, 90 },
    { 0x81412fc8, 61 },
    { 0x23f12fc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4712fcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15912fd3, 47 },
    { 0x11d12fd3, 98 },
    { 0xce912fd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce12fdb, 55 },
    { 0xf6912fdc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec12fe7, 34 },
    { 0xca712fe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f12feb, 17 },
    { 0x76a12feb, 61 },
    { 0xf5712fed, 95 },
    { 0x22d12fee, 77 },
    { 0x14212fef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2512ff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32012ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39412ff9, 95 },
    { 0, 0 },
    { 0x8612ffb, 61 },
    { 0xf6412ffc, 61 },
    { 0x72112ffd, 90 },
    { 0x2a712ffe, 83 },
    { 0xa312ffc, 33 },
    { 0xa8713000, 33 },
    { 0, 0 },
    { 0xed13002, 90 },
    { 0xc3513003, 66 },
    { 0, 0 },
    { 0xdb813005, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b01300a, 46 },
    { 0x3bd1300b, 91 },
    { 0x5241300c, 98 },
    { 0x48c1300b, 33 },
    { 0xab71300e, 23 },
    { 0x43a1300e, 90 },
    { 0x8691300a, 90 },
    { 0x1821300a, 40 },
    { 0x2f713011, 95 },
    { 0x7981300f, 90 },
    { 0xa0013014, 23 },
    { 0x26313014, 61 },
    { 0, 0 },
    { 0x6fd13017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c1301a, 99 },
    { 0xc1e1301b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd013021, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b13024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c13028, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21302b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7891302f, 46 },
    { 0xff41302f, 55 },
    { 0, 0 },
    { 0x12d13032, 95 },
    { 0, 0 },
    { 0xd8613034, 61 },
    { 0x6a813034, 98 },
    { 0x68b13036, 77 },
    { 0, 0 },
    { 0xb1113038, 33 },
    { 0xcee13039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa881303c, 67 },
    { 0xd7a1303d, 90 },
    { 0x9b01303e, 55 },
    { 0xb321303f, 90 },
    { 0xb1713040, 23 },
    { 0xbe713041, 66 },
    { 0x3b413042, 77 },
    { 0, 0 },
    { 0x9eb13044, 64 },
    { 0, 0 },
    { 0x73213046, 95 },
    { 0x46e13047, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c1304d, 94 },
    { 0xb481304e, 99 },
    { 0x4631304e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b413054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1305b, 61 },
    { 0, 0 },
    { 0xc1b1305d, 90 },
    { 0x73e1305e, 23 },
    { 0x6ed1305f, 55 },
    { 0, 0 },
    { 0x1113061, 97 },
    { 0x28913061, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813067, 90 },
    { 0xec213068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b11306b, 81 },
    { 0x7531306c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8941306f, 61 },
    { 0x55213070, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed13074, 90 },
    { 0x16d13075, 98 },
    { 0, 0 },
    { 0xbd813077, 90 },
    { 0, 0 },
    { 0x7b913079, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d1307d, 61 },
    { 0xe8c1307e, 91 },
    { 0x2d21307f, 33 },
    { 0x7401307e, 85 },
    { 0xe6c13081, 33 },
    { 0x8dd1307e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec213087, 33 },
    { 0, 0 },
    { 0x1c613089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa551308c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5613090, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7113099, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf991309d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523130a1, 95 },
    { 0x61130a2, 23 },
    { 0xd10130a3, 90 },
    { 0xc55130a3, 90 },
    { 0x986130a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba130a8, 77 },
    { 0, 0 },
    { 0xcb3130aa, 98 },
    { 0xe130aa, 23 },
    { 0x48b130ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be130b8, 34 },
    { 0, 0 },
    { 0xb50130ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7130be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30130c1, 61 },
    { 0xb87130c1, 43 },
    { 0xaed130c1, 90 },
    { 0, 0 },
    { 0x4b7130c5, 55 },
    { 0xc50130c6, 90 },
    { 0, 0 },
    { 0x9ff130c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff130cd, 76 },
    { 0xd95130cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27130d1, 85 },
    { 0xff9130d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef130d5, 76 },
    { 0, 0 },
    { 0x89130d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae130db, 90 },
    { 0, 0 },
    { 0x91e130dd, 78 },
    { 0xb1f130de, 33 },
    { 0x44b130df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31130e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393130e6, 91 },
    { 0x9a7130e7, 90 },
    { 0, 0 },
    { 0xc91130e9, 98 },
    { 0x2be130ea, 33 },
    { 0, 0 },
    { 0x893130ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb130f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa130f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc130f9, 46 },
    { 0x6c7130f9, 95 },
    { 0, 0 },
    { 0xce4130fc, 33 },
    { 0x4d9130fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a13100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68413104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5613107, 61 },
    { 0xa9013108, 75 },
    { 0, 0 },
    { 0x8721310a, 95 },
    { 0xba81310b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d913112, 23 },
    { 0xcbc13113, 98 },
    { 0x60a13114, 85 },
    { 0, 0 },
    { 0x3e113116, 55 },
    { 0, 0 },
    { 0xcf713118, 95 },
    { 0xcbd13119, 76 },
    { 0x7b013118, 33 },
    { 0xa7713118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a71311f, 33 },
    { 0xe491311f, 66 },
    { 0xdf213121, 90 },
    { 0, 0 },
    { 0x55b13123, 78 },
    { 0x66813123, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39513128, 76 },
    { 0xe6213129, 77 },
    { 0x8091312a, 91 },
    { 0, 0 },
    { 0x2791312c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7071312f, 46 },
    { 0xda13130, 76 },
    { 0x7dd13131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30613135, 99 },
    { 0xe8613136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d1313b, 98 },
    { 0xb121313c, 23 },
    { 0, 0 },
    { 0x311313e, 43 },
    { 0xcdf1313f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad1314a, 43 },
    { 0x62d1314b, 75 },
    { 0xbed1314c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3613151, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf513155, 95 },
    { 0x68a13155, 100 },
    { 0xb413157, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c1315d, 98 },
    { 0, 0 },
    { 0xa531315f, 33 },
    { 0x52b1315f, 43 },
    { 0, 0 },
    { 0x5b513162, 91 },
    { 0x91d13162, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2413167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851316a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf91316d, 23 },
    { 0x161316d, 33 },
    { 0x2dc1316d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55113173, 99 },
    { 0x94213174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2013177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81317d, 61 },
    { 0xd2d1317d, 61 },
    { 0xc721317d, 90 },
    { 0x5b313180, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7113184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13187, 33 },
    { 0x7f113188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a31318d, 33 },
    { 0xa851318e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c013192, 55 },
    { 0, 0 },
    { 0x75913194, 55 },
    { 0, 0 },
    { 0x66b13196, 90 },
    { 0xf2313197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378131a1, 90 },
    { 0x190131a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74131a5, 23 },
    { 0, 0 },
    { 0xb38131a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a131ab, 81 },
    { 0, 0 },
    { 0xfe8131ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c131b0, 23 },
    { 0, 0 },
    { 0xeb3131b2, 61 },
    { 0x83b131b2, 33 },
    { 0, 0 },
    { 0xc0c131b5, 66 },
    { 0x598131b6, 90 },
    { 0x471131b6, 93 },
    { 0, 0 },
    { 0x2d0131b9, 75 },
    { 0, 0 },
    { 0x512131bb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c131be, 23 },
    { 0, 0 },
    { 0x55d131c0, 93 },
    { 0x6f9131c1, 99 },
    { 0x9d3131c2, 23 },
    { 0x5bd131c2, 34 },
    { 0xe98131c3, 90 },
    { 0x1f3131c5, 34 },
    { 0x4b5131c6, 76 },
    { 0x336131c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26131cd, 90 },
    { 0, 0 },
    { 0x4b7131cf, 77 },
    { 0xf35131cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c131e5, 46 },
    { 0, 0 },
    { 0x6e4131e7, 95 },
    { 0xfed131e8, 67 },
    { 0xfb131e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1131ee, 90 },
    { 0x181131ef, 23 },
    { 0x2ad131f0, 23 },
    { 0xd94131ef, 93 },
    { 0x738131f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e131f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf131fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8513203, 90 },
    { 0xc813204, 91 },
    { 0, 0 },
    { 0x78b13206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e1320a, 33 },
    { 0xbbc1320a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac91320e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa613215, 23 },
    { 0, 0 },
    { 0xeaf13217, 95 },
    { 0x41513218, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2041321d, 23 },
    { 0, 0 },
    { 0x63f1321f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57613222, 33 },
    { 0x2bb13223, 95 },
    { 0x91813224, 23 },
    { 0xc6f13225, 33 },
    { 0x1fa13226, 34 },
    { 0xf0e13227, 61 },
    { 0x2aa13223, 90 },
    { 0x4913225, 95 },
    { 0x8a713226, 68 },
    { 0x71713225, 93 },
    { 0xfa91322a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b213230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69213234, 81 },
    { 0, 0 },
    { 0x10a13236, 46 },
    { 0x46613237, 90 },
    { 0xf6113237, 100 },
    { 0x60a13239, 23 },
    { 0xd0e1323a, 61 },
    { 0, 0 },
    { 0x2b51323c, 76 },
    { 0, 0 },
    { 0xb741323e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b213241, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff13244, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc961324a, 76 },
    { 0x5571324a, 36 },
    { 0x8501324c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc13253, 34 },
    { 0xb2a13254, 66 },
    { 0x16813255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c51325a, 34 },
    { 0xa971325a, 76 },
    { 0xe1b1325a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5451325f, 98 },
    { 0, 0 },
    { 0x86e13261, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b713264, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b13268, 91 },
    { 0, 0 },
    { 0x95c1326a, 34 },
    { 0x12b1326a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7421326f, 85 },
    { 0x19813270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e13273, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f1327b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d313280, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca13286, 76 },
    { 0, 0 },
    { 0x1d013288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5331328f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2313297, 76 },
    { 0, 0 },
    { 0xcf813299, 84 },
    { 0x5911329a, 34 },
    { 0, 0 },
    { 0x5711329c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de132a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d132a5, 59 },
    { 0x132132a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09132b4, 90 },
    { 0xc42132b5, 43 },
    { 0xff5132b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1132bb, 33 },
    { 0x5b3132bc, 84 },
    { 0x152132bd, 100 },
    { 0xbec132bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119132c2, 95 },
    { 0x701132c3, 34 },
    { 0x1dd132c4, 91 },
    { 0x275132c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c132ca, 98 },
    { 0, 0 },
    { 0xa55132cc, 23 },
    { 0x8d5132cc, 88 },
    { 0xa66132ce, 33 },
    { 0x931132cc, 38 },
    { 0xcd8132d0, 46 },
    { 0, 0 },
    { 0x56f132d2, 76 },
    { 0x755132d3, 98 },
    { 0xf30132d4, 85 },
    { 0xdc7132d4, 66 },
    { 0, 0 },
    { 0xcb3132d7, 66 },
    { 0x400132d8, 95 },
    { 0x449132d8, 33 },
    { 0xcb6132d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e132e3, 95 },
    { 0, 0 },
    { 0xcc132e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7132ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x707132f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2132fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f113303, 90 },
    { 0x16c13303, 34 },
    { 0, 0 },
    { 0x10b13306, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e613309, 87 },
    { 0xce1330a, 23 },
    { 0xe9d1330b, 25 },
    { 0x9201330c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b1330f, 43 },
    { 0x18613310, 34 },
    { 0xc813311, 23 },
    { 0xa4613312, 95 },
    { 0x51513313, 95 },
    { 0x29b13310, 23 },
    { 0x5241330f, 96 },
    { 0x74013312, 95 },
    { 0xa1a13310, 43 },
    { 0xc5213318, 90 },
    { 0x2fa13319, 99 },
    { 0x88f1331a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac11331e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b13325, 90 },
    { 0xbf13326, 43 },
    { 0xd8813326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8911332a, 85 },
    { 0x5e1332b, 66 },
    { 0xfa11332c, 61 },
    { 0xc5a1332d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xae113330, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37013334, 95 },
    { 0, 0 },
    { 0x59913336, 67 },
    { 0x13113337, 61 },
    { 0x18813338, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe11333b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd913341, 100 },
    { 0xa2313341, 95 },
    { 0x42c13343, 33 },
    { 0xad813343, 98 },
    { 0xfba13341, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe13348, 91 },
    { 0, 0 },
    { 0xd3d1334a, 76 },
    { 0x6971334a, 61 },
    { 0, 0 },
    { 0x74e1334d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f13359, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921335c, 98 },
    { 0x121335d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61413361, 95 },
    { 0x1d813362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa313365, 23 },
    { 0x34113366, 33 },
    { 0xee613366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc1336a, 95 },
    { 0x3d1336b, 66 },
    { 0x2a61336c, 99 },
    { 0x8ba1336c, 55 },
    { 0x6051336e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x213371, 90 },
    { 0x68513372, 23 },
    { 0x94c13373, 65 },
    { 0, 0 },
    { 0x5bf13375, 33 },
    { 0x22613376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90913379, 61 },
    { 0x8781337a, 68 },
    { 0x6a31337a, 95 },
    { 0xf401337a, 66 },
    { 0, 0 },
    { 0xa961337e, 43 },
    { 0xdb11337f, 93 },
    { 0xe0313380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae13385, 95 },
    { 0, 0 },
    { 0xa7d13387, 46 },
    { 0xc0c13388, 55 },
    { 0x8a913387, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe911338f, 87 },
    { 0, 0 },
    { 0x84613391, 91 },
    { 0, 0 },
    { 0xcd513393, 85 },
    { 0xd4c13394, 61 },
    { 0x6a013395, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31339a, 23 },
    { 0xc721339a, 61 },
    { 0xec71339a, 90 },
    { 0x2e61339d, 99 },
    { 0xf6d1339a, 43 },
    { 0, 0 },
    { 0x50d133a0, 98 },
    { 0xe48133a0, 80 },
    { 0x737133a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639133a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4133ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36133ae, 78 },
    { 0, 0 },
    { 0x424133b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a133b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb133bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf133c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd133c6, 99 },
    { 0xccd133c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3133ca, 91 },
    { 0, 0 },
    { 0x8a3133cc, 87 },
    { 0xa49133cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133d0, 33 },
    { 0xe59133d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3133d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6133da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c133e3, 23 },
    { 0, 0 },
    { 0x4d0133e5, 93 },
    { 0xc1e133e6, 52 },
    { 0xcca133e7, 95 },
    { 0, 0 },
    { 0x905133e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432133ed, 87 },
    { 0, 0 },
    { 0xa07133ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8133f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d133f7, 33 },
    { 0xa5c133f8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1713403, 88 },
    { 0x10113403, 55 },
    { 0xf1413403, 78 },
    { 0xb0113406, 91 },
    { 0x14513407, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3591340c, 73 },
    { 0xe1c1340c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c13415, 84 },
    { 0x47213415, 33 },
    { 0x43213417, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21341c, 55 },
    { 0xc141341c, 46 },
    { 0, 0 },
    { 0x6f41341f, 43 },
    { 0x9c41341f, 90 },
    { 0xc361341f, 90 },
    { 0x4ad13420, 98 },
    { 0, 0 },
    { 0xbf813424, 98 },
    { 0x36c13424, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92513428, 90 },
    { 0x64413429, 90 },
    { 0x26e13428, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c1342e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95413432, 34 },
    { 0, 0 },
    { 0x5f013434, 95 },
    { 0, 0 },
    { 0xc0313436, 40 },
    { 0xdec13436, 90 },
    { 0, 0 },
    { 0xe4813439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb591343f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4413442, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x74613445, 85 },
    { 0x95113446, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a61344a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61344d, 33 },
    { 0x9cd1344e, 69 },
    { 0x8341344f, 8 },
    { 0, 0 },
    { 0x59713451, 91 },
    { 0xe3913451, 93 },
    { 0xd6013453, 23 },
    { 0, 0 },
    { 0xba313455, 43 },
    { 0xd1913456, 55 },
    { 0x79b13457, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf981345f, 87 },
    { 0xaa21345f, 98 },
    { 0x61513461, 80 },
    { 0xf5213460, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f113465, 99 },
    { 0x26613466, 61 },
    { 0x66f13466, 23 },
    { 0x45b13465, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e31346d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89413470, 46 },
    { 0x34a13471, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9213476, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad13479, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf831347f, 95 },
    { 0x9981347f, 76 },
    { 0xfff13481, 61 },
    { 0x79b13482, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43313487, 76 },
    { 0x75f13488, 93 },
    { 0x6313489, 61 },
    { 0xfd013489, 33 },
    { 0xaad1348a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e31348e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35213493, 33 },
    { 0xea613493, 33 },
    { 0xf013495, 56 },
    { 0, 0 },
    { 0x2b613497, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf21349a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d134a0, 33 },
    { 0xdfd134a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949134ae, 61 },
    { 0xd67134af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd134b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86134b5, 61 },
    { 0xd58134b6, 33 },
    { 0xcc1134b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100134c0, 90 },
    { 0xf61134c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x930134c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81134c7, 61 },
    { 0x37c134c8, 95 },
    { 0x6a2134c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x841134cc, 76 },
    { 0x22c134cc, 99 },
    { 0x560134cd, 90 },
    { 0xfd2134cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x771134d2, 77 },
    { 0xe31134d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32134da, 33 },
    { 0x250134db, 33 },
    { 0x8df134dc, 91 },
    { 0xd30134dd, 98 },
    { 0xafe134dd, 90 },
    { 0x3ad134df, 76 },
    { 0xdb2134e0, 61 },
    { 0xfb5134df, 90 },
    { 0, 0 },
    { 0x8d2134e3, 91 },
    { 0x48134e3, 84 },
    { 0, 0 },
    { 0x590134e6, 33 },
    { 0, 0 },
    { 0x736134e8, 95 },
    { 0x10e134e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed134ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8134f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e113501, 55 },
    { 0xd5b13501, 61 },
    { 0xc4413502, 33 },
    { 0x69c13504, 34 },
    { 0x62913505, 90 },
    { 0x63b13506, 61 },
    { 0, 0 },
    { 0x1b213508, 33 },
    { 0xb1f13508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81013512, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0413515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b713519, 84 },
    { 0x181351a, 23 },
    { 0x8971351b, 98 },
    { 0xaee1351b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5051351f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f313526, 76 },
    { 0xcdc13527, 61 },
    { 0x33513528, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d1352b, 90 },
    { 0x6731352b, 23 },
    { 0xd091352c, 83 },
    { 0, 0 },
    { 0xd5f1352f, 43 },
    { 0, 0 },
    { 0xfd913531, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7013536, 90 },
    { 0, 0 },
    { 0xdfe13538, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d91354c, 87 },
    { 0xdde1354d, 93 },
    { 0xe3f1354d, 34 },
    { 0, 0 },
    { 0x3d513550, 53 },
    { 0xf0913551, 61 },
    { 0x8e513552, 34 },
    { 0x4a213552, 76 },
    { 0x8c213553, 55 },
    { 0, 0 },
    { 0xf0513556, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67113559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf971355e, 66 },
    { 0xb1f1355f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61013563, 34 },
    { 0xc6513564, 33 },
    { 0xab213565, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd051356f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36613573, 95 },
    { 0, 0 },
    { 0x1e113575, 61 },
    { 0, 0 },
    { 0x8c913577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1991357b, 34 },
    { 0x9e81357c, 44 },
    { 0x9841357d, 90 },
    { 0, 0 },
    { 0xb201357f, 95 },
    { 0xb4813580, 93 },
    { 0x58113581, 67 },
    { 0xe3413581, 98 },
    { 0x9a13583, 33 },
    { 0xabe13581, 98 },
    { 0x30113584, 23 },
    { 0x13713586, 98 },
    { 0xb5f13580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf071358b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb151358e, 43 },
    { 0, 0 },
    { 0x8df13590, 55 },
    { 0, 0 },
    { 0x10413592, 88 },
    { 0, 0 },
    { 0xb5313594, 95 },
    { 0xb3513595, 76 },
    { 0x9ab13596, 95 },
    { 0xc4613597, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221359d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded135a6, 23 },
    { 0xd10135a6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x78135aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800135b1, 90 },
    { 0xb41135b1, 90 },
    { 0x458135b3, 33 },
    { 0x501135b4, 84 },
    { 0x77f135b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b135b9, 66 },
    { 0x3b5135b9, 90 },
    { 0xacf135bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24135c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46135c4, 43 },
    { 0, 0 },
    { 0x4fd135c6, 61 },
    { 0x268135c6, 90 },
    { 0x2da135c8, 93 },
    { 0xf45135c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f135cd, 91 },
    { 0x79c135ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa135d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01135d9, 34 },
    { 0xefb135da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d135e2, 84 },
    { 0, 0 },
    { 0xa1b135e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba135e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb135ec, 61 },
    { 0, 0 },
    { 0xd30135ee, 23 },
    { 0xee6135ef, 98 },
    { 0x469135ef, 91 },
    { 0, 0 },
    { 0xb54135f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83135f6, 91 },
    { 0x246135f7, 90 },
    { 0x469135f8, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1135fb, 100 },
    { 0x6ba135fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca13607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1360c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1360f, 23 },
    { 0x9811360f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d13615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1361b, 91 },
    { 0x631361c, 23 },
    { 0x86a1361d, 23 },
    { 0, 0 },
    { 0x3cc1361f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11713623, 33 },
    { 0x4e513624, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a13627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d31362a, 52 },
    { 0, 0 },
    { 0xb331362c, 95 },
    { 0x4961362c, 52 },
    { 0, 0 },
    { 0xff81362f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f113632, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f13635, 23 },
    { 0xfcb13636, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761363a, 23 },
    { 0xd2a1363b, 98 },
    { 0, 0 },
    { 0x26e1363d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e413641, 76 },
    { 0x58613642, 98 },
    { 0xbab13642, 77 },
    { 0x92f13643, 90 },
    { 0, 0 },
    { 0x1b213646, 93 },
    { 0x7c813647, 60 },
    { 0xfef13648, 76 },
    { 0, 0 },
    { 0x5071364a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e013652, 46 },
    { 0xe3a13653, 67 },
    { 0xfc113652, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e13658, 90 },
    { 0x21e13659, 76 },
    { 0, 0 },
    { 0x4d11365b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a413660, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3613663, 34 },
    { 0x2d13664, 76 },
    { 0xf9e13663, 23 },
    { 0xc9013666, 23 },
    { 0xa2c13667, 55 },
    { 0x3cb13668, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20213672, 33 },
    { 0x6f513672, 29 },
    { 0xed913674, 90 },
    { 0, 0 },
    { 0xbcd13676, 93 },
    { 0x99713676, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c1367a, 23 },
    { 0x8121367b, 43 },
    { 0xcc41367c, 95 },
    { 0xfa91367d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4913682, 61 },
    { 0xb1213683, 33 },
    { 0, 0 },
    { 0x46a13685, 43 },
    { 0, 0 },
    { 0xdf313687, 85 },
    { 0xf813688, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d913690, 76 },
    { 0xcc513691, 33 },
    { 0xbf913692, 90 },
    { 0x64513693, 23 },
    { 0x86413692, 95 },
    { 0xe5913695, 61 },
    { 0, 0 },
    { 0x6da13697, 98 },
    { 0x89513698, 81 },
    { 0x43113698, 90 },
    { 0x76d13699, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f31369d, 77 },
    { 0, 0 },
    { 0x9511369f, 33 },
    { 0x348136a0, 90 },
    { 0x5e136a0, 65 },
    { 0xbb1136a1, 90 },
    { 0x29c136a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42136a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x438136aa, 98 },
    { 0xb5e136ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70136ae, 61 },
    { 0x1e6136af, 61 },
    { 0x961136b0, 76 },
    { 0x351136b0, 55 },
    { 0xb6136b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39136b9, 52 },
    { 0, 0 },
    { 0xc05136bb, 76 },
    { 0x7a2136bb, 46 },
    { 0x117136bb, 23 },
    { 0xaa0136bb, 95 },
    { 0x5df136bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120136c4, 90 },
    { 0x6cb136c5, 23 },
    { 0, 0 },
    { 0xf5f136c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6136ca, 93 },
    { 0xeed136cb, 34 },
    { 0, 0 },
    { 0x8c136cd, 33 },
    { 0x4d7136ce, 90 },
    { 0xce0136cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7136d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4136d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d136db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x285136de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84136e1, 23 },
    { 0x356136e1, 87 },
    { 0x33d136e1, 93 },
    { 0, 0 },
    { 0xea136e5, 34 },
    { 0xba7136e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d136ee, 55 },
    { 0x8cb136ee, 99 },
    { 0x583136ef, 43 },
    { 0, 0 },
    { 0xdcc136f2, 91 },
    { 0x3ee136f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242136f8, 95 },
    { 0x1ab136f9, 98 },
    { 0xc57136fa, 34 },
    { 0x8dc136f9, 93 },
    { 0, 0 },
    { 0xf7b136fd, 93 },
    { 0x41e136fe, 23 },
    { 0xe97136fe, 33 },
    { 0xc2d13700, 95 },
    { 0xf4913701, 61 },
    { 0xaf113701, 46 },
    { 0, 0 },
    { 0x81013704, 67 },
    { 0xb8f13705, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10513709, 52 },
    { 0x7641370a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77413710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29013713, 17 },
    { 0x8e013713, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27413717, 33 },
    { 0xc9e13718, 84 },
    { 0x76913717, 87 },
    { 0x7c41371a, 95 },
    { 0x6ac13719, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d1371e, 99 },
    { 0, 0 },
    { 0xdce13720, 88 },
    { 0x15d13720, 95 },
    { 0xcaf13721, 87 },
    { 0x12913722, 23 },
    { 0, 0 },
    { 0xa6913725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa13728, 76 },
    { 0xd8e13728, 90 },
    { 0x28c1372a, 61 },
    { 0xeed1372b, 87 },
    { 0x36b13728, 61 },
    { 0x44213728, 95 },
    { 0xbd61372c, 61 },
    { 0xbc013728, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99413733, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a513736, 55 },
    { 0xe9113737, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff1373a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71373d, 66 },
    { 0x4ca1373d, 46 },
    { 0x3631373f, 23 },
    { 0xd3e1373d, 34 },
    { 0xfbc13741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c13748, 98 },
    { 0, 0 },
    { 0x2c71374a, 67 },
    { 0xc1f1374b, 98 },
    { 0, 0 },
    { 0x1f21374d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d613751, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7513755, 93 },
    { 0x4ad13756, 90 },
    { 0x41f13757, 34 },
    { 0x1c13756, 23 },
    { 0xf8613758, 76 },
    { 0xbda1375a, 85 },
    { 0xe01375a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c13760, 90 },
    { 0xc213760, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1613764, 95 },
    { 0x90c13764, 67 },
    { 0, 0 },
    { 0xe5413767, 55 },
    { 0x6a813768, 86 },
    { 0, 0 },
    { 0xd821376a, 46 },
    { 0xef11376a, 38 },
    { 0xb501376c, 98 },
    { 0xfb61376a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf013770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x39813773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff13779, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68413780, 90 },
    { 0, 0 },
    { 0x40113782, 23 },
    { 0x87813783, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf13786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6651378e, 66 },
    { 0xed21378e, 61 },
    { 0xda31378e, 76 },
    { 0, 0 },
    { 0xdbe13792, 90 },
    { 0x4213793, 85 },
    { 0xf2513793, 55 },
    { 0, 0 },
    { 0x57a13796, 33 },
    { 0x1c613797, 85 },
    { 0xa1f13798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d1379c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d137a1, 34 },
    { 0, 0 },
    { 0x4a137a3, 95 },
    { 0x4bb137a3, 47 },
    { 0, 0 },
    { 0xa87137a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272137ab, 61 },
    { 0, 0 },
    { 0x32f137ad, 90 },
    { 0, 0 },
    { 0xd33137af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f137b7, 23 },
    { 0xa6137b8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42137bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4137c1, 52 },
    { 0xfcd137c2, 90 },
    { 0x9e9137c2, 43 },
    { 0x6bb137c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269137ca, 17 },
    { 0x481137cb, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac137ce, 76 },
    { 0xcd7137cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09137dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09137df, 69 },
    { 0x737137df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4137e3, 98 },
    { 0xfc5137e4, 34 },
    { 0xeb8137e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527137e9, 34 },
    { 0, 0 },
    { 0xbbf137eb, 23 },
    { 0xf1e137eb, 66 },
    { 0xb47137ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70137f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5137f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc137fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e313801, 90 },
    { 0xf6b13802, 90 },
    { 0, 0 },
    { 0xc0813804, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f13807, 33 },
    { 0x6d313808, 40 },
    { 0xaa813808, 43 },
    { 0x6e31380a, 95 },
    { 0, 0 },
    { 0xedb1380c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b13810, 34 },
    { 0, 0 },
    { 0x24a13812, 55 },
    { 0, 0 },
    { 0x68413814, 33 },
    { 0, 0 },
    { 0xd7a13816, 52 },
    { 0xd513816, 90 },
    { 0xa1413818, 87 },
    { 0x28b13818, 33 },
    { 0xed513819, 33 },
    { 0, 0 },
    { 0x80e1381c, 98 },
    { 0xd491381c, 81 },
    { 0x5051381c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46113823, 90 },
    { 0xaa713823, 90 },
    { 0, 0 },
    { 0x78c13826, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f13829, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb751382e, 65 },
    { 0x64c1382f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68113835, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xece13838, 33 },
    { 0, 0 },
    { 0x77b1383a, 90 },
    { 0xe791383b, 95 },
    { 0x3bf1383c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72313841, 91 },
    { 0x2f413842, 23 },
    { 0x1f113842, 61 },
    { 0xbc513844, 33 },
    { 0x2f313845, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57813858, 76 },
    { 0xd0413859, 67 },
    { 0x4641385a, 98 },
    { 0, 0 },
    { 0x1ea1385c, 90 },
    { 0xb131385c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a913866, 61 },
    { 0x60913866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e1386a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3213871, 23 },
    { 0x45613872, 77 },
    { 0xf4413873, 23 },
    { 0xdd413874, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d13877, 99 },
    { 0xfa113878, 68 },
    { 0xb0113879, 99 },
    { 0x4e13879, 55 },
    { 0, 0 },
    { 0x9091387c, 98 },
    { 0x4761387d, 95 },
    { 0x7d1387d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f13882, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1113886, 98 },
    { 0xe5613887, 52 },
    { 0xd0713887, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6391388d, 43 },
    { 0, 0 },
    { 0x6211388f, 95 },
    { 0x4f113890, 76 },
    { 0, 0 },
    { 0x22513892, 87 },
    { 0xb9c13892, 91 },
    { 0x60f13894, 23 },
    { 0x81113893, 85 },
    { 0, 0 },
    { 0xe9313897, 78 },
    { 0, 0 },
    { 0xfbf13899, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b81389e, 43 },
    { 0, 0 },
    { 0x872138a0, 66 },
    { 0x48138a0, 55 },
    { 0x1c6138a2, 33 },
    { 0xc0e138a0, 99 },
    { 0xb08138a4, 33 },
    { 0x643138a5, 33 },
    { 0, 0 },
    { 0xaf2138a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x514138aa, 95 },
    { 0x14138aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38138af, 66 },
    { 0xc49138b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67138b5, 23 },
    { 0xd43138b6, 93 },
    { 0x124138b5, 23 },
    { 0xacc138b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8138bb, 95 },
    { 0x54c138bc, 23 },
    { 0x6db138bc, 33 },
    { 0x2fc138bd, 34 },
    { 0x47a138bf, 66 },
    { 0xcf1138c0, 84 },
    { 0x15b138be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e138c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6138c8, 23 },
    { 0, 0 },
    { 0xddb138ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e138cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6138d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226138d5, 76 },
    { 0xb3a138d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x593138d9, 90 },
    { 0x3ab138da, 77 },
    { 0x5a0138d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2138e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c138e5, 78 },
    { 0xaaf138e6, 76 },
    { 0x96b138e7, 33 },
    { 0x7df138e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e138ed, 84 },
    { 0x66e138ee, 33 },
    { 0xcfd138ef, 46 },
    { 0xa52138ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x856138f3, 88 },
    { 0xd42138f3, 95 },
    { 0x5a8138f5, 61 },
    { 0x6dd138f6, 33 },
    { 0x270138f4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd813900, 76 },
    { 0xe813900, 99 },
    { 0x7c013902, 67 },
    { 0xf4c13900, 90 },
    { 0x59b13903, 23 },
    { 0x33913905, 98 },
    { 0x66813905, 23 },
    { 0, 0 },
    { 0x49113908, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f1390c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3013910, 95 },
    { 0x1f113910, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce413915, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc813918, 40 },
    { 0xfb613919, 90 },
    { 0xe9c1391a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b13921, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a13924, 43 },
    { 0xaf313925, 61 },
    { 0x28813926, 90 },
    { 0x2ec13926, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc1392d, 90 },
    { 0, 0 },
    { 0xbb81392f, 93 },
    { 0x3491392f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x22113933, 81 },
    { 0, 0 },
    { 0x13c13935, 90 },
    { 0, 0 },
    { 0x57d13937, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1393b, 95 },
    { 0xc431393b, 33 },
    { 0x1821393b, 23 },
    { 0xaca1393e, 46 },
    { 0xbd91393f, 98 },
    { 0x1cc1393e, 33 },
    { 0x3131393f, 99 },
    { 0xdde13942, 95 },
    { 0xea713943, 99 },
    { 0xef51393d, 76 },
    { 0x14313945, 43 },
    { 0, 0 },
    { 0xb6613947, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81394c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4313951, 95 },
    { 0, 0 },
    { 0x4ad13953, 23 },
    { 0xc1d13954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a313957, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd1395d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7313964, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c13967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2031396c, 90 },
    { 0xf1a1396d, 77 },
    { 0xf041396e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a513973, 93 },
    { 0, 0 },
    { 0x60913975, 69 },
    { 0xa113976, 95 },
    { 0x9ce13975, 99 },
    { 0, 0 },
    { 0x3ee13979, 55 },
    { 0, 0 },
    { 0x8741397b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43513982, 33 },
    { 0, 0 },
    { 0x3c13984, 90 },
    { 0x94d13985, 52 },
    { 0x5e213984, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1931398a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf1398d, 84 },
    { 0, 0 },
    { 0x7ba1398f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a13996, 90 },
    { 0xc1d13997, 61 },
    { 0x9b713996, 61 },
    { 0, 0 },
    { 0xb6b1399a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e1399d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6139a6, 90 },
    { 0, 0 },
    { 0x2a7139a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548139b6, 95 },
    { 0, 0 },
    { 0x9d4139b8, 91 },
    { 0x3ec139b8, 66 },
    { 0xc99139ba, 38 },
    { 0x3f3139ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2139bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775139c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b139c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661139ce, 93 },
    { 0xab9139ce, 90 },
    { 0xf0139cf, 98 },
    { 0x432139d1, 33 },
    { 0xeef139d1, 33 },
    { 0x308139d0, 66 },
    { 0, 0 },
    { 0x45a139d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39139dc, 61 },
    { 0x309139dd, 33 },
    { 0x208139dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8139e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a139eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595139f1, 23 },
    { 0x911139f1, 61 },
    { 0xcf2139f3, 99 },
    { 0x294139f3, 55 },
    { 0, 0 },
    { 0x81a139f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611139fd, 33 },
    { 0, 0 },
    { 0x84b139ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc13a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6013a19, 90 },
    { 0x43513a1a, 100 },
    { 0xcd613a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7113a21, 43 },
    { 0xc3013a21, 90 },
    { 0x79413a23, 93 },
    { 0x93713a24, 98 },
    { 0x47e13a25, 65 },
    { 0x4c213a26, 43 },
    { 0xf7913a21, 77 },
    { 0x26313a28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36213a2e, 55 },
    { 0, 0 },
    { 0xec813a30, 34 },
    { 0x59513a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24613a34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b13a39, 90 },
    { 0xc2413a3a, 98 },
    { 0x4bc13a3b, 23 },
    { 0x55213a3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16313a3f, 61 },
    { 0, 0 },
    { 0xe13a41, 99 },
    { 0x44e13a42, 61 },
    { 0x60613a43, 61 },
    { 0xb7c13a43, 23 },
    { 0x2e013a45, 90 },
    { 0xea013a46, 98 },
    { 0x15813a46, 88 },
    { 0, 0 },
    { 0x8cf13a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5013a4c, 23 },
    { 0x52513a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82813a52, 77 },
    { 0x6a713a53, 78 },
    { 0xc4713a54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f613a57, 99 },
    { 0xcad13a58, 99 },
    { 0xee813a59, 61 },
    { 0x2ac13a57, 23 },
    { 0x1c113a57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59113a60, 33 },
    { 0xf513a61, 23 },
    { 0x41013a61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20213a65, 46 },
    { 0, 0 },
    { 0xef513a67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45813a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63513a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd13a75, 61 },
    { 0, 0 },
    { 0x8a713a77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba013a7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4113a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d813a88, 55 },
    { 0x79a13a89, 66 },
    { 0x28d13a8a, 61 },
    { 0xacb13a8b, 91 },
    { 0x9eb13a8c, 95 },
    { 0x6b13a8d, 23 },
    { 0x90013a89, 84 },
    { 0xe4213a8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85613a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90613a9a, 34 },
    { 0xcfc13a9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d913a9f, 91 },
    { 0xe9713aa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19513aa3, 61 },
    { 0, 0 },
    { 0x34513aa5, 83 },
    { 0, 0 },
    { 0xb0d13aa7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c13aac, 76 },
    { 0x1fa13aad, 46 },
    { 0x30c13aae, 100 },
    { 0x35013aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14813ab3, 90 },
    { 0xe0d13ab3, 76 },
    { 0x51313ab5, 93 },
    { 0xb8a13ab5, 33 },
    { 0xd3b13ab7, 78 },
    { 0x8bd13ab7, 46 },
    { 0x9b913ab8, 98 },
    { 0xb2413ab8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26813abd, 61 },
    { 0x92a13abe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5813ac3, 95 },
    { 0, 0 },
    { 0xa3e13ac5, 33 },
    { 0x4a813ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76313acb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a113ad1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc13ad8, 34 },
    { 0, 0 },
    { 0x29d13ada, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7613ae2, 33 },
    { 0, 0 },
    { 0x1e913ae4, 95 },
    { 0x45113ae5, 33 },
    { 0, 0 },
    { 0x5e213ae7, 98 },
    { 0x22413ae8, 52 },
    { 0x22f13ae9, 90 },
    { 0x65413aea, 75 },
    { 0xe9413aea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17313aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea813af3, 33 },
    { 0, 0 },
    { 0x60c13af5, 98 },
    { 0xd0e13af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe813af9, 61 },
    { 0x8e313af9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e813b00, 46 },
    { 0, 0 },
    { 0xccc13b02, 84 },
    { 0x3bb13b03, 33 },
    { 0xbc713b04, 95 },
    { 0xe2513b03, 88 },
    { 0xdf413b06, 55 },
    { 0x25c13b07, 76 },
    { 0xd2213b07, 98 },
    { 0xfb013b09, 61 },
    { 0x1bd13b0a, 68 },
    { 0xc6413b06, 65 },
    { 0x1fc13b0b, 61 },
    { 0xd1813b0d, 33 },
    { 0xec413b0d, 98 },
    { 0xb2b13b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d13b14, 61 },
    { 0, 0 },
    { 0x87713b16, 90 },
    { 0xcef13b16, 55 },
    { 0xf1013b17, 23 },
    { 0x89413b16, 46 },
    { 0x69213b1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f013b21, 90 },
    { 0, 0 },
    { 0x33a13b23, 52 },
    { 0, 0 },
    { 0x87e13b25, 33 },
    { 0x2be13b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded13b2b, 46 },
    { 0xd7d13b2c, 33 },
    { 0xf1d13b2d, 52 },
    { 0, 0 },
    { 0x28c13b2f, 98 },
    { 0x64213b30, 84 },
    { 0x81113b2f, 84 },
    { 0x6f313b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61613b38, 53 },
    { 0, 0 },
    { 0xcee13b3a, 55 },
    { 0x49813b3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec13b40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d13b44, 55 },
    { 0, 0 },
    { 0xb1a13b46, 90 },
    { 0xfeb13b46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af13b4e, 67 },
    { 0, 0 },
    { 0x55613b50, 66 },
    { 0, 0 },
    { 0x80213b52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8313b55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47513b59, 61 },
    { 0x3dd13b59, 95 },
    { 0x1ee13b5b, 23 },
    { 0x413b5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e213b5f, 54 },
    { 0xb5d13b60, 98 },
    { 0, 0 },
    { 0xbd13b62, 98 },
    { 0x34f13b62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f413b69, 91 },
    { 0x41613b69, 96 },
    { 0xbfd13b6b, 90 },
    { 0, 0 },
    { 0xbb113b6d, 46 },
    { 0, 0 },
    { 0x4d713b6f, 90 },
    { 0, 0 },
    { 0xd7513b71, 23 },
    { 0x79e13b72, 33 },
    { 0x32813b72, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd13b76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6013b7e, 90 },
    { 0x15313b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38413b82, 23 },
    { 0, 0 },
    { 0x83113b84, 46 },
    { 0x9b013b84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f413b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b13b8f, 52 },
    { 0x6c813b8f, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf13b96, 76 },
    { 0, 0 },
    { 0xb2d13b98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83413ba0, 77 },
    { 0x85c13ba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a13ba6, 84 },
    { 0x95313ba7, 84 },
    { 0xf4713ba8, 61 },
    { 0, 0 },
    { 0xd8513baa, 55 },
    { 0x61813bab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb513bae, 68 },
    { 0x9d413baf, 55 },
    { 0, 0 },
    { 0xc3013bb1, 23 },
    { 0x8e13bb2, 46 },
    { 0, 0 },
    { 0xa9e13bb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1213bb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61313bbd, 90 },
    { 0xa8913bbe, 23 },
    { 0x36613bbf, 90 },
    { 0xbc613bc0, 46 },
    { 0x78c13bc1, 33 },
    { 0x5f13bbd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f13bc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd413bcb, 66 },
    { 0x3a413bcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a13bd2, 46 },
    { 0x20613bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2713bd6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca13bdf, 61 },
    { 0xe9513be0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe813be3, 52 },
    { 0xff813be4, 90 },
    { 0xa0213be3, 61 },
    { 0x9a13be6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde13bea, 90 },
    { 0x9c713beb, 90 },
    { 0x7aa13bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a13bf3, 90 },
    { 0x5013bf3, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a13bfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d513c05, 95 },
    { 0, 0 },
    { 0x37213c07, 76 },
    { 0x1e113c08, 100 },
    { 0x59513c09, 90 },
    { 0xb813c0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d013c0f, 76 },
    { 0xc6713c0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc13c13, 46 },
    { 0x43a13c14, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a713c17, 43 },
    { 0x98513c18, 46 },
    { 0xc8413c17, 91 },
    { 0x28113c1a, 95 },
    { 0, 0 },
    { 0x73e13c1c, 91 },
    { 0xeec13c1d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10813c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d13c29, 33 },
    { 0x7e813c2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72113c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9513c3a, 93 },
    { 0xe4913c3a, 23 },
    { 0xe7713c3a, 33 },
    { 0xc913c3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b13c40, 99 },
    { 0, 0 },
    { 0x11813c42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71213c48, 55 },
    { 0, 0 },
    { 0xb4b13c4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b13c4e, 55 },
    { 0x4e413c4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d13c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46013c60, 61 },
    { 0, 0 },
    { 0x27f13c62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a513c69, 17 },
    { 0x1b113c6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46013c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e13c74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8313c7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a13c7e, 90 },
    { 0, 0 },
    { 0xe9f13c80, 84 },
    { 0x71213c81, 75 },
    { 0x68413c81, 84 },
    { 0xc0713c82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60813c86, 33 },
    { 0xafd13c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63613c8a, 93 },
    { 0xf0213c8a, 95 },
    { 0x26413c8c, 34 },
    { 0x4ff13c8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b13c91, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0913c94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9713c98, 55 },
    { 0x16b13c99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45013c9f, 23 },
    { 0xbcb13c9f, 90 },
    { 0, 0 },
    { 0x16f13ca2, 90 },
    { 0x9c13ca2, 46 },
    { 0x113ca4, 36 },
    { 0x89913ca5, 33 },
    { 0x3af13ca5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48813cb0, 87 },
    { 0x42e13cb1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c13cb6, 93 },
    { 0x99b13cb7, 90 },
    { 0, 0 },
    { 0x68913cb9, 90 },
    { 0, 0 },
    { 0x10813cbb, 61 },
    { 0, 0 },
    { 0x70b13cbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb13cc2, 90 },
    { 0, 0 },
    { 0x16e13cc4, 56 },
    { 0, 0 },
    { 0x90713cc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2513ccc, 90 },
    { 0x70e13ccd, 91 },
    { 0x57513ccc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80113cd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b13cd4, 100 },
    { 0xc8b13cd5, 87 },
    { 0x70613cd6, 34 },
    { 0xea013cd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9913cdd, 61 },
    { 0xa2e13cde, 46 },
    { 0xf7c13cdf, 76 },
    { 0, 0 },
    { 0xa9013ce1, 98 },
    { 0x3b113ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e13ce5, 98 },
    { 0x3c613ce6, 67 },
    { 0x3e413ce5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1613cea, 95 },
    { 0, 0 },
    { 0xd4e13cec, 98 },
    { 0, 0 },
    { 0xed713cee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a113cf1, 99 },
    { 0, 0 },
    { 0x93b13cf3, 76 },
    { 0x21a13cf4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb113cf7, 95 },
    { 0xcf713cf8, 90 },
    { 0xbe13cf8, 23 },
    { 0, 0 },
    { 0x6f413cfb, 78 },
    { 0x90213cfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d13d01, 76 },
    { 0, 0 },
    { 0xe3f13d03, 65 },
    { 0xac813d04, 90 },
    { 0x2ec13d04, 46 },
    { 0, 0 },
    { 0x96d13d07, 98 },
    { 0xe9513d08, 93 },
    { 0x4ca13d09, 38 },
    { 0xb8613d0a, 33 },
    { 0, 0 },
    { 0x8cd13d0c, 61 },
    { 0x1f113d0d, 95 },
    { 0x48513d0e, 87 },
    { 0x55113d0e, 95 },
    { 0xac13d0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a13d13, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82413d17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62113d1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad13d1e, 78 },
    { 0x32813d1f, 91 },
    { 0x50013d1e, 61 },
    { 0, 0 },
    { 0x2a13d22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a13d25, 61 },
    { 0, 0 },
    { 0x7f113d27, 33 },
    { 0xf3913d28, 61 },
    { 0xaa013d28, 99 },
    { 0xbd813d2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b113d2e, 95 },
    { 0, 0 },
    { 0x8e813d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda713d33, 61 },
    { 0, 0 },
    { 0x63c13d35, 33 },
    { 0, 0 },
    { 0xd4113d37, 90 },
    { 0xfa513d38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86413d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b13d3f, 40 },
    { 0, 0 },
    { 0x5f313d41, 33 },
    { 0xe5a13d42, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0813d46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f113d49, 90 },
    { 0x36113d4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c413d4e, 46 },
    { 0xd2813d4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x15413d52, 98 },
    { 0x9f213d53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce13d57, 33 },
    { 0x9c13d57, 66 },
    { 0x37813d58, 65 },
    { 0xcd213d57, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f13d5d, 91 },
    { 0x36a13d5d, 33 },
    { 0x38813d5f, 91 },
    { 0x80613d60, 91 },
    { 0, 0 },
    { 0xdf713d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd13d65, 96 },
    { 0x96513d66, 55 },
    { 0x36913d67, 52 },
    { 0x6bd13d68, 95 },
    { 0x65c13d69, 23 },
    { 0, 0 },
    { 0x84a13d6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce113d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78913d71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72613d74, 84 },
    { 0, 0 },
    { 0x50a13d76, 93 },
    { 0xea513d77, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41013d7b, 68 },
    { 0, 0 },
    { 0x49c13d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88513d84, 90 },
    { 0x7cf13d84, 88 },
    { 0x42d13d86, 34 },
    { 0x31e13d86, 100 },
    { 0x9c513d86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad13d8c, 55 },
    { 0x1db13d8c, 90 },
    { 0x6213d8e, 90 },
    { 0x10713d8f, 33 },
    { 0, 0 },
    { 0x84413d91, 99 },
    { 0, 0 },
    { 0xb5213d93, 98 },
    { 0xb1a13d94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e13d98, 84 },
    { 0x13c13d99, 36 },
    { 0, 0 },
    { 0x7a413d9b, 90 },
    { 0x59213d9c, 66 },
    { 0x4bf13d9d, 43 },
    { 0x24f13d9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee13da3, 95 },
    { 0xdb113da3, 55 },
    { 0x12513da5, 67 },
    { 0, 0 },
    { 0xdf013da7, 100 },
    { 0, 0 },
    { 0xeed13da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3313dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e013db3, 95 },
    { 0, 0 },
    { 0x81b13db5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a13dbb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc613dc7, 67 },
    { 0x18b13dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f13dcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24613dd1, 43 },
    { 0xa2d13dd1, 23 },
    { 0x75713dd3, 61 },
    { 0x63413dd4, 61 },
    { 0x5a413dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd13ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1513ddf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b13de9, 90 },
    { 0, 0 },
    { 0x46813deb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36613dee, 33 },
    { 0, 0 },
    { 0xe1f13df0, 61 },
    { 0x75e13df1, 76 },
    { 0xd013df2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc13df7, 90 },
    { 0xa5a13df8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec013dfe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb13e04, 93 },
    { 0x63413e04, 55 },
    { 0, 0 },
    { 0x97413e07, 83 },
    { 0, 0 },
    { 0xb2713e09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34713e0d, 90 },
    { 0xcd613e0e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db13e11, 88 },
    { 0xba013e11, 33 },
    { 0x11413e11, 90 },
    { 0x5b913e13, 66 },
    { 0xdf613e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa113e18, 76 },
    { 0xf4513e19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a513e1c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a713e1f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4113e22, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55813e29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee13e2c, 98 },
    { 0xeae13e2c, 61 },
    { 0, 0 },
    { 0x5cb13e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee13e36, 66 },
    { 0, 0 },
    { 0xaf113e38, 23 },
    { 0xe9013e39, 76 },
    { 0xea413e3a, 78 },
    { 0x79b13e3b, 99 },
    { 0xe2513e39, 61 },
    { 0xcd113e3d, 23 },
    { 0xa9213e3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed13e43, 23 },
    { 0x4d913e44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4213e48, 90 },
    { 0, 0 },
    { 0xe5113e4a, 55 },
    { 0x73d13e4b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c13e4e, 95 },
    { 0xa9413e4f, 19 },
    { 0x30813e50, 90 },
    { 0x59f13e50, 90 },
    { 0x31e13e50, 33 },
    { 0x31313e53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27613e56, 34 },
    { 0x2c313e57, 98 },
    { 0xe2013e57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c13e5c, 67 },
    { 0, 0 },
    { 0x8cd13e5e, 43 },
    { 0, 0 },
    { 0xdd313e60, 90 },
    { 0, 0 },
    { 0x4413e62, 33 },
    { 0x63c13e63, 99 },
    { 0xdb513e62, 95 },
    { 0, 0 },
    { 0x2ab13e66, 90 },
    { 0, 0 },
    { 0x8f513e68, 85 },
    { 0, 0 },
    { 0x84b13e6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc813e71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d013e75, 98 },
    { 0, 0 },
    { 0x25a13e77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51713e7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0113e85, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc513e89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45713e8d, 68 },
    { 0x70513e8e, 98 },
    { 0xdbb13e8e, 56 },
    { 0x30813e90, 61 },
    { 0x65613e90, 33 },
    { 0xee513e92, 55 },
    { 0xea313e93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d113e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e13e9c, 23 },
    { 0xd5713e9c, 90 },
    { 0, 0 },
    { 0x3f013e9f, 61 },
    { 0x5a413e9f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e13ea5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d113ea8, 100 },
    { 0x80413ea9, 78 },
    { 0x68a13eaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6113eaf, 61 },
    { 0, 0 },
    { 0x73613eb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35513eb9, 91 },
    { 0x17d13eba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x42613ebd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e613ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13ec5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a013eca, 90 },
    { 0x7c13eca, 76 },
    { 0x75e13ecc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1713ed4, 95 },
    { 0x2c813ed5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6513ede, 43 },
    { 0, 0 },
    { 0xfd13ee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85213ee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c13ee6, 98 },
    { 0xaff13ee7, 90 },
    { 0x87f13ee6, 55 },
    { 0x59313ee9, 61 },
    { 0x7f313ee6, 66 },
    { 0x8b913eeb, 99 },
    { 0x6f713ee9, 89 },
    { 0, 0 },
    { 0xf2c13eee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8913ef1, 88 },
    { 0xde213ef2, 90 },
    { 0x79113ef2, 88 },
    { 0x7d713ef4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c413ef8, 95 },
    { 0, 0 },
    { 0xac113efa, 99 },
    { 0xfc113efb, 61 },
    { 0xeb913efa, 34 },
    { 0, 0 },
    { 0xd5813efe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80313f02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e13f06, 34 },
    { 0x52113f06, 33 },
    { 0xb2013f08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c13f12, 66 },
    { 0, 0 },
    { 0x2513f14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c13f1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a13f24, 46 },
    { 0x5c113f24, 75 },
    { 0, 0 },
    { 0x2d413f27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36113f2a, 90 },
    { 0xc5a13f2b, 90 },
    { 0x5c413f2c, 95 },
    { 0, 0 },
    { 0xa413f2e, 61 },
    { 0xd1413f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8113f33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51313f39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e613f3c, 90 },
    { 0xd2313f3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29213f42, 76 },
    { 0, 0 },
    { 0x15f13f44, 33 },
    { 0, 0 },
    { 0xff213f46, 84 },
    { 0x91013f47, 95 },
    { 0x2113f48, 17 },
    { 0x3f613f48, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4013f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9113f4f, 33 },
    { 0x3aa13f50, 34 },
    { 0x3ac13f51, 61 },
    { 0x7b613f51, 95 },
    { 0, 0 },
    { 0xbaa13f54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4613f58, 80 },
    { 0x78013f58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c513f5f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d13f62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22413f65, 91 },
    { 0, 0 },
    { 0x8db13f67, 95 },
    { 0x7f713f68, 76 },
    { 0x4cf13f68, 61 },
    { 0xb2113f6a, 33 },
    { 0x9ee13f6b, 33 },
    { 0x62f13f69, 65 },
    { 0x99513f6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d13f72, 33 },
    { 0xd2913f72, 61 },
    { 0, 0 },
    { 0xe7c13f75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7013f79, 84 },
    { 0x2ea13f7a, 78 },
    { 0, 0 },
    { 0x54513f7c, 33 },
    { 0x2e713f7d, 46 },
    { 0xe9913f7e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e13f81, 76 },
    { 0xae13f82, 100 },
    { 0xc1013f82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77813f88, 65 },
    { 0x8bf13f89, 17 },
    { 0x21d13f8a, 61 },
    { 0x75013f8b, 46 },
    { 0x44f13f8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44213f90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf413f95, 59 },
    { 0, 0 },
    { 0xf6b13f97, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed13f9c, 55 },
    { 0x53413f9d, 93 },
    { 0x36313f9e, 84 },
    { 0xd4a13f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19213fa2, 61 },
    { 0, 0 },
    { 0xfa613fa4, 90 },
    { 0x9b713fa5, 33 },
    { 0xfe513fa4, 61 },
    { 0, 0 },
    { 0xa7a13fa8, 52 },
    { 0x9dc13fa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea913fac, 46 },
    { 0, 0 },
    { 0x37413fae, 2 },
    { 0, 0 },
    { 0x50213fb0, 33 },
    { 0x57f13fb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c13fb5, 61 },
    { 0, 0 },
    { 0x54d13fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38013fbf, 61 },
    { 0, 0 },
    { 0x1313fc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5713fc5, 90 },
    { 0xde213fc6, 61 },
    { 0xe2c13fc6, 95 },
    { 0xd8213fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5113fcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c13fd2, 77 },
    { 0xa5713fd3, 98 },
    { 0, 0 },
    { 0x3c13fd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf313fdd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d013fe7, 78 },
    { 0xe9213fe8, 90 },
    { 0xf5113fe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c13fec, 61 },
    { 0xfba13fec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc513ff0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75113ffc, 76 },
    { 0xe5113ffd, 34 },
    { 0x98613ffe, 23 },
    { 0, 0 },
    { 0x81714000, 99 },
    { 0, 0 },
    { 0xae714002, 93 },
    { 0x30414003, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81314006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe14009, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc21400d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71714010, 48 },
    { 0, 0 },
    { 0x9d614012, 95 },
    { 0, 0 },
    { 0x21a14014, 98 },
    { 0x81514015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe14019, 61 },
    { 0xb311401a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e14021, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c714024, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a814027, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc71402b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a1402e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e14031, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac314034, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x34214037, 95 },
    { 0x16414038, 90 },
    { 0xaae14038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd1403c, 84 },
    { 0, 0 },
    { 0xc601403e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88714047, 66 },
    { 0x95a14048, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b71404c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c714050, 75 },
    { 0x11514051, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef014056, 90 },
    { 0, 0 },
    { 0xd4a14058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8681405b, 93 },
    { 0xdb11405c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd14061, 90 },
    { 0, 0 },
    { 0xf6c14063, 17 },
    { 0x34d14064, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a1406d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe314071, 23 },
    { 0x95414071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90914078, 90 },
    { 0, 0 },
    { 0x6cd1407a, 84 },
    { 0, 0 },
    { 0x2031407c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b14082, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf111408c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34414091, 76 },
    { 0, 0 },
    { 0xba714093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb14097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab1409b, 90 },
    { 0x9e41409b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191409f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935140a3, 93 },
    { 0, 0 },
    { 0x812140a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09140aa, 23 },
    { 0, 0 },
    { 0x63a140ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40140b0, 33 },
    { 0xbf1140b0, 17 },
    { 0, 0 },
    { 0x517140b3, 95 },
    { 0xab1140b3, 95 },
    { 0, 0 },
    { 0x840140b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10140ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326140bf, 66 },
    { 0xad7140c0, 90 },
    { 0xd00140c1, 61 },
    { 0xe54140bf, 43 },
    { 0, 0 },
    { 0x4dc140c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee140c8, 23 },
    { 0x573140c9, 100 },
    { 0x3c6140ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863140cf, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24140d6, 33 },
    { 0, 0 },
    { 0x21c140d8, 69 },
    { 0xa8140d9, 91 },
    { 0xe44140da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf140dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa140e4, 23 },
    { 0, 0 },
    { 0x214140e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4140ea, 61 },
    { 0x9a6140ea, 90 },
    { 0xac8140ec, 98 },
    { 0xd17140eb, 95 },
    { 0x8b1140ee, 98 },
    { 0x9c9140ef, 98 },
    { 0xee6140ec, 61 },
    { 0xdfb140f1, 100 },
    { 0xdc2140f2, 33 },
    { 0x3b140f1, 95 },
    { 0x8ab140ed, 91 },
    { 0, 0 },
    { 0x8c140f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e140fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a14109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc461410c, 90 },
    { 0x57f1410d, 100 },
    { 0xc271410e, 95 },
    { 0x8e31410f, 61 },
    { 0xb181410e, 33 },
    { 0xa0914111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2214115, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba51411b, 33 },
    { 0, 0 },
    { 0x56c1411d, 43 },
    { 0x2d31411e, 95 },
    { 0xbcf1411e, 33 },
    { 0x59f14120, 90 },
    { 0, 0 },
    { 0x7f814122, 46 },
    { 0, 0 },
    { 0x78214124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5514127, 84 },
    { 0x94014128, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b01412d, 90 },
    { 0xc0b1412e, 91 },
    { 0x3001412f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e21413a, 100 },
    { 0xcd11413b, 33 },
    { 0x47f1413b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8114143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91414b, 65 },
    { 0x4ef1414c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2931414f, 90 },
    { 0, 0 },
    { 0x18f14151, 76 },
    { 0, 0 },
    { 0x32414153, 23 },
    { 0, 0 },
    { 0x9414155, 61 },
    { 0x9b414155, 43 },
    { 0xde314157, 78 },
    { 0x7814158, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771415b, 33 },
    { 0x9941415c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c14161, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab014166, 23 },
    { 0x17e14167, 84 },
    { 0xaf814168, 33 },
    { 0x73f14169, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8791416c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e21416f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19414172, 23 },
    { 0, 0 },
    { 0x77b14174, 85 },
    { 0xfe14174, 23 },
    { 0xb8914174, 43 },
    { 0x31d14174, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d14180, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f14187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e11418a, 61 },
    { 0, 0 },
    { 0xb9c1418c, 46 },
    { 0, 0 },
    { 0xb511418e, 33 },
    { 0x3e61418f, 55 },
    { 0xd6914190, 33 },
    { 0x8c71418f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a14196, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a914199, 90 },
    { 0xc171419a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7141a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6141a5, 76 },
    { 0x4b0141a6, 82 },
    { 0xe4b141a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb141ab, 61 },
    { 0xea0141ac, 90 },
    { 0x329141ad, 98 },
    { 0x59a141ad, 55 },
    { 0x23f141af, 65 },
    { 0x829141af, 80 },
    { 0, 0 },
    { 0x4ed141b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x33141b5, 34 },
    { 0xd70141b5, 90 },
    { 0x4a141b6, 43 },
    { 0x5b4141b8, 61 },
    { 0xb3c141b8, 33 },
    { 0x9c6141ba, 98 },
    { 0xf14141bb, 88 },
    { 0x6af141bc, 86 },
    { 0xfba141bc, 23 },
    { 0x20d141be, 67 },
    { 0x33f141b5, 61 },
    { 0xa06141b8, 99 },
    { 0, 0 },
    { 0x5ef141c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c141c5, 96 },
    { 0x113141c6, 98 },
    { 0xffc141c7, 96 },
    { 0, 0 },
    { 0xde4141c9, 98 },
    { 0xd30141c9, 34 },
    { 0, 0 },
    { 0xf51141cc, 67 },
    { 0, 0 },
    { 0xd7f141ce, 55 },
    { 0x9cc141ce, 93 },
    { 0x2bc141ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba141db, 96 },
    { 0x7e0141db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd141df, 93 },
    { 0, 0 },
    { 0xe2a141e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c141eb, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504141f6, 75 },
    { 0x836141f7, 61 },
    { 0xd61141f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a141fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb1420b, 90 },
    { 0, 0 },
    { 0x1d81420d, 98 },
    { 0xa091420e, 90 },
    { 0x3271420d, 55 },
    { 0, 0 },
    { 0xeb214211, 23 },
    { 0x8014212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79014222, 67 },
    { 0xfce14223, 91 },
    { 0, 0 },
    { 0xb0a14225, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe14229, 61 },
    { 0, 0 },
    { 0x7081422b, 61 },
    { 0x1a71422c, 55 },
    { 0xf2c1422d, 98 },
    { 0xc231422d, 33 },
    { 0x63a1422f, 93 },
    { 0xe0414230, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a14234, 61 },
    { 0xa8214235, 95 },
    { 0x32b14236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5114239, 24 },
    { 0x5d81423a, 34 },
    { 0x58c14239, 98 },
    { 0x341423c, 23 },
    { 0x43d1423d, 43 },
    { 0, 0 },
    { 0xf0d1423f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40414245, 33 },
    { 0, 0 },
    { 0x5c714247, 23 },
    { 0xad014247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee71424e, 69 },
    { 0x1701424f, 98 },
    { 0xe2214250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5114253, 34 },
    { 0, 0 },
    { 0x37714255, 61 },
    { 0, 0 },
    { 0xb6514257, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb1425d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2714267, 81 },
    { 0xa8e14268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd61426c, 80 },
    { 0xb871426d, 91 },
    { 0x5d41426e, 75 },
    { 0, 0 },
    { 0x19614270, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc14273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1014278, 80 },
    { 0x96f14278, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c14281, 90 },
    { 0, 0 },
    { 0xfef14283, 55 },
    { 0x57614284, 95 },
    { 0xe4714285, 61 },
    { 0xbd814286, 61 },
    { 0x26a14284, 98 },
    { 0x91914288, 34 },
    { 0xa7f14283, 91 },
    { 0x16b14284, 55 },
    { 0x2531428b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b1428f, 85 },
    { 0x5da1428f, 61 },
    { 0x64714291, 98 },
    { 0x6c614291, 61 },
    { 0x2d814293, 95 },
    { 0, 0 },
    { 0x3e014295, 88 },
    { 0x47414295, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81429b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f142a0, 66 },
    { 0x94a142a0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975142a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a142ad, 55 },
    { 0x18d142ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b142b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab142ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51142bf, 33 },
    { 0x751142c0, 78 },
    { 0xac142bf, 66 },
    { 0, 0 },
    { 0x855142c3, 78 },
    { 0, 0 },
    { 0xf7f142c5, 66 },
    { 0x2c142c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739142cb, 76 },
    { 0xd17142cc, 33 },
    { 0x75e142cd, 93 },
    { 0, 0 },
    { 0x24b142cf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f142d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda142d9, 90 },
    { 0x828142da, 76 },
    { 0x748142db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67142e2, 95 },
    { 0x6cc142e3, 98 },
    { 0x3d8142e2, 33 },
    { 0xc90142e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585142eb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613142f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a142f9, 34 },
    { 0x236142f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb142ff, 34 },
    { 0x9db142ff, 33 },
    { 0x8cf14301, 65 },
    { 0, 0 },
    { 0x48714303, 100 },
    { 0, 0 },
    { 0xdb114305, 34 },
    { 0x6a414306, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f14309, 98 },
    { 0xda114309, 23 },
    { 0xac21430b, 33 },
    { 0xd5a1430c, 17 },
    { 0xb431430d, 98 },
    { 0xdd21430e, 98 },
    { 0x4601430b, 67 },
    { 0, 0 },
    { 0x8d814311, 33 },
    { 0x76314312, 33 },
    { 0, 0 },
    { 0x58c14314, 33 },
    { 0x1f514315, 33 },
    { 0, 0 },
    { 0xcdd14317, 66 },
    { 0x4614318, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb791431b, 61 },
    { 0, 0 },
    { 0xf0f1431d, 90 },
    { 0x98c1431e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9314322, 98 },
    { 0xb4614323, 33 },
    { 0xf914323, 98 },
    { 0, 0 },
    { 0x12714326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac71432b, 76 },
    { 0x66b1432b, 61 },
    { 0, 0 },
    { 0xef91432e, 68 },
    { 0x7901432e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67714333, 88 },
    { 0x73d14334, 33 },
    { 0x35614335, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc311433b, 34 },
    { 0x12d1433c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5414340, 23 },
    { 0x52514341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24614345, 34 },
    { 0xaa414346, 98 },
    { 0xde314347, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf01434b, 90 },
    { 0x9ec1434c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd14350, 23 },
    { 0xe1114351, 33 },
    { 0xb1d14350, 91 },
    { 0x5c314353, 61 },
    { 0x6fc14354, 87 },
    { 0x7b814353, 87 },
    { 0xc4b14356, 100 },
    { 0x23314357, 23 },
    { 0xc0a14353, 23 },
    { 0xe4014351, 34 },
    { 0x35114350, 95 },
    { 0x61f14350, 95 },
    { 0xadd14353, 46 },
    { 0xeb31435d, 33 },
    { 0xa141435d, 61 },
    { 0xde31435f, 90 },
    { 0x3ce14360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24814363, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3714368, 90 },
    { 0, 0 },
    { 0xe01436a, 99 },
    { 0, 0 },
    { 0xd5a1436c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb014371, 61 },
    { 0, 0 },
    { 0x90f14373, 90 },
    { 0xe2714374, 34 },
    { 0x2be14373, 17 },
    { 0x23414376, 87 },
    { 0, 0 },
    { 0xf1d14378, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c1437c, 95 },
    { 0xcef1437d, 95 },
    { 0x8311437e, 78 },
    { 0, 0 },
    { 0xc514380, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x21714383, 95 },
    { 0, 0 },
    { 0x1bb14385, 33 },
    { 0x1ec14386, 69 },
    { 0x40814385, 91 },
    { 0x8da14388, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf21438b, 66 },
    { 0xd2f1438c, 40 },
    { 0x66a1438d, 61 },
    { 0xa1f1438d, 66 },
    { 0xf4e1438f, 46 },
    { 0xdd214390, 23 },
    { 0xdb314390, 98 },
    { 0x5a71438f, 33 },
    { 0x22714393, 61 },
    { 0x3ec14393, 76 },
    { 0x8d514394, 98 },
    { 0xd0914392, 85 },
    { 0xa4a1438e, 84 },
    { 0xd0714390, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b1439f, 76 },
    { 0x82f1439f, 33 },
    { 0xc01143a1, 96 },
    { 0, 0 },
    { 0xb96143a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790143ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b143b7, 90 },
    { 0, 0 },
    { 0x76c143b9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb143bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad143c2, 33 },
    { 0xf44143c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x750143c6, 95 },
    { 0x4d5143c6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e143ca, 99 },
    { 0x77c143cb, 61 },
    { 0xbd143cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x784143cf, 34 },
    { 0x532143cf, 90 },
    { 0, 0 },
    { 0x3b6143d2, 90 },
    { 0, 0 },
    { 0xedf143d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96143da, 55 },
    { 0, 0 },
    { 0x693143dc, 33 },
    { 0x157143dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3143e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7143e9, 98 },
    { 0x3f5143ea, 78 },
    { 0, 0 },
    { 0x607143ec, 98 },
    { 0x46b143ec, 98 },
    { 0xd0f143ee, 52 },
    { 0xdf9143ef, 88 },
    { 0x257143f0, 90 },
    { 0x483143f1, 93 },
    { 0xc9c143f2, 90 },
    { 0x6e7143f3, 98 },
    { 0xad5143f1, 26 },
    { 0xbe5143f5, 48 },
    { 0xf87143f6, 90 },
    { 0xda5143f7, 90 },
    { 0x89a143f8, 23 },
    { 0x404143f1, 90 },
    { 0xafb143fa, 34 },
    { 0x576143fb, 95 },
    { 0, 0 },
    { 0x666143fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b14402, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f814406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821440a, 98 },
    { 0, 0 },
    { 0x2c91440c, 95 },
    { 0x77f1440d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d14414, 90 },
    { 0, 0 },
    { 0xa114416, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x52014419, 78 },
    { 0x6991441a, 23 },
    { 0xf341441b, 93 },
    { 0, 0 },
    { 0x74a1441d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35914421, 21 },
    { 0, 0 },
    { 0, 0 },
    { 0x714424, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b814427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf071442a, 42 },
    { 0, 0 },
    { 0x7d31442c, 90 },
    { 0x8a91442c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a114432, 98 },
    { 0xd5214433, 61 },
    { 0, 0 },
    { 0x6914435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d1443c, 34 },
    { 0, 0 },
    { 0x73d1443e, 95 },
    { 0xad11443f, 98 },
    { 0x9cf14440, 23 },
    { 0xdbd14441, 95 },
    { 0xef714440, 98 },
    { 0x11614443, 61 },
    { 0xf014443, 23 },
    { 0, 0 },
    { 0x63214446, 78 },
    { 0xe714447, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b1444b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5914450, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae514457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa801445c, 61 },
    { 0, 0 },
    { 0xd6a1445e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa014466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4171446b, 61 },
    { 0xe8f1446c, 61 },
    { 0x6961446c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc14470, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e14473, 94 },
    { 0xe214473, 61 },
    { 0xd7414473, 76 },
    { 0x3d214476, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf14479, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf31447f, 33 },
    { 0xd0f14480, 55 },
    { 0x62614480, 92 },
    { 0xd2e14480, 34 },
    { 0x76f14483, 61 },
    { 0x3c714482, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd14489, 85 },
    { 0xa651448a, 99 },
    { 0x6e71448b, 26 },
    { 0x8fd1448c, 38 },
    { 0xec41448d, 90 },
    { 0xded1448e, 23 },
    { 0x2f1448e, 33 },
    { 0xef614490, 90 },
    { 0xf714489, 95 },
    { 0x6d514489, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc014495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3011449a, 46 },
    { 0xe31449b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a144aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0144b3, 76 },
    { 0xdae144b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3144bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c144c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd144cd, 88 },
    { 0x62b144ce, 95 },
    { 0xe1f144ce, 90 },
    { 0x93b144ce, 43 },
    { 0xc1f144d1, 76 },
    { 0x8bc144d2, 100 },
    { 0x21c144d2, 98 },
    { 0x860144d4, 43 },
    { 0x464144d5, 33 },
    { 0x413144cd, 76 },
    { 0x7ec144d6, 95 },
    { 0, 0 },
    { 0x81f144d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae144e0, 99 },
    { 0x4fc144e0, 93 },
    { 0xcce144e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8144e6, 81 },
    { 0, 0 },
    { 0x22c144e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db144eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee144ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49144f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a144f7, 85 },
    { 0xbd6144f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e214504, 61 },
    { 0xbe414505, 36 },
    { 0xbf14505, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef51450a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd814511, 17 },
    { 0, 0 },
    { 0xbdf14513, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe141451b, 23 },
    { 0, 0 },
    { 0x8e61451d, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x75014520, 90 },
    { 0x9ca14521, 90 },
    { 0xdaa14521, 61 },
    { 0x80514522, 55 },
    { 0x6f314524, 76 },
    { 0xb5414524, 98 },
    { 0xa3714526, 81 },
    { 0x56914520, 34 },
    { 0xcbb14526, 61 },
    { 0, 0 },
    { 0x4551452a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc1452d, 23 },
    { 0xc491452e, 90 },
    { 0xa361452f, 55 },
    { 0, 0 },
    { 0xaa314531, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a414535, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb071453b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5414544, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52314548, 67 },
    { 0, 0 },
    { 0x5641454a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3314550, 55 },
    { 0x2fd14551, 23 },
    { 0xdfd14551, 61 },
    { 0x7e414553, 98 },
    { 0x3ba14553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd001455b, 98 },
    { 0x1c51455c, 33 },
    { 0xf5f1455d, 98 },
    { 0x8591455d, 61 },
    { 0x74c1455f, 40 },
    { 0xe081455f, 90 },
    { 0x99b1455c, 99 },
    { 0x74914561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f14565, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b14568, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1456f, 76 },
    { 0, 0 },
    { 0xac814571, 61 },
    { 0, 0 },
    { 0xae514573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92114576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b914579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1457d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e14583, 33 },
    { 0, 0 },
    { 0xc1314585, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a21458a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33114593, 33 },
    { 0x6a914593, 35 },
    { 0, 0 },
    { 0x91714596, 61 },
    { 0xc5b14596, 23 },
    { 0xf8a14598, 93 },
    { 0xf9214596, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd31459e, 52 },
    { 0, 0 },
    { 0xf62145a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce145a9, 88 },
    { 0xca1145aa, 90 },
    { 0x872145ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124145af, 34 },
    { 0x22a145b0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55145b6, 33 },
    { 0x1b9145b7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e145be, 33 },
    { 0x84c145bf, 90 },
    { 0x941145be, 61 },
    { 0, 0 },
    { 0xa33145c2, 55 },
    { 0x1c0145c3, 76 },
    { 0x8c1145c2, 95 },
    { 0x336145c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b145c8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a145cb, 75 },
    { 0, 0 },
    { 0x894145cd, 76 },
    { 0, 0 },
    { 0x3c1145cf, 43 },
    { 0x68145d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c145d3, 97 },
    { 0x497145d4, 61 },
    { 0x5d8145d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79145d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72145df, 84 },
    { 0, 0 },
    { 0xad0145e1, 52 },
    { 0x292145e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3145e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a145ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc145ef, 76 },
    { 0, 0 },
    { 0x859145f1, 34 },
    { 0xe38145f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97145f5, 25 },
    { 0x1a145f6, 87 },
    { 0x6bc145f7, 90 },
    { 0x64a145f8, 95 },
    { 0xbab145f9, 93 },
    { 0xb10145fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a14601, 98 },
    { 0x48414602, 33 },
    { 0, 0 },
    { 0x4b14604, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96314609, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98314612, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6614615, 90 },
    { 0xa3814615, 46 },
    { 0, 0 },
    { 0xe0014618, 55 },
    { 0xcfb14619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7851461c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d14623, 43 },
    { 0xfea14624, 96 },
    { 0x32414623, 84 },
    { 0x1814624, 99 },
    { 0x46614627, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1462a, 90 },
    { 0, 0 },
    { 0x4971462c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e14632, 43 },
    { 0xcd314633, 33 },
    { 0x38714634, 34 },
    { 0x6c214634, 23 },
    { 0xe4214636, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x18514639, 46 },
    { 0, 0 },
    { 0x1ea1463b, 91 },
    { 0x3541463c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2114644, 33 },
    { 0x5a914644, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4014652, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f14655, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1465e, 81 },
    { 0, 0 },
    { 0x21214660, 90 },
    { 0x9cc14661, 98 },
    { 0x83d14660, 23 },
    { 0x3df14661, 38 },
    { 0x56314661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e14668, 91 },
    { 0x67914668, 98 },
    { 0x1661466a, 33 },
    { 0xe741466b, 88 },
    { 0xc11466b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de1466f, 23 },
    { 0x241466f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f14679, 65 },
    { 0xb741467a, 95 },
    { 0, 0 },
    { 0x7b21467c, 87 },
    { 0x5d51467d, 98 },
    { 0xadd1467c, 23 },
    { 0, 0 },
    { 0xc5914680, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b114684, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1468b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a51468f, 61 },
    { 0x4ec14690, 95 },
    { 0x2a914690, 55 },
    { 0x8f114692, 95 },
    { 0x76914691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe14698, 24 },
    { 0x4ec14699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6061469c, 33 },
    { 0, 0 },
    { 0x8891469e, 84 },
    { 0, 0 },
    { 0x52b146a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4146a3, 33 },
    { 0x414146a4, 33 },
    { 0xb5a146a3, 80 },
    { 0x968146a6, 55 },
    { 0xb7c146a5, 55 },
    { 0x773146a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f146b0, 53 },
    { 0x411146b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3146b9, 93 },
    { 0, 0 },
    { 0x4f146bb, 95 },
    { 0x251146bc, 95 },
    { 0x846146bd, 23 },
    { 0xce7146be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf146c1, 80 },
    { 0x15f146c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9146c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea146d8, 98 },
    { 0x465146d9, 23 },
    { 0x36b146da, 61 },
    { 0x529146da, 33 },
    { 0x9a5146d8, 98 },
    { 0x2f3146d8, 23 },
    { 0xcf3146dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a146e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6146f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5146fe, 61 },
    { 0x3de146ff, 17 },
    { 0x5ba14700, 93 },
    { 0, 0 },
    { 0xc2914702, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14705, 34 },
    { 0x1ee14705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78214709, 61 },
    { 0x4371470a, 23 },
    { 0x28d1470b, 83 },
    { 0, 0 },
    { 0xc8f1470d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc214711, 23 },
    { 0xfcc14712, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f14718, 81 },
    { 0xea514719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b1471e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47114727, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4901472c, 33 },
    { 0x38b1472d, 100 },
    { 0xd091472e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa14734, 46 },
    { 0x41714735, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b14739, 66 },
    { 0xa981473a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8671473d, 33 },
    { 0x4d51473e, 98 },
    { 0, 0 },
    { 0x47014740, 61 },
    { 0x2814741, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb14746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7051475a, 98 },
    { 0, 0 },
    { 0xfad1475c, 93 },
    { 0x1221475c, 23 },
    { 0x61a1475c, 33 },
    { 0x6ba1475f, 26 },
    { 0x2de1475f, 68 },
    { 0xfa51475f, 90 },
    { 0, 0 },
    { 0x45214763, 61 },
    { 0x15914764, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87114769, 43 },
    { 0x75d14769, 84 },
    { 0, 0 },
    { 0xf691476c, 43 },
    { 0xa811476d, 33 },
    { 0xc3e1476d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b14775, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b1477b, 100 },
    { 0, 0 },
    { 0xc3a1477d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47014781, 90 },
    { 0x89114782, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c14787, 69 },
    { 0xe1d14788, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea1479c, 55 },
    { 0xa801479c, 76 },
    { 0x7b71479e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0147a1, 76 },
    { 0x783147a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a147a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb147a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27147ad, 55 },
    { 0xa2c147ae, 98 },
    { 0, 0 },
    { 0xc80147b0, 46 },
    { 0x36d147b1, 81 },
    { 0x78a147b2, 90 },
    { 0x41b147b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e147b6, 93 },
    { 0x66f147b7, 55 },
    { 0xf2d147b8, 33 },
    { 0xc01147b9, 98 },
    { 0, 0 },
    { 0xcb7147bb, 61 },
    { 0, 0 },
    { 0xe84147bd, 90 },
    { 0xd95147bd, 90 },
    { 0x6f2147bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0147c4, 98 },
    { 0xd7a147c5, 84 },
    { 0x8f6147c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351147cf, 33 },
    { 0xb62147d0, 46 },
    { 0x77d147d1, 52 },
    { 0x79e147d1, 90 },
    { 0x7fa147cf, 43 },
    { 0x9b2147d4, 46 },
    { 0xd8a147d3, 66 },
    { 0x312147d5, 90 },
    { 0x6e147d7, 98 },
    { 0x6c9147d8, 33 },
    { 0xe03147d9, 100 },
    { 0, 0 },
    { 0x65a147db, 61 },
    { 0x706147db, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09147e2, 78 },
    { 0x162147e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f147e9, 33 },
    { 0xd98147ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1147ee, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7147f3, 34 },
    { 0, 0 },
    { 0x3a7147f5, 95 },
    { 0x802147f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6147fa, 93 },
    { 0x13e147fb, 95 },
    { 0x4a6147fc, 33 },
    { 0x32b147fd, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d14800, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6414803, 23 },
    { 0xd9014803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8014809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac714810, 87 },
    { 0xb7914810, 23 },
    { 0x57514810, 23 },
    { 0x36c14813, 67 },
    { 0, 0 },
    { 0x55214815, 34 },
    { 0xd3314816, 43 },
    { 0xcc714816, 23 },
    { 0x70714816, 43 },
    { 0, 0 },
    { 0x35f1481a, 76 },
    { 0x7d81481b, 78 },
    { 0xd2c1481c, 90 },
    { 0x9bd1481b, 95 },
    { 0, 0 },
    { 0xf8c1481f, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c14822, 61 },
    { 0x3b214823, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe451482a, 95 },
    { 0xaf21482b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d91482f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1483b, 43 },
    { 0, 0 },
    { 0x8f71483d, 91 },
    { 0x72f1483e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b714841, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4914847, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb21484a, 33 },
    { 0xe821484b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa501484f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e14855, 99 },
    { 0xc2214856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d1485a, 61 },
    { 0, 0 },
    { 0x1c41485c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18114861, 34 },
    { 0x7f014862, 90 },
    { 0, 0 },
    { 0x91114864, 76 },
    { 0x25514864, 61 },
    { 0xc4c14865, 90 },
    { 0x34a14867, 55 },
    { 0xfaf14867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa1486b, 33 },
    { 0x21e1486c, 95 },
    { 0x1d41486b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e14873, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e14877, 93 },
    { 0, 0 },
    { 0x19414879, 66 },
    { 0x3a41487a, 95 },
    { 0, 0 },
    { 0xf931487c, 84 },
    { 0x4491487d, 90 },
    { 0xc7d1487e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x90114881, 61 },
    { 0x26114881, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1488f, 46 },
    { 0xcad1488f, 61 },
    { 0xdc514891, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a14899, 33 },
    { 0, 0 },
    { 0x6831489b, 97 },
    { 0, 0 },
    { 0x15d1489d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f148a0, 76 },
    { 0x139148a0, 85 },
    { 0x2f4148a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6148a7, 90 },
    { 0, 0 },
    { 0x9bc148a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1148ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2148b6, 90 },
    { 0, 0 },
    { 0x59c148b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2148bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x972148be, 98 },
    { 0x608148bf, 61 },
    { 0x683148c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x180148c3, 52 },
    { 0x537148c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc148c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d148d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61148d8, 83 },
    { 0x8e9148d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d148dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c148e2, 34 },
    { 0x759148e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f148e7, 61 },
    { 0, 0 },
    { 0xd28148e9, 85 },
    { 0x3be148ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa148ed, 61 },
    { 0x197148ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00148f4, 43 },
    { 0, 0 },
    { 0xc91148f6, 85 },
    { 0xb81148f7, 98 },
    { 0, 0 },
    { 0x420148f9, 66 },
    { 0xf53148fa, 90 },
    { 0x50c148fb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43148fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc214901, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad214905, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61490f, 61 },
    { 0x8ac14910, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf014916, 84 },
    { 0xd0014917, 23 },
    { 0, 0 },
    { 0xe7014919, 90 },
    { 0x36514919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d14920, 90 },
    { 0xbf14921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7714927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451492d, 46 },
    { 0xe9f1492e, 98 },
    { 0x6351492f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca414933, 98 },
    { 0x56f14934, 98 },
    { 0x54514935, 65 },
    { 0xdac14936, 95 },
    { 0xcac14933, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc1493c, 33 },
    { 0xfb71493d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95114941, 23 },
    { 0, 0 },
    { 0x1be14943, 61 },
    { 0x38814943, 90 },
    { 0x4cb14944, 33 },
    { 0x60a14945, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf941494e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc14951, 33 },
    { 0x5b414952, 33 },
    { 0xc8114952, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1495a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db1495f, 95 },
    { 0x9e714960, 88 },
    { 0x39714961, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2571496c, 98 },
    { 0x3801496d, 93 },
    { 0x1621496d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f14972, 87 },
    { 0x21614973, 23 },
    { 0x43714974, 33 },
    { 0x2d914975, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3691497d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d14983, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75014988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd21498c, 33 },
    { 0, 0 },
    { 0x201498e, 90 },
    { 0x411498f, 61 },
    { 0, 0 },
    { 0xdbc14991, 90 },
    { 0x3c214991, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c14995, 95 },
    { 0, 0 },
    { 0xc1e14997, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd851499a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6611499e, 100 },
    { 0x2911499f, 76 },
    { 0x46b149a0, 77 },
    { 0x90a149a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62149a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593149ad, 90 },
    { 0xffb149ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f149b4, 100 },
    { 0, 0 },
    { 0xe5c149b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46149bb, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5149c5, 98 },
    { 0x626149c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7149c9, 90 },
    { 0x16149ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769149d0, 61 },
    { 0, 0 },
    { 0x555149d2, 61 },
    { 0, 0 },
    { 0xf6a149d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977149d8, 33 },
    { 0xe42149d9, 93 },
    { 0, 0 },
    { 0x10f149db, 33 },
    { 0, 0 },
    { 0x49149dd, 46 },
    { 0x219149de, 93 },
    { 0x3e1149df, 98 },
    { 0xcbd149dd, 23 },
    { 0x4e149e1, 95 },
    { 0xe00149e0, 67 },
    { 0x994149df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395149ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34149ed, 95 },
    { 0, 0 },
    { 0x6ef149ef, 17 },
    { 0, 0 },
    { 0xb149f1, 66 },
    { 0xa72149f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f149f6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da149f9, 66 },
    { 0x83e149f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2149fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d14a05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9114a08, 65 },
    { 0, 0 },
    { 0xf2514a0a, 34 },
    { 0xecd14a0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14a14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c14a18, 43 },
    { 0xc2314a19, 91 },
    { 0, 0 },
    { 0x45d14a1b, 76 },
    { 0, 0 },
    { 0xc2314a1d, 90 },
    { 0xbce14a1e, 100 },
    { 0x8b014a1e, 36 },
    { 0x2d414a1e, 85 },
    { 0x63214a20, 23 },
    { 0x73714a20, 33 },
    { 0x2c14a21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8614a27, 98 },
    { 0, 0 },
    { 0x81814a29, 43 },
    { 0, 0 },
    { 0xf0914a2b, 34 },
    { 0x1a414a2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xade14a2f, 61 },
    { 0, 0 },
    { 0x5fc14a31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b14a34, 34 },
    { 0x14114a34, 23 },
    { 0x5a914a36, 64 },
    { 0x57614a37, 98 },
    { 0x2cb14a38, 23 },
    { 0x9cc14a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4214a3d, 23 },
    { 0xb1814a3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd14a42, 87 },
    { 0x4d014a43, 67 },
    { 0, 0 },
    { 0xf0414a45, 84 },
    { 0xd0114a46, 61 },
    { 0, 0 },
    { 0xaa714a48, 90 },
    { 0xc3414a49, 56 },
    { 0, 0 },
    { 0x48e14a4b, 90 },
    { 0x5cd14a4c, 76 },
    { 0x9cb14a4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b014a51, 66 },
    { 0xddf14a52, 61 },
    { 0xe9514a51, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b14a59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8414a65, 98 },
    { 0x68214a66, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b14a6a, 76 },
    { 0x81414a6a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b14a6e, 93 },
    { 0x3ec14a6e, 33 },
    { 0x47714a70, 38 },
    { 0xa2714a70, 69 },
    { 0xdf314a72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f714a75, 40 },
    { 0xc214a76, 33 },
    { 0x22514a77, 33 },
    { 0x1cd14a78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64914a7f, 96 },
    { 0, 0 },
    { 0x29614a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e14a8d, 55 },
    { 0x5d814a8e, 77 },
    { 0x9d114a8f, 61 },
    { 0, 0 },
    { 0x23414a91, 97 },
    { 0x7d714a92, 69 },
    { 0x7af14a91, 33 },
    { 0x61414a94, 34 },
    { 0xb714a95, 52 },
    { 0x6d014a95, 33 },
    { 0, 0 },
    { 0x23514a98, 23 },
    { 0x25e14a98, 33 },
    { 0xfe814a9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8514a9d, 90 },
    { 0, 0 },
    { 0x36214a9f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x82114aa2, 55 },
    { 0x8a914aa3, 61 },
    { 0x33414aa2, 43 },
    { 0x12b14aa2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead14aaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15514ab2, 61 },
    { 0xf7a14ab2, 100 },
    { 0, 0 },
    { 0xc7a14ab5, 90 },
    { 0xfbc14ab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9614aba, 46 },
    { 0xa9814abb, 95 },
    { 0x7e614abc, 33 },
    { 0x3b714abd, 34 },
    { 0, 0 },
    { 0x4e214abf, 34 },
    { 0, 0 },
    { 0x13214ac1, 55 },
    { 0x64814ac2, 81 },
    { 0, 0 },
    { 0xd6514ac4, 90 },
    { 0x87814ac5, 61 },
    { 0, 0 },
    { 0xf5114ac7, 46 },
    { 0xf4014ac8, 23 },
    { 0x26214ac9, 46 },
    { 0x44c14ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50714ace, 98 },
    { 0, 0 },
    { 0xb9614ad0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50014ad8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e14adc, 33 },
    { 0, 0 },
    { 0xfe114ade, 98 },
    { 0x1814adf, 85 },
    { 0x5f14ae0, 55 },
    { 0x3c014ae0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc814ae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b14ae9, 33 },
    { 0x1f14aea, 90 },
    { 0, 0 },
    { 0xc5514aec, 23 },
    { 0x75014aec, 61 },
    { 0, 0 },
    { 0x4ba14aef, 34 },
    { 0, 0 },
    { 0xbbf14af1, 61 },
    { 0x5e914af2, 90 },
    { 0xeb414af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a14af6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e914afa, 61 },
    { 0x80614afb, 87 },
    { 0x18914afc, 100 },
    { 0x92a14afd, 76 },
    { 0xf8214afa, 93 },
    { 0xb1a14aff, 33 },
    { 0x64f14aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f14b04, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f214b08, 76 },
    { 0x4c614b09, 33 },
    { 0xea814b09, 33 },
    { 0x55414b0b, 78 },
    { 0, 0 },
    { 0x74414b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18414b10, 93 },
    { 0xff014b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc14b18, 61 },
    { 0xaf114b19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f14b1c, 83 },
    { 0xd0214b1d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d014b22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb214b25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd414b29, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e14b2c, 33 },
    { 0x13014b2d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a14b35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a14b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c14b3b, 75 },
    { 0x34e14b3c, 34 },
    { 0xf3d14b3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c14b44, 95 },
    { 0x4c114b44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10314b4c, 23 },
    { 0, 0 },
    { 0x77a14b4e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f14b52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd14b5c, 23 },
    { 0xdd414b5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7314b61, 93 },
    { 0, 0 },
    { 0xaa514b63, 61 },
    { 0, 0 },
    { 0x4ed14b65, 78 },
    { 0, 0 },
    { 0xffb14b67, 33 },
    { 0xd0914b67, 33 },
    { 0xe0314b67, 93 },
    { 0x30114b6a, 23 },
    { 0xa7514b6a, 76 },
    { 0xd2114b6c, 19 },
    { 0x73614b6c, 17 },
    { 0, 0 },
    { 0xc0614b6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe14b72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f14b75, 61 },
    { 0x3a814b76, 93 },
    { 0x6d14b76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c414b7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a14b82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2314b89, 98 },
    { 0x69314b8a, 55 },
    { 0x55514b8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd414b8e, 87 },
    { 0, 0 },
    { 0x4fa14b90, 91 },
    { 0, 0 },
    { 0x60914b92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9314b95, 98 },
    { 0xd9814b95, 46 },
    { 0x1bc14b97, 90 },
    { 0xa0814b98, 46 },
    { 0x5fe14b98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51614b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd214ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e314baf, 95 },
    { 0x2f214bb0, 88 },
    { 0x55d14bb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25814bb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8614bbb, 23 },
    { 0x24d14bbc, 33 },
    { 0x6a414bbd, 68 },
    { 0, 0 },
    { 0x4bc14bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a014bc4, 95 },
    { 0, 0 },
    { 0xb6814bc6, 67 },
    { 0x93114bc7, 95 },
    { 0x9e514bc7, 33 },
    { 0x6b414bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f14bcf, 33 },
    { 0, 0 },
    { 0x87314bd1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a14bd7, 90 },
    { 0x72214bd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42314bdc, 23 },
    { 0x8ce14bdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14be0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6414be3, 100 },
    { 0xdea14be3, 76 },
    { 0x29014be5, 43 },
    { 0, 0 },
    { 0x7e014be7, 95 },
    { 0, 0 },
    { 0x43a14be9, 90 },
    { 0x52514bea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3714bef, 82 },
    { 0, 0 },
    { 0xed114bf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b14bf8, 90 },
    { 0, 0 },
    { 0x5314bfa, 76 },
    { 0, 0 },
    { 0x2fe14bfc, 78 },
    { 0x5e014bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed14c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf14c03, 55 },
    { 0, 0 },
    { 0x6d314c05, 90 },
    { 0, 0 },
    { 0x67714c07, 93 },
    { 0xf6314c08, 77 },
    { 0x9e014c08, 23 },
    { 0x8c314c09, 55 },
    { 0x8f414c07, 75 },
    { 0x38114c0c, 75 },
    { 0x6214c07, 95 },
    { 0x3114c0e, 90 },
    { 0x8d114c0f, 55 },
    { 0xad614c10, 67 },
    { 0x37f14c11, 43 },
    { 0xab514c12, 55 },
    { 0xa7c14c11, 90 },
    { 0x5b214c0e, 100 },
    { 0x9b414c15, 90 },
    { 0xf1214c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba14c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4114c1d, 43 },
    { 0xf814c1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4014c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a14c2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39514c2d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10114c34, 97 },
    { 0xdfa14c34, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e14c38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd14c3c, 78 },
    { 0x5314c3c, 76 },
    { 0, 0 },
    { 0x24114c3f, 98 },
    { 0xf9514c40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc14c45, 99 },
    { 0xf9a14c46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41714c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7314c52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb614c57, 76 },
    { 0x26514c58, 90 },
    { 0, 0 },
    { 0x6a114c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3914c5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24314c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f514c65, 98 },
    { 0, 0 },
    { 0x88314c67, 77 },
    { 0, 0 },
    { 0x29a14c69, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb314c6e, 55 },
    { 0x9b814c6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b14c72, 23 },
    { 0x13714c72, 87 },
    { 0xb8214c74, 34 },
    { 0xd7414c75, 76 },
    { 0xde614c74, 76 },
    { 0xfcd14c74, 98 },
    { 0x4a014c78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9214c7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e14c7f, 95 },
    { 0xfe614c7f, 34 },
    { 0x38514c80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8714c84, 78 },
    { 0, 0 },
    { 0x83e14c86, 23 },
    { 0x85114c87, 95 },
    { 0x42414c86, 23 },
    { 0x9514c88, 95 },
    { 0, 0 },
    { 0xf9414c8b, 85 },
    { 0x35914c8c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f14c94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b14c98, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e14ca4, 68 },
    { 0x23e14ca5, 55 },
    { 0x38b14ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52014cab, 55 },
    { 0xe1e14cac, 89 },
    { 0x38914cad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a914cb0, 87 },
    { 0x26514cb1, 33 },
    { 0xb4414cb0, 23 },
    { 0xfe114cb1, 33 },
    { 0xa4b14cb4, 88 },
    { 0x2ef14cb1, 43 },
    { 0x58614cb6, 55 },
    { 0x3af14cb7, 98 },
    { 0xe4314cb7, 52 },
    { 0, 0 },
    { 0xe0d14cba, 33 },
    { 0xa4014cbb, 53 },
    { 0xae014cbb, 68 },
    { 0x70114cbc, 95 },
    { 0x7f014cbe, 90 },
    { 0x914cbe, 98 },
    { 0x9a114cc0, 43 },
    { 0x7f314cbe, 33 },
    { 0xc614cc2, 43 },
    { 0x57414cc2, 90 },
    { 0xff314cc4, 85 },
    { 0x80014cc5, 33 },
    { 0x48714cc5, 61 },
    { 0xa1c14cc1, 99 },
    { 0xfa314cc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e14cd1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd14cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75814ce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a14ce9, 88 },
    { 0x66014cea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0014cee, 66 },
    { 0x61b14cee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57214cf4, 95 },
    { 0xd014cf5, 33 },
    { 0x1d814cf6, 90 },
    { 0, 0 },
    { 0x3f914cf8, 52 },
    { 0, 0 },
    { 0xdb214cfa, 95 },
    { 0, 0 },
    { 0x89914cfc, 33 },
    { 0x64214cfd, 98 },
    { 0xec814cfd, 95 },
    { 0x8de14cff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc14d02, 43 },
    { 0x14114d02, 75 },
    { 0, 0 },
    { 0xcc114d05, 66 },
    { 0, 0 },
    { 0x47e14d07, 90 },
    { 0x64914d08, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6714d0d, 23 },
    { 0x56214d0e, 95 },
    { 0x55914d0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb614d14, 33 },
    { 0x85e14d15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde14d18, 98 },
    { 0x82814d19, 23 },
    { 0, 0 },
    { 0x56f14d1b, 87 },
    { 0x45714d1b, 99 },
    { 0xd7d14d1d, 98 },
    { 0xcb114d1e, 61 },
    { 0, 0 },
    { 0xb6414d20, 52 },
    { 0, 0 },
    { 0x60b14d22, 93 },
    { 0x5ee14d23, 76 },
    { 0x33514d24, 66 },
    { 0xdeb14d24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c14d2a, 66 },
    { 0x40514d2b, 61 },
    { 0x15b14d2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e14d32, 69 },
    { 0, 0 },
    { 0x30c14d34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa14d3a, 87 },
    { 0xd7a14d3b, 23 },
    { 0x4e914d3c, 55 },
    { 0x27514d3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea614d42, 98 },
    { 0, 0 },
    { 0xfc114d44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c14d49, 69 },
    { 0x49b14d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e914d4d, 91 },
    { 0, 0 },
    { 0x77014d4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4114d52, 95 },
    { 0, 0 },
    { 0xaa414d54, 43 },
    { 0x4fb14d55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48814d58, 95 },
    { 0, 0 },
    { 0xab614d5a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8014d5e, 46 },
    { 0xce414d5f, 95 },
    { 0x47614d60, 76 },
    { 0x5f914d5e, 76 },
    { 0, 0 },
    { 0xca314d63, 46 },
    { 0x4dd14d64, 98 },
    { 0xa0b14d63, 23 },
    { 0xe0514d65, 90 },
    { 0x4814d67, 74 },
    { 0x33e14d67, 90 },
    { 0xe4614d68, 93 },
    { 0xfbe14d63, 95 },
    { 0xa3e14d6a, 76 },
    { 0x3e014d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa514d72, 55 },
    { 0x6f114d72, 33 },
    { 0xc2714d74, 71 },
    { 0x5614d75, 95 },
    { 0x86214d75, 95 },
    { 0x49814d77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87514d7a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c14d7e, 90 },
    { 0, 0 },
    { 0x24214d80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78214d86, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e114d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1414d8c, 61 },
    { 0xab214d8d, 90 },
    { 0, 0 },
    { 0x4a814d8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5814d92, 90 },
    { 0, 0 },
    { 0xa7714d94, 33 },
    { 0x6bc14d95, 76 },
    { 0x69d14d96, 93 },
    { 0, 0 },
    { 0xb3914d98, 93 },
    { 0, 0 },
    { 0xa4914d9a, 52 },
    { 0x17214d9a, 55 },
    { 0xccc14d9c, 88 },
    { 0x79214d9d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac14da0, 23 },
    { 0x7114da0, 33 },
    { 0x60514da2, 99 },
    { 0xbf14da3, 23 },
    { 0, 0 },
    { 0xfab14da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa214da8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe314dac, 91 },
    { 0xd1814dac, 95 },
    { 0x8cf14dad, 52 },
    { 0x14014daf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c14db2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c314db9, 95 },
    { 0x75514db9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e214dbd, 23 },
    { 0xb8514dbe, 95 },
    { 0x1e914dbf, 61 },
    { 0x8c914dbf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95214dc9, 43 },
    { 0x8b114dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e414dcd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb14dd1, 53 },
    { 0, 0 },
    { 0x32814dd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb14dd6, 95 },
    { 0x6bb14dd6, 87 },
    { 0x4c114dd8, 55 },
    { 0, 0 },
    { 0x6a914dda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa14ddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d514de3, 36 },
    { 0x97d14de4, 33 },
    { 0x81d14de5, 55 },
    { 0xcdc14de6, 61 },
    { 0x1114de5, 34 },
    { 0x9614de8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c14df0, 90 },
    { 0x77e14df0, 67 },
    { 0x21414df2, 67 },
    { 0xc8214df3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a14dfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4314e02, 61 },
    { 0, 0 },
    { 0x4a114e04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a14e07, 99 },
    { 0x4cb14e07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8414e0b, 100 },
    { 0x93014e0c, 23 },
    { 0x76f14e0b, 65 },
    { 0x3ad14e0e, 85 },
    { 0x58a14e0c, 90 },
    { 0xfec14e10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9714e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce114e1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0014e25, 34 },
    { 0x5ee14e26, 34 },
    { 0xffe14e25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d14e2a, 33 },
    { 0, 0 },
    { 0xac14e2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b414e35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab514e3c, 23 },
    { 0x2b14e3d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf14e42, 55 },
    { 0, 0 },
    { 0xba514e44, 61 },
    { 0xf914e44, 90 },
    { 0x93914e46, 61 },
    { 0xdf114e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec114e4a, 23 },
    { 0x28814e4a, 75 },
    { 0xcdb14e4c, 83 },
    { 0xf9014e4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb414e51, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb714e54, 43 },
    { 0x48614e54, 90 },
    { 0, 0 },
    { 0xc2214e57, 79 },
    { 0xad314e57, 77 },
    { 0xb6314e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e14e5c, 95 },
    { 0, 0 },
    { 0x614e5e, 34 },
    { 0x32414e5e, 23 },
    { 0x2b014e60, 52 },
    { 0, 0 },
    { 0x1fc14e62, 90 },
    { 0, 0 },
    { 0x5214e64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c14e68, 43 },
    { 0, 0 },
    { 0x95414e6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0614e6d, 23 },
    { 0x7cd14e6d, 76 },
    { 0xcc14e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e14e73, 61 },
    { 0, 0 },
    { 0x32b14e75, 95 },
    { 0x44914e76, 81 },
    { 0xd7814e75, 98 },
    { 0x75914e76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53414e7c, 78 },
    { 0xc0e14e7d, 77 },
    { 0x8a814e7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9014e83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d14e8b, 52 },
    { 0, 0 },
    { 0x66814e8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf14e94, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e14e97, 90 },
    { 0x80d14e98, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad14e9c, 61 },
    { 0xce214e9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44514ea3, 52 },
    { 0, 0 },
    { 0x28d14ea5, 98 },
    { 0xa8c14ea6, 78 },
    { 0, 0 },
    { 0x68114ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae14ead, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x7014eb0, 90 },
    { 0, 0 },
    { 0x9a414eb2, 61 },
    { 0xd2114eb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e14eb9, 97 },
    { 0x1f014eba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73614ebf, 76 },
    { 0x75714ec0, 23 },
    { 0x2914ec1, 90 },
    { 0xfb714ec2, 91 },
    { 0xc5514ec3, 98 },
    { 0x1e514ec3, 91 },
    { 0xd3614ec5, 95 },
    { 0, 0 },
    { 0x20814ec7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf14eca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad414ed5, 61 },
    { 0x93414ed6, 98 },
    { 0xbd14ed6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73414eda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4714edd, 23 },
    { 0, 0 },
    { 0x47814edf, 61 },
    { 0x69114ee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b914ee5, 76 },
    { 0x26814ee6, 99 },
    { 0xc6814ee7, 90 },
    { 0, 0 },
    { 0x95914ee9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16414eed, 33 },
    { 0, 0 },
    { 0x5b214eef, 55 },
    { 0xd2314ef0, 98 },
    { 0, 0 },
    { 0x2de14ef2, 88 },
    { 0x9a614ef2, 66 },
    { 0xbf214ef2, 90 },
    { 0, 0 },
    { 0xa0314ef6, 95 },
    { 0xf0714ef7, 33 },
    { 0x64214ef8, 61 },
    { 0, 0 },
    { 0x49114efa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc14f01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9214f04, 43 },
    { 0x8ad14f05, 98 },
    { 0x23314f06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90214f09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22614f0d, 81 },
    { 0xf614f0e, 90 },
    { 0xb3e14f0f, 34 },
    { 0xd6014f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf14f13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a14f1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a14f20, 81 },
    { 0xa1d14f21, 33 },
    { 0x4c114f20, 55 },
    { 0x24e14f23, 90 },
    { 0xf2114f24, 46 },
    { 0, 0 },
    { 0x7cf14f26, 33 },
    { 0x9f514f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe314f2b, 78 },
    { 0x9bc14f2c, 90 },
    { 0xda214f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf014f30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0214f35, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x21514f38, 23 },
    { 0x36a14f39, 23 },
    { 0, 0 },
    { 0x56814f3b, 90 },
    { 0x57814f3c, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d14f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94714f47, 33 },
    { 0xd1414f48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59314f52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c214f55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63514f59, 43 },
    { 0, 0 },
    { 0x61314f5b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda14f5f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5214f62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7614f6a, 76 },
    { 0x56814f6a, 76 },
    { 0x68a14f6a, 98 },
    { 0, 0 },
    { 0xcf714f6e, 90 },
    { 0xf6714f6f, 91 },
    { 0, 0 },
    { 0xe0114f71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e714f76, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14f79, 93 },
    { 0x18d14f7a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda014f7f, 90 },
    { 0, 0 },
    { 0xe0514f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a14f85, 90 },
    { 0, 0 },
    { 0x10b14f87, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5314f8a, 98 },
    { 0, 0 },
    { 0xff114f8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74714f91, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14f99, 95 },
    { 0, 0 },
    { 0xf5814f9b, 33 },
    { 0x1dc14f9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad514fa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x37314fa3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a14fa6, 98 },
    { 0xa9e14fa6, 17 },
    { 0x6e614fa7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55014fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54914fb1, 61 },
    { 0xab614fb2, 99 },
    { 0, 0 },
    { 0xa14fb4, 95 },
    { 0x42414fb4, 95 },
    { 0xd5114fb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf214fba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a14fc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9514fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58414fca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15314fcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2814fd2, 61 },
    { 0, 0 },
    { 0xe3314fd4, 61 },
    { 0x1fe14fd5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87314fda, 33 },
    { 0, 0 },
    { 0x80514fdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62914fdf, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x70714fe2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b14fe7, 33 },
    { 0, 0 },
    { 0x8fe14fe9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76414fed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x51314ff0, 98 },
    { 0x84614ff1, 75 },
    { 0xf8814ff2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa14ff7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b414ffb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac14ffe, 46 },
    { 0x48214fff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f15004, 23 },
    { 0xfc015005, 23 },
    { 0x82f15006, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221500d, 88 },
    { 0, 0 },
    { 0xf7a1500f, 46 },
    { 0x26e15010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e15014, 43 },
    { 0xc2815015, 90 },
    { 0xca315016, 33 },
    { 0x44a15015, 23 },
    { 0, 0 },
    { 0x43d15019, 61 },
    { 0xe961501a, 23 },
    { 0xfa1501a, 76 },
    { 0, 0 },
    { 0x1b91501d, 33 },
    { 0, 0 },
    { 0xb271501f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e015023, 33 },
    { 0x2c215024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47915028, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4691502d, 98 },
    { 0xe4c1502d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe915031, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3415034, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa151503f, 23 },
    { 0x1c515040, 33 },
    { 0x6bb15041, 81 },
    { 0, 0 },
    { 0x14215043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f01504a, 61 },
    { 0xc071504b, 90 },
    { 0xaf1504b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c15050, 77 },
    { 0xcfa15051, 55 },
    { 0xa7315052, 91 },
    { 0xde915052, 98 },
    { 0xc8f15054, 87 },
    { 0x97b15052, 95 },
    { 0xfd015052, 95 },
    { 0xcca15057, 93 },
    { 0xc0515058, 23 },
    { 0x58015059, 76 },
    { 0x8f71505a, 67 },
    { 0x2915058, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53515068, 34 },
    { 0xef115069, 46 },
    { 0xa5a1506a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1506f, 61 },
    { 0xb8d15070, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab115074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef15079, 64 },
    { 0xd4d1507a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251507f, 33 },
    { 0x24615080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab15083, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30315088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc301508d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde515091, 61 },
    { 0, 0 },
    { 0x27215093, 90 },
    { 0x79015094, 34 },
    { 0xfdc15094, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f615099, 84 },
    { 0xc591509a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf671509f, 76 },
    { 0xc4f1509f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e150a5, 90 },
    { 0x586150a6, 61 },
    { 0, 0 },
    { 0xf5e150a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x441150ab, 50 },
    { 0, 0 },
    { 0x493150ad, 68 },
    { 0, 0 },
    { 0x83c150af, 98 },
    { 0xd04150b0, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a150b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x947150b6, 55 },
    { 0x282150b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78150be, 90 },
    { 0xa9b150bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924150c7, 55 },
    { 0x56c150c8, 90 },
    { 0xde6150c7, 95 },
    { 0xc7e150ca, 87 },
    { 0x23b150cb, 34 },
    { 0xbce150cb, 95 },
    { 0xf37150c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b150d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4150dd, 90 },
    { 0xe8b150dd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9150e5, 52 },
    { 0xb5f150e6, 90 },
    { 0x697150e7, 90 },
    { 0xa05150e8, 95 },
    { 0x5bd150e5, 55 },
    { 0x5e8150e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x947150ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x471150f0, 90 },
    { 0xe3f150f1, 33 },
    { 0, 0 },
    { 0xa46150f3, 90 },
    { 0, 0 },
    { 0x6a8150f5, 43 },
    { 0xac8150f6, 23 },
    { 0xd8b150f6, 33 },
    { 0xbc3150f7, 91 },
    { 0x3fe150f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e15105, 95 },
    { 0, 0 },
    { 0xd2415107, 23 },
    { 0xb8815107, 95 },
    { 0x2c015109, 67 },
    { 0x14f15109, 90 },
    { 0, 0 },
    { 0x3111510c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe215110, 61 },
    { 0x4115111, 95 },
    { 0x2cc15112, 76 },
    { 0xa2115113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f15116, 98 },
    { 0xe7215117, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd15122, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8615126, 90 },
    { 0, 0 },
    { 0x93215128, 61 },
    { 0, 0 },
    { 0x9091512a, 64 },
    { 0x8e61512b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5615130, 61 },
    { 0x9c115130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac715138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d1513d, 33 },
    { 0, 0 },
    { 0x7481513f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f715142, 43 },
    { 0x24d15143, 33 },
    { 0x7ad15143, 33 },
    { 0xab615144, 33 },
    { 0xeeb15146, 90 },
    { 0xf2815143, 61 },
    { 0x69a15145, 34 },
    { 0x47e15149, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce115150, 55 },
    { 0xe6215150, 33 },
    { 0xad915152, 90 },
    { 0x3c115152, 95 },
    { 0x43715154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd15158, 76 },
    { 0x22015159, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1515c, 96 },
    { 0x3e61515d, 91 },
    { 0x3e51515c, 23 },
    { 0xfe01515c, 23 },
    { 0x21415160, 90 },
    { 0xc6615161, 80 },
    { 0x8b015162, 43 },
    { 0x3b81515c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a515168, 93 },
    { 0x7a115169, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc271516e, 23 },
    { 0xd8a1516f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed215174, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f415177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa971517a, 87 },
    { 0xaa81517b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d41517f, 34 },
    { 0xeb215180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e15183, 90 },
    { 0, 0 },
    { 0x68a15185, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e91518f, 85 },
    { 0, 0 },
    { 0xce815191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a715195, 100 },
    { 0xa9715196, 34 },
    { 0, 0 },
    { 0x37715198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb71519e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50151a3, 33 },
    { 0, 0 },
    { 0xe8b151a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7151a9, 91 },
    { 0xbd8151a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x793151ad, 85 },
    { 0xd75151ae, 61 },
    { 0, 0 },
    { 0x738151b0, 76 },
    { 0x88b151b0, 61 },
    { 0, 0 },
    { 0x631151b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x397151b6, 33 },
    { 0x3be151b7, 98 },
    { 0xaa5151b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8151bb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17151be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e151c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815151c8, 91 },
    { 0x430151c9, 61 },
    { 0x35d151c9, 90 },
    { 0x716151c9, 87 },
    { 0xbb9151cc, 23 },
    { 0x690151cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2151d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd151d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c151d8, 46 },
    { 0xfe4151d9, 98 },
    { 0x477151d9, 80 },
    { 0x4e1151db, 33 },
    { 0xc66151dc, 88 },
    { 0xc34151dd, 53 },
    { 0x5a1151de, 55 },
    { 0x3b9151df, 93 },
    { 0xa3f151db, 40 },
    { 0x14b151e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7151e7, 33 },
    { 0x1da151e8, 61 },
    { 0x7ba151e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10151ed, 44 },
    { 0x796151ee, 93 },
    { 0xd70151ef, 100 },
    { 0x5de151ef, 81 },
    { 0xd88151ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c151f8, 44 },
    { 0xc7a151f9, 90 },
    { 0x6ef151f9, 76 },
    { 0x7f3151fb, 61 },
    { 0xdd2151fc, 90 },
    { 0xd98151fd, 23 },
    { 0xbcf151fb, 61 },
    { 0x544151fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b15202, 80 },
    { 0x76c15202, 61 },
    { 0, 0 },
    { 0xb9e15205, 23 },
    { 0, 0 },
    { 0xb7b15207, 33 },
    { 0, 0 },
    { 0xd8915209, 93 },
    { 0, 0 },
    { 0xc6d1520b, 76 },
    { 0xf351520b, 23 },
    { 0, 0 },
    { 0x83f1520e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae15212, 23 },
    { 0, 0 },
    { 0x7a015214, 98 },
    { 0x52515214, 61 },
    { 0, 0 },
    { 0x1b215217, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec41521c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a1521f, 61 },
    { 0x79f15220, 87 },
    { 0xb8615221, 33 },
    { 0x3a915222, 33 },
    { 0, 0 },
    { 0xf1c15224, 23 },
    { 0x6e415225, 61 },
    { 0x36715226, 55 },
    { 0x8f115226, 61 },
    { 0x6f815227, 95 },
    { 0, 0 },
    { 0x6921522a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a1522e, 76 },
    { 0x4e31522f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb15232, 23 },
    { 0xd1315232, 23 },
    { 0xc4815232, 43 },
    { 0x6f715235, 80 },
    { 0xbce15235, 76 },
    { 0xeeb15234, 33 },
    { 0x58115238, 99 },
    { 0xb7715239, 87 },
    { 0x8561523a, 98 },
    { 0xf9e15237, 88 },
    { 0xf0415238, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81523f, 80 },
    { 0, 0 },
    { 0xeff15241, 33 },
    { 0x5f15242, 98 },
    { 0x87b15242, 23 },
    { 0x33415244, 67 },
    { 0x89915244, 64 },
    { 0xe415243, 98 },
    { 0xf0e15247, 61 },
    { 0xc2415248, 40 },
    { 0x95b15248, 92 },
    { 0xa7f15242, 90 },
    { 0xf9a15241, 23 },
    { 0x6341524c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33415250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f215253, 46 },
    { 0x7ea15254, 98 },
    { 0x4315254, 98 },
    { 0x94a15255, 81 },
    { 0, 0 },
    { 0xeb815258, 91 },
    { 0x82c15259, 87 },
    { 0, 0 },
    { 0x8481525b, 34 },
    { 0x4401525c, 95 },
    { 0xe011525d, 33 },
    { 0xfc71525e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd15263, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3541526e, 98 },
    { 0, 0 },
    { 0x93e15270, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2315277, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b1527a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9315280, 95 },
    { 0x7a415281, 90 },
    { 0xe7715282, 98 },
    { 0x65d15283, 76 },
    { 0xbf015284, 67 },
    { 0x5f515285, 78 },
    { 0x24a15286, 33 },
    { 0x64f15287, 76 },
    { 0, 0 },
    { 0x95d15289, 93 },
    { 0xa771528a, 76 },
    { 0x8921528b, 23 },
    { 0x471528c, 23 },
    { 0x59b1528d, 33 },
    { 0xd2f1528e, 98 },
    { 0x4db1528f, 95 },
    { 0x2d15290, 92 },
    { 0x415291, 90 },
    { 0x3cc15292, 93 },
    { 0xf7c15293, 98 },
    { 0x5f015290, 90 },
    { 0xd2f15295, 78 },
    { 0x89a15295, 23 },
    { 0xfb215297, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e1529b, 23 },
    { 0, 0 },
    { 0xcdb1529d, 36 },
    { 0xcbf1529e, 95 },
    { 0x47f1529f, 98 },
    { 0x241152a0, 90 },
    { 0xc3c152a0, 90 },
    { 0xd10152a2, 87 },
    { 0xcc152a2, 33 },
    { 0xa66152a2, 23 },
    { 0xc81152a0, 33 },
    { 0x20b152a6, 33 },
    { 0xe5d152a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf152ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd152b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b152b6, 34 },
    { 0x8e9152b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78152bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0152c0, 66 },
    { 0x37152c1, 90 },
    { 0x1c1152c2, 76 },
    { 0, 0 },
    { 0x9a9152c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961152c9, 33 },
    { 0xbbd152ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561152d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x171152d7, 76 },
    { 0, 0 },
    { 0xe3152d9, 47 },
    { 0x745152da, 100 },
    { 0, 0 },
    { 0x17c152dc, 81 },
    { 0xf82152dd, 81 },
    { 0x150152dc, 33 },
    { 0xc9b152df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd152e8, 90 },
    { 0, 0 },
    { 0x5c152ea, 33 },
    { 0, 0 },
    { 0xf06152ec, 78 },
    { 0x344152ed, 61 },
    { 0x384152ee, 76 },
    { 0xf08152ee, 93 },
    { 0x624152ee, 95 },
    { 0xbc152f0, 38 },
    { 0, 0 },
    { 0xec0152f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x871152f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31152fa, 93 },
    { 0xbe8152fa, 61 },
    { 0, 0 },
    { 0xd23152fd, 90 },
    { 0, 0 },
    { 0x384152ff, 76 },
    { 0x49f15300, 33 },
    { 0x61315300, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e15307, 85 },
    { 0xcc815308, 43 },
    { 0, 0 },
    { 0x11d1530a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37015314, 98 },
    { 0, 0 },
    { 0xba115316, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6815319, 61 },
    { 0, 0 },
    { 0xe131531b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1531f, 66 },
    { 0x4bb15320, 93 },
    { 0x59115321, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a15327, 65 },
    { 0x27515328, 66 },
    { 0x9d915329, 23 },
    { 0x2231532a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2841532d, 55 },
    { 0x1b51532e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d15334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f315338, 38 },
    { 0x5aa15339, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b21533d, 23 },
    { 0x6611533e, 87 },
    { 0xdc81533d, 95 },
    { 0, 0 },
    { 0x5df15341, 93 },
    { 0x1b115342, 93 },
    { 0x69015342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4215346, 43 },
    { 0x10215347, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1231534e, 88 },
    { 0xfbb1534f, 98 },
    { 0xd5b15350, 76 },
    { 0xa1e15350, 61 },
    { 0xf4615352, 87 },
    { 0x9731534f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4815359, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c1535f, 61 },
    { 0, 0 },
    { 0xdd615361, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88515366, 33 },
    { 0, 0 },
    { 0x93b15368, 90 },
    { 0xdf315369, 76 },
    { 0x63e15368, 98 },
    { 0xd6d15368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f1536e, 91 },
    { 0x9481536e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e415372, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8461537b, 76 },
    { 0xbe91537c, 17 },
    { 0x7271537b, 95 },
    { 0x1751537d, 46 },
    { 0xd321537e, 99 },
    { 0x32d15380, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff15386, 91 },
    { 0xb5c15387, 52 },
    { 0x5ba15388, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a1538c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74815392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea15395, 33 },
    { 0x7915396, 61 },
    { 0, 0 },
    { 0x36615398, 98 },
    { 0xa6215399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2881539c, 61 },
    { 0, 0 },
    { 0x3341539e, 61 },
    { 0x9061539f, 43 },
    { 0x8831539f, 43 },
    { 0xf9153a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b153a5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83153ab, 55 },
    { 0xd71153ac, 99 },
    { 0, 0 },
    { 0x7de153ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7153b1, 90 },
    { 0, 0 },
    { 0x7b3153b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1153b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56153bd, 91 },
    { 0x24a153bd, 77 },
    { 0x6b153bf, 33 },
    { 0x938153be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68153c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc153c7, 95 },
    { 0xfe7153c8, 65 },
    { 0xe46153c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd153cd, 68 },
    { 0, 0 },
    { 0x2d153cf, 76 },
    { 0, 0 },
    { 0xfb2153d1, 38 },
    { 0, 0 },
    { 0x14b153d3, 87 },
    { 0x59b153d3, 90 },
    { 0x1c2153d5, 100 },
    { 0x1d153d6, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929153dd, 61 },
    { 0x8b5153de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9153e3, 90 },
    { 0xdb153e4, 33 },
    { 0, 0 },
    { 0x250153e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804153eb, 61 },
    { 0, 0 },
    { 0xf88153ed, 95 },
    { 0x94c153ee, 33 },
    { 0, 0 },
    { 0xabd153f0, 90 },
    { 0xf92153f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a153f4, 76 },
    { 0x191153f4, 98 },
    { 0, 0 },
    { 0x37c153f7, 33 },
    { 0xd4c153f7, 43 },
    { 0xfa3153f9, 66 },
    { 0xea6153f9, 99 },
    { 0xbf3153fb, 43 },
    { 0x1af153f9, 61 },
    { 0x419153f9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b15401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf215407, 33 },
    { 0x76a15408, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a31540d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd15410, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79315415, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1541b, 52 },
    { 0, 0 },
    { 0x2231541d, 61 },
    { 0xa381541e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7815421, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b15425, 85 },
    { 0, 0 },
    { 0x63515427, 88 },
    { 0x34815427, 78 },
    { 0, 0 },
    { 0x2cc1542a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf21542d, 69 },
    { 0xbb11542e, 17 },
    { 0xb951542d, 95 },
    { 0, 0 },
    { 0x9515431, 95 },
    { 0, 0 },
    { 0x3f815433, 98 },
    { 0x51515434, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff15439, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc141543e, 55 },
    { 0, 0 },
    { 0x3bb15440, 43 },
    { 0xdc415441, 85 },
    { 0x16d15442, 34 },
    { 0x1b815443, 33 },
    { 0x27015440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49015447, 98 },
    { 0x28315448, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a91544f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b15458, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78815460, 66 },
    { 0, 0 },
    { 0xfe315462, 98 },
    { 0, 0 },
    { 0xd8115464, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e15467, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61546c, 90 },
    { 0x6e51546d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1415472, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a415475, 90 },
    { 0xa9415475, 94 },
    { 0xe8f15477, 33 },
    { 0x14415478, 61 },
    { 0xacf15475, 76 },
    { 0xcee1547a, 90 },
    { 0x6821547b, 55 },
    { 0x7e615475, 98 },
    { 0xfb715477, 33 },
    { 0x20e1547e, 99 },
    { 0x81a1547b, 96 },
    { 0, 0 },
    { 0x97015481, 90 },
    { 0, 0 },
    { 0x88415483, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba15487, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1548b, 64 },
    { 0x83b1548c, 55 },
    { 0x1531548d, 100 },
    { 0x9851548c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf815493, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa1549e, 43 },
    { 0x5ae1549f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d154a2, 61 },
    { 0, 0 },
    { 0xa84154a4, 69 },
    { 0x6cf154a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835154ab, 99 },
    { 0xd61154ac, 90 },
    { 0x348154ad, 33 },
    { 0, 0 },
    { 0x73b154af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a154b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e154b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0154c4, 61 },
    { 0xae9154c4, 95 },
    { 0xa7a154c5, 33 },
    { 0x9f1154c7, 48 },
    { 0x3dc154c5, 87 },
    { 0x84e154c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40154cd, 95 },
    { 0x865154ce, 90 },
    { 0, 0 },
    { 0x340154d0, 46 },
    { 0x2ca154d1, 93 },
    { 0x530154d1, 34 },
    { 0x790154d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b154d6, 43 },
    { 0xdd2154d7, 100 },
    { 0xfe0154d7, 76 },
    { 0xd5d154d7, 55 },
    { 0xcc2154d6, 76 },
    { 0x32d154d9, 84 },
    { 0xe4d154dc, 87 },
    { 0xcbe154dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84154e0, 55 },
    { 0xa27154e1, 23 },
    { 0x334154e2, 78 },
    { 0, 0 },
    { 0x4ef154e4, 61 },
    { 0xf3e154e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335154ec, 55 },
    { 0x3a2154ed, 76 },
    { 0x67b154ec, 95 },
    { 0x8df154ef, 56 },
    { 0, 0 },
    { 0x9ac154f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c154f7, 84 },
    { 0, 0 },
    { 0x26a154f9, 61 },
    { 0xd67154fa, 60 },
    { 0x1ea154fb, 43 },
    { 0, 0 },
    { 0x12a154fd, 34 },
    { 0xda3154fe, 93 },
    { 0x6ba154fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98715503, 76 },
    { 0, 0 },
    { 0x26615505, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21615509, 95 },
    { 0x33f1550a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1515512, 61 },
    { 0xbea15513, 75 },
    { 0, 0 },
    { 0xc2915515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb115519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf1551f, 34 },
    { 0xf5e1551f, 100 },
    { 0xd1215521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b615524, 33 },
    { 0x2e415525, 90 },
    { 0, 0 },
    { 0x30415527, 52 },
    { 0, 0 },
    { 0x42815529, 61 },
    { 0, 0 },
    { 0xcb31552b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191552e, 34 },
    { 0x8651552f, 99 },
    { 0x5315530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3115534, 61 },
    { 0x8c615534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe15538, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101553d, 100 },
    { 0x3f81553d, 33 },
    { 0x1371553d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee15542, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd815545, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3191555a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b15560, 61 },
    { 0, 0 },
    { 0x8fe15562, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28515566, 95 },
    { 0xb5c15567, 98 },
    { 0x9b915568, 34 },
    { 0x85315568, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801556e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79415575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3451557b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6971557e, 87 },
    { 0x9ea1557f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41515583, 98 },
    { 0, 0 },
    { 0x54015585, 43 },
    { 0, 0 },
    { 0xbb315587, 95 },
    { 0x5df15588, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd1558b, 23 },
    { 0x3ed1558b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b15590, 74 },
    { 0x75515591, 23 },
    { 0x8ea15591, 33 },
    { 0x59b15592, 61 },
    { 0xa015594, 23 },
    { 0, 0 },
    { 0xec415596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5815599, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb901559e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7155a1, 23 },
    { 0x3dc155a1, 90 },
    { 0xd37155a3, 23 },
    { 0, 0 },
    { 0x28e155a5, 66 },
    { 0, 0 },
    { 0x18e155a7, 90 },
    { 0xd6155a8, 99 },
    { 0, 0 },
    { 0xc58155aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb155b2, 77 },
    { 0xb57155b3, 52 },
    { 0, 0 },
    { 0x7ce155b5, 33 },
    { 0xbb155b6, 33 },
    { 0x149155b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160155bc, 33 },
    { 0x383155bd, 95 },
    { 0xc0f155be, 91 },
    { 0x7f4155bc, 46 },
    { 0x2e9155c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6155c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0155cf, 98 },
    { 0, 0 },
    { 0x804155d1, 98 },
    { 0x34155d1, 33 },
    { 0xba8155d2, 33 },
    { 0, 0 },
    { 0x356155d5, 33 },
    { 0xf41155d6, 23 },
    { 0x49d155d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6155dc, 95 },
    { 0, 0 },
    { 0x6b6155de, 91 },
    { 0x8c2155de, 61 },
    { 0xa5f155de, 55 },
    { 0, 0 },
    { 0x7e9155e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a155e5, 98 },
    { 0, 0 },
    { 0x3f3155e7, 99 },
    { 0x42c155e8, 23 },
    { 0x7bf155e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x923155ec, 76 },
    { 0x7bf155ed, 61 },
    { 0xe91155ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5155f1, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b155f4, 76 },
    { 0xf75155f5, 43 },
    { 0xa8e155f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84155f9, 75 },
    { 0x100155fa, 95 },
    { 0, 0 },
    { 0x5f3155fc, 34 },
    { 0xd8e155fd, 23 },
    { 0xba155fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x98015601, 46 },
    { 0x1fa15601, 46 },
    { 0, 0 },
    { 0xf0e15604, 96 },
    { 0x2fe15604, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e15611, 84 },
    { 0x97615611, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6631561b, 87 },
    { 0x2bc1561c, 90 },
    { 0x291561b, 22 },
    { 0xe501561e, 66 },
    { 0x1871561f, 87 },
    { 0x7601561f, 33 },
    { 0x51315620, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c15626, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d1562b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d61563a, 93 },
    { 0xc251563b, 33 },
    { 0, 0 },
    { 0x4d81563d, 98 },
    { 0xcf1563d, 52 },
    { 0x7a81563f, 67 },
    { 0xb0c1563e, 61 },
    { 0xadf15641, 90 },
    { 0x76c15642, 98 },
    { 0x77c15643, 53 },
    { 0x3ea15640, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af15652, 23 },
    { 0, 0 },
    { 0xf1b15654, 90 },
    { 0, 0 },
    { 0xdc515656, 95 },
    { 0xe6815657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5121565b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b15661, 61 },
    { 0xaf715662, 33 },
    { 0x415663, 90 },
    { 0xd6e15664, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27815667, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd515672, 87 },
    { 0xfcf15673, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b15677, 90 },
    { 0x7da15678, 67 },
    { 0xcb15677, 61 },
    { 0xdf315677, 66 },
    { 0xf261567b, 90 },
    { 0xf1d1567b, 93 },
    { 0x4c61567c, 33 },
    { 0x261567e, 52 },
    { 0x8ff1567f, 90 },
    { 0xae415680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9515683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6315688, 33 },
    { 0x66915689, 76 },
    { 0x22e15689, 55 },
    { 0xbde1568b, 46 },
    { 0xa5d1568c, 90 },
    { 0x5ea1568c, 90 },
    { 0x6ad1568d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99715692, 61 },
    { 0x3f15692, 61 },
    { 0, 0 },
    { 0x92a15695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e215698, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d156a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22156a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06156ab, 98 },
    { 0, 0 },
    { 0x6bc156ad, 95 },
    { 0, 0 },
    { 0x67a156af, 23 },
    { 0xf45156b0, 98 },
    { 0, 0 },
    { 0xe7b156b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d156b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c156ba, 61 },
    { 0x128156bb, 81 },
    { 0xf0c156bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3156c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a156c8, 90 },
    { 0xb02156c9, 34 },
    { 0xaf7156c9, 33 },
    { 0, 0 },
    { 0x8a5156cc, 77 },
    { 0, 0 },
    { 0xaf9156ce, 87 },
    { 0, 0 },
    { 0xe1b156d0, 61 },
    { 0xdd5156d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a156d6, 55 },
    { 0, 0 },
    { 0xa60156d8, 90 },
    { 0xa56156d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73156dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b156e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861156e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68156ea, 95 },
    { 0x97d156ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2156ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb156f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1156f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f156f8, 55 },
    { 0xcf7156f9, 91 },
    { 0x35a156fa, 82 },
    { 0, 0 },
    { 0x1de156fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x914156ff, 90 },
    { 0, 0 },
    { 0xdc515701, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a15704, 76 },
    { 0xd1315705, 23 },
    { 0, 0 },
    { 0xf9515707, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc41570a, 33 },
    { 0x3c71570b, 61 },
    { 0x9191570c, 69 },
    { 0x3531570d, 100 },
    { 0x97e1570d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea215712, 55 },
    { 0x6fe15713, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7961571d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72115721, 90 },
    { 0x76115722, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6731572e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x50915731, 98 },
    { 0x78c15732, 52 },
    { 0x35015733, 87 },
    { 0xdad15732, 61 },
    { 0xb815732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc1573d, 84 },
    { 0xf531573d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2815745, 94 },
    { 0, 0 },
    { 0xf0815747, 90 },
    { 0x6315747, 65 },
    { 0x67715749, 78 },
    { 0x1901574a, 53 },
    { 0x2801574b, 46 },
    { 0x91115749, 90 },
    { 0, 0 },
    { 0x9451574e, 77 },
    { 0xb291574f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371575b, 34 },
    { 0x5f31575b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23515764, 33 },
    { 0, 0 },
    { 0x69715766, 55 },
    { 0xbba15767, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e1576a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1576f, 77 },
    { 0x2061576f, 98 },
    { 0xa1c15770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff15774, 94 },
    { 0x12d15775, 33 },
    { 0, 0 },
    { 0x5db15777, 33 },
    { 0x1c115778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f315786, 78 },
    { 0x4e715786, 33 },
    { 0, 0 },
    { 0x97515789, 67 },
    { 0x63b1578a, 61 },
    { 0, 0 },
    { 0xc9a1578c, 33 },
    { 0x2951578c, 33 },
    { 0xf041578c, 33 },
    { 0xd951578e, 34 },
    { 0xe8f1578f, 33 },
    { 0x6541578f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd115796, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x12515799, 61 },
    { 0xf2b1579a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f1579d, 34 },
    { 0, 0 },
    { 0x5691579f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa157a6, 55 },
    { 0xc64157a7, 55 },
    { 0x9ca157a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301157ae, 34 },
    { 0x2f1157ae, 61 },
    { 0, 0 },
    { 0xc3157b1, 84 },
    { 0xe9d157b2, 61 },
    { 0x93a157b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e157be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24157c1, 95 },
    { 0, 0 },
    { 0xe3b157c3, 78 },
    { 0x57d157c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05157ca, 43 },
    { 0xcbe157cb, 61 },
    { 0x666157cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49157cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112157d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7157e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8157f9, 33 },
    { 0, 0 },
    { 0xaa4157fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x765157fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3915801, 67 },
    { 0x35f15802, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99015808, 43 },
    { 0x24615809, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8281580d, 100 },
    { 0, 0 },
    { 0x2971580f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc15813, 78 },
    { 0x28115814, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa615818, 90 },
    { 0, 0 },
    { 0x95e1581a, 93 },
    { 0xf261581b, 55 },
    { 0x20f1581c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f15822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7715826, 90 },
    { 0xc1515827, 95 },
    { 0x59515828, 90 },
    { 0x3fc15829, 33 },
    { 0xfbc1582a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa1582d, 77 },
    { 0x4941582d, 69 },
    { 0x821582e, 90 },
    { 0xab115830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21715836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c15839, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce815841, 23 },
    { 0x83115842, 33 },
    { 0, 0 },
    { 0xd7e15844, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a1584c, 70 },
    { 0xf691584d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b15854, 87 },
    { 0x58315855, 33 },
    { 0xe4315856, 33 },
    { 0x74215857, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f1585a, 98 },
    { 0xf7d1585b, 34 },
    { 0x4341585b, 55 },
    { 0x1441585b, 99 },
    { 0, 0 },
    { 0xcca1585f, 55 },
    { 0x3c815860, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015863, 33 },
    { 0, 0 },
    { 0x1b815865, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a15868, 34 },
    { 0, 0 },
    { 0x331586a, 76 },
    { 0, 0 },
    { 0x81586c, 33 },
    { 0xc1a1586c, 55 },
    { 0x3401586e, 33 },
    { 0, 0 },
    { 0x65915870, 98 },
    { 0xe4e15871, 98 },
    { 0x73d15871, 23 },
    { 0, 0 },
    { 0xdb915874, 99 },
    { 0xd2015875, 90 },
    { 0xbde15875, 67 },
    { 0x1d815877, 97 },
    { 0xa9515874, 98 },
    { 0x62615879, 90 },
    { 0x24a1587a, 95 },
    { 0x3e215879, 98 },
    { 0x5c51587c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d01587f, 33 },
    { 0, 0 },
    { 0xccd15881, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c15887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a61588a, 46 },
    { 0x59c1588b, 52 },
    { 0xd3c1588b, 23 },
    { 0, 0 },
    { 0xed71588e, 55 },
    { 0xce41588f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58115892, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb615896, 91 },
    { 0xf3e15897, 90 },
    { 0xb3115898, 99 },
    { 0, 0 },
    { 0x7491589a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1589e, 34 },
    { 0xd1f1589f, 34 },
    { 0x82c1589f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x67158a3, 17 },
    { 0xaf8158a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff158a7, 61 },
    { 0xe0d158a8, 33 },
    { 0, 0 },
    { 0x30f158aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x395158ad, 87 },
    { 0xd51158ad, 55 },
    { 0xb3c158af, 61 },
    { 0x7b8158b0, 98 },
    { 0x86c158b1, 88 },
    { 0, 0 },
    { 0x4ca158b3, 61 },
    { 0xea7158b4, 61 },
    { 0x96b158b4, 40 },
    { 0, 0 },
    { 0x1a4158b7, 33 },
    { 0x23b158b8, 67 },
    { 0x97a158b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7158bc, 87 },
    { 0xd3d158bc, 98 },
    { 0x3f8158be, 98 },
    { 0, 0 },
    { 0x859158c0, 43 },
    { 0x6e6158c1, 90 },
    { 0xdf9158c1, 61 },
    { 0x6f4158c3, 90 },
    { 0xa04158c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44158cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa158d0, 68 },
    { 0xa34158d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567158de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96158e1, 59 },
    { 0xca4158e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2158e5, 98 },
    { 0xbb5158e6, 64 },
    { 0x75f158e6, 90 },
    { 0x404158e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e158ed, 55 },
    { 0x882158ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff158f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4158fb, 98 },
    { 0x7f7158fc, 84 },
    { 0x94e158fd, 90 },
    { 0xad7158fe, 90 },
    { 0x2dc158ff, 97 },
    { 0, 0 },
    { 0x45615901, 84 },
    { 0, 0 },
    { 0x4a615903, 33 },
    { 0x7bd15904, 91 },
    { 0x38215905, 33 },
    { 0xf0115905, 91 },
    { 0x84615905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a1590b, 84 },
    { 0x5321590c, 76 },
    { 0, 0 },
    { 0x4f1590e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3115912, 67 },
    { 0xe1215913, 23 },
    { 0x64615914, 66 },
    { 0xa8215912, 33 },
    { 0xedb15916, 87 },
    { 0, 0 },
    { 0xaab15918, 34 },
    { 0, 0 },
    { 0x4b21591a, 98 },
    { 0, 0 },
    { 0x5861591c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e15921, 84 },
    { 0x5f315922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b015926, 61 },
    { 0x2e315926, 34 },
    { 0x2d515927, 90 },
    { 0xbc815927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb691592f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a15933, 61 },
    { 0x3a815933, 90 },
    { 0xbfe15934, 33 },
    { 0x42b15936, 78 },
    { 0xa515935, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe081593c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b715944, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x85015947, 95 },
    { 0x10815948, 33 },
    { 0x7c15949, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75615952, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a15955, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d15958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921595d, 17 },
    { 0x5cb1595e, 93 },
    { 0xa3c1595f, 90 },
    { 0x8191595f, 76 },
    { 0x25a15961, 33 },
    { 0, 0 },
    { 0x3a515963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6215969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe031596c, 33 },
    { 0xc41596d, 90 },
    { 0xbdd1596e, 98 },
    { 0xfd71596e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d715973, 90 },
    { 0, 0 },
    { 0xd615975, 92 },
    { 0, 0 },
    { 0xd0215977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf081597b, 95 },
    { 0x611597b, 23 },
    { 0, 0 },
    { 0xfdc1597e, 98 },
    { 0, 0 },
    { 0x38b15980, 96 },
    { 0xc9b15981, 98 },
    { 0x4b015982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee715986, 84 },
    { 0, 0 },
    { 0x4fd15988, 97 },
    { 0xa8015989, 33 },
    { 0x45415989, 95 },
    { 0xaac15989, 98 },
    { 0xd631598c, 90 },
    { 0, 0 },
    { 0x4931598e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a15991, 23 },
    { 0xdc115992, 83 },
    { 0xc2315991, 61 },
    { 0x38c15994, 23 },
    { 0x1bb15995, 98 },
    { 0xd8315995, 91 },
    { 0x22115992, 34 },
    { 0xe4a15998, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e21599c, 95 },
    { 0xe7f1599d, 61 },
    { 0xd3b1599c, 61 },
    { 0, 0 },
    { 0x9cf159a0, 33 },
    { 0x28a159a1, 87 },
    { 0xa45159a2, 33 },
    { 0x224159a3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98159a7, 91 },
    { 0x9d2159a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00159ab, 90 },
    { 0x829159ac, 98 },
    { 0x603159ac, 95 },
    { 0x987159ae, 61 },
    { 0x46b159ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8159b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e159bb, 61 },
    { 0, 0 },
    { 0xf67159bd, 93 },
    { 0x9b1159be, 98 },
    { 0xe99159bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f159c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9159cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38159d7, 23 },
    { 0, 0 },
    { 0x9ab159d9, 90 },
    { 0x158159da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0159de, 98 },
    { 0xeec159de, 66 },
    { 0xe7e159de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc159e3, 90 },
    { 0, 0 },
    { 0x86b159e5, 66 },
    { 0, 0 },
    { 0x3a5159e7, 23 },
    { 0x64e159e8, 55 },
    { 0xf36159e9, 23 },
    { 0x278159e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a159ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8159f1, 98 },
    { 0, 0 },
    { 0x1f0159f3, 76 },
    { 0x53a159f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9159f7, 55 },
    { 0, 0 },
    { 0xbe5159f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d159fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15a00, 91 },
    { 0x6e715a01, 76 },
    { 0, 0 },
    { 0xfd415a03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b215a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54315a12, 91 },
    { 0, 0 },
    { 0xc0515a14, 90 },
    { 0xa2315a15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a515a20, 98 },
    { 0, 0 },
    { 0x41815a22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f15a26, 95 },
    { 0, 0 },
    { 0xb3215a28, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b15a2b, 33 },
    { 0xd5d15a2c, 76 },
    { 0x69715a2d, 100 },
    { 0xbbc15a2d, 91 },
    { 0x54815a2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd15a35, 23 },
    { 0x7b915a35, 95 },
    { 0xdfd15a37, 95 },
    { 0x50f15a38, 23 },
    { 0xa3d15a39, 23 },
    { 0x75615a39, 95 },
    { 0x58c15a37, 33 },
    { 0xeb215a37, 84 },
    { 0x28815a38, 98 },
    { 0xfb815a39, 66 },
    { 0xff015a3f, 23 },
    { 0x26d15a40, 52 },
    { 0x6df15a3f, 95 },
    { 0x92115a3a, 33 },
    { 0xca615a3c, 76 },
    { 0x59c15a38, 43 },
    { 0x9a15a45, 90 },
    { 0x70c15a45, 95 },
    { 0x83815a47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e215a4b, 61 },
    { 0x43c15a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d15a4f, 53 },
    { 0x4c115a50, 59 },
    { 0, 0 },
    { 0x9c615a52, 100 },
    { 0x5e515a52, 80 },
    { 0, 0 },
    { 0xb9715a55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91915a59, 90 },
    { 0x5c115a59, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x28315a5d, 84 },
    { 0xe2d15a5e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed15a68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b215a6e, 61 },
    { 0x65f15a6f, 65 },
    { 0x7d515a6f, 76 },
    { 0xcb615a71, 23 },
    { 0x58515a72, 55 },
    { 0, 0 },
    { 0xf915a74, 95 },
    { 0x92e15a74, 76 },
    { 0x83915a76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43715a7c, 98 },
    { 0x46e15a7c, 46 },
    { 0xf0e15a7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae15a83, 66 },
    { 0x85a15a83, 33 },
    { 0xaf215a84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e215a8d, 34 },
    { 0x1e15a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8815a93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d15a9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd215a9f, 33 },
    { 0xf5715a9f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b15aa5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93915aa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e515aac, 90 },
    { 0x11715aac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73715ab0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x44315ab3, 95 },
    { 0x40b15ab3, 78 },
    { 0x89a15ab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf15ab8, 61 },
    { 0x93115ab9, 46 },
    { 0, 0 },
    { 0x76415abb, 43 },
    { 0, 0 },
    { 0x4e115abd, 90 },
    { 0xef715abe, 33 },
    { 0, 0 },
    { 0x4cb15ac0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10515ac5, 95 },
    { 0xe9815ac6, 43 },
    { 0x53f15ac6, 90 },
    { 0xf4815ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f15acb, 90 },
    { 0x61315acb, 98 },
    { 0xbf315acc, 95 },
    { 0xb1c15ace, 61 },
    { 0x25d15acf, 52 },
    { 0x32015ace, 91 },
    { 0x3a915acf, 99 },
    { 0xeb715acd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a15ad5, 67 },
    { 0, 0 },
    { 0xb0915ad7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96915ada, 90 },
    { 0x70615adb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x96615ade, 93 },
    { 0xd8415adf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x27015ae2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7415ae5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7115ae9, 98 },
    { 0x84a15aea, 98 },
    { 0x61315aea, 33 },
    { 0x5b815aeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd915af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a15af7, 23 },
    { 0xb9d15af8, 23 },
    { 0xbe815af7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f015afc, 93 },
    { 0xd9815afd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a15b04, 23 },
    { 0x7cc15b05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e515b09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f15b0c, 98 },
    { 0x89715b0c, 61 },
    { 0xa2e15b0e, 98 },
    { 0xbf515b0f, 17 },
    { 0xfb615b10, 46 },
    { 0x28f15b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc15b16, 61 },
    { 0x9fc15b17, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c415b1c, 61 },
    { 0xfb215b1d, 31 },
    { 0, 0 },
    { 0x2ee15b1f, 90 },
    { 0xaa415b20, 98 },
    { 0xad315b21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f15b29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6715b2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1415b33, 76 },
    { 0x3db15b33, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf15b37, 33 },
    { 0x8ec15b37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6015b3b, 55 },
    { 0, 0 },
    { 0x8c515b3d, 98 },
    { 0x77b15b3e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815b41, 53 },
    { 0xd7a15b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85515b49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa15b4c, 38 },
    { 0x9a515b4d, 68 },
    { 0x41e15b4e, 98 },
    { 0, 0 },
    { 0xc3a15b50, 33 },
    { 0xc2015b51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb15b54, 33 },
    { 0xf6e15b55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4615b5a, 33 },
    { 0, 0 },
    { 0x75715b5c, 87 },
    { 0x89815b5c, 33 },
    { 0, 0 },
    { 0x42915b5f, 68 },
    { 0xa2215b5f, 90 },
    { 0xd115b61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d15b68, 46 },
    { 0, 0 },
    { 0xeb615b6a, 43 },
    { 0x7f115b6b, 40 },
    { 0x9d415b6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad15b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c015b82, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4015b85, 61 },
    { 0, 0 },
    { 0x46215b87, 90 },
    { 0xe9815b88, 97 },
    { 0x44115b87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11415b8c, 61 },
    { 0, 0 },
    { 0xa6515b8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1115b91, 93 },
    { 0x55f15b92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8115b95, 88 },
    { 0xd215b96, 33 },
    { 0x36615b97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb315b9d, 91 },
    { 0x6a015b9e, 52 },
    { 0x4215b9f, 95 },
    { 0x80415ba0, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8715ba3, 77 },
    { 0x4d215ba4, 90 },
    { 0x8b915ba3, 61 },
    { 0, 0 },
    { 0x30815ba7, 61 },
    { 0xca915ba8, 23 },
    { 0xbef15ba8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x46115bac, 90 },
    { 0xb1a15bad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc515bb1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x17615bb4, 46 },
    { 0x34615bb5, 90 },
    { 0xbbe15bb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85115bbe, 95 },
    { 0x3ea15bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c915bc2, 33 },
    { 0, 0 },
    { 0x38e15bc4, 61 },
    { 0x8ff15bc5, 90 },
    { 0x84f15bc6, 90 },
    { 0, 0 },
    { 0x8ec15bc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6315bcb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x55615bce, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0215bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38615bda, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e15bdd, 61 },
    { 0x9be15bde, 46 },
    { 0xa0915bde, 43 },
    { 0x47215be0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef615be4, 17 },
    { 0xb7515be5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b215bea, 67 },
    { 0x5e615beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e15bee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xded15bf1, 66 },
    { 0xd4315bf1, 85 },
    { 0, 0 },
    { 0x98415bf4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc15bfe, 23 },
    { 0, 0 },
    { 0xd3415c00, 33 },
    { 0x38d15c00, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815c09, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b15c15, 85 },
    { 0x26315c15, 95 },
    { 0, 0 },
    { 0x7c515c18, 95 },
    { 0x5ad15c19, 87 },
    { 0xfaa15c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f15c1d, 55 },
    { 0x9c415c1d, 76 },
    { 0x68015c1d, 99 },
    { 0x75615c20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a15c26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a15c29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3815c2f, 33 },
    { 0xca115c30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd15c34, 61 },
    { 0x6f715c35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a15c38, 46 },
    { 0, 0 },
    { 0xb4f15c3a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f15c3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8815c42, 23 },
    { 0x42115c43, 76 },
    { 0, 0 },
    { 0xc2815c45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8115c49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b415c4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4015c52, 34 },
    { 0, 0 },
    { 0x11815c54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f15c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f15c61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a15c66, 88 },
    { 0x79a15c66, 33 },
    { 0x85c15c68, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63615c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f715c72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e315c75, 95 },
    { 0, 0 },
    { 0x5c415c77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715c7b, 98 },
    { 0xcef15c7b, 67 },
    { 0xe215c7d, 99 },
    { 0x53015c7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b115c82, 46 },
    { 0, 0 },
    { 0x71115c84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c215c87, 66 },
    { 0xfbf15c87, 76 },
    { 0xbf15c89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb15c8c, 61 },
    { 0, 0 },
    { 0xbbf15c8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a015c92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63915c96, 83 },
    { 0x20f15c97, 61 },
    { 0x40415c97, 91 },
    { 0, 0 },
    { 0xfef15c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9515c9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3415ca1, 33 },
    { 0, 0 },
    { 0x4db15ca3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6515ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1115cac, 66 },
    { 0xd0e15cad, 90 },
    { 0xe1c15cae, 52 },
    { 0, 0 },
    { 0x29f15cb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc15cb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d15cba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e015cbf, 85 },
    { 0, 0 },
    { 0x9fc15cc1, 61 },
    { 0x85615cc2, 52 },
    { 0, 0 },
    { 0xa0c15cc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e15cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd15cca, 55 },
    { 0x7e915ccb, 87 },
    { 0x7e815ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64915ccf, 64 },
    { 0, 0 },
    { 0x63b15cd1, 90 },
    { 0x8df15cd2, 76 },
    { 0x99a15cd3, 95 },
    { 0x91315cd4, 98 },
    { 0xa9115cd4, 100 },
    { 0xbc15cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b15cde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e15ce4, 96 },
    { 0x4c115ce4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34215ce8, 66 },
    { 0xd7215ce8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x51315cec, 78 },
    { 0x13615cec, 90 },
    { 0x1d515cee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2715cf3, 93 },
    { 0x6ed15cf3, 33 },
    { 0x2f015cf5, 46 },
    { 0, 0 },
    { 0xd4915cf7, 33 },
    { 0x40a15cf8, 33 },
    { 0x66e15cf8, 52 },
    { 0, 0 },
    { 0xd1315cfb, 23 },
    { 0x1f015cfc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9015cff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3015d03, 95 },
    { 0x82415d03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c715d07, 95 },
    { 0x53715d08, 33 },
    { 0xfa415d09, 55 },
    { 0x1c015d0a, 24 },
    { 0x38215d0b, 90 },
    { 0, 0 },
    { 0x10515d0d, 56 },
    { 0, 0 },
    { 0xfa15d0f, 33 },
    { 0xcb315d10, 90 },
    { 0, 0 },
    { 0xa2315d12, 65 },
    { 0xa7e15d13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef15d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7515d1a, 95 },
    { 0, 0 },
    { 0xe415d1c, 17 },
    { 0, 0 },
    { 0xe0b15d1e, 33 },
    { 0xf7d15d1f, 95 },
    { 0x41d15d1f, 33 },
    { 0xd9d15d1e, 98 },
    { 0, 0 },
    { 0x17315d23, 23 },
    { 0x13515d24, 61 },
    { 0x7a515d25, 90 },
    { 0, 0 },
    { 0x60915d27, 55 },
    { 0x8215d27, 61 },
    { 0x5ce15d27, 93 },
    { 0xe9b15d2a, 95 },
    { 0xd1815d29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de15d33, 23 },
    { 0x5d715d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad015d3b, 90 },
    { 0x2e315d3c, 55 },
    { 0xc9c15d3d, 95 },
    { 0xdba15d3e, 91 },
    { 0xc0815d3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8915d45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d15d4d, 46 },
    { 0x11915d4e, 61 },
    { 0xc5915d4d, 76 },
    { 0, 0 },
    { 0xfa315d51, 23 },
    { 0, 0 },
    { 0x29e15d53, 60 },
    { 0x7ee15d54, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96515d57, 99 },
    { 0, 0 },
    { 0x49415d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0115d5c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc815d5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f15d63, 33 },
    { 0x3ec15d64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6715d67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35815d6a, 85 },
    { 0, 0 },
    { 0xc0115d6c, 90 },
    { 0xf1215d6c, 61 },
    { 0x8515d6e, 43 },
    { 0x39115d6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c15d73, 55 },
    { 0x44d15d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc15d7b, 93 },
    { 0x36d15d7c, 90 },
    { 0x18915d7d, 91 },
    { 0x87215d7e, 40 },
    { 0xb015d7f, 87 },
    { 0xec515d80, 46 },
    { 0xf7415d7c, 61 },
    { 0xcc115d7c, 100 },
    { 0x41a15d82, 46 },
    { 0x5e015d81, 55 },
    { 0x84015d85, 94 },
    { 0xf1015d85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e15d89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18215d8d, 81 },
    { 0xd3815d8e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b15d95, 96 },
    { 0, 0 },
    { 0x24f15d97, 66 },
    { 0xd5215d98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f15d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d15da4, 33 },
    { 0, 0 },
    { 0x12915da6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd15daa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc715db3, 95 },
    { 0, 0 },
    { 0x11b15db5, 23 },
    { 0xb2315db6, 61 },
    { 0, 0 },
    { 0x90815db8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef15dbf, 98 },
    { 0x5a615dc0, 98 },
    { 0, 0 },
    { 0xc4315dc2, 55 },
    { 0x48a15dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc15dc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a615dcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e15dcf, 23 },
    { 0xa2f15dd0, 90 },
    { 0xcdd15dcf, 95 },
    { 0, 0 },
    { 0x8c015dd3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31015dd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5215de0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d15de6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b015dee, 33 },
    { 0x3fd15def, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1515df7, 23 },
    { 0xcdb15df7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67615dfb, 46 },
    { 0, 0 },
    { 0x37b15dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e15e03, 17 },
    { 0xb0515e04, 67 },
    { 0x77015e04, 88 },
    { 0xb4515e03, 23 },
    { 0x14715e07, 87 },
    { 0xbf15e03, 90 },
    { 0xad615e05, 78 },
    { 0x67d15e0a, 67 },
    { 0, 0 },
    { 0x31415e0c, 77 },
    { 0, 0 },
    { 0xa8915e0e, 23 },
    { 0, 0 },
    { 0x97515e10, 88 },
    { 0xa1215e10, 93 },
    { 0, 0 },
    { 0xcfc15e13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a15e17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60615e1a, 34 },
    { 0xcfd15e1b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a015e1e, 53 },
    { 0x9d515e1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d15e22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41115e29, 98 },
    { 0x47c15e2a, 23 },
    { 0x3ae15e2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c515e31, 84 },
    { 0, 0 },
    { 0x86015e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5615e36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7215e39, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6415e3d, 33 },
    { 0x9515e3e, 61 },
    { 0, 0 },
    { 0x14015e40, 33 },
    { 0x31515e41, 76 },
    { 0x2b915e40, 55 },
    { 0, 0 },
    { 0xbf115e44, 33 },
    { 0x34815e44, 56 },
    { 0x9b815e46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa915e4d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c15e51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f15e55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84215e58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb15e5e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x96115e61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2215e67, 55 },
    { 0xe015e68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b15e71, 99 },
    { 0xe6315e72, 98 },
    { 0x15215e73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee15e77, 76 },
    { 0, 0 },
    { 0x29b15e79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x83415e7c, 43 },
    { 0, 0 },
    { 0x1f315e7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82415e84, 45 },
    { 0, 0 },
    { 0xaed15e86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82715e8a, 61 },
    { 0x7eb15e8b, 76 },
    { 0x1c815e8c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b215e91, 90 },
    { 0x18115e92, 90 },
    { 0x58015e93, 76 },
    { 0xc8f15e94, 90 },
    { 0, 0 },
    { 0x4aa15e96, 85 },
    { 0x30b15e96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10615e9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac115ea2, 75 },
    { 0xb015ea3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd15ea7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af15eac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b15eb1, 35 },
    { 0xc8515eb2, 76 },
    { 0, 0 },
    { 0xbd115eb4, 33 },
    { 0x7f215eb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7615eb8, 76 },
    { 0x5215eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6915ebf, 34 },
    { 0, 0 },
    { 0x63115ec1, 98 },
    { 0x23115ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea15ec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88015ed1, 98 },
    { 0, 0 },
    { 0x8f915ed3, 93 },
    { 0x75215ed4, 93 },
    { 0x74615ed4, 33 },
    { 0xfa715ed3, 90 },
    { 0x73715ed7, 33 },
    { 0xb8915ed8, 23 },
    { 0, 0 },
    { 0xc1115eda, 43 },
    { 0, 0 },
    { 0xe5715edc, 14 },
    { 0x7c615edd, 55 },
    { 0, 0 },
    { 0x7e615edf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84915ee2, 55 },
    { 0x77915ee3, 95 },
    { 0x6cc15ee2, 61 },
    { 0xb6115ee5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb515eea, 42 },
    { 0x8aa15eeb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15eee, 90 },
    { 0x4ce15eef, 33 },
    { 0xda615ef0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8715ef6, 61 },
    { 0xcb215ef7, 61 },
    { 0x1b15ef7, 23 },
    { 0x16e15ef8, 95 },
    { 0x74615ef8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e15efe, 88 },
    { 0xbad15efe, 98 },
    { 0, 0 },
    { 0x48215f01, 33 },
    { 0x81a15f02, 34 },
    { 0xb4e15f02, 33 },
    { 0x5ba15f04, 33 },
    { 0xc6b15f05, 55 },
    { 0x42715f05, 85 },
    { 0xc2815f03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66915f0e, 90 },
    { 0, 0 },
    { 0xaf415f10, 91 },
    { 0x4c115f10, 90 },
    { 0, 0 },
    { 0xe4215f13, 61 },
    { 0xaec15f13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3315f18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc115f20, 77 },
    { 0x9815f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60415f25, 91 },
    { 0xecd15f25, 80 },
    { 0xb4d15f27, 61 },
    { 0xd4215f28, 90 },
    { 0, 0 },
    { 0x89f15f2a, 90 },
    { 0, 0 },
    { 0x85015f2c, 61 },
    { 0xfed15f2d, 66 },
    { 0, 0 },
    { 0x6c815f2f, 95 },
    { 0x78915f30, 23 },
    { 0xe5d15f31, 61 },
    { 0, 0 },
    { 0x97d15f33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c415f39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c15f3f, 67 },
    { 0, 0 },
    { 0x94615f41, 90 },
    { 0x38115f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e615f46, 61 },
    { 0x34715f47, 90 },
    { 0x93715f46, 67 },
    { 0xb7015f49, 98 },
    { 0, 0 },
    { 0x1d115f4b, 76 },
    { 0xb615f4c, 23 },
    { 0xdf315f4b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xff15f50, 67 },
    { 0, 0 },
    { 0x18e15f52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb15f58, 61 },
    { 0xe9715f58, 61 },
    { 0, 0 },
    { 0xdb815f5b, 91 },
    { 0x23815f5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015f61, 81 },
    { 0x20415f61, 33 },
    { 0xd9815f63, 55 },
    { 0xc6415f64, 36 },
    { 0x18915f65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec815f68, 33 },
    { 0x1b115f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb615f6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86015f77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c315f7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca15f7f, 61 },
    { 0, 0 },
    { 0x20715f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12415f89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad515f8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc15f91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7715f94, 77 },
    { 0, 0 },
    { 0x69515f96, 67 },
    { 0xaa315f96, 33 },
    { 0xf7715f98, 95 },
    { 0xc1a15f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf515fa0, 76 },
    { 0x66515fa1, 64 },
    { 0x2fc15fa1, 90 },
    { 0x1d815fa2, 65 },
    { 0, 0 },
    { 0xe8215fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4615fa8, 93 },
    { 0, 0 },
    { 0xceb15faa, 81 },
    { 0x50515faa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13215fae, 98 },
    { 0x41615faf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8015fb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a15fb5, 100 },
    { 0xcdb15fb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b15fba, 43 },
    { 0xbb515fbb, 99 },
    { 0xcad15fbb, 61 },
    { 0xe0415fbc, 84 },
    { 0x7e115fbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb515fc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5315fc7, 23 },
    { 0x80915fc7, 33 },
    { 0xb1115fc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa15fcf, 33 },
    { 0x59815fcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf15fdb, 55 },
    { 0xed215fdc, 46 },
    { 0x35315fdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec15fe2, 61 },
    { 0x10815fe3, 23 },
    { 0x81c15fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb15fe8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c15fec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b15ff3, 99 },
    { 0xec415ff4, 23 },
    { 0x1c815ff4, 61 },
    { 0x47315ff6, 84 },
    { 0x47e15ff5, 91 },
    { 0, 0 },
    { 0x3a15ff9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a015ffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27016003, 90 },
    { 0x9616004, 99 },
    { 0, 0 },
    { 0x78816006, 34 },
    { 0, 0 },
    { 0xb8916008, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x3931600b, 78 },
    { 0, 0 },
    { 0x61a1600d, 84 },
    { 0, 0 },
    { 0xed71600f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e516016, 98 },
    { 0x12d16017, 91 },
    { 0xd1816017, 61 },
    { 0xee816017, 43 },
    { 0x90316017, 34 },
    { 0x22616017, 90 },
    { 0xbde1601c, 23 },
    { 0, 0 },
    { 0x7151601e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30516022, 76 },
    { 0x33016022, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab16027, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf216033, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18516038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba1603b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1603e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c1604a, 90 },
    { 0x54e1604b, 85 },
    { 0xc491604c, 66 },
    { 0xa5b1604d, 33 },
    { 0x6551604d, 38 },
    { 0x25f1604f, 23 },
    { 0xc8c16050, 33 },
    { 0x6631604d, 93 },
    { 0x3db1604e, 33 },
    { 0xd9d16053, 68 },
    { 0x22416053, 75 },
    { 0xc0216055, 90 },
    { 0xab716056, 23 },
    { 0xa671604a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e1605c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d016064, 66 },
    { 0x62216065, 80 },
    { 0xa8816066, 95 },
    { 0x2db16064, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b1606a, 98 },
    { 0xdb51606a, 61 },
    { 0, 0 },
    { 0x171606d, 40 },
    { 0xfb41606e, 67 },
    { 0x2431606f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43716078, 90 },
    { 0xe5116079, 61 },
    { 0x25916079, 95 },
    { 0, 0 },
    { 0xaca1607c, 65 },
    { 0xecd1607d, 90 },
    { 0xcca1607e, 55 },
    { 0xac91607e, 61 },
    { 0x701607e, 34 },
    { 0, 0 },
    { 0xdb616082, 61 },
    { 0xf16083, 52 },
    { 0x46916084, 81 },
    { 0xe8d16084, 93 },
    { 0x4e916084, 36 },
    { 0xad916087, 88 },
    { 0xe9f16088, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf851608c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d16092, 66 },
    { 0, 0 },
    { 0x6d116094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b1609d, 61 },
    { 0x9ca1609e, 98 },
    { 0, 0 },
    { 0x93d160a0, 33 },
    { 0x73160a0, 43 },
    { 0xd8e160a2, 61 },
    { 0x2e4160a3, 55 },
    { 0xb2c160a4, 52 },
    { 0xfe2160a5, 46 },
    { 0xbc1160a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d160aa, 98 },
    { 0, 0 },
    { 0xdce160ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e160b3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31160b7, 33 },
    { 0x14e160b8, 15 },
    { 0x7b8160b8, 85 },
    { 0xac9160ba, 95 },
    { 0x441160ba, 90 },
    { 0xaf4160b8, 98 },
    { 0x4d1160bd, 36 },
    { 0x462160bd, 76 },
    { 0, 0 },
    { 0x225160c0, 76 },
    { 0x8c3160c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af160c4, 77 },
    { 0, 0 },
    { 0x481160c6, 43 },
    { 0xc30160c7, 98 },
    { 0x9d3160c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914160cd, 78 },
    { 0xbf0160cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0160d2, 98 },
    { 0xb39160d3, 23 },
    { 0xe5c160d3, 95 },
    { 0xdca160d3, 95 },
    { 0, 0 },
    { 0x16f160d7, 93 },
    { 0x4a7160d8, 95 },
    { 0, 0 },
    { 0xa09160da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1160de, 61 },
    { 0xfdc160df, 95 },
    { 0x94b160df, 76 },
    { 0xf65160e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x999160e4, 43 },
    { 0xdb0160e5, 77 },
    { 0xb9c160e5, 98 },
    { 0x422160e7, 23 },
    { 0x4f8160e5, 77 },
    { 0x35160e9, 90 },
    { 0xeb4160ea, 36 },
    { 0, 0 },
    { 0x8b8160ec, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874160f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5160f9, 61 },
    { 0xef7160fa, 81 },
    { 0xcc160fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f16100, 98 },
    { 0, 0 },
    { 0xeaf16102, 17 },
    { 0xdfe16103, 33 },
    { 0xbc816103, 87 },
    { 0x1ac16104, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc851610b, 76 },
    { 0x9a01610c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xded1610f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c16114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6616118, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9561611e, 46 },
    { 0, 0 },
    { 0x49b16120, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40316125, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4411612a, 55 },
    { 0, 0 },
    { 0x6bb1612c, 91 },
    { 0x8681612d, 98 },
    { 0, 0 },
    { 0x5c91612f, 90 },
    { 0xc2d16130, 43 },
    { 0, 0 },
    { 0x96316132, 23 },
    { 0x51816132, 98 },
    { 0x60816134, 33 },
    { 0x2e916134, 91 },
    { 0, 0 },
    { 0x9a416137, 43 },
    { 0xf4f16138, 76 },
    { 0x72216139, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf931613c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62216142, 91 },
    { 0, 0 },
    { 0x5df16144, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c31614b, 95 },
    { 0xa441614c, 34 },
    { 0, 0 },
    { 0xe71614e, 75 },
    { 0x46f1614f, 90 },
    { 0x6931614f, 33 },
    { 0x5091614e, 90 },
    { 0x52716151, 55 },
    { 0x68b1614e, 33 },
    { 0xc2a16153, 75 },
    { 0xad716155, 95 },
    { 0x2b16155, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd116159, 78 },
    { 0xde016159, 23 },
    { 0x5921615b, 99 },
    { 0x2fe1615b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7616160, 34 },
    { 0, 0 },
    { 0xed816162, 90 },
    { 0, 0 },
    { 0x98716164, 88 },
    { 0x73d16165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1481616d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea516175, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d01617b, 43 },
    { 0xb871617c, 76 },
    { 0x4881617d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc16182, 90 },
    { 0x77e16183, 100 },
    { 0x5616182, 34 },
    { 0x4f316185, 46 },
    { 0x86716186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f1618a, 61 },
    { 0x2501618a, 98 },
    { 0xd6f1618b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd16191, 33 },
    { 0x9c316192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d16198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd1619b, 99 },
    { 0xd761619c, 84 },
    { 0x6f61619d, 68 },
    { 0x5981619e, 98 },
    { 0xc0b1619f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7161a3, 46 },
    { 0x479161a3, 87 },
    { 0xb58161a3, 43 },
    { 0, 0 },
    { 0x668161a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5161ad, 95 },
    { 0x1cb161ae, 61 },
    { 0, 0 },
    { 0xb1e161b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1161b4, 84 },
    { 0xc13161b5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1161bb, 61 },
    { 0xf5161bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83161bf, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1161c2, 67 },
    { 0x765161c3, 61 },
    { 0xf13161c4, 66 },
    { 0xd65161c3, 95 },
    { 0, 0 },
    { 0x397161c7, 38 },
    { 0, 0 },
    { 0x5f6161c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558161cd, 76 },
    { 0, 0 },
    { 0x4b9161cf, 33 },
    { 0, 0 },
    { 0x3aa161d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d161db, 61 },
    { 0, 0 },
    { 0x310161dd, 52 },
    { 0, 0 },
    { 0x8b0161df, 52 },
    { 0, 0 },
    { 0x1ee161e1, 90 },
    { 0x3ec161e1, 55 },
    { 0, 0 },
    { 0x837161e4, 23 },
    { 0xfd5161e5, 95 },
    { 0x987161e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e161e9, 88 },
    { 0x37b161e9, 68 },
    { 0x541161eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161ef, 98 },
    { 0x18c161f0, 55 },
    { 0x514161f0, 95 },
    { 0x9a7161f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6161f6, 43 },
    { 0xdc6161f6, 90 },
    { 0, 0 },
    { 0xe02161f9, 17 },
    { 0xd83161f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x164161fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b016201, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9916204, 34 },
    { 0xef216205, 43 },
    { 0x9d916206, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda216209, 95 },
    { 0, 0 },
    { 0x3ca1620b, 76 },
    { 0, 0 },
    { 0xe5f1620d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e16211, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54416215, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d916218, 95 },
    { 0x61416219, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd31621c, 95 },
    { 0x4901621d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb716225, 76 },
    { 0, 0 },
    { 0xbe516227, 48 },
    { 0x9e016227, 43 },
    { 0x6916229, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3331622d, 65 },
    { 0, 0 },
    { 0x2381622f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c016234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5616237, 85 },
    { 0, 0 },
    { 0xc1216239, 99 },
    { 0x6dc1623a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb381623f, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c16242, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9316245, 90 },
    { 0x25316245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33016249, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f1624c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84416250, 40 },
    { 0x65616250, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a016258, 17 },
    { 0x44a16258, 76 },
    { 0xdf91625a, 33 },
    { 0, 0 },
    { 0xba11625c, 77 },
    { 0x9f81625d, 23 },
    { 0, 0 },
    { 0xb101625f, 66 },
    { 0x81625f, 78 },
    { 0x53d16261, 78 },
    { 0, 0 },
    { 0x10016263, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c16267, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc016270, 90 },
    { 0xd2216271, 98 },
    { 0x72816271, 61 },
    { 0xb3116271, 90 },
    { 0x16716274, 90 },
    { 0xbc516273, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b1627a, 95 },
    { 0xd0f1627b, 95 },
    { 0xa0e1627c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0016280, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c16286, 85 },
    { 0, 0 },
    { 0xb7316288, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b1628e, 98 },
    { 0x18c1628e, 95 },
    { 0x7b216290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec16294, 98 },
    { 0, 0 },
    { 0x4aa16296, 65 },
    { 0x3a716297, 23 },
    { 0x7c516296, 33 },
    { 0xcc416299, 33 },
    { 0x25916299, 61 },
    { 0x50d16299, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3162a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28162a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2162a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x194162aa, 34 },
    { 0xc20162ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a162af, 43 },
    { 0x2f9162b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5162b3, 90 },
    { 0xdca162b4, 23 },
    { 0x699162b3, 17 },
    { 0xa8162b4, 90 },
    { 0xa3d162b4, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb162bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4162be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391162c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3162c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134162ca, 98 },
    { 0, 0 },
    { 0x22a162cc, 98 },
    { 0x314162cd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a162d0, 33 },
    { 0xfef162d1, 88 },
    { 0x618162d2, 98 },
    { 0xd22162d0, 33 },
    { 0xc30162d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17162dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f162df, 91 },
    { 0x649162e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7162e4, 33 },
    { 0x6f9162e5, 90 },
    { 0x216162e5, 91 },
    { 0xff8162e7, 23 },
    { 0xdf5162e8, 90 },
    { 0x9d5162e5, 76 },
    { 0x303162e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x509162ed, 98 },
    { 0x9f2162ed, 95 },
    { 0, 0 },
    { 0xaed162f0, 76 },
    { 0x924162f1, 34 },
    { 0x515162f2, 76 },
    { 0xb5e162f2, 95 },
    { 0xbed162f4, 76 },
    { 0xcef162f5, 33 },
    { 0x9d4162f6, 101 },
    { 0x5c9162f1, 98 },
    { 0, 0 },
    { 0x2b9162f9, 91 },
    { 0, 0 },
    { 0x368162fb, 98 },
    { 0x8a162fb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0162ff, 23 },
    { 0xd88162ff, 55 },
    { 0, 0 },
    { 0xedc16302, 43 },
    { 0xeda16303, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e616314, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc816317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61216324, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd16328, 55 },
    { 0xa2316329, 92 },
    { 0xc0516329, 46 },
    { 0x4251632b, 33 },
    { 0xe1b1632b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13316332, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f416337, 99 },
    { 0xb9516337, 84 },
    { 0xcd116337, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe1633c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee16344, 98 },
    { 0xc5216345, 23 },
    { 0, 0 },
    { 0x5ce16347, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1941634d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb16352, 81 },
    { 0xc9516353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d16358, 100 },
    { 0, 0 },
    { 0xc2d1635a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb871635e, 23 },
    { 0, 0 },
    { 0x3f916360, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e216363, 93 },
    { 0x9a816364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd16369, 55 },
    { 0xaac1636a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d1636f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df1637a, 33 },
    { 0xa5e1637b, 93 },
    { 0x6201637c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62616380, 67 },
    { 0, 0 },
    { 0x50716382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57416385, 33 },
    { 0, 0 },
    { 0x3f316387, 80 },
    { 0xe7816388, 33 },
    { 0x52216388, 67 },
    { 0xe061638a, 98 },
    { 0, 0 },
    { 0x82c1638c, 98 },
    { 0x62f1638d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a16391, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6016397, 98 },
    { 0, 0 },
    { 0x65e16399, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd51639d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x911163a0, 65 },
    { 0x57e163a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3163a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e163a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727163ad, 84 },
    { 0xf79163ad, 95 },
    { 0, 0 },
    { 0x6e163b0, 58 },
    { 0xc85163b1, 90 },
    { 0x62d163b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48163b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55163c2, 95 },
    { 0, 0 },
    { 0x845163c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e163ca, 17 },
    { 0, 0 },
    { 0x4f0163cc, 76 },
    { 0x36b163cc, 43 },
    { 0x5163ce, 34 },
    { 0x4bf163cf, 61 },
    { 0x2fe163ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61163d3, 46 },
    { 0, 0 },
    { 0x673163d5, 98 },
    { 0xda2163d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc163de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171163e3, 61 },
    { 0xcff163e3, 34 },
    { 0x26163e3, 78 },
    { 0, 0 },
    { 0xdb5163e7, 78 },
    { 0xb76163e8, 34 },
    { 0x2ab163e9, 90 },
    { 0x19d163e9, 52 },
    { 0xd28163ea, 90 },
    { 0, 0 },
    { 0xa73163ed, 61 },
    { 0, 0 },
    { 0xccc163ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92163f2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469163fc, 95 },
    { 0x342163fd, 76 },
    { 0x83a163fe, 97 },
    { 0x72d163fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64116404, 55 },
    { 0x79016405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f1640a, 61 },
    { 0x8dc1640b, 93 },
    { 0x9d31640c, 55 },
    { 0x38a1640d, 55 },
    { 0x1951640d, 23 },
    { 0xddf1640c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79216414, 46 },
    { 0x1fd16415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c16422, 52 },
    { 0xfdf16422, 23 },
    { 0x49516423, 98 },
    { 0x58416423, 46 },
    { 0xb6916426, 33 },
    { 0xc8e16426, 61 },
    { 0x2816425, 95 },
    { 0xf5516428, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a1642c, 23 },
    { 0x4fd1642d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25816434, 23 },
    { 0xe3516434, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2016438, 100 },
    { 0x8c816439, 61 },
    { 0x3e116439, 85 },
    { 0x3ef16439, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7431643f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9416443, 90 },
    { 0x4da16444, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a1644b, 90 },
    { 0, 0 },
    { 0x1b11644d, 90 },
    { 0x2cf1644d, 98 },
    { 0xc8b1644f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6316452, 33 },
    { 0xed16453, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x12616456, 76 },
    { 0, 0 },
    { 0x75516458, 34 },
    { 0xc8916459, 75 },
    { 0xadc1645a, 76 },
    { 0xdff1645b, 33 },
    { 0x82e1645c, 33 },
    { 0x10a1645d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23616463, 88 },
    { 0x4a416464, 23 },
    { 0, 0 },
    { 0x6ec16466, 95 },
    { 0x89216467, 43 },
    { 0, 0 },
    { 0x78016469, 95 },
    { 0x8041646a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83916470, 95 },
    { 0xc6e16471, 90 },
    { 0x7e116472, 33 },
    { 0xf2316470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4616478, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591647b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c61647e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x58216481, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7616484, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc981648a, 90 },
    { 0x6ff1648a, 81 },
    { 0xe991648c, 90 },
    { 0x39b1648c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b16494, 8 },
    { 0, 0 },
    { 0x58f16496, 61 },
    { 0xf2216497, 33 },
    { 0x3bd16496, 61 },
    { 0xd5916499, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd21649d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b164a2, 95 },
    { 0, 0 },
    { 0xb9d164a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb164a9, 76 },
    { 0xabe164aa, 90 },
    { 0, 0 },
    { 0x54d164ac, 81 },
    { 0x406164ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd164b4, 17 },
    { 0x7af164b4, 100 },
    { 0x1d0164b4, 34 },
    { 0x63b164b7, 55 },
    { 0, 0 },
    { 0xa33164b9, 23 },
    { 0, 0 },
    { 0x8d6164bb, 88 },
    { 0xe27164bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2164bf, 78 },
    { 0, 0 },
    { 0x427164c1, 90 },
    { 0xee3164c1, 91 },
    { 0xc19164c1, 100 },
    { 0xb02164c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d164c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad164cc, 91 },
    { 0, 0 },
    { 0xaa4164ce, 19 },
    { 0, 0 },
    { 0xe9b164d0, 69 },
    { 0xaf3164d0, 91 },
    { 0, 0 },
    { 0x5bd164d3, 83 },
    { 0xa68164d4, 87 },
    { 0x254164d5, 91 },
    { 0x60b164d5, 95 },
    { 0x764164d5, 23 },
    { 0, 0 },
    { 0xf1b164d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a164dc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850164e0, 43 },
    { 0x759164e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b164e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d164e9, 46 },
    { 0, 0 },
    { 0xa9b164eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf164f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b16501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb416508, 61 },
    { 0, 0 },
    { 0x12e1650a, 55 },
    { 0, 0 },
    { 0xaa41650c, 85 },
    { 0x73c1650c, 23 },
    { 0xa921650c, 78 },
    { 0xac01650f, 52 },
    { 0xa4716510, 61 },
    { 0xf5716511, 91 },
    { 0xcd816512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b816516, 33 },
    { 0, 0 },
    { 0x9a816518, 33 },
    { 0, 0 },
    { 0x2c41651a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1651d, 46 },
    { 0x80e1651e, 76 },
    { 0x3fe1651f, 90 },
    { 0x50016520, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33716524, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x88116527, 93 },
    { 0, 0 },
    { 0xca416529, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef1652c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c21652f, 87 },
    { 0x43a16530, 55 },
    { 0xdd516531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e16534, 98 },
    { 0xb9e16534, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5431653a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3571653e, 23 },
    { 0x52f1653e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d16546, 61 },
    { 0x3b16547, 75 },
    { 0xa2416548, 95 },
    { 0, 0 },
    { 0x7d11654a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94016553, 93 },
    { 0x68816554, 60 },
    { 0x8d416554, 61 },
    { 0, 0 },
    { 0xfe716557, 61 },
    { 0, 0 },
    { 0xce816559, 52 },
    { 0, 0 },
    { 0xb551655b, 61 },
    { 0x7991655c, 90 },
    { 0xe9b1655b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7716564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a716567, 90 },
    { 0, 0 },
    { 0x3ca16569, 90 },
    { 0xee316569, 78 },
    { 0, 0 },
    { 0x1e31656c, 88 },
    { 0, 0 },
    { 0x27a1656e, 95 },
    { 0x4821656e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3516572, 78 },
    { 0x88a16572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e16576, 33 },
    { 0, 0 },
    { 0xd0b16578, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27116582, 95 },
    { 0, 0 },
    { 0xa716584, 67 },
    { 0xcc616585, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec1658e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af16594, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa516599, 33 },
    { 0, 0 },
    { 0x4791659b, 84 },
    { 0x7ab1659c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d1659f, 95 },
    { 0, 0 },
    { 0xfbd165a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373165a6, 90 },
    { 0x2fe165a7, 87 },
    { 0x5e9165a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69165af, 98 },
    { 0x54165b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667165b7, 61 },
    { 0x3fa165b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c165bb, 95 },
    { 0xf4f165bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a165c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc165c6, 76 },
    { 0xe59165c7, 33 },
    { 0, 0 },
    { 0xb9e165c9, 81 },
    { 0xc83165ca, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c165ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18165d2, 67 },
    { 0x2ca165d3, 61 },
    { 0xd4c165d4, 36 },
    { 0, 0 },
    { 0xfa1165d6, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c165dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a165e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e165e3, 84 },
    { 0, 0 },
    { 0xca9165e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x760165e8, 23 },
    { 0xeb2165e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3165f3, 33 },
    { 0xe16165f4, 95 },
    { 0x85165f5, 55 },
    { 0xf87165f5, 61 },
    { 0xc73165f4, 84 },
    { 0xca3165f8, 90 },
    { 0, 0 },
    { 0x99d165fa, 90 },
    { 0xb85165fb, 61 },
    { 0, 0 },
    { 0x2e0165fd, 95 },
    { 0xa6d165fd, 36 },
    { 0x4ce165ff, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b416604, 98 },
    { 0x94516604, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9516608, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91660e, 76 },
    { 0xb691660f, 90 },
    { 0x5eb16610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d516616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a61661b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad16620, 43 },
    { 0, 0 },
    { 0x6ce16622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d16626, 88 },
    { 0x3c316627, 34 },
    { 0xdf616628, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb1662b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a71662f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea16632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2816635, 52 },
    { 0x5ce16636, 98 },
    { 0x63816635, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3251663c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46316640, 84 },
    { 0xedf16641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32716644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9316648, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa601664c, 98 },
    { 0, 0 },
    { 0xe321664e, 95 },
    { 0, 0 },
    { 0xf5316650, 23 },
    { 0x8a516651, 33 },
    { 0xae916652, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e16655, 101 },
    { 0, 0 },
    { 0x8c16657, 61 },
    { 0xb0016658, 98 },
    { 0x84b16659, 23 },
    { 0xf0d16659, 23 },
    { 0x87d16659, 34 },
    { 0x33916658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f1665f, 95 },
    { 0, 0 },
    { 0x52e16661, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a16664, 65 },
    { 0xe6016664, 90 },
    { 0x5cc16665, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5816669, 90 },
    { 0x3fa1666a, 46 },
    { 0x2b61666b, 90 },
    { 0x8a81666b, 90 },
    { 0xe501666d, 46 },
    { 0xc551666e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d16671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5616674, 61 },
    { 0, 0 },
    { 0x56716676, 84 },
    { 0x9216677, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b1667b, 90 },
    { 0x53a1667c, 85 },
    { 0, 0 },
    { 0x2fc1667e, 61 },
    { 0xe4d1667f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf116686, 76 },
    { 0x92116686, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e61668d, 52 },
    { 0x63e1668e, 23 },
    { 0, 0 },
    { 0xb4b16690, 33 },
    { 0, 0 },
    { 0x40e16692, 23 },
    { 0x93916693, 43 },
    { 0x36916694, 33 },
    { 0x4e416694, 90 },
    { 0x6a16696, 23 },
    { 0, 0 },
    { 0x2f616698, 33 },
    { 0xf9616699, 76 },
    { 0, 0 },
    { 0x6421669b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4166a3, 98 },
    { 0x836166a3, 90 },
    { 0x20b166a5, 66 },
    { 0xcb8166a6, 93 },
    { 0x9e6166a5, 43 },
    { 0xd3166a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x239166ab, 98 },
    { 0x418166ac, 17 },
    { 0x4c4166ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679166b5, 95 },
    { 0x21a166b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc166c1, 90 },
    { 0x5fc166c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1166c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f166cb, 33 },
    { 0x2d5166cc, 61 },
    { 0, 0 },
    { 0xb42166ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1166d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991166d6, 61 },
    { 0xcb5166d6, 98 },
    { 0xf4f166d8, 81 },
    { 0x542166d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519166e0, 91 },
    { 0, 0 },
    { 0x26d166e2, 90 },
    { 0x2b0166e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993166ec, 61 },
    { 0xdab166ec, 87 },
    { 0x7d2166ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x646166f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526166fa, 34 },
    { 0x5ec166fb, 90 },
    { 0, 0 },
    { 0x57f166fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1616702, 96 },
    { 0x67516703, 66 },
    { 0x83f16704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd916709, 68 },
    { 0x16e1670a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd16711, 76 },
    { 0x82f16711, 65 },
    { 0xe0616713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c16718, 90 },
    { 0x4cd16719, 61 },
    { 0, 0 },
    { 0xf2d1671b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa471671e, 93 },
    { 0xaf1671f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae16724, 100 },
    { 0x89116725, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7751672a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac16730, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f216736, 55 },
    { 0xd5c16737, 95 },
    { 0xf3016737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e1673c, 61 },
    { 0x50f1673d, 98 },
    { 0xe1a1673e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d16751, 33 },
    { 0x65916752, 90 },
    { 0, 0 },
    { 0x50d16754, 46 },
    { 0x13e16755, 75 },
    { 0, 0 },
    { 0xa0916757, 55 },
    { 0xdaf16758, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c1675f, 98 },
    { 0, 0 },
    { 0x85a16761, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d816767, 90 },
    { 0, 0 },
    { 0x86216769, 52 },
    { 0x20316769, 56 },
    { 0x1261676b, 67 },
    { 0xf371676b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x491676f, 66 },
    { 0x47216770, 33 },
    { 0xc516771, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c016774, 23 },
    { 0x61f16775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c16779, 35 },
    { 0xd491677a, 99 },
    { 0x9f01677b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a11677f, 23 },
    { 0xa1816780, 90 },
    { 0xa3416780, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed16785, 61 },
    { 0x5e816785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e16789, 90 },
    { 0, 0 },
    { 0x3551678b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca1678e, 90 },
    { 0xf881678f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f16793, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d16797, 93 },
    { 0, 0 },
    { 0x49716799, 34 },
    { 0x64016799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f167a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a167a5, 90 },
    { 0xf2b167a5, 95 },
    { 0x973167a7, 82 },
    { 0x943167a7, 98 },
    { 0x91c167a8, 83 },
    { 0xa04167aa, 90 },
    { 0x1cc167ab, 77 },
    { 0x865167ab, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9167af, 38 },
    { 0x558167b0, 99 },
    { 0, 0 },
    { 0x289167b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9167b5, 33 },
    { 0, 0 },
    { 0x81f167b7, 90 },
    { 0xdac167b8, 51 },
    { 0xd11167b7, 33 },
    { 0xcd8167b7, 84 },
    { 0x602167b7, 90 },
    { 0, 0 },
    { 0x8d167bd, 76 },
    { 0x33f167be, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77167c2, 90 },
    { 0, 0 },
    { 0x573167c4, 95 },
    { 0xef0167c5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99167d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b167db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e167de, 41 },
    { 0, 0 },
    { 0x323167e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42167e4, 33 },
    { 0x47a167e5, 38 },
    { 0x317167e6, 95 },
    { 0x4ed167e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d167ea, 23 },
    { 0x11167eb, 33 },
    { 0x92e167ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1167ef, 95 },
    { 0xf8c167ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246167f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad167f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1167fc, 46 },
    { 0x8fc167fd, 95 },
    { 0, 0 },
    { 0x6c2167ff, 23 },
    { 0x8fd167ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa16804, 55 },
    { 0xd6416805, 33 },
    { 0x52016806, 95 },
    { 0x5e216804, 67 },
    { 0xf5f16808, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6971680b, 38 },
    { 0x5281680c, 23 },
    { 0x9b31680d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c516815, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b16818, 76 },
    { 0xcfe16819, 23 },
    { 0x1ad1681a, 43 },
    { 0x7011681a, 93 },
    { 0x20216819, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39016820, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f16823, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23816827, 33 },
    { 0x49a16828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe661682b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c51682f, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b16833, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e16838, 87 },
    { 0xc1616839, 23 },
    { 0xb6716839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8381683e, 90 },
    { 0x4701683f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c16843, 61 },
    { 0x7e316843, 33 },
    { 0xbda16843, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d11684b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa341684f, 98 },
    { 0x3ae16850, 95 },
    { 0, 0 },
    { 0x85a16852, 98 },
    { 0x63316853, 33 },
    { 0x51516853, 90 },
    { 0x30616855, 33 },
    { 0xdfe16856, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c61685f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e416862, 87 },
    { 0x62f16863, 34 },
    { 0xee216862, 93 },
    { 0x2d716865, 95 },
    { 0xa6316862, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x23916869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc1686c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3201686f, 95 },
    { 0, 0 },
    { 0xa6716871, 90 },
    { 0x97d16872, 23 },
    { 0xc1216873, 67 },
    { 0x6e616874, 57 },
    { 0x94816875, 85 },
    { 0, 0 },
    { 0x6bc16877, 87 },
    { 0, 0 },
    { 0x1c116879, 98 },
    { 0x6391687a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b116880, 61 },
    { 0, 0 },
    { 0xb6c16882, 33 },
    { 0, 0 },
    { 0x4b16884, 33 },
    { 0, 0 },
    { 0xe2a16886, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94616889, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3561688d, 43 },
    { 0xbff1688e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca616892, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf916897, 55 },
    { 0xcf716897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a1689b, 33 },
    { 0x6fc1689c, 61 },
    { 0x3921689c, 90 },
    { 0xf01689e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a168a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe168a5, 61 },
    { 0, 0 },
    { 0x31b168a7, 17 },
    { 0xae6168a8, 33 },
    { 0x887168a9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325168ae, 52 },
    { 0, 0 },
    { 0x6ba168b0, 87 },
    { 0x5aa168b0, 90 },
    { 0, 0 },
    { 0x92c168b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668168bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9168c0, 34 },
    { 0x429168c1, 43 },
    { 0, 0 },
    { 0xe9168c3, 43 },
    { 0xd98168c3, 68 },
    { 0, 0 },
    { 0xd48168c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x899168c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7168cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992168d1, 40 },
    { 0xfed168d1, 61 },
    { 0, 0 },
    { 0x536168d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f168d7, 69 },
    { 0xd3f168d7, 100 },
    { 0x2ec168d7, 95 },
    { 0xcc2168da, 61 },
    { 0x849168da, 43 },
    { 0xdf2168dc, 78 },
    { 0x9e3168dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x331168e0, 99 },
    { 0x83e168e1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6168e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e168ee, 76 },
    { 0xe70168ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33168f2, 98 },
    { 0xcf2168f3, 33 },
    { 0x774168f4, 98 },
    { 0xadc168f3, 61 },
    { 0x9c8168f6, 98 },
    { 0x354168f3, 90 },
    { 0xc3f168f8, 84 },
    { 0xa5c168f9, 34 },
    { 0x471168f5, 81 },
    { 0xe65168fb, 52 },
    { 0x660168fc, 84 },
    { 0, 0 },
    { 0xff6168fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0416901, 61 },
    { 0x1c416901, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24316906, 93 },
    { 0, 0 },
    { 0x5aa16908, 101 },
    { 0xc1d16909, 98 },
    { 0, 0 },
    { 0x60b1690b, 78 },
    { 0, 0 },
    { 0xa391690d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x81616910, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9516916, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4331691c, 68 },
    { 0, 0 },
    { 0x5591691e, 34 },
    { 0x7c61691e, 33 },
    { 0x88516920, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3516924, 90 },
    { 0, 0 },
    { 0x88916926, 76 },
    { 0x5ba16926, 90 },
    { 0x40516928, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd791692f, 23 },
    { 0x93f16930, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15916936, 23 },
    { 0, 0 },
    { 0xaaf16938, 95 },
    { 0x7c416939, 23 },
    { 0x2e61693a, 80 },
    { 0x4c316938, 52 },
    { 0, 0 },
    { 0x1871693d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee16943, 23 },
    { 0x94f16944, 98 },
    { 0xc4916944, 61 },
    { 0x1b516946, 78 },
    { 0, 0 },
    { 0x7f416948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4951694d, 76 },
    { 0xdc91694e, 76 },
    { 0, 0 },
    { 0xc9516950, 95 },
    { 0x53116950, 98 },
    { 0x8de16952, 100 },
    { 0x8e216952, 95 },
    { 0xf7316952, 46 },
    { 0xc1116951, 33 },
    { 0xb116954, 40 },
    { 0x1d016953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c1695a, 61 },
    { 0xf671695b, 85 },
    { 0x3921695b, 91 },
    { 0xc8f1695d, 55 },
    { 0xe961695b, 95 },
    { 0xd7b1695f, 33 },
    { 0xf516960, 98 },
    { 0xce11695d, 33 },
    { 0xb9416962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9416965, 34 },
    { 0xbdf16965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58816969, 23 },
    { 0x6cb16969, 66 },
    { 0xfa71696b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab21696f, 95 },
    { 0, 0 },
    { 0xec16971, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0516974, 8 },
    { 0x49416975, 43 },
    { 0x57b16976, 34 },
    { 0x3dd16977, 33 },
    { 0, 0 },
    { 0x50b16979, 95 },
    { 0, 0 },
    { 0xf291697b, 61 },
    { 0xa311697c, 90 },
    { 0, 0 },
    { 0xf851697e, 61 },
    { 0, 0 },
    { 0x38616980, 95 },
    { 0, 0 },
    { 0x4a16982, 23 },
    { 0x9316983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7916989, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe81698d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e16992, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83816997, 55 },
    { 0, 0 },
    { 0x1d916999, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa41699c, 85 },
    { 0, 0 },
    { 0x6351699e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b169a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d169a7, 52 },
    { 0, 0 },
    { 0xb07169a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4169b3, 90 },
    { 0x472169b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5169c0, 33 },
    { 0xd0a169c1, 42 },
    { 0x8cc169c2, 17 },
    { 0xd03169c3, 38 },
    { 0, 0 },
    { 0x1a4169c5, 99 },
    { 0x412169c6, 95 },
    { 0xcf169c7, 95 },
    { 0xc92169c6, 78 },
    { 0x418169c9, 78 },
    { 0x777169ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba169cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee169d1, 43 },
    { 0xab8169d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165169d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46169db, 55 },
    { 0x7a169db, 33 },
    { 0x14b169dd, 95 },
    { 0x232169dd, 23 },
    { 0x8b0169dd, 98 },
    { 0, 0 },
    { 0xfd5169e1, 43 },
    { 0xa67169e2, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab169ee, 25 },
    { 0, 0 },
    { 0xed0169f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c169f3, 98 },
    { 0x62f169f4, 23 },
    { 0xc08169f5, 33 },
    { 0x7a6169f3, 61 },
    { 0x409169f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4169fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea169fd, 61 },
    { 0x537169fe, 90 },
    { 0xc3e169ff, 33 },
    { 0x1a3169ff, 33 },
    { 0x6f0169fd, 52 },
    { 0, 0 },
    { 0x7e816a03, 83 },
    { 0xc1716a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e16a09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf916a0d, 81 },
    { 0x4aa16a0d, 93 },
    { 0x1fa16a0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3216a14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae616a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15116a20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0316a23, 90 },
    { 0x96316a23, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe016a27, 61 },
    { 0, 0 },
    { 0x26516a29, 98 },
    { 0x79616a2a, 33 },
    { 0x5e016a29, 95 },
    { 0xe7f16a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98716a2f, 34 },
    { 0x66116a30, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de16a33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16816a37, 23 },
    { 0xf5916a38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b116a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15616a41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e16a45, 98 },
    { 0xc5d16a45, 34 },
    { 0, 0 },
    { 0x82a16a48, 43 },
    { 0x13716a48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b16a4c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e916a4f, 33 },
    { 0, 0 },
    { 0x51016a51, 95 },
    { 0, 0 },
    { 0x21f16a53, 65 },
    { 0xf7716a54, 38 },
    { 0x64c16a53, 55 },
    { 0x9716a55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b216a59, 33 },
    { 0, 0 },
    { 0xe7916a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec016a60, 98 },
    { 0x28816a61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4016a65, 91 },
    { 0x5bd16a66, 55 },
    { 0, 0 },
    { 0x44c16a68, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb816a6d, 90 },
    { 0x36f16a6e, 23 },
    { 0x9a516a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d16a72, 84 },
    { 0x3c116a73, 90 },
    { 0xa1816a74, 33 },
    { 0x44f16a72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b16a79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63416a85, 46 },
    { 0xa5f16a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de16a8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf716a90, 76 },
    { 0, 0 },
    { 0xf2d16a92, 90 },
    { 0x6bc16a93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b16a96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d16a9a, 93 },
    { 0xbd616a9a, 46 },
    { 0xc9d16a9c, 90 },
    { 0, 0 },
    { 0xaf616a9e, 80 },
    { 0, 0 },
    { 0xf1a16aa0, 90 },
    { 0x52316aa0, 33 },
    { 0xee116aa0, 100 },
    { 0, 0 },
    { 0x51116aa4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec716aa9, 69 },
    { 0x4e416aaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0816ab8, 45 },
    { 0x50d16ab8, 23 },
    { 0x53416aba, 17 },
    { 0x56f16abb, 33 },
    { 0x66a16abb, 76 },
    { 0x74116abd, 65 },
    { 0xd2c16abd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2716ac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd16ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90316acc, 90 },
    { 0xec316acc, 55 },
    { 0x4fd16ace, 95 },
    { 0, 0 },
    { 0xc3116ad0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d16ad4, 76 },
    { 0xc4916ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91816ada, 76 },
    { 0, 0 },
    { 0x13e16adc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1016ae1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb16ae8, 33 },
    { 0, 0 },
    { 0x6c316aea, 61 },
    { 0xa7e16aea, 67 },
    { 0xfa616aec, 91 },
    { 0xcea16aec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25916afa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d16b05, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c816b0a, 23 },
    { 0, 0 },
    { 0xabf16b0c, 98 },
    { 0xfc116b0c, 90 },
    { 0x9b216b0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4116b16, 76 },
    { 0x19d16b16, 33 },
    { 0, 0 },
    { 0x43116b19, 46 },
    { 0x84e16b1a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd16b1d, 95 },
    { 0, 0 },
    { 0x8db16b1f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a16b22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9916b25, 12 },
    { 0x71c16b26, 85 },
    { 0, 0 },
    { 0x40716b28, 90 },
    { 0x19f16b29, 90 },
    { 0xba516b28, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34616b2e, 36 },
    { 0, 0 },
    { 0xe2816b30, 90 },
    { 0, 0 },
    { 0xc0416b32, 95 },
    { 0xde716b33, 76 },
    { 0, 0 },
    { 0x3d816b35, 76 },
    { 0xe3516b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9716b40, 34 },
    { 0xa2d16b40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47816b49, 90 },
    { 0, 0 },
    { 0x4aa16b4b, 93 },
    { 0xf8116b4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f16b51, 100 },
    { 0x48316b52, 98 },
    { 0, 0 },
    { 0x13916b54, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x10716b57, 95 },
    { 0xfd216b58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63616b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5316b60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa216b63, 75 },
    { 0xf9516b64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e16b6d, 91 },
    { 0, 0 },
    { 0x83c16b6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a16b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64816b7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a816b83, 85 },
    { 0xb3d16b83, 87 },
    { 0xd9f16b85, 78 },
    { 0x53416b86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c16b89, 56 },
    { 0xb3916b89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a16b8e, 46 },
    { 0x7916b8f, 61 },
    { 0xd1a16b90, 98 },
    { 0xb2b16b91, 55 },
    { 0x88716b91, 90 },
    { 0, 0 },
    { 0x65a16b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f516b97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6816b9b, 61 },
    { 0x81e16b9b, 34 },
    { 0x7a316b9b, 90 },
    { 0xb416b9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc16ba1, 34 },
    { 0xcf416ba2, 98 },
    { 0x2cc16ba3, 75 },
    { 0x5c816ba4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e16ba7, 84 },
    { 0xa0b16ba7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80116bab, 61 },
    { 0x83516bab, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b16baf, 33 },
    { 0x7416bb0, 68 },
    { 0x25816bb1, 55 },
    { 0x5cf16bb0, 85 },
    { 0xdba16bb3, 61 },
    { 0x5e516bb2, 95 },
    { 0xd3b16bb5, 61 },
    { 0x94b16bb5, 23 },
    { 0x60216bb5, 23 },
    { 0x68816bb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a316bbc, 76 },
    { 0, 0 },
    { 0xa5d16bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93216bc2, 55 },
    { 0x83916bc3, 82 },
    { 0, 0 },
    { 0x40a16bc5, 43 },
    { 0x27f16bc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9016bcb, 90 },
    { 0x8b716bcc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d16bcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d416bd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc16bde, 34 },
    { 0, 0 },
    { 0x42516be0, 65 },
    { 0x3eb16be0, 61 },
    { 0x2dc16be2, 90 },
    { 0, 0 },
    { 0x34416be4, 80 },
    { 0xe5116be4, 90 },
    { 0xd016be6, 95 },
    { 0, 0 },
    { 0x24416be8, 76 },
    { 0, 0 },
    { 0x2e416bea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87416bee, 95 },
    { 0xc3516bee, 90 },
    { 0x7d916bee, 98 },
    { 0xb8816bf0, 93 },
    { 0x38316bf2, 90 },
    { 0xfde16bf3, 93 },
    { 0x93016bf4, 76 },
    { 0xf5116bf2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe416bf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f16c00, 98 },
    { 0xcbf16c01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c16c04, 33 },
    { 0xe416c05, 98 },
    { 0xc4816c06, 76 },
    { 0, 0 },
    { 0xc2016c08, 61 },
    { 0x61f16c09, 90 },
    { 0x79916c08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e416c10, 66 },
    { 0x37016c10, 98 },
    { 0, 0 },
    { 0x57f16c13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b16c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71016c20, 90 },
    { 0x90316c21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d916c24, 98 },
    { 0, 0 },
    { 0x4d616c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d16c2e, 51 },
    { 0, 0 },
    { 0xd2d16c30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94516c36, 90 },
    { 0xa9016c37, 90 },
    { 0xf8c16c37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb16c3d, 93 },
    { 0, 0 },
    { 0x8c016c3f, 66 },
    { 0xab116c3f, 23 },
    { 0x84d16c41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f816c44, 87 },
    { 0xe8a16c44, 98 },
    { 0xb0216c44, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e016c49, 34 },
    { 0x78816c4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16c53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f716c56, 98 },
    { 0x2e816c57, 95 },
    { 0xc7c16c58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4016c5c, 61 },
    { 0x56816c5c, 43 },
    { 0x7d916c5e, 76 },
    { 0, 0 },
    { 0x8016c60, 67 },
    { 0x81116c61, 36 },
    { 0, 0 },
    { 0x80e16c63, 61 },
    { 0x8e716c64, 61 },
    { 0xa2c16c65, 95 },
    { 0xc4516c66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa816c69, 76 },
    { 0, 0 },
    { 0xe3816c6b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x64716c6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x89716c71, 90 },
    { 0x9c716c72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c16c7a, 61 },
    { 0x3e316c7b, 100 },
    { 0, 0 },
    { 0x3ee16c7d, 23 },
    { 0x25216c7e, 61 },
    { 0x40416c7f, 23 },
    { 0, 0 },
    { 0x5f816c81, 98 },
    { 0, 0 },
    { 0x12a16c83, 95 },
    { 0x79216c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9516c87, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff16c8f, 40 },
    { 0xbd516c90, 61 },
    { 0, 0 },
    { 0x8c716c92, 76 },
    { 0x92416c93, 33 },
    { 0x2116c93, 66 },
    { 0xd316c93, 95 },
    { 0xfc16c96, 83 },
    { 0x67016c97, 90 },
    { 0x27516c92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20616c9d, 67 },
    { 0x8f216c9d, 95 },
    { 0x89b16c9d, 95 },
    { 0x28516ca0, 93 },
    { 0xa7e16c9e, 23 },
    { 0x99616c9d, 61 },
    { 0, 0 },
    { 0x28416ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ca7, 61 },
    { 0xa9f16ca7, 10 },
    { 0x9df16ca9, 84 },
    { 0x6ac16ca8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de16cad, 90 },
    { 0, 0 },
    { 0x4d16caf, 98 },
    { 0x6e916cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e916cb3, 23 },
    { 0, 0 },
    { 0x6916cb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816cbb, 43 },
    { 0, 0 },
    { 0x7fe16cbd, 90 },
    { 0xabf16cbd, 23 },
    { 0xb4e16cbd, 33 },
    { 0, 0 },
    { 0x53c16cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d116cc7, 77 },
    { 0x90216cc7, 34 },
    { 0xc8516cc8, 33 },
    { 0x7016cca, 23 },
    { 0xb9e16ccb, 33 },
    { 0xbb816ccc, 53 },
    { 0, 0 },
    { 0xe6d16cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8616cd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63616cd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f16cd9, 91 },
    { 0, 0 },
    { 0x3ab16cdb, 23 },
    { 0xcc16cdb, 80 },
    { 0x9ab16cdd, 95 },
    { 0xfab16cde, 43 },
    { 0x72a16cde, 33 },
    { 0, 0 },
    { 0x9d316ce1, 61 },
    { 0x7fb16ce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc416ce5, 55 },
    { 0, 0 },
    { 0xc6c16ce7, 98 },
    { 0x5ce16ce8, 90 },
    { 0x73816ce9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a16cef, 91 },
    { 0x9cf16cf0, 95 },
    { 0xed016cf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x87516cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c16cfa, 98 },
    { 0x84716cfb, 33 },
    { 0x39516cfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36516d00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41216d07, 90 },
    { 0x79816d08, 93 },
    { 0, 0 },
    { 0xf8e16d0a, 34 },
    { 0x60416d0b, 43 },
    { 0, 0 },
    { 0x116d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f16d10, 33 },
    { 0x1da16d11, 61 },
    { 0x1ef16d12, 23 },
    { 0x1b416d13, 90 },
    { 0, 0 },
    { 0x2c816d15, 99 },
    { 0, 0 },
    { 0x9fd16d17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4316d1b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9316d21, 78 },
    { 0x11516d21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1916d25, 90 },
    { 0xdc316d25, 91 },
    { 0x20a16d25, 33 },
    { 0xf2816d27, 69 },
    { 0x57016d29, 95 },
    { 0x9c416d2a, 93 },
    { 0, 0 },
    { 0x3b216d2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb416d2f, 61 },
    { 0, 0 },
    { 0xedd16d31, 90 },
    { 0, 0 },
    { 0x80716d33, 90 },
    { 0, 0 },
    { 0xe0a16d35, 52 },
    { 0x42a16d36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a16d3a, 90 },
    { 0x86016d3b, 61 },
    { 0x34016d3b, 95 },
    { 0, 0 },
    { 0xe0e16d3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e16d42, 23 },
    { 0x1616d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e16d48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81916d4c, 90 },
    { 0xbe16d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb16d50, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71616d54, 95 },
    { 0, 0 },
    { 0x22716d56, 40 },
    { 0xd5516d57, 76 },
    { 0x62516d58, 78 },
    { 0x57016d56, 98 },
    { 0, 0 },
    { 0x47916d5b, 55 },
    { 0xcb916d5c, 91 },
    { 0xbf16d5d, 93 },
    { 0x4e016d5e, 77 },
    { 0x85016d5f, 33 },
    { 0x89f16d5e, 93 },
    { 0xae016d61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb16d66, 76 },
    { 0x47416d67, 68 },
    { 0xaba16d67, 36 },
    { 0, 0 },
    { 0x38016d6a, 23 },
    { 0x5d516d6b, 23 },
    { 0x4a916d6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1516d6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a616d72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36416d75, 61 },
    { 0x18316d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc516d79, 55 },
    { 0xf9516d79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c16d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d16d85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f016d8a, 34 },
    { 0x35f16d8b, 55 },
    { 0, 0 },
    { 0xaf616d8d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb616d92, 98 },
    { 0xe4f16d93, 76 },
    { 0x55116d94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44616d9a, 100 },
    { 0x6416d9b, 90 },
    { 0x98416d9b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x57316d9f, 95 },
    { 0, 0 },
    { 0x89916da1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98316da7, 34 },
    { 0xdfa16da8, 98 },
    { 0xfde16da9, 95 },
    { 0xd4116da9, 91 },
    { 0, 0 },
    { 0x9ce16dac, 90 },
    { 0xa3116dad, 55 },
    { 0, 0 },
    { 0xde816daf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe416db4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b16dc5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f16dce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd16dd1, 23 },
    { 0xa2e16dd2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a16dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa16ddb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c016ddf, 46 },
    { 0x14e16de0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98916de5, 46 },
    { 0x8b216de6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c916deb, 36 },
    { 0xc5416dec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71816df0, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e16df7, 78 },
    { 0x9e816df8, 90 },
    { 0xb4416df9, 90 },
    { 0xab216df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27016dfe, 84 },
    { 0, 0 },
    { 0x31116e00, 85 },
    { 0xd3516e00, 98 },
    { 0x4bd16e02, 61 },
    { 0x96416e03, 34 },
    { 0x82c16e04, 98 },
    { 0xd8816e05, 90 },
    { 0x95d16e05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66416e0d, 52 },
    { 0xbcf16e0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a16e11, 33 },
    { 0x26c16e11, 98 },
    { 0x2ea16e12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd516e1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c016e27, 43 },
    { 0x45416e28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc416e2c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3716e31, 44 },
    { 0x44c16e32, 95 },
    { 0xe3416e32, 43 },
    { 0x82f16e31, 23 },
    { 0xd7116e31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c16e3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28516e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a16e41, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9816e44, 23 },
    { 0, 0 },
    { 0x29716e46, 85 },
    { 0xe1e16e47, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad716e4a, 33 },
    { 0xb8616e4b, 55 },
    { 0x67d16e4c, 95 },
    { 0x6c816e4d, 66 },
    { 0x3e016e4e, 59 },
    { 0xc216e4c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4316e55, 100 },
    { 0xa316e56, 90 },
    { 0x73216e56, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0916e5a, 33 },
    { 0, 0 },
    { 0x82916e5c, 17 },
    { 0x9ff16e5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81816e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88816e68, 33 },
    { 0x34c16e69, 91 },
    { 0x1ca16e6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11216e6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f516e75, 52 },
    { 0xe916e76, 33 },
    { 0xbf16e77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67116e7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f416e80, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec916e86, 43 },
    { 0, 0 },
    { 0x57a16e88, 43 },
    { 0, 0 },
    { 0x8a516e8a, 83 },
    { 0xd4b16e8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f16e8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f16e93, 34 },
    { 0, 0 },
    { 0xd5416e95, 91 },
    { 0xbe116e95, 90 },
    { 0xe4f16e97, 91 },
    { 0x2a416e98, 69 },
    { 0x26816e99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf16e9c, 95 },
    { 0, 0 },
    { 0x23816e9e, 61 },
    { 0xf616e9f, 77 },
    { 0, 0 },
    { 0xb116ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8916ea9, 90 },
    { 0xacd16ea9, 23 },
    { 0xe6516eab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec16eaf, 90 },
    { 0, 0 },
    { 0x53f16eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d916ec1, 95 },
    { 0x23816ec2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf16ec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d16ecb, 98 },
    { 0xde616ecb, 33 },
    { 0x1bf16ecc, 61 },
    { 0, 0 },
    { 0x23c16ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b16ed6, 23 },
    { 0xfb216ed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a316edb, 90 },
    { 0, 0 },
    { 0xafd16edd, 46 },
    { 0, 0 },
    { 0x85a16edf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5016ee3, 66 },
    { 0x68116ee4, 34 },
    { 0xb5616ee3, 84 },
    { 0x4c216ee6, 66 },
    { 0, 0 },
    { 0xe9b16ee8, 61 },
    { 0xf916ee8, 95 },
    { 0xe0816ee8, 95 },
    { 0, 0 },
    { 0x28e16eec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf216ef0, 88 },
    { 0, 0 },
    { 0x1b416ef2, 33 },
    { 0, 0 },
    { 0x3016ef4, 90 },
    { 0x4516ef5, 61 },
    { 0xae816ef5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec16eff, 61 },
    { 0x70316eff, 98 },
    { 0x38716f01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e616f06, 61 },
    { 0, 0 },
    { 0x9ee16f08, 46 },
    { 0xdfa16f09, 33 },
    { 0, 0 },
    { 0x10116f0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd116f0e, 77 },
    { 0x74216f0f, 33 },
    { 0x37b16f10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a16f13, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f16f18, 95 },
    { 0, 0 },
    { 0x57416f1a, 76 },
    { 0x9c516f1a, 43 },
    { 0, 0 },
    { 0xb5816f1d, 90 },
    { 0, 0 },
    { 0x1e416f1f, 76 },
    { 0x3eb16f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc016f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e16f28, 95 },
    { 0, 0 },
    { 0xf2216f2a, 33 },
    { 0xa7e16f2b, 85 },
    { 0, 0 },
    { 0x72a16f2d, 33 },
    { 0xe0016f2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e316f32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee16f38, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc16f3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c16f43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda16f46, 33 },
    { 0, 0 },
    { 0x7fb16f48, 52 },
    { 0, 0 },
    { 0x7e316f4a, 95 },
    { 0, 0 },
    { 0x88316f4c, 23 },
    { 0, 0 },
    { 0xd9f16f4e, 61 },
    { 0x4316f4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a16f54, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd216f5b, 78 },
    { 0, 0 },
    { 0x2d816f5d, 68 },
    { 0x95016f5d, 90 },
    { 0x23e16f5f, 95 },
    { 0x36116f60, 33 },
    { 0xaee16f61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea16f6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x98416f72, 95 },
    { 0x87d16f73, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xee616f76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77416f79, 98 },
    { 0, 0 },
    { 0x21a16f7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa016f81, 61 },
    { 0xfd916f82, 46 },
    { 0x45716f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba16f86, 93 },
    { 0, 0 },
    { 0x716f88, 37 },
    { 0, 0 },
    { 0xe5116f8a, 95 },
    { 0, 0 },
    { 0x88a16f8c, 55 },
    { 0x11216f8c, 65 },
    { 0, 0 },
    { 0xccb16f8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae16f9c, 55 },
    { 0x45116f9d, 33 },
    { 0, 0 },
    { 0x89216f9f, 98 },
    { 0, 0 },
    { 0xf9616fa1, 76 },
    { 0x83816fa2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b016fb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47816fb6, 61 },
    { 0xabb16fb6, 95 },
    { 0x61216fb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e016fbc, 23 },
    { 0, 0 },
    { 0x9f416fbe, 66 },
    { 0xe4016fbf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77516fc5, 93 },
    { 0xf5e16fc5, 33 },
    { 0xadd16fc7, 90 },
    { 0xc0416fc8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c316fcc, 40 },
    { 0, 0 },
    { 0xa5916fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da16fd2, 61 },
    { 0, 0 },
    { 0x84616fd4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af16fd9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b16fdc, 98 },
    { 0xbe616fdd, 67 },
    { 0x71516fdd, 95 },
    { 0xc8c16fdf, 33 },
    { 0x82416fde, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f16fe5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d16fe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e816fee, 90 },
    { 0x5f316fee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x83116ff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26716ff7, 34 },
    { 0x51316ff7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67117002, 34 },
    { 0x27317002, 43 },
    { 0, 0 },
    { 0xae217005, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa371700a, 46 },
    { 0, 0 },
    { 0xfd21700c, 95 },
    { 0, 0 },
    { 0xbf71700e, 34 },
    { 0x2d21700f, 61 },
    { 0x3291700f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e17013, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10017016, 90 },
    { 0x19e17016, 90 },
    { 0x86417018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb111701c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed1701f, 56 },
    { 0, 0 },
    { 0x91717021, 33 },
    { 0x9fe17021, 87 },
    { 0xc917023, 43 },
    { 0x4e617023, 95 },
    { 0x6fb17025, 61 },
    { 0xf1017025, 55 },
    { 0x4a017027, 33 },
    { 0x3c517028, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e1702f, 100 },
    { 0, 0 },
    { 0x13f17031, 98 },
    { 0xd1a17032, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c17035, 90 },
    { 0xb2b17035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f417046, 44 },
    { 0, 0 },
    { 0x29b17048, 75 },
    { 0, 0 },
    { 0xc5a1704a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2601704d, 33 },
    { 0x9bb1704d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb517051, 95 },
    { 0, 0 },
    { 0xe7517053, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9621705a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6301705d, 88 },
    { 0x21705e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a17061, 76 },
    { 0, 0 },
    { 0x29c17063, 87 },
    { 0, 0 },
    { 0xa6217065, 23 },
    { 0x6ca17065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d17069, 33 },
    { 0, 0 },
    { 0x13a1706b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8217076, 76 },
    { 0x65b17076, 55 },
    { 0xb1717077, 33 },
    { 0xf2117079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc1707c, 90 },
    { 0x4921707d, 23 },
    { 0x17b1707d, 33 },
    { 0x4c11707f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36817084, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43917088, 93 },
    { 0x31717088, 76 },
    { 0xe9c1708a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab1708f, 99 },
    { 0, 0 },
    { 0x12d17091, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7317095, 95 },
    { 0xef717096, 90 },
    { 0xa2917096, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a81709a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd01709d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab170a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a170a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56170aa, 77 },
    { 0x949170aa, 90 },
    { 0x1e2170ac, 66 },
    { 0x9a9170ac, 76 },
    { 0, 0 },
    { 0x320170af, 61 },
    { 0xda2170b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80170b7, 55 },
    { 0, 0 },
    { 0x33170b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8170bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f170c2, 23 },
    { 0xe78170c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c170c8, 76 },
    { 0xe3d170c9, 96 },
    { 0xe53170ca, 98 },
    { 0x426170c9, 92 },
    { 0xc37170cc, 23 },
    { 0xc07170cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3170d2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9170d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9170e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e170e7, 95 },
    { 0, 0 },
    { 0xe97170e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37170ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f170f0, 43 },
    { 0, 0 },
    { 0x50c170f2, 95 },
    { 0, 0 },
    { 0xf02170f4, 90 },
    { 0x6dd170f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47170f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581170fc, 90 },
    { 0xa45170fc, 69 },
    { 0, 0 },
    { 0xb91170ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b617103, 61 },
    { 0xdfc17104, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed17107, 73 },
    { 0xd3317108, 55 },
    { 0xc4d17109, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41710d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7617112, 90 },
    { 0x2ce17112, 90 },
    { 0, 0 },
    { 0x42b17115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11711c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7491711f, 33 },
    { 0x70317120, 61 },
    { 0xabe17121, 76 },
    { 0xeb517122, 46 },
    { 0x7a117120, 90 },
    { 0, 0 },
    { 0x28d17125, 93 },
    { 0x60817125, 99 },
    { 0xdc617127, 93 },
    { 0x29d17126, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41712b, 76 },
    { 0x68d1712c, 55 },
    { 0x5f11712d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x70717130, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe17134, 33 },
    { 0xea817135, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5717139, 90 },
    { 0x9581713a, 19 },
    { 0x25b1713b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db1713e, 23 },
    { 0x2a01713f, 90 },
    { 0, 0 },
    { 0x7217141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79617145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f017149, 95 },
    { 0xe5f1714a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee81714f, 34 },
    { 0x11d1714f, 61 },
    { 0, 0 },
    { 0xb2c17152, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3317156, 95 },
    { 0, 0 },
    { 0x6ad17158, 33 },
    { 0x13c17159, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c1715c, 23 },
    { 0xfd81715d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf17165, 55 },
    { 0xe3a17166, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831716b, 40 },
    { 0, 0 },
    { 0x4271716d, 61 },
    { 0, 0 },
    { 0xc7b1716f, 23 },
    { 0, 0 },
    { 0xd4817171, 33 },
    { 0, 0 },
    { 0xfbe17173, 66 },
    { 0x91517174, 90 },
    { 0xd4c17174, 90 },
    { 0x64717176, 84 },
    { 0x3a817177, 43 },
    { 0x40e17178, 43 },
    { 0x66817177, 99 },
    { 0xc3b17179, 40 },
    { 0xc0f17177, 95 },
    { 0, 0 },
    { 0xd6b1717d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40917181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f17186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb811718e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef217191, 34 },
    { 0xc4a17192, 40 },
    { 0x5c117193, 98 },
    { 0x9f717194, 91 },
    { 0xeab17195, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f217198, 91 },
    { 0, 0 },
    { 0xd21719a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7141719f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731171a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1171aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4171ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3171b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903171bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7171c0, 90 },
    { 0x548171c1, 91 },
    { 0xa27171c0, 88 },
    { 0, 0 },
    { 0xf05171c4, 23 },
    { 0xc12171c4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3171c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x539171cb, 61 },
    { 0x286171cb, 46 },
    { 0x705171cd, 43 },
    { 0x4ef171ce, 61 },
    { 0, 0 },
    { 0xe42171d0, 76 },
    { 0xa02171d0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df171d9, 98 },
    { 0xe46171da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5171e1, 98 },
    { 0x11d171e2, 90 },
    { 0xa1b171e1, 33 },
    { 0x2d0171e3, 90 },
    { 0, 0 },
    { 0x9c6171e6, 46 },
    { 0x732171e7, 48 },
    { 0, 0 },
    { 0xcaa171e9, 90 },
    { 0xc3b171ea, 23 },
    { 0, 0 },
    { 0x1e4171ec, 46 },
    { 0x63b171ed, 91 },
    { 0x65d171ed, 61 },
    { 0xe47171ed, 95 },
    { 0x700171ef, 55 },
    { 0x7e5171f1, 61 },
    { 0xab6171f1, 46 },
    { 0xbb8171f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40171f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0171fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e171fd, 90 },
    { 0xd1a171fe, 33 },
    { 0xb66171ff, 61 },
    { 0xc0e171fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3217203, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7117208, 76 },
    { 0, 0 },
    { 0xa671720a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d31720f, 23 },
    { 0x18c17210, 33 },
    { 0xd0117211, 100 },
    { 0x36217210, 85 },
    { 0x8a017213, 46 },
    { 0, 0 },
    { 0xe8f17215, 61 },
    { 0x95917215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e1721a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb41721d, 95 },
    { 0x8831721e, 95 },
    { 0x6c1721e, 101 },
    { 0x5aa1721f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x75717223, 65 },
    { 0xfc417224, 61 },
    { 0xa0d17223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d117228, 99 },
    { 0xf2317229, 61 },
    { 0x1611722a, 78 },
    { 0, 0 },
    { 0x56f1722c, 98 },
    { 0, 0 },
    { 0xc1a1722e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d17235, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc41723b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3191723f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23017242, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d617246, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a17249, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1724d, 23 },
    { 0, 0 },
    { 0x39c1724f, 95 },
    { 0x8241724f, 99 },
    { 0x5d817251, 98 },
    { 0xe3717252, 43 },
    { 0, 0 },
    { 0x92e17254, 43 },
    { 0xd217255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f17259, 98 },
    { 0, 0 },
    { 0xda81725b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc1725f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27617264, 61 },
    { 0xd017265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77517269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f71726e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac917274, 76 },
    { 0x71217275, 55 },
    { 0x94617274, 46 },
    { 0x67517274, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa571727f, 61 },
    { 0x5cb17280, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f417285, 43 },
    { 0x3cf17286, 33 },
    { 0x46a17287, 93 },
    { 0xed117288, 34 },
    { 0xdad17289, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1728c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce717290, 66 },
    { 0xdb417290, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1371729b, 40 },
    { 0x4b81729b, 95 },
    { 0x20c1729c, 55 },
    { 0xadd1729e, 55 },
    { 0xd8c1729f, 67 },
    { 0x3971729b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55172a3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a172a6, 34 },
    { 0xfea172a6, 61 },
    { 0xe33172a8, 81 },
    { 0x253172a9, 81 },
    { 0x6ce172a6, 61 },
    { 0, 0 },
    { 0xee9172ac, 98 },
    { 0x287172ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15172b0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x100172b3, 75 },
    { 0, 0 },
    { 0x306172b5, 75 },
    { 0xc1f172b6, 55 },
    { 0, 0 },
    { 0x55d172b8, 90 },
    { 0xc08172b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9172bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x507172bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a172c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0172c8, 96 },
    { 0xf1c172c9, 61 },
    { 0x613172c9, 23 },
    { 0xcdf172cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd172d3, 33 },
    { 0xb45172d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x715172d7, 23 },
    { 0, 0 },
    { 0xd92172d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573172e1, 23 },
    { 0, 0 },
    { 0xa28172e3, 46 },
    { 0x1c0172e4, 61 },
    { 0, 0 },
    { 0x860172e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46172ea, 34 },
    { 0, 0 },
    { 0x58b172ec, 90 },
    { 0x6af172ed, 93 },
    { 0xf2e172ec, 98 },
    { 0, 0 },
    { 0xc1d172f0, 46 },
    { 0, 0 },
    { 0x515172f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e172f8, 46 },
    { 0, 0 },
    { 0xe8b172fa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0172fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e17300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb17303, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a17308, 33 },
    { 0, 0 },
    { 0x7921730a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa121730e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac17313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2117316, 91 },
    { 0, 0 },
    { 0xf7017318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae1731e, 33 },
    { 0xb331731f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8717322, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4717327, 95 },
    { 0x73517328, 85 },
    { 0, 0 },
    { 0x7ce1732a, 90 },
    { 0x8f11732a, 98 },
    { 0x2591732c, 91 },
    { 0, 0 },
    { 0x6bf1732e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5017336, 56 },
    { 0xc7717337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8891733c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c417343, 69 },
    { 0xe5217343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c1734d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x71817350, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d417353, 99 },
    { 0x17817354, 85 },
    { 0xe7317354, 90 },
    { 0xb4917355, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd41735a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa631735f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e217362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf17366, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb1736a, 17 },
    { 0, 0 },
    { 0x5721736c, 84 },
    { 0xc4e1736d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b17372, 78 },
    { 0, 0 },
    { 0xc3917374, 93 },
    { 0xcec17374, 23 },
    { 0x3d817376, 98 },
    { 0xc417376, 23 },
    { 0xf3617378, 98 },
    { 0xd6017379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a17380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc17383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8451738a, 96 },
    { 0, 0 },
    { 0x16c1738c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8917392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd1739c, 8 },
    { 0x1761739c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2173a1, 43 },
    { 0x74b173a2, 61 },
    { 0xc3a173a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb173a6, 98 },
    { 0, 0 },
    { 0xa5a173a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741173ac, 98 },
    { 0x290173ad, 66 },
    { 0x679173ae, 91 },
    { 0x7aa173af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf173b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e173b7, 23 },
    { 0x198173b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e173be, 90 },
    { 0x9cb173bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7173c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122173ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0173cf, 43 },
    { 0x8bf173d0, 46 },
    { 0x206173d0, 33 },
    { 0x63b173d2, 99 },
    { 0xe10173d2, 46 },
    { 0x2cc173d2, 76 },
    { 0x8dd173d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x865173d8, 33 },
    { 0x909173d9, 90 },
    { 0x6dd173da, 61 },
    { 0x54c173d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56173e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc173e4, 52 },
    { 0x98c173e5, 90 },
    { 0xa29173e6, 23 },
    { 0, 0 },
    { 0xb31173e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd173eb, 34 },
    { 0, 0 },
    { 0x51a173ed, 23 },
    { 0xf01173ee, 46 },
    { 0, 0 },
    { 0xfac173f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e173f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4173fb, 55 },
    { 0xec3173fc, 55 },
    { 0xd56173fd, 33 },
    { 0, 0 },
    { 0xe3f173ff, 76 },
    { 0xfc117400, 61 },
    { 0, 0 },
    { 0xf5317402, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d617405, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c517408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf91740f, 33 },
    { 0x36c1740f, 78 },
    { 0x5e417411, 90 },
    { 0, 0 },
    { 0x3d217413, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf517416, 95 },
    { 0, 0 },
    { 0x80017418, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a21741c, 76 },
    { 0, 0 },
    { 0x9dd1741e, 23 },
    { 0, 0 },
    { 0x56d17420, 52 },
    { 0x94217421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94217426, 52 },
    { 0x39b17426, 61 },
    { 0xdc317428, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x8471742b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9541742e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea417431, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33317436, 33 },
    { 0x17417436, 61 },
    { 0x29617438, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d1743c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89717440, 97 },
    { 0, 0 },
    { 0xc5217442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e17448, 95 },
    { 0x72117449, 95 },
    { 0xd0317449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9061744d, 61 },
    { 0xa9a1744d, 34 },
    { 0x351744e, 95 },
    { 0x1e317450, 98 },
    { 0x8c217451, 90 },
    { 0xc5d1744d, 95 },
    { 0xa1017452, 61 },
    { 0, 0 },
    { 0x61817455, 46 },
    { 0x41d17456, 90 },
    { 0x12c17457, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6271745b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c1745e, 90 },
    { 0xc4e1745f, 46 },
    { 0x3bb1745f, 95 },
    { 0xca617461, 61 },
    { 0xb1317462, 66 },
    { 0, 0 },
    { 0xda17464, 23 },
    { 0x8be17465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab917468, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e1746c, 78 },
    { 0, 0 },
    { 0xfaf1746e, 90 },
    { 0xba81746f, 94 },
    { 0x3a61746f, 43 },
    { 0x15917470, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a317475, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c517478, 90 },
    { 0, 0 },
    { 0x1231747a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb517485, 90 },
    { 0, 0 },
    { 0xdb17487, 23 },
    { 0, 0 },
    { 0x42917489, 33 },
    { 0x6fa17489, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6031748e, 52 },
    { 0x471748f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0517494, 23 },
    { 0, 0 },
    { 0x58217496, 66 },
    { 0xf9f17497, 90 },
    { 0, 0 },
    { 0xe8f17499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e1749c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x331749f, 91 },
    { 0, 0 },
    { 0x416174a1, 98 },
    { 0, 0 },
    { 0xe6d174a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c174a7, 90 },
    { 0x58a174a8, 81 },
    { 0xaac174a7, 23 },
    { 0x7a0174a7, 90 },
    { 0, 0 },
    { 0x97a174ac, 61 },
    { 0x8cb174ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477174b1, 78 },
    { 0, 0 },
    { 0x19e174b3, 33 },
    { 0xb1c174b3, 95 },
    { 0x25a174b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba174b8, 98 },
    { 0, 0 },
    { 0x3b1174ba, 95 },
    { 0x859174ba, 33 },
    { 0x9f8174bb, 83 },
    { 0x222174bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe174c1, 95 },
    { 0x90174c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420174cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78174d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b174d7, 95 },
    { 0, 0 },
    { 0x58e174d9, 88 },
    { 0x412174d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e174e1, 61 },
    { 0xaa2174e2, 90 },
    { 0xde5174e3, 85 },
    { 0x58d174e3, 33 },
    { 0xb87174e5, 33 },
    { 0x3eb174e5, 55 },
    { 0x4a3174e3, 61 },
    { 0x16d174e4, 43 },
    { 0, 0 },
    { 0xdba174ea, 95 },
    { 0, 0 },
    { 0x9c8174ec, 43 },
    { 0, 0 },
    { 0x94174ee, 68 },
    { 0, 0 },
    { 0x8e2174f0, 46 },
    { 0, 0 },
    { 0xee5174f2, 85 },
    { 0xfe7174f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e174f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9174f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd174fe, 76 },
    { 0x9fc174ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c17504, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa17507, 33 },
    { 0, 0 },
    { 0x96417509, 90 },
    { 0, 0 },
    { 0x33f1750b, 55 },
    { 0, 0 },
    { 0xa961750d, 90 },
    { 0x38c1750d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a17512, 76 },
    { 0x9db17513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c117519, 100 },
    { 0, 0 },
    { 0x8941751b, 98 },
    { 0xd651751c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc17527, 43 },
    { 0x4c717528, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b41752b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53617536, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3631753a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e1753f, 55 },
    { 0x4c917540, 90 },
    { 0x11017540, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b817544, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b17548, 90 },
    { 0x8bb17549, 76 },
    { 0xb1a1754a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d1754d, 100 },
    { 0, 0 },
    { 0x1031754f, 34 },
    { 0, 0 },
    { 0x2c17551, 98 },
    { 0xee317552, 90 },
    { 0xa0a17551, 90 },
    { 0x49e17551, 95 },
    { 0x19017555, 33 },
    { 0xc8d17556, 61 },
    { 0xec117556, 95 },
    { 0x917551, 99 },
    { 0x8ae17559, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe631755c, 61 },
    { 0, 0 },
    { 0xc5d1755e, 85 },
    { 0xd491755e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75317567, 76 },
    { 0, 0 },
    { 0x6f217569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd1756d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e1757a, 34 },
    { 0xea01757b, 98 },
    { 0x4241757a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f617584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89917588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd831758b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1758f, 33 },
    { 0x50017590, 84 },
    { 0, 0 },
    { 0x6f117592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46917595, 90 },
    { 0x82d17596, 91 },
    { 0xb0417597, 43 },
    { 0x37117598, 95 },
    { 0xed117596, 33 },
    { 0x1db17596, 95 },
    { 0x2551759b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1175a4, 67 },
    { 0x3d8175a5, 61 },
    { 0, 0 },
    { 0xd2b175a7, 84 },
    { 0, 0 },
    { 0xbdb175a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab175ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9175b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a175b7, 67 },
    { 0x3a175b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963175c4, 61 },
    { 0, 0 },
    { 0x6f8175c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x569175c9, 87 },
    { 0x11c175ca, 72 },
    { 0xa25175c9, 98 },
    { 0x8a7175cb, 90 },
    { 0x8ae175cd, 98 },
    { 0x4b7175ce, 67 },
    { 0xa20175cf, 95 },
    { 0xdb3175cf, 33 },
    { 0x975175cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f175d4, 66 },
    { 0xfe3175d5, 53 },
    { 0x7d1175d5, 55 },
    { 0x246175d4, 95 },
    { 0xa3175d6, 95 },
    { 0x3f4175d9, 61 },
    { 0xc63175d9, 81 },
    { 0xcc3175da, 43 },
    { 0x5f7175dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8175e1, 23 },
    { 0, 0 },
    { 0x977175e3, 61 },
    { 0x7a7175e4, 66 },
    { 0xf1e175e5, 95 },
    { 0, 0 },
    { 0xe6d175e7, 98 },
    { 0xa97175e7, 98 },
    { 0, 0 },
    { 0x340175ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9175ef, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763175f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4175fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d17601, 55 },
    { 0xfb517602, 95 },
    { 0xd9017603, 61 },
    { 0, 0 },
    { 0xaa517605, 100 },
    { 0xdaf17606, 68 },
    { 0x1e217607, 55 },
    { 0xa8317608, 65 },
    { 0xe1017605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3771760d, 95 },
    { 0, 0 },
    { 0x13a1760f, 43 },
    { 0x80c1760f, 34 },
    { 0, 0 },
    { 0x9b717612, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a317616, 93 },
    { 0xc1e17617, 100 },
    { 0, 0 },
    { 0x57517619, 88 },
    { 0, 0 },
    { 0x9141761b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b1761f, 78 },
    { 0, 0 },
    { 0xa5117621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea217626, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc1762b, 61 },
    { 0xbf01762b, 33 },
    { 0, 0 },
    { 0x8671762e, 46 },
    { 0xd861762f, 90 },
    { 0x6811762f, 33 },
    { 0xa801762e, 61 },
    { 0xb6117630, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e17636, 46 },
    { 0x5e517637, 23 },
    { 0x40517637, 98 },
    { 0xbeb17638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee01763d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14917640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba1764a, 91 },
    { 0xd7d1764b, 76 },
    { 0xdde1764c, 78 },
    { 0x8c31764b, 84 },
    { 0x1881764d, 43 },
    { 0x37e1764f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc717653, 91 },
    { 0xfa317654, 43 },
    { 0x88117655, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x73417658, 33 },
    { 0x8a117658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f17662, 23 },
    { 0, 0 },
    { 0x13317664, 64 },
    { 0, 0 },
    { 0x6d17666, 61 },
    { 0x67717667, 55 },
    { 0, 0 },
    { 0xc3317669, 99 },
    { 0xa9d1766a, 78 },
    { 0x851766b, 23 },
    { 0xf181766a, 98 },
    { 0xed31766d, 98 },
    { 0, 0 },
    { 0x5b81766f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x45517672, 78 },
    { 0x1db17672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d17676, 61 },
    { 0, 0 },
    { 0xfd517678, 95 },
    { 0xb7f17679, 91 },
    { 0, 0 },
    { 0x8b31767b, 99 },
    { 0x7571767c, 17 },
    { 0x7531767d, 90 },
    { 0xbc61767d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf17681, 78 },
    { 0xca417681, 61 },
    { 0, 0 },
    { 0x88817684, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd317689, 34 },
    { 0x3431768a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d17694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7717698, 33 },
    { 0x63117699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca1769d, 91 },
    { 0, 0 },
    { 0xcf1769f, 55 },
    { 0x18d1769f, 34 },
    { 0x3c41769f, 93 },
    { 0x9ab176a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2176a6, 93 },
    { 0, 0 },
    { 0x417176a8, 98 },
    { 0, 0 },
    { 0x4ba176aa, 33 },
    { 0xf97176aa, 33 },
    { 0x9ab176ab, 33 },
    { 0, 0 },
    { 0xd65176ae, 33 },
    { 0x499176af, 90 },
    { 0x485176b0, 85 },
    { 0xde9176ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x321176b4, 61 },
    { 0, 0 },
    { 0xe27176b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06176b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f176bf, 33 },
    { 0, 0 },
    { 0x6de176c1, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170176cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0176d0, 33 },
    { 0x979176d0, 61 },
    { 0x9be176d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382176db, 34 },
    { 0, 0 },
    { 0xa16176dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51176e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e176e8, 66 },
    { 0, 0 },
    { 0x74d176ea, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb176f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3176fd, 95 },
    { 0, 0 },
    { 0xd60176ff, 95 },
    { 0xd2d17700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0417703, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc617707, 95 },
    { 0x5f717708, 98 },
    { 0x30b17709, 23 },
    { 0x38a17707, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27317710, 98 },
    { 0, 0 },
    { 0xbbb17712, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70317715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb17719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b01771c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a817722, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2917725, 40 },
    { 0, 0 },
    { 0xedf17727, 38 },
    { 0xd2017728, 46 },
    { 0, 0 },
    { 0xee31772a, 76 },
    { 0x2251772a, 33 },
    { 0, 0 },
    { 0x2af1772d, 99 },
    { 0xada1772d, 23 },
    { 0, 0 },
    { 0x37f17730, 90 },
    { 0x75b17731, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2311773c, 90 },
    { 0x7271773d, 34 },
    { 0xc971773e, 33 },
    { 0x1f51773f, 85 },
    { 0xe5c17740, 55 },
    { 0x5e21773d, 33 },
    { 0, 0 },
    { 0xae917743, 78 },
    { 0x6a717744, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe17747, 95 },
    { 0, 0 },
    { 0xfdb17749, 55 },
    { 0xe8817749, 46 },
    { 0x7517749, 34 },
    { 0xceb1774c, 98 },
    { 0x9081774c, 33 },
    { 0x9561774e, 76 },
    { 0xab417749, 33 },
    { 0xd7817750, 77 },
    { 0xa551774a, 76 },
    { 0xdb217752, 95 },
    { 0xdab17752, 96 },
    { 0x22b17754, 91 },
    { 0xd7217749, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c17759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc1775c, 55 },
    { 0, 0 },
    { 0xdfa1775e, 23 },
    { 0x6851775e, 23 },
    { 0xd9f17760, 98 },
    { 0x16e17761, 87 },
    { 0, 0 },
    { 0x6f917763, 33 },
    { 0xf8117764, 43 },
    { 0, 0 },
    { 0xfee17766, 93 },
    { 0x60217767, 61 },
    { 0x7eb17767, 69 },
    { 0, 0 },
    { 0x6f21776a, 90 },
    { 0, 0 },
    { 0xa0d1776c, 81 },
    { 0, 0 },
    { 0x8491776e, 93 },
    { 0x7591776f, 61 },
    { 0x5f51776e, 95 },
    { 0xc2717771, 23 },
    { 0x4ef17772, 43 },
    { 0x9317771, 23 },
    { 0x87f17771, 76 },
    { 0, 0 },
    { 0x23f17776, 34 },
    { 0x30417776, 67 },
    { 0x4d317778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a21777b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22117785, 99 },
    { 0, 0 },
    { 0xd0717787, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b01778a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68017792, 87 },
    { 0x83917793, 98 },
    { 0x74417794, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3717797, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf177aa, 77 },
    { 0x30e177ab, 76 },
    { 0xea7177aa, 33 },
    { 0xaa6177ad, 34 },
    { 0, 0 },
    { 0x592177af, 95 },
    { 0, 0 },
    { 0x6aa177b1, 90 },
    { 0x678177b2, 55 },
    { 0x648177b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f177b6, 85 },
    { 0, 0 },
    { 0x322177b8, 23 },
    { 0x209177b9, 90 },
    { 0xc94177ba, 84 },
    { 0x794177bb, 61 },
    { 0, 0 },
    { 0x2a8177bd, 93 },
    { 0x358177be, 61 },
    { 0x374177bd, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157177c3, 23 },
    { 0x192177c4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a177ca, 78 },
    { 0xe23177cb, 88 },
    { 0x94d177ca, 52 },
    { 0, 0 },
    { 0xac6177ce, 90 },
    { 0x8e6177cf, 76 },
    { 0x629177d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f177d3, 95 },
    { 0, 0 },
    { 0x89177d5, 34 },
    { 0, 0 },
    { 0x22f177d7, 23 },
    { 0x59e177d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4177dc, 55 },
    { 0x1b5177dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b177e3, 95 },
    { 0xe48177e4, 34 },
    { 0, 0 },
    { 0x6f1177e6, 34 },
    { 0xdce177e7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4177ec, 90 },
    { 0x41c177ed, 91 },
    { 0x570177ed, 66 },
    { 0xce0177ef, 90 },
    { 0xe50177ed, 61 },
    { 0, 0 },
    { 0x843177f2, 76 },
    { 0, 0 },
    { 0x1fb177f4, 52 },
    { 0, 0 },
    { 0xe71177f6, 61 },
    { 0, 0 },
    { 0xde3177f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a617800, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb17806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f71780b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b17811, 61 },
    { 0xc4317812, 76 },
    { 0xdbd17811, 66 },
    { 0x42017811, 61 },
    { 0xdfc17815, 87 },
    { 0x80617813, 33 },
    { 0x11017811, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41781a, 76 },
    { 0x6391781b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf17820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e17829, 61 },
    { 0xebf17829, 55 },
    { 0x7641782b, 29 },
    { 0xa1e1782c, 87 },
    { 0xa721782c, 95 },
    { 0xb941782e, 52 },
    { 0xbe91782e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c17838, 23 },
    { 0xc2417839, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e17840, 85 },
    { 0xa7017840, 61 },
    { 0xbb317842, 76 },
    { 0x47317841, 33 },
    { 0x6c617843, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021784c, 23 },
    { 0, 0 },
    { 0x60c1784e, 55 },
    { 0, 0 },
    { 0x38c17850, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc617857, 61 },
    { 0x6c217858, 23 },
    { 0xf2c17859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae1785c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca217860, 98 },
    { 0, 0 },
    { 0xfeb17862, 61 },
    { 0, 0 },
    { 0x73317864, 61 },
    { 0, 0 },
    { 0x85c17866, 90 },
    { 0x10317866, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71786b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17017870, 98 },
    { 0x51117871, 23 },
    { 0x21317872, 17 },
    { 0, 0 },
    { 0x81117874, 23 },
    { 0x29417875, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95717879, 85 },
    { 0xa061787a, 88 },
    { 0xdc81787a, 87 },
    { 0, 0 },
    { 0xd821787d, 76 },
    { 0xca01787d, 98 },
    { 0x1691787e, 90 },
    { 0x6d517880, 99 },
    { 0x891787d, 55 },
    { 0x7081787d, 23 },
    { 0xb6417883, 98 },
    { 0x4d61787e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e017887, 95 },
    { 0xd8217888, 95 },
    { 0xa7717888, 33 },
    { 0xc41788a, 95 },
    { 0xef11788b, 33 },
    { 0xed1788b, 90 },
    { 0, 0 },
    { 0x3bb1788e, 91 },
    { 0, 0 },
    { 0x58017890, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a17896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x91617899, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb1789e, 90 },
    { 0, 0 },
    { 0x2ab178a0, 46 },
    { 0xc8e178a1, 84 },
    { 0, 0 },
    { 0x6e178a3, 33 },
    { 0x1dc178a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304178ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c178af, 34 },
    { 0xa86178af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77178b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d178b7, 33 },
    { 0xfe0178b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132178bc, 33 },
    { 0, 0 },
    { 0x818178be, 90 },
    { 0, 0 },
    { 0xd8e178c0, 78 },
    { 0x31a178c0, 98 },
    { 0x86f178c2, 98 },
    { 0x9fa178c2, 33 },
    { 0x959178c2, 99 },
    { 0, 0 },
    { 0x57178c6, 46 },
    { 0x8d1178c6, 61 },
    { 0xca6178c8, 95 },
    { 0x118178c9, 17 },
    { 0x873178ca, 90 },
    { 0x70178c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a178ce, 100 },
    { 0, 0 },
    { 0x8fa178d0, 76 },
    { 0, 0 },
    { 0x2db178d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe178d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66178da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1178dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66178e1, 90 },
    { 0x1ea178e1, 90 },
    { 0x103178e3, 91 },
    { 0, 0 },
    { 0x58a178e5, 61 },
    { 0x51d178e6, 81 },
    { 0x204178e6, 77 },
    { 0x607178e7, 56 },
    { 0x4ef178e9, 98 },
    { 0x2c178ea, 23 },
    { 0xa37178e7, 90 },
    { 0, 0 },
    { 0xd7b178ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82178f5, 95 },
    { 0x69a178f5, 90 },
    { 0xaea178f7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84178ff, 24 },
    { 0x5d617900, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d117906, 24 },
    { 0x36917906, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5821790c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a17911, 88 },
    { 0, 0 },
    { 0xf17913, 78 },
    { 0x91017914, 33 },
    { 0x13217914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f31791b, 90 },
    { 0x9411791c, 90 },
    { 0x3211791d, 68 },
    { 0x6601791b, 90 },
    { 0xba91791d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c17924, 90 },
    { 0, 0 },
    { 0x7bd17926, 23 },
    { 0xf2417927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa1792a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9421792e, 33 },
    { 0, 0 },
    { 0x9f717930, 23 },
    { 0xe6617931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd17934, 43 },
    { 0x65417934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c17938, 43 },
    { 0xf6b17939, 95 },
    { 0x7eb17939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb411793e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa221794f, 34 },
    { 0, 0 },
    { 0xfa917951, 34 },
    { 0x77f17952, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d17955, 91 },
    { 0, 0 },
    { 0xdc217957, 55 },
    { 0x7df17958, 33 },
    { 0, 0 },
    { 0x181795a, 65 },
    { 0xefd1795b, 90 },
    { 0, 0 },
    { 0x90a1795d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3317963, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae81796e, 98 },
    { 0x2cf1796f, 90 },
    { 0x70f1796f, 90 },
    { 0, 0 },
    { 0x4e717972, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a517976, 55 },
    { 0, 0 },
    { 0x99917978, 90 },
    { 0, 0 },
    { 0x3271797a, 23 },
    { 0xa751797a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd281797f, 97 },
    { 0xe131797f, 98 },
    { 0x6af17981, 43 },
    { 0, 0 },
    { 0x9e717983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1517987, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c1798c, 33 },
    { 0xda21798d, 46 },
    { 0xcc61798d, 95 },
    { 0x6e01798f, 23 },
    { 0x57717990, 90 },
    { 0x6301798f, 33 },
    { 0xdb41798d, 67 },
    { 0x53217991, 61 },
    { 0x3fa17994, 24 },
    { 0xb7e1798c, 93 },
    { 0xffb17996, 33 },
    { 0xe8b17997, 95 },
    { 0xe2017998, 46 },
    { 0x67617999, 90 },
    { 0xf6417997, 90 },
    { 0, 0 },
    { 0x3b01799c, 75 },
    { 0xcb91799d, 76 },
    { 0, 0 },
    { 0xfe81799f, 90 },
    { 0, 0 },
    { 0xe38179a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190179aa, 23 },
    { 0, 0 },
    { 0x8c3179ac, 43 },
    { 0x41a179ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92179b2, 81 },
    { 0x79d179b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b179b8, 33 },
    { 0xf97179b9, 88 },
    { 0x939179ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a179bd, 61 },
    { 0, 0 },
    { 0x582179bf, 95 },
    { 0xa77179c0, 90 },
    { 0x7c2179c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e179c7, 61 },
    { 0x5c0179c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20179cd, 76 },
    { 0x3d6179cd, 90 },
    { 0, 0 },
    { 0x1b1179d0, 90 },
    { 0, 0 },
    { 0xae179d2, 91 },
    { 0, 0 },
    { 0x119179d4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2179d8, 61 },
    { 0, 0 },
    { 0x3cc179da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7179e1, 98 },
    { 0, 0 },
    { 0x310179e3, 61 },
    { 0x5ad179e3, 33 },
    { 0x4b2179e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e179eb, 61 },
    { 0x226179ec, 98 },
    { 0xad2179ed, 33 },
    { 0x87a179ee, 90 },
    { 0xeca179ec, 46 },
    { 0x559179ef, 90 },
    { 0x9a179f1, 95 },
    { 0, 0 },
    { 0x442179f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7179fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce179fe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf17a02, 87 },
    { 0xd0617a03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c17a06, 34 },
    { 0x91917a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31917a0e, 61 },
    { 0x82817a0e, 87 },
    { 0x4317a10, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40517a1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8417a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d17a27, 93 },
    { 0xfd717a28, 61 },
    { 0, 0 },
    { 0x3de17a2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb17a2d, 68 },
    { 0, 0 },
    { 0xca317a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42317a3e, 98 },
    { 0, 0 },
    { 0x2617a40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d317a43, 84 },
    { 0, 0 },
    { 0x6dc17a45, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba17a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70917a4d, 46 },
    { 0, 0 },
    { 0x27117a4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8317a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d17a58, 95 },
    { 0x52317a58, 43 },
    { 0xc0417a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c17a5e, 83 },
    { 0x73d17a5f, 100 },
    { 0, 0 },
    { 0x4dc17a61, 33 },
    { 0x19917a62, 77 },
    { 0, 0 },
    { 0xed417a64, 46 },
    { 0x3ea17a65, 99 },
    { 0, 0 },
    { 0x82f17a67, 55 },
    { 0, 0 },
    { 0x9d717a69, 84 },
    { 0xb5017a6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c17a6d, 23 },
    { 0, 0 },
    { 0x73c17a6f, 76 },
    { 0x9e917a6f, 33 },
    { 0xd7f17a70, 95 },
    { 0, 0 },
    { 0x40317a73, 61 },
    { 0, 0 },
    { 0xb2617a75, 78 },
    { 0xe6717a76, 88 },
    { 0x4f717a76, 33 },
    { 0, 0 },
    { 0x1d117a79, 85 },
    { 0x87c17a7a, 33 },
    { 0x10117a7b, 55 },
    { 0xee717a7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea17a80, 95 },
    { 0x1da17a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc17a84, 46 },
    { 0xa5017a85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d17a8d, 23 },
    { 0x24517a8e, 67 },
    { 0xdc17a8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf17a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f17a98, 90 },
    { 0x69c17a99, 33 },
    { 0xf4c17a9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8517a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed117aa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8717aa6, 85 },
    { 0, 0 },
    { 0x37817aa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73717aab, 88 },
    { 0x56517aac, 55 },
    { 0, 0 },
    { 0xbe117aae, 33 },
    { 0x1d117aaf, 93 },
    { 0xa2017aae, 43 },
    { 0x37417ab0, 66 },
    { 0x32917ab2, 61 },
    { 0, 0 },
    { 0x2e117ab4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb717aba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3717abd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd17ac3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff017ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b17ace, 33 },
    { 0x6fe17ace, 100 },
    { 0x1e117ad0, 33 },
    { 0x71f17ad1, 90 },
    { 0, 0 },
    { 0x7b517ad3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x30117ad6, 55 },
    { 0, 0 },
    { 0x9b17ad8, 34 },
    { 0xa717ad9, 90 },
    { 0xe7d17ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda17ae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40817ae5, 34 },
    { 0x6a117ae5, 99 },
    { 0x6ec17ae6, 23 },
    { 0, 0 },
    { 0xcfb17ae9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e517aec, 55 },
    { 0x40e17aec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87717af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf617af7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa17afc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20917aff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f17b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f17b05, 98 },
    { 0, 0 },
    { 0x49a17b07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30017b0b, 95 },
    { 0x14217b0c, 46 },
    { 0xcfc17b0c, 77 },
    { 0x5b917b0d, 91 },
    { 0xafa17b0f, 76 },
    { 0xa017b0d, 76 },
    { 0x11617b0d, 23 },
    { 0xcc417b12, 93 },
    { 0xb1117b13, 90 },
    { 0x40217b12, 90 },
    { 0x80417b12, 82 },
    { 0, 0 },
    { 0x37017b17, 98 },
    { 0x14f17b17, 76 },
    { 0x23317b19, 17 },
    { 0x3317b1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f17b1e, 100 },
    { 0, 0 },
    { 0x71117b20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55217b25, 90 },
    { 0x62017b26, 93 },
    { 0xea117b26, 61 },
    { 0, 0 },
    { 0xd4417b29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf717b2c, 95 },
    { 0xa017b2d, 34 },
    { 0, 0 },
    { 0xf9617b2f, 95 },
    { 0xaa17b30, 33 },
    { 0x59b17b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80917b39, 90 },
    { 0x4fa17b3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b817b42, 33 },
    { 0, 0 },
    { 0xe3817b44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8917b48, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92417b51, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd17b5c, 34 },
    { 0x5f17b5d, 33 },
    { 0xb5417b5e, 34 },
    { 0xbb917b5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94017b6a, 90 },
    { 0xb5117b6a, 75 },
    { 0x2ad17b6c, 61 },
    { 0x38017b6a, 69 },
    { 0, 0 },
    { 0x82017b6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70117b72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d217b75, 85 },
    { 0x7da17b76, 93 },
    { 0x51917b77, 88 },
    { 0xd6917b75, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b17b83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e17b87, 33 },
    { 0x24d17b88, 65 },
    { 0, 0 },
    { 0xfc417b8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d417b8f, 90 },
    { 0x8d717b8f, 90 },
    { 0xb9417b8f, 23 },
    { 0x97417b8f, 56 },
    { 0x40717b8f, 23 },
    { 0, 0 },
    { 0x20217b95, 69 },
    { 0, 0 },
    { 0x27117b97, 76 },
    { 0xfd017b98, 61 },
    { 0, 0 },
    { 0x1ef17b9a, 85 },
    { 0x1cc17b9a, 62 },
    { 0x22f17b9a, 90 },
    { 0, 0 },
    { 0x5b817b9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f317ba1, 100 },
    { 0x87d17ba1, 76 },
    { 0, 0 },
    { 0x47b17ba4, 76 },
    { 0, 0 },
    { 0x24317ba6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded17baa, 95 },
    { 0x3ab17bab, 93 },
    { 0xdca17bac, 81 },
    { 0xfda17bab, 69 },
    { 0x24c17bad, 34 },
    { 0xf6617baa, 61 },
    { 0, 0 },
    { 0x51117bb1, 77 },
    { 0x98017bb1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7917bb6, 43 },
    { 0x9bf17bb7, 33 },
    { 0, 0 },
    { 0x39117bb9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19317bc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c17bc3, 61 },
    { 0, 0 },
    { 0xfc517bc5, 52 },
    { 0x8c17bc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47417bc9, 90 },
    { 0x59917bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee17bcd, 95 },
    { 0x4a817bcd, 76 },
    { 0x2bc17bcf, 90 },
    { 0x4b917bd0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc17bdb, 90 },
    { 0x19417bdb, 98 },
    { 0xf5d17bdc, 33 },
    { 0, 0 },
    { 0x8ff17bdf, 34 },
    { 0xb1517be0, 67 },
    { 0x75a17be0, 40 },
    { 0xcb117be2, 34 },
    { 0xe817be3, 55 },
    { 0, 0 },
    { 0xdff17be5, 33 },
    { 0, 0 },
    { 0xae917be7, 90 },
    { 0xb5617be8, 23 },
    { 0x2a017be9, 33 },
    { 0x8cf17bea, 43 },
    { 0x4b917beb, 52 },
    { 0xea17bea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b17bf1, 95 },
    { 0x1d017bf2, 33 },
    { 0, 0 },
    { 0x27917bf4, 66 },
    { 0x22317bf5, 34 },
    { 0xf1e17bf6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64417bfc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82617c0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a717c0e, 90 },
    { 0, 0 },
    { 0x1f317c10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51417c13, 83 },
    { 0, 0 },
    { 0xc5c17c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e117c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17c1c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab17c21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x41217c24, 61 },
    { 0x48117c25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61117c2b, 61 },
    { 0x3e917c2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45217c31, 95 },
    { 0xb2717c32, 76 },
    { 0x4d317c32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea17c37, 43 },
    { 0xf9817c37, 33 },
    { 0x77717c38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f17c40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df17c48, 55 },
    { 0, 0 },
    { 0x82317c4a, 61 },
    { 0, 0 },
    { 0x3e217c4c, 55 },
    { 0, 0 },
    { 0x12517c4e, 64 },
    { 0xdeb17c4e, 69 },
    { 0xc9917c4f, 100 },
    { 0xa417c51, 76 },
    { 0x88317c51, 100 },
    { 0x8f217c4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e17c59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea17c5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7117c60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51317c64, 17 },
    { 0x80c17c65, 90 },
    { 0x7ba17c66, 61 },
    { 0, 0 },
    { 0x21317c68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c17c6c, 88 },
    { 0x1cf17c6c, 93 },
    { 0xc0317c6e, 34 },
    { 0, 0 },
    { 0xbf017c70, 34 },
    { 0x19717c71, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b617c79, 34 },
    { 0, 0 },
    { 0x8d317c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9217c7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2617c88, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b17c8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77217c93, 43 },
    { 0xbcf17c94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd17c99, 33 },
    { 0, 0 },
    { 0x1ed17c9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc17ca2, 34 },
    { 0xe1a17ca2, 76 },
    { 0x68117ca4, 90 },
    { 0x89417ca3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a217ca8, 90 },
    { 0xf9017ca8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c717cac, 23 },
    { 0x12b17cad, 98 },
    { 0xbe817cae, 76 },
    { 0, 0 },
    { 0xbb817cb0, 76 },
    { 0xeee17cb1, 61 },
    { 0xd2317cb0, 33 },
    { 0xdcf17cb3, 90 },
    { 0, 0 },
    { 0x58917cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb17cbb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af17cbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8317cc3, 61 },
    { 0, 0 },
    { 0x85c17cc5, 90 },
    { 0xed517cc5, 95 },
    { 0, 0 },
    { 0x6d517cc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63217ccc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1017ccf, 66 },
    { 0x71617cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d417cd6, 52 },
    { 0x5e117cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1417cda, 98 },
    { 0, 0 },
    { 0x7e917cdc, 61 },
    { 0x99417cdc, 76 },
    { 0, 0 },
    { 0xde317cdf, 23 },
    { 0xeda17ce0, 77 },
    { 0xd4817ce1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f917ce5, 33 },
    { 0, 0 },
    { 0xcc17ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b17ceb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde017cf1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d17cf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa17cfa, 33 },
    { 0, 0 },
    { 0xfc917cfc, 76 },
    { 0x48b17cfd, 95 },
    { 0x27717cfd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24417d03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c117d0e, 80 },
    { 0, 0 },
    { 0x7d817d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df17d15, 55 },
    { 0, 0 },
    { 0xf5117d17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b17d1c, 76 },
    { 0x87d17d1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34417d2d, 90 },
    { 0x56017d2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de17d33, 43 },
    { 0xf4717d34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0217d3c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c17d3f, 33 },
    { 0, 0 },
    { 0x5d317d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b17d48, 55 },
    { 0x41d17d48, 23 },
    { 0x2f617d48, 55 },
    { 0, 0 },
    { 0xb1817d4c, 67 },
    { 0x10817d4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37817d52, 90 },
    { 0xec117d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c17d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f17d60, 23 },
    { 0x8a817d60, 90 },
    { 0x44b17d60, 90 },
    { 0x5a917d60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93617d66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b17d69, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e17d6f, 76 },
    { 0, 0 },
    { 0xe5d17d71, 33 },
    { 0, 0 },
    { 0x77e17d73, 91 },
    { 0x7b717d74, 61 },
    { 0x9d717d74, 33 },
    { 0xc9917d76, 90 },
    { 0xc0c17d77, 76 },
    { 0x31917d77, 55 },
    { 0, 0 },
    { 0xbf17d7a, 23 },
    { 0x47717d7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0917d7f, 98 },
    { 0, 0 },
    { 0x3e917d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f217d88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb217d8b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0817d90, 55 },
    { 0x5e617d91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde517d96, 76 },
    { 0x9b817d97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e17d9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5617d9e, 55 },
    { 0, 0 },
    { 0x52617da0, 33 },
    { 0xe0e17da0, 33 },
    { 0, 0 },
    { 0x57617da3, 98 },
    { 0, 0 },
    { 0x7fd17da5, 34 },
    { 0x3ce17da6, 46 },
    { 0x33717da7, 95 },
    { 0x7f317da8, 88 },
    { 0xc0f17da6, 97 },
    { 0x9b317daa, 43 },
    { 0xca317dab, 43 },
    { 0, 0 },
    { 0xda17dad, 87 },
    { 0x6a17dae, 100 },
    { 0xffc17daf, 98 },
    { 0x51b17dae, 23 },
    { 0x9da17db0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc17db4, 98 },
    { 0, 0 },
    { 0x3bf17db6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45117db9, 95 },
    { 0xd2d17dba, 98 },
    { 0x74f17dbb, 76 },
    { 0x9d817dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb17dbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c17dc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8617dca, 55 },
    { 0xffd17dcb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d17dd2, 33 },
    { 0x2e617dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b917dda, 90 },
    { 0x67717ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe17de1, 55 },
    { 0x4617de1, 90 },
    { 0, 0 },
    { 0x65517de4, 33 },
    { 0, 0 },
    { 0x8e217de6, 98 },
    { 0x71c17de7, 23 },
    { 0x4ca17de7, 55 },
    { 0x87617de8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27417dec, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c17df3, 61 },
    { 0x91f17df3, 98 },
    { 0x38c17df4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x43017df8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83217dfb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba17dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7817e04, 90 },
    { 0, 0 },
    { 0xdb617e06, 90 },
    { 0x98017e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8717e0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5017e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95217e16, 40 },
    { 0, 0 },
    { 0x36417e18, 93 },
    { 0xd717e19, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e317e1c, 85 },
    { 0x74717e1d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae117e22, 76 },
    { 0x37817e23, 91 },
    { 0, 0 },
    { 0x8dc17e25, 94 },
    { 0x77d17e26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b17e30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99417e33, 88 },
    { 0, 0 },
    { 0x9d517e35, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2617e3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c117e45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf317e48, 91 },
    { 0xd6d17e48, 61 },
    { 0x7bf17e48, 33 },
    { 0, 0 },
    { 0xdfb17e4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca17e51, 98 },
    { 0x94217e52, 95 },
    { 0xdf317e52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa17e58, 90 },
    { 0, 0 },
    { 0x63117e5a, 90 },
    { 0xadb17e5a, 23 },
    { 0xe9017e5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85117e61, 52 },
    { 0xaea17e62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf17e65, 61 },
    { 0x58a17e66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b717e6d, 61 },
    { 0xd8c17e6e, 23 },
    { 0x7e117e6f, 61 },
    { 0x38c17e70, 43 },
    { 0, 0 },
    { 0x64a17e72, 81 },
    { 0xf817e72, 43 },
    { 0xd5517e74, 90 },
    { 0, 0 },
    { 0x55517e76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c517e79, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b17e7c, 98 },
    { 0, 0 },
    { 0x8a117e7e, 93 },
    { 0, 0 },
    { 0xc4317e80, 52 },
    { 0, 0 },
    { 0xa3017e82, 76 },
    { 0x8ae17e83, 55 },
    { 0x1e717e84, 85 },
    { 0xb2817e85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf317e8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89417e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e95, 23 },
    { 0x33517e95, 87 },
    { 0x2d717e97, 23 },
    { 0xeb517e97, 98 },
    { 0xdd617e99, 34 },
    { 0xadd17e97, 93 },
    { 0, 0 },
    { 0x2e317e9c, 90 },
    { 0x24b17e9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11717ea1, 55 },
    { 0xc6d17ea1, 36 },
    { 0, 0 },
    { 0xa7d17ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a17ea7, 84 },
    { 0x18817ea8, 90 },
    { 0xaa017ea9, 61 },
    { 0x1fa17ea9, 55 },
    { 0, 0 },
    { 0x78b17eac, 23 },
    { 0, 0 },
    { 0xaef17eae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc17eb2, 23 },
    { 0, 0 },
    { 0x89517eb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e817eb7, 40 },
    { 0x6417eb8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e117ec0, 100 },
    { 0, 0 },
    { 0xc6717ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe17ed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e17ed3, 90 },
    { 0x4bd17ed3, 98 },
    { 0xaf917ed5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c17eda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e617edd, 98 },
    { 0x52317ede, 23 },
    { 0x9bc17ede, 61 },
    { 0x3017ee0, 61 },
    { 0xd9c17ee1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae617ee5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xda717ee8, 78 },
    { 0, 0 },
    { 0xba717eea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec17ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e17ef4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a17ef8, 61 },
    { 0xd1717ef9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae17efd, 100 },
    { 0xb7417efe, 90 },
    { 0, 0 },
    { 0xff117f00, 98 },
    { 0x3b717f01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85117f08, 77 },
    { 0xc5017f09, 76 },
    { 0, 0 },
    { 0xaed17f0b, 100 },
    { 0, 0 },
    { 0x94c17f0d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb617f10, 23 },
    { 0, 0 },
    { 0x6d117f12, 95 },
    { 0xb3817f12, 34 },
    { 0x71317f12, 61 },
    { 0xfaa17f15, 85 },
    { 0x9fd17f16, 23 },
    { 0x68617f14, 99 },
    { 0, 0 },
    { 0xfc417f19, 95 },
    { 0x22317f1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54517f20, 67 },
    { 0x81217f20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c817f29, 23 },
    { 0x87917f2a, 76 },
    { 0x93217f2b, 90 },
    { 0xf4617f2a, 52 },
    { 0x3f817f2d, 77 },
    { 0xb117f2d, 84 },
    { 0xc5317f2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c017f35, 95 },
    { 0, 0 },
    { 0xf7817f37, 78 },
    { 0x58217f38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd717f3b, 100 },
    { 0xd1a17f3b, 61 },
    { 0x14c17f3d, 34 },
    { 0, 0 },
    { 0xbb117f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41117f4c, 33 },
    { 0, 0 },
    { 0x1617f4e, 98 },
    { 0x9fd17f4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d17f54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e17f57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12217f5e, 90 },
    { 0x5cc17f5f, 98 },
    { 0, 0 },
    { 0x96817f61, 66 },
    { 0, 0 },
    { 0x21b17f63, 23 },
    { 0, 0 },
    { 0x6cd17f65, 56 },
    { 0, 0 },
    { 0x93d17f67, 55 },
    { 0xb217f67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70517f6c, 46 },
    { 0x1d217f6c, 55 },
    { 0x56c17f6c, 40 },
    { 0xc9217f6d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a017f73, 61 },
    { 0xeef17f74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62017f7a, 81 },
    { 0x73617f7b, 85 },
    { 0x5cd17f7b, 61 },
    { 0xc9717f7d, 61 },
    { 0x5b517f7d, 61 },
    { 0, 0 },
    { 0x9017f80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9817f83, 98 },
    { 0, 0 },
    { 0x65f17f85, 34 },
    { 0xeaa17f86, 91 },
    { 0x71f17f87, 95 },
    { 0x44617f88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd17f8b, 90 },
    { 0x71d17f8b, 33 },
    { 0xa5317f8c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa17f91, 84 },
    { 0x51817f92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd17f95, 33 },
    { 0x2d917f95, 88 },
    { 0x40c17f97, 23 },
    { 0xd2317f97, 76 },
    { 0x1b17f98, 55 },
    { 0xcde17f9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e17fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66017fa8, 33 },
    { 0, 0 },
    { 0x2d417faa, 33 },
    { 0x7d617fab, 95 },
    { 0xd5d17fac, 67 },
    { 0xf7617fad, 61 },
    { 0x5e817faa, 90 },
    { 0, 0 },
    { 0xf9717fb0, 80 },
    { 0, 0 },
    { 0x65517fb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f517fb7, 61 },
    { 0x92317fb8, 98 },
    { 0xe2317fb8, 90 },
    { 0, 0 },
    { 0x31117fbb, 55 },
    { 0x80217fbc, 76 },
    { 0, 0 },
    { 0x71f17fbe, 33 },
    { 0xa3f17fbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5817fc2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8617fc6, 98 },
    { 0, 0 },
    { 0x53717fc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a717fcb, 55 },
    { 0, 0 },
    { 0xbd17fcd, 46 },
    { 0xf6c17fce, 55 },
    { 0, 0 },
    { 0x58817fd0, 90 },
    { 0x82a17fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe417fd6, 46 },
    { 0xe9617fd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc17fdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57917fde, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x19717fe1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee917fe7, 66 },
    { 0, 0 },
    { 0xed317fe9, 34 },
    { 0x57217fea, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d17ff4, 98 },
    { 0x34d17ff5, 90 },
    { 0x57617ff6, 33 },
    { 0x6ee17ff7, 77 },
    { 0x99017ff8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb417ffd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c18000, 55 },
    { 0x2a118001, 61 },
    { 0x7ca18002, 96 },
    { 0x48818003, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6518006, 46 },
    { 0, 0 },
    { 0x6e218008, 88 },
    { 0, 0 },
    { 0xb281800a, 78 },
    { 0, 0 },
    { 0xe61800c, 33 },
    { 0x3991800d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45318012, 90 },
    { 0x2a218013, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb18017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc1801d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f518021, 33 },
    { 0, 0 },
    { 0xf5518023, 93 },
    { 0, 0 },
    { 0x7bf18025, 77 },
    { 0x1b018025, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d21802b, 33 },
    { 0x9a31802c, 48 },
    { 0, 0 },
    { 0x4f61802e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e918034, 90 },
    { 0x85d18035, 33 },
    { 0x62c18034, 90 },
    { 0, 0 },
    { 0xf7018038, 99 },
    { 0x3dc18039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8951803d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb018042, 90 },
    { 0x35418043, 33 },
    { 0, 0 },
    { 0x44318045, 61 },
    { 0, 0 },
    { 0xcbb18047, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e41804d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5018056, 95 },
    { 0, 0 },
    { 0x3c018058, 78 },
    { 0x6618058, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8018060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74518066, 46 },
    { 0, 0 },
    { 0x19818068, 23 },
    { 0x6618069, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc711806d, 78 },
    { 0x6451806e, 67 },
    { 0, 0 },
    { 0xcf118070, 34 },
    { 0, 0 },
    { 0x11718072, 90 },
    { 0xa0f18072, 38 },
    { 0x3a218073, 93 },
    { 0, 0 },
    { 0xccd18076, 66 },
    { 0xf9018076, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc161807a, 88 },
    { 0xbff1807a, 33 },
    { 0x9a81807b, 61 },
    { 0x8161807d, 99 },
    { 0xcd21807b, 95 },
    { 0, 0 },
    { 0xadb18080, 33 },
    { 0x36118080, 90 },
    { 0xdf918082, 95 },
    { 0x99518082, 85 },
    { 0x59518084, 76 },
    { 0x12c18082, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a1808d, 93 },
    { 0, 0 },
    { 0xeb61808f, 33 },
    { 0x3f018090, 33 },
    { 0xe418091, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a18094, 93 },
    { 0, 0 },
    { 0x65b18096, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab1809a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a1809d, 33 },
    { 0xad31809e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e180a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd180ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527180b6, 87 },
    { 0xbab180b7, 67 },
    { 0x313180b8, 76 },
    { 0, 0 },
    { 0x912180ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b180bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d180c2, 95 },
    { 0xc51180c3, 87 },
    { 0xb71180c4, 23 },
    { 0, 0 },
    { 0xb8a180c6, 95 },
    { 0x1ec180c6, 17 },
    { 0, 0 },
    { 0xcef180c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x671180cc, 90 },
    { 0, 0 },
    { 0x552180ce, 33 },
    { 0x98f180cf, 90 },
    { 0, 0 },
    { 0x88d180d1, 43 },
    { 0, 0 },
    { 0xacf180d3, 90 },
    { 0, 0 },
    { 0xd49180d5, 98 },
    { 0x3f3180d5, 33 },
    { 0x6ac180d7, 17 },
    { 0xcb5180d6, 23 },
    { 0x6eb180d6, 61 },
    { 0xf3f180d5, 90 },
    { 0x6a180db, 23 },
    { 0xc77180dc, 95 },
    { 0, 0 },
    { 0x691180de, 61 },
    { 0x2c1180de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826180e6, 69 },
    { 0, 0 },
    { 0xc4f180e8, 95 },
    { 0x52e180e9, 43 },
    { 0xf7c180ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3180ee, 87 },
    { 0x57180ef, 98 },
    { 0, 0 },
    { 0x449180f1, 23 },
    { 0xbb5180f1, 23 },
    { 0, 0 },
    { 0x32e180f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3180f8, 61 },
    { 0x569180f9, 93 },
    { 0x5d9180f9, 90 },
    { 0x351180fb, 37 },
    { 0xb22180fc, 61 },
    { 0xaa2180fb, 90 },
    { 0, 0 },
    { 0xf78180ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e18107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb1810a, 76 },
    { 0x96b1810b, 90 },
    { 0x9ba1810c, 76 },
    { 0x1321810c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d318111, 55 },
    { 0, 0 },
    { 0x44318113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af18116, 95 },
    { 0, 0 },
    { 0x1e318118, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc501811e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32818122, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x32518125, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab1812a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4001812e, 61 },
    { 0x9921812f, 98 },
    { 0x9f01812f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a618134, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3818137, 43 },
    { 0x65818138, 23 },
    { 0x33618139, 61 },
    { 0xb9d18139, 95 },
    { 0xe0918139, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b518143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b18147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61814d, 65 },
    { 0x4531814e, 66 },
    { 0xef51814f, 93 },
    { 0xbbe18150, 98 },
    { 0xd751814f, 69 },
    { 0x60b1814e, 87 },
    { 0x721814f, 100 },
    { 0xf1f18154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1815b, 34 },
    { 0, 0 },
    { 0x4e81815d, 80 },
    { 0, 0 },
    { 0xab01815f, 33 },
    { 0xda018160, 34 },
    { 0x2fa18161, 91 },
    { 0, 0 },
    { 0x9ee18163, 99 },
    { 0xe418164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb18167, 33 },
    { 0, 0 },
    { 0x4ac18169, 23 },
    { 0, 0 },
    { 0xe4a1816b, 90 },
    { 0, 0 },
    { 0x5801816d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c918171, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f18175, 95 },
    { 0x56218176, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5718179, 55 },
    { 0, 0 },
    { 0x30e1817b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa801817e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac18181, 59 },
    { 0, 0 },
    { 0x29e18183, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9318187, 61 },
    { 0, 0 },
    { 0x51e18189, 98 },
    { 0x20b1818a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x7481818d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd18190, 84 },
    { 0, 0 },
    { 0xf2d18192, 90 },
    { 0xc8d18193, 55 },
    { 0x51318192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d18197, 81 },
    { 0xa5d18197, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1819c, 43 },
    { 0x9431819d, 78 },
    { 0x9e41819d, 87 },
    { 0x41c1819f, 56 },
    { 0, 0 },
    { 0x626181a1, 23 },
    { 0x276181a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648181a6, 100 },
    { 0x549181a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a181aa, 23 },
    { 0, 0 },
    { 0xd68181ac, 90 },
    { 0x822181ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb181b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76181b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe181b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61181be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e181c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f181ca, 95 },
    { 0, 0 },
    { 0xce8181cc, 61 },
    { 0, 0 },
    { 0xb35181ce, 90 },
    { 0, 0 },
    { 0x230181d0, 95 },
    { 0xd02181d1, 67 },
    { 0, 0 },
    { 0xc87181d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b181d6, 23 },
    { 0xd9a181d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18181db, 93 },
    { 0, 0 },
    { 0xaad181dd, 90 },
    { 0, 0 },
    { 0xcf1181df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x273181e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2181e5, 33 },
    { 0, 0 },
    { 0x7d5181e7, 34 },
    { 0x917181e8, 61 },
    { 0, 0 },
    { 0x668181ea, 33 },
    { 0xf41181eb, 65 },
    { 0xce4181ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687181f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4181f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x181f8, 55 },
    { 0xb72181f8, 67 },
    { 0x935181fa, 93 },
    { 0, 0 },
    { 0x2a9181fc, 98 },
    { 0x694181fc, 95 },
    { 0x1e2181fe, 61 },
    { 0xdf6181ff, 97 },
    { 0xb70181fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e18204, 100 },
    { 0, 0 },
    { 0xb7518206, 33 },
    { 0x8db18207, 33 },
    { 0x6118208, 66 },
    { 0x72618209, 90 },
    { 0, 0 },
    { 0xffb1820b, 90 },
    { 0xa931820b, 91 },
    { 0xfb21820b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2118215, 17 },
    { 0x6a118216, 90 },
    { 0x22618216, 33 },
    { 0xf3118216, 77 },
    { 0x23a18219, 8 },
    { 0x13018215, 23 },
    { 0x7b518219, 98 },
    { 0xd9e1821c, 40 },
    { 0x19918218, 98 },
    { 0, 0 },
    { 0x60f1821f, 43 },
    { 0x1e218220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91918223, 68 },
    { 0, 0 },
    { 0x63018225, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c118228, 99 },
    { 0x518228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88618231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d318234, 90 },
    { 0x7a318235, 88 },
    { 0xb2718235, 43 },
    { 0, 0 },
    { 0x89518238, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d1823e, 90 },
    { 0, 0 },
    { 0x4e18240, 59 },
    { 0, 0 },
    { 0x88b18242, 90 },
    { 0, 0 },
    { 0x93618244, 77 },
    { 0, 0 },
    { 0x1c818246, 90 },
    { 0x50b18247, 38 },
    { 0x4618248, 11 },
    { 0x21a18248, 34 },
    { 0x28218246, 61 },
    { 0x6521824b, 95 },
    { 0x92f1824b, 61 },
    { 0x9311824b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df18250, 69 },
    { 0xe918251, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f18254, 90 },
    { 0, 0 },
    { 0x80518256, 78 },
    { 0, 0 },
    { 0xf0518258, 98 },
    { 0xdac18258, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd1825e, 95 },
    { 0x2401825f, 88 },
    { 0x501825f, 23 },
    { 0xcd01825e, 84 },
    { 0xe3c18262, 68 },
    { 0x35f18260, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e18268, 95 },
    { 0, 0 },
    { 0x2eb1826a, 99 },
    { 0, 0 },
    { 0x1981826c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab18270, 43 },
    { 0x29618271, 90 },
    { 0x61918271, 55 },
    { 0x18c18273, 43 },
    { 0, 0 },
    { 0x9bb18275, 95 },
    { 0x8fe18276, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x85918279, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a1827d, 33 },
    { 0x74a1827d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b18284, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x75518287, 76 },
    { 0x75f18288, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe231828f, 81 },
    { 0x95d1828f, 99 },
    { 0, 0 },
    { 0xece18292, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc618295, 23 },
    { 0x69c18296, 33 },
    { 0x95a18296, 23 },
    { 0xd2118298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba01829c, 98 },
    { 0xacb1829d, 23 },
    { 0x5b31829c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3182aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca182b1, 66 },
    { 0x2bc182b2, 61 },
    { 0x260182b2, 61 },
    { 0x23b182b2, 61 },
    { 0x8e4182b5, 90 },
    { 0x7c5182b2, 52 },
    { 0x828182b7, 33 },
    { 0x90b182b3, 55 },
    { 0x18182b2, 95 },
    { 0x3fb182b1, 40 },
    { 0xd1f182b5, 40 },
    { 0xdf7182bc, 74 },
    { 0xb99182b8, 90 },
    { 0x869182be, 33 },
    { 0xfee182b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f182c7, 95 },
    { 0x4da182c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5182d0, 98 },
    { 0x105182d1, 93 },
    { 0x7b2182d2, 76 },
    { 0xaf8182d0, 61 },
    { 0xb3a182d4, 33 },
    { 0xa36182d4, 91 },
    { 0xc1182d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d182dc, 76 },
    { 0x6a182dd, 61 },
    { 0x79b182dc, 38 },
    { 0, 0 },
    { 0x53f182e0, 95 },
    { 0, 0 },
    { 0xdf7182e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x438182e5, 55 },
    { 0x3b1182e6, 68 },
    { 0x292182e6, 66 },
    { 0xa07182e8, 93 },
    { 0x2c7182e9, 33 },
    { 0x90e182e9, 66 },
    { 0x6f4182eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x394182ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285182f2, 66 },
    { 0xb63182f2, 33 },
    { 0, 0 },
    { 0x774182f5, 91 },
    { 0xc81182f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x569182f9, 61 },
    { 0xb02182fa, 74 },
    { 0x7b0182fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d18303, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e18307, 95 },
    { 0x7af18308, 95 },
    { 0x67718308, 81 },
    { 0xa7b1830a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d31830f, 87 },
    { 0x3c718310, 52 },
    { 0, 0 },
    { 0x64918312, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30118315, 78 },
    { 0x73418315, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d71831b, 61 },
    { 0, 0 },
    { 0xa6f1831d, 33 },
    { 0x5db1831e, 23 },
    { 0xc6a1831d, 34 },
    { 0x3bb18320, 33 },
    { 0x63d1831d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e318325, 34 },
    { 0xe8218326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e318329, 90 },
    { 0, 0 },
    { 0x3671832b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b18335, 55 },
    { 0, 0 },
    { 0x19b18337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51833b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d81833e, 90 },
    { 0xb4f1833f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c18344, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5618349, 98 },
    { 0x92518349, 84 },
    { 0xc9e1834b, 33 },
    { 0xf3e1834c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb18353, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b18356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9611835e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69718365, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b18377, 98 },
    { 0xac118378, 95 },
    { 0x7a018377, 75 },
    { 0xab61837a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371837e, 36 },
    { 0x3651837f, 95 },
    { 0xdd41837f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5418385, 87 },
    { 0xb918386, 87 },
    { 0xa5a18386, 65 },
    { 0xd9b18387, 98 },
    { 0x1f018385, 99 },
    { 0, 0 },
    { 0xd4f1838b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd321838e, 33 },
    { 0x47f1838f, 33 },
    { 0x13618390, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52218395, 77 },
    { 0xcae18396, 95 },
    { 0xb4a18397, 61 },
    { 0, 0 },
    { 0x91718399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251839c, 43 },
    { 0, 0 },
    { 0x76d1839e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959183a5, 98 },
    { 0xdfa183a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5183a9, 98 },
    { 0x6c8183aa, 76 },
    { 0, 0 },
    { 0xc38183ac, 55 },
    { 0x88b183ad, 90 },
    { 0xe91183ae, 90 },
    { 0xb65183af, 98 },
    { 0, 0 },
    { 0xad2183b1, 99 },
    { 0, 0 },
    { 0x351183b3, 91 },
    { 0x918183b3, 34 },
    { 0x3a5183b5, 23 },
    { 0x32183b6, 98 },
    { 0, 0 },
    { 0xe4e183b8, 92 },
    { 0xe2183b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62183bc, 95 },
    { 0xa07183bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601183c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xac183c4, 61 },
    { 0xc98183c4, 96 },
    { 0x775183c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02183c9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e183ce, 23 },
    { 0x8c1183cf, 83 },
    { 0x7d7183d0, 95 },
    { 0x824183d1, 35 },
    { 0xd61183d1, 98 },
    { 0x42e183d1, 34 },
    { 0x4da183d4, 93 },
    { 0xec4183d5, 95 },
    { 0x87d183d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29183da, 90 },
    { 0, 0 },
    { 0x69c183dc, 23 },
    { 0, 0 },
    { 0x5ef183de, 61 },
    { 0x7e6183df, 99 },
    { 0x928183e0, 77 },
    { 0x23c183e0, 43 },
    { 0x8a0183e2, 90 },
    { 0xea183e1, 67 },
    { 0x30f183e4, 90 },
    { 0xfa5183e4, 92 },
    { 0, 0 },
    { 0x73183e7, 93 },
    { 0x2b7183e8, 77 },
    { 0x860183e7, 98 },
    { 0, 0 },
    { 0xa5d183eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7183f0, 80 },
    { 0x546183f0, 100 },
    { 0xc6f183f0, 98 },
    { 0, 0 },
    { 0x32e183f4, 95 },
    { 0x937183f5, 87 },
    { 0x845183f5, 90 },
    { 0x916183f7, 33 },
    { 0x1b4183f5, 98 },
    { 0xf74183f9, 90 },
    { 0xe32183fa, 98 },
    { 0x893183fb, 23 },
    { 0x48d183fb, 90 },
    { 0xd63183fd, 76 },
    { 0x136183fd, 95 },
    { 0x909183ff, 76 },
    { 0xa1d18400, 76 },
    { 0xf73183ff, 33 },
    { 0x619183fb, 33 },
    { 0xbfa18403, 91 },
    { 0xf2618404, 68 },
    { 0, 0 },
    { 0xa18406, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf18409, 95 },
    { 0xbb81840a, 23 },
    { 0x1501840a, 98 },
    { 0xa1e1840a, 82 },
    { 0xab11840c, 43 },
    { 0x2dd1840e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52318412, 55 },
    { 0x10718413, 90 },
    { 0xfaf18413, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90918418, 67 },
    { 0, 0 },
    { 0x63f1841a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161841f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af18425, 34 },
    { 0, 0 },
    { 0x68418427, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021842e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39918432, 88 },
    { 0x6d518433, 67 },
    { 0xd6918434, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65918437, 61 },
    { 0, 0 },
    { 0xe6c18439, 90 },
    { 0x39f1843a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd1843e, 36 },
    { 0xdb91843e, 55 },
    { 0, 0 },
    { 0xbca18441, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b18445, 33 },
    { 0x7bb18446, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a18449, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1844d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b18452, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97818456, 33 },
    { 0xe0618457, 90 },
    { 0xcfe18457, 23 },
    { 0xd0618457, 33 },
    { 0x5cb1845a, 43 },
    { 0xe3b18456, 43 },
    { 0, 0 },
    { 0x4f41845d, 33 },
    { 0, 0 },
    { 0x9fe1845f, 90 },
    { 0x70718460, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff18464, 61 },
    { 0x5fb18464, 90 },
    { 0x57a18464, 33 },
    { 0, 0 },
    { 0xc4618468, 33 },
    { 0x66d18469, 23 },
    { 0xfc118469, 33 },
    { 0, 0 },
    { 0xbf41846c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff18470, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe718474, 93 },
    { 0xd9f18474, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb311847a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e18482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64218486, 91 },
    { 0x71618487, 61 },
    { 0x6de18486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc1848f, 33 },
    { 0x45118490, 61 },
    { 0, 0 },
    { 0xa8218492, 95 },
    { 0xa6218493, 23 },
    { 0x80118494, 95 },
    { 0, 0 },
    { 0x1c818496, 33 },
    { 0, 0 },
    { 0x97b18498, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e1849c, 76 },
    { 0x6881849c, 100 },
    { 0, 0 },
    { 0x1301849f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b184a2, 43 },
    { 0x22e184a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab184a7, 87 },
    { 0x6b2184a8, 33 },
    { 0x668184a9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x394184ac, 61 },
    { 0x153184ad, 76 },
    { 0x6aa184ac, 98 },
    { 0x48b184af, 61 },
    { 0x7ad184af, 33 },
    { 0xc6f184b1, 93 },
    { 0x565184b2, 61 },
    { 0, 0 },
    { 0x29a184b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2184b7, 33 },
    { 0xed2184b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1184c0, 91 },
    { 0x7a6184c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753184c5, 87 },
    { 0x480184c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db184c9, 85 },
    { 0, 0 },
    { 0x2dc184cb, 83 },
    { 0, 0 },
    { 0xf52184cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655184d2, 61 },
    { 0, 0 },
    { 0x4a4184d4, 33 },
    { 0x555184d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d184db, 93 },
    { 0x873184db, 87 },
    { 0x8cf184dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5184e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85184ea, 76 },
    { 0xece184eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6184f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26184fc, 76 },
    { 0x237184fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e18502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d18508, 90 },
    { 0, 0 },
    { 0xea01850a, 61 },
    { 0, 0 },
    { 0xcc91850c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc081850f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e18514, 65 },
    { 0x8c518515, 95 },
    { 0, 0 },
    { 0xdce18517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe51851b, 36 },
    { 0xce71851b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c18520, 98 },
    { 0xe4418521, 93 },
    { 0x2ac18522, 85 },
    { 0xf6018523, 61 },
    { 0, 0 },
    { 0x4fd18525, 100 },
    { 0, 0 },
    { 0xbb318527, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f1852c, 23 },
    { 0x9d91852c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x16918530, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c18536, 90 },
    { 0, 0 },
    { 0x89618538, 43 },
    { 0, 0 },
    { 0x9e01853a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4371853e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85518541, 81 },
    { 0xb7818542, 33 },
    { 0xca718543, 81 },
    { 0, 0 },
    { 0x7ae18545, 61 },
    { 0xd18546, 65 },
    { 0x17718545, 61 },
    { 0xac318548, 84 },
    { 0xe7818549, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1751854d, 33 },
    { 0x1d1854e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74718551, 33 },
    { 0x37518552, 66 },
    { 0, 0 },
    { 0xde718554, 95 },
    { 0x5db18554, 76 },
    { 0, 0 },
    { 0x95c18557, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8241855a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f91855f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e18562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd018568, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae418570, 61 },
    { 0, 0 },
    { 0x35d18572, 61 },
    { 0x7de18573, 76 },
    { 0xd3118573, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18218577, 34 },
    { 0, 0 },
    { 0x9f118579, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071857c, 61 },
    { 0xe6c1857d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf18580, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e918587, 78 },
    { 0x59318587, 90 },
    { 0x3cc18587, 33 },
    { 0x48b1858a, 98 },
    { 0x2591858a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6601858e, 98 },
    { 0, 0 },
    { 0xee718590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb18594, 99 },
    { 0x8be18595, 90 },
    { 0x47018596, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd31859b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26185a0, 88 },
    { 0, 0 },
    { 0x79c185a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9185a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68185ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586185b0, 76 },
    { 0xfeb185b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff185b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03185c2, 33 },
    { 0xdd6185c3, 95 },
    { 0xafb185c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c185c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64185ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408185ce, 33 },
    { 0x2eb185cf, 90 },
    { 0, 0 },
    { 0xdae185d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e185d6, 90 },
    { 0x12c185d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4185dd, 91 },
    { 0xd61185de, 83 },
    { 0x7e185dd, 95 },
    { 0, 0 },
    { 0x444185e1, 66 },
    { 0x35a185e2, 33 },
    { 0x341185e1, 76 },
    { 0x531185e4, 23 },
    { 0xea4185e5, 34 },
    { 0x5fa185e3, 99 },
    { 0x754185e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7185eb, 52 },
    { 0x13a185ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16185ef, 98 },
    { 0x45d185f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2185f4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c185f8, 84 },
    { 0, 0 },
    { 0x9db185fa, 84 },
    { 0x9c8185fb, 94 },
    { 0xad1185fc, 61 },
    { 0, 0 },
    { 0x2ee185fe, 75 },
    { 0, 0 },
    { 0xc5718600, 90 },
    { 0x7ca18601, 33 },
    { 0x89e18601, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4818605, 98 },
    { 0, 0 },
    { 0xe7218607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc781860a, 33 },
    { 0x321860b, 98 },
    { 0, 0 },
    { 0xbb61860d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd018612, 90 },
    { 0x1d118612, 84 },
    { 0x8dd18613, 98 },
    { 0xa6018612, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d418618, 61 },
    { 0x1eb18619, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7331861d, 80 },
    { 0x9b51861e, 33 },
    { 0x4981861d, 55 },
    { 0x92b1861d, 90 },
    { 0xc9418621, 55 },
    { 0x73a1861e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1618625, 100 },
    { 0, 0 },
    { 0x12e18627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe71862c, 84 },
    { 0, 0 },
    { 0x8df1862e, 99 },
    { 0x7ba1862f, 78 },
    { 0x4418630, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f918634, 98 },
    { 0xe0418635, 90 },
    { 0x58118635, 19 },
    { 0xb7e18637, 95 },
    { 0x27318637, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d1863f, 61 },
    { 0, 0 },
    { 0xd6518641, 61 },
    { 0, 0 },
    { 0x78018643, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9318646, 61 },
    { 0xa218647, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1865a, 87 },
    { 0x8261865b, 33 },
    { 0xfa31865a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d318668, 23 },
    { 0x35f18668, 98 },
    { 0x1c51866a, 61 },
    { 0x1311866b, 90 },
    { 0x5c018668, 76 },
    { 0xa661866b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb018670, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e18673, 61 },
    { 0, 0 },
    { 0x93518675, 33 },
    { 0x7e518675, 67 },
    { 0x82318677, 33 },
    { 0x99918677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211867e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc18682, 95 },
    { 0xeae18683, 76 },
    { 0xf1718684, 76 },
    { 0x90c18684, 61 },
    { 0x72718684, 66 },
    { 0x89d18686, 61 },
    { 0, 0 },
    { 0xc5418689, 23 },
    { 0, 0 },
    { 0xdc91868b, 34 },
    { 0xe311868b, 61 },
    { 0x4e51868d, 67 },
    { 0x4dd1868e, 90 },
    { 0, 0 },
    { 0x5ed18690, 97 },
    { 0xb1418691, 61 },
    { 0x88718692, 55 },
    { 0xd3518692, 34 },
    { 0xabc18692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b1869e, 99 },
    { 0x97c1869f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff186a4, 55 },
    { 0x42186a5, 34 },
    { 0x7e6186a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a186a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119186b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b186b6, 69 },
    { 0xe9f186b6, 65 },
    { 0x791186b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b186bb, 46 },
    { 0, 0 },
    { 0x7d0186bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb186c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c186cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57186d0, 61 },
    { 0x632186d1, 95 },
    { 0xa75186d2, 34 },
    { 0, 0 },
    { 0xe23186d4, 90 },
    { 0xd86186d5, 98 },
    { 0x4f4186d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3186dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a186e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91186e4, 87 },
    { 0x21a186e4, 95 },
    { 0x66b186e6, 98 },
    { 0xb60186e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1186ed, 61 },
    { 0x8d1186ed, 98 },
    { 0, 0 },
    { 0xcca186f0, 91 },
    { 0, 0 },
    { 0x19c186f2, 84 },
    { 0xd99186f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957186f7, 88 },
    { 0, 0 },
    { 0x9ac186f9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b186fd, 61 },
    { 0, 0 },
    { 0xa26186ff, 61 },
    { 0, 0 },
    { 0xe0f18701, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43418705, 46 },
    { 0xc0c18705, 23 },
    { 0xb6018707, 93 },
    { 0x39318705, 95 },
    { 0xbe418709, 95 },
    { 0, 0 },
    { 0x7f11870b, 95 },
    { 0, 0 },
    { 0x64c1870d, 93 },
    { 0, 0 },
    { 0x6f51870f, 95 },
    { 0xbb418710, 23 },
    { 0x63518711, 46 },
    { 0x56a18710, 55 },
    { 0xf6d18713, 55 },
    { 0, 0 },
    { 0xacc18715, 88 },
    { 0, 0 },
    { 0x6c418717, 52 },
    { 0x4118718, 75 },
    { 0, 0 },
    { 0x41871a, 98 },
    { 0x1101871a, 61 },
    { 0x7271871c, 43 },
    { 0xfc91871b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff818722, 77 },
    { 0x7d118723, 33 },
    { 0xf6e18723, 61 },
    { 0x3d918723, 34 },
    { 0, 0 },
    { 0x1f318727, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce1872a, 93 },
    { 0xbb01872a, 90 },
    { 0xd891872c, 23 },
    { 0, 0 },
    { 0x8541872e, 61 },
    { 0xfbc1872f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f18733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea18739, 98 },
    { 0, 0 },
    { 0x7dd1873b, 67 },
    { 0, 0 },
    { 0xc871873d, 34 },
    { 0x2371873e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6218741, 90 },
    { 0, 0 },
    { 0x4d818743, 98 },
    { 0x1ad18744, 95 },
    { 0xa7418744, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8518748, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4461874c, 34 },
    { 0, 0 },
    { 0x3401874e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e18754, 66 },
    { 0, 0 },
    { 0x1e218756, 100 },
    { 0xeb718757, 76 },
    { 0x4e918757, 87 },
    { 0x60918759, 98 },
    { 0x2a31875a, 61 },
    { 0x1df1875b, 61 },
    { 0, 0 },
    { 0x8f1875d, 61 },
    { 0x3c21875e, 76 },
    { 0xcb81875f, 69 },
    { 0x3c41875f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16018763, 90 },
    { 0x8c618764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8591876b, 34 },
    { 0x2691876c, 33 },
    { 0, 0 },
    { 0x9191876e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88618771, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e18779, 23 },
    { 0xd061877a, 98 },
    { 0xf8f1877a, 98 },
    { 0xfac1877c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63218781, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97318787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b1878d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9918790, 99 },
    { 0xce718790, 81 },
    { 0x52618791, 91 },
    { 0x30118791, 61 },
    { 0x42518794, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d818798, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x611879b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2241879f, 23 },
    { 0x3a3187a0, 23 },
    { 0xe54187a1, 46 },
    { 0x3c2187a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee187a7, 23 },
    { 0x2b187a8, 33 },
    { 0xf98187a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4187ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8187b0, 90 },
    { 0xb2d187b1, 36 },
    { 0xdf3187b2, 76 },
    { 0xb55187b3, 98 },
    { 0xca3187b0, 61 },
    { 0xf8187b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5187b8, 46 },
    { 0x330187b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67187c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6187c9, 95 },
    { 0xd05187ca, 76 },
    { 0x54d187cb, 91 },
    { 0, 0 },
    { 0x992187cd, 95 },
    { 0, 0 },
    { 0x53e187cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7187d9, 33 },
    { 0xbca187da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e187e5, 99 },
    { 0xf54187e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4187ea, 67 },
    { 0x67c187eb, 61 },
    { 0x9cc187ea, 95 },
    { 0xe2187ed, 90 },
    { 0, 0 },
    { 0x30b187ef, 34 },
    { 0xd5d187f0, 23 },
    { 0, 0 },
    { 0xa42187f2, 61 },
    { 0xcfd187f2, 33 },
    { 0xba9187f4, 34 },
    { 0x515187f5, 90 },
    { 0x8ac187f6, 93 },
    { 0, 0 },
    { 0x230187f8, 99 },
    { 0, 0 },
    { 0x4c8187fa, 90 },
    { 0x7cd187fb, 55 },
    { 0xb66187fa, 61 },
    { 0x74f187fd, 98 },
    { 0x11d187fb, 33 },
    { 0x4ca187ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e18802, 68 },
    { 0x1f718802, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d118807, 61 },
    { 0xa2218808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e1880d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab18814, 61 },
    { 0, 0 },
    { 0xaca18816, 93 },
    { 0, 0 },
    { 0xc8118818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f1881c, 33 },
    { 0x79f1881d, 34 },
    { 0x4301881c, 81 },
    { 0x4ca1881d, 90 },
    { 0, 0 },
    { 0x14218821, 95 },
    { 0, 0 },
    { 0xa218823, 77 },
    { 0xbf318823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58518827, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4741882a, 84 },
    { 0, 0 },
    { 0xe081882c, 46 },
    { 0, 0 },
    { 0x7ec1882e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21418834, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d18838, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21883b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4631883f, 55 },
    { 0, 0 },
    { 0x54118841, 93 },
    { 0, 0 },
    { 0x42c18843, 23 },
    { 0x7d218844, 55 },
    { 0x93c18844, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b51884b, 93 },
    { 0xc751884c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2781884f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a18855, 52 },
    { 0, 0 },
    { 0xfaf18857, 69 },
    { 0x12718858, 23 },
    { 0xafa18859, 98 },
    { 0x9d918859, 85 },
    { 0xe2c18859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e618860, 76 },
    { 0xba818861, 98 },
    { 0x24018862, 33 },
    { 0x96b18863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae418867, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21886a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f618874, 98 },
    { 0x61718874, 87 },
    { 0xd0b18875, 98 },
    { 0, 0 },
    { 0x88418878, 33 },
    { 0x79118879, 93 },
    { 0, 0 },
    { 0xd531887b, 33 },
    { 0xdd31887c, 99 },
    { 0x5811887c, 33 },
    { 0xbbf1887c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d718886, 61 },
    { 0xc6f18887, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f1888f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76518892, 95 },
    { 0, 0 },
    { 0x84718894, 61 },
    { 0x1a718895, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98188a0, 23 },
    { 0xf8b188a1, 90 },
    { 0x7f8188a1, 55 },
    { 0x8d6188a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de188a6, 95 },
    { 0x809188a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1188aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833188af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401188b6, 33 },
    { 0, 0 },
    { 0x1b188b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371188c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2188cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f188d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be188d8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01188dd, 61 },
    { 0, 0 },
    { 0x3ec188df, 90 },
    { 0xcf9188e0, 67 },
    { 0, 0 },
    { 0x8cb188e2, 66 },
    { 0, 0 },
    { 0xe9188e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58188e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c188ee, 55 },
    { 0xd2e188ee, 23 },
    { 0, 0 },
    { 0x348188f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0188f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269188f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502188fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1118901, 33 },
    { 0, 0 },
    { 0xf518903, 61 },
    { 0xcb818903, 33 },
    { 0x8318905, 61 },
    { 0x90818906, 61 },
    { 0x21918907, 69 },
    { 0x7bc18907, 55 },
    { 0xfee18909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7018912, 90 },
    { 0xa3618913, 98 },
    { 0, 0 },
    { 0xc3c18915, 61 },
    { 0, 0 },
    { 0xcfa18917, 46 },
    { 0xdad18918, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f1891b, 100 },
    { 0xb811891c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1718924, 82 },
    { 0, 0 },
    { 0xefe18926, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a1892d, 76 },
    { 0x1111892e, 46 },
    { 0x7601892e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65718935, 55 },
    { 0xbd818936, 98 },
    { 0, 0 },
    { 0x13918938, 46 },
    { 0xcbc18938, 95 },
    { 0x47118939, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc451893e, 36 },
    { 0, 0 },
    { 0xfba18940, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda18944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4418948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb031894d, 33 },
    { 0, 0 },
    { 0x8211894f, 55 },
    { 0xf1a18950, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75918956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc021895d, 34 },
    { 0, 0 },
    { 0xd5c1895f, 33 },
    { 0xfe518960, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8218966, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f71896c, 93 },
    { 0xd2f1896d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6718970, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7118973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b18976, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51897c, 90 },
    { 0x7861897d, 66 },
    { 0, 0 },
    { 0xf471897f, 55 },
    { 0x9ca18980, 84 },
    { 0x2c518980, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c018984, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2518987, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd511898d, 23 },
    { 0, 0 },
    { 0xea31898f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e618992, 46 },
    { 0xe3b18992, 90 },
    { 0, 0 },
    { 0xe3a18995, 76 },
    { 0, 0 },
    { 0xafe18997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7371899d, 91 },
    { 0x5501899e, 90 },
    { 0xb3e1899d, 58 },
    { 0x3d21899d, 33 },
    { 0, 0 },
    { 0xab5189a2, 66 },
    { 0, 0 },
    { 0xd5d189a4, 95 },
    { 0, 0 },
    { 0xae0189a6, 23 },
    { 0x529189a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595189b1, 98 },
    { 0, 0 },
    { 0xb5e189b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0189b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66189bb, 99 },
    { 0, 0 },
    { 0xa05189bd, 99 },
    { 0xb36189be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c189c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166189ce, 90 },
    { 0, 0 },
    { 0xb5c189d0, 76 },
    { 0, 0 },
    { 0xae3189d2, 23 },
    { 0x421189d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6189d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3189db, 47 },
    { 0, 0 },
    { 0xb74189dd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9189e1, 76 },
    { 0, 0 },
    { 0xc25189e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7189e8, 17 },
    { 0x461189e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b189f6, 43 },
    { 0xc26189f7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b189fb, 93 },
    { 0xc6b189fc, 95 },
    { 0, 0 },
    { 0xe2f189fe, 33 },
    { 0, 0 },
    { 0x55318a00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b18a08, 17 },
    { 0x55418a09, 23 },
    { 0x40018a08, 84 },
    { 0xe2e18a0b, 80 },
    { 0x71318a0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53318a10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb918a17, 33 },
    { 0, 0 },
    { 0x96118a19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd18a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a118a23, 90 },
    { 0, 0 },
    { 0xdf118a25, 98 },
    { 0x7fa18a26, 48 },
    { 0xb8618a25, 33 },
    { 0, 0 },
    { 0xa2a18a29, 84 },
    { 0, 0 },
    { 0xb4318a2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83618a2e, 33 },
    { 0xad818a2f, 95 },
    { 0, 0 },
    { 0xd5118a31, 52 },
    { 0xcb818a31, 23 },
    { 0xb6918a33, 99 },
    { 0x69f18a32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e18a37, 34 },
    { 0x10018a37, 88 },
    { 0xceb18a37, 33 },
    { 0x7b218a3a, 98 },
    { 0xacc18a3b, 91 },
    { 0x29c18a3c, 66 },
    { 0, 0 },
    { 0xf9718a3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb718a48, 90 },
    { 0xd3418a48, 65 },
    { 0, 0 },
    { 0xed918a4b, 90 },
    { 0, 0 },
    { 0x38b18a4d, 98 },
    { 0, 0 },
    { 0x21318a4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c18a54, 61 },
    { 0xf1d18a55, 61 },
    { 0x25518a55, 90 },
    { 0, 0 },
    { 0x9c218a58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c18a5c, 61 },
    { 0, 0 },
    { 0x26918a5e, 66 },
    { 0xc3118a5f, 95 },
    { 0, 0 },
    { 0x95c18a61, 76 },
    { 0xc018a61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe218a67, 61 },
    { 0, 0 },
    { 0xf2218a69, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10118a6e, 55 },
    { 0xe918a6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75018a73, 90 },
    { 0, 0 },
    { 0xb2c18a75, 100 },
    { 0, 0 },
    { 0x32918a77, 40 },
    { 0x7d518a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d218a80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd818a83, 90 },
    { 0x77b18a84, 33 },
    { 0x8e18a85, 91 },
    { 0, 0 },
    { 0xd0c18a87, 61 },
    { 0x2f618a88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae518a95, 61 },
    { 0, 0 },
    { 0x3da18a97, 65 },
    { 0, 0 },
    { 0x96d18a99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb18a9c, 55 },
    { 0, 0 },
    { 0x56e18a9e, 55 },
    { 0, 0 },
    { 0x32218aa0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c18aa5, 23 },
    { 0xaa318aa6, 95 },
    { 0x21f18aa7, 66 },
    { 0xf0d18aa7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f18aac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9418ab0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b18ab6, 76 },
    { 0x6a018ab6, 77 },
    { 0xbd518ab8, 90 },
    { 0x70618ab7, 45 },
    { 0xf2718aba, 55 },
    { 0x99618ab7, 76 },
    { 0x46618ab8, 100 },
    { 0xced18ab9, 95 },
    { 0xb4c18ab9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2618ac2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b18ac9, 33 },
    { 0x2fe18aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c18acd, 90 },
    { 0xe0618acd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb18ad2, 90 },
    { 0xcf918ad3, 76 },
    { 0x58018ad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab18ada, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a18ae2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b818ae7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf18aea, 76 },
    { 0xe4918aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b118af0, 90 },
    { 0x81d18af0, 23 },
    { 0x6c218af2, 95 },
    { 0x59d18af0, 95 },
    { 0x78b18af4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76918af8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a118afb, 90 },
    { 0x50918afb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be18aff, 33 },
    { 0x35a18aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27218b03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa18b06, 98 },
    { 0x3a118b06, 67 },
    { 0x86418b06, 100 },
    { 0, 0 },
    { 0xeb718b0a, 90 },
    { 0, 0 },
    { 0x7318b0c, 78 },
    { 0xc3918b0d, 85 },
    { 0x63118b0e, 61 },
    { 0x68418b0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b818b15, 65 },
    { 0x43218b16, 99 },
    { 0xd9218b17, 23 },
    { 0, 0 },
    { 0x63518b19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d18b21, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x50218b24, 95 },
    { 0, 0 },
    { 0xcc718b26, 33 },
    { 0x5e418b27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5718b2b, 98 },
    { 0, 0 },
    { 0xd1118b2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f118b33, 34 },
    { 0, 0 },
    { 0x10918b35, 23 },
    { 0x7f418b35, 91 },
    { 0x97718b37, 34 },
    { 0x92518b38, 55 },
    { 0x5d718b39, 55 },
    { 0x3ea18b39, 55 },
    { 0xfce18b37, 61 },
    { 0xca418b3c, 78 },
    { 0xf8218b3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed118b41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6118b46, 98 },
    { 0x2b218b46, 90 },
    { 0, 0 },
    { 0xf0318b49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17518b4d, 95 },
    { 0x2218b4e, 90 },
    { 0x1e718b4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0518b55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa18b5b, 23 },
    { 0x13b18b5b, 36 },
    { 0, 0 },
    { 0x6818b5e, 61 },
    { 0, 0 },
    { 0x62818b60, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x54218b63, 33 },
    { 0, 0 },
    { 0xb6918b65, 90 },
    { 0x1d518b66, 98 },
    { 0, 0 },
    { 0x9c618b68, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15318b74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc18b77, 23 },
    { 0x518b78, 90 },
    { 0xe8f18b77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60218b85, 90 },
    { 0x68218b86, 100 },
    { 0, 0 },
    { 0x34518b88, 98 },
    { 0xb6118b88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91018b8e, 96 },
    { 0, 0 },
    { 0x3e818b90, 61 },
    { 0x4ea18b90, 98 },
    { 0xe5018b90, 23 },
    { 0x17618b91, 93 },
    { 0, 0 },
    { 0x87018b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3018b99, 90 },
    { 0, 0 },
    { 0xc1d18b9b, 61 },
    { 0xf7218b9c, 95 },
    { 0x6118b9c, 98 },
    { 0x9f118b9b, 95 },
    { 0x93a18b9f, 66 },
    { 0xbb918ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93018ba4, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb18ba7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21818baa, 90 },
    { 0, 0 },
    { 0x8df18bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca18bb0, 90 },
    { 0xffc18bb1, 98 },
    { 0xbea18bb1, 61 },
    { 0x85918bb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66918bb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86018bbc, 17 },
    { 0, 0 },
    { 0x75d18bbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe018bc2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a18bc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63818bc9, 88 },
    { 0x6cc18bc9, 98 },
    { 0x8bc18bca, 23 },
    { 0x47c18bcb, 90 },
    { 0xa4218bcd, 23 },
    { 0xa6118bce, 33 },
    { 0xc3e18bce, 78 },
    { 0x81e18bd0, 33 },
    { 0, 0 },
    { 0x88918bd2, 43 },
    { 0, 0 },
    { 0xfbf18bd4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35618bd8, 98 },
    { 0, 0 },
    { 0x79518bda, 90 },
    { 0x20718bdb, 90 },
    { 0x4aa18bdc, 34 },
    { 0x57f18bdd, 34 },
    { 0x80718bdc, 91 },
    { 0xb7518bde, 34 },
    { 0x20818be0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f18be5, 55 },
    { 0, 0 },
    { 0x98918be7, 61 },
    { 0, 0 },
    { 0xe2f18be9, 33 },
    { 0x13018be9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c18bf1, 91 },
    { 0x3a218bf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e518bf8, 93 },
    { 0x61918bf8, 98 },
    { 0, 0 },
    { 0xed18bfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc318c02, 61 },
    { 0x7b318c03, 98 },
    { 0xe6418c04, 77 },
    { 0x1fd18c04, 55 },
    { 0xc5318c04, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c818c0e, 95 },
    { 0x43e18c0e, 90 },
    { 0x2c718c0f, 90 },
    { 0x38418c11, 46 },
    { 0x17e18c11, 76 },
    { 0x72b18c13, 61 },
    { 0xf2418c12, 34 },
    { 0xf9118c15, 95 },
    { 0, 0 },
    { 0x90118c17, 33 },
    { 0xbd718c17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa918c1e, 34 },
    { 0, 0 },
    { 0xbd718c20, 99 },
    { 0x6cb18c21, 90 },
    { 0xe4818c22, 95 },
    { 0x6ba18c23, 98 },
    { 0, 0 },
    { 0x88c18c25, 55 },
    { 0x38618c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b318c2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42818c31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1718c34, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6818c38, 98 },
    { 0x84418c39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d018c3c, 34 },
    { 0, 0 },
    { 0xd5c18c3e, 33 },
    { 0x50e18c3f, 95 },
    { 0xdf118c40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0818c48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f18c4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e18c50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c718c53, 90 },
    { 0x90318c54, 33 },
    { 0x35518c53, 95 },
    { 0x73f18c56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce18c5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a18c60, 69 },
    { 0x2218c61, 61 },
    { 0xe0518c62, 95 },
    { 0x58f18c63, 67 },
    { 0xfd218c60, 55 },
    { 0, 0 },
    { 0x86118c66, 33 },
    { 0, 0 },
    { 0xfc318c68, 33 },
    { 0x75818c68, 61 },
    { 0xd5a18c6a, 34 },
    { 0x94018c6b, 58 },
    { 0xfee18c6b, 95 },
    { 0, 0 },
    { 0x6fa18c6e, 61 },
    { 0x61418c6f, 98 },
    { 0x30918c70, 99 },
    { 0x51118c71, 46 },
    { 0xf0f18c70, 98 },
    { 0x9ce18c73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6818c77, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f18c82, 97 },
    { 0x6c18c83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818c86, 66 },
    { 0x6ce18c87, 33 },
    { 0xcbf18c87, 98 },
    { 0, 0 },
    { 0xcba18c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd18c91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c18c96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e118c99, 91 },
    { 0, 0 },
    { 0x1e218c9b, 33 },
    { 0xd9e18c9c, 33 },
    { 0x33718c9d, 84 },
    { 0xa3718c9e, 46 },
    { 0, 0 },
    { 0x76c18ca0, 55 },
    { 0xe1118ca1, 33 },
    { 0, 0 },
    { 0xb6c18ca3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5718ca7, 23 },
    { 0xe8e18ca7, 98 },
    { 0x5c418ca8, 98 },
    { 0x44118caa, 98 },
    { 0x19618ca7, 99 },
    { 0x1b118cac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d318cb2, 33 },
    { 0x27918cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41618cb7, 90 },
    { 0xabc18cb8, 98 },
    { 0x69618cb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a618cbd, 91 },
    { 0xc818cbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf18cc6, 85 },
    { 0x19618cc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed318cce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4018cd1, 61 },
    { 0x6ae18cd1, 66 },
    { 0x6f318cd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3318cd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9018cde, 98 },
    { 0x46718cde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a18ce5, 95 },
    { 0x99918ce6, 91 },
    { 0x17e18ce6, 33 },
    { 0x92918ce7, 95 },
    { 0xd2318ce8, 55 },
    { 0, 0 },
    { 0xb8918ceb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d618cf2, 61 },
    { 0x6918cf3, 46 },
    { 0x34b18cf4, 95 },
    { 0x58b18cf3, 40 },
    { 0x7b318cf6, 33 },
    { 0, 0 },
    { 0xa5218cf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b18cfb, 100 },
    { 0x2018cfc, 46 },
    { 0xc9018cfd, 90 },
    { 0x37518cfd, 55 },
    { 0xb8418cfe, 76 },
    { 0x4c418cff, 67 },
    { 0xc0418d01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a18d06, 68 },
    { 0xfee18d07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad18d0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b118d11, 98 },
    { 0x92518d11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca218d16, 98 },
    { 0xdc918d17, 23 },
    { 0xc2118d16, 76 },
    { 0xf0c18d19, 23 },
    { 0x9bc18d1a, 33 },
    { 0x97818d1a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8418d1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b318d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab18d27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd618d2e, 90 },
    { 0x39b18d2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf18d32, 98 },
    { 0, 0 },
    { 0xc8818d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57918d3d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa718d40, 95 },
    { 0, 0 },
    { 0x5e318d42, 90 },
    { 0x8c918d42, 93 },
    { 0xee518d44, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7318d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c18d51, 98 },
    { 0, 0 },
    { 0x8fd18d53, 98 },
    { 0x46b18d53, 90 },
    { 0, 0 },
    { 0xfa818d56, 87 },
    { 0, 0 },
    { 0xf0418d58, 43 },
    { 0, 0 },
    { 0xe9318d5a, 76 },
    { 0, 0 },
    { 0x90a18d5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b718d61, 55 },
    { 0xe2b18d61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6318d65, 76 },
    { 0x88b18d66, 46 },
    { 0xe2918d67, 90 },
    { 0x7c018d68, 90 },
    { 0xf2118d65, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f018d6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c318d72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27618d77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f18d7b, 78 },
    { 0, 0 },
    { 0x45418d7d, 99 },
    { 0x78718d7d, 55 },
    { 0x9c418d7f, 98 },
    { 0xc4718d80, 90 },
    { 0x78f18d81, 66 },
    { 0x62f18d80, 98 },
    { 0xfad18d80, 33 },
    { 0x14118d82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a18d87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31218d90, 90 },
    { 0xf4718d91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8918d95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28918d98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff18d9d, 95 },
    { 0, 0 },
    { 0x62b18d9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf418da3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f18daf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63618db5, 90 },
    { 0, 0 },
    { 0x8b018db7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85718dc2, 98 },
    { 0xdbf18dc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa418dc7, 91 },
    { 0x36c18dc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c18dcb, 23 },
    { 0, 0 },
    { 0xef918dcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f18dd4, 98 },
    { 0, 0 },
    { 0xd2318dd6, 90 },
    { 0x35e18dd7, 78 },
    { 0x69718dd7, 40 },
    { 0xab118dd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e618ddc, 87 },
    { 0, 0 },
    { 0x80618dde, 66 },
    { 0x83f18dde, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd18de2, 95 },
    { 0x20218de2, 23 },
    { 0xd9618de4, 90 },
    { 0x7f18de3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b518de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf018dec, 90 },
    { 0xeb618ded, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8218df2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2618df9, 98 },
    { 0x46418dfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d118dfe, 66 },
    { 0x2be18dfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19718e03, 84 },
    { 0, 0 },
    { 0xe3218e05, 48 },
    { 0x89918e06, 76 },
    { 0xf0118e06, 88 },
    { 0x49b18e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14718e0b, 61 },
    { 0, 0 },
    { 0x9d918e0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b18e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80118e1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c18e21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5318e24, 66 },
    { 0, 0 },
    { 0xe8518e26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d18e29, 99 },
    { 0xc7318e2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14618e32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e18e36, 98 },
    { 0x4ee18e37, 61 },
    { 0x28318e37, 61 },
    { 0xa0918e36, 93 },
    { 0xf1a18e38, 43 },
    { 0xd4e18e39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f518e3e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18e43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79718e47, 61 },
    { 0, 0 },
    { 0x26118e49, 46 },
    { 0, 0 },
    { 0xc7c18e4b, 95 },
    { 0, 0 },
    { 0x86018e4d, 33 },
    { 0, 0 },
    { 0x41618e4f, 90 },
    { 0, 0 },
    { 0x52a18e51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63518e54, 33 },
    { 0x5518e55, 61 },
    { 0x4ba18e56, 98 },
    { 0x9a518e55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b18e5a, 43 },
    { 0, 0 },
    { 0x68a18e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6318e61, 76 },
    { 0xdc518e62, 67 },
    { 0, 0 },
    { 0xe5618e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2318e69, 55 },
    { 0xed818e6a, 68 },
    { 0xda918e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b18e6e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc518e76, 33 },
    { 0, 0 },
    { 0xec118e78, 23 },
    { 0, 0 },
    { 0x6de18e7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78018e7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d18e86, 34 },
    { 0, 0 },
    { 0x25c18e88, 85 },
    { 0x2ff18e89, 64 },
    { 0, 0 },
    { 0x3e618e8b, 23 },
    { 0x5c118e8c, 33 },
    { 0, 0 },
    { 0xc4418e8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3118e93, 90 },
    { 0xdce18e94, 80 },
    { 0x12e18e94, 66 },
    { 0xa5518e96, 87 },
    { 0x1ff18e97, 61 },
    { 0x4de18e98, 33 },
    { 0xe1a18e93, 93 },
    { 0xb2c18e9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7418ea0, 61 },
    { 0xd7818ea1, 98 },
    { 0xa3018ea2, 61 },
    { 0, 0 },
    { 0xeb18ea4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea18ea7, 76 },
    { 0x2e618ea7, 77 },
    { 0xbf018ea8, 55 },
    { 0, 0 },
    { 0xc0f18eab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e718eaf, 95 },
    { 0x16f18eaf, 55 },
    { 0xb818eb1, 76 },
    { 0xdf818eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c18eb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e18ebf, 91 },
    { 0x2e718ec0, 90 },
    { 0x35718ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36118ec6, 90 },
    { 0, 0 },
    { 0x1a318ec8, 90 },
    { 0, 0 },
    { 0x4d018eca, 23 },
    { 0x80e18eca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3618ece, 98 },
    { 0x7f218ecf, 90 },
    { 0x71c18ece, 90 },
    { 0xc1918ed1, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40518ed6, 91 },
    { 0xb5018ed6, 34 },
    { 0xec018ed8, 23 },
    { 0xf1a18ed8, 95 },
    { 0xbf618eda, 80 },
    { 0, 0 },
    { 0x67018edc, 90 },
    { 0x4a18edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c18ee0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b018ee4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a018ee9, 90 },
    { 0, 0 },
    { 0xd2e18eeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a918ef0, 33 },
    { 0x11318ef1, 33 },
    { 0x57018ef2, 46 },
    { 0x6d18ef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba18ef8, 65 },
    { 0x30a18ef8, 25 },
    { 0x47818ef9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d18efe, 55 },
    { 0xa8c18efe, 64 },
    { 0, 0 },
    { 0x3d918f01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce318f06, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5318f09, 61 },
    { 0, 0 },
    { 0x3c818f0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a418f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c18f11, 61 },
    { 0, 0 },
    { 0x74d18f13, 67 },
    { 0x30b18f13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab18f18, 33 },
    { 0x84d18f19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b18f1c, 55 },
    { 0x90818f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d18f23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54518f28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6718f2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6018f33, 98 },
    { 0x90818f34, 34 },
    { 0xe5518f35, 76 },
    { 0xaee18f36, 33 },
    { 0xfe318f37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3218f3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d518f3e, 55 },
    { 0, 0 },
    { 0x44718f40, 33 },
    { 0x81a18f41, 85 },
    { 0xcf418f41, 23 },
    { 0xfb318f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d718f49, 95 },
    { 0x1da18f4a, 90 },
    { 0xa4d18f4b, 61 },
    { 0xbdf18f4c, 55 },
    { 0x5a918f4a, 33 },
    { 0x35b18f4e, 95 },
    { 0xd5c18f4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6918f52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd718f57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc18f5e, 24 },
    { 0x57118f5f, 88 },
    { 0xf0a18f60, 33 },
    { 0, 0 },
    { 0xa7218f62, 23 },
    { 0x10118f62, 33 },
    { 0x88418f62, 23 },
    { 0, 0 },
    { 0x5b418f66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc218f69, 95 },
    { 0x9918f6a, 33 },
    { 0x8f418f6a, 90 },
    { 0, 0 },
    { 0x5b18f6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97518f71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17118f77, 90 },
    { 0xa7d18f77, 93 },
    { 0x4f718f78, 95 },
    { 0x2ff18f7a, 95 },
    { 0xfda18f7b, 99 },
    { 0xad518f7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5518f84, 84 },
    { 0x4718f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb18f89, 23 },
    { 0x73018f89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86218f8e, 81 },
    { 0x8c518f8e, 76 },
    { 0, 0 },
    { 0x5ed18f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6718f94, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6718f9b, 99 },
    { 0x6e518f9b, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe418fa0, 23 },
    { 0x86b18fa0, 97 },
    { 0x2618fa2, 90 },
    { 0x8418fa2, 76 },
    { 0, 0 },
    { 0x29518fa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62718fa8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61318fac, 90 },
    { 0x55718fac, 33 },
    { 0, 0 },
    { 0x2ce18faf, 43 },
    { 0, 0 },
    { 0x83118fb1, 61 },
    { 0, 0 },
    { 0x90518fb3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe118fb8, 99 },
    { 0, 0 },
    { 0xf2b18fba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69818fbe, 87 },
    { 0, 0 },
    { 0xcea18fc0, 61 },
    { 0, 0 },
    { 0xe2818fc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c18fc7, 85 },
    { 0xe1018fc8, 33 },
    { 0x35418fc7, 52 },
    { 0xd6e18fc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b18fce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e18fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a18fd5, 76 },
    { 0, 0 },
    { 0x80018fd7, 98 },
    { 0x6d318fd7, 55 },
    { 0x57118fd9, 55 },
    { 0x1218fd9, 93 },
    { 0x11318fdb, 95 },
    { 0x84418fdc, 64 },
    { 0x57418fdd, 75 },
    { 0x18518fde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d18fea, 99 },
    { 0x5d218feb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8218fef, 52 },
    { 0xd5618ff0, 23 },
    { 0xbe18ff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc618ff8, 56 },
    { 0xe5618ff9, 43 },
    { 0xd1a18ff8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18018fff, 85 },
    { 0x60c18fff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28219004, 52 },
    { 0xb8819005, 75 },
    { 0x91819006, 65 },
    { 0xb0219007, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb491900c, 78 },
    { 0, 0 },
    { 0x38e1900e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f19012, 76 },
    { 0x3f519013, 98 },
    { 0xa3d19014, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b619019, 61 },
    { 0x8991901a, 88 },
    { 0x8821901b, 61 },
    { 0, 0 },
    { 0xb1901d, 84 },
    { 0, 0 },
    { 0xeb91901f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47519022, 90 },
    { 0xf5e19023, 52 },
    { 0x29a19023, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc619027, 23 },
    { 0x6ce19028, 98 },
    { 0, 0 },
    { 0xe731902a, 61 },
    { 0, 0 },
    { 0xe281902c, 23 },
    { 0x20a1902d, 84 },
    { 0xd531902c, 53 },
    { 0, 0 },
    { 0xdaf19030, 87 },
    { 0, 0 },
    { 0x8c219032, 78 },
    { 0, 0 },
    { 0x10019034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d19039, 95 },
    { 0, 0 },
    { 0x7c11903b, 34 },
    { 0, 0 },
    { 0x6741903d, 82 },
    { 0xd641903e, 75 },
    { 0xa7c1903f, 90 },
    { 0x1f71903f, 95 },
    { 0x1ef19041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c19048, 61 },
    { 0xcf519049, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331904f, 90 },
    { 0x56019050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36019054, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96819058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9691905e, 23 },
    { 0xde81905f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2919063, 23 },
    { 0, 0 },
    { 0xfa619065, 47 },
    { 0, 0 },
    { 0x96219067, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a41906c, 23 },
    { 0x55f1906d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff919070, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8919073, 97 },
    { 0, 0 },
    { 0x77519075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3519079, 56 },
    { 0x3821907a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd19080, 66 },
    { 0xffb19080, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d619084, 90 },
    { 0, 0 },
    { 0x55919086, 61 },
    { 0xccf19087, 61 },
    { 0x4e119086, 95 },
    { 0, 0 },
    { 0xb8a1908a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6019095, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1519099, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6190a0, 90 },
    { 0xd2190a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f190a9, 91 },
    { 0, 0 },
    { 0x671190ab, 90 },
    { 0, 0 },
    { 0x53d190ad, 53 },
    { 0, 0 },
    { 0xd10190af, 65 },
    { 0x6bd190b0, 39 },
    { 0x2dc190b0, 34 },
    { 0x435190b2, 23 },
    { 0xa1a190b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7190bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45190c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f190c5, 23 },
    { 0xda2190c5, 90 },
    { 0x67f190c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a190d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x811190d4, 33 },
    { 0xa25190d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72190db, 90 },
    { 0x213190dc, 34 },
    { 0xf44190dd, 34 },
    { 0, 0 },
    { 0xa0b190df, 34 },
    { 0x30b190e0, 77 },
    { 0x3a3190e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381190e9, 55 },
    { 0x675190ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44190ef, 100 },
    { 0x780190f0, 23 },
    { 0xa2e190f1, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455190f5, 34 },
    { 0x57c190f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711190fa, 43 },
    { 0x4d3190fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95519100, 95 },
    { 0xee319101, 67 },
    { 0xd5f19102, 90 },
    { 0x19102, 61 },
    { 0x39619103, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd11910a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22019110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7519113, 100 },
    { 0x3b919114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27519117, 96 },
    { 0x58319118, 43 },
    { 0x71c19118, 77 },
    { 0, 0 },
    { 0xc701911b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed19121, 38 },
    { 0x62919122, 23 },
    { 0x5ac19123, 95 },
    { 0x23219124, 43 },
    { 0x82e19124, 33 },
    { 0xfe419126, 61 },
    { 0x95919125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c01912a, 61 },
    { 0x4b1912b, 90 },
    { 0, 0 },
    { 0x8261912d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b19130, 76 },
    { 0x9b419130, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed819135, 23 },
    { 0, 0 },
    { 0xc719137, 55 },
    { 0xf2e19137, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11913d, 98 },
    { 0, 0 },
    { 0x9711913f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc819142, 34 },
    { 0x1f019143, 61 },
    { 0x92919144, 90 },
    { 0x77019145, 96 },
    { 0x6c419146, 61 },
    { 0x86419143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a01914d, 91 },
    { 0x2e71914d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31919153, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee319159, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad71915d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1619160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce19167, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61916d, 98 },
    { 0x73a1916d, 95 },
    { 0, 0 },
    { 0x50519170, 25 },
    { 0, 0 },
    { 0x44c19172, 90 },
    { 0x21519172, 93 },
    { 0xce919172, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6741917d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a19183, 55 },
    { 0x70f19184, 90 },
    { 0x3fb19185, 85 },
    { 0x82919185, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db19189, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7461918e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60419192, 93 },
    { 0, 0 },
    { 0x39d19194, 95 },
    { 0xf3519195, 93 },
    { 0x95e19195, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c61919a, 23 },
    { 0x4311919b, 99 },
    { 0xd7f1919c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19191a0, 99 },
    { 0x3a6191a1, 61 },
    { 0x47f191a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8191aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375191af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f191b2, 34 },
    { 0x812191b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08191b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8191bd, 100 },
    { 0x7c191be, 23 },
    { 0xb48191be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0191c3, 61 },
    { 0xf00191c4, 61 },
    { 0x7df191c4, 61 },
    { 0x9ec191c5, 90 },
    { 0x81191c4, 98 },
    { 0x532191c8, 61 },
    { 0x33d191c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c191cf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59191d4, 23 },
    { 0xcd1191d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67191e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925191e7, 76 },
    { 0xd1a191e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b191ec, 43 },
    { 0xae7191ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39191f0, 77 },
    { 0xb7a191f0, 98 },
    { 0x693191f1, 33 },
    { 0x867191f2, 61 },
    { 0x90d191f1, 33 },
    { 0xdbf191f5, 43 },
    { 0, 0 },
    { 0xb76191f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9191fb, 76 },
    { 0, 0 },
    { 0xe9c191fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d919202, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74219206, 95 },
    { 0x3a19206, 90 },
    { 0, 0 },
    { 0x4bd19209, 76 },
    { 0xa3119209, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb1920e, 98 },
    { 0xec01920e, 78 },
    { 0x90f19210, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4819213, 93 },
    { 0x34619213, 95 },
    { 0x4f19213, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd951921a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb19227, 33 },
    { 0x49a19228, 33 },
    { 0xbfe19229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1421922e, 33 },
    { 0x3561922f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b19233, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52319236, 61 },
    { 0, 0 },
    { 0xc619238, 61 },
    { 0xec619239, 34 },
    { 0x4a219239, 61 },
    { 0x7f41923b, 78 },
    { 0, 0 },
    { 0x2a11923d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e19241, 34 },
    { 0xdbc19241, 33 },
    { 0x98919243, 43 },
    { 0x18119241, 90 },
    { 0, 0 },
    { 0x7da19246, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f1924c, 61 },
    { 0x37b1924d, 98 },
    { 0xdc11924c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb19254, 55 },
    { 0xcc919254, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8941925b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7719266, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d219269, 85 },
    { 0xfa319269, 33 },
    { 0x8971926b, 55 },
    { 0x4231926a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da1926f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e19272, 33 },
    { 0, 0 },
    { 0x1b719274, 33 },
    { 0xf0819275, 66 },
    { 0x98d19276, 98 },
    { 0xf419276, 98 },
    { 0xfad19277, 93 },
    { 0x11019277, 67 },
    { 0x3a21927a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf19289, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc071928c, 68 },
    { 0x68b1928c, 78 },
    { 0, 0 },
    { 0xc811928f, 59 },
    { 0xd9a19290, 65 },
    { 0xeed19290, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf319296, 61 },
    { 0, 0 },
    { 0x23b19298, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff01929c, 48 },
    { 0x1621929c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x374192a0, 17 },
    { 0x71c192a1, 98 },
    { 0, 0 },
    { 0xbba192a3, 95 },
    { 0x8a0192a4, 88 },
    { 0, 0 },
    { 0x1d3192a6, 33 },
    { 0xf41192a7, 77 },
    { 0, 0 },
    { 0x3ee192a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5192ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380192b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d192b5, 91 },
    { 0x7f6192b6, 23 },
    { 0, 0 },
    { 0x710192b8, 77 },
    { 0, 0 },
    { 0x607192ba, 87 },
    { 0, 0 },
    { 0xb84192bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7192bf, 45 },
    { 0, 0 },
    { 0x3ce192c1, 55 },
    { 0, 0 },
    { 0xca6192c3, 61 },
    { 0x34a192c4, 95 },
    { 0x69192c5, 52 },
    { 0, 0 },
    { 0x2b6192c7, 78 },
    { 0, 0 },
    { 0x144192c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e192cf, 34 },
    { 0, 0 },
    { 0x413192d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab192da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631192e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da192e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08192eb, 90 },
    { 0, 0 },
    { 0xc83192ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c192f1, 90 },
    { 0xeb192f2, 33 },
    { 0xa1d192f2, 76 },
    { 0x200192f4, 46 },
    { 0x9c8192f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2192fd, 90 },
    { 0xb63192fe, 90 },
    { 0, 0 },
    { 0xc0419300, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1819309, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe01930c, 52 },
    { 0, 0 },
    { 0x30c1930e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x23519311, 23 },
    { 0xd6019311, 33 },
    { 0, 0 },
    { 0x93c19314, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x47119317, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc1931b, 61 },
    { 0xa731931c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa091931f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87919322, 66 },
    { 0x8a519323, 98 },
    { 0, 0 },
    { 0x30c19325, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3619328, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871932c, 95 },
    { 0x4651932d, 55 },
    { 0xa421932d, 76 },
    { 0x6b1932c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8019332, 34 },
    { 0x2df19333, 23 },
    { 0xc2919332, 33 },
    { 0, 0 },
    { 0x7fe19336, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf1933f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b19342, 90 },
    { 0x1fc19343, 61 },
    { 0xa9119343, 76 },
    { 0xf3519343, 98 },
    { 0x5e219346, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c81934a, 66 },
    { 0xfe1934a, 34 },
    { 0x4c71934a, 61 },
    { 0, 0 },
    { 0xb731934e, 43 },
    { 0x14e1934f, 95 },
    { 0xd5c1934e, 98 },
    { 0xba61934f, 61 },
    { 0, 0 },
    { 0xbee19353, 55 },
    { 0x99319353, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d01935d, 33 },
    { 0x5b31935d, 99 },
    { 0xdff1935f, 23 },
    { 0xcb519360, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce19363, 23 },
    { 0x1b919364, 43 },
    { 0x77f19364, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9181936a, 23 },
    { 0, 0 },
    { 0xafe1936c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c19374, 61 },
    { 0, 0 },
    { 0x59319376, 61 },
    { 0xc7419377, 66 },
    { 0x77119377, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e51937e, 93 },
    { 0, 0 },
    { 0x7c619380, 98 },
    { 0xe3219381, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a1938a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891938d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9719393, 90 },
    { 0x6c719394, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc131939a, 90 },
    { 0x1691939b, 55 },
    { 0, 0 },
    { 0x44e1939d, 91 },
    { 0x1691939d, 90 },
    { 0xab71939e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd193a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb193a9, 90 },
    { 0x3fe193aa, 46 },
    { 0x8a1193aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de193b1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c193c5, 23 },
    { 0x671193c6, 61 },
    { 0x71c193c7, 67 },
    { 0x475193c8, 90 },
    { 0x231193c8, 98 },
    { 0, 0 },
    { 0x88e193cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e193d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935193d7, 76 },
    { 0xc5c193d7, 75 },
    { 0x6eb193d9, 95 },
    { 0x56e193da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d193e1, 23 },
    { 0x35e193e2, 68 },
    { 0xee8193e3, 95 },
    { 0xc2b193e4, 95 },
    { 0xe63193e4, 46 },
    { 0x306193e4, 99 },
    { 0xe28193e4, 33 },
    { 0xd2a193e8, 23 },
    { 0x341193e9, 23 },
    { 0xa3f193e9, 90 },
    { 0x1d193eb, 34 },
    { 0x433193e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1193f1, 33 },
    { 0x483193f1, 33 },
    { 0x891193f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29193f9, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90193fd, 90 },
    { 0xd04193fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0419402, 93 },
    { 0, 0 },
    { 0xb919404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19919409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1940e, 67 },
    { 0x4ef1940e, 34 },
    { 0xb9f19410, 38 },
    { 0xba819411, 23 },
    { 0xfa19412, 100 },
    { 0xcf01940e, 34 },
    { 0x8c019414, 95 },
    { 0x28219413, 95 },
    { 0xd6619413, 95 },
    { 0xe9719417, 95 },
    { 0, 0 },
    { 0xc9919419, 43 },
    { 0xe4f1941a, 23 },
    { 0, 0 },
    { 0x5121941c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c1941f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb19423, 85 },
    { 0xab819424, 90 },
    { 0xa1019424, 90 },
    { 0xc8e19426, 46 },
    { 0xe7019424, 95 },
    { 0, 0 },
    { 0xb2219429, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d1942d, 91 },
    { 0x2e41942e, 46 },
    { 0x2981942f, 90 },
    { 0, 0 },
    { 0x5b019431, 23 },
    { 0, 0 },
    { 0xef419433, 98 },
    { 0x80019433, 61 },
    { 0x54419434, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc901943a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9871943f, 88 },
    { 0x39119440, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a19444, 43 },
    { 0xde419445, 33 },
    { 0, 0 },
    { 0xe2219447, 98 },
    { 0x63619448, 61 },
    { 0xd6819449, 61 },
    { 0, 0 },
    { 0x9b71944b, 33 },
    { 0x93b1944c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3919450, 40 },
    { 0, 0 },
    { 0xe9b19452, 39 },
    { 0x44919453, 46 },
    { 0x29e19452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa19459, 98 },
    { 0x43019459, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e19461, 46 },
    { 0x8a719461, 23 },
    { 0, 0 },
    { 0x12d19464, 39 },
    { 0xae019464, 78 },
    { 0, 0 },
    { 0x70219467, 91 },
    { 0, 0 },
    { 0x81719469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c91946c, 90 },
    { 0xb071946d, 23 },
    { 0x1f01946e, 90 },
    { 0x3841946e, 61 },
    { 0x9421946d, 33 },
    { 0x4011946f, 98 },
    { 0x2b119472, 90 },
    { 0x5b419470, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5119477, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e1947c, 55 },
    { 0x6a91947c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf119480, 33 },
    { 0, 0 },
    { 0xc7119482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b919487, 90 },
    { 0, 0 },
    { 0x79019489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd11948c, 66 },
    { 0x7a21948d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a19493, 95 },
    { 0xa5519493, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa19497, 95 },
    { 0x33619498, 90 },
    { 0xb8f19497, 23 },
    { 0xde419498, 43 },
    { 0x7a11949b, 23 },
    { 0x6e11949c, 33 },
    { 0, 0 },
    { 0x5691949e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a194a1, 90 },
    { 0x7df194a2, 90 },
    { 0, 0 },
    { 0x32194a4, 87 },
    { 0x682194a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de194a8, 33 },
    { 0x359194a9, 55 },
    { 0xebc194a9, 55 },
    { 0xc6b194ab, 99 },
    { 0x512194ac, 61 },
    { 0x6e4194ac, 46 },
    { 0x751194ae, 78 },
    { 0xe21194af, 93 },
    { 0xd19194aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830194b5, 61 },
    { 0xcc2194b6, 61 },
    { 0, 0 },
    { 0x8b3194b8, 46 },
    { 0, 0 },
    { 0xcd6194ba, 93 },
    { 0xe7e194bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab194bf, 43 },
    { 0, 0 },
    { 0x37a194c1, 61 },
    { 0x83a194c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2194c7, 98 },
    { 0x326194c8, 98 },
    { 0x5a7194c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0194cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac194d0, 8 },
    { 0x901194d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e194d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182194e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25194e8, 78 },
    { 0xf55194e9, 34 },
    { 0x49a194e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x105194ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8194f1, 33 },
    { 0x176194f2, 91 },
    { 0x520194f2, 85 },
    { 0x50a194f2, 61 },
    { 0xb5e194f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8194f9, 67 },
    { 0x1bd194fa, 33 },
    { 0x2dd194fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a19501, 84 },
    { 0, 0 },
    { 0x5fe19503, 95 },
    { 0x8e019503, 95 },
    { 0xebc19503, 34 },
    { 0x77819506, 99 },
    { 0xdb119507, 34 },
    { 0x7119508, 87 },
    { 0, 0 },
    { 0x9f41950a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f19511, 33 },
    { 0, 0 },
    { 0x59c19513, 84 },
    { 0x96f19513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59819518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4461951b, 61 },
    { 0x6731951c, 52 },
    { 0x13b1951d, 38 },
    { 0xa011951e, 23 },
    { 0xdce1951d, 46 },
    { 0xe8c1951f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3419528, 78 },
    { 0x9da19529, 23 },
    { 0x2271952a, 39 },
    { 0x6d11952a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7919531, 33 },
    { 0x97419531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99419536, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78119545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9219549, 95 },
    { 0, 0 },
    { 0x641954b, 76 },
    { 0, 0 },
    { 0x1f61954d, 88 },
    { 0x3131954e, 43 },
    { 0x14b1954f, 85 },
    { 0x16e19550, 66 },
    { 0xdba1954f, 95 },
    { 0, 0 },
    { 0x7e519553, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd19556, 93 },
    { 0x4ae19556, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f11955b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91955e, 33 },
    { 0xc151955f, 76 },
    { 0x3d719560, 55 },
    { 0x5b019561, 76 },
    { 0x80e19562, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15819566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19570, 17 },
    { 0x80419570, 33 },
    { 0xb9f19572, 23 },
    { 0x6c19572, 33 },
    { 0x77319572, 33 },
    { 0x38519575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60119579, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e019586, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e1958b, 98 },
    { 0xa371958b, 35 },
    { 0x9c61958d, 33 },
    { 0x4251958d, 84 },
    { 0x8a1958f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e319593, 95 },
    { 0, 0 },
    { 0x2ab19595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b19598, 43 },
    { 0x1a219599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d11959e, 23 },
    { 0, 0 },
    { 0xe44195a0, 61 },
    { 0x763195a1, 33 },
    { 0, 0 },
    { 0x467195a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05195a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0195ab, 90 },
    { 0, 0 },
    { 0xc03195ad, 95 },
    { 0, 0 },
    { 0x778195af, 76 },
    { 0x9e3195b0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af195b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x411195b8, 34 },
    { 0, 0 },
    { 0x8ef195ba, 61 },
    { 0xf37195bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896195bf, 52 },
    { 0, 0 },
    { 0x5f4195c1, 55 },
    { 0, 0 },
    { 0xa57195c3, 23 },
    { 0xeec195c4, 85 },
    { 0x81a195c5, 76 },
    { 0x15b195c6, 23 },
    { 0x6b3195c4, 33 },
    { 0xe11195c8, 23 },
    { 0x367195c9, 33 },
    { 0x5ed195c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba195cd, 90 },
    { 0xf4f195ce, 67 },
    { 0xd25195cf, 90 },
    { 0, 0 },
    { 0x3f3195d1, 66 },
    { 0x5cc195d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af195d7, 33 },
    { 0x494195d7, 23 },
    { 0, 0 },
    { 0x68b195da, 66 },
    { 0, 0 },
    { 0xa54195dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4195df, 90 },
    { 0, 0 },
    { 0x4f5195e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb195e4, 85 },
    { 0xb68195e4, 95 },
    { 0x487195e6, 61 },
    { 0xac195e7, 100 },
    { 0xf8e195e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a195eb, 23 },
    { 0xd62195eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1195ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6195f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9195f6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13195fd, 88 },
    { 0xc6195fe, 95 },
    { 0xc7195ff, 34 },
    { 0xc15195fd, 93 },
    { 0, 0 },
    { 0x20319602, 90 },
    { 0x10019602, 23 },
    { 0x33119604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7119607, 61 },
    { 0x67c19608, 33 },
    { 0, 0 },
    { 0xfbc1960a, 84 },
    { 0x5141960b, 90 },
    { 0, 0 },
    { 0x13b1960d, 76 },
    { 0x38a1960e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2219616, 61 },
    { 0xc1219617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1961a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4601961e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a19621, 69 },
    { 0x4a719621, 87 },
    { 0, 0 },
    { 0xcc919624, 23 },
    { 0, 0 },
    { 0x53b19626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d31962e, 43 },
    { 0, 0 },
    { 0x77b19630, 90 },
    { 0, 0 },
    { 0xfd619632, 83 },
    { 0x36819633, 33 },
    { 0x39219632, 33 },
    { 0x49419633, 33 },
    { 0xca419636, 91 },
    { 0, 0 },
    { 0x83a19638, 85 },
    { 0, 0 },
    { 0xeab1963a, 83 },
    { 0, 0 },
    { 0xe491963c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c1963f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15019642, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e19649, 33 },
    { 0xcd519649, 33 },
    { 0, 0 },
    { 0x9d21964c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e1964f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22619652, 43 },
    { 0xbf719653, 34 },
    { 0, 0 },
    { 0x20219655, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271965d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b19661, 34 },
    { 0xd5e19662, 98 },
    { 0, 0 },
    { 0xb7a19664, 95 },
    { 0xd8d19665, 61 },
    { 0x45019665, 67 },
    { 0x29919667, 34 },
    { 0xfc219668, 100 },
    { 0x99219668, 98 },
    { 0x5d21966a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e1966d, 23 },
    { 0x861966e, 87 },
    { 0x44e1966e, 93 },
    { 0, 0 },
    { 0x49a19671, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62919677, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc471967b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e419681, 55 },
    { 0x8a419681, 90 },
    { 0x5b119683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db19687, 23 },
    { 0xf6419687, 90 },
    { 0xcc819689, 98 },
    { 0xaf919688, 23 },
    { 0xe4d1968b, 33 },
    { 0, 0 },
    { 0xa391968d, 55 },
    { 0x98f1968e, 77 },
    { 0xf341968f, 55 },
    { 0x32019690, 90 },
    { 0x75719690, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f019694, 76 },
    { 0, 0 },
    { 0xea419696, 91 },
    { 0, 0 },
    { 0xdd219698, 78 },
    { 0x40819699, 23 },
    { 0x92519698, 100 },
    { 0, 0 },
    { 0x84c1969c, 80 },
    { 0, 0 },
    { 0xb851969e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7196a2, 98 },
    { 0, 0 },
    { 0xc9196a4, 67 },
    { 0xdd2196a4, 99 },
    { 0, 0 },
    { 0x157196a7, 33 },
    { 0x608196a8, 99 },
    { 0xafa196a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2196b0, 23 },
    { 0x932196b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c196b5, 33 },
    { 0, 0 },
    { 0x91c196b7, 33 },
    { 0xbfd196b8, 98 },
    { 0xad5196b9, 90 },
    { 0x73d196b9, 51 },
    { 0x429196bb, 61 },
    { 0xf1f196bb, 33 },
    { 0xb60196bd, 91 },
    { 0x4ab196be, 87 },
    { 0xcc6196bf, 95 },
    { 0x1e4196bc, 95 },
    { 0xd35196c1, 98 },
    { 0x948196c0, 33 },
    { 0xe8196c2, 78 },
    { 0x4e5196c0, 55 },
    { 0xf53196c5, 43 },
    { 0, 0 },
    { 0x398196c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9196ca, 23 },
    { 0x493196ca, 34 },
    { 0x779196cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719196d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79196d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1196d8, 23 },
    { 0xa80196d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87196de, 98 },
    { 0x19c196df, 90 },
    { 0, 0 },
    { 0x9aa196e1, 23 },
    { 0x86b196e1, 90 },
    { 0xb5d196e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23196e6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40196e9, 61 },
    { 0x645196ea, 76 },
    { 0xe42196eb, 95 },
    { 0x1e9196ea, 43 },
    { 0xf3b196e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce196f8, 100 },
    { 0xcc1196f9, 90 },
    { 0, 0 },
    { 0xf24196fb, 90 },
    { 0, 0 },
    { 0xf5e196fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66019701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f919705, 40 },
    { 0, 0 },
    { 0xfcd19707, 35 },
    { 0x5e319708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a1970b, 87 },
    { 0xb441970c, 33 },
    { 0x7821970b, 76 },
    { 0x2d41970e, 23 },
    { 0x9f41970f, 46 },
    { 0x29519710, 43 },
    { 0x1b319711, 76 },
    { 0xf4e19710, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf31971a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a81971e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b619722, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a819727, 100 },
    { 0, 0 },
    { 0xe719729, 33 },
    { 0x4c11972a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9419735, 66 },
    { 0xfdd19736, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa961973b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b719740, 98 },
    { 0xe3519741, 76 },
    { 0x8a919741, 96 },
    { 0x75f19743, 46 },
    { 0x2c619742, 90 },
    { 0, 0 },
    { 0x16c19746, 88 },
    { 0, 0 },
    { 0xe1319748, 34 },
    { 0, 0 },
    { 0xf9d1974a, 46 },
    { 0xb681974b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa19751, 61 },
    { 0xb19752, 93 },
    { 0xe4f19751, 90 },
    { 0x8b519754, 85 },
    { 0x9f119755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd819759, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf151975d, 99 },
    { 0, 0 },
    { 0x6e1975f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4219763, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe11976b, 94 },
    { 0xeba1976c, 95 },
    { 0x4641976b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c419770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b19773, 33 },
    { 0xa9619774, 46 },
    { 0xd4d19774, 33 },
    { 0x51e19776, 90 },
    { 0, 0 },
    { 0x44819778, 95 },
    { 0, 0 },
    { 0x8c41977a, 61 },
    { 0x8151977b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a919780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d419783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b019786, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xab219789, 55 },
    { 0x92a19789, 23 },
    { 0x7341978b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1978f, 55 },
    { 0xbab1978f, 33 },
    { 0xaa619791, 98 },
    { 0x1a419792, 100 },
    { 0x12119793, 61 },
    { 0x52019794, 23 },
    { 0xa3b19793, 61 },
    { 0, 0 },
    { 0xf3119797, 95 },
    { 0, 0 },
    { 0x3cd19799, 95 },
    { 0x1c519799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8197a1, 90 },
    { 0, 0 },
    { 0x7d7197a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e197a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0197ab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8197ae, 61 },
    { 0xe8197af, 2 },
    { 0x2d6197af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7197ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f197be, 46 },
    { 0xa42197be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97197ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d197ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85197d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de197d4, 78 },
    { 0x762197d5, 61 },
    { 0xe76197d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0197d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7197dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b197e4, 78 },
    { 0, 0 },
    { 0x90d197e6, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63197ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c197f0, 66 },
    { 0xc39197f1, 90 },
    { 0, 0 },
    { 0xa98197f3, 55 },
    { 0x7fd197f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85197f7, 34 },
    { 0, 0 },
    { 0x677197f9, 90 },
    { 0, 0 },
    { 0x60b197fb, 76 },
    { 0x324197fb, 43 },
    { 0x4c6197fd, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d19806, 87 },
    { 0x3d519806, 78 },
    { 0xf8919808, 61 },
    { 0, 0 },
    { 0x3061980a, 93 },
    { 0, 0 },
    { 0xd2d1980c, 98 },
    { 0xc261980c, 61 },
    { 0, 0 },
    { 0x44a1980f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca1981e, 34 },
    { 0xec41981f, 61 },
    { 0xfea19820, 33 },
    { 0, 0 },
    { 0xcfc19822, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab519825, 34 },
    { 0, 0 },
    { 0x2ca19827, 61 },
    { 0, 0 },
    { 0x95719829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c1982e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93319837, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e1983c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf19841, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea19844, 55 },
    { 0x7c519845, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a19848, 95 },
    { 0, 0 },
    { 0xbd1984a, 90 },
    { 0x1891984a, 34 },
    { 0x99e1984c, 17 },
    { 0xd521984d, 66 },
    { 0, 0 },
    { 0x5c1984f, 61 },
    { 0x1211984f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb919853, 52 },
    { 0, 0 },
    { 0x79f19855, 93 },
    { 0x43c19856, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb981985d, 98 },
    { 0x5091985e, 34 },
    { 0xc0b1985f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a419866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc441986d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e19871, 46 },
    { 0x72319872, 23 },
    { 0xd4a19873, 66 },
    { 0xca119872, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81919877, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf41987b, 48 },
    { 0x6c11987c, 61 },
    { 0, 0 },
    { 0x2d31987e, 61 },
    { 0, 0 },
    { 0x19e19880, 98 },
    { 0, 0 },
    { 0x23f19882, 93 },
    { 0xec519882, 98 },
    { 0, 0 },
    { 0x8d819885, 95 },
    { 0x25019886, 23 },
    { 0xbbe19887, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9451988e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec419892, 98 },
    { 0x37f19892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f19898, 55 },
    { 0xd719899, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9441989d, 76 },
    { 0, 0 },
    { 0x5121989f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570198a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f198a7, 59 },
    { 0xe15198a7, 33 },
    { 0x813198a7, 98 },
    { 0x191198aa, 78 },
    { 0xe3e198ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0198ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4198b1, 55 },
    { 0, 0 },
    { 0x579198b3, 90 },
    { 0x4e198b4, 33 },
    { 0x508198b3, 99 },
    { 0, 0 },
    { 0x344198b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4198ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f198be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645198c5, 61 },
    { 0xea0198c6, 23 },
    { 0, 0 },
    { 0xe6c198c8, 61 },
    { 0xd76198c9, 95 },
    { 0, 0 },
    { 0x20b198cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09198d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284198da, 23 },
    { 0xf74198da, 98 },
    { 0xa8b198dc, 98 },
    { 0, 0 },
    { 0x247198de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83198e1, 90 },
    { 0x607198e2, 76 },
    { 0xb69198e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0198ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d198f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8198f4, 90 },
    { 0x72c198f4, 95 },
    { 0x260198f6, 34 },
    { 0, 0 },
    { 0x9aa198f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07198fd, 67 },
    { 0xd0198fe, 23 },
    { 0, 0 },
    { 0x37219900, 61 },
    { 0x6e19900, 85 },
    { 0x65819901, 44 },
    { 0, 0 },
    { 0xcd919904, 90 },
    { 0x1b19904, 98 },
    { 0xb19906, 98 },
    { 0, 0 },
    { 0xf9e19908, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1211990c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8019910, 33 },
    { 0, 0 },
    { 0x84a19912, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f19915, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7819920, 76 },
    { 0, 0 },
    { 0x1fc19922, 55 },
    { 0x1b719923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42819929, 61 },
    { 0, 0 },
    { 0xae81992b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431992e, 40 },
    { 0, 0 },
    { 0xc5719930, 90 },
    { 0, 0 },
    { 0xbb219932, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d19937, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431993a, 76 },
    { 0xc511993b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b81993e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3919941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c71994b, 66 },
    { 0x3d01994b, 77 },
    { 0x19d1994d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0519951, 90 },
    { 0x7cf19952, 40 },
    { 0xaf19953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1219956, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e31995b, 61 },
    { 0, 0 },
    { 0x4ee1995d, 61 },
    { 0x7f71995e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef419962, 55 },
    { 0xb719962, 34 },
    { 0x4b319964, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf19967, 43 },
    { 0xc8a19968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1996b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e119971, 90 },
    { 0, 0 },
    { 0x27119973, 90 },
    { 0x13319973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97819977, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e81997b, 32 },
    { 0xdd91997c, 23 },
    { 0, 0 },
    { 0x9491997e, 89 },
    { 0x7c1997f, 55 },
    { 0x6181997f, 33 },
    { 0x83d19981, 67 },
    { 0x81e1997e, 90 },
    { 0x7801997f, 61 },
    { 0x6661997e, 33 },
    { 0x96819985, 98 },
    { 0x9dc19985, 90 },
    { 0x70519987, 66 },
    { 0x4b719987, 76 },
    { 0xff19985, 78 },
    { 0x28419985, 55 },
    { 0x41d1998b, 90 },
    { 0, 0 },
    { 0x9041998d, 95 },
    { 0x501998d, 33 },
    { 0x5b41998f, 98 },
    { 0x7019990, 87 },
    { 0x34f19991, 85 },
    { 0x3f119992, 17 },
    { 0x33719991, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69919998, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc1999c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac1999f, 98 },
    { 0xfa4199a0, 95 },
    { 0x90b199a0, 67 },
    { 0xbe2199a2, 66 },
    { 0x678199a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de199a9, 87 },
    { 0, 0 },
    { 0x714199ab, 96 },
    { 0x672199ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f199b0, 98 },
    { 0x73a199b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3199b4, 55 },
    { 0x670199b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1199c0, 61 },
    { 0x919199c0, 90 },
    { 0xcad199c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b199c6, 55 },
    { 0xe8e199c7, 61 },
    { 0xdbc199c8, 85 },
    { 0xb46199c6, 96 },
    { 0, 0 },
    { 0x9e3199cb, 33 },
    { 0x7d9199cb, 76 },
    { 0x2c4199cd, 61 },
    { 0xe1b199ce, 66 },
    { 0xc5f199cf, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7199d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0199d5, 67 },
    { 0, 0 },
    { 0xfcb199d7, 33 },
    { 0, 0 },
    { 0xefc199d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e199e0, 88 },
    { 0, 0 },
    { 0x789199e2, 43 },
    { 0x54b199e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0199e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d199ee, 26 },
    { 0x6bb199ef, 98 },
    { 0x639199ee, 95 },
    { 0x773199f1, 76 },
    { 0xa39199f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df199f6, 23 },
    { 0xcef199f6, 95 },
    { 0x4c5199f8, 43 },
    { 0x98f199f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee19a00, 85 },
    { 0x77419a00, 53 },
    { 0xa9e19a02, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8019a05, 61 },
    { 0xa7919a06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc19a0a, 88 },
    { 0xa6519a0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a19a11, 33 },
    { 0xf0819a11, 23 },
    { 0x8e619a13, 50 },
    { 0x8d419a11, 98 },
    { 0xe5819a15, 88 },
    { 0x8c119a15, 61 },
    { 0xe5119a16, 66 },
    { 0x51419a16, 58 },
    { 0xf119a15, 90 },
    { 0xa9019a11, 87 },
    { 0xc0919a14, 61 },
    { 0xd1d19a1c, 55 },
    { 0x54719a1b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c19a22, 23 },
    { 0x29c19a22, 23 },
    { 0x1f419a24, 87 },
    { 0x3a319a25, 95 },
    { 0, 0 },
    { 0xb9319a27, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x42019a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe619a2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb19a33, 98 },
    { 0, 0 },
    { 0x3f919a35, 55 },
    { 0x93519a36, 75 },
    { 0x6019a37, 90 },
    { 0x9919a38, 33 },
    { 0x71119a36, 90 },
    { 0xb2919a36, 90 },
    { 0x47719a3b, 98 },
    { 0xc919a39, 90 },
    { 0xa0219a3d, 99 },
    { 0xe2e19a3d, 93 },
    { 0x93f19a3f, 93 },
    { 0, 0 },
    { 0xe3919a41, 33 },
    { 0x34a19a42, 95 },
    { 0xc4719a41, 76 },
    { 0x64019a44, 23 },
    { 0, 0 },
    { 0x89b19a46, 61 },
    { 0x47119a47, 23 },
    { 0xd4f19a48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f019a4b, 61 },
    { 0x92a19a4b, 40 },
    { 0xf6c19a4d, 90 },
    { 0x8bb19a4e, 23 },
    { 0x23019a4f, 90 },
    { 0x2319a4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d19a55, 98 },
    { 0x61e19a56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4119a59, 52 },
    { 0xa919a59, 90 },
    { 0xd1419a59, 34 },
    { 0xac019a5c, 84 },
    { 0xf7619a5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a419a60, 90 },
    { 0, 0 },
    { 0xf8919a62, 78 },
    { 0x79319a63, 43 },
    { 0x97a19a62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f19a69, 66 },
    { 0x14219a69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c19a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f19a74, 100 },
    { 0x67419a75, 87 },
    { 0x9f219a74, 55 },
    { 0x4cb19a77, 43 },
    { 0, 0 },
    { 0xa6a19a79, 84 },
    { 0, 0 },
    { 0xf6219a7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f19a7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67219a82, 61 },
    { 0, 0 },
    { 0xf3a19a84, 90 },
    { 0x6c219a84, 95 },
    { 0, 0 },
    { 0xc3419a87, 91 },
    { 0x5c19a87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39519a8b, 69 },
    { 0, 0 },
    { 0x42119a8d, 85 },
    { 0x72019a8e, 33 },
    { 0x83619a8f, 95 },
    { 0x45819a90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4819a93, 43 },
    { 0xceb19a93, 90 },
    { 0, 0 },
    { 0x2b19a96, 90 },
    { 0, 0 },
    { 0xfb719a98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43219a9b, 95 },
    { 0, 0 },
    { 0xf0419a9d, 33 },
    { 0xf1319a9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61719aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f119aaa, 61 },
    { 0xa2019aab, 88 },
    { 0xbca19aaa, 90 },
    { 0x6a319aad, 95 },
    { 0xa4419aad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e019ab1, 93 },
    { 0x68219ab2, 33 },
    { 0x8e619ab3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a19ab9, 69 },
    { 0xc5219aba, 98 },
    { 0, 0 },
    { 0x5e19abc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc19abf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17619ac4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec19ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41719acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16519ad2, 90 },
    { 0x21d19ad3, 75 },
    { 0, 0 },
    { 0x87d19ad5, 77 },
    { 0xe6319ad6, 23 },
    { 0xab619ad6, 34 },
    { 0xace19ad8, 93 },
    { 0xa6719ad7, 90 },
    { 0x6b519ada, 65 },
    { 0x40a19adb, 91 },
    { 0x4e119adb, 99 },
    { 0xfcb19adb, 90 },
    { 0x5f119adc, 33 },
    { 0x1af19adf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b419ae5, 100 },
    { 0x1e619ae6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12619aea, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de19aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f419af2, 81 },
    { 0, 0 },
    { 0xc5119af4, 78 },
    { 0xc2419af5, 33 },
    { 0x48319af6, 95 },
    { 0x18b19af7, 66 },
    { 0, 0 },
    { 0xb8d19af9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67019b00, 61 },
    { 0, 0 },
    { 0x52719b02, 55 },
    { 0xb7019b02, 95 },
    { 0, 0 },
    { 0xc9119b05, 23 },
    { 0, 0 },
    { 0x80a19b07, 33 },
    { 0x85119b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9819b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f19b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d19b11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac19b1a, 61 },
    { 0xa2319b1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb19b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22719b28, 76 },
    { 0xb2919b29, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff19b2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e19b30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8619b36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f619b3c, 75 },
    { 0x4aa19b3d, 95 },
    { 0xe4019b3c, 95 },
    { 0x2b19b3f, 66 },
    { 0xed919b40, 90 },
    { 0x83b19b41, 77 },
    { 0xd6719b3e, 23 },
    { 0, 0 },
    { 0x79919b44, 95 },
    { 0x85319b45, 61 },
    { 0x37919b45, 46 },
    { 0x14f19b47, 91 },
    { 0x8b519b45, 40 },
    { 0, 0 },
    { 0x61e19b4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a419b4d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3919b52, 69 },
    { 0, 0 },
    { 0xb6f19b54, 33 },
    { 0, 0 },
    { 0x7da19b56, 93 },
    { 0x19d19b56, 76 },
    { 0xe3e19b58, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0719b5e, 76 },
    { 0, 0 },
    { 0x54e19b60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82519b66, 85 },
    { 0, 0 },
    { 0xf5519b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c19b6c, 46 },
    { 0x94819b6d, 55 },
    { 0x3a819b6d, 61 },
    { 0xdc919b6d, 68 },
    { 0x5bc19b70, 61 },
    { 0x4b319b6f, 84 },
    { 0xfa319b6d, 61 },
    { 0xe7019b73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a19b76, 33 },
    { 0, 0 },
    { 0xa6f19b78, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d919b7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde19b80, 95 },
    { 0x48219b81, 98 },
    { 0x2b119b82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede19b87, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e19b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa19b8d, 33 },
    { 0xd4519b8d, 17 },
    { 0, 0 },
    { 0xc1519b90, 98 },
    { 0, 0 },
    { 0x3819b92, 90 },
    { 0x7219b93, 100 },
    { 0x1f119b93, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x58319b97, 90 },
    { 0x4e719b98, 33 },
    { 0x22819b97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a19b9c, 61 },
    { 0x27019b9c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x52319ba0, 66 },
    { 0x5fa19ba1, 76 },
    { 0, 0 },
    { 0xc9c19ba3, 61 },
    { 0x61b19ba4, 55 },
    { 0, 0 },
    { 0x44519ba6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86619baa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2219bae, 43 },
    { 0, 0 },
    { 0xe7519bb0, 55 },
    { 0xd319bb1, 23 },
    { 0x7d319bb2, 78 },
    { 0xf5019bb0, 90 },
    { 0x97a19bb4, 66 },
    { 0xf9419bb5, 55 },
    { 0xbbd19bb4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e19bc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd19bc5, 33 },
    { 0, 0 },
    { 0xadd19bc7, 34 },
    { 0x78b19bc8, 99 },
    { 0, 0 },
    { 0xe9f19bca, 61 },
    { 0, 0 },
    { 0xe1b19bcc, 93 },
    { 0xe2819bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0319bd5, 77 },
    { 0x72519bd6, 95 },
    { 0x8c119bd7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b19bda, 95 },
    { 0x16519bdb, 95 },
    { 0xa0e19bda, 61 },
    { 0x36019bdd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b19be1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xca119be4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa019bea, 77 },
    { 0xcaa19bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f19bef, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31019bf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70319bfb, 98 },
    { 0x17919bfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a419c00, 99 },
    { 0x15019c01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8319c06, 99 },
    { 0, 0 },
    { 0x2c19c08, 61 },
    { 0x25e19c09, 23 },
    { 0, 0 },
    { 0xd6e19c0b, 90 },
    { 0x2be19c0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb319c0f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f19c1b, 83 },
    { 0xdcc19c1b, 43 },
    { 0, 0 },
    { 0x23819c1e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f19c21, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb19c24, 34 },
    { 0xf3c19c25, 61 },
    { 0x11119c26, 46 },
    { 0xb1a19c26, 99 },
    { 0xb4819c28, 23 },
    { 0x4c219c28, 55 },
    { 0x14f19c2a, 33 },
    { 0x65d19c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b919c2e, 61 },
    { 0xa9919c2f, 23 },
    { 0x6e519c2f, 34 },
    { 0xfe019c2e, 90 },
    { 0xcbd19c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a19c39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e819c41, 90 },
    { 0x31919c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea719c45, 91 },
    { 0xff919c45, 90 },
    { 0x3b519c47, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff19c4a, 23 },
    { 0x28119c4a, 17 },
    { 0, 0 },
    { 0xc8519c4d, 52 },
    { 0xdf919c4d, 85 },
    { 0x7e119c4f, 90 },
    { 0, 0 },
    { 0xa3e19c51, 99 },
    { 0x70819c52, 46 },
    { 0x3be19c53, 61 },
    { 0, 0 },
    { 0x35319c55, 90 },
    { 0, 0 },
    { 0x7b19c57, 77 },
    { 0x73a19c57, 95 },
    { 0xdd119c59, 52 },
    { 0x7b519c5a, 61 },
    { 0, 0 },
    { 0x79d19c5c, 61 },
    { 0x74b19c5d, 90 },
    { 0xb7719c5d, 55 },
    { 0x3b119c5e, 90 },
    { 0x17319c5f, 98 },
    { 0xafb19c61, 95 },
    { 0x82219c5f, 46 },
    { 0, 0 },
    { 0x15419c64, 98 },
    { 0, 0 },
    { 0xb1b19c66, 34 },
    { 0, 0 },
    { 0xbd319c68, 90 },
    { 0, 0 },
    { 0x17719c6a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb19c6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec19c70, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18519c75, 98 },
    { 0x54a19c76, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce19c7a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb619c7f, 46 },
    { 0x5f919c7f, 34 },
    { 0, 0 },
    { 0x1f419c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1519c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa19c8b, 55 },
    { 0x38119c8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd119c91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c519c94, 84 },
    { 0, 0 },
    { 0x21b19c96, 90 },
    { 0x53f19c97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3319c9c, 61 },
    { 0x3b419c9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db19ca0, 46 },
    { 0xa0519ca0, 23 },
    { 0x39619ca2, 61 },
    { 0, 0 },
    { 0x7db19ca4, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91719ca9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2619cac, 43 },
    { 0x16c19cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad19cb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda19cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86719cc1, 76 },
    { 0xee019cc2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d719cc5, 98 },
    { 0, 0 },
    { 0xe8719cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c19ccb, 90 },
    { 0x73519ccc, 69 },
    { 0xb9219ccc, 93 },
    { 0, 0 },
    { 0xd7f19ccf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1119cd4, 93 },
    { 0, 0 },
    { 0x14219cd6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x43919cd9, 61 },
    { 0xf6d19cda, 95 },
    { 0xf2119cdb, 46 },
    { 0xe0919cdb, 99 },
    { 0x81919cdc, 85 },
    { 0x7ec19cde, 99 },
    { 0x76a19cdc, 33 },
    { 0x56a19cdb, 23 },
    { 0xbad19cda, 24 },
    { 0x20d19ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b19ce9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b19cf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e19cf9, 95 },
    { 0, 0 },
    { 0x7c119cfb, 55 },
    { 0x43d19cfc, 93 },
    { 0x39819cfd, 85 },
    { 0xbc219cfd, 46 },
    { 0x51c19cfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c819d02, 33 },
    { 0, 0 },
    { 0x7ac19d04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e19d0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68419d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37119d13, 95 },
    { 0x7db19d13, 46 },
    { 0xdda19d15, 34 },
    { 0, 0 },
    { 0x5a019d17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a619d1b, 98 },
    { 0, 0 },
    { 0x32019d1d, 90 },
    { 0, 0 },
    { 0xf7819d1f, 87 },
    { 0xd1519d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51719d24, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2619d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be19d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26719d31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a19d35, 95 },
    { 0x91319d36, 77 },
    { 0x72519d35, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d319d3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3019d3d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f19d42, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea19d48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e219d4e, 33 },
    { 0x8f119d4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe19d53, 23 },
    { 0x2f619d54, 97 },
    { 0xd5019d54, 94 },
    { 0x82a19d56, 66 },
    { 0xfbc19d53, 95 },
    { 0xd0619d58, 80 },
    { 0, 0 },
    { 0xfce19d5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a19d5d, 99 },
    { 0x60619d5e, 95 },
    { 0, 0 },
    { 0xfd619d60, 61 },
    { 0xa1519d60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df19d66, 76 },
    { 0x49f19d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed19d6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec19d73, 97 },
    { 0, 0 },
    { 0xf0019d75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719d7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8119d81, 84 },
    { 0x77e19d82, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc19d87, 61 },
    { 0, 0 },
    { 0xb5319d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50119d8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3419d91, 52 },
    { 0x5f719d91, 90 },
    { 0x4b19d93, 33 },
    { 0x38719d94, 76 },
    { 0xaf219d95, 33 },
    { 0x98819d95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d19d9a, 87 },
    { 0, 0 },
    { 0x4f619d9c, 88 },
    { 0x13e19d9c, 23 },
    { 0x9da19d9c, 45 },
    { 0x49619d9f, 81 },
    { 0x60819d9f, 67 },
    { 0x33919da0, 38 },
    { 0x41719d9d, 61 },
    { 0, 0 },
    { 0x52b19da4, 75 },
    { 0x8f719da5, 61 },
    { 0x2ee19da4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a19dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a119daf, 90 },
    { 0xa1319db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c919db3, 92 },
    { 0x4a919db3, 34 },
    { 0x51519db5, 61 },
    { 0, 0 },
    { 0x71119db7, 95 },
    { 0x94419db8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c519dbb, 66 },
    { 0x8b19dbc, 90 },
    { 0, 0 },
    { 0x53219dbe, 61 },
    { 0, 0 },
    { 0x92419dc0, 90 },
    { 0, 0 },
    { 0x6e719dc2, 90 },
    { 0x7b719dc3, 61 },
    { 0x92d19dc4, 81 },
    { 0xf9a19dc4, 90 },
    { 0x10d19dc2, 101 },
    { 0x7fc19dc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf19dcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19dcf, 61 },
    { 0x11c19dd0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c19dd3, 100 },
    { 0, 0 },
    { 0x40d19dd5, 12 },
    { 0, 0 },
    { 0x54619dd7, 56 },
    { 0xfd019dd7, 99 },
    { 0x46719dd9, 95 },
    { 0x55e19dda, 23 },
    { 0x1bf19ddb, 76 },
    { 0, 0 },
    { 0x58119ddd, 93 },
    { 0xfc819ddd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf19de5, 43 },
    { 0x9f419de6, 76 },
    { 0, 0 },
    { 0x16719de8, 90 },
    { 0x54419de9, 90 },
    { 0, 0 },
    { 0x53c19deb, 64 },
    { 0xce819dec, 90 },
    { 0xa6619ded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23919df1, 87 },
    { 0, 0 },
    { 0xc8319df3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8419df6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b19df9, 61 },
    { 0xd7d19dfa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de19e07, 64 },
    { 0, 0 },
    { 0x42919e09, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c19e0d, 33 },
    { 0, 0 },
    { 0xc3c19e0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f19e13, 46 },
    { 0x2b19e14, 90 },
    { 0xa2919e15, 55 },
    { 0x2e219e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e019e1a, 52 },
    { 0xf6719e1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe419e1e, 33 },
    { 0, 0 },
    { 0x81519e20, 98 },
    { 0x8ce19e21, 76 },
    { 0, 0 },
    { 0xf8f19e23, 88 },
    { 0, 0 },
    { 0xe0e19e25, 98 },
    { 0x7f619e25, 90 },
    { 0x22019e27, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd819e2f, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25619e34, 78 },
    { 0x8619e35, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b19e38, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c319e3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57619e41, 98 },
    { 0, 0 },
    { 0x54519e43, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f119e52, 68 },
    { 0, 0 },
    { 0xda619e54, 98 },
    { 0x14e19e55, 61 },
    { 0x7ac19e55, 91 },
    { 0x83a19e55, 90 },
    { 0, 0 },
    { 0xea519e59, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc19e65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1119e6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4219e6f, 42 },
    { 0xa7219e70, 61 },
    { 0xba419e71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe419e74, 43 },
    { 0x96619e75, 76 },
    { 0x17419e76, 61 },
    { 0, 0 },
    { 0x75419e78, 52 },
    { 0, 0 },
    { 0xfa719e7a, 95 },
    { 0, 0 },
    { 0xfec19e7c, 87 },
    { 0, 0 },
    { 0x9ce19e7e, 98 },
    { 0x4f119e7f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e219e82, 43 },
    { 0x97e19e83, 98 },
    { 0x28c19e84, 97 },
    { 0, 0 },
    { 0x43a19e86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae19e8a, 95 },
    { 0xdcf19e8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b19e8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf819e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe19e95, 91 },
    { 0x32419e96, 65 },
    { 0xc7919e95, 75 },
    { 0x52219e98, 23 },
    { 0xd3919e98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4419ea0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff19ea8, 93 },
    { 0xd3b19ea8, 98 },
    { 0xc9719ea9, 33 },
    { 0, 0 },
    { 0x6c719eac, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x76019eaf, 90 },
    { 0xa3119eb0, 23 },
    { 0x43719eb1, 90 },
    { 0x34c19eb0, 90 },
    { 0xb6119eb0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89919eb8, 90 },
    { 0xd4019eb8, 78 },
    { 0x8c519eba, 76 },
    { 0, 0 },
    { 0x17819ebc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae619ec1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c219ec5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4519eca, 46 },
    { 0x9b619eca, 90 },
    { 0, 0 },
    { 0x6e419ecd, 33 },
    { 0xdf019ece, 81 },
    { 0xaa19ecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f19ed3, 46 },
    { 0, 0 },
    { 0xdde19ed5, 87 },
    { 0x67b19ed6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a19ede, 33 },
    { 0x11019ede, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52019ee2, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd19ee5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb19ee9, 43 },
    { 0, 0 },
    { 0x9da19eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b119eee, 76 },
    { 0, 0 },
    { 0xec19ef0, 33 },
    { 0x30d19ef1, 90 },
    { 0x8db19ef1, 95 },
    { 0, 0 },
    { 0xe6a19ef4, 98 },
    { 0, 0 },
    { 0x97619ef6, 46 },
    { 0x4f419ef7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f19efa, 34 },
    { 0x6b119efa, 93 },
    { 0xeab19efb, 90 },
    { 0x7f019efd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb119f00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28519f04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f719f08, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54419f0c, 90 },
    { 0xbc519f0d, 98 },
    { 0x3cb19f0e, 76 },
    { 0xb7919f0f, 61 },
    { 0x5bd19f10, 23 },
    { 0x4d319f11, 95 },
    { 0xb0519f0d, 61 },
    { 0xaae19f11, 61 },
    { 0x5a519f14, 33 },
    { 0x46b19f15, 69 },
    { 0x4ee19f14, 24 },
    { 0xf419f17, 90 },
    { 0, 0 },
    { 0x20319f19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61019f1e, 67 },
    { 0, 0 },
    { 0xb1919f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68019f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e719f2d, 85 },
    { 0x28819f2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f19f35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed19f38, 17 },
    { 0xb9219f38, 61 },
    { 0, 0 },
    { 0x5ff19f3b, 98 },
    { 0x1d319f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4219f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b419f46, 95 },
    { 0xbd519f47, 33 },
    { 0xf1519f46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc919f4c, 100 },
    { 0x56619f4d, 52 },
    { 0xa1319f4d, 99 },
    { 0xef619f4d, 66 },
    { 0, 0 },
    { 0xfdc19f51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5419f54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82919f57, 55 },
    { 0, 0 },
    { 0x2d019f59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd19f5f, 33 },
    { 0x8a319f60, 67 },
    { 0x1c19f60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2019f64, 90 },
    { 0, 0 },
    { 0xb7a19f66, 76 },
    { 0x21b19f66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65619f6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f19f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4719f75, 98 },
    { 0x35d19f76, 26 },
    { 0, 0 },
    { 0x2f419f78, 67 },
    { 0x66119f78, 55 },
    { 0x67219f7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3719f83, 33 },
    { 0xcff19f83, 23 },
    { 0x48819f85, 61 },
    { 0x90919f84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b19f89, 98 },
    { 0xca019f8a, 33 },
    { 0xf8819f8a, 78 },
    { 0x1b319f8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c19f91, 61 },
    { 0, 0 },
    { 0x37219f93, 99 },
    { 0, 0 },
    { 0xc1d19f95, 77 },
    { 0xff419f95, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b19f99, 90 },
    { 0xcff19f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7419f9d, 98 },
    { 0, 0 },
    { 0x8a019f9f, 76 },
    { 0x37f19fa0, 55 },
    { 0, 0 },
    { 0x1f919fa2, 90 },
    { 0x66f19fa2, 90 },
    { 0xa6219fa4, 90 },
    { 0x42619fa5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f19fa9, 33 },
    { 0, 0 },
    { 0x4ab19fab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b19fae, 61 },
    { 0x71d19faf, 52 },
    { 0xb2d19fb0, 83 },
    { 0x90319fae, 33 },
    { 0xaa919fb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb719fb5, 76 },
    { 0xd9d19fb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60419fbb, 23 },
    { 0x3b819fbc, 33 },
    { 0xcaa19fbb, 90 },
    { 0x6719fbc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f519fc3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x14819fc6, 95 },
    { 0xbf219fc7, 93 },
    { 0x70819fc7, 78 },
    { 0x25f19fc9, 90 },
    { 0xd5b19fc9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d19fcd, 61 },
    { 0x57419fce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef919fd4, 23 },
    { 0, 0 },
    { 0x2ea19fd6, 95 },
    { 0x41419fd7, 95 },
    { 0x18c19fd8, 23 },
    { 0xc4b19fd9, 55 },
    { 0, 0 },
    { 0x6bb19fdb, 95 },
    { 0x2dc19fdc, 85 },
    { 0x42b19fdd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x76019fe0, 81 },
    { 0x98919fe1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad19fe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a19fe7, 32 },
    { 0x2319fe8, 64 },
    { 0x29819fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0219fef, 78 },
    { 0x19d19fef, 76 },
    { 0x42819ff1, 43 },
    { 0x2f519ff1, 99 },
    { 0xf6119ff3, 78 },
    { 0xc0219fef, 99 },
    { 0, 0 },
    { 0xec319ff6, 33 },
    { 0, 0 },
    { 0x36c19ff8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b1a000, 68 },
    { 0, 0 },
    { 0x2a71a002, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb1a005, 35 },
    { 0, 0 },
    { 0xec1a007, 90 },
    { 0x7d01a007, 33 },
    { 0x5201a009, 65 },
    { 0x94f1a009, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf151a00d, 90 },
    { 0, 0 },
    { 0x8681a00f, 98 },
    { 0xa131a010, 95 },
    { 0x6b81a010, 93 },
    { 0x6de1a012, 91 },
    { 0, 0 },
    { 0xbee1a014, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a1a01a, 61 },
    { 0xac31a01b, 46 },
    { 0x4a61a01b, 46 },
    { 0xb8c1a01d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a1a020, 100 },
    { 0, 0 },
    { 0xcc71a022, 91 },
    { 0, 0 },
    { 0xa751a024, 90 },
    { 0xd8c1a024, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb721a028, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf161a033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe341a038, 43 },
    { 0x571a038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff1a046, 93 },
    { 0xb1b1a047, 61 },
    { 0x91f1a046, 95 },
    { 0, 0 },
    { 0x931a04a, 33 },
    { 0x6791a04b, 33 },
    { 0xe561a04c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f61a04f, 93 },
    { 0x6701a04f, 88 },
    { 0xe3a1a051, 98 },
    { 0x1b31a052, 69 },
    { 0x13b1a053, 61 },
    { 0xbff1a054, 55 },
    { 0x3f31a055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671a058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5821a05c, 88 },
    { 0, 0 },
    { 0x1641a05e, 66 },
    { 0xfe1a05f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b11a063, 85 },
    { 0xc921a064, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e1a06a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891a072, 61 },
    { 0xd761a073, 66 },
    { 0, 0 },
    { 0x2d31a075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771a078, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f1a080, 95 },
    { 0x2f81a080, 23 },
    { 0x14f1a082, 98 },
    { 0x55a1a082, 55 },
    { 0, 0 },
    { 0x8d61a085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e1a088, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1a08c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a21a099, 90 },
    { 0x5e11a099, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41a09d, 40 },
    { 0, 0 },
    { 0x2711a09f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe21a0a4, 98 },
    { 0, 0 },
    { 0x7931a0a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b1a0a9, 33 },
    { 0xba01a0aa, 90 },
    { 0xa1b1a0ab, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc991a0b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a0b7, 34 },
    { 0x8a1a0b8, 33 },
    { 0x2e51a0b9, 90 },
    { 0xa191a0ba, 90 },
    { 0x54c1a0bb, 33 },
    { 0xfec1a0ba, 61 },
    { 0, 0 },
    { 0x5221a0be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc671a0c2, 61 },
    { 0x1601a0c2, 95 },
    { 0, 0 },
    { 0x3a91a0c5, 98 },
    { 0xe51a0c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc91a0c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e1a0cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4471a0d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e1a0dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe701a0e0, 46 },
    { 0x51d1a0e1, 33 },
    { 0xd8f1a0e0, 76 },
    { 0x6561a0e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c1a0e8, 34 },
    { 0x2861a0e9, 34 },
    { 0, 0 },
    { 0x56d1a0eb, 90 },
    { 0xd591a0ec, 33 },
    { 0xc5f1a0ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda01a0f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b01a0f9, 90 },
    { 0x1941a0f9, 98 },
    { 0x6ab1a0fb, 76 },
    { 0x78e1a0fc, 76 },
    { 0x5831a0fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1a100, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x7271a103, 75 },
    { 0x1341a104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf291a10a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb571a116, 23 },
    { 0, 0 },
    { 0x4721a118, 78 },
    { 0xbd01a118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c01a11c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b11a123, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a128, 23 },
    { 0xcbe1a129, 78 },
    { 0, 0 },
    { 0x1c1a12b, 33 },
    { 0x1e41a12c, 61 },
    { 0x8f81a12d, 90 },
    { 0xf6d1a12e, 55 },
    { 0xc6a1a12e, 76 },
    { 0xbf91a130, 61 },
    { 0x99c1a12f, 23 },
    { 0xa1b1a12d, 84 },
    { 0xc601a12f, 23 },
    { 0xe951a134, 95 },
    { 0x59b1a135, 90 },
    { 0xaeb1a12f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71a139, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4061a13d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee1a144, 33 },
    { 0x961a145, 84 },
    { 0xd591a146, 33 },
    { 0x2041a145, 74 },
    { 0x2b21a148, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf1a14f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc1a154, 23 },
    { 0x411a154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc1a160, 34 },
    { 0x6e91a161, 98 },
    { 0xe51a162, 61 },
    { 0x1671a163, 98 },
    { 0x1d31a164, 84 },
    { 0x181a165, 87 },
    { 0, 0 },
    { 0x9791a167, 91 },
    { 0x9801a167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a16b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe181a171, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b41a175, 85 },
    { 0xad41a175, 52 },
    { 0x4f81a175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1771a17a, 86 },
    { 0x5e51a17b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3701a181, 87 },
    { 0xb8b1a182, 76 },
    { 0xabe1a183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c1a188, 98 },
    { 0x56d1a189, 17 },
    { 0xe131a189, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd11a191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad01a195, 23 },
    { 0xaa71a195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1521a19c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d1a1a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31a1a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3391a1a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da1a1ac, 95 },
    { 0x8c71a1ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4121a1b1, 55 },
    { 0x2241a1b2, 43 },
    { 0x3a51a1b2, 64 },
    { 0x4441a1b2, 90 },
    { 0, 0 },
    { 0x6e01a1b6, 95 },
    { 0x1b21a1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f81a1bb, 98 },
    { 0x7d11a1bc, 61 },
    { 0, 0 },
    { 0x18d1a1be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4621a1c1, 23 },
    { 0x451a1c2, 90 },
    { 0x4711a1c3, 90 },
    { 0x5e21a1c3, 68 },
    { 0xa131a1c3, 90 },
    { 0, 0 },
    { 0x56b1a1c7, 34 },
    { 0xb391a1c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a1a1cc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6301a1cf, 33 },
    { 0, 0 },
    { 0x8e51a1d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6371a1da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa1a1dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf511a1e0, 50 },
    { 0x9e41a1e1, 17 },
    { 0xdca1a1e1, 23 },
    { 0xe4b1a1e1, 90 },
    { 0, 0 },
    { 0x5561a1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e01a1e9, 91 },
    { 0x5611a1e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc1a1ed, 43 },
    { 0, 0 },
    { 0x9fc1a1ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf21a1f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5631a1f9, 95 },
    { 0, 0 },
    { 0x6451a1fb, 65 },
    { 0, 0 },
    { 0x60a1a1fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3851a206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6711a20a, 91 },
    { 0xf331a20b, 33 },
    { 0, 0 },
    { 0x4191a20d, 23 },
    { 0x1f11a20d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf711a212, 90 },
    { 0, 0 },
    { 0xe541a214, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1671a219, 33 },
    { 0x1b21a21a, 98 },
    { 0x8391a21b, 95 },
    { 0x91e1a21b, 23 },
    { 0x60c1a21c, 93 },
    { 0x9e31a21e, 90 },
    { 0xd5c1a21f, 76 },
    { 0, 0 },
    { 0x2731a221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe081a227, 84 },
    { 0x2781a228, 100 },
    { 0, 0 },
    { 0x2ee1a22a, 88 },
    { 0, 0 },
    { 0xab1a22c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b21a230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b1a238, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e1a23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x711a23f, 55 },
    { 0x961a23f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf1a243, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b31a248, 77 },
    { 0, 0 },
    { 0x9581a24a, 68 },
    { 0x3aa1a24a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1a250, 95 },
    { 0x4f01a251, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1a259, 91 },
    { 0, 0 },
    { 0xc3a1a25b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa691a25f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021a267, 93 },
    { 0xa221a267, 98 },
    { 0, 0 },
    { 0xa7b1a26a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e21a26f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae1a272, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41a276, 61 },
    { 0x71b1a277, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc531a27a, 98 },
    { 0, 0 },
    { 0x2d61a27c, 90 },
    { 0, 0 },
    { 0x5431a27e, 76 },
    { 0, 0 },
    { 0x2c51a280, 46 },
    { 0, 0 },
    { 0xa171a282, 52 },
    { 0x60f1a283, 33 },
    { 0x84a1a284, 23 },
    { 0x2701a283, 33 },
    { 0, 0 },
    { 0x7a1a287, 52 },
    { 0, 0 },
    { 0x26d1a289, 76 },
    { 0x8fd1a28a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa391a28d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41a290, 98 },
    { 0, 0 },
    { 0x9051a292, 90 },
    { 0, 0 },
    { 0x8d31a294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f1a298, 90 },
    { 0, 0 },
    { 0x91b1a29a, 90 },
    { 0xb541a29b, 52 },
    { 0x1591a29c, 33 },
    { 0xfc71a29b, 90 },
    { 0x5b91a29b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a61a2a2, 93 },
    { 0x17f1a2a2, 23 },
    { 0, 0 },
    { 0xfa61a2a5, 33 },
    { 0, 0 },
    { 0x9b01a2a7, 33 },
    { 0xb811a2a8, 94 },
    { 0x78c1a2a9, 88 },
    { 0x6b31a2aa, 90 },
    { 0xcf81a2a7, 61 },
    { 0xb941a2ac, 66 },
    { 0xc621a2ad, 90 },
    { 0x7031a2ae, 23 },
    { 0x65a1a2ae, 81 },
    { 0xf451a2ae, 87 },
    { 0x2461a2b1, 69 },
    { 0xf351a2b1, 61 },
    { 0xbe91a2b1, 87 },
    { 0x9101a2b4, 93 },
    { 0xc7a1a2b4, 90 },
    { 0x8471a2a9, 95 },
    { 0x1871a2b7, 33 },
    { 0xe291a2b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa841a2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5351a2c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e1a2c8, 85 },
    { 0x5a41a2c9, 23 },
    { 0, 0 },
    { 0x9841a2cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b1a2d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f1a2db, 93 },
    { 0xf9f1a2dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa111a2e1, 33 },
    { 0x25a1a2e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a1a2e5, 61 },
    { 0x1d01a2e6, 90 },
    { 0xc3b1a2e7, 61 },
    { 0x4431a2e8, 95 },
    { 0x36e1a2e6, 61 },
    { 0xb591a2ea, 93 },
    { 0x4c91a2e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf371a2f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e1a2f4, 76 },
    { 0x8e61a2f5, 61 },
    { 0x6a81a2f6, 90 },
    { 0xafd1a2f5, 33 },
    { 0, 0 },
    { 0x21c1a2f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071a2fc, 98 },
    { 0, 0 },
    { 0x6441a2fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f01a303, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb331a30a, 98 },
    { 0, 0 },
    { 0x5351a30c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9801a30f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a1a312, 99 },
    { 0x3761a313, 88 },
    { 0, 0 },
    { 0xdeb1a315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae51a31a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b1a31e, 65 },
    { 0, 0 },
    { 0x1411a320, 76 },
    { 0x7261a321, 61 },
    { 0x1191a322, 43 },
    { 0x9741a323, 88 },
    { 0xb9b1a323, 34 },
    { 0x85d1a325, 85 },
    { 0xb211a326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4521a32a, 55 },
    { 0xe161a32b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf1a330, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e1a337, 90 },
    { 0x4331a337, 93 },
    { 0x8211a338, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd891a341, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea01a34c, 85 },
    { 0x8111a34c, 46 },
    { 0xf0e1a34e, 36 },
    { 0x2851a34f, 97 },
    { 0x1631a350, 66 },
    { 0x52d1a350, 98 },
    { 0xebf1a350, 40 },
    { 0xbe31a351, 90 },
    { 0x4cc1a354, 5 },
    { 0, 0 },
    { 0x4c1a356, 78 },
    { 0x41a1a357, 33 },
    { 0x31f1a357, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781a35b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761a35e, 78 },
    { 0, 0 },
    { 0x94a1a360, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd781a365, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6451a36b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee1a36e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e1a376, 76 },
    { 0xe501a376, 91 },
    { 0xcc81a378, 34 },
    { 0x4701a379, 61 },
    { 0xe1c1a379, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b61a37d, 85 },
    { 0x9ae1a37e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4881a382, 33 },
    { 0x6e1a383, 88 },
    { 0xe1e1a384, 61 },
    { 0x2ef1a382, 90 },
    { 0x5c91a386, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca1a38e, 90 },
    { 0xdea1a38f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c1a397, 33 },
    { 0x9551a398, 90 },
    { 0, 0 },
    { 0x4a61a39a, 61 },
    { 0, 0 },
    { 0x7681a39c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9291a3a1, 83 },
    { 0x9051a3a2, 55 },
    { 0xbf71a3a3, 94 },
    { 0xc981a3a4, 90 },
    { 0, 0 },
    { 0xa2a1a3a6, 91 },
    { 0, 0 },
    { 0xedd1a3a8, 23 },
    { 0x3971a3a9, 84 },
    { 0x41b1a3a8, 43 },
    { 0xa5d1a3a8, 23 },
    { 0x6f11a3ac, 64 },
    { 0, 0 },
    { 0x6f61a3ae, 87 },
    { 0x9cf1a3ae, 91 },
    { 0, 0 },
    { 0x2031a3b1, 61 },
    { 0, 0 },
    { 0xbdf1a3b3, 76 },
    { 0x18c1a3b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa571a3b7, 76 },
    { 0x9451a3b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e11a3c0, 81 },
    { 0x51a1a3c0, 95 },
    { 0, 0 },
    { 0xf011a3c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd811a3c9, 34 },
    { 0x28c1a3ca, 55 },
    { 0x62b1a3cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b11a3ce, 61 },
    { 0, 0 },
    { 0x8571a3d0, 93 },
    { 0xff71a3d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101a3d5, 34 },
    { 0, 0 },
    { 0x4ae1a3d7, 33 },
    { 0xae61a3d8, 95 },
    { 0, 0 },
    { 0xd4d1a3da, 33 },
    { 0x48d1a3da, 43 },
    { 0x5c11a3da, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d1a3e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e31a3f2, 23 },
    { 0, 0 },
    { 0xfeb1a3f4, 33 },
    { 0xac31a3f4, 23 },
    { 0x15d1a3f4, 98 },
    { 0xddd1a3f7, 46 },
    { 0xa5f1a3f7, 90 },
    { 0xa5c1a3f5, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x5301a3fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda01a407, 99 },
    { 0, 0 },
    { 0xb8f1a409, 55 },
    { 0x1081a40a, 61 },
    { 0x2181a40a, 95 },
    { 0x4f61a40c, 98 },
    { 0xacf1a40c, 90 },
    { 0, 0 },
    { 0xbcd1a40f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b41a412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b91a417, 87 },
    { 0x4e21a418, 23 },
    { 0x64e1a418, 98 },
    { 0xcbe1a419, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e1a41d, 46 },
    { 0x8351a41d, 95 },
    { 0xd771a41f, 76 },
    { 0, 0 },
    { 0x8be1a421, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f1a426, 52 },
    { 0x3c81a426, 85 },
    { 0x13f1a427, 98 },
    { 0xd2d1a427, 55 },
    { 0x5ff1a42a, 76 },
    { 0x6c81a428, 95 },
    { 0, 0 },
    { 0x47c1a42d, 78 },
    { 0x6331a42d, 90 },
    { 0x1b51a42f, 95 },
    { 0, 0 },
    { 0xb561a431, 90 },
    { 0x621a431, 87 },
    { 0x2e81a431, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f61a436, 100 },
    { 0, 0 },
    { 0x2df1a438, 33 },
    { 0x7c91a439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee1a43f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a1a443, 33 },
    { 0x2161a443, 76 },
    { 0x471a443, 91 },
    { 0xf061a445, 77 },
    { 0xfee1a447, 90 },
    { 0, 0 },
    { 0xa631a449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc91a44c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e1a453, 61 },
    { 0xf721a454, 61 },
    { 0x3021a455, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d1a458, 87 },
    { 0, 0 },
    { 0xce11a45a, 90 },
    { 0, 0 },
    { 0x1701a45c, 55 },
    { 0, 0 },
    { 0x7ac1a45e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1a464, 55 },
    { 0, 0 },
    { 0xb6c1a466, 33 },
    { 0x2d21a467, 23 },
    { 0xa041a468, 23 },
    { 0x7301a467, 87 },
    { 0xaa81a467, 98 },
    { 0xa411a466, 23 },
    { 0xcd1a469, 76 },
    { 0xfa1a46d, 33 },
    { 0xd641a46d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe411a471, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e1a475, 91 },
    { 0xd11a475, 61 },
    { 0xc9f1a477, 33 },
    { 0xe5b1a477, 75 },
    { 0x5981a479, 65 },
    { 0x8071a475, 33 },
    { 0, 0 },
    { 0xbe21a47c, 33 },
    { 0xbcc1a47d, 90 },
    { 0, 0 },
    { 0x7461a47f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b1a482, 55 },
    { 0, 0 },
    { 0x981a484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191a48c, 90 },
    { 0, 0 },
    { 0xce01a48e, 33 },
    { 0x9831a48f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1a493, 98 },
    { 0x1c21a493, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe571a497, 61 },
    { 0x4151a498, 33 },
    { 0x71b1a498, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c1a4a1, 61 },
    { 0x3f91a4a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8991a4ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d61a4b1, 33 },
    { 0, 0 },
    { 0xb5e1a4b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae1a4b6, 33 },
    { 0x18f1a4b7, 90 },
    { 0x3481a4b6, 61 },
    { 0x1331a4b9, 43 },
    { 0x97e1a4b9, 95 },
    { 0xbd41a4b7, 90 },
    { 0xbb21a4bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f41a4c2, 90 },
    { 0x1cb1a4c2, 90 },
    { 0xc571a4c4, 23 },
    { 0x5181a4c5, 66 },
    { 0xe611a4c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf21a4ca, 95 },
    { 0, 0 },
    { 0xf801a4cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b11a4d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281a4d6, 84 },
    { 0xa4a1a4d7, 78 },
    { 0x5ac1a4d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1a4dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5621a4e2, 33 },
    { 0x1b41a4e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf251a4e7, 66 },
    { 0, 0 },
    { 0xb771a4e9, 78 },
    { 0x7ab1a4ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf11a4ee, 61 },
    { 0, 0 },
    { 0x34f1a4f0, 61 },
    { 0xe6d1a4f1, 97 },
    { 0, 0 },
    { 0x8a41a4f3, 61 },
    { 0x70f1a4f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5951a4f8, 76 },
    { 0xf811a4f9, 61 },
    { 0xde21a4f8, 93 },
    { 0xaee1a4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351a4fe, 76 },
    { 0x2411a4ff, 61 },
    { 0, 0 },
    { 0xd9b1a501, 88 },
    { 0x71c1a501, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc21a508, 95 },
    { 0x7491a509, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf071a50e, 85 },
    { 0x6e71a50f, 84 },
    { 0, 0 },
    { 0x6b01a511, 90 },
    { 0xdd71a512, 55 },
    { 0x45c1a512, 93 },
    { 0, 0 },
    { 0x29e1a515, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b91a51b, 43 },
    { 0, 0 },
    { 0x3111a51d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1831a521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2811a525, 66 },
    { 0, 0 },
    { 0x1081a527, 98 },
    { 0, 0 },
    { 0xa131a529, 90 },
    { 0x2261a52a, 78 },
    { 0x9371a52b, 90 },
    { 0x74a1a52a, 90 },
    { 0xc441a52c, 98 },
    { 0x8a41a529, 84 },
    { 0x5011a52a, 66 },
    { 0xcb51a52c, 99 },
    { 0, 0 },
    { 0x5021a532, 93 },
    { 0, 0 },
    { 0x7d21a534, 46 },
    { 0xfbb1a535, 51 },
    { 0, 0 },
    { 0xb211a537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851a53b, 43 },
    { 0x7131a53c, 95 },
    { 0, 0 },
    { 0x1421a53e, 98 },
    { 0x4651a53f, 34 },
    { 0xcdd1a540, 61 },
    { 0, 0 },
    { 0x52c1a542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd861a54b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab1a54e, 65 },
    { 0, 0 },
    { 0x3bc1a550, 93 },
    { 0xa521a550, 100 },
    { 0x8371a551, 33 },
    { 0xac41a553, 98 },
    { 0x3931a554, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b51a55a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d1a55d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3741a560, 33 },
    { 0, 0 },
    { 0xbf61a562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b1a56a, 91 },
    { 0x6a51a56a, 23 },
    { 0x1771a56c, 46 },
    { 0x8f61a56d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7951a572, 87 },
    { 0, 0 },
    { 0xebb1a574, 98 },
    { 0xf471a575, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e1a57d, 85 },
    { 0xcaa1a57d, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe061a589, 76 },
    { 0x6f11a58a, 98 },
    { 0xfa81a58b, 90 },
    { 0x4531a58c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f1a591, 93 },
    { 0, 0 },
    { 0x3ac1a593, 52 },
    { 0, 0 },
    { 0x74d1a595, 90 },
    { 0x451a596, 61 },
    { 0x9f91a597, 33 },
    { 0xda01a597, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f1a59d, 90 },
    { 0xaff1a59e, 78 },
    { 0xcfc1a59e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c01a5a2, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db1a5a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e21a5a9, 34 },
    { 0x13d1a5aa, 95 },
    { 0x6a91a5a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6311a5af, 23 },
    { 0, 0 },
    { 0x83c1a5b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc501a5b5, 33 },
    { 0x87b1a5b6, 43 },
    { 0xbbe1a5b6, 90 },
    { 0xe9d1a5b8, 33 },
    { 0, 0 },
    { 0xea41a5ba, 34 },
    { 0x5921a5bb, 33 },
    { 0, 0 },
    { 0x1391a5bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a5c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b21a5c4, 34 },
    { 0x8721a5c5, 90 },
    { 0x54d1a5c6, 34 },
    { 0xdfe1a5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c1a5cb, 76 },
    { 0, 0 },
    { 0xa011a5cd, 90 },
    { 0x67d1a5ce, 90 },
    { 0x40a1a5cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xef11a5d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de1a5d5, 99 },
    { 0, 0 },
    { 0xaad1a5d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391a5dc, 88 },
    { 0, 0 },
    { 0xab51a5de, 95 },
    { 0x521a5df, 95 },
    { 0x3e91a5e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa61a5e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7241a5e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de1a5ed, 55 },
    { 0x9271a5ed, 90 },
    { 0xa6b1a5ef, 93 },
    { 0xe201a5f0, 76 },
    { 0x7f81a5f0, 34 },
    { 0, 0 },
    { 0x9f91a5f3, 23 },
    { 0, 0 },
    { 0x2061a5f5, 46 },
    { 0x98b1a5f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1a5fc, 33 },
    { 0x7c11a5fd, 95 },
    { 0x98d1a5fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b71a601, 33 },
    { 0x45f1a602, 85 },
    { 0xe3d1a602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71a609, 76 },
    { 0xacb1a609, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1401a60e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb1a614, 90 },
    { 0x1511a614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba51a61a, 61 },
    { 0, 0 },
    { 0x9f81a61c, 55 },
    { 0, 0 },
    { 0x74f1a61e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc71a622, 38 },
    { 0xd21a623, 55 },
    { 0xdb31a624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a1a627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6311a62b, 95 },
    { 0xb8f1a62c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba1a630, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda1a637, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f21a63c, 98 },
    { 0, 0 },
    { 0x2501a63e, 46 },
    { 0xa3b1a63f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a71a645, 90 },
    { 0xb5b1a646, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f1a64b, 90 },
    { 0x1431a64b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf61a651, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3291a655, 33 },
    { 0xd1a656, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd21a65d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d1a661, 55 },
    { 0, 0 },
    { 0x3cb1a663, 23 },
    { 0xab61a664, 90 },
    { 0x7231a665, 95 },
    { 0, 0 },
    { 0xab91a667, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5511a66b, 51 },
    { 0x8181a66c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1951a66f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e1a674, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa821a678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341a67b, 33 },
    { 0, 0 },
    { 0x35f1a67d, 95 },
    { 0, 0 },
    { 0x1be1a67f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8691a682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf1a685, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe461a68a, 90 },
    { 0, 0 },
    { 0x5b21a68c, 33 },
    { 0x3461a68c, 76 },
    { 0x7881a68c, 90 },
    { 0xbf51a68e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c1a692, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b1a695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c1a69c, 23 },
    { 0x6731a69d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc1a6a2, 90 },
    { 0, 0 },
    { 0xad81a6a4, 93 },
    { 0xf7d1a6a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1a6a9, 55 },
    { 0xcbf1a6a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a6af, 90 },
    { 0, 0 },
    { 0xb351a6b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd531a6b4, 98 },
    { 0, 0 },
    { 0x3651a6b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e1a6ba, 90 },
    { 0x5771a6ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561a6bf, 55 },
    { 0, 0 },
    { 0x30d1a6c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a91a6c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b1a6d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x391a6d6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51a6d9, 75 },
    { 0, 0 },
    { 0x64e1a6db, 85 },
    { 0, 0 },
    { 0xb611a6dd, 84 },
    { 0x95d1a6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71a6e6, 75 },
    { 0x6561a6e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde91a6ef, 61 },
    { 0xc971a6f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca41a6f6, 46 },
    { 0xe4f1a6f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1a6fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb781a703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee1a708, 3 },
    { 0x3541a709, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb41a70c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa821a70f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa471a712, 61 },
    { 0xac81a713, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381a718, 99 },
    { 0xe101a719, 52 },
    { 0x1c81a71a, 92 },
    { 0x4791a71b, 98 },
    { 0x4ae1a71a, 87 },
    { 0xb2f1a71d, 99 },
    { 0x13d1a71e, 95 },
    { 0x5181a71f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a723, 87 },
    { 0x7c41a724, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1a727, 17 },
    { 0, 0 },
    { 0x1751a729, 85 },
    { 0xfa81a72a, 81 },
    { 0, 0 },
    { 0xb501a72c, 33 },
    { 0xf2f1a72c, 98 },
    { 0xf261a72d, 61 },
    { 0x1b21a72f, 61 },
    { 0xedb1a72d, 95 },
    { 0x46f1a72e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb871a739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d61a73e, 76 },
    { 0x4351a73f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c51a743, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac41a746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff1a749, 61 },
    { 0x6661a749, 61 },
    { 0x66f1a74b, 85 },
    { 0xd4a1a74c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f1a750, 23 },
    { 0, 0 },
    { 0xb671a752, 33 },
    { 0, 0 },
    { 0xe071a754, 98 },
    { 0x29e1a755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf731a758, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b1a75b, 88 },
    { 0xccc1a75b, 98 },
    { 0x28e1a75d, 46 },
    { 0x9871a75d, 34 },
    { 0x44b1a75e, 100 },
    { 0, 0 },
    { 0xdd1a761, 46 },
    { 0xe8e1a762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1701a766, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa841a76b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e1a771, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9371a776, 100 },
    { 0xd571a776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a1a77d, 98 },
    { 0x1481a77e, 52 },
    { 0x2b71a77e, 43 },
    { 0, 0 },
    { 0x9111a781, 23 },
    { 0xa831a782, 90 },
    { 0x3ff1a783, 100 },
    { 0, 0 },
    { 0xc991a785, 100 },
    { 0x78c1a786, 38 },
    { 0xcb21a785, 95 },
    { 0x53a1a788, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2581a78c, 23 },
    { 0xfb81a78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b71a790, 34 },
    { 0, 0 },
    { 0x3b81a792, 98 },
    { 0xc381a792, 33 },
    { 0xbc1a794, 23 },
    { 0x76f1a795, 78 },
    { 0x5591a795, 33 },
    { 0x1491a794, 95 },
    { 0xa7e1a798, 95 },
    { 0xa3d1a795, 61 },
    { 0x1251a79a, 33 },
    { 0x4ba1a79a, 90 },
    { 0x7e21a79a, 61 },
    { 0x6751a79c, 93 },
    { 0x3d81a79e, 33 },
    { 0x4b61a79f, 77 },
    { 0xef61a79c, 34 },
    { 0x4071a79b, 95 },
    { 0xa151a7a2, 95 },
    { 0xa931a7a0, 33 },
    { 0x83f1a79c, 76 },
    { 0, 0 },
    { 0x2881a7a6, 76 },
    { 0xe211a7a7, 33 },
    { 0x5cc1a7a8, 76 },
    { 0x5d41a7a9, 33 },
    { 0x6671a7aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11a7ad, 98 },
    { 0xea1a7ad, 33 },
    { 0xf81a7af, 75 },
    { 0x2501a7b0, 55 },
    { 0, 0 },
    { 0x55a1a7b2, 90 },
    { 0, 0 },
    { 0xb431a7b4, 52 },
    { 0, 0 },
    { 0x1d61a7b6, 87 },
    { 0x8ce1a7b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d1a7bc, 98 },
    { 0, 0 },
    { 0xb261a7be, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6691a7c3, 55 },
    { 0, 0 },
    { 0x79d1a7c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad81a7c9, 56 },
    { 0xa621a7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e1a7d4, 43 },
    { 0x7261a7d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a7db, 33 },
    { 0x7151a7dc, 78 },
    { 0x1a81a7dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9711a7e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e1a7e5, 95 },
    { 0x811a7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b41a7e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c1a7f1, 75 },
    { 0x9151a7f2, 76 },
    { 0x59d1a7f2, 33 },
    { 0x4f1a7f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe41a7ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e21a803, 33 },
    { 0x6e31a804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd691a807, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c1a80a, 98 },
    { 0, 0 },
    { 0x7871a80c, 19 },
    { 0x4dc1a80d, 55 },
    { 0x6051a80c, 52 },
    { 0xc3a1a80d, 43 },
    { 0x59a1a810, 90 },
    { 0x4461a811, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c11a816, 93 },
    { 0x5ec1a816, 87 },
    { 0xd241a818, 17 },
    { 0x8181a817, 65 },
    { 0x7571a81a, 33 },
    { 0xe51a81b, 23 },
    { 0x6df1a81c, 33 },
    { 0, 0 },
    { 0xb941a81e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c1a821, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481a82c, 61 },
    { 0x7171a82d, 98 },
    { 0x7f51a82c, 90 },
    { 0x31c1a82e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7801a832, 61 },
    { 0xac81a832, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb81a836, 98 },
    { 0x13e1a837, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3921a83b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc691a843, 61 },
    { 0x6d41a843, 55 },
    { 0x2be1a844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1a84a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc631a852, 36 },
    { 0xa0e1a853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf451a858, 95 },
    { 0x8321a858, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a61a85e, 90 },
    { 0x2421a85f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb81a86c, 90 },
    { 0x1441a86c, 55 },
    { 0x9b21a86e, 93 },
    { 0x6d11a86f, 46 },
    { 0x7981a86f, 23 },
    { 0x71a1a870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5481a878, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb221a87b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9851a880, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a1a884, 53 },
    { 0, 0 },
    { 0xe11a886, 95 },
    { 0, 0 },
    { 0x371a888, 87 },
    { 0xdea1a888, 65 },
    { 0xa361a88a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f71a891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e1a895, 56 },
    { 0x3911a896, 91 },
    { 0, 0 },
    { 0x6841a898, 78 },
    { 0x5cb1a898, 66 },
    { 0, 0 },
    { 0x8b71a89b, 90 },
    { 0x3ec1a89c, 33 },
    { 0x4271a89d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61a8a2, 34 },
    { 0xd551a8a2, 67 },
    { 0x30c1a8a4, 33 },
    { 0x7dc1a8a5, 23 },
    { 0x1d91a8a6, 96 },
    { 0x7511a8a4, 33 },
    { 0x5c01a8a2, 99 },
    { 0xdce1a8a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431a8b0, 23 },
    { 0xc621a8b0, 76 },
    { 0, 0 },
    { 0x7331a8b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c1a8b8, 23 },
    { 0x8ea1a8b9, 23 },
    { 0xdb31a8b8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce1a8bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b1a8c1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1a8c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5891a8ca, 98 },
    { 0xfb1a8ca, 34 },
    { 0x47e1a8cc, 23 },
    { 0, 0 },
    { 0xbae1a8ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab31a8d1, 99 },
    { 0, 0 },
    { 0xff11a8d3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa041a8db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3501a8e1, 53 },
    { 0x1f01a8e2, 33 },
    { 0, 0 },
    { 0x6cc1a8e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3351a8e7, 98 },
    { 0, 0 },
    { 0xae01a8e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f41a8ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d91a8f1, 46 },
    { 0xe141a8f2, 61 },
    { 0xfb81a8f2, 76 },
    { 0x99c1a8f4, 87 },
    { 0x3321a8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9271a8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d1a8fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e31a903, 87 },
    { 0x4351a903, 90 },
    { 0xdd31a905, 34 },
    { 0, 0 },
    { 0x6d21a907, 64 },
    { 0x8451a908, 33 },
    { 0x6a81a909, 90 },
    { 0x24b1a909, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b71a910, 95 },
    { 0xe061a911, 23 },
    { 0x1e71a911, 98 },
    { 0x25f1a913, 33 },
    { 0, 0 },
    { 0xe251a915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x701a918, 90 },
    { 0x6ab1a918, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd251a91c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea1a91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xede1a922, 33 },
    { 0xc0c1a923, 58 },
    { 0xa351a924, 85 },
    { 0x8091a925, 90 },
    { 0xc851a925, 98 },
    { 0xd811a927, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4261a92e, 95 },
    { 0xae11a92f, 98 },
    { 0, 0 },
    { 0x3761a931, 33 },
    { 0xa241a932, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8681a937, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521a942, 53 },
    { 0x6831a942, 23 },
    { 0xdc51a944, 84 },
    { 0x8dc1a944, 95 },
    { 0xa921a946, 90 },
    { 0, 0 },
    { 0x7e1a948, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9651a94b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe291a954, 33 },
    { 0xb231a954, 76 },
    { 0, 0 },
    { 0xbeb1a957, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e1a95b, 98 },
    { 0xe61a95c, 33 },
    { 0x2f71a95b, 76 },
    { 0x73a1a95d, 90 },
    { 0xf01a95d, 17 },
    { 0x8321a960, 76 },
    { 0xee81a960, 76 },
    { 0x9ad1a961, 90 },
    { 0x1b51a963, 23 },
    { 0xeb71a964, 91 },
    { 0x71d1a965, 61 },
    { 0x8be1a966, 98 },
    { 0, 0 },
    { 0x3041a968, 90 },
    { 0x9261a968, 61 },
    { 0, 0 },
    { 0x64d1a96b, 33 },
    { 0xb351a96c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11a970, 91 },
    { 0, 0 },
    { 0xf2d1a972, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c81a977, 98 },
    { 0xd1d1a978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe961a97e, 33 },
    { 0xddf1a97e, 91 },
    { 0xf281a97e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf71a986, 23 },
    { 0x5681a986, 95 },
    { 0xf961a988, 90 },
    { 0x7a21a986, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d1a98e, 98 },
    { 0x98e1a98f, 61 },
    { 0x2c71a990, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc11a993, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4031a996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a81a999, 43 },
    { 0xcbd1a99a, 93 },
    { 0x1e1a999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e1a9a2, 67 },
    { 0x3f61a9a2, 23 },
    { 0x1cc1a9a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd1a9a7, 90 },
    { 0xfad1a9a8, 98 },
    { 0xcb11a9a8, 95 },
    { 0x4751a9a9, 91 },
    { 0x9a41a9ab, 93 },
    { 0xf61a9ab, 98 },
    { 0x5ac1a9ad, 66 },
    { 0x541a9ae, 34 },
    { 0, 0 },
    { 0xafc1a9b0, 34 },
    { 0x4561a9b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c1a9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf31a9bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1411a9c0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2141a9c3, 90 },
    { 0x8a21a9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe851a9ca, 90 },
    { 0xafc1a9cb, 91 },
    { 0, 0 },
    { 0xb971a9cd, 90 },
    { 0xa4d1a9ce, 33 },
    { 0xafd1a9cf, 93 },
    { 0, 0 },
    { 0x8b1a9d1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8741a9d9, 46 },
    { 0xae81a9da, 46 },
    { 0x4c91a9d9, 83 },
    { 0x3731a9dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a1a9df, 95 },
    { 0, 0 },
    { 0xe321a9e1, 76 },
    { 0xb781a9e2, 90 },
    { 0xed1a9e1, 76 },
    { 0x2061a9e4, 33 },
    { 0xd0a1a9e5, 99 },
    { 0x4d51a9e2, 84 },
    { 0, 0 },
    { 0xfd21a9e8, 23 },
    { 0x6c51a9e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb141a9ec, 93 },
    { 0xa6b1a9ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1a9f6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a9ff, 43 },
    { 0, 0 },
    { 0x7691aa01, 78 },
    { 0xd091aa01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b1aa05, 43 },
    { 0xb581aa05, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd111aa0a, 33 },
    { 0x7551aa0b, 90 },
    { 0xa871aa0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df1aa10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3761aa15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b91aa1a, 95 },
    { 0xba1aa1b, 65 },
    { 0x8ba1aa1c, 76 },
    { 0x9cc1aa1b, 95 },
    { 0, 0 },
    { 0xbce1aa1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd1aa23, 99 },
    { 0xb131aa24, 78 },
    { 0, 0 },
    { 0x9b11aa26, 61 },
    { 0x6901aa26, 55 },
    { 0xcf11aa27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a1aa2b, 90 },
    { 0xcb31aa2c, 61 },
    { 0x8d71aa2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c1aa30, 33 },
    { 0, 0 },
    { 0x7ac1aa32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b41aa3a, 76 },
    { 0x26d1aa3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1621aa3e, 88 },
    { 0, 0 },
    { 0xe551aa40, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61aa43, 52 },
    { 0x4d31aa44, 34 },
    { 0, 0 },
    { 0x56a1aa46, 61 },
    { 0xa941aa46, 85 },
    { 0x3281aa48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe101aa4b, 33 },
    { 0x1f81aa4c, 78 },
    { 0x8841aa4d, 33 },
    { 0xc711aa4e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb091aa53, 90 },
    { 0, 0 },
    { 0x98d1aa55, 95 },
    { 0, 0 },
    { 0x16e1aa57, 61 },
    { 0xc9f1aa58, 34 },
    { 0x61b1aa57, 95 },
    { 0x57d1aa5a, 78 },
    { 0x3101aa5a, 55 },
    { 0, 0 },
    { 0x6d41aa5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a91aa61, 23 },
    { 0xc771aa62, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6341aa6a, 84 },
    { 0, 0 },
    { 0x4321aa6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d1aa70, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9021aa74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8961aa7c, 33 },
    { 0, 0 },
    { 0x7f31aa7e, 98 },
    { 0x7e1aa7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed31aa83, 68 },
    { 0x2851aa83, 80 },
    { 0xff41aa83, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221aa8a, 97 },
    { 0x2e31aa8b, 76 },
    { 0, 0 },
    { 0x3771aa8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa001aa93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a01aa96, 99 },
    { 0xaca1aa97, 97 },
    { 0xa251aa96, 66 },
    { 0x5381aa99, 33 },
    { 0x6dc1aa9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb51aa9f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b81aaa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191aaa5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1aaab, 5 },
    { 0xcf91aaac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac61aaaf, 88 },
    { 0, 0 },
    { 0xecd1aab1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2281aab6, 95 },
    { 0xdf31aab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4311aabb, 66 },
    { 0, 0 },
    { 0xb121aabd, 78 },
    { 0xb171aabe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e1aac1, 88 },
    { 0x2c51aac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb171aaca, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e21aacd, 95 },
    { 0, 0 },
    { 0x2cf1aacf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9141aad5, 43 },
    { 0x4f21aad6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd51aad9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb1aae4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61aaea, 95 },
    { 0, 0 },
    { 0x7801aaec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf931aaf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa41aaf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251aafa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b1aafe, 68 },
    { 0xa401aaff, 91 },
    { 0xa221ab00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41ab03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf671ab06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b51ab0d, 61 },
    { 0x69f1ab0e, 34 },
    { 0x7421ab0f, 34 },
    { 0xf761ab0e, 93 },
    { 0, 0 },
    { 0x92c1ab12, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb491ab16, 98 },
    { 0x3e61ab17, 98 },
    { 0, 0 },
    { 0xb301ab19, 98 },
    { 0xcca1ab19, 43 },
    { 0xcf1ab1b, 90 },
    { 0xde71ab1b, 77 },
    { 0xf2c1ab1c, 33 },
    { 0, 0 },
    { 0x5d11ab1f, 99 },
    { 0, 0 },
    { 0xd6a1ab21, 55 },
    { 0, 0 },
    { 0x9811ab23, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a1ab26, 76 },
    { 0xa601ab27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b1ab2a, 61 },
    { 0xecd1ab2a, 66 },
    { 0xd3e1ab2a, 76 },
    { 0, 0 },
    { 0x1971ab2e, 52 },
    { 0x77f1ab2e, 94 },
    { 0x7881ab30, 100 },
    { 0xb991ab31, 55 },
    { 0, 0 },
    { 0x9191ab33, 61 },
    { 0, 0 },
    { 0x2031ab35, 78 },
    { 0x3461ab35, 76 },
    { 0xbc41ab36, 67 },
    { 0x60c1ab36, 98 },
    { 0x6d91ab39, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc1ab3c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1ab41, 70 },
    { 0x16d1ab41, 95 },
    { 0, 0 },
    { 0xb381ab44, 95 },
    { 0x3a51ab45, 95 },
    { 0xd951ab46, 33 },
    { 0x3bc1ab47, 95 },
    { 0x7cc1ab48, 43 },
    { 0xbd1ab44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3811ab4c, 34 },
    { 0, 0 },
    { 0x3391ab4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac31ab52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa931ab55, 23 },
    { 0, 0 },
    { 0x2051ab57, 61 },
    { 0x211ab58, 61 },
    { 0x9461ab57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef1ab5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8701ab60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f1ab63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda1ab68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c61ab6d, 98 },
    { 0xd211ab6e, 11 },
    { 0x6321ab6f, 90 },
    { 0x14b1ab6f, 66 },
    { 0x75a1ab6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6831ab74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5021ab77, 90 },
    { 0xa491ab78, 95 },
    { 0xf8e1ab79, 95 },
    { 0xf7f1ab77, 76 },
    { 0x3831ab77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7221ab84, 33 },
    { 0xe341ab84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c1ab8c, 90 },
    { 0xab41ab8d, 98 },
    { 0xea81ab8e, 36 },
    { 0xa21ab8c, 91 },
    { 0x9671ab8d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe541ab95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7321ab9b, 61 },
    { 0xcad1ab9c, 94 },
    { 0, 0 },
    { 0x3841ab9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9881aba4, 43 },
    { 0x12a1aba4, 52 },
    { 0x5b31aba6, 88 },
    { 0x5241aba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1abad, 52 },
    { 0x6b51abae, 85 },
    { 0x22a1abaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9351abb2, 55 },
    { 0, 0 },
    { 0xec61abb4, 34 },
    { 0xeb81abb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6001abbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af1abc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c1abca, 95 },
    { 0x2f81abcb, 90 },
    { 0xf641abcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8971abd1, 93 },
    { 0, 0 },
    { 0xe001abd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961abd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1abdf, 90 },
    { 0, 0 },
    { 0x9a31abe1, 98 },
    { 0x6e31abe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e51abe7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8771abea, 93 },
    { 0x5761abea, 23 },
    { 0xb641abeb, 34 },
    { 0, 0 },
    { 0xf3b1abee, 55 },
    { 0xa951abef, 23 },
    { 0x8161abef, 90 },
    { 0x1b21abef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921abf6, 23 },
    { 0x7c21abf6, 66 },
    { 0xa11abf8, 90 },
    { 0x8131abf8, 33 },
    { 0, 0 },
    { 0xf8c1abfb, 100 },
    { 0xa3b1abfc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc1abff, 61 },
    { 0, 0 },
    { 0x3191ac01, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa271ac0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7551ac14, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71ac17, 90 },
    { 0xf971ac17, 61 },
    { 0, 0 },
    { 0xa091ac1a, 90 },
    { 0x8761ac1b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x901ac1e, 99 },
    { 0xba41ac1f, 97 },
    { 0xe641ac20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a1ac26, 88 },
    { 0x9011ac26, 87 },
    { 0xb031ac27, 78 },
    { 0xf721ac26, 98 },
    { 0x7591ac2a, 33 },
    { 0x35a1ac26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b31ac2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad1ac36, 33 },
    { 0x3571ac37, 98 },
    { 0, 0 },
    { 0x67a1ac39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad81ac3c, 61 },
    { 0, 0 },
    { 0x4331ac3e, 90 },
    { 0xcd31ac3f, 76 },
    { 0xba61ac3f, 98 },
    { 0x1301ac41, 46 },
    { 0, 0 },
    { 0x4171ac43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c31ac46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a71ac4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc1ac50, 34 },
    { 0, 0 },
    { 0x2a01ac52, 95 },
    { 0x2821ac53, 61 },
    { 0xb7b1ac54, 78 },
    { 0x3141ac55, 33 },
    { 0xda41ac56, 90 },
    { 0, 0 },
    { 0xdd31ac58, 90 },
    { 0, 0 },
    { 0xa581ac5a, 95 },
    { 0x7311ac5a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941ac5f, 23 },
    { 0x59c1ac5f, 30 },
    { 0x66b1ac60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e51ac65, 46 },
    { 0, 0 },
    { 0xe091ac67, 95 },
    { 0x3451ac67, 75 },
    { 0x5821ac69, 94 },
    { 0x9711ac6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe321ac76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f1ac7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c1ac7e, 40 },
    { 0xd541ac7e, 78 },
    { 0x90b1ac7f, 90 },
    { 0, 0 },
    { 0x8cb1ac82, 90 },
    { 0xcc81ac83, 33 },
    { 0, 0 },
    { 0x8d51ac85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b91ac89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d41ac8c, 55 },
    { 0, 0 },
    { 0xf191ac8e, 78 },
    { 0x75e1ac8f, 91 },
    { 0, 0 },
    { 0xb6f1ac91, 98 },
    { 0xc5d1ac92, 76 },
    { 0xd751ac92, 95 },
    { 0xcd31ac92, 98 },
    { 0, 0 },
    { 0x1641ac96, 59 },
    { 0xfec1ac97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4151ac9a, 78 },
    { 0, 0 },
    { 0x8471ac9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec1aca2, 91 },
    { 0, 0 },
    { 0xe8a1aca4, 55 },
    { 0x5941aca5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf411aca9, 95 },
    { 0, 0 },
    { 0x5f91acab, 33 },
    { 0xc601acac, 33 },
    { 0x16b1acac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc021acb0, 61 },
    { 0xd911acb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf121acb6, 23 },
    { 0x9da1acb6, 83 },
    { 0xe121acb8, 61 },
    { 0, 0 },
    { 0x2b1acba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa701acbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1acc1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba1acc4, 65 },
    { 0, 0 },
    { 0x38a1acc6, 67 },
    { 0xc3c1acc7, 46 },
    { 0xf101acc8, 61 },
    { 0x1a01acc7, 91 },
    { 0xffc1acc9, 61 },
    { 0x191acc6, 90 },
    { 0xe21acc8, 95 },
    { 0, 0 },
    { 0x15e1acce, 40 },
    { 0x9f91accf, 98 },
    { 0xf3d1accf, 95 },
    { 0x401accf, 33 },
    { 0, 0 },
    { 0x6f71acd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11acd6, 61 },
    { 0xc501acd7, 23 },
    { 0x9e61acd8, 100 },
    { 0x941acd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a71acdd, 90 },
    { 0, 0 },
    { 0xac91acdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb741ace8, 52 },
    { 0, 0 },
    { 0x9e11acea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4641aced, 98 },
    { 0x3dd1acee, 98 },
    { 0, 0 },
    { 0xa5f1acf0, 75 },
    { 0xa871acf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d1acf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b1acfb, 78 },
    { 0xef91acfb, 90 },
    { 0x3b91acfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc1ad06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d1ad0f, 90 },
    { 0xa1f1ad0f, 65 },
    { 0x1ad0f, 43 },
    { 0x8091ad12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991ad16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781ad1e, 95 },
    { 0xca21ad1e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e1ad25, 90 },
    { 0x4901ad25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4631ad2f, 90 },
    { 0x6121ad2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb841ad33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be1ad36, 61 },
    { 0, 0 },
    { 0x3101ad38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8601ad3c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb101ad47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3161ad4b, 43 },
    { 0, 0 },
    { 0xc521ad4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f1ad50, 61 },
    { 0, 0 },
    { 0x58a1ad52, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c1ad55, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a1ad5c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e21ad61, 91 },
    { 0x96e1ad62, 55 },
    { 0x8061ad63, 90 },
    { 0x3d81ad64, 90 },
    { 0x91d1ad65, 90 },
    { 0xcb91ad66, 61 },
    { 0x3cc1ad65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7841ad6c, 95 },
    { 0xdf1ad6d, 55 },
    { 0, 0 },
    { 0x26b1ad6f, 61 },
    { 0, 0 },
    { 0xa431ad71, 98 },
    { 0x9571ad71, 95 },
    { 0xd5a1ad73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa1ad76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe901ad7a, 23 },
    { 0x7b91ad7b, 23 },
    { 0xa3a1ad7c, 100 },
    { 0x311ad7a, 52 },
    { 0x7871ad7e, 98 },
    { 0x2e31ad7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa441ad84, 76 },
    { 0x2361ad84, 43 },
    { 0x81c1ad84, 78 },
    { 0xe6d1ad86, 76 },
    { 0xd421ad87, 61 },
    { 0xe71ad89, 52 },
    { 0x2c61ad89, 90 },
    { 0x7851ad85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4721ad93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb431ad99, 98 },
    { 0, 0 },
    { 0x93a1ad9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb1ad9e, 23 },
    { 0xc1e1ad9f, 98 },
    { 0xc6e1ada0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91ada3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a1ada7, 23 },
    { 0, 0 },
    { 0x8e41ada9, 34 },
    { 0x761adaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d1adb2, 95 },
    { 0, 0 },
    { 0x2181adb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1adb8, 90 },
    { 0, 0 },
    { 0x88d1adba, 76 },
    { 0x1b61adba, 33 },
    { 0, 0 },
    { 0x5f21adbd, 90 },
    { 0x9911adbd, 90 },
    { 0, 0 },
    { 0xa4e1adc0, 95 },
    { 0, 0 },
    { 0x2481adc2, 84 },
    { 0xea51adc2, 33 },
    { 0x8371adc4, 90 },
    { 0x4cb1adc5, 99 },
    { 0x4ad1adc6, 70 },
    { 0x7751adc5, 95 },
    { 0xd6a1adc8, 80 },
    { 0, 0 },
    { 0x8351adca, 92 },
    { 0xac01adcb, 98 },
    { 0, 0 },
    { 0xf3b1adcd, 91 },
    { 0x5f21adcd, 95 },
    { 0x5ad1adcf, 90 },
    { 0xbc51adcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2291add3, 90 },
    { 0, 0 },
    { 0xfba1add5, 99 },
    { 0x78d1add6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2071addf, 95 },
    { 0x89d1ade0, 33 },
    { 0x48f1addf, 95 },
    { 0xec1ade2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc1ade6, 61 },
    { 0, 0 },
    { 0x8af1ade8, 43 },
    { 0x37d1ade9, 90 },
    { 0x8ea1adea, 76 },
    { 0x50b1ade9, 93 },
    { 0xe11adec, 23 },
    { 0x701aded, 33 },
    { 0xb261adeb, 90 },
    { 0xd4a1aded, 84 },
    { 0x78b1adec, 61 },
    { 0xeb71adf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091adf7, 68 },
    { 0x3eb1adf7, 43 },
    { 0x75f1adf8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c91adfc, 85 },
    { 0, 0 },
    { 0xf1f1adfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9201ae01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c81ae08, 23 },
    { 0x651ae09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d1ae0d, 75 },
    { 0x10c1ae0e, 91 },
    { 0, 0 },
    { 0x6111ae10, 87 },
    { 0, 0 },
    { 0xeeb1ae12, 95 },
    { 0x3f01ae13, 93 },
    { 0xfe31ae14, 85 },
    { 0xabd1ae13, 100 },
    { 0, 0 },
    { 0x2391ae17, 2 },
    { 0x7ad1ae18, 46 },
    { 0xc291ae19, 91 },
    { 0x6a61ae18, 100 },
    { 0x90a1ae1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7051ae1f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf121ae22, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f1ae27, 17 },
    { 0x76e1ae28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b1ae2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e1ae30, 46 },
    { 0xd601ae31, 34 },
    { 0, 0 },
    { 0x56c1ae33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c1ae36, 98 },
    { 0, 0 },
    { 0x9801ae38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8341ae3c, 33 },
    { 0, 0 },
    { 0x95a1ae3e, 78 },
    { 0xcd61ae3f, 33 },
    { 0xe1f1ae40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4621ae43, 98 },
    { 0, 0 },
    { 0x1dc1ae45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a1ae4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd641ae53, 93 },
    { 0, 0 },
    { 0xf4c1ae55, 90 },
    { 0, 0 },
    { 0x2a91ae57, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xab51ae5a, 91 },
    { 0xfdb1ae5b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf1ae62, 33 },
    { 0xf121ae63, 90 },
    { 0xbfe1ae64, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5261ae69, 23 },
    { 0xd7b1ae69, 61 },
    { 0xff61ae6a, 90 },
    { 0x38d1ae6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d1ae79, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c1ae7c, 33 },
    { 0x9601ae7c, 87 },
    { 0, 0 },
    { 0x6731ae7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb1ae85, 76 },
    { 0x21e1ae85, 76 },
    { 0xac31ae86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7291ae97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d1ae9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d1ae9e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa1aea1, 98 },
    { 0, 0 },
    { 0xc2f1aea3, 91 },
    { 0xd6c1aea4, 49 },
    { 0x1f41aea4, 95 },
    { 0xb581aea6, 90 },
    { 0x1321aea4, 67 },
    { 0x4e51aea5, 23 },
    { 0x9071aea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4011aeb0, 99 },
    { 0xf9e1aeb1, 95 },
    { 0, 0 },
    { 0x87e1aeb3, 90 },
    { 0x4a41aeb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d31aeb8, 95 },
    { 0, 0 },
    { 0xa9e1aeba, 46 },
    { 0x2991aebb, 99 },
    { 0, 0 },
    { 0xaa41aebd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8271aec3, 91 },
    { 0xd701aec3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa11aec7, 90 },
    { 0x9321aec8, 90 },
    { 0xb351aec9, 95 },
    { 0xb191aec9, 78 },
    { 0x2521aec9, 95 },
    { 0x4bb1aecc, 33 },
    { 0x8371aecc, 95 },
    { 0xc391aecc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c71aed1, 61 },
    { 0x44e1aed2, 95 },
    { 0xca01aed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c21aede, 66 },
    { 0, 0 },
    { 0x8e61aee0, 34 },
    { 0xc391aee1, 84 },
    { 0x5d51aee2, 87 },
    { 0x3061aee1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c71aeee, 61 },
    { 0x4a11aeee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5871aef3, 95 },
    { 0x221aef4, 65 },
    { 0x4a51aef5, 33 },
    { 0xc361aef6, 90 },
    { 0x25f1aef6, 61 },
    { 0x29f1aef7, 95 },
    { 0xd4c1aef8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf541aefd, 90 },
    { 0xb8c1aefd, 90 },
    { 0x2391aefd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf81af02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1af05, 90 },
    { 0xbd51af05, 43 },
    { 0xc761af07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1af0a, 98 },
    { 0x1341af0b, 90 },
    { 0xc381af0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb151af0f, 33 },
    { 0, 0 },
    { 0x55c1af11, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1af14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa051af1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b1af26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3881af29, 33 },
    { 0, 0 },
    { 0xd7b1af2b, 61 },
    { 0, 0 },
    { 0x3791af2d, 84 },
    { 0xef41af2d, 33 },
    { 0, 0 },
    { 0x7ae1af30, 43 },
    { 0x4fe1af31, 55 },
    { 0xdfe1af30, 34 },
    { 0, 0 },
    { 0x48a1af34, 61 },
    { 0x94c1af35, 98 },
    { 0x80d1af36, 23 },
    { 0, 0 },
    { 0x1e61af38, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01af40, 93 },
    { 0x83b1af41, 93 },
    { 0x7f71af41, 23 },
    { 0x1b01af40, 65 },
    { 0xfe91af44, 33 },
    { 0x7721af44, 98 },
    { 0, 0 },
    { 0xf021af47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81af52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b1af57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e1af5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5741af5d, 86 },
    { 0xe721af5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c1af65, 91 },
    { 0, 0 },
    { 0x8901af67, 33 },
    { 0x9bc1af68, 55 },
    { 0xb241af68, 46 },
    { 0, 0 },
    { 0x77f1af6b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad1af6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc241af73, 87 },
    { 0xd6e1af74, 90 },
    { 0xf441af74, 76 },
    { 0xfee1af76, 76 },
    { 0, 0 },
    { 0xfbd1af78, 33 },
    { 0x1e31af79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4311af7d, 83 },
    { 0x17b1af7d, 99 },
    { 0x1b71af7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981af86, 95 },
    { 0x5f81af87, 55 },
    { 0xf371af86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c1af8e, 98 },
    { 0x60c1af8e, 95 },
    { 0x1e1af90, 43 },
    { 0x4bc1af90, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d1af94, 76 },
    { 0, 0 },
    { 0x5b51af96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5631af9b, 69 },
    { 0xb1d1af9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3991afa0, 23 },
    { 0xf9a1afa1, 95 },
    { 0xa801afa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2951afa5, 46 },
    { 0, 0 },
    { 0x5481afa7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f91afab, 90 },
    { 0x2111afab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d1afb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db1afb7, 43 },
    { 0, 0 },
    { 0x23d1afb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e61afbd, 67 },
    { 0x3351afbe, 46 },
    { 0, 0 },
    { 0x50d1afc0, 90 },
    { 0, 0 },
    { 0x4c01afc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5091afc6, 34 },
    { 0x5241afc7, 33 },
    { 0xe741afc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e1afd0, 98 },
    { 0, 0 },
    { 0x48c1afd2, 90 },
    { 0x1141afd3, 34 },
    { 0, 0 },
    { 0xd3f1afd5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf001afdb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b1afdf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a91afe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc651aff3, 84 },
    { 0, 0 },
    { 0x3551aff5, 90 },
    { 0xc821aff6, 95 },
    { 0x2541aff7, 91 },
    { 0x5241aff8, 98 },
    { 0, 0 },
    { 0x1e51affa, 87 },
    { 0x77f1affa, 61 },
    { 0x9401affc, 61 },
    { 0, 0 },
    { 0xa641affe, 98 },
    { 0xbea1affe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c11b007, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821b011, 95 },
    { 0, 0 },
    { 0xd851b013, 23 },
    { 0, 0 },
    { 0xa921b015, 33 },
    { 0, 0 },
    { 0xfc21b017, 90 },
    { 0x9b41b017, 39 },
    { 0, 0 },
    { 0x4a31b01a, 33 },
    { 0x60a1b01b, 91 },
    { 0x4dd1b01a, 33 },
    { 0, 0 },
    { 0xf951b01e, 34 },
    { 0x97e1b01e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c1b024, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc141b02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c31b036, 75 },
    { 0x9711b037, 33 },
    { 0xf811b038, 90 },
    { 0x2821b037, 90 },
    { 0xbd01b03a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d1b041, 87 },
    { 0x9d71b042, 23 },
    { 0x98a1b043, 98 },
    { 0xd901b044, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b1b048, 75 },
    { 0x9ea1b048, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d91b051, 33 },
    { 0, 0 },
    { 0x2291b053, 98 },
    { 0x63e1b054, 43 },
    { 0x8f51b055, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5861b05a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc61b05f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc181b062, 66 },
    { 0xe6d1b063, 90 },
    { 0xa841b064, 61 },
    { 0x5771b065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a1b068, 34 },
    { 0, 0 },
    { 0x9031b06a, 61 },
    { 0x46c1b06b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b1b071, 61 },
    { 0, 0 },
    { 0x30d1b073, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e1b076, 91 },
    { 0, 0 },
    { 0x6081b078, 33 },
    { 0x5631b079, 98 },
    { 0, 0 },
    { 0xb8a1b07b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb11b081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c61b084, 76 },
    { 0, 0 },
    { 0x83f1b086, 91 },
    { 0xeaa1b087, 33 },
    { 0xde71b088, 95 },
    { 0x2dc1b089, 55 },
    { 0x6351b089, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb241b08d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf721b09e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b0a2, 23 },
    { 0x2441b0a3, 78 },
    { 0, 0 },
    { 0x1c71b0a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1731b0a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9441b0ab, 23 },
    { 0x11e1b0ac, 33 },
    { 0xc3c1b0ad, 40 },
    { 0x3c31b0ae, 43 },
    { 0x3371b0af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb41b0b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c1b0b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b21b0b8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2841b0bc, 67 },
    { 0x9441b0bc, 93 },
    { 0x3e71b0bd, 46 },
    { 0x2c71b0bf, 95 },
    { 0x3c51b0be, 43 },
    { 0x67d1b0c1, 99 },
    { 0xb8b1b0c2, 95 },
    { 0xe1a1b0c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc01b0c6, 23 },
    { 0x5c41b0c7, 46 },
    { 0x59c1b0c8, 61 },
    { 0, 0 },
    { 0x9fe1b0ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d1b0ce, 43 },
    { 0xf81b0cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f1b0d2, 33 },
    { 0x3131b0d2, 61 },
    { 0x1051b0d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1b0d8, 90 },
    { 0, 0 },
    { 0x9b1b0da, 90 },
    { 0xa991b0db, 82 },
    { 0xb451b0dc, 23 },
    { 0xecd1b0dc, 95 },
    { 0xce71b0de, 96 },
    { 0x4f11b0de, 81 },
    { 0x58e1b0e0, 75 },
    { 0, 0 },
    { 0xee1b0e2, 33 },
    { 0x8f31b0e3, 99 },
    { 0x5861b0e4, 46 },
    { 0x6821b0e5, 87 },
    { 0x8c31b0e4, 55 },
    { 0, 0 },
    { 0x6751b0e8, 33 },
    { 0x88d1b0e8, 23 },
    { 0, 0 },
    { 0x2ae1b0eb, 76 },
    { 0, 0 },
    { 0x10c1b0ed, 85 },
    { 0x4651b0ed, 76 },
    { 0xa141b0ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1b0f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b0f6, 23 },
    { 0, 0 },
    { 0x9f61b0f8, 91 },
    { 0, 0 },
    { 0x2361b0fa, 78 },
    { 0x1121b0fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3451b0ff, 51 },
    { 0x6ce1b0ff, 23 },
    { 0, 0 },
    { 0x5a31b102, 47 },
    { 0, 0 },
    { 0xdf11b104, 40 },
    { 0xed91b105, 67 },
    { 0x5af1b106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e1b109, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a91b10d, 23 },
    { 0, 0 },
    { 0xf721b10f, 90 },
    { 0, 0 },
    { 0xedb1b111, 52 },
    { 0xd851b112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d61b117, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8391b11a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8521b11e, 95 },
    { 0xd0f1b11f, 76 },
    { 0xac31b11f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3521b124, 33 },
    { 0x3631b124, 66 },
    { 0xf341b126, 61 },
    { 0xd051b124, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991b12b, 66 },
    { 0x24b1b12c, 90 },
    { 0, 0 },
    { 0x6de1b12e, 55 },
    { 0xec61b12f, 61 },
    { 0, 0 },
    { 0xdb21b131, 66 },
    { 0x20a1b132, 76 },
    { 0, 0 },
    { 0xe3c1b134, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb31b137, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4751b13d, 95 },
    { 0x5871b13e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6221b143, 52 },
    { 0, 0 },
    { 0x2481b145, 91 },
    { 0x5d61b146, 91 },
    { 0x9c01b145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7411b14a, 90 },
    { 0x58d1b14b, 93 },
    { 0, 0 },
    { 0xc901b14d, 61 },
    { 0x3a51b14d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b151, 46 },
    { 0x1201b151, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd1b155, 87 },
    { 0xdce1b155, 66 },
    { 0x801b155, 55 },
    { 0xd231b157, 48 },
    { 0x5091b159, 98 },
    { 0xb8f1b15a, 87 },
    { 0x58e1b15a, 61 },
    { 0xe8e1b15c, 78 },
    { 0x84a1b156, 76 },
    { 0x8d51b15e, 100 },
    { 0x9d91b15e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8071b164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b1b167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81b175, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251b179, 98 },
    { 0, 0 },
    { 0x5951b17b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521b17f, 23 },
    { 0, 0 },
    { 0xde61b181, 55 },
    { 0, 0 },
    { 0x5e21b183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631b187, 43 },
    { 0, 0 },
    { 0xb321b189, 17 },
    { 0, 0 },
    { 0x5ca1b18b, 95 },
    { 0x1a31b18b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821b190, 33 },
    { 0, 0 },
    { 0x6b81b192, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xade1b195, 61 },
    { 0xfa81b196, 84 },
    { 0x8301b197, 33 },
    { 0x6d41b197, 23 },
    { 0xe291b199, 78 },
    { 0x9be1b199, 66 },
    { 0xdd71b19b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c71b1a1, 93 },
    { 0x65a1b1a1, 33 },
    { 0x7561b1a3, 90 },
    { 0, 0 },
    { 0x3981b1a5, 46 },
    { 0xcb11b1a5, 87 },
    { 0xaca1b1a7, 52 },
    { 0x93a1b1a5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x7901b1ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71b1ae, 33 },
    { 0x9251b1ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4251b1b5, 93 },
    { 0xaa41b1b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f01b1b9, 56 },
    { 0x7b91b1ba, 90 },
    { 0x6591b1b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa281b1be, 76 },
    { 0, 0 },
    { 0xea61b1c0, 98 },
    { 0, 0 },
    { 0x4201b1c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1121b1c6, 93 },
    { 0, 0 },
    { 0xec91b1c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7591b1d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c1b1d5, 66 },
    { 0, 0 },
    { 0xe3b1b1d7, 33 },
    { 0, 0 },
    { 0x6611b1d9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f1b1dc, 98 },
    { 0x81a1b1dc, 23 },
    { 0x1781b1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a31b1e1, 55 },
    { 0xc0b1b1e1, 34 },
    { 0x6981b1e3, 95 },
    { 0, 0 },
    { 0xd1b1e5, 99 },
    { 0xc71b1e5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7071b1eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad1b1f1, 43 },
    { 0xaba1b1f1, 38 },
    { 0, 0 },
    { 0x5e71b1f4, 23 },
    { 0x2391b1f5, 33 },
    { 0xc7e1b1f6, 90 },
    { 0xf641b1f6, 61 },
    { 0xc7e1b1f5, 98 },
    { 0, 0 },
    { 0x1861b1fa, 90 },
    { 0, 0 },
    { 0xa461b1fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6401b200, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e1b20b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd021b20f, 43 },
    { 0xcca1b20f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe631b214, 90 },
    { 0xe571b215, 46 },
    { 0xaf61b216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa131b21c, 43 },
    { 0x70d1b21d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb031b225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9871b22b, 33 },
    { 0x8441b22b, 90 },
    { 0x4861b22c, 98 },
    { 0x7fd1b22e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6951b231, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a1b234, 61 },
    { 0, 0 },
    { 0xf5a1b236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca01b239, 33 },
    { 0, 0 },
    { 0x5491b23b, 84 },
    { 0x7d61b23b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9381b241, 43 },
    { 0x1811b242, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa211b245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1b24f, 88 },
    { 0x1411b250, 78 },
    { 0x66e1b250, 61 },
    { 0x6f51b251, 43 },
    { 0xfe11b253, 46 },
    { 0x4a21b24f, 90 },
    { 0x14d1b252, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3891b258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd681b25b, 34 },
    { 0, 0 },
    { 0x90a1b25d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb1b261, 33 },
    { 0x1ea1b262, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071b268, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa631b26b, 78 },
    { 0xedb1b26c, 95 },
    { 0xed81b26c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11b271, 46 },
    { 0x1ba1b271, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd611b275, 23 },
    { 0x5c01b276, 33 },
    { 0xa9d1b276, 80 },
    { 0x3cf1b278, 23 },
    { 0xaba1b276, 98 },
    { 0x8ba1b27a, 66 },
    { 0, 0 },
    { 0xb201b27c, 61 },
    { 0x4271b27d, 43 },
    { 0, 0 },
    { 0xc011b27f, 90 },
    { 0xba81b27f, 66 },
    { 0xd811b281, 98 },
    { 0xb3c1b280, 61 },
    { 0xa5c1b283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d1b28d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb81b290, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf91b293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a1b296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac1b29a, 23 },
    { 0xaa81b29a, 66 },
    { 0xc401b29c, 90 },
    { 0x4411b29d, 92 },
    { 0x551b29e, 3 },
    { 0xd3e1b29e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3261b2a2, 90 },
    { 0xc011b2a2, 90 },
    { 0, 0 },
    { 0x17b1b2a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f21b2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf781b2b1, 46 },
    { 0xb4d1b2b2, 61 },
    { 0x8bc1b2b2, 91 },
    { 0xae41b2b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec11b2b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e71b2c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd11b2cc, 90 },
    { 0, 0 },
    { 0xdfa1b2ce, 23 },
    { 0x59f1b2cf, 78 },
    { 0, 0 },
    { 0x19d1b2d1, 98 },
    { 0, 0 },
    { 0xad01b2d3, 85 },
    { 0, 0 },
    { 0x5ec1b2d5, 33 },
    { 0xbd1b2d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91b2d9, 95 },
    { 0xfa91b2d9, 90 },
    { 0x6ba1b2db, 98 },
    { 0, 0 },
    { 0x7501b2dd, 80 },
    { 0, 0 },
    { 0xa8b1b2df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb901b2e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c1b2e7, 77 },
    { 0x2191b2e8, 90 },
    { 0x8401b2e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce1b2ed, 90 },
    { 0x2661b2ed, 61 },
    { 0xbfe1b2ee, 90 },
    { 0, 0 },
    { 0x4a1b2f1, 46 },
    { 0xb5e1b2f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161b2f5, 52 },
    { 0x3d91b2f6, 90 },
    { 0, 0 },
    { 0x98c1b2f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1b2fc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1b301, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1111b306, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc931b30a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde31b30d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6721b310, 95 },
    { 0, 0 },
    { 0x3321b312, 90 },
    { 0xe8d1b313, 55 },
    { 0x25f1b312, 65 },
    { 0, 0 },
    { 0x43a1b316, 46 },
    { 0x1bd1b317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e1b31b, 90 },
    { 0xa141b31c, 76 },
    { 0x231b31c, 98 },
    { 0, 0 },
    { 0x7ea1b31f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d51b323, 23 },
    { 0, 0 },
    { 0x2881b325, 96 },
    { 0xbc41b325, 23 },
    { 0xcc11b327, 85 },
    { 0x9b01b328, 98 },
    { 0xbd01b327, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b21b330, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe501b335, 75 },
    { 0, 0 },
    { 0x10b1b337, 33 },
    { 0xbf81b337, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4511b33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c1b345, 55 },
    { 0, 0 },
    { 0xda11b347, 33 },
    { 0, 0 },
    { 0x7f81b349, 52 },
    { 0x6c21b349, 33 },
    { 0x9501b34b, 61 },
    { 0xe741b34c, 90 },
    { 0xc181b34b, 95 },
    { 0xf881b34a, 90 },
    { 0x8d31b34f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce1b357, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf1b35a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f1b35d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521b361, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb41b368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b1b36b, 23 },
    { 0x34e1b36c, 33 },
    { 0xe421b36b, 33 },
    { 0, 0 },
    { 0x62c1b36f, 90 },
    { 0, 0 },
    { 0x3381b371, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4691b379, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb311b37c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc31b37f, 76 },
    { 0x9dc1b380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1b387, 17 },
    { 0x49f1b387, 24 },
    { 0x8191b389, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81b392, 68 },
    { 0x1ef1b392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a1b399, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x941b39c, 98 },
    { 0xb161b39d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf961b3a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e41b3a6, 61 },
    { 0xeef1b3a6, 61 },
    { 0, 0 },
    { 0xadd1b3a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe501b3ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd991b3b1, 87 },
    { 0x11b1b3b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4301b3b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1b3ba, 43 },
    { 0xc481b3bb, 61 },
    { 0x6241b3bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa591b3bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b61b3c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b1b3c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5131b3ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa651b3ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b3d1, 87 },
    { 0, 0 },
    { 0x23b1b3d3, 36 },
    { 0, 0 },
    { 0x9491b3d5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d1b3db, 90 },
    { 0x9ff1b3dc, 55 },
    { 0x57a1b3db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1b3e5, 76 },
    { 0x5031b3e6, 78 },
    { 0xb6e1b3e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371b3eb, 81 },
    { 0x9c61b3eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa91b3f2, 85 },
    { 0x8a41b3f3, 95 },
    { 0x3761b3f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251b3f9, 61 },
    { 0x3a11b3fa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe341b3ff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e1b407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f71b414, 90 },
    { 0xd2c1b414, 52 },
    { 0x6c51b416, 99 },
    { 0, 0 },
    { 0xd311b418, 87 },
    { 0, 0 },
    { 0x9571b41a, 95 },
    { 0xdd41b41b, 33 },
    { 0xea71b41b, 90 },
    { 0xa001b41c, 93 },
    { 0x3401b41c, 100 },
    { 0x4911b41f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a11b424, 43 },
    { 0x7a51b424, 61 },
    { 0xe081b426, 51 },
    { 0x6ab1b426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9311b42d, 90 },
    { 0xbab1b42e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee41b435, 33 },
    { 0, 0 },
    { 0xd091b437, 95 },
    { 0, 0 },
    { 0xb8c1b439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3821b447, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901b44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b31b450, 76 },
    { 0xf811b451, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21b457, 33 },
    { 0x6af1b458, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11b461, 90 },
    { 0x7d71b462, 98 },
    { 0x84b1b463, 95 },
    { 0xd0c1b464, 33 },
    { 0x5c71b465, 95 },
    { 0x5001b463, 61 },
    { 0x36b1b463, 55 },
    { 0x7211b467, 34 },
    { 0x5c11b464, 76 },
    { 0, 0 },
    { 0xa381b46b, 88 },
    { 0x9511b46b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9411b477, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2491b47b, 98 },
    { 0xf51b47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf1b47f, 40 },
    { 0x6d31b480, 87 },
    { 0xa981b480, 68 },
    { 0xe321b482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb1b485, 76 },
    { 0, 0 },
    { 0xf741b487, 34 },
    { 0xcff1b488, 23 },
    { 0, 0 },
    { 0xc521b48a, 33 },
    { 0x1bb1b48a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b1b490, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b493, 90 },
    { 0xab21b494, 95 },
    { 0x52d1b493, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1301b498, 100 },
    { 0x8db1b499, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc291b49e, 90 },
    { 0, 0 },
    { 0xef61b4a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc611b4a6, 43 },
    { 0, 0 },
    { 0x5ae1b4a8, 48 },
    { 0xd221b4a9, 33 },
    { 0, 0 },
    { 0xd8b1b4ab, 22 },
    { 0x3cb1b4ac, 93 },
    { 0x16a1b4ac, 55 },
    { 0, 0 },
    { 0x1fc1b4af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f1b4b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1771b4b9, 98 },
    { 0xe721b4b9, 34 },
    { 0xdd71b4bb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7311b4c1, 95 },
    { 0xfdb1b4c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e11b4c7, 43 },
    { 0x2431b4c8, 68 },
    { 0x7781b4c7, 99 },
    { 0x60e1b4c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b1b4cd, 23 },
    { 0xd1e1b4ce, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a1b4d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f71b4d8, 74 },
    { 0xf6c1b4d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee41b4dd, 33 },
    { 0, 0 },
    { 0x37b1b4df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f61b4e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871b4ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb01b4f3, 90 },
    { 0xa6f1b4f4, 76 },
    { 0, 0 },
    { 0x37f1b4f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a51b4fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac71b4fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f1b502, 99 },
    { 0x3de1b502, 84 },
    { 0x1871b503, 98 },
    { 0xdd11b505, 61 },
    { 0, 0 },
    { 0x471b507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1b50a, 23 },
    { 0x9421b50b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad71b510, 61 },
    { 0, 0 },
    { 0x7f11b512, 17 },
    { 0x2781b512, 95 },
    { 0xd2f1b514, 87 },
    { 0x67e1b515, 87 },
    { 0x84d1b515, 83 },
    { 0x9081b517, 99 },
    { 0x93e1b518, 90 },
    { 0x20c1b519, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4131b51c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b11b521, 76 },
    { 0x4901b521, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa501b525, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b1b52a, 98 },
    { 0x27d1b52b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9841b535, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d81b539, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b1b53e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b41b545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5971b548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7511b54b, 23 },
    { 0, 0 },
    { 0xb001b54d, 84 },
    { 0, 0 },
    { 0xdb91b54f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a1b556, 75 },
    { 0xec01b557, 61 },
    { 0x78b1b558, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3721b55c, 93 },
    { 0x8141b55d, 95 },
    { 0, 0 },
    { 0xeda1b55f, 98 },
    { 0x7871b55f, 43 },
    { 0xd761b55f, 90 },
    { 0x68e1b562, 95 },
    { 0xfc71b55f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd241b566, 85 },
    { 0, 0 },
    { 0x3bb1b568, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d1b56c, 95 },
    { 0, 0 },
    { 0x4bc1b56e, 36 },
    { 0x8d61b56e, 61 },
    { 0x43f1b56e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4901b574, 87 },
    { 0x5941b575, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a31b57b, 90 },
    { 0xfcd1b57b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb21b582, 61 },
    { 0, 0 },
    { 0xfbe1b584, 61 },
    { 0xa451b584, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a1b58a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d1b58e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd01b592, 23 },
    { 0x6591b592, 87 },
    { 0x2931b592, 46 },
    { 0xd161b595, 84 },
    { 0x9191b596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1b599, 61 },
    { 0x2dc1b599, 98 },
    { 0xce1b59b, 36 },
    { 0x2fb1b59b, 100 },
    { 0xd921b59b, 95 },
    { 0, 0 },
    { 0x6981b59f, 17 },
    { 0xac91b59f, 61 },
    { 0x6021b5a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f1b5a4, 23 },
    { 0xd351b5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f1b5aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea1b5af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1b5b4, 33 },
    { 0xba91b5b5, 55 },
    { 0xfeb1b5b6, 61 },
    { 0, 0 },
    { 0x27e1b5b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa761b5c3, 67 },
    { 0, 0 },
    { 0xc7a1b5c5, 76 },
    { 0x9b51b5c6, 90 },
    { 0xca41b5c7, 55 },
    { 0, 0 },
    { 0xaf1b5c9, 33 },
    { 0xef31b5ca, 90 },
    { 0xba81b5cb, 33 },
    { 0x4b11b5ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1351b5cf, 61 },
    { 0xb3f1b5d0, 90 },
    { 0xd3d1b5d1, 99 },
    { 0x7ef1b5d2, 84 },
    { 0x1561b5cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc171b5d7, 34 },
    { 0, 0 },
    { 0xe5f1b5d9, 93 },
    { 0x9301b5da, 44 },
    { 0x9d41b5db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd421b5e0, 61 },
    { 0, 0 },
    { 0x4ee1b5e2, 95 },
    { 0x5d01b5e2, 87 },
    { 0x9aa1b5e4, 90 },
    { 0x8911b5e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d1b5ec, 53 },
    { 0x4f1b5ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21b5f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a1b5f4, 23 },
    { 0, 0 },
    { 0x5f21b5f6, 33 },
    { 0xa8f1b5f7, 61 },
    { 0xb01b5f7, 61 },
    { 0x4561b5f9, 95 },
    { 0, 0 },
    { 0x5dd1b5fb, 80 },
    { 0x2761b5fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca1b5ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea61b604, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb051b60a, 90 },
    { 0xa6b1b60b, 98 },
    { 0x42e1b60c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a1b613, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c1b616, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e1b61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a1b620, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c1b626, 90 },
    { 0, 0 },
    { 0xef71b628, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad11b62d, 98 },
    { 0, 0 },
    { 0xfc51b62f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8341b632, 98 },
    { 0, 0 },
    { 0x67a1b634, 55 },
    { 0xed31b635, 81 },
    { 0, 0 },
    { 0x8c11b637, 84 },
    { 0xfc91b637, 61 },
    { 0, 0 },
    { 0x9c41b63a, 90 },
    { 0x6c1b63b, 77 },
    { 0x6bb1b63c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d51b640, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb11b643, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851b647, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c61b64b, 61 },
    { 0x3ce1b64c, 52 },
    { 0x72c1b64c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b11b652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b1b658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd1b65d, 61 },
    { 0, 0 },
    { 0xd9a1b65f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef1b663, 98 },
    { 0xb0a1b663, 98 },
    { 0xe581b664, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8661b668, 90 },
    { 0xb311b669, 98 },
    { 0xfad1b66a, 33 },
    { 0xdc91b66b, 61 },
    { 0xca41b66c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2751b66f, 46 },
    { 0xa131b670, 23 },
    { 0x2411b670, 98 },
    { 0xb5f1b672, 34 },
    { 0x3241b673, 61 },
    { 0xbd11b674, 90 },
    { 0x3b61b675, 93 },
    { 0x1ce1b676, 33 },
    { 0, 0 },
    { 0x2741b678, 23 },
    { 0x3861b679, 95 },
    { 0xeb71b67a, 98 },
    { 0x5bc1b67a, 61 },
    { 0, 0 },
    { 0x3131b67d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5391b680, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd881b684, 61 },
    { 0xfa91b684, 46 },
    { 0xc821b686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b68c, 55 },
    { 0x2491b68c, 95 },
    { 0xffd1b68e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca71b691, 33 },
    { 0, 0 },
    { 0x49a1b693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa471b69d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc71b6a3, 81 },
    { 0xbec1b6a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f1b6a7, 61 },
    { 0x29e1b6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c1b6af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7321b6b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b1b6b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae71b6bc, 98 },
    { 0, 0 },
    { 0x8c91b6be, 91 },
    { 0x6ba1b6bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc1b6c4, 46 },
    { 0, 0 },
    { 0x6231b6c6, 34 },
    { 0x6b11b6c7, 76 },
    { 0, 0 },
    { 0xecf1b6c9, 33 },
    { 0, 0 },
    { 0xd021b6cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba1b6ce, 85 },
    { 0xb721b6cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e1b6da, 95 },
    { 0, 0 },
    { 0x53f1b6dc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd921b6e2, 61 },
    { 0xd3e1b6e3, 33 },
    { 0xb931b6e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca1b6e7, 98 },
    { 0, 0 },
    { 0xc6a1b6e9, 76 },
    { 0x3201b6ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a51b6ed, 61 },
    { 0, 0 },
    { 0x1371b6ef, 90 },
    { 0x9361b6f0, 90 },
    { 0, 0 },
    { 0x9241b6f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb1b6f7, 34 },
    { 0xaa91b6f8, 90 },
    { 0xacb1b6f7, 93 },
    { 0x7d61b6fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2471b6ff, 95 },
    { 0xf7a1b700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3491b708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf391b70b, 95 },
    { 0xbf11b70c, 46 },
    { 0, 0 },
    { 0xb431b70e, 91 },
    { 0xca1b70f, 33 },
    { 0xf791b70e, 99 },
    { 0xbe31b711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c1b714, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9491b71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2331b71d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2001b722, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa011b726, 23 },
    { 0xf1c1b727, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd11b72b, 61 },
    { 0x7991b72b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8571b733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761b736, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b51b73b, 61 },
    { 0x9b51b73c, 90 },
    { 0x3ae1b73c, 98 },
    { 0x17b1b73e, 90 },
    { 0xca41b73f, 93 },
    { 0x8e21b740, 93 },
    { 0xbd11b741, 90 },
    { 0x6931b742, 52 },
    { 0x8ca1b742, 61 },
    { 0x561b742, 84 },
    { 0x74b1b743, 46 },
    { 0xcb1b746, 87 },
    { 0x6791b746, 33 },
    { 0, 0 },
    { 0xb1b749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa281b754, 84 },
    { 0x1a91b754, 33 },
    { 0x1ff1b756, 33 },
    { 0x6e51b757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb11b75d, 89 },
    { 0xb21b75d, 85 },
    { 0xeff1b75e, 78 },
    { 0xbc41b75f, 46 },
    { 0x2811b761, 95 },
    { 0x3a91b762, 98 },
    { 0xc871b763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe561b769, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2571b76e, 99 },
    { 0x6971b76f, 93 },
    { 0x71a1b76f, 61 },
    { 0x3771b76e, 95 },
    { 0x37a1b76e, 78 },
    { 0x3041b76e, 75 },
    { 0, 0 },
    { 0xd171b775, 98 },
    { 0x4e01b775, 52 },
    { 0x94e1b777, 81 },
    { 0xe391b778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c51b780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b91b783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d1b788, 23 },
    { 0xaa41b789, 33 },
    { 0, 0 },
    { 0xff41b78b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd011b792, 34 },
    { 0x7c71b793, 68 },
    { 0x49f1b792, 61 },
    { 0xcab1b793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf401b79c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf1b79f, 34 },
    { 0, 0 },
    { 0x8ff1b7a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb981b7a5, 33 },
    { 0, 0 },
    { 0xaac1b7a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c1b7aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3441b7ad, 17 },
    { 0, 0 },
    { 0xdb31b7af, 55 },
    { 0xfa41b7b0, 76 },
    { 0, 0 },
    { 0xb111b7b2, 34 },
    { 0, 0 },
    { 0x23f1b7b4, 33 },
    { 0xafb1b7b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7671b7b9, 98 },
    { 0x2931b7ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be1b7c0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd1b7c3, 95 },
    { 0x6e81b7c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c31b7c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4581b7cd, 81 },
    { 0xef21b7cd, 95 },
    { 0x8b01b7ce, 98 },
    { 0xf251b7ce, 66 },
    { 0x7e1b7d1, 52 },
    { 0x4041b7d2, 68 },
    { 0x68f1b7d3, 90 },
    { 0x7281b7d4, 61 },
    { 0x91b1b7d4, 33 },
    { 0x34a1b7d1, 61 },
    { 0x3351b7d7, 33 },
    { 0xf551b7d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd111b7dd, 98 },
    { 0x161b7de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e1b7e4, 88 },
    { 0x86d1b7e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef01b7e9, 101 },
    { 0xea01b7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671b7f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f1b7f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f1b7fb, 68 },
    { 0x8e1b7fc, 61 },
    { 0x3111b7fc, 67 },
    { 0x5501b7fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd41b804, 61 },
    { 0xf901b805, 46 },
    { 0, 0 },
    { 0x2261b807, 19 },
    { 0x4021b807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc1b80c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f21b810, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811b819, 33 },
    { 0x16d1b819, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8261b81d, 98 },
    { 0xc991b81d, 76 },
    { 0xe341b81f, 90 },
    { 0xebb1b81f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda71b828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9601b82c, 43 },
    { 0x63b1b82d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb1b831, 66 },
    { 0xfc61b832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6561b835, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb971b83a, 90 },
    { 0x40e1b83b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa971b83f, 33 },
    { 0xf2b1b83f, 76 },
    { 0xbf41b841, 46 },
    { 0x1ea1b841, 100 },
    { 0x8c01b841, 46 },
    { 0x9391b842, 90 },
    { 0xfd31b845, 75 },
    { 0x6531b841, 98 },
    { 0, 0 },
    { 0x4ba1b848, 33 },
    { 0, 0 },
    { 0xbb01b84a, 46 },
    { 0x5f51b84a, 75 },
    { 0, 0 },
    { 0xcde1b84d, 33 },
    { 0xadd1b84e, 55 },
    { 0, 0 },
    { 0xef51b850, 90 },
    { 0, 0 },
    { 0xfa41b852, 90 },
    { 0, 0 },
    { 0xf2c1b854, 33 },
    { 0x2e1b855, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf81b85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6221b85f, 33 },
    { 0x1aa1b85f, 84 },
    { 0, 0 },
    { 0xc2e1b862, 77 },
    { 0, 0 },
    { 0x26e1b864, 23 },
    { 0x6b21b865, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf041b86a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221b86e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f1b874, 23 },
    { 0, 0 },
    { 0x5801b876, 88 },
    { 0x1561b876, 90 },
    { 0xaba1b878, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d51b87c, 67 },
    { 0x3621b87d, 34 },
    { 0x3831b87d, 84 },
    { 0x2051b87f, 91 },
    { 0x76e1b880, 61 },
    { 0x7c81b880, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb611b884, 33 },
    { 0xda21b884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f1b892, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6451b897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda1b89b, 90 },
    { 0x28c1b89c, 75 },
    { 0x4651b89c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c81b8a0, 81 },
    { 0x2c11b8a1, 55 },
    { 0xe811b8a1, 43 },
    { 0, 0 },
    { 0x9ad1b8a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa1b8a7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f1b8aa, 33 },
    { 0x2631b8ab, 34 },
    { 0xad61b8ac, 66 },
    { 0xe2e1b8ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9431b8b0, 87 },
    { 0x2b21b8b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b51b8b5, 91 },
    { 0xc761b8b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e31b8b9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a1b8bc, 76 },
    { 0x55e1b8bc, 61 },
    { 0, 0 },
    { 0x4bb1b8bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda81b8c2, 66 },
    { 0x8bb1b8c3, 85 },
    { 0x44e1b8c4, 23 },
    { 0x1fc1b8c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe581b8c8, 90 },
    { 0x2271b8c9, 76 },
    { 0x931b8ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5741b8ce, 61 },
    { 0, 0 },
    { 0xb1f1b8d0, 33 },
    { 0x5d11b8d1, 23 },
    { 0xdbd1b8d0, 95 },
    { 0x2ae1b8d0, 99 },
    { 0xa1c1b8d3, 23 },
    { 0xf301b8d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261b8d8, 77 },
    { 0xc851b8d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb501b8dd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4121b8e0, 94 },
    { 0, 0 },
    { 0x31c1b8e2, 85 },
    { 0x54c1b8e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc951b8e6, 65 },
    { 0xf121b8e6, 65 },
    { 0, 0 },
    { 0x49b1b8e9, 55 },
    { 0xe6a1b8e9, 55 },
    { 0x4581b8eb, 98 },
    { 0xfdc1b8ec, 90 },
    { 0x3221b8ed, 91 },
    { 0xcd01b8ed, 95 },
    { 0x3951b8eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3451b8f2, 85 },
    { 0, 0 },
    { 0xed31b8f4, 98 },
    { 0, 0 },
    { 0x5e1b8f6, 90 },
    { 0x1e21b8f6, 90 },
    { 0, 0 },
    { 0xea11b8f9, 95 },
    { 0x9981b8fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b21b8ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c71b904, 23 },
    { 0, 0 },
    { 0x7f1b906, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4401b909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa71b90c, 33 },
    { 0x2f81b90c, 61 },
    { 0xee1b90e, 99 },
    { 0xe111b90f, 90 },
    { 0, 0 },
    { 0x7ab1b911, 98 },
    { 0, 0 },
    { 0xff91b913, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b919, 46 },
    { 0, 0 },
    { 0x5081b91b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e1b922, 98 },
    { 0, 0 },
    { 0x7361b924, 85 },
    { 0xde81b925, 52 },
    { 0xf8e1b926, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd371b92d, 98 },
    { 0x9a31b92d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc01b939, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2181b93c, 33 },
    { 0xe821b93c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7911b944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa31b948, 47 },
    { 0, 0 },
    { 0x921b94a, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b1b94f, 23 },
    { 0x9e51b950, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb141b955, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd581b95c, 77 },
    { 0xd351b95c, 99 },
    { 0xb2e1b95d, 100 },
    { 0xec1b95f, 36 },
    { 0x7601b95f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df1b963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3591b966, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac1b96b, 33 },
    { 0x67e1b96b, 33 },
    { 0, 0 },
    { 0xe951b96e, 90 },
    { 0x6961b96f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b975, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad31b97a, 77 },
    { 0, 0 },
    { 0xb961b97c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec51b981, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8081b985, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc931b989, 33 },
    { 0xa91b98a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3321b98d, 93 },
    { 0x9591b98e, 65 },
    { 0xfc81b98e, 90 },
    { 0x8551b98e, 55 },
    { 0xa601b98e, 98 },
    { 0, 0 },
    { 0x6af1b993, 91 },
    { 0xc521b994, 98 },
    { 0, 0 },
    { 0x36e1b996, 61 },
    { 0, 0 },
    { 0x73a1b998, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9961b99b, 61 },
    { 0x88c1b99c, 23 },
    { 0, 0 },
    { 0x94b1b99e, 61 },
    { 0xc61b99f, 33 },
    { 0xbf41b99f, 23 },
    { 0xa841b9a1, 91 },
    { 0xab91b99f, 87 },
    { 0x1d71b9a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5401b9a7, 61 },
    { 0xe1a1b9a8, 90 },
    { 0xb7a1b9a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb091b9ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa261b9b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c1b9b4, 65 },
    { 0x6d61b9b5, 36 },
    { 0x78e1b9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd951b9bc, 98 },
    { 0x1f31b9bd, 90 },
    { 0, 0 },
    { 0x53b1b9bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa1b9c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921b9c7, 90 },
    { 0x91d1b9c8, 15 },
    { 0x7e31b9c9, 90 },
    { 0x56b1b9c8, 90 },
    { 0xefc1b9cb, 95 },
    { 0x821b9cc, 90 },
    { 0x90b1b9cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b61b9d1, 81 },
    { 0xdf21b9d2, 76 },
    { 0x6e41b9d3, 95 },
    { 0xa921b9d4, 34 },
    { 0, 0 },
    { 0x5e11b9d6, 36 },
    { 0x6ca1b9d7, 33 },
    { 0x38f1b9d8, 90 },
    { 0x92f1b9d7, 90 },
    { 0xc571b9da, 46 },
    { 0xf141b9da, 33 },
    { 0x3c41b9d8, 33 },
    { 0xa531b9dd, 48 },
    { 0x1581b9dd, 90 },
    { 0xcc61b9df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a51b9e7, 76 },
    { 0x3231b9e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6881b9ee, 33 },
    { 0x9941b9ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b9f2, 55 },
    { 0xb71b9f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1b9f7, 80 },
    { 0x2131b9f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c71b9fe, 78 },
    { 0, 0 },
    { 0x3d51ba00, 91 },
    { 0xe0f1ba00, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f1ba05, 23 },
    { 0x8071ba06, 95 },
    { 0, 0 },
    { 0xd041ba08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda1ba0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb1ba12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371ba17, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x6571ba1a, 23 },
    { 0xaae1ba1b, 98 },
    { 0xb651ba1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d81ba23, 95 },
    { 0, 0 },
    { 0xe751ba25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d31ba2c, 66 },
    { 0xa2c1ba2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc1ba30, 90 },
    { 0, 0 },
    { 0x2a21ba32, 67 },
    { 0xa2d1ba32, 43 },
    { 0x6f91ba34, 61 },
    { 0x17b1ba35, 78 },
    { 0x6ff1ba35, 61 },
    { 0x6791ba37, 98 },
    { 0xd9b1ba34, 34 },
    { 0x8d11ba34, 61 },
    { 0xa431ba3a, 55 },
    { 0xa4a1ba3b, 40 },
    { 0x1661ba3a, 55 },
    { 0x4f01ba3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6441ba43, 33 },
    { 0, 0 },
    { 0x71e1ba45, 61 },
    { 0, 0 },
    { 0x4ac1ba47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb1ba4a, 76 },
    { 0x18c1ba4b, 52 },
    { 0, 0 },
    { 0x3d21ba4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b1ba55, 87 },
    { 0, 0 },
    { 0x6a71ba57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d1ba5c, 61 },
    { 0x1cf1ba5d, 61 },
    { 0x5f61ba5d, 88 },
    { 0x6b91ba5f, 95 },
    { 0x7c21ba60, 33 },
    { 0x8081ba61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb01ba65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f1ba6a, 95 },
    { 0, 0 },
    { 0x5c71ba6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa341ba6f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e71ba72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b61ba75, 55 },
    { 0, 0 },
    { 0x7251ba77, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7501ba7b, 90 },
    { 0, 0 },
    { 0x6de1ba7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a1ba80, 95 },
    { 0xca81ba81, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad1ba88, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71ba8d, 61 },
    { 0, 0 },
    { 0xabc1ba8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e1ba92, 23 },
    { 0xf421ba92, 23 },
    { 0x3b11ba94, 100 },
    { 0xfd71ba95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf911ba98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca1baa1, 98 },
    { 0x5541baa2, 91 },
    { 0, 0 },
    { 0x80c1baa4, 55 },
    { 0xfc51baa4, 38 },
    { 0, 0 },
    { 0xa441baa7, 61 },
    { 0xf481baa8, 55 },
    { 0, 0 },
    { 0xaeb1baaa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651baae, 87 },
    { 0xdaf1baae, 43 },
    { 0, 0 },
    { 0x4a1bab1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41bab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6751bab8, 90 },
    { 0x4e01bab8, 52 },
    { 0x72d1bab9, 46 },
    { 0, 0 },
    { 0x3c11babc, 23 },
    { 0xaee1babc, 76 },
    { 0xb941babc, 95 },
    { 0x7c91babc, 95 },
    { 0x4fb1bac0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9921bac5, 61 },
    { 0xad71bac6, 98 },
    { 0, 0 },
    { 0x4401bac8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2331bad2, 85 },
    { 0x51a1bad3, 80 },
    { 0x2221bad4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c11bad8, 90 },
    { 0xcd71bad9, 90 },
    { 0x4fd1bada, 34 },
    { 0x60f1badb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff1badf, 43 },
    { 0, 0 },
    { 0xee31bae1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c1baeb, 68 },
    { 0, 0 },
    { 0xcd01baed, 55 },
    { 0, 0 },
    { 0xc6f1baef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a41baf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1771baf5, 43 },
    { 0x4481baf6, 95 },
    { 0x2621baf6, 23 },
    { 0x1011baf8, 81 },
    { 0xd0f1baf9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1001bafd, 23 },
    { 0x9241bafe, 52 },
    { 0x8dd1baff, 61 },
    { 0xf611bb00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c1bb06, 87 },
    { 0x3d81bb06, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2381bb0f, 34 },
    { 0xa4f1bb10, 77 },
    { 0, 0 },
    { 0x5641bb12, 67 },
    { 0, 0 },
    { 0x20b1bb14, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f1bb20, 99 },
    { 0, 0 },
    { 0x1421bb22, 55 },
    { 0x4ae1bb23, 83 },
    { 0x68a1bb23, 90 },
    { 0x79f1bb25, 100 },
    { 0x4261bb26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c21bb29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea91bb34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4301bb38, 33 },
    { 0xf481bb39, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed11bb3d, 78 },
    { 0x9271bb3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf11bb41, 78 },
    { 0, 0 },
    { 0x4601bb43, 84 },
    { 0xaa31bb44, 98 },
    { 0, 0 },
    { 0x6841bb46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de1bb4c, 90 },
    { 0x78e1bb4d, 80 },
    { 0, 0 },
    { 0x8741bb4f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb71bb53, 98 },
    { 0, 0 },
    { 0x5041bb55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b41bb5b, 95 },
    { 0xc791bb5b, 85 },
    { 0x6e61bb5b, 17 },
    { 0, 0 },
    { 0x44f1bb5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8391bb62, 33 },
    { 0, 0 },
    { 0x9941bb64, 90 },
    { 0x10b1bb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe031bb68, 88 },
    { 0x9991bb68, 95 },
    { 0, 0 },
    { 0x1031bb6b, 61 },
    { 0xaa71bb6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c1bb70, 32 },
    { 0x5aa1bb70, 23 },
    { 0, 0 },
    { 0x2d1bb73, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb041bb76, 95 },
    { 0xa0e1bb76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1bb7b, 67 },
    { 0x8d71bb7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f1bb81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1901bb87, 85 },
    { 0xfc71bb88, 90 },
    { 0x1961bb89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a1bb8e, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd271bb92, 61 },
    { 0x62f1bb92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021bb96, 55 },
    { 0xe081bb97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21bb9a, 43 },
    { 0x17e1bb9b, 23 },
    { 0x7381bb9b, 77 },
    { 0x5361bb9d, 76 },
    { 0x8a01bb9e, 61 },
    { 0xd051bb9c, 90 },
    { 0, 0 },
    { 0x5a51bba1, 90 },
    { 0, 0 },
    { 0x5b11bba3, 34 },
    { 0x1d21bba4, 68 },
    { 0x1241bba5, 91 },
    { 0x3ad1bba6, 33 },
    { 0xa2f1bba5, 61 },
    { 0, 0 },
    { 0x11a1bba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc1bbad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc1bbb3, 90 },
    { 0, 0 },
    { 0x1181bbb5, 40 },
    { 0, 0 },
    { 0xc31bbb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6161bbbb, 43 },
    { 0xad01bbbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901bbbf, 17 },
    { 0, 0 },
    { 0xec1bbc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb481bbc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc1bbcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551bbd4, 46 },
    { 0x2c91bbd4, 90 },
    { 0x251bbd6, 98 },
    { 0x4f21bbd7, 55 },
    { 0xe581bbd4, 60 },
    { 0x9d91bbd4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd1bbdc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b1bbdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba1bbe5, 60 },
    { 0, 0 },
    { 0xbde1bbe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8991bbea, 98 },
    { 0x2c01bbeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc1bbee, 91 },
    { 0x41a1bbee, 76 },
    { 0xe021bbf0, 43 },
    { 0x1351bbf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d1bbf5, 90 },
    { 0x4aa1bbf6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5961bc01, 23 },
    { 0xd8a1bc02, 98 },
    { 0x3181bc02, 80 },
    { 0xb3c1bc04, 34 },
    { 0x4751bc05, 98 },
    { 0x99f1bc04, 33 },
    { 0x2c51bc06, 80 },
    { 0xae31bc04, 81 },
    { 0x8cb1bc09, 53 },
    { 0xf4b1bc0a, 95 },
    { 0x2ed1bc08, 33 },
    { 0xf2b1bc02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d1bc10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6701bc13, 55 },
    { 0, 0 },
    { 0x6cb1bc15, 67 },
    { 0xa51bc16, 98 },
    { 0x7d11bc17, 23 },
    { 0x6e01bc18, 33 },
    { 0x5fa1bc17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2491bc24, 98 },
    { 0x4421bc25, 95 },
    { 0, 0 },
    { 0x7831bc27, 61 },
    { 0, 0 },
    { 0xecf1bc29, 90 },
    { 0xace1bc2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3261bc2f, 76 },
    { 0x2aa1bc30, 49 },
    { 0x1f1bc30, 95 },
    { 0x1a71bc31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a1bc37, 95 },
    { 0x3a31bc38, 23 },
    { 0x5621bc38, 95 },
    { 0x6221bc3a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e51bc43, 33 },
    { 0x9891bc44, 81 },
    { 0x7b21bc45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1bc48, 76 },
    { 0xa371bc49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1bc4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc1bc51, 100 },
    { 0x5961bc52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f31bc56, 85 },
    { 0xc881bc57, 33 },
    { 0xbcb1bc57, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b11bc5b, 33 },
    { 0x2d91bc5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa461bc62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6461bc66, 90 },
    { 0x84f1bc66, 90 },
    { 0x6c1bc66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e1bc6b, 75 },
    { 0, 0 },
    { 0xa981bc6d, 33 },
    { 0x8fd1bc6d, 23 },
    { 0, 0 },
    { 0x371bc70, 33 },
    { 0, 0 },
    { 0x5251bc72, 66 },
    { 0, 0 },
    { 0xad51bc74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f81bc77, 90 },
    { 0x5261bc78, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec61bc7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af1bc80, 61 },
    { 0x1b81bc81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b71bc86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf91bc8b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea1bc8e, 98 },
    { 0, 0 },
    { 0xd1f1bc90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d01bc95, 98 },
    { 0xd811bc96, 76 },
    { 0xf4b1bc96, 55 },
    { 0x9b81bc97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a1bc9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd221bca7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d31bcab, 100 },
    { 0x751bcab, 90 },
    { 0xe851bcab, 95 },
    { 0, 0 },
    { 0x5eb1bcaf, 98 },
    { 0x8611bcb0, 55 },
    { 0, 0 },
    { 0x1551bcb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11bcb7, 90 },
    { 0x46e1bcb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e1bcbe, 84 },
    { 0x7771bcbe, 90 },
    { 0xf701bcc0, 90 },
    { 0x9871bcc1, 52 },
    { 0x6441bcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1bcc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd671bccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd31bcdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3021bcdf, 34 },
    { 0x5bd1bce0, 95 },
    { 0x6b11bce1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e91bce9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf31bcee, 33 },
    { 0x1ee1bcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9751bcf2, 88 },
    { 0xb881bcf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e1bcf6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f21bcfb, 98 },
    { 0x9201bcfc, 23 },
    { 0x5ed1bcfc, 33 },
    { 0x58f1bcfe, 95 },
    { 0x6a31bcff, 61 },
    { 0, 0 },
    { 0x4761bd01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce1bd04, 68 },
    { 0xc051bd05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1bd08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe1bd0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3891bd11, 33 },
    { 0x79c1bd12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d91bd1a, 11 },
    { 0, 0 },
    { 0xa661bd1c, 90 },
    { 0x2f61bd1d, 77 },
    { 0x4e51bd1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61bd23, 33 },
    { 0x37f1bd24, 55 },
    { 0x1d01bd25, 34 },
    { 0xdea1bd25, 33 },
    { 0x38e1bd27, 46 },
    { 0x2af1bd28, 61 },
    { 0, 0 },
    { 0x851bd2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4921bd31, 23 },
    { 0x1851bd32, 90 },
    { 0xf821bd31, 96 },
    { 0x3071bd33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba1bd44, 40 },
    { 0xdf41bd44, 33 },
    { 0, 0 },
    { 0xfaa1bd47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc1bd4b, 40 },
    { 0xa501bd4c, 33 },
    { 0, 0 },
    { 0x68d1bd4e, 90 },
    { 0, 0 },
    { 0xaf61bd50, 61 },
    { 0x2a91bd51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd1bd55, 33 },
    { 0x3e01bd55, 55 },
    { 0, 0 },
    { 0x52b1bd58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee51bd5b, 90 },
    { 0xa351bd5b, 90 },
    { 0xd2d1bd5b, 93 },
    { 0xee91bd5e, 93 },
    { 0xcda1bd5f, 61 },
    { 0x3b41bd60, 33 },
    { 0xbc91bd5f, 98 },
    { 0x9a91bd5b, 91 },
    { 0xd271bd5d, 90 },
    { 0x2341bd64, 49 },
    { 0x1401bd64, 33 },
    { 0x8401bd66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41bd6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1bd73, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461bd7f, 98 },
    { 0, 0 },
    { 0xb401bd81, 17 },
    { 0, 0 },
    { 0xc6c1bd83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11bd88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec91bd8c, 93 },
    { 0, 0 },
    { 0xb791bd8e, 95 },
    { 0xfe21bd8f, 43 },
    { 0, 0 },
    { 0xc21bd91, 76 },
    { 0x24e1bd92, 60 },
    { 0x3c31bd93, 46 },
    { 0x90e1bd92, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f1bd9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a21bda0, 66 },
    { 0x74c1bda1, 95 },
    { 0xf911bda2, 43 },
    { 0, 0 },
    { 0x8411bda4, 98 },
    { 0x5211bda5, 93 },
    { 0x5131bda4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb1bda9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d51bdae, 61 },
    { 0xb3f1bdaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c1bdb7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c1bdbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd501bdc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf911bdc4, 83 },
    { 0x1a1bdc4, 95 },
    { 0x49e1bdc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc1bdc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b1bdcc, 99 },
    { 0x7141bdcc, 100 },
    { 0xc181bdce, 90 },
    { 0x9bf1bdce, 95 },
    { 0x85e1bdcf, 55 },
    { 0xed31bdcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac1bdda, 33 },
    { 0x2f41bdda, 85 },
    { 0x51bdda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4911bde0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d1bde5, 90 },
    { 0x4081bde6, 76 },
    { 0x1401bde6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac51bdf6, 23 },
    { 0xe321bdf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba1bdfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1911be03, 23 },
    { 0, 0 },
    { 0x6631be05, 33 },
    { 0x1c61be06, 100 },
    { 0x8e1be07, 90 },
    { 0x85b1be07, 51 },
    { 0xc341be09, 90 },
    { 0x67c1be08, 61 },
    { 0x301be0a, 76 },
    { 0x58f1be0c, 93 },
    { 0x40b1be07, 33 },
    { 0, 0 },
    { 0x8791be0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b1be16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71be1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a91be2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe821be32, 55 },
    { 0x4d51be32, 43 },
    { 0, 0 },
    { 0xc051be35, 67 },
    { 0x4821be36, 95 },
    { 0x5391be35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac91be3a, 61 },
    { 0xe171be3b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1be43, 85 },
    { 0xc5a1be44, 93 },
    { 0xfd81be45, 61 },
    { 0x9be1be46, 23 },
    { 0x98a1be45, 78 },
    { 0x68d1be44, 99 },
    { 0x55b1be49, 98 },
    { 0x5041be43, 61 },
    { 0, 0 },
    { 0x9961be4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9381be51, 14 },
    { 0, 0 },
    { 0xa5f1be53, 23 },
    { 0, 0 },
    { 0x77e1be55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4591be59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c81be5f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81be62, 36 },
    { 0xd841be63, 90 },
    { 0x9921be64, 95 },
    { 0x9a61be65, 68 },
    { 0x7f51be65, 84 },
    { 0x1ae1be67, 85 },
    { 0x9121be63, 23 },
    { 0, 0 },
    { 0xac81be6a, 33 },
    { 0, 0 },
    { 0xf261be6c, 33 },
    { 0x2f21be6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc1be74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde41be7b, 61 },
    { 0xb2e1be7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6201be80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a1be84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd601be91, 66 },
    { 0, 0 },
    { 0x6241be93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41be99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf671be9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4981bea2, 23 },
    { 0xf901bea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51beaa, 23 },
    { 0xa941beab, 90 },
    { 0xa071beac, 90 },
    { 0x39f1bead, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6211beb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6151beb4, 90 },
    { 0, 0 },
    { 0x8931beb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb941bebe, 37 },
    { 0, 0 },
    { 0xf481bec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca1bec6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd1becd, 84 },
    { 0, 0 },
    { 0x2641becf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7441bed9, 77 },
    { 0x5031bed9, 66 },
    { 0xb4a1beda, 95 },
    { 0x62f1bedc, 52 },
    { 0, 0 },
    { 0xf8f1bede, 90 },
    { 0, 0 },
    { 0xa551bee0, 90 },
    { 0xcd91bee1, 82 },
    { 0x9bd1bee2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9051bee8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f61beec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b1bef3, 55 },
    { 0xb721bef4, 90 },
    { 0x64a1bef5, 61 },
    { 0xce61bef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba91beff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b1bf03, 33 },
    { 0, 0 },
    { 0x6991bf05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341bf0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e81bf11, 33 },
    { 0xfdb1bf12, 65 },
    { 0xa951bf11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d1bf18, 64 },
    { 0, 0 },
    { 0x9851bf1a, 55 },
    { 0xf4d1bf1a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa321bf1e, 95 },
    { 0xba81bf1f, 98 },
    { 0x69f1bf20, 33 },
    { 0x9071bf21, 34 },
    { 0xaf01bf1f, 90 },
    { 0x34b1bf20, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7921bf29, 90 },
    { 0x2d21bf29, 77 },
    { 0xa1d1bf2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c1bf2f, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb881bf35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf121bf3a, 23 },
    { 0x5601bf3b, 33 },
    { 0x9761bf3b, 33 },
    { 0xabb1bf3b, 34 },
    { 0, 0 },
    { 0x9071bf3f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e1bf43, 90 },
    { 0xb3b1bf43, 90 },
    { 0x43f1bf43, 90 },
    { 0x5b31bf45, 17 },
    { 0x70f1bf47, 23 },
    { 0x6151bf48, 46 },
    { 0xbfe1bf49, 66 },
    { 0x4a1bf4a, 33 },
    { 0x9291bf48, 55 },
    { 0xaf31bf4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd41bf52, 55 },
    { 0, 0 },
    { 0xa1c1bf54, 90 },
    { 0x86d1bf55, 23 },
    { 0x3791bf54, 36 },
    { 0xf331bf57, 90 },
    { 0xdc71bf58, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe741bf5c, 34 },
    { 0, 0 },
    { 0x9571bf5e, 33 },
    { 0, 0 },
    { 0x2981bf60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae1bf66, 76 },
    { 0xf741bf67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71bf6a, 61 },
    { 0, 0 },
    { 0x6491bf6c, 90 },
    { 0xd511bf6d, 98 },
    { 0x6451bf6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a1bf71, 66 },
    { 0x65e1bf71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a1bf75, 33 },
    { 0, 0 },
    { 0x571bf77, 23 },
    { 0, 0 },
    { 0xcf1bf79, 90 },
    { 0x1e81bf7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a21bf7d, 95 },
    { 0, 0 },
    { 0xf351bf7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb1bf86, 76 },
    { 0x41c1bf87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb1bf8a, 91 },
    { 0xf921bf8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5681bf8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec1bf91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1851bf94, 95 },
    { 0xe1c1bf94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb381bf98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8041bf9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2631bfa3, 61 },
    { 0x8411bfa4, 98 },
    { 0xf961bfa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31bfaa, 90 },
    { 0, 0 },
    { 0x4371bfac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351bfb4, 100 },
    { 0x5fd1bfb5, 85 },
    { 0xf71bfb4, 36 },
    { 0xbfe1bfb7, 23 },
    { 0xf711bfb5, 85 },
    { 0x3531bfb5, 33 },
    { 0x12a1bfba, 84 },
    { 0x181bfb9, 34 },
    { 0x6741bfbc, 95 },
    { 0, 0 },
    { 0x8391bfbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31bfc1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f1bfc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1401bfca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a1bfce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf461bfd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc821bfd8, 90 },
    { 0xca21bfd9, 95 },
    { 0, 0 },
    { 0x1011bfdb, 85 },
    { 0, 0 },
    { 0x3ea1bfdd, 55 },
    { 0x9da1bfde, 78 },
    { 0x96f1bfdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101bfee, 34 },
    { 0xda51bfef, 92 },
    { 0x2b51bfee, 23 },
    { 0xc491bff1, 34 },
    { 0x5a71bff2, 46 },
    { 0x7531bff3, 61 },
    { 0x61e1bff2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc791bff7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb821bffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1431bffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3981c004, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd591c009, 98 },
    { 0x3f81c00a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe1c010, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1c016, 76 },
    { 0x34a1c017, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c1c01c, 87 },
    { 0x3351c01d, 77 },
    { 0xe111c01e, 99 },
    { 0xb21c01c, 91 },
    { 0x6941c020, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9681c023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b1c026, 33 },
    { 0x4d1c026, 93 },
    { 0, 0 },
    { 0xae71c029, 23 },
    { 0x95c1c029, 76 },
    { 0x1de1c02b, 76 },
    { 0xdb1c02c, 33 },
    { 0x2781c02d, 91 },
    { 0xe911c02e, 23 },
    { 0xe0b1c02f, 76 },
    { 0xb371c02e, 33 },
    { 0, 0 },
    { 0xa9d1c032, 55 },
    { 0xf7b1c032, 93 },
    { 0xcb91c034, 33 },
    { 0x5f91c035, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7331c039, 90 },
    { 0, 0 },
    { 0x1d51c03b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a71c042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d11c04c, 46 },
    { 0, 0 },
    { 0xd4d1c04e, 76 },
    { 0x5131c04f, 34 },
    { 0x13f1c050, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde1c053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11c05b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2131c05f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e21c062, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451c066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a1c06b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b1c06e, 61 },
    { 0, 0 },
    { 0x3ab1c070, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e1c076, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xd741c079, 43 },
    { 0xd9c1c07a, 98 },
    { 0x2601c07b, 76 },
    { 0x1941c07b, 43 },
    { 0xcd91c07b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6451c081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241c085, 90 },
    { 0, 0 },
    { 0x5251c087, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571c08d, 90 },
    { 0x151c08e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa771c09f, 65 },
    { 0x2931c0a0, 98 },
    { 0x871c0a1, 55 },
    { 0xfa51c0a2, 76 },
    { 0, 0 },
    { 0xcd11c0a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4041c0a8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf651c0ac, 61 },
    { 0xb981c0ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f1c0b1, 61 },
    { 0xa9c1c0b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d11c0b9, 75 },
    { 0x3b21c0ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a1c0c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9611c0c4, 65 },
    { 0xc201c0c5, 23 },
    { 0x97d1c0c6, 33 },
    { 0, 0 },
    { 0x7e01c0c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe631c0cf, 85 },
    { 0x31d1c0cf, 76 },
    { 0x21a1c0cf, 88 },
    { 0x1df1c0d2, 34 },
    { 0x6471c0cf, 61 },
    { 0x1871c0cf, 43 },
    { 0x74e1c0d5, 43 },
    { 0x4c51c0d5, 61 },
    { 0x8131c0d1, 98 },
    { 0x95e1c0d8, 90 },
    { 0x6991c0d8, 61 },
    { 0xf281c0d7, 75 },
    { 0xc441c0d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431c0e5, 98 },
    { 0, 0 },
    { 0x2061c0e7, 76 },
    { 0xf8a1c0e8, 66 },
    { 0xb8e1c0e9, 95 },
    { 0xdea1c0ea, 67 },
    { 0xeb31c0eb, 55 },
    { 0x9a61c0ec, 76 },
    { 0, 0 },
    { 0x53e1c0ee, 61 },
    { 0xf4c1c0ef, 43 },
    { 0xf861c0f0, 76 },
    { 0xf101c0f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee1c0f5, 95 },
    { 0xc8f1c0f5, 76 },
    { 0, 0 },
    { 0xb671c0f8, 61 },
    { 0xf1d1c0f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c0fc, 75 },
    { 0x5e21c0fd, 98 },
    { 0, 0 },
    { 0x1761c0ff, 61 },
    { 0x3461c100, 95 },
    { 0, 0 },
    { 0x3dd1c102, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7891c106, 84 },
    { 0, 0 },
    { 0x5381c108, 19 },
    { 0, 0 },
    { 0x7851c10a, 33 },
    { 0, 0 },
    { 0xb001c10c, 33 },
    { 0xaef1c10d, 46 },
    { 0xa991c10e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4241c112, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1c119, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1c11d, 23 },
    { 0, 0 },
    { 0x2fa1c11f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d1c123, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f1c128, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae91c12c, 61 },
    { 0xc491c12c, 33 },
    { 0, 0 },
    { 0x6ac1c12f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f41c133, 61 },
    { 0x8681c134, 95 },
    { 0x7b81c134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61c13a, 90 },
    { 0x8eb1c13b, 52 },
    { 0xd41c13b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c1c146, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd491c149, 33 },
    { 0, 0 },
    { 0xf7d1c14b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d1c158, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b1c15c, 100 },
    { 0, 0 },
    { 0xf091c15e, 98 },
    { 0xfb1c15e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241c166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f1c16a, 46 },
    { 0, 0 },
    { 0xdb81c16c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae1c172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb71c176, 46 },
    { 0xd8a1c176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2381c17c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d1c17f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c01c182, 90 },
    { 0x24f1c182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a31c186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe701c18b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e1c18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8941c197, 61 },
    { 0, 0 },
    { 0x5781c199, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9221c19f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc71c1a4, 34 },
    { 0x6eb1c1a4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b1c1a8, 46 },
    { 0x6aa1c1a9, 78 },
    { 0xf521c1a8, 90 },
    { 0x7211c1aa, 100 },
    { 0x8dc1c1a8, 61 },
    { 0xf711c1ad, 69 },
    { 0, 0 },
    { 0x3fe1c1af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb1c1b3, 98 },
    { 0xd141c1b4, 33 },
    { 0, 0 },
    { 0xfb01c1b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e11c1b9, 90 },
    { 0, 0 },
    { 0xf11c1bb, 90 },
    { 0, 0 },
    { 0x7d31c1bd, 84 },
    { 0, 0 },
    { 0x82c1c1bf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71c1c3, 66 },
    { 0x421c1c4, 75 },
    { 0, 0 },
    { 0x4b21c1c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5301c1c9, 83 },
    { 0, 0 },
    { 0xb301c1cb, 55 },
    { 0x7591c1cc, 53 },
    { 0xaad1c1cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e1c1d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6601c1da, 68 },
    { 0x45a1c1da, 61 },
    { 0xc91c1dc, 34 },
    { 0xa981c1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7521c1e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c1c1e4, 34 },
    { 0, 0 },
    { 0x9311c1e6, 33 },
    { 0xba1c1e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5191c1ea, 61 },
    { 0xf481c1eb, 95 },
    { 0x9e71c1eb, 80 },
    { 0x27c1c1ed, 61 },
    { 0x5d91c1ee, 23 },
    { 0xeb01c1ed, 17 },
    { 0x42c1c1ec, 78 },
    { 0x7fd1c1ef, 61 },
    { 0, 0 },
    { 0xa11c1f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b1c1f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d1c1fa, 33 },
    { 0xb141c1fa, 90 },
    { 0x7ff1c1fb, 23 },
    { 0xac51c1fd, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781c200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba01c205, 91 },
    { 0, 0 },
    { 0x5a51c207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91c20e, 61 },
    { 0, 0 },
    { 0x7c21c210, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1951c214, 98 },
    { 0x88a1c214, 17 },
    { 0x23f1c216, 78 },
    { 0, 0 },
    { 0xadb1c218, 43 },
    { 0, 0 },
    { 0x2d41c21a, 61 },
    { 0xa611c21b, 95 },
    { 0x6e71c21c, 95 },
    { 0xc031c21a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4731c224, 90 },
    { 0xe441c225, 23 },
    { 0xce91c226, 90 },
    { 0, 0 },
    { 0x4c01c228, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11c22d, 33 },
    { 0xb631c22e, 95 },
    { 0, 0 },
    { 0xaf1c230, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c1c234, 98 },
    { 0, 0 },
    { 0x2a71c236, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a1c239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc1c240, 98 },
    { 0, 0 },
    { 0x7761c242, 98 },
    { 0, 0 },
    { 0xf71c244, 68 },
    { 0x12c1c244, 69 },
    { 0, 0 },
    { 0xd291c247, 91 },
    { 0, 0 },
    { 0x9f81c249, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f1c24e, 61 },
    { 0, 0 },
    { 0xecf1c250, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2021c255, 23 },
    { 0xd5f1c255, 76 },
    { 0x3a81c257, 23 },
    { 0x9751c258, 43 },
    { 0x86f1c259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf21c25d, 99 },
    { 0, 0 },
    { 0xf8c1c25f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71c265, 67 },
    { 0x3e51c265, 75 },
    { 0xff91c267, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4451c26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111c270, 61 },
    { 0x3701c270, 33 },
    { 0xf061c271, 61 },
    { 0, 0 },
    { 0x4f51c274, 17 },
    { 0, 0 },
    { 0x7be1c276, 43 },
    { 0x2fd1c276, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a1c27a, 61 },
    { 0xbf31c27b, 98 },
    { 0x6f91c27c, 90 },
    { 0xdb61c27b, 55 },
    { 0x1011c27e, 76 },
    { 0x6381c27e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef71c286, 61 },
    { 0, 0 },
    { 0xd621c288, 33 },
    { 0x6e61c289, 90 },
    { 0x8cb1c28a, 43 },
    { 0x6d71c28b, 17 },
    { 0x3c41c28c, 23 },
    { 0, 0 },
    { 0x45f1c28e, 55 },
    { 0xcaa1c28f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf711c292, 90 },
    { 0xc891c293, 99 },
    { 0x3fa1c294, 33 },
    { 0xb091c293, 33 },
    { 0, 0 },
    { 0x63b1c297, 33 },
    { 0xc1c1c298, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6261c29e, 76 },
    { 0, 0 },
    { 0x9641c2a0, 33 },
    { 0xa311c2a0, 48 },
    { 0xaba1c2a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd81c2a5, 78 },
    { 0x571c2a5, 95 },
    { 0x96e1c2a7, 93 },
    { 0x1d81c2a8, 83 },
    { 0, 0 },
    { 0x4e81c2aa, 90 },
    { 0, 0 },
    { 0xe5d1c2ac, 23 },
    { 0x8c31c2ad, 80 },
    { 0, 0 },
    { 0x15f1c2af, 46 },
    { 0xf71c2b0, 33 },
    { 0xf671c2b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc161c2b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe091c2b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1c2be, 53 },
    { 0, 0 },
    { 0x5291c2c0, 86 },
    { 0xa4c1c2c1, 78 },
    { 0x5541c2c0, 87 },
    { 0xda1c2c3, 93 },
    { 0xc8c1c2c4, 90 },
    { 0x82a1c2c1, 33 },
    { 0, 0 },
    { 0xbb61c2c7, 67 },
    { 0x82f1c2c8, 90 },
    { 0xc381c2c9, 90 },
    { 0x6b01c2c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a1c2cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241c2d1, 23 },
    { 0, 0 },
    { 0xe571c2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c2d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d1c2da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac1c2df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd81c2eb, 76 },
    { 0x91f1c2eb, 76 },
    { 0xab91c2ed, 66 },
    { 0xe821c2eb, 55 },
    { 0x851c2ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb71c2f2, 61 },
    { 0xf8e1c2f3, 33 },
    { 0x5671c2f4, 55 },
    { 0x11d1c2f4, 90 },
    { 0xcce1c2f6, 95 },
    { 0x3ee1c2f7, 95 },
    { 0x7721c2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a91c2fb, 95 },
    { 0x6f01c2fc, 90 },
    { 0xb9d1c2fc, 90 },
    { 0x6cb1c2fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8821c302, 88 },
    { 0, 0 },
    { 0x511c304, 98 },
    { 0, 0 },
    { 0x2581c306, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd91c30b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5851c30e, 75 },
    { 0x901c30f, 76 },
    { 0xd381c30f, 95 },
    { 0x7271c30f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e11c31a, 34 },
    { 0xf71c31a, 87 },
    { 0x1851c31b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b1c324, 95 },
    { 0, 0 },
    { 0xc381c326, 46 },
    { 0, 0 },
    { 0x9bd1c328, 77 },
    { 0, 0 },
    { 0x331c32a, 43 },
    { 0x69a1c32a, 34 },
    { 0xc0f1c32c, 81 },
    { 0x9761c32a, 98 },
    { 0x83a1c32e, 39 },
    { 0x64f1c32b, 40 },
    { 0, 0 },
    { 0xf781c331, 95 },
    { 0, 0 },
    { 0x3e1c333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd661c338, 36 },
    { 0, 0 },
    { 0x7f51c33a, 36 },
    { 0x3241c33a, 95 },
    { 0, 0 },
    { 0xc421c33d, 33 },
    { 0x67a1c33e, 43 },
    { 0x9801c33e, 33 },
    { 0x8351c340, 90 },
    { 0xc831c341, 95 },
    { 0xd601c340, 69 },
    { 0x5cc1c342, 55 },
    { 0x75e1c344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b1c34c, 69 },
    { 0, 0 },
    { 0xa0e1c34e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef1c352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb541c35a, 66 },
    { 0xb431c35a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811c363, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1c36a, 99 },
    { 0x6441c36b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2591c36e, 76 },
    { 0, 0 },
    { 0x3031c370, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c81c374, 95 },
    { 0xde31c375, 61 },
    { 0x6271c374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b1c379, 95 },
    { 0xaec1c37a, 46 },
    { 0x65f1c37a, 90 },
    { 0xe891c37b, 98 },
    { 0xb1f1c37b, 93 },
    { 0, 0 },
    { 0x9d31c37f, 61 },
    { 0xbac1c380, 55 },
    { 0x211c381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d1c384, 33 },
    { 0x2161c384, 90 },
    { 0, 0 },
    { 0x84f1c387, 95 },
    { 0, 0 },
    { 0xe231c389, 98 },
    { 0x65e1c38a, 98 },
    { 0, 0 },
    { 0xd961c38c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d91c392, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec1c396, 34 },
    { 0xc21c397, 61 },
    { 0x77f1c398, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3511c39d, 78 },
    { 0x3411c39e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2611c3a3, 98 },
    { 0x76b1c3a4, 61 },
    { 0x26c1c3a5, 33 },
    { 0x841c3a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf1c3ae, 95 },
    { 0xd871c3af, 46 },
    { 0x3761c3ae, 23 },
    { 0xb001c3b0, 61 },
    { 0x8181c3b1, 75 },
    { 0x75e1c3ae, 34 },
    { 0x42a1c3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f21c3c1, 55 },
    { 0x8b01c3c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed31c3cb, 66 },
    { 0xf981c3cc, 33 },
    { 0x2111c3cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa981c3d2, 26 },
    { 0, 0 },
    { 0x9941c3d4, 90 },
    { 0, 0 },
    { 0xee31c3d6, 77 },
    { 0x83e1c3d7, 90 },
    { 0, 0 },
    { 0xc141c3d9, 88 },
    { 0x37b1c3d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181c3df, 33 },
    { 0, 0 },
    { 0x2fe1c3e1, 34 },
    { 0xf741c3e1, 90 },
    { 0xf3e1c3e3, 76 },
    { 0, 0 },
    { 0xbb91c3e5, 69 },
    { 0, 0 },
    { 0x86d1c3e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf861c3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01c3f2, 33 },
    { 0xb9d1c3f2, 98 },
    { 0x91d1c3f4, 98 },
    { 0x9531c3f5, 55 },
    { 0xf0d1c3f6, 61 },
    { 0, 0 },
    { 0xede1c3f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb61c3fb, 85 },
    { 0x2551c3fc, 81 },
    { 0xdcc1c3fc, 61 },
    { 0x681c3fe, 23 },
    { 0x9841c3ff, 77 },
    { 0xeb1c400, 34 },
    { 0x6f11c400, 46 },
    { 0x4281c3fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6941c40c, 65 },
    { 0x1791c40c, 46 },
    { 0x2a81c40d, 19 },
    { 0xa201c40f, 95 },
    { 0x80e1c40d, 33 },
    { 0, 0 },
    { 0x2521c412, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e1c415, 61 },
    { 0, 0 },
    { 0xcea1c417, 78 },
    { 0xfd1c417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111c41e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1c423, 43 },
    { 0xbfd1c424, 61 },
    { 0x2021c423, 95 },
    { 0x9361c426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411c429, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc541c42e, 55 },
    { 0x3541c42f, 23 },
    { 0, 0 },
    { 0x97f1c431, 90 },
    { 0, 0 },
    { 0x7871c433, 93 },
    { 0xae01c434, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e41c43c, 61 },
    { 0xd5e1c43d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e1c440, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9661c444, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d51c448, 90 },
    { 0x7321c448, 33 },
    { 0xc221c44a, 66 },
    { 0x4bc1c449, 93 },
    { 0x6321c44c, 68 },
    { 0x7c71c44c, 61 },
    { 0x4751c44a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f51c458, 76 },
    { 0, 0 },
    { 0xffa1c45a, 91 },
    { 0x2191c45b, 55 },
    { 0x8361c45a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec11c463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd931c469, 90 },
    { 0x30d1c469, 69 },
    { 0x63f1c469, 98 },
    { 0x29c1c46c, 34 },
    { 0x1cd1c46d, 76 },
    { 0, 0 },
    { 0x42e1c46f, 77 },
    { 0, 0 },
    { 0xa511c471, 76 },
    { 0, 0 },
    { 0xeb11c473, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf141c479, 55 },
    { 0, 0 },
    { 0x91d1c47b, 43 },
    { 0x7a91c47c, 38 },
    { 0, 0 },
    { 0x2811c47e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c485, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e1c488, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7861c495, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c1c49b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d1c49e, 33 },
    { 0x4de1c49e, 66 },
    { 0x2cb1c4a0, 84 },
    { 0x34f1c4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf661c4ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc151c4b1, 78 },
    { 0, 0 },
    { 0x46a1c4b3, 61 },
    { 0xaa51c4b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1c4bb, 34 },
    { 0x7961c4bb, 67 },
    { 0xda81c4bc, 90 },
    { 0xb431c4be, 61 },
    { 0x3221c4be, 43 },
    { 0, 0 },
    { 0xbd21c4c1, 90 },
    { 0, 0 },
    { 0xc491c4c3, 48 },
    { 0xdf1c4c4, 98 },
    { 0xaf81c4c4, 90 },
    { 0xd151c4c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e1c4cb, 61 },
    { 0x5631c4cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111c4d0, 33 },
    { 0x2661c4d1, 98 },
    { 0x52e1c4d2, 98 },
    { 0x6621c4d1, 52 },
    { 0x28e1c4d4, 99 },
    { 0x9f1c4d2, 84 },
    { 0xf8d1c4d1, 33 },
    { 0x9cd1c4d2, 98 },
    { 0x7c51c4d5, 99 },
    { 0xf131c4d9, 33 },
    { 0x521c4d7, 55 },
    { 0xc9d1c4d3, 33 },
    { 0xf041c4db, 61 },
    { 0, 0 },
    { 0xb6e1c4de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb51c4e1, 95 },
    { 0, 0 },
    { 0x89c1c4e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa151c4e8, 43 },
    { 0, 0 },
    { 0x72c1c4ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5161c4f0, 55 },
    { 0xc501c4f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771c4f4, 48 },
    { 0, 0 },
    { 0x16e1c4f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e31c4fb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe121c4fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa61c503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021c506, 8 },
    { 0x6321c507, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f1c50b, 33 },
    { 0xcf61c50c, 23 },
    { 0xfb91c50c, 34 },
    { 0, 0 },
    { 0x1481c50f, 34 },
    { 0, 0 },
    { 0xac31c511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a1c514, 84 },
    { 0xe3a1c515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4651c51b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf571c51f, 23 },
    { 0, 0 },
    { 0xa4e1c521, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71c524, 34 },
    { 0xe221c525, 52 },
    { 0xadc1c526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc071c52b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c1c534, 99 },
    { 0xe71c535, 46 },
    { 0xadc1c534, 98 },
    { 0x85b1c537, 33 },
    { 0x22a1c537, 89 },
    { 0x7ff1c537, 19 },
    { 0x7721c536, 98 },
    { 0, 0 },
    { 0x4cd1c53c, 76 },
    { 0, 0 },
    { 0xa7d1c53e, 33 },
    { 0x8cf1c53f, 91 },
    { 0, 0 },
    { 0xd081c541, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8861c544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef1c548, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a1c54f, 33 },
    { 0xc601c54f, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc121c555, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21c55d, 66 },
    { 0xc5b1c55d, 33 },
    { 0, 0 },
    { 0x6241c560, 93 },
    { 0xade1c560, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6861c56a, 43 },
    { 0x3131c56a, 23 },
    { 0x2ce1c56a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3111c574, 33 },
    { 0xf6e1c575, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3041c578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e1c57b, 90 },
    { 0, 0 },
    { 0x38d1c57d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551c580, 91 },
    { 0xa711c581, 34 },
    { 0, 0 },
    { 0x3271c583, 76 },
    { 0, 0 },
    { 0xc881c585, 93 },
    { 0xf471c585, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631c58a, 61 },
    { 0x9451c58a, 95 },
    { 0x42d1c58c, 90 },
    { 0xf081c58d, 43 },
    { 0x6031c58e, 33 },
    { 0xc2c1c58f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931c594, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd291c5a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b61c5a7, 93 },
    { 0, 0 },
    { 0x6ed1c5a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b31c5ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5841c5b3, 77 },
    { 0, 0 },
    { 0x4301c5b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce71c5ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb1c5bf, 66 },
    { 0x40a1c5c0, 101 },
    { 0x4581c5c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7321c5c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6681c5cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a1c5ce, 85 },
    { 0x4d51c5ce, 34 },
    { 0xf7a1c5d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6521c5d4, 17 },
    { 0x4ff1c5d5, 98 },
    { 0xc161c5d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4221c5d9, 90 },
    { 0, 0 },
    { 0x2791c5db, 76 },
    { 0xe9f1c5dc, 96 },
    { 0x6391c5dc, 76 },
    { 0xfb41c5dc, 90 },
    { 0xecb1c5df, 3 },
    { 0x21c5df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa441c5ee, 98 },
    { 0, 0 },
    { 0x7b1c5f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e1c5f4, 78 },
    { 0x8c31c5f5, 23 },
    { 0x2c81c5f5, 66 },
    { 0x6421c5f7, 95 },
    { 0xc421c5f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6211c5fb, 23 },
    { 0x28b1c5fc, 95 },
    { 0xc301c5fd, 23 },
    { 0xbbf1c5fd, 55 },
    { 0xd251c5fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a41c602, 85 },
    { 0xe931c603, 45 },
    { 0xbf11c602, 98 },
    { 0xcab1c604, 33 },
    { 0x12e1c606, 100 },
    { 0xf921c607, 91 },
    { 0x6661c608, 23 },
    { 0x8b31c609, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c21c60d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e31c613, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361c617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead1c61b, 95 },
    { 0, 0 },
    { 0xa191c61d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c61c622, 33 },
    { 0xfc61c622, 61 },
    { 0x2041c624, 100 },
    { 0x3a41c625, 55 },
    { 0, 0 },
    { 0x1c11c627, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391c62f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa321c639, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41c63d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a1c640, 93 },
    { 0x4c21c641, 61 },
    { 0x7a01c642, 52 },
    { 0x7ee1c642, 23 },
    { 0, 0 },
    { 0x1f61c645, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5541c648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc221c64e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2691c652, 40 },
    { 0, 0 },
    { 0xc471c654, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc1c659, 43 },
    { 0xa4f1c65a, 43 },
    { 0xbde1c65b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9391c661, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5141c667, 90 },
    { 0x7071c668, 77 },
    { 0x431c667, 76 },
    { 0xfc91c668, 34 },
    { 0x40b1c66b, 46 },
    { 0xbbc1c668, 75 },
    { 0xa841c66d, 98 },
    { 0, 0 },
    { 0x5201c66f, 23 },
    { 0x7281c66f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb041c675, 33 },
    { 0x5db1c676, 100 },
    { 0x2731c677, 78 },
    { 0x51c1c677, 41 },
    { 0x5d51c675, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec01c67d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2531c688, 66 },
    { 0, 0 },
    { 0x3371c68a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f1c690, 33 },
    { 0xb461c691, 95 },
    { 0, 0 },
    { 0xc751c693, 88 },
    { 0x8111c693, 23 },
    { 0xba11c693, 76 },
    { 0x63d1c696, 55 },
    { 0x5d61c697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa181c69b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c91c69e, 98 },
    { 0x1c41c69f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd1c6a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb21c6a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a81c6ad, 61 },
    { 0xf431c6ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb1c6b1, 90 },
    { 0x7ea1c6b2, 98 },
    { 0x92a1c6b2, 90 },
    { 0x2e41c6b4, 33 },
    { 0x6741c6b5, 98 },
    { 0xdd31c6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c11c6ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb981c6c5, 34 },
    { 0, 0 },
    { 0x5281c6c7, 23 },
    { 0x5191c6c8, 95 },
    { 0x99f1c6c7, 65 },
    { 0, 0 },
    { 0xd331c6cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a51c6ce, 61 },
    { 0xfc41c6cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb1c6d2, 61 },
    { 0xbd01c6d3, 95 },
    { 0x6671c6d3, 34 },
    { 0x5251c6d5, 78 },
    { 0x7141c6d6, 90 },
    { 0x32d1c6d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f61c6da, 90 },
    { 0, 0 },
    { 0xaf21c6dc, 43 },
    { 0x36f1c6dd, 61 },
    { 0xe31c6de, 94 },
    { 0, 0 },
    { 0x7fd1c6e0, 23 },
    { 0xbea1c6e0, 70 },
    { 0x52f1c6e0, 87 },
    { 0, 0 },
    { 0x9e41c6e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5411c6e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e1c6eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621c6ef, 33 },
    { 0x2ac1c6f0, 93 },
    { 0x6331c6ef, 67 },
    { 0, 0 },
    { 0x321c6f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1931c700, 95 },
    { 0x8d01c700, 95 },
    { 0xa351c702, 23 },
    { 0, 0 },
    { 0xf4e1c704, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8321c707, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d91c70c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd1c70f, 90 },
    { 0x6e71c70f, 33 },
    { 0xe551c70f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd371c71a, 93 },
    { 0x7291c71a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591c71f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce1c724, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea1c729, 23 },
    { 0xaec1c72a, 98 },
    { 0xda11c72b, 23 },
    { 0xe7f1c72b, 76 },
    { 0xc831c72a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f51c730, 23 },
    { 0, 0 },
    { 0x4481c732, 90 },
    { 0x53f1c733, 98 },
    { 0xa711c734, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef71c738, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc131c73c, 80 },
    { 0x97c1c73d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d1c741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf1c744, 76 },
    { 0x7ef1c744, 61 },
    { 0xbb01c744, 34 },
    { 0x3791c744, 33 },
    { 0x2d71c748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e81c74d, 52 },
    { 0xfc41c74e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b1c752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a1c755, 90 },
    { 0x4a81c756, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5571c75a, 95 },
    { 0xed1c75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a61c75e, 100 },
    { 0x7381c75f, 95 },
    { 0x7901c75f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa651c763, 90 },
    { 0x8571c764, 35 },
    { 0x6d51c765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b1c768, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d41c76c, 90 },
    { 0xaa51c76d, 90 },
    { 0, 0 },
    { 0x6821c76f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2461c775, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1861c779, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd1c781, 95 },
    { 0, 0 },
    { 0x7fd1c783, 61 },
    { 0, 0 },
    { 0x8191c785, 61 },
    { 0xde61c786, 90 },
    { 0, 0 },
    { 0x8fb1c788, 66 },
    { 0, 0 },
    { 0xbc51c78a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7901c790, 90 },
    { 0, 0 },
    { 0x2291c792, 77 },
    { 0x3171c793, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1c798, 61 },
    { 0x2871c798, 55 },
    { 0x7f01c79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5781c7a2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf061c7a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c1c7a9, 33 },
    { 0, 0 },
    { 0xe81c7ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf111c7ae, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e1c7b4, 93 },
    { 0x21c1c7b4, 52 },
    { 0xf2f1c7b6, 83 },
    { 0x5581c7b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2461c7bb, 91 },
    { 0xfc61c7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c31c7c1, 77 },
    { 0x2351c7c2, 78 },
    { 0x28e1c7c2, 52 },
    { 0x8d61c7c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a1c7c7, 33 },
    { 0x1c7c8, 90 },
    { 0x5381c7c9, 61 },
    { 0x1761c7ca, 90 },
    { 0x61b1c7cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111c7ce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff1c7d7, 93 },
    { 0, 0 },
    { 0x7141c7d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe61c7dc, 78 },
    { 0, 0 },
    { 0x13c1c7de, 94 },
    { 0, 0 },
    { 0x6ea1c7e0, 90 },
    { 0x7b61c7e1, 55 },
    { 0xac1c7e0, 33 },
    { 0xb921c7e2, 95 },
    { 0x7351c7e1, 85 },
    { 0x6a31c7e1, 77 },
    { 0, 0 },
    { 0x7071c7e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb1c7ec, 93 },
    { 0, 0 },
    { 0xd761c7ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7761c7f2, 51 },
    { 0x34f1c7f3, 23 },
    { 0xe761c7f4, 98 },
    { 0, 0 },
    { 0x93e1c7f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef1c7fa, 95 },
    { 0xec11c7fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c21c7fe, 23 },
    { 0xaed1c7ff, 90 },
    { 0x1ae1c800, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671c803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8201c807, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f1c80c, 90 },
    { 0x7641c80d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf1c815, 95 },
    { 0x76e1c816, 95 },
    { 0, 0 },
    { 0xe361c818, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21c81b, 88 },
    { 0, 0 },
    { 0x9401c81d, 23 },
    { 0xd4e1c81d, 83 },
    { 0, 0 },
    { 0x9131c820, 77 },
    { 0x2091c820, 96 },
    { 0, 0 },
    { 0xdb81c823, 46 },
    { 0xb731c823, 90 },
    { 0x4e71c825, 33 },
    { 0xc921c825, 90 },
    { 0, 0 },
    { 0xca31c828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd21c82d, 76 },
    { 0xb121c82e, 90 },
    { 0, 0 },
    { 0x2471c830, 76 },
    { 0, 0 },
    { 0xb4b1c832, 55 },
    { 0xecb1c833, 33 },
    { 0xf911c834, 95 },
    { 0, 0 },
    { 0xc481c836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161c83a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f1c83e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4441c843, 95 },
    { 0x3c11c844, 76 },
    { 0x9681c845, 33 },
    { 0x9661c845, 90 },
    { 0x3261c846, 78 },
    { 0x5fc1c846, 98 },
    { 0xc651c844, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e1c850, 23 },
    { 0xe21c851, 95 },
    { 0xf311c850, 90 },
    { 0xc851c852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd891c858, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c11c863, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb451c866, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6691c86b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5951c872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb1c876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc841c87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf581c87f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2571c883, 55 },
    { 0xaf91c884, 33 },
    { 0x1531c884, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e1c88b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc321c88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec1c892, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd461c89a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f41c89d, 85 },
    { 0x9941c89d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371c8a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1c8a6, 61 },
    { 0x32c1c8a6, 61 },
    { 0, 0 },
    { 0x50c1c8a9, 76 },
    { 0x57f1c8a9, 55 },
    { 0x5071c8aa, 95 },
    { 0x6a31c8ab, 61 },
    { 0x5fd1c8ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2621c8b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1651c8b6, 98 },
    { 0x5d01c8b7, 98 },
    { 0, 0 },
    { 0x3ef1c8b9, 77 },
    { 0x7831c8ba, 95 },
    { 0x8ee1c8ba, 100 },
    { 0xea91c8bc, 45 },
    { 0xb1a1c8bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e71c8c2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c1c8c6, 95 },
    { 0x18b1c8c7, 78 },
    { 0x2b81c8c7, 23 },
    { 0x3d41c8c9, 80 },
    { 0x8f11c8ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c31c8cd, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4041c8d3, 90 },
    { 0xbc71c8d4, 33 },
    { 0x1d01c8d3, 90 },
    { 0xf771c8d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe041c8da, 90 },
    { 0x7c61c8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f1c8de, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d1c8e4, 68 },
    { 0, 0 },
    { 0x1281c8e6, 91 },
    { 0xc531c8e6, 33 },
    { 0x9c61c8e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf01c8ef, 43 },
    { 0x4791c8f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf1c8f4, 33 },
    { 0, 0 },
    { 0x1881c8f6, 100 },
    { 0xe201c8f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd1c900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c61c90d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd381c915, 33 },
    { 0, 0 },
    { 0x7af1c917, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e01c91b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a1c91f, 33 },
    { 0, 0 },
    { 0xa821c921, 90 },
    { 0x8f01c922, 43 },
    { 0x9601c923, 95 },
    { 0xabb1c924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e1c927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea1c92a, 23 },
    { 0x9a61c92a, 23 },
    { 0, 0 },
    { 0x31f1c92d, 76 },
    { 0, 0 },
    { 0x34f1c92f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c1c932, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a1c935, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac1c93a, 33 },
    { 0, 0 },
    { 0x60f1c93c, 23 },
    { 0x5211c93c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231c948, 33 },
    { 0xcc81c949, 95 },
    { 0xcc01c94a, 36 },
    { 0x6af1c94b, 46 },
    { 0, 0 },
    { 0x9901c94d, 84 },
    { 0, 0 },
    { 0x8631c94f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd1c953, 90 },
    { 0x4711c954, 67 },
    { 0xc8c1c955, 91 },
    { 0x3d61c956, 46 },
    { 0xb001c956, 43 },
    { 0x6101c954, 33 },
    { 0xd101c959, 61 },
    { 0xf6f1c953, 52 },
    { 0xffc1c958, 17 },
    { 0x6d1c95c, 36 },
    { 0, 0 },
    { 0xf271c95e, 61 },
    { 0x7fe1c95f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x261c962, 88 },
    { 0xc3c1c963, 35 },
    { 0x6b81c962, 55 },
    { 0x90f1c965, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e11c969, 98 },
    { 0x9741c969, 90 },
    { 0, 0 },
    { 0xb981c96c, 33 },
    { 0x2b91c96c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7551c977, 61 },
    { 0, 0 },
    { 0x5131c979, 61 },
    { 0, 0 },
    { 0x9ed1c97b, 98 },
    { 0x35c1c97b, 90 },
    { 0, 0 },
    { 0xa621c97e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7531c983, 34 },
    { 0, 0 },
    { 0xecd1c985, 98 },
    { 0, 0 },
    { 0x14e1c987, 91 },
    { 0, 0 },
    { 0xb4b1c989, 98 },
    { 0xa8e1c989, 95 },
    { 0xabc1c98a, 99 },
    { 0x2671c98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5091c99c, 95 },
    { 0x99b1c99d, 23 },
    { 0x3d11c99d, 61 },
    { 0xccb1c99f, 23 },
    { 0x3281c99f, 61 },
    { 0, 0 },
    { 0x93d1c9a2, 23 },
    { 0, 0 },
    { 0x6fc1c9a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1c9a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d81c9ad, 43 },
    { 0xa961c9ae, 33 },
    { 0, 0 },
    { 0x51c1c9b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce61c9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61c9bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1c9c0, 61 },
    { 0x4331c9c0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7291c9c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa331c9c7, 46 },
    { 0xbb31c9c8, 95 },
    { 0x8bb1c9c9, 43 },
    { 0x7841c9c7, 76 },
    { 0x9c21c9cb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed1c9d0, 81 },
    { 0x1ae1c9d0, 23 },
    { 0xa781c9d2, 84 },
    { 0xf161c9d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e61c9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a1c9dd, 43 },
    { 0, 0 },
    { 0xcfe1c9df, 61 },
    { 0, 0 },
    { 0xf001c9e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc791c9e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce1c9e7, 33 },
    { 0xcfe1c9e8, 84 },
    { 0xa2a1c9e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7471c9ec, 99 },
    { 0x22a1c9ed, 76 },
    { 0xdf31c9ed, 61 },
    { 0xa921c9ee, 96 },
    { 0xbd11c9f0, 33 },
    { 0xbcf1c9f1, 52 },
    { 0xd241c9f1, 61 },
    { 0, 0 },
    { 0x1531c9f4, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c1c9f7, 95 },
    { 0x9d1c9f8, 61 },
    { 0x6e1c9f7, 95 },
    { 0, 0 },
    { 0x2401c9fb, 61 },
    { 0x3ba1c9fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f1ca02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4441ca05, 61 },
    { 0xbff1ca06, 55 },
    { 0xb7f1ca05, 90 },
    { 0x87f1ca08, 67 },
    { 0x3351ca07, 78 },
    { 0xc8c1ca08, 34 },
    { 0, 0 },
    { 0x3cb1ca0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f31ca15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb51ca19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2681ca1d, 88 },
    { 0x3281ca1d, 34 },
    { 0x8b41ca1f, 95 },
    { 0x9761ca1e, 95 },
    { 0xebf1ca1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e1ca24, 95 },
    { 0x7091ca25, 76 },
    { 0x52a1ca24, 35 },
    { 0x8f91ca27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5611ca2a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7201ca2e, 55 },
    { 0, 0 },
    { 0x15f1ca30, 85 },
    { 0x9411ca31, 43 },
    { 0x7321ca31, 98 },
    { 0xb531ca33, 85 },
    { 0x9f31ca34, 88 },
    { 0x3191ca35, 66 },
    { 0x2fd1ca36, 98 },
    { 0x7c31ca34, 84 },
    { 0xca71ca38, 76 },
    { 0x941ca38, 90 },
    { 0x5c41ca39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c1ca40, 55 },
    { 0, 0 },
    { 0x4361ca42, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd111ca45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca1ca48, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1ca4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5071ca4f, 95 },
    { 0, 0 },
    { 0xbe01ca51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca1ca54, 33 },
    { 0xf51ca55, 23 },
    { 0xa311ca55, 76 },
    { 0x3261ca56, 87 },
    { 0xb221ca58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf791ca5b, 65 },
    { 0x2b81ca5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd11ca61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b91ca65, 43 },
    { 0, 0 },
    { 0x14f1ca67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb341ca6a, 95 },
    { 0x4c61ca6b, 46 },
    { 0x1291ca6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a31ca70, 23 },
    { 0xf7f1ca71, 33 },
    { 0xa081ca72, 98 },
    { 0x6ce1ca73, 52 },
    { 0x66a1ca71, 55 },
    { 0x3f71ca75, 55 },
    { 0xa6d1ca74, 76 },
    { 0x6e71ca77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1ca7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e1ca7d, 34 },
    { 0xe691ca7e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a1ca86, 61 },
    { 0, 0 },
    { 0x6f61ca88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1ca8c, 95 },
    { 0x3a01ca8d, 33 },
    { 0x8e71ca8d, 95 },
    { 0x4dd1ca8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae1ca94, 97 },
    { 0, 0 },
    { 0x2801ca96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce41ca99, 23 },
    { 0xa111ca99, 68 },
    { 0, 0 },
    { 0xb8c1ca9c, 98 },
    { 0, 0 },
    { 0xad11ca9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4501caa2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a1caa5, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4031caab, 33 },
    { 0, 0 },
    { 0xe8d1caad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda1cab6, 95 },
    { 0xbef1cab6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf611cabd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e1cac0, 61 },
    { 0, 0 },
    { 0xfa41cac2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da1cac9, 55 },
    { 0x8081cac9, 98 },
    { 0x5c21cacb, 76 },
    { 0, 0 },
    { 0xc851cacd, 93 },
    { 0x3381cace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b71cad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d91cad9, 76 },
    { 0x9441cada, 88 },
    { 0xb6f1cad9, 94 },
    { 0x40e1cad9, 98 },
    { 0x31c1cadd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a1cae0, 61 },
    { 0, 0 },
    { 0x47c1cae2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1cae7, 33 },
    { 0xf441cae8, 93 },
    { 0xe6e1cae8, 23 },
    { 0xd081cae7, 55 },
    { 0x88e1cae7, 90 },
    { 0, 0 },
    { 0x9b61caed, 93 },
    { 0, 0 },
    { 0x3c51caef, 46 },
    { 0, 0 },
    { 0xb6f1caf1, 46 },
    { 0xac1caf1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1caf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3381cafb, 33 },
    { 0x7961cafc, 23 },
    { 0xc191cafd, 100 },
    { 0xbff1cafb, 34 },
    { 0xd1d1cafe, 98 },
    { 0xe6f1cafe, 33 },
    { 0, 0 },
    { 0x6fe1cb02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461cb05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a71cb0c, 48 },
    { 0x1941cb0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5201cb12, 76 },
    { 0xdc71cb13, 90 },
    { 0, 0 },
    { 0xd371cb15, 23 },
    { 0xd411cb15, 88 },
    { 0xd401cb17, 55 },
    { 0, 0 },
    { 0x3181cb19, 99 },
    { 0x981cb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc051cb1f, 99 },
    { 0x5381cb20, 61 },
    { 0xa031cb20, 23 },
    { 0x1d1cb20, 95 },
    { 0x3ff1cb23, 31 },
    { 0xda11cb24, 23 },
    { 0xdc61cb23, 33 },
    { 0xe511cb26, 90 },
    { 0xf2e1cb27, 33 },
    { 0x2c81cb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4031cb2b, 76 },
    { 0xdac1cb2b, 88 },
    { 0xf7f1cb2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc81cb30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b1cb33, 33 },
    { 0xe651cb33, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd311cb3b, 98 },
    { 0x28b1cb3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e1cb3f, 100 },
    { 0, 0 },
    { 0x5e21cb41, 46 },
    { 0xc231cb42, 55 },
    { 0xcc91cb42, 23 },
    { 0, 0 },
    { 0x6e61cb45, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c71cb48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a1cb4b, 68 },
    { 0xe6d1cb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21cb4f, 67 },
    { 0, 0 },
    { 0x3ba1cb51, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a1cb54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6051cb57, 23 },
    { 0x59d1cb58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01cb5c, 61 },
    { 0xfb21cb5d, 33 },
    { 0x55e1cb5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed1cb64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e1cb67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521cb6b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1cb70, 33 },
    { 0x7891cb71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f01cb76, 93 },
    { 0x1ec1cb77, 98 },
    { 0x12f1cb78, 75 },
    { 0, 0 },
    { 0xdd81cb7a, 93 },
    { 0xaac1cb7b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd151cb80, 91 },
    { 0xe801cb80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7341cb88, 77 },
    { 0x27d1cb89, 33 },
    { 0, 0 },
    { 0xddb1cb8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5631cb8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f1cb92, 76 },
    { 0x3cd1cb93, 80 },
    { 0x3e81cb93, 66 },
    { 0, 0 },
    { 0x9211cb96, 98 },
    { 0, 0 },
    { 0x69b1cb98, 33 },
    { 0x8bf1cb99, 78 },
    { 0x34d1cb9a, 95 },
    { 0, 0 },
    { 0x2871cb9c, 61 },
    { 0x4331cb9d, 61 },
    { 0, 0 },
    { 0xdd61cb9f, 46 },
    { 0, 0 },
    { 0x8cb1cba1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31cba6, 95 },
    { 0x5971cba6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a41cbb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb331cbb6, 91 },
    { 0x5c81cbb7, 46 },
    { 0xf7c1cbb8, 34 },
    { 0, 0 },
    { 0xe8a1cbba, 98 },
    { 0x62b1cbbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c1cbbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e1cbc2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f1cbcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1cbce, 23 },
    { 0x1571cbce, 33 },
    { 0x3081cbd0, 90 },
    { 0x19c1cbd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda1cbd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b1cbd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1cbdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a1cbe2, 17 },
    { 0, 0 },
    { 0x3bb1cbe4, 23 },
    { 0x97b1cbe4, 43 },
    { 0, 0 },
    { 0x2f01cbe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5321cbec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c11cbf1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c1cbf5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251cbf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f1cbff, 40 },
    { 0x5f91cc00, 95 },
    { 0, 0 },
    { 0xec01cc02, 62 },
    { 0x5f11cc03, 23 },
    { 0x9c91cc04, 88 },
    { 0x6311cc05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1811cc0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5891cc15, 43 },
    { 0x7801cc15, 17 },
    { 0, 0 },
    { 0x7961cc18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b61cc1c, 61 },
    { 0x2f41cc1c, 33 },
    { 0x2dc1cc1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda1cc22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd1cc25, 76 },
    { 0, 0 },
    { 0x9041cc27, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a1cc2e, 95 },
    { 0, 0 },
    { 0x45a1cc30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3081cc35, 96 },
    { 0x7601cc36, 90 },
    { 0xe81cc37, 75 },
    { 0xfd41cc38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4101cc3b, 61 },
    { 0, 0 },
    { 0xe611cc3d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xff91cc40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8811cc46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe1cc4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1661cc51, 33 },
    { 0, 0 },
    { 0xab91cc53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1cc5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1861cc5e, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x7831cc61, 87 },
    { 0x42d1cc61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b1cc66, 84 },
    { 0xab11cc67, 33 },
    { 0x43a1cc68, 61 },
    { 0, 0 },
    { 0x91f1cc6a, 76 },
    { 0, 0 },
    { 0xd681cc6c, 61 },
    { 0x4231cc6d, 46 },
    { 0xbbd1cc6d, 23 },
    { 0x9d81cc6f, 76 },
    { 0x7691cc70, 98 },
    { 0x1ba1cc6e, 83 },
    { 0xd3a1cc6e, 34 },
    { 0xec11cc72, 95 },
    { 0x6041cc74, 34 },
    { 0x2801cc75, 100 },
    { 0x7381cc75, 55 },
    { 0x47e1cc77, 76 },
    { 0x6e61cc77, 55 },
    { 0x7661cc79, 65 },
    { 0xf5e1cc76, 33 },
    { 0x9e81cc74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf421cc7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb161cc82, 66 },
    { 0, 0 },
    { 0xe121cc84, 98 },
    { 0, 0 },
    { 0xaac1cc86, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc41cc8c, 33 },
    { 0, 0 },
    { 0x261cc8e, 90 },
    { 0xef51cc8f, 87 },
    { 0, 0 },
    { 0xc271cc91, 76 },
    { 0x8a31cc92, 43 },
    { 0x6321cc93, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461cc99, 90 },
    { 0x6cb1cc9a, 76 },
    { 0x5181cc9a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7911cc9e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e1cca5, 55 },
    { 0xd571cca6, 61 },
    { 0, 0 },
    { 0xd4d1cca8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9461ccab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc431ccb2, 93 },
    { 0xb8e1ccb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce1ccb7, 95 },
    { 0x61d1ccb8, 66 },
    { 0x4d61ccb9, 91 },
    { 0x2481ccba, 61 },
    { 0, 0 },
    { 0x56f1ccbc, 23 },
    { 0x9c71ccbc, 23 },
    { 0, 0 },
    { 0x2131ccbf, 40 },
    { 0xb761ccc0, 61 },
    { 0, 0 },
    { 0x8251ccc2, 87 },
    { 0xcbc1ccc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4601ccc6, 34 },
    { 0xd1a1ccc6, 33 },
    { 0xe241ccc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e1cccc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de1ccd3, 93 },
    { 0x8111ccd4, 98 },
    { 0xf661ccd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4541ccd9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a71ccec, 33 },
    { 0, 0 },
    { 0x24a1ccee, 55 },
    { 0x1c51ccef, 53 },
    { 0x64c1ccee, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc1ccf6, 33 },
    { 0xe071ccf6, 17 },
    { 0x3791ccf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7381ccfb, 23 },
    { 0, 0 },
    { 0x541ccfd, 76 },
    { 0, 0 },
    { 0x9d91ccff, 65 },
    { 0x8771cd00, 61 },
    { 0xa3d1ccff, 55 },
    { 0, 0 },
    { 0x171cd03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d71cd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4741cd09, 34 },
    { 0x2c51cd0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1cd0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901cd10, 65 },
    { 0xa1f1cd11, 32 },
    { 0, 0 },
    { 0x2981cd13, 90 },
    { 0x48a1cd13, 40 },
    { 0, 0 },
    { 0x5d11cd16, 23 },
    { 0, 0 },
    { 0x8c01cd18, 76 },
    { 0, 0 },
    { 0xae71cd1a, 88 },
    { 0x5f1cd1b, 90 },
    { 0x6ba1cd1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed51cd26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3131cd2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a51cd2d, 23 },
    { 0, 0 },
    { 0x4401cd2f, 52 },
    { 0, 0 },
    { 0xc251cd31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e1cd3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd51cd3f, 17 },
    { 0xb9e1cd3f, 100 },
    { 0xbe41cd40, 67 },
    { 0x3871cd42, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941cd48, 90 },
    { 0x4de1cd48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d1cd4e, 55 },
    { 0, 0 },
    { 0x8421cd50, 68 },
    { 0, 0 },
    { 0x7511cd52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f1cd56, 33 },
    { 0x7011cd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb811cd5b, 61 },
    { 0, 0 },
    { 0x6ee1cd5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f1cd60, 76 },
    { 0, 0 },
    { 0xa051cd62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d61cd65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421cd69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d1cd6d, 76 },
    { 0xd01cd6e, 34 },
    { 0, 0 },
    { 0xc991cd70, 23 },
    { 0x9a01cd71, 23 },
    { 0xe2f1cd71, 90 },
    { 0, 0 },
    { 0xa981cd74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11cd7d, 33 },
    { 0xfe21cd7d, 81 },
    { 0, 0 },
    { 0x241cd80, 91 },
    { 0xfbd1cd80, 90 },
    { 0, 0 },
    { 0xf461cd83, 90 },
    { 0xa2f1cd83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341cd88, 85 },
    { 0x41f1cd89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c01cd90, 90 },
    { 0xe491cd91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61cd96, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a1cd9c, 65 },
    { 0x9421cd9d, 77 },
    { 0, 0 },
    { 0x88f1cd9f, 90 },
    { 0xde31cda0, 33 },
    { 0x5b01cda1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1cda4, 69 },
    { 0, 0 },
    { 0xf6a1cda6, 52 },
    { 0, 0 },
    { 0x9af1cda8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c1cdac, 78 },
    { 0xabe1cdad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf601cdb1, 34 },
    { 0x9571cdb2, 95 },
    { 0xf6e1cdb3, 93 },
    { 0, 0 },
    { 0xc11cdb5, 55 },
    { 0, 0 },
    { 0x16c1cdb7, 76 },
    { 0, 0 },
    { 0x3b71cdb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb661cdc0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff21cdcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d1cdcf, 77 },
    { 0x6a31cdd0, 33 },
    { 0xe791cdd1, 98 },
    { 0, 0 },
    { 0xe951cdd3, 33 },
    { 0x13a1cdd4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f41cdd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d1cddd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6081cde3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3401cde6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3621cde9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e1cded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a1cdff, 100 },
    { 0x2d31ce00, 61 },
    { 0xeb51ce00, 61 },
    { 0x5a81ce02, 23 },
    { 0, 0 },
    { 0xe591ce04, 84 },
    { 0x6b11ce05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f1ce09, 76 },
    { 0x7521ce0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe431ce10, 90 },
    { 0x4d41ce10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7501ce14, 90 },
    { 0, 0 },
    { 0x3a91ce16, 98 },
    { 0, 0 },
    { 0xcb31ce18, 34 },
    { 0, 0 },
    { 0xcbb1ce1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc81ce21, 98 },
    { 0, 0 },
    { 0xfdf1ce23, 87 },
    { 0, 0 },
    { 0x5641ce25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1371ce28, 33 },
    { 0, 0 },
    { 0x5b71ce2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e1ce31, 95 },
    { 0x9e21ce32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c1ce39, 61 },
    { 0, 0 },
    { 0x1361ce3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xde41ce3e, 23 },
    { 0x5e51ce3f, 85 },
    { 0xd351ce40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e1ce43, 51 },
    { 0xa521ce43, 100 },
    { 0, 0 },
    { 0x3e81ce46, 55 },
    { 0, 0 },
    { 0x9c31ce48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5111ce4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x961ce50, 95 },
    { 0x1d71ce51, 93 },
    { 0, 0 },
    { 0x2dd1ce53, 34 },
    { 0xbe41ce54, 98 },
    { 0xafa1ce53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a1ce5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b1ce61, 52 },
    { 0x3591ce62, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f1ce67, 98 },
    { 0xba11ce67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c1ce6c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d61ce78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d1ce7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d1ce84, 95 },
    { 0x1d01ce85, 95 },
    { 0x13d1ce86, 33 },
    { 0, 0 },
    { 0xec01ce88, 23 },
    { 0x59a1ce89, 40 },
    { 0, 0 },
    { 0x7b51ce8b, 90 },
    { 0xe2c1ce8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae41ce97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf51ce9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11ce9e, 90 },
    { 0x98f1ce9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a1cea5, 98 },
    { 0x8191cea6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8861cea9, 17 },
    { 0xef41cea9, 91 },
    { 0, 0 },
    { 0xe8d1ceac, 84 },
    { 0x5141cead, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e91ceb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b91ceb6, 91 },
    { 0xc0c1ceb7, 23 },
    { 0, 0 },
    { 0xdbf1ceb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4891cebe, 64 },
    { 0xf611cebf, 78 },
    { 0, 0 },
    { 0xf921cec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade1cec8, 48 },
    { 0, 0 },
    { 0x6251ceca, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a1ced1, 55 },
    { 0x181ced1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21ced8, 90 },
    { 0x6481ced8, 66 },
    { 0x4581ceda, 46 },
    { 0x5951ceda, 98 },
    { 0, 0 },
    { 0xd7f1cedd, 61 },
    { 0xf121cedd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db1cee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5281cee5, 90 },
    { 0, 0 },
    { 0x9f01cee7, 33 },
    { 0xf941cee8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071ceee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111cef1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c1cef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c01cefe, 90 },
    { 0x83d1ceff, 90 },
    { 0x2aa1cf00, 98 },
    { 0xc11cf00, 23 },
    { 0xba71ceff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c71cf06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe1cf11, 33 },
    { 0x1d41cf12, 61 },
    { 0xe321cf12, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc71cf20, 95 },
    { 0x9641cf21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9861cf24, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c1cf2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1cf31, 88 },
    { 0x1401cf32, 34 },
    { 0xc341cf31, 98 },
    { 0x1781cf34, 93 },
    { 0x51f1cf35, 91 },
    { 0x29a1cf32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea1cf3b, 65 },
    { 0xa2b1cf3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf661cf41, 9 },
    { 0, 0 },
    { 0x5261cf43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a1cf47, 90 },
    { 0, 0 },
    { 0x1641cf49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac01cf4c, 61 },
    { 0, 0 },
    { 0xfb11cf4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1691cf51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3131cf55, 99 },
    { 0, 0 },
    { 0x27d1cf57, 61 },
    { 0x20b1cf58, 33 },
    { 0xb791cf59, 90 },
    { 0, 0 },
    { 0x7401cf5b, 33 },
    { 0x2a01cf5c, 61 },
    { 0xe721cf5b, 76 },
    { 0x16e1cf5e, 88 },
    { 0x4171cf5f, 33 },
    { 0x5001cf60, 98 },
    { 0x7441cf61, 34 },
    { 0x9111cf61, 64 },
    { 0xe0b1cf62, 24 },
    { 0x8561cf5d, 34 },
    { 0, 0 },
    { 0xe881cf66, 90 },
    { 0, 0 },
    { 0x95d1cf68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a1cf6c, 61 },
    { 0xbf51cf6c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc881cf70, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc001cf73, 90 },
    { 0x8a1cf74, 55 },
    { 0, 0 },
    { 0x561cf76, 76 },
    { 0xca21cf77, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae81cf7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d1cf81, 93 },
    { 0x1921cf82, 33 },
    { 0x5721cf81, 33 },
    { 0x43f1cf82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe281cf8d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8821cf93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e1cf96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9111cf9b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f1cfa3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1cfab, 23 },
    { 0xee1cfac, 66 },
    { 0, 0 },
    { 0x80f1cfae, 90 },
    { 0xce81cfaf, 91 },
    { 0xe081cfb0, 46 },
    { 0xb191cfb1, 30 },
    { 0x8651cfb1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9141cfbd, 61 },
    { 0, 0 },
    { 0xdc31cfbf, 43 },
    { 0, 0 },
    { 0x8e91cfc1, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5131cfc9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a1cfcd, 90 },
    { 0, 0 },
    { 0x8be1cfcf, 75 },
    { 0xfbf1cfd0, 33 },
    { 0x491cfd0, 34 },
    { 0x6671cfd2, 78 },
    { 0x5c51cfd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c1cfd9, 90 },
    { 0, 0 },
    { 0xe151cfdb, 66 },
    { 0, 0 },
    { 0x6bf1cfdd, 95 },
    { 0, 0 },
    { 0x8bc1cfdf, 67 },
    { 0x7b21cfdf, 90 },
    { 0x4bb1cfe0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b1cfe7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf51cfeb, 61 },
    { 0x42b1cfec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d21cfef, 33 },
    { 0x5481cfef, 98 },
    { 0xd331cff1, 33 },
    { 0xc801cff2, 46 },
    { 0, 0 },
    { 0x171cff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91cff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d51cfff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e1d005, 65 },
    { 0, 0 },
    { 0xcc31d007, 90 },
    { 0xb701d008, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7781d011, 55 },
    { 0, 0 },
    { 0xc401d013, 68 },
    { 0xf761d013, 98 },
    { 0xb2c1d015, 81 },
    { 0x9601d013, 98 },
    { 0x2801d017, 34 },
    { 0, 0 },
    { 0xaf01d019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1891d01d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1d020, 33 },
    { 0x601d020, 95 },
    { 0, 0 },
    { 0xf2f1d023, 95 },
    { 0xebf1d023, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb61d027, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc1d02b, 99 },
    { 0x9351d02c, 33 },
    { 0xc2b1d02d, 91 },
    { 0x3ba1d02d, 43 },
    { 0xa811d02b, 61 },
    { 0x3161d02e, 33 },
    { 0x5a51d031, 43 },
    { 0x64f1d032, 93 },
    { 0x1e61d02c, 78 },
    { 0, 0 },
    { 0xae1d035, 98 },
    { 0x6381d035, 90 },
    { 0x3521d035, 67 },
    { 0x40e1d038, 61 },
    { 0x681d039, 61 },
    { 0xc051d03a, 55 },
    { 0, 0 },
    { 0xe2b1d03c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa1d041, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf1d047, 34 },
    { 0, 0 },
    { 0xba31d049, 61 },
    { 0x1e61d04a, 90 },
    { 0xa121d04b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe41d054, 78 },
    { 0, 0 },
    { 0xa91d056, 95 },
    { 0xc241d057, 98 },
    { 0x6681d057, 33 },
    { 0x43d1d058, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf1d05c, 87 },
    { 0xf9d1d05d, 52 },
    { 0xc161d05d, 95 },
    { 0x81d05f, 76 },
    { 0xa31d060, 95 },
    { 0x701d05c, 75 },
    { 0x2b01d05f, 93 },
    { 0x831d063, 33 },
    { 0x3d21d05c, 61 },
    { 0, 0 },
    { 0xaba1d066, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc561d06e, 75 },
    { 0xf9e1d06f, 90 },
    { 0x6b1d06f, 78 },
    { 0xe3a1d071, 23 },
    { 0xf351d071, 34 },
    { 0xa981d073, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b1d07a, 35 },
    { 0x6a41d07a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6581d07e, 66 },
    { 0x7991d07f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4421d082, 61 },
    { 0x9a71d083, 76 },
    { 0, 0 },
    { 0xf861d085, 61 },
    { 0x6101d086, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc851d089, 78 },
    { 0xccd1d089, 61 },
    { 0xb0a1d08a, 98 },
    { 0x7291d08c, 33 },
    { 0x6fc1d08c, 90 },
    { 0x4d1d08e, 76 },
    { 0x8d71d08b, 98 },
    { 0x10d1d090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2161d093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021d097, 46 },
    { 0xd671d098, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f1d09b, 17 },
    { 0, 0 },
    { 0x2aa1d09d, 76 },
    { 0xd8b1d09e, 61 },
    { 0x18f1d09e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb1d0a2, 33 },
    { 0x8d51d0a3, 68 },
    { 0, 0 },
    { 0xa741d0a5, 33 },
    { 0, 0 },
    { 0x8791d0a7, 90 },
    { 0xe5a1d0a8, 33 },
    { 0x2f11d0a9, 66 },
    { 0x2621d0aa, 33 },
    { 0x59d1d0ab, 55 },
    { 0x87d1d0aa, 43 },
    { 0x7c11d0a9, 23 },
    { 0x77d1d0ae, 55 },
    { 0xffc1d0af, 95 },
    { 0xfa01d0aa, 23 },
    { 0x6851d0b1, 95 },
    { 0x4771d0b2, 76 },
    { 0x1a51d0b2, 23 },
    { 0xee11d0ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a1d0b7, 55 },
    { 0x89b1d0b7, 23 },
    { 0, 0 },
    { 0x6831d0ba, 61 },
    { 0x3611d0bb, 90 },
    { 0x1581d0ba, 66 },
    { 0, 0 },
    { 0xe9c1d0be, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb71d0c1, 66 },
    { 0, 0 },
    { 0x4e41d0c3, 33 },
    { 0x8a81d0c4, 90 },
    { 0x9071d0c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d0c9, 51 },
    { 0, 0 },
    { 0xf761d0cb, 84 },
    { 0xfb31d0cb, 59 },
    { 0x3341d0cd, 91 },
    { 0xd71d0cc, 90 },
    { 0, 0 },
    { 0x5e1d0d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4471d0d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251d0d7, 87 },
    { 0x411d0d7, 23 },
    { 0, 0 },
    { 0x6cd1d0da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6701d0e0, 95 },
    { 0x7811d0e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d31d0e5, 23 },
    { 0xf421d0e6, 90 },
    { 0x70c1d0e7, 33 },
    { 0xd751d0e7, 100 },
    { 0x6351d0e8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd1d0f2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb1d0f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871d0fc, 88 },
    { 0, 0 },
    { 0xa6d1d0fe, 94 },
    { 0xcd91d0ff, 90 },
    { 0x21d100, 91 },
    { 0xb571d100, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf741d107, 61 },
    { 0, 0 },
    { 0xdee1d109, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf851d10e, 93 },
    { 0xff31d10e, 84 },
    { 0x9e11d110, 83 },
    { 0x6c71d10f, 98 },
    { 0x7bc1d112, 90 },
    { 0x58a1d113, 17 },
    { 0xe21d112, 53 },
    { 0x6231d115, 68 },
    { 0x39c1d115, 75 },
    { 0x33d1d117, 78 },
    { 0x3dd1d117, 65 },
    { 0xcde1d117, 95 },
    { 0, 0 },
    { 0xcd51d11b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4931d11e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b11d122, 33 },
    { 0xbfe1d122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1d126, 98 },
    { 0x6671d127, 90 },
    { 0x6d1d127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4701d12d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f1d134, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5941d13c, 77 },
    { 0xfde1d13d, 55 },
    { 0xd3d1d13e, 87 },
    { 0x9351d13f, 99 },
    { 0, 0 },
    { 0x42f1d141, 76 },
    { 0xd191d142, 61 },
    { 0x3381d142, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e1d147, 90 },
    { 0x79b1d147, 96 },
    { 0xb7a1d149, 90 },
    { 0x311d14a, 95 },
    { 0x9ff1d14b, 98 },
    { 0xb971d14c, 75 },
    { 0x5361d14d, 33 },
    { 0x7471d14d, 95 },
    { 0x7571d14e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e71d152, 88 },
    { 0x23c1d153, 46 },
    { 0x34a1d153, 43 },
    { 0xf11d154, 95 },
    { 0x6041d156, 99 },
    { 0x4d21d157, 46 },
    { 0x95e1d158, 23 },
    { 0xe831d155, 61 },
    { 0, 0 },
    { 0xba1d15b, 34 },
    { 0, 0 },
    { 0x4561d15d, 33 },
    { 0xcca1d15d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa91d163, 90 },
    { 0x63e1d164, 95 },
    { 0x10a1d165, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a1d16e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f21d177, 99 },
    { 0x8401d177, 81 },
    { 0x19b1d179, 61 },
    { 0x8b91d177, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c91d17f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd551d183, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb61d189, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb411d191, 23 },
    { 0x6bd1d192, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d1d19c, 90 },
    { 0xe641d19d, 98 },
    { 0xb41d19e, 40 },
    { 0x2b61d19d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f81d1a5, 61 },
    { 0xc141d1a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac1d1a9, 98 },
    { 0x1eb1d1aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391d1ad, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a71d1b1, 95 },
    { 0, 0 },
    { 0xb8c1d1b3, 33 },
    { 0, 0 },
    { 0xcda1d1b5, 99 },
    { 0x6631d1b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac1d1b9, 61 },
    { 0xc71d1ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5351d1c2, 90 },
    { 0, 0 },
    { 0x21a1d1c4, 33 },
    { 0x4491d1c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5871d1cf, 17 },
    { 0x5041d1d0, 76 },
    { 0xf9b1d1cf, 90 },
    { 0x83e1d1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a1d1d5, 81 },
    { 0x2241d1d6, 98 },
    { 0x9a41d1d6, 23 },
    { 0x8041d1d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa41d1e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc11d1ed, 46 },
    { 0, 0 },
    { 0xb0c1d1ef, 78 },
    { 0x1fa1d1ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b1d1f4, 98 },
    { 0, 0 },
    { 0xefe1d1f6, 95 },
    { 0, 0 },
    { 0x8ac1d1f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd81d1fb, 23 },
    { 0xb911d1fb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e41d201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8701d207, 38 },
    { 0, 0 },
    { 0xfde1d209, 81 },
    { 0x8551d209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b1d20e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5181d211, 76 },
    { 0xa3f1d212, 33 },
    { 0, 0 },
    { 0xd2d1d214, 95 },
    { 0xa451d215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de1d21c, 66 },
    { 0x1c31d21d, 34 },
    { 0, 0 },
    { 0x72f1d21f, 98 },
    { 0xc731d21f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e1d223, 87 },
    { 0, 0 },
    { 0x3261d225, 61 },
    { 0x3ce1d225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd611d232, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f31d23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e01d241, 81 },
    { 0xa0f1d241, 34 },
    { 0xc7a1d243, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8631d24d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb71d251, 42 },
    { 0xa4d1d252, 93 },
    { 0x401d251, 34 },
    { 0, 0 },
    { 0xd91d255, 80 },
    { 0x6dd1d256, 23 },
    { 0, 0 },
    { 0x8631d258, 33 },
    { 0, 0 },
    { 0x5b01d25a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d1d25e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b41d263, 33 },
    { 0x64b1d264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa01d267, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5951d26a, 90 },
    { 0x1581d26b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc121d270, 90 },
    { 0xe81d270, 90 },
    { 0x94e1d271, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61d278, 34 },
    { 0xe2a1d279, 52 },
    { 0xe3f1d27a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6661d27f, 61 },
    { 0xfc01d280, 83 },
    { 0xae81d280, 23 },
    { 0x8561d282, 93 },
    { 0xc981d283, 76 },
    { 0x7171d281, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1d287, 93 },
    { 0, 0 },
    { 0xf401d289, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa051d28f, 93 },
    { 0x9c01d28f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881d295, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9001d29b, 33 },
    { 0xf201d29c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d29f, 61 },
    { 0x5571d29f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf11d2a7, 33 },
    { 0xe1c1d2a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b1d2ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2771d2b2, 33 },
    { 0, 0 },
    { 0x66d1d2b4, 76 },
    { 0, 0 },
    { 0x6041d2b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4211d2ba, 90 },
    { 0x14b1d2bb, 46 },
    { 0x51a1d2bc, 23 },
    { 0xa271d2bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb091d2c0, 93 },
    { 0x531d2c0, 90 },
    { 0, 0 },
    { 0x351d2c3, 87 },
    { 0x4c21d2c4, 69 },
    { 0x68d1d2c4, 90 },
    { 0x3bc1d2c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3461d2c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9911d2ce, 90 },
    { 0x62c1d2ce, 55 },
    { 0, 0 },
    { 0xa511d2d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8561d2d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5211d2da, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e1d2dd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf1d2e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261d2e4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c1d2e9, 61 },
    { 0xf4a1d2ea, 90 },
    { 0, 0 },
    { 0x1aa1d2ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf1d2f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e1d2f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31d2fd, 99 },
    { 0, 0 },
    { 0xa4c1d2ff, 23 },
    { 0, 0 },
    { 0xf81d301, 98 },
    { 0xdc91d302, 98 },
    { 0xf151d302, 93 },
    { 0xb371d303, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x591d307, 17 },
    { 0xa1a1d308, 82 },
    { 0, 0 },
    { 0x4c61d30a, 76 },
    { 0x1181d30b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9581d30f, 23 },
    { 0xc5f1d310, 90 },
    { 0x81e1d310, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa631d314, 61 },
    { 0, 0 },
    { 0xe981d316, 67 },
    { 0x8d91d317, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1d31d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91d324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1d32d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb421d337, 88 },
    { 0, 0 },
    { 0x9821d339, 95 },
    { 0, 0 },
    { 0xb3a1d33b, 55 },
    { 0x28d1d33c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91d33f, 17 },
    { 0xfb1d340, 77 },
    { 0xd01d341, 98 },
    { 0xcfa1d342, 78 },
    { 0x8601d342, 52 },
    { 0xec41d341, 34 },
    { 0x9101d345, 23 },
    { 0xc461d340, 23 },
    { 0x6cf1d346, 95 },
    { 0, 0 },
    { 0x3761d349, 43 },
    { 0x2d61d349, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e51d34e, 90 },
    { 0x1b21d34f, 90 },
    { 0x91c1d34f, 75 },
    { 0x4231d351, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f1d357, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d35f, 43 },
    { 0x7a11d360, 55 },
    { 0xec41d361, 90 },
    { 0xd751d362, 64 },
    { 0, 0 },
    { 0xd201d364, 43 },
    { 0x48f1d365, 66 },
    { 0, 0 },
    { 0x45b1d367, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c1d36b, 98 },
    { 0, 0 },
    { 0xe981d36d, 90 },
    { 0xc2e1d36d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb331d371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb01d377, 61 },
    { 0xd061d378, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb1d37c, 98 },
    { 0x27f1d37d, 69 },
    { 0xc5d1d37d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca11d382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1d387, 98 },
    { 0xc261d388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf1d38e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9841d392, 90 },
    { 0, 0 },
    { 0x7bc1d394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8381d399, 33 },
    { 0, 0 },
    { 0x1611d39b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f21d3a0, 90 },
    { 0x1b11d3a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf271d3a4, 61 },
    { 0, 0 },
    { 0xb241d3a6, 33 },
    { 0x8321d3a7, 69 },
    { 0x1af1d3a7, 76 },
    { 0x6fe1d3a8, 23 },
    { 0xa711d3aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d3ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a1d3b2, 46 },
    { 0x4f11d3b3, 23 },
    { 0xb141d3b3, 43 },
    { 0xd331d3b3, 91 },
    { 0x1fa1d3b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c1d3c0, 55 },
    { 0x9ee1d3c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5801d3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e21d3cc, 84 },
    { 0, 0 },
    { 0xc471d3ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f1d3d2, 52 },
    { 0xb211d3d2, 96 },
    { 0x7611d3d4, 84 },
    { 0x6b1d3d2, 98 },
    { 0, 0 },
    { 0x6321d3d7, 93 },
    { 0x1f41d3d8, 98 },
    { 0x9c31d3d9, 76 },
    { 0x2301d3d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a1d3dd, 95 },
    { 0x3951d3de, 43 },
    { 0x62e1d3dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a1d3e6, 55 },
    { 0, 0 },
    { 0x5091d3e8, 61 },
    { 0, 0 },
    { 0xece1d3ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771d3f5, 90 },
    { 0x1b1d3f6, 98 },
    { 0xc301d3f7, 61 },
    { 0xc4e1d3f8, 95 },
    { 0xad61d3f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb1d3fd, 33 },
    { 0xdef1d3fe, 33 },
    { 0x22d1d3fe, 23 },
    { 0x6501d3ff, 55 },
    { 0x6011d401, 33 },
    { 0, 0 },
    { 0x6431d403, 101 },
    { 0x371d404, 61 },
    { 0, 0 },
    { 0x2d31d406, 90 },
    { 0, 0 },
    { 0x8271d408, 34 },
    { 0, 0 },
    { 0xdf01d40a, 98 },
    { 0, 0 },
    { 0x7181d40c, 92 },
    { 0xf3d1d40c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21d410, 48 },
    { 0x5ea1d411, 61 },
    { 0xa1c1d412, 34 },
    { 0x44e1d413, 78 },
    { 0x1d51d413, 61 },
    { 0x7f01d412, 95 },
    { 0x4521d413, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1d41b, 98 },
    { 0, 0 },
    { 0x69f1d41d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4661d421, 95 },
    { 0xc401d422, 37 },
    { 0, 0 },
    { 0x16e1d424, 23 },
    { 0x5f51d425, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce41d42c, 33 },
    { 0x7321d42c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe221d430, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6001d434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd511d438, 33 },
    { 0x9011d439, 95 },
    { 0x6f1d43a, 81 },
    { 0x6471d43a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a1d43e, 52 },
    { 0, 0 },
    { 0xcb21d440, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c1d444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f1d448, 36 },
    { 0, 0 },
    { 0x4a01d44a, 61 },
    { 0, 0 },
    { 0xa3d1d44c, 40 },
    { 0x3b01d44c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351d450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671d455, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf1d459, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc631d45f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf1d462, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e11d465, 90 },
    { 0, 0 },
    { 0x2ce1d467, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81d46d, 34 },
    { 0, 0 },
    { 0xe491d46f, 76 },
    { 0xc61d46f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3641d474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b51d479, 67 },
    { 0, 0 },
    { 0xb051d47b, 67 },
    { 0xd5c1d47c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7801d481, 95 },
    { 0x40a1d482, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf521d485, 33 },
    { 0, 0 },
    { 0xc751d487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91d492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3881d498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a1d49e, 61 },
    { 0x5051d49f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc41d4a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9301d4a5, 36 },
    { 0x11d1d4a6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd1d4aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa391d4ae, 33 },
    { 0xfd81d4af, 95 },
    { 0x1c31d4b0, 95 },
    { 0xed61d4b1, 52 },
    { 0xaed1d4b2, 91 },
    { 0x2401d4b3, 95 },
    { 0xadf1d4b4, 90 },
    { 0xebe1d4b5, 43 },
    { 0x1b61d4b5, 23 },
    { 0x11d1d4b7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a1d4ba, 61 },
    { 0xb891d4bb, 95 },
    { 0xac21d4bc, 67 },
    { 0x54d1d4ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x911d4c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9511d4c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd01d4c6, 34 },
    { 0x1531d4c7, 66 },
    { 0x4cc1d4c7, 55 },
    { 0x94c1d4c7, 34 },
    { 0x92a1d4c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d4ce, 91 },
    { 0x3c81d4ce, 43 },
    { 0x9ac1d4cf, 100 },
    { 0, 0 },
    { 0xfcb1d4d2, 23 },
    { 0, 0 },
    { 0x411d4d4, 98 },
    { 0x1971d4d5, 95 },
    { 0x2bd1d4d6, 46 },
    { 0, 0 },
    { 0x75e1d4d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b1d4df, 81 },
    { 0x88a1d4e0, 34 },
    { 0x9dc1d4df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee1d4e8, 100 },
    { 0xdf31d4e9, 61 },
    { 0, 0 },
    { 0x6ab1d4eb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c1d4f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb281d4f9, 55 },
    { 0x351d4f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3551d4fd, 98 },
    { 0, 0 },
    { 0x6981d4ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac1d503, 55 },
    { 0x5221d504, 91 },
    { 0x7441d505, 90 },
    { 0x591d506, 33 },
    { 0xe261d505, 46 },
    { 0x3e51d508, 90 },
    { 0, 0 },
    { 0xe361d50a, 100 },
    { 0xcda1d50a, 100 },
    { 0, 0 },
    { 0xef1d50d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf81d511, 48 },
    { 0x6f41d512, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea1d515, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e81d51a, 52 },
    { 0xc211d51b, 33 },
    { 0xbca1d51b, 87 },
    { 0, 0 },
    { 0x3e81d51e, 33 },
    { 0x18d1d51f, 55 },
    { 0x3ee1d51e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a11d525, 76 },
    { 0, 0 },
    { 0x5e41d527, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a1d52a, 91 },
    { 0xfc61d52b, 98 },
    { 0x1ad1d52c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e1d532, 85 },
    { 0xc4b1d533, 61 },
    { 0x8581d533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f1d53d, 43 },
    { 0x271d53e, 99 },
    { 0x38b1d53e, 33 },
    { 0xebd1d53d, 23 },
    { 0xbb11d53f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec1d544, 61 },
    { 0x90d1d545, 52 },
    { 0xe791d546, 55 },
    { 0, 0 },
    { 0xaea1d548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a1d54d, 76 },
    { 0x8dc1d54e, 77 },
    { 0xd9d1d54e, 43 },
    { 0, 0 },
    { 0xc921d551, 91 },
    { 0, 0 },
    { 0xbc1d553, 34 },
    { 0x4881d554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7041d558, 61 },
    { 0x22d1d559, 77 },
    { 0xed01d55a, 8 },
    { 0xe411d55b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a61d55f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf191d562, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c61d565, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1d56f, 90 },
    { 0, 0 },
    { 0x3ee1d571, 76 },
    { 0, 0 },
    { 0xd591d573, 76 },
    { 0xf5f1d573, 33 },
    { 0, 0 },
    { 0xd511d576, 98 },
    { 0xadf1d577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b1d57d, 83 },
    { 0x87e1d57d, 98 },
    { 0, 0 },
    { 0xada1d580, 95 },
    { 0, 0 },
    { 0x7491d582, 90 },
    { 0x1611d583, 85 },
    { 0xf281d583, 99 },
    { 0x9c01d585, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f1d58a, 93 },
    { 0xb551d58a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5891d590, 93 },
    { 0, 0 },
    { 0x1701d592, 95 },
    { 0xfd1d593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b31d598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e1d5a3, 78 },
    { 0xebc1d5a3, 99 },
    { 0x86c1d5a5, 61 },
    { 0x39e1d5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa421d5aa, 33 },
    { 0, 0 },
    { 0xdc91d5ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1211d5af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5221d5b2, 87 },
    { 0x3811d5b2, 61 },
    { 0x6081d5b4, 17 },
    { 0, 0 },
    { 0xe2c1d5b6, 90 },
    { 0xc0c1d5b6, 95 },
    { 0, 0 },
    { 0xb161d5b9, 46 },
    { 0xa8b1d5b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae1d5c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e1d5c3, 90 },
    { 0xe871d5c4, 96 },
    { 0xd231d5c5, 68 },
    { 0, 0 },
    { 0x7b41d5c7, 61 },
    { 0x8f11d5c8, 90 },
    { 0x5e91d5c7, 43 },
    { 0x8b21d5ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d5cd, 97 },
    { 0, 0 },
    { 0xe271d5cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef1d5d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1d5d9, 58 },
    { 0xaf1d5da, 91 },
    { 0xf7f1d5d9, 99 },
    { 0xe451d5d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed1d5e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd1d5e8, 85 },
    { 0x83f1d5e8, 61 },
    { 0x6d31d5ea, 44 },
    { 0x4f81d5eb, 90 },
    { 0xce21d5ec, 61 },
    { 0xf321d5e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe351d5f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b1d5fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6641d5fe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf1d601, 23 },
    { 0x3351d602, 98 },
    { 0xf091d602, 76 },
    { 0, 0 },
    { 0xfdf1d605, 55 },
    { 0xae11d606, 95 },
    { 0x36a1d607, 87 },
    { 0, 0 },
    { 0x5fa1d609, 33 },
    { 0x4741d60a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe161d612, 23 },
    { 0, 0 },
    { 0xdc91d614, 33 },
    { 0x4721d615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4341d618, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde31d61c, 43 },
    { 0x8c31d61d, 55 },
    { 0, 0 },
    { 0x88c1d61f, 95 },
    { 0x8241d620, 33 },
    { 0xd6d1d621, 99 },
    { 0x9ea1d622, 95 },
    { 0x8241d622, 33 },
    { 0x1501d624, 93 },
    { 0x10b1d625, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271d62d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb1d630, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c51d635, 90 },
    { 0, 0 },
    { 0x2ea1d637, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d1d63a, 84 },
    { 0x4a31d63b, 95 },
    { 0x5521d63a, 98 },
    { 0x5ad1d63d, 18 },
    { 0x5591d63d, 34 },
    { 0x8331d63f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb91d644, 93 },
    { 0xdec1d644, 90 },
    { 0xdcb1d646, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a1d64a, 88 },
    { 0xa251d64a, 67 },
    { 0xb321d64b, 76 },
    { 0xcfb1d64c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a1d655, 55 },
    { 0x5c31d656, 46 },
    { 0, 0 },
    { 0xf5a1d658, 34 },
    { 0, 0 },
    { 0x25a1d65a, 95 },
    { 0x6cf1d65a, 69 },
    { 0x9f1d65b, 34 },
    { 0xcb31d65c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4591d660, 89 },
    { 0xfc11d661, 66 },
    { 0xc991d660, 55 },
    { 0xce1d663, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f1d668, 93 },
    { 0x52c1d669, 61 },
    { 0, 0 },
    { 0xa61d66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3261d671, 95 },
    { 0, 0 },
    { 0x9771d673, 98 },
    { 0x7d51d674, 85 },
    { 0, 0 },
    { 0xf691d676, 97 },
    { 0x8631d676, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321d67b, 78 },
    { 0x10a1d67b, 34 },
    { 0x4751d67c, 46 },
    { 0xdf21d67e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a1d681, 90 },
    { 0x1921d682, 66 },
    { 0, 0 },
    { 0xa401d684, 46 },
    { 0x6421d685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e31d68f, 90 },
    { 0, 0 },
    { 0xd251d691, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5371d694, 99 },
    { 0x19f1d695, 17 },
    { 0xc611d695, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7381d69d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe191d6a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c41d6ae, 93 },
    { 0xde21d6af, 66 },
    { 0x9d71d6ae, 90 },
    { 0x8971d6b1, 33 },
    { 0x8cc1d6b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf171d6b6, 61 },
    { 0x9361d6b6, 33 },
    { 0x18f1d6b6, 90 },
    { 0x8771d6b9, 98 },
    { 0x9661d6ba, 23 },
    { 0xd3f1d6bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe91d6c2, 23 },
    { 0x62c1d6c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc771d6c6, 75 },
    { 0, 0 },
    { 0x5431d6c8, 95 },
    { 0, 0 },
    { 0x4171d6ca, 61 },
    { 0, 0 },
    { 0xc201d6cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc411d6d0, 84 },
    { 0x46a1d6d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d6da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc01d6dd, 90 },
    { 0x3991d6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161d6e1, 23 },
    { 0xeb1d6e2, 95 },
    { 0, 0 },
    { 0xef41d6e4, 80 },
    { 0x8371d6e4, 33 },
    { 0, 0 },
    { 0x8941d6e7, 33 },
    { 0x9841d6e8, 33 },
    { 0x311d6e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8791d6ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d71d6f3, 55 },
    { 0x5371d6f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4641d6f9, 55 },
    { 0, 0 },
    { 0xf271d6fb, 84 },
    { 0x76c1d6fb, 90 },
    { 0x6ea1d6fd, 46 },
    { 0x1381d6fd, 33 },
    { 0xc971d6fd, 34 },
    { 0xf3c1d700, 93 },
    { 0x6321d6ff, 61 },
    { 0x4361d702, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b31d705, 43 },
    { 0x6ed1d705, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e1d709, 66 },
    { 0x8571d70a, 61 },
    { 0, 0 },
    { 0xd5a1d70c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf751d712, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b11d716, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3901d71b, 77 },
    { 0x67c1d71c, 98 },
    { 0xb3a1d71d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb791d727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd171d72b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6811d72e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b71d732, 33 },
    { 0, 0 },
    { 0xa3f1d734, 93 },
    { 0, 0 },
    { 0x6711d736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1111d73a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d1d743, 81 },
    { 0x5871d744, 76 },
    { 0xbfc1d743, 90 },
    { 0x4c1d746, 40 },
    { 0xc8c1d747, 19 },
    { 0xa101d748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1d74e, 90 },
    { 0, 0 },
    { 0xae31d750, 98 },
    { 0, 0 },
    { 0xa4a1d752, 33 },
    { 0, 0 },
    { 0x82c1d754, 95 },
    { 0x3131d754, 93 },
    { 0x2411d756, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf431d75b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a91d75f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf1d770, 19 },
    { 0xb491d771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1961d778, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a1d77d, 90 },
    { 0xddb1d77d, 78 },
    { 0xa651d77f, 95 },
    { 0, 0 },
    { 0xdb1d781, 87 },
    { 0xa391d782, 55 },
    { 0xc931d783, 93 },
    { 0, 0 },
    { 0x1561d785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2661d789, 61 },
    { 0x331d789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd771d78e, 98 },
    { 0x8eb1d78f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f1d798, 80 },
    { 0xec81d799, 90 },
    { 0xefc1d798, 93 },
    { 0xf491d79b, 61 },
    { 0x5e91d79b, 77 },
    { 0x1be1d79a, 34 },
    { 0x1071d79e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1d7a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff81d7a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d7a8, 61 },
    { 0, 0 },
    { 0x7f21d7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab1d7b0, 46 },
    { 0, 0 },
    { 0x92c1d7b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4691d7b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4381d7b9, 95 },
    { 0xa5f1d7ba, 23 },
    { 0, 0 },
    { 0x8561d7bc, 76 },
    { 0xe0e1d7bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f1d7c0, 33 },
    { 0, 0 },
    { 0xb381d7c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2361d7c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6541d7ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1d7d1, 55 },
    { 0, 0 },
    { 0xfed1d7d3, 67 },
    { 0xa9c1d7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe071d7d7, 23 },
    { 0x6951d7d7, 76 },
    { 0xc781d7d9, 85 },
    { 0x9821d7da, 85 },
    { 0xd451d7d7, 90 },
    { 0xbe71d7d8, 66 },
    { 0xdbf1d7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f1d7e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a1d7e9, 90 },
    { 0x3e61d7e9, 23 },
    { 0xe121d7eb, 53 },
    { 0x8d31d7e9, 99 },
    { 0x46c1d7ed, 93 },
    { 0xdf61d7ee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d7f2, 34 },
    { 0, 0 },
    { 0x6a81d7f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b1d7fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a1d800, 56 },
    { 0xa5b1d801, 91 },
    { 0x61c1d800, 75 },
    { 0x41d1d803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae1d807, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9921d80a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e1d80d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c11d812, 95 },
    { 0x9551d813, 90 },
    { 0x3121d814, 90 },
    { 0xee81d814, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c1d819, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2931d81c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c1d821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b41d824, 90 },
    { 0xbe81d825, 98 },
    { 0xcce1d826, 66 },
    { 0xd271d825, 33 },
    { 0x74b1d828, 23 },
    { 0x2301d828, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc631d832, 95 },
    { 0, 0 },
    { 0x80d1d834, 90 },
    { 0xa4d1d835, 98 },
    { 0x4491d835, 77 },
    { 0xd3a1d837, 66 },
    { 0x8731d835, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1d83b, 33 },
    { 0xaf41d83c, 98 },
    { 0xcad1d83d, 98 },
    { 0x67e1d83d, 95 },
    { 0x6791d83e, 90 },
    { 0, 0 },
    { 0x6bb1d841, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c1d846, 33 },
    { 0, 0 },
    { 0x1251d848, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd521d851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1d854, 77 },
    { 0, 0 },
    { 0x5bf1d856, 90 },
    { 0x4ce1d857, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x4161d85a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d85f, 23 },
    { 0x8a01d85f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a1d864, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f71d868, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e1d86c, 33 },
    { 0xdb61d86d, 66 },
    { 0xfe1d86e, 55 },
    { 0x3ea1d86e, 87 },
    { 0, 0 },
    { 0x6f91d871, 17 },
    { 0xfc81d871, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2821d87b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3001d87f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411d883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4171d886, 68 },
    { 0x57c1d887, 98 },
    { 0xdd01d888, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101d88d, 98 },
    { 0x131d88e, 55 },
    { 0x6061d88d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb321d893, 66 },
    { 0x2a61d893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb501d899, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe391d89c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7211d8a7, 77 },
    { 0x8f11d8a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3831d8ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd1d8b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e31d8b5, 26 },
    { 0, 0 },
    { 0xcc11d8b7, 99 },
    { 0, 0 },
    { 0x6041d8b9, 68 },
    { 0, 0 },
    { 0xd8d1d8bb, 61 },
    { 0x9491d8bc, 34 },
    { 0xcea1d8bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec1d8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1741d8c8, 94 },
    { 0x4171d8c9, 90 },
    { 0xf001d8ca, 61 },
    { 0x9731d8cb, 61 },
    { 0xd6b1d8cc, 34 },
    { 0x3ac1d8cb, 23 },
    { 0xa801d8ce, 44 },
    { 0x6bb1d8c8, 78 },
    { 0x24a1d8d0, 95 },
    { 0x2ed1d8cd, 91 },
    { 0x8a31d8cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b61d8d6, 55 },
    { 0xf2d1d8d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901d8db, 35 },
    { 0, 0 },
    { 0x3b1d8dd, 91 },
    { 0x69e1d8dd, 23 },
    { 0, 0 },
    { 0xca61d8e0, 76 },
    { 0x9621d8e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b1d8eb, 55 },
    { 0x8681d8ec, 33 },
    { 0x6701d8ec, 43 },
    { 0xf521d8ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf1d8f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b31d8f6, 33 },
    { 0x1b01d8f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91d8fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da1d903, 46 },
    { 0xcde1d903, 52 },
    { 0xb7e1d905, 98 },
    { 0x3a1d906, 43 },
    { 0xba81d907, 33 },
    { 0x6491d903, 67 },
    { 0xdaa1d909, 90 },
    { 0xcf31d906, 95 },
    { 0, 0 },
    { 0x9521d90c, 23 },
    { 0xdd41d90d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b1d911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe871d914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca1d918, 91 },
    { 0, 0 },
    { 0x9541d91a, 99 },
    { 0xe7d1d91b, 76 },
    { 0x3d31d91c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd1d920, 98 },
    { 0x451d921, 95 },
    { 0x1fd1d922, 33 },
    { 0x6521d923, 65 },
    { 0x7861d920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc1d928, 23 },
    { 0, 0 },
    { 0x9351d92a, 98 },
    { 0xd961d92b, 77 },
    { 0, 0 },
    { 0xcc1d92d, 61 },
    { 0, 0 },
    { 0xa471d92f, 43 },
    { 0x6de1d930, 95 },
    { 0xb1b1d931, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd1d936, 93 },
    { 0x6571d937, 78 },
    { 0x3331d937, 90 },
    { 0x7a91d938, 23 },
    { 0x7641d93a, 87 },
    { 0, 0 },
    { 0xde61d93c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa321d93f, 98 },
    { 0xcee1d940, 33 },
    { 0, 0 },
    { 0x3061d942, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab1d947, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed21d94b, 95 },
    { 0, 0 },
    { 0x9ad1d94d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6281d951, 61 },
    { 0x7491d951, 61 },
    { 0, 0 },
    { 0x86f1d954, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1681d95f, 98 },
    { 0, 0 },
    { 0x2f11d961, 98 },
    { 0, 0 },
    { 0x8621d963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c41d966, 78 },
    { 0x1881d967, 61 },
    { 0xbd41d967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab11d96c, 36 },
    { 0, 0 },
    { 0x72e1d96e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1331d973, 61 },
    { 0x75d1d974, 90 },
    { 0x5cf1d974, 46 },
    { 0x6be1d973, 34 },
    { 0, 0 },
    { 0xb8c1d978, 98 },
    { 0xb1e1d979, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261d97c, 95 },
    { 0x3441d97d, 90 },
    { 0xebb1d97c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d1d983, 34 },
    { 0, 0 },
    { 0x2651d985, 33 },
    { 0xfc1d985, 52 },
    { 0x7ee1d985, 76 },
    { 0x5011d988, 98 },
    { 0x77d1d989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb1d98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a1d992, 77 },
    { 0xe81d992, 91 },
    { 0x4b01d994, 46 },
    { 0x9801d992, 52 },
    { 0x14e1d996, 98 },
    { 0x88a1d995, 61 },
    { 0xe871d998, 91 },
    { 0xf081d998, 61 },
    { 0x70f1d996, 95 },
    { 0xea11d995, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9961d99f, 33 },
    { 0x9701d9a0, 98 },
    { 0, 0 },
    { 0xc201d9a2, 90 },
    { 0x261d9a3, 76 },
    { 0, 0 },
    { 0xed1d9a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d1d9a8, 68 },
    { 0x21b1d9a9, 56 },
    { 0xaf01d9aa, 98 },
    { 0xed81d9aa, 64 },
    { 0x3501d9ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81d9af, 33 },
    { 0x2091d9af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa571d9b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e91d9c1, 52 },
    { 0xd3a1d9c2, 55 },
    { 0, 0 },
    { 0xa261d9c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c1d9c7, 33 },
    { 0, 0 },
    { 0xb181d9c9, 90 },
    { 0x28d1d9ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be1d9cd, 90 },
    { 0, 0 },
    { 0xa881d9cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb531d9d2, 33 },
    { 0, 0 },
    { 0xe4e1d9d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2521d9d9, 23 },
    { 0xfc81d9da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631d9e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31d9e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd941d9ea, 23 },
    { 0xf761d9ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a1d9f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc71d9f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2831d9fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1da04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6531da07, 90 },
    { 0x5681da07, 61 },
    { 0xb5a1da09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b81da0d, 87 },
    { 0, 0 },
    { 0x9691da0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e1da12, 95 },
    { 0, 0 },
    { 0xa041da14, 76 },
    { 0x8731da14, 90 },
    { 0x7e21da16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6441da1a, 65 },
    { 0xbba1da1b, 34 },
    { 0x2691da1b, 95 },
    { 0x4a51da1d, 90 },
    { 0xa561da1a, 90 },
    { 0xfc81da1f, 61 },
    { 0xa761da20, 98 },
    { 0xcb81da20, 90 },
    { 0x11d1da22, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f41da26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c91da2b, 98 },
    { 0x21e1da2c, 61 },
    { 0x7891da2c, 95 },
    { 0x6fa1da2e, 95 },
    { 0x7211da2f, 91 },
    { 0x14c1da2c, 23 },
    { 0xa4c1da2e, 76 },
    { 0x7871da2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a1da35, 61 },
    { 0x3ae1da36, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271da39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c11da3c, 90 },
    { 0x93a1da3d, 98 },
    { 0x6541da3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed41da45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe201da48, 36 },
    { 0, 0 },
    { 0xfd21da4a, 34 },
    { 0x14c1da4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec91da4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e71da59, 61 },
    { 0x1da5a, 61 },
    { 0x4001da5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f71da61, 34 },
    { 0x8ff1da61, 33 },
    { 0x63a1da63, 61 },
    { 0x6b01da64, 34 },
    { 0x6d1da65, 61 },
    { 0x2d51da63, 84 },
    { 0, 0 },
    { 0xf761da68, 98 },
    { 0, 0 },
    { 0xa1f1da6a, 52 },
    { 0xf6c1da6a, 65 },
    { 0x9251da6a, 87 },
    { 0xb6e1da6b, 34 },
    { 0x3531da6e, 91 },
    { 0x3301da6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3011da73, 84 },
    { 0, 0 },
    { 0xdeb1da75, 33 },
    { 0x1f91da75, 76 },
    { 0, 0 },
    { 0xd031da78, 75 },
    { 0, 0 },
    { 0xd651da7a, 95 },
    { 0x2b91da7b, 76 },
    { 0x1111da7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb711da83, 85 },
    { 0xbbe1da84, 90 },
    { 0x3e71da85, 76 },
    { 0xbdd1da86, 78 },
    { 0x5c31da87, 75 },
    { 0, 0 },
    { 0x5451da89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe81da8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51da91, 43 },
    { 0xf6f1da92, 52 },
    { 0xad81da91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb011da96, 98 },
    { 0x6271da97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4281da9b, 90 },
    { 0xaa51da9b, 34 },
    { 0, 0 },
    { 0x7cd1da9e, 52 },
    { 0x7501da9e, 23 },
    { 0x8cb1daa0, 93 },
    { 0x78e1daa1, 76 },
    { 0x3551daa2, 64 },
    { 0xdc01daa2, 78 },
    { 0x1bc1daa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb1dab3, 81 },
    { 0x4cf1dab4, 95 },
    { 0x3fd1dab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7951dab9, 33 },
    { 0, 0 },
    { 0x7501dabb, 33 },
    { 0xb631dabc, 33 },
    { 0x8401dabc, 76 },
    { 0xcf41dabc, 34 },
    { 0, 0 },
    { 0x5f51dac0, 95 },
    { 0x2ef1dac1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x321dac4, 98 },
    { 0, 0 },
    { 0x9761dac6, 76 },
    { 0, 0 },
    { 0x77c1dac8, 82 },
    { 0, 0 },
    { 0x9421daca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b11dacd, 100 },
    { 0, 0 },
    { 0xe351dacf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1dad2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd531dad5, 33 },
    { 0xffc1dad6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c71dadd, 23 },
    { 0, 0 },
    { 0x45f1dadf, 95 },
    { 0, 0 },
    { 0x3db1dae1, 99 },
    { 0x6f11dae1, 43 },
    { 0, 0 },
    { 0x83f1dae4, 55 },
    { 0xb7c1dae5, 90 },
    { 0xdc01dae6, 52 },
    { 0x49d1dae7, 65 },
    { 0x6bb1dae8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411daec, 61 },
    { 0xfb1daed, 33 },
    { 0x5c11daee, 85 },
    { 0xe31daef, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a1daf2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b31daf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab1dafd, 76 },
    { 0, 0 },
    { 0x9fc1daff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d1db04, 52 },
    { 0x50c1db05, 95 },
    { 0xd471db06, 43 },
    { 0, 0 },
    { 0x6dd1db08, 43 },
    { 0x84e1db09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba1db0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a91db14, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1db18, 76 },
    { 0, 0 },
    { 0x3b41db1a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d1db1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe71db24, 33 },
    { 0xca41db24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c41db2d, 46 },
    { 0x3a11db2e, 34 },
    { 0x5d31db2f, 67 },
    { 0xcf91db30, 76 },
    { 0xb721db2f, 90 },
    { 0, 0 },
    { 0xb6a1db33, 85 },
    { 0, 0 },
    { 0xd9f1db35, 55 },
    { 0x3091db36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe371db39, 55 },
    { 0, 0 },
    { 0x7c51db3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa361db3e, 43 },
    { 0xed1db3e, 66 },
    { 0, 0 },
    { 0xc4e1db41, 33 },
    { 0x5901db42, 95 },
    { 0xd541db43, 64 },
    { 0xec51db41, 95 },
    { 0x5cc1db45, 33 },
    { 0x67f1db46, 55 },
    { 0x4421db47, 55 },
    { 0x1701db41, 75 },
    { 0x1351db49, 92 },
    { 0xa8e1db4a, 80 },
    { 0, 0 },
    { 0x1651db4c, 61 },
    { 0xd551db4c, 61 },
    { 0xa0b1db4e, 33 },
    { 0x4b51db4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc411db54, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d11db58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e1db5b, 34 },
    { 0x2fd1db5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee1db5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee81db64, 76 },
    { 0x5f41db64, 55 },
    { 0, 0 },
    { 0xc3c1db67, 85 },
    { 0x91c1db68, 23 },
    { 0x6c61db68, 100 },
    { 0x36e1db68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1db6d, 98 },
    { 0, 0 },
    { 0x6291db6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e41db77, 85 },
    { 0x33c1db78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc11db7d, 76 },
    { 0xa011db7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1181db82, 78 },
    { 0x271db83, 34 },
    { 0x14c1db82, 90 },
    { 0, 0 },
    { 0x9e21db86, 84 },
    { 0x4bb1db87, 90 },
    { 0x83f1db88, 61 },
    { 0x55f1db89, 98 },
    { 0xb7b1db8a, 98 },
    { 0x9111db8b, 98 },
    { 0xcca1db8c, 98 },
    { 0x3491db8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6611db94, 33 },
    { 0xfc61db95, 91 },
    { 0x7de1db96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1db9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b1dba3, 33 },
    { 0xd1f1dba4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1dbac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51dbaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71dbbd, 61 },
    { 0x81d1dbbe, 76 },
    { 0xe9b1dbbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf281dbc5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc671dbc8, 46 },
    { 0x1c31dbc8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d1dbcc, 95 },
    { 0, 0 },
    { 0x94c1dbce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771dbd1, 23 },
    { 0x2011dbd2, 90 },
    { 0x74f1dbd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11dbdc, 23 },
    { 0, 0 },
    { 0x3951dbde, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb531dbe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d1dbe6, 55 },
    { 0, 0 },
    { 0xfff1dbe8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1dbf0, 76 },
    { 0x32d1dbf0, 93 },
    { 0x7251dbf2, 17 },
    { 0xc961dbf2, 76 },
    { 0, 0 },
    { 0xedc1dbf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa701dbf8, 17 },
    { 0x2e71dbf8, 61 },
    { 0x8ff1dbf9, 33 },
    { 0x541dbfb, 61 },
    { 0x7211dbfb, 95 },
    { 0, 0 },
    { 0x89b1dbfe, 77 },
    { 0x54f1dbff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c01dc04, 98 },
    { 0, 0 },
    { 0x1211dc06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b1dc09, 23 },
    { 0, 0 },
    { 0x2741dc0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271dc0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771dc13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c41dc19, 76 },
    { 0, 0 },
    { 0xa311dc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a21dc1e, 33 },
    { 0x3a01dc1e, 90 },
    { 0, 0 },
    { 0x75c1dc21, 90 },
    { 0x6a91dc22, 88 },
    { 0, 0 },
    { 0x2101dc24, 52 },
    { 0, 0 },
    { 0x2391dc26, 33 },
    { 0, 0 },
    { 0x451dc28, 100 },
    { 0x3281dc28, 76 },
    { 0x9271dc28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab71dc2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3551dc31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf551dc3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5201dc46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a1dc4a, 55 },
    { 0, 0 },
    { 0xd5c1dc4c, 93 },
    { 0x9761dc4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21dc50, 34 },
    { 0, 0 },
    { 0x5c41dc52, 90 },
    { 0xb161dc52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6121dc58, 33 },
    { 0, 0 },
    { 0xba11dc5a, 69 },
    { 0x2371dc5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a1dc5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b81dc63, 33 },
    { 0x6f51dc64, 23 },
    { 0x2fb1dc63, 33 },
    { 0xbc11dc66, 90 },
    { 0x17d1dc67, 66 },
    { 0xc641dc67, 95 },
    { 0xe161dc69, 33 },
    { 0xa1c1dc69, 75 },
    { 0x6d81dc69, 55 },
    { 0x38d1dc6c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9041dc72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe601dc77, 90 },
    { 0x78d1dc77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7151dc7b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f1dc7e, 66 },
    { 0x7051dc7f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d51dc82, 46 },
    { 0xcd41dc83, 33 },
    { 0x3ee1dc84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba1dc88, 61 },
    { 0xbcb1dc89, 91 },
    { 0x56b1dc88, 40 },
    { 0x1be1dc8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a1dc91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4401dc95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc1dc98, 23 },
    { 0x3e21dc98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3061dc9c, 95 },
    { 0, 0 },
    { 0x6dd1dc9e, 23 },
    { 0, 0 },
    { 0xf301dca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6781dca4, 91 },
    { 0, 0 },
    { 0x4e21dca6, 33 },
    { 0x7521dca7, 34 },
    { 0, 0 },
    { 0x8681dca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7101dcb1, 76 },
    { 0xc701dcb1, 80 },
    { 0xa131dcb2, 23 },
    { 0, 0 },
    { 0x49a1dcb5, 99 },
    { 0xcae1dcb6, 34 },
    { 0xd21dcb7, 34 },
    { 0xd031dcb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101dcc1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4041dcc4, 33 },
    { 0x1861dcc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6711dcc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b1dccf, 66 },
    { 0x9911dcd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c1dcd3, 34 },
    { 0, 0 },
    { 0xbb1dcd5, 61 },
    { 0x8cb1dcd6, 98 },
    { 0xa0f1dcd6, 34 },
    { 0xb6b1dcd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6491dcdf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf161dce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa91dce6, 90 },
    { 0x7ff1dce6, 61 },
    { 0xc4d1dce8, 99 },
    { 0x9fd1dce8, 97 },
    { 0x7af1dce8, 46 },
    { 0x3f41dceb, 93 },
    { 0, 0 },
    { 0x69a1dced, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b1dcf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de1dcf3, 84 },
    { 0, 0 },
    { 0x2531dcf5, 61 },
    { 0x8251dcf6, 98 },
    { 0x57b1dcf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe231dcfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe11dd03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d1dd08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed1dd16, 90 },
    { 0, 0 },
    { 0x8581dd18, 23 },
    { 0, 0 },
    { 0x6a41dd1a, 34 },
    { 0, 0 },
    { 0x6b31dd1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd231dd1f, 95 },
    { 0, 0 },
    { 0x5461dd21, 98 },
    { 0xeac1dd21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b1dd2a, 61 },
    { 0xdd61dd2a, 90 },
    { 0x2011dd2c, 90 },
    { 0, 0 },
    { 0xaf71dd2e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e51dd35, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa231dd38, 98 },
    { 0, 0 },
    { 0xcf71dd3a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e1dd40, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba1dd43, 46 },
    { 0x84f1dd44, 87 },
    { 0, 0 },
    { 0x69b1dd46, 43 },
    { 0xaed1dd47, 23 },
    { 0xd711dd48, 61 },
    { 0x6641dd49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d1dd4c, 90 },
    { 0xd4e1dd4c, 68 },
    { 0x39b1dd4c, 34 },
    { 0xbfb1dd4f, 67 },
    { 0x2551dd4f, 95 },
    { 0xdac1dd51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c1dd55, 64 },
    { 0, 0 },
    { 0xb541dd57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61dd5b, 90 },
    { 0x67c1dd5c, 61 },
    { 0, 0 },
    { 0x95e1dd5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811dd6a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d1dd6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e1dd72, 98 },
    { 0x4e61dd72, 84 },
    { 0xd281dd74, 95 },
    { 0x3b11dd72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f1dd80, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4571dd84, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae71dd8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd8e, 23 },
    { 0x2741dd8f, 90 },
    { 0, 0 },
    { 0x5ec1dd91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa281dd95, 33 },
    { 0, 0 },
    { 0xf621dd97, 34 },
    { 0, 0 },
    { 0x96b1dd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c51dd9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5381dda1, 36 },
    { 0, 0 },
    { 0x5191dda3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd1dda6, 92 },
    { 0xa171dda7, 77 },
    { 0x6c1dda8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c81ddac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21ddb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3951ddb7, 34 },
    { 0x68f1ddb7, 90 },
    { 0x7411ddb9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1ddbc, 66 },
    { 0x97e1ddbc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e1ddc0, 61 },
    { 0, 0 },
    { 0xf041ddc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb191ddc5, 99 },
    { 0, 0 },
    { 0x53a1ddc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9451ddca, 61 },
    { 0, 0 },
    { 0xf4f1ddcc, 33 },
    { 0xe1e1ddcc, 98 },
    { 0x2d01ddce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b1ddd2, 100 },
    { 0xd1e1ddd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc511ddda, 98 },
    { 0x83d1dddb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1dddf, 33 },
    { 0x6361dddf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc071dde8, 75 },
    { 0x4391dde8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7591ddec, 55 },
    { 0, 0 },
    { 0xfd71ddee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e1ddf2, 61 },
    { 0x91d1ddf3, 23 },
    { 0x47f1ddf4, 38 },
    { 0x94b1ddf4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a31ddf9, 90 },
    { 0xc621ddfa, 55 },
    { 0xa1d1ddfb, 23 },
    { 0x7a01ddfc, 46 },
    { 0x1161ddfc, 93 },
    { 0, 0 },
    { 0xade1ddff, 76 },
    { 0xe851ddff, 33 },
    { 0x8d81de01, 46 },
    { 0xc4e1de02, 46 },
    { 0x98d1de00, 33 },
    { 0x2221de04, 46 },
    { 0x35d1de05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4571de0a, 76 },
    { 0x88e1de0b, 65 },
    { 0, 0 },
    { 0x4171de0d, 80 },
    { 0, 0 },
    { 0xfc91de0f, 33 },
    { 0xbc91de10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e1de19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb81de1f, 74 },
    { 0xafb1de20, 90 },
    { 0x17f1de20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf591de25, 99 },
    { 0, 0 },
    { 0xc931de27, 40 },
    { 0x2fd1de28, 99 },
    { 0xe8f1de29, 90 },
    { 0, 0 },
    { 0x4a41de2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb1de2e, 76 },
    { 0, 0 },
    { 0xaa1de30, 43 },
    { 0x8c51de31, 34 },
    { 0, 0 },
    { 0x36b1de33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf1de36, 90 },
    { 0x64e1de37, 95 },
    { 0x3101de37, 61 },
    { 0xb061de38, 55 },
    { 0, 0 },
    { 0x3a71de3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc1de3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc741de43, 90 },
    { 0x7061de44, 98 },
    { 0xd9a1de44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c1de49, 43 },
    { 0xd8d1de4a, 23 },
    { 0x2701de4b, 95 },
    { 0xfcc1de4c, 23 },
    { 0xb4f1de49, 98 },
    { 0x7be1de4e, 76 },
    { 0xaa1de4f, 101 },
    { 0xd8f1de4c, 95 },
    { 0, 0 },
    { 0x40d1de52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7101de56, 77 },
    { 0x5681de57, 98 },
    { 0xccd1de58, 33 },
    { 0xa611de59, 33 },
    { 0x6cf1de59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f1de5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61de60, 76 },
    { 0x9d01de60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631de67, 56 },
    { 0xfac1de67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d31de6d, 33 },
    { 0x4a11de6d, 87 },
    { 0, 0 },
    { 0x69e1de70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1de73, 91 },
    { 0xf121de73, 93 },
    { 0x3091de75, 90 },
    { 0, 0 },
    { 0x27a1de77, 90 },
    { 0, 0 },
    { 0x7ef1de79, 61 },
    { 0, 0 },
    { 0x1001de7b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb1de81, 76 },
    { 0x83f1de82, 76 },
    { 0xd441de83, 23 },
    { 0xd6d1de83, 90 },
    { 0xe3c1de83, 67 },
    { 0x48a1de84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb21de89, 38 },
    { 0xb701de89, 33 },
    { 0x3d01de8a, 23 },
    { 0, 0 },
    { 0xb871de8d, 19 },
    { 0x60a1de8d, 87 },
    { 0xc701de8d, 61 },
    { 0, 0 },
    { 0x4841de91, 90 },
    { 0xa131de92, 77 },
    { 0xe651de92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf261de97, 90 },
    { 0x20d1de98, 33 },
    { 0xf791de99, 61 },
    { 0x7a31de98, 33 },
    { 0, 0 },
    { 0x35e1de9c, 75 },
    { 0, 0 },
    { 0x45a1de9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1741dea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581dea6, 99 },
    { 0, 0 },
    { 0x11c1dea8, 61 },
    { 0x87e1dea9, 64 },
    { 0x4651deaa, 76 },
    { 0xf721dea9, 61 },
    { 0xdea1deac, 43 },
    { 0, 0 },
    { 0x8ba1deae, 33 },
    { 0xd371deaf, 95 },
    { 0, 0 },
    { 0x8161deb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd971deb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa651dec3, 66 },
    { 0x1da1dec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef51ded0, 55 },
    { 0xa7a1ded1, 61 },
    { 0x62b1ded2, 90 },
    { 0xc6a1ded0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb1ded6, 93 },
    { 0xffb1ded7, 78 },
    { 0x3f51ded7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a1dedb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4291dee5, 40 },
    { 0, 0 },
    { 0x1e1dee7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4071deee, 23 },
    { 0x2de1deee, 33 },
    { 0x96d1def0, 61 },
    { 0xddc1def1, 23 },
    { 0, 0 },
    { 0x7f01def3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391defa, 91 },
    { 0x2d41defb, 90 },
    { 0x5701defa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7361deff, 66 },
    { 0x9241deff, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451df0e, 100 },
    { 0x37f1df0e, 84 },
    { 0xecb1df10, 90 },
    { 0x8e01df11, 66 },
    { 0xf7c1df12, 74 },
    { 0xb0d1df0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b1df1a, 61 },
    { 0, 0 },
    { 0x9671df1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc1df1f, 43 },
    { 0x2401df20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51df24, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c1df28, 61 },
    { 0xc381df28, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd21df32, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb1df35, 46 },
    { 0x6451df35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21df3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a61df41, 90 },
    { 0, 0 },
    { 0x6881df43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5311df47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc1df4c, 91 },
    { 0xd141df4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea1df50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf081df54, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf11df57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a1df5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c1df5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f1df62, 68 },
    { 0, 0 },
    { 0x6971df64, 90 },
    { 0x20b1df65, 33 },
    { 0x5501df65, 61 },
    { 0, 0 },
    { 0xdf51df68, 95 },
    { 0xa561df68, 43 },
    { 0, 0 },
    { 0x6881df6b, 95 },
    { 0x8ae1df6c, 34 },
    { 0x53f1df6b, 91 },
    { 0, 0 },
    { 0x4631df6f, 95 },
    { 0x1c01df70, 46 },
    { 0x4c1df70, 34 },
    { 0x7de1df72, 93 },
    { 0x3661df73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc51df7c, 51 },
    { 0xcec1df7d, 91 },
    { 0, 0 },
    { 0xb661df7f, 98 },
    { 0xdfa1df80, 33 },
    { 0xbe71df81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c31df84, 34 },
    { 0x34a1df85, 36 },
    { 0, 0 },
    { 0x20d1df87, 81 },
    { 0xb8b1df87, 69 },
    { 0xab1df87, 55 },
    { 0x551df8a, 95 },
    { 0xb371df8a, 84 },
    { 0, 0 },
    { 0xd291df8d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa891df90, 84 },
    { 0xb421df91, 89 },
    { 0xb751df92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd981df95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d1df99, 33 },
    { 0x111df9a, 55 },
    { 0x7d91df9b, 93 },
    { 0xa311df9c, 75 },
    { 0x2461df9d, 90 },
    { 0x5071df9e, 61 },
    { 0x1391df9d, 98 },
    { 0xe8e1df9f, 94 },
    { 0x9e21dfa1, 34 },
    { 0x7061dfa2, 76 },
    { 0xa1a1dfa3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f1dfa7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa1dfad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f1dfb2, 75 },
    { 0, 0 },
    { 0x69f1dfb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1dfbb, 76 },
    { 0x3581dfbc, 55 },
    { 0, 0 },
    { 0x6961dfbe, 77 },
    { 0xc501dfbf, 61 },
    { 0x10b1dfc0, 40 },
    { 0xf041dfc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61dfc5, 90 },
    { 0xd691dfc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c31dfcc, 33 },
    { 0x1001dfcc, 22 },
    { 0x9ff1dfce, 46 },
    { 0xe711dfce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa1dfd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c1dfdb, 91 },
    { 0x9ae1dfdc, 98 },
    { 0, 0 },
    { 0x4061dfde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6631dfe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec1dfec, 93 },
    { 0x9921dfed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb021dff1, 61 },
    { 0x6291dff2, 23 },
    { 0, 0 },
    { 0x5701dff4, 94 },
    { 0xd981dff4, 77 },
    { 0xc3c1dff6, 52 },
    { 0x27b1dff6, 98 },
    { 0, 0 },
    { 0xb71dff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf011dffd, 95 },
    { 0, 0 },
    { 0x66f1dfff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a81e009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b01e00d, 40 },
    { 0, 0 },
    { 0x9a81e00f, 81 },
    { 0xc421e00f, 23 },
    { 0x7f61e011, 52 },
    { 0xb421e012, 81 },
    { 0xa211e00f, 90 },
    { 0x90b1e014, 48 },
    { 0, 0 },
    { 0x9d81e016, 91 },
    { 0xf531e017, 90 },
    { 0x23d1e018, 33 },
    { 0, 0 },
    { 0xeeb1e01a, 98 },
    { 0, 0 },
    { 0x3791e01c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c91e021, 76 },
    { 0x2661e022, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd361e026, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc261e02a, 61 },
    { 0xb1a1e02b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9341e02f, 95 },
    { 0x4ac1e030, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b1e034, 52 },
    { 0, 0 },
    { 0x6481e036, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f1e047, 98 },
    { 0x2da1e048, 81 },
    { 0x5eb1e049, 61 },
    { 0x4a11e048, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c1e04d, 90 },
    { 0x4351e04e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11e055, 40 },
    { 0xb271e056, 66 },
    { 0, 0 },
    { 0x79c1e058, 49 },
    { 0x3181e059, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da1e05c, 90 },
    { 0xf361e05d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe791e060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e064, 84 },
    { 0x2331e065, 98 },
    { 0, 0 },
    { 0x42c1e067, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac1e06b, 52 },
    { 0xd051e06c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1001e076, 85 },
    { 0x6231e076, 33 },
    { 0x9ae1e076, 90 },
    { 0x7581e079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7451e07d, 81 },
    { 0xdd11e07d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e083, 98 },
    { 0x6981e084, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb461e091, 90 },
    { 0xbbd1e092, 87 },
    { 0x33b1e093, 23 },
    { 0x75d1e091, 34 },
    { 0xfc21e095, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a1e09d, 87 },
    { 0x3ea1e09e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7251e0a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e1e0a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9791e0af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8141e0b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f81e0b9, 90 },
    { 0x34e1e0ba, 76 },
    { 0x1cb1e0bb, 93 },
    { 0xcbd1e0bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a1e0c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e1e0c4, 52 },
    { 0, 0 },
    { 0x51c1e0c6, 76 },
    { 0x4051e0c7, 46 },
    { 0x3e41e0c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1e0d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c1e0da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b1e0e3, 90 },
    { 0xbc61e0e3, 90 },
    { 0xb891e0e4, 90 },
    { 0, 0 },
    { 0xc8d1e0e7, 55 },
    { 0x8151e0e8, 61 },
    { 0x80d1e0e9, 33 },
    { 0xcfd1e0e9, 23 },
    { 0x8141e0eb, 90 },
    { 0xbd1e0e9, 43 },
    { 0x14a1e0ed, 99 },
    { 0xa8e1e0ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b91e0f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6481e0f4, 83 },
    { 0x8751e0f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab1e0f8, 98 },
    { 0xc771e0f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1021e101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae31e106, 76 },
    { 0xfa11e107, 76 },
    { 0, 0 },
    { 0x2eb1e109, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e1e10d, 90 },
    { 0, 0 },
    { 0xeed1e10f, 17 },
    { 0xfe41e110, 33 },
    { 0xb821e111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a1e114, 90 },
    { 0xb261e115, 93 },
    { 0x1b11e115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4451e11a, 43 },
    { 0xeff1e11b, 90 },
    { 0x6411e11b, 23 },
    { 0xdd01e11d, 76 },
    { 0x53e1e11d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c1e124, 33 },
    { 0, 0 },
    { 0xdd81e126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7781e133, 98 },
    { 0xe4b1e134, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7461e137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc1e13a, 99 },
    { 0x4561e13b, 61 },
    { 0x5ed1e13b, 61 },
    { 0x4bd1e13d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff71e141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b91e14a, 76 },
    { 0x531e14a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab1e14f, 33 },
    { 0x3b11e150, 95 },
    { 0x53b1e150, 65 },
    { 0xc1c1e150, 75 },
    { 0, 0 },
    { 0x87b1e154, 43 },
    { 0xa411e155, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e1e159, 95 },
    { 0xae1e15a, 77 },
    { 0x3701e15b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe731e160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4821e163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd91e167, 98 },
    { 0x3951e168, 34 },
    { 0xfbb1e169, 52 },
    { 0x8e61e168, 78 },
    { 0x1d1e16b, 95 },
    { 0xac51e168, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae1e16f, 61 },
    { 0xbd21e170, 65 },
    { 0xd9b1e16f, 91 },
    { 0x95c1e172, 93 },
    { 0x5c41e173, 61 },
    { 0x4a11e171, 34 },
    { 0xd1e175, 17 },
    { 0x58c1e176, 55 },
    { 0xe5f1e177, 34 },
    { 0x7901e178, 91 },
    { 0, 0 },
    { 0x3cc1e17a, 100 },
    { 0x55a1e17b, 33 },
    { 0xd6a1e17b, 90 },
    { 0xe7e1e17b, 98 },
    { 0xfa81e17b, 95 },
    { 0, 0 },
    { 0xf551e180, 95 },
    { 0xa111e181, 93 },
    { 0x1581e182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d1e18c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e1e191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5601e197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a1e19b, 55 },
    { 0x92a1e19b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d61e1a4, 61 },
    { 0, 0 },
    { 0x8701e1a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6871e1ae, 33 },
    { 0x4ca1e1ae, 98 },
    { 0xa571e1b0, 61 },
    { 0, 0 },
    { 0x3421e1b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6721e1ba, 100 },
    { 0x83e1e1bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41e1bf, 48 },
    { 0xeef1e1bf, 55 },
    { 0, 0 },
    { 0x5d91e1c2, 61 },
    { 0xc271e1c2, 85 },
    { 0, 0 },
    { 0x23f1e1c5, 23 },
    { 0x86b1e1c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a1e1cb, 55 },
    { 0x1831e1cc, 52 },
    { 0x55d1e1cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b1e1d1, 33 },
    { 0, 0 },
    { 0xf941e1d3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241e1d8, 33 },
    { 0x3861e1d9, 34 },
    { 0, 0 },
    { 0xa781e1db, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa1e1e3, 98 },
    { 0x2b71e1e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb841e1e7, 53 },
    { 0xdb91e1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4101e1ef, 95 },
    { 0x80e1e1ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc681e1f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f1e1fd, 67 },
    { 0xe771e1fe, 88 },
    { 0x3421e1fe, 76 },
    { 0, 0 },
    { 0x6011e201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f1e206, 55 },
    { 0xb361e206, 47 },
    { 0, 0 },
    { 0xfa71e209, 95 },
    { 0, 0 },
    { 0x5251e20b, 61 },
    { 0, 0 },
    { 0xb451e20d, 88 },
    { 0xba41e20d, 90 },
    { 0xd81e20f, 61 },
    { 0x4ac1e20f, 61 },
    { 0xdd71e211, 23 },
    { 0x8f01e20e, 61 },
    { 0x3711e213, 95 },
    { 0x3c11e213, 94 },
    { 0x4fb1e215, 33 },
    { 0, 0 },
    { 0x2ab1e217, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9751e21b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde61e224, 34 },
    { 0xde81e225, 33 },
    { 0xa2c1e226, 46 },
    { 0x7f01e227, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d91e22d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9931e230, 76 },
    { 0, 0 },
    { 0xfce1e232, 33 },
    { 0, 0 },
    { 0x3641e234, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac1e237, 46 },
    { 0x31a1e238, 48 },
    { 0xd391e239, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6911e23d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791e242, 33 },
    { 0, 0 },
    { 0x551e244, 40 },
    { 0x4a71e244, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1e249, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a1e24d, 43 },
    { 0, 0 },
    { 0x74e1e24f, 33 },
    { 0xf501e250, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d51e254, 33 },
    { 0x9c71e254, 46 },
    { 0, 0 },
    { 0xd1a1e257, 76 },
    { 0x8b51e258, 46 },
    { 0x7911e258, 90 },
    { 0x19f1e25a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d51e261, 33 },
    { 0x1561e261, 100 },
    { 0xd161e263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b1e268, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be1e26e, 75 },
    { 0, 0 },
    { 0xd871e270, 33 },
    { 0, 0 },
    { 0xca1e272, 23 },
    { 0, 0 },
    { 0x6201e274, 33 },
    { 0x1c1e274, 93 },
    { 0x3f81e276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1e280, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce71e285, 77 },
    { 0xd851e285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1e289, 33 },
    { 0, 0 },
    { 0xe641e28b, 93 },
    { 0, 0 },
    { 0xdde1e28d, 90 },
    { 0, 0 },
    { 0x2921e28f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e81e294, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e1e297, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c1e29a, 66 },
    { 0xe8e1e29b, 75 },
    { 0x3491e29a, 90 },
    { 0x2281e29d, 38 },
    { 0xa11e29d, 67 },
    { 0, 0 },
    { 0x32c1e2a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba1e2a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721e2ac, 33 },
    { 0x7dc1e2ac, 76 },
    { 0x2071e2ae, 81 },
    { 0, 0 },
    { 0x9281e2b0, 61 },
    { 0x13d1e2b1, 95 },
    { 0xc561e2b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2371e2b5, 36 },
    { 0, 0 },
    { 0x1221e2b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe251e2bb, 93 },
    { 0x2d91e2bb, 47 },
    { 0x98d1e2bd, 90 },
    { 0, 0 },
    { 0x15d1e2bf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5741e2c9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc81e2cc, 55 },
    { 0x5fb1e2cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941e2d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e31e2d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a1e2dd, 33 },
    { 0xeca1e2de, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2091e2e1, 68 },
    { 0x7e11e2e2, 95 },
    { 0, 0 },
    { 0x8d81e2e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc31e2e8, 55 },
    { 0xb091e2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde1e2ec, 23 },
    { 0, 0 },
    { 0x4351e2ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb021e2f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81e2f8, 78 },
    { 0, 0 },
    { 0x3e51e2fa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c1e2fd, 66 },
    { 0x38a1e2fd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe541e301, 93 },
    { 0x1d21e302, 77 },
    { 0x4ac1e301, 61 },
    { 0, 0 },
    { 0xf671e305, 98 },
    { 0x3f91e306, 33 },
    { 0x5201e306, 23 },
    { 0x4021e308, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b1e30d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b1e310, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7941e318, 55 },
    { 0x2631e319, 90 },
    { 0, 0 },
    { 0xb4b1e31b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a51e31e, 33 },
    { 0x1d51e31e, 46 },
    { 0xd681e320, 55 },
    { 0x1f51e320, 36 },
    { 0, 0 },
    { 0x21c1e323, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b01e32b, 88 },
    { 0, 0 },
    { 0xc911e32d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8411e330, 33 },
    { 0x7131e331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd31e33b, 77 },
    { 0, 0 },
    { 0x8411e33d, 88 },
    { 0x1b41e33e, 51 },
    { 0x8b21e33f, 95 },
    { 0x9741e340, 66 },
    { 0xa3b1e341, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e1e344, 90 },
    { 0, 0 },
    { 0xa701e346, 33 },
    { 0x1541e346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b91e34e, 98 },
    { 0x6931e34f, 99 },
    { 0, 0 },
    { 0xb671e351, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc431e357, 66 },
    { 0, 0 },
    { 0xbde1e359, 43 },
    { 0xd941e35a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8861e35d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc121e360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff1e366, 95 },
    { 0, 0 },
    { 0xf9e1e368, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c1e36b, 95 },
    { 0, 0 },
    { 0xcf81e36d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe1e370, 46 },
    { 0xf331e371, 90 },
    { 0xc051e372, 98 },
    { 0x3581e373, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a1e378, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f51e383, 100 },
    { 0xf61e383, 91 },
    { 0xa781e385, 95 },
    { 0x93c1e383, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c1e38a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e51e38d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd81e392, 68 },
    { 0, 0 },
    { 0xe9e1e394, 34 },
    { 0x1e01e394, 98 },
    { 0xff11e396, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8721e39a, 98 },
    { 0xe4b1e39a, 95 },
    { 0xfe71e39c, 95 },
    { 0x7701e39d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b1e3a0, 93 },
    { 0, 0 },
    { 0x471e3a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb851e3aa, 83 },
    { 0xe5e1e3ab, 55 },
    { 0xa6d1e3ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8761e3b0, 95 },
    { 0, 0 },
    { 0x7b91e3b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4651e3bc, 100 },
    { 0, 0 },
    { 0x3671e3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4771e3c1, 33 },
    { 0x8321e3c2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d71e3c5, 43 },
    { 0x7741e3c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b1e3ca, 23 },
    { 0x491e3cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7871e3d9, 85 },
    { 0x9c31e3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa881e3de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01e3e1, 98 },
    { 0, 0 },
    { 0x6891e3e3, 61 },
    { 0xa671e3e4, 85 },
    { 0x2431e3e4, 87 },
    { 0x1321e3e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce41e3e9, 95 },
    { 0x5751e3e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4151e3ed, 23 },
    { 0x6001e3ee, 76 },
    { 0xd9d1e3ee, 93 },
    { 0xbfa1e3f0, 90 },
    { 0x3221e3ee, 90 },
    { 0xc851e3f1, 90 },
    { 0xce51e3f2, 33 },
    { 0xb9a1e3f4, 76 },
    { 0x6771e3f5, 78 },
    { 0xb501e3f6, 36 },
    { 0x6c71e3f7, 23 },
    { 0x941e3f6, 91 },
    { 0x5481e3f6, 98 },
    { 0x5d71e3fa, 93 },
    { 0x4581e3fb, 23 },
    { 0, 0 },
    { 0x9b51e3fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc481e400, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e1e403, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3101e40f, 90 },
    { 0xb71e40f, 99 },
    { 0, 0 },
    { 0xada1e412, 34 },
    { 0x23f1e413, 76 },
    { 0, 0 },
    { 0xe2c1e415, 80 },
    { 0x9891e416, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd651e419, 34 },
    { 0, 0 },
    { 0xb451e41b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c1e41f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6731e42b, 90 },
    { 0, 0 },
    { 0x48f1e42d, 87 },
    { 0, 0 },
    { 0x46e1e42f, 55 },
    { 0, 0 },
    { 0xd331e431, 53 },
    { 0x1621e432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b1e435, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab1e439, 67 },
    { 0, 0 },
    { 0x3c91e43b, 98 },
    { 0x66f1e43b, 81 },
    { 0xf1f1e43c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d1e443, 33 },
    { 0, 0 },
    { 0xf981e445, 90 },
    { 0x5301e446, 67 },
    { 0x6511e447, 14 },
    { 0, 0 },
    { 0x7461e449, 66 },
    { 0, 0 },
    { 0xfd11e44b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf941e44e, 33 },
    { 0xcf41e44e, 23 },
    { 0xcfd1e450, 76 },
    { 0xee91e451, 88 },
    { 0x6ba1e452, 85 },
    { 0xe641e451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab1e459, 90 },
    { 0x59b1e45a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8121e45d, 82 },
    { 0x3461e45e, 61 },
    { 0, 0 },
    { 0xe621e460, 2 },
    { 0xb461e461, 76 },
    { 0x3081e462, 90 },
    { 0x1dc1e462, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf71e466, 90 },
    { 0xd3e1e466, 87 },
    { 0xb901e467, 100 },
    { 0x6f01e469, 33 },
    { 0x7bf1e467, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c1e46f, 98 },
    { 0x4551e470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab1e473, 33 },
    { 0xeaf1e474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d41e477, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f11e47f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d1e483, 46 },
    { 0xde51e484, 90 },
    { 0, 0 },
    { 0x5801e486, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461e48b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a21e490, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea81e494, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c1e49a, 23 },
    { 0xa671e49b, 61 },
    { 0xadb1e49c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd1e49f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc1e4aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6641e4ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f1e4b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf681e4b7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1e4be, 33 },
    { 0x4541e4be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1001e4c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8371e4c5, 93 },
    { 0x8591e4c5, 76 },
    { 0, 0 },
    { 0xe3c1e4c8, 34 },
    { 0, 0 },
    { 0xe181e4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc1e4cd, 84 },
    { 0, 0 },
    { 0xc3f1e4cf, 78 },
    { 0, 0 },
    { 0x44f1e4d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a1e4da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a51e4e4, 69 },
    { 0, 0 },
    { 0xd0d1e4e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b1e4eb, 55 },
    { 0x93e1e4ec, 67 },
    { 0xead1e4ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111e4f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd091e4f6, 68 },
    { 0x3851e4f6, 55 },
    { 0, 0 },
    { 0xff61e4f9, 34 },
    { 0, 0 },
    { 0x2ac1e4fb, 52 },
    { 0x76d1e4fc, 34 },
    { 0x7911e4fb, 33 },
    { 0, 0 },
    { 0x8121e4ff, 34 },
    { 0xe961e4ff, 43 },
    { 0, 0 },
    { 0xd121e502, 90 },
    { 0xc101e503, 90 },
    { 0x3171e504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7311e508, 61 },
    { 0xd311e509, 34 },
    { 0xc1a1e509, 61 },
    { 0x9e51e50a, 90 },
    { 0x3811e50c, 69 },
    { 0x5911e50c, 55 },
    { 0xd661e50e, 55 },
    { 0, 0 },
    { 0x1561e510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd1e515, 33 },
    { 0, 0 },
    { 0x62b1e517, 95 },
    { 0x81b1e517, 91 },
    { 0x45b1e518, 81 },
    { 0, 0 },
    { 0x6871e51b, 93 },
    { 0xe3f1e51c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef1e51f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b1e522, 23 },
    { 0x7c81e523, 90 },
    { 0x1a1e524, 43 },
    { 0x5841e525, 23 },
    { 0xcc41e525, 99 },
    { 0x7811e523, 23 },
    { 0, 0 },
    { 0x6311e529, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf621e52e, 46 },
    { 0x34c1e52f, 85 },
    { 0xad51e52f, 93 },
    { 0, 0 },
    { 0x4981e532, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef1e535, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc11e538, 75 },
    { 0, 0 },
    { 0x9b71e53a, 95 },
    { 0xbc21e53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b1e541, 76 },
    { 0, 0 },
    { 0x7cf1e543, 95 },
    { 0, 0 },
    { 0x251e545, 61 },
    { 0, 0 },
    { 0x9ca1e547, 53 },
    { 0x4221e548, 23 },
    { 0x5091e549, 78 },
    { 0x8e11e54a, 61 },
    { 0, 0 },
    { 0x4371e54c, 95 },
    { 0x9371e54d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf001e551, 84 },
    { 0x4971e552, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d71e555, 33 },
    { 0x9861e556, 90 },
    { 0, 0 },
    { 0x2621e558, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e1e55c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a1e55f, 61 },
    { 0xc371e55f, 93 },
    { 0x49e1e561, 98 },
    { 0x9a41e561, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b1e569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa491e572, 66 },
    { 0x8f11e573, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc091e576, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e1e579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581e57e, 98 },
    { 0, 0 },
    { 0xc1b1e580, 61 },
    { 0, 0 },
    { 0x8181e582, 43 },
    { 0x971e582, 78 },
    { 0, 0 },
    { 0x4141e585, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc71e58f, 61 },
    { 0xb271e590, 98 },
    { 0xd781e591, 55 },
    { 0xd061e590, 33 },
    { 0xa4c1e58f, 98 },
    { 0, 0 },
    { 0x82f1e595, 99 },
    { 0, 0 },
    { 0x3bf1e597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff1e59b, 55 },
    { 0xfe31e59b, 90 },
    { 0, 0 },
    { 0x8921e59e, 95 },
    { 0x26d1e59f, 95 },
    { 0x9811e59f, 61 },
    { 0x26b1e59f, 98 },
    { 0, 0 },
    { 0x3d1e5a3, 71 },
    { 0x4d21e5a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1e5a9, 68 },
    { 0, 0 },
    { 0x7e41e5ab, 90 },
    { 0x4fd1e5ac, 55 },
    { 0x5d21e5ac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed41e5b6, 34 },
    { 0, 0 },
    { 0xe811e5b8, 33 },
    { 0, 0 },
    { 0x9eb1e5ba, 34 },
    { 0x8a31e5bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf31e5be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe1e5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c1e5c6, 100 },
    { 0x61f1e5c7, 84 },
    { 0x6b1e5c6, 93 },
    { 0x87c1e5c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d1e5d0, 33 },
    { 0xfa01e5d0, 83 },
    { 0x2f71e5d2, 76 },
    { 0, 0 },
    { 0x69e1e5d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc1e5da, 23 },
    { 0, 0 },
    { 0x9e11e5dc, 95 },
    { 0x6ef1e5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f1e5e2, 51 },
    { 0, 0 },
    { 0x8a41e5e4, 33 },
    { 0, 0 },
    { 0x4e71e5e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3811e5ee, 98 },
    { 0x9fc1e5ef, 98 },
    { 0, 0 },
    { 0x4b31e5f1, 61 },
    { 0x4961e5f2, 64 },
    { 0x5611e5f2, 95 },
    { 0xe9a1e5f4, 90 },
    { 0x2001e5f3, 90 },
    { 0xeba1e5f4, 55 },
    { 0, 0 },
    { 0x2231e5f8, 96 },
    { 0, 0 },
    { 0xc981e5fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x521e5fd, 95 },
    { 0x6b1e5fe, 90 },
    { 0x2f81e5ff, 90 },
    { 0x3c01e5ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd1e615, 33 },
    { 0x7101e616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61e619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db1e61c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a81e623, 61 },
    { 0x31c1e624, 77 },
    { 0x26e1e625, 90 },
    { 0x2341e626, 61 },
    { 0xa611e626, 90 },
    { 0xf5a1e624, 95 },
    { 0xc3b1e629, 98 },
    { 0xddf1e629, 98 },
    { 0x1861e629, 52 },
    { 0xa781e62c, 76 },
    { 0xd771e628, 33 },
    { 0x3561e62e, 90 },
    { 0x1281e62a, 90 },
    { 0, 0 },
    { 0x6b21e631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e1e636, 33 },
    { 0x81e637, 95 },
    { 0x2581e638, 53 },
    { 0x9c61e637, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a51e63c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x291e63f, 34 },
    { 0xf3c1e640, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1731e643, 46 },
    { 0x6761e643, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace1e649, 76 },
    { 0, 0 },
    { 0xfde1e64b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd471e64f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa1e652, 90 },
    { 0, 0 },
    { 0x131e654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef01e660, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9911e667, 90 },
    { 0x89c1e668, 75 },
    { 0x53e1e668, 67 },
    { 0xf681e66a, 90 },
    { 0xfc21e66b, 61 },
    { 0, 0 },
    { 0xa2a1e66d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa131e671, 98 },
    { 0, 0 },
    { 0x3c51e673, 86 },
    { 0, 0 },
    { 0xf6a1e675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a1e678, 40 },
    { 0, 0 },
    { 0x3ab1e67a, 61 },
    { 0xdf21e67b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2041e680, 23 },
    { 0, 0 },
    { 0xff81e682, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd41e688, 61 },
    { 0x7f51e689, 23 },
    { 0x34e1e689, 73 },
    { 0x64a1e688, 33 },
    { 0x7c21e689, 23 },
    { 0, 0 },
    { 0x3851e68e, 23 },
    { 0x8761e68f, 98 },
    { 0, 0 },
    { 0xf561e691, 76 },
    { 0, 0 },
    { 0x7b11e693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb831e69b, 53 },
    { 0x2f91e69c, 81 },
    { 0, 0 },
    { 0xd1f1e69e, 90 },
    { 0xa051e69f, 61 },
    { 0x83b1e6a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e1e6a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9051e6a9, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xc421e6ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e21e6af, 52 },
    { 0xa911e6b0, 55 },
    { 0, 0 },
    { 0xd221e6b2, 90 },
    { 0x2551e6b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf31e6b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2681e6b9, 23 },
    { 0x1d01e6ba, 66 },
    { 0x68e1e6bb, 23 },
    { 0xfaf1e6bc, 61 },
    { 0xab81e6ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61e6c1, 90 },
    { 0x57a1e6c1, 97 },
    { 0xe701e6c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81e6ce, 90 },
    { 0xc441e6cf, 90 },
    { 0x7811e6cf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1e6d3, 61 },
    { 0xc641e6d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9521e6d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb71e6db, 98 },
    { 0x35b1e6dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe671e6e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe351e6e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311e6ea, 98 },
    { 0, 0 },
    { 0x1441e6ec, 61 },
    { 0xaef1e6ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d1e6f0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x8521e6f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991e6fa, 61 },
    { 0xf8b1e6fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa941e6ff, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad81e705, 95 },
    { 0x2a51e706, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e1e70a, 90 },
    { 0x36a1e70a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91e712, 33 },
    { 0x4bb1e713, 93 },
    { 0xe8c1e712, 69 },
    { 0xdf11e715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e1e71c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd561e71f, 93 },
    { 0xe291e720, 100 },
    { 0, 0 },
    { 0x96c1e722, 76 },
    { 0, 0 },
    { 0x3291e724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf1e72d, 23 },
    { 0x2301e72e, 46 },
    { 0xdf91e72d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a1e738, 33 },
    { 0xf8f1e739, 23 },
    { 0x1471e739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b41e73d, 93 },
    { 0, 0 },
    { 0xa991e73f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e71e743, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5401e748, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d1e74c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4251e74f, 33 },
    { 0x5b41e750, 98 },
    { 0, 0 },
    { 0x78e1e752, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc51e756, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7771e759, 85 },
    { 0x27e1e75a, 88 },
    { 0x98c1e759, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671e75e, 55 },
    { 0xc861e75e, 33 },
    { 0xfca1e760, 43 },
    { 0x42d1e75e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd1e769, 91 },
    { 0xecc1e769, 55 },
    { 0xe1c1e76a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x271e76e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc841e774, 77 },
    { 0x6e61e775, 92 },
    { 0x7d91e774, 61 },
    { 0x1001e775, 55 },
    { 0xfa41e778, 66 },
    { 0xa5f1e778, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b1e77d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e61e780, 90 },
    { 0x7e41e781, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e784, 66 },
    { 0x5f91e785, 43 },
    { 0x89a1e785, 46 },
    { 0x6171e787, 33 },
    { 0, 0 },
    { 0x9f21e789, 55 },
    { 0x781e78a, 93 },
    { 0x77d1e78b, 90 },
    { 0xe2b1e78c, 61 },
    { 0x731e78d, 23 },
    { 0x5eb1e78c, 90 },
    { 0x1261e78f, 87 },
    { 0xf761e78d, 38 },
    { 0x83b1e791, 55 },
    { 0x1531e78d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc911e795, 34 },
    { 0x7a41e795, 88 },
    { 0, 0 },
    { 0xece1e798, 84 },
    { 0, 0 },
    { 0xc5e1e79a, 33 },
    { 0x96b1e79b, 61 },
    { 0, 0 },
    { 0x6901e79d, 33 },
    { 0, 0 },
    { 0x7a81e79f, 95 },
    { 0, 0 },
    { 0x4491e7a1, 61 },
    { 0x5bb1e7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b81e7a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5771e7a9, 98 },
    { 0xcca1e7aa, 80 },
    { 0xaee1e7aa, 98 },
    { 0xf61e7ac, 33 },
    { 0xd511e7ac, 66 },
    { 0xe9d1e7ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf321e7b5, 67 },
    { 0xcb81e7b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d1e7bb, 47 },
    { 0x2231e7bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5621e7bf, 61 },
    { 0, 0 },
    { 0x2331e7c1, 95 },
    { 0x3ab1e7c2, 23 },
    { 0x8ae1e7c3, 98 },
    { 0x8171e7c3, 33 },
    { 0xf9c1e7c4, 90 },
    { 0xfbc1e7c3, 33 },
    { 0x8721e7c7, 23 },
    { 0xb521e7c8, 100 },
    { 0x72e1e7c1, 95 },
    { 0xcc01e7c7, 95 },
    { 0x60a1e7c5, 100 },
    { 0xa331e7cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb151e7cf, 85 },
    { 0xab61e7d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f1e7d9, 34 },
    { 0xd91e7d9, 76 },
    { 0, 0 },
    { 0xcc1e7dc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31e7e1, 61 },
    { 0, 0 },
    { 0xc911e7e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2851e7e6, 90 },
    { 0, 0 },
    { 0x5bd1e7e8, 90 },
    { 0x9f81e7e8, 98 },
    { 0x8121e7ea, 87 },
    { 0x71d1e7eb, 33 },
    { 0x7a51e7e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b01e7ef, 90 },
    { 0x81e7f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81e7f6, 98 },
    { 0x8421e7f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9991e7fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3961e7fe, 90 },
    { 0x9c41e7ff, 95 },
    { 0, 0 },
    { 0xf811e801, 87 },
    { 0x74c1e802, 34 },
    { 0, 0 },
    { 0xbba1e804, 93 },
    { 0, 0 },
    { 0xd051e806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a1e80c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5941e810, 98 },
    { 0, 0 },
    { 0x2791e812, 90 },
    { 0x8251e813, 61 },
    { 0x1531e814, 34 },
    { 0x9161e815, 55 },
    { 0, 0 },
    { 0x5121e817, 78 },
    { 0xa821e818, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e1e81b, 81 },
    { 0xf381e81c, 76 },
    { 0x1d81e81d, 61 },
    { 0xabb1e81e, 34 },
    { 0xa361e81f, 95 },
    { 0xd011e81d, 77 },
    { 0, 0 },
    { 0x3091e822, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe441e829, 61 },
    { 0x1151e82a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2831e82f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a1e833, 75 },
    { 0x3661e834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa091e839, 34 },
    { 0xf5e1e83a, 90 },
    { 0, 0 },
    { 0x5931e83c, 87 },
    { 0x5151e83d, 55 },
    { 0xa161e83d, 78 },
    { 0x4d1e83c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa581e847, 33 },
    { 0x1a61e848, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4411e84b, 90 },
    { 0x3921e84c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb71e84f, 90 },
    { 0x5851e850, 46 },
    { 0x1791e850, 95 },
    { 0x1231e850, 76 },
    { 0, 0 },
    { 0x9be1e854, 34 },
    { 0, 0 },
    { 0xddb1e856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e1e85a, 33 },
    { 0x5f11e85a, 78 },
    { 0xe731e85c, 66 },
    { 0, 0 },
    { 0x9531e85e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc1e863, 95 },
    { 0, 0 },
    { 0x2fd1e865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d1e86c, 61 },
    { 0x7c61e86d, 95 },
    { 0x9261e86e, 100 },
    { 0x9501e86f, 91 },
    { 0xeb1e86f, 98 },
    { 0, 0 },
    { 0xba71e872, 61 },
    { 0, 0 },
    { 0xb621e874, 95 },
    { 0x74b1e875, 84 },
    { 0xc31e874, 95 },
    { 0, 0 },
    { 0xf801e878, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e71e87b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d1e87f, 90 },
    { 0xf101e880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2491e884, 61 },
    { 0x8401e884, 14 },
    { 0x29c1e886, 33 },
    { 0x92c1e887, 61 },
    { 0, 0 },
    { 0x7681e889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb811e88e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed1e892, 34 },
    { 0xc6f1e893, 85 },
    { 0, 0 },
    { 0x2551e895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a1e899, 34 },
    { 0xad11e89a, 67 },
    { 0x5771e89a, 33 },
    { 0, 0 },
    { 0x2cc1e89d, 30 },
    { 0xa591e89d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e8a2, 90 },
    { 0, 0 },
    { 0xdad1e8a4, 61 },
    { 0x9321e8a4, 33 },
    { 0xa6f1e8a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d41e8ac, 33 },
    { 0x3111e8ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f1e8b0, 85 },
    { 0x5231e8b0, 43 },
    { 0xad1e8b1, 43 },
    { 0xebd1e8b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb661e8b7, 61 },
    { 0x641e8b8, 23 },
    { 0x8cd1e8b7, 90 },
    { 0xf0b1e8b7, 90 },
    { 0xfcc1e8b8, 76 },
    { 0x2341e8bc, 87 },
    { 0x60f1e8bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e31e8c1, 90 },
    { 0xb661e8c1, 95 },
    { 0x3821e8c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd651e8cd, 92 },
    { 0x3ae1e8ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1e8d1, 61 },
    { 0, 0 },
    { 0xe7c1e8d3, 76 },
    { 0, 0 },
    { 0xb7b1e8d5, 76 },
    { 0x1181e8d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3871e8da, 90 },
    { 0x6481e8da, 95 },
    { 0xf621e8dc, 95 },
    { 0x5af1e8da, 55 },
    { 0x10b1e8dd, 55 },
    { 0xd261e8dc, 55 },
    { 0x6521e8e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e1e8e4, 61 },
    { 0, 0 },
    { 0x3c21e8e6, 61 },
    { 0x5ab1e8e7, 90 },
    { 0, 0 },
    { 0x7a21e8e9, 85 },
    { 0xc671e8e9, 34 },
    { 0, 0 },
    { 0x9d11e8ec, 23 },
    { 0xe741e8ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c1e8f2, 90 },
    { 0x77f1e8f3, 97 },
    { 0xc8b1e8f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5971e8f7, 68 },
    { 0x50c1e8f8, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4651e8fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7391e900, 61 },
    { 0, 0 },
    { 0xbe81e902, 23 },
    { 0x2c31e902, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4131e909, 91 },
    { 0xe7d1e90a, 98 },
    { 0xf3c1e90a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed1e90f, 23 },
    { 0, 0 },
    { 0xb541e911, 66 },
    { 0x36a1e912, 34 },
    { 0x98c1e912, 33 },
    { 0xa001e914, 90 },
    { 0x40c1e915, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c1e91a, 78 },
    { 0x89f1e91a, 99 },
    { 0xd571e91c, 78 },
    { 0x62b1e91d, 98 },
    { 0x6041e91b, 85 },
    { 0, 0 },
    { 0xfa61e920, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5821e923, 17 },
    { 0x3e01e923, 23 },
    { 0x861e923, 46 },
    { 0xefe1e926, 61 },
    { 0x15f1e927, 90 },
    { 0xc191e924, 66 },
    { 0, 0 },
    { 0xf5a1e92a, 90 },
    { 0xb871e92b, 23 },
    { 0, 0 },
    { 0x3441e92d, 90 },
    { 0x5921e92d, 23 },
    { 0xb8c1e92e, 76 },
    { 0x5be1e92f, 33 },
    { 0xff51e931, 85 },
    { 0x5671e931, 84 },
    { 0xa6e1e92e, 90 },
    { 0x5d51e934, 99 },
    { 0x9a11e92d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1e93a, 23 },
    { 0xaed1e93b, 90 },
    { 0x5281e93b, 61 },
    { 0xf421e93d, 23 },
    { 0, 0 },
    { 0x4f11e93f, 43 },
    { 0x5011e93f, 46 },
    { 0x1741e941, 98 },
    { 0, 0 },
    { 0x6a21e943, 61 },
    { 0x3f11e944, 61 },
    { 0x4151e945, 93 },
    { 0xfda1e945, 33 },
    { 0xa6e1e945, 33 },
    { 0x1f21e944, 55 },
    { 0x2b21e943, 90 },
    { 0, 0 },
    { 0xeec1e94b, 61 },
    { 0x44d1e94c, 100 },
    { 0x4d91e94d, 98 },
    { 0x731e94e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec1e951, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b01e956, 23 },
    { 0, 0 },
    { 0xa571e958, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581e95c, 23 },
    { 0x6941e95d, 55 },
    { 0xc341e95e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1e963, 61 },
    { 0xeaa1e964, 90 },
    { 0x9bd1e963, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b1e969, 23 },
    { 0, 0 },
    { 0x311e96b, 98 },
    { 0x1a71e96b, 23 },
    { 0x5401e96b, 90 },
    { 0xbc1e96c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb1e971, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1331e976, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba1e979, 23 },
    { 0xb441e979, 34 },
    { 0x17b1e97b, 95 },
    { 0x5231e97c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161e982, 61 },
    { 0x6d61e982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f31e986, 75 },
    { 0xdb61e986, 33 },
    { 0xad41e988, 43 },
    { 0, 0 },
    { 0x7711e98a, 61 },
    { 0, 0 },
    { 0x1871e98c, 46 },
    { 0xebd1e98d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a1e994, 85 },
    { 0x4c11e994, 43 },
    { 0x3b21e996, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a81e99a, 43 },
    { 0, 0 },
    { 0xb821e99c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea41e99f, 98 },
    { 0, 0 },
    { 0xe631e9a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d1e9a4, 66 },
    { 0x3321e9a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf111e9a8, 98 },
    { 0x1751e9a9, 43 },
    { 0x7101e9aa, 68 },
    { 0x89e1e9ab, 23 },
    { 0x5111e9aa, 17 },
    { 0x7621e9ac, 65 },
    { 0, 0 },
    { 0xcce1e9af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3591e9b4, 85 },
    { 0x1ab1e9b5, 98 },
    { 0x7931e9b6, 23 },
    { 0, 0 },
    { 0x56e1e9b8, 76 },
    { 0xf411e9b9, 61 },
    { 0xdf21e9b8, 33 },
    { 0x1ee1e9bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef1e9be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41e9c6, 23 },
    { 0x68b1e9c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f1e9d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b1e9d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c1e9db, 52 },
    { 0xf0b1e9dc, 81 },
    { 0xc31e9dc, 5 },
    { 0, 0 },
    { 0x17e1e9df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21e9e4, 69 },
    { 0, 0 },
    { 0x51e1e9e6, 33 },
    { 0, 0 },
    { 0x39b1e9e8, 76 },
    { 0, 0 },
    { 0x6921e9ea, 98 },
    { 0x5251e9eb, 98 },
    { 0x3271e9ec, 88 },
    { 0x5001e9ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe1e9f1, 66 },
    { 0xf661e9f2, 43 },
    { 0x1e01e9f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f71e9f6, 61 },
    { 0xad01e9f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5641e9fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd31ea00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff41ea03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e1ea0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1271ea11, 78 },
    { 0xfff1ea12, 98 },
    { 0x8151ea12, 33 },
    { 0, 0 },
    { 0x8db1ea15, 40 },
    { 0, 0 },
    { 0x1d51ea17, 90 },
    { 0x8381ea18, 98 },
    { 0, 0 },
    { 0x6f81ea1a, 93 },
    { 0x9801ea1b, 76 },
    { 0x9df1ea1c, 46 },
    { 0x72c1ea1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8621ea22, 85 },
    { 0x7791ea23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d1ea2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1ea31, 98 },
    { 0x7db1ea31, 17 },
    { 0xf921ea32, 99 },
    { 0x80e1ea31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3981ea3a, 46 },
    { 0x94e1ea3a, 69 },
    { 0x3521ea3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1131ea40, 98 },
    { 0x7f01ea41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab1ea47, 46 },
    { 0, 0 },
    { 0xd4e1ea49, 90 },
    { 0x3d91ea49, 38 },
    { 0, 0 },
    { 0x1da1ea4c, 90 },
    { 0, 0 },
    { 0x2d01ea4e, 90 },
    { 0x8511ea4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b1ea5c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3341ea60, 98 },
    { 0xbb11ea60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5511ea65, 98 },
    { 0x8eb1ea65, 95 },
    { 0, 0 },
    { 0xeed1ea68, 23 },
    { 0x2c1ea68, 23 },
    { 0x3cf1ea69, 98 },
    { 0x3b01ea6b, 91 },
    { 0x8e21ea69, 90 },
    { 0xdd41ea6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f21ea72, 61 },
    { 0x8d31ea73, 23 },
    { 0, 0 },
    { 0x1d81ea75, 95 },
    { 0x18b1ea75, 99 },
    { 0x81b1ea75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea51ea7a, 90 },
    { 0xcb01ea7a, 88 },
    { 0xa261ea7c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd981ea81, 61 },
    { 0, 0 },
    { 0xeee1ea83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2391ea86, 84 },
    { 0xfc21ea87, 83 },
    { 0, 0 },
    { 0x4d91ea89, 81 },
    { 0x301ea8a, 61 },
    { 0xe11ea8a, 98 },
    { 0, 0 },
    { 0xfca1ea8d, 99 },
    { 0, 0 },
    { 0xc761ea8f, 98 },
    { 0x4d61ea90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb271ea96, 61 },
    { 0xda31ea97, 30 },
    { 0, 0 },
    { 0x8c61ea99, 33 },
    { 0, 0 },
    { 0xe671ea9b, 90 },
    { 0, 0 },
    { 0x5261ea9d, 88 },
    { 0, 0 },
    { 0x92b1ea9f, 52 },
    { 0x4711ea9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eaa5, 23 },
    { 0x1341eaa6, 43 },
    { 0x4e21eaa7, 33 },
    { 0x3351eaa8, 90 },
    { 0xd3d1eaa9, 90 },
    { 0xb531eaa8, 90 },
    { 0x1811eaaa, 98 },
    { 0xba91eaac, 91 },
    { 0x37d1eaac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb681eab2, 90 },
    { 0xe501eab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf61eab8, 90 },
    { 0, 0 },
    { 0x2311eaba, 69 },
    { 0xa4f1eaba, 61 },
    { 0x7c21eabc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2081eac2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a61eac9, 93 },
    { 0x2381eaca, 98 },
    { 0x151eacb, 84 },
    { 0, 0 },
    { 0xfd31eacd, 23 },
    { 0x1441eace, 34 },
    { 0, 0 },
    { 0x2551ead0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad1ead3, 99 },
    { 0, 0 },
    { 0x7751ead5, 33 },
    { 0xe21ead6, 43 },
    { 0xd8e1ead7, 69 },
    { 0, 0 },
    { 0x5da1ead9, 33 },
    { 0xe5f1eada, 69 },
    { 0xbbf1eadb, 95 },
    { 0, 0 },
    { 0x9351eadd, 90 },
    { 0x20b1eade, 33 },
    { 0, 0 },
    { 0x1f1eae0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c1eae4, 76 },
    { 0xf301eae4, 55 },
    { 0xbbb1eae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef1eae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d61eaec, 52 },
    { 0x5ed1eaed, 87 },
    { 0x1011eaee, 33 },
    { 0x3181eaee, 33 },
    { 0x82c1eaf0, 90 },
    { 0x7901eaf0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61eaf4, 33 },
    { 0x1c1eaf5, 95 },
    { 0xf2c1eaf6, 90 },
    { 0x9841eaf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba81eafa, 95 },
    { 0xfa21eafb, 61 },
    { 0x6e1eafc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd1eb00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e21eb03, 33 },
    { 0, 0 },
    { 0xdbf1eb05, 46 },
    { 0x27d1eb05, 93 },
    { 0xb1d1eb07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe411eb0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7411eb11, 81 },
    { 0, 0 },
    { 0x2121eb13, 23 },
    { 0x5fc1eb14, 61 },
    { 0xdde1eb15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc81eb1c, 84 },
    { 0xcd81eb1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc41eb21, 90 },
    { 0xbeb1eb22, 43 },
    { 0xe831eb21, 95 },
    { 0, 0 },
    { 0xe151eb25, 95 },
    { 0x68f1eb26, 65 },
    { 0, 0 },
    { 0x6201eb28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7481eb2f, 61 },
    { 0, 0 },
    { 0xeb61eb31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb51eb38, 46 },
    { 0x5851eb38, 23 },
    { 0x2611eb39, 43 },
    { 0, 0 },
    { 0xb1e1eb3c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4421eb3f, 100 },
    { 0x7da1eb3f, 33 },
    { 0x63f1eb3f, 93 },
    { 0x5a41eb3f, 90 },
    { 0x8541eb42, 33 },
    { 0xcea1eb44, 98 },
    { 0, 0 },
    { 0x8941eb46, 61 },
    { 0xab61eb47, 91 },
    { 0xdd91eb47, 90 },
    { 0xa6b1eb49, 98 },
    { 0xe401eb4a, 61 },
    { 0x2e41eb4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf181eb54, 23 },
    { 0, 0 },
    { 0xe5f1eb56, 66 },
    { 0x2bc1eb56, 33 },
    { 0, 0 },
    { 0xbaa1eb59, 33 },
    { 0x4a71eb59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c1eb63, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f51eb66, 43 },
    { 0xdf61eb67, 55 },
    { 0x3631eb68, 100 },
    { 0xc771eb66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b1eb6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e1eb71, 85 },
    { 0xae31eb71, 95 },
    { 0, 0 },
    { 0xf841eb74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e1eb7c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1831eb7f, 68 },
    { 0xeb81eb80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad31eb84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb701eb89, 95 },
    { 0, 0 },
    { 0xcb21eb8b, 95 },
    { 0, 0 },
    { 0x5f71eb8d, 23 },
    { 0, 0 },
    { 0xd381eb8f, 95 },
    { 0x3281eb90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2211eb93, 84 },
    { 0xe761eb94, 100 },
    { 0x18d1eb94, 40 },
    { 0xfec1eb96, 96 },
    { 0xa41eb97, 95 },
    { 0xfc91eb98, 90 },
    { 0x6bb1eb99, 33 },
    { 0x6481eb99, 99 },
    { 0x4aa1eb98, 80 },
    { 0xf991eb9c, 46 },
    { 0xc951eb9c, 61 },
    { 0x29c1eb98, 90 },
    { 0xe061eb9f, 17 },
    { 0x9351eba0, 46 },
    { 0xd3f1eb98, 99 },
    { 0xa711eba2, 95 },
    { 0xe371eba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff1ebac, 61 },
    { 0, 0 },
    { 0x10a1ebae, 95 },
    { 0x2a01ebae, 61 },
    { 0, 0 },
    { 0x2da1ebb1, 93 },
    { 0x5491ebb2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x871ebb5, 98 },
    { 0x9b31ebb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc91ebba, 98 },
    { 0, 0 },
    { 0xe81ebbc, 33 },
    { 0x3fb1ebbd, 90 },
    { 0x3851ebbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c61ebc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f81ebc7, 33 },
    { 0xaa81ebc8, 98 },
    { 0x8591ebc9, 88 },
    { 0x9e11ebc8, 46 },
    { 0, 0 },
    { 0xa3d1ebcc, 38 },
    { 0, 0 },
    { 0xd7e1ebce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f1ebd3, 95 },
    { 0xbb91ebd4, 98 },
    { 0xd7c1ebd5, 95 },
    { 0, 0 },
    { 0x2621ebd7, 61 },
    { 0xb5f1ebd8, 33 },
    { 0xef01ebd8, 95 },
    { 0x1381ebda, 88 },
    { 0xf3b1ebd9, 95 },
    { 0x4da1ebdc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d1ebe3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d1ebe6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371ebea, 61 },
    { 0xfbc1ebeb, 34 },
    { 0xbe01ebea, 75 },
    { 0, 0 },
    { 0x71d1ebee, 90 },
    { 0, 0 },
    { 0xd061ebf0, 93 },
    { 0, 0 },
    { 0xe51ebf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1ebf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6731ebfb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2731ebfe, 76 },
    { 0x3371ebff, 43 },
    { 0, 0 },
    { 0xc291ec01, 34 },
    { 0, 0 },
    { 0x4931ec03, 98 },
    { 0x8d71ec04, 95 },
    { 0x1711ec05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef1ec11, 91 },
    { 0xc181ec12, 52 },
    { 0x21f1ec12, 61 },
    { 0, 0 },
    { 0x9471ec15, 43 },
    { 0x2d21ec16, 34 },
    { 0x2701ec16, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6541ec1c, 33 },
    { 0x97d1ec1d, 38 },
    { 0, 0 },
    { 0x5c71ec1f, 85 },
    { 0, 0 },
    { 0xe11ec21, 76 },
    { 0x9b41ec22, 82 },
    { 0x8e81ec22, 30 },
    { 0xdd1ec23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2081ec2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d1ec30, 68 },
    { 0x331ec31, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a31ec36, 78 },
    { 0xedb1ec37, 33 },
    { 0x1b1ec38, 33 },
    { 0xa71ec39, 43 },
    { 0xa5f1ec3a, 55 },
    { 0, 0 },
    { 0x84a1ec3c, 65 },
    { 0xe4b1ec3d, 90 },
    { 0x16a1ec3c, 61 },
    { 0xd9f1ec3f, 76 },
    { 0xe7d1ec3f, 84 },
    { 0x38a1ec41, 46 },
    { 0x53b1ec41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2851ec48, 100 },
    { 0x2701ec49, 98 },
    { 0, 0 },
    { 0xd4f1ec4b, 76 },
    { 0x62a1ec4c, 76 },
    { 0, 0 },
    { 0x16e1ec4e, 55 },
    { 0xf771ec4e, 33 },
    { 0x2041ec50, 23 },
    { 0xd9c1ec4e, 23 },
    { 0xec31ec51, 61 },
    { 0x4671ec53, 100 },
    { 0xe371ec54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6551ec59, 93 },
    { 0x5491ec5a, 85 },
    { 0xd81ec5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee1ec5e, 67 },
    { 0, 0 },
    { 0x66a1ec60, 90 },
    { 0x2e1ec61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b1ec64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc471ec67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da1ec6a, 23 },
    { 0x2841ec6a, 33 },
    { 0xc611ec6a, 17 },
    { 0, 0 },
    { 0xda31ec6e, 61 },
    { 0, 0 },
    { 0xba21ec70, 46 },
    { 0x6f41ec71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4431ec79, 34 },
    { 0, 0 },
    { 0xba21ec7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca1ec82, 23 },
    { 0x7f21ec83, 33 },
    { 0x6f21ec84, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d21ec88, 94 },
    { 0, 0 },
    { 0x1a21ec8a, 68 },
    { 0xbf31ec8a, 23 },
    { 0x88c1ec8a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d1ec90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee1ec96, 34 },
    { 0xcd81ec96, 100 },
    { 0xb591ec96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe61ec9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe121eca2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e01eca5, 33 },
    { 0, 0 },
    { 0x5801eca7, 33 },
    { 0x8fe1eca7, 55 },
    { 0x8f41eca9, 76 },
    { 0x2bc1ecaa, 33 },
    { 0xff91ecab, 55 },
    { 0x3ac1ecac, 95 },
    { 0xeca1eca7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3161ecb2, 90 },
    { 0x96a1ecb3, 34 },
    { 0, 0 },
    { 0x93e1ecb5, 23 },
    { 0x5e41ecb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e1ecba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb1ecc0, 87 },
    { 0, 0 },
    { 0xc041ecc2, 88 },
    { 0xa331ecc3, 76 },
    { 0x8591ecc3, 99 },
    { 0xf411ecc5, 85 },
    { 0, 0 },
    { 0x9721ecc7, 93 },
    { 0x6811ecc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d1eccc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b1ecd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d1ecd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde1ecd9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4201ecde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8781ece1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe161ece6, 66 },
    { 0x451ece7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9841ecf9, 33 },
    { 0, 0 },
    { 0xb51ecfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e1ecff, 77 },
    { 0xf0b1ed00, 90 },
    { 0x22b1ed00, 90 },
    { 0, 0 },
    { 0xf001ed03, 90 },
    { 0x2db1ed04, 76 },
    { 0x3891ed03, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f11ed08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf331ed0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd481ed10, 76 },
    { 0, 0 },
    { 0xc71ed12, 78 },
    { 0, 0 },
    { 0xf741ed14, 95 },
    { 0x2d91ed15, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f1ed1d, 87 },
    { 0x20d1ed1e, 95 },
    { 0xd6c1ed1f, 93 },
    { 0x3ba1ed1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be1ed29, 90 },
    { 0xbbe1ed2a, 90 },
    { 0xee01ed2b, 90 },
    { 0xcae1ed2a, 66 },
    { 0, 0 },
    { 0x4e61ed2e, 85 },
    { 0x2b41ed2f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe281ed33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf591ed36, 95 },
    { 0x15f1ed37, 34 },
    { 0xf481ed38, 76 },
    { 0xabb1ed39, 33 },
    { 0x4811ed39, 67 },
    { 0x321ed3b, 61 },
    { 0xdea1ed3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf01ed40, 98 },
    { 0x8671ed40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3961ed44, 99 },
    { 0xf51ed45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda01ed4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9541ed53, 23 },
    { 0x2041ed54, 33 },
    { 0x3801ed55, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb1ed59, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f1ed5c, 81 },
    { 0x5ab1ed5d, 90 },
    { 0x6ca1ed5e, 89 },
    { 0x74f1ed5c, 90 },
    { 0, 0 },
    { 0xa821ed61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc1ed66, 90 },
    { 0xf0f1ed67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d1ed6a, 87 },
    { 0x7b31ed6b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d1ed6f, 99 },
    { 0xcf71ed70, 75 },
    { 0xc6e1ed6f, 33 },
    { 0x1da1ed72, 33 },
    { 0x9801ed70, 34 },
    { 0x1821ed74, 91 },
    { 0x77a1ed74, 23 },
    { 0x70f1ed6f, 66 },
    { 0x4d1ed73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa531ed7a, 43 },
    { 0x1f91ed7a, 93 },
    { 0x5e21ed7c, 43 },
    { 0, 0 },
    { 0x59f1ed7e, 52 },
    { 0xe391ed7e, 95 },
    { 0x7381ed80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1ed84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b31ed8c, 23 },
    { 0, 0 },
    { 0x79f1ed8e, 23 },
    { 0, 0 },
    { 0xcc41ed90, 61 },
    { 0x17c1ed91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ed94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b1ed98, 61 },
    { 0x1201ed99, 43 },
    { 0xb861ed9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8011eda0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc531eda3, 33 },
    { 0, 0 },
    { 0x3091eda5, 87 },
    { 0, 0 },
    { 0x2ba1eda7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7701edad, 55 },
    { 0xdcd1edae, 76 },
    { 0, 0 },
    { 0xf2d1edb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb601edb3, 36 },
    { 0, 0 },
    { 0x78f1edb5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb1edb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc11edbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f1edc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131edc7, 98 },
    { 0xc9f1edc8, 84 },
    { 0x7ee1edc7, 33 },
    { 0x2fb1edca, 90 },
    { 0, 0 },
    { 0x7361edcc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad11edd1, 61 },
    { 0xaa81edd1, 87 },
    { 0x48d1edd3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1ede0, 23 },
    { 0, 0 },
    { 0x8b41ede2, 23 },
    { 0, 0 },
    { 0xdd1ede4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xee71ede7, 98 },
    { 0xa81ede8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d1eded, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e1edf1, 90 },
    { 0, 0 },
    { 0xd311edf3, 33 },
    { 0, 0 },
    { 0x291edf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff1edf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc1ee02, 90 },
    { 0x7f11ee03, 100 },
    { 0x4fc1ee04, 98 },
    { 0xce81ee05, 43 },
    { 0xee61ee05, 61 },
    { 0xbb51ee07, 33 },
    { 0xa241ee08, 76 },
    { 0x54a1ee09, 67 },
    { 0x7b11ee0a, 98 },
    { 0, 0 },
    { 0x82c1ee0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a1ee10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f71ee14, 90 },
    { 0xd511ee14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8731ee18, 99 },
    { 0x5071ee19, 33 },
    { 0x2da1ee1a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b21ee23, 88 },
    { 0xd8d1ee24, 33 },
    { 0xd791ee24, 68 },
    { 0xb281ee26, 88 },
    { 0x6331ee26, 66 },
    { 0x47e1ee23, 61 },
    { 0x3351ee26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b1ee2d, 33 },
    { 0x1061ee2e, 61 },
    { 0xb8f1ee2d, 96 },
    { 0, 0 },
    { 0x40b1ee31, 85 },
    { 0xec1ee31, 67 },
    { 0x4241ee32, 61 },
    { 0x9061ee34, 98 },
    { 0x4351ee35, 43 },
    { 0x6521ee36, 46 },
    { 0xab91ee32, 34 },
    { 0xc781ee33, 43 },
    { 0xa6e1ee31, 23 },
    { 0x3ae1ee3a, 46 },
    { 0x8671ee3a, 76 },
    { 0x131ee3c, 23 },
    { 0x1dc1ee34, 23 },
    { 0x6bb1ee3d, 97 },
    { 0, 0 },
    { 0xcad1ee40, 52 },
    { 0xd851ee41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc591ee45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81ee4a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d81ee54, 61 },
    { 0x4081ee54, 87 },
    { 0x1f61ee56, 78 },
    { 0xd11ee54, 33 },
    { 0xcd71ee55, 93 },
    { 0xa5d1ee55, 61 },
    { 0xef41ee5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d21ee5d, 33 },
    { 0x9641ee5e, 52 },
    { 0, 0 },
    { 0xd451ee60, 95 },
    { 0x9d21ee60, 67 },
    { 0xce71ee62, 61 },
    { 0xbc1ee63, 76 },
    { 0x2a31ee60, 95 },
    { 0x361ee65, 33 },
    { 0x2e81ee62, 98 },
    { 0xcf01ee62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2161ee6e, 84 },
    { 0x9161ee6f, 46 },
    { 0xcaa1ee70, 51 },
    { 0xe3c1ee71, 17 },
    { 0xb171ee72, 66 },
    { 0x5a11ee73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe781ee77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b81ee7d, 100 },
    { 0xdc81ee7e, 33 },
    { 0x8021ee7e, 55 },
    { 0x15d1ee7d, 61 },
    { 0, 0 },
    { 0xe91ee82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f91ee86, 34 },
    { 0x10b1ee86, 52 },
    { 0xa051ee88, 98 },
    { 0xb8e1ee89, 78 },
    { 0x3331ee8a, 76 },
    { 0xf971ee89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3931ee92, 98 },
    { 0xdd81ee92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d1ee97, 33 },
    { 0x251ee97, 75 },
    { 0x3781ee99, 90 },
    { 0xc1ee9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf51ee9e, 34 },
    { 0, 0 },
    { 0x9941eea0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e1eea6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b31eead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd191eeb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c1eeb5, 43 },
    { 0x9211eeb6, 85 },
    { 0x5ba1eeb7, 93 },
    { 0x9451eeb8, 77 },
    { 0x8441eeb7, 55 },
    { 0x3511eeb8, 33 },
    { 0xe351eebb, 98 },
    { 0xbdd1eeb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde1eebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8361eec2, 78 },
    { 0x2cf1eec3, 78 },
    { 0x8761eec3, 95 },
    { 0x7cd1eec3, 69 },
    { 0xac91eec6, 98 },
    { 0xe5b1eec6, 93 },
    { 0x5b11eec2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9331eecc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a1eed1, 33 },
    { 0, 0 },
    { 0x8091eed3, 23 },
    { 0, 0 },
    { 0x1811eed5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551eedd, 55 },
    { 0x8231eedd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c51eee4, 95 },
    { 0xec41eee4, 23 },
    { 0x5071eee6, 46 },
    { 0x5061eee6, 55 },
    { 0xf6d1eee8, 66 },
    { 0x40c1eee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6201eeee, 83 },
    { 0x9e1eeef, 33 },
    { 0x19b1eef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd1eef4, 90 },
    { 0x21d1eef4, 84 },
    { 0x6301eef6, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b1eefa, 93 },
    { 0, 0 },
    { 0xda41eefc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe761eeff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b1ef03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2691ef06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad1ef09, 23 },
    { 0x2501ef0a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf361ef0d, 76 },
    { 0x9a91ef0e, 93 },
    { 0x7c01ef0f, 81 },
    { 0xa8d1ef10, 55 },
    { 0x7ce1ef11, 33 },
    { 0xf41ef12, 61 },
    { 0x12c1ef13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561ef18, 87 },
    { 0, 0 },
    { 0xbc71ef1a, 64 },
    { 0xece1ef1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef1ef22, 95 },
    { 0x7a81ef23, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a1ef29, 90 },
    { 0x74b1ef2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a71ef2e, 75 },
    { 0xe9a1ef2f, 90 },
    { 0x7df1ef2e, 23 },
    { 0xee41ef31, 98 },
    { 0x1361ef32, 33 },
    { 0x7f51ef32, 66 },
    { 0x9231ef34, 43 },
    { 0x75e1ef34, 93 },
    { 0, 0 },
    { 0x5ee1ef37, 85 },
    { 0x2be1ef37, 91 },
    { 0x8301ef38, 33 },
    { 0, 0 },
    { 0x5511ef3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f1ef42, 95 },
    { 0x4491ef43, 90 },
    { 0x92b1ef43, 61 },
    { 0x29f1ef43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf1ef4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c31ef4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6771ef52, 95 },
    { 0x47d1ef53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91ef5d, 91 },
    { 0x5da1ef5d, 55 },
    { 0xd521ef5f, 87 },
    { 0, 0 },
    { 0x5961ef61, 33 },
    { 0, 0 },
    { 0x6771ef63, 49 },
    { 0x3cc1ef64, 81 },
    { 0xc2b1ef65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe981ef6a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9301ef6d, 66 },
    { 0x3f41ef6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c1ef71, 61 },
    { 0x7371ef72, 33 },
    { 0xaf31ef72, 67 },
    { 0xe4d1ef74, 33 },
    { 0x1751ef72, 61 },
    { 0x9271ef71, 90 },
    { 0x3ab1ef77, 33 },
    { 0x5d41ef78, 33 },
    { 0x3471ef79, 90 },
    { 0xa171ef78, 33 },
    { 0xe181ef78, 23 },
    { 0x9bc1ef7c, 61 },
    { 0xe211ef79, 65 },
    { 0x19c1ef7e, 23 },
    { 0xfa41ef7f, 91 },
    { 0x9b01ef80, 87 },
    { 0x8db1ef74, 61 },
    { 0x53d1ef82, 58 },
    { 0, 0 },
    { 0xe7e1ef84, 100 },
    { 0xda51ef85, 88 },
    { 0x1e61ef86, 23 },
    { 0xe1ef87, 90 },
    { 0xe041ef88, 23 },
    { 0x2c71ef88, 55 },
    { 0x3331ef88, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b1ef8d, 33 },
    { 0x171ef8e, 90 },
    { 0x8e61ef8f, 76 },
    { 0, 0 },
    { 0x9541ef91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c91ef94, 95 },
    { 0x81e1ef95, 33 },
    { 0x3fc1ef95, 61 },
    { 0x4c21ef97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a61ef9d, 61 },
    { 0xd171ef9e, 52 },
    { 0x8a21ef9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac1efa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb71efa7, 46 },
    { 0x191efa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a31efad, 23 },
    { 0, 0 },
    { 0x8a21efaf, 90 },
    { 0x3dd1efaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a1efb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b1efb8, 61 },
    { 0, 0 },
    { 0x1ed1efba, 23 },
    { 0, 0 },
    { 0xbcd1efbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7731efc0, 33 },
    { 0x3d81efc1, 33 },
    { 0, 0 },
    { 0x13e1efc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b1efcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec21efd0, 33 },
    { 0x4fd1efd0, 87 },
    { 0xea51efd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111efd7, 46 },
    { 0, 0 },
    { 0xc61efd9, 55 },
    { 0x7cc1efda, 81 },
    { 0, 0 },
    { 0xd91efdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5241efe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb1efe6, 33 },
    { 0x4ea1efe7, 43 },
    { 0x7fe1efe6, 43 },
    { 0x90f1efe9, 98 },
    { 0xb201efea, 23 },
    { 0x3dc1efeb, 23 },
    { 0x3cc1efec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891efef, 61 },
    { 0xaf61eff0, 52 },
    { 0xb8c1eff1, 76 },
    { 0xb631eff1, 98 },
    { 0, 0 },
    { 0xd9d1eff4, 77 },
    { 0xb1a1eff5, 98 },
    { 0, 0 },
    { 0x67b1eff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc1effa, 98 },
    { 0x5c11effb, 75 },
    { 0x17e1effc, 61 },
    { 0xe191effb, 90 },
    { 0, 0 },
    { 0x6d61efff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac21f002, 55 },
    { 0x921f003, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5451f007, 85 },
    { 0x6761f008, 33 },
    { 0xb311f009, 88 },
    { 0x6021f00a, 23 },
    { 0xfba1f00b, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f21f00f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f1f012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e1f017, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf1f01a, 90 },
    { 0x7241f01b, 99 },
    { 0x4341f01c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xed51f01f, 34 },
    { 0xc21f01f, 65 },
    { 0, 0 },
    { 0xaf41f022, 98 },
    { 0x5cc1f022, 33 },
    { 0x9f91f024, 78 },
    { 0x4f91f025, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7101f02f, 43 },
    { 0x9cd1f030, 93 },
    { 0, 0 },
    { 0x1071f032, 83 },
    { 0xd2d1f033, 59 },
    { 0x9131f034, 33 },
    { 0x8fa1f034, 23 },
    { 0x7b11f035, 40 },
    { 0x6111f036, 38 },
    { 0x2e31f038, 33 },
    { 0xea81f039, 76 },
    { 0x3eb1f03a, 34 },
    { 0x5d11f03a, 23 },
    { 0xdc91f03b, 33 },
    { 0, 0 },
    { 0xed31f03e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a61f041, 85 },
    { 0x5e31f041, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5671f047, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb1f04a, 66 },
    { 0, 0 },
    { 0x4b51f04c, 68 },
    { 0, 0 },
    { 0xe231f04e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6511f053, 33 },
    { 0, 0 },
    { 0xc591f055, 33 },
    { 0, 0 },
    { 0xe401f057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e1f05b, 99 },
    { 0x6ad1f05b, 84 },
    { 0x1811f05d, 46 },
    { 0, 0 },
    { 0x5431f05f, 61 },
    { 0xa51f060, 95 },
    { 0x7d41f05f, 76 },
    { 0x341f060, 46 },
    { 0, 0 },
    { 0x3631f064, 98 },
    { 0, 0 },
    { 0x8531f066, 95 },
    { 0, 0 },
    { 0x12a1f068, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a51f06b, 95 },
    { 0xd551f06c, 100 },
    { 0x9c61f06c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb251f076, 23 },
    { 0x5941f077, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa1f07c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c1f07f, 61 },
    { 0x5831f080, 98 },
    { 0x13b1f081, 98 },
    { 0xf481f081, 49 },
    { 0xd121f083, 61 },
    { 0xbb1f082, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb441f089, 55 },
    { 0x1b41f08a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1531f08d, 95 },
    { 0, 0 },
    { 0xf151f08f, 76 },
    { 0xf211f090, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef1f094, 33 },
    { 0x7361f094, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901f098, 95 },
    { 0xf981f099, 33 },
    { 0x131f09a, 43 },
    { 0x691f099, 23 },
    { 0x6801f09c, 95 },
    { 0, 0 },
    { 0x4cc1f09e, 55 },
    { 0x1c21f09e, 90 },
    { 0x95b1f0a0, 55 },
    { 0xc9f1f0a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f91f0a6, 61 },
    { 0x5081f0a7, 55 },
    { 0x12c1f0a8, 98 },
    { 0, 0 },
    { 0x4821f0aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6301f0b2, 96 },
    { 0, 0 },
    { 0xd681f0b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db1f0ba, 67 },
    { 0x2401f0ba, 38 },
    { 0x4a71f0bc, 41 },
    { 0xb131f0bd, 33 },
    { 0xff31f0ba, 52 },
    { 0, 0 },
    { 0xb171f0c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa181f0c4, 93 },
    { 0, 0 },
    { 0x36d1f0c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee21f0cb, 61 },
    { 0x8e91f0cc, 85 },
    { 0xb4c1f0cc, 95 },
    { 0x7b71f0ce, 66 },
    { 0x80e1f0ce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8801f0d9, 61 },
    { 0xf7c1f0da, 95 },
    { 0x4be1f0db, 61 },
    { 0xfd1f0da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb941f0e2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c91f0e8, 98 },
    { 0xfd11f0e9, 23 },
    { 0x88e1f0e9, 33 },
    { 0x8b1f0e8, 98 },
    { 0, 0 },
    { 0x11f0ed, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d1f0f0, 90 },
    { 0x1261f0f1, 68 },
    { 0x1271f0f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d91f0f5, 68 },
    { 0, 0 },
    { 0x8731f0f7, 90 },
    { 0, 0 },
    { 0xa91f0f9, 23 },
    { 0x3d71f0fa, 23 },
    { 0x5d61f0fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a1f0ff, 33 },
    { 0xa6b1f100, 46 },
    { 0, 0 },
    { 0xe801f102, 34 },
    { 0xf851f103, 95 },
    { 0xdcf1f104, 43 },
    { 0xabb1f104, 55 },
    { 0x91d1f106, 33 },
    { 0, 0 },
    { 0x2621f108, 61 },
    { 0, 0 },
    { 0x3c31f10a, 90 },
    { 0xf441f10b, 100 },
    { 0x4d11f10c, 34 },
    { 0, 0 },
    { 0x4f81f10e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4511f113, 67 },
    { 0xad81f114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa111f119, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a1f11c, 61 },
    { 0, 0 },
    { 0xca51f11e, 23 },
    { 0xaee1f11e, 95 },
    { 0xfeb1f11f, 43 },
    { 0x1af1f121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc731f129, 61 },
    { 0xbbc1f129, 61 },
    { 0, 0 },
    { 0x29a1f12c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3311f12f, 23 },
    { 0x6891f130, 81 },
    { 0, 0 },
    { 0x4d51f132, 34 },
    { 0x2031f133, 34 },
    { 0xf821f133, 33 },
    { 0x31f1f135, 23 },
    { 0x2c81f133, 95 },
    { 0, 0 },
    { 0xcdf1f138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f1f13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae21f13e, 33 },
    { 0, 0 },
    { 0x3e41f140, 33 },
    { 0, 0 },
    { 0x4a21f142, 55 },
    { 0x7d41f143, 64 },
    { 0x3ad1f144, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae61f148, 33 },
    { 0, 0 },
    { 0xe241f14a, 77 },
    { 0xe281f14b, 87 },
    { 0xd181f14a, 34 },
    { 0xb031f14d, 66 },
    { 0, 0 },
    { 0xa31f14f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee1f156, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a1f159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f31f15d, 55 },
    { 0x7e01f15e, 90 },
    { 0, 0 },
    { 0xf5d1f160, 98 },
    { 0xeec1f161, 87 },
    { 0, 0 },
    { 0x4b41f163, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21f167, 99 },
    { 0x1951f167, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e1f16b, 43 },
    { 0xc8b1f16c, 90 },
    { 0x8721f16d, 56 },
    { 0x28a1f16e, 55 },
    { 0x8a71f16f, 23 },
    { 0x7c21f16b, 91 },
    { 0xba61f171, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31f174, 61 },
    { 0x1551f175, 55 },
    { 0, 0 },
    { 0x2261f177, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1f17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef41f17f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9721f184, 61 },
    { 0xa371f185, 90 },
    { 0, 0 },
    { 0x8761f187, 98 },
    { 0, 0 },
    { 0x18d1f189, 61 },
    { 0x4581f18a, 101 },
    { 0xc781f18b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7771f18f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa771f192, 55 },
    { 0, 0 },
    { 0x8191f194, 33 },
    { 0x8f41f194, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc481f198, 23 },
    { 0x69d1f199, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e01f1a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc031f1a4, 36 },
    { 0xec51f1a5, 90 },
    { 0x61b1f1a5, 87 },
    { 0x6d81f1a6, 98 },
    { 0xd231f1a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e91f1ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5421f1b5, 90 },
    { 0x3f81f1b6, 84 },
    { 0, 0 },
    { 0xaf61f1b8, 95 },
    { 0xdc61f1b8, 95 },
    { 0x1d1f1ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2631f1bd, 75 },
    { 0x26f1f1be, 95 },
    { 0xa5e1f1bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb91f1c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd1f1c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa161f1ca, 34 },
    { 0xccc1f1cb, 61 },
    { 0xc121f1cb, 66 },
    { 0x3fd1f1cc, 55 },
    { 0, 0 },
    { 0x4a11f1cf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21f1d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe461f1d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe311f1db, 96 },
    { 0xb01f1dc, 77 },
    { 0, 0 },
    { 0xdeb1f1de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4091f1e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e1f1e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc1f1ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1f1f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b1f1f8, 33 },
    { 0, 0 },
    { 0x7b91f1fa, 46 },
    { 0, 0 },
    { 0xf251f1fc, 76 },
    { 0, 0 },
    { 0xee31f1fe, 17 },
    { 0, 0 },
    { 0xf0f1f200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1f206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea1f209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c1f20d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc501f210, 55 },
    { 0xab81f211, 91 },
    { 0x3351f212, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f1f215, 61 },
    { 0x12f1f216, 33 },
    { 0xe5f1f216, 90 },
    { 0x3e91f218, 90 },
    { 0xcc51f219, 98 },
    { 0, 0 },
    { 0x2f91f21b, 33 },
    { 0xa201f21b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe641f220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8691f223, 93 },
    { 0x6b61f224, 43 },
    { 0x31a1f224, 98 },
    { 0x2fe1f226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b1f229, 90 },
    { 0x42f1f22a, 90 },
    { 0x6af1f22b, 33 },
    { 0x30c1f22b, 55 },
    { 0x29a1f22d, 84 },
    { 0xdbf1f22e, 96 },
    { 0xde01f22f, 33 },
    { 0x8e71f230, 90 },
    { 0xa701f22f, 90 },
    { 0xd9d1f231, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e1f23f, 36 },
    { 0x7c11f23f, 61 },
    { 0xdeb1f240, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f1f244, 62 },
    { 0x761f245, 33 },
    { 0x6be1f246, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd1f24c, 43 },
    { 0, 0 },
    { 0xb901f24e, 98 },
    { 0, 0 },
    { 0x4c1f250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21f257, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e1f25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5231f25e, 34 },
    { 0x81e1f25e, 90 },
    { 0x5751f260, 46 },
    { 0x8741f25e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde11f264, 23 },
    { 0x6711f265, 95 },
    { 0x8581f266, 90 },
    { 0xab51f265, 98 },
    { 0x6a1f267, 33 },
    { 0, 0 },
    { 0x8e41f26a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d1f26d, 61 },
    { 0x23a1f26d, 33 },
    { 0xc911f26e, 23 },
    { 0, 0 },
    { 0xa141f271, 61 },
    { 0x3251f272, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb621f275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c1f27b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7561f27f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871f283, 61 },
    { 0x7b91f283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291f288, 93 },
    { 0x7dd1f289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671f28e, 33 },
    { 0, 0 },
    { 0xf951f290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5611f295, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f1f29b, 90 },
    { 0xf51f29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf771f2a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd21f2a7, 95 },
    { 0x98b1f2a7, 85 },
    { 0x4cd1f2a8, 36 },
    { 0x5de1f2a7, 67 },
    { 0x45a1f2ab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf61f2b1, 76 },
    { 0x51b1f2b2, 96 },
    { 0xd5f1f2b3, 91 },
    { 0x5fd1f2b3, 95 },
    { 0x3191f2b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d51f2b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4621f2be, 76 },
    { 0x93a1f2bf, 90 },
    { 0x1841f2c0, 61 },
    { 0x1861f2c1, 76 },
    { 0xc601f2c2, 90 },
    { 0, 0 },
    { 0xa5d1f2c4, 61 },
    { 0, 0 },
    { 0x1da1f2c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f1f2cf, 90 },
    { 0x63f1f2d0, 43 },
    { 0, 0 },
    { 0x2ce1f2d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5791f2d9, 90 },
    { 0, 0 },
    { 0x9321f2db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc1f2e4, 88 },
    { 0x8901f2e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf461f2ed, 91 },
    { 0x7061f2ee, 17 },
    { 0, 0 },
    { 0x4401f2f0, 52 },
    { 0, 0 },
    { 0xf901f2f2, 100 },
    { 0, 0 },
    { 0xed41f2f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9671f2f8, 33 },
    { 0, 0 },
    { 0x9b41f2fa, 55 },
    { 0xd7f1f2fb, 91 },
    { 0xf851f2fc, 95 },
    { 0x6511f2fd, 34 },
    { 0x61a1f2fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821f302, 33 },
    { 0x4fd1f302, 55 },
    { 0x8021f303, 46 },
    { 0, 0 },
    { 0x35e1f306, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf91f309, 76 },
    { 0x5931f309, 76 },
    { 0x7f71f30b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551f310, 61 },
    { 0, 0 },
    { 0xba31f312, 33 },
    { 0x48e1f313, 93 },
    { 0xfdf1f314, 33 },
    { 0x9801f314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2761f319, 90 },
    { 0x6481f319, 91 },
    { 0xe81f31b, 98 },
    { 0xba01f31c, 33 },
    { 0x4f51f31d, 90 },
    { 0x43c1f31c, 33 },
    { 0xa1f1f31f, 77 },
    { 0x6ab1f320, 23 },
    { 0x5d71f319, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b1f326, 93 },
    { 0, 0 },
    { 0x6351f328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31f32b, 33 },
    { 0x57e1f32c, 75 },
    { 0xafe1f32d, 52 },
    { 0x9c61f32c, 33 },
    { 0x931f32c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe51f332, 43 },
    { 0xac61f333, 100 },
    { 0x8f61f334, 33 },
    { 0, 0 },
    { 0xf861f336, 46 },
    { 0x1ee1f336, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7701f33c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11f349, 93 },
    { 0x3da1f34a, 34 },
    { 0xcba1f34b, 95 },
    { 0, 0 },
    { 0x9961f34d, 33 },
    { 0x29b1f34e, 76 },
    { 0xd3f1f34f, 87 },
    { 0xda01f34d, 98 },
    { 0, 0 },
    { 0x1d61f352, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b1f35f, 52 },
    { 0x6d31f360, 93 },
    { 0xa641f361, 33 },
    { 0xbc81f362, 90 },
    { 0x8211f363, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee31f367, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d61f36a, 33 },
    { 0xb161f36b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1f370, 93 },
    { 0xb91f371, 46 },
    { 0xf271f372, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa561f375, 95 },
    { 0xb3b1f376, 98 },
    { 0, 0 },
    { 0x32a1f378, 98 },
    { 0x3311f378, 55 },
    { 0x8eb1f37a, 95 },
    { 0xa31f37b, 55 },
    { 0, 0 },
    { 0x62d1f37d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab21f380, 34 },
    { 0xa1f381, 98 },
    { 0, 0 },
    { 0x3031f383, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7261f387, 91 },
    { 0x261f388, 23 },
    { 0xea21f387, 98 },
    { 0x31b1f38a, 33 },
    { 0, 0 },
    { 0x8831f38c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5731f390, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1141f395, 95 },
    { 0x2251f395, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a31f39d, 98 },
    { 0x7f91f39d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5721f3a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b1f3a7, 61 },
    { 0x321f3a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f31f3ab, 90 },
    { 0x5e01f3ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b1f3b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf021f3b3, 84 },
    { 0x35f1f3b3, 98 },
    { 0, 0 },
    { 0x86b1f3b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe501f3ba, 64 },
    { 0x52c1f3bb, 61 },
    { 0x2231f3ba, 75 },
    { 0, 0 },
    { 0xdab1f3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb71f3c1, 99 },
    { 0xde61f3c1, 23 },
    { 0xd621f3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b1f3c9, 98 },
    { 0x4cd1f3c9, 91 },
    { 0x3c71f3c9, 98 },
    { 0x34d1f3ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6391f3d5, 90 },
    { 0xff71f3d6, 90 },
    { 0x7561f3d7, 93 },
    { 0, 0 },
    { 0xa541f3d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9251f3e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c91f3e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e11f3ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa921f3f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d1f3f3, 87 },
    { 0x6f31f3f3, 101 },
    { 0x5a61f3f5, 61 },
    { 0x1ed1f3f5, 91 },
    { 0xcb21f3f7, 55 },
    { 0xee81f3f7, 34 },
    { 0x3e51f3f5, 46 },
    { 0x5211f3f7, 61 },
    { 0x9fc1f3f6, 33 },
    { 0x9bd1f3fb, 95 },
    { 0xf581f3fd, 90 },
    { 0, 0 },
    { 0x5a81f3ff, 93 },
    { 0xacd1f400, 33 },
    { 0x6ad1f401, 61 },
    { 0x3141f401, 55 },
    { 0x8e91f401, 98 },
    { 0x3981f404, 33 },
    { 0, 0 },
    { 0xb871f406, 90 },
    { 0x1451f407, 76 },
    { 0xa681f408, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce1f40b, 61 },
    { 0x1251f40b, 23 },
    { 0xcd71f40d, 76 },
    { 0x7da1f40e, 23 },
    { 0xdf1f40f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd581f414, 90 },
    { 0, 0 },
    { 0x61a1f416, 33 },
    { 0x94b1f417, 90 },
    { 0, 0 },
    { 0xc61f419, 3 },
    { 0xd531f41a, 35 },
    { 0xe051f41b, 61 },
    { 0, 0 },
    { 0x3ca1f41d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff91f424, 76 },
    { 0x9d61f425, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec1f428, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd421f42b, 33 },
    { 0x41a1f42c, 33 },
    { 0, 0 },
    { 0x11c1f42e, 61 },
    { 0x3111f42f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f1f434, 61 },
    { 0xab31f434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5481f438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01f43c, 61 },
    { 0xde91f43d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd1f441, 55 },
    { 0x3f91f442, 61 },
    { 0x7451f442, 55 },
    { 0x3841f444, 38 },
    { 0x6651f445, 66 },
    { 0xdf41f444, 95 },
    { 0x53c1f443, 33 },
    { 0xd841f448, 46 },
    { 0, 0 },
    { 0xfb81f44a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e1f453, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c1f45c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4411f45f, 33 },
    { 0x5d31f460, 90 },
    { 0xf8e1f45f, 66 },
    { 0x8dd1f462, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e1f466, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b1f46b, 95 },
    { 0xa071f46c, 33 },
    { 0, 0 },
    { 0x83d1f46e, 90 },
    { 0xdb21f46e, 55 },
    { 0x7251f46e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab91f474, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc71f477, 81 },
    { 0, 0 },
    { 0xa441f479, 98 },
    { 0x8441f47a, 34 },
    { 0x3b41f47b, 90 },
    { 0x4da1f47a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5781f47f, 17 },
    { 0, 0 },
    { 0x7f21f481, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe691f484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441f48d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d1f491, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591f498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd081f4a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf81f4aa, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7241f4ae, 76 },
    { 0xeb31f4ae, 24 },
    { 0xf461f4b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d01f4b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6781f4b7, 61 },
    { 0xb7a1f4b7, 98 },
    { 0xd5e1f4b8, 90 },
    { 0x7811f4ba, 95 },
    { 0, 0 },
    { 0x5e51f4bc, 67 },
    { 0x1c1f4bd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xca61f4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f4c3, 95 },
    { 0, 0 },
    { 0xae51f4c5, 81 },
    { 0xaa51f4c5, 34 },
    { 0, 0 },
    { 0x21d1f4c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5911f4cf, 61 },
    { 0x9a91f4d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa901f4d4, 34 },
    { 0, 0 },
    { 0xeb51f4d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61f4dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc151f4e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e1f4e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2011f4e7, 90 },
    { 0, 0 },
    { 0xb811f4e9, 98 },
    { 0x8881f4ea, 66 },
    { 0x5ac1f4eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a61f4ee, 95 },
    { 0xbe1f4ef, 66 },
    { 0x5321f4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df1f4fc, 33 },
    { 0, 0 },
    { 0x40e1f4fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e21f503, 90 },
    { 0x4031f504, 48 },
    { 0x2231f504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca1f508, 98 },
    { 0x2e11f509, 90 },
    { 0, 0 },
    { 0xedd1f50b, 23 },
    { 0x4421f50c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f1f511, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb341f514, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e51f518, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d1f51b, 23 },
    { 0x3671f51c, 98 },
    { 0, 0 },
    { 0xfb41f51e, 101 },
    { 0xeec1f51f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b1f524, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df1f527, 34 },
    { 0x63b1f528, 90 },
    { 0xdb01f528, 90 },
    { 0x70f1f52a, 88 },
    { 0x99d1f528, 98 },
    { 0xd551f52c, 90 },
    { 0xf631f52d, 98 },
    { 0xce61f52d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a11f535, 33 },
    { 0x30e1f536, 75 },
    { 0xbed1f535, 90 },
    { 0xc9d1f538, 98 },
    { 0xaad1f539, 90 },
    { 0xa1f1f536, 95 },
    { 0x4b1f53b, 33 },
    { 0x7741f539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51f544, 69 },
    { 0x1f21f545, 61 },
    { 0x2a21f545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc61f54a, 76 },
    { 0x94f1f54b, 99 },
    { 0, 0 },
    { 0x8491f54d, 34 },
    { 0x3d31f54d, 90 },
    { 0x7a11f54f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b1f552, 95 },
    { 0x4031f552, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd1f557, 23 },
    { 0, 0 },
    { 0x9a31f559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3541f55e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7271f567, 90 },
    { 0, 0 },
    { 0x89f1f569, 33 },
    { 0xc481f569, 90 },
    { 0x3ef1f56b, 90 },
    { 0xee91f56b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd101f56f, 81 },
    { 0xb431f56f, 40 },
    { 0, 0 },
    { 0x7751f572, 84 },
    { 0x7f61f573, 33 },
    { 0xf4c1f573, 43 },
    { 0x75d1f573, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1f579, 75 },
    { 0x6931f57a, 95 },
    { 0x9ed1f579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a1f57f, 87 },
    { 0, 0 },
    { 0x3641f581, 55 },
    { 0x2bc1f582, 34 },
    { 0x36a1f581, 34 },
    { 0x6ab1f584, 61 },
    { 0xea31f585, 99 },
    { 0x5cb1f585, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe01f58b, 95 },
    { 0x8c91f58b, 52 },
    { 0, 0 },
    { 0xbf1f58e, 17 },
    { 0, 0 },
    { 0x4631f590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa611f594, 90 },
    { 0xb8f1f595, 23 },
    { 0xb01f595, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db1f599, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae11f5a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f71f5ae, 98 },
    { 0, 0 },
    { 0xee01f5b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3571f5b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa711f5b7, 61 },
    { 0xfee1f5b8, 90 },
    { 0xc321f5b9, 55 },
    { 0x9351f5ba, 55 },
    { 0xc1f1f5bb, 33 },
    { 0x3e51f5bc, 90 },
    { 0x84f1f5bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41f5c1, 91 },
    { 0x7651f5c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f1f5cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971f5d0, 46 },
    { 0xbe1f5d1, 98 },
    { 0, 0 },
    { 0xfe01f5d3, 76 },
    { 0x8dd1f5d4, 76 },
    { 0x98d1f5d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c1f5d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1121f5dc, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4301f5e3, 90 },
    { 0, 0 },
    { 0x4291f5e5, 95 },
    { 0x3aa1f5e6, 33 },
    { 0xa9f1f5e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb1f5ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d1f5f0, 55 },
    { 0x6031f5f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2661f5f4, 61 },
    { 0xd831f5f5, 43 },
    { 0x2931f5f4, 95 },
    { 0x90f1f5f5, 33 },
    { 0x8a61f5f4, 61 },
    { 0x9cb1f5f8, 23 },
    { 0x63a1f5fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda1f5fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2761f603, 90 },
    { 0x9641f604, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6831f608, 76 },
    { 0, 0 },
    { 0x8921f60a, 87 },
    { 0, 0 },
    { 0x9221f60c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6611f612, 90 },
    { 0x5b01f613, 88 },
    { 0xcf01f614, 23 },
    { 0x6c61f613, 34 },
    { 0xd1c1f616, 95 },
    { 0x5f51f617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e81f61c, 33 },
    { 0xf431f61c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b1f620, 95 },
    { 0, 0 },
    { 0x5761f622, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd1f625, 33 },
    { 0x6901f625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa021f62e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb141f635, 98 },
    { 0, 0 },
    { 0x4801f637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d81f63e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c61f647, 69 },
    { 0, 0 },
    { 0xd11f649, 23 },
    { 0x571f64a, 34 },
    { 0x7111f64b, 66 },
    { 0xd9a1f64c, 33 },
    { 0xe7b1f64d, 76 },
    { 0x9ad1f64e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b41f655, 87 },
    { 0x9541f655, 98 },
    { 0x9d91f657, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb21f65b, 52 },
    { 0x2821f65b, 23 },
    { 0x55d1f65c, 75 },
    { 0, 0 },
    { 0x2cd1f65f, 100 },
    { 0xca11f660, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb41f663, 23 },
    { 0, 0 },
    { 0xe331f665, 76 },
    { 0xa991f665, 23 },
    { 0x5601f666, 34 },
    { 0xb2a1f668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3991f66e, 33 },
    { 0x4881f66f, 61 },
    { 0xc5b1f670, 84 },
    { 0x8001f671, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de1f679, 34 },
    { 0, 0 },
    { 0xd211f67b, 33 },
    { 0x7c41f67c, 88 },
    { 0xfd31f67d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc551f682, 61 },
    { 0, 0 },
    { 0xb011f684, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa851f687, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9701f68a, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b41f68f, 66 },
    { 0x1911f68f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a1f694, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4091f697, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe1f69e, 91 },
    { 0, 0 },
    { 0x9bc1f6a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c31f6a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2501f6ad, 83 },
    { 0, 0 },
    { 0xc0f1f6af, 76 },
    { 0x8a91f6b0, 66 },
    { 0x2d81f6af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7231f6b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb581f6ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad1f6c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a1f6c8, 95 },
    { 0xdd11f6c9, 33 },
    { 0, 0 },
    { 0xa311f6cb, 88 },
    { 0, 0 },
    { 0x2241f6cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa1f6d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a81f6db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f21f6e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a1f6e8, 46 },
    { 0x12c1f6e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81f6ed, 55 },
    { 0x13b1f6ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa41f6f1, 78 },
    { 0, 0 },
    { 0xa941f6f3, 95 },
    { 0x8ab1f6f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d1f6fa, 98 },
    { 0x2211f6fb, 78 },
    { 0xa8d1f6fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc1f705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5381f708, 43 },
    { 0x9871f708, 36 },
    { 0x3dd1f708, 87 },
    { 0x351f70b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b1f710, 17 },
    { 0x70a1f711, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f1f714, 76 },
    { 0x7db1f715, 95 },
    { 0x4411f714, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd091f71a, 33 },
    { 0x1b71f71b, 98 },
    { 0, 0 },
    { 0x1f71f71d, 98 },
    { 0xd9c1f71d, 90 },
    { 0x2d71f71f, 95 },
    { 0x47a1f720, 90 },
    { 0xfb41f71d, 8 },
    { 0, 0 },
    { 0xf4e1f723, 90 },
    { 0xac1f724, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5961f729, 23 },
    { 0, 0 },
    { 0xeae1f72b, 75 },
    { 0x14d1f72c, 96 },
    { 0, 0 },
    { 0x33a1f72e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d1f731, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa1f734, 34 },
    { 0, 0 },
    { 0xa0b1f736, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71f739, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3491f73e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f1f741, 100 },
    { 0x2cb1f742, 23 },
    { 0x3631f743, 77 },
    { 0x62b1f744, 61 },
    { 0xf391f745, 23 },
    { 0xcd41f746, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d1f749, 98 },
    { 0xe8b1f74a, 90 },
    { 0x3ab1f74b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb81f753, 90 },
    { 0x491f754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7821f757, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba91f75a, 76 },
    { 0x71f1f75b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c1f75f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3421f765, 98 },
    { 0x651f765, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d71f769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f51f76e, 68 },
    { 0, 0 },
    { 0xa911f770, 34 },
    { 0xf0d1f771, 90 },
    { 0x7891f772, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2051f778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241f77f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2671f787, 75 },
    { 0, 0 },
    { 0xe781f789, 90 },
    { 0, 0 },
    { 0x6271f78b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d41f793, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d1f797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee1f79b, 76 },
    { 0x7241f79c, 90 },
    { 0x111f79d, 43 },
    { 0, 0 },
    { 0x2af1f79f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8481f7a3, 55 },
    { 0xc671f7a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2011f7a9, 90 },
    { 0x5071f7a9, 61 },
    { 0xfbf1f7a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf581f7b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241f7b6, 90 },
    { 0, 0 },
    { 0xee41f7b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b1f7bc, 77 },
    { 0xd3e1f7bc, 78 },
    { 0xfef1f7be, 46 },
    { 0xde1f7bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e11f7c3, 90 },
    { 0xc651f7c4, 90 },
    { 0xf531f7c5, 33 },
    { 0xd2a1f7c6, 98 },
    { 0x51f7c7, 23 },
    { 0x59b1f7c8, 69 },
    { 0xc081f7c9, 76 },
    { 0xad1f7ca, 90 },
    { 0xdbd1f7ca, 90 },
    { 0x9651f7cb, 33 },
    { 0x6591f7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6041f7d2, 90 },
    { 0x4e71f7d3, 46 },
    { 0x2861f7d4, 95 },
    { 0x5841f7d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd771f7d8, 90 },
    { 0x26d1f7d9, 90 },
    { 0x11a1f7da, 34 },
    { 0xc821f7d8, 90 },
    { 0xdc81f7dc, 99 },
    { 0, 0 },
    { 0xfae1f7de, 61 },
    { 0x14d1f7df, 90 },
    { 0x7d1f7df, 90 },
    { 0xd661f7e1, 23 },
    { 0, 0 },
    { 0x90c1f7e3, 52 },
    { 0xf581f7e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c01f7ec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xac21f7ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f21f7f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b1f7f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f91f7fb, 90 },
    { 0x521f7fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871f801, 99 },
    { 0x7d61f802, 76 },
    { 0xeed1f803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd1f80f, 34 },
    { 0xff91f810, 61 },
    { 0x7f61f811, 98 },
    { 0x1381f811, 90 },
    { 0xb611f813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3151f81b, 77 },
    { 0xf131f81c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8311f821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b1f826, 98 },
    { 0, 0 },
    { 0xf81f828, 76 },
    { 0x2a21f829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef1f82c, 66 },
    { 0x78c1f82d, 33 },
    { 0xdbe1f82e, 90 },
    { 0x2cc1f82d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd681f834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6561f838, 95 },
    { 0xa031f838, 55 },
    { 0xf921f839, 33 },
    { 0x5ca1f839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df1f83f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd591f844, 55 },
    { 0xdf91f845, 33 },
    { 0, 0 },
    { 0x2841f847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e21f84d, 56 },
    { 0x2cd1f84d, 33 },
    { 0, 0 },
    { 0x16e1f850, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d01f856, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981f85c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f21f865, 33 },
    { 0xc8d1f865, 33 },
    { 0, 0 },
    { 0xd881f868, 95 },
    { 0x30d1f869, 98 },
    { 0xe4f1f86a, 61 },
    { 0xa391f86b, 23 },
    { 0x7c91f86b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb1f870, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb251f873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c1f87b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea1f87f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5961f884, 68 },
    { 0xd21f884, 76 },
    { 0x4571f886, 33 },
    { 0x8bd1f884, 23 },
    { 0, 0 },
    { 0x50e1f889, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1f88e, 34 },
    { 0xdaa1f88f, 23 },
    { 0x7d1f890, 64 },
    { 0x981f890, 23 },
    { 0x5741f88e, 98 },
    { 0xecf1f892, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d1f897, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb111f89c, 61 },
    { 0, 0 },
    { 0x1c1f89e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e71f8a1, 61 },
    { 0xce51f8a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b1f8a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d1f8ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e1f8b4, 23 },
    { 0x1f71f8b5, 76 },
    { 0, 0 },
    { 0x2b21f8b7, 90 },
    { 0, 0 },
    { 0x49e1f8b9, 33 },
    { 0xa8a1f8ba, 61 },
    { 0xc991f8bb, 90 },
    { 0x1731f8ba, 61 },
    { 0x1821f8bd, 99 },
    { 0x1451f8be, 84 },
    { 0x1a61f8bf, 87 },
    { 0x3a41f8bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd601f8c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e01f8c9, 95 },
    { 0xfae1f8ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791f8ce, 61 },
    { 0, 0 },
    { 0xd0c1f8d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x481f8d3, 33 },
    { 0x6b01f8d4, 95 },
    { 0, 0 },
    { 0x2e61f8d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e11f8da, 66 },
    { 0, 0 },
    { 0xd631f8dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f1f8e9, 43 },
    { 0, 0 },
    { 0xd181f8eb, 93 },
    { 0x5551f8eb, 23 },
    { 0x13d1f8ed, 46 },
    { 0xef41f8eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a21f8f1, 61 },
    { 0x8491f8f2, 95 },
    { 0xdc41f8f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa251f8f8, 99 },
    { 0x18b1f8f8, 95 },
    { 0, 0 },
    { 0x9381f8fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d41f8fe, 84 },
    { 0x7f11f8fe, 43 },
    { 0xbe1f900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51f907, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41f90c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2511f912, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a1f919, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a51f91c, 43 },
    { 0x8111f91d, 23 },
    { 0xcca1f91e, 65 },
    { 0xf991f91e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f1f923, 95 },
    { 0, 0 },
    { 0xde21f925, 90 },
    { 0xac1f926, 61 },
    { 0x4e51f926, 98 },
    { 0x1d61f928, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21f92f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf001f933, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb931f936, 85 },
    { 0xb611f937, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa41f93b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e51f93e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7581f943, 61 },
    { 0x39a1f943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec1f94d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5151f95b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d1f95e, 94 },
    { 0xcf51f95f, 76 },
    { 0xcda1f95f, 33 },
    { 0xde31f961, 61 },
    { 0, 0 },
    { 0x38f1f963, 85 },
    { 0x2ab1f963, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551f967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb91f96d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd211f972, 43 },
    { 0xd981f973, 95 },
    { 0x7421f972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a91f977, 33 },
    { 0xb891f977, 98 },
    { 0, 0 },
    { 0x401f97a, 46 },
    { 0xe391f97b, 95 },
    { 0, 0 },
    { 0x2771f97d, 76 },
    { 0, 0 },
    { 0x9a71f97f, 46 },
    { 0xe3c1f97f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5711f987, 52 },
    { 0x85e1f988, 61 },
    { 0x3f1f987, 93 },
    { 0, 0 },
    { 0x6db1f98b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b1f990, 67 },
    { 0x6661f991, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce91f996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe441f999, 100 },
    { 0xb701f999, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e91f99d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4511f9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8031f9aa, 40 },
    { 0, 0 },
    { 0x1b41f9ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c1f9bb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3061f9c6, 55 },
    { 0x19f1f9c7, 98 },
    { 0, 0 },
    { 0x9631f9c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141f9ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a51f9da, 95 },
    { 0, 0 },
    { 0x5e11f9dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde61f9df, 51 },
    { 0xcbf1f9df, 55 },
    { 0xc3a1f9e1, 90 },
    { 0x8f01f9e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd171f9e5, 34 },
    { 0xba01f9e6, 96 },
    { 0x6481f9e7, 23 },
    { 0x9071f9e5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb1f9ec, 91 },
    { 0x9561f9ed, 43 },
    { 0, 0 },
    { 0x5a41f9ef, 51 },
    { 0, 0 },
    { 0x3f81f9f1, 87 },
    { 0x2cb1f9f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6431f9fa, 99 },
    { 0, 0 },
    { 0xdc11f9fc, 61 },
    { 0x64e1f9fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1181fa00, 61 },
    { 0, 0 },
    { 0x8c71fa02, 90 },
    { 0x8f21fa03, 87 },
    { 0, 0 },
    { 0xa0d1fa05, 98 },
    { 0, 0 },
    { 0xcf51fa07, 90 },
    { 0, 0 },
    { 0x7551fa09, 43 },
    { 0, 0 },
    { 0x6ea1fa0b, 87 },
    { 0x8331fa0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be1fa0f, 33 },
    { 0xced1fa10, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5271fa15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781fa1b, 100 },
    { 0x80b1fa1c, 33 },
    { 0xefe1fa1d, 98 },
    { 0, 0 },
    { 0xad61fa1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e21fa23, 61 },
    { 0x87d1fa24, 98 },
    { 0xb5c1fa25, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd1fa2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce11fa36, 23 },
    { 0x20d1fa36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161fa3c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f1fa41, 52 },
    { 0, 0 },
    { 0x4cf1fa43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1901fa47, 87 },
    { 0x3131fa48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5041fa4b, 95 },
    { 0x77c1fa4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf31fa56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a1fa59, 98 },
    { 0x4641fa59, 33 },
    { 0xa2a1fa5b, 34 },
    { 0x2e61fa5b, 34 },
    { 0xb791fa5c, 95 },
    { 0, 0 },
    { 0x27a1fa5f, 23 },
    { 0xd971fa60, 77 },
    { 0xb551fa60, 98 },
    { 0xfd81fa5f, 23 },
    { 0, 0 },
    { 0xd3d1fa64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141fa69, 95 },
    { 0, 0 },
    { 0x111fa6b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1fa73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca1fa76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1671fa7d, 66 },
    { 0x9b11fa7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d41fa86, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c91fa89, 61 },
    { 0x3161fa8a, 78 },
    { 0xf621fa8a, 34 },
    { 0x84c1fa8b, 34 },
    { 0x74d1fa8a, 33 },
    { 0xdc91fa8c, 90 },
    { 0x2861fa8f, 90 },
    { 0x8b91fa89, 99 },
    { 0xd731fa91, 90 },
    { 0xc91fa91, 66 },
    { 0xa161fa92, 23 },
    { 0, 0 },
    { 0x7b41fa95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81fa98, 91 },
    { 0x6841fa98, 23 },
    { 0x8dc1fa9a, 80 },
    { 0x1971fa98, 88 },
    { 0xace1fa9a, 66 },
    { 0x33b1fa9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1211faa0, 61 },
    { 0xf541faa1, 52 },
    { 0xa761faa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e1faa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a1faa9, 90 },
    { 0, 0 },
    { 0x121faab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb191fab0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd131fab3, 87 },
    { 0x1101fab3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4921faba, 84 },
    { 0xc071faba, 23 },
    { 0x281fabb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d1fac4, 76 },
    { 0x9121fac5, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761fac9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde11facd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e01fad1, 95 },
    { 0x6dc1fad1, 99 },
    { 0xa031fad1, 55 },
    { 0, 0 },
    { 0x5ae1fad5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc91fad8, 40 },
    { 0x9c61fad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea41fadc, 23 },
    { 0, 0 },
    { 0xb1f1fade, 33 },
    { 0x7601fadf, 95 },
    { 0xd521fade, 98 },
    { 0x89e1fae0, 90 },
    { 0xb0c1fae2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81fae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff1fae8, 98 },
    { 0x3b01fae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4911faee, 95 },
    { 0x8561faef, 76 },
    { 0x1d61faef, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5311faf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c1fafc, 55 },
    { 0x3961fafc, 23 },
    { 0xb51fafc, 43 },
    { 0xc611fafe, 95 },
    { 0, 0 },
    { 0x8281fb01, 23 },
    { 0x3421fb02, 33 },
    { 0, 0 },
    { 0xaf01fb04, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd91fb0c, 90 },
    { 0xad1fb0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c1fb14, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb51fb19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a1fb1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf311fb22, 40 },
    { 0x9ae1fb22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8251fb2a, 33 },
    { 0xdf91fb2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc01fb2f, 61 },
    { 0, 0 },
    { 0xd5d1fb31, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd731fb34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c1fb37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b1fb3a, 43 },
    { 0, 0 },
    { 0x4a11fb3c, 61 },
    { 0, 0 },
    { 0xeb41fb3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe1fb41, 99 },
    { 0, 0 },
    { 0xcc11fb43, 23 },
    { 0x69c1fb44, 90 },
    { 0, 0 },
    { 0x7ea1fb46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5521fb52, 76 },
    { 0x4c61fb53, 90 },
    { 0x8d11fb54, 61 },
    { 0x70b1fb53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc1fb5c, 42 },
    { 0xe6b1fb5c, 95 },
    { 0x1f1fb5e, 76 },
    { 0x6e41fb5d, 23 },
    { 0xf851fb5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e1fb67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1fb6e, 61 },
    { 0x7201fb6e, 34 },
    { 0, 0 },
    { 0x7371fb71, 95 },
    { 0xfa91fb72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa261fb76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f1fb79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa601fb7e, 52 },
    { 0xef21fb7f, 99 },
    { 0x20f1fb80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4291fb86, 90 },
    { 0, 0 },
    { 0x91b1fb88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec1fb90, 33 },
    { 0x7d51fb90, 77 },
    { 0xa041fb92, 34 },
    { 0x4031fb93, 90 },
    { 0, 0 },
    { 0xe091fb95, 84 },
    { 0, 0 },
    { 0xd161fb97, 78 },
    { 0, 0 },
    { 0xf691fb99, 52 },
    { 0x48a1fb99, 34 },
    { 0x7de1fb99, 95 },
    { 0, 0 },
    { 0x7941fb9d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc401fba1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9311fba4, 76 },
    { 0, 0 },
    { 0x6761fba6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd061fba9, 90 },
    { 0x5ad1fbaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8651fbae, 85 },
    { 0, 0 },
    { 0xbb31fbb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d41fbb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91fbb7, 91 },
    { 0, 0 },
    { 0x4be1fbb9, 34 },
    { 0xcac1fbb9, 61 },
    { 0, 0 },
    { 0x42b1fbbc, 46 },
    { 0xe3c1fbbc, 68 },
    { 0, 0 },
    { 0xe2e1fbbf, 90 },
    { 0, 0 },
    { 0xcf21fbc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa1fbc4, 33 },
    { 0, 0 },
    { 0x7271fbc6, 90 },
    { 0x9d1fbc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf231fbcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf821fbd0, 97 },
    { 0xb1c1fbd1, 99 },
    { 0xea41fbd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb91fbd5, 90 },
    { 0xc811fbd5, 68 },
    { 0x58a1fbd7, 66 },
    { 0xc9c1fbd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3021fbdd, 33 },
    { 0x151fbde, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe911fbe2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea1fbe9, 95 },
    { 0x3321fbea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f1fbed, 33 },
    { 0xdc41fbee, 98 },
    { 0x1281fbef, 95 },
    { 0, 0 },
    { 0x7c31fbf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7851fbf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5321fc01, 76 },
    { 0xa2c1fc02, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c81fc0b, 87 },
    { 0xc4b1fc0c, 93 },
    { 0, 0 },
    { 0xffa1fc0e, 95 },
    { 0xcd51fc0f, 52 },
    { 0x6f21fc10, 23 },
    { 0xc511fc11, 98 },
    { 0xa2e1fc11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c1fc1a, 46 },
    { 0x57b1fc1b, 84 },
    { 0, 0 },
    { 0x5001fc1d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a1fc20, 61 },
    { 0, 0 },
    { 0x63f1fc22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f91fc25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa331fc29, 90 },
    { 0xe3d1fc2a, 66 },
    { 0x1711fc2b, 90 },
    { 0x7ec1fc2c, 52 },
    { 0, 0 },
    { 0x8681fc2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e1fc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd1fc37, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa61fc3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf061fc3f, 67 },
    { 0x4bb1fc40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a1fc47, 46 },
    { 0x50d1fc48, 17 },
    { 0x40b1fc49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3861fc4d, 23 },
    { 0, 0 },
    { 0x4731fc4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f51fc52, 76 },
    { 0x7e71fc52, 61 },
    { 0, 0 },
    { 0xc441fc55, 90 },
    { 0x211fc56, 51 },
    { 0xfa11fc57, 55 },
    { 0xe4b1fc58, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5381fc5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a91fc60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c1fc63, 33 },
    { 0x7b11fc63, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9241fc67, 76 },
    { 0x5201fc68, 23 },
    { 0x96f1fc69, 90 },
    { 0xd291fc6a, 61 },
    { 0xf841fc67, 99 },
    { 0xde1fc6c, 76 },
    { 0xf001fc6d, 64 },
    { 0, 0 },
    { 0x99d1fc6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2111fc73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d1fc76, 55 },
    { 0x7eb1fc76, 90 },
    { 0xd941fc78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21fc7e, 98 },
    { 0x1f01fc7e, 40 },
    { 0x201fc80, 92 },
    { 0x2e11fc81, 87 },
    { 0, 0 },
    { 0x1b51fc83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5591fc87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc1fc8a, 33 },
    { 0, 0 },
    { 0x6661fc8c, 90 },
    { 0x6891fc8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b1fc93, 95 },
    { 0xa771fc94, 81 },
    { 0xe351fc95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9401fc98, 81 },
    { 0x9751fc99, 46 },
    { 0x7da1fc98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb361fc9e, 91 },
    { 0x8f11fc9e, 97 },
    { 0x57b1fca0, 87 },
    { 0x6e31fc9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa031fca4, 43 },
    { 0, 0 },
    { 0x43f1fca6, 95 },
    { 0x5221fca6, 46 },
    { 0x8111fca8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f1fcab, 96 },
    { 0x18f1fcab, 43 },
    { 0, 0 },
    { 0x5351fcae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a1fcb1, 98 },
    { 0x401fcb1, 98 },
    { 0x85f1fcb3, 99 },
    { 0xd281fcb4, 89 },
    { 0x88e1fcb5, 33 },
    { 0x4121fcb4, 34 },
    { 0x5e81fcb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81fcba, 99 },
    { 0, 0 },
    { 0x4ba1fcbc, 97 },
    { 0xd6c1fcbc, 91 },
    { 0xdc51fcbe, 61 },
    { 0x811fcbe, 95 },
    { 0x2f51fcbe, 76 },
    { 0x5051fcbe, 61 },
    { 0x8d61fcbd, 33 },
    { 0xeb11fcc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb41fcc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f1fcc9, 23 },
    { 0xabc1fcc9, 95 },
    { 0xccd1fccb, 90 },
    { 0x64a1fcca, 34 },
    { 0x6251fccc, 98 },
    { 0xe7a1fcca, 75 },
    { 0x9d81fcc9, 55 },
    { 0, 0 },
    { 0x3b71fcd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db1fcd4, 90 },
    { 0, 0 },
    { 0x5b01fcd6, 33 },
    { 0xa601fcd6, 87 },
    { 0xc841fcd8, 22 },
    { 0xa3d1fcd9, 99 },
    { 0xeb51fcd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb51fcdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf51fce0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd041fcea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d1fced, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xad11fcf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5981fcf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1461fcf8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fd01, 95 },
    { 0xeb81fd02, 95 },
    { 0, 0 },
    { 0x42c1fd04, 90 },
    { 0, 0 },
    { 0xd291fd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331fd0e, 55 },
    { 0x47a1fd0e, 23 },
    { 0xee11fd10, 68 },
    { 0x4db1fd0e, 55 },
    { 0xf7f1fd11, 61 },
    { 0, 0 },
    { 0xdea1fd14, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a1fd1d, 33 },
    { 0, 0 },
    { 0xe441fd1f, 33 },
    { 0xfb81fd20, 95 },
    { 0xe851fd21, 95 },
    { 0x8b1fd22, 33 },
    { 0x3b71fd22, 46 },
    { 0x7341fd23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe241fd29, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf1fd2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8741fd36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb11fd3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2681fd3f, 66 },
    { 0x2251fd40, 65 },
    { 0xd1e1fd41, 89 },
    { 0x8e41fd41, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21fd45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6901fd48, 33 },
    { 0xa4b1fd49, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b81fd4f, 95 },
    { 0x9921fd50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa841fd53, 100 },
    { 0x8881fd53, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf1fd57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec21fd5b, 95 },
    { 0, 0 },
    { 0x7d51fd5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9601fd60, 43 },
    { 0x6451fd61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db1fd64, 90 },
    { 0, 0 },
    { 0xb291fd66, 33 },
    { 0, 0 },
    { 0x7531fd68, 91 },
    { 0x1781fd69, 76 },
    { 0x8351fd6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9541fd71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe111fd74, 46 },
    { 0x3be1fd75, 17 },
    { 0x5fb1fd76, 90 },
    { 0x1041fd74, 90 },
    { 0x94c1fd74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2961fd7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a1fd81, 33 },
    { 0, 0 },
    { 0x1331fd83, 66 },
    { 0xaec1fd84, 33 },
    { 0x17e1fd83, 87 },
    { 0xd5a1fd85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe81fd89, 55 },
    { 0, 0 },
    { 0xb4c1fd8b, 33 },
    { 0x3751fd8c, 98 },
    { 0x12a1fd8d, 55 },
    { 0x1bd1fd8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe571fd93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a01fd96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4341fd9e, 85 },
    { 0x8b51fd9f, 98 },
    { 0xab51fda0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81fda3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8711fda6, 22 },
    { 0xef1fda6, 98 },
    { 0x9cb1fda8, 43 },
    { 0, 0 },
    { 0x1951fdaa, 33 },
    { 0x4d11fdab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa691fdb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed1fdb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba1fdba, 95 },
    { 0, 0 },
    { 0xe941fdbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a1fdc1, 33 },
    { 0x67e1fdc2, 33 },
    { 0x79d1fdc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b11fdc7, 61 },
    { 0xb3d1fdc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8431fdcd, 43 },
    { 0xa391fdce, 55 },
    { 0, 0 },
    { 0x8e41fdd0, 61 },
    { 0xe9a1fdd0, 95 },
    { 0xa501fdd2, 98 },
    { 0x7781fdd3, 84 },
    { 0xfcb1fdd4, 90 },
    { 0xda71fdd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd41fddc, 46 },
    { 0xddd1fddd, 69 },
    { 0xa821fdde, 33 },
    { 0, 0 },
    { 0x7891fde0, 90 },
    { 0, 0 },
    { 0x7001fde2, 90 },
    { 0xded1fde2, 33 },
    { 0x6ed1fde4, 85 },
    { 0x7b01fde5, 78 },
    { 0, 0 },
    { 0x6d01fde7, 46 },
    { 0xfc51fde8, 61 },
    { 0x9ce1fde9, 96 },
    { 0x88a1fdea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df1fded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e1fdf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c51fdf3, 34 },
    { 0x4dc1fdf3, 76 },
    { 0, 0 },
    { 0x9621fdf6, 40 },
    { 0x5e21fdf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c1fdfb, 33 },
    { 0x2241fdfc, 66 },
    { 0, 0 },
    { 0xb911fdfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c31fe01, 33 },
    { 0xe211fe01, 33 },
    { 0x4681fe03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1411fe07, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7151fe0e, 84 },
    { 0, 0 },
    { 0xa921fe10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb961fe17, 93 },
    { 0x4761fe18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd1fe1f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be1fe22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1fe29, 40 },
    { 0, 0 },
    { 0xd991fe2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf1fe30, 69 },
    { 0xea81fe30, 98 },
    { 0xd131fe30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe971fe36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc21fe3d, 61 },
    { 0xf221fe3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8481fe42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e11fe46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401fe4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd891fe4e, 95 },
    { 0xa881fe4e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9731fe54, 96 },
    { 0xbd31fe55, 90 },
    { 0xd391fe54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b11fe5e, 23 },
    { 0xde81fe5f, 52 },
    { 0x1bd1fe60, 61 },
    { 0x6b21fe61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8761fe65, 33 },
    { 0x85d1fe65, 61 },
    { 0x6f41fe65, 33 },
    { 0x7401fe68, 34 },
    { 0, 0 },
    { 0xf801fe6a, 91 },
    { 0x5981fe6a, 43 },
    { 0x6841fe6c, 33 },
    { 0xab61fe6d, 55 },
    { 0xcfe1fe6e, 98 },
    { 0x1d61fe6e, 33 },
    { 0xcd1fe70, 34 },
    { 0, 0 },
    { 0xa961fe72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb31fe79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d51fe7c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd1fe7f, 34 },
    { 0x6441fe80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf351fe85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d1fe8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2511fe99, 90 },
    { 0xbd61fe9a, 95 },
    { 0xb311fe9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6511fea1, 46 },
    { 0x59d1fea2, 61 },
    { 0x7891fea2, 80 },
    { 0, 0 },
    { 0xd331fea5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551fea8, 61 },
    { 0, 0 },
    { 0x76a1feaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4871feb0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee1feb3, 46 },
    { 0x6e51feb4, 33 },
    { 0x40d1feb4, 34 },
    { 0xeaa1feb6, 98 },
    { 0x9fd1feb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2751febb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa1febf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c1fec2, 33 },
    { 0, 0 },
    { 0x9511fec4, 97 },
    { 0x7ca1fec5, 95 },
    { 0xcf91fec5, 91 },
    { 0x84e1fec6, 67 },
    { 0x2ec1fec4, 33 },
    { 0, 0 },
    { 0xca61feca, 43 },
    { 0x1e71fecb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e1fece, 90 },
    { 0xece1fece, 40 },
    { 0, 0 },
    { 0x53a1fed1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1fed8, 98 },
    { 0, 0 },
    { 0xc41feda, 80 },
    { 0x4b71fedb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce1fee2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4281fee5, 90 },
    { 0, 0 },
    { 0x9be1fee7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd671fef0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd91fef3, 61 },
    { 0x6111fef4, 52 },
    { 0xd431fef5, 90 },
    { 0, 0 },
    { 0xc4c1fef7, 34 },
    { 0, 0 },
    { 0x2031fef9, 90 },
    { 0xea01fefa, 84 },
    { 0x22c1fefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb01fefe, 77 },
    { 0x9b21feff, 61 },
    { 0, 0 },
    { 0xd141ff01, 56 },
    { 0x56e1ff01, 90 },
    { 0x7371ff03, 33 },
    { 0xf81ff04, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e1ff08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f11ff0b, 23 },
    { 0x7361ff0b, 34 },
    { 0x7031ff0d, 48 },
    { 0xfb31ff0e, 61 },
    { 0x1101ff0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b1ff12, 43 },
    { 0, 0 },
    { 0x33f1ff14, 87 },
    { 0xf521ff14, 43 },
    { 0xdc01ff15, 90 },
    { 0x5811ff14, 34 },
    { 0x1c41ff18, 90 },
    { 0x2141ff19, 61 },
    { 0, 0 },
    { 0x5f61ff1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9091ff1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb1ff24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3791ff2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e1ff39, 19 },
    { 0, 0 },
    { 0xa561ff3b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d1ff41, 17 },
    { 0x9c61ff42, 39 },
    { 0xf6d1ff42, 95 },
    { 0xacf1ff42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc441ff48, 84 },
    { 0x21c1ff49, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9721ff51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd301ff55, 90 },
    { 0, 0 },
    { 0x7961ff57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b21ff5f, 61 },
    { 0xa9e1ff5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4581ff64, 95 },
    { 0x83c1ff65, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xef21ff68, 91 },
    { 0, 0 },
    { 0x2fc1ff6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b81ff71, 34 },
    { 0xf261ff72, 98 },
    { 0x5e91ff71, 61 },
    { 0, 0 },
    { 0xf131ff75, 43 },
    { 0x26b1ff76, 23 },
    { 0x6041ff77, 98 },
    { 0, 0 },
    { 0x8ff1ff79, 88 },
    { 0, 0 },
    { 0x7631ff7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb001ff80, 98 },
    { 0, 0 },
    { 0x3ed1ff82, 98 },
    { 0xd391ff83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51ff87, 33 },
    { 0x3b01ff87, 76 },
    { 0, 0 },
    { 0x1fc1ff8a, 33 },
    { 0x9871ff8b, 76 },
    { 0x4b51ff8c, 53 },
    { 0xd791ff8d, 33 },
    { 0x8f81ff8e, 55 },
    { 0xe11ff8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5211ff9e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3021ffa2, 87 },
    { 0x7141ffa3, 93 },
    { 0xb601ffa2, 33 },
    { 0x5d31ffa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd1ffaa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb1ffae, 88 },
    { 0, 0 },
    { 0x2841ffb0, 34 },
    { 0x8661ffb1, 46 },
    { 0x2611ffb1, 78 },
    { 0xdec1ffb1, 34 },
    { 0, 0 },
    { 0x9831ffb5, 23 },
    { 0x33c1ffb6, 67 },
    { 0, 0 },
    { 0x5351ffb8, 33 },
    { 0x55f1ffb9, 95 },
    { 0, 0 },
    { 0x6231ffbb, 43 },
    { 0xd101ffbc, 98 },
    { 0xba51ffbc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e1ffc0, 69 },
    { 0xfb41ffc1, 43 },
    { 0x9f61ffc1, 90 },
    { 0x9d31ffc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb91ffc7, 46 },
    { 0x2e31ffc8, 92 },
    { 0xbc11ffc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee61ffcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5551ffd3, 81 },
    { 0, 0 },
    { 0x941ffd5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d1ffd9, 81 },
    { 0xa4e1ffda, 46 },
    { 0x89f1ffdb, 95 },
    { 0xffe1ffd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9371ffe2, 100 },
    { 0x5281ffe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc521ffeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61ffee, 61 },
    { 0x7aa1ffef, 90 },
    { 0xfd1ffee, 33 },
    { 0x7b41fff1, 46 },
    { 0x471fff2, 48 },
    { 0xf6d1fff2, 95 },
    { 0x2491fff4, 48 },
    { 0, 0 },
    { 0x31a1fff6, 91 },
    { 0x8a01fff7, 61 },
    { 0xf6d1fff8, 43 },
    { 0x8cd1fff9, 77 },
    { 0x4b41fffa, 67 },
    { 0, 0 },
    { 0x39f1fffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3720005, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6582000a, 95 },
    { 0x28b2000b, 95 },
    { 0xd1d2000c, 94 },
    { 0x9c2000d, 93 },
    { 0xe0e2000e, 95 },
    { 0xbd82000f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e20013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58620018, 55 },
    { 0, 0 },
    { 0x54b2001a, 34 },
    { 0xdea2001b, 33 },
    { 0x5762001a, 23 },
    { 0, 0 },
    { 0x4ee2001e, 96 },
    { 0, 0 },
    { 0xc4d20020, 91 },
    { 0xb3520021, 78 },
    { 0xa5520022, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe420027, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262002a, 94 },
    { 0, 0 },
    { 0x6be2002c, 90 },
    { 0xfcb2002d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c220030, 52 },
    { 0xdec20030, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78220036, 90 },
    { 0, 0 },
    { 0x2f320038, 61 },
    { 0, 0 },
    { 0x2ea2003a, 46 },
    { 0, 0 },
    { 0x4a52003c, 34 },
    { 0, 0 },
    { 0x9062003e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7720043, 33 },
    { 0xaaa20043, 69 },
    { 0, 0 },
    { 0x44b20046, 61 },
    { 0, 0 },
    { 0xd8b20048, 76 },
    { 0, 0 },
    { 0x2f22004a, 53 },
    { 0xc5d2004a, 34 },
    { 0, 0 },
    { 0x7d52004d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x89020050, 68 },
    { 0x69820051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e2006c, 90 },
    { 0x6152006d, 23 },
    { 0xebe2006e, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f20076, 61 },
    { 0x48620077, 98 },
    { 0, 0 },
    { 0x48620079, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd42007c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98520080, 61 },
    { 0, 0 },
    { 0xb0b20082, 55 },
    { 0x42020083, 76 },
    { 0xa5b20083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb32008a, 33 },
    { 0x1902008b, 52 },
    { 0xcc22008c, 45 },
    { 0xda62008d, 98 },
    { 0x4ce2008c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50820094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf42009e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed200a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22200ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1200b1, 98 },
    { 0xf83200b2, 23 },
    { 0x75b200b3, 33 },
    { 0xc82200b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4200b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d200bc, 46 },
    { 0xcb2200bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51200c2, 33 },
    { 0, 0 },
    { 0x28d200c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e200c7, 85 },
    { 0xc8b200c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe200d2, 61 },
    { 0, 0 },
    { 0x86d200d4, 93 },
    { 0xaa5200d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea200dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e200e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151200e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c200eb, 84 },
    { 0, 0 },
    { 0x8a9200ed, 43 },
    { 0x403200ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad200f4, 61 },
    { 0x380200f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5200fa, 98 },
    { 0x76c200fb, 23 },
    { 0xe15200fc, 90 },
    { 0x7eb200fd, 55 },
    { 0x61b200fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11820104, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74120108, 61 },
    { 0, 0 },
    { 0x7722010a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b20110, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47820116, 83 },
    { 0xfc320116, 33 },
    { 0x72020118, 87 },
    { 0xddd20117, 96 },
    { 0xc582011a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa192011e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f420122, 65 },
    { 0, 0 },
    { 0xee120124, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9020128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f22012b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90620130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab20133, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20136, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d20144, 90 },
    { 0xf7220145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c520149, 34 },
    { 0, 0 },
    { 0xc912014b, 76 },
    { 0, 0 },
    { 0x6c62014d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86420153, 33 },
    { 0xaa120154, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a20159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c2015d, 90 },
    { 0, 0 },
    { 0xd3e2015f, 90 },
    { 0x3542015f, 90 },
    { 0, 0 },
    { 0x7ad20162, 33 },
    { 0, 0 },
    { 0xf2920164, 33 },
    { 0x3a20164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf182016d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac20170, 90 },
    { 0x69e20170, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f20178, 33 },
    { 0, 0 },
    { 0x6b32017a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2652017e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe20183, 99 },
    { 0x6b120184, 90 },
    { 0x14d20185, 90 },
    { 0xcd520185, 34 },
    { 0x3420185, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292018c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf20195, 90 },
    { 0, 0 },
    { 0x46b20197, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1462019b, 33 },
    { 0, 0 },
    { 0x99a2019d, 75 },
    { 0, 0 },
    { 0xa652019f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20201a9, 98 },
    { 0xbdf201a9, 77 },
    { 0x517201aa, 55 },
    { 0, 0 },
    { 0x74b201ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c201b4, 76 },
    { 0xbd201b4, 84 },
    { 0, 0 },
    { 0x1fe201b7, 43 },
    { 0, 0 },
    { 0xe7201b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a201bc, 95 },
    { 0, 0 },
    { 0x360201be, 93 },
    { 0x3c4201bf, 43 },
    { 0x167201c0, 46 },
    { 0xbf9201c1, 88 },
    { 0x8dc201c1, 95 },
    { 0x5a4201c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x208201c6, 98 },
    { 0, 0 },
    { 0x3ee201c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd201cc, 85 },
    { 0x198201cd, 46 },
    { 0x958201ce, 33 },
    { 0xc79201cf, 43 },
    { 0x21b201d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42201d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73201dd, 85 },
    { 0, 0 },
    { 0xf7f201df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89201e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986201e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x117201ea, 33 },
    { 0xf6d201ea, 98 },
    { 0x5db201ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b201f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16201f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827201fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb20200, 84 },
    { 0, 0 },
    { 0xdde20202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89420205, 43 },
    { 0, 0 },
    { 0xb6920207, 33 },
    { 0x61120208, 47 },
    { 0x71e20208, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf62020c, 98 },
    { 0xbf32020d, 23 },
    { 0x2622020c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d920212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43420218, 90 },
    { 0x80020219, 23 },
    { 0x81120219, 23 },
    { 0x5402021b, 90 },
    { 0, 0 },
    { 0xba42021d, 55 },
    { 0xc712021e, 98 },
    { 0, 0 },
    { 0xf0720220, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2920224, 37 },
    { 0, 0 },
    { 0x3020226, 87 },
    { 0x4b020227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f2022d, 98 },
    { 0, 0 },
    { 0xc6e2022f, 98 },
    { 0, 0 },
    { 0x80a20231, 68 },
    { 0x19320231, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8120235, 23 },
    { 0, 0 },
    { 0x6c420237, 93 },
    { 0x65720237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1622023c, 52 },
    { 0x86b2023c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5920242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae720245, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f20248, 78 },
    { 0, 0 },
    { 0x9ec2024a, 98 },
    { 0xf532024b, 95 },
    { 0x28c2024b, 53 },
    { 0x4c52024c, 23 },
    { 0x5992024e, 76 },
    { 0xa932024e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db20252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9720259, 55 },
    { 0xf2b2025a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf362025d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3520261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45220270, 68 },
    { 0xb5d20271, 61 },
    { 0x38520272, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f20275, 33 },
    { 0xce120275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e02027a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e2027f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5220283, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c20287, 33 },
    { 0xa3120288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b2028c, 33 },
    { 0x64d2028c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9920292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2029d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3202a5, 87 },
    { 0x423202a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459202ad, 55 },
    { 0x669202ae, 33 },
    { 0x331202ad, 95 },
    { 0xd88202b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84202b5, 98 },
    { 0x670202b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b202b9, 90 },
    { 0, 0 },
    { 0x73d202bb, 77 },
    { 0, 0 },
    { 0x998202bd, 76 },
    { 0, 0 },
    { 0xd4f202bf, 33 },
    { 0x281202bf, 33 },
    { 0xf18202c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318202c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69202d1, 66 },
    { 0x5a7202d1, 34 },
    { 0, 0 },
    { 0x557202d4, 90 },
    { 0, 0 },
    { 0x6c5202d6, 34 },
    { 0, 0 },
    { 0x714202d8, 90 },
    { 0, 0 },
    { 0x298202da, 75 },
    { 0x144202db, 33 },
    { 0xe57202db, 93 },
    { 0x93d202dd, 90 },
    { 0x9d202dd, 78 },
    { 0x774202dc, 61 },
    { 0x9cb202da, 66 },
    { 0xb9202e1, 87 },
    { 0, 0 },
    { 0xe66202e3, 61 },
    { 0xc43202e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05202ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1202ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x701202f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e202f9, 61 },
    { 0xbeb202fa, 93 },
    { 0xd6b202fa, 55 },
    { 0xada202fc, 90 },
    { 0, 0 },
    { 0x54c202fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c20302, 95 },
    { 0, 0 },
    { 0xe7720304, 67 },
    { 0x9d520305, 46 },
    { 0xc3520305, 95 },
    { 0x9220307, 66 },
    { 0xadd20308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac42030b, 61 },
    { 0x24a2030c, 65 },
    { 0, 0 },
    { 0x182030e, 61 },
    { 0, 0 },
    { 0x20220310, 87 },
    { 0xd0820311, 99 },
    { 0, 0 },
    { 0xd5a20313, 90 },
    { 0xbee20314, 33 },
    { 0xfb120315, 55 },
    { 0x14d20313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f2031a, 33 },
    { 0x3a52031a, 87 },
    { 0x5e2031b, 90 },
    { 0x99a2031d, 61 },
    { 0xc572031b, 90 },
    { 0x4ea2031f, 33 },
    { 0x69420320, 95 },
    { 0x1652031b, 78 },
    { 0, 0 },
    { 0x7c220323, 78 },
    { 0xd4320324, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f20328, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c2032b, 76 },
    { 0, 0 },
    { 0xff42032d, 61 },
    { 0x6d2032d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a20337, 33 },
    { 0x7fd20338, 33 },
    { 0xd5f20339, 90 },
    { 0x31c2033a, 95 },
    { 0x3fb20337, 61 },
    { 0xc542033c, 100 },
    { 0xff2033c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68820340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b220344, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x35220347, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2682034c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe320350, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87320355, 95 },
    { 0x36c20355, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2035a, 76 },
    { 0, 0 },
    { 0x95c2035c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94120369, 33 },
    { 0, 0 },
    { 0x9dd2036b, 75 },
    { 0, 0 },
    { 0x38f2036d, 88 },
    { 0xcc82036d, 94 },
    { 0x5c2036f, 100 },
    { 0xc2420370, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e320374, 90 },
    { 0xa2d20375, 98 },
    { 0x8b620376, 43 },
    { 0x2a220377, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5422037b, 61 },
    { 0x5572037c, 55 },
    { 0xda32037d, 76 },
    { 0xcb2037e, 33 },
    { 0xcf2037f, 66 },
    { 0x4432037b, 78 },
    { 0xb3f20381, 65 },
    { 0x34720382, 98 },
    { 0x5f920381, 90 },
    { 0x2bf2037f, 67 },
    { 0, 0 },
    { 0xa4d20386, 34 },
    { 0, 0 },
    { 0x49a20388, 55 },
    { 0x7da20389, 40 },
    { 0, 0 },
    { 0xe5b2038b, 100 },
    { 0, 0 },
    { 0x6c72038d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc20392, 23 },
    { 0x48820393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d20396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2039c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2039f, 17 },
    { 0x9af203a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e203a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03203a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371203ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x465203ae, 90 },
    { 0x4b2203af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7203b7, 61 },
    { 0xdf0203b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d203bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x219203be, 33 },
    { 0x3e3203bf, 88 },
    { 0xab0203c0, 98 },
    { 0x43b203c0, 95 },
    { 0xf43203c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30203c6, 55 },
    { 0xfcc203c7, 95 },
    { 0x1a5203c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d203cc, 61 },
    { 0x575203cd, 17 },
    { 0xe2b203cd, 23 },
    { 0x8f1203cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x188203d2, 33 },
    { 0x687203d3, 61 },
    { 0x229203d4, 61 },
    { 0xcce203d5, 61 },
    { 0x690203d6, 61 },
    { 0x32b203d5, 23 },
    { 0x15203d4, 93 },
    { 0xa8203d9, 98 },
    { 0x1bf203d2, 100 },
    { 0xea0203d2, 76 },
    { 0x4a9203dc, 98 },
    { 0x2d4203d4, 100 },
    { 0x83e203de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b203ed, 17 },
    { 0, 0 },
    { 0x410203ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f203f4, 65 },
    { 0, 0 },
    { 0x887203f6, 68 },
    { 0xd36203f7, 91 },
    { 0xb00203f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b20404, 93 },
    { 0xef120405, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e2040a, 90 },
    { 0x43c2040a, 61 },
    { 0x7d32040c, 61 },
    { 0xd702040c, 55 },
    { 0xf372040c, 98 },
    { 0xf032040f, 68 },
    { 0xfe92040f, 99 },
    { 0x50d20411, 95 },
    { 0xc2b2040a, 90 },
    { 0, 0 },
    { 0x23620414, 55 },
    { 0x5620415, 90 },
    { 0x120416, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f20425, 91 },
    { 0xb1e20426, 76 },
    { 0xb8920427, 75 },
    { 0x3b320428, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d32042c, 90 },
    { 0xfb92042d, 90 },
    { 0x222042d, 55 },
    { 0x422042f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2420432, 61 },
    { 0, 0 },
    { 0xfd520434, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee22043d, 66 },
    { 0x9e92043e, 76 },
    { 0x8d92043f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0320442, 76 },
    { 0, 0 },
    { 0x73020444, 100 },
    { 0xac20445, 99 },
    { 0, 0 },
    { 0x3e20447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d2044b, 66 },
    { 0xc472044b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2122044f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a420454, 90 },
    { 0, 0 },
    { 0xa2d20456, 99 },
    { 0x4ad20456, 78 },
    { 0x5b320458, 90 },
    { 0xa1b20459, 78 },
    { 0x72020459, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a20460, 23 },
    { 0, 0 },
    { 0x5c120462, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93120468, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92046b, 33 },
    { 0x4d82046c, 76 },
    { 0x972046d, 76 },
    { 0, 0 },
    { 0x5602046f, 85 },
    { 0, 0 },
    { 0xc2e20471, 33 },
    { 0x70820472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff820477, 99 },
    { 0xbf920478, 61 },
    { 0x7ba20477, 88 },
    { 0, 0 },
    { 0x7192047b, 61 },
    { 0x2f42047b, 65 },
    { 0x4532047d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa20480, 55 },
    { 0x7d520481, 33 },
    { 0x1bd20482, 98 },
    { 0, 0 },
    { 0xf7720484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c20488, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9482048b, 75 },
    { 0x9912048b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2048f, 17 },
    { 0x88320490, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3420493, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d20496, 52 },
    { 0x86920497, 76 },
    { 0xb3920498, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6102049f, 98 },
    { 0x2fa204a0, 90 },
    { 0, 0 },
    { 0xe16204a2, 61 },
    { 0x446204a3, 76 },
    { 0xd8d204a4, 77 },
    { 0, 0 },
    { 0x107204a6, 61 },
    { 0, 0 },
    { 0x797204a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6204b0, 66 },
    { 0x715204b1, 55 },
    { 0xe09204b2, 61 },
    { 0x706204b2, 98 },
    { 0, 0 },
    { 0xf31204b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b204ba, 84 },
    { 0xbd8204bb, 93 },
    { 0x712204ba, 43 },
    { 0xd9d204bd, 90 },
    { 0x6d2204be, 81 },
    { 0x5b2204bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29204c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e204cb, 33 },
    { 0, 0 },
    { 0x8b6204cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f204d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4204d6, 23 },
    { 0xae9204d7, 98 },
    { 0, 0 },
    { 0x71f204d9, 77 },
    { 0, 0 },
    { 0xb10204db, 33 },
    { 0x5d1204dc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87204e1, 76 },
    { 0x7d4204e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c204e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d204ef, 76 },
    { 0x268204f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x142204f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05204f8, 33 },
    { 0, 0 },
    { 0x640204fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c20501, 78 },
    { 0x68120502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a20505, 85 },
    { 0x92220506, 100 },
    { 0xce820507, 68 },
    { 0x2fa20507, 78 },
    { 0xfd920509, 88 },
    { 0xdae20508, 95 },
    { 0, 0 },
    { 0x4ce2050c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89720513, 33 },
    { 0x5e820514, 17 },
    { 0, 0 },
    { 0x71d20516, 33 },
    { 0, 0 },
    { 0x17a20518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5582051c, 87 },
    { 0x2ee2051c, 23 },
    { 0, 0 },
    { 0x9df2051f, 90 },
    { 0, 0 },
    { 0x31e20521, 33 },
    { 0x33e20522, 98 },
    { 0, 0 },
    { 0x46b20524, 91 },
    { 0, 0 },
    { 0x3d020526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e42052a, 100 },
    { 0, 0 },
    { 0xaa2052c, 43 },
    { 0xc322052d, 98 },
    { 0x3aa2052e, 55 },
    { 0x4c42052f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5220536, 98 },
    { 0x9b820537, 90 },
    { 0x20f20538, 98 },
    { 0, 0 },
    { 0x1042053a, 23 },
    { 0, 0 },
    { 0x5022053c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48520541, 93 },
    { 0x4ed20542, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c920548, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc2054c, 88 },
    { 0x5632054d, 95 },
    { 0x2e82054d, 98 },
    { 0x5622054f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df20554, 60 },
    { 0x36920555, 23 },
    { 0xe7a20556, 77 },
    { 0xa3e20554, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b2055a, 38 },
    { 0x66a2055b, 95 },
    { 0xfa62055c, 46 },
    { 0, 0 },
    { 0x5242055e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64320563, 76 },
    { 0xea720563, 87 },
    { 0x50620565, 78 },
    { 0, 0 },
    { 0x15020567, 61 },
    { 0xed020568, 33 },
    { 0xfc220568, 98 },
    { 0xf3720567, 76 },
    { 0xc0f2056a, 38 },
    { 0, 0 },
    { 0x8e62056d, 34 },
    { 0x91f2056e, 33 },
    { 0x47f2056f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f20572, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f720575, 90 },
    { 0x12220575, 95 },
    { 0xc8320577, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a72057c, 61 },
    { 0x1172057d, 61 },
    { 0, 0 },
    { 0xa842057f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd20584, 55 },
    { 0, 0 },
    { 0x49720586, 17 },
    { 0, 0 },
    { 0x10820588, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf52058c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c2058f, 90 },
    { 0xbcb20590, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b20593, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6520596, 33 },
    { 0x84520596, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd352059e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x654205a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x461205a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd205aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f205af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f205b4, 91 },
    { 0x1a4205b5, 90 },
    { 0xd72205b4, 93 },
    { 0x1bc205b6, 90 },
    { 0x34d205b8, 90 },
    { 0xa59205b9, 37 },
    { 0x5dc205b9, 90 },
    { 0xdf0205bb, 66 },
    { 0xe5d205b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2205bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17205c2, 76 },
    { 0x2f3205c3, 61 },
    { 0x7ac205c3, 90 },
    { 0xd63205c5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321205cc, 99 },
    { 0xfcd205cc, 93 },
    { 0x50c205ce, 76 },
    { 0x2c3205cf, 52 },
    { 0xaf4205cd, 43 },
    { 0x65b205d1, 33 },
    { 0x1b6205cd, 61 },
    { 0x1c1205cc, 93 },
    { 0x622205d4, 76 },
    { 0x352205d5, 17 },
    { 0xa01205d6, 76 },
    { 0x288205d0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x439205da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6205e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90205e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa205e9, 23 },
    { 0xfe9205ea, 33 },
    { 0, 0 },
    { 0x6205ec, 33 },
    { 0, 0 },
    { 0x127205ee, 98 },
    { 0, 0 },
    { 0x3ce205f0, 98 },
    { 0xc6b205f1, 55 },
    { 0x362205f0, 33 },
    { 0xad8205f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd205f9, 33 },
    { 0xad8205fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7205ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48020606, 33 },
    { 0x12920606, 55 },
    { 0x99320608, 76 },
    { 0xef820608, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c2060d, 51 },
    { 0xa842060e, 90 },
    { 0, 0 },
    { 0xceb20610, 33 },
    { 0x90320610, 90 },
    { 0, 0 },
    { 0x24920613, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd20618, 90 },
    { 0, 0 },
    { 0x91c2061a, 100 },
    { 0x84a2061b, 67 },
    { 0x1452061c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87720623, 95 },
    { 0xca320624, 90 },
    { 0xdca20625, 98 },
    { 0x6aa20626, 33 },
    { 0x83c20627, 98 },
    { 0xf8a20627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe522062b, 85 },
    { 0x9a2062b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b420634, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78620637, 95 },
    { 0x94b20638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b20640, 34 },
    { 0, 0 },
    { 0xfc820642, 98 },
    { 0x66020642, 76 },
    { 0x94d20643, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f420648, 33 },
    { 0x40820649, 23 },
    { 0xdb920649, 34 },
    { 0x9b32064b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa20651, 76 },
    { 0x83e20652, 85 },
    { 0x75720653, 61 },
    { 0x43720652, 87 },
    { 0x7c620652, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e320658, 43 },
    { 0, 0 },
    { 0x7f62065a, 90 },
    { 0, 0 },
    { 0x2772065c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde20660, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49520665, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38620668, 61 },
    { 0, 0 },
    { 0xb772066a, 23 },
    { 0x6cd2066b, 61 },
    { 0, 0 },
    { 0x9d82066d, 90 },
    { 0x72d2066d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd520671, 34 },
    { 0x90b20671, 23 },
    { 0xf7e20673, 61 },
    { 0, 0 },
    { 0x9fd20675, 61 },
    { 0xa9d20676, 34 },
    { 0, 0 },
    { 0x86a20678, 78 },
    { 0xe9220679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1452067e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5120681, 98 },
    { 0, 0 },
    { 0xbf420683, 98 },
    { 0x7b020683, 76 },
    { 0x2ca20683, 55 },
    { 0x67420686, 98 },
    { 0xb8020685, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9312068a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142068f, 90 },
    { 0, 0 },
    { 0x63c20691, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a220694, 90 },
    { 0x8e320695, 34 },
    { 0x47020696, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2069b, 90 },
    { 0, 0 },
    { 0xed12069d, 61 },
    { 0, 0 },
    { 0xc7d2069f, 84 },
    { 0, 0 },
    { 0x790206a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a206a5, 33 },
    { 0x3a0206a5, 90 },
    { 0, 0 },
    { 0xa57206a8, 98 },
    { 0xb24206a9, 53 },
    { 0xd50206aa, 34 },
    { 0xb17206aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859206b0, 93 },
    { 0x802206b0, 98 },
    { 0xe4e206b0, 95 },
    { 0xc0d206b1, 90 },
    { 0x221206b4, 98 },
    { 0x1da206b5, 23 },
    { 0x8d3206b4, 77 },
    { 0xf05206b4, 43 },
    { 0, 0 },
    { 0xd2f206b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a206bd, 93 },
    { 0, 0 },
    { 0x844206bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2206c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45206c6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03206cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9206d2, 98 },
    { 0x366206d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4206da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde206de, 100 },
    { 0x9a3206df, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300206e3, 43 },
    { 0xdf9206e4, 98 },
    { 0x10206e4, 33 },
    { 0x37a206e4, 61 },
    { 0x68e206e3, 43 },
    { 0, 0 },
    { 0x98b206e9, 100 },
    { 0xc33206ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198206ef, 67 },
    { 0, 0 },
    { 0xd37206f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614206f8, 87 },
    { 0xe1f206f8, 90 },
    { 0x5f6206f9, 93 },
    { 0x1ff206fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5206ff, 81 },
    { 0xd8d20700, 93 },
    { 0x6ba206ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692070a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6642070d, 23 },
    { 0xa102070d, 23 },
    { 0x232070e, 90 },
    { 0, 0 },
    { 0x66520711, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e20717, 43 },
    { 0xea20717, 61 },
    { 0x34620719, 98 },
    { 0, 0 },
    { 0x4092071b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37320723, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12920727, 80 },
    { 0xcc320728, 81 },
    { 0x9d720727, 23 },
    { 0x49a2072a, 61 },
    { 0x4f92072b, 85 },
    { 0xee82072a, 98 },
    { 0x6e12072d, 46 },
    { 0x7082072e, 77 },
    { 0xaf02072d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef820735, 33 },
    { 0, 0 },
    { 0xe2020737, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca42073b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc732073e, 66 },
    { 0, 0 },
    { 0x1d920740, 77 },
    { 0, 0 },
    { 0x79020742, 55 },
    { 0x20a20743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80420747, 93 },
    { 0, 0 },
    { 0xbae20749, 93 },
    { 0x90720749, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2074d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed520754, 33 },
    { 0x96720754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd2075c, 55 },
    { 0x90e2075d, 90 },
    { 0x2a02075e, 23 },
    { 0x9e82075f, 33 },
    { 0xe3a2075d, 33 },
    { 0x4892075c, 98 },
    { 0, 0 },
    { 0xb2820763, 34 },
    { 0xf1e20764, 98 },
    { 0, 0 },
    { 0xd8320766, 98 },
    { 0x98c20766, 34 },
    { 0, 0 },
    { 0x39920769, 46 },
    { 0x32020769, 64 },
    { 0, 0 },
    { 0x8f2076c, 98 },
    { 0x1e52076d, 76 },
    { 0x4532076e, 34 },
    { 0x3652076e, 46 },
    { 0x8b22076d, 61 },
    { 0x1d220771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd220775, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd602077c, 66 },
    { 0xade2077d, 90 },
    { 0xfdc2077e, 85 },
    { 0x8bd2077f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7920782, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a20785, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda52078c, 95 },
    { 0x6142078d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0520796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b2079a, 59 },
    { 0x9092079b, 98 },
    { 0x46f2079a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf462079f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b207a5, 33 },
    { 0, 0 },
    { 0x427207a7, 90 },
    { 0x4cd207a8, 99 },
    { 0xebf207a9, 98 },
    { 0x4ab207a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x575207ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b207b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203207bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f207bf, 90 },
    { 0xfb8207bf, 61 },
    { 0x8d0207c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82207c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8207c7, 78 },
    { 0, 0 },
    { 0x914207c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645207d0, 98 },
    { 0x39a207d0, 33 },
    { 0x48e207d1, 98 },
    { 0xd18207d2, 93 },
    { 0, 0 },
    { 0xf54207d5, 98 },
    { 0x9f207d5, 76 },
    { 0xe0a207d7, 90 },
    { 0x5e4207d7, 85 },
    { 0xfaa207d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25207dd, 61 },
    { 0, 0 },
    { 0xbcb207df, 76 },
    { 0x136207e0, 33 },
    { 0, 0 },
    { 0x956207e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb207e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd207eb, 23 },
    { 0xc05207ec, 76 },
    { 0x7b6207eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce207f2, 68 },
    { 0xf40207f2, 46 },
    { 0x98f207f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e207fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512207fe, 49 },
    { 0xca1207fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x46220802, 95 },
    { 0xcb920803, 33 },
    { 0xd6820802, 75 },
    { 0x2df20805, 43 },
    { 0, 0 },
    { 0x52e20807, 61 },
    { 0x89220808, 98 },
    { 0, 0 },
    { 0xadc2080a, 2 },
    { 0xe482080b, 33 },
    { 0xbc92080c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d62080f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e20813, 100 },
    { 0xe3d20814, 87 },
    { 0x6ef20815, 90 },
    { 0x57520814, 61 },
    { 0x4d120816, 90 },
    { 0x12920818, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf72081c, 23 },
    { 0, 0 },
    { 0xcae2081e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6220826, 46 },
    { 0x79f20826, 90 },
    { 0, 0 },
    { 0xb420829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf262082f, 95 },
    { 0, 0 },
    { 0xc5d20831, 93 },
    { 0xeee20832, 23 },
    { 0x2020833, 90 },
    { 0x77d20831, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b20837, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d02083f, 76 },
    { 0x89020840, 90 },
    { 0xea120841, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66820845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e20849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e2084d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b220850, 33 },
    { 0x6b520851, 52 },
    { 0, 0 },
    { 0xe3120853, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de20858, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d20862, 61 },
    { 0x78520863, 61 },
    { 0x8cb20864, 43 },
    { 0x3c420862, 2 },
    { 0, 0 },
    { 0x28220867, 95 },
    { 0xcfd20868, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2086f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d20873, 75 },
    { 0x6de20873, 99 },
    { 0x41020874, 87 },
    { 0x1a420873, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf72087d, 23 },
    { 0, 0 },
    { 0x2c22087f, 90 },
    { 0x1242087f, 33 },
    { 0x7a920881, 81 },
    { 0xf6220882, 90 },
    { 0x94b20883, 95 },
    { 0x4020882, 95 },
    { 0xa7120882, 33 },
    { 0xc4820886, 34 },
    { 0x77820887, 61 },
    { 0x74f20888, 65 },
    { 0x1cf20887, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe142088c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e20893, 76 },
    { 0x98d20894, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120898, 55 },
    { 0, 0 },
    { 0xaa82089a, 95 },
    { 0, 0 },
    { 0x9342089c, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a2089f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c208a2, 52 },
    { 0x4fe208a3, 90 },
    { 0x96b208a4, 34 },
    { 0x61208a5, 77 },
    { 0x3f0208a5, 67 },
    { 0xde208a7, 33 },
    { 0x89a208a8, 61 },
    { 0xc14208a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38208af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48208b6, 34 },
    { 0, 0 },
    { 0x267208b8, 61 },
    { 0x482208b9, 55 },
    { 0, 0 },
    { 0x6bb208bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16208bf, 94 },
    { 0, 0 },
    { 0x7e8208c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86208c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb208ca, 100 },
    { 0xabc208ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373208d1, 52 },
    { 0x4ed208d1, 55 },
    { 0xcb2208d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda208d7, 77 },
    { 0, 0 },
    { 0x474208d9, 46 },
    { 0, 0 },
    { 0xfb7208db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c208df, 33 },
    { 0, 0 },
    { 0x8d7208e1, 91 },
    { 0xbe7208e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1208e6, 23 },
    { 0xdcb208e7, 67 },
    { 0x717208e8, 61 },
    { 0xd29208e9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f208ec, 81 },
    { 0, 0 },
    { 0xd1a208ee, 34 },
    { 0, 0 },
    { 0x677208f0, 95 },
    { 0x513208f1, 95 },
    { 0, 0 },
    { 0xb1b208f3, 88 },
    { 0x232208f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x517208f7, 95 },
    { 0, 0 },
    { 0xa74208f9, 33 },
    { 0xac0208fa, 76 },
    { 0xd94208fb, 90 },
    { 0, 0 },
    { 0x9f5208fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d20905, 84 },
    { 0x99020905, 100 },
    { 0xde620906, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d92090a, 48 },
    { 0, 0 },
    { 0xcb22090c, 66 },
    { 0, 0 },
    { 0x5862090e, 98 },
    { 0, 0 },
    { 0x75d20910, 88 },
    { 0xdd320911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7620919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c20922, 43 },
    { 0x4e620923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39020929, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd20930, 66 },
    { 0x6d520931, 90 },
    { 0x6db20931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9420935, 34 },
    { 0xc9520936, 91 },
    { 0x4ed20936, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9612093d, 23 },
    { 0xd492093e, 98 },
    { 0xfa42093f, 80 },
    { 0, 0 },
    { 0x11a20941, 90 },
    { 0xc7d20941, 99 },
    { 0, 0 },
    { 0xc1720944, 80 },
    { 0xcc820945, 23 },
    { 0x55720945, 93 },
    { 0x3f520946, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0620950, 88 },
    { 0x79a20950, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc20955, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1720959, 92 },
    { 0xc032095a, 77 },
    { 0x6242095b, 36 },
    { 0x2002095c, 90 },
    { 0x9b32095d, 34 },
    { 0x6a220959, 90 },
    { 0x2c42095f, 52 },
    { 0x342095f, 80 },
    { 0, 0 },
    { 0xa6320962, 90 },
    { 0xae020963, 95 },
    { 0x7e820964, 99 },
    { 0, 0 },
    { 0x2d320966, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8620969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2096e, 81 },
    { 0, 0 },
    { 0x5ef20970, 81 },
    { 0x72020970, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b20977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd462097b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc692097e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b520982, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a20987, 52 },
    { 0xc9420987, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2098e, 95 },
    { 0, 0 },
    { 0x41020990, 90 },
    { 0, 0 },
    { 0x8e220992, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52320998, 34 },
    { 0, 0 },
    { 0x3b22099a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef209a0, 91 },
    { 0xd54209a1, 90 },
    { 0xadc209a1, 61 },
    { 0xc6a209a3, 76 },
    { 0x268209a4, 43 },
    { 0x19209a5, 99 },
    { 0xcdc209a5, 66 },
    { 0xd08209a7, 23 },
    { 0x18f209a7, 52 },
    { 0xc32209a9, 67 },
    { 0xc20209a2, 47 },
    { 0x970209a5, 90 },
    { 0, 0 },
    { 0xe8e209ad, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7209b0, 61 },
    { 0x44f209b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90209b5, 91 },
    { 0xe2e209b5, 61 },
    { 0x7d5209b7, 33 },
    { 0xb6209b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96209bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d209c3, 23 },
    { 0x6e0209c3, 55 },
    { 0x21f209c3, 33 },
    { 0xfbe209c6, 67 },
    { 0x37e209c3, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2209cc, 100 },
    { 0xba4209cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280209d1, 55 },
    { 0xdab209d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c209d6, 33 },
    { 0xbfd209d6, 33 },
    { 0, 0 },
    { 0x22c209d9, 95 },
    { 0x2c6209d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x933209dd, 98 },
    { 0xb0b209de, 98 },
    { 0x454209de, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e209ea, 61 },
    { 0x6a5209eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9209ee, 95 },
    { 0xbb1209ef, 61 },
    { 0xad3209f0, 33 },
    { 0xb13209f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8209f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf20a01, 90 },
    { 0x7fa20a01, 90 },
    { 0x3aa20a03, 46 },
    { 0x68220a04, 90 },
    { 0, 0 },
    { 0xd0d20a06, 90 },
    { 0x1e320a07, 95 },
    { 0xdd20a08, 66 },
    { 0x38520a06, 61 },
    { 0x71d20a06, 95 },
    { 0xb20a0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb420a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c620a12, 61 },
    { 0x8ea20a13, 43 },
    { 0x53820a14, 90 },
    { 0xea420a15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf020a18, 76 },
    { 0xd8820a19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c20a1e, 61 },
    { 0x57020a1e, 61 },
    { 0xaa820a20, 33 },
    { 0x9420a1e, 90 },
    { 0x7fa20a22, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x43520a25, 33 },
    { 0x64220a26, 95 },
    { 0, 0 },
    { 0x63e20a28, 76 },
    { 0x89720a29, 61 },
    { 0, 0 },
    { 0x9e820a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb120a30, 93 },
    { 0, 0 },
    { 0x8b20a32, 87 },
    { 0xd7f20a32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6120a36, 84 },
    { 0x17020a37, 95 },
    { 0x7e920a38, 95 },
    { 0x4e820a39, 61 },
    { 0x7bc20a3a, 66 },
    { 0x8ea20a3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51020a3f, 90 },
    { 0x70720a3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa420a44, 98 },
    { 0x5b120a45, 95 },
    { 0x5ed20a44, 46 },
    { 0x3bf20a47, 61 },
    { 0, 0 },
    { 0xed520a49, 76 },
    { 0xaa220a4a, 98 },
    { 0, 0 },
    { 0x82c20a4c, 55 },
    { 0x13220a4d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49020a55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94020a5c, 36 },
    { 0x4bb20a5d, 23 },
    { 0xb8f20a5d, 33 },
    { 0x56420a5f, 90 },
    { 0xdda20a60, 75 },
    { 0, 0 },
    { 0xcf220a62, 90 },
    { 0x28520a62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71320a66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27320a6a, 33 },
    { 0x4c620a6b, 95 },
    { 0x3db20a6a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c20a70, 90 },
    { 0xe5220a70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18920a75, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c20a7a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd920a7e, 98 },
    { 0x60b20a7f, 55 },
    { 0x37520a80, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a20a87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd20a90, 55 },
    { 0x50620a91, 91 },
    { 0xb2820a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0920a95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd20a9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20a9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20220aa2, 85 },
    { 0xec020aa2, 76 },
    { 0, 0 },
    { 0x22c20aa5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36820aa8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d20aae, 44 },
    { 0x5a420aaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x84720ab2, 77 },
    { 0xd2320ab3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e20ab9, 100 },
    { 0, 0 },
    { 0x40220abb, 33 },
    { 0xa0a20abc, 34 },
    { 0x15720abb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b20ac0, 33 },
    { 0, 0 },
    { 0x45920ac2, 95 },
    { 0, 0 },
    { 0xec020ac4, 23 },
    { 0x93220ac5, 95 },
    { 0, 0 },
    { 0x23420ac7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f020acb, 95 },
    { 0x3ca20acc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d20ad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f20ad9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c20add, 90 },
    { 0x4bf20add, 91 },
    { 0, 0 },
    { 0x31620ae0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82320aeb, 34 },
    { 0xab220aec, 23 },
    { 0x48d20aed, 34 },
    { 0xec920aee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c220af6, 22 },
    { 0, 0 },
    { 0xb5320af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78120b06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e20b0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24320b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91920b19, 97 },
    { 0xed20b1a, 55 },
    { 0x30820b19, 95 },
    { 0xad220b1c, 87 },
    { 0xb7820b1d, 98 },
    { 0x1b320b1e, 46 },
    { 0x2f020b1e, 100 },
    { 0xf9920b20, 23 },
    { 0, 0 },
    { 0x72120b22, 76 },
    { 0, 0 },
    { 0xb3f20b24, 43 },
    { 0, 0 },
    { 0xf0f20b26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6420b29, 98 },
    { 0x14420b2a, 95 },
    { 0x29520b2a, 33 },
    { 0, 0 },
    { 0xe0020b2d, 23 },
    { 0, 0 },
    { 0x1a120b2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd20b32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53120b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e420b39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd120b3f, 101 },
    { 0, 0 },
    { 0x92d20b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc920b44, 77 },
    { 0x4c20b44, 43 },
    { 0x2cb20b45, 33 },
    { 0xb7720b45, 61 },
    { 0x43e20b48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6820b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66020b54, 55 },
    { 0x7720b55, 55 },
    { 0, 0 },
    { 0xbf520b57, 95 },
    { 0x1e920b58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c20b5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88620b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4720b64, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58120b68, 78 },
    { 0x87020b69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d20b6e, 33 },
    { 0x2da20b6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf720b73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e820b76, 99 },
    { 0, 0 },
    { 0xe9c20b78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b920b7c, 98 },
    { 0x4b520b7d, 91 },
    { 0xda220b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5620b82, 90 },
    { 0xb0620b82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53620b8f, 90 },
    { 0x77b20b90, 61 },
    { 0x49320b90, 34 },
    { 0, 0 },
    { 0x15a20b93, 77 },
    { 0, 0 },
    { 0x1720b95, 93 },
    { 0x1720b96, 93 },
    { 0x90320b95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15220b9d, 90 },
    { 0x3d320b9d, 11 },
    { 0x8d420b9f, 77 },
    { 0x1c20b9d, 61 },
    { 0x7cb20ba1, 55 },
    { 0x68520ba2, 33 },
    { 0xefd20ba1, 98 },
    { 0x9b20ba2, 97 },
    { 0xc9520ba5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66320ba9, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91520bb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e20bba, 46 },
    { 0xa5a20bba, 33 },
    { 0xf5c20bbc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xda020bbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3120bc3, 91 },
    { 0xeea20bc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2820bcb, 90 },
    { 0x98e20bcc, 61 },
    { 0xf4520bcc, 66 },
    { 0, 0 },
    { 0x77120bcf, 95 },
    { 0x2ae20bd0, 90 },
    { 0, 0 },
    { 0x30e20bd2, 98 },
    { 0x9a220bd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5020bde, 61 },
    { 0x4b820bde, 85 },
    { 0, 0 },
    { 0x1ba20be1, 99 },
    { 0, 0 },
    { 0x10220be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee20be9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d20bed, 33 },
    { 0, 0 },
    { 0xff220bef, 61 },
    { 0x1f220bf0, 34 },
    { 0x85e20bf0, 78 },
    { 0x75220bf1, 61 },
    { 0x7c120bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f320bf8, 76 },
    { 0xd3720bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9220bfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac20c00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c20c03, 87 },
    { 0, 0 },
    { 0x86d20c05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9220c09, 23 },
    { 0x52f20c0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b20c0e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b20c14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c20c1c, 81 },
    { 0x3af20c1d, 34 },
    { 0xc1820c1c, 93 },
    { 0xfe820c1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9020c24, 99 },
    { 0xd7520c24, 34 },
    { 0x63220c25, 68 },
    { 0, 0 },
    { 0x92120c28, 93 },
    { 0x88620c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c820c2c, 61 },
    { 0xcd720c2d, 95 },
    { 0xb820c2c, 95 },
    { 0, 0 },
    { 0x56b20c30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f20c33, 81 },
    { 0xf3920c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc20c37, 84 },
    { 0x2d220c37, 33 },
    { 0xad820c37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b20c3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf120c43, 78 },
    { 0xdaf20c44, 90 },
    { 0x48120c44, 61 },
    { 0x16b20c44, 90 },
    { 0x47720c47, 33 },
    { 0x64d20c48, 61 },
    { 0xa1620c48, 33 },
    { 0x34720c44, 95 },
    { 0x93d20c4b, 33 },
    { 0xfc320c48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae20c55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24120c5b, 95 },
    { 0xceb20c5c, 95 },
    { 0x67720c5d, 84 },
    { 0x33220c5e, 98 },
    { 0x13a20c5f, 61 },
    { 0xcd720c60, 98 },
    { 0x36420c5b, 61 },
    { 0x53820c62, 91 },
    { 0x34020c63, 84 },
    { 0x1c620c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a20c67, 61 },
    { 0x1cf20c68, 93 },
    { 0xfc20c67, 33 },
    { 0x2c220c68, 98 },
    { 0xe2920c6b, 23 },
    { 0x7c920c6b, 95 },
    { 0x15820c6b, 98 },
    { 0x95c20c67, 76 },
    { 0x9ab20c6f, 33 },
    { 0xc3a20c70, 91 },
    { 0x76b20c70, 53 },
    { 0x46420c6f, 99 },
    { 0x81c20c73, 55 },
    { 0x48020c67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39420c77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2220c7b, 33 },
    { 0x2f420c7c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc220c83, 61 },
    { 0xfac20c84, 90 },
    { 0xb1320c85, 76 },
    { 0xde620c83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba20c89, 33 },
    { 0xa0020c89, 34 },
    { 0x7a620c8b, 23 },
    { 0x58f20c8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d20c92, 90 },
    { 0x4ec20c93, 61 },
    { 0x6ab20c94, 17 },
    { 0xb2a20c94, 34 },
    { 0x50920c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea20c99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5020ca1, 34 },
    { 0, 0 },
    { 0xbb020ca3, 56 },
    { 0xc1920ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de20ca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87020cac, 95 },
    { 0, 0 },
    { 0xa5420cae, 100 },
    { 0x35820caf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa20cb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a20cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f20cbe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d220cc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4820ccb, 98 },
    { 0xb1120ccc, 33 },
    { 0x8a220ccd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0620cd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac20cd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab20cdc, 33 },
    { 0x3f620cdc, 66 },
    { 0xc5d20cdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65120ce5, 98 },
    { 0x56720ce5, 90 },
    { 0x6520ce7, 91 },
    { 0, 0 },
    { 0x3f020ce9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1620cee, 23 },
    { 0, 0 },
    { 0xd6620cf0, 90 },
    { 0x88120cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc20cf6, 87 },
    { 0x2c320cf7, 34 },
    { 0xea020cf8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23420cfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27120d00, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c20d04, 76 },
    { 0x1b020d04, 90 },
    { 0x8a920d06, 46 },
    { 0, 0 },
    { 0xbe020d08, 40 },
    { 0x8de20d08, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f20d10, 90 },
    { 0xbfb20d11, 95 },
    { 0x6ae20d12, 75 },
    { 0x6c120d13, 43 },
    { 0xf3220d14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1920d19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7720d1c, 98 },
    { 0x89920d1d, 52 },
    { 0xa8120d1e, 90 },
    { 0x89020d1c, 65 },
    { 0xc2f20d1c, 95 },
    { 0, 0 },
    { 0xe7e20d22, 33 },
    { 0x47720d23, 66 },
    { 0x1e420d24, 33 },
    { 0x48020d23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee520d2a, 95 },
    { 0, 0 },
    { 0x71020d2c, 90 },
    { 0xfdb20d2d, 55 },
    { 0, 0 },
    { 0xf9c20d2f, 76 },
    { 0x85c20d30, 76 },
    { 0x70c20d31, 34 },
    { 0, 0 },
    { 0x43720d33, 98 },
    { 0, 0 },
    { 0xb0720d35, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa320d3b, 46 },
    { 0, 0 },
    { 0x77520d3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab20d49, 43 },
    { 0, 0 },
    { 0x6f020d4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6020d50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c20d53, 61 },
    { 0xe5c20d53, 61 },
    { 0x35220d55, 95 },
    { 0xf8820d55, 23 },
    { 0xf4120d57, 47 },
    { 0xbd420d54, 100 },
    { 0x8ed20d55, 90 },
    { 0x2a320d57, 33 },
    { 0, 0 },
    { 0x6a620d5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e520d5f, 66 },
    { 0xb120d60, 95 },
    { 0, 0 },
    { 0x9f920d62, 23 },
    { 0xd2e20d62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa20d69, 99 },
    { 0xc420d69, 46 },
    { 0, 0 },
    { 0x40b20d6c, 97 },
    { 0x34b20d6d, 61 },
    { 0xde920d6e, 77 },
    { 0, 0 },
    { 0xf6d20d70, 53 },
    { 0xa2620d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b20d76, 52 },
    { 0, 0 },
    { 0x51d20d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58820d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a20d83, 33 },
    { 0xd5720d84, 55 },
    { 0x90320d85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb520d8a, 43 },
    { 0, 0 },
    { 0xc7720d8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f20d8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51220d94, 33 },
    { 0xa20d94, 98 },
    { 0x3d120d94, 52 },
    { 0x6c720d97, 25 },
    { 0x53420d98, 34 },
    { 0x97120d95, 40 },
    { 0, 0 },
    { 0xca520d9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa20d9e, 23 },
    { 0x38f20d9f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b320da2, 93 },
    { 0xd9220da3, 90 },
    { 0x94220da2, 76 },
    { 0xfd120da2, 91 },
    { 0, 0 },
    { 0x57b20da7, 95 },
    { 0xca020da8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x20120dab, 46 },
    { 0x89b20dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a20db1, 90 },
    { 0xe1e20db1, 61 },
    { 0xffe20db3, 55 },
    { 0, 0 },
    { 0x7520db5, 32 },
    { 0x9f620db6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb20db9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c820dc0, 99 },
    { 0, 0 },
    { 0x9d420dc2, 88 },
    { 0xaec20dc2, 95 },
    { 0xed620dc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c720dd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea20dd4, 61 },
    { 0xd4420dd4, 76 },
    { 0xd8b20dd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d20ddc, 94 },
    { 0xf4120ddd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e20de0, 98 },
    { 0xe5b20de1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b20de4, 84 },
    { 0, 0 },
    { 0x7e520de6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d20dea, 33 },
    { 0x69120deb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e20def, 80 },
    { 0, 0 },
    { 0x1da20df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53820df5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc20df8, 33 },
    { 0, 0 },
    { 0xa0e20dfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f20e03, 99 },
    { 0, 0 },
    { 0xe0120e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6920e0f, 33 },
    { 0x3ab20e10, 95 },
    { 0xfb620e10, 90 },
    { 0xed620e12, 61 },
    { 0x60320e13, 46 },
    { 0, 0 },
    { 0x52e20e15, 55 },
    { 0x89320e16, 76 },
    { 0xd0620e17, 90 },
    { 0x19e20e17, 85 },
    { 0, 0 },
    { 0x21f20e1a, 76 },
    { 0xb4620e1b, 77 },
    { 0, 0 },
    { 0x8ac20e1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a20e21, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e820e25, 61 },
    { 0x48720e26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3620e34, 78 },
    { 0x1f420e35, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a20e3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3920e43, 43 },
    { 0x53f20e44, 85 },
    { 0x80820e45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b20e48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff020e4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae20e55, 43 },
    { 0x84520e56, 52 },
    { 0x43420e56, 98 },
    { 0x72820e56, 96 },
    { 0x1a20e59, 43 },
    { 0xbe920e57, 66 },
    { 0, 0 },
    { 0x30320e5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e20e61, 78 },
    { 0, 0 },
    { 0xbb320e63, 85 },
    { 0, 0 },
    { 0x6c520e65, 89 },
    { 0, 0 },
    { 0x96320e67, 90 },
    { 0, 0 },
    { 0x48220e69, 98 },
    { 0, 0 },
    { 0xe220e6b, 95 },
    { 0, 0 },
    { 0x88d20e6d, 33 },
    { 0, 0 },
    { 0xa1420e6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54220e72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c20e81, 77 },
    { 0x18f20e82, 84 },
    { 0x6db20e83, 33 },
    { 0xacd20e82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39620e88, 76 },
    { 0, 0 },
    { 0x80120e8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f20e8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7320e93, 61 },
    { 0xae120e93, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0120e98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16220ea4, 52 },
    { 0x23920ea5, 33 },
    { 0, 0 },
    { 0x7c920ea7, 91 },
    { 0x52320ea7, 93 },
    { 0x23720ea9, 33 },
    { 0xfe20ea7, 23 },
    { 0x4a820eab, 48 },
    { 0xb5720eac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c20eb4, 90 },
    { 0, 0 },
    { 0x4eb20eb6, 33 },
    { 0, 0 },
    { 0x33e20eb8, 95 },
    { 0, 0 },
    { 0x92f20eba, 23 },
    { 0x45d20eba, 23 },
    { 0x70e20ebc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0420ec4, 90 },
    { 0x5f420ec5, 95 },
    { 0xadd20ec5, 2 },
    { 0x74820ec6, 34 },
    { 0x62620ec8, 84 },
    { 0x22520ec5, 90 },
    { 0x62120eca, 61 },
    { 0x77a20ec6, 61 },
    { 0xbc220ec5, 61 },
    { 0x26120ec6, 90 },
    { 0, 0 },
    { 0x86220ecf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x74320ed2, 98 },
    { 0x9a520ed2, 55 },
    { 0xc4320ed3, 88 },
    { 0xc7a20ed4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60520edb, 61 },
    { 0, 0 },
    { 0xe5920edd, 23 },
    { 0xb1d20ede, 61 },
    { 0xf4320ede, 34 },
    { 0x27c20ede, 84 },
    { 0x6420ee0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d20ee4, 98 },
    { 0, 0 },
    { 0x72720ee6, 99 },
    { 0, 0 },
    { 0x68520ee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa20eeb, 90 },
    { 0, 0 },
    { 0x84e20eed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8820ef6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d920efa, 76 },
    { 0xc5820efb, 17 },
    { 0xaa420efc, 46 },
    { 0x4bf20efd, 98 },
    { 0x76c20efe, 80 },
    { 0x54420efe, 98 },
    { 0, 0 },
    { 0x62220f01, 61 },
    { 0, 0 },
    { 0x1fd20f03, 23 },
    { 0x26b20f03, 55 },
    { 0xe5720f05, 90 },
    { 0, 0 },
    { 0x82420f07, 68 },
    { 0x67220f08, 43 },
    { 0x6e420f08, 90 },
    { 0, 0 },
    { 0x81f20f0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e20f11, 77 },
    { 0xd9820f12, 33 },
    { 0x9be20f13, 78 },
    { 0, 0 },
    { 0xcfb20f15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e20f19, 93 },
    { 0, 0 },
    { 0x5a920f1b, 95 },
    { 0x6f220f1c, 46 },
    { 0, 0 },
    { 0xa1120f1e, 95 },
    { 0, 0 },
    { 0xba820f20, 93 },
    { 0xb6820f21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a20f25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da20f2e, 90 },
    { 0xfc120f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e20f33, 60 },
    { 0x8b820f34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d420f37, 90 },
    { 0x91b20f38, 39 },
    { 0x29820f39, 98 },
    { 0xc9720f38, 55 },
    { 0, 0 },
    { 0xc0c20f3c, 46 },
    { 0xca20f3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a20f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e20f44, 40 },
    { 0xd9b20f44, 55 },
    { 0x62220f46, 37 },
    { 0x49120f47, 91 },
    { 0xe5f20f48, 33 },
    { 0, 0 },
    { 0x7520f4a, 34 },
    { 0xeb720f4b, 78 },
    { 0xe4b20f4b, 61 },
    { 0x40f20f4a, 95 },
    { 0xbf120f4e, 93 },
    { 0xbd220f4f, 55 },
    { 0x44220f50, 99 },
    { 0xb9920f4e, 23 },
    { 0x7d120f51, 61 },
    { 0xe5f20f53, 33 },
    { 0x76220f53, 34 },
    { 0xdbb20f4f, 91 },
    { 0xaea20f52, 33 },
    { 0xab220f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b20f5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a720f5d, 90 },
    { 0xf820f5e, 82 },
    { 0x5e520f5f, 33 },
    { 0xa720f60, 61 },
    { 0x6bd20f5f, 33 },
    { 0xe5420f5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84020f65, 26 },
    { 0x3a620f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8620f76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4820f7b, 66 },
    { 0x8be20f7c, 100 },
    { 0x2a420f7c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d20f81, 34 },
    { 0, 0 },
    { 0x1df20f83, 23 },
    { 0, 0 },
    { 0x5a20f85, 69 },
    { 0x3a720f85, 75 },
    { 0xd3420f87, 33 },
    { 0xfea20f88, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd20f8b, 84 },
    { 0, 0 },
    { 0xa2420f8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c20f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81820f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d420f9d, 65 },
    { 0, 0 },
    { 0x70020f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff920fa6, 100 },
    { 0x81c20fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15320fab, 55 },
    { 0x71c20fac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae820faf, 90 },
    { 0xfd420fb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82520fb4, 23 },
    { 0, 0 },
    { 0x47520fb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b20fbc, 98 },
    { 0, 0 },
    { 0x48c20fbe, 95 },
    { 0, 0 },
    { 0x47e20fc0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7720fc3, 52 },
    { 0x6f320fc3, 61 },
    { 0x95b20fc3, 98 },
    { 0x25620fc4, 65 },
    { 0xc3b20fc5, 98 },
    { 0x8b520fc8, 93 },
    { 0x2dd20fc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6620fd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec020fdc, 95 },
    { 0xce220fdd, 81 },
    { 0, 0 },
    { 0x7db20fdf, 98 },
    { 0x1bd20fe0, 55 },
    { 0x12520fdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e120fe6, 76 },
    { 0, 0 },
    { 0xe8c20fe8, 85 },
    { 0xbad20fe9, 90 },
    { 0x8d620fea, 93 },
    { 0x20020feb, 90 },
    { 0xfd420fea, 98 },
    { 0, 0 },
    { 0xbe920fee, 43 },
    { 0, 0 },
    { 0x43a20ff0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb20ff3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb20ff7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5021000, 46 },
    { 0x7b121000, 90 },
    { 0x2dd21002, 90 },
    { 0x69521003, 61 },
    { 0xd7721001, 34 },
    { 0x6fc21005, 98 },
    { 0x94021006, 33 },
    { 0x6b921007, 23 },
    { 0xa3821007, 95 },
    { 0xd0421008, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae02100c, 23 },
    { 0x3ff2100d, 76 },
    { 0x5962100d, 93 },
    { 0x48b2100e, 33 },
    { 0, 0 },
    { 0x8f521011, 98 },
    { 0x1b321011, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e121015, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e21019, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2101d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d621023, 76 },
    { 0xf7c21024, 96 },
    { 0x80b21025, 33 },
    { 0x28521026, 98 },
    { 0x8e421027, 100 },
    { 0x85621028, 81 },
    { 0x8d721024, 61 },
    { 0, 0 },
    { 0xbbc2102b, 90 },
    { 0, 0 },
    { 0x7882102d, 91 },
    { 0x2232102e, 33 },
    { 0x4d12102f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee21036, 69 },
    { 0xe1021036, 43 },
    { 0xe121038, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b21042, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3921049, 33 },
    { 0, 0 },
    { 0x76b2104b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b2104e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf621051, 33 },
    { 0xd0921052, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e21057, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8152105f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1421065, 61 },
    { 0x62821066, 98 },
    { 0, 0 },
    { 0xe7721068, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1892106b, 33 },
    { 0, 0 },
    { 0x6582106d, 100 },
    { 0xef42106e, 33 },
    { 0xcf72106f, 90 },
    { 0x22121070, 95 },
    { 0, 0 },
    { 0x90221072, 69 },
    { 0, 0 },
    { 0x27921074, 61 },
    { 0, 0 },
    { 0x54121076, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac2107d, 76 },
    { 0, 0 },
    { 0xda22107f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90821082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d21085, 88 },
    { 0xbed21085, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32108e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57821094, 33 },
    { 0xef721095, 90 },
    { 0, 0 },
    { 0xd7621097, 90 },
    { 0, 0 },
    { 0x9aa21099, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd2109e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x541210a1, 93 },
    { 0xef2210a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2210a5, 33 },
    { 0, 0 },
    { 0x49c210a7, 91 },
    { 0xca5210a7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961210b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6210b7, 61 },
    { 0x57e210b7, 76 },
    { 0xccb210b7, 46 },
    { 0, 0 },
    { 0x257210bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60210c0, 52 },
    { 0x873210c0, 95 },
    { 0xfc7210c1, 90 },
    { 0xd38210c1, 61 },
    { 0x637210c1, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30210c8, 91 },
    { 0x570210c9, 90 },
    { 0, 0 },
    { 0x88a210cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb210d3, 93 },
    { 0xfe7210d3, 61 },
    { 0xac7210d5, 100 },
    { 0, 0 },
    { 0x66210d7, 76 },
    { 0xa5b210d8, 98 },
    { 0x524210d7, 99 },
    { 0x7f9210da, 52 },
    { 0xb74210da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86210e1, 76 },
    { 0x1db210e1, 43 },
    { 0x101210e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7210e7, 95 },
    { 0xd210e8, 98 },
    { 0xa62210e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8210ed, 34 },
    { 0, 0 },
    { 0x9a210ef, 69 },
    { 0xe67210ef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66210f7, 98 },
    { 0x945210f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3210fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83221103, 95 },
    { 0xb3e21104, 61 },
    { 0, 0 },
    { 0xa6a21106, 93 },
    { 0, 0 },
    { 0xf2c21108, 95 },
    { 0x2bd21109, 55 },
    { 0xf921109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c2110d, 34 },
    { 0, 0 },
    { 0x2a72110f, 46 },
    { 0x9792110f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c21113, 90 },
    { 0xec21114, 61 },
    { 0xb8221115, 98 },
    { 0xf8921115, 90 },
    { 0xcf321117, 90 },
    { 0x1c721113, 55 },
    { 0xa9421118, 61 },
    { 0, 0 },
    { 0x98e2111b, 64 },
    { 0xf7d2111c, 83 },
    { 0x5ad2111c, 99 },
    { 0xd9e2111b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45921128, 85 },
    { 0x4f721129, 76 },
    { 0xbf621128, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1921130, 67 },
    { 0x8e821131, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3321135, 98 },
    { 0x93b21136, 40 },
    { 0, 0 },
    { 0x36521138, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c21146, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb612114a, 76 },
    { 0x3262114b, 87 },
    { 0, 0 },
    { 0x6cd2114d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d821152, 23 },
    { 0x12921153, 90 },
    { 0, 0 },
    { 0x3d321155, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d21158, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f72115b, 95 },
    { 0x2aa2115c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f421160, 90 },
    { 0xafe21161, 52 },
    { 0x3d221160, 55 },
    { 0x63f21163, 43 },
    { 0, 0 },
    { 0x9c821165, 93 },
    { 0x6dc21166, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3052116d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21170, 90 },
    { 0x87e21171, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6521174, 88 },
    { 0, 0 },
    { 0x71f21176, 53 },
    { 0, 0 },
    { 0x1f21178, 34 },
    { 0, 0 },
    { 0xe302117a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2117f, 34 },
    { 0x9e121180, 23 },
    { 0xcf82117f, 78 },
    { 0, 0 },
    { 0x94b21183, 90 },
    { 0x88921183, 98 },
    { 0x97121185, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f2118e, 78 },
    { 0, 0 },
    { 0x8b121190, 33 },
    { 0x45021191, 98 },
    { 0, 0 },
    { 0xe9121193, 34 },
    { 0, 0 },
    { 0x6a921195, 90 },
    { 0x5b421196, 90 },
    { 0x22021196, 95 },
    { 0x7b21196, 98 },
    { 0x8b221199, 76 },
    { 0x7992119a, 90 },
    { 0x43421198, 95 },
    { 0xa0221196, 76 },
    { 0x15a21195, 95 },
    { 0xdb121196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4211a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6211a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5211a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e211ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2211b1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754211b6, 61 },
    { 0, 0 },
    { 0x5ed211b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5211bd, 98 },
    { 0, 0 },
    { 0x43b211bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d211c9, 78 },
    { 0x4cb211ca, 68 },
    { 0xbc2211c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3211ce, 33 },
    { 0x5eb211cf, 98 },
    { 0x24e211d0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166211d4, 23 },
    { 0, 0 },
    { 0x615211d6, 23 },
    { 0x66b211d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f211db, 80 },
    { 0, 0 },
    { 0x40f211dd, 95 },
    { 0xf81211de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8211e5, 98 },
    { 0x6dc211e6, 87 },
    { 0x438211e7, 77 },
    { 0x751211e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8211eb, 33 },
    { 0x68e211ec, 95 },
    { 0x71c211ed, 67 },
    { 0xe8a211ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2211f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb211f7, 88 },
    { 0, 0 },
    { 0x3ef211f9, 90 },
    { 0x66a211f9, 17 },
    { 0, 0 },
    { 0x156211fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b21200, 52 },
    { 0x9cc21200, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63321204, 69 },
    { 0, 0 },
    { 0x32d21206, 78 },
    { 0x5a421207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c2120b, 17 },
    { 0x2422120c, 98 },
    { 0x5d82120d, 90 },
    { 0x1c82120d, 81 },
    { 0xcff2120c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d021214, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9221219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde21221, 33 },
    { 0xd721222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a921225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42122b, 84 },
    { 0, 0 },
    { 0x4bb2122d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f721230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf21234, 67 },
    { 0x6bd21235, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf521238, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7742123c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb21241, 61 },
    { 0, 0 },
    { 0x14f21243, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b21246, 23 },
    { 0xd6d21247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2124a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef22124e, 66 },
    { 0, 0 },
    { 0x4e21250, 33 },
    { 0xf2521250, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad21258, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2125b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c21262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d32126c, 61 },
    { 0x7732126c, 93 },
    { 0x40a2126c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d21271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0321275, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97321278, 90 },
    { 0, 0 },
    { 0x42f2127a, 76 },
    { 0x7dc2127a, 67 },
    { 0xff52127c, 74 },
    { 0xbc92127c, 95 },
    { 0xe662127e, 90 },
    { 0x17f2127e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af21282, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b21286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7582128a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2192128e, 99 },
    { 0x2c32128e, 98 },
    { 0x39c21290, 33 },
    { 0xb9a21290, 33 },
    { 0x68021292, 55 },
    { 0x6be21291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a421298, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42129d, 100 },
    { 0x1ca2129d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6212a2, 76 },
    { 0, 0 },
    { 0x143212a4, 95 },
    { 0xe3e212a4, 23 },
    { 0x8ff212a6, 61 },
    { 0x832212a4, 55 },
    { 0x7b6212a8, 84 },
    { 0, 0 },
    { 0x807212aa, 61 },
    { 0xbe9212ab, 55 },
    { 0x192212aa, 98 },
    { 0x34c212ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2212b1, 95 },
    { 0xd9c212b1, 98 },
    { 0xb7a212b3, 23 },
    { 0xebb212b4, 95 },
    { 0x164212b5, 90 },
    { 0x79c212b6, 90 },
    { 0x8a9212b7, 33 },
    { 0xbd9212b7, 23 },
    { 0x9b1212b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6212be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248212c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c212c9, 90 },
    { 0, 0 },
    { 0xa16212cb, 61 },
    { 0xaa5212cc, 75 },
    { 0x1a5212cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e212d3, 34 },
    { 0, 0 },
    { 0x27e212d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x356212d8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e212db, 76 },
    { 0x4c212dc, 91 },
    { 0x87a212dc, 90 },
    { 0x276212dc, 91 },
    { 0x17b212dc, 66 },
    { 0x619212dd, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2212ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6212f6, 95 },
    { 0xdbf212f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a212fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02212fd, 85 },
    { 0xa12212fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c21303, 99 },
    { 0xa8a21304, 87 },
    { 0, 0 },
    { 0x4fa21306, 23 },
    { 0xe6221307, 90 },
    { 0xa6221308, 66 },
    { 0x9e921309, 95 },
    { 0xea21309, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2130d, 61 },
    { 0, 0 },
    { 0x392130f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b221312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e21315, 95 },
    { 0x53021315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8721319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c21325, 33 },
    { 0xf4f21326, 90 },
    { 0, 0 },
    { 0xe2a21328, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932132c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa21336, 99 },
    { 0, 0 },
    { 0xe321338, 46 },
    { 0, 0 },
    { 0x5a42133a, 55 },
    { 0x4c32133b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x822133e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28621341, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d21347, 95 },
    { 0x3421348, 52 },
    { 0x56d21348, 99 },
    { 0xfab21349, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d21352, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e21355, 98 },
    { 0, 0 },
    { 0x57921357, 93 },
    { 0, 0 },
    { 0xbd621359, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7952135d, 90 },
    { 0x85c2135e, 95 },
    { 0xf9b2135d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2421362, 33 },
    { 0xb0621363, 60 },
    { 0, 0 },
    { 0x39021365, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5821368, 75 },
    { 0, 0 },
    { 0xe4c2136a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5712136d, 52 },
    { 0x9682136d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93421375, 33 },
    { 0x77621375, 75 },
    { 0, 0 },
    { 0xd9921378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca2137b, 98 },
    { 0x6a32137b, 76 },
    { 0, 0 },
    { 0xda12137e, 23 },
    { 0x6572137f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd021383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20221388, 61 },
    { 0x12b21389, 61 },
    { 0, 0 },
    { 0x2112138b, 90 },
    { 0xb932138b, 93 },
    { 0xcdd2138d, 66 },
    { 0x5fb2138e, 95 },
    { 0xa622138f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e21395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b721398, 93 },
    { 0x97321398, 95 },
    { 0x1921398, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104213a0, 23 },
    { 0, 0 },
    { 0xd17213a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4213a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4213ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690213af, 90 },
    { 0x95f213b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x823213b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3213b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d213bc, 40 },
    { 0, 0 },
    { 0xba6213be, 95 },
    { 0xd04213bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98213c2, 87 },
    { 0x8d2213c3, 93 },
    { 0xb72213c4, 68 },
    { 0, 0 },
    { 0x1e8213c6, 61 },
    { 0, 0 },
    { 0x5a1213c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f213cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae213ce, 90 },
    { 0x29b213cf, 61 },
    { 0, 0 },
    { 0xfdf213d1, 90 },
    { 0, 0 },
    { 0x29e213d3, 33 },
    { 0, 0 },
    { 0xe86213d5, 98 },
    { 0xcd3213d6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1213db, 34 },
    { 0x96f213dc, 90 },
    { 0, 0 },
    { 0x6c4213de, 61 },
    { 0, 0 },
    { 0x2b8213e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe213e9, 76 },
    { 0, 0 },
    { 0x284213eb, 33 },
    { 0x599213ec, 47 },
    { 0xeb8213eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x445213f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d213f3, 81 },
    { 0, 0 },
    { 0x27e213f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f213fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8221400, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2021408, 33 },
    { 0x7f21408, 61 },
    { 0xd3321408, 100 },
    { 0xec621408, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93621412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b2141c, 52 },
    { 0, 0 },
    { 0x6792141e, 40 },
    { 0, 0 },
    { 0xd7921420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c21429, 61 },
    { 0x3cf21429, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a2142d, 61 },
    { 0x9752142e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf321431, 61 },
    { 0, 0 },
    { 0xbd721433, 98 },
    { 0x20821433, 34 },
    { 0, 0 },
    { 0xd7e21436, 43 },
    { 0, 0 },
    { 0x19621438, 33 },
    { 0, 0 },
    { 0xb162143a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2143f, 23 },
    { 0xfc72143f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x94321443, 33 },
    { 0x73521443, 33 },
    { 0xd3321444, 34 },
    { 0, 0 },
    { 0xc8921447, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a42144d, 93 },
    { 0x7db2144e, 85 },
    { 0x9d32144f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b21452, 69 },
    { 0, 0 },
    { 0xfa821454, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x76021457, 95 },
    { 0x52521458, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2145e, 76 },
    { 0xba52145e, 33 },
    { 0, 0 },
    { 0x7eb21461, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd21468, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2146d, 61 },
    { 0x3952146d, 85 },
    { 0xc1b2146e, 99 },
    { 0xcd42146f, 98 },
    { 0, 0 },
    { 0x83f21472, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c21477, 90 },
    { 0xfcb21478, 61 },
    { 0x6bc21478, 90 },
    { 0xe022147a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb472147f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d621483, 98 },
    { 0x9b21484, 77 },
    { 0x1c921484, 69 },
    { 0, 0 },
    { 0x6621487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4372148c, 95 },
    { 0, 0 },
    { 0xd0a2148e, 95 },
    { 0x31f2148f, 98 },
    { 0xf392148e, 95 },
    { 0x60d21491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a21494, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66921498, 98 },
    { 0, 0 },
    { 0x47b2149a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b2149e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5214a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df214a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6214ac, 36 },
    { 0, 0 },
    { 0xa0b214ae, 55 },
    { 0, 0 },
    { 0x17b214b0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59214ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c214c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1214c3, 66 },
    { 0x5da214c3, 95 },
    { 0x59c214c5, 100 },
    { 0, 0 },
    { 0xb50214c7, 66 },
    { 0x53a214c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f214ce, 95 },
    { 0, 0 },
    { 0xc3a214d0, 52 },
    { 0xc20214d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca214d9, 33 },
    { 0x8bb214da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b214e0, 87 },
    { 0xa5e214e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1214e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84214ec, 78 },
    { 0x58214ed, 78 },
    { 0xd21214ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536214f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51214fa, 38 },
    { 0x832214fb, 78 },
    { 0xa9214fc, 98 },
    { 0, 0 },
    { 0x5a8214fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab721502, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a321506, 95 },
    { 0xfd721507, 56 },
    { 0xce821507, 91 },
    { 0xa4921509, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32150d, 61 },
    { 0x7f62150d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a621511, 23 },
    { 0xcf721512, 33 },
    { 0, 0 },
    { 0x64721514, 95 },
    { 0x46b21515, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea221518, 90 },
    { 0, 0 },
    { 0x3992151a, 23 },
    { 0, 0 },
    { 0x8002151c, 87 },
    { 0xde92151d, 76 },
    { 0, 0 },
    { 0x15e2151f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e21524, 61 },
    { 0x2af21525, 90 },
    { 0xc2721525, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86921529, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e2152d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55721533, 90 },
    { 0x20321533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd21538, 33 },
    { 0x88721539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9462153c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3521540, 55 },
    { 0x64e21541, 98 },
    { 0x66d21541, 33 },
    { 0x80021543, 75 },
    { 0, 0 },
    { 0x1e21545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a21548, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682154b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2062154e, 61 },
    { 0, 0 },
    { 0x7bf21550, 99 },
    { 0x58f21551, 61 },
    { 0x82c21552, 61 },
    { 0x79321550, 4 },
    { 0xb8621554, 78 },
    { 0x3821555, 99 },
    { 0x45021551, 76 },
    { 0x30621557, 90 },
    { 0xc8a21558, 33 },
    { 0, 0 },
    { 0xf0e2155a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3552155f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b821563, 87 },
    { 0xb4521563, 100 },
    { 0, 0 },
    { 0xbec21566, 33 },
    { 0, 0 },
    { 0x87521568, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4782156b, 90 },
    { 0x4972156b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd21577, 99 },
    { 0x74b21578, 34 },
    { 0x49521579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a52157c, 33 },
    { 0x1ec2157c, 98 },
    { 0, 0 },
    { 0x9622157f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69521582, 95 },
    { 0x22f21583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3372158a, 52 },
    { 0x90c2158b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2342158e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a721595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d21598, 87 },
    { 0x33c21598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981215a9, 93 },
    { 0, 0 },
    { 0xc27215ab, 91 },
    { 0x3c9215ab, 80 },
    { 0x44f215ab, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374215b4, 88 },
    { 0xc02215b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2215b9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99215bd, 64 },
    { 0x650215be, 53 },
    { 0x851215bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe215c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148215ca, 90 },
    { 0x6aa215cb, 33 },
    { 0xdbd215cc, 33 },
    { 0x2a1215cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd215d3, 34 },
    { 0x917215d4, 61 },
    { 0x8e5215d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49215d8, 81 },
    { 0xa1d215d9, 98 },
    { 0x4f5215d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a215e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96215e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33215e9, 23 },
    { 0, 0 },
    { 0xe3b215eb, 66 },
    { 0xf6f215ec, 68 },
    { 0, 0 },
    { 0xc9c215ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360215f7, 98 },
    { 0, 0 },
    { 0xd7e215f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518215fe, 61 },
    { 0x169215fe, 65 },
    { 0xa2b215ff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca21603, 33 },
    { 0, 0 },
    { 0x13d21605, 76 },
    { 0, 0 },
    { 0x19221607, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf32160a, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a2160e, 95 },
    { 0, 0 },
    { 0x57321610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d21617, 76 },
    { 0xc4d21618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722161b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd2161f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b21622, 55 },
    { 0, 0 },
    { 0x10b21624, 33 },
    { 0xa8921625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a2162d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f821635, 93 },
    { 0xbab21635, 34 },
    { 0, 0 },
    { 0xebd21638, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee321641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a21647, 76 },
    { 0, 0 },
    { 0x7ec21649, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5421650, 46 },
    { 0xd5e21651, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b421654, 61 },
    { 0x96021655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae2165b, 93 },
    { 0x2962165c, 90 },
    { 0xbbe2165d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf21660, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed21668, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb21674, 33 },
    { 0x3b521675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4021679, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d42167c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab21684, 90 },
    { 0x33c21684, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd621688, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3812168d, 90 },
    { 0x9bd2168d, 33 },
    { 0x4292168f, 95 },
    { 0xf7c21690, 55 },
    { 0x7d621690, 76 },
    { 0x6ad21692, 98 },
    { 0xfc921693, 90 },
    { 0xb6821694, 95 },
    { 0xfb221694, 61 },
    { 0x23b21695, 33 },
    { 0x52121691, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd2169a, 34 },
    { 0x47e2169a, 55 },
    { 0, 0 },
    { 0x6642169d, 33 },
    { 0x76a2169d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33216a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f216a7, 33 },
    { 0xfab216a8, 98 },
    { 0, 0 },
    { 0x1d8216aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec216ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8216b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4216b6, 46 },
    { 0, 0 },
    { 0xa23216b8, 91 },
    { 0xbb1216b9, 91 },
    { 0x264216b9, 77 },
    { 0x654216bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe216bf, 90 },
    { 0x146216c0, 99 },
    { 0x217216c1, 76 },
    { 0x95b216c2, 61 },
    { 0, 0 },
    { 0xf07216c4, 61 },
    { 0xb78216c5, 55 },
    { 0xc78216c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3216c9, 95 },
    { 0x7bd216c9, 95 },
    { 0x7d2216cb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf216d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282216d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38216d9, 75 },
    { 0xa42216da, 95 },
    { 0x69d216d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6216de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb216e1, 43 },
    { 0xa40216e1, 95 },
    { 0x4e6216e3, 90 },
    { 0xeb8216e1, 43 },
    { 0xc6a216e5, 90 },
    { 0x17f216e4, 67 },
    { 0xe60216e7, 98 },
    { 0, 0 },
    { 0xdbe216e9, 95 },
    { 0x91e216ea, 17 },
    { 0, 0 },
    { 0xfbf216ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541216f2, 33 },
    { 0x676216f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0216f7, 88 },
    { 0x617216f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54216fb, 95 },
    { 0, 0 },
    { 0xf3e216fd, 77 },
    { 0, 0 },
    { 0xe9f216ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa21707, 33 },
    { 0x91e21708, 52 },
    { 0x86521708, 43 },
    { 0x6d21707, 43 },
    { 0xfd02170b, 33 },
    { 0x5cf2170b, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf21713, 55 },
    { 0, 0 },
    { 0x20f21715, 55 },
    { 0, 0 },
    { 0xff421717, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2732171b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e421720, 33 },
    { 0x9d821721, 55 },
    { 0xec721722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a21729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f2172c, 98 },
    { 0x5d62172d, 90 },
    { 0x1712172d, 36 },
    { 0xa8c2172f, 84 },
    { 0x92721730, 61 },
    { 0xd0c21731, 36 },
    { 0xf5a21730, 88 },
    { 0x4bf21732, 61 },
    { 0x3e721730, 33 },
    { 0, 0 },
    { 0x82621736, 95 },
    { 0x2bd21737, 33 },
    { 0, 0 },
    { 0x99321739, 61 },
    { 0, 0 },
    { 0x23e2173b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b2173e, 98 },
    { 0x79e2173e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c621742, 66 },
    { 0x80121742, 61 },
    { 0xed721744, 100 },
    { 0x88b21742, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1652174b, 88 },
    { 0x36e2174b, 91 },
    { 0xc032174d, 76 },
    { 0xa882174b, 5 },
    { 0x7942174e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5421756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9221759, 52 },
    { 0xbc12175a, 98 },
    { 0x8682175b, 91 },
    { 0x1932175c, 33 },
    { 0x9c2175d, 52 },
    { 0x25c2175c, 70 },
    { 0xb642175c, 90 },
    { 0xde21760, 23 },
    { 0xa0021761, 17 },
    { 0, 0 },
    { 0xfe021763, 91 },
    { 0x78021764, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a21767, 95 },
    { 0, 0 },
    { 0x86621769, 99 },
    { 0, 0 },
    { 0x2772176b, 90 },
    { 0xfea2176b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd21778, 33 },
    { 0xd4a21779, 75 },
    { 0, 0 },
    { 0x8842177b, 91 },
    { 0, 0 },
    { 0xc4f2177d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92021781, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d21785, 90 },
    { 0, 0 },
    { 0x24a21787, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42178b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de21792, 90 },
    { 0, 0 },
    { 0xb2a21794, 90 },
    { 0xe6421794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a62179d, 52 },
    { 0xc932179e, 99 },
    { 0x9942179f, 17 },
    { 0x7aa217a0, 61 },
    { 0x7f3217a1, 61 },
    { 0xa49217a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b217ab, 76 },
    { 0xf0a217ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3217af, 98 },
    { 0xdfb217b0, 99 },
    { 0x555217b0, 93 },
    { 0x3a4217b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0217ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578217be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714217c2, 33 },
    { 0xf36217c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c217c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b217cb, 55 },
    { 0, 0 },
    { 0xc08217cd, 61 },
    { 0x5b8217ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e217d5, 23 },
    { 0, 0 },
    { 0xdf1217d7, 43 },
    { 0, 0 },
    { 0xa1c217d9, 66 },
    { 0xd6c217da, 61 },
    { 0x6d6217db, 67 },
    { 0xe39217dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67217e0, 90 },
    { 0, 0 },
    { 0x90217e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35217e5, 46 },
    { 0x123217e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e217e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd217f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa217f4, 95 },
    { 0x992217f5, 61 },
    { 0x61217f5, 61 },
    { 0x75a217f7, 90 },
    { 0x5d7217f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6217fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x741217fe, 90 },
    { 0x3d4217ff, 34 },
    { 0xb10217ff, 95 },
    { 0x1e021801, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91021806, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8922180b, 76 },
    { 0xefc2180c, 9 },
    { 0x5532180d, 98 },
    { 0xc702180e, 52 },
    { 0x2242180d, 46 },
    { 0x9372180f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7321813, 95 },
    { 0xa4121814, 33 },
    { 0xff221813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44721818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512181e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2021822, 95 },
    { 0xd9621822, 95 },
    { 0x60a21824, 78 },
    { 0x4fc21823, 76 },
    { 0, 0 },
    { 0x82021827, 95 },
    { 0xaf121828, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f21833, 90 },
    { 0, 0 },
    { 0x5f321835, 97 },
    { 0x82521836, 76 },
    { 0x8df21837, 34 },
    { 0x8fd21836, 100 },
    { 0x5e321839, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24821840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d421845, 52 },
    { 0xb7a21845, 68 },
    { 0xbd121847, 55 },
    { 0x29621848, 36 },
    { 0xb7d21848, 101 },
    { 0, 0 },
    { 0xa2b2184b, 61 },
    { 0xbda2184c, 95 },
    { 0, 0 },
    { 0x6b12184e, 55 },
    { 0x7fd2184f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34021853, 61 },
    { 0xce921853, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb21857, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b2185b, 84 },
    { 0, 0 },
    { 0xe242185d, 33 },
    { 0, 0 },
    { 0x2a92185f, 95 },
    { 0x24d2185f, 55 },
    { 0xcc421861, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56521866, 23 },
    { 0x36221867, 91 },
    { 0x5c321868, 90 },
    { 0x12c21869, 87 },
    { 0x45021869, 98 },
    { 0x1f921869, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f21872, 93 },
    { 0, 0 },
    { 0x9ea21874, 98 },
    { 0, 0 },
    { 0x2021876, 76 },
    { 0, 0 },
    { 0xbb521878, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb72187d, 84 },
    { 0xa662187e, 100 },
    { 0x6a82187e, 52 },
    { 0x22521880, 91 },
    { 0x83921880, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d821885, 90 },
    { 0x4e821886, 90 },
    { 0xe1521886, 95 },
    { 0xec821885, 90 },
    { 0x3f821889, 76 },
    { 0xacb21888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2582188d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4121891, 98 },
    { 0, 0 },
    { 0xfc021893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed21896, 23 },
    { 0x81421897, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4192189a, 23 },
    { 0, 0 },
    { 0x52a2189c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e218a0, 61 },
    { 0, 0 },
    { 0xd0f218a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8218a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d218a8, 33 },
    { 0, 0 },
    { 0xb52218aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23218ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8218b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x38218b6, 33 },
    { 0, 0 },
    { 0xb56218b8, 98 },
    { 0x2db218b9, 77 },
    { 0x604218ba, 91 },
    { 0, 0 },
    { 0x4f1218bc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8218c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f218c4, 52 },
    { 0xf4e218c4, 69 },
    { 0, 0 },
    { 0xfc5218c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b218d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561218d9, 98 },
    { 0xbe5218d9, 33 },
    { 0xdec218db, 68 },
    { 0x570218d9, 23 },
    { 0xd15218d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc218e1, 55 },
    { 0xe4d218e1, 61 },
    { 0, 0 },
    { 0x527218e4, 34 },
    { 0, 0 },
    { 0x68f218e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750218f0, 90 },
    { 0x326218f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e218f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f218f9, 98 },
    { 0x788218f9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153218fe, 95 },
    { 0, 0 },
    { 0x63921900, 77 },
    { 0x46221901, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17921907, 46 },
    { 0xb6a21908, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2190d, 34 },
    { 0x5352190e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f621915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2191a, 98 },
    { 0xb152191b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f32191f, 93 },
    { 0xa9a21920, 99 },
    { 0x2172191f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f21926, 43 },
    { 0x3e821927, 61 },
    { 0, 0 },
    { 0x10921929, 88 },
    { 0, 0 },
    { 0x18d2192b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6121932, 61 },
    { 0x25021933, 46 },
    { 0xf8d21934, 32 },
    { 0xe3721934, 33 },
    { 0xe1a21936, 23 },
    { 0xe6a21937, 90 },
    { 0, 0 },
    { 0x41021939, 46 },
    { 0xa772193a, 61 },
    { 0xc432193a, 58 },
    { 0x2c32193c, 52 },
    { 0xa4c2193d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93321944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0221947, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6262194a, 61 },
    { 0, 0 },
    { 0x1452194c, 61 },
    { 0, 0 },
    { 0xf2c2194e, 33 },
    { 0, 0 },
    { 0x8b521950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53221954, 55 },
    { 0x8d621955, 61 },
    { 0x6a221954, 46 },
    { 0xd4f21954, 95 },
    { 0x37421958, 34 },
    { 0x1921959, 98 },
    { 0x6cc21956, 46 },
    { 0xe182195b, 85 },
    { 0x2fb2195c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2902195f, 61 },
    { 0x7c021960, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8221963, 99 },
    { 0, 0 },
    { 0xfb21965, 98 },
    { 0xfe21965, 98 },
    { 0x95f21965, 76 },
    { 0, 0 },
    { 0x2bb21969, 36 },
    { 0x6221969, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf21972, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5121978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e2197c, 95 },
    { 0x7292197d, 95 },
    { 0, 0 },
    { 0xe612197f, 95 },
    { 0x61f21980, 91 },
    { 0x64e2197f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a821985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc02198a, 33 },
    { 0x47b2198b, 95 },
    { 0xdc52198c, 90 },
    { 0xd332198d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a221995, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d21999, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3332199c, 61 },
    { 0, 0 },
    { 0xc8f2199e, 88 },
    { 0xdf72199f, 90 },
    { 0x6e72199f, 46 },
    { 0x3e0219a1, 61 },
    { 0xf13219a1, 55 },
    { 0xf8b219a3, 52 },
    { 0xb2219a3, 98 },
    { 0x2b5219a5, 23 },
    { 0xfd0219a4, 95 },
    { 0, 0 },
    { 0xefa219a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e219b2, 33 },
    { 0x74a219b2, 84 },
    { 0xcad219b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a219b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8219bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6219c1, 55 },
    { 0x9ef219c2, 76 },
    { 0x3a2219c3, 93 },
    { 0, 0 },
    { 0xbd4219c5, 23 },
    { 0x335219c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x222219c9, 17 },
    { 0, 0 },
    { 0xd21219cb, 90 },
    { 0xca0219cb, 90 },
    { 0x20a219cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2219d4, 34 },
    { 0x585219d5, 84 },
    { 0, 0 },
    { 0xd01219d7, 61 },
    { 0xb8a219d8, 61 },
    { 0xfe6219d9, 55 },
    { 0, 0 },
    { 0x8f3219db, 90 },
    { 0, 0 },
    { 0x811219dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c219e0, 33 },
    { 0, 0 },
    { 0xff4219e2, 95 },
    { 0x547219e3, 99 },
    { 0x35d219e4, 64 },
    { 0, 0 },
    { 0x5fe219e6, 76 },
    { 0xb03219e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb219f0, 23 },
    { 0, 0 },
    { 0xd74219f2, 25 },
    { 0xd3b219f3, 91 },
    { 0x85219f4, 95 },
    { 0xe7a219f4, 30 },
    { 0, 0 },
    { 0x49c219f7, 88 },
    { 0xf4e219f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x206219fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e21a01, 83 },
    { 0, 0 },
    { 0x77821a03, 76 },
    { 0xc1521a03, 43 },
    { 0xf8a21a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea21a0c, 61 },
    { 0, 0 },
    { 0x3bd21a0e, 98 },
    { 0x94e21a0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb321a16, 61 },
    { 0xb5021a16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92121a1a, 33 },
    { 0x1ab21a1b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c21a1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6721a24, 91 },
    { 0x2621a25, 23 },
    { 0xa8621a26, 66 },
    { 0, 0 },
    { 0xfaa21a28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d21a2b, 55 },
    { 0xcfc21a2c, 80 },
    { 0x7f521a2d, 98 },
    { 0, 0 },
    { 0x43e21a2f, 98 },
    { 0x1e221a2f, 23 },
    { 0x8b921a30, 78 },
    { 0xb3621a30, 76 },
    { 0xd8a21a33, 33 },
    { 0, 0 },
    { 0x11821a35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa21a38, 25 },
    { 0xd1621a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c21a3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74921a42, 67 },
    { 0, 0 },
    { 0x62421a44, 33 },
    { 0x86f21a45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4521a4a, 55 },
    { 0x54321a4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69521a4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf721a52, 95 },
    { 0, 0 },
    { 0xa3321a54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e121a5a, 93 },
    { 0, 0 },
    { 0x2f21a5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4021a64, 46 },
    { 0, 0 },
    { 0x16621a66, 95 },
    { 0x81221a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17521a6b, 55 },
    { 0, 0 },
    { 0x1cb21a6d, 46 },
    { 0xb6121a6d, 90 },
    { 0xe6c21a6f, 68 },
    { 0xef521a6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb21a73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7721a76, 61 },
    { 0, 0 },
    { 0x67021a78, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd21a7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a21a7e, 90 },
    { 0, 0 },
    { 0x96621a80, 43 },
    { 0x98d21a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35121a8a, 43 },
    { 0x79621a8b, 98 },
    { 0, 0 },
    { 0x7ab21a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce21a93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48821a9f, 68 },
    { 0x4c321a9f, 81 },
    { 0xcd421aa1, 38 },
    { 0x33921a9f, 76 },
    { 0x5e621aa3, 84 },
    { 0x62521aa4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac21aa7, 33 },
    { 0x55a21aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed21aab, 33 },
    { 0x82c21aab, 33 },
    { 0, 0 },
    { 0x54321aae, 91 },
    { 0x29a21aaf, 23 },
    { 0x18921ab0, 52 },
    { 0x79121aaf, 65 },
    { 0xf0b21ab0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d321ab8, 97 },
    { 0x6f021ab9, 98 },
    { 0x70c21aba, 55 },
    { 0x65d21ab9, 36 },
    { 0x59121abc, 90 },
    { 0x8d921abd, 90 },
    { 0x21321abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc21ac1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31421ac5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9021ac8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c21acb, 68 },
    { 0x3fb21acb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b21ad4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6321ad8, 90 },
    { 0xce721ad9, 43 },
    { 0x4eb21ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee21ade, 95 },
    { 0, 0 },
    { 0x34721ae0, 98 },
    { 0x3ae21ae1, 34 },
    { 0xf7921ae2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20221ae8, 95 },
    { 0, 0 },
    { 0x5821aea, 23 },
    { 0x60821aeb, 83 },
    { 0, 0 },
    { 0x5c921aed, 90 },
    { 0xe3721aed, 61 },
    { 0x45f21aed, 76 },
    { 0, 0 },
    { 0xcb221af1, 95 },
    { 0xb6f21af2, 52 },
    { 0, 0 },
    { 0xc3421af4, 81 },
    { 0xcb921af4, 53 },
    { 0x5da21af5, 90 },
    { 0xb3421af7, 91 },
    { 0x8f821af6, 90 },
    { 0, 0 },
    { 0xf3621afa, 42 },
    { 0x40f21afb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be21b00, 33 },
    { 0x77d21b01, 46 },
    { 0xb0421b00, 17 },
    { 0x10621b00, 93 },
    { 0, 0 },
    { 0xf5f21b05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d21b08, 61 },
    { 0x2a821b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca21b0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29021b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b21b20, 93 },
    { 0, 0 },
    { 0xc0421b22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8221b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24021b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf21b2d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf321b30, 61 },
    { 0, 0 },
    { 0x55621b32, 90 },
    { 0xcfb21b33, 90 },
    { 0xfd521b34, 95 },
    { 0xf3921b35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b821b38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc21b3c, 23 },
    { 0, 0 },
    { 0xdd921b3e, 81 },
    { 0xc4521b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca321b42, 38 },
    { 0, 0 },
    { 0x9e121b44, 52 },
    { 0x9dd21b44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee21b49, 98 },
    { 0x98421b4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f21b4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78521b50, 95 },
    { 0xaa621b50, 98 },
    { 0xdcf21b52, 67 },
    { 0, 0 },
    { 0x9ab21b54, 93 },
    { 0xa9d21b54, 91 },
    { 0xccc21b56, 61 },
    { 0x8c321b57, 23 },
    { 0xb1921b56, 95 },
    { 0x32721b57, 95 },
    { 0x10921b5a, 98 },
    { 0, 0 },
    { 0xb0c21b5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f21b61, 33 },
    { 0, 0 },
    { 0xcca21b63, 23 },
    { 0x77521b64, 93 },
    { 0, 0 },
    { 0xf5721b66, 33 },
    { 0, 0 },
    { 0x2f321b68, 43 },
    { 0, 0 },
    { 0xaa621b6a, 93 },
    { 0, 0 },
    { 0x41821b6c, 61 },
    { 0, 0 },
    { 0x3b21b6e, 67 },
    { 0x44721b6f, 76 },
    { 0x5af21b70, 38 },
    { 0x8ba21b71, 90 },
    { 0x73821b72, 90 },
    { 0x42e21b72, 91 },
    { 0, 0 },
    { 0x2a421b75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e21b7a, 88 },
    { 0x99b21b7b, 90 },
    { 0xcdc21b7c, 33 },
    { 0x41721b7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87721b89, 100 },
    { 0x83021b8a, 58 },
    { 0x32921b8b, 76 },
    { 0x21b21b8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64021b8f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f21b92, 90 },
    { 0xfb421b93, 93 },
    { 0x25421b94, 23 },
    { 0xd5321b95, 95 },
    { 0x19a21b93, 98 },
    { 0xe1821b97, 88 },
    { 0x6f421b98, 90 },
    { 0x54c21b97, 100 },
    { 0x67f21b98, 43 },
    { 0x9b521b98, 98 },
    { 0x79a21b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32321ba2, 33 },
    { 0x9321ba3, 90 },
    { 0x96d21ba2, 99 },
    { 0, 0 },
    { 0xeac21ba6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84821baa, 61 },
    { 0, 0 },
    { 0x3821bac, 90 },
    { 0xd5b21bac, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x37821bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a21bb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c21bbe, 52 },
    { 0xeb421bbe, 83 },
    { 0xa7821bc0, 56 },
    { 0, 0 },
    { 0x23b21bc2, 90 },
    { 0, 0 },
    { 0x28a21bc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f521bc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a21bcc, 98 },
    { 0x62621bcd, 23 },
    { 0xe5121bce, 100 },
    { 0, 0 },
    { 0x86221bd0, 70 },
    { 0x2621bd1, 61 },
    { 0xc0c21bd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1621bd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf321bdc, 77 },
    { 0xb2021bdd, 99 },
    { 0x4e521bdd, 99 },
    { 0x13321bdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df21be2, 23 },
    { 0xeb821be3, 95 },
    { 0, 0 },
    { 0xd4821be5, 98 },
    { 0, 0 },
    { 0x49121be7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6821beb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a921bee, 95 },
    { 0, 0 },
    { 0x7d621bf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79021bf5, 46 },
    { 0x97421bf6, 95 },
    { 0x3eb21bf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85721bfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b21c02, 90 },
    { 0, 0 },
    { 0x9d521c04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72221c07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee21c0d, 33 },
    { 0, 0 },
    { 0x92721c0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6921c19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2821c1c, 98 },
    { 0, 0 },
    { 0xe8021c1e, 61 },
    { 0x57521c1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78121c26, 33 },
    { 0, 0 },
    { 0x6d421c28, 87 },
    { 0x52421c29, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x28421c2c, 23 },
    { 0, 0 },
    { 0xed221c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d21c31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69221c35, 34 },
    { 0, 0 },
    { 0xc0d21c37, 95 },
    { 0, 0 },
    { 0xbe921c39, 95 },
    { 0xf6621c3a, 40 },
    { 0xeb121c39, 76 },
    { 0xd5221c3c, 23 },
    { 0xdbd21c3a, 98 },
    { 0xde821c3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f021c41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a021c46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67121c4a, 23 },
    { 0, 0 },
    { 0xdee21c4c, 87 },
    { 0x9ff21c4d, 90 },
    { 0, 0 },
    { 0xdff21c4f, 61 },
    { 0x5f521c4f, 76 },
    { 0xffe21c51, 23 },
    { 0x53821c52, 55 },
    { 0x56d21c53, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed221c5a, 96 },
    { 0x94a21c5b, 46 },
    { 0x64121c5c, 98 },
    { 0xabc21c5d, 98 },
    { 0x28e21c5d, 95 },
    { 0x93f21c5d, 67 },
    { 0xdee21c5e, 93 },
    { 0xb5821c61, 20 },
    { 0x5d521c62, 87 },
    { 0xd2921c63, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d21c66, 43 },
    { 0x90921c67, 66 },
    { 0, 0 },
    { 0x6b521c69, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac221c6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2121c70, 98 },
    { 0xb421c71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9121c75, 76 },
    { 0xd1121c76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3021c7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43021c81, 67 },
    { 0, 0 },
    { 0x4c021c83, 33 },
    { 0, 0 },
    { 0xe121c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b21c8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49321c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf821c97, 43 },
    { 0x2ee21c97, 90 },
    { 0x82121c98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8421c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf821ca0, 95 },
    { 0, 0 },
    { 0xaef21ca2, 77 },
    { 0x6e021ca3, 97 },
    { 0xf2521ca3, 55 },
    { 0xeba21ca5, 23 },
    { 0xfc421ca5, 23 },
    { 0x60c21ca7, 98 },
    { 0x3fc21ca6, 33 },
    { 0x3e21ca9, 45 },
    { 0, 0 },
    { 0x51821cab, 87 },
    { 0x2b921cac, 98 },
    { 0, 0 },
    { 0xf2321cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c21cb2, 55 },
    { 0x85a21cb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f21cb6, 61 },
    { 0x3f421cb7, 90 },
    { 0x67b21cb8, 76 },
    { 0xfaa21cb8, 17 },
    { 0xc1321cb9, 33 },
    { 0x84721cb9, 98 },
    { 0x3b421cbc, 43 },
    { 0, 0 },
    { 0xc7621cbe, 56 },
    { 0x6f721cbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee21cc2, 55 },
    { 0x42a21cc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a21ccc, 90 },
    { 0x53321ccc, 90 },
    { 0x7321ccc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x89721cd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2721cda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28321ce3, 46 },
    { 0, 0 },
    { 0x30521ce5, 98 },
    { 0xe4921ce6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5121ce9, 61 },
    { 0xf5f21cea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x57121ced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d721cf0, 46 },
    { 0xe1021cf1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe21cf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe21cf9, 90 },
    { 0xffa21cfa, 95 },
    { 0x42a21cfb, 33 },
    { 0, 0 },
    { 0xf3e21cfd, 61 },
    { 0x1a521cfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7521d02, 98 },
    { 0x921d03, 67 },
    { 0xa7921d04, 33 },
    { 0xeaf21d04, 91 },
    { 0xe6521d06, 23 },
    { 0x69b21d06, 33 },
    { 0x89521d08, 98 },
    { 0xa4a21d08, 85 },
    { 0, 0 },
    { 0x22421d0b, 17 },
    { 0x31e21d0c, 98 },
    { 0x8c921d0c, 98 },
    { 0xe6021d0e, 84 },
    { 0xff21d0f, 100 },
    { 0xdcc21d0e, 91 },
    { 0x68a21d0f, 33 },
    { 0x42921d12, 100 },
    { 0xe5a21d13, 67 },
    { 0x60a21d14, 34 },
    { 0, 0 },
    { 0xe0b21d16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c21d19, 99 },
    { 0x3a821d19, 15 },
    { 0xded21d19, 90 },
    { 0xacc21d1c, 90 },
    { 0x58a21d1d, 33 },
    { 0x1d21d1c, 100 },
    { 0x76121d1d, 33 },
    { 0xf6021d20, 43 },
    { 0x6ea21d1b, 90 },
    { 0, 0 },
    { 0x41821d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57621d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f21d32, 77 },
    { 0x3ed21d33, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59521d37, 33 },
    { 0xf0f21d37, 95 },
    { 0x26721d38, 95 },
    { 0, 0 },
    { 0x3d621d3b, 34 },
    { 0x92b21d3c, 91 },
    { 0, 0 },
    { 0xf6f21d3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76121d41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d21d4b, 93 },
    { 0xebf21d4c, 33 },
    { 0x76321d4d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd221d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc21d56, 98 },
    { 0xdda21d57, 23 },
    { 0xb3e21d57, 33 },
    { 0xe2321d57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c721d5e, 34 },
    { 0x24b21d5f, 90 },
    { 0x86121d5f, 34 },
    { 0x84321d61, 33 },
    { 0x24221d61, 87 },
    { 0xf1421d5f, 34 },
    { 0x1a521d64, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b21d69, 61 },
    { 0, 0 },
    { 0x6d221d6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e21d71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51421d77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5221d7d, 99 },
    { 0xaec21d7e, 84 },
    { 0x93821d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b21d82, 52 },
    { 0x49421d82, 99 },
    { 0x1dd21d84, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b21d87, 95 },
    { 0, 0 },
    { 0x73e21d89, 99 },
    { 0, 0 },
    { 0x58e21d8b, 43 },
    { 0x5ea21d8c, 33 },
    { 0x96621d8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80321d92, 76 },
    { 0x79521d92, 55 },
    { 0, 0 },
    { 0xaf121d95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa521d9d, 56 },
    { 0xe1021d9e, 33 },
    { 0x88c21d9f, 33 },
    { 0, 0 },
    { 0xe2221da1, 98 },
    { 0x3df21da1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15021da5, 95 },
    { 0x2c321da5, 95 },
    { 0, 0 },
    { 0x6c421da8, 55 },
    { 0xff021da9, 90 },
    { 0x72721da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d21db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33121db4, 34 },
    { 0x5b21db5, 87 },
    { 0, 0 },
    { 0xe5121db7, 23 },
    { 0, 0 },
    { 0x12621db9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb21dbc, 46 },
    { 0xf8821dbd, 33 },
    { 0x94621dbd, 43 },
    { 0x10921dbf, 15 },
    { 0, 0 },
    { 0x3b721dc1, 68 },
    { 0x14721dc1, 95 },
    { 0, 0 },
    { 0x2c821dc4, 34 },
    { 0x55621dc4, 90 },
    { 0x25e21dc6, 33 },
    { 0, 0 },
    { 0xb21dc8, 98 },
    { 0x58d21dc8, 75 },
    { 0xfa421dca, 93 },
    { 0, 0 },
    { 0x66d21dcc, 23 },
    { 0x58421dcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf21dd0, 61 },
    { 0, 0 },
    { 0x24721dd2, 52 },
    { 0x29121dd2, 23 },
    { 0x78121dd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c721dd8, 82 },
    { 0x46f21dd9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55921de1, 90 },
    { 0x18921de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f221de5, 52 },
    { 0xf021de6, 66 },
    { 0, 0 },
    { 0x79921de8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c721deb, 95 },
    { 0x3ec21dec, 61 },
    { 0x17821ded, 76 },
    { 0x80b21deb, 90 },
    { 0x27821def, 66 },
    { 0, 0 },
    { 0xbd21df1, 91 },
    { 0x6ab21df2, 95 },
    { 0, 0 },
    { 0x4da21df4, 33 },
    { 0x2ed21df4, 46 },
    { 0xb7f21df4, 90 },
    { 0, 0 },
    { 0x18521df8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e21dfc, 33 },
    { 0, 0 },
    { 0x1bd21dfe, 33 },
    { 0x26b21dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef21e06, 80 },
    { 0x32521e07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66921e0f, 90 },
    { 0, 0 },
    { 0x55a21e11, 91 },
    { 0, 0 },
    { 0x7b621e13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f21e19, 33 },
    { 0x30721e19, 90 },
    { 0x9c21e1b, 67 },
    { 0x20a21e1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf721e1f, 90 },
    { 0xd3421e20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b21e25, 91 },
    { 0xda021e26, 33 },
    { 0x43821e27, 85 },
    { 0xf721e25, 76 },
    { 0x2c921e28, 88 },
    { 0, 0 },
    { 0x5a821e2b, 91 },
    { 0xa6621e2c, 95 },
    { 0x61b21e2d, 23 },
    { 0, 0 },
    { 0xe6921e2f, 23 },
    { 0xed921e30, 33 },
    { 0x9f321e31, 33 },
    { 0xbb821e32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2321e37, 80 },
    { 0xdef21e38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d21e40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4621e45, 90 },
    { 0xe8c21e46, 93 },
    { 0x12321e46, 34 },
    { 0, 0 },
    { 0xcab21e49, 61 },
    { 0xb2021e49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13421e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c121e53, 90 },
    { 0x6821e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d21e57, 90 },
    { 0x81121e58, 52 },
    { 0xa8821e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39621e5f, 23 },
    { 0x61921e60, 76 },
    { 0x56921e60, 76 },
    { 0x93821e60, 23 },
    { 0xfa821e63, 95 },
    { 0x9b821e63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb821e6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf421e70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d21e77, 17 },
    { 0x51821e78, 99 },
    { 0x38721e79, 61 },
    { 0x28021e7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd21e7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf421e83, 61 },
    { 0xd2f21e84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c21e87, 76 },
    { 0, 0 },
    { 0xed321e89, 64 },
    { 0, 0 },
    { 0xd0e21e8b, 90 },
    { 0xc3921e8c, 90 },
    { 0, 0 },
    { 0xf1e21e8e, 85 },
    { 0x42721e8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff321e93, 91 },
    { 0xe9e21e94, 33 },
    { 0x6da21e94, 95 },
    { 0x7a21e93, 61 },
    { 0xf4821e97, 43 },
    { 0x6cd21e98, 55 },
    { 0xe9721e98, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f21ea2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f21eaa, 98 },
    { 0x7821eab, 33 },
    { 0xb621eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11821eaf, 23 },
    { 0, 0 },
    { 0xd2c21eb1, 34 },
    { 0x4fe21eb2, 61 },
    { 0x3b321eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba21eb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a21ebf, 90 },
    { 0xbbe21ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc21ec3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e221ecb, 33 },
    { 0x32321ecb, 88 },
    { 0x8bf21ecb, 90 },
    { 0x66421ece, 90 },
    { 0xfde21ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa21ed4, 69 },
    { 0, 0 },
    { 0x80521ed6, 90 },
    { 0xfe821ed7, 95 },
    { 0xbac21ed7, 84 },
    { 0x4321ed9, 90 },
    { 0x2ab21eda, 98 },
    { 0x421ed9, 95 },
    { 0x64f21ed6, 99 },
    { 0xed621edd, 55 },
    { 0x85a21ede, 95 },
    { 0x78921ede, 61 },
    { 0x2aa21ee0, 52 },
    { 0x59421ee1, 68 },
    { 0xf5c21ee0, 90 },
    { 0, 0 },
    { 0x54021ee4, 23 },
    { 0xcfb21ee5, 23 },
    { 0xbca21ee4, 76 },
    { 0x92921ee7, 95 },
    { 0xa7021ee8, 66 },
    { 0x3f121ee9, 78 },
    { 0x70721ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69521eee, 23 },
    { 0xc2921eef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39921ef3, 95 },
    { 0, 0 },
    { 0x52821ef5, 55 },
    { 0, 0 },
    { 0x57221ef7, 98 },
    { 0x37221ef8, 34 },
    { 0xad021ef9, 53 },
    { 0x6eb21ef7, 43 },
    { 0x23b21ef8, 93 },
    { 0, 0 },
    { 0xdda21efd, 94 },
    { 0x88e21efe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b21f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61721f07, 98 },
    { 0x91a21f08, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7021f0d, 76 },
    { 0, 0 },
    { 0x82621f0f, 55 },
    { 0, 0 },
    { 0x33821f11, 95 },
    { 0, 0 },
    { 0x73621f13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da21f17, 83 },
    { 0x8e021f18, 69 },
    { 0xaa021f19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c121f20, 99 },
    { 0x5e421f21, 96 },
    { 0xc6621f22, 94 },
    { 0xcdc21f23, 85 },
    { 0xd7421f24, 23 },
    { 0, 0 },
    { 0x5e321f26, 61 },
    { 0x68c21f26, 43 },
    { 0x73a21f28, 88 },
    { 0xc21f27, 78 },
    { 0, 0 },
    { 0xb5421f2b, 87 },
    { 0x43021f2b, 98 },
    { 0x1cd21f2c, 75 },
    { 0x5121f2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8021f32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a21f3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e21f42, 33 },
    { 0, 0 },
    { 0xea721f44, 33 },
    { 0x26521f45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87121f4a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a21f51, 90 },
    { 0x28521f52, 76 },
    { 0x4ab21f53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37621f56, 33 },
    { 0x57921f57, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c21f5b, 61 },
    { 0x27d21f5c, 2 },
    { 0xff021f5c, 66 },
    { 0x5ab21f5c, 90 },
    { 0, 0 },
    { 0x77c21f60, 66 },
    { 0, 0 },
    { 0xe7321f62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93421f69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96821f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a21f72, 55 },
    { 0, 0 },
    { 0xbaf21f74, 66 },
    { 0x77f21f74, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d21f7a, 98 },
    { 0, 0 },
    { 0xdb621f7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c21f81, 98 },
    { 0, 0 },
    { 0x67121f83, 93 },
    { 0, 0 },
    { 0xe1421f85, 34 },
    { 0xb1c21f86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95321f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e21f97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a821f9b, 85 },
    { 0, 0 },
    { 0x5b821f9d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9021fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da21faa, 98 },
    { 0xae621faa, 93 },
    { 0xb9121fac, 77 },
    { 0xd0821fac, 66 },
    { 0, 0 },
    { 0x57a21faf, 83 },
    { 0x88321faf, 90 },
    { 0xd9721fb1, 61 },
    { 0xd9421faf, 76 },
    { 0xeb721fb3, 33 },
    { 0, 0 },
    { 0x16c21fb5, 90 },
    { 0, 0 },
    { 0xc8021fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4221fbc, 78 },
    { 0, 0 },
    { 0xa2221fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e21fc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8121fcc, 90 },
    { 0x98421fcd, 23 },
    { 0xb8521fcd, 61 },
    { 0xd6721fce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81021fd2, 77 },
    { 0x5e21fd3, 99 },
    { 0x4a021fd2, 93 },
    { 0x14421fd3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f721fd9, 33 },
    { 0xec21fda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c21fde, 98 },
    { 0xed721fde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d21fe2, 87 },
    { 0xac721fe3, 76 },
    { 0x8dc21fe4, 77 },
    { 0x9f221fe3, 43 },
    { 0x3aa21fe4, 90 },
    { 0xbad21fe7, 46 },
    { 0x5e621fe8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef21feb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f021fef, 91 },
    { 0, 0 },
    { 0x12621ff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4721ff4, 76 },
    { 0, 0 },
    { 0x68f21ff6, 61 },
    { 0x74421ff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf21ffe, 90 },
    { 0x62321ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f22003, 96 },
    { 0xc5522004, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35922008, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf822200b, 61 },
    { 0xbef2200c, 23 },
    { 0x6ee2200d, 93 },
    { 0x9f2200c, 46 },
    { 0xd8f2200f, 23 },
    { 0x58f22010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16022013, 98 },
    { 0x28d22014, 90 },
    { 0xd4822015, 78 },
    { 0x8ae22016, 85 },
    { 0, 0 },
    { 0x9822018, 98 },
    { 0, 0 },
    { 0x4a72201a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf12201d, 61 },
    { 0x732201e, 38 },
    { 0, 0 },
    { 0x66522020, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe22027, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c2202a, 95 },
    { 0xe032202b, 33 },
    { 0xb172202b, 99 },
    { 0xb862202b, 90 },
    { 0x2952202b, 93 },
    { 0xe922202e, 55 },
    { 0xf042202a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d22033, 43 },
    { 0, 0 },
    { 0x3a122035, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e22038, 17 },
    { 0xab922039, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b42203c, 98 },
    { 0xf832203d, 95 },
    { 0x71b2203c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e522041, 91 },
    { 0x49622042, 95 },
    { 0x91022043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c2204b, 90 },
    { 0x3cc2204b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2204f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d22052, 95 },
    { 0x322053, 65 },
    { 0xb8e22054, 76 },
    { 0, 0 },
    { 0x34c22056, 98 },
    { 0x5af22056, 33 },
    { 0x70b22058, 46 },
    { 0xa2022058, 34 },
    { 0, 0 },
    { 0xb6f2205b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9322205e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f822061, 84 },
    { 0xd22061, 34 },
    { 0, 0 },
    { 0x83622064, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3622069, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe22070, 85 },
    { 0x66222071, 88 },
    { 0, 0 },
    { 0xbd222073, 76 },
    { 0, 0 },
    { 0x69c22075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e2207e, 90 },
    { 0, 0 },
    { 0xcbd22080, 61 },
    { 0, 0 },
    { 0xe3022082, 46 },
    { 0, 0 },
    { 0x90522084, 90 },
    { 0, 0 },
    { 0x53222086, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6022090, 76 },
    { 0xa3e22091, 55 },
    { 0xe6d22091, 75 },
    { 0xb5722093, 90 },
    { 0, 0 },
    { 0x4dc22095, 46 },
    { 0x69622096, 60 },
    { 0x15122097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b72209a, 67 },
    { 0x7f32209b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637220a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a220a7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a220aa, 90 },
    { 0x9ed220ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2220af, 55 },
    { 0x5b220af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6220b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b220ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b220bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x378220c0, 98 },
    { 0xbed220c0, 33 },
    { 0x657220c2, 90 },
    { 0, 0 },
    { 0xe0f220c4, 93 },
    { 0xf1a220c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7220cb, 91 },
    { 0x277220cc, 61 },
    { 0xfbe220cd, 33 },
    { 0x56b220cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x327220d1, 98 },
    { 0x18b220d2, 85 },
    { 0x2ae220d3, 23 },
    { 0xf86220d1, 77 },
    { 0x421220d3, 95 },
    { 0x65b220d6, 33 },
    { 0x506220d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x456220da, 34 },
    { 0, 0 },
    { 0x241220dc, 61 },
    { 0x9c6220dd, 23 },
    { 0xe2f220dd, 84 },
    { 0, 0 },
    { 0x442220e0, 46 },
    { 0x9a7220e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27220e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4220ec, 33 },
    { 0, 0 },
    { 0x97a220ee, 90 },
    { 0, 0 },
    { 0x5d0220f0, 98 },
    { 0, 0 },
    { 0x4cc220f2, 99 },
    { 0, 0 },
    { 0x424220f4, 97 },
    { 0, 0 },
    { 0xa0f220f6, 98 },
    { 0x68d220f7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487220fb, 76 },
    { 0xcf7220fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x188220ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb22107, 43 },
    { 0xfab22107, 91 },
    { 0, 0 },
    { 0x2a2210a, 33 },
    { 0xd7d2210b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc2210f, 55 },
    { 0x65822110, 33 },
    { 0x9022111, 61 },
    { 0, 0 },
    { 0xe9722113, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b222116, 90 },
    { 0xc6a22117, 98 },
    { 0x98b22116, 90 },
    { 0x71122119, 23 },
    { 0x79e2211a, 95 },
    { 0, 0 },
    { 0xef72211c, 95 },
    { 0, 0 },
    { 0xdd02211e, 61 },
    { 0, 0 },
    { 0x58d22120, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb22128, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae2212b, 90 },
    { 0xdac2212b, 90 },
    { 0, 0 },
    { 0x2a32212e, 90 },
    { 0xcc92212e, 90 },
    { 0, 0 },
    { 0x27d22131, 95 },
    { 0, 0 },
    { 0x4a822133, 33 },
    { 0, 0 },
    { 0xd0522135, 76 },
    { 0, 0 },
    { 0xfee22137, 55 },
    { 0, 0 },
    { 0x61b22139, 23 },
    { 0x4572213a, 90 },
    { 0xb9c2213a, 90 },
    { 0x62e2213c, 61 },
    { 0xf3e2213c, 90 },
    { 0x6f022139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd22142, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18822146, 33 },
    { 0xac922147, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a2214b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21922152, 23 },
    { 0xff122153, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52322156, 90 },
    { 0x34222156, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa22215e, 55 },
    { 0xbb72215f, 90 },
    { 0x9ee2215f, 55 },
    { 0xe2422160, 55 },
    { 0x6ce22160, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56422165, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde422169, 33 },
    { 0, 0 },
    { 0x2512216b, 33 },
    { 0, 0 },
    { 0xefd2216d, 76 },
    { 0x9102216e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62722173, 90 },
    { 0x99f22173, 23 },
    { 0xa0522175, 76 },
    { 0xf8b22176, 90 },
    { 0xd3322176, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf72217c, 33 },
    { 0x26b2217c, 61 },
    { 0x12c2217e, 95 },
    { 0xe952217f, 61 },
    { 0x9ee22180, 98 },
    { 0x1382217d, 34 },
    { 0x22322182, 76 },
    { 0xb2f22183, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde922187, 89 },
    { 0x86722188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1072218b, 90 },
    { 0xf002218c, 33 },
    { 0, 0 },
    { 0x2022218e, 90 },
    { 0, 0 },
    { 0xd9422190, 90 },
    { 0x8a222190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88222195, 76 },
    { 0x6e22196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc452219c, 68 },
    { 0x772219d, 84 },
    { 0x9752219d, 34 },
    { 0x2872219f, 23 },
    { 0x2219f, 33 },
    { 0x66d221a1, 61 },
    { 0x3842219d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a221a5, 95 },
    { 0x8ed221a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d221aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8221ad, 98 },
    { 0, 0 },
    { 0xef4221af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe221b2, 91 },
    { 0x917221b3, 55 },
    { 0xb66221b2, 90 },
    { 0, 0 },
    { 0xac3221b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c221b9, 46 },
    { 0x2f6221ba, 36 },
    { 0, 0 },
    { 0x5ba221bc, 66 },
    { 0x25c221bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x431221c0, 84 },
    { 0, 0 },
    { 0xb2e221c2, 23 },
    { 0xc58221c3, 55 },
    { 0xb37221c4, 33 },
    { 0xb74221c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a221c8, 55 },
    { 0x7a5221c9, 17 },
    { 0xcb6221c9, 52 },
    { 0, 0 },
    { 0x468221cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b221d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e221d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1221e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab221e7, 76 },
    { 0xb96221e8, 95 },
    { 0, 0 },
    { 0x7a5221ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e221ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579221f5, 99 },
    { 0x980221f5, 93 },
    { 0xa51221f7, 90 },
    { 0, 0 },
    { 0x5ad221f9, 95 },
    { 0, 0 },
    { 0x1f6221fb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58122205, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x47922208, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce52220d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35622213, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352221c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e22221, 95 },
    { 0, 0 },
    { 0x27822223, 68 },
    { 0x12e22224, 33 },
    { 0x5b622224, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b22228, 43 },
    { 0xfd822229, 55 },
    { 0xff72222a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c2222d, 61 },
    { 0, 0 },
    { 0x5c42222f, 61 },
    { 0x16722230, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b22234, 33 },
    { 0x5c822234, 90 },
    { 0xda022236, 90 },
    { 0x13922237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7462223f, 52 },
    { 0x73f22240, 33 },
    { 0xa2b22240, 95 },
    { 0x4c02223f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c022247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5792224b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b22250, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42822253, 78 },
    { 0, 0 },
    { 0xd3422255, 23 },
    { 0x8b622255, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd052225a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a22262, 34 },
    { 0x16e22263, 76 },
    { 0x57d22263, 76 },
    { 0x42722262, 61 },
    { 0x18622266, 98 },
    { 0x36222266, 91 },
    { 0xfdd22267, 87 },
    { 0xa2422269, 95 },
    { 0, 0 },
    { 0x9602226b, 90 },
    { 0x6062226c, 91 },
    { 0, 0 },
    { 0x7712226e, 95 },
    { 0x622226e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d222272, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb222276, 66 },
    { 0x1cf22276, 65 },
    { 0xa4d22276, 33 },
    { 0xeee22279, 52 },
    { 0x3702227a, 23 },
    { 0xa132227b, 55 },
    { 0xe7f2227b, 61 },
    { 0, 0 },
    { 0xc582227e, 48 },
    { 0x3932227f, 84 },
    { 0, 0 },
    { 0x71c22281, 99 },
    { 0, 0 },
    { 0x4422283, 23 },
    { 0x90b22284, 90 },
    { 0, 0 },
    { 0xcea22286, 33 },
    { 0x41922287, 90 },
    { 0x3f622287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2732228b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb2228e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40122291, 36 },
    { 0x6e822292, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad2229b, 76 },
    { 0xa552229c, 43 },
    { 0xb922229b, 33 },
    { 0x1a02229b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45222a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x238222a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406222ae, 43 },
    { 0, 0 },
    { 0xf9c222b0, 90 },
    { 0x142222b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb222b6, 90 },
    { 0x946222b7, 33 },
    { 0x2ea222b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59222bf, 23 },
    { 0x2222c0, 52 },
    { 0xd6c222bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11222c4, 78 },
    { 0, 0 },
    { 0x6de222c6, 23 },
    { 0, 0 },
    { 0xd62222c8, 90 },
    { 0, 0 },
    { 0x943222ca, 61 },
    { 0x4c6222cb, 55 },
    { 0xf63222cc, 76 },
    { 0xb81222cd, 61 },
    { 0, 0 },
    { 0x18f222cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x757222d2, 61 },
    { 0x47b222d3, 42 },
    { 0xdb5222d4, 100 },
    { 0x376222d5, 55 },
    { 0x24222d6, 33 },
    { 0xbda222d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3222df, 61 },
    { 0xaf0222e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68222e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206222e8, 98 },
    { 0x466222e9, 55 },
    { 0x651222e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526222f0, 55 },
    { 0, 0 },
    { 0xdce222f2, 23 },
    { 0x56d222f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98222f6, 90 },
    { 0, 0 },
    { 0x857222f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58222fe, 90 },
    { 0xc9c222ff, 93 },
    { 0, 0 },
    { 0xb8722301, 69 },
    { 0x59722302, 61 },
    { 0x42922303, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2230c, 33 },
    { 0xba2230d, 78 },
    { 0x4262230d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d22311, 76 },
    { 0, 0 },
    { 0x26c22313, 43 },
    { 0, 0 },
    { 0xf1c22315, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8502231b, 33 },
    { 0x5c52231c, 90 },
    { 0x7cd2231d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f22322, 98 },
    { 0, 0 },
    { 0x26822324, 48 },
    { 0xe2d22325, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3022328, 91 },
    { 0xafb22328, 55 },
    { 0, 0 },
    { 0xbb02232b, 95 },
    { 0x33b2232b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef22334, 34 },
    { 0x21222334, 61 },
    { 0xfe722335, 90 },
    { 0xef422337, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2233e, 58 },
    { 0xea12233f, 61 },
    { 0xd1f22340, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf22345, 98 },
    { 0x2a222346, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f52234a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f22350, 67 },
    { 0, 0 },
    { 0xa3622352, 46 },
    { 0, 0 },
    { 0x3a322354, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78622358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd2235f, 95 },
    { 0x2e52235f, 90 },
    { 0, 0 },
    { 0x8b022362, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f52236a, 95 },
    { 0x47b2236a, 100 },
    { 0x14f2236b, 33 },
    { 0x3f2236d, 55 },
    { 0x58e2236e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b22371, 46 },
    { 0x7be22372, 61 },
    { 0xd5b22372, 95 },
    { 0x6722373, 43 },
    { 0, 0 },
    { 0x9e422376, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8122237a, 76 },
    { 0, 0 },
    { 0x6a92237c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5802237f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b22387, 90 },
    { 0x39522388, 90 },
    { 0xf122387, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62238c, 38 },
    { 0x9c62238d, 46 },
    { 0, 0 },
    { 0x63e2238f, 91 },
    { 0x10922390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a22394, 34 },
    { 0, 0 },
    { 0x17522396, 98 },
    { 0x86b22397, 95 },
    { 0xe1322398, 98 },
    { 0xac422399, 33 },
    { 0x3652239a, 34 },
    { 0, 0 },
    { 0xe3c2239c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2239f, 98 },
    { 0x4f62239f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d223a3, 23 },
    { 0xf2223a4, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69223a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff223ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x929223b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a223b4, 55 },
    { 0x716223b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865223b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x360223bc, 33 },
    { 0, 0 },
    { 0xb91223be, 84 },
    { 0xe11223bf, 77 },
    { 0x9ed223be, 33 },
    { 0xe5a223be, 101 },
    { 0xa3b223c2, 98 },
    { 0x4e4223c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881223cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f223d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd223d7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d223db, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85223f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x504223f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797223f7, 23 },
    { 0x19b223f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1223fd, 23 },
    { 0xae7223fe, 66 },
    { 0xdea223ff, 100 },
    { 0xe7222400, 75 },
    { 0xe4222400, 75 },
    { 0xfc222402, 90 },
    { 0x20922403, 66 },
    { 0x84522403, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd22408, 95 },
    { 0x23222408, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf22411, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23822417, 46 },
    { 0xbf222418, 33 },
    { 0xa9722419, 61 },
    { 0x80722417, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3802241e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752242a, 61 },
    { 0, 0 },
    { 0x5162242c, 52 },
    { 0x7392242d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e22431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f22434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95022438, 90 },
    { 0xaab22439, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2243d, 98 },
    { 0, 0 },
    { 0x29b2243f, 76 },
    { 0xbac22440, 34 },
    { 0, 0 },
    { 0xb0522442, 84 },
    { 0, 0 },
    { 0x98422444, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94222448, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8642244d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac022455, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71422458, 33 },
    { 0x39322458, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db2245e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1922462, 98 },
    { 0xc9622463, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3722468, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d92246d, 76 },
    { 0x2892246e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60822472, 91 },
    { 0, 0 },
    { 0x99322474, 87 },
    { 0x76522475, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92247a, 33 },
    { 0, 0 },
    { 0xb032247c, 84 },
    { 0xc242247c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa22480, 61 },
    { 0, 0 },
    { 0xc0522482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf222486, 90 },
    { 0x65922487, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e32248a, 66 },
    { 0x3002248b, 90 },
    { 0x1792248c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c62248f, 33 },
    { 0x3912248f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222493, 33 },
    { 0xf022494, 61 },
    { 0x46322494, 91 },
    { 0x19922496, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352249a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1602249e, 90 },
    { 0, 0 },
    { 0x509224a0, 55 },
    { 0xe3e224a0, 76 },
    { 0, 0 },
    { 0x291224a3, 33 },
    { 0, 0 },
    { 0x947224a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636224aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83224ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82224b1, 34 },
    { 0, 0 },
    { 0xdbc224b3, 88 },
    { 0x3c6224b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678224b8, 84 },
    { 0xe3c224b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea224ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc224cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x525224d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f224d8, 83 },
    { 0, 0 },
    { 0x270224da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c224dd, 85 },
    { 0, 0 },
    { 0x24d224df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c224e5, 33 },
    { 0, 0 },
    { 0x6bc224e7, 61 },
    { 0x50e224e8, 61 },
    { 0x612224e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1224ec, 76 },
    { 0xe2a224ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x949224f0, 34 },
    { 0x272224f0, 34 },
    { 0x5e3224f2, 52 },
    { 0x752224f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f224f6, 93 },
    { 0, 0 },
    { 0xffb224f8, 55 },
    { 0xfee224f9, 90 },
    { 0x919224f9, 61 },
    { 0x615224fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff422500, 95 },
    { 0xb2e22500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b22505, 76 },
    { 0x18422506, 23 },
    { 0x2d222507, 23 },
    { 0x29322507, 43 },
    { 0x38122509, 90 },
    { 0xba52250a, 90 },
    { 0x3a22250b, 65 },
    { 0x4372250a, 23 },
    { 0xd5f22508, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3122511, 36 },
    { 0, 0 },
    { 0x86a22513, 91 },
    { 0x17b22514, 43 },
    { 0, 0 },
    { 0x21c22516, 61 },
    { 0, 0 },
    { 0xd1e22518, 93 },
    { 0x6b822519, 95 },
    { 0x6362251a, 76 },
    { 0x67a2251b, 43 },
    { 0xbaa2251c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6422521, 61 },
    { 0, 0 },
    { 0x50022523, 87 },
    { 0, 0 },
    { 0x4622525, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77222529, 88 },
    { 0x83f2252a, 76 },
    { 0, 0 },
    { 0x9fb2252c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f32252f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40622532, 98 },
    { 0x5d122532, 85 },
    { 0x7422534, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f22538, 100 },
    { 0, 0 },
    { 0x1d82253a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802253d, 43 },
    { 0x7de2253e, 61 },
    { 0, 0 },
    { 0xcd922540, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d022547, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c2254a, 52 },
    { 0xce02254b, 55 },
    { 0xee62254c, 23 },
    { 0x7b02254d, 98 },
    { 0x3322254c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9622557, 90 },
    { 0, 0 },
    { 0x2d822559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd022564, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4442256e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb2257b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e22581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec22587, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd82258a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb2258d, 93 },
    { 0x5fe2258d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f522591, 76 },
    { 0x38422591, 95 },
    { 0x66c22593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34422596, 17 },
    { 0, 0 },
    { 0x93622598, 33 },
    { 0x77222599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f2259e, 66 },
    { 0, 0 },
    { 0x5fa225a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5225a8, 55 },
    { 0, 0 },
    { 0x629225aa, 96 },
    { 0xd56225ab, 78 },
    { 0x48e225ab, 61 },
    { 0x83e225ab, 95 },
    { 0x53a225ad, 91 },
    { 0x663225ab, 43 },
    { 0, 0 },
    { 0xbf7225b1, 67 },
    { 0, 0 },
    { 0xe35225b3, 76 },
    { 0xd225b3, 69 },
    { 0x26a225b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3225b8, 84 },
    { 0x3b225b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d225bd, 91 },
    { 0x174225be, 87 },
    { 0x6ac225bf, 46 },
    { 0xc89225be, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839225ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9225ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e225d2, 98 },
    { 0, 0 },
    { 0x5b0225d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9225db, 43 },
    { 0xa35225dc, 61 },
    { 0xfd7225dd, 55 },
    { 0, 0 },
    { 0x38e225df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f225e8, 52 },
    { 0x956225e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef225ed, 98 },
    { 0x6c5225ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30225f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d225f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b225fc, 87 },
    { 0x6fd225fc, 55 },
    { 0, 0 },
    { 0x3d5225ff, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c22605, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c22609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb722610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd722613, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7722619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99622621, 23 },
    { 0x3422621, 61 },
    { 0xa622622, 76 },
    { 0xd4222624, 95 },
    { 0x4a022625, 100 },
    { 0xf7822626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d822629, 93 },
    { 0x3a02262a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c222632, 100 },
    { 0xf2b22632, 23 },
    { 0x21d22633, 60 },
    { 0xed622635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a922639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8242263c, 98 },
    { 0x9222263d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a522643, 90 },
    { 0, 0 },
    { 0x9ab22645, 99 },
    { 0x2d622646, 89 },
    { 0x47722646, 78 },
    { 0, 0 },
    { 0xafd22649, 80 },
    { 0x7d92264a, 84 },
    { 0x14c2264b, 61 },
    { 0, 0 },
    { 0xbfa2264d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13122652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f22656, 98 },
    { 0x3db22656, 23 },
    { 0xa4522657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a222663, 61 },
    { 0, 0 },
    { 0x81522665, 90 },
    { 0xbd322666, 43 },
    { 0x6c622665, 61 },
    { 0x5c122668, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb32266c, 55 },
    { 0xb9d2266d, 95 },
    { 0, 0 },
    { 0xe862266f, 90 },
    { 0x5562266f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb522674, 85 },
    { 0, 0 },
    { 0x59c22676, 85 },
    { 0, 0 },
    { 0x67422678, 76 },
    { 0x1cd22679, 98 },
    { 0x2b022678, 90 },
    { 0xa6f2267b, 88 },
    { 0xdec2267b, 61 },
    { 0xfea2267c, 91 },
    { 0x9c2267a, 84 },
    { 0x8882267f, 90 },
    { 0x5aa2267f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f22684, 99 },
    { 0xfc122685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2312268a, 90 },
    { 0, 0 },
    { 0x3f82268c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1622690, 90 },
    { 0x21c22690, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x98722694, 90 },
    { 0, 0 },
    { 0xd9522696, 93 },
    { 0, 0 },
    { 0x6ef22698, 55 },
    { 0xb2a22699, 46 },
    { 0xb9a22699, 90 },
    { 0, 0 },
    { 0x7f02269c, 61 },
    { 0xe672269d, 34 },
    { 0xdae2269e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd226a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73226a9, 88 },
    { 0xbba226a9, 52 },
    { 0xb8a226ab, 61 },
    { 0x923226a9, 33 },
    { 0x158226ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba226b5, 33 },
    { 0, 0 },
    { 0x96226b7, 76 },
    { 0x6ec226b8, 43 },
    { 0x675226b9, 43 },
    { 0, 0 },
    { 0x3eb226bb, 95 },
    { 0x5bf226bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8226c0, 95 },
    { 0xa60226c1, 90 },
    { 0xc226c0, 90 },
    { 0x1c8226c2, 84 },
    { 0xe42226c4, 98 },
    { 0, 0 },
    { 0x861226c6, 98 },
    { 0x6a3226c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x686226ca, 98 },
    { 0xd78226cb, 98 },
    { 0x745226cb, 30 },
    { 0xd8f226cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43226d2, 23 },
    { 0x68c226d3, 99 },
    { 0, 0 },
    { 0xc82226d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f226d8, 55 },
    { 0xa69226d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a226dc, 34 },
    { 0x706226dd, 23 },
    { 0x252226dd, 55 },
    { 0, 0 },
    { 0xe24226e0, 51 },
    { 0xfd226e1, 95 },
    { 0xa92226e1, 33 },
    { 0xc9f226e3, 76 },
    { 0xa72226e4, 98 },
    { 0x94b226e5, 95 },
    { 0x727226e6, 76 },
    { 0, 0 },
    { 0x3a1226e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x802226eb, 95 },
    { 0, 0 },
    { 0x4fa226ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a226f1, 23 },
    { 0x66c226f1, 61 },
    { 0, 0 },
    { 0x40b226f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41226fa, 77 },
    { 0, 0 },
    { 0x520226fc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c022701, 23 },
    { 0xe3d22701, 90 },
    { 0x2c222703, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d22706, 34 },
    { 0x67922707, 93 },
    { 0xb7422708, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e22711, 34 },
    { 0x1e722711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2322715, 95 },
    { 0x81e22716, 91 },
    { 0x6222716, 43 },
    { 0x14e22715, 23 },
    { 0xab522716, 95 },
    { 0x19c2271a, 90 },
    { 0xcb2271a, 61 },
    { 0xc0522719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed22720, 100 },
    { 0xe9422721, 46 },
    { 0x43c22722, 76 },
    { 0xf2b22722, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f22729, 52 },
    { 0x9132272a, 88 },
    { 0, 0 },
    { 0x5422272c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec322732, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd222735, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e422738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc572273b, 90 },
    { 0x6f22273b, 90 },
    { 0, 0 },
    { 0x6762273e, 23 },
    { 0x69d2273f, 23 },
    { 0x7302273e, 90 },
    { 0x85f2273e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf022746, 90 },
    { 0x24722747, 61 },
    { 0xb0322748, 52 },
    { 0xe2f22749, 87 },
    { 0xaff2274a, 90 },
    { 0x2062274b, 90 },
    { 0x7272274c, 65 },
    { 0x14e22749, 46 },
    { 0, 0 },
    { 0x6f32274f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2822752, 90 },
    { 0x16322753, 61 },
    { 0xfbb22754, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19722758, 81 },
    { 0x1d922759, 90 },
    { 0, 0 },
    { 0x91b2275b, 98 },
    { 0, 0 },
    { 0xa712275d, 68 },
    { 0x7cb2275e, 90 },
    { 0xb1f2275f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2022763, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d322768, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd2276b, 33 },
    { 0x842276c, 100 },
    { 0xff52276c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52522772, 23 },
    { 0x22222773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe222778, 98 },
    { 0x71322778, 90 },
    { 0xd3e22778, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e22787, 84 },
    { 0, 0 },
    { 0x2ca22789, 33 },
    { 0x21422789, 33 },
    { 0xf172278b, 95 },
    { 0x4f32278c, 93 },
    { 0xa612278d, 90 },
    { 0x1cb2278d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f22793, 99 },
    { 0x37322793, 95 },
    { 0, 0 },
    { 0xe6922796, 61 },
    { 0x13022797, 34 },
    { 0, 0 },
    { 0xa6022799, 46 },
    { 0xa432279a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f227a9, 84 },
    { 0xa82227aa, 98 },
    { 0xc12227aa, 55 },
    { 0, 0 },
    { 0x830227ad, 23 },
    { 0, 0 },
    { 0xd7d227af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78227b6, 100 },
    { 0, 0 },
    { 0xd01227b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a227bb, 91 },
    { 0xa71227bb, 90 },
    { 0x83f227bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8227c1, 33 },
    { 0, 0 },
    { 0xae6227c3, 84 },
    { 0x4aa227c4, 8 },
    { 0x5a1227c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e227c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2227cb, 33 },
    { 0x11c227cc, 46 },
    { 0xe1d227cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff227d0, 34 },
    { 0x2d7227d1, 90 },
    { 0x63c227d2, 55 },
    { 0x95e227d3, 99 },
    { 0x2b6227d4, 91 },
    { 0x4a6227d4, 98 },
    { 0, 0 },
    { 0x8227d7, 19 },
    { 0, 0 },
    { 0x6a5227d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0227de, 67 },
    { 0xfe9227de, 33 },
    { 0x8b5227df, 97 },
    { 0x69f227e1, 33 },
    { 0x453227df, 23 },
    { 0, 0 },
    { 0xa4e227e4, 33 },
    { 0, 0 },
    { 0xbfa227e6, 98 },
    { 0x1bc227e7, 33 },
    { 0, 0 },
    { 0x889227e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1227ee, 23 },
    { 0x498227ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea227f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479227fd, 34 },
    { 0, 0 },
    { 0xde5227ff, 90 },
    { 0x46622800, 55 },
    { 0x781227ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b122807, 23 },
    { 0xccc22808, 90 },
    { 0x32f22809, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c2280f, 43 },
    { 0, 0 },
    { 0x7da22811, 76 },
    { 0, 0 },
    { 0x31922813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40922818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde522823, 33 },
    { 0x71322824, 90 },
    { 0x82222825, 4 },
    { 0x5f922826, 78 },
    { 0x22822827, 34 },
    { 0x20822828, 76 },
    { 0, 0 },
    { 0x1022282a, 33 },
    { 0, 0 },
    { 0x2ad2282c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c922835, 76 },
    { 0xe4b22836, 34 },
    { 0x68e22836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e2283a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb2283f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x11722842, 55 },
    { 0, 0 },
    { 0xfbb22844, 93 },
    { 0xd4d22844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c52284a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f2284d, 98 },
    { 0x8e12284e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a22851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda22855, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa822858, 68 },
    { 0, 0 },
    { 0xcc12285a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c2285d, 98 },
    { 0x1022285e, 95 },
    { 0x20e2285f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c522862, 33 },
    { 0, 0 },
    { 0xda122864, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f2286b, 61 },
    { 0xf242286c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b22872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71122881, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b422884, 69 },
    { 0x77622885, 55 },
    { 0xad922885, 98 },
    { 0xc9622887, 90 },
    { 0, 0 },
    { 0xc022889, 33 },
    { 0xefc2288a, 76 },
    { 0, 0 },
    { 0xd532288c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5522892, 61 },
    { 0, 0 },
    { 0xe0922894, 52 },
    { 0x9dc22895, 65 },
    { 0xe4a22894, 33 },
    { 0x6522896, 90 },
    { 0xaeb22897, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876228a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c228a8, 91 },
    { 0x6ec228a9, 91 },
    { 0, 0 },
    { 0x5c7228ab, 74 },
    { 0, 0 },
    { 0xbf9228ad, 76 },
    { 0, 0 },
    { 0x5e4228af, 98 },
    { 0x1ac228b0, 61 },
    { 0xd93228af, 33 },
    { 0, 0 },
    { 0xac4228b3, 36 },
    { 0x9e2228b3, 98 },
    { 0, 0 },
    { 0x9ba228b6, 61 },
    { 0, 0 },
    { 0x36b228b8, 61 },
    { 0x79c228b8, 33 },
    { 0x57d228ba, 61 },
    { 0x89d228bb, 77 },
    { 0x7b3228b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea228c1, 40 },
    { 0, 0 },
    { 0x7e7228c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902228c8, 52 },
    { 0x60a228c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5228cc, 61 },
    { 0x42b228cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43228d0, 61 },
    { 0, 0 },
    { 0x192228d2, 98 },
    { 0x9c7228d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5228d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5228da, 85 },
    { 0x4b4228db, 33 },
    { 0x71a228db, 96 },
    { 0, 0 },
    { 0x3b7228de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x926228e1, 61 },
    { 0xc59228e2, 23 },
    { 0x6ec228e3, 61 },
    { 0x897228e4, 75 },
    { 0xb9d228e5, 37 },
    { 0x638228e3, 90 },
    { 0xd21228e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257228ed, 95 },
    { 0, 0 },
    { 0x1ad228ef, 95 },
    { 0, 0 },
    { 0x269228f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70228f8, 33 },
    { 0, 0 },
    { 0xab4228fa, 90 },
    { 0, 0 },
    { 0x8f1228fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1822900, 98 },
    { 0xb0122901, 66 },
    { 0x7b222902, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c2290a, 100 },
    { 0xc4d2290a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7412290e, 93 },
    { 0x6e32290f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e22916, 33 },
    { 0xfdf22917, 33 },
    { 0x9c322918, 38 },
    { 0x68522919, 33 },
    { 0x95d22917, 76 },
    { 0x7fd2291b, 93 },
    { 0xb8e2291a, 93 },
    { 0, 0 },
    { 0xa62291e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a222921, 98 },
    { 0x6d22922, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae2292c, 95 },
    { 0x1352292d, 90 },
    { 0x7002292e, 52 },
    { 0xe432292f, 84 },
    { 0xb0922930, 95 },
    { 0, 0 },
    { 0xeed22932, 93 },
    { 0x12622933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2922937, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62293d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b22941, 61 },
    { 0, 0 },
    { 0x3f922943, 98 },
    { 0xce122943, 95 },
    { 0x87222945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a2294c, 93 },
    { 0x57b2294d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93222956, 95 },
    { 0, 0 },
    { 0x4122958, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d922967, 90 },
    { 0x96922968, 88 },
    { 0, 0 },
    { 0x1e2296a, 33 },
    { 0, 0 },
    { 0x1ba2296c, 61 },
    { 0xa552296d, 67 },
    { 0x96f2296c, 33 },
    { 0x3a02296f, 98 },
    { 0x5d82296f, 76 },
    { 0x52122971, 53 },
    { 0x97222971, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed222976, 43 },
    { 0xf22977, 90 },
    { 0x86122978, 34 },
    { 0x5fd22977, 90 },
    { 0, 0 },
    { 0x6972297b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa162297f, 66 },
    { 0xcf42297f, 34 },
    { 0, 0 },
    { 0x7d322982, 99 },
    { 0x4b322982, 61 },
    { 0xd7122984, 55 },
    { 0xd5722983, 34 },
    { 0x9c922985, 33 },
    { 0xea722987, 66 },
    { 0x4df22988, 81 },
    { 0x2d722988, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722998, 23 },
    { 0x32622998, 33 },
    { 0x8832299a, 43 },
    { 0, 0 },
    { 0x1f12299c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee2299f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291229a5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8229aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb229b4, 23 },
    { 0x754229b5, 52 },
    { 0x9fb229b5, 61 },
    { 0xe71229b7, 98 },
    { 0x6de229b8, 34 },
    { 0x17e229b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee229c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177229c6, 61 },
    { 0xe2d229c7, 33 },
    { 0, 0 },
    { 0x3f9229c9, 95 },
    { 0x9bd229c9, 23 },
    { 0x27f229ca, 90 },
    { 0x2b1229cc, 23 },
    { 0x59d229cd, 76 },
    { 0xb01229cd, 97 },
    { 0x6db229cd, 95 },
    { 0, 0 },
    { 0x58a229d1, 61 },
    { 0xef4229d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430229d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684229dc, 76 },
    { 0x5aa229dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe229e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423229ea, 43 },
    { 0, 0 },
    { 0xcad229ec, 65 },
    { 0, 0 },
    { 0xede229ee, 90 },
    { 0x6ea229ee, 98 },
    { 0xfa1229ee, 33 },
    { 0, 0 },
    { 0xa5229f2, 90 },
    { 0x39229f3, 61 },
    { 0x25b229f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb229f8, 95 },
    { 0, 0 },
    { 0xcba229fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888229fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e22a03, 75 },
    { 0xb0222a04, 61 },
    { 0xccf22a05, 33 },
    { 0x9af22a06, 86 },
    { 0, 0 },
    { 0xacf22a08, 23 },
    { 0x2ec22a09, 99 },
    { 0x91622a0a, 76 },
    { 0, 0 },
    { 0x72222a0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb22a1a, 90 },
    { 0x6a322a1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62022a1f, 98 },
    { 0x70522a1f, 76 },
    { 0, 0 },
    { 0xb1922a22, 23 },
    { 0x1d522a22, 34 },
    { 0xe5a22a24, 90 },
    { 0, 0 },
    { 0x66f22a26, 55 },
    { 0x92f22a26, 69 },
    { 0x73622a28, 61 },
    { 0xe2b22a26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30422a30, 61 },
    { 0x80922a30, 33 },
    { 0x25722a32, 99 },
    { 0x6be22a33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae22a39, 55 },
    { 0, 0 },
    { 0x2322a3b, 78 },
    { 0, 0 },
    { 0xade22a3d, 100 },
    { 0xf5222a3d, 61 },
    { 0, 0 },
    { 0x8122a40, 95 },
    { 0x88822a40, 17 },
    { 0x2b22a42, 81 },
    { 0xfc522a42, 35 },
    { 0xf0422a42, 33 },
    { 0, 0 },
    { 0x50422a46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54922a56, 77 },
    { 0, 0 },
    { 0xdf622a58, 61 },
    { 0xd4422a59, 55 },
    { 0x42122a59, 93 },
    { 0, 0 },
    { 0x3fa22a5c, 23 },
    { 0, 0 },
    { 0xa22a5e, 61 },
    { 0, 0 },
    { 0x37322a60, 33 },
    { 0x23a22a60, 33 },
    { 0x42c22a62, 33 },
    { 0, 0 },
    { 0xde422a64, 81 },
    { 0x62122a65, 17 },
    { 0, 0 },
    { 0x57422a67, 76 },
    { 0, 0 },
    { 0x29722a69, 56 },
    { 0x15922a6a, 33 },
    { 0xd7222a6b, 90 },
    { 0, 0 },
    { 0x20722a6d, 95 },
    { 0x75022a6e, 33 },
    { 0xc8922a6f, 33 },
    { 0xae022a70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71922a74, 90 },
    { 0x83b22a75, 81 },
    { 0x16222a75, 66 },
    { 0x7be22a77, 33 },
    { 0, 0 },
    { 0xa4022a79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c22a7d, 61 },
    { 0xd8e22a7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5622a82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f822a88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd22a8b, 91 },
    { 0, 0 },
    { 0x93e22a8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb22a92, 90 },
    { 0x6b222a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed22a98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c22a9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd122a9f, 61 },
    { 0x4ef22a9f, 33 },
    { 0x6fb22aa1, 100 },
    { 0x74522aa2, 90 },
    { 0xc8822aa1, 95 },
    { 0xb4222aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d22aad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5522ab2, 34 },
    { 0xde022ab3, 23 },
    { 0xb022ab4, 52 },
    { 0, 0 },
    { 0x8ee22ab6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b22aba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed722ac4, 90 },
    { 0x56d22ac5, 43 },
    { 0x98822ac4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19422acb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9622ad0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff22ad5, 90 },
    { 0x90522ad6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2322add, 23 },
    { 0x9c22ade, 23 },
    { 0xfc822adf, 100 },
    { 0, 0 },
    { 0x28a22ae1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be22ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81522ae8, 93 },
    { 0xbaf22ae9, 55 },
    { 0x90622aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c22aed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74522af2, 91 },
    { 0xaea22af3, 100 },
    { 0x5a822af3, 99 },
    { 0x23022af5, 90 },
    { 0x1d722af5, 33 },
    { 0, 0 },
    { 0x19922af8, 23 },
    { 0x32022af8, 95 },
    { 0x3cc22afa, 52 },
    { 0, 0 },
    { 0xce322afc, 98 },
    { 0x44522afd, 95 },
    { 0xdfe22afd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90722b05, 93 },
    { 0, 0 },
    { 0x8e022b07, 93 },
    { 0xb4422b08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6022b0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba22b0e, 90 },
    { 0x77122b0e, 90 },
    { 0x85322b10, 87 },
    { 0x9b922b10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5322b15, 93 },
    { 0x6ee22b15, 93 },
    { 0, 0 },
    { 0xab22b18, 93 },
    { 0x4f322b19, 87 },
    { 0x31f22b1a, 66 },
    { 0xe6b22b1b, 23 },
    { 0x40522b1c, 23 },
    { 0xf5322b1c, 43 },
    { 0x6dd22b1c, 61 },
    { 0x3cd22b1f, 23 },
    { 0x32f22b20, 23 },
    { 0x6f922b1c, 34 },
    { 0, 0 },
    { 0x2e022b23, 43 },
    { 0x5ed22b24, 43 },
    { 0xd9822b25, 64 },
    { 0xfec22b25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d622b2d, 91 },
    { 0xaae22b2d, 90 },
    { 0x29222b2f, 98 },
    { 0x5de22b2f, 90 },
    { 0xb9922b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e422b38, 95 },
    { 0xcc722b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c622b3c, 34 },
    { 0, 0 },
    { 0xc1722b3e, 33 },
    { 0xa3522b3f, 23 },
    { 0x71a22b3f, 90 },
    { 0x8fa22b3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1422b45, 53 },
    { 0x50522b46, 87 },
    { 0xe422b45, 61 },
    { 0, 0 },
    { 0x9c822b49, 46 },
    { 0x4622b49, 34 },
    { 0xdf322b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d22b55, 78 },
    { 0xe4322b56, 33 },
    { 0x2a22b57, 90 },
    { 0xee522b57, 90 },
    { 0x26722b57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed22b60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43322b66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74622b6d, 95 },
    { 0x1fa22b6e, 61 },
    { 0xfe022b6f, 90 },
    { 0xe6a22b6f, 98 },
    { 0x35a22b6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9122b74, 67 },
    { 0xb8c22b74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a22b78, 33 },
    { 0x61a22b79, 99 },
    { 0, 0 },
    { 0x11422b7b, 33 },
    { 0xfd122b7b, 91 },
    { 0x50f22b7c, 98 },
    { 0, 0 },
    { 0xfd22b7f, 90 },
    { 0xdb522b7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa22b83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca22b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97022b90, 61 },
    { 0x51f22b91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0422b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf22b97, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed122b9b, 46 },
    { 0x2e022b9c, 76 },
    { 0x2b222b9c, 100 },
    { 0xb822b9e, 76 },
    { 0, 0 },
    { 0x4622ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb22ba3, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c22ba9, 90 },
    { 0, 0 },
    { 0xd5b22bab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e722bb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb322bb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f22bbc, 52 },
    { 0x63522bbc, 61 },
    { 0xf6822bbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80922bc2, 33 },
    { 0x1d522bc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58122bc8, 78 },
    { 0, 0 },
    { 0x99422bca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9022bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae522bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22bd4, 33 },
    { 0, 0 },
    { 0x3dc22bd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c922bd9, 78 },
    { 0, 0 },
    { 0x6d922bdb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a22be3, 84 },
    { 0x1b222be4, 78 },
    { 0, 0 },
    { 0xc4522be6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef822beb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b22bef, 90 },
    { 0xbb922bef, 72 },
    { 0xe7d22bf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60122bf4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe22bf7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e922bfc, 76 },
    { 0, 0 },
    { 0x78d22bfe, 85 },
    { 0xc6a22bfe, 81 },
    { 0xd8b22bfe, 33 },
    { 0xf5c22bff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12222c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc22c0c, 17 },
    { 0, 0 },
    { 0x88822c0e, 91 },
    { 0xf6722c0f, 33 },
    { 0xa0c22c10, 65 },
    { 0, 0 },
    { 0x91422c12, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df22c19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc222c1c, 98 },
    { 0x6a722c1c, 76 },
    { 0x6f422c1e, 90 },
    { 0x5a022c1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68522c24, 95 },
    { 0x5bd22c24, 90 },
    { 0x1a322c26, 95 },
    { 0xf522c27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x32422c2a, 95 },
    { 0x74422c2b, 23 },
    { 0x74022c2c, 90 },
    { 0x25022c2d, 90 },
    { 0xb1922c2c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53022c32, 23 },
    { 0xf6822c33, 98 },
    { 0x6f322c33, 85 },
    { 0, 0 },
    { 0xe7222c36, 61 },
    { 0, 0 },
    { 0xf6622c38, 61 },
    { 0x82e22c39, 85 },
    { 0xa7122c3a, 97 },
    { 0, 0 },
    { 0xa6d22c3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88222c47, 91 },
    { 0, 0 },
    { 0x87d22c49, 77 },
    { 0xb8822c4a, 52 },
    { 0, 0 },
    { 0xff922c4c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b22c50, 90 },
    { 0, 0 },
    { 0xa9022c52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x38122c55, 95 },
    { 0xa3c22c56, 95 },
    { 0, 0 },
    { 0x19b22c58, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f422c5e, 67 },
    { 0x3c322c5f, 33 },
    { 0x15122c5f, 91 },
    { 0xb3022c61, 23 },
    { 0x36e22c61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb922c66, 55 },
    { 0x44522c67, 61 },
    { 0xfd622c68, 99 },
    { 0, 0 },
    { 0xeb022c6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c022c6d, 85 },
    { 0, 0 },
    { 0xbdb22c6f, 23 },
    { 0x60122c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec822c75, 52 },
    { 0xe6422c75, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46822c7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0622c88, 33 },
    { 0xb2222c89, 95 },
    { 0x31122c8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20822c8e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2422c92, 95 },
    { 0x56c22c93, 98 },
    { 0x33522c93, 33 },
    { 0, 0 },
    { 0x54422c96, 55 },
    { 0xfb822c97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71822c9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69922ca4, 23 },
    { 0xdea22ca5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc22ca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89822cae, 78 },
    { 0x26422cae, 90 },
    { 0x7c122cb0, 100 },
    { 0x55e22cb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9122cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7022cb9, 90 },
    { 0, 0 },
    { 0x9e322cbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f522cbf, 17 },
    { 0x35122cbf, 90 },
    { 0, 0 },
    { 0x15c22cc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9722cc6, 43 },
    { 0, 0 },
    { 0xd1822cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc22cce, 95 },
    { 0xace22ccf, 98 },
    { 0x60122cd0, 95 },
    { 0x22322ccf, 99 },
    { 0x49d22cd2, 23 },
    { 0, 0 },
    { 0x36022cd4, 66 },
    { 0xb8a22cd5, 61 },
    { 0xfd622cd4, 95 },
    { 0xc8722cd7, 90 },
    { 0x40622cd8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56222cde, 33 },
    { 0x49522cde, 66 },
    { 0, 0 },
    { 0x2e722ce1, 66 },
    { 0x65c22ce2, 33 },
    { 0x81022ce2, 85 },
    { 0xfc722ce4, 95 },
    { 0xdc322ce5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d222ce8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27222cef, 34 },
    { 0xa2722cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1922cf4, 61 },
    { 0, 0 },
    { 0x95f22cf6, 67 },
    { 0xebd22cf7, 68 },
    { 0x7de22cf8, 40 },
    { 0x9922cf8, 87 },
    { 0x91922cf6, 34 },
    { 0x5a822cfb, 34 },
    { 0x85722cfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xade22cff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7022d03, 78 },
    { 0xe322d04, 33 },
    { 0, 0 },
    { 0x8c822d06, 90 },
    { 0xb1522d06, 82 },
    { 0, 0 },
    { 0xb3c22d09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2722d0d, 17 },
    { 0, 0 },
    { 0xc3822d0f, 95 },
    { 0x77722d0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db22d13, 90 },
    { 0xe6d22d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b22d18, 61 },
    { 0x58322d18, 90 },
    { 0, 0 },
    { 0x8a522d1b, 76 },
    { 0, 0 },
    { 0xbfa22d1d, 46 },
    { 0, 0 },
    { 0x83822d1f, 17 },
    { 0x43922d20, 95 },
    { 0x7ed22d21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d922d26, 33 },
    { 0xb2722d27, 65 },
    { 0x65a22d26, 98 },
    { 0x29222d26, 95 },
    { 0x36e22d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f22d2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93622d35, 78 },
    { 0xadf22d36, 53 },
    { 0xe1f22d36, 33 },
    { 0x39b22d38, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0222d3c, 99 },
    { 0, 0 },
    { 0x5af22d3e, 52 },
    { 0x7a922d3f, 98 },
    { 0x7522d40, 98 },
    { 0, 0 },
    { 0x66422d42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422d46, 34 },
    { 0xe7222d46, 77 },
    { 0x64b22d48, 38 },
    { 0, 0 },
    { 0xe3f22d4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b22d4d, 61 },
    { 0, 0 },
    { 0x86622d4f, 66 },
    { 0, 0 },
    { 0x38322d51, 100 },
    { 0, 0 },
    { 0x5f222d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a22d57, 90 },
    { 0, 0 },
    { 0x26322d59, 52 },
    { 0xf6422d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f922d5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b922d65, 33 },
    { 0, 0 },
    { 0x32222d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d22d6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7222d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f622d73, 23 },
    { 0xcf022d74, 68 },
    { 0x6b422d73, 98 },
    { 0xcea22d76, 33 },
    { 0x7f222d77, 88 },
    { 0xa8522d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d22d7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d22d7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b22d86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd422d8b, 90 },
    { 0xc8722d8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1222d91, 65 },
    { 0x91d22d91, 55 },
    { 0, 0 },
    { 0xa9a22d94, 87 },
    { 0x26f22d94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e22d98, 61 },
    { 0, 0 },
    { 0x3b722d9a, 33 },
    { 0xac422d9b, 76 },
    { 0x13822d9c, 68 },
    { 0, 0 },
    { 0x34222d9e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77822da8, 33 },
    { 0x24322da8, 55 },
    { 0x9822daa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3722db4, 36 },
    { 0x7fb22db4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea22db9, 95 },
    { 0, 0 },
    { 0x2b822dbb, 76 },
    { 0xc8f22dbc, 90 },
    { 0xd5922dbc, 33 },
    { 0x51e22dbb, 61 },
    { 0, 0 },
    { 0xa2e22dc0, 33 },
    { 0x9022dc1, 61 },
    { 0xce822dc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b22dc9, 98 },
    { 0x5ed22dca, 34 },
    { 0xa5122dcb, 33 },
    { 0x5c122dca, 66 },
    { 0, 0 },
    { 0x44f22dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d22dd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66622dd8, 34 },
    { 0x8f522dd9, 90 },
    { 0, 0 },
    { 0x18022ddb, 90 },
    { 0, 0 },
    { 0x40a22ddd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6122de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1422de5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f22de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a22dec, 76 },
    { 0x5f522ded, 95 },
    { 0x7b122dee, 90 },
    { 0, 0 },
    { 0x9c922df0, 23 },
    { 0, 0 },
    { 0xf7d22df2, 76 },
    { 0xfd822df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e22df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a22dfb, 78 },
    { 0xd0822dfc, 81 },
    { 0xd4422dfc, 67 },
    { 0, 0 },
    { 0x36922dff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed322e02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a22e05, 77 },
    { 0x96e22e06, 43 },
    { 0x9f122e07, 90 },
    { 0xa2722e08, 55 },
    { 0, 0 },
    { 0x6aa22e0a, 65 },
    { 0xada22e0a, 95 },
    { 0x65322e0c, 46 },
    { 0x63f22e0a, 33 },
    { 0, 0 },
    { 0xe3f22e0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32122e13, 33 },
    { 0x822e13, 90 },
    { 0, 0 },
    { 0x60c22e16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92422e1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf922e1e, 55 },
    { 0x22722e1f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4222e22, 46 },
    { 0x53222e23, 61 },
    { 0x31122e23, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e22e27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f222e2b, 46 },
    { 0, 0 },
    { 0xb4822e2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb322e30, 46 },
    { 0xb5822e30, 90 },
    { 0xa0e22e32, 23 },
    { 0xd1322e31, 90 },
    { 0, 0 },
    { 0xf2622e35, 93 },
    { 0x11b22e35, 33 },
    { 0x61922e36, 46 },
    { 0x61822e35, 23 },
    { 0xe0d22e35, 55 },
    { 0xcc522e35, 93 },
    { 0x35222e39, 90 },
    { 0, 0 },
    { 0xef822e3d, 67 },
    { 0x34f22e3e, 90 },
    { 0, 0 },
    { 0x89622e40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0522e43, 46 },
    { 0xecf22e44, 68 },
    { 0, 0 },
    { 0x20c22e46, 95 },
    { 0, 0 },
    { 0x9fe22e48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f422e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c22e4f, 43 },
    { 0, 0 },
    { 0x60522e51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27422e57, 33 },
    { 0, 0 },
    { 0x1be22e59, 90 },
    { 0xeb22e5a, 23 },
    { 0xbd622e59, 94 },
    { 0, 0 },
    { 0x52f22e5d, 98 },
    { 0x13e22e5d, 98 },
    { 0xcd922e5f, 93 },
    { 0x45822e60, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a22e64, 61 },
    { 0, 0 },
    { 0xf0e22e66, 99 },
    { 0xfe022e67, 46 },
    { 0x2bf22e67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d22e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f122e72, 90 },
    { 0xed322e72, 98 },
    { 0x70922e74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f22e7a, 34 },
    { 0x9f622e7b, 61 },
    { 0x2ba22e7c, 95 },
    { 0x7ff22e7c, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42022e83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b22e87, 84 },
    { 0xf6222e87, 43 },
    { 0, 0 },
    { 0xc4c22e8a, 78 },
    { 0xae622e8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc422e8e, 33 },
    { 0x5b522e8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d22e93, 33 },
    { 0, 0 },
    { 0x2e22e95, 23 },
    { 0, 0 },
    { 0xf1422e97, 90 },
    { 0xb6222e97, 66 },
    { 0x56422e98, 61 },
    { 0xeb222e9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d22e9f, 77 },
    { 0, 0 },
    { 0x7c822ea1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29522ea6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e22eaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d22eba, 43 },
    { 0xb7f22ebb, 36 },
    { 0, 0 },
    { 0xc122ebd, 91 },
    { 0x20c22ebd, 90 },
    { 0x30c22ebf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20622ec7, 90 },
    { 0x5a522ec8, 33 },
    { 0x7a822ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46422ecd, 66 },
    { 0x85822ecd, 98 },
    { 0x77822ecf, 33 },
    { 0, 0 },
    { 0xaf822ed1, 90 },
    { 0xe822ed2, 36 },
    { 0xa7722ed2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60822ed8, 61 },
    { 0x8c522ed9, 85 },
    { 0x66322ed9, 61 },
    { 0xb4722edb, 98 },
    { 0, 0 },
    { 0x77522edd, 98 },
    { 0x75922edd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4522ee2, 43 },
    { 0x10d22ee2, 76 },
    { 0, 0 },
    { 0xdd122ee5, 34 },
    { 0xbe022ee6, 40 },
    { 0xd6622ee6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4122eea, 43 },
    { 0x38222eeb, 90 },
    { 0x3e222eec, 91 },
    { 0x80422eec, 61 },
    { 0, 0 },
    { 0x8c122eef, 61 },
    { 0x7b822ef0, 23 },
    { 0, 0 },
    { 0xc8022ef2, 76 },
    { 0x92f22ef3, 88 },
    { 0xb9822ef4, 85 },
    { 0, 0 },
    { 0xe2622ef6, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8522efc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c22eff, 78 },
    { 0, 0 },
    { 0x62422f01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f422f09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8422f0d, 33 },
    { 0, 0 },
    { 0x1df22f0f, 90 },
    { 0x50a22f10, 61 },
    { 0, 0 },
    { 0x6922f12, 77 },
    { 0, 0 },
    { 0x2d522f14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b522f1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf22f1f, 67 },
    { 0, 0 },
    { 0xfc422f21, 98 },
    { 0x3bd22f22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c22f28, 83 },
    { 0x80722f29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a22f30, 78 },
    { 0, 0 },
    { 0x2422f32, 98 },
    { 0xb9522f33, 90 },
    { 0, 0 },
    { 0xe0622f35, 66 },
    { 0, 0 },
    { 0xdae22f37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50522f3b, 55 },
    { 0xfa322f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c22f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad22f44, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e22f48, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c922f53, 61 },
    { 0xf8522f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f22f6a, 81 },
    { 0x50622f6b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb122f6f, 52 },
    { 0xcec22f6f, 34 },
    { 0x52622f71, 55 },
    { 0x5b922f72, 34 },
    { 0, 0 },
    { 0x6922f74, 55 },
    { 0x61522f75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca22f78, 33 },
    { 0x7222f79, 93 },
    { 0, 0 },
    { 0x2d922f7b, 61 },
    { 0, 0 },
    { 0x322f7d, 90 },
    { 0, 0 },
    { 0x53a22f7f, 84 },
    { 0xca22f80, 23 },
    { 0xdd22f81, 97 },
    { 0x37d22f7f, 34 },
    { 0xa1722f83, 23 },
    { 0x9da22f84, 95 },
    { 0x12f22f80, 90 },
    { 0x38122f86, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa522f8b, 95 },
    { 0x79b22f8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22f94, 95 },
    { 0xb3322f95, 87 },
    { 0x12522f95, 90 },
    { 0xefc22f95, 55 },
    { 0, 0 },
    { 0x22222f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56522f9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc222fa2, 61 },
    { 0x76922fa2, 99 },
    { 0x34022fa4, 66 },
    { 0x9bf22fa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c22fa9, 76 },
    { 0x37a22faa, 77 },
    { 0xcfc22faa, 67 },
    { 0x67722fac, 34 },
    { 0xc2822fad, 66 },
    { 0x28422fac, 61 },
    { 0x3fb22fad, 33 },
    { 0, 0 },
    { 0x5f722fb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63922fb9, 34 },
    { 0x7e622fb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1922fc1, 52 },
    { 0, 0 },
    { 0xe9522fc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b222fc6, 76 },
    { 0xe0b22fc7, 55 },
    { 0xa2b22fc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222fcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8422fd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96322fd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86522fd7, 23 },
    { 0, 0 },
    { 0xd1f22fd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea22fdc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a22fe1, 55 },
    { 0x90822fe2, 34 },
    { 0x26122fe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27322fe6, 23 },
    { 0x50822fe6, 76 },
    { 0, 0 },
    { 0x6d222fe9, 17 },
    { 0, 0 },
    { 0x79a22feb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf22fee, 76 },
    { 0, 0 },
    { 0x11022ff0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98822ff3, 100 },
    { 0, 0 },
    { 0xaa322ff5, 90 },
    { 0xef722ff6, 90 },
    { 0xaf22ff5, 23 },
    { 0x6f122ff8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x23122ffb, 87 },
    { 0x92922ffb, 55 },
    { 0, 0 },
    { 0xb4022ffe, 68 },
    { 0, 0 },
    { 0xd7423000, 40 },
    { 0xd1f23001, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad23007, 95 },
    { 0, 0 },
    { 0xea823009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b2300e, 78 },
    { 0, 0 },
    { 0x6a923010, 90 },
    { 0x82f23011, 65 },
    { 0xf8c23012, 90 },
    { 0, 0 },
    { 0x34523014, 36 },
    { 0xf5a23014, 90 },
    { 0, 0 },
    { 0x59f23017, 90 },
    { 0, 0 },
    { 0xef223019, 84 },
    { 0xac12301a, 84 },
    { 0x30b2301a, 90 },
    { 0, 0 },
    { 0x5b32301d, 23 },
    { 0xe5f2301e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9823026, 61 },
    { 0x2f923027, 96 },
    { 0xf1123028, 46 },
    { 0, 0 },
    { 0xdb82302a, 98 },
    { 0x7152302b, 90 },
    { 0xdd12302b, 90 },
    { 0xdcd2302b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4023030, 55 },
    { 0, 0 },
    { 0x25f23032, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4323036, 33 },
    { 0xe4c23037, 61 },
    { 0x89723038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f23045, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f723048, 23 },
    { 0x2ea23048, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f2304e, 33 },
    { 0x5f32304f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa23052, 34 },
    { 0x45523053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e823057, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a623061, 76 },
    { 0x3f323062, 95 },
    { 0x8723061, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6192306b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722306e, 33 },
    { 0x402306f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f23072, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78923078, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca2307e, 99 },
    { 0xb012307e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab23082, 66 },
    { 0xefd23082, 33 },
    { 0, 0 },
    { 0x98a23085, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362308b, 90 },
    { 0x3662308c, 33 },
    { 0, 0 },
    { 0x6ee2308e, 95 },
    { 0, 0 },
    { 0xa9b23090, 52 },
    { 0, 0 },
    { 0xe7d23092, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61123096, 69 },
    { 0xa0123097, 99 },
    { 0xf1923096, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb12309c, 98 },
    { 0xab52309c, 61 },
    { 0xcd62309c, 90 },
    { 0xec2309f, 61 },
    { 0x742309f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x205230a3, 55 },
    { 0xa65230a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x285230a7, 61 },
    { 0x3bf230a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d230b0, 84 },
    { 0x8d0230b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965230b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x182230b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16230c9, 12 },
    { 0xa97230ca, 55 },
    { 0, 0 },
    { 0xc9d230cc, 55 },
    { 0, 0 },
    { 0x513230ce, 76 },
    { 0, 0 },
    { 0xed4230d0, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34230d6, 61 },
    { 0xabb230d7, 91 },
    { 0x8dc230d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe230db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80230e0, 61 },
    { 0, 0 },
    { 0xc76230e2, 78 },
    { 0x185230e2, 93 },
    { 0x32c230e2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c230e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6230eb, 55 },
    { 0, 0 },
    { 0xeb5230ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d230f0, 33 },
    { 0, 0 },
    { 0x312230f2, 90 },
    { 0xa46230f2, 43 },
    { 0xfa230f4, 33 },
    { 0x8a8230f5, 23 },
    { 0x7ba230f6, 8 },
    { 0x2a9230f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e230fd, 77 },
    { 0x67f230fd, 33 },
    { 0, 0 },
    { 0x85723100, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x33723103, 99 },
    { 0x8c223104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51623107, 93 },
    { 0, 0 },
    { 0xaa923109, 34 },
    { 0x9d22310a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x612310d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa623110, 95 },
    { 0xf323110, 43 },
    { 0xeeb23112, 75 },
    { 0xc8123110, 34 },
    { 0x88b23114, 91 },
    { 0x80123115, 93 },
    { 0xc0823113, 61 },
    { 0, 0 },
    { 0x65f23118, 33 },
    { 0xd0723119, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9552311c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf32311f, 23 },
    { 0x3d2311f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c123123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd2312a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5023132, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d323139, 99 },
    { 0x60b2313a, 40 },
    { 0xdd52313a, 23 },
    { 0, 0 },
    { 0x82e2313d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b923144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa23148, 98 },
    { 0xd4b23149, 90 },
    { 0, 0 },
    { 0xcea2314b, 33 },
    { 0x3362314b, 76 },
    { 0x8452314b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf23151, 95 },
    { 0xf9c23152, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72315b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e23160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a223166, 68 },
    { 0x4723167, 61 },
    { 0x8db23168, 65 },
    { 0x96c23169, 87 },
    { 0xfbf2316a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de23171, 76 },
    { 0x8d23172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c023176, 33 },
    { 0, 0 },
    { 0x94823178, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2317b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9332317f, 90 },
    { 0x52b23180, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62723184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd23189, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b2318d, 90 },
    { 0x2662318e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef523192, 52 },
    { 0, 0 },
    { 0x58723194, 33 },
    { 0, 0 },
    { 0xe2e23196, 98 },
    { 0x6ab23197, 55 },
    { 0xfe523197, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d231a1, 23 },
    { 0, 0 },
    { 0x9ff231a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a231a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8231ab, 77 },
    { 0xd50231ac, 61 },
    { 0x647231ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf231b2, 95 },
    { 0, 0 },
    { 0xea231b4, 68 },
    { 0xf5a231b5, 33 },
    { 0xdb4231b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f231be, 78 },
    { 0xb4231be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f231c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162231c8, 23 },
    { 0x62b231c8, 95 },
    { 0xd01231c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c231d0, 76 },
    { 0, 0 },
    { 0xbf6231d2, 33 },
    { 0xa7b231d2, 61 },
    { 0x84f231d4, 23 },
    { 0, 0 },
    { 0xe35231d6, 40 },
    { 0x8c2231d6, 33 },
    { 0x493231d6, 85 },
    { 0x5aa231d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df231dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d231e0, 95 },
    { 0x621231e1, 23 },
    { 0xb16231e2, 33 },
    { 0x810231e3, 90 },
    { 0, 0 },
    { 0xc61231e5, 78 },
    { 0x15e231e6, 99 },
    { 0, 0 },
    { 0x21b231e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a231eb, 33 },
    { 0x62d231ec, 90 },
    { 0xa00231ec, 33 },
    { 0x30d231ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947231f9, 65 },
    { 0, 0 },
    { 0xc89231fb, 17 },
    { 0xb0c231fb, 23 },
    { 0x108231fb, 69 },
    { 0xe78231fb, 38 },
    { 0x204231ff, 68 },
    { 0x76f231fc, 90 },
    { 0, 0 },
    { 0x50123202, 43 },
    { 0xa8923202, 90 },
    { 0, 0 },
    { 0x81f23205, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b923209, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a2320e, 61 },
    { 0, 0 },
    { 0x20123210, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e323214, 76 },
    { 0xa1e23214, 76 },
    { 0xca523216, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f2321d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d23221, 76 },
    { 0x85123221, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf23226, 98 },
    { 0x8d723227, 33 },
    { 0x10c23227, 36 },
    { 0x46823229, 52 },
    { 0xa902322a, 64 },
    { 0, 0 },
    { 0xa052322c, 87 },
    { 0xa8a2322d, 90 },
    { 0x4512322d, 61 },
    { 0x1b42322f, 61 },
    { 0xdc22322f, 76 },
    { 0x10823231, 90 },
    { 0xa632322f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd23236, 38 },
    { 0x8fc23236, 69 },
    { 0, 0 },
    { 0x92023239, 53 },
    { 0xf8f23239, 43 },
    { 0x5252323b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99323246, 93 },
    { 0xce223247, 34 },
    { 0x91223246, 95 },
    { 0x90c23249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d2324d, 98 },
    { 0, 0 },
    { 0x76d2324f, 95 },
    { 0x22b23250, 55 },
    { 0, 0 },
    { 0x20d23252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a723255, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c023258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a2325c, 90 },
    { 0x4e92325d, 33 },
    { 0x7a42325e, 100 },
    { 0xe9c2325f, 98 },
    { 0, 0 },
    { 0xaa323261, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45723266, 61 },
    { 0x80e23267, 98 },
    { 0x98e23266, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e2326d, 90 },
    { 0x13b2326e, 48 },
    { 0x692326f, 55 },
    { 0, 0 },
    { 0xc0023271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a23274, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb023277, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372327b, 43 },
    { 0x8662327b, 23 },
    { 0x28b2327b, 23 },
    { 0x8eb2327e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e23289, 61 },
    { 0x4292328a, 46 },
    { 0xeb32328a, 61 },
    { 0, 0 },
    { 0x45d2328d, 95 },
    { 0xb9a2328e, 33 },
    { 0x2a42328e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0923293, 90 },
    { 0x5d023293, 90 },
    { 0x72323294, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5822329a, 85 },
    { 0xfbe2329b, 77 },
    { 0xb062329c, 95 },
    { 0xe5e2329b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8232a1, 90 },
    { 0x482232a2, 76 },
    { 0xacb232a3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e232a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8232ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x758232ae, 88 },
    { 0, 0 },
    { 0x12d232b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x535232b3, 68 },
    { 0, 0 },
    { 0xe35232b5, 33 },
    { 0x4ce232b5, 91 },
    { 0x75232b7, 76 },
    { 0x1f4232b7, 61 },
    { 0x831232b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7232bd, 90 },
    { 0x397232be, 66 },
    { 0xfb1232bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8232c4, 61 },
    { 0xe90232c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54232c8, 52 },
    { 0x3bf232c9, 65 },
    { 0x78a232ca, 93 },
    { 0xd39232c9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a232d2, 23 },
    { 0xcda232d3, 68 },
    { 0xec2232d3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2232da, 61 },
    { 0x3f232da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad232e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0232e6, 33 },
    { 0x74a232e7, 55 },
    { 0xd72232e8, 23 },
    { 0x247232e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe232ec, 95 },
    { 0x254232ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708232f3, 61 },
    { 0, 0 },
    { 0x575232f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4232f8, 23 },
    { 0x47c232f9, 55 },
    { 0x25f232fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb232fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4923306, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92923312, 23 },
    { 0xa9523312, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1323316, 66 },
    { 0x22923317, 90 },
    { 0, 0 },
    { 0xc8c23319, 95 },
    { 0, 0 },
    { 0xd7d2331b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b32331e, 93 },
    { 0, 0 },
    { 0x63f23320, 61 },
    { 0xe9023321, 34 },
    { 0xce123320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3852332a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad523330, 91 },
    { 0xa5a23331, 33 },
    { 0, 0 },
    { 0x95823333, 43 },
    { 0xdd823333, 95 },
    { 0x10f23334, 61 },
    { 0x85a23335, 95 },
    { 0xb2f23336, 61 },
    { 0, 0 },
    { 0x6eb23339, 99 },
    { 0, 0 },
    { 0x5252333b, 76 },
    { 0xb2f2333c, 76 },
    { 0x2972333d, 90 },
    { 0x6ec2333e, 91 },
    { 0xb742333f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f323342, 90 },
    { 0x3e223343, 88 },
    { 0, 0 },
    { 0x29d23345, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2334e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a23352, 91 },
    { 0, 0 },
    { 0xfe023354, 93 },
    { 0, 0 },
    { 0x45a23356, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b23359, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4772335d, 61 },
    { 0, 0 },
    { 0x57c2335f, 33 },
    { 0x27823360, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8023363, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932336e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e823372, 93 },
    { 0, 0 },
    { 0x72423374, 90 },
    { 0xd3d23375, 90 },
    { 0xc0623375, 98 },
    { 0xa1023377, 95 },
    { 0x6ef23378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e2337e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f423384, 34 },
    { 0x3d123384, 23 },
    { 0, 0 },
    { 0x323387, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa72338c, 47 },
    { 0, 0 },
    { 0x5742338e, 34 },
    { 0, 0 },
    { 0x30323390, 23 },
    { 0x2c223390, 19 },
    { 0xa4423392, 61 },
    { 0xdc423392, 76 },
    { 0x53323394, 23 },
    { 0xd2323393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f323398, 17 },
    { 0x59d23399, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad2339e, 95 },
    { 0xd482339e, 93 },
    { 0x2b5233a0, 98 },
    { 0xe8d233a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2233a7, 43 },
    { 0x11f233a8, 59 },
    { 0x725233a9, 34 },
    { 0xbcc233a9, 81 },
    { 0x253233a8, 43 },
    { 0xfd7233ac, 61 },
    { 0xc28233ad, 33 },
    { 0xef233ab, 76 },
    { 0xe2233ac, 95 },
    { 0x4bc233b0, 67 },
    { 0x63e233b1, 78 },
    { 0xb89233af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9233b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae233bf, 90 },
    { 0, 0 },
    { 0xb38233c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567233c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09233c9, 61 },
    { 0, 0 },
    { 0x632233cb, 85 },
    { 0, 0 },
    { 0x312233cd, 55 },
    { 0x45233ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x958233d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883233d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7233de, 43 },
    { 0, 0 },
    { 0x819233e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6233e3, 68 },
    { 0x788233e4, 33 },
    { 0x85f233e5, 90 },
    { 0xc63233e4, 61 },
    { 0xd2b233e6, 61 },
    { 0xc07233e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb233ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94233f4, 50 },
    { 0x25c233f4, 99 },
    { 0, 0 },
    { 0xd09233f7, 23 },
    { 0, 0 },
    { 0x599233f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb233fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e23405, 90 },
    { 0x11723406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c2340b, 55 },
    { 0x16d2340c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12423414, 99 },
    { 0x7a523415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f23418, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2341f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7823422, 33 },
    { 0xbfd23423, 55 },
    { 0x6a023424, 98 },
    { 0xba323425, 61 },
    { 0x9e123426, 23 },
    { 0xf6623426, 84 },
    { 0xfc923425, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc082342b, 23 },
    { 0, 0 },
    { 0x2ed2342d, 34 },
    { 0x2b42342e, 76 },
    { 0xa872342d, 43 },
    { 0x2a52342d, 17 },
    { 0, 0 },
    { 0xa6223432, 61 },
    { 0, 0 },
    { 0xb6323434, 91 },
    { 0x70023435, 90 },
    { 0x66e23434, 76 },
    { 0, 0 },
    { 0xe1c23438, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b2343d, 61 },
    { 0xb4e2343d, 90 },
    { 0x6e22343d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71623447, 33 },
    { 0, 0 },
    { 0x69323449, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32344c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed23450, 98 },
    { 0xe6d23451, 90 },
    { 0, 0 },
    { 0xcd923453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4323456, 33 },
    { 0, 0 },
    { 0x73b23458, 76 },
    { 0x5d823459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812345d, 40 },
    { 0, 0 },
    { 0x8e82345f, 55 },
    { 0x64f23460, 90 },
    { 0xb4f23461, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb23464, 55 },
    { 0x74823465, 85 },
    { 0x9ec23466, 98 },
    { 0x5f223467, 23 },
    { 0, 0 },
    { 0xfae23469, 23 },
    { 0x71c23469, 90 },
    { 0xc482346b, 90 },
    { 0, 0 },
    { 0x98f2346d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89423472, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d23476, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed23479, 68 },
    { 0x5d02347a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b23481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a2348b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2062348e, 90 },
    { 0, 0 },
    { 0x6e623490, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6323493, 98 },
    { 0, 0 },
    { 0xac623495, 90 },
    { 0x13e23496, 65 },
    { 0x9ce23495, 33 },
    { 0, 0 },
    { 0x80d23499, 78 },
    { 0x7e02349a, 60 },
    { 0, 0 },
    { 0x3cf2349c, 23 },
    { 0, 0 },
    { 0x5272349e, 67 },
    { 0xa9f2349f, 90 },
    { 0x64c234a0, 23 },
    { 0x7da234a1, 33 },
    { 0, 0 },
    { 0x53f234a3, 23 },
    { 0x1c2234a4, 90 },
    { 0x99a234a5, 81 },
    { 0, 0 },
    { 0x86d234a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced234ac, 76 },
    { 0xd7a234ad, 33 },
    { 0xe16234ae, 33 },
    { 0x70d234af, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627234b3, 95 },
    { 0xca6234b3, 17 },
    { 0, 0 },
    { 0xccd234b6, 95 },
    { 0x653234b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed234bb, 34 },
    { 0xe5e234bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8234c1, 23 },
    { 0x649234c1, 95 },
    { 0, 0 },
    { 0x824234c4, 93 },
    { 0xfea234c5, 93 },
    { 0, 0 },
    { 0xb35234c7, 33 },
    { 0x36a234c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe234cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x859234ce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4234d2, 85 },
    { 0, 0 },
    { 0x51d234d4, 34 },
    { 0xf35234d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187234d9, 78 },
    { 0x159234da, 90 },
    { 0, 0 },
    { 0x4fc234dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471234e0, 34 },
    { 0x96a234e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x265234e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85234e8, 84 },
    { 0xd2e234e9, 61 },
    { 0x1c9234ea, 33 },
    { 0xdf9234eb, 81 },
    { 0x642234ec, 17 },
    { 0xbe0234ed, 46 },
    { 0xaea234ed, 75 },
    { 0x564234ef, 61 },
    { 0x19234f0, 35 },
    { 0x928234ed, 95 },
    { 0, 0 },
    { 0xf81234f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4234f6, 23 },
    { 0xbae234f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f923500, 61 },
    { 0xd0e23501, 90 },
    { 0x46c23501, 61 },
    { 0x1123500, 69 },
    { 0xee23502, 33 },
    { 0, 0 },
    { 0x72823506, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae2350c, 33 },
    { 0x2a42350d, 53 },
    { 0x3732350e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa23512, 46 },
    { 0, 0 },
    { 0xa4723514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f23517, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8723520, 33 },
    { 0, 0 },
    { 0x18423522, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70423527, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2352a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x432352d, 90 },
    { 0, 0 },
    { 0xe12352f, 61 },
    { 0xd7a2352f, 34 },
    { 0, 0 },
    { 0x5d323532, 98 },
    { 0x7a223532, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe23536, 34 },
    { 0x53023536, 85 },
    { 0x47d23536, 85 },
    { 0, 0 },
    { 0x4f22353a, 90 },
    { 0, 0 },
    { 0x5262353c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d22353f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c23548, 78 },
    { 0xb3c23549, 90 },
    { 0, 0 },
    { 0x6682354b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36323550, 8 },
    { 0, 0 },
    { 0x8b623552, 34 },
    { 0x8a523553, 23 },
    { 0xe2923554, 91 },
    { 0x29e23552, 99 },
    { 0x15123556, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c623559, 69 },
    { 0xba72355a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42355e, 90 },
    { 0x89b2355f, 23 },
    { 0x7fb2355f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb423565, 95 },
    { 0x6e23565, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca02356a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c223570, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6623575, 90 },
    { 0xed823576, 61 },
    { 0x2bf23577, 54 },
    { 0xa2b23577, 34 },
    { 0, 0 },
    { 0x4ec2357a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2672357e, 33 },
    { 0xa232357f, 61 },
    { 0xddd2357f, 90 },
    { 0x2b823581, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xad023584, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d523587, 55 },
    { 0x8d623587, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99623590, 98 },
    { 0xa7123591, 40 },
    { 0x62a23592, 34 },
    { 0x1fe23590, 76 },
    { 0x8d523594, 95 },
    { 0, 0 },
    { 0x91b23596, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c23599, 98 },
    { 0xcec2359a, 61 },
    { 0x13c23599, 55 },
    { 0, 0 },
    { 0x8a62359d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf235a0, 90 },
    { 0x1ed235a1, 90 },
    { 0x357235a1, 95 },
    { 0xada235a1, 61 },
    { 0xbd3235a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f235a8, 49 },
    { 0x131235a8, 90 },
    { 0x2dc235a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e235ad, 61 },
    { 0xefb235ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b235b1, 93 },
    { 0x741235b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93235b6, 60 },
    { 0x588235b7, 19 },
    { 0x324235b8, 23 },
    { 0xc70235b9, 90 },
    { 0xe99235ba, 43 },
    { 0, 0 },
    { 0x9a5235bc, 34 },
    { 0x339235bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6235c3, 98 },
    { 0x2ac235c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4235c9, 19 },
    { 0xc3f235ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08235cf, 33 },
    { 0, 0 },
    { 0x289235d1, 46 },
    { 0xff235d1, 43 },
    { 0, 0 },
    { 0x7cb235d4, 95 },
    { 0, 0 },
    { 0x531235d6, 95 },
    { 0x53c235d6, 85 },
    { 0, 0 },
    { 0x155235d9, 78 },
    { 0xab0235da, 55 },
    { 0, 0 },
    { 0xa4c235dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d235df, 98 },
    { 0, 0 },
    { 0x325235e1, 19 },
    { 0x240235e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd235e6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a235e9, 95 },
    { 0x1d5235ea, 90 },
    { 0x393235eb, 95 },
    { 0xae0235ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b235ef, 91 },
    { 0x3b6235f0, 95 },
    { 0xabe235f0, 43 },
    { 0, 0 },
    { 0xed0235f3, 33 },
    { 0x79f235f3, 61 },
    { 0, 0 },
    { 0x4c235f6, 46 },
    { 0, 0 },
    { 0xa32235f8, 33 },
    { 0xc91235f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d235fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae23605, 33 },
    { 0x2b923606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc2360b, 61 },
    { 0, 0 },
    { 0xbf52360d, 46 },
    { 0, 0 },
    { 0xee42360f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x91323612, 23 },
    { 0x13423612, 90 },
    { 0x5f23613, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76023618, 61 },
    { 0x1a223619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e22361c, 34 },
    { 0xf022361d, 100 },
    { 0, 0 },
    { 0x2bd2361f, 55 },
    { 0, 0 },
    { 0x62b23621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c23626, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf23629, 33 },
    { 0, 0 },
    { 0xe02362b, 61 },
    { 0xfed2362c, 90 },
    { 0, 0 },
    { 0x32e2362e, 55 },
    { 0x35a2362e, 59 },
    { 0, 0 },
    { 0x1a423631, 76 },
    { 0xb8223632, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13023635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9423638, 90 },
    { 0x6f223639, 98 },
    { 0x8d223638, 75 },
    { 0xb4a2363b, 23 },
    { 0xf592363c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3132363f, 61 },
    { 0, 0 },
    { 0x76123641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1923644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a423647, 61 },
    { 0x67323648, 90 },
    { 0x60523649, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc492364f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83823655, 52 },
    { 0xd5d23655, 95 },
    { 0, 0 },
    { 0xb1a23658, 98 },
    { 0xea423659, 33 },
    { 0x1eb2365a, 95 },
    { 0, 0 },
    { 0x47a2365c, 76 },
    { 0x6932365d, 33 },
    { 0xce92365e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36523664, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b2366b, 90 },
    { 0x2522366b, 23 },
    { 0xf042366c, 34 },
    { 0xb0d2366b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f923674, 95 },
    { 0x6ff23675, 61 },
    { 0x3c123675, 33 },
    { 0x72423677, 87 },
    { 0x90023678, 61 },
    { 0, 0 },
    { 0x74e2367a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f2367e, 95 },
    { 0x36a2367f, 95 },
    { 0x162367f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd123684, 90 },
    { 0x6bf23685, 23 },
    { 0x19f23686, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b23689, 77 },
    { 0x45e2368a, 33 },
    { 0, 0 },
    { 0x3b82368c, 23 },
    { 0x4de2368d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e323691, 46 },
    { 0xe0a23691, 75 },
    { 0, 0 },
    { 0x1af23694, 95 },
    { 0xf823695, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf662369b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6822369f, 66 },
    { 0xfe02369f, 38 },
    { 0x602236a1, 33 },
    { 0, 0 },
    { 0xe7c236a3, 33 },
    { 0x195236a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b236a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34236ad, 23 },
    { 0x954236ae, 90 },
    { 0x2e8236ad, 98 },
    { 0, 0 },
    { 0xf2236b1, 61 },
    { 0, 0 },
    { 0x32c236b3, 97 },
    { 0xcfa236b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c236b8, 61 },
    { 0xdf3236b9, 95 },
    { 0x3e5236ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314236be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7236c4, 33 },
    { 0x5af236c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b236cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c236d0, 69 },
    { 0x871236d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b236d5, 33 },
    { 0x8a4236d6, 90 },
    { 0x69a236d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x534236da, 84 },
    { 0x741236da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7236e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a236e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309236ec, 90 },
    { 0xf86236ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88236f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77236f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14236fc, 46 },
    { 0, 0 },
    { 0xe80236fe, 48 },
    { 0xddc236fe, 100 },
    { 0, 0 },
    { 0x11d23701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c23704, 95 },
    { 0x10c23705, 99 },
    { 0, 0 },
    { 0xfa023707, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d2370a, 51 },
    { 0x9a12370b, 43 },
    { 0x7112370c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8422370f, 90 },
    { 0x10c23710, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed223713, 90 },
    { 0, 0 },
    { 0x99f23715, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b23719, 34 },
    { 0x57d2371a, 95 },
    { 0xdc62371b, 98 },
    { 0xfa22371c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa772371f, 43 },
    { 0xd152371f, 36 },
    { 0xe5a2371f, 46 },
    { 0xd9a23722, 78 },
    { 0x8aa23722, 84 },
    { 0xb4d23722, 98 },
    { 0x45123725, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3323728, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7672372e, 76 },
    { 0x9972372f, 67 },
    { 0, 0 },
    { 0x6c423731, 98 },
    { 0x5af23731, 43 },
    { 0xfed23731, 90 },
    { 0, 0 },
    { 0xec923735, 33 },
    { 0, 0 },
    { 0xe2223737, 99 },
    { 0x88c23738, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee82373e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c923744, 33 },
    { 0xd5623745, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d2374e, 95 },
    { 0, 0 },
    { 0xcb123750, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86923754, 61 },
    { 0xbcb23754, 61 },
    { 0x3ef23756, 85 },
    { 0xfe023756, 77 },
    { 0xfdd23757, 61 },
    { 0, 0 },
    { 0xbdc2375a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d2375e, 75 },
    { 0xda92375f, 33 },
    { 0, 0 },
    { 0xc0023761, 99 },
    { 0xf3223761, 23 },
    { 0x7a623763, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f023768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b22376e, 53 },
    { 0xfe22376f, 98 },
    { 0xc1623770, 78 },
    { 0xd072376e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b23774, 76 },
    { 0x2d123775, 33 },
    { 0xf9f23776, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7052377a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b23780, 78 },
    { 0, 0 },
    { 0x29923782, 95 },
    { 0x55623783, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec823787, 46 },
    { 0xb6d23788, 47 },
    { 0xc4a23789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47123791, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed23798, 33 },
    { 0x81b23799, 67 },
    { 0x5de2379a, 95 },
    { 0xabc2379a, 34 },
    { 0xba82379c, 33 },
    { 0x24323799, 91 },
    { 0x91c2379e, 33 },
    { 0, 0 },
    { 0x6c5237a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6237a6, 33 },
    { 0, 0 },
    { 0xc3f237a8, 55 },
    { 0x7237a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7237b3, 98 },
    { 0x681237b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80237b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d237bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64237c4, 84 },
    { 0xb45237c5, 66 },
    { 0x156237c6, 33 },
    { 0xa64237c7, 61 },
    { 0xe49237c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618237cc, 93 },
    { 0x730237cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2237d1, 93 },
    { 0x6cc237d2, 34 },
    { 0x2f2237d2, 98 },
    { 0xa27237d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81237d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6237dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6237df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f237e4, 77 },
    { 0, 0 },
    { 0xe05237e6, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x418237e9, 76 },
    { 0xafd237e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792237ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a237f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7237f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc237fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca237ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40523803, 95 },
    { 0xcc23804, 98 },
    { 0xb8423803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb512380b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd2380f, 95 },
    { 0xaf023810, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c23816, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf23819, 17 },
    { 0x2ed2381a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f2381d, 90 },
    { 0, 0 },
    { 0xe522381f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d23822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb662382a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c2382e, 64 },
    { 0xf452382e, 100 },
    { 0x39423830, 8 },
    { 0xac623831, 90 },
    { 0, 0 },
    { 0x4e223833, 33 },
    { 0, 0 },
    { 0x11523835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3823838, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e2383d, 90 },
    { 0xe142383e, 43 },
    { 0x8a82383d, 33 },
    { 0, 0 },
    { 0xf3b23841, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f23846, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf23849, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea2384c, 90 },
    { 0, 0 },
    { 0x1df2384e, 19 },
    { 0x4722384f, 93 },
    { 0, 0 },
    { 0x3423851, 67 },
    { 0x64823851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64123855, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb2385c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a23860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe23868, 90 },
    { 0x50723869, 33 },
    { 0xa3f2386a, 24 },
    { 0xa762386b, 58 },
    { 0xda62386c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc023870, 100 },
    { 0xdf323870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15523875, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12023879, 95 },
    { 0x492387a, 46 },
    { 0, 0 },
    { 0x9aa2387c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f23881, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc223884, 34 },
    { 0, 0 },
    { 0x7bc23886, 95 },
    { 0x66c23887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca23891, 95 },
    { 0, 0 },
    { 0x94a23893, 61 },
    { 0x50023893, 90 },
    { 0xd923895, 93 },
    { 0x58a23895, 98 },
    { 0x40523897, 98 },
    { 0x8f123898, 78 },
    { 0, 0 },
    { 0x37f2389a, 23 },
    { 0x3f72389b, 43 },
    { 0, 0 },
    { 0x8f62389d, 23 },
    { 0xa7b2389e, 23 },
    { 0, 0 },
    { 0xdfc238a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x169238a3, 55 },
    { 0, 0 },
    { 0x30d238a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9238a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69238ab, 55 },
    { 0, 0 },
    { 0xee4238ad, 33 },
    { 0, 0 },
    { 0xf81238af, 61 },
    { 0, 0 },
    { 0xc57238b1, 61 },
    { 0xb56238b2, 95 },
    { 0, 0 },
    { 0x5a2238b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81238ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x745238bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5238c8, 93 },
    { 0, 0 },
    { 0x1e0238ca, 75 },
    { 0, 0 },
    { 0x10d238cc, 61 },
    { 0, 0 },
    { 0xfd9238ce, 67 },
    { 0xff5238cf, 90 },
    { 0x510238ce, 61 },
    { 0x491238cf, 43 },
    { 0xc15238d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69238da, 34 },
    { 0, 0 },
    { 0xe90238dc, 43 },
    { 0, 0 },
    { 0x51f238de, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651238e2, 90 },
    { 0xb65238e3, 90 },
    { 0x5fc238e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd238f3, 95 },
    { 0xc05238f3, 23 },
    { 0x386238f5, 33 },
    { 0, 0 },
    { 0x183238f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b238fb, 33 },
    { 0x75f238fb, 98 },
    { 0, 0 },
    { 0x5dd238fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca623902, 90 },
    { 0x77623903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf223907, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f2390d, 98 },
    { 0xbd22390e, 61 },
    { 0xfde2390f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f23916, 17 },
    { 0x85623917, 88 },
    { 0, 0 },
    { 0x5223919, 69 },
    { 0x5b32391a, 98 },
    { 0xa772391b, 92 },
    { 0x18a2391a, 87 },
    { 0x19023919, 33 },
    { 0x4f82391e, 76 },
    { 0x5b2391a, 90 },
    { 0xc7d23920, 98 },
    { 0x8c92391d, 98 },
    { 0, 0 },
    { 0xc1823923, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2392e, 61 },
    { 0, 0 },
    { 0xd3023930, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22023934, 95 },
    { 0, 0 },
    { 0xba723936, 23 },
    { 0, 0 },
    { 0xc1223938, 55 },
    { 0x3a223939, 90 },
    { 0, 0 },
    { 0x62c2393b, 90 },
    { 0, 0 },
    { 0xe552393d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39223941, 61 },
    { 0x59223942, 76 },
    { 0x3d823941, 76 },
    { 0x41123942, 61 },
    { 0xed423944, 33 },
    { 0x1d623946, 95 },
    { 0x69023947, 95 },
    { 0x2d023947, 98 },
    { 0xb9e23949, 84 },
    { 0x54723941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d23952, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c23957, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f2395e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd523961, 76 },
    { 0, 0 },
    { 0x69623963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f23968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d2396c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda123971, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac623974, 84 },
    { 0xcc223975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2397a, 90 },
    { 0x4ab2397a, 99 },
    { 0, 0 },
    { 0xc8e2397d, 93 },
    { 0x7202397e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32223982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6822398d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c23992, 43 },
    { 0xcc423993, 43 },
    { 0x1ea23994, 77 },
    { 0xe0623994, 23 },
    { 0x28e23996, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb42399d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45239b0, 40 },
    { 0x616239b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae239b4, 23 },
    { 0x35f239b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f239bb, 84 },
    { 0xf5239bc, 93 },
    { 0, 0 },
    { 0xbe9239be, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed239cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a239d2, 85 },
    { 0x1e3239d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed239d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754239dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e239e7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c239ec, 52 },
    { 0xfd239ed, 23 },
    { 0x5e6239ee, 33 },
    { 0x57b239ef, 61 },
    { 0x536239f0, 33 },
    { 0xdc1239f1, 95 },
    { 0x849239f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50239f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a239fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e239ff, 33 },
    { 0, 0 },
    { 0x9ed23a01, 90 },
    { 0xf0723a02, 91 },
    { 0x2fb23a01, 90 },
    { 0, 0 },
    { 0x37923a05, 99 },
    { 0, 0 },
    { 0xc3823a07, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f23a0e, 95 },
    { 0x61723a0f, 76 },
    { 0x97b23a0e, 78 },
    { 0x1bc23a10, 56 },
    { 0xa1a23a12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b23a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3523a1d, 98 },
    { 0x66123a1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9923a21, 81 },
    { 0x95223a21, 93 },
    { 0x65723a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa23a28, 33 },
    { 0x79c23a29, 90 },
    { 0xd5423a2a, 68 },
    { 0x19d23a2a, 98 },
    { 0xc8523a2b, 100 },
    { 0x8e223a2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b23a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e823a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc023a3d, 98 },
    { 0x30623a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d23a42, 61 },
    { 0xb6a23a42, 64 },
    { 0, 0 },
    { 0xae823a45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc223a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5023a4d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e723a53, 99 },
    { 0x64923a54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe23a59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23a5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92323a67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8123a6b, 90 },
    { 0x84323a6c, 33 },
    { 0, 0 },
    { 0x67323a6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x96323a71, 64 },
    { 0xcd423a72, 100 },
    { 0x69823a72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e23a7c, 90 },
    { 0xef423a7d, 61 },
    { 0x1d023a7e, 33 },
    { 0xe5523a7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9423a82, 23 },
    { 0x61b23a82, 52 },
    { 0xbf923a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30923a88, 93 },
    { 0x36023a89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac23a8f, 67 },
    { 0x8c423a8f, 68 },
    { 0xb5723a91, 86 },
    { 0x87323a92, 87 },
    { 0x13d23a93, 87 },
    { 0xdd523a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77123a97, 96 },
    { 0, 0 },
    { 0x5e723a99, 33 },
    { 0xb1f23a9a, 76 },
    { 0, 0 },
    { 0x2b423a9c, 55 },
    { 0xb5523a9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67523aa2, 43 },
    { 0, 0 },
    { 0xae423aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9423aa7, 61 },
    { 0xf8423aa8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f23aab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a23aaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e23abc, 76 },
    { 0x5b523abd, 84 },
    { 0xe2a23abe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c23ac4, 95 },
    { 0xa3023ac5, 34 },
    { 0, 0 },
    { 0x74723ac7, 52 },
    { 0xe5c23ac7, 95 },
    { 0, 0 },
    { 0xd1523aca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86523ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5723ada, 66 },
    { 0, 0 },
    { 0x8ff23adc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c823ae2, 66 },
    { 0xe8823ae3, 34 },
    { 0x7e523ae4, 87 },
    { 0, 0 },
    { 0x26d23ae6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a23ae9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52823aed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc23af0, 34 },
    { 0xa9623af1, 95 },
    { 0x12123af2, 77 },
    { 0, 0 },
    { 0x44423af4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d523afc, 46 },
    { 0x57823afc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc423b02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb923b05, 96 },
    { 0xde523b06, 10 },
    { 0x2c923b05, 34 },
    { 0x63223b05, 61 },
    { 0x2e423b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6823b0e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6323b11, 98 },
    { 0x89b23b12, 23 },
    { 0x33f23b11, 90 },
    { 0xdcc23b14, 88 },
    { 0x8923b12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a23b18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc23b20, 90 },
    { 0xa6a23b20, 33 },
    { 0x34a23b20, 33 },
    { 0x1d023b21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a323b27, 67 },
    { 0xc6123b28, 34 },
    { 0, 0 },
    { 0x69723b2a, 95 },
    { 0, 0 },
    { 0xbcf23b2c, 55 },
    { 0xbc323b2c, 43 },
    { 0x91523b2e, 90 },
    { 0, 0 },
    { 0x4323b30, 67 },
    { 0, 0 },
    { 0x21023b32, 43 },
    { 0xc3f23b32, 34 },
    { 0x3ae23b33, 61 },
    { 0x2bd23b35, 99 },
    { 0x17623b36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6823b39, 33 },
    { 0xb6123b39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f323b3e, 43 },
    { 0xc3d23b3f, 95 },
    { 0xe1d23b40, 85 },
    { 0x6c223b3f, 33 },
    { 0x68523b42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca23b47, 61 },
    { 0x93023b47, 45 },
    { 0x8e023b49, 91 },
    { 0xb4b23b4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3923b4e, 61 },
    { 0, 0 },
    { 0xac223b50, 90 },
    { 0xecb23b51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae23b56, 90 },
    { 0, 0 },
    { 0x54e23b58, 99 },
    { 0xed23b59, 61 },
    { 0xa9f23b59, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87723b61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95223b66, 46 },
    { 0x7ae23b67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99323b6a, 95 },
    { 0, 0 },
    { 0x5b723b6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c23b6f, 52 },
    { 0xcb423b6f, 90 },
    { 0x61223b70, 43 },
    { 0xfac23b72, 33 },
    { 0xc9f23b73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf523b78, 23 },
    { 0xb1923b79, 98 },
    { 0xb2d23b7a, 23 },
    { 0xd7e23b78, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d23b7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8223b87, 55 },
    { 0x8c423b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f23b90, 95 },
    { 0xff623b90, 90 },
    { 0x3f223b92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd023ba0, 33 },
    { 0x81223ba1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5823bab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f23bb0, 95 },
    { 0xcad23bb0, 90 },
    { 0xc8223bb2, 83 },
    { 0xb0123bb2, 98 },
    { 0xff723bb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e123bb8, 75 },
    { 0, 0 },
    { 0x8e223bba, 78 },
    { 0xab323bbb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba23bbe, 95 },
    { 0, 0 },
    { 0x49323bc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6523bc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e23bc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x81023bcb, 12 },
    { 0, 0 },
    { 0x3a123bcd, 90 },
    { 0xf9c23bce, 33 },
    { 0x32623bcd, 75 },
    { 0xfc923bd0, 23 },
    { 0, 0 },
    { 0xcf523bd2, 85 },
    { 0xefa23bd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf23bd6, 23 },
    { 0x5c323bd7, 95 },
    { 0xe4d23bd6, 90 },
    { 0xd6c23bd9, 61 },
    { 0, 0 },
    { 0x71923bdb, 85 },
    { 0xc9223bdc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd023bdf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89323be3, 61 },
    { 0x4e23be4, 91 },
    { 0x32d23be5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb723bf0, 91 },
    { 0xb7223bf1, 93 },
    { 0x9a423bf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64423bf5, 85 },
    { 0x29d23bf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c23bf9, 90 },
    { 0x3dc23bfa, 33 },
    { 0xc7c23bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f23bfe, 84 },
    { 0x48e23bff, 87 },
    { 0xd5923c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52023c06, 88 },
    { 0xf3723c07, 88 },
    { 0xdfc23c08, 61 },
    { 0x44c23c09, 23 },
    { 0xddb23c0a, 77 },
    { 0, 0 },
    { 0x96a23c0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d323c0f, 98 },
    { 0, 0 },
    { 0x60d23c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26023c18, 35 },
    { 0xaa923c19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae23c1e, 91 },
    { 0, 0 },
    { 0xee723c20, 90 },
    { 0x12323c21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa723c25, 23 },
    { 0x58523c25, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34823c33, 78 },
    { 0x61623c34, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68223c38, 95 },
    { 0x43a23c38, 76 },
    { 0, 0 },
    { 0x1cb23c3b, 52 },
    { 0xc1823c3c, 33 },
    { 0x96f23c3c, 55 },
    { 0, 0 },
    { 0x5fc23c3f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef023c43, 98 },
    { 0xc9923c44, 90 },
    { 0, 0 },
    { 0x87923c46, 46 },
    { 0x7d223c46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f23c4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad23c4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f23c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3023c5c, 99 },
    { 0, 0 },
    { 0xaa523c5e, 90 },
    { 0xf3d23c5e, 34 },
    { 0xb5123c60, 98 },
    { 0, 0 },
    { 0x3b423c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c523c6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26023c6d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb23c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92723c73, 33 },
    { 0, 0 },
    { 0x1d423c75, 90 },
    { 0xbf323c76, 61 },
    { 0, 0 },
    { 0x74a23c78, 33 },
    { 0, 0 },
    { 0x5d823c7a, 98 },
    { 0xba923c7b, 33 },
    { 0x5fb23c7c, 90 },
    { 0, 0 },
    { 0x4ad23c7e, 33 },
    { 0x44223c7f, 98 },
    { 0xbe023c80, 84 },
    { 0xeec23c81, 77 },
    { 0xce723c81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f23c85, 95 },
    { 0x9323c86, 76 },
    { 0xab23c87, 90 },
    { 0x5f23c86, 78 },
    { 0x37623c88, 33 },
    { 0xb0623c8a, 55 },
    { 0xa9023c8b, 100 },
    { 0xe6723c8a, 85 },
    { 0, 0 },
    { 0x8e23c8e, 76 },
    { 0x5f723c8f, 78 },
    { 0, 0 },
    { 0x16c23c91, 90 },
    { 0, 0 },
    { 0xdce23c93, 90 },
    { 0x54e23c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd23c98, 23 },
    { 0x41423c98, 90 },
    { 0, 0 },
    { 0x3ff23c9b, 61 },
    { 0xa3a23c9b, 91 },
    { 0x46923c9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb623ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c23ca5, 93 },
    { 0x6c23ca6, 78 },
    { 0x44323ca7, 64 },
    { 0x5d123ca7, 87 },
    { 0xad823ca9, 91 },
    { 0, 0 },
    { 0xbbc23cab, 52 },
    { 0x1b723cac, 90 },
    { 0x6c223cad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x12723cb0, 84 },
    { 0xdb623cb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40023cb7, 78 },
    { 0x5bb23cb7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26923cbc, 33 },
    { 0x3e523cbc, 40 },
    { 0, 0 },
    { 0x22823cbf, 95 },
    { 0, 0 },
    { 0xd1323cc1, 90 },
    { 0, 0 },
    { 0x88423cc3, 23 },
    { 0, 0 },
    { 0x23c23cc5, 90 },
    { 0, 0 },
    { 0x63623cc7, 43 },
    { 0x37a23cc8, 90 },
    { 0x97823cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37823ccf, 33 },
    { 0x98623cd0, 43 },
    { 0x40223cd1, 90 },
    { 0x82a23cd2, 87 },
    { 0xa0323cd3, 43 },
    { 0x52f23cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7923cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92423cdd, 98 },
    { 0x35d23cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d723ce3, 46 },
    { 0x80823ce3, 23 },
    { 0x27423ce5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1323ce9, 75 },
    { 0, 0 },
    { 0xfee23ceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4223cef, 65 },
    { 0xd5d23cf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5123cf3, 66 },
    { 0, 0 },
    { 0x4e623cf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed123cf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7523cfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba623cfe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9223d01, 90 },
    { 0xeb923d02, 33 },
    { 0x83b23d02, 55 },
    { 0x56123d04, 95 },
    { 0x48123d05, 78 },
    { 0x8f023d06, 85 },
    { 0, 0 },
    { 0x62923d08, 98 },
    { 0x9fe23d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed923d0d, 78 },
    { 0xf6d23d0e, 80 },
    { 0, 0 },
    { 0x6d723d10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb323d14, 66 },
    { 0, 0 },
    { 0xa4423d16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef23d1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12023d23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5823d26, 40 },
    { 0x8c523d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7223d2b, 34 },
    { 0x21f23d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26123d2f, 91 },
    { 0x68623d30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56423d36, 23 },
    { 0xc5d23d36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec23d3b, 98 },
    { 0xf9823d3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf23d42, 91 },
    { 0x323d42, 66 },
    { 0x58123d43, 95 },
    { 0x44b23d45, 93 },
    { 0x57d23d46, 40 },
    { 0x6fd23d42, 76 },
    { 0, 0 },
    { 0x8cc23d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de23d55, 23 },
    { 0xf5c23d55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f23d5d, 98 },
    { 0xc7323d5e, 93 },
    { 0, 0 },
    { 0xf5a23d60, 98 },
    { 0x89c23d61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec23d64, 95 },
    { 0x7e323d65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b23d6b, 90 },
    { 0x95b23d6b, 78 },
    { 0x64b23d6b, 23 },
    { 0x3b323d6d, 76 },
    { 0x3a323d6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0423d72, 91 },
    { 0x5b523d73, 85 },
    { 0xc3923d73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d523d77, 98 },
    { 0x72a23d78, 55 },
    { 0xf3523d78, 91 },
    { 0, 0 },
    { 0x7ea23d7b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f023d83, 55 },
    { 0xe0423d83, 23 },
    { 0, 0 },
    { 0x6b423d86, 93 },
    { 0, 0 },
    { 0x4bf23d88, 63 },
    { 0, 0 },
    { 0xfdc23d8a, 55 },
    { 0, 0 },
    { 0xf7f23d8c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0523d91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d23d98, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c23d9b, 90 },
    { 0x65523d9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e523d9f, 34 },
    { 0x26c23da0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc23da4, 95 },
    { 0x17223da5, 33 },
    { 0xad123da6, 91 },
    { 0x33623da7, 23 },
    { 0x73723da7, 61 },
    { 0xa623da8, 43 },
    { 0xca23da8, 55 },
    { 0xcae23da5, 33 },
    { 0xd2d23da9, 33 },
    { 0xa123dad, 95 },
    { 0x19523da5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x11323db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c823db7, 68 },
    { 0, 0 },
    { 0x13523db9, 33 },
    { 0xee023db9, 99 },
    { 0xef723db9, 76 },
    { 0x4b323dba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66823dc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0723dc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c23dca, 98 },
    { 0xb4e23dcb, 46 },
    { 0, 0 },
    { 0xf1123dcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a23ddf, 90 },
    { 0xb3e23de0, 33 },
    { 0xe3323de1, 61 },
    { 0x53e23de0, 34 },
    { 0xccc23ddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c23de6, 93 },
    { 0x6123de7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8123dee, 33 },
    { 0, 0 },
    { 0x35123df0, 87 },
    { 0x3c723df1, 98 },
    { 0x91e23df2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6123df6, 90 },
    { 0, 0 },
    { 0xe1b23df8, 98 },
    { 0xa2423df8, 95 },
    { 0x3223dfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a23dfd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd323e01, 23 },
    { 0xc6423e02, 34 },
    { 0xacc23e02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4723e0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da23e0d, 90 },
    { 0xa7623e0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8223e11, 66 },
    { 0, 0 },
    { 0x1723e13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec23e18, 52 },
    { 0, 0 },
    { 0xc8e23e1a, 66 },
    { 0xd2823e1b, 98 },
    { 0x25523e1a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29823e20, 65 },
    { 0x7d823e21, 75 },
    { 0, 0 },
    { 0x84923e23, 43 },
    { 0, 0 },
    { 0x4da23e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe23e30, 76 },
    { 0xdeb23e31, 55 },
    { 0xb6a23e30, 95 },
    { 0xb5e23e33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b923e3b, 96 },
    { 0xe2623e3c, 76 },
    { 0xa2223e3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d23e41, 43 },
    { 0x62c23e42, 90 },
    { 0xaff23e43, 61 },
    { 0xb3623e43, 52 },
    { 0xa0e23e44, 99 },
    { 0x50b23e41, 90 },
    { 0x78d23e46, 23 },
    { 0, 0 },
    { 0x4d123e49, 55 },
    { 0x13e23e4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3823e4d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x36023e50, 95 },
    { 0x80923e50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a623e56, 33 },
    { 0x38f23e57, 46 },
    { 0, 0 },
    { 0x92123e59, 75 },
    { 0x1e023e59, 90 },
    { 0x38323e59, 93 },
    { 0, 0 },
    { 0x68223e5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64323e60, 61 },
    { 0xc6e23e61, 90 },
    { 0xba323e62, 33 },
    { 0x9d223e60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28123e67, 75 },
    { 0, 0 },
    { 0xff123e69, 61 },
    { 0xe3c23e6a, 23 },
    { 0x84223e6b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a523e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9223e76, 46 },
    { 0x46f23e77, 95 },
    { 0xc6623e76, 23 },
    { 0xc0b23e76, 76 },
    { 0xbcc23e7a, 34 },
    { 0, 0 },
    { 0x9d623e7c, 34 },
    { 0xfbe23e7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c423e82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7423e88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae23e8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2423e90, 90 },
    { 0, 0 },
    { 0x8923e92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6223ea0, 61 },
    { 0x9c023ea1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f23ea8, 90 },
    { 0xe1d23ea9, 95 },
    { 0x5f323eaa, 33 },
    { 0xb2e23eab, 33 },
    { 0x9dd23ea9, 33 },
    { 0xfd723ead, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd623eb0, 52 },
    { 0, 0 },
    { 0x85a23eb2, 78 },
    { 0, 0 },
    { 0x47323eb4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b23eb7, 45 },
    { 0x7bf23eb8, 55 },
    { 0xd4723eb9, 61 },
    { 0x75d23eb8, 98 },
    { 0x14123ebb, 61 },
    { 0, 0 },
    { 0xe2123ebd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a423ec0, 91 },
    { 0xcdf23ec1, 61 },
    { 0x10923ec1, 61 },
    { 0, 0 },
    { 0xa8523ec4, 66 },
    { 0x98323ec5, 55 },
    { 0xaf523ec6, 61 },
    { 0, 0 },
    { 0xf0423ec8, 34 },
    { 0, 0 },
    { 0xdb823eca, 61 },
    { 0x1e723ecb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f23ed3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd223ed7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18123ee7, 23 },
    { 0, 0 },
    { 0xaa723ee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5623eef, 99 },
    { 0xc2a23ef0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b23ef5, 55 },
    { 0x50323ef6, 95 },
    { 0xa8123ef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6623efc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae23eff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63323f06, 100 },
    { 0x65323f07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd23f0b, 43 },
    { 0, 0 },
    { 0xd6b23f0d, 90 },
    { 0x1ec23f0d, 33 },
    { 0xfa223f0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41923f13, 64 },
    { 0x21623f14, 90 },
    { 0x95b23f14, 84 },
    { 0, 0 },
    { 0xd7e23f17, 34 },
    { 0xb9323f17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4923f1d, 90 },
    { 0x8a223f1d, 76 },
    { 0x41423f1e, 98 },
    { 0x27b23f20, 98 },
    { 0xd4023f20, 93 },
    { 0, 0 },
    { 0xc5723f23, 43 },
    { 0x25723f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a123f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34023f2b, 76 },
    { 0x81323f2c, 46 },
    { 0xf0623f2d, 61 },
    { 0, 0 },
    { 0xdcf23f2f, 33 },
    { 0xb6523f30, 61 },
    { 0x10723f30, 93 },
    { 0, 0 },
    { 0x5c923f33, 23 },
    { 0, 0 },
    { 0x37823f35, 75 },
    { 0, 0 },
    { 0x73523f37, 66 },
    { 0x44223f38, 61 },
    { 0x8e623f38, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x19623f3c, 76 },
    { 0x91923f3c, 23 },
    { 0x71623f3e, 41 },
    { 0, 0 },
    { 0x63723f40, 43 },
    { 0xfe323f40, 90 },
    { 0x2b323f42, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4723f46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c223f49, 46 },
    { 0xa2f23f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f423f51, 33 },
    { 0, 0 },
    { 0x12623f53, 98 },
    { 0x6a23f54, 95 },
    { 0xd7a23f54, 56 },
    { 0x37e23f54, 84 },
    { 0, 0 },
    { 0x4b023f58, 68 },
    { 0, 0 },
    { 0x93323f5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4623f5d, 23 },
    { 0, 0 },
    { 0x6d123f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd23f62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f23f66, 93 },
    { 0, 0 },
    { 0xe0923f68, 33 },
    { 0x2b123f68, 33 },
    { 0, 0 },
    { 0x32e23f6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4623f6e, 34 },
    { 0, 0 },
    { 0x6ee23f70, 23 },
    { 0x57723f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed23f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe123f7a, 98 },
    { 0x52323f7a, 55 },
    { 0xed723f7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a123f8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e23f94, 90 },
    { 0x29123f95, 17 },
    { 0x7a223f94, 67 },
    { 0x44f23f97, 95 },
    { 0, 0 },
    { 0x88723f99, 55 },
    { 0xba923f9a, 90 },
    { 0x2a523f9b, 98 },
    { 0x39b23f99, 100 },
    { 0xf6723f9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f123fa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf623fa7, 90 },
    { 0x9b323fa8, 50 },
    { 0, 0 },
    { 0xd4223faa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb23fae, 33 },
    { 0x8ce23faf, 33 },
    { 0xc3b23fae, 23 },
    { 0xedc23fb1, 98 },
    { 0x7e623fb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b423fb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xef923fb8, 90 },
    { 0, 0 },
    { 0xaa423fba, 61 },
    { 0, 0 },
    { 0x7f223fbc, 88 },
    { 0x4323fbc, 90 },
    { 0x54e23fbc, 98 },
    { 0x47923fbe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1723fc6, 90 },
    { 0, 0 },
    { 0x74723fc8, 90 },
    { 0, 0 },
    { 0xc8b23fca, 61 },
    { 0, 0 },
    { 0x3f823fcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df23fd2, 98 },
    { 0, 0 },
    { 0x3523fd4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc23fd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a23fdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7723fe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae223fea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35423ff7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77723ffb, 34 },
    { 0x2bc23ffb, 95 },
    { 0, 0 },
    { 0xe8023ffe, 85 },
    { 0x8ee23ffe, 91 },
    { 0x43124000, 23 },
    { 0x8e224000, 78 },
    { 0xe8d23ffe, 98 },
    { 0xb1124003, 61 },
    { 0x67e24004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c124009, 23 },
    { 0xc892400a, 77 },
    { 0xfa924009, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62400f, 96 },
    { 0x63924010, 76 },
    { 0x8f224010, 82 },
    { 0x1a624012, 95 },
    { 0x4e624010, 76 },
    { 0xe9724014, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d024019, 23 },
    { 0x2562401a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a24021, 23 },
    { 0x30824021, 77 },
    { 0x69624023, 76 },
    { 0, 0 },
    { 0x2ae24025, 93 },
    { 0x12124026, 61 },
    { 0x1924026, 33 },
    { 0xf2c24026, 23 },
    { 0x7c024029, 98 },
    { 0x29224027, 23 },
    { 0x6b724028, 95 },
    { 0x9542402c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8124030, 33 },
    { 0x85924030, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c24038, 95 },
    { 0xe1924039, 81 },
    { 0, 0 },
    { 0x7652403b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7524040, 58 },
    { 0x87c24040, 90 },
    { 0, 0 },
    { 0x2824043, 33 },
    { 0x1af24043, 85 },
    { 0xb9524044, 90 },
    { 0xe3624044, 98 },
    { 0xd7024047, 90 },
    { 0x62824048, 90 },
    { 0xe1a24048, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f12404c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc92404f, 91 },
    { 0xa462404f, 100 },
    { 0x4d424051, 94 },
    { 0xfc824052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e024056, 34 },
    { 0x8d924057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf832405a, 95 },
    { 0, 0 },
    { 0x93b2405c, 38 },
    { 0xdce2405c, 90 },
    { 0x1652405e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8224064, 93 },
    { 0x3a24065, 99 },
    { 0, 0 },
    { 0xfd24067, 95 },
    { 0xd4d24068, 87 },
    { 0x58524069, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28624070, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df24073, 23 },
    { 0, 0 },
    { 0x72b24075, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda624078, 95 },
    { 0, 0 },
    { 0x5072407a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce2407e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca24081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a24086, 90 },
    { 0, 0 },
    { 0x3c524088, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf282408d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79124091, 95 },
    { 0xf8124092, 90 },
    { 0xace24093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c424099, 33 },
    { 0x28b2409a, 33 },
    { 0x2262409b, 34 },
    { 0, 0 },
    { 0xdb42409d, 33 },
    { 0xf612409e, 88 },
    { 0x7b62409f, 23 },
    { 0xbfa240a0, 76 },
    { 0xb8f240a1, 66 },
    { 0xfbc240a1, 90 },
    { 0xcb4240a2, 23 },
    { 0xc3c240a4, 33 },
    { 0x6c0240a5, 52 },
    { 0, 0 },
    { 0x5e4240a7, 98 },
    { 0x36240a7, 33 },
    { 0x1a3240a9, 77 },
    { 0xab9240a9, 47 },
    { 0x8d2240ab, 99 },
    { 0xfb1240ac, 23 },
    { 0x15f240ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61240b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d240bc, 90 },
    { 0, 0 },
    { 0xab6240be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746240c8, 23 },
    { 0, 0 },
    { 0xe66240ca, 98 },
    { 0xb9b240cb, 34 },
    { 0xbfe240cb, 69 },
    { 0, 0 },
    { 0x64d240ce, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd240d1, 33 },
    { 0xa51240d1, 33 },
    { 0x3e5240d3, 98 },
    { 0, 0 },
    { 0x795240d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a240d8, 95 },
    { 0x36240d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5240dc, 95 },
    { 0x509240dd, 78 },
    { 0x92e240de, 88 },
    { 0x40f240df, 34 },
    { 0xa9240df, 80 },
    { 0x8cc240e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a240e5, 90 },
    { 0x5bf240e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9240e9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x169240ec, 61 },
    { 0x338240ed, 95 },
    { 0x3b9240ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08240f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871240f7, 34 },
    { 0, 0 },
    { 0x23c240f9, 52 },
    { 0x9e0240fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529240fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee224101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3524104, 17 },
    { 0x5ab24104, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2410a, 78 },
    { 0x9152410a, 55 },
    { 0, 0 },
    { 0x93d2410d, 90 },
    { 0xf7c2410e, 17 },
    { 0x6562410f, 55 },
    { 0xfdc24110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f24113, 23 },
    { 0, 0 },
    { 0x59324115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe24119, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e92411f, 90 },
    { 0xccf2411f, 95 },
    { 0, 0 },
    { 0x3bc24122, 61 },
    { 0xe3924123, 98 },
    { 0, 0 },
    { 0x33b24125, 61 },
    { 0x3ca24126, 98 },
    { 0x24924127, 23 },
    { 0xb6124128, 90 },
    { 0xb8024129, 95 },
    { 0, 0 },
    { 0xd052412b, 50 },
    { 0x2f52412b, 45 },
    { 0x6ec2412d, 55 },
    { 0, 0 },
    { 0x1a02412f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d24137, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb92413b, 76 },
    { 0, 0 },
    { 0x62d2413d, 80 },
    { 0xac62413e, 43 },
    { 0x2732413f, 33 },
    { 0xcf024140, 85 },
    { 0x6de24141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e24145, 95 },
    { 0, 0 },
    { 0x6dd24147, 33 },
    { 0, 0 },
    { 0x6924149, 98 },
    { 0, 0 },
    { 0xce62414b, 98 },
    { 0, 0 },
    { 0x6532414d, 75 },
    { 0xd3e2414e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e24151, 67 },
    { 0xc9a24152, 34 },
    { 0, 0 },
    { 0x1224154, 100 },
    { 0, 0 },
    { 0x58124156, 90 },
    { 0, 0 },
    { 0x90824158, 23 },
    { 0x21b24159, 76 },
    { 0x85c2415a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da2415e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba24161, 98 },
    { 0x66124162, 56 },
    { 0xabf24162, 67 },
    { 0x34d24164, 43 },
    { 0x8f124164, 95 },
    { 0x97324165, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4424169, 78 },
    { 0, 0 },
    { 0x26d2416b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63224179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6442417c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a24183, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2418e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea24192, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8424198, 52 },
    { 0xe5124199, 90 },
    { 0xfcb24198, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c2419f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f241a9, 76 },
    { 0x745241a9, 99 },
    { 0x360241ab, 95 },
    { 0xcee241ab, 23 },
    { 0x14e241ad, 34 },
    { 0x317241ab, 17 },
    { 0x1d1241aa, 90 },
    { 0x4cc241b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x454241b3, 98 },
    { 0x9e5241b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9241b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0241bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4241c0, 61 },
    { 0xfb6241c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa241c4, 61 },
    { 0xb1d241c5, 23 },
    { 0, 0 },
    { 0x9f4241c7, 75 },
    { 0xe7241c7, 33 },
    { 0, 0 },
    { 0xf4a241ca, 65 },
    { 0x492241cb, 23 },
    { 0x805241cb, 90 },
    { 0x44f241ca, 61 },
    { 0, 0 },
    { 0x8b6241cf, 90 },
    { 0x1b6241d0, 61 },
    { 0xbdd241d1, 80 },
    { 0xacd241d1, 34 },
    { 0, 0 },
    { 0xe26241d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94241d7, 55 },
    { 0x1a6241d8, 76 },
    { 0, 0 },
    { 0x39e241da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1241dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e241e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6241e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e241ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39241f0, 34 },
    { 0, 0 },
    { 0x31c241f2, 99 },
    { 0xa90241f2, 95 },
    { 0x274241f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2241f8, 98 },
    { 0x54a241f9, 33 },
    { 0x4a2241f9, 91 },
    { 0xdb5241f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad24206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac2420c, 61 },
    { 0, 0 },
    { 0x8652420e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3224211, 52 },
    { 0x96024212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82324216, 33 },
    { 0xf6c24216, 33 },
    { 0, 0 },
    { 0x26124219, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe2421c, 90 },
    { 0, 0 },
    { 0x5c82421e, 61 },
    { 0x8392421e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f24223, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea2422a, 33 },
    { 0, 0 },
    { 0x7322422c, 61 },
    { 0, 0 },
    { 0xd9d2422e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66724235, 61 },
    { 0xe2324235, 90 },
    { 0, 0 },
    { 0x31e24238, 95 },
    { 0xffa24239, 55 },
    { 0xbea24239, 43 },
    { 0x43024239, 98 },
    { 0xf522423a, 98 },
    { 0xdea2423c, 46 },
    { 0x16f24238, 90 },
    { 0, 0 },
    { 0xa2a24240, 61 },
    { 0, 0 },
    { 0x7a524242, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb924246, 61 },
    { 0, 0 },
    { 0x80024248, 33 },
    { 0x27c24249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a82424e, 33 },
    { 0x4ee2424f, 98 },
    { 0x93c24250, 76 },
    { 0x1872424e, 40 },
    { 0x6a924252, 34 },
    { 0, 0 },
    { 0xf2b24254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a2425e, 68 },
    { 0, 0 },
    { 0x5a924260, 46 },
    { 0, 0 },
    { 0x64524262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde124268, 95 },
    { 0, 0 },
    { 0xec22426a, 61 },
    { 0xfa92426b, 66 },
    { 0x18f2426c, 17 },
    { 0xcf82426d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412427a, 40 },
    { 0, 0 },
    { 0x1a62427c, 34 },
    { 0, 0 },
    { 0xa0d2427e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44024286, 46 },
    { 0xff424287, 90 },
    { 0xa6524287, 61 },
    { 0x10524286, 61 },
    { 0xcd24289, 90 },
    { 0x44b24287, 43 },
    { 0, 0 },
    { 0x4372428d, 90 },
    { 0x5802428e, 98 },
    { 0x2c42428f, 33 },
    { 0x3292428f, 91 },
    { 0xaa524291, 98 },
    { 0xf7824292, 87 },
    { 0xe2224293, 87 },
    { 0x7de24291, 84 },
    { 0, 0 },
    { 0xb8224296, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262429c, 76 },
    { 0, 0 },
    { 0x41a2429e, 90 },
    { 0xf282429f, 85 },
    { 0xc5f2429f, 23 },
    { 0x3d9242a1, 85 },
    { 0xc91242a2, 84 },
    { 0xafc242a2, 98 },
    { 0, 0 },
    { 0xf87242a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e242a9, 61 },
    { 0, 0 },
    { 0x68d242ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1242b1, 98 },
    { 0xa3242b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b242b5, 90 },
    { 0x701242b5, 99 },
    { 0xd11242b6, 98 },
    { 0x551242b8, 98 },
    { 0x48b242b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5242bc, 61 },
    { 0, 0 },
    { 0xcda242be, 55 },
    { 0xcab242be, 78 },
    { 0xfaa242c0, 100 },
    { 0x742242c1, 33 },
    { 0, 0 },
    { 0xa4d242c3, 61 },
    { 0, 0 },
    { 0xa0e242c5, 55 },
    { 0x17f242c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda242cb, 33 },
    { 0x29b242cc, 98 },
    { 0x42c242cc, 66 },
    { 0x704242ce, 43 },
    { 0x1e5242cb, 88 },
    { 0, 0 },
    { 0x6c2242d1, 81 },
    { 0x9cd242d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75242d9, 23 },
    { 0x7f7242d9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c242de, 33 },
    { 0, 0 },
    { 0xc58242e0, 76 },
    { 0, 0 },
    { 0xa5c242e2, 34 },
    { 0xeea242e3, 90 },
    { 0, 0 },
    { 0x873242e5, 38 },
    { 0xd08242e6, 43 },
    { 0, 0 },
    { 0x41d242e8, 55 },
    { 0, 0 },
    { 0x452242ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0242ed, 33 },
    { 0, 0 },
    { 0x40d242ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0242f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x282242f8, 33 },
    { 0, 0 },
    { 0x97c242fa, 98 },
    { 0x4ba242fb, 87 },
    { 0xe1d242fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd424303, 78 },
    { 0, 0 },
    { 0xba924305, 90 },
    { 0x10d24306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2430a, 98 },
    { 0, 0 },
    { 0xd512430c, 43 },
    { 0x10d2430d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef024311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3424317, 98 },
    { 0xc1824317, 65 },
    { 0x11524319, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b2431c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19924322, 61 },
    { 0x6f124322, 40 },
    { 0xc9f24323, 76 },
    { 0xb924325, 55 },
    { 0, 0 },
    { 0x18424327, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c2432c, 100 },
    { 0xcca2432d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab24333, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa824336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5212433b, 78 },
    { 0xc5a2433c, 93 },
    { 0x1d32433d, 61 },
    { 0, 0 },
    { 0x1c52433f, 75 },
    { 0x70424340, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1124345, 23 },
    { 0x2ad24346, 55 },
    { 0x95b24345, 33 },
    { 0x48924348, 61 },
    { 0x51e24346, 23 },
    { 0xdbd24345, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9742434d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc24350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a24355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b2435e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4024361, 36 },
    { 0xcf424361, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b324366, 23 },
    { 0x24024367, 35 },
    { 0x2d924368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4332436c, 23 },
    { 0xa972436d, 76 },
    { 0x2dd2436e, 33 },
    { 0xf82436f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c24373, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b82437a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb24381, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb24384, 95 },
    { 0, 0 },
    { 0x9ec24386, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a2438a, 83 },
    { 0xaa32438b, 90 },
    { 0, 0 },
    { 0xe002438d, 78 },
    { 0x4bf2438d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c24394, 95 },
    { 0x64024394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a724399, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe2439c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1243a1, 33 },
    { 0x2f243a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f243a5, 91 },
    { 0, 0 },
    { 0x9eb243a7, 100 },
    { 0, 0 },
    { 0x1f8243a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736243ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0243c1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4243c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b243c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e243ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x787243cd, 55 },
    { 0, 0 },
    { 0x396243cf, 90 },
    { 0, 0 },
    { 0xc66243d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a243d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855243df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0243ea, 95 },
    { 0xbda243eb, 88 },
    { 0x37f243ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82243f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8243f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba243fa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b24404, 81 },
    { 0, 0 },
    { 0x9a124406, 98 },
    { 0x99224407, 67 },
    { 0, 0 },
    { 0xc2924409, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f62440d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324411, 90 },
    { 0, 0 },
    { 0x88e24413, 90 },
    { 0, 0 },
    { 0xfd024415, 100 },
    { 0, 0 },
    { 0x2124417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3442441b, 88 },
    { 0, 0 },
    { 0xeeb2441d, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f24422, 61 },
    { 0x1c424422, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf124427, 76 },
    { 0xdb824428, 90 },
    { 0, 0 },
    { 0xc6a2442a, 90 },
    { 0x9822442b, 55 },
    { 0x79d2442c, 33 },
    { 0x8862442d, 88 },
    { 0x27f2442e, 33 },
    { 0x19d2442f, 90 },
    { 0x5b24430, 85 },
    { 0x45c24431, 93 },
    { 0x162442b, 98 },
    { 0xb5a2442e, 55 },
    { 0x30224434, 69 },
    { 0, 0 },
    { 0x35024436, 61 },
    { 0xfb224437, 23 },
    { 0, 0 },
    { 0xd0824439, 55 },
    { 0, 0 },
    { 0xb292443b, 61 },
    { 0xc102443c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b624448, 61 },
    { 0x57724449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e24450, 95 },
    { 0xd7424451, 75 },
    { 0x3b324452, 23 },
    { 0x9ac24453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c24457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c32445a, 95 },
    { 0x4242445b, 87 },
    { 0, 0 },
    { 0xb1b2445d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1624462, 78 },
    { 0xbda24462, 99 },
    { 0, 0 },
    { 0x61124465, 68 },
    { 0, 0 },
    { 0x2c924467, 85 },
    { 0xd1424468, 34 },
    { 0x16724469, 23 },
    { 0x8da2446a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea24471, 34 },
    { 0xcb624471, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa2447b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70524482, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x61524485, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a124488, 98 },
    { 0x35524489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca924493, 76 },
    { 0, 0 },
    { 0xb8c24495, 33 },
    { 0xe9324496, 88 },
    { 0xe8124497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a82449b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb92449e, 33 },
    { 0x1c62449e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6244a8, 23 },
    { 0xb50244a9, 23 },
    { 0, 0 },
    { 0xf77244ab, 61 },
    { 0xfa9244ac, 65 },
    { 0x2dd244ac, 33 },
    { 0x56a244ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3244b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e244b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782244b8, 61 },
    { 0x56e244b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1244be, 100 },
    { 0x3be244bf, 98 },
    { 0x880244c0, 33 },
    { 0, 0 },
    { 0x583244c2, 76 },
    { 0x18c244c3, 56 },
    { 0x616244c4, 93 },
    { 0x278244c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8244ca, 40 },
    { 0x8e3244cb, 23 },
    { 0xaae244cc, 91 },
    { 0x55f244cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3244d2, 61 },
    { 0xfff244d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e244d9, 78 },
    { 0x327244d9, 91 },
    { 0, 0 },
    { 0x230244dc, 76 },
    { 0x900244dd, 23 },
    { 0, 0 },
    { 0x8ef244df, 55 },
    { 0x2e0244e0, 23 },
    { 0xd8244e1, 90 },
    { 0x6dc244e2, 93 },
    { 0xf10244e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83244e6, 61 },
    { 0x2cd244e7, 90 },
    { 0x7dd244e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77244ee, 47 },
    { 0xc90244ee, 90 },
    { 0, 0 },
    { 0x29b244f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555244f8, 78 },
    { 0x748244f8, 33 },
    { 0x977244fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82244fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15824501, 33 },
    { 0x91724502, 81 },
    { 0x1a424503, 38 },
    { 0x6e924502, 46 },
    { 0x80a24503, 23 },
    { 0x4ff24506, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c52450f, 23 },
    { 0x99224510, 61 },
    { 0x56624510, 76 },
    { 0x91624512, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7424515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f12451d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe224523, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4232452b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac42452e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f24532, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e24536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832453a, 23 },
    { 0xae12453b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85524542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c24545, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7272454e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30624552, 61 },
    { 0xa9824553, 44 },
    { 0, 0 },
    { 0x624555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1472455e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80624562, 92 },
    { 0x3ec24562, 33 },
    { 0, 0 },
    { 0x9624565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a224568, 34 },
    { 0x88724569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd2456e, 90 },
    { 0, 0 },
    { 0xe7e24570, 95 },
    { 0, 0 },
    { 0xabe24572, 93 },
    { 0xc824573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c24576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e2457f, 88 },
    { 0x7b124580, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0924586, 23 },
    { 0x29024587, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2458a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9312458e, 90 },
    { 0x5262458e, 85 },
    { 0xacf2458e, 98 },
    { 0x10524591, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae24598, 90 },
    { 0x8c924599, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b2459c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf2459f, 17 },
    { 0, 0 },
    { 0x805245a1, 46 },
    { 0, 0 },
    { 0xc8245a3, 34 },
    { 0x22e245a4, 76 },
    { 0x593245a4, 76 },
    { 0x8a3245a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369245b1, 43 },
    { 0xd08245b2, 23 },
    { 0x76245b3, 17 },
    { 0x654245b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2245b9, 96 },
    { 0, 0 },
    { 0x7e6245bb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96245be, 43 },
    { 0x72d245bf, 55 },
    { 0xd1c245bf, 55 },
    { 0x71c245c1, 52 },
    { 0, 0 },
    { 0xec6245c3, 93 },
    { 0xff2245c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388245cf, 99 },
    { 0xac5245d0, 85 },
    { 0xd1d245d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28245d8, 61 },
    { 0x2af245d9, 90 },
    { 0, 0 },
    { 0xf00245db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3245de, 61 },
    { 0x9b5245df, 33 },
    { 0x4fd245df, 75 },
    { 0x753245de, 23 },
    { 0xe87245e1, 25 },
    { 0x9b9245e3, 23 },
    { 0xacb245e4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x320245e7, 90 },
    { 0xa63245e7, 61 },
    { 0, 0 },
    { 0x9e6245ea, 87 },
    { 0x3f3245ea, 98 },
    { 0xa1a245ea, 23 },
    { 0x6d8245ea, 95 },
    { 0x307245ee, 23 },
    { 0, 0 },
    { 0x444245f0, 88 },
    { 0x3cc245f0, 33 },
    { 0, 0 },
    { 0x75f245f3, 35 },
    { 0x7b9245f4, 78 },
    { 0, 0 },
    { 0x354245f6, 56 },
    { 0xe99245f7, 91 },
    { 0x79a245f7, 76 },
    { 0x328245f9, 76 },
    { 0x8bf245f7, 90 },
    { 0xaf1245f8, 55 },
    { 0xec245fc, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7924602, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d724608, 33 },
    { 0xdd024608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed52460f, 90 },
    { 0, 0 },
    { 0xeb624611, 33 },
    { 0x64224612, 98 },
    { 0x95824613, 76 },
    { 0xc3a24614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd2461f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33624622, 76 },
    { 0x42c24623, 97 },
    { 0x52124624, 61 },
    { 0x3e924624, 95 },
    { 0x7ba24626, 33 },
    { 0x3ed24624, 34 },
    { 0xfa24626, 61 },
    { 0x24c24623, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2462d, 55 },
    { 0x79e2462d, 76 },
    { 0x23c2462f, 78 },
    { 0x95e2462f, 88 },
    { 0x4482462e, 67 },
    { 0x5a824632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b24635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c2463a, 100 },
    { 0xbce2463a, 61 },
    { 0xb9d2463a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21824642, 61 },
    { 0, 0 },
    { 0x53324644, 33 },
    { 0, 0 },
    { 0xa9a24646, 61 },
    { 0x62324646, 76 },
    { 0x91524648, 87 },
    { 0, 0 },
    { 0x5c2464a, 90 },
    { 0, 0 },
    { 0xd172464c, 90 },
    { 0xdb12464d, 75 },
    { 0, 0 },
    { 0x6622464f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba024654, 78 },
    { 0xb0f24655, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed82465b, 95 },
    { 0x2722465b, 99 },
    { 0, 0 },
    { 0xe9b2465e, 98 },
    { 0x9012465e, 69 },
    { 0xe1e24660, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb24663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49524667, 23 },
    { 0, 0 },
    { 0x85d24669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc772466e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e124671, 34 },
    { 0x6a24672, 33 },
    { 0, 0 },
    { 0x40024674, 91 },
    { 0, 0 },
    { 0xf2c24676, 23 },
    { 0x14d24676, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1002467d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc24680, 23 },
    { 0xc3024681, 23 },
    { 0x91824681, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a824685, 34 },
    { 0x76f24686, 84 },
    { 0x5024687, 34 },
    { 0xb24688, 77 },
    { 0x44124689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2024692, 90 },
    { 0x46324693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38924696, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f92469a, 23 },
    { 0xc692469b, 100 },
    { 0xbc12469c, 67 },
    { 0xe8a2469d, 55 },
    { 0xaee2469e, 55 },
    { 0xe362469f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753246a3, 23 },
    { 0x62e246a4, 33 },
    { 0, 0 },
    { 0xa29246a6, 90 },
    { 0, 0 },
    { 0x29a246a8, 33 },
    { 0x33d246a9, 93 },
    { 0, 0 },
    { 0x97e246ab, 43 },
    { 0xc7246ac, 98 },
    { 0xd5e246ad, 95 },
    { 0x5c9246ae, 23 },
    { 0x8ac246ab, 33 },
    { 0xd1f246b0, 98 },
    { 0, 0 },
    { 0x519246b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271246b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220246bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c246bf, 33 },
    { 0xdcc246c0, 90 },
    { 0x427246c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7246c4, 61 },
    { 0x83d246c5, 76 },
    { 0, 0 },
    { 0xc22246c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c246cd, 43 },
    { 0x95246ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35246d3, 61 },
    { 0x6e4246d3, 33 },
    { 0, 0 },
    { 0x19a246d6, 93 },
    { 0, 0 },
    { 0xeb0246d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2246dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d246e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8246e3, 98 },
    { 0x8c3246e3, 98 },
    { 0, 0 },
    { 0x424246e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e246eb, 33 },
    { 0xa5246eb, 66 },
    { 0xc75246eb, 98 },
    { 0x29d246ed, 33 },
    { 0x45c246ef, 86 },
    { 0xefe246f0, 61 },
    { 0xb21246f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291246f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2246fa, 78 },
    { 0xd4c246fa, 99 },
    { 0, 0 },
    { 0xe7e246fd, 95 },
    { 0x996246fe, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6824707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e2470a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf2470d, 43 },
    { 0x4f82470e, 61 },
    { 0x7f52470e, 93 },
    { 0, 0 },
    { 0x49224711, 90 },
    { 0, 0 },
    { 0x43024713, 76 },
    { 0x12324713, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c24719, 66 },
    { 0x26b2471a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7072471f, 40 },
    { 0xe082471f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b124727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7722472a, 23 },
    { 0x28d2472b, 95 },
    { 0x18c2472c, 90 },
    { 0, 0 },
    { 0x8332472e, 95 },
    { 0xacb2472f, 23 },
    { 0x88524730, 34 },
    { 0xf4424731, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54824738, 90 },
    { 0, 0 },
    { 0x4952473a, 76 },
    { 0, 0 },
    { 0x98d2473c, 68 },
    { 0x4db2473c, 90 },
    { 0x1c42473c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d724742, 64 },
    { 0xe9924742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272474c, 23 },
    { 0x45e2474d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d624750, 33 },
    { 0xab224751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d52475b, 95 },
    { 0x5df2475b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5692475f, 33 },
    { 0x17724760, 98 },
    { 0xa0c24761, 34 },
    { 0x5ca24760, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc24765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa24775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b524779, 61 },
    { 0xf062477a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf692477d, 61 },
    { 0xf8c2477e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74024783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d024786, 90 },
    { 0xe3b24787, 76 },
    { 0x4b524787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32478c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a2478f, 99 },
    { 0, 0 },
    { 0x61124791, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91824797, 52 },
    { 0, 0 },
    { 0x6124799, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef22479d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0247a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x836247a3, 99 },
    { 0xa00247a3, 23 },
    { 0, 0 },
    { 0x89247a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a247ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15247b3, 33 },
    { 0xeaf247b3, 55 },
    { 0x642247b4, 33 },
    { 0x19247b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0247b9, 78 },
    { 0xd83247b9, 99 },
    { 0, 0 },
    { 0x84a247bc, 34 },
    { 0x788247bd, 61 },
    { 0x9d9247be, 98 },
    { 0xcc7247be, 77 },
    { 0x4b1247c0, 23 },
    { 0x555247c1, 90 },
    { 0x772247c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3247c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8247ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951247d3, 61 },
    { 0x36e247d3, 61 },
    { 0x40f247d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6247db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff247e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b247e7, 95 },
    { 0x40247e8, 93 },
    { 0x335247e9, 61 },
    { 0, 0 },
    { 0x65a247eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c247f2, 90 },
    { 0x77c247f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef247f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc22480b, 98 },
    { 0xfa72480c, 61 },
    { 0x5792480b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d24810, 55 },
    { 0x43724811, 33 },
    { 0xcba24811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe924816, 23 },
    { 0xeb424816, 100 },
    { 0x4e324818, 99 },
    { 0xdf024819, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d2481c, 23 },
    { 0x7392481d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce124820, 23 },
    { 0x4f424821, 43 },
    { 0xfbf24821, 93 },
    { 0x96524823, 33 },
    { 0xb7d24822, 61 },
    { 0xec324824, 95 },
    { 0x87124825, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xace24829, 80 },
    { 0x18d2482a, 44 },
    { 0xa052482b, 88 },
    { 0xa3d24829, 99 },
    { 0x4762482d, 33 },
    { 0xae22482b, 55 },
    { 0x3662482f, 95 },
    { 0x2a224830, 76 },
    { 0xb8f24830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b224838, 66 },
    { 0xa3624839, 52 },
    { 0, 0 },
    { 0x59f2483b, 100 },
    { 0xaa32483c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf24840, 33 },
    { 0x9d424841, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e724846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d2484c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88124851, 55 },
    { 0x59c24852, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe24857, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f2485a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972485d, 95 },
    { 0xbf72485e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf624865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd24868, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc72486b, 87 },
    { 0xeff2486c, 90 },
    { 0xf922486b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b124870, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b024875, 98 },
    { 0x7d824875, 78 },
    { 0x8fa24877, 66 },
    { 0xc5524877, 52 },
    { 0xcdb24879, 76 },
    { 0x4c12487a, 23 },
    { 0xf2a2487a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72424883, 90 },
    { 0x20024884, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89424889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45124892, 77 },
    { 0, 0 },
    { 0xdd424894, 23 },
    { 0, 0 },
    { 0xb224896, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed2489f, 95 },
    { 0xd8a248a0, 61 },
    { 0x3b5248a0, 98 },
    { 0x982248a1, 33 },
    { 0x34e248a3, 55 },
    { 0xf2e248a4, 33 },
    { 0x73c248a3, 33 },
    { 0x5a4248a6, 98 },
    { 0x980248a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec248ac, 96 },
    { 0x667248ad, 55 },
    { 0xd2a248ac, 43 },
    { 0, 0 },
    { 0x616248b0, 61 },
    { 0, 0 },
    { 0x8fc248b2, 90 },
    { 0x1f0248b2, 90 },
    { 0xe1248b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f248bd, 95 },
    { 0, 0 },
    { 0xd2248bf, 67 },
    { 0x277248c0, 93 },
    { 0, 0 },
    { 0x402248c2, 52 },
    { 0, 0 },
    { 0xa02248c4, 90 },
    { 0, 0 },
    { 0xb83248c6, 40 },
    { 0x493248c7, 33 },
    { 0xc5e248c6, 76 },
    { 0, 0 },
    { 0x166248ca, 96 },
    { 0x3f5248cb, 23 },
    { 0xfa4248cb, 80 },
    { 0x4ac248cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d248d4, 90 },
    { 0, 0 },
    { 0xe16248d6, 98 },
    { 0xdf4248d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b248da, 67 },
    { 0, 0 },
    { 0x610248dc, 87 },
    { 0xfad248dc, 90 },
    { 0xe2e248de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416248e2, 81 },
    { 0, 0 },
    { 0xa63248e4, 100 },
    { 0xc3c248e5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x592248e8, 23 },
    { 0xcd248e9, 55 },
    { 0, 0 },
    { 0x555248eb, 90 },
    { 0x187248ec, 23 },
    { 0, 0 },
    { 0x3f7248ee, 68 },
    { 0, 0 },
    { 0x2b1248f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80248f5, 55 },
    { 0xc55248f5, 90 },
    { 0, 0 },
    { 0x3db248f8, 98 },
    { 0, 0 },
    { 0xf6c248fa, 90 },
    { 0x378248fb, 98 },
    { 0xde5248fb, 95 },
    { 0, 0 },
    { 0x21d248fe, 61 },
    { 0xa12248ff, 46 },
    { 0, 0 },
    { 0x3ea24901, 33 },
    { 0xc4924901, 98 },
    { 0, 0 },
    { 0x1c24904, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1442490a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f2490d, 90 },
    { 0x3882490d, 33 },
    { 0xad42490f, 90 },
    { 0, 0 },
    { 0x4724911, 33 },
    { 0xf0624912, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa924918, 98 },
    { 0x65324919, 76 },
    { 0x27524918, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b2491d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59224920, 66 },
    { 0xf1624921, 33 },
    { 0x60024922, 91 },
    { 0xc0e24920, 61 },
    { 0xd2b24920, 23 },
    { 0x52324922, 87 },
    { 0xc224925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3942492a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7912492e, 95 },
    { 0x99b2492f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e24932, 87 },
    { 0xcbc24932, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77124939, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8752493c, 23 },
    { 0x6f42493c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e24942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8424945, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b24949, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5162494c, 84 },
    { 0xedd2494c, 33 },
    { 0, 0 },
    { 0xef52494f, 34 },
    { 0, 0 },
    { 0x8a724951, 90 },
    { 0x78724952, 90 },
    { 0x8c24953, 66 },
    { 0, 0 },
    { 0xae724955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd2495c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac24960, 68 },
    { 0x7824961, 83 },
    { 0x37d24962, 95 },
    { 0, 0 },
    { 0x2ac24964, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7102496f, 33 },
    { 0xf892496f, 99 },
    { 0x8b624971, 66 },
    { 0, 0 },
    { 0xac024973, 90 },
    { 0xe4e24974, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x12324977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6262497b, 78 },
    { 0x8392497b, 46 },
    { 0xac12497b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6924984, 23 },
    { 0, 0 },
    { 0x4224986, 81 },
    { 0x20c24987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4822498b, 95 },
    { 0x98e2498b, 98 },
    { 0x7a42498d, 61 },
    { 0x8f92498e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e24995, 91 },
    { 0x47d24995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf982499a, 66 },
    { 0x3132499a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a2499e, 90 },
    { 0x31e2499f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f249b3, 23 },
    { 0x352249b3, 33 },
    { 0, 0 },
    { 0x216249b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9249ba, 33 },
    { 0, 0 },
    { 0x21f249bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf249c8, 90 },
    { 0, 0 },
    { 0x176249ca, 23 },
    { 0x961249cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985249d2, 52 },
    { 0xce249d3, 33 },
    { 0x99e249d4, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377249da, 66 },
    { 0xa48249db, 91 },
    { 0, 0 },
    { 0xff6249dd, 90 },
    { 0xa1b249de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831249e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57249e6, 98 },
    { 0x988249e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c249ea, 67 },
    { 0, 0 },
    { 0x741249ec, 33 },
    { 0, 0 },
    { 0x8e0249ee, 55 },
    { 0xa70249ef, 93 },
    { 0x8eb249f0, 95 },
    { 0x1b8249f1, 98 },
    { 0x4e9249f2, 33 },
    { 0xd07249f2, 91 },
    { 0x755249f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e249f7, 95 },
    { 0, 0 },
    { 0x6db249f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8249ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a24a05, 93 },
    { 0x4724a06, 61 },
    { 0xb0324a07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc24a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab24a12, 98 },
    { 0xd324a13, 95 },
    { 0x7ff24a13, 33 },
    { 0x86724a15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8224a1b, 88 },
    { 0x6f924a1b, 76 },
    { 0, 0 },
    { 0x3e324a1e, 61 },
    { 0xb5424a1f, 69 },
    { 0xb0b24a20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f024a23, 93 },
    { 0, 0 },
    { 0x49524a25, 33 },
    { 0x31124a25, 93 },
    { 0x8fa24a26, 34 },
    { 0x15124a28, 98 },
    { 0x30624a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f124a34, 91 },
    { 0, 0 },
    { 0xf0724a36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f924a43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e924a47, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b24a4a, 61 },
    { 0x4b224a4b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0924a50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4524a54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b24a57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9424a5c, 34 },
    { 0, 0 },
    { 0xd5424a5e, 48 },
    { 0xeb824a5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56024a64, 33 },
    { 0xdbb24a64, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x51224a68, 61 },
    { 0xb6924a68, 78 },
    { 0xc0724a69, 61 },
    { 0, 0 },
    { 0x6df24a6c, 65 },
    { 0x36924a6c, 90 },
    { 0x8cf24a6e, 46 },
    { 0x2e024a6d, 90 },
    { 0xa6424a6c, 67 },
    { 0xb3e24a6c, 90 },
    { 0x59f24a6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67124a76, 84 },
    { 0, 0 },
    { 0x5bd24a78, 23 },
    { 0xce924a78, 43 },
    { 0xd0024a78, 95 },
    { 0xebd24a7a, 76 },
    { 0xd5a24a7a, 23 },
    { 0, 0 },
    { 0x66024a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab724a82, 61 },
    { 0, 0 },
    { 0xb5724a84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4024a87, 98 },
    { 0x69e24a88, 100 },
    { 0x79424a87, 90 },
    { 0x6a124a8a, 95 },
    { 0xaf024a8a, 75 },
    { 0xdec24a87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6424a97, 98 },
    { 0x17324a98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc924a9b, 40 },
    { 0xc8224a9c, 75 },
    { 0x73224a9c, 75 },
    { 0x5924a9e, 34 },
    { 0xa2424a9f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad324aa8, 61 },
    { 0, 0 },
    { 0xbe724aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35824aad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40024ab2, 95 },
    { 0, 0 },
    { 0x7fd24ab4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e124ab9, 61 },
    { 0x5e324aba, 99 },
    { 0, 0 },
    { 0xb2024abc, 55 },
    { 0xf0524abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e524ac2, 77 },
    { 0xcba24ac3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0124ac9, 76 },
    { 0xb0b24ac9, 76 },
    { 0, 0 },
    { 0x84824acc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe24ad1, 95 },
    { 0, 0 },
    { 0x25d24ad3, 34 },
    { 0, 0 },
    { 0xfdb24ad5, 45 },
    { 0x75624ad6, 23 },
    { 0, 0 },
    { 0xf6724ad8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba24adb, 69 },
    { 0xb1e24adc, 33 },
    { 0, 0 },
    { 0x42124ade, 34 },
    { 0x38b24ade, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c24ae2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87924ae5, 66 },
    { 0x3e324ae5, 55 },
    { 0, 0 },
    { 0x35424ae8, 90 },
    { 0, 0 },
    { 0xe4624aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe724aed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb724af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c424af3, 98 },
    { 0x2e324af4, 33 },
    { 0xefa24af3, 90 },
    { 0, 0 },
    { 0xc1a24af7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa824b02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56524b06, 34 },
    { 0, 0 },
    { 0x54224b08, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24324b12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f624b18, 55 },
    { 0x9d124b19, 33 },
    { 0xf5124b19, 34 },
    { 0xc2424b1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3124b26, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81924b2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73624b30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a324b35, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a24b39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d24b3c, 68 },
    { 0x6f624b3c, 23 },
    { 0x2aa24b3e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x24724b41, 61 },
    { 0xe824b42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce724b49, 23 },
    { 0, 0 },
    { 0xff724b4b, 24 },
    { 0xeac24b4b, 88 },
    { 0x4a424b4b, 90 },
    { 0xd7524b4c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1024b56, 23 },
    { 0, 0 },
    { 0x5a724b58, 23 },
    { 0x1c24b59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c24b5e, 48 },
    { 0, 0 },
    { 0x34324b60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e24b66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de24b6f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58824b74, 95 },
    { 0x28724b74, 98 },
    { 0x7bb24b75, 23 },
    { 0, 0 },
    { 0x7dc24b78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc24b7b, 95 },
    { 0x60d24b7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e24b7f, 90 },
    { 0x66324b80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c24b87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b324b8a, 93 },
    { 0, 0 },
    { 0x84924b8c, 88 },
    { 0, 0 },
    { 0xd8724b8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3724b91, 95 },
    { 0xf7624b92, 55 },
    { 0x7c024b93, 43 },
    { 0xa3524b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f424b99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d24ba4, 98 },
    { 0xd4224ba5, 23 },
    { 0xda624ba6, 61 },
    { 0x32224ba7, 19 },
    { 0, 0 },
    { 0xf824ba9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb624bae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da24bb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab24bba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x30524bbd, 100 },
    { 0xc2e24bbe, 33 },
    { 0x79f24bbe, 90 },
    { 0x7224bbf, 33 },
    { 0, 0 },
    { 0x82524bc2, 84 },
    { 0xb3c24bc3, 90 },
    { 0xa9624bc4, 76 },
    { 0x38524bc4, 90 },
    { 0x3b624bc6, 95 },
    { 0, 0 },
    { 0x92024bc8, 75 },
    { 0x11824bc9, 33 },
    { 0, 0 },
    { 0x84f24bcb, 76 },
    { 0x9d24bcc, 61 },
    { 0xf1224bcc, 55 },
    { 0xde124bce, 90 },
    { 0xdcb24bcf, 78 },
    { 0xa2f24bcf, 23 },
    { 0x5b124bcf, 61 },
    { 0x61624bcc, 43 },
    { 0xebb24bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c224bd6, 78 },
    { 0x36e24bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a24bdb, 34 },
    { 0, 0 },
    { 0x4ae24bdd, 33 },
    { 0xf8724bdd, 33 },
    { 0xc0d24bde, 61 },
    { 0x13724be0, 52 },
    { 0xaba24be0, 98 },
    { 0x7ac24be2, 95 },
    { 0xa9224be3, 55 },
    { 0, 0 },
    { 0xe7424be5, 85 },
    { 0x11c24be6, 33 },
    { 0x7c624be7, 98 },
    { 0x77b24be7, 90 },
    { 0x7b124be7, 61 },
    { 0x96e24bea, 98 },
    { 0, 0 },
    { 0x9df24bec, 76 },
    { 0, 0 },
    { 0x28424bee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db24bf5, 93 },
    { 0xae524bf6, 93 },
    { 0xee624bf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f24bfd, 88 },
    { 0xc4b24bfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0524c03, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f824c09, 90 },
    { 0x34424c09, 33 },
    { 0xf8e24c0a, 46 },
    { 0x76e24c09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95324c15, 33 },
    { 0x1aa24c16, 43 },
    { 0x68624c17, 98 },
    { 0, 0 },
    { 0xf9324c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a624c23, 23 },
    { 0, 0 },
    { 0x2f024c25, 34 },
    { 0xe7c24c26, 90 },
    { 0, 0 },
    { 0xf9024c28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9524c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57124c30, 95 },
    { 0x90924c31, 61 },
    { 0x28924c32, 36 },
    { 0x56c24c33, 23 },
    { 0x67a24c34, 98 },
    { 0x66324c34, 95 },
    { 0, 0 },
    { 0x15d24c37, 90 },
    { 0x2c924c37, 23 },
    { 0xcdf24c38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d224c3e, 95 },
    { 0xa9224c3f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x97424c42, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad24c45, 61 },
    { 0xf0124c46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11324c50, 23 },
    { 0xed824c51, 87 },
    { 0xe5324c52, 95 },
    { 0, 0 },
    { 0xb2124c54, 23 },
    { 0xb9c24c55, 87 },
    { 0xf9c24c55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56124c59, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a24c5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f124c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13024c65, 61 },
    { 0xe8024c65, 43 },
    { 0xa9e24c65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb24c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24824c77, 55 },
    { 0, 0 },
    { 0x30624c79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf324c7d, 17 },
    { 0x9fb24c7e, 43 },
    { 0xcf224c7d, 55 },
    { 0, 0 },
    { 0x9bb24c81, 95 },
    { 0, 0 },
    { 0xffc24c83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9324c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e24c8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee324c95, 66 },
    { 0x2e24c96, 88 },
    { 0x6bb24c96, 98 },
    { 0x6e924c95, 98 },
    { 0x1de24c99, 95 },
    { 0xea124c9a, 68 },
    { 0xdef24c97, 95 },
    { 0x7d024c97, 75 },
    { 0, 0 },
    { 0xba224c9e, 76 },
    { 0x16324c9f, 46 },
    { 0xc5924c9f, 33 },
    { 0, 0 },
    { 0x13c24ca2, 76 },
    { 0, 0 },
    { 0xddf24ca4, 87 },
    { 0, 0 },
    { 0x87d24ca6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66024ca9, 95 },
    { 0x56824caa, 34 },
    { 0xe9124cab, 90 },
    { 0xc7c24cac, 91 },
    { 0x7d524cad, 99 },
    { 0x5ab24cac, 98 },
    { 0x65624caf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f24cb5, 68 },
    { 0, 0 },
    { 0xee724cb7, 55 },
    { 0x6d624cb8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd24cc4, 61 },
    { 0, 0 },
    { 0xd124cc6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf324ccb, 100 },
    { 0x84324ccc, 33 },
    { 0, 0 },
    { 0x39624cce, 34 },
    { 0xf3624ccf, 68 },
    { 0x94724cd0, 90 },
    { 0x40624cd1, 43 },
    { 0, 0 },
    { 0x98c24cd3, 90 },
    { 0xc6b24cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc924cdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20224ce3, 33 },
    { 0, 0 },
    { 0x50b24ce5, 98 },
    { 0x47f24ce6, 23 },
    { 0x54124ce7, 23 },
    { 0, 0 },
    { 0xe8524ce9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed24ced, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b724cf2, 90 },
    { 0x54a24cf2, 33 },
    { 0x9ca24cf4, 90 },
    { 0xa3e24cf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a424cf8, 75 },
    { 0, 0 },
    { 0xd2224cfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde24d00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f24d07, 17 },
    { 0xb8124d08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80024d0d, 90 },
    { 0xfc24d0e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x75024d11, 95 },
    { 0xd1c24d12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52624d1a, 33 },
    { 0x21b24d1b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7224d1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8224d22, 95 },
    { 0xad124d22, 95 },
    { 0x4a324d24, 78 },
    { 0x99a24d25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c24d2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f24d30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee924d34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6524d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd024d3e, 61 },
    { 0, 0 },
    { 0x84b24d40, 95 },
    { 0, 0 },
    { 0xe8e24d42, 99 },
    { 0, 0 },
    { 0x23b24d44, 66 },
    { 0x18024d45, 90 },
    { 0x17524d46, 23 },
    { 0x36b24d47, 33 },
    { 0, 0 },
    { 0x24024d49, 23 },
    { 0, 0 },
    { 0x2b324d4b, 61 },
    { 0xa724d4c, 78 },
    { 0x52924d4d, 95 },
    { 0x74024d4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1824d51, 59 },
    { 0, 0 },
    { 0x38124d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79024d58, 90 },
    { 0, 0 },
    { 0xcf24d5a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac24d60, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc24d67, 61 },
    { 0x13924d68, 33 },
    { 0, 0 },
    { 0x94424d6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa024d6f, 98 },
    { 0x22424d6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea24d74, 61 },
    { 0x3f24d75, 76 },
    { 0x18324d76, 46 },
    { 0x6db24d77, 93 },
    { 0xf8524d77, 34 },
    { 0xf6624d78, 23 },
    { 0x19b24d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9824d7e, 33 },
    { 0x5ea24d7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c024d82, 56 },
    { 0, 0 },
    { 0x8bd24d84, 61 },
    { 0x93424d85, 55 },
    { 0, 0 },
    { 0xe7e24d87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe824d8a, 93 },
    { 0x56c24d8b, 68 },
    { 0, 0 },
    { 0x3d924d8d, 87 },
    { 0x90d24d8e, 83 },
    { 0xf5b24d8f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa624d93, 90 },
    { 0x74724d93, 78 },
    { 0x3da24d95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed24d9a, 91 },
    { 0x92824d9b, 33 },
    { 0x40c24d9c, 33 },
    { 0xb1124d9c, 33 },
    { 0xe6824d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c24da1, 33 },
    { 0x81824da2, 33 },
    { 0x3524da3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59824da6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db24da9, 90 },
    { 0x7a724daa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75424dad, 88 },
    { 0xb2724dad, 23 },
    { 0x77d24dae, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7024db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b624dbc, 40 },
    { 0x98124dbc, 23 },
    { 0x5ea24dbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a24dc1, 23 },
    { 0x72824dc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed124dc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c24dc8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41024dcc, 55 },
    { 0, 0 },
    { 0x21e24dce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec324dd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed24dd7, 61 },
    { 0x11d24dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3024ddc, 61 },
    { 0xd9c24ddc, 46 },
    { 0, 0 },
    { 0xce224ddf, 33 },
    { 0x20c24de0, 23 },
    { 0xa7824de0, 23 },
    { 0x3e324de0, 76 },
    { 0x7b024de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d24de8, 53 },
    { 0x97d24de8, 55 },
    { 0x6a24dea, 76 },
    { 0x9ed24deb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f24def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca524df2, 76 },
    { 0x96f24df3, 23 },
    { 0xf0e24df4, 66 },
    { 0x24924df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c124e05, 67 },
    { 0x20d24e05, 43 },
    { 0x3bf24e07, 55 },
    { 0x42424e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85924e0c, 90 },
    { 0xe6924e0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e24e10, 90 },
    { 0, 0 },
    { 0x3e24e12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b24e15, 33 },
    { 0, 0 },
    { 0x78324e17, 76 },
    { 0, 0 },
    { 0x50224e19, 46 },
    { 0, 0 },
    { 0x48724e1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e24e23, 99 },
    { 0xf0324e24, 43 },
    { 0x4b924e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a224e32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99824e36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1824e39, 43 },
    { 0, 0 },
    { 0x71024e3b, 61 },
    { 0, 0 },
    { 0xc5224e3d, 55 },
    { 0x34824e3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d24e41, 52 },
    { 0x21c24e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a24e46, 34 },
    { 0, 0 },
    { 0xd8524e48, 61 },
    { 0, 0 },
    { 0x5b424e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46924e4f, 61 },
    { 0, 0 },
    { 0xd4024e51, 100 },
    { 0xe3524e51, 77 },
    { 0xc8a24e53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39524e5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x45024e5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc624e62, 98 },
    { 0x82b24e63, 95 },
    { 0, 0 },
    { 0xf3824e65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2824e6c, 98 },
    { 0x4dd24e6c, 91 },
    { 0xfbe24e6e, 23 },
    { 0x67924e6e, 38 },
    { 0xece24e70, 90 },
    { 0x5a24e71, 93 },
    { 0x2b124e6e, 98 },
    { 0xfb224e73, 23 },
    { 0x3a624e74, 84 },
    { 0x4324e6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c24e7b, 23 },
    { 0xf6824e7b, 90 },
    { 0, 0 },
    { 0xbf824e7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0824e82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b24e86, 55 },
    { 0, 0 },
    { 0x56724e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9224e8d, 87 },
    { 0xbd124e8e, 23 },
    { 0, 0 },
    { 0x20224e90, 84 },
    { 0x6ae24e90, 95 },
    { 0x2bb24e91, 61 },
    { 0x91724e93, 87 },
    { 0x26a24e94, 100 },
    { 0, 0 },
    { 0x99324e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53924e9b, 61 },
    { 0x47d24e9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5124ea3, 98 },
    { 0xf1424ea4, 91 },
    { 0, 0 },
    { 0x5524ea6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf24ead, 61 },
    { 0, 0 },
    { 0x5c124eaf, 76 },
    { 0, 0 },
    { 0x63a24eb1, 52 },
    { 0, 0 },
    { 0xaeb24eb3, 25 },
    { 0, 0 },
    { 0x54824eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58324ec0, 98 },
    { 0x3ad24ec1, 61 },
    { 0x5d724ec2, 23 },
    { 0x72324ec2, 17 },
    { 0xde324ec0, 99 },
    { 0, 0 },
    { 0xf0b24ec6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xec224ec9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d24ed0, 91 },
    { 0x34b24ed1, 90 },
    { 0x3824ed2, 90 },
    { 0x47324ed2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ed8, 61 },
    { 0xeb024ed9, 95 },
    { 0, 0 },
    { 0x18b24edb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd24ede, 33 },
    { 0, 0 },
    { 0x2c824ee0, 93 },
    { 0x48a24ee1, 62 },
    { 0xa4924ee2, 93 },
    { 0x4c224ee2, 76 },
    { 0, 0 },
    { 0xa0d24ee5, 78 },
    { 0x2c324ee5, 33 },
    { 0x12b24ee7, 23 },
    { 0x5a124ee8, 98 },
    { 0x36b24ee7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa24eed, 81 },
    { 0xcfb24eee, 95 },
    { 0xd0124eef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de24efa, 99 },
    { 0x76e24efa, 100 },
    { 0, 0 },
    { 0xb7024efd, 43 },
    { 0x60e24efe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f24f01, 98 },
    { 0x97624f01, 55 },
    { 0, 0 },
    { 0x49124f04, 76 },
    { 0x24924f05, 91 },
    { 0xbcb24f06, 90 },
    { 0x84624f07, 98 },
    { 0x6f324f07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f24f12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe224f15, 95 },
    { 0, 0 },
    { 0x89224f17, 23 },
    { 0x5f224f18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b24f1b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad24f24, 91 },
    { 0xf0524f25, 33 },
    { 0x82524f24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a124f2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab24f34, 98 },
    { 0xffc24f35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8924f38, 95 },
    { 0, 0 },
    { 0xef524f3a, 33 },
    { 0x39f24f3b, 33 },
    { 0xc1224f3c, 33 },
    { 0, 0 },
    { 0x4a24f3e, 33 },
    { 0xd5b24f3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee424f44, 95 },
    { 0, 0 },
    { 0x7bc24f46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b24f4a, 33 },
    { 0x7924f4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f524f4f, 34 },
    { 0x7b724f4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1824f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c24f59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5124f5c, 100 },
    { 0x11d24f5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3724f61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a124f64, 95 },
    { 0xdfe24f65, 23 },
    { 0x80e24f66, 99 },
    { 0x3224f67, 98 },
    { 0, 0 },
    { 0x44524f69, 61 },
    { 0x12c24f6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc24f6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0124f74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd24f77, 69 },
    { 0x79924f78, 34 },
    { 0x51424f79, 36 },
    { 0xf0a24f7a, 61 },
    { 0x10924f7a, 61 },
    { 0xe8624f7c, 23 },
    { 0xdf124f7c, 61 },
    { 0xfba24f7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e024f81, 61 },
    { 0xf224f81, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x95224f85, 91 },
    { 0xbae24f85, 93 },
    { 0, 0 },
    { 0xc7224f88, 43 },
    { 0xee024f89, 98 },
    { 0x6f424f8a, 61 },
    { 0xa024f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd624f91, 28 },
    { 0, 0 },
    { 0x31c24f93, 90 },
    { 0, 0 },
    { 0x89c24f95, 84 },
    { 0, 0 },
    { 0x20524f97, 61 },
    { 0x86a24f98, 23 },
    { 0xef124f99, 90 },
    { 0x4c024f99, 67 },
    { 0xa2a24f9a, 43 },
    { 0, 0 },
    { 0x9cd24f9d, 84 },
    { 0xe0f24f9e, 86 },
    { 0x97e24f9f, 90 },
    { 0xf9e24f9d, 91 },
    { 0, 0 },
    { 0x18a24fa2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99524fa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f24faf, 55 },
    { 0x45024faf, 88 },
    { 0x20a24fb0, 98 },
    { 0xe5c24fb2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc024fb6, 95 },
    { 0, 0 },
    { 0x50124fb8, 43 },
    { 0x51c24fb9, 90 },
    { 0x36224fba, 61 },
    { 0, 0 },
    { 0x2a24fbc, 33 },
    { 0x52a24fbd, 81 },
    { 0x5a024fbd, 61 },
    { 0, 0 },
    { 0xae124fc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2224fc5, 46 },
    { 0xc1924fc5, 95 },
    { 0, 0 },
    { 0x76c24fc8, 94 },
    { 0, 0 },
    { 0xcb224fca, 78 },
    { 0x37724fca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e24fd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d24fd5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1024fd8, 66 },
    { 0x30a24fd8, 95 },
    { 0xf5a24fda, 35 },
    { 0xe5624fda, 33 },
    { 0xdd524fdc, 23 },
    { 0x37524fdd, 98 },
    { 0x4224fdd, 68 },
    { 0x5e024fdf, 33 },
    { 0x7c24fe0, 87 },
    { 0xffe24fd8, 36 },
    { 0x86e24fde, 93 },
    { 0, 0 },
    { 0x43524fe4, 52 },
    { 0xa3824fe4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4824feb, 61 },
    { 0x9024fec, 95 },
    { 0x48b24feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef224ff2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e624ff9, 90 },
    { 0, 0 },
    { 0xa3724ffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a624ffe, 90 },
    { 0xf1024fff, 61 },
    { 0x2524fff, 33 },
    { 0xcb924fff, 98 },
    { 0, 0 },
    { 0x1ef25003, 66 },
    { 0xf0425003, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38925008, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b2500b, 33 },
    { 0xd802500c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf425015, 55 },
    { 0x6e525015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf102501a, 95 },
    { 0, 0 },
    { 0xe952501c, 8 },
    { 0x6f32501c, 61 },
    { 0x59f2501d, 23 },
    { 0x7392501f, 76 },
    { 0x9232501d, 23 },
    { 0x30125021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c525024, 65 },
    { 0x6fa25025, 23 },
    { 0xdd625026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da2502b, 82 },
    { 0, 0 },
    { 0x2982502d, 87 },
    { 0xd4c2502e, 80 },
    { 0, 0 },
    { 0x37b25030, 33 },
    { 0xb4025030, 46 },
    { 0xa3725030, 90 },
    { 0x60425033, 61 },
    { 0xc3e25033, 33 },
    { 0xe3525035, 91 },
    { 0xeb825034, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa592503a, 55 },
    { 0xfce2503b, 61 },
    { 0, 0 },
    { 0xb0e2503d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad425045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13425049, 17 },
    { 0x50a2504a, 91 },
    { 0x6892504a, 61 },
    { 0xb3c2504c, 33 },
    { 0xbd32504c, 87 },
    { 0, 0 },
    { 0xc192504f, 101 },
    { 0xe225050, 33 },
    { 0x82d25051, 33 },
    { 0x66425052, 34 },
    { 0xd6a25050, 33 },
    { 0, 0 },
    { 0x82725055, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb62505d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2625064, 90 },
    { 0xaa325064, 46 },
    { 0x51425066, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef72506b, 76 },
    { 0x3a02506b, 17 },
    { 0x9322506d, 95 },
    { 0x7032506d, 55 },
    { 0x3fe2506f, 90 },
    { 0xc4125070, 23 },
    { 0, 0 },
    { 0xc1125072, 33 },
    { 0x72f25073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1725076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67925079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252507c, 90 },
    { 0, 0 },
    { 0x71f2507e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38425081, 90 },
    { 0x9d625082, 33 },
    { 0x8d125082, 55 },
    { 0xc3525082, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb102508f, 55 },
    { 0, 0 },
    { 0xd1725091, 91 },
    { 0x3e225091, 95 },
    { 0x4ac25093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb72509a, 90 },
    { 0xd342509b, 90 },
    { 0, 0 },
    { 0x89e2509d, 61 },
    { 0xcf42509d, 43 },
    { 0x7332509f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e250a2, 95 },
    { 0xff0250a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982250a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454250ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd250b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd250b4, 23 },
    { 0x5fd250b4, 84 },
    { 0xc6b250b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c250bb, 33 },
    { 0xf4c250bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0250c3, 89 },
    { 0x90f250c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740250c8, 82 },
    { 0x66c250c8, 52 },
    { 0, 0 },
    { 0x20f250cb, 43 },
    { 0, 0 },
    { 0x81a250cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e250d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299250d4, 23 },
    { 0xb9b250d5, 23 },
    { 0x2d9250d4, 98 },
    { 0, 0 },
    { 0xf6e250d8, 80 },
    { 0, 0 },
    { 0x8c1250da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c250dd, 55 },
    { 0x69a250de, 98 },
    { 0, 0 },
    { 0xc64250e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f250e3, 55 },
    { 0, 0 },
    { 0x5a6250e5, 34 },
    { 0x236250e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19250e9, 23 },
    { 0x544250ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4250f2, 91 },
    { 0, 0 },
    { 0x262250f4, 95 },
    { 0x87a250f5, 95 },
    { 0xd6f250f6, 95 },
    { 0xdd8250f7, 23 },
    { 0x1a3250f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x484250fb, 87 },
    { 0, 0 },
    { 0xd68250fd, 34 },
    { 0x992250fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1825111, 98 },
    { 0, 0 },
    { 0x72f25113, 36 },
    { 0, 0 },
    { 0x8b525115, 93 },
    { 0x88125116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15225122, 43 },
    { 0xcef25123, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb925126, 90 },
    { 0, 0 },
    { 0x93625128, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7332512e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xef725131, 61 },
    { 0, 0 },
    { 0x95825133, 46 },
    { 0, 0 },
    { 0x9f225135, 95 },
    { 0x4e025136, 81 },
    { 0, 0 },
    { 0xc9f25138, 76 },
    { 0, 0 },
    { 0x1b42513a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd002513d, 95 },
    { 0xd9a2513e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c25144, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952514c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9225153, 76 },
    { 0xa0725154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e625157, 23 },
    { 0xa7525157, 23 },
    { 0xede25159, 55 },
    { 0x2602515a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc52515e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d25164, 95 },
    { 0x53125164, 95 },
    { 0x64825166, 33 },
    { 0x97f25167, 23 },
    { 0xf1125168, 17 },
    { 0x90625169, 43 },
    { 0x91625164, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2516d, 36 },
    { 0x1192516e, 88 },
    { 0x1032516e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd925172, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c25175, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d25179, 98 },
    { 0x14425179, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c32517f, 68 },
    { 0, 0 },
    { 0x58f25181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19225185, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a25189, 66 },
    { 0x1522518a, 91 },
    { 0xf1a2518b, 67 },
    { 0x8102518a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc782518f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7925194, 91 },
    { 0, 0 },
    { 0xe2225196, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d2519b, 55 },
    { 0xb332519c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7251a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db251a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8251b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d251ba, 33 },
    { 0x8ea251ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0251c2, 43 },
    { 0, 0 },
    { 0xdda251c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d251cc, 95 },
    { 0x83d251cc, 95 },
    { 0xcd1251ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d251d1, 33 },
    { 0xee9251d1, 34 },
    { 0, 0 },
    { 0x30251d4, 90 },
    { 0x9d7251d5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x119251d8, 55 },
    { 0, 0 },
    { 0xe6a251da, 93 },
    { 0xe00251db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x624251de, 33 },
    { 0xbf3251df, 90 },
    { 0x595251e0, 98 },
    { 0xf6251e0, 91 },
    { 0xbc251df, 95 },
    { 0x49d251e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e251e6, 84 },
    { 0x746251e7, 99 },
    { 0x3e6251e7, 88 },
    { 0x6d7251e6, 33 },
    { 0xe19251ea, 23 },
    { 0x15d251ea, 76 },
    { 0, 0 },
    { 0xf36251ed, 60 },
    { 0x301251ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2251f2, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2251f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d25203, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d82520f, 75 },
    { 0, 0 },
    { 0x1ca25211, 98 },
    { 0xabc25211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df25215, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a25218, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c62521c, 46 },
    { 0x8ff2521d, 90 },
    { 0, 0 },
    { 0xdb82521f, 90 },
    { 0, 0 },
    { 0x3025221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20825224, 34 },
    { 0xab25225, 33 },
    { 0xe1a25225, 61 },
    { 0xfcc25227, 90 },
    { 0x6725228, 78 },
    { 0, 0 },
    { 0x6812522a, 61 },
    { 0x9d2522b, 95 },
    { 0xa592522c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9022522f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd2523d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a25241, 33 },
    { 0, 0 },
    { 0x2cb25243, 61 },
    { 0, 0 },
    { 0x5eb25245, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b2524a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b025253, 98 },
    { 0, 0 },
    { 0x56b25255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68425259, 34 },
    { 0x7222525a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd25263, 90 },
    { 0xe825263, 98 },
    { 0x3f025263, 61 },
    { 0, 0 },
    { 0xee225267, 61 },
    { 0xbc025268, 99 },
    { 0xb3125267, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd322526f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b525273, 34 },
    { 0, 0 },
    { 0xc5925275, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f62527e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd25282, 77 },
    { 0, 0 },
    { 0xa1525284, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf2528a, 23 },
    { 0x7a12528b, 78 },
    { 0, 0 },
    { 0x69c2528d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0925295, 23 },
    { 0, 0 },
    { 0xebe25297, 95 },
    { 0x60425297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b2529d, 95 },
    { 0x3c92529e, 77 },
    { 0x7fe2529d, 75 },
    { 0x31f252a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12252a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c252aa, 90 },
    { 0x349252aa, 95 },
    { 0xef4252ac, 34 },
    { 0x706252ad, 61 },
    { 0x81252ae, 76 },
    { 0x58252af, 17 },
    { 0x498252ae, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28252b9, 33 },
    { 0, 0 },
    { 0xd7d252bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88252bf, 34 },
    { 0xd7b252c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33252c3, 61 },
    { 0x1e3252c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a252cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a252cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448252d4, 61 },
    { 0x8e5252d5, 87 },
    { 0x2ec252d5, 90 },
    { 0x834252d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968252df, 90 },
    { 0xb7f252df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xede252e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7252ec, 55 },
    { 0xa74252ec, 91 },
    { 0x9ec252ee, 77 },
    { 0xaae252ef, 33 },
    { 0, 0 },
    { 0x8d3252f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a252f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b252fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2225300, 95 },
    { 0x91225301, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e25304, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe125307, 69 },
    { 0x6ee25308, 23 },
    { 0, 0 },
    { 0x5542530a, 81 },
    { 0, 0 },
    { 0xb082530c, 95 },
    { 0, 0 },
    { 0x14a2530e, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d2531b, 100 },
    { 0, 0 },
    { 0x9c82531d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b25321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d25325, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa12532a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5072532e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c25331, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4125335, 36 },
    { 0x4d825336, 68 },
    { 0x18225337, 95 },
    { 0, 0 },
    { 0xfff25339, 90 },
    { 0x1882533a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d2533e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d925341, 87 },
    { 0x25f25341, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc25347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f2534d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4725351, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d825356, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de2535d, 90 },
    { 0, 0 },
    { 0x7072535f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f625362, 75 },
    { 0, 0 },
    { 0x8c025364, 98 },
    { 0x62025365, 87 },
    { 0, 0 },
    { 0x18425367, 78 },
    { 0xd8725367, 40 },
    { 0, 0 },
    { 0x9c72536a, 43 },
    { 0, 0 },
    { 0x2932536c, 81 },
    { 0, 0 },
    { 0x4f22536e, 90 },
    { 0x2df2536f, 78 },
    { 0xe282536f, 40 },
    { 0x30925371, 46 },
    { 0x96c25372, 78 },
    { 0xf3c25371, 90 },
    { 0xc1d25374, 34 },
    { 0x6242536f, 23 },
    { 0x4af25375, 76 },
    { 0x4b625377, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c25382, 81 },
    { 0x36525382, 46 },
    { 0x4cb25383, 95 },
    { 0x52425385, 84 },
    { 0, 0 },
    { 0xb8125387, 95 },
    { 0x3a925387, 23 },
    { 0x4cd25388, 88 },
    { 0xacb25388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32538d, 69 },
    { 0x182538e, 90 },
    { 0, 0 },
    { 0x87425390, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f425394, 75 },
    { 0x86025394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d25398, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7942539c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4652539f, 76 },
    { 0, 0 },
    { 0x55d253a1, 93 },
    { 0x581253a1, 90 },
    { 0x579253a3, 61 },
    { 0x9a9253a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a253a7, 33 },
    { 0xff253a8, 61 },
    { 0x730253a9, 76 },
    { 0, 0 },
    { 0xd79253ab, 61 },
    { 0x328253ac, 33 },
    { 0x767253ad, 76 },
    { 0x7b6253ae, 90 },
    { 0, 0 },
    { 0x34d253b0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f253b5, 75 },
    { 0xbe3253b6, 61 },
    { 0xda2253b7, 64 },
    { 0x41253b8, 23 },
    { 0x59b253b9, 84 },
    { 0x835253b9, 69 },
    { 0xd9e253ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c253c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307253cd, 52 },
    { 0, 0 },
    { 0xdf7253cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70253d6, 85 },
    { 0, 0 },
    { 0xa74253d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8253db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3253df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa253e5, 100 },
    { 0x1c1253e6, 33 },
    { 0, 0 },
    { 0x54b253e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d253ee, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71253f1, 90 },
    { 0x8a253f1, 61 },
    { 0xc54253f2, 66 },
    { 0x42d253f1, 78 },
    { 0x1f253f5, 99 },
    { 0x8c253f4, 80 },
    { 0xa9e253f7, 90 },
    { 0x22f253f8, 99 },
    { 0, 0 },
    { 0xd10253fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca253ff, 33 },
    { 0x42d253ff, 67 },
    { 0xb5025401, 61 },
    { 0xa0a25402, 100 },
    { 0xa825402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c25409, 99 },
    { 0, 0 },
    { 0x3282540b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe822540e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f125411, 33 },
    { 0xb6b25412, 23 },
    { 0xee225412, 77 },
    { 0x82825414, 77 },
    { 0x21b25414, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f25418, 76 },
    { 0, 0 },
    { 0xc242541a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc25420, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4025426, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e2542d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8425432, 68 },
    { 0x71d25433, 43 },
    { 0, 0 },
    { 0x9225435, 61 },
    { 0, 0 },
    { 0x26f25437, 88 },
    { 0x6bf25438, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62543b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2543f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e25447, 90 },
    { 0, 0 },
    { 0x52625449, 78 },
    { 0, 0 },
    { 0x5152544b, 95 },
    { 0x3f32544c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd325450, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0125454, 95 },
    { 0, 0 },
    { 0x42025456, 43 },
    { 0, 0 },
    { 0xd5125458, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c2545c, 95 },
    { 0, 0 },
    { 0xfea2545e, 85 },
    { 0x7d52545e, 99 },
    { 0xedd2545f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5025465, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f52546d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20425472, 61 },
    { 0x2a625473, 99 },
    { 0, 0 },
    { 0x7d225475, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44825478, 90 },
    { 0, 0 },
    { 0xab72547a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5925480, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a125483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac25486, 52 },
    { 0, 0 },
    { 0x28625488, 76 },
    { 0, 0 },
    { 0xac52548a, 95 },
    { 0xcd82548a, 69 },
    { 0x32e2548c, 85 },
    { 0x5442548d, 95 },
    { 0, 0 },
    { 0xd672548f, 43 },
    { 0xe8125490, 95 },
    { 0xff02548f, 61 },
    { 0, 0 },
    { 0x4a425493, 90 },
    { 0x1ac25493, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a225499, 33 },
    { 0x21a2549a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4254a2, 23 },
    { 0xabf254a3, 46 },
    { 0x58f254a4, 84 },
    { 0x17e254a5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8254a8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57254ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4254b1, 61 },
    { 0, 0 },
    { 0x51d254b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f254bb, 90 },
    { 0xbca254bc, 76 },
    { 0xd6b254bd, 61 },
    { 0xa47254bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91254c2, 33 },
    { 0, 0 },
    { 0x86f254c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416254ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b254d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x463254d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563254d8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306254de, 46 },
    { 0x2d3254df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc254e5, 38 },
    { 0x5b4254e6, 33 },
    { 0xc73254e6, 34 },
    { 0, 0 },
    { 0x4fa254e9, 61 },
    { 0xe87254ea, 45 },
    { 0x29254eb, 33 },
    { 0xa35254ec, 33 },
    { 0x95b254ed, 61 },
    { 0x1bf254ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2254f3, 90 },
    { 0x179254f4, 60 },
    { 0x61c254f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8254fc, 78 },
    { 0, 0 },
    { 0x752254fe, 61 },
    { 0x17a254ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a025502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b125509, 52 },
    { 0, 0 },
    { 0xc1f2550b, 98 },
    { 0x5b22550b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed925514, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b25517, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e2551c, 55 },
    { 0, 0 },
    { 0xa522551e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a725526, 34 },
    { 0, 0 },
    { 0x5af25528, 23 },
    { 0xbc25529, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf042552d, 55 },
    { 0xe122552e, 33 },
    { 0xde02552d, 95 },
    { 0x36825530, 87 },
    { 0x8ea25531, 91 },
    { 0xb2f2552e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a425535, 90 },
    { 0x9a725535, 98 },
    { 0, 0 },
    { 0xd8725538, 90 },
    { 0x4ed25539, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0225541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a25546, 61 },
    { 0xc3125546, 97 },
    { 0x71325548, 33 },
    { 0x1f25549, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f02554e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d25553, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a25556, 97 },
    { 0x7a725556, 61 },
    { 0x1be25558, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5772555b, 98 },
    { 0x2232555c, 76 },
    { 0, 0 },
    { 0xfac2555e, 78 },
    { 0xc732555f, 19 },
    { 0xa2725560, 33 },
    { 0x7eb2555f, 33 },
    { 0xd3725561, 95 },
    { 0xfdd25560, 81 },
    { 0x44f25564, 98 },
    { 0xdf025565, 23 },
    { 0x99125566, 33 },
    { 0, 0 },
    { 0xe8425568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372556e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f25573, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a2557c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2557f, 33 },
    { 0, 0 },
    { 0x8f825581, 61 },
    { 0, 0 },
    { 0x16a25583, 83 },
    { 0xa7325584, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c25587, 91 },
    { 0, 0 },
    { 0x8e925589, 34 },
    { 0x21125589, 95 },
    { 0x8352558b, 61 },
    { 0xcab2558b, 23 },
    { 0xd4a2558d, 34 },
    { 0xa522558e, 85 },
    { 0x5db2558f, 34 },
    { 0xa382558b, 97 },
    { 0x592558e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39025596, 61 },
    { 0, 0 },
    { 0x9d925598, 88 },
    { 0xae525599, 23 },
    { 0x302559a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea255a2, 77 },
    { 0xfb0255a3, 98 },
    { 0, 0 },
    { 0x8b8255a5, 90 },
    { 0xa3255a5, 12 },
    { 0xc3e255a7, 95 },
    { 0xb8f255a8, 46 },
    { 0xec6255a7, 34 },
    { 0x5c9255aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7255ae, 98 },
    { 0xd8b255af, 98 },
    { 0xb29255af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0255b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828255bd, 33 },
    { 0x544255be, 95 },
    { 0xa8e255bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc255c5, 99 },
    { 0, 0 },
    { 0x42f255c7, 55 },
    { 0, 0 },
    { 0x5ae255c9, 90 },
    { 0, 0 },
    { 0xde6255cb, 61 },
    { 0, 0 },
    { 0x32e255cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a255d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95255e2, 81 },
    { 0xd5a255e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2255e6, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9255ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298255f7, 95 },
    { 0x879255f8, 77 },
    { 0x971255f8, 98 },
    { 0x725255f9, 61 },
    { 0x334255fb, 78 },
    { 0x6f5255f7, 55 },
    { 0xf7f255fd, 100 },
    { 0x309255fe, 61 },
    { 0x8d6255ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28025603, 33 },
    { 0x6b225604, 98 },
    { 0, 0 },
    { 0xf425606, 65 },
    { 0xc825607, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212560b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5625612, 61 },
    { 0, 0 },
    { 0xf525614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68325624, 76 },
    { 0x61025625, 61 },
    { 0x82225625, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd112562a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2562d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c25630, 76 },
    { 0x14225630, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9425637, 33 },
    { 0, 0 },
    { 0x17225639, 33 },
    { 0, 0 },
    { 0xccb2563b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8252563f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b25642, 43 },
    { 0xd7125643, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb925648, 93 },
    { 0xd5c25648, 100 },
    { 0x3f625648, 91 },
    { 0x34625648, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3402564e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd725652, 33 },
    { 0, 0 },
    { 0xe4325654, 90 },
    { 0xe5b25655, 66 },
    { 0xf5325656, 33 },
    { 0x28325656, 99 },
    { 0xd725656, 55 },
    { 0x1e725658, 78 },
    { 0x77b25657, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2565d, 61 },
    { 0x392565e, 98 },
    { 0x5882565f, 85 },
    { 0x63d2565d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x71525663, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad92566a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92425671, 90 },
    { 0x6b625672, 33 },
    { 0xfb125673, 91 },
    { 0, 0 },
    { 0xc4125675, 90 },
    { 0x6ea25676, 69 },
    { 0, 0 },
    { 0x7c225678, 17 },
    { 0xe8f25679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3725681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d25686, 95 },
    { 0xcb825686, 61 },
    { 0x6fe25688, 61 },
    { 0x17e25689, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb2568f, 95 },
    { 0, 0 },
    { 0xbb125691, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad25695, 66 },
    { 0xfb025695, 90 },
    { 0x6fc25696, 61 },
    { 0, 0 },
    { 0x21725699, 33 },
    { 0xadc2569a, 90 },
    { 0x6d22569a, 90 },
    { 0, 0 },
    { 0x4f82569d, 33 },
    { 0xf1d2569d, 76 },
    { 0xdb62569d, 34 },
    { 0x6ee256a0, 33 },
    { 0xf90256a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7256a4, 88 },
    { 0, 0 },
    { 0x4de256a6, 95 },
    { 0x792256a7, 81 },
    { 0xc7b256a7, 52 },
    { 0, 0 },
    { 0x3d5256aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e256b0, 95 },
    { 0, 0 },
    { 0x404256b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679256bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd256bf, 81 },
    { 0, 0 },
    { 0x48256c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f256c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b256cb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d256d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0256d7, 76 },
    { 0x5cf256d7, 34 },
    { 0xce5256d7, 95 },
    { 0xf00256da, 76 },
    { 0x3e4256da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d256de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa256e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b256ee, 76 },
    { 0x16a256ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803256f4, 33 },
    { 0, 0 },
    { 0xd11256f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6256fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x233256ff, 61 },
    { 0xfb525700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b525703, 46 },
    { 0x21e25704, 36 },
    { 0xcdf25704, 90 },
    { 0x65e25704, 43 },
    { 0xfbe25705, 99 },
    { 0xb5525708, 33 },
    { 0, 0 },
    { 0xb1b2570a, 34 },
    { 0x7222570b, 90 },
    { 0x6872570c, 98 },
    { 0xd3e2570c, 90 },
    { 0xbd2570c, 95 },
    { 0x3512570d, 61 },
    { 0x6672570c, 34 },
    { 0x8b525711, 81 },
    { 0x2d62570e, 52 },
    { 0xf5b2570e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be25719, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a2571f, 90 },
    { 0, 0 },
    { 0x4b425721, 98 },
    { 0x9ef25722, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f25725, 76 },
    { 0xabc25726, 95 },
    { 0, 0 },
    { 0xd3425728, 61 },
    { 0x23b25728, 95 },
    { 0x9cb25729, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa922572d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e425730, 23 },
    { 0, 0 },
    { 0xe8b25732, 34 },
    { 0x97b25732, 43 },
    { 0x63f25734, 61 },
    { 0xbde25734, 39 },
    { 0xe5525736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a225739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e92573c, 49 },
    { 0, 0 },
    { 0x22c2573e, 98 },
    { 0x5eb2573f, 55 },
    { 0x64b2573f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f325743, 34 },
    { 0x7e025744, 34 },
    { 0x61f25745, 68 },
    { 0xeec25746, 61 },
    { 0xd6b25743, 43 },
    { 0x4d625748, 90 },
    { 0xed725744, 85 },
    { 0x8c225745, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a32574e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b25756, 100 },
    { 0xd6c25757, 55 },
    { 0xfe625758, 82 },
    { 0x58325759, 90 },
    { 0x9fc2575a, 23 },
    { 0x2b525758, 83 },
    { 0, 0 },
    { 0x7ed2575d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f725763, 23 },
    { 0, 0 },
    { 0xc0925765, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78625768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf652576c, 91 },
    { 0xb962576c, 33 },
    { 0xbbb2576c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20925771, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc2577a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc25782, 88 },
    { 0xb7d25783, 67 },
    { 0xe6025784, 98 },
    { 0x77025783, 64 },
    { 0x5df25786, 42 },
    { 0xc7e25787, 33 },
    { 0x50b25787, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e2578e, 66 },
    { 0, 0 },
    { 0x74d25790, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18125797, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b22579d, 33 },
    { 0, 0 },
    { 0xbbe2579f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7257a5, 82 },
    { 0x259257a6, 98 },
    { 0x490257a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9257ab, 91 },
    { 0xc97257ac, 99 },
    { 0xfcf257ac, 46 },
    { 0x8a1257ab, 33 },
    { 0x62a257af, 98 },
    { 0xad6257af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2257b8, 33 },
    { 0x9d9257b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x569257bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986257c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33257c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x780257c7, 56 },
    { 0xfcf257c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a257d3, 76 },
    { 0x53c257d4, 52 },
    { 0x83f257d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x402257d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0257df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987257e5, 100 },
    { 0, 0 },
    { 0xb70257e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x588257ea, 33 },
    { 0x730257eb, 34 },
    { 0xe6a257ec, 33 },
    { 0, 0 },
    { 0xd84257ee, 75 },
    { 0x23b257ef, 85 },
    { 0x1b257f0, 23 },
    { 0, 0 },
    { 0x3e4257f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x433257f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d325800, 90 },
    { 0x41225801, 80 },
    { 0xe7525802, 77 },
    { 0x50f25802, 91 },
    { 0, 0 },
    { 0x2d825805, 23 },
    { 0, 0 },
    { 0xb8e25807, 52 },
    { 0, 0 },
    { 0xb4c25809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92580f, 33 },
    { 0, 0 },
    { 0xd8d25811, 55 },
    { 0, 0 },
    { 0x5cb25813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222581a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f2581d, 55 },
    { 0x3c2581e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef725828, 55 },
    { 0, 0 },
    { 0xb462582a, 90 },
    { 0x782582b, 90 },
    { 0x5b52582b, 93 },
    { 0x9812582b, 95 },
    { 0xafb2582e, 90 },
    { 0, 0 },
    { 0xd3525830, 90 },
    { 0, 0 },
    { 0x1bd25832, 33 },
    { 0x55325833, 84 },
    { 0x39925834, 33 },
    { 0xe6d25835, 76 },
    { 0x1a425836, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a625839, 23 },
    { 0, 0 },
    { 0x8032583b, 34 },
    { 0x3c42583c, 84 },
    { 0x3e22583d, 61 },
    { 0x1692583d, 88 },
    { 0x6622583b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb125843, 33 },
    { 0x4c625844, 23 },
    { 0x2525844, 95 },
    { 0x1be25844, 33 },
    { 0xf3425847, 98 },
    { 0, 0 },
    { 0xe0e25849, 78 },
    { 0x1692584a, 23 },
    { 0xd682584a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73325850, 65 },
    { 0, 0 },
    { 0x9c525852, 96 },
    { 0xb8925853, 93 },
    { 0x6c25854, 90 },
    { 0x6fe25855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a25859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9812585c, 98 },
    { 0x5312585d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db25869, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2586d, 23 },
    { 0x6e92586e, 98 },
    { 0x3772586f, 84 },
    { 0xb9b25870, 95 },
    { 0xb8d2586e, 34 },
    { 0x5982586f, 93 },
    { 0x1c625872, 88 },
    { 0x98425870, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c925877, 61 },
    { 0x40f25878, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d2587b, 98 },
    { 0, 0 },
    { 0xb282587d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4725880, 98 },
    { 0, 0 },
    { 0xa1525882, 78 },
    { 0xef725883, 36 },
    { 0x70825884, 76 },
    { 0xb0625885, 33 },
    { 0xf6c25886, 98 },
    { 0, 0 },
    { 0x73c25888, 90 },
    { 0x8f625889, 95 },
    { 0, 0 },
    { 0xc72588b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c2588f, 33 },
    { 0, 0 },
    { 0xe1625891, 90 },
    { 0x3b25892, 95 },
    { 0x2c925893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72225896, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f32589a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d2589d, 38 },
    { 0x84f2589d, 98 },
    { 0, 0 },
    { 0xc75258a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36258a4, 90 },
    { 0x2f3258a5, 67 },
    { 0x1da258a6, 55 },
    { 0x7d0258a5, 100 },
    { 0xc7a258a8, 46 },
    { 0x979258a5, 76 },
    { 0x7e0258aa, 95 },
    { 0x3c9258ab, 84 },
    { 0, 0 },
    { 0x325258ad, 43 },
    { 0, 0 },
    { 0xc90258af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82258b7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b258bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88258be, 33 },
    { 0, 0 },
    { 0xc96258c0, 95 },
    { 0, 0 },
    { 0xa1f258c2, 23 },
    { 0xdf3258c2, 34 },
    { 0x768258c4, 23 },
    { 0x365258c5, 33 },
    { 0xda8258c5, 33 },
    { 0, 0 },
    { 0x468258c8, 23 },
    { 0x259258c9, 90 },
    { 0, 0 },
    { 0x97258cb, 33 },
    { 0xb03258cb, 23 },
    { 0x623258cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e258d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f258d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db258da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757258e4, 88 },
    { 0x44e258e5, 23 },
    { 0x78b258e6, 90 },
    { 0, 0 },
    { 0xe9258e8, 77 },
    { 0xc74258e8, 98 },
    { 0, 0 },
    { 0x535258eb, 33 },
    { 0xa9b258ec, 46 },
    { 0, 0 },
    { 0xd9b258ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646258fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2425902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac925905, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b2590a, 97 },
    { 0xe092590a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41125911, 76 },
    { 0x23125911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf25915, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5522591e, 66 },
    { 0x5532591f, 61 },
    { 0, 0 },
    { 0xdeb25921, 43 },
    { 0x88325922, 78 },
    { 0x81a25923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b25928, 61 },
    { 0x7ce25928, 98 },
    { 0x76c2592a, 90 },
    { 0, 0 },
    { 0xb6f2592c, 46 },
    { 0xd0e2592c, 33 },
    { 0x4a82592e, 90 },
    { 0, 0 },
    { 0x7c25930, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e25941, 46 },
    { 0x90b25941, 95 },
    { 0x1bf25943, 61 },
    { 0xbf225944, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c225948, 33 },
    { 0, 0 },
    { 0x3872594a, 90 },
    { 0x6d52594b, 33 },
    { 0xd502594c, 90 },
    { 0x25d2594c, 65 },
    { 0x8a62594b, 43 },
    { 0xdee2594e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc25954, 89 },
    { 0x8aa25955, 90 },
    { 0x9e525955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67225959, 34 },
    { 0x4e02595a, 90 },
    { 0xaf2595a, 33 },
    { 0, 0 },
    { 0x1d12595d, 61 },
    { 0x73c2595e, 61 },
    { 0, 0 },
    { 0x1825960, 65 },
    { 0x40625961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b25967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d22596b, 33 },
    { 0x4e22596c, 95 },
    { 0xe422596d, 17 },
    { 0x30b2596e, 34 },
    { 0x2b32596c, 43 },
    { 0x5582596b, 87 },
    { 0x25525971, 98 },
    { 0xc1a25972, 93 },
    { 0x3ff25973, 95 },
    { 0x4402596f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31625977, 100 },
    { 0xddf25978, 61 },
    { 0, 0 },
    { 0x56c2597a, 90 },
    { 0, 0 },
    { 0xfb32597c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f25980, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1325983, 34 },
    { 0, 0 },
    { 0x3f325985, 33 },
    { 0, 0 },
    { 0xd4025987, 55 },
    { 0xe4c25988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf32598b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa452598f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd25998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e2599b, 90 },
    { 0, 0 },
    { 0x2ac2599d, 23 },
    { 0xc552599e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f259a4, 61 },
    { 0xd7e259a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00259a8, 55 },
    { 0x69f259a9, 55 },
    { 0, 0 },
    { 0xcbe259ab, 61 },
    { 0xe6b259ab, 90 },
    { 0x59d259ad, 52 },
    { 0x4c8259ae, 90 },
    { 0x8e9259ae, 66 },
    { 0xfc6259af, 98 },
    { 0, 0 },
    { 0x806259b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3259b5, 33 },
    { 0x270259b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f259b9, 34 },
    { 0x70a259b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63259c4, 88 },
    { 0, 0 },
    { 0x45f259c6, 90 },
    { 0, 0 },
    { 0x346259c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a259cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee259cf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d259d2, 99 },
    { 0xd19259d3, 84 },
    { 0, 0 },
    { 0x7e5259d5, 33 },
    { 0, 0 },
    { 0xaca259d7, 33 },
    { 0xde259d8, 69 },
    { 0x62c259d9, 69 },
    { 0, 0 },
    { 0xba0259db, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1259e0, 33 },
    { 0xb17259e1, 76 },
    { 0xb62259e0, 85 },
    { 0x80e259e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed259e8, 55 },
    { 0x8cc259e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac259ef, 93 },
    { 0, 0 },
    { 0x331259f1, 61 },
    { 0, 0 },
    { 0x82c259f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab259f6, 95 },
    { 0x439259f6, 66 },
    { 0x320259f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8725a03, 46 },
    { 0x96425a04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e25a0b, 94 },
    { 0x56725a0c, 93 },
    { 0xa1625a0d, 76 },
    { 0x1c825a0e, 33 },
    { 0, 0 },
    { 0xa5225a10, 67 },
    { 0, 0 },
    { 0x79325a12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e25a19, 85 },
    { 0xc2c25a1a, 98 },
    { 0x9be25a1a, 90 },
    { 0x38425a1a, 95 },
    { 0xfd625a1d, 85 },
    { 0, 0 },
    { 0xe1625a1f, 95 },
    { 0x66b25a20, 43 },
    { 0x7cd25a21, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0225a25, 33 },
    { 0, 0 },
    { 0x71f25a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0825a2a, 61 },
    { 0x35425a2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1825a32, 85 },
    { 0, 0 },
    { 0x23125a34, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11525a3c, 61 },
    { 0x9cf25a3c, 76 },
    { 0x72a25a3e, 90 },
    { 0x84625a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e25a42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b25a47, 43 },
    { 0, 0 },
    { 0x52725a49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff25a4e, 90 },
    { 0x24325a4e, 33 },
    { 0xbc625a50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6625a56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad25a5b, 90 },
    { 0, 0 },
    { 0xc7b25a5d, 66 },
    { 0x38825a5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9625a62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57625a65, 95 },
    { 0x2fd25a65, 55 },
    { 0, 0 },
    { 0xf0825a68, 68 },
    { 0, 0 },
    { 0xee225a6a, 40 },
    { 0x99925a6b, 84 },
    { 0, 0 },
    { 0x76525a6d, 61 },
    { 0x96a25a6e, 61 },
    { 0, 0 },
    { 0x1125a70, 99 },
    { 0, 0 },
    { 0xd2d25a72, 34 },
    { 0xbe325a72, 33 },
    { 0xe3225a74, 46 },
    { 0xd5125a75, 36 },
    { 0x7d725a75, 61 },
    { 0x63125a77, 65 },
    { 0xc8425a77, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xee625a7b, 23 },
    { 0xe1825a7c, 46 },
    { 0, 0 },
    { 0x99625a7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea725a81, 95 },
    { 0xc4725a82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa25a85, 17 },
    { 0, 0 },
    { 0x4625a87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925a8a, 33 },
    { 0x8fe25a8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17025a92, 33 },
    { 0, 0 },
    { 0x3b525a94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d25a99, 76 },
    { 0, 0 },
    { 0xd4e25a9b, 43 },
    { 0, 0 },
    { 0x27b25a9d, 23 },
    { 0x7a25a9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba25aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d25aa4, 85 },
    { 0x76225aa5, 33 },
    { 0x31525aa5, 90 },
    { 0xdc725aa7, 98 },
    { 0x96125aa6, 90 },
    { 0x67c25aa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e725aad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df25ab0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x53825ab3, 90 },
    { 0x2be25ab4, 91 },
    { 0xc1625ab5, 66 },
    { 0xefe25ab6, 93 },
    { 0x1c425ab7, 90 },
    { 0xc3625ab8, 61 },
    { 0xb9a25ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f325abe, 61 },
    { 0, 0 },
    { 0xf5e25ac0, 98 },
    { 0, 0 },
    { 0x57325ac2, 95 },
    { 0, 0 },
    { 0xa9025ac4, 68 },
    { 0xe1225ac5, 65 },
    { 0x99725ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba25aca, 33 },
    { 0xb1d25aca, 90 },
    { 0, 0 },
    { 0x2b425acd, 55 },
    { 0, 0 },
    { 0xf0d25acf, 99 },
    { 0xb3f25acf, 38 },
    { 0, 0 },
    { 0x51f25ad2, 52 },
    { 0, 0 },
    { 0xb8225ad4, 87 },
    { 0xb8825ad5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91125ad9, 65 },
    { 0x5c125ada, 23 },
    { 0xc3e25adb, 92 },
    { 0x6a225adc, 95 },
    { 0, 0 },
    { 0x9e125ade, 53 },
    { 0x10a25adf, 68 },
    { 0x38325ae0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31325ae4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37725ae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf625aee, 91 },
    { 0x9cb25aef, 84 },
    { 0x91f25aef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b325af3, 52 },
    { 0x6a125af4, 90 },
    { 0x3ab25af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe625af8, 85 },
    { 0, 0 },
    { 0xbc325afa, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf25afe, 76 },
    { 0xfeb25aff, 34 },
    { 0x3025b00, 68 },
    { 0, 0 },
    { 0x56e25b02, 95 },
    { 0x40125b02, 98 },
    { 0, 0 },
    { 0x33025b05, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf25b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac825b0c, 76 },
    { 0, 0 },
    { 0xbde25b0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc25b12, 61 },
    { 0xbb025b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e25b17, 23 },
    { 0x27625b18, 23 },
    { 0x2c525b18, 99 },
    { 0xeca25b17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55125b1f, 97 },
    { 0xfea25b1f, 77 },
    { 0xaff25b1f, 33 },
    { 0xc2725b22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59425b2b, 55 },
    { 0xf5c25b2b, 90 },
    { 0xd3e25b2d, 95 },
    { 0x76d25b2d, 67 },
    { 0, 0 },
    { 0x10c25b30, 90 },
    { 0, 0 },
    { 0xcde25b32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70025b39, 34 },
    { 0xa3c25b39, 84 },
    { 0xfda25b39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5325b40, 61 },
    { 0xe3225b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb25b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c25b4d, 33 },
    { 0xa3825b4e, 55 },
    { 0xeb925b4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85125b52, 93 },
    { 0xd5825b53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda525b59, 98 },
    { 0x61e25b5a, 55 },
    { 0, 0 },
    { 0xcc925b5c, 23 },
    { 0xcb425b5c, 98 },
    { 0x9d525b5e, 65 },
    { 0, 0 },
    { 0xefb25b60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8825b67, 23 },
    { 0xb1725b67, 38 },
    { 0x45c25b68, 33 },
    { 0, 0 },
    { 0x2bb25b6b, 33 },
    { 0x72b25b6b, 91 },
    { 0x2ed25b6d, 61 },
    { 0, 0 },
    { 0xfe25b6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d25b77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec625b7e, 96 },
    { 0xb6225b7f, 90 },
    { 0x51125b80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49925b85, 33 },
    { 0, 0 },
    { 0x33f25b87, 78 },
    { 0, 0 },
    { 0x49825b89, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f525b8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c25b91, 98 },
    { 0x4bd25b91, 23 },
    { 0, 0 },
    { 0x17d25b94, 61 },
    { 0xb25b95, 77 },
    { 0x64225b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58225b9b, 90 },
    { 0xdfb25b9c, 77 },
    { 0x56625b9c, 78 },
    { 0x1bd25b9e, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76325ba2, 100 },
    { 0xf0b25ba3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f925ba8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf25bad, 76 },
    { 0x53325bad, 83 },
    { 0xb0525bae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a425bb3, 61 },
    { 0, 0 },
    { 0x5fa25bb5, 90 },
    { 0, 0 },
    { 0xe6025bb7, 98 },
    { 0, 0 },
    { 0x6ab25bb9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7625bbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3025bc1, 81 },
    { 0x925bc1, 23 },
    { 0xbfa25bc1, 23 },
    { 0xde525bc1, 81 },
    { 0x87f25bc3, 61 },
    { 0, 0 },
    { 0xd225bc7, 95 },
    { 0x9ec25bc8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f325bce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x68925bd1, 55 },
    { 0xe5025bd1, 76 },
    { 0x4a025bd3, 34 },
    { 0xcf925bd2, 40 },
    { 0xd5525bd2, 23 },
    { 0xd9925bd4, 65 },
    { 0x8f325bd7, 77 },
    { 0xe3325bd2, 23 },
    { 0xa5525bd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d425beb, 23 },
    { 0xb5325bec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0825bef, 55 },
    { 0x52225bf0, 53 },
    { 0xc225bf1, 33 },
    { 0x6ca25bf2, 52 },
    { 0x5c725bf1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f25bfb, 43 },
    { 0, 0 },
    { 0x97825bfd, 46 },
    { 0, 0 },
    { 0xff725bff, 90 },
    { 0, 0 },
    { 0xa6c25c01, 61 },
    { 0xb0625c02, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a25c05, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66225c0a, 98 },
    { 0, 0 },
    { 0x18e25c0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e25c11, 95 },
    { 0xc5025c11, 67 },
    { 0x87c25c13, 55 },
    { 0xa7d25c12, 90 },
    { 0x97425c15, 66 },
    { 0x8a625c13, 100 },
    { 0xc7025c13, 76 },
    { 0, 0 },
    { 0xa2a25c19, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb625c1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8025c1f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27525c23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff825c26, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da25c2b, 23 },
    { 0, 0 },
    { 0xb0f25c2d, 55 },
    { 0, 0 },
    { 0x22c25c2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87825c38, 66 },
    { 0, 0 },
    { 0xd1325c3a, 98 },
    { 0, 0 },
    { 0xa1225c3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa25c40, 90 },
    { 0xd3025c41, 75 },
    { 0x87525c42, 61 },
    { 0x5325c43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e25c46, 98 },
    { 0, 0 },
    { 0x76a25c48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31825c4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30125c50, 55 },
    { 0x7d425c51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b525c56, 76 },
    { 0x8e325c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a225c5a, 83 },
    { 0, 0 },
    { 0x9d025c5c, 77 },
    { 0, 0 },
    { 0x80225c5e, 91 },
    { 0xd6a25c5f, 61 },
    { 0, 0 },
    { 0x1c025c61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab925c67, 93 },
    { 0x70425c68, 90 },
    { 0x90025c69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb225c71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f25c74, 23 },
    { 0xaf425c75, 33 },
    { 0x62a25c76, 90 },
    { 0xc6a25c76, 95 },
    { 0, 0 },
    { 0xbe25c79, 24 },
    { 0xfa325c7a, 100 },
    { 0, 0 },
    { 0x1d725c7c, 98 },
    { 0x20225c7d, 84 },
    { 0xeef25c7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d25c84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb425c8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76725c91, 99 },
    { 0xd0b25c92, 76 },
    { 0, 0 },
    { 0xcab25c94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59325c9a, 61 },
    { 0x4e725c9a, 61 },
    { 0xb25c9c, 98 },
    { 0x1eb25c9c, 98 },
    { 0, 0 },
    { 0x10e25c9f, 33 },
    { 0xcad25ca0, 52 },
    { 0xe0a25ca0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c25ca6, 85 },
    { 0, 0 },
    { 0xb0225ca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad25cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe25cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c25cc1, 46 },
    { 0, 0 },
    { 0x41f25cc3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4725cc7, 91 },
    { 0x4125cc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf225ccc, 95 },
    { 0x32025ccc, 23 },
    { 0xbb625ccd, 23 },
    { 0, 0 },
    { 0xaee25cd0, 55 },
    { 0x97925cd1, 64 },
    { 0, 0 },
    { 0xeae25cd3, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9525cd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde25cda, 33 },
    { 0xea725cda, 95 },
    { 0, 0 },
    { 0x36625cdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16925ce0, 95 },
    { 0x8dd25ce1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe625ce4, 33 },
    { 0, 0 },
    { 0x1e425ce6, 93 },
    { 0, 0 },
    { 0xbfe25ce8, 55 },
    { 0, 0 },
    { 0x25125cea, 95 },
    { 0x6a725ceb, 33 },
    { 0x36025ceb, 33 },
    { 0xba225cea, 90 },
    { 0x37b25cee, 90 },
    { 0x74325cee, 23 },
    { 0x5a425cf0, 93 },
    { 0xe225cf1, 90 },
    { 0x21925ceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5925cf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f25cfa, 81 },
    { 0xb25cfb, 46 },
    { 0xcd225cfb, 95 },
    { 0, 0 },
    { 0xd8925cfe, 93 },
    { 0, 0 },
    { 0x1825d00, 66 },
    { 0xa6025d01, 99 },
    { 0x3d325d02, 55 },
    { 0x9725d01, 33 },
    { 0x2bb25d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb25d08, 69 },
    { 0xc3325d09, 76 },
    { 0xf5425d08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4725d0f, 61 },
    { 0xbfb25d10, 46 },
    { 0x1d425d10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91125d14, 23 },
    { 0x80e25d14, 77 },
    { 0x4fd25d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94325d1a, 84 },
    { 0x3fb25d1a, 61 },
    { 0, 0 },
    { 0xd6f25d1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d25d24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3125d27, 23 },
    { 0xb2b25d28, 66 },
    { 0, 0 },
    { 0x9b525d2a, 55 },
    { 0x9d025d2a, 40 },
    { 0xdda25d2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6025d30, 70 },
    { 0xc9b25d31, 95 },
    { 0, 0 },
    { 0xb5025d33, 90 },
    { 0x48025d33, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb25d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2925d3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa025d3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e25d47, 98 },
    { 0xf0f25d48, 33 },
    { 0x5c25d49, 40 },
    { 0, 0 },
    { 0x79925d4b, 98 },
    { 0, 0 },
    { 0x8425d4d, 55 },
    { 0x7d625d4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69725d51, 34 },
    { 0x88925d52, 75 },
    { 0x5225d53, 68 },
    { 0x71225d54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f025d57, 76 },
    { 0x8f725d57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d25d5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a025d62, 95 },
    { 0x1db25d63, 100 },
    { 0x5af25d62, 33 },
    { 0x44725d62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30325d69, 98 },
    { 0xb725d6a, 33 },
    { 0x5825d6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef25d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa225d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b525d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24325d7d, 90 },
    { 0xe3325d7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c25d81, 67 },
    { 0x8b425d81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11025d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67525d88, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a25d8d, 33 },
    { 0, 0 },
    { 0xa3e25d8f, 33 },
    { 0xb0a25d90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd25d98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8725d9c, 66 },
    { 0xc0425d9d, 98 },
    { 0x97225d9e, 61 },
    { 0x8db25d9d, 34 },
    { 0xa7625da0, 61 },
    { 0xeba25da0, 95 },
    { 0x44d25da2, 66 },
    { 0, 0 },
    { 0x27525da4, 95 },
    { 0x8e125da5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2125daf, 87 },
    { 0, 0 },
    { 0x80625db1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x15425db4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b25db7, 95 },
    { 0x55825db8, 61 },
    { 0x9d925db7, 85 },
    { 0, 0 },
    { 0x40f25dbb, 43 },
    { 0x34f25dbc, 84 },
    { 0xf2025dbb, 43 },
    { 0x6ed25dbe, 90 },
    { 0x8a625dbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36825dc2, 93 },
    { 0x87625dc3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc25dd5, 34 },
    { 0x5c825dd6, 98 },
    { 0x1c925dd6, 61 },
    { 0x51825dd8, 23 },
    { 0xad825dd6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd25ddc, 95 },
    { 0xb6325ddd, 98 },
    { 0xc7425dde, 90 },
    { 0x70b25dde, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f825de3, 76 },
    { 0x75425de4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e25de9, 66 },
    { 0, 0 },
    { 0x80425deb, 33 },
    { 0xde625dec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a25def, 75 },
    { 0xe3325def, 91 },
    { 0x50025def, 90 },
    { 0xfb425df2, 84 },
    { 0xdbb25df3, 88 },
    { 0, 0 },
    { 0x7df25df5, 88 },
    { 0xf1025df6, 61 },
    { 0xe7025df5, 43 },
    { 0, 0 },
    { 0xf5225df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc25dfd, 33 },
    { 0x31d25dfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf425e06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80025e0c, 67 },
    { 0x2a025e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df25e11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2125e14, 98 },
    { 0x5e225e15, 76 },
    { 0, 0 },
    { 0x60425e17, 33 },
    { 0x16a25e18, 98 },
    { 0x49425e19, 88 },
    { 0x99825e18, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59625e1d, 90 },
    { 0x5cb25e1e, 61 },
    { 0x15c25e1e, 98 },
    { 0xdeb25e1d, 33 },
    { 0xad625e21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x47425e24, 87 },
    { 0, 0 },
    { 0xac725e26, 98 },
    { 0x8f625e27, 33 },
    { 0, 0 },
    { 0xaac25e29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5425e2d, 98 },
    { 0xee525e2d, 61 },
    { 0x70925e2f, 76 },
    { 0x35625e2f, 90 },
    { 0x89b25e31, 23 },
    { 0, 0 },
    { 0x78725e33, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e25e37, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2625e3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30725e42, 98 },
    { 0xcd225e43, 98 },
    { 0x4fe25e44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a25e4c, 95 },
    { 0x58a25e4d, 85 },
    { 0, 0 },
    { 0x25225e4f, 61 },
    { 0x42525e50, 69 },
    { 0, 0 },
    { 0x32125e52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c225e56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e425e5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95525e5e, 33 },
    { 0xe2525e5e, 76 },
    { 0x2c625e5f, 90 },
    { 0x90125e60, 33 },
    { 0xa5125e62, 55 },
    { 0x58225e63, 98 },
    { 0x93c25e64, 77 },
    { 0xc9225e65, 33 },
    { 0xde825e66, 66 },
    { 0xa625e5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a25e6a, 75 },
    { 0x93625e6a, 43 },
    { 0x39b25e6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68625e70, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x85325e73, 76 },
    { 0, 0 },
    { 0xac725e75, 85 },
    { 0x6ff25e75, 41 },
    { 0x94325e77, 40 },
    { 0, 0 },
    { 0xf4b25e79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa025e7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d25e80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b25e83, 90 },
    { 0x8a125e84, 90 },
    { 0x79c25e85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7525e89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e025e8c, 90 },
    { 0, 0 },
    { 0xab25e8e, 61 },
    { 0x84725e8f, 61 },
    { 0xc9325e90, 93 },
    { 0x5f025e8f, 80 },
    { 0x90e25e92, 78 },
    { 0x1025e93, 93 },
    { 0, 0 },
    { 0x85b25e95, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b725e99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a325e9d, 61 },
    { 0, 0 },
    { 0x4d025e9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42825ea3, 23 },
    { 0x77f25ea4, 95 },
    { 0x46325ea4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de25ea9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f25ead, 95 },
    { 0x12325ead, 33 },
    { 0xc6725eaf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc25eb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef925eba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb25ebe, 85 },
    { 0xb7b25ebe, 61 },
    { 0, 0 },
    { 0xbb525ec1, 55 },
    { 0x2c425ec2, 23 },
    { 0x9f525ec3, 90 },
    { 0, 0 },
    { 0x9b025ec5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a25ec8, 90 },
    { 0x38625ec9, 90 },
    { 0xfab25eca, 66 },
    { 0x12e25ec8, 95 },
    { 0x1da25ecc, 98 },
    { 0xc9a25ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f725ed0, 90 },
    { 0xec825ed0, 78 },
    { 0xb2125ed2, 66 },
    { 0x27625ed3, 61 },
    { 0, 0 },
    { 0x86825ed5, 23 },
    { 0x50625ed6, 33 },
    { 0x70125ed7, 90 },
    { 0x5c225ed8, 76 },
    { 0x29225ed9, 66 },
    { 0x57d25ed7, 61 },
    { 0x88a25edb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb25ee0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb25ee6, 77 },
    { 0x4cb25ee7, 34 },
    { 0xa4d25ee8, 93 },
    { 0xe5525ee8, 61 },
    { 0xeae25ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b25eef, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa425ef2, 84 },
    { 0, 0 },
    { 0x32225ef4, 98 },
    { 0x7e125ef5, 61 },
    { 0x77925ef6, 61 },
    { 0x77625ef5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea825efb, 61 },
    { 0, 0 },
    { 0x3a25efd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325f02, 85 },
    { 0, 0 },
    { 0x2d325f04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f625f09, 90 },
    { 0x92425f0a, 93 },
    { 0xf8025f0b, 98 },
    { 0, 0 },
    { 0x8fa25f0d, 90 },
    { 0, 0 },
    { 0x4cd25f0f, 90 },
    { 0x6c225f0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae25f13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c25f19, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6225f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f725f23, 43 },
    { 0, 0 },
    { 0x2bb25f25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f25f29, 23 },
    { 0xc6425f2a, 90 },
    { 0x16d25f2b, 34 },
    { 0xf2625f2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b25f32, 93 },
    { 0, 0 },
    { 0xaf225f34, 98 },
    { 0xde525f35, 33 },
    { 0, 0 },
    { 0xfdc25f37, 95 },
    { 0x94925f38, 83 },
    { 0x11e25f39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7425f3d, 98 },
    { 0xf5225f3e, 75 },
    { 0, 0 },
    { 0xc9a25f40, 93 },
    { 0x64925f41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85225f4b, 55 },
    { 0, 0 },
    { 0x3d625f4d, 88 },
    { 0x69d25f4d, 96 },
    { 0, 0 },
    { 0x82825f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a625f53, 90 },
    { 0x6325f54, 65 },
    { 0, 0 },
    { 0x78125f56, 55 },
    { 0x31025f56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf25f5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c25f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc25f61, 23 },
    { 0xb5f25f62, 95 },
    { 0x3be25f63, 90 },
    { 0xbbd25f64, 100 },
    { 0xb3225f61, 95 },
    { 0x4f125f63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37625f69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d325f6e, 90 },
    { 0xf1025f6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99025f75, 35 },
    { 0xb2525f75, 61 },
    { 0x5f425f75, 90 },
    { 0, 0 },
    { 0x7425f79, 33 },
    { 0xa1b25f7a, 90 },
    { 0, 0 },
    { 0x9c325f7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60125f82, 69 },
    { 0, 0 },
    { 0xf7525f84, 33 },
    { 0, 0 },
    { 0xe8f25f86, 98 },
    { 0, 0 },
    { 0x41525f88, 76 },
    { 0x54625f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5125f92, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde25f96, 98 },
    { 0x93325f96, 34 },
    { 0, 0 },
    { 0x60625f99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea25f9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0625fa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13725fa9, 77 },
    { 0x13c25faa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc25fad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf625fb5, 36 },
    { 0, 0 },
    { 0x27425fb7, 98 },
    { 0x97f25fb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf025fbd, 96 },
    { 0x28c25fbd, 61 },
    { 0xe6d25fbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f925fc4, 90 },
    { 0x30325fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5025fcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d25fd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71125fd5, 98 },
    { 0x7fe25fd6, 65 },
    { 0x33125fd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45925fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10625fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57525fe8, 90 },
    { 0, 0 },
    { 0x73725fea, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf25fee, 98 },
    { 0x36325fee, 61 },
    { 0x97725ff0, 95 },
    { 0, 0 },
    { 0xeef25ff2, 78 },
    { 0x1dd25ff3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b425ffa, 95 },
    { 0x12925ffb, 55 },
    { 0x93c25ffc, 34 },
    { 0x30c25ffd, 98 },
    { 0xe4225ffe, 78 },
    { 0, 0 },
    { 0x3ea26000, 34 },
    { 0, 0 },
    { 0xf0f26002, 90 },
    { 0x55826003, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f526008, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa222600c, 61 },
    { 0x3a42600c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10926014, 33 },
    { 0, 0 },
    { 0xcf026016, 95 },
    { 0, 0 },
    { 0x57726018, 61 },
    { 0, 0 },
    { 0x70d2601a, 40 },
    { 0xc012601b, 52 },
    { 0xe832601c, 61 },
    { 0xad72601c, 90 },
    { 0x4562601e, 68 },
    { 0x1452601f, 45 },
    { 0x5512601d, 98 },
    { 0, 0 },
    { 0x91e26022, 98 },
    { 0, 0 },
    { 0x49626024, 78 },
    { 0x20926025, 76 },
    { 0, 0 },
    { 0x89c26027, 75 },
    { 0xdbc26028, 67 },
    { 0x5b126027, 34 },
    { 0, 0 },
    { 0x5892602b, 52 },
    { 0xf9e2602b, 90 },
    { 0x75e2602c, 23 },
    { 0x12f2602e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac26031, 90 },
    { 0xb3026032, 90 },
    { 0x20926032, 76 },
    { 0xc0926032, 90 },
    { 0x5be26035, 98 },
    { 0xd2e26032, 85 },
    { 0, 0 },
    { 0x7926038, 33 },
    { 0x2c726039, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62603e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe426041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d26044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d2604d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a526056, 23 },
    { 0x3ff26056, 90 },
    { 0x71926058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d2605d, 53 },
    { 0, 0 },
    { 0x6ed2605f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2126063, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd26068, 61 },
    { 0, 0 },
    { 0xec02606a, 100 },
    { 0x802606a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e72606f, 43 },
    { 0, 0 },
    { 0xad326071, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf926074, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c26078, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a2607d, 87 },
    { 0x862607d, 33 },
    { 0x8072607f, 33 },
    { 0xca626080, 95 },
    { 0, 0 },
    { 0x1ef26082, 33 },
    { 0xf3b26083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50626086, 90 },
    { 0x9bd26087, 76 },
    { 0, 0 },
    { 0xa8626089, 78 },
    { 0x6d726089, 98 },
    { 0, 0 },
    { 0x3bf2608c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc26096, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda826099, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc952609d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd260a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6260ad, 90 },
    { 0x48c260ad, 61 },
    { 0x429260ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7260b3, 61 },
    { 0x279260b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba260bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x287260c0, 90 },
    { 0x67c260c0, 78 },
    { 0, 0 },
    { 0xcd1260c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974260c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x857260ca, 76 },
    { 0x296260ca, 95 },
    { 0xb2c260cc, 98 },
    { 0x136260cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab260d3, 23 },
    { 0x9be260d4, 67 },
    { 0, 0 },
    { 0x797260d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc260da, 98 },
    { 0x6cf260db, 55 },
    { 0, 0 },
    { 0xd1e260dd, 61 },
    { 0x912260de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66260e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c260ea, 95 },
    { 0xfc7260eb, 51 },
    { 0x76260ec, 75 },
    { 0, 0 },
    { 0xf99260ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e260f1, 34 },
    { 0, 0 },
    { 0x417260f3, 82 },
    { 0xdf5260f4, 76 },
    { 0x3e0260f4, 90 },
    { 0x77b260f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba260fa, 33 },
    { 0, 0 },
    { 0xcb6260fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca526100, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41926109, 90 },
    { 0x78a2610a, 95 },
    { 0x3b02610b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9022610e, 64 },
    { 0x6ba2610f, 78 },
    { 0x9bb26110, 76 },
    { 0, 0 },
    { 0xcc26112, 23 },
    { 0, 0 },
    { 0x25126114, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b26117, 90 },
    { 0x1026118, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f2611b, 76 },
    { 0x46d2611b, 61 },
    { 0x6882611d, 84 },
    { 0x1b52611b, 90 },
    { 0x8d82611f, 93 },
    { 0xc8f26120, 78 },
    { 0x41926121, 46 },
    { 0x5b326122, 98 },
    { 0xc762611c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6026127, 33 },
    { 0, 0 },
    { 0xf4826129, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b2612d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b26131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb026139, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9226146, 99 },
    { 0x8dc26147, 77 },
    { 0x44826146, 61 },
    { 0xe1e26149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb232614c, 98 },
    { 0, 0 },
    { 0xd2614e, 23 },
    { 0, 0 },
    { 0xc926150, 46 },
    { 0, 0 },
    { 0xf1d26152, 76 },
    { 0x73126153, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9226158, 33 },
    { 0, 0 },
    { 0x9982615a, 23 },
    { 0x5a32615b, 34 },
    { 0xf152615c, 61 },
    { 0xea92615d, 33 },
    { 0x4dc2615e, 17 },
    { 0x7dd2615e, 75 },
    { 0xfe26160, 93 },
    { 0x60e26161, 91 },
    { 0xb326161, 55 },
    { 0x62226162, 23 },
    { 0x13d26164, 76 },
    { 0xf3726164, 66 },
    { 0xf932615b, 61 },
    { 0x89d26162, 55 },
    { 0xc2126168, 75 },
    { 0x44b26169, 90 },
    { 0xd9f26169, 33 },
    { 0xef02616b, 40 },
    { 0x2802616b, 87 },
    { 0x40726168, 23 },
    { 0x26167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb26174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72726179, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb112617e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f826181, 48 },
    { 0x9a326181, 84 },
    { 0xe5626183, 69 },
    { 0xee926184, 68 },
    { 0xb3c26184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe226188, 33 },
    { 0xce826189, 33 },
    { 0x77f2618a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb002618f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b826199, 76 },
    { 0, 0 },
    { 0xbdc2619b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7812619f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf261a2, 87 },
    { 0x37d261a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb261a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48261a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f261ae, 23 },
    { 0xbf0261ae, 56 },
    { 0x18e261ae, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x577261b3, 33 },
    { 0, 0 },
    { 0xe66261b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd261be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8261c9, 76 },
    { 0, 0 },
    { 0x834261cb, 95 },
    { 0xa96261cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e261d1, 90 },
    { 0x2a5261d2, 90 },
    { 0, 0 },
    { 0x5fc261d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab261d7, 55 },
    { 0, 0 },
    { 0xa4a261d9, 36 },
    { 0x84261d9, 23 },
    { 0x89c261d9, 33 },
    { 0, 0 },
    { 0xe7f261dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0261e1, 95 },
    { 0, 0 },
    { 0xb5c261e3, 76 },
    { 0x687261e4, 90 },
    { 0, 0 },
    { 0x1d0261e6, 91 },
    { 0x687261e6, 77 },
    { 0x8b261e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b261ed, 23 },
    { 0x694261ed, 43 },
    { 0, 0 },
    { 0x964261f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb261f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f261f7, 90 },
    { 0xb19261f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a261fc, 87 },
    { 0x332261fd, 43 },
    { 0xb20261fd, 95 },
    { 0, 0 },
    { 0xb4026200, 98 },
    { 0xc0f26201, 33 },
    { 0x3eb26202, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8326216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef26219, 98 },
    { 0xf6b2621a, 98 },
    { 0x68f2621b, 68 },
    { 0xbe62621b, 90 },
    { 0x102621d, 90 },
    { 0x61b2621e, 91 },
    { 0xeb42621e, 95 },
    { 0xf9f2621f, 90 },
    { 0xb8a2621e, 69 },
    { 0x94f26222, 76 },
    { 0x2ce26221, 23 },
    { 0, 0 },
    { 0xd3b26225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27326228, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc2622c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26231, 33 },
    { 0x90426232, 34 },
    { 0xc0626233, 84 },
    { 0x5e526234, 33 },
    { 0xe4826235, 95 },
    { 0x6e226236, 23 },
    { 0, 0 },
    { 0x92a26238, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda52623b, 59 },
    { 0, 0 },
    { 0x7e52623d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3626243, 69 },
    { 0x48226243, 55 },
    { 0x61d26244, 98 },
    { 0x99126244, 98 },
    { 0, 0 },
    { 0x1e226248, 23 },
    { 0x39f26248, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952624c, 61 },
    { 0xc1b2624d, 61 },
    { 0x8702624e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64826253, 90 },
    { 0xb8426254, 23 },
    { 0xa5e26255, 98 },
    { 0x8f826255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c2625d, 98 },
    { 0, 0 },
    { 0x5ba2625f, 52 },
    { 0x94226260, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33726264, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9226267, 61 },
    { 0x1b526268, 93 },
    { 0xef726269, 33 },
    { 0x2112626a, 34 },
    { 0x6aa2626a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2626f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x89526272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50326277, 80 },
    { 0x3926278, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42627d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1126284, 33 },
    { 0x73926285, 90 },
    { 0x51526286, 98 },
    { 0x9ab26284, 90 },
    { 0x1e526288, 84 },
    { 0xfad26284, 23 },
    { 0, 0 },
    { 0x5282628b, 95 },
    { 0xa962628c, 34 },
    { 0xdd72628d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79026293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af26299, 36 },
    { 0x5212629a, 76 },
    { 0xa1b2629b, 90 },
    { 0x6e32629a, 87 },
    { 0, 0 },
    { 0xfbd2629e, 33 },
    { 0x5b12629f, 93 },
    { 0, 0 },
    { 0x612262a1, 51 },
    { 0, 0 },
    { 0x8e8262a3, 34 },
    { 0x7b3262a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb262aa, 55 },
    { 0x1f6262ab, 61 },
    { 0x2db262ac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8262af, 87 },
    { 0x854262af, 87 },
    { 0x81e262b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e262b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67262b8, 33 },
    { 0x11d262b8, 23 },
    { 0x70c262b8, 23 },
    { 0x8c262bb, 40 },
    { 0x435262bc, 76 },
    { 0, 0 },
    { 0x649262be, 61 },
    { 0x419262be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1262c2, 75 },
    { 0xed2262c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3262c6, 23 },
    { 0xb29262c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28262cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10262cf, 48 },
    { 0x73d262d0, 83 },
    { 0, 0 },
    { 0xfdd262d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x839262d5, 61 },
    { 0xa7f262d6, 91 },
    { 0xf0f262d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55262db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f262de, 43 },
    { 0x6b0262df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6262e2, 90 },
    { 0, 0 },
    { 0x805262e4, 61 },
    { 0xbcf262e4, 90 },
    { 0, 0 },
    { 0xee2262e7, 77 },
    { 0x8f262e8, 95 },
    { 0, 0 },
    { 0x91d262ea, 52 },
    { 0xb59262eb, 85 },
    { 0x651262ec, 68 },
    { 0x393262ed, 98 },
    { 0x862262eb, 69 },
    { 0x233262ed, 61 },
    { 0x3cf262ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x754262f3, 33 },
    { 0, 0 },
    { 0xe25262f5, 76 },
    { 0x2b0262f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x362262f9, 98 },
    { 0x1f6262fa, 33 },
    { 0xf68262fb, 93 },
    { 0x3bd262fb, 61 },
    { 0x226262fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75226300, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a826303, 33 },
    { 0, 0 },
    { 0xb9926305, 98 },
    { 0x99126305, 23 },
    { 0, 0 },
    { 0xf2726308, 98 },
    { 0xdda26308, 90 },
    { 0xfc62630a, 61 },
    { 0x24c26308, 23 },
    { 0xb202630a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3626310, 99 },
    { 0, 0 },
    { 0x6c726312, 90 },
    { 0, 0 },
    { 0xb2c26314, 75 },
    { 0x85326314, 84 },
    { 0x1ef26316, 61 },
    { 0x28b26317, 17 },
    { 0xd0b26316, 61 },
    { 0, 0 },
    { 0x3ca2631a, 33 },
    { 0x30c2631b, 76 },
    { 0xb932631c, 34 },
    { 0x3442631d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e26329, 76 },
    { 0x8ce2632a, 66 },
    { 0x56c2632b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd112632e, 23 },
    { 0, 0 },
    { 0xb3826330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca26334, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7826338, 68 },
    { 0x9a126339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa2633c, 61 },
    { 0x5a2633d, 76 },
    { 0xc7f2633d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91726341, 90 },
    { 0x9d526342, 91 },
    { 0, 0 },
    { 0x2fd26344, 61 },
    { 0x38b26345, 84 },
    { 0xb7226346, 90 },
    { 0xfb426347, 100 },
    { 0xa0a26344, 95 },
    { 0x39726349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2634c, 66 },
    { 0xf252634c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94226351, 76 },
    { 0xfc426352, 98 },
    { 0x5bb26353, 33 },
    { 0x19826353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c426359, 87 },
    { 0, 0 },
    { 0x8052635b, 100 },
    { 0x12a2635c, 87 },
    { 0, 0 },
    { 0xae72635e, 90 },
    { 0xc7c2635f, 85 },
    { 0xa9926360, 55 },
    { 0x18c26361, 33 },
    { 0xf1b26361, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2112636a, 43 },
    { 0x6872636b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f62637e, 76 },
    { 0xd4e2637f, 91 },
    { 0, 0 },
    { 0x5eb26381, 95 },
    { 0x78926382, 98 },
    { 0x3c626382, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d52638d, 98 },
    { 0x18b2638e, 76 },
    { 0x6442638f, 23 },
    { 0, 0 },
    { 0x3a526391, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86926394, 43 },
    { 0x7fe26395, 17 },
    { 0x8c726395, 94 },
    { 0, 0 },
    { 0xcf426398, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4263a5, 98 },
    { 0, 0 },
    { 0xb8b263a7, 68 },
    { 0x5dc263a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c263ac, 98 },
    { 0x5b263ad, 93 },
    { 0x6bd263ae, 98 },
    { 0x973263ac, 34 },
    { 0x47263b0, 43 },
    { 0x51d263b1, 95 },
    { 0xf56263b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e263b8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff263bb, 84 },
    { 0, 0 },
    { 0x808263bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c263c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de263c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e263cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x949263cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35263d3, 33 },
    { 0xfa9263d3, 46 },
    { 0, 0 },
    { 0x384263d6, 90 },
    { 0x9f1263d7, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb263dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d263e2, 90 },
    { 0x40e263e3, 33 },
    { 0xa25263e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da263e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x853263ea, 34 },
    { 0x583263eb, 35 },
    { 0x8e1263eb, 80 },
    { 0xf59263ed, 98 },
    { 0, 0 },
    { 0x327263ef, 33 },
    { 0x93263ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222263f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc263fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7726400, 76 },
    { 0, 0 },
    { 0xc7626402, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x23326405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21526408, 33 },
    { 0, 0 },
    { 0xca12640a, 98 },
    { 0x4982640a, 61 },
    { 0x3822640c, 23 },
    { 0x4192640b, 90 },
    { 0x8dd2640d, 23 },
    { 0xf3d2640f, 61 },
    { 0xbc26410, 33 },
    { 0xe7626411, 100 },
    { 0xdd126412, 23 },
    { 0x54d2640b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49326417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5742641c, 43 },
    { 0x7092641c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7826424, 55 },
    { 0xe126425, 52 },
    { 0, 0 },
    { 0x2af26427, 76 },
    { 0xbe426428, 90 },
    { 0x40626429, 61 },
    { 0xa2126428, 91 },
    { 0x3da26428, 90 },
    { 0xee32642c, 100 },
    { 0x36f2642c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef26430, 46 },
    { 0xfcb26430, 52 },
    { 0xd5a26431, 98 },
    { 0xff526433, 78 },
    { 0x4d426432, 99 },
    { 0xb9e26435, 56 },
    { 0x2c826432, 90 },
    { 0xcd526431, 46 },
    { 0x9a426430, 91 },
    { 0x85c26439, 98 },
    { 0xe9526437, 90 },
    { 0x2d42643b, 34 },
    { 0x46f26437, 43 },
    { 0x5402643d, 23 },
    { 0x5482643b, 95 },
    { 0, 0 },
    { 0x65f26440, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c126443, 84 },
    { 0xdd526444, 43 },
    { 0x36426445, 90 },
    { 0x9e426446, 43 },
    { 0xd7326444, 9 },
    { 0xf6e26446, 61 },
    { 0xd8a26449, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a02644d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73826451, 33 },
    { 0, 0 },
    { 0x1a926453, 65 },
    { 0xd4726454, 95 },
    { 0xc9e26453, 55 },
    { 0xbb826456, 73 },
    { 0x86f26455, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb2645b, 88 },
    { 0, 0 },
    { 0xd8f2645d, 98 },
    { 0xfaf2645e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16226463, 33 },
    { 0xbb326464, 33 },
    { 0x84926465, 88 },
    { 0xc3f26464, 33 },
    { 0x3f726467, 61 },
    { 0x5a526468, 33 },
    { 0x7026466, 12 },
    { 0x8832646a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d62646d, 98 },
    { 0, 0 },
    { 0xe8f2646f, 61 },
    { 0x2dc26470, 61 },
    { 0x90c2646f, 75 },
    { 0x71626470, 33 },
    { 0x42b26473, 33 },
    { 0xe7526473, 61 },
    { 0xa6a26475, 83 },
    { 0xb2326476, 75 },
    { 0x3c426477, 67 },
    { 0xd5226478, 46 },
    { 0x2762646f, 95 },
    { 0, 0 },
    { 0x9b42647b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2647f, 84 },
    { 0x43c26480, 98 },
    { 0x3fd26480, 90 },
    { 0x36426482, 76 },
    { 0x73c26480, 93 },
    { 0x3cb26484, 61 },
    { 0xa4d26485, 52 },
    { 0xece26484, 66 },
    { 0xefc26487, 60 },
    { 0xf2f26488, 61 },
    { 0x41526487, 98 },
    { 0x62c26488, 91 },
    { 0xad626484, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb26490, 91 },
    { 0xb8026490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa26494, 46 },
    { 0x7b626495, 76 },
    { 0x35526494, 98 },
    { 0x45526497, 90 },
    { 0x74e26497, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e32649b, 61 },
    { 0x512649c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e2649f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7264a7, 46 },
    { 0x299264a8, 61 },
    { 0x5a5264a9, 99 },
    { 0, 0 },
    { 0x3f264ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638264bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f264c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d264cb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8264d0, 78 },
    { 0x2f6264d1, 33 },
    { 0x7c6264d0, 68 },
    { 0, 0 },
    { 0x58f264d4, 38 },
    { 0, 0 },
    { 0xa5e264d6, 43 },
    { 0, 0 },
    { 0xe3c264d8, 23 },
    { 0x4264d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770264e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7264ea, 61 },
    { 0xd91264eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b264ef, 23 },
    { 0xe10264f0, 90 },
    { 0x9c4264ef, 76 },
    { 0xaf8264f2, 100 },
    { 0xa9b264f1, 43 },
    { 0xc51264f4, 61 },
    { 0xca6264f5, 85 },
    { 0x98264ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8264fb, 90 },
    { 0xca5264fc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d26500, 46 },
    { 0x95726501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd26506, 95 },
    { 0, 0 },
    { 0xae526508, 33 },
    { 0x42a26509, 87 },
    { 0x8f326508, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52650d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf26511, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9326516, 75 },
    { 0x72b26517, 78 },
    { 0, 0 },
    { 0x4ca26519, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae2651e, 95 },
    { 0x2e12651e, 55 },
    { 0xb2651f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6926524, 75 },
    { 0, 0 },
    { 0xbff26526, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x35626529, 33 },
    { 0, 0 },
    { 0xef22652b, 67 },
    { 0, 0 },
    { 0x34b2652d, 23 },
    { 0xe092652e, 93 },
    { 0xae82652e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c26533, 100 },
    { 0x88526534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10426539, 34 },
    { 0, 0 },
    { 0x1e22653b, 100 },
    { 0x8162653c, 95 },
    { 0xce52653d, 84 },
    { 0, 0 },
    { 0xa0f2653f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de26548, 40 },
    { 0x5d126549, 23 },
    { 0, 0 },
    { 0x3802654b, 33 },
    { 0, 0 },
    { 0xdaa2654d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4526551, 90 },
    { 0, 0 },
    { 0x80e26553, 34 },
    { 0xb8526554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6122655d, 61 },
    { 0x56e2655d, 34 },
    { 0x46a2655f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a26566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f26569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9262656c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64826570, 90 },
    { 0, 0 },
    { 0x8d726572, 33 },
    { 0, 0 },
    { 0xa5026574, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce26578, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb092657b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3226580, 98 },
    { 0xf2c26581, 90 },
    { 0x30826580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c726586, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8302658a, 90 },
    { 0x8742658b, 90 },
    { 0, 0 },
    { 0x9462658d, 66 },
    { 0xaed2658e, 34 },
    { 0x8ba2658f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa26593, 90 },
    { 0x2ed26593, 34 },
    { 0, 0 },
    { 0x76326596, 61 },
    { 0xac26597, 76 },
    { 0x90626598, 85 },
    { 0x7de26596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e2659e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26265a1, 76 },
    { 0xaea265a2, 23 },
    { 0x3f8265a3, 33 },
    { 0, 0 },
    { 0x66d265a5, 34 },
    { 0x5e5265a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a265a9, 61 },
    { 0xd75265aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9265af, 23 },
    { 0x44b265b0, 90 },
    { 0xa66265b1, 61 },
    { 0x271265b2, 33 },
    { 0x9b9265b3, 61 },
    { 0xff3265b4, 76 },
    { 0xe67265b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94265b8, 98 },
    { 0x179265b8, 66 },
    { 0x46d265ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322265c2, 81 },
    { 0, 0 },
    { 0xdf265c4, 90 },
    { 0, 0 },
    { 0x5f8265c6, 98 },
    { 0x4a6265c6, 90 },
    { 0x53d265c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb265cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5265d9, 99 },
    { 0xc2c265d9, 34 },
    { 0, 0 },
    { 0xf10265dc, 95 },
    { 0x3a0265dd, 19 },
    { 0x444265dc, 90 },
    { 0, 0 },
    { 0xf5f265e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4265e6, 23 },
    { 0xd53265e6, 90 },
    { 0xd63265e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae265ed, 36 },
    { 0, 0 },
    { 0xd10265ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9265f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75265fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e26602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2326609, 46 },
    { 0x85e2660a, 65 },
    { 0x5012660b, 33 },
    { 0, 0 },
    { 0xf252660d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e26610, 61 },
    { 0x25626611, 95 },
    { 0xdf726611, 34 },
    { 0x2af26613, 33 },
    { 0x37d26613, 95 },
    { 0x9c126615, 76 },
    { 0xe4026616, 55 },
    { 0xe4626617, 68 },
    { 0x77b26612, 95 },
    { 0x6a826617, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26620, 90 },
    { 0xf3026620, 34 },
    { 0xc26621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3326625, 23 },
    { 0x8d426626, 91 },
    { 0x89126625, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5752662a, 65 },
    { 0x7d2662b, 76 },
    { 0, 0 },
    { 0x3db2662d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c26632, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa26635, 66 },
    { 0xa5f26635, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d42663a, 23 },
    { 0x49e2663b, 33 },
    { 0x98a2663c, 90 },
    { 0x9c32663b, 76 },
    { 0x41a2663e, 2 },
    { 0x34e2663e, 90 },
    { 0x6b92663f, 34 },
    { 0x63d26641, 61 },
    { 0xfc926642, 46 },
    { 0xc8326642, 93 },
    { 0x2c926644, 61 },
    { 0x14c26643, 66 },
    { 0xe7f26646, 40 },
    { 0x29926647, 33 },
    { 0x3dc26647, 90 },
    { 0xd6f26643, 43 },
    { 0x952664a, 34 },
    { 0xf3a2664b, 76 },
    { 0x342664c, 33 },
    { 0x7f92663a, 23 },
    { 0xfa02663f, 95 },
    { 0x13f2664a, 55 },
    { 0xf9f26650, 71 },
    { 0xb2526650, 90 },
    { 0x1f72664c, 23 },
    { 0xa572664c, 80 },
    { 0xfeb26654, 90 },
    { 0, 0 },
    { 0x3fc26656, 100 },
    { 0x22126657, 90 },
    { 0x60f26658, 98 },
    { 0, 0 },
    { 0xddf2665a, 75 },
    { 0, 0 },
    { 0x3e82665c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f526666, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a22666b, 85 },
    { 0xe1f2666c, 61 },
    { 0x7482666d, 85 },
    { 0xf4e2666e, 93 },
    { 0xfb2666e, 76 },
    { 0, 0 },
    { 0x1c26671, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c626680, 98 },
    { 0x8826681, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa26685, 87 },
    { 0, 0 },
    { 0x43826687, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f12668a, 33 },
    { 0, 0 },
    { 0x6832668c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b826692, 100 },
    { 0x68726693, 98 },
    { 0, 0 },
    { 0x50e26695, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef52669a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf362669e, 45 },
    { 0x20a2669f, 46 },
    { 0xfb0266a0, 81 },
    { 0x3db266a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x567266a4, 90 },
    { 0, 0 },
    { 0x211266a6, 55 },
    { 0, 0 },
    { 0x3a6266a8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c266ab, 53 },
    { 0xa24266ab, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6266b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd266b5, 33 },
    { 0x6e2266b6, 55 },
    { 0, 0 },
    { 0x725266b8, 90 },
    { 0xac6266b9, 33 },
    { 0xfe1266ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9266bd, 93 },
    { 0xa8d266be, 23 },
    { 0x8b8266bf, 61 },
    { 0xdc3266c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a266d0, 76 },
    { 0xd1a266d1, 43 },
    { 0x44a266d2, 95 },
    { 0x252266d3, 33 },
    { 0x847266d4, 25 },
    { 0xdaa266d1, 23 },
    { 0x2c0266d2, 12 },
    { 0xc19266d0, 55 },
    { 0, 0 },
    { 0xb45266d9, 23 },
    { 0xb6a266da, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453266de, 33 },
    { 0xd0c266de, 98 },
    { 0xa6b266e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x116266e3, 61 },
    { 0xb8a266e3, 88 },
    { 0x30f266e3, 99 },
    { 0x14a266e6, 33 },
    { 0x559266e7, 78 },
    { 0xa99266e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7266ed, 85 },
    { 0xea266ee, 76 },
    { 0x429266ee, 52 },
    { 0x94d266f0, 95 },
    { 0x6f6266f1, 90 },
    { 0, 0 },
    { 0x7e5266f3, 33 },
    { 0x382266f4, 98 },
    { 0x3f4266f4, 23 },
    { 0x7ec266f3, 23 },
    { 0x2c0266f7, 90 },
    { 0xceb266f8, 85 },
    { 0, 0 },
    { 0xb99266fa, 72 },
    { 0xc84266fb, 90 },
    { 0x727266fb, 61 },
    { 0xbd266fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f26702, 90 },
    { 0x90726703, 77 },
    { 0x26326704, 34 },
    { 0xf1c26702, 33 },
    { 0x9eb26703, 34 },
    { 0, 0 },
    { 0xe3b26708, 90 },
    { 0x92026709, 40 },
    { 0x3b926708, 34 },
    { 0xa42670a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb2670f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb126714, 43 },
    { 0xf9726715, 90 },
    { 0, 0 },
    { 0xa3a26717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c26721, 76 },
    { 0xc2f26721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76626726, 66 },
    { 0, 0 },
    { 0x6c926728, 23 },
    { 0x94126728, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c626732, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf226741, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a126744, 66 },
    { 0x31226745, 78 },
    { 0, 0 },
    { 0xcdf26747, 93 },
    { 0, 0 },
    { 0x69526749, 88 },
    { 0xf492674a, 77 },
    { 0x7a326749, 33 },
    { 0, 0 },
    { 0xeb22674d, 61 },
    { 0xcc32674e, 34 },
    { 0xfd72674f, 55 },
    { 0, 0 },
    { 0xe8b26751, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea726755, 33 },
    { 0, 0 },
    { 0x4b826757, 90 },
    { 0, 0 },
    { 0x7f426759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2675f, 23 },
    { 0xaf826760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e26768, 34 },
    { 0, 0 },
    { 0xac02676a, 90 },
    { 0x4a62676b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6852676f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b26773, 43 },
    { 0xbe126774, 90 },
    { 0x97426775, 61 },
    { 0xfb426776, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20526779, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a2677d, 61 },
    { 0x5d12677e, 91 },
    { 0x5c32677e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc52678c, 97 },
    { 0x6852678c, 99 },
    { 0, 0 },
    { 0xf3f2678f, 23 },
    { 0xb5926790, 90 },
    { 0x2d326790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74626794, 23 },
    { 0x69526795, 95 },
    { 0x94d26796, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a26799, 98 },
    { 0x78c2679a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982679e, 23 },
    { 0xb0a2679f, 84 },
    { 0, 0 },
    { 0xb0f267a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd267a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f267a9, 95 },
    { 0x563267aa, 76 },
    { 0x647267ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8267af, 96 },
    { 0, 0 },
    { 0xb4b267b1, 61 },
    { 0, 0 },
    { 0xb66267b3, 98 },
    { 0x922267b4, 64 },
    { 0xafa267b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e267bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db267be, 33 },
    { 0xd1d267bf, 34 },
    { 0, 0 },
    { 0xf2f267c1, 95 },
    { 0x7e5267c2, 76 },
    { 0x7a0267c1, 90 },
    { 0x3aa267c4, 34 },
    { 0x48e267c4, 90 },
    { 0x9aa267c4, 95 },
    { 0x445267c7, 55 },
    { 0xcbf267c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718267cd, 98 },
    { 0xef1267ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb267d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d267da, 98 },
    { 0x560267db, 76 },
    { 0x781267db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14267df, 95 },
    { 0, 0 },
    { 0x43e267e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f267e6, 23 },
    { 0x9b5267e7, 61 },
    { 0, 0 },
    { 0x11f267e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d267f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2267f7, 55 },
    { 0, 0 },
    { 0xd85267f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58267fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3782680a, 95 },
    { 0xcab2680b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb226810, 88 },
    { 0, 0 },
    { 0x44d26812, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e126815, 98 },
    { 0x76126816, 78 },
    { 0x6dd26816, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc192681d, 67 },
    { 0xf032681e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68026823, 74 },
    { 0x40626823, 55 },
    { 0, 0 },
    { 0xc6d26826, 98 },
    { 0, 0 },
    { 0x5b726828, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a22682b, 99 },
    { 0xdcf2682c, 98 },
    { 0x38f2682c, 34 },
    { 0xeaa2682e, 61 },
    { 0xd112682c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba26832, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd026839, 33 },
    { 0xeb626839, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec2683d, 98 },
    { 0xd492683e, 17 },
    { 0, 0 },
    { 0xb9a26840, 93 },
    { 0xf6026841, 34 },
    { 0xb4926842, 33 },
    { 0xff626841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd926847, 90 },
    { 0xe6426848, 76 },
    { 0x64126849, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed2684f, 34 },
    { 0, 0 },
    { 0x1bd26851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d526855, 33 },
    { 0, 0 },
    { 0x4f526857, 83 },
    { 0x4e826857, 98 },
    { 0x10426859, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5692685e, 98 },
    { 0xb372685f, 67 },
    { 0, 0 },
    { 0x8be26861, 75 },
    { 0, 0 },
    { 0x31226863, 84 },
    { 0, 0 },
    { 0x7b026865, 98 },
    { 0x1ca26865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac726869, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f42686f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x82326872, 99 },
    { 0xc2c26873, 61 },
    { 0, 0 },
    { 0xa7426875, 90 },
    { 0x15d26875, 76 },
    { 0, 0 },
    { 0x85f26878, 55 },
    { 0xef526879, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea2687d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad926889, 55 },
    { 0, 0 },
    { 0x1b12688b, 23 },
    { 0xfb92688c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e26890, 33 },
    { 0xb8726891, 23 },
    { 0x42726892, 33 },
    { 0, 0 },
    { 0x74226894, 93 },
    { 0xd9726895, 98 },
    { 0x85626896, 98 },
    { 0xce526896, 61 },
    { 0, 0 },
    { 0x3af26899, 90 },
    { 0, 0 },
    { 0x2742689b, 23 },
    { 0x8932689b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d268a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb268aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342268ae, 23 },
    { 0x87c268af, 98 },
    { 0, 0 },
    { 0xe76268b1, 90 },
    { 0x4e3268b2, 55 },
    { 0, 0 },
    { 0x376268b4, 33 },
    { 0, 0 },
    { 0xe7d268b6, 23 },
    { 0, 0 },
    { 0x49d268b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5268bc, 90 },
    { 0xf74268bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15268c5, 33 },
    { 0, 0 },
    { 0x2c6268c7, 23 },
    { 0xfdb268c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c268cb, 23 },
    { 0x767268cb, 43 },
    { 0, 0 },
    { 0x9ff268ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a268d1, 43 },
    { 0x39268d2, 67 },
    { 0xe8a268d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f268d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe268de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4268e6, 61 },
    { 0xa56268e6, 43 },
    { 0, 0 },
    { 0xb44268e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2268ed, 46 },
    { 0xcc6268ed, 61 },
    { 0xc6a268ef, 61 },
    { 0xa82268f0, 34 },
    { 0x9bf268f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c268f4, 34 },
    { 0, 0 },
    { 0x696268f6, 23 },
    { 0xe2d268f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3268fa, 23 },
    { 0x67b268fb, 95 },
    { 0x78c268fc, 95 },
    { 0x602268fd, 75 },
    { 0x6e9268fe, 94 },
    { 0x1e2268ff, 61 },
    { 0xdac268ff, 78 },
    { 0xe28268fe, 95 },
    { 0x33826902, 61 },
    { 0x42a26903, 23 },
    { 0x88126904, 84 },
    { 0x41826902, 61 },
    { 0x60126906, 69 },
    { 0x5cb26907, 95 },
    { 0x98426907, 98 },
    { 0x30f26909, 61 },
    { 0xb8826906, 84 },
    { 0, 0 },
    { 0x77b2690c, 66 },
    { 0, 0 },
    { 0xb942690e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a26911, 90 },
    { 0, 0 },
    { 0x2226913, 76 },
    { 0, 0 },
    { 0xc3f26915, 95 },
    { 0xf1026916, 46 },
    { 0x91526917, 34 },
    { 0x9c26917, 95 },
    { 0x9926919, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef82691e, 34 },
    { 0xcb42691f, 98 },
    { 0x9fa26920, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e526926, 87 },
    { 0xd4526927, 33 },
    { 0x6f526926, 75 },
    { 0, 0 },
    { 0x4922692a, 95 },
    { 0, 0 },
    { 0x2792692c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c26930, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb26935, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77926938, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d2693d, 96 },
    { 0x5982693e, 87 },
    { 0, 0 },
    { 0x92826940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b26943, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1626946, 33 },
    { 0x86226947, 61 },
    { 0, 0 },
    { 0xcf726949, 55 },
    { 0xd352694a, 33 },
    { 0, 0 },
    { 0xc362694c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x792694f, 33 },
    { 0x77526950, 98 },
    { 0, 0 },
    { 0x97326952, 93 },
    { 0x52d26953, 90 },
    { 0x24e26954, 56 },
    { 0x95526953, 33 },
    { 0x70626956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb26961, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1126964, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f92696f, 90 },
    { 0xe4126970, 61 },
    { 0xf2026971, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a26974, 61 },
    { 0x28d26975, 34 },
    { 0x2ed26974, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9726979, 33 },
    { 0xdc326979, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb62697e, 85 },
    { 0xa782697f, 23 },
    { 0x2be26980, 76 },
    { 0xf1426981, 43 },
    { 0x9692697e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf26985, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7452698c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca26993, 55 },
    { 0, 0 },
    { 0x21226995, 33 },
    { 0x55226996, 95 },
    { 0xa1d26996, 83 },
    { 0xccb26998, 24 },
    { 0xcc926999, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5062699d, 95 },
    { 0xa812699d, 95 },
    { 0xb6c2699f, 33 },
    { 0, 0 },
    { 0xcd7269a1, 100 },
    { 0, 0 },
    { 0xfac269a3, 90 },
    { 0, 0 },
    { 0xc33269a5, 23 },
    { 0x6a3269a6, 98 },
    { 0x24f269a7, 90 },
    { 0x29f269a8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c269b0, 73 },
    { 0x23269b1, 55 },
    { 0x99a269b2, 95 },
    { 0x18c269b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d269b6, 95 },
    { 0x9d9269b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332269bf, 23 },
    { 0x84b269bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f269c3, 61 },
    { 0x88f269c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88269c7, 80 },
    { 0x34f269c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8269cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65269d1, 95 },
    { 0x939269d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x703269d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90269d8, 98 },
    { 0xe72269d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5269df, 33 },
    { 0, 0 },
    { 0xbb3269e1, 61 },
    { 0xa68269e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15269e6, 55 },
    { 0xafd269e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce269ec, 90 },
    { 0, 0 },
    { 0xf61269ee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2269f3, 90 },
    { 0xc87269f3, 43 },
    { 0x8b7269f5, 99 },
    { 0xc21269f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7269fc, 95 },
    { 0xf22269fc, 34 },
    { 0, 0 },
    { 0x2b6269ff, 46 },
    { 0, 0 },
    { 0xdc26a01, 61 },
    { 0xd8226a02, 55 },
    { 0x37926a03, 90 },
    { 0, 0 },
    { 0x34f26a05, 100 },
    { 0xb5526a05, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0826a0b, 76 },
    { 0, 0 },
    { 0xbc426a0d, 90 },
    { 0, 0 },
    { 0x25626a0f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da26a13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a26a22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d26a29, 61 },
    { 0xec126a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d326a32, 90 },
    { 0, 0 },
    { 0x85026a34, 46 },
    { 0, 0 },
    { 0x25626a36, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d26a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f26a40, 81 },
    { 0xf0226a40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee26a47, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58826a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda926a50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab626a5a, 33 },
    { 0x9c226a5b, 90 },
    { 0x3b726a5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e826a60, 75 },
    { 0x9a926a61, 33 },
    { 0x28926a60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c26a65, 23 },
    { 0xba626a66, 34 },
    { 0x4b926a67, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83926a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e26a6f, 76 },
    { 0xa6c26a70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1626a75, 98 },
    { 0xe5026a75, 90 },
    { 0, 0 },
    { 0x43826a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47926a7c, 61 },
    { 0, 0 },
    { 0x91226a7e, 90 },
    { 0, 0 },
    { 0xba026a80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57226a83, 61 },
    { 0xcec26a84, 90 },
    { 0, 0 },
    { 0x98e26a86, 90 },
    { 0, 0 },
    { 0xe9026a88, 76 },
    { 0xe0d26a89, 95 },
    { 0x58326a8a, 61 },
    { 0xd7826a89, 43 },
    { 0xd3f26a8c, 61 },
    { 0xdee26a88, 90 },
    { 0xa5526a8c, 43 },
    { 0x1d226a89, 75 },
    { 0, 0 },
    { 0xe3b26a91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52226a94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea26a99, 85 },
    { 0xdb226a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee26aa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd26aa6, 34 },
    { 0x46326aa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe26aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a726aae, 91 },
    { 0, 0 },
    { 0xc1126ab0, 91 },
    { 0, 0 },
    { 0x67a26ab2, 76 },
    { 0x33f26ab3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf926ab7, 33 },
    { 0xc0426ab7, 99 },
    { 0, 0 },
    { 0x8ce26aba, 90 },
    { 0, 0 },
    { 0x4af26abc, 95 },
    { 0x37d26abd, 90 },
    { 0x15d26abe, 23 },
    { 0x3b926abd, 61 },
    { 0xbed26ac0, 78 },
    { 0x59126ac1, 95 },
    { 0xde826ac0, 40 },
    { 0x8f26ac3, 95 },
    { 0xee026ac4, 33 },
    { 0xf6426ac5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d726ace, 95 },
    { 0x3f726acf, 90 },
    { 0x22b26acf, 95 },
    { 0x5e826ad0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55126ad5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d26ae8, 67 },
    { 0x9f426ae8, 33 },
    { 0xcfd26aea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27726aee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1226af1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d26af5, 69 },
    { 0x11c26af6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3126af9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46526afe, 81 },
    { 0, 0 },
    { 0x9e626b00, 66 },
    { 0x24e26b01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82226b04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa26b07, 29 },
    { 0x74b26b08, 78 },
    { 0x26226b09, 90 },
    { 0, 0 },
    { 0x73c26b0b, 85 },
    { 0, 0 },
    { 0x6fa26b0d, 90 },
    { 0x78826b0d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d26b12, 76 },
    { 0, 0 },
    { 0xdd826b14, 52 },
    { 0, 0 },
    { 0xee026b16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74626b20, 78 },
    { 0x8026b21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89026b24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb26b28, 78 },
    { 0x32626b28, 76 },
    { 0x56926b29, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc26b2d, 77 },
    { 0, 0 },
    { 0x89f26b2f, 98 },
    { 0x2f826b30, 77 },
    { 0xb1d26b31, 90 },
    { 0x48b26b31, 100 },
    { 0x6dc26b32, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf526b37, 55 },
    { 0xfe926b38, 23 },
    { 0x49d26b39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5626b40, 33 },
    { 0xac526b41, 95 },
    { 0, 0 },
    { 0x95226b43, 91 },
    { 0xd4826b44, 90 },
    { 0, 0 },
    { 0x11c26b46, 46 },
    { 0xeb026b46, 90 },
    { 0x23126b47, 46 },
    { 0x1c126b46, 98 },
    { 0x90626b4a, 61 },
    { 0xb6b26b48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf026b4e, 95 },
    { 0x8726b4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed926b55, 52 },
    { 0x54b26b56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a026b59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c26b5d, 55 },
    { 0xa7526b5d, 33 },
    { 0x27f26b5e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45026b68, 95 },
    { 0xec926b69, 55 },
    { 0, 0 },
    { 0xe5e26b6b, 52 },
    { 0x8926b6c, 90 },
    { 0x31c26b6d, 91 },
    { 0xf3026b6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8426b74, 90 },
    { 0x60026b75, 90 },
    { 0x44126b75, 91 },
    { 0, 0 },
    { 0x8c26b78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82326b7d, 51 },
    { 0, 0 },
    { 0x55526b7f, 90 },
    { 0xdd626b80, 33 },
    { 0x93526b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3326b87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86026b8d, 33 },
    { 0, 0 },
    { 0x8f426b8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd26b97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb726b9a, 98 },
    { 0xaee26b9b, 76 },
    { 0, 0 },
    { 0x18c26b9d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31326ba1, 68 },
    { 0, 0 },
    { 0xee26ba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a526ba8, 84 },
    { 0, 0 },
    { 0xfd926baa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6326bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d26bb2, 43 },
    { 0, 0 },
    { 0xff726bb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6626bbc, 43 },
    { 0, 0 },
    { 0xca726bbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52226bc2, 81 },
    { 0x1a026bc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17926bc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75126bcc, 98 },
    { 0xb7626bcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c126bd0, 76 },
    { 0xe4226bd1, 43 },
    { 0, 0 },
    { 0xc9c26bd3, 61 },
    { 0, 0 },
    { 0x26a26bd5, 46 },
    { 0xa1926bd6, 78 },
    { 0x80526bd6, 33 },
    { 0x6db26bd6, 33 },
    { 0x5f526bd6, 23 },
    { 0xb7426bd9, 33 },
    { 0xef426bd7, 84 },
    { 0x5f726bdc, 90 },
    { 0x27a26bdc, 52 },
    { 0x1ee26bde, 93 },
    { 0x4fa26bd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21026be2, 33 },
    { 0x7ce26be3, 76 },
    { 0xe0026be4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5826be9, 98 },
    { 0x19e26bea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29626bed, 33 },
    { 0, 0 },
    { 0x8da26bef, 23 },
    { 0x4ac26bef, 34 },
    { 0x92a26bf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea26bf8, 76 },
    { 0, 0 },
    { 0x9fc26bfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42926bfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a26c09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d426c0c, 55 },
    { 0xec226c0d, 56 },
    { 0, 0 },
    { 0xaf526c0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa26c16, 93 },
    { 0x86526c17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf826c1a, 61 },
    { 0x1ad26c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c21, 43 },
    { 0xadb26c22, 90 },
    { 0x94926c23, 90 },
    { 0xbe326c24, 23 },
    { 0x46326c22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30326c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac26c2c, 46 },
    { 0, 0 },
    { 0xc5826c2e, 98 },
    { 0xdfa26c2e, 99 },
    { 0, 0 },
    { 0xbb426c31, 77 },
    { 0xaf626c31, 66 },
    { 0x34926c33, 90 },
    { 0x3fc26c34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a526c38, 90 },
    { 0x6c626c39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9626c3f, 61 },
    { 0x58526c40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d26c43, 90 },
    { 0, 0 },
    { 0x50226c45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc326c49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9726c51, 99 },
    { 0x63b26c52, 55 },
    { 0x9fc26c53, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d26c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5626c59, 33 },
    { 0xeae26c59, 90 },
    { 0, 0 },
    { 0xd6d26c5c, 90 },
    { 0x8cd26c5d, 87 },
    { 0xa1726c5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa426c67, 48 },
    { 0, 0 },
    { 0x8e626c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f26c6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66626c73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826c77, 95 },
    { 0x5c626c78, 100 },
    { 0, 0 },
    { 0xb0126c7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d26c82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a26c85, 61 },
    { 0, 0 },
    { 0x4cd26c87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4226c8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af26c92, 61 },
    { 0, 0 },
    { 0x7ff26c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1926c9c, 95 },
    { 0x3e726c9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a726ca2, 90 },
    { 0xa3926ca3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc426ca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22726cab, 66 },
    { 0x3fb26cac, 33 },
    { 0, 0 },
    { 0x4c426cae, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec726cb3, 76 },
    { 0xd3826cb4, 34 },
    { 0xb4c26cb5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x53426cb8, 78 },
    { 0xcee26cb9, 23 },
    { 0xb0026cba, 87 },
    { 0xf2e26cb9, 77 },
    { 0x44e26cbb, 95 },
    { 0x20426cbc, 33 },
    { 0x2e26cbb, 61 },
    { 0x8cf26cbf, 34 },
    { 0x42326cb8, 98 },
    { 0x1f026cc1, 23 },
    { 0x78f26cc2, 85 },
    { 0xb4626cc3, 23 },
    { 0xc5326cc4, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e26cc8, 43 },
    { 0xb6726cc9, 95 },
    { 0, 0 },
    { 0xa4c26ccb, 66 },
    { 0, 0 },
    { 0x4a26ccd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3826cd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2826cd5, 95 },
    { 0x3b26cd6, 51 },
    { 0xf3326cd7, 90 },
    { 0x7f126cd8, 87 },
    { 0, 0 },
    { 0x9bb26cda, 87 },
    { 0xbef26cda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea26ce3, 55 },
    { 0xb7d26ce4, 90 },
    { 0xb7826ce4, 90 },
    { 0x81c26ce6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde226cec, 91 },
    { 0x18026ced, 75 },
    { 0x3426cee, 23 },
    { 0, 0 },
    { 0xbc726cf0, 91 },
    { 0, 0 },
    { 0x43926cf2, 60 },
    { 0x5a526cf3, 67 },
    { 0x31a26cf4, 91 },
    { 0x15f26cf5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91426cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf126d00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9726d04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c26d08, 87 },
    { 0xe9026d08, 99 },
    { 0xe5326d0a, 98 },
    { 0xeca26d0b, 61 },
    { 0, 0 },
    { 0x4f226d0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45626d12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8626d15, 83 },
    { 0, 0 },
    { 0x8fb26d17, 91 },
    { 0xec626d17, 76 },
    { 0x58826d17, 98 },
    { 0xe9c26d1a, 55 },
    { 0x67426d17, 76 },
    { 0x66a26d1c, 90 },
    { 0xb9626d1a, 98 },
    { 0x1d726d1e, 95 },
    { 0x9c826d1f, 55 },
    { 0x4de26d1f, 84 },
    { 0x2a026d21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d226d24, 55 },
    { 0, 0 },
    { 0x4d226d26, 76 },
    { 0x15826d27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f26d2a, 33 },
    { 0, 0 },
    { 0xe2f26d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d26d2f, 78 },
    { 0x51e26d30, 98 },
    { 0xad326d30, 90 },
    { 0xe5726d32, 33 },
    { 0x2b326d32, 90 },
    { 0xe826d34, 66 },
    { 0, 0 },
    { 0x3b026d36, 98 },
    { 0x79c26d37, 23 },
    { 0xfef26d38, 33 },
    { 0x1b626d39, 67 },
    { 0x32b26d39, 61 },
    { 0xb4126d3b, 55 },
    { 0x7626d3a, 95 },
    { 0x1c026d3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19826d41, 98 },
    { 0x92f26d42, 98 },
    { 0x3fa26d43, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xea826d46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca026d4e, 23 },
    { 0x94b26d4f, 34 },
    { 0, 0 },
    { 0xf9a26d51, 98 },
    { 0x64b26d51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50326d57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc26d5b, 85 },
    { 0, 0 },
    { 0x61f26d5d, 91 },
    { 0x62426d5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba26d64, 46 },
    { 0, 0 },
    { 0x53c26d66, 52 },
    { 0xe0226d66, 40 },
    { 0x86626d66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49226d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc226d72, 34 },
    { 0xcc926d73, 99 },
    { 0xb3a26d74, 55 },
    { 0x10326d74, 85 },
    { 0xf8126d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad26d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d726d80, 76 },
    { 0xff726d80, 59 },
    { 0x82426d82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96626d86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40926d8a, 61 },
    { 0xbc826d8b, 23 },
    { 0xd1426d8c, 66 },
    { 0xdbc26d8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x30826d90, 33 },
    { 0x46426d91, 78 },
    { 0x8fd26d92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36926d9c, 90 },
    { 0, 0 },
    { 0xdcf26d9e, 33 },
    { 0xfa626d9e, 67 },
    { 0x5cc26da0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d26da4, 95 },
    { 0x17626da5, 87 },
    { 0x48126da5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe226db6, 78 },
    { 0xa4326db6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c26dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c26dbf, 33 },
    { 0xe7926dc0, 95 },
    { 0x17326dbf, 90 },
    { 0xdc826dc2, 52 },
    { 0xe0126dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48826dca, 95 },
    { 0x45a26dcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b26dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4926dd2, 61 },
    { 0, 0 },
    { 0x4fd26dd4, 67 },
    { 0x22f26dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b26ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6626de0, 86 },
    { 0xc3e26de1, 99 },
    { 0x66626de0, 33 },
    { 0xdee26de3, 55 },
    { 0, 0 },
    { 0xf1526de5, 87 },
    { 0x23c26de5, 76 },
    { 0x75d26de7, 95 },
    { 0x81a26de6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75826dec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f126df7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4526dfd, 78 },
    { 0x36826dfe, 23 },
    { 0, 0 },
    { 0x76126e00, 87 },
    { 0x34026e00, 33 },
    { 0x8b126e02, 23 },
    { 0, 0 },
    { 0xc3126e04, 23 },
    { 0x97826e05, 100 },
    { 0, 0 },
    { 0x2bc26e07, 55 },
    { 0xd526e07, 95 },
    { 0, 0 },
    { 0xbd626e0a, 34 },
    { 0xdc526e0b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b926e10, 33 },
    { 0x1cc26e10, 33 },
    { 0, 0 },
    { 0x7c226e13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac26e1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b26e1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d26e23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69126e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd226e2f, 33 },
    { 0, 0 },
    { 0x4f326e31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f26e38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8026e3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5626e44, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6726e49, 23 },
    { 0x1c026e49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a226e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f26e54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f026e62, 33 },
    { 0xe8726e63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa26e68, 61 },
    { 0, 0 },
    { 0x9726e6a, 52 },
    { 0x7bc26e6b, 77 },
    { 0x24b26e6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51626e6f, 98 },
    { 0xcbb26e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e26e79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb26e7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40826e80, 95 },
    { 0, 0 },
    { 0x4cb26e82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57726e85, 84 },
    { 0xcc626e86, 76 },
    { 0xdb426e87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b826e8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97026e91, 34 },
    { 0xfb326e92, 85 },
    { 0, 0 },
    { 0x36d26e94, 34 },
    { 0x2a426e95, 61 },
    { 0x6bf26e96, 90 },
    { 0x11a26e97, 23 },
    { 0x5ee26e96, 87 },
    { 0xa3f26e96, 34 },
    { 0x63126e98, 61 },
    { 0xadc26e9b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f426ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d26ea6, 78 },
    { 0x89126ea7, 23 },
    { 0xce526ea8, 43 },
    { 0xa926ea6, 52 },
    { 0x8ff26ea8, 23 },
    { 0x2f026eab, 90 },
    { 0x4dc26ea9, 33 },
    { 0, 0 },
    { 0x34726eae, 34 },
    { 0x86726eaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63626eb5, 33 },
    { 0, 0 },
    { 0xac926eb7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b26ebc, 99 },
    { 0, 0 },
    { 0x58726ebe, 72 },
    { 0xd2326ebe, 46 },
    { 0x86e26ebf, 35 },
    { 0, 0 },
    { 0x9f226ec2, 17 },
    { 0, 0 },
    { 0x13d26ec4, 90 },
    { 0x56426ec5, 17 },
    { 0x94426ec5, 19 },
    { 0x80c26ec7, 68 },
    { 0x33526ec4, 17 },
    { 0xb9d26ec9, 33 },
    { 0x36526ec9, 98 },
    { 0xaa026ec9, 98 },
    { 0xe6d26ecc, 85 },
    { 0x41f26eca, 98 },
    { 0xea26ecc, 91 },
    { 0xd1326ecf, 88 },
    { 0, 0 },
    { 0xc9626ed1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e226ed7, 23 },
    { 0x37926ed8, 90 },
    { 0x91726ed9, 61 },
    { 0, 0 },
    { 0xe8b26edb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec26ee6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb626eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60026eef, 98 },
    { 0x50526ef0, 67 },
    { 0x67a26ef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c26ef6, 33 },
    { 0xe8526ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe026efb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66226f04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b526f10, 52 },
    { 0x47c26f10, 34 },
    { 0xa4726f12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b26f15, 61 },
    { 0x75326f15, 33 },
    { 0xcdc26f16, 23 },
    { 0, 0 },
    { 0x55726f19, 90 },
    { 0x7e626f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1926f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74526f22, 55 },
    { 0, 0 },
    { 0x2c626f24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f26f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33626f2b, 69 },
    { 0xf6726f2c, 93 },
    { 0x96926f2d, 61 },
    { 0x32726f2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4126f31, 46 },
    { 0x69526f32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86526f36, 44 },
    { 0, 0 },
    { 0xf7d26f38, 93 },
    { 0xbe026f39, 34 },
    { 0, 0 },
    { 0x90b26f3b, 90 },
    { 0xd8226f3c, 33 },
    { 0x25b26f3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd726f44, 95 },
    { 0x9ed26f45, 98 },
    { 0x87426f46, 90 },
    { 0, 0 },
    { 0xec426f48, 84 },
    { 0, 0 },
    { 0xf0f26f4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba126f4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f26f53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e26f56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed26f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b26f5c, 66 },
    { 0xca126f5c, 76 },
    { 0, 0 },
    { 0xac526f5f, 98 },
    { 0x6ce26f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f26f69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f26f71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e26f76, 23 },
    { 0xfb26f77, 61 },
    { 0xb1426f78, 90 },
    { 0x5df26f79, 46 },
    { 0xe5126f7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f26f7f, 55 },
    { 0x1ae26f7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b26f86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a326f8e, 93 },
    { 0, 0 },
    { 0xba126f90, 61 },
    { 0x82c26f90, 93 },
    { 0, 0 },
    { 0xfab26f93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b26f97, 84 },
    { 0, 0 },
    { 0xf5c26f99, 98 },
    { 0, 0 },
    { 0x75126f9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b26f9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e326fa7, 90 },
    { 0x5726fa8, 91 },
    { 0, 0 },
    { 0x4b826faa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40926fb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d26fb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d26fba, 85 },
    { 0xbbc26fba, 33 },
    { 0, 0 },
    { 0x6b26fbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51926fc1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1326fc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d26fc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93626fca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a26fd1, 33 },
    { 0xcfe26fd1, 61 },
    { 0x78b26fd1, 66 },
    { 0, 0 },
    { 0x54d26fd5, 61 },
    { 0xbd326fd5, 98 },
    { 0xa0526fd6, 90 },
    { 0xa9726fd8, 100 },
    { 0xe0026fd8, 67 },
    { 0xc0e26fd9, 77 },
    { 0xe2726fdb, 33 },
    { 0x8c026fdc, 66 },
    { 0x30426fdd, 33 },
    { 0x7fc26fdc, 95 },
    { 0x57426fd8, 99 },
    { 0x68726fd7, 34 },
    { 0, 0 },
    { 0x5ae26fe2, 61 },
    { 0x4026fe2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a26fe7, 90 },
    { 0, 0 },
    { 0xe6b26fe9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64926fee, 99 },
    { 0x4e126fee, 98 },
    { 0x45026ff0, 17 },
    { 0x37526ff1, 78 },
    { 0x67826ff1, 76 },
    { 0xc6226fef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59026ffb, 51 },
    { 0x32026ffb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93527002, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c2700a, 43 },
    { 0xdd92700b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb27011, 99 },
    { 0x5d227012, 23 },
    { 0, 0 },
    { 0x79c27014, 90 },
    { 0x50727014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc427019, 52 },
    { 0xa192701a, 61 },
    { 0x64a2701a, 17 },
    { 0x73d2701c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a27023, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d927028, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb2702d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30527031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e27035, 91 },
    { 0xb1c27036, 33 },
    { 0x38327037, 93 },
    { 0xf0627038, 90 },
    { 0, 0 },
    { 0xf0e2703a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad927040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71227043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb92704a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a62704f, 33 },
    { 0, 0 },
    { 0x43d27051, 98 },
    { 0xf2027052, 46 },
    { 0, 0 },
    { 0x22127054, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a27058, 68 },
    { 0, 0 },
    { 0xf6d2705a, 61 },
    { 0, 0 },
    { 0xf982705c, 33 },
    { 0x5592705d, 61 },
    { 0x8572705e, 95 },
    { 0x5082705f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x47327062, 23 },
    { 0x89827063, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29127068, 34 },
    { 0x17c27069, 98 },
    { 0x37a2706a, 33 },
    { 0x90d2706b, 98 },
    { 0x94627068, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff27074, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2707a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f2707d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65327082, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10327088, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472708d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca127093, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b27096, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae27099, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe732709e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801270a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb270ab, 33 },
    { 0xc56270ac, 90 },
    { 0xc05270ad, 90 },
    { 0xcc7270ad, 33 },
    { 0, 0 },
    { 0xd91270b0, 76 },
    { 0, 0 },
    { 0x394270b2, 90 },
    { 0xe53270b3, 90 },
    { 0xe49270b4, 90 },
    { 0x37270b3, 90 },
    { 0x7ca270b4, 34 },
    { 0xc31270b7, 33 },
    { 0, 0 },
    { 0x79f270b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6270c4, 46 },
    { 0xdfa270c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0270cd, 61 },
    { 0x706270ce, 96 },
    { 0x62b270ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5270d5, 33 },
    { 0xf47270d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9270dc, 55 },
    { 0x6fe270dc, 46 },
    { 0x595270dd, 23 },
    { 0x558270df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93270eb, 52 },
    { 0x58e270eb, 90 },
    { 0x68f270eb, 33 },
    { 0, 0 },
    { 0x74c270ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b270f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8270f5, 33 },
    { 0x164270f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc270f9, 33 },
    { 0xc4c270f9, 43 },
    { 0xadd270fb, 17 },
    { 0, 0 },
    { 0x547270fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8327105, 95 },
    { 0xcc727105, 66 },
    { 0x47827107, 2 },
    { 0x1b027107, 34 },
    { 0xdf727105, 98 },
    { 0x16a2710a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf327111, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2327115, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c2711d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58827123, 61 },
    { 0xc0c27123, 95 },
    { 0x79927125, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a52712c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9627133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e27139, 66 },
    { 0x802713a, 52 },
    { 0x22327139, 61 },
    { 0x1982713c, 33 },
    { 0x5a72713d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1227140, 95 },
    { 0x60627141, 66 },
    { 0x2cb27142, 34 },
    { 0, 0 },
    { 0xf3c27144, 98 },
    { 0, 0 },
    { 0x4f127146, 34 },
    { 0, 0 },
    { 0x7a827148, 90 },
    { 0x3d527149, 87 },
    { 0x6c827148, 95 },
    { 0x6b42714b, 90 },
    { 0x4bd2714b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f27153, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a27156, 99 },
    { 0, 0 },
    { 0xb2527158, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e2715d, 80 },
    { 0x6d2715d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc927161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3972716a, 76 },
    { 0x7a22716b, 48 },
    { 0x69f2716c, 55 },
    { 0x2402716d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68027172, 61 },
    { 0, 0 },
    { 0x8be27174, 55 },
    { 0, 0 },
    { 0x47927176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532717a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd562717e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad27184, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b27188, 55 },
    { 0xc1a27189, 68 },
    { 0x7827189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf27191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b27195, 85 },
    { 0x1c27196, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe62719f, 93 },
    { 0, 0 },
    { 0x6ae271a1, 33 },
    { 0x8c1271a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e271a6, 90 },
    { 0x672271a7, 33 },
    { 0x2ed271a6, 91 },
    { 0, 0 },
    { 0x928271aa, 33 },
    { 0xe51271aa, 43 },
    { 0, 0 },
    { 0xf04271ad, 95 },
    { 0xfb3271ad, 91 },
    { 0xfc0271af, 98 },
    { 0x895271ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4271b6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c271ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749271bf, 76 },
    { 0, 0 },
    { 0xacb271c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a271c4, 95 },
    { 0, 0 },
    { 0xfb5271c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c271cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37271d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4271d3, 100 },
    { 0x7b8271d4, 98 },
    { 0xb99271d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f271dc, 33 },
    { 0x1d2271dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f271e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77271e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc271ee, 95 },
    { 0x4ff271ef, 33 },
    { 0, 0 },
    { 0x2ab271f1, 91 },
    { 0, 0 },
    { 0xf27271f3, 98 },
    { 0xc7e271f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632271f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08271fd, 81 },
    { 0x285271fe, 61 },
    { 0x58271fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f27207, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac62720e, 66 },
    { 0xa402720f, 55 },
    { 0x39b27210, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7227215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e22721c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a027222, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44827228, 52 },
    { 0, 0 },
    { 0x49c2722a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f2722d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77727230, 33 },
    { 0x8d127231, 34 },
    { 0x9c927232, 98 },
    { 0x11827233, 95 },
    { 0xdd327234, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29627237, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02723a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7232723d, 61 },
    { 0, 0 },
    { 0x29e2723f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e827243, 23 },
    { 0, 0 },
    { 0xd3827245, 95 },
    { 0x3c227246, 55 },
    { 0x38a27246, 98 },
    { 0, 0 },
    { 0x80427249, 61 },
    { 0xed2724a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb2724d, 100 },
    { 0, 0 },
    { 0x6382724f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e2725f, 98 },
    { 0, 0 },
    { 0x4d27261, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9427265, 90 },
    { 0xac327265, 33 },
    { 0x68a27267, 55 },
    { 0x82827268, 33 },
    { 0xd7027269, 67 },
    { 0x3da2726a, 61 },
    { 0x9662726b, 98 },
    { 0xaf2726c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7427276, 61 },
    { 0x1cd27276, 46 },
    { 0xcef27277, 33 },
    { 0x17627279, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582727e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e27286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba2728d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93627291, 33 },
    { 0, 0 },
    { 0xc427293, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd27296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6427299, 88 },
    { 0, 0 },
    { 0x7772729b, 88 },
    { 0, 0 },
    { 0xf1e2729d, 23 },
    { 0x14e2729d, 46 },
    { 0x5c72729e, 52 },
    { 0x36e272a0, 87 },
    { 0, 0 },
    { 0x2d1272a2, 61 },
    { 0, 0 },
    { 0xe07272a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e272a9, 67 },
    { 0xfa3272aa, 55 },
    { 0x85a272ab, 23 },
    { 0x86f272aa, 17 },
    { 0x979272ad, 33 },
    { 0x58272ab, 33 },
    { 0x27a272af, 33 },
    { 0x5c3272b0, 61 },
    { 0xd48272b1, 40 },
    { 0xe45272aa, 92 },
    { 0, 0 },
    { 0x5ad272b4, 90 },
    { 0xee3272b5, 95 },
    { 0, 0 },
    { 0x3f9272b7, 89 },
    { 0xadd272b7, 78 },
    { 0xc80272b9, 33 },
    { 0x69f272ba, 98 },
    { 0xfc9272ba, 70 },
    { 0xed5272bc, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab272bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7272c5, 74 },
    { 0xf40272c6, 95 },
    { 0, 0 },
    { 0xc38272c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501272d1, 98 },
    { 0x292272d2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x524272d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4272d9, 33 },
    { 0x935272da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f272e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669272e8, 77 },
    { 0xb23272e8, 61 },
    { 0, 0 },
    { 0x97d272eb, 90 },
    { 0xea6272ec, 33 },
    { 0x205272ed, 52 },
    { 0x2fd272ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5272f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10272f4, 98 },
    { 0x675272f5, 98 },
    { 0x226272f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd272fe, 61 },
    { 0x66272ff, 33 },
    { 0xfb27300, 67 },
    { 0x72327301, 91 },
    { 0xa5427302, 33 },
    { 0x59427300, 98 },
    { 0x85b27304, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43027307, 69 },
    { 0xd4b27307, 55 },
    { 0x68427309, 55 },
    { 0, 0 },
    { 0x67d2730b, 90 },
    { 0xe862730c, 33 },
    { 0x4f72730c, 43 },
    { 0x83c2730c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c27317, 67 },
    { 0xe6727318, 34 },
    { 0xa1227319, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d2731c, 87 },
    { 0x55d2731c, 52 },
    { 0xc3e2731c, 66 },
    { 0, 0 },
    { 0xfac27320, 85 },
    { 0x13827320, 95 },
    { 0x36127322, 95 },
    { 0x3d727323, 101 },
    { 0, 0 },
    { 0x2bd27325, 78 },
    { 0, 0 },
    { 0x44a27327, 33 },
    { 0x5ee27327, 23 },
    { 0x5d927329, 95 },
    { 0x9c02732a, 33 },
    { 0xab62732b, 34 },
    { 0x32127327, 97 },
    { 0, 0 },
    { 0x4722732e, 91 },
    { 0x1fb2732f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f527332, 91 },
    { 0, 0 },
    { 0x89027334, 65 },
    { 0, 0 },
    { 0xf8527336, 61 },
    { 0x5ca27337, 33 },
    { 0x13b27337, 75 },
    { 0xbfe27338, 43 },
    { 0x1052733a, 76 },
    { 0xb882733b, 93 },
    { 0x43e2733c, 92 },
    { 0x43d2733a, 95 },
    { 0xc382733e, 33 },
    { 0x91d2733e, 55 },
    { 0x9a227340, 76 },
    { 0xec027339, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8627344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5427347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332734b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf27354, 66 },
    { 0x51f27354, 91 },
    { 0xa6a27356, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f2735c, 66 },
    { 0xa2a2735c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba327360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60627363, 90 },
    { 0xd0c27364, 95 },
    { 0xf4b27363, 85 },
    { 0, 0 },
    { 0x4b427367, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a27370, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b27378, 95 },
    { 0xb0427379, 23 },
    { 0x7f42737a, 46 },
    { 0xfe52737b, 95 },
    { 0x6e2737c, 98 },
    { 0x3822737c, 61 },
    { 0, 0 },
    { 0xd102737f, 98 },
    { 0x78427380, 66 },
    { 0xd3b2737f, 67 },
    { 0, 0 },
    { 0xba127383, 98 },
    { 0x80a27384, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c27387, 61 },
    { 0x8e27388, 95 },
    { 0, 0 },
    { 0x2512738a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa2738e, 98 },
    { 0x2292738f, 61 },
    { 0, 0 },
    { 0x9127391, 91 },
    { 0x71d27392, 61 },
    { 0, 0 },
    { 0x4d427394, 33 },
    { 0x16d27394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9882739a, 91 },
    { 0xbb22739b, 61 },
    { 0xc7d2739c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212273a1, 61 },
    { 0x4bc273a2, 46 },
    { 0x461273a3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c273a7, 95 },
    { 0, 0 },
    { 0x744273a9, 61 },
    { 0, 0 },
    { 0x5d1273ab, 75 },
    { 0, 0 },
    { 0x52a273ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7273b1, 23 },
    { 0xd91273b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82273b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23273bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x610273be, 95 },
    { 0xc13273bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd273c5, 90 },
    { 0x9ac273c5, 76 },
    { 0xdfe273c7, 90 },
    { 0, 0 },
    { 0x287273c9, 33 },
    { 0x11273ca, 90 },
    { 0xaa0273c9, 66 },
    { 0xbcb273cc, 100 },
    { 0, 0 },
    { 0x45a273ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf273d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f273d7, 61 },
    { 0x77f273d7, 67 },
    { 0, 0 },
    { 0x47f273da, 98 },
    { 0x3a6273da, 98 },
    { 0x3ae273dc, 93 },
    { 0x113273da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x632273e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9273e4, 95 },
    { 0x5c9273e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc273e9, 90 },
    { 0x219273e9, 38 },
    { 0xa46273ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c273ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32273f1, 68 },
    { 0xe41273f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9273f6, 23 },
    { 0x57e273f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b273fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd527401, 90 },
    { 0xc0f27402, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af27405, 76 },
    { 0xa4727406, 43 },
    { 0, 0 },
    { 0x21927408, 46 },
    { 0, 0 },
    { 0x30c2740a, 17 },
    { 0x3442740a, 98 },
    { 0x1d2740b, 90 },
    { 0xdf52740d, 34 },
    { 0x7fd2740c, 93 },
    { 0x3c32740a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9527418, 61 },
    { 0, 0 },
    { 0x7342741a, 33 },
    { 0xe952741b, 87 },
    { 0x9432741c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a27421, 93 },
    { 0x2c027422, 46 },
    { 0x78427422, 75 },
    { 0x7fe27424, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8627429, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7762742e, 95 },
    { 0xc052742f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6327434, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b27438, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f02743d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b827441, 61 },
    { 0x59727442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0627445, 90 },
    { 0, 0 },
    { 0x81527447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c2744b, 61 },
    { 0x4c92744b, 99 },
    { 0x7332744d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f27454, 85 },
    { 0x98527455, 33 },
    { 0x6627456, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e32745f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8227469, 33 },
    { 0xd6927469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9632746e, 95 },
    { 0x4922746f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f27472, 93 },
    { 0xd1327472, 23 },
    { 0x6a727472, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a27477, 92 },
    { 0x29f27478, 43 },
    { 0, 0 },
    { 0x38e2747a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f12747d, 33 },
    { 0xf792747e, 61 },
    { 0xa9f2747f, 87 },
    { 0xa3a2747d, 90 },
    { 0x90e27481, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be27486, 23 },
    { 0x64c27487, 94 },
    { 0x3ae27488, 99 },
    { 0x9127486, 61 },
    { 0xd3b2748a, 19 },
    { 0x73f27486, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c427496, 95 },
    { 0x39227497, 77 },
    { 0, 0 },
    { 0x45f27499, 61 },
    { 0x9452749a, 98 },
    { 0x6172749a, 61 },
    { 0x4f2749c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc274a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe274a8, 61 },
    { 0, 0 },
    { 0xadd274aa, 95 },
    { 0x80d274ab, 34 },
    { 0x17a274ac, 90 },
    { 0xb5274aa, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72274b0, 45 },
    { 0, 0 },
    { 0x7f4274b2, 95 },
    { 0, 0 },
    { 0x1eb274b4, 61 },
    { 0x6b2274b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb274bd, 76 },
    { 0, 0 },
    { 0x929274bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6274c3, 95 },
    { 0, 0 },
    { 0xf9f274c5, 61 },
    { 0x1d6274c6, 55 },
    { 0x133274c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac274cb, 85 },
    { 0x925274cb, 76 },
    { 0, 0 },
    { 0xd8a274ce, 61 },
    { 0x9bf274cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b274d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc274df, 23 },
    { 0x50a274e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x615274e3, 33 },
    { 0x5fe274e4, 61 },
    { 0x18f274e3, 61 },
    { 0x7b0274e5, 95 },
    { 0, 0 },
    { 0xa14274e8, 77 },
    { 0, 0 },
    { 0xe0c274ea, 90 },
    { 0, 0 },
    { 0xd63274ec, 68 },
    { 0, 0 },
    { 0x995274ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355274f4, 91 },
    { 0x7e0274f5, 17 },
    { 0, 0 },
    { 0x45b274f7, 33 },
    { 0x393274f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec27502, 33 },
    { 0xbf927502, 95 },
    { 0x48027504, 93 },
    { 0x73d27505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f27508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1427513, 91 },
    { 0x40c27514, 23 },
    { 0x72d27515, 61 },
    { 0x21b27516, 55 },
    { 0, 0 },
    { 0x24527518, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a2751b, 101 },
    { 0x61c2751c, 85 },
    { 0x7872751b, 15 },
    { 0xf602751e, 44 },
    { 0x3192751e, 43 },
    { 0x68627520, 33 },
    { 0x59f27520, 61 },
    { 0xce427522, 61 },
    { 0x7a927523, 33 },
    { 0x67a2751f, 61 },
    { 0x35027525, 95 },
    { 0x7c227525, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x53327529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db2752d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc27531, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c27536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b42753b, 98 },
    { 0x8462753c, 55 },
    { 0xdbc2753d, 78 },
    { 0xee42753e, 90 },
    { 0, 0 },
    { 0xf4527540, 23 },
    { 0x3e427541, 76 },
    { 0x58c27540, 90 },
    { 0xbd727542, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af27546, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c427549, 61 },
    { 0x4342754a, 84 },
    { 0xea2754b, 90 },
    { 0xc292754c, 33 },
    { 0xe4f2754d, 78 },
    { 0xf7c2754e, 90 },
    { 0xd4f2754d, 46 },
    { 0, 0 },
    { 0x9fc27551, 55 },
    { 0, 0 },
    { 0x96027553, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff82755c, 61 },
    { 0, 0 },
    { 0x9b42755e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe27561, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f27567, 33 },
    { 0x38f27568, 90 },
    { 0, 0 },
    { 0x95d2756a, 61 },
    { 0, 0 },
    { 0x67d2756c, 23 },
    { 0xcbc2756d, 90 },
    { 0xa402756d, 23 },
    { 0, 0 },
    { 0xc1627570, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e427573, 98 },
    { 0xa1027574, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3927577, 61 },
    { 0x80e27578, 34 },
    { 0xb4427577, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd2757e, 95 },
    { 0xbd82757f, 81 },
    { 0xb782757e, 76 },
    { 0x60d27580, 52 },
    { 0xa2627582, 66 },
    { 0x1b32757f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b227589, 33 },
    { 0x5352758a, 61 },
    { 0xb332758a, 55 },
    { 0x3ca2758c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91527590, 23 },
    { 0, 0 },
    { 0xb6c27592, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee127596, 76 },
    { 0xe5827597, 23 },
    { 0xf2d27596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5275a0, 90 },
    { 0x2ea275a1, 61 },
    { 0, 0 },
    { 0x657275a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494275a9, 33 },
    { 0xccf275aa, 52 },
    { 0x4c0275ab, 55 },
    { 0xa05275ac, 98 },
    { 0xa41275ab, 17 },
    { 0x590275ad, 61 },
    { 0x498275af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f275b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95275bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d275bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4275c2, 33 },
    { 0, 0 },
    { 0x72a275c4, 100 },
    { 0xb8275c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x467275c8, 98 },
    { 0xb2d275c8, 33 },
    { 0xeb2275ca, 98 },
    { 0xbd5275cb, 68 },
    { 0x710275cb, 98 },
    { 0xc2f275cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77275d1, 99 },
    { 0x9a9275d2, 66 },
    { 0xe8c275d3, 61 },
    { 0x2e4275d3, 76 },
    { 0x512275d4, 46 },
    { 0x14f275d1, 99 },
    { 0x9dc275d7, 61 },
    { 0x422275d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be275e0, 91 },
    { 0x531275e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b275e7, 33 },
    { 0xeba275e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3275ef, 33 },
    { 0xdcd275ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x260275f3, 90 },
    { 0xd37275f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765275f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74275fb, 88 },
    { 0x25e275fb, 90 },
    { 0x850275fb, 76 },
    { 0xccc275fc, 66 },
    { 0xba4275ff, 61 },
    { 0x509275fd, 90 },
    { 0x81c275fd, 61 },
    { 0x67a275fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627608, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d2760b, 91 },
    { 0, 0 },
    { 0xe082760d, 43 },
    { 0x12f2760e, 33 },
    { 0x95f2760d, 90 },
    { 0, 0 },
    { 0x3da27611, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b27614, 61 },
    { 0xac727615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83827618, 76 },
    { 0, 0 },
    { 0xe622761a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d2761d, 67 },
    { 0x3542761d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5727627, 90 },
    { 0xcd027628, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda32762f, 46 },
    { 0x18e27630, 43 },
    { 0x32e27630, 61 },
    { 0x2d727630, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8627636, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9127639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a27642, 90 },
    { 0x90327643, 52 },
    { 0xdaf27643, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f27647, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9712764a, 81 },
    { 0, 0 },
    { 0x8992764c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95127656, 66 },
    { 0, 0 },
    { 0x53a27658, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f627661, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0127666, 33 },
    { 0x14427667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912766c, 99 },
    { 0x69b2766c, 100 },
    { 0x60b2766e, 90 },
    { 0xc132766e, 95 },
    { 0x18227670, 95 },
    { 0, 0 },
    { 0x9b127672, 90 },
    { 0xc0127672, 93 },
    { 0xa1a27674, 98 },
    { 0x6827673, 98 },
    { 0, 0 },
    { 0xdf827677, 95 },
    { 0, 0 },
    { 0x2d027679, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c2767c, 23 },
    { 0, 0 },
    { 0x8fe2767e, 76 },
    { 0, 0 },
    { 0x4f27680, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2527683, 90 },
    { 0x51927683, 69 },
    { 0xeb27685, 87 },
    { 0x94527683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a727689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9142768c, 76 },
    { 0x13b2768c, 55 },
    { 0, 0 },
    { 0x592768f, 83 },
    { 0xf0b27690, 33 },
    { 0, 0 },
    { 0x37927692, 23 },
    { 0, 0 },
    { 0x47527694, 98 },
    { 0x4d27694, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa427699, 98 },
    { 0x6582769a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b32769e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf276a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf276a9, 23 },
    { 0, 0 },
    { 0xbdf276ab, 85 },
    { 0x1dc276ac, 77 },
    { 0xcb0276ad, 61 },
    { 0xafe276ad, 95 },
    { 0x93c276ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3276b3, 34 },
    { 0, 0 },
    { 0x462276b5, 90 },
    { 0, 0 },
    { 0x869276b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744276bb, 77 },
    { 0x3a0276bc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39276c2, 33 },
    { 0, 0 },
    { 0x9b9276c4, 93 },
    { 0x161276c5, 76 },
    { 0xa7d276c5, 55 },
    { 0, 0 },
    { 0x3a276c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58276cc, 33 },
    { 0xab1276cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda276d5, 43 },
    { 0xb3d276d6, 91 },
    { 0x7ba276d7, 61 },
    { 0xa33276d8, 87 },
    { 0xa5276d7, 95 },
    { 0x260276da, 33 },
    { 0x980276da, 90 },
    { 0, 0 },
    { 0x89f276dd, 66 },
    { 0xe29276de, 90 },
    { 0x1df276df, 43 },
    { 0xf2b276e0, 95 },
    { 0, 0 },
    { 0xc67276e2, 87 },
    { 0, 0 },
    { 0x7ec276e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b276ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7276f2, 99 },
    { 0, 0 },
    { 0x955276f4, 55 },
    { 0x753276f5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f276f8, 90 },
    { 0x5fe276f9, 34 },
    { 0x380276fa, 17 },
    { 0xcdf276fb, 61 },
    { 0, 0 },
    { 0xb4276fd, 17 },
    { 0, 0 },
    { 0x862276ff, 90 },
    { 0, 0 },
    { 0x5ef27701, 85 },
    { 0, 0 },
    { 0xf3627703, 76 },
    { 0x66a27703, 75 },
    { 0, 0 },
    { 0xace27706, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc292770c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80927711, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf27714, 75 },
    { 0x5e327715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a2771b, 34 },
    { 0, 0 },
    { 0xab82771d, 61 },
    { 0x4a92771e, 90 },
    { 0x6812771f, 61 },
    { 0, 0 },
    { 0x71f27721, 33 },
    { 0x9f127721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f827727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca22772a, 67 },
    { 0x9b52772a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11127730, 76 },
    { 0xd6227731, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f27734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9982773a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92773d, 66 },
    { 0xf462773e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f27744, 100 },
    { 0, 0 },
    { 0xf5727746, 88 },
    { 0xd0727747, 91 },
    { 0x31227747, 87 },
    { 0x8027748, 61 },
    { 0x60f2774a, 97 },
    { 0xb902774b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a2774e, 66 },
    { 0x8482774e, 75 },
    { 0, 0 },
    { 0xf1427751, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2227754, 61 },
    { 0x8a427754, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d27758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f02775d, 90 },
    { 0, 0 },
    { 0xd5d2775f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1927769, 93 },
    { 0xc882776a, 34 },
    { 0x44b2776b, 66 },
    { 0x95f2776a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce427772, 61 },
    { 0, 0 },
    { 0x4f827774, 98 },
    { 0x14027775, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752777a, 93 },
    { 0, 0 },
    { 0xbb72777c, 33 },
    { 0x3502777c, 23 },
    { 0x87f2777d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d27781, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb27784, 17 },
    { 0x96e27785, 85 },
    { 0x9b727786, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac27789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072778c, 61 },
    { 0xf752778c, 33 },
    { 0, 0 },
    { 0xf462778f, 23 },
    { 0xbf72778f, 52 },
    { 0x9ab27791, 61 },
    { 0, 0 },
    { 0x81527793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d27796, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b27799, 55 },
    { 0xcd727799, 90 },
    { 0, 0 },
    { 0x84a2779c, 67 },
    { 0, 0 },
    { 0x20b2779e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83277a5, 61 },
    { 0, 0 },
    { 0xaf7277a7, 76 },
    { 0xd32277a7, 33 },
    { 0xef7277a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693277b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e277b8, 52 },
    { 0x82e277b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0277c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106277c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d277c9, 90 },
    { 0x39d277ca, 77 },
    { 0xee0277cb, 23 },
    { 0xdf9277cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6277cf, 95 },
    { 0x29277d0, 61 },
    { 0xe85277d1, 3 },
    { 0x646277d0, 99 },
    { 0x927277d3, 90 },
    { 0x8c277d4, 52 },
    { 0x1bb277d4, 34 },
    { 0xd1d277d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9277d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08277de, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8277e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6277e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413277ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08277f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92277fe, 61 },
    { 0, 0 },
    { 0x94227800, 90 },
    { 0x32d27801, 93 },
    { 0x78427802, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe427809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72780f, 90 },
    { 0x55d27810, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d2781a, 24 },
    { 0x9112781b, 33 },
    { 0, 0 },
    { 0x6492781d, 76 },
    { 0x40f2781e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d27822, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5427825, 33 },
    { 0, 0 },
    { 0x9d727827, 85 },
    { 0x57d27828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a62782c, 61 },
    { 0x5342782d, 36 },
    { 0x10d2782e, 95 },
    { 0xb782782e, 61 },
    { 0x87a2782d, 43 },
    { 0x3a72782e, 33 },
    { 0x87127832, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3727835, 98 },
    { 0, 0 },
    { 0x98827837, 76 },
    { 0x2ba27837, 91 },
    { 0x6dc27837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2783e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f27843, 61 },
    { 0x53727843, 34 },
    { 0, 0 },
    { 0xba27846, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f27849, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6127850, 33 },
    { 0x1d927850, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37627855, 33 },
    { 0, 0 },
    { 0x9b827857, 76 },
    { 0x7ef27857, 61 },
    { 0x93927859, 73 },
    { 0x47027859, 90 },
    { 0x66a2785a, 23 },
    { 0x5bd2785c, 76 },
    { 0x39727859, 98 },
    { 0x66d2785e, 23 },
    { 0xd512785f, 90 },
    { 0x9d72785f, 61 },
    { 0x53b27861, 90 },
    { 0x42927861, 67 },
    { 0, 0 },
    { 0x2b227864, 75 },
    { 0, 0 },
    { 0xc6d27866, 55 },
    { 0x8427867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x482786a, 52 },
    { 0xf802786a, 55 },
    { 0, 0 },
    { 0xc032786d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e927873, 98 },
    { 0xa1527874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88427877, 23 },
    { 0x9e627877, 55 },
    { 0x8b327879, 91 },
    { 0xd192787a, 64 },
    { 0, 0 },
    { 0xc792787c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec27882, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1327888, 33 },
    { 0x7fb27889, 55 },
    { 0xc362788a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb92788e, 90 },
    { 0x9c02788e, 90 },
    { 0x2c27890, 61 },
    { 0x47c27890, 76 },
    { 0xb027892, 23 },
    { 0xa3a27892, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb322789c, 95 },
    { 0x80a2789c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266278a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6278a8, 94 },
    { 0, 0 },
    { 0xda4278aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2278ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8278b7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8278bd, 85 },
    { 0xe29278be, 67 },
    { 0, 0 },
    { 0x6e278c0, 93 },
    { 0, 0 },
    { 0x3ef278c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228278c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650278ce, 55 },
    { 0x25a278ce, 68 },
    { 0xbfc278cf, 61 },
    { 0xd53278d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0278d6, 33 },
    { 0, 0 },
    { 0xc5a278d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c278dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6278e0, 90 },
    { 0x6d9278e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8278e5, 55 },
    { 0x448278e6, 95 },
    { 0x366278e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10278ea, 90 },
    { 0xbc8278eb, 85 },
    { 0x9c1278eb, 76 },
    { 0x233278ed, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38278f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41278f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f278f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00278fa, 98 },
    { 0x9ae278fb, 77 },
    { 0, 0 },
    { 0x44c278fd, 90 },
    { 0xb95278fe, 90 },
    { 0, 0 },
    { 0x83227900, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7982790a, 49 },
    { 0x81e2790b, 61 },
    { 0xf0b2790b, 98 },
    { 0x3892790b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf27910, 23 },
    { 0, 0 },
    { 0xf6527912, 33 },
    { 0x26e27913, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb227916, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea52791d, 90 },
    { 0, 0 },
    { 0x1952791f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77127926, 33 },
    { 0xb5427926, 78 },
    { 0xca627928, 76 },
    { 0x3f327926, 33 },
    { 0xf6c2792a, 61 },
    { 0xef82792b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92027930, 98 },
    { 0, 0 },
    { 0x7c27932, 95 },
    { 0x5f927933, 36 },
    { 0xb0327933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5327937, 90 },
    { 0xa8e27938, 61 },
    { 0x1b427939, 93 },
    { 0xc7b2793a, 61 },
    { 0, 0 },
    { 0xf082793c, 55 },
    { 0x9b12793d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28127949, 76 },
    { 0xf752794a, 23 },
    { 0xda12794b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f27951, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1027956, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d2795b, 90 },
    { 0xa1f2795c, 23 },
    { 0, 0 },
    { 0x57f2795e, 76 },
    { 0, 0 },
    { 0x87527960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b627964, 76 },
    { 0x43927964, 83 },
    { 0x6627964, 90 },
    { 0x9ce27965, 33 },
    { 0xfe227964, 61 },
    { 0, 0 },
    { 0x34d2796a, 66 },
    { 0xdee2796b, 43 },
    { 0x5162796b, 98 },
    { 0, 0 },
    { 0x5792796e, 93 },
    { 0x88d2796e, 90 },
    { 0, 0 },
    { 0x50227971, 35 },
    { 0, 0 },
    { 0xb5027973, 77 },
    { 0, 0 },
    { 0x93f27975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4512797b, 90 },
    { 0x312797c, 23 },
    { 0xd602797d, 55 },
    { 0xc512797e, 90 },
    { 0x9ac2797f, 66 },
    { 0x9bb27980, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2827984, 90 },
    { 0, 0 },
    { 0x36a27986, 87 },
    { 0x8ec27986, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b22798c, 64 },
    { 0, 0 },
    { 0xd042798e, 95 },
    { 0xa392798e, 90 },
    { 0x76627990, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1127995, 33 },
    { 0xa9727996, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2527999, 33 },
    { 0x1727999, 91 },
    { 0x7122799b, 100 },
    { 0xaa32799c, 61 },
    { 0xc332799c, 52 },
    { 0x8fe2799e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162279a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc279a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x176279ab, 17 },
    { 0, 0 },
    { 0x3ce279ad, 91 },
    { 0x617279ae, 90 },
    { 0, 0 },
    { 0x197279b0, 61 },
    { 0xa43279b0, 90 },
    { 0x6ce279b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b279b7, 76 },
    { 0, 0 },
    { 0x6c279b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0279be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98279c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4279c9, 65 },
    { 0x461279ca, 33 },
    { 0xa14279c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b279d2, 55 },
    { 0xf82279d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a279d9, 61 },
    { 0x400279d9, 55 },
    { 0, 0 },
    { 0xf58279dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2279df, 33 },
    { 0x9c5279df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71279ea, 61 },
    { 0x76a279eb, 90 },
    { 0x710279ec, 98 },
    { 0xdba279ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d279f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x229279f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3279fa, 76 },
    { 0x193279fa, 34 },
    { 0x4a4279fb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85527a02, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c27a05, 68 },
    { 0, 0 },
    { 0xc7a27a07, 80 },
    { 0xc9e27a07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6927a0d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa27a13, 34 },
    { 0, 0 },
    { 0x22127a15, 61 },
    { 0, 0 },
    { 0xa5627a17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e827a1e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7227a26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27a29, 90 },
    { 0x2b927a2a, 85 },
    { 0x2fc27a2a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec927a30, 92 },
    { 0, 0 },
    { 0xbea27a32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3327a3c, 66 },
    { 0, 0 },
    { 0x89727a3e, 98 },
    { 0, 0 },
    { 0x17d27a40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b227a44, 68 },
    { 0x35127a44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf427a4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce27a4d, 95 },
    { 0, 0 },
    { 0x24827a4f, 90 },
    { 0x7b227a50, 33 },
    { 0x8aa27a50, 23 },
    { 0, 0 },
    { 0xd8627a53, 98 },
    { 0, 0 },
    { 0xbad27a55, 78 },
    { 0xa6e27a56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45227a59, 76 },
    { 0x2f127a5a, 46 },
    { 0, 0 },
    { 0xd4527a5c, 98 },
    { 0, 0 },
    { 0x2bd27a5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83327a61, 33 },
    { 0xed527a61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c27a68, 43 },
    { 0xecd27a69, 33 },
    { 0x5fb27a69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b227a6d, 76 },
    { 0, 0 },
    { 0xd8027a6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e27a74, 33 },
    { 0x8ca27a74, 78 },
    { 0x70e27a76, 23 },
    { 0x1f627a77, 77 },
    { 0x9d627a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a27a7e, 90 },
    { 0xab527a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a527a88, 77 },
    { 0x7a827a89, 61 },
    { 0x9f427a88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc27a8e, 95 },
    { 0xd8c27a8e, 33 },
    { 0x33327a90, 98 },
    { 0x44627a8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f27a95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23927a98, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d27a9e, 61 },
    { 0xf6727a9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78727aa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d27aab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d27ab0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd927ac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d427ac5, 55 },
    { 0xf6a27ac6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc27ac9, 90 },
    { 0x95927aca, 84 },
    { 0xff27ac9, 33 },
    { 0xdf727acc, 98 },
    { 0x45e27acd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c427ad0, 95 },
    { 0, 0 },
    { 0x93c27ad2, 43 },
    { 0xd0d27ad3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d327ad6, 61 },
    { 0, 0 },
    { 0x23927ad8, 78 },
    { 0x26727ad8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1827ade, 33 },
    { 0, 0 },
    { 0x3327ae0, 98 },
    { 0, 0 },
    { 0xf1527ae2, 66 },
    { 0x25927ae2, 80 },
    { 0x38f27ae2, 76 },
    { 0xea827ae5, 34 },
    { 0xf9827ae6, 23 },
    { 0xa8527ae3, 76 },
    { 0, 0 },
    { 0xd7a27ae9, 90 },
    { 0x73927ae9, 23 },
    { 0, 0 },
    { 0xee27aec, 34 },
    { 0, 0 },
    { 0x74027aee, 90 },
    { 0x14627aee, 90 },
    { 0xc5827aee, 97 },
    { 0x2be27aee, 38 },
    { 0, 0 },
    { 0xe9f27af3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2627b02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74727b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14827b0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1627b0f, 61 },
    { 0x11627b10, 95 },
    { 0x85f27b10, 76 },
    { 0x34527b0f, 95 },
    { 0, 0 },
    { 0x6827b14, 33 },
    { 0x7b627b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb227b20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0727b23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda227b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b27b35, 40 },
    { 0, 0 },
    { 0x31a27b37, 95 },
    { 0xaad27b38, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6527b3d, 33 },
    { 0, 0 },
    { 0xf8827b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97527b42, 40 },
    { 0, 0 },
    { 0xab627b44, 67 },
    { 0, 0 },
    { 0xe8927b46, 98 },
    { 0x7d027b47, 93 },
    { 0x6c227b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b527b4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb27b54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb27b57, 32 },
    { 0, 0 },
    { 0xba527b59, 95 },
    { 0, 0 },
    { 0x43127b5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8827b5f, 78 },
    { 0x21627b60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c27b66, 84 },
    { 0xf7a27b66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff27b6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47327b6e, 90 },
    { 0xc2c27b6f, 93 },
    { 0x6c727b6f, 95 },
    { 0, 0 },
    { 0xba227b72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8827b75, 95 },
    { 0x49127b76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf927b7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a327b84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d27b87, 90 },
    { 0, 0 },
    { 0x99927b89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b27b8e, 87 },
    { 0, 0 },
    { 0x33f27b90, 90 },
    { 0x83027b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9027b97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1827ba0, 91 },
    { 0, 0 },
    { 0x55127ba2, 95 },
    { 0xd0227ba3, 41 },
    { 0x88627ba3, 33 },
    { 0xe1027ba5, 61 },
    { 0xb8127ba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca027bab, 78 },
    { 0x37b27bab, 33 },
    { 0, 0 },
    { 0xa9a27bae, 85 },
    { 0, 0 },
    { 0xdcb27bb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e27bb3, 61 },
    { 0, 0 },
    { 0x92c27bb5, 91 },
    { 0xe3527bb6, 80 },
    { 0x89d27bb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x83527bba, 52 },
    { 0, 0 },
    { 0x93527bbc, 55 },
    { 0, 0 },
    { 0x10b27bbe, 33 },
    { 0xf4827bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8627bcc, 61 },
    { 0x627bcd, 99 },
    { 0x65827bce, 61 },
    { 0x28027bce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa27bd2, 75 },
    { 0x2827bd2, 19 },
    { 0, 0 },
    { 0x9cb27bd5, 23 },
    { 0x80327bd6, 98 },
    { 0xb6227bd7, 90 },
    { 0, 0 },
    { 0xe6c27bd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd427bde, 62 },
    { 0x32e27bde, 40 },
    { 0, 0 },
    { 0x6c827be1, 78 },
    { 0x11f27be2, 90 },
    { 0x9c27be3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727be7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c827bef, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6527bf2, 92 },
    { 0x33327bf3, 90 },
    { 0x17a27bf4, 81 },
    { 0xf1c27bf2, 46 },
    { 0xa3027bf6, 46 },
    { 0x7a827bf7, 46 },
    { 0, 0 },
    { 0x66b27bf9, 84 },
    { 0x5cd27bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde27bfe, 93 },
    { 0, 0 },
    { 0xadd27c00, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1627c0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f327c0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e527c13, 90 },
    { 0, 0 },
    { 0x56927c15, 23 },
    { 0x20427c15, 76 },
    { 0xc4327c17, 77 },
    { 0x6b027c17, 95 },
    { 0xa0427c19, 95 },
    { 0x5a327c1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87127c1e, 97 },
    { 0, 0 },
    { 0xee327c20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b527c23, 76 },
    { 0, 0 },
    { 0xa9c27c25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac927c28, 91 },
    { 0x26b27c29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34627c2d, 100 },
    { 0xb8727c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae327c31, 66 },
    { 0xc6527c32, 77 },
    { 0, 0 },
    { 0x6d527c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb27c3c, 55 },
    { 0x3b927c3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52127c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d27c45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab027c49, 76 },
    { 0x6e827c49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b27c4d, 33 },
    { 0, 0 },
    { 0xb3727c4f, 23 },
    { 0x47827c50, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96627c55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d27c5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd27c64, 90 },
    { 0xec227c64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96927c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a27c6b, 23 },
    { 0x65c27c6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d27c70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9327c73, 33 },
    { 0, 0 },
    { 0x89a27c75, 90 },
    { 0, 0 },
    { 0x2a327c77, 61 },
    { 0, 0 },
    { 0x88527c79, 5 },
    { 0x53327c7a, 77 },
    { 0xc7327c7a, 33 },
    { 0xb8d27c7c, 90 },
    { 0x6ee27c7c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x13527c80, 92 },
    { 0x7727c81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae727c85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd27c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d27c8d, 95 },
    { 0, 0 },
    { 0xd9927c8f, 98 },
    { 0x11527c90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c27c93, 98 },
    { 0, 0 },
    { 0x74e27c95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18527c9a, 93 },
    { 0x2b27c9a, 84 },
    { 0xe8e27c9c, 66 },
    { 0x3dc27c9a, 90 },
    { 0, 0 },
    { 0x1ec27c9f, 85 },
    { 0xf5127c9f, 99 },
    { 0x3aa27ca1, 95 },
    { 0x13827ca2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd27ca5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb27ca8, 100 },
    { 0, 0 },
    { 0x94027caa, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f27cad, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9327cb2, 90 },
    { 0x27cb3, 90 },
    { 0xcb627cb3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed27cb8, 76 },
    { 0x7e027cb9, 23 },
    { 0xd5f27cba, 33 },
    { 0xe7e27cba, 61 },
    { 0x54127cb9, 55 },
    { 0x39727cb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc127cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c27cc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a627cc7, 76 },
    { 0x9c127cc8, 90 },
    { 0x35127cc8, 24 },
    { 0x8bb27cca, 23 },
    { 0x1d227ccb, 33 },
    { 0x66c27ccc, 55 },
    { 0x5d127ccd, 46 },
    { 0x89e27cce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c27cd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59227cda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3727ce0, 95 },
    { 0x21227ce1, 33 },
    { 0x3f627ce2, 99 },
    { 0x51227ce0, 76 },
    { 0x2b727ce4, 90 },
    { 0x4f927ce4, 84 },
    { 0x89a27ce0, 80 },
    { 0x4f927ce6, 90 },
    { 0xc6327ce8, 33 },
    { 0, 0 },
    { 0xe0a27cea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac227ced, 98 },
    { 0x54527ced, 90 },
    { 0x5e727ced, 77 },
    { 0, 0 },
    { 0x23027cf1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf727cf4, 95 },
    { 0xa0f27cf5, 93 },
    { 0x4a227cf6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb427cfa, 77 },
    { 0x89e27cfb, 95 },
    { 0xac927cfc, 98 },
    { 0, 0 },
    { 0xddd27cfe, 52 },
    { 0xa9927cff, 61 },
    { 0x99e27cff, 90 },
    { 0, 0 },
    { 0x64127d02, 95 },
    { 0xa5327d03, 90 },
    { 0xb3327d04, 43 },
    { 0x99627d05, 90 },
    { 0x40a27d04, 81 },
    { 0x9cf27d07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90127d0e, 61 },
    { 0, 0 },
    { 0xe2327d10, 98 },
    { 0, 0 },
    { 0xc5327d12, 61 },
    { 0x84327d13, 33 },
    { 0, 0 },
    { 0x7b027d15, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf27d19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e427d1c, 98 },
    { 0xc9127d1d, 33 },
    { 0, 0 },
    { 0x54827d1f, 61 },
    { 0xdff27d1f, 33 },
    { 0x4d27d1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe527d25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc27d29, 23 },
    { 0x5e827d2a, 36 },
    { 0x7fb27d2a, 61 },
    { 0xa7027d29, 58 },
    { 0x71e27d2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f27d32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16427d38, 33 },
    { 0xa5e27d38, 33 },
    { 0x9fc27d38, 95 },
    { 0xe3c27d3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb327d3e, 76 },
    { 0xe3627d3e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3727d43, 68 },
    { 0xc2b27d44, 95 },
    { 0xb5827d45, 34 },
    { 0xec627d43, 98 },
    { 0x78d27d46, 90 },
    { 0, 0 },
    { 0xbff27d49, 61 },
    { 0xc6727d4a, 100 },
    { 0xd0a27d49, 65 },
    { 0x4e727d4c, 23 },
    { 0xed927d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b27d52, 90 },
    { 0, 0 },
    { 0x5fd27d54, 34 },
    { 0x19627d55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff27d59, 61 },
    { 0xf1427d5a, 76 },
    { 0, 0 },
    { 0x84a27d5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f27d62, 100 },
    { 0x40c27d62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a927d66, 90 },
    { 0x72827d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe127d6e, 61 },
    { 0xe1027d6f, 90 },
    { 0x76227d70, 93 },
    { 0x7ed27d6f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f27d78, 90 },
    { 0xa7d27d78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d27d81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62527d85, 69 },
    { 0x8e027d85, 55 },
    { 0xf1827d86, 23 },
    { 0x37227d86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b27d8b, 61 },
    { 0xab527d8b, 93 },
    { 0x33727d8d, 92 },
    { 0x6eb27d8e, 23 },
    { 0xaaf27d8f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b727d92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d027d9b, 61 },
    { 0x50127d9c, 87 },
    { 0x2bf27d9c, 33 },
    { 0x1e827d9d, 17 },
    { 0x3a427d9e, 34 },
    { 0x93127da0, 55 },
    { 0xcbf27d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50127da6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d27dac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1027db0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0327db5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7627db9, 75 },
    { 0, 0 },
    { 0xbe727dbb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21327dbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e27dc2, 34 },
    { 0, 0 },
    { 0xfbd27dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e27dce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a827dd1, 33 },
    { 0, 0 },
    { 0xf4527dd3, 90 },
    { 0, 0 },
    { 0xfc127dd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3027dd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2427ddc, 90 },
    { 0x2bc27ddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30027de0, 84 },
    { 0x37427de1, 78 },
    { 0xfd527de0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83427deb, 99 },
    { 0x31227dec, 98 },
    { 0x28427ded, 52 },
    { 0x48327dec, 90 },
    { 0, 0 },
    { 0xb0e27df0, 61 },
    { 0x66c27df1, 46 },
    { 0, 0 },
    { 0xbbf27df3, 17 },
    { 0, 0 },
    { 0x3f27df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de27dfa, 91 },
    { 0, 0 },
    { 0x27127dfc, 23 },
    { 0x7a427dfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5727e07, 61 },
    { 0xe7627e08, 98 },
    { 0, 0 },
    { 0x68d27e0a, 61 },
    { 0, 0 },
    { 0x47027e0c, 87 },
    { 0x64e27e0c, 33 },
    { 0xadd27e0c, 61 },
    { 0, 0 },
    { 0x50b27e10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c27e15, 78 },
    { 0, 0 },
    { 0x1d227e17, 23 },
    { 0x48e27e18, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b27e1d, 95 },
    { 0xe6a27e1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x10827e21, 97 },
    { 0xb5827e21, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac27e29, 61 },
    { 0, 0 },
    { 0x6eb27e2b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b227e31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a27e34, 98 },
    { 0, 0 },
    { 0x42227e36, 61 },
    { 0x70d27e37, 91 },
    { 0xa5727e37, 96 },
    { 0x75e27e38, 95 },
    { 0x29727e3a, 90 },
    { 0, 0 },
    { 0x6ed27e3c, 46 },
    { 0x6b327e3d, 66 },
    { 0x7c27e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b27e43, 88 },
    { 0, 0 },
    { 0x99527e45, 33 },
    { 0x87527e45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f927e4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7027e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67227e51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d27e54, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2727e57, 33 },
    { 0x85a27e57, 84 },
    { 0xfda27e59, 99 },
    { 0x9ce27e59, 96 },
    { 0xfed27e5b, 43 },
    { 0, 0 },
    { 0xfbb27e5d, 34 },
    { 0x24027e5e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c627e64, 98 },
    { 0x4fc27e65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98827e69, 43 },
    { 0x87c27e6a, 98 },
    { 0xa6827e6b, 33 },
    { 0x7927e6b, 98 },
    { 0xa0b27e6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f27e71, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f27e77, 90 },
    { 0xb4727e78, 66 },
    { 0x4e027e77, 46 },
    { 0x20227e7a, 98 },
    { 0xf6927e7b, 90 },
    { 0xdfa27e7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98227e80, 45 },
    { 0xc2e27e80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e527e8d, 33 },
    { 0x37227e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9427e91, 90 },
    { 0x85e27e92, 46 },
    { 0x8fa27e93, 19 },
    { 0, 0 },
    { 0xa1027e95, 88 },
    { 0x3d127e95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc227e99, 87 },
    { 0, 0 },
    { 0x11f27e9b, 98 },
    { 0, 0 },
    { 0x7527e9d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8027ea0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f27ea4, 55 },
    { 0x6e527ea5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f27ea8, 70 },
    { 0x9d727ea9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef27eac, 90 },
    { 0xd127ead, 85 },
    { 0x7b627eac, 55 },
    { 0x8227eac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b27eb2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d27eb6, 99 },
    { 0, 0 },
    { 0xcd527eb8, 98 },
    { 0, 0 },
    { 0x33127eba, 23 },
    { 0xf5427ebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64527ec0, 59 },
    { 0x94d27ec1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d827ec7, 99 },
    { 0x6e727ec8, 84 },
    { 0x4e227ec8, 76 },
    { 0x66f27ec8, 90 },
    { 0xb3927eca, 76 },
    { 0x2e127ecb, 98 },
    { 0, 0 },
    { 0x42927ece, 46 },
    { 0, 0 },
    { 0xfd827ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe27ed5, 61 },
    { 0, 0 },
    { 0x84227ed7, 61 },
    { 0x47727ed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e27edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37427edf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d527ee6, 33 },
    { 0xd5727ee6, 100 },
    { 0, 0 },
    { 0x22627ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12927eed, 90 },
    { 0, 0 },
    { 0x4c527eef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e27ef2, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e27ef7, 90 },
    { 0, 0 },
    { 0xd9c27ef9, 90 },
    { 0x67c27ef9, 33 },
    { 0, 0 },
    { 0x7a927efc, 34 },
    { 0x13627efd, 61 },
    { 0xcbb27efe, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x8927f01, 100 },
    { 0x14827f02, 55 },
    { 0x36d27f03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd927f08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a27f0f, 98 },
    { 0, 0 },
    { 0xa7727f11, 65 },
    { 0xa7327f12, 95 },
    { 0xee627f11, 93 },
    { 0, 0 },
    { 0x48127f15, 85 },
    { 0x84d27f16, 33 },
    { 0, 0 },
    { 0x34927f18, 84 },
    { 0, 0 },
    { 0x10f27f1a, 55 },
    { 0, 0 },
    { 0x6d827f1c, 98 },
    { 0, 0 },
    { 0xa0927f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44427f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b27f29, 78 },
    { 0, 0 },
    { 0x3df27f2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53127f2f, 34 },
    { 0, 0 },
    { 0xdc327f31, 87 },
    { 0x7e627f31, 23 },
    { 0x8227f33, 33 },
    { 0xc4d27f34, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x88727f37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b27f3a, 76 },
    { 0x63a27f3b, 84 },
    { 0x47f27f3b, 98 },
    { 0xdd527f3d, 78 },
    { 0x54f27f3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d27f43, 98 },
    { 0x1b227f43, 94 },
    { 0x20727f43, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd427f4c, 23 },
    { 0, 0 },
    { 0xc6d27f4e, 95 },
    { 0x5d827f4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a27f53, 52 },
    { 0x16c27f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f27f57, 83 },
    { 0xaf227f57, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a27f5d, 98 },
    { 0, 0 },
    { 0x73f27f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7927f63, 93 },
    { 0, 0 },
    { 0x9bc27f65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x90727f68, 90 },
    { 0xc1027f69, 43 },
    { 0x6b727f6a, 46 },
    { 0x1bc27f6b, 65 },
    { 0xc4327f68, 88 },
    { 0xa8f27f6d, 76 },
    { 0x2d327f69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d27f74, 23 },
    { 0xcad27f75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a27f78, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae727f7d, 61 },
    { 0xd2327f7d, 76 },
    { 0x9d927f7f, 76 },
    { 0x8db27f7d, 34 },
    { 0x74b27f81, 76 },
    { 0x86127f81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a27f85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8727f8b, 33 },
    { 0x9b27f8b, 81 },
    { 0xe6327f8c, 90 },
    { 0x7b627f8c, 33 },
    { 0xf6d27f8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8327f95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d727f98, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e27f9b, 61 },
    { 0, 0 },
    { 0xa9c27f9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c327fa0, 33 },
    { 0, 0 },
    { 0x30c27fa2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb027fa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f27fae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d27fb4, 90 },
    { 0, 0 },
    { 0xcb27fb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a27fba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b927fbe, 100 },
    { 0, 0 },
    { 0xf2627fc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82027fc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a827fca, 68 },
    { 0, 0 },
    { 0x9fb27fcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf27fd4, 55 },
    { 0, 0 },
    { 0xfa227fd6, 61 },
    { 0, 0 },
    { 0x3cf27fd8, 34 },
    { 0, 0 },
    { 0x47e27fda, 23 },
    { 0x43a27fdb, 93 },
    { 0x45627fdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad27fe4, 55 },
    { 0xb7f27fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3827fe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb27feb, 98 },
    { 0x43c27fec, 98 },
    { 0, 0 },
    { 0x53d27fee, 98 },
    { 0x44c27fef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c27ff2, 61 },
    { 0, 0 },
    { 0x53827ff4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b28000, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb228006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c28009, 95 },
    { 0x7cf2800a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42800f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e928012, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde628017, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd442801a, 90 },
    { 0xa2e2801b, 55 },
    { 0x2472801c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e28020, 77 },
    { 0xc3a28020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e28026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f928029, 75 },
    { 0xc9328029, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e28033, 98 },
    { 0x37128034, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e28037, 98 },
    { 0x44f28038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b2803c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b428043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c28049, 76 },
    { 0, 0 },
    { 0xf1d2804b, 52 },
    { 0xe602804c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf822804f, 98 },
    { 0xd6328050, 95 },
    { 0xcdc2804f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac2805d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64228060, 93 },
    { 0x82128061, 61 },
    { 0xf5e28060, 85 },
    { 0x74828063, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2806d, 78 },
    { 0x5d2806e, 84 },
    { 0, 0 },
    { 0x59d28070, 90 },
    { 0xfb728071, 87 },
    { 0x1da28070, 61 },
    { 0x58e28073, 23 },
    { 0x62128074, 95 },
    { 0x42928075, 55 },
    { 0, 0 },
    { 0xe6f28077, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1512807c, 33 },
    { 0x39f2807c, 90 },
    { 0, 0 },
    { 0x4c22807f, 33 },
    { 0, 0 },
    { 0xdd428081, 91 },
    { 0, 0 },
    { 0xeb28083, 33 },
    { 0x62d28084, 63 },
    { 0xdf128085, 91 },
    { 0xd3728085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af28089, 76 },
    { 0, 0 },
    { 0x66e2808b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42928096, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5342809e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69280a4, 98 },
    { 0x36e280a5, 90 },
    { 0xbb6280a5, 61 },
    { 0x249280a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce280ad, 78 },
    { 0x487280ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d280b1, 55 },
    { 0xd08280b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x482280b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29280b9, 78 },
    { 0, 0 },
    { 0xaa9280bb, 95 },
    { 0x7d1280bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187280c1, 55 },
    { 0, 0 },
    { 0x6e4280c3, 90 },
    { 0xde280c3, 61 },
    { 0, 0 },
    { 0x3ac280c6, 95 },
    { 0, 0 },
    { 0xb0280c8, 33 },
    { 0xef0280c8, 90 },
    { 0x5bb280c9, 98 },
    { 0x8e3280cb, 99 },
    { 0x245280cc, 84 },
    { 0xbe8280cc, 95 },
    { 0, 0 },
    { 0xf7e280cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08280d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655280d9, 98 },
    { 0x146280da, 95 },
    { 0, 0 },
    { 0x81b280dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5280e0, 95 },
    { 0, 0 },
    { 0xc96280e2, 90 },
    { 0x6d6280e2, 100 },
    { 0x8b9280e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1280ee, 98 },
    { 0x3fa280ef, 90 },
    { 0x1c9280ee, 23 },
    { 0, 0 },
    { 0xa92280f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf280f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11280fc, 93 },
    { 0, 0 },
    { 0x526280fe, 33 },
    { 0xbc7280ff, 76 },
    { 0xecc28100, 61 },
    { 0, 0 },
    { 0x19228102, 23 },
    { 0x15b28103, 83 },
    { 0xb8128103, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec928109, 24 },
    { 0x7222810a, 23 },
    { 0x342810b, 91 },
    { 0x8e228109, 55 },
    { 0xd712810a, 90 },
    { 0x3202810d, 74 },
    { 0x24e2810f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb28115, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf128118, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac82811d, 93 },
    { 0, 0 },
    { 0x7f02811f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c328122, 85 },
    { 0, 0 },
    { 0xb3c28124, 33 },
    { 0, 0 },
    { 0x36e28126, 90 },
    { 0xf1228127, 61 },
    { 0x9a428126, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8792812c, 61 },
    { 0x4e22812c, 85 },
    { 0xb512812e, 88 },
    { 0xeb2812d, 33 },
    { 0x4542812c, 52 },
    { 0xd7728130, 61 },
    { 0x43228132, 23 },
    { 0, 0 },
    { 0x98c28134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46528137, 81 },
    { 0x13628138, 43 },
    { 0xf0928139, 66 },
    { 0x9b92813a, 46 },
    { 0, 0 },
    { 0xd422813c, 23 },
    { 0x8f12813d, 61 },
    { 0, 0 },
    { 0x15a2813f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b928144, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c32814b, 91 },
    { 0, 0 },
    { 0xa662814d, 33 },
    { 0x4ee2814d, 55 },
    { 0xf602814d, 43 },
    { 0x7b12814f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b28154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e28159, 77 },
    { 0xd32815a, 43 },
    { 0xc502815b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9628161, 43 },
    { 0x2ec28162, 61 },
    { 0x2ee28163, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19428167, 90 },
    { 0xbba28168, 17 },
    { 0x92d28169, 33 },
    { 0x65d2816a, 55 },
    { 0x53528168, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56628173, 76 },
    { 0x2dc28173, 76 },
    { 0x23b28173, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2112817a, 95 },
    { 0xd8f2817b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e12817e, 17 },
    { 0, 0 },
    { 0xd4128180, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7228183, 33 },
    { 0x12e28184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f28189, 53 },
    { 0, 0 },
    { 0x7a52818b, 93 },
    { 0, 0 },
    { 0x482818d, 98 },
    { 0x7902818e, 61 },
    { 0, 0 },
    { 0x17928190, 61 },
    { 0, 0 },
    { 0xf9528192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd2819a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf202819e, 95 },
    { 0x1972819f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd281a4, 33 },
    { 0xcd1281a5, 23 },
    { 0x41281a5, 98 },
    { 0x7d2281a7, 66 },
    { 0, 0 },
    { 0x8ee281a9, 93 },
    { 0x841281aa, 23 },
    { 0xc4c281a9, 33 },
    { 0x785281ab, 33 },
    { 0x541281a9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628281b3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce281bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x201281c0, 90 },
    { 0x239281c1, 93 },
    { 0xd4c281c2, 40 },
    { 0, 0 },
    { 0x385281c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc281ce, 91 },
    { 0x7e9281cf, 66 },
    { 0x3f2281d0, 90 },
    { 0, 0 },
    { 0x1be281d2, 99 },
    { 0x3db281d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed281d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba281dc, 43 },
    { 0xd0e281dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04281e8, 77 },
    { 0, 0 },
    { 0x178281ea, 55 },
    { 0xfc6281eb, 98 },
    { 0, 0 },
    { 0xeae281ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d281f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a281f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a281f9, 90 },
    { 0, 0 },
    { 0xbdc281fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d328202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef2820b, 17 },
    { 0x5202820b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8728211, 52 },
    { 0, 0 },
    { 0x37e28213, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b28218, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2072821c, 95 },
    { 0, 0 },
    { 0x962821e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc28222, 84 },
    { 0, 0 },
    { 0x84a28224, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a2822c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e28230, 95 },
    { 0x4ca28231, 33 },
    { 0x48528230, 81 },
    { 0xc2328233, 98 },
    { 0x8f228233, 52 },
    { 0x9d728233, 58 },
    { 0xb2128236, 90 },
    { 0xfa928237, 90 },
    { 0x1c328232, 38 },
    { 0x7d728239, 30 },
    { 0, 0 },
    { 0x5f52823b, 90 },
    { 0x9ae2823c, 90 },
    { 0x6462823b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f28240, 17 },
    { 0, 0 },
    { 0x95b28242, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17528249, 90 },
    { 0x8e32824a, 34 },
    { 0x8b72824b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad2824f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c28252, 85 },
    { 0, 0 },
    { 0x94928254, 98 },
    { 0x48228255, 65 },
    { 0x85228256, 61 },
    { 0, 0 },
    { 0x4f628258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3922825b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0328262, 75 },
    { 0x8e228263, 87 },
    { 0, 0 },
    { 0xafb28265, 90 },
    { 0x33028266, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f2826b, 95 },
    { 0, 0 },
    { 0xc22826d, 90 },
    { 0xb6d2826e, 34 },
    { 0x62826f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc728273, 84 },
    { 0, 0 },
    { 0x58228275, 90 },
    { 0x36028276, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca28279, 52 },
    { 0xbe42827a, 33 },
    { 0xa552827b, 76 },
    { 0, 0 },
    { 0x9792827d, 99 },
    { 0, 0 },
    { 0xa9e2827f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8128282, 17 },
    { 0, 0 },
    { 0x4d228284, 101 },
    { 0x5c28285, 61 },
    { 0, 0 },
    { 0xb9528287, 98 },
    { 0xff828287, 43 },
    { 0, 0 },
    { 0xd262828a, 33 },
    { 0xd882828b, 33 },
    { 0xbd22828b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd852828f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce628293, 90 },
    { 0x51728294, 34 },
    { 0xb0a28295, 90 },
    { 0, 0 },
    { 0x1c628297, 61 },
    { 0x36b28298, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32829f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb282a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132282a8, 66 },
    { 0x63d282a8, 55 },
    { 0xf60282a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d282ad, 95 },
    { 0xd06282ae, 98 },
    { 0xff1282af, 38 },
    { 0x187282b0, 67 },
    { 0, 0 },
    { 0xa7b282b2, 65 },
    { 0x20d282b2, 76 },
    { 0, 0 },
    { 0x739282b5, 36 },
    { 0xb7d282b5, 36 },
    { 0, 0 },
    { 0x3d9282b8, 90 },
    { 0x155282b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a282bd, 52 },
    { 0x8df282be, 60 },
    { 0, 0 },
    { 0x59e282c0, 90 },
    { 0xb47282c1, 61 },
    { 0x448282c1, 93 },
    { 0, 0 },
    { 0x98b282c4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe282c8, 100 },
    { 0xf2d282c8, 99 },
    { 0xea282c8, 17 },
    { 0, 0 },
    { 0x227282cc, 87 },
    { 0x191282cd, 93 },
    { 0xda7282cd, 95 },
    { 0, 0 },
    { 0xdf1282d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a282d5, 77 },
    { 0xace282d6, 90 },
    { 0x166282d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27282dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940282e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9282e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5282ea, 93 },
    { 0xb94282eb, 68 },
    { 0xbd8282eb, 61 },
    { 0xf67282ed, 83 },
    { 0, 0 },
    { 0x5a9282ef, 33 },
    { 0xfe6282f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc282f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63282f8, 61 },
    { 0, 0 },
    { 0x87c282fa, 23 },
    { 0x172282fb, 90 },
    { 0x91f282fc, 84 },
    { 0x539282fd, 33 },
    { 0, 0 },
    { 0x804282ff, 98 },
    { 0xa2728300, 98 },
    { 0x3de28301, 91 },
    { 0xde728302, 61 },
    { 0xfc7282ff, 95 },
    { 0x50b28304, 90 },
    { 0x7c228300, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2830a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a32830d, 98 },
    { 0x7572830e, 90 },
    { 0x132830e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d928312, 100 },
    { 0x6dc28312, 98 },
    { 0xef028314, 98 },
    { 0, 0 },
    { 0xf0728316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be28319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9582831f, 66 },
    { 0x51e2831f, 100 },
    { 0, 0 },
    { 0x27b28322, 23 },
    { 0x61228323, 93 },
    { 0, 0 },
    { 0x19328325, 61 },
    { 0x45b28326, 75 },
    { 0xd8c28327, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd572832a, 23 },
    { 0x6342832a, 90 },
    { 0, 0 },
    { 0xd4c2832d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8928330, 90 },
    { 0xa4e28330, 81 },
    { 0, 0 },
    { 0x5628333, 56 },
    { 0x77128334, 95 },
    { 0x4df28335, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9228338, 34 },
    { 0x8da28339, 55 },
    { 0xee12833a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f2833f, 95 },
    { 0xa022833f, 90 },
    { 0x22d2833f, 61 },
    { 0xe7b28342, 101 },
    { 0, 0 },
    { 0x75928344, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86828348, 33 },
    { 0xa2c28349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae2834c, 33 },
    { 0x8d92834d, 97 },
    { 0, 0 },
    { 0x6932834f, 98 },
    { 0, 0 },
    { 0x9ed28351, 76 },
    { 0x61628352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b92835c, 65 },
    { 0xa082835c, 43 },
    { 0x3202835e, 35 },
    { 0xe432835d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9228362, 92 },
    { 0xba728363, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df28369, 55 },
    { 0xa8f28369, 84 },
    { 0x9922836b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3902836e, 61 },
    { 0, 0 },
    { 0xeff28370, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf28376, 76 },
    { 0x4828376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f2837b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628384, 33 },
    { 0, 0 },
    { 0xb2a28386, 95 },
    { 0x11a28387, 81 },
    { 0xbc628386, 38 },
    { 0x42c28389, 33 },
    { 0x6a728389, 88 },
    { 0x97e2838a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd2838f, 33 },
    { 0x8128390, 90 },
    { 0x6f92838f, 98 },
    { 0, 0 },
    { 0xb0528393, 90 },
    { 0x73928394, 33 },
    { 0x5b728395, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c28399, 91 },
    { 0x44e28399, 55 },
    { 0xd9b28399, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f283a0, 55 },
    { 0x4b283a1, 66 },
    { 0, 0 },
    { 0xbbf283a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91283a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f283b0, 56 },
    { 0x94d283b0, 61 },
    { 0, 0 },
    { 0x18a283b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f283b7, 61 },
    { 0, 0 },
    { 0xb9d283b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d283c0, 55 },
    { 0xaa4283c0, 52 },
    { 0, 0 },
    { 0xbdc283c3, 40 },
    { 0, 0 },
    { 0xec2283c5, 90 },
    { 0, 0 },
    { 0xcb2283c7, 98 },
    { 0x492283c8, 84 },
    { 0xcb1283c9, 46 },
    { 0x61f283ca, 77 },
    { 0xeb1283c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d283cf, 23 },
    { 0xc24283d0, 90 },
    { 0x11e283cf, 90 },
    { 0x51d283d2, 23 },
    { 0x9e3283d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4283df, 34 },
    { 0x8bc283df, 46 },
    { 0, 0 },
    { 0xc2283e2, 90 },
    { 0xecc283e2, 93 },
    { 0x376283e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6283ee, 90 },
    { 0, 0 },
    { 0x13e283f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf283f4, 68 },
    { 0, 0 },
    { 0xc8a283f6, 84 },
    { 0xe17283f7, 88 },
    { 0xd48283f8, 33 },
    { 0x702283f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f283fe, 66 },
    { 0x8d1283fe, 93 },
    { 0, 0 },
    { 0x33c28401, 95 },
    { 0x17528402, 90 },
    { 0xb7628402, 61 },
    { 0x7d28404, 99 },
    { 0xaaa28401, 23 },
    { 0x8d228406, 33 },
    { 0x12f28407, 76 },
    { 0xdf628408, 68 },
    { 0, 0 },
    { 0x3b2840a, 84 },
    { 0, 0 },
    { 0xa192840c, 23 },
    { 0x8a52840c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc28410, 61 },
    { 0xf6228411, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde328417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf142841c, 52 },
    { 0x9772841d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d28420, 34 },
    { 0xb4828421, 23 },
    { 0, 0 },
    { 0xe8728423, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86328426, 97 },
    { 0x14928427, 95 },
    { 0xe6828426, 98 },
    { 0x2ad28428, 75 },
    { 0xcb12842a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc62842e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x70428431, 90 },
    { 0xa9928432, 78 },
    { 0x46d28431, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf128437, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc682843a, 59 },
    { 0xa762843b, 95 },
    { 0xf4d2843b, 80 },
    { 0xb7e2843c, 46 },
    { 0xce82843d, 87 },
    { 0xd492843b, 95 },
    { 0, 0 },
    { 0xa9528441, 84 },
    { 0xca428442, 23 },
    { 0x1a028441, 91 },
    { 0xdf928444, 76 },
    { 0x7c928444, 95 },
    { 0x328445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d28449, 61 },
    { 0xc852844a, 66 },
    { 0, 0 },
    { 0xe0d2844c, 78 },
    { 0xd22844c, 90 },
    { 0x2b62844c, 17 },
    { 0x48d2844f, 36 },
    { 0x3ce28450, 90 },
    { 0x48328450, 90 },
    { 0x43a28451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272845a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed72845e, 100 },
    { 0x6582845f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2328464, 33 },
    { 0xbca28464, 95 },
    { 0x7428466, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9882846d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0328471, 23 },
    { 0xf7328471, 61 },
    { 0x73b28473, 23 },
    { 0x6f628473, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2528478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2847b, 61 },
    { 0xa502847c, 33 },
    { 0xdd62847d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6328481, 98 },
    { 0x8e28482, 96 },
    { 0, 0 },
    { 0x9f528484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77128488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c2848b, 80 },
    { 0xb772848b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59928490, 76 },
    { 0xbde28490, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2128498, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a42849e, 95 },
    { 0x1882849e, 55 },
    { 0x77f284a0, 33 },
    { 0xad1284a1, 61 },
    { 0xe70284a2, 23 },
    { 0, 0 },
    { 0xf92284a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d284a7, 90 },
    { 0x751284a7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808284ac, 78 },
    { 0x717284ad, 61 },
    { 0xa3a284ae, 55 },
    { 0x4f5284ae, 23 },
    { 0, 0 },
    { 0xe4a284b1, 66 },
    { 0, 0 },
    { 0xaa284b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98284b7, 55 },
    { 0x2ff284b8, 33 },
    { 0x6e3284b9, 98 },
    { 0, 0 },
    { 0x17f284bb, 90 },
    { 0xc1c284bc, 98 },
    { 0, 0 },
    { 0xdc6284be, 90 },
    { 0x55284bf, 81 },
    { 0xf28284c0, 94 },
    { 0x752284c1, 95 },
    { 0x93284c2, 84 },
    { 0x4cf284c3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4284c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f284cc, 76 },
    { 0, 0 },
    { 0x5bf284ce, 33 },
    { 0xabe284ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea284d7, 100 },
    { 0xa86284d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e284db, 43 },
    { 0x5fc284db, 76 },
    { 0x9b2284dd, 43 },
    { 0xa68284de, 61 },
    { 0xb0b284de, 90 },
    { 0xa0b284de, 45 },
    { 0x27d284e0, 98 },
    { 0x5ef284df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4284e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e284ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x547284f1, 78 },
    { 0x8c1284f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07284f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980284ff, 90 },
    { 0x94228500, 33 },
    { 0x37f28501, 87 },
    { 0, 0 },
    { 0x3cb28503, 85 },
    { 0, 0 },
    { 0x68528505, 99 },
    { 0xb5528506, 90 },
    { 0x8e428507, 33 },
    { 0x2fd28507, 91 },
    { 0x52f28507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a52850c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe28510, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c28518, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832851e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f28522, 90 },
    { 0x74428523, 90 },
    { 0, 0 },
    { 0xa828525, 69 },
    { 0x19b28525, 90 },
    { 0x86028525, 66 },
    { 0x29428528, 93 },
    { 0, 0 },
    { 0x58d2852a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3028531, 98 },
    { 0x5b228532, 90 },
    { 0x86e28532, 46 },
    { 0x5c428534, 85 },
    { 0xca228534, 90 },
    { 0x50228532, 61 },
    { 0x12f28537, 34 },
    { 0xf7c28536, 43 },
    { 0xef028539, 53 },
    { 0x48428539, 90 },
    { 0xb4a2853a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552853e, 27 },
    { 0x4b02853e, 33 },
    { 0, 0 },
    { 0xe7d28541, 87 },
    { 0x13528542, 90 },
    { 0, 0 },
    { 0xa3e28544, 34 },
    { 0xc2828544, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d628548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1942854b, 98 },
    { 0x4a32854b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7202854f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf128552, 61 },
    { 0x10028553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a028557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b28561, 87 },
    { 0xd0f28562, 98 },
    { 0, 0 },
    { 0x61828564, 98 },
    { 0, 0 },
    { 0x43628566, 76 },
    { 0, 0 },
    { 0x33928568, 77 },
    { 0xec28569, 23 },
    { 0xe462856a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2272856f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38928573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc428578, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee2857b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f2857e, 95 },
    { 0, 0 },
    { 0x99d28580, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37428585, 33 },
    { 0xd4228586, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb528589, 78 },
    { 0x8272858a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9472858d, 95 },
    { 0, 0 },
    { 0x6b2858f, 90 },
    { 0x10f2858f, 66 },
    { 0x1ab28591, 99 },
    { 0xecd28591, 95 },
    { 0, 0 },
    { 0x24228594, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8232859a, 90 },
    { 0x5492859b, 76 },
    { 0x3962859a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66285a0, 36 },
    { 0, 0 },
    { 0x411285a2, 91 },
    { 0xfa8285a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b285aa, 76 },
    { 0x9c6285ab, 95 },
    { 0x32a285ac, 61 },
    { 0x944285ac, 91 },
    { 0x847285ae, 98 },
    { 0x5ff285ac, 84 },
    { 0x80d285ad, 90 },
    { 0x33d285ad, 99 },
    { 0xfe4285b1, 95 },
    { 0xb49285b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3285b6, 91 },
    { 0, 0 },
    { 0xb93285b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb285bb, 95 },
    { 0xaab285bc, 95 },
    { 0, 0 },
    { 0x624285be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5285c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b285c8, 100 },
    { 0x3bf285c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23285cf, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd285d3, 61 },
    { 0xa4d285d3, 49 },
    { 0, 0 },
    { 0x72285d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b285d9, 23 },
    { 0, 0 },
    { 0xcd8285db, 76 },
    { 0x91f285dc, 69 },
    { 0x204285dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca285e1, 87 },
    { 0x94f285e2, 61 },
    { 0x907285e2, 23 },
    { 0x285285e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0285e8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e285eb, 93 },
    { 0xd0285eb, 90 },
    { 0x2f6285ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab285f0, 55 },
    { 0x975285f0, 58 },
    { 0, 0 },
    { 0x870285f3, 33 },
    { 0x4b8285f4, 23 },
    { 0, 0 },
    { 0x33a285f6, 17 },
    { 0, 0 },
    { 0x2c285f8, 46 },
    { 0x9d6285f9, 90 },
    { 0x26e285fa, 61 },
    { 0, 0 },
    { 0xc72285fc, 81 },
    { 0x3c1285fd, 90 },
    { 0x55a285fe, 23 },
    { 0xfd9285ff, 98 },
    { 0x22828600, 61 },
    { 0xc2c28601, 95 },
    { 0x38f28601, 61 },
    { 0, 0 },
    { 0xffe28604, 32 },
    { 0, 0 },
    { 0x43d28606, 95 },
    { 0, 0 },
    { 0x87328608, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe2860b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d72860e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c28611, 61 },
    { 0x85328612, 33 },
    { 0x34128613, 33 },
    { 0x9728614, 23 },
    { 0xe5f28615, 33 },
    { 0x86a28611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8312861e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca528621, 52 },
    { 0xf328622, 91 },
    { 0x9c628621, 84 },
    { 0x61028624, 46 },
    { 0, 0 },
    { 0xe4a28626, 95 },
    { 0, 0 },
    { 0x3c328628, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2862c, 90 },
    { 0xbd32862d, 90 },
    { 0x4d42862d, 61 },
    { 0, 0 },
    { 0x76c28630, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c28635, 86 },
    { 0xda28635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f28639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2863d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa128640, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f328643, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb28646, 93 },
    { 0, 0 },
    { 0x6b128648, 91 },
    { 0xfd528649, 95 },
    { 0, 0 },
    { 0xa912864b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb392864f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2528652, 98 },
    { 0x5b28653, 52 },
    { 0x8cc28653, 98 },
    { 0xc5a28655, 55 },
    { 0xafd28655, 33 },
    { 0x70d28654, 93 },
    { 0, 0 },
    { 0x1ef28659, 95 },
    { 0, 0 },
    { 0xc6b2865b, 35 },
    { 0x7ca2865c, 84 },
    { 0, 0 },
    { 0x6e62865e, 36 },
    { 0, 0 },
    { 0x3c628660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e628665, 40 },
    { 0xdaa28666, 61 },
    { 0x84028667, 23 },
    { 0xa7128667, 23 },
    { 0x2928666, 90 },
    { 0, 0 },
    { 0x7412866b, 90 },
    { 0x55f2866b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7402866f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc028674, 98 },
    { 0xa828675, 55 },
    { 0x91128676, 95 },
    { 0xc1328677, 23 },
    { 0xd4728677, 55 },
    { 0x16228679, 90 },
    { 0x3542867a, 95 },
    { 0x3762867b, 67 },
    { 0x752867c, 90 },
    { 0xc5428679, 33 },
    { 0x8c42867e, 90 },
    { 0x3852867b, 95 },
    { 0x89428680, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x56528683, 23 },
    { 0xef728683, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f2868d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a828695, 23 },
    { 0x21328695, 33 },
    { 0x8c028697, 40 },
    { 0x1c728696, 61 },
    { 0xe3f28699, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1762869e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c286a1, 61 },
    { 0x3d3286a1, 34 },
    { 0, 0 },
    { 0xba2286a4, 90 },
    { 0, 0 },
    { 0x51a286a6, 55 },
    { 0x28d286a6, 95 },
    { 0, 0 },
    { 0x3af286a9, 33 },
    { 0x876286aa, 55 },
    { 0xce9286aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9286af, 93 },
    { 0, 0 },
    { 0x134286b1, 23 },
    { 0x4a3286b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d286b5, 46 },
    { 0xb63286b6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808286ba, 33 },
    { 0x48e286bb, 98 },
    { 0xa72286ba, 90 },
    { 0xb1d286bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776286c1, 55 },
    { 0, 0 },
    { 0x70a286c3, 33 },
    { 0x115286c3, 91 },
    { 0x846286c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b286cb, 95 },
    { 0x6ee286cc, 76 },
    { 0, 0 },
    { 0x39e286ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a286d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c286d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6286db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130286df, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd286e2, 61 },
    { 0, 0 },
    { 0x39f286e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d286e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc286ea, 98 },
    { 0xd1286eb, 90 },
    { 0x9fd286ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a286f0, 61 },
    { 0xabc286f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e286f4, 33 },
    { 0x9fb286f5, 55 },
    { 0, 0 },
    { 0x350286f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1228701, 90 },
    { 0x94e28702, 91 },
    { 0x80628702, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb028706, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8972870f, 23 },
    { 0x5d828710, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45728714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc2871d, 76 },
    { 0, 0 },
    { 0x75d2871f, 33 },
    { 0x8cb2871f, 78 },
    { 0x8b228721, 90 },
    { 0xfe028721, 92 },
    { 0xe4b28723, 61 },
    { 0x17328723, 90 },
    { 0x79e28724, 78 },
    { 0x43c28726, 91 },
    { 0x1b428726, 90 },
    { 0x57828724, 90 },
    { 0x5f28729, 95 },
    { 0x1f128722, 55 },
    { 0, 0 },
    { 0xb822872c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e28736, 84 },
    { 0x7e28737, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee92873e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e28745, 52 },
    { 0x40528746, 95 },
    { 0, 0 },
    { 0xedb28748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2874c, 75 },
    { 0x2de2874d, 98 },
    { 0, 0 },
    { 0x7332874f, 34 },
    { 0x31928750, 33 },
    { 0xb8128751, 43 },
    { 0xa2e2874f, 95 },
    { 0, 0 },
    { 0x3ac28754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d528759, 61 },
    { 0x3102875a, 98 },
    { 0x1d32875b, 95 },
    { 0xc322875c, 95 },
    { 0xfad2875d, 78 },
    { 0x6152875d, 76 },
    { 0, 0 },
    { 0x93428760, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d2876a, 46 },
    { 0xd232876a, 90 },
    { 0x6762876c, 33 },
    { 0x71e2876c, 95 },
    { 0x3b32876c, 98 },
    { 0, 0 },
    { 0x5fa28770, 90 },
    { 0xb4928770, 23 },
    { 0x8828771, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32877b, 17 },
    { 0x17f2877c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29328781, 61 },
    { 0x9e528781, 33 },
    { 0, 0 },
    { 0x3d728784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a28788, 83 },
    { 0x5ee28788, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x752878c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b628792, 80 },
    { 0x10f28793, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8128797, 98 },
    { 0xf9028798, 95 },
    { 0, 0 },
    { 0xe872879a, 23 },
    { 0x9372879b, 90 },
    { 0xfbc2879c, 98 },
    { 0xcd02879d, 52 },
    { 0, 0 },
    { 0x7a92879f, 61 },
    { 0x23a287a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547287a6, 33 },
    { 0xa42287a7, 98 },
    { 0x332287a7, 66 },
    { 0, 0 },
    { 0x4a4287aa, 98 },
    { 0, 0 },
    { 0x3ff287ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf287af, 95 },
    { 0x11d287b0, 46 },
    { 0, 0 },
    { 0x9df287b2, 65 },
    { 0, 0 },
    { 0x671287b4, 90 },
    { 0x82a287b5, 33 },
    { 0x215287b4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x590287b9, 55 },
    { 0, 0 },
    { 0x3af287bb, 78 },
    { 0xab0287bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887287c0, 91 },
    { 0x985287c1, 43 },
    { 0xedf287c1, 76 },
    { 0x8c6287c2, 61 },
    { 0xaa3287c4, 61 },
    { 0x31287c3, 90 },
    { 0x31a287c5, 76 },
    { 0x387287c7, 90 },
    { 0xe8b287c8, 66 },
    { 0xc07287c9, 55 },
    { 0xa0a287ca, 98 },
    { 0x424287c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59287cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b287d5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e287dc, 23 },
    { 0xae5287dd, 92 },
    { 0x3287dc, 33 },
    { 0, 0 },
    { 0xd1287e0, 90 },
    { 0, 0 },
    { 0x1a4287e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0287e6, 95 },
    { 0x916287e6, 99 },
    { 0x85f287e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3287ed, 90 },
    { 0x371287ed, 61 },
    { 0, 0 },
    { 0x46b287f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac287f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389287f8, 88 },
    { 0x447287f9, 77 },
    { 0, 0 },
    { 0x8b8287fb, 17 },
    { 0, 0 },
    { 0xcbb287fd, 55 },
    { 0xcf2287fe, 93 },
    { 0, 0 },
    { 0x6d728800, 99 },
    { 0, 0 },
    { 0x95a28802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2880a, 90 },
    { 0xe702880b, 98 },
    { 0x6e22880c, 98 },
    { 0x7772880b, 95 },
    { 0x5352880d, 98 },
    { 0x77b2880f, 33 },
    { 0, 0 },
    { 0x74228811, 84 },
    { 0xfe128812, 81 },
    { 0x1ed28813, 90 },
    { 0x59e28814, 51 },
    { 0xb4328811, 98 },
    { 0x30328816, 33 },
    { 0, 0 },
    { 0x8f128818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5202881d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f28820, 100 },
    { 0x5c928820, 17 },
    { 0xbb28822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba28826, 55 },
    { 0xb928827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1482882c, 75 },
    { 0, 0 },
    { 0x62b2882e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e28834, 90 },
    { 0, 0 },
    { 0xd4728836, 93 },
    { 0xe3628836, 98 },
    { 0xe4428838, 95 },
    { 0x5e928836, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2883e, 90 },
    { 0xe282883f, 43 },
    { 0xc7428840, 52 },
    { 0x4ad28840, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b28845, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802884b, 55 },
    { 0, 0 },
    { 0x65a2884d, 84 },
    { 0x4b42884e, 23 },
    { 0, 0 },
    { 0xe6e28850, 100 },
    { 0x35128851, 95 },
    { 0xbd028852, 66 },
    { 0xd0528853, 87 },
    { 0xf2228854, 88 },
    { 0xc6528855, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e2885a, 91 },
    { 0x6ec2885b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f02885e, 43 },
    { 0xc842885e, 43 },
    { 0xc292885e, 98 },
    { 0, 0 },
    { 0xa2d28862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab28866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d2886a, 46 },
    { 0x2ac2886b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6842886e, 67 },
    { 0x2ad2886f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8228872, 90 },
    { 0x44f28872, 33 },
    { 0, 0 },
    { 0xba028875, 98 },
    { 0x84528876, 84 },
    { 0x7c528877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf62887f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c828885, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d28888, 90 },
    { 0x88e28889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e2888e, 66 },
    { 0, 0 },
    { 0xdfa28890, 90 },
    { 0x99328891, 100 },
    { 0, 0 },
    { 0x44428893, 81 },
    { 0x4a528894, 98 },
    { 0xc2228895, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8328898, 61 },
    { 0xd9c28899, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2889d, 98 },
    { 0x5432889d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9288a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd288a4, 98 },
    { 0, 0 },
    { 0x233288a6, 76 },
    { 0xc72288a7, 61 },
    { 0, 0 },
    { 0x41e288a9, 76 },
    { 0xfab288a9, 95 },
    { 0, 0 },
    { 0x4ea288ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724288b1, 90 },
    { 0x216288b2, 55 },
    { 0, 0 },
    { 0x821288b4, 40 },
    { 0x362288b4, 46 },
    { 0xedf288b5, 43 },
    { 0, 0 },
    { 0xf9d288b8, 23 },
    { 0xbb8288b8, 66 },
    { 0, 0 },
    { 0x286288bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c288bf, 90 },
    { 0x5e9288bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b288c4, 88 },
    { 0x568288c4, 55 },
    { 0x10c288c6, 43 },
    { 0x281288c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25288cc, 61 },
    { 0x335288cd, 17 },
    { 0xeb1288ce, 61 },
    { 0xa8a288cc, 34 },
    { 0x32e288d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681288d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531288da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5288dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9288e4, 90 },
    { 0x889288e5, 90 },
    { 0x586288e6, 23 },
    { 0xbe288e7, 52 },
    { 0xd9a288e5, 100 },
    { 0xc5b288e9, 76 },
    { 0xa72288ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a288ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679288f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x698288f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520288fe, 99 },
    { 0xc4d288fe, 52 },
    { 0x90228900, 23 },
    { 0x455288ff, 33 },
    { 0x7028902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a128906, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7022890a, 95 },
    { 0xe9a2890a, 84 },
    { 0x59e2890c, 93 },
    { 0xa8d2890b, 88 },
    { 0xd0e2890c, 91 },
    { 0xab82890e, 23 },
    { 0x8aa2890e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c328918, 46 },
    { 0x99d28919, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2891c, 23 },
    { 0x44e2891c, 98 },
    { 0x58d2891e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e28922, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0428925, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50928931, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b528937, 23 },
    { 0x3df28938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced28944, 23 },
    { 0xdeb28945, 20 },
    { 0, 0 },
    { 0xb4028947, 33 },
    { 0, 0 },
    { 0x52b28949, 65 },
    { 0x8de2894a, 87 },
    { 0x96b2894b, 61 },
    { 0x3602894a, 90 },
    { 0x992894d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a28950, 46 },
    { 0xc3728951, 61 },
    { 0xa5128952, 88 },
    { 0, 0 },
    { 0xca828954, 55 },
    { 0x56828954, 23 },
    { 0x40428956, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2895e, 33 },
    { 0x1d82895f, 98 },
    { 0xedf28960, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d128965, 5 },
    { 0x5eb28966, 76 },
    { 0xe7128965, 98 },
    { 0xf1228968, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce028970, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d28974, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1128978, 61 },
    { 0x5dd28978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a2897c, 33 },
    { 0xfbb2897d, 17 },
    { 0xc962897e, 23 },
    { 0xae82897f, 23 },
    { 0, 0 },
    { 0x22328981, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5828985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec72898a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7028998, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd02899f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x943289a2, 90 },
    { 0x8ac289a3, 93 },
    { 0x138289a4, 76 },
    { 0x1f4289a3, 61 },
    { 0, 0 },
    { 0x335289a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c289b0, 95 },
    { 0xc5e289b0, 33 },
    { 0x72a289b2, 23 },
    { 0x4c5289b3, 89 },
    { 0x361289b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6289b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31289bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404289c4, 43 },
    { 0xf16289c4, 33 },
    { 0, 0 },
    { 0xc8f289c7, 34 },
    { 0x85e289c8, 17 },
    { 0x380289c9, 55 },
    { 0x697289c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53289cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af289d2, 46 },
    { 0, 0 },
    { 0x3f1289d4, 33 },
    { 0, 0 },
    { 0x230289d6, 90 },
    { 0xac0289d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8289db, 68 },
    { 0xda2289db, 61 },
    { 0x1e7289dd, 61 },
    { 0x995289dd, 17 },
    { 0x3df289dd, 43 },
    { 0x7f4289e0, 81 },
    { 0xe9289e0, 23 },
    { 0xa61289e2, 78 },
    { 0xefe289e0, 52 },
    { 0x373289e4, 33 },
    { 0x6fd289e4, 66 },
    { 0, 0 },
    { 0x6d8289e7, 95 },
    { 0xbd8289e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x563289eb, 43 },
    { 0x93d289eb, 95 },
    { 0x72f289ed, 98 },
    { 0x212289ee, 23 },
    { 0xfe3289eb, 76 },
    { 0xb07289f0, 33 },
    { 0x2dd289f0, 23 },
    { 0, 0 },
    { 0x1c7289f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775289f7, 76 },
    { 0xfc289f8, 33 },
    { 0x5c4289f9, 76 },
    { 0x7b9289f9, 61 },
    { 0, 0 },
    { 0x8bc289fc, 87 },
    { 0xf36289fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f28a01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54028a06, 93 },
    { 0, 0 },
    { 0x8528a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf28a1a, 23 },
    { 0, 0 },
    { 0xca528a1c, 95 },
    { 0x78228a1d, 98 },
    { 0, 0 },
    { 0x6d528a1f, 23 },
    { 0x64528a20, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d628a23, 76 },
    { 0xd6d28a24, 91 },
    { 0xcfe28a24, 90 },
    { 0x72928a26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63428a2a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6828a2d, 38 },
    { 0x6d428a2e, 33 },
    { 0x49228a2f, 34 },
    { 0x9d928a2f, 76 },
    { 0xb628a31, 100 },
    { 0xfeb28a32, 46 },
    { 0xb8028a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba28a3b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e28a3f, 100 },
    { 0x41928a3f, 98 },
    { 0x9c228a40, 33 },
    { 0xe7228a3f, 46 },
    { 0xc8728a3f, 33 },
    { 0, 0 },
    { 0xb5f28a45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf28a51, 87 },
    { 0xf3f28a52, 23 },
    { 0xe7528a53, 23 },
    { 0x15828a54, 65 },
    { 0, 0 },
    { 0xff28a56, 78 },
    { 0x48428a57, 78 },
    { 0xe728a58, 55 },
    { 0x19028a56, 90 },
    { 0, 0 },
    { 0xf6c28a5b, 87 },
    { 0x91828a5c, 61 },
    { 0xb5a28a5d, 98 },
    { 0, 0 },
    { 0xce928a5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d728a62, 46 },
    { 0x88428a62, 98 },
    { 0x64928a64, 91 },
    { 0x6a428a65, 84 },
    { 0x48128a65, 23 },
    { 0x5b628a62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62728a6b, 90 },
    { 0x51c28a6c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb228a74, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f28a7b, 99 },
    { 0x18428a7c, 46 },
    { 0x1f628a7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x48928a80, 61 },
    { 0, 0 },
    { 0xebd28a82, 76 },
    { 0, 0 },
    { 0x4c28a84, 52 },
    { 0x96d28a85, 90 },
    { 0x86c28a86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef28a8a, 34 },
    { 0xa0e28a8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d928a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b28a93, 98 },
    { 0xd228a93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0828aa1, 93 },
    { 0, 0 },
    { 0x6bb28aa3, 100 },
    { 0x65c28aa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d28aa7, 90 },
    { 0x7928aa8, 17 },
    { 0xc5c28aa9, 90 },
    { 0x7cb28aa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac28aae, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f328ab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e28ab5, 76 },
    { 0x2d928ab5, 98 },
    { 0x46228ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17528abb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e28ac1, 91 },
    { 0xe7d28ac2, 90 },
    { 0x3af28ac1, 33 },
    { 0x88428ac4, 95 },
    { 0x23128ac5, 23 },
    { 0, 0 },
    { 0xa3128ac7, 90 },
    { 0xb0028ac7, 55 },
    { 0x4bb28ac8, 45 },
    { 0, 0 },
    { 0x3a228acb, 76 },
    { 0x50628acc, 78 },
    { 0x41c28acd, 68 },
    { 0x6ca28ace, 99 },
    { 0xe2528acf, 69 },
    { 0x60928ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25228ad3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x19228ad6, 33 },
    { 0x6b528ad7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e428ada, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e28adf, 95 },
    { 0xd5128ae0, 33 },
    { 0, 0 },
    { 0xe9f28ae2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71328ae7, 33 },
    { 0xdef28ae8, 43 },
    { 0xc0028ae9, 75 },
    { 0xbcb28aea, 93 },
    { 0xc2a28ae9, 55 },
    { 0, 0 },
    { 0xbba28aed, 95 },
    { 0xc2328aee, 90 },
    { 0x30128aed, 95 },
    { 0x74928aef, 76 },
    { 0xc9b28aed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x54128af4, 43 },
    { 0x46e28af5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba28afa, 61 },
    { 0x50628afb, 76 },
    { 0x6f228afb, 80 },
    { 0xaa728afc, 61 },
    { 0, 0 },
    { 0xe2128aff, 95 },
    { 0xb3a28aff, 90 },
    { 0xea628b00, 76 },
    { 0xbca28b02, 88 },
    { 0xba928b03, 33 },
    { 0x1e028b03, 68 },
    { 0x4b328b05, 98 },
    { 0xb1f28b06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c28b09, 23 },
    { 0, 0 },
    { 0x39528b0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e428b0f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x38528b12, 95 },
    { 0x13b28b13, 78 },
    { 0x59528b12, 34 },
    { 0, 0 },
    { 0x35c28b16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a28b19, 23 },
    { 0, 0 },
    { 0x5da28b1b, 61 },
    { 0xf9028b1c, 76 },
    { 0xf1028b1c, 76 },
    { 0xa9728b1e, 33 },
    { 0xda28b1f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d28b24, 95 },
    { 0x3cd28b25, 76 },
    { 0x40d28b26, 76 },
    { 0x90d28b27, 98 },
    { 0xa8128b26, 46 },
    { 0xb9828b29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f28b2e, 61 },
    { 0x89128b2f, 33 },
    { 0, 0 },
    { 0xd1528b31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d928b34, 98 },
    { 0x29428b35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b428b38, 98 },
    { 0xc9c28b39, 95 },
    { 0xb8828b38, 95 },
    { 0xfe328b3b, 33 },
    { 0, 0 },
    { 0x4c528b3d, 98 },
    { 0x2a228b3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d28b4c, 93 },
    { 0x12428b4d, 34 },
    { 0xf7a28b4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7428b51, 8 },
    { 0, 0 },
    { 0xdfd28b53, 65 },
    { 0x83428b53, 76 },
    { 0, 0 },
    { 0x2628b56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7128b5a, 100 },
    { 0x72128b5a, 91 },
    { 0x8e328b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f28b60, 33 },
    { 0xa3628b61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15728b6c, 76 },
    { 0, 0 },
    { 0x42128b6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de28b72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46628b76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb28b7d, 68 },
    { 0x50228b7d, 61 },
    { 0x25b28b7f, 95 },
    { 0, 0 },
    { 0x27628b81, 78 },
    { 0x32528b81, 33 },
    { 0x4d928b83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce28b96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5428b99, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb28b9c, 15 },
    { 0xed028b9c, 95 },
    { 0, 0 },
    { 0x54328b9f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f28ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76428ba6, 23 },
    { 0x4ee28ba7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c128baf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce128bb3, 23 },
    { 0xf7e28bb4, 90 },
    { 0x1ee28bb4, 34 },
    { 0x73228bb6, 33 },
    { 0xcf228bb4, 91 },
    { 0xa9f28bb8, 90 },
    { 0x63328bb9, 84 },
    { 0x58828bba, 33 },
    { 0x78528bb8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d128bc2, 90 },
    { 0x74028bc3, 43 },
    { 0, 0 },
    { 0x69f28bc5, 66 },
    { 0, 0 },
    { 0x2e828bc7, 90 },
    { 0xa0328bc8, 85 },
    { 0x36028bc8, 76 },
    { 0, 0 },
    { 0x43a28bcb, 76 },
    { 0, 0 },
    { 0x86028bcd, 95 },
    { 0xf7628bce, 85 },
    { 0x2ee28bcf, 33 },
    { 0x5e928bce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc28bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b628bda, 76 },
    { 0xcd228bdb, 98 },
    { 0, 0 },
    { 0x52728bdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb228be0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d28be3, 23 },
    { 0x2b028be3, 43 },
    { 0xc2a28be4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a28be9, 91 },
    { 0, 0 },
    { 0x79828beb, 33 },
    { 0, 0 },
    { 0xc1128bed, 93 },
    { 0x93f28bee, 98 },
    { 0, 0 },
    { 0xd0528bf0, 23 },
    { 0x86e28bf1, 78 },
    { 0x52b28bf2, 23 },
    { 0xd5128bf2, 36 },
    { 0xb1428bf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef328bfb, 33 },
    { 0, 0 },
    { 0x6828bfd, 90 },
    { 0, 0 },
    { 0xe6128bff, 76 },
    { 0xa7e28bff, 61 },
    { 0, 0 },
    { 0x9fa28c02, 34 },
    { 0x8b828c02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d28c06, 23 },
    { 0xf0b28c07, 90 },
    { 0x70828c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0228c0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea28c12, 93 },
    { 0x2e228c13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59428c17, 88 },
    { 0x6e628c18, 76 },
    { 0x96728c18, 95 },
    { 0x59828c1a, 33 },
    { 0x8a528c18, 55 },
    { 0x98728c17, 55 },
    { 0xf0828c1d, 90 },
    { 0x38128c1d, 95 },
    { 0x5df28c1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab028c28, 100 },
    { 0x45828c29, 98 },
    { 0xe5528c29, 34 },
    { 0x75128c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e828c2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a28c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8028c3d, 61 },
    { 0x55728c3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc28c46, 90 },
    { 0x5fc28c47, 87 },
    { 0, 0 },
    { 0x63a28c49, 90 },
    { 0, 0 },
    { 0x64728c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d28c4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a28c53, 23 },
    { 0x42428c53, 61 },
    { 0xea428c53, 91 },
    { 0x7b28c56, 34 },
    { 0x51228c56, 64 },
    { 0x37028c58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad28c5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c728c61, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc828c65, 52 },
    { 0xc5728c66, 61 },
    { 0x3ea28c67, 90 },
    { 0x6428c66, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec028c6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76828c70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d28c76, 68 },
    { 0, 0 },
    { 0x22528c78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c628c80, 90 },
    { 0x9d328c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd628c8c, 95 },
    { 0x58128c8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24328c91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3528c94, 91 },
    { 0, 0 },
    { 0x1ea28c96, 97 },
    { 0xd2c28c97, 82 },
    { 0xf8d28c96, 17 },
    { 0, 0 },
    { 0x22728c9a, 46 },
    { 0x3ba28c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98128c9f, 90 },
    { 0x74a28ca0, 34 },
    { 0x7dc28ca0, 100 },
    { 0x828c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2028ca5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc28ca8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba28caf, 90 },
    { 0xfec28cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d28cb4, 98 },
    { 0xdac28cb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1128cba, 23 },
    { 0x44f28cbb, 98 },
    { 0, 0 },
    { 0x60e28cbd, 55 },
    { 0, 0 },
    { 0xee428cbf, 99 },
    { 0x7e028cc0, 23 },
    { 0x7a928cc1, 90 },
    { 0x28d28cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab28ccf, 61 },
    { 0, 0 },
    { 0x5b128cd1, 62 },
    { 0x5b828cd1, 23 },
    { 0xbf428cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81828cdb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b928ce1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff428ce5, 90 },
    { 0xa9728ce5, 23 },
    { 0xa0f28ce5, 33 },
    { 0xd2828ce5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb28cee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f28cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38928cf5, 43 },
    { 0x7d228cf5, 65 },
    { 0x24e28cf6, 61 },
    { 0x4f028cf8, 99 },
    { 0x2a628cf9, 76 },
    { 0x90b28cf5, 95 },
    { 0xef328cf7, 93 },
    { 0x5c428cfc, 61 },
    { 0x78728cfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e528d03, 65 },
    { 0, 0 },
    { 0x3f228d05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d28d08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed28d13, 33 },
    { 0xf6928d14, 100 },
    { 0x24128d14, 91 },
    { 0xfec28d16, 66 },
    { 0x7e728d16, 93 },
    { 0x6a828d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce28d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85728d21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd28d25, 65 },
    { 0, 0 },
    { 0xc1928d27, 77 },
    { 0x2ad28d28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b828d2c, 61 },
    { 0x3ab28d2d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a28d32, 78 },
    { 0x9c928d32, 76 },
    { 0x9be28d34, 55 },
    { 0x58d28d33, 61 },
    { 0xa2328d33, 95 },
    { 0, 0 },
    { 0x3e828d38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b28d3d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc028d41, 52 },
    { 0, 0 },
    { 0x34528d43, 95 },
    { 0, 0 },
    { 0x16f28d45, 10 },
    { 0xd5728d46, 46 },
    { 0, 0 },
    { 0x85e28d48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde28d4c, 46 },
    { 0x5a928d4d, 61 },
    { 0x93d28d4e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b28d53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d28d5c, 33 },
    { 0xbc28d5c, 33 },
    { 0, 0 },
    { 0x72f28d5f, 33 },
    { 0xea628d60, 83 },
    { 0x37928d61, 100 },
    { 0xbf628d5f, 96 },
    { 0xe1428d63, 46 },
    { 0, 0 },
    { 0xfe028d65, 96 },
    { 0x2c628d66, 68 },
    { 0, 0 },
    { 0x45428d68, 90 },
    { 0x8e028d68, 91 },
    { 0x86028d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc28d6e, 84 },
    { 0xcfc28d6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b28d78, 77 },
    { 0x24a28d79, 98 },
    { 0, 0 },
    { 0x3e828d7b, 76 },
    { 0, 0 },
    { 0xc2328d7d, 61 },
    { 0xfee28d7e, 69 },
    { 0xaf628d7f, 77 },
    { 0x7e28d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94528d84, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a28d8a, 66 },
    { 0x37f28d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa528d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c28da0, 55 },
    { 0, 0 },
    { 0x6c928da2, 90 },
    { 0x40228da3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc028da7, 81 },
    { 0, 0 },
    { 0x59828da9, 33 },
    { 0, 0 },
    { 0x89628dab, 34 },
    { 0x78628dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e428db1, 61 },
    { 0x64828db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e628db6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd028dbc, 34 },
    { 0, 0 },
    { 0x7d128dbe, 66 },
    { 0x16628dbf, 68 },
    { 0x9ce28dc0, 61 },
    { 0xf8e28dc0, 90 },
    { 0xde28dc2, 87 },
    { 0x4ec28dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd828dc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f328dcb, 23 },
    { 0x95f28dcb, 65 },
    { 0x9a928dcd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a28dd4, 98 },
    { 0, 0 },
    { 0xd4028dd6, 98 },
    { 0xe4128dd6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61328dda, 95 },
    { 0, 0 },
    { 0x8aa28ddc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67928de0, 33 },
    { 0, 0 },
    { 0xa7f28de2, 65 },
    { 0, 0 },
    { 0xac828de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6528de7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43128dee, 66 },
    { 0x3b128dee, 61 },
    { 0, 0 },
    { 0x71628df1, 98 },
    { 0, 0 },
    { 0x25828df3, 46 },
    { 0, 0 },
    { 0x9d628df5, 96 },
    { 0, 0 },
    { 0x1b728df7, 90 },
    { 0xeba28df7, 76 },
    { 0, 0 },
    { 0x6ce28dfa, 33 },
    { 0x81128dfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e28dff, 90 },
    { 0xeb828dff, 17 },
    { 0x1c728e01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45728e0d, 90 },
    { 0xd6f28e0d, 88 },
    { 0x8ff28e0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7628e14, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x23428e17, 90 },
    { 0, 0 },
    { 0xed028e19, 100 },
    { 0, 0 },
    { 0x5d428e1b, 23 },
    { 0, 0 },
    { 0x90928e1d, 95 },
    { 0, 0 },
    { 0xdc428e1f, 90 },
    { 0x9a828e20, 84 },
    { 0, 0 },
    { 0xf0d28e22, 99 },
    { 0x76c28e23, 33 },
    { 0x8dc28e22, 33 },
    { 0xee928e24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee28e2c, 55 },
    { 0x53f28e2d, 88 },
    { 0x4928e2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba28e31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69828e38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6528e3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d28e43, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc28e47, 33 },
    { 0, 0 },
    { 0xb5928e49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd828e55, 76 },
    { 0x44928e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d28e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e28e5f, 40 },
    { 0x3c628e60, 96 },
    { 0x90228e61, 78 },
    { 0, 0 },
    { 0x2e728e63, 95 },
    { 0, 0 },
    { 0xc228e65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x31028e68, 98 },
    { 0x5328e68, 98 },
    { 0x14b28e68, 43 },
    { 0x83028e6b, 87 },
    { 0, 0 },
    { 0x20528e6d, 34 },
    { 0xb4728e6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c328e73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8828e77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61728e84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e28e89, 23 },
    { 0x86328e89, 61 },
    { 0x4e628e8a, 84 },
    { 0x3ac28e8c, 95 },
    { 0x1aa28e8d, 33 },
    { 0x55028e8e, 88 },
    { 0xff228e8f, 100 },
    { 0x85728e90, 95 },
    { 0xd2128e90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32928e95, 99 },
    { 0x10928e96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a428e9e, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x77328ea1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35628ea7, 61 },
    { 0x21e28ea8, 90 },
    { 0xc3028ea9, 83 },
    { 0xe2d28ea9, 55 },
    { 0, 0 },
    { 0xc1f28eac, 43 },
    { 0x37f28eac, 77 },
    { 0xb8428eae, 34 },
    { 0xde28eaf, 61 },
    { 0x8d128eac, 33 },
    { 0xe3228eb1, 33 },
    { 0x40b28eae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13828eb7, 61 },
    { 0, 0 },
    { 0x32f28eb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc28ebc, 90 },
    { 0xed028ebd, 43 },
    { 0x28a28ebe, 95 },
    { 0x26128ebf, 90 },
    { 0x25928ebd, 90 },
    { 0, 0 },
    { 0xf28ec2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95928ec6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2128ec9, 90 },
    { 0, 0 },
    { 0xa2328ecb, 93 },
    { 0x7d528ecc, 84 },
    { 0x2d728ecd, 90 },
    { 0x12f28ecd, 34 },
    { 0x9ee28ecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95628edd, 43 },
    { 0xa6828ede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34428ee1, 55 },
    { 0x33128ee2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9728ee6, 90 },
    { 0x11b28ee7, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a28eec, 76 },
    { 0, 0 },
    { 0xfd228eee, 23 },
    { 0, 0 },
    { 0xebb28ef0, 85 },
    { 0x5c28ef1, 34 },
    { 0xfb928ef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d28ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38028efc, 76 },
    { 0xd9928efc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f228f03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e28f07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc28f0e, 61 },
    { 0xa1528f0f, 61 },
    { 0x88f28f10, 46 },
    { 0, 0 },
    { 0x19e28f12, 95 },
    { 0xe5928f12, 91 },
    { 0x3fa28f13, 98 },
    { 0, 0 },
    { 0x50a28f16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a28f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11028f1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe28f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f28f23, 95 },
    { 0xa9628f24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42728f28, 34 },
    { 0xf2b28f29, 90 },
    { 0x20128f2a, 99 },
    { 0xba828f2a, 55 },
    { 0xca928f29, 95 },
    { 0x39328f2b, 90 },
    { 0xb2628f2d, 90 },
    { 0xeaa28f2f, 34 },
    { 0x56e28f30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99328f37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62728f3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b28f3f, 61 },
    { 0x20c28f40, 100 },
    { 0, 0 },
    { 0xc1428f42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1428f46, 33 },
    { 0, 0 },
    { 0x6e928f48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad028f4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9028f59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11928f5d, 91 },
    { 0xfdf28f5e, 23 },
    { 0, 0 },
    { 0x85828f60, 78 },
    { 0, 0 },
    { 0xd8928f62, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x63728f65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb28f6a, 100 },
    { 0, 0 },
    { 0x1d728f6c, 95 },
    { 0x39128f6d, 87 },
    { 0x56028f6c, 33 },
    { 0x79828f6c, 33 },
    { 0xd7b28f70, 92 },
    { 0x9b828f6d, 23 },
    { 0x38428f72, 98 },
    { 0x61c28f73, 98 },
    { 0x5e028f73, 95 },
    { 0x10128f6c, 90 },
    { 0x9ab28f76, 61 },
    { 0xe6128f77, 34 },
    { 0x64428f77, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f28f7b, 85 },
    { 0x84e28f7c, 23 },
    { 0x3e728f7d, 33 },
    { 0xe0828f7e, 90 },
    { 0x89928f7f, 93 },
    { 0x5f628f80, 34 },
    { 0x95428f7f, 95 },
    { 0xfad28f82, 61 },
    { 0x50c28f7c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac28f89, 17 },
    { 0x94f28f89, 55 },
    { 0xfe628f8b, 38 },
    { 0, 0 },
    { 0x74928f8d, 61 },
    { 0x4a028f8e, 93 },
    { 0x81628f8e, 98 },
    { 0, 0 },
    { 0x7af28f91, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef428f9f, 24 },
    { 0xc1728fa0, 61 },
    { 0x83a28fa1, 33 },
    { 0xcea28fa2, 90 },
    { 0x44f28fa2, 95 },
    { 0xe5928fa2, 61 },
    { 0xed528fa4, 90 },
    { 0x49a28fa6, 33 },
    { 0x54328fa7, 91 },
    { 0x5528fa8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34928fab, 90 },
    { 0, 0 },
    { 0x49b28fad, 76 },
    { 0x49a28fae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48828fb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x10328fb4, 52 },
    { 0xcca28fb5, 43 },
    { 0, 0 },
    { 0x9a928fb7, 80 },
    { 0xfdd28fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99728fbc, 61 },
    { 0, 0 },
    { 0x61d28fbe, 90 },
    { 0, 0 },
    { 0x9d928fc0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30328fc7, 23 },
    { 0xbba28fc8, 74 },
    { 0, 0 },
    { 0x6e228fca, 95 },
    { 0xa4f28fca, 33 },
    { 0xcc328fcc, 52 },
    { 0, 0 },
    { 0xa228fce, 90 },
    { 0xf8728fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee28fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c28fd8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef028fe5, 40 },
    { 0xa4928fe5, 98 },
    { 0xb8628fe5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x62028fea, 43 },
    { 0, 0 },
    { 0xe1a28fec, 76 },
    { 0x3e028fed, 61 },
    { 0, 0 },
    { 0x5fe28fef, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92828ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3228ffb, 95 },
    { 0, 0 },
    { 0x3ed28ffd, 76 },
    { 0, 0 },
    { 0xa6e28fff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd29008, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34829013, 33 },
    { 0, 0 },
    { 0x90c29015, 98 },
    { 0xf3329015, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512901b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc062901f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda129023, 90 },
    { 0x9d29024, 93 },
    { 0x6ec29024, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d29029, 98 },
    { 0xc342902a, 91 },
    { 0xf4229029, 93 },
    { 0x522902a, 67 },
    { 0xe8c2902d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99529032, 87 },
    { 0x1c229033, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba2903e, 67 },
    { 0, 0 },
    { 0x68229040, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28929044, 98 },
    { 0, 0 },
    { 0x48829046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3852904e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5629051, 61 },
    { 0x8c329051, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe29055, 90 },
    { 0xc8029055, 61 },
    { 0x7a529057, 95 },
    { 0xaf429058, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f29060, 95 },
    { 0xea629061, 61 },
    { 0xeb529062, 95 },
    { 0x90329060, 33 },
    { 0x2ac29062, 95 },
    { 0, 0 },
    { 0xe0f29066, 76 },
    { 0x9e929067, 90 },
    { 0x89629068, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc32906f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a29077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa312907c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5282907f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f29084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5029087, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d72908b, 56 },
    { 0xbe82908c, 97 },
    { 0x4b72908d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec929091, 90 },
    { 0, 0 },
    { 0x18129093, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f29096, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c2909a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd322909d, 34 },
    { 0x3302909e, 33 },
    { 0, 0 },
    { 0x6c9290a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1290a8, 23 },
    { 0, 0 },
    { 0x44f290aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355290af, 95 },
    { 0x83c290af, 33 },
    { 0x8ab290b1, 33 },
    { 0x754290b0, 98 },
    { 0xde6290b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7290ba, 100 },
    { 0, 0 },
    { 0x4fc290bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc290c0, 40 },
    { 0x986290c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a290c7, 67 },
    { 0x361290c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769290cf, 43 },
    { 0xe78290cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538290d4, 53 },
    { 0xb9e290d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00290d9, 66 },
    { 0x4b2290d9, 90 },
    { 0x7c8290db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8290e0, 34 },
    { 0xd45290e1, 81 },
    { 0x36d290e2, 90 },
    { 0x818290e3, 23 },
    { 0xfc5290e4, 91 },
    { 0x63b290e5, 68 },
    { 0xb98290e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632290eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547290ef, 93 },
    { 0, 0 },
    { 0x141290f1, 66 },
    { 0, 0 },
    { 0x77e290f3, 23 },
    { 0x4d0290f4, 90 },
    { 0xed2290f5, 61 },
    { 0x858290f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0029103, 90 },
    { 0xe5929104, 23 },
    { 0xac129105, 61 },
    { 0x13129104, 34 },
    { 0x68d29107, 61 },
    { 0x17029108, 33 },
    { 0x4c029104, 95 },
    { 0xdfd29104, 95 },
    { 0x9f229107, 61 },
    { 0x7962910c, 61 },
    { 0x83c2910c, 98 },
    { 0x97c2910e, 76 },
    { 0xc712910d, 61 },
    { 0xf3229110, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d29113, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7342911c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd02911f, 95 },
    { 0x98729120, 55 },
    { 0x4f429121, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a29125, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85029129, 93 },
    { 0xc192912a, 90 },
    { 0x4022912b, 23 },
    { 0x16d2912a, 33 },
    { 0xca92912d, 52 },
    { 0, 0 },
    { 0x71f2912f, 33 },
    { 0xb3e29130, 17 },
    { 0x8bb29131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7629135, 52 },
    { 0xda229135, 33 },
    { 0xdb729135, 61 },
    { 0x3a729138, 34 },
    { 0xbe629137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f82913c, 33 },
    { 0x1cd2913d, 90 },
    { 0x7982913c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc229142, 61 },
    { 0x65229142, 61 },
    { 0, 0 },
    { 0x13929145, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e29149, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc29156, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec29161, 76 },
    { 0xbb329161, 76 },
    { 0xe29163, 33 },
    { 0xfc29164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd482916a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91329170, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x49829173, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95529177, 61 },
    { 0xb6629178, 66 },
    { 0x56d29177, 34 },
    { 0x7232917a, 99 },
    { 0x8432917b, 61 },
    { 0x11a2917b, 90 },
    { 0x9e52917d, 95 },
    { 0x25f2917e, 90 },
    { 0xb372917f, 90 },
    { 0, 0 },
    { 0x3a629181, 90 },
    { 0x15029182, 76 },
    { 0xb8129182, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab29189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c429191, 55 },
    { 0xb3c29192, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f29198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d291a1, 33 },
    { 0xddb291a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f291a5, 91 },
    { 0, 0 },
    { 0x94f291a7, 77 },
    { 0, 0 },
    { 0xe71291a9, 34 },
    { 0x9b7291aa, 36 },
    { 0xa9e291ab, 98 },
    { 0x37f291ac, 93 },
    { 0x8f291a9, 84 },
    { 0, 0 },
    { 0x719291af, 99 },
    { 0x550291af, 43 },
    { 0, 0 },
    { 0x700291b2, 23 },
    { 0xbf8291b3, 76 },
    { 0x5c291b2, 76 },
    { 0x6a7291b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9291b9, 61 },
    { 0x362291b9, 55 },
    { 0x69c291bb, 76 },
    { 0x547291bb, 98 },
    { 0, 0 },
    { 0x3f6291be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2291c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9291ce, 98 },
    { 0x47b291ce, 87 },
    { 0xbee291d0, 23 },
    { 0xb6f291d0, 90 },
    { 0, 0 },
    { 0xd2a291d3, 98 },
    { 0x402291d3, 66 },
    { 0, 0 },
    { 0x622291d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba291d9, 91 },
    { 0x5e6291da, 66 },
    { 0xbe8291d9, 52 },
    { 0x2a5291dc, 95 },
    { 0x626291dc, 87 },
    { 0, 0 },
    { 0xd6e291df, 66 },
    { 0x67e291df, 100 },
    { 0xfe3291df, 81 },
    { 0x7c291e2, 76 },
    { 0xaf6291e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b291e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211291ed, 52 },
    { 0xdaf291ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe291f4, 98 },
    { 0x42c291f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3291fc, 55 },
    { 0xc5a291fd, 90 },
    { 0, 0 },
    { 0x354291ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd929205, 68 },
    { 0x6ce29206, 88 },
    { 0x85329207, 87 },
    { 0, 0 },
    { 0x6bd29209, 23 },
    { 0x1ce2920a, 55 },
    { 0x76c2920a, 55 },
    { 0, 0 },
    { 0xcdb2920d, 33 },
    { 0x6582920d, 85 },
    { 0, 0 },
    { 0x87129210, 34 },
    { 0xe1629210, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a29215, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b29218, 76 },
    { 0x27929218, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8029223, 61 },
    { 0x86129224, 61 },
    { 0x6c629225, 98 },
    { 0, 0 },
    { 0x19e29227, 87 },
    { 0x9d629227, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2922f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c729233, 33 },
    { 0xfca29234, 95 },
    { 0, 0 },
    { 0x3d429236, 95 },
    { 0, 0 },
    { 0xa5a29238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e2923b, 33 },
    { 0xe12923c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2923f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c29242, 23 },
    { 0x26329243, 95 },
    { 0xa0c29243, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15729248, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5352924e, 68 },
    { 0, 0 },
    { 0xb2529250, 61 },
    { 0x21c29251, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xab829254, 65 },
    { 0xbd629255, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca2925a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8662925e, 90 },
    { 0xaa72925e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2129262, 55 },
    { 0x17829263, 23 },
    { 0xff29262, 23 },
    { 0xd4b29264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f2926d, 45 },
    { 0x552926e, 33 },
    { 0x1822926f, 87 },
    { 0, 0 },
    { 0x6ef29271, 76 },
    { 0xbff29272, 95 },
    { 0, 0 },
    { 0x1b329274, 38 },
    { 0xfe029274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b29278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5452927e, 98 },
    { 0x2b32927f, 77 },
    { 0x8922927f, 33 },
    { 0xc3629281, 23 },
    { 0x1b02927e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a029285, 96 },
    { 0x3fa29285, 100 },
    { 0xe5429286, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b2928c, 33 },
    { 0, 0 },
    { 0x9c62928e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d29291, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf29295, 93 },
    { 0x9b629296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9729299, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2929e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a292a2, 33 },
    { 0, 0 },
    { 0x326292a4, 87 },
    { 0xa2b292a5, 76 },
    { 0x45f292a5, 98 },
    { 0x42f292a6, 91 },
    { 0x3c3292a6, 23 },
    { 0xfae292a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1292ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d292af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b292b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x426292ba, 23 },
    { 0, 0 },
    { 0x587292bc, 76 },
    { 0x694292bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e292c6, 46 },
    { 0xecc292c7, 67 },
    { 0, 0 },
    { 0xdf292c9, 90 },
    { 0x1c0292ca, 33 },
    { 0x324292cb, 98 },
    { 0x7a9292cb, 90 },
    { 0x683292ca, 90 },
    { 0x704292ce, 55 },
    { 0x41b292ce, 76 },
    { 0x306292cf, 98 },
    { 0xb57292d1, 95 },
    { 0x53d292d1, 90 },
    { 0x782292d3, 40 },
    { 0, 0 },
    { 0xd74292d5, 33 },
    { 0x75c292d6, 99 },
    { 0, 0 },
    { 0x177292d8, 23 },
    { 0, 0 },
    { 0x8db292da, 33 },
    { 0xacd292db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b292df, 61 },
    { 0x3a2292df, 55 },
    { 0xf30292e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f292ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd292f4, 95 },
    { 0, 0 },
    { 0x5b8292f6, 55 },
    { 0xf91292f7, 34 },
    { 0xb11292f7, 98 },
    { 0x5c8292f9, 46 },
    { 0x3dd292f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0292ff, 55 },
    { 0xa80292ff, 98 },
    { 0xf4f29301, 95 },
    { 0xfa229302, 90 },
    { 0xb3629302, 95 },
    { 0x36229304, 61 },
    { 0xe2029302, 33 },
    { 0xa5229303, 76 },
    { 0x81829307, 61 },
    { 0x84629308, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3482930b, 43 },
    { 0x3662930c, 43 },
    { 0, 0 },
    { 0x2192930e, 90 },
    { 0, 0 },
    { 0x77029310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8729319, 100 },
    { 0xdff2931a, 99 },
    { 0x8da2931b, 90 },
    { 0xe282931b, 88 },
    { 0x59d2931d, 55 },
    { 0xbe2931c, 61 },
    { 0x8b22931f, 76 },
    { 0x5ab2931b, 55 },
    { 0xcf02931a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1929324, 61 },
    { 0xcab29324, 87 },
    { 0x85029326, 93 },
    { 0x97729327, 33 },
    { 0xfa229326, 66 },
    { 0xe9729329, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c2932d, 90 },
    { 0, 0 },
    { 0x512932f, 91 },
    { 0, 0 },
    { 0xa7d29331, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x96229334, 95 },
    { 0, 0 },
    { 0x8c929336, 75 },
    { 0, 0 },
    { 0xdb129338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f129344, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e29353, 90 },
    { 0, 0 },
    { 0x48629355, 91 },
    { 0x3cc29355, 67 },
    { 0, 0 },
    { 0x30129358, 99 },
    { 0, 0 },
    { 0x5ee2935a, 90 },
    { 0xd4d2935b, 61 },
    { 0xed82935c, 98 },
    { 0x6d92935d, 90 },
    { 0xca22935e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae29367, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb432936a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa29370, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a29374, 89 },
    { 0x79e29375, 61 },
    { 0x84f29376, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff2937a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc429382, 84 },
    { 0xf6729383, 52 },
    { 0x59e29384, 90 },
    { 0xa2429385, 80 },
    { 0xa629383, 80 },
    { 0, 0 },
    { 0xd2629388, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea2938e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c629391, 61 },
    { 0xa2e29391, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56029398, 98 },
    { 0, 0 },
    { 0x9272939a, 75 },
    { 0x472939b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x242939e, 23 },
    { 0x93a2939f, 85 },
    { 0, 0 },
    { 0x8cf293a1, 90 },
    { 0, 0 },
    { 0xfbf293a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621293ac, 61 },
    { 0xf58293ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4293b8, 85 },
    { 0x861293b9, 77 },
    { 0x322293ba, 100 },
    { 0xf77293bb, 90 },
    { 0x910293ba, 33 },
    { 0, 0 },
    { 0xbf0293be, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3293c1, 61 },
    { 0, 0 },
    { 0xb7d293c3, 81 },
    { 0x413293c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592293c9, 61 },
    { 0, 0 },
    { 0x59a293cb, 43 },
    { 0xb48293cc, 33 },
    { 0x827293cc, 76 },
    { 0xbb7293ce, 78 },
    { 0xf36293cf, 33 },
    { 0x97f293d0, 90 },
    { 0x189293d1, 78 },
    { 0x22293d1, 43 },
    { 0xbe6293d2, 91 },
    { 0x3fe293d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7293d8, 61 },
    { 0xc34293d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643293df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54293e3, 90 },
    { 0xf56293e4, 33 },
    { 0x5a7293e4, 95 },
    { 0, 0 },
    { 0x57293e7, 66 },
    { 0, 0 },
    { 0x869293e9, 90 },
    { 0xa65293ea, 23 },
    { 0x34b293eb, 100 },
    { 0x867293ea, 95 },
    { 0, 0 },
    { 0x562293ee, 23 },
    { 0xc5c293ee, 56 },
    { 0, 0 },
    { 0x6c2293f1, 94 },
    { 0, 0 },
    { 0xc2e293f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d293f9, 68 },
    { 0, 0 },
    { 0x1bc293fb, 61 },
    { 0x716293fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb29406, 90 },
    { 0, 0 },
    { 0xfb329408, 79 },
    { 0, 0 },
    { 0xbf2940a, 45 },
    { 0x1022940b, 66 },
    { 0, 0 },
    { 0xd892940d, 33 },
    { 0xec02940d, 33 },
    { 0x6112940e, 61 },
    { 0xbcd29410, 85 },
    { 0xd4729411, 90 },
    { 0, 0 },
    { 0xd6e29413, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d29418, 66 },
    { 0x86829418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf372941c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d29422, 23 },
    { 0, 0 },
    { 0xdeb29424, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e2942a, 33 },
    { 0x4382942b, 90 },
    { 0x5dd2942c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c29430, 78 },
    { 0x78129431, 23 },
    { 0x3ef29432, 75 },
    { 0x81a29433, 76 },
    { 0xa2b29430, 33 },
    { 0xf5529433, 33 },
    { 0xf4d29433, 23 },
    { 0x2ed29431, 55 },
    { 0x9dc29438, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc72943c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d29440, 34 },
    { 0x2ba29441, 90 },
    { 0, 0 },
    { 0xc4f29443, 51 },
    { 0xda29444, 76 },
    { 0, 0 },
    { 0x1cb29446, 23 },
    { 0x3af29447, 61 },
    { 0x7e629447, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b02944c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd902944f, 36 },
    { 0xac629450, 93 },
    { 0, 0 },
    { 0xd5a29452, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6629455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c2945b, 38 },
    { 0x2622945c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87929465, 95 },
    { 0, 0 },
    { 0x4529467, 93 },
    { 0x9a029467, 87 },
    { 0, 0 },
    { 0xc0a2946a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab12946d, 77 },
    { 0x9042946d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11029472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27129475, 33 },
    { 0xf1b29476, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9622947a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8382947d, 100 },
    { 0x1a52947d, 98 },
    { 0x4f82947f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64129484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11229489, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf02948e, 33 },
    { 0x3872948e, 80 },
    { 0, 0 },
    { 0xac429491, 23 },
    { 0xae329492, 98 },
    { 0xdb529492, 98 },
    { 0x5cd29494, 81 },
    { 0x67c29493, 61 },
    { 0xb529496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d2949e, 97 },
    { 0, 0 },
    { 0xa8f294a0, 23 },
    { 0, 0 },
    { 0xe3d294a2, 77 },
    { 0xbdb294a3, 98 },
    { 0x800294a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8294a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f294ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7294b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df294b5, 46 },
    { 0, 0 },
    { 0x42a294b7, 87 },
    { 0x9b0294b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95294bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928294c1, 17 },
    { 0, 0 },
    { 0x6df294c3, 88 },
    { 0xb72294c4, 61 },
    { 0, 0 },
    { 0x80d294c6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x338294c9, 55 },
    { 0xdc6294ca, 52 },
    { 0x99a294cb, 23 },
    { 0, 0 },
    { 0xb8c294cd, 17 },
    { 0xecb294ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc294d2, 65 },
    { 0x48294d3, 61 },
    { 0, 0 },
    { 0x6a1294d5, 84 },
    { 0xe0e294d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64294dd, 95 },
    { 0x8e4294de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f294e2, 87 },
    { 0x4da294e2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a294e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46294ef, 88 },
    { 0xa53294f0, 33 },
    { 0xcef294ef, 23 },
    { 0, 0 },
    { 0x814294f3, 66 },
    { 0xac4294f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15294f8, 88 },
    { 0xc5d294f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a294fd, 33 },
    { 0xb2d294fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x91729501, 46 },
    { 0x5fb29501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f529507, 55 },
    { 0x6f229507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d2950b, 33 },
    { 0x2742950c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a29512, 33 },
    { 0x8a829513, 46 },
    { 0x50429512, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0929517, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b12951f, 99 },
    { 0xcd729520, 90 },
    { 0x24429521, 61 },
    { 0x39129522, 69 },
    { 0xef629523, 33 },
    { 0, 0 },
    { 0xe3729525, 52 },
    { 0xb9f29526, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252952e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1529535, 68 },
    { 0x35629536, 34 },
    { 0x7429537, 90 },
    { 0, 0 },
    { 0xdd729539, 98 },
    { 0x9ad29539, 90 },
    { 0x7d62953b, 33 },
    { 0, 0 },
    { 0xdf42953d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1329542, 95 },
    { 0xfbb29543, 61 },
    { 0x23229544, 95 },
    { 0xa2629542, 61 },
    { 0, 0 },
    { 0x21429547, 61 },
    { 0xe9029548, 90 },
    { 0x11929548, 85 },
    { 0x8129548, 23 },
    { 0x87729548, 33 },
    { 0, 0 },
    { 0xdbd2954d, 90 },
    { 0, 0 },
    { 0xf872954f, 98 },
    { 0xd2b29550, 77 },
    { 0x70129551, 98 },
    { 0x18929552, 67 },
    { 0x24f29552, 55 },
    { 0xb1829552, 64 },
    { 0x6af29552, 33 },
    { 0x80529556, 93 },
    { 0x26f29556, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7732955d, 95 },
    { 0xd5d2955e, 64 },
    { 0x90c2955e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a629564, 91 },
    { 0, 0 },
    { 0x2c329566, 95 },
    { 0x9ff29567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8062956a, 90 },
    { 0x81b2956b, 100 },
    { 0x5a32956b, 33 },
    { 0x51e2956c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82729571, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40729574, 95 },
    { 0x6d629574, 99 },
    { 0x7e029576, 75 },
    { 0x9cc29577, 99 },
    { 0x8fe29578, 33 },
    { 0xec129576, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8342957d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29580, 23 },
    { 0x8be29581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb29587, 33 },
    { 0x8b729587, 76 },
    { 0, 0 },
    { 0xea12958a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71029590, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98429594, 34 },
    { 0xb4629594, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb262959c, 38 },
    { 0x96a2959c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38295a1, 33 },
    { 0x7d9295a1, 75 },
    { 0, 0 },
    { 0xb72295a4, 33 },
    { 0xf5f295a5, 23 },
    { 0x12c295a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1295aa, 76 },
    { 0, 0 },
    { 0xbfa295ac, 23 },
    { 0, 0 },
    { 0x6ab295ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce295ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0295bd, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x407295c0, 89 },
    { 0x226295c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1295c8, 90 },
    { 0x56e295c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e295cf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x485295d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff295d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1295dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e295df, 55 },
    { 0x28a295e0, 17 },
    { 0x830295e1, 95 },
    { 0, 0 },
    { 0xa5a295e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03295ec, 90 },
    { 0xbd5295ed, 90 },
    { 0x66e295ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25295f1, 95 },
    { 0, 0 },
    { 0x826295f3, 61 },
    { 0, 0 },
    { 0xb08295f5, 33 },
    { 0xf98295f5, 33 },
    { 0x904295f5, 55 },
    { 0, 0 },
    { 0xda295f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae295fe, 76 },
    { 0xf21295ff, 99 },
    { 0, 0 },
    { 0xd3b29601, 46 },
    { 0x11a29602, 23 },
    { 0x4a729602, 91 },
    { 0x78b29604, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62529609, 61 },
    { 0x1da29609, 23 },
    { 0x7482960b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b29611, 43 },
    { 0, 0 },
    { 0xc829613, 66 },
    { 0, 0 },
    { 0x5ed29615, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2029619, 23 },
    { 0, 0 },
    { 0x1212961b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f42961f, 46 },
    { 0xd9829620, 93 },
    { 0x32e2961f, 61 },
    { 0x2db29622, 90 },
    { 0x6129620, 90 },
    { 0x43229621, 95 },
    { 0x4c029625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc429629, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe472962d, 33 },
    { 0x26b2962e, 99 },
    { 0x6092962f, 95 },
    { 0xd0729630, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a929635, 55 },
    { 0, 0 },
    { 0x3ca29637, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f2963a, 33 },
    { 0, 0 },
    { 0x17b2963c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b2963f, 90 },
    { 0x1dc2963f, 98 },
    { 0, 0 },
    { 0x78829642, 86 },
    { 0xc3229642, 17 },
    { 0xe2e29643, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65929648, 90 },
    { 0xd7529649, 61 },
    { 0x5d229649, 84 },
    { 0xd4b2964b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x352964e, 33 },
    { 0xbdc2964e, 95 },
    { 0x3d129650, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e29654, 55 },
    { 0x94729655, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x31129658, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc782965b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85729661, 68 },
    { 0x48a29662, 87 },
    { 0x25429663, 61 },
    { 0xd5e29661, 55 },
    { 0xd7d29665, 46 },
    { 0x96029666, 66 },
    { 0x38329666, 78 },
    { 0x9ca29668, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b29670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7229677, 61 },
    { 0x45629678, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff2967c, 97 },
    { 0x4d82967d, 36 },
    { 0, 0 },
    { 0x23e2967f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x84529682, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6129686, 100 },
    { 0, 0 },
    { 0xb7529688, 43 },
    { 0, 0 },
    { 0x2372968a, 33 },
    { 0, 0 },
    { 0xec2968c, 98 },
    { 0, 0 },
    { 0xf9e2968e, 100 },
    { 0x4f02968f, 33 },
    { 0, 0 },
    { 0x42729691, 90 },
    { 0xa7e29691, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f529695, 55 },
    { 0xc5229695, 91 },
    { 0xb9729697, 33 },
    { 0, 0 },
    { 0xfc429699, 98 },
    { 0xdfb29699, 78 },
    { 0x5392969b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5296a8, 43 },
    { 0xac6296a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90296b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f296b5, 95 },
    { 0x77d296b5, 23 },
    { 0x4c9296b7, 83 },
    { 0x52296b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267296bd, 43 },
    { 0x213296be, 95 },
    { 0xb8a296bf, 98 },
    { 0, 0 },
    { 0x522296c1, 55 },
    { 0xd1c296c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe296cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd296d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6296d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x619296da, 34 },
    { 0xdc4296da, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359296e5, 90 },
    { 0xbd8296e6, 99 },
    { 0x7d8296e7, 33 },
    { 0x793296e7, 76 },
    { 0, 0 },
    { 0xeba296ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3296ed, 95 },
    { 0xf74296ed, 40 },
    { 0xe78296ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a296f5, 61 },
    { 0, 0 },
    { 0x898296f7, 88 },
    { 0, 0 },
    { 0x26c296f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d296fc, 61 },
    { 0xc9296fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67329707, 90 },
    { 0x6a129707, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9332970e, 55 },
    { 0x3de2970f, 34 },
    { 0xb1a29710, 23 },
    { 0xd0a2970f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e29716, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e29719, 33 },
    { 0x13429719, 90 },
    { 0x36e2971b, 90 },
    { 0xbba2971c, 55 },
    { 0x1702971d, 55 },
    { 0x5c62971e, 43 },
    { 0, 0 },
    { 0x3d529720, 66 },
    { 0, 0 },
    { 0x82629722, 91 },
    { 0, 0 },
    { 0x39e29724, 23 },
    { 0, 0 },
    { 0xd4129726, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d2972a, 23 },
    { 0x2e22972b, 99 },
    { 0x3e42972a, 95 },
    { 0x3322972d, 98 },
    { 0xd542972d, 34 },
    { 0, 0 },
    { 0xe3f29730, 33 },
    { 0xd2029731, 97 },
    { 0x40629732, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b29735, 99 },
    { 0, 0 },
    { 0x56a29737, 98 },
    { 0, 0 },
    { 0xd0429739, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9852973c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf529746, 53 },
    { 0, 0 },
    { 0x99029748, 43 },
    { 0, 0 },
    { 0x3442974a, 61 },
    { 0, 0 },
    { 0x112974c, 23 },
    { 0x3312974c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4229754, 33 },
    { 0xae229754, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67629759, 90 },
    { 0xb8729759, 23 },
    { 0xbf82975b, 34 },
    { 0x5572975c, 61 },
    { 0x1ee2975c, 90 },
    { 0, 0 },
    { 0xd682975f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb429763, 91 },
    { 0xd5029763, 100 },
    { 0xfd629763, 85 },
    { 0x95329766, 61 },
    { 0x1b29766, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d32976d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41229770, 55 },
    { 0x2d129770, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262977f, 95 },
    { 0, 0 },
    { 0x6cb29781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a29785, 61 },
    { 0xd2529785, 81 },
    { 0, 0 },
    { 0xc5229788, 33 },
    { 0x86929789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd29790, 99 },
    { 0, 0 },
    { 0x7c429792, 33 },
    { 0, 0 },
    { 0x9a829794, 91 },
    { 0x1ab29795, 33 },
    { 0, 0 },
    { 0xea629797, 55 },
    { 0x9de29797, 33 },
    { 0xeb229799, 95 },
    { 0x6eb2979a, 100 },
    { 0, 0 },
    { 0x85f2979c, 90 },
    { 0x6152979c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e297a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547297a9, 23 },
    { 0xeec297a9, 23 },
    { 0x141297a9, 90 },
    { 0x8c2297ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d297b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4297b5, 23 },
    { 0x5fa297b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede297ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888297bf, 98 },
    { 0x5e3297bf, 98 },
    { 0x7ef297c1, 23 },
    { 0xabb297c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4297c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53297cc, 23 },
    { 0x5f297cd, 55 },
    { 0x591297ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4297d1, 98 },
    { 0x368297d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23297d9, 98 },
    { 0x1be297d9, 51 },
    { 0x870297d9, 61 },
    { 0x83e297dc, 90 },
    { 0xf5d297dd, 33 },
    { 0x57b297de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f297e1, 98 },
    { 0x7bf297e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b297e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0297ed, 90 },
    { 0x66d297ee, 76 },
    { 0xa40297ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x199297f2, 76 },
    { 0xdaf297f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254297f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x268297fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83297fe, 90 },
    { 0, 0 },
    { 0xdc229800, 90 },
    { 0, 0 },
    { 0x69b29802, 92 },
    { 0xce829803, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae529806, 76 },
    { 0x67229807, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f62980a, 61 },
    { 0, 0 },
    { 0x8212980c, 33 },
    { 0, 0 },
    { 0x1f32980e, 95 },
    { 0x1792980f, 53 },
    { 0x24029810, 23 },
    { 0xd429811, 88 },
    { 0, 0 },
    { 0x57b29813, 55 },
    { 0xa1f29814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c29817, 23 },
    { 0xf8229817, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda92981d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc929820, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f29824, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc242982c, 100 },
    { 0, 0 },
    { 0xa532982e, 90 },
    { 0, 0 },
    { 0x52129830, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c29834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9272983a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9282983d, 95 },
    { 0xd262983e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b29843, 61 },
    { 0, 0 },
    { 0x51e29845, 52 },
    { 0, 0 },
    { 0xbae29847, 98 },
    { 0, 0 },
    { 0x3c329849, 33 },
    { 0, 0 },
    { 0x2e42984b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa29850, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3929853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce529859, 46 },
    { 0xa9c2985a, 61 },
    { 0, 0 },
    { 0xfa22985c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb29861, 64 },
    { 0xc4a29861, 46 },
    { 0xb5829863, 98 },
    { 0x68629863, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e29867, 98 },
    { 0x5ad29868, 90 },
    { 0x5cd29868, 61 },
    { 0x69429868, 98 },
    { 0x4952986a, 76 },
    { 0x1782986a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce029870, 61 },
    { 0xf1229871, 98 },
    { 0x1ea29871, 98 },
    { 0xe0929873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d2987c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2029883, 65 },
    { 0xa1129884, 90 },
    { 0xd3f29885, 55 },
    { 0x5f629886, 61 },
    { 0, 0 },
    { 0x9329888, 90 },
    { 0x7bd29888, 98 },
    { 0x7d92988a, 23 },
    { 0xc5c2988b, 33 },
    { 0x3cd2988c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75929894, 91 },
    { 0xec029895, 46 },
    { 0xd4e29896, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8432989a, 92 },
    { 0xb412989b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf982989e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572298a2, 66 },
    { 0x643298a2, 98 },
    { 0xd85298a4, 33 },
    { 0x930298a5, 66 },
    { 0xcd3298a3, 55 },
    { 0xa2a298a7, 98 },
    { 0x579298a8, 91 },
    { 0xaac298a9, 98 },
    { 0x184298aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee298ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea298b6, 55 },
    { 0x8af298b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa298bd, 46 },
    { 0xe2298bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31298c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98298c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23298c8, 23 },
    { 0x51d298c9, 91 },
    { 0x16f298ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x201298cd, 95 },
    { 0xbd0298ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf298d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355298dd, 98 },
    { 0x177298de, 99 },
    { 0, 0 },
    { 0x488298e0, 98 },
    { 0x9e6298e0, 89 },
    { 0, 0 },
    { 0x875298e3, 75 },
    { 0x84e298e4, 33 },
    { 0x822298e5, 88 },
    { 0xf56298e3, 91 },
    { 0xecf298e7, 98 },
    { 0x771298e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2298f1, 97 },
    { 0, 0 },
    { 0x31e298f3, 43 },
    { 0x2b7298f4, 24 },
    { 0xe2d298f5, 66 },
    { 0x141298f5, 61 },
    { 0x71298f5, 91 },
    { 0xe5d298f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364298fc, 33 },
    { 0x44298fd, 46 },
    { 0x98b298fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61729907, 43 },
    { 0x86429907, 61 },
    { 0x70d29908, 100 },
    { 0, 0 },
    { 0xb5a2990b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce29912, 34 },
    { 0xf3829913, 90 },
    { 0x68629912, 87 },
    { 0, 0 },
    { 0x8e429916, 78 },
    { 0x94f29917, 95 },
    { 0, 0 },
    { 0x9f229919, 96 },
    { 0, 0 },
    { 0xd8b2991b, 90 },
    { 0, 0 },
    { 0xb692991d, 61 },
    { 0x5522991e, 33 },
    { 0xc522991e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d929925, 78 },
    { 0x99429926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab2992d, 23 },
    { 0, 0 },
    { 0xef2992f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a829932, 76 },
    { 0, 0 },
    { 0x69e29934, 90 },
    { 0xc6529935, 98 },
    { 0x1829934, 84 },
    { 0, 0 },
    { 0x98229938, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba92993c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f2993f, 46 },
    { 0, 0 },
    { 0xd9129941, 61 },
    { 0x65929942, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a29945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39729948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54829950, 85 },
    { 0x3d129950, 91 },
    { 0x10b29952, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6229955, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb29958, 33 },
    { 0, 0 },
    { 0x43b2995a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df2995d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47429961, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d029967, 98 },
    { 0, 0 },
    { 0x4b629969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d2996c, 55 },
    { 0, 0 },
    { 0xfd72996e, 90 },
    { 0, 0 },
    { 0xf9c29970, 100 },
    { 0xf7929971, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e29978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2512997c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb542997f, 68 },
    { 0x80a2997f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c29984, 61 },
    { 0, 0 },
    { 0x35729986, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d29989, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1412998e, 85 },
    { 0x32e2998f, 76 },
    { 0xfe929990, 46 },
    { 0x69d2998e, 45 },
    { 0x7dd29992, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d29998, 95 },
    { 0x25b29999, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb12999c, 23 },
    { 0xd2e2999d, 61 },
    { 0x4202999e, 34 },
    { 0x28c2999f, 33 },
    { 0xde82999c, 55 },
    { 0x8482999c, 55 },
    { 0xb61299a0, 33 },
    { 0x698299a3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a299ab, 33 },
    { 0x283299ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33299b0, 90 },
    { 0x11b299b0, 33 },
    { 0, 0 },
    { 0x1d299b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683299b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x956299ba, 61 },
    { 0x1f8299bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab299be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d299c3, 90 },
    { 0, 0 },
    { 0xf54299c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125299c9, 33 },
    { 0, 0 },
    { 0x153299cb, 76 },
    { 0xe6299cc, 93 },
    { 0xd23299cb, 90 },
    { 0x268299cb, 77 },
    { 0, 0 },
    { 0xc38299d0, 100 },
    { 0x103299d0, 2 },
    { 0, 0 },
    { 0x756299d3, 98 },
    { 0x8d5299d4, 65 },
    { 0, 0 },
    { 0xb0e299d6, 61 },
    { 0, 0 },
    { 0x633299d8, 88 },
    { 0, 0 },
    { 0x5df299da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f299dd, 95 },
    { 0, 0 },
    { 0x37299df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901299e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da299ed, 23 },
    { 0xb1c299ed, 55 },
    { 0xaa2299ef, 55 },
    { 0x900299ef, 94 },
    { 0xa95299ef, 98 },
    { 0x942299ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376299f8, 98 },
    { 0, 0 },
    { 0x2bf299fa, 40 },
    { 0x952299fb, 95 },
    { 0, 0 },
    { 0x173299fd, 76 },
    { 0xa6c299fe, 95 },
    { 0x3f0299ff, 84 },
    { 0x32c299ff, 55 },
    { 0xcc4299fd, 87 },
    { 0, 0 },
    { 0xd2829a03, 88 },
    { 0xc0e29a03, 33 },
    { 0xf7129a05, 23 },
    { 0x9f429a03, 95 },
    { 0x93329a05, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x38729a0a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad29a0e, 87 },
    { 0x16c29a0f, 90 },
    { 0x96029a0f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a929a13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d29a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70929a21, 55 },
    { 0, 0 },
    { 0x12429a23, 90 },
    { 0xb4129a23, 61 },
    { 0xb5129a25, 91 },
    { 0x94d29a25, 90 },
    { 0x4b529a26, 93 },
    { 0, 0 },
    { 0x63c29a29, 43 },
    { 0x89929a29, 61 },
    { 0xebb29a2b, 99 },
    { 0x83a29a2b, 88 },
    { 0, 0 },
    { 0x1bd29a2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x22729a31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b29a36, 65 },
    { 0x99b29a36, 76 },
    { 0, 0 },
    { 0x36329a39, 98 },
    { 0xfa129a3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0229a46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12729a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83829a4e, 78 },
    { 0, 0 },
    { 0xc4b29a50, 33 },
    { 0, 0 },
    { 0xa6d29a52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93029a55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc429a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f29a5b, 97 },
    { 0xf0129a5c, 62 },
    { 0xbbe29a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b29a64, 90 },
    { 0, 0 },
    { 0x24029a66, 33 },
    { 0xfe429a66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1629a70, 23 },
    { 0x18b29a70, 95 },
    { 0x64e29a70, 61 },
    { 0, 0 },
    { 0x7c929a74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1329a78, 17 },
    { 0xcf29a79, 90 },
    { 0x29f29a7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16329a83, 76 },
    { 0x90129a83, 90 },
    { 0x94c29a84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f629a8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4229a91, 99 },
    { 0xefe29a92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b29a96, 93 },
    { 0, 0 },
    { 0x21f29a98, 33 },
    { 0, 0 },
    { 0xfd29a9a, 90 },
    { 0x19029a9b, 78 },
    { 0x7429a9b, 93 },
    { 0x50229a9d, 61 },
    { 0x28229a9e, 90 },
    { 0xc3829a9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2629aa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82229aaa, 33 },
    { 0x54e29aaa, 90 },
    { 0xa29aab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5229ab4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce429abb, 98 },
    { 0, 0 },
    { 0xda829abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe529ac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea829ac6, 90 },
    { 0, 0 },
    { 0xf1429ac8, 76 },
    { 0, 0 },
    { 0x9f929aca, 23 },
    { 0x329aca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9429ace, 88 },
    { 0x89b29acf, 90 },
    { 0x78d29ace, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52929ad4, 76 },
    { 0x17a29ad5, 61 },
    { 0x6b129ad5, 33 },
    { 0x70429ad7, 95 },
    { 0x37029ad8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c29adc, 75 },
    { 0x1429add, 23 },
    { 0x19829ade, 33 },
    { 0xbf229ade, 97 },
    { 0xf7529adf, 23 },
    { 0x98729add, 33 },
    { 0x21429adf, 34 },
    { 0x98729ae3, 33 },
    { 0x7f529ae0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab29aec, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b29af7, 87 },
    { 0xfa29af7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c29afc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5429b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde829b10, 98 },
    { 0, 0 },
    { 0x2c29b12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe29b18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e629b1b, 68 },
    { 0, 0 },
    { 0xa7629b1d, 90 },
    { 0x8929b1e, 68 },
    { 0x7fa29b1d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x87129b22, 23 },
    { 0x28b29b22, 43 },
    { 0xa0b29b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc29b28, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5429b2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d29b2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af29b31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a29b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8829b3b, 80 },
    { 0x7c029b3c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9629b3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de29b44, 98 },
    { 0x9ef29b45, 43 },
    { 0, 0 },
    { 0xa2c29b47, 77 },
    { 0, 0 },
    { 0x1b029b49, 77 },
    { 0, 0 },
    { 0x7e329b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb29b52, 90 },
    { 0xa729b52, 33 },
    { 0x7e529b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db29b58, 33 },
    { 0, 0 },
    { 0x70e29b5a, 23 },
    { 0, 0 },
    { 0x6f29b5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b29b5f, 34 },
    { 0x8f929b60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1529b65, 98 },
    { 0, 0 },
    { 0xf5929b67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f29b6c, 65 },
    { 0, 0 },
    { 0xa1e29b6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4329b71, 100 },
    { 0xee029b72, 98 },
    { 0x1329b71, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d29b77, 33 },
    { 0x1e429b77, 90 },
    { 0xf6929b79, 55 },
    { 0xa29b7a, 100 },
    { 0xa8a29b7a, 93 },
    { 0xfa429b7c, 76 },
    { 0x78029b7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac29b81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62229b86, 90 },
    { 0x1e129b87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64529b8a, 95 },
    { 0x8b129b8b, 61 },
    { 0, 0 },
    { 0xcab29b8d, 33 },
    { 0, 0 },
    { 0x16f29b8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd29b95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9229b99, 85 },
    { 0, 0 },
    { 0x79829b9b, 98 },
    { 0x25629b9c, 90 },
    { 0x7c229b9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf129ba0, 87 },
    { 0x77929ba1, 99 },
    { 0xadf29ba0, 90 },
    { 0x10a29ba3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40329ba8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24829bac, 87 },
    { 0x52429bad, 55 },
    { 0x96d29bae, 55 },
    { 0, 0 },
    { 0xee829bb0, 43 },
    { 0x6cf29bb1, 93 },
    { 0xe0829bb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d029bb5, 76 },
    { 0x98229bb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c29bba, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d229bbd, 93 },
    { 0xe4c29bbe, 90 },
    { 0x42829bbf, 61 },
    { 0, 0 },
    { 0x94829bc1, 34 },
    { 0xd8b29bc2, 23 },
    { 0xb5529bc2, 43 },
    { 0, 0 },
    { 0x7a929bc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d829bcc, 61 },
    { 0x14729bcd, 69 },
    { 0x3229bcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa29bd6, 66 },
    { 0x2e729bd7, 33 },
    { 0x43e29bd8, 85 },
    { 0, 0 },
    { 0x24029bda, 76 },
    { 0xac929bdb, 90 },
    { 0, 0 },
    { 0x18229bdd, 90 },
    { 0, 0 },
    { 0xab929bdf, 46 },
    { 0xcfc29be0, 91 },
    { 0, 0 },
    { 0xc2229be2, 61 },
    { 0x27e29be3, 61 },
    { 0x3c529be2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab29bec, 90 },
    { 0, 0 },
    { 0x64929bee, 77 },
    { 0x23829bee, 83 },
    { 0, 0 },
    { 0x9ef29bf1, 91 },
    { 0x29829bf1, 52 },
    { 0x7d929bf3, 34 },
    { 0x94629bf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a429c01, 61 },
    { 0xfa029c02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66429c05, 46 },
    { 0, 0 },
    { 0x88329c07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88529c0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22629c14, 33 },
    { 0, 0 },
    { 0x91429c16, 90 },
    { 0xe3429c17, 95 },
    { 0x98f29c18, 66 },
    { 0xbdc29c18, 90 },
    { 0x4aa29c18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8629c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21129c25, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x97129c28, 75 },
    { 0xaa329c28, 90 },
    { 0xf529c2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b29c32, 81 },
    { 0x23f29c32, 90 },
    { 0xf7d29c34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b829c3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc29c3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a29c44, 34 },
    { 0, 0 },
    { 0xd0b29c46, 90 },
    { 0xe3929c47, 55 },
    { 0x66d29c48, 23 },
    { 0xbc29c49, 78 },
    { 0x27629c4a, 61 },
    { 0x34f29c4b, 90 },
    { 0x41d29c4c, 83 },
    { 0xf6829c4d, 76 },
    { 0x96e29c4e, 65 },
    { 0xbf329c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd29c52, 36 },
    { 0xc6e29c52, 43 },
    { 0, 0 },
    { 0x31429c55, 43 },
    { 0x12029c56, 98 },
    { 0xd4a29c57, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e29c5e, 90 },
    { 0x66e29c5f, 90 },
    { 0, 0 },
    { 0x9e729c61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7129c65, 61 },
    { 0x24229c65, 90 },
    { 0xfb629c67, 91 },
    { 0xd3629c68, 66 },
    { 0x10029c69, 95 },
    { 0xeb229c6a, 61 },
    { 0x28529c6b, 59 },
    { 0xe9029c6c, 43 },
    { 0xe1029c6c, 55 },
    { 0xc3829c6e, 98 },
    { 0x55129c6f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e29c77, 66 },
    { 0x29129c78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee29c7b, 61 },
    { 0, 0 },
    { 0x9629c7d, 64 },
    { 0x72b29c7d, 81 },
    { 0, 0 },
    { 0x55c29c80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a29c83, 55 },
    { 0x1fb29c83, 76 },
    { 0xf3c29c83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e29c89, 61 },
    { 0, 0 },
    { 0x1f129c8b, 61 },
    { 0x7dc29c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc29c8f, 66 },
    { 0xf9f29c8f, 68 },
    { 0, 0 },
    { 0xf3e29c92, 33 },
    { 0x90229c92, 61 },
    { 0xa2a29c94, 90 },
    { 0x78429c95, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c029c9b, 56 },
    { 0x36129c9b, 76 },
    { 0xf9929c9b, 61 },
    { 0, 0 },
    { 0x52e29c9f, 68 },
    { 0x2fb29ca0, 66 },
    { 0, 0 },
    { 0x6c029ca2, 55 },
    { 0x4ba29ca3, 98 },
    { 0xa6029ca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62829ca9, 25 },
    { 0, 0 },
    { 0xd2d29cab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0929cb0, 33 },
    { 0, 0 },
    { 0xa6f29cb2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c29cb5, 61 },
    { 0xedd29cb6, 87 },
    { 0xd5b29cb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6429cbd, 91 },
    { 0, 0 },
    { 0x14529cbf, 33 },
    { 0, 0 },
    { 0xfee29cc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a29cc4, 67 },
    { 0xf4329cc4, 61 },
    { 0xc2529cc6, 95 },
    { 0, 0 },
    { 0x7b129cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe529ccc, 81 },
    { 0x3729ccd, 90 },
    { 0xd3c29cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac629cd4, 61 },
    { 0, 0 },
    { 0xb3029cd6, 43 },
    { 0x30229cd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64829cdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb29ce2, 90 },
    { 0xed329ce2, 95 },
    { 0x55129ce4, 98 },
    { 0x75c29ce4, 87 },
    { 0x27a29ce5, 77 },
    { 0, 0 },
    { 0x61d29ce8, 81 },
    { 0, 0 },
    { 0x68229cea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc29cee, 95 },
    { 0xd5129cee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2529cf6, 95 },
    { 0x9ac29cf7, 98 },
    { 0xbef29cf8, 66 },
    { 0x71729cf6, 90 },
    { 0, 0 },
    { 0xd9929cfb, 85 },
    { 0x1cd29cfc, 43 },
    { 0x22b29cfd, 23 },
    { 0x87229cfc, 66 },
    { 0xc0829cff, 100 },
    { 0x93729d00, 90 },
    { 0x87829cfe, 75 },
    { 0xad829d02, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d829d08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d129d0c, 61 },
    { 0x3e629d0d, 91 },
    { 0x9b829d0e, 95 },
    { 0xc7e29d0f, 96 },
    { 0, 0 },
    { 0xd5e29d11, 34 },
    { 0x76c29d12, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb29d15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c29d1c, 78 },
    { 0, 0 },
    { 0x3fd29d1e, 55 },
    { 0x89929d1f, 49 },
    { 0x8db29d1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf429d25, 88 },
    { 0xfdc29d26, 90 },
    { 0x68229d27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c429d2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c29d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9429d36, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c929d3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd29d3d, 96 },
    { 0xf6529d3e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6229d41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xace29d44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60829d4d, 61 },
    { 0xa0229d4e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e929d55, 76 },
    { 0x39329d56, 76 },
    { 0, 0 },
    { 0x62b29d58, 90 },
    { 0, 0 },
    { 0x55f29d5a, 23 },
    { 0xaa429d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34829d5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6229d62, 61 },
    { 0, 0 },
    { 0x40c29d64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a129d6d, 46 },
    { 0xbae29d6d, 83 },
    { 0xf6429d6d, 33 },
    { 0xa1729d70, 55 },
    { 0x3b229d6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43329d74, 33 },
    { 0x25329d75, 23 },
    { 0xc4729d76, 46 },
    { 0, 0 },
    { 0x47029d78, 43 },
    { 0, 0 },
    { 0xdc029d7a, 67 },
    { 0x88829d7b, 84 },
    { 0x54529d7c, 93 },
    { 0, 0 },
    { 0x13f29d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f29d86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129d8d, 98 },
    { 0x40729d8d, 90 },
    { 0xe1929d8d, 98 },
    { 0x4fa29d90, 100 },
    { 0x2c029d90, 17 },
    { 0, 0 },
    { 0x2fa29d93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62129d96, 68 },
    { 0x43829d97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a29d9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff29da2, 23 },
    { 0x81e29da2, 61 },
    { 0x8cd29da3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d29da8, 95 },
    { 0x19c29da9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c29daf, 90 },
    { 0x6ef29db0, 34 },
    { 0x8c029db0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20729db8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9629dbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc629dc0, 77 },
    { 0x64729dc1, 77 },
    { 0x14229dc0, 99 },
    { 0xe4f29dc3, 93 },
    { 0x95a29dc0, 61 },
    { 0xd2b29dc0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0429dc8, 55 },
    { 0x64029dc9, 33 },
    { 0, 0 },
    { 0x9d29dcb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3029dce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41729dd1, 100 },
    { 0x76529dd2, 61 },
    { 0x7e329dd1, 85 },
    { 0x3ab29dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33629dd9, 85 },
    { 0xf9529dda, 61 },
    { 0xbe529dda, 23 },
    { 0x86229ddc, 23 },
    { 0, 0 },
    { 0x6fd29dde, 93 },
    { 0xedb29dde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6029de2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6029de5, 66 },
    { 0xf4129de5, 97 },
    { 0, 0 },
    { 0x4f029de8, 43 },
    { 0, 0 },
    { 0xb0a29dea, 90 },
    { 0xe7f29dea, 90 },
    { 0xd7629dec, 96 },
    { 0xfda29ded, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff29df1, 87 },
    { 0, 0 },
    { 0x4bd29df3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e929df7, 87 },
    { 0, 0 },
    { 0xd2029df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e29e02, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x40929e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61929e08, 55 },
    { 0x1ea29e08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b29e14, 65 },
    { 0xa2329e14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1729e19, 90 },
    { 0x43f29e1a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11429e1e, 61 },
    { 0x6e529e1f, 34 },
    { 0x9cb29e20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef629e28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc629e2b, 91 },
    { 0x86a29e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f29e31, 95 },
    { 0x7ed29e32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79929e3b, 97 },
    { 0x39629e3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff329e3f, 81 },
    { 0x96129e40, 69 },
    { 0x1029e41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a29e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e529e49, 55 },
    { 0, 0 },
    { 0x7829e4b, 33 },
    { 0, 0 },
    { 0x55529e4d, 95 },
    { 0xbcf29e4e, 65 },
    { 0x64429e4f, 95 },
    { 0x98729e50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb629e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79229e59, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x56929e5c, 68 },
    { 0x53129e5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54529e61, 46 },
    { 0, 0 },
    { 0x65529e63, 23 },
    { 0, 0 },
    { 0xfdd29e65, 17 },
    { 0x9be29e66, 61 },
    { 0xe6129e67, 90 },
    { 0x93529e68, 66 },
    { 0xf0629e67, 98 },
    { 0x7c829e6a, 90 },
    { 0xec929e6b, 34 },
    { 0, 0 },
    { 0x8b429e6d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93529e71, 90 },
    { 0, 0 },
    { 0x22a29e73, 90 },
    { 0, 0 },
    { 0xd2c29e75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6929e7a, 76 },
    { 0, 0 },
    { 0xb6c29e7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11629e7f, 76 },
    { 0, 0 },
    { 0xa5a29e81, 95 },
    { 0x41429e81, 98 },
    { 0x74a29e81, 95 },
    { 0, 0 },
    { 0xe6929e85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20629e8d, 33 },
    { 0x94b29e8d, 95 },
    { 0xa4829e8f, 23 },
    { 0x58429e8d, 33 },
    { 0x97d29e91, 90 },
    { 0xe2d29e92, 98 },
    { 0xa6a29e91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee29e96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde629e9b, 33 },
    { 0x47129e9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11029e9f, 90 },
    { 0, 0 },
    { 0xeaa29ea1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd29ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9229ead, 98 },
    { 0xbd229eae, 61 },
    { 0x73129eae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab029eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e29eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e29ebd, 77 },
    { 0x75229ebd, 95 },
    { 0x9c129ebf, 95 },
    { 0x61929ec0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x83429ec3, 91 },
    { 0xc5229ec4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e29ec9, 53 },
    { 0, 0 },
    { 0x25329ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f29ed1, 33 },
    { 0xc4229ed2, 55 },
    { 0xfb829ed3, 66 },
    { 0, 0 },
    { 0x2ee29ed5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8329ed9, 23 },
    { 0, 0 },
    { 0xc2929edb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea29ee0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0929ee3, 95 },
    { 0x8c629ee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f29eea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3929ef2, 46 },
    { 0xcca29ef2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5129ef8, 90 },
    { 0x3e529ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2029efc, 23 },
    { 0xd8429efc, 98 },
    { 0xb8f29efe, 95 },
    { 0x7a629eff, 85 },
    { 0xa1829f00, 34 },
    { 0x8df29eff, 76 },
    { 0x6d29f02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc29f05, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f29f0a, 61 },
    { 0, 0 },
    { 0xaf729f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea629f0f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34329f13, 92 },
    { 0, 0 },
    { 0xdeb29f15, 48 },
    { 0x65129f16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e29f19, 90 },
    { 0x26d29f1a, 61 },
    { 0, 0 },
    { 0xa29f1c, 95 },
    { 0xa0d29f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac529f20, 48 },
    { 0xb7729f21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d29f25, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25329f29, 33 },
    { 0xf6129f2a, 61 },
    { 0xa5f29f2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0229f30, 85 },
    { 0x99c29f30, 90 },
    { 0x85c29f31, 98 },
    { 0xfa329f30, 52 },
    { 0xada29f34, 98 },
    { 0xef329f35, 90 },
    { 0xea329f34, 34 },
    { 0, 0 },
    { 0x2c929f38, 76 },
    { 0, 0 },
    { 0x15429f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff829f3d, 23 },
    { 0x2929f3e, 76 },
    { 0xfd129f3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f729f46, 34 },
    { 0xae929f46, 90 },
    { 0, 0 },
    { 0x55529f49, 84 },
    { 0x1bb29f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63029f50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97929f54, 95 },
    { 0, 0 },
    { 0xcb129f56, 95 },
    { 0x16e29f57, 34 },
    { 0, 0 },
    { 0x34929f59, 61 },
    { 0, 0 },
    { 0xa7729f5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96029f5f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e29f62, 95 },
    { 0, 0 },
    { 0xbcd29f64, 46 },
    { 0xfa329f65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40229f69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54929f6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db29f70, 61 },
    { 0xec029f71, 33 },
    { 0x10f29f72, 33 },
    { 0x8b829f70, 23 },
    { 0x8cf29f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5229f78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93229f7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e29f82, 99 },
    { 0, 0 },
    { 0x11729f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13429f8d, 66 },
    { 0, 0 },
    { 0x27629f8f, 98 },
    { 0xe5429f90, 99 },
    { 0x4e929f91, 61 },
    { 0xbce29f92, 93 },
    { 0x9eb29f90, 87 },
    { 0x43d29f93, 23 },
    { 0, 0 },
    { 0x19b29f96, 90 },
    { 0, 0 },
    { 0xb2929f98, 76 },
    { 0xe6329f98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c929fa1, 34 },
    { 0x7d429fa2, 100 },
    { 0x64d29fa3, 95 },
    { 0x78529fa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11229fa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a29fac, 78 },
    { 0x7a929fac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d429fb2, 33 },
    { 0, 0 },
    { 0x65329fb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba329fbb, 46 },
    { 0x2b629fbc, 33 },
    { 0x2ef29fbd, 76 },
    { 0x1c829fbe, 61 },
    { 0x24429fbf, 85 },
    { 0xa7129fbc, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5829fc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74629fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5529fcb, 43 },
    { 0, 0 },
    { 0xc8329fcd, 43 },
    { 0xb8729fcd, 34 },
    { 0xcda29fcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb129fd2, 98 },
    { 0x1c429fd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d529fdc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60429fe4, 46 },
    { 0xc1b29fe5, 90 },
    { 0x9da29fe5, 98 },
    { 0xbf829fe5, 66 },
    { 0x75029fe7, 98 },
    { 0, 0 },
    { 0xffa29fea, 90 },
    { 0x4d129feb, 33 },
    { 0xfa529fec, 34 },
    { 0, 0 },
    { 0xf6829fee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c029ff2, 61 },
    { 0x55a29ff3, 61 },
    { 0xe8229ff3, 43 },
    { 0x69029ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb52a000, 55 },
    { 0, 0 },
    { 0x10c2a002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe492a005, 23 },
    { 0x8952a006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a2a00b, 76 },
    { 0xa552a00c, 61 },
    { 0, 0 },
    { 0x9392a00e, 90 },
    { 0, 0 },
    { 0x16e2a010, 61 },
    { 0x8882a011, 84 },
    { 0xfe22a010, 61 },
    { 0, 0 },
    { 0xd322a014, 90 },
    { 0x4c02a015, 34 },
    { 0x4bf2a016, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2a01e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba2a022, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb882a02b, 87 },
    { 0x6592a02c, 33 },
    { 0, 0 },
    { 0xcb72a02e, 93 },
    { 0, 0 },
    { 0xf172a030, 61 },
    { 0x3932a031, 84 },
    { 0xfc82a030, 33 },
    { 0x3bb2a031, 43 },
    { 0xb382a030, 23 },
    { 0, 0 },
    { 0xcb62a036, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e2a03c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe572a03f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a2a045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f2a048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb72a052, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7202a055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d22a05c, 33 },
    { 0x27b2a05d, 98 },
    { 0, 0 },
    { 0x2a72a05f, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c2a063, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf272a06b, 90 },
    { 0x4b32a06c, 90 },
    { 0x95d2a06b, 75 },
    { 0x5282a06e, 23 },
    { 0, 0 },
    { 0x8012a070, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa52a077, 93 },
    { 0x7472a078, 71 },
    { 0xd462a079, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e2a07d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2a081, 88 },
    { 0x9ee2a082, 93 },
    { 0x2612a083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f2a087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7532a08a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1882a08d, 90 },
    { 0, 0 },
    { 0xbdd2a08f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3622a094, 61 },
    { 0x36c2a095, 85 },
    { 0, 0 },
    { 0xa7b2a097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2a09d, 99 },
    { 0x9332a09d, 67 },
    { 0, 0 },
    { 0x49f2a0a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4292a0a4, 61 },
    { 0xdeb2a0a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f2a0a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252a0b0, 100 },
    { 0x6b32a0b0, 3 },
    { 0x4812a0b2, 55 },
    { 0xcc02a0b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb02a0b9, 61 },
    { 0x5482a0ba, 90 },
    { 0x96a2a0b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3572a0c1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142a0c8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae2a0cb, 88 },
    { 0x3882a0cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc422a0d3, 93 },
    { 0x1612a0d3, 95 },
    { 0, 0 },
    { 0xa9d2a0d6, 95 },
    { 0xeaa2a0d7, 61 },
    { 0xc6f2a0d7, 98 },
    { 0x87f2a0d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7002a0e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef2a0e4, 90 },
    { 0x10c2a0e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb2a0e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72a0ed, 34 },
    { 0x702a0ed, 17 },
    { 0, 0 },
    { 0xa0b2a0f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb392a0f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd72a0fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb82a102, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa652a106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292a10c, 23 },
    { 0, 0 },
    { 0xb552a10e, 90 },
    { 0xc902a10e, 85 },
    { 0x5112a10e, 95 },
    { 0x95a2a111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5782a114, 93 },
    { 0x7142a115, 23 },
    { 0x4012a116, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc362a11a, 77 },
    { 0x4382a11b, 95 },
    { 0x8842a11b, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed2a121, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f2a124, 61 },
    { 0x6f22a124, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42a128, 33 },
    { 0x8642a129, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e2a130, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf822a134, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7492a139, 95 },
    { 0x55d2a13a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2792a13e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d2a141, 87 },
    { 0, 0 },
    { 0x6702a143, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f2a147, 95 },
    { 0xbf12a147, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2a14b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22a14f, 33 },
    { 0x8be2a150, 23 },
    { 0xb032a14f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc872a155, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42a158, 55 },
    { 0x36d2a158, 67 },
    { 0x732a15a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d2a161, 61 },
    { 0xf8b2a162, 52 },
    { 0x28d2a163, 93 },
    { 0x6a02a164, 84 },
    { 0x7052a165, 53 },
    { 0xd1f2a165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c2a16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc692a16d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2a170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf2a174, 66 },
    { 0, 0 },
    { 0x2f62a176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6832a17b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd2a17f, 46 },
    { 0x56d2a180, 99 },
    { 0x4e82a181, 78 },
    { 0xa4c2a181, 38 },
    { 0x4c32a181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a2a186, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf02a18a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b12a18f, 77 },
    { 0xe7e2a190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662a194, 90 },
    { 0x66f2a195, 66 },
    { 0, 0 },
    { 0x9282a197, 91 },
    { 0, 0 },
    { 0xaca2a199, 93 },
    { 0x46a2a19a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2a19d, 90 },
    { 0x7ff2a19d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf2a1a3, 66 },
    { 0x3ce2a1a4, 33 },
    { 0x4012a1a5, 76 },
    { 0x1e02a1a5, 98 },
    { 0x1922a1a7, 23 },
    { 0x17a2a1a8, 23 },
    { 0x8ce2a1a8, 76 },
    { 0x4452a1aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762a1b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f2a1b4, 33 },
    { 0, 0 },
    { 0xaa22a1b6, 76 },
    { 0x7f02a1b7, 33 },
    { 0x7f52a1b8, 33 },
    { 0x3c52a1b9, 95 },
    { 0xf462a1ba, 81 },
    { 0xb7a2a1b8, 67 },
    { 0x8e42a1b8, 43 },
    { 0x4be2a1ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3072a1c1, 99 },
    { 0x9f22a1c2, 55 },
    { 0x58f2a1c3, 61 },
    { 0x72c2a1c4, 17 },
    { 0, 0 },
    { 0x53d2a1c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf2a1c9, 43 },
    { 0xea72a1ca, 61 },
    { 0x1912a1cb, 23 },
    { 0x192a1ca, 46 },
    { 0x7cd2a1cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62a1d4, 55 },
    { 0, 0 },
    { 0xd432a1d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe042a1d9, 64 },
    { 0, 0 },
    { 0x1582a1db, 33 },
    { 0x5e42a1db, 34 },
    { 0x27a2a1dd, 75 },
    { 0x91c2a1db, 61 },
    { 0x6c52a1df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2a1e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b2a1e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5812a1ec, 64 },
    { 0x6572a1ed, 33 },
    { 0x53f2a1ee, 95 },
    { 0x94d2a1ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa602a1f6, 93 },
    { 0x6612a1f6, 23 },
    { 0, 0 },
    { 0x4942a1f9, 61 },
    { 0xb782a1fa, 33 },
    { 0, 0 },
    { 0xbb72a1fc, 95 },
    { 0xcd92a1fd, 90 },
    { 0xcfb2a1fc, 98 },
    { 0xd012a1fd, 43 },
    { 0, 0 },
    { 0xa042a201, 95 },
    { 0, 0 },
    { 0x6d02a203, 95 },
    { 0xe402a204, 53 },
    { 0x3d42a205, 61 },
    { 0x1632a205, 61 },
    { 0x5aa2a203, 55 },
    { 0x9932a208, 95 },
    { 0xa702a208, 91 },
    { 0xb152a20a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d32a210, 33 },
    { 0xef52a211, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b2a216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5972a219, 100 },
    { 0, 0 },
    { 0xb432a21b, 90 },
    { 0, 0 },
    { 0x352a21d, 61 },
    { 0x27e2a21d, 90 },
    { 0xd542a21f, 76 },
    { 0x7132a220, 76 },
    { 0xd352a220, 66 },
    { 0, 0 },
    { 0x9f82a223, 95 },
    { 0xadf2a224, 91 },
    { 0xf772a224, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf2a228, 46 },
    { 0x4c32a229, 33 },
    { 0x6632a22a, 91 },
    { 0, 0 },
    { 0x592a22c, 23 },
    { 0, 0 },
    { 0x322a22e, 64 },
    { 0x5c72a22f, 34 },
    { 0xe7a2a230, 33 },
    { 0xf82a231, 61 },
    { 0x9c62a230, 67 },
    { 0x8052a230, 98 },
    { 0x6722a234, 23 },
    { 0xc972a22f, 28 },
    { 0x38f2a236, 90 },
    { 0x8b22a237, 98 },
    { 0x9732a236, 33 },
    { 0x1e2a239, 33 },
    { 0x2f22a23a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9192a23d, 100 },
    { 0, 0 },
    { 0x6052a23f, 55 },
    { 0x7de2a240, 34 },
    { 0x6ad2a241, 98 },
    { 0x7872a240, 23 },
    { 0x9cb2a243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9042a247, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22a24f, 66 },
    { 0xf9e2a250, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e52a255, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba2a25a, 36 },
    { 0x6ad2a25b, 46 },
    { 0x77e2a25c, 99 },
    { 0, 0 },
    { 0xba82a25e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c2a261, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf442a265, 84 },
    { 0x7722a266, 33 },
    { 0, 0 },
    { 0xb712a268, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1692a26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7882a272, 33 },
    { 0, 0 },
    { 0x9502a274, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa2a278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22a280, 61 },
    { 0, 0 },
    { 0xa5c2a282, 98 },
    { 0, 0 },
    { 0xa5b2a284, 90 },
    { 0xf1d2a284, 55 },
    { 0x3852a286, 43 },
    { 0xe0f2a287, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea2a28e, 61 },
    { 0x4392a28f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172a293, 90 },
    { 0xd472a294, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d2a297, 61 },
    { 0, 0 },
    { 0xea32a299, 98 },
    { 0, 0 },
    { 0x5222a29b, 98 },
    { 0x7ac2a29c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd2a2a1, 95 },
    { 0, 0 },
    { 0x9b62a2a3, 95 },
    { 0x29b2a2a3, 83 },
    { 0x80d2a2a5, 61 },
    { 0x3b62a2a4, 66 },
    { 0x1b32a2a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3592a2ab, 81 },
    { 0x7812a2ac, 84 },
    { 0, 0 },
    { 0x222a2ae, 34 },
    { 0x50e2a2ae, 84 },
    { 0x1bb2a2ae, 98 },
    { 0x1eb2a2b1, 33 },
    { 0x5492a2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8002a2b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1602a2b8, 81 },
    { 0xd3d2a2b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b52a2bc, 88 },
    { 0, 0 },
    { 0x91b2a2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8662a2c2, 46 },
    { 0, 0 },
    { 0x5372a2c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b32a2cb, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d42a2ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2602a2d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1272a2dd, 98 },
    { 0xc052a2de, 61 },
    { 0xfb42a2de, 33 },
    { 0x8272a2de, 23 },
    { 0x2b62a2df, 33 },
    { 0xce22a2e0, 87 },
    { 0x4cd2a2df, 46 },
    { 0x1682a2e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f52a2e8, 43 },
    { 0xa1c2a2e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942a2f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a2a2f7, 84 },
    { 0xd612a2f8, 33 },
    { 0x4cb2a2f7, 84 },
    { 0, 0 },
    { 0xd5d2a2fb, 67 },
    { 0x69f2a2fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb2a304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe022a308, 33 },
    { 0, 0 },
    { 0x4622a30a, 66 },
    { 0x4892a30b, 90 },
    { 0x5202a30c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a2a310, 95 },
    { 0xc9f2a311, 67 },
    { 0x6cc2a312, 33 },
    { 0x8292a313, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd22a316, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe152a319, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c32a31c, 78 },
    { 0xb4a2a31c, 41 },
    { 0xdef2a31e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1902a321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f2a324, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d2a327, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd032a32a, 95 },
    { 0, 0 },
    { 0xec32a32c, 34 },
    { 0, 0 },
    { 0xd312a32e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac72a331, 49 },
    { 0xa5f2a332, 66 },
    { 0xa262a333, 90 },
    { 0, 0 },
    { 0x5bc2a335, 33 },
    { 0, 0 },
    { 0xb862a337, 65 },
    { 0xbab2a338, 61 },
    { 0xa862a339, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8372a33e, 93 },
    { 0xe212a33f, 34 },
    { 0, 0 },
    { 0x5f72a341, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1182a349, 23 },
    { 0xb2e2a349, 98 },
    { 0x3962a349, 23 },
    { 0x1d62a34c, 67 },
    { 0xa4d2a34d, 43 },
    { 0xc332a34c, 61 },
    { 0x6ab2a349, 61 },
    { 0x7652a34e, 59 },
    { 0x70d2a351, 90 },
    { 0xb2e2a352, 23 },
    { 0x5b2a352, 13 },
    { 0xc182a352, 33 },
    { 0x6b32a353, 85 },
    { 0xa202a356, 46 },
    { 0xe0e2a357, 84 },
    { 0x5e2a353, 90 },
    { 0x6a92a355, 84 },
    { 0x29f2a356, 76 },
    { 0x88e2a355, 33 },
    { 0x95a2a34c, 61 },
    { 0xd082a35d, 81 },
    { 0xdcc2a35d, 98 },
    { 0x4972a35f, 33 },
    { 0xf5e2a352, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5402a365, 23 },
    { 0, 0 },
    { 0xa8d2a367, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe612a36b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4752a378, 61 },
    { 0x9112a378, 95 },
    { 0xceb2a37a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4072a380, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb772a386, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e2a389, 33 },
    { 0xad92a389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f02a392, 90 },
    { 0, 0 },
    { 0x4b82a394, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9512a397, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e72a39e, 61 },
    { 0x6952a39f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a72a3a5, 34 },
    { 0xb752a3a6, 90 },
    { 0xbac2a3a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982a3ad, 34 },
    { 0x5712a3ae, 95 },
    { 0xb122a3ad, 34 },
    { 0x7912a3b0, 90 },
    { 0x4ae2a3b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe422a3b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2a3bc, 61 },
    { 0xa962a3bd, 33 },
    { 0x8702a3be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c22a3c1, 61 },
    { 0, 0 },
    { 0x3d02a3c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa922a3c7, 66 },
    { 0x252a3c8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa732a3cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c12a3d0, 90 },
    { 0, 0 },
    { 0x10c2a3d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab22a3d5, 91 },
    { 0x2002a3d6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad2a3dc, 68 },
    { 0, 0 },
    { 0xecc2a3de, 23 },
    { 0xf092a3de, 52 },
    { 0, 0 },
    { 0x64c2a3e1, 46 },
    { 0x61f2a3e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7222a3e6, 95 },
    { 0x2a82a3e7, 61 },
    { 0x4432a3e8, 33 },
    { 0x65c2a3e9, 33 },
    { 0x7662a3ea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212a3ed, 43 },
    { 0x5f2a3ed, 77 },
    { 0, 0 },
    { 0x4dd2a3f0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c2a3f5, 46 },
    { 0x4252a3f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba2a3f9, 46 },
    { 0x18a2a3fa, 61 },
    { 0x8332a3fb, 90 },
    { 0x14d2a3fa, 100 },
    { 0x2972a3f9, 46 },
    { 0x152a3fb, 90 },
    { 0x6792a3ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4412a403, 90 },
    { 0x2532a404, 90 },
    { 0xfc52a405, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a408, 98 },
    { 0x9bd2a409, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b72a411, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5112a415, 99 },
    { 0x8c92a416, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2a41a, 84 },
    { 0xcd12a41b, 98 },
    { 0xcbe2a41b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b32a420, 46 },
    { 0x1532a421, 95 },
    { 0x61a2a421, 76 },
    { 0xf122a423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc92a42a, 90 },
    { 0x8e52a42b, 95 },
    { 0xd892a42c, 23 },
    { 0x8822a42d, 88 },
    { 0xf602a42e, 33 },
    { 0x5252a42e, 90 },
    { 0x57e2a430, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede2a436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b2a439, 70 },
    { 0x3752a43a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7052a43e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb542a443, 61 },
    { 0x4df2a444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2a448, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa102a44e, 61 },
    { 0, 0 },
    { 0x72e2a450, 78 },
    { 0x9122a451, 33 },
    { 0xadf2a452, 78 },
    { 0xb082a450, 61 },
    { 0x13b2a454, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2a457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2a45f, 84 },
    { 0, 0 },
    { 0xa582a461, 90 },
    { 0, 0 },
    { 0x7672a463, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf2a467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e22a473, 52 },
    { 0, 0 },
    { 0x50b2a475, 61 },
    { 0xc712a475, 95 },
    { 0x942a475, 90 },
    { 0x3232a478, 61 },
    { 0x5eb2a479, 67 },
    { 0x5cc2a47a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa142a47e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2a486, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d2a489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072a48c, 93 },
    { 0xa792a48d, 85 },
    { 0xc6f2a48e, 90 },
    { 0xded2a48f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3642a492, 33 },
    { 0, 0 },
    { 0x2452a494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c32a497, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f2a49a, 76 },
    { 0, 0 },
    { 0x2982a49c, 90 },
    { 0x43d2a49d, 33 },
    { 0xe562a49d, 90 },
    { 0x8932a49f, 92 },
    { 0x8d62a49c, 33 },
    { 0x52a4a1, 90 },
    { 0x5bc2a4a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b2a4a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a42a4b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd2a4b8, 99 },
    { 0xb252a4b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2262a4c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb02a4cb, 84 },
    { 0xf2c2a4cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a2a4d2, 95 },
    { 0, 0 },
    { 0x26e2a4d4, 69 },
    { 0x3bb2a4d4, 90 },
    { 0x9bf2a4d4, 33 },
    { 0x1222a4d7, 93 },
    { 0x392a4d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3532a4dd, 90 },
    { 0x81c2a4de, 100 },
    { 0xd602a4dd, 87 },
    { 0x9ee2a4e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f82a4e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9872a4e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc2a4ef, 95 },
    { 0xbf82a4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2a4f4, 33 },
    { 0x8eb2a4f5, 33 },
    { 0, 0 },
    { 0x2b02a4f7, 33 },
    { 0x7092a4f8, 62 },
    { 0x49b2a4f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf402a4fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d2a502, 95 },
    { 0, 0 },
    { 0xfa92a504, 90 },
    { 0, 0 },
    { 0x27c2a506, 100 },
    { 0x5cb2a506, 23 },
    { 0x6702a508, 90 },
    { 0, 0 },
    { 0x6bc2a50a, 76 },
    { 0x3152a50b, 85 },
    { 0x6822a50c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e12a50f, 95 },
    { 0x97a2a50f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb92a514, 90 },
    { 0xd462a514, 34 },
    { 0x23c2a514, 61 },
    { 0, 0 },
    { 0x7602a518, 55 },
    { 0, 0 },
    { 0xfde2a51a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b12a51d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a2a520, 61 },
    { 0x85c2a521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc12a527, 61 },
    { 0, 0 },
    { 0x1572a529, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc952a52f, 33 },
    { 0, 0 },
    { 0xb022a531, 61 },
    { 0, 0 },
    { 0x7f62a533, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12a537, 61 },
    { 0, 0 },
    { 0xb22a539, 55 },
    { 0xa62a53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7592a53d, 43 },
    { 0, 0 },
    { 0xaea2a53f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8902a54e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd2a553, 87 },
    { 0x84d2a554, 88 },
    { 0x652a553, 55 },
    { 0x96f2a556, 23 },
    { 0x9de2a557, 98 },
    { 0xf242a556, 91 },
    { 0xec32a556, 84 },
    { 0x9cb2a55a, 33 },
    { 0x362a557, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2a562, 95 },
    { 0x2842a562, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce52a566, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb2a569, 95 },
    { 0, 0 },
    { 0x7c22a56b, 62 },
    { 0xbae2a56c, 43 },
    { 0xa52a56c, 93 },
    { 0x57c2a56c, 55 },
    { 0, 0 },
    { 0xce02a570, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2a573, 68 },
    { 0x1742a574, 61 },
    { 0x6952a574, 61 },
    { 0x6032a573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4132a579, 61 },
    { 0xd2a57a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1432a580, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8622a589, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a2a58e, 23 },
    { 0xd9c2a58e, 53 },
    { 0xf282a590, 77 },
    { 0x422a58f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2192a596, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2a59b, 33 },
    { 0xc0b2a59c, 87 },
    { 0xc682a59d, 87 },
    { 0, 0 },
    { 0x8102a59f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f2a5a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082a5a5, 91 },
    { 0x39e2a5a5, 76 },
    { 0x532a5a5, 34 },
    { 0, 0 },
    { 0xd822a5a9, 77 },
    { 0, 0 },
    { 0x91c2a5ab, 76 },
    { 0, 0 },
    { 0x2352a5ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c2a5b0, 78 },
    { 0xafb2a5b0, 90 },
    { 0x83a2a5b2, 33 },
    { 0xba42a5b3, 33 },
    { 0x9592a5b3, 77 },
    { 0x8612a5b0, 55 },
    { 0x67f2a5b6, 67 },
    { 0x1712a5b7, 61 },
    { 0, 0 },
    { 0x8362a5b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e2a5bd, 33 },
    { 0x1562a5be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92a5c3, 90 },
    { 0x8492a5c4, 91 },
    { 0xbec2a5c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf102a5c9, 87 },
    { 0x59f2a5ca, 91 },
    { 0x4962a5cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2a5d1, 95 },
    { 0x1e82a5d1, 90 },
    { 0x372a5d3, 51 },
    { 0x1092a5d4, 64 },
    { 0x3522a5d5, 61 },
    { 0x9642a5d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce12a5da, 98 },
    { 0xb882a5da, 55 },
    { 0x27b2a5dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb962a5e2, 64 },
    { 0x9462a5e2, 61 },
    { 0, 0 },
    { 0x3bf2a5e5, 17 },
    { 0x9102a5e6, 61 },
    { 0xd712a5e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7452a5ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa182a5f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02a5f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe952a5f9, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad2a602, 61 },
    { 0x1642a603, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1322a608, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f2a60b, 33 },
    { 0, 0 },
    { 0x14b2a60d, 61 },
    { 0xb922a60d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc2a614, 90 },
    { 0x8d2a615, 68 },
    { 0xd892a615, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc932a61c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f2a61f, 46 },
    { 0xf362a61f, 33 },
    { 0x5762a61f, 45 },
    { 0x1722a622, 23 },
    { 0xf0f2a623, 33 },
    { 0xecd2a622, 90 },
    { 0x3b52a625, 55 },
    { 0x7f2a624, 33 },
    { 0x1ac2a627, 76 },
    { 0xe232a626, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4882a62b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2832a62f, 85 },
    { 0x5de2a62f, 33 },
    { 0xa472a631, 33 },
    { 0x2fd2a632, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f42a639, 33 },
    { 0xf432a639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd22a63d, 61 },
    { 0x5942a63e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb2a644, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2a647, 95 },
    { 0x7b2a647, 98 },
    { 0xad72a649, 33 },
    { 0, 0 },
    { 0x70b2a64b, 90 },
    { 0x7792a64c, 98 },
    { 0xcb32a64d, 23 },
    { 0, 0 },
    { 0x4872a64f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2a652, 76 },
    { 0x5f72a653, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf872a65a, 88 },
    { 0, 0 },
    { 0x5d92a65c, 55 },
    { 0x5072a65d, 33 },
    { 0x4512a65e, 98 },
    { 0, 0 },
    { 0xb882a660, 61 },
    { 0, 0 },
    { 0x8292a662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x512a665, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe2a66a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc302a671, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb2a678, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2a67b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5482a67e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1102a681, 76 },
    { 0x7652a682, 95 },
    { 0xa452a681, 95 },
    { 0x5082a684, 43 },
    { 0x22a685, 61 },
    { 0x6052a684, 76 },
    { 0x2ac2a687, 23 },
    { 0xeea2a688, 97 },
    { 0x1ed2a686, 90 },
    { 0x6a72a687, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6272a68e, 90 },
    { 0xf92a68f, 99 },
    { 0, 0 },
    { 0xaa72a691, 61 },
    { 0x26b2a692, 43 },
    { 0x77c2a693, 95 },
    { 0x7eb2a692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc2a697, 93 },
    { 0, 0 },
    { 0x7d72a699, 90 },
    { 0x7d82a69a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c2a6a6, 33 },
    { 0xc7a2a6a6, 93 },
    { 0, 0 },
    { 0xca82a6a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252a6ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc2a6b5, 23 },
    { 0, 0 },
    { 0x70e2a6b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff2a6bb, 96 },
    { 0xf972a6bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6952a6bf, 23 },
    { 0xe3e2a6c0, 91 },
    { 0x95f2a6bf, 76 },
    { 0x6602a6c2, 61 },
    { 0xada2a6c2, 76 },
    { 0xd292a6c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2562a6c8, 95 },
    { 0xfb82a6c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832a6cf, 76 },
    { 0x83c2a6d0, 55 },
    { 0x5042a6d1, 60 },
    { 0x2452a6d0, 98 },
    { 0x89f2a6d1, 33 },
    { 0xea82a6d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2a6d7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef2a6da, 76 },
    { 0x22c2a6db, 52 },
    { 0, 0 },
    { 0x3142a6dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1672a6e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e2a6e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9582a6f0, 76 },
    { 0xba52a6f1, 98 },
    { 0xac2a6f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9782a6f7, 66 },
    { 0x48f2a6f8, 48 },
    { 0x57a2a6f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc402a6fc, 98 },
    { 0x1662a6fd, 90 },
    { 0xa222a6fd, 95 },
    { 0x1b72a6ff, 84 },
    { 0x13d2a700, 46 },
    { 0x76e2a701, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b2a708, 96 },
    { 0x2462a709, 90 },
    { 0, 0 },
    { 0x7982a70b, 61 },
    { 0xddc2a70b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a2a719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf42a71e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7772a721, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d2a725, 95 },
    { 0, 0 },
    { 0x7762a727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2662a72b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca2a72e, 95 },
    { 0xed72a72e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc2a735, 33 },
    { 0x3d72a736, 58 },
    { 0x2be2a735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe12a73b, 95 },
    { 0xb592a73b, 52 },
    { 0, 0 },
    { 0xde42a73e, 43 },
    { 0, 0 },
    { 0x25e2a740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x912a743, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a12a746, 90 },
    { 0, 0 },
    { 0x1d2a748, 33 },
    { 0, 0 },
    { 0x3eb2a74a, 92 },
    { 0xf3d2a74a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9622a74f, 52 },
    { 0x2e92a74f, 33 },
    { 0x6a22a751, 68 },
    { 0xf302a752, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b2a75d, 76 },
    { 0x2ac2a75e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a2a762, 95 },
    { 0x8642a762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1832a766, 34 },
    { 0, 0 },
    { 0x5752a768, 98 },
    { 0, 0 },
    { 0x5572a76a, 98 },
    { 0, 0 },
    { 0x66c2a76c, 61 },
    { 0x802a76c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1712a772, 23 },
    { 0x9fe2a773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f2a778, 91 },
    { 0x8442a779, 93 },
    { 0x4b92a778, 95 },
    { 0xb902a77b, 95 },
    { 0x4682a77c, 95 },
    { 0xbf2a77d, 33 },
    { 0x9f72a77e, 92 },
    { 0x7332a77d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc832a782, 91 },
    { 0xfd52a783, 90 },
    { 0xb4c2a784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4372a795, 64 },
    { 0xde02a795, 85 },
    { 0x1042a797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc432a79f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7042a7a5, 61 },
    { 0x2f02a7a6, 95 },
    { 0x9272a7a7, 33 },
    { 0xccb2a7a7, 90 },
    { 0, 0 },
    { 0x9df2a7aa, 85 },
    { 0x15c2a7ab, 90 },
    { 0x8ed2a7ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc532a7b6, 76 },
    { 0, 0 },
    { 0x8f12a7b8, 95 },
    { 0xfc52a7b9, 34 },
    { 0x7792a7ba, 90 },
    { 0x3ac2a7bb, 87 },
    { 0xd5d2a7ba, 61 },
    { 0xed2a7bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf62a7c7, 90 },
    { 0xded2a7c8, 33 },
    { 0xe532a7c8, 61 },
    { 0x85d2a7c8, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c32a7cf, 77 },
    { 0xce72a7cf, 99 },
    { 0x4462a7d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb72a7d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e2a7d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af2a7da, 95 },
    { 0xde02a7db, 61 },
    { 0x542a7dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc82a7e3, 61 },
    { 0, 0 },
    { 0x9732a7e5, 61 },
    { 0x5292a7e6, 100 },
    { 0xdea2a7e5, 23 },
    { 0x662a7e8, 81 },
    { 0, 0 },
    { 0x3852a7ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f2a7ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc042a7f0, 90 },
    { 0xaeb2a7f1, 23 },
    { 0x3e02a7f1, 98 },
    { 0, 0 },
    { 0xa5d2a7f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a62a7f7, 23 },
    { 0xe972a7f7, 90 },
    { 0xa7e2a7f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2a7fd, 76 },
    { 0xa3c2a7fe, 90 },
    { 0xcc12a7fd, 90 },
    { 0x5182a800, 33 },
    { 0, 0 },
    { 0x1592a802, 74 },
    { 0, 0 },
    { 0x7272a804, 46 },
    { 0x2892a805, 67 },
    { 0, 0 },
    { 0x3cd2a807, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c32a80b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7142a810, 33 },
    { 0xc7d2a810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf2a815, 90 },
    { 0x4d82a815, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b82a81a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f2a81f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b2a822, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd2a825, 61 },
    { 0x3e12a825, 55 },
    { 0, 0 },
    { 0x7892a828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d12a82d, 90 },
    { 0x35f2a82d, 31 },
    { 0x33b2a82f, 91 },
    { 0xce82a82f, 98 },
    { 0x4ef2a82f, 98 },
    { 0x2f22a830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa62a836, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6942a839, 90 },
    { 0x60c2a83a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed2a840, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc002a843, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9002a847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e32a84a, 43 },
    { 0xa242a84b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae2a851, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2a857, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d92a85b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e2a860, 67 },
    { 0x46e2a860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2312a864, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b02a867, 98 },
    { 0x4302a868, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52a86b, 96 },
    { 0x9982a86b, 33 },
    { 0x6272a86d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb182a870, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a72a876, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da2a87b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8142a883, 85 },
    { 0x5492a884, 98 },
    { 0xb682a885, 61 },
    { 0xd462a884, 78 },
    { 0, 0 },
    { 0x1752a888, 76 },
    { 0, 0 },
    { 0x6b82a88a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e62a88f, 61 },
    { 0, 0 },
    { 0x9592a891, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d2a896, 17 },
    { 0, 0 },
    { 0x4692a898, 98 },
    { 0x40b2a899, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c32a89e, 66 },
    { 0xadf2a89f, 98 },
    { 0, 0 },
    { 0xd842a8a1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f2a8a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e2a8ad, 33 },
    { 0, 0 },
    { 0xd0d2a8af, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e12a8b3, 90 },
    { 0, 0 },
    { 0x9032a8b5, 98 },
    { 0xbab2a8b6, 78 },
    { 0x2142a8b7, 23 },
    { 0x5892a8b8, 46 },
    { 0, 0 },
    { 0x21f2a8ba, 23 },
    { 0, 0 },
    { 0xfd42a8bc, 33 },
    { 0x8362a8bd, 34 },
    { 0x9ec2a8be, 84 },
    { 0x34e2a8bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc32a8c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f2a8c5, 98 },
    { 0, 0 },
    { 0x2102a8c7, 61 },
    { 0, 0 },
    { 0x5432a8c9, 61 },
    { 0x5c02a8c9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe42a8cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2a8d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f2a8d5, 34 },
    { 0xec62a8d6, 91 },
    { 0xe2a2a8d7, 90 },
    { 0x7b62a8d8, 34 },
    { 0x3902a8d8, 33 },
    { 0x95e2a8da, 55 },
    { 0xecc2a8db, 76 },
    { 0x2f32a8d9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a8e1, 90 },
    { 0xe352a8e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5442a8e8, 85 },
    { 0x6752a8e9, 78 },
    { 0, 0 },
    { 0x132a8eb, 23 },
    { 0x51b2a8eb, 90 },
    { 0xdec2a8ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422a8f0, 88 },
    { 0, 0 },
    { 0xdc52a8f2, 74 },
    { 0xa482a8f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f2a8f7, 52 },
    { 0xd682a8f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea2a8fe, 98 },
    { 0x2f12a8fe, 61 },
    { 0xb2a2a900, 61 },
    { 0, 0 },
    { 0xf3e2a902, 33 },
    { 0x9752a902, 23 },
    { 0, 0 },
    { 0xc842a905, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xab52a908, 90 },
    { 0x58f2a909, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe822a90c, 99 },
    { 0xde2a90d, 98 },
    { 0xdbe2a90e, 33 },
    { 0x6c72a90f, 52 },
    { 0, 0 },
    { 0xd3e2a911, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22a917, 33 },
    { 0xd802a918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd702a91d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef2a926, 68 },
    { 0xc622a927, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a2a92c, 61 },
    { 0x6e22a92d, 99 },
    { 0x5ed2a92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8752a934, 61 },
    { 0xdf2a935, 98 },
    { 0x2a02a936, 33 },
    { 0x38b2a937, 98 },
    { 0xd452a938, 33 },
    { 0xece2a934, 33 },
    { 0xad72a937, 91 },
    { 0, 0 },
    { 0x4dc2a93c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c2a945, 90 },
    { 0xcc02a946, 80 },
    { 0x8d2a945, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a94a, 43 },
    { 0xf742a94b, 90 },
    { 0x70a2a94a, 43 },
    { 0, 0 },
    { 0x6cb2a94e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e2a952, 61 },
    { 0xcf2a953, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe712a95c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6892a962, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8212a969, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b2a96f, 52 },
    { 0x8452a970, 38 },
    { 0xc7c2a970, 76 },
    { 0x5f32a970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf2a977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92a97b, 47 },
    { 0, 0 },
    { 0x2c02a97d, 95 },
    { 0, 0 },
    { 0x5682a97f, 95 },
    { 0x6812a980, 61 },
    { 0xbdd2a980, 33 },
    { 0x85c2a97f, 61 },
    { 0x78e2a983, 61 },
    { 0xdc72a983, 33 },
    { 0x45f2a985, 51 },
    { 0, 0 },
    { 0x3242a987, 98 },
    { 0xd9f2a988, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a82a98b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f2a98e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472a99a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f2a99d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f82a9a0, 76 },
    { 0xa782a9a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c2a9a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d62a9ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7072a9bb, 61 },
    { 0x2f52a9bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6292a9c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f2a9cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d2a9d3, 33 },
    { 0x9152a9d4, 52 },
    { 0xeac2a9d4, 33 },
    { 0x8a02a9d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff2a9d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe842a9dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd602a9e0, 33 },
    { 0x3d62a9e1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf562a9e5, 61 },
    { 0x7cc2a9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd992a9eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf432a9ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e72a9f6, 93 },
    { 0xe4c2a9f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d2a9fa, 84 },
    { 0, 0 },
    { 0xce82a9fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa282a9ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb2aa06, 43 },
    { 0, 0 },
    { 0xef32aa08, 76 },
    { 0x4972aa09, 38 },
    { 0xd222aa0a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd2aa0f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a2aa12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2962aa15, 55 },
    { 0x6d52aa16, 98 },
    { 0xe0f2aa16, 95 },
    { 0, 0 },
    { 0xc762aa19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc2aa1d, 90 },
    { 0xb92aa1e, 90 },
    { 0xd782aa1d, 98 },
    { 0x8a02aa20, 88 },
    { 0x4672aa21, 55 },
    { 0x3802aa22, 46 },
    { 0x5fb2aa23, 98 },
    { 0xc352aa24, 52 },
    { 0, 0 },
    { 0x7442aa26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b2aa2b, 55 },
    { 0x81e2aa2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca2aa33, 76 },
    { 0x76c2aa33, 43 },
    { 0x5e52aa35, 67 },
    { 0x2aa2aa35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc2aa39, 95 },
    { 0x81f2aa3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2aa42, 98 },
    { 0x3da2aa43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2aa46, 85 },
    { 0x3122aa47, 90 },
    { 0x3912aa46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf92aa56, 76 },
    { 0x7be2aa57, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf122aa5b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d92aa5e, 76 },
    { 0, 0 },
    { 0xbc52aa60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f2aa63, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6702aa67, 91 },
    { 0x1842aa68, 98 },
    { 0x31d2aa67, 52 },
    { 0x7382aa69, 99 },
    { 0, 0 },
    { 0xd602aa6c, 85 },
    { 0x6a22aa6d, 69 },
    { 0xb412aa6c, 23 },
    { 0x2fd2aa6f, 90 },
    { 0, 0 },
    { 0x9e52aa71, 84 },
    { 0x8dc2aa72, 61 },
    { 0x9722aa73, 61 },
    { 0x1ab2aa71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde92aa77, 23 },
    { 0, 0 },
    { 0x8b72aa79, 61 },
    { 0x6572aa7a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e2aa81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802aa8c, 86 },
    { 0x80c2aa8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4842aa91, 33 },
    { 0xf3c2aa92, 61 },
    { 0, 0 },
    { 0xf5c2aa94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e82aa97, 95 },
    { 0x532aa98, 90 },
    { 0xc362aa99, 90 },
    { 0x1192aa9a, 23 },
    { 0x6b12aa9a, 76 },
    { 0x7a22aa9c, 61 },
    { 0x6e42aa9d, 90 },
    { 0x8562aa9b, 77 },
    { 0x6be2aa9f, 69 },
    { 0x7322aaa0, 23 },
    { 0xee82aa98, 33 },
    { 0xd3c2aaa2, 90 },
    { 0xbb02aaa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f2aaa6, 61 },
    { 0x8482aaa7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd142aaab, 66 },
    { 0, 0 },
    { 0xa412aaad, 98 },
    { 0x72d2aaad, 98 },
    { 0x2362aaaf, 98 },
    { 0xf0e2aab0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c22aab3, 34 },
    { 0x6342aab4, 75 },
    { 0x2632aab4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc42aab9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a22aabc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea2aabf, 61 },
    { 0, 0 },
    { 0x7a2aac1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f2aac4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5822aac8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c2aacc, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8842aad1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2aad5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8282aad8, 67 },
    { 0x71d2aad8, 23 },
    { 0, 0 },
    { 0xc662aadb, 95 },
    { 0, 0 },
    { 0x1962aadd, 46 },
    { 0, 0 },
    { 0x15a2aadf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a62aae2, 95 },
    { 0xe492aae2, 61 },
    { 0, 0 },
    { 0xa452aae5, 43 },
    { 0x1b2aae6, 61 },
    { 0x4bc2aae7, 98 },
    { 0xbeb2aae5, 34 },
    { 0xbae2aae9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a2aaed, 33 },
    { 0, 0 },
    { 0x83b2aaef, 33 },
    { 0x9832aaf0, 55 },
    { 0, 0 },
    { 0xf752aaf2, 33 },
    { 0x7ec2aaf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d2aaf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8142aafb, 52 },
    { 0x332aafc, 61 },
    { 0x9d02aafd, 93 },
    { 0x4fc2aafd, 19 },
    { 0, 0 },
    { 0x7362ab00, 61 },
    { 0x522ab01, 34 },
    { 0xdb62ab02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d92ab08, 61 },
    { 0x7182ab09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea2ab16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa792ab1b, 61 },
    { 0xb932ab1b, 55 },
    { 0, 0 },
    { 0x33c2ab1e, 98 },
    { 0, 0 },
    { 0x9a02ab20, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc62ab25, 34 },
    { 0xf692ab25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a82ab2d, 61 },
    { 0x8c62ab2e, 90 },
    { 0, 0 },
    { 0xa912ab30, 34 },
    { 0x2872ab31, 33 },
    { 0xe202ab31, 90 },
    { 0x6b22ab33, 76 },
    { 0, 0 },
    { 0x6222ab35, 61 },
    { 0, 0 },
    { 0x172ab37, 61 },
    { 0x62b2ab37, 33 },
    { 0x1d2ab37, 33 },
    { 0x2f82ab3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a2ab42, 91 },
    { 0x952ab43, 99 },
    { 0, 0 },
    { 0xbee2ab45, 93 },
    { 0xcb92ab45, 76 },
    { 0, 0 },
    { 0x7252ab48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ab4f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c2ab52, 43 },
    { 0, 0 },
    { 0x1652ab54, 23 },
    { 0, 0 },
    { 0xc942ab56, 98 },
    { 0xccd2ab57, 23 },
    { 0xfe42ab58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6242ab5c, 90 },
    { 0, 0 },
    { 0x77f2ab5e, 33 },
    { 0xab92ab5e, 90 },
    { 0x29a2ab60, 90 },
    { 0x9222ab61, 61 },
    { 0, 0 },
    { 0x1d52ab63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2ab67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b2ab6f, 90 },
    { 0x7fe2ab70, 43 },
    { 0xe22ab71, 77 },
    { 0x2fe2ab70, 43 },
    { 0xc502ab70, 93 },
    { 0xf312ab71, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d2ab78, 95 },
    { 0xb342ab78, 98 },
    { 0, 0 },
    { 0xb452ab7b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a22ab7e, 33 },
    { 0x292ab7e, 33 },
    { 0xf922ab80, 95 },
    { 0x2002ab81, 67 },
    { 0xd522ab82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde2ab87, 33 },
    { 0x3892ab88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a2ab8f, 55 },
    { 0x49d2ab90, 95 },
    { 0, 0 },
    { 0x4b52ab92, 55 },
    { 0, 0 },
    { 0xbe52ab94, 78 },
    { 0x96b2ab94, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde22ab9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c52ab9e, 76 },
    { 0, 0 },
    { 0xaee2aba0, 55 },
    { 0x3472aba1, 91 },
    { 0, 0 },
    { 0x56b2aba3, 90 },
    { 0x3182aba3, 90 },
    { 0x6812aba5, 33 },
    { 0x7542aba5, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb32aba9, 90 },
    { 0x8a12aba9, 76 },
    { 0x3b02aba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a2abae, 90 },
    { 0, 0 },
    { 0xee82abb0, 66 },
    { 0x8e62abb1, 95 },
    { 0x14c2abb1, 33 },
    { 0xe4c2abb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2abb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb302abba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252abbd, 91 },
    { 0x3e02abbe, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b2abc1, 61 },
    { 0x4222abc2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e2abc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4252abcd, 33 },
    { 0, 0 },
    { 0x7302abcf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2abd4, 90 },
    { 0, 0 },
    { 0xa2abd6, 33 },
    { 0, 0 },
    { 0xee82abd8, 85 },
    { 0, 0 },
    { 0x5602abda, 34 },
    { 0x2002abda, 55 },
    { 0xfb12abdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c2abdf, 95 },
    { 0x1e62abe0, 33 },
    { 0, 0 },
    { 0xb3b2abe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7752abe7, 66 },
    { 0x76d2abe8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d2abec, 90 },
    { 0x3822abed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe902abf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c2abf4, 76 },
    { 0x38e2abf5, 34 },
    { 0x6ed2abf6, 85 },
    { 0xc1e2abf7, 77 },
    { 0, 0 },
    { 0xddb2abf9, 90 },
    { 0xbd52abf9, 100 },
    { 0x6e72abfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa362ac03, 90 },
    { 0, 0 },
    { 0x77b2ac05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf552ac0d, 90 },
    { 0, 0 },
    { 0xf8d2ac0f, 90 },
    { 0xac32ac10, 87 },
    { 0, 0 },
    { 0x7c32ac12, 36 },
    { 0x91b2ac12, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db2ac16, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9382ac1c, 33 },
    { 0, 0 },
    { 0x8202ac1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5372ac25, 76 },
    { 0xf062ac25, 90 },
    { 0, 0 },
    { 0x8af2ac28, 33 },
    { 0, 0 },
    { 0xa232ac2a, 55 },
    { 0, 0 },
    { 0xbb92ac2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f02ac32, 23 },
    { 0, 0 },
    { 0xa352ac34, 23 },
    { 0, 0 },
    { 0xa532ac36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb102ac3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f2ac3e, 90 },
    { 0, 0 },
    { 0x76b2ac40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d82ac45, 40 },
    { 0x79d2ac46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92ac4a, 90 },
    { 0x1392ac4b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1542ac50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d52ac5a, 90 },
    { 0x8862ac5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b52ac5e, 61 },
    { 0, 0 },
    { 0xd5a2ac60, 33 },
    { 0, 0 },
    { 0x9e92ac62, 95 },
    { 0xc1c2ac62, 85 },
    { 0xb342ac64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f2ac68, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc2ac75, 99 },
    { 0x19a2ac75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c2ac7b, 87 },
    { 0x5812ac7c, 95 },
    { 0x6c82ac7d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d62ac82, 33 },
    { 0x3612ac82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe132ac86, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b42ac8b, 77 },
    { 0, 0 },
    { 0x50e2ac8d, 98 },
    { 0, 0 },
    { 0xebc2ac8f, 78 },
    { 0x8002ac8f, 76 },
    { 0x3292ac91, 76 },
    { 0x9da2ac92, 88 },
    { 0x6772ac93, 51 },
    { 0x2902ac8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2362ac97, 76 },
    { 0xe022ac97, 65 },
    { 0, 0 },
    { 0xf5e2ac9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde2aca6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc32acaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc132acb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d2acb9, 43 },
    { 0x92e2acba, 76 },
    { 0x6b32acbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e2acbe, 34 },
    { 0xf202acbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3572acc2, 65 },
    { 0x5f2acc2, 23 },
    { 0x8ad2acc4, 33 },
    { 0x82acc5, 33 },
    { 0x57b2acc4, 43 },
    { 0xa362acc6, 23 },
    { 0x46b2acc7, 76 },
    { 0x3282acc3, 90 },
    { 0, 0 },
    { 0x5152accb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc32accf, 43 },
    { 0x4672acd0, 66 },
    { 0xe632acd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a2acd4, 33 },
    { 0x17e2acd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e2acda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef2acdd, 61 },
    { 0xe532acde, 61 },
    { 0xf6a2acdf, 61 },
    { 0xdc62acde, 98 },
    { 0x4ed2ace1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b2ace4, 95 },
    { 0xf62ace5, 43 },
    { 0, 0 },
    { 0x6802ace7, 23 },
    { 0x892ace8, 55 },
    { 0xb7c2ace8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4182acf1, 81 },
    { 0xa422acf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a2acfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac2ad04, 84 },
    { 0xc6f2ad05, 62 },
    { 0x7e02ad06, 33 },
    { 0, 0 },
    { 0x2722ad08, 66 },
    { 0x5b62ad09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2ad0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a62ad14, 55 },
    { 0x1052ad15, 95 },
    { 0xd592ad16, 61 },
    { 0, 0 },
    { 0xef22ad18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2ad22, 95 },
    { 0, 0 },
    { 0x1632ad24, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1552ad29, 98 },
    { 0xb682ad2a, 90 },
    { 0, 0 },
    { 0xa332ad2c, 90 },
    { 0xe022ad2d, 98 },
    { 0x1262ad2e, 67 },
    { 0x9be2ad2f, 90 },
    { 0, 0 },
    { 0x802ad31, 98 },
    { 0xb292ad32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3382ad36, 55 },
    { 0xf2d2ad37, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2ad3c, 76 },
    { 0, 0 },
    { 0x5eb2ad3e, 77 },
    { 0x4db2ad3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6182ad47, 52 },
    { 0xdb82ad48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e2ad4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f82ad50, 34 },
    { 0x2642ad51, 33 },
    { 0x2032ad50, 46 },
    { 0x27c2ad53, 23 },
    { 0xf982ad54, 100 },
    { 0xaf02ad55, 67 },
    { 0x5902ad55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f2ad5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322ad61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed2ad67, 43 },
    { 0x6b22ad67, 33 },
    { 0xe932ad69, 67 },
    { 0xc632ad6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4352ad6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d2ad74, 91 },
    { 0x882ad74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8232ad84, 90 },
    { 0x4cd2ad85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2ad8a, 99 },
    { 0, 0 },
    { 0x4e02ad8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d82ad8f, 99 },
    { 0x8782ad90, 77 },
    { 0x1422ad91, 68 },
    { 0x4992ad90, 34 },
    { 0, 0 },
    { 0xac22ad94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3692ad9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe202ada2, 33 },
    { 0xbdd2ada3, 98 },
    { 0, 0 },
    { 0x2c12ada5, 61 },
    { 0x5ba2ada6, 95 },
    { 0, 0 },
    { 0xde12ada8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5002adac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92adb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d02adb5, 61 },
    { 0xd752adb5, 95 },
    { 0, 0 },
    { 0x94c2adb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad22adbb, 98 },
    { 0x9fd2adbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef2adc5, 76 },
    { 0x6232adc5, 98 },
    { 0xe772adc7, 61 },
    { 0xce72adc7, 95 },
    { 0x6742adc9, 95 },
    { 0, 0 },
    { 0xb802adcb, 66 },
    { 0, 0 },
    { 0x3e2adcd, 90 },
    { 0xa8a2adce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2add3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa02add6, 76 },
    { 0xccc2add6, 55 },
    { 0x5012add8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa672addb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea52addf, 90 },
    { 0xc592ade0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1782ade5, 95 },
    { 0x7572ade5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a2ade9, 33 },
    { 0xe482ade9, 93 },
    { 0, 0 },
    { 0xc172adec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9402adef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac42adf2, 60 },
    { 0xf602adf3, 33 },
    { 0x66b2adf4, 76 },
    { 0xa912adf3, 84 },
    { 0xa8f2adf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc672adf9, 61 },
    { 0, 0 },
    { 0x7ba2adfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6662ae01, 90 },
    { 0, 0 },
    { 0x7562ae03, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3482ae0d, 95 },
    { 0x3ee2ae0e, 64 },
    { 0x7a92ae0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d2ae13, 90 },
    { 0x5ea2ae14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x532ae17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b02ae1b, 99 },
    { 0, 0 },
    { 0x7cc2ae1d, 33 },
    { 0xdc82ae1d, 90 },
    { 0x5ef2ae1f, 55 },
    { 0, 0 },
    { 0xeb42ae21, 43 },
    { 0xd5a2ae21, 61 },
    { 0xe0f2ae21, 43 },
    { 0, 0 },
    { 0xdd82ae25, 98 },
    { 0, 0 },
    { 0xf042ae27, 55 },
    { 0, 0 },
    { 0x83c2ae29, 23 },
    { 0, 0 },
    { 0x7962ae2b, 77 },
    { 0xa842ae2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642ae31, 78 },
    { 0xfda2ae31, 46 },
    { 0, 0 },
    { 0xc492ae34, 61 },
    { 0, 0 },
    { 0x83d2ae36, 90 },
    { 0, 0 },
    { 0x3662ae38, 52 },
    { 0xd812ae39, 61 },
    { 0x64e2ae39, 23 },
    { 0x3152ae3b, 55 },
    { 0x9b2ae3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e52ae42, 87 },
    { 0x8bd2ae43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ae4e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac2ae53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42ae56, 90 },
    { 0x1cf2ae57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa822ae5a, 34 },
    { 0, 0 },
    { 0x8ad2ae5c, 90 },
    { 0x9432ae5c, 95 },
    { 0x6d52ae5c, 90 },
    { 0xfab2ae5f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5002ae68, 76 },
    { 0x66c2ae68, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a2ae70, 84 },
    { 0xa792ae70, 56 },
    { 0x51d2ae72, 53 },
    { 0xb522ae73, 55 },
    { 0x98e2ae72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2ae78, 76 },
    { 0xedc2ae78, 98 },
    { 0x6e42ae78, 95 },
    { 0, 0 },
    { 0x3cc2ae7c, 90 },
    { 0xb612ae7c, 52 },
    { 0xba22ae7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832ae81, 85 },
    { 0x21c2ae82, 95 },
    { 0x54e2ae81, 66 },
    { 0xf652ae84, 90 },
    { 0x63a2ae83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c2ae88, 55 },
    { 0xc122ae89, 33 },
    { 0x3a42ae8a, 55 },
    { 0x2482ae8a, 61 },
    { 0x4ae2ae8c, 95 },
    { 0xd262ae8c, 61 },
    { 0x1c32ae8e, 99 },
    { 0x86b2ae8e, 99 },
    { 0x762ae90, 76 },
    { 0xa432ae90, 98 },
    { 0x6702ae92, 85 },
    { 0x3902ae93, 23 },
    { 0x4fd2ae93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc2ae9e, 40 },
    { 0x27d2ae9e, 33 },
    { 0x3b22aea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb232aea7, 90 },
    { 0x2a02aea8, 33 },
    { 0x3e82aea9, 81 },
    { 0, 0 },
    { 0x69d2aeab, 93 },
    { 0, 0 },
    { 0x7952aead, 23 },
    { 0, 0 },
    { 0x1bf2aeaf, 80 },
    { 0, 0 },
    { 0x882aeb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee72aeb4, 78 },
    { 0x8d2aeb5, 81 },
    { 0x8612aeb4, 23 },
    { 0xe982aeb4, 61 },
    { 0xd232aeb4, 34 },
    { 0xdff2aeb8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6952aebd, 100 },
    { 0xd6d2aebd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa062aec7, 67 },
    { 0x23a2aec7, 78 },
    { 0xb282aec9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2aecd, 61 },
    { 0xf62aece, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3112aed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c2aed5, 95 },
    { 0, 0 },
    { 0xf6a2aed7, 100 },
    { 0x3f62aed8, 61 },
    { 0x95c2aed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e22aee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f2aeeb, 23 },
    { 0x6122aeec, 90 },
    { 0x3982aeec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a2aef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec32aefe, 33 },
    { 0x68d2aefe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6722af05, 36 },
    { 0, 0 },
    { 0xf432af07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2af0a, 59 },
    { 0, 0 },
    { 0x2712af0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f52af11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf322af16, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd2af1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b2af20, 95 },
    { 0, 0 },
    { 0xb702af22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c2af26, 90 },
    { 0xad42af26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d2af33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f2af36, 76 },
    { 0x4462af37, 99 },
    { 0, 0 },
    { 0x4ee2af39, 33 },
    { 0x4ae2af3a, 61 },
    { 0xa652af3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec82af3f, 90 },
    { 0xdd82af3f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x492af43, 90 },
    { 0xeeb2af44, 61 },
    { 0xb222af45, 98 },
    { 0, 0 },
    { 0x4112af47, 69 },
    { 0, 0 },
    { 0xda72af49, 61 },
    { 0xce2af4a, 61 },
    { 0x82af4a, 90 },
    { 0x9942af4c, 33 },
    { 0x7392af4d, 61 },
    { 0xcc02af4d, 95 },
    { 0xc562af4f, 55 },
    { 0x38f2af4e, 95 },
    { 0, 0 },
    { 0x9ae2af52, 84 },
    { 0xdec2af52, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f02af59, 88 },
    { 0x6802af5a, 34 },
    { 0xf6a2af59, 76 },
    { 0x9392af5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9282af60, 90 },
    { 0xe182af61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c2af67, 46 },
    { 0x1432af68, 88 },
    { 0, 0 },
    { 0x9042af6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde2af71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc2af75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a32af7a, 23 },
    { 0xa0a2af7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2202af7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb2af87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b82af8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b02af90, 33 },
    { 0x39f2af91, 84 },
    { 0xa5e2af92, 90 },
    { 0, 0 },
    { 0x752af94, 61 },
    { 0x69b2af95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae02af9b, 34 },
    { 0, 0 },
    { 0xec42af9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c2afa0, 78 },
    { 0, 0 },
    { 0xded2afa2, 66 },
    { 0, 0 },
    { 0x77b2afa4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb2afa7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd92afac, 61 },
    { 0x4662afad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c2afb1, 33 },
    { 0x91d2afb2, 95 },
    { 0x2d22afb1, 98 },
    { 0x83d2afb4, 76 },
    { 0, 0 },
    { 0xb682afb6, 85 },
    { 0xf092afb7, 95 },
    { 0x1e02afb8, 46 },
    { 0xd572afb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e32afbc, 34 },
    { 0xf9f2afbc, 69 },
    { 0, 0 },
    { 0xc592afbf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd2afc2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a82afc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd392afcb, 95 },
    { 0x8872afcc, 34 },
    { 0x7422afcd, 61 },
    { 0, 0 },
    { 0xff62afcf, 43 },
    { 0, 0 },
    { 0xdf12afd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b2afd4, 23 },
    { 0, 0 },
    { 0xcfe2afd6, 40 },
    { 0xa062afd7, 90 },
    { 0x932afd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b2afdd, 90 },
    { 0xc12afde, 90 },
    { 0x4ac2afdf, 52 },
    { 0x9602afde, 43 },
    { 0, 0 },
    { 0x5b42afe2, 76 },
    { 0, 0 },
    { 0xdb52afe4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e42afed, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc2aff0, 87 },
    { 0x58d2aff1, 33 },
    { 0xc2a2aff1, 43 },
    { 0x8332aff1, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e2aff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd742affa, 91 },
    { 0, 0 },
    { 0xe632affc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e02b001, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc172b008, 23 },
    { 0x20c2b008, 61 },
    { 0, 0 },
    { 0x7542b00b, 90 },
    { 0, 0 },
    { 0xd812b00d, 61 },
    { 0xd482b00d, 55 },
    { 0x3772b00f, 96 },
    { 0x7442b00d, 95 },
    { 0, 0 },
    { 0xe612b012, 76 },
    { 0x1c22b012, 90 },
    { 0x3e92b013, 43 },
    { 0xbee2b015, 61 },
    { 0, 0 },
    { 0xb422b017, 93 },
    { 0xad52b018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb32b01b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e2b01e, 95 },
    { 0x222b01e, 95 },
    { 0, 0 },
    { 0x6a12b021, 90 },
    { 0, 0 },
    { 0x59a2b023, 61 },
    { 0x6502b024, 33 },
    { 0xf252b024, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e2b02d, 100 },
    { 0xc2e2b02d, 61 },
    { 0xcf42b02f, 61 },
    { 0x5542b030, 95 },
    { 0x7b72b030, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf92b036, 85 },
    { 0, 0 },
    { 0xa702b038, 87 },
    { 0x5002b038, 34 },
    { 0, 0 },
    { 0x6662b03b, 88 },
    { 0, 0 },
    { 0x5002b03d, 33 },
    { 0, 0 },
    { 0xca52b03f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c2b044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f2b04a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9012b04e, 61 },
    { 0x1ff2b04f, 90 },
    { 0x5e52b050, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea2b059, 67 },
    { 0, 0 },
    { 0xdeb2b05b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb12b05e, 55 },
    { 0, 0 },
    { 0x8062b060, 83 },
    { 0xc572b060, 96 },
    { 0x7652b062, 98 },
    { 0x7a22b063, 46 },
    { 0x9332b062, 90 },
    { 0xea72b065, 34 },
    { 0x7202b066, 91 },
    { 0xcff2b067, 61 },
    { 0xf122b065, 33 },
    { 0xfcf2b067, 33 },
    { 0xf0e2b06a, 95 },
    { 0x6502b06b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c02b072, 90 },
    { 0, 0 },
    { 0xab22b074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c2b077, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b02b07c, 90 },
    { 0x4772b07d, 90 },
    { 0, 0 },
    { 0x7442b07f, 46 },
    { 0, 0 },
    { 0xaa42b081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9792b085, 90 },
    { 0x9602b085, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a2b089, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be2b08d, 90 },
    { 0, 0 },
    { 0x7e62b08f, 23 },
    { 0x7e02b08f, 23 },
    { 0x7652b091, 90 },
    { 0xe172b092, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f2b095, 55 },
    { 0xae02b096, 61 },
    { 0x17c2b095, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72b09a, 55 },
    { 0x5612b09a, 43 },
    { 0x9ae2b09b, 95 },
    { 0x9432b09b, 68 },
    { 0x7fa2b09e, 98 },
    { 0xdec2b09f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252b0a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9472b0a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5242b0ac, 81 },
    { 0x8c52b0ad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc502b0b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d2b0b6, 34 },
    { 0xf422b0b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea82b0ba, 99 },
    { 0, 0 },
    { 0xe1b2b0bc, 81 },
    { 0, 0 },
    { 0x6fd2b0be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf52b0c1, 23 },
    { 0x1a62b0c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd692b0cf, 90 },
    { 0, 0 },
    { 0x88f2b0d1, 61 },
    { 0x2822b0d1, 40 },
    { 0x7bd2b0d1, 98 },
    { 0, 0 },
    { 0x4282b0d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5902b0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e2b0de, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f22b0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb572b0ee, 40 },
    { 0xcfc2b0ef, 98 },
    { 0x9292b0f0, 98 },
    { 0x62c2b0ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee2b0f4, 88 },
    { 0, 0 },
    { 0xe82b0f6, 76 },
    { 0, 0 },
    { 0x2882b0f8, 98 },
    { 0x3c62b0f8, 90 },
    { 0xdac2b0fa, 23 },
    { 0xd12b0fa, 95 },
    { 0, 0 },
    { 0xe292b0fd, 95 },
    { 0xdc92b0fe, 90 },
    { 0x2fb2b0fe, 95 },
    { 0x1792b100, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb42b106, 90 },
    { 0xbc42b107, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc72b10c, 68 },
    { 0x16e2b10d, 17 },
    { 0, 0 },
    { 0x13c2b10f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a02b116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a2b11a, 90 },
    { 0, 0 },
    { 0x8422b11c, 78 },
    { 0, 0 },
    { 0x97a2b11e, 66 },
    { 0, 0 },
    { 0x592b120, 99 },
    { 0, 0 },
    { 0x7872b122, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3782b125, 68 },
    { 0x6fa2b126, 33 },
    { 0x4d62b127, 46 },
    { 0xe092b128, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1692b12f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b2b134, 93 },
    { 0x2782b135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c2b139, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7172b13d, 98 },
    { 0x5892b13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7922b142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa2b147, 93 },
    { 0xc022b148, 33 },
    { 0x9692b147, 95 },
    { 0x4db2b14a, 85 },
    { 0x1d12b148, 66 },
    { 0x8d32b14a, 76 },
    { 0, 0 },
    { 0x2332b14e, 43 },
    { 0xbe32b14e, 34 },
    { 0, 0 },
    { 0x1232b151, 34 },
    { 0, 0 },
    { 0x4322b153, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c52b156, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd62b15e, 33 },
    { 0x2fe2b15e, 98 },
    { 0, 0 },
    { 0x6c92b161, 85 },
    { 0x4de2b161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f2b169, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec2b16c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1922b170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc162b175, 76 },
    { 0, 0 },
    { 0x3c12b177, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd832b17b, 90 },
    { 0x482b17b, 79 },
    { 0x7392b17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7712b182, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d2b187, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe022b18e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c2b191, 61 },
    { 0x96f2b192, 90 },
    { 0xad42b193, 55 },
    { 0, 0 },
    { 0x2012b195, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b2b199, 90 },
    { 0x3b22b19a, 33 },
    { 0x6132b19b, 100 },
    { 0x5eb2b19b, 93 },
    { 0x4e22b19b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2b1a0, 78 },
    { 0x3142b1a1, 34 },
    { 0x6522b1a2, 61 },
    { 0xee22b1a2, 34 },
    { 0, 0 },
    { 0xd362b1a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f2b1ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde22b1ae, 99 },
    { 0x66e2b1af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c2b1b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe82b1b8, 99 },
    { 0xa462b1b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d2b1bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042b1c0, 75 },
    { 0xb5c2b1c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1312b1c6, 81 },
    { 0, 0 },
    { 0x9522b1c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc2b1cc, 100 },
    { 0, 0 },
    { 0x4932b1ce, 34 },
    { 0xa092b1cf, 33 },
    { 0x92b1d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3252b1db, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5142b1df, 43 },
    { 0xb22b1e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2b1f2, 98 },
    { 0x4fb2b1f3, 61 },
    { 0x8f82b1f3, 90 },
    { 0x86b2b1f5, 98 },
    { 0xffb2b1f6, 85 },
    { 0x98d2b1f3, 31 },
    { 0x2432b1f6, 33 },
    { 0x1d72b1f9, 34 },
    { 0x5c2b1fa, 34 },
    { 0xc5b2b1f8, 34 },
    { 0x66b2b1f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2742b1ff, 55 },
    { 0, 0 },
    { 0x3072b201, 33 },
    { 0xef02b201, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c72b20d, 34 },
    { 0x6952b20d, 90 },
    { 0xa2d2b20f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6572b213, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b218, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072b220, 98 },
    { 0, 0 },
    { 0x90a2b222, 98 },
    { 0x8ee2b222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a2b22a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b62b22d, 44 },
    { 0xda82b22e, 100 },
    { 0xc482b22f, 61 },
    { 0xb3b2b22e, 90 },
    { 0xb222b22f, 23 },
    { 0x7412b231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf2b236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b2b239, 88 },
    { 0, 0 },
    { 0xe642b23b, 98 },
    { 0xb602b23c, 55 },
    { 0x3922b23d, 93 },
    { 0x9422b23e, 23 },
    { 0x8b32b23e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5612b243, 90 },
    { 0, 0 },
    { 0xc382b245, 43 },
    { 0, 0 },
    { 0x4a62b247, 34 },
    { 0x2632b248, 93 },
    { 0, 0 },
    { 0x7ad2b24a, 95 },
    { 0, 0 },
    { 0xe462b24c, 33 },
    { 0x19d2b24c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed92b250, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6092b253, 33 },
    { 0xf682b253, 55 },
    { 0x6d92b254, 78 },
    { 0x77d2b256, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b25c, 88 },
    { 0xa232b25c, 90 },
    { 0xc4e2b25d, 98 },
    { 0, 0 },
    { 0xfdf2b260, 66 },
    { 0x3fe2b261, 33 },
    { 0x8232b260, 67 },
    { 0x98d2b263, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b2b266, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef2b26b, 43 },
    { 0xd9c2b26c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3122b275, 100 },
    { 0x6e62b276, 61 },
    { 0xa9c2b277, 43 },
    { 0x8ae2b278, 55 },
    { 0xfe32b277, 33 },
    { 0x28d2b276, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe942b27d, 77 },
    { 0xd8b2b27e, 90 },
    { 0x4fb2b27f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6062b283, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xda32b286, 52 },
    { 0x44d2b286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1362b28a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2b28d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd02b291, 61 },
    { 0x2482b292, 95 },
    { 0x4e62b293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa472b298, 90 },
    { 0x6932b298, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd322b29d, 98 },
    { 0xb9e2b29e, 81 },
    { 0x5f42b29f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a2b2a2, 91 },
    { 0x7142b2a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe802b2ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf752b2b4, 46 },
    { 0xf472b2b5, 34 },
    { 0x6602b2b6, 95 },
    { 0, 0 },
    { 0xd7b2b2b8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa972b2bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd642b2c3, 93 },
    { 0xc312b2c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd42b2ca, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc2b2ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd2b2d5, 90 },
    { 0, 0 },
    { 0xe062b2d7, 55 },
    { 0xe5c2b2d8, 61 },
    { 0xbe32b2d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb272b2e3, 33 },
    { 0, 0 },
    { 0xad52b2e5, 61 },
    { 0, 0 },
    { 0x6162b2e7, 66 },
    { 0xc502b2e7, 66 },
    { 0x2722b2e7, 33 },
    { 0x6922b2ea, 90 },
    { 0, 0 },
    { 0x6832b2ec, 61 },
    { 0, 0 },
    { 0x3b42b2ee, 61 },
    { 0, 0 },
    { 0x7b02b2f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2b2f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca2b2fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae02b2fd, 98 },
    { 0xcd62b2fe, 17 },
    { 0xb812b2ff, 95 },
    { 0, 0 },
    { 0xebc2b301, 85 },
    { 0, 0 },
    { 0x99d2b303, 100 },
    { 0, 0 },
    { 0x9272b305, 61 },
    { 0xece2b305, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d2b30b, 75 },
    { 0, 0 },
    { 0x5a12b30d, 93 },
    { 0x9f2b30e, 43 },
    { 0x35e2b30f, 99 },
    { 0x102b310, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2b31a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf782b31d, 33 },
    { 0x442b31e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf682b321, 95 },
    { 0xd442b322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2152b327, 40 },
    { 0x3362b328, 46 },
    { 0xcab2b327, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd52b335, 90 },
    { 0xa6b2b336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df2b339, 17 },
    { 0, 0 },
    { 0xd992b33b, 90 },
    { 0, 0 },
    { 0x5502b33d, 66 },
    { 0xa1b2b33d, 98 },
    { 0, 0 },
    { 0xd452b340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a12b343, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3892b346, 90 },
    { 0xf532b347, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x562b34a, 33 },
    { 0, 0 },
    { 0xcf32b34c, 43 },
    { 0x3fe2b34d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72b352, 61 },
    { 0, 0 },
    { 0x3752b354, 90 },
    { 0x6c42b355, 88 },
    { 0x1112b356, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a82b35b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd62b35e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3452b363, 95 },
    { 0x17e2b364, 23 },
    { 0, 0 },
    { 0x8c22b366, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc202b369, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba72b36f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa152b374, 33 },
    { 0x19a2b375, 98 },
    { 0x8b32b376, 90 },
    { 0x9152b374, 23 },
    { 0xfe22b376, 83 },
    { 0xced2b379, 85 },
    { 0x1d2b378, 98 },
    { 0x2b2b375, 52 },
    { 0xf052b37c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2b382, 95 },
    { 0x16b2b382, 93 },
    { 0, 0 },
    { 0x53a2b385, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac2b388, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92b38b, 49 },
    { 0x72d2b38c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3382b391, 33 },
    { 0, 0 },
    { 0x7a2b393, 61 },
    { 0x9692b394, 81 },
    { 0, 0 },
    { 0x942b396, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x292b399, 52 },
    { 0xdd62b39a, 17 },
    { 0x6c22b39a, 69 },
    { 0x9ff2b39c, 23 },
    { 0x7252b39d, 55 },
    { 0, 0 },
    { 0xdae2b39f, 90 },
    { 0x88b2b3a0, 90 },
    { 0x8612b3a1, 61 },
    { 0x2502b3a2, 90 },
    { 0x1d42b3a3, 23 },
    { 0, 0 },
    { 0x6c72b3a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb12b3a8, 88 },
    { 0, 0 },
    { 0xdc92b3aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd22b3ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d2b3b0, 68 },
    { 0xea62b3b1, 95 },
    { 0, 0 },
    { 0x7b82b3b3, 93 },
    { 0, 0 },
    { 0xf8b2b3b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e12b3c1, 66 },
    { 0x9582b3c2, 90 },
    { 0x6452b3c3, 95 },
    { 0x3b32b3c3, 78 },
    { 0xe282b3c2, 23 },
    { 0, 0 },
    { 0x3462b3c7, 33 },
    { 0xbc72b3c7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c2b3cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc172b3d0, 95 },
    { 0xcfe2b3d1, 93 },
    { 0x3a22b3d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e32b3d5, 88 },
    { 0, 0 },
    { 0x8702b3d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2b3de, 98 },
    { 0, 0 },
    { 0x182b3e0, 93 },
    { 0xa672b3e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2782b3e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1332b3ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1812b3ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e72b3f0, 23 },
    { 0x5b22b3f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe562b3f4, 93 },
    { 0x2432b3f4, 33 },
    { 0x8462b3f6, 64 },
    { 0xae42b3f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2b3fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c2b3fd, 61 },
    { 0xfa02b3fe, 80 },
    { 0x7302b3ff, 17 },
    { 0x412b3fd, 43 },
    { 0, 0 },
    { 0x4b72b402, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a2b40a, 90 },
    { 0x68e2b40b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b32b412, 23 },
    { 0x6442b413, 33 },
    { 0x7a2b414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be2b418, 23 },
    { 0x89a2b419, 76 },
    { 0xda02b41a, 46 },
    { 0x6ea2b41b, 76 },
    { 0x8a42b419, 61 },
    { 0xe602b41c, 90 },
    { 0, 0 },
    { 0xe8e2b41f, 55 },
    { 0, 0 },
    { 0x91f2b421, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc872b427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf452b42a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f2b42e, 45 },
    { 0, 0 },
    { 0x8e42b430, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf02b433, 93 },
    { 0x3472b434, 91 },
    { 0xbe22b435, 100 },
    { 0x18e2b435, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b22b43e, 61 },
    { 0x2982b43f, 23 },
    { 0xd5e2b43e, 90 },
    { 0x5f92b441, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe082b444, 46 },
    { 0x212b444, 33 },
    { 0, 0 },
    { 0x2552b447, 36 },
    { 0xc432b447, 23 },
    { 0x9542b448, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e42b44c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2b451, 33 },
    { 0, 0 },
    { 0xa402b453, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2b456, 85 },
    { 0xf772b456, 23 },
    { 0, 0 },
    { 0xcf2b459, 95 },
    { 0x62b2b45a, 66 },
    { 0x2092b45a, 33 },
    { 0x19d2b45b, 76 },
    { 0xd412b45c, 95 },
    { 0, 0 },
    { 0xf4a2b45f, 99 },
    { 0, 0 },
    { 0xe42b461, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2092b466, 61 },
    { 0x7f12b466, 87 },
    { 0x7602b468, 93 },
    { 0xb372b469, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2b46e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292b471, 84 },
    { 0, 0 },
    { 0x8462b473, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3392b477, 61 },
    { 0x31d2b477, 61 },
    { 0, 0 },
    { 0xcd32b47a, 93 },
    { 0x8b52b47b, 34 },
    { 0, 0 },
    { 0xb0d2b47d, 98 },
    { 0x35f2b47e, 36 },
    { 0x70d2b47f, 87 },
    { 0x6642b480, 34 },
    { 0xd502b481, 33 },
    { 0x1712b482, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xce02b485, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c2b48a, 46 },
    { 0x742b48b, 46 },
    { 0x2d62b48c, 52 },
    { 0x9112b48b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5092b492, 23 },
    { 0x3c82b492, 95 },
    { 0xddd2b492, 90 },
    { 0xca02b495, 65 },
    { 0x68f2b495, 95 },
    { 0, 0 },
    { 0x7802b498, 98 },
    { 0x2a02b499, 84 },
    { 0x2232b499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2b49e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef2b4a1, 95 },
    { 0xa992b4a2, 93 },
    { 0, 0 },
    { 0x4a92b4a4, 100 },
    { 0x8492b4a5, 33 },
    { 0x7042b4a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d2b4aa, 23 },
    { 0xc0b2b4ab, 61 },
    { 0x4492b4ab, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d2b4b0, 95 },
    { 0xb192b4b1, 75 },
    { 0x79e2b4b2, 17 },
    { 0x902b4b3, 90 },
    { 0, 0 },
    { 0x48f2b4b5, 55 },
    { 0, 0 },
    { 0x9c72b4b7, 95 },
    { 0, 0 },
    { 0xa022b4b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa2b4bd, 77 },
    { 0x36f2b4be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa2b4c1, 23 },
    { 0, 0 },
    { 0x3bc2b4c3, 34 },
    { 0x6952b4c3, 91 },
    { 0x15b2b4c3, 33 },
    { 0x3372b4c3, 61 },
    { 0xf3f2b4c7, 66 },
    { 0, 0 },
    { 0xf562b4c9, 33 },
    { 0x5592b4ca, 55 },
    { 0, 0 },
    { 0x8712b4cc, 61 },
    { 0xad22b4cc, 98 },
    { 0x6702b4ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf052b4d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422b4d7, 23 },
    { 0x5f2b4d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2b4dd, 23 },
    { 0xb932b4de, 87 },
    { 0, 0 },
    { 0x87b2b4e0, 17 },
    { 0, 0 },
    { 0xffc2b4e2, 93 },
    { 0x21e2b4e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc972b4ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9632b4f6, 55 },
    { 0x4412b4f7, 66 },
    { 0xab52b4f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b12b4fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf22b4ff, 61 },
    { 0xddf2b500, 68 },
    { 0x762b500, 33 },
    { 0, 0 },
    { 0x6692b503, 95 },
    { 0x62a2b503, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2b507, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9782b50c, 33 },
    { 0x6bc2b50d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba2b513, 61 },
    { 0x34b2b513, 98 },
    { 0xeed2b513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d2b519, 23 },
    { 0x6262b51a, 81 },
    { 0xddd2b51b, 53 },
    { 0x4a22b51c, 95 },
    { 0xce22b51c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02b520, 90 },
    { 0x59e2b520, 23 },
    { 0x21a2b521, 90 },
    { 0xdba2b523, 38 },
    { 0xa782b524, 90 },
    { 0xfc12b524, 68 },
    { 0, 0 },
    { 0x3902b527, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6632b52a, 33 },
    { 0, 0 },
    { 0x7492b52c, 23 },
    { 0x4402b52d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf2b531, 33 },
    { 0, 0 },
    { 0x5dd2b533, 67 },
    { 0xafb2b534, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b2b539, 95 },
    { 0xf432b53a, 55 },
    { 0x9852b53a, 61 },
    { 0x3312b53b, 98 },
    { 0x9082b53a, 90 },
    { 0xf752b53a, 90 },
    { 0xb952b53f, 55 },
    { 0x5e62b540, 33 },
    { 0x9af2b541, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf272b548, 55 },
    { 0xd3b2b549, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b2b54c, 34 },
    { 0, 0 },
    { 0xb032b54e, 23 },
    { 0, 0 },
    { 0x23d2b550, 61 },
    { 0x7062b550, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4442b558, 33 },
    { 0, 0 },
    { 0xdad2b55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4852b55d, 90 },
    { 0x57b2b55e, 46 },
    { 0x2f22b55f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd302b563, 90 },
    { 0x23a2b564, 61 },
    { 0xffb2b564, 90 },
    { 0, 0 },
    { 0x4d92b567, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f2b56b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe542b572, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb932b578, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe512b57c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a2b58b, 77 },
    { 0x76a2b58b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd452b58f, 34 },
    { 0x4432b58f, 98 },
    { 0x8922b591, 90 },
    { 0x75d2b591, 4 },
    { 0x5022b593, 23 },
    { 0x152b58f, 90 },
    { 0xf0a2b590, 61 },
    { 0xfa92b596, 95 },
    { 0xc7b2b591, 78 },
    { 0x3392b598, 85 },
    { 0x8702b599, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d2b59d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d72b5a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1022b5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212b5aa, 33 },
    { 0, 0 },
    { 0xb0d2b5ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc072b5af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e62b5b4, 33 },
    { 0, 0 },
    { 0x3b02b5b6, 90 },
    { 0x19e2b5b7, 78 },
    { 0x5742b5b7, 23 },
    { 0x4232b5b9, 93 },
    { 0xa772b5b8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d2b5bd, 95 },
    { 0, 0 },
    { 0x55b2b5bf, 23 },
    { 0, 0 },
    { 0xe3b2b5c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8362b5c5, 87 },
    { 0x7262b5c6, 95 },
    { 0xea02b5c5, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba2b5cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9872b5cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7342b5d2, 33 },
    { 0x7a92b5d3, 33 },
    { 0xd752b5d4, 33 },
    { 0x4f2b5d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4592b5d8, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b92b5dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf062b5e8, 33 },
    { 0xf612b5e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7692b5ed, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa22b5f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc2b5f6, 78 },
    { 0xd172b5f7, 90 },
    { 0x4102b5f6, 98 },
    { 0xbe72b5f6, 95 },
    { 0x51f2b5f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd2b5fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c2b600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6252b605, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812b611, 95 },
    { 0, 0 },
    { 0x8342b613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd32b616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf072b61c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a2b620, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e42b623, 33 },
    { 0x3dd2b624, 33 },
    { 0, 0 },
    { 0x5a62b626, 99 },
    { 0, 0 },
    { 0x11b2b628, 43 },
    { 0x2f52b629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8022b62c, 95 },
    { 0xd7d2b62d, 43 },
    { 0x30c2b62e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7622b631, 33 },
    { 0x6092b632, 33 },
    { 0x47c2b632, 76 },
    { 0xa012b631, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2b63a, 23 },
    { 0xd072b63b, 23 },
    { 0x9062b63b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2312b640, 61 },
    { 0x36f2b641, 90 },
    { 0x132b640, 43 },
    { 0xb332b643, 98 },
    { 0x4e82b643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e2b648, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3772b64e, 95 },
    { 0, 0 },
    { 0xa912b650, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c2b653, 90 },
    { 0, 0 },
    { 0xb042b655, 77 },
    { 0xf612b655, 91 },
    { 0xeb2b656, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa412b65c, 34 },
    { 0x2712b65d, 81 },
    { 0xf2a2b65d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f62b663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752b667, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7032b681, 90 },
    { 0x6d2b681, 61 },
    { 0, 0 },
    { 0xe22b684, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa572b68a, 90 },
    { 0x2af2b68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552b68f, 85 },
    { 0xe852b690, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42b693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552b696, 23 },
    { 0xb9f2b697, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8002b6a1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb432b6a4, 33 },
    { 0x1cd2b6a5, 68 },
    { 0, 0 },
    { 0xdd32b6a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d2b6ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c32b6ae, 66 },
    { 0xb312b6af, 65 },
    { 0xe552b6af, 52 },
    { 0x9682b6b1, 91 },
    { 0x4dd2b6b2, 93 },
    { 0x482b6b3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb272b6be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a2b6c3, 91 },
    { 0x6902b6c4, 95 },
    { 0x86f2b6c5, 33 },
    { 0, 0 },
    { 0x1e12b6c7, 95 },
    { 0x4e02b6c7, 90 },
    { 0, 0 },
    { 0x5452b6ca, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be2b6cd, 38 },
    { 0xb7b2b6ce, 34 },
    { 0, 0 },
    { 0xd402b6d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe572b6d3, 90 },
    { 0xc552b6d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8772b6d8, 98 },
    { 0x4c62b6d9, 91 },
    { 0xdca2b6da, 100 },
    { 0xaaa2b6db, 52 },
    { 0, 0 },
    { 0xaf02b6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe212b6e4, 52 },
    { 0x2952b6e5, 43 },
    { 0x562b6e6, 95 },
    { 0x3dd2b6e7, 90 },
    { 0xd7f2b6e7, 76 },
    { 0xa622b6e6, 99 },
    { 0x6562b6ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a42b6ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b2b6f1, 100 },
    { 0x2382b6f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd52b6f9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7932b6fd, 55 },
    { 0x8462b6fd, 98 },
    { 0x1cf2b6ff, 23 },
    { 0x9a22b700, 55 },
    { 0xd452b700, 98 },
    { 0, 0 },
    { 0xe472b703, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe052b707, 61 },
    { 0xd1b2b708, 90 },
    { 0x5e02b707, 78 },
    { 0x54e2b708, 95 },
    { 0x2312b709, 90 },
    { 0xb912b70c, 98 },
    { 0, 0 },
    { 0x49b2b70e, 61 },
    { 0xe782b70e, 95 },
    { 0x6c62b70e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7522b713, 55 },
    { 0x7842b714, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6742b719, 33 },
    { 0, 0 },
    { 0xeaf2b71b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d2b720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb342b724, 61 },
    { 0xf3f2b724, 100 },
    { 0x1d82b726, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a82b729, 95 },
    { 0xb5f2b72a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d2b734, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e2b739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532b744, 76 },
    { 0, 0 },
    { 0xeb42b746, 95 },
    { 0, 0 },
    { 0x4432b748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2b750, 76 },
    { 0x94f2b751, 61 },
    { 0x2a52b750, 61 },
    { 0x5a82b753, 33 },
    { 0x5ae2b754, 83 },
    { 0, 0 },
    { 0xb9a2b756, 66 },
    { 0xad92b757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7492b75a, 23 },
    { 0, 0 },
    { 0xcd02b75c, 69 },
    { 0x5b72b75c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6222b762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe462b767, 93 },
    { 0xfce2b767, 38 },
    { 0, 0 },
    { 0xf52b76a, 95 },
    { 0xa1b2b76b, 76 },
    { 0x8342b76c, 78 },
    { 0, 0 },
    { 0x16a2b76e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa2b776, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c12b77e, 34 },
    { 0x5c12b77e, 93 },
    { 0, 0 },
    { 0xbe62b781, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db2b785, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5192b78b, 33 },
    { 0x5882b78b, 55 },
    { 0xae42b78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa72b793, 33 },
    { 0x14e2b794, 90 },
    { 0, 0 },
    { 0x9152b796, 34 },
    { 0xbe42b797, 95 },
    { 0x8f12b797, 76 },
    { 0x53b2b796, 46 },
    { 0x2bf2b79a, 61 },
    { 0, 0 },
    { 0x79d2b79c, 90 },
    { 0x51d2b79d, 33 },
    { 0, 0 },
    { 0x2522b79f, 98 },
    { 0x7cf2b79f, 33 },
    { 0, 0 },
    { 0x5832b7a2, 87 },
    { 0x62b2b7a3, 98 },
    { 0x4572b7a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a42b7a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e2b7ac, 23 },
    { 0, 0 },
    { 0x54a2b7ae, 76 },
    { 0xee02b7af, 46 },
    { 0xda2b7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b7b3, 98 },
    { 0x1552b7b4, 46 },
    { 0x1d22b7b5, 17 },
    { 0x7c52b7b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe2b7b9, 90 },
    { 0x4272b7ba, 40 },
    { 0, 0 },
    { 0x422b7bc, 43 },
    { 0, 0 },
    { 0x7b82b7be, 38 },
    { 0, 0 },
    { 0xb752b7c0, 34 },
    { 0x8d62b7c0, 23 },
    { 0x5cb2b7c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b2b7c8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff2b7cb, 95 },
    { 0xcf02b7cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc282b7cf, 90 },
    { 0, 0 },
    { 0xda62b7d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa612b7d4, 67 },
    { 0x82d2b7d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd2b7d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca2b7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4272b7e4, 90 },
    { 0xdf92b7e5, 68 },
    { 0xe7a2b7e6, 35 },
    { 0xa872b7e7, 34 },
    { 0xb582b7e8, 98 },
    { 0x42b2b7e4, 76 },
    { 0x64c2b7e8, 95 },
    { 0x9382b7ea, 85 },
    { 0x6122b7ec, 91 },
    { 0x12b2b7ed, 61 },
    { 0x58c2b7ee, 61 },
    { 0x53a2b7ee, 90 },
    { 0x8e82b7ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb2b7f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1882b7f6, 34 },
    { 0xe8c2b7f7, 66 },
    { 0x44e2b7f6, 52 },
    { 0x3a52b7f7, 90 },
    { 0, 0 },
    { 0xd8d2b7fb, 61 },
    { 0x26c2b7fc, 95 },
    { 0x7612b7fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c2b800, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea2b806, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c62b80a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e2b80e, 75 },
    { 0xaff2b80e, 95 },
    { 0xc632b810, 61 },
    { 0xbd52b811, 23 },
    { 0, 0 },
    { 0x81f2b813, 99 },
    { 0xfd52b814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac82b81a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a42b81d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa102b820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad92b825, 46 },
    { 0, 0 },
    { 0xb722b827, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d2b82e, 91 },
    { 0, 0 },
    { 0xaee2b830, 76 },
    { 0x6262b830, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b2b836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2b839, 55 },
    { 0, 0 },
    { 0x1f2b83b, 76 },
    { 0x3cd2b83c, 23 },
    { 0, 0 },
    { 0xc812b83e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2b841, 95 },
    { 0x9372b842, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd2b847, 90 },
    { 0x7552b847, 76 },
    { 0xc712b849, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7142b84d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df2b851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf492b855, 95 },
    { 0x2772b856, 90 },
    { 0, 0 },
    { 0x6dc2b858, 100 },
    { 0x6bd2b859, 95 },
    { 0, 0 },
    { 0xbdd2b85b, 33 },
    { 0xf562b85c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5432b85f, 66 },
    { 0, 0 },
    { 0xc972b861, 84 },
    { 0x5752b861, 33 },
    { 0, 0 },
    { 0x8122b864, 65 },
    { 0, 0 },
    { 0xe582b866, 93 },
    { 0x32d2b867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f2b86a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce2b86d, 23 },
    { 0xb7b2b86e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a2b872, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22b876, 23 },
    { 0x2712b877, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b2b87a, 61 },
    { 0, 0 },
    { 0xcab2b87c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6702b880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf32b883, 33 },
    { 0xef22b884, 52 },
    { 0x31a2b885, 46 },
    { 0xd422b886, 100 },
    { 0xf042b885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb702b88a, 61 },
    { 0, 0 },
    { 0xcfd2b88c, 93 },
    { 0, 0 },
    { 0x72e2b88e, 91 },
    { 0x732b88f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d82b892, 61 },
    { 0x4b72b893, 95 },
    { 0x7072b892, 67 },
    { 0, 0 },
    { 0xd562b896, 33 },
    { 0, 0 },
    { 0x5422b898, 90 },
    { 0x6ab2b899, 55 },
    { 0xd682b898, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb912b89d, 33 },
    { 0x85c2b89d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2b8a5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9352b8af, 95 },
    { 0x5ab2b8b0, 66 },
    { 0x3c2b8af, 98 },
    { 0xc032b8b2, 33 },
    { 0, 0 },
    { 0xe372b8b4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe862b8b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4972b8ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c2b8c1, 61 },
    { 0, 0 },
    { 0x8872b8c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4512b8c6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd772b8c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832b8cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352b8d0, 76 },
    { 0, 0 },
    { 0xeb42b8d2, 84 },
    { 0xe8b2b8d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2b8d9, 76 },
    { 0xc5d2b8da, 33 },
    { 0, 0 },
    { 0xc2e2b8dc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb042b8e0, 55 },
    { 0xec22b8e0, 87 },
    { 0xb352b8e0, 91 },
    { 0x4522b8e0, 90 },
    { 0x9502b8e0, 82 },
    { 0, 0 },
    { 0xde92b8e6, 90 },
    { 0xf372b8e6, 66 },
    { 0x2952b8e8, 95 },
    { 0xf802b8e9, 85 },
    { 0xa802b8e9, 91 },
    { 0, 0 },
    { 0x80e2b8ec, 61 },
    { 0, 0 },
    { 0x60d2b8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df2b8f1, 43 },
    { 0x1912b8f2, 76 },
    { 0x15e2b8f2, 90 },
    { 0x94d2b8f4, 34 },
    { 0x4672b8f4, 88 },
    { 0xb72b8f5, 90 },
    { 0xb3b2b8f6, 98 },
    { 0x6a42b8f7, 98 },
    { 0, 0 },
    { 0x8f92b8fa, 53 },
    { 0x3ee2b8fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a52b8fe, 52 },
    { 0x2012b8ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c82b907, 77 },
    { 0, 0 },
    { 0xfe2b909, 98 },
    { 0x4192b90a, 90 },
    { 0, 0 },
    { 0x9102b90c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5662b915, 98 },
    { 0x2c82b916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f2b920, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552b923, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe152b927, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1172b92a, 98 },
    { 0, 0 },
    { 0xc0d2b92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d2b92f, 90 },
    { 0, 0 },
    { 0x8782b931, 98 },
    { 0, 0 },
    { 0xfc02b933, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd2b937, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca02b93a, 96 },
    { 0, 0 },
    { 0x4622b93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b52b940, 46 },
    { 0xc6b2b940, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2702b947, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b2b94a, 90 },
    { 0x6a42b94a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7862b94f, 33 },
    { 0, 0 },
    { 0xc432b951, 76 },
    { 0, 0 },
    { 0x44b2b953, 33 },
    { 0xcba2b953, 90 },
    { 0x3a42b954, 92 },
    { 0xbd02b956, 32 },
    { 0xee72b956, 23 },
    { 0x6332b958, 34 },
    { 0, 0 },
    { 0x6ab2b95a, 34 },
    { 0, 0 },
    { 0xc382b95c, 61 },
    { 0, 0 },
    { 0x2a72b95e, 93 },
    { 0x1832b95f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2b963, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b52b96a, 61 },
    { 0x30f2b96b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6032b96f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6732b974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee22b97b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce2b980, 69 },
    { 0xe9d2b981, 95 },
    { 0x8c32b982, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d2b986, 95 },
    { 0, 0 },
    { 0x34f2b988, 33 },
    { 0x74b2b989, 61 },
    { 0x3822b98a, 33 },
    { 0xa342b98b, 95 },
    { 0xdeb2b98a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb692b990, 95 },
    { 0x1782b991, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b2b995, 61 },
    { 0x7d52b996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5252b99e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3612b9a2, 83 },
    { 0xfcd2b9a2, 90 },
    { 0x95b2b9a4, 76 },
    { 0xcd12b9a5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de2b9a8, 33 },
    { 0xe2b2b9a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd62b9af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d2b9b5, 93 },
    { 0x3af2b9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d2b9ba, 90 },
    { 0x2972b9ba, 23 },
    { 0, 0 },
    { 0x2cd2b9bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a2b9c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe432b9c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7002b9cc, 36 },
    { 0xc132b9cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc2b9d4, 67 },
    { 0, 0 },
    { 0xc8e2b9d6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa62b9dc, 90 },
    { 0xde72b9dc, 74 },
    { 0xbe12b9de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc2b9e4, 95 },
    { 0, 0 },
    { 0x98e2b9e6, 61 },
    { 0x1fc2b9e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f62b9ea, 61 },
    { 0x3bc2b9ea, 61 },
    { 0x4e82b9ec, 98 },
    { 0x2342b9ec, 61 },
    { 0x8382b9ee, 93 },
    { 0xf412b9ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc72b9f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302b9f5, 43 },
    { 0, 0 },
    { 0x9b92b9f7, 34 },
    { 0, 0 },
    { 0xa0c2b9f9, 33 },
    { 0, 0 },
    { 0xb1a2b9fb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf2b9fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f62ba02, 55 },
    { 0xf9e2ba02, 33 },
    { 0x1cd2ba03, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e2ba0a, 75 },
    { 0, 0 },
    { 0xdfa2ba0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4852ba0f, 23 },
    { 0x5932ba0f, 90 },
    { 0x4902ba11, 90 },
    { 0x13b2ba11, 84 },
    { 0x29a2ba13, 95 },
    { 0xc92ba14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb2ba17, 33 },
    { 0xf0a2ba17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea2ba23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f2ba26, 84 },
    { 0x9882ba27, 90 },
    { 0x7212ba28, 91 },
    { 0x2ba2ba26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552ba31, 66 },
    { 0, 0 },
    { 0x6f92ba33, 61 },
    { 0x222ba34, 23 },
    { 0xf4e2ba35, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b2ba39, 23 },
    { 0x8e22ba39, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf162ba3e, 84 },
    { 0xe812ba3f, 83 },
    { 0x3022ba3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f2ba43, 90 },
    { 0x2822ba43, 98 },
    { 0x7732ba44, 90 },
    { 0x9052ba46, 99 },
    { 0xf9c2ba44, 95 },
    { 0x5722ba47, 80 },
    { 0xb2f2ba49, 90 },
    { 0xeb32ba4a, 66 },
    { 0xb1d2ba4b, 48 },
    { 0xb652ba47, 80 },
    { 0xd852ba47, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf882ba52, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc742ba55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b32ba5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e2ba63, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe882ba66, 66 },
    { 0, 0 },
    { 0x4582ba68, 66 },
    { 0x1de2ba69, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc502ba6d, 91 },
    { 0x6742ba6e, 23 },
    { 0x632ba6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c2ba79, 90 },
    { 0xad2ba7a, 98 },
    { 0x4f82ba7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb22ba80, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62ba84, 33 },
    { 0x1e2ba85, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982ba88, 99 },
    { 0x51e2ba89, 90 },
    { 0x6ca2ba8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c2ba8d, 84 },
    { 0xac12ba8e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2ba91, 76 },
    { 0, 0 },
    { 0xafd2ba93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7982ba97, 96 },
    { 0x6d72ba98, 46 },
    { 0x8482ba99, 23 },
    { 0xc62ba9a, 90 },
    { 0xebb2ba97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d2ba9e, 93 },
    { 0xa232ba9f, 56 },
    { 0, 0 },
    { 0x4d82baa1, 90 },
    { 0x7ae2baa1, 78 },
    { 0, 0 },
    { 0xc932baa4, 34 },
    { 0xb572baa5, 61 },
    { 0x8b22baa5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752baab, 55 },
    { 0x3c02baab, 72 },
    { 0, 0 },
    { 0xc7a2baae, 90 },
    { 0, 0 },
    { 0xc382bab0, 90 },
    { 0xac32bab1, 87 },
    { 0, 0 },
    { 0x3162bab3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2262bab9, 98 },
    { 0, 0 },
    { 0x8d12babb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe272bac3, 90 },
    { 0x7b52bac4, 61 },
    { 0, 0 },
    { 0x3d2bac6, 33 },
    { 0x52e2bac7, 61 },
    { 0, 0 },
    { 0xe1b2bac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf2bacd, 90 },
    { 0xd6e2bace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca2bad2, 95 },
    { 0x7a42bad3, 52 },
    { 0, 0 },
    { 0xf662bad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b92badb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b2bae2, 34 },
    { 0xdd22bae3, 90 },
    { 0x3282bae3, 98 },
    { 0x6282bae4, 61 },
    { 0x5f02bae3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2192baf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd062baf6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd32bafa, 76 },
    { 0xb3e2bafb, 78 },
    { 0x6dd2bafc, 90 },
    { 0, 0 },
    { 0x2882bafe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552bb02, 95 },
    { 0x27d2bb02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d52bb07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b12bb0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2bb13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed92bb17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7532bb1c, 66 },
    { 0, 0 },
    { 0x7622bb1e, 23 },
    { 0, 0 },
    { 0x7e72bb20, 98 },
    { 0x36d2bb21, 33 },
    { 0x81c2bb20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a32bb25, 33 },
    { 0x7052bb25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d2bb29, 91 },
    { 0x6342bb2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed62bb31, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd442bb38, 34 },
    { 0x7db2bb38, 98 },
    { 0xc462bb38, 33 },
    { 0xc7a2bb3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb422bb43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf72bb46, 98 },
    { 0xaba2bb47, 85 },
    { 0x88a2bb48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9412bb4f, 98 },
    { 0x3ef2bb4f, 90 },
    { 0x7652bb51, 100 },
    { 0xb652bb52, 95 },
    { 0xbaa2bb50, 98 },
    { 0x1d62bb52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d92bb57, 90 },
    { 0xf9d2bb58, 87 },
    { 0, 0 },
    { 0xdf12bb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa412bb5f, 76 },
    { 0xf902bb60, 88 },
    { 0x15b2bb60, 98 },
    { 0x1ff2bb60, 23 },
    { 0, 0 },
    { 0x9952bb64, 61 },
    { 0x4c62bb64, 55 },
    { 0, 0 },
    { 0xb572bb67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x932bb6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2782bb72, 43 },
    { 0xd7b2bb72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe482bb7c, 98 },
    { 0xd982bb7c, 90 },
    { 0xfa02bb7c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae2bb83, 66 },
    { 0xa712bb83, 17 },
    { 0x24b2bb83, 90 },
    { 0x5f62bb86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22bb8a, 95 },
    { 0x6e12bb8b, 55 },
    { 0, 0 },
    { 0x43c2bb8d, 34 },
    { 0, 0 },
    { 0x6222bb8f, 55 },
    { 0, 0 },
    { 0x29a2bb91, 90 },
    { 0xd772bb92, 33 },
    { 0xea32bb93, 61 },
    { 0x2f82bb94, 87 },
    { 0x2292bb94, 100 },
    { 0x6d52bb96, 34 },
    { 0xff42bb97, 61 },
    { 0xbbe2bb97, 100 },
    { 0xb272bb99, 61 },
    { 0x29f2bb99, 85 },
    { 0xeb22bb9b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2bba1, 78 },
    { 0x3562bba2, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9982bba6, 43 },
    { 0, 0 },
    { 0x6912bba8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092bbab, 95 },
    { 0x9122bbab, 76 },
    { 0x5442bbab, 61 },
    { 0x89d2bbae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1722bbb2, 33 },
    { 0x51f2bbb3, 95 },
    { 0, 0 },
    { 0xe9e2bbb5, 98 },
    { 0x7022bbb5, 90 },
    { 0xf8c2bbb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde92bbbc, 33 },
    { 0xa1f2bbbd, 90 },
    { 0x2f82bbbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6702bbc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5282bbca, 33 },
    { 0x59a2bbca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe2bbcf, 77 },
    { 0x8812bbd0, 23 },
    { 0, 0 },
    { 0xd182bbd2, 61 },
    { 0xda72bbd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a52bbd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa2bbd9, 81 },
    { 0x6472bbd9, 98 },
    { 0x61f2bbdb, 33 },
    { 0x50e2bbdc, 85 },
    { 0x1732bbdd, 61 },
    { 0xc6b2bbdc, 55 },
    { 0, 0 },
    { 0x9c32bbe0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a2bbe5, 33 },
    { 0, 0 },
    { 0x7d72bbe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a2bbec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72bbef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3372bbf5, 61 },
    { 0xc672bbf6, 46 },
    { 0xecf2bbf5, 43 },
    { 0x5e42bbf5, 78 },
    { 0xf932bbf5, 93 },
    { 0x6342bbfa, 90 },
    { 0, 0 },
    { 0x1202bbfc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2bc00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1242bc0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62bc11, 33 },
    { 0xae52bc11, 91 },
    { 0x4c92bc12, 33 },
    { 0xa402bc14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc2bc18, 90 },
    { 0x34c2bc19, 76 },
    { 0xc562bc1a, 33 },
    { 0xe4e2bc1b, 34 },
    { 0, 0 },
    { 0xe542bc1d, 43 },
    { 0, 0 },
    { 0x5dc2bc1f, 61 },
    { 0xc012bc1f, 90 },
    { 0, 0 },
    { 0x2292bc22, 66 },
    { 0, 0 },
    { 0x5fc2bc24, 78 },
    { 0x8f72bc25, 98 },
    { 0xbba2bc25, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2bc2a, 61 },
    { 0, 0 },
    { 0x2282bc2c, 97 },
    { 0xfe12bc2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb732bc36, 97 },
    { 0xf2b2bc37, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc2bc3f, 98 },
    { 0x8782bc40, 78 },
    { 0xcb62bc41, 76 },
    { 0, 0 },
    { 0x5d12bc43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd2bc46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d72bc49, 33 },
    { 0xba62bc4a, 93 },
    { 0x6452bc4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d2bc52, 93 },
    { 0xee22bc53, 78 },
    { 0, 0 },
    { 0xaa92bc55, 33 },
    { 0xcfb2bc56, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a02bc5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142bc5f, 76 },
    { 0x2702bc60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e2bc66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc92bc6a, 95 },
    { 0, 0 },
    { 0x7042bc6c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f2bc70, 61 },
    { 0x5ec2bc70, 95 },
    { 0, 0 },
    { 0x5712bc73, 68 },
    { 0, 0 },
    { 0x5b22bc75, 55 },
    { 0x4312bc76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2bc7a, 85 },
    { 0, 0 },
    { 0x97a2bc7c, 95 },
    { 0, 0 },
    { 0xc702bc7e, 61 },
    { 0, 0 },
    { 0x2312bc80, 61 },
    { 0x88b2bc80, 55 },
    { 0x3012bc82, 88 },
    { 0x7c62bc82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a2bc86, 33 },
    { 0x96b2bc87, 93 },
    { 0xd832bc87, 23 },
    { 0x7362bc87, 61 },
    { 0x7a42bc88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6622bc8e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e2bc91, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5562bc95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca2bc98, 33 },
    { 0x4282bc98, 61 },
    { 0xce62bc98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d2bc9d, 100 },
    { 0x2862bc9d, 98 },
    { 0x42b2bc9f, 78 },
    { 0xb712bc9f, 90 },
    { 0x83f2bc9f, 92 },
    { 0xb2c2bc9e, 33 },
    { 0x82a2bca3, 84 },
    { 0xd222bca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1522bca9, 55 },
    { 0, 0 },
    { 0xdd32bcab, 93 },
    { 0xaec2bcac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb012bcaf, 93 },
    { 0xf772bcb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb2bcb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b12bcb9, 76 },
    { 0xc522bcba, 98 },
    { 0, 0 },
    { 0xb9d2bcbc, 91 },
    { 0xab12bcbd, 100 },
    { 0xbef2bcbe, 98 },
    { 0x8e82bcbf, 61 },
    { 0x9d32bcbf, 95 },
    { 0x9d72bcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe32bcc6, 55 },
    { 0xaa72bcc6, 33 },
    { 0x8a12bcc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db2bccd, 90 },
    { 0, 0 },
    { 0x4822bccf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d2bcd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bcda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c2bcdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e2bce4, 90 },
    { 0x8dc2bce5, 23 },
    { 0xb5e2bce5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df2bcf0, 85 },
    { 0x55d2bcf1, 98 },
    { 0x61e2bcf1, 33 },
    { 0xb5a2bcf1, 43 },
    { 0x56d2bcf1, 95 },
    { 0xc6d2bcf4, 40 },
    { 0xff52bcf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c12bcfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb42bd01, 92 },
    { 0x37b2bd02, 55 },
    { 0xa2bd02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab02bd06, 90 },
    { 0, 0 },
    { 0x352bd08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2bd0c, 90 },
    { 0, 0 },
    { 0x98a2bd0e, 33 },
    { 0, 0 },
    { 0xc442bd10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3602bd13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc082bd16, 90 },
    { 0x3fd2bd16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc302bd1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a32bd1e, 55 },
    { 0, 0 },
    { 0xf062bd20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2bd26, 90 },
    { 0, 0 },
    { 0x8f02bd28, 23 },
    { 0x2412bd29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b2bd2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f2bd34, 99 },
    { 0xd4f2bd35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda42bd39, 55 },
    { 0x78e2bd3a, 17 },
    { 0x89f2bd3a, 61 },
    { 0x1cc2bd39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a2bd3f, 61 },
    { 0, 0 },
    { 0x6a42bd41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb062bd44, 98 },
    { 0xb42bd45, 98 },
    { 0xed52bd45, 75 },
    { 0, 0 },
    { 0x5f22bd48, 98 },
    { 0x4c52bd48, 93 },
    { 0x3a42bd4a, 23 },
    { 0xceb2bd4a, 95 },
    { 0xa442bd4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8882bd53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7672bd57, 75 },
    { 0, 0 },
    { 0x98b2bd59, 90 },
    { 0xbfe2bd5a, 33 },
    { 0xbec2bd5b, 23 },
    { 0xc0e2bd5b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf292bd62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa72bd66, 90 },
    { 0xd732bd67, 95 },
    { 0x28b2bd67, 95 },
    { 0xd082bd67, 95 },
    { 0x7772bd6a, 91 },
    { 0, 0 },
    { 0x1f72bd6c, 90 },
    { 0xbe72bd6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6082bd74, 61 },
    { 0xb462bd75, 76 },
    { 0x34f2bd76, 55 },
    { 0x3192bd77, 55 },
    { 0xe772bd77, 55 },
    { 0, 0 },
    { 0xc512bd7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c2bd7f, 51 },
    { 0, 0 },
    { 0x30d2bd81, 84 },
    { 0xa952bd82, 33 },
    { 0x9372bd81, 90 },
    { 0x7a12bd84, 87 },
    { 0x9bb2bd84, 48 },
    { 0x17e2bd86, 98 },
    { 0x9e2bd85, 90 },
    { 0xec62bd81, 93 },
    { 0xb992bd84, 55 },
    { 0, 0 },
    { 0xb522bd8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a2bd8e, 23 },
    { 0, 0 },
    { 0x1f12bd90, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b2bd93, 61 },
    { 0, 0 },
    { 0x5e62bd95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4382bd9c, 99 },
    { 0xc2c2bd9c, 64 },
    { 0, 0 },
    { 0xa742bd9f, 95 },
    { 0x2c42bda0, 61 },
    { 0, 0 },
    { 0x8512bda2, 55 },
    { 0x5c72bda3, 38 },
    { 0x7d82bda2, 61 },
    { 0x77c2bda4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda92bda8, 53 },
    { 0, 0 },
    { 0x6262bdaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f12bdaf, 80 },
    { 0x6862bdb0, 23 },
    { 0xc2e2bdb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e2bdb4, 90 },
    { 0, 0 },
    { 0xea2bdb6, 33 },
    { 0xe112bdb7, 98 },
    { 0x7142bdb8, 98 },
    { 0x38a2bdb8, 43 },
    { 0x1d62bdba, 95 },
    { 0x11e2bdb9, 98 },
    { 0xb2bdbb, 98 },
    { 0xa4c2bdbc, 43 },
    { 0, 0 },
    { 0x74e2bdbf, 81 },
    { 0x47d2bdbf, 90 },
    { 0xc402bdc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2bdc6, 87 },
    { 0x2762bdc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7112bdca, 17 },
    { 0x3dc2bdcb, 99 },
    { 0x3bc2bdcc, 98 },
    { 0x3b42bdcd, 98 },
    { 0x7992bdcd, 61 },
    { 0xdf92bdcd, 61 },
    { 0x7c72bdcd, 81 },
    { 0x53d2bdcb, 23 },
    { 0x4112bdd2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6862bdd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd82bddd, 93 },
    { 0xbcd2bdde, 96 },
    { 0xa1c2bddf, 74 },
    { 0, 0 },
    { 0x26d2bde1, 90 },
    { 0x4692bde2, 90 },
    { 0x712bde2, 98 },
    { 0x5012bde3, 95 },
    { 0xcc02bde4, 49 },
    { 0x12b2bde6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca92bdea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d2bded, 55 },
    { 0xd732bdee, 98 },
    { 0xc782bdef, 61 },
    { 0x3f82bdf0, 78 },
    { 0xd92bdf1, 95 },
    { 0xda72bdf2, 76 },
    { 0x55e2bdf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b32bdf6, 55 },
    { 0x7952bdf6, 83 },
    { 0xa872bdf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7582bdfd, 33 },
    { 0xcca2bdfe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd2be04, 52 },
    { 0, 0 },
    { 0xb752be06, 88 },
    { 0, 0 },
    { 0xe9e2be08, 33 },
    { 0, 0 },
    { 0x3862be0a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa052be0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2812be15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682be1b, 46 },
    { 0x8c02be1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b2be25, 83 },
    { 0x292be25, 90 },
    { 0xe192be27, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3432be2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92be2e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa32be35, 33 },
    { 0xfe32be36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a2be3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982be3e, 33 },
    { 0, 0 },
    { 0xcf02be40, 90 },
    { 0x4872be41, 64 },
    { 0x18f2be40, 23 },
    { 0x5f2be43, 99 },
    { 0x4f82be44, 33 },
    { 0x1ba2be44, 66 },
    { 0x652be44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e2be4b, 91 },
    { 0x79f2be4b, 90 },
    { 0x4452be4d, 23 },
    { 0, 0 },
    { 0xdb72be4f, 33 },
    { 0xec32be4f, 90 },
    { 0xbff2be51, 98 },
    { 0x1df2be52, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef2be5c, 91 },
    { 0, 0 },
    { 0x1f42be5e, 76 },
    { 0xfaf2be5e, 61 },
    { 0xfb2be60, 66 },
    { 0xef92be61, 34 },
    { 0x1cf2be60, 61 },
    { 0, 0 },
    { 0x852be64, 61 },
    { 0, 0 },
    { 0x4332be66, 23 },
    { 0x1ed2be67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d2be6f, 46 },
    { 0, 0 },
    { 0x54e2be71, 67 },
    { 0, 0 },
    { 0xf8a2be73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832be76, 34 },
    { 0, 0 },
    { 0x5ea2be78, 55 },
    { 0xbeb2be78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb2be7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb72be7f, 78 },
    { 0, 0 },
    { 0xb3d2be81, 82 },
    { 0xbf12be82, 23 },
    { 0xd8f2be83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b2be87, 95 },
    { 0x4582be88, 33 },
    { 0x6ca2be87, 90 },
    { 0x2372be89, 61 },
    { 0xcb2be87, 98 },
    { 0x5842be8c, 90 },
    { 0x6a02be88, 93 },
    { 0x8b12be8c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f2be91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e2be98, 91 },
    { 0x6932be98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a2bea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af2bea9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f2beac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8172beaf, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d2beb5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7852beb9, 33 },
    { 0, 0 },
    { 0x8732bebb, 81 },
    { 0, 0 },
    { 0xf532bebd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2492bec0, 75 },
    { 0, 0 },
    { 0xe022bec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd2beca, 33 },
    { 0, 0 },
    { 0x91b2becc, 23 },
    { 0x11d2becd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a2bed1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee2bed4, 17 },
    { 0xb442bed5, 90 },
    { 0xfae2bed6, 76 },
    { 0xa622bed7, 77 },
    { 0xdcc2bed5, 95 },
    { 0x3af2bed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc2bee0, 61 },
    { 0x2282bee1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1902bee5, 61 },
    { 0xe5e2bee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e2bee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c12beef, 23 },
    { 0, 0 },
    { 0xce52bef1, 98 },
    { 0, 0 },
    { 0x7bb2bef3, 55 },
    { 0x79d2bef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f2bef7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd192befa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b2bf01, 61 },
    { 0xdf02bf01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2bf07, 90 },
    { 0x5952bf07, 95 },
    { 0x99d2bf09, 23 },
    { 0x54a2bf0a, 65 },
    { 0xecf2bf07, 98 },
    { 0x57e2bf0c, 84 },
    { 0x7d12bf0d, 66 },
    { 0xf5d2bf08, 33 },
    { 0, 0 },
    { 0xdfd2bf10, 87 },
    { 0xe2c2bf11, 34 },
    { 0, 0 },
    { 0xd442bf13, 90 },
    { 0xb872bf14, 90 },
    { 0x45e2bf15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2bf18, 53 },
    { 0x8482bf19, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d2bf1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc02bf2d, 95 },
    { 0xf462bf2d, 33 },
    { 0xdb32bf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d2bf34, 98 },
    { 0xfcb2bf34, 61 },
    { 0xb962bf36, 90 },
    { 0xa4c2bf35, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe872bf3b, 80 },
    { 0x68b2bf3c, 87 },
    { 0x3cd2bf3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2bf41, 90 },
    { 0xd012bf42, 33 },
    { 0, 0 },
    { 0xdd72bf44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92bf47, 100 },
    { 0xdff2bf48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3212bf4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2bf51, 90 },
    { 0x7922bf52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8912bf56, 67 },
    { 0, 0 },
    { 0xa0e2bf58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182bf5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe992bf63, 76 },
    { 0, 0 },
    { 0xaa92bf65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b2bf6c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3722bf6f, 76 },
    { 0x1612bf70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd662bf79, 95 },
    { 0xab72bf7a, 17 },
    { 0xbdb2bf7a, 95 },
    { 0x8ac2bf7c, 99 },
    { 0x50b2bf7a, 76 },
    { 0, 0 },
    { 0xe7e2bf7f, 95 },
    { 0, 0 },
    { 0xe9a2bf81, 33 },
    { 0x4482bf82, 99 },
    { 0xe1a2bf83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e2bf87, 17 },
    { 0, 0 },
    { 0x9832bf89, 34 },
    { 0x23b2bf89, 76 },
    { 0x5f72bf8b, 90 },
    { 0x1822bf8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2072bf90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d2bf95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb12bf98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad2bfa0, 98 },
    { 0x9d12bfa1, 95 },
    { 0x1bb2bfa1, 95 },
    { 0xf5c2bfa1, 33 },
    { 0xf462bfa4, 98 },
    { 0x75c2bfa4, 33 },
    { 0, 0 },
    { 0x9d42bfa7, 95 },
    { 0x9992bfa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb162bfab, 76 },
    { 0xc6e2bfab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb2bfb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce2bfba, 33 },
    { 0xa882bfbb, 66 },
    { 0x2d52bfba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32bfbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa492bfc3, 90 },
    { 0x5cd2bfc4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4312bfc9, 23 },
    { 0, 0 },
    { 0xb3d2bfcb, 84 },
    { 0, 0 },
    { 0xb792bfcd, 90 },
    { 0x75e2bfcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf2bfd2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5372bfd5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb902bfd8, 98 },
    { 0, 0 },
    { 0xb1d2bfda, 68 },
    { 0xcdd2bfdb, 87 },
    { 0xfa92bfda, 66 },
    { 0, 0 },
    { 0x2872bfde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5632bfe3, 95 },
    { 0xcf2bfe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d2bfee, 95 },
    { 0xb22bfef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd102bff5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b2bffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7702bffe, 99 },
    { 0xc8e2bfff, 66 },
    { 0, 0 },
    { 0x342c001, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a12c007, 81 },
    { 0, 0 },
    { 0xf0e2c009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a72c00d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c02c012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c2c016, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd2c01d, 34 },
    { 0, 0 },
    { 0xc662c01f, 55 },
    { 0xf472c020, 61 },
    { 0xfec2c01f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b2c026, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f22c029, 23 },
    { 0, 0 },
    { 0x1d12c02b, 17 },
    { 0xa0a2c02b, 98 },
    { 0xc8c2c02d, 33 },
    { 0, 0 },
    { 0x6d92c02f, 33 },
    { 0xd232c02f, 33 },
    { 0xae12c031, 95 },
    { 0xcca2c031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02c03c, 90 },
    { 0, 0 },
    { 0xe2f2c03e, 78 },
    { 0xe1d2c03f, 23 },
    { 0x6b82c040, 23 },
    { 0x7332c041, 23 },
    { 0x45e2c042, 90 },
    { 0x1c2c043, 91 },
    { 0xd8d2c041, 23 },
    { 0x3022c045, 90 },
    { 0xc4c2c046, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f2c04b, 61 },
    { 0, 0 },
    { 0x9cb2c04d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc52c052, 61 },
    { 0, 0 },
    { 0xcf12c054, 91 },
    { 0, 0 },
    { 0x4a2c056, 77 },
    { 0x93f2c057, 98 },
    { 0x80d2c058, 61 },
    { 0x6952c059, 98 },
    { 0xdc22c058, 81 },
    { 0x3382c05b, 83 },
    { 0xeeb2c05c, 95 },
    { 0xcd12c05d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c060, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae2c064, 23 },
    { 0xc22c065, 43 },
    { 0xd2a2c065, 34 },
    { 0, 0 },
    { 0x6c02c068, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a62c06c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9422c06f, 76 },
    { 0x2bb2c070, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8782c073, 90 },
    { 0x56a2c074, 91 },
    { 0, 0 },
    { 0xecd2c076, 77 },
    { 0xee62c077, 61 },
    { 0x7d42c078, 90 },
    { 0xfcd2c079, 23 },
    { 0xcb82c077, 98 },
    { 0x6012c07b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf672c080, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x832c083, 61 },
    { 0, 0 },
    { 0xf542c085, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe22c08e, 61 },
    { 0x2d22c08f, 67 },
    { 0xf822c090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92c099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf872c09c, 33 },
    { 0, 0 },
    { 0x4022c09e, 23 },
    { 0x1eb2c09f, 95 },
    { 0, 0 },
    { 0xac82c0a1, 61 },
    { 0, 0 },
    { 0xf3e2c0a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc402c0a9, 87 },
    { 0x7552c0aa, 61 },
    { 0x5d42c0ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc772c0b2, 78 },
    { 0x3c42c0b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee2c0ba, 90 },
    { 0, 0 },
    { 0x2002c0bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb52c0bf, 95 },
    { 0x2d62c0c0, 90 },
    { 0, 0 },
    { 0xcbe2c0c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b2c0c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe192c0c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a22c0cd, 81 },
    { 0xf722c0ce, 90 },
    { 0, 0 },
    { 0x1092c0d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb062c0d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692c0d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc2c0df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262c0e3, 96 },
    { 0x9762c0e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa352c0e8, 91 },
    { 0, 0 },
    { 0x4c02c0ea, 76 },
    { 0xf152c0eb, 76 },
    { 0x1b22c0eb, 43 },
    { 0xc822c0ea, 90 },
    { 0xf532c0ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd2c0f6, 61 },
    { 0x2342c0f7, 46 },
    { 0x25c2c0f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b2c0ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9972c106, 98 },
    { 0, 0 },
    { 0x1fc2c108, 98 },
    { 0, 0 },
    { 0x8222c10a, 83 },
    { 0, 0 },
    { 0xed32c10c, 33 },
    { 0xb9e2c10d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe82c110, 61 },
    { 0xa032c111, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912c118, 95 },
    { 0, 0 },
    { 0x9c82c11a, 95 },
    { 0xce42c11b, 93 },
    { 0xe832c11a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc792c121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832c125, 81 },
    { 0x38c2c126, 68 },
    { 0x10b2c126, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c92c12c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b52c132, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b2c135, 33 },
    { 0, 0 },
    { 0x7002c137, 90 },
    { 0, 0 },
    { 0xa392c139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512c140, 23 },
    { 0x6682c141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7672c144, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce2c147, 95 },
    { 0x7222c148, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa062c14b, 88 },
    { 0xd062c14c, 23 },
    { 0xb1e2c14d, 46 },
    { 0x5e12c14e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac2c152, 90 },
    { 0x9962c152, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b92c159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad2c15d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c2c161, 33 },
    { 0xdbb2c162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6932c168, 98 },
    { 0xda22c169, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3882c173, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f42c179, 66 },
    { 0xddf2c17a, 68 },
    { 0x1b42c17a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9482c182, 23 },
    { 0, 0 },
    { 0x882c184, 95 },
    { 0x5ec2c185, 46 },
    { 0x4252c186, 43 },
    { 0xef62c184, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2c18d, 98 },
    { 0x9c02c18d, 43 },
    { 0x2bf2c18f, 33 },
    { 0xdef2c190, 98 },
    { 0, 0 },
    { 0x212c192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2c197, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b2c1a2, 91 },
    { 0x3f2c1a2, 90 },
    { 0xf742c1a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9742c1b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e2c1b6, 85 },
    { 0, 0 },
    { 0x1922c1b8, 88 },
    { 0xab62c1b9, 23 },
    { 0x71d2c1ba, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf252c1bd, 33 },
    { 0x9712c1be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf812c1c3, 90 },
    { 0xb312c1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc2c1cd, 46 },
    { 0x5482c1ce, 55 },
    { 0x3542c1cf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc212c1d2, 52 },
    { 0, 0 },
    { 0x40f2c1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e2c1df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3802c1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd352c1ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4402c1ef, 87 },
    { 0x1532c1f0, 34 },
    { 0x4882c1ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e2c1f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa62c1fa, 33 },
    { 0xcfd2c1fa, 98 },
    { 0x3ee2c1fc, 93 },
    { 0x8f42c1fd, 98 },
    { 0xf22c1fb, 95 },
    { 0x25d2c1fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d2c203, 34 },
    { 0xd3e2c203, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52c208, 23 },
    { 0, 0 },
    { 0x1e62c20a, 33 },
    { 0x5fd2c20a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae2c212, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f2c216, 66 },
    { 0, 0 },
    { 0x40d2c218, 23 },
    { 0x722c219, 95 },
    { 0, 0 },
    { 0x2882c21b, 91 },
    { 0x7a12c21c, 90 },
    { 0x1962c21c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6452c221, 76 },
    { 0xbef2c222, 90 },
    { 0xeae2c223, 55 },
    { 0x40f2c224, 91 },
    { 0x20d2c225, 91 },
    { 0x872c226, 90 },
    { 0xb242c227, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd452c22a, 90 },
    { 0, 0 },
    { 0x8242c22c, 90 },
    { 0x4c32c22c, 95 },
    { 0xa252c22c, 93 },
    { 0xa0e2c22f, 23 },
    { 0x3022c230, 78 },
    { 0xab12c22f, 69 },
    { 0xc3e2c22c, 33 },
    { 0x3552c233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2c237, 61 },
    { 0x9232c238, 53 },
    { 0, 0 },
    { 0xd252c23a, 61 },
    { 0x5b72c23a, 43 },
    { 0x5c62c23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a2c243, 61 },
    { 0x2a52c244, 90 },
    { 0x5b22c245, 90 },
    { 0x4fa2c246, 53 },
    { 0x5e02c246, 55 },
    { 0xadd2c248, 90 },
    { 0xe642c244, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2c24d, 43 },
    { 0x19b2c24d, 61 },
    { 0, 0 },
    { 0x8f92c250, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xea72c253, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a22c258, 23 },
    { 0x9ea2c259, 80 },
    { 0xa2e2c25a, 95 },
    { 0xd352c259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2c25e, 93 },
    { 0x4f62c25e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf2c263, 90 },
    { 0xcfd2c264, 23 },
    { 0x6272c263, 55 },
    { 0x8502c266, 23 },
    { 0, 0 },
    { 0x4862c268, 65 },
    { 0, 0 },
    { 0xf322c26a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4042c26e, 23 },
    { 0xb022c26e, 95 },
    { 0x3b72c26e, 98 },
    { 0xbb92c271, 52 },
    { 0x6fd2c272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f2c277, 98 },
    { 0x28d2c277, 90 },
    { 0xb482c279, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b2c27d, 98 },
    { 0x85c2c27e, 61 },
    { 0, 0 },
    { 0x8b02c280, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d2c284, 78 },
    { 0xaf62c285, 85 },
    { 0xbe22c286, 23 },
    { 0xb8f2c287, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf252c28c, 99 },
    { 0xfa92c28d, 61 },
    { 0xd5e2c28d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f2c291, 33 },
    { 0, 0 },
    { 0xdb82c293, 76 },
    { 0x72c294, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2c299, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa852c29c, 55 },
    { 0x61b2c29c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x522c2a0, 23 },
    { 0xf632c2a0, 85 },
    { 0x7752c2a0, 90 },
    { 0x85e2c2a1, 34 },
    { 0x61e2c2a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af2c2a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c2c2aa, 97 },
    { 0, 0 },
    { 0xbaa2c2ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4532c2af, 61 },
    { 0, 0 },
    { 0x9a2c2b1, 98 },
    { 0xa812c2b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a2c2b6, 23 },
    { 0, 0 },
    { 0xb202c2b8, 90 },
    { 0, 0 },
    { 0x8d62c2ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1182c2be, 27 },
    { 0x8e62c2bf, 61 },
    { 0x4cd2c2c0, 99 },
    { 0x95b2c2bf, 61 },
    { 0, 0 },
    { 0xa992c2c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f22c2c7, 98 },
    { 0, 0 },
    { 0x7192c2c9, 78 },
    { 0, 0 },
    { 0xa8e2c2cb, 81 },
    { 0, 0 },
    { 0xa342c2cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad2c2d0, 23 },
    { 0x6de2c2d1, 87 },
    { 0xf1f2c2d1, 34 },
    { 0x4ed2c2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c2c2d8, 93 },
    { 0xdc02c2d9, 61 },
    { 0x3832c2da, 55 },
    { 0xdfc2c2db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee2c2e3, 69 },
    { 0xa022c2e4, 61 },
    { 0xf5f2c2e5, 61 },
    { 0, 0 },
    { 0x3a22c2e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc532c2eb, 61 },
    { 0x5f52c2ec, 68 },
    { 0, 0 },
    { 0x4552c2ee, 76 },
    { 0x5f52c2ef, 33 },
    { 0x3292c2ef, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa562c2f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f52c2fb, 84 },
    { 0x1a12c2fc, 95 },
    { 0, 0 },
    { 0xd9d2c2fe, 23 },
    { 0x9ad2c2ff, 67 },
    { 0x5272c2ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592c303, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa882c30a, 17 },
    { 0, 0 },
    { 0x59d2c30c, 95 },
    { 0x4162c30c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec52c311, 61 },
    { 0x9822c312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec12c316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb782c31a, 23 },
    { 0, 0 },
    { 0x3622c31c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd552c321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5292c326, 95 },
    { 0x3772c327, 66 },
    { 0x2ec2c328, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x652c32b, 76 },
    { 0x54c2c32b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf012c32f, 90 },
    { 0, 0 },
    { 0x4992c331, 90 },
    { 0, 0 },
    { 0xd122c333, 61 },
    { 0, 0 },
    { 0x57d2c335, 90 },
    { 0xf002c335, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a2c339, 55 },
    { 0x2f72c33a, 33 },
    { 0, 0 },
    { 0xf9f2c33c, 55 },
    { 0xfc32c33d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee2c349, 34 },
    { 0x2f02c349, 90 },
    { 0x8b52c34b, 85 },
    { 0xd272c34c, 76 },
    { 0, 0 },
    { 0x90e2c34e, 90 },
    { 0, 0 },
    { 0x2ae2c350, 61 },
    { 0, 0 },
    { 0xddf2c352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c2c355, 90 },
    { 0, 0 },
    { 0xa7b2c357, 98 },
    { 0, 0 },
    { 0xeef2c359, 90 },
    { 0xb8c2c35a, 34 },
    { 0, 0 },
    { 0x72e2c35c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb42c365, 33 },
    { 0x5f12c366, 61 },
    { 0, 0 },
    { 0x7502c368, 61 },
    { 0, 0 },
    { 0x22c36a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc2c375, 43 },
    { 0x97e2c376, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc242c379, 90 },
    { 0x5632c37a, 35 },
    { 0, 0 },
    { 0xea62c37c, 87 },
    { 0, 0 },
    { 0xe2e2c37e, 84 },
    { 0xbbf2c37f, 95 },
    { 0, 0 },
    { 0xd1e2c381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f2c389, 90 },
    { 0xee52c38a, 93 },
    { 0xcfc2c38b, 98 },
    { 0, 0 },
    { 0x1402c38d, 46 },
    { 0x3a72c38d, 98 },
    { 0xe7c2c38d, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d2c392, 78 },
    { 0, 0 },
    { 0x8f2c394, 91 },
    { 0, 0 },
    { 0x6182c396, 61 },
    { 0xd462c396, 34 },
    { 0xacd2c398, 100 },
    { 0xcc22c397, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5582c39e, 66 },
    { 0, 0 },
    { 0xec2c3a0, 98 },
    { 0xc122c3a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b42c3a7, 61 },
    { 0x69d2c3a7, 33 },
    { 0x3052c3a8, 90 },
    { 0x77a2c3a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa2c3ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd172c3b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae62c3b7, 90 },
    { 0xc9f2c3b8, 23 },
    { 0, 0 },
    { 0x9aa2c3ba, 34 },
    { 0x3762c3bb, 55 },
    { 0, 0 },
    { 0x65b2c3bd, 90 },
    { 0x6892c3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd2c3c1, 33 },
    { 0x6a42c3c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc792c3c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac2c3cb, 61 },
    { 0x3222c3cb, 84 },
    { 0xf692c3cc, 23 },
    { 0xa502c3cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed62c3d4, 95 },
    { 0xc082c3d5, 55 },
    { 0xc3b2c3d6, 77 },
    { 0, 0 },
    { 0x3f92c3d8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4732c3df, 95 },
    { 0xa4d2c3e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1642c3e4, 85 },
    { 0, 0 },
    { 0x3782c3e6, 33 },
    { 0, 0 },
    { 0xcd22c3e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac22c3ec, 98 },
    { 0, 0 },
    { 0x5dc2c3ee, 23 },
    { 0x9d2c3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc52c3f7, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b2c3fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92c404, 55 },
    { 0xcf52c404, 23 },
    { 0x9052c404, 33 },
    { 0xfcb2c407, 61 },
    { 0, 0 },
    { 0xbd32c409, 52 },
    { 0x5732c409, 34 },
    { 0x4732c40b, 90 },
    { 0x242c40c, 23 },
    { 0x5b02c40c, 55 },
    { 0, 0 },
    { 0x2b72c40f, 61 },
    { 0, 0 },
    { 0x4842c411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e22c414, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2c41b, 93 },
    { 0, 0 },
    { 0xcce2c41d, 84 },
    { 0x7022c41d, 55 },
    { 0xcc02c41d, 34 },
    { 0x67f2c420, 95 },
    { 0xa082c41e, 23 },
    { 0xd1a2c420, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8532c425, 78 },
    { 0xa242c426, 90 },
    { 0, 0 },
    { 0x4762c428, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab2c42d, 55 },
    { 0xd582c42e, 95 },
    { 0x14c2c42f, 17 },
    { 0, 0 },
    { 0xf312c431, 75 },
    { 0x8ad2c432, 90 },
    { 0, 0 },
    { 0x61d2c434, 98 },
    { 0x8e22c434, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb2c438, 90 },
    { 0xa612c439, 33 },
    { 0, 0 },
    { 0xc5f2c43b, 90 },
    { 0, 0 },
    { 0xda42c43d, 61 },
    { 0x5ac2c43d, 33 },
    { 0xef62c43f, 90 },
    { 0, 0 },
    { 0x33d2c441, 40 },
    { 0x3c22c442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d2c44c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb432c44f, 90 },
    { 0x1b72c450, 87 },
    { 0xc632c451, 33 },
    { 0x7c82c452, 33 },
    { 0x3112c453, 64 },
    { 0xe152c450, 76 },
    { 0x3e72c455, 75 },
    { 0, 0 },
    { 0x3382c457, 33 },
    { 0, 0 },
    { 0x19f2c459, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d2c45f, 90 },
    { 0xdac2c460, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d02c464, 76 },
    { 0x3122c465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb332c468, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5732c46d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc92c471, 61 },
    { 0xc9e2c472, 61 },
    { 0xacc2c473, 100 },
    { 0xe462c474, 77 },
    { 0, 0 },
    { 0xd682c476, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca22c479, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc2c47d, 98 },
    { 0xd2d2c47e, 90 },
    { 0, 0 },
    { 0xf7e2c480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b92c483, 91 },
    { 0x6e42c484, 90 },
    { 0x26f2c484, 43 },
    { 0, 0 },
    { 0x99b2c487, 93 },
    { 0x8f92c487, 98 },
    { 0, 0 },
    { 0x6a72c48a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6942c48f, 97 },
    { 0x4b62c48f, 98 },
    { 0, 0 },
    { 0xee12c492, 33 },
    { 0, 0 },
    { 0x3792c494, 78 },
    { 0x4292c494, 94 },
    { 0xc3f2c495, 19 },
    { 0, 0 },
    { 0x1922c498, 75 },
    { 0x8952c499, 90 },
    { 0x9a52c49a, 88 },
    { 0x8c32c498, 76 },
    { 0xc482c498, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b52c49f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c4a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d2c4aa, 95 },
    { 0xfc32c4ab, 95 },
    { 0x7d82c4ac, 93 },
    { 0xe12c4ad, 33 },
    { 0, 0 },
    { 0x91b2c4af, 91 },
    { 0x7752c4af, 23 },
    { 0x4a92c4af, 33 },
    { 0x7b42c4b2, 90 },
    { 0, 0 },
    { 0xe8b2c4b4, 90 },
    { 0x6092c4b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7822c4bf, 33 },
    { 0x77c2c4bf, 90 },
    { 0x3072c4c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2c4c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d12c4ca, 43 },
    { 0, 0 },
    { 0x7f52c4cc, 61 },
    { 0, 0 },
    { 0x99a2c4ce, 34 },
    { 0x7002c4cf, 23 },
    { 0, 0 },
    { 0x7e42c4d1, 61 },
    { 0, 0 },
    { 0x1cf2c4d3, 93 },
    { 0xf532c4d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c2c4d8, 17 },
    { 0xe5c2c4d8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad2c4dc, 95 },
    { 0xa392c4dc, 98 },
    { 0xea92c4dc, 84 },
    { 0x68a2c4df, 55 },
    { 0x34f2c4df, 76 },
    { 0xa02c4df, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12c4e6, 46 },
    { 0x75a2c4e7, 23 },
    { 0xef2c4e8, 98 },
    { 0xee82c4e9, 34 },
    { 0xb412c4e8, 33 },
    { 0, 0 },
    { 0x4642c4ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa822c4f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2c4f5, 90 },
    { 0xb22c4f5, 33 },
    { 0x16a2c4f7, 90 },
    { 0x35e2c4f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8382c4fc, 90 },
    { 0, 0 },
    { 0xc412c4fe, 61 },
    { 0xa8a2c4ff, 61 },
    { 0, 0 },
    { 0x55b2c501, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3172c506, 61 },
    { 0x2962c507, 68 },
    { 0x67b2c506, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72c50b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be2c50f, 33 },
    { 0, 0 },
    { 0x892c511, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5322c515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d2c51d, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b92c520, 61 },
    { 0, 0 },
    { 0x3242c522, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e2c525, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582c529, 23 },
    { 0xea12c529, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302c52d, 61 },
    { 0x4332c52e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2c532, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe2c536, 99 },
    { 0, 0 },
    { 0xc112c538, 99 },
    { 0xb8b2c538, 76 },
    { 0x66a2c53a, 43 },
    { 0xea62c53b, 90 },
    { 0xdce2c539, 33 },
    { 0x4542c539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc2c542, 95 },
    { 0xeac2c542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a52c546, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b2c54b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e2c54e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a2c553, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a22c556, 98 },
    { 0, 0 },
    { 0x3202c558, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad2c55c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d2c562, 61 },
    { 0x6f82c563, 84 },
    { 0x8a22c564, 93 },
    { 0x6292c564, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc052c56c, 46 },
    { 0, 0 },
    { 0x79b2c56e, 23 },
    { 0x2da2c56f, 95 },
    { 0x1502c56f, 84 },
    { 0, 0 },
    { 0x9b72c572, 90 },
    { 0xec12c572, 90 },
    { 0x3dd2c572, 88 },
    { 0xd222c575, 93 },
    { 0x7ef2c575, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722c580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7382c583, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d2c586, 90 },
    { 0xcdf2c587, 98 },
    { 0xf192c588, 45 },
    { 0x9592c588, 90 },
    { 0x6102c587, 76 },
    { 0x3f22c589, 43 },
    { 0, 0 },
    { 0xf452c58d, 65 },
    { 0, 0 },
    { 0xd3d2c58f, 55 },
    { 0x3352c58f, 33 },
    { 0xf7f2c591, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a2c599, 61 },
    { 0xb982c59a, 93 },
    { 0, 0 },
    { 0xfbf2c59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd2c5a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac2c5a6, 98 },
    { 0x1d02c5a7, 76 },
    { 0x7c82c5a7, 55 },
    { 0x9372c5a7, 68 },
    { 0, 0 },
    { 0x7352c5ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b2c5b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3502c5b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe492c5b8, 91 },
    { 0xec72c5b9, 61 },
    { 0x9272c5b9, 66 },
    { 0xa752c5ba, 68 },
    { 0x7972c5bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa222c5c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d2c5c5, 23 },
    { 0, 0 },
    { 0x5c72c5c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe582c5cc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8932c5d0, 61 },
    { 0, 0 },
    { 0x4192c5d2, 61 },
    { 0x8bd2c5d2, 45 },
    { 0x6182c5d4, 23 },
    { 0x8ab2c5d5, 43 },
    { 0x34d2c5d2, 90 },
    { 0xd332c5d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d52c5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1672c5e2, 95 },
    { 0, 0 },
    { 0xcf32c5e4, 90 },
    { 0x19e2c5e4, 33 },
    { 0x27b2c5e6, 84 },
    { 0x8f42c5e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce22c5ec, 66 },
    { 0xe3a2c5ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8162c5f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb522c5f5, 84 },
    { 0x8d52c5f6, 93 },
    { 0x28a2c5f6, 98 },
    { 0, 0 },
    { 0x9432c5f9, 36 },
    { 0x62c2c5fa, 23 },
    { 0x152c5f9, 33 },
    { 0xe382c5fc, 88 },
    { 0x2932c5fd, 90 },
    { 0xe3e2c5fe, 90 },
    { 0xe7b2c5fc, 91 },
    { 0x9952c5ff, 90 },
    { 0x2fa2c601, 61 },
    { 0xf8a2c5fc, 55 },
    { 0x4b82c603, 33 },
    { 0xa6a2c603, 80 },
    { 0x2d2c605, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5732c608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f2c60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea22c60e, 23 },
    { 0, 0 },
    { 0xa662c610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4972c615, 95 },
    { 0, 0 },
    { 0xc312c617, 95 },
    { 0xb5a2c618, 61 },
    { 0x6b52c619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5262c61f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e2c623, 33 },
    { 0, 0 },
    { 0xaf12c625, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2742c62c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a2c635, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2752c639, 43 },
    { 0xf542c63a, 61 },
    { 0x182c63a, 76 },
    { 0xd782c63c, 55 },
    { 0x3f32c63a, 76 },
    { 0xcad2c63e, 34 },
    { 0x3872c63e, 96 },
    { 0x3f92c639, 33 },
    { 0x4e72c639, 33 },
    { 0xda82c642, 88 },
    { 0x3772c63d, 61 },
    { 0xb042c644, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c2c647, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b2c64b, 87 },
    { 0x4182c64b, 76 },
    { 0x1d22c64b, 23 },
    { 0, 0 },
    { 0xc702c64f, 61 },
    { 0xb342c64f, 33 },
    { 0x68e2c651, 61 },
    { 0x5292c652, 34 },
    { 0xac2c651, 34 },
    { 0x1a72c653, 75 },
    { 0x7492c653, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a72c65d, 76 },
    { 0, 0 },
    { 0xfab2c65f, 90 },
    { 0xac62c660, 76 },
    { 0x9ca2c65f, 93 },
    { 0, 0 },
    { 0x7e2c663, 43 },
    { 0x1d02c663, 98 },
    { 0x6a42c665, 100 },
    { 0xe1b2c665, 93 },
    { 0xd0b2c667, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f2c66b, 66 },
    { 0, 0 },
    { 0x3a22c66d, 96 },
    { 0x1ea2c66e, 65 },
    { 0x72b2c66d, 98 },
    { 0x7c2c66e, 23 },
    { 0x3e72c671, 68 },
    { 0x47d2c672, 23 },
    { 0xf052c671, 23 },
    { 0, 0 },
    { 0xf462c675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa022c681, 33 },
    { 0, 0 },
    { 0xcb62c683, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab12c68b, 76 },
    { 0x1a12c68c, 90 },
    { 0x6732c68d, 72 },
    { 0x4502c68e, 90 },
    { 0x52e2c68d, 34 },
    { 0x10f2c68d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d2c69a, 100 },
    { 0x7aa2c69b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c12c69f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe732c6a2, 98 },
    { 0xcf92c6a3, 55 },
    { 0x72d2c6a3, 90 },
    { 0x1e02c6a5, 43 },
    { 0xf9d2c6a6, 23 },
    { 0x4f52c6a7, 17 },
    { 0x9f52c6a8, 66 },
    { 0xb112c6a9, 33 },
    { 0, 0 },
    { 0xac82c6ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2432c6af, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf2c6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f22c6ba, 99 },
    { 0x4d62c6ba, 78 },
    { 0xda52c6bc, 83 },
    { 0xb192c6bd, 95 },
    { 0x9752c6bc, 33 },
    { 0x6a72c6bf, 98 },
    { 0, 0 },
    { 0x2152c6c1, 90 },
    { 0xb0c2c6c1, 61 },
    { 0x51a2c6c3, 90 },
    { 0x4912c6c3, 33 },
    { 0x8c32c6c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee2c6cb, 23 },
    { 0, 0 },
    { 0xdee2c6cd, 55 },
    { 0xa42c6ce, 95 },
    { 0xea62c6cf, 84 },
    { 0, 0 },
    { 0xfc62c6d1, 23 },
    { 0, 0 },
    { 0x3912c6d3, 95 },
    { 0xd5d2c6d4, 52 },
    { 0, 0 },
    { 0x5962c6d6, 95 },
    { 0xeda2c6d7, 56 },
    { 0xe8d2c6d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe902c6de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a42c6e3, 93 },
    { 0x7422c6e4, 98 },
    { 0x23f2c6e5, 90 },
    { 0x9372c6e5, 23 },
    { 0x7fe2c6e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e2c6ed, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2c6f0, 46 },
    { 0xbbf2c6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2c6f4, 34 },
    { 0xb2d2c6f5, 61 },
    { 0xa82c6f4, 61 },
    { 0, 0 },
    { 0x8952c6f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a02c6fb, 61 },
    { 0xfa02c6fb, 90 },
    { 0, 0 },
    { 0xef62c6fe, 88 },
    { 0, 0 },
    { 0x9682c700, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd92c707, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc402c70d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5122c711, 61 },
    { 0, 0 },
    { 0xbe2c713, 66 },
    { 0, 0 },
    { 0x5252c715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2c71a, 61 },
    { 0xb302c71b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd772c71e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d2c723, 90 },
    { 0, 0 },
    { 0x69e2c725, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262c729, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2862c72c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722c731, 66 },
    { 0, 0 },
    { 0x2b2c733, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c2c736, 34 },
    { 0, 0 },
    { 0x8612c738, 98 },
    { 0x6a62c738, 23 },
    { 0x3cc2c73a, 23 },
    { 0xf8e2c73b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe202c73e, 77 },
    { 0xeca2c73f, 99 },
    { 0x4cb2c73e, 99 },
    { 0x6822c741, 23 },
    { 0, 0 },
    { 0x8e02c743, 33 },
    { 0x4dd2c744, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d52c747, 95 },
    { 0x17b2c748, 55 },
    { 0, 0 },
    { 0x5e2c74a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc2c74d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8432c750, 100 },
    { 0, 0 },
    { 0x58c2c752, 61 },
    { 0x6af2c753, 91 },
    { 0xa4d2c754, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d2c759, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6332c75d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca82c764, 98 },
    { 0x3a02c765, 90 },
    { 0x3892c766, 55 },
    { 0xfd02c767, 75 },
    { 0x6ad2c764, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed22c76b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5982c76e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e2c771, 98 },
    { 0x8232c772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc462c777, 23 },
    { 0, 0 },
    { 0x582c779, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f2c77c, 90 },
    { 0, 0 },
    { 0x3192c77e, 23 },
    { 0xa8e2c77e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac2c782, 91 },
    { 0xdd22c783, 90 },
    { 0x6882c782, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b2c787, 99 },
    { 0x6112c788, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc92c790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab2c797, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a2c79c, 98 },
    { 0xb8d2c79d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd72c7a0, 90 },
    { 0x1082c7a0, 76 },
    { 0xa8c2c7a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb12c7a6, 55 },
    { 0x8cd2c7a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a2c7aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba2c7ae, 100 },
    { 0, 0 },
    { 0x4602c7b0, 95 },
    { 0x1d82c7b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2c7b5, 33 },
    { 0x69c2c7b5, 90 },
    { 0xc2c2c7b6, 33 },
    { 0x6732c7b8, 33 },
    { 0x83b2c7b9, 77 },
    { 0x6472c7b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd52c7c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb052c7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f2c7cb, 23 },
    { 0, 0 },
    { 0xbfa2c7cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc832c7d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e22c7dd, 55 },
    { 0xb3a2c7de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b2c7e3, 95 },
    { 0, 0 },
    { 0x75f2c7e5, 90 },
    { 0, 0 },
    { 0x71b2c7e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc2c7ed, 56 },
    { 0, 0 },
    { 0x7d2c7ef, 33 },
    { 0x17f2c7f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b2c7f6, 95 },
    { 0x8492c7f6, 81 },
    { 0xb4b2c7f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c2c7fe, 90 },
    { 0xaaf2c7ff, 97 },
    { 0xf512c7ff, 95 },
    { 0, 0 },
    { 0x79e2c802, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe552c806, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5192c80c, 98 },
    { 0x1d72c80d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3692c812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb302c818, 93 },
    { 0x94b2c818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd452c81f, 76 },
    { 0xbe92c820, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b2c823, 64 },
    { 0x6f92c824, 98 },
    { 0x34a2c823, 98 },
    { 0, 0 },
    { 0xfda2c827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2c82a, 100 },
    { 0x5b12c82b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d2c831, 87 },
    { 0, 0 },
    { 0xbca2c833, 98 },
    { 0x74f2c834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4692c838, 23 },
    { 0xe6c2c838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082c83c, 23 },
    { 0x5cb2c83d, 90 },
    { 0, 0 },
    { 0xee72c83f, 33 },
    { 0, 0 },
    { 0x3022c841, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f52c845, 76 },
    { 0, 0 },
    { 0x48f2c847, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2c84b, 46 },
    { 0, 0 },
    { 0x4602c84d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2c852, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c2c855, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df2c858, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd242c85c, 98 },
    { 0x80e2c85d, 75 },
    { 0, 0 },
    { 0xbfe2c85f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1922c862, 95 },
    { 0, 0 },
    { 0x8542c864, 87 },
    { 0xb592c864, 91 },
    { 0x9702c865, 61 },
    { 0, 0 },
    { 0xced2c868, 61 },
    { 0xd7d2c869, 87 },
    { 0xca2c868, 48 },
    { 0, 0 },
    { 0xe5d2c86c, 38 },
    { 0, 0 },
    { 0x16b2c86e, 38 },
    { 0, 0 },
    { 0x53e2c870, 56 },
    { 0x31c2c871, 90 },
    { 0x1672c872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9262c876, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b2c87d, 23 },
    { 0x6462c87e, 69 },
    { 0xa82c87d, 40 },
    { 0xe982c880, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c2c885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1192c88c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce2c893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2152c896, 61 },
    { 0x3882c897, 77 },
    { 0, 0 },
    { 0x4d92c899, 61 },
    { 0x7d22c89a, 90 },
    { 0xd1a2c899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea32c89f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3042c8a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692c8aa, 90 },
    { 0, 0 },
    { 0x3392c8ac, 34 },
    { 0, 0 },
    { 0x1852c8ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c2c8b2, 87 },
    { 0xca22c8b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3432c8b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2362c8bc, 99 },
    { 0, 0 },
    { 0x69d2c8be, 61 },
    { 0x7d32c8bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc2c8c2, 76 },
    { 0x4252c8c2, 93 },
    { 0xe52c8c2, 98 },
    { 0x8562c8c3, 90 },
    { 0x5362c8c5, 67 },
    { 0xa742c8c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xca62c8ca, 100 },
    { 0x5452c8cb, 61 },
    { 0, 0 },
    { 0x4922c8cd, 87 },
    { 0x6702c8ce, 90 },
    { 0xa622c8cd, 90 },
    { 0x2732c8d0, 55 },
    { 0x4a2c8d1, 100 },
    { 0, 0 },
    { 0x6532c8d3, 81 },
    { 0xaa12c8d3, 43 },
    { 0, 0 },
    { 0xbfc2c8d6, 56 },
    { 0x2a12c8d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd12c8dc, 33 },
    { 0, 0 },
    { 0x4d12c8de, 99 },
    { 0xffe2c8de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e2c8e4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e2c8ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe662c903, 46 },
    { 0xbc82c903, 33 },
    { 0xd492c903, 34 },
    { 0, 0 },
    { 0x2f42c907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d82c90b, 98 },
    { 0x4d12c90c, 98 },
    { 0xf672c90d, 55 },
    { 0, 0 },
    { 0x8e72c90f, 55 },
    { 0x4632c90f, 33 },
    { 0x3ce2c911, 76 },
    { 0x7b92c912, 77 },
    { 0xf712c913, 23 },
    { 0x7872c914, 52 },
    { 0x7d2c915, 98 },
    { 0xdec2c916, 33 },
    { 0x55e2c917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa542c91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe052c923, 98 },
    { 0x3702c923, 34 },
    { 0xbc32c925, 55 },
    { 0xc892c924, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa312c929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2c92d, 33 },
    { 0xbea2c92e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2c932, 90 },
    { 0xb7d2c933, 84 },
    { 0, 0 },
    { 0xa932c935, 33 },
    { 0, 0 },
    { 0x4552c937, 33 },
    { 0x7b92c938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4492c93b, 76 },
    { 0x67d2c93c, 52 },
    { 0x4132c93d, 90 },
    { 0x25a2c93d, 91 },
    { 0xcd22c93f, 33 },
    { 0xd8a2c93f, 33 },
    { 0xca42c93f, 52 },
    { 0xcc52c940, 75 },
    { 0, 0 },
    { 0x17e2c944, 98 },
    { 0xf52c945, 77 },
    { 0xee32c946, 33 },
    { 0x5a82c945, 23 },
    { 0x1622c948, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952c94e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f2c951, 61 },
    { 0, 0 },
    { 0x8fe2c953, 55 },
    { 0, 0 },
    { 0x19d2c955, 90 },
    { 0x25e2c956, 100 },
    { 0x4072c956, 90 },
    { 0, 0 },
    { 0xb3d2c959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c2c95c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4892c961, 61 },
    { 0, 0 },
    { 0xfa2c963, 61 },
    { 0x8e2c964, 61 },
    { 0, 0 },
    { 0xe0b2c966, 67 },
    { 0x87e2c967, 55 },
    { 0, 0 },
    { 0x332c969, 55 },
    { 0xe762c969, 61 },
    { 0xad32c96b, 34 },
    { 0x77e2c96a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c52c972, 90 },
    { 0x6d82c973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5592c976, 95 },
    { 0xf52c976, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a2c983, 93 },
    { 0x4902c984, 77 },
    { 0x2b22c985, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb2c98a, 78 },
    { 0x72f2c98b, 33 },
    { 0x9162c98c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d2c98f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12c993, 95 },
    { 0, 0 },
    { 0xc2f2c995, 90 },
    { 0x5f02c995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa222c999, 23 },
    { 0x7fe2c99a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa2c99e, 90 },
    { 0, 0 },
    { 0x4dc2c9a0, 33 },
    { 0x1742c9a1, 95 },
    { 0, 0 },
    { 0xda42c9a3, 34 },
    { 0, 0 },
    { 0x2192c9a5, 90 },
    { 0xb8c2c9a6, 91 },
    { 0, 0 },
    { 0x1de2c9a8, 23 },
    { 0xe6e2c9a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe872c9af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2c9b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce62c9b5, 34 },
    { 0xab72c9b6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e2c9b9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4132c9c1, 23 },
    { 0x3d72c9c2, 33 },
    { 0x6822c9c3, 76 },
    { 0xbfc2c9c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3362c9c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d02c9ca, 33 },
    { 0xab2c9ca, 95 },
    { 0x1012c9cc, 33 },
    { 0xd522c9cd, 23 },
    { 0x8f32c9ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c2c9d5, 33 },
    { 0, 0 },
    { 0xf192c9d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb2c9dd, 23 },
    { 0x6892c9dd, 61 },
    { 0xeac2c9df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d2c9e2, 61 },
    { 0x3f92c9e3, 83 },
    { 0x7582c9e2, 33 },
    { 0xca22c9e5, 98 },
    { 0xef42c9e6, 46 },
    { 0x52c2c9e2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe72c9ec, 98 },
    { 0xa7d2c9ec, 17 },
    { 0x6932c9ed, 34 },
    { 0x8522c9ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c2c9f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd662c9fb, 97 },
    { 0, 0 },
    { 0x8142c9fd, 40 },
    { 0xc802c9fe, 61 },
    { 0x7072c9fd, 98 },
    { 0x3e42c9ff, 98 },
    { 0xa6a2ca01, 91 },
    { 0xe342ca01, 98 },
    { 0x6232ca03, 91 },
    { 0xf8d2ca04, 33 },
    { 0x9742ca04, 98 },
    { 0x4852ca06, 46 },
    { 0x7802ca07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe162ca14, 90 },
    { 0, 0 },
    { 0x73f2ca16, 90 },
    { 0xa122ca17, 61 },
    { 0xb0d2ca17, 33 },
    { 0x8152ca17, 33 },
    { 0x3e12ca18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32ca23, 61 },
    { 0x6f12ca24, 76 },
    { 0xa102ca25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb222ca28, 43 },
    { 0xe0f2ca29, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a52ca2e, 61 },
    { 0, 0 },
    { 0xa302ca30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b42ca35, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f52ca39, 43 },
    { 0xe712ca39, 101 },
    { 0x58d2ca3b, 33 },
    { 0x53e2ca3c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2112ca40, 78 },
    { 0, 0 },
    { 0xa482ca42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac2ca45, 65 },
    { 0x1a12ca45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x212ca49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca4c, 76 },
    { 0x7082ca4d, 34 },
    { 0xd42ca4e, 76 },
    { 0xa642ca4f, 61 },
    { 0, 0 },
    { 0x3582ca51, 61 },
    { 0, 0 },
    { 0x3122ca53, 93 },
    { 0x6f02ca53, 94 },
    { 0x6542ca55, 61 },
    { 0, 0 },
    { 0xf152ca57, 76 },
    { 0x33a2ca58, 90 },
    { 0x95d2ca57, 61 },
    { 0x8a42ca57, 61 },
    { 0x25e2ca5b, 90 },
    { 0x87e2ca5b, 81 },
    { 0x6ad2ca59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e2ca60, 61 },
    { 0xfae2ca61, 33 },
    { 0xd432ca62, 95 },
    { 0, 0 },
    { 0xe302ca64, 91 },
    { 0, 0 },
    { 0xb252ca66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc012ca6a, 61 },
    { 0, 0 },
    { 0x2022ca6c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e2ca6f, 33 },
    { 0, 0 },
    { 0x3a02ca71, 93 },
    { 0xf982ca72, 95 },
    { 0, 0 },
    { 0x6032ca74, 33 },
    { 0x6b12ca75, 98 },
    { 0x51e2ca76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732ca7b, 84 },
    { 0, 0 },
    { 0x6532ca7d, 75 },
    { 0xbf62ca7e, 99 },
    { 0x9e12ca7f, 93 },
    { 0, 0 },
    { 0xd292ca81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e2ca8b, 61 },
    { 0x6b02ca8c, 80 },
    { 0xe422ca8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda2ca97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6752ca9e, 78 },
    { 0, 0 },
    { 0x57b2caa0, 43 },
    { 0x1f92caa1, 33 },
    { 0x7bd2caa1, 23 },
    { 0, 0 },
    { 0xc392caa4, 90 },
    { 0, 0 },
    { 0xa9a2caa6, 23 },
    { 0xae82caa7, 23 },
    { 0x2cd2caa8, 98 },
    { 0xd532caa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc012caac, 23 },
    { 0, 0 },
    { 0xc642caae, 95 },
    { 0xf892caaf, 34 },
    { 0xdce2cab0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b2cab6, 98 },
    { 0, 0 },
    { 0xa692cab8, 98 },
    { 0, 0 },
    { 0xc4f2caba, 90 },
    { 0x4492cabb, 33 },
    { 0, 0 },
    { 0xfc82cabd, 90 },
    { 0x2132cabe, 91 },
    { 0xe472cabe, 61 },
    { 0, 0 },
    { 0xc3f2cac1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb2cac4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b62caca, 95 },
    { 0x7362cacb, 43 },
    { 0x6b52cacc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3662cad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e2cadd, 90 },
    { 0x5a12cade, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72cae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3682caeb, 98 },
    { 0x4542caeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9692caf0, 61 },
    { 0x2472caf1, 90 },
    { 0x8a2caf2, 66 },
    { 0xa8c2caf3, 93 },
    { 0, 0 },
    { 0xeb02caf5, 33 },
    { 0, 0 },
    { 0x7512caf7, 61 },
    { 0x7b82caf8, 78 },
    { 0x9972caf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd222cafe, 90 },
    { 0x5042cafe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa52cb04, 46 },
    { 0, 0 },
    { 0x48e2cb06, 61 },
    { 0, 0 },
    { 0x62a2cb08, 52 },
    { 0x4b32cb09, 34 },
    { 0xc922cb09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a2cb0d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd082cb19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe632cb22, 61 },
    { 0, 0 },
    { 0x9bf2cb24, 23 },
    { 0x48b2cb25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2cb2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad2cb2d, 33 },
    { 0xc832cb2e, 93 },
    { 0, 0 },
    { 0xc7a2cb30, 90 },
    { 0, 0 },
    { 0x6de2cb32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe2cb37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb2cb3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932cb48, 90 },
    { 0x8402cb49, 90 },
    { 0x4b22cb4a, 23 },
    { 0xc582cb4a, 90 },
    { 0x2092cb4c, 43 },
    { 0, 0 },
    { 0x8c92cb4e, 61 },
    { 0, 0 },
    { 0x3a92cb50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d2cb53, 90 },
    { 0xae62cb53, 61 },
    { 0xd732cb55, 52 },
    { 0xa872cb54, 98 },
    { 0xb412cb56, 91 },
    { 0xc842cb58, 87 },
    { 0x9732cb58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea2cb60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb22cb64, 61 },
    { 0xe862cb65, 61 },
    { 0x322cb64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d2cb6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e2cb77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f2cb7d, 95 },
    { 0, 0 },
    { 0x18e2cb7f, 90 },
    { 0xbae2cb80, 61 },
    { 0, 0 },
    { 0x13a2cb82, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b42cb85, 23 },
    { 0, 0 },
    { 0x5302cb87, 85 },
    { 0x3e32cb88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262cb8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb742cb93, 78 },
    { 0xcf82cb94, 98 },
    { 0xfe82cb95, 100 },
    { 0xd7a2cb96, 98 },
    { 0x70f2cb97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x302cb9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1042cb9e, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f2cba3, 90 },
    { 0, 0 },
    { 0x3802cba5, 99 },
    { 0, 0 },
    { 0x18f2cba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8292cbab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6432cbaf, 90 },
    { 0xe212cbb0, 90 },
    { 0, 0 },
    { 0x3722cbb2, 36 },
    { 0x79c2cbb2, 61 },
    { 0x2562cbb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2cbb7, 61 },
    { 0x4c02cbb7, 100 },
    { 0x51e2cbb9, 23 },
    { 0xad12cbba, 88 },
    { 0x6862cbbb, 95 },
    { 0, 0 },
    { 0xb4d2cbbd, 90 },
    { 0, 0 },
    { 0x16c2cbbf, 34 },
    { 0x1822cbc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a2cbc6, 95 },
    { 0x1a62cbc7, 91 },
    { 0, 0 },
    { 0x80f2cbc9, 43 },
    { 0x41d2cbca, 93 },
    { 0xedd2cbcb, 33 },
    { 0xc582cbcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc692cbd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7432cbd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c2cbd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8422cbda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2032cbde, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db2cbe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c2cbeb, 93 },
    { 0x6712cbec, 61 },
    { 0x5892cbed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f92cbf1, 43 },
    { 0, 0 },
    { 0x8c02cbf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4372cbf6, 33 },
    { 0x6a72cbf7, 52 },
    { 0xcd92cbf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f2cbfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9562cc01, 34 },
    { 0x5d82cc01, 90 },
    { 0, 0 },
    { 0xb7b2cc04, 98 },
    { 0x8b12cc05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f2cc08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2cc0b, 84 },
    { 0, 0 },
    { 0x4b62cc0d, 77 },
    { 0x82b2cc0e, 61 },
    { 0, 0 },
    { 0xe5d2cc10, 23 },
    { 0, 0 },
    { 0x1bf2cc12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a2cc17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd852cc1a, 55 },
    { 0, 0 },
    { 0x9d72cc1c, 17 },
    { 0x4f2cc1d, 34 },
    { 0xfb52cc1c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2cc22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3222cc27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2672cc2b, 90 },
    { 0xd152cc2c, 90 },
    { 0x2e62cc2d, 78 },
    { 0xf3e2cc2c, 34 },
    { 0xe812cc2e, 90 },
    { 0x8292cc30, 34 },
    { 0x3552cc30, 85 },
    { 0x83c2cc2c, 91 },
    { 0x4532cc31, 19 },
    { 0, 0 },
    { 0xa8e2cc35, 23 },
    { 0xbae2cc35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2cc39, 23 },
    { 0, 0 },
    { 0x8aa2cc3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3952cc3e, 23 },
    { 0x6d2cc3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd362cc42, 98 },
    { 0x4e92cc43, 75 },
    { 0x96c2cc44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3082cc4a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1462cc4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c2cc51, 55 },
    { 0x9012cc52, 90 },
    { 0, 0 },
    { 0x7c82cc54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c2cc58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e32cc5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb12cc62, 36 },
    { 0x17b2cc63, 61 },
    { 0x2412cc64, 90 },
    { 0, 0 },
    { 0xfeb2cc66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e2cc6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d2cc6d, 78 },
    { 0, 0 },
    { 0x3552cc6f, 90 },
    { 0, 0 },
    { 0x79e2cc71, 43 },
    { 0x7dc2cc71, 61 },
    { 0, 0 },
    { 0x76b2cc74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf2cc78, 46 },
    { 0xcbc2cc78, 95 },
    { 0, 0 },
    { 0xd0e2cc7b, 61 },
    { 0, 0 },
    { 0x13f2cc7d, 90 },
    { 0xf392cc7e, 85 },
    { 0, 0 },
    { 0x6082cc80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e92cc84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e2cc8b, 88 },
    { 0xdd92cc8b, 96 },
    { 0x6c52cc8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2482cc95, 23 },
    { 0x3a22cc95, 98 },
    { 0xee32cc97, 90 },
    { 0xdf72cc98, 23 },
    { 0x4ef2cc98, 34 },
    { 0xda32cc9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd2cc9d, 23 },
    { 0x98f2cc9d, 85 },
    { 0, 0 },
    { 0x5d72cca0, 90 },
    { 0, 0 },
    { 0x8542cca2, 52 },
    { 0xd762cca3, 90 },
    { 0xaa82cca2, 23 },
    { 0x47d2cca2, 90 },
    { 0x6b22cca6, 91 },
    { 0x5a52cca3, 34 },
    { 0, 0 },
    { 0xb5e2cca9, 98 },
    { 0, 0 },
    { 0x7ef2ccab, 23 },
    { 0x3202ccac, 99 },
    { 0x5ad2ccad, 34 },
    { 0xde92ccae, 98 },
    { 0x3e42ccaf, 33 },
    { 0xc672ccad, 98 },
    { 0, 0 },
    { 0x99c2ccb2, 34 },
    { 0xe62ccb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff2ccb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7832ccb9, 90 },
    { 0x762ccba, 33 },
    { 0, 0 },
    { 0x3b62ccbc, 23 },
    { 0x7da2ccbd, 90 },
    { 0xc4e2ccbc, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x5872ccc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xda62ccc4, 84 },
    { 0, 0 },
    { 0x66a2ccc6, 67 },
    { 0xb7d2ccc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d82ccca, 33 },
    { 0, 0 },
    { 0x7212cccc, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7952ccd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592ccd6, 66 },
    { 0, 0 },
    { 0xba62ccd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3302ccdc, 77 },
    { 0xd892ccdd, 61 },
    { 0, 0 },
    { 0x99c2ccdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a2cce2, 33 },
    { 0x1132cce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8462cce8, 93 },
    { 0xdb32cce8, 33 },
    { 0xe92cce8, 33 },
    { 0xfa52cce9, 33 },
    { 0x3592ccec, 61 },
    { 0x7bc2cceb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7152ccf1, 61 },
    { 0xcf12ccf2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02ccf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f2ccf9, 61 },
    { 0, 0 },
    { 0x6e22ccfb, 33 },
    { 0x7312ccfc, 87 },
    { 0x1d62ccfd, 90 },
    { 0x51a2ccfc, 19 },
    { 0, 0 },
    { 0xa012cd00, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad22cd05, 55 },
    { 0, 0 },
    { 0x3ac2cd07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f2cd0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f2cd1b, 90 },
    { 0, 0 },
    { 0x88d2cd1d, 95 },
    { 0xc6c2cd1e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2cd21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c52cd25, 43 },
    { 0x2e32cd26, 33 },
    { 0xc9f2cd27, 99 },
    { 0, 0 },
    { 0xd942cd29, 61 },
    { 0xed22cd2a, 90 },
    { 0, 0 },
    { 0xd2e2cd2c, 76 },
    { 0xd962cd2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2cd30, 55 },
    { 0x1182cd31, 90 },
    { 0xc092cd31, 61 },
    { 0xc072cd33, 43 },
    { 0x7eb2cd30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1982cd3b, 93 },
    { 0xaa82cd3c, 100 },
    { 0x7fe2cd3d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc962cd44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e2cd47, 91 },
    { 0x4a2cd48, 95 },
    { 0x3512cd49, 43 },
    { 0xedf2cd47, 78 },
    { 0xdf2cd4b, 66 },
    { 0xda42cd49, 33 },
    { 0, 0 },
    { 0x54f2cd4e, 100 },
    { 0, 0 },
    { 0x5532cd50, 98 },
    { 0, 0 },
    { 0x86c2cd52, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b2cd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e2cd5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22cd5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e2cd62, 23 },
    { 0x77a2cd62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca52cd6a, 61 },
    { 0, 0 },
    { 0x7312cd6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f02cd72, 76 },
    { 0, 0 },
    { 0x5b92cd74, 66 },
    { 0x90e2cd75, 90 },
    { 0x9902cd74, 99 },
    { 0xf0d2cd74, 33 },
    { 0x3c62cd74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd552cd80, 23 },
    { 0xb22cd81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f42cd84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb2cd89, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c2cd8e, 93 },
    { 0xe7c2cd8f, 43 },
    { 0x6652cd90, 90 },
    { 0, 0 },
    { 0xcb62cd92, 95 },
    { 0xc862cd93, 93 },
    { 0x47e2cd93, 23 },
    { 0, 0 },
    { 0x3182cd96, 80 },
    { 0, 0 },
    { 0x1ed2cd98, 90 },
    { 0x8862cd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592cd9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52cda0, 34 },
    { 0, 0 },
    { 0x7b52cda2, 55 },
    { 0, 0 },
    { 0x5462cda4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a2cda9, 34 },
    { 0x2672cdaa, 42 },
    { 0xb232cda9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb742cdae, 77 },
    { 0x6c32cdae, 33 },
    { 0x9ec2cdb0, 49 },
    { 0, 0 },
    { 0xda22cdb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xed12cdb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2cdb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a02cdbd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db2cdc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42cdc7, 90 },
    { 0x11d2cdc7, 90 },
    { 0, 0 },
    { 0xd702cdca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3502cdcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4532cdd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd962cdd9, 66 },
    { 0, 0 },
    { 0x55a2cddb, 65 },
    { 0xfc2cddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c82cde1, 98 },
    { 0, 0 },
    { 0xb9d2cde3, 33 },
    { 0xb242cde4, 34 },
    { 0, 0 },
    { 0x5442cde6, 78 },
    { 0x54e2cde7, 61 },
    { 0x542cde8, 100 },
    { 0x2212cde9, 81 },
    { 0x472cde9, 68 },
    { 0xb842cdea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d22cdee, 51 },
    { 0, 0 },
    { 0x85d2cdf0, 85 },
    { 0, 0 },
    { 0x7762cdf2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7592cdf5, 80 },
    { 0x9ed2cdf6, 76 },
    { 0xd602cdf6, 76 },
    { 0x5202cdf7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f2cdfe, 43 },
    { 0, 0 },
    { 0x4352ce00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2ce0a, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2542ce0e, 85 },
    { 0x6c52ce0f, 61 },
    { 0xd6f2ce0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d2ce14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942ce18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d32ce1c, 61 },
    { 0x1f52ce1c, 23 },
    { 0xf792ce1e, 90 },
    { 0xda02ce1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e82ce25, 61 },
    { 0xff92ce26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf612ce2a, 78 },
    { 0, 0 },
    { 0x9592ce2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6472ce30, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e42ce35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a2ce39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb862ce48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2552ce4d, 76 },
    { 0x9e12ce4e, 76 },
    { 0, 0 },
    { 0xa032ce50, 52 },
    { 0x4162ce51, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7562ce55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe952ce58, 33 },
    { 0xd7e2ce59, 23 },
    { 0xdb12ce59, 61 },
    { 0x42d2ce5b, 33 },
    { 0x6dd2ce5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de2ce60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe562ce64, 95 },
    { 0x952ce65, 65 },
    { 0xbf92ce66, 61 },
    { 0xd602ce65, 43 },
    { 0xd5a2ce68, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ce6b, 61 },
    { 0xd712ce6b, 90 },
    { 0, 0 },
    { 0x51c2ce6e, 90 },
    { 0x1d42ce6f, 98 },
    { 0x1d2ce70, 33 },
    { 0x35e2ce71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6392ce74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb02ce78, 99 },
    { 0, 0 },
    { 0x3ad2ce7a, 95 },
    { 0, 0 },
    { 0x70f2ce7c, 23 },
    { 0x812ce7c, 76 },
    { 0xb442ce7e, 61 },
    { 0x5292ce7e, 98 },
    { 0x9d42ce7c, 99 },
    { 0x53e2ce7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf712ce8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc132ce93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4212cea1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2cea6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc2ceaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42cec0, 95 },
    { 0x5c92cec1, 90 },
    { 0x2ab2cec2, 98 },
    { 0x1482cec0, 66 },
    { 0x76a2cec4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f32cecb, 34 },
    { 0xe572cecc, 61 },
    { 0, 0 },
    { 0x6822cece, 33 },
    { 0x4a92cecf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8642ced6, 98 },
    { 0, 0 },
    { 0x4ba2ced8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc42cedb, 95 },
    { 0x3272cedc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3402cee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2cee3, 33 },
    { 0xf882cee3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b72cee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e2ceea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4992ceed, 33 },
    { 0, 0 },
    { 0x3cb2ceef, 78 },
    { 0xd112ceef, 46 },
    { 0x2b62ceef, 95 },
    { 0xa652cef2, 78 },
    { 0x4ab2cef2, 99 },
    { 0, 0 },
    { 0x2082cef5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c2cefd, 84 },
    { 0xc0d2cefd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc2cf01, 77 },
    { 0xa2d2cf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8452cf07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7392cf0d, 90 },
    { 0xe1d2cf0e, 61 },
    { 0, 0 },
    { 0x1202cf10, 100 },
    { 0xcca2cf10, 43 },
    { 0x6e52cf12, 61 },
    { 0x7b92cf10, 100 },
    { 0x6432cf14, 43 },
    { 0x8642cf13, 90 },
    { 0xe402cf14, 66 },
    { 0x3682cf15, 81 },
    { 0x4502cf17, 61 },
    { 0x2462cf19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa212cf23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd42cf29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1102cf2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f2cf31, 61 },
    { 0xa632cf31, 33 },
    { 0x752cf33, 90 },
    { 0x86d2cf34, 95 },
    { 0xd322cf35, 23 },
    { 0xc5c2cf36, 98 },
    { 0xa8d2cf37, 78 },
    { 0xcd62cf32, 98 },
    { 0x6fd2cf39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc242cf3c, 61 },
    { 0x9262cf3c, 95 },
    { 0x9312cf3e, 76 },
    { 0x13c2cf3d, 84 },
    { 0, 0 },
    { 0xcdb2cf41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a62cf45, 46 },
    { 0xcb92cf46, 33 },
    { 0x4382cf47, 93 },
    { 0, 0 },
    { 0x49e2cf49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a2cf4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe902cf51, 98 },
    { 0x16a2cf51, 33 },
    { 0xb212cf51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa92cf5c, 88 },
    { 0xf9e2cf5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d22cf60, 76 },
    { 0x3292cf61, 90 },
    { 0xafb2cf62, 90 },
    { 0x96a2cf63, 66 },
    { 0x662cf64, 91 },
    { 0x5ba2cf65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a2cf6a, 34 },
    { 0xffa2cf6b, 33 },
    { 0, 0 },
    { 0xfbb2cf6d, 33 },
    { 0x9a92cf6d, 90 },
    { 0x21c2cf6d, 90 },
    { 0x4452cf70, 90 },
    { 0, 0 },
    { 0xb522cf72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf2cf76, 61 },
    { 0, 0 },
    { 0x2c12cf78, 64 },
    { 0x4b92cf79, 64 },
    { 0, 0 },
    { 0xc202cf7b, 81 },
    { 0x13a2cf7c, 93 },
    { 0x2592cf7d, 33 },
    { 0xa772cf7d, 33 },
    { 0x2db2cf7d, 99 },
    { 0x89d2cf7d, 78 },
    { 0x7132cf7f, 61 },
    { 0xb572cf7c, 19 },
    { 0xc8f2cf83, 61 },
    { 0x9ed2cf83, 52 },
    { 0xf5b2cf85, 95 },
    { 0, 0 },
    { 0x3702cf87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f2cf8d, 34 },
    { 0xbba2cf8e, 43 },
    { 0x2542cf8f, 33 },
    { 0x5402cf90, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa2cf93, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52cf98, 93 },
    { 0x7e62cf99, 33 },
    { 0xc4a2cf9a, 38 },
    { 0x8402cf98, 61 },
    { 0xe652cf9c, 33 },
    { 0xc102cf9d, 36 },
    { 0x38e2cf9e, 67 },
    { 0x7d72cf9f, 93 },
    { 0x4102cfa0, 90 },
    { 0x9f2cfa1, 33 },
    { 0x4b82cf9b, 91 },
    { 0xdd82cfa3, 77 },
    { 0x5eb2cfa4, 33 },
    { 0xc212cfa3, 76 },
    { 0, 0 },
    { 0x3fc2cfa7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf12cfae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a2cfb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6052cfbb, 98 },
    { 0x3562cfbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e02cfc2, 90 },
    { 0x5dd2cfc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b32cfcb, 95 },
    { 0x75a2cfcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd22cfcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd862cfd3, 78 },
    { 0x3192cfd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b2cfdf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee2cfe5, 76 },
    { 0, 0 },
    { 0xddb2cfe7, 84 },
    { 0x7c92cfe8, 66 },
    { 0, 0 },
    { 0xae52cfea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9462cfee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc362cff2, 90 },
    { 0xa392cff2, 55 },
    { 0xf312cff3, 95 },
    { 0, 0 },
    { 0xa8e2cff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942cffc, 85 },
    { 0xe1f2cffc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b92d001, 85 },
    { 0xb2d2d002, 90 },
    { 0xf142d001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da2d007, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea42d00c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752d010, 90 },
    { 0xde72d010, 95 },
    { 0xc1e2d012, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea2d018, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c2d01b, 99 },
    { 0x3652d01c, 52 },
    { 0x7272d01b, 98 },
    { 0xe6b2d01e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a2d02d, 23 },
    { 0, 0 },
    { 0x4e72d02f, 33 },
    { 0x6c42d030, 95 },
    { 0xa472d030, 23 },
    { 0x56e2d032, 61 },
    { 0x25a2d033, 95 },
    { 0x91d2d033, 46 },
    { 0x26b2d032, 93 },
    { 0x1592d036, 95 },
    { 0xcc62d036, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc312d03b, 53 },
    { 0x1c2d03c, 90 },
    { 0x1132d03b, 17 },
    { 0, 0 },
    { 0x9a82d03f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9582d042, 90 },
    { 0, 0 },
    { 0x4ae2d044, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3602d047, 91 },
    { 0xb432d048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2d04b, 90 },
    { 0xecc2d04b, 90 },
    { 0, 0 },
    { 0x6122d04e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe352d053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e82d056, 81 },
    { 0, 0 },
    { 0xedb2d058, 61 },
    { 0, 0 },
    { 0x352d05a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52d05e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db2d061, 90 },
    { 0xa0b2d062, 90 },
    { 0x35a2d063, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a2d06b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5892d071, 90 },
    { 0, 0 },
    { 0x3652d073, 95 },
    { 0x54b2d074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee2d07a, 33 },
    { 0xcaf2d07b, 61 },
    { 0x9cb2d07a, 76 },
    { 0x4452d07d, 98 },
    { 0x6f32d07a, 23 },
    { 0xb2f2d07f, 95 },
    { 0x21a2d07f, 61 },
    { 0x8bf2d07b, 33 },
    { 0, 0 },
    { 0xde62d083, 64 },
    { 0x2362d084, 90 },
    { 0x4bd2d085, 80 },
    { 0x8652d086, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xb532d089, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2142d08f, 55 },
    { 0x2232d08f, 34 },
    { 0x8582d091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc782d095, 90 },
    { 0xeea2d095, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5332d09c, 56 },
    { 0x90d2d09c, 98 },
    { 0x1742d09c, 90 },
    { 0x7292d09f, 23 },
    { 0x2172d0a0, 84 },
    { 0x6022d0a0, 95 },
    { 0x7682d0a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e52d0a5, 61 },
    { 0xd6b2d0a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e12d0ab, 95 },
    { 0, 0 },
    { 0x9902d0ad, 23 },
    { 0x9102d0ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed2d0b2, 75 },
    { 0x4292d0b2, 34 },
    { 0x8482d0b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb2d0b7, 67 },
    { 0x6962d0b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3052d0c0, 34 },
    { 0x6882d0c0, 90 },
    { 0x24a2d0c0, 33 },
    { 0x7982d0c3, 61 },
    { 0x9852d0c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc922d0c7, 69 },
    { 0, 0 },
    { 0x89b2d0c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82d0d1, 38 },
    { 0, 0 },
    { 0x2962d0d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a2d0d6, 33 },
    { 0, 0 },
    { 0x23e2d0d8, 87 },
    { 0, 0 },
    { 0xe3d2d0da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf552d0de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e2d0e1, 17 },
    { 0, 0 },
    { 0x2872d0e3, 52 },
    { 0, 0 },
    { 0x23c2d0e5, 85 },
    { 0x7402d0e6, 33 },
    { 0xa262d0e6, 90 },
    { 0x1e62d0e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce2d0ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca72d0f7, 95 },
    { 0, 0 },
    { 0xf672d0f9, 61 },
    { 0, 0 },
    { 0x2b82d0fb, 55 },
    { 0x9192d0fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b42d0ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62d102, 84 },
    { 0, 0 },
    { 0x4ac2d104, 84 },
    { 0xbd32d104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be2d108, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d12d10e, 98 },
    { 0, 0 },
    { 0x3222d110, 52 },
    { 0xf4a2d111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae2d117, 90 },
    { 0x5e52d118, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e2d11f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8462d125, 90 },
    { 0x9fe2d126, 90 },
    { 0xc1a2d125, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa352d12a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe082d12f, 95 },
    { 0, 0 },
    { 0x80e2d131, 76 },
    { 0xeef2d132, 95 },
    { 0x382d132, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5332d137, 95 },
    { 0xfac2d138, 95 },
    { 0xfde2d138, 61 },
    { 0, 0 },
    { 0x2fa2d13b, 33 },
    { 0x38c2d13c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e22d140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f02d147, 65 },
    { 0xd3a2d147, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec2d14b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9962d150, 23 },
    { 0x2fd2d151, 90 },
    { 0x2a52d152, 76 },
    { 0x2332d153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf292d15a, 33 },
    { 0xeb32d15b, 93 },
    { 0xc7c2d15c, 90 },
    { 0x6662d15d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8052d161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee2d164, 68 },
    { 0x75c2d164, 85 },
    { 0x8372d164, 33 },
    { 0xa572d167, 52 },
    { 0xf012d167, 23 },
    { 0x6dc2d168, 23 },
    { 0xc492d169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c2d16d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8522d170, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d2d173, 23 },
    { 0x7f2d173, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a2d177, 23 },
    { 0xa582d178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b92d17b, 87 },
    { 0, 0 },
    { 0xe722d17d, 93 },
    { 0x5452d17d, 99 },
    { 0x5c52d17f, 55 },
    { 0xfae2d180, 40 },
    { 0x6b12d17d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b2d184, 33 },
    { 0x7e62d185, 61 },
    { 0x5262d186, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b2d191, 95 },
    { 0xd382d192, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd102d196, 17 },
    { 0xec92d196, 33 },
    { 0x5b42d198, 52 },
    { 0xcfc2d197, 55 },
    { 0x95f2d19a, 43 },
    { 0x1a82d199, 65 },
    { 0xe332d19c, 33 },
    { 0xa32d19d, 75 },
    { 0x4422d19a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc562d1a1, 61 },
    { 0x5422d1a1, 66 },
    { 0x9da2d1a3, 65 },
    { 0x4762d1a3, 43 },
    { 0x4242d1a5, 23 },
    { 0xec52d1a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf2d1aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa562d1b2, 90 },
    { 0x4f22d1b2, 95 },
    { 0, 0 },
    { 0x9562d1b5, 33 },
    { 0xaa12d1b6, 55 },
    { 0xd4e2d1b6, 33 },
    { 0x6242d1b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082d1bb, 95 },
    { 0x5b32d1bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8992d1c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e2d1c4, 95 },
    { 0x56f2d1c4, 84 },
    { 0x2d1c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e2d1ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972d1d0, 90 },
    { 0xc6e2d1d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd472d1d7, 91 },
    { 0, 0 },
    { 0x5202d1d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f2d1dc, 33 },
    { 0x4d72d1dc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe932d1e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a2d1e3, 84 },
    { 0, 0 },
    { 0x852d1e5, 99 },
    { 0x99a2d1e6, 33 },
    { 0xc902d1e5, 33 },
    { 0x4b92d1e7, 91 },
    { 0xdb32d1e6, 76 },
    { 0xa7c2d1e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b52d1ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e72d1f7, 91 },
    { 0, 0 },
    { 0x682d1f9, 95 },
    { 0x66a2d1fa, 91 },
    { 0xc9a2d1fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2d205, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa022d209, 33 },
    { 0xd6e2d209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d42d210, 95 },
    { 0, 0 },
    { 0x4242d212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c2d215, 55 },
    { 0x3f72d216, 90 },
    { 0xe652d217, 43 },
    { 0x8902d217, 23 },
    { 0xf682d218, 95 },
    { 0xfd82d216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e2d21f, 88 },
    { 0xa142d21f, 81 },
    { 0xf762d220, 33 },
    { 0, 0 },
    { 0x362d223, 33 },
    { 0xd2d224, 33 },
    { 0x7712d223, 55 },
    { 0x6802d226, 33 },
    { 0xf772d226, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd252d22b, 67 },
    { 0xf332d22c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8772d230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb262d235, 97 },
    { 0x9cd2d235, 91 },
    { 0, 0 },
    { 0xf682d238, 23 },
    { 0, 0 },
    { 0x1e32d23a, 76 },
    { 0x942d23a, 33 },
    { 0, 0 },
    { 0x1d02d23d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b2d240, 95 },
    { 0xab82d241, 33 },
    { 0x2d02d242, 77 },
    { 0, 0 },
    { 0xad02d244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c2d248, 90 },
    { 0x40f2d248, 98 },
    { 0xfb62d24a, 43 },
    { 0x2ec2d24b, 61 },
    { 0x2652d24c, 43 },
    { 0x8452d24c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb2d254, 90 },
    { 0, 0 },
    { 0x8802d256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c2d25a, 46 },
    { 0, 0 },
    { 0xdc52d25c, 90 },
    { 0x1e22d25c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b2d262, 95 },
    { 0x81e2d262, 98 },
    { 0, 0 },
    { 0x8712d265, 76 },
    { 0, 0 },
    { 0x4762d267, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe2d26a, 46 },
    { 0xbfd2d26a, 90 },
    { 0, 0 },
    { 0xb902d26d, 33 },
    { 0x7ae2d26d, 61 },
    { 0, 0 },
    { 0x5132d270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a2d275, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd2d279, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa52d27d, 90 },
    { 0, 0 },
    { 0xe9d2d27f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca52d282, 52 },
    { 0x7312d283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc272d287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb842d28b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d62d28e, 93 },
    { 0x4662d28f, 84 },
    { 0x5f2d290, 62 },
    { 0x4a32d290, 33 },
    { 0xd712d292, 61 },
    { 0x2062d290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f2d298, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2d29c, 23 },
    { 0, 0 },
    { 0x95a2d29e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412d2a2, 91 },
    { 0xb12d2a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c2d2a6, 23 },
    { 0xaad2d2a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b2d2aa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d02d2ad, 90 },
    { 0x7572d2ae, 91 },
    { 0xe082d2af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df2d2b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc322d2b6, 93 },
    { 0x8242d2b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf22d2bb, 90 },
    { 0, 0 },
    { 0xed82d2bd, 84 },
    { 0, 0 },
    { 0xba52d2bf, 43 },
    { 0, 0 },
    { 0x99c2d2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a2d2c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8872d2cf, 81 },
    { 0xc682d2d0, 43 },
    { 0x25e2d2cf, 61 },
    { 0xd282d2d0, 98 },
    { 0xba92d2d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd002d2d7, 90 },
    { 0, 0 },
    { 0x1dd2d2d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2d2dc, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2d2e4, 76 },
    { 0x1992d2e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc802d2f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6712d2f5, 55 },
    { 0xfe32d2f5, 90 },
    { 0, 0 },
    { 0xd132d2f8, 85 },
    { 0x6602d2f9, 90 },
    { 0x26d2d2fa, 33 },
    { 0x92e2d2f9, 34 },
    { 0x70b2d2fc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c2d300, 75 },
    { 0x4b92d301, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752d305, 90 },
    { 0x9e82d306, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de2d311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa372d314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca82d318, 95 },
    { 0x90f2d318, 98 },
    { 0, 0 },
    { 0xb4f2d31b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a12d31f, 55 },
    { 0xa032d320, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa2d323, 87 },
    { 0x2f12d324, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3122d328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf42d32b, 75 },
    { 0xe392d32b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5152d332, 66 },
    { 0, 0 },
    { 0x4152d334, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e92d337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf82d33a, 76 },
    { 0xcba2d33b, 90 },
    { 0x34f2d33b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1572d342, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc322d345, 95 },
    { 0x4512d346, 76 },
    { 0, 0 },
    { 0xd292d348, 98 },
    { 0, 0 },
    { 0x6312d34a, 23 },
    { 0xf922d34a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1732d350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6992d354, 98 },
    { 0, 0 },
    { 0x9642d356, 43 },
    { 0, 0 },
    { 0x9bb2d358, 33 },
    { 0xf192d358, 23 },
    { 0xff72d358, 90 },
    { 0x4422d358, 98 },
    { 0x84e2d35c, 90 },
    { 0x5f22d35c, 75 },
    { 0x6d32d35e, 66 },
    { 0xe092d35f, 33 },
    { 0xd92d35f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b2d366, 87 },
    { 0x9422d366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c62d36b, 33 },
    { 0x7532d36b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c72d36f, 98 },
    { 0, 0 },
    { 0xae72d371, 76 },
    { 0x4342d372, 90 },
    { 0xa822d373, 33 },
    { 0x6142d372, 83 },
    { 0xfb72d372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e2d378, 91 },
    { 0x6252d379, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a02d37c, 77 },
    { 0x4e82d37d, 98 },
    { 0xe652d37e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b2d381, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5242d384, 90 },
    { 0, 0 },
    { 0x2a32d386, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af2d389, 95 },
    { 0x2a12d38a, 93 },
    { 0x55d2d38a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da2d397, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c2d39a, 98 },
    { 0, 0 },
    { 0x1112d39c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2d3a1, 85 },
    { 0, 0 },
    { 0x9c92d3a3, 98 },
    { 0, 0 },
    { 0xdb52d3a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a2d3a9, 23 },
    { 0, 0 },
    { 0x2a12d3ab, 52 },
    { 0, 0 },
    { 0xfd62d3ad, 81 },
    { 0, 0 },
    { 0x4bc2d3af, 95 },
    { 0x13e2d3b0, 23 },
    { 0xe5c2d3b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa192d3b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f22d3b7, 93 },
    { 0xb522d3b7, 98 },
    { 0xf682d3b8, 95 },
    { 0xeb2d3ba, 84 },
    { 0xb952d3bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee42d3be, 34 },
    { 0, 0 },
    { 0xb582d3c0, 43 },
    { 0xec52d3c0, 61 },
    { 0x8392d3c2, 98 },
    { 0, 0 },
    { 0x57f2d3c4, 61 },
    { 0x51d2d3c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf542d3c9, 43 },
    { 0, 0 },
    { 0xb2e2d3cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb212d3ce, 85 },
    { 0, 0 },
    { 0x7a32d3d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa172d3d3, 99 },
    { 0x82e2d3d4, 90 },
    { 0x9f12d3d5, 76 },
    { 0xb402d3d3, 33 },
    { 0xf162d3d7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7412d3da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd2d3dd, 23 },
    { 0, 0 },
    { 0xea02d3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d92d3e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a2d3e9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f2d3ec, 68 },
    { 0x3132d3ec, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd2d3f2, 90 },
    { 0, 0 },
    { 0x3c32d3f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22d3f7, 33 },
    { 0, 0 },
    { 0xab42d3f9, 74 },
    { 0x2b02d3fa, 85 },
    { 0x2152d3fa, 34 },
    { 0x23e2d3fc, 95 },
    { 0xc6a2d3fd, 61 },
    { 0x6142d3fa, 33 },
    { 0x50e2d3fb, 34 },
    { 0, 0 },
    { 0x7ba2d401, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe352d408, 76 },
    { 0x4422d408, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e2d40e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3812d411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b2d416, 76 },
    { 0x15f2d417, 84 },
    { 0xa612d417, 61 },
    { 0xbc72d417, 23 },
    { 0x9392d41a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae2d423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8132d426, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd2d42b, 61 },
    { 0x97d2d42b, 61 },
    { 0, 0 },
    { 0x3302d42e, 81 },
    { 0xaa12d42f, 17 },
    { 0x1642d42e, 23 },
    { 0xa4f2d430, 61 },
    { 0x8af2d432, 61 },
    { 0x2652d432, 98 },
    { 0xff82d42e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9532d438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f72d43b, 87 },
    { 0x1ed2d43b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe2d440, 91 },
    { 0x56a2d441, 66 },
    { 0, 0 },
    { 0xbc82d443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2d449, 76 },
    { 0xa632d449, 34 },
    { 0xebb2d44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd962d452, 46 },
    { 0, 0 },
    { 0xd862d454, 90 },
    { 0, 0 },
    { 0x8632d456, 34 },
    { 0x932d457, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d22d45a, 61 },
    { 0, 0 },
    { 0xdd2d45c, 33 },
    { 0, 0 },
    { 0xbe52d45e, 95 },
    { 0x11d2d45f, 34 },
    { 0, 0 },
    { 0x92a2d461, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc2d465, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252d468, 90 },
    { 0xec52d469, 90 },
    { 0, 0 },
    { 0xb442d46b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe2d46f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe462d474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b62d47a, 52 },
    { 0x2db2d47a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2d480, 23 },
    { 0x2962d481, 98 },
    { 0x1a52d482, 76 },
    { 0x8cc2d483, 90 },
    { 0xf972d484, 17 },
    { 0xc4d2d481, 61 },
    { 0xccb2d484, 100 },
    { 0x6a22d485, 23 },
    { 0x4c52d488, 43 },
    { 0, 0 },
    { 0xaca2d48a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2d490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba2d493, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8882d497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2d49b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9022d49f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df2d4a2, 34 },
    { 0, 0 },
    { 0x3632d4a4, 43 },
    { 0x9ee2d4a5, 55 },
    { 0x8042d4a6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72d4a9, 42 },
    { 0x3b82d4aa, 78 },
    { 0x152d4aa, 5 },
    { 0xb6b2d4ac, 23 },
    { 0, 0 },
    { 0x5a52d4ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5212d4b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe492d4b7, 55 },
    { 0x1c42d4b7, 95 },
    { 0x46e2d4b7, 88 },
    { 0x6da2d4ba, 93 },
    { 0xaf72d4bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2d4c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d12d4c3, 84 },
    { 0x7452d4c4, 95 },
    { 0, 0 },
    { 0x3202d4c6, 98 },
    { 0, 0 },
    { 0x9c72d4c8, 98 },
    { 0x14f2d4c8, 90 },
    { 0, 0 },
    { 0x8942d4cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae2d4ce, 68 },
    { 0xbe52d4ce, 76 },
    { 0x7a2d4d0, 76 },
    { 0, 0 },
    { 0x3252d4d2, 61 },
    { 0, 0 },
    { 0xd082d4d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e82d4d9, 99 },
    { 0x3a42d4d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa22d4e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972d4e4, 98 },
    { 0xaa12d4e5, 101 },
    { 0x8c92d4e6, 33 },
    { 0xbad2d4e7, 90 },
    { 0x9a72d4e5, 61 },
    { 0xb492d4e9, 95 },
    { 0x55f2d4ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd2d4f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2d4f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f82d4fb, 90 },
    { 0xa312d4fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6512d50e, 95 },
    { 0x2dc2d50f, 61 },
    { 0x9c2d510, 61 },
    { 0, 0 },
    { 0xaff2d512, 55 },
    { 0, 0 },
    { 0x6ea2d514, 99 },
    { 0xc52d514, 95 },
    { 0, 0 },
    { 0x4482d517, 90 },
    { 0xaaf2d518, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc422d520, 33 },
    { 0x7752d521, 90 },
    { 0, 0 },
    { 0x4ed2d523, 95 },
    { 0xe852d524, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce2d52e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde82d532, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x5542d535, 77 },
    { 0x8ed2d536, 23 },
    { 0xdc12d537, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a2d53a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c2d542, 33 },
    { 0x1b32d542, 34 },
    { 0, 0 },
    { 0x3782d545, 98 },
    { 0xde32d545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe762d549, 95 },
    { 0xf9b2d54a, 33 },
    { 0, 0 },
    { 0x3e32d54c, 67 },
    { 0xb112d54d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52d550, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e12d554, 33 },
    { 0xc3a2d555, 100 },
    { 0xd022d556, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd992d559, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb2d55d, 98 },
    { 0xf5a2d55d, 95 },
    { 0x4542d55d, 100 },
    { 0, 0 },
    { 0xf8d2d561, 61 },
    { 0x38f2d562, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b2d565, 77 },
    { 0x8ce2d566, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf52d569, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d2d56f, 98 },
    { 0, 0 },
    { 0xca02d571, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d576, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1132d579, 93 },
    { 0xd9d2d57a, 33 },
    { 0xbd22d57b, 65 },
    { 0, 0 },
    { 0x3f82d57d, 61 },
    { 0xea72d57e, 61 },
    { 0, 0 },
    { 0x2912d580, 33 },
    { 0xe7d2d581, 61 },
    { 0xde2d582, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b2d58b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa582d590, 94 },
    { 0, 0 },
    { 0xf12d592, 66 },
    { 0xd242d593, 90 },
    { 0xd622d594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc802d598, 61 },
    { 0xca2d599, 23 },
    { 0x1982d59a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce2d5a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2122d5a4, 95 },
    { 0, 0 },
    { 0x8be2d5a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302d5af, 93 },
    { 0, 0 },
    { 0x13e2d5b1, 69 },
    { 0, 0 },
    { 0xda02d5b3, 33 },
    { 0x2b32d5b3, 95 },
    { 0x2482d5b5, 52 },
    { 0xb162d5b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c2d5c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6772d5c7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd392d5cb, 34 },
    { 0x1a42d5cb, 98 },
    { 0, 0 },
    { 0xb3c2d5ce, 23 },
    { 0x42d5cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5532d5d2, 90 },
    { 0xca22d5d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8362d5d6, 84 },
    { 0x1132d5d6, 95 },
    { 0, 0 },
    { 0xfe52d5d9, 46 },
    { 0xe912d5d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272d5e1, 87 },
    { 0xc6e2d5e2, 33 },
    { 0x9dc2d5e3, 95 },
    { 0x5f62d5e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c52d5e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e2d5ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1532d5f2, 61 },
    { 0x3652d5f3, 38 },
    { 0x66d2d5f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf852d5f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd182d5fb, 52 },
    { 0x80b2d5fc, 92 },
    { 0, 0 },
    { 0x6fb2d5fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca2d602, 90 },
    { 0, 0 },
    { 0x8ed2d604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d72d608, 90 },
    { 0x5d92d609, 93 },
    { 0, 0 },
    { 0x2562d60b, 34 },
    { 0, 0 },
    { 0x4df2d60d, 55 },
    { 0x5372d60d, 95 },
    { 0x5342d60e, 95 },
    { 0, 0 },
    { 0x46e2d611, 55 },
    { 0x1262d612, 100 },
    { 0x9e82d613, 33 },
    { 0xa212d613, 98 },
    { 0x83d2d615, 23 },
    { 0x6c62d616, 81 },
    { 0, 0 },
    { 0xfcc2d618, 61 },
    { 0x25b2d619, 100 },
    { 0x9e82d61a, 31 },
    { 0x3d2d618, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa2d61f, 98 },
    { 0, 0 },
    { 0x12f2d621, 33 },
    { 0x47c2d622, 61 },
    { 0x6602d623, 98 },
    { 0x70a2d621, 23 },
    { 0xbcf2d625, 46 },
    { 0, 0 },
    { 0x2522d627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8082d62d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d22d632, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb422d63b, 66 },
    { 0xc9f2d63c, 90 },
    { 0x6b12d63c, 90 },
    { 0xc6f2d63c, 61 },
    { 0xd4b2d63d, 23 },
    { 0x5792d63b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252d644, 61 },
    { 0xa292d645, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff72d648, 100 },
    { 0x6af2d648, 33 },
    { 0xa882d64a, 33 },
    { 0xd112d64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf632d653, 99 },
    { 0x8522d653, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef2d657, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c42d65d, 85 },
    { 0x5b92d65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e2d663, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1952d669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a92d672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2d67a, 40 },
    { 0, 0 },
    { 0x4d02d67c, 87 },
    { 0x5592d67c, 100 },
    { 0xd9a2d67e, 87 },
    { 0x7bf2d67f, 87 },
    { 0xfd92d67f, 46 },
    { 0x1532d681, 95 },
    { 0x1662d67c, 67 },
    { 0xa2e2d683, 46 },
    { 0x6b72d67e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb62d68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5462d68f, 43 },
    { 0, 0 },
    { 0x8932d691, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be2d695, 61 },
    { 0x2222d695, 66 },
    { 0, 0 },
    { 0xe3c2d698, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d69b, 61 },
    { 0, 0 },
    { 0x7102d69d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc02d6a5, 34 },
    { 0, 0 },
    { 0x96c2d6a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2d6af, 66 },
    { 0, 0 },
    { 0xa6a2d6b1, 33 },
    { 0x6582d6b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2632d6ba, 81 },
    { 0xc512d6ba, 95 },
    { 0, 0 },
    { 0x1362d6bd, 98 },
    { 0x632d6be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d2d6cd, 46 },
    { 0x3d72d6ce, 90 },
    { 0, 0 },
    { 0xa132d6d0, 23 },
    { 0xbb72d6d0, 33 },
    { 0x99c2d6d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf632d6d8, 76 },
    { 0, 0 },
    { 0x1dd2d6da, 23 },
    { 0, 0 },
    { 0x9ee2d6dc, 33 },
    { 0x17f2d6dc, 61 },
    { 0x9b12d6dc, 43 },
    { 0xdbd2d6df, 90 },
    { 0x99c2d6df, 90 },
    { 0xb8d2d6e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb472d6e5, 90 },
    { 0, 0 },
    { 0x9032d6e7, 99 },
    { 0x55a2d6e7, 98 },
    { 0xaa32d6e9, 23 },
    { 0x4df2d6ea, 33 },
    { 0x7fb2d6eb, 98 },
    { 0x6e2d6eb, 34 },
    { 0x2732d6e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe502d6f0, 85 },
    { 0x5a22d6f1, 61 },
    { 0, 0 },
    { 0x302d6f3, 33 },
    { 0xb602d6f4, 33 },
    { 0x5812d6f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2402d6f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db2d6fb, 61 },
    { 0xa1f2d6fc, 95 },
    { 0x69f2d6fd, 61 },
    { 0x9be2d6fe, 98 },
    { 0x5a72d6ff, 81 },
    { 0x3822d700, 98 },
    { 0x7c22d6fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae2d704, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e2d709, 76 },
    { 0x11a2d70a, 95 },
    { 0x44f2d70b, 77 },
    { 0xba02d70a, 98 },
    { 0xe2b2d709, 75 },
    { 0xe8e2d70e, 55 },
    { 0x3b92d70f, 33 },
    { 0xbba2d710, 91 },
    { 0xe282d711, 23 },
    { 0xbd22d711, 34 },
    { 0x3b32d713, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a2d719, 65 },
    { 0, 0 },
    { 0x5c02d71b, 52 },
    { 0xd762d71b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd372d720, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe352d724, 90 },
    { 0x2862d725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe302d729, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc762d72d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62d730, 51 },
    { 0, 0 },
    { 0xa3c2d732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a2d736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d2d73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c2d740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2d749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa2d74d, 66 },
    { 0x57e2d74e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b2d751, 33 },
    { 0, 0 },
    { 0x1172d753, 34 },
    { 0xc532d753, 61 },
    { 0x5ee2d754, 95 },
    { 0, 0 },
    { 0x212d757, 23 },
    { 0xf172d758, 76 },
    { 0x8572d759, 23 },
    { 0xf2d2d759, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc2d764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa402d768, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc812d76e, 33 },
    { 0x5952d76f, 98 },
    { 0xa8d2d76f, 34 },
    { 0x7f62d76f, 46 },
    { 0x6e52d76f, 76 },
    { 0x4ed2d773, 76 },
    { 0xa322d774, 99 },
    { 0xd802d775, 61 },
    { 0xe082d776, 61 },
    { 0xc252d777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c62d77a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e2d782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x452d785, 90 },
    { 0x2ae2d785, 61 },
    { 0x212d787, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a2d78b, 98 },
    { 0x41e2d78b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf502d792, 77 },
    { 0xf1b2d792, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2d796, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd572d79b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf452d7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae2d7a6, 100 },
    { 0x6df2d7a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c72d7aa, 90 },
    { 0, 0 },
    { 0x65a2d7ac, 91 },
    { 0xed12d7ad, 90 },
    { 0, 0 },
    { 0x2f92d7af, 61 },
    { 0xef72d7b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6972d7b4, 85 },
    { 0xe322d7b5, 40 },
    { 0, 0 },
    { 0xc922d7b7, 34 },
    { 0xdf72d7b8, 19 },
    { 0x2d42d7b8, 61 },
    { 0x43f2d7ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a2d7c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b2d7cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a2d7d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8532d7da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f2d7e1, 91 },
    { 0x7812d7e2, 95 },
    { 0, 0 },
    { 0xa412d7e4, 61 },
    { 0xdd2d7e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d62d7e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc572d7ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf52d7f1, 90 },
    { 0xee92d7f2, 23 },
    { 0x4f72d7f2, 98 },
    { 0x42c2d7f4, 90 },
    { 0, 0 },
    { 0x76a2d7f6, 76 },
    { 0, 0 },
    { 0x54a2d7f8, 78 },
    { 0x3d92d7f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3462d7fe, 90 },
    { 0xbb42d7ff, 23 },
    { 0, 0 },
    { 0x402d801, 81 },
    { 0, 0 },
    { 0xea2d803, 43 },
    { 0x2aa2d804, 34 },
    { 0x7302d804, 23 },
    { 0xbef2d806, 95 },
    { 0, 0 },
    { 0xa102d808, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2d80b, 46 },
    { 0xe7d2d80b, 91 },
    { 0, 0 },
    { 0xbf2d80e, 88 },
    { 0, 0 },
    { 0xa222d810, 91 },
    { 0xeb62d811, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232d817, 91 },
    { 0x6202d817, 55 },
    { 0x3a92d817, 17 },
    { 0xaa02d81a, 96 },
    { 0xb692d817, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e2d81e, 76 },
    { 0x6b92d81f, 61 },
    { 0x4332d820, 23 },
    { 0x3432d81e, 66 },
    { 0, 0 },
    { 0x63d2d823, 23 },
    { 0xdcb2d823, 52 },
    { 0xbac2d825, 43 },
    { 0xd7f2d826, 33 },
    { 0xc272d825, 95 },
    { 0, 0 },
    { 0x1302d829, 55 },
    { 0x3032d82a, 76 },
    { 0x842d82b, 23 },
    { 0xe452d82b, 61 },
    { 0x4a72d82d, 81 },
    { 0x32e2d82d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e02d834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552d837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f2d83e, 90 },
    { 0x9c02d83f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92d844, 78 },
    { 0, 0 },
    { 0x9302d846, 75 },
    { 0, 0 },
    { 0xee52d848, 55 },
    { 0, 0 },
    { 0x2e32d84a, 23 },
    { 0, 0 },
    { 0x2312d84c, 68 },
    { 0xd252d84d, 93 },
    { 0xf132d84d, 55 },
    { 0xa0d2d84f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc882d853, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc912d858, 33 },
    { 0xa1c2d859, 61 },
    { 0xb002d859, 34 },
    { 0xaad2d85b, 90 },
    { 0xbcf2d85c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc012d85f, 61 },
    { 0, 0 },
    { 0xaac2d861, 95 },
    { 0xb072d862, 61 },
    { 0xf92d861, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af2d868, 95 },
    { 0xba62d868, 87 },
    { 0x1472d86a, 98 },
    { 0x30d2d86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d2d876, 95 },
    { 0, 0 },
    { 0xd2a2d878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6982d87c, 33 },
    { 0xa3b2d87c, 61 },
    { 0, 0 },
    { 0x652d87f, 95 },
    { 0x57f2d87f, 95 },
    { 0, 0 },
    { 0xd192d882, 93 },
    { 0x9a22d882, 76 },
    { 0xd252d884, 78 },
    { 0xf6f2d885, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e2d888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c82d88f, 55 },
    { 0, 0 },
    { 0x1ed2d891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf212d894, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9932d89c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2d8a1, 95 },
    { 0, 0 },
    { 0x9fe2d8a3, 55 },
    { 0x7472d8a4, 61 },
    { 0xfb02d8a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5212d8aa, 61 },
    { 0xeb02d8ab, 90 },
    { 0, 0 },
    { 0xab22d8ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b22d8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f02d8b9, 33 },
    { 0, 0 },
    { 0x1ea2d8bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2d8c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6212d8c6, 61 },
    { 0x3632d8c6, 33 },
    { 0x5e42d8c8, 52 },
    { 0, 0 },
    { 0xc3c2d8ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92d8cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a62d8d8, 98 },
    { 0xec52d8d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb2d8dc, 98 },
    { 0x1fd2d8dc, 33 },
    { 0, 0 },
    { 0x7632d8df, 33 },
    { 0, 0 },
    { 0x5ac2d8e1, 23 },
    { 0x3b2d8e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca32d8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5132d8f2, 76 },
    { 0x3042d8f3, 36 },
    { 0xfa72d8f4, 95 },
    { 0x74d2d8f5, 81 },
    { 0xd1d2d8f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf712d8ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8412d90c, 95 },
    { 0, 0 },
    { 0x2052d90e, 90 },
    { 0x6032d90f, 90 },
    { 0x6ed2d910, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf82d913, 61 },
    { 0x49d2d914, 46 },
    { 0xc612d915, 61 },
    { 0xfdb2d916, 43 },
    { 0x42c2d917, 61 },
    { 0xc3c2d917, 33 },
    { 0xf052d919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab2d91c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec52d91f, 88 },
    { 0x1a32d91f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db2d923, 90 },
    { 0x2442d924, 75 },
    { 0x82f2d925, 17 },
    { 0xc552d925, 37 },
    { 0x2042d927, 23 },
    { 0xf882d927, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2d92b, 95 },
    { 0x262d92c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f72d92f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3922d933, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f2d937, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc2d93c, 90 },
    { 0xe912d93d, 95 },
    { 0x1002d93e, 90 },
    { 0x4ae2d93f, 34 },
    { 0x41e2d93f, 33 },
    { 0x23a2d93f, 76 },
    { 0x1ff2d941, 84 },
    { 0, 0 },
    { 0x4732d944, 46 },
    { 0, 0 },
    { 0xb222d946, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe192d94a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5542d94d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4392d951, 84 },
    { 0xe322d951, 95 },
    { 0x48d2d952, 100 },
    { 0, 0 },
    { 0x43f2d955, 90 },
    { 0xb442d956, 78 },
    { 0xc172d956, 76 },
    { 0x2692d957, 33 },
    { 0x8452d959, 100 },
    { 0xd682d95a, 23 },
    { 0x2e2d95b, 33 },
    { 0x1c2d95c, 33 },
    { 0, 0 },
    { 0x5032d95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa2d96b, 43 },
    { 0x9c42d96c, 61 },
    { 0x7ec2d96d, 23 },
    { 0x7d2d96e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd2d97e, 98 },
    { 0xf842d97f, 66 },
    { 0x8af2d980, 76 },
    { 0x42f2d981, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa412d984, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc52d987, 33 },
    { 0x97e2d987, 78 },
    { 0x16d2d987, 80 },
    { 0x12b2d98a, 95 },
    { 0xc472d98b, 66 },
    { 0x3f82d98b, 61 },
    { 0xfc12d98d, 23 },
    { 0x95e2d98e, 95 },
    { 0, 0 },
    { 0x52e2d990, 76 },
    { 0xfec2d990, 61 },
    { 0, 0 },
    { 0x8942d993, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42d999, 95 },
    { 0, 0 },
    { 0x23d2d99b, 61 },
    { 0, 0 },
    { 0xe062d99d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02d9a1, 85 },
    { 0x20b2d9a2, 33 },
    { 0xa082d9a3, 61 },
    { 0xb002d9a4, 99 },
    { 0xfcc2d9a3, 61 },
    { 0x9842d9a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8942d9a9, 90 },
    { 0x272d9aa, 33 },
    { 0xb562d9a9, 95 },
    { 0xf42d9ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be2d9af, 67 },
    { 0x8892d9b0, 77 },
    { 0x2452d9b1, 85 },
    { 0x4ed2d9b2, 95 },
    { 0x552d9b3, 23 },
    { 0xda72d9b3, 33 },
    { 0, 0 },
    { 0xe0f2d9b6, 85 },
    { 0, 0 },
    { 0xb592d9b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb2d9be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe392d9c8, 99 },
    { 0x74d2d9c9, 34 },
    { 0x9232d9ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7502d9d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e2d9d5, 33 },
    { 0xfa62d9d5, 100 },
    { 0xa772d9d7, 95 },
    { 0x5e02d9d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa022d9dc, 90 },
    { 0xee52d9dd, 87 },
    { 0xf9e2d9dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e2d9e3, 23 },
    { 0, 0 },
    { 0x1672d9e5, 61 },
    { 0, 0 },
    { 0x7572d9e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed2d9ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b2d9ef, 95 },
    { 0x1522d9f0, 98 },
    { 0xe2d9f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f2d9f4, 33 },
    { 0xf492d9f5, 98 },
    { 0x8eb2d9f6, 95 },
    { 0x2112d9f4, 55 },
    { 0x5622d9f8, 43 },
    { 0, 0 },
    { 0x2b2d9fa, 76 },
    { 0x8f22d9fa, 90 },
    { 0xb1c2d9fc, 76 },
    { 0x5e12d9fd, 99 },
    { 0x932d9fe, 61 },
    { 0xc812d9ff, 34 },
    { 0x3532d9fa, 23 },
    { 0x832d9fd, 67 },
    { 0, 0 },
    { 0xe642da03, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a2da09, 33 },
    { 0x1252da0a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d2da18, 68 },
    { 0xdce2da19, 55 },
    { 0xd142da18, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1862da21, 28 },
    { 0x26f2da21, 49 },
    { 0x33c2da23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf352da26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d92da2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a2da2d, 91 },
    { 0xcc42da2e, 98 },
    { 0x4432da2f, 61 },
    { 0, 0 },
    { 0x5da2da31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4892da37, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6442da3d, 67 },
    { 0x952da3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf2da41, 33 },
    { 0xbf52da42, 33 },
    { 0x35b2da41, 98 },
    { 0x45b2da44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a2da47, 98 },
    { 0xfac2da48, 55 },
    { 0xf702da49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6912da4c, 33 },
    { 0, 0 },
    { 0x312da4e, 76 },
    { 0x3632da4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d2da52, 98 },
    { 0, 0 },
    { 0xa0d2da54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c32da5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea2da63, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf62da6a, 61 },
    { 0xbef2da6b, 98 },
    { 0, 0 },
    { 0x3442da6d, 23 },
    { 0xc882da6e, 90 },
    { 0x2fd2da6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702da73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7842da76, 95 },
    { 0x66e2da77, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d12da7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb72da83, 23 },
    { 0xd3b2da84, 90 },
    { 0x4e82da83, 33 },
    { 0xfb82da85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee2da8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc172da8d, 61 },
    { 0xd0a2da8e, 66 },
    { 0xb872da8f, 35 },
    { 0x6e82da90, 43 },
    { 0x5ec2da91, 34 },
    { 0x502da92, 40 },
    { 0x1db2da93, 61 },
    { 0x4df2da90, 61 },
    { 0xef2da91, 93 },
    { 0x6862da8e, 33 },
    { 0x79b2da91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6302daa1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa82daae, 33 },
    { 0, 0 },
    { 0xc9b2dab0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c2dab3, 91 },
    { 0x6252dab3, 23 },
    { 0xa682dab4, 23 },
    { 0x1a12dab6, 23 },
    { 0, 0 },
    { 0xe9d2dab8, 61 },
    { 0x4482dab9, 55 },
    { 0xc2d2dab8, 95 },
    { 0, 0 },
    { 0xee92dabc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x652dabf, 55 },
    { 0x24c2dac0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382dac4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee2daca, 76 },
    { 0x8552dacb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da2dad0, 23 },
    { 0, 0 },
    { 0x8c22dad2, 84 },
    { 0x7e52dad3, 91 },
    { 0xad72dad4, 33 },
    { 0xea02dad4, 76 },
    { 0x582dad5, 98 },
    { 0xf6f2dad7, 95 },
    { 0xa0e2dad8, 90 },
    { 0xbf52dad8, 98 },
    { 0xe0a2dada, 95 },
    { 0xbb12dadb, 47 },
    { 0x4772dadc, 33 },
    { 0, 0 },
    { 0x6eb2dade, 43 },
    { 0x8672dade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d2dae3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe172daee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd482daf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe52daf8, 78 },
    { 0x6fa2daf9, 23 },
    { 0, 0 },
    { 0x7662dafb, 17 },
    { 0x87c2dafc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1332daff, 99 },
    { 0, 0 },
    { 0x4db2db01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552db07, 23 },
    { 0, 0 },
    { 0xb732db09, 88 },
    { 0x7a72db0a, 75 },
    { 0x5dd2db0b, 52 },
    { 0x5af2db0b, 95 },
    { 0, 0 },
    { 0x4532db0e, 39 },
    { 0, 0 },
    { 0xb6a2db10, 66 },
    { 0, 0 },
    { 0xb9d2db12, 66 },
    { 0, 0 },
    { 0x77b2db14, 83 },
    { 0xd3d2db15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6412db18, 90 },
    { 0x4ff2db19, 33 },
    { 0x5632db19, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa602db1d, 65 },
    { 0x3132db1e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa342db22, 43 },
    { 0, 0 },
    { 0xe522db24, 100 },
    { 0x5152db24, 99 },
    { 0xa82db26, 99 },
    { 0xf722db26, 76 },
    { 0, 0 },
    { 0x1f72db29, 98 },
    { 0x3252db2a, 61 },
    { 0x43c2db2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x982db2e, 23 },
    { 0xb802db2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e2db37, 98 },
    { 0x8382db38, 98 },
    { 0, 0 },
    { 0xb202db3a, 55 },
    { 0x9682db3b, 65 },
    { 0x4902db3b, 61 },
    { 0x60d2db3d, 76 },
    { 0, 0 },
    { 0x9cf2db3f, 87 },
    { 0xbcd2db3f, 61 },
    { 0x13e2db41, 24 },
    { 0xcd2db42, 100 },
    { 0x2ba2db42, 33 },
    { 0xf162db44, 23 },
    { 0x3fe2db43, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d2db48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a2db4f, 61 },
    { 0xdd22db50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad52db54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a42db59, 23 },
    { 0x7b72db5a, 33 },
    { 0x2082db59, 43 },
    { 0xd802db5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9532db5f, 95 },
    { 0, 0 },
    { 0x84e2db61, 33 },
    { 0, 0 },
    { 0xd502db63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d2db67, 61 },
    { 0xfb2db68, 61 },
    { 0x5902db67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b22db6d, 95 },
    { 0xfe12db6e, 90 },
    { 0x1ab2db6f, 75 },
    { 0xd02db6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed02db77, 67 },
    { 0xdcb2db78, 98 },
    { 0, 0 },
    { 0x2c92db7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d52db7d, 23 },
    { 0, 0 },
    { 0xf5f2db7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea2db82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2db86, 40 },
    { 0, 0 },
    { 0x4892db88, 91 },
    { 0, 0 },
    { 0xc802db8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3612db8f, 33 },
    { 0xdd12db90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa882db94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c52db99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c62db9c, 77 },
    { 0x4bf2db9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1382dba2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f2dba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad22dba9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc2dbad, 61 },
    { 0xbc12dbad, 90 },
    { 0x8b02dbaf, 33 },
    { 0x6b82dbb0, 66 },
    { 0xb5e2dbb1, 76 },
    { 0, 0 },
    { 0x8d42dbb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1162dbb7, 61 },
    { 0x7272dbb8, 34 },
    { 0x1902dbb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752dbbd, 34 },
    { 0x25f2dbbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e2dbc2, 34 },
    { 0x6cc2dbc3, 23 },
    { 0xa8f2dbc4, 91 },
    { 0xa232dbc5, 84 },
    { 0xb342dbc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a2dbc9, 76 },
    { 0x6da2dbc9, 99 },
    { 0, 0 },
    { 0x1f22dbcc, 43 },
    { 0x5972dbcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf072dbd0, 95 },
    { 0xab02dbd1, 95 },
    { 0xe642dbd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d2dbd5, 90 },
    { 0x23d2dbd6, 61 },
    { 0x67e2dbd7, 98 },
    { 0x32a2dbd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7402dbdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb072dbdf, 34 },
    { 0x1622dbe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3912dbe5, 75 },
    { 0, 0 },
    { 0x12e2dbe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7642dbee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4502dbf1, 98 },
    { 0, 0 },
    { 0xbbe2dbf3, 98 },
    { 0xaa2dbf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f2dbf8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6332dbff, 95 },
    { 0, 0 },
    { 0x4922dc01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db2dc04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72dc07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d2dc0d, 95 },
    { 0x45c2dc0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc2dc16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7502dc21, 61 },
    { 0xfb62dc22, 76 },
    { 0xfe82dc22, 68 },
    { 0x7992dc22, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c52dc2c, 90 },
    { 0x9972dc2d, 76 },
    { 0xa8a2dc2e, 33 },
    { 0xe582dc2f, 76 },
    { 0x3592dc2f, 61 },
    { 0x3a32dc2e, 90 },
    { 0xd0a2dc32, 23 },
    { 0x8602dc32, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2dc37, 90 },
    { 0x43e2dc38, 98 },
    { 0xce22dc37, 90 },
    { 0x43e2dc3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82dc3e, 95 },
    { 0xc3e2dc3f, 87 },
    { 0, 0 },
    { 0x9fa2dc41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c2dc48, 100 },
    { 0x6952dc49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c22dc4d, 100 },
    { 0xa792dc4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda02dc54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9172dc58, 33 },
    { 0xfa62dc59, 43 },
    { 0, 0 },
    { 0x7f22dc5b, 23 },
    { 0, 0 },
    { 0x5512dc5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7872dc6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c2dc6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2dc73, 61 },
    { 0, 0 },
    { 0xf7c2dc75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf082dc80, 33 },
    { 0x74c2dc81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd12dc88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b52dc8b, 95 },
    { 0xd2b2dc8b, 90 },
    { 0xa982dc8d, 33 },
    { 0x8b42dc8e, 55 },
    { 0x9292dc8f, 75 },
    { 0x78e2dc8f, 98 },
    { 0, 0 },
    { 0x46c2dc92, 93 },
    { 0, 0 },
    { 0xe432dc94, 85 },
    { 0x23a2dc95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822dc99, 55 },
    { 0x1fd2dc9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d2dc9e, 67 },
    { 0x56e2dc9e, 95 },
    { 0, 0 },
    { 0xac82dca1, 33 },
    { 0x8e72dca2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb662dca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa2dcab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a2dcae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a2dcb3, 64 },
    { 0xd302dcb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c2dcba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf2dcbe, 81 },
    { 0x68e2dcbf, 95 },
    { 0x3b42dcbf, 33 },
    { 0, 0 },
    { 0x10c2dcc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb2dcca, 33 },
    { 0x1592dccb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2dcce, 33 },
    { 0x9672dcce, 83 },
    { 0x5132dcd0, 93 },
    { 0xb572dcd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b12dcd8, 55 },
    { 0xdd2dcd8, 56 },
    { 0, 0 },
    { 0x85c2dcdb, 42 },
    { 0, 0 },
    { 0xf152dcdd, 98 },
    { 0, 0 },
    { 0xf482dcdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c12dce4, 85 },
    { 0xbcb2dce5, 90 },
    { 0x6f2dce4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca2dceb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2dcf0, 61 },
    { 0x352dcf1, 78 },
    { 0, 0 },
    { 0x72f2dcf3, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f72dcfc, 95 },
    { 0x34d2dcfd, 34 },
    { 0x7b42dcfd, 95 },
    { 0xac2dcfc, 100 },
    { 0xcb2dd00, 61 },
    { 0, 0 },
    { 0xfba2dd02, 43 },
    { 0x9d22dd03, 83 },
    { 0xe1f2dd03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c2dd09, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd612dd0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2dd11, 98 },
    { 0, 0 },
    { 0xc4a2dd13, 33 },
    { 0xf482dd14, 55 },
    { 0x4cd2dd15, 90 },
    { 0xe672dd15, 66 },
    { 0xb542dd17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32dd1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb312dd22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1212dd25, 98 },
    { 0xe992dd26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2392dd2f, 90 },
    { 0x6ed2dd2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa2dd33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6372dd38, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da2dd3f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4782dd42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f22dd45, 33 },
    { 0x9102dd46, 55 },
    { 0xdb52dd46, 91 },
    { 0xef72dd48, 34 },
    { 0, 0 },
    { 0xb162dd4a, 55 },
    { 0x5ec2dd4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac2dd4e, 98 },
    { 0x9252dd4f, 76 },
    { 0x5c62dd4f, 23 },
    { 0x5462dd50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2dd56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd02dd61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3182dd64, 55 },
    { 0x2312dd65, 95 },
    { 0x4112dd66, 100 },
    { 0xf352dd67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2942dd6a, 76 },
    { 0xe212dd6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a52dd75, 5 },
    { 0, 0 },
    { 0x70b2dd77, 46 },
    { 0, 0 },
    { 0x6ca2dd79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e2dd7d, 33 },
    { 0x9d22dd7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d2dd83, 90 },
    { 0, 0 },
    { 0xba72dd85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf342dd8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac2dd8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf592dd92, 88 },
    { 0x16d2dd93, 98 },
    { 0, 0 },
    { 0xdbf2dd95, 23 },
    { 0x3892dd96, 90 },
    { 0x2c32dd97, 95 },
    { 0x5702dd97, 85 },
    { 0xaf62dd98, 55 },
    { 0x4d92dd97, 100 },
    { 0, 0 },
    { 0x97f2dd9c, 64 },
    { 0x3312dd9d, 75 },
    { 0, 0 },
    { 0xcf92dd9f, 76 },
    { 0xe1d2dda0, 80 },
    { 0x5102dda1, 61 },
    { 0xfed2dd9f, 98 },
    { 0xbfb2dd9f, 17 },
    { 0xe752dda4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8662dda7, 52 },
    { 0, 0 },
    { 0xecb2dda9, 90 },
    { 0, 0 },
    { 0x8c22ddab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa082ddaf, 76 },
    { 0x76c2ddaf, 52 },
    { 0, 0 },
    { 0x6272ddb2, 99 },
    { 0x9342ddb2, 76 },
    { 0xd222ddb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe062ddb7, 67 },
    { 0x6c92ddb8, 55 },
    { 0x7892ddb9, 33 },
    { 0x5d72ddba, 55 },
    { 0xe12ddba, 84 },
    { 0xcd62ddba, 95 },
    { 0, 0 },
    { 0xd352ddbe, 90 },
    { 0xd262ddbf, 90 },
    { 0xa1f2ddc0, 33 },
    { 0x5e72ddc1, 100 },
    { 0, 0 },
    { 0xd792ddc3, 98 },
    { 0x4e62ddc4, 43 },
    { 0x75b2ddc5, 46 },
    { 0xefd2ddc3, 91 },
    { 0x97b2ddc3, 61 },
    { 0x3062ddc8, 35 },
    { 0x9b62ddc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6422ddcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c2ddd4, 56 },
    { 0xd4b2ddd5, 98 },
    { 0, 0 },
    { 0x8462ddd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5452dddb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b52dde0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc442ddea, 65 },
    { 0, 0 },
    { 0xd62ddec, 90 },
    { 0, 0 },
    { 0x8672ddee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2ddf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a2ddf8, 98 },
    { 0x5372ddf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a52ddfc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532de01, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f12de04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac92de08, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592de0b, 33 },
    { 0, 0 },
    { 0x82c2de0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b2de11, 61 },
    { 0x7662de12, 55 },
    { 0x63a2de11, 76 },
    { 0x6232de14, 90 },
    { 0x3242de14, 90 },
    { 0x1cf2de15, 98 },
    { 0, 0 },
    { 0x48f2de18, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b62de21, 55 },
    { 0xeac2de21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd2de25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2302de28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c2de31, 90 },
    { 0, 0 },
    { 0xd2a2de33, 95 },
    { 0, 0 },
    { 0x1a02de35, 90 },
    { 0x4632de35, 78 },
    { 0x9032de36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb132de3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b42de45, 46 },
    { 0xa6a2de46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a2de4f, 93 },
    { 0x8eb2de4f, 33 },
    { 0xb912de4f, 76 },
    { 0, 0 },
    { 0xb7d2de53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8522de57, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2de63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552de67, 33 },
    { 0xb852de68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf642de6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4902de70, 98 },
    { 0, 0 },
    { 0xcdc2de72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf82de75, 64 },
    { 0x9072de76, 55 },
    { 0x7dd2de77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe122de7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb2de80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f62de84, 46 },
    { 0, 0 },
    { 0x8112de86, 34 },
    { 0xbce2de87, 43 },
    { 0xb7c2de88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa542de8d, 90 },
    { 0x23a2de8d, 33 },
    { 0x6032de8f, 33 },
    { 0xf562de90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf762de93, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xd682de96, 90 },
    { 0xdd2de97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c2dea2, 33 },
    { 0xc62dea3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e2dea6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8802deaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f2deae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x412deb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b2deb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2792deb8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd432debb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c2debe, 56 },
    { 0xfa02debf, 52 },
    { 0x5562dec0, 23 },
    { 0x1c92dec1, 61 },
    { 0, 0 },
    { 0x9f12dec3, 76 },
    { 0xb0d2dec4, 61 },
    { 0x8dc2dec3, 61 },
    { 0x46e2dec3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb52deca, 90 },
    { 0x4cf2deca, 34 },
    { 0, 0 },
    { 0xab12decd, 90 },
    { 0, 0 },
    { 0x6e42decf, 90 },
    { 0xf542decf, 90 },
    { 0xc402ded0, 34 },
    { 0x3932ded2, 46 },
    { 0x9122decf, 51 },
    { 0xb442ded1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab2ded9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a32dedd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b2dee1, 90 },
    { 0xeb82dee2, 46 },
    { 0xaf72dee2, 66 },
    { 0x2ae2dee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32deeb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc2deee, 67 },
    { 0x11c2deef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb222def7, 68 },
    { 0x51f2def8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac2defb, 61 },
    { 0xac22defc, 33 },
    { 0x2c82defd, 23 },
    { 0x3f92defe, 61 },
    { 0xeb12deff, 90 },
    { 0x5222df00, 23 },
    { 0, 0 },
    { 0xe52df02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982df05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba2df0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf912df13, 75 },
    { 0x7b92df14, 33 },
    { 0xeae2df15, 66 },
    { 0xb222df15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582df1b, 95 },
    { 0, 0 },
    { 0x9f32df1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c2df20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec32df24, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6142df27, 55 },
    { 0x19c2df28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e42df2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8472df2e, 98 },
    { 0x5332df2e, 34 },
    { 0x7f32df2e, 33 },
    { 0, 0 },
    { 0xd622df32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde42df36, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc2df3c, 51 },
    { 0, 0 },
    { 0x5552df3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xef12df41, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb962df46, 81 },
    { 0x31a2df46, 23 },
    { 0, 0 },
    { 0x8522df49, 66 },
    { 0, 0 },
    { 0x38c2df4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc22df50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b32df53, 81 },
    { 0x17c2df54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e2df57, 43 },
    { 0xc1b2df57, 61 },
    { 0x2da2df59, 61 },
    { 0x5f72df5a, 88 },
    { 0xb9d2df59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f2df66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8012df69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc22df6d, 55 },
    { 0xa622df6e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d2df72, 43 },
    { 0xf882df73, 90 },
    { 0x1042df72, 34 },
    { 0, 0 },
    { 0x50f2df76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf2df79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1082df7c, 33 },
    { 0xa822df7d, 95 },
    { 0xc652df7d, 95 },
    { 0x7472df7f, 17 },
    { 0x2f62df7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12df83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e2df86, 98 },
    { 0, 0 },
    { 0x33f2df88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda2df8b, 90 },
    { 0x4cd2df8c, 33 },
    { 0x39c2df8c, 43 },
    { 0xd6b2df8e, 93 },
    { 0, 0 },
    { 0xcc02df90, 68 },
    { 0x1822df91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582df96, 88 },
    { 0xb9d2df96, 98 },
    { 0xfe12df98, 33 },
    { 0x76b2df99, 95 },
    { 0x3322df9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5712df9e, 61 },
    { 0x7082df9f, 90 },
    { 0xcfb2df9f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f22dfa6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa702dfad, 55 },
    { 0xe1c2dfae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d2dfb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7152dfbc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8132dfc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9892dfca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3762dfcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca2dfd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb312dfda, 65 },
    { 0, 0 },
    { 0xa472dfdc, 23 },
    { 0, 0 },
    { 0xb3f2dfde, 95 },
    { 0xa722dfdf, 99 },
    { 0xbf22dfe0, 61 },
    { 0x852dfdf, 90 },
    { 0x4e42dfde, 95 },
    { 0xc8c2dfe0, 95 },
    { 0xd752dfe4, 61 },
    { 0, 0 },
    { 0x4cb2dfe6, 23 },
    { 0xddc2dfe7, 99 },
    { 0xc972dfe8, 61 },
    { 0x9e32dfe7, 46 },
    { 0x3c22dfe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b2dfed, 93 },
    { 0x1872dfed, 66 },
    { 0x6632dfef, 100 },
    { 0xc892dff0, 95 },
    { 0x7e32dff1, 55 },
    { 0x7f92dfef, 95 },
    { 0xe302dff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa152dff6, 33 },
    { 0xf852dff7, 99 },
    { 0x4c82dff7, 98 },
    { 0xc782dff9, 46 },
    { 0x5fe2dffa, 78 },
    { 0x1e92dffb, 90 },
    { 0x1a22dffc, 43 },
    { 0xcee2dffd, 90 },
    { 0xa732dff8, 23 },
    { 0xf492dffb, 61 },
    { 0x6a22dffe, 76 },
    { 0, 0 },
    { 0xaa62e002, 88 },
    { 0x3762e002, 61 },
    { 0xa452e004, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd812e007, 33 },
    { 0x6032e007, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a2e00c, 84 },
    { 0xf302e00d, 98 },
    { 0xa822e00c, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f2e012, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8192e018, 23 },
    { 0, 0 },
    { 0xcc02e01a, 98 },
    { 0x3662e01b, 76 },
    { 0, 0 },
    { 0xdb92e01d, 69 },
    { 0xfbe2e01d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1982e022, 93 },
    { 0, 0 },
    { 0x7602e024, 33 },
    { 0x2552e025, 61 },
    { 0xe2a2e024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5442e029, 77 },
    { 0, 0 },
    { 0xfbb2e02b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2652e02e, 46 },
    { 0xa532e02f, 95 },
    { 0xd752e02f, 90 },
    { 0x96e2e02f, 66 },
    { 0x1972e032, 23 },
    { 0x4cd2e033, 33 },
    { 0xf342e034, 33 },
    { 0, 0 },
    { 0x6662e036, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c32e039, 76 },
    { 0xe62e03a, 91 },
    { 0x9472e03b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b2e03f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a2e042, 68 },
    { 0x1d92e043, 78 },
    { 0xa692e044, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6692e048, 95 },
    { 0xe532e049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1652e04d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa2e055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3672e05b, 61 },
    { 0x9852e05b, 91 },
    { 0, 0 },
    { 0x41b2e05e, 98 },
    { 0x8e02e05e, 33 },
    { 0, 0 },
    { 0x4412e061, 17 },
    { 0, 0 },
    { 0x9522e063, 23 },
    { 0xfd92e064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8532e06e, 91 },
    { 0xe2a2e06f, 55 },
    { 0x2cd2e06e, 90 },
    { 0x8172e06e, 76 },
    { 0xcec2e071, 33 },
    { 0, 0 },
    { 0x3712e074, 76 },
    { 0x302e074, 66 },
    { 0xf9e2e076, 90 },
    { 0xa192e077, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af2e07a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x752e07d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd422e081, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7172e085, 33 },
    { 0x1af2e085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe462e08c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a2e08f, 91 },
    { 0, 0 },
    { 0x2122e091, 46 },
    { 0x91c2e091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c02e096, 93 },
    { 0, 0 },
    { 0xa6c2e098, 95 },
    { 0x292e099, 33 },
    { 0, 0 },
    { 0x5502e09b, 85 },
    { 0x8d82e09c, 33 },
    { 0, 0 },
    { 0xdbd2e09e, 98 },
    { 0x8322e09e, 95 },
    { 0, 0 },
    { 0x3e92e0a1, 65 },
    { 0x46d2e0a2, 91 },
    { 0xd8b2e0a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa092e0a6, 55 },
    { 0xe8e2e0a7, 91 },
    { 0, 0 },
    { 0x72b2e0a9, 33 },
    { 0x77d2e0aa, 61 },
    { 0, 0 },
    { 0xf7c2e0ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a02e0af, 17 },
    { 0x21d2e0b0, 95 },
    { 0x1652e0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1682e0b4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b02e0b7, 95 },
    { 0, 0 },
    { 0xd752e0b9, 98 },
    { 0x7f2e0ba, 64 },
    { 0x2a82e0ba, 91 },
    { 0xd672e0bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe2e0c2, 90 },
    { 0xed32e0c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf552e0c8, 98 },
    { 0, 0 },
    { 0x8b42e0ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e0ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b2e0d1, 78 },
    { 0x7f42e0d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a2e0d6, 33 },
    { 0, 0 },
    { 0x4fe2e0d8, 75 },
    { 0, 0 },
    { 0x9c02e0da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4542e0df, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b2e0e5, 66 },
    { 0, 0 },
    { 0x1022e0e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f2e0ec, 61 },
    { 0xaf12e0ec, 55 },
    { 0, 0 },
    { 0xff82e0ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd2e0f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e0fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9122e0fe, 67 },
    { 0xb662e0ff, 46 },
    { 0x2d52e100, 91 },
    { 0x7ca2e101, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6882e104, 39 },
    { 0x7992e104, 34 },
    { 0x8c72e106, 46 },
    { 0x5902e107, 95 },
    { 0x69e2e106, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e2e10e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5522e114, 98 },
    { 0xbc2e114, 75 },
    { 0, 0 },
    { 0x5f22e117, 55 },
    { 0xd9d2e118, 61 },
    { 0x1212e118, 33 },
    { 0, 0 },
    { 0xb622e11b, 55 },
    { 0, 0 },
    { 0x2fc2e11d, 55 },
    { 0xda22e11d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b2e123, 36 },
    { 0x47e2e124, 81 },
    { 0x7842e125, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6462e12a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3122e135, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9392e13a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf612e13d, 76 },
    { 0, 0 },
    { 0x3c52e13f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc372e144, 66 },
    { 0xa7a2e145, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc722e14b, 75 },
    { 0xcd22e14b, 98 },
    { 0x9dc2e14d, 61 },
    { 0x73e2e14e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22e155, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6972e160, 17 },
    { 0xab32e161, 34 },
    { 0x112e161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe752e16c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2232e172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a2e179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba2e17d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3442e180, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3262e18a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332e18e, 28 },
    { 0xcb22e18f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3962e192, 33 },
    { 0, 0 },
    { 0x9c42e194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8122e198, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe42e19b, 61 },
    { 0, 0 },
    { 0x2a22e19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea32e1a2, 33 },
    { 0, 0 },
    { 0xd4d2e1a4, 81 },
    { 0xba42e1a4, 84 },
    { 0xc3f2e1a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5042e1ae, 55 },
    { 0xcf32e1af, 90 },
    { 0xf522e1b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6682e1b3, 99 },
    { 0xffc2e1b4, 46 },
    { 0x542e1b5, 95 },
    { 0x1b42e1b6, 43 },
    { 0, 0 },
    { 0x38b2e1b8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e1bd, 90 },
    { 0x3af2e1be, 61 },
    { 0xa962e1bf, 100 },
    { 0x7b22e1bf, 61 },
    { 0x1772e1c1, 23 },
    { 0x4c62e1bd, 95 },
    { 0x7a92e1c0, 46 },
    { 0, 0 },
    { 0xa9b2e1c5, 90 },
    { 0xd852e1c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe992e1cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc2e1cf, 43 },
    { 0, 0 },
    { 0x86f2e1d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8832e1d7, 46 },
    { 0, 0 },
    { 0x34e2e1d9, 78 },
    { 0x57f2e1da, 95 },
    { 0, 0 },
    { 0x8f2e1dc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d2e1e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e12e1e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e72e1ea, 95 },
    { 0x8462e1eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d2e1f1, 61 },
    { 0x8922e1f1, 33 },
    { 0, 0 },
    { 0x2762e1f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1012e1f9, 90 },
    { 0, 0 },
    { 0x4532e1fb, 98 },
    { 0, 0 },
    { 0x4b22e1fd, 90 },
    { 0xe632e1fe, 61 },
    { 0x33c2e1ff, 84 },
    { 0x1432e1ff, 93 },
    { 0, 0 },
    { 0xbab2e202, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5052e207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb2e20a, 61 },
    { 0x61e2e20a, 87 },
    { 0x7f82e20c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8152e20f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd592e218, 33 },
    { 0, 0 },
    { 0xf42e21a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2e21f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db2e229, 97 },
    { 0x6f32e229, 67 },
    { 0x6b92e22b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1172e22f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a02e232, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd2e236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82e23d, 23 },
    { 0xbb42e23d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382e243, 90 },
    { 0xa752e244, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e22e249, 52 },
    { 0, 0 },
    { 0x9102e24b, 90 },
    { 0xcb92e24c, 55 },
    { 0x2f32e24c, 88 },
    { 0x9512e24e, 95 },
    { 0xf42e24e, 90 },
    { 0xf732e24f, 33 },
    { 0xe9a2e24f, 43 },
    { 0xed82e252, 77 },
    { 0x8312e253, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd442e25a, 23 },
    { 0x27d2e25b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4572e25e, 90 },
    { 0x5fa2e25f, 75 },
    { 0x5462e25f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c62e264, 46 },
    { 0xb92e265, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3062e269, 23 },
    { 0xa662e26a, 23 },
    { 0x8b2e269, 96 },
    { 0, 0 },
    { 0x3632e26d, 53 },
    { 0x88c2e26e, 23 },
    { 0x5c22e26f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882e273, 99 },
    { 0xe8e2e273, 84 },
    { 0, 0 },
    { 0x5d82e276, 33 },
    { 0xa22e277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7082e27a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b2e280, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e289, 61 },
    { 0x31a2e28a, 90 },
    { 0, 0 },
    { 0xf902e28c, 93 },
    { 0x6182e28d, 95 },
    { 0xb772e28d, 84 },
    { 0x28c2e28f, 70 },
    { 0xde52e290, 91 },
    { 0x1e82e291, 55 },
    { 0xed92e290, 93 },
    { 0x9972e293, 98 },
    { 0x4992e294, 33 },
    { 0x5772e295, 95 },
    { 0xa62e292, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1092e29d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd92e2a5, 61 },
    { 0xd542e2a6, 61 },
    { 0xe3c2e2a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d2e2aa, 71 },
    { 0xcf02e2aa, 90 },
    { 0x4cb2e2aa, 87 },
    { 0xed12e2ad, 100 },
    { 0, 0 },
    { 0x90b2e2af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e92e2b2, 52 },
    { 0x15e2e2b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2092e2b8, 91 },
    { 0xdf32e2b8, 43 },
    { 0x6d2e2ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4582e2bf, 90 },
    { 0, 0 },
    { 0x9222e2c1, 98 },
    { 0, 0 },
    { 0xaf42e2c3, 43 },
    { 0, 0 },
    { 0x8412e2c5, 98 },
    { 0xca92e2c6, 99 },
    { 0, 0 },
    { 0xe422e2c8, 43 },
    { 0x9342e2c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd202e2cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc552e2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d2e2d6, 70 },
    { 0, 0 },
    { 0x9812e2d8, 87 },
    { 0xd92e2d9, 75 },
    { 0x7432e2da, 61 },
    { 0x8e92e2db, 77 },
    { 0x26c2e2d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8032e2df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b2e2e2, 90 },
    { 0x2dd2e2e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c2e2e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc42e2e9, 90 },
    { 0, 0 },
    { 0x7be2e2eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7042e2ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd2e2fa, 61 },
    { 0x9f32e2fb, 61 },
    { 0x452e2fa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a82e2ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9342e304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd492e307, 85 },
    { 0xe752e308, 61 },
    { 0x2272e308, 33 },
    { 0xeaa2e309, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f2e30d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2e313, 95 },
    { 0xd602e313, 95 },
    { 0xc852e314, 95 },
    { 0, 0 },
    { 0xe662e317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3812e31c, 77 },
    { 0xf892e31d, 76 },
    { 0xe492e31d, 25 },
    { 0x9022e31f, 93 },
    { 0xab42e320, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab2e326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa092e32d, 99 },
    { 0, 0 },
    { 0xcde2e32f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d2e332, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc822e336, 95 },
    { 0xb2f2e336, 55 },
    { 0x7542e337, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5922e33c, 34 },
    { 0xa2e2e33d, 61 },
    { 0x1772e33e, 98 },
    { 0, 0 },
    { 0x1022e340, 95 },
    { 0xdb82e341, 93 },
    { 0xf7c2e342, 33 },
    { 0, 0 },
    { 0x7ef2e344, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa12e351, 90 },
    { 0x3c92e352, 33 },
    { 0x2382e353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b2e356, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc922e359, 95 },
    { 0x7232e35a, 99 },
    { 0x96a2e35b, 66 },
    { 0xf132e35a, 98 },
    { 0, 0 },
    { 0xc492e35e, 90 },
    { 0xaf32e35e, 34 },
    { 0, 0 },
    { 0xcf02e361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a2e365, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42e368, 100 },
    { 0x8772e368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a2e36c, 76 },
    { 0x72e2e36d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea72e371, 95 },
    { 0xd4a2e371, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe392e377, 23 },
    { 0xd1c2e378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f2e37b, 55 },
    { 0, 0 },
    { 0x8972e37d, 93 },
    { 0x95f2e37e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8032e382, 59 },
    { 0x4fb2e382, 23 },
    { 0, 0 },
    { 0x44f2e385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2102e388, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e2e392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec2e396, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef2e39b, 90 },
    { 0xe5d2e39c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb622e39f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8322e3a4, 76 },
    { 0x2112e3a5, 24 },
    { 0xf6c2e3a5, 55 },
    { 0x9e32e3a7, 75 },
    { 0, 0 },
    { 0x6992e3a9, 90 },
    { 0x85d2e3aa, 33 },
    { 0xcb22e3aa, 23 },
    { 0xf502e3ab, 55 },
    { 0x8402e3ad, 33 },
    { 0xad72e3ac, 99 },
    { 0x7362e3af, 55 },
    { 0x3692e3ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf192e3bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4002e3c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba2e3c5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf232e3c8, 95 },
    { 0, 0 },
    { 0xd2b2e3ca, 34 },
    { 0xb452e3ca, 90 },
    { 0x1572e3cc, 98 },
    { 0xde02e3cd, 23 },
    { 0x47d2e3cc, 61 },
    { 0xbb02e3cf, 23 },
    { 0x2c2e3d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb2e3d3, 33 },
    { 0x49d2e3d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb252e3de, 95 },
    { 0x55e2e3df, 91 },
    { 0x9202e3e0, 95 },
    { 0xf942e3df, 52 },
    { 0x7922e3df, 33 },
    { 0, 0 },
    { 0x6512e3e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f2e3f0, 81 },
    { 0x9282e3f1, 33 },
    { 0, 0 },
    { 0x4522e3f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f2e3f9, 91 },
    { 0xc982e3f9, 66 },
    { 0xe132e3fb, 95 },
    { 0, 0 },
    { 0x7542e3fd, 90 },
    { 0x3f32e3fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292e402, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4522e405, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4012e408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42e40f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6722e414, 76 },
    { 0, 0 },
    { 0x7cc2e416, 98 },
    { 0x9182e416, 75 },
    { 0xb182e418, 53 },
    { 0, 0 },
    { 0x26c2e41a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8972e41d, 83 },
    { 0xeeb2e41e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62e423, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb172e426, 33 },
    { 0x1882e426, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd42e42a, 33 },
    { 0x9c12e42b, 90 },
    { 0x4782e42c, 33 },
    { 0x53f2e42d, 78 },
    { 0x69f2e42e, 76 },
    { 0x5482e42a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e433, 66 },
    { 0xa902e433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2e437, 93 },
    { 0, 0 },
    { 0xfd52e439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6342e443, 95 },
    { 0xb762e443, 66 },
    { 0xd8b2e445, 98 },
    { 0xa412e444, 33 },
    { 0xc122e446, 34 },
    { 0x5c02e443, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7222e44b, 46 },
    { 0xba82e44c, 75 },
    { 0xb682e44c, 34 },
    { 0xd2a2e44e, 90 },
    { 0xe2f2e44f, 90 },
    { 0x2132e44f, 33 },
    { 0xdb72e450, 85 },
    { 0x2212e452, 23 },
    { 0xe1e2e452, 23 },
    { 0xcaf2e44e, 23 },
    { 0x9a2e453, 95 },
    { 0x2912e451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e461, 23 },
    { 0x5372e462, 90 },
    { 0xcfd2e463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b12e466, 76 },
    { 0x37d2e466, 61 },
    { 0x52a2e468, 46 },
    { 0, 0 },
    { 0x5732e46a, 95 },
    { 0, 0 },
    { 0xec32e46c, 91 },
    { 0xab82e46c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3182e470, 98 },
    { 0, 0 },
    { 0x70b2e472, 61 },
    { 0, 0 },
    { 0x2ed2e474, 34 },
    { 0xf532e474, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb192e478, 98 },
    { 0xdf92e479, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd72e480, 98 },
    { 0xf972e481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6502e488, 95 },
    { 0x1222e488, 95 },
    { 0x8bb2e48a, 87 },
    { 0x45c2e48b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf632e493, 90 },
    { 0, 0 },
    { 0xbe52e495, 61 },
    { 0, 0 },
    { 0xaf62e497, 98 },
    { 0, 0 },
    { 0x6e12e499, 53 },
    { 0, 0 },
    { 0x15e2e49b, 96 },
    { 0x53f2e49b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee72e4a0, 55 },
    { 0, 0 },
    { 0xba52e4a2, 98 },
    { 0, 0 },
    { 0xdfa2e4a4, 61 },
    { 0x9322e4a4, 90 },
    { 0, 0 },
    { 0x9e12e4a7, 98 },
    { 0x41d2e4a8, 98 },
    { 0xab62e4a9, 33 },
    { 0xf6d2e4aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c12e4ae, 91 },
    { 0, 0 },
    { 0x4b92e4b0, 95 },
    { 0x2982e4b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf482e4b6, 33 },
    { 0x9f02e4b7, 76 },
    { 0, 0 },
    { 0x8a32e4b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9642e4bf, 98 },
    { 0, 0 },
    { 0x9172e4c1, 23 },
    { 0x91d2e4c2, 90 },
    { 0x2a52e4c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a72e4c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d2e4ce, 99 },
    { 0, 0 },
    { 0x2242e4d0, 46 },
    { 0x93f2e4d0, 33 },
    { 0x3a72e4d0, 33 },
    { 0x2fc2e4d0, 95 },
    { 0xc3d2e4d2, 66 },
    { 0x1aa2e4d1, 95 },
    { 0xf322e4d6, 55 },
    { 0x4fa2e4d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272e4dc, 90 },
    { 0, 0 },
    { 0xd2c2e4de, 78 },
    { 0xb7c2e4df, 98 },
    { 0x27f2e4e0, 28 },
    { 0xc512e4e0, 61 },
    { 0xb7e2e4e0, 87 },
    { 0x4eb2e4e2, 98 },
    { 0x442e4e4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xf172e4e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb2e4ea, 61 },
    { 0x32e2e4eb, 37 },
    { 0xb932e4ea, 34 },
    { 0xa172e4ec, 68 },
    { 0, 0 },
    { 0xde52e4ef, 78 },
    { 0x63f2e4f0, 76 },
    { 0xa9d2e4f1, 76 },
    { 0x5252e4f2, 98 },
    { 0xec62e4f3, 93 },
    { 0x7c82e4f2, 33 },
    { 0x3432e4f4, 76 },
    { 0xfb82e4f6, 34 },
    { 0x6c92e4f6, 33 },
    { 0xc72e4f6, 33 },
    { 0x2b12e4f9, 61 },
    { 0x14d2e4f9, 100 },
    { 0x11a2e4ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d2e4fe, 43 },
    { 0, 0 },
    { 0x5d92e500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2062e503, 85 },
    { 0x7112e503, 95 },
    { 0, 0 },
    { 0x65a2e506, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad82e50e, 90 },
    { 0, 0 },
    { 0xeb02e510, 33 },
    { 0, 0 },
    { 0x1552e512, 43 },
    { 0, 0 },
    { 0x5982e514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1432e51b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd2e51e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce22e522, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92e52b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf2e52f, 90 },
    { 0, 0 },
    { 0x8f22e531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a2e535, 33 },
    { 0xc7b2e536, 61 },
    { 0xbc82e536, 90 },
    { 0xff22e537, 55 },
    { 0xc6e2e539, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1262e53d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f02e549, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa392e54d, 55 },
    { 0x9712e54d, 67 },
    { 0xf3d2e54f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a2e556, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b2e559, 67 },
    { 0x9392e55a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b32e55e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f2e564, 17 },
    { 0x8602e565, 87 },
    { 0xfa12e566, 69 },
    { 0x3752e564, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x5332e56a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd2e56d, 90 },
    { 0, 0 },
    { 0xc572e56f, 90 },
    { 0x2cc2e570, 33 },
    { 0, 0 },
    { 0x4562e572, 91 },
    { 0x2e42e573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc882e576, 99 },
    { 0x6db2e576, 55 },
    { 0x4f32e578, 99 },
    { 0xb642e579, 93 },
    { 0xf462e579, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f2e585, 33 },
    { 0, 0 },
    { 0xf1c2e587, 33 },
    { 0xbc82e588, 84 },
    { 0xd8e2e589, 23 },
    { 0, 0 },
    { 0xd602e58b, 93 },
    { 0x5a92e58c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b52e58f, 61 },
    { 0, 0 },
    { 0x2912e591, 76 },
    { 0x1fc2e592, 88 },
    { 0, 0 },
    { 0xaa62e594, 61 },
    { 0x86d2e594, 34 },
    { 0x74d2e595, 90 },
    { 0, 0 },
    { 0x6c02e598, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e22e5a1, 76 },
    { 0xafd2e5a2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad2e5a5, 33 },
    { 0, 0 },
    { 0x8042e5a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d2e5ab, 66 },
    { 0x2fc2e5ac, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb02e5b2, 90 },
    { 0x2da2e5b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92e5b8, 91 },
    { 0x33b2e5b9, 87 },
    { 0x7ff2e5b8, 100 },
    { 0xb992e5bb, 85 },
    { 0xd8e2e5bc, 66 },
    { 0xfd12e5bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4912e5c3, 98 },
    { 0, 0 },
    { 0xa842e5c5, 76 },
    { 0xf2b2e5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed02e5ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd2e5cf, 61 },
    { 0, 0 },
    { 0x97b2e5d1, 95 },
    { 0xcca2e5d2, 67 },
    { 0, 0 },
    { 0x2a62e5d4, 66 },
    { 0, 0 },
    { 0x4f32e5d6, 33 },
    { 0x2222e5d7, 90 },
    { 0, 0 },
    { 0x2bb2e5d9, 76 },
    { 0x88c2e5da, 98 },
    { 0x6232e5d9, 99 },
    { 0xd5c2e5dc, 90 },
    { 0xdb62e5dd, 90 },
    { 0xb112e5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82e5e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e5e7, 93 },
    { 0x1e42e5e8, 55 },
    { 0xa312e5e8, 61 },
    { 0, 0 },
    { 0x9fc2e5eb, 91 },
    { 0xfa62e5ec, 34 },
    { 0, 0 },
    { 0xae72e5ee, 67 },
    { 0xb752e5ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92e5f4, 85 },
    { 0xfaf2e5f5, 52 },
    { 0x5412e5f5, 67 },
    { 0xd292e5f6, 33 },
    { 0xa5f2e5f8, 33 },
    { 0xf712e5f9, 95 },
    { 0xf52e5f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc52e5fe, 95 },
    { 0xf932e5ff, 61 },
    { 0x7302e600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa122e60a, 90 },
    { 0x9d42e60b, 95 },
    { 0x4732e60c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb02e60f, 98 },
    { 0x86e2e610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a92e615, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42e61a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c22e61d, 55 },
    { 0xa032e61d, 98 },
    { 0x2322e61d, 52 },
    { 0xe502e620, 33 },
    { 0xce82e621, 23 },
    { 0xee32e622, 33 },
    { 0xa812e623, 23 },
    { 0xa512e623, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1002e627, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3732e62c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b2e630, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9792e634, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9222e637, 40 },
    { 0, 0 },
    { 0x2692e639, 34 },
    { 0xb72e63a, 34 },
    { 0, 0 },
    { 0x3ca2e63c, 95 },
    { 0x9b42e63d, 90 },
    { 0x2622e63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f2e641, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2282e645, 67 },
    { 0xf032e646, 81 },
    { 0x45e2e645, 76 },
    { 0x1022e647, 43 },
    { 0x2b72e647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf912e64c, 80 },
    { 0xf012e64d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe632e650, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5362e653, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c2e65a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a2e65e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e42e662, 90 },
    { 0x98f2e663, 23 },
    { 0x5882e663, 61 },
    { 0xa202e665, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8472e669, 17 },
    { 0x1a92e66a, 33 },
    { 0xe6b2e66b, 46 },
    { 0xb162e66b, 67 },
    { 0, 0 },
    { 0x7c62e66e, 98 },
    { 0x7132e66f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2e674, 98 },
    { 0x8782e675, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c2e678, 43 },
    { 0xff32e679, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7102e67c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff2e680, 93 },
    { 0x74a2e681, 85 },
    { 0x97b2e682, 91 },
    { 0x5bc2e681, 61 },
    { 0x1e52e683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa922e687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e2e68b, 81 },
    { 0x60c2e68b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c2e691, 34 },
    { 0x3ed2e691, 46 },
    { 0x31f2e693, 61 },
    { 0xbb72e694, 33 },
    { 0x39a2e695, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd302e6a0, 61 },
    { 0x372e6a0, 90 },
    { 0x6a22e6a0, 90 },
    { 0, 0 },
    { 0x5632e6a4, 78 },
    { 0xc092e6a5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9572e6aa, 81 },
    { 0xdc62e6ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8412e6b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f72e6bb, 52 },
    { 0x8542e6bc, 90 },
    { 0x6502e6bd, 56 },
    { 0x9d42e6be, 33 },
    { 0xfed2e6be, 78 },
    { 0x82e2e6bd, 93 },
    { 0xf602e6be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b2e6c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e32e6c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2e6cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1962e6d0, 78 },
    { 0, 0 },
    { 0x18e2e6d2, 99 },
    { 0, 0 },
    { 0xb272e6d4, 98 },
    { 0xe9a2e6d4, 98 },
    { 0, 0 },
    { 0x7272e6d7, 95 },
    { 0, 0 },
    { 0xa7c2e6d9, 95 },
    { 0x7232e6da, 90 },
    { 0, 0 },
    { 0x2392e6dc, 34 },
    { 0x2d42e6dd, 100 },
    { 0, 0 },
    { 0xd8c2e6df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f2e6e6, 67 },
    { 0xbd62e6e7, 76 },
    { 0xe5a2e6e8, 23 },
    { 0x3d2e6e7, 23 },
    { 0x3392e6e7, 76 },
    { 0x7512e6e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd82e6ee, 90 },
    { 0, 0 },
    { 0xc9b2e6f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e2e6f3, 81 },
    { 0xc912e6f3, 75 },
    { 0x2192e6f5, 55 },
    { 0xd9b2e6f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c32e6ff, 95 },
    { 0x4b2e700, 33 },
    { 0xf142e701, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2762e704, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c2e708, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b2e70b, 78 },
    { 0, 0 },
    { 0x8dd2e70d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9602e712, 99 },
    { 0x6bf2e713, 34 },
    { 0x9a52e714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa242e718, 90 },
    { 0xf2e719, 98 },
    { 0xd7e2e719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d2e71d, 68 },
    { 0xc122e71e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3582e726, 61 },
    { 0, 0 },
    { 0x8802e728, 55 },
    { 0xd8c2e729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292e72d, 93 },
    { 0xa832e72e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a52e732, 98 },
    { 0x3bf2e733, 55 },
    { 0x4e82e732, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc372e73a, 34 },
    { 0, 0 },
    { 0x4992e73c, 87 },
    { 0x7a12e73d, 84 },
    { 0x9732e73d, 87 },
    { 0x3fd2e73f, 77 },
    { 0, 0 },
    { 0x83e2e741, 67 },
    { 0xe342e742, 98 },
    { 0, 0 },
    { 0xdd32e744, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5462e747, 61 },
    { 0xdb92e747, 76 },
    { 0, 0 },
    { 0x29c2e74a, 48 },
    { 0x2092e74a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c2e74e, 43 },
    { 0x8442e74e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe622e753, 36 },
    { 0xc242e754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c42e758, 34 },
    { 0xc342e759, 33 },
    { 0xfdf2e758, 98 },
    { 0, 0 },
    { 0xe542e75c, 83 },
    { 0x2562e75d, 55 },
    { 0, 0 },
    { 0xe1f2e75f, 82 },
    { 0xece2e760, 61 },
    { 0x28c2e760, 8 },
    { 0x87e2e760, 46 },
    { 0x5cf2e763, 23 },
    { 0x5202e764, 43 },
    { 0xd152e762, 33 },
    { 0xae52e763, 90 },
    { 0x7c42e767, 85 },
    { 0x5632e768, 17 },
    { 0xe6a2e769, 95 },
    { 0x6782e76a, 42 },
    { 0x97c2e76a, 81 },
    { 0x3e12e75f, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a72e776, 98 },
    { 0xb4e2e776, 61 },
    { 0x7082e778, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e2e77c, 23 },
    { 0x792e77d, 34 },
    { 0, 0 },
    { 0xd262e77f, 76 },
    { 0x4a62e77f, 90 },
    { 0x62e781, 95 },
    { 0xb7d2e782, 53 },
    { 0x9d2e783, 99 },
    { 0, 0 },
    { 0xf4e2e785, 87 },
    { 0x252e786, 33 },
    { 0x2072e787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1002e78a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be2e78d, 51 },
    { 0x5e92e78e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c72e792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa772e796, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e79d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa2e7a3, 98 },
    { 0x6112e7a4, 23 },
    { 0x17a2e7a5, 95 },
    { 0x8cd2e7a5, 33 },
    { 0x7732e7a7, 52 },
    { 0xeac2e7a8, 93 },
    { 0, 0 },
    { 0x4702e7aa, 95 },
    { 0xfe72e7ab, 77 },
    { 0x4692e7aa, 66 },
    { 0x6bd2e7ad, 68 },
    { 0, 0 },
    { 0xc0f2e7af, 95 },
    { 0x7352e7b0, 33 },
    { 0, 0 },
    { 0x462e7b2, 61 },
    { 0xe572e7b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7182e7b6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b22e7b9, 93 },
    { 0x7512e7b9, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xe962e7bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2402e7c0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e7c3, 98 },
    { 0x5552e7c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b2e7ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7742e7cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda2e7d1, 90 },
    { 0xc032e7d2, 52 },
    { 0x6022e7d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9662e7d8, 95 },
    { 0xb0e2e7d9, 33 },
    { 0x4602e7d9, 76 },
    { 0x2ca2e7d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4092e7e1, 93 },
    { 0x1bb2e7e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4122e7ea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9132e7f2, 90 },
    { 0x3f2e7f2, 95 },
    { 0x16b2e7f4, 90 },
    { 0, 0 },
    { 0x21b2e7f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2e7fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a02e7ff, 61 },
    { 0x19c2e800, 61 },
    { 0x4652e801, 91 },
    { 0x5242e801, 34 },
    { 0xd8a2e803, 95 },
    { 0xbc02e804, 66 },
    { 0xf702e804, 52 },
    { 0x69a2e804, 95 },
    { 0xe02e807, 23 },
    { 0, 0 },
    { 0xa1b2e809, 46 },
    { 0x9642e80a, 88 },
    { 0x49e2e80b, 23 },
    { 0xaa72e80b, 98 },
    { 0, 0 },
    { 0xa092e80e, 61 },
    { 0xbd42e80f, 98 },
    { 0xa1f2e80e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7252e81c, 76 },
    { 0, 0 },
    { 0xb972e81e, 33 },
    { 0xe4e2e81f, 47 },
    { 0x5c22e820, 91 },
    { 0xd8f2e820, 43 },
    { 0x9f82e822, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d62e825, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce12e829, 90 },
    { 0x2f42e82a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff2e82d, 23 },
    { 0x7782e82e, 66 },
    { 0, 0 },
    { 0x9a32e830, 52 },
    { 0, 0 },
    { 0xab2e832, 84 },
    { 0, 0 },
    { 0x28f2e834, 34 },
    { 0, 0 },
    { 0x1f72e836, 87 },
    { 0x1c52e837, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f2e83f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1992e851, 91 },
    { 0x4e12e851, 93 },
    { 0x1fc2e852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd2e85d, 33 },
    { 0, 0 },
    { 0xd982e85f, 23 },
    { 0xbde2e85f, 93 },
    { 0x90a2e860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4672e865, 61 },
    { 0xf2d2e866, 61 },
    { 0x5e02e867, 98 },
    { 0x5b92e868, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f22e86b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d12e871, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f52e875, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92e878, 91 },
    { 0xcfe2e879, 95 },
    { 0xdc22e87a, 23 },
    { 0, 0 },
    { 0x6212e87c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c42e883, 55 },
    { 0, 0 },
    { 0x57a2e885, 65 },
    { 0x6e72e886, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b2e889, 90 },
    { 0, 0 },
    { 0x5c2e88b, 46 },
    { 0xca92e88c, 95 },
    { 0xeb52e88d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32e890, 80 },
    { 0x37c2e891, 90 },
    { 0xd7b2e890, 46 },
    { 0xa482e890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda2e896, 90 },
    { 0x52c2e897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf22e89c, 66 },
    { 0, 0 },
    { 0x6be2e89e, 55 },
    { 0x7512e89e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c2e8aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f2e8b1, 93 },
    { 0xb4b2e8b2, 55 },
    { 0x5b42e8b2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf72e8b9, 51 },
    { 0x1982e8ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed2e8bd, 90 },
    { 0x7dd2e8be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c12e8c2, 52 },
    { 0x9792e8c3, 85 },
    { 0x22a2e8c2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b32e8ca, 85 },
    { 0xadc2e8cb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b2e8d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd2e8dd, 40 },
    { 0xf792e8de, 98 },
    { 0x3a32e8df, 33 },
    { 0xb922e8df, 33 },
    { 0, 0 },
    { 0xebc2e8e2, 43 },
    { 0xc482e8e3, 90 },
    { 0x1522e8e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2e8ed, 61 },
    { 0xe992e8ed, 93 },
    { 0, 0 },
    { 0x1b12e8f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf122e8f4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d2e8f8, 33 },
    { 0, 0 },
    { 0x1932e8fa, 55 },
    { 0xdd92e8fb, 23 },
    { 0xe2b2e8fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a22e900, 23 },
    { 0, 0 },
    { 0x22d2e902, 33 },
    { 0xdab2e902, 61 },
    { 0x2332e904, 23 },
    { 0xa842e905, 95 },
    { 0x7b42e906, 23 },
    { 0x4892e904, 90 },
    { 0, 0 },
    { 0x9a52e909, 43 },
    { 0, 0 },
    { 0xc802e90b, 91 },
    { 0x98d2e90c, 90 },
    { 0xf212e90b, 100 },
    { 0, 0 },
    { 0x7472e90f, 98 },
    { 0x4ba2e910, 98 },
    { 0xcda2e911, 85 },
    { 0xbfe2e912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92e915, 77 },
    { 0, 0 },
    { 0xf9b2e917, 66 },
    { 0x42d2e917, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d2e924, 100 },
    { 0x2e925, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2e92a, 33 },
    { 0, 0 },
    { 0xe2d2e92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b2e930, 90 },
    { 0x60c2e931, 76 },
    { 0x1422e930, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1652e935, 55 },
    { 0x2c32e936, 55 },
    { 0, 0 },
    { 0x49e2e938, 90 },
    { 0xa532e939, 76 },
    { 0, 0 },
    { 0xeaf2e93b, 78 },
    { 0xc02e93c, 23 },
    { 0x2dd2e93d, 52 },
    { 0x72a2e93e, 23 },
    { 0x4b92e93e, 55 },
    { 0xe932e940, 46 },
    { 0xcaf2e941, 35 },
    { 0, 0 },
    { 0xa412e943, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c2e948, 90 },
    { 0x922e949, 33 },
    { 0x7ed2e94a, 33 },
    { 0xf722e949, 95 },
    { 0, 0 },
    { 0xd912e94d, 55 },
    { 0x1d82e94e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8952e951, 87 },
    { 0xfa02e951, 90 },
    { 0x3b2e952, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1982e957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab2e95e, 78 },
    { 0xe3c2e95e, 55 },
    { 0x1252e960, 98 },
    { 0x2fa2e961, 23 },
    { 0x3672e962, 98 },
    { 0x5102e962, 23 },
    { 0, 0 },
    { 0x52e965, 95 },
    { 0xd402e965, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e969, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d72e972, 90 },
    { 0x9cf2e972, 61 },
    { 0x5a82e974, 55 },
    { 0x83a2e975, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x162e978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9542e97d, 98 },
    { 0x2692e97e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd002e983, 33 },
    { 0, 0 },
    { 0xa7f2e985, 95 },
    { 0x6182e985, 23 },
    { 0xd8c2e987, 55 },
    { 0x7732e988, 43 },
    { 0xa6b2e986, 90 },
    { 0x6232e98a, 34 },
    { 0x4d52e98b, 33 },
    { 0x6c22e98c, 66 },
    { 0, 0 },
    { 0x40f2e98e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932e991, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d2e995, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd32e99b, 33 },
    { 0xa12e99c, 67 },
    { 0xf2e99b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf422e9a0, 78 },
    { 0xede2e9a0, 61 },
    { 0, 0 },
    { 0x842e9a3, 61 },
    { 0x5de2e9a4, 84 },
    { 0x91a2e9a5, 43 },
    { 0x6962e9a5, 93 },
    { 0x7d12e9a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c2e9ae, 93 },
    { 0, 0 },
    { 0x99b2e9b0, 61 },
    { 0x32a2e9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e2e9b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7242e9b7, 95 },
    { 0x8532e9b7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2e9bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372e9be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8792e9c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f92e9c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b2e9cf, 76 },
    { 0x7942e9cf, 55 },
    { 0xb122e9d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa932e9d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc2e9dc, 78 },
    { 0x7512e9dd, 46 },
    { 0x5192e9dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef2e9e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b42e9e7, 76 },
    { 0, 0 },
    { 0xa352e9e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2172e9ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc62e9f3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc62e9fc, 90 },
    { 0, 0 },
    { 0x10c2e9fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf02ea02, 90 },
    { 0x3ac2ea03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c2ea0d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc2ea13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b2ea16, 34 },
    { 0xcf02ea17, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5622ea1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9162ea21, 61 },
    { 0x5162ea21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6992ea28, 90 },
    { 0xc982ea28, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d2ea2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6642ea37, 81 },
    { 0x12b2ea37, 95 },
    { 0xb7c2ea39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f12ea42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f2ea45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd32ea48, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d2ea4c, 90 },
    { 0x5d2ea4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f52ea52, 95 },
    { 0x1592ea53, 61 },
    { 0x1c92ea54, 23 },
    { 0x3f52ea53, 99 },
    { 0x4982ea56, 55 },
    { 0, 0 },
    { 0x5902ea58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8462ea5b, 95 },
    { 0x8e62ea5c, 23 },
    { 0x8c32ea5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e2ea60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4352ea68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3542ea6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92ea6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e32ea73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd2ea77, 66 },
    { 0x1b82ea78, 90 },
    { 0x68e2ea79, 46 },
    { 0, 0 },
    { 0x5992ea7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ea7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3422ea81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf22ea89, 33 },
    { 0x5842ea8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f52ea8d, 93 },
    { 0, 0 },
    { 0xd5e2ea8f, 90 },
    { 0xf22ea90, 93 },
    { 0xd8b2ea91, 61 },
    { 0, 0 },
    { 0xdd62ea93, 80 },
    { 0x5dc2ea94, 55 },
    { 0x80c2ea94, 76 },
    { 0xcb52ea96, 98 },
    { 0x5f52ea97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa42eaa7, 61 },
    { 0x9f02eaa8, 91 },
    { 0x8422eaa7, 87 },
    { 0x12eaa7, 88 },
    { 0x6d02eaa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a32eaae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf852eab1, 76 },
    { 0x1d42eab2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe422eab5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad2eaba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc2eac1, 65 },
    { 0x54e2eac2, 67 },
    { 0xaa12eac2, 85 },
    { 0x1ca2eac4, 51 },
    { 0, 0 },
    { 0x3582eac6, 75 },
    { 0xbf72eac6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902eacb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec82eacf, 32 },
    { 0xe4e2ead0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf142ead3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d72ead6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b2eadc, 98 },
    { 0x52c2eadc, 33 },
    { 0x9da2eadd, 98 },
    { 0xa82eadf, 95 },
    { 0, 0 },
    { 0xda52eae1, 61 },
    { 0, 0 },
    { 0xbb42eae3, 87 },
    { 0x1c52eae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4842eae9, 90 },
    { 0x12eaea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a72eaed, 23 },
    { 0x1212eaed, 46 },
    { 0, 0 },
    { 0x5042eaf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b02eaf3, 98 },
    { 0x3e2eaf3, 33 },
    { 0xe702eaf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8962eafc, 69 },
    { 0x7de2eafc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde72eb07, 23 },
    { 0x7d42eb08, 88 },
    { 0x2052eb09, 91 },
    { 0x7042eb0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c2eb0d, 83 },
    { 0x4dd2eb0e, 90 },
    { 0xb572eb0e, 33 },
    { 0xcc72eb10, 23 },
    { 0x8872eb11, 76 },
    { 0x6362eb12, 95 },
    { 0x4142eb11, 85 },
    { 0xdb42eb14, 23 },
    { 0xf422eb12, 90 },
    { 0xfcc2eb16, 23 },
    { 0xe392eb17, 61 },
    { 0x472eb15, 98 },
    { 0xc162eb13, 98 },
    { 0xa782eb18, 33 },
    { 0x682eb10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda2eb1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe2eb27, 90 },
    { 0xc532eb28, 34 },
    { 0x1532eb29, 68 },
    { 0x2f92eb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf032eb2d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1222eb30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de2eb34, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2382eb38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2eb42, 61 },
    { 0x4532eb42, 33 },
    { 0x3312eb44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d62eb47, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a2eb4a, 23 },
    { 0x5ae2eb4b, 98 },
    { 0x9fd2eb4a, 61 },
    { 0xd6e2eb4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2eb51, 56 },
    { 0x4982eb51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe2eb57, 23 },
    { 0xb352eb57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee2eb5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632eb60, 84 },
    { 0xc2c2eb61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c42eb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd432eb74, 98 },
    { 0x2812eb75, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4882eb78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x212eb7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8842eb81, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd712eb89, 87 },
    { 0xb902eb89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb2eb8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a2eb92, 90 },
    { 0, 0 },
    { 0x5db2eb94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec2eb9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5362eb9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc202eba1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a2eba6, 93 },
    { 0x8b92eba7, 98 },
    { 0x19f2eba8, 65 },
    { 0x1382eba8, 78 },
    { 0x8232eba8, 98 },
    { 0x27d2ebaa, 95 },
    { 0, 0 },
    { 0x26b2ebad, 91 },
    { 0x8aa2ebae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe2ebb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb32ebb6, 33 },
    { 0x5742ebb7, 90 },
    { 0xfc82ebb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b2ebc4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb832ebc7, 84 },
    { 0x3322ebc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2ebcc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b2ebd1, 23 },
    { 0, 0 },
    { 0xa482ebd3, 46 },
    { 0x47e2ebd4, 87 },
    { 0x4572ebd5, 23 },
    { 0x7a2ebd4, 23 },
    { 0x3a52ebd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f2ebde, 33 },
    { 0x1e82ebdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb2ebe5, 91 },
    { 0x9ac2ebe6, 90 },
    { 0xb52ebe6, 61 },
    { 0x50d2ebe8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4882ebeb, 67 },
    { 0x53d2ebec, 33 },
    { 0x3882ebec, 55 },
    { 0x8392ebee, 98 },
    { 0xe822ebee, 8 },
    { 0xcc12ebef, 55 },
    { 0xe62ebee, 84 },
    { 0x20e2ebf2, 55 },
    { 0x6352ebf3, 23 },
    { 0x81a2ebf4, 93 },
    { 0x1ac2ebf5, 61 },
    { 0xdfc2ebf4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf2ebf9, 52 },
    { 0x3bd2ebf9, 33 },
    { 0xdf82ebfb, 90 },
    { 0xa732ebfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f2ec01, 98 },
    { 0, 0 },
    { 0x9572ec03, 98 },
    { 0, 0 },
    { 0x4892ec05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f42ec09, 91 },
    { 0x8932ec09, 61 },
    { 0, 0 },
    { 0x8662ec0c, 93 },
    { 0, 0 },
    { 0xad22ec0e, 84 },
    { 0xbfb2ec0f, 96 },
    { 0xfb32ec0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc02ec15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef2ec18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1062ec1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d2ec23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb552ec27, 33 },
    { 0, 0 },
    { 0xb02ec29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d2ec2d, 90 },
    { 0, 0 },
    { 0x43a2ec2f, 46 },
    { 0xa082ec30, 33 },
    { 0x5552ec31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee2ec36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c72ec39, 68 },
    { 0, 0 },
    { 0x85c2ec3b, 34 },
    { 0, 0 },
    { 0x4422ec3d, 68 },
    { 0x512ec3e, 76 },
    { 0xb022ec3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e2ec45, 98 },
    { 0, 0 },
    { 0xf9d2ec47, 33 },
    { 0xa382ec48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d2ec4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b2ec56, 100 },
    { 0, 0 },
    { 0x2c42ec58, 33 },
    { 0x54c2ec58, 98 },
    { 0xe0b2ec5a, 90 },
    { 0xd062ec58, 95 },
    { 0, 0 },
    { 0xa982ec5d, 100 },
    { 0x3f2ec5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe632ec61, 33 },
    { 0xde72ec62, 92 },
    { 0xdf52ec61, 95 },
    { 0xfce2ec64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b2ec67, 51 },
    { 0xb1f2ec67, 95 },
    { 0x5bf2ec69, 23 },
    { 0x7862ec6a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4312ec6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f2ec71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe02ec76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3692ec79, 91 },
    { 0x3902ec79, 4 },
    { 0x8252ec7b, 61 },
    { 0x9cb2ec7c, 33 },
    { 0x6cf2ec7d, 95 },
    { 0xc182ec7e, 85 },
    { 0x8c52ec7f, 61 },
    { 0x1cc2ec7e, 76 },
    { 0x9672ec81, 95 },
    { 0xd312ec7a, 59 },
    { 0, 0 },
    { 0x1eb2ec84, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c92ec88, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf742ec8b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb802ec90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3722ec95, 55 },
    { 0xeb02ec96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc662eca0, 23 },
    { 0x62eca1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d2eca4, 95 },
    { 0xbd12eca4, 33 },
    { 0xa7b2eca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa652ecac, 43 },
    { 0, 0 },
    { 0x75e2ecae, 23 },
    { 0xd802ecaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d82ecb4, 93 },
    { 0xa202ecb5, 95 },
    { 0xb972ecb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3412ecb9, 85 },
    { 0, 0 },
    { 0x8a2ecbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd642ecbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb2ecc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad2ecca, 65 },
    { 0, 0 },
    { 0x7c72eccc, 55 },
    { 0xcba2eccc, 87 },
    { 0xddb2ecce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b2ecd4, 76 },
    { 0, 0 },
    { 0x5112ecd6, 76 },
    { 0, 0 },
    { 0x3bd2ecd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7812ecdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4082ece5, 23 },
    { 0, 0 },
    { 0xa592ece7, 61 },
    { 0xdeb2ece8, 90 },
    { 0x8fc2ece8, 90 },
    { 0x7fe2ece8, 99 },
    { 0xaa2ece9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72ecf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62ecf4, 33 },
    { 0x6b2ecf5, 87 },
    { 0, 0 },
    { 0xb222ecf7, 95 },
    { 0xad32ecf8, 95 },
    { 0, 0 },
    { 0x4de2ecfa, 95 },
    { 0xf092ecfa, 93 },
    { 0, 0 },
    { 0x3bd2ecfd, 34 },
    { 0x7ee2ecfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d82ed02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272ed05, 90 },
    { 0x7d52ed06, 33 },
    { 0, 0 },
    { 0xc622ed08, 78 },
    { 0x7172ed09, 43 },
    { 0xfe22ed0a, 33 },
    { 0xcad2ed0b, 95 },
    { 0x4652ed0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7882ed10, 43 },
    { 0xda82ed11, 87 },
    { 0x8f52ed12, 75 },
    { 0x98d2ed11, 43 },
    { 0, 0 },
    { 0x8b72ed15, 90 },
    { 0xc572ed16, 90 },
    { 0x1e42ed17, 99 },
    { 0x6092ed16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c02ed1c, 87 },
    { 0, 0 },
    { 0xafb2ed1e, 61 },
    { 0, 0 },
    { 0xe1d2ed20, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ed24, 77 },
    { 0, 0 },
    { 0x532ed26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c2ed30, 100 },
    { 0, 0 },
    { 0x2132ed32, 90 },
    { 0xb5c2ed32, 98 },
    { 0, 0 },
    { 0x4eb2ed35, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc32ed38, 97 },
    { 0x4342ed39, 61 },
    { 0, 0 },
    { 0xf952ed3b, 43 },
    { 0x95b2ed3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc12ed3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead2ed4d, 98 },
    { 0x2632ed4d, 98 },
    { 0x1e92ed4f, 101 },
    { 0xcf2ed50, 19 },
    { 0, 0 },
    { 0x9b72ed52, 95 },
    { 0x3932ed53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c2ed57, 23 },
    { 0, 0 },
    { 0xa9f2ed59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3832ed5c, 85 },
    { 0, 0 },
    { 0xfc22ed5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce2ed62, 33 },
    { 0x9a82ed62, 33 },
    { 0, 0 },
    { 0xa152ed65, 33 },
    { 0xe22ed66, 23 },
    { 0xceb2ed67, 55 },
    { 0xeed2ed68, 60 },
    { 0x6ff2ed69, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b82ed6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2ed73, 98 },
    { 0, 0 },
    { 0x1952ed75, 76 },
    { 0xee82ed75, 61 },
    { 0xfcc2ed77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c2ed7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f2ed83, 98 },
    { 0x7902ed84, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092ed90, 61 },
    { 0xfad2ed90, 34 },
    { 0, 0 },
    { 0x77c2ed93, 93 },
    { 0xb522ed94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f2ed9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x152ed9f, 61 },
    { 0, 0 },
    { 0x2822eda1, 55 },
    { 0, 0 },
    { 0x6752eda3, 33 },
    { 0x5102eda3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2edaa, 33 },
    { 0x7102edaa, 76 },
    { 0x9bd2edaa, 69 },
    { 0xd962edad, 90 },
    { 0x46c2edae, 66 },
    { 0, 0 },
    { 0xa2d2edb0, 95 },
    { 0x16e2edb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02edb7, 65 },
    { 0, 0 },
    { 0x7102edb9, 76 },
    { 0x27b2edb9, 61 },
    { 0x45f2edbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade2edbf, 33 },
    { 0x3822edbf, 33 },
    { 0x6742edc1, 77 },
    { 0x5b92edbf, 52 },
    { 0, 0 },
    { 0xd4d2edc4, 33 },
    { 0xa3c2edc5, 43 },
    { 0xb7e2edc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b2edca, 88 },
    { 0x9d02edcb, 87 },
    { 0, 0 },
    { 0x6402edcd, 20 },
    { 0x402edcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf542edd1, 33 },
    { 0x14f2edd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f82edd7, 46 },
    { 0, 0 },
    { 0xe3b2edd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f62eddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4162ede1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7512ede6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4242edeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32edf1, 61 },
    { 0, 0 },
    { 0x4d82edf3, 76 },
    { 0, 0 },
    { 0xaa2edf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f2edf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa172edfd, 23 },
    { 0x52f2edfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f32ee02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf2ee05, 67 },
    { 0x78a2ee06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b2ee09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02ee0e, 61 },
    { 0x16a2ee0f, 90 },
    { 0x39d2ee10, 34 },
    { 0x5e32ee11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2ee14, 76 },
    { 0x77b2ee15, 76 },
    { 0x382ee16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb452ee19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d52ee1e, 76 },
    { 0xae82ee1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b2ee26, 33 },
    { 0, 0 },
    { 0x8622ee28, 33 },
    { 0, 0 },
    { 0x6652ee2a, 78 },
    { 0, 0 },
    { 0x8042ee2c, 78 },
    { 0x6662ee2d, 61 },
    { 0, 0 },
    { 0x7822ee2f, 90 },
    { 0, 0 },
    { 0xe9a2ee31, 43 },
    { 0, 0 },
    { 0xf762ee33, 33 },
    { 0, 0 },
    { 0xc8c2ee35, 34 },
    { 0x27c2ee36, 87 },
    { 0x79a2ee37, 88 },
    { 0, 0 },
    { 0x4a42ee39, 90 },
    { 0x4092ee39, 90 },
    { 0, 0 },
    { 0x7fd2ee3c, 31 },
    { 0xf002ee3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb22ee40, 95 },
    { 0, 0 },
    { 0xce12ee42, 23 },
    { 0x2e92ee43, 76 },
    { 0xaf82ee43, 87 },
    { 0x1072ee45, 95 },
    { 0xec52ee45, 61 },
    { 0x1e12ee43, 33 },
    { 0xaeb2ee43, 43 },
    { 0x9a72ee49, 90 },
    { 0x6a52ee49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4822ee50, 95 },
    { 0x2f52ee51, 64 },
    { 0, 0 },
    { 0x3022ee53, 85 },
    { 0x5042ee54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d32ee5a, 90 },
    { 0x1292ee5b, 84 },
    { 0xc0e2ee5c, 87 },
    { 0xec72ee5d, 99 },
    { 0, 0 },
    { 0x1792ee5f, 23 },
    { 0x5752ee60, 61 },
    { 0x84f2ee60, 55 },
    { 0x7342ee61, 66 },
    { 0, 0 },
    { 0xa4f2ee64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3472ee68, 23 },
    { 0x2c22ee69, 90 },
    { 0xdfa2ee69, 87 },
    { 0x7f72ee69, 33 },
    { 0x1f62ee69, 75 },
    { 0xd232ee6d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf802ee74, 17 },
    { 0x5d62ee74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd2ee83, 98 },
    { 0, 0 },
    { 0x3ed2ee85, 87 },
    { 0xcc52ee85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6962ee9e, 90 },
    { 0x85e2ee9e, 23 },
    { 0x3de2eea0, 98 },
    { 0x19a2eea1, 33 },
    { 0xbf22eea2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982eea6, 90 },
    { 0, 0 },
    { 0xad92eea8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd282eeab, 34 },
    { 0xced2eeac, 90 },
    { 0x7be2eead, 87 },
    { 0, 0 },
    { 0xd112eeaf, 95 },
    { 0, 0 },
    { 0x6e82eeb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72eebb, 90 },
    { 0, 0 },
    { 0xb612eebd, 90 },
    { 0x6fe2eebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912eec2, 87 },
    { 0xf2f2eec3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb542eecb, 23 },
    { 0x22d2eecc, 61 },
    { 0x1f32eecd, 90 },
    { 0x7602eece, 33 },
    { 0x27b2eece, 33 },
    { 0x26b2eece, 68 },
    { 0xc842eed0, 100 },
    { 0x1622eecc, 90 },
    { 0xd742eed2, 95 },
    { 0xbac2eecd, 93 },
    { 0, 0 },
    { 0xf4a2eed6, 25 },
    { 0, 0 },
    { 0xd9a2eed8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62eee2, 67 },
    { 0xb752eee3, 98 },
    { 0, 0 },
    { 0xb322eee5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa322eeea, 61 },
    { 0x2972eeea, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d02eef2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d52eef6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8482eef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2eefe, 90 },
    { 0, 0 },
    { 0x3f2ef00, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c2ef04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f2ef08, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f2ef0e, 61 },
    { 0x9ac2ef0f, 95 },
    { 0, 0 },
    { 0x2ed2ef11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b2ef18, 76 },
    { 0xa972ef19, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3682ef1f, 91 },
    { 0, 0 },
    { 0xcf72ef21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd92ef2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed2ef30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd72ef33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4772ef36, 23 },
    { 0x1962ef37, 23 },
    { 0x4e22ef37, 94 },
    { 0x4c12ef38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3482ef3c, 81 },
    { 0, 0 },
    { 0xd972ef3e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e42ef42, 17 },
    { 0x9812ef43, 90 },
    { 0x7202ef44, 34 },
    { 0, 0 },
    { 0x2222ef46, 98 },
    { 0x93f2ef47, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf22ef4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab52ef4e, 87 },
    { 0xdb2ef4f, 98 },
    { 0, 0 },
    { 0x4522ef51, 94 },
    { 0x8ce2ef52, 33 },
    { 0, 0 },
    { 0xe332ef54, 100 },
    { 0xe082ef55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a2ef58, 90 },
    { 0x7952ef58, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ef5c, 88 },
    { 0x9882ef5d, 85 },
    { 0x9b02ef5e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc2ef68, 85 },
    { 0xfcd2ef68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3142ef6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc2ef6f, 68 },
    { 0xcea2ef6f, 33 },
    { 0xf7a2ef71, 90 },
    { 0x20f2ef71, 23 },
    { 0, 0 },
    { 0x1ea2ef74, 98 },
    { 0, 0 },
    { 0xa772ef76, 33 },
    { 0, 0 },
    { 0xa342ef78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7312ef7b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa782ef7e, 81 },
    { 0xf2d2ef7f, 98 },
    { 0x4c32ef80, 98 },
    { 0xb872ef80, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d2ef84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b2ef8a, 91 },
    { 0, 0 },
    { 0x4452ef8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e2ef90, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec2ef93, 98 },
    { 0, 0 },
    { 0xc822ef95, 74 },
    { 0, 0 },
    { 0x3aa2ef97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d82ef9d, 77 },
    { 0xbe62ef9e, 61 },
    { 0, 0 },
    { 0xaaa2efa0, 76 },
    { 0, 0 },
    { 0xa202efa2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe732efa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c2efa8, 95 },
    { 0x2322efa9, 33 },
    { 0x5e42efaa, 90 },
    { 0x1a42efa9, 98 },
    { 0x92d2efab, 95 },
    { 0xd8f2efad, 93 },
    { 0, 0 },
    { 0xd782efaf, 88 },
    { 0xe502efb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8842efb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd2efbc, 91 },
    { 0xdd02efbd, 33 },
    { 0x8e22efbe, 98 },
    { 0xf2d2efbf, 23 },
    { 0xe542efbc, 95 },
    { 0xd422efc1, 23 },
    { 0x3c02efbc, 61 },
    { 0x1a22efc3, 61 },
    { 0, 0 },
    { 0x24a2efc5, 33 },
    { 0xf0b2efc5, 61 },
    { 0x2122efc7, 90 },
    { 0x5912efc8, 95 },
    { 0x3262efc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee92efcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd752efd0, 55 },
    { 0x2592efd1, 95 },
    { 0x1652efd2, 78 },
    { 0xbec2efd1, 61 },
    { 0xd042efd4, 61 },
    { 0xc42efd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6152efe0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d82efe3, 33 },
    { 0x4272efe4, 93 },
    { 0x1462efe3, 90 },
    { 0x4fd2efe6, 90 },
    { 0xf2a2efe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf452efea, 98 },
    { 0, 0 },
    { 0x9bf2efec, 55 },
    { 0x31d2efec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62eff5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc2eff9, 95 },
    { 0, 0 },
    { 0x6f42effb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d2f000, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d2f004, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf802f00d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda62f011, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf602f014, 93 },
    { 0, 0 },
    { 0xb482f016, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa082f019, 91 },
    { 0xc5f2f019, 85 },
    { 0x88c2f01a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde12f01e, 88 },
    { 0x2e12f01f, 52 },
    { 0x1b52f020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa032f023, 33 },
    { 0xdef2f024, 52 },
    { 0x9c22f024, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1922f028, 77 },
    { 0x13b2f028, 23 },
    { 0x2f72f02a, 80 },
    { 0xc8c2f02b, 98 },
    { 0x302f02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8622f039, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a2f03e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72f042, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb962f04f, 87 },
    { 0xb9b2f04f, 85 },
    { 0, 0 },
    { 0xa0b2f052, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb2f056, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2f059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1322f05f, 95 },
    { 0x88b2f060, 98 },
    { 0, 0 },
    { 0xd202f062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe2f068, 98 },
    { 0xe742f069, 84 },
    { 0xde42f06a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa22f06d, 66 },
    { 0x2b92f06e, 98 },
    { 0xf0f2f06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a2f07e, 33 },
    { 0x65f2f07f, 80 },
    { 0, 0 },
    { 0xaa72f081, 23 },
    { 0xf152f081, 61 },
    { 0, 0 },
    { 0x6662f084, 61 },
    { 0, 0 },
    { 0xde82f086, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6252f08b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c22f090, 60 },
    { 0x6602f091, 95 },
    { 0, 0 },
    { 0x4172f093, 33 },
    { 0x78a2f094, 87 },
    { 0, 0 },
    { 0xb9c2f096, 95 },
    { 0x9ef2f097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2f0a0, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a2f0a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f12f0ac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea2f0af, 75 },
    { 0xfaf2f0b0, 99 },
    { 0xdfa2f0b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb22f0b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2f0bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9002f0c1, 46 },
    { 0x4a22f0c2, 17 },
    { 0, 0 },
    { 0x8502f0c4, 34 },
    { 0, 0 },
    { 0x6382f0c6, 76 },
    { 0xdca2f0c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc152f0cc, 95 },
    { 0xb922f0cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe2f0d0, 95 },
    { 0xae12f0d1, 33 },
    { 0x6542f0d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb32f0d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e32f0d8, 77 },
    { 0, 0 },
    { 0xb082f0da, 61 },
    { 0x6292f0da, 61 },
    { 0xeb62f0db, 90 },
    { 0xb482f0dd, 23 },
    { 0xe2f2f0de, 76 },
    { 0x8e52f0dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2652f0e3, 95 },
    { 0x4e62f0e3, 77 },
    { 0xd972f0e5, 76 },
    { 0, 0 },
    { 0x19c2f0e7, 61 },
    { 0x34b2f0e8, 55 },
    { 0x2102f0e7, 33 },
    { 0, 0 },
    { 0x21e2f0eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab2f0f0, 46 },
    { 0x4de2f0f1, 23 },
    { 0x5ab2f0f1, 95 },
    { 0x2842f0f1, 37 },
    { 0x8412f0f4, 91 },
    { 0xa632f0f4, 83 },
    { 0, 0 },
    { 0xc002f0f7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf2f0fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae2f0fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb2f101, 61 },
    { 0x23e2f102, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a2f106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c2f10c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc2f110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde12f114, 46 },
    { 0xf452f115, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc92f11b, 75 },
    { 0, 0 },
    { 0x7fc2f11d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7782f124, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872f129, 65 },
    { 0x9f92f129, 77 },
    { 0, 0 },
    { 0xc732f12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9472f12f, 93 },
    { 0xf712f130, 66 },
    { 0x9ad2f131, 95 },
    { 0xa842f132, 95 },
    { 0, 0 },
    { 0xd0e2f134, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf2f13d, 90 },
    { 0x4962f13e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d72f141, 90 },
    { 0x8c02f141, 33 },
    { 0xf7a2f143, 46 },
    { 0xe112f142, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7962f14a, 98 },
    { 0xabc2f14a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2432f150, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed2f154, 61 },
    { 0xd7f2f154, 99 },
    { 0xd592f155, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9142f15e, 61 },
    { 0xf192f15f, 23 },
    { 0x9592f15e, 78 },
    { 0xc2e2f161, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf62f166, 61 },
    { 0x1cb2f167, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9622f16b, 90 },
    { 0, 0 },
    { 0x7682f16d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6202f171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2882f178, 87 },
    { 0, 0 },
    { 0xf2f2f17a, 17 },
    { 0xa4b2f17b, 98 },
    { 0xb0b2f17b, 61 },
    { 0x8082f17b, 61 },
    { 0x4742f17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8282f18e, 78 },
    { 0x5382f18f, 90 },
    { 0x7aa2f18e, 55 },
    { 0, 0 },
    { 0x25a2f192, 90 },
    { 0, 0 },
    { 0xceb2f194, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2562f198, 23 },
    { 0, 0 },
    { 0xcf92f19a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf672f1a1, 90 },
    { 0x4eb2f1a2, 90 },
    { 0x77e2f1a2, 61 },
    { 0, 0 },
    { 0x35e2f1a5, 76 },
    { 0x5b72f1a6, 76 },
    { 0, 0 },
    { 0x5ac2f1a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf312f1ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3402f1b2, 61 },
    { 0x9e22f1b3, 34 },
    { 0x1072f1b4, 91 },
    { 0x5982f1b5, 76 },
    { 0, 0 },
    { 0x1e22f1b7, 33 },
    { 0x8e42f1b7, 61 },
    { 0xe5a2f1b9, 95 },
    { 0xef32f1ba, 52 },
    { 0xd922f1bb, 61 },
    { 0x96a2f1bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x482f1bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2f1c5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce2f1cb, 76 },
    { 0xa7a2f1cb, 33 },
    { 0xde92f1cb, 90 },
    { 0xd302f1cb, 90 },
    { 0xaa52f1cb, 86 },
    { 0x31c2f1cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1722f1d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7192f1d8, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb2f1df, 61 },
    { 0xe4c2f1e0, 46 },
    { 0x7862f1e0, 67 },
    { 0x5022f1e2, 90 },
    { 0x25b2f1e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3632f1e7, 61 },
    { 0x35a2f1e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab2f1eb, 90 },
    { 0, 0 },
    { 0x2c62f1ed, 61 },
    { 0x51a2f1ee, 100 },
    { 0x8e62f1ef, 61 },
    { 0xa052f1f0, 55 },
    { 0x632f1f0, 52 },
    { 0x7752f1f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d72f1f7, 67 },
    { 0, 0 },
    { 0xda2f1f9, 61 },
    { 0, 0 },
    { 0xb412f1fb, 98 },
    { 0, 0 },
    { 0xe6e2f1fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff2f201, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f205, 33 },
    { 0, 0 },
    { 0xf602f207, 33 },
    { 0, 0 },
    { 0x90d2f209, 84 },
    { 0xaae2f209, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa2f211, 34 },
    { 0x41d2f212, 33 },
    { 0xab62f212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f21a, 55 },
    { 0, 0 },
    { 0x5672f21c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x762f21f, 65 },
    { 0x48e2f220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe062f225, 90 },
    { 0x1962f225, 87 },
    { 0xaf32f227, 76 },
    { 0xc2c2f228, 90 },
    { 0x4172f229, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8192f22e, 90 },
    { 0, 0 },
    { 0x4702f230, 90 },
    { 0x43f2f231, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3172f235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac2f238, 99 },
    { 0x9f2f239, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d2f23d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af2f240, 78 },
    { 0x1d22f241, 98 },
    { 0x8742f242, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb22f248, 95 },
    { 0xf5e2f248, 61 },
    { 0x9b22f248, 98 },
    { 0xe0b2f249, 61 },
    { 0x4f52f248, 76 },
    { 0x59d2f24d, 65 },
    { 0xe892f24e, 55 },
    { 0x46c2f24f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3382f252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9322f255, 61 },
    { 0xdc72f256, 98 },
    { 0xf9e2f256, 66 },
    { 0, 0 },
    { 0xc562f259, 43 },
    { 0, 0 },
    { 0x29a2f25b, 90 },
    { 0, 0 },
    { 0xc522f25d, 90 },
    { 0x99f2f25e, 66 },
    { 0xa812f25f, 33 },
    { 0, 0 },
    { 0x6fa2f261, 98 },
    { 0, 0 },
    { 0x80b2f263, 23 },
    { 0x39e2f263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e52f268, 90 },
    { 0xbf42f269, 90 },
    { 0x372f26a, 61 },
    { 0xe8f2f268, 66 },
    { 0xdd62f26a, 33 },
    { 0xf4a2f26d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262f272, 46 },
    { 0x7132f273, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e2f27a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8022f27f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6502f285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe912f289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6802f28d, 52 },
    { 0, 0 },
    { 0x7862f28f, 17 },
    { 0, 0 },
    { 0x9af2f291, 61 },
    { 0xca72f292, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f2f295, 33 },
    { 0x76a2f296, 61 },
    { 0x8582f297, 87 },
    { 0xd032f296, 90 },
    { 0, 0 },
    { 0x3792f29a, 69 },
    { 0xe6e2f29b, 23 },
    { 0x362f29b, 70 },
    { 0x4be2f29c, 33 },
    { 0xe182f29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5362f2a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf02f2af, 33 },
    { 0, 0 },
    { 0xcfd2f2b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92f2b4, 61 },
    { 0, 0 },
    { 0x9582f2b6, 34 },
    { 0, 0 },
    { 0x7372f2b8, 68 },
    { 0x412f2b8, 90 },
    { 0x54a2f2b8, 33 },
    { 0x5152f2b9, 55 },
    { 0x2fe2f2bc, 76 },
    { 0xb4b2f2bd, 99 },
    { 0xe462f2bd, 33 },
    { 0x80a2f2b8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc672f2c2, 76 },
    { 0, 0 },
    { 0x8a52f2c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4492f2cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe42f2d0, 90 },
    { 0xb182f2d1, 33 },
    { 0xbfc2f2d2, 34 },
    { 0, 0 },
    { 0x2162f2d4, 95 },
    { 0x2802f2d5, 46 },
    { 0xcbb2f2d5, 23 },
    { 0xa8f2f2d7, 98 },
    { 0x9fb2f2d5, 33 },
    { 0, 0 },
    { 0xa6e2f2da, 61 },
    { 0xc842f2db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e22f2e2, 98 },
    { 0xcd82f2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9192f2e8, 33 },
    { 0xd6e2f2e9, 91 },
    { 0x4ef2f2e8, 52 },
    { 0x9692f2eb, 65 },
    { 0xebd2f2ec, 53 },
    { 0, 0 },
    { 0xd6e2f2ee, 33 },
    { 0, 0 },
    { 0xba72f2f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3442f2f8, 61 },
    { 0xa6c2f2f8, 76 },
    { 0x58e2f2fa, 76 },
    { 0x8a22f2fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2f2ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d82f303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc72f308, 90 },
    { 0xd2f309, 88 },
    { 0, 0 },
    { 0x3bb2f30b, 61 },
    { 0x7662f30b, 33 },
    { 0x812f30d, 76 },
    { 0xe352f30d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2f311, 99 },
    { 0x3152f312, 90 },
    { 0x12c2f313, 78 },
    { 0xa1b2f313, 98 },
    { 0xb9d2f313, 55 },
    { 0x9b12f314, 61 },
    { 0x7e62f311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde2f31a, 98 },
    { 0, 0 },
    { 0x4782f31c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2f320, 23 },
    { 0x2c22f320, 100 },
    { 0x2732f320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2f326, 95 },
    { 0xfa42f327, 83 },
    { 0x8672f328, 75 },
    { 0xa402f329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e2f32d, 76 },
    { 0xe6f2f32e, 100 },
    { 0x75b2f32d, 90 },
    { 0xda72f32d, 36 },
    { 0xc7f2f331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592f338, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc362f33d, 90 },
    { 0xbb02f33e, 96 },
    { 0x1d2f33e, 95 },
    { 0x23c2f340, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c2f345, 55 },
    { 0x34d2f346, 55 },
    { 0x3b92f347, 23 },
    { 0, 0 },
    { 0xf142f349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d2f34c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf112f350, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e12f355, 61 },
    { 0xea2f356, 33 },
    { 0x74c2f356, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4032f365, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e2f369, 34 },
    { 0xeeb2f369, 33 },
    { 0x33f2f369, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c2f36f, 76 },
    { 0xd1b2f36f, 23 },
    { 0, 0 },
    { 0x1be2f372, 34 },
    { 0, 0 },
    { 0xbef2f374, 46 },
    { 0, 0 },
    { 0x4952f376, 60 },
    { 0x8e62f377, 33 },
    { 0x4382f376, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc852f37b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6772f37e, 33 },
    { 0, 0 },
    { 0x4a02f380, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e52f387, 83 },
    { 0, 0 },
    { 0xb722f389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f52f38c, 90 },
    { 0, 0 },
    { 0x9ee2f38e, 33 },
    { 0x6e02f38f, 90 },
    { 0xff02f38f, 33 },
    { 0xe262f38e, 55 },
    { 0x2722f391, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c82f399, 76 },
    { 0, 0 },
    { 0x7b92f39b, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xf362f39e, 33 },
    { 0xaab2f39f, 95 },
    { 0x4f62f3a0, 90 },
    { 0x9cb2f3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a2f3a9, 90 },
    { 0, 0 },
    { 0x302f3ab, 76 },
    { 0xf12f3ac, 55 },
    { 0x67c2f3ad, 90 },
    { 0x1002f3ad, 66 },
    { 0x54c2f3af, 46 },
    { 0x68a2f3b0, 90 },
    { 0xdb42f3b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa192f3b4, 60 },
    { 0, 0 },
    { 0x5ac2f3b6, 46 },
    { 0x5f32f3b7, 23 },
    { 0x9832f3b8, 49 },
    { 0xff12f3b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2f3bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c92f3c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc922f3c8, 46 },
    { 0x1482f3c9, 55 },
    { 0, 0 },
    { 0xefe2f3cb, 17 },
    { 0, 0 },
    { 0xe312f3cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2f3d4, 91 },
    { 0x13f2f3d4, 61 },
    { 0, 0 },
    { 0x2242f3d7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b2f3da, 80 },
    { 0x9cc2f3da, 76 },
    { 0x19e2f3db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f62f3e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912f3ed, 43 },
    { 0, 0 },
    { 0x79a2f3ef, 91 },
    { 0xae72f3ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b2f3f4, 52 },
    { 0x4d32f3f5, 76 },
    { 0xa12f3f6, 38 },
    { 0, 0 },
    { 0x2b72f3f8, 33 },
    { 0, 0 },
    { 0x5942f3fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f2f3fe, 99 },
    { 0xe132f3ff, 90 },
    { 0x94f2f3fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72f404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f2f408, 61 },
    { 0, 0 },
    { 0xc1d2f40a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f413, 95 },
    { 0x1c82f413, 34 },
    { 0, 0 },
    { 0x8342f416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc2f41a, 96 },
    { 0xf7b2f41b, 33 },
    { 0x8c2f41c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4452f420, 61 },
    { 0x6222f421, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4862f426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d2f429, 84 },
    { 0xee92f42a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa972f42e, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f12f434, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3062f43d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b2f440, 78 },
    { 0, 0 },
    { 0x98c2f442, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa932f447, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b2f44d, 87 },
    { 0, 0 },
    { 0x5cc2f44f, 84 },
    { 0, 0 },
    { 0x2c72f451, 64 },
    { 0, 0 },
    { 0xf402f453, 95 },
    { 0x7d12f454, 55 },
    { 0x40a2f455, 82 },
    { 0x6142f456, 90 },
    { 0xafe2f457, 61 },
    { 0x50f2f458, 23 },
    { 0x55c2f459, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b2f461, 90 },
    { 0x55e2f462, 90 },
    { 0xc092f463, 23 },
    { 0x5a12f463, 81 },
    { 0x2bb2f463, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa82f46d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a2f472, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982f47b, 78 },
    { 0x29e2f47c, 78 },
    { 0, 0 },
    { 0x8352f47e, 84 },
    { 0xc742f47f, 91 },
    { 0xf7a2f47e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6822f487, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc2f48f, 65 },
    { 0x1282f490, 80 },
    { 0xb352f491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d2f498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e2f49c, 52 },
    { 0xb252f49c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d62f4a3, 33 },
    { 0, 0 },
    { 0xc142f4a5, 33 },
    { 0xd132f4a6, 33 },
    { 0x4482f4a7, 66 },
    { 0x7f82f4a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a2f4ac, 95 },
    { 0x27e2f4ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7762f4b1, 95 },
    { 0x4482f4b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a2f4b6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da2f4be, 95 },
    { 0, 0 },
    { 0xef32f4c0, 61 },
    { 0, 0 },
    { 0xcc92f4c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed2f4d2, 77 },
    { 0, 0 },
    { 0xda62f4d4, 61 },
    { 0xa882f4d5, 43 },
    { 0xa362f4d6, 98 },
    { 0xdd02f4d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e72f4df, 43 },
    { 0x8f52f4e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae32f4e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2102f4e9, 23 },
    { 0xe582f4e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd562f4f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd42f4f5, 100 },
    { 0x42e2f4f5, 76 },
    { 0xbc92f4f5, 90 },
    { 0xcdd2f4f8, 95 },
    { 0, 0 },
    { 0xbb22f4fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d2f4fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa002f502, 76 },
    { 0x7f02f503, 84 },
    { 0x7b62f504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb2f509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4182f50f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c62f512, 34 },
    { 0x2df2f513, 76 },
    { 0xa0e2f513, 61 },
    { 0xd812f515, 40 },
    { 0x6972f515, 43 },
    { 0x7f22f516, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f02f51d, 61 },
    { 0x1a82f51e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8782f521, 78 },
    { 0, 0 },
    { 0x84d2f523, 23 },
    { 0xa6f2f523, 33 },
    { 0x4a02f525, 98 },
    { 0, 0 },
    { 0x99e2f527, 98 },
    { 0x9422f528, 87 },
    { 0xa8f2f529, 88 },
    { 0xd232f527, 23 },
    { 0, 0 },
    { 0x4362f52c, 90 },
    { 0x8ec2f52d, 90 },
    { 0xb3f2f52c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd92f534, 90 },
    { 0, 0 },
    { 0xedb2f536, 95 },
    { 0, 0 },
    { 0xdb62f538, 33 },
    { 0, 0 },
    { 0x5e12f53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b2f53d, 61 },
    { 0, 0 },
    { 0xbd52f53f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6792f542, 90 },
    { 0, 0 },
    { 0x84a2f544, 23 },
    { 0x4192f545, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3162f54a, 61 },
    { 0xcc22f54a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2402f54e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e2f552, 90 },
    { 0xb312f553, 46 },
    { 0xf7d2f554, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad82f557, 76 },
    { 0xdcd2f558, 61 },
    { 0, 0 },
    { 0x74c2f55a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a52f55e, 90 },
    { 0xef82f55e, 34 },
    { 0, 0 },
    { 0xd3c2f561, 33 },
    { 0, 0 },
    { 0xd902f563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa2f568, 90 },
    { 0xfd42f569, 93 },
    { 0x7ef2f568, 76 },
    { 0xbbd2f56b, 84 },
    { 0x61e2f56c, 87 },
    { 0x9252f56d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe062f572, 34 },
    { 0, 0 },
    { 0xc4e2f574, 33 },
    { 0x9f42f574, 93 },
    { 0x77a2f574, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9182f57b, 23 },
    { 0xd482f57b, 34 },
    { 0xbd32f57b, 61 },
    { 0x8352f57e, 90 },
    { 0xdc22f57e, 55 },
    { 0, 0 },
    { 0x8152f581, 8 },
    { 0xb9d2f582, 33 },
    { 0xfc52f583, 91 },
    { 0, 0 },
    { 0x2472f585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f2f588, 40 },
    { 0x27d2f588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf692f58e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f597, 33 },
    { 0xa892f598, 98 },
    { 0x3582f599, 33 },
    { 0xb3c2f59a, 23 },
    { 0xf5b2f59a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92f5a0, 61 },
    { 0xc792f5a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6942f5a5, 3 },
    { 0, 0 },
    { 0xa9d2f5a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2f5ad, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd42f5b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b62f5b8, 93 },
    { 0, 0 },
    { 0xc422f5ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef72f5c4, 33 },
    { 0, 0 },
    { 0xfae2f5c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d2f5ca, 76 },
    { 0x6112f5cb, 52 },
    { 0x6212f5ca, 90 },
    { 0, 0 },
    { 0x33e2f5ce, 98 },
    { 0, 0 },
    { 0x5502f5d0, 66 },
    { 0x5e02f5d0, 33 },
    { 0, 0 },
    { 0x3482f5d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a42f5d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc322f5db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d62f5e2, 33 },
    { 0, 0 },
    { 0xf7d2f5e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4482f5eb, 34 },
    { 0, 0 },
    { 0x7d92f5ed, 23 },
    { 0, 0 },
    { 0x5502f5ef, 23 },
    { 0x1072f5f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b2f5f3, 93 },
    { 0xa882f5f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f5fa, 78 },
    { 0x84e2f5fa, 76 },
    { 0x1302f5fc, 90 },
    { 0xae12f5fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2012f600, 99 },
    { 0xd9e2f601, 95 },
    { 0x8c52f602, 93 },
    { 0, 0 },
    { 0xd852f604, 33 },
    { 0x9602f605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4802f60b, 90 },
    { 0, 0 },
    { 0xec12f60d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf2f614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6502f617, 34 },
    { 0, 0 },
    { 0x5752f619, 33 },
    { 0, 0 },
    { 0xb112f61b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e72f61e, 43 },
    { 0, 0 },
    { 0xee02f620, 33 },
    { 0xac82f620, 34 },
    { 0xbbd2f622, 98 },
    { 0, 0 },
    { 0xa0c2f624, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a72f628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc222f62b, 90 },
    { 0xfcd2f62c, 61 },
    { 0x1f32f62d, 66 },
    { 0, 0 },
    { 0xa432f62f, 61 },
    { 0xb982f62f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee32f634, 61 },
    { 0, 0 },
    { 0x14c2f636, 98 },
    { 0xc0f2f637, 61 },
    { 0xa722f637, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a2f63d, 76 },
    { 0, 0 },
    { 0x62c2f63f, 95 },
    { 0x7212f640, 61 },
    { 0, 0 },
    { 0xf942f642, 98 },
    { 0, 0 },
    { 0x2f62f644, 90 },
    { 0, 0 },
    { 0x9c22f646, 65 },
    { 0x2f2f647, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2f64a, 94 },
    { 0x5cb2f64b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe772f64e, 66 },
    { 0x6e82f64f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1382f656, 61 },
    { 0x1eb2f657, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b2f65a, 87 },
    { 0x7b22f65a, 95 },
    { 0, 0 },
    { 0x632f65d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa2f660, 90 },
    { 0xf262f660, 23 },
    { 0xc722f662, 33 },
    { 0, 0 },
    { 0xbf52f664, 84 },
    { 0, 0 },
    { 0x1e12f666, 40 },
    { 0x9972f667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b2f66f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb632f673, 46 },
    { 0x8aa2f673, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb512f67a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b2f67f, 77 },
    { 0xab72f680, 40 },
    { 0x6f42f67f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca32f687, 100 },
    { 0xd432f688, 46 },
    { 0x92e2f689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad2f693, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2f697, 90 },
    { 0xd8c2f698, 23 },
    { 0xe442f699, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec42f6a0, 33 },
    { 0x2002f6a1, 66 },
    { 0x5e2f6a2, 88 },
    { 0xbb02f6a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc82f6b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f2f6b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4702f6b6, 61 },
    { 0x6e52f6b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3462f6ba, 46 },
    { 0x4e42f6ba, 34 },
    { 0x8a32f6bc, 33 },
    { 0xdc82f6ba, 33 },
    { 0x39a2f6be, 90 },
    { 0x7e12f6bf, 99 },
    { 0x9372f6bd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f22f6c9, 66 },
    { 0x12a2f6c9, 61 },
    { 0x2c52f6cb, 33 },
    { 0xcfa2f6c9, 99 },
    { 0, 0 },
    { 0x78d2f6ce, 95 },
    { 0, 0 },
    { 0xdc82f6d0, 95 },
    { 0x9212f6d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x262f6d4, 95 },
    { 0xda42f6d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5592f6db, 85 },
    { 0xb982f6dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd2f6e3, 95 },
    { 0, 0 },
    { 0xffa2f6e5, 5 },
    { 0, 0 },
    { 0xaea2f6e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2f6ee, 90 },
    { 0x7ca2f6ef, 23 },
    { 0x40b2f6f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa792f6f3, 77 },
    { 0x6d32f6f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2f6fc, 98 },
    { 0, 0 },
    { 0xcb12f6fe, 61 },
    { 0, 0 },
    { 0xf4a2f700, 85 },
    { 0xdb32f701, 61 },
    { 0x4a52f702, 93 },
    { 0xc72f701, 61 },
    { 0, 0 },
    { 0xfb2f705, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2532f70a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba2f70d, 88 },
    { 0x4db2f70e, 90 },
    { 0xf312f70f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3272f718, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf2f71d, 90 },
    { 0x272f71e, 90 },
    { 0x852f71f, 80 },
    { 0x8652f71f, 95 },
    { 0xb682f71d, 98 },
    { 0, 0 },
    { 0x6d12f723, 90 },
    { 0xc222f724, 33 },
    { 0xc442f723, 46 },
    { 0x35c2f726, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92f729, 91 },
    { 0, 0 },
    { 0xd622f72b, 90 },
    { 0x2512f72c, 90 },
    { 0xb3d2f72d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b62f730, 81 },
    { 0x6702f731, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b2f73b, 33 },
    { 0x1a72f73c, 23 },
    { 0, 0 },
    { 0x4a62f73e, 33 },
    { 0x35b2f73e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1992f742, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f2f745, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad32f748, 61 },
    { 0xe312f749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb442f750, 55 },
    { 0, 0 },
    { 0x3142f752, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7512f756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c2f75a, 84 },
    { 0xe6a2f75a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c12f760, 61 },
    { 0, 0 },
    { 0x3e12f762, 98 },
    { 0xe762f763, 33 },
    { 0, 0 },
    { 0x5852f765, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1352f768, 61 },
    { 0xa7a2f769, 98 },
    { 0xcd02f769, 61 },
    { 0, 0 },
    { 0xbed2f76c, 33 },
    { 0xe5a2f76d, 43 },
    { 0x27f2f76d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e2f772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb2f777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb2f77c, 90 },
    { 0, 0 },
    { 0x7372f77e, 90 },
    { 0xc0e2f77f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c2f782, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f52f785, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd2f78a, 95 },
    { 0x2d62f78b, 98 },
    { 0xcc72f78c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b2f794, 76 },
    { 0x2a2f795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d2f79a, 68 },
    { 0x4292f79b, 81 },
    { 0, 0 },
    { 0x5642f79d, 99 },
    { 0x72a2f79e, 98 },
    { 0x39d2f79f, 90 },
    { 0xdce2f7a0, 84 },
    { 0x17a2f79d, 55 },
    { 0x80c2f79d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da2f7a7, 95 },
    { 0x50c2f7a7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x6102f7ab, 88 },
    { 0xa402f7ac, 61 },
    { 0xd12f7ac, 93 },
    { 0xebd2f7ae, 23 },
    { 0x6e82f7ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d32f7b3, 61 },
    { 0x2f7b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c2f7bb, 61 },
    { 0xff2f7bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf42f7c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a2f7c5, 23 },
    { 0x3412f7c6, 98 },
    { 0x8412f7c7, 23 },
    { 0x36b2f7c7, 98 },
    { 0x1d52f7c9, 61 },
    { 0xb252f7c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba2f7d0, 95 },
    { 0xf22f7d0, 87 },
    { 0, 0 },
    { 0x602f7d3, 61 },
    { 0x5692f7d4, 100 },
    { 0x7972f7d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb912f7d8, 100 },
    { 0xd402f7d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd52f7de, 33 },
    { 0x8422f7de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb2f7e2, 76 },
    { 0xb1d2f7e2, 34 },
    { 0x5652f7e3, 55 },
    { 0, 0 },
    { 0x4562f7e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f82f7ec, 66 },
    { 0x9fa2f7ed, 96 },
    { 0xd382f7ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9712f7f3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2142f7f8, 98 },
    { 0, 0 },
    { 0xb2a2f7fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1362f7fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b2f801, 61 },
    { 0x3de2f802, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b2f806, 90 },
    { 0x8872f807, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6612f80b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b2f814, 67 },
    { 0, 0 },
    { 0x2142f816, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b2f825, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef62f82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d2f830, 52 },
    { 0x90a2f831, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e92f834, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e2f83a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c22f83f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2232f845, 23 },
    { 0xb6e2f846, 17 },
    { 0x7472f847, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e52f84f, 43 },
    { 0, 0 },
    { 0xf482f851, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff2f85e, 52 },
    { 0, 0 },
    { 0x13e2f860, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f2f866, 90 },
    { 0x1e12f867, 61 },
    { 0xbdd2f867, 34 },
    { 0x1b12f869, 23 },
    { 0x3022f867, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6602f86e, 85 },
    { 0xf022f86f, 52 },
    { 0, 0 },
    { 0xaa92f871, 40 },
    { 0, 0 },
    { 0xf8c2f873, 99 },
    { 0xba72f873, 52 },
    { 0x5522f875, 34 },
    { 0x6fc2f876, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb322f87a, 85 },
    { 0x2d72f87b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac2f87e, 33 },
    { 0x2da2f87f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe2f883, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112f888, 68 },
    { 0, 0 },
    { 0x9382f88a, 95 },
    { 0, 0 },
    { 0x12f88c, 68 },
    { 0, 0 },
    { 0xada2f88e, 61 },
    { 0x7a22f88f, 34 },
    { 0x2402f88e, 23 },
    { 0x2be2f88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d2f895, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc102f898, 34 },
    { 0, 0 },
    { 0x5822f89a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2212f89f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2f8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e2f8a8, 33 },
    { 0, 0 },
    { 0xe372f8aa, 61 },
    { 0xec2f8ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc42f8ae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f2f8b2, 55 },
    { 0x3dd2f8b3, 90 },
    { 0x5012f8b2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf92f8b8, 61 },
    { 0, 0 },
    { 0x6a32f8ba, 67 },
    { 0, 0 },
    { 0xbb2f8bc, 77 },
    { 0x2952f8bd, 98 },
    { 0xe4e2f8be, 43 },
    { 0x8cf2f8bf, 84 },
    { 0xaa32f8bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b52f8c3, 23 },
    { 0x4f52f8c3, 23 },
    { 0, 0 },
    { 0x4842f8c6, 61 },
    { 0x87b2f8c6, 75 },
    { 0x76f2f8c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b2f8cb, 33 },
    { 0, 0 },
    { 0x61b2f8cd, 90 },
    { 0, 0 },
    { 0xd412f8cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a2f8d2, 90 },
    { 0xc1d2f8d3, 90 },
    { 0, 0 },
    { 0xffb2f8d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe582f8d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d82f8df, 90 },
    { 0x63d2f8e0, 23 },
    { 0, 0 },
    { 0xd392f8e2, 52 },
    { 0, 0 },
    { 0x6fb2f8e4, 68 },
    { 0x562f8e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b12f8ed, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b2f8f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1232f8f7, 80 },
    { 0x5d82f8f8, 95 },
    { 0, 0 },
    { 0x4bb2f8fa, 66 },
    { 0x3892f8fb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9492f8fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7762f904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a2f909, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8552f910, 95 },
    { 0x2582f911, 90 },
    { 0, 0 },
    { 0x4e02f913, 61 },
    { 0xd442f914, 93 },
    { 0x4642f913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752f91d, 90 },
    { 0, 0 },
    { 0xa5c2f91f, 95 },
    { 0, 0 },
    { 0x6222f921, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b82f925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7652f92b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7852f931, 67 },
    { 0, 0 },
    { 0xd442f933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4042f93d, 85 },
    { 0, 0 },
    { 0xf3a2f93f, 83 },
    { 0xe752f93f, 46 },
    { 0xbfa2f941, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd2f948, 90 },
    { 0, 0 },
    { 0x42a2f94a, 55 },
    { 0, 0 },
    { 0xa022f94c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba2f94f, 52 },
    { 0xbe82f950, 43 },
    { 0x2802f951, 90 },
    { 0x4692f951, 46 },
    { 0xa82f950, 76 },
    { 0x78d2f954, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf22f958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a92f95b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d2f95e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7962f961, 43 },
    { 0, 0 },
    { 0xff32f963, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e2f966, 95 },
    { 0, 0 },
    { 0xdc82f968, 61 },
    { 0x3e12f969, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf812f96d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a62f972, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d22f978, 77 },
    { 0x5b32f979, 33 },
    { 0xa9e2f97a, 93 },
    { 0x5fe2f97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a02f982, 95 },
    { 0x71c2f982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f82f987, 95 },
    { 0, 0 },
    { 0xa4b2f989, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2f98c, 90 },
    { 0xde02f98d, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c82f990, 33 },
    { 0xee22f990, 98 },
    { 0, 0 },
    { 0x972f993, 93 },
    { 0xa8c2f994, 90 },
    { 0x9682f995, 95 },
    { 0xf402f995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4632f99f, 34 },
    { 0x67b2f9a0, 33 },
    { 0x612f9a1, 90 },
    { 0x68a2f99f, 17 },
    { 0x52b2f9a3, 61 },
    { 0x26b2f9a4, 46 },
    { 0xf3e2f9a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc2f9aa, 87 },
    { 0xf6e2f9ab, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b22f9b4, 87 },
    { 0x7872f9b4, 91 },
    { 0, 0 },
    { 0xf852f9b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x572f9ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1032f9bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb442f9c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d2f9c7, 90 },
    { 0x2a92f9c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd2f9cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2f9d1, 90 },
    { 0xc02f9d2, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2f9d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532f9d9, 85 },
    { 0xec92f9da, 40 },
    { 0x3182f9d9, 90 },
    { 0xfa32f9dc, 23 },
    { 0, 0 },
    { 0xd9c2f9de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b2f9e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc2f9e7, 55 },
    { 0xd352f9e8, 90 },
    { 0xdd52f9e8, 99 },
    { 0x3292f9e7, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea2f9ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf932f9f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c2f9f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c2fa05, 46 },
    { 0x8042fa06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c82fa09, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c22fa0d, 66 },
    { 0x7172fa0e, 61 },
    { 0xc0a2fa0e, 98 },
    { 0xbc52fa0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd2fa15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d2fa1f, 34 },
    { 0x3a62fa20, 77 },
    { 0xf1e2fa21, 52 },
    { 0, 0 },
    { 0x4242fa23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5322fa26, 90 },
    { 0xd62fa27, 61 },
    { 0, 0 },
    { 0x9fe2fa29, 66 },
    { 0x32fa29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372fa32, 61 },
    { 0x1f12fa33, 23 },
    { 0x8462fa32, 99 },
    { 0x2282fa35, 23 },
    { 0x872fa36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092fa3a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e2fa3d, 98 },
    { 0x8a62fa3d, 61 },
    { 0x3382fa3e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2fa44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8802fa4c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b52fa4f, 76 },
    { 0xa452fa50, 43 },
    { 0x81b2fa50, 55 },
    { 0x59f2fa52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c2fa55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe752fa59, 33 },
    { 0, 0 },
    { 0xfa82fa5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb452fa5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f32fa63, 43 },
    { 0xe512fa64, 98 },
    { 0xfdb2fa65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1152fa6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1752fa6f, 76 },
    { 0, 0 },
    { 0x81a2fa71, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2fa77, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f92fa83, 33 },
    { 0x45d2fa84, 100 },
    { 0, 0 },
    { 0x85e2fa86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f32fa8f, 95 },
    { 0xb842fa90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe2fa97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c2fa9a, 33 },
    { 0x96f2fa9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e42fa9e, 61 },
    { 0x3992fa9f, 90 },
    { 0x46d2fa9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c52faa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a2faa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9362faac, 55 },
    { 0, 0 },
    { 0x9e22faae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa742fab2, 90 },
    { 0, 0 },
    { 0xefb2fab4, 81 },
    { 0, 0 },
    { 0xbc12fab6, 61 },
    { 0x1622fab6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9492faba, 98 },
    { 0, 0 },
    { 0x4272fabc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3842fac1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6282fac6, 43 },
    { 0x4da2fac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce2facb, 66 },
    { 0x7c92facc, 33 },
    { 0, 0 },
    { 0x8d12face, 46 },
    { 0xf262facf, 99 },
    { 0x9982facf, 55 },
    { 0xd802fad1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1542fad4, 76 },
    { 0, 0 },
    { 0x5982fad6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3852fadb, 95 },
    { 0, 0 },
    { 0x4152fadd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a32fae1, 81 },
    { 0x9792fae2, 95 },
    { 0xef92fae3, 98 },
    { 0x83d2fae4, 84 },
    { 0, 0 },
    { 0xb102fae6, 23 },
    { 0, 0 },
    { 0x252fae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de2faec, 98 },
    { 0xebb2faed, 76 },
    { 0xb62faed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a2faf2, 90 },
    { 0, 0 },
    { 0x1cc2faf4, 90 },
    { 0xa282faf4, 98 },
    { 0, 0 },
    { 0x43c2faf7, 91 },
    { 0, 0 },
    { 0x36d2faf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf72fafc, 55 },
    { 0, 0 },
    { 0x3d52fafe, 23 },
    { 0, 0 },
    { 0xd632fb00, 81 },
    { 0, 0 },
    { 0x3a2fb02, 43 },
    { 0x31b2fb03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc2fb10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2fb14, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2fb18, 95 },
    { 0, 0 },
    { 0xc502fb1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7982fb1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d92fb22, 17 },
    { 0x9362fb23, 84 },
    { 0x8582fb22, 98 },
    { 0x6bd2fb25, 87 },
    { 0x9702fb24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa2fb2a, 66 },
    { 0x7502fb2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa612fb32, 95 },
    { 0, 0 },
    { 0xede2fb34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa002fb38, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2002fb3c, 46 },
    { 0, 0 },
    { 0x5a72fb3e, 95 },
    { 0, 0 },
    { 0x85d2fb40, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa2fb4c, 80 },
    { 0xe5f2fb4d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de2fb52, 98 },
    { 0, 0 },
    { 0x9012fb54, 34 },
    { 0x2262fb55, 61 },
    { 0x8f92fb55, 43 },
    { 0, 0 },
    { 0x64f2fb58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2fb5b, 46 },
    { 0x32a2fb5c, 69 },
    { 0xa852fb5c, 84 },
    { 0xd1d2fb5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4682fb61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb352fb66, 77 },
    { 0x2ca2fb67, 34 },
    { 0, 0 },
    { 0xb372fb69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8742fb70, 90 },
    { 0x8652fb71, 23 },
    { 0, 0 },
    { 0xcaf2fb73, 84 },
    { 0x7a2fb73, 93 },
    { 0xb732fb75, 61 },
    { 0, 0 },
    { 0x9122fb77, 76 },
    { 0x3cb2fb78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e12fb7e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb72fb86, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe2fb89, 61 },
    { 0x4ed2fb8a, 33 },
    { 0x2b2fb8a, 87 },
    { 0, 0 },
    { 0x3c72fb8d, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd922fb90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3632fb93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a2fb98, 84 },
    { 0, 0 },
    { 0x1512fb9a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc252fb9d, 23 },
    { 0x9382fb9e, 91 },
    { 0x7aa2fb9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3392fba2, 23 },
    { 0x8fc2fba3, 90 },
    { 0x2d82fba4, 87 },
    { 0xdae2fba5, 46 },
    { 0xa762fba4, 90 },
    { 0x76b2fba7, 33 },
    { 0x8b82fba4, 78 },
    { 0xe922fba5, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc12fbb2, 95 },
    { 0, 0 },
    { 0x83e2fbb4, 90 },
    { 0x8d52fbb5, 93 },
    { 0x172fbb6, 38 },
    { 0, 0 },
    { 0x7e52fbb8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc882fbbb, 98 },
    { 0, 0 },
    { 0xc62fbbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1692fbc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1622fbc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e2fbca, 95 },
    { 0xa892fbcb, 55 },
    { 0xc012fbca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3752fbd2, 95 },
    { 0xaf22fbd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd2fbd6, 23 },
    { 0xa4d2fbd6, 87 },
    { 0x6e32fbd8, 98 },
    { 0x9be2fbd9, 90 },
    { 0x682fbd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec2fbde, 43 },
    { 0, 0 },
    { 0xec2fbe0, 75 },
    { 0xf62fbe1, 55 },
    { 0, 0 },
    { 0xad52fbe3, 69 },
    { 0xe802fbe3, 98 },
    { 0x8612fbe5, 95 },
    { 0, 0 },
    { 0xad92fbe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc62fbeb, 55 },
    { 0x99e2fbec, 98 },
    { 0xdde2fbec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1612fbf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4442fbf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7462fbfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9862fc02, 85 },
    { 0xd772fc02, 55 },
    { 0xf902fc04, 90 },
    { 0x3172fc05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d2fc0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6092fc11, 12 },
    { 0xa552fc12, 93 },
    { 0x7ed2fc13, 95 },
    { 0x68f2fc13, 33 },
    { 0xc6c2fc15, 23 },
    { 0, 0 },
    { 0xf142fc17, 61 },
    { 0, 0 },
    { 0x5c62fc19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02fc1c, 55 },
    { 0, 0 },
    { 0x15d2fc1e, 46 },
    { 0, 0 },
    { 0xe1f2fc20, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb2fc23, 77 },
    { 0xdd72fc24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe2fc2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3322fc2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3172fc31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd912fc39, 34 },
    { 0x762fc3a, 85 },
    { 0xa62fc3b, 95 },
    { 0xa772fc3a, 61 },
    { 0x452fc3b, 67 },
    { 0x1a22fc3e, 55 },
    { 0, 0 },
    { 0xd852fc40, 23 },
    { 0x5de2fc41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9502fc46, 61 },
    { 0, 0 },
    { 0x5422fc48, 61 },
    { 0xa932fc48, 90 },
    { 0x9f82fc4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5622fc4f, 34 },
    { 0x5da2fc4f, 33 },
    { 0x96f2fc4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc2fc55, 90 },
    { 0x14d2fc56, 61 },
    { 0x2492fc57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db2fc5a, 56 },
    { 0xe632fc5b, 84 },
    { 0xe852fc5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2712fc60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1642fc66, 90 },
    { 0, 0 },
    { 0x28e2fc68, 76 },
    { 0x6792fc69, 65 },
    { 0, 0 },
    { 0xe102fc6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c2fc78, 33 },
    { 0x80f2fc79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc082fc7c, 90 },
    { 0, 0 },
    { 0xe8b2fc7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a2fc82, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1802fc87, 66 },
    { 0x92d2fc88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e2fc8b, 61 },
    { 0, 0 },
    { 0xa162fc8d, 95 },
    { 0, 0 },
    { 0x1ae2fc8f, 90 },
    { 0, 0 },
    { 0xbb92fc91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1152fc94, 62 },
    { 0x4e2fc94, 66 },
    { 0xd3b2fc96, 43 },
    { 0xae72fc94, 91 },
    { 0xb52fc98, 33 },
    { 0xa052fc94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb362fc9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d22fc9f, 23 },
    { 0, 0 },
    { 0x3152fca1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1812fca6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4482fcaa, 100 },
    { 0x52a2fcab, 23 },
    { 0, 0 },
    { 0x9182fcad, 90 },
    { 0xf602fcad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7222fcb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd2fcba, 61 },
    { 0, 0 },
    { 0xe612fcbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3392fcbf, 81 },
    { 0, 0 },
    { 0xfe82fcc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c2fcc4, 51 },
    { 0x2f32fcc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5442fcc9, 33 },
    { 0, 0 },
    { 0xa8d2fccb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe512fcd0, 61 },
    { 0x8342fcd0, 33 },
    { 0xaff2fcd1, 33 },
    { 0x6392fcd3, 61 },
    { 0xe5c2fcd4, 67 },
    { 0x6952fcd4, 33 },
    { 0xf172fcd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf2fcda, 76 },
    { 0xcfe2fcda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1132fcdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3362fce4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52fcea, 76 },
    { 0xa4c2fceb, 75 },
    { 0xf92fcec, 61 },
    { 0xd6a2fcec, 61 },
    { 0xe882fcee, 91 },
    { 0x8bf2fcef, 95 },
    { 0x5322fcea, 61 },
    { 0, 0 },
    { 0xfde2fcf2, 34 },
    { 0x88c2fcf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a92fcf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c72fcfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb2fd00, 61 },
    { 0, 0 },
    { 0xe6f2fd02, 55 },
    { 0, 0 },
    { 0xc5b2fd04, 76 },
    { 0xb062fd05, 33 },
    { 0x2752fd06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6132fd1b, 98 },
    { 0x2962fd1b, 100 },
    { 0x6932fd1c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa602fd20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc82fd23, 90 },
    { 0, 0 },
    { 0x95a2fd25, 61 },
    { 0, 0 },
    { 0xf242fd27, 46 },
    { 0xe3f2fd27, 34 },
    { 0, 0 },
    { 0x6f72fd2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2152fd2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e2fd30, 33 },
    { 0x7d52fd31, 43 },
    { 0xc72fd32, 76 },
    { 0xfde2fd33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c2fd3a, 65 },
    { 0, 0 },
    { 0xdb2fd3c, 95 },
    { 0x5532fd3d, 34 },
    { 0xb172fd3e, 88 },
    { 0xf592fd3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe72fd44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d2fd48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc2fd4b, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x2962fd4e, 69 },
    { 0, 0 },
    { 0x2d22fd50, 97 },
    { 0, 0 },
    { 0xbd42fd52, 98 },
    { 0x4622fd53, 98 },
    { 0xd632fd54, 38 },
    { 0x5ae2fd54, 100 },
    { 0xfd2fd55, 33 },
    { 0xef12fd56, 23 },
    { 0, 0 },
    { 0x6d22fd59, 17 },
    { 0xdb62fd59, 84 },
    { 0xb302fd59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a2fd62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e2fd68, 84 },
    { 0xe292fd68, 90 },
    { 0x2942fd68, 95 },
    { 0x2182fd68, 93 },
    { 0xa5d2fd6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8192fd6f, 76 },
    { 0x21d2fd6f, 46 },
    { 0x9402fd71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b2fd74, 90 },
    { 0x9cf2fd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b92fd78, 33 },
    { 0x1ed2fd78, 95 },
    { 0x79e2fd7a, 95 },
    { 0xe232fd7b, 78 },
    { 0xdf12fd7c, 78 },
    { 0xa272fd7d, 90 },
    { 0xd372fd7c, 76 },
    { 0xdf82fd78, 43 },
    { 0x96c2fd80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b2fd85, 46 },
    { 0x8842fd85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4192fd89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f32fd8e, 38 },
    { 0, 0 },
    { 0x40f2fd90, 98 },
    { 0xc462fd90, 76 },
    { 0x9712fd92, 90 },
    { 0xb62fd93, 74 },
    { 0, 0 },
    { 0x5b72fd95, 43 },
    { 0x2f82fd95, 95 },
    { 0, 0 },
    { 0x9d22fd98, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2fd9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4722fda1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f12fda4, 95 },
    { 0x3502fda4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe22fdaa, 46 },
    { 0x88b2fdaa, 67 },
    { 0x4892fdac, 23 },
    { 0x9102fdaa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff72fdb1, 76 },
    { 0x72fdb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682fdb5, 67 },
    { 0xdce2fdb6, 87 },
    { 0xbfe2fdb5, 98 },
    { 0x1e22fdb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d2fdbb, 95 },
    { 0x9f62fdbc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb052fdbf, 61 },
    { 0, 0 },
    { 0x4392fdc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c72fdc4, 23 },
    { 0x1392fdc5, 83 },
    { 0xec72fdc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa772fdd0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea2fdd5, 33 },
    { 0, 0 },
    { 0xaae2fdd7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3572fdda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e2fde5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da2fded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a72fdf0, 33 },
    { 0, 0 },
    { 0x6ee2fdf2, 34 },
    { 0x1ea2fdf3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf772fdf6, 88 },
    { 0x1792fdf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d2fe00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d2fe03, 77 },
    { 0x31a2fe04, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c2fe07, 90 },
    { 0xcda2fe08, 43 },
    { 0x97c2fe08, 43 },
    { 0xa2c2fe08, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca2fe0f, 67 },
    { 0x242fe10, 95 },
    { 0x10a2fe11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad62fe15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f72fe1d, 98 },
    { 0x55d2fe1d, 33 },
    { 0xacf2fe1f, 33 },
    { 0xfcf2fe20, 95 },
    { 0xc1a2fe1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5062fe24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc2fe2a, 90 },
    { 0x6b92fe2b, 69 },
    { 0xe7a2fe2b, 99 },
    { 0x9d12fe2d, 33 },
    { 0, 0 },
    { 0x4e42fe2f, 95 },
    { 0, 0 },
    { 0x7712fe31, 43 },
    { 0x7472fe32, 90 },
    { 0, 0 },
    { 0x7c52fe34, 23 },
    { 0, 0 },
    { 0x2312fe36, 33 },
    { 0x19b2fe37, 33 },
    { 0x1632fe37, 90 },
    { 0xf022fe39, 33 },
    { 0x5ea2fe37, 90 },
    { 0, 0 },
    { 0x7082fe3c, 90 },
    { 0xe4b2fe3c, 23 },
    { 0x3c32fe3e, 87 },
    { 0xdfc2fe3f, 91 },
    { 0x2d12fe3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2fe43, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4502fe48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b2fe4d, 95 },
    { 0, 0 },
    { 0xd072fe4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d72fe53, 23 },
    { 0xe1b2fe53, 33 },
    { 0x3c22fe55, 95 },
    { 0x8282fe55, 52 },
    { 0x2a42fe54, 23 },
    { 0xa792fe58, 52 },
    { 0x2d02fe57, 23 },
    { 0x9042fe54, 100 },
    { 0, 0 },
    { 0xe0c2fe5c, 33 },
    { 0, 0 },
    { 0x5fe2fe5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d2fe61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82fe67, 90 },
    { 0x9412fe67, 43 },
    { 0, 0 },
    { 0x7b12fe6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e02fe6f, 76 },
    { 0, 0 },
    { 0x5982fe71, 98 },
    { 0, 0 },
    { 0x4562fe73, 99 },
    { 0x7672fe73, 90 },
    { 0x24a2fe74, 98 },
    { 0, 0 },
    { 0xe412fe77, 98 },
    { 0x91d2fe78, 98 },
    { 0x2f32fe79, 66 },
    { 0xd782fe79, 96 },
    { 0, 0 },
    { 0x7972fe7c, 93 },
    { 0, 0 },
    { 0x1562fe7e, 95 },
    { 0xf792fe7f, 90 },
    { 0xa5d2fe7f, 98 },
    { 0xd3e2fe80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba12fe84, 90 },
    { 0, 0 },
    { 0xc82fe86, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd72fe8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7752fe95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe2fe98, 43 },
    { 0x912fe99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52fe9d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c2fea1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c2fea4, 61 },
    { 0, 0 },
    { 0x3a82fea6, 66 },
    { 0xc102fea7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e2feab, 95 },
    { 0xc0f2feab, 61 },
    { 0xb992fead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee2feb2, 91 },
    { 0xd782feb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb902feba, 76 },
    { 0xec02feba, 90 },
    { 0x3992feba, 90 },
    { 0xc502febd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd2fed0, 91 },
    { 0x78a2fed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6752fed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72fed9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5782fedc, 55 },
    { 0, 0 },
    { 0xa1d2fede, 61 },
    { 0xdbe2fedf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d72fee2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042fee6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd2feea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff12fef2, 33 },
    { 0xe2d2fef3, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c2fef7, 66 },
    { 0xff32fef8, 46 },
    { 0x16e2fef7, 33 },
    { 0xfa12fef8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af2fefd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3922ff00, 87 },
    { 0x95b2ff01, 77 },
    { 0xeef2ff01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6232ff08, 33 },
    { 0, 0 },
    { 0x1a72ff0a, 33 },
    { 0xdf02ff0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b92ff11, 17 },
    { 0xcf72ff12, 76 },
    { 0x1f02ff13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2642ff18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e12ff1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c2ff1f, 43 },
    { 0x4b2ff1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce2ff26, 43 },
    { 0, 0 },
    { 0xd452ff28, 55 },
    { 0, 0 },
    { 0xc152ff2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e2ff2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd2ff30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2292ff34, 33 },
    { 0x52a2ff35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed2ff38, 95 },
    { 0, 0 },
    { 0x8ed2ff3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa792ff3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4362ff42, 93 },
    { 0, 0 },
    { 0x33a2ff44, 33 },
    { 0xd732ff45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc2ff4b, 87 },
    { 0, 0 },
    { 0xa692ff4d, 90 },
    { 0xb362ff4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5872ff51, 90 },
    { 0, 0 },
    { 0x4032ff53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f2ff58, 83 },
    { 0x9402ff58, 100 },
    { 0xd452ff5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce2ff64, 55 },
    { 0xc1f2ff65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e22ff6b, 68 },
    { 0x7a22ff6b, 61 },
    { 0xa312ff6d, 90 },
    { 0x44e2ff6e, 95 },
    { 0x86c2ff6f, 91 },
    { 0x1762ff6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe612ff77, 90 },
    { 0, 0 },
    { 0xfe2ff79, 46 },
    { 0xd72ff7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f12ff81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2ff84, 98 },
    { 0, 0 },
    { 0xcab2ff86, 91 },
    { 0, 0 },
    { 0xc0e2ff88, 90 },
    { 0x1162ff89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a62ff90, 100 },
    { 0xe812ff90, 33 },
    { 0x3f52ff92, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d2ff95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ff98, 61 },
    { 0, 0 },
    { 0xca32ff9a, 87 },
    { 0x3682ff9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed2ffa2, 61 },
    { 0x70c2ffa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2ffa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7422ffab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab82ffb5, 38 },
    { 0x2d52ffb5, 61 },
    { 0x2332ffb5, 90 },
    { 0xf492ffb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b2ffbe, 46 },
    { 0, 0 },
    { 0x6412ffc0, 93 },
    { 0xdc82ffc1, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f92ffc7, 61 },
    { 0xade2ffc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef32ffd6, 80 },
    { 0, 0 },
    { 0x4ad2ffd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3652ffde, 55 },
    { 0x3f42ffdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c2ffe4, 23 },
    { 0xf6d2ffe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b2ffed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e32fff5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c42fffa, 77 },
    { 0xf492fffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7782fffe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca230004, 84 },
    { 0x67c30005, 90 },
    { 0x90030006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40230009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e3000d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f30011, 61 },
    { 0x86630012, 33 },
    { 0xbcb30013, 46 },
    { 0, 0 },
    { 0x33330015, 98 },
    { 0x8f130016, 33 },
    { 0x50530017, 99 },
    { 0, 0 },
    { 0xae530019, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65230021, 26 },
    { 0xf3330022, 55 },
    { 0xa2e30023, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf73002d, 90 },
    { 0x7dd3002d, 23 },
    { 0x7543002f, 95 },
    { 0xd5930030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea30037, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2213004d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff30050, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b30054, 76 },
    { 0x4ca30055, 81 },
    { 0xe5a30056, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b530059, 90 },
    { 0x1723005a, 55 },
    { 0x2543005a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4130060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce30063, 23 },
    { 0, 0 },
    { 0x65330065, 33 },
    { 0x42a30066, 76 },
    { 0x94e30067, 34 },
    { 0x4e130066, 75 },
    { 0x70030068, 47 },
    { 0xa0c30067, 17 },
    { 0xc933006b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a530072, 23 },
    { 0, 0 },
    { 0xaf330074, 90 },
    { 0x2a030075, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0030078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c3007b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac30081, 33 },
    { 0x7cd30082, 98 },
    { 0, 0 },
    { 0x77130084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e230097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778300a0, 78 },
    { 0x81300a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x420300a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774300ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7300af, 33 },
    { 0xb81300b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d300b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c300bc, 90 },
    { 0x906300bd, 33 },
    { 0xf6e300bc, 90 },
    { 0x7d300bc, 90 },
    { 0xbc2300c0, 46 },
    { 0x8b1300bd, 98 },
    { 0xbcc300c2, 93 },
    { 0xcfc300c3, 100 },
    { 0x48f300c4, 33 },
    { 0x4ee300c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a300ca, 93 },
    { 0xd1f300cb, 76 },
    { 0xa4d300cb, 33 },
    { 0x97e300cd, 23 },
    { 0xa4a300ca, 93 },
    { 0xbdd300ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5300d8, 40 },
    { 0xb67300d9, 90 },
    { 0x50c300da, 76 },
    { 0x7b5300d9, 90 },
    { 0, 0 },
    { 0xc2f300dd, 96 },
    { 0, 0 },
    { 0x782300df, 69 },
    { 0, 0 },
    { 0x489300e1, 90 },
    { 0x509300e2, 61 },
    { 0, 0 },
    { 0xe9b300e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x917300e7, 85 },
    { 0, 0 },
    { 0x7e9300e9, 90 },
    { 0, 0 },
    { 0x413300eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9300ee, 98 },
    { 0, 0 },
    { 0x677300f0, 84 },
    { 0xb28300f1, 23 },
    { 0, 0 },
    { 0xb4f300f3, 85 },
    { 0, 0 },
    { 0xff5300f5, 98 },
    { 0xe4300f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22300fb, 90 },
    { 0, 0 },
    { 0x6f8300fd, 93 },
    { 0x4a4300fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e30102, 95 },
    { 0xe9f30103, 64 },
    { 0x13130104, 88 },
    { 0x8c330105, 43 },
    { 0x89b30106, 67 },
    { 0xfeb30107, 55 },
    { 0x79630106, 38 },
    { 0x69130109, 90 },
    { 0xfa030105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e30110, 33 },
    { 0, 0 },
    { 0x32f30112, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1530115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac23011d, 23 },
    { 0xb453011d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83430128, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9693012c, 55 },
    { 0, 0 },
    { 0x4f3012e, 61 },
    { 0, 0 },
    { 0xd1830130, 76 },
    { 0, 0 },
    { 0xb5930132, 66 },
    { 0x84130132, 76 },
    { 0xbc730134, 95 },
    { 0x32730135, 90 },
    { 0x1ed30135, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b30139, 76 },
    { 0xaee3013a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5953013e, 23 },
    { 0x8583013f, 55 },
    { 0, 0 },
    { 0x45230141, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67030145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc30148, 55 },
    { 0x1d930148, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b3014c, 86 },
    { 0xa753014c, 98 },
    { 0x2323014c, 33 },
    { 0xef33014f, 55 },
    { 0x3a13014c, 90 },
    { 0xee230151, 33 },
    { 0, 0 },
    { 0x50030153, 65 },
    { 0xd9130154, 95 },
    { 0xfc330155, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be30158, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4883015e, 33 },
    { 0x3913015f, 77 },
    { 0x3f630160, 23 },
    { 0x1233015f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8230168, 23 },
    { 0, 0 },
    { 0x7773016a, 33 },
    { 0, 0 },
    { 0x6af3016c, 90 },
    { 0xd373016d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a30175, 78 },
    { 0xeee30175, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca30179, 33 },
    { 0, 0 },
    { 0xb903017b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a230183, 61 },
    { 0x44230184, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2730188, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d3018b, 98 },
    { 0x8083018c, 77 },
    { 0xa163018d, 95 },
    { 0x63f3018d, 98 },
    { 0xfad3018f, 34 },
    { 0x83c30190, 81 },
    { 0x8763018f, 98 },
    { 0x8b73018f, 98 },
    { 0xf8c30193, 46 },
    { 0xbae30191, 46 },
    { 0x6463018b, 33 },
    { 0x8283018f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x36630199, 46 },
    { 0x2453019a, 33 },
    { 0x6f930199, 23 },
    { 0x8893019c, 67 },
    { 0x66f3019d, 95 },
    { 0x6f93019b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777301a4, 93 },
    { 0, 0 },
    { 0xf3c301a6, 70 },
    { 0x807301a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca301aa, 34 },
    { 0x1b1301aa, 76 },
    { 0xde0301ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x745301af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a301b2, 60 },
    { 0xb0d301b3, 34 },
    { 0, 0 },
    { 0x5b301b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7301c1, 99 },
    { 0, 0 },
    { 0x431301c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc301c9, 23 },
    { 0xf3a301ca, 76 },
    { 0xdcf301cb, 90 },
    { 0xae1301cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00301d1, 43 },
    { 0x55b301d2, 68 },
    { 0x8fb301d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e301d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2301dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4301e3, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26301e7, 17 },
    { 0, 0 },
    { 0x4a2301e9, 66 },
    { 0, 0 },
    { 0x8e9301eb, 23 },
    { 0xe5f301ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b301ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7301f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x497301f6, 85 },
    { 0x566301f6, 23 },
    { 0x2f4301f8, 85 },
    { 0, 0 },
    { 0xca1301fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7230202, 95 },
    { 0, 0 },
    { 0xdac30204, 93 },
    { 0, 0 },
    { 0x72730206, 17 },
    { 0x20430207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3020a, 65 },
    { 0, 0 },
    { 0xe863020c, 61 },
    { 0x16f3020c, 33 },
    { 0xe103020e, 23 },
    { 0, 0 },
    { 0xc330210, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b730216, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb163021b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd030223, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97930229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef3022c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1930233, 76 },
    { 0xbe530234, 33 },
    { 0, 0 },
    { 0x6d230236, 91 },
    { 0x6ba30237, 90 },
    { 0, 0 },
    { 0xb8630239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3933023c, 55 },
    { 0x9a3023d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2230242, 76 },
    { 0x39530242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b730246, 46 },
    { 0x77730247, 90 },
    { 0x45130246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe3024c, 98 },
    { 0, 0 },
    { 0x48f3024e, 96 },
    { 0xa3c3024f, 90 },
    { 0, 0 },
    { 0x84630251, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f30256, 33 },
    { 0, 0 },
    { 0x45d30258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa903025e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9130261, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30330268, 85 },
    { 0, 0 },
    { 0xe8b3026a, 80 },
    { 0, 0 },
    { 0xa8a3026c, 98 },
    { 0xe63026d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b30276, 33 },
    { 0, 0 },
    { 0xaa530278, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3913027c, 23 },
    { 0x7bd3027d, 61 },
    { 0, 0 },
    { 0x3123027f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e30284, 98 },
    { 0x98530285, 95 },
    { 0xf5e30285, 46 },
    { 0x95230287, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f33028a, 90 },
    { 0xccf3028b, 17 },
    { 0, 0 },
    { 0x2b83028d, 55 },
    { 0x6c23028e, 88 },
    { 0x90f3028d, 65 },
    { 0x6553028d, 90 },
    { 0, 0 },
    { 0x72b30292, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98630297, 33 },
    { 0x56f30298, 90 },
    { 0, 0 },
    { 0xd803029a, 76 },
    { 0x8df3029b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46302a1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8302a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1302ab, 90 },
    { 0, 0 },
    { 0x676302ad, 34 },
    { 0x90a302ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af302b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8302ba, 90 },
    { 0, 0 },
    { 0x540302bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750302c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab302c7, 99 },
    { 0x6f9302c7, 43 },
    { 0, 0 },
    { 0x6a7302ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be302d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a302d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc302db, 90 },
    { 0, 0 },
    { 0x24f302dd, 78 },
    { 0x7c5302de, 46 },
    { 0x352302dd, 76 },
    { 0, 0 },
    { 0x56302e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7302e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141302e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4302ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11302f1, 33 },
    { 0x39302f1, 88 },
    { 0x66d302f1, 33 },
    { 0, 0 },
    { 0x8f6302f5, 87 },
    { 0xc0d302f5, 90 },
    { 0x7a6302f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e302fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1130300, 95 },
    { 0, 0 },
    { 0x70b30302, 23 },
    { 0xfd030303, 100 },
    { 0, 0 },
    { 0xab30305, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d30309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7713030c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66930312, 66 },
    { 0, 0 },
    { 0xdd430314, 33 },
    { 0xd6330314, 90 },
    { 0, 0 },
    { 0xbdf30317, 95 },
    { 0xaff30317, 90 },
    { 0xcdf30319, 85 },
    { 0x30230319, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9030322, 34 },
    { 0x7a230323, 90 },
    { 0x16330322, 66 },
    { 0, 0 },
    { 0x99c30326, 98 },
    { 0x4930327, 67 },
    { 0x17530327, 61 },
    { 0x86930327, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2630330, 95 },
    { 0, 0 },
    { 0xc9630332, 34 },
    { 0xa7830333, 34 },
    { 0x75130334, 90 },
    { 0xca730334, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25430338, 33 },
    { 0x4fc30338, 33 },
    { 0xe1c3033a, 36 },
    { 0x5953033b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf3033e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2330343, 98 },
    { 0x49e30344, 90 },
    { 0x70930345, 98 },
    { 0, 0 },
    { 0xd7d30347, 90 },
    { 0, 0 },
    { 0x6cc30349, 78 },
    { 0, 0 },
    { 0xd63034b, 61 },
    { 0x7603034b, 98 },
    { 0xdf3034b, 100 },
    { 0xac83034e, 36 },
    { 0x2283034f, 98 },
    { 0x7493034d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38430354, 95 },
    { 0x71130355, 43 },
    { 0x72130356, 98 },
    { 0x75230354, 96 },
    { 0xc2d30358, 95 },
    { 0x78630359, 91 },
    { 0x58f3035a, 90 },
    { 0, 0 },
    { 0x7de3035c, 75 },
    { 0, 0 },
    { 0x4033035e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d730364, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b3036d, 23 },
    { 0xc833036e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b730371, 55 },
    { 0, 0 },
    { 0xdbd30373, 95 },
    { 0x4c130374, 33 },
    { 0, 0 },
    { 0xd5b30376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d3037a, 87 },
    { 0, 0 },
    { 0xec53037c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a130382, 61 },
    { 0xdf30383, 81 },
    { 0x92e30384, 98 },
    { 0x6ce30385, 33 },
    { 0xf5d30385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f3038b, 61 },
    { 0, 0 },
    { 0x3543038d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b030390, 61 },
    { 0x52c30390, 90 },
    { 0x18330392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1930397, 61 },
    { 0x34330397, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19303a0, 90 },
    { 0x2e1303a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9303a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c303a8, 96 },
    { 0, 0 },
    { 0xf2c303aa, 23 },
    { 0x426303aa, 33 },
    { 0xd34303ac, 52 },
    { 0x46d303ad, 33 },
    { 0xa59303ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c303b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e303ba, 98 },
    { 0xe78303ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e303be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed303c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462303c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d303cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2303d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42303d6, 75 },
    { 0, 0 },
    { 0xe42303d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xece303db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c303de, 33 },
    { 0x71a303de, 33 },
    { 0x7b9303e0, 33 },
    { 0xf29303e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b303e7, 61 },
    { 0x27303e8, 95 },
    { 0xf25303e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d303ee, 23 },
    { 0x54b303ef, 81 },
    { 0xd3303f0, 33 },
    { 0, 0 },
    { 0x9aa303f2, 67 },
    { 0x737303f3, 34 },
    { 0x760303f2, 61 },
    { 0x230303f5, 84 },
    { 0xf04303f6, 43 },
    { 0x106303f7, 33 },
    { 0xd28303f3, 85 },
    { 0x40d303f5, 46 },
    { 0x97d303fa, 93 },
    { 0x98303f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce30404, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40930407, 66 },
    { 0x35a30408, 66 },
    { 0, 0 },
    { 0xcbf3040a, 55 },
    { 0x3813040b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3813040e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e30414, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d30417, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf63041b, 61 },
    { 0x7713041c, 75 },
    { 0x1bf3041d, 61 },
    { 0xfcd3041e, 90 },
    { 0x1b3041f, 95 },
    { 0xf7f30420, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67830425, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68230429, 33 },
    { 0x6a43042a, 19 },
    { 0xed73042b, 100 },
    { 0xb873042a, 98 },
    { 0xd523042d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc30437, 52 },
    { 0xb7e30438, 49 },
    { 0x11d30439, 33 },
    { 0, 0 },
    { 0x9eb3043b, 90 },
    { 0xd843043c, 99 },
    { 0, 0 },
    { 0x9d93043e, 78 },
    { 0x9763043e, 90 },
    { 0, 0 },
    { 0x6dc30441, 100 },
    { 0x6c230442, 90 },
    { 0, 0 },
    { 0xa9930444, 61 },
    { 0, 0 },
    { 0x22c30446, 90 },
    { 0xf9130446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3903044e, 98 },
    { 0xb7d3044f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd230457, 61 },
    { 0xe1a30457, 91 },
    { 0x11330459, 90 },
    { 0xca73045a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663045e, 34 },
    { 0xe1e3045f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32630463, 33 },
    { 0xb7030463, 33 },
    { 0xa8e30463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf30468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a3046d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac30474, 33 },
    { 0xf0c30474, 61 },
    { 0x3e230476, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a30483, 90 },
    { 0x26430484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1330487, 85 },
    { 0x5b630488, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc963048e, 61 },
    { 0x8f3048f, 90 },
    { 0x39530490, 17 },
    { 0, 0 },
    { 0x3b730492, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96630496, 90 },
    { 0, 0 },
    { 0x19830498, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816304a1, 61 },
    { 0xd31304a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c304ac, 23 },
    { 0x54a304ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6304b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5304b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b304b6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c304bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b304c0, 61 },
    { 0x1c3304c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b304c4, 88 },
    { 0x9eb304c4, 95 },
    { 0x677304c6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8304c9, 23 },
    { 0xe55304ca, 46 },
    { 0xfb9304c9, 33 },
    { 0x8a0304cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c304cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3304d2, 90 },
    { 0xedb304d2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96304d7, 46 },
    { 0xf23304d8, 61 },
    { 0, 0 },
    { 0xcd1304da, 33 },
    { 0x37c304db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab304e3, 75 },
    { 0, 0 },
    { 0x4ec304e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228304ea, 95 },
    { 0, 0 },
    { 0x1b1304ec, 93 },
    { 0x349304ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3304f2, 90 },
    { 0x6f8304f3, 55 },
    { 0xe0f304f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd304f8, 61 },
    { 0x25c304f9, 61 },
    { 0x120304fa, 95 },
    { 0, 0 },
    { 0x650304fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e30501, 40 },
    { 0x24730502, 61 },
    { 0x81e30503, 76 },
    { 0xb4030504, 76 },
    { 0xb0530505, 61 },
    { 0xac730506, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2130509, 90 },
    { 0x9313050a, 55 },
    { 0x6e630509, 55 },
    { 0x32a3050c, 33 },
    { 0x6de3050d, 90 },
    { 0, 0 },
    { 0x5893050f, 98 },
    { 0xe1230510, 100 },
    { 0xe730511, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d33051d, 76 },
    { 0xf63051e, 76 },
    { 0xa4b3051f, 93 },
    { 0x7043051f, 93 },
    { 0xeef30521, 52 },
    { 0x74d30521, 98 },
    { 0x9c73051f, 61 },
    { 0x8cc30524, 69 },
    { 0x5f730525, 23 },
    { 0x89430526, 34 },
    { 0xe6830525, 85 },
    { 0x59130526, 98 },
    { 0xf9030522, 23 },
    { 0xb5c3051e, 95 },
    { 0, 0 },
    { 0x9243052c, 34 },
    { 0x2823052d, 61 },
    { 0xfc3052e, 23 },
    { 0x3e83052d, 85 },
    { 0xcc03052d, 69 },
    { 0xcc930531, 95 },
    { 0, 0 },
    { 0x8130533, 61 },
    { 0, 0 },
    { 0xf0130535, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65830539, 61 },
    { 0xbd130539, 33 },
    { 0x5b3053b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34130541, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2130544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa3054d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3530552, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90630556, 33 },
    { 0x45b30557, 55 },
    { 0, 0 },
    { 0xfda30559, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a30560, 93 },
    { 0x48d30560, 33 },
    { 0xae030561, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc30569, 23 },
    { 0x1a30569, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82830573, 91 },
    { 0xb9830574, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce30579, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2443057c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b3057f, 95 },
    { 0x5593057f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80630584, 95 },
    { 0x13630585, 95 },
    { 0x15e30586, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9730589, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf593058d, 76 },
    { 0x6c73058e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2630592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f3059c, 43 },
    { 0xbd43059c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22305a0, 68 },
    { 0xb94305a0, 61 },
    { 0xa20305a1, 98 },
    { 0, 0 },
    { 0x63e305a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c305a7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d305aa, 23 },
    { 0x541305ab, 56 },
    { 0x3f1305aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5305af, 98 },
    { 0, 0 },
    { 0xd64305b1, 90 },
    { 0, 0 },
    { 0xce7305b3, 87 },
    { 0x7d0305b4, 99 },
    { 0x3ca305b4, 43 },
    { 0, 0 },
    { 0x357305b7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28305bb, 61 },
    { 0, 0 },
    { 0x4fe305bd, 55 },
    { 0x3ac305be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511305c6, 98 },
    { 0, 0 },
    { 0x6f3305c8, 75 },
    { 0x64a305c9, 43 },
    { 0x38d305ca, 61 },
    { 0x4ff305c8, 61 },
    { 0x575305c9, 33 },
    { 0xbc6305c8, 55 },
    { 0x44d305ce, 61 },
    { 0x335305cf, 85 },
    { 0, 0 },
    { 0x565305d1, 76 },
    { 0x51f305d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98305da, 23 },
    { 0x5ee305da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b305e2, 98 },
    { 0, 0 },
    { 0xf40305e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x175305e7, 90 },
    { 0, 0 },
    { 0x378305e9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e305ec, 90 },
    { 0x93b305ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1305f2, 81 },
    { 0x26d305f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f305f7, 98 },
    { 0x53a305f7, 23 },
    { 0, 0 },
    { 0x940305fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7530600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc3060b, 95 },
    { 0xb933060b, 61 },
    { 0, 0 },
    { 0x3643060e, 61 },
    { 0, 0 },
    { 0xa6630610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e30616, 88 },
    { 0xab730617, 95 },
    { 0x2ae30616, 87 },
    { 0x99130616, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a3061f, 98 },
    { 0xad93061f, 61 },
    { 0xb1230621, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b30624, 33 },
    { 0xdcb30624, 34 },
    { 0x29330625, 76 },
    { 0x38730627, 85 },
    { 0xcba30625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8973062b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db3062e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30930632, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb130635, 55 },
    { 0x5c830635, 90 },
    { 0, 0 },
    { 0x5d330638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4983063c, 95 },
    { 0x24e3063c, 69 },
    { 0x7dd3063c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f130641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd430644, 61 },
    { 0, 0 },
    { 0x94030646, 80 },
    { 0, 0 },
    { 0xf5e30648, 99 },
    { 0x9b630649, 90 },
    { 0x7bb3064a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda63064d, 90 },
    { 0xf0a3064e, 90 },
    { 0xd113064d, 87 },
    { 0xc0d30650, 55 },
    { 0x42f30651, 55 },
    { 0xc1430652, 55 },
    { 0xbce30653, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c30656, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xad230659, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d3065d, 95 },
    { 0x34f3065e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15130665, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3066a, 61 },
    { 0xf83066a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3066f, 90 },
    { 0xee63066f, 90 },
    { 0x1b930671, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa930675, 34 },
    { 0x49230675, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb83067e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9830681, 33 },
    { 0x9f630682, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c930687, 46 },
    { 0x4330688, 33 },
    { 0x61e30689, 76 },
    { 0x61530689, 91 },
    { 0x1fe30687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e130693, 4 },
    { 0xef630694, 90 },
    { 0x1f730695, 90 },
    { 0x2d930696, 46 },
    { 0xa1730696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd33069a, 68 },
    { 0, 0 },
    { 0x9993069c, 76 },
    { 0, 0 },
    { 0xfc73069e, 61 },
    { 0xa223069f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9306a2, 90 },
    { 0x5d306a3, 84 },
    { 0xce6306a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771306a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc306af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a306b3, 75 },
    { 0x6d9306b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500306ba, 65 },
    { 0x228306bb, 66 },
    { 0x288306ba, 69 },
    { 0x372306bd, 90 },
    { 0x14d306bc, 52 },
    { 0x1ba306bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2306c2, 91 },
    { 0, 0 },
    { 0x77a306c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439306ca, 84 },
    { 0, 0 },
    { 0xa8a306cc, 33 },
    { 0x3f8306cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5306d4, 91 },
    { 0xf5e306d5, 91 },
    { 0x7b7306d5, 43 },
    { 0xa74306d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6306da, 46 },
    { 0xac6306db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63306de, 33 },
    { 0xe4f306df, 98 },
    { 0x947306de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb306e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8306ef, 61 },
    { 0xe0d306f0, 33 },
    { 0x1d7306f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1306f6, 76 },
    { 0x416306f6, 90 },
    { 0, 0 },
    { 0xad3306f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4306fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab30701, 61 },
    { 0, 0 },
    { 0xf1b30703, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb630706, 99 },
    { 0x29e30707, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd243070c, 46 },
    { 0, 0 },
    { 0x4373070e, 99 },
    { 0x3a73070f, 94 },
    { 0xcfb30710, 99 },
    { 0xbd53070e, 84 },
    { 0x45530712, 40 },
    { 0x8d73070f, 95 },
    { 0x62130713, 95 },
    { 0, 0 },
    { 0x85c30716, 46 },
    { 0x4a930717, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe673071e, 52 },
    { 0xea3071e, 90 },
    { 0x3c53071e, 33 },
    { 0xa0430720, 43 },
    { 0x85c30722, 76 },
    { 0xaaf30722, 33 },
    { 0xff130724, 78 },
    { 0, 0 },
    { 0xc5630726, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9383072f, 33 },
    { 0xc0d3072f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28430734, 90 },
    { 0x95230734, 98 },
    { 0xeee30736, 100 },
    { 0xb0c30734, 34 },
    { 0xd9730736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1813073f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b30742, 23 },
    { 0xda30743, 78 },
    { 0x56f30744, 100 },
    { 0x1aa30745, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43074a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633074d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b930750, 61 },
    { 0, 0 },
    { 0xe3c30752, 55 },
    { 0x28330753, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee30757, 84 },
    { 0x54330758, 56 },
    { 0x2d930757, 90 },
    { 0, 0 },
    { 0xa5b3075b, 87 },
    { 0xbdd3075c, 98 },
    { 0x5e83075c, 100 },
    { 0, 0 },
    { 0x9393075f, 67 },
    { 0, 0 },
    { 0xed830761, 55 },
    { 0x12530761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3076d, 81 },
    { 0x5e73076e, 67 },
    { 0x8ac3076e, 76 },
    { 0, 0 },
    { 0x88c30771, 93 },
    { 0, 0 },
    { 0x50d30773, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62830776, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf3077e, 69 },
    { 0x2423077f, 100 },
    { 0xc6e30780, 23 },
    { 0xa3b30781, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a30784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc3078d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5230791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1307a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x738307a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda307aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0307b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67307ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28307bf, 23 },
    { 0x50f307c0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e307c8, 33 },
    { 0x488307c9, 33 },
    { 0, 0 },
    { 0x47c307cb, 61 },
    { 0xb3e307cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145307d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d307d3, 33 },
    { 0x2e1307d4, 65 },
    { 0, 0 },
    { 0x52b307d6, 90 },
    { 0x51f307d7, 90 },
    { 0, 0 },
    { 0x7ff307d9, 76 },
    { 0, 0 },
    { 0xeff307db, 67 },
    { 0x218307dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf307e0, 77 },
    { 0x188307e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86307e4, 33 },
    { 0x5ab307e4, 55 },
    { 0, 0 },
    { 0xfd6307e7, 76 },
    { 0x79b307e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2307eb, 90 },
    { 0xf24307ec, 61 },
    { 0, 0 },
    { 0x9c3307ee, 100 },
    { 0xddc307ef, 33 },
    { 0x580307f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78307f7, 76 },
    { 0x245307f8, 61 },
    { 0, 0 },
    { 0x49e307fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae330800, 91 },
    { 0x84530801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7930807, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8833080c, 61 },
    { 0x8493080d, 88 },
    { 0x74b3080d, 76 },
    { 0, 0 },
    { 0x27c30810, 61 },
    { 0x2d730810, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d30815, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673081b, 90 },
    { 0xfff3081c, 98 },
    { 0x5433081d, 93 },
    { 0xe6a3081e, 98 },
    { 0xab13081d, 76 },
    { 0x18730820, 61 },
    { 0xcbe30821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d30829, 55 },
    { 0x6243082a, 95 },
    { 0xe483082b, 84 },
    { 0x9dc3082c, 91 },
    { 0x5b83082c, 98 },
    { 0x8813082b, 55 },
    { 0, 0 },
    { 0x43730830, 69 },
    { 0xb7b30831, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b30838, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f30843, 98 },
    { 0x91530844, 93 },
    { 0, 0 },
    { 0xf5130846, 46 },
    { 0xb5a30847, 55 },
    { 0x19530848, 88 },
    { 0, 0 },
    { 0x9243084a, 61 },
    { 0x9683084b, 61 },
    { 0x6df3084b, 98 },
    { 0x2a73084a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0330851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5630856, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5073085b, 61 },
    { 0, 0 },
    { 0x7f73085d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6230865, 23 },
    { 0xadb30865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f30872, 61 },
    { 0x24830872, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xde630876, 90 },
    { 0x98530877, 61 },
    { 0x8b030877, 23 },
    { 0xbcd30879, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf83087d, 95 },
    { 0xa643087d, 43 },
    { 0, 0 },
    { 0x1c130880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82430883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c730886, 98 },
    { 0x4b230887, 66 },
    { 0, 0 },
    { 0xc2630889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3088f, 76 },
    { 0x7f23088f, 61 },
    { 0, 0 },
    { 0xb8f30892, 33 },
    { 0x10730893, 61 },
    { 0xf2130893, 34 },
    { 0x5be30895, 75 },
    { 0, 0 },
    { 0x7a330897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa783089a, 43 },
    { 0, 0 },
    { 0x1523089c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae308a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437308a7, 88 },
    { 0xa07308a8, 95 },
    { 0x620308a9, 36 },
    { 0x750308a7, 95 },
    { 0x7f2308a9, 100 },
    { 0xd30308ac, 99 },
    { 0xbd308ad, 61 },
    { 0xd70308ae, 68 },
    { 0xcfa308ad, 55 },
    { 0, 0 },
    { 0xe1c308b1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x420308b4, 61 },
    { 0x54c308b5, 90 },
    { 0x82d308b6, 19 },
    { 0x22a308b7, 61 },
    { 0xc05308b5, 99 },
    { 0x658308b9, 90 },
    { 0x8c308ba, 95 },
    { 0x102308bb, 61 },
    { 0x41f308bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0308c0, 66 },
    { 0x900308c1, 95 },
    { 0, 0 },
    { 0x333308c3, 90 },
    { 0x536308c4, 78 },
    { 0xc36308c5, 98 },
    { 0x651308c6, 55 },
    { 0xcdb308c7, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x867308ca, 90 },
    { 0xb46308cb, 46 },
    { 0xbbc308cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f308d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b308d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94308db, 33 },
    { 0xe3a308dc, 90 },
    { 0xaa1308dd, 46 },
    { 0xd3d308de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37308e1, 100 },
    { 0xf19308e2, 67 },
    { 0xbed308e3, 100 },
    { 0x89a308e4, 33 },
    { 0x367308e1, 95 },
    { 0xdd7308e6, 90 },
    { 0xc06308e7, 33 },
    { 0x719308e7, 34 },
    { 0x67a308e7, 43 },
    { 0xa2c308ea, 33 },
    { 0x47d308ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24308f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x897308f3, 33 },
    { 0xf2e308f4, 77 },
    { 0xca6308f4, 46 },
    { 0x65e308f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e308fa, 99 },
    { 0, 0 },
    { 0x96c308fc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11308ff, 98 },
    { 0, 0 },
    { 0xe5330901, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7930907, 76 },
    { 0xdc130907, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933090c, 76 },
    { 0xa2c3090d, 40 },
    { 0, 0 },
    { 0x2ba3090f, 66 },
    { 0, 0 },
    { 0xe330911, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba430914, 23 },
    { 0xc8530915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af30919, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf3091c, 23 },
    { 0x30c3091d, 90 },
    { 0x89f3091e, 43 },
    { 0, 0 },
    { 0x10e30920, 95 },
    { 0x3c330920, 95 },
    { 0xb1c30922, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36930927, 23 },
    { 0xb4330928, 95 },
    { 0x93030929, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2413092c, 98 },
    { 0x23c3092d, 90 },
    { 0x28d3092e, 78 },
    { 0xf7b3092f, 95 },
    { 0xe1e3092e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e230934, 34 },
    { 0x96e30935, 33 },
    { 0x7a530934, 90 },
    { 0, 0 },
    { 0xe2d30938, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23093b, 95 },
    { 0x6f13093c, 55 },
    { 0x54a3093c, 95 },
    { 0x5d3093e, 98 },
    { 0x90e3093e, 55 },
    { 0xb013093c, 46 },
    { 0x12b30941, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d30944, 23 },
    { 0xd2830945, 99 },
    { 0x61f30946, 100 },
    { 0xd2630946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf43094a, 90 },
    { 0xf173094b, 23 },
    { 0xab13094b, 33 },
    { 0x5b3094d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f530952, 99 },
    { 0x18630953, 33 },
    { 0xd4d30953, 84 },
    { 0xf230954, 23 },
    { 0, 0 },
    { 0x11e30957, 78 },
    { 0xa6130958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a73095b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1793095e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb30968, 85 },
    { 0, 0 },
    { 0xa2e3096a, 33 },
    { 0, 0 },
    { 0xdb43096c, 55 },
    { 0xee63096d, 95 },
    { 0x9f13096c, 81 },
    { 0, 0 },
    { 0xcaa30970, 34 },
    { 0, 0 },
    { 0xad930972, 23 },
    { 0, 0 },
    { 0xaea30974, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a130979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e30983, 93 },
    { 0x5be30984, 46 },
    { 0x3e30985, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb23098a, 99 },
    { 0x8683098b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c230991, 67 },
    { 0xd6530992, 95 },
    { 0xc0030993, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab30996, 98 },
    { 0, 0 },
    { 0x9cd30998, 66 },
    { 0x69030999, 46 },
    { 0x38d30998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab309a9, 90 },
    { 0, 0 },
    { 0x3e309ab, 61 },
    { 0x7c1309ab, 95 },
    { 0xec309ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9309b3, 86 },
    { 0, 0 },
    { 0xc9e309b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be309b8, 98 },
    { 0, 0 },
    { 0xc98309ba, 76 },
    { 0, 0 },
    { 0x714309bc, 95 },
    { 0x83e309bd, 98 },
    { 0, 0 },
    { 0x18a309bf, 43 },
    { 0xee2309c0, 55 },
    { 0, 0 },
    { 0x7f8309c2, 33 },
    { 0xb68309c2, 76 },
    { 0x8f6309c4, 66 },
    { 0x257309c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19309c9, 95 },
    { 0, 0 },
    { 0x18309cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949309d0, 75 },
    { 0x98309d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a309d4, 75 },
    { 0xb2e309d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x325309d8, 52 },
    { 0, 0 },
    { 0xbde309da, 55 },
    { 0x950309db, 65 },
    { 0, 0 },
    { 0xb1a309dd, 61 },
    { 0xf39309dd, 33 },
    { 0, 0 },
    { 0x3ad309e0, 33 },
    { 0x690309e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c309f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c309f5, 43 },
    { 0, 0 },
    { 0xd1a309f7, 55 },
    { 0x4b5309f8, 69 },
    { 0xf3f309f9, 61 },
    { 0x241309f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c309ff, 98 },
    { 0x9a1309ff, 98 },
    { 0, 0 },
    { 0x73730a02, 78 },
    { 0xaf830a02, 95 },
    { 0x46a30a04, 23 },
    { 0, 0 },
    { 0xafd30a06, 67 },
    { 0, 0 },
    { 0xa7930a08, 23 },
    { 0x50730a09, 33 },
    { 0x10030a09, 33 },
    { 0xbf330a0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7830a0f, 95 },
    { 0xaa730a10, 23 },
    { 0x1d430a0f, 98 },
    { 0x70830a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba30a15, 43 },
    { 0, 0 },
    { 0x5d30a17, 85 },
    { 0x66730a17, 96 },
    { 0, 0 },
    { 0x19430a1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df30a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76830a28, 23 },
    { 0, 0 },
    { 0x33930a2a, 61 },
    { 0, 0 },
    { 0x6a130a2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7530a2f, 33 },
    { 0, 0 },
    { 0x84b30a31, 52 },
    { 0x65530a32, 23 },
    { 0xdae30a33, 33 },
    { 0xec130a34, 100 },
    { 0xce130a35, 43 },
    { 0xa4e30a35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd30a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd730a3f, 98 },
    { 0, 0 },
    { 0x9b830a41, 88 },
    { 0, 0 },
    { 0xf9430a43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e630a4a, 68 },
    { 0x9fc30a4a, 22 },
    { 0xcf430a4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b30a52, 52 },
    { 0, 0 },
    { 0x77130a54, 33 },
    { 0xa0530a54, 87 },
    { 0xe7f30a56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f30a59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c30a5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c130a62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38630a6a, 76 },
    { 0x6dc30a6b, 69 },
    { 0, 0 },
    { 0x4d230a6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d30a72, 98 },
    { 0, 0 },
    { 0xe7230a74, 33 },
    { 0x44930a75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed30a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda430a87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86130a8b, 90 },
    { 0x82c30a8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b030a93, 23 },
    { 0xfe430a94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b030a9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be30aa1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba30aa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb30aab, 75 },
    { 0x2ea30aac, 90 },
    { 0x64e30aad, 93 },
    { 0xe6530aae, 33 },
    { 0, 0 },
    { 0x4cf30ab0, 81 },
    { 0, 0 },
    { 0x67e30ab2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf30ab5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60630ac0, 95 },
    { 0, 0 },
    { 0xbfe30ac2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94030ac7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e030aca, 46 },
    { 0x58a30aca, 95 },
    { 0, 0 },
    { 0x67730acd, 98 },
    { 0, 0 },
    { 0x37d30acf, 90 },
    { 0x60830ad0, 76 },
    { 0x18530ad1, 90 },
    { 0xf9930ad0, 34 },
    { 0, 0 },
    { 0x25630ad4, 95 },
    { 0x64930ad5, 78 },
    { 0, 0 },
    { 0x9d130ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa30ade, 95 },
    { 0, 0 },
    { 0xb9430ae0, 33 },
    { 0x8d730ae1, 99 },
    { 0, 0 },
    { 0x20630ae3, 23 },
    { 0xa1430ae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc830aea, 93 },
    { 0x5be30aeb, 98 },
    { 0xc8530aec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x96330aef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1030af2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e30af7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a430afc, 33 },
    { 0xc7030afd, 33 },
    { 0, 0 },
    { 0x53130aff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc30b02, 33 },
    { 0, 0 },
    { 0xa4430b04, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c30b08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c930b0e, 23 },
    { 0x11130b0f, 87 },
    { 0x73730b10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83730b14, 98 },
    { 0, 0 },
    { 0x62930b16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32630b1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19130b1e, 85 },
    { 0, 0 },
    { 0x4b630b20, 61 },
    { 0xede30b21, 98 },
    { 0xdf30b22, 66 },
    { 0xa6a30b21, 78 },
    { 0x74730b24, 33 },
    { 0x75630b21, 23 },
    { 0xf5630b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc30b2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa30b34, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b30b3f, 84 },
    { 0x74e30b40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b630b46, 46 },
    { 0x9db30b47, 33 },
    { 0xab230b48, 61 },
    { 0x64730b49, 76 },
    { 0, 0 },
    { 0xa1330b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97930b56, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac30b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c30b60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98630b65, 33 },
    { 0xb2030b66, 90 },
    { 0x25b30b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f30b6b, 23 },
    { 0x6c230b6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4630b6f, 88 },
    { 0x65330b6f, 87 },
    { 0x9c430b71, 76 },
    { 0, 0 },
    { 0xd9a30b73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d630b77, 75 },
    { 0x40930b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91730b7f, 90 },
    { 0xe3730b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed30b84, 84 },
    { 0xbea30b84, 95 },
    { 0, 0 },
    { 0x830b87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a830b8d, 52 },
    { 0xd6f30b8e, 77 },
    { 0xbd530b8f, 95 },
    { 0, 0 },
    { 0x5e330b91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75030b94, 61 },
    { 0xc6430b95, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42030b9c, 90 },
    { 0x77c30b9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb630ba0, 66 },
    { 0xe9e30ba0, 65 },
    { 0x4b230ba2, 95 },
    { 0x33f30ba2, 34 },
    { 0x75030ba3, 90 },
    { 0, 0 },
    { 0x3ee30ba6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c930baa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2230bae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc30bb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d30bb5, 90 },
    { 0x10a30bb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c30bba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e30bc0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a630bc8, 45 },
    { 0xd9830bc8, 43 },
    { 0x6530bca, 100 },
    { 0xe1d30bcb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da30bd0, 76 },
    { 0, 0 },
    { 0x4b930bd2, 84 },
    { 0x8ff30bd3, 61 },
    { 0x49a30bd3, 33 },
    { 0x94330bd4, 98 },
    { 0x63230bd6, 23 },
    { 0x42b30bd7, 98 },
    { 0, 0 },
    { 0xeac30bd9, 55 },
    { 0, 0 },
    { 0x1130bdb, 90 },
    { 0x90530bdc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xde30bdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9330be3, 36 },
    { 0x5bc30be4, 95 },
    { 0x7b630be4, 95 },
    { 0xb30be3, 98 },
    { 0x61330be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa130beb, 78 },
    { 0x63030bec, 33 },
    { 0x91430bed, 90 },
    { 0xac630bed, 90 },
    { 0x82130bef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68930bf5, 23 },
    { 0, 0 },
    { 0xae430bf7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea930bfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa30c04, 94 },
    { 0xc0f30c04, 76 },
    { 0x33f30c04, 98 },
    { 0xc8a30c05, 93 },
    { 0x13330c08, 81 },
    { 0x17030c09, 66 },
    { 0xca230c0a, 33 },
    { 0x7bb30c0a, 33 },
    { 0, 0 },
    { 0xe9230c0d, 65 },
    { 0xafc30c0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c230c1a, 52 },
    { 0, 0 },
    { 0x4530c1c, 78 },
    { 0x18430c1d, 34 },
    { 0xe9a30c1e, 65 },
    { 0xc2e30c1f, 93 },
    { 0x2b730c1e, 96 },
    { 0xec430c20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c230c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8330c2c, 61 },
    { 0x33730c2d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f430c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea30c3f, 93 },
    { 0, 0 },
    { 0x17b30c41, 66 },
    { 0, 0 },
    { 0x18830c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4230c4a, 61 },
    { 0, 0 },
    { 0x59b30c4c, 95 },
    { 0xcc630c4d, 100 },
    { 0xef930c4e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x16630c51, 95 },
    { 0, 0 },
    { 0xaee30c53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91630c56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee930c59, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56130c5d, 90 },
    { 0, 0 },
    { 0x7c830c5f, 95 },
    { 0xf2830c5f, 75 },
    { 0, 0 },
    { 0xc3530c62, 23 },
    { 0x5a330c62, 78 },
    { 0xbc430c64, 17 },
    { 0x1e830c65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16630c69, 33 },
    { 0, 0 },
    { 0xb3130c6b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e30c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c30c75, 33 },
    { 0, 0 },
    { 0xd9e30c77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc30c7b, 98 },
    { 0x43630c7c, 77 },
    { 0, 0 },
    { 0xf6830c7e, 23 },
    { 0x95730c7f, 43 },
    { 0, 0 },
    { 0x1df30c81, 33 },
    { 0, 0 },
    { 0x65530c83, 61 },
    { 0x33230c84, 100 },
    { 0x21c30c85, 23 },
    { 0x87a30c83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc930c8a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb30c8d, 100 },
    { 0xa9830c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f030c91, 78 },
    { 0, 0 },
    { 0xc3b30c93, 23 },
    { 0, 0 },
    { 0x17e30c95, 93 },
    { 0xb1d30c96, 95 },
    { 0, 0 },
    { 0xf5030c98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e30c9d, 91 },
    { 0xeeb30c9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb30ca1, 100 },
    { 0, 0 },
    { 0x37630ca3, 95 },
    { 0xbe30ca4, 33 },
    { 0x5e130ca5, 9 },
    { 0x80d30ca6, 40 },
    { 0xd3a30ca3, 90 },
    { 0x8e830ca8, 98 },
    { 0xd4030ca4, 61 },
    { 0x4a130ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf630caf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87430cba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6930cbd, 33 },
    { 0, 0 },
    { 0x78430cbf, 23 },
    { 0x9f030cc0, 95 },
    { 0xb6c30cc1, 33 },
    { 0x72930cc2, 46 },
    { 0x8ae30cc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59030cc9, 23 },
    { 0xb0330cca, 76 },
    { 0x2130cca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b30cd0, 85 },
    { 0, 0 },
    { 0x4b730cd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c30cd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10330cdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1930ce2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64730ce6, 40 },
    { 0, 0 },
    { 0xa8e30ce8, 33 },
    { 0x33430ce8, 46 },
    { 0x85330ce8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71130cee, 66 },
    { 0, 0 },
    { 0xab730cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26830cf6, 61 },
    { 0x12130cf7, 43 },
    { 0xbf630cf8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f730cfb, 76 },
    { 0, 0 },
    { 0x2a730cfd, 95 },
    { 0, 0 },
    { 0x7dc30cff, 61 },
    { 0x1f330d00, 76 },
    { 0x50d30d00, 95 },
    { 0, 0 },
    { 0xce830d03, 33 },
    { 0x4b30d04, 90 },
    { 0x5b330d04, 33 },
    { 0x3a130d06, 33 },
    { 0xc9030d04, 90 },
    { 0xe9b30d08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa30d0b, 95 },
    { 0xd4b30d0b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc30d12, 23 },
    { 0xc3f30d12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35830d17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d230d21, 87 },
    { 0xdf230d22, 36 },
    { 0x23430d22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62530d28, 90 },
    { 0x6030d29, 90 },
    { 0xdb230d2a, 78 },
    { 0xe230d2b, 46 },
    { 0x64a30d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f30d30, 98 },
    { 0, 0 },
    { 0xcc630d32, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11730d36, 23 },
    { 0x35930d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e630d3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39530d46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a30d4a, 75 },
    { 0x3aa30d4a, 33 },
    { 0x66e30d4a, 46 },
    { 0x56330d4d, 88 },
    { 0x8fb30d4d, 90 },
    { 0x35830d4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84130d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf30d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b30d5b, 17 },
    { 0x6c730d5c, 95 },
    { 0xc7a30d5d, 98 },
    { 0x7c130d5e, 69 },
    { 0xc130d5e, 91 },
    { 0, 0 },
    { 0x73e30d61, 91 },
    { 0, 0 },
    { 0xf0830d63, 100 },
    { 0x54530d64, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70530d69, 84 },
    { 0x55430d69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac30d6d, 87 },
    { 0, 0 },
    { 0xe5c30d6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59330d73, 99 },
    { 0xd9c30d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff30d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c30d80, 66 },
    { 0xb3c30d81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa30d87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3430d8d, 76 },
    { 0x39530d8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8530d93, 84 },
    { 0, 0 },
    { 0x98130d95, 33 },
    { 0xfd30d95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d30d9a, 90 },
    { 0, 0 },
    { 0x38930d9c, 99 },
    { 0, 0 },
    { 0x11230d9e, 84 },
    { 0xbff30d9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e30da2, 34 },
    { 0xcae30da3, 23 },
    { 0, 0 },
    { 0xdb630da5, 77 },
    { 0x44c30da5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e30da9, 40 },
    { 0, 0 },
    { 0x56130dab, 55 },
    { 0x27930dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19530db0, 33 },
    { 0, 0 },
    { 0x29a30db2, 98 },
    { 0x3230db3, 95 },
    { 0x86330db4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3330dba, 91 },
    { 0, 0 },
    { 0x10430dbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30730dbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc630dc7, 33 },
    { 0xbd430dc8, 34 },
    { 0x3230dc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd30dd0, 90 },
    { 0, 0 },
    { 0xd8930dd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa30dd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f30dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff30de2, 95 },
    { 0x10230de3, 76 },
    { 0x95530de4, 23 },
    { 0xc0f30de3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be30deb, 98 },
    { 0x1f830dec, 90 },
    { 0x6330ded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7930df0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9430df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee30dfb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd30dfe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8130e03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e30e07, 90 },
    { 0x4ff30e08, 76 },
    { 0x99330e07, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74630e0d, 61 },
    { 0x12130e0e, 67 },
    { 0x6e530e0f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9830e12, 55 },
    { 0x14530e12, 85 },
    { 0xc3530e14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25330e1c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x28930e1f, 33 },
    { 0x55530e20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75430e24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa30e27, 61 },
    { 0xe2830e28, 90 },
    { 0x40a30e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa30e2d, 90 },
    { 0x27030e2d, 33 },
    { 0, 0 },
    { 0x7c530e30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b530e3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8430e42, 98 },
    { 0xbc930e43, 90 },
    { 0x76c30e42, 95 },
    { 0xf8830e43, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d30e4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc30e4f, 76 },
    { 0, 0 },
    { 0xe6e30e51, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f430e54, 77 },
    { 0x3cb30e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad630e58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd30e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb30e60, 33 },
    { 0, 0 },
    { 0x9e630e62, 99 },
    { 0, 0 },
    { 0x9d930e64, 76 },
    { 0x12f30e64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5130e6a, 91 },
    { 0, 0 },
    { 0xa0b30e6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb30e73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2830e7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4930e7e, 76 },
    { 0xe4b30e7e, 76 },
    { 0x69530e7e, 93 },
    { 0x12830e81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1230e85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85930e8a, 98 },
    { 0, 0 },
    { 0x17f30e8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e30e93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd30e96, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x23830e99, 95 },
    { 0x7d130e99, 90 },
    { 0, 0 },
    { 0xe0430e9c, 77 },
    { 0, 0 },
    { 0x3330e9e, 23 },
    { 0xcea30e9f, 76 },
    { 0xc7d30ea0, 100 },
    { 0xea230ea0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c30ea4, 23 },
    { 0x45d30ea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2430ea8, 90 },
    { 0x4f30ea9, 46 },
    { 0, 0 },
    { 0xcb030eab, 100 },
    { 0x1a30eab, 95 },
    { 0x5ab30ead, 90 },
    { 0xade30ead, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d30eb6, 33 },
    { 0, 0 },
    { 0xba130eb8, 78 },
    { 0xdd330eb9, 93 },
    { 0x3e630eb9, 98 },
    { 0x2eb30ebb, 55 },
    { 0xe1830ebc, 23 },
    { 0x2430ebd, 98 },
    { 0x4e330ebe, 34 },
    { 0, 0 },
    { 0x5d730ec0, 61 },
    { 0x5230ec0, 67 },
    { 0, 0 },
    { 0x6ec30ec3, 90 },
    { 0x81130ec4, 90 },
    { 0x4cc30ec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6630ec9, 95 },
    { 0xfb30ec9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd30ece, 76 },
    { 0x12930ece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d30ed4, 98 },
    { 0, 0 },
    { 0x32c30ed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc30edb, 33 },
    { 0, 0 },
    { 0xe7230edd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2930ee5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1230ee8, 66 },
    { 0, 0 },
    { 0xdda30eea, 43 },
    { 0xf5030eea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c30ef2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31430ef8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf30efb, 98 },
    { 0xaab30efb, 76 },
    { 0, 0 },
    { 0x3c230efe, 56 },
    { 0x97030eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f30f04, 97 },
    { 0xee130f05, 94 },
    { 0, 0 },
    { 0x49b30f07, 91 },
    { 0xadc30f07, 90 },
    { 0, 0 },
    { 0xa4830f0a, 55 },
    { 0xec630f0a, 66 },
    { 0, 0 },
    { 0xc7730f0d, 78 },
    { 0x71030f0e, 61 },
    { 0x19630f0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae630f15, 95 },
    { 0, 0 },
    { 0x2b930f17, 46 },
    { 0x54130f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb130f1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90330f20, 23 },
    { 0, 0 },
    { 0x89930f22, 55 },
    { 0x73430f23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c330f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2930f2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c30f2e, 33 },
    { 0x6f30f2f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x15730f32, 43 },
    { 0x14b30f33, 95 },
    { 0, 0 },
    { 0x59430f35, 95 },
    { 0xde30f36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16230f40, 95 },
    { 0x49830f41, 78 },
    { 0, 0 },
    { 0x37830f43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e30f46, 98 },
    { 0x49d30f46, 91 },
    { 0xb1330f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2930f4b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5930f4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c30f51, 90 },
    { 0x23630f51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba930f59, 81 },
    { 0, 0 },
    { 0x12330f5b, 95 },
    { 0, 0 },
    { 0xd3130f5d, 23 },
    { 0, 0 },
    { 0xd0330f5f, 40 },
    { 0x9c930f60, 98 },
    { 0xfc430f61, 98 },
    { 0xf1f30f60, 76 },
    { 0x60930f63, 90 },
    { 0x4d30f63, 46 },
    { 0xa5230f65, 61 },
    { 0x1030f60, 60 },
    { 0x8230f67, 95 },
    { 0x79630f62, 23 },
    { 0, 0 },
    { 0xf9930f6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1530f70, 98 },
    { 0xe2430f70, 61 },
    { 0x7ce30f70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46430f7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b30f7d, 98 },
    { 0x3ee30f7e, 95 },
    { 0xf4e30f7f, 33 },
    { 0x4a30f80, 81 },
    { 0xf1930f7d, 90 },
    { 0x58d30f82, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630f88, 83 },
    { 0x4630f89, 98 },
    { 0x5d230f89, 52 },
    { 0x40930f8b, 90 },
    { 0, 0 },
    { 0xb0330f8d, 90 },
    { 0xde230f8e, 93 },
    { 0xa0930f8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a430f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7430f95, 52 },
    { 0x3c330f95, 43 },
    { 0x4b30f97, 95 },
    { 0xd8b30f97, 78 },
    { 0xebe30f98, 50 },
    { 0xc2630f9a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9530fa0, 43 },
    { 0xd5e30fa0, 76 },
    { 0, 0 },
    { 0x73130fa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6530fa6, 98 },
    { 0xe8d30fa7, 76 },
    { 0x4530fa6, 73 },
    { 0x74c30fa7, 65 },
    { 0x26630faa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a30fb2, 23 },
    { 0, 0 },
    { 0x8c730fb4, 95 },
    { 0, 0 },
    { 0xd1930fb6, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b30fbb, 90 },
    { 0x31630fbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b30fbf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71430fc8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d30fcc, 61 },
    { 0, 0 },
    { 0xb8e30fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5230fd2, 34 },
    { 0x95230fd3, 76 },
    { 0x2f730fd4, 95 },
    { 0xe7330fd5, 23 },
    { 0xca330fd5, 66 },
    { 0x3530fd7, 61 },
    { 0, 0 },
    { 0x19130fd9, 34 },
    { 0x43630fda, 90 },
    { 0xa5430fda, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3630fe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a30fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef30fea, 23 },
    { 0x14b30fea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad30ff1, 76 },
    { 0x85130ff1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc30ff6, 55 },
    { 0, 0 },
    { 0x7a030ff8, 95 },
    { 0, 0 },
    { 0x2f730ffa, 81 },
    { 0x17430ffb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c031001, 61 },
    { 0x28531001, 48 },
    { 0x9ea31003, 90 },
    { 0, 0 },
    { 0x78531005, 33 },
    { 0xe331006, 23 },
    { 0, 0 },
    { 0x6ed31008, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a3100b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5353100f, 64 },
    { 0xf0031010, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc31015, 90 },
    { 0xa7931016, 77 },
    { 0x25831017, 61 },
    { 0, 0 },
    { 0x91031019, 98 },
    { 0xa7431019, 98 },
    { 0xefe31019, 90 },
    { 0x3033101c, 34 },
    { 0x1473101a, 61 },
    { 0xc0a3101d, 90 },
    { 0x21c3101f, 33 },
    { 0xc7a31020, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a31025, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea31028, 99 },
    { 0x2ae31029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3102c, 90 },
    { 0, 0 },
    { 0x183102e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9031031, 76 },
    { 0xc5631032, 90 },
    { 0xd3831031, 76 },
    { 0x4a231033, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13231038, 100 },
    { 0, 0 },
    { 0x1213103a, 34 },
    { 0x75f3103b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39531041, 46 },
    { 0x1831041, 33 },
    { 0, 0 },
    { 0xba931044, 65 },
    { 0xf7731045, 90 },
    { 0x57931045, 61 },
    { 0xed531047, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f3104a, 98 },
    { 0x4693104a, 83 },
    { 0x92d3104b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d31051, 95 },
    { 0, 0 },
    { 0xcef31053, 75 },
    { 0, 0 },
    { 0xadf31055, 84 },
    { 0x23831056, 90 },
    { 0x2131057, 90 },
    { 0, 0 },
    { 0xd8331059, 76 },
    { 0xf5b3105a, 88 },
    { 0x99f3105b, 99 },
    { 0x76b3105a, 95 },
    { 0x9383105b, 33 },
    { 0, 0 },
    { 0x2db3105f, 17 },
    { 0xb1831060, 95 },
    { 0xb583105f, 76 },
    { 0x73931062, 23 },
    { 0x76a31061, 90 },
    { 0x5e531060, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb3106a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3583106d, 84 },
    { 0x6af3106e, 90 },
    { 0, 0 },
    { 0x7f931070, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b31074, 87 },
    { 0x95531074, 45 },
    { 0, 0 },
    { 0x4a31077, 40 },
    { 0xbc31078, 90 },
    { 0x51b31079, 95 },
    { 0xadd3107a, 98 },
    { 0x4ee31079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5031080, 23 },
    { 0, 0 },
    { 0xe1b31082, 98 },
    { 0x4e531083, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c31086, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4153108c, 77 },
    { 0x5f93108d, 91 },
    { 0x7813108e, 91 },
    { 0x9663108f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff31095, 95 },
    { 0xb1a31096, 64 },
    { 0x931097, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e3109c, 98 },
    { 0x9443109d, 100 },
    { 0, 0 },
    { 0x623109f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55310a7, 43 },
    { 0xa5b310a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea310ad, 78 },
    { 0x5a0310ad, 33 },
    { 0, 0 },
    { 0x8ba310b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb310b4, 90 },
    { 0x72e310b5, 61 },
    { 0, 0 },
    { 0xa41310b7, 100 },
    { 0x270310b8, 52 },
    { 0x591310b9, 90 },
    { 0, 0 },
    { 0x7ce310bb, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60310be, 55 },
    { 0x812310be, 61 },
    { 0x126310c0, 67 },
    { 0xb310c0, 90 },
    { 0, 0 },
    { 0x967310c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c310c6, 61 },
    { 0x48f310c7, 33 },
    { 0x206310c8, 90 },
    { 0x599310c8, 68 },
    { 0xc97310ca, 46 },
    { 0x754310cb, 52 },
    { 0, 0 },
    { 0x3e2310cd, 78 },
    { 0x410310ce, 64 },
    { 0x649310cf, 55 },
    { 0xd55310d0, 56 },
    { 0xb47310cd, 98 },
    { 0, 0 },
    { 0x6f0310d3, 93 },
    { 0, 0 },
    { 0xddc310d5, 69 },
    { 0, 0 },
    { 0xef1310d7, 55 },
    { 0x52310d8, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e310db, 97 },
    { 0x562310dc, 98 },
    { 0x521310dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6310e1, 17 },
    { 0x57c310e2, 33 },
    { 0x6e0310e2, 78 },
    { 0, 0 },
    { 0xfd6310e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e310e9, 100 },
    { 0, 0 },
    { 0x417310eb, 61 },
    { 0x2c4310ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419310f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc310fa, 34 },
    { 0x76d310fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f310fe, 46 },
    { 0x756310ff, 61 },
    { 0, 0 },
    { 0xe331101, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e431105, 46 },
    { 0x7d131106, 98 },
    { 0, 0 },
    { 0x2a531108, 78 },
    { 0x1e231109, 95 },
    { 0xa323110a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc853110d, 23 },
    { 0x19d3110d, 97 },
    { 0xf4a3110d, 47 },
    { 0xbb531110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf31115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11431118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f3111b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f31125, 90 },
    { 0, 0 },
    { 0x1a631127, 90 },
    { 0xa8831128, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a3112d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0031132, 95 },
    { 0x33431133, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xad631136, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e31139, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde73113d, 43 },
    { 0, 0 },
    { 0x2603113f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb31143, 90 },
    { 0x6ab31143, 95 },
    { 0x9f931144, 99 },
    { 0xcf631146, 76 },
    { 0xcee31147, 90 },
    { 0, 0 },
    { 0x6ba31149, 34 },
    { 0x6b83114a, 76 },
    { 0, 0 },
    { 0x67d3114c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55331155, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff731159, 95 },
    { 0x8ee31159, 90 },
    { 0, 0 },
    { 0xc883115c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023115f, 61 },
    { 0, 0 },
    { 0xdc831161, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33431165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc83116d, 67 },
    { 0x5963116e, 61 },
    { 0x9b03116e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f731174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5031178, 98 },
    { 0x6ba31179, 17 },
    { 0x50931179, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc663117d, 23 },
    { 0xcad3117e, 80 },
    { 0xd8e3117f, 61 },
    { 0xa6e3117f, 61 },
    { 0xf663117f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b31184, 98 },
    { 0x3f931184, 95 },
    { 0xef531185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca31189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f3118d, 34 },
    { 0x9513118e, 55 },
    { 0, 0 },
    { 0xed631190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb31193, 33 },
    { 0x73531194, 23 },
    { 0x51931195, 99 },
    { 0x51631193, 52 },
    { 0x5bc31197, 98 },
    { 0x3c731198, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3119f, 90 },
    { 0x3b5311a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c311a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d311ac, 76 },
    { 0xb66311ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d311b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a311b6, 55 },
    { 0, 0 },
    { 0x19e311b8, 90 },
    { 0xeff311b9, 33 },
    { 0, 0 },
    { 0x49a311bb, 55 },
    { 0, 0 },
    { 0x1b2311bd, 87 },
    { 0x26d311be, 76 },
    { 0x5f0311bf, 33 },
    { 0x58c311be, 61 },
    { 0x35c311c0, 23 },
    { 0xa16311c0, 55 },
    { 0xde8311bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53311c6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2311cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c311d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d311d4, 76 },
    { 0x19f311d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551311dc, 33 },
    { 0, 0 },
    { 0x503311de, 33 },
    { 0x17f311df, 33 },
    { 0xf1f311e0, 90 },
    { 0xaa1311e1, 85 },
    { 0x1e6311e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5311e5, 95 },
    { 0, 0 },
    { 0x254311e7, 33 },
    { 0xca6311e7, 33 },
    { 0, 0 },
    { 0x2ca311ea, 34 },
    { 0, 0 },
    { 0x973311ec, 43 },
    { 0xdc7311ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc311f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39311f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df311fd, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0331201, 55 },
    { 0x631202, 90 },
    { 0, 0 },
    { 0x3b631204, 81 },
    { 0, 0 },
    { 0xc8531206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a31209, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d431210, 90 },
    { 0x1ac31211, 34 },
    { 0x4f331211, 93 },
    { 0, 0 },
    { 0xa0231214, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a31217, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf3121b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123121f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x53531222, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63831225, 34 },
    { 0x9a231225, 34 },
    { 0x96231225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8953122a, 34 },
    { 0xea63122b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e3122e, 33 },
    { 0x8853122f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f31235, 95 },
    { 0, 0 },
    { 0x59331237, 90 },
    { 0x2131238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3123d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f931247, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3124d, 23 },
    { 0x2543124e, 61 },
    { 0xb093124e, 90 },
    { 0xdcf3124f, 95 },
    { 0, 0 },
    { 0x6e531252, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6831256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3923125c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f3125f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23331265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6531269, 23 },
    { 0x42c3126a, 60 },
    { 0, 0 },
    { 0x2e43126c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa31270, 33 },
    { 0x21231271, 46 },
    { 0x44931271, 90 },
    { 0, 0 },
    { 0xfcf31274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e3127f, 65 },
    { 0xa0631280, 98 },
    { 0, 0 },
    { 0x81131282, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d31285, 77 },
    { 0, 0 },
    { 0xa1a31287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f93128b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30231292, 33 },
    { 0xebc31293, 52 },
    { 0, 0 },
    { 0x9b231295, 90 },
    { 0xce531295, 66 },
    { 0xc2d31295, 90 },
    { 0xbee31298, 75 },
    { 0x1e631298, 81 },
    { 0, 0 },
    { 0x9c33129b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3129e, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef312a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810312a6, 85 },
    { 0, 0 },
    { 0xccd312a8, 90 },
    { 0, 0 },
    { 0x777312aa, 90 },
    { 0, 0 },
    { 0x92b312ac, 61 },
    { 0xf9b312ad, 61 },
    { 0xc5d312ae, 61 },
    { 0, 0 },
    { 0x2c4312b0, 33 },
    { 0x821312b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b312b6, 87 },
    { 0x4d5312b7, 73 },
    { 0x55312b8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0312bf, 55 },
    { 0xa84312bf, 100 },
    { 0x4ba312c1, 61 },
    { 0xbb9312c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6312c6, 90 },
    { 0, 0 },
    { 0x925312c8, 95 },
    { 0xb25312c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86312d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849312da, 43 },
    { 0x930312da, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd312de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d312e1, 93 },
    { 0, 0 },
    { 0x38a312e3, 23 },
    { 0xfaf312e3, 43 },
    { 0x341312e5, 87 },
    { 0xe63312e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912312eb, 80 },
    { 0x902312ec, 53 },
    { 0xfc3312eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234312f1, 61 },
    { 0x6e2312f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2312f6, 23 },
    { 0, 0 },
    { 0x252312f8, 95 },
    { 0, 0 },
    { 0x980312fa, 61 },
    { 0xf36312fb, 61 },
    { 0x63f312fb, 95 },
    { 0xa16312fd, 95 },
    { 0x3f8312fe, 84 },
    { 0x512312ff, 61 },
    { 0x70331300, 84 },
    { 0x4f131300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a31304, 90 },
    { 0x1ae31305, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53031310, 33 },
    { 0xd7431311, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91931315, 95 },
    { 0x6f031315, 90 },
    { 0, 0 },
    { 0x89e31318, 55 },
    { 0x45f31319, 77 },
    { 0, 0 },
    { 0xd203131b, 61 },
    { 0xe343131c, 93 },
    { 0x6303131d, 69 },
    { 0x8673131c, 61 },
    { 0x3483131f, 95 },
    { 0xdff3131f, 23 },
    { 0x8b931320, 90 },
    { 0x4513131f, 64 },
    { 0x8c13131e, 95 },
    { 0, 0 },
    { 0x6a031325, 78 },
    { 0, 0 },
    { 0xd8731327, 90 },
    { 0, 0 },
    { 0x32731329, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee3132e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x56131331, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd931336, 88 },
    { 0x83831336, 85 },
    { 0xdcd31338, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53133b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d31340, 33 },
    { 0x48031340, 55 },
    { 0xd0331342, 23 },
    { 0x89931340, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e31348, 33 },
    { 0, 0 },
    { 0x62c3134a, 92 },
    { 0x1c93134a, 23 },
    { 0x6bd3134c, 52 },
    { 0xab23134d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b31354, 42 },
    { 0xe5c31355, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6423135a, 34 },
    { 0x1193135b, 33 },
    { 0, 0 },
    { 0xd4a3135d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98531361, 80 },
    { 0x64131361, 100 },
    { 0xcb031363, 85 },
    { 0, 0 },
    { 0x7d31365, 93 },
    { 0, 0 },
    { 0x7e131367, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33136a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e31370, 61 },
    { 0x6e131370, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f931374, 66 },
    { 0x74731375, 95 },
    { 0x93d31375, 55 },
    { 0x1b231377, 34 },
    { 0xeaf31377, 97 },
    { 0, 0 },
    { 0x1be3137a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b53137f, 66 },
    { 0, 0 },
    { 0x11431381, 98 },
    { 0x51931381, 33 },
    { 0, 0 },
    { 0x93d31384, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7463138d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22531396, 95 },
    { 0x89831397, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa413139a, 34 },
    { 0, 0 },
    { 0x3c93139c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe573139f, 60 },
    { 0, 0 },
    { 0xbd9313a1, 67 },
    { 0x8f4313a2, 65 },
    { 0x9b3313a3, 55 },
    { 0xdfb313a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2313af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28313b3, 90 },
    { 0x313313b3, 66 },
    { 0x415313b4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1313bd, 55 },
    { 0xc4b313be, 33 },
    { 0, 0 },
    { 0x944313c0, 95 },
    { 0, 0 },
    { 0xeaa313c2, 90 },
    { 0, 0 },
    { 0xe80313c4, 90 },
    { 0xd67313c5, 95 },
    { 0x6fb313c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x557313c9, 76 },
    { 0x74e313ca, 55 },
    { 0xc60313ca, 76 },
    { 0x2c6313ca, 95 },
    { 0xa22313cd, 61 },
    { 0xd07313cb, 75 },
    { 0xdca313cc, 33 },
    { 0xf63313ca, 24 },
    { 0x7bc313cb, 61 },
    { 0x240313cb, 17 },
    { 0, 0 },
    { 0x2af313d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de313d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88313db, 99 },
    { 0x78f313db, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a313e3, 95 },
    { 0x9c8313e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48313eb, 76 },
    { 0xdc313eb, 90 },
    { 0x8b0313ed, 23 },
    { 0xe73313ee, 92 },
    { 0xdb9313eb, 84 },
    { 0x404313f0, 43 },
    { 0x67c313f1, 23 },
    { 0, 0 },
    { 0x88f313f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d313fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x906313fe, 95 },
    { 0x81f313ff, 98 },
    { 0xb48313fe, 34 },
    { 0x4b9313fe, 47 },
    { 0xb331402, 23 },
    { 0x1a031403, 98 },
    { 0x79431402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96731410, 23 },
    { 0, 0 },
    { 0x1c831412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47231416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe423141e, 67 },
    { 0x4693141f, 46 },
    { 0x87f3141f, 61 },
    { 0, 0 },
    { 0xa3031422, 91 },
    { 0xd5331422, 90 },
    { 0x87731424, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc831428, 61 },
    { 0x63e31429, 43 },
    { 0xa0631429, 90 },
    { 0x36431428, 36 },
    { 0x21f31428, 87 },
    { 0x9803142d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e31432, 61 },
    { 0x7b131433, 76 },
    { 0x25531433, 23 },
    { 0xaff31435, 33 },
    { 0x75231436, 34 },
    { 0xb7331437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c31440, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43031445, 65 },
    { 0x49831445, 90 },
    { 0, 0 },
    { 0xbd431448, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab3144b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5593144e, 95 },
    { 0x1373144f, 91 },
    { 0xba231450, 61 },
    { 0x5431451, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9473145e, 33 },
    { 0x1963145f, 88 },
    { 0x5413145f, 44 },
    { 0xaea31460, 23 },
    { 0xf3d3145f, 23 },
    { 0x72d31462, 33 },
    { 0, 0 },
    { 0x4dd31465, 99 },
    { 0, 0 },
    { 0xa1c31467, 90 },
    { 0, 0 },
    { 0x6f231469, 61 },
    { 0, 0 },
    { 0x43e3146b, 43 },
    { 0x9453146b, 43 },
    { 0, 0 },
    { 0x223146e, 61 },
    { 0x7a13146f, 53 },
    { 0xd103146f, 78 },
    { 0, 0 },
    { 0x331472, 100 },
    { 0x39a31473, 33 },
    { 0xde931473, 34 },
    { 0, 0 },
    { 0xead31476, 84 },
    { 0xcb731477, 67 },
    { 0xf1e31478, 43 },
    { 0x68f31479, 75 },
    { 0xa7d3147a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a43147e, 76 },
    { 0xb533147f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f31482, 95 },
    { 0x51831482, 66 },
    { 0x48431484, 34 },
    { 0x4bd31485, 91 },
    { 0x9da31486, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc73148b, 61 },
    { 0x12f3148c, 58 },
    { 0xf843148d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a31490, 33 },
    { 0, 0 },
    { 0x39731492, 66 },
    { 0, 0 },
    { 0x15831494, 61 },
    { 0, 0 },
    { 0xce931496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd603149a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc3149f, 61 },
    { 0xc4314a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d314a6, 61 },
    { 0x884314a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac314ad, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde314b1, 43 },
    { 0x6fb314b1, 34 },
    { 0xb27314b1, 90 },
    { 0xc3d314b4, 76 },
    { 0xf0e314b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7314bd, 46 },
    { 0x4ad314be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5314c3, 65 },
    { 0xc14314c4, 23 },
    { 0x98314c5, 55 },
    { 0x967314c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e314cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x857314ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1314d1, 61 },
    { 0x92b314d1, 23 },
    { 0, 0 },
    { 0x881314d4, 46 },
    { 0xe17314d5, 90 },
    { 0xa12314d6, 40 },
    { 0x340314d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e314dd, 23 },
    { 0xac3314dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a314e3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3314e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788314eb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4314ef, 33 },
    { 0x78f314ef, 34 },
    { 0x993314f1, 55 },
    { 0xe65314f2, 68 },
    { 0x633314f2, 91 },
    { 0x5cb314f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca314f8, 98 },
    { 0, 0 },
    { 0x542314fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb314fd, 61 },
    { 0x647314fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd731503, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a31507, 99 },
    { 0x4d331507, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c63150b, 98 },
    { 0xab53150c, 90 },
    { 0x4e83150b, 66 },
    { 0xff53150b, 33 },
    { 0x8e93150c, 36 },
    { 0xa3531510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e531513, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103151e, 84 },
    { 0x1483151e, 100 },
    { 0, 0 },
    { 0xec631521, 76 },
    { 0xb2731522, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8231525, 23 },
    { 0, 0 },
    { 0xa2731527, 76 },
    { 0x87731528, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb13152d, 95 },
    { 0, 0 },
    { 0xc4a3152f, 61 },
    { 0, 0 },
    { 0x14e31531, 75 },
    { 0xb1b31532, 84 },
    { 0x8f531533, 90 },
    { 0x7d531531, 69 },
    { 0xc8f31532, 76 },
    { 0, 0 },
    { 0x91031537, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3153a, 84 },
    { 0x8623153b, 92 },
    { 0xdaa3153b, 61 },
    { 0xcec3153c, 93 },
    { 0, 0 },
    { 0xe043153f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19131548, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x303154b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9253154e, 90 },
    { 0x2b83154e, 67 },
    { 0x80431550, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10131557, 90 },
    { 0xee231557, 65 },
    { 0xc0031557, 64 },
    { 0x68d3155a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf3155d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e31561, 23 },
    { 0x31c31561, 92 },
    { 0x4aa31562, 90 },
    { 0xa31561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f31568, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d3156c, 34 },
    { 0xf893156c, 84 },
    { 0x6443156d, 78 },
    { 0x32e3156f, 95 },
    { 0x2913156c, 90 },
    { 0xee13156c, 90 },
    { 0x2a531571, 90 },
    { 0x91931573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb31577, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d3157a, 75 },
    { 0x73e3157b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd3157e, 100 },
    { 0xced3157e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e31585, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de31588, 33 },
    { 0x90f31589, 24 },
    { 0xee731589, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd03158d, 33 },
    { 0, 0 },
    { 0xeb3158f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1631593, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b31599, 76 },
    { 0, 0 },
    { 0xeb93159b, 90 },
    { 0, 0 },
    { 0x1b73159d, 36 },
    { 0, 0 },
    { 0xe923159f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d315a3, 95 },
    { 0, 0 },
    { 0x4ef315a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d315a9, 61 },
    { 0x6f3315aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa315af, 84 },
    { 0, 0 },
    { 0x8a5315b1, 98 },
    { 0xb50315b2, 90 },
    { 0x854315b1, 55 },
    { 0x99d315b4, 84 },
    { 0xaf2315b5, 61 },
    { 0x935315b1, 33 },
    { 0x13d315b7, 81 },
    { 0xba7315b8, 98 },
    { 0xce0315b9, 75 },
    { 0xd38315b9, 43 },
    { 0x4a1315b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5315c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e315c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a315c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3315ca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef315d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c315d9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7315df, 68 },
    { 0x429315e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7315e3, 76 },
    { 0x887315e4, 43 },
    { 0xd37315e5, 33 },
    { 0xc08315e6, 75 },
    { 0x338315e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d315ea, 91 },
    { 0, 0 },
    { 0x5d8315ec, 90 },
    { 0x889315ec, 95 },
    { 0xa6b315ec, 95 },
    { 0xe3315ef, 77 },
    { 0x8ce315f0, 98 },
    { 0x374315f1, 52 },
    { 0x173315f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7315fa, 23 },
    { 0xc21315fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd931600, 61 },
    { 0x5e731601, 95 },
    { 0x5ad31602, 34 },
    { 0, 0 },
    { 0xf2e31604, 83 },
    { 0, 0 },
    { 0xec831606, 76 },
    { 0xe5b31607, 93 },
    { 0xd4d31608, 23 },
    { 0xedc31609, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853160f, 91 },
    { 0xf973160f, 98 },
    { 0, 0 },
    { 0x74c31612, 77 },
    { 0xd0531612, 98 },
    { 0xace31612, 33 },
    { 0, 0 },
    { 0x1f331616, 75 },
    { 0, 0 },
    { 0xe9131618, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8753161c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93731620, 23 },
    { 0, 0 },
    { 0x68231622, 33 },
    { 0, 0 },
    { 0x3de31624, 55 },
    { 0x3e131625, 33 },
    { 0x18131626, 95 },
    { 0x831627, 61 },
    { 0, 0 },
    { 0xb0931629, 96 },
    { 0, 0 },
    { 0x2783162b, 90 },
    { 0x4ad3162c, 61 },
    { 0xa783162d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb31630, 33 },
    { 0xbbd31631, 33 },
    { 0x96b31632, 85 },
    { 0xddb31633, 95 },
    { 0x67d31633, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4231637, 55 },
    { 0, 0 },
    { 0xe1c31639, 88 },
    { 0x6a93163a, 90 },
    { 0xb7d3163b, 33 },
    { 0xe8d3163c, 90 },
    { 0x7cd3163c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc731640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e031643, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d31646, 23 },
    { 0xff731646, 95 },
    { 0x3bb31646, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5343164c, 76 },
    { 0x5763164d, 86 },
    { 0xd33164c, 98 },
    { 0xbfe3164f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0831652, 76 },
    { 0x9831653, 33 },
    { 0x1c231654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9231658, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c431660, 98 },
    { 0, 0 },
    { 0x98c31662, 61 },
    { 0x71731663, 23 },
    { 0xa6831663, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2293166a, 95 },
    { 0x16c3166b, 33 },
    { 0x75e3166c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7383166f, 61 },
    { 0xced3166f, 33 },
    { 0, 0 },
    { 0xa7b31672, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c031678, 61 },
    { 0, 0 },
    { 0xf663167a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe513167d, 98 },
    { 0xdd23167e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de31687, 90 },
    { 0xf0b31687, 98 },
    { 0x19131687, 75 },
    { 0xbfa3168a, 55 },
    { 0x2103168b, 90 },
    { 0x7493168c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf631691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa731698, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3733169b, 95 },
    { 0xb2a3169b, 98 },
    { 0, 0 },
    { 0x5e03169e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650316a2, 48 },
    { 0, 0 },
    { 0xea2316a4, 68 },
    { 0, 0 },
    { 0x1af316a6, 33 },
    { 0x7f9316a7, 98 },
    { 0xce9316a7, 61 },
    { 0, 0 },
    { 0x5a5316aa, 33 },
    { 0xc79316ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9316b0, 61 },
    { 0, 0 },
    { 0x54a316b2, 77 },
    { 0xf1c316b2, 52 },
    { 0x711316b2, 34 },
    { 0xda9316b5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96316bd, 93 },
    { 0, 0 },
    { 0x5c0316bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3316c5, 91 },
    { 0xa99316c6, 61 },
    { 0x203316c7, 61 },
    { 0xb26316c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62316cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4316d1, 93 },
    { 0, 0 },
    { 0x7bc316d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f316db, 46 },
    { 0x936316db, 61 },
    { 0, 0 },
    { 0x3bc316de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55316e1, 95 },
    { 0, 0 },
    { 0x560316e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa316e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa316ea, 90 },
    { 0x880316eb, 33 },
    { 0x421316eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f316f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4316f4, 90 },
    { 0, 0 },
    { 0xa49316f6, 46 },
    { 0xdef316f6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5316fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee031700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a31707, 84 },
    { 0x2ba31708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e3170c, 90 },
    { 0xcb3170d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad31712, 95 },
    { 0x4ba31713, 95 },
    { 0, 0 },
    { 0x91d31715, 52 },
    { 0x70431716, 78 },
    { 0x6c231717, 81 },
    { 0, 0 },
    { 0x41b31719, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7383171c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b531721, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8631724, 91 },
    { 0, 0 },
    { 0xcfa31726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d31729, 34 },
    { 0x6533172a, 84 },
    { 0xeb3172b, 96 },
    { 0, 0 },
    { 0x4723172d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4231731, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d431734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a93173e, 61 },
    { 0xf083173e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6131742, 91 },
    { 0xabc31743, 95 },
    { 0x72931744, 43 },
    { 0xb3631745, 90 },
    { 0xd3a31743, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f531749, 61 },
    { 0, 0 },
    { 0xb1e3174b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd633174f, 90 },
    { 0xd3931750, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22031754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c31758, 43 },
    { 0, 0 },
    { 0xf873175a, 96 },
    { 0x69d3175a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d3175f, 76 },
    { 0x34731760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f431765, 95 },
    { 0xdbc31766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f93176a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2343176d, 43 },
    { 0xb1e3176e, 95 },
    { 0xde33176e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49531772, 33 },
    { 0x36931772, 61 },
    { 0x59431774, 17 },
    { 0xbc731775, 46 },
    { 0xf2a31775, 77 },
    { 0xd2231776, 61 },
    { 0xe2b31773, 95 },
    { 0x9f431779, 92 },
    { 0x35b31779, 66 },
    { 0, 0 },
    { 0x7c03177c, 66 },
    { 0xc2d3177c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b31780, 88 },
    { 0xb5b31780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9031785, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b31789, 61 },
    { 0x3573178a, 98 },
    { 0x87d3178b, 90 },
    { 0x6ad3178c, 33 },
    { 0x5463178d, 98 },
    { 0x2223178e, 95 },
    { 0xbe43178e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d31795, 90 },
    { 0x33d31796, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3623179c, 95 },
    { 0x2323179c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8317a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26317a5, 90 },
    { 0, 0 },
    { 0x401317a7, 61 },
    { 0x24c317a8, 78 },
    { 0xffd317a8, 23 },
    { 0x503317aa, 81 },
    { 0xaa6317aa, 95 },
    { 0xf0e317a9, 61 },
    { 0x934317aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b317b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5317b4, 99 },
    { 0xfb2317b4, 98 },
    { 0xac2317b4, 90 },
    { 0xbfd317b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5317be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61317c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1317cc, 46 },
    { 0x215317cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470317d1, 84 },
    { 0xa23317d2, 55 },
    { 0x436317d3, 90 },
    { 0xb70317d4, 76 },
    { 0xac317d2, 33 },
    { 0xffa317d1, 90 },
    { 0xf95317d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f317da, 98 },
    { 0x911317db, 55 },
    { 0xfec317dc, 76 },
    { 0x193317dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab317e1, 87 },
    { 0xf0f317e1, 98 },
    { 0xda9317e2, 46 },
    { 0x51317e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95317ef, 92 },
    { 0, 0 },
    { 0x4ab317f1, 33 },
    { 0x311317f2, 90 },
    { 0x8eb317f3, 98 },
    { 0x2f4317f3, 55 },
    { 0xd67317f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb317fb, 66 },
    { 0, 0 },
    { 0x1a7317fd, 61 },
    { 0, 0 },
    { 0x442317ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a31803, 61 },
    { 0x71e31804, 90 },
    { 0xa5231804, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b3180e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c31811, 95 },
    { 0, 0 },
    { 0x50731813, 66 },
    { 0xd6431814, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb831818, 34 },
    { 0x96831819, 76 },
    { 0xf0d3181a, 61 },
    { 0xc9f31818, 95 },
    { 0x9a33181a, 87 },
    { 0x6543181d, 99 },
    { 0, 0 },
    { 0x33b3181f, 53 },
    { 0x8e23181f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee31823, 66 },
    { 0, 0 },
    { 0xfba31825, 23 },
    { 0xdf331826, 66 },
    { 0x8c931827, 90 },
    { 0x2c231828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8573182d, 99 },
    { 0x6a13182e, 98 },
    { 0x543182d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4731833, 33 },
    { 0x90331834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a31838, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b3183b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e3183e, 77 },
    { 0x8213183f, 90 },
    { 0xf5831840, 61 },
    { 0xb713183f, 33 },
    { 0x2e831842, 76 },
    { 0x4bc31843, 46 },
    { 0x54931843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf31849, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a3184d, 61 },
    { 0xc1c3184e, 55 },
    { 0xf373184f, 95 },
    { 0x3bd3184e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be31856, 88 },
    { 0, 0 },
    { 0xf7a31858, 43 },
    { 0xcbf31859, 33 },
    { 0x4b31859, 61 },
    { 0xfb83185a, 100 },
    { 0x7de3185c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc31862, 46 },
    { 0x7d731863, 84 },
    { 0xd3131862, 55 },
    { 0xd4831863, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea831869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf3186d, 33 },
    { 0xd0f3186d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf531871, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d31879, 61 },
    { 0x9ec31879, 76 },
    { 0xdc33187a, 48 },
    { 0x96d3187c, 9 },
    { 0x9303187d, 81 },
    { 0x8673187d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f031884, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3188b, 90 },
    { 0, 0 },
    { 0xf3c3188d, 90 },
    { 0xc4e3188d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71431891, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3189c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f318a0, 91 },
    { 0x7a2318a0, 68 },
    { 0, 0 },
    { 0x35f318a3, 87 },
    { 0, 0 },
    { 0x8f318a5, 76 },
    { 0xcef318a5, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef318a9, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3318ad, 90 },
    { 0xa8318ae, 23 },
    { 0x669318af, 34 },
    { 0x354318b0, 33 },
    { 0x9a4318b1, 61 },
    { 0xf6d318af, 76 },
    { 0, 0 },
    { 0x24e318b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72318b7, 90 },
    { 0, 0 },
    { 0xeba318b9, 61 },
    { 0, 0 },
    { 0x8f3318bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c318bf, 33 },
    { 0x38b318c0, 33 },
    { 0x71f318bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e318c7, 95 },
    { 0xc33318c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c318cb, 61 },
    { 0xf1d318cc, 33 },
    { 0, 0 },
    { 0x7e7318ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565318d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5318d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6318da, 33 },
    { 0, 0 },
    { 0x882318dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1318df, 61 },
    { 0, 0 },
    { 0x434318e1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3318e5, 33 },
    { 0, 0 },
    { 0x477318e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632318ee, 33 },
    { 0x90a318ee, 23 },
    { 0xa7a318f0, 55 },
    { 0x1d4318f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8318f9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x455318fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f31900, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38831905, 98 },
    { 0x11431906, 95 },
    { 0, 0 },
    { 0x56731908, 33 },
    { 0x82131908, 33 },
    { 0xdd431908, 84 },
    { 0x25b3190b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35331917, 55 },
    { 0x7e131918, 90 },
    { 0xfbc31917, 85 },
    { 0, 0 },
    { 0xe6e3191b, 61 },
    { 0xe253191c, 90 },
    { 0x6033191c, 90 },
    { 0x5ec3191e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9431923, 76 },
    { 0x61d31923, 61 },
    { 0, 0 },
    { 0x79331926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d31939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3193d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95631946, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe473194a, 81 },
    { 0xb383194a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7231950, 84 },
    { 0, 0 },
    { 0x6ba31952, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98131956, 90 },
    { 0xa1a31956, 76 },
    { 0x37831956, 61 },
    { 0xf4331957, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36131961, 61 },
    { 0, 0 },
    { 0x50931963, 98 },
    { 0x4c831964, 46 },
    { 0, 0 },
    { 0x1ed31966, 52 },
    { 0, 0 },
    { 0xce431968, 68 },
    { 0x37a31969, 76 },
    { 0x40f31969, 33 },
    { 0, 0 },
    { 0xb293196c, 33 },
    { 0x60c3196d, 90 },
    { 0xdb63196e, 93 },
    { 0, 0 },
    { 0x1cf31970, 43 },
    { 0xc231971, 85 },
    { 0xb0931972, 46 },
    { 0x1f331972, 93 },
    { 0x64831974, 33 },
    { 0x98931974, 23 },
    { 0xc4831974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c3197a, 34 },
    { 0, 0 },
    { 0xb003197c, 55 },
    { 0x4213197c, 23 },
    { 0x61f3197c, 61 },
    { 0x9cd3197e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14731985, 93 },
    { 0, 0 },
    { 0xafc31987, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa283198f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5231994, 61 },
    { 0, 0 },
    { 0x34831996, 76 },
    { 0x1a31997, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f3199d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba319a0, 40 },
    { 0x9b6319a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2319a4, 85 },
    { 0, 0 },
    { 0x606319a6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144319aa, 78 },
    { 0xc7c319aa, 93 },
    { 0x54a319ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9319b4, 33 },
    { 0x64c319b5, 33 },
    { 0, 0 },
    { 0x495319b7, 85 },
    { 0, 0 },
    { 0x9c2319b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1319bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2319c3, 75 },
    { 0xaf9319c3, 90 },
    { 0, 0 },
    { 0x9bd319c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a319ca, 90 },
    { 0, 0 },
    { 0x97f319cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde319d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254319d5, 52 },
    { 0x807319d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e319de, 40 },
    { 0xd7c319df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13319e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc319e7, 90 },
    { 0x8fe319e8, 61 },
    { 0x12d319e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb319ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9319f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19319f5, 61 },
    { 0, 0 },
    { 0x18b319f7, 66 },
    { 0x603319f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x359319fb, 78 },
    { 0x51319fb, 23 },
    { 0xd52319fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e731a01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x17431a04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e531a07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37331a0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20831a0d, 33 },
    { 0x8a431a0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4531a11, 90 },
    { 0, 0 },
    { 0x8731a13, 68 },
    { 0xb7931a14, 46 },
    { 0xb6731a14, 61 },
    { 0x98431a13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd31a21, 55 },
    { 0xc3931a22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59431a25, 90 },
    { 0x66031a25, 23 },
    { 0xed531a27, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7631a2c, 33 },
    { 0x9f031a2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd31a30, 61 },
    { 0x37131a30, 24 },
    { 0xe8431a31, 61 },
    { 0x77331a33, 43 },
    { 0xcda31a34, 46 },
    { 0xb4931a35, 98 },
    { 0xd8431a36, 34 },
    { 0xf5631a33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9431a3a, 52 },
    { 0x5a531a3a, 95 },
    { 0x27331a3c, 55 },
    { 0x1f631a3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f31a41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae131a45, 61 },
    { 0x36e31a45, 33 },
    { 0x49531a46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22131a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd431a5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e31a60, 91 },
    { 0xe9f31a61, 90 },
    { 0, 0 },
    { 0x80831a63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18431a6b, 46 },
    { 0x1a231a6b, 90 },
    { 0, 0 },
    { 0x6ed31a6e, 91 },
    { 0x53331a6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9031a74, 43 },
    { 0xbb231a75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1931a7d, 76 },
    { 0xd0f31a7d, 34 },
    { 0x12331a7f, 34 },
    { 0xbe431a7f, 98 },
    { 0xda031a81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d331a85, 32 },
    { 0, 0 },
    { 0x8d531a87, 96 },
    { 0x61b31a87, 33 },
    { 0, 0 },
    { 0xfef31a8a, 61 },
    { 0x26731a8b, 51 },
    { 0x4a031a8c, 52 },
    { 0xcb31a8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5631a90, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca31a93, 52 },
    { 0x51c31a94, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2131a97, 90 },
    { 0, 0 },
    { 0x71b31a99, 84 },
    { 0, 0 },
    { 0xe6831a9b, 90 },
    { 0, 0 },
    { 0x2b331a9d, 33 },
    { 0, 0 },
    { 0xcfb31a9f, 99 },
    { 0x89031a9f, 90 },
    { 0x30731aa1, 66 },
    { 0x6631aa1, 61 },
    { 0x57e31aa3, 95 },
    { 0x63931aa3, 66 },
    { 0xaa731aa0, 91 },
    { 0x5c031aa0, 34 },
    { 0, 0 },
    { 0x65231aa8, 33 },
    { 0x5b131aa8, 90 },
    { 0xd0b31aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96131aad, 95 },
    { 0, 0 },
    { 0xaf731aaf, 23 },
    { 0x12331aaf, 61 },
    { 0x1a331ab0, 43 },
    { 0x9f331aaf, 38 },
    { 0xd5731ab2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba31ab6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d431aba, 23 },
    { 0x54c31abb, 52 },
    { 0x98631abc, 76 },
    { 0x72931abb, 76 },
    { 0, 0 },
    { 0xf5331abf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f31ac6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x86831ac9, 90 },
    { 0, 0 },
    { 0x6f31acb, 33 },
    { 0, 0 },
    { 0x8fb31acd, 95 },
    { 0x76931ace, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef31ad1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83231ad5, 55 },
    { 0x52031ad5, 33 },
    { 0x7b631ad6, 43 },
    { 0xebf31ad8, 61 },
    { 0x7e31ad9, 98 },
    { 0x2d531ada, 66 },
    { 0x70c31adb, 91 },
    { 0, 0 },
    { 0xf531add, 23 },
    { 0xfda31ade, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b31ae4, 83 },
    { 0xfd531ae5, 76 },
    { 0x91a31ae6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f831aeb, 90 },
    { 0, 0 },
    { 0x6ef31aed, 95 },
    { 0, 0 },
    { 0xfe631aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac31af6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c31af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5431aff, 43 },
    { 0xf9e31b00, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec31b03, 61 },
    { 0xc1d31b03, 23 },
    { 0, 0 },
    { 0x94331b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c231b09, 65 },
    { 0x19a31b0a, 95 },
    { 0x5131b0b, 90 },
    { 0x67631b0c, 66 },
    { 0x1dd31b0c, 98 },
    { 0xe1931b09, 23 },
    { 0, 0 },
    { 0x2c131b10, 34 },
    { 0x5dd31b10, 90 },
    { 0xe2931b12, 81 },
    { 0x9a831b11, 69 },
    { 0xc0431b14, 66 },
    { 0xe4531b13, 90 },
    { 0xd0231b16, 33 },
    { 0xaeb31b10, 33 },
    { 0x9f031b14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21031b1b, 66 },
    { 0xef231b1b, 76 },
    { 0xf1e31b1c, 90 },
    { 0x85c31b1e, 23 },
    { 0x55531b1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2331b22, 33 },
    { 0x93c31b22, 90 },
    { 0x2ae31b24, 52 },
    { 0, 0 },
    { 0xe1431b26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b31b29, 81 },
    { 0xef531b2a, 95 },
    { 0xba231b2b, 87 },
    { 0xd6831b2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc731b2f, 87 },
    { 0x51231b30, 98 },
    { 0x9d531b2f, 90 },
    { 0xaca31b32, 52 },
    { 0xeae31b33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f31b3b, 55 },
    { 0, 0 },
    { 0x6bc31b3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3631b42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84031b4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd731b4e, 90 },
    { 0, 0 },
    { 0xbc831b50, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23831b55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31b59, 85 },
    { 0x8d031b5a, 99 },
    { 0x35d31b5b, 95 },
    { 0x84231b5a, 43 },
    { 0xfaa31b5d, 90 },
    { 0, 0 },
    { 0xbbe31b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f31b64, 52 },
    { 0, 0 },
    { 0x85731b66, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45531b6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3931b6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd631b76, 61 },
    { 0x30031b76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6531b82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe31b85, 33 },
    { 0x9d931b86, 55 },
    { 0xa331b87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6631b8b, 78 },
    { 0x7d631b8b, 33 },
    { 0, 0 },
    { 0xb3531b8e, 52 },
    { 0x8eb31b8f, 90 },
    { 0x38631b8e, 61 },
    { 0xea931b91, 52 },
    { 0xecc31b92, 98 },
    { 0x5ae31b92, 95 },
    { 0x98531b94, 33 },
    { 0xaac31b94, 61 },
    { 0xe8631b94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c31b9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1431ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf531ba7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x24231baa, 95 },
    { 0, 0 },
    { 0x4e431bac, 90 },
    { 0x12131bad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab31bb1, 33 },
    { 0, 0 },
    { 0xfcb31bb3, 100 },
    { 0, 0 },
    { 0x97731bb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f31bb8, 33 },
    { 0x65b31bb9, 55 },
    { 0x90531bba, 68 },
    { 0, 0 },
    { 0x13731bbc, 98 },
    { 0, 0 },
    { 0x43a31bbe, 90 },
    { 0x4e31bbf, 90 },
    { 0xf1831bbe, 90 },
    { 0xa3131bc1, 23 },
    { 0xf0831bc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38631bcb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a31bd7, 81 },
    { 0x95a31bd8, 76 },
    { 0x93031bd9, 98 },
    { 0xb0231bd7, 34 },
    { 0x1f431bdb, 98 },
    { 0xaef31bdc, 98 },
    { 0x84d31bdd, 23 },
    { 0xad931bdc, 33 },
    { 0x48c31bdb, 76 },
    { 0xf931be0, 93 },
    { 0xf8831be0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb31be4, 90 },
    { 0, 0 },
    { 0x4f931be6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef31bf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c31bfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa31bff, 38 },
    { 0xc2e31bff, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f531c03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f31c09, 84 },
    { 0xc3631c0a, 95 },
    { 0x36b31c0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd31c0e, 98 },
    { 0x71f31c0e, 100 },
    { 0x2e231c0e, 95 },
    { 0x1ba31c10, 66 },
    { 0, 0 },
    { 0xce031c13, 99 },
    { 0x2a331c14, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb331c1a, 99 },
    { 0xe2031c1b, 23 },
    { 0x5de31c1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5431c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c31c25, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e31c29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e731c2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea31c31, 90 },
    { 0xff831c32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b31c38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7931c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56531c42, 46 },
    { 0, 0 },
    { 0x91831c44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94831c51, 99 },
    { 0x6c431c52, 98 },
    { 0x1dc31c53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x58531c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31c62, 90 },
    { 0x62231c63, 95 },
    { 0x74031c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37031c68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d31c6e, 43 },
    { 0, 0 },
    { 0x88b31c70, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e31c75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9831c78, 99 },
    { 0x6a331c78, 23 },
    { 0x32a31c7a, 23 },
    { 0xc4731c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f431c7e, 90 },
    { 0x28831c7f, 34 },
    { 0x2f431c7f, 55 },
    { 0xcfe31c81, 66 },
    { 0x67d31c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2631c85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd931c8a, 43 },
    { 0, 0 },
    { 0x10831c8c, 98 },
    { 0xdb231c8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f31c93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b231c9f, 33 },
    { 0x25131ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b931ca3, 66 },
    { 0xc31ca4, 95 },
    { 0xed031ca3, 93 },
    { 0x3f431ca5, 33 },
    { 0, 0 },
    { 0xe3631ca8, 64 },
    { 0, 0 },
    { 0xba31caa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52031cb1, 78 },
    { 0, 0 },
    { 0x8a231cb3, 34 },
    { 0x86631cb3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d31cb9, 90 },
    { 0, 0 },
    { 0x28331cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf231cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e631cc7, 33 },
    { 0x84931cc7, 98 },
    { 0x2dc31cc8, 34 },
    { 0xbd31cca, 90 },
    { 0xf831ccb, 99 },
    { 0x49731ccb, 43 },
    { 0xfac31cc8, 17 },
    { 0x98431cce, 95 },
    { 0x5431ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde31cd5, 68 },
    { 0x7ba31cd6, 33 },
    { 0, 0 },
    { 0xa3631cd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c31cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d31ce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab31ce7, 23 },
    { 0, 0 },
    { 0x6b31ce9, 98 },
    { 0x61231cea, 33 },
    { 0x7ca31cea, 91 },
    { 0x95631cec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef31cf2, 36 },
    { 0xf1231cf3, 61 },
    { 0, 0 },
    { 0xa2e31cf5, 99 },
    { 0x9a231cf6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38931cfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0131cfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a631d05, 98 },
    { 0x3d831d06, 90 },
    { 0x94331d06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad31d0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16031d0e, 98 },
    { 0, 0 },
    { 0xe4c31d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa031d1e, 67 },
    { 0, 0 },
    { 0x79d31d20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3931d23, 61 },
    { 0x4ac31d24, 95 },
    { 0, 0 },
    { 0x8eb31d26, 78 },
    { 0, 0 },
    { 0xf4931d28, 90 },
    { 0x95431d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12231d31, 78 },
    { 0xc3e31d31, 90 },
    { 0xbc431d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed31d3a, 34 },
    { 0x85131d3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d31d3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53531d42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8131d45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e31d4b, 85 },
    { 0x8cd31d4c, 90 },
    { 0x7cd31d4b, 95 },
    { 0, 0 },
    { 0x48231d4f, 90 },
    { 0xfd531d50, 46 },
    { 0x60831d4f, 33 },
    { 0x74831d51, 91 },
    { 0x3ac31d53, 33 },
    { 0x8de31d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b31d57, 68 },
    { 0x7e931d58, 99 },
    { 0x64d31d59, 61 },
    { 0xeb131d5a, 55 },
    { 0x62e31d5b, 95 },
    { 0x77e31d5c, 55 },
    { 0x81231d5d, 90 },
    { 0xed731d5e, 61 },
    { 0, 0 },
    { 0x37231d60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80431d68, 90 },
    { 0xf5931d68, 90 },
    { 0x8e631d6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b031d6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd931d72, 77 },
    { 0, 0 },
    { 0xd9131d74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c431d77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a31d7a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c31d7d, 23 },
    { 0, 0 },
    { 0x7231d7f, 23 },
    { 0x3b331d7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9731d84, 34 },
    { 0x34331d84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a731d88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99331d8b, 34 },
    { 0, 0 },
    { 0x5a831d8d, 33 },
    { 0xb8931d8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7431d91, 23 },
    { 0, 0 },
    { 0x6b031d93, 90 },
    { 0x10531d93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7031d98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd31d9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe131d9e, 90 },
    { 0x77e31d9e, 33 },
    { 0xb5731da0, 95 },
    { 0x90731da0, 99 },
    { 0x9a231da2, 93 },
    { 0x8e331da3, 98 },
    { 0xf3031da1, 76 },
    { 0x40d31da3, 61 },
    { 0, 0 },
    { 0x17631da7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f931db0, 95 },
    { 0, 0 },
    { 0x56f31db2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d131db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a31db9, 77 },
    { 0, 0 },
    { 0x65331dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29331dbe, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da31dc2, 24 },
    { 0x1a731dc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a531dc8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87631dd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6031dd3, 98 },
    { 0xd8a31dd4, 33 },
    { 0xbcc31dd5, 95 },
    { 0x3931dd3, 51 },
    { 0xcfb31dd7, 55 },
    { 0, 0 },
    { 0x3c931dd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0131de3, 46 },
    { 0, 0 },
    { 0x1c231de5, 55 },
    { 0x91831de6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c31de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7031df3, 52 },
    { 0x3a931df3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a31df9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd931dfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e31e02, 55 },
    { 0x3f731e03, 55 },
    { 0xd9531e02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a31e08, 93 },
    { 0, 0 },
    { 0xf5b31e0a, 61 },
    { 0x61e31e0b, 61 },
    { 0xd9931e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7131e12, 99 },
    { 0x73c31e12, 34 },
    { 0x79631e12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d931e19, 17 },
    { 0x33331e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f31e1d, 90 },
    { 0x85931e1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1831e21, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f131e25, 61 },
    { 0, 0 },
    { 0xc3431e27, 84 },
    { 0xa4131e27, 69 },
    { 0, 0 },
    { 0x11131e2a, 90 },
    { 0x75531e2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd31e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67731e32, 46 },
    { 0xa1731e33, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91731e40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c31e43, 34 },
    { 0x88131e44, 76 },
    { 0, 0 },
    { 0xdb231e46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b31e4a, 95 },
    { 0x78731e4a, 98 },
    { 0xc2131e4c, 90 },
    { 0xedf31e4c, 89 },
    { 0x9a331e4e, 46 },
    { 0xde031e4e, 66 },
    { 0xb4a31e4f, 61 },
    { 0x3bc31e4e, 66 },
    { 0, 0 },
    { 0xc9d31e53, 33 },
    { 0x10231e54, 98 },
    { 0, 0 },
    { 0x5b131e56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7231e5c, 95 },
    { 0x10331e5d, 61 },
    { 0xb31e5e, 17 },
    { 0xa6e31e5f, 99 },
    { 0x8cc31e5f, 55 },
    { 0xec831e5d, 90 },
    { 0xf2331e62, 95 },
    { 0x82231e62, 76 },
    { 0x85c31e61, 66 },
    { 0x80c31e65, 86 },
    { 0x4dd31e60, 88 },
    { 0xe2031e5f, 36 },
    { 0xa2031e60, 90 },
    { 0xd2b31e69, 95 },
    { 0x47d31e6a, 98 },
    { 0xc8e31e6b, 99 },
    { 0x5b131e5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x16231e6f, 90 },
    { 0x76e31e70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2831e78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2731e7b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x62931e7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a31e85, 43 },
    { 0x2cf31e86, 33 },
    { 0xd1d31e87, 76 },
    { 0, 0 },
    { 0x3a431e89, 33 },
    { 0x91531e89, 85 },
    { 0x20731e8b, 91 },
    { 0x6aa31e8b, 43 },
    { 0xd3a31e8d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70531e98, 35 },
    { 0x38431e99, 76 },
    { 0, 0 },
    { 0xf4e31e9b, 66 },
    { 0x70c31e9c, 33 },
    { 0, 0 },
    { 0xdb431e9e, 90 },
    { 0, 0 },
    { 0xf4631ea0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29631ea4, 33 },
    { 0x30631ea4, 98 },
    { 0, 0 },
    { 0xcc031ea7, 43 },
    { 0xa8231ea8, 33 },
    { 0, 0 },
    { 0x67c31eaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31731eae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e31eb3, 78 },
    { 0xd2131eb3, 61 },
    { 0x7ff31eb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87231eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d631ebc, 77 },
    { 0, 0 },
    { 0xdf431ebe, 33 },
    { 0x4d731ebf, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e31ec5, 55 },
    { 0, 0 },
    { 0xbe931ec7, 55 },
    { 0x77831ec7, 33 },
    { 0, 0 },
    { 0x78c31eca, 66 },
    { 0, 0 },
    { 0x7a31ecc, 55 },
    { 0x7b031ecc, 90 },
    { 0, 0 },
    { 0x2ce31ecf, 90 },
    { 0xd2a31ed0, 46 },
    { 0, 0 },
    { 0x7fe31ed2, 33 },
    { 0, 0 },
    { 0x84a31ed4, 61 },
    { 0, 0 },
    { 0x68931ed6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca31ed9, 34 },
    { 0xb9c31eda, 61 },
    { 0xec131eda, 69 },
    { 0xbc531eda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d231ee3, 95 },
    { 0x98531ee4, 61 },
    { 0x56531ee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea731eea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69631eef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc131ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a31ef8, 90 },
    { 0xd5031ef8, 93 },
    { 0x2c431efa, 33 },
    { 0xd1a31efb, 46 },
    { 0xadd31efb, 55 },
    { 0xe7631efa, 90 },
    { 0xb0131efb, 98 },
    { 0x70131eff, 61 },
    { 0xbce31efe, 23 },
    { 0x97d31eff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca31f05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af31f0f, 93 },
    { 0, 0 },
    { 0x9cc31f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef31f16, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf731f1b, 55 },
    { 0x50b31f1c, 90 },
    { 0, 0 },
    { 0xcc831f1e, 34 },
    { 0, 0 },
    { 0x1f431f20, 87 },
    { 0x27431f21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb831f24, 90 },
    { 0xf6b31f25, 90 },
    { 0xe4e31f26, 84 },
    { 0x66f31f26, 34 },
    { 0x39231f27, 61 },
    { 0x18731f26, 95 },
    { 0x27d31f25, 90 },
    { 0xf5031f2b, 95 },
    { 0, 0 },
    { 0xa3a31f2d, 66 },
    { 0x12231f2e, 46 },
    { 0x56b31f2f, 95 },
    { 0, 0 },
    { 0x44f31f31, 90 },
    { 0xdd531f31, 85 },
    { 0xdd031f32, 100 },
    { 0x38c31f31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca31f37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c231f3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b31f40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e31f43, 90 },
    { 0xc9b31f44, 23 },
    { 0xc4531f43, 98 },
    { 0, 0 },
    { 0x31c31f47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22331f4a, 77 },
    { 0xbbc31f4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f31f52, 76 },
    { 0x36031f53, 88 },
    { 0xf7231f54, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e631f5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f31f64, 98 },
    { 0xc9b31f65, 51 },
    { 0, 0 },
    { 0x36b31f67, 84 },
    { 0x21e31f67, 76 },
    { 0x2a931f69, 81 },
    { 0xac331f6a, 61 },
    { 0x7ce31f6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d31f6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b231f71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72631f75, 90 },
    { 0x20431f75, 61 },
    { 0xca831f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b31f7a, 83 },
    { 0xf4e31f7a, 90 },
    { 0xb8731f7c, 36 },
    { 0x66531f7a, 90 },
    { 0, 0 },
    { 0x81d31f7f, 90 },
    { 0x34031f80, 23 },
    { 0x37731f7f, 33 },
    { 0xccb31f82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d31f85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d631f88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d231f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62031f8f, 17 },
    { 0xa6131f90, 61 },
    { 0xa7731f91, 100 },
    { 0x60131f8f, 90 },
    { 0x42531f93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1131f9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c31faa, 33 },
    { 0xb2d31fab, 61 },
    { 0x13231fab, 61 },
    { 0, 0 },
    { 0x7b431fae, 61 },
    { 0xfef31faf, 61 },
    { 0x48e31fae, 95 },
    { 0, 0 },
    { 0x85731fb2, 43 },
    { 0, 0 },
    { 0x4d931fb4, 25 },
    { 0x34831fb5, 98 },
    { 0x99f31fb6, 46 },
    { 0, 0 },
    { 0xb5031fb8, 74 },
    { 0, 0 },
    { 0xefd31fba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f31fbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae431fc1, 65 },
    { 0x3af31fc1, 46 },
    { 0xf4731fc2, 46 },
    { 0x92a31fc4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b31fc7, 68 },
    { 0x4cf31fc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d31fcc, 34 },
    { 0, 0 },
    { 0xa1031fce, 61 },
    { 0xaa131fcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf831fd2, 93 },
    { 0xac431fd3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9731fd6, 91 },
    { 0xd3431fd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c31fde, 46 },
    { 0xeb431fde, 100 },
    { 0xf1c31fe0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61431fe4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b831fe7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab31ff2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29631ff7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b31ffa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f431ffd, 98 },
    { 0x85731ffe, 55 },
    { 0x8bf31ffd, 61 },
    { 0x68232000, 100 },
    { 0xba331ffd, 61 },
    { 0, 0 },
    { 0xb5332003, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed32009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e63200c, 77 },
    { 0, 0 },
    { 0xd0a3200e, 98 },
    { 0x783200f, 95 },
    { 0x29432010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e32013, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f32018, 61 },
    { 0, 0 },
    { 0xb2b3201a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38832020, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a332024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be32030, 33 },
    { 0, 0 },
    { 0x7bf32032, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71632036, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb703203a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b3203d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c32045, 33 },
    { 0x2ef32045, 61 },
    { 0x4cf32045, 96 },
    { 0xe3f32047, 33 },
    { 0x83132045, 76 },
    { 0, 0 },
    { 0xd643204b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82232054, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41632059, 97 },
    { 0x5783205a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb3205d, 52 },
    { 0x2d43205d, 95 },
    { 0xc963205e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc43206d, 34 },
    { 0, 0 },
    { 0x2f03206f, 36 },
    { 0xedd32070, 40 },
    { 0xba032071, 55 },
    { 0x92332072, 65 },
    { 0x6bd32072, 93 },
    { 0x5d932070, 85 },
    { 0, 0 },
    { 0x4e732076, 90 },
    { 0x7f432077, 23 },
    { 0xa432078, 90 },
    { 0x16032078, 100 },
    { 0, 0 },
    { 0xe213207b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf03207f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f32086, 90 },
    { 0x1f532087, 98 },
    { 0x42d32087, 78 },
    { 0, 0 },
    { 0x25a3208a, 90 },
    { 0, 0 },
    { 0x7f03208c, 23 },
    { 0x5503208d, 33 },
    { 0x6343208e, 33 },
    { 0x3dd3208f, 40 },
    { 0x9993208e, 99 },
    { 0, 0 },
    { 0x2f732092, 23 },
    { 0x2ee32093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7732096, 33 },
    { 0xb4232097, 55 },
    { 0x69332097, 95 },
    { 0x89a32099, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985320a2, 67 },
    { 0, 0 },
    { 0x902320a4, 76 },
    { 0x3b9320a5, 34 },
    { 0, 0 },
    { 0xef7320a7, 98 },
    { 0xa44320a7, 98 },
    { 0xa35320a9, 34 },
    { 0xa0320aa, 85 },
    { 0x4ed320a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7320ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2320b9, 99 },
    { 0x525320ba, 55 },
    { 0xc5320bb, 95 },
    { 0xd3e320bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863320c0, 90 },
    { 0, 0 },
    { 0x92f320c2, 23 },
    { 0xbf0320c3, 95 },
    { 0, 0 },
    { 0x92f320c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e320c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403320d2, 95 },
    { 0xdd1320d2, 90 },
    { 0x3bf320d2, 61 },
    { 0, 0 },
    { 0x3e4320d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247320dc, 78 },
    { 0, 0 },
    { 0x860320de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf320e3, 52 },
    { 0, 0 },
    { 0x97b320e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45320e9, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25320ec, 61 },
    { 0, 0 },
    { 0x167320ee, 52 },
    { 0, 0 },
    { 0x786320f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9320f5, 33 },
    { 0, 0 },
    { 0xba3320f7, 61 },
    { 0x227320f8, 23 },
    { 0, 0 },
    { 0x8db320fa, 80 },
    { 0xb53320fb, 55 },
    { 0, 0 },
    { 0x536320fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7232100, 90 },
    { 0x10732101, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f32105, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e32109, 98 },
    { 0, 0 },
    { 0xf033210b, 76 },
    { 0xbbb3210c, 49 },
    { 0, 0 },
    { 0x9cd3210e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c32116, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c3211d, 77 },
    { 0x1b73211d, 61 },
    { 0xdc83211e, 90 },
    { 0x1cc3211d, 90 },
    { 0xc8a32121, 90 },
    { 0, 0 },
    { 0xdc232123, 95 },
    { 0xa4f32124, 99 },
    { 0x34932125, 46 },
    { 0x77b32126, 66 },
    { 0xd1632125, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff3212a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8283212e, 66 },
    { 0x2653212f, 61 },
    { 0x29a3212e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d32134, 76 },
    { 0x63f32134, 90 },
    { 0xcd332136, 66 },
    { 0xa7432136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13213a, 90 },
    { 0x5133213b, 78 },
    { 0x6a83213c, 36 },
    { 0x3c73213d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a332141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e32145, 33 },
    { 0, 0 },
    { 0x35532147, 76 },
    { 0x95132148, 33 },
    { 0, 0 },
    { 0x54d3214a, 46 },
    { 0, 0 },
    { 0xcc33214c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde03214f, 69 },
    { 0, 0 },
    { 0xb32151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b32155, 95 },
    { 0x9332156, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a3215c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c32161, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21632165, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e832168, 61 },
    { 0xb1532168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b43216e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd32172, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b32179, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa3217c, 61 },
    { 0, 0 },
    { 0x443217e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87532186, 61 },
    { 0xa0b32187, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc103218a, 98 },
    { 0, 0 },
    { 0x8c93218c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c03218f, 93 },
    { 0x56232190, 90 },
    { 0x77a32191, 66 },
    { 0x13832192, 90 },
    { 0, 0 },
    { 0xa5732194, 81 },
    { 0x81032195, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f632199, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c3219e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399321af, 76 },
    { 0x40321b0, 33 },
    { 0xa8d321b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x475321b4, 90 },
    { 0x5d4321b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2321bb, 76 },
    { 0xb9d321bc, 17 },
    { 0xb9321bc, 99 },
    { 0xf69321bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86321c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695321c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e321c9, 88 },
    { 0x76e321c9, 77 },
    { 0x3ee321ca, 95 },
    { 0x23f321cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686321d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210321da, 95 },
    { 0xc3c321db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9321e4, 98 },
    { 0x919321e5, 61 },
    { 0, 0 },
    { 0xd7321e7, 23 },
    { 0x707321e7, 74 },
    { 0, 0 },
    { 0x5b321ea, 19 },
    { 0x72e321eb, 52 },
    { 0xbc0321ea, 90 },
    { 0x75f321ec, 61 },
    { 0x533321eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847321f2, 76 },
    { 0xb75321f2, 61 },
    { 0x49321f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e321f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c32201, 100 },
    { 0xd4932202, 23 },
    { 0, 0 },
    { 0xf4132204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8632207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c3220a, 93 },
    { 0x1103220b, 88 },
    { 0, 0 },
    { 0x92b3220d, 98 },
    { 0, 0 },
    { 0xc6a3220f, 61 },
    { 0xab332210, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2332216, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a32220, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda032229, 23 },
    { 0, 0 },
    { 0xa003222b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe673222e, 23 },
    { 0x6a33222f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b832232, 97 },
    { 0x9ad32232, 77 },
    { 0xa232232, 34 },
    { 0xf1b32235, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x29132238, 76 },
    { 0, 0 },
    { 0xc8f3223a, 66 },
    { 0xc743223b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf532241, 61 },
    { 0, 0 },
    { 0x24332243, 90 },
    { 0xab32244, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa3224b, 95 },
    { 0, 0 },
    { 0x143224d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a32250, 17 },
    { 0x87c32251, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5032254, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e232259, 33 },
    { 0x91a3225a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa093225d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f232262, 23 },
    { 0x25d32262, 43 },
    { 0x80832264, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c03226b, 34 },
    { 0x47a3226c, 61 },
    { 0, 0 },
    { 0x7523226e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73832271, 77 },
    { 0xcce32271, 61 },
    { 0x5d32272, 61 },
    { 0x14b32274, 78 },
    { 0x1fe32272, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50132278, 23 },
    { 0, 0 },
    { 0xaf13227a, 46 },
    { 0x5693227b, 33 },
    { 0x9a03227c, 98 },
    { 0xf8f3227d, 23 },
    { 0xf1d3227c, 95 },
    { 0x7863227c, 46 },
    { 0xf2e3227f, 33 },
    { 0xd5d32281, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xce632284, 46 },
    { 0, 0 },
    { 0x91e32286, 61 },
    { 0, 0 },
    { 0x2c132288, 88 },
    { 0x8bf32289, 95 },
    { 0x4043228a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc803228d, 95 },
    { 0, 0 },
    { 0x6cd3228f, 98 },
    { 0x8f23228f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd32295, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c832298, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb322a3, 76 },
    { 0x76a322a4, 46 },
    { 0x8bd322a5, 98 },
    { 0xf51322a3, 85 },
    { 0x7c4322a5, 75 },
    { 0x67d322a3, 95 },
    { 0xf2a322a7, 100 },
    { 0x889322a4, 33 },
    { 0x50b322ab, 78 },
    { 0x4f6322ac, 95 },
    { 0xdb1322a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a322b0, 34 },
    { 0xfa2322b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f322b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x577322b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a322bb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b322cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75322d3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b322d9, 33 },
    { 0x4d5322d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46322e1, 76 },
    { 0xfef322e2, 98 },
    { 0x2b322e2, 61 },
    { 0xd71322e4, 95 },
    { 0, 0 },
    { 0xcb0322e6, 33 },
    { 0x41c322e7, 84 },
    { 0xe25322e6, 90 },
    { 0xe9d322e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97322f3, 52 },
    { 0x470322f3, 85 },
    { 0xb3e322f5, 66 },
    { 0xe0a322f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23322f9, 91 },
    { 0xd62322fa, 98 },
    { 0x69b322fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44322ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f32304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a932308, 93 },
    { 0, 0 },
    { 0x7623230a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f43230d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f732311, 67 },
    { 0, 0 },
    { 0x6d332313, 76 },
    { 0x14b32314, 58 },
    { 0, 0 },
    { 0xe2232316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef32319, 99 },
    { 0xd43231a, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e43231d, 33 },
    { 0x2e93231e, 55 },
    { 0, 0 },
    { 0xd8332320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b32324, 95 },
    { 0xb7532324, 66 },
    { 0x7d132326, 55 },
    { 0x66532327, 95 },
    { 0x79b32327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db3232e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f332331, 66 },
    { 0x22832332, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd983233e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd332341, 23 },
    { 0xc7532342, 95 },
    { 0x48b32341, 98 },
    { 0xbf132344, 76 },
    { 0x7da32344, 91 },
    { 0, 0 },
    { 0x31632347, 43 },
    { 0x6c932348, 78 },
    { 0xff132349, 55 },
    { 0xce432349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7432351, 33 },
    { 0xdd432351, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f432358, 7 },
    { 0, 0 },
    { 0x3b13235a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f3235e, 100 },
    { 0x9ba3235f, 52 },
    { 0x57932360, 98 },
    { 0xbcd32360, 96 },
    { 0xbba3235f, 90 },
    { 0, 0 },
    { 0x7a032364, 90 },
    { 0, 0 },
    { 0x10532366, 68 },
    { 0xdeb32366, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d3236a, 68 },
    { 0xf193236a, 100 },
    { 0x44a3236a, 95 },
    { 0xbdb3236c, 98 },
    { 0x4d3236e, 91 },
    { 0xf123236b, 78 },
    { 0xa1f32370, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x64932373, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a232378, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f3237b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f32383, 43 },
    { 0xa1232384, 33 },
    { 0x9fa32385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa393238b, 61 },
    { 0, 0 },
    { 0xfa23238d, 23 },
    { 0xf5d3238e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6132393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c32397, 91 },
    { 0xdb332398, 90 },
    { 0x93c32399, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb3239d, 95 },
    { 0x45a3239d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250323a4, 48 },
    { 0, 0 },
    { 0x737323a6, 85 },
    { 0xb39323a7, 43 },
    { 0x245323a8, 98 },
    { 0x213323a9, 43 },
    { 0x2b6323aa, 67 },
    { 0, 0 },
    { 0xdf5323ac, 61 },
    { 0x566323ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1323b8, 83 },
    { 0xb24323b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49323be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b323c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x946323c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec323ce, 80 },
    { 0x6dc323cf, 61 },
    { 0xdaa323ce, 23 },
    { 0x2ce323d1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7323d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2323d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8323e4, 91 },
    { 0xb81323e5, 61 },
    { 0xa65323e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb323ef, 87 },
    { 0x635323ef, 33 },
    { 0xc09323ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad323f5, 55 },
    { 0, 0 },
    { 0x4f5323f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5323fc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7323ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c32403, 81 },
    { 0xa5132403, 61 },
    { 0xf0432405, 46 },
    { 0, 0 },
    { 0x10032407, 34 },
    { 0, 0 },
    { 0xa1332409, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63532411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c32416, 95 },
    { 0x12032416, 99 },
    { 0x11332418, 33 },
    { 0x73b32419, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d3241f, 23 },
    { 0x7ae32420, 90 },
    { 0x36032420, 91 },
    { 0, 0 },
    { 0x98732423, 33 },
    { 0xeb632424, 85 },
    { 0x99e32425, 98 },
    { 0, 0 },
    { 0xbba32427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d3242c, 93 },
    { 0x8903242d, 77 },
    { 0xb943242e, 23 },
    { 0, 0 },
    { 0xcf132430, 88 },
    { 0, 0 },
    { 0x9ad32432, 77 },
    { 0x76432433, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb32436, 95 },
    { 0, 0 },
    { 0x21332438, 33 },
    { 0x33832438, 23 },
    { 0x5e232438, 97 },
    { 0xbe03243b, 61 },
    { 0x3432439, 75 },
    { 0x21832439, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce32449, 40 },
    { 0x4f53244a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e732454, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe83245a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc32460, 61 },
    { 0xd0932461, 91 },
    { 0, 0 },
    { 0x3bc32463, 78 },
    { 0xa7832463, 55 },
    { 0, 0 },
    { 0x35e32466, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1673246e, 90 },
    { 0x29e3246f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9233247a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c3247e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38832484, 33 },
    { 0xe3d32485, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d32489, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c23248c, 46 },
    { 0xa163248d, 46 },
    { 0x9ff3248e, 98 },
    { 0xcab3248f, 87 },
    { 0x5bb3248c, 98 },
    { 0x18332491, 90 },
    { 0x5e832492, 77 },
    { 0, 0 },
    { 0x3dc32494, 95 },
    { 0x98632495, 67 },
    { 0xa6332496, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a432499, 99 },
    { 0xad93249a, 95 },
    { 0xe3d3249b, 90 },
    { 0, 0 },
    { 0x8b23249d, 93 },
    { 0x10f3249e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b324a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262324a5, 61 },
    { 0x58c324a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615324ab, 85 },
    { 0x79324ac, 17 },
    { 0, 0 },
    { 0x6c3324ae, 76 },
    { 0x76324af, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65324b3, 33 },
    { 0x413324b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55324b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3324bc, 33 },
    { 0, 0 },
    { 0xabe324be, 33 },
    { 0x70a324bf, 23 },
    { 0xb61324c0, 61 },
    { 0, 0 },
    { 0xcd8324c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971324c6, 66 },
    { 0xd7324c6, 90 },
    { 0, 0 },
    { 0xa88324c9, 17 },
    { 0x1e2324ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b324ce, 43 },
    { 0x141324cf, 98 },
    { 0xb31324cf, 95 },
    { 0x6e5324d1, 61 },
    { 0xd87324d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888324d5, 61 },
    { 0x747324d5, 98 },
    { 0xcb324d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d324db, 95 },
    { 0, 0 },
    { 0x6d8324dd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab324e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f324e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d324e7, 76 },
    { 0, 0 },
    { 0x459324e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03324ed, 46 },
    { 0x404324ee, 43 },
    { 0x50a324ef, 33 },
    { 0x7cf324f0, 91 },
    { 0x4c4324f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a324f9, 99 },
    { 0x479324fa, 46 },
    { 0x324324fb, 55 },
    { 0x6ab324fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e32500, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7232503, 52 },
    { 0x1c32504, 91 },
    { 0xc6132505, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0232508, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe013250e, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b32512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18032519, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c32523, 61 },
    { 0x23132524, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13252b, 31 },
    { 0x7743252c, 38 },
    { 0x5e53252c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d32538, 66 },
    { 0, 0 },
    { 0x6ab3253a, 76 },
    { 0, 0 },
    { 0xde23253c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39932540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e32543, 65 },
    { 0x4bc32543, 55 },
    { 0x20f32545, 90 },
    { 0xa7732544, 61 },
    { 0x1df32547, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd3254a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf123254f, 91 },
    { 0xaf132550, 95 },
    { 0, 0 },
    { 0xec032552, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa343255a, 90 },
    { 0, 0 },
    { 0x2033255c, 33 },
    { 0xcd03255c, 34 },
    { 0xd013255e, 34 },
    { 0, 0 },
    { 0x5d132560, 90 },
    { 0xb2832560, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa583256c, 76 },
    { 0x91b3256d, 90 },
    { 0x39a3256c, 56 },
    { 0x5183256f, 23 },
    { 0x9c53256f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b32579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223257f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43732588, 76 },
    { 0x26c32589, 95 },
    { 0x5503258a, 91 },
    { 0xca73258b, 66 },
    { 0x56832589, 95 },
    { 0x42f3258d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f32591, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc32594, 46 },
    { 0x1f132595, 99 },
    { 0x55832596, 98 },
    { 0xf132596, 33 },
    { 0x70332596, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a3259b, 95 },
    { 0x2663259c, 17 },
    { 0xb943259d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86325a1, 34 },
    { 0, 0 },
    { 0x2af325a3, 95 },
    { 0x8d6325a4, 90 },
    { 0, 0 },
    { 0x2ee325a6, 23 },
    { 0, 0 },
    { 0xd5c325a8, 33 },
    { 0xca1325a8, 90 },
    { 0, 0 },
    { 0x410325ab, 66 },
    { 0x114325ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615325b1, 95 },
    { 0x99a325b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94325b6, 90 },
    { 0xcca325b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3325ba, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51325c0, 95 },
    { 0xdb325c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd325c4, 61 },
    { 0, 0 },
    { 0x1b5325c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x340325c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67325cc, 90 },
    { 0x48e325cd, 99 },
    { 0xef1325ce, 34 },
    { 0, 0 },
    { 0x259325d0, 90 },
    { 0, 0 },
    { 0x5d7325d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264325d9, 61 },
    { 0x63325da, 85 },
    { 0x810325da, 34 },
    { 0x9be325dc, 23 },
    { 0xfca325dd, 43 },
    { 0x735325de, 93 },
    { 0xd45325dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff325e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88325ea, 52 },
    { 0x19c325ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0325f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33325fb, 91 },
    { 0xaf7325fb, 95 },
    { 0x749325fd, 61 },
    { 0xa16325fc, 61 },
    { 0x9e7325ff, 90 },
    { 0x3ad325ff, 33 },
    { 0xbdf32601, 61 },
    { 0xeba32602, 88 },
    { 0xa7832602, 33 },
    { 0x9c632604, 46 },
    { 0xac532605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a832608, 77 },
    { 0x54132609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa632615, 23 },
    { 0xbf132616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d632619, 76 },
    { 0, 0 },
    { 0x91b3261b, 55 },
    { 0x51c3261c, 77 },
    { 0x5c33261d, 23 },
    { 0, 0 },
    { 0x2473261f, 76 },
    { 0x2f32620, 23 },
    { 0, 0 },
    { 0x71232622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c432628, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d3262e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc32631, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9533263c, 61 },
    { 0, 0 },
    { 0x7aa3263e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c32645, 52 },
    { 0x9232645, 93 },
    { 0x11932647, 100 },
    { 0xe7e32648, 61 },
    { 0x6ad32649, 61 },
    { 0, 0 },
    { 0x1ab3264b, 98 },
    { 0xf163264c, 85 },
    { 0x4503264d, 91 },
    { 0x53d3264b, 33 },
    { 0x65f3264f, 95 },
    { 0, 0 },
    { 0xc7732651, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5732658, 23 },
    { 0xfd832659, 98 },
    { 0x1b932659, 67 },
    { 0xe4532658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f32661, 98 },
    { 0xcba32661, 55 },
    { 0xc7b32661, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee232673, 66 },
    { 0x43732674, 61 },
    { 0xdb532675, 23 },
    { 0xecb32676, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5432679, 90 },
    { 0, 0 },
    { 0x8183267b, 76 },
    { 0xa603267b, 90 },
    { 0x9473267d, 61 },
    { 0xa383267e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4832683, 33 },
    { 0x7da32684, 77 },
    { 0x48732683, 33 },
    { 0xb8d32686, 90 },
    { 0xd3932686, 76 },
    { 0x6cb32688, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa3268c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a3268f, 46 },
    { 0x103268f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a32695, 76 },
    { 0, 0 },
    { 0x4bd32697, 90 },
    { 0xe7632698, 100 },
    { 0x1c832697, 55 },
    { 0x74a3269a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c326a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c326a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329326a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba326ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee326b2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7326b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7326ba, 55 },
    { 0xd13326bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af326be, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe326c4, 88 },
    { 0, 0 },
    { 0xb9d326c6, 100 },
    { 0xc40326c6, 23 },
    { 0x8ab326c8, 97 },
    { 0, 0 },
    { 0x3fe326ca, 66 },
    { 0, 0 },
    { 0xd79326cc, 55 },
    { 0xb68326cc, 46 },
    { 0x73f326ce, 34 },
    { 0xf5b326cf, 66 },
    { 0x2e8326d0, 98 },
    { 0x5a3326d0, 99 },
    { 0x592326d2, 91 },
    { 0xb0e326d1, 66 },
    { 0xd1326d4, 43 },
    { 0x7ec326d1, 33 },
    { 0xbbf326d1, 75 },
    { 0xcc0326d7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8326dd, 83 },
    { 0x52d326de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76326e6, 85 },
    { 0xa2c326e6, 75 },
    { 0x58d326e8, 77 },
    { 0x451326e6, 36 },
    { 0, 0 },
    { 0x828326eb, 91 },
    { 0x6f6326ec, 90 },
    { 0xfdb326ed, 43 },
    { 0x45e326ed, 76 },
    { 0x5cd326ed, 84 },
    { 0, 0 },
    { 0x186326f1, 90 },
    { 0xa40326f2, 33 },
    { 0x4df326f2, 98 },
    { 0x7bf326f4, 61 },
    { 0xc97326f5, 61 },
    { 0x7ba326f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82326fc, 94 },
    { 0xe52326fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59432701, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29232709, 43 },
    { 0x1f03270a, 33 },
    { 0x5643270a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb93270f, 23 },
    { 0x3663270f, 43 },
    { 0x24e32711, 90 },
    { 0x72d32712, 90 },
    { 0xdf532713, 90 },
    { 0x71032713, 87 },
    { 0x9ad32713, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3763271c, 33 },
    { 0x853271d, 98 },
    { 0xda53271d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9323272a, 76 },
    { 0x4d63272a, 34 },
    { 0x9a93272c, 90 },
    { 0x5243272a, 55 },
    { 0xb4e3272d, 90 },
    { 0, 0 },
    { 0x6f432730, 33 },
    { 0xe9332730, 17 },
    { 0x7e32732, 90 },
    { 0x6d932732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f63273a, 95 },
    { 0xa213273b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03273e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65832741, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91732746, 65 },
    { 0x55a32747, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf32750, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e732755, 33 },
    { 0x4e932756, 23 },
    { 0, 0 },
    { 0xe132758, 34 },
    { 0x14c32759, 61 },
    { 0x5773275a, 34 },
    { 0xb2b3275b, 77 },
    { 0x1623275c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b3275f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24132763, 61 },
    { 0, 0 },
    { 0xe4f32765, 90 },
    { 0xa7c32766, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb3276b, 33 },
    { 0, 0 },
    { 0xf23276d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b32773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b3277d, 90 },
    { 0, 0 },
    { 0xab03277f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc3278b, 46 },
    { 0xbcf3278c, 76 },
    { 0x4473278d, 61 },
    { 0x3b73278e, 61 },
    { 0x8043278c, 98 },
    { 0x4b63278d, 93 },
    { 0xf7032791, 91 },
    { 0x52f32792, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c132795, 23 },
    { 0x5d332796, 95 },
    { 0xdaa32797, 33 },
    { 0, 0 },
    { 0x76d32799, 34 },
    { 0x13b32799, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xec73279d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d327a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613327a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914327ae, 85 },
    { 0, 0 },
    { 0x765327b0, 52 },
    { 0xe2b327b0, 85 },
    { 0, 0 },
    { 0xd88327b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x910327b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f327b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea327bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f327c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898327c7, 98 },
    { 0, 0 },
    { 0x96327c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf327ce, 95 },
    { 0xa4b327ce, 100 },
    { 0x89d327cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e327d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8327da, 34 },
    { 0x216327db, 52 },
    { 0xc70327dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x496327df, 90 },
    { 0, 0 },
    { 0xe6d327e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd327e4, 81 },
    { 0x211327e5, 93 },
    { 0xd02327e5, 33 },
    { 0x351327e7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f327ea, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x706327ed, 90 },
    { 0x5b3327ee, 78 },
    { 0x8f3327ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea327f2, 61 },
    { 0, 0 },
    { 0x992327f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60327fd, 95 },
    { 0xd1e327fe, 77 },
    { 0x45c327fe, 90 },
    { 0xd3832800, 33 },
    { 0x6a932801, 46 },
    { 0x51832801, 93 },
    { 0xc5532803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3280e, 52 },
    { 0xbe43280f, 61 },
    { 0x64232810, 61 },
    { 0xd2b3280e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e32816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67232819, 100 },
    { 0x7c03281a, 46 },
    { 0x1e63281a, 85 },
    { 0x5f23281c, 23 },
    { 0, 0 },
    { 0x5553281e, 98 },
    { 0xedc3281f, 90 },
    { 0x5013281e, 66 },
    { 0xd0732821, 93 },
    { 0xf0a32822, 90 },
    { 0x7713281f, 76 },
    { 0xf932820, 87 },
    { 0, 0 },
    { 0x4432826, 23 },
    { 0xf0532827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5713282b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e3282f, 98 },
    { 0x80a32830, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9632833, 90 },
    { 0, 0 },
    { 0x42232835, 33 },
    { 0x97132836, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d3283e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61432845, 88 },
    { 0x2ca32845, 55 },
    { 0xd4d32847, 34 },
    { 0, 0 },
    { 0xe1032849, 78 },
    { 0x9193284a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963284f, 56 },
    { 0x4d32850, 61 },
    { 0, 0 },
    { 0x53132852, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce932856, 85 },
    { 0x1032856, 90 },
    { 0x2d632858, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c3285e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d632861, 65 },
    { 0, 0 },
    { 0x25632863, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a32866, 99 },
    { 0x59a32867, 90 },
    { 0xe8a32868, 75 },
    { 0xd7a32867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e43286c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf832872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c432875, 76 },
    { 0x12332875, 23 },
    { 0x7ae32875, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1423287a, 61 },
    { 0x7fd3287b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef53287f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f432882, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe32887, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe003288d, 33 },
    { 0x2bf3288e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e732899, 95 },
    { 0xdc032899, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe93289e, 95 },
    { 0x6c33289f, 90 },
    { 0, 0 },
    { 0xbf2328a1, 34 },
    { 0x1e9328a2, 61 },
    { 0xf07328a3, 33 },
    { 0, 0 },
    { 0x4cc328a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf328a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72328ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501328b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4328b8, 98 },
    { 0, 0 },
    { 0xe78328ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03328bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897328c1, 91 },
    { 0, 0 },
    { 0xea9328c3, 23 },
    { 0x216328c4, 76 },
    { 0x95b328c4, 95 },
    { 0, 0 },
    { 0x9ea328c7, 96 },
    { 0x98f328c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x798328cb, 23 },
    { 0, 0 },
    { 0xc38328cd, 93 },
    { 0x4b8328ce, 91 },
    { 0xe91328cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f328d2, 17 },
    { 0x45328d3, 33 },
    { 0, 0 },
    { 0xa54328d5, 75 },
    { 0, 0 },
    { 0xfd5328d7, 98 },
    { 0, 0 },
    { 0x3d0328d9, 61 },
    { 0xe87328d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba328dd, 84 },
    { 0x318328de, 90 },
    { 0, 0 },
    { 0xc57328e0, 61 },
    { 0xa01328e1, 98 },
    { 0x6c3328e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa328e6, 61 },
    { 0xd9b328e7, 23 },
    { 0, 0 },
    { 0x7e9328e9, 99 },
    { 0xa91328e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87328f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20328fa, 98 },
    { 0x7af328fb, 66 },
    { 0xad0328fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb32903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b32909, 91 },
    { 0xad33290a, 52 },
    { 0, 0 },
    { 0x4c73290c, 87 },
    { 0xb3f3290c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a32913, 99 },
    { 0x7c932914, 93 },
    { 0x22332915, 77 },
    { 0xa7932916, 23 },
    { 0x1a332917, 98 },
    { 0, 0 },
    { 0xd4432919, 55 },
    { 0x6b332919, 61 },
    { 0, 0 },
    { 0xc63291c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1032920, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c32923, 46 },
    { 0x29f32923, 84 },
    { 0x33a32924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0032929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f3292c, 34 },
    { 0, 0 },
    { 0x2113292e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5832931, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19132936, 90 },
    { 0, 0 },
    { 0x53232938, 90 },
    { 0xa3032939, 33 },
    { 0xc6832939, 95 },
    { 0, 0 },
    { 0x5813293c, 91 },
    { 0, 0 },
    { 0x8ac3293e, 95 },
    { 0x3713293e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe332945, 87 },
    { 0xb5f32946, 93 },
    { 0xcb132947, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85032953, 39 },
    { 0xb5632954, 38 },
    { 0x7aa32955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3663295e, 90 },
    { 0x4e93295e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14132968, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f3296b, 81 },
    { 0, 0 },
    { 0x6f53296d, 23 },
    { 0xee3296d, 90 },
    { 0, 0 },
    { 0xc5d32970, 34 },
    { 0, 0 },
    { 0x66232972, 23 },
    { 0xea632973, 99 },
    { 0x2db32972, 34 },
    { 0xa2532975, 99 },
    { 0x6a532976, 88 },
    { 0x59832977, 95 },
    { 0, 0 },
    { 0x80132979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a53297c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be32988, 86 },
    { 0xb3332989, 90 },
    { 0x2d03298a, 95 },
    { 0x3d832989, 55 },
    { 0xa013298c, 52 },
    { 0x1683298b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce32990, 23 },
    { 0xfee32991, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a32996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84832999, 34 },
    { 0xd493299a, 98 },
    { 0, 0 },
    { 0x1de3299c, 90 },
    { 0x92d3299d, 34 },
    { 0xad53299d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c329a7, 61 },
    { 0x7a4329a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8329ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa329b2, 90 },
    { 0, 0 },
    { 0x83b329b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb329b7, 76 },
    { 0xa08329b8, 84 },
    { 0xb1b329b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8329bc, 43 },
    { 0x116329bd, 91 },
    { 0xfba329bd, 55 },
    { 0xcc6329bf, 90 },
    { 0x85e329c0, 71 },
    { 0xfe329c1, 88 },
    { 0x87b329c2, 33 },
    { 0, 0 },
    { 0xc6a329c4, 91 },
    { 0xe8a329c4, 95 },
    { 0xc10329c6, 23 },
    { 0xa87329c6, 100 },
    { 0, 0 },
    { 0x78f329c9, 66 },
    { 0xebc329ca, 33 },
    { 0x1f1329ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b329ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb329d1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38329d5, 100 },
    { 0, 0 },
    { 0xe55329d7, 46 },
    { 0x719329d7, 90 },
    { 0, 0 },
    { 0xefc329da, 46 },
    { 0x428329da, 95 },
    { 0, 0 },
    { 0x133329dd, 91 },
    { 0x1d4329de, 77 },
    { 0xf61329df, 98 },
    { 0xce3329df, 91 },
    { 0x2d3329de, 95 },
    { 0xef2329e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66329ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b329f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab329fb, 61 },
    { 0x378329fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e432a05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7132a08, 61 },
    { 0x9732a09, 23 },
    { 0xe9232a0a, 33 },
    { 0x85a32a08, 61 },
    { 0, 0 },
    { 0xbc332a0d, 78 },
    { 0, 0 },
    { 0x3d932a0f, 90 },
    { 0x6cf32a10, 95 },
    { 0xf6b32a11, 88 },
    { 0, 0 },
    { 0x98b32a13, 33 },
    { 0x3b532a13, 76 },
    { 0x1a432a14, 84 },
    { 0xfa132a15, 55 },
    { 0xd4b32a17, 23 },
    { 0xe3f32a16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde232a1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b32a23, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e332a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49132a2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a532a2f, 98 },
    { 0xe9132a30, 95 },
    { 0xccb32a31, 52 },
    { 0, 0 },
    { 0xc9632a33, 34 },
    { 0xaba32a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94232a37, 46 },
    { 0xb7732a37, 61 },
    { 0x44732a39, 95 },
    { 0, 0 },
    { 0xfaf32a3b, 90 },
    { 0x61c32a3b, 99 },
    { 0xfa132a3d, 95 },
    { 0xa2732a3e, 91 },
    { 0x15532a3f, 61 },
    { 0xb7532a3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59732a43, 23 },
    { 0xff832a43, 95 },
    { 0x55432a45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e532a48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3632a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f32a4f, 76 },
    { 0, 0 },
    { 0xfb832a51, 33 },
    { 0, 0 },
    { 0xc4e32a53, 90 },
    { 0xf6032a53, 95 },
    { 0x74132a54, 76 },
    { 0, 0 },
    { 0xfb632a57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc32a5c, 23 },
    { 0, 0 },
    { 0x52432a5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1132a63, 66 },
    { 0x24332a63, 36 },
    { 0xff132a63, 100 },
    { 0x432a66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52432a6a, 61 },
    { 0, 0 },
    { 0xf032a6c, 78 },
    { 0x49732a6d, 87 },
    { 0x33632a6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb332a73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d32a7a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc32a7e, 95 },
    { 0x50a32a7f, 55 },
    { 0x7ce32a7e, 23 },
    { 0, 0 },
    { 0x61f32a82, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa732a8a, 99 },
    { 0x93d32a8b, 93 },
    { 0xb132a8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e32a91, 90 },
    { 0xcc232a91, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab32a95, 90 },
    { 0x7dd32a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5732a99, 23 },
    { 0x25d32a99, 43 },
    { 0, 0 },
    { 0x19e32a9c, 23 },
    { 0xc6732a9d, 93 },
    { 0x92832a9e, 52 },
    { 0xb2d32a9e, 98 },
    { 0, 0 },
    { 0x8a732aa1, 19 },
    { 0xd0e32aa1, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55132aa8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb832aae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4432ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6932abe, 98 },
    { 0, 0 },
    { 0x4a632ac0, 98 },
    { 0x8f632ac0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc432ac6, 80 },
    { 0x68632ac7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2432aca, 90 },
    { 0, 0 },
    { 0x71532acc, 61 },
    { 0x81d32acc, 66 },
    { 0, 0 },
    { 0x64932acf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x62332ad2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef332ad8, 52 },
    { 0x51f32ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b32ae1, 98 },
    { 0, 0 },
    { 0x3b332ae3, 46 },
    { 0x8e532ae3, 90 },
    { 0xc5932ae5, 87 },
    { 0x15c32ae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea32aed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b932af2, 61 },
    { 0, 0 },
    { 0x67232af4, 98 },
    { 0, 0 },
    { 0x36332af6, 43 },
    { 0x90a32af7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28132afc, 55 },
    { 0, 0 },
    { 0xb9432afe, 90 },
    { 0xf5b32afe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56832b02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90932b08, 91 },
    { 0x4ea32b09, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xada32b0c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18032b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f32b17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce332b1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6832b24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e32b28, 88 },
    { 0x2df32b29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7732b2c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1732b30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d32b34, 93 },
    { 0, 0 },
    { 0x69c32b36, 90 },
    { 0xa6d32b37, 61 },
    { 0, 0 },
    { 0x8a732b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4332b3d, 33 },
    { 0x67232b3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e32b47, 97 },
    { 0, 0 },
    { 0xab032b49, 23 },
    { 0, 0 },
    { 0xa3432b4b, 33 },
    { 0xa8f32b4c, 90 },
    { 0xbbf32b4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3832b50, 61 },
    { 0, 0 },
    { 0xd7e32b52, 76 },
    { 0xf9b32b53, 61 },
    { 0xe6232b54, 23 },
    { 0xef832b54, 23 },
    { 0, 0 },
    { 0x54232b57, 61 },
    { 0xea732b58, 55 },
    { 0xa3232b57, 34 },
    { 0xa8f32b5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83832b60, 55 },
    { 0x23432b61, 93 },
    { 0, 0 },
    { 0xe5732b63, 93 },
    { 0, 0 },
    { 0x22032b65, 76 },
    { 0, 0 },
    { 0x9dc32b67, 55 },
    { 0x99132b68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x29332b6b, 98 },
    { 0x65e32b6c, 67 },
    { 0x29f32b6b, 33 },
    { 0, 0 },
    { 0x1fc32b6f, 46 },
    { 0x53f32b70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f432b73, 93 },
    { 0x4b832b74, 98 },
    { 0x18d32b75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab732b78, 33 },
    { 0, 0 },
    { 0xfe732b7a, 87 },
    { 0xb4832b7a, 76 },
    { 0x35832b7c, 85 },
    { 0x32632b7d, 100 },
    { 0x75b32b7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f32b83, 90 },
    { 0, 0 },
    { 0xadc32b85, 23 },
    { 0, 0 },
    { 0xd4c32b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0832b95, 33 },
    { 0xc1d32b95, 23 },
    { 0, 0 },
    { 0x1e532b98, 27 },
    { 0x66932b98, 34 },
    { 0x5d632b9a, 76 },
    { 0, 0 },
    { 0x2a232b9c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xca032b9f, 84 },
    { 0x78532ba0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36432ba7, 61 },
    { 0x63f32ba8, 100 },
    { 0, 0 },
    { 0x9d632baa, 33 },
    { 0x6d432baa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e32bb2, 76 },
    { 0xa4032bb3, 90 },
    { 0, 0 },
    { 0x7e932bb5, 33 },
    { 0x62732bb6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d32bbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b32bbf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4032bc2, 95 },
    { 0, 0 },
    { 0xcd432bc4, 52 },
    { 0x7e632bc5, 90 },
    { 0xa7b32bc4, 90 },
    { 0, 0 },
    { 0xa7d32bc8, 23 },
    { 0xf832bc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d32bce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b32bd2, 76 },
    { 0xad532bd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef32bd6, 87 },
    { 0, 0 },
    { 0x96b32bd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94932bdd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22132be2, 75 },
    { 0xb9232be2, 95 },
    { 0, 0 },
    { 0x4cf32be5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c32bea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a32bee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17132bf2, 61 },
    { 0xd5d32bf3, 98 },
    { 0x6ed32bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d32bf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b432c01, 33 },
    { 0x52732c02, 33 },
    { 0, 0 },
    { 0x8f032c04, 66 },
    { 0xfb232c05, 90 },
    { 0x6c732c05, 33 },
    { 0, 0 },
    { 0x60132c08, 76 },
    { 0, 0 },
    { 0x7bd32c0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb332c16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d32c1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda932c1f, 67 },
    { 0x65232c1f, 98 },
    { 0x64632c21, 95 },
    { 0xf32c22, 98 },
    { 0x52832c23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a32c2c, 61 },
    { 0, 0 },
    { 0xdd32c2e, 98 },
    { 0xc7432c2f, 61 },
    { 0xe7032c2f, 90 },
    { 0xeab32c31, 90 },
    { 0xc8032c31, 98 },
    { 0x4a132c31, 78 },
    { 0xf3132c34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84332c37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8032c3b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3032c3e, 43 },
    { 0, 0 },
    { 0xfaf32c40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f632c43, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5032c4d, 23 },
    { 0, 0 },
    { 0x87732c4f, 88 },
    { 0x65332c50, 90 },
    { 0, 0 },
    { 0x69b32c52, 90 },
    { 0x5af32c53, 95 },
    { 0xd2432c53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f32c5e, 46 },
    { 0xd4232c5f, 55 },
    { 0, 0 },
    { 0xa8732c61, 98 },
    { 0x58032c62, 93 },
    { 0x5cf32c62, 33 },
    { 0x50e32c61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e32c70, 61 },
    { 0xde132c71, 66 },
    { 0x25332c72, 90 },
    { 0x9a332c73, 61 },
    { 0xf0132c74, 93 },
    { 0, 0 },
    { 0x57b32c76, 61 },
    { 0, 0 },
    { 0xd9532c78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a32c7c, 88 },
    { 0x4b032c7c, 90 },
    { 0, 0 },
    { 0x89232c7f, 99 },
    { 0, 0 },
    { 0x4ae32c81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a32c84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8832c8d, 33 },
    { 0xe3a32c8e, 61 },
    { 0x5c932c8f, 98 },
    { 0xfbb32c90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b32c93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d32c96, 91 },
    { 0x9b832c97, 46 },
    { 0x7832c98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xead32c9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5632c9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d32ca2, 95 },
    { 0, 0 },
    { 0x55a32ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff32ca7, 100 },
    { 0xba32ca8, 95 },
    { 0, 0 },
    { 0x5da32caa, 77 },
    { 0xa5032caa, 101 },
    { 0x55432cac, 51 },
    { 0x91132cad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e32cb1, 95 },
    { 0, 0 },
    { 0x75532cb3, 33 },
    { 0x57132cb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d032cb8, 59 },
    { 0x36332cb8, 95 },
    { 0xb4232cba, 98 },
    { 0xca032cb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18032cbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c32cc5, 33 },
    { 0xc6032cc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6532cc9, 75 },
    { 0x13432cc9, 95 },
    { 0, 0 },
    { 0xcba32ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32632cd6, 93 },
    { 0xb0432cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda432cdd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a932ce1, 87 },
    { 0x2de32ce2, 23 },
    { 0xb5832ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a532ce8, 100 },
    { 0x34a32ce8, 55 },
    { 0xf5e32cea, 81 },
    { 0, 0 },
    { 0x72332cec, 23 },
    { 0, 0 },
    { 0x19d32cee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c32cf5, 33 },
    { 0x53c32cf6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f232cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f732cff, 90 },
    { 0, 0 },
    { 0x46432d01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff32d04, 90 },
    { 0xf0f32d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb532d08, 75 },
    { 0x2ce32d09, 23 },
    { 0x5b932d0a, 52 },
    { 0x64832d0b, 66 },
    { 0x5fa32d0b, 90 },
    { 0x8632d0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20232d10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf232d13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4432d18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a332d1c, 67 },
    { 0, 0 },
    { 0x71032d1e, 98 },
    { 0xe1032d1f, 33 },
    { 0, 0 },
    { 0xb0832d21, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0532d28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f32d2b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80632d32, 78 },
    { 0x79432d33, 43 },
    { 0xde332d34, 61 },
    { 0x6fb32d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6832d39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa32d3d, 81 },
    { 0xb7532d3e, 98 },
    { 0, 0 },
    { 0xe4b32d40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92132d43, 33 },
    { 0x7cf32d44, 91 },
    { 0xc7232d45, 98 },
    { 0x43732d46, 46 },
    { 0, 0 },
    { 0xa632d48, 61 },
    { 0, 0 },
    { 0x1de32d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d932d4d, 33 },
    { 0xec932d4e, 61 },
    { 0x5fe32d4d, 33 },
    { 0xb7132d50, 61 },
    { 0, 0 },
    { 0x7ad32d52, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15232d55, 88 },
    { 0, 0 },
    { 0xbf132d57, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce32d5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c32d5f, 33 },
    { 0xf7132d60, 68 },
    { 0xe332d5f, 36 },
    { 0x44e32d60, 91 },
    { 0, 0 },
    { 0x61b32d64, 23 },
    { 0x89d32d64, 95 },
    { 0x37c32d64, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb832d6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf532d6e, 61 },
    { 0x37532d6f, 90 },
    { 0xbe532d6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a332d74, 99 },
    { 0xca432d75, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d32d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff32d7c, 52 },
    { 0xc4932d7c, 98 },
    { 0x9bd32d7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8132d82, 81 },
    { 0, 0 },
    { 0x2fc32d84, 34 },
    { 0x11732d85, 33 },
    { 0xa6432d86, 43 },
    { 0x14732d87, 93 },
    { 0, 0 },
    { 0xf6132d89, 98 },
    { 0, 0 },
    { 0x8aa32d8b, 90 },
    { 0x3e732d8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62932d90, 55 },
    { 0x16132d91, 43 },
    { 0xbc432d92, 95 },
    { 0x91732d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2132d9c, 23 },
    { 0x69d32d9c, 76 },
    { 0xdef32d9e, 98 },
    { 0x35a32d9f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x54932da2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5132da8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a32dae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95732db3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b32db7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5832dc2, 98 },
    { 0, 0 },
    { 0x86832dc4, 17 },
    { 0x77d32dc5, 80 },
    { 0xbf32dc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2732dcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e32dd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5932dd6, 95 },
    { 0, 0 },
    { 0xfcc32dd8, 90 },
    { 0xd7732dd8, 17 },
    { 0xa5332dda, 76 },
    { 0x25a32ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10432ddf, 90 },
    { 0xe2d32de0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85232de6, 36 },
    { 0x68132de7, 98 },
    { 0xcaa32de7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6732deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52132dee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7432df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe832df5, 95 },
    { 0x37f32df5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b432dfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc32e01, 81 },
    { 0xf4c32e02, 90 },
    { 0, 0 },
    { 0x8bf32e04, 90 },
    { 0xcd532e05, 93 },
    { 0xeb532e04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e832e0f, 61 },
    { 0, 0 },
    { 0xb3632e11, 90 },
    { 0x7f232e11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25632e19, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4932e1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b832e20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43632e27, 33 },
    { 0x18732e27, 61 },
    { 0xc8f32e28, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4532e2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba932e30, 90 },
    { 0, 0 },
    { 0x8ca32e32, 76 },
    { 0x92a32e32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x81832e36, 93 },
    { 0x34632e36, 90 },
    { 0x64732e38, 23 },
    { 0x6e232e37, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x22432e3c, 90 },
    { 0x35b32e3d, 43 },
    { 0x50c32e3d, 90 },
    { 0x9f032e3f, 33 },
    { 0xb3632e3e, 95 },
    { 0xdce32e3f, 48 },
    { 0, 0 },
    { 0x5e532e43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11332e49, 33 },
    { 0, 0 },
    { 0xdf332e4b, 93 },
    { 0xe9c32e4c, 33 },
    { 0x1d932e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b32e52, 85 },
    { 0xde132e53, 98 },
    { 0xa9832e53, 90 },
    { 0xeed32e55, 84 },
    { 0x21032e56, 93 },
    { 0, 0 },
    { 0x9e432e58, 55 },
    { 0xcd932e59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f32e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90232e69, 61 },
    { 0, 0 },
    { 0x8c032e6b, 66 },
    { 0, 0 },
    { 0x8e32e6d, 33 },
    { 0xb6132e6e, 90 },
    { 0xcff32e6f, 81 },
    { 0x20832e6e, 93 },
    { 0xda932e70, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c32e77, 40 },
    { 0x49232e77, 33 },
    { 0, 0 },
    { 0xd5732e7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7732e7e, 33 },
    { 0x23832e7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77232e82, 55 },
    { 0x68932e83, 76 },
    { 0x1ab32e84, 98 },
    { 0x96a32e83, 90 },
    { 0xb8932e86, 66 },
    { 0x7632e84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e32e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6532e91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36032e95, 90 },
    { 0x2632e96, 52 },
    { 0x29b32e97, 90 },
    { 0xef732e98, 33 },
    { 0, 0 },
    { 0x86432e9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a32e9e, 43 },
    { 0x96632e9e, 43 },
    { 0x39e32e9f, 91 },
    { 0x1bb32ea1, 23 },
    { 0xaa532ea1, 76 },
    { 0x96232ea3, 34 },
    { 0, 0 },
    { 0x6e232ea5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c32eaa, 33 },
    { 0x5d032eab, 34 },
    { 0x5cc32eac, 43 },
    { 0xe8d32ead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab332eb3, 33 },
    { 0x39d32eb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd32eb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5132ebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1732ebf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff32ec5, 91 },
    { 0xa5932ec5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47832ecc, 90 },
    { 0x7fa32ecc, 98 },
    { 0x8ea32ece, 33 },
    { 0xa0e32ecf, 90 },
    { 0xbd632ecd, 61 },
    { 0xe7e32ed1, 23 },
    { 0x93a32ed2, 95 },
    { 0x64e32ecd, 78 },
    { 0x88e32ed4, 33 },
    { 0xe2032ed5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x11932ed8, 90 },
    { 0xf4932ed9, 23 },
    { 0x50032ed9, 33 },
    { 0x31a32ed8, 46 },
    { 0xa8a32edc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead32ee0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4132ee7, 95 },
    { 0x6af32ee8, 90 },
    { 0x7f032ee9, 33 },
    { 0, 0 },
    { 0x5a632eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8132ef5, 52 },
    { 0x81b32ef5, 90 },
    { 0, 0 },
    { 0x27a32ef8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47632efb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d32efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa632f02, 100 },
    { 0xca132f03, 93 },
    { 0xb3032f04, 81 },
    { 0x25832f05, 77 },
    { 0x8bf32f06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c32f09, 36 },
    { 0, 0 },
    { 0x12032f0b, 76 },
    { 0x9e832f0c, 90 },
    { 0x47832f0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc432f11, 61 },
    { 0x4c832f12, 46 },
    { 0x2a932f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba332f24, 90 },
    { 0xee532f25, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f032f2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46832f2d, 93 },
    { 0xd1132f2e, 23 },
    { 0xce32f2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6932f35, 33 },
    { 0x33532f36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed32f3b, 23 },
    { 0xc6f32f3c, 61 },
    { 0x59b32f3b, 90 },
    { 0xa8332f3b, 95 },
    { 0, 0 },
    { 0x5a332f40, 99 },
    { 0, 0 },
    { 0xd0332f42, 61 },
    { 0x53d32f42, 61 },
    { 0, 0 },
    { 0x88632f45, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e632f4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e32f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7032f57, 98 },
    { 0x6e232f58, 66 },
    { 0, 0 },
    { 0xef432f5a, 95 },
    { 0x25832f5b, 42 },
    { 0, 0 },
    { 0xbe532f5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c32f66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a32f6c, 43 },
    { 0x21f32f6d, 76 },
    { 0xf3b32f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e32f7a, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58632f7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc432f81, 87 },
    { 0xfbe32f82, 98 },
    { 0, 0 },
    { 0x82332f84, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97332f88, 33 },
    { 0xccc32f89, 76 },
    { 0, 0 },
    { 0x20932f8b, 90 },
    { 0, 0 },
    { 0x2a232f8d, 91 },
    { 0, 0 },
    { 0xbd932f8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78532f96, 90 },
    { 0, 0 },
    { 0xe8332f98, 23 },
    { 0xba132f99, 33 },
    { 0, 0 },
    { 0x59a32f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c032fa2, 90 },
    { 0xe0932fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53432fa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75832fac, 98 },
    { 0xc9f32fac, 61 },
    { 0xa6f32fae, 81 },
    { 0xf6732faf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d32fb3, 67 },
    { 0x16b32fb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f32fc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc9, 62 },
    { 0xbc432fca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b32fce, 90 },
    { 0x60d32fcf, 77 },
    { 0x9ff32fcf, 95 },
    { 0x50a32fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb32fd4, 98 },
    { 0, 0 },
    { 0x9f432fd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92032fe0, 53 },
    { 0, 0 },
    { 0x92c32fe2, 98 },
    { 0, 0 },
    { 0x93932fe4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5932fe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b32fed, 61 },
    { 0x28532fee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b32ff3, 98 },
    { 0, 0 },
    { 0x65632ff5, 23 },
    { 0x9b32ff6, 55 },
    { 0, 0 },
    { 0x59332ff8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67632ffb, 90 },
    { 0x3ff32ffc, 40 },
    { 0x61432ffd, 76 },
    { 0x6ac32ffe, 33 },
    { 0x75932fff, 33 },
    { 0xd5032ffb, 98 },
    { 0, 0 },
    { 0x49633002, 55 },
    { 0xac233003, 66 },
    { 0, 0 },
    { 0x83233005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac633008, 93 },
    { 0, 0 },
    { 0xc653300a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3300e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1833016, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993301c, 55 },
    { 0, 0 },
    { 0xcc23301e, 55 },
    { 0, 0 },
    { 0x5cc33020, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e433025, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53302e, 76 },
    { 0x4103302e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb33036, 87 },
    { 0x34e33037, 34 },
    { 0, 0 },
    { 0x90f33039, 80 },
    { 0x4763303a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc583304d, 66 },
    { 0x2293304e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d733052, 17 },
    { 0x9f033053, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70133056, 84 },
    { 0x2f033057, 90 },
    { 0xa3333058, 98 },
    { 0, 0 },
    { 0x72f3305a, 95 },
    { 0xae3305b, 99 },
    { 0, 0 },
    { 0x3ac3305d, 23 },
    { 0, 0 },
    { 0x5a3305f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a33075, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca33079, 61 },
    { 0xe9533079, 61 },
    { 0xc9d3307b, 61 },
    { 0x8ed3307a, 66 },
    { 0xf123307d, 55 },
    { 0x8853307d, 33 },
    { 0x2da3307f, 61 },
    { 0, 0 },
    { 0x75c33081, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4323308c, 17 },
    { 0, 0 },
    { 0xb363308e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4793309b, 90 },
    { 0x3493309c, 95 },
    { 0xce83309b, 33 },
    { 0, 0 },
    { 0x7483309f, 100 },
    { 0xa61330a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e330a5, 33 },
    { 0, 0 },
    { 0x829330a7, 77 },
    { 0x67b330a7, 33 },
    { 0x360330a9, 98 },
    { 0, 0 },
    { 0x6c7330ab, 88 },
    { 0xbba330ac, 90 },
    { 0xd1a330ad, 90 },
    { 0x2cf330ae, 62 },
    { 0xed4330ae, 90 },
    { 0xf12330ad, 61 },
    { 0x163330ab, 76 },
    { 0x83a330b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e330b5, 43 },
    { 0x24c330b6, 33 },
    { 0x9d1330b7, 55 },
    { 0x1e0330b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c330c0, 95 },
    { 0x9bd330c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3330c4, 76 },
    { 0x7cd330c5, 99 },
    { 0x80e330c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791330cb, 95 },
    { 0x3c330cb, 34 },
    { 0x1be330cd, 94 },
    { 0xbf6330cc, 99 },
    { 0x296330cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35330d5, 90 },
    { 0x7a8330d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f330e0, 33 },
    { 0xb30330e1, 33 },
    { 0xe5f330e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f330e5, 61 },
    { 0x45e330e6, 90 },
    { 0xee2330e6, 91 },
    { 0x35330e6, 84 },
    { 0x8a0330e9, 87 },
    { 0x211330e9, 52 },
    { 0x163330eb, 23 },
    { 0x99c330e9, 99 },
    { 0xc00330ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6330f0, 23 },
    { 0, 0 },
    { 0xa9c330f2, 53 },
    { 0xb4a330f3, 33 },
    { 0, 0 },
    { 0xa1d330f5, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753330fa, 96 },
    { 0, 0 },
    { 0xdeb330fc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3330ff, 90 },
    { 0xf2733100, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53833104, 76 },
    { 0xa3333105, 76 },
    { 0x35c33106, 55 },
    { 0xffb33105, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e83310a, 76 },
    { 0xbea3310b, 61 },
    { 0x1453310a, 90 },
    { 0x6513310c, 61 },
    { 0x3df3310a, 34 },
    { 0xac63310f, 98 },
    { 0, 0 },
    { 0xec533111, 43 },
    { 0xb0433112, 61 },
    { 0xb8b33113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92533118, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9893311b, 76 },
    { 0, 0 },
    { 0xd133311d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42133123, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f033127, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e63312e, 68 },
    { 0x13f3312f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef33134, 75 },
    { 0xba633135, 33 },
    { 0xf5533136, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0133139, 61 },
    { 0xa5833139, 76 },
    { 0x1383313a, 58 },
    { 0x8573313b, 98 },
    { 0x83f3313d, 33 },
    { 0xace3313d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3833141, 33 },
    { 0xc433142, 55 },
    { 0xbb633143, 23 },
    { 0x63a33143, 23 },
    { 0xfeb33145, 33 },
    { 0, 0 },
    { 0x3f833147, 53 },
    { 0x97133148, 90 },
    { 0xd2533149, 64 },
    { 0, 0 },
    { 0xff3314b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf23314f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33153, 77 },
    { 0x53733153, 91 },
    { 0x7ac33155, 61 },
    { 0x5533156, 33 },
    { 0, 0 },
    { 0xac533158, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf3315b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb683315e, 23 },
    { 0x66f3315f, 90 },
    { 0, 0 },
    { 0x4ab33161, 98 },
    { 0x8a033162, 95 },
    { 0x6ae33162, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce733167, 90 },
    { 0x94b33168, 98 },
    { 0x1c533169, 98 },
    { 0xa1e33168, 61 },
    { 0, 0 },
    { 0xf5b3316c, 98 },
    { 0x8133316c, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7633171, 90 },
    { 0, 0 },
    { 0x45533173, 76 },
    { 0xc3e33173, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e933177, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b3317f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de33187, 97 },
    { 0x84c33188, 66 },
    { 0xdf133189, 95 },
    { 0, 0 },
    { 0x7d83318b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f33195, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c233199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a73319e, 23 },
    { 0xca13319f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728331a4, 49 },
    { 0x93f331a5, 90 },
    { 0x807331a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52331a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40331ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x264331b0, 33 },
    { 0, 0 },
    { 0x3a5331b2, 97 },
    { 0, 0 },
    { 0xe5e331b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7331b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34331bb, 61 },
    { 0x19331bb, 67 },
    { 0, 0 },
    { 0xee3331be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430331c2, 61 },
    { 0x7f8331c2, 76 },
    { 0x3d8331c2, 98 },
    { 0xa3a331c2, 87 },
    { 0x8d331c6, 43 },
    { 0xdc5331c4, 100 },
    { 0xa64331c8, 46 },
    { 0x64e331c8, 98 },
    { 0x8aa331c8, 34 },
    { 0, 0 },
    { 0xa7d331cc, 93 },
    { 0, 0 },
    { 0xfed331ce, 98 },
    { 0, 0 },
    { 0x5dd331d0, 23 },
    { 0, 0 },
    { 0x27f331d2, 38 },
    { 0, 0 },
    { 0x95d331d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856331d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c331dc, 90 },
    { 0x315331dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7331e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4331e7, 91 },
    { 0, 0 },
    { 0xe14331e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983331ed, 56 },
    { 0x83c331ee, 43 },
    { 0x4f1331ee, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80331f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9331fa, 88 },
    { 0, 0 },
    { 0x997331fc, 98 },
    { 0, 0 },
    { 0xc32331fe, 87 },
    { 0xc26331ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb733205, 68 },
    { 0, 0 },
    { 0xab333207, 90 },
    { 0x29033208, 87 },
    { 0xce233209, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8633320d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c233212, 40 },
    { 0x1de33213, 90 },
    { 0xaf133212, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b233218, 61 },
    { 0xab233219, 85 },
    { 0x4c133218, 100 },
    { 0x8de3321b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a53321f, 87 },
    { 0x13a33220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14733223, 81 },
    { 0x85f33224, 99 },
    { 0x50833225, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe093322a, 90 },
    { 0x9223322a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa3322e, 33 },
    { 0x3c23322f, 43 },
    { 0x4e233230, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f33233, 33 },
    { 0, 0 },
    { 0x7bb33235, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8273323c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b33240, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a33245, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4133324b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c3324e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db33251, 95 },
    { 0xe9333251, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b23325a, 43 },
    { 0x4413325a, 38 },
    { 0xc613325a, 91 },
    { 0xbb53325d, 95 },
    { 0, 0 },
    { 0x9a63325f, 61 },
    { 0, 0 },
    { 0x75533261, 90 },
    { 0, 0 },
    { 0x9e333263, 93 },
    { 0x6d433263, 76 },
    { 0x2f633263, 55 },
    { 0, 0 },
    { 0x9e933267, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf583326b, 23 },
    { 0x5eb3326c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af3326f, 33 },
    { 0xcfe33270, 99 },
    { 0xc8233271, 52 },
    { 0x3053326f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19833275, 81 },
    { 0x40d33275, 61 },
    { 0x6cd33277, 95 },
    { 0xbd433276, 90 },
    { 0, 0 },
    { 0x3113327a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5673327e, 98 },
    { 0x8813327e, 88 },
    { 0x9f93327e, 55 },
    { 0x64133281, 33 },
    { 0x68c33281, 55 },
    { 0, 0 },
    { 0xe4d33284, 33 },
    { 0, 0 },
    { 0xab733286, 91 },
    { 0, 0 },
    { 0x5cf33288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3403328d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f933294, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb33298, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa73329b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40332a1, 98 },
    { 0xba1332a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a332aa, 95 },
    { 0x4f9332aa, 98 },
    { 0x5a9332ac, 91 },
    { 0, 0 },
    { 0xd09332ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d332bd, 98 },
    { 0x8a4332bd, 66 },
    { 0, 0 },
    { 0xa7a332c0, 98 },
    { 0x58e332c0, 90 },
    { 0xe7c332c1, 64 },
    { 0x677332c3, 43 },
    { 0xbca332c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170332ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f332da, 46 },
    { 0x653332db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39332e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de332ed, 55 },
    { 0x6b0332ee, 90 },
    { 0, 0 },
    { 0x8e9332f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4332f5, 23 },
    { 0xf59332f6, 95 },
    { 0x43c332f7, 77 },
    { 0x53d332f6, 91 },
    { 0x425332f9, 66 },
    { 0xf49332fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5833300, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b33307, 100 },
    { 0, 0 },
    { 0xf1c33309, 61 },
    { 0xcf3330a, 90 },
    { 0x513330b, 90 },
    { 0x70e3330c, 99 },
    { 0x183330d, 90 },
    { 0x87a3330e, 65 },
    { 0x4103330f, 61 },
    { 0, 0 },
    { 0xe4a33311, 61 },
    { 0, 0 },
    { 0xb1f33313, 91 },
    { 0xc9a33314, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19333318, 65 },
    { 0xee333318, 23 },
    { 0xd213331a, 61 },
    { 0xd833331b, 87 },
    { 0x5a533318, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba3331f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef133323, 98 },
    { 0x26a33323, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd3332b, 98 },
    { 0x6e63332b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c433331, 61 },
    { 0x79e33332, 90 },
    { 0, 0 },
    { 0xf1833334, 76 },
    { 0, 0 },
    { 0x46533336, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x29233339, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093333c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3643333f, 61 },
    { 0xd1733340, 76 },
    { 0x4ee33341, 90 },
    { 0x56a33342, 52 },
    { 0x2d033343, 56 },
    { 0, 0 },
    { 0xaa533345, 76 },
    { 0xcc33346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe3334e, 55 },
    { 0x3df3334f, 67 },
    { 0x4833334e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9933355, 68 },
    { 0x10833356, 33 },
    { 0, 0 },
    { 0x2ae33358, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7453335b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f3335e, 36 },
    { 0x3943335f, 23 },
    { 0, 0 },
    { 0x11433361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e33365, 33 },
    { 0x8b533365, 95 },
    { 0xe8c33366, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a3336d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3123337a, 99 },
    { 0, 0 },
    { 0xd343337c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3333382, 90 },
    { 0x37633382, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6033386, 23 },
    { 0, 0 },
    { 0x7be33388, 33 },
    { 0, 0 },
    { 0x2ae3338a, 67 },
    { 0, 0 },
    { 0x8783338c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf333393, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e33396, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee3339a, 100 },
    { 0, 0 },
    { 0x923339c, 61 },
    { 0x34b3339d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b333a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db333a5, 23 },
    { 0x88333a6, 33 },
    { 0xe6333a7, 76 },
    { 0x611333a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e333ab, 61 },
    { 0xfe4333ab, 33 },
    { 0x814333ab, 23 },
    { 0xbe5333ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x584333b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218333b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x419333b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197333bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9333c1, 55 },
    { 0x443333c2, 61 },
    { 0x638333c3, 90 },
    { 0xef1333c2, 99 },
    { 0xaab333c2, 96 },
    { 0xb6d333c6, 55 },
    { 0xe34333c7, 93 },
    { 0x76a333c7, 34 },
    { 0xdbc333c8, 55 },
    { 0xe40333c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7333ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3333d4, 77 },
    { 0, 0 },
    { 0x326333d6, 33 },
    { 0x623333d6, 98 },
    { 0xecd333d6, 95 },
    { 0x93f333d8, 23 },
    { 0, 0 },
    { 0xfa2333db, 23 },
    { 0x2b2333db, 90 },
    { 0x377333dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45333e0, 23 },
    { 0xea1333e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf333e6, 55 },
    { 0x613333e6, 76 },
    { 0xcca333e7, 98 },
    { 0xca0333e9, 61 },
    { 0x1b4333e6, 48 },
    { 0, 0 },
    { 0xc58333ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d333ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea333f2, 80 },
    { 0x3d6333f2, 61 },
    { 0, 0 },
    { 0x86b333f5, 33 },
    { 0xb2a333f6, 43 },
    { 0xd38333f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1333fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b333ff, 100 },
    { 0, 0 },
    { 0x13733401, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x43433404, 66 },
    { 0, 0 },
    { 0x77c33406, 23 },
    { 0, 0 },
    { 0xf0d33408, 98 },
    { 0, 0 },
    { 0xeeb3340a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a3340d, 23 },
    { 0x89f3340e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x40333411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff633414, 46 },
    { 0xa5a33414, 23 },
    { 0x48c33416, 99 },
    { 0x11d33416, 33 },
    { 0, 0 },
    { 0xe8933419, 90 },
    { 0x5d73341a, 33 },
    { 0x4f53341b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb373341e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba33423, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2893342c, 90 },
    { 0x2e63342d, 99 },
    { 0xdb23342c, 33 },
    { 0xf593342f, 95 },
    { 0x5973342c, 95 },
    { 0xed233431, 99 },
    { 0xd9a33431, 65 },
    { 0xf1133432, 90 },
    { 0, 0 },
    { 0x8a533435, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4613343b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533343f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c33442, 61 },
    { 0, 0 },
    { 0xb7e33444, 46 },
    { 0, 0 },
    { 0xf9a33446, 76 },
    { 0xd9033447, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c43344e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc33451, 90 },
    { 0xf9a33452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d033459, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9803345e, 61 },
    { 0, 0 },
    { 0xbec33460, 78 },
    { 0x96133460, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a33464, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f633468, 90 },
    { 0x1c033469, 55 },
    { 0xbde33468, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc603346d, 90 },
    { 0x2233346e, 93 },
    { 0x7773346e, 43 },
    { 0x8963346e, 43 },
    { 0xb9933471, 33 },
    { 0x38b33472, 23 },
    { 0x24233472, 67 },
    { 0x45e33470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65533478, 23 },
    { 0xa7333479, 95 },
    { 0x6ad3347a, 34 },
    { 0xf8a3347b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb153347f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0033482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada33486, 33 },
    { 0, 0 },
    { 0xff33488, 76 },
    { 0, 0 },
    { 0x32e3348a, 78 },
    { 0x4ae3348a, 61 },
    { 0x9793348c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d33493, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x78433496, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5733499, 33 },
    { 0, 0 },
    { 0xf273349b, 95 },
    { 0, 0 },
    { 0x8003349d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x550334a0, 90 },
    { 0, 0 },
    { 0x7ec334a2, 52 },
    { 0x738334a3, 76 },
    { 0x5bb334a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a334ab, 61 },
    { 0x993334ac, 76 },
    { 0xc8c334ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x837334b0, 34 },
    { 0xd1f334b0, 98 },
    { 0xfbf334b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e334b7, 91 },
    { 0, 0 },
    { 0x307334b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3334be, 90 },
    { 0x3e2334bf, 95 },
    { 0xe41334c0, 33 },
    { 0xbf7334bf, 90 },
    { 0x9e7334be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d334c7, 98 },
    { 0, 0 },
    { 0xa3d334c9, 23 },
    { 0xa9c334ca, 61 },
    { 0x1f5334cb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f334cf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46334d3, 93 },
    { 0, 0 },
    { 0xec3334d5, 76 },
    { 0, 0 },
    { 0xc9b334d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8334df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1334e2, 78 },
    { 0, 0 },
    { 0xb1a334e4, 91 },
    { 0x5e334e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88334e8, 75 },
    { 0, 0 },
    { 0x932334ea, 95 },
    { 0x551334eb, 66 },
    { 0, 0 },
    { 0x5b2334ed, 76 },
    { 0x3cc334ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04334f4, 34 },
    { 0, 0 },
    { 0xe65334f6, 93 },
    { 0, 0 },
    { 0xd20334f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835334fc, 33 },
    { 0, 0 },
    { 0xb1f334fe, 35 },
    { 0x54334ff, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x44433502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a033505, 33 },
    { 0, 0 },
    { 0x91b33507, 90 },
    { 0x63733508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc3350c, 69 },
    { 0x723350c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd33516, 90 },
    { 0, 0 },
    { 0xa6133518, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa453351b, 55 },
    { 0, 0 },
    { 0xeef3351d, 46 },
    { 0xdde3351d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88333521, 55 },
    { 0, 0 },
    { 0x3b133523, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda33526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c3352d, 59 },
    { 0x8a3352e, 95 },
    { 0xabd3352f, 52 },
    { 0x9f733530, 87 },
    { 0x6c533531, 100 },
    { 0x35333530, 98 },
    { 0x9c133533, 23 },
    { 0x62f33531, 23 },
    { 0xc8533535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab733539, 61 },
    { 0xd513353a, 34 },
    { 0xb7d3353b, 43 },
    { 0xd203353c, 85 },
    { 0x613353d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6833540, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1133545, 78 },
    { 0, 0 },
    { 0xfa833547, 61 },
    { 0x8333547, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c3354e, 81 },
    { 0xca33354f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c33555, 33 },
    { 0x28233556, 33 },
    { 0x48233557, 81 },
    { 0x59233558, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f23355b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde3355e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a3356b, 55 },
    { 0x3313356b, 100 },
    { 0, 0 },
    { 0xfea3356e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70033576, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f633579, 17 },
    { 0x31933579, 99 },
    { 0x48933579, 98 },
    { 0, 0 },
    { 0xec83357d, 78 },
    { 0xc613357e, 61 },
    { 0xf573357d, 55 },
    { 0xa803357d, 33 },
    { 0x5b933581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9333587, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe843358c, 95 },
    { 0x4933358d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85033591, 95 },
    { 0xf1333591, 65 },
    { 0, 0 },
    { 0x6ed33594, 34 },
    { 0x43e33595, 85 },
    { 0x53c33595, 33 },
    { 0xbf633595, 98 },
    { 0x4fa33598, 34 },
    { 0x41333594, 33 },
    { 0xa8a33596, 90 },
    { 0, 0 },
    { 0x5a73359c, 61 },
    { 0x9cf3359c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194335a1, 98 },
    { 0xafb335a2, 33 },
    { 0, 0 },
    { 0x38c335a4, 61 },
    { 0x174335a5, 43 },
    { 0x839335a6, 46 },
    { 0x4d2335a7, 61 },
    { 0xbc7335a7, 33 },
    { 0x404335a4, 98 },
    { 0xf75335a5, 43 },
    { 0x917335ab, 33 },
    { 0, 0 },
    { 0x3335ad, 95 },
    { 0, 0 },
    { 0x65f335af, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4335b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97335b8, 95 },
    { 0x8df335b8, 39 },
    { 0x80d335ba, 33 },
    { 0xe20335b8, 61 },
    { 0xf05335bc, 33 },
    { 0, 0 },
    { 0xf19335be, 55 },
    { 0, 0 },
    { 0x9d6335c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5335c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76335cd, 23 },
    { 0x947335ce, 33 },
    { 0, 0 },
    { 0xc20335d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26335d3, 90 },
    { 0, 0 },
    { 0x22f335d5, 95 },
    { 0x98a335d6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb335da, 23 },
    { 0x303335db, 90 },
    { 0xa4b335dc, 67 },
    { 0xd6f335dd, 99 },
    { 0, 0 },
    { 0xd2b335df, 23 },
    { 0x216335df, 61 },
    { 0, 0 },
    { 0xf8e335e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0335e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978335eb, 17 },
    { 0xcc9335ec, 93 },
    { 0xa8c335ed, 95 },
    { 0xab1335ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d335f1, 33 },
    { 0, 0 },
    { 0x12b335f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x952335f6, 33 },
    { 0x24f335f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3335fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73933601, 76 },
    { 0xd733601, 90 },
    { 0x89b33603, 95 },
    { 0, 0 },
    { 0x68833605, 33 },
    { 0x87733606, 91 },
    { 0xe6d33607, 98 },
    { 0xce133606, 90 },
    { 0, 0 },
    { 0x50b3360a, 33 },
    { 0x8473360b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66133613, 34 },
    { 0x60f33613, 43 },
    { 0x19c33615, 46 },
    { 0x25433616, 43 },
    { 0, 0 },
    { 0x69f33618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc73361b, 85 },
    { 0x8193361b, 33 },
    { 0x2ea3361d, 33 },
    { 0x3943361e, 90 },
    { 0x86a3361f, 95 },
    { 0x72b33620, 78 },
    { 0xdcf3361c, 23 },
    { 0x6a23361c, 23 },
    { 0, 0 },
    { 0xb6833624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a33628, 23 },
    { 0xf933628, 95 },
    { 0xc163362a, 88 },
    { 0xdcc3362a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e33636, 95 },
    { 0, 0 },
    { 0x5ab33638, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4103363b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0433642, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc33645, 55 },
    { 0xd333646, 98 },
    { 0, 0 },
    { 0xbab33648, 64 },
    { 0x37c33649, 23 },
    { 0x73033648, 81 },
    { 0, 0 },
    { 0x5f73364c, 98 },
    { 0xfd23364d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9233650, 66 },
    { 0x20d33651, 85 },
    { 0xa3633650, 95 },
    { 0x6d533652, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f933657, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e33365b, 98 },
    { 0, 0 },
    { 0x6e23365d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60133669, 95 },
    { 0x11e3366a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9063366e, 93 },
    { 0x6d53366f, 61 },
    { 0xf8b3366f, 46 },
    { 0xba73366f, 61 },
    { 0, 0 },
    { 0xde433673, 33 },
    { 0x2f533674, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e13367a, 80 },
    { 0x4f53367b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed3367e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e33685, 98 },
    { 0xbae33686, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xab533689, 84 },
    { 0x45b33689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb23368d, 33 },
    { 0xee63368d, 81 },
    { 0x9e53368d, 33 },
    { 0, 0 },
    { 0x2f533691, 95 },
    { 0, 0 },
    { 0x20633693, 61 },
    { 0xf333693, 99 },
    { 0xe8933695, 55 },
    { 0x7b633695, 95 },
    { 0x33733696, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13336a0, 23 },
    { 0x3fd336a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9336a5, 92 },
    { 0xdb3336a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b336a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65336ad, 98 },
    { 0xcaf336ad, 43 },
    { 0, 0 },
    { 0x1af336b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b336b3, 23 },
    { 0xcc8336b4, 34 },
    { 0x5d1336b4, 55 },
    { 0xff9336b6, 91 },
    { 0x8ac336b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a336c3, 95 },
    { 0x8d7336c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea336c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x856336cc, 23 },
    { 0x27f336cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e336d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781336db, 81 },
    { 0, 0 },
    { 0x69a336dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141336e1, 55 },
    { 0, 0 },
    { 0x3b1336e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae336e8, 95 },
    { 0xf0f336e9, 66 },
    { 0x7aa336ea, 81 },
    { 0x9bb336eb, 90 },
    { 0, 0 },
    { 0x1a2336ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a336f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78336fc, 87 },
    { 0x4c9336fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c33701, 90 },
    { 0, 0 },
    { 0xf3233703, 98 },
    { 0xce333704, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af33707, 95 },
    { 0x7c933708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d3370c, 43 },
    { 0x4ca3370d, 91 },
    { 0xe723370e, 61 },
    { 0x6c13370d, 93 },
    { 0, 0 },
    { 0x33933711, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af33718, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c3371b, 66 },
    { 0, 0 },
    { 0xbd53371d, 99 },
    { 0, 0 },
    { 0x8c43371f, 46 },
    { 0xac933720, 95 },
    { 0xcf033721, 88 },
    { 0xbb633722, 23 },
    { 0xd8e33720, 33 },
    { 0x8c733722, 33 },
    { 0xbeb33724, 52 },
    { 0x32233726, 90 },
    { 0, 0 },
    { 0x68b33728, 52 },
    { 0x82d33728, 34 },
    { 0x2e93372a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf3372d, 46 },
    { 0, 0 },
    { 0x273372f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf3373a, 43 },
    { 0x2a53373a, 43 },
    { 0x9b53373c, 8 },
    { 0, 0 },
    { 0x2a43373e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94933741, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae33744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d73374a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a33374d, 61 },
    { 0x4193374d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a33754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8933757, 98 },
    { 0xfae33757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9823375e, 33 },
    { 0xae83375f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e33763, 23 },
    { 0x60633763, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb33767, 61 },
    { 0xd6a33768, 33 },
    { 0xb9133769, 78 },
    { 0xa383376a, 98 },
    { 0x7913376a, 95 },
    { 0xd93376b, 33 },
    { 0, 0 },
    { 0xb833376e, 95 },
    { 0x1053376f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d33775, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc013377a, 34 },
    { 0x52e3377a, 55 },
    { 0, 0 },
    { 0xf183377d, 46 },
    { 0xb4b3377d, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b33781, 76 },
    { 0x94933781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15433788, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea73378d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6133791, 90 },
    { 0, 0 },
    { 0xf9133793, 61 },
    { 0, 0 },
    { 0x2a533795, 23 },
    { 0xf2e33795, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e733799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c337a6, 88 },
    { 0x6ed337a6, 67 },
    { 0x7df337a8, 98 },
    { 0xdfd337a9, 90 },
    { 0x806337aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470337b1, 90 },
    { 0, 0 },
    { 0xbe4337b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40337b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42337bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee337c4, 43 },
    { 0, 0 },
    { 0x596337c6, 90 },
    { 0xe0b337c7, 76 },
    { 0xd06337c6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362337d0, 23 },
    { 0, 0 },
    { 0xc3b337d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c337d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef337dc, 40 },
    { 0xe5e337dc, 95 },
    { 0x1f8337dd, 34 },
    { 0x781337df, 98 },
    { 0x5b337e0, 93 },
    { 0x27e337df, 23 },
    { 0x750337e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d337e5, 23 },
    { 0x88d337e5, 46 },
    { 0x1f8337e7, 23 },
    { 0xfba337e8, 95 },
    { 0x45e337e6, 96 },
    { 0, 0 },
    { 0xead337eb, 33 },
    { 0xf00337ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b337ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250337f3, 23 },
    { 0, 0 },
    { 0xdb3337f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c337ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f33802, 61 },
    { 0x5d333802, 34 },
    { 0, 0 },
    { 0xfad33805, 93 },
    { 0, 0 },
    { 0xc3033807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf493380c, 87 },
    { 0x4223380d, 33 },
    { 0x9af3380d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c33381b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b3381f, 66 },
    { 0x69033820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c33824, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc33827, 90 },
    { 0xc4a33828, 43 },
    { 0xd6533829, 23 },
    { 0, 0 },
    { 0x74f3382b, 61 },
    { 0, 0 },
    { 0xf383382d, 95 },
    { 0, 0 },
    { 0x9b03382f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2293383a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f3383d, 90 },
    { 0x6e53383d, 23 },
    { 0x12f3383f, 46 },
    { 0xcc633840, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c33843, 61 },
    { 0xa6e33844, 99 },
    { 0x36f33844, 33 },
    { 0, 0 },
    { 0xe1333847, 98 },
    { 0, 0 },
    { 0x4d733849, 90 },
    { 0xfc233849, 90 },
    { 0xb3033849, 34 },
    { 0xd6f33849, 34 },
    { 0xf633384d, 68 },
    { 0x8923384b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88433855, 23 },
    { 0, 0 },
    { 0x33433857, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13385f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99433864, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f33867, 90 },
    { 0xef833867, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca3386c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8533870, 91 },
    { 0x4af33870, 90 },
    { 0, 0 },
    { 0x12c33873, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0633878, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x443387b, 80 },
    { 0, 0 },
    { 0xb1e3387d, 84 },
    { 0x3813387e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf733887, 33 },
    { 0xbac33887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f33890, 23 },
    { 0, 0 },
    { 0xa0133892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f333895, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c3389a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1338a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13338a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46338aa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7338ad, 33 },
    { 0, 0 },
    { 0xbf8338af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11338b4, 98 },
    { 0x517338b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2338b8, 43 },
    { 0, 0 },
    { 0x88a338ba, 98 },
    { 0x51c338bb, 78 },
    { 0xfae338ba, 90 },
    { 0xa7d338bc, 23 },
    { 0, 0 },
    { 0x5d6338bf, 67 },
    { 0xf5f338c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8338c5, 61 },
    { 0xc5a338c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c338ca, 95 },
    { 0x21f338cb, 34 },
    { 0xffe338cc, 95 },
    { 0, 0 },
    { 0x284338ce, 99 },
    { 0, 0 },
    { 0xb19338d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14338d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d338d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f338db, 95 },
    { 0xfa2338dc, 95 },
    { 0xe51338db, 34 },
    { 0x2ac338dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a338e3, 91 },
    { 0x99338e3, 33 },
    { 0x369338e5, 90 },
    { 0x106338e4, 52 },
    { 0xfc2338e5, 33 },
    { 0x13338e5, 67 },
    { 0x546338e9, 46 },
    { 0xcea338e9, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34338ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x201338f2, 61 },
    { 0, 0 },
    { 0x6ba338f4, 95 },
    { 0xcd1338f4, 98 },
    { 0xe6a338f6, 95 },
    { 0xa5d338f6, 33 },
    { 0, 0 },
    { 0xf6b338f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126338ff, 55 },
    { 0xff233900, 43 },
    { 0xd7b33900, 91 },
    { 0x74333902, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe633908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b3390d, 83 },
    { 0x4803390e, 23 },
    { 0x5343390f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33915, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd833919, 43 },
    { 0, 0 },
    { 0x64d3391b, 76 },
    { 0xffa3391c, 91 },
    { 0, 0 },
    { 0x5b03391e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533921, 90 },
    { 0, 0 },
    { 0x43e33923, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4333926, 95 },
    { 0xa7433926, 33 },
    { 0xf1033928, 66 },
    { 0xdd033929, 99 },
    { 0xb073392a, 100 },
    { 0, 0 },
    { 0x8613392c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad033931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b33934, 95 },
    { 0x5aa33934, 23 },
    { 0, 0 },
    { 0xd9733937, 46 },
    { 0xa6b33938, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a53393f, 97 },
    { 0x6f833940, 99 },
    { 0x8cc33941, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12333946, 43 },
    { 0, 0 },
    { 0xe0033948, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf3394b, 33 },
    { 0, 0 },
    { 0x3f3394d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7833951, 43 },
    { 0, 0 },
    { 0x50a33953, 76 },
    { 0x8d433953, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53133957, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db3395c, 98 },
    { 0x583395d, 98 },
    { 0xff3395c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94533964, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c633967, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a3396b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a3396f, 33 },
    { 0x1c733970, 75 },
    { 0x9d033971, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c833974, 46 },
    { 0, 0 },
    { 0x2fe33976, 34 },
    { 0xd3433976, 8 },
    { 0, 0 },
    { 0x55c33979, 90 },
    { 0, 0 },
    { 0xd6c3397b, 55 },
    { 0xd923397b, 90 },
    { 0xed73397c, 17 },
    { 0x4983397e, 98 },
    { 0, 0 },
    { 0xfff33980, 77 },
    { 0x89c33980, 98 },
    { 0xedb33982, 23 },
    { 0, 0 },
    { 0x84233984, 67 },
    { 0x61833985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22333988, 90 },
    { 0, 0 },
    { 0xfde3398a, 66 },
    { 0xdaf3398b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a3398f, 35 },
    { 0x1c53398f, 98 },
    { 0xabc33990, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd33994, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82633999, 98 },
    { 0xe6033999, 66 },
    { 0xcaf3399b, 33 },
    { 0x7b93399c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace339a2, 61 },
    { 0xa35339a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x868339a6, 61 },
    { 0, 0 },
    { 0x793339a8, 55 },
    { 0x858339a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319339af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3339b2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c339b5, 98 },
    { 0xb07339b6, 46 },
    { 0x147339b7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90339ba, 78 },
    { 0, 0 },
    { 0x5a9339bc, 95 },
    { 0, 0 },
    { 0xd43339be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7339c1, 76 },
    { 0, 0 },
    { 0xc5b339c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739339ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb339d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0339d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c339dd, 98 },
    { 0xd39339de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a339e2, 95 },
    { 0xc07339e3, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f339e6, 95 },
    { 0x46e339e7, 34 },
    { 0x829339e8, 69 },
    { 0x721339e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2339ee, 95 },
    { 0, 0 },
    { 0x50339f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae339f4, 87 },
    { 0x599339f5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f339f9, 98 },
    { 0, 0 },
    { 0x810339fb, 23 },
    { 0xf83339fb, 66 },
    { 0x9a8339fb, 90 },
    { 0xec3339fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2933a05, 90 },
    { 0xb3633a06, 97 },
    { 0, 0 },
    { 0x82a33a08, 91 },
    { 0, 0 },
    { 0xb3633a0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99733a0f, 23 },
    { 0xa3f33a10, 84 },
    { 0xc3e33a10, 99 },
    { 0x19233a12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc33a1b, 61 },
    { 0x54c33a1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f633a21, 90 },
    { 0x77a33a22, 78 },
    { 0x87433a22, 52 },
    { 0, 0 },
    { 0x6e433a25, 90 },
    { 0xf1f33a26, 61 },
    { 0x85433a27, 34 },
    { 0x64733a26, 61 },
    { 0, 0 },
    { 0x4fc33a2a, 91 },
    { 0xfb633a2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f533a34, 66 },
    { 0, 0 },
    { 0x36a33a36, 90 },
    { 0x21733a37, 98 },
    { 0xba533a37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a33a3b, 33 },
    { 0xafc33a3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b33a3f, 33 },
    { 0x93933a40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39333a45, 33 },
    { 0x18733a46, 33 },
    { 0x54e33a47, 33 },
    { 0, 0 },
    { 0xc6d33a49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d233a4c, 66 },
    { 0xba733a4d, 23 },
    { 0x25a33a4c, 33 },
    { 0x7ea33a4e, 52 },
    { 0x9b033a4e, 75 },
    { 0x91a33a51, 61 },
    { 0, 0 },
    { 0xd5033a53, 55 },
    { 0, 0 },
    { 0x59733a55, 98 },
    { 0x6c33a55, 90 },
    { 0xfd233a56, 33 },
    { 0, 0 },
    { 0xee333a59, 90 },
    { 0xeb933a5a, 93 },
    { 0xcd433a5a, 33 },
    { 0x4a233a5c, 55 },
    { 0xeac33a59, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb733a60, 23 },
    { 0x25f33a60, 95 },
    { 0x5a133a62, 98 },
    { 0x9e633a63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69633a69, 98 },
    { 0xe1b33a6a, 98 },
    { 0, 0 },
    { 0xc9533a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5933a6f, 33 },
    { 0xe7833a70, 43 },
    { 0, 0 },
    { 0x7133a72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef33a77, 76 },
    { 0x43533a77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c33a7b, 95 },
    { 0, 0 },
    { 0x8be33a7d, 69 },
    { 0x18f33a7e, 34 },
    { 0x40233a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd33a83, 69 },
    { 0xe3233a84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf633a8a, 17 },
    { 0xdad33a8b, 52 },
    { 0xcf533a8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5733a91, 95 },
    { 0x4a633a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea133a96, 76 },
    { 0xaee33a96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef933aa1, 98 },
    { 0x12d33aa1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3433aa6, 90 },
    { 0x46f33aa6, 33 },
    { 0x26233aa6, 90 },
    { 0x27833aa6, 43 },
    { 0xfa933aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa33aae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c33ab1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee33ab4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e733ab7, 61 },
    { 0xe2933ab7, 23 },
    { 0x64a33ab9, 85 },
    { 0, 0 },
    { 0xa8033abb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3333ac0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a333aca, 95 },
    { 0, 0 },
    { 0x3a133acc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38533ad0, 67 },
    { 0x94d33ad1, 94 },
    { 0x48033ad2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9433adb, 23 },
    { 0, 0 },
    { 0x21f33add, 76 },
    { 0, 0 },
    { 0x3a333adf, 23 },
    { 0, 0 },
    { 0x9433ae1, 61 },
    { 0x7f733ae2, 75 },
    { 0, 0 },
    { 0x3ea33ae4, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x12933ae7, 23 },
    { 0x77633ae8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff33aeb, 85 },
    { 0x7c533aec, 17 },
    { 0xc5c33aed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f33af0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e633af7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed333aff, 87 },
    { 0xa8233b00, 67 },
    { 0x83d33b00, 66 },
    { 0x9d133b00, 23 },
    { 0x2c433b03, 95 },
    { 0x76b33b04, 24 },
    { 0xc3d33b05, 99 },
    { 0x86233b05, 33 },
    { 0, 0 },
    { 0x40733b08, 39 },
    { 0x1f33b08, 91 },
    { 0xc733b09, 34 },
    { 0xfce33b0b, 61 },
    { 0xbda33b0c, 66 },
    { 0x20a33b0d, 23 },
    { 0, 0 },
    { 0x89533b0f, 78 },
    { 0, 0 },
    { 0xcca33b11, 88 },
    { 0xa7933b11, 90 },
    { 0xcda33b13, 55 },
    { 0x8aa33b14, 91 },
    { 0xb8833b12, 95 },
    { 0x20333b13, 66 },
    { 0xad333b17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74633b1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d33b1d, 33 },
    { 0, 0 },
    { 0xa6133b1f, 33 },
    { 0x1fd33b20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3433b23, 95 },
    { 0x99f33b24, 78 },
    { 0, 0 },
    { 0x9bd33b26, 33 },
    { 0, 0 },
    { 0x2d733b28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c633b2e, 98 },
    { 0, 0 },
    { 0x13e33b30, 90 },
    { 0x55333b30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d433b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b45, 23 },
    { 0x10833b46, 33 },
    { 0xbfe33b46, 23 },
    { 0, 0 },
    { 0xba433b49, 98 },
    { 0, 0 },
    { 0xd33b4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b33b4e, 61 },
    { 0x86f33b4e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc33b53, 95 },
    { 0x67533b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d533b59, 93 },
    { 0, 0 },
    { 0xbdb33b5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71433b5f, 8 },
    { 0, 0 },
    { 0x633b61, 76 },
    { 0, 0 },
    { 0xe9233b63, 67 },
    { 0xd1433b64, 95 },
    { 0x8b733b65, 33 },
    { 0x76433b65, 33 },
    { 0x8a33b67, 98 },
    { 0x20f33b68, 95 },
    { 0, 0 },
    { 0xef933b6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9833b6d, 61 },
    { 0x3aa33b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d633b73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d33b76, 85 },
    { 0, 0 },
    { 0x87833b78, 68 },
    { 0x23933b78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec133b7d, 33 },
    { 0x17f33b7e, 90 },
    { 0x81033b7e, 61 },
    { 0xffc33b7d, 33 },
    { 0, 0 },
    { 0xfce33b82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58733b87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb33b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f33b93, 90 },
    { 0x68033b93, 95 },
    { 0, 0 },
    { 0x51e33b96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e33b9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d33ba0, 95 },
    { 0x17433ba1, 90 },
    { 0xb9833ba0, 93 },
    { 0xd2933ba1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d33ba9, 43 },
    { 0, 0 },
    { 0x34333bab, 87 },
    { 0, 0 },
    { 0xb3233bad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e33bb0, 23 },
    { 0xc833bb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf633bbc, 34 },
    { 0, 0 },
    { 0xf0a33bbe, 87 },
    { 0x8e233bbf, 66 },
    { 0, 0 },
    { 0x40f33bc1, 95 },
    { 0, 0 },
    { 0x8ea33bc3, 46 },
    { 0xd5f33bc4, 33 },
    { 0, 0 },
    { 0x2fc33bc6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b333bca, 93 },
    { 0, 0 },
    { 0x4d633bcc, 90 },
    { 0x59433bcd, 98 },
    { 0, 0 },
    { 0x95433bcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66833bd5, 100 },
    { 0xa7f33bd6, 93 },
    { 0x41133bd7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xca133bda, 95 },
    { 0xc1633bda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46033bde, 61 },
    { 0xf833bde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6633be2, 99 },
    { 0x73b33be3, 76 },
    { 0x35233be4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47433beb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9433bee, 81 },
    { 0x3733bee, 95 },
    { 0xb633bee, 90 },
    { 0, 0 },
    { 0xe9833bf2, 95 },
    { 0, 0 },
    { 0x50f33bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e33bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6833c08, 85 },
    { 0xbf633c09, 34 },
    { 0x28233c0a, 93 },
    { 0x78c33c09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98033c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a33c15, 85 },
    { 0x31533c15, 81 },
    { 0xaa933c17, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9233c1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda333c24, 76 },
    { 0x1a433c25, 90 },
    { 0x5b433c26, 34 },
    { 0x46c33c27, 90 },
    { 0x85133c25, 46 },
    { 0xb9933c26, 81 },
    { 0xae633c2a, 61 },
    { 0x96733c2a, 84 },
    { 0x95933c2c, 46 },
    { 0xaae33c2c, 98 },
    { 0xc2d33c2e, 90 },
    { 0x99a33c2e, 34 },
    { 0xeb633c2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf733c33, 23 },
    { 0x55433c34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3233c37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb133c3c, 95 },
    { 0x1933c3c, 61 },
    { 0xe3433c3e, 33 },
    { 0x9f733c3e, 46 },
    { 0xa6933c40, 84 },
    { 0xe0833c40, 34 },
    { 0xfe433c41, 78 },
    { 0x26d33c3e, 98 },
    { 0xdb433c3e, 33 },
    { 0x9a33c45, 90 },
    { 0x9a333c3f, 99 },
    { 0, 0 },
    { 0x10b33c48, 95 },
    { 0, 0 },
    { 0x8f33c4a, 61 },
    { 0x4c633c4b, 77 },
    { 0x4833c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67533c54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b33c5a, 23 },
    { 0x7b633c5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9833c62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d33c67, 55 },
    { 0x57133c67, 98 },
    { 0x99d33c69, 33 },
    { 0x13a33c6a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1433c6e, 85 },
    { 0, 0 },
    { 0x28b33c70, 33 },
    { 0x3c833c71, 52 },
    { 0xb733c71, 76 },
    { 0xef733c73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a33c78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4033c80, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5533c83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33c8f, 40 },
    { 0xd5533c90, 33 },
    { 0x50c33c8f, 90 },
    { 0, 0 },
    { 0xeca33c93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3833c96, 90 },
    { 0, 0 },
    { 0xf4233c98, 66 },
    { 0, 0 },
    { 0x18533c9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23633c9d, 98 },
    { 0xa0133c9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c33ca2, 61 },
    { 0xe5933ca2, 76 },
    { 0xdc233ca3, 99 },
    { 0xd3933ca5, 95 },
    { 0x38e33ca3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe33cac, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x12633caf, 84 },
    { 0xc9333cb0, 33 },
    { 0x32f33cb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b33cb5, 23 },
    { 0xd2a33cb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e333cb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac933cbe, 95 },
    { 0, 0 },
    { 0xdd833cc0, 99 },
    { 0x94133cc1, 98 },
    { 0xe4733cc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3333cc9, 34 },
    { 0xf4333cca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65833cce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b33cd1, 98 },
    { 0, 0 },
    { 0x9a233cd3, 100 },
    { 0x53033cd4, 95 },
    { 0xd433cd3, 34 },
    { 0xc533cd6, 90 },
    { 0x44233cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d33cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd33cdf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x63633ce2, 61 },
    { 0xaae33ce2, 100 },
    { 0, 0 },
    { 0x63233ce5, 76 },
    { 0x33d33ce5, 98 },
    { 0, 0 },
    { 0x6fd33ce8, 40 },
    { 0x3a833ce9, 87 },
    { 0xbaf33cea, 61 },
    { 0xc5a33ce9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14233cee, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2733cf1, 100 },
    { 0, 0 },
    { 0xfdf33cf3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb633cf8, 66 },
    { 0x74733cf9, 33 },
    { 0xf2733cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4133cfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a533d00, 33 },
    { 0x54533d01, 99 },
    { 0x48f33d01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0733d07, 68 },
    { 0x46c33d07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77333d0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf33d0f, 81 },
    { 0, 0 },
    { 0x84733d11, 61 },
    { 0x86f33d12, 23 },
    { 0, 0 },
    { 0x78033d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba33d19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8733d1d, 33 },
    { 0x83233d1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f833d22, 33 },
    { 0, 0 },
    { 0x52a33d24, 88 },
    { 0x8e033d25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86033d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c33d30, 51 },
    { 0xf2d33d31, 76 },
    { 0, 0 },
    { 0x86b33d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee233d36, 61 },
    { 0x88533d37, 77 },
    { 0x32333d38, 46 },
    { 0x8c733d39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b33d3f, 34 },
    { 0xdc633d3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91533d44, 90 },
    { 0, 0 },
    { 0x26233d46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5233d4b, 33 },
    { 0xbb133d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8833d52, 52 },
    { 0, 0 },
    { 0xcad33d54, 91 },
    { 0xe8633d55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38933d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a933d5c, 93 },
    { 0xb0833d5d, 33 },
    { 0x29d33d5e, 76 },
    { 0xf933d5e, 43 },
    { 0x13433d5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc533d65, 61 },
    { 0x42d33d66, 33 },
    { 0xdcc33d66, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2133d6f, 95 },
    { 0, 0 },
    { 0x5b333d71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6633d74, 61 },
    { 0xecb33d75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b33d7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49233d7f, 93 },
    { 0xcb733d80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e33d84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b33d87, 43 },
    { 0x47a33d88, 33 },
    { 0x7c933d89, 33 },
    { 0, 0 },
    { 0x9a33d8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb233d8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x63433d91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49933d95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff133d99, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51633d9d, 23 },
    { 0, 0 },
    { 0xdca33d9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd33da5, 61 },
    { 0xc8c33da6, 55 },
    { 0xe2733da6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda33dab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e33db2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9933db6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf33dbe, 33 },
    { 0x69833dbe, 66 },
    { 0x2a833dc0, 93 },
    { 0xa5933dbf, 77 },
    { 0x33d33dc2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7033dc5, 93 },
    { 0xceb33dc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d633dc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c33dcd, 68 },
    { 0x93e33dce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9633dd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8333dd4, 77 },
    { 0, 0 },
    { 0xcd633dd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f033ddd, 46 },
    { 0x2033ddd, 94 },
    { 0, 0 },
    { 0xb1133de0, 66 },
    { 0x90633de1, 33 },
    { 0x65e33de2, 12 },
    { 0, 0 },
    { 0x88133de4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e033dee, 99 },
    { 0xb3a33dee, 85 },
    { 0xb3933df0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a733df4, 90 },
    { 0x76933df5, 90 },
    { 0xe4433df6, 92 },
    { 0, 0 },
    { 0x6c433df8, 98 },
    { 0, 0 },
    { 0x66633dfa, 88 },
    { 0x50633dfb, 75 },
    { 0x77033dfc, 90 },
    { 0x8133dfc, 90 },
    { 0x3dd33dfe, 33 },
    { 0xf333dfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14533e02, 85 },
    { 0, 0 },
    { 0xa9d33e04, 98 },
    { 0x4ec33e05, 84 },
    { 0x3b633e06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42033e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81033e0f, 23 },
    { 0x19533e10, 33 },
    { 0x31733e10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda33e16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7533e1c, 98 },
    { 0, 0 },
    { 0x67833e1e, 61 },
    { 0, 0 },
    { 0x11133e20, 76 },
    { 0, 0 },
    { 0x59d33e22, 55 },
    { 0, 0 },
    { 0x3dc33e24, 90 },
    { 0x8f533e24, 40 },
    { 0xcbd33e25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6433e29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d333e2d, 55 },
    { 0, 0 },
    { 0x76c33e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69833e33, 78 },
    { 0x6af33e34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e33e37, 34 },
    { 0x41633e38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d233e3b, 98 },
    { 0, 0 },
    { 0xcdf33e3d, 87 },
    { 0, 0 },
    { 0x5b233e3f, 61 },
    { 0xb4a33e40, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8633e43, 90 },
    { 0, 0 },
    { 0xd2e33e45, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x64133e48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30933e4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad33e50, 77 },
    { 0x38a33e51, 33 },
    { 0x9a033e52, 34 },
    { 0x99f33e53, 77 },
    { 0, 0 },
    { 0xbdf33e55, 90 },
    { 0xfd533e56, 52 },
    { 0, 0 },
    { 0x7233e58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6233e5e, 53 },
    { 0x3f933e5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db33e65, 76 },
    { 0x9db33e66, 76 },
    { 0xfc933e66, 95 },
    { 0x81f33e68, 55 },
    { 0x51233e67, 76 },
    { 0x28933e6a, 61 },
    { 0x3d33e6a, 33 },
    { 0x86a33e6a, 81 },
    { 0x7ef33e6a, 98 },
    { 0xe2533e6e, 23 },
    { 0, 0 },
    { 0xd8433e70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0933e74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a833e77, 90 },
    { 0xd633e78, 23 },
    { 0, 0 },
    { 0xdf033e7a, 23 },
    { 0, 0 },
    { 0x44433e7c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf833e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d533e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91633e90, 98 },
    { 0, 0 },
    { 0xe833e92, 90 },
    { 0xe533e92, 38 },
    { 0, 0 },
    { 0x24c33e95, 34 },
    { 0x4d533e95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d33e9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd33ea4, 100 },
    { 0x9e733ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33233eaf, 76 },
    { 0xdac33eb0, 35 },
    { 0x5ce33eb1, 55 },
    { 0xff33eb1, 46 },
    { 0, 0 },
    { 0xa4a33eb4, 90 },
    { 0xa7f33eb5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74433eb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd33ebe, 95 },
    { 0x3ca33ebf, 55 },
    { 0x81833ec0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14133ec6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c833ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c933ecf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf33ed2, 52 },
    { 0xfe533ed2, 33 },
    { 0x7ce33ed4, 95 },
    { 0x20233ed3, 98 },
    { 0, 0 },
    { 0x22633ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b733edb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc633ee1, 90 },
    { 0x9c33ee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2633ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80033eec, 84 },
    { 0, 0 },
    { 0x56533eee, 23 },
    { 0, 0 },
    { 0x3dd33ef0, 98 },
    { 0xdc33ef1, 99 },
    { 0x15133ef2, 98 },
    { 0, 0 },
    { 0x40d33ef4, 23 },
    { 0, 0 },
    { 0x34733ef6, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x35133ef9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45933efd, 33 },
    { 0, 0 },
    { 0x29733eff, 33 },
    { 0x34133f00, 98 },
    { 0x93c33f01, 61 },
    { 0x7fe33f00, 48 },
    { 0x3cf33f03, 61 },
    { 0x4f733f04, 61 },
    { 0x4ec33f04, 76 },
    { 0x3be33f06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed33f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39333f11, 61 },
    { 0x38233f12, 33 },
    { 0x12b33f13, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef33f16, 95 },
    { 0, 0 },
    { 0xbf133f18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b33f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8833f23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d33f26, 90 },
    { 0xbe33f26, 90 },
    { 0, 0 },
    { 0x2f033f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb033f2c, 99 },
    { 0, 0 },
    { 0xda033f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a33f34, 76 },
    { 0xa2033f34, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf33f3a, 90 },
    { 0xaf333f3a, 43 },
    { 0xa0133f3c, 95 },
    { 0xde033f3d, 33 },
    { 0x8333f3e, 34 },
    { 0x10a33f3e, 85 },
    { 0xebd33f40, 100 },
    { 0, 0 },
    { 0xa1733f42, 46 },
    { 0xd1433f43, 76 },
    { 0x32733f44, 99 },
    { 0x1a033f44, 61 },
    { 0xc7233f45, 95 },
    { 0xabe33f42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0233f4a, 90 },
    { 0, 0 },
    { 0x5a233f4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2833f51, 75 },
    { 0x8b633f52, 43 },
    { 0x6b633f53, 95 },
    { 0x4d133f51, 100 },
    { 0x77333f55, 33 },
    { 0x2b333f56, 34 },
    { 0x6aa33f56, 98 },
    { 0x6b833f56, 98 },
    { 0, 0 },
    { 0x6ad33f5a, 95 },
    { 0x96933f5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0433f5f, 68 },
    { 0x11b33f5f, 95 },
    { 0, 0 },
    { 0xfd033f62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed33f65, 17 },
    { 0xa8633f65, 92 },
    { 0xf3033f67, 95 },
    { 0x4ad33f68, 87 },
    { 0xc2233f69, 90 },
    { 0x51333f69, 55 },
    { 0x3ad33f6a, 90 },
    { 0xdfb33f6b, 95 },
    { 0x5f633f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f33f76, 66 },
    { 0x23533f77, 61 },
    { 0xe5133f78, 34 },
    { 0xef233f78, 90 },
    { 0x40333f77, 95 },
    { 0xc6733f7b, 98 },
    { 0, 0 },
    { 0x1cf33f7d, 33 },
    { 0x6533f7e, 46 },
    { 0xcb333f7f, 90 },
    { 0x69733f80, 76 },
    { 0, 0 },
    { 0xed633f82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff533f85, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3833f88, 90 },
    { 0xd7033f89, 46 },
    { 0x75c33f8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96133f8e, 61 },
    { 0x6df33f8e, 90 },
    { 0x3f633f8e, 90 },
    { 0x47633f91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9633f95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd33fa0, 76 },
    { 0x42b33fa1, 98 },
    { 0, 0 },
    { 0x83733fa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e33fa7, 84 },
    { 0xb0f33fa8, 68 },
    { 0xde333fa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e733fad, 90 },
    { 0x27933fad, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6633fb1, 90 },
    { 0, 0 },
    { 0x51133fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4633fb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f33fbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36733fc2, 33 },
    { 0xd6a33fc3, 98 },
    { 0, 0 },
    { 0xaad33fc5, 98 },
    { 0, 0 },
    { 0x74033fc7, 33 },
    { 0, 0 },
    { 0x8ea33fc9, 61 },
    { 0xad533fc9, 88 },
    { 0x28a33fcb, 55 },
    { 0x2cc33fcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e33fd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd33fd3, 99 },
    { 0, 0 },
    { 0xd8733fd5, 98 },
    { 0, 0 },
    { 0x4c433fd7, 98 },
    { 0, 0 },
    { 0x60d33fd9, 61 },
    { 0, 0 },
    { 0xf333fdb, 17 },
    { 0x74333fdb, 61 },
    { 0xa1d33fdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59133fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70533fea, 93 },
    { 0xf4633feb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e33ff0, 76 },
    { 0xca233ff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec33ffb, 61 },
    { 0, 0 },
    { 0xd6833ffd, 90 },
    { 0, 0 },
    { 0x8a433fff, 95 },
    { 0, 0 },
    { 0x16534001, 61 },
    { 0x60634002, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75134007, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e3400b, 36 },
    { 0x91e3400c, 17 },
    { 0x85f3400d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d34010, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa34016, 68 },
    { 0, 0 },
    { 0x33334018, 98 },
    { 0x77334019, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54134024, 61 },
    { 0xf0134025, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f534028, 52 },
    { 0x67634029, 67 },
    { 0xb2534029, 90 },
    { 0x5863402b, 94 },
    { 0x4bc3402c, 33 },
    { 0x4b23402d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5034030, 90 },
    { 0xc8534030, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a34034, 33 },
    { 0, 0 },
    { 0x3034036, 61 },
    { 0, 0 },
    { 0xc2b34038, 33 },
    { 0x27d34038, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6253403e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4434045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d3404a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a3404e, 43 },
    { 0, 0 },
    { 0x45934050, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e34053, 95 },
    { 0xa7534054, 90 },
    { 0x16e34055, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f3405a, 55 },
    { 0xd043405a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b34060, 68 },
    { 0x21234061, 80 },
    { 0xe6f34061, 52 },
    { 0x27f34060, 33 },
    { 0, 0 },
    { 0xcba34065, 61 },
    { 0xb0c34066, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e3406b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e3406f, 38 },
    { 0x31a34070, 90 },
    { 0x73c34071, 34 },
    { 0x57934072, 99 },
    { 0xc5634073, 33 },
    { 0x6c134074, 77 },
    { 0x77334074, 61 },
    { 0xd9a34070, 76 },
    { 0xb6934073, 96 },
    { 0xa6f34071, 43 },
    { 0x64334079, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af3407d, 38 },
    { 0x8323407d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98734084, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8034091, 55 },
    { 0xe7d34092, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103340a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f340a8, 90 },
    { 0x1c0340a9, 23 },
    { 0x446340a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94340ae, 77 },
    { 0x1fa340ae, 55 },
    { 0xa98340b0, 98 },
    { 0xd4d340b1, 23 },
    { 0x95c340b1, 61 },
    { 0, 0 },
    { 0x4aa340b4, 90 },
    { 0x865340b5, 93 },
    { 0xd5e340b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c340bb, 76 },
    { 0xe3a340bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74340bf, 98 },
    { 0xca8340c0, 95 },
    { 0xcc3340c0, 61 },
    { 0x499340c2, 90 },
    { 0x98e340bf, 23 },
    { 0xefe340c4, 23 },
    { 0x472340c5, 90 },
    { 0xae3340c6, 90 },
    { 0x452340c0, 95 },
    { 0x73f340c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9340cb, 90 },
    { 0x64d340cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f340d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5340d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff340da, 90 },
    { 0x3dc340db, 23 },
    { 0xf6c340db, 90 },
    { 0x44d340dd, 98 },
    { 0x9bd340de, 23 },
    { 0, 0 },
    { 0x432340e0, 61 },
    { 0x25f340e1, 46 },
    { 0x62b340e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9340e6, 76 },
    { 0, 0 },
    { 0x24e340e8, 75 },
    { 0, 0 },
    { 0xf03340ea, 90 },
    { 0x462340eb, 23 },
    { 0xdd6340ec, 100 },
    { 0xd29340ed, 66 },
    { 0, 0 },
    { 0xa09340ef, 46 },
    { 0x7e340ef, 98 },
    { 0x5db340f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6340f5, 34 },
    { 0xd6340f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5340fa, 61 },
    { 0, 0 },
    { 0x751340fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e340ff, 40 },
    { 0x7234100, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61334104, 66 },
    { 0, 0 },
    { 0xc4d34106, 87 },
    { 0xa7534107, 23 },
    { 0xc1d34108, 33 },
    { 0, 0 },
    { 0xb5b3410a, 91 },
    { 0, 0 },
    { 0x17d3410c, 33 },
    { 0x8833410c, 95 },
    { 0xc3d3410e, 61 },
    { 0x7853410f, 23 },
    { 0x3ed3410f, 43 },
    { 0xd3434111, 80 },
    { 0, 0 },
    { 0x3534113, 34 },
    { 0, 0 },
    { 0xe6034115, 61 },
    { 0xb9a34116, 33 },
    { 0x75934115, 98 },
    { 0x2a234118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63411b, 33 },
    { 0, 0 },
    { 0x1973411d, 90 },
    { 0, 0 },
    { 0x8c93411f, 99 },
    { 0x4a63411f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2033412c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb034130, 78 },
    { 0x35234131, 95 },
    { 0, 0 },
    { 0x34434133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85334137, 23 },
    { 0x6d434138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2683413f, 81 },
    { 0x80d34140, 90 },
    { 0x39d3413f, 23 },
    { 0x2573413f, 33 },
    { 0xab334143, 95 },
    { 0xe9934143, 43 },
    { 0xbad34140, 61 },
    { 0x4ff34146, 66 },
    { 0xa9234141, 98 },
    { 0, 0 },
    { 0xb8d34149, 61 },
    { 0xe953414a, 94 },
    { 0x4d33414b, 85 },
    { 0, 0 },
    { 0x28d3414d, 46 },
    { 0xa083414d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10534153, 33 },
    { 0x9dd34153, 90 },
    { 0xbfa34155, 90 },
    { 0x9c734156, 55 },
    { 0xb8934156, 61 },
    { 0xb4d34158, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb3415d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34160, 95 },
    { 0x64d34161, 90 },
    { 0x39234162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2534165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653416d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7634177, 95 },
    { 0x33f34178, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d3417d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e134180, 33 },
    { 0, 0 },
    { 0x7034182, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42634186, 23 },
    { 0xd4734186, 23 },
    { 0x2b534186, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb523418b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd883418e, 98 },
    { 0xbfb3418f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f34192, 91 },
    { 0x26d34192, 33 },
    { 0x8a534193, 90 },
    { 0xec634195, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e634198, 84 },
    { 0, 0 },
    { 0x1743419a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9341a0, 33 },
    { 0xf1e341a1, 83 },
    { 0x579341a1, 76 },
    { 0x9a9341a3, 75 },
    { 0x98e341a2, 76 },
    { 0xe04341a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867341ab, 55 },
    { 0, 0 },
    { 0xa8a341ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7341b2, 33 },
    { 0, 0 },
    { 0x52c341b4, 78 },
    { 0x44b341b5, 93 },
    { 0, 0 },
    { 0xc61341b7, 87 },
    { 0, 0 },
    { 0xedf341b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab341bf, 61 },
    { 0x8e0341bf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a341c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389341ce, 43 },
    { 0, 0 },
    { 0xbc7341d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee341d4, 76 },
    { 0, 0 },
    { 0x38c341d6, 55 },
    { 0, 0 },
    { 0xae3341d8, 33 },
    { 0, 0 },
    { 0x641341da, 76 },
    { 0x546341db, 68 },
    { 0x337341dc, 90 },
    { 0x6b1341dc, 55 },
    { 0x2e6341dd, 76 },
    { 0x520341df, 23 },
    { 0x94341e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23341e4, 43 },
    { 0x76c341e4, 55 },
    { 0xb53341e5, 99 },
    { 0xdc5341e5, 100 },
    { 0, 0 },
    { 0xa5f341e9, 95 },
    { 0x4e341ea, 52 },
    { 0x59d341eb, 61 },
    { 0, 0 },
    { 0x6d5341ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e341f2, 90 },
    { 0x777341f3, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439341f8, 95 },
    { 0, 0 },
    { 0x50c341fa, 96 },
    { 0x2dd341fb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6034200, 66 },
    { 0, 0 },
    { 0xd2034202, 61 },
    { 0, 0 },
    { 0x8bf34204, 46 },
    { 0, 0 },
    { 0x6a634206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e3420a, 85 },
    { 0x6d43420a, 42 },
    { 0, 0 },
    { 0x3433420d, 76 },
    { 0x3ec3420e, 33 },
    { 0, 0 },
    { 0x3ce34210, 68 },
    { 0x56434211, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd934217, 33 },
    { 0x8cb34217, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd543421e, 95 },
    { 0x2da3421f, 76 },
    { 0xc9834220, 100 },
    { 0x7103421e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf3422b, 61 },
    { 0xca53422c, 98 },
    { 0xacd3422d, 88 },
    { 0x85e3422c, 90 },
    { 0x1d03422d, 33 },
    { 0x5d3422d, 23 },
    { 0x4763422b, 100 },
    { 0xf2234232, 33 },
    { 0xd0434230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f53423e, 90 },
    { 0, 0 },
    { 0xe7634240, 98 },
    { 0xe4334240, 33 },
    { 0xba334240, 95 },
    { 0xd2c34243, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1334247, 84 },
    { 0xa1334248, 61 },
    { 0x7c734249, 85 },
    { 0, 0 },
    { 0xaa63424b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c34250, 88 },
    { 0xe1534251, 93 },
    { 0x76634252, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39534256, 91 },
    { 0x5ad34256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5093425d, 55 },
    { 0, 0 },
    { 0x16b3425f, 90 },
    { 0xa203425f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9134266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3426d, 77 },
    { 0x9c3426e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d134271, 95 },
    { 0x32f34272, 66 },
    { 0x99c34273, 67 },
    { 0x5a234273, 87 },
    { 0xb334273, 85 },
    { 0xb4f34274, 61 },
    { 0, 0 },
    { 0x5ff34278, 33 },
    { 0x93334278, 23 },
    { 0x3e34278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd3427d, 93 },
    { 0x5463427e, 68 },
    { 0xec23427e, 23 },
    { 0xd183427f, 76 },
    { 0, 0 },
    { 0x4f234282, 61 },
    { 0, 0 },
    { 0x6d134284, 23 },
    { 0xe1d34285, 33 },
    { 0x6fe34286, 90 },
    { 0x1cb34287, 76 },
    { 0x31a34287, 98 },
    { 0xb9434287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2334293, 36 },
    { 0, 0 },
    { 0xf3834295, 78 },
    { 0xd0234296, 23 },
    { 0x49c34296, 90 },
    { 0xab34297, 91 },
    { 0x2f34299, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504342a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d342af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a342b4, 90 },
    { 0xda8342b4, 61 },
    { 0x457342b5, 96 },
    { 0xd3a342b4, 98 },
    { 0, 0 },
    { 0x905342b9, 100 },
    { 0xf07342ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4342be, 95 },
    { 0, 0 },
    { 0xf2c342c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981342c5, 33 },
    { 0, 0 },
    { 0xd77342c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709342cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x481342cf, 98 },
    { 0x85a342d0, 23 },
    { 0x337342d0, 43 },
    { 0x6ca342d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71342da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52342e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03342ea, 95 },
    { 0x3af342eb, 90 },
    { 0, 0 },
    { 0x820342ed, 55 },
    { 0, 0 },
    { 0x86b342ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10342f2, 95 },
    { 0, 0 },
    { 0x9bd342f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e342f9, 84 },
    { 0x132342fa, 61 },
    { 0, 0 },
    { 0x5aa342fc, 43 },
    { 0x5ff342fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x63534300, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f34303, 81 },
    { 0xc9f34304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a3430b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6963430e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62234311, 33 },
    { 0x9b534312, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e3431c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a34325, 98 },
    { 0x9db34326, 99 },
    { 0x60534326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943432b, 66 },
    { 0, 0 },
    { 0x7ad3432d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec34332, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80434336, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6333433a, 90 },
    { 0x84c3433b, 34 },
    { 0, 0 },
    { 0x7f3433d, 95 },
    { 0x28f3433e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a34344, 61 },
    { 0x60434344, 81 },
    { 0x77834346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a3434c, 66 },
    { 0xe163434c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234353, 66 },
    { 0x4fc34353, 98 },
    { 0xddc34355, 23 },
    { 0x75334355, 99 },
    { 0x16834355, 90 },
    { 0x92834358, 23 },
    { 0xac34359, 87 },
    { 0x6c43435a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193435e, 76 },
    { 0x84d3435f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6534362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf853436a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b34375, 34 },
    { 0xc1d34376, 61 },
    { 0, 0 },
    { 0xba734378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b83437e, 95 },
    { 0xf9f3437f, 33 },
    { 0x6813437e, 33 },
    { 0xd9034381, 90 },
    { 0x26134382, 61 },
    { 0x5e34383, 23 },
    { 0x87a34384, 55 },
    { 0, 0 },
    { 0xc4234386, 95 },
    { 0x78a34387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x413438a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783438d, 98 },
    { 0xde33438d, 33 },
    { 0, 0 },
    { 0xd0f34390, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e34397, 34 },
    { 0x2ac34398, 90 },
    { 0, 0 },
    { 0x6b33439a, 98 },
    { 0xb9f3439b, 98 },
    { 0xc283439c, 84 },
    { 0, 0 },
    { 0x8a33439e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89343a3, 43 },
    { 0xfd4343a4, 88 },
    { 0xffb343a4, 76 },
    { 0x52d343a4, 55 },
    { 0x68f343a7, 33 },
    { 0x557343a7, 90 },
    { 0xd4343a4, 98 },
    { 0xbb343aa, 100 },
    { 0x13d343ab, 23 },
    { 0xd15343ac, 66 },
    { 0x48a343aa, 95 },
    { 0, 0 },
    { 0xd8343af, 90 },
    { 0, 0 },
    { 0xffb343b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981343be, 33 },
    { 0xe58343bf, 61 },
    { 0xa5e343c0, 98 },
    { 0x74343be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a343c9, 34 },
    { 0xc4b343ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc343cd, 33 },
    { 0, 0 },
    { 0x488343cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850343d3, 17 },
    { 0x7be343d4, 23 },
    { 0x5af343d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d343db, 90 },
    { 0xbe1343dc, 87 },
    { 0, 0 },
    { 0xb7d343de, 95 },
    { 0, 0 },
    { 0xb80343e0, 90 },
    { 0x3fd343e0, 95 },
    { 0x924343e1, 90 },
    { 0xc26343e2, 23 },
    { 0x899343e4, 66 },
    { 0, 0 },
    { 0x60c343e6, 88 },
    { 0xda9343e7, 33 },
    { 0xc60343e8, 61 },
    { 0xddc343e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444343ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad343f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422343f9, 61 },
    { 0x1cb343fa, 88 },
    { 0x8be343f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d34400, 33 },
    { 0x92734400, 90 },
    { 0x3b334402, 33 },
    { 0x5ee34403, 34 },
    { 0x2a234404, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8443440b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1853440e, 23 },
    { 0, 0 },
    { 0x1dc34410, 91 },
    { 0xc6c34411, 95 },
    { 0xc0734411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11834415, 55 },
    { 0, 0 },
    { 0x43234417, 98 },
    { 0x8f834418, 33 },
    { 0x9dd34419, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3441d, 97 },
    { 0, 0 },
    { 0x1193441f, 38 },
    { 0xc0f34420, 55 },
    { 0x86534421, 98 },
    { 0x4ae34421, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8634425, 91 },
    { 0x81c34425, 98 },
    { 0x9cb34427, 23 },
    { 0x1ef34426, 95 },
    { 0, 0 },
    { 0xec53442a, 98 },
    { 0, 0 },
    { 0xb293442c, 40 },
    { 0x1333442d, 85 },
    { 0x4203442e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a734433, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c934436, 33 },
    { 0xbb334437, 34 },
    { 0x75034438, 55 },
    { 0, 0 },
    { 0x89d3443a, 33 },
    { 0, 0 },
    { 0x70e3443c, 61 },
    { 0, 0 },
    { 0xde83443e, 98 },
    { 0x50a3443e, 90 },
    { 0x9df3443e, 93 },
    { 0xd7b3443f, 90 },
    { 0x22434442, 95 },
    { 0, 0 },
    { 0xd0734444, 43 },
    { 0x1f834445, 95 },
    { 0x61534446, 91 },
    { 0x13434447, 33 },
    { 0xc9534448, 90 },
    { 0x7ed34448, 90 },
    { 0xcb934445, 90 },
    { 0x5b53444a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f34451, 36 },
    { 0xfb34452, 61 },
    { 0xd2834453, 43 },
    { 0x9534453, 98 },
    { 0x18d34455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb3445c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6083445f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe34463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b334466, 43 },
    { 0, 0 },
    { 0x81c34468, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd3446d, 76 },
    { 0xf533446e, 84 },
    { 0, 0 },
    { 0xc1134470, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa34473, 33 },
    { 0x2b434474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff34479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4433447c, 43 },
    { 0xd543447d, 61 },
    { 0x8383447c, 77 },
    { 0xfb33447f, 93 },
    { 0xf2e34480, 84 },
    { 0xfc934481, 61 },
    { 0, 0 },
    { 0x4af34483, 52 },
    { 0, 0 },
    { 0x60134485, 52 },
    { 0xf9a34485, 90 },
    { 0x11c34486, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be3448a, 93 },
    { 0, 0 },
    { 0xf793448c, 91 },
    { 0x5ce3448d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be34490, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc734496, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f3449c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459344a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42344a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x346344a8, 90 },
    { 0, 0 },
    { 0xadb344aa, 61 },
    { 0, 0 },
    { 0xc21344ac, 61 },
    { 0, 0 },
    { 0x6b6344ae, 46 },
    { 0x4fd344af, 81 },
    { 0xe67344af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b344b3, 78 },
    { 0xc7344b3, 78 },
    { 0xee0344b4, 95 },
    { 0xd8344b6, 52 },
    { 0xc5f344b6, 90 },
    { 0xdb8344b8, 33 },
    { 0xf47344b9, 61 },
    { 0, 0 },
    { 0x401344bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704344c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72344c9, 34 },
    { 0x71d344ca, 23 },
    { 0xd13344ca, 87 },
    { 0xc8d344ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90344d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5344d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb344df, 61 },
    { 0x1a1344e0, 23 },
    { 0xfa3344e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3344e4, 46 },
    { 0, 0 },
    { 0xfe1344e6, 23 },
    { 0, 0 },
    { 0x5cd344e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d344ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c344f2, 90 },
    { 0xfd6344f3, 34 },
    { 0xa51344f4, 33 },
    { 0xef6344f3, 34 },
    { 0x63f344f4, 90 },
    { 0xca5344f7, 98 },
    { 0xb91344f8, 61 },
    { 0x7c9344f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d344fc, 33 },
    { 0xaa3344fd, 61 },
    { 0xe37344fe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba34504, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a3450a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b34512, 34 },
    { 0xc4b34513, 36 },
    { 0xd4234514, 98 },
    { 0x60f34515, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e34518, 95 },
    { 0x6c334519, 95 },
    { 0xfb434518, 95 },
    { 0xd6a3451b, 33 },
    { 0x7093451c, 77 },
    { 0xf3a3451a, 91 },
    { 0x20134518, 90 },
    { 0xb1734519, 43 },
    { 0x39a3451f, 98 },
    { 0x2534521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62534526, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64434529, 61 },
    { 0xae13452a, 76 },
    { 0x6e334529, 93 },
    { 0x40e3452c, 98 },
    { 0, 0 },
    { 0x28b3452e, 84 },
    { 0x3c13452f, 84 },
    { 0, 0 },
    { 0xa6e34531, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5734538, 55 },
    { 0x2534539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc3453c, 64 },
    { 0x983453c, 99 },
    { 0xa6d3453d, 76 },
    { 0, 0 },
    { 0xafe34540, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f34545, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c13454c, 88 },
    { 0xfe33454d, 33 },
    { 0xe773454d, 85 },
    { 0xb733454c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff234553, 43 },
    { 0x6bd34554, 46 },
    { 0x2de34555, 69 },
    { 0x6bb34556, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c3455c, 34 },
    { 0x6e3455d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4134567, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa243456b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6534570, 90 },
    { 0x34234571, 33 },
    { 0x82934572, 55 },
    { 0x3034573, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a34576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e3457a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe313457d, 90 },
    { 0x23e3457e, 55 },
    { 0x7223457f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f34584, 93 },
    { 0x48b34585, 95 },
    { 0x3534586, 34 },
    { 0, 0 },
    { 0x67c34588, 33 },
    { 0x82534588, 34 },
    { 0, 0 },
    { 0x96e3458b, 87 },
    { 0xeba3458c, 90 },
    { 0, 0 },
    { 0x9ed3458e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x19034591, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a34595, 48 },
    { 0, 0 },
    { 0x1b834597, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713459b, 46 },
    { 0x8843459c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29345a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8345a4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1345a7, 90 },
    { 0x42f345a8, 61 },
    { 0x224345a8, 90 },
    { 0x66d345aa, 33 },
    { 0, 0 },
    { 0x6c4345ac, 61 },
    { 0x910345ad, 93 },
    { 0xee9345ac, 43 },
    { 0x931345af, 46 },
    { 0xd19345ae, 95 },
    { 0x5dc345af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b345b4, 90 },
    { 0x289345b5, 52 },
    { 0, 0 },
    { 0x8f8345b7, 46 },
    { 0x70a345b8, 90 },
    { 0xd3e345b7, 33 },
    { 0x946345ba, 90 },
    { 0xfc4345bb, 90 },
    { 0, 0 },
    { 0xd47345bd, 80 },
    { 0x9bf345be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20345c8, 95 },
    { 0xad7345c9, 69 },
    { 0x7ab345ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7345ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a345d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x854345dc, 19 },
    { 0, 0 },
    { 0x20f345de, 55 },
    { 0, 0 },
    { 0x44a345e0, 91 },
    { 0, 0 },
    { 0x6a9345e2, 90 },
    { 0, 0 },
    { 0xbce345e4, 33 },
    { 0x651345e5, 58 },
    { 0xf62345e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3345eb, 61 },
    { 0x625345eb, 61 },
    { 0xbbf345ed, 99 },
    { 0x84f345eb, 33 },
    { 0xb75345eb, 61 },
    { 0x4b0345f0, 48 },
    { 0x60b345ee, 90 },
    { 0x904345f2, 23 },
    { 0xe97345f3, 46 },
    { 0xce4345f3, 78 },
    { 0x131345f5, 90 },
    { 0xd7c345f6, 90 },
    { 0x562345f7, 33 },
    { 0x892345f6, 90 },
    { 0x236345f8, 95 },
    { 0x98b345f5, 78 },
    { 0x848345eb, 43 },
    { 0, 0 },
    { 0xf0e345fd, 34 },
    { 0x615345fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12334601, 90 },
    { 0, 0 },
    { 0x62434603, 95 },
    { 0x7134604, 91 },
    { 0xe0d34605, 33 },
    { 0, 0 },
    { 0x59734607, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd3460f, 33 },
    { 0, 0 },
    { 0x35734611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e34614, 95 },
    { 0, 0 },
    { 0x5d334616, 52 },
    { 0x98e34617, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a3461d, 90 },
    { 0xbac3461d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4234625, 90 },
    { 0x8ad34626, 100 },
    { 0, 0 },
    { 0x19034628, 55 },
    { 0, 0 },
    { 0xeb3462a, 78 },
    { 0xcff3462b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b83462e, 55 },
    { 0x15f3462f, 98 },
    { 0xb253462f, 43 },
    { 0x43d3462f, 46 },
    { 0xee134632, 33 },
    { 0xa0534633, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a734639, 98 },
    { 0xcca3463a, 95 },
    { 0xcad3463b, 66 },
    { 0x1f234639, 91 },
    { 0x3f63463a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b34641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b83464a, 33 },
    { 0x3313464b, 33 },
    { 0x8e13464c, 66 },
    { 0xa43464c, 98 },
    { 0xb123464e, 23 },
    { 0xb003464f, 34 },
    { 0, 0 },
    { 0x98834651, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b34656, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec3465a, 93 },
    { 0x53465b, 61 },
    { 0, 0 },
    { 0xec43465d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c934660, 68 },
    { 0, 0 },
    { 0x14c34662, 98 },
    { 0, 0 },
    { 0x42b34664, 33 },
    { 0xca534664, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd34669, 23 },
    { 0x6ef3466a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763466e, 95 },
    { 0xcc53466e, 72 },
    { 0x32d3466f, 85 },
    { 0, 0 },
    { 0x23c34672, 33 },
    { 0xbe834673, 61 },
    { 0xb734673, 99 },
    { 0x5b934674, 98 },
    { 0x6ff34674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d3467c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd34683, 98 },
    { 0x69d34684, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53234688, 98 },
    { 0xbe334688, 95 },
    { 0xd323468a, 99 },
    { 0x6283468b, 78 },
    { 0x23634689, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d434690, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35234693, 61 },
    { 0xb1534694, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223469a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783469f, 88 },
    { 0xe76346a0, 98 },
    { 0xca5346a1, 93 },
    { 0, 0 },
    { 0xe9c346a3, 76 },
    { 0x1d2346a4, 36 },
    { 0xe08346a5, 68 },
    { 0x890346a5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61346a9, 61 },
    { 0x3ae346aa, 23 },
    { 0, 0 },
    { 0x1bb346ac, 33 },
    { 0, 0 },
    { 0xded346ae, 61 },
    { 0x8f8346af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01346b4, 90 },
    { 0, 0 },
    { 0xefe346b6, 90 },
    { 0x212346b6, 55 },
    { 0x2fa346b6, 23 },
    { 0xbb7346b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4346bc, 55 },
    { 0xc29346bc, 90 },
    { 0xc55346bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13346c1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x927346c4, 90 },
    { 0x9de346c5, 61 },
    { 0xcfc346c6, 98 },
    { 0, 0 },
    { 0x926346c8, 43 },
    { 0, 0 },
    { 0x389346ca, 33 },
    { 0xdb4346ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb346d0, 52 },
    { 0, 0 },
    { 0xf15346d2, 46 },
    { 0xd4f346d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21346d8, 33 },
    { 0x5a2346d8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1346dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464346e3, 95 },
    { 0xc3d346e4, 55 },
    { 0, 0 },
    { 0x687346e6, 93 },
    { 0, 0 },
    { 0x1ca346e8, 76 },
    { 0, 0 },
    { 0x226346ea, 90 },
    { 0x7b9346eb, 91 },
    { 0xcc4346ec, 77 },
    { 0, 0 },
    { 0xbd7346ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596346f5, 98 },
    { 0x9fc346f6, 95 },
    { 0x112346f5, 40 },
    { 0xbce346f8, 76 },
    { 0x106346f8, 65 },
    { 0x5ea346fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d346fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78134704, 47 },
    { 0x8c434704, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab34708, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c3470d, 90 },
    { 0x9683470e, 55 },
    { 0xb9e3470f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa334713, 52 },
    { 0x1ae34713, 95 },
    { 0x25534715, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90534719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc3471d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff34721, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b34724, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a934728, 23 },
    { 0x53c34729, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233472d, 61 },
    { 0xe53472e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e934731, 61 },
    { 0x47234732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed34735, 78 },
    { 0x71534736, 38 },
    { 0xe6534736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa13473c, 95 },
    { 0x5fc3473c, 90 },
    { 0x2e13473c, 90 },
    { 0, 0 },
    { 0x19a34740, 93 },
    { 0xdd634741, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5834744, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f034748, 17 },
    { 0, 0 },
    { 0xc923474a, 85 },
    { 0, 0 },
    { 0x36f3474c, 93 },
    { 0x6e63474d, 95 },
    { 0x91b3474e, 98 },
    { 0x38b3474e, 55 },
    { 0xb2734750, 90 },
    { 0x93c34751, 33 },
    { 0x65a34752, 98 },
    { 0xfa03474c, 91 },
    { 0x6f934754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda534759, 95 },
    { 0x19934759, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7213475e, 66 },
    { 0x5663475f, 23 },
    { 0x5b34760, 76 },
    { 0xd6c34760, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f34764, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f134769, 92 },
    { 0, 0 },
    { 0xcf93476b, 34 },
    { 0x7a23476b, 90 },
    { 0xb63476d, 55 },
    { 0x16d3476e, 55 },
    { 0xda73476e, 33 },
    { 0xdd43476e, 76 },
    { 0xc2a34771, 33 },
    { 0, 0 },
    { 0x2ad34773, 100 },
    { 0x83d34774, 90 },
    { 0x23e34774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb83477d, 33 },
    { 0x2913477e, 43 },
    { 0, 0 },
    { 0xbc334780, 90 },
    { 0x82634781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea534784, 100 },
    { 0xb6834784, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2334789, 78 },
    { 0, 0 },
    { 0x593478b, 61 },
    { 0, 0 },
    { 0xda13478d, 75 },
    { 0x2da3478e, 69 },
    { 0xc7d3478f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b334792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de34797, 43 },
    { 0, 0 },
    { 0xdf734799, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6347a7, 43 },
    { 0, 0 },
    { 0x7fc347a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4347b0, 98 },
    { 0, 0 },
    { 0xb08347b2, 33 },
    { 0x258347b3, 55 },
    { 0, 0 },
    { 0x39b347b5, 55 },
    { 0x3f3347b6, 90 },
    { 0x393347b7, 80 },
    { 0x52f347b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50347bc, 98 },
    { 0x8d5347bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1347c4, 91 },
    { 0x895347c5, 33 },
    { 0, 0 },
    { 0xd3b347c7, 55 },
    { 0x365347c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51347cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39347d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x389347da, 23 },
    { 0xf91347db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27347df, 61 },
    { 0, 0 },
    { 0xf21347e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa347e6, 46 },
    { 0x67f347e6, 100 },
    { 0x9af347e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615347ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3347f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81434800, 95 },
    { 0xbb034801, 61 },
    { 0xa1334802, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31334805, 61 },
    { 0xc7734806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80734809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc3480d, 87 },
    { 0x35c3480d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b34813, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b34816, 33 },
    { 0x93c34817, 98 },
    { 0xc9334818, 65 },
    { 0x84d34816, 61 },
    { 0, 0 },
    { 0xea93481b, 33 },
    { 0x7ec3481c, 78 },
    { 0, 0 },
    { 0x48b3481e, 98 },
    { 0x91d3481e, 55 },
    { 0, 0 },
    { 0xae834821, 96 },
    { 0, 0 },
    { 0x1e234823, 90 },
    { 0x1bf34823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5634830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d34835, 23 },
    { 0, 0 },
    { 0x44f34837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd34842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2534845, 95 },
    { 0x3d134846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda834851, 33 },
    { 0xd0534852, 23 },
    { 0xe7534853, 90 },
    { 0x6f34854, 97 },
    { 0x70734855, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92934859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f134865, 17 },
    { 0xebd34866, 95 },
    { 0xd4f34866, 69 },
    { 0xeb734865, 36 },
    { 0x91034869, 38 },
    { 0x9843486a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d334872, 95 },
    { 0, 0 },
    { 0x62034874, 33 },
    { 0x68a34874, 61 },
    { 0x4f634875, 95 },
    { 0x5a134877, 95 },
    { 0, 0 },
    { 0x1b734879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9703487d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c34880, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7834893, 33 },
    { 0x64334894, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd23489e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc348a1, 40 },
    { 0xf5b348a2, 63 },
    { 0x6c3348a3, 98 },
    { 0xd95348a4, 23 },
    { 0x668348a3, 61 },
    { 0, 0 },
    { 0x125348a7, 87 },
    { 0xbb8348a7, 95 },
    { 0xd30348a9, 61 },
    { 0x76d348aa, 94 },
    { 0xcb9348ab, 90 },
    { 0x968348aa, 23 },
    { 0x6c7348ad, 55 },
    { 0x1e7348ae, 78 },
    { 0, 0 },
    { 0xaa8348b0, 55 },
    { 0x900348b1, 61 },
    { 0, 0 },
    { 0x1f9348b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea348b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463348ba, 93 },
    { 0x5e4348bb, 99 },
    { 0x363348bc, 40 },
    { 0x498348bd, 76 },
    { 0x29d348bd, 23 },
    { 0xbe4348bf, 33 },
    { 0x458348bd, 23 },
    { 0x2e0348bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2348c5, 98 },
    { 0xb8d348c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850348cb, 95 },
    { 0, 0 },
    { 0xde1348cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb348d2, 90 },
    { 0x9d2348d2, 61 },
    { 0xfc2348d2, 98 },
    { 0xc32348d5, 76 },
    { 0xfce348d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7348da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x988348dd, 69 },
    { 0, 0 },
    { 0x589348df, 81 },
    { 0, 0 },
    { 0x55f348e1, 90 },
    { 0, 0 },
    { 0xa74348e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36348e6, 76 },
    { 0x1e0348e7, 77 },
    { 0x320348e6, 90 },
    { 0x5b4348e9, 23 },
    { 0x6b0348e9, 98 },
    { 0xf6e348eb, 90 },
    { 0x2f8348ec, 93 },
    { 0x20e348ed, 23 },
    { 0x2c0348ec, 85 },
    { 0xb8a348e7, 55 },
    { 0xaa2348ec, 98 },
    { 0xba2348f1, 98 },
    { 0x92d348f1, 98 },
    { 0, 0 },
    { 0x603348f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e348fc, 95 },
    { 0xf22348fc, 98 },
    { 0xe02348fd, 100 },
    { 0, 0 },
    { 0x9be34900, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc243490b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c3490f, 17 },
    { 0, 0 },
    { 0x3f734911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a234915, 33 },
    { 0, 0 },
    { 0x9734917, 98 },
    { 0x1df34918, 66 },
    { 0, 0 },
    { 0xb783491a, 61 },
    { 0xd2a3491b, 84 },
    { 0x66d3491b, 95 },
    { 0xcff3491d, 43 },
    { 0x9a3491d, 56 },
    { 0x1523491f, 76 },
    { 0xe113491d, 55 },
    { 0x6323491c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b34924, 90 },
    { 0xd4034925, 90 },
    { 0x82b34924, 93 },
    { 0xa9734924, 61 },
    { 0, 0 },
    { 0x98134929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f13492c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d3492f, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a34932, 43 },
    { 0xb8034933, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26234939, 90 },
    { 0xccc3493a, 43 },
    { 0x7f33493b, 46 },
    { 0xf5b3493a, 61 },
    { 0x7e33493d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad34945, 90 },
    { 0xa8634945, 100 },
    { 0x1a934947, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb193494a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf013494e, 98 },
    { 0, 0 },
    { 0xaf734950, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66734957, 91 },
    { 0, 0 },
    { 0xcfd34959, 95 },
    { 0xc5834959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f3495d, 34 },
    { 0x4a3495d, 90 },
    { 0xc353495f, 43 },
    { 0x7cd3495f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d34963, 46 },
    { 0x4c334963, 76 },
    { 0x74c34963, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0734968, 61 },
    { 0xeb434969, 95 },
    { 0xee53496a, 76 },
    { 0, 0 },
    { 0x4a93496c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec34970, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8034973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43034978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5034980, 61 },
    { 0, 0 },
    { 0x73b34982, 36 },
    { 0, 0 },
    { 0x83534984, 95 },
    { 0x5d534985, 88 },
    { 0, 0 },
    { 0xb7534987, 98 },
    { 0xa1234988, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d3498b, 90 },
    { 0x8053498c, 76 },
    { 0xdba3498d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee34991, 55 },
    { 0, 0 },
    { 0x8934993, 99 },
    { 0x13234994, 98 },
    { 0xd0934995, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33834998, 61 },
    { 0xb9934999, 88 },
    { 0xeb534999, 61 },
    { 0, 0 },
    { 0xc393499c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81349a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e349a8, 90 },
    { 0xc2e349a8, 68 },
    { 0x222349aa, 23 },
    { 0x5d7349a8, 90 },
    { 0x936349ac, 81 },
    { 0xde9349ad, 38 },
    { 0xb7a349ad, 90 },
    { 0x9ba349af, 34 },
    { 0xa28349ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16349b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455349bd, 61 },
    { 0x136349be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80349c1, 23 },
    { 0xd8d349c2, 90 },
    { 0x329349c3, 98 },
    { 0x6b5349c3, 66 },
    { 0xe5c349c3, 61 },
    { 0xdbc349c3, 78 },
    { 0x29b349c5, 96 },
    { 0x31e349c8, 33 },
    { 0x68d349c9, 61 },
    { 0x1d7349ca, 34 },
    { 0xac349cb, 33 },
    { 0x1349cc, 43 },
    { 0x6bb349cd, 64 },
    { 0, 0 },
    { 0x76b349cf, 23 },
    { 0xb73349d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7349d3, 61 },
    { 0, 0 },
    { 0x26c349d5, 90 },
    { 0x21b349d6, 33 },
    { 0xd43349d5, 98 },
    { 0x6ba349d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d349dc, 33 },
    { 0xe72349dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139349e4, 90 },
    { 0x48f349e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84349e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552349ef, 95 },
    { 0, 0 },
    { 0xb80349f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc349f4, 23 },
    { 0x176349f5, 99 },
    { 0x89c349f4, 76 },
    { 0x181349f5, 61 },
    { 0, 0 },
    { 0x86349f9, 52 },
    { 0x7d5349fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27434a01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b34a09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b534a0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b334a11, 93 },
    { 0x87d34a12, 90 },
    { 0, 0 },
    { 0xf7d34a14, 95 },
    { 0xeb834a14, 33 },
    { 0x41b34a14, 90 },
    { 0x534a17, 61 },
    { 0x83a34a14, 98 },
    { 0xf5534a19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6034a1e, 61 },
    { 0x4b134a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c34a23, 34 },
    { 0xb9c34a23, 95 },
    { 0x7d134a25, 33 },
    { 0x64034a26, 69 },
    { 0, 0 },
    { 0xeeb34a28, 34 },
    { 0xabc34a29, 76 },
    { 0, 0 },
    { 0xe1e34a2b, 93 },
    { 0x68734a2b, 95 },
    { 0x21034a2d, 55 },
    { 0, 0 },
    { 0xa0a34a2f, 52 },
    { 0, 0 },
    { 0x3a134a31, 68 },
    { 0, 0 },
    { 0x31034a33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb34a38, 98 },
    { 0x96134a39, 88 },
    { 0x97734a39, 61 },
    { 0, 0 },
    { 0x4fa34a3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22534a43, 61 },
    { 0x1d034a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd34a47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f34a4a, 90 },
    { 0x24634a4a, 87 },
    { 0xa034a4c, 66 },
    { 0x7f034a4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d34a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e34a5b, 33 },
    { 0, 0 },
    { 0x48b34a5d, 66 },
    { 0x89034a5e, 84 },
    { 0xce34a5f, 100 },
    { 0x74a34a5e, 95 },
    { 0x6ac34a61, 90 },
    { 0, 0 },
    { 0x40434a63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb034a6e, 55 },
    { 0x8de34a6f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2534a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc34a76, 90 },
    { 0, 0 },
    { 0x9c534a78, 96 },
    { 0, 0 },
    { 0xecd34a7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff34a7f, 76 },
    { 0, 0 },
    { 0xad534a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f034a84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e334a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d34a8e, 23 },
    { 0x65134a8f, 55 },
    { 0x5ab34a90, 90 },
    { 0x26734a90, 90 },
    { 0xbac34a92, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b134a98, 90 },
    { 0x25b34a98, 95 },
    { 0xf7034a9a, 80 },
    { 0x89c34a99, 61 },
    { 0x87b34a9c, 100 },
    { 0xe534a9b, 95 },
    { 0x54634a9c, 76 },
    { 0xf9934a9f, 90 },
    { 0x58634a9a, 95 },
    { 0, 0 },
    { 0x86834aa2, 40 },
    { 0x27f34aa3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a734aa6, 78 },
    { 0, 0 },
    { 0xfd734aa8, 93 },
    { 0, 0 },
    { 0xf0c34aaa, 90 },
    { 0x4fe34aab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b234aae, 100 },
    { 0x4ff34aae, 98 },
    { 0xa1e34ab0, 23 },
    { 0, 0 },
    { 0x1b734ab2, 75 },
    { 0, 0 },
    { 0x72c34ab4, 46 },
    { 0x71034ab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee34ab8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba34abe, 100 },
    { 0x17834abf, 61 },
    { 0, 0 },
    { 0xad534ac1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a34ac4, 90 },
    { 0x2d34ac5, 93 },
    { 0x94434ac6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb34ac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77834ace, 90 },
    { 0x7a434acf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50834ad7, 76 },
    { 0, 0 },
    { 0x22c34ad9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13034adc, 33 },
    { 0xf5434add, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c34ae2, 61 },
    { 0, 0 },
    { 0x83834ae4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4334ae7, 98 },
    { 0, 0 },
    { 0xfb734ae9, 99 },
    { 0xbaa34aea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15534af7, 66 },
    { 0xfb334af7, 93 },
    { 0x8b234af9, 61 },
    { 0x91734af8, 76 },
    { 0, 0 },
    { 0x13c34afc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e34aff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f434b02, 46 },
    { 0, 0 },
    { 0xda434b04, 61 },
    { 0, 0 },
    { 0x2d734b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d934b0b, 93 },
    { 0xe4c34b0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a034b10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b934b19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5134b1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad734b22, 43 },
    { 0, 0 },
    { 0x74434b24, 33 },
    { 0x2cd34b25, 93 },
    { 0x95834b26, 19 },
    { 0xdc734b24, 46 },
    { 0x7a834b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf834b2d, 43 },
    { 0xdb234b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c534b38, 43 },
    { 0xc5734b39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c34b3c, 33 },
    { 0, 0 },
    { 0xbe634b3e, 90 },
    { 0, 0 },
    { 0xf8a34b40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30234b4a, 23 },
    { 0x54934b4b, 78 },
    { 0xefe34b4a, 52 },
    { 0, 0 },
    { 0xf234b4e, 67 },
    { 0xa6234b4e, 95 },
    { 0x89c34b4e, 34 },
    { 0x8d34b51, 100 },
    { 0x81834b52, 65 },
    { 0, 0 },
    { 0x1e034b54, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1234b59, 23 },
    { 0x85134b5a, 66 },
    { 0x9ef34b5a, 90 },
    { 0x17134b5c, 98 },
    { 0x6634b59, 90 },
    { 0xbd534b5e, 95 },
    { 0, 0 },
    { 0x38634b60, 85 },
    { 0, 0 },
    { 0x83134b62, 46 },
    { 0, 0 },
    { 0xdd934b64, 61 },
    { 0x5f434b65, 23 },
    { 0x16134b66, 98 },
    { 0xbe534b67, 90 },
    { 0x44d34b67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5434b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff34b72, 98 },
    { 0xc6134b72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2534b7f, 93 },
    { 0, 0 },
    { 0x17f34b81, 97 },
    { 0x4e34b82, 88 },
    { 0, 0 },
    { 0xf0834b84, 61 },
    { 0xa3034b85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd834b88, 61 },
    { 0xc1534b89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd934b8e, 80 },
    { 0, 0 },
    { 0x4f34b90, 98 },
    { 0x8d634b91, 34 },
    { 0xb6934b90, 19 },
    { 0x5d134b93, 33 },
    { 0x72734b94, 93 },
    { 0xebd34b92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3034b99, 90 },
    { 0xc7934b9a, 33 },
    { 0, 0 },
    { 0x6b834b9c, 90 },
    { 0, 0 },
    { 0xb1e34b9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81034ba3, 100 },
    { 0xb4b34ba3, 100 },
    { 0x83634ba5, 98 },
    { 0x1cc34ba6, 77 },
    { 0x87734ba7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b734bac, 66 },
    { 0xc1d34bad, 90 },
    { 0x6d334bae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf34bb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c934bba, 61 },
    { 0xee134bbb, 95 },
    { 0x48934bbc, 76 },
    { 0x5ff34bbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3334bc1, 61 },
    { 0x46b34bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2734bc9, 98 },
    { 0x86134bca, 81 },
    { 0xd2334bc9, 93 },
    { 0, 0 },
    { 0x52f34bcd, 98 },
    { 0x8a434bce, 33 },
    { 0, 0 },
    { 0x68134bd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90534bd3, 95 },
    { 0, 0 },
    { 0xd3a34bd5, 33 },
    { 0x7e234bd6, 23 },
    { 0, 0 },
    { 0x7cb34bd8, 61 },
    { 0xbc734bd9, 99 },
    { 0, 0 },
    { 0x7b034bdb, 98 },
    { 0, 0 },
    { 0x69e34bdd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e34be5, 38 },
    { 0, 0 },
    { 0x94734be7, 23 },
    { 0, 0 },
    { 0xb4934be9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa034bf0, 55 },
    { 0xa234bf1, 90 },
    { 0xc3d34bf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a534bf8, 23 },
    { 0xacb34bf8, 90 },
    { 0x99334bf8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe34c02, 87 },
    { 0x14134c03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee434c06, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x55434c09, 81 },
    { 0xcba34c0a, 90 },
    { 0x22834c0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b34c0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1234c18, 43 },
    { 0x45d34c18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1634c1c, 85 },
    { 0x3e234c1d, 90 },
    { 0x7ad34c1e, 55 },
    { 0x89334c1c, 34 },
    { 0xb0b34c1f, 55 },
    { 0xc3b34c1f, 98 },
    { 0xd8334c22, 61 },
    { 0x6ec34c21, 61 },
    { 0, 0 },
    { 0xe3434c25, 90 },
    { 0, 0 },
    { 0x6134c27, 33 },
    { 0xefc34c28, 65 },
    { 0x3e634c27, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf734c2c, 55 },
    { 0xd0f34c2c, 61 },
    { 0xda334c2e, 95 },
    { 0, 0 },
    { 0x9de34c30, 95 },
    { 0x82534c30, 43 },
    { 0xf5934c31, 95 },
    { 0x98a34c33, 66 },
    { 0x71334c34, 23 },
    { 0xd5034c33, 5 },
    { 0, 0 },
    { 0x30734c37, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5134c3a, 61 },
    { 0, 0 },
    { 0x91134c3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da34c3f, 46 },
    { 0x7e134c40, 55 },
    { 0xb2d34c40, 38 },
    { 0xf334c3f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba34c47, 55 },
    { 0xe1e34c48, 23 },
    { 0xcd34c49, 52 },
    { 0x3fc34c49, 61 },
    { 0x72734c49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5334c4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2034c53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35634c58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xba734c5b, 46 },
    { 0, 0 },
    { 0xb9434c5d, 55 },
    { 0x38c34c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68834c66, 99 },
    { 0, 0 },
    { 0xaf134c68, 33 },
    { 0x99434c69, 90 },
    { 0x54a34c6a, 33 },
    { 0, 0 },
    { 0xa6c34c6c, 93 },
    { 0x3fe34c6d, 61 },
    { 0x1c534c6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd34c73, 90 },
    { 0x50f34c73, 34 },
    { 0, 0 },
    { 0xec334c76, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17034c80, 98 },
    { 0, 0 },
    { 0xd3134c82, 90 },
    { 0x1f634c83, 90 },
    { 0x68234c82, 75 },
    { 0x26034c85, 95 },
    { 0xd0f34c86, 76 },
    { 0x64434c86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a34c91, 90 },
    { 0xa7934c92, 61 },
    { 0x60534c91, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed34c96, 90 },
    { 0x1c34c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51334c9c, 91 },
    { 0xf3934c9c, 34 },
    { 0x3d234c9e, 95 },
    { 0x59334c9f, 33 },
    { 0, 0 },
    { 0xcd34ca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa834ca6, 46 },
    { 0xe8834ca7, 23 },
    { 0x82834ca8, 89 },
    { 0xc7234ca9, 90 },
    { 0, 0 },
    { 0xcfb34cab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a34cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4734cb3, 33 },
    { 0xe8d34cb4, 61 },
    { 0x39a34cb4, 36 },
    { 0x8d434cb6, 76 },
    { 0x38234cb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a034cbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed734cc2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26634cc6, 61 },
    { 0x3934cc7, 90 },
    { 0, 0 },
    { 0x24734cc9, 98 },
    { 0, 0 },
    { 0x1fd34ccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6434cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd34cd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11034cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b34cda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c34cdd, 67 },
    { 0, 0 },
    { 0x84d34cdf, 76 },
    { 0x6b934ce0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51134ce5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3434ce9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52534cec, 93 },
    { 0x4e034cec, 90 },
    { 0, 0 },
    { 0x92734cef, 23 },
    { 0xda834cf0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d34cf3, 90 },
    { 0xd6234cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b34cf8, 23 },
    { 0, 0 },
    { 0xa3f34cfa, 76 },
    { 0x2a34cfb, 98 },
    { 0xc6834cfc, 78 },
    { 0x55c34cfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc34d07, 61 },
    { 0xd534d07, 34 },
    { 0, 0 },
    { 0x5ab34d0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d334d0d, 90 },
    { 0xf6c34d0e, 99 },
    { 0xbd534d0d, 98 },
    { 0x4d434d10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33134d15, 90 },
    { 0xe3834d16, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad34d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec34d1d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed034d26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e34d2d, 43 },
    { 0, 0 },
    { 0xf4b34d2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a34d33, 46 },
    { 0x58434d34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d34d39, 98 },
    { 0xf0834d3a, 23 },
    { 0xbc334d3b, 95 },
    { 0xf9934d3c, 43 },
    { 0xb1534d3b, 38 },
    { 0x7534d3e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6034d41, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb734d4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70734d4f, 76 },
    { 0, 0 },
    { 0xa34d51, 61 },
    { 0x90834d51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d34d57, 43 },
    { 0, 0 },
    { 0xfb34d59, 95 },
    { 0x46434d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64634d5f, 95 },
    { 0x6f934d60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc834d64, 61 },
    { 0xc1b34d65, 66 },
    { 0, 0 },
    { 0x2e934d67, 90 },
    { 0x45434d67, 55 },
    { 0, 0 },
    { 0xd834d6a, 98 },
    { 0, 0 },
    { 0x46c34d6c, 46 },
    { 0x42234d6d, 55 },
    { 0x78b34d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21934d72, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80834d84, 90 },
    { 0x6d434d85, 33 },
    { 0x9e134d85, 95 },
    { 0x88334d87, 90 },
    { 0x8ae34d87, 95 },
    { 0x3a034d89, 78 },
    { 0xae634d88, 23 },
    { 0, 0 },
    { 0x90a34d8c, 66 },
    { 0x5a634d8d, 95 },
    { 0x5d234d8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e34d92, 65 },
    { 0, 0 },
    { 0x1e134d94, 95 },
    { 0x9f334d94, 23 },
    { 0x3ff34d96, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x64534d99, 33 },
    { 0, 0 },
    { 0x73134d9b, 55 },
    { 0x5ec34d9b, 90 },
    { 0, 0 },
    { 0xcac34d9e, 33 },
    { 0x32a34d9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b34da5, 34 },
    { 0x1e634da6, 55 },
    { 0, 0 },
    { 0x2bb34da8, 90 },
    { 0x6c334da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f34dae, 95 },
    { 0, 0 },
    { 0x1f734db0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e434db5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86634dba, 93 },
    { 0x46434dbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e34dbf, 33 },
    { 0, 0 },
    { 0xc3534dc1, 61 },
    { 0, 0 },
    { 0x75e34dc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d34dcb, 95 },
    { 0x3b34dcb, 55 },
    { 0x4c834dcc, 38 },
    { 0x7ea34dce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee34dd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6934dd4, 33 },
    { 0, 0 },
    { 0xd0d34dd6, 75 },
    { 0x68a34dd7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77734dde, 84 },
    { 0x80c34ddf, 90 },
    { 0xfc534de0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b34de7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a34dea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb34df1, 61 },
    { 0xd2a34df2, 98 },
    { 0xe0e34df3, 23 },
    { 0xb3034df4, 91 },
    { 0xd3934df5, 61 },
    { 0xb1334df6, 33 },
    { 0, 0 },
    { 0x8c534df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde34dfb, 23 },
    { 0xe2134dfb, 90 },
    { 0xb1434dfc, 98 },
    { 0x9e34dfb, 91 },
    { 0x48d34dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa34e02, 78 },
    { 0, 0 },
    { 0x38c34e04, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98534e0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59434e17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f34e1d, 76 },
    { 0x2f634e1d, 23 },
    { 0x5a134e1d, 33 },
    { 0xa6234e1d, 61 },
    { 0x7334e21, 76 },
    { 0xd9a34e20, 98 },
    { 0xcb334e23, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc34e26, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df34e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16534e34, 55 },
    { 0, 0 },
    { 0xc7e34e36, 99 },
    { 0xae434e36, 43 },
    { 0x82b34e36, 17 },
    { 0xa9c34e38, 23 },
    { 0xa0234e3a, 66 },
    { 0, 0 },
    { 0x6c934e3c, 94 },
    { 0x57434e3c, 91 },
    { 0xc9f34e3e, 34 },
    { 0x5f834e3e, 33 },
    { 0x9b934e3e, 70 },
    { 0xfd834e40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a534e44, 69 },
    { 0, 0 },
    { 0x14834e46, 91 },
    { 0x1c934e47, 55 },
    { 0, 0 },
    { 0xa4534e49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a34e4e, 97 },
    { 0xcaf34e4f, 17 },
    { 0xe9b34e4f, 61 },
    { 0x8d834e51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f34e54, 90 },
    { 0x8cd34e55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d34e62, 80 },
    { 0, 0 },
    { 0x70f34e64, 99 },
    { 0xcfc34e64, 77 },
    { 0x91734e66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a434e6c, 98 },
    { 0xeb234e6d, 23 },
    { 0x7e334e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77934e73, 98 },
    { 0xfdd34e73, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc34e79, 33 },
    { 0xe2034e79, 98 },
    { 0x54634e79, 23 },
    { 0x12534e7c, 46 },
    { 0x20234e7d, 76 },
    { 0x63934e7c, 33 },
    { 0xfa834e7c, 77 },
    { 0x4cc34e80, 55 },
    { 0x82134e7a, 90 },
    { 0x7f934e7a, 76 },
    { 0xefa34e82, 85 },
    { 0x7f34e84, 61 },
    { 0x5a034e84, 43 },
    { 0xc8334e86, 43 },
    { 0xd7734e86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf34e8a, 93 },
    { 0xcd034e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18334e8e, 87 },
    { 0x18734e8f, 90 },
    { 0x9bb34e90, 90 },
    { 0x52f34e8e, 90 },
    { 0xfac34e91, 95 },
    { 0, 0 },
    { 0x49634e94, 61 },
    { 0xb1634e94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16234e98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf634e9f, 61 },
    { 0x95934ea0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2634ea4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x49634ea7, 90 },
    { 0x3a734ea8, 61 },
    { 0x47234ea9, 87 },
    { 0, 0 },
    { 0xdfa34eab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c34eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d734eb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc34eb9, 90 },
    { 0, 0 },
    { 0x6fb34ebb, 43 },
    { 0, 0 },
    { 0x1ba34ebd, 81 },
    { 0xb5334ebe, 65 },
    { 0x79d34ebf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34ec3, 68 },
    { 0xcdd34ec4, 33 },
    { 0xc6d34ec3, 34 },
    { 0x92e34ec6, 62 },
    { 0, 0 },
    { 0x96f34ec8, 66 },
    { 0x2da34ec9, 46 },
    { 0x99334ec9, 61 },
    { 0x6b334ecb, 90 },
    { 0, 0 },
    { 0x2c734ecd, 61 },
    { 0x8c434ecd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c034ed5, 100 },
    { 0x1b434ed5, 100 },
    { 0x4a34ed6, 98 },
    { 0x44a34ed8, 76 },
    { 0xa8d34ed8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7634edc, 34 },
    { 0xf0234edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34ee0, 33 },
    { 0xe4f34ee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98634ee4, 61 },
    { 0x71334ee5, 61 },
    { 0x14e34ee6, 33 },
    { 0x7aa34ee4, 23 },
    { 0xdae34ee8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4434eec, 23 },
    { 0, 0 },
    { 0x6d234eee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da34ef5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a34ef9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a034efc, 33 },
    { 0, 0 },
    { 0xede34efe, 33 },
    { 0x51034efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac634f05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xab334f08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11734f0d, 76 },
    { 0xf5134f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54734f11, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a234f14, 98 },
    { 0x6fe34f15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3634f18, 61 },
    { 0x5dd34f19, 61 },
    { 0, 0 },
    { 0x7a234f1b, 90 },
    { 0x61334f1c, 91 },
    { 0xd6334f1d, 33 },
    { 0x79334f1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x19734f21, 33 },
    { 0, 0 },
    { 0x34f23, 76 },
    { 0x17934f23, 76 },
    { 0, 0 },
    { 0xa9e34f26, 95 },
    { 0, 0 },
    { 0x4ef34f28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f34f2b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6734f30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e434f38, 90 },
    { 0x9e234f38, 98 },
    { 0x5bf34f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba034f3d, 76 },
    { 0x69c34f3e, 91 },
    { 0x22534f3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9734f43, 33 },
    { 0x4b134f44, 43 },
    { 0x66134f43, 98 },
    { 0x86f34f46, 33 },
    { 0x68e34f44, 76 },
    { 0, 0 },
    { 0x6ab34f49, 98 },
    { 0xf234f49, 76 },
    { 0x28134f4b, 77 },
    { 0x90a34f4b, 55 },
    { 0x44e34f4d, 90 },
    { 0xd0b34f4e, 95 },
    { 0x31c34f4f, 55 },
    { 0x7ff34f50, 66 },
    { 0x3ff34f51, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50434f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b434f59, 61 },
    { 0, 0 },
    { 0xfee34f5b, 46 },
    { 0xb3734f5c, 23 },
    { 0, 0 },
    { 0x1a034f5e, 78 },
    { 0x32634f5f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea34f62, 61 },
    { 0xcb934f63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e34f67, 52 },
    { 0x3bb34f68, 33 },
    { 0x56434f67, 52 },
    { 0xaed34f6a, 33 },
    { 0xda434f6a, 76 },
    { 0xe34f6c, 98 },
    { 0xcbc34f6a, 61 },
    { 0x79034f69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c34f75, 55 },
    { 0x46934f76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec234f7e, 33 },
    { 0x6dc34f7e, 90 },
    { 0, 0 },
    { 0xee334f81, 61 },
    { 0xa5734f82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d34f8a, 93 },
    { 0xd0e34f8a, 61 },
    { 0x80334f8c, 61 },
    { 0, 0 },
    { 0xfe034f8e, 90 },
    { 0x34934f8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae34f94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6734f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc134fa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db34fa8, 33 },
    { 0xb8034fa8, 75 },
    { 0x2d334faa, 42 },
    { 0x1ad34faa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec34fb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e034fbe, 33 },
    { 0x5d834fbf, 87 },
    { 0x6e534fbf, 33 },
    { 0x6f434fbf, 90 },
    { 0, 0 },
    { 0x6e134fc3, 23 },
    { 0, 0 },
    { 0x5ad34fc5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37034fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50934fd1, 33 },
    { 0x3b34fd2, 33 },
    { 0, 0 },
    { 0xa9834fd4, 90 },
    { 0, 0 },
    { 0xf7634fd6, 87 },
    { 0, 0 },
    { 0xd6b34fd8, 98 },
    { 0xa9e34fd8, 43 },
    { 0, 0 },
    { 0x4f234fdb, 55 },
    { 0, 0 },
    { 0x16734fdd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1134fe1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f234fe6, 34 },
    { 0x4ac34fe7, 23 },
    { 0x8e134fe7, 96 },
    { 0x93f34fe9, 55 },
    { 0, 0 },
    { 0xaf534feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c34ff3, 99 },
    { 0xc4434ff4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82634ff9, 52 },
    { 0xfc34ff9, 100 },
    { 0, 0 },
    { 0xab034ffc, 23 },
    { 0x8a34ffd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf735001, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d35005, 34 },
    { 0xcc235006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a35009, 90 },
    { 0x49f3500a, 76 },
    { 0xfc03500b, 34 },
    { 0, 0 },
    { 0xeb83500d, 43 },
    { 0x4553500e, 61 },
    { 0x5e43500f, 52 },
    { 0x22e3500e, 93 },
    { 0, 0 },
    { 0xb1c35012, 40 },
    { 0xea335013, 85 },
    { 0x4735012, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe35018, 95 },
    { 0x25f35019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda635020, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9735029, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2473502e, 85 },
    { 0x1623502f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14835032, 61 },
    { 0, 0 },
    { 0x43f35034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34935037, 33 },
    { 0, 0 },
    { 0xf2035039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a35040, 76 },
    { 0, 0 },
    { 0x81935042, 34 },
    { 0, 0 },
    { 0x48c35044, 90 },
    { 0x94b35044, 23 },
    { 0x16335045, 33 },
    { 0xa5735045, 23 },
    { 0x47d35045, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2203504d, 84 },
    { 0xf9a3504e, 61 },
    { 0x94f3504f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e335052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb35056, 55 },
    { 0xc9235056, 61 },
    { 0, 0 },
    { 0x21435059, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e3505e, 98 },
    { 0x5253505f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8335064, 33 },
    { 0x6ed35065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a35069, 97 },
    { 0xecc35069, 98 },
    { 0x89a35069, 90 },
    { 0x4093506c, 76 },
    { 0, 0 },
    { 0xc2a3506e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c935074, 23 },
    { 0x61d35075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47835078, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c3507d, 69 },
    { 0xd9a3507e, 76 },
    { 0, 0 },
    { 0xb1635080, 90 },
    { 0x86435081, 23 },
    { 0, 0 },
    { 0xba335083, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3335086, 33 },
    { 0xe7235087, 33 },
    { 0xf7035088, 87 },
    { 0xa7535089, 76 },
    { 0x19b35089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e23508f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3535092, 46 },
    { 0x54735093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac435097, 46 },
    { 0x55235098, 55 },
    { 0, 0 },
    { 0x7c23509a, 34 },
    { 0x5ac3509a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7623509e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72350a5, 61 },
    { 0, 0 },
    { 0xdc7350a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c350ac, 90 },
    { 0xfa7350ad, 77 },
    { 0x8da350ae, 43 },
    { 0xeb1350af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b350b2, 90 },
    { 0, 0 },
    { 0x88a350b4, 40 },
    { 0xa5c350b5, 34 },
    { 0x73f350b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34350b9, 33 },
    { 0xbd0350b9, 61 },
    { 0xd08350b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f350bf, 46 },
    { 0x291350bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27350ca, 75 },
    { 0x65d350cb, 66 },
    { 0, 0 },
    { 0xc28350cd, 76 },
    { 0xd0d350ce, 43 },
    { 0x1e0350ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f350d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa350d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x898350da, 83 },
    { 0xd9f350db, 78 },
    { 0xe65350dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b350e0, 90 },
    { 0x94f350e1, 61 },
    { 0x5c9350e0, 98 },
    { 0x355350e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95350e8, 52 },
    { 0xeec350e9, 66 },
    { 0x6b5350e9, 100 },
    { 0xa40350ea, 98 },
    { 0x86350ea, 23 },
    { 0x5ed350ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c350f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b350f6, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e350fa, 47 },
    { 0x86f350fa, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x415350fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e635102, 95 },
    { 0, 0 },
    { 0x31a35104, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76135107, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae535110, 90 },
    { 0x11335110, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81035114, 61 },
    { 0, 0 },
    { 0xc9535116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e3511b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d35120, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b35124, 90 },
    { 0x4bc35125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d53512c, 33 },
    { 0x46e3512d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6535133, 90 },
    { 0x86b35134, 90 },
    { 0x6435134, 61 },
    { 0xceb35135, 46 },
    { 0x42c35137, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda13513a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd13513d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56535141, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d3514c, 43 },
    { 0x13e3514d, 100 },
    { 0xf193514e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c35151, 33 },
    { 0, 0 },
    { 0xab735153, 93 },
    { 0x29435154, 55 },
    { 0, 0 },
    { 0xd7635156, 93 },
    { 0xd3e35157, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8963515f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d935163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58435168, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e3516e, 95 },
    { 0xbe63516f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2535174, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b23517b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf3517e, 23 },
    { 0xa63517e, 83 },
    { 0x4ad35180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e35183, 61 },
    { 0x98635184, 43 },
    { 0xd5735185, 52 },
    { 0x77635186, 85 },
    { 0x54835183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e63518a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99435191, 61 },
    { 0xcce35191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc35196, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1535199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa351a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864351a7, 55 },
    { 0xbbb351a8, 93 },
    { 0, 0 },
    { 0x6c7351aa, 61 },
    { 0, 0 },
    { 0xf7e351ac, 98 },
    { 0, 0 },
    { 0x2e8351ae, 90 },
    { 0, 0 },
    { 0x4eb351b0, 66 },
    { 0x503351b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33351b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df351bc, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8351c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de351c8, 34 },
    { 0, 0 },
    { 0xaa8351ca, 98 },
    { 0x80f351ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999351cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5351d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a351df, 76 },
    { 0x3c4351e0, 19 },
    { 0, 0 },
    { 0xd3e351e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17351ea, 95 },
    { 0xa2d351ea, 61 },
    { 0x698351ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a351ef, 61 },
    { 0x209351f0, 90 },
    { 0, 0 },
    { 0x6e5351f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a351f8, 61 },
    { 0xb29351f9, 46 },
    { 0, 0 },
    { 0x325351fb, 99 },
    { 0x1c0351fc, 23 },
    { 0xf8f351fd, 33 },
    { 0xc92351fe, 78 },
    { 0xab8351ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb35203, 61 },
    { 0x76335203, 33 },
    { 0, 0 },
    { 0xbe835206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a35209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3520c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76635210, 23 },
    { 0x75b35211, 90 },
    { 0xdf335212, 90 },
    { 0, 0 },
    { 0xc4c35214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32035218, 64 },
    { 0xf4d35219, 55 },
    { 0x1573521a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c3521d, 23 },
    { 0xf203521e, 98 },
    { 0x6e33521f, 93 },
    { 0, 0 },
    { 0x37f35221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed435224, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1735229, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd3522c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6293522f, 61 },
    { 0xeb235230, 23 },
    { 0x88835231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3835235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd135238, 95 },
    { 0, 0 },
    { 0xdaa3523a, 94 },
    { 0xd663523b, 33 },
    { 0xbef3523c, 23 },
    { 0xbb53523a, 93 },
    { 0xe7c3523a, 23 },
    { 0, 0 },
    { 0xb9935240, 66 },
    { 0, 0 },
    { 0xa5135242, 34 },
    { 0, 0 },
    { 0x89835244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18235247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3524b, 98 },
    { 0, 0 },
    { 0xc9e3524d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d35253, 90 },
    { 0xeb035253, 85 },
    { 0, 0 },
    { 0x6ad35256, 33 },
    { 0, 0 },
    { 0x86c35258, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc3525c, 33 },
    { 0x6f23525d, 43 },
    { 0xb133525c, 76 },
    { 0, 0 },
    { 0xbc235260, 78 },
    { 0x2a35261, 95 },
    { 0x6c235260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf35265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de35268, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3526d, 23 },
    { 0xad63526e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b235271, 95 },
    { 0x2b735272, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca35275, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e435279, 61 },
    { 0, 0 },
    { 0xb9d3527b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3723527f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c35285, 95 },
    { 0, 0 },
    { 0xcc935287, 98 },
    { 0x34a35288, 43 },
    { 0x13335289, 90 },
    { 0x50535289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f35296, 95 },
    { 0, 0 },
    { 0xc7235298, 61 },
    { 0x9e735299, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3423529d, 95 },
    { 0xcc13529e, 90 },
    { 0xf113529f, 76 },
    { 0, 0 },
    { 0xd36352a1, 87 },
    { 0x1d4352a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240352a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437352ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c352b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e352b6, 64 },
    { 0x2a6352b7, 90 },
    { 0, 0 },
    { 0xfb8352b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4352bd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478352c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309352c6, 23 },
    { 0xf8d352c6, 98 },
    { 0, 0 },
    { 0xc07352c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62352cc, 61 },
    { 0, 0 },
    { 0xa01352ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e352d1, 34 },
    { 0xdbb352d2, 34 },
    { 0xdd0352d2, 61 },
    { 0, 0 },
    { 0xf66352d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x198352d8, 98 },
    { 0xa3f352d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c352dc, 52 },
    { 0x9e6352dc, 91 },
    { 0x3c8352dd, 98 },
    { 0, 0 },
    { 0xa31352e0, 90 },
    { 0x535352e1, 83 },
    { 0, 0 },
    { 0xc1d352e3, 78 },
    { 0, 0 },
    { 0x142352e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b352ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b352ef, 90 },
    { 0x4f7352ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8352f3, 95 },
    { 0x211352f3, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa352f9, 90 },
    { 0x6d8352fa, 82 },
    { 0x5cf352fb, 95 },
    { 0x6df352f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a235300, 93 },
    { 0x11c35300, 99 },
    { 0, 0 },
    { 0x3ee35303, 90 },
    { 0x1ac35304, 61 },
    { 0, 0 },
    { 0xa0e35306, 90 },
    { 0xeca35307, 52 },
    { 0x22135307, 95 },
    { 0x6d135308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e3530c, 52 },
    { 0, 0 },
    { 0x1ce3530e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee35316, 95 },
    { 0x86735316, 100 },
    { 0, 0 },
    { 0x91c35319, 33 },
    { 0, 0 },
    { 0x1b83531b, 61 },
    { 0xf123531c, 52 },
    { 0xa573531d, 43 },
    { 0x4dc3531e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9835322, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe535327, 85 },
    { 0x8f035328, 98 },
    { 0x2dc35329, 90 },
    { 0x5035329, 90 },
    { 0x1a13532b, 91 },
    { 0x9a53532c, 66 },
    { 0xa5c3532b, 23 },
    { 0x4363532c, 90 },
    { 0xcda3532f, 33 },
    { 0xc0d35329, 55 },
    { 0x3d635331, 9 },
    { 0, 0 },
    { 0x43035333, 76 },
    { 0xf3235333, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b35338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1153533b, 97 },
    { 0xffe3533c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e35342, 90 },
    { 0x7f535343, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae35348, 95 },
    { 0x1ab35349, 98 },
    { 0x9943534a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7733534d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4535352, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea35358, 61 },
    { 0x42535359, 58 },
    { 0, 0 },
    { 0x1a53535b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe333535e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd35364, 43 },
    { 0x3d335364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e3536b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c435371, 66 },
    { 0x78535372, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a35375, 93 },
    { 0, 0 },
    { 0xdae35377, 51 },
    { 0xeee35378, 78 },
    { 0, 0 },
    { 0xd753537a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d3537f, 61 },
    { 0x5a73537f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab35389, 65 },
    { 0xb9235389, 99 },
    { 0, 0 },
    { 0x6bb3538c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b35390, 91 },
    { 0xac735391, 90 },
    { 0x5f235392, 17 },
    { 0x1fd35392, 23 },
    { 0xf8535394, 37 },
    { 0, 0 },
    { 0x30d35396, 23 },
    { 0x4ec35397, 90 },
    { 0, 0 },
    { 0x6635399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3353a6, 84 },
    { 0x322353a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142353ab, 75 },
    { 0x519353ac, 43 },
    { 0, 0 },
    { 0xf34353ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13353b8, 61 },
    { 0x6c3353b9, 95 },
    { 0xf8b353ba, 80 },
    { 0x9b3353b9, 61 },
    { 0xd5c353b8, 98 },
    { 0x770353bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f353c1, 33 },
    { 0, 0 },
    { 0xf89353c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f353c8, 90 },
    { 0x1c0353c8, 90 },
    { 0xa60353c8, 95 },
    { 0, 0 },
    { 0x1fb353cc, 90 },
    { 0, 0 },
    { 0xca353ce, 55 },
    { 0, 0 },
    { 0xcc3353d0, 95 },
    { 0x7a6353d0, 77 },
    { 0x670353d1, 40 },
    { 0xa92353d0, 90 },
    { 0x8f8353d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56353d7, 98 },
    { 0x4df353d7, 78 },
    { 0x608353d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724353de, 98 },
    { 0x8b2353de, 66 },
    { 0xea6353e0, 98 },
    { 0xd82353e1, 67 },
    { 0x747353e1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291353eb, 17 },
    { 0x840353ec, 76 },
    { 0x198353ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35353f4, 80 },
    { 0, 0 },
    { 0xec6353f6, 46 },
    { 0x79b353f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c353fa, 58 },
    { 0x13c353fb, 23 },
    { 0x7c7353fb, 87 },
    { 0x6353fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2635401, 23 },
    { 0xc6f35402, 84 },
    { 0x81635401, 67 },
    { 0x51635404, 56 },
    { 0xc4d35405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc63540a, 95 },
    { 0xee73540b, 61 },
    { 0x3873540b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3540f, 98 },
    { 0x61235410, 90 },
    { 0x7f835411, 98 },
    { 0xd8f35410, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef35418, 61 },
    { 0xbdc35419, 33 },
    { 0xb483541a, 84 },
    { 0xb143541b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf3541e, 55 },
    { 0, 0 },
    { 0x5b335420, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x97735423, 98 },
    { 0xd5f35424, 98 },
    { 0x12c35425, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa683542a, 33 },
    { 0xdb63542b, 84 },
    { 0x793542b, 90 },
    { 0x7f13542d, 90 },
    { 0xdeb3542e, 34 },
    { 0x3f33542f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca835432, 98 },
    { 0, 0 },
    { 0xae535434, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f35437, 76 },
    { 0x3a135438, 33 },
    { 0, 0 },
    { 0x203543a, 17 },
    { 0xb6d3543b, 33 },
    { 0x58d3543a, 95 },
    { 0x7893543d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70535445, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe603544e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa435453, 64 },
    { 0x9ff35454, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1903545c, 38 },
    { 0x5203545c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2335460, 59 },
    { 0, 0 },
    { 0x47d35462, 66 },
    { 0xcac35463, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba43546e, 50 },
    { 0xa013546e, 90 },
    { 0x5583546f, 76 },
    { 0x2563546f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a35475, 77 },
    { 0x81735476, 99 },
    { 0xde735475, 33 },
    { 0x4a535477, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e3547c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc3547f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7535486, 33 },
    { 0x48435487, 64 },
    { 0x20c35488, 91 },
    { 0xf0435486, 67 },
    { 0x8113548a, 95 },
    { 0x33d3548b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f035495, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a735498, 33 },
    { 0, 0 },
    { 0xc3f3549a, 55 },
    { 0x1d53549a, 33 },
    { 0, 0 },
    { 0xe0e3549d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5354a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d354a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f354ae, 61 },
    { 0x213354af, 61 },
    { 0x7d9354b0, 95 },
    { 0xa58354b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d354b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8354ba, 84 },
    { 0xbea354ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x166354be, 8 },
    { 0x741354bf, 98 },
    { 0xbeb354c0, 91 },
    { 0, 0 },
    { 0xef8354c2, 23 },
    { 0xf31354c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43354c9, 75 },
    { 0x4b354ca, 66 },
    { 0, 0 },
    { 0x7d5354cc, 33 },
    { 0x42a354cc, 95 },
    { 0, 0 },
    { 0x16c354cf, 98 },
    { 0, 0 },
    { 0x787354d1, 98 },
    { 0, 0 },
    { 0xed6354d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852354d7, 46 },
    { 0xf8c354d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9354dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530354e2, 87 },
    { 0x755354e3, 90 },
    { 0x9b7354e4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1354e9, 76 },
    { 0x918354e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5354ee, 87 },
    { 0x5c4354ee, 33 },
    { 0xac6354ef, 99 },
    { 0x24c354ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b354f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3354f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a135507, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b43550b, 98 },
    { 0, 0 },
    { 0x9953550d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e35511, 33 },
    { 0x55c35511, 90 },
    { 0x1d135512, 33 },
    { 0x79035514, 33 },
    { 0x4f135514, 55 },
    { 0x68a35512, 95 },
    { 0, 0 },
    { 0x25e35518, 90 },
    { 0x23c35519, 90 },
    { 0, 0 },
    { 0x72d3551b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72735521, 100 },
    { 0xf6735521, 61 },
    { 0x54f35523, 90 },
    { 0x3ef35524, 78 },
    { 0x27835521, 95 },
    { 0x98e35522, 90 },
    { 0x94c35527, 33 },
    { 0xe9035527, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd653552c, 61 },
    { 0x5493552d, 34 },
    { 0x4053552e, 95 },
    { 0, 0 },
    { 0x45b35530, 34 },
    { 0x7ef35531, 77 },
    { 0xc3b35532, 90 },
    { 0x64c35533, 33 },
    { 0, 0 },
    { 0x87a35535, 76 },
    { 0, 0 },
    { 0x38235537, 90 },
    { 0x82235538, 95 },
    { 0x82d35537, 95 },
    { 0xc663553a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b23553f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd135542, 90 },
    { 0, 0 },
    { 0xfee35544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74935549, 87 },
    { 0x8703554a, 98 },
    { 0x55f3554b, 98 },
    { 0x2443554b, 76 },
    { 0x6023554c, 61 },
    { 0, 0 },
    { 0x5293554f, 66 },
    { 0, 0 },
    { 0x7735551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3555a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab3555d, 90 },
    { 0xaab3555e, 76 },
    { 0xd133555f, 33 },
    { 0, 0 },
    { 0xbfb35561, 34 },
    { 0, 0 },
    { 0xba235563, 43 },
    { 0, 0 },
    { 0x5a935565, 98 },
    { 0, 0 },
    { 0xd735567, 23 },
    { 0xdc435568, 76 },
    { 0x69935569, 43 },
    { 0x6ca3556a, 33 },
    { 0, 0 },
    { 0xf633556c, 83 },
    { 0x1623556d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3835574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8603557c, 33 },
    { 0x213557d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e35580, 61 },
    { 0x77535581, 34 },
    { 0, 0 },
    { 0x2835583, 95 },
    { 0, 0 },
    { 0x7d535585, 61 },
    { 0x65c35586, 76 },
    { 0x66d35586, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8235591, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d35595, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc003559e, 90 },
    { 0xc223559f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451355a5, 88 },
    { 0, 0 },
    { 0x4d7355a7, 61 },
    { 0x4fa355a8, 61 },
    { 0x91c355a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5355af, 44 },
    { 0, 0 },
    { 0xa2e355b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x179355b4, 95 },
    { 0x1dc355b5, 81 },
    { 0x192355b6, 33 },
    { 0x6f6355b6, 67 },
    { 0, 0 },
    { 0x885355b9, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e355be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17355c3, 95 },
    { 0xcdf355c4, 66 },
    { 0x1ca355c5, 93 },
    { 0x1dc355c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562355ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4355d1, 76 },
    { 0xd9355d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e355d8, 100 },
    { 0x49355d8, 33 },
    { 0x2e1355da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1355dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305355e2, 33 },
    { 0, 0 },
    { 0xb79355e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3355e7, 67 },
    { 0x3c0355e8, 76 },
    { 0xaf9355e9, 65 },
    { 0x859355ea, 65 },
    { 0, 0 },
    { 0x65d355ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d355f5, 95 },
    { 0, 0 },
    { 0x7a6355f7, 37 },
    { 0x58f355f8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d355fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9235603, 55 },
    { 0xc7135603, 90 },
    { 0xc9335603, 55 },
    { 0x33e35606, 43 },
    { 0, 0 },
    { 0xe5435608, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb63560c, 33 },
    { 0x1893560d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47935610, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd035614, 66 },
    { 0x12035615, 99 },
    { 0x6d635616, 98 },
    { 0x13735617, 33 },
    { 0x3a635618, 61 },
    { 0x92335617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d43561c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e35622, 91 },
    { 0x3d035623, 33 },
    { 0xd0a35624, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d35629, 76 },
    { 0x31135629, 33 },
    { 0x5113562b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xba03562e, 61 },
    { 0, 0 },
    { 0xc1535630, 95 },
    { 0, 0 },
    { 0xae535632, 60 },
    { 0x73235632, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x14435636, 23 },
    { 0x3f435637, 95 },
    { 0x39e35638, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363563c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d35640, 43 },
    { 0xb8035641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c35647, 84 },
    { 0xe6635648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3263564b, 55 },
    { 0xf1d3564c, 69 },
    { 0, 0 },
    { 0xc1d3564e, 66 },
    { 0x3f23564e, 55 },
    { 0xa7b35650, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9535658, 48 },
    { 0xa2535659, 90 },
    { 0x9f63565a, 33 },
    { 0x3323565b, 43 },
    { 0x9a53565b, 98 },
    { 0x4193565c, 97 },
    { 0xa0a3565e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b35661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5243566a, 98 },
    { 0x37f3566b, 33 },
    { 0xe603566c, 78 },
    { 0x9813566b, 34 },
    { 0xddb3566b, 98 },
    { 0x8593566f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f35673, 61 },
    { 0, 0 },
    { 0xe7435675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc335679, 93 },
    { 0xa373567a, 34 },
    { 0x21c3567b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c35682, 33 },
    { 0, 0 },
    { 0xef635684, 33 },
    { 0, 0 },
    { 0xd9a35686, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3703568a, 95 },
    { 0x7a33568b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae35692, 61 },
    { 0xacb35693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49735696, 90 },
    { 0, 0 },
    { 0x7db35698, 34 },
    { 0, 0 },
    { 0xe3b3569a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4203569e, 33 },
    { 0xc3b3569f, 99 },
    { 0, 0 },
    { 0x42d356a1, 43 },
    { 0xd7356a1, 34 },
    { 0x55e356a3, 46 },
    { 0xa3f356a1, 84 },
    { 0, 0 },
    { 0x9d356a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2356b3, 77 },
    { 0, 0 },
    { 0xe43356b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1356bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f356bf, 95 },
    { 0xea7356c0, 23 },
    { 0x59c356c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c356d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x330356d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b356d8, 90 },
    { 0, 0 },
    { 0xcfb356da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0356dd, 33 },
    { 0xab8356de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee356e1, 34 },
    { 0x6ce356e1, 33 },
    { 0x600356e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd356e7, 34 },
    { 0x268356e7, 2 },
    { 0xbe0356e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a356ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x597356ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78356f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6356ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e35702, 90 },
    { 0x9d035703, 23 },
    { 0xaba35704, 90 },
    { 0, 0 },
    { 0x69c35706, 98 },
    { 0xd5035707, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa3570c, 34 },
    { 0xa643570d, 64 },
    { 0x3753570e, 90 },
    { 0x9f63570e, 88 },
    { 0x8dc3570e, 95 },
    { 0xf6f3570e, 90 },
    { 0x21735711, 90 },
    { 0x5a35713, 84 },
    { 0x39735713, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4535718, 94 },
    { 0xd3a35718, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce73571c, 78 },
    { 0xb143571c, 99 },
    { 0, 0 },
    { 0xdd63571f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9035724, 90 },
    { 0x1ef35725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30435729, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c13572c, 33 },
    { 0xf783572c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae35732, 33 },
    { 0, 0 },
    { 0xaa835734, 55 },
    { 0, 0 },
    { 0xe7535736, 61 },
    { 0x6a035737, 85 },
    { 0x15435738, 77 },
    { 0x81435739, 98 },
    { 0, 0 },
    { 0x25d3573b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc3573f, 90 },
    { 0x66135740, 95 },
    { 0x36935741, 52 },
    { 0xf2a35742, 61 },
    { 0, 0 },
    { 0x2ac35744, 90 },
    { 0x23b35744, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3574a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c3574d, 76 },
    { 0xca13574e, 36 },
    { 0x6953574e, 23 },
    { 0xd723574f, 90 },
    { 0x50c3574f, 76 },
    { 0xecd35752, 85 },
    { 0xcca35753, 43 },
    { 0, 0 },
    { 0x6035755, 78 },
    { 0xd0835756, 33 },
    { 0, 0 },
    { 0x23f35758, 55 },
    { 0, 0 },
    { 0x1163575a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb813575e, 80 },
    { 0x9843575f, 66 },
    { 0x7843575e, 33 },
    { 0x593575f, 55 },
    { 0xa773575f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14135767, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b3576e, 46 },
    { 0xd293576e, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c35772, 90 },
    { 0, 0 },
    { 0xbde35774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37435778, 98 },
    { 0, 0 },
    { 0x61f3577a, 95 },
    { 0x5423577b, 34 },
    { 0x2cb3577a, 43 },
    { 0x9103577d, 43 },
    { 0, 0 },
    { 0x3473577f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x21135782, 33 },
    { 0, 0 },
    { 0xb9a35784, 90 },
    { 0x4da35785, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x32235788, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c3578b, 90 },
    { 0, 0 },
    { 0x2943578d, 66 },
    { 0, 0 },
    { 0x4903578f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc435792, 46 },
    { 0xafa35793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb35799, 55 },
    { 0, 0 },
    { 0x8763579b, 95 },
    { 0xd233579c, 33 },
    { 0, 0 },
    { 0x67a3579e, 98 },
    { 0xfa33579e, 84 },
    { 0x1ca357a0, 33 },
    { 0xdb4357a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f357a5, 38 },
    { 0xcb2357a6, 61 },
    { 0, 0 },
    { 0x48d357a8, 78 },
    { 0x50c357a9, 34 },
    { 0x2ad357aa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf357af, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf357b3, 90 },
    { 0xb66357b3, 61 },
    { 0x785357b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3357ba, 34 },
    { 0, 0 },
    { 0xa0f357bc, 23 },
    { 0, 0 },
    { 0x2f2357be, 76 },
    { 0, 0 },
    { 0x23d357c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b357c6, 76 },
    { 0x36c357c7, 47 },
    { 0x6c357c7, 90 },
    { 0, 0 },
    { 0x351357ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93357cf, 23 },
    { 0x5d3357d0, 98 },
    { 0xa42357d0, 23 },
    { 0xa78357d0, 98 },
    { 0, 0 },
    { 0xf7e357d4, 75 },
    { 0, 0 },
    { 0xac0357d6, 23 },
    { 0x8a2357d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707357db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98357e0, 74 },
    { 0xd7c357e1, 66 },
    { 0x3c4357e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24357e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55357e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f357f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7357f6, 78 },
    { 0x904357f7, 76 },
    { 0, 0 },
    { 0x91a357f9, 43 },
    { 0x8c5357f9, 93 },
    { 0x94c357f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c835800, 61 },
    { 0x82335801, 40 },
    { 0x3d635802, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf35805, 33 },
    { 0x25e35805, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa35812, 95 },
    { 0, 0 },
    { 0xebe35814, 90 },
    { 0xe2135815, 89 },
    { 0x57735816, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce93581a, 90 },
    { 0, 0 },
    { 0x8943581c, 33 },
    { 0x9ef3581d, 76 },
    { 0x4ea3581e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88035823, 33 },
    { 0xdad35824, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74335829, 78 },
    { 0x29e3582a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5113582e, 91 },
    { 0x9603582f, 33 },
    { 0xafd35830, 55 },
    { 0xbbe35830, 90 },
    { 0x16135832, 8 },
    { 0x81135832, 23 },
    { 0xbd935834, 78 },
    { 0x9c335833, 33 },
    { 0x9be3582f, 90 },
    { 0x1435837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc173583b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec83583e, 90 },
    { 0xe8f3583f, 100 },
    { 0x2243583e, 98 },
    { 0xde3583f, 90 },
    { 0xe6b35842, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50735847, 43 },
    { 0xb6735848, 98 },
    { 0, 0 },
    { 0x20a3584a, 91 },
    { 0x2c73584a, 33 },
    { 0xaf3584a, 76 },
    { 0, 0 },
    { 0x753584e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2535851, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16035854, 61 },
    { 0x60435854, 33 },
    { 0, 0 },
    { 0xc1c35857, 53 },
    { 0, 0 },
    { 0xb9535859, 34 },
    { 0xec735859, 93 },
    { 0, 0 },
    { 0xc3e3585c, 33 },
    { 0x5503585c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd735868, 23 },
    { 0, 0 },
    { 0x6353586a, 43 },
    { 0, 0 },
    { 0xfae3586c, 61 },
    { 0xbf23586d, 95 },
    { 0x1d93586d, 61 },
    { 0x60c3586d, 61 },
    { 0, 0 },
    { 0xb6e35871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb35876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a635879, 61 },
    { 0xd7f3587a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74835880, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9035884, 90 },
    { 0x96835884, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3183588f, 76 },
    { 0x63c3588f, 23 },
    { 0xd2535891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed135897, 100 },
    { 0x19335898, 90 },
    { 0x60d35899, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4413589c, 80 },
    { 0, 0 },
    { 0x7643589e, 90 },
    { 0x77f3589f, 55 },
    { 0xb77358a0, 61 },
    { 0x997358a0, 93 },
    { 0x86a358a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x30358a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81358ad, 90 },
    { 0x112358ae, 93 },
    { 0x65a358ad, 85 },
    { 0, 0 },
    { 0xb19358b1, 46 },
    { 0x5fe358b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec358b6, 67 },
    { 0, 0 },
    { 0xdbf358b8, 82 },
    { 0x380358b9, 90 },
    { 0x83c358ba, 33 },
    { 0x9c0358ba, 90 },
    { 0x733358bc, 34 },
    { 0, 0 },
    { 0xb26358be, 93 },
    { 0xc09358bf, 75 },
    { 0, 0 },
    { 0x7cf358c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6358c7, 17 },
    { 0x8af358c7, 76 },
    { 0xae1358c9, 61 },
    { 0x659358c8, 93 },
    { 0xece358cb, 61 },
    { 0x45358c8, 23 },
    { 0, 0 },
    { 0xc43358ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904358d2, 95 },
    { 0x70358d3, 91 },
    { 0x60f358d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4358d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c358da, 43 },
    { 0xa65358db, 67 },
    { 0x540358dc, 34 },
    { 0x28b358dd, 43 },
    { 0x207358dd, 90 },
    { 0, 0 },
    { 0xf21358e0, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x939358e3, 61 },
    { 0x306358e4, 33 },
    { 0x40f358e4, 23 },
    { 0x318358e4, 52 },
    { 0x30f358e4, 90 },
    { 0x6b2358e8, 90 },
    { 0x144358e9, 95 },
    { 0x711358ea, 33 },
    { 0x5a358e8, 81 },
    { 0x161358ec, 33 },
    { 0x9f5358ec, 76 },
    { 0xac7358ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785358f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4358f9, 81 },
    { 0x792358f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1358ff, 95 },
    { 0x421358ff, 90 },
    { 0x11835900, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6635904, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a3590a, 61 },
    { 0, 0 },
    { 0xe3c3590c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8423590f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4535916, 61 },
    { 0x41f35917, 61 },
    { 0xb2335918, 74 },
    { 0x9ba35919, 98 },
    { 0x7df35919, 61 },
    { 0x893591b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28535924, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b35928, 66 },
    { 0xa1d35929, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf3592c, 77 },
    { 0x2ad3592d, 87 },
    { 0x4b03592c, 23 },
    { 0, 0 },
    { 0x62035930, 90 },
    { 0x49435931, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad35936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2435939, 33 },
    { 0, 0 },
    { 0xdc73593b, 61 },
    { 0, 0 },
    { 0xe103593d, 61 },
    { 0, 0 },
    { 0x8643593f, 90 },
    { 0, 0 },
    { 0xf7a35941, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e35945, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5535949, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf043594c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68935952, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c35956, 93 },
    { 0x75b35957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3595d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b35961, 76 },
    { 0x50f35962, 93 },
    { 0xc2835963, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5235967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d83596c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0835974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62635979, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c3597d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd35981, 61 },
    { 0, 0 },
    { 0x49935983, 77 },
    { 0, 0 },
    { 0x6e135985, 61 },
    { 0x92835986, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa135991, 87 },
    { 0, 0 },
    { 0x12c35993, 84 },
    { 0xfd35994, 52 },
    { 0x2ed35995, 95 },
    { 0x81035995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7635999, 87 },
    { 0xa003599a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa283599d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9359a1, 95 },
    { 0xdd8359a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229359a8, 33 },
    { 0x9ea359a8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371359af, 61 },
    { 0x289359af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac359b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa359bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f359bf, 33 },
    { 0x230359c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85359c3, 34 },
    { 0x6ec359c4, 90 },
    { 0x3c3359c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x992359c8, 81 },
    { 0xf7c359c8, 55 },
    { 0xef5359ca, 93 },
    { 0xd50359ca, 98 },
    { 0x666359cc, 81 },
    { 0xb3a359cc, 87 },
    { 0x7aa359ce, 33 },
    { 0, 0 },
    { 0x910359d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7359d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x161359d6, 90 },
    { 0x597359d7, 17 },
    { 0, 0 },
    { 0x29359d9, 91 },
    { 0, 0 },
    { 0xef8359db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f359e0, 98 },
    { 0x64a359e1, 61 },
    { 0xc62359e0, 88 },
    { 0x1e0359e3, 17 },
    { 0xc7359e3, 23 },
    { 0xea2359e4, 99 },
    { 0x424359e6, 61 },
    { 0, 0 },
    { 0xf2c359e8, 34 },
    { 0xfa9359e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc359ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63359f3, 74 },
    { 0xc85359f4, 95 },
    { 0x828359f4, 33 },
    { 0, 0 },
    { 0x26f359f7, 90 },
    { 0, 0 },
    { 0x275359f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8359ff, 34 },
    { 0, 0 },
    { 0xc4635a01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58435a05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e35a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f135a19, 90 },
    { 0xcc435a1a, 78 },
    { 0xfa435a19, 53 },
    { 0x5b635a1c, 100 },
    { 0x34035a1d, 78 },
    { 0x39635a1d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db35a21, 76 },
    { 0x8b935a22, 90 },
    { 0, 0 },
    { 0xd5c35a24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b035a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d335a30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8435a3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42035a3f, 90 },
    { 0x61535a3f, 66 },
    { 0x6a135a41, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d635a47, 81 },
    { 0x1ef35a48, 94 },
    { 0x1bf35a48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd35a4c, 23 },
    { 0xb7835a4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71935a53, 2 },
    { 0x4d835a53, 95 },
    { 0, 0 },
    { 0x7735a56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8335a5d, 98 },
    { 0x9035a5d, 23 },
    { 0xbd135a5f, 90 },
    { 0x94735a60, 95 },
    { 0x24e35a5e, 33 },
    { 0xcea35a62, 95 },
    { 0, 0 },
    { 0x60935a64, 95 },
    { 0x4c435a64, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea35a68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34735a76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c35a79, 46 },
    { 0xb6b35a7a, 93 },
    { 0xada35a7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd35a7e, 23 },
    { 0, 0 },
    { 0xefb35a80, 34 },
    { 0, 0 },
    { 0xbd035a82, 90 },
    { 0x80b35a83, 66 },
    { 0xca835a84, 33 },
    { 0x77d35a84, 99 },
    { 0x78a35a85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9635a8a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x83735a8d, 61 },
    { 0xebc35a8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f35a91, 98 },
    { 0, 0 },
    { 0xe135a93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9535a97, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb635a9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1035aa1, 90 },
    { 0, 0 },
    { 0x57d35aa3, 76 },
    { 0x50335aa4, 90 },
    { 0, 0 },
    { 0x9d635aa6, 55 },
    { 0x40335aa7, 95 },
    { 0x9b935aa7, 55 },
    { 0xf8d35aa9, 23 },
    { 0x69935aa8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d335ab0, 95 },
    { 0, 0 },
    { 0x52f35ab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8835ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb35abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe35ac4, 98 },
    { 0x7bc35ac4, 65 },
    { 0xa5935ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f135ac9, 93 },
    { 0x84235ac9, 33 },
    { 0xced35aca, 95 },
    { 0x16435ac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc35ad4, 56 },
    { 0x2fb35ad4, 23 },
    { 0xc7035ad6, 98 },
    { 0, 0 },
    { 0xdb635ad8, 23 },
    { 0x50e35ad8, 55 },
    { 0xa0835ada, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b35add, 46 },
    { 0xacb35add, 66 },
    { 0xddf35ade, 76 },
    { 0xd2535ae0, 95 },
    { 0xe3135ae1, 93 },
    { 0xb5635ae2, 85 },
    { 0xb1c35ade, 76 },
    { 0xd7d35ae4, 23 },
    { 0x84235ae1, 66 },
    { 0x49135ade, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x60835ae9, 84 },
    { 0xee735aea, 84 },
    { 0, 0 },
    { 0x8d235aec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b35af0, 33 },
    { 0x14835af1, 33 },
    { 0x2f135af1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d35af6, 61 },
    { 0x76b35af6, 61 },
    { 0x2e435af8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c35afc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb35aff, 69 },
    { 0, 0 },
    { 0x8bd35b01, 55 },
    { 0xebe35b01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xda335b05, 23 },
    { 0x7d235b06, 23 },
    { 0xdd835b07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc535b0b, 90 },
    { 0, 0 },
    { 0x3ef35b0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94135b12, 78 },
    { 0xbb435b13, 76 },
    { 0x53c35b13, 23 },
    { 0x5ac35b13, 46 },
    { 0x42935b16, 87 },
    { 0xd6c35b16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28135b1b, 80 },
    { 0, 0 },
    { 0xb6235b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d035b23, 61 },
    { 0x3d835b24, 90 },
    { 0xc7735b23, 23 },
    { 0, 0 },
    { 0xa3e35b27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf935b2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d835b2e, 33 },
    { 0x30a35b2e, 87 },
    { 0x87c35b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba735b37, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x96535b3a, 61 },
    { 0x7e135b3a, 43 },
    { 0, 0 },
    { 0xfc135b3d, 23 },
    { 0, 0 },
    { 0x36e35b3f, 55 },
    { 0xf1f35b3f, 33 },
    { 0, 0 },
    { 0x75835b42, 61 },
    { 0x4535b43, 33 },
    { 0xf8035b44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d35b48, 33 },
    { 0xc3435b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa35b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3235b52, 98 },
    { 0x5e635b53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac35b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb335b5c, 78 },
    { 0xec035b5c, 95 },
    { 0x50135b5e, 43 },
    { 0x6c835b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28835b65, 90 },
    { 0, 0 },
    { 0x55235b67, 77 },
    { 0xa9235b68, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a35b6c, 34 },
    { 0, 0 },
    { 0x7dc35b6e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a35b77, 90 },
    { 0, 0 },
    { 0xd0635b79, 33 },
    { 0x9e835b79, 67 },
    { 0xf2435b79, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b35b7f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b35b83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d035b87, 76 },
    { 0, 0 },
    { 0xad235b89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed735b8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83135b93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b35b9d, 95 },
    { 0, 0 },
    { 0xb1d35b9f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f35ba6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b35ba9, 84 },
    { 0x4dd35ba9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd35bb2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e35bb5, 95 },
    { 0xf2c35bb6, 23 },
    { 0x79f35bb7, 43 },
    { 0x85a35bb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9835bc1, 87 },
    { 0x10b35bc1, 61 },
    { 0x81f35bc3, 33 },
    { 0, 0 },
    { 0x56435bc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad535bcb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c35bd0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b35bd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb635bd6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55235bdf, 90 },
    { 0x88735bdf, 55 },
    { 0x5e135be0, 84 },
    { 0x85f35bdf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6235bea, 33 },
    { 0x5ac35beb, 76 },
    { 0x77735beb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e935bf5, 33 },
    { 0x47a35bf6, 23 },
    { 0, 0 },
    { 0x71a35bf8, 61 },
    { 0, 0 },
    { 0x23a35bfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9635bfe, 55 },
    { 0, 0 },
    { 0x48935c00, 90 },
    { 0, 0 },
    { 0x7c935c02, 84 },
    { 0xe4c35c03, 76 },
    { 0xf635c02, 23 },
    { 0x1b335c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c35c09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82635c0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x25635c12, 77 },
    { 0x4e535c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf535c17, 46 },
    { 0x12135c18, 23 },
    { 0, 0 },
    { 0x3de35c1a, 23 },
    { 0, 0 },
    { 0xc035c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e135c20, 23 },
    { 0, 0 },
    { 0x5e335c22, 33 },
    { 0x2a635c23, 76 },
    { 0x56235c24, 61 },
    { 0xe3235c25, 46 },
    { 0xc435c25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7135c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44135c2d, 90 },
    { 0x65735c2e, 95 },
    { 0xa2035c2d, 34 },
    { 0xa3735c2d, 52 },
    { 0x33f35c31, 66 },
    { 0xde035c32, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce35c35, 78 },
    { 0x5b435c36, 34 },
    { 0, 0 },
    { 0x17e35c38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa35c3c, 34 },
    { 0xd8f35c3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f35c41, 33 },
    { 0x1df35c42, 77 },
    { 0, 0 },
    { 0x38335c44, 61 },
    { 0xc8a35c45, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff35c52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e35c55, 33 },
    { 0x90635c56, 99 },
    { 0x9835c57, 61 },
    { 0x35435c58, 95 },
    { 0, 0 },
    { 0xc4635c5a, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38835c61, 90 },
    { 0x21c35c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e35c6e, 90 },
    { 0x3ec35c6f, 90 },
    { 0x63435c70, 40 },
    { 0xbde35c70, 76 },
    { 0xeef35c72, 17 },
    { 0, 0 },
    { 0x50835c74, 100 },
    { 0xdce35c75, 65 },
    { 0x63535c75, 34 },
    { 0, 0 },
    { 0x4e135c78, 23 },
    { 0x18335c79, 87 },
    { 0, 0 },
    { 0x86f35c7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f835c7e, 46 },
    { 0x9235c7f, 67 },
    { 0, 0 },
    { 0x74c35c81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced35c8a, 61 },
    { 0x3be35c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f35c8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a35c91, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a35c95, 66 },
    { 0x96535c96, 43 },
    { 0x8e935c97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c35c9a, 34 },
    { 0xf0d35c9b, 90 },
    { 0x3a135c9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a35ca3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1235ca6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c35ca9, 76 },
    { 0, 0 },
    { 0x8b235cab, 33 },
    { 0, 0 },
    { 0xa635cad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c335cb2, 78 },
    { 0, 0 },
    { 0x12235cb4, 91 },
    { 0x25635cb5, 95 },
    { 0, 0 },
    { 0x51835cb7, 34 },
    { 0xf6a35cb8, 76 },
    { 0x94035cb9, 33 },
    { 0x5a435cb9, 88 },
    { 0x8ba35cbb, 88 },
    { 0x18435cbc, 93 },
    { 0, 0 },
    { 0x8fe35cbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a35cc4, 91 },
    { 0xc7c35cc4, 33 },
    { 0xbc335cc5, 95 },
    { 0xf7935cc7, 95 },
    { 0x36d35cc8, 66 },
    { 0x7635cc9, 33 },
    { 0x12f35cc5, 43 },
    { 0xe0e35cc6, 95 },
    { 0x7c835ccc, 34 },
    { 0xa8e35ccd, 17 },
    { 0x96f35cce, 76 },
    { 0x69735cc5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b235cd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c35cd8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5135cdc, 98 },
    { 0xf7a35cdd, 95 },
    { 0x35935cde, 87 },
    { 0x65f35cde, 67 },
    { 0x2e535cdd, 78 },
    { 0xa8f35cdd, 33 },
    { 0xae835ce2, 90 },
    { 0x72535ce3, 99 },
    { 0, 0 },
    { 0xa8435ce5, 95 },
    { 0x9ed35ce6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd35ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a35cf0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6635cf5, 93 },
    { 0, 0 },
    { 0xbbc35cf7, 33 },
    { 0xa235cf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f35cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea35d03, 98 },
    { 0x3f935d04, 61 },
    { 0xf3f35d05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e35d08, 33 },
    { 0x66f35d08, 43 },
    { 0, 0 },
    { 0x63d35d0b, 43 },
    { 0, 0 },
    { 0xf5d35d0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a35d11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c035d14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44435d18, 61 },
    { 0xf4a35d18, 98 },
    { 0x29f35d18, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57735d22, 65 },
    { 0, 0 },
    { 0x20135d24, 95 },
    { 0x26635d25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef35d28, 76 },
    { 0x6ab35d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96335d2e, 34 },
    { 0x1f635d2e, 91 },
    { 0x81c35d2e, 91 },
    { 0, 0 },
    { 0xc8b35d32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e35d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f35d3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e35d42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa35d45, 100 },
    { 0xcd035d46, 23 },
    { 0x23835d47, 43 },
    { 0, 0 },
    { 0xe9835d49, 90 },
    { 0x92135d49, 95 },
    { 0x9a435d49, 84 },
    { 0, 0 },
    { 0x9bf35d4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a35d52, 95 },
    { 0xbe535d53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61335d57, 76 },
    { 0xd9e35d57, 93 },
    { 0x1a435d57, 90 },
    { 0x50a35d58, 88 },
    { 0, 0 },
    { 0x72135d5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x62335d5f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61335d69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc35d6c, 76 },
    { 0x9ba35d6d, 34 },
    { 0xa8135d6c, 61 },
    { 0xb3535d6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76135d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2835d77, 90 },
    { 0xf9935d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34735d80, 33 },
    { 0xb8335d81, 55 },
    { 0x51135d82, 76 },
    { 0x3b035d82, 100 },
    { 0, 0 },
    { 0x6e635d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b35d89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a35d8e, 98 },
    { 0, 0 },
    { 0xcee35d90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab35d93, 90 },
    { 0xb7f35d93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd35d97, 98 },
    { 0x60935d97, 90 },
    { 0xf8035d99, 98 },
    { 0x6a435d9a, 68 },
    { 0x90d35d9a, 23 },
    { 0, 0 },
    { 0x2ee35d9d, 95 },
    { 0xdf635d9e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f35da5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0035dab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62935daf, 99 },
    { 0xcf635daf, 98 },
    { 0x43d35db0, 90 },
    { 0xcb35db2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da35db5, 92 },
    { 0x43435db5, 90 },
    { 0xe1f35db5, 95 },
    { 0, 0 },
    { 0x14235db9, 90 },
    { 0x6a035dba, 33 },
    { 0x8a235dba, 90 },
    { 0x5f535dbc, 75 },
    { 0xb9535dbd, 68 },
    { 0x8eb35dbe, 78 },
    { 0xb2c35dbf, 55 },
    { 0, 0 },
    { 0x1aa35dc1, 52 },
    { 0x29835dc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9035dc5, 95 },
    { 0xb135dc6, 33 },
    { 0xf1735dc5, 33 },
    { 0, 0 },
    { 0x7ec35dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52735dcc, 34 },
    { 0xd7035dcd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435dd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92735dd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf35ddb, 33 },
    { 0x43135ddc, 33 },
    { 0, 0 },
    { 0xc0335dde, 33 },
    { 0x39735ddf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48635de2, 23 },
    { 0xa9635de2, 95 },
    { 0xe8235de4, 67 },
    { 0xad335de2, 95 },
    { 0, 0 },
    { 0x7b435de7, 87 },
    { 0x4b735de8, 84 },
    { 0x90835de9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c035df2, 68 },
    { 0x51535df3, 67 },
    { 0x13135df2, 61 },
    { 0x64a35df5, 33 },
    { 0x5b935df6, 98 },
    { 0x16e35df7, 94 },
    { 0x55235df7, 33 },
    { 0xc7035df8, 90 },
    { 0x8c635dfa, 78 },
    { 0x78235dfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc35e00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f35e06, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x34135e09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd35e10, 55 },
    { 0x55535e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa35e14, 90 },
    { 0xa6d35e15, 92 },
    { 0x4bd35e15, 33 },
    { 0, 0 },
    { 0xd1b35e18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd235e1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c735e23, 61 },
    { 0x1d835e23, 83 },
    { 0xab935e25, 98 },
    { 0xcd735e23, 23 },
    { 0x51335e24, 81 },
    { 0x98235e25, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf35e30, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f135e38, 66 },
    { 0x30a35e39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd335e3c, 33 },
    { 0, 0 },
    { 0x49835e3e, 33 },
    { 0x5b935e3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a035e49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f035e4c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed35e50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8135e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88735e5a, 33 },
    { 0x49c35e5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1535e5e, 95 },
    { 0, 0 },
    { 0x95d35e60, 90 },
    { 0, 0 },
    { 0xe6935e62, 82 },
    { 0xf5f35e63, 95 },
    { 0xd1035e64, 98 },
    { 0x47535e65, 90 },
    { 0x98435e63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13535e6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9335e6f, 52 },
    { 0x4e35e70, 43 },
    { 0xed035e6f, 61 },
    { 0xd8835e72, 61 },
    { 0x53335e71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f035e76, 90 },
    { 0, 0 },
    { 0x26735e78, 55 },
    { 0x5d035e78, 61 },
    { 0x85c35e79, 61 },
    { 0x98735e7b, 90 },
    { 0xe0f35e7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c35e83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b135e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d35e8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f35e91, 33 },
    { 0, 0 },
    { 0x8b35e93, 55 },
    { 0x16a35e94, 99 },
    { 0, 0 },
    { 0x9d135e96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7635e9e, 52 },
    { 0x6c535e9f, 17 },
    { 0x39535ea0, 81 },
    { 0x63335ea1, 85 },
    { 0x13a35ea2, 77 },
    { 0xf2135e9e, 34 },
    { 0x20b35ea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0435ea7, 98 },
    { 0xf6f35ea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2435eab, 46 },
    { 0xca735eac, 99 },
    { 0x53b35ead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65235eb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd35eb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e35eb7, 87 },
    { 0, 0 },
    { 0x5db35eb9, 95 },
    { 0xd9a35eba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd435ebe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435ec1, 97 },
    { 0xfdf35ec2, 61 },
    { 0, 0 },
    { 0xe6335ec4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf35ec9, 33 },
    { 0x62835ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64635ecd, 55 },
    { 0, 0 },
    { 0xb2635ecf, 33 },
    { 0xf635ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93535ed3, 33 },
    { 0x1e235ed4, 98 },
    { 0x7b835ed5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e535ed9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea35ede, 93 },
    { 0xd9335edf, 95 },
    { 0x76835ede, 95 },
    { 0, 0 },
    { 0x75a35ee2, 89 },
    { 0x41135ee2, 87 },
    { 0x21a35ee4, 61 },
    { 0x65b35ee3, 66 },
    { 0x4bb35ee4, 93 },
    { 0xfc635ee7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b235ef1, 34 },
    { 0x1dd35ef1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45035ef9, 98 },
    { 0x1bc35ef9, 85 },
    { 0xf4035efb, 61 },
    { 0x15635efc, 90 },
    { 0x9aa35efd, 98 },
    { 0xd4935efd, 93 },
    { 0x6a635efd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab035f03, 91 },
    { 0x7e435f04, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8535f08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d35f0b, 23 },
    { 0, 0 },
    { 0xe2c35f0d, 95 },
    { 0x3eb35f0e, 76 },
    { 0xbcf35f0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d335f12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3735f15, 34 },
    { 0x21c35f15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b35f19, 92 },
    { 0x69835f19, 98 },
    { 0xe8535f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97035f1e, 90 },
    { 0x6a835f1f, 99 },
    { 0x335f20, 98 },
    { 0x95535f21, 78 },
    { 0xe8b35f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5835f25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3235f2a, 61 },
    { 0xef535f2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd535f2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1335f32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48735f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc535f3c, 90 },
    { 0xf6335f3d, 95 },
    { 0, 0 },
    { 0x51435f3f, 65 },
    { 0, 0 },
    { 0x54735f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b35f49, 61 },
    { 0xf4235f49, 61 },
    { 0, 0 },
    { 0x51e35f4c, 99 },
    { 0x27735f4d, 33 },
    { 0, 0 },
    { 0xcf335f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71635f53, 23 },
    { 0x2fd35f53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9035f58, 98 },
    { 0xc35f59, 98 },
    { 0x9f935f58, 61 },
    { 0x51f35f5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9235f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa235f64, 34 },
    { 0xd5135f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf35f69, 91 },
    { 0x43635f6a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a35f73, 61 },
    { 0x80c35f73, 80 },
    { 0x5f735f73, 64 },
    { 0, 0 },
    { 0x5a235f77, 68 },
    { 0x3ba35f78, 23 },
    { 0xa2235f79, 34 },
    { 0x93435f77, 34 },
    { 0, 0 },
    { 0x2a035f7c, 98 },
    { 0xafe35f7d, 90 },
    { 0x45e35f7e, 77 },
    { 0xdd335f7f, 90 },
    { 0x83335f80, 93 },
    { 0x9dc35f81, 90 },
    { 0, 0 },
    { 0x6b535f83, 84 },
    { 0x1235f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e35f87, 40 },
    { 0x98e35f87, 76 },
    { 0x39535f87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70135f8f, 33 },
    { 0xfac35f8f, 34 },
    { 0x46035f91, 90 },
    { 0x6cf35f92, 95 },
    { 0x48935f92, 98 },
    { 0x54135f94, 61 },
    { 0, 0 },
    { 0xf1135f96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b835f9a, 95 },
    { 0xd1f35f9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82835fa0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a035fa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b35fac, 95 },
    { 0x5fa35fad, 61 },
    { 0, 0 },
    { 0x25135faf, 95 },
    { 0x34c35fb0, 87 },
    { 0, 0 },
    { 0xfc635fb2, 61 },
    { 0xcfc35fb2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2335fb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40735fbb, 65 },
    { 0x74e35fbb, 33 },
    { 0, 0 },
    { 0x4c535fbe, 87 },
    { 0xd2735fbf, 23 },
    { 0xa2135fc0, 5 },
    { 0, 0 },
    { 0x4b735fc2, 23 },
    { 0x8cd35fc3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a35fc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c035fce, 64 },
    { 0x23235fcf, 33 },
    { 0x3c435fd0, 61 },
    { 0xe2f35fd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35835fd4, 23 },
    { 0x97835fd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38635fdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd35fde, 76 },
    { 0x7cf35fde, 95 },
    { 0xbf135fe0, 33 },
    { 0, 0 },
    { 0x4d535fe2, 91 },
    { 0, 0 },
    { 0x26035fe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53535fee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b935ff1, 83 },
    { 0x6fe35ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b35ff8, 33 },
    { 0x87e35ff8, 43 },
    { 0, 0 },
    { 0xad435ffb, 99 },
    { 0xf2b35ffc, 40 },
    { 0xf135ffd, 52 },
    { 0xd6435ffe, 90 },
    { 0x3f535fff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5036004, 59 },
    { 0xb5736005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec36008, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b3600c, 91 },
    { 0, 0 },
    { 0x4223600e, 76 },
    { 0xbb73600f, 87 },
    { 0x3ef36010, 66 },
    { 0x21c3600e, 34 },
    { 0x2a736012, 33 },
    { 0xd8c36012, 77 },
    { 0xa6536011, 84 },
    { 0, 0 },
    { 0x51536016, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9036019, 61 },
    { 0, 0 },
    { 0x2f93601b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf136021, 46 },
    { 0x58236022, 64 },
    { 0xd0d36021, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8493602a, 46 },
    { 0x2c53602b, 87 },
    { 0x4253602a, 95 },
    { 0x5793602d, 33 },
    { 0, 0 },
    { 0x48c3602f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b036035, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f3603a, 43 },
    { 0xa5b3603b, 52 },
    { 0xe8a3603b, 99 },
    { 0, 0 },
    { 0xa583603e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e36041, 43 },
    { 0x6c836041, 90 },
    { 0, 0 },
    { 0x33936044, 95 },
    { 0, 0 },
    { 0x30a36046, 77 },
    { 0, 0 },
    { 0x34736048, 43 },
    { 0xc6c36049, 78 },
    { 0xc0936049, 33 },
    { 0, 0 },
    { 0x6943604c, 34 },
    { 0xe4c3604d, 95 },
    { 0, 0 },
    { 0x6813604f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b36052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f36055, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5936059, 38 },
    { 0, 0 },
    { 0xcee3605b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d236062, 95 },
    { 0x48736063, 90 },
    { 0, 0 },
    { 0x4bb36065, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69336069, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e53606d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd36070, 33 },
    { 0x4c336071, 23 },
    { 0xc3b36072, 33 },
    { 0, 0 },
    { 0x40836074, 33 },
    { 0xfc236074, 33 },
    { 0x47236075, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3607b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b3607f, 84 },
    { 0x99136080, 17 },
    { 0x3da3607f, 33 },
    { 0x82a36080, 23 },
    { 0xb4536083, 67 },
    { 0, 0 },
    { 0x2ab36085, 84 },
    { 0xe2a36086, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb136093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc36096, 43 },
    { 0x76e36097, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999360a0, 61 },
    { 0xd1e360a1, 100 },
    { 0xb57360a2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e360a9, 81 },
    { 0, 0 },
    { 0x2de360ab, 55 },
    { 0, 0 },
    { 0xeb1360ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837360b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x491360b9, 46 },
    { 0xec360ba, 46 },
    { 0xc34360bb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506360bf, 78 },
    { 0x2bf360c0, 95 },
    { 0x18d360c1, 85 },
    { 0, 0 },
    { 0x810360c3, 90 },
    { 0, 0 },
    { 0xdf1360c5, 77 },
    { 0, 0 },
    { 0xd4c360c7, 46 },
    { 0x2c7360c7, 17 },
    { 0xa25360c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x828360cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c360d4, 55 },
    { 0, 0 },
    { 0xf0d360d6, 33 },
    { 0, 0 },
    { 0xaf360d8, 95 },
    { 0x1c3360d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b360dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c360e2, 61 },
    { 0, 0 },
    { 0x223360e4, 55 },
    { 0xf2c360e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x259360e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2360ef, 90 },
    { 0xfe3360f0, 61 },
    { 0, 0 },
    { 0xe58360f2, 76 },
    { 0x634360f3, 75 },
    { 0, 0 },
    { 0x956360f5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98360f9, 61 },
    { 0x20a360f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8360fd, 55 },
    { 0xe9a360fe, 98 },
    { 0, 0 },
    { 0xdac36100, 98 },
    { 0xf3e36101, 34 },
    { 0xc4336101, 76 },
    { 0x1b036103, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x98636106, 33 },
    { 0x1c036107, 55 },
    { 0, 0 },
    { 0x26f36109, 77 },
    { 0x27b3610a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee36110, 55 },
    { 0x50136110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8636114, 34 },
    { 0, 0 },
    { 0x4c436116, 68 },
    { 0x8a136116, 61 },
    { 0, 0 },
    { 0xbe636119, 95 },
    { 0, 0 },
    { 0xb1e3611b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d03611e, 88 },
    { 0xbdd3611f, 80 },
    { 0xc6b36120, 43 },
    { 0xb103611f, 33 },
    { 0xcfd36122, 90 },
    { 0x39936121, 91 },
    { 0x103611f, 61 },
    { 0x54436120, 99 },
    { 0x9d336123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab93612b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb53612f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34236132, 90 },
    { 0, 0 },
    { 0xfa836134, 95 },
    { 0, 0 },
    { 0xbc436136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad33613b, 61 },
    { 0xf763613b, 100 },
    { 0x9c73613c, 99 },
    { 0, 0 },
    { 0xbbc3613f, 24 },
    { 0xdaf36140, 76 },
    { 0x93036140, 98 },
    { 0xb6e36141, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9213614e, 90 },
    { 0, 0 },
    { 0xbb636150, 84 },
    { 0, 0 },
    { 0x23a36152, 33 },
    { 0, 0 },
    { 0xba436154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3615b, 55 },
    { 0x3e73615c, 93 },
    { 0x5f53615d, 100 },
    { 0x5b83615e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7036161, 55 },
    { 0, 0 },
    { 0x4cf36163, 34 },
    { 0xcbb36163, 23 },
    { 0x11036164, 77 },
    { 0xcff36166, 90 },
    { 0x53b36164, 90 },
    { 0x92b36164, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf663616d, 98 },
    { 0xd7c3616d, 52 },
    { 0x3333616f, 61 },
    { 0xf3036170, 78 },
    { 0xcde36171, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac36174, 23 },
    { 0, 0 },
    { 0x5e136176, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd3617b, 40 },
    { 0xaf63617c, 33 },
    { 0, 0 },
    { 0x5f93617e, 33 },
    { 0, 0 },
    { 0x9bd36180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e36183, 61 },
    { 0, 0 },
    { 0xddf36185, 23 },
    { 0xc9b36186, 33 },
    { 0x73f36187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4013618a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1013618e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd36191, 70 },
    { 0x8da36192, 33 },
    { 0xda336191, 67 },
    { 0x58936194, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c3619d, 46 },
    { 0x8a73619d, 55 },
    { 0xfe53619f, 99 },
    { 0x31c3619f, 95 },
    { 0xed13619e, 43 },
    { 0x95a361a2, 90 },
    { 0, 0 },
    { 0xb10361a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08361ab, 61 },
    { 0x8cb361ac, 68 },
    { 0xade361ad, 61 },
    { 0x14361ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43361b1, 52 },
    { 0xb08361b2, 76 },
    { 0x655361b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x946361b6, 76 },
    { 0x833361b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3361bb, 33 },
    { 0, 0 },
    { 0x7dc361bd, 61 },
    { 0x762361be, 95 },
    { 0, 0 },
    { 0xee1361c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3361c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89361cc, 27 },
    { 0, 0 },
    { 0x13b361ce, 90 },
    { 0, 0 },
    { 0xbcf361d0, 95 },
    { 0xb0b361d1, 77 },
    { 0x454361d2, 90 },
    { 0x299361d1, 34 },
    { 0, 0 },
    { 0x5da361d5, 46 },
    { 0, 0 },
    { 0x525361d7, 23 },
    { 0xae7361d8, 90 },
    { 0x71b361d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d361dd, 61 },
    { 0x911361de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1361e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0361e5, 100 },
    { 0, 0 },
    { 0x8af361e7, 95 },
    { 0x859361e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28361f4, 90 },
    { 0xd4e361f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113361f9, 95 },
    { 0xcd8361fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a36206, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e43620d, 90 },
    { 0, 0 },
    { 0x9993620f, 98 },
    { 0, 0 },
    { 0xbbb36211, 95 },
    { 0xf3436212, 23 },
    { 0x9a336212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c36216, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1013621d, 77 },
    { 0, 0 },
    { 0xe773621f, 61 },
    { 0x26936220, 61 },
    { 0xc5836221, 77 },
    { 0x37d36221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d3622c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8933622f, 95 },
    { 0, 0 },
    { 0xdd836231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82236236, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd36239, 23 },
    { 0, 0 },
    { 0xbfb3623b, 77 },
    { 0, 0 },
    { 0xd753623d, 90 },
    { 0, 0 },
    { 0x5f13623f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78836242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b36246, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b3624b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde36251, 40 },
    { 0x61136252, 76 },
    { 0x89136253, 55 },
    { 0x8cd36251, 34 },
    { 0x24d36255, 91 },
    { 0, 0 },
    { 0x7f336257, 61 },
    { 0xca536258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e3625c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5636261, 90 },
    { 0x26d36261, 98 },
    { 0xd4736263, 49 },
    { 0, 0 },
    { 0x6df36265, 99 },
    { 0x4f536265, 33 },
    { 0xf5836265, 87 },
    { 0x31636268, 23 },
    { 0x30536269, 78 },
    { 0xc5436266, 43 },
    { 0, 0 },
    { 0xc9f3626c, 92 },
    { 0x313626c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c36272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4336275, 76 },
    { 0, 0 },
    { 0x73536277, 95 },
    { 0x89936277, 90 },
    { 0x5d36279, 33 },
    { 0xf8d36279, 90 },
    { 0xce23627b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef336286, 91 },
    { 0xf4136286, 56 },
    { 0x3036288, 40 },
    { 0x9e236289, 95 },
    { 0xafa36289, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac3628d, 93 },
    { 0, 0 },
    { 0xfdd3628f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d236294, 33 },
    { 0, 0 },
    { 0x86836296, 23 },
    { 0x61636297, 33 },
    { 0xa1536296, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3603629b, 90 },
    { 0, 0 },
    { 0x6a73629d, 98 },
    { 0x3d3629e, 90 },
    { 0xd1f3629e, 98 },
    { 0xe85362a0, 33 },
    { 0x3be362a0, 52 },
    { 0xe2362a0, 34 },
    { 0x8f5362a3, 95 },
    { 0xab6362a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca362a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d362b2, 33 },
    { 0xb20362b2, 98 },
    { 0xe91362b3, 66 },
    { 0x15a362b5, 34 },
    { 0x510362b2, 38 },
    { 0x4a0362b5, 90 },
    { 0, 0 },
    { 0x608362b9, 56 },
    { 0xee8362b9, 95 },
    { 0x4f9362bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09362be, 23 },
    { 0, 0 },
    { 0xc22362c0, 100 },
    { 0, 0 },
    { 0x5c2362c2, 88 },
    { 0, 0 },
    { 0x98f362c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32362c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22362ca, 33 },
    { 0xd16362cb, 81 },
    { 0xe25362cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x716362cf, 95 },
    { 0, 0 },
    { 0x18c362d1, 23 },
    { 0xd9362d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33362d9, 52 },
    { 0, 0 },
    { 0xe17362db, 53 },
    { 0, 0 },
    { 0x6a362dd, 78 },
    { 0, 0 },
    { 0x5a7362df, 61 },
    { 0, 0 },
    { 0xfef362e1, 46 },
    { 0xc2362e2, 91 },
    { 0x723362e3, 33 },
    { 0x65a362e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e362ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7362ef, 3 },
    { 0, 0 },
    { 0xb14362f1, 88 },
    { 0x294362f1, 78 },
    { 0xbb1362f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe362f6, 61 },
    { 0x8b3362f7, 34 },
    { 0x234362f7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc362fc, 61 },
    { 0, 0 },
    { 0xb65362fe, 95 },
    { 0x91f362ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57636302, 33 },
    { 0xb4936303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d3630b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f13630e, 85 },
    { 0x87b3630f, 98 },
    { 0xb343630e, 31 },
    { 0xcb536311, 95 },
    { 0xf623630f, 33 },
    { 0, 0 },
    { 0xba636314, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd253631c, 95 },
    { 0xd9f3631d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec36321, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97436328, 88 },
    { 0x9cf36329, 90 },
    { 0x1c63632a, 61 },
    { 0x44036328, 61 },
    { 0x93c3632a, 75 },
    { 0, 0 },
    { 0xebe3632e, 66 },
    { 0x52f3632e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3936335, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e36340, 43 },
    { 0x12a36340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0536344, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3433634e, 23 },
    { 0x7df3634f, 46 },
    { 0x73d3634e, 99 },
    { 0, 0 },
    { 0x72336352, 90 },
    { 0xf136353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9136360, 90 },
    { 0, 0 },
    { 0x2e736362, 84 },
    { 0x90c36363, 33 },
    { 0x2f436364, 90 },
    { 0xa6236362, 33 },
    { 0, 0 },
    { 0xa0636367, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c3636a, 33 },
    { 0xf413636b, 34 },
    { 0xb463636c, 90 },
    { 0x49d3636c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9636371, 98 },
    { 0, 0 },
    { 0x5b36373, 90 },
    { 0x5c536374, 52 },
    { 0x54f36375, 55 },
    { 0x73536375, 33 },
    { 0xcfb36377, 61 },
    { 0x1be36374, 87 },
    { 0x35336378, 92 },
    { 0xecb36373, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903637f, 61 },
    { 0xcb436380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65936389, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7203638f, 98 },
    { 0x71b36390, 61 },
    { 0xb4d36390, 61 },
    { 0x9a536390, 55 },
    { 0, 0 },
    { 0xf6036394, 68 },
    { 0x31336394, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a3639a, 95 },
    { 0x6583639b, 76 },
    { 0x8c93639c, 77 },
    { 0x7fe3639d, 90 },
    { 0x8053639e, 34 },
    { 0, 0 },
    { 0xa01363a0, 95 },
    { 0x68363a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4363a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b363a9, 90 },
    { 0xd48363aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11363b0, 68 },
    { 0xb8363b1, 87 },
    { 0, 0 },
    { 0x442363b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3363b6, 61 },
    { 0x7a7363b6, 8 },
    { 0x989363b6, 95 },
    { 0, 0 },
    { 0xb56363ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed363bf, 33 },
    { 0xcf9363bf, 80 },
    { 0x664363c1, 98 },
    { 0, 0 },
    { 0x1e3363c3, 68 },
    { 0xa7a363c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1363c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8363d0, 23 },
    { 0x675363d1, 90 },
    { 0, 0 },
    { 0xda363d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x156363d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875363e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b363e3, 98 },
    { 0xa9f363e4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745363ea, 93 },
    { 0x4ea363eb, 33 },
    { 0x13c363eb, 46 },
    { 0xce4363eb, 95 },
    { 0x31a363ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9363f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff363f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9363fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x590363ff, 93 },
    { 0x6d2363ff, 90 },
    { 0xa9a36401, 93 },
    { 0xd1936402, 61 },
    { 0xfc436401, 61 },
    { 0x27636402, 76 },
    { 0xc4b36405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b636418, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2353641b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8013641f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x39836422, 23 },
    { 0xf7136423, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e36426, 76 },
    { 0x5cd36427, 90 },
    { 0x34a36428, 98 },
    { 0x1cb36427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca03642d, 98 },
    { 0x3e3642e, 61 },
    { 0, 0 },
    { 0x4b736430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb236436, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86836439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1153643d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40536442, 34 },
    { 0x5136443, 85 },
    { 0xd0f36444, 95 },
    { 0x5c136445, 85 },
    { 0x51936445, 100 },
    { 0xa9b36443, 76 },
    { 0xc4c36448, 36 },
    { 0xbc236445, 99 },
    { 0x2013644a, 33 },
    { 0x53a3644b, 93 },
    { 0x2763644c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17736451, 95 },
    { 0x7e636452, 76 },
    { 0x5e936452, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d436456, 33 },
    { 0x71736457, 95 },
    { 0, 0 },
    { 0x45936459, 23 },
    { 0xf6b3645a, 78 },
    { 0xe183645b, 84 },
    { 0x9793645b, 91 },
    { 0, 0 },
    { 0x9bb3645e, 98 },
    { 0x17b3645f, 77 },
    { 0x49a36460, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5436463, 61 },
    { 0x8dc36464, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d3646a, 46 },
    { 0xeb3646b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a336471, 90 },
    { 0x1636472, 43 },
    { 0xd0936471, 90 },
    { 0, 0 },
    { 0x6c436475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5936480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52836489, 33 },
    { 0xe113648a, 65 },
    { 0xb8736489, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1836490, 38 },
    { 0x45536490, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4553649b, 95 },
    { 0xbcf3649c, 100 },
    { 0, 0 },
    { 0x2473649e, 90 },
    { 0x6f3649f, 91 },
    { 0, 0 },
    { 0x450364a1, 76 },
    { 0x382364a1, 23 },
    { 0, 0 },
    { 0x6e7364a4, 66 },
    { 0x541364a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf364a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x367364ab, 95 },
    { 0, 0 },
    { 0x41c364ad, 76 },
    { 0x50a364ae, 77 },
    { 0x379364af, 55 },
    { 0xa62364af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65364b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb364b7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x189364ba, 76 },
    { 0xda6364ba, 95 },
    { 0x489364bc, 33 },
    { 0, 0 },
    { 0x1db364be, 76 },
    { 0, 0 },
    { 0xed0364c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x361364c3, 84 },
    { 0xa8364c4, 85 },
    { 0x615364c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4364c8, 76 },
    { 0x817364c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa364cd, 100 },
    { 0xce0364cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3364d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604364d5, 33 },
    { 0, 0 },
    { 0xa1e364d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x438364da, 76 },
    { 0x695364db, 61 },
    { 0x27c364dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6364e0, 87 },
    { 0, 0 },
    { 0x594364e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7364e7, 98 },
    { 0x8a7364e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5364eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda364ee, 61 },
    { 0, 0 },
    { 0xaeb364f0, 90 },
    { 0x9a5364f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9364fb, 61 },
    { 0x932364fc, 98 },
    { 0x490364fd, 34 },
    { 0xd41364fc, 23 },
    { 0x8fb364ff, 33 },
    { 0x20d36500, 90 },
    { 0x8536501, 55 },
    { 0xca136502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc36507, 76 },
    { 0, 0 },
    { 0x9f136509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3650c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3650f, 61 },
    { 0, 0 },
    { 0x92d36511, 34 },
    { 0x76936511, 61 },
    { 0xc2436513, 23 },
    { 0x91136514, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2936519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd3651f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb36523, 43 },
    { 0xe2c36524, 23 },
    { 0, 0 },
    { 0x1fb36526, 23 },
    { 0x6f936527, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363652a, 67 },
    { 0, 0 },
    { 0x5a53652c, 87 },
    { 0xef03652d, 33 },
    { 0x2433652e, 88 },
    { 0, 0 },
    { 0x6f936530, 90 },
    { 0xe5f36530, 96 },
    { 0xa1436532, 33 },
    { 0, 0 },
    { 0x62736534, 22 },
    { 0x17f36535, 78 },
    { 0x92436536, 90 },
    { 0x38336536, 33 },
    { 0xd4536535, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4253653b, 33 },
    { 0x2813653c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2433653f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd36543, 98 },
    { 0x32a36544, 90 },
    { 0x85a36544, 65 },
    { 0x63536546, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb36550, 76 },
    { 0xd5a36550, 95 },
    { 0x23536550, 33 },
    { 0x52c36553, 23 },
    { 0x59e36554, 90 },
    { 0xa436555, 77 },
    { 0x56836553, 23 },
    { 0x16636557, 34 },
    { 0x42f36551, 85 },
    { 0, 0 },
    { 0x4303655a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3136562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d3656f, 61 },
    { 0x6ec36570, 67 },
    { 0, 0 },
    { 0xe2e36572, 98 },
    { 0x6aa36573, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9236576, 95 },
    { 0x38136577, 76 },
    { 0xf6c36577, 76 },
    { 0x50136579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e336581, 99 },
    { 0, 0 },
    { 0x36536583, 99 },
    { 0xcf436583, 55 },
    { 0xb0336585, 33 },
    { 0, 0 },
    { 0x17c36587, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3453658b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e53658e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe636596, 98 },
    { 0, 0 },
    { 0xe3236598, 33 },
    { 0x9a936599, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e43659e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66365a1, 90 },
    { 0xaa2365a1, 33 },
    { 0, 0 },
    { 0xe93365a4, 93 },
    { 0x36f365a5, 78 },
    { 0xb4a365a4, 23 },
    { 0xddc365a5, 23 },
    { 0, 0 },
    { 0x2e4365a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81365af, 33 },
    { 0x17a365b0, 100 },
    { 0xa4c365b1, 55 },
    { 0x46e365b2, 90 },
    { 0xe7365b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55365b6, 98 },
    { 0, 0 },
    { 0xab7365b8, 97 },
    { 0, 0 },
    { 0x725365ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7365bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b365c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7365c6, 90 },
    { 0, 0 },
    { 0x269365c8, 68 },
    { 0, 0 },
    { 0x43a365ca, 46 },
    { 0xd4365ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x428365ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02365d2, 61 },
    { 0x38d365d3, 85 },
    { 0x83c365d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671365d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97365e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad365e3, 33 },
    { 0x690365e4, 76 },
    { 0x1f0365e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede365e9, 61 },
    { 0x67f365e9, 90 },
    { 0, 0 },
    { 0x4e3365ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79365ef, 29 },
    { 0xd4d365f0, 61 },
    { 0x8e8365f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f365f5, 33 },
    { 0, 0 },
    { 0x496365f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6365ff, 99 },
    { 0x6a2365ff, 76 },
    { 0xf6a36600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f436606, 34 },
    { 0x7a936607, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b3660a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee3660d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80136615, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc53661b, 98 },
    { 0x21a3661b, 98 },
    { 0xb253661d, 98 },
    { 0x9d53661e, 55 },
    { 0, 0 },
    { 0x3bc36620, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd536627, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4823662b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc753662e, 85 },
    { 0x1f13662f, 90 },
    { 0x28d3662e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe833663c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b3663f, 98 },
    { 0x4b53663f, 61 },
    { 0x7f636641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c036645, 34 },
    { 0x3336646, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb036649, 90 },
    { 0x2853664a, 33 },
    { 0x52b36649, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99736651, 78 },
    { 0, 0 },
    { 0xa2a36653, 52 },
    { 0x98c36654, 90 },
    { 0, 0 },
    { 0x58a36656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b236659, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3665c, 90 },
    { 0xe4d3665d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2636664, 93 },
    { 0xa5d36665, 65 },
    { 0xf5136666, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f3666a, 78 },
    { 0xb303666b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a36672, 33 },
    { 0x29336673, 40 },
    { 0x8336674, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb473667c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1933667f, 98 },
    { 0x5c136680, 65 },
    { 0x9d336681, 98 },
    { 0, 0 },
    { 0xea36683, 93 },
    { 0, 0 },
    { 0x15d36685, 33 },
    { 0xdd636686, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab36689, 23 },
    { 0x42136689, 98 },
    { 0, 0 },
    { 0xbd83668c, 90 },
    { 0, 0 },
    { 0x9813668e, 61 },
    { 0xf833668e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9636692, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e36696, 23 },
    { 0, 0 },
    { 0x51136698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d3669c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3713669f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a366a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780366a9, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6366ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e366af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2366b3, 34 },
    { 0xe7c366b3, 61 },
    { 0x9ca366b5, 33 },
    { 0x274366b3, 90 },
    { 0x194366b7, 75 },
    { 0, 0 },
    { 0xcb6366b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68366be, 90 },
    { 0x725366bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc366c3, 33 },
    { 0xcb0366c4, 76 },
    { 0xe4c366c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543366d0, 23 },
    { 0x48f366d1, 33 },
    { 0xbd9366d2, 90 },
    { 0x2be366d3, 67 },
    { 0x9ca366d4, 76 },
    { 0x1d0366d5, 93 },
    { 0, 0 },
    { 0x5ec366d7, 61 },
    { 0, 0 },
    { 0xe60366d9, 52 },
    { 0, 0 },
    { 0x2e7366db, 95 },
    { 0xf53366dc, 98 },
    { 0, 0 },
    { 0xaa0366de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32366e2, 96 },
    { 0x71366e3, 33 },
    { 0xe4f366e4, 93 },
    { 0, 0 },
    { 0x933366e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0366e9, 66 },
    { 0xf0a366ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8366ed, 95 },
    { 0, 0 },
    { 0xdcd366ef, 98 },
    { 0, 0 },
    { 0xa35366f1, 33 },
    { 0x5c3366f2, 93 },
    { 0xc98366f2, 77 },
    { 0x2e7366f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7366f8, 81 },
    { 0x94c366f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x692366fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8366ff, 91 },
    { 0xe7f36700, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45736704, 99 },
    { 0x39d36704, 46 },
    { 0xbe36704, 69 },
    { 0xf936707, 87 },
    { 0xd2136708, 61 },
    { 0x29136705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1123670c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3936714, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8236717, 90 },
    { 0xccd36717, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6043671d, 33 },
    { 0x2f3671e, 76 },
    { 0x2703671f, 87 },
    { 0x30836720, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d36726, 61 },
    { 0x1b936727, 98 },
    { 0, 0 },
    { 0x81236729, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf673672d, 61 },
    { 0, 0 },
    { 0xa7f3672f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f636736, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c3673a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85736740, 55 },
    { 0x1ea36741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a636748, 33 },
    { 0x25336749, 90 },
    { 0x6453674a, 46 },
    { 0, 0 },
    { 0x4533674c, 90 },
    { 0xe623674c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90436750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c36753, 23 },
    { 0xaf136754, 98 },
    { 0x17436755, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d36758, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf483675c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae636760, 90 },
    { 0, 0 },
    { 0x94f36762, 36 },
    { 0, 0 },
    { 0x13a36764, 33 },
    { 0xd7936765, 85 },
    { 0xd1236765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd03676a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5536779, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9773677e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d3678c, 23 },
    { 0xf123678d, 67 },
    { 0x14d3678e, 68 },
    { 0x28f3678e, 61 },
    { 0x77436790, 33 },
    { 0x7973678e, 95 },
    { 0xcbf3678d, 61 },
    { 0, 0 },
    { 0x7f836794, 43 },
    { 0xe936794, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40836798, 43 },
    { 0xb2e36799, 95 },
    { 0xc053679a, 55 },
    { 0x9473679a, 61 },
    { 0xeb33679b, 85 },
    { 0, 0 },
    { 0x5e33679e, 34 },
    { 0x623679e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38367a4, 66 },
    { 0x8dd367a5, 55 },
    { 0x5f8367a6, 87 },
    { 0, 0 },
    { 0x5a1367a8, 33 },
    { 0, 0 },
    { 0xd6e367aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1367b0, 69 },
    { 0x5c3367b1, 33 },
    { 0, 0 },
    { 0xdfc367b3, 23 },
    { 0xfb5367b3, 33 },
    { 0x616367b5, 33 },
    { 0, 0 },
    { 0x72c367b7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21367c0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x874367c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502367d0, 55 },
    { 0x476367d0, 46 },
    { 0, 0 },
    { 0x12b367d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5367d7, 53 },
    { 0xf46367d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e367dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62367e2, 52 },
    { 0x9bf367e3, 33 },
    { 0xafc367e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6367e8, 33 },
    { 0, 0 },
    { 0xea6367ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db367ee, 90 },
    { 0, 0 },
    { 0x82b367f0, 98 },
    { 0x202367f1, 98 },
    { 0xaed367f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf367fa, 61 },
    { 0, 0 },
    { 0x5b9367fc, 33 },
    { 0xbf9367fd, 90 },
    { 0xc91367fe, 95 },
    { 0x81e367ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9436802, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45336806, 100 },
    { 0xcd136807, 98 },
    { 0xa6a36807, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6636812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d36817, 98 },
    { 0x51236818, 34 },
    { 0x59436819, 90 },
    { 0xc8f3681a, 91 },
    { 0xa3b3681b, 33 },
    { 0xbf53681a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb836821, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a936827, 95 },
    { 0xbf336828, 46 },
    { 0x14536828, 100 },
    { 0xf383682a, 23 },
    { 0, 0 },
    { 0xcbc3682c, 95 },
    { 0x8963682d, 52 },
    { 0x4103682d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9236834, 61 },
    { 0, 0 },
    { 0xa1836836, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe533683e, 33 },
    { 0x9e43683e, 76 },
    { 0x3cd36840, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54036845, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15236853, 50 },
    { 0xb1936854, 90 },
    { 0x3f636855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f36858, 65 },
    { 0x59136858, 98 },
    { 0xf1f3685a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3536864, 55 },
    { 0x1af36864, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a3686e, 84 },
    { 0, 0 },
    { 0x71436870, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d36873, 61 },
    { 0, 0 },
    { 0xb5036875, 91 },
    { 0xc6636876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee3687a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d3687d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b36882, 98 },
    { 0, 0 },
    { 0x5aa36884, 63 },
    { 0x11636884, 90 },
    { 0, 0 },
    { 0x11e36887, 95 },
    { 0, 0 },
    { 0x3d36889, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373688d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32736892, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa536899, 52 },
    { 0xd113689a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c93689e, 98 },
    { 0x7cc3689e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e368a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x910368a5, 98 },
    { 0x698368a6, 76 },
    { 0, 0 },
    { 0x153368a8, 90 },
    { 0x9bb368a9, 55 },
    { 0xb7b368a9, 90 },
    { 0x5ff368aa, 77 },
    { 0x86a368ab, 90 },
    { 0x8d368ad, 66 },
    { 0xd23368aa, 76 },
    { 0xef1368af, 34 },
    { 0x9c7368b0, 55 },
    { 0xc61368af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e368b4, 87 },
    { 0xe5e368b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34368bb, 23 },
    { 0xbe368bc, 78 },
    { 0xf90368bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x742368c0, 33 },
    { 0xa2d368c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd368c7, 99 },
    { 0x22f368c7, 40 },
    { 0xf7e368c9, 23 },
    { 0x657368ca, 55 },
    { 0xfb1368cb, 81 },
    { 0xd03368c9, 98 },
    { 0x904368cc, 98 },
    { 0xd06368ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f368d3, 90 },
    { 0, 0 },
    { 0xa80368d5, 61 },
    { 0x686368d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2368d9, 77 },
    { 0, 0 },
    { 0x4ee368db, 33 },
    { 0, 0 },
    { 0x84f368dd, 93 },
    { 0, 0 },
    { 0xbc1368df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x986368e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec368e6, 77 },
    { 0, 0 },
    { 0x431368e8, 55 },
    { 0x32b368e9, 53 },
    { 0x710368e8, 61 },
    { 0xfc2368eb, 52 },
    { 0x8e3368ec, 100 },
    { 0x9f1368eb, 23 },
    { 0, 0 },
    { 0x6b5368ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f368f3, 78 },
    { 0xae4368f4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x677368f7, 61 },
    { 0x47f368f8, 76 },
    { 0xb1e368f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742368fd, 86 },
    { 0, 0 },
    { 0x8f8368ff, 43 },
    { 0xc2b36900, 90 },
    { 0xa9a368ff, 98 },
    { 0xfd736902, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc36906, 34 },
    { 0xf2a36907, 93 },
    { 0x7f636908, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f36911, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c36915, 61 },
    { 0xb8136916, 85 },
    { 0xad736917, 23 },
    { 0xb6136916, 43 },
    { 0x25536919, 98 },
    { 0xfdd3691a, 75 },
    { 0x8353691b, 33 },
    { 0xe4736917, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa936927, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa473692a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f3692d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe136933, 46 },
    { 0x75936933, 23 },
    { 0xd0636935, 43 },
    { 0x51836936, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b236939, 76 },
    { 0, 0 },
    { 0x2d33693b, 98 },
    { 0xe6f3693c, 33 },
    { 0, 0 },
    { 0xebc3693e, 92 },
    { 0xbf43693e, 76 },
    { 0xae236940, 87 },
    { 0x97736941, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e036945, 43 },
    { 0xeb636945, 90 },
    { 0x40c36947, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b43694a, 87 },
    { 0xda33694b, 61 },
    { 0x2e53694b, 55 },
    { 0x7663694c, 88 },
    { 0xd6f3694e, 95 },
    { 0xcd13694a, 90 },
    { 0xcaf3694d, 61 },
    { 0x90e36950, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e36956, 55 },
    { 0, 0 },
    { 0x94f36958, 66 },
    { 0x73336959, 90 },
    { 0, 0 },
    { 0xa53695b, 33 },
    { 0, 0 },
    { 0xde43695d, 53 },
    { 0xb283695d, 98 },
    { 0x81f3695f, 98 },
    { 0x8643695f, 99 },
    { 0x84e36961, 91 },
    { 0x6b836962, 76 },
    { 0x37436961, 76 },
    { 0xe2b36961, 93 },
    { 0x9493695d, 67 },
    { 0, 0 },
    { 0xdb036967, 33 },
    { 0xac236968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc3696e, 90 },
    { 0xbe33696f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a936972, 90 },
    { 0xea836972, 61 },
    { 0x95136972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee36978, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2903697e, 43 },
    { 0x9a43697f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c3698a, 33 },
    { 0, 0 },
    { 0xf253698c, 95 },
    { 0x81d3698d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4636990, 84 },
    { 0x87836990, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff36994, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe436999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce53699c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb213699f, 43 },
    { 0xaf6369a0, 43 },
    { 0, 0 },
    { 0xa90369a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f369a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e369ae, 98 },
    { 0xdb3369af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c369b3, 90 },
    { 0xe68369b4, 23 },
    { 0x892369b5, 95 },
    { 0xf6369b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b369b9, 90 },
    { 0x88d369ba, 76 },
    { 0xad0369bb, 61 },
    { 0x8e7369bc, 90 },
    { 0xaaa369ba, 90 },
    { 0xa8f369bc, 33 },
    { 0x1c2369bd, 98 },
    { 0xdc9369bf, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61369c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c369ca, 100 },
    { 0x8de369ca, 90 },
    { 0xff0369cb, 90 },
    { 0x35c369cb, 34 },
    { 0x55c369ca, 76 },
    { 0xff5369cf, 55 },
    { 0x490369ca, 98 },
    { 0xba1369d1, 43 },
    { 0, 0 },
    { 0xf85369d3, 91 },
    { 0, 0 },
    { 0xa5b369d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265369dc, 61 },
    { 0, 0 },
    { 0x940369de, 90 },
    { 0x677369df, 23 },
    { 0x6d5369df, 77 },
    { 0xb66369e1, 95 },
    { 0x512369de, 22 },
    { 0, 0 },
    { 0x5a9369e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6369e9, 61 },
    { 0x6b8369e9, 52 },
    { 0, 0 },
    { 0x1a2369ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac369f5, 61 },
    { 0xaaf369f5, 78 },
    { 0xd9c369f7, 90 },
    { 0xa6b369f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7369fb, 78 },
    { 0x13c369fc, 95 },
    { 0xdba369fd, 98 },
    { 0x929369fe, 91 },
    { 0x84b369fd, 91 },
    { 0x26936a00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27836a05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e736a08, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba36a0d, 55 },
    { 0x7c536a0e, 33 },
    { 0, 0 },
    { 0xd1636a10, 95 },
    { 0xe9e36a11, 61 },
    { 0x35836a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84836a16, 90 },
    { 0x48e36a17, 98 },
    { 0, 0 },
    { 0x79436a19, 90 },
    { 0x33436a19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b36a22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36936a26, 43 },
    { 0, 0 },
    { 0x86e36a28, 98 },
    { 0xaa636a29, 99 },
    { 0x32936a29, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb36a2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3836a32, 52 },
    { 0x37536a33, 76 },
    { 0xe0736a32, 90 },
    { 0xefc36a34, 93 },
    { 0x3dd36a34, 23 },
    { 0x54736a35, 23 },
    { 0xa9736a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71136a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82536a40, 90 },
    { 0xab736a41, 90 },
    { 0xf7436a40, 61 },
    { 0xcdc36a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14936a46, 98 },
    { 0xcb36a47, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe036a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea36a4f, 65 },
    { 0xcb336a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8136a54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9336a58, 61 },
    { 0x93936a58, 43 },
    { 0x92336a5a, 33 },
    { 0x49336a5a, 33 },
    { 0xb1336a59, 58 },
    { 0xbe436a5d, 84 },
    { 0xd9536a5d, 90 },
    { 0x9f036a5e, 34 },
    { 0xddd36a60, 95 },
    { 0, 0 },
    { 0x85a36a62, 87 },
    { 0x8ef36a62, 61 },
    { 0x76136a62, 76 },
    { 0x4cd36a64, 67 },
    { 0xd3636a66, 46 },
    { 0x48036a64, 95 },
    { 0x30136a68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a36a6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7536a75, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xde436a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de36a80, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d36a86, 61 },
    { 0, 0 },
    { 0x3f036a88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89736a8c, 93 },
    { 0x7fb36a8d, 61 },
    { 0xf2c36a8e, 80 },
    { 0x3d936a8e, 90 },
    { 0xaa036a8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c36a93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b736a97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb736a9c, 95 },
    { 0, 0 },
    { 0xf6636a9e, 33 },
    { 0, 0 },
    { 0x97136aa0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x67836aa3, 95 },
    { 0xbb736aa4, 90 },
    { 0x82a36aa5, 61 },
    { 0, 0 },
    { 0x5ca36aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65836ab5, 23 },
    { 0x79c36ab6, 87 },
    { 0x3ee36ab7, 98 },
    { 0x8c836ab8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd036abd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b36ac4, 33 },
    { 0x38a36ac4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e536aca, 46 },
    { 0x2fd36acb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77536ace, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2136ad5, 33 },
    { 0xce336ad5, 95 },
    { 0xe3436ad6, 99 },
    { 0x72336ad5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7236adb, 95 },
    { 0, 0 },
    { 0x52836add, 98 },
    { 0, 0 },
    { 0x5bd36adf, 93 },
    { 0x7a836adf, 34 },
    { 0x68536ae0, 33 },
    { 0x8a136ae1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8436ae5, 23 },
    { 0x36f36ae6, 95 },
    { 0x12d36ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71336aeb, 39 },
    { 0xa2336aec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad36af0, 61 },
    { 0x29036af1, 90 },
    { 0x84636af2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4536af6, 66 },
    { 0xc0936af6, 23 },
    { 0xe2436af7, 95 },
    { 0x89036af9, 95 },
    { 0x85d36af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a336afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d536b00, 36 },
    { 0, 0 },
    { 0x56a36b02, 83 },
    { 0x2336b03, 61 },
    { 0xf2a36b04, 85 },
    { 0xb9436b04, 90 },
    { 0x2ef36b06, 46 },
    { 0x3c536b04, 61 },
    { 0x3ee36b02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa236b0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42036b10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc36b13, 66 },
    { 0, 0 },
    { 0xdab36b15, 95 },
    { 0, 0 },
    { 0x1a936b17, 98 },
    { 0xb336b18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72236b20, 23 },
    { 0, 0 },
    { 0x2e036b22, 55 },
    { 0, 0 },
    { 0x36136b24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a36b27, 52 },
    { 0x48736b27, 52 },
    { 0x95936b27, 90 },
    { 0xdd036b2a, 90 },
    { 0, 0 },
    { 0xcac36b2c, 90 },
    { 0x7d36b2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36b30, 88 },
    { 0x34236b30, 98 },
    { 0x35436b32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7836b36, 90 },
    { 0x6f636b37, 34 },
    { 0x3f36b37, 87 },
    { 0xf0436b38, 17 },
    { 0x41536b37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78036b43, 76 },
    { 0xc8836b43, 78 },
    { 0xb5736b45, 99 },
    { 0xfe736b46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e36b4c, 98 },
    { 0, 0 },
    { 0x90536b4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9336b53, 99 },
    { 0x1b336b54, 90 },
    { 0, 0 },
    { 0x1ae36b56, 78 },
    { 0x6de36b57, 55 },
    { 0x24f36b56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x51436b5b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b36b5f, 17 },
    { 0, 0 },
    { 0xab136b61, 65 },
    { 0x4536b62, 90 },
    { 0x41736b61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d36b6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f36b74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce36b77, 95 },
    { 0, 0 },
    { 0xe536b79, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f36b82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf836b85, 55 },
    { 0xc9c36b85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c36b8b, 61 },
    { 0x97b36b8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b36b8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e336b94, 55 },
    { 0, 0 },
    { 0x59d36b96, 61 },
    { 0xe2136b96, 98 },
    { 0, 0 },
    { 0x8f36b99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30936b9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe936ba1, 55 },
    { 0x3c936ba1, 90 },
    { 0, 0 },
    { 0xdbc36ba4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76236ba8, 78 },
    { 0x64236ba9, 90 },
    { 0xa0636baa, 98 },
    { 0xa0d36ba8, 98 },
    { 0x90d36bac, 95 },
    { 0x76336bad, 91 },
    { 0x21e36bae, 76 },
    { 0x42b36bad, 90 },
    { 0xe6636ba8, 43 },
    { 0, 0 },
    { 0xf2336bb2, 23 },
    { 0xb8136bb3, 77 },
    { 0x8a936bb4, 23 },
    { 0x21b36bb5, 66 },
    { 0x40636bb6, 34 },
    { 0xbcf36bb7, 23 },
    { 0, 0 },
    { 0x6e136bb9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa36bbd, 76 },
    { 0, 0 },
    { 0x39d36bbf, 46 },
    { 0, 0 },
    { 0xa6d36bc1, 95 },
    { 0, 0 },
    { 0x7aa36bc3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba36bc7, 55 },
    { 0, 0 },
    { 0xf0a36bc9, 92 },
    { 0xea136bca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f36bcd, 17 },
    { 0xdde36bce, 98 },
    { 0xcff36bcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1236bd9, 95 },
    { 0, 0 },
    { 0x90036bdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c336be1, 98 },
    { 0xcdc36be2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d736be8, 42 },
    { 0x64b36be9, 65 },
    { 0xfa336bea, 76 },
    { 0x2236bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b36bef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1636bf8, 19 },
    { 0, 0 },
    { 0x81836bfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37836bfe, 19 },
    { 0xceb36bff, 61 },
    { 0x1a936bff, 85 },
    { 0xe7936c01, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a636c07, 98 },
    { 0xfee36c07, 46 },
    { 0x64c36c09, 78 },
    { 0xeac36c09, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b36c10, 55 },
    { 0, 0 },
    { 0x2e036c12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34236c18, 43 },
    { 0xcdd36c19, 95 },
    { 0x58736c1a, 23 },
    { 0xae136c1b, 34 },
    { 0, 0 },
    { 0x5d736c1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19036c20, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd36c23, 76 },
    { 0xde536c24, 34 },
    { 0x1bf36c25, 95 },
    { 0x3a336c25, 90 },
    { 0xf6836c23, 95 },
    { 0x4f936c28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a36c2f, 99 },
    { 0x50636c30, 93 },
    { 0xcca36c2f, 100 },
    { 0xcf536c32, 66 },
    { 0, 0 },
    { 0x37636c34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c136c37, 98 },
    { 0xeb636c38, 90 },
    { 0, 0 },
    { 0x79436c3a, 87 },
    { 0xcd136c3b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47236c3f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c736c45, 33 },
    { 0x48d36c46, 61 },
    { 0xc7936c45, 23 },
    { 0, 0 },
    { 0x42036c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e36c4d, 76 },
    { 0, 0 },
    { 0xc3036c4f, 95 },
    { 0xeb036c50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e936c55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e236c58, 84 },
    { 0xb936c59, 90 },
    { 0xbf636c5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30136c5d, 61 },
    { 0x1f236c5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236c61, 33 },
    { 0x45036c62, 33 },
    { 0xd6036c63, 33 },
    { 0x4c636c63, 61 },
    { 0xb6f36c65, 95 },
    { 0, 0 },
    { 0x2aa36c67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9036c6f, 98 },
    { 0x95f36c70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c36c77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54236c7b, 93 },
    { 0xc0f36c7b, 90 },
    { 0x49836c7d, 33 },
    { 0, 0 },
    { 0x8d136c7f, 61 },
    { 0xe3836c80, 33 },
    { 0x93a36c81, 66 },
    { 0x8de36c81, 90 },
    { 0x44936c80, 61 },
    { 0, 0 },
    { 0x16936c85, 90 },
    { 0xf1e36c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d736c8b, 66 },
    { 0, 0 },
    { 0x53d36c8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b836c90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e536c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81036c99, 90 },
    { 0xd9b36c99, 34 },
    { 0xdfd36c9a, 95 },
    { 0xb6036c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4536ca4, 23 },
    { 0, 0 },
    { 0x66536ca6, 95 },
    { 0x9e636ca6, 90 },
    { 0xb7b36ca8, 98 },
    { 0x1436ca7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64136cae, 93 },
    { 0xa3536caf, 23 },
    { 0xaef36cb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc36cb3, 90 },
    { 0, 0 },
    { 0xd4136cb5, 33 },
    { 0x87536cb6, 46 },
    { 0xc4436cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9436cbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d636cbe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91136cc3, 61 },
    { 0x78236cc4, 95 },
    { 0xda736cc5, 33 },
    { 0x80536cc6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc36ccc, 90 },
    { 0xea136ccd, 81 },
    { 0x64836cce, 87 },
    { 0x2f636ccd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e36cdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e36ce1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63936ce8, 33 },
    { 0, 0 },
    { 0xef536cea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91336ced, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb36d01, 90 },
    { 0x51836d02, 99 },
    { 0x87c36d03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5436d09, 85 },
    { 0, 0 },
    { 0x73f36d0b, 55 },
    { 0x2e936d0c, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd336d12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84536d15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c36d18, 61 },
    { 0x48136d18, 84 },
    { 0xd536d1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34936d23, 78 },
    { 0x91d36d24, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e36d29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48936d2d, 61 },
    { 0x73f36d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42236d32, 46 },
    { 0, 0 },
    { 0xccb36d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17936d3a, 90 },
    { 0, 0 },
    { 0xf9736d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86736d44, 43 },
    { 0xe5136d44, 98 },
    { 0x49736d46, 85 },
    { 0xe4336d44, 98 },
    { 0x99c36d47, 90 },
    { 0x3f036d49, 85 },
    { 0x62036d4a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c636d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41136d5f, 17 },
    { 0x9d736d60, 61 },
    { 0x58636d61, 52 },
    { 0, 0 },
    { 0xcd336d63, 88 },
    { 0x8a736d63, 85 },
    { 0x5f336d63, 55 },
    { 0, 0 },
    { 0xddf36d67, 46 },
    { 0x90b36d67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39836d6c, 90 },
    { 0x28636d6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4036d70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d36d75, 61 },
    { 0, 0 },
    { 0xd1436d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6236d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc36d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93136d80, 90 },
    { 0, 0 },
    { 0xde936d82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x88336d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2036d89, 95 },
    { 0, 0 },
    { 0xe4c36d8b, 78 },
    { 0x7d336d8c, 75 },
    { 0, 0 },
    { 0x83336d8e, 60 },
    { 0xa2236d8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c436d94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd36d99, 55 },
    { 0xc4536d9a, 17 },
    { 0, 0 },
    { 0xf9636d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a336d9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d036da3, 78 },
    { 0x96f36da4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96636da7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14736dab, 84 },
    { 0xc9136dac, 23 },
    { 0, 0 },
    { 0xba236dae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad036db2, 33 },
    { 0xed636db2, 61 },
    { 0x6236db2, 23 },
    { 0, 0 },
    { 0xc3436db6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e36db9, 43 },
    { 0x38836dba, 87 },
    { 0x34436dbb, 69 },
    { 0xd3b36dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c036dc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d336dc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef36dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac36dd2, 90 },
    { 0xda736dd3, 94 },
    { 0xb3236dd3, 23 },
    { 0x3fd36dd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea36de2, 90 },
    { 0xc6236de2, 40 },
    { 0xbff36de2, 98 },
    { 0x3dc36de3, 61 },
    { 0xe9336de3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6036de9, 61 },
    { 0xc3436dea, 34 },
    { 0, 0 },
    { 0xf2036dec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a36df4, 33 },
    { 0x83a36df5, 78 },
    { 0xeb436df6, 23 },
    { 0x36e36df6, 33 },
    { 0xb9036df8, 23 },
    { 0, 0 },
    { 0xdfe36dfa, 2 },
    { 0, 0 },
    { 0xffd36dfc, 23 },
    { 0, 0 },
    { 0xabd36dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b36e04, 65 },
    { 0x52a36e05, 45 },
    { 0, 0 },
    { 0xf0136e07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e36e0a, 34 },
    { 0x1b536e0b, 91 },
    { 0, 0 },
    { 0xea036e0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d036e13, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32836e24, 55 },
    { 0, 0 },
    { 0xf7e36e26, 91 },
    { 0xa3036e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf936e2a, 76 },
    { 0x7db36e2b, 46 },
    { 0x70236e2c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47236e31, 66 },
    { 0, 0 },
    { 0x8a236e33, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a36e3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e836e3e, 90 },
    { 0, 0 },
    { 0x10c36e40, 23 },
    { 0xc9936e41, 88 },
    { 0xda136e42, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0636e46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94536e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7836e50, 55 },
    { 0, 0 },
    { 0x10236e52, 91 },
    { 0, 0 },
    { 0x40036e54, 66 },
    { 0xd136e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa236e5b, 33 },
    { 0xdfc36e5c, 55 },
    { 0xbbb36e5d, 98 },
    { 0xce736e5b, 55 },
    { 0x92a36e5f, 93 },
    { 0xd6836e5f, 87 },
    { 0x84d36e61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d36e66, 98 },
    { 0x93436e67, 23 },
    { 0x4ff36e68, 34 },
    { 0, 0 },
    { 0xd2036e6a, 96 },
    { 0, 0 },
    { 0x64b36e6c, 25 },
    { 0xf3a36e6d, 61 },
    { 0, 0 },
    { 0x92236e6f, 23 },
    { 0x45836e6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88836e73, 95 },
    { 0xcca36e74, 90 },
    { 0xbad36e73, 95 },
    { 0xd0336e76, 61 },
    { 0x74136e77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17936e82, 76 },
    { 0x41836e83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e36e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7136e8d, 98 },
    { 0xd0d36e8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f36e91, 23 },
    { 0xca436e91, 98 },
    { 0, 0 },
    { 0xa4736e94, 81 },
    { 0, 0 },
    { 0x2e236e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50636e9d, 88 },
    { 0x9b636e9e, 90 },
    { 0x5ee36e9f, 100 },
    { 0xe9136e9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30936ea6, 91 },
    { 0x56836ea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e736eaa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc36eb0, 91 },
    { 0xb936eb0, 87 },
    { 0xca436eb1, 46 },
    { 0xafc36eb3, 98 },
    { 0x6fd36eb3, 90 },
    { 0x49f36eb5, 23 },
    { 0xd4636eb5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe436eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9536ebc, 24 },
    { 0x9c136ebd, 46 },
    { 0x8cc36ebd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b36ec8, 23 },
    { 0xb0136ec8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99836ed0, 61 },
    { 0x16136ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5636edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d136ee2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1036ee5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a36eea, 23 },
    { 0, 0 },
    { 0x9ce36eec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x74736eef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0536ef3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10936ef8, 80 },
    { 0x4ff36ef8, 33 },
    { 0xce136ef9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d36efd, 76 },
    { 0xbf836efd, 33 },
    { 0x7df36eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b36f03, 61 },
    { 0, 0 },
    { 0x6c336f05, 95 },
    { 0x37f36f06, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7136f09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25736f0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f736f0f, 93 },
    { 0, 0 },
    { 0x6e536f11, 90 },
    { 0xdc536f12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9436f19, 34 },
    { 0xf8836f19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83136f1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d436f21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d536f2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe36f32, 17 },
    { 0x4a236f33, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad436f36, 90 },
    { 0xbda36f37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf36f3d, 87 },
    { 0, 0 },
    { 0xea436f3f, 33 },
    { 0x68236f3f, 66 },
    { 0x66836f40, 23 },
    { 0xc6b36f42, 95 },
    { 0x42a36f41, 33 },
    { 0x68536f43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32636f49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee236f52, 34 },
    { 0x59136f53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c36f57, 95 },
    { 0xe0e36f57, 33 },
    { 0x99236f59, 61 },
    { 0x6b336f57, 80 },
    { 0xeab36f5b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c36f62, 33 },
    { 0x36836f63, 61 },
    { 0, 0 },
    { 0xd5536f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf236f68, 90 },
    { 0xf1636f69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81136f71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf36f76, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99936f7a, 23 },
    { 0xd8836f7b, 90 },
    { 0, 0 },
    { 0x8ce36f7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e36f80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c36f84, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e36f87, 95 },
    { 0xfda36f87, 61 },
    { 0x35e36f88, 40 },
    { 0x9ca36f87, 52 },
    { 0x5b236f8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x57336f8e, 90 },
    { 0xa0e36f8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8736f94, 98 },
    { 0x4e636f95, 23 },
    { 0, 0 },
    { 0x9db36f97, 23 },
    { 0x66736f98, 34 },
    { 0xeb36f99, 23 },
    { 0x6ad36f9a, 66 },
    { 0x14a36f9b, 98 },
    { 0, 0 },
    { 0x8cc36f9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d36fa1, 98 },
    { 0xd2836fa2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x78036fa5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc436fa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a36faf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e336fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d36fb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59536fbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35636fc2, 17 },
    { 0, 0 },
    { 0x97e36fc4, 23 },
    { 0x5836fc4, 90 },
    { 0x3b836fc4, 52 },
    { 0, 0 },
    { 0x12036fc8, 61 },
    { 0x71036fc9, 61 },
    { 0xa536fc9, 23 },
    { 0x2b636fcb, 46 },
    { 0, 0 },
    { 0x46436fcd, 23 },
    { 0, 0 },
    { 0xa5736fcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec36fd5, 23 },
    { 0xd2e36fd5, 34 },
    { 0x8ea36fd5, 67 },
    { 0, 0 },
    { 0x1bc36fd9, 17 },
    { 0x2ea36fda, 85 },
    { 0x83e36fd9, 95 },
    { 0, 0 },
    { 0x31836fdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a36fe1, 90 },
    { 0x52736fe2, 76 },
    { 0x6cd36fe3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de36fe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d36fee, 90 },
    { 0xa8436fef, 23 },
    { 0, 0 },
    { 0xbba36ff1, 93 },
    { 0x33c36ff2, 33 },
    { 0, 0 },
    { 0xa5c36ff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec36ff8, 23 },
    { 0x22b36ff9, 66 },
    { 0x1b236ff8, 95 },
    { 0xa7c36ffb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec37004, 55 },
    { 0xd537005, 91 },
    { 0xe7437006, 98 },
    { 0x7837007, 90 },
    { 0x26537008, 94 },
    { 0xd9e37009, 46 },
    { 0xb2b3700a, 98 },
    { 0xe8537004, 76 },
    { 0xd9a3700c, 87 },
    { 0x39a3700d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50137011, 99 },
    { 0, 0 },
    { 0xc6137013, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3701c, 87 },
    { 0x5913701d, 52 },
    { 0x5873701d, 95 },
    { 0x8683701f, 23 },
    { 0x1e237020, 61 },
    { 0x9bd3701f, 34 },
    { 0, 0 },
    { 0x3fd37023, 98 },
    { 0x53437024, 98 },
    { 0x27f37025, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee3702a, 33 },
    { 0, 0 },
    { 0xcb3702c, 76 },
    { 0x7543702c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e937032, 95 },
    { 0xeea37032, 66 },
    { 0x8c437034, 34 },
    { 0, 0 },
    { 0xcc737036, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b37043, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d37048, 91 },
    { 0, 0 },
    { 0xfda3704a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17437052, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc337058, 33 },
    { 0x88237058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3705e, 17 },
    { 0, 0 },
    { 0x27a37060, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa37063, 87 },
    { 0xc437064, 23 },
    { 0x2b737064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b37069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f3706c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d37072, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be37076, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8737079, 33 },
    { 0, 0 },
    { 0xf8e3707b, 23 },
    { 0x603707b, 33 },
    { 0xebf3707b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c37081, 76 },
    { 0, 0 },
    { 0x2fd37083, 23 },
    { 0xc9f37084, 75 },
    { 0, 0 },
    { 0x30037086, 90 },
    { 0xaa037087, 76 },
    { 0x3b937088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023708b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b37090, 66 },
    { 0x85d37091, 84 },
    { 0xdb537091, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f537096, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd03709f, 90 },
    { 0, 0 },
    { 0x928370a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174370a5, 95 },
    { 0x6f4370a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3370aa, 90 },
    { 0, 0 },
    { 0x3d4370ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x599370af, 78 },
    { 0, 0 },
    { 0xf45370b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48370bb, 61 },
    { 0x7e370bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4370c1, 95 },
    { 0xd19370c2, 34 },
    { 0xe6f370c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79370cd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df370d2, 98 },
    { 0x99a370d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2370d6, 34 },
    { 0xc41370d7, 76 },
    { 0xa65370d7, 61 },
    { 0x303370d9, 61 },
    { 0x13f370d8, 43 },
    { 0xed8370db, 23 },
    { 0, 0 },
    { 0xf11370dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e370e1, 33 },
    { 0x898370e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d370e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05370f2, 55 },
    { 0x6fd370f3, 87 },
    { 0xd76370f2, 95 },
    { 0, 0 },
    { 0x231370f6, 91 },
    { 0, 0 },
    { 0x38370f8, 76 },
    { 0, 0 },
    { 0xefd370fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f370ff, 66 },
    { 0xe3c370ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf37104, 75 },
    { 0x72137105, 34 },
    { 0xd4f37106, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda37110, 98 },
    { 0x78f37111, 85 },
    { 0xfd837112, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5343711b, 61 },
    { 0, 0 },
    { 0x4b83711d, 61 },
    { 0xe043711e, 97 },
    { 0xe323711f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c37123, 61 },
    { 0xc9637123, 34 },
    { 0x31837125, 98 },
    { 0xc637126, 61 },
    { 0xca37126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb3712a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3712d, 55 },
    { 0x8323712e, 95 },
    { 0x9a83712f, 43 },
    { 0x7d83712f, 46 },
    { 0, 0 },
    { 0x48e37132, 69 },
    { 0xf0737133, 95 },
    { 0xd7537134, 33 },
    { 0x1fa37135, 77 },
    { 0x8ae37132, 93 },
    { 0x56f37136, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b83713d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1337148, 66 },
    { 0xe6237149, 80 },
    { 0x25037149, 53 },
    { 0x18e3714b, 90 },
    { 0x1113714b, 90 },
    { 0x7c37148, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7337150, 90 },
    { 0x12e37150, 61 },
    { 0xf6437151, 90 },
    { 0, 0 },
    { 0x6137154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d537157, 22 },
    { 0x5a137158, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283715c, 33 },
    { 0, 0 },
    { 0xb493715e, 95 },
    { 0xb1e3715e, 90 },
    { 0xa653715f, 33 },
    { 0x15f37161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3837164, 33 },
    { 0, 0 },
    { 0x80437166, 100 },
    { 0, 0 },
    { 0xb8737168, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0237170, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89737174, 93 },
    { 0xe2337174, 98 },
    { 0x55137176, 61 },
    { 0x65937176, 90 },
    { 0xeb337177, 99 },
    { 0, 0 },
    { 0x2333717a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e23717e, 33 },
    { 0xb453717e, 23 },
    { 0x87e3717f, 38 },
    { 0x74237181, 90 },
    { 0, 0 },
    { 0x74437183, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0737188, 95 },
    { 0x64537188, 78 },
    { 0xf8137189, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2583718e, 55 },
    { 0xd653718f, 90 },
    { 0x41d3718f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa337195, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4637198, 95 },
    { 0x6e337199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a3719d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e371a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb371a4, 33 },
    { 0, 0 },
    { 0xca3371a6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483371ab, 90 },
    { 0, 0 },
    { 0x293371ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b371b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x966371b3, 98 },
    { 0xff8371b3, 87 },
    { 0x84d371b5, 95 },
    { 0x11b371b6, 95 },
    { 0xafc371b4, 90 },
    { 0, 0 },
    { 0x5c8371b9, 90 },
    { 0xd8d371ba, 93 },
    { 0x6cd371ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca371be, 34 },
    { 0x9e5371bf, 95 },
    { 0xc49371bf, 95 },
    { 0xef4371c0, 75 },
    { 0x1c2371c2, 55 },
    { 0x8fa371c1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1371c6, 99 },
    { 0xbfa371c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce371cb, 95 },
    { 0, 0 },
    { 0x4c7371cd, 33 },
    { 0x182371ce, 23 },
    { 0x2ab371cd, 33 },
    { 0xde0371cd, 100 },
    { 0x285371cf, 33 },
    { 0x243371d2, 90 },
    { 0x8b1371cf, 91 },
    { 0x924371d4, 43 },
    { 0x423371d5, 98 },
    { 0, 0 },
    { 0x59a371d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6371de, 23 },
    { 0x8df371de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca371e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a371ed, 98 },
    { 0xec3371ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616371f2, 87 },
    { 0x98371f2, 90 },
    { 0, 0 },
    { 0xec2371f5, 61 },
    { 0, 0 },
    { 0x5d2371f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a371fa, 95 },
    { 0xff371fb, 52 },
    { 0x3b371fc, 46 },
    { 0xae8371fd, 90 },
    { 0xcc371fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3637204, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b3720b, 55 },
    { 0x2fb3720c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe313720f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf137216, 90 },
    { 0, 0 },
    { 0xdbb37218, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe443721b, 98 },
    { 0xa733721c, 90 },
    { 0xedb3721c, 61 },
    { 0x1b93721b, 84 },
    { 0x6a73721d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8137224, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2637227, 66 },
    { 0x7f637227, 33 },
    { 0, 0 },
    { 0x6013722a, 93 },
    { 0xa723722b, 61 },
    { 0x69c3722b, 95 },
    { 0xcc43722c, 90 },
    { 0, 0 },
    { 0x7d03722f, 84 },
    { 0, 0 },
    { 0x85e37231, 90 },
    { 0x28e37232, 23 },
    { 0xebe37232, 61 },
    { 0x4737233, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a037237, 87 },
    { 0x84737237, 76 },
    { 0xb3937239, 36 },
    { 0, 0 },
    { 0x5a33723b, 90 },
    { 0x1683723c, 61 },
    { 0xa783723b, 23 },
    { 0x1e93723e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20437241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1137244, 76 },
    { 0, 0 },
    { 0x8d137246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf63724a, 71 },
    { 0x65d3724b, 95 },
    { 0, 0 },
    { 0xc903724d, 87 },
    { 0, 0 },
    { 0x24f3724f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6737261, 91 },
    { 0xdf937261, 61 },
    { 0, 0 },
    { 0x7cf37264, 76 },
    { 0, 0 },
    { 0xa2a37266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb837269, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e23726e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2337271, 23 },
    { 0, 0 },
    { 0x63637273, 55 },
    { 0, 0 },
    { 0xae237275, 88 },
    { 0, 0 },
    { 0x637277, 98 },
    { 0, 0 },
    { 0xc0237279, 34 },
    { 0xf703727a, 90 },
    { 0x62637279, 23 },
    { 0, 0 },
    { 0x3a43727d, 34 },
    { 0, 0 },
    { 0x72f3727f, 76 },
    { 0xf6a3727f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7237283, 77 },
    { 0x32837284, 98 },
    { 0xaae37284, 90 },
    { 0xac037286, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9503728d, 98 },
    { 0xb183728e, 95 },
    { 0xfec3728f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc837293, 98 },
    { 0xf8137294, 78 },
    { 0x2e37294, 61 },
    { 0xc537296, 88 },
    { 0x47737294, 23 },
    { 0x21e37297, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a3729d, 23 },
    { 0, 0 },
    { 0x8fb3729f, 90 },
    { 0x172372a0, 93 },
    { 0, 0 },
    { 0x532372a2, 65 },
    { 0xe12372a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6372a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99372aa, 91 },
    { 0x75b372ab, 98 },
    { 0x3e4372ac, 100 },
    { 0x628372aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3372b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0372b3, 66 },
    { 0x9d4372b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4372c0, 33 },
    { 0, 0 },
    { 0x10e372c2, 61 },
    { 0xf6a372c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x949372c6, 61 },
    { 0, 0 },
    { 0xbae372c8, 61 },
    { 0x520372c8, 33 },
    { 0, 0 },
    { 0xa4e372cb, 93 },
    { 0, 0 },
    { 0xbaf372cd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84372d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad372db, 76 },
    { 0xcb372dc, 98 },
    { 0x6b3372dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0372e8, 76 },
    { 0, 0 },
    { 0xdf372ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d372ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2372f2, 33 },
    { 0, 0 },
    { 0xc20372f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c372f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318372fe, 67 },
    { 0xc91372ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b37314, 52 },
    { 0x2a037314, 98 },
    { 0, 0 },
    { 0xbc37317, 39 },
    { 0, 0 },
    { 0xee637319, 93 },
    { 0x74437319, 93 },
    { 0, 0 },
    { 0xdea3731c, 55 },
    { 0xff13731d, 78 },
    { 0x5ec3731e, 33 },
    { 0x8ac3731e, 76 },
    { 0x4737320, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88337325, 23 },
    { 0, 0 },
    { 0xa4c37327, 85 },
    { 0x31537328, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373732c, 90 },
    { 0x7b93732d, 75 },
    { 0x1293732e, 23 },
    { 0x8e53732f, 90 },
    { 0x59e37330, 98 },
    { 0x473732e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d237334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e537337, 83 },
    { 0, 0 },
    { 0xe9837339, 55 },
    { 0, 0 },
    { 0xb3b3733b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a37340, 99 },
    { 0x60e37341, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23037349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef3734d, 46 },
    { 0xb623734e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b37351, 61 },
    { 0x55b37351, 55 },
    { 0xd537352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d837357, 97 },
    { 0x3d737358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd183735c, 90 },
    { 0x3133735d, 46 },
    { 0x7a23735e, 55 },
    { 0x4373735f, 75 },
    { 0x8d13735f, 98 },
    { 0, 0 },
    { 0x4a137362, 92 },
    { 0xac337362, 66 },
    { 0, 0 },
    { 0x6e637365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf3736c, 98 },
    { 0x7383736d, 33 },
    { 0x37b3736e, 98 },
    { 0xbce3736c, 76 },
    { 0, 0 },
    { 0x78b37371, 33 },
    { 0x79137371, 95 },
    { 0x9c437373, 90 },
    { 0xc6637371, 95 },
    { 0x1c937374, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1173737a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb537380, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79637387, 61 },
    { 0xdcc37387, 93 },
    { 0xe5237389, 98 },
    { 0xb3937389, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93837391, 33 },
    { 0xc2337391, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01373a3, 23 },
    { 0, 0 },
    { 0xb4d373a5, 77 },
    { 0x7e7373a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844373aa, 93 },
    { 0xe1e373ab, 67 },
    { 0, 0 },
    { 0x424373ad, 90 },
    { 0, 0 },
    { 0xb6f373af, 76 },
    { 0x734373b0, 85 },
    { 0x3f373af, 81 },
    { 0, 0 },
    { 0xb6f373b3, 55 },
    { 0, 0 },
    { 0x91d373b5, 90 },
    { 0xad3373b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c373c2, 34 },
    { 0xc30373c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d373c8, 73 },
    { 0xc87373c8, 98 },
    { 0x357373c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560373ce, 90 },
    { 0xcde373cf, 76 },
    { 0, 0 },
    { 0xc60373d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f373d5, 33 },
    { 0, 0 },
    { 0x915373d7, 14 },
    { 0, 0 },
    { 0x232373d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7373e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428373e9, 23 },
    { 0, 0 },
    { 0x8e2373eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12373f7, 90 },
    { 0, 0 },
    { 0x3a0373f9, 55 },
    { 0xaf1373f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d637402, 76 },
    { 0x55a37402, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e37406, 76 },
    { 0xb4637406, 60 },
    { 0x7337408, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d3740c, 95 },
    { 0x3483740c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5737411, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a37415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d3741a, 78 },
    { 0xf783741b, 80 },
    { 0x4ba3741c, 100 },
    { 0, 0 },
    { 0x1183741e, 53 },
    { 0x8df3741f, 46 },
    { 0xc5e3741f, 88 },
    { 0xc9337421, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2903742d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24037438, 90 },
    { 0xa5e37438, 90 },
    { 0x6fd3743a, 55 },
    { 0x8483743b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9553743f, 95 },
    { 0xbf537440, 95 },
    { 0xb6337440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91837445, 61 },
    { 0xa4b37445, 95 },
    { 0xbbf37445, 43 },
    { 0x69137448, 98 },
    { 0x31b37449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e03744c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45437454, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69437457, 23 },
    { 0xfc937458, 76 },
    { 0xd6037459, 75 },
    { 0xe8a37457, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2913745e, 88 },
    { 0xc743745f, 55 },
    { 0, 0 },
    { 0x86037461, 66 },
    { 0xba437462, 76 },
    { 0xf5e37463, 86 },
    { 0, 0 },
    { 0x29e37465, 33 },
    { 0, 0 },
    { 0x83437467, 23 },
    { 0, 0 },
    { 0xe7937469, 44 },
    { 0, 0 },
    { 0xdfe3746b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd913746e, 90 },
    { 0x2a73746f, 23 },
    { 0xe513746e, 61 },
    { 0x9a63746e, 90 },
    { 0xc5237472, 55 },
    { 0x4e837472, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb3747f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce737482, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6337485, 99 },
    { 0, 0 },
    { 0xc2837487, 90 },
    { 0xf6837488, 99 },
    { 0x9037488, 98 },
    { 0x9543748a, 90 },
    { 0x8f83748a, 98 },
    { 0x44d3748c, 61 },
    { 0x1a33748d, 76 },
    { 0xe4f3748e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b37497, 34 },
    { 0x4bf37498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313749b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44374a0, 33 },
    { 0x48b374a1, 76 },
    { 0, 0 },
    { 0x6fd374a3, 43 },
    { 0xdab374a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875374a8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80374ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5374b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d374b5, 23 },
    { 0, 0 },
    { 0x2cc374b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b374bd, 55 },
    { 0xc16374be, 34 },
    { 0xe14374be, 52 },
    { 0x6f3374bf, 98 },
    { 0x834374c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29374c7, 81 },
    { 0x9c6374c8, 93 },
    { 0x2b9374c9, 90 },
    { 0x141374c9, 90 },
    { 0x93a374c7, 95 },
    { 0, 0 },
    { 0x74f374cd, 65 },
    { 0xc75374ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147374d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662374db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3374e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431374e9, 78 },
    { 0xdf5374e9, 33 },
    { 0, 0 },
    { 0x40b374ec, 90 },
    { 0xb20374ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba374f0, 90 },
    { 0x416374f1, 33 },
    { 0xde0374f2, 23 },
    { 0, 0 },
    { 0xc19374f4, 85 },
    { 0xdaa374f4, 90 },
    { 0, 0 },
    { 0x7e2374f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1374fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79437503, 33 },
    { 0x63137504, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b37509, 55 },
    { 0x1273750a, 46 },
    { 0x3e93750b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d37511, 23 },
    { 0x44137512, 55 },
    { 0xa7137512, 33 },
    { 0x2bd37512, 90 },
    { 0xafa37513, 90 },
    { 0x9a837511, 70 },
    { 0x1df37511, 33 },
    { 0x82537518, 90 },
    { 0x32937518, 67 },
    { 0xc6f37512, 33 },
    { 0x6ff3751b, 77 },
    { 0xcc53751c, 61 },
    { 0x2f83751a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95237522, 101 },
    { 0, 0 },
    { 0xcb037524, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d23752d, 90 },
    { 0xc2c3752e, 90 },
    { 0x19f3752f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b37535, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd283753e, 33 },
    { 0x6573753f, 61 },
    { 0, 0 },
    { 0x1f437541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc537544, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2937549, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a3754e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d37552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea23755a, 95 },
    { 0x94c3755a, 84 },
    { 0x3263755c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd37560, 90 },
    { 0x4237561, 81 },
    { 0x5f437562, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3737566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8737569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85037570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e37575, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f37578, 34 },
    { 0, 0 },
    { 0x9bc3757a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b437582, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca37588, 33 },
    { 0xfee37589, 93 },
    { 0x7e03758a, 76 },
    { 0xe943758a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe253758e, 90 },
    { 0, 0 },
    { 0xdb737590, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb37593, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5237598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac3759c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1375a4, 34 },
    { 0xc1c375a5, 99 },
    { 0, 0 },
    { 0xfe5375a7, 33 },
    { 0, 0 },
    { 0xb6d375a9, 23 },
    { 0x7a3375aa, 33 },
    { 0, 0 },
    { 0xe31375ac, 66 },
    { 0x342375ac, 98 },
    { 0x5375ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90375b2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae375b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a375bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3375cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff375d5, 90 },
    { 0x46f375d6, 80 },
    { 0x243375d5, 90 },
    { 0x181375d8, 98 },
    { 0x131375d9, 61 },
    { 0x730375da, 90 },
    { 0xc1f375d8, 43 },
    { 0x751375d9, 61 },
    { 0x4b3375dd, 98 },
    { 0xa02375de, 33 },
    { 0x6c9375df, 38 },
    { 0xcf375e0, 90 },
    { 0xa50375dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1375e5, 98 },
    { 0x915375e5, 80 },
    { 0xf49375e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5375ec, 95 },
    { 0x461375ec, 77 },
    { 0xba7375ee, 90 },
    { 0x725375ef, 68 },
    { 0xf76375ee, 78 },
    { 0x65c375f1, 66 },
    { 0x7b7375f0, 33 },
    { 0xcb4375f0, 76 },
    { 0x1a9375ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f375f8, 76 },
    { 0, 0 },
    { 0x4db375fa, 91 },
    { 0x9d8375fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63037601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3760c, 48 },
    { 0xb153760d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71437614, 23 },
    { 0xae237615, 90 },
    { 0x76d37616, 77 },
    { 0, 0 },
    { 0x5ec37618, 90 },
    { 0, 0 },
    { 0x1d53761a, 23 },
    { 0xf5f3761b, 98 },
    { 0, 0 },
    { 0xe663761d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3137620, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81137628, 61 },
    { 0x47d37629, 101 },
    { 0, 0 },
    { 0xfff3762b, 34 },
    { 0, 0 },
    { 0xfa63762d, 95 },
    { 0x2b43762e, 61 },
    { 0, 0 },
    { 0x24a37630, 61 },
    { 0xc7137630, 23 },
    { 0xd5737632, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c37635, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e37639, 90 },
    { 0xb0b3763a, 76 },
    { 0xd933763a, 23 },
    { 0x4a13763b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e37649, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e3764e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3437651, 90 },
    { 0x86537651, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f37655, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49237660, 23 },
    { 0, 0 },
    { 0x71c37662, 95 },
    { 0, 0 },
    { 0xfc737664, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b3766a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7873766d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95737674, 66 },
    { 0x48b37675, 90 },
    { 0x6ba37676, 95 },
    { 0, 0 },
    { 0xafa37678, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf37680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc037686, 61 },
    { 0, 0 },
    { 0x92637688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba3768f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a37693, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa37696, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e37699, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3769d, 90 },
    { 0xc6f3769e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe376a1, 95 },
    { 0x4e7376a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951376a7, 61 },
    { 0x4cd376a7, 23 },
    { 0x309376a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940376ad, 66 },
    { 0, 0 },
    { 0x304376af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b376b7, 98 },
    { 0, 0 },
    { 0x45e376b9, 76 },
    { 0x43376ba, 46 },
    { 0x290376b9, 98 },
    { 0, 0 },
    { 0xf0a376bd, 87 },
    { 0x669376be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9376c4, 93 },
    { 0, 0 },
    { 0x27c376c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8376ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7376cf, 53 },
    { 0, 0 },
    { 0x837376d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x173376d4, 77 },
    { 0x49d376d5, 95 },
    { 0x6f376d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea376d9, 85 },
    { 0, 0 },
    { 0xed8376db, 46 },
    { 0x67c376dc, 64 },
    { 0, 0 },
    { 0xcff376de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d376e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c376e7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4376ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x188376f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e376f8, 90 },
    { 0, 0 },
    { 0x22f376fa, 34 },
    { 0xb8e376fa, 90 },
    { 0x987376fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0376ff, 55 },
    { 0, 0 },
    { 0x2f737701, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4937705, 90 },
    { 0xb1837706, 51 },
    { 0x1e737707, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3770b, 33 },
    { 0, 0 },
    { 0x4b23770d, 33 },
    { 0xec13770d, 61 },
    { 0, 0 },
    { 0xa4a37710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba037719, 90 },
    { 0, 0 },
    { 0xa2b3771b, 93 },
    { 0x32e3771c, 76 },
    { 0, 0 },
    { 0x1593771e, 98 },
    { 0x2753771e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a37724, 76 },
    { 0, 0 },
    { 0xaea37726, 75 },
    { 0x6bf37726, 90 },
    { 0x3ee37728, 23 },
    { 0, 0 },
    { 0x6b43772a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac37730, 91 },
    { 0x6c737731, 83 },
    { 0, 0 },
    { 0x55a37733, 77 },
    { 0x4c037733, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf837738, 55 },
    { 0x4a537739, 24 },
    { 0xe6d3773a, 98 },
    { 0xa7237738, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8537741, 43 },
    { 0, 0 },
    { 0xff537743, 87 },
    { 0xf1937744, 90 },
    { 0, 0 },
    { 0x52237746, 33 },
    { 0, 0 },
    { 0x32337748, 61 },
    { 0x29e37749, 34 },
    { 0x2d437749, 33 },
    { 0xb6737748, 33 },
    { 0, 0 },
    { 0xe893774d, 75 },
    { 0xa413774e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71437753, 78 },
    { 0xab037754, 90 },
    { 0x4a037755, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d537758, 33 },
    { 0xb9637759, 81 },
    { 0x96237759, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d3775d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x537760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80637763, 95 },
    { 0, 0 },
    { 0x7dd37765, 33 },
    { 0xc5237765, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3776a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21037770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2937773, 87 },
    { 0x65037774, 90 },
    { 0, 0 },
    { 0x5d537776, 90 },
    { 0xf7c37777, 61 },
    { 0xfd437778, 95 },
    { 0xb8e37778, 55 },
    { 0x93b3777a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7313777d, 95 },
    { 0xa703777e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f637782, 98 },
    { 0, 0 },
    { 0xe3237784, 76 },
    { 0x33137785, 85 },
    { 0, 0 },
    { 0x8b237787, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5343778f, 90 },
    { 0x2e737790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65337794, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a37797, 43 },
    { 0, 0 },
    { 0xf6637799, 61 },
    { 0x9443779a, 61 },
    { 0x48037799, 85 },
    { 0xb913779a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e3779f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef377a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a377a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585377ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85377b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x254377b6, 93 },
    { 0xd33377b7, 61 },
    { 0, 0 },
    { 0xee2377b9, 90 },
    { 0x7df377ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91377bd, 52 },
    { 0xcd1377be, 33 },
    { 0, 0 },
    { 0x902377c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f377c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0377cc, 93 },
    { 0, 0 },
    { 0xca0377ce, 66 },
    { 0x7cd377cf, 52 },
    { 0x26e377d0, 33 },
    { 0x5de377cf, 52 },
    { 0x50d377d2, 90 },
    { 0x368377d3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60377d6, 95 },
    { 0, 0 },
    { 0x25b377d8, 61 },
    { 0x8a2377d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7377e3, 100 },
    { 0x5a9377e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89377ea, 61 },
    { 0, 0 },
    { 0x4db377ec, 95 },
    { 0xdc9377ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22377f1, 23 },
    { 0x942377f2, 90 },
    { 0x50377f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x407377f6, 23 },
    { 0x24d377f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x250377fa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366377ff, 88 },
    { 0x6ed37800, 33 },
    { 0, 0 },
    { 0xd0c37802, 76 },
    { 0, 0 },
    { 0x78337804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75237808, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb763780b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c3780f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd537812, 33 },
    { 0, 0 },
    { 0xaec37814, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa693781a, 43 },
    { 0x37c3781a, 95 },
    { 0xb303781b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f337820, 90 },
    { 0xc3e37821, 75 },
    { 0xae037822, 98 },
    { 0x33937823, 23 },
    { 0xd6437824, 95 },
    { 0x9af37824, 33 },
    { 0xb8a37823, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2537829, 67 },
    { 0, 0 },
    { 0x623782b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb943782f, 68 },
    { 0x20a3782f, 95 },
    { 0xf0037831, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac37835, 95 },
    { 0x55337836, 91 },
    { 0, 0 },
    { 0xfab37838, 58 },
    { 0x52e37839, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7843783e, 91 },
    { 0x6143783f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0337843, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d37846, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba37849, 98 },
    { 0x44037849, 43 },
    { 0, 0 },
    { 0xc833784c, 76 },
    { 0xbfd3784d, 64 },
    { 0, 0 },
    { 0x3213784f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd737856, 23 },
    { 0x13737857, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd3785a, 33 },
    { 0, 0 },
    { 0x4aa3785c, 98 },
    { 0, 0 },
    { 0x5ee3785e, 90 },
    { 0, 0 },
    { 0xd1437860, 90 },
    { 0x76337860, 95 },
    { 0xb8537862, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf737867, 85 },
    { 0, 0 },
    { 0xa7837869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3786e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b37876, 61 },
    { 0x62137877, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9713787b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a3787e, 67 },
    { 0, 0 },
    { 0xc2c37880, 23 },
    { 0, 0 },
    { 0x6df37882, 98 },
    { 0xaf537883, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7037886, 33 },
    { 0xfea37886, 90 },
    { 0x42d37887, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be3788e, 52 },
    { 0xe23788f, 81 },
    { 0x6353788e, 90 },
    { 0xd5737891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b37897, 98 },
    { 0x3fd37898, 33 },
    { 0xc5e37899, 84 },
    { 0xbdd37899, 95 },
    { 0x1b13789b, 83 },
    { 0xedd3789b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc663789f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7378a4, 61 },
    { 0xd00378a5, 23 },
    { 0, 0 },
    { 0xc4e378a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf378b0, 61 },
    { 0xfe9378b0, 43 },
    { 0xfa6378b2, 95 },
    { 0, 0 },
    { 0x287378b4, 33 },
    { 0, 0 },
    { 0x4e8378b6, 61 },
    { 0, 0 },
    { 0xae5378b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x358378bb, 98 },
    { 0, 0 },
    { 0x72c378bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219378c2, 91 },
    { 0, 0 },
    { 0xca4378c4, 87 },
    { 0, 0 },
    { 0xd72378c6, 93 },
    { 0x9d378c7, 90 },
    { 0, 0 },
    { 0x914378c9, 82 },
    { 0x259378c9, 90 },
    { 0x3d9378cb, 99 },
    { 0xa9a378c9, 23 },
    { 0x5e1378c9, 85 },
    { 0x6ec378ce, 90 },
    { 0x6f5378cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35378d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb378d7, 61 },
    { 0x96378d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624378de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77378e4, 75 },
    { 0x388378e5, 35 },
    { 0x215378e6, 23 },
    { 0xeb0378e7, 78 },
    { 0x645378e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227378ec, 52 },
    { 0, 0 },
    { 0x5f1378ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe378f1, 33 },
    { 0x5a7378f1, 95 },
    { 0xaf8378f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a378f9, 23 },
    { 0, 0 },
    { 0x153378fb, 23 },
    { 0x67a378fb, 95 },
    { 0x240378fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17737903, 99 },
    { 0x1ee37904, 98 },
    { 0, 0 },
    { 0xd4c37906, 69 },
    { 0, 0 },
    { 0x30c37908, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad37910, 90 },
    { 0x7f37911, 66 },
    { 0x6d437912, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c537916, 90 },
    { 0xd6b37917, 88 },
    { 0xd4b37918, 34 },
    { 0xc1637916, 90 },
    { 0, 0 },
    { 0x4aa3791b, 58 },
    { 0x663791b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b137920, 98 },
    { 0, 0 },
    { 0x18a37922, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f37925, 90 },
    { 0, 0 },
    { 0x69f37927, 98 },
    { 0xbc837928, 33 },
    { 0xf1037929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa123792c, 95 },
    { 0, 0 },
    { 0x5de3792e, 91 },
    { 0, 0 },
    { 0xf7837930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a37934, 66 },
    { 0, 0 },
    { 0xe1737936, 23 },
    { 0, 0 },
    { 0x5d37938, 90 },
    { 0xb0d37939, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f13793d, 43 },
    { 0xfaf3793e, 90 },
    { 0, 0 },
    { 0x93837940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9737943, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b37947, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3683794d, 95 },
    { 0x6293794e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce37951, 36 },
    { 0xc837951, 76 },
    { 0xd0e37953, 95 },
    { 0x20b37952, 33 },
    { 0xb2837951, 23 },
    { 0x3ed37956, 76 },
    { 0x9e037952, 61 },
    { 0, 0 },
    { 0x4f237959, 90 },
    { 0, 0 },
    { 0xec73795b, 33 },
    { 0x7433795c, 100 },
    { 0x7f03795d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb037960, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc537966, 34 },
    { 0x69e37966, 90 },
    { 0x27137968, 80 },
    { 0xea337966, 55 },
    { 0x72c3796a, 95 },
    { 0, 0 },
    { 0x553796c, 91 },
    { 0, 0 },
    { 0xa0b3796e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d37971, 90 },
    { 0x13837972, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55037976, 95 },
    { 0xb3f37976, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde13797b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae63797e, 90 },
    { 0xde53797f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60937986, 76 },
    { 0x46237987, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb3798e, 33 },
    { 0x6523798f, 95 },
    { 0x23b37990, 33 },
    { 0x5e73798e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f337994, 99 },
    { 0, 0 },
    { 0x7f837996, 90 },
    { 0, 0 },
    { 0xcde37998, 95 },
    { 0, 0 },
    { 0xaf43799a, 52 },
    { 0x6893799b, 95 },
    { 0x96e3799c, 67 },
    { 0xee83799d, 87 },
    { 0xaf33799a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0379a2, 23 },
    { 0, 0 },
    { 0x5bd379a4, 100 },
    { 0x655379a5, 87 },
    { 0x918379a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df379aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c379ad, 90 },
    { 0, 0 },
    { 0x815379af, 33 },
    { 0x1b7379b0, 99 },
    { 0x589379b0, 17 },
    { 0xe57379af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c379b6, 55 },
    { 0, 0 },
    { 0xe29379b8, 53 },
    { 0, 0 },
    { 0x53e379ba, 17 },
    { 0x6da379ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46379be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321379c8, 93 },
    { 0xeb5379c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21379cc, 95 },
    { 0x111379cd, 61 },
    { 0xbf4379ce, 97 },
    { 0x664379ce, 95 },
    { 0x44d379cd, 96 },
    { 0x441379d1, 90 },
    { 0, 0 },
    { 0xc43379d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6379da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290379de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27379e2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221379e6, 90 },
    { 0x869379e6, 76 },
    { 0x1a5379e8, 98 },
    { 0, 0 },
    { 0x609379ea, 33 },
    { 0x1ad379ea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770379f2, 88 },
    { 0x5e0379f2, 43 },
    { 0x618379f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0379f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef37a02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea37a07, 98 },
    { 0, 0 },
    { 0x26a37a09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8537a0c, 95 },
    { 0, 0 },
    { 0x72137a0e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81737a12, 61 },
    { 0x43237a13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c37a17, 43 },
    { 0, 0 },
    { 0x29d37a19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2737a1c, 81 },
    { 0, 0 },
    { 0x46737a1e, 72 },
    { 0, 0 },
    { 0xc8537a20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e537a26, 95 },
    { 0x50837a27, 55 },
    { 0xe2b37a28, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d37a2b, 33 },
    { 0, 0 },
    { 0x36237a2d, 23 },
    { 0, 0 },
    { 0x4337a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23437a36, 61 },
    { 0, 0 },
    { 0x20137a38, 76 },
    { 0x2537a39, 23 },
    { 0x97937a3a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9637a3d, 61 },
    { 0, 0 },
    { 0x71037a3f, 78 },
    { 0, 0 },
    { 0x59437a41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62437a45, 90 },
    { 0x69c37a45, 55 },
    { 0x77937a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef837a4d, 98 },
    { 0xbb937a4e, 61 },
    { 0x74537a4d, 55 },
    { 0, 0 },
    { 0xb2637a51, 76 },
    { 0xe537a51, 55 },
    { 0x97637a52, 78 },
    { 0xb9a37a54, 33 },
    { 0x3d737a54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22837a5b, 76 },
    { 0, 0 },
    { 0xc5b37a5d, 90 },
    { 0, 0 },
    { 0x77737a5f, 34 },
    { 0x3f337a60, 90 },
    { 0x6937a61, 95 },
    { 0x9ae37a62, 33 },
    { 0xcd137a5f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c737a66, 90 },
    { 0xecb37a66, 55 },
    { 0x14337a66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7537a6c, 87 },
    { 0xd3237a6d, 98 },
    { 0xe0537a6e, 91 },
    { 0, 0 },
    { 0x14337a70, 90 },
    { 0x6b537a71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19237a78, 98 },
    { 0, 0 },
    { 0xdf837a7a, 98 },
    { 0x4a37a7b, 46 },
    { 0, 0 },
    { 0x19a37a7d, 98 },
    { 0xc4e37a7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77137a81, 33 },
    { 0, 0 },
    { 0x24637a83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51637a88, 98 },
    { 0, 0 },
    { 0xf1037a8a, 61 },
    { 0, 0 },
    { 0xc2e37a8c, 46 },
    { 0x46237a8c, 90 },
    { 0xe1c37a8c, 23 },
    { 0xe2e37a8f, 90 },
    { 0xe8737a90, 93 },
    { 0, 0 },
    { 0x2b37a92, 93 },
    { 0, 0 },
    { 0x3ea37a94, 91 },
    { 0x5c37a95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0137a9c, 23 },
    { 0x38537a9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3037aa6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b37aa9, 61 },
    { 0, 0 },
    { 0xcd237aab, 81 },
    { 0xf9137aac, 78 },
    { 0xea637aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1537ab0, 23 },
    { 0xb4337ab1, 84 },
    { 0, 0 },
    { 0xb3a37ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4537ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0637abc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97737ac1, 92 },
    { 0xc1437ac2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db37ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58737ac8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x98637acb, 33 },
    { 0xdd437acc, 95 },
    { 0x78437acc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa37ad1, 98 },
    { 0x54637ad2, 34 },
    { 0x5e437ad3, 76 },
    { 0x44537ad2, 95 },
    { 0xe2437ad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e237ad9, 93 },
    { 0x39c37ad9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2237ade, 46 },
    { 0x69837adf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12137ae5, 61 },
    { 0xbf537ae6, 34 },
    { 0xed737ae5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe37aeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76937af2, 67 },
    { 0xcd637af3, 61 },
    { 0, 0 },
    { 0x10037af5, 23 },
    { 0, 0 },
    { 0xed037af7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e837afa, 90 },
    { 0x3dc37afa, 34 },
    { 0, 0 },
    { 0x83637afd, 33 },
    { 0x63637afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e37b01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f437b04, 61 },
    { 0, 0 },
    { 0x8f37b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64037b0d, 33 },
    { 0x7ce37b0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a337b1a, 90 },
    { 0, 0 },
    { 0xad337b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b37b1f, 43 },
    { 0, 0 },
    { 0x30937b21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35537b28, 61 },
    { 0xc4637b29, 76 },
    { 0xa9937b2a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e37b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b37b34, 95 },
    { 0xc2037b34, 23 },
    { 0, 0 },
    { 0x47d37b37, 95 },
    { 0xe9637b38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d37b3b, 78 },
    { 0x3b437b3c, 23 },
    { 0xc2237b3d, 23 },
    { 0x6de37b3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c937b41, 66 },
    { 0, 0 },
    { 0xee37b43, 96 },
    { 0, 0 },
    { 0x95837b45, 58 },
    { 0x32237b45, 23 },
    { 0xb8937b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3637b4c, 90 },
    { 0, 0 },
    { 0x4ab37b4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5037b52, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75137b58, 61 },
    { 0xcdd37b58, 87 },
    { 0, 0 },
    { 0x3c137b5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59837b68, 95 },
    { 0, 0 },
    { 0x83d37b6a, 98 },
    { 0x21137b6b, 61 },
    { 0, 0 },
    { 0x26a37b6d, 61 },
    { 0xa7b37b6e, 55 },
    { 0x3f037b6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac37b77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e737b7b, 53 },
    { 0, 0 },
    { 0x39137b7d, 46 },
    { 0xf9437b7e, 33 },
    { 0, 0 },
    { 0x137b80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f37b83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be37b87, 98 },
    { 0xffd37b88, 100 },
    { 0x21337b88, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95137b93, 66 },
    { 0, 0 },
    { 0xd737b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea137b9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe37b9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd37ba7, 98 },
    { 0x5d037ba7, 98 },
    { 0xbd537ba9, 99 },
    { 0xc8737baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c537bb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d437bb5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a37bbb, 88 },
    { 0, 0 },
    { 0x5c437bbd, 98 },
    { 0xcf137bbe, 84 },
    { 0xb9337bbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437bc2, 33 },
    { 0x30b37bc3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b37bcb, 95 },
    { 0x3f337bcb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9337bd1, 84 },
    { 0xa7f37bd2, 56 },
    { 0xec537bd1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7837bd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc137be4, 33 },
    { 0x7d337be4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef37bec, 69 },
    { 0x8c337bed, 23 },
    { 0xb4f37bed, 90 },
    { 0, 0 },
    { 0xc4b37bf0, 90 },
    { 0, 0 },
    { 0xe0b37bf2, 66 },
    { 0x85b37bf3, 65 },
    { 0x92237bf3, 55 },
    { 0x81b37bf2, 90 },
    { 0, 0 },
    { 0x11b37bf7, 17 },
    { 0, 0 },
    { 0x9d537bf9, 90 },
    { 0xc5d37bf9, 90 },
    { 0, 0 },
    { 0x95c37bfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b37c00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c237c05, 46 },
    { 0, 0 },
    { 0x8f237c07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c637c0a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7937c0f, 33 },
    { 0xa9f37c10, 33 },
    { 0, 0 },
    { 0x40237c12, 61 },
    { 0xbdf37c13, 88 },
    { 0x36d37c12, 33 },
    { 0, 0 },
    { 0x29537c16, 23 },
    { 0, 0 },
    { 0xac337c18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed37c1b, 61 },
    { 0x26737c1c, 90 },
    { 0xd2837c1d, 33 },
    { 0xf2737c1d, 100 },
    { 0x5be37c1f, 99 },
    { 0x58c37c1d, 99 },
    { 0xa6037c1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b937c25, 98 },
    { 0xa8137c26, 90 },
    { 0xa8937c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5437c2b, 90 },
    { 0x5737c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d37c34, 55 },
    { 0, 0 },
    { 0x79c37c36, 23 },
    { 0xd3637c36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c37c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b837c42, 95 },
    { 0x1cf37c43, 78 },
    { 0xcd837c43, 98 },
    { 0x7a637c43, 23 },
    { 0x8fd37c45, 98 },
    { 0, 0 },
    { 0x8437c48, 55 },
    { 0x2d837c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a37c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90237c5b, 67 },
    { 0x98837c5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28837c60, 98 },
    { 0, 0 },
    { 0x1a037c62, 33 },
    { 0, 0 },
    { 0xbd37c64, 90 },
    { 0xd4437c65, 55 },
    { 0x19b37c64, 43 },
    { 0x98237c65, 76 },
    { 0, 0 },
    { 0xbda37c69, 98 },
    { 0x2db37c6a, 55 },
    { 0, 0 },
    { 0x44037c6c, 43 },
    { 0xf4737c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7337c70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a037c78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc37c7b, 95 },
    { 0xec037c7c, 33 },
    { 0x49f37c7c, 76 },
    { 0, 0 },
    { 0xad437c7f, 84 },
    { 0x6fe37c80, 61 },
    { 0x12a37c7f, 93 },
    { 0x3fe37c82, 17 },
    { 0xae237c83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a37c88, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77637c8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5937c8f, 90 },
    { 0, 0 },
    { 0x52637c91, 90 },
    { 0x15737c92, 99 },
    { 0xe4237c92, 84 },
    { 0, 0 },
    { 0xeb337c95, 23 },
    { 0xf1e37c96, 43 },
    { 0xace37c96, 95 },
    { 0x1de37c98, 55 },
    { 0xefc37c98, 67 },
    { 0x50337c99, 4 },
    { 0x33537c98, 95 },
    { 0x92337c9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b37c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef937ca4, 55 },
    { 0x30c37ca5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x29637ca8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b137cad, 23 },
    { 0xf2237cae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56937cb3, 33 },
    { 0xda037cb4, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b37cb9, 66 },
    { 0xf2f37cb9, 95 },
    { 0x32237cbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd137cbf, 66 },
    { 0xafa37cbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40337cc8, 95 },
    { 0x28037cc9, 76 },
    { 0x12e37cc9, 75 },
    { 0x7be37cc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b737cce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd737cd1, 33 },
    { 0x81237cd2, 23 },
    { 0, 0 },
    { 0xd5c37cd4, 93 },
    { 0xa7937cd5, 67 },
    { 0, 0 },
    { 0xec937cd7, 33 },
    { 0x6c237cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d37ce0, 90 },
    { 0x3be37ce0, 95 },
    { 0x4f937ce2, 93 },
    { 0x537ce0, 34 },
    { 0x83b37ce4, 99 },
    { 0, 0 },
    { 0x20837ce6, 90 },
    { 0x3037ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81337cee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f937cf1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2737cf4, 43 },
    { 0xf0137cf5, 23 },
    { 0x53237cf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1237cfb, 98 },
    { 0xb3b37cfb, 67 },
    { 0x9b37cfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5337d02, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8037d08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc37d0e, 90 },
    { 0, 0 },
    { 0xa8837d10, 90 },
    { 0xe4137d10, 90 },
    { 0xc8437d11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4737d15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc837d1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42037d20, 61 },
    { 0x6e837d20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e37d24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc937d27, 55 },
    { 0x7b237d28, 23 },
    { 0x1637d29, 99 },
    { 0x40137d2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66537d2e, 95 },
    { 0x61b37d2f, 23 },
    { 0x13f37d2e, 61 },
    { 0xb8337d31, 93 },
    { 0x89737d32, 38 },
    { 0xfbc37d30, 66 },
    { 0x65837d34, 98 },
    { 0x26d37d34, 85 },
    { 0x9537d36, 88 },
    { 0xd0137d34, 98 },
    { 0x81737d38, 23 },
    { 0x93c37d34, 90 },
    { 0, 0 },
    { 0x1b837d3b, 98 },
    { 0x44d37d3c, 61 },
    { 0, 0 },
    { 0x80337d3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea37d44, 33 },
    { 0xb8d37d45, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b37d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e237d4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc37d52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac37d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc37d5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69737d66, 76 },
    { 0x77737d67, 33 },
    { 0xa0b37d68, 96 },
    { 0xb5b37d66, 66 },
    { 0, 0 },
    { 0xae837d6b, 90 },
    { 0, 0 },
    { 0xdaa37d6d, 94 },
    { 0, 0 },
    { 0xd1337d6f, 98 },
    { 0x56537d6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb37d75, 98 },
    { 0xd6e37d76, 33 },
    { 0, 0 },
    { 0x65a37d78, 81 },
    { 0xbec37d79, 90 },
    { 0, 0 },
    { 0x88237d7b, 100 },
    { 0xa0737d7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b37d80, 43 },
    { 0x9e37d81, 61 },
    { 0x74037d82, 23 },
    { 0xdbb37d81, 47 },
    { 0, 0 },
    { 0x84d37d85, 90 },
    { 0xab637d85, 76 },
    { 0, 0 },
    { 0x35f37d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8437d8e, 99 },
    { 0x7f737d8f, 46 },
    { 0x5c637d90, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50237da3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e337da8, 23 },
    { 0x66537da8, 90 },
    { 0xef937da9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9137db1, 84 },
    { 0x5c837db2, 52 },
    { 0x8ca37db3, 53 },
    { 0x62137db2, 91 },
    { 0x95937db5, 95 },
    { 0x8e637db6, 90 },
    { 0xaa237db5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf037dbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa37dc2, 61 },
    { 0x5ff37dc3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f37dc9, 66 },
    { 0x13c37dca, 78 },
    { 0xac237dc9, 91 },
    { 0x87a37dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f37dcf, 33 },
    { 0xe4837dd0, 46 },
    { 0x64c37dcf, 23 },
    { 0xe6337dd2, 23 },
    { 0x2dc37dd3, 33 },
    { 0x33337dd4, 43 },
    { 0x70d37dd4, 66 },
    { 0x11a37dd2, 75 },
    { 0x4cf37dd3, 55 },
    { 0, 0 },
    { 0x73337dd9, 66 },
    { 0x92d37dd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25537ddd, 33 },
    { 0x90437dde, 33 },
    { 0, 0 },
    { 0x1b837de0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59637de4, 34 },
    { 0, 0 },
    { 0x48b37de6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6937dee, 66 },
    { 0, 0 },
    { 0xa1a37df0, 95 },
    { 0xe5537df1, 98 },
    { 0x9fe37df2, 34 },
    { 0x83f37df3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d037df6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be37dfa, 61 },
    { 0, 0 },
    { 0x20c37dfc, 90 },
    { 0x2f637dfd, 90 },
    { 0xcf137dfc, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf37e04, 85 },
    { 0xb1537e04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb937e08, 33 },
    { 0, 0 },
    { 0x77d37e0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d37e10, 98 },
    { 0x2af37e11, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe37e15, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c837e1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b37e1e, 33 },
    { 0x2f837e1f, 23 },
    { 0xaa537e1f, 76 },
    { 0x3e137e1f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc237e27, 27 },
    { 0x4cc37e27, 90 },
    { 0x7e537e29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16337e2d, 90 },
    { 0xa0a37e2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b37e31, 89 },
    { 0x30f37e32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f37e35, 91 },
    { 0x4a137e35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b137e3a, 65 },
    { 0x5bd37e3a, 90 },
    { 0x2ed37e3c, 33 },
    { 0x5cf37e3c, 88 },
    { 0x35c37e3a, 91 },
    { 0x3f337e3d, 61 },
    { 0xfe237e3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c37e43, 87 },
    { 0, 0 },
    { 0xab637e45, 90 },
    { 0xbf637e45, 23 },
    { 0xacd37e45, 55 },
    { 0x9af37e48, 95 },
    { 0x7ec37e49, 23 },
    { 0, 0 },
    { 0xd1837e4b, 95 },
    { 0, 0 },
    { 0x26237e4d, 33 },
    { 0x93737e4e, 46 },
    { 0xfba37e4d, 17 },
    { 0x50e37e50, 61 },
    { 0x4f37e4d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa37e54, 85 },
    { 0x5e737e55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x54337e58, 90 },
    { 0, 0 },
    { 0x6dd37e5a, 61 },
    { 0, 0 },
    { 0x6e637e5c, 66 },
    { 0, 0 },
    { 0x93237e5e, 23 },
    { 0x21237e5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44237e65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d237e6c, 64 },
    { 0xc1f37e6d, 33 },
    { 0x50937e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30837e77, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f37e7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e37e7f, 23 },
    { 0xbcf37e80, 90 },
    { 0xdc137e81, 98 },
    { 0xbb437e81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xded37e85, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab37e88, 93 },
    { 0x12737e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57537e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1037e9b, 33 },
    { 0xd0437e9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae437ea3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8237ea7, 38 },
    { 0, 0 },
    { 0x6fe37ea9, 23 },
    { 0, 0 },
    { 0xac537eab, 95 },
    { 0, 0 },
    { 0xbdc37ead, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x89437eb0, 55 },
    { 0, 0 },
    { 0x9d337eb2, 87 },
    { 0, 0 },
    { 0x1b37eb4, 87 },
    { 0xc9a37eb5, 61 },
    { 0x7f437eb6, 55 },
    { 0x6a37eb6, 34 },
    { 0xcbd37eb8, 98 },
    { 0x86237eb7, 46 },
    { 0x25737eb4, 91 },
    { 0x58b37eb4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f37ec6, 43 },
    { 0, 0 },
    { 0x3b237ec8, 46 },
    { 0x9f537ec8, 61 },
    { 0x7a737ec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9237ecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c137ed2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23537ed5, 34 },
    { 0, 0 },
    { 0xf5d37ed7, 46 },
    { 0x5a637ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92437edd, 61 },
    { 0x2b437edd, 95 },
    { 0x7b537edf, 55 },
    { 0, 0 },
    { 0x6f37ee1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb037ee5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d537eeb, 95 },
    { 0xb8b37eeb, 33 },
    { 0xe4d37eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74937ef3, 55 },
    { 0xc337ef4, 76 },
    { 0, 0 },
    { 0xa7337ef6, 33 },
    { 0x92a37ef7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf837efb, 78 },
    { 0x8a037efc, 95 },
    { 0, 0 },
    { 0xa3f37efe, 33 },
    { 0x13137eff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb137f02, 90 },
    { 0x58a37f02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58037f0d, 76 },
    { 0, 0 },
    { 0xb4037f0f, 17 },
    { 0x70637f0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab37f14, 33 },
    { 0x57137f15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad37f1d, 93 },
    { 0xaa637f1d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d937f24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95337f2a, 66 },
    { 0xc137f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f37f2e, 80 },
    { 0x47d37f2f, 61 },
    { 0x49337f30, 93 },
    { 0xed337f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f37f35, 95 },
    { 0x66337f36, 23 },
    { 0x21c37f37, 66 },
    { 0x37f37f36, 98 },
    { 0x49137f39, 98 },
    { 0x2d437f3a, 66 },
    { 0x15b37f3a, 34 },
    { 0xcc537f3c, 23 },
    { 0xc1437f3a, 33 },
    { 0xff837f3a, 95 },
    { 0xe7e37f3f, 98 },
    { 0x7e337f3e, 33 },
    { 0x6ed37f41, 33 },
    { 0x78937f42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6737f45, 23 },
    { 0xcfd37f45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c37f4c, 95 },
    { 0, 0 },
    { 0x91437f4e, 66 },
    { 0x90037f4e, 95 },
    { 0xd037f4e, 61 },
    { 0xa2d37f51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc37f5a, 69 },
    { 0x8d337f5b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29037f64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e837f6a, 43 },
    { 0x4d437f6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x61937f6e, 17 },
    { 0x50537f6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc737f77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7037f7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2637f82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d037f88, 23 },
    { 0, 0 },
    { 0x6b37f8a, 23 },
    { 0x87337f8b, 99 },
    { 0x6cb37f8b, 95 },
    { 0x1e137f8d, 77 },
    { 0xc4b37f8c, 98 },
    { 0xf8037f8f, 34 },
    { 0, 0 },
    { 0xc4737f91, 67 },
    { 0x9d637f91, 95 },
    { 0xc6437f93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c37f99, 76 },
    { 0, 0 },
    { 0x72f37f9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5337f9e, 34 },
    { 0, 0 },
    { 0xffb37fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51837fa7, 90 },
    { 0, 0 },
    { 0x54237fa9, 66 },
    { 0x6c137faa, 33 },
    { 0x85037fab, 87 },
    { 0xbe637fab, 93 },
    { 0xe7237fad, 95 },
    { 0xcf37fae, 23 },
    { 0x9fd37faf, 67 },
    { 0x67a37fb0, 66 },
    { 0xa1c37fb0, 95 },
    { 0x2e37fae, 51 },
    { 0, 0 },
    { 0x97b37fb4, 90 },
    { 0xf4f37fb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e37fbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32237fbe, 98 },
    { 0x41237fbf, 46 },
    { 0x60a37fc0, 95 },
    { 0, 0 },
    { 0xd8337fc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4237fc7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c237fca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a737fcd, 98 },
    { 0x8bd37fce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a37fd4, 77 },
    { 0x91237fd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b37fdc, 95 },
    { 0x43b37fdd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a37fe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a37fe5, 98 },
    { 0, 0 },
    { 0x64d37fe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x68237fea, 90 },
    { 0xf6a37feb, 88 },
    { 0x6db37fea, 33 },
    { 0x51337fed, 85 },
    { 0x64037fed, 85 },
    { 0, 0 },
    { 0x50437ff0, 95 },
    { 0xde637ff1, 23 },
    { 0xb1837ff1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32137ff5, 43 },
    { 0x62f37ff6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0237ff9, 33 },
    { 0xbb637ffa, 90 },
    { 0xb9337ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2438004, 34 },
    { 0x16138004, 23 },
    { 0, 0 },
    { 0x99338007, 93 },
    { 0x9b338008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe723800b, 76 },
    { 0x9943800b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a23800f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7338012, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x52138015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25938018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72738020, 66 },
    { 0x95e38021, 99 },
    { 0xf4438020, 98 },
    { 0x88a38023, 34 },
    { 0, 0 },
    { 0x23b38025, 61 },
    { 0x27638026, 98 },
    { 0x73d38027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f3802c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87538034, 88 },
    { 0x81838035, 69 },
    { 0x37438036, 95 },
    { 0xba038034, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc3803f, 78 },
    { 0xab938040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8138043, 33 },
    { 0, 0 },
    { 0xe1138045, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed38048, 90 },
    { 0, 0 },
    { 0x9123804a, 76 },
    { 0x6c43804a, 90 },
    { 0, 0 },
    { 0x2833804d, 81 },
    { 0x26f3804e, 61 },
    { 0, 0 },
    { 0x23b38050, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e138054, 33 },
    { 0x54a38055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f3805a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a38063, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d38066, 46 },
    { 0x3bd38066, 76 },
    { 0x3a538066, 52 },
    { 0, 0 },
    { 0x64c3806a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14738073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b738078, 90 },
    { 0x67538079, 86 },
    { 0x2173807a, 33 },
    { 0x46b38078, 84 },
    { 0x4773807c, 34 },
    { 0x1173807a, 76 },
    { 0, 0 },
    { 0x7593807f, 90 },
    { 0xb7938080, 23 },
    { 0x68d38081, 90 },
    { 0x7ea38082, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d38088, 61 },
    { 0x85838088, 100 },
    { 0x7783808a, 98 },
    { 0x21d3808b, 33 },
    { 0, 0 },
    { 0xcc33808d, 95 },
    { 0x5e63808e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1138091, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2838095, 69 },
    { 0, 0 },
    { 0x1c338097, 33 },
    { 0, 0 },
    { 0xcdf38099, 67 },
    { 0x9b33809a, 98 },
    { 0x79b38099, 98 },
    { 0x6863809c, 34 },
    { 0x7db3809c, 76 },
    { 0, 0 },
    { 0x673809f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8380a2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e380a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0380aa, 85 },
    { 0, 0 },
    { 0x59380ac, 61 },
    { 0x733380ad, 33 },
    { 0xe8b380ac, 33 },
    { 0xa12380af, 90 },
    { 0x70e380af, 98 },
    { 0x165380ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a380b6, 84 },
    { 0, 0 },
    { 0xf4380b8, 61 },
    { 0x429380b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e380bc, 66 },
    { 0x597380bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x688380c0, 61 },
    { 0x4a5380c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37380c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b380cc, 76 },
    { 0x998380cd, 85 },
    { 0, 0 },
    { 0x609380cf, 17 },
    { 0xa4b380cf, 33 },
    { 0, 0 },
    { 0x9e6380d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88380d6, 55 },
    { 0xa87380d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450380de, 95 },
    { 0, 0 },
    { 0x2a1380e0, 93 },
    { 0, 0 },
    { 0xced380e2, 95 },
    { 0x364380e3, 93 },
    { 0xf8d380e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5380e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73380ed, 76 },
    { 0xe8d380ee, 90 },
    { 0x203380ef, 99 },
    { 0xc41380f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x923380f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c380fa, 23 },
    { 0, 0 },
    { 0x3ab380fc, 76 },
    { 0, 0 },
    { 0xeac380fe, 98 },
    { 0xc00380fe, 33 },
    { 0x8dc38100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6938104, 17 },
    { 0xfbf38105, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df38110, 90 },
    { 0, 0 },
    { 0xb1538112, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed38115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71338119, 37 },
    { 0, 0 },
    { 0xa63811b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd38123, 75 },
    { 0xf9a38124, 33 },
    { 0, 0 },
    { 0xc2e38126, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b33812a, 40 },
    { 0x4cd3812b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4953812e, 90 },
    { 0, 0 },
    { 0xe9738130, 52 },
    { 0xa5c38130, 65 },
    { 0, 0 },
    { 0xd738133, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f23813a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e38140, 66 },
    { 0, 0 },
    { 0xdb138142, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f638145, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33338148, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7703814d, 33 },
    { 0xbc63814e, 33 },
    { 0x6303814f, 90 },
    { 0xeb3814f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893815d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f38165, 95 },
    { 0x9d38166, 98 },
    { 0x67e38165, 87 },
    { 0x9d638168, 95 },
    { 0x2ec38168, 67 },
    { 0, 0 },
    { 0x9703816b, 43 },
    { 0xfb83816b, 33 },
    { 0, 0 },
    { 0x3d53816e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b38175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4083817a, 23 },
    { 0, 0 },
    { 0x8213817c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e38180, 23 },
    { 0x65338180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd133818a, 46 },
    { 0x6483818a, 61 },
    { 0x1103818b, 39 },
    { 0x7273818d, 61 },
    { 0xb1f3818d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32438195, 33 },
    { 0xa0b38196, 43 },
    { 0x2ae38197, 61 },
    { 0x88d38198, 90 },
    { 0x5b938199, 52 },
    { 0x9b738198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a13819f, 68 },
    { 0x622381a0, 46 },
    { 0x96a381a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x494381a4, 33 },
    { 0xc4f381a4, 90 },
    { 0x1f6381a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8381a9, 98 },
    { 0, 0 },
    { 0xc1b381ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6381b3, 61 },
    { 0xdfa381b3, 77 },
    { 0, 0 },
    { 0x751381b6, 90 },
    { 0x4df381b7, 33 },
    { 0x448381b7, 33 },
    { 0x66e381b6, 90 },
    { 0x598381b8, 90 },
    { 0xc06381bb, 52 },
    { 0x4f4381bc, 90 },
    { 0xb05381bd, 33 },
    { 0x5c381be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f381c3, 33 },
    { 0x8b0381c3, 34 },
    { 0x8b381c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59381cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c381ce, 36 },
    { 0x4a5381cf, 66 },
    { 0x412381cf, 95 },
    { 0xeee381d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489381d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5381dd, 95 },
    { 0x333381dd, 48 },
    { 0, 0 },
    { 0xcc3381e0, 17 },
    { 0xe06381e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780381e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852381ee, 23 },
    { 0xf7c381ee, 43 },
    { 0x478381ef, 33 },
    { 0xe7381f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43381f5, 99 },
    { 0xa1a381f6, 61 },
    { 0x898381f7, 90 },
    { 0xa4b381f8, 90 },
    { 0xaee381f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831381fe, 43 },
    { 0xb4b381ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba538203, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb238206, 93 },
    { 0, 0 },
    { 0x9b438208, 46 },
    { 0x2e238208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd3820c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd38210, 33 },
    { 0x85e38210, 90 },
    { 0, 0 },
    { 0x3fb38213, 38 },
    { 0xbcd38214, 90 },
    { 0x74538215, 61 },
    { 0x4d738216, 43 },
    { 0xaee38216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb3821b, 33 },
    { 0x7853821c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf038220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d38223, 34 },
    { 0x49038223, 23 },
    { 0x12838223, 98 },
    { 0x2b938224, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713822c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92738238, 71 },
    { 0x16838239, 61 },
    { 0x9253823a, 77 },
    { 0xe438239, 40 },
    { 0xba03823c, 61 },
    { 0x7bc3823d, 61 },
    { 0xe4c38239, 33 },
    { 0xd023823f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e638243, 91 },
    { 0x7c238244, 34 },
    { 0x31438243, 23 },
    { 0x40638244, 90 },
    { 0x13138246, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38238251, 93 },
    { 0, 0 },
    { 0x88438253, 23 },
    { 0x3ae38253, 99 },
    { 0xd6538255, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb738258, 66 },
    { 0, 0 },
    { 0x1423825a, 64 },
    { 0xf543825b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f3825e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9238261, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c638267, 34 },
    { 0xb7438268, 76 },
    { 0, 0 },
    { 0xfef3826a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x933826d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x48038270, 61 },
    { 0, 0 },
    { 0xd1c38272, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c3827b, 98 },
    { 0x66d3827c, 34 },
    { 0x20d3827b, 33 },
    { 0, 0 },
    { 0xddb3827f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64438283, 61 },
    { 0xfe438284, 33 },
    { 0x7ae38284, 33 },
    { 0x8ea38286, 80 },
    { 0x4ae38286, 90 },
    { 0x20138288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4033828d, 66 },
    { 0x6f63828d, 98 },
    { 0x14f3828d, 61 },
    { 0x38d38290, 98 },
    { 0x96d3828f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f38294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d538299, 95 },
    { 0, 0 },
    { 0x1d63829b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2193829f, 85 },
    { 0xcb23829f, 66 },
    { 0xbe3382a1, 84 },
    { 0, 0 },
    { 0x753382a3, 55 },
    { 0x677382a4, 33 },
    { 0x94c382a5, 84 },
    { 0, 0 },
    { 0xbb6382a7, 76 },
    { 0x4e9382a8, 34 },
    { 0x754382a7, 91 },
    { 0x814382a9, 61 },
    { 0x5ad382ab, 90 },
    { 0x59f382ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8382af, 88 },
    { 0x7c382b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8382b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3382b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b382ba, 95 },
    { 0x50d382ba, 78 },
    { 0xe382bc, 99 },
    { 0, 0 },
    { 0x56e382be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48382c7, 23 },
    { 0xdc4382c7, 61 },
    { 0xbd5382c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4382cc, 91 },
    { 0x61382cc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x502382d0, 90 },
    { 0, 0 },
    { 0xa48382d2, 90 },
    { 0x872382d3, 90 },
    { 0xe20382d3, 90 },
    { 0, 0 },
    { 0x257382d6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d382da, 88 },
    { 0xee2382db, 37 },
    { 0xc51382da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25382e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x545382e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b382e7, 87 },
    { 0, 0 },
    { 0x5cc382e9, 99 },
    { 0xee5382ea, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9382ed, 69 },
    { 0, 0 },
    { 0xa3e382ef, 100 },
    { 0x4cc382f0, 98 },
    { 0, 0 },
    { 0xe3a382f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7382f5, 23 },
    { 0, 0 },
    { 0x904382f7, 61 },
    { 0x659382f7, 34 },
    { 0x68382f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec382fc, 33 },
    { 0x76e382fc, 95 },
    { 0x475382fe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74238303, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a38307, 23 },
    { 0xd238308, 93 },
    { 0xa4338309, 87 },
    { 0x6fd38309, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb38312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a738315, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f38319, 46 },
    { 0, 0 },
    { 0x603831b, 90 },
    { 0, 0 },
    { 0xe903831d, 23 },
    { 0x5103831e, 19 },
    { 0x8513831f, 98 },
    { 0x86038320, 61 },
    { 0, 0 },
    { 0x98238322, 98 },
    { 0x3e138323, 93 },
    { 0x5f038322, 66 },
    { 0xb0b38324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd083832a, 52 },
    { 0, 0 },
    { 0x2073832c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1038330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9838333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae38338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d23833c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63238341, 90 },
    { 0, 0 },
    { 0x22538343, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4238348, 37 },
    { 0x88c38349, 98 },
    { 0, 0 },
    { 0xaf03834b, 98 },
    { 0, 0 },
    { 0x4283834d, 90 },
    { 0x8193834e, 76 },
    { 0x2853834d, 95 },
    { 0, 0 },
    { 0x82438351, 90 },
    { 0xce38352, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b38355, 23 },
    { 0xc2b38356, 95 },
    { 0x1da38356, 23 },
    { 0x38838355, 90 },
    { 0, 0 },
    { 0xcdd3835a, 52 },
    { 0x4553835b, 98 },
    { 0x2083835c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c3835f, 66 },
    { 0xf9d38360, 95 },
    { 0xa8938361, 91 },
    { 0, 0 },
    { 0x6538363, 98 },
    { 0xd8a38364, 61 },
    { 0x82e38365, 33 },
    { 0xa0d38366, 90 },
    { 0x10138363, 40 },
    { 0xb0638368, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd3836b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b83836e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd38374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c938377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b3837b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f13837e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b938382, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc538385, 33 },
    { 0xdf438386, 17 },
    { 0x8ff38387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53838d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c38391, 92 },
    { 0x6bd38392, 98 },
    { 0x1fa38393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d38398, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623839e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01383a1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a383a5, 76 },
    { 0x1f6383a6, 46 },
    { 0x713383a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483383ab, 87 },
    { 0xccc383ac, 76 },
    { 0, 0 },
    { 0xaac383ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2383b3, 23 },
    { 0xce3383b3, 76 },
    { 0xc85383b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177383ba, 61 },
    { 0xef0383bb, 98 },
    { 0x6ba383bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x905383bf, 76 },
    { 0x95e383c0, 61 },
    { 0x220383c1, 87 },
    { 0xfd5383c0, 55 },
    { 0x793383c3, 61 },
    { 0xc43383c4, 88 },
    { 0xc71383c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c383cd, 33 },
    { 0, 0 },
    { 0x16f383cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa383d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39383d8, 95 },
    { 0xb69383d9, 61 },
    { 0, 0 },
    { 0x616383db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5383df, 33 },
    { 0xd8b383e0, 98 },
    { 0xf2c383e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a383e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e383ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3383f1, 76 },
    { 0xa62383f2, 98 },
    { 0xb5a383f2, 23 },
    { 0x269383f4, 98 },
    { 0xbfa383f5, 33 },
    { 0xdd5383f6, 33 },
    { 0x7c6383f6, 93 },
    { 0xa3a383f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd383fb, 91 },
    { 0, 0 },
    { 0xfd2383fd, 78 },
    { 0x6bb383fd, 99 },
    { 0x6ab383ff, 23 },
    { 0, 0 },
    { 0x3d138401, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4038409, 93 },
    { 0x8da3840a, 98 },
    { 0x6823840a, 90 },
    { 0, 0 },
    { 0x633840d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b338410, 98 },
    { 0x5db38410, 61 },
    { 0xc9638412, 33 },
    { 0xb938413, 61 },
    { 0x27038414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6423841e, 95 },
    { 0x36a3841e, 90 },
    { 0, 0 },
    { 0x7a338421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c38427, 66 },
    { 0x37b38428, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d33842b, 34 },
    { 0xe1a3842c, 78 },
    { 0xd353842d, 23 },
    { 0xb7c3842c, 98 },
    { 0, 0 },
    { 0x9ed38430, 23 },
    { 0, 0 },
    { 0x69338432, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da38436, 64 },
    { 0, 0 },
    { 0xdc438438, 81 },
    { 0xe3838439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12838440, 23 },
    { 0x7cc38441, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec038446, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9138449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23844d, 81 },
    { 0x4753844e, 90 },
    { 0xf373844f, 85 },
    { 0x2b83844e, 33 },
    { 0x4713844f, 53 },
    { 0x8ce3844f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70638457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43845a, 98 },
    { 0x1f33845a, 95 },
    { 0xd9f3845b, 61 },
    { 0x1dc3845d, 61 },
    { 0xab53845c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa538468, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f438473, 52 },
    { 0xd1538474, 90 },
    { 0xcea38473, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113847c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb613847f, 23 },
    { 0xa2238480, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc38483, 80 },
    { 0xaec38483, 43 },
    { 0xc6e38485, 23 },
    { 0x68038486, 61 },
    { 0x88e38487, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb973848a, 55 },
    { 0x68d3848b, 90 },
    { 0x9de3848c, 33 },
    { 0x4103848d, 98 },
    { 0, 0 },
    { 0x6a13848f, 76 },
    { 0xc8038490, 76 },
    { 0x738491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49938494, 87 },
    { 0x39738495, 76 },
    { 0x2ca38495, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8583849a, 95 },
    { 0x8873849b, 42 },
    { 0xe953849b, 33 },
    { 0x8653849d, 81 },
    { 0, 0 },
    { 0x44a3849f, 90 },
    { 0xda1384a0, 81 },
    { 0x374384a1, 90 },
    { 0x1e6384a2, 46 },
    { 0xcba384a0, 67 },
    { 0xd8f384a4, 100 },
    { 0xc9b384a2, 61 },
    { 0, 0 },
    { 0x7f6384a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d384aa, 76 },
    { 0x653384aa, 23 },
    { 0x592384ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8384b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3384b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4384bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a384ca, 34 },
    { 0, 0 },
    { 0x7e3384cc, 61 },
    { 0xd88384cc, 76 },
    { 0, 0 },
    { 0xe5c384cf, 33 },
    { 0xc8f384d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba384d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503384d7, 78 },
    { 0x7fa384d8, 98 },
    { 0, 0 },
    { 0x6ae384da, 39 },
    { 0xc2b384db, 33 },
    { 0xbf0384da, 77 },
    { 0x5fc384db, 55 },
    { 0x25384de, 88 },
    { 0xf67384df, 23 },
    { 0x642384dc, 95 },
    { 0xaa5384df, 95 },
    { 0, 0 },
    { 0xd11384e3, 61 },
    { 0x556384e3, 98 },
    { 0x1cf384e5, 68 },
    { 0xf0a384e5, 61 },
    { 0x541384e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b384ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48384ee, 23 },
    { 0, 0 },
    { 0x3bc384f0, 95 },
    { 0, 0 },
    { 0x96e384f2, 61 },
    { 0x595384f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c384fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef38504, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca38507, 33 },
    { 0x9c738508, 55 },
    { 0x53438508, 76 },
    { 0x6be38509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8423850e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1638514, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a38518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3851c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa138523, 43 },
    { 0xd3f38524, 23 },
    { 0, 0 },
    { 0x6ca38526, 90 },
    { 0xe338527, 61 },
    { 0, 0 },
    { 0xd4938529, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f93852f, 91 },
    { 0x61138530, 84 },
    { 0x3f238531, 23 },
    { 0x8e238532, 23 },
    { 0, 0 },
    { 0xf8838534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f53853d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4838540, 55 },
    { 0x35038541, 100 },
    { 0xb8338541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd038545, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27538548, 68 },
    { 0x22d38548, 55 },
    { 0, 0 },
    { 0x4383854b, 23 },
    { 0, 0 },
    { 0xbcd3854d, 55 },
    { 0x9553854e, 95 },
    { 0, 0 },
    { 0xf3338550, 81 },
    { 0, 0 },
    { 0xc1838552, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b038556, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x80138559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa633855c, 61 },
    { 0x78e3855d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83038561, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae438566, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d3856e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e38572, 33 },
    { 0xc4638573, 77 },
    { 0x31c38573, 61 },
    { 0x25438574, 98 },
    { 0xb9b38574, 20 },
    { 0xa6538577, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9803857a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3033857e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7438584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x50138587, 91 },
    { 0xa6738588, 33 },
    { 0xcd738589, 61 },
    { 0xf313858a, 90 },
    { 0, 0 },
    { 0x33858c, 61 },
    { 0x10c3858d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d438590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a38594, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb073859b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3859e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68385a5, 91 },
    { 0xb56385a6, 95 },
    { 0, 0 },
    { 0x80f385a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66385ab, 23 },
    { 0xecb385ab, 75 },
    { 0xd4a385ad, 34 },
    { 0x639385ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae385b1, 75 },
    { 0x91c385b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1385b5, 90 },
    { 0, 0 },
    { 0xaa1385b7, 95 },
    { 0xb01385b8, 95 },
    { 0x7c385b9, 91 },
    { 0x7e0385b7, 55 },
    { 0x152385ba, 61 },
    { 0x366385b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f385c2, 90 },
    { 0xb15385c2, 46 },
    { 0, 0 },
    { 0xd9e385c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d385d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x153385d4, 34 },
    { 0xaac385d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a385dc, 76 },
    { 0x782385dd, 93 },
    { 0xde0385dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556385e9, 33 },
    { 0xd0c385ea, 66 },
    { 0, 0 },
    { 0x47e385ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66385ef, 61 },
    { 0x118385f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c385f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x766385fc, 52 },
    { 0x62c385fd, 34 },
    { 0x415385fc, 46 },
    { 0, 0 },
    { 0xd2c38600, 99 },
    { 0, 0 },
    { 0x95f38602, 100 },
    { 0xf9c38603, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f63860a, 93 },
    { 0xb483860a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d3860f, 55 },
    { 0xb9638610, 23 },
    { 0xac838611, 91 },
    { 0, 0 },
    { 0xab738613, 23 },
    { 0x74f38614, 55 },
    { 0x49c38614, 23 },
    { 0x8e038616, 46 },
    { 0xd2038617, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x1993861a, 90 },
    { 0, 0 },
    { 0x1373861c, 98 },
    { 0x55c3861d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a38620, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae538626, 76 },
    { 0x11738627, 43 },
    { 0x24c38628, 90 },
    { 0xb8438629, 90 },
    { 0x52a3862a, 33 },
    { 0x5a438629, 90 },
    { 0, 0 },
    { 0xae33862d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b38632, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc238637, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda23863d, 90 },
    { 0x5e3863d, 34 },
    { 0x6b23863e, 98 },
    { 0, 0 },
    { 0xca438641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f538647, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd673864a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3864d, 93 },
    { 0xadf3864e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0038656, 43 },
    { 0x5f38657, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33865d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc238663, 55 },
    { 0x1d038664, 81 },
    { 0x8da38665, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11938669, 93 },
    { 0, 0 },
    { 0xfed3866b, 93 },
    { 0xf163866c, 93 },
    { 0xd173866d, 61 },
    { 0, 0 },
    { 0xa333866f, 98 },
    { 0xef938670, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2238675, 23 },
    { 0x81338676, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be38685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d838688, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b3868d, 23 },
    { 0x5543868e, 81 },
    { 0xde23868f, 76 },
    { 0x1693868f, 34 },
    { 0x6103868e, 61 },
    { 0x20c38692, 55 },
    { 0, 0 },
    { 0x90338694, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc38697, 61 },
    { 0xe5e38697, 90 },
    { 0, 0 },
    { 0xcfd3869a, 33 },
    { 0, 0 },
    { 0xb5b3869c, 56 },
    { 0, 0 },
    { 0xd7d3869e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704386a4, 78 },
    { 0x37b386a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1386a9, 78 },
    { 0, 0 },
    { 0xa92386ab, 33 },
    { 0x80f386ac, 34 },
    { 0, 0 },
    { 0x75f386ae, 69 },
    { 0xbd1386ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66386b4, 85 },
    { 0, 0 },
    { 0xa0e386b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192386bb, 38 },
    { 0xbbf386bc, 95 },
    { 0x845386bd, 23 },
    { 0x1c2386be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159386c2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753386ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54386d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9386d5, 33 },
    { 0, 0 },
    { 0x8c0386d7, 33 },
    { 0x8f5386d8, 90 },
    { 0, 0 },
    { 0x303386da, 90 },
    { 0, 0 },
    { 0xa07386dc, 33 },
    { 0xde2386dd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f386e0, 95 },
    { 0xd73386e1, 19 },
    { 0, 0 },
    { 0x18d386e3, 52 },
    { 0x4a9386e4, 61 },
    { 0x2f386e4, 76 },
    { 0xcbf386e6, 95 },
    { 0x3d386e7, 90 },
    { 0x75a386e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd386ed, 95 },
    { 0, 0 },
    { 0xced386ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337386f3, 33 },
    { 0x5fc386f3, 77 },
    { 0x915386f4, 46 },
    { 0x8e1386f5, 90 },
    { 0x8cb386f7, 33 },
    { 0xd7b386f5, 95 },
    { 0x7e6386f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c38707, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab3870a, 33 },
    { 0x133870b, 98 },
    { 0x3b63870c, 33 },
    { 0x24b3870c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37138710, 33 },
    { 0xc7338711, 46 },
    { 0x5c138711, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c238718, 93 },
    { 0x74238718, 38 },
    { 0x5193871a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa413871e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd038724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d38727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf13872b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3738733, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b38737, 81 },
    { 0xbe538738, 84 },
    { 0x3bf38738, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9483873c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c38744, 84 },
    { 0x4f038745, 95 },
    { 0x58f38746, 61 },
    { 0xba538745, 84 },
    { 0x3b438747, 34 },
    { 0, 0 },
    { 0x4143874a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f3874e, 55 },
    { 0x9393874e, 43 },
    { 0x4d03874e, 55 },
    { 0xfc3874e, 55 },
    { 0x7f338752, 84 },
    { 0x4d838752, 55 },
    { 0x87938753, 65 },
    { 0, 0 },
    { 0x25738756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a38764, 46 },
    { 0x66538764, 92 },
    { 0xfe538764, 98 },
    { 0x2a038766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e3876d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf38773, 90 },
    { 0xaf738774, 61 },
    { 0x26038775, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5338779, 96 },
    { 0x4493877a, 88 },
    { 0xf923877b, 90 },
    { 0x2de3877c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83938783, 95 },
    { 0, 0 },
    { 0xe3238785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86938789, 87 },
    { 0x4b43878a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1453878d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed38792, 99 },
    { 0, 0 },
    { 0xdff38794, 84 },
    { 0xfb838795, 55 },
    { 0x9c438796, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe38799, 90 },
    { 0, 0 },
    { 0x9823879b, 56 },
    { 0x4643879b, 46 },
    { 0xe0d3879c, 22 },
    { 0x1b43879e, 77 },
    { 0xa653879f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848387a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48387a8, 61 },
    { 0x150387a8, 33 },
    { 0xa6d387a8, 76 },
    { 0x1fe387ab, 55 },
    { 0x22f387aa, 61 },
    { 0xf48387aa, 90 },
    { 0xa8387ae, 98 },
    { 0x50f387ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6387b3, 98 },
    { 0x675387b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x975387b7, 91 },
    { 0x4387b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa387bb, 95 },
    { 0xbb6387bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513387c1, 90 },
    { 0, 0 },
    { 0x62d387c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f387ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0387d0, 17 },
    { 0x569387d0, 34 },
    { 0xaa8387d2, 34 },
    { 0x588387d2, 98 },
    { 0xa1387d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e387d8, 61 },
    { 0xbb4387d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6387df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06387e2, 99 },
    { 0xd0e387e3, 61 },
    { 0xcac387e3, 95 },
    { 0x5fc387e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae387e9, 55 },
    { 0xe53387e9, 95 },
    { 0x9b2387ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae387ee, 19 },
    { 0x9f387ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba387f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e387f7, 90 },
    { 0, 0 },
    { 0x23e387f9, 90 },
    { 0x3fa387fa, 86 },
    { 0, 0 },
    { 0xee3387fc, 95 },
    { 0xc73387fd, 46 },
    { 0x460387fd, 5 },
    { 0x866387ff, 77 },
    { 0, 0 },
    { 0x50438801, 90 },
    { 0x8e338801, 23 },
    { 0x4e38803, 87 },
    { 0x55438804, 87 },
    { 0xef738801, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe3880b, 59 },
    { 0x2073880b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d38811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c38815, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9438818, 90 },
    { 0x91838819, 52 },
    { 0, 0 },
    { 0x7273881b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6038823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb693882a, 90 },
    { 0x4b53882b, 34 },
    { 0, 0 },
    { 0x3553882d, 34 },
    { 0x65d3882e, 76 },
    { 0xc3e3882e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d38832, 53 },
    { 0xc6538833, 33 },
    { 0x9a538834, 55 },
    { 0x6d338835, 55 },
    { 0, 0 },
    { 0x37a38837, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843883b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52938841, 61 },
    { 0x47e38842, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5538845, 46 },
    { 0x34138845, 93 },
    { 0, 0 },
    { 0x93638848, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b3884b, 100 },
    { 0xf473884b, 95 },
    { 0x4a33884d, 23 },
    { 0xa793884b, 34 },
    { 0, 0 },
    { 0x7a338850, 90 },
    { 0, 0 },
    { 0x3538852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e3885c, 33 },
    { 0xf7c3885c, 33 },
    { 0, 0 },
    { 0x97d3885f, 42 },
    { 0xd9238860, 61 },
    { 0x9ee38860, 34 },
    { 0x37e38862, 34 },
    { 0xe9938863, 95 },
    { 0x3213885f, 90 },
    { 0x78338865, 81 },
    { 0x14638865, 61 },
    { 0xc7738867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf353886a, 99 },
    { 0xbb83886b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d93886f, 67 },
    { 0x6e438870, 100 },
    { 0x58438871, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32638874, 98 },
    { 0x91e38875, 33 },
    { 0, 0 },
    { 0xcd638877, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb413887c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba338884, 88 },
    { 0x59b38885, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99138888, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf923888b, 61 },
    { 0x203888b, 55 },
    { 0xc63888d, 90 },
    { 0xbca3888d, 70 },
    { 0xa423888f, 85 },
    { 0x62438890, 76 },
    { 0, 0 },
    { 0x2c638892, 23 },
    { 0x5df38893, 78 },
    { 0xf138893, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5723889e, 67 },
    { 0xbcd3889f, 33 },
    { 0, 0 },
    { 0xfb9388a1, 61 },
    { 0x1388a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f388aa, 93 },
    { 0, 0 },
    { 0x333388ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3388b0, 61 },
    { 0x7b5388b0, 85 },
    { 0x5e4388b1, 98 },
    { 0, 0 },
    { 0x59388b4, 87 },
    { 0xdeb388b5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x985388b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43388bc, 52 },
    { 0xac2388bc, 61 },
    { 0xf61388be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed388c3, 75 },
    { 0xc56388c4, 46 },
    { 0x4f7388c5, 98 },
    { 0x40c388c5, 90 },
    { 0, 0 },
    { 0xe6b388c8, 76 },
    { 0xe14388c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6388cc, 55 },
    { 0, 0 },
    { 0xdbd388ce, 33 },
    { 0x44c388ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd388d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc388d7, 61 },
    { 0x7c388d8, 23 },
    { 0x375388d8, 88 },
    { 0, 0 },
    { 0x64c388db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef388e0, 99 },
    { 0, 0 },
    { 0xbd9388e2, 93 },
    { 0x7aa388e2, 95 },
    { 0xdc5388e4, 33 },
    { 0x3ae388e3, 34 },
    { 0xe44388e6, 90 },
    { 0x4a2388e7, 33 },
    { 0x7d1388e7, 33 },
    { 0x585388e8, 55 },
    { 0xf1b388e8, 67 },
    { 0xe0c388ea, 33 },
    { 0, 0 },
    { 0x94c388ed, 98 },
    { 0, 0 },
    { 0xccc388ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18388f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4238902, 75 },
    { 0xd6e38903, 95 },
    { 0x1ab38904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f38907, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c23890c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81138910, 78 },
    { 0, 0 },
    { 0x36338912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98038917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b83891a, 55 },
    { 0x7703891a, 33 },
    { 0x64e3891c, 34 },
    { 0x1e03891d, 76 },
    { 0x1da3891e, 34 },
    { 0x2cf3891f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d38924, 90 },
    { 0x24c38925, 46 },
    { 0x80c38925, 96 },
    { 0x3c638925, 33 },
    { 0x83038924, 61 },
    { 0x39138929, 23 },
    { 0x86738927, 61 },
    { 0xafa38928, 55 },
    { 0xa173892c, 87 },
    { 0x31b3892c, 77 },
    { 0xf1f3892d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28638932, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c038939, 98 },
    { 0x67d38939, 55 },
    { 0x1073893a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9538942, 95 },
    { 0x91138942, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc38948, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f38953, 17 },
    { 0x4c438954, 90 },
    { 0xd2338954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11838958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa543895f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6738964, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c93896a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6283896f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8738975, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533897a, 23 },
    { 0, 0 },
    { 0xc5f3897c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad238981, 55 },
    { 0xc5638982, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838986, 78 },
    { 0, 0 },
    { 0xaff38988, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba3898e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de38991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e338994, 90 },
    { 0x6e238995, 76 },
    { 0x2938996, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec3899f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6389a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb389ab, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569389b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65389b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15389c3, 93 },
    { 0x158389c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7389c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1389cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc389d3, 91 },
    { 0xb24389d4, 91 },
    { 0x249389d5, 23 },
    { 0x3a6389d6, 46 },
    { 0x507389d7, 90 },
    { 0xf8c389d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901389dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6389ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f389f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b389f5, 77 },
    { 0, 0 },
    { 0xb8c389f7, 95 },
    { 0, 0 },
    { 0x8b7389f9, 40 },
    { 0, 0 },
    { 0xd4e389fb, 98 },
    { 0, 0 },
    { 0xa62389fd, 87 },
    { 0x890389fe, 98 },
    { 0x4ca389ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe38a02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9838a07, 66 },
    { 0, 0 },
    { 0x85e38a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe438a11, 55 },
    { 0x81738a12, 33 },
    { 0x44d38a13, 93 },
    { 0x7b838a11, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3538a1b, 33 },
    { 0x3c38a1b, 76 },
    { 0, 0 },
    { 0xa3b38a1e, 52 },
    { 0, 0 },
    { 0xc1c38a20, 91 },
    { 0, 0 },
    { 0xd7038a22, 100 },
    { 0x19838a22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d38a26, 61 },
    { 0, 0 },
    { 0x7ef38a28, 100 },
    { 0xf6338a29, 3 },
    { 0xeda38a2a, 23 },
    { 0xa4138a2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19238a30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47138a33, 61 },
    { 0xe0c38a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76538a37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b338a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b338a45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b38a48, 98 },
    { 0xf8a38a49, 81 },
    { 0, 0 },
    { 0x65a38a4b, 23 },
    { 0, 0 },
    { 0xc3538a4d, 87 },
    { 0, 0 },
    { 0x51238a4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f538a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8838a5c, 46 },
    { 0x84f38a5d, 34 },
    { 0, 0 },
    { 0xa7738a5f, 33 },
    { 0x2ab38a5f, 33 },
    { 0, 0 },
    { 0x37038a62, 33 },
    { 0, 0 },
    { 0xa738a64, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9338a68, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b738a72, 46 },
    { 0x88038a72, 90 },
    { 0, 0 },
    { 0x99838a75, 90 },
    { 0x61338a76, 43 },
    { 0xaa738a75, 33 },
    { 0xf5638a77, 61 },
    { 0xdb538a76, 99 },
    { 0, 0 },
    { 0x94438a7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad638a83, 61 },
    { 0, 0 },
    { 0x37838a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94438a88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a138a8b, 33 },
    { 0x26c38a8c, 95 },
    { 0xd0b38a8d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d38a91, 98 },
    { 0x35838a92, 93 },
    { 0xe5c38a93, 98 },
    { 0x29938a94, 23 },
    { 0xb8538a92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f38a9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5738aa3, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x18838aa6, 61 },
    { 0, 0 },
    { 0xaad38aa8, 46 },
    { 0, 0 },
    { 0xbf238aaa, 90 },
    { 0x6138aab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84338aaf, 55 },
    { 0x1a138ab0, 33 },
    { 0x64638ab1, 23 },
    { 0xb0638ab2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff338ab7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a038aba, 61 },
    { 0x92838abb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e938abf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc838aca, 98 },
    { 0, 0 },
    { 0x80238acc, 90 },
    { 0xabc38acd, 95 },
    { 0x3e938acc, 98 },
    { 0, 0 },
    { 0xb8e38ad0, 61 },
    { 0xc7d38ad0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff38ad5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c38adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e38ade, 85 },
    { 0x1c038adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53238ae2, 84 },
    { 0x53d38ae3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c38ae6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a38aec, 81 },
    { 0, 0 },
    { 0xc7d38aee, 101 },
    { 0x8bd38aee, 92 },
    { 0x4cf38af0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15338af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c738af8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da38afc, 87 },
    { 0xfed38afc, 87 },
    { 0x35b38afe, 67 },
    { 0x18638aff, 90 },
    { 0, 0 },
    { 0x57538b01, 34 },
    { 0x5dd38b02, 91 },
    { 0xde038b03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f438b06, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8438b09, 66 },
    { 0x37038b09, 53 },
    { 0xdb138b0b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac38b10, 33 },
    { 0xf6f38b11, 34 },
    { 0xf7038b12, 76 },
    { 0, 0 },
    { 0x40138b14, 61 },
    { 0, 0 },
    { 0x87138b16, 76 },
    { 0, 0 },
    { 0x14938b18, 34 },
    { 0, 0 },
    { 0x7ab38b1a, 66 },
    { 0, 0 },
    { 0xb0238b1c, 95 },
    { 0x51f38b1d, 98 },
    { 0xdc938b1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee138b22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c38b25, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe738b2d, 33 },
    { 0x4d338b2e, 87 },
    { 0x3a238b2e, 90 },
    { 0, 0 },
    { 0x65b38b31, 33 },
    { 0xee838b32, 43 },
    { 0xc8b38b33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4738b38, 95 },
    { 0xd8e38b39, 90 },
    { 0x7b938b38, 67 },
    { 0xd2338b3b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf838b3f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x38738b42, 99 },
    { 0x54f38b43, 47 },
    { 0xb8938b44, 23 },
    { 0x28138b44, 55 },
    { 0, 0 },
    { 0xcd238b47, 85 },
    { 0xabf38b47, 13 },
    { 0, 0 },
    { 0xdb338b4a, 76 },
    { 0x38238b4a, 55 },
    { 0xf7e38b4a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab38b4f, 90 },
    { 0, 0 },
    { 0x38538b51, 98 },
    { 0x7f638b51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88938b56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e838b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce938b64, 55 },
    { 0x37238b65, 98 },
    { 0xde638b64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c38b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1138b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb438b72, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x10838b75, 33 },
    { 0x53d38b76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40638b7b, 43 },
    { 0x24f38b7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e838b7f, 90 },
    { 0x1a738b80, 90 },
    { 0xfb738b80, 91 },
    { 0xe8138b80, 80 },
    { 0xee538b81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4538b86, 90 },
    { 0x90b38b87, 34 },
    { 0x5ce38b87, 34 },
    { 0x17938b87, 95 },
    { 0x7e38b86, 95 },
    { 0xdab38b8b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b38b9b, 33 },
    { 0x9a638b9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69838ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a38ba4, 95 },
    { 0x92538ba4, 55 },
    { 0, 0 },
    { 0x7138ba7, 61 },
    { 0, 0 },
    { 0x68438ba9, 34 },
    { 0x72538ba9, 90 },
    { 0x7d038bab, 87 },
    { 0x4e838baa, 33 },
    { 0, 0 },
    { 0xc2b38bae, 100 },
    { 0x23438bae, 95 },
    { 0xca338bb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b438bb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3538bbc, 98 },
    { 0, 0 },
    { 0x9d838bbe, 91 },
    { 0x10938bbf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638bc6, 17 },
    { 0x11138bc7, 100 },
    { 0x6d38bc8, 95 },
    { 0xd9138bc9, 34 },
    { 0x7bb38bc8, 90 },
    { 0x7e038bc7, 90 },
    { 0xf6c38bcc, 90 },
    { 0, 0 },
    { 0x4df38bce, 23 },
    { 0, 0 },
    { 0xd9338bd0, 67 },
    { 0x24538bd0, 98 },
    { 0, 0 },
    { 0x6b638bd3, 90 },
    { 0x99038bd4, 90 },
    { 0, 0 },
    { 0x57938bd6, 34 },
    { 0, 0 },
    { 0x33438bd8, 87 },
    { 0xfe638bd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe338bdd, 87 },
    { 0x1c738bde, 98 },
    { 0x2cc38bdd, 76 },
    { 0x22538be0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f638be3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd38beb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9638bef, 33 },
    { 0, 0 },
    { 0x19138bf1, 101 },
    { 0x3eb38bf1, 33 },
    { 0x59d38bf3, 61 },
    { 0xb8a38bf2, 55 },
    { 0xb4b38bf4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b38bf9, 23 },
    { 0x3ea38bf9, 52 },
    { 0xeae38bfa, 46 },
    { 0, 0 },
    { 0x75e38bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc38c05, 98 },
    { 0xb5938c06, 61 },
    { 0xf9038c06, 61 },
    { 0x9fa38c08, 61 },
    { 0x99538c09, 34 },
    { 0x5ea38c0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c38c0e, 76 },
    { 0x21738c0f, 34 },
    { 0xd4738c0f, 90 },
    { 0xef638c11, 2 },
    { 0x1338c12, 95 },
    { 0x32738c13, 90 },
    { 0xa6d38c13, 90 },
    { 0xb4638c15, 87 },
    { 0x31138c0f, 98 },
    { 0, 0 },
    { 0x35738c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96338c1f, 33 },
    { 0xa7538c20, 90 },
    { 0x8d938c1f, 94 },
    { 0x59238c1f, 76 },
    { 0x29138c23, 90 },
    { 0, 0 },
    { 0xaee38c25, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db38c28, 90 },
    { 0xf7d38c28, 98 },
    { 0xf38c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a838c2d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d38c33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a238c37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8638c3b, 17 },
    { 0xea738c3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b38c49, 33 },
    { 0, 0 },
    { 0x2b338c4b, 81 },
    { 0x6df38c4c, 33 },
    { 0x33438c4d, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5538c51, 88 },
    { 0x5e038c51, 90 },
    { 0, 0 },
    { 0x9c38c54, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8438c57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c838c5b, 61 },
    { 0x31e38c5b, 90 },
    { 0x87338c5d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19938c62, 85 },
    { 0xe3938c63, 95 },
    { 0, 0 },
    { 0x14838c65, 90 },
    { 0x53f38c65, 33 },
    { 0x1f738c65, 33 },
    { 0xb6038c65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd38c6e, 52 },
    { 0xd4d38c6f, 99 },
    { 0xf6838c70, 90 },
    { 0xf2c38c6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce838c74, 98 },
    { 0x6d238c74, 56 },
    { 0, 0 },
    { 0xc1c38c77, 33 },
    { 0xe8b38c78, 61 },
    { 0x16d38c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d38c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0938c85, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3438c88, 87 },
    { 0, 0 },
    { 0xf7738c8a, 84 },
    { 0x1f838c8b, 80 },
    { 0xc938c8c, 23 },
    { 0x5bf38c8d, 76 },
    { 0x5c938c8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538c91, 91 },
    { 0, 0 },
    { 0xadf38c93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe38c97, 17 },
    { 0x93538c98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d038c9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a38ca1, 98 },
    { 0xa7538ca1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c338ca7, 33 },
    { 0x65438ca7, 76 },
    { 0x5af38ca9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48038caf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87038cb5, 61 },
    { 0xbf738cb5, 94 },
    { 0x1a338cb7, 93 },
    { 0x71e38cb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b38cbf, 33 },
    { 0xa4238cbf, 33 },
    { 0xe6b38cbf, 76 },
    { 0x87738cc0, 95 },
    { 0, 0 },
    { 0x7cf38cc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d038cc8, 90 },
    { 0x9b738cc9, 95 },
    { 0xdfa38cca, 90 },
    { 0x75138cc9, 95 },
    { 0, 0 },
    { 0xc3b38ccd, 43 },
    { 0xd9c38ccd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef38cd5, 46 },
    { 0xb6138cd6, 23 },
    { 0x78238cd7, 23 },
    { 0x10e38cd8, 43 },
    { 0x87338cd9, 33 },
    { 0x46438cd8, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d38cdd, 61 },
    { 0, 0 },
    { 0x2ec38cdf, 33 },
    { 0x5c938cdf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd38ce8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab38cf1, 17 },
    { 0xcc938cf1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7838cf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea38cf8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63438cff, 90 },
    { 0x7c638d00, 61 },
    { 0xa2e38d01, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df38d04, 90 },
    { 0, 0 },
    { 0x64a38d06, 56 },
    { 0xd1a38d07, 46 },
    { 0, 0 },
    { 0x4fa38d09, 84 },
    { 0, 0 },
    { 0x5ed38d0b, 61 },
    { 0, 0 },
    { 0xa5a38d0d, 95 },
    { 0, 0 },
    { 0xf6538d0f, 61 },
    { 0x99b38d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13438d17, 92 },
    { 0, 0 },
    { 0x77b38d19, 85 },
    { 0, 0 },
    { 0xb3a38d1b, 87 },
    { 0, 0 },
    { 0x95138d1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e538d20, 76 },
    { 0, 0 },
    { 0xc8c38d22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1838d26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe38d31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a38d35, 61 },
    { 0x5d38d36, 23 },
    { 0x3ec38d37, 98 },
    { 0x68038d37, 90 },
    { 0x29638d39, 61 },
    { 0, 0 },
    { 0x73438d3b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f38d3e, 33 },
    { 0x58e38d3f, 40 },
    { 0, 0 },
    { 0x5cd38d41, 90 },
    { 0x7ef38d41, 34 },
    { 0xc9338d43, 76 },
    { 0, 0 },
    { 0xe3438d45, 80 },
    { 0x2a838d46, 90 },
    { 0, 0 },
    { 0xff538d48, 33 },
    { 0, 0 },
    { 0x8ab38d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e38d4d, 33 },
    { 0x49138d4d, 90 },
    { 0x4b738d4d, 90 },
    { 0, 0 },
    { 0x5838d51, 33 },
    { 0, 0 },
    { 0x4938d53, 76 },
    { 0, 0 },
    { 0x17d38d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b38d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53538d5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d38d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c438d6a, 33 },
    { 0xa4f38d6b, 17 },
    { 0x3b438d6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d38d73, 95 },
    { 0x76f38d74, 43 },
    { 0xa1a38d75, 90 },
    { 0, 0 },
    { 0x46638d77, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50538d7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e338d82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96238d85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f038d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b38d95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71538d99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1838d9d, 52 },
    { 0, 0 },
    { 0xa3838d9f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb38dae, 90 },
    { 0xca838daf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d438db6, 43 },
    { 0x87238db7, 68 },
    { 0x5138db8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x18838dbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de38dc6, 67 },
    { 0x16438dc7, 55 },
    { 0x58e38dc7, 98 },
    { 0x45738dc6, 55 },
    { 0x9f538dca, 61 },
    { 0x93d38dc9, 23 },
    { 0, 0 },
    { 0xe1038dcd, 33 },
    { 0x5d438dce, 88 },
    { 0, 0 },
    { 0x7f138dd0, 33 },
    { 0x67438dd0, 90 },
    { 0xd3338dd0, 33 },
    { 0x25a38dd3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d38dd6, 17 },
    { 0xb3538dd7, 44 },
    { 0, 0 },
    { 0x97f38dd9, 76 },
    { 0xf3538dda, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b38dde, 23 },
    { 0xd8e38dde, 76 },
    { 0x4c338ddf, 61 },
    { 0xeba38ddf, 23 },
    { 0, 0 },
    { 0xa5238de3, 43 },
    { 0x9fe38de3, 76 },
    { 0, 0 },
    { 0x93238de6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b38df0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c738df6, 61 },
    { 0xbd038df7, 61 },
    { 0xafd38df7, 90 },
    { 0x7a938df8, 40 },
    { 0xef138dfa, 33 },
    { 0xa6138dfb, 34 },
    { 0x96738dfa, 34 },
    { 0x9ee38dfb, 76 },
    { 0x7b738dfe, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96538e03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb438e0e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3738e11, 90 },
    { 0, 0 },
    { 0xd7038e13, 23 },
    { 0xb2138e13, 90 },
    { 0x438e15, 33 },
    { 0x23e38e16, 33 },
    { 0x2dc38e16, 33 },
    { 0x1c938e13, 95 },
    { 0x16f38e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d938e1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f238e21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea38e28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4838e2b, 90 },
    { 0xb7338e2c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6738e30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac38e35, 76 },
    { 0x14738e36, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a538e3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e38e44, 46 },
    { 0, 0 },
    { 0x16b38e46, 93 },
    { 0xe0d38e47, 76 },
    { 0x1e38e48, 23 },
    { 0x74338e48, 23 },
    { 0xf3c38e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d38e4d, 98 },
    { 0x6c038e4e, 61 },
    { 0, 0 },
    { 0x5ed38e50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b838e53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e38e5a, 78 },
    { 0x10338e5a, 84 },
    { 0xa3138e5c, 76 },
    { 0x2138e5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca38e62, 33 },
    { 0xc3938e63, 23 },
    { 0xff438e64, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab238e68, 90 },
    { 0x86238e68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4838e6d, 80 },
    { 0, 0 },
    { 0xefa38e6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b638e7c, 33 },
    { 0x88138e7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8938e80, 95 },
    { 0xe6b38e80, 23 },
    { 0x61338e82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c638e87, 90 },
    { 0, 0 },
    { 0x92238e89, 34 },
    { 0, 0 },
    { 0x10438e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd38e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28138e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b38e9e, 98 },
    { 0x2e238e9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f38ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78138ea9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b38eac, 95 },
    { 0x8e38eac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd738eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee38eb6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b38ebc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e938ebf, 91 },
    { 0, 0 },
    { 0xfa038ec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83738ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2238ec9, 36 },
    { 0, 0 },
    { 0x99438ecb, 61 },
    { 0x8ac38ecc, 90 },
    { 0xc1638ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d38ed0, 23 },
    { 0x36d38ed1, 90 },
    { 0x21c38ed0, 55 },
    { 0xf8138ed0, 43 },
    { 0, 0 },
    { 0xb7838ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c038eda, 55 },
    { 0, 0 },
    { 0x9d938edc, 81 },
    { 0x15b38edd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a38ee2, 87 },
    { 0x85938ee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c38eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b38ef2, 61 },
    { 0xbe38ef2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac38ef7, 66 },
    { 0x61538ef7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c438efd, 95 },
    { 0, 0 },
    { 0xa1c38eff, 77 },
    { 0, 0 },
    { 0x7c838f01, 61 },
    { 0x25a38f02, 98 },
    { 0, 0 },
    { 0x82538f04, 17 },
    { 0x5d38f04, 75 },
    { 0xfa838f06, 98 },
    { 0x7a538f07, 93 },
    { 0xa5e38f08, 33 },
    { 0xad338f09, 52 },
    { 0xe5238f07, 55 },
    { 0x20f38f0b, 84 },
    { 0xf3e38f04, 46 },
    { 0x46838f0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46738f14, 65 },
    { 0x238f15, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f138f18, 55 },
    { 0xa5938f19, 46 },
    { 0xb5b38f1a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c38f1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8738f25, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad138f2a, 98 },
    { 0, 0 },
    { 0xa5738f2c, 90 },
    { 0x7d038f2d, 60 },
    { 0, 0 },
    { 0x54538f2f, 78 },
    { 0xf2838f30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19738f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda838f3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9338f44, 90 },
    { 0, 0 },
    { 0x71838f46, 55 },
    { 0x46a38f47, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b938f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e38f4f, 33 },
    { 0xef638f50, 76 },
    { 0x8f738f51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d38f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17438f57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae38f60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438f67, 90 },
    { 0x12238f68, 49 },
    { 0x52638f68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b138f6d, 33 },
    { 0, 0 },
    { 0xaba38f6f, 98 },
    { 0, 0 },
    { 0x1138f71, 43 },
    { 0x26038f72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d638f7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d38f85, 33 },
    { 0xeb138f85, 98 },
    { 0x1c238f86, 66 },
    { 0xb8d38f88, 61 },
    { 0x82c38f89, 80 },
    { 0, 0 },
    { 0xb1c38f8b, 98 },
    { 0xd0f38f8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c738f98, 98 },
    { 0, 0 },
    { 0xe1938f9a, 98 },
    { 0x64538f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e38f9f, 98 },
    { 0, 0 },
    { 0xa4e38fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d38fa4, 61 },
    { 0xf4938fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f38fab, 90 },
    { 0xad138fab, 33 },
    { 0xa6438fac, 98 },
    { 0xca738fab, 90 },
    { 0x41638fad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd138fb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c38fb8, 93 },
    { 0, 0 },
    { 0x64838fba, 61 },
    { 0, 0 },
    { 0x82a38fbc, 55 },
    { 0x15338fbd, 95 },
    { 0xad438fbd, 90 },
    { 0, 0 },
    { 0x76138fc0, 90 },
    { 0, 0 },
    { 0x11b38fc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc638fc5, 90 },
    { 0x19738fc6, 95 },
    { 0, 0 },
    { 0x47e38fc8, 95 },
    { 0xe4438fc9, 61 },
    { 0, 0 },
    { 0x83538fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e38fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d738fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38938fdd, 90 },
    { 0x2a538fdd, 34 },
    { 0x3cb38fdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed338fe3, 100 },
    { 0xf6338fe3, 85 },
    { 0x85838fe5, 80 },
    { 0x6f438fe5, 98 },
    { 0xcac38fe7, 66 },
    { 0x63138fe5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c38ff0, 66 },
    { 0, 0 },
    { 0xe8b38ff2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce38ff5, 84 },
    { 0xa5438ff6, 98 },
    { 0xf9238ff7, 87 },
    { 0x78538ff8, 90 },
    { 0x9de38ff9, 67 },
    { 0xa1538ffa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb38ffe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20939003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d39006, 55 },
    { 0xed739007, 76 },
    { 0xf939008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c33900c, 33 },
    { 0, 0 },
    { 0x20f3900e, 52 },
    { 0x1de3900f, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x74539012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68639015, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b39019, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb3901e, 75 },
    { 0, 0 },
    { 0xd6039020, 43 },
    { 0x80f39021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f39024, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e3902b, 87 },
    { 0, 0 },
    { 0x4413902d, 33 },
    { 0xde63902e, 90 },
    { 0xc393902f, 90 },
    { 0, 0 },
    { 0x7c739031, 33 },
    { 0x58139032, 46 },
    { 0x2d639032, 76 },
    { 0x29739032, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d039037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe263903a, 46 },
    { 0, 0 },
    { 0x6c93903c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed53903f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d39042, 84 },
    { 0, 0 },
    { 0xff39044, 34 },
    { 0x85e39045, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c39048, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a3904f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19839057, 55 },
    { 0xf7339058, 55 },
    { 0x4e239059, 33 },
    { 0, 0 },
    { 0xb8e3905b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9793905e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x97039061, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e39066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e39069, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b13906c, 55 },
    { 0xea63906d, 23 },
    { 0xfd73906d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca39073, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d39076, 43 },
    { 0x42c39076, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8593907a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd783907d, 61 },
    { 0xe4a3907e, 95 },
    { 0x3153907e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc039084, 53 },
    { 0xdf339085, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e39089, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff739090, 61 },
    { 0, 0 },
    { 0xfd439092, 85 },
    { 0x8e739093, 23 },
    { 0x8ca39094, 23 },
    { 0xbc439095, 90 },
    { 0x4639096, 76 },
    { 0xa1639096, 33 },
    { 0xd0439096, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81390a0, 61 },
    { 0x302390a1, 90 },
    { 0x77d390a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572390a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3390ad, 90 },
    { 0xb13390ae, 78 },
    { 0xfb0390af, 23 },
    { 0x7ec390b0, 98 },
    { 0xcb2390af, 90 },
    { 0x3f8390af, 33 },
    { 0xe9b390b3, 33 },
    { 0x1be390b4, 33 },
    { 0, 0 },
    { 0x435390b6, 55 },
    { 0x122390b6, 81 },
    { 0x236390b8, 52 },
    { 0, 0 },
    { 0x27c390ba, 98 },
    { 0xc1f390bb, 61 },
    { 0x6d1390bc, 52 },
    { 0xdd390bd, 91 },
    { 0xb48390bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa390c4, 61 },
    { 0x97e390c5, 84 },
    { 0xbca390c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94390d1, 61 },
    { 0, 0 },
    { 0xf73390d3, 61 },
    { 0, 0 },
    { 0xc53390d5, 55 },
    { 0xecb390d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e390db, 23 },
    { 0x743390dc, 34 },
    { 0x137390db, 93 },
    { 0x3b4390de, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92390e2, 93 },
    { 0x952390e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca390e7, 33 },
    { 0x76d390e7, 52 },
    { 0x8b390e9, 90 },
    { 0xd390ea, 55 },
    { 0, 0 },
    { 0xbf8390ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c390ef, 90 },
    { 0x421390ef, 33 },
    { 0x70f390ef, 33 },
    { 0xa1e390f1, 98 },
    { 0, 0 },
    { 0x47c390f4, 66 },
    { 0, 0 },
    { 0xd0d390f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53390fa, 61 },
    { 0x2db390fb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539100, 61 },
    { 0xbeb39100, 33 },
    { 0x65239102, 34 },
    { 0, 0 },
    { 0x5a139104, 90 },
    { 0, 0 },
    { 0xd2739106, 33 },
    { 0x18739107, 67 },
    { 0, 0 },
    { 0x3b939109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8739111, 95 },
    { 0, 0 },
    { 0xcfb39113, 33 },
    { 0, 0 },
    { 0xd0439115, 33 },
    { 0x7fb39116, 33 },
    { 0x43339117, 43 },
    { 0x8be39118, 85 },
    { 0x95c39118, 55 },
    { 0xffe39119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3823911e, 33 },
    { 0xd633911e, 90 },
    { 0, 0 },
    { 0x1cb39121, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1039126, 90 },
    { 0xb2939127, 95 },
    { 0x40439127, 34 },
    { 0x1e139129, 61 },
    { 0x1233912a, 95 },
    { 0x8aa3912b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9543912e, 48 },
    { 0, 0 },
    { 0x93a39130, 98 },
    { 0x78839131, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d39136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23539139, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7823913e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x98839141, 95 },
    { 0x3939141, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e39145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee339151, 76 },
    { 0, 0 },
    { 0x6e039153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d39156, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22239159, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac23915c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba439163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e83916b, 33 },
    { 0x203916c, 33 },
    { 0xc383916b, 33 },
    { 0x3ca3916e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1939174, 98 },
    { 0x9f239175, 76 },
    { 0, 0 },
    { 0x2c939177, 23 },
    { 0xec739178, 43 },
    { 0x75b39179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e13917e, 40 },
    { 0x9743917f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26439189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xade3918c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96339191, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x66739194, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28539199, 30 },
    { 0x6183919a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74391a4, 46 },
    { 0, 0 },
    { 0xe55391a6, 52 },
    { 0x75b391a6, 33 },
    { 0x302391a7, 55 },
    { 0x4b5391a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d391ad, 96 },
    { 0x14391ae, 90 },
    { 0x191391ad, 90 },
    { 0xb87391af, 55 },
    { 0, 0 },
    { 0xf8391b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe391b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9391bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02391c0, 40 },
    { 0xea2391c0, 90 },
    { 0xa8391c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca391c5, 33 },
    { 0x5ee391c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91391cc, 95 },
    { 0x80e391cd, 90 },
    { 0xb56391cd, 98 },
    { 0x62f391cf, 61 },
    { 0x74d391cf, 61 },
    { 0x1cf391d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8391d7, 77 },
    { 0xfc391d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e391de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52391e3, 95 },
    { 0xcca391e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4391e7, 33 },
    { 0x7a8391e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf391eb, 33 },
    { 0x836391eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf391ef, 46 },
    { 0xb5e391f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d391f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3391fe, 85 },
    { 0xe38391fe, 61 },
    { 0x3e939200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e739209, 23 },
    { 0x44f3920a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c63920f, 34 },
    { 0xb593920f, 65 },
    { 0x67439210, 33 },
    { 0, 0 },
    { 0x73139213, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4739216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3439219, 95 },
    { 0xdbf3921a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc333921e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49939221, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24039228, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff83922b, 99 },
    { 0x6a73922c, 55 },
    { 0xd113922d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80739230, 33 },
    { 0xbe139231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8363923b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b39240, 46 },
    { 0xbeb39241, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d039244, 77 },
    { 0x95039245, 98 },
    { 0x5539246, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17839251, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1039254, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3925a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd3925d, 64 },
    { 0x71c3925e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c39262, 95 },
    { 0, 0 },
    { 0x1ac39264, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16539269, 61 },
    { 0xe13926a, 90 },
    { 0x3c139269, 76 },
    { 0x3d13926a, 90 },
    { 0x3943926d, 55 },
    { 0xe73926d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11439271, 95 },
    { 0x2a339272, 90 },
    { 0, 0 },
    { 0x89139274, 61 },
    { 0x8be39275, 90 },
    { 0x6739276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32339281, 23 },
    { 0x56139282, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c39287, 90 },
    { 0x69639288, 78 },
    { 0x3b239288, 66 },
    { 0x7de39288, 90 },
    { 0, 0 },
    { 0xa3b3928c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a39293, 100 },
    { 0xa7539294, 96 },
    { 0x9139295, 95 },
    { 0, 0 },
    { 0x28939297, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa223929b, 90 },
    { 0x1f53929b, 23 },
    { 0, 0 },
    { 0x1f13929e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa392a4, 33 },
    { 0, 0 },
    { 0x5c392a6, 95 },
    { 0x359392a7, 76 },
    { 0xd30392a7, 95 },
    { 0xed3392a9, 23 },
    { 0x12d392a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc392b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956392b7, 97 },
    { 0x4ec392b8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a392bb, 38 },
    { 0x921392bc, 76 },
    { 0xfc0392bc, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49392c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369392cc, 99 },
    { 0x276392cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709392d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9392d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b392da, 40 },
    { 0, 0 },
    { 0x574392dc, 78 },
    { 0xf74392dd, 98 },
    { 0, 0 },
    { 0xd2c392df, 98 },
    { 0x2a6392df, 90 },
    { 0, 0 },
    { 0x112392e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0392e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d392e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4392ed, 84 },
    { 0, 0 },
    { 0x153392ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x940392f2, 43 },
    { 0xd0a392f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89392fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab39300, 90 },
    { 0x29839300, 33 },
    { 0x64a39300, 90 },
    { 0, 0 },
    { 0x88239304, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b39308, 85 },
    { 0, 0 },
    { 0x2fe3930a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d13930f, 98 },
    { 0, 0 },
    { 0x54339311, 55 },
    { 0x38539312, 43 },
    { 0x41039313, 66 },
    { 0, 0 },
    { 0x15e39315, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd239318, 66 },
    { 0x8ef39319, 90 },
    { 0x7d039318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc39321, 99 },
    { 0xc1639322, 52 },
    { 0xf9439323, 23 },
    { 0x33239324, 46 },
    { 0xca39324, 94 },
    { 0, 0 },
    { 0xa7339327, 23 },
    { 0x93539328, 33 },
    { 0xb6439328, 90 },
    { 0x70139329, 95 },
    { 0, 0 },
    { 0xb2a3932c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd239336, 78 },
    { 0x82639337, 55 },
    { 0x60639338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5273933c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c39340, 76 },
    { 0x7ff39341, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf3934a, 90 },
    { 0x4f43934b, 23 },
    { 0xef03934c, 98 },
    { 0x76d3934d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81939354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f3935f, 46 },
    { 0x72339360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15039366, 90 },
    { 0x6d439367, 23 },
    { 0, 0 },
    { 0x45739369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c03936c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c39372, 61 },
    { 0xc0139373, 95 },
    { 0x66639374, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e739377, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf39380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9139384, 33 },
    { 0xf9139385, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2939388, 80 },
    { 0xfa739389, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5523938d, 87 },
    { 0xec03938e, 46 },
    { 0x3853938f, 97 },
    { 0x84f3938e, 43 },
    { 0x1a139390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b039395, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6063939a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb243939d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82393a0, 90 },
    { 0, 0 },
    { 0x9b3393a2, 33 },
    { 0x6ae393a3, 90 },
    { 0x545393a4, 66 },
    { 0x854393a4, 90 },
    { 0x9e2393a5, 87 },
    { 0x1c3393a7, 95 },
    { 0x366393a7, 34 },
    { 0, 0 },
    { 0x13c393aa, 61 },
    { 0x911393ab, 76 },
    { 0x208393ac, 23 },
    { 0x84f393ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e393b0, 23 },
    { 0x83f393b0, 88 },
    { 0, 0 },
    { 0xcf5393b3, 76 },
    { 0, 0 },
    { 0xf65393b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c393b8, 66 },
    { 0x8a1393b9, 61 },
    { 0xfb5393b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96393c3, 33 },
    { 0, 0 },
    { 0x676393c5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce393c9, 90 },
    { 0, 0 },
    { 0xfb9393cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a393d6, 61 },
    { 0, 0 },
    { 0x8be393d8, 23 },
    { 0x803393d9, 76 },
    { 0x5f7393da, 33 },
    { 0xdb3393db, 43 },
    { 0, 0 },
    { 0xa3e393dd, 90 },
    { 0xb6e393de, 55 },
    { 0x715393dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef393e6, 95 },
    { 0, 0 },
    { 0x201393e8, 55 },
    { 0, 0 },
    { 0xce4393ea, 95 },
    { 0xef5393eb, 90 },
    { 0x57f393ec, 83 },
    { 0x932393ed, 98 },
    { 0x523393ee, 90 },
    { 0, 0 },
    { 0x194393f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11393f5, 90 },
    { 0xdf9393f6, 23 },
    { 0xfc7393f7, 95 },
    { 0xe6393f7, 99 },
    { 0, 0 },
    { 0xd91393fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6393fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6639402, 90 },
    { 0x61c39402, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc39407, 100 },
    { 0x69e39408, 90 },
    { 0x7e439409, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c3940f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xac639412, 100 },
    { 0, 0 },
    { 0x9b339414, 95 },
    { 0x39e39414, 100 },
    { 0x82e39416, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b3941c, 66 },
    { 0x22f3941d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81839421, 90 },
    { 0x79939421, 55 },
    { 0xd439422, 66 },
    { 0x1bd39424, 46 },
    { 0x15d39425, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc93942c, 60 },
    { 0x86a3942c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d639431, 84 },
    { 0x89339431, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f039435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a39439, 52 },
    { 0x2803943a, 46 },
    { 0x7c63943b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e3943f, 55 },
    { 0x4b03943f, 90 },
    { 0xfad39441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44739445, 61 },
    { 0xfbb39446, 90 },
    { 0, 0 },
    { 0xf0439448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3944b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb3944f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5239452, 43 },
    { 0xf4a39452, 95 },
    { 0x98239453, 100 },
    { 0xcf539455, 61 },
    { 0x3f039455, 46 },
    { 0, 0 },
    { 0x98b39458, 23 },
    { 0xad439459, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f3945f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd39462, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4739468, 91 },
    { 0x5d739468, 55 },
    { 0x7ee3946a, 33 },
    { 0x1d3946b, 46 },
    { 0x80a39468, 61 },
    { 0xe103946d, 46 },
    { 0xc4a3946d, 95 },
    { 0x8fd3946d, 69 },
    { 0, 0 },
    { 0x67739471, 55 },
    { 0x94b39472, 46 },
    { 0x77839471, 53 },
    { 0x21939474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc339478, 95 },
    { 0, 0 },
    { 0xd4e3947a, 76 },
    { 0xcfd3947b, 98 },
    { 0x87c3947c, 33 },
    { 0x2923947d, 36 },
    { 0xbf23947e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46839481, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4339487, 98 },
    { 0x4dc39488, 33 },
    { 0, 0 },
    { 0xa073948a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453948f, 61 },
    { 0, 0 },
    { 0x6a439491, 87 },
    { 0xa7639491, 95 },
    { 0x22e39491, 33 },
    { 0xbee39494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e3949c, 61 },
    { 0, 0 },
    { 0x2943949e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297394a3, 55 },
    { 0xbcd394a4, 95 },
    { 0, 0 },
    { 0xf8e394a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0394ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x416394ae, 83 },
    { 0x231394af, 90 },
    { 0, 0 },
    { 0x80d394b1, 90 },
    { 0xd44394b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4394b7, 77 },
    { 0xf0c394b8, 23 },
    { 0xc6f394b9, 100 },
    { 0x5d3394b9, 98 },
    { 0x506394bb, 95 },
    { 0xcc0394b9, 46 },
    { 0xc89394bd, 95 },
    { 0x33d394ba, 65 },
    { 0xe27394bf, 61 },
    { 0x9fa394c0, 23 },
    { 0x1ea394c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717394c5, 75 },
    { 0x778394c6, 61 },
    { 0xcaf394c6, 23 },
    { 0xa8e394c8, 33 },
    { 0, 0 },
    { 0x1f8394ca, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a394cf, 61 },
    { 0xbd7394d0, 76 },
    { 0, 0 },
    { 0xe72394d2, 60 },
    { 0xd3b394d3, 84 },
    { 0, 0 },
    { 0x7ae394d5, 61 },
    { 0xcfa394d6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5394d9, 41 },
    { 0, 0 },
    { 0x9fe394db, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76394e0, 95 },
    { 0xb94394e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05394e6, 90 },
    { 0x5dc394e6, 52 },
    { 0x401394e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2394f1, 93 },
    { 0xe8c394f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd394f7, 33 },
    { 0xc7b394f8, 23 },
    { 0x351394f9, 52 },
    { 0xa3a394fa, 33 },
    { 0x742394fb, 67 },
    { 0xa05394fa, 33 },
    { 0xe50394fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f39502, 77 },
    { 0, 0 },
    { 0x4ba39504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b539507, 31 },
    { 0x4f939507, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8143950c, 90 },
    { 0xa553950d, 23 },
    { 0x5ca3950e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16639512, 99 },
    { 0x5f339513, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb39518, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f139520, 61 },
    { 0, 0 },
    { 0x75f39522, 90 },
    { 0xc8c39522, 95 },
    { 0xaa39524, 90 },
    { 0x37939523, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5733952a, 90 },
    { 0x1123952b, 99 },
    { 0xe8b3952c, 23 },
    { 0xba53952a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e39531, 84 },
    { 0xd2039532, 34 },
    { 0, 0 },
    { 0x54139534, 33 },
    { 0x51439534, 98 },
    { 0x7d639536, 80 },
    { 0x1239535, 30 },
    { 0x91539538, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c23953d, 76 },
    { 0xbee3953e, 90 },
    { 0xf483953f, 95 },
    { 0xf9c3953f, 99 },
    { 0x7a139541, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e839548, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec3954b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1603954f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e39553, 66 },
    { 0xc7639554, 61 },
    { 0xed239554, 98 },
    { 0x40b39553, 98 },
    { 0x17d39555, 77 },
    { 0, 0 },
    { 0xb7439559, 98 },
    { 0, 0 },
    { 0x3433955b, 99 },
    { 0x27b3955c, 69 },
    { 0x1973955d, 33 },
    { 0x1153955d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9639561, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc39564, 99 },
    { 0xdf039564, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84939568, 90 },
    { 0x6c039569, 33 },
    { 0, 0 },
    { 0x8bc3956b, 33 },
    { 0x8893956c, 62 },
    { 0x7583956c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac39576, 17 },
    { 0, 0 },
    { 0x50939578, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe3957b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13957e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5303958b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae53959a, 33 },
    { 0x9883959b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f63959f, 74 },
    { 0x7fb395a0, 66 },
    { 0x8de395a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e395a6, 84 },
    { 0xd8b395a7, 34 },
    { 0x799395a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7395ac, 90 },
    { 0, 0 },
    { 0xea5395ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f395b2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb395ba, 98 },
    { 0xdac395ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2395bf, 99 },
    { 0x763395c0, 95 },
    { 0x26d395c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7395c7, 88 },
    { 0x814395c7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280395cc, 84 },
    { 0, 0 },
    { 0xaf4395ce, 81 },
    { 0xcec395cf, 96 },
    { 0, 0 },
    { 0x5c6395d1, 95 },
    { 0xc34395d2, 88 },
    { 0x113395d2, 52 },
    { 0x476395d2, 55 },
    { 0x625395d1, 23 },
    { 0, 0 },
    { 0xe42395d7, 88 },
    { 0xd1d395d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f395df, 55 },
    { 0x440395e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a395e4, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd395eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5395f0, 43 },
    { 0x3d395f0, 61 },
    { 0, 0 },
    { 0xda9395f3, 66 },
    { 0, 0 },
    { 0x33a395f5, 90 },
    { 0, 0 },
    { 0xaa0395f7, 61 },
    { 0x667395f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd339600, 95 },
    { 0xf5339600, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab39605, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac63960a, 23 },
    { 0, 0 },
    { 0x21b3960c, 66 },
    { 0, 0 },
    { 0x7e73960e, 95 },
    { 0xd043960f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c39615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133961c, 34 },
    { 0xd6f3961d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21439621, 17 },
    { 0, 0 },
    { 0x4039623, 43 },
    { 0x9af39623, 33 },
    { 0x45039624, 95 },
    { 0xa6639623, 23 },
    { 0x99139627, 99 },
    { 0x95c39625, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e63962e, 55 },
    { 0, 0 },
    { 0x29439630, 84 },
    { 0xe6439630, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb39636, 98 },
    { 0xa1e39636, 23 },
    { 0xfd839638, 23 },
    { 0, 0 },
    { 0x5cd3963a, 23 },
    { 0, 0 },
    { 0x5573963c, 93 },
    { 0, 0 },
    { 0x6df3963e, 55 },
    { 0, 0 },
    { 0x3fe39640, 99 },
    { 0xc7a39640, 33 },
    { 0x4f439642, 34 },
    { 0x5339643, 55 },
    { 0xc7b39644, 90 },
    { 0xa0b39645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50639651, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c39658, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf693965e, 61 },
    { 0x5563965e, 93 },
    { 0x4dd39660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c39665, 34 },
    { 0xace39665, 91 },
    { 0x14939665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb53966b, 81 },
    { 0x5353966b, 61 },
    { 0, 0 },
    { 0xfc03966e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9639672, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b39676, 90 },
    { 0xa4e39677, 61 },
    { 0x63739678, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f63967f, 90 },
    { 0x9053967f, 98 },
    { 0, 0 },
    { 0x38e39682, 34 },
    { 0x4df39683, 55 },
    { 0xe9c39684, 99 },
    { 0, 0 },
    { 0xde339686, 61 },
    { 0x61339686, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a3968b, 46 },
    { 0xed23968b, 90 },
    { 0, 0 },
    { 0xf243968e, 99 },
    { 0x59c3968f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37239695, 76 },
    { 0x32339696, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db3969c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee3969f, 85 },
    { 0xe073969f, 95 },
    { 0x15a396a1, 79 },
    { 0x584396a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e396ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260396b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c396b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x575396bc, 46 },
    { 0xe9a396bd, 98 },
    { 0x29e396be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78396c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad396c8, 46 },
    { 0x15396c9, 61 },
    { 0, 0 },
    { 0x6e396cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c396cf, 100 },
    { 0x32f396d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962396d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22396d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45396e0, 76 },
    { 0xfa396e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a396e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e396ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0396f3, 87 },
    { 0, 0 },
    { 0xb5e396f5, 76 },
    { 0, 0 },
    { 0xbb1396f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec396fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7396fe, 95 },
    { 0x9b396ff, 61 },
    { 0xd2839700, 98 },
    { 0x15a396fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad639705, 33 },
    { 0xcc839706, 95 },
    { 0x26d39707, 55 },
    { 0x5bc39708, 66 },
    { 0x28739705, 46 },
    { 0xb8e39705, 55 },
    { 0xff83970b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea39713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f3971a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf39726, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8483972f, 61 },
    { 0xf9f39730, 46 },
    { 0xa2a3972f, 95 },
    { 0x1e739732, 95 },
    { 0x7df39733, 76 },
    { 0, 0 },
    { 0x57239735, 46 },
    { 0x90e39736, 90 },
    { 0, 0 },
    { 0xf0739738, 95 },
    { 0, 0 },
    { 0x4ef3973a, 34 },
    { 0x9e63973b, 91 },
    { 0xfa93973b, 46 },
    { 0, 0 },
    { 0xe9e3973e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2539742, 23 },
    { 0, 0 },
    { 0x17839744, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3839749, 34 },
    { 0, 0 },
    { 0x8023974b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2363974f, 33 },
    { 0xe6c39750, 61 },
    { 0xb4a39751, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f43975c, 91 },
    { 0x3a43975c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1739761, 85 },
    { 0, 0 },
    { 0x57839763, 61 },
    { 0xcbf39764, 76 },
    { 0xd6a39765, 46 },
    { 0x18c39765, 91 },
    { 0x1a39766, 33 },
    { 0x47939767, 95 },
    { 0xdf139765, 76 },
    { 0x5713976a, 90 },
    { 0x4213976a, 23 },
    { 0xba23976c, 40 },
    { 0x5c839767, 95 },
    { 0x2663976e, 12 },
    { 0x7743976f, 90 },
    { 0xda439769, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5239773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e3977b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8439780, 76 },
    { 0xbc139781, 33 },
    { 0xeeb39780, 95 },
    { 0x81739783, 43 },
    { 0xca439784, 90 },
    { 0xca339784, 98 },
    { 0, 0 },
    { 0x19439787, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc3978a, 61 },
    { 0xf443978a, 38 },
    { 0xae63978c, 55 },
    { 0, 0 },
    { 0x2843978e, 90 },
    { 0xa703978f, 90 },
    { 0, 0 },
    { 0x57a39791, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b39794, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1703979e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01397a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a397a5, 98 },
    { 0xeae397a6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x714397a9, 95 },
    { 0, 0 },
    { 0xec1397ab, 67 },
    { 0xa13397ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e397b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x594397b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1397ba, 55 },
    { 0, 0 },
    { 0xef9397bc, 33 },
    { 0, 0 },
    { 0xfd1397be, 90 },
    { 0x9c397bf, 33 },
    { 0xcc2397be, 55 },
    { 0, 0 },
    { 0xe18397c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d397c6, 90 },
    { 0x1d6397c7, 95 },
    { 0x77e397c8, 99 },
    { 0x79397c9, 33 },
    { 0, 0 },
    { 0x4f397cb, 95 },
    { 0, 0 },
    { 0x865397cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3397d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1397d9, 46 },
    { 0x2b5397da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66397e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4397ec, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb397f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989397f7, 52 },
    { 0x3ee397f7, 90 },
    { 0x15c397f9, 69 },
    { 0xe3d397fa, 95 },
    { 0xd4e397fb, 34 },
    { 0x6e6397fc, 83 },
    { 0x4de397fb, 93 },
    { 0xcf2397fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3539803, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e3980a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe939812, 90 },
    { 0x57f39813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe739816, 100 },
    { 0x58739817, 33 },
    { 0xc0339818, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5883981d, 98 },
    { 0xfda3981e, 85 },
    { 0, 0 },
    { 0x51d39820, 23 },
    { 0xe2339820, 37 },
    { 0xeb439821, 78 },
    { 0x58439821, 55 },
    { 0x85839820, 33 },
    { 0xa5239822, 33 },
    { 0xde39826, 52 },
    { 0x69139823, 90 },
    { 0, 0 },
    { 0xfb439829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13982c, 99 },
    { 0, 0 },
    { 0xc803982e, 83 },
    { 0x1d3982e, 76 },
    { 0x59b39830, 68 },
    { 0xd023982f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73039836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae39839, 76 },
    { 0x4133983a, 77 },
    { 0x2bd39839, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c3983e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9739841, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed239845, 61 },
    { 0x9139846, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4439852, 66 },
    { 0x21d39853, 68 },
    { 0xbb339853, 93 },
    { 0x47739855, 68 },
    { 0xa8439855, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3985a, 33 },
    { 0x4a03985b, 98 },
    { 0, 0 },
    { 0x1613985d, 61 },
    { 0, 0 },
    { 0x9a13985f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f539864, 38 },
    { 0, 0 },
    { 0xb1939866, 33 },
    { 0x68d39867, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8243986b, 19 },
    { 0xc413986c, 33 },
    { 0x9273986c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe239874, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c239877, 61 },
    { 0x57939877, 95 },
    { 0x29539879, 40 },
    { 0xabe39877, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53987d, 66 },
    { 0x7b93987e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x65239881, 76 },
    { 0x7a839881, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45739889, 90 },
    { 0x8b93988a, 55 },
    { 0xc3039889, 87 },
    { 0, 0 },
    { 0x5b93988d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb39892, 46 },
    { 0x94739893, 61 },
    { 0x74239894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7739897, 64 },
    { 0xd8e39898, 23 },
    { 0, 0 },
    { 0x5a23989a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf3989f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507398a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927398ad, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x701398b0, 98 },
    { 0x1d1398b0, 91 },
    { 0xa7398b2, 78 },
    { 0x84b398b2, 90 },
    { 0xc35398b4, 43 },
    { 0x457398b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1398bd, 90 },
    { 0, 0 },
    { 0xfdb398bf, 73 },
    { 0x357398c0, 52 },
    { 0, 0 },
    { 0xa92398c2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1398c6, 98 },
    { 0x18e398c7, 55 },
    { 0x487398c8, 98 },
    { 0xcc2398c7, 78 },
    { 0x6ed398ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd398d0, 23 },
    { 0x401398d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120398d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e398db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f398de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6398e1, 90 },
    { 0x713398e2, 87 },
    { 0, 0 },
    { 0xec3398e4, 90 },
    { 0, 0 },
    { 0x44c398e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09398e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb398ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9398f1, 33 },
    { 0x2de398f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260398f6, 33 },
    { 0x823398f7, 23 },
    { 0, 0 },
    { 0x8f1398f9, 33 },
    { 0, 0 },
    { 0x2b3398fb, 43 },
    { 0xde0398fc, 33 },
    { 0x364398fd, 17 },
    { 0xf15398fd, 33 },
    { 0xf2398ff, 23 },
    { 0xf8939900, 61 },
    { 0xa6e398fe, 90 },
    { 0x78398fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62239905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d639909, 77 },
    { 0xfa13990a, 55 },
    { 0x69339909, 98 },
    { 0, 0 },
    { 0xdfe3990d, 23 },
    { 0x5be3990d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b39914, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43991b, 91 },
    { 0, 0 },
    { 0x1c23991d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f239922, 78 },
    { 0, 0 },
    { 0xd9d39924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a539929, 90 },
    { 0, 0 },
    { 0xdcd3992b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde839931, 46 },
    { 0xe0139932, 61 },
    { 0, 0 },
    { 0xf6139934, 61 },
    { 0xfbf39935, 61 },
    { 0xd1f39936, 95 },
    { 0x75d39935, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f3993f, 98 },
    { 0xe13993f, 17 },
    { 0x4683993f, 33 },
    { 0x12c39942, 98 },
    { 0x9043993f, 61 },
    { 0xaa139944, 23 },
    { 0, 0 },
    { 0xe0339946, 98 },
    { 0x3e839947, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f639950, 67 },
    { 0xfc639951, 34 },
    { 0, 0 },
    { 0x6ed39953, 83 },
    { 0xfe339953, 98 },
    { 0xa0939955, 55 },
    { 0, 0 },
    { 0x2c39957, 95 },
    { 0x5039958, 69 },
    { 0, 0 },
    { 0x7de3995a, 93 },
    { 0x2a23995b, 33 },
    { 0, 0 },
    { 0x49c3995d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19039960, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b3996b, 76 },
    { 0xffa3996b, 55 },
    { 0x273996d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca39978, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333997e, 98 },
    { 0x9783997f, 55 },
    { 0x29839980, 100 },
    { 0x62b39980, 91 },
    { 0x38b39982, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13939989, 61 },
    { 0, 0 },
    { 0x5183998b, 90 },
    { 0x5673998b, 95 },
    { 0xc433998d, 87 },
    { 0, 0 },
    { 0x493998f, 93 },
    { 0xab039990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd639993, 55 },
    { 0, 0 },
    { 0x9ee39995, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f39998, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc399a0, 99 },
    { 0x62a399a1, 87 },
    { 0x1ad399a0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af399a6, 61 },
    { 0, 0 },
    { 0x693399a8, 40 },
    { 0x1399a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09399b0, 43 },
    { 0x47d399b1, 100 },
    { 0x842399b2, 90 },
    { 0x678399b3, 43 },
    { 0xffd399b2, 44 },
    { 0, 0 },
    { 0x8ee399b6, 58 },
    { 0, 0 },
    { 0x1ca399b8, 17 },
    { 0x863399b8, 66 },
    { 0, 0 },
    { 0x5eb399bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814399bf, 95 },
    { 0x579399bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5399c8, 61 },
    { 0, 0 },
    { 0x68c399ca, 78 },
    { 0x8cb399cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897399d1, 33 },
    { 0xf30399d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69399d5, 96 },
    { 0x92f399d6, 99 },
    { 0xcc3399d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202399dd, 91 },
    { 0x4e4399de, 98 },
    { 0x35e399df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f399eb, 90 },
    { 0, 0 },
    { 0xbf5399ed, 59 },
    { 0xc99399ee, 93 },
    { 0x16f399ef, 98 },
    { 0x53399f0, 61 },
    { 0, 0 },
    { 0x91c399f2, 33 },
    { 0xc72399f3, 23 },
    { 0x129399f3, 61 },
    { 0xb41399f2, 90 },
    { 0xe45399f6, 33 },
    { 0xb9f399f7, 61 },
    { 0, 0 },
    { 0xe91399f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1399fc, 75 },
    { 0xe99399fd, 33 },
    { 0, 0 },
    { 0xb18399ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7239a05, 61 },
    { 0, 0 },
    { 0x2d239a07, 55 },
    { 0xf8b39a07, 46 },
    { 0x61439a09, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f039a0f, 90 },
    { 0, 0 },
    { 0xc1039a11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e39a1a, 100 },
    { 0, 0 },
    { 0x7df39a1c, 93 },
    { 0, 0 },
    { 0x1f539a1e, 55 },
    { 0x51639a1f, 100 },
    { 0xd3a39a1e, 31 },
    { 0xdbe39a21, 55 },
    { 0xc6b39a22, 34 },
    { 0xaee39a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b39a26, 100 },
    { 0xad739a26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60439a2d, 100 },
    { 0x1b939a2e, 90 },
    { 0x60e39a2f, 61 },
    { 0xec239a2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd039a33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71839a42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd39a45, 90 },
    { 0x95c39a46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c39a4b, 43 },
    { 0, 0 },
    { 0xefc39a4d, 80 },
    { 0xda639a4e, 33 },
    { 0xa0639a4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75239a56, 76 },
    { 0xe3239a57, 90 },
    { 0xdf239a56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c39a5c, 98 },
    { 0x2cb39a5c, 90 },
    { 0xb1739a5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f739a62, 33 },
    { 0x1d239a63, 46 },
    { 0xb4a39a64, 61 },
    { 0, 0 },
    { 0x5ff39a66, 90 },
    { 0x63239a67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed239a6c, 95 },
    { 0x80639a6c, 43 },
    { 0x13e39a6c, 95 },
    { 0xf7e39a6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d739a75, 98 },
    { 0x9db39a75, 46 },
    { 0x9a39a77, 98 },
    { 0xf8939a77, 33 },
    { 0x38439a79, 78 },
    { 0xc3439a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70639a83, 100 },
    { 0x2a339a83, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93439a8a, 40 },
    { 0, 0 },
    { 0xe6239a8c, 98 },
    { 0, 0 },
    { 0xf6d39a8e, 84 },
    { 0, 0 },
    { 0xbc339a90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd39a94, 33 },
    { 0xe2939a95, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c39a99, 34 },
    { 0, 0 },
    { 0x1c639a9b, 90 },
    { 0xea139a9c, 68 },
    { 0, 0 },
    { 0xc0439a9e, 95 },
    { 0, 0 },
    { 0xcb139aa0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc839aa3, 90 },
    { 0x3ae39aa3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee339aaa, 61 },
    { 0, 0 },
    { 0x6bb39aac, 61 },
    { 0xa7e39aad, 43 },
    { 0x8539aad, 95 },
    { 0x73b39aac, 99 },
    { 0xdc739aaf, 90 },
    { 0x5b939ab0, 61 },
    { 0x1439ab2, 90 },
    { 0xd6239ab3, 85 },
    { 0x59739ab4, 90 },
    { 0, 0 },
    { 0x62239ab6, 87 },
    { 0, 0 },
    { 0x9ab39ab8, 38 },
    { 0x26839ab9, 78 },
    { 0x76439aba, 40 },
    { 0xb8039abb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f839abe, 97 },
    { 0x6fc39abf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d339acf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9039ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c39adf, 61 },
    { 0, 0 },
    { 0xc7c39ae1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13739ae5, 55 },
    { 0, 0 },
    { 0x6439ae7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25139af4, 98 },
    { 0xbc439af4, 90 },
    { 0xf0c39af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb39afa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c739b05, 95 },
    { 0xc8039b06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d39b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5839b0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3039b10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45239b13, 90 },
    { 0xec39b14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0239b19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7839b1e, 33 },
    { 0, 0 },
    { 0x11439b20, 95 },
    { 0, 0 },
    { 0x2e139b22, 90 },
    { 0, 0 },
    { 0xd6039b24, 46 },
    { 0, 0 },
    { 0x1b739b26, 85 },
    { 0x45b39b27, 90 },
    { 0, 0 },
    { 0xe639b29, 33 },
    { 0xc5d39b2a, 46 },
    { 0x65539b2b, 91 },
    { 0x54239b2b, 23 },
    { 0x84a39b2d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a939b32, 87 },
    { 0x4d939b33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x92439b36, 95 },
    { 0x57539b37, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a39b3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6239b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba39b44, 61 },
    { 0xa1839b45, 90 },
    { 0x5e039b46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a39b4c, 87 },
    { 0x17139b4d, 95 },
    { 0xe6339b4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a39b53, 90 },
    { 0, 0 },
    { 0x25439b55, 23 },
    { 0x5da39b55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe39b59, 34 },
    { 0x44139b5a, 61 },
    { 0x81639b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b39b68, 66 },
    { 0x59739b69, 46 },
    { 0x96939b6a, 23 },
    { 0x71039b6b, 43 },
    { 0x7e139b68, 33 },
    { 0x70b39b6d, 33 },
    { 0x19739b6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92039b75, 61 },
    { 0x88139b76, 33 },
    { 0, 0 },
    { 0xefa39b78, 90 },
    { 0x39539b79, 61 },
    { 0xe5b39b7a, 75 },
    { 0x60339b7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc039b7e, 90 },
    { 0xe6739b7f, 69 },
    { 0x60b39b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46239b83, 90 },
    { 0xf3a39b84, 98 },
    { 0x7da39b85, 64 },
    { 0x95e39b85, 55 },
    { 0x75539b87, 33 },
    { 0xef839b85, 61 },
    { 0x4ae39b88, 61 },
    { 0x38239b8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c039b8d, 43 },
    { 0xf1b39b8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f839b91, 61 },
    { 0x11f39b92, 33 },
    { 0xaa239b93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f539b98, 87 },
    { 0xe3939b98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59139b9d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a39ba2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d39ba5, 34 },
    { 0x8d339ba5, 95 },
    { 0x17239ba7, 66 },
    { 0, 0 },
    { 0x87939ba9, 90 },
    { 0, 0 },
    { 0x75339bab, 52 },
    { 0x9b539bab, 93 },
    { 0xd0a39bab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae739bb0, 43 },
    { 0x56c39bb1, 90 },
    { 0x38839bb0, 84 },
    { 0xdad39bb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d439bb7, 55 },
    { 0x12639bb8, 55 },
    { 0x5d639bb9, 98 },
    { 0x5439bba, 23 },
    { 0, 0 },
    { 0x50139bbc, 95 },
    { 0, 0 },
    { 0x42f39bbe, 90 },
    { 0x84c39bbe, 43 },
    { 0x62139bc0, 77 },
    { 0x1539bbf, 90 },
    { 0x8439bc2, 46 },
    { 0x7bb39bc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2539bcd, 90 },
    { 0x2ce39bcd, 77 },
    { 0x7a339bcd, 61 },
    { 0x48039bd0, 33 },
    { 0xfe439bd1, 90 },
    { 0x16039bd2, 61 },
    { 0xc6039bd3, 55 },
    { 0x71a39bd2, 96 },
    { 0, 0 },
    { 0xa39bd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb39bde, 98 },
    { 0x92039bdf, 65 },
    { 0, 0 },
    { 0xa1e39be1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c39be5, 23 },
    { 0, 0 },
    { 0x3b139be7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e39bea, 46 },
    { 0xf8739bea, 67 },
    { 0x69239bec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b39bf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf039bf4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7839bf7, 83 },
    { 0xe6f39bf7, 90 },
    { 0xe3439bf9, 33 },
    { 0x7d939bfa, 43 },
    { 0x48d39bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94839c00, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb439c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f39c06, 33 },
    { 0x44c39c07, 52 },
    { 0x9ea39c06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe39c0e, 99 },
    { 0, 0 },
    { 0x34039c10, 78 },
    { 0x73939c10, 90 },
    { 0x7d139c12, 34 },
    { 0x72c39c13, 95 },
    { 0x5d39c11, 43 },
    { 0xedf39c12, 90 },
    { 0xd0139c10, 69 },
    { 0, 0 },
    { 0xff739c18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8739c27, 95 },
    { 0x30b39c28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7539c2c, 61 },
    { 0x3b839c2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c39c32, 90 },
    { 0, 0 },
    { 0xead39c34, 43 },
    { 0, 0 },
    { 0xb9539c36, 55 },
    { 0, 0 },
    { 0x82139c38, 34 },
    { 0xa3639c39, 43 },
    { 0x33139c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e39c3e, 75 },
    { 0x39b39c3f, 55 },
    { 0x11939c40, 90 },
    { 0xfb239c41, 43 },
    { 0x6339c41, 33 },
    { 0x62839c43, 23 },
    { 0xc9239c43, 76 },
    { 0x30739c40, 61 },
    { 0xf8439c45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77839c49, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c39c4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae39c50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb39c53, 78 },
    { 0xa0739c54, 61 },
    { 0x9a739c55, 33 },
    { 0x42339c56, 81 },
    { 0, 0 },
    { 0x26c39c58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde039c5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71339c62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49539c6b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb39c70, 81 },
    { 0xa5739c71, 55 },
    { 0xcc39c72, 33 },
    { 0xa9c39c70, 90 },
    { 0, 0 },
    { 0x3eb39c75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c139c79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d39c82, 99 },
    { 0x1e339c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26739c88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97639c8c, 90 },
    { 0xbbf39c8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6039c91, 46 },
    { 0xd0939c92, 95 },
    { 0x76339c93, 90 },
    { 0xce039c93, 96 },
    { 0x3439c94, 98 },
    { 0x22d39c96, 90 },
    { 0xe3039c97, 55 },
    { 0x4c239c91, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e39c9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c39c9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf39ca1, 100 },
    { 0xd0839ca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11339ca5, 90 },
    { 0, 0 },
    { 0x60739ca7, 66 },
    { 0xc939ca8, 91 },
    { 0xc8139ca9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c639cac, 17 },
    { 0xe4d39cad, 61 },
    { 0, 0 },
    { 0x59e39caf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96239cb3, 83 },
    { 0x37139cb3, 17 },
    { 0, 0 },
    { 0x84139cb6, 23 },
    { 0x3b839cb7, 33 },
    { 0xb0739cb8, 87 },
    { 0x9ba39cb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639cbf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x45839cc2, 87 },
    { 0, 0 },
    { 0x46339cc4, 23 },
    { 0, 0 },
    { 0x85b39cc6, 95 },
    { 0x13e39cc7, 52 },
    { 0x25639cc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d39ccf, 98 },
    { 0x32f39cd0, 93 },
    { 0xc4139cd1, 40 },
    { 0xdb839cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f39cd6, 68 },
    { 0xc0039cd6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c339cda, 93 },
    { 0xa6139cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb39cdf, 90 },
    { 0x84439ce0, 99 },
    { 0, 0 },
    { 0x11039ce2, 90 },
    { 0xd4c39ce3, 76 },
    { 0xc0139ce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a39ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54239cec, 72 },
    { 0xdbe39ced, 66 },
    { 0xd7439cee, 98 },
    { 0xaa639cee, 94 },
    { 0xf7839cf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9139cf3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e939cfb, 76 },
    { 0, 0 },
    { 0x36439cfd, 43 },
    { 0x3d239cfe, 90 },
    { 0x70839cfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7139d04, 90 },
    { 0x79039d05, 61 },
    { 0x5d739d04, 66 },
    { 0xbe639d07, 69 },
    { 0, 0 },
    { 0xc6239d09, 61 },
    { 0xe4939d09, 43 },
    { 0x88639d0b, 90 },
    { 0xcfe39d0c, 61 },
    { 0xe1439d0a, 76 },
    { 0x23b39d0e, 90 },
    { 0xfb539d0f, 33 },
    { 0x5e939d0f, 43 },
    { 0xbbd39d11, 90 },
    { 0xf2539d0d, 17 },
    { 0xa2839d0e, 90 },
    { 0xe3b39d14, 95 },
    { 0x27f39d15, 87 },
    { 0x13939d0f, 46 },
    { 0x27139d17, 55 },
    { 0xa9a39d0f, 34 },
    { 0x7f839d19, 23 },
    { 0x3f839d11, 76 },
    { 0xd4f39d1b, 98 },
    { 0x63a39d1b, 33 },
    { 0x5d239d1d, 98 },
    { 0xff039d0d, 61 },
    { 0xb2739d1f, 93 },
    { 0xb4739d20, 33 },
    { 0xe5b39d21, 23 },
    { 0x6f939d0d, 43 },
    { 0x64d39d23, 34 },
    { 0, 0 },
    { 0x7d339d25, 66 },
    { 0xd4b39d26, 55 },
    { 0, 0 },
    { 0xbcf39d28, 88 },
    { 0xdf539d28, 34 },
    { 0xedd39d2a, 52 },
    { 0x61f39d2b, 53 },
    { 0x45239d2c, 90 },
    { 0, 0 },
    { 0x96a39d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c39d35, 55 },
    { 0, 0 },
    { 0x65a39d37, 85 },
    { 0x5c439d38, 101 },
    { 0, 0 },
    { 0x72e39d3a, 76 },
    { 0, 0 },
    { 0x87039d3c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca39d3f, 33 },
    { 0x2f139d40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4639d43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe39d47, 76 },
    { 0x21039d48, 66 },
    { 0xb5239d48, 46 },
    { 0x35739d47, 23 },
    { 0xa8439d4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5739d51, 90 },
    { 0x5d39d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61739d55, 19 },
    { 0xf3639d56, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d39d5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27239d62, 78 },
    { 0x99a39d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41739d69, 52 },
    { 0x50539d69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x44239d6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb639d77, 34 },
    { 0, 0 },
    { 0x83239d79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc39d88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x99339d8b, 53 },
    { 0, 0 },
    { 0xb6a39d8d, 91 },
    { 0x51f39d8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2339d92, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f39d96, 93 },
    { 0xf4539d97, 78 },
    { 0x20a39d98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b39d9e, 76 },
    { 0x55d39d9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76239da8, 33 },
    { 0xcdc39da9, 19 },
    { 0xcd139da9, 37 },
    { 0xd6e39dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9739db5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a39db8, 84 },
    { 0, 0 },
    { 0x7f439dba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e39dbd, 76 },
    { 0xccf39dbd, 90 },
    { 0xdb439dbf, 33 },
    { 0x87539dc0, 81 },
    { 0, 0 },
    { 0x54739dc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef639dc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2839dcb, 99 },
    { 0x12839dcc, 98 },
    { 0x46239dcc, 33 },
    { 0xf5b39dce, 98 },
    { 0xdb139dcf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d39dd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba39dd9, 98 },
    { 0, 0 },
    { 0x5e639ddb, 76 },
    { 0x62239ddb, 43 },
    { 0x7f739ddb, 33 },
    { 0, 0 },
    { 0xfdd39ddf, 23 },
    { 0x5ff39de0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f939deb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97539def, 46 },
    { 0xd5139df0, 90 },
    { 0xe1739def, 23 },
    { 0, 0 },
    { 0x639df3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2039dfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed839dfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77439e02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee439e05, 23 },
    { 0, 0 },
    { 0x33e39e07, 80 },
    { 0, 0 },
    { 0x62539e09, 57 },
    { 0xad839e0a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8339e0f, 61 },
    { 0xc039e10, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa39e13, 23 },
    { 0, 0 },
    { 0x3af39e15, 76 },
    { 0, 0 },
    { 0x23f39e17, 95 },
    { 0, 0 },
    { 0xc4939e19, 78 },
    { 0x19b39e1a, 90 },
    { 0xa4039e1b, 75 },
    { 0xf9339e1c, 76 },
    { 0x14039e1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd539e20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf539e23, 78 },
    { 0xa5039e24, 23 },
    { 0xef639e25, 33 },
    { 0, 0 },
    { 0x10839e27, 93 },
    { 0x3ae39e28, 34 },
    { 0, 0 },
    { 0x45239e2a, 90 },
    { 0x42239e2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4639e34, 90 },
    { 0x36b39e35, 61 },
    { 0x8a139e34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e39e39, 84 },
    { 0x86f39e3a, 95 },
    { 0x49739e3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73439e3f, 98 },
    { 0x2c439e40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2039e43, 55 },
    { 0x93339e44, 33 },
    { 0xc7839e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a39e49, 55 },
    { 0x9bf39e49, 33 },
    { 0x81339e4b, 55 },
    { 0xaab39e4c, 77 },
    { 0, 0 },
    { 0xb3139e4e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa39e51, 23 },
    { 0xa9639e52, 34 },
    { 0x97f39e53, 65 },
    { 0, 0 },
    { 0x6239e55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80339e5d, 90 },
    { 0x2a39e5d, 23 },
    { 0x6d239e5f, 90 },
    { 0x3a639e60, 61 },
    { 0x21939e5e, 23 },
    { 0, 0 },
    { 0x16839e63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84539e6b, 66 },
    { 0xae239e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0139e70, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14539e77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a39e85, 33 },
    { 0x83a39e86, 76 },
    { 0, 0 },
    { 0x68939e88, 46 },
    { 0x17b39e89, 90 },
    { 0x3a639e89, 95 },
    { 0xd0639e8b, 95 },
    { 0x7b939e8c, 90 },
    { 0x28539e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b39e90, 95 },
    { 0x25e39e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa39e97, 40 },
    { 0x8ef39e98, 66 },
    { 0xc9e39e97, 90 },
    { 0, 0 },
    { 0x51039e9b, 90 },
    { 0xab739e9c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2439ea2, 52 },
    { 0, 0 },
    { 0x20839ea4, 90 },
    { 0, 0 },
    { 0xf5a39ea6, 52 },
    { 0, 0 },
    { 0x21739ea8, 90 },
    { 0x6a739ea9, 61 },
    { 0x52f39eaa, 90 },
    { 0x54839ea9, 43 },
    { 0xa8439ea9, 90 },
    { 0, 0 },
    { 0xfa639eae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd39eb4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca39eb9, 46 },
    { 0x36239eb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c39ec9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c39ecd, 61 },
    { 0xd5939ece, 55 },
    { 0xab139ecd, 76 },
    { 0xfee39ecd, 90 },
    { 0xcb939ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x80739ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd39eda, 90 },
    { 0xd0c39edb, 23 },
    { 0, 0 },
    { 0x71539edd, 90 },
    { 0x5ea39edd, 77 },
    { 0x6d739ede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5639ee2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f39ee6, 91 },
    { 0, 0 },
    { 0x61239ee8, 96 },
    { 0x6b739ee8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a39eee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b839ef1, 61 },
    { 0, 0 },
    { 0x37139ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd39ef8, 68 },
    { 0xca139ef8, 76 },
    { 0xb6139efa, 100 },
    { 0xe9c39efa, 98 },
    { 0x50b39efc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9239f01, 81 },
    { 0, 0 },
    { 0xca439f03, 23 },
    { 0, 0 },
    { 0x44939f05, 61 },
    { 0xc1b39f06, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xff539f09, 61 },
    { 0x5d539f0a, 66 },
    { 0x12039f0a, 55 },
    { 0x3ce39f0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50439f10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6239f13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6139f18, 68 },
    { 0x7339f19, 81 },
    { 0x11839f1a, 23 },
    { 0x4be39f19, 34 },
    { 0x34e39f1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa39f20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7939f2a, 93 },
    { 0xdaa39f2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd39f2e, 90 },
    { 0, 0 },
    { 0x97a39f30, 33 },
    { 0xc8539f31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3239f36, 36 },
    { 0xfd739f37, 99 },
    { 0, 0 },
    { 0x29939f39, 33 },
    { 0x25839f39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0939f3f, 91 },
    { 0xc8f39f3f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b39f4b, 98 },
    { 0, 0 },
    { 0xf2b39f4d, 34 },
    { 0xd7b39f4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54139f59, 99 },
    { 0xe1739f5a, 34 },
    { 0x1f39f59, 76 },
    { 0, 0 },
    { 0xf5039f5d, 84 },
    { 0, 0 },
    { 0xcef39f5f, 98 },
    { 0x35139f60, 99 },
    { 0, 0 },
    { 0x39d39f62, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8139f65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb39f69, 68 },
    { 0, 0 },
    { 0x57039f6b, 43 },
    { 0xc8539f6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62839f71, 64 },
    { 0xafe39f72, 76 },
    { 0xf1239f72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a39f76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f39f7a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a39f7d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9239f80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a39f86, 78 },
    { 0x23d39f87, 81 },
    { 0x8ff39f86, 98 },
    { 0xd6e39f88, 55 },
    { 0x63739f87, 95 },
    { 0xbab39f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9439f8e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb839f91, 95 },
    { 0x2d339f91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed39f97, 23 },
    { 0x73e39f98, 90 },
    { 0xcb839f97, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1939f9c, 23 },
    { 0x85739f9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf39fa0, 55 },
    { 0x2a939fa1, 90 },
    { 0x9af39fa2, 95 },
    { 0x3c439fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5839fa9, 33 },
    { 0xa9539faa, 90 },
    { 0x8ed39fab, 76 },
    { 0xd3939fac, 61 },
    { 0x7fd39fad, 55 },
    { 0xb6639fae, 99 },
    { 0xfa939fab, 43 },
    { 0xdba39fb0, 61 },
    { 0, 0 },
    { 0x69039fb2, 60 },
    { 0, 0 },
    { 0xc4739fb4, 90 },
    { 0, 0 },
    { 0x9eb39fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d439fbe, 23 },
    { 0xa6939fbf, 55 },
    { 0x12039fc0, 90 },
    { 0x96b39fbf, 95 },
    { 0, 0 },
    { 0xd3339fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0439fc7, 93 },
    { 0x8de39fc8, 93 },
    { 0x49c39fc9, 90 },
    { 0x1939fca, 66 },
    { 0xa6e39fcb, 90 },
    { 0x26c39fc9, 33 },
    { 0xf2239fcc, 85 },
    { 0, 0 },
    { 0x46639fcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3139fd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74939fdd, 55 },
    { 0, 0 },
    { 0x6e239fdf, 46 },
    { 0x68339fe0, 90 },
    { 0x7339fe1, 90 },
    { 0x15c39fe1, 90 },
    { 0xf8839fe3, 98 },
    { 0, 0 },
    { 0xd539fe5, 93 },
    { 0, 0 },
    { 0xc7139fe7, 55 },
    { 0, 0 },
    { 0x2f139fe9, 95 },
    { 0x3ae39fea, 55 },
    { 0x3a439feb, 80 },
    { 0x37a39fea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe39ff3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6239ff6, 61 },
    { 0, 0 },
    { 0xde839ff8, 61 },
    { 0x26f39ff9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x639ffc, 65 },
    { 0xc7439ffd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9133a002, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe213a006, 43 },
    { 0x3483a007, 61 },
    { 0x6e03a008, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0x7553a00b, 33 },
    { 0x8fa3a00c, 94 },
    { 0xf113a00d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed3a011, 84 },
    { 0x9c63a012, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f3a015, 61 },
    { 0xc0c3a016, 80 },
    { 0, 0 },
    { 0x2303a018, 99 },
    { 0xa983a019, 95 },
    { 0, 0 },
    { 0x2f03a01b, 23 },
    { 0, 0 },
    { 0x9e13a01d, 98 },
    { 0, 0 },
    { 0xcd03a01f, 76 },
    { 0x5fb3a020, 23 },
    { 0xea03a020, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4373a027, 90 },
    { 0x7383a027, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb43a02f, 98 },
    { 0xb953a02f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3403a035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df3a038, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a33a03c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd23a041, 84 },
    { 0xd983a042, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c3a048, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd3a04c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa913a04f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a3a055, 55 },
    { 0x48a3a056, 90 },
    { 0x43a3a056, 61 },
    { 0x14c3a055, 98 },
    { 0, 0 },
    { 0xeb73a05a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd523a05d, 100 },
    { 0x7cc3a05d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a3a061, 43 },
    { 0xb423a061, 101 },
    { 0, 0 },
    { 0x72c3a064, 95 },
    { 0xe843a064, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b3a06b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3743a070, 91 },
    { 0, 0 },
    { 0x33e3a072, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9673a078, 75 },
    { 0x5723a079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7623a07e, 33 },
    { 0x1783a07f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac3a082, 95 },
    { 0xf663a083, 87 },
    { 0x2663a083, 95 },
    { 0xef63a083, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7013a088, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b3a08b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c3a08f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab3a093, 98 },
    { 0x8e13a094, 98 },
    { 0xd013a095, 85 },
    { 0xd533a096, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb3a09a, 85 },
    { 0, 0 },
    { 0xc73a09c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1303a0a0, 46 },
    { 0x5e93a0a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e93a0a5, 95 },
    { 0xf53a0a6, 33 },
    { 0x71b3a0a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c3a0ad, 90 },
    { 0x9673a0ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f3a0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d93a0b5, 33 },
    { 0x8a3a0b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2423a0bb, 90 },
    { 0xb693a0bc, 34 },
    { 0xd923a0bd, 77 },
    { 0x2dc3a0be, 72 },
    { 0x8503a0be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873a0c2, 33 },
    { 0x853a0c3, 95 },
    { 0, 0 },
    { 0x7383a0c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d23a0ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b3a0ce, 52 },
    { 0xdab3a0cf, 55 },
    { 0, 0 },
    { 0x3fc3a0d1, 90 },
    { 0x9ac3a0d2, 43 },
    { 0, 0 },
    { 0x20d3a0d4, 90 },
    { 0, 0 },
    { 0x2603a0d6, 55 },
    { 0, 0 },
    { 0x6833a0d8, 33 },
    { 0x8403a0d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e3a0dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf53a0e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5493a0e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf133a0eb, 55 },
    { 0, 0 },
    { 0xf6c3a0ed, 78 },
    { 0xf813a0ed, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4293a0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13a0f5, 78 },
    { 0, 0 },
    { 0xb7b3a0f7, 67 },
    { 0xc23a0f7, 61 },
    { 0xc753a0f9, 23 },
    { 0, 0 },
    { 0x1113a0fb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1693a0fe, 98 },
    { 0, 0 },
    { 0x1c93a100, 85 },
    { 0xd6b3a100, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d3a104, 98 },
    { 0, 0 },
    { 0x21a3a106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc3a10a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f3a115, 93 },
    { 0x7b3a116, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c33a11d, 33 },
    { 0xeb63a11e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f83a125, 96 },
    { 0x7d13a126, 90 },
    { 0, 0 },
    { 0xc53a128, 61 },
    { 0, 0 },
    { 0xf103a12a, 26 },
    { 0xb383a12b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b3a131, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c43a136, 78 },
    { 0x7033a136, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc953a13a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d73a13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e3a142, 68 },
    { 0x6ab3a143, 43 },
    { 0x9483a144, 55 },
    { 0x4163a143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93a14c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3473a14f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc593a156, 55 },
    { 0x9a83a157, 83 },
    { 0x2b3a158, 88 },
    { 0x73a157, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9663a15f, 38 },
    { 0x8833a160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f3a164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a63a16a, 55 },
    { 0x1df3a16b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933a171, 38 },
    { 0xbf13a172, 99 },
    { 0x6773a173, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd983a177, 90 },
    { 0x2673a178, 61 },
    { 0x6053a177, 46 },
    { 0, 0 },
    { 0x4143a17b, 61 },
    { 0, 0 },
    { 0x8d63a17d, 23 },
    { 0x2e33a17d, 95 },
    { 0, 0 },
    { 0x1e93a180, 61 },
    { 0xda83a181, 95 },
    { 0xf263a182, 95 },
    { 0x58d3a183, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103a18b, 95 },
    { 0xc5c3a18b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef3a191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323a197, 98 },
    { 0x6a13a197, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac3a19b, 91 },
    { 0x70a3a19c, 87 },
    { 0xfa83a19d, 78 },
    { 0xf2a3a19e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa3a1a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf063a1a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc73a1b6, 85 },
    { 0x9b43a1b7, 55 },
    { 0xb4f3a1b8, 52 },
    { 0x1633a1b8, 98 },
    { 0x47e3a1ba, 52 },
    { 0, 0 },
    { 0x5a53a1bc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c3a1bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c3a1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d83a1c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb193a1cc, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3353a1d4, 34 },
    { 0x2ee3a1d4, 95 },
    { 0, 0 },
    { 0xe0f3a1d7, 33 },
    { 0x9c3a1d7, 66 },
    { 0, 0 },
    { 0xf263a1da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc3a1df, 98 },
    { 0xd273a1e0, 90 },
    { 0, 0 },
    { 0x1a53a1e2, 99 },
    { 0x1b93a1e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc3a1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe933a1eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc163a1ee, 34 },
    { 0xaf53a1ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe813a1f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b3a1f6, 46 },
    { 0, 0 },
    { 0xca33a1f8, 93 },
    { 0xa383a1f9, 76 },
    { 0xd893a1fa, 95 },
    { 0x98e3a1fb, 100 },
    { 0x8a13a1fc, 76 },
    { 0x73d3a1fb, 33 },
    { 0xb6c3a1fe, 95 },
    { 0xd1d3a1ff, 90 },
    { 0, 0 },
    { 0x7a93a201, 90 },
    { 0, 0 },
    { 0x4183a203, 33 },
    { 0xfe53a204, 91 },
    { 0, 0 },
    { 0x5573a206, 90 },
    { 0x9663a207, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b3a20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3a214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e3a21e, 95 },
    { 0x33c3a21f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e3a223, 90 },
    { 0xf2b3a223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8463a227, 46 },
    { 0x1e83a227, 98 },
    { 0x2b23a227, 91 },
    { 0, 0 },
    { 0x6303a22b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763a234, 33 },
    { 0x8333a235, 45 },
    { 0x79b3a236, 23 },
    { 0, 0 },
    { 0x84d3a238, 95 },
    { 0, 0 },
    { 0xc933a23a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3a23e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d3a242, 61 },
    { 0x73c3a243, 98 },
    { 0xd373a244, 55 },
    { 0x6ec3a244, 90 },
    { 0x8613a243, 75 },
    { 0xf523a245, 33 },
    { 0x1783a244, 23 },
    { 0, 0 },
    { 0xc0d3a24a, 100 },
    { 0xad3a24b, 90 },
    { 0, 0 },
    { 0x54b3a24d, 93 },
    { 0x7b3a24d, 23 },
    { 0xf6a3a24d, 43 },
    { 0x26e3a250, 61 },
    { 0xb723a251, 78 },
    { 0, 0 },
    { 0x5a3a253, 78 },
    { 0, 0 },
    { 0x5863a255, 55 },
    { 0, 0 },
    { 0x7a3a257, 33 },
    { 0, 0 },
    { 0x9243a259, 98 },
    { 0x2f93a25a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03a261, 98 },
    { 0xb6c3a261, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b3a26d, 98 },
    { 0x71e3a26e, 81 },
    { 0xe2e3a26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd433a275, 90 },
    { 0, 0 },
    { 0x3d23a277, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a27c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc93a281, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9613a285, 33 },
    { 0x613a286, 33 },
    { 0xbfd3a285, 17 },
    { 0xce13a288, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef73a28e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7993a293, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa3a297, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b63a29b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243a2a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8443a2a5, 76 },
    { 0, 0 },
    { 0xd6a3a2a7, 33 },
    { 0x95a3a2a8, 90 },
    { 0xb873a2a9, 87 },
    { 0x2bb3a2aa, 98 },
    { 0x5873a2a9, 98 },
    { 0x3183a2ac, 23 },
    { 0x5873a2ad, 33 },
    { 0xe1a3a2ac, 55 },
    { 0x68f3a2ac, 68 },
    { 0, 0 },
    { 0xc593a2b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7213a2b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e83a2b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b73a2c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7633a2c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9273a2d1, 43 },
    { 0x3663a2d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013a2d5, 61 },
    { 0x3003a2d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2813a2dc, 85 },
    { 0xa503a2dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe893a2e1, 90 },
    { 0x8723a2e1, 98 },
    { 0, 0 },
    { 0x7f63a2e4, 43 },
    { 0x6523a2e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33a2ef, 91 },
    { 0xd043a2f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf483a2fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1103a304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b3a307, 23 },
    { 0xbda3a307, 52 },
    { 0xf713a307, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca3a30c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7583a311, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6153a314, 91 },
    { 0x4f03a315, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb283a319, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323a327, 90 },
    { 0x2773a327, 90 },
    { 0xe2b3a329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563a32c, 61 },
    { 0, 0 },
    { 0x57f3a32e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce3a331, 75 },
    { 0x1e63a332, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b3a335, 46 },
    { 0, 0 },
    { 0x2cb3a337, 98 },
    { 0xb653a338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da3a33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a345, 61 },
    { 0xf4b3a346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe773a34a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f3a34d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xed43a350, 65 },
    { 0xbe33a350, 61 },
    { 0x35c3a352, 81 },
    { 0x89a3a353, 76 },
    { 0x28a3a354, 98 },
    { 0xd43a355, 76 },
    { 0x3f23a355, 23 },
    { 0, 0 },
    { 0xe733a358, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb3a35e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x663a361, 95 },
    { 0x6df3a362, 66 },
    { 0, 0 },
    { 0x38a3a364, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf553a367, 90 },
    { 0x793a368, 98 },
    { 0x86c3a369, 93 },
    { 0, 0 },
    { 0xd213a36b, 66 },
    { 0x6b83a36c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8833a372, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb993a376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a3a379, 76 },
    { 0, 0 },
    { 0xf283a37b, 90 },
    { 0xbef3a37c, 95 },
    { 0x7c03a37b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf473a384, 33 },
    { 0xf83a385, 78 },
    { 0x6913a386, 90 },
    { 0x1f53a387, 23 },
    { 0x4183a385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d83a38c, 76 },
    { 0x14a3a38d, 80 },
    { 0x3c83a38c, 95 },
    { 0xca03a38c, 69 },
    { 0x8e3a390, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b3a395, 95 },
    { 0xa6c3a396, 61 },
    { 0x6d73a397, 33 },
    { 0x9023a396, 90 },
    { 0x6fa3a399, 98 },
    { 0, 0 },
    { 0x2b13a39b, 99 },
    { 0, 0 },
    { 0x1873a39d, 87 },
    { 0x1013a39e, 61 },
    { 0x743a39e, 90 },
    { 0x19f3a39f, 90 },
    { 0xf933a3a1, 66 },
    { 0x6143a3a1, 43 },
    { 0x6ff3a3a3, 55 },
    { 0xd343a3a3, 67 },
    { 0x1543a39d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe303a3a8, 43 },
    { 0, 0 },
    { 0xc583a3aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763a3ae, 76 },
    { 0x2fb3a3ae, 61 },
    { 0x3653a3b0, 23 },
    { 0xaa83a3b0, 90 },
    { 0xc33a3b2, 66 },
    { 0x9283a3b2, 46 },
    { 0xf4b3a3af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9643a3b8, 33 },
    { 0xd053a3b9, 33 },
    { 0xa9a3a3ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3a3bf, 40 },
    { 0x2353a3c0, 88 },
    { 0x3bf3a3c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13a3c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd953a3c8, 95 },
    { 0x4e13a3c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e3a3d0, 43 },
    { 0x6893a3d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3a3d7, 100 },
    { 0, 0 },
    { 0x77b3a3d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd3a3de, 100 },
    { 0x8993a3df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e3a3e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb3a3ed, 99 },
    { 0x3c43a3ee, 78 },
    { 0x3693a3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf3a3f2, 95 },
    { 0x6dd3a3f3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b03a3f8, 85 },
    { 0x9ca3a3f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f83a3fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a401, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a404, 87 },
    { 0, 0 },
    { 0x4143a406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73a40e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a33a411, 76 },
    { 0x7733a412, 65 },
    { 0xc353a411, 61 },
    { 0x5393a414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae3a418, 95 },
    { 0xf1a3a419, 55 },
    { 0xdf13a418, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6113a41d, 23 },
    { 0x57b3a41d, 75 },
    { 0, 0 },
    { 0xf393a420, 66 },
    { 0xdea3a420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff3a429, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93a42f, 84 },
    { 0x1dc3a42f, 93 },
    { 0xb8b3a42f, 90 },
    { 0xaf13a42f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f3a436, 95 },
    { 0, 0 },
    { 0xc813a438, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2693a440, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53a444, 55 },
    { 0, 0 },
    { 0xf6a3a446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de3a44c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3a44f, 34 },
    { 0, 0 },
    { 0xea43a451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe613a454, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd3a457, 90 },
    { 0, 0 },
    { 0x60d3a459, 90 },
    { 0xe083a45a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc273a462, 100 },
    { 0x5e23a463, 91 },
    { 0x91b3a463, 95 },
    { 0x6bd3a465, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a3a468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b43a46c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc263a475, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f3a479, 61 },
    { 0, 0 },
    { 0x4993a47b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd3a47e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f3a483, 90 },
    { 0, 0 },
    { 0xac83a485, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce3a48b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133a498, 23 },
    { 0x7893a499, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d53a49c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5183a4a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5733a4aa, 76 },
    { 0xbf13a4ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6643a4ae, 65 },
    { 0x7893a4af, 95 },
    { 0xb6e3a4b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1183a4b3, 33 },
    { 0xa1e3a4b4, 23 },
    { 0xfd33a4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9093a4b9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec43a4cb, 33 },
    { 0, 0 },
    { 0x8a83a4cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133a4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd33a4d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f73a4d7, 95 },
    { 0x4e3a4d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa013a4dc, 65 },
    { 0x63a4dd, 66 },
    { 0x15d3a4dd, 33 },
    { 0xef23a4df, 85 },
    { 0xca33a4df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b3a4e3, 36 },
    { 0xd443a4e4, 95 },
    { 0x8823a4e5, 90 },
    { 0xf903a4e4, 98 },
    { 0x84d3a4e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb423a4ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c53a4f0, 91 },
    { 0xb153a4f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c83a4f4, 98 },
    { 0x97e3a4f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea3a4fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893a500, 95 },
    { 0, 0 },
    { 0xb6a3a502, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc473a50c, 95 },
    { 0x7c43a50d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e3a511, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593a514, 33 },
    { 0xfb83a514, 77 },
    { 0xbee3a516, 98 },
    { 0x8bd3a517, 66 },
    { 0x3c13a518, 90 },
    { 0x3733a515, 90 },
    { 0, 0 },
    { 0x7d73a51b, 33 },
    { 0, 0 },
    { 0x8a93a51d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3a520, 33 },
    { 0x2293a520, 85 },
    { 0, 0 },
    { 0x4543a523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e3a527, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3363a533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca3a538, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a3a544, 61 },
    { 0xee83a544, 61 },
    { 0, 0 },
    { 0x4d53a547, 98 },
    { 0, 0 },
    { 0x11a3a549, 61 },
    { 0xcc83a54a, 76 },
    { 0x58a3a54b, 98 },
    { 0xcc43a54c, 87 },
    { 0xd703a54d, 43 },
    { 0x39a3a54a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa3a551, 90 },
    { 0x3ee3a551, 33 },
    { 0x7ef3a553, 78 },
    { 0xc793a553, 85 },
    { 0, 0 },
    { 0xf353a556, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e3a55e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6493a565, 95 },
    { 0x1c23a566, 95 },
    { 0x9193a567, 95 },
    { 0x4823a568, 33 },
    { 0xb233a566, 52 },
    { 0x663a56a, 85 },
    { 0xbd03a56b, 55 },
    { 0x3473a56c, 90 },
    { 0x1b3a568, 98 },
    { 0xf603a56a, 23 },
    { 0x5123a566, 56 },
    { 0x2e93a570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd193a574, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd743a579, 23 },
    { 0, 0 },
    { 0x5ff3a57b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba3a57e, 33 },
    { 0x1a73a57e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4533a584, 33 },
    { 0, 0 },
    { 0x6483a586, 98 },
    { 0xcba3a586, 33 },
    { 0x5863a588, 78 },
    { 0xa003a588, 98 },
    { 0x5c3a589, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf3a592, 78 },
    { 0x4523a592, 98 },
    { 0, 0 },
    { 0x3a43a595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a3a599, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9593a5a0, 5 },
    { 0, 0 },
    { 0xc803a5a2, 43 },
    { 0x9983a5a2, 33 },
    { 0x4603a5a4, 90 },
    { 0x4ae3a5a2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad3a5a9, 90 },
    { 0x8ab3a5aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc3a5b2, 90 },
    { 0, 0 },
    { 0xafa3a5b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6263a5b9, 23 },
    { 0x693a5b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3a5bf, 23 },
    { 0x5af3a5c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd03a5c3, 88 },
    { 0, 0 },
    { 0xe413a5c5, 33 },
    { 0x903a5c6, 65 },
    { 0, 0 },
    { 0x2303a5c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a3a5ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663a5d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03a5d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3a5d9, 23 },
    { 0, 0 },
    { 0xbfe3a5db, 34 },
    { 0xecb3a5dc, 90 },
    { 0xba13a5dd, 80 },
    { 0, 0 },
    { 0xa743a5df, 66 },
    { 0xdc73a5e0, 98 },
    { 0xd83a5e1, 76 },
    { 0x62c3a5e2, 91 },
    { 0x64b3a5e0, 46 },
    { 0x6f3a5e4, 43 },
    { 0xa9b3a5e4, 33 },
    { 0x693a5e6, 33 },
    { 0x7033a5e6, 33 },
    { 0, 0 },
    { 0x2e93a5e9, 76 },
    { 0x93b3a5e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a3a5ed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013a5f5, 69 },
    { 0xbcc3a5f5, 98 },
    { 0xa353a5f7, 61 },
    { 0, 0 },
    { 0x1933a5f9, 61 },
    { 0x8d23a5f9, 61 },
    { 0x2763a5fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f03a5ff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb63a603, 98 },
    { 0, 0 },
    { 0x3b43a605, 23 },
    { 0, 0 },
    { 0x1083a607, 95 },
    { 0xb813a608, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9983a615, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a53a61b, 61 },
    { 0, 0 },
    { 0x7d13a61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4603a622, 23 },
    { 0xb013a622, 61 },
    { 0xd463a622, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e3a629, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc3a62d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a3a633, 23 },
    { 0, 0 },
    { 0x6583a635, 100 },
    { 0xa4d3a636, 91 },
    { 0xacd3a635, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda53a63e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d93a646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf153a649, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9033a64e, 55 },
    { 0x8383a64e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a3a652, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b3a657, 58 },
    { 0, 0 },
    { 0xe393a659, 98 },
    { 0xa5c3a65a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a93a65f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8483a663, 98 },
    { 0, 0 },
    { 0xe303a665, 78 },
    { 0x77b3a666, 17 },
    { 0x5153a665, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1953a66a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec3a671, 66 },
    { 0xd143a672, 34 },
    { 0x6d03a673, 61 },
    { 0x1e03a671, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf3a682, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf743a687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa63a68a, 33 },
    { 0x1763a68a, 43 },
    { 0x1613a68c, 84 },
    { 0xcb93a68c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e3a690, 95 },
    { 0xe183a690, 33 },
    { 0xcc33a692, 61 },
    { 0x3183a693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb953a697, 99 },
    { 0xc303a698, 98 },
    { 0xaf03a698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb93a6a1, 23 },
    { 0, 0 },
    { 0x8c23a6a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f3a6a6, 52 },
    { 0xdf93a6a7, 36 },
    { 0, 0 },
    { 0x2733a6a9, 61 },
    { 0, 0 },
    { 0xd1c3a6ab, 90 },
    { 0x8663a6ac, 33 },
    { 0xf8e3a6ad, 90 },
    { 0xead3a6ac, 76 },
    { 0xb903a6af, 78 },
    { 0xd023a6af, 33 },
    { 0x14a3a6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a3a6b4, 46 },
    { 0x2883a6b5, 49 },
    { 0x73d3a6b4, 43 },
    { 0x5de3a6b6, 33 },
    { 0xfe03a6b5, 61 },
    { 0x7b03a6b8, 90 },
    { 0x82a3a6ba, 55 },
    { 0, 0 },
    { 0xca53a6bc, 83 },
    { 0xe453a6bd, 84 },
    { 0x8403a6bd, 66 },
    { 0x3cc3a6bf, 33 },
    { 0x94b3a6bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b3a6c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e3a6c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873a6cc, 76 },
    { 0x1323a6cd, 34 },
    { 0x8453a6ce, 98 },
    { 0, 0 },
    { 0x39f3a6d0, 33 },
    { 0xf033a6d1, 98 },
    { 0, 0 },
    { 0x6343a6d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d3a6d9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e3a6de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003a6e3, 98 },
    { 0xe063a6e4, 61 },
    { 0xdec3a6e5, 91 },
    { 0x3be3a6e6, 93 },
    { 0, 0 },
    { 0xff33a6e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9073a6ec, 33 },
    { 0x7f3a6ed, 43 },
    { 0xb933a6ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2043a6f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e43a6f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a3a6fd, 98 },
    { 0, 0 },
    { 0x4963a6ff, 98 },
    { 0xe393a6ff, 61 },
    { 0xb3b3a700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053a705, 33 },
    { 0xe923a705, 76 },
    { 0x1f43a707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc3a70a, 69 },
    { 0x18d3a70a, 93 },
    { 0xe4a3a70b, 98 },
    { 0x9863a70d, 61 },
    { 0, 0 },
    { 0xf063a70f, 90 },
    { 0, 0 },
    { 0x8e13a711, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f3a71b, 55 },
    { 0xb953a71c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93a71f, 23 },
    { 0x93a3a71f, 95 },
    { 0x5783a720, 43 },
    { 0x60e3a722, 90 },
    { 0x40a3a71f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3a726, 6 },
    { 0x7b63a727, 67 },
    { 0xfa73a728, 87 },
    { 0, 0 },
    { 0xce83a72a, 61 },
    { 0, 0 },
    { 0x55e3a72c, 61 },
    { 0xd6f3a72c, 33 },
    { 0, 0 },
    { 0x1723a72f, 76 },
    { 0xf013a72f, 46 },
    { 0x6123a730, 61 },
    { 0, 0 },
    { 0x79e3a733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa243a737, 90 },
    { 0x6463a738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a73c, 90 },
    { 0x3013a73d, 52 },
    { 0xda43a73d, 94 },
    { 0x2d13a73c, 61 },
    { 0x2363a740, 93 },
    { 0xf0c3a73c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b3a744, 95 },
    { 0xea23a745, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6403a74a, 93 },
    { 0, 0 },
    { 0x2d13a74c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3a755, 61 },
    { 0xc23a756, 33 },
    { 0xacb3a757, 55 },
    { 0xdc33a755, 34 },
    { 0xeb03a759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b3a75d, 33 },
    { 0, 0 },
    { 0xf1f3a75f, 55 },
    { 0x22c3a760, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9763a766, 85 },
    { 0, 0 },
    { 0xf113a768, 55 },
    { 0x9743a769, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d3a773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f3a777, 95 },
    { 0x4ac3a778, 68 },
    { 0, 0 },
    { 0xc153a77a, 43 },
    { 0, 0 },
    { 0x3553a77c, 23 },
    { 0, 0 },
    { 0x2cf3a77e, 90 },
    { 0, 0 },
    { 0xd93a780, 98 },
    { 0, 0 },
    { 0x3e13a782, 78 },
    { 0xdfc3a783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c83a786, 33 },
    { 0, 0 },
    { 0x7463a788, 33 },
    { 0xd423a789, 61 },
    { 0x61f3a789, 96 },
    { 0x64c3a789, 69 },
    { 0, 0 },
    { 0x7e93a78d, 95 },
    { 0, 0 },
    { 0x9f43a78f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a794, 33 },
    { 0, 0 },
    { 0xc423a796, 78 },
    { 0, 0 },
    { 0xef83a798, 33 },
    { 0xd753a799, 61 },
    { 0x41a3a799, 33 },
    { 0x3013a79b, 90 },
    { 0xfb43a79b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5973a79f, 100 },
    { 0xa373a79f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc723a7a7, 43 },
    { 0x1813a7a8, 84 },
    { 0x4e43a7a8, 87 },
    { 0x5453a7aa, 52 },
    { 0, 0 },
    { 0xa413a7ac, 90 },
    { 0x7de3a7ac, 90 },
    { 0x1083a7ad, 93 },
    { 0x3883a7af, 52 },
    { 0xac53a7ac, 52 },
    { 0xc383a7b0, 85 },
    { 0x6563a7b2, 61 },
    { 0xf8d3a7b3, 61 },
    { 0x3703a7b3, 76 },
    { 0x5b93a7b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e3a7be, 84 },
    { 0, 0 },
    { 0x1093a7c0, 88 },
    { 0xfd53a7c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad3a7c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8003a7d2, 90 },
    { 0x73d3a7d2, 33 },
    { 0x8693a7d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3863a7dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc493a7df, 9 },
    { 0x66b3a7e0, 90 },
    { 0xdf03a7e1, 46 },
    { 0xff13a7e2, 77 },
    { 0xc0c3a7e2, 55 },
    { 0x2433a7e0, 98 },
    { 0xad03a7e5, 52 },
    { 0x1d23a7e6, 98 },
    { 0x80f3a7e5, 90 },
    { 0x943a7e8, 76 },
    { 0x3173a7e9, 43 },
    { 0x7e63a7e5, 90 },
    { 0x91e3a7eb, 43 },
    { 0x1363a7ea, 55 },
    { 0x2093a7e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e3a7f1, 61 },
    { 0xe7e3a7f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde33a7f8, 93 },
    { 0x37b3a7f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2083a7fe, 43 },
    { 0x2993a7ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633a811, 98 },
    { 0, 0 },
    { 0x283a813, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f3a817, 76 },
    { 0x3c83a818, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a73a81c, 33 },
    { 0x2a03a81d, 99 },
    { 0xd113a81e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba3a825, 23 },
    { 0, 0 },
    { 0x8d73a827, 61 },
    { 0xc2f3a827, 76 },
    { 0x3e93a829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee83a82c, 23 },
    { 0xba53a82d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd903a839, 46 },
    { 0x3b63a83a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b3a841, 67 },
    { 0, 0 },
    { 0xf6e3a843, 23 },
    { 0xc6f3a844, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xac63a847, 93 },
    { 0x7af3a847, 99 },
    { 0, 0 },
    { 0x1cd3a84a, 33 },
    { 0, 0 },
    { 0xc73a84c, 100 },
    { 0xda73a84c, 33 },
    { 0xe33a84e, 61 },
    { 0xfca3a84d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313a857, 90 },
    { 0, 0 },
    { 0x75b3a859, 61 },
    { 0, 0 },
    { 0x2c93a85b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e3a85e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb83a861, 90 },
    { 0, 0 },
    { 0xaaa3a863, 95 },
    { 0, 0 },
    { 0xa573a865, 96 },
    { 0x2bf3a865, 23 },
    { 0x6693a867, 95 },
    { 0, 0 },
    { 0x2d03a869, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783a86c, 90 },
    { 0, 0 },
    { 0xb933a86e, 76 },
    { 0x1f53a86f, 36 },
    { 0x99f3a870, 98 },
    { 0xeea3a871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6513a877, 23 },
    { 0x6143a878, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4713a87c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f13a87f, 90 },
    { 0xed83a880, 100 },
    { 0x6583a87f, 95 },
    { 0xf643a882, 81 },
    { 0x5953a882, 85 },
    { 0xb9e3a884, 55 },
    { 0x29a3a885, 90 },
    { 0xa983a886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5963a889, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8623a88d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c3a891, 100 },
    { 0x65f3a892, 28 },
    { 0, 0 },
    { 0x75c3a894, 61 },
    { 0xebf3a894, 90 },
    { 0, 0 },
    { 0x9f83a897, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead3a89e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b3a8a1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc83a8a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6213a8af, 33 },
    { 0xfa33a8b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f3a8b3, 23 },
    { 0xca43a8b4, 90 },
    { 0, 0 },
    { 0x39d3a8b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff83a8c0, 33 },
    { 0x23c3a8c1, 90 },
    { 0x9863a8c1, 61 },
    { 0xa0e3a8c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9153a8c6, 43 },
    { 0, 0 },
    { 0xf0e3a8c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd3a8cc, 91 },
    { 0xb243a8cd, 81 },
    { 0xc093a8cc, 33 },
    { 0xdae3a8cf, 34 },
    { 0x2213a8cc, 17 },
    { 0xa1b3a8d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c3a8db, 95 },
    { 0x1c93a8db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee3a8e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed3a8e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa793a8ec, 85 },
    { 0, 0 },
    { 0xc433a8ee, 86 },
    { 0xbfd3a8ef, 43 },
    { 0xbf23a8f0, 98 },
    { 0xf53a8ef, 33 },
    { 0xba23a8ef, 23 },
    { 0, 0 },
    { 0x1dd3a8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d23a8f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f3a8fc, 61 },
    { 0, 0 },
    { 0x2c63a8fe, 80 },
    { 0x5393a8ff, 98 },
    { 0xc4d3a8fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e3a904, 90 },
    { 0x3d3a904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d3a908, 61 },
    { 0xb613a909, 80 },
    { 0x6cb3a909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053a915, 23 },
    { 0xec03a915, 23 },
    { 0x2fa3a917, 90 },
    { 0xb393a918, 94 },
    { 0xc3f3a917, 100 },
    { 0xb0a3a91a, 61 },
    { 0xa083a91b, 90 },
    { 0xa013a919, 46 },
    { 0x7173a915, 95 },
    { 0x1b33a91e, 76 },
    { 0x5a23a917, 98 },
    { 0, 0 },
    { 0x4d73a921, 90 },
    { 0, 0 },
    { 0xe653a923, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd3a926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5993a929, 95 },
    { 0xa863a92a, 78 },
    { 0xa2c3a92a, 90 },
    { 0, 0 },
    { 0x4bd3a92d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x773a930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c33a93c, 76 },
    { 0x99c3a93d, 95 },
    { 0x8ec3a93d, 98 },
    { 0xddc3a93f, 76 },
    { 0, 0 },
    { 0x7e73a941, 90 },
    { 0x9773a942, 33 },
    { 0x1c03a943, 61 },
    { 0x5fd3a943, 34 },
    { 0x2a03a945, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a3a954, 95 },
    { 0xd93a954, 83 },
    { 0, 0 },
    { 0x73a3a957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a3a95d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383a963, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c3a968, 76 },
    { 0x2443a969, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f3a96f, 23 },
    { 0xea33a96f, 100 },
    { 0x90b3a971, 85 },
    { 0xd523a970, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7503a976, 98 },
    { 0x5153a977, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c3a97d, 61 },
    { 0x41f3a97e, 90 },
    { 0xee13a97f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb023a982, 33 },
    { 0x5da3a982, 99 },
    { 0x5253a982, 61 },
    { 0x6aa3a983, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3a98a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd973a98d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6983a995, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc933a999, 61 },
    { 0, 0 },
    { 0x1b13a99b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa193a99e, 33 },
    { 0, 0 },
    { 0xba93a9a0, 55 },
    { 0x253a9a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3213a9a4, 77 },
    { 0x19c3a9a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb53a9a8, 23 },
    { 0, 0 },
    { 0x25b3a9aa, 95 },
    { 0xbc53a9ab, 87 },
    { 0, 0 },
    { 0x14c3a9ad, 90 },
    { 0, 0 },
    { 0xff43a9af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa613a9b2, 55 },
    { 0xc953a9b3, 17 },
    { 0x97d3a9b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f53a9b7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc3a9ba, 33 },
    { 0xd83a9ba, 55 },
    { 0, 0 },
    { 0xcae3a9bd, 98 },
    { 0, 0 },
    { 0x6923a9bf, 55 },
    { 0x6b63a9c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a33a9c6, 90 },
    { 0xe7d3a9c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a13a9ce, 40 },
    { 0, 0 },
    { 0xe3c3a9d0, 33 },
    { 0x2ef3a9d0, 23 },
    { 0xb73a9d2, 61 },
    { 0, 0 },
    { 0x6c33a9d4, 55 },
    { 0x1723a9d5, 76 },
    { 0, 0 },
    { 0x7a63a9d7, 23 },
    { 0x6223a9d8, 93 },
    { 0x6f73a9d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9533a9dd, 61 },
    { 0xb13a9dd, 90 },
    { 0xec63a9df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23a9e3, 52 },
    { 0, 0 },
    { 0x1ed3a9e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd3a9e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e3a9ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab3a9ef, 23 },
    { 0xc553a9ef, 95 },
    { 0xcd43a9f0, 85 },
    { 0x2bc3a9f0, 93 },
    { 0, 0 },
    { 0x4103a9f4, 90 },
    { 0, 0 },
    { 0xd3f3a9f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e3a9fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753aa01, 36 },
    { 0x45e3aa01, 33 },
    { 0, 0 },
    { 0x48e3aa04, 33 },
    { 0xf423aa05, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7653aa09, 78 },
    { 0x6db3aa0a, 33 },
    { 0, 0 },
    { 0xa8d3aa0c, 91 },
    { 0x4ea3aa0d, 55 },
    { 0xbf63aa0e, 98 },
    { 0, 0 },
    { 0xd1e3aa10, 90 },
    { 0, 0 },
    { 0xd703aa12, 68 },
    { 0x4583aa13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83aa17, 61 },
    { 0, 0 },
    { 0x6a33aa19, 90 },
    { 0xccd3aa1a, 33 },
    { 0x4d43aa19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d63aa1f, 55 },
    { 0xd813aa20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e3aa23, 33 },
    { 0x3c33aa24, 76 },
    { 0x5433aa23, 90 },
    { 0x72a3aa26, 34 },
    { 0x8a83aa27, 61 },
    { 0x62a3aa28, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253aa31, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113aa36, 88 },
    { 0, 0 },
    { 0x24a3aa38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483aa3b, 90 },
    { 0x4503aa3c, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd3aa3f, 91 },
    { 0, 0 },
    { 0x7ca3aa41, 23 },
    { 0xf993aa41, 23 },
    { 0, 0 },
    { 0x33f3aa44, 90 },
    { 0x7493aa45, 95 },
    { 0xb9c3aa46, 33 },
    { 0xc6b3aa47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3323aa4a, 34 },
    { 0xcb03aa4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c23aa50, 55 },
    { 0x7ec3aa51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d3aa56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2953aa5a, 76 },
    { 0x33a3aa5b, 90 },
    { 0, 0 },
    { 0x97c3aa5d, 93 },
    { 0xc533aa5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa693aa61, 33 },
    { 0x2be3aa61, 90 },
    { 0x62d3aa63, 34 },
    { 0x3c53aa64, 65 },
    { 0xdac3aa61, 91 },
    { 0x8d33aa65, 77 },
    { 0, 0 },
    { 0x12a3aa68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc853aa6b, 98 },
    { 0xdfa3aa6c, 43 },
    { 0x5a43aa6d, 77 },
    { 0x1c53aa6e, 98 },
    { 0x7373aa6f, 43 },
    { 0xc253aa6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa933aa73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe843aa77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3aa7d, 87 },
    { 0x3a93aa7d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f23aa8c, 46 },
    { 0, 0 },
    { 0xb953aa8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3633aa93, 91 },
    { 0, 0 },
    { 0x4463aa95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f13aa99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b3aa9e, 85 },
    { 0, 0 },
    { 0x7d43aaa0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5763aaa3, 90 },
    { 0x1203aaa4, 68 },
    { 0xadb3aaa5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b3aaa9, 100 },
    { 0x8e13aaaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd03aaae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8453aab2, 67 },
    { 0x323aab3, 44 },
    { 0x2673aab4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f3aab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e3aaba, 61 },
    { 0x3383aaba, 33 },
    { 0xdc13aaba, 55 },
    { 0, 0 },
    { 0x2783aabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc553aac2, 55 },
    { 0, 0 },
    { 0x6383aac4, 76 },
    { 0x6113aac5, 90 },
    { 0xf143aac4, 90 },
    { 0, 0 },
    { 0xde33aac8, 42 },
    { 0, 0 },
    { 0xd0d3aaca, 78 },
    { 0x8783aacb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d3aad7, 52 },
    { 0x6083aad7, 46 },
    { 0, 0 },
    { 0x293aada, 34 },
    { 0x54b3aadb, 101 },
    { 0x63a3aadc, 34 },
    { 0, 0 },
    { 0xa0f3aade, 61 },
    { 0, 0 },
    { 0x4dc3aae0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe13aae5, 78 },
    { 0, 0 },
    { 0xa763aae7, 95 },
    { 0, 0 },
    { 0xa2b3aae9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc3aaed, 33 },
    { 0x2553aaee, 90 },
    { 0x70f3aaee, 98 },
    { 0, 0 },
    { 0x4173aaf1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb3aaf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5123aafa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f3ab01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9143ab04, 61 },
    { 0, 0 },
    { 0xd0e3ab06, 43 },
    { 0xac3ab07, 33 },
    { 0xeb43ab06, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce3ab0b, 34 },
    { 0, 0 },
    { 0xf2f3ab0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c3ab14, 55 },
    { 0x2d23ab15, 23 },
    { 0xbb3ab15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa823ab19, 52 },
    { 0, 0 },
    { 0x6963ab1b, 64 },
    { 0x9463ab1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9413ab20, 95 },
    { 0x76b3ab21, 61 },
    { 0, 0 },
    { 0xd143ab23, 90 },
    { 0xbfc3ab24, 90 },
    { 0x75e3ab25, 78 },
    { 0x1953ab26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c3ab31, 61 },
    { 0x7973ab32, 55 },
    { 0xf13ab31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa3ab38, 55 },
    { 0xbcf3ab39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d3ab3c, 55 },
    { 0, 0 },
    { 0x2d13ab3e, 61 },
    { 0x60c3ab3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8543ab42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5613ab48, 93 },
    { 0, 0 },
    { 0xead3ab4a, 90 },
    { 0, 0 },
    { 0x8293ab4c, 98 },
    { 0, 0 },
    { 0xcff3ab4e, 52 },
    { 0x2dd3ab4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c3ab52, 98 },
    { 0x74d3ab53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8943ab58, 23 },
    { 0x8403ab58, 95 },
    { 0x3493ab58, 33 },
    { 0xa513ab5b, 99 },
    { 0x10d3ab5c, 23 },
    { 0x4b03ab5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc163ab65, 87 },
    { 0, 0 },
    { 0xcf23ab67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd923ab71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d3ab75, 23 },
    { 0xec23ab75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c3ab79, 34 },
    { 0x9fc3ab7a, 93 },
    { 0xce53ab7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c3ab7f, 46 },
    { 0x7ef3ab7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093ab85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1473ab88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f93ab8b, 61 },
    { 0, 0 },
    { 0xd393ab8d, 43 },
    { 0xd563ab8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93ab91, 90 },
    { 0x8dc3ab91, 23 },
    { 0xded3ab93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173ab98, 61 },
    { 0, 0 },
    { 0x8153ab9a, 33 },
    { 0xfa13ab9b, 88 },
    { 0xc8d3ab9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9913ab9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa993aba2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53aba9, 93 },
    { 0xb03abaa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633abad, 23 },
    { 0xeea3abad, 99 },
    { 0x66d3abae, 34 },
    { 0xcf93abb0, 61 },
    { 0xdf63abb0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3abb6, 76 },
    { 0x68f3abb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe243abbb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e3abbf, 90 },
    { 0x9483abc0, 90 },
    { 0x35c3abc0, 81 },
    { 0xf3a3abc1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b3abc5, 95 },
    { 0, 0 },
    { 0x1293abc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a63abcc, 95 },
    { 0xc0f3abcc, 61 },
    { 0, 0 },
    { 0xdde3abcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a3abd3, 33 },
    { 0x8f03abd3, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff3abd7, 95 },
    { 0x3113abd7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a3abdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6513abe0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c53abe8, 52 },
    { 0x9603abe9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7443abee, 88 },
    { 0x6c93abee, 85 },
    { 0, 0 },
    { 0xd313abf1, 61 },
    { 0x7803abf1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c53abf6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6293abfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33ac00, 23 },
    { 0x9503ac01, 23 },
    { 0x95b3ac00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea43ac06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe543ac0c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3943ac13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3ac16, 90 },
    { 0, 0 },
    { 0x61f3ac18, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd53ac1d, 61 },
    { 0x78f3ac1e, 90 },
    { 0x1903ac1f, 99 },
    { 0x1cd3ac1e, 90 },
    { 0, 0 },
    { 0x5173ac22, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f3ac26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d3ac29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263ac2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac3ac33, 61 },
    { 0x2283ac34, 33 },
    { 0, 0 },
    { 0x81a3ac36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3ac39, 34 },
    { 0xe2c3ac3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e3ac3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e3ac41, 33 },
    { 0x4843ac41, 34 },
    { 0xc2f3ac41, 98 },
    { 0, 0 },
    { 0x3303ac45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf13ac4b, 78 },
    { 0, 0 },
    { 0xc9f3ac4d, 99 },
    { 0x7563ac4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6933ac51, 76 },
    { 0xf0c3ac52, 33 },
    { 0x38f3ac53, 95 },
    { 0xd0c3ac52, 55 },
    { 0x413ac55, 53 },
    { 0x43c3ac52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab3ac59, 95 },
    { 0xb8a3ac5a, 96 },
    { 0x1f23ac5b, 68 },
    { 0xcc03ac5b, 76 },
    { 0x7853ac5d, 46 },
    { 0x2aa3ac5d, 80 },
    { 0x7873ac5d, 91 },
    { 0x4a03ac5e, 98 },
    { 0x613ac5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a83ac64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f73ac67, 33 },
    { 0x4373ac67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d43ac6d, 95 },
    { 0xe493ac6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca3ac71, 23 },
    { 0x9283ac71, 61 },
    { 0x6d93ac71, 77 },
    { 0x9163ac74, 33 },
    { 0x7cc3ac75, 17 },
    { 0xf4a3ac75, 90 },
    { 0x2853ac72, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf3ac84, 98 },
    { 0, 0 },
    { 0x3413ac86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c3ac8c, 85 },
    { 0xb323ac8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d23ac95, 84 },
    { 0, 0 },
    { 0x3f63ac97, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f3ac9a, 78 },
    { 0, 0 },
    { 0xd503ac9c, 81 },
    { 0x703ac9c, 95 },
    { 0x5c53ac9e, 34 },
    { 0x32c3ac9f, 95 },
    { 0xe273aca0, 36 },
    { 0xf593aca0, 34 },
    { 0xbbd3aca2, 67 },
    { 0xabc3aca1, 55 },
    { 0x1873aca4, 33 },
    { 0xbc03aca4, 98 },
    { 0x9d83aca5, 55 },
    { 0xc093aca7, 43 },
    { 0x76f3aca8, 99 },
    { 0x2893aca9, 61 },
    { 0xa313aca9, 61 },
    { 0xfeb3acab, 99 },
    { 0x65a3acab, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd573acb3, 90 },
    { 0, 0 },
    { 0x6023acb5, 23 },
    { 0xa2e3acb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a3acbc, 61 },
    { 0xd113acbd, 61 },
    { 0x4753acbe, 67 },
    { 0x8fd3acbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa3acc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc3acc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3983acc8, 85 },
    { 0xb033acc8, 33 },
    { 0x9f33acc8, 61 },
    { 0xe1b3acc8, 90 },
    { 0, 0 },
    { 0x5503accd, 61 },
    { 0x9ec3accd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f3acd1, 55 },
    { 0xcf13acd1, 23 },
    { 0, 0 },
    { 0xb9e3acd4, 93 },
    { 0x1ab3acd4, 98 },
    { 0xa4c3acd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3acda, 98 },
    { 0x4c03acdb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c3ace3, 33 },
    { 0, 0 },
    { 0xcb93ace5, 33 },
    { 0, 0 },
    { 0xece3ace7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc613aced, 90 },
    { 0x9273acee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c93acf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3acfa, 38 },
    { 0x5353acfa, 34 },
    { 0x7a13acfb, 23 },
    { 0x4263acfd, 78 },
    { 0xbc63acfe, 67 },
    { 0x4a3acff, 55 },
    { 0x1f43acfa, 61 },
    { 0, 0 },
    { 0xb743ad02, 46 },
    { 0xe283ad03, 67 },
    { 0, 0 },
    { 0x1843ad05, 98 },
    { 0, 0 },
    { 0x4cb3ad07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee3ad0b, 98 },
    { 0, 0 },
    { 0x17b3ad0d, 34 },
    { 0xede3ad0d, 46 },
    { 0xa433ad0f, 33 },
    { 0x2e03ad0d, 76 },
    { 0x5243ad10, 93 },
    { 0x91b3ad12, 33 },
    { 0x5563ad0e, 87 },
    { 0, 0 },
    { 0xae93ad15, 61 },
    { 0x3e13ad16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d3ad19, 76 },
    { 0x50a3ad19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf013ad1d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9963ad20, 43 },
    { 0x7bc3ad21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b93ad25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2263ad2c, 85 },
    { 0x8603ad2c, 90 },
    { 0x7553ad2e, 23 },
    { 0x4773ad2c, 100 },
    { 0, 0 },
    { 0x3d63ad31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e3ad36, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef3ad3a, 43 },
    { 0x9573ad3b, 93 },
    { 0xebb3ad3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493ad40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3ad46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed3ad4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3ad50, 76 },
    { 0x8fa3ad51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2963ad55, 93 },
    { 0x6ce3ad55, 76 },
    { 0xea23ad57, 85 },
    { 0x4003ad57, 76 },
    { 0x30e3ad59, 98 },
    { 0x19e3ad59, 90 },
    { 0, 0 },
    { 0xff13ad5c, 76 },
    { 0xd693ad5d, 43 },
    { 0xfeb3ad5d, 43 },
    { 0xb133ad5f, 90 },
    { 0x31d3ad5d, 76 },
    { 0, 0 },
    { 0xaf63ad62, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa143ad66, 99 },
    { 0x563ad67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad33ad6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd403ad6d, 98 },
    { 0x97c3ad6e, 95 },
    { 0xee53ad6f, 90 },
    { 0xddb3ad6f, 23 },
    { 0xb083ad71, 95 },
    { 0x18d3ad6f, 23 },
    { 0x4f23ad73, 33 },
    { 0xd483ad74, 75 },
    { 0x76e3ad6e, 61 },
    { 0x8553ad76, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a3ad7b, 90 },
    { 0, 0 },
    { 0x82a3ad7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7303ad81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9373ad85, 95 },
    { 0xc7f3ad86, 23 },
    { 0x7d03ad87, 52 },
    { 0xd7f3ad86, 61 },
    { 0x9813ad86, 84 },
    { 0xbcc3ad8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b93ad91, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c3ad94, 33 },
    { 0xe6a3ad95, 91 },
    { 0xb5b3ad95, 84 },
    { 0xcd83ad95, 90 },
    { 0xb83ad98, 99 },
    { 0x4633ad98, 94 },
    { 0x24b3ad98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a13ada4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa3adac, 34 },
    { 0x3ca3adad, 33 },
    { 0x103adad, 93 },
    { 0xcdb3adaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b3adb3, 98 },
    { 0xe703adb4, 76 },
    { 0, 0 },
    { 0x51d3adb6, 99 },
    { 0x1103adb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743adbd, 88 },
    { 0x42b3adbd, 91 },
    { 0, 0 },
    { 0x3c83adc0, 83 },
    { 0xef93adc1, 52 },
    { 0x74e3adc2, 33 },
    { 0xc8b3adc2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe273adc7, 80 },
    { 0x2093adc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf93adcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb3add1, 95 },
    { 0x63c3add2, 61 },
    { 0xced3add3, 61 },
    { 0, 0 },
    { 0x1f83add5, 61 },
    { 0, 0 },
    { 0x9673add7, 33 },
    { 0xb403add8, 33 },
    { 0, 0 },
    { 0xac93adda, 98 },
    { 0xb393addb, 61 },
    { 0xd953addb, 33 },
    { 0xc023addb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe113ade1, 46 },
    { 0xb3e3ade2, 95 },
    { 0, 0 },
    { 0x3a33ade4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a23ade7, 23 },
    { 0x86b3ade8, 61 },
    { 0, 0 },
    { 0x6963adea, 90 },
    { 0xb5b3adeb, 98 },
    { 0, 0 },
    { 0xc9b3aded, 23 },
    { 0, 0 },
    { 0xc383adef, 23 },
    { 0, 0 },
    { 0x3933adf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa863adfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f3adff, 34 },
    { 0x32f3ae00, 61 },
    { 0x9fc3ae01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb573ae05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d63ae08, 84 },
    { 0x13e3ae09, 46 },
    { 0x2823ae08, 34 },
    { 0x8b83ae09, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff33ae15, 34 },
    { 0, 0 },
    { 0x44a3ae17, 61 },
    { 0, 0 },
    { 0x12a3ae19, 61 },
    { 0xafc3ae1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1663ae1d, 98 },
    { 0xed23ae1e, 64 },
    { 0x4003ae1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b83ae22, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa453ae26, 95 },
    { 0xc373ae27, 12 },
    { 0xf063ae27, 76 },
    { 0xa93ae26, 87 },
    { 0xba13ae2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5123ae2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2583ae34, 55 },
    { 0xbad3ae34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc603ae38, 52 },
    { 0xaac3ae38, 85 },
    { 0, 0 },
    { 0xae23ae3b, 23 },
    { 0xa6e3ae3c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd443ae42, 46 },
    { 0x3093ae43, 95 },
    { 0, 0 },
    { 0xe833ae45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b3ae4b, 33 },
    { 0x4423ae4c, 34 },
    { 0xf273ae4d, 93 },
    { 0x4333ae4c, 61 },
    { 0xd383ae4e, 43 },
    { 0x8f3ae50, 68 },
    { 0xa633ae51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc3ae57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873ae5b, 61 },
    { 0, 0 },
    { 0x4a3ae5d, 52 },
    { 0xe8d3ae5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a3ae62, 76 },
    { 0x9813ae62, 53 },
    { 0x6343ae62, 33 },
    { 0x5393ae62, 90 },
    { 0xef73ae65, 66 },
    { 0x2f03ae67, 100 },
    { 0x2633ae68, 87 },
    { 0x1af3ae69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f3ae6f, 34 },
    { 0, 0 },
    { 0x3803ae71, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe43ae74, 68 },
    { 0x29d3ae74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c3ae78, 33 },
    { 0xfb83ae78, 61 },
    { 0, 0 },
    { 0x4fb3ae7b, 76 },
    { 0x8163ae7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff73ae80, 90 },
    { 0x34b3ae81, 95 },
    { 0x8fb3ae82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb033ae89, 33 },
    { 0x5623ae8a, 23 },
    { 0x973ae8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103ae8f, 61 },
    { 0xcc3ae90, 95 },
    { 0x7b33ae91, 33 },
    { 0x85b3ae92, 82 },
    { 0x40d3ae93, 33 },
    { 0, 0 },
    { 0x2013ae95, 36 },
    { 0, 0 },
    { 0x9723ae97, 95 },
    { 0, 0 },
    { 0x8b43ae99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e33ae9f, 33 },
    { 0, 0 },
    { 0xc3aea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1823aea6, 61 },
    { 0x2d93aea7, 78 },
    { 0xce3aea8, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab3aeaf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d03aeb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf283aebd, 33 },
    { 0, 0 },
    { 0x57c3aebf, 90 },
    { 0, 0 },
    { 0x263aec1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c13aec5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f3aecb, 23 },
    { 0xd5a3aecb, 98 },
    { 0x4803aecd, 75 },
    { 0xaf63aece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2663aed2, 55 },
    { 0, 0 },
    { 0x8f83aed4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba83aed8, 61 },
    { 0x2093aed9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f3aedf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b33aee2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e3aee6, 65 },
    { 0x853aee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6573aeeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e3aef3, 17 },
    { 0xe223aef3, 95 },
    { 0x7e03aef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c3aefc, 33 },
    { 0x8dd3aefd, 90 },
    { 0xa4e3aefd, 66 },
    { 0xeb83aeff, 85 },
    { 0xc893af00, 69 },
    { 0xce93af01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd03af06, 43 },
    { 0xdbb3af07, 43 },
    { 0x4383af07, 97 },
    { 0xd3b3af09, 33 },
    { 0x53f3af0a, 33 },
    { 0, 0 },
    { 0x52a3af0c, 95 },
    { 0x7cf3af0d, 61 },
    { 0, 0 },
    { 0x8153af0f, 43 },
    { 0xd383af10, 98 },
    { 0x2103af11, 100 },
    { 0xe73af12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f83af1d, 91 },
    { 0x52d3af1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9913af21, 61 },
    { 0x3dd3af21, 93 },
    { 0x4c23af23, 36 },
    { 0, 0 },
    { 0x3e93af25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e3af28, 34 },
    { 0xf563af29, 90 },
    { 0x3433af2a, 75 },
    { 0, 0 },
    { 0x3b53af2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7293af31, 100 },
    { 0, 0 },
    { 0x4063af33, 75 },
    { 0x9353af34, 61 },
    { 0x1263af33, 61 },
    { 0xe653af36, 87 },
    { 0x7be3af37, 90 },
    { 0xbb63af38, 34 },
    { 0x58e3af36, 61 },
    { 0x8723af3a, 77 },
    { 0, 0 },
    { 0x5703af3c, 98 },
    { 0x8f53af3d, 95 },
    { 0xbfa3af3e, 90 },
    { 0, 0 },
    { 0x9523af40, 98 },
    { 0x4d93af41, 67 },
    { 0xb9b3af41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f73af45, 23 },
    { 0x9933af46, 98 },
    { 0, 0 },
    { 0x6b03af48, 76 },
    { 0x54f3af49, 23 },
    { 0x8c63af48, 80 },
    { 0x8893af4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6593af4e, 90 },
    { 0xc8c3af4e, 95 },
    { 0x6c73af4f, 33 },
    { 0x6953af51, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1603af54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d3af59, 95 },
    { 0xd873af5a, 96 },
    { 0, 0 },
    { 0x82b3af5c, 23 },
    { 0x7fa3af5d, 84 },
    { 0xa803af5e, 67 },
    { 0x34e3af5f, 23 },
    { 0xefa3af60, 76 },
    { 0xc7f3af61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f3af66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c3af69, 95 },
    { 0x1953af69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd73af6f, 43 },
    { 0xd953af70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc573af73, 23 },
    { 0, 0 },
    { 0x6823af75, 55 },
    { 0, 0 },
    { 0xf3c3af77, 55 },
    { 0x4143af78, 95 },
    { 0x1143af77, 98 },
    { 0, 0 },
    { 0x783af7b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3af80, 90 },
    { 0xccf3af81, 66 },
    { 0x45f3af82, 52 },
    { 0x1593af80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f3af88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf083af98, 32 },
    { 0x4ec3af98, 23 },
    { 0x6ff3af9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3af9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7073afa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf503afac, 76 },
    { 0xb9b3afad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9193afb0, 77 },
    { 0, 0 },
    { 0xa0f3afb2, 98 },
    { 0, 0 },
    { 0x20e3afb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3afb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3603afbb, 95 },
    { 0, 0 },
    { 0x8663afbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3afc3, 55 },
    { 0, 0 },
    { 0xde83afc5, 33 },
    { 0, 0 },
    { 0xd763afc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073afcc, 33 },
    { 0, 0 },
    { 0xb4d3afce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd113afd1, 90 },
    { 0x6bc3afd1, 61 },
    { 0, 0 },
    { 0x2263afd4, 91 },
    { 0xa433afd5, 90 },
    { 0x4023afd6, 33 },
    { 0x4483afd7, 61 },
    { 0xa5d3afd8, 61 },
    { 0x46d3afd4, 23 },
    { 0x1f23afda, 68 },
    { 0, 0 },
    { 0xdfa3afdc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5133afdf, 23 },
    { 0, 0 },
    { 0x9123afe1, 79 },
    { 0x19a3afe2, 43 },
    { 0xc2b3afe3, 23 },
    { 0x5823afe4, 34 },
    { 0xc8a3afe1, 98 },
    { 0xab33afe6, 90 },
    { 0x69a3afe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2803afea, 33 },
    { 0, 0 },
    { 0x5ab3afec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b83afef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d3aff3, 95 },
    { 0xf273aff3, 61 },
    { 0x6213aff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e3aff8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f3affb, 62 },
    { 0, 0 },
    { 0xc2d3affd, 23 },
    { 0, 0 },
    { 0xac53afff, 43 },
    { 0, 0 },
    { 0x3e73b001, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa893b005, 34 },
    { 0x14a3b006, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6063b009, 34 },
    { 0xf5a3b00a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc123b00d, 78 },
    { 0xcca3b00d, 34 },
    { 0xde13b00d, 61 },
    { 0xd293b010, 23 },
    { 0x9463b00f, 34 },
    { 0xf0a3b011, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d3b017, 43 },
    { 0x1d73b018, 90 },
    { 0x15a3b019, 23 },
    { 0x19e3b017, 90 },
    { 0, 0 },
    { 0xa423b01c, 95 },
    { 0x2a13b01d, 61 },
    { 0x533b01e, 61 },
    { 0xb783b01f, 61 },
    { 0x23b020, 93 },
    { 0x6243b021, 90 },
    { 0x27b3b01e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f3b025, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363b02a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e53b02f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb23b034, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9703b038, 90 },
    { 0, 0 },
    { 0x7173b03a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef03b040, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e3b044, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b04a, 33 },
    { 0, 0 },
    { 0x8b73b04c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9773b052, 43 },
    { 0, 0 },
    { 0x63a3b054, 23 },
    { 0, 0 },
    { 0x4be3b056, 68 },
    { 0x2173b057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1223b05e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc353b062, 33 },
    { 0x43e3b063, 90 },
    { 0, 0 },
    { 0x703b065, 91 },
    { 0, 0 },
    { 0xae3b067, 61 },
    { 0x4513b068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd713b070, 38 },
    { 0x1d33b071, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4613b075, 90 },
    { 0x8d03b076, 95 },
    { 0xfb53b075, 98 },
    { 0xef3b078, 84 },
    { 0xbbd3b078, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc23b07c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8853b082, 68 },
    { 0x9e53b083, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc253b08e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e43b093, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f43b098, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c3b09d, 55 },
    { 0, 0 },
    { 0xcf53b09f, 99 },
    { 0x3ba3b0a0, 33 },
    { 0x11f3b0a0, 46 },
    { 0x1843b0a2, 43 },
    { 0x2133b0a3, 76 },
    { 0, 0 },
    { 0x1a3b0a5, 23 },
    { 0xea83b0a6, 90 },
    { 0xb853b0a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903b0ab, 85 },
    { 0x8893b0ab, 98 },
    { 0x443b0ad, 33 },
    { 0x9cc3b0ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3b0b1, 95 },
    { 0x20e3b0b2, 90 },
    { 0x69f3b0b1, 78 },
    { 0x37e3b0b1, 91 },
    { 0xb493b0b5, 90 },
    { 0x6073b0b4, 55 },
    { 0xc053b0b6, 34 },
    { 0xc123b0b5, 67 },
    { 0xa6d3b0b5, 33 },
    { 0x5003b0ba, 95 },
    { 0xfd83b0bb, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd33b0c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3b0c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa463b0cb, 52 },
    { 0xfa13b0cc, 76 },
    { 0, 0 },
    { 0x22b3b0ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413b0d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3b0d4, 46 },
    { 0x3a93b0d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7253b0da, 61 },
    { 0x6863b0db, 96 },
    { 0, 0 },
    { 0xa433b0dd, 33 },
    { 0, 0 },
    { 0xc383b0df, 99 },
    { 0x4dd3b0e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1073b0e4, 36 },
    { 0xd783b0e5, 61 },
    { 0xcaa3b0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d3b0ed, 85 },
    { 0x82a3b0ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1793b0f1, 43 },
    { 0x8f13b0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823b0f8, 98 },
    { 0, 0 },
    { 0x6e33b0fa, 67 },
    { 0x4c43b0fb, 90 },
    { 0, 0 },
    { 0xc343b0fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe653b100, 95 },
    { 0x9ad3b101, 85 },
    { 0xa2f3b102, 98 },
    { 0x1c43b103, 43 },
    { 0x5d73b104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf53b109, 33 },
    { 0xad63b10a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f73b10d, 77 },
    { 0x6303b10e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5773b118, 90 },
    { 0xbaa3b118, 100 },
    { 0x6143b11a, 98 },
    { 0x53e3b11a, 23 },
    { 0x5db3b11a, 85 },
    { 0x5463b11d, 93 },
    { 0x4533b11e, 90 },
    { 0x2383b11c, 55 },
    { 0, 0 },
    { 0x47e3b121, 83 },
    { 0x3623b122, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede3b126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa433b129, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x6363b12c, 46 },
    { 0, 0 },
    { 0xe433b12e, 33 },
    { 0x8f13b12f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b3b135, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3b139, 95 },
    { 0, 0 },
    { 0xe0e3b13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53b13e, 67 },
    { 0x5cb3b13e, 33 },
    { 0x3a93b140, 34 },
    { 0x3dc3b141, 95 },
    { 0x4f93b140, 99 },
    { 0, 0 },
    { 0x4a03b144, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2103b14b, 84 },
    { 0, 0 },
    { 0xe943b14d, 98 },
    { 0x49b3b14d, 90 },
    { 0xb983b14e, 55 },
    { 0x80d3b150, 88 },
    { 0x7f83b150, 61 },
    { 0x4b23b152, 34 },
    { 0x6923b152, 33 },
    { 0xaa33b152, 33 },
    { 0x7293b152, 95 },
    { 0, 0 },
    { 0x7113b157, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1983b15a, 67 },
    { 0xb8d3b15b, 98 },
    { 0x8283b15b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b165, 91 },
    { 0, 0 },
    { 0xbf13b167, 90 },
    { 0xb293b167, 88 },
    { 0xcc63b169, 55 },
    { 0x4f43b16a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa73b16e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7103b174, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa63b178, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c33b17d, 61 },
    { 0xbe33b17e, 90 },
    { 0x7103b17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8243b185, 23 },
    { 0xb823b186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd63b18a, 90 },
    { 0, 0 },
    { 0x67c3b18c, 98 },
    { 0x5f43b18d, 61 },
    { 0x7f03b18e, 52 },
    { 0xa5b3b18f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4323b193, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b73b1a0, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd73b1a4, 85 },
    { 0, 0 },
    { 0x2e23b1a6, 90 },
    { 0x1233b1a7, 33 },
    { 0xb1b3b1a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e3b1ab, 66 },
    { 0, 0 },
    { 0x84d3b1ad, 93 },
    { 0, 0 },
    { 0xc803b1af, 98 },
    { 0, 0 },
    { 0x69c3b1b1, 95 },
    { 0xa833b1b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee3b1b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3b1b9, 87 },
    { 0x2c43b1ba, 23 },
    { 0xb13b1b9, 93 },
    { 0xb2e3b1ba, 34 },
    { 0x80b3b1bb, 15 },
    { 0xc993b1be, 90 },
    { 0xeb13b1bf, 61 },
    { 0xc903b1bf, 66 },
    { 0xa3b1c1, 85 },
    { 0x933b1c2, 95 },
    { 0, 0 },
    { 0xe773b1c4, 95 },
    { 0x9823b1c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc793b1cd, 52 },
    { 0x5973b1cd, 46 },
    { 0x6ca3b1cf, 33 },
    { 0, 0 },
    { 0xbff3b1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3033b1d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e3b1da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b3b1df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3b1e2, 88 },
    { 0x1c53b1e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa573b1e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9533b1ea, 49 },
    { 0, 0 },
    { 0x9fc3b1ec, 93 },
    { 0, 0 },
    { 0x7a33b1ee, 95 },
    { 0x5973b1ef, 90 },
    { 0x9c03b1f0, 77 },
    { 0, 0 },
    { 0xce93b1f2, 90 },
    { 0x1d73b1f2, 23 },
    { 0x9a13b1f2, 33 },
    { 0, 0 },
    { 0xa933b1f6, 95 },
    { 0x4f83b1f6, 90 },
    { 0xf1c3b1f8, 95 },
    { 0, 0 },
    { 0xef93b1fa, 99 },
    { 0x8683b1fa, 23 },
    { 0x9d63b1fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb33b1ff, 33 },
    { 0, 0 },
    { 0xc6d3b201, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc3b204, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf723b207, 88 },
    { 0xef73b208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce3b20b, 90 },
    { 0x9993b20b, 85 },
    { 0x2be3b20b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b23b213, 61 },
    { 0x8113b214, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b3b218, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa473b21d, 61 },
    { 0x1f23b21e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643b224, 33 },
    { 0, 0 },
    { 0x8673b226, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43b22f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd03b23b, 33 },
    { 0xeaa3b23c, 23 },
    { 0xa7a3b23d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb3b241, 76 },
    { 0xe4a3b241, 90 },
    { 0, 0 },
    { 0x33b244, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f3b249, 55 },
    { 0, 0 },
    { 0xc373b24b, 43 },
    { 0x7863b24c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c3b24f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3b255, 61 },
    { 0xa383b256, 90 },
    { 0x2be3b257, 33 },
    { 0x6123b255, 55 },
    { 0xdb03b256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3b25d, 23 },
    { 0xd703b25e, 23 },
    { 0xbd23b25f, 46 },
    { 0xa833b260, 95 },
    { 0x8043b261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d3b264, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e3b267, 98 },
    { 0x1b23b267, 93 },
    { 0x96d3b269, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced3b270, 41 },
    { 0xa883b270, 95 },
    { 0x2cc3b271, 23 },
    { 0x2bf3b273, 90 },
    { 0x6df3b270, 96 },
    { 0, 0 },
    { 0xb473b276, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff3b27b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa843b283, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9293b287, 33 },
    { 0, 0 },
    { 0x5403b289, 43 },
    { 0, 0 },
    { 0x1973b28b, 33 },
    { 0, 0 },
    { 0xfc43b28d, 90 },
    { 0, 0 },
    { 0x1353b28f, 90 },
    { 0, 0 },
    { 0x1573b291, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf103b295, 23 },
    { 0x4283b296, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad83b29a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7473b29f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb53b2a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca3b2a7, 33 },
    { 0, 0 },
    { 0x3173b2a9, 82 },
    { 0, 0 },
    { 0xd1d3b2ab, 90 },
    { 0, 0 },
    { 0x6093b2ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa553b2b1, 98 },
    { 0x7753b2b2, 33 },
    { 0x9f03b2b3, 90 },
    { 0xb223b2b3, 66 },
    { 0, 0 },
    { 0x35b3b2b6, 99 },
    { 0x4b63b2b7, 87 },
    { 0x9ef3b2b8, 43 },
    { 0x3223b2b8, 66 },
    { 0, 0 },
    { 0x9603b2bb, 90 },
    { 0, 0 },
    { 0xc343b2bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103b2c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3b2c3, 88 },
    { 0x35a3b2c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c3b2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc3b2d3, 90 },
    { 0xf193b2d4, 66 },
    { 0x3043b2d5, 56 },
    { 0, 0 },
    { 0x8de3b2d7, 33 },
    { 0x13d3b2d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe273b2e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e93b2ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb683b2ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd3b2f2, 81 },
    { 0x12e3b2f2, 56 },
    { 0x1123b2f4, 36 },
    { 0, 0 },
    { 0xe3b3b2f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533b300, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe53b308, 33 },
    { 0xea33b309, 91 },
    { 0x6253b309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a3b30e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xef63b311, 96 },
    { 0, 0 },
    { 0xbf33b313, 23 },
    { 0x7d23b314, 33 },
    { 0, 0 },
    { 0xf913b316, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5613b31b, 34 },
    { 0, 0 },
    { 0xdf3b31d, 46 },
    { 0x97e3b31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda3b322, 99 },
    { 0, 0 },
    { 0xfe13b324, 93 },
    { 0, 0 },
    { 0xc733b326, 33 },
    { 0x8e23b327, 55 },
    { 0x49f3b327, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf3b32c, 33 },
    { 0xac93b32d, 77 },
    { 0x59f3b32e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9853b336, 90 },
    { 0x6573b337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed3b33a, 78 },
    { 0, 0 },
    { 0x7df3b33c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053b33f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7843b343, 85 },
    { 0x7043b344, 98 },
    { 0x793b345, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e33b349, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b3b34c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f3b34f, 61 },
    { 0x7d53b350, 90 },
    { 0xb563b34f, 34 },
    { 0, 0 },
    { 0xb4e3b353, 98 },
    { 0xf163b353, 98 },
    { 0xfb13b355, 90 },
    { 0, 0 },
    { 0x4783b357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b361, 61 },
    { 0x5003b361, 61 },
    { 0, 0 },
    { 0xc683b364, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403b368, 55 },
    { 0, 0 },
    { 0x2bc3b36a, 98 },
    { 0xc683b36b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b36e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3b374, 34 },
    { 0xfda3b374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7873b37b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3b381, 81 },
    { 0x4353b381, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43b385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6553b38a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a3b391, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb3b394, 62 },
    { 0xbbb3b395, 98 },
    { 0, 0 },
    { 0x613b397, 37 },
    { 0xeeb3b398, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef3b39b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3b39e, 23 },
    { 0x2323b39f, 76 },
    { 0x33f3b39f, 37 },
    { 0x3253b3a1, 66 },
    { 0xb63b39f, 34 },
    { 0xa163b3a3, 33 },
    { 0xcb3b3a4, 100 },
    { 0xc0b3b3a3, 61 },
    { 0x8233b3a6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3b3ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa523b3b5, 61 },
    { 0x43b3b6, 56 },
    { 0x1673b3b7, 55 },
    { 0x7c23b3b7, 84 },
    { 0x5103b3b9, 65 },
    { 0x87a3b3ba, 66 },
    { 0, 0 },
    { 0xf063b3bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd93b3bf, 90 },
    { 0, 0 },
    { 0x24d3b3c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273b3c5, 95 },
    { 0, 0 },
    { 0x5643b3c7, 95 },
    { 0, 0 },
    { 0xfa53b3c9, 33 },
    { 0, 0 },
    { 0x5da3b3cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d03b3cf, 93 },
    { 0x90a3b3cf, 99 },
    { 0, 0 },
    { 0xa03b3d2, 52 },
    { 0x57f3b3d3, 78 },
    { 0xc7d3b3d4, 33 },
    { 0x2e13b3d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad63b3db, 90 },
    { 0x5fd3b3dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec53b3df, 23 },
    { 0xe3f3b3e0, 68 },
    { 0xa163b3e0, 46 },
    { 0, 0 },
    { 0x4093b3e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f83b3e6, 61 },
    { 0, 0 },
    { 0xb673b3e8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f3b3ef, 90 },
    { 0xc03b3f0, 95 },
    { 0x9f3b3f0, 75 },
    { 0xc4a3b3f2, 55 },
    { 0x8363b3f3, 77 },
    { 0xaf33b3f4, 93 },
    { 0x8eb3b3f4, 90 },
    { 0xb593b3f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9313b3f9, 33 },
    { 0x8c33b3f9, 33 },
    { 0, 0 },
    { 0x1033b3fc, 57 },
    { 0x9303b3fd, 33 },
    { 0x3b93b3fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3b402, 76 },
    { 0xa8d3b403, 76 },
    { 0, 0 },
    { 0x9563b405, 33 },
    { 0, 0 },
    { 0x4b63b407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413b40a, 98 },
    { 0x58c3b40b, 43 },
    { 0, 0 },
    { 0xa8d3b40d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf3b41c, 98 },
    { 0x93a3b41c, 90 },
    { 0x1da3b41e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad3b422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f3b426, 33 },
    { 0x3e23b427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e3b42e, 27 },
    { 0, 0 },
    { 0xc843b430, 23 },
    { 0x2b23b431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf723b437, 67 },
    { 0x72b3b437, 90 },
    { 0, 0 },
    { 0x7f83b43a, 98 },
    { 0xb323b43b, 78 },
    { 0x19f3b43b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xa353b43f, 90 },
    { 0, 0 },
    { 0xef23b441, 76 },
    { 0xb393b442, 90 },
    { 0xd763b443, 86 },
    { 0, 0 },
    { 0xe773b445, 87 },
    { 0x5143b446, 91 },
    { 0x9dc3b446, 61 },
    { 0xd8f3b446, 85 },
    { 0xcc23b446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13b44c, 76 },
    { 0x4793b44c, 33 },
    { 0x3c73b44e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7443b452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5273b456, 90 },
    { 0x21a3b457, 90 },
    { 0xf0e3b457, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d3b45c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd3b467, 83 },
    { 0xc803b468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf3b46b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e3b471, 91 },
    { 0x5733b472, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e3b475, 23 },
    { 0xded3b476, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d63b47a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b3b47e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc353b481, 90 },
    { 0, 0 },
    { 0xd333b483, 76 },
    { 0, 0 },
    { 0xf603b485, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc753b488, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf083b48d, 33 },
    { 0x8ab3b48d, 90 },
    { 0x2283b48e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb3b493, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb3b498, 52 },
    { 0xc0f3b499, 95 },
    { 0xc2c3b49a, 90 },
    { 0, 0 },
    { 0x6343b49c, 91 },
    { 0xd123b49c, 33 },
    { 0x5833b49e, 32 },
    { 0x35f3b49f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763b4a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8543b4b1, 61 },
    { 0x6953b4b2, 55 },
    { 0, 0 },
    { 0x3353b4b4, 98 },
    { 0xc923b4b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e3b4b8, 90 },
    { 0x2e13b4b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab33b4bc, 76 },
    { 0, 0 },
    { 0x493b4be, 33 },
    { 0xad23b4bf, 67 },
    { 0x53c3b4c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8383b4c3, 33 },
    { 0x3dd3b4c4, 95 },
    { 0, 0 },
    { 0x7d93b4c6, 23 },
    { 0xa233b4c7, 98 },
    { 0x1c93b4c6, 98 },
    { 0x3b83b4c6, 23 },
    { 0x7b03b4c9, 61 },
    { 0xcb23b4cb, 76 },
    { 0x1513b4cc, 61 },
    { 0xe053b4cc, 87 },
    { 0x57d3b4ce, 33 },
    { 0x2103b4c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1343b4d2, 73 },
    { 0xccc3b4d2, 33 },
    { 0xef53b4d4, 23 },
    { 0, 0 },
    { 0x46f3b4d6, 76 },
    { 0, 0 },
    { 0xac03b4d8, 76 },
    { 0, 0 },
    { 0xdc03b4da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3b4dd, 61 },
    { 0x82a3b4de, 61 },
    { 0x5e53b4df, 76 },
    { 0x9d33b4e0, 95 },
    { 0x33b3b4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7753b4e4, 61 },
    { 0x3783b4e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b3b4ee, 36 },
    { 0x70f3b4ee, 34 },
    { 0xe803b4f0, 40 },
    { 0, 0 },
    { 0xf203b4f2, 90 },
    { 0, 0 },
    { 0x3053b4f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1273b4f7, 99 },
    { 0x7a03b4f8, 23 },
    { 0x4a3b4f7, 55 },
    { 0, 0 },
    { 0x83d3b4fb, 23 },
    { 0x3f53b4fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e3b4ff, 55 },
    { 0x2f83b4ff, 52 },
    { 0x2333b501, 95 },
    { 0x1203b501, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f83b509, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653b50d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4123b513, 36 },
    { 0xd5a3b513, 87 },
    { 0, 0 },
    { 0x973b516, 66 },
    { 0xe7c3b516, 52 },
    { 0, 0 },
    { 0xbc93b519, 34 },
    { 0x7a33b519, 90 },
    { 0xeb03b51b, 55 },
    { 0xa133b51a, 95 },
    { 0xd343b51d, 76 },
    { 0, 0 },
    { 0xe9f3b51f, 77 },
    { 0x92f3b51f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc33b523, 33 },
    { 0xff03b524, 61 },
    { 0x5473b525, 90 },
    { 0, 0 },
    { 0x5db3b527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f3b52a, 23 },
    { 0x493b52b, 33 },
    { 0x85e3b52c, 33 },
    { 0xb833b52c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c3b530, 85 },
    { 0, 0 },
    { 0x4d73b532, 81 },
    { 0x1fa3b533, 66 },
    { 0xe813b534, 98 },
    { 0xb9d3b535, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee3b53e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa03b541, 23 },
    { 0, 0 },
    { 0x3c13b543, 95 },
    { 0x9a03b543, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f3b547, 33 },
    { 0xf563b548, 33 },
    { 0xed13b549, 76 },
    { 0, 0 },
    { 0xc903b54b, 95 },
    { 0x1653b54b, 33 },
    { 0, 0 },
    { 0x29a3b54e, 23 },
    { 0x9353b54f, 55 },
    { 0x86a3b54e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd123b555, 77 },
    { 0xab03b555, 33 },
    { 0x303b555, 95 },
    { 0xf193b558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3b55f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b03b562, 93 },
    { 0x1b03b563, 90 },
    { 0, 0 },
    { 0x44c3b565, 52 },
    { 0xd563b565, 90 },
    { 0, 0 },
    { 0x3913b568, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa413b573, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c3b576, 33 },
    { 0, 0 },
    { 0x4913b578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3b57d, 9 },
    { 0xb403b57e, 55 },
    { 0x8623b57e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6423b58b, 43 },
    { 0, 0 },
    { 0x253b58d, 93 },
    { 0xfaa3b58d, 55 },
    { 0xf343b58e, 33 },
    { 0xe0f3b590, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc693b594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673b59c, 81 },
    { 0xdc53b59d, 76 },
    { 0x2f63b59e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe963b5a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec3b5ab, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113b5af, 76 },
    { 0xd523b5af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8473b5b5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d33b5b8, 91 },
    { 0x91f3b5b9, 34 },
    { 0xda63b5b9, 61 },
    { 0, 0 },
    { 0x79f3b5bc, 46 },
    { 0x86a3b5bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac3b5cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7753b5ce, 91 },
    { 0x7ae3b5ce, 98 },
    { 0x473b5d0, 61 },
    { 0x2d33b5d0, 90 },
    { 0x4bb3b5d2, 33 },
    { 0x21b3b5d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa503b5d9, 90 },
    { 0, 0 },
    { 0xe3f3b5db, 77 },
    { 0x4a53b5dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a93b5df, 99 },
    { 0x7be3b5e0, 90 },
    { 0x3553b5e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7553b5e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc523b5ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb143b5f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d3b5f5, 76 },
    { 0xce93b5f5, 99 },
    { 0, 0 },
    { 0x53a3b5f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3b5fd, 34 },
    { 0x8313b5fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8413b602, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f3b605, 34 },
    { 0, 0 },
    { 0x5d93b607, 95 },
    { 0x7543b608, 52 },
    { 0xa063b607, 55 },
    { 0xbb83b60a, 66 },
    { 0, 0 },
    { 0xa5e3b60c, 23 },
    { 0x85e3b60d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd033b616, 61 },
    { 0xbd23b617, 90 },
    { 0x49f3b618, 99 },
    { 0xe3d3b619, 99 },
    { 0xcb23b61a, 78 },
    { 0x2543b61b, 61 },
    { 0xa13b619, 61 },
    { 0x7b23b61c, 93 },
    { 0xb1e3b61b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a23b625, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7173b62a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af3b630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff3b635, 23 },
    { 0, 0 },
    { 0x8513b637, 95 },
    { 0x80e3b638, 85 },
    { 0, 0 },
    { 0x2e3b63a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713b63e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a83b643, 33 },
    { 0xb593b643, 23 },
    { 0xc303b645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a3b64a, 95 },
    { 0, 0 },
    { 0xa5a3b64c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe93b650, 98 },
    { 0x5313b651, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5933b654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e3b657, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c3b65b, 90 },
    { 0x7ff3b65c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503b660, 23 },
    { 0x9b93b660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb793b669, 88 },
    { 0, 0 },
    { 0x1043b66b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a53b66f, 43 },
    { 0x1303b670, 33 },
    { 0x76a3b671, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae3b67b, 98 },
    { 0xf6c3b67c, 75 },
    { 0xec03b67d, 23 },
    { 0x7393b67b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe573b681, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa953b685, 34 },
    { 0, 0 },
    { 0x9763b687, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc953b68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b43b690, 95 },
    { 0, 0 },
    { 0x1a83b692, 66 },
    { 0x6cd3b693, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6043b696, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6143b69a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e3b69d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413b6a0, 34 },
    { 0, 0 },
    { 0xa913b6a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f3b6a7, 95 },
    { 0xd9e3b6a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7113b6ac, 67 },
    { 0x9553b6ad, 34 },
    { 0xd9b3b6ae, 95 },
    { 0, 0 },
    { 0x333b6b0, 33 },
    { 0, 0 },
    { 0x7183b6b2, 17 },
    { 0, 0 },
    { 0x6963b6b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e3b6b7, 88 },
    { 0, 0 },
    { 0xda63b6b9, 33 },
    { 0x95c3b6ba, 90 },
    { 0, 0 },
    { 0x1503b6bc, 33 },
    { 0xa1c3b6bd, 43 },
    { 0x363b6bd, 98 },
    { 0xb913b6bf, 23 },
    { 0x1cb3b6bf, 61 },
    { 0x9e43b6bc, 98 },
    { 0xcfb3b6c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533b6c7, 90 },
    { 0, 0 },
    { 0xcce3b6c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec3b6cf, 95 },
    { 0, 0 },
    { 0xa9b3b6d1, 69 },
    { 0x71d3b6d2, 98 },
    { 0x97e3b6d3, 34 },
    { 0xfdf3b6d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a83b6d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d3b6e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b83b6e6, 67 },
    { 0x9d33b6e7, 66 },
    { 0xe163b6e7, 66 },
    { 0xae03b6e9, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2213b6ed, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93b6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe493b6f3, 23 },
    { 0x2373b6f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be3b6ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3b704, 61 },
    { 0, 0 },
    { 0xa9e3b706, 90 },
    { 0xb33b707, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa3b70e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb853b712, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb913b716, 91 },
    { 0x35c3b717, 76 },
    { 0x16c3b718, 61 },
    { 0x66d3b719, 33 },
    { 0, 0 },
    { 0x84d3b71b, 90 },
    { 0xcff3b71c, 100 },
    { 0x953b71d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e53b720, 99 },
    { 0, 0 },
    { 0x4f63b722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba3b727, 43 },
    { 0x59e3b728, 85 },
    { 0x4c23b729, 90 },
    { 0xa463b728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3413b733, 33 },
    { 0xaae3b734, 55 },
    { 0xedf3b735, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d3b738, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae43b73f, 68 },
    { 0x5a43b740, 43 },
    { 0x8fa3b741, 55 },
    { 0, 0 },
    { 0x2693b743, 61 },
    { 0xa173b744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a3b747, 101 },
    { 0xe0c3b748, 84 },
    { 0, 0 },
    { 0xdc23b74a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd93b74d, 65 },
    { 0, 0 },
    { 0x2863b74f, 17 },
    { 0, 0 },
    { 0xcc13b751, 77 },
    { 0, 0 },
    { 0x44d3b753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad3b756, 52 },
    { 0xcc03b756, 23 },
    { 0x9943b757, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c73b75b, 34 },
    { 0x48e3b75c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8543b75f, 66 },
    { 0xe223b760, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a83b76b, 23 },
    { 0x3bb3b76c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2643b774, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d3b77b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f43b780, 100 },
    { 0x633b781, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b784, 67 },
    { 0xda03b785, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b83b789, 98 },
    { 0xc563b78a, 95 },
    { 0x4123b789, 33 },
    { 0x6ba3b78c, 98 },
    { 0x53a3b78d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b3b790, 34 },
    { 0xf53b791, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e3b796, 98 },
    { 0, 0 },
    { 0x9093b798, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x8033b79b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d3b7a2, 55 },
    { 0xa723b7a3, 33 },
    { 0xcc63b7a3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb33b7a7, 61 },
    { 0x82c3b7a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e3b7ab, 95 },
    { 0xec63b7ac, 46 },
    { 0xea63b7ac, 98 },
    { 0xcb3b7ae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403b7b1, 43 },
    { 0xcfd3b7b2, 95 },
    { 0x22c3b7b2, 66 },
    { 0, 0 },
    { 0xbce3b7b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d53b7b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f3b7bf, 93 },
    { 0x2943b7bf, 33 },
    { 0xdbf3b7c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c3b7c6, 78 },
    { 0x5133b7c7, 85 },
    { 0x4df3b7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa813b7cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d3b7d0, 33 },
    { 0, 0 },
    { 0xb6c3b7d2, 76 },
    { 0x3493b7d2, 23 },
    { 0, 0 },
    { 0x4923b7d5, 33 },
    { 0, 0 },
    { 0xf9f3b7d7, 61 },
    { 0, 0 },
    { 0xc753b7d9, 34 },
    { 0, 0 },
    { 0xdf63b7db, 52 },
    { 0x52a3b7db, 33 },
    { 0x8af3b7dd, 85 },
    { 0xf063b7de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc3b7e2, 93 },
    { 0, 0 },
    { 0xb1c3b7e4, 99 },
    { 0xfbe3b7e5, 90 },
    { 0x6393b7e6, 33 },
    { 0xc643b7e7, 67 },
    { 0x1913b7e6, 90 },
    { 0xbb03b7e9, 93 },
    { 0x8073b7ea, 23 },
    { 0xbe93b7ea, 61 },
    { 0, 0 },
    { 0x4163b7ed, 61 },
    { 0, 0 },
    { 0x193b7ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9013b7f4, 84 },
    { 0x1fe3b7f5, 90 },
    { 0, 0 },
    { 0x6323b7f7, 43 },
    { 0x2303b7f8, 61 },
    { 0, 0 },
    { 0xb7b3b7fa, 43 },
    { 0x34b3b7fa, 33 },
    { 0x4ab3b7fc, 43 },
    { 0xe3b7fa, 93 },
    { 0x4e23b7fa, 61 },
    { 0xf5b3b7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe953b802, 55 },
    { 0, 0 },
    { 0x79a3b804, 91 },
    { 0x7833b805, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b3b80a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33b810, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a73b817, 85 },
    { 0x73a3b818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b3b81b, 93 },
    { 0x8843b81c, 92 },
    { 0xefd3b81d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7043b823, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae33b828, 76 },
    { 0x8f63b829, 99 },
    { 0x7be3b82a, 36 },
    { 0x4743b82b, 93 },
    { 0xd223b82a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c03b830, 91 },
    { 0xf393b831, 91 },
    { 0, 0 },
    { 0x8cc3b833, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1873b837, 10 },
    { 0xdff3b838, 33 },
    { 0, 0 },
    { 0x4c83b83a, 91 },
    { 0xb133b83b, 95 },
    { 0x8403b83c, 83 },
    { 0x23b3b83c, 59 },
    { 0x9eb3b83c, 34 },
    { 0xa3f3b83f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6713b846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563b849, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3b84c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c3b84f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3893b85d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a3b862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d3b866, 90 },
    { 0x1d3b867, 33 },
    { 0x2ad3b866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e3b86e, 33 },
    { 0x9ba3b86e, 93 },
    { 0x6bd3b870, 99 },
    { 0xdfa3b871, 90 },
    { 0x3483b86e, 61 },
    { 0xad53b871, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93b879, 85 },
    { 0, 0 },
    { 0x9873b87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9573b87e, 100 },
    { 0, 0 },
    { 0x8ce3b880, 95 },
    { 0x6423b880, 91 },
    { 0, 0 },
    { 0x2543b883, 99 },
    { 0x1ab3b884, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba93b88c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a3b892, 95 },
    { 0x533b892, 33 },
    { 0x4c23b894, 95 },
    { 0, 0 },
    { 0x5cb3b896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf533b89d, 99 },
    { 0, 0 },
    { 0x19f3b89f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3b8a2, 61 },
    { 0x14b3b8a2, 46 },
    { 0, 0 },
    { 0x27c3b8a5, 90 },
    { 0x8ef3b8a6, 34 },
    { 0, 0 },
    { 0x3b73b8a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f23b8ae, 34 },
    { 0xbb63b8af, 23 },
    { 0, 0 },
    { 0x9f53b8b1, 90 },
    { 0x25d3b8b1, 95 },
    { 0xebb3b8b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e3b8b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073b8bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d3b8c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf023b8cd, 85 },
    { 0x2a83b8ce, 46 },
    { 0x4e3b8cf, 90 },
    { 0x3043b8d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2723b8d5, 66 },
    { 0xe833b8d6, 43 },
    { 0x50c3b8d7, 61 },
    { 0x7ba3b8d7, 90 },
    { 0xe673b8d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1313b8dc, 76 },
    { 0, 0 },
    { 0xf873b8de, 66 },
    { 0xc793b8df, 95 },
    { 0x4203b8e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c3b8e7, 98 },
    { 0x8313b8e8, 52 },
    { 0x3683b8e9, 66 },
    { 0xbc53b8e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d3b8f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d83b8f8, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe333b8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb563b908, 43 },
    { 0, 0 },
    { 0x33c3b90a, 59 },
    { 0xe913b90b, 40 },
    { 0x3b3b90b, 93 },
    { 0, 0 },
    { 0xa873b90e, 90 },
    { 0xa1e3b90f, 78 },
    { 0xf983b910, 61 },
    { 0x4d63b910, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b23b916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b919, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf463b91c, 95 },
    { 0x36d3b91c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9153b920, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d3b926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf53b92d, 98 },
    { 0xaf83b92e, 61 },
    { 0x83e3b92f, 100 },
    { 0x9d23b92d, 98 },
    { 0x9193b931, 61 },
    { 0xa323b932, 64 },
    { 0xc073b933, 33 },
    { 0xbde3b933, 61 },
    { 0xb413b935, 61 },
    { 0x9223b934, 46 },
    { 0x5363b937, 95 },
    { 0xbb3b937, 61 },
    { 0x2213b933, 90 },
    { 0xdcd3b93a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c43b93e, 61 },
    { 0xd153b93f, 18 },
    { 0, 0 },
    { 0x22c3b941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323b944, 85 },
    { 0x9043b944, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd103b948, 87 },
    { 0xc253b949, 46 },
    { 0x4303b94a, 93 },
    { 0xaa13b94b, 76 },
    { 0xd6a3b94c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf563b94f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3b952, 33 },
    { 0, 0 },
    { 0x4a63b954, 90 },
    { 0, 0 },
    { 0xf963b956, 93 },
    { 0xf833b956, 90 },
    { 0, 0 },
    { 0x6233b959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc3b961, 65 },
    { 0x7d63b961, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b3b96c, 67 },
    { 0xb733b96d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db3b975, 23 },
    { 0, 0 },
    { 0xdb53b977, 90 },
    { 0xad03b978, 17 },
    { 0x24f3b979, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5383b97c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d3b97f, 90 },
    { 0x9e73b980, 95 },
    { 0x63a3b981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a63b987, 95 },
    { 0x2853b988, 23 },
    { 0, 0 },
    { 0x7043b98a, 33 },
    { 0x65c3b98b, 55 },
    { 0x9de3b98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5093b993, 46 },
    { 0x5903b993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec63b99a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b03b9a2, 100 },
    { 0x6053b9a3, 43 },
    { 0, 0 },
    { 0x6373b9a5, 90 },
    { 0x6d33b9a6, 55 },
    { 0xd1e3b9a7, 33 },
    { 0xb5f3b9a5, 90 },
    { 0x68c3b9a9, 34 },
    { 0, 0 },
    { 0x63b3b9ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d3b9ae, 61 },
    { 0xa1e3b9af, 55 },
    { 0x6203b9ae, 34 },
    { 0, 0 },
    { 0x82f3b9b2, 87 },
    { 0x2a03b9b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e63b9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63b9bd, 97 },
    { 0x5643b9bd, 33 },
    { 0xb5a3b9bf, 98 },
    { 0xb513b9c0, 43 },
    { 0xc263b9c1, 66 },
    { 0x7cd3b9bf, 23 },
    { 0, 0 },
    { 0x52f3b9c4, 46 },
    { 0x303b9c4, 87 },
    { 0x6533b9c4, 33 },
    { 0xae23b9c5, 33 },
    { 0xc293b9c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2333b9cf, 84 },
    { 0, 0 },
    { 0x74e3b9d1, 17 },
    { 0x18e3b9d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6313b9d6, 95 },
    { 0x5bd3b9d7, 61 },
    { 0xf2c3b9d8, 43 },
    { 0xf8d3b9d9, 95 },
    { 0x5213b9da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6673b9e1, 99 },
    { 0x80b3b9e1, 90 },
    { 0x8b73b9e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a3b9e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e3b9e9, 93 },
    { 0x3623b9ea, 46 },
    { 0x3283b9eb, 46 },
    { 0xa023b9ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3763b9f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde3b9fa, 81 },
    { 0x7863b9fa, 33 },
    { 0x68c3b9fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa353b9ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f03ba07, 90 },
    { 0xc633ba07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33ba0b, 33 },
    { 0xa93ba0c, 98 },
    { 0xfa33ba0c, 46 },
    { 0x6d03ba0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a3ba12, 84 },
    { 0xf2a3ba12, 98 },
    { 0x62d3ba14, 66 },
    { 0xa263ba15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2973ba1b, 61 },
    { 0xe2e3ba1b, 33 },
    { 0, 0 },
    { 0xacc3ba1e, 98 },
    { 0x2293ba1e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb3ba28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f3ba2c, 98 },
    { 0, 0 },
    { 0xbef3ba2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c33ba34, 61 },
    { 0x2ba3ba35, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d3ba3f, 85 },
    { 0x543ba3f, 95 },
    { 0x6fc3ba40, 55 },
    { 0, 0 },
    { 0xe053ba43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903ba47, 33 },
    { 0xa983ba48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d3ba4e, 50 },
    { 0xc463ba4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6453ba53, 61 },
    { 0, 0 },
    { 0xa5b3ba55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53ba60, 90 },
    { 0x1e43ba61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5493ba64, 33 },
    { 0xb93ba64, 61 },
    { 0x9943ba66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9813ba69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7933ba6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e3ba75, 90 },
    { 0x6623ba76, 95 },
    { 0xa1e3ba77, 46 },
    { 0xf923ba77, 43 },
    { 0x7203ba79, 98 },
    { 0xb923ba79, 98 },
    { 0x7b93ba7b, 67 },
    { 0x823ba7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b3ba81, 91 },
    { 0xa633ba81, 34 },
    { 0, 0 },
    { 0x3113ba84, 23 },
    { 0xfa03ba85, 90 },
    { 0xa7a3ba86, 61 },
    { 0x3403ba87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa3ba8a, 33 },
    { 0, 0 },
    { 0x3293ba8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc3ba90, 61 },
    { 0x3a43ba90, 19 },
    { 0, 0 },
    { 0x49b3ba93, 61 },
    { 0xa833ba94, 34 },
    { 0, 0 },
    { 0xaa03ba96, 23 },
    { 0xd583ba96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b3ba9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa473ba9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b63baa5, 33 },
    { 0xe203baa6, 61 },
    { 0x8673baa7, 90 },
    { 0xf193baa8, 78 },
    { 0, 0 },
    { 0x8ef3baaa, 98 },
    { 0x1603baab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa3baaf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3173bab7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d3babb, 90 },
    { 0x2bf3babb, 95 },
    { 0x8403babd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9653bac1, 55 },
    { 0x4f23bac1, 90 },
    { 0, 0 },
    { 0x5763bac4, 85 },
    { 0x5b13bac5, 91 },
    { 0x2dd3bac6, 85 },
    { 0x61c3bac6, 61 },
    { 0x4743bac6, 78 },
    { 0x2d53bac4, 93 },
    { 0xf43baca, 33 },
    { 0x5ed3bac5, 76 },
    { 0, 0 },
    { 0x1ae3bacd, 100 },
    { 0xb1f3bacd, 90 },
    { 0xe4c3bacd, 34 },
    { 0x6ff3bad0, 33 },
    { 0, 0 },
    { 0xbd93bad2, 33 },
    { 0, 0 },
    { 0xa3b3bad4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4003bad9, 55 },
    { 0x3793bada, 90 },
    { 0, 0 },
    { 0x1013badc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc13badf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a3bae2, 84 },
    { 0x8f43bae3, 23 },
    { 0xcd13bae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5713baea, 61 },
    { 0xe833baeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383baee, 46 },
    { 0, 0 },
    { 0xd103baf0, 98 },
    { 0xe303baf1, 61 },
    { 0, 0 },
    { 0xfdc3baf3, 98 },
    { 0x8823baf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6153baf7, 61 },
    { 0xd8b3baf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda93bb05, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d23bb08, 33 },
    { 0, 0 },
    { 0xa483bb0a, 98 },
    { 0, 0 },
    { 0xfa53bb0c, 33 },
    { 0xfc23bb0d, 98 },
    { 0, 0 },
    { 0xccc3bb0f, 46 },
    { 0, 0 },
    { 0x9173bb11, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b3bb17, 33 },
    { 0xbda3bb18, 33 },
    { 0x6c43bb19, 99 },
    { 0x1523bb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6963bb20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1223bb29, 23 },
    { 0xf253bb29, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c93bb30, 98 },
    { 0xd363bb31, 93 },
    { 0x5883bb32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a63bb36, 90 },
    { 0x9913bb37, 38 },
    { 0x5593bb37, 64 },
    { 0xb783bb39, 95 },
    { 0x1da3bb37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe93bb3e, 46 },
    { 0xa7a3bb3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe843bb44, 34 },
    { 0x8d43bb45, 23 },
    { 0xd7d3bb46, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a3bb49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d3bb4c, 81 },
    { 0x3743bb4d, 23 },
    { 0xa093bb4c, 24 },
    { 0x77b3bb4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d33bb55, 85 },
    { 0x7313bb56, 76 },
    { 0x68b3bb57, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae93bb5c, 98 },
    { 0x17b3bb5d, 33 },
    { 0x5893bb5e, 90 },
    { 0x6c13bb5f, 99 },
    { 0, 0 },
    { 0x6b83bb61, 93 },
    { 0xa743bb62, 99 },
    { 0, 0 },
    { 0x98b3bb64, 95 },
    { 0xb653bb65, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a3bb6c, 23 },
    { 0xec13bb6d, 51 },
    { 0xb313bb6e, 69 },
    { 0x533bb6f, 61 },
    { 0x2913bb6f, 23 },
    { 0xc073bb71, 66 },
    { 0x3923bb72, 90 },
    { 0xb283bb71, 99 },
    { 0x44a3bb6e, 61 },
    { 0x9b03bb75, 66 },
    { 0, 0 },
    { 0x4663bb77, 90 },
    { 0x89f3bb78, 87 },
    { 0xf8a3bb79, 33 },
    { 0x1e83bb7a, 90 },
    { 0x51d3bb7b, 97 },
    { 0, 0 },
    { 0x8073bb7d, 77 },
    { 0x77d3bb7d, 90 },
    { 0x8253bb7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5173bb82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad63bb88, 23 },
    { 0x7743bb89, 61 },
    { 0x2293bb8a, 76 },
    { 0x7143bb89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3bb90, 34 },
    { 0x2fb3bb91, 77 },
    { 0x3f43bb92, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6753bb96, 93 },
    { 0xb063bb97, 70 },
    { 0x8313bb98, 95 },
    { 0x9d53bb98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca3bb9c, 77 },
    { 0, 0 },
    { 0x98f3bb9e, 36 },
    { 0, 0 },
    { 0xd353bba0, 84 },
    { 0x6653bba1, 66 },
    { 0, 0 },
    { 0xb1d3bba3, 40 },
    { 0, 0 },
    { 0x3683bba5, 98 },
    { 0xd9f3bba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f33bbaa, 12 },
    { 0, 0 },
    { 0x1f43bbac, 99 },
    { 0x1a93bbad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e3bbb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad3bbbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7473bbc6, 80 },
    { 0xe4b3bbc7, 66 },
    { 0xb2e3bbc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e63bbce, 95 },
    { 0x38b3bbcf, 90 },
    { 0, 0 },
    { 0x4f73bbd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1403bbd5, 76 },
    { 0xf063bbd6, 23 },
    { 0x7953bbd6, 76 },
    { 0x8473bbd8, 83 },
    { 0x103bbd9, 66 },
    { 0, 0 },
    { 0x3803bbdb, 43 },
    { 0xa83bbdc, 98 },
    { 0x8e23bbdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8103bbe4, 33 },
    { 0xff3bbe4, 77 },
    { 0x2d93bbe6, 61 },
    { 0xab93bbe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263bbee, 90 },
    { 0x9283bbef, 91 },
    { 0x1c53bbee, 55 },
    { 0x9e13bbef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5423bbf7, 46 },
    { 0, 0 },
    { 0x9893bbf9, 88 },
    { 0x95c3bbfa, 46 },
    { 0x6fa3bbfa, 91 },
    { 0, 0 },
    { 0xc863bbfd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe633bc00, 33 },
    { 0x50b3bc01, 90 },
    { 0, 0 },
    { 0xedb3bc03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f3bc06, 98 },
    { 0x19b3bc07, 87 },
    { 0x3c3bc08, 98 },
    { 0x1f73bc09, 90 },
    { 0, 0 },
    { 0x4783bc0b, 95 },
    { 0, 0 },
    { 0x33bc0d, 95 },
    { 0xc773bc0e, 98 },
    { 0, 0 },
    { 0x6403bc10, 61 },
    { 0, 0 },
    { 0x6d53bc12, 36 },
    { 0x27f3bc13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf3bc16, 43 },
    { 0x733bc17, 23 },
    { 0x6f83bc18, 61 },
    { 0xecf3bc18, 99 },
    { 0, 0 },
    { 0x3a33bc1b, 95 },
    { 0x8e23bc1c, 95 },
    { 0xaae3bc1d, 19 },
    { 0x1f33bc1e, 76 },
    { 0x3863bc1d, 98 },
    { 0x57f3bc20, 55 },
    { 0x2853bc1c, 90 },
    { 0x613bc1e, 55 },
    { 0xda83bc1d, 34 },
    { 0x9fa3bc1b, 61 },
    { 0xb1b3bc25, 95 },
    { 0x9153bc26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3bc2b, 98 },
    { 0, 0 },
    { 0xcf13bc2d, 90 },
    { 0x2e83bc2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3453bc31, 33 },
    { 0xff33bc32, 61 },
    { 0x89d3bc32, 61 },
    { 0x2643bc31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b3bc38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x163bc3b, 33 },
    { 0xbd23bc3b, 98 },
    { 0xbb53bc3b, 23 },
    { 0x5c53bc3e, 75 },
    { 0xf293bc3f, 64 },
    { 0x9ac3bc3e, 61 },
    { 0x93d3bc41, 84 },
    { 0, 0 },
    { 0x5cb3bc43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd663bc47, 98 },
    { 0x1143bc48, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3173bc4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1273bc52, 76 },
    { 0xace3bc53, 33 },
    { 0x1053bc54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4943bc57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc023bc5e, 84 },
    { 0, 0 },
    { 0xa833bc60, 43 },
    { 0, 0 },
    { 0xf143bc62, 90 },
    { 0xce33bc63, 66 },
    { 0, 0 },
    { 0xf3c3bc65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b3bc6d, 33 },
    { 0xe3d3bc6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb463bc71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e3bc76, 46 },
    { 0xe003bc76, 75 },
    { 0x5f83bc78, 87 },
    { 0xc943bc79, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad33bc7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f3bc82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc473bc86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2853bc8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f63bc8d, 38 },
    { 0x4b43bc8e, 99 },
    { 0x48f3bc8f, 33 },
    { 0x45e3bc90, 90 },
    { 0x8c13bc8f, 76 },
    { 0x7d3bc92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a43bc95, 93 },
    { 0xb793bc96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3bc9a, 33 },
    { 0x5dd3bc9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe803bc9e, 90 },
    { 0, 0 },
    { 0x4583bca0, 33 },
    { 0xd123bca1, 84 },
    { 0x1b33bca2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9973bca6, 67 },
    { 0x5643bca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53bcaa, 90 },
    { 0x2833bcaa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8963bcb1, 90 },
    { 0x79e3bcb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee93bcb7, 52 },
    { 0, 0 },
    { 0x5af3bcb9, 61 },
    { 0, 0 },
    { 0xb473bcbb, 33 },
    { 0xab53bcbb, 23 },
    { 0, 0 },
    { 0x32e3bcbe, 46 },
    { 0xffb3bcbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe363bcc2, 84 },
    { 0x81a3bcc3, 65 },
    { 0xf023bcc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1383bcca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c3bccf, 33 },
    { 0x9603bcd0, 90 },
    { 0, 0 },
    { 0x6883bcd2, 78 },
    { 0xa1b3bcd3, 76 },
    { 0xee33bcd4, 52 },
    { 0, 0 },
    { 0xfb83bcd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f3bcdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba03bce0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd703bce5, 23 },
    { 0xd253bce5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde23bcee, 91 },
    { 0xcbe3bcef, 91 },
    { 0xf3bcf0, 98 },
    { 0x9503bcf0, 61 },
    { 0x2a03bcf1, 46 },
    { 0x2b23bcf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d3bcfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf763bd03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783bd08, 90 },
    { 0xc013bd09, 33 },
    { 0x3e93bd0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3843bd0d, 90 },
    { 0x7b33bd0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f3bd11, 99 },
    { 0x5c83bd11, 95 },
    { 0, 0 },
    { 0x3363bd14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53bd17, 55 },
    { 0xc933bd18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d23bd1c, 33 },
    { 0xbf63bd1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xed73bd20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c3bd25, 90 },
    { 0xc7a3bd25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3bd2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b53bd30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7913bd33, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x453bd36, 65 },
    { 0xf003bd37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3533bd3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3bd3e, 77 },
    { 0xe3bd3e, 90 },
    { 0x413bd40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1003bd47, 43 },
    { 0x8433bd48, 61 },
    { 0xc243bd48, 55 },
    { 0x6c3bd4a, 40 },
    { 0x9a3bd4b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f3bd56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff3bd5d, 33 },
    { 0x7a63bd5e, 33 },
    { 0x2f73bd5f, 23 },
    { 0x7d3bd60, 95 },
    { 0xce3bd61, 34 },
    { 0x3583bd5e, 77 },
    { 0, 0 },
    { 0x88d3bd64, 33 },
    { 0, 0 },
    { 0xa0b3bd66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53bd6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a83bd70, 61 },
    { 0xfaf3bd70, 85 },
    { 0x5ea3bd72, 61 },
    { 0x32e3bd72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a3bd77, 33 },
    { 0, 0 },
    { 0x66b3bd79, 33 },
    { 0x1cb3bd79, 90 },
    { 0x9b33bd7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb823bd7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b3bd81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783bd84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf03bd8a, 98 },
    { 0, 0 },
    { 0x9de3bd8c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5083bd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec3bd94, 23 },
    { 0, 0 },
    { 0x7863bd96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xff13bd99, 61 },
    { 0, 0 },
    { 0x9d63bd9b, 23 },
    { 0x37c3bd9b, 95 },
    { 0, 0 },
    { 0xc063bd9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3bda2, 90 },
    { 0x123bda3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac3bda9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5013bdaf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d3bdb4, 33 },
    { 0x4113bdb5, 33 },
    { 0xa7e3bdb4, 90 },
    { 0xb303bdb7, 91 },
    { 0x9213bdb8, 95 },
    { 0, 0 },
    { 0x61e3bdba, 87 },
    { 0, 0 },
    { 0x3753bdbc, 61 },
    { 0x7613bdbc, 76 },
    { 0xc33bdbe, 95 },
    { 0, 0 },
    { 0xbed3bdc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae3bdc6, 93 },
    { 0, 0 },
    { 0xdee3bdc8, 90 },
    { 0x1923bdc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb3bdcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe073bdd2, 76 },
    { 0, 0 },
    { 0x21b3bdd4, 90 },
    { 0, 0 },
    { 0xfec3bdd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493bddc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc3bde1, 95 },
    { 0, 0 },
    { 0xbe83bde3, 34 },
    { 0, 0 },
    { 0xdab3bde5, 52 },
    { 0, 0 },
    { 0xb813bde7, 98 },
    { 0x6643bde7, 95 },
    { 0xbe3bde8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3bded, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b3bdf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7063bdf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c3bdfa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c3bdfe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb23be03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c3be0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e3be13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f3be21, 81 },
    { 0xe923be21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a23be25, 61 },
    { 0x1ae3be25, 90 },
    { 0x2a03be25, 61 },
    { 0x4833be28, 33 },
    { 0x6273be29, 52 },
    { 0xc73be2a, 23 },
    { 0xdd3be2b, 23 },
    { 0x77a3be26, 99 },
    { 0xebe3be26, 67 },
    { 0x91d3be26, 90 },
    { 0x1f43be2f, 90 },
    { 0xe223be2e, 76 },
    { 0, 0 },
    { 0xd63be32, 23 },
    { 0x2333be33, 80 },
    { 0x9083be34, 81 },
    { 0x4623be33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb43be38, 95 },
    { 0, 0 },
    { 0xa83be3a, 33 },
    { 0xca33be3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b3be3f, 80 },
    { 0x45e3be3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3be44, 23 },
    { 0x9983be45, 95 },
    { 0, 0 },
    { 0x4603be47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653be4c, 76 },
    { 0xbe3be4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3be52, 98 },
    { 0, 0 },
    { 0x63a3be54, 90 },
    { 0x22e3be55, 61 },
    { 0xbd53be55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc943be5f, 69 },
    { 0x3e73be60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c63be63, 81 },
    { 0xb283be64, 67 },
    { 0x6a93be63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb3be68, 95 },
    { 0x20b3be69, 90 },
    { 0x5993be69, 46 },
    { 0, 0 },
    { 0xa633be6c, 33 },
    { 0xca83be6d, 33 },
    { 0xca93be6d, 95 },
    { 0xafd3be6c, 87 },
    { 0x8763be70, 90 },
    { 0xb4a3be6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4973be75, 46 },
    { 0xc563be75, 33 },
    { 0xccd3be76, 94 },
    { 0, 0 },
    { 0xa0e3be79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7743be7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6613be80, 64 },
    { 0x46e3be81, 61 },
    { 0x5f53be82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a3be86, 78 },
    { 0x8193be87, 55 },
    { 0x3683be86, 77 },
    { 0x50a3be89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa793be8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a3be91, 55 },
    { 0, 0 },
    { 0x75a3be93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae3be97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca3be9f, 34 },
    { 0x483bea0, 90 },
    { 0xa4c3bea1, 23 },
    { 0x3713bea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca3bea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3053beaa, 76 },
    { 0, 0 },
    { 0x53f3beac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a3beaf, 81 },
    { 0x1fb3beaf, 95 },
    { 0, 0 },
    { 0x1cd3beb2, 69 },
    { 0x77b3beb3, 34 },
    { 0, 0 },
    { 0xa613beb5, 90 },
    { 0x1603beb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5823bebc, 76 },
    { 0xf913bebc, 61 },
    { 0xcff3bebe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb763bec2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3833bec6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7463bec9, 95 },
    { 0xf2f3beca, 61 },
    { 0x83e3beca, 33 },
    { 0x5c23becb, 95 },
    { 0xc443becd, 83 },
    { 0x1de3bece, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a3bed1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b3bed4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb33bedb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf83bede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed3bee3, 90 },
    { 0x1823bee4, 76 },
    { 0x3ff3bee5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d63beea, 85 },
    { 0, 0 },
    { 0x58b3beec, 61 },
    { 0, 0 },
    { 0x85c3beee, 76 },
    { 0xcd73beef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43bef5, 95 },
    { 0x4153bef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1753bef9, 61 },
    { 0x95d3befa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4793beff, 85 },
    { 0x5113bf00, 90 },
    { 0x1343beff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023bf0a, 61 },
    { 0, 0 },
    { 0x4a73bf0c, 99 },
    { 0x1f33bf0c, 46 },
    { 0x7523bf0c, 65 },
    { 0x11c3bf0c, 43 },
    { 0x6b53bf0d, 61 },
    { 0x54c3bf11, 61 },
    { 0x6153bf0c, 90 },
    { 0xe173bf0c, 66 },
    { 0, 0 },
    { 0x2d13bf15, 52 },
    { 0x3293bf16, 98 },
    { 0, 0 },
    { 0xeb53bf18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf103bf1b, 95 },
    { 0xe3e3bf1b, 61 },
    { 0x7903bf1c, 75 },
    { 0, 0 },
    { 0x7803bf1f, 61 },
    { 0xd3b3bf20, 76 },
    { 0xd413bf20, 66 },
    { 0x5483bf22, 83 },
    { 0xad23bf22, 55 },
    { 0xc33bf23, 90 },
    { 0x6b83bf22, 66 },
    { 0x8ab3bf26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc23bf2b, 65 },
    { 0x9273bf2b, 95 },
    { 0, 0 },
    { 0x3ce3bf2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e3bf37, 61 },
    { 0xd853bf38, 93 },
    { 0x53bf38, 34 },
    { 0x63a3bf3a, 33 },
    { 0, 0 },
    { 0xaa83bf3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab3bf3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f63bf43, 55 },
    { 0xb583bf44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313bf48, 61 },
    { 0xaff3bf49, 61 },
    { 0xccf3bf4a, 33 },
    { 0xa7b3bf4b, 46 },
    { 0x473bf48, 90 },
    { 0x703bf4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3bf54, 56 },
    { 0x4223bf55, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e3bf59, 61 },
    { 0xae73bf5a, 84 },
    { 0xf733bf5a, 81 },
    { 0xf433bf5c, 33 },
    { 0, 0 },
    { 0x3f53bf5e, 46 },
    { 0x29d3bf5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec83bf63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b3bf68, 84 },
    { 0x2b43bf69, 98 },
    { 0x9ff3bf69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad3bf6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac3bf71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc063bf76, 95 },
    { 0x3af3bf76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3553bf7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003bf85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe063bf8d, 61 },
    { 0x8633bf8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f3bf92, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e3bf97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa3bfa4, 98 },
    { 0, 0 },
    { 0x3033bfa6, 98 },
    { 0, 0 },
    { 0xe1f3bfa8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc413bfab, 95 },
    { 0x88a3bfab, 33 },
    { 0x2b23bfab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc3bfb0, 90 },
    { 0xe703bfb1, 33 },
    { 0, 0 },
    { 0x4f33bfb3, 98 },
    { 0x22c3bfb4, 67 },
    { 0xe7b3bfb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc563bfc1, 61 },
    { 0x2563bfc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5573bfc6, 95 },
    { 0, 0 },
    { 0x9793bfc8, 23 },
    { 0x90e3bfc8, 33 },
    { 0x4153bfc8, 76 },
    { 0x703bfc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed3bfce, 61 },
    { 0xaeb3bfcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc183bfd2, 33 },
    { 0xd1e3bfd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f13bfdd, 77 },
    { 0x1903bfdd, 90 },
    { 0x6b63bfdd, 97 },
    { 0x2233bfe0, 78 },
    { 0x9c63bfe1, 95 },
    { 0x1e03bfe0, 61 },
    { 0x36d3bfe3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae83bfe7, 33 },
    { 0, 0 },
    { 0x52b3bfe9, 98 },
    { 0x8b53bfea, 98 },
    { 0x7d83bfeb, 95 },
    { 0xce13bfeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f3bfef, 93 },
    { 0x77e3bff0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b3bff3, 94 },
    { 0, 0 },
    { 0x98d3bff5, 23 },
    { 0x6a03bff6, 42 },
    { 0, 0 },
    { 0xa413bff8, 98 },
    { 0xb7b3bff8, 33 },
    { 0xa7f3bffa, 66 },
    { 0x8543bffb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb3c000, 95 },
    { 0x4093c001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf643c006, 61 },
    { 0, 0 },
    { 0xc693c008, 52 },
    { 0xe7c3c008, 33 },
    { 0xfb33c00a, 23 },
    { 0x4ba3c00a, 33 },
    { 0xb923c008, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2623c011, 68 },
    { 0x95e3c012, 55 },
    { 0, 0 },
    { 0x4643c014, 33 },
    { 0xb303c014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3543c01e, 71 },
    { 0, 0 },
    { 0xb03c020, 85 },
    { 0xdd03c021, 95 },
    { 0x42f3c020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da3c029, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d3c031, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3643c035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f23c038, 55 },
    { 0xe453c039, 23 },
    { 0, 0 },
    { 0x1233c03b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe3c046, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393c04d, 46 },
    { 0x523c04e, 33 },
    { 0, 0 },
    { 0x5b93c050, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea73c053, 76 },
    { 0, 0 },
    { 0x8f3c055, 97 },
    { 0x18e3c056, 33 },
    { 0xa9c3c056, 33 },
    { 0x8d03c055, 23 },
    { 0x5bb3c057, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a3c05c, 78 },
    { 0x4183c05d, 95 },
    { 0xd5a3c05c, 76 },
    { 0xeb3c05d, 99 },
    { 0xb503c060, 23 },
    { 0xd2a3c05f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c067, 90 },
    { 0, 0 },
    { 0xb363c069, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43c06d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf3c071, 23 },
    { 0x56a3c072, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663c075, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe603c079, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d3c07e, 95 },
    { 0x26d3c07e, 98 },
    { 0x1f63c080, 23 },
    { 0xb003c081, 65 },
    { 0x53c3c081, 33 },
    { 0x6213c083, 43 },
    { 0xf93c082, 95 },
    { 0x7c73c082, 61 },
    { 0x7743c080, 61 },
    { 0xf973c087, 84 },
    { 0, 0 },
    { 0xb383c089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa3c08e, 66 },
    { 0, 0 },
    { 0xe803c090, 90 },
    { 0x61d3c090, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae83c095, 17 },
    { 0x4e23c095, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d3c099, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d3c09c, 55 },
    { 0xd663c09d, 75 },
    { 0x10d3c09e, 81 },
    { 0x9b63c09e, 90 },
    { 0, 0 },
    { 0x3a53c0a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f3c0a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e3c0a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7823c0ab, 23 },
    { 0xf723c0ab, 23 },
    { 0x3f13c0ab, 66 },
    { 0x7363c0ae, 90 },
    { 0x2c43c0af, 81 },
    { 0x7de3c0b0, 76 },
    { 0xdf53c0ad, 93 },
    { 0, 0 },
    { 0x3963c0b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b3c0b9, 76 },
    { 0, 0 },
    { 0xee93c0bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf333c0be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1983c0c1, 76 },
    { 0x4233c0c1, 33 },
    { 0xc1c3c0c2, 46 },
    { 0xf273c0c4, 33 },
    { 0xcfc3c0c5, 98 },
    { 0x4e83c0c6, 55 },
    { 0x4633c0c7, 77 },
    { 0x643c0c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe3c0cc, 23 },
    { 0xc6e3c0cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2483c0d2, 75 },
    { 0xca53c0d3, 93 },
    { 0x9833c0d4, 98 },
    { 0xff43c0d5, 90 },
    { 0xd5f3c0d6, 90 },
    { 0xa9d3c0d6, 91 },
    { 0xef03c0d8, 61 },
    { 0xf853c0d9, 33 },
    { 0xdb03c0d9, 61 },
    { 0x2613c0db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd343c0e9, 43 },
    { 0x4b03c0ea, 33 },
    { 0xc73c0eb, 95 },
    { 0x3e83c0ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7333c0ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc83c0f3, 95 },
    { 0xdd33c0f3, 33 },
    { 0, 0 },
    { 0x9423c0f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f3c0fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9353c101, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed83c106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2603c10a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633c112, 55 },
    { 0xac33c113, 99 },
    { 0, 0 },
    { 0x2943c115, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe403c118, 90 },
    { 0xc0e3c119, 98 },
    { 0xa163c119, 93 },
    { 0, 0 },
    { 0xffe3c11c, 93 },
    { 0, 0 },
    { 0x1d13c11e, 45 },
    { 0x483c11f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563c126, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee3c12d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc723c13a, 95 },
    { 0, 0 },
    { 0xb763c13c, 68 },
    { 0x2c63c13d, 76 },
    { 0x5723c13d, 91 },
    { 0xd03c13c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a3c144, 91 },
    { 0x6f13c145, 43 },
    { 0xe3b3c144, 100 },
    { 0x5663c144, 90 },
    { 0xea23c148, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c3c14c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a83c150, 93 },
    { 0x3f13c150, 66 },
    { 0x9183c150, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb3c15b, 100 },
    { 0x6003c15c, 87 },
    { 0x9673c15c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b3c162, 98 },
    { 0, 0 },
    { 0x8753c164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a93c16f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe3c172, 55 },
    { 0xb7d3c172, 23 },
    { 0x5413c173, 23 },
    { 0xb343c174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4423c179, 95 },
    { 0x283c179, 66 },
    { 0x70d3c17b, 61 },
    { 0xe483c17c, 96 },
    { 0, 0 },
    { 0xe9d3c17e, 61 },
    { 0x5d13c17f, 23 },
    { 0xa543c17f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3c183, 98 },
    { 0x69d3c184, 61 },
    { 0, 0 },
    { 0x2393c186, 23 },
    { 0xfc13c187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a3c18b, 33 },
    { 0xaed3c18c, 61 },
    { 0, 0 },
    { 0x3d73c18e, 76 },
    { 0x5233c18f, 93 },
    { 0, 0 },
    { 0xea23c191, 93 },
    { 0x25b3c192, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3963c196, 99 },
    { 0, 0 },
    { 0xc5c3c198, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3c19b, 90 },
    { 0xa813c19c, 61 },
    { 0xdc13c19d, 34 },
    { 0x4753c19b, 61 },
    { 0xc133c19f, 43 },
    { 0, 0 },
    { 0xa703c1a1, 76 },
    { 0x5993c1a2, 90 },
    { 0, 0 },
    { 0x6f43c1a4, 43 },
    { 0, 0 },
    { 0x1a03c1a6, 90 },
    { 0xd3b3c1a7, 33 },
    { 0xddc3c1a7, 91 },
    { 0, 0 },
    { 0x5c13c1aa, 92 },
    { 0xad43c1aa, 67 },
    { 0, 0 },
    { 0xb553c1ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a83c1b0, 91 },
    { 0xef93c1b1, 77 },
    { 0xba83c1b0, 93 },
    { 0xa043c1b3, 34 },
    { 0xe773c1b3, 95 },
    { 0xdfe3c1b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c3c1bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c1c5, 61 },
    { 0x2513c1c6, 46 },
    { 0xb323c1c6, 98 },
    { 0x95e3c1c6, 95 },
    { 0, 0 },
    { 0xd333c1ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9793c1cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43c1d2, 61 },
    { 0xc613c1d3, 78 },
    { 0x7813c1d4, 52 },
    { 0xc893c1d5, 23 },
    { 0xd4f3c1d6, 99 },
    { 0xe053c1d7, 61 },
    { 0x693c1d8, 76 },
    { 0, 0 },
    { 0x8323c1da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf23c1dd, 66 },
    { 0, 0 },
    { 0x1993c1df, 98 },
    { 0x31f3c1df, 61 },
    { 0, 0 },
    { 0x6e23c1e2, 23 },
    { 0xdd13c1e3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b3c1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab3c1ec, 98 },
    { 0x14e3c1ed, 23 },
    { 0x3ee3c1ee, 23 },
    { 0x1f63c1ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea83c1f2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5523c1f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9233c1f9, 76 },
    { 0, 0 },
    { 0x75b3c1fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3c203, 33 },
    { 0xed33c204, 38 },
    { 0x10a3c205, 90 },
    { 0xf843c206, 88 },
    { 0x39e3c207, 90 },
    { 0, 0 },
    { 0x2843c209, 98 },
    { 0xd6b3c20a, 66 },
    { 0x79f3c20b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb963c213, 95 },
    { 0x73b3c214, 61 },
    { 0x37b3c215, 23 },
    { 0, 0 },
    { 0xcfe3c217, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d3c21a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf183c223, 34 },
    { 0x5b23c224, 68 },
    { 0, 0 },
    { 0xc583c226, 23 },
    { 0, 0 },
    { 0x31a3c228, 33 },
    { 0, 0 },
    { 0xae03c22a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403c22d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c83c230, 34 },
    { 0xfda3c230, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9323c235, 90 },
    { 0x3e13c236, 40 },
    { 0, 0 },
    { 0x31f3c238, 95 },
    { 0, 0 },
    { 0xf683c23a, 23 },
    { 0, 0 },
    { 0x8283c23c, 61 },
    { 0xcbb3c23d, 76 },
    { 0x9ad3c23d, 34 },
    { 0xb383c23f, 88 },
    { 0xd5c3c240, 33 },
    { 0x1bc3c241, 55 },
    { 0xa863c242, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d3c245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3c24e, 77 },
    { 0x61a3c24f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d03c254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d3c25a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc93c25f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313c267, 95 },
    { 0x74b3c267, 33 },
    { 0x20e3c269, 55 },
    { 0x633c26a, 61 },
    { 0x82c3c26b, 33 },
    { 0x8333c26a, 95 },
    { 0x2c23c26b, 96 },
    { 0x2ba3c269, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd3c276, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e3c27e, 97 },
    { 0xfad3c27f, 76 },
    { 0x95e3c280, 61 },
    { 0xcd83c27e, 33 },
    { 0x78f3c282, 23 },
    { 0xac33c283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3163c286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3173c289, 98 },
    { 0xa093c28a, 93 },
    { 0x7e73c28a, 90 },
    { 0, 0 },
    { 0x5de3c28d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f3c295, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b3c298, 90 },
    { 0xe853c299, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d53c29c, 96 },
    { 0x71d3c29d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda3c2a0, 33 },
    { 0xe533c2a1, 90 },
    { 0x16d3c2a2, 55 },
    { 0xcf13c2a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d3c2a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93c2ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3533c2b2, 80 },
    { 0x74a3c2b3, 83 },
    { 0xfd83c2b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3353c2bc, 98 },
    { 0x5923c2bd, 101 },
    { 0x3683c2bd, 98 },
    { 0x37b3c2bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf33c2c8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa3c2cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb03c2d4, 55 },
    { 0x1113c2d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e33c2dc, 78 },
    { 0x9b33c2dd, 84 },
    { 0x17d3c2dc, 43 },
    { 0x2fe3c2dd, 55 },
    { 0xd2e3c2e0, 23 },
    { 0xcf83c2e1, 61 },
    { 0x56b3c2e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee3c2ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23c2ed, 23 },
    { 0x4ed3c2ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a53c2fa, 65 },
    { 0, 0 },
    { 0x3823c2fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6853c30a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf03c30e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483c313, 95 },
    { 0, 0 },
    { 0x6d03c315, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f3c31d, 34 },
    { 0x2983c31e, 91 },
    { 0x7233c31f, 52 },
    { 0xec13c31e, 95 },
    { 0xe073c321, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1733c32b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e73c32e, 46 },
    { 0x7543c32e, 43 },
    { 0, 0 },
    { 0x30c3c331, 43 },
    { 0, 0 },
    { 0x5643c333, 52 },
    { 0xcec3c333, 46 },
    { 0xf523c335, 90 },
    { 0, 0 },
    { 0x22b3c337, 34 },
    { 0x2f63c338, 33 },
    { 0x59b3c338, 87 },
    { 0x12f3c33a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa203c33d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce23c342, 61 },
    { 0x4213c343, 84 },
    { 0xa4b3c344, 22 },
    { 0x8d23c345, 33 },
    { 0xa0e3c346, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3c34a, 61 },
    { 0xc273c34b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2333c34e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8503c352, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223c357, 52 },
    { 0xb113c358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3c35b, 93 },
    { 0x8b53c35b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e3c361, 33 },
    { 0x1ab3c362, 34 },
    { 0, 0 },
    { 0x5c53c364, 99 },
    { 0xaef3c364, 90 },
    { 0x8583c366, 98 },
    { 0xfeb3c367, 90 },
    { 0x6ae3c368, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd293c36b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da3c36e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd83c375, 66 },
    { 0xf983c376, 66 },
    { 0x3fa3c375, 98 },
    { 0, 0 },
    { 0x9cb3c379, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e63c37c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e3c37f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a3c382, 78 },
    { 0, 0 },
    { 0x2d73c384, 61 },
    { 0x9233c385, 46 },
    { 0x9ef3c384, 95 },
    { 0, 0 },
    { 0x6583c388, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d73c38b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6773c38f, 90 },
    { 0x1f73c390, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe353c396, 23 },
    { 0xc163c396, 76 },
    { 0x3b83c398, 46 },
    { 0xfb73c399, 46 },
    { 0xe53c398, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa893c3ab, 23 },
    { 0, 0 },
    { 0x33c3ad, 95 },
    { 0x673c3ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b3c3b1, 88 },
    { 0xb963c3b2, 43 },
    { 0x4883c3b2, 100 },
    { 0x8273c3b4, 77 },
    { 0xf7e3c3b4, 33 },
    { 0x3363c3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd3c3ba, 98 },
    { 0x12f3c3bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b3c3c0, 87 },
    { 0, 0 },
    { 0x47a3c3c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a13c3c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1223c3cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa333c3d1, 96 },
    { 0x9bf3c3d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3c3d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4813c3df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf073c3ee, 90 },
    { 0x6fb3c3ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3033c3f3, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b3c3f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb183c3fd, 95 },
    { 0xfdb3c3fe, 52 },
    { 0x2053c3ff, 66 },
    { 0xabd3c400, 33 },
    { 0, 0 },
    { 0x6263c402, 33 },
    { 0x60d3c403, 61 },
    { 0, 0 },
    { 0xbd3c405, 55 },
    { 0xc973c406, 46 },
    { 0x1603c407, 52 },
    { 0x1df3c407, 43 },
    { 0x83d3c409, 33 },
    { 0x39c3c409, 99 },
    { 0xcbc3c40b, 23 },
    { 0xf043c40c, 33 },
    { 0x9aa3c40c, 67 },
    { 0x8193c40e, 67 },
    { 0xba23c40f, 98 },
    { 0x1393c408, 43 },
    { 0x53c3c40a, 55 },
    { 0x5f63c412, 90 },
    { 0, 0 },
    { 0xa943c414, 61 },
    { 0xcf03c414, 76 },
    { 0xad43c415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23c419, 23 },
    { 0xfc73c41a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3403c424, 98 },
    { 0x44e3c425, 90 },
    { 0x3023c426, 33 },
    { 0x483c427, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f3c42b, 81 },
    { 0x7e93c42c, 90 },
    { 0xfdf3c42d, 33 },
    { 0xd513c42c, 23 },
    { 0x46e3c42f, 34 },
    { 0x813c42e, 87 },
    { 0xc493c42f, 95 },
    { 0x4e03c432, 76 },
    { 0x3d13c432, 34 },
    { 0xe843c434, 77 },
    { 0x40f3c434, 43 },
    { 0xa323c42e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce93c43b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf03c444, 55 },
    { 0xb523c445, 91 },
    { 0, 0 },
    { 0x13c3c447, 33 },
    { 0xdbe3c448, 76 },
    { 0x46b3c449, 84 },
    { 0x9b33c449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c3c44f, 33 },
    { 0x263c450, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563c453, 95 },
    { 0xe0d3c453, 61 },
    { 0, 0 },
    { 0xd9e3c456, 98 },
    { 0xb1a3c457, 98 },
    { 0xe1e3c457, 91 },
    { 0, 0 },
    { 0x8e43c45a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3503c45e, 99 },
    { 0, 0 },
    { 0xa423c460, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c465, 46 },
    { 0x3653c465, 61 },
    { 0xd233c467, 95 },
    { 0xcf13c468, 23 },
    { 0x673c468, 33 },
    { 0, 0 },
    { 0x2d33c46b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf583c46e, 66 },
    { 0x6863c46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb883c472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d3c476, 93 },
    { 0x21f3c477, 55 },
    { 0x8d63c477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8923c47c, 61 },
    { 0, 0 },
    { 0x6d03c47e, 43 },
    { 0xc5e3c47f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb3c486, 98 },
    { 0x3c83c487, 46 },
    { 0, 0 },
    { 0xa093c489, 33 },
    { 0x60d3c48a, 100 },
    { 0x8c33c48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe353c490, 33 },
    { 0x6313c490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3303c498, 33 },
    { 0, 0 },
    { 0xacd3c49a, 67 },
    { 0xa063c49a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93c4a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f3c4a4, 95 },
    { 0xba43c4a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5853c4a9, 46 },
    { 0, 0 },
    { 0xef83c4ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a3c4af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb3c4b2, 61 },
    { 0xa083c4b3, 34 },
    { 0x1023c4b3, 61 },
    { 0, 0 },
    { 0x71e3c4b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c23c4b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53c4bc, 23 },
    { 0x61d3c4bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf813c4c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b3c4c5, 95 },
    { 0, 0 },
    { 0x27b3c4c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa3c4ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e3c4d2, 33 },
    { 0, 0 },
    { 0x5a03c4d4, 100 },
    { 0x70a3c4d5, 90 },
    { 0, 0 },
    { 0xddc3c4d7, 84 },
    { 0, 0 },
    { 0xbbf3c4d9, 90 },
    { 0x2833c4da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e3c4dd, 98 },
    { 0xd263c4dd, 85 },
    { 0xc4b3c4df, 33 },
    { 0x3f83c4df, 100 },
    { 0xa253c4de, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b3c4e5, 61 },
    { 0xb3d3c4e5, 33 },
    { 0, 0 },
    { 0xf4d3c4e8, 76 },
    { 0xde33c4e9, 33 },
    { 0x92c3c4e9, 61 },
    { 0x65b3c4ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb003c4f2, 95 },
    { 0x26b3c4f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c53c4f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3c4fb, 76 },
    { 0x1d23c4fb, 98 },
    { 0xf883c4fd, 90 },
    { 0, 0 },
    { 0x4023c4ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe783c502, 52 },
    { 0x71b3c502, 91 },
    { 0x9773c504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b3c50c, 91 },
    { 0, 0 },
    { 0xc113c50e, 33 },
    { 0xdd03c50e, 95 },
    { 0x8ae3c510, 93 },
    { 0x6203c510, 90 },
    { 0x50a3c50e, 61 },
    { 0xc943c512, 76 },
    { 0xc323c514, 61 },
    { 0x25b3c513, 93 },
    { 0, 0 },
    { 0xe4c3c517, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c3c51a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d23c51d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c3c523, 69 },
    { 0x9ef3c524, 66 },
    { 0x5583c524, 61 },
    { 0xdbb3c525, 98 },
    { 0x2553c525, 95 },
    { 0x91a3c527, 35 },
    { 0x1493c529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6333c535, 34 },
    { 0, 0 },
    { 0x70c3c537, 33 },
    { 0, 0 },
    { 0x3753c539, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee03c53f, 91 },
    { 0, 0 },
    { 0x44d3c541, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3683c547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33c550, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e3c555, 55 },
    { 0x2a83c556, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b3c55a, 95 },
    { 0x7633c55b, 91 },
    { 0x9cb3c55a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d3c560, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa973c564, 65 },
    { 0x5b73c564, 95 },
    { 0xc003c565, 76 },
    { 0x8743c566, 66 },
    { 0x9f83c564, 100 },
    { 0x3db3c569, 95 },
    { 0x8c03c56a, 64 },
    { 0x3a43c564, 90 },
    { 0x47b3c564, 93 },
    { 0, 0 },
    { 0xeb13c56e, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a63c573, 76 },
    { 0xeba3c574, 68 },
    { 0x8113c575, 76 },
    { 0x9773c575, 34 },
    { 0xcf63c575, 90 },
    { 0xb713c578, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c43c57b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3c57e, 33 },
    { 0xf373c57f, 23 },
    { 0x6053c57e, 76 },
    { 0x67c3c57f, 75 },
    { 0x7753c582, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b33c585, 98 },
    { 0x1f03c586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9143c58b, 95 },
    { 0xc693c58c, 61 },
    { 0x1bd3c58d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e43c593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c63c599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b73c59c, 95 },
    { 0x9bf3c59d, 61 },
    { 0xeae3c59d, 43 },
    { 0, 0 },
    { 0x7cc3c5a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a3c5ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7523c5af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f3c5b2, 23 },
    { 0xb0a3c5b3, 69 },
    { 0x1a33c5b3, 43 },
    { 0, 0 },
    { 0x7b83c5b6, 35 },
    { 0, 0 },
    { 0x7783c5b8, 33 },
    { 0xb8b3c5b9, 98 },
    { 0, 0 },
    { 0x9013c5bb, 61 },
    { 0x2583c5bc, 33 },
    { 0xfc43c5bd, 100 },
    { 0xdcf3c5bd, 68 },
    { 0xf733c5bf, 78 },
    { 0x5c63c5bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873c5c4, 23 },
    { 0x71b3c5c5, 76 },
    { 0, 0 },
    { 0xea53c5c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6883c5cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc433c5d1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe903c5d7, 33 },
    { 0x6583c5d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1363c5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa513c5e7, 76 },
    { 0xb2d3c5e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd333c5eb, 77 },
    { 0, 0 },
    { 0xe0f3c5ed, 33 },
    { 0xa6b3c5ee, 78 },
    { 0x52e3c5ef, 78 },
    { 0xd6c3c5ef, 33 },
    { 0x7153c5f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d3c5fe, 55 },
    { 0, 0 },
    { 0xa663c600, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f3c606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac3c609, 33 },
    { 0x9533c609, 90 },
    { 0xe803c609, 98 },
    { 0x3ab3c60c, 66 },
    { 0xd3d3c60d, 61 },
    { 0x9ad3c60a, 61 },
    { 0x5383c609, 95 },
    { 0xf2a3c610, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd63c614, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013c618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e3c61f, 46 },
    { 0xfb53c620, 65 },
    { 0x43c3c621, 90 },
    { 0x2683c620, 23 },
    { 0x7a63c61f, 33 },
    { 0x9fc3c624, 34 },
    { 0x6903c61f, 91 },
    { 0xcec3c624, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa563c62b, 90 },
    { 0, 0 },
    { 0xad23c62d, 90 },
    { 0x4d33c62e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d93c633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac3c636, 90 },
    { 0xa5a3c636, 34 },
    { 0, 0 },
    { 0xb2f3c639, 78 },
    { 0x24d3c63a, 99 },
    { 0, 0 },
    { 0xc583c63c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63c644, 55 },
    { 0xd153c645, 23 },
    { 0, 0 },
    { 0x8633c647, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc83c64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c3c64e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e3c658, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f3c65b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263c65f, 33 },
    { 0x7243c660, 52 },
    { 0x7333c661, 12 },
    { 0x22e3c662, 23 },
    { 0xa983c663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933c667, 33 },
    { 0x3823c668, 90 },
    { 0, 0 },
    { 0xc8a3c66a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed3c66d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b3c670, 95 },
    { 0x3903c671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd3c674, 90 },
    { 0x3693c675, 23 },
    { 0x9293c676, 77 },
    { 0x9e63c675, 95 },
    { 0x9a03c674, 76 },
    { 0x95d3c674, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd063c67d, 33 },
    { 0x9603c67e, 33 },
    { 0xf603c67d, 95 },
    { 0xb783c680, 33 },
    { 0, 0 },
    { 0xe8a3c682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd373c686, 95 },
    { 0, 0 },
    { 0x6363c688, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9693c68c, 93 },
    { 0x18f3c68c, 99 },
    { 0, 0 },
    { 0x76a3c68f, 61 },
    { 0x41c3c68f, 43 },
    { 0xed13c691, 61 },
    { 0xd883c692, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9633c69a, 52 },
    { 0x4c33c69b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d3c69e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3c6a2, 90 },
    { 0xfd13c6a3, 33 },
    { 0x11e3c6a4, 33 },
    { 0x943c6a5, 33 },
    { 0, 0 },
    { 0x7873c6a7, 61 },
    { 0, 0 },
    { 0xb423c6a9, 33 },
    { 0x3173c6aa, 99 },
    { 0, 0 },
    { 0x6083c6ac, 76 },
    { 0x1e93c6ad, 60 },
    { 0x4033c6ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad43c6b2, 78 },
    { 0xeb33c6b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce93c6b9, 61 },
    { 0x5a33c6ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2423c6bd, 34 },
    { 0xb763c6bd, 43 },
    { 0, 0 },
    { 0x3c53c6c0, 23 },
    { 0, 0 },
    { 0xb213c6c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f43c6c7, 43 },
    { 0xa883c6c8, 66 },
    { 0x7b03c6c9, 90 },
    { 0xbf3c6c9, 65 },
    { 0x8163c6cb, 98 },
    { 0x5983c6c7, 34 },
    { 0x85e3c6cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb453c6d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a3c6d6, 95 },
    { 0xa4a3c6d7, 98 },
    { 0xa173c6d6, 93 },
    { 0x3933c6d8, 90 },
    { 0xbfa3c6da, 33 },
    { 0x55f3c6d9, 99 },
    { 0x9973c6d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743c6e5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3c6e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a23c6f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb23c6fa, 23 },
    { 0x7ae3c6fb, 52 },
    { 0xa303c6fc, 55 },
    { 0x8b03c6fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec3c700, 90 },
    { 0xb8f3c701, 34 },
    { 0xd2e3c701, 33 },
    { 0x11b3c700, 98 },
    { 0xd5b3c704, 100 },
    { 0x6393c704, 95 },
    { 0x4073c704, 90 },
    { 0x3493c704, 98 },
    { 0x46b3c708, 61 },
    { 0xa483c706, 95 },
    { 0xf583c709, 76 },
    { 0, 0 },
    { 0x9c73c70c, 90 },
    { 0x57c3c70c, 43 },
    { 0xb103c70e, 38 },
    { 0x59c3c70e, 76 },
    { 0xfc93c710, 98 },
    { 0x5d33c70e, 33 },
    { 0x4283c70c, 61 },
    { 0x90a3c70e, 33 },
    { 0, 0 },
    { 0x72f3c715, 34 },
    { 0, 0 },
    { 0xa43c717, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf653c71c, 98 },
    { 0, 0 },
    { 0xf933c71e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d3c724, 23 },
    { 0xda83c725, 33 },
    { 0, 0 },
    { 0xed63c727, 55 },
    { 0xf423c727, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783c734, 61 },
    { 0, 0 },
    { 0xfb23c736, 23 },
    { 0x43a3c736, 23 },
    { 0x32e3c737, 66 },
    { 0x773c737, 90 },
    { 0x5d43c73a, 33 },
    { 0, 0 },
    { 0x2173c73c, 43 },
    { 0xb343c73c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c3c740, 95 },
    { 0x20e3c741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4013c746, 61 },
    { 0xae3c747, 61 },
    { 0x6b43c747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3c74b, 34 },
    { 0xfe43c74c, 95 },
    { 0x7a13c74d, 43 },
    { 0x6603c74b, 95 },
    { 0xac83c74f, 93 },
    { 0xccf3c750, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b3c755, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c3c758, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e23c75c, 90 },
    { 0x4413c75d, 90 },
    { 0xc223c75c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a93c763, 61 },
    { 0xd323c764, 17 },
    { 0xfd73c765, 65 },
    { 0xab03c766, 100 },
    { 0x33f3c766, 33 },
    { 0, 0 },
    { 0xc803c769, 98 },
    { 0x57b3c769, 98 },
    { 0, 0 },
    { 0x36f3c76c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4573c76f, 33 },
    { 0x9013c770, 47 },
    { 0, 0 },
    { 0x9fd3c772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e3c776, 17 },
    { 0, 0 },
    { 0xd013c778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b3c781, 33 },
    { 0, 0 },
    { 0xf153c783, 98 },
    { 0x1993c784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe323c787, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b3c78a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe993c793, 33 },
    { 0xbcf3c794, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc3c797, 49 },
    { 0, 0 },
    { 0x3843c799, 55 },
    { 0xd123c79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7443c79e, 85 },
    { 0x48e3c79f, 46 },
    { 0xc473c7a0, 95 },
    { 0x523c7a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933c7a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb003c7ac, 76 },
    { 0xb943c7ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe033c7b2, 85 },
    { 0xf9a3c7b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3463c7b6, 23 },
    { 0xe1e3c7b6, 34 },
    { 0x6cc3c7b8, 78 },
    { 0x1d53c7b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7493c7bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5793c7c3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a33c7c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c3c7c9, 99 },
    { 0x8e23c7ca, 61 },
    { 0xef93c7cb, 46 },
    { 0xa203c7cb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e3c7d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1813c7d5, 15 },
    { 0xe053c7d6, 97 },
    { 0, 0 },
    { 0x7333c7d8, 95 },
    { 0x1303c7d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd623c7df, 67 },
    { 0, 0 },
    { 0x8953c7e1, 66 },
    { 0x2493c7e2, 95 },
    { 0, 0 },
    { 0x28e3c7e4, 76 },
    { 0xbb3c7e5, 95 },
    { 0xd633c7e6, 33 },
    { 0x1263c7e4, 55 },
    { 0xd4f3c7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9943c7ec, 91 },
    { 0x5b33c7ed, 85 },
    { 0xcf3c7ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae3c7f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe563c7f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8113c7fe, 33 },
    { 0, 0 },
    { 0x7ef3c800, 90 },
    { 0xffb3c801, 91 },
    { 0xb933c802, 98 },
    { 0x4b63c803, 55 },
    { 0xd823c804, 33 },
    { 0, 0 },
    { 0xe053c806, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6663c80a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3c814, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad3c817, 78 },
    { 0x4c63c818, 61 },
    { 0, 0 },
    { 0x73b3c81a, 75 },
    { 0, 0 },
    { 0x2f53c81c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4833c821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9213c825, 23 },
    { 0x5673c826, 90 },
    { 0, 0 },
    { 0x5653c828, 90 },
    { 0xdcb3c829, 33 },
    { 0x90a3c828, 34 },
    { 0x3e13c82b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe463c82e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb3c833, 76 },
    { 0x2ed3c833, 78 },
    { 0x9983c835, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4993c838, 78 },
    { 0xb553c839, 61 },
    { 0xa723c838, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c3c841, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b3c84d, 95 },
    { 0x7953c84e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a3c852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1313c858, 76 },
    { 0x8c63c858, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae3c861, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3803c864, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba3c86a, 100 },
    { 0, 0 },
    { 0x3e53c86c, 88 },
    { 0x43a3c86d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa303c870, 90 },
    { 0x37c3c871, 77 },
    { 0x193c870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1103c876, 95 },
    { 0, 0 },
    { 0x7a03c878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b93c87e, 95 },
    { 0, 0 },
    { 0x4083c880, 90 },
    { 0, 0 },
    { 0xc2d3c882, 33 },
    { 0, 0 },
    { 0x7af3c884, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f73c888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f3c88b, 98 },
    { 0, 0 },
    { 0xc003c88d, 23 },
    { 0x8ab3c88e, 21 },
    { 0xc5c3c88f, 100 },
    { 0xb8b3c890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe783c894, 90 },
    { 0xd1e3c895, 101 },
    { 0x6f93c896, 34 },
    { 0x933c897, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd283c89a, 78 },
    { 0xb783c89a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b3c8a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf923c8a6, 23 },
    { 0xd9f3c8a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4413c8b3, 33 },
    { 0, 0 },
    { 0xd263c8b5, 23 },
    { 0x9b33c8b6, 61 },
    { 0xd5b3c8b7, 90 },
    { 0x7703c8b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb783c8bc, 23 },
    { 0x513c8bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a3c8c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3c8cc, 59 },
    { 0, 0 },
    { 0x6e03c8ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe493c8d3, 23 },
    { 0x2ad3c8d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc3c8e1, 99 },
    { 0xf793c8e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd3c8e5, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xd583c8e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f3c8ee, 55 },
    { 0x8853c8ef, 98 },
    { 0, 0 },
    { 0x2823c8f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da3c8f4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4273c8f7, 90 },
    { 0x1423c8f7, 76 },
    { 0x2d23c8f9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc603c902, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df3c906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c90b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd893c90e, 68 },
    { 0x3a73c90f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7213c914, 90 },
    { 0, 0 },
    { 0x4ce3c916, 90 },
    { 0x14e3c916, 61 },
    { 0xdaa3c918, 83 },
    { 0x9d63c919, 43 },
    { 0, 0 },
    { 0x193c91b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb3c91f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93c922, 55 },
    { 0, 0 },
    { 0x26a3c924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b43c927, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e23c930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8883c939, 19 },
    { 0x8113c939, 33 },
    { 0, 0 },
    { 0x6533c93c, 98 },
    { 0xed03c93c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd73c946, 90 },
    { 0, 0 },
    { 0xa973c948, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e3c94c, 77 },
    { 0xc2c3c94d, 88 },
    { 0x24e3c94c, 68 },
    { 0xa7a3c94d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd3c954, 68 },
    { 0x5943c955, 93 },
    { 0x7cf3c956, 84 },
    { 0x1823c954, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd3c95b, 17 },
    { 0xdb83c95c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a3c964, 98 },
    { 0x7563c965, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb953c96a, 90 },
    { 0x9633c96b, 46 },
    { 0x79e3c96c, 52 },
    { 0x7003c96d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1033c970, 61 },
    { 0xab73c970, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea63c974, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5083c978, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3c97b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa3c97f, 90 },
    { 0x6933c980, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf933c984, 76 },
    { 0xa6a3c985, 17 },
    { 0x3283c986, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3c98e, 81 },
    { 0, 0 },
    { 0xc1f3c990, 95 },
    { 0x1fc3c990, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac3c994, 52 },
    { 0, 0 },
    { 0xc793c996, 90 },
    { 0xd913c997, 98 },
    { 0x9d83c998, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xec33c99b, 77 },
    { 0, 0 },
    { 0x3893c99d, 98 },
    { 0x2603c99e, 90 },
    { 0xa903c99f, 95 },
    { 0, 0 },
    { 0x19f3c9a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c73c9aa, 33 },
    { 0x31f3c9ab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4433c9ae, 98 },
    { 0x4d63c9af, 17 },
    { 0x3d3c9af, 91 },
    { 0xd53c9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad3c9b5, 40 },
    { 0, 0 },
    { 0xa713c9b7, 36 },
    { 0xba53c9b8, 90 },
    { 0x9c83c9b8, 85 },
    { 0x1a23c9b9, 90 },
    { 0, 0 },
    { 0x2263c9bc, 61 },
    { 0, 0 },
    { 0x53e3c9be, 85 },
    { 0xeb43c9be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f33c9c6, 95 },
    { 0xd193c9c6, 80 },
    { 0x3d53c9c6, 43 },
    { 0x94b3c9c7, 34 },
    { 0xcc93c9ca, 34 },
    { 0x9f63c9cb, 67 },
    { 0x3e43c9cc, 90 },
    { 0, 0 },
    { 0xd03c9ce, 23 },
    { 0xe343c9cf, 84 },
    { 0xdaf3c9d0, 35 },
    { 0x2633c9d1, 23 },
    { 0x8183c9cf, 90 },
    { 0x2aa3c9d3, 95 },
    { 0, 0 },
    { 0xc343c9d5, 95 },
    { 0xdcb3c9d6, 95 },
    { 0x2b43c9d7, 90 },
    { 0x2a33c9d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8533c9e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c63c9ed, 76 },
    { 0, 0 },
    { 0xf3e3c9ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b53c9f3, 98 },
    { 0x2e23c9f4, 98 },
    { 0xe423c9f4, 100 },
    { 0x35a3c9f6, 61 },
    { 0xc873c9f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae23c9ff, 95 },
    { 0, 0 },
    { 0x62e3ca01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3ca05, 95 },
    { 0xa5e3ca06, 23 },
    { 0x4853ca06, 61 },
    { 0xce03ca08, 84 },
    { 0xcb23ca06, 65 },
    { 0x5a33ca06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53ca0e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1613ca14, 98 },
    { 0x1c63ca14, 98 },
    { 0x7733ca16, 46 },
    { 0xa573ca15, 23 },
    { 0x3f33ca16, 61 },
    { 0x12b3ca19, 52 },
    { 0, 0 },
    { 0xfd53ca1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893ca21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6803ca24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f43ca2e, 33 },
    { 0, 0 },
    { 0x78a3ca30, 46 },
    { 0x5073ca31, 23 },
    { 0xb8d3ca32, 44 },
    { 0x3913ca32, 23 },
    { 0xa8f3ca32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d3ca3c, 92 },
    { 0, 0 },
    { 0xe943ca3e, 58 },
    { 0x7b43ca3f, 87 },
    { 0, 0 },
    { 0x8673ca41, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2953ca47, 23 },
    { 0x2c73ca48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c3ca4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd113ca50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a3ca53, 69 },
    { 0, 0 },
    { 0xe813ca55, 55 },
    { 0, 0 },
    { 0x4663ca57, 40 },
    { 0xd3b3ca58, 98 },
    { 0x3343ca57, 85 },
    { 0, 0 },
    { 0xbe53ca5b, 61 },
    { 0x2863ca5c, 61 },
    { 0xe053ca5c, 33 },
    { 0x98d3ca5d, 90 },
    { 0x9643ca5f, 66 },
    { 0x24d3ca5f, 90 },
    { 0, 0 },
    { 0x17c3ca62, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e03ca65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b83ca68, 90 },
    { 0xde43ca69, 98 },
    { 0xbdf3ca6a, 61 },
    { 0x9553ca6b, 23 },
    { 0x8133ca6b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f3ca6f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e23ca73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3153ca76, 76 },
    { 0xde73ca77, 99 },
    { 0x7143ca78, 17 },
    { 0xa743ca79, 90 },
    { 0, 0 },
    { 0x70b3ca7b, 91 },
    { 0xfe63ca7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6433ca7f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153ca8c, 20 },
    { 0, 0 },
    { 0x3523ca8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de3ca92, 23 },
    { 0xbb93ca92, 55 },
    { 0, 0 },
    { 0x353ca95, 35 },
    { 0x213ca95, 90 },
    { 0x55e3ca95, 55 },
    { 0xaa3ca98, 66 },
    { 0x8c43ca99, 95 },
    { 0x7ba3ca9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad53caa2, 23 },
    { 0xefa3caa3, 76 },
    { 0, 0 },
    { 0x4953caa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1493caa8, 65 },
    { 0, 0 },
    { 0x8ed3caaa, 46 },
    { 0x26a3caab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3cab0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c3cab4, 33 },
    { 0xa563cab5, 32 },
    { 0, 0 },
    { 0x30f3cab7, 98 },
    { 0, 0 },
    { 0x4ec3cab9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1113cabe, 98 },
    { 0x5f03cabf, 90 },
    { 0xb273cabe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf683cac8, 17 },
    { 0, 0 },
    { 0x3623caca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3cacd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6293cad0, 61 },
    { 0x15e3cad1, 43 },
    { 0, 0 },
    { 0x4ed3cad3, 65 },
    { 0x9343cad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed3cad7, 33 },
    { 0, 0 },
    { 0x49e3cad9, 61 },
    { 0, 0 },
    { 0x3ee3cadb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc3cade, 98 },
    { 0x3cb3cade, 76 },
    { 0xced3cae0, 97 },
    { 0xe1c3cae1, 66 },
    { 0x4643cae2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3cae6, 61 },
    { 0x1e43cae7, 58 },
    { 0xa03cae8, 90 },
    { 0xc183cae9, 83 },
    { 0xd723caea, 43 },
    { 0, 0 },
    { 0xc7f3caec, 46 },
    { 0x7ff3caed, 23 },
    { 0x8f43caec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4713caf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc703caf7, 98 },
    { 0, 0 },
    { 0xd6b3caf9, 23 },
    { 0xf363cafa, 93 },
    { 0, 0 },
    { 0xb3d3cafc, 55 },
    { 0, 0 },
    { 0x2f83cafe, 92 },
    { 0x4bf3cafe, 99 },
    { 0x483cb00, 36 },
    { 0, 0 },
    { 0x3ec3cb02, 33 },
    { 0xf673cb03, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc983cb08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5373cb0c, 78 },
    { 0x85b3cb0d, 96 },
    { 0xb1f3cb0d, 61 },
    { 0xb323cb0e, 95 },
    { 0xb83cb10, 93 },
    { 0x3d03cb11, 99 },
    { 0xb13cb10, 90 },
    { 0x1253cb13, 49 },
    { 0xa573cb14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c3cb1b, 90 },
    { 0x293cb1c, 98 },
    { 0x37d3cb1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd53cb20, 23 },
    { 0, 0 },
    { 0x6653cb22, 23 },
    { 0, 0 },
    { 0x8793cb24, 76 },
    { 0x7fa3cb25, 90 },
    { 0, 0 },
    { 0x9c43cb27, 95 },
    { 0, 0 },
    { 0x5433cb29, 67 },
    { 0xae13cb2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73cb2e, 100 },
    { 0x8f33cb2e, 91 },
    { 0, 0 },
    { 0x903cb31, 23 },
    { 0, 0 },
    { 0xe483cb33, 24 },
    { 0x5b3cb34, 76 },
    { 0xf763cb35, 23 },
    { 0x6953cb36, 33 },
    { 0xe43cb37, 101 },
    { 0x98b3cb37, 90 },
    { 0, 0 },
    { 0x89f3cb3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc3cb3d, 75 },
    { 0, 0 },
    { 0xf2e3cb3f, 90 },
    { 0xa6b3cb40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d3cb44, 91 },
    { 0xb0d3cb45, 85 },
    { 0xfb93cb45, 90 },
    { 0, 0 },
    { 0xa803cb48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e33cb4b, 34 },
    { 0xaf23cb4b, 35 },
    { 0x9a63cb4d, 99 },
    { 0xb873cb4e, 90 },
    { 0x63cb4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3cb53, 76 },
    { 0, 0 },
    { 0x92a3cb55, 33 },
    { 0, 0 },
    { 0xf8e3cb57, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf3cb5a, 90 },
    { 0x7a23cb5a, 43 },
    { 0xd8a3cb5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f3cb5f, 34 },
    { 0x4e83cb5f, 83 },
    { 0x3a63cb61, 33 },
    { 0, 0 },
    { 0xd4f3cb63, 33 },
    { 0x5dd3cb64, 66 },
    { 0, 0 },
    { 0xfeb3cb66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093cb69, 61 },
    { 0, 0 },
    { 0x84a3cb6b, 88 },
    { 0xb4a3cb6c, 61 },
    { 0xf1f3cb6d, 61 },
    { 0x2d13cb6d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8353cb76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a23cb7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c53cb82, 90 },
    { 0x6873cb83, 90 },
    { 0x7e43cb83, 24 },
    { 0, 0 },
    { 0x10d3cb86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f3cb89, 61 },
    { 0xd813cb8a, 34 },
    { 0x8223cb8b, 84 },
    { 0x4333cb8c, 99 },
    { 0xe133cb8d, 99 },
    { 0xab73cb8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3cb93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a3cb97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc43cb9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba53cb9e, 17 },
    { 0xa883cb9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7993cba3, 90 },
    { 0x9463cba3, 95 },
    { 0xc943cba5, 76 },
    { 0, 0 },
    { 0xa2c3cba7, 68 },
    { 0, 0 },
    { 0x78a3cba9, 95 },
    { 0x6623cbaa, 34 },
    { 0xbf93cbab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac43cbae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6553cbb1, 87 },
    { 0xc7b3cbb1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3783cbb5, 33 },
    { 0, 0 },
    { 0x5b83cbb7, 95 },
    { 0, 0 },
    { 0x74e3cbb9, 87 },
    { 0, 0 },
    { 0x4063cbbb, 81 },
    { 0x6123cbbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b3cbc0, 66 },
    { 0, 0 },
    { 0x5b33cbc2, 90 },
    { 0xabc3cbc3, 90 },
    { 0xc8c3cbc3, 61 },
    { 0x7d53cbc3, 61 },
    { 0x7563cbc6, 52 },
    { 0xe3a3cbc7, 23 },
    { 0xcaf3cbc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e3cbcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2183cbd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3cbd7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb73cbdd, 61 },
    { 0xdf53cbde, 90 },
    { 0x43b3cbdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a83cbe3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec3cbe6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533cbe9, 61 },
    { 0xd8f3cbea, 64 },
    { 0xfc93cbea, 23 },
    { 0xdc83cbec, 33 },
    { 0x66f3cbea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b3cbf1, 61 },
    { 0x51a3cbf2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x963cbf5, 90 },
    { 0x6953cbf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da3cbfb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe713cbfe, 90 },
    { 0, 0 },
    { 0x3443cc00, 100 },
    { 0xa093cc01, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be3cc0c, 23 },
    { 0x6fd3cc0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb03cc1f, 95 },
    { 0x8eb3cc20, 60 },
    { 0x7c43cc1f, 17 },
    { 0xa683cc21, 33 },
    { 0, 0 },
    { 0xf2f3cc24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc3cc29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc913cc2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e3cc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d33cc37, 76 },
    { 0x9333cc37, 61 },
    { 0xac53cc39, 95 },
    { 0, 0 },
    { 0x46a3cc3b, 95 },
    { 0, 0 },
    { 0x3153cc3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc513cc42, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc223cc48, 23 },
    { 0x34e3cc49, 33 },
    { 0xe523cc49, 23 },
    { 0, 0 },
    { 0xc33cc4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5903cc51, 98 },
    { 0x2953cc52, 17 },
    { 0x2083cc53, 90 },
    { 0x4363cc53, 95 },
    { 0xb8c3cc55, 90 },
    { 0x5453cc56, 68 },
    { 0xce03cc51, 99 },
    { 0x2713cc58, 93 },
    { 0xa4e3cc56, 84 },
    { 0, 0 },
    { 0x6723cc5b, 99 },
    { 0x8de3cc5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf93cc5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe083cc63, 88 },
    { 0x4493cc64, 78 },
    { 0xc883cc65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b93cc6a, 90 },
    { 0xbb03cc6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa533cc72, 56 },
    { 0xc993cc73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3cc76, 88 },
    { 0x54e3cc77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd333cc7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec3cc8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03cc93, 33 },
    { 0x58b3cc93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6203cc97, 9 },
    { 0x26d3cc98, 98 },
    { 0xeb63cc97, 83 },
    { 0xb4b3cc99, 95 },
    { 0x1f3cc98, 55 },
    { 0x753cc9c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f3cc9f, 76 },
    { 0xe5a3cca0, 33 },
    { 0x2f33cc9f, 33 },
    { 0xf0a3cca0, 66 },
    { 0x5993cca3, 95 },
    { 0x9823cca4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3ccae, 61 },
    { 0, 0 },
    { 0x1bb3ccb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d93ccb5, 84 },
    { 0xbec3ccb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb3ccbc, 33 },
    { 0, 0 },
    { 0x44a3ccbe, 65 },
    { 0, 0 },
    { 0xce93ccc0, 90 },
    { 0x9c43ccc1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d93ccc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a3ccc9, 90 },
    { 0xb943ccca, 61 },
    { 0x5763cccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4423cccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c3ccd5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe503ccd9, 77 },
    { 0x5f3ccda, 23 },
    { 0x3253ccda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf63cce3, 55 },
    { 0xf113cce4, 43 },
    { 0xed93cce5, 46 },
    { 0xf6a3cce3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2933cce9, 61 },
    { 0, 0 },
    { 0x7d53cceb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb833ccee, 33 },
    { 0, 0 },
    { 0x7773ccf0, 22 },
    { 0x1083ccf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6553ccf7, 23 },
    { 0x1ec3ccf8, 66 },
    { 0, 0 },
    { 0xdb43ccfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3ccfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d3cd00, 90 },
    { 0, 0 },
    { 0xa753cd02, 90 },
    { 0xf693cd03, 33 },
    { 0, 0 },
    { 0x33b3cd05, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3cd0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f83cd11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb73cd14, 87 },
    { 0xd4e3cd15, 23 },
    { 0, 0 },
    { 0xbeb3cd17, 33 },
    { 0x40e3cd18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93cd1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb83cd21, 47 },
    { 0x2c23cd21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7363cd25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7003cd28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6103cd2c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6843cd2f, 33 },
    { 0, 0 },
    { 0x4bc3cd31, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf3cd34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca3cd37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf13cd3b, 90 },
    { 0x90c3cd3b, 90 },
    { 0x28c3cd3d, 76 },
    { 0x7d3cd3e, 66 },
    { 0x1a73cd3b, 33 },
    { 0x2f03cd40, 33 },
    { 0, 0 },
    { 0x61f3cd42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e43cd48, 61 },
    { 0x9b43cd48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba3cd4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3cd55, 23 },
    { 0, 0 },
    { 0x67d3cd57, 34 },
    { 0x73cd57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd973cd5d, 33 },
    { 0x9aa3cd5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd623cd61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43cd67, 90 },
    { 0xb4d3cd67, 34 },
    { 0xb823cd69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf733cd6d, 98 },
    { 0x8c53cd6d, 61 },
    { 0x3ca3cd6f, 76 },
    { 0x3353cd70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe643cd74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb253cd79, 90 },
    { 0x8bb3cd7a, 98 },
    { 0xd543cd79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3cd7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc273cd82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a03cd85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd703cd8b, 23 },
    { 0, 0 },
    { 0x3fb3cd8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe333cd90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7753cd99, 61 },
    { 0, 0 },
    { 0xac63cd9b, 66 },
    { 0, 0 },
    { 0x3e73cd9d, 88 },
    { 0, 0 },
    { 0x11a3cd9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5043cda2, 24 },
    { 0, 0 },
    { 0xeca3cda4, 90 },
    { 0x713cda5, 55 },
    { 0xe6a3cda5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3cdab, 95 },
    { 0, 0 },
    { 0x5493cdad, 97 },
    { 0, 0 },
    { 0xb123cdaf, 61 },
    { 0, 0 },
    { 0xbaf3cdb1, 90 },
    { 0, 0 },
    { 0xb0c3cdb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa483cdb6, 23 },
    { 0x6c93cdb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea63cdbf, 23 },
    { 0xf2a3cdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca3cdc4, 95 },
    { 0x3943cdc5, 91 },
    { 0x6ee3cdc4, 87 },
    { 0, 0 },
    { 0xccd3cdc8, 33 },
    { 0x2763cdc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c3cdce, 78 },
    { 0xc153cdcf, 90 },
    { 0xd3d3cdd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb3cdd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c43cdd9, 33 },
    { 0x1593cdda, 98 },
    { 0xbf43cddb, 95 },
    { 0xa2a3cddb, 61 },
    { 0, 0 },
    { 0xab73cdde, 61 },
    { 0xe143cddf, 65 },
    { 0, 0 },
    { 0x3ca3cde1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c3cde7, 61 },
    { 0x7713cde8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf253cdeb, 33 },
    { 0xe583cdec, 90 },
    { 0xede3cdeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf23cdf1, 99 },
    { 0xeb03cdf2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e03cdf5, 95 },
    { 0xe453cdf6, 49 },
    { 0, 0 },
    { 0x8453cdf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d03cdfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5413ce00, 80 },
    { 0x283ce01, 33 },
    { 0, 0 },
    { 0x1d53ce03, 61 },
    { 0, 0 },
    { 0x9433ce05, 61 },
    { 0xfd53ce06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603ce0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd223ce16, 34 },
    { 0, 0 },
    { 0x38b3ce18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed33ce1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6223ce22, 52 },
    { 0xef23ce22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e63ce26, 66 },
    { 0xfdb3ce27, 33 },
    { 0xe793ce27, 85 },
    { 0x73d3ce29, 38 },
    { 0, 0 },
    { 0x20c3ce2b, 98 },
    { 0x8be3ce2c, 98 },
    { 0, 0 },
    { 0xee83ce2e, 95 },
    { 0xa8c3ce2e, 95 },
    { 0, 0 },
    { 0x7a3ce31, 90 },
    { 0xf083ce31, 68 },
    { 0x83d3ce33, 34 },
    { 0, 0 },
    { 0xfe43ce35, 23 },
    { 0xa933ce35, 84 },
    { 0xe613ce36, 90 },
    { 0x2763ce38, 98 },
    { 0x1383ce39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383ce3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e93ce3f, 55 },
    { 0xa123ce40, 67 },
    { 0x8443ce41, 23 },
    { 0x9e63ce40, 95 },
    { 0xe513ce3f, 23 },
    { 0x4803ce3f, 61 },
    { 0, 0 },
    { 0x1f53ce46, 61 },
    { 0x37e3ce47, 61 },
    { 0x5283ce48, 67 },
    { 0x8163ce46, 95 },
    { 0, 0 },
    { 0xfac3ce4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd73ce51, 88 },
    { 0xe283ce52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd993ce5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213ce60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df3ce63, 61 },
    { 0, 0 },
    { 0x7af3ce65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f43ce69, 84 },
    { 0xf8e3ce6a, 76 },
    { 0, 0 },
    { 0xb8b3ce6c, 33 },
    { 0xe453ce6d, 78 },
    { 0xecb3ce6d, 46 },
    { 0xd893ce6f, 88 },
    { 0x4a33ce6e, 61 },
    { 0xe443ce6f, 33 },
    { 0x5353ce72, 90 },
    { 0x6fd3ce73, 55 },
    { 0xbdc3ce74, 55 },
    { 0, 0 },
    { 0xcc3ce76, 78 },
    { 0xd553ce76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa3ce7b, 76 },
    { 0xb873ce7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993ce85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb363ce8c, 33 },
    { 0xd613ce8d, 100 },
    { 0xb683ce8e, 55 },
    { 0, 0 },
    { 0xf13ce90, 33 },
    { 0, 0 },
    { 0xbc63ce92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a3ce9a, 34 },
    { 0x5dd3ce9b, 33 },
    { 0xa393ce9c, 81 },
    { 0x5683ce9b, 23 },
    { 0x2503ce9e, 61 },
    { 0xf843ce9f, 84 },
    { 0xa613ce9f, 90 },
    { 0xf0b3cea1, 66 },
    { 0xe803cea0, 96 },
    { 0, 0 },
    { 0x1e43cea4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5783ceab, 98 },
    { 0, 0 },
    { 0xb7c3cead, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a73ceb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce23ceb5, 93 },
    { 0x9d43ceb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1153ceb9, 95 },
    { 0x9e53ceb9, 55 },
    { 0xfd73cebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4853cebe, 85 },
    { 0xbc03cebf, 99 },
    { 0x5363cebf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7813cec3, 34 },
    { 0, 0 },
    { 0xf7f3cec5, 98 },
    { 0, 0 },
    { 0x6c3cec7, 23 },
    { 0x8d33cec8, 43 },
    { 0x4a43cec8, 23 },
    { 0xe643ceca, 84 },
    { 0x8143cec8, 55 },
    { 0, 0 },
    { 0xbd03cecd, 61 },
    { 0xf083cecd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf23ced4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f3ced7, 63 },
    { 0x3473ced8, 98 },
    { 0, 0 },
    { 0x4633ceda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa063cee0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8663cee3, 46 },
    { 0x19c3cee3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a93cee7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7763ceec, 34 },
    { 0, 0 },
    { 0x5213ceee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813cef5, 61 },
    { 0, 0 },
    { 0xcab3cef7, 55 },
    { 0xc8f3cef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab3cefb, 95 },
    { 0x65f3cefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153ceff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a83cf03, 77 },
    { 0, 0 },
    { 0x7b13cf05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd3cf09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc3cf0c, 33 },
    { 0x31c3cf0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9393cf12, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf423cf18, 95 },
    { 0, 0 },
    { 0x93cf1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd3cf21, 61 },
    { 0xc483cf21, 90 },
    { 0x68a3cf22, 88 },
    { 0x66b3cf24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe623cf27, 61 },
    { 0xcb23cf27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e3cf2b, 38 },
    { 0x38a3cf2c, 66 },
    { 0xb0c3cf2c, 33 },
    { 0xa333cf2e, 76 },
    { 0, 0 },
    { 0xf8a3cf30, 91 },
    { 0x2e3cf31, 66 },
    { 0x77c3cf32, 46 },
    { 0xbc53cf31, 91 },
    { 0x8343cf33, 94 },
    { 0xa8f3cf32, 33 },
    { 0x7043cf36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b3cf39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c73cf3d, 90 },
    { 0, 0 },
    { 0x6823cf3f, 91 },
    { 0x42c3cf40, 90 },
    { 0x9a13cf3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3cf46, 90 },
    { 0, 0 },
    { 0x6c53cf48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d3cf4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde73cf52, 55 },
    { 0, 0 },
    { 0x4a33cf54, 95 },
    { 0x9713cf55, 98 },
    { 0xfa23cf56, 76 },
    { 0xab3cf56, 87 },
    { 0, 0 },
    { 0x8633cf59, 51 },
    { 0, 0 },
    { 0xafb3cf5b, 52 },
    { 0x57e3cf5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c3cf61, 46 },
    { 0x6373cf61, 36 },
    { 0x3803cf63, 99 },
    { 0xeac3cf63, 93 },
    { 0xed93cf65, 66 },
    { 0xe0b3cf65, 80 },
    { 0, 0 },
    { 0xbae3cf68, 33 },
    { 0, 0 },
    { 0x8ad3cf6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f83cf74, 90 },
    { 0x5e73cf75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d3cf78, 40 },
    { 0xc063cf79, 33 },
    { 0xfbe3cf79, 33 },
    { 0xa373cf78, 33 },
    { 0, 0 },
    { 0xd313cf7d, 68 },
    { 0x493cf7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5793cf81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4893cf86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913cf8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe123cf91, 69 },
    { 0, 0 },
    { 0x9a33cf93, 99 },
    { 0xd283cf93, 90 },
    { 0xd293cf95, 33 },
    { 0, 0 },
    { 0x9ef3cf97, 23 },
    { 0xf703cf98, 95 },
    { 0, 0 },
    { 0xf143cf9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3cf9d, 90 },
    { 0xe1e3cf9e, 93 },
    { 0x83f3cf9f, 43 },
    { 0x2133cf9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7813cfa3, 90 },
    { 0x7a03cfa3, 95 },
    { 0x8ba3cfa4, 90 },
    { 0xa523cfa6, 34 },
    { 0xffb3cfa6, 76 },
    { 0x6733cfa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df3cfad, 95 },
    { 0x77f3cfae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b3cfb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6833cfb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa173cfbe, 88 },
    { 0, 0 },
    { 0xc893cfc0, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba43cfc4, 95 },
    { 0xac73cfc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903cfc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff3cfcb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d3cfcf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f3cfd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a3cfda, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3cfdf, 33 },
    { 0x72d3cfdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b43cfe7, 66 },
    { 0x3973cfe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9843cfeb, 51 },
    { 0x1033cfeb, 98 },
    { 0x93cfed, 83 },
    { 0x56e3cfed, 66 },
    { 0xca13cfef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673cff2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4043cff6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e3cffc, 43 },
    { 0xbd83cffc, 33 },
    { 0x35a3cffc, 100 },
    { 0x6ad3cfff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5213d003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b83d007, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da3d00b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7013d010, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393d013, 34 },
    { 0, 0 },
    { 0x9693d015, 77 },
    { 0, 0 },
    { 0xe483d017, 61 },
    { 0, 0 },
    { 0x1243d019, 88 },
    { 0xc443d01a, 45 },
    { 0, 0 },
    { 0xfb63d01c, 43 },
    { 0xe013d01c, 77 },
    { 0xca13d01e, 61 },
    { 0, 0 },
    { 0xb273d020, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee3d024, 33 },
    { 0, 0 },
    { 0xf203d026, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb3d029, 98 },
    { 0x3363d02a, 40 },
    { 0, 0 },
    { 0xfde3d02c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba3d030, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd23d034, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9223d037, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b63d03a, 76 },
    { 0, 0 },
    { 0xbfe3d03c, 55 },
    { 0xa3f3d03d, 93 },
    { 0x6b23d03d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd543d041, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f3d045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd53d04b, 90 },
    { 0, 0 },
    { 0x64f3d04d, 34 },
    { 0xba53d04e, 98 },
    { 0x1133d04f, 55 },
    { 0xcb63d04f, 93 },
    { 0, 0 },
    { 0xe43d052, 23 },
    { 0x2503d053, 87 },
    { 0x6aa3d054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d3d058, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d05e, 77 },
    { 0xb3a3d05f, 98 },
    { 0, 0 },
    { 0x7d13d061, 90 },
    { 0xd8d3d062, 34 },
    { 0xcc43d062, 46 },
    { 0, 0 },
    { 0x1d03d065, 34 },
    { 0x14b3d065, 23 },
    { 0x3ff3d067, 90 },
    { 0x9b3d068, 55 },
    { 0x2d33d069, 23 },
    { 0, 0 },
    { 0x4753d06b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2573d06f, 98 },
    { 0x5773d06f, 43 },
    { 0xca13d06f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8273d07b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe433d082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4663d08a, 33 },
    { 0x2473d08b, 95 },
    { 0x2c73d08c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc63d08f, 23 },
    { 0x6bc3d090, 33 },
    { 0xe883d091, 95 },
    { 0x1a23d092, 90 },
    { 0xcb13d093, 61 },
    { 0x9e73d090, 90 },
    { 0, 0 },
    { 0xfa13d096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf3d099, 23 },
    { 0x8e53d09a, 65 },
    { 0xa0a3d099, 67 },
    { 0, 0 },
    { 0xf8a3d09d, 33 },
    { 0, 0 },
    { 0xab03d09f, 75 },
    { 0x1af3d0a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb3d0a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac43d0ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f3d0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3573d0b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8853d0bf, 90 },
    { 0xe193d0c0, 88 },
    { 0x803d0bf, 77 },
    { 0x8803d0c0, 33 },
    { 0x7a83d0bf, 23 },
    { 0x37f3d0c2, 98 },
    { 0x8843d0c2, 85 },
    { 0x45f3d0c6, 61 },
    { 0, 0 },
    { 0x8333d0c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e83d0cb, 95 },
    { 0, 0 },
    { 0xfb3d0cd, 43 },
    { 0, 0 },
    { 0x7263d0cf, 52 },
    { 0x6b23d0d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a83d0d3, 90 },
    { 0xfb63d0d3, 33 },
    { 0x44a3d0d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a3d0d8, 33 },
    { 0xc513d0d8, 23 },
    { 0, 0 },
    { 0x34c3d0db, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb93d0de, 78 },
    { 0, 0 },
    { 0x3ba3d0e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce23d0e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4183d0e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc3d0ee, 88 },
    { 0, 0 },
    { 0x3b33d0f0, 33 },
    { 0x2633d0f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea3d0f4, 43 },
    { 0x66d3d0f5, 61 },
    { 0x9fd3d0f5, 90 },
    { 0x98d3d0f6, 47 },
    { 0x36e3d0f6, 91 },
    { 0x843d0f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd323d0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf003d100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e13d104, 46 },
    { 0x5393d104, 91 },
    { 0, 0 },
    { 0x93a3d107, 85 },
    { 0xf693d107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b3d10c, 100 },
    { 0x4043d10d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e93d111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c3d114, 88 },
    { 0, 0 },
    { 0x2733d116, 55 },
    { 0x7193d117, 55 },
    { 0xa753d118, 23 },
    { 0xf863d118, 71 },
    { 0xdbe3d117, 33 },
    { 0xa5c3d116, 90 },
    { 0, 0 },
    { 0xeb83d11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b3d121, 78 },
    { 0x8ec3d122, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd873d125, 66 },
    { 0xe863d126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf793d12c, 61 },
    { 0xc2e3d12d, 93 },
    { 0x8263d12d, 61 },
    { 0, 0 },
    { 0x7de3d130, 34 },
    { 0xb5d3d131, 99 },
    { 0, 0 },
    { 0x4f3d133, 76 },
    { 0, 0 },
    { 0xb783d135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3d138, 95 },
    { 0xcd43d139, 34 },
    { 0xa443d13a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf3d142, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a03d145, 95 },
    { 0, 0 },
    { 0x9493d147, 98 },
    { 0xf023d147, 33 },
    { 0xdcc3d149, 23 },
    { 0x5be3d148, 100 },
    { 0, 0 },
    { 0x9183d14c, 23 },
    { 0, 0 },
    { 0xbf13d14e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3993d151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec3d154, 98 },
    { 0, 0 },
    { 0x8763d156, 90 },
    { 0, 0 },
    { 0x55b3d158, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8163d15f, 98 },
    { 0x2323d15f, 98 },
    { 0x4f83d15f, 33 },
    { 0xda23d15f, 98 },
    { 0x1b93d163, 55 },
    { 0x32f3d164, 66 },
    { 0, 0 },
    { 0x7db3d166, 76 },
    { 0x91e3d167, 95 },
    { 0, 0 },
    { 0xe2f3d169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9513d16f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3d173, 95 },
    { 0, 0 },
    { 0x9563d175, 81 },
    { 0x7413d175, 100 },
    { 0x7f33d177, 93 },
    { 0x3843d175, 90 },
    { 0x8423d179, 95 },
    { 0, 0 },
    { 0xcf73d17b, 23 },
    { 0xc933d17c, 23 },
    { 0x32c3d17c, 98 },
    { 0x9b3d17d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e3d182, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb353d186, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3d190, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af3d194, 61 },
    { 0, 0 },
    { 0xf953d196, 35 },
    { 0x8e83d197, 95 },
    { 0, 0 },
    { 0xccd3d199, 87 },
    { 0xde03d19a, 76 },
    { 0x6d33d199, 43 },
    { 0x8c13d19c, 33 },
    { 0x29f3d19c, 88 },
    { 0xa4e3d19d, 95 },
    { 0x1883d19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b3d1a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9603d1ab, 90 },
    { 0xf513d1ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23d1b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa63d1bf, 98 },
    { 0xcd13d1c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9503d1c3, 77 },
    { 0, 0 },
    { 0xfa63d1c5, 87 },
    { 0xd783d1c6, 100 },
    { 0, 0 },
    { 0x3d03d1c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5393d1ce, 33 },
    { 0xda23d1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f3d1d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43d1d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a3d1db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad3d1de, 90 },
    { 0xcec3d1df, 33 },
    { 0x9943d1e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6613d1e3, 93 },
    { 0xbb93d1e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb3d1e7, 52 },
    { 0x7363d1e8, 76 },
    { 0, 0 },
    { 0xa453d1ea, 95 },
    { 0, 0 },
    { 0x8263d1ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e3d1f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef33d1f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd253d1f7, 86 },
    { 0x4643d1f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc3d1fd, 23 },
    { 0, 0 },
    { 0xc283d1ff, 34 },
    { 0, 0 },
    { 0x2e83d201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903d20b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd33d20e, 55 },
    { 0xfad3d20f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee3d212, 91 },
    { 0x6523d212, 98 },
    { 0, 0 },
    { 0x5a73d215, 100 },
    { 0xf2a3d216, 61 },
    { 0x9ba3d216, 67 },
    { 0xb803d218, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3d220, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3d225, 83 },
    { 0, 0 },
    { 0x1813d227, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133d22f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53d232, 66 },
    { 0, 0 },
    { 0x4f93d234, 34 },
    { 0, 0 },
    { 0xc603d236, 33 },
    { 0, 0 },
    { 0xbf23d238, 68 },
    { 0, 0 },
    { 0x65d3d23a, 33 },
    { 0, 0 },
    { 0xd203d23c, 98 },
    { 0xdf03d23d, 76 },
    { 0x7333d23e, 85 },
    { 0x3f13d23e, 95 },
    { 0x76c3d240, 95 },
    { 0, 0 },
    { 0xc9e3d242, 23 },
    { 0, 0 },
    { 0x5643d244, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf33d249, 91 },
    { 0x2cf3d24a, 23 },
    { 0xf423d24b, 97 },
    { 0x8f33d24c, 46 },
    { 0xd8c3d24d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf3d250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd713d254, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53d257, 95 },
    { 0x21b3d258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7193d25b, 98 },
    { 0xf8c3d25b, 23 },
    { 0xba3d25b, 43 },
    { 0x8923d25e, 78 },
    { 0xb6b3d25f, 43 },
    { 0xb443d260, 61 },
    { 0x1cd3d261, 43 },
    { 0x9303d25f, 95 },
    { 0xf123d263, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3d266, 87 },
    { 0xfbe3d267, 90 },
    { 0x91f3d268, 90 },
    { 0x47b3d268, 98 },
    { 0x5843d26a, 95 },
    { 0x95c3d268, 90 },
    { 0x2f63d26b, 90 },
    { 0x7033d26d, 43 },
    { 0xa993d26e, 23 },
    { 0x7f33d26b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483d278, 22 },
    { 0x25f3d278, 66 },
    { 0xa793d278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e33d27f, 23 },
    { 0x36c3d280, 23 },
    { 0x4d73d280, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43d287, 34 },
    { 0x8e23d288, 61 },
    { 0x2ce3d289, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a3d291, 95 },
    { 0, 0 },
    { 0xeff3d293, 33 },
    { 0, 0 },
    { 0x7ec3d295, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f3d299, 76 },
    { 0x2b93d299, 90 },
    { 0xf1a3d29b, 20 },
    { 0xb853d29c, 23 },
    { 0xd13d299, 33 },
    { 0xa4f3d29e, 90 },
    { 0xb23d29f, 90 },
    { 0x90c3d29f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f33d2a4, 76 },
    { 0x4143d2a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9523d2ab, 34 },
    { 0x3d2ac, 34 },
    { 0xa243d2ab, 95 },
    { 0x74d3d2ae, 99 },
    { 0xcb73d2af, 23 },
    { 0x5ea3d2b0, 46 },
    { 0x3a53d2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xada3d2b4, 46 },
    { 0x9a33d2b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3d2bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb03d2c1, 55 },
    { 0, 0 },
    { 0x9603d2c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3063d2d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d2d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc263d2da, 23 },
    { 0, 0 },
    { 0x62e3d2dc, 95 },
    { 0x78c3d2dd, 77 },
    { 0, 0 },
    { 0xa263d2df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f3d2e4, 99 },
    { 0, 0 },
    { 0x71d3d2e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6903d2ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6743d2f0, 33 },
    { 0x3203d2f1, 33 },
    { 0, 0 },
    { 0x1c83d2f3, 90 },
    { 0x1633d2f4, 88 },
    { 0x3c83d2f4, 23 },
    { 0x8dc3d2f6, 95 },
    { 0xa153d2f6, 23 },
    { 0xad03d2f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef3d2fd, 66 },
    { 0xa393d2fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e3d301, 43 },
    { 0, 0 },
    { 0x2f93d303, 89 },
    { 0, 0 },
    { 0xf6b3d305, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a83d30d, 90 },
    { 0xe8b3d30e, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9513d312, 23 },
    { 0x4813d313, 47 },
    { 0x4533d313, 95 },
    { 0, 0 },
    { 0x41a3d316, 23 },
    { 0, 0 },
    { 0xb133d318, 95 },
    { 0xced3d318, 33 },
    { 0xf593d31a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3d321, 33 },
    { 0xa5d3d322, 90 },
    { 0x9ef3d323, 61 },
    { 0, 0 },
    { 0xb333d325, 75 },
    { 0x6d23d326, 9 },
    { 0x7e63d325, 61 },
    { 0xae43d328, 87 },
    { 0x5e3d327, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a43d334, 99 },
    { 0, 0 },
    { 0x1513d336, 99 },
    { 0xc83d336, 98 },
    { 0x6f3d338, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e3d33e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1693d343, 61 },
    { 0xa763d343, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3d349, 66 },
    { 0, 0 },
    { 0xbfa3d34b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c3d350, 90 },
    { 0xb83d350, 95 },
    { 0x9ed3d352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9723d356, 98 },
    { 0x1fb3d357, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e3d35c, 91 },
    { 0xfb13d35c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d3d363, 34 },
    { 0, 0 },
    { 0x8983d365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c23d36a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e3d36e, 55 },
    { 0x1cb3d36f, 61 },
    { 0x70b3d36f, 33 },
    { 0x99c3d36f, 90 },
    { 0x93c3d372, 100 },
    { 0x3a43d372, 90 },
    { 0, 0 },
    { 0x88b3d375, 95 },
    { 0x2bf3d376, 55 },
    { 0, 0 },
    { 0xbef3d378, 90 },
    { 0, 0 },
    { 0x7143d37a, 90 },
    { 0xdbe3d37b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e3d380, 98 },
    { 0, 0 },
    { 0xaf13d382, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb053d386, 33 },
    { 0xe683d387, 55 },
    { 0, 0 },
    { 0x243d389, 85 },
    { 0, 0 },
    { 0xef3d38b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2043d38e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c3d39d, 77 },
    { 0xe573d39d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c3d3a5, 33 },
    { 0x4253d3a6, 33 },
    { 0x8fb3d3a6, 43 },
    { 0x7293d3a8, 101 },
    { 0xf2e3d3a6, 75 },
    { 0, 0 },
    { 0xe443d3ab, 22 },
    { 0, 0 },
    { 0x2463d3ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8683d3b1, 77 },
    { 0, 0 },
    { 0xbb03d3b3, 90 },
    { 0x33a3d3b4, 34 },
    { 0, 0 },
    { 0x7d73d3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d63d3b9, 78 },
    { 0, 0 },
    { 0x1e43d3bb, 90 },
    { 0xbda3d3bc, 55 },
    { 0xfc03d3bd, 61 },
    { 0xe453d3bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4033d3c3, 66 },
    { 0, 0 },
    { 0x9953d3c5, 23 },
    { 0x8dd3d3c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c53d3cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa3d3ce, 55 },
    { 0x2583d3ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c13d3d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf3d3de, 90 },
    { 0, 0 },
    { 0x9b13d3e0, 95 },
    { 0, 0 },
    { 0xbcf3d3e2, 81 },
    { 0x8fa3d3e3, 53 },
    { 0x70b3d3e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc3d3e7, 61 },
    { 0xf693d3e7, 33 },
    { 0, 0 },
    { 0x7f3d3ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa983d3ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af3d3f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe13d3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc623d3fb, 91 },
    { 0x2423d3fb, 90 },
    { 0, 0 },
    { 0x3013d3fe, 76 },
    { 0x3e13d3ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f73d402, 90 },
    { 0x2a63d402, 17 },
    { 0xc4b3d402, 93 },
    { 0xcec3d402, 98 },
    { 0xff03d406, 75 },
    { 0xa7c3d406, 46 },
    { 0x2163d408, 46 },
    { 0x6c33d408, 90 },
    { 0xf4a3d406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f3d40e, 98 },
    { 0x9923d40e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf303d413, 90 },
    { 0x3793d414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43d417, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e3d41a, 95 },
    { 0x5d03d41b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd663d41f, 81 },
    { 0xba43d41f, 99 },
    { 0xdc03d421, 67 },
    { 0xfad3d41f, 87 },
    { 0xd9b3d422, 98 },
    { 0xc403d422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc3d427, 34 },
    { 0x8ca3d428, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d3d42d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b3d433, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8813d437, 91 },
    { 0x3e33d438, 23 },
    { 0, 0 },
    { 0x5353d43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e73d43d, 66 },
    { 0x8723d43e, 33 },
    { 0xcd43d43f, 61 },
    { 0xba23d440, 33 },
    { 0xb143d441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc053d446, 52 },
    { 0xf743d446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d3d454, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d3d457, 98 },
    { 0, 0 },
    { 0xe063d459, 90 },
    { 0x7c73d45a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5463d45d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3d467, 43 },
    { 0, 0 },
    { 0x86d3d469, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd633d46c, 90 },
    { 0xb643d46d, 40 },
    { 0xa113d46e, 55 },
    { 0xd003d46f, 77 },
    { 0x4623d470, 95 },
    { 0xfd3d470, 76 },
    { 0x61d3d470, 33 },
    { 0, 0 },
    { 0xe193d474, 90 },
    { 0xc013d475, 98 },
    { 0xe7b3d476, 33 },
    { 0xfd13d477, 77 },
    { 0x1e83d475, 61 },
    { 0xbe33d477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf433d47d, 17 },
    { 0x1d43d47d, 95 },
    { 0x1123d47f, 66 },
    { 0, 0 },
    { 0xdf53d481, 52 },
    { 0, 0 },
    { 0xd6c3d483, 33 },
    { 0x8123d483, 33 },
    { 0x5773d485, 78 },
    { 0x4973d486, 52 },
    { 0x1463d487, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xed13d48a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c3d48e, 87 },
    { 0xe943d48f, 76 },
    { 0xf063d48e, 90 },
    { 0x2613d490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe523d496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d13d49a, 40 },
    { 0x5d83d49b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace3d4a1, 55 },
    { 0, 0 },
    { 0x8963d4a3, 76 },
    { 0xc633d4a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed93d4a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d3d4ab, 93 },
    { 0x3923d4ac, 90 },
    { 0x1f83d4ab, 85 },
    { 0xe053d4ad, 61 },
    { 0x8d53d4ae, 61 },
    { 0xc3d3d4ab, 93 },
    { 0x8ab3d4b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb3d4b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc973d4b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43d4bf, 98 },
    { 0xd273d4c0, 66 },
    { 0x8cb3d4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce3d4c6, 98 },
    { 0x9e53d4c6, 95 },
    { 0x43d4c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8893d4cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b13d4d1, 33 },
    { 0xa9a3d4d2, 61 },
    { 0, 0 },
    { 0xa333d4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe883d4d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe33d4de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7843d4ec, 91 },
    { 0xc933d4ed, 55 },
    { 0x5ec3d4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7723d4f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d3d4f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3153d4fa, 95 },
    { 0, 0 },
    { 0x4773d4fc, 61 },
    { 0x6ec3d4fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a53d500, 81 },
    { 0, 0 },
    { 0x4e93d502, 61 },
    { 0xa5b3d503, 33 },
    { 0x4263d504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053d508, 23 },
    { 0x5b13d508, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xced3d50c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8453d50f, 98 },
    { 0x88e3d510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493d513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb23d51e, 90 },
    { 0x3f3d51f, 90 },
    { 0x5c83d51f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9053d525, 98 },
    { 0x5a23d525, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c53d529, 91 },
    { 0, 0 },
    { 0x45a3d52b, 95 },
    { 0x77b3d52c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa893d530, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d23d539, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2773d53c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb363d53f, 34 },
    { 0, 0 },
    { 0xca43d541, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4593d544, 33 },
    { 0xeaf3d545, 90 },
    { 0, 0 },
    { 0x1c83d547, 61 },
    { 0, 0 },
    { 0x6763d549, 56 },
    { 0xd1e3d54a, 61 },
    { 0x4d73d549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4773d550, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3303d554, 78 },
    { 0xfaf3d555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c53d561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c3d564, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc73d568, 98 },
    { 0, 0 },
    { 0x5f33d56a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e63d56e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb3d571, 23 },
    { 0xc993d572, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd903d578, 76 },
    { 0, 0 },
    { 0x67a3d57a, 34 },
    { 0x5763d57b, 98 },
    { 0, 0 },
    { 0x14e3d57d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d3d580, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1253d585, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc493d58c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb3d594, 33 },
    { 0x4ab3d595, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa983d598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad03d59b, 95 },
    { 0, 0 },
    { 0x74a3d59d, 52 },
    { 0x1c03d59d, 33 },
    { 0x6773d59f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d3d5a2, 23 },
    { 0x5f63d5a3, 98 },
    { 0x363d5a4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd23d5a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e43d5ac, 78 },
    { 0, 0 },
    { 0x5be3d5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde63d5b3, 95 },
    { 0xdf83d5b4, 52 },
    { 0x7643d5b3, 33 },
    { 0x6d3d5b6, 61 },
    { 0x3773d5b7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd3d5be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a3d5c6, 76 },
    { 0, 0 },
    { 0xe363d5c8, 87 },
    { 0xe113d5c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc3d5cd, 17 },
    { 0, 0 },
    { 0x9d33d5cf, 33 },
    { 0xe993d5d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d03d5d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf863d5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa803d5db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6323d5df, 67 },
    { 0xc853d5df, 90 },
    { 0x1873d5e0, 95 },
    { 0xf293d5e2, 90 },
    { 0xe873d5e0, 95 },
    { 0xcb83d5e0, 98 },
    { 0xe303d5e5, 33 },
    { 0xc7a3d5e2, 23 },
    { 0xeb73d5e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b3d5eb, 23 },
    { 0, 0 },
    { 0xb8f3d5ed, 90 },
    { 0xc143d5ee, 61 },
    { 0x4fb3d5ee, 23 },
    { 0x5923d5ee, 85 },
    { 0xdcf3d5f1, 42 },
    { 0x69e3d5ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4383d5f5, 34 },
    { 0x73a3d5f6, 98 },
    { 0x9d43d5f5, 98 },
    { 0x5aa3d5f6, 34 },
    { 0x46f3d5f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813d5fe, 81 },
    { 0x1ed3d5ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3d602, 33 },
    { 0xc3f3d603, 33 },
    { 0x5483d604, 84 },
    { 0xcfb3d605, 76 },
    { 0xeb53d606, 90 },
    { 0x97e3d606, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5183d60f, 90 },
    { 0, 0 },
    { 0xac53d611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d43d620, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7693d626, 33 },
    { 0x7e93d626, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc543d62a, 34 },
    { 0x48a3d62a, 96 },
    { 0, 0 },
    { 0x6623d62d, 98 },
    { 0xd8c3d62d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3183d632, 90 },
    { 0xc563d633, 76 },
    { 0x3683d634, 33 },
    { 0x3e63d635, 98 },
    { 0x78e3d633, 98 },
    { 0x5453d637, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c43d63c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c73d63f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b33d646, 87 },
    { 0, 0 },
    { 0x92e3d648, 80 },
    { 0x4133d649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e3d64f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b3d652, 98 },
    { 0, 0 },
    { 0x55c3d654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec3d66b, 90 },
    { 0x5703d66b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f3d670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e3d677, 76 },
    { 0xc373d677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb3d67b, 33 },
    { 0x8b03d67c, 23 },
    { 0x99d3d67d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d3d680, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2873d686, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f3d68c, 100 },
    { 0, 0 },
    { 0x3e73d68e, 52 },
    { 0x983d68f, 95 },
    { 0x26d3d690, 76 },
    { 0x5063d68f, 93 },
    { 0, 0 },
    { 0xdad3d693, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5773d699, 55 },
    { 0, 0 },
    { 0xf3d69b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf3d69e, 95 },
    { 0x27f3d69f, 95 },
    { 0xac33d69f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb843d6a6, 43 },
    { 0x6183d6a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e3d6aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc623d6ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43d6b0, 34 },
    { 0, 0 },
    { 0x75f3d6b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3393d6b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad93d6bf, 90 },
    { 0xb0a3d6c0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb3d6c3, 100 },
    { 0x8433d6c4, 95 },
    { 0xca03d6c5, 95 },
    { 0xb3d3d6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd673d6ca, 43 },
    { 0x2073d6cb, 23 },
    { 0x6f43d6cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073d6d0, 31 },
    { 0, 0 },
    { 0x6603d6d2, 90 },
    { 0, 0 },
    { 0xbbe3d6d4, 61 },
    { 0xbb93d6d4, 46 },
    { 0x6113d6d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7633d6d9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae53d6df, 23 },
    { 0x3293d6e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc3d6eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc3d6ee, 78 },
    { 0x5893d6ef, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea33d6f5, 67 },
    { 0x94b3d6f5, 90 },
    { 0xfa23d6f7, 87 },
    { 0x2553d6f6, 93 },
    { 0x75a3d6f9, 85 },
    { 0x5583d6fa, 88 },
    { 0xc183d6fb, 95 },
    { 0, 0 },
    { 0x4273d6fd, 61 },
    { 0, 0 },
    { 0x54d3d6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813d70a, 76 },
    { 0x23d70b, 55 },
    { 0x1803d70c, 78 },
    { 0x61a3d70c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba93d711, 34 },
    { 0xd603d712, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93d716, 95 },
    { 0xf603d717, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f83d71a, 67 },
    { 0, 0 },
    { 0x71d3d71c, 90 },
    { 0x27a3d71c, 23 },
    { 0xa683d71c, 90 },
    { 0, 0 },
    { 0x5dc3d720, 98 },
    { 0x20d3d721, 43 },
    { 0x54e3d722, 33 },
    { 0x103d723, 46 },
    { 0, 0 },
    { 0xc0e3d725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7773d72a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b23d72d, 23 },
    { 0, 0 },
    { 0x9c93d72f, 33 },
    { 0x56c3d730, 33 },
    { 0x5023d731, 33 },
    { 0xd303d730, 76 },
    { 0xec53d732, 93 },
    { 0x7e83d734, 55 },
    { 0, 0 },
    { 0x8dc3d736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe093d739, 23 },
    { 0x5f3d73a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd463d740, 87 },
    { 0, 0 },
    { 0x6883d742, 61 },
    { 0, 0 },
    { 0x94f3d744, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2233d748, 36 },
    { 0, 0 },
    { 0xc9e3d74a, 33 },
    { 0x6183d74b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad13d74e, 90 },
    { 0, 0 },
    { 0xa4c3d750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23d757, 67 },
    { 0, 0 },
    { 0x3353d759, 76 },
    { 0x993d75a, 98 },
    { 0xefb3d75b, 23 },
    { 0x90b3d75a, 23 },
    { 0, 0 },
    { 0x1513d75e, 33 },
    { 0xfa83d75e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee73d763, 80 },
    { 0, 0 },
    { 0x1873d765, 61 },
    { 0xab73d766, 40 },
    { 0x96a3d767, 98 },
    { 0, 0 },
    { 0x71a3d769, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3d76d, 52 },
    { 0x95b3d76d, 95 },
    { 0, 0 },
    { 0xb793d770, 84 },
    { 0x9573d771, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8123d77d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a3d781, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d33d787, 67 },
    { 0, 0 },
    { 0x8e13d789, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef3d78d, 68 },
    { 0xc673d78d, 95 },
    { 0, 0 },
    { 0x77c3d790, 52 },
    { 0xd123d790, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3d797, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5403d79a, 98 },
    { 0x1af3d79b, 68 },
    { 0x61b3d79b, 33 },
    { 0, 0 },
    { 0xfdb3d79e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d7a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e3d7a5, 33 },
    { 0x20f3d7a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2113d7a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d33d7b9, 65 },
    { 0xc243d7b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc83d7bf, 81 },
    { 0x6973d7c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5543d7c3, 98 },
    { 0xcf13d7c3, 95 },
    { 0, 0 },
    { 0xa913d7c6, 61 },
    { 0, 0 },
    { 0x9c23d7c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc333d7cc, 98 },
    { 0, 0 },
    { 0xdc3d7ce, 33 },
    { 0x64a3d7ce, 46 },
    { 0xf803d7ce, 93 },
    { 0, 0 },
    { 0xb193d7d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca3d7d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae3d7da, 66 },
    { 0, 0 },
    { 0xa0b3d7dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6683d7e0, 84 },
    { 0x2313d7e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d3d7e5, 55 },
    { 0x5d93d7e6, 38 },
    { 0x3143d7e5, 17 },
    { 0x2263d7e8, 61 },
    { 0x68e3d7e9, 90 },
    { 0x1b43d7e9, 46 },
    { 0x9c53d7e7, 87 },
    { 0x5923d7ec, 61 },
    { 0x30e3d7ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e73d7f2, 68 },
    { 0xb4a3d7f2, 33 },
    { 0, 0 },
    { 0xee13d7f5, 95 },
    { 0, 0 },
    { 0x20f3d7f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43d7fa, 90 },
    { 0x7373d7fb, 90 },
    { 0xc573d7fa, 93 },
    { 0x88b3d7fd, 55 },
    { 0x6783d7fb, 53 },
    { 0xbd63d7fc, 78 },
    { 0, 0 },
    { 0xe653d801, 61 },
    { 0x1363d802, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa733d806, 33 },
    { 0, 0 },
    { 0x2ab3d808, 69 },
    { 0xec53d809, 76 },
    { 0, 0 },
    { 0xc233d80b, 95 },
    { 0xa663d80c, 61 },
    { 0xa403d80d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9553d813, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa93d816, 33 },
    { 0x4863d817, 93 },
    { 0xc523d818, 33 },
    { 0xdcc3d819, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd873d820, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b93d823, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c3d827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb313d82d, 61 },
    { 0x8123d82d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3d838, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd743d83b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9573d83e, 66 },
    { 0xb623d83e, 95 },
    { 0xfc53d840, 90 },
    { 0x4fb3d840, 23 },
    { 0xbbf3d842, 33 },
    { 0x6543d841, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb3d854, 67 },
    { 0xd713d855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e53d858, 61 },
    { 0x3363d859, 61 },
    { 0, 0 },
    { 0x3a3d85b, 98 },
    { 0xec93d85b, 33 },
    { 0x9dd3d85c, 38 },
    { 0x5103d85e, 93 },
    { 0xbcf3d85f, 75 },
    { 0x7e03d860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6833d868, 99 },
    { 0, 0 },
    { 0x79a3d86a, 43 },
    { 0xa153d86a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc03d86f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe923d872, 98 },
    { 0x6773d873, 61 },
    { 0x1233d872, 90 },
    { 0xebd3d875, 87 },
    { 0, 0 },
    { 0xbfa3d877, 98 },
    { 0x2d33d878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe483d87d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1953d880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3d884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7263d888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd3d88f, 99 },
    { 0, 0 },
    { 0xf253d891, 23 },
    { 0, 0 },
    { 0xf773d893, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f3d89a, 66 },
    { 0, 0 },
    { 0x6eb3d89c, 33 },
    { 0xdfb3d89c, 23 },
    { 0, 0 },
    { 0xe1b3d89f, 52 },
    { 0, 0 },
    { 0xd283d8a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb3d8a4, 90 },
    { 0x9583d8a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3d8a8, 95 },
    { 0x61a3d8a9, 76 },
    { 0, 0 },
    { 0x7203d8ab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb3d8b1, 91 },
    { 0xf553d8b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe433d8ba, 91 },
    { 0x42d3d8bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c13d8be, 33 },
    { 0x1293d8bf, 61 },
    { 0xdd93d8bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3d8c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d3d8c9, 33 },
    { 0, 0 },
    { 0x8383d8cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e3d8d0, 52 },
    { 0xf5f3d8d0, 88 },
    { 0x2783d8d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a3d8d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7233d8db, 95 },
    { 0, 0 },
    { 0x93b3d8dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe3d8e2, 90 },
    { 0xe6a3d8e3, 23 },
    { 0x30c3d8e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3d8e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f3d8eb, 90 },
    { 0x46a3d8eb, 61 },
    { 0, 0 },
    { 0xc4c3d8ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9953d8f1, 23 },
    { 0x8893d8f1, 99 },
    { 0, 0 },
    { 0xd2c3d8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f3d8f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33d8fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4843d906, 91 },
    { 0x3673d906, 99 },
    { 0x9fb3d907, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf3d90b, 33 },
    { 0x4cf3d90c, 76 },
    { 0x9ad3d90b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e93d910, 34 },
    { 0xc4b3d910, 97 },
    { 0, 0 },
    { 0x9163d913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa3d917, 100 },
    { 0x1fb3d917, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8613d91b, 23 },
    { 0, 0 },
    { 0xe153d91d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7093d921, 36 },
    { 0x6163d921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe03d926, 61 },
    { 0, 0 },
    { 0x77d3d928, 98 },
    { 0x4ee3d929, 61 },
    { 0x1d33d92a, 78 },
    { 0x2ae3d929, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d3d92e, 90 },
    { 0x4763d92f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f3d933, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef3d937, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5803d93a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae3d93e, 81 },
    { 0xb833d93f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5053d943, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e3d94c, 61 },
    { 0x29b3d94d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf563d951, 84 },
    { 0, 0 },
    { 0x2183d953, 8 },
    { 0x7ab3d954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d95b, 95 },
    { 0x20a3d95c, 33 },
    { 0xaea3d95d, 23 },
    { 0xf923d95e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd983d962, 61 },
    { 0xa023d962, 61 },
    { 0xa803d963, 61 },
    { 0xf383d965, 93 },
    { 0xd453d966, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893d96a, 23 },
    { 0xabe3d96b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7523d96e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7893d972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e3d975, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb3d97a, 61 },
    { 0x1493d97b, 90 },
    { 0, 0 },
    { 0x19a3d97d, 46 },
    { 0, 0 },
    { 0xfaa3d97f, 95 },
    { 0xd753d97f, 33 },
    { 0, 0 },
    { 0x9d33d982, 95 },
    { 0x7233d982, 33 },
    { 0xb753d984, 33 },
    { 0, 0 },
    { 0x6a23d986, 33 },
    { 0, 0 },
    { 0x6833d988, 43 },
    { 0, 0 },
    { 0x20a3d98a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6493d98d, 90 },
    { 0x8f13d98e, 61 },
    { 0x6d23d98f, 98 },
    { 0xeae3d990, 90 },
    { 0xcfd3d991, 33 },
    { 0x6ca3d992, 33 },
    { 0x7c33d993, 55 },
    { 0x55d3d994, 81 },
    { 0x2f23d995, 95 },
    { 0, 0 },
    { 0xfb83d997, 77 },
    { 0x1f23d998, 61 },
    { 0x4303d997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d3d99d, 98 },
    { 0, 0 },
    { 0xd243d99f, 76 },
    { 0, 0 },
    { 0x9c93d9a1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6193d9a9, 95 },
    { 0xc793d9aa, 68 },
    { 0x5313d9ab, 76 },
    { 0, 0 },
    { 0x5ac3d9ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a63d9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a43d9b5, 33 },
    { 0, 0 },
    { 0x6be3d9b7, 33 },
    { 0, 0 },
    { 0xf713d9b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3283d9c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb763d9c5, 23 },
    { 0xd893d9c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf163d9cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc593d9d1, 80 },
    { 0x6f3d9d2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a53d9d9, 48 },
    { 0, 0 },
    { 0xa103d9db, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3d9e0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd483d9e3, 12 },
    { 0x77b3d9e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x693d9e7, 77 },
    { 0xc243d9e7, 34 },
    { 0x1673d9e9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e3d9ec, 91 },
    { 0x8f23d9ed, 56 },
    { 0, 0 },
    { 0x2f83d9ef, 55 },
    { 0x1a23d9f0, 95 },
    { 0, 0 },
    { 0x9ca3d9f2, 33 },
    { 0, 0 },
    { 0xce3d9f4, 76 },
    { 0, 0 },
    { 0xf853d9f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c73d9f9, 34 },
    { 0x17a3d9fa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5743d9fd, 61 },
    { 0x513d9fe, 61 },
    { 0xeb03d9ff, 66 },
    { 0x7083d9fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1183da05, 90 },
    { 0xa963da06, 90 },
    { 0x9253da07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a3da0a, 43 },
    { 0, 0 },
    { 0x2c03da0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3533da0f, 76 },
    { 0, 0 },
    { 0x5133da11, 90 },
    { 0x8d93da11, 43 },
    { 0xea53da13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f3da18, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c3da21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a3da29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9523da2c, 55 },
    { 0x1fc3da2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a53da34, 40 },
    { 0x7dc3da35, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2843da38, 98 },
    { 0, 0 },
    { 0x59b3da3a, 87 },
    { 0x3de3da3a, 77 },
    { 0x6e63da3a, 90 },
    { 0x84b3da3b, 95 },
    { 0x4393da3b, 81 },
    { 0xe8e3da3b, 87 },
    { 0x87d3da3c, 55 },
    { 0xf2a3da3d, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff3da44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe053da48, 61 },
    { 0xae63da49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3da4d, 92 },
    { 0, 0 },
    { 0xf3da4f, 23 },
    { 0x1473da4f, 95 },
    { 0x93a3da51, 75 },
    { 0x4ac3da52, 66 },
    { 0, 0 },
    { 0x36b3da54, 55 },
    { 0, 0 },
    { 0x3a33da56, 33 },
    { 0xac83da56, 96 },
    { 0, 0 },
    { 0x96b3da59, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc753da5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc123da62, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe573da67, 90 },
    { 0xfe33da68, 61 },
    { 0x2503da67, 93 },
    { 0, 0 },
    { 0xb0b3da6b, 90 },
    { 0, 0 },
    { 0x6203da6d, 23 },
    { 0xa493da6e, 17 },
    { 0, 0 },
    { 0x44a3da70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7803da73, 61 },
    { 0x40f3da74, 75 },
    { 0x8833da74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b63da7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf263da7d, 90 },
    { 0, 0 },
    { 0x1233da7f, 96 },
    { 0, 0 },
    { 0xe023da81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf793da84, 61 },
    { 0x90c3da84, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe3da88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a3da8e, 78 },
    { 0x9dd3da8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a3da93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce3da97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf33da9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f3daa3, 68 },
    { 0xa253daa4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3daa7, 98 },
    { 0, 0 },
    { 0xf993daa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8093daaf, 61 },
    { 0x2cc3dab0, 23 },
    { 0x133dab0, 33 },
    { 0x5cc3dab2, 90 },
    { 0xa9d3dab2, 98 },
    { 0, 0 },
    { 0x6d83dab5, 95 },
    { 0xce13dab5, 91 },
    { 0x9243dab7, 95 },
    { 0x4643dab5, 61 },
    { 0xc123dab9, 61 },
    { 0xbb83dab9, 95 },
    { 0x56f3dabb, 46 },
    { 0x9a43dabc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023dac4, 23 },
    { 0xaa13dac5, 90 },
    { 0x38f3dac6, 78 },
    { 0x6d83dac7, 46 },
    { 0xf813dac6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93dacd, 100 },
    { 0xf413dace, 90 },
    { 0x3353dace, 95 },
    { 0x7e63dacf, 90 },
    { 0x6b63dad0, 61 },
    { 0, 0 },
    { 0x4f93dad3, 87 },
    { 0x4a3dad3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe593dad8, 91 },
    { 0x6a33dad9, 43 },
    { 0x57c3dada, 76 },
    { 0x4373dadb, 33 },
    { 0, 0 },
    { 0xb083dadd, 61 },
    { 0, 0 },
    { 0xc3b3dadf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda13dae2, 55 },
    { 0x7963dae3, 61 },
    { 0xa073dae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3563dae9, 52 },
    { 0, 0 },
    { 0xc4e3daeb, 55 },
    { 0xca3daec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed3daf2, 98 },
    { 0xfc93daf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4453daf6, 98 },
    { 0x9e53daf7, 52 },
    { 0x3363daf7, 81 },
    { 0x2893daf9, 33 },
    { 0xf8a3daf7, 78 },
    { 0, 0 },
    { 0x5493dafc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae3db00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5383db07, 61 },
    { 0xd003db08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e43db0b, 43 },
    { 0, 0 },
    { 0xf033db0d, 87 },
    { 0, 0 },
    { 0x9ff3db0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee93db12, 100 },
    { 0, 0 },
    { 0x5b83db14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1993db17, 43 },
    { 0x6ed3db18, 76 },
    { 0, 0 },
    { 0x37d3db1a, 90 },
    { 0, 0 },
    { 0x4293db1c, 55 },
    { 0xee3db1d, 84 },
    { 0, 0 },
    { 0x8843db1f, 85 },
    { 0xa173db20, 91 },
    { 0x21f3db21, 46 },
    { 0xb183db22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c83db26, 55 },
    { 0x5ca3db27, 50 },
    { 0xff73db28, 41 },
    { 0, 0 },
    { 0xa683db2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc773db2e, 95 },
    { 0xecb3db2f, 95 },
    { 0, 0 },
    { 0xcd53db31, 46 },
    { 0x76a3db32, 90 },
    { 0, 0 },
    { 0xbcf3db34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb203db37, 95 },
    { 0, 0 },
    { 0xfca3db39, 95 },
    { 0x6103db3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc413db40, 61 },
    { 0xa6f3db41, 52 },
    { 0x1e43db42, 99 },
    { 0x9883db43, 23 },
    { 0xfbf3db44, 95 },
    { 0, 0 },
    { 0x8393db46, 76 },
    { 0x4b93db47, 68 },
    { 0x8cf3db46, 55 },
    { 0xb883db47, 100 },
    { 0, 0 },
    { 0xb6c3db4b, 98 },
    { 0, 0 },
    { 0x6373db4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd3db52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9673db59, 46 },
    { 0x9c83db5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c93db5e, 17 },
    { 0xe103db5f, 90 },
    { 0x2643db5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3db66, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b03db6c, 84 },
    { 0x8613db6d, 95 },
    { 0xe623db6e, 88 },
    { 0x983db6f, 23 },
    { 0, 0 },
    { 0xb6b3db71, 98 },
    { 0xc4c3db71, 100 },
    { 0xaf43db73, 94 },
    { 0xe933db74, 90 },
    { 0x37f3db75, 96 },
    { 0x1e73db71, 34 },
    { 0x4d83db77, 90 },
    { 0x2463db76, 17 },
    { 0xca43db76, 61 },
    { 0xca3db79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b23db81, 98 },
    { 0x90d3db81, 43 },
    { 0xcd93db82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6653db86, 95 },
    { 0, 0 },
    { 0x39a3db88, 61 },
    { 0x7393db89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf943db8c, 96 },
    { 0x9b13db8d, 46 },
    { 0x2d53db8e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4283db94, 90 },
    { 0xafb3db94, 95 },
    { 0xf5e3db95, 98 },
    { 0x4b23db95, 95 },
    { 0x3d53db98, 81 },
    { 0x1453db98, 61 },
    { 0, 0 },
    { 0x1bc3db9b, 93 },
    { 0, 0 },
    { 0xb423db9d, 90 },
    { 0x9b13db9d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b3dba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1413dbac, 84 },
    { 0x96e3dbad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd563dbb0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9183dbb3, 90 },
    { 0x4dd3dbb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a63dbb7, 95 },
    { 0, 0 },
    { 0xc913dbb9, 90 },
    { 0, 0 },
    { 0x2f73dbbb, 61 },
    { 0xf0c3dbbb, 95 },
    { 0x6e43dbbc, 100 },
    { 0x6e33dbbb, 95 },
    { 0x3c3dbbf, 61 },
    { 0xb823dbbb, 38 },
    { 0xa453dbc1, 87 },
    { 0x2273dbc1, 90 },
    { 0x28d3dbc2, 90 },
    { 0x9d73dbbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d3dbcd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3dbd1, 55 },
    { 0x883dbd2, 33 },
    { 0x2f03dbd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7153dbda, 95 },
    { 0xf193dbdb, 91 },
    { 0x4e83dbdc, 23 },
    { 0x6023dbdd, 100 },
    { 0xc9b3dbda, 78 },
    { 0, 0 },
    { 0x8073dbe0, 43 },
    { 0, 0 },
    { 0x63f3dbe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3753dbe5, 66 },
    { 0xcbf3dbe6, 43 },
    { 0, 0 },
    { 0xb953dbe8, 61 },
    { 0xfeb3dbe9, 43 },
    { 0x2b63dbea, 95 },
    { 0x3d23dbeb, 83 },
    { 0x2b93dbea, 90 },
    { 0x47c3dbea, 34 },
    { 0xacb3dbee, 23 },
    { 0x4613dbe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b63dbf2, 34 },
    { 0x5853dbf3, 46 },
    { 0x31a3dbf4, 95 },
    { 0xbe73dbf4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b3dbf8, 77 },
    { 0x68d3dbf9, 52 },
    { 0x3793dbf9, 91 },
    { 0xade3dbfa, 33 },
    { 0x7a23dbf9, 34 },
    { 0, 0 },
    { 0x9843dbfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3493dc05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd93dc0b, 75 },
    { 0, 0 },
    { 0xb2c3dc0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf93dc10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb3dc15, 33 },
    { 0x2f43dc15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd3dc19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c3dc1c, 95 },
    { 0xbda3dc1c, 91 },
    { 0, 0 },
    { 0xe4d3dc1f, 93 },
    { 0, 0 },
    { 0x7023dc21, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc303dc27, 34 },
    { 0x74c3dc28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2773dc33, 76 },
    { 0xa153dc34, 33 },
    { 0xc793dc33, 34 },
    { 0xb73dc36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe443dc39, 46 },
    { 0, 0 },
    { 0x7c13dc3b, 46 },
    { 0x25c3dc3c, 33 },
    { 0x9013dc3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e53dc4c, 93 },
    { 0, 0 },
    { 0xf9e3dc4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda93dc54, 81 },
    { 0x2943dc55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5023dc59, 83 },
    { 0x1313dc5a, 23 },
    { 0x4e3dc5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3dc66, 33 },
    { 0, 0 },
    { 0xa9f3dc68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa3dc6b, 98 },
    { 0x33c3dc6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xba53dc6f, 95 },
    { 0, 0 },
    { 0x8b23dc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3dc75, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c33dc7b, 61 },
    { 0x2253dc7c, 95 },
    { 0x31d3dc7c, 55 },
    { 0x1ca3dc7e, 66 },
    { 0, 0 },
    { 0x58c3dc80, 90 },
    { 0x4963dc81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc63dc84, 61 },
    { 0x9203dc85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e3dc88, 56 },
    { 0x2903dc89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff93dc9a, 75 },
    { 0xbf53dc9b, 98 },
    { 0x7fd3dc9b, 91 },
    { 0x6cf3dc9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c53dca0, 95 },
    { 0, 0 },
    { 0x8333dca2, 23 },
    { 0x47f3dca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d83dcb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f3dcba, 34 },
    { 0xff73dcba, 33 },
    { 0x7753dcbc, 23 },
    { 0x5013dcbc, 95 },
    { 0x82d3dcbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b3dcc1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33dcc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee93dcc8, 33 },
    { 0, 0 },
    { 0x1c23dcca, 77 },
    { 0xc0d3dcca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc373dcce, 34 },
    { 0, 0 },
    { 0x8da3dcd0, 33 },
    { 0x1413dcd1, 61 },
    { 0, 0 },
    { 0xb9b3dcd3, 34 },
    { 0x1023dcd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f3dcd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee83dcda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3dce7, 85 },
    { 0x4cd3dce8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3dceb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6563dcee, 98 },
    { 0, 0 },
    { 0xb4d3dcf0, 95 },
    { 0, 0 },
    { 0x1323dcf2, 23 },
    { 0xe2a3dcf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e3dcf7, 46 },
    { 0, 0 },
    { 0xfec3dcf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa243dcfd, 90 },
    { 0xc5f3dcfd, 98 },
    { 0xa263dcff, 66 },
    { 0x7c13dcff, 23 },
    { 0x31e3dcfe, 61 },
    { 0, 0 },
    { 0x2043dd03, 90 },
    { 0x8983dd04, 33 },
    { 0, 0 },
    { 0x66e3dd06, 76 },
    { 0xb083dd06, 23 },
    { 0xda3dd08, 33 },
    { 0x4fd3dd06, 23 },
    { 0, 0 },
    { 0x8213dd0b, 86 },
    { 0, 0 },
    { 0xe43dd0d, 80 },
    { 0x7bc3dd0e, 77 },
    { 0x5643dd0e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc763dd13, 76 },
    { 0, 0 },
    { 0x7c43dd15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc3dd18, 86 },
    { 0xa153dd19, 33 },
    { 0, 0 },
    { 0xa623dd1b, 12 },
    { 0xaa93dd1c, 98 },
    { 0x5203dd1d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1233dd25, 75 },
    { 0x1183dd26, 43 },
    { 0xe6f3dd26, 98 },
    { 0xf673dd28, 81 },
    { 0xf243dd26, 98 },
    { 0, 0 },
    { 0x6433dd2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d3dd32, 17 },
    { 0xe663dd32, 90 },
    { 0, 0 },
    { 0x2303dd35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c3dd3f, 95 },
    { 0x1a63dd40, 90 },
    { 0xd9c3dd41, 84 },
    { 0, 0 },
    { 0xa643dd43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7243dd48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd53dd4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb3dd56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a3dd5a, 90 },
    { 0, 0 },
    { 0xb173dd5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c3dd5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec3dd65, 68 },
    { 0x2943dd66, 33 },
    { 0xfd53dd67, 55 },
    { 0xe543dd67, 90 },
    { 0xccf3dd69, 76 },
    { 0x8db3dd66, 91 },
    { 0, 0 },
    { 0x6113dd6c, 90 },
    { 0, 0 },
    { 0x1dc3dd6e, 33 },
    { 0x1be3dd6f, 90 },
    { 0x2c03dd6f, 78 },
    { 0x4e43dd71, 55 },
    { 0x86f3dd72, 43 },
    { 0x5ff3dd6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8993dd7c, 61 },
    { 0xa3e3dd7d, 33 },
    { 0xce43dd7d, 68 },
    { 0x663dd7e, 78 },
    { 0x9693dd7e, 46 },
    { 0x7073dd7d, 61 },
    { 0, 0 },
    { 0x4dd3dd83, 61 },
    { 0x2c73dd83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f3dd87, 55 },
    { 0x5433dd88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a3dd8e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2513dd94, 98 },
    { 0, 0 },
    { 0xfa03dd96, 87 },
    { 0x1303dd97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa3dd9d, 61 },
    { 0, 0 },
    { 0xbae3dd9f, 93 },
    { 0x6f63dda0, 33 },
    { 0, 0 },
    { 0x36c3dda2, 97 },
    { 0x8a53dda3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3523ddbc, 55 },
    { 0x5903ddbd, 91 },
    { 0x753ddbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe3ddc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6653ddc6, 61 },
    { 0, 0 },
    { 0x4a73ddc8, 61 },
    { 0xb703ddc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8283ddd0, 23 },
    { 0x4bc3ddd1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac23ddd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc113ddd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb3dddc, 55 },
    { 0xb763dddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc913dde0, 33 },
    { 0xee63dde1, 84 },
    { 0x2f43dde2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a3dde5, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf13ddea, 55 },
    { 0x3ac3ddea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa403ddef, 90 },
    { 0x88a3ddef, 98 },
    { 0x7213ddef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf283ddf4, 75 },
    { 0xaeb3ddf4, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b03ddf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f3ddfd, 97 },
    { 0xdbc3ddfd, 76 },
    { 0xddf3ddfe, 98 },
    { 0xaf23ddfe, 23 },
    { 0, 0 },
    { 0x7663de02, 77 },
    { 0xf973de03, 90 },
    { 0xce53de03, 33 },
    { 0x6db3de05, 61 },
    { 0xb113de06, 23 },
    { 0xf3c3de07, 43 },
    { 0x1483de05, 67 },
    { 0x2bb3de09, 90 },
    { 0, 0 },
    { 0x663de0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3de0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e3de19, 61 },
    { 0x4b3de19, 33 },
    { 0x12b3de19, 17 },
    { 0, 0 },
    { 0xe6f3de1d, 81 },
    { 0, 0 },
    { 0xb2b3de1f, 78 },
    { 0, 0 },
    { 0x1003de21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba3de26, 55 },
    { 0x5543de27, 61 },
    { 0, 0 },
    { 0x6ae3de29, 33 },
    { 0xbf33de2a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa103de2e, 23 },
    { 0x7173de2f, 61 },
    { 0x18e3de30, 90 },
    { 0xc703de2e, 55 },
    { 0xd3e3de32, 99 },
    { 0x3403de33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d43de37, 90 },
    { 0xad63de38, 33 },
    { 0x8dd3de39, 76 },
    { 0, 0 },
    { 0x5023de3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983de42, 33 },
    { 0, 0 },
    { 0xc13de44, 95 },
    { 0, 0 },
    { 0x16f3de46, 33 },
    { 0, 0 },
    { 0xf3e3de48, 99 },
    { 0xa743de49, 98 },
    { 0x3fe3de4a, 95 },
    { 0x83d3de4b, 98 },
    { 0x9393de4c, 43 },
    { 0x4c63de4c, 23 },
    { 0xc773de4a, 23 },
    { 0, 0 },
    { 0xb633de50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5093de55, 90 },
    { 0xff53de56, 90 },
    { 0x4d53de55, 61 },
    { 0, 0 },
    { 0xea23de59, 61 },
    { 0x44d3de5a, 33 },
    { 0xc2e3de5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b23de5e, 76 },
    { 0x10c3de5e, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc813de67, 90 },
    { 0xd813de68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d3de6b, 67 },
    { 0x25e3de6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe453de6f, 91 },
    { 0, 0 },
    { 0xe883de71, 33 },
    { 0, 0 },
    { 0x2183de73, 84 },
    { 0x8fb3de74, 81 },
    { 0, 0 },
    { 0x3b83de76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b3de7e, 100 },
    { 0x88d3de7e, 33 },
    { 0x3f63de7e, 95 },
    { 0x1043de81, 95 },
    { 0xb2f3de7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53de8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae3de91, 95 },
    { 0, 0 },
    { 0x43de93, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc3de96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa783de99, 23 },
    { 0x7af3de9a, 34 },
    { 0, 0 },
    { 0xc7b3de9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5483dea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca3dea9, 78 },
    { 0, 0 },
    { 0xb5e3deab, 33 },
    { 0xa23deac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f3deb3, 95 },
    { 0, 0 },
    { 0xece3deb5, 98 },
    { 0, 0 },
    { 0xdd3deb7, 46 },
    { 0x6b53deb8, 76 },
    { 0x8bf3deb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae43debe, 52 },
    { 0xbe03debe, 76 },
    { 0xf233dec0, 33 },
    { 0x80f3dec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c3dec8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb63ded1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b33ded4, 98 },
    { 0, 0 },
    { 0x3903ded6, 17 },
    { 0x24b3ded7, 88 },
    { 0x1c13ded7, 90 },
    { 0x4233ded8, 88 },
    { 0xb3b3ded9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e63dee2, 56 },
    { 0x9c73dee2, 76 },
    { 0xf903dee2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f43deef, 61 },
    { 0xc6d3def0, 55 },
    { 0, 0 },
    { 0x3413def2, 100 },
    { 0, 0 },
    { 0xa2c3def4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e93def7, 52 },
    { 0xd573def7, 33 },
    { 0xa933def8, 87 },
    { 0xd303defa, 43 },
    { 0, 0 },
    { 0x4643defc, 78 },
    { 0xb783defd, 55 },
    { 0xc463defd, 33 },
    { 0x68b3deff, 33 },
    { 0x2113defe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc943df05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e33df08, 90 },
    { 0xa8c3df08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb923df0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1593df0f, 75 },
    { 0x3f53df10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b43df14, 98 },
    { 0x3763df15, 93 },
    { 0x9a63df15, 81 },
    { 0x43f3df15, 98 },
    { 0x2173df17, 34 },
    { 0x4e03df19, 23 },
    { 0xeba3df1a, 23 },
    { 0, 0 },
    { 0x9b43df1c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e43df20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc443df28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa483df2b, 43 },
    { 0x33df2c, 65 },
    { 0x7f3df2b, 90 },
    { 0, 0 },
    { 0x7033df2f, 93 },
    { 0, 0 },
    { 0x6043df31, 80 },
    { 0x4e23df32, 88 },
    { 0xdcd3df32, 98 },
    { 0xcb33df33, 43 },
    { 0x8b83df35, 61 },
    { 0xd723df36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e3df45, 76 },
    { 0x47d3df45, 66 },
    { 0, 0 },
    { 0x8133df48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d63df4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e3df4e, 98 },
    { 0x55b3df4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c23df55, 52 },
    { 0x963df55, 34 },
    { 0xcd93df57, 66 },
    { 0, 0 },
    { 0xc83df59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f3df65, 61 },
    { 0x333df66, 76 },
    { 0xacc3df67, 95 },
    { 0xb543df68, 98 },
    { 0, 0 },
    { 0x9f53df6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed3df6f, 90 },
    { 0x86b3df6f, 33 },
    { 0, 0 },
    { 0x2dd3df72, 98 },
    { 0, 0 },
    { 0x8443df74, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc833df83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f3df87, 33 },
    { 0, 0 },
    { 0x1303df89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8963df8e, 33 },
    { 0xadf3df8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9963df93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b3df97, 99 },
    { 0, 0 },
    { 0xad33df99, 90 },
    { 0x3e43df9a, 95 },
    { 0, 0 },
    { 0x53e3df9c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb53dfa2, 46 },
    { 0xf4a3dfa3, 43 },
    { 0, 0 },
    { 0x5a83dfa5, 66 },
    { 0, 0 },
    { 0x2b53dfa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb613dfaa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc3dfb1, 33 },
    { 0, 0 },
    { 0xe2f3dfb3, 78 },
    { 0xf713dfb4, 95 },
    { 0x7053dfb3, 98 },
    { 0x2133dfb6, 33 },
    { 0x3ec3dfb7, 61 },
    { 0x423dfb7, 23 },
    { 0x68a3dfb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4333dfbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b3dfc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2013dfc5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1723dfca, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e3dfcf, 90 },
    { 0xa183dfd0, 61 },
    { 0, 0 },
    { 0x3863dfd2, 61 },
    { 0xc983dfd2, 91 },
    { 0x7e13dfd2, 95 },
    { 0x4f73dfd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe213dfd8, 33 },
    { 0, 0 },
    { 0x8a13dfda, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2193dfe0, 23 },
    { 0x2f63dfe0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe63dfe5, 55 },
    { 0x4263dfe6, 33 },
    { 0xf173dfe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd383dfee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3dff5, 90 },
    { 0xb2f3dff6, 52 },
    { 0x5ef3dff5, 93 },
    { 0x24f3dff8, 78 },
    { 0, 0 },
    { 0x5083dffa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa713e007, 65 },
    { 0xc573e008, 95 },
    { 0, 0 },
    { 0xb373e00a, 61 },
    { 0xa4d3e00b, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b73e00f, 55 },
    { 0, 0 },
    { 0x42b3e011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b3e015, 97 },
    { 0, 0 },
    { 0x99b3e017, 61 },
    { 0x7d33e018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c63e01c, 55 },
    { 0, 0 },
    { 0x76f3e01e, 43 },
    { 0x1d53e01f, 60 },
    { 0x183e01f, 34 },
    { 0x4d3e021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe313e025, 23 },
    { 0x2b33e026, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b3e02b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa773e032, 65 },
    { 0x5ae3e032, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd23e039, 33 },
    { 0x9253e039, 23 },
    { 0x13b3e03b, 97 },
    { 0xfb13e03a, 93 },
    { 0x6053e03c, 99 },
    { 0xec83e03e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d3e041, 17 },
    { 0x8dc3e042, 23 },
    { 0xd33e043, 95 },
    { 0x7cd3e044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb293e048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb23e04c, 90 },
    { 0x5fc3e04c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c73e050, 52 },
    { 0, 0 },
    { 0xc153e052, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403e059, 76 },
    { 0x46d3e05a, 23 },
    { 0x2aa3e05b, 61 },
    { 0, 0 },
    { 0x6f73e05d, 33 },
    { 0xdd63e05e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c3e061, 98 },
    { 0, 0 },
    { 0x79e3e063, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca03e066, 98 },
    { 0, 0 },
    { 0x4e73e068, 33 },
    { 0x1433e069, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa73e06e, 78 },
    { 0x2073e06f, 95 },
    { 0, 0 },
    { 0x8443e071, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f33e074, 90 },
    { 0, 0 },
    { 0x98c3e076, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe523e07a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2183e07f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x613e082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf93e086, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f3e08e, 23 },
    { 0x5fa3e08e, 33 },
    { 0xb03e08f, 88 },
    { 0x203e091, 98 },
    { 0x5953e090, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe73e095, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa493e098, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5833e09c, 61 },
    { 0x9c93e09d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153e0a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2813e0a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec3e0a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b73e0b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e0bd, 33 },
    { 0xb2e3e0be, 23 },
    { 0x9253e0bf, 46 },
    { 0xdc43e0bf, 98 },
    { 0x33c3e0bf, 90 },
    { 0x5f13e0c2, 43 },
    { 0xda43e0bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f3e0ca, 95 },
    { 0xa633e0cb, 95 },
    { 0, 0 },
    { 0xa9c3e0cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c3e0d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd583e0d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a3e0da, 34 },
    { 0x29b3e0db, 52 },
    { 0x2373e0dc, 61 },
    { 0x3eb3e0dd, 91 },
    { 0x9c3e0de, 95 },
    { 0xdb03e0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4013e0e4, 90 },
    { 0x20d3e0e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b3e0e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc63e0eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9853e0f1, 90 },
    { 0x39d3e0f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d3e0fb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9033e0ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b3e103, 61 },
    { 0x6613e103, 33 },
    { 0xc373e105, 95 },
    { 0xffa3e106, 93 },
    { 0x8363e104, 23 },
    { 0xb0a3e108, 76 },
    { 0xabd3e108, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3103e10d, 33 },
    { 0, 0 },
    { 0x9533e10f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd203e113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a33e116, 66 },
    { 0xc1a3e117, 76 },
    { 0xa183e118, 46 },
    { 0x1b83e119, 33 },
    { 0, 0 },
    { 0xc533e11b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc853e124, 76 },
    { 0x37f3e125, 23 },
    { 0xeb23e126, 34 },
    { 0x1743e126, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e3e12b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c43e12e, 99 },
    { 0x8b3e12e, 95 },
    { 0, 0 },
    { 0x8963e131, 93 },
    { 0x8453e132, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd333e139, 68 },
    { 0xe143e13a, 55 },
    { 0x8463e13a, 34 },
    { 0x3483e13c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6903e140, 90 },
    { 0x7b43e140, 61 },
    { 0x5843e141, 51 },
    { 0, 0 },
    { 0xb3d3e144, 61 },
    { 0xc7f3e145, 99 },
    { 0, 0 },
    { 0xbc3e147, 91 },
    { 0x6603e148, 95 },
    { 0x26d3e149, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3633e14d, 51 },
    { 0x3773e14d, 61 },
    { 0xbe83e14d, 93 },
    { 0x1213e14d, 23 },
    { 0, 0 },
    { 0xbb43e152, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1513e155, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8193e158, 99 },
    { 0x633e159, 90 },
    { 0xe273e159, 98 },
    { 0xac3e158, 43 },
    { 0x5c93e15c, 98 },
    { 0xa143e15d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f3e161, 17 },
    { 0, 0 },
    { 0x2f93e163, 90 },
    { 0x9e73e164, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df3e167, 61 },
    { 0xf553e168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x373e16b, 98 },
    { 0xe003e16c, 78 },
    { 0, 0 },
    { 0xead3e16e, 96 },
    { 0, 0 },
    { 0x4ef3e170, 90 },
    { 0, 0 },
    { 0x18a3e172, 23 },
    { 0, 0 },
    { 0x6713e174, 23 },
    { 0, 0 },
    { 0xc253e176, 57 },
    { 0xc423e177, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a3e181, 91 },
    { 0xdf23e181, 36 },
    { 0, 0 },
    { 0x5583e184, 33 },
    { 0, 0 },
    { 0xdaf3e186, 95 },
    { 0xe253e186, 61 },
    { 0, 0 },
    { 0x6fe3e189, 61 },
    { 0x7a93e18a, 66 },
    { 0, 0 },
    { 0x47f3e18c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc3e19c, 98 },
    { 0x4963e19d, 95 },
    { 0xf7a3e19d, 33 },
    { 0x89a3e19e, 36 },
    { 0x2993e1a0, 55 },
    { 0xa083e1a1, 93 },
    { 0, 0 },
    { 0xcbb3e1a3, 33 },
    { 0x8693e1a4, 55 },
    { 0x2a23e1a3, 23 },
    { 0x3b73e1a4, 95 },
    { 0x6393e1a7, 23 },
    { 0xadf3e1a8, 23 },
    { 0, 0 },
    { 0x2a33e1aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8813e1ae, 46 },
    { 0, 0 },
    { 0x1f03e1b0, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1453e1b9, 76 },
    { 0xd6e3e1ba, 95 },
    { 0, 0 },
    { 0x57d3e1bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb3e1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf3e1c9, 95 },
    { 0x9833e1ca, 93 },
    { 0xb7f3e1c9, 33 },
    { 0x58d3e1cc, 98 },
    { 0x73c3e1cd, 77 },
    { 0x22f3e1ce, 33 },
    { 0xf033e1cb, 90 },
    { 0x3333e1cb, 95 },
    { 0x9d33e1ce, 33 },
    { 0, 0 },
    { 0x7c83e1d3, 61 },
    { 0x8db3e1d4, 93 },
    { 0x5cf3e1d5, 55 },
    { 0, 0 },
    { 0x3713e1d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e3e1da, 98 },
    { 0x9c43e1da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c93e1de, 61 },
    { 0, 0 },
    { 0x5d43e1e0, 33 },
    { 0x2723e1e1, 33 },
    { 0xdde3e1e2, 90 },
    { 0x6883e1e3, 55 },
    { 0xaf93e1e3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03e1ea, 34 },
    { 0x46d3e1ea, 90 },
    { 0x2e13e1ea, 33 },
    { 0, 0 },
    { 0x5623e1ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e23e1f1, 61 },
    { 0x5d33e1f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d3e1f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9933e1fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d3e201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053e20a, 90 },
    { 0x1e3e20b, 90 },
    { 0, 0 },
    { 0x94c3e20d, 65 },
    { 0, 0 },
    { 0x5373e20f, 33 },
    { 0xa383e210, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d3e215, 55 },
    { 0x7f63e216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa663e219, 17 },
    { 0x5f23e21a, 33 },
    { 0x7283e21b, 77 },
    { 0, 0 },
    { 0x6ac3e21d, 78 },
    { 0, 0 },
    { 0xa8c3e21f, 46 },
    { 0x3783e21f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a3e224, 97 },
    { 0, 0 },
    { 0x3593e226, 95 },
    { 0, 0 },
    { 0x32d3e228, 76 },
    { 0, 0 },
    { 0xc9c3e22a, 90 },
    { 0x5fd3e22a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4463e22e, 61 },
    { 0x97a3e22f, 93 },
    { 0xbc93e22e, 61 },
    { 0, 0 },
    { 0x3053e232, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c3e23b, 23 },
    { 0, 0 },
    { 0x7cf3e23d, 86 },
    { 0xea13e23e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b3e242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e3e24f, 76 },
    { 0, 0 },
    { 0xad63e251, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9403e254, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a73e25f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c3e266, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5443e269, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc273e26d, 98 },
    { 0, 0 },
    { 0x56c3e26f, 76 },
    { 0xd6e3e270, 76 },
    { 0x9293e271, 52 },
    { 0x7bd3e270, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3e275, 69 },
    { 0xc13e275, 100 },
    { 0x1833e277, 84 },
    { 0xfea3e278, 90 },
    { 0x2493e279, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d3e27f, 95 },
    { 0x69e3e280, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c3e285, 98 },
    { 0xb703e286, 95 },
    { 0x53d3e286, 95 },
    { 0x1d63e286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf33e28d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b83e291, 33 },
    { 0x8683e291, 43 },
    { 0, 0 },
    { 0xa023e294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf953e298, 88 },
    { 0x1ae3e299, 88 },
    { 0xb443e299, 38 },
    { 0xdaf3e299, 95 },
    { 0xe403e29c, 95 },
    { 0xa633e29d, 81 },
    { 0, 0 },
    { 0x8c73e29f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa443e2a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5553e2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6263e2b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a3e2b5, 99 },
    { 0xf9c3e2b5, 33 },
    { 0xb2a3e2b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc3e2bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b93e2c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc93e2c5, 98 },
    { 0x9fb3e2c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3e2cd, 66 },
    { 0, 0 },
    { 0x9513e2cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb3e2d2, 90 },
    { 0x76b3e2d3, 53 },
    { 0xfc03e2d4, 90 },
    { 0xd3e3e2d4, 95 },
    { 0x8743e2d2, 98 },
    { 0, 0 },
    { 0x9de3e2d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6593e2e2, 90 },
    { 0, 0 },
    { 0x6213e2e4, 85 },
    { 0x4213e2e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb3e2e9, 90 },
    { 0xf863e2e9, 23 },
    { 0, 0 },
    { 0xdfb3e2ec, 66 },
    { 0x68a3e2ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1793e2f1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d3e2f7, 100 },
    { 0x9883e2f7, 23 },
    { 0xc803e2f9, 76 },
    { 0xc933e2fa, 61 },
    { 0x3d63e2f8, 99 },
    { 0xfbb3e2f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd643e308, 19 },
    { 0x36a3e309, 55 },
    { 0xc33e309, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a3e310, 33 },
    { 0x5b23e310, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b63e315, 90 },
    { 0x7c33e316, 46 },
    { 0, 0 },
    { 0x4633e318, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc83e31b, 34 },
    { 0x85f3e31c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab3e322, 76 },
    { 0x2ff3e323, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c3e326, 66 },
    { 0xe983e327, 46 },
    { 0x9223e326, 61 },
    { 0x9ad3e326, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c3e32d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c73e334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb063e33f, 90 },
    { 0, 0 },
    { 0x72f3e341, 91 },
    { 0x763e341, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f63e34a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6913e34d, 90 },
    { 0, 0 },
    { 0x3dc3e34f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3e352, 65 },
    { 0xbfa3e353, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973e357, 76 },
    { 0xf7b3e358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8343e363, 91 },
    { 0x26d3e364, 90 },
    { 0x9113e363, 90 },
    { 0x7f73e363, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1313e36a, 93 },
    { 0xe03e36b, 90 },
    { 0, 0 },
    { 0x9e83e36d, 88 },
    { 0xd933e36e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3e371, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2313e377, 34 },
    { 0, 0 },
    { 0x6c73e379, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd363e37d, 55 },
    { 0xca83e37d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f3e383, 46 },
    { 0xb6b3e384, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3e387, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff53e38c, 68 },
    { 0, 0 },
    { 0xf3f3e38e, 76 },
    { 0xc2b3e38f, 91 },
    { 0xf433e38f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf793e393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c13e398, 90 },
    { 0x2e33e399, 43 },
    { 0xed43e39a, 88 },
    { 0x11e3e399, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea3e39e, 33 },
    { 0, 0 },
    { 0x3993e3a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe913e3a4, 84 },
    { 0x2c03e3a5, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa73e3ac, 23 },
    { 0x603e3ac, 61 },
    { 0, 0 },
    { 0xf6e3e3af, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3e3b2, 52 },
    { 0x7fc3e3b3, 33 },
    { 0, 0 },
    { 0xf283e3b5, 65 },
    { 0, 0 },
    { 0x4a53e3b7, 87 },
    { 0x8df3e3b8, 23 },
    { 0xc633e3b7, 33 },
    { 0x2c23e3ba, 61 },
    { 0x823e3b9, 23 },
    { 0, 0 },
    { 0xfde3e3bd, 55 },
    { 0xa5b3e3be, 84 },
    { 0xc2f3e3be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec3e3c6, 84 },
    { 0xd5e3e3c6, 33 },
    { 0x4663e3c8, 61 },
    { 0xd9c3e3c8, 34 },
    { 0x1413e3ca, 61 },
    { 0xfe83e3cb, 68 },
    { 0x6b03e3cc, 33 },
    { 0xec53e3ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6183e3d3, 98 },
    { 0xcb33e3d4, 55 },
    { 0x8f73e3d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b3e3d9, 100 },
    { 0xa6c3e3da, 40 },
    { 0xe483e3db, 17 },
    { 0, 0 },
    { 0x56a3e3dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4773e3e0, 76 },
    { 0xc3d3e3e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a3e3e5, 69 },
    { 0x8cb3e3e6, 33 },
    { 0x4743e3e7, 33 },
    { 0x8703e3e8, 77 },
    { 0xf973e3e8, 84 },
    { 0, 0 },
    { 0x6c03e3eb, 55 },
    { 0, 0 },
    { 0xb1f3e3ed, 80 },
    { 0x6d53e3ed, 23 },
    { 0x4863e3ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4643e3f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e33e3f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba3e3f8, 33 },
    { 0, 0 },
    { 0x2203e3fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43e3fe, 98 },
    { 0xc223e3ff, 87 },
    { 0xe163e3ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a3e403, 95 },
    { 0, 0 },
    { 0xaeb3e405, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1253e40d, 61 },
    { 0xd403e40e, 90 },
    { 0, 0 },
    { 0xc703e410, 66 },
    { 0x8673e411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3e418, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3e420, 99 },
    { 0x1543e420, 67 },
    { 0xb353e420, 95 },
    { 0x8d43e423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af3e428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf33e42c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b53e431, 23 },
    { 0x3043e432, 68 },
    { 0xb4c3e432, 61 },
    { 0x59c3e433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d3e43d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7773e444, 51 },
    { 0x6103e445, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d3e44b, 78 },
    { 0x2393e44b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb873e451, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b33e455, 33 },
    { 0, 0 },
    { 0xf1d3e457, 93 },
    { 0, 0 },
    { 0x3513e459, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743e45c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7233e45f, 95 },
    { 0x15b3e460, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c63e464, 40 },
    { 0, 0 },
    { 0xc4a3e466, 90 },
    { 0x7ab3e466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d3e46b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263e46e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f3e474, 78 },
    { 0x3a13e475, 33 },
    { 0x1613e475, 43 },
    { 0x1283e477, 90 },
    { 0x763e477, 65 },
    { 0xfcd3e475, 90 },
    { 0, 0 },
    { 0xa223e47b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb43e487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb143e48d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a3e496, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b13e49a, 90 },
    { 0x4293e49a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7623e4a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7043e4a9, 76 },
    { 0x5b83e4a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e4b4, 98 },
    { 0, 0 },
    { 0xc0c3e4b6, 23 },
    { 0x6ea3e4b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c3e4bb, 43 },
    { 0xa873e4bc, 90 },
    { 0, 0 },
    { 0x8543e4be, 88 },
    { 0x6fe3e4bf, 85 },
    { 0, 0 },
    { 0x5d23e4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5603e4c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd673e4c9, 23 },
    { 0x4c13e4ca, 90 },
    { 0x7763e4cb, 99 },
    { 0x4d3e4cc, 93 },
    { 0xb2e3e4cd, 80 },
    { 0x5323e4ce, 55 },
    { 0x6733e4cf, 33 },
    { 0xca33e4d0, 90 },
    { 0x4fb3e4cc, 93 },
    { 0x703e4cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e3e4da, 33 },
    { 0xbe83e4db, 55 },
    { 0x92f3e4db, 95 },
    { 0, 0 },
    { 0xfdd3e4de, 95 },
    { 0x11f3e4df, 95 },
    { 0x4693e4e0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df3e4e4, 99 },
    { 0x9f33e4e5, 33 },
    { 0xdf23e4e6, 98 },
    { 0xc673e4e4, 85 },
    { 0, 0 },
    { 0x4663e4e9, 95 },
    { 0xd773e4ea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce43e4f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e33e4f8, 33 },
    { 0x5663e4f9, 23 },
    { 0xe2d3e4f8, 100 },
    { 0x3d83e4fb, 23 },
    { 0, 0 },
    { 0xcfd3e4fd, 34 },
    { 0x9a3e4fe, 66 },
    { 0xab73e4fe, 43 },
    { 0, 0 },
    { 0x6ec3e501, 87 },
    { 0, 0 },
    { 0x3393e503, 90 },
    { 0x2de3e503, 76 },
    { 0xab03e505, 98 },
    { 0x7f53e506, 75 },
    { 0xb2d3e503, 43 },
    { 0xa13e508, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8013e50b, 52 },
    { 0xd1d3e50c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e3e511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff3e514, 85 },
    { 0x8813e515, 90 },
    { 0xb743e516, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023e51e, 95 },
    { 0x1fe3e51f, 78 },
    { 0, 0 },
    { 0x12a3e521, 76 },
    { 0xb3e3e521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e43e527, 84 },
    { 0xebd3e527, 23 },
    { 0xb9a3e529, 23 },
    { 0x833e52a, 90 },
    { 0x9f83e52b, 76 },
    { 0xbd03e52c, 64 },
    { 0xb493e52d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf803e530, 95 },
    { 0xf1e3e530, 90 },
    { 0x38c3e531, 95 },
    { 0x2a03e533, 76 },
    { 0, 0 },
    { 0x66a3e535, 23 },
    { 0x1e33e536, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c3e539, 85 },
    { 0x1d03e53a, 90 },
    { 0x1a93e53a, 90 },
    { 0x7313e539, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b3e53f, 61 },
    { 0xad63e540, 99 },
    { 0xd413e541, 98 },
    { 0x9f33e542, 95 },
    { 0x7553e543, 43 },
    { 0x2373e543, 23 },
    { 0x40a3e544, 76 },
    { 0xad03e546, 66 },
    { 0xfd03e546, 55 },
    { 0x29a3e53f, 84 },
    { 0x1683e549, 47 },
    { 0xe653e545, 55 },
    { 0, 0 },
    { 0x4853e54c, 46 },
    { 0xf833e54d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6673e551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a43e554, 77 },
    { 0x4033e555, 33 },
    { 0xd643e556, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1613e559, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f3e55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc573e55f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce33e56c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa933e573, 55 },
    { 0x9773e574, 92 },
    { 0x763e574, 23 },
    { 0x2eb3e576, 34 },
    { 0x5043e577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93e57c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e3e583, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf433e58b, 78 },
    { 0xe533e58c, 76 },
    { 0xc03e58b, 76 },
    { 0x8d83e58b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4993e596, 99 },
    { 0x8b53e596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d83e59a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3e59d, 33 },
    { 0xc393e59e, 64 },
    { 0x4f33e59f, 76 },
    { 0xcfa3e5a0, 33 },
    { 0xe63e5a1, 66 },
    { 0xadb3e5a2, 61 },
    { 0x88c3e5a3, 69 },
    { 0xb563e5a4, 23 },
    { 0x393e5a5, 90 },
    { 0x48a3e5a6, 90 },
    { 0, 0 },
    { 0x3883e5a8, 90 },
    { 0, 0 },
    { 0xf6d3e5aa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7673e5ad, 95 },
    { 0, 0 },
    { 0x8943e5af, 98 },
    { 0, 0 },
    { 0xf4a3e5b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed3e5b6, 83 },
    { 0, 0 },
    { 0xba73e5b8, 99 },
    { 0xa43e5b9, 95 },
    { 0, 0 },
    { 0x8f93e5bb, 90 },
    { 0, 0 },
    { 0x3083e5bd, 78 },
    { 0xd253e5be, 23 },
    { 0x3d13e5be, 52 },
    { 0, 0 },
    { 0xae13e5c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa3e5c8, 66 },
    { 0xf8c3e5c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6593e5cd, 61 },
    { 0xbb33e5ce, 76 },
    { 0, 0 },
    { 0xc563e5d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e3e5d4, 45 },
    { 0x4ae3e5d5, 61 },
    { 0, 0 },
    { 0xb053e5d7, 55 },
    { 0xdfc3e5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee3e5dc, 23 },
    { 0x6913e5dc, 99 },
    { 0, 0 },
    { 0x77a3e5df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e3e5e4, 81 },
    { 0x7033e5e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2983e5eb, 43 },
    { 0x4c13e5ec, 52 },
    { 0x43b3e5ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x653e5f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a3e5f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3e5fc, 61 },
    { 0, 0 },
    { 0xdea3e5fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb043e602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e3e608, 99 },
    { 0, 0 },
    { 0x7273e60a, 33 },
    { 0x94f3e60b, 34 },
    { 0, 0 },
    { 0xaeb3e60d, 65 },
    { 0x4933e60e, 48 },
    { 0x5f13e60f, 83 },
    { 0xf043e60d, 76 },
    { 0x1ba3e611, 90 },
    { 0x5d03e612, 77 },
    { 0x59a3e613, 43 },
    { 0x27f3e60f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1333e619, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f53e61c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f3e61f, 93 },
    { 0x8863e620, 90 },
    { 0x2a83e620, 81 },
    { 0xd1d3e622, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b3e625, 98 },
    { 0xb1d3e626, 23 },
    { 0x5113e627, 61 },
    { 0xf653e628, 55 },
    { 0xbca3e629, 61 },
    { 0x63b3e629, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7303e62e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe153e632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba3e635, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xda03e638, 68 },
    { 0xb7a3e639, 38 },
    { 0xae73e63a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e63d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f13e642, 76 },
    { 0xe83e642, 68 },
    { 0x9c33e644, 23 },
    { 0xb993e643, 99 },
    { 0x7a43e646, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe733e649, 76 },
    { 0x3af3e649, 76 },
    { 0x3b93e64b, 76 },
    { 0xb9a3e64a, 98 },
    { 0x3e93e64d, 61 },
    { 0, 0 },
    { 0xbf73e64f, 98 },
    { 0x6d73e650, 55 },
    { 0x9603e651, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7553e654, 34 },
    { 0x4033e655, 95 },
    { 0xeb83e654, 87 },
    { 0x2c03e657, 95 },
    { 0, 0 },
    { 0xc03e659, 55 },
    { 0xa8f3e65a, 23 },
    { 0x6933e65b, 34 },
    { 0, 0 },
    { 0x5cf3e65d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b93e661, 93 },
    { 0xaff3e662, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xee33e665, 17 },
    { 0xae93e666, 80 },
    { 0x68c3e666, 33 },
    { 0, 0 },
    { 0x6c43e669, 90 },
    { 0x3c73e66a, 93 },
    { 0x4593e66a, 33 },
    { 0x4b3e66c, 61 },
    { 0, 0 },
    { 0x1ab3e66e, 76 },
    { 0xffc3e66f, 98 },
    { 0x3c23e670, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc743e675, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d3e67e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e3e685, 76 },
    { 0x6b23e686, 23 },
    { 0x4073e687, 98 },
    { 0x9e73e688, 90 },
    { 0x9cb3e688, 61 },
    { 0x87d3e688, 23 },
    { 0x78c3e687, 95 },
    { 0xd923e68c, 46 },
    { 0, 0 },
    { 0x3723e68e, 61 },
    { 0x2353e68f, 23 },
    { 0xe9d3e68f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa3e698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3483e69d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb933e6a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c3e6a3, 90 },
    { 0x83a3e6a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7253e6a8, 95 },
    { 0xdba3e6a9, 23 },
    { 0xc313e6aa, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa3e6ae, 91 },
    { 0, 0 },
    { 0xc713e6b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc3e6b4, 90 },
    { 0xf083e6b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f3e6b8, 95 },
    { 0, 0 },
    { 0xcb23e6ba, 66 },
    { 0x8be3e6bb, 51 },
    { 0x2013e6ba, 90 },
    { 0, 0 },
    { 0x83f3e6be, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5693e6c1, 90 },
    { 0, 0 },
    { 0x8433e6c3, 38 },
    { 0x3e33e6c4, 52 },
    { 0x7713e6c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d43e6c8, 64 },
    { 0xd1a3e6c9, 67 },
    { 0x1073e6c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe223e6d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc333e6d8, 61 },
    { 0x59a3e6d9, 76 },
    { 0x37d3e6d9, 91 },
    { 0, 0 },
    { 0xef63e6dc, 52 },
    { 0x7d63e6dd, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b3e6e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2443e6e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f43e6eb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x5953e6ee, 81 },
    { 0xe823e6ee, 23 },
    { 0xb1e3e6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb3e6f4, 90 },
    { 0x2253e6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc883e6fa, 26 },
    { 0, 0 },
    { 0x18b3e6fc, 33 },
    { 0xf93e6fc, 95 },
    { 0xcfc3e6fe, 68 },
    { 0xa123e6ff, 61 },
    { 0, 0 },
    { 0xdbc3e701, 93 },
    { 0x8203e702, 93 },
    { 0, 0 },
    { 0xecc3e704, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3e707, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e70c, 65 },
    { 0x57e3e70d, 90 },
    { 0xc283e70d, 61 },
    { 0, 0 },
    { 0x4d23e710, 90 },
    { 0x3583e711, 67 },
    { 0x6783e711, 34 },
    { 0xfe03e713, 23 },
    { 0x6853e714, 95 },
    { 0xc703e715, 68 },
    { 0x2753e715, 98 },
    { 0xddf3e716, 46 },
    { 0xb813e715, 76 },
    { 0x1503e717, 17 },
    { 0x2ed3e718, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c73e724, 66 },
    { 0xe0b3e725, 33 },
    { 0x9823e726, 61 },
    { 0x1863e727, 99 },
    { 0xced3e726, 95 },
    { 0x2153e729, 52 },
    { 0xd9a3e724, 66 },
    { 0x2513e72a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8863e731, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf483e734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb73e737, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b3e73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b03e73f, 98 },
    { 0x4463e740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb553e747, 52 },
    { 0, 0 },
    { 0x7ae3e749, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2553e74d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b3e751, 90 },
    { 0xda23e752, 55 },
    { 0x5393e753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd083e75a, 34 },
    { 0x6ab3e75b, 23 },
    { 0x6ec3e75b, 33 },
    { 0x3773e75c, 34 },
    { 0xdaa3e75e, 61 },
    { 0x1293e75f, 93 },
    { 0xd4b3e760, 46 },
    { 0x2e73e760, 61 },
    { 0, 0 },
    { 0xf1f3e763, 55 },
    { 0x3263e763, 43 },
    { 0, 0 },
    { 0x1933e766, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d03e76a, 95 },
    { 0, 0 },
    { 0x1243e76c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c3e772, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13e776, 90 },
    { 0xf553e776, 99 },
    { 0x1963e776, 33 },
    { 0xd0b3e779, 23 },
    { 0x2823e77a, 85 },
    { 0xe7b3e778, 55 },
    { 0x96e3e778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a53e77f, 61 },
    { 0, 0 },
    { 0xbb63e781, 38 },
    { 0, 0 },
    { 0xbfb3e783, 95 },
    { 0x7e23e784, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7463e78c, 98 },
    { 0x1073e78d, 76 },
    { 0x8a13e78e, 98 },
    { 0x2693e78c, 99 },
    { 0xa683e790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f3e794, 90 },
    { 0xcb93e795, 27 },
    { 0x2243e796, 87 },
    { 0x5c23e796, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3e79c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e33e7a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a23e7a3, 66 },
    { 0, 0 },
    { 0x9883e7a5, 46 },
    { 0, 0 },
    { 0xfb13e7a7, 38 },
    { 0xffc3e7a8, 67 },
    { 0xd393e7a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff3e7b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9543e7b4, 43 },
    { 0x8473e7b4, 95 },
    { 0x12e3e7b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693e7c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f3e7c4, 34 },
    { 0x1903e7c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d83e7d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5013e7d7, 33 },
    { 0, 0 },
    { 0xb33e7d9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc33e7dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf03e7df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e3e7e2, 90 },
    { 0x433e7e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a3e7ee, 90 },
    { 0x6053e7ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3453e7f4, 80 },
    { 0x2bd3e7f5, 76 },
    { 0, 0 },
    { 0xc33e7f7, 61 },
    { 0xf7e3e7f7, 33 },
    { 0xf143e7f9, 93 },
    { 0xb3b3e7fa, 34 },
    { 0xdb13e7fb, 23 },
    { 0xd063e7fc, 90 },
    { 0x3f43e7fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8233e803, 23 },
    { 0x81e3e804, 61 },
    { 0, 0 },
    { 0x9fc3e806, 85 },
    { 0, 0 },
    { 0xf513e808, 43 },
    { 0x2803e808, 91 },
    { 0xae73e80a, 90 },
    { 0xbf73e808, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4363e80f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f23e813, 23 },
    { 0xf53e813, 61 },
    { 0xe703e815, 34 },
    { 0xcea3e815, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb3e819, 23 },
    { 0x3d43e81a, 95 },
    { 0x71c3e81b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a43e81e, 33 },
    { 0x71c3e81e, 78 },
    { 0, 0 },
    { 0x783e821, 90 },
    { 0x71f3e821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c3e827, 90 },
    { 0, 0 },
    { 0x9413e829, 23 },
    { 0x1fd3e829, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd23e82e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3e83d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3e845, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d33e84b, 98 },
    { 0x7163e84c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9943e850, 90 },
    { 0x51e3e850, 84 },
    { 0, 0 },
    { 0x9f33e853, 33 },
    { 0xa0a3e854, 91 },
    { 0, 0 },
    { 0xa243e856, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d3e85b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd603e860, 76 },
    { 0xb2c3e861, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef63e865, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd3e869, 43 },
    { 0, 0 },
    { 0x8813e86b, 84 },
    { 0xbb53e86c, 55 },
    { 0x6ae3e86c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b3e870, 66 },
    { 0x8863e871, 78 },
    { 0x4823e871, 98 },
    { 0xddd3e873, 95 },
    { 0x6133e873, 75 },
    { 0xf13e873, 34 },
    { 0x9623e874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e3e87b, 90 },
    { 0x25c3e87c, 43 },
    { 0xcbc3e87c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1443e885, 52 },
    { 0x5df3e885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6713e889, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c73e88d, 33 },
    { 0x6653e88d, 50 },
    { 0xb823e88f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b13e895, 46 },
    { 0x39b3e896, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33e899, 61 },
    { 0xbbe3e89a, 38 },
    { 0, 0 },
    { 0xf43e89c, 95 },
    { 0x5533e89c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d53e8a2, 34 },
    { 0x2823e8a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023e8a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd83e8a9, 38 },
    { 0x7d03e8aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c63e8ad, 90 },
    { 0x62d3e8ad, 96 },
    { 0, 0 },
    { 0x9993e8b0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c3e8b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae13e8b6, 17 },
    { 0x7df3e8b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c3e8bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8503e8c6, 55 },
    { 0xdaa3e8c6, 67 },
    { 0xe963e8c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e8d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6013e8de, 47 },
    { 0xdc53e8df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b23e8e4, 99 },
    { 0x8bc3e8e5, 23 },
    { 0xf6a3e8e4, 33 },
    { 0xe373e8e7, 23 },
    { 0, 0 },
    { 0x69b3e8e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a43e8ec, 90 },
    { 0, 0 },
    { 0xeb53e8ee, 99 },
    { 0x35e3e8ef, 23 },
    { 0x6473e8ef, 90 },
    { 0, 0 },
    { 0x7543e8f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8223e8f8, 98 },
    { 0x9413e8f8, 76 },
    { 0xc073e8fa, 66 },
    { 0xdac3e8fa, 95 },
    { 0x5f63e8fb, 98 },
    { 0xcd83e8f9, 66 },
    { 0xc213e8fe, 33 },
    { 0, 0 },
    { 0xecf3e900, 61 },
    { 0xa313e901, 88 },
    { 0x9bd3e902, 76 },
    { 0xdeb3e903, 55 },
    { 0x8ff3e901, 56 },
    { 0xfdf3e903, 67 },
    { 0xdd03e902, 85 },
    { 0x8e03e901, 93 },
    { 0, 0 },
    { 0xe143e909, 98 },
    { 0, 0 },
    { 0x15e3e90b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb13e90f, 61 },
    { 0, 0 },
    { 0xfaa3e911, 46 },
    { 0xafb3e911, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f3e91c, 90 },
    { 0x2fd3e91d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5943e920, 95 },
    { 0xe873e920, 61 },
    { 0xe083e922, 61 },
    { 0, 0 },
    { 0x5a23e924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba3e928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe203e92f, 46 },
    { 0x8e93e930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e3e934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f43e938, 90 },
    { 0x6663e938, 98 },
    { 0x53c3e93a, 33 },
    { 0, 0 },
    { 0x2dc3e93c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff3e941, 76 },
    { 0x38b3e941, 80 },
    { 0xe543e941, 95 },
    { 0, 0 },
    { 0xf413e945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3e948, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa3e94d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e63e952, 56 },
    { 0x1ea3e952, 34 },
    { 0x65a3e954, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c3e958, 87 },
    { 0xc153e958, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc3e95c, 67 },
    { 0, 0 },
    { 0xf3a3e95e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a3e964, 95 },
    { 0, 0 },
    { 0x49d3e966, 90 },
    { 0x783e966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d3e96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3e96f, 33 },
    { 0xb343e96f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423e974, 17 },
    { 0xffa3e975, 67 },
    { 0xd943e974, 55 },
    { 0, 0 },
    { 0x8623e978, 95 },
    { 0x78f3e978, 98 },
    { 0x1413e97a, 91 },
    { 0xd373e97b, 90 },
    { 0x2523e97c, 55 },
    { 0, 0 },
    { 0x7253e97e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc3e981, 68 },
    { 0x9283e982, 61 },
    { 0x9aa3e981, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e98b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e990, 48 },
    { 0, 0 },
    { 0xe3e992, 98 },
    { 0, 0 },
    { 0x3b63e994, 55 },
    { 0x7b63e995, 52 },
    { 0xee43e994, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa573e9a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea3e9a8, 98 },
    { 0, 0 },
    { 0x89e3e9aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f3e9b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f3e9b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb443e9bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db3e9c0, 95 },
    { 0x95a3e9c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc3e9c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83e9ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e9ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd543e9db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003e9e4, 23 },
    { 0x5cf3e9e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f33e9ea, 90 },
    { 0, 0 },
    { 0x5663e9ec, 78 },
    { 0x253e9ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e3e9f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b3e9f3, 90 },
    { 0x2e3e9f4, 85 },
    { 0x60c3e9f4, 90 },
    { 0x8503e9f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac43ea04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f53ea08, 61 },
    { 0x41f3ea09, 100 },
    { 0xa5e3ea0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa3ea0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4853ea11, 66 },
    { 0x5cc3ea12, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03ea16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4763ea19, 78 },
    { 0x7b73ea1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5863ea24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe713ea28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3ea2b, 61 },
    { 0xbd63ea2c, 68 },
    { 0x12d3ea2d, 66 },
    { 0xa463ea2c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7103ea32, 78 },
    { 0, 0 },
    { 0x2823ea34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4483ea38, 23 },
    { 0x4cb3ea39, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc3ea41, 33 },
    { 0x4db3ea42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c03ea46, 34 },
    { 0x4ad3ea46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead3ea4b, 78 },
    { 0x5c93ea4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a73ea4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc43ea52, 48 },
    { 0xb893ea52, 43 },
    { 0x5c63ea54, 43 },
    { 0, 0 },
    { 0x4013ea56, 98 },
    { 0xa833ea57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a3ea5c, 93 },
    { 0x6fc3ea5d, 78 },
    { 0xc843ea5e, 23 },
    { 0x4753ea5f, 46 },
    { 0xbd13ea5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df3ea63, 33 },
    { 0, 0 },
    { 0x24a3ea65, 44 },
    { 0x5583ea66, 78 },
    { 0, 0 },
    { 0xb8f3ea68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4803ea71, 99 },
    { 0x5193ea72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe3ea76, 90 },
    { 0, 0 },
    { 0x3d3ea78, 40 },
    { 0, 0 },
    { 0xe33ea7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec23ea7e, 95 },
    { 0xc9a3ea7e, 90 },
    { 0, 0 },
    { 0x3f83ea81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af3ea85, 87 },
    { 0x5333ea85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb3ea8b, 61 },
    { 0xcb33ea8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3ea95, 95 },
    { 0x9923ea95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc3ea9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f3eaa0, 99 },
    { 0x69f3eaa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd43eaa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4903eaab, 23 },
    { 0xf763eaac, 98 },
    { 0x3ff3eaad, 90 },
    { 0xc323eaae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a73eab1, 55 },
    { 0x5d33eab2, 43 },
    { 0xa9d3eab3, 95 },
    { 0xeb13eab4, 66 },
    { 0xfd33eab5, 61 },
    { 0, 0 },
    { 0x9063eab7, 33 },
    { 0x293eab8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43eabb, 23 },
    { 0xaf03eabc, 76 },
    { 0xd9f3eabb, 98 },
    { 0x43f3eabd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa13eac3, 61 },
    { 0x513eac4, 61 },
    { 0x1f33eac3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e3eac9, 90 },
    { 0xe003eaca, 61 },
    { 0, 0 },
    { 0x1ad3eacc, 95 },
    { 0, 0 },
    { 0xfd23eace, 34 },
    { 0, 0 },
    { 0x2583ead0, 61 },
    { 0xba23ead0, 98 },
    { 0xbed3ead2, 34 },
    { 0x8143ead3, 95 },
    { 0xaad3ead4, 90 },
    { 0, 0 },
    { 0xbd03ead6, 98 },
    { 0xce63ead6, 33 },
    { 0xd843ead6, 33 },
    { 0x6743ead8, 46 },
    { 0, 0 },
    { 0x3e63eadb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c3eae3, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x1033eae6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc33eaed, 84 },
    { 0x5353eaee, 55 },
    { 0xace3eaef, 33 },
    { 0x8863eaee, 77 },
    { 0x6b53eaf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e43eaf5, 95 },
    { 0xb3a3eaf5, 90 },
    { 0xbd73eaf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353eafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a03eb01, 61 },
    { 0x1963eb01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53eb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b3eb0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed3eb13, 33 },
    { 0xf243eb14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce3eb1f, 33 },
    { 0, 0 },
    { 0x7af3eb21, 76 },
    { 0x86c3eb22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4173eb25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb3eb2c, 98 },
    { 0, 0 },
    { 0xa593eb2e, 95 },
    { 0x9f63eb2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f3eb32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba3eb35, 33 },
    { 0, 0 },
    { 0x193eb37, 33 },
    { 0x7843eb38, 69 },
    { 0x4d63eb37, 46 },
    { 0xc663eb38, 78 },
    { 0xddf3eb3b, 61 },
    { 0, 0 },
    { 0x4b43eb3d, 43 },
    { 0x7e43eb3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8683eb41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb3eb46, 66 },
    { 0x7283eb46, 100 },
    { 0x1e33eb48, 91 },
    { 0, 0 },
    { 0x54d3eb4a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb883eb51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab03eb56, 23 },
    { 0x8673eb56, 61 },
    { 0x6e43eb58, 61 },
    { 0xb2d3eb58, 33 },
    { 0x7663eb5a, 23 },
    { 0x8863eb5b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb743eb5e, 95 },
    { 0xfdd3eb5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f3eb64, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed3eb68, 85 },
    { 0, 0 },
    { 0xac73eb6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3eb71, 76 },
    { 0x6f13eb71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f3eb75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4203eb79, 90 },
    { 0xb993eb7a, 55 },
    { 0x20f3eb7b, 100 },
    { 0x84e3eb7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b3eb84, 98 },
    { 0, 0 },
    { 0xa863eb86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2033eb8a, 55 },
    { 0, 0 },
    { 0x8b83eb8c, 98 },
    { 0xf523eb8d, 67 },
    { 0x15f3eb8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e13eb95, 90 },
    { 0, 0 },
    { 0x3223eb97, 90 },
    { 0, 0 },
    { 0x5063eb99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b3eb9c, 23 },
    { 0x44a3eb9d, 95 },
    { 0x66c3eb9e, 90 },
    { 0x2693eb9e, 33 },
    { 0xe0b3eba0, 78 },
    { 0xb9b3eba0, 95 },
    { 0xce33eba0, 99 },
    { 0, 0 },
    { 0x65d3eba4, 95 },
    { 0x5423eba5, 95 },
    { 0x7143eba4, 98 },
    { 0x4e73eba7, 61 },
    { 0x80a3eba8, 85 },
    { 0x4113eba9, 23 },
    { 0, 0 },
    { 0x7b73ebab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ebae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3423ebb1, 68 },
    { 0x4bf3ebb2, 33 },
    { 0xad23ebb2, 90 },
    { 0xb083ebb4, 85 },
    { 0x4763ebb2, 55 },
    { 0, 0 },
    { 0x27c3ebb7, 94 },
    { 0x4d53ebb8, 90 },
    { 0x4e53ebb8, 98 },
    { 0x6193ebba, 43 },
    { 0, 0 },
    { 0xd7b3ebbc, 55 },
    { 0xec43ebbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb833ebc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d3ebcf, 78 },
    { 0xdda3ebcf, 90 },
    { 0, 0 },
    { 0xc6e3ebd2, 95 },
    { 0x9ec3ebd3, 94 },
    { 0, 0 },
    { 0x80d3ebd5, 88 },
    { 0, 0 },
    { 0xc13ebd7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a73ebdc, 93 },
    { 0x1f63ebdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d3ebe0, 33 },
    { 0x9263ebe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93ebe6, 78 },
    { 0, 0 },
    { 0x5863ebe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe533ebee, 98 },
    { 0x76b3ebee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe653ebf4, 61 },
    { 0xc503ebf5, 52 },
    { 0x5cd3ebf6, 100 },
    { 0xe193ebf7, 61 },
    { 0x9613ebf7, 98 },
    { 0xb243ebf9, 61 },
    { 0x2193ebfa, 93 },
    { 0xed03ebfa, 33 },
    { 0x6203ebf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf703ec04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xece3ec07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb513ec0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553ec13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5553ec19, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3143ec21, 90 },
    { 0x8f13ec21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3383ec26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb53ec29, 90 },
    { 0, 0 },
    { 0x5993ec2b, 66 },
    { 0x30f3ec2c, 95 },
    { 0xb023ec2c, 34 },
    { 0x2fd3ec2d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b3ec35, 98 },
    { 0x6163ec36, 34 },
    { 0x1e03ec37, 101 },
    { 0xecd3ec38, 95 },
    { 0, 0 },
    { 0x8783ec3a, 65 },
    { 0x7be3ec3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa403ec3f, 23 },
    { 0xa503ec40, 86 },
    { 0x77a3ec41, 23 },
    { 0, 0 },
    { 0xf6d3ec43, 90 },
    { 0xc203ec44, 33 },
    { 0xac03ec45, 95 },
    { 0, 0 },
    { 0xdef3ec47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3083ec4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d3ec57, 95 },
    { 0xdf73ec57, 34 },
    { 0x7d93ec59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e93ec64, 90 },
    { 0x4bf3ec65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b3ec6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b83ec72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd93ec76, 91 },
    { 0, 0 },
    { 0x3133ec78, 90 },
    { 0, 0 },
    { 0x2983ec7a, 61 },
    { 0xdd23ec7b, 98 },
    { 0, 0 },
    { 0x8343ec7d, 36 },
    { 0x9ad3ec7e, 33 },
    { 0, 0 },
    { 0x9563ec80, 61 },
    { 0x80f3ec81, 78 },
    { 0x1d83ec80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a83ec87, 20 },
    { 0xbb13ec88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd3ec8e, 52 },
    { 0xa43ec8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b3ec93, 80 },
    { 0xc8a3ec93, 12 },
    { 0x4143ec95, 76 },
    { 0, 0 },
    { 0x8dd3ec97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b93ec9e, 100 },
    { 0x4dd3ec9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a3eca2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a83ecab, 76 },
    { 0xcab3ecab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4353ecb1, 90 },
    { 0, 0 },
    { 0x2bb3ecb3, 90 },
    { 0x76b3ecb4, 33 },
    { 0xf393ecb5, 100 },
    { 0xa633ecb5, 61 },
    { 0x6b43ecb7, 33 },
    { 0xbf83ecb7, 77 },
    { 0x7373ecb5, 66 },
    { 0x7823ecb7, 93 },
    { 0x6e33ecba, 93 },
    { 0, 0 },
    { 0x85e3ecbd, 78 },
    { 0x60c3ecbe, 88 },
    { 0x3a3ecbf, 90 },
    { 0x86a3ecc0, 76 },
    { 0x7363ecc1, 87 },
    { 0x43c3ecc2, 90 },
    { 0x70a3ecc3, 61 },
    { 0x8f83ecc1, 33 },
    { 0, 0 },
    { 0xf483ecc6, 90 },
    { 0x1503ecc7, 87 },
    { 0x483ecc6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9353ecd1, 98 },
    { 0xb43ecd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa833ecd5, 76 },
    { 0x1893ecd6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a3ecde, 23 },
    { 0x4d53ecdf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f3ece3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf23ece9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc3ecef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3193ecf6, 90 },
    { 0x5923ecf6, 96 },
    { 0, 0 },
    { 0x8e63ecf9, 33 },
    { 0x3ecf9, 61 },
    { 0, 0 },
    { 0xfd23ecfc, 78 },
    { 0xd2b3ecfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f3ed02, 33 },
    { 0xba63ed02, 95 },
    { 0, 0 },
    { 0xd3ed05, 36 },
    { 0xd513ed06, 23 },
    { 0, 0 },
    { 0x74a3ed08, 90 },
    { 0x2e83ed08, 38 },
    { 0, 0 },
    { 0x1143ed0b, 95 },
    { 0, 0 },
    { 0x1893ed0d, 95 },
    { 0x2d93ed0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff3ed14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e3ed1b, 98 },
    { 0, 0 },
    { 0x3d23ed1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c3ed22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd853ed25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8933ed2b, 23 },
    { 0x86e3ed2b, 34 },
    { 0x9de3ed2b, 23 },
    { 0x7453ed2e, 61 },
    { 0xea3ed2f, 87 },
    { 0x3433ed2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3563ed3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a23ed3e, 91 },
    { 0x5c03ed3f, 95 },
    { 0x2da3ed40, 99 },
    { 0, 0 },
    { 0x5c23ed42, 76 },
    { 0x1093ed43, 61 },
    { 0, 0 },
    { 0x7b13ed45, 95 },
    { 0, 0 },
    { 0x2d53ed47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b3ed4b, 76 },
    { 0x1343ed4c, 98 },
    { 0xbe13ed4c, 99 },
    { 0xd103ed4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf383ed52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2313ed55, 34 },
    { 0x5f93ed55, 90 },
    { 0x8e53ed57, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e93ed5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f3ed63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef13ed71, 85 },
    { 0, 0 },
    { 0xed13ed73, 90 },
    { 0, 0 },
    { 0x4dc3ed75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd403ed79, 34 },
    { 0x6483ed79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d3ed7d, 90 },
    { 0x76e3ed7e, 61 },
    { 0x8c93ed7e, 23 },
    { 0x2403ed80, 85 },
    { 0, 0 },
    { 0xdef3ed82, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd3ed87, 61 },
    { 0x6823ed88, 61 },
    { 0x3c13ed89, 98 },
    { 0xc343ed89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7083ed8d, 52 },
    { 0x4e73ed8e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a3ed97, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e3ed9c, 95 },
    { 0, 0 },
    { 0x123ed9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb063eda2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7273eda5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab3eda8, 77 },
    { 0x75f3eda8, 23 },
    { 0, 0 },
    { 0xc333edab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe3edaf, 67 },
    { 0xcaf3edb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1093edb3, 90 },
    { 0, 0 },
    { 0x6fd3edb5, 95 },
    { 0, 0 },
    { 0x81b3edb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1603edba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c3edc0, 45 },
    { 0, 0 },
    { 0xc943edc2, 56 },
    { 0, 0 },
    { 0xea73edc4, 98 },
    { 0, 0 },
    { 0x24e3edc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba3edc9, 95 },
    { 0x7db3edca, 87 },
    { 0xec23edcb, 93 },
    { 0x2d13edca, 48 },
    { 0xa0f3edcd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb93edd2, 37 },
    { 0x1943edd3, 34 },
    { 0xc933edd4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3eddb, 91 },
    { 0, 0 },
    { 0xdb33eddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb473ede6, 33 },
    { 0xad43ede6, 33 },
    { 0x8693ede8, 55 },
    { 0x4a53ede8, 93 },
    { 0xe083ede8, 75 },
    { 0x7183ede7, 33 },
    { 0x4843edec, 96 },
    { 0x8403edec, 34 },
    { 0x1ac3eded, 33 },
    { 0x9593edef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc693edf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f3edf8, 95 },
    { 0xb213edf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c3edfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7963ee01, 61 },
    { 0, 0 },
    { 0x5773ee03, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc613ee07, 61 },
    { 0x9c13ee07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb33ee0b, 93 },
    { 0x3673ee0b, 90 },
    { 0x2ed3ee0d, 90 },
    { 0, 0 },
    { 0x99b3ee0f, 88 },
    { 0, 0 },
    { 0x1203ee11, 90 },
    { 0xb9d3ee11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be3ee15, 55 },
    { 0, 0 },
    { 0x7f13ee17, 60 },
    { 0xfb73ee17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee83ee1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e3ee24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda43ee29, 91 },
    { 0xf233ee2a, 55 },
    { 0x8733ee2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd143ee30, 46 },
    { 0x7cb3ee30, 55 },
    { 0, 0 },
    { 0x3233ee33, 90 },
    { 0xcb3ee34, 95 },
    { 0x1483ee34, 76 },
    { 0, 0 },
    { 0xda93ee37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5373ee3f, 90 },
    { 0xb383ee40, 95 },
    { 0x323ee40, 78 },
    { 0xca53ee42, 90 },
    { 0x1173ee41, 95 },
    { 0x4923ee40, 52 },
    { 0, 0 },
    { 0x3e63ee46, 61 },
    { 0xc9c3ee46, 100 },
    { 0x4f73ee48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5903ee4d, 68 },
    { 0, 0 },
    { 0x80c3ee4f, 92 },
    { 0, 0 },
    { 0x84a3ee51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993ee55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5653ee58, 33 },
    { 0x9333ee58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e3ee62, 87 },
    { 0x213ee63, 58 },
    { 0x9603ee62, 95 },
    { 0, 0 },
    { 0x5dc3ee66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb323ee69, 90 },
    { 0x8463ee69, 98 },
    { 0x853ee6a, 61 },
    { 0x4e53ee6a, 43 },
    { 0, 0 },
    { 0x9733ee6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393ee75, 33 },
    { 0x4a93ee76, 23 },
    { 0x57f3ee77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8883ee7c, 52 },
    { 0x6a23ee7c, 33 },
    { 0xd8b3ee7e, 98 },
    { 0x3af3ee7f, 95 },
    { 0, 0 },
    { 0xe0c3ee81, 95 },
    { 0, 0 },
    { 0x1423ee83, 46 },
    { 0x4c33ee83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d3ee8a, 98 },
    { 0x8023ee8b, 43 },
    { 0xa983ee8c, 61 },
    { 0xeaa3ee8d, 84 },
    { 0xdae3ee8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc843ee94, 61 },
    { 0, 0 },
    { 0x1f73ee96, 98 },
    { 0, 0 },
    { 0xef23ee98, 33 },
    { 0x2563ee99, 33 },
    { 0xbac3ee99, 66 },
    { 0x2643ee9a, 12 },
    { 0, 0 },
    { 0x5cb3ee9d, 90 },
    { 0x2e33ee9e, 98 },
    { 0xf93ee9d, 90 },
    { 0xf0e3ee9f, 90 },
    { 0, 0 },
    { 0x5073eea2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e73eea5, 80 },
    { 0x9253eea5, 55 },
    { 0, 0 },
    { 0xe4f3eea8, 98 },
    { 0x1633eea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3eeb8, 84 },
    { 0x5f83eeb9, 95 },
    { 0xc3c3eeba, 98 },
    { 0xc2f3eeba, 78 },
    { 0x6113eebc, 77 },
    { 0, 0 },
    { 0xb603eebe, 46 },
    { 0, 0 },
    { 0xcff3eec0, 95 },
    { 0, 0 },
    { 0x54f3eec2, 34 },
    { 0x2443eec2, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc3eec6, 88 },
    { 0, 0 },
    { 0x9353eec8, 76 },
    { 0xbb83eec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d3eecf, 61 },
    { 0x4143eecf, 33 },
    { 0xfa3eed0, 95 },
    { 0x1953eed2, 91 },
    { 0x9743eed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3eed6, 99 },
    { 0x6403eed7, 90 },
    { 0, 0 },
    { 0x9853eed9, 90 },
    { 0x473eeda, 76 },
    { 0xb033eedb, 46 },
    { 0xb2d3eedb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e13eedf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc43eee5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d73eee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb3eef1, 61 },
    { 0, 0 },
    { 0xd763eef3, 76 },
    { 0x3713eef3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd223eef9, 98 },
    { 0x7063eefa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8883eefd, 34 },
    { 0x7353eefe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f13ef0a, 23 },
    { 0, 0 },
    { 0x1fa3ef0c, 55 },
    { 0xcb83ef0d, 52 },
    { 0, 0 },
    { 0xd903ef0f, 61 },
    { 0x62d3ef0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c53ef13, 78 },
    { 0, 0 },
    { 0x3fb3ef15, 9 },
    { 0xfa43ef16, 98 },
    { 0xdca3ef15, 95 },
    { 0, 0 },
    { 0x15d3ef19, 98 },
    { 0x44c3ef1a, 93 },
    { 0xa373ef1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf3ef25, 93 },
    { 0xf943ef26, 85 },
    { 0, 0 },
    { 0xdd83ef28, 65 },
    { 0x7103ef28, 76 },
    { 0, 0 },
    { 0x4403ef2b, 78 },
    { 0xa803ef2b, 52 },
    { 0, 0 },
    { 0xafb3ef2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2573ef32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1103ef35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7653ef38, 33 },
    { 0x7f03ef38, 77 },
    { 0xdd73ef3a, 61 },
    { 0, 0 },
    { 0x5623ef3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb3ef42, 61 },
    { 0, 0 },
    { 0xd493ef44, 61 },
    { 0x14a3ef44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2653ef4c, 33 },
    { 0x1bf3ef4c, 90 },
    { 0xd143ef4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd3ef55, 55 },
    { 0x82f3ef56, 61 },
    { 0xa133ef55, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3ef5a, 90 },
    { 0, 0 },
    { 0x6353ef5c, 17 },
    { 0x7b3ef5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd3ef65, 23 },
    { 0, 0 },
    { 0x1e83ef67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a93ef6b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae3ef6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d3ef78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf413ef7b, 90 },
    { 0x4e63ef7c, 76 },
    { 0, 0 },
    { 0xdab3ef7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3ef83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc73ef87, 78 },
    { 0, 0 },
    { 0x15d3ef89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5623ef8d, 90 },
    { 0, 0 },
    { 0xaf93ef8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb03ef92, 80 },
    { 0x7d43ef93, 68 },
    { 0, 0 },
    { 0x3a43ef95, 99 },
    { 0x2383ef96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff3ef9e, 42 },
    { 0x8653ef9f, 61 },
    { 0x99c3ef9f, 95 },
    { 0xcf13efa1, 90 },
    { 0x34a3ef9f, 23 },
    { 0x9363efa2, 23 },
    { 0xa9f3efa1, 33 },
    { 0x7603efa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e63efa9, 76 },
    { 0, 0 },
    { 0xaee3efab, 43 },
    { 0xbc83efab, 67 },
    { 0xc4c3efad, 95 },
    { 0, 0 },
    { 0x33c3efaf, 100 },
    { 0, 0 },
    { 0x2083efb1, 33 },
    { 0xfd83efb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4053efb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab3efb9, 95 },
    { 0x33e3efba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe33efc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8943efc8, 46 },
    { 0xc333efc9, 61 },
    { 0xcd23efca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4603efd1, 33 },
    { 0xc5e3efd2, 93 },
    { 0x2bd3efd3, 95 },
    { 0xe823efd1, 76 },
    { 0xbc3efd3, 55 },
    { 0x92d3efd1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a43efd9, 99 },
    { 0, 0 },
    { 0x5203efdb, 93 },
    { 0x2143efdc, 90 },
    { 0, 0 },
    { 0x7543efde, 76 },
    { 0, 0 },
    { 0x8033efe0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb973efe3, 66 },
    { 0, 0 },
    { 0xedd3efe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc63efea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2993efee, 46 },
    { 0x62a3efef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc03effb, 55 },
    { 0xbca3effb, 67 },
    { 0x4bc3effd, 23 },
    { 0, 0 },
    { 0x5643efff, 61 },
    { 0x8ef3f000, 23 },
    { 0xc8f3efff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba3f007, 46 },
    { 0x53a3f008, 33 },
    { 0xb5a3f007, 90 },
    { 0xd2c3f008, 24 },
    { 0xa913f00b, 99 },
    { 0x5253f00b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f016, 66 },
    { 0x38b3f016, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b13f01b, 88 },
    { 0x5363f01c, 85 },
    { 0xe53f01d, 87 },
    { 0xa593f01e, 76 },
    { 0xaed3f01b, 69 },
    { 0xc2e3f01e, 90 },
    { 0xc23f01d, 95 },
    { 0xb6c3f022, 95 },
    { 0x6693f023, 61 },
    { 0xd5e3f024, 92 },
    { 0x7423f024, 95 },
    { 0xa313f026, 95 },
    { 0x393f027, 98 },
    { 0xb873f01c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e3f02b, 52 },
    { 0xcea3f02c, 88 },
    { 0xd8c3f02c, 95 },
    { 0, 0 },
    { 0x4a23f02f, 90 },
    { 0x2ed3f030, 61 },
    { 0, 0 },
    { 0x3cb3f032, 77 },
    { 0x93b3f033, 61 },
    { 0x7243f033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a43f038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce3f03c, 90 },
    { 0x8b23f03d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a63f040, 90 },
    { 0x4b73f041, 34 },
    { 0xa6f3f041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d3f046, 23 },
    { 0x4ef3f046, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8243f04b, 99 },
    { 0, 0 },
    { 0x1223f04d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e3f052, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x9653f055, 76 },
    { 0xe433f056, 94 },
    { 0xc4f3f055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9863f05b, 33 },
    { 0xb2e3f05b, 61 },
    { 0, 0 },
    { 0xcbb3f05e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x673f061, 34 },
    { 0, 0 },
    { 0xbb33f063, 90 },
    { 0, 0 },
    { 0xa3a3f065, 85 },
    { 0x7743f066, 80 },
    { 0x7653f067, 53 },
    { 0, 0 },
    { 0xff83f069, 23 },
    { 0xb723f06a, 51 },
    { 0x6573f069, 24 },
    { 0, 0 },
    { 0xcf43f06d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f3f070, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf3f073, 93 },
    { 0x46e3f073, 33 },
    { 0, 0 },
    { 0x5463f076, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a3f079, 23 },
    { 0, 0 },
    { 0xede3f07b, 93 },
    { 0x5d13f07c, 75 },
    { 0x4823f07c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a23f084, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c3f08f, 61 },
    { 0x7dd3f090, 61 },
    { 0, 0 },
    { 0x1c23f092, 33 },
    { 0x3f73f093, 90 },
    { 0x8b3f093, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a3f098, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3f09b, 93 },
    { 0, 0 },
    { 0x27d3f09d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c63f0a3, 61 },
    { 0, 0 },
    { 0xd63f0a5, 27 },
    { 0x8e73f0a5, 33 },
    { 0x14b3f0a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed3f0aa, 75 },
    { 0x6793f0ab, 61 },
    { 0x3073f0ac, 67 },
    { 0xdb83f0ad, 66 },
    { 0xb423f0ae, 46 },
    { 0xf323f0ad, 85 },
    { 0xe1d3f0ae, 95 },
    { 0x3a3f0b1, 90 },
    { 0xd193f0b1, 90 },
    { 0, 0 },
    { 0xba33f0b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3183f0ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c3f0bd, 61 },
    { 0x673f0be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb33f0c7, 81 },
    { 0, 0 },
    { 0x8413f0c9, 66 },
    { 0x603f0ca, 46 },
    { 0x94a3f0cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd153f0ce, 81 },
    { 0x7493f0cf, 34 },
    { 0x4413f0d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6563f0d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e33f0db, 88 },
    { 0, 0 },
    { 0x77a3f0dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a3f0e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f0e9, 34 },
    { 0, 0 },
    { 0xd7a3f0eb, 34 },
    { 0xa843f0eb, 95 },
    { 0xb833f0ed, 90 },
    { 0xb4b3f0ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d23f0f7, 90 },
    { 0, 0 },
    { 0xe023f0f9, 95 },
    { 0, 0 },
    { 0xf603f0fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf3f0ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb123f108, 61 },
    { 0, 0 },
    { 0x63f10a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e3f10d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2543f118, 76 },
    { 0xd7f3f119, 56 },
    { 0x5e3f119, 34 },
    { 0xed63f11b, 90 },
    { 0x90f3f119, 90 },
    { 0xa523f119, 55 },
    { 0xd193f11a, 46 },
    { 0, 0 },
    { 0xd943f120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d3f124, 76 },
    { 0, 0 },
    { 0xfd3f126, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe103f129, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d83f12d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d3f130, 75 },
    { 0x1b93f131, 95 },
    { 0, 0 },
    { 0xa473f133, 90 },
    { 0x8063f134, 61 },
    { 0xb2e3f135, 46 },
    { 0, 0 },
    { 0x2b23f137, 78 },
    { 0xff33f137, 55 },
    { 0, 0 },
    { 0xbeb3f13a, 98 },
    { 0x52a3f13a, 98 },
    { 0x77a3f13c, 66 },
    { 0x123f13b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea13f142, 93 },
    { 0xc73f142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e03f147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd083f14e, 90 },
    { 0x1913f14f, 95 },
    { 0xcf63f150, 76 },
    { 0, 0 },
    { 0xae63f152, 90 },
    { 0, 0 },
    { 0x2d83f154, 52 },
    { 0x70a3f155, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e3f158, 76 },
    { 0x1bf3f158, 85 },
    { 0xed53f159, 91 },
    { 0, 0 },
    { 0xb863f15c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6873f160, 36 },
    { 0x343f161, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d3f16b, 23 },
    { 0xc3b3f16c, 76 },
    { 0, 0 },
    { 0x47a3f16e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723f173, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6863f177, 23 },
    { 0x4003f178, 90 },
    { 0xbe33f179, 61 },
    { 0, 0 },
    { 0x5d83f17b, 85 },
    { 0xd0e3f17c, 17 },
    { 0, 0 },
    { 0x99e3f17e, 17 },
    { 0x95d3f17f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c3f182, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc73f18d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a3f190, 61 },
    { 0xed43f191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a83f194, 90 },
    { 0x88d3f194, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf823f198, 77 },
    { 0xf973f199, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc623f19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d3f1a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5483f1a6, 55 },
    { 0, 0 },
    { 0x7c13f1a8, 100 },
    { 0x1ae3f1a9, 87 },
    { 0xd213f1aa, 34 },
    { 0xf7c3f1a9, 90 },
    { 0x8033f1ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3f1b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc323f1b4, 33 },
    { 0x8f73f1b4, 81 },
    { 0x893f1b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf3f1b9, 99 },
    { 0xc1a3f1ba, 23 },
    { 0x4d43f1bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda33f1be, 76 },
    { 0x8f33f1bf, 95 },
    { 0xb753f1c0, 23 },
    { 0x4723f1c1, 66 },
    { 0xf913f1be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f3f1c6, 43 },
    { 0x2433f1c7, 68 },
    { 0xd8c3f1c7, 98 },
    { 0, 0 },
    { 0x20f3f1ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93f1cd, 61 },
    { 0xcd33f1cd, 33 },
    { 0x7313f1ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d3f1d4, 87 },
    { 0x40e3f1d5, 90 },
    { 0x8013f1d6, 43 },
    { 0x1a93f1d7, 98 },
    { 0, 0 },
    { 0xd523f1d9, 61 },
    { 0xa813f1d9, 90 },
    { 0, 0 },
    { 0x7633f1dc, 23 },
    { 0xcc3f1dd, 66 },
    { 0x803f1de, 78 },
    { 0x8313f1df, 33 },
    { 0xabe3f1de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d3f1e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5893f1ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e33f1f1, 33 },
    { 0x8203f1f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd203f1f7, 76 },
    { 0x873f1f8, 34 },
    { 0x53a3f1f8, 61 },
    { 0, 0 },
    { 0x8c43f1fb, 76 },
    { 0x6643f1fc, 90 },
    { 0xef83f1fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb93f203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf23f206, 76 },
    { 0xff3f207, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd3f20a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3f20f, 45 },
    { 0, 0 },
    { 0x99d3f211, 34 },
    { 0, 0 },
    { 0x4ae3f213, 34 },
    { 0xb493f213, 93 },
    { 0x9ac3f213, 81 },
    { 0x7073f215, 98 },
    { 0x72d3f214, 43 },
    { 0x1393f218, 95 },
    { 0xc473f218, 90 },
    { 0xefd3f215, 90 },
    { 0x45e3f21b, 97 },
    { 0xb273f21c, 34 },
    { 0, 0 },
    { 0x8223f21e, 98 },
    { 0xc813f21e, 34 },
    { 0x9203f220, 92 },
    { 0x5df3f221, 33 },
    { 0xfe73f222, 61 },
    { 0xc113f21e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43f229, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7253f231, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe33f235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5003f238, 98 },
    { 0xe793f238, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b13f23e, 61 },
    { 0, 0 },
    { 0x4573f240, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec3f245, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2643f248, 78 },
    { 0x7c53f248, 90 },
    { 0xd063f24a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8023f250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db3f255, 76 },
    { 0x93f256, 23 },
    { 0, 0 },
    { 0x70c3f258, 61 },
    { 0xca33f259, 95 },
    { 0x7bc3f25a, 61 },
    { 0, 0 },
    { 0xe923f25c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1053f261, 23 },
    { 0x563f261, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a3f265, 76 },
    { 0xbac3f265, 23 },
    { 0x5343f267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b53f270, 95 },
    { 0, 0 },
    { 0x4cf3f272, 69 },
    { 0xa523f273, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa23f278, 61 },
    { 0, 0 },
    { 0x25b3f27a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2883f27d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7193f280, 90 },
    { 0xa893f280, 88 },
    { 0xe953f282, 96 },
    { 0x7893f282, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc793f28b, 34 },
    { 0, 0 },
    { 0xdec3f28d, 95 },
    { 0xcf73f28e, 61 },
    { 0x54d3f28f, 34 },
    { 0xfde3f28e, 61 },
    { 0x8143f291, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83f299, 23 },
    { 0x63f29a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc253f2a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1753f2a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde93f2b2, 46 },
    { 0xe783f2b3, 85 },
    { 0x2273f2b4, 33 },
    { 0x11c3f2b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab93f2be, 24 },
    { 0, 0 },
    { 0xc593f2c0, 90 },
    { 0x8053f2c1, 76 },
    { 0xa383f2c2, 17 },
    { 0xb9f3f2c3, 95 },
    { 0xe8e3f2c1, 23 },
    { 0x11f3f2c5, 75 },
    { 0x2473f2c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d3f2ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d13f2d6, 61 },
    { 0x15d3f2d6, 14 },
    { 0x52d3f2d8, 91 },
    { 0x8b73f2d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a3f2dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a3f2e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d3f2f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e3f2f6, 33 },
    { 0x6563f2f6, 61 },
    { 0x2b03f2f8, 95 },
    { 0x6833f2f9, 76 },
    { 0x35e3f2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae3f2fd, 93 },
    { 0x7cf3f2fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c03f302, 93 },
    { 0x10b3f302, 95 },
    { 0, 0 },
    { 0xdaf3f305, 23 },
    { 0, 0 },
    { 0xeb53f307, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f3f30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8053f30d, 55 },
    { 0x36b3f30e, 90 },
    { 0xd123f30e, 33 },
    { 0xbb53f310, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b3f31b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d3f322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa3f328, 87 },
    { 0, 0 },
    { 0x4173f32a, 90 },
    { 0, 0 },
    { 0xfac3f32c, 34 },
    { 0, 0 },
    { 0xb7d3f32e, 93 },
    { 0xe203f32f, 8 },
    { 0x5443f330, 33 },
    { 0x6623f32e, 33 },
    { 0x84b3f330, 33 },
    { 0, 0 },
    { 0x733f334, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd943f337, 76 },
    { 0x1613f338, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9733f33f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf83f344, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf103f34e, 66 },
    { 0xfd23f34e, 90 },
    { 0x94c3f350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa523f353, 78 },
    { 0x94e3f354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1483f358, 98 },
    { 0x7c33f358, 93 },
    { 0x94f3f35a, 23 },
    { 0xddf3f35a, 85 },
    { 0xbab3f35c, 33 },
    { 0x1e53f35c, 95 },
    { 0x5663f35c, 33 },
    { 0xe243f35f, 90 },
    { 0xb313f35b, 76 },
    { 0x6c03f361, 33 },
    { 0x8f63f35c, 61 },
    { 0, 0 },
    { 0xc443f364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5923f367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e3f36a, 61 },
    { 0x8293f36b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6903f36f, 33 },
    { 0, 0 },
    { 0x33a3f371, 34 },
    { 0xa683f371, 65 },
    { 0x79b3f373, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2063f379, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f63f37c, 33 },
    { 0, 0 },
    { 0xa3f37e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce3f381, 52 },
    { 0x3673f381, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c63f385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd3f38b, 64 },
    { 0x5743f38b, 61 },
    { 0x24f3f38d, 90 },
    { 0xb6c3f38e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a73f392, 33 },
    { 0xb413f393, 100 },
    { 0xa3f393, 76 },
    { 0x9ab3f393, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a43f39c, 9 },
    { 0x90e3f39d, 43 },
    { 0xead3f39e, 80 },
    { 0x7ba3f39f, 61 },
    { 0xae73f39e, 66 },
    { 0x60f3f39d, 76 },
    { 0x89c3f39e, 33 },
    { 0xf693f39d, 98 },
    { 0, 0 },
    { 0x1403f3a5, 55 },
    { 0xdb13f3a5, 98 },
    { 0x52a3f3a7, 76 },
    { 0x843f3a8, 93 },
    { 0xe6f3f3a9, 55 },
    { 0xc093f3a9, 23 },
    { 0x6713f3ab, 98 },
    { 0xedc3f3ac, 75 },
    { 0x5b03f3ad, 23 },
    { 0xbd63f3ad, 85 },
    { 0, 0 },
    { 0x2e73f3b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c3f3ba, 66 },
    { 0xd073f3ba, 77 },
    { 0x1553f3bb, 90 },
    { 0x4143f3ba, 61 },
    { 0x5243f3be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc383f3c5, 98 },
    { 0x1953f3c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e3f3cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f3f3d0, 33 },
    { 0x2f93f3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf3f3d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d13f3d7, 84 },
    { 0, 0 },
    { 0x2993f3d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb903f3de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23f3e2, 43 },
    { 0, 0 },
    { 0x2043f3e4, 90 },
    { 0, 0 },
    { 0xf803f3e6, 98 },
    { 0x76c3f3e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb873f3ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8633f3f4, 88 },
    { 0xc1b3f3f4, 95 },
    { 0xa023f3f6, 95 },
    { 0x7473f3f7, 76 },
    { 0, 0 },
    { 0x92e3f3f9, 95 },
    { 0x333f3fa, 100 },
    { 0x55d3f3fb, 46 },
    { 0xd053f3fb, 76 },
    { 0xcef3f3fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb3f402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb283f405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a53f40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed23f419, 90 },
    { 0x2683f41a, 98 },
    { 0x1693f41b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d3f41e, 23 },
    { 0xfe03f41f, 34 },
    { 0xd7f3f41f, 77 },
    { 0x4523f41f, 90 },
    { 0x6e23f422, 23 },
    { 0, 0 },
    { 0xe183f424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6663f42a, 99 },
    { 0x4fe3f42b, 90 },
    { 0x7a03f42c, 70 },
    { 0, 0 },
    { 0x4c63f42e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b13f435, 87 },
    { 0x933f436, 98 },
    { 0, 0 },
    { 0xbe73f438, 90 },
    { 0x5e3f439, 55 },
    { 0x1563f43a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c3f43d, 90 },
    { 0xdd23f43e, 33 },
    { 0x5053f43f, 95 },
    { 0xcf43f440, 90 },
    { 0, 0 },
    { 0x4b13f442, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de3f445, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4513f458, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c3f45d, 23 },
    { 0, 0 },
    { 0x43f45f, 69 },
    { 0xd103f460, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a3f466, 99 },
    { 0x7773f467, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a3f46a, 100 },
    { 0x7043f46b, 65 },
    { 0x22a3f46c, 61 },
    { 0xf973f46c, 87 },
    { 0x4ce3f46b, 55 },
    { 0xcf83f46b, 90 },
    { 0x22b3f46b, 90 },
    { 0x90d3f46b, 98 },
    { 0, 0 },
    { 0xf3e3f473, 65 },
    { 0xa683f473, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf533f479, 33 },
    { 0xbd23f47a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x483f47d, 33 },
    { 0x2933f47d, 95 },
    { 0xe443f47f, 93 },
    { 0, 0 },
    { 0xef93f481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c83f489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a3f48c, 33 },
    { 0, 0 },
    { 0x77d3f48e, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a3f493, 33 },
    { 0x2c73f494, 33 },
    { 0x8b53f495, 61 },
    { 0x4643f495, 23 },
    { 0x55c3f494, 55 },
    { 0x5053f498, 34 },
    { 0x33c3f499, 90 },
    { 0xd4f3f49a, 93 },
    { 0x8d73f49a, 90 },
    { 0xe43f49c, 61 },
    { 0x2a83f49c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e3f4a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c3f4a3, 92 },
    { 0xb343f4a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f4ae, 55 },
    { 0x3f83f4af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e03f4b4, 67 },
    { 0xc033f4b4, 23 },
    { 0, 0 },
    { 0x8da3f4b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6513f4bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9553f4bf, 61 },
    { 0x5a83f4c0, 33 },
    { 0x6db3f4c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9803f4c9, 98 },
    { 0, 0 },
    { 0xc1e3f4cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a73f4cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7313f4d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x723f4d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f3f4db, 61 },
    { 0xb6a3f4db, 95 },
    { 0xef43f4db, 33 },
    { 0x2a83f4de, 99 },
    { 0x87c3f4db, 61 },
    { 0x79e3f4db, 46 },
    { 0x9d63f4dd, 17 },
    { 0x7da3f4e2, 61 },
    { 0x2ef3f4e0, 33 },
    { 0x5763f4e4, 76 },
    { 0x7d43f4e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9963f4ed, 55 },
    { 0x5d13f4ed, 33 },
    { 0x3d13f4ef, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4433f4fb, 76 },
    { 0xecd3f4fb, 100 },
    { 0, 0 },
    { 0x59a3f4fe, 90 },
    { 0x293f4fe, 23 },
    { 0x3a23f4ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f3f503, 55 },
    { 0x1e13f504, 66 },
    { 0, 0 },
    { 0xf813f506, 84 },
    { 0x3033f507, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2383f50a, 90 },
    { 0x6333f50b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb3f50e, 52 },
    { 0x9643f50e, 33 },
    { 0xeba3f510, 33 },
    { 0xc113f510, 95 },
    { 0xfb83f512, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e53f517, 66 },
    { 0x5713f518, 33 },
    { 0x2bf3f518, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5593f51c, 55 },
    { 0, 0 },
    { 0xa6a3f51e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c43f523, 23 },
    { 0xf753f524, 87 },
    { 0xbe13f525, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd663f528, 33 },
    { 0, 0 },
    { 0x1b63f52a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6953f52d, 43 },
    { 0xb553f52d, 34 },
    { 0xfb53f52d, 77 },
    { 0x7ed3f52e, 98 },
    { 0xeed3f531, 61 },
    { 0xb63f532, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb713f535, 91 },
    { 0, 0 },
    { 0xbd53f537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb093f53b, 98 },
    { 0xe633f53b, 55 },
    { 0x8d53f53d, 23 },
    { 0x7663f53e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa793f543, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca3f546, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc463f54a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb43f54e, 66 },
    { 0xa0d3f54f, 98 },
    { 0, 0 },
    { 0x91c3f551, 55 },
    { 0x8793f552, 95 },
    { 0xbd93f553, 90 },
    { 0xb1b3f553, 90 },
    { 0x2693f555, 33 },
    { 0x3963f551, 33 },
    { 0xbf23f552, 90 },
    { 0xf843f552, 98 },
    { 0x823f559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3f55d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff93f562, 34 },
    { 0xd603f562, 66 },
    { 0xb593f564, 36 },
    { 0x6a13f565, 34 },
    { 0x6643f565, 98 },
    { 0x6273f567, 33 },
    { 0x5ea3f568, 33 },
    { 0xd853f568, 43 },
    { 0x31d3f56a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f3f576, 33 },
    { 0xee53f576, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc903f57b, 36 },
    { 0, 0 },
    { 0x56a3f57d, 52 },
    { 0xf353f57d, 98 },
    { 0x68e3f57e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b3f583, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4453f588, 61 },
    { 0x7363f589, 77 },
    { 0x58f3f58a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac3f591, 76 },
    { 0x2723f592, 23 },
    { 0xa863f593, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa373f596, 97 },
    { 0xb03f596, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa343f59c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e03f5a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f5a5, 80 },
    { 0x8e3f5a6, 33 },
    { 0x203f5a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d3f5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f3f5b2, 34 },
    { 0, 0 },
    { 0x6ae3f5b4, 33 },
    { 0x2e23f5b4, 90 },
    { 0, 0 },
    { 0x4603f5b7, 98 },
    { 0, 0 },
    { 0xf6b3f5b9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f03f5bf, 46 },
    { 0xe313f5bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba3f5c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe983f5c6, 67 },
    { 0, 0 },
    { 0x17c3f5c8, 95 },
    { 0, 0 },
    { 0x7b03f5ca, 65 },
    { 0xe793f5cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6463f5d0, 75 },
    { 0x6ff3f5d1, 76 },
    { 0x9713f5d2, 61 },
    { 0x50d3f5d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2043f5d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c63f5dc, 95 },
    { 0x6373f5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf163f5e4, 95 },
    { 0x5a13f5e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8583f5ee, 61 },
    { 0, 0 },
    { 0x6ef3f5f0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f5f3, 77 },
    { 0x2f63f5f4, 95 },
    { 0xea83f5f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa293f601, 2 },
    { 0, 0 },
    { 0xf0d3f603, 33 },
    { 0, 0 },
    { 0x6613f605, 90 },
    { 0xe493f605, 98 },
    { 0x2ff3f606, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b3f60b, 90 },
    { 0, 0 },
    { 0xba83f60d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf813f610, 23 },
    { 0, 0 },
    { 0xd563f612, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb83f617, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa373f61a, 61 },
    { 0xabd3f61b, 76 },
    { 0xf583f61b, 87 },
    { 0, 0 },
    { 0xa363f61e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8603f621, 76 },
    { 0x4333f622, 33 },
    { 0xbd83f623, 77 },
    { 0x3263f623, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f3f62f, 81 },
    { 0, 0 },
    { 0x473f631, 90 },
    { 0x9413f632, 33 },
    { 0x58c3f632, 90 },
    { 0x4133f633, 43 },
    { 0x19f3f635, 46 },
    { 0x9113f631, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a53f63a, 90 },
    { 0, 0 },
    { 0x9a03f63c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c3f63f, 85 },
    { 0, 0 },
    { 0xbd73f641, 84 },
    { 0x9973f642, 95 },
    { 0, 0 },
    { 0xa343f644, 55 },
    { 0xba93f645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5753f648, 52 },
    { 0, 0 },
    { 0x6753f64a, 61 },
    { 0x943f64b, 33 },
    { 0, 0 },
    { 0x9d93f64d, 98 },
    { 0xdec3f64d, 33 },
    { 0x42a3f64f, 33 },
    { 0xdd83f650, 23 },
    { 0xe8f3f650, 46 },
    { 0xadd3f652, 90 },
    { 0x4fb3f652, 95 },
    { 0x5f93f654, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb3f657, 91 },
    { 0xa393f657, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3f65d, 23 },
    { 0, 0 },
    { 0xc913f65f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1423f66a, 99 },
    { 0xdbc3f66b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc03f670, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde3f67c, 67 },
    { 0x4413f67c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9783f682, 90 },
    { 0, 0 },
    { 0x6ad3f684, 93 },
    { 0xf173f685, 61 },
    { 0xc6e3f685, 90 },
    { 0xd623f685, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee3f68c, 23 },
    { 0xd393f68d, 33 },
    { 0x3753f68c, 34 },
    { 0xa523f68f, 43 },
    { 0xcd73f690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e3f693, 33 },
    { 0, 0 },
    { 0x8ea3f695, 90 },
    { 0xa853f696, 34 },
    { 0x57f3f696, 76 },
    { 0xc123f698, 76 },
    { 0, 0 },
    { 0x8e53f69a, 100 },
    { 0x82f3f69a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb3f69e, 98 },
    { 0x80d3f69f, 90 },
    { 0, 0 },
    { 0x5143f6a1, 90 },
    { 0x5d53f6a2, 90 },
    { 0, 0 },
    { 0xaee3f6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9313f6a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3f6ae, 76 },
    { 0x6153f6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e3f6b4, 95 },
    { 0x623f6b5, 65 },
    { 0x4153f6b6, 76 },
    { 0xbd33f6b7, 61 },
    { 0x16f3f6b8, 76 },
    { 0xd183f6b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a3f6bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a3f6c0, 87 },
    { 0, 0 },
    { 0xf683f6c2, 61 },
    { 0x7913f6c3, 40 },
    { 0x7373f6c3, 46 },
    { 0xc7b3f6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df3f6d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d3f6d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc393f6dd, 66 },
    { 0x1a73f6de, 98 },
    { 0xff83f6df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e33f6e2, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d3f6e9, 75 },
    { 0x5c23f6ea, 98 },
    { 0x5323f6e9, 84 },
    { 0x6d93f6e9, 43 },
    { 0x3693f6ed, 55 },
    { 0, 0 },
    { 0x9ff3f6ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4403f6f3, 46 },
    { 0x1303f6f4, 58 },
    { 0x7aa3f6f5, 68 },
    { 0x1113f6f6, 51 },
    { 0xf4c3f6f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f33f6fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23f6ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3093f704, 98 },
    { 0, 0 },
    { 0xdfe3f706, 33 },
    { 0x9cd3f706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8763f70b, 61 },
    { 0, 0 },
    { 0x2ba3f70d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea3f711, 23 },
    { 0xa463f711, 75 },
    { 0xa033f713, 33 },
    { 0, 0 },
    { 0x9333f715, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d13f71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b3f71e, 81 },
    { 0xaf33f71f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be3f722, 23 },
    { 0x6073f722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff3f726, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd13f72c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf633f731, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe173f736, 98 },
    { 0xbe93f737, 52 },
    { 0x4663f738, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6713f73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa3f743, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc973f747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb803f74a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f3f74e, 98 },
    { 0x87e3f74f, 61 },
    { 0, 0 },
    { 0x363f751, 43 },
    { 0, 0 },
    { 0x4493f753, 84 },
    { 0x4363f754, 61 },
    { 0, 0 },
    { 0xaf23f756, 95 },
    { 0, 0 },
    { 0x9403f758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe993f75d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca3f762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4253f767, 56 },
    { 0x7193f768, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa973f76b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8093f76f, 98 },
    { 0x4753f76f, 77 },
    { 0x5e83f771, 90 },
    { 0x6f03f772, 88 },
    { 0x2d43f771, 98 },
    { 0x4873f774, 76 },
    { 0xd063f775, 95 },
    { 0x3da3f776, 98 },
    { 0xe3a3f775, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e23f77c, 34 },
    { 0xe673f77d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3783f780, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5543f784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5493f789, 61 },
    { 0, 0 },
    { 0xfb53f78b, 43 },
    { 0xe9b3f78c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c3f78f, 43 },
    { 0x5953f790, 84 },
    { 0, 0 },
    { 0x8b43f792, 77 },
    { 0xf563f793, 74 },
    { 0xa153f792, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323f798, 95 },
    { 0xc6c3f798, 23 },
    { 0x9c63f79a, 90 },
    { 0x3973f798, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c3f79f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a83f7af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb3f7b2, 98 },
    { 0x56a3f7b2, 66 },
    { 0x8913f7b4, 33 },
    { 0, 0 },
    { 0x9493f7b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb33f7ba, 90 },
    { 0x3e03f7ba, 34 },
    { 0x7c83f7ba, 90 },
    { 0x2963f7bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f3f7c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3133f7c5, 90 },
    { 0, 0 },
    { 0xbbd3f7c7, 98 },
    { 0x8433f7c7, 66 },
    { 0x3c53f7c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93f7ce, 90 },
    { 0x2533f7ce, 98 },
    { 0x7c3f7d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f3f7d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c93f7d8, 33 },
    { 0xa653f7d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd113f7dd, 87 },
    { 0xc53f7de, 77 },
    { 0x5c43f7df, 100 },
    { 0, 0 },
    { 0x7933f7e1, 33 },
    { 0, 0 },
    { 0x88c3f7e3, 100 },
    { 0x9e93f7e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d3f7e8, 23 },
    { 0x2153f7e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac33f7ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d93f7f1, 61 },
    { 0x1d33f7f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a83f7f6, 23 },
    { 0xd73f7f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe3f7fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c3f801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f3f804, 23 },
    { 0xd233f805, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe513f808, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb3f80c, 55 },
    { 0x30d3f80d, 61 },
    { 0x8173f80e, 43 },
    { 0x4823f80f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b63f818, 61 },
    { 0x9db3f819, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4273f81d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4673f821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb373f825, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c3f829, 95 },
    { 0x7a03f82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1983f838, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e3f83e, 33 },
    { 0xb0a3f83f, 76 },
    { 0xca23f83f, 90 },
    { 0xa8a3f83f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f93f845, 53 },
    { 0xd1a3f846, 23 },
    { 0x9f53f847, 95 },
    { 0xcde3f846, 100 },
    { 0x9f73f846, 95 },
    { 0x5d03f84a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3f84d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa203f853, 34 },
    { 0, 0 },
    { 0x4da3f855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af3f85d, 100 },
    { 0x2a13f85e, 39 },
    { 0x8b3f85f, 93 },
    { 0x90c3f85e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee33f863, 88 },
    { 0, 0 },
    { 0x4a73f865, 76 },
    { 0, 0 },
    { 0xf693f867, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4493f86a, 23 },
    { 0x1c13f86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd63f870, 90 },
    { 0, 0 },
    { 0x2fa3f872, 91 },
    { 0x4fd3f873, 55 },
    { 0x3b13f872, 66 },
    { 0, 0 },
    { 0xc283f876, 61 },
    { 0x4603f877, 81 },
    { 0, 0 },
    { 0x4a93f879, 90 },
    { 0x76b3f879, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b3f880, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c3f883, 87 },
    { 0x4b03f884, 98 },
    { 0, 0 },
    { 0xfcc3f886, 93 },
    { 0, 0 },
    { 0xdc83f888, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7033f88b, 90 },
    { 0x3cb3f88b, 55 },
    { 0x2053f88d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1993f890, 84 },
    { 0x623f890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423f895, 98 },
    { 0x1ca3f896, 33 },
    { 0xb5a3f897, 61 },
    { 0, 0 },
    { 0x1f23f899, 33 },
    { 0, 0 },
    { 0x1043f89b, 33 },
    { 0, 0 },
    { 0x6ab3f89d, 23 },
    { 0xde03f89e, 95 },
    { 0xe53f89e, 33 },
    { 0x1ac3f8a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8033f8a3, 33 },
    { 0xa5c3f8a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1503f8a8, 90 },
    { 0xc153f8a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade3f8ae, 67 },
    { 0xa733f8af, 90 },
    { 0xbe33f8b0, 34 },
    { 0x17e3f8ae, 75 },
    { 0x9143f8b2, 43 },
    { 0x60f3f8b3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e3f8b6, 33 },
    { 0, 0 },
    { 0x75f3f8b8, 77 },
    { 0x4e33f8b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c3f8bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d33f8c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b3f8c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3f8ca, 91 },
    { 0x6f53f8ca, 52 },
    { 0xa8c3f8cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c93f8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee23f8d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf613f8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b3f8dc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3473f8e4, 46 },
    { 0xca33f8e5, 55 },
    { 0xfe63f8e5, 78 },
    { 0x5223f8e6, 23 },
    { 0x4d43f8e8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3f8ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f3f8f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd3f8f5, 61 },
    { 0xc793f8f6, 33 },
    { 0, 0 },
    { 0xba03f8f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a3f8fb, 61 },
    { 0, 0 },
    { 0xc203f8fd, 90 },
    { 0, 0 },
    { 0x3953f8ff, 85 },
    { 0x6643f900, 61 },
    { 0x6f03f8ff, 23 },
    { 0xd3a3f902, 34 },
    { 0x23d3f903, 46 },
    { 0x1bd3f902, 95 },
    { 0xaf93f905, 61 },
    { 0, 0 },
    { 0xd363f907, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa523f90a, 33 },
    { 0x5ef3f90b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4713f910, 95 },
    { 0xb2a3f911, 98 },
    { 0, 0 },
    { 0x80e3f913, 98 },
    { 0x463f913, 99 },
    { 0, 0 },
    { 0x953f916, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe863f91a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e63f91e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde3f921, 55 },
    { 0x7ac3f921, 33 },
    { 0xa973f922, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d73f926, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc033f92a, 95 },
    { 0, 0 },
    { 0xa493f92c, 34 },
    { 0, 0 },
    { 0xaea3f92e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63f936, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2233f93e, 81 },
    { 0xe123f93f, 98 },
    { 0xc283f93f, 87 },
    { 0x1d3f940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b3f948, 33 },
    { 0x2123f949, 98 },
    { 0, 0 },
    { 0x5a63f94b, 23 },
    { 0x7703f94c, 75 },
    { 0x29f3f94d, 61 },
    { 0xb623f94e, 90 },
    { 0x5253f94f, 76 },
    { 0xfce3f950, 90 },
    { 0xdfc3f950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d63f954, 33 },
    { 0x7e43f955, 90 },
    { 0, 0 },
    { 0xa5f3f957, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4483f95b, 65 },
    { 0, 0 },
    { 0x9123f95d, 84 },
    { 0xb733f95e, 93 },
    { 0x9723f95f, 87 },
    { 0xa6a3f960, 90 },
    { 0xb883f961, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d43f965, 61 },
    { 0, 0 },
    { 0x9fb3f967, 95 },
    { 0, 0 },
    { 0x69e3f969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4333f96d, 98 },
    { 0xe973f96d, 61 },
    { 0, 0 },
    { 0xd393f970, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b3f973, 99 },
    { 0xe5d3f973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3f97e, 43 },
    { 0x7d23f97f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2933f986, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b93f989, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a3f992, 98 },
    { 0x31c3f992, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e73f996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f3f99c, 34 },
    { 0xef53f99d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63f9a0, 66 },
    { 0x9c03f9a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d13f9a4, 87 },
    { 0x2bd3f9a5, 55 },
    { 0x42d3f9a6, 33 },
    { 0xd833f9a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd093f9aa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c3f9ad, 19 },
    { 0, 0 },
    { 0x3893f9af, 61 },
    { 0, 0 },
    { 0xb3d3f9b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf3f9b5, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff3f9bb, 67 },
    { 0x4013f9bb, 23 },
    { 0, 0 },
    { 0xc183f9be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a73f9c2, 90 },
    { 0xd533f9c3, 95 },
    { 0, 0 },
    { 0xa2a3f9c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5113f9cb, 90 },
    { 0x6ee3f9cc, 91 },
    { 0xbfa3f9cd, 95 },
    { 0x5623f9ce, 76 },
    { 0x3353f9cf, 23 },
    { 0x3853f9d0, 98 },
    { 0xe2b3f9cd, 67 },
    { 0, 0 },
    { 0x33f9d3, 33 },
    { 0, 0 },
    { 0xbb93f9d5, 87 },
    { 0xb2d3f9d6, 33 },
    { 0x3b13f9d6, 85 },
    { 0xf7a3f9d7, 95 },
    { 0xdbe3f9d7, 23 },
    { 0x27b3f9da, 95 },
    { 0x3373f9db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f93f9e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf63f9e9, 23 },
    { 0xd463f9ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313f9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f9f8, 43 },
    { 0x8af3f9f8, 61 },
    { 0, 0 },
    { 0xe313f9fb, 95 },
    { 0xef03f9fc, 23 },
    { 0, 0 },
    { 0x9913f9fe, 33 },
    { 0xeaf3f9ff, 23 },
    { 0x3743fa00, 95 },
    { 0x1a43fa00, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe3fa06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe043fa0c, 91 },
    { 0x64b3fa0d, 65 },
    { 0x7123fa0d, 33 },
    { 0x3a13fa0f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb903fa12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6683fa17, 87 },
    { 0x6583fa18, 61 },
    { 0xed03fa18, 55 },
    { 0xee63fa1a, 90 },
    { 0xd83fa1b, 42 },
    { 0x9553fa1c, 76 },
    { 0, 0 },
    { 0x2e13fa1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe563fa22, 87 },
    { 0x2373fa22, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c33fa27, 17 },
    { 0xf753fa27, 55 },
    { 0x663fa29, 67 },
    { 0xd4b3fa2a, 81 },
    { 0, 0 },
    { 0xd833fa2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2533fa32, 61 },
    { 0xf683fa32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e53fa39, 83 },
    { 0x1263fa3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd33fa3e, 61 },
    { 0x8583fa3f, 98 },
    { 0xd043fa3e, 61 },
    { 0xf263fa41, 51 },
    { 0x5ee3fa42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b53fa46, 33 },
    { 0xd143fa46, 61 },
    { 0xf5a3fa48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f3fa4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b53fa54, 90 },
    { 0xeaf3fa55, 23 },
    { 0xdb73fa56, 95 },
    { 0xb5b3fa56, 76 },
    { 0x7543fa58, 95 },
    { 0x94a3fa57, 95 },
    { 0x4083fa5a, 61 },
    { 0, 0 },
    { 0x32b3fa5c, 33 },
    { 0, 0 },
    { 0xc383fa5e, 23 },
    { 0xb563fa5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a93fa62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd33fa66, 90 },
    { 0xcb43fa66, 101 },
    { 0xe03fa67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf663fa6e, 23 },
    { 0, 0 },
    { 0xb703fa70, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1063fa78, 66 },
    { 0xc93fa79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b3fa7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe53fa85, 78 },
    { 0xc1e3fa86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e83fa89, 95 },
    { 0x4b03fa89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da3fa8f, 78 },
    { 0x2cf3fa8f, 61 },
    { 0, 0 },
    { 0xd733fa92, 46 },
    { 0, 0 },
    { 0x1e73fa94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f3fa97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d3fa9b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c3faa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593faa9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b33fab3, 99 },
    { 0x6253fab4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d3fab7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3fabb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe3fabe, 34 },
    { 0xc833fabe, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec53fac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1043facb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5893face, 61 },
    { 0xf913facf, 33 },
    { 0xbc53fad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x433fad3, 33 },
    { 0xb2b3fad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b3fadb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f93fade, 23 },
    { 0, 0 },
    { 0x1ac3fae0, 61 },
    { 0, 0 },
    { 0xa143fae2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8043fae5, 95 },
    { 0, 0 },
    { 0x28c3fae7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc3faf0, 43 },
    { 0x8b23faf1, 98 },
    { 0x1a03faf2, 56 },
    { 0, 0 },
    { 0x5253faf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef3fafa, 90 },
    { 0x38c3fafa, 98 },
    { 0x6b83fafb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab83faff, 83 },
    { 0, 0 },
    { 0xadf3fb01, 98 },
    { 0xeb63fb02, 33 },
    { 0xf313fb02, 23 },
    { 0x71e3fb01, 95 },
    { 0, 0 },
    { 0x2d23fb06, 76 },
    { 0xfa13fb07, 90 },
    { 0, 0 },
    { 0x2993fb09, 95 },
    { 0, 0 },
    { 0xab93fb0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d3fb0f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a3fb1d, 90 },
    { 0xd583fb1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9253fb22, 76 },
    { 0, 0 },
    { 0x1323fb24, 100 },
    { 0, 0 },
    { 0xe753fb26, 95 },
    { 0xb483fb27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d3fb2f, 61 },
    { 0x6653fb30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a3fb33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c43fb37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7913fb3c, 95 },
    { 0x8f83fb3c, 87 },
    { 0x9053fb3e, 33 },
    { 0, 0 },
    { 0x1d83fb40, 93 },
    { 0x7e93fb40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3213fb44, 91 },
    { 0x7423fb44, 93 },
    { 0x9483fb44, 78 },
    { 0xb023fb47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f3fb4a, 67 },
    { 0x5483fb4b, 84 },
    { 0, 0 },
    { 0xda23fb4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce3fb51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243fb56, 17 },
    { 0, 0 },
    { 0xcc03fb58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded3fb5c, 33 },
    { 0x79d3fb5c, 95 },
    { 0x1c13fb5d, 84 },
    { 0, 0 },
    { 0xc423fb60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e13fb63, 61 },
    { 0, 0 },
    { 0xe23fb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb353fb6c, 52 },
    { 0xf773fb6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9443fb72, 95 },
    { 0x5753fb73, 88 },
    { 0xc653fb74, 76 },
    { 0xaa33fb73, 46 },
    { 0, 0 },
    { 0x7e63fb77, 90 },
    { 0x9d93fb78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b3fb7e, 90 },
    { 0x6e53fb7f, 90 },
    { 0, 0 },
    { 0x4723fb81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd3fb90, 90 },
    { 0x6c63fb91, 100 },
    { 0x8123fb91, 95 },
    { 0, 0 },
    { 0x3223fb94, 33 },
    { 0x3953fb95, 33 },
    { 0xc2f3fb96, 40 },
    { 0x5ef3fb97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a3fb9b, 55 },
    { 0, 0 },
    { 0xff03fb9d, 90 },
    { 0xbc53fb9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc973fba1, 93 },
    { 0x39c3fba2, 33 },
    { 0xf053fba3, 61 },
    { 0xac23fba2, 61 },
    { 0xbe43fba3, 88 },
    { 0x97b3fba6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf43fbab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc3fbb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323fbb9, 90 },
    { 0x8993fbb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc933fbbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e3fbc4, 98 },
    { 0xee73fbc5, 93 },
    { 0xe3e3fbc4, 95 },
    { 0x20e3fbc7, 61 },
    { 0x82b3fbc8, 34 },
    { 0x96e3fbc9, 23 },
    { 0xae93fbca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b3fbcd, 90 },
    { 0, 0 },
    { 0x5873fbcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a53fbd2, 55 },
    { 0xa553fbd3, 93 },
    { 0x5653fbd4, 90 },
    { 0x1333fbd5, 61 },
    { 0x6ac3fbd4, 34 },
    { 0x7913fbd7, 53 },
    { 0xab93fbd7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2303fbdf, 90 },
    { 0x93c3fbe0, 85 },
    { 0xa2a3fbe0, 90 },
    { 0x4ac3fbe2, 33 },
    { 0xdd73fbe3, 93 },
    { 0x22e3fbe2, 93 },
    { 0x3633fbe4, 95 },
    { 0x87e3fbe4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6063fbea, 90 },
    { 0x45c3fbea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe923fbf1, 61 },
    { 0x6e33fbf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed93fbfc, 87 },
    { 0xaf43fbfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d3fc01, 23 },
    { 0xa6a3fc02, 95 },
    { 0x18d3fc03, 61 },
    { 0x77f3fc04, 90 },
    { 0xbdb3fc05, 91 },
    { 0x6053fc01, 34 },
    { 0x51c3fc02, 76 },
    { 0, 0 },
    { 0xdf83fc09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb073fc0e, 17 },
    { 0x5023fc0f, 55 },
    { 0xf73fc10, 100 },
    { 0x1883fc0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb3fc15, 33 },
    { 0xb6f3fc16, 93 },
    { 0x4e13fc17, 77 },
    { 0x1243fc17, 95 },
    { 0xc2c3fc15, 76 },
    { 0, 0 },
    { 0xf7f3fc1b, 98 },
    { 0, 0 },
    { 0x5a03fc1d, 67 },
    { 0xcf53fc1d, 98 },
    { 0x7e3fc1e, 23 },
    { 0xa3fc1d, 76 },
    { 0, 0 },
    { 0xa163fc22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee53fc28, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe253fc2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1863fc2e, 91 },
    { 0x5033fc2f, 61 },
    { 0x7053fc2f, 55 },
    { 0xd903fc2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b3fc37, 90 },
    { 0, 0 },
    { 0x7173fc39, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3803fc49, 36 },
    { 0, 0 },
    { 0xe423fc4b, 46 },
    { 0x5ba3fc4b, 95 },
    { 0, 0 },
    { 0x19d3fc4e, 61 },
    { 0x22d3fc4f, 65 },
    { 0x6193fc50, 33 },
    { 0xfa03fc51, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d3fc55, 61 },
    { 0x27d3fc56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1333fc5a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c3fc5f, 90 },
    { 0x8703fc5f, 34 },
    { 0x80a3fc5f, 24 },
    { 0xf5e3fc62, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d3fc68, 99 },
    { 0x4373fc69, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe3fc72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab3fc77, 90 },
    { 0x67c3fc77, 76 },
    { 0x15c3fc77, 95 },
    { 0x6b03fc79, 61 },
    { 0, 0 },
    { 0x2e73fc7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd893fc7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc3fc82, 23 },
    { 0xd0b3fc82, 85 },
    { 0x9313fc83, 95 },
    { 0xdb33fc85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa3fc8e, 96 },
    { 0xb9e3fc8f, 99 },
    { 0, 0 },
    { 0x4543fc91, 90 },
    { 0xade3fc91, 66 },
    { 0x1ce3fc91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed03fc9a, 33 },
    { 0xa3e3fc9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a3fc9f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b13fca3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d3fca8, 76 },
    { 0, 0 },
    { 0xcef3fcaa, 55 },
    { 0, 0 },
    { 0xd243fcac, 78 },
    { 0xdf3fcad, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e23fcb2, 76 },
    { 0, 0 },
    { 0xa103fcb4, 90 },
    { 0xe273fcb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2723fcbc, 33 },
    { 0, 0 },
    { 0x7143fcbe, 33 },
    { 0, 0 },
    { 0x85c3fcc0, 75 },
    { 0xbe13fcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7903fcc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c3fcc8, 95 },
    { 0xed13fcc9, 91 },
    { 0x653fcc9, 61 },
    { 0xa8a3fcca, 34 },
    { 0xd273fcc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1573fcd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb473fcd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad73fcdb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d3fce0, 90 },
    { 0xbe93fce1, 78 },
    { 0, 0 },
    { 0xee3fce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf13fceb, 66 },
    { 0x1a53fcec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda93fcf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b3fcf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5323fcff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f3fd03, 61 },
    { 0xc2e3fd04, 33 },
    { 0x4ba3fd04, 33 },
    { 0x1293fd06, 55 },
    { 0x7103fd07, 76 },
    { 0xbd3fd07, 23 },
    { 0, 0 },
    { 0xdba3fd0a, 98 },
    { 0xd3a3fd0b, 34 },
    { 0, 0 },
    { 0xd053fd0d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd703fd10, 98 },
    { 0x5fa3fd11, 43 },
    { 0, 0 },
    { 0x79a3fd13, 95 },
    { 0xffe3fd14, 55 },
    { 0xfeb3fd14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7513fd19, 33 },
    { 0xda43fd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a3fd1d, 66 },
    { 0x30f3fd1d, 90 },
    { 0, 0 },
    { 0x5c43fd20, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2233fd23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd73fd31, 34 },
    { 0, 0 },
    { 0x42f3fd33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d3fd3b, 98 },
    { 0x2353fd3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b3fd43, 61 },
    { 0x5523fd43, 34 },
    { 0x5d53fd45, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1773fd4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f3fd4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf3fd63, 85 },
    { 0x4ba3fd64, 68 },
    { 0x6263fd65, 33 },
    { 0xfcf3fd66, 68 },
    { 0xa033fd66, 23 },
    { 0x5393fd63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a03fd6b, 66 },
    { 0xea73fd6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a3fd6f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a93fd72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf913fd75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd173fd78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d3fd7f, 23 },
    { 0x84f3fd80, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x503fd83, 90 },
    { 0x223fd83, 95 },
    { 0x2803fd85, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7403fd8a, 76 },
    { 0, 0 },
    { 0x2513fd8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad3fd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e23fd94, 46 },
    { 0x2b33fd94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e43fd9d, 61 },
    { 0xd053fd9d, 55 },
    { 0, 0 },
    { 0xd043fda0, 68 },
    { 0, 0 },
    { 0xedd3fda2, 55 },
    { 0, 0 },
    { 0x963fda4, 66 },
    { 0, 0 },
    { 0x3293fda6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa903fdac, 37 },
    { 0x5773fdad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac3fdb1, 33 },
    { 0, 0 },
    { 0xa533fdb3, 99 },
    { 0xbef3fdb4, 34 },
    { 0xf013fdb5, 100 },
    { 0xb373fdb5, 95 },
    { 0x2c03fdb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb73fdba, 75 },
    { 0x793fdba, 61 },
    { 0, 0 },
    { 0xeee3fdbd, 66 },
    { 0xa493fdbe, 33 },
    { 0, 0 },
    { 0x5273fdc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9973fdc6, 23 },
    { 0, 0 },
    { 0xb853fdc8, 95 },
    { 0xad83fdc9, 90 },
    { 0x3f53fdc9, 95 },
    { 0x1b33fdcb, 95 },
    { 0xff53fdcc, 55 },
    { 0x9fc3fdcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab23fdd2, 33 },
    { 0x61b3fdd3, 34 },
    { 0xfac3fdd4, 99 },
    { 0, 0 },
    { 0x5b83fdd6, 61 },
    { 0xd523fdd7, 66 },
    { 0xba13fdd8, 98 },
    { 0x55b3fdd9, 61 },
    { 0, 0 },
    { 0xaaa3fddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3fde0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd553fde4, 23 },
    { 0x61c3fde5, 33 },
    { 0x63c3fde6, 75 },
    { 0x6e63fde6, 34 },
    { 0xd333fde5, 90 },
    { 0, 0 },
    { 0x2503fdea, 33 },
    { 0x9183fdea, 33 },
    { 0x7573fdeb, 17 },
    { 0x8c43fded, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c23fdf0, 33 },
    { 0, 0 },
    { 0xec03fdf2, 61 },
    { 0x75e3fdf3, 87 },
    { 0xadd3fdf4, 61 },
    { 0, 0 },
    { 0xccc3fdf6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db3fdf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe223fdfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc3fe03, 60 },
    { 0x8833fe03, 98 },
    { 0, 0 },
    { 0x81e3fe06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5043fe13, 61 },
    { 0, 0 },
    { 0xd453fe15, 91 },
    { 0, 0 },
    { 0x2af3fe17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d23fe1b, 52 },
    { 0x1d43fe1c, 98 },
    { 0x3a03fe1d, 95 },
    { 0x8e33fe1e, 55 },
    { 0xe0e3fe1c, 98 },
    { 0xbd23fe1f, 33 },
    { 0x9e33fe1e, 81 },
    { 0x7bb3fe1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d93fe25, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d3fe2b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc043fe2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3fe35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f3fe39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2203fe3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc53fe42, 55 },
    { 0x2923fe43, 56 },
    { 0x1b3fe44, 91 },
    { 0xf63fe43, 90 },
    { 0, 0 },
    { 0x85e3fe47, 35 },
    { 0x9003fe48, 43 },
    { 0xa3c3fe48, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b53fe50, 46 },
    { 0xfb83fe51, 90 },
    { 0x2ea3fe50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b63fe58, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c3fe5c, 17 },
    { 0x5273fe5d, 95 },
    { 0x80e3fe5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf63fe62, 61 },
    { 0xfd3fe63, 91 },
    { 0xd633fe63, 33 },
    { 0x98d3fe65, 95 },
    { 0x29b3fe66, 66 },
    { 0x3fb3fe66, 33 },
    { 0x1e53fe68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c3fe74, 61 },
    { 0, 0 },
    { 0xa3c3fe76, 61 },
    { 0xf473fe77, 33 },
    { 0xfc73fe78, 23 },
    { 0x7713fe77, 33 },
    { 0xff13fe76, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b3fe7f, 61 },
    { 0x2643fe80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf903fe84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf903fe87, 91 },
    { 0, 0 },
    { 0xc873fe89, 33 },
    { 0xa73fe8a, 98 },
    { 0x5383fe8b, 90 },
    { 0x2e43fe89, 77 },
    { 0xc0b3fe8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd3fe93, 95 },
    { 0, 0 },
    { 0xacc3fe95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a3fe9f, 33 },
    { 0, 0 },
    { 0x89a3fea1, 61 },
    { 0x6733fea2, 34 },
    { 0x4bc3fea3, 33 },
    { 0, 0 },
    { 0x23a3fea5, 68 },
    { 0xb183fea6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b3feaa, 33 },
    { 0xf163feaa, 90 },
    { 0x1573feac, 66 },
    { 0x1f63fead, 43 },
    { 0, 0 },
    { 0x7b3feaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b43feb5, 85 },
    { 0xe883feb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7023febc, 98 },
    { 0xb1d3febc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c3fec1, 61 },
    { 0xec33fec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb933fecb, 46 },
    { 0, 0 },
    { 0xcfb3fecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb3fed0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93fed5, 91 },
    { 0xf5f3fed5, 46 },
    { 0x7fb3fed7, 69 },
    { 0x1323fed6, 98 },
    { 0, 0 },
    { 0xd2f3feda, 81 },
    { 0x6413feda, 55 },
    { 0x18b3fedb, 98 },
    { 0xcc93fedd, 61 },
    { 0xd3b3fedc, 85 },
    { 0x5063fede, 61 },
    { 0x2983fee0, 18 },
    { 0x5843fedd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb3fee6, 23 },
    { 0, 0 },
    { 0x8833fee8, 34 },
    { 0xdb63fee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a3feed, 100 },
    { 0xba93feee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a3fef9, 68 },
    { 0x92b3fef9, 98 },
    { 0x7ce3fefb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f3ff00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9063ff10, 61 },
    { 0, 0 },
    { 0x6a83ff12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c3ff17, 76 },
    { 0xec43ff18, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9503ff1b, 90 },
    { 0x63e3ff1c, 90 },
    { 0x3b03ff1b, 93 },
    { 0x8443ff1e, 38 },
    { 0xf833ff1b, 90 },
    { 0xdb53ff20, 61 },
    { 0x34a3ff20, 33 },
    { 0x78f3ff22, 33 },
    { 0x4d73ff23, 23 },
    { 0xe403ff20, 95 },
    { 0xe333ff24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda13ff28, 93 },
    { 0, 0 },
    { 0xf23ff2a, 33 },
    { 0x6773ff2b, 66 },
    { 0x5633ff2a, 47 },
    { 0x6f03ff2c, 33 },
    { 0xeaf3ff2e, 90 },
    { 0x4d23ff2d, 55 },
    { 0, 0 },
    { 0x15e3ff31, 76 },
    { 0xb383ff32, 33 },
    { 0xf123ff33, 23 },
    { 0xb923ff34, 67 },
    { 0xf493ff34, 90 },
    { 0, 0 },
    { 0x93d3ff37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f3ff3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf073ff42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab63ff45, 90 },
    { 0x9d83ff45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe43ff4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b3ff58, 23 },
    { 0, 0 },
    { 0x5963ff5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8163ff5d, 33 },
    { 0, 0 },
    { 0xbc03ff5f, 43 },
    { 0xf2f3ff60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a63ff64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a63ff6d, 90 },
    { 0, 0 },
    { 0x4d63ff6f, 34 },
    { 0xa893ff6f, 90 },
    { 0, 0 },
    { 0xa6d3ff72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2813ff77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f93ff7b, 91 },
    { 0xeaf3ff7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6133ff87, 98 },
    { 0x67a3ff88, 95 },
    { 0x3b03ff88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b3ff8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823ff95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593ff9a, 55 },
    { 0, 0 },
    { 0x5ec3ff9c, 60 },
    { 0xbc23ff9d, 40 },
    { 0x55f3ff9c, 23 },
    { 0x84f3ff9d, 90 },
    { 0x5c73ffa0, 61 },
    { 0, 0 },
    { 0x16e3ffa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e43ffae, 61 },
    { 0x6193ffaf, 23 },
    { 0, 0 },
    { 0xfab3ffb1, 56 },
    { 0x71f3ffb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1653ffb7, 23 },
    { 0xe2a3ffb8, 85 },
    { 0x2843ffb7, 34 },
    { 0x4943ffb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b3ffc4, 23 },
    { 0xfb83ffc5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc463ffca, 61 },
    { 0xc713ffcb, 95 },
    { 0xd053ffcc, 100 },
    { 0, 0 },
    { 0xa353ffce, 90 },
    { 0, 0 },
    { 0xb3ffd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c3ffdc, 95 },
    { 0, 0 },
    { 0x6f83ffde, 33 },
    { 0x9cb3ffdf, 98 },
    { 0x5f43ffe0, 90 },
    { 0x4513ffe1, 78 },
    { 0xe323ffde, 76 },
    { 0x893ffe0, 95 },
    { 0x9e83ffe4, 95 },
    { 0xa1b3ffde, 93 },
    { 0xe6b3ffe6, 33 },
    { 0xb533ffe7, 68 },
    { 0xdc3ffe7, 90 },
    { 0, 0 },
    { 0xe223ffea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f63ffed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e63fff1, 90 },
    { 0xa8a3fff1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf573fffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a43fffd, 67 },
    { 0x3403fffe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a40001, 33 },
    { 0xfc40002, 95 },
    { 0x3a40003, 99 },
    { 0, 0 },
    { 0x9da40005, 87 },
    { 0x2cf40006, 33 },
    { 0x17e40007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a740013, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a40018, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb634001b, 46 },
    { 0x4864001c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d40021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c40026, 95 },
    { 0xc5240027, 98 },
    { 0x8a640026, 76 },
    { 0xf8440028, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede40030, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b40033, 98 },
    { 0x81240033, 98 },
    { 0x3de40035, 85 },
    { 0x92040036, 33 },
    { 0xb4740036, 95 },
    { 0xdc140038, 93 },
    { 0xd6240035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b4003e, 61 },
    { 0, 0 },
    { 0xad240040, 88 },
    { 0x9ff40041, 46 },
    { 0xe3040041, 52 },
    { 0, 0 },
    { 0xad140044, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe40047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5514004b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74004f, 66 },
    { 0, 0 },
    { 0x7c340051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2764005b, 61 },
    { 0x7d74005c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad40062, 61 },
    { 0x96140063, 93 },
    { 0x6f740063, 23 },
    { 0x3da40065, 90 },
    { 0x2be40062, 35 },
    { 0x61140067, 61 },
    { 0x3db40067, 34 },
    { 0x99640069, 85 },
    { 0xf6340067, 67 },
    { 0x6604006b, 76 },
    { 0xada4006b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf40073, 33 },
    { 0, 0 },
    { 0x1640075, 77 },
    { 0x6040075, 55 },
    { 0xabc40075, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d4007b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d240082, 99 },
    { 0x7d340083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5740086, 61 },
    { 0, 0 },
    { 0x76d40088, 81 },
    { 0x14f40088, 66 },
    { 0, 0 },
    { 0xf064008b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d40092, 90 },
    { 0x96e40092, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2740096, 34 },
    { 0xf0e40096, 51 },
    { 0xea540098, 23 },
    { 0, 0 },
    { 0x1b94009a, 95 },
    { 0, 0 },
    { 0x9964009c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa4009f, 90 },
    { 0x734400a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f400aa, 98 },
    { 0xa9e400ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e400b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15400ba, 93 },
    { 0, 0 },
    { 0xb59400bc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f400c5, 95 },
    { 0, 0 },
    { 0xf20400c7, 23 },
    { 0x224400c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2400cf, 84 },
    { 0x46400d0, 95 },
    { 0x550400d1, 90 },
    { 0, 0 },
    { 0xdcb400d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791400d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8400e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a400ea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d400f0, 84 },
    { 0x648400f0, 23 },
    { 0x2e1400f2, 23 },
    { 0x668400f0, 33 },
    { 0xdfe400f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87400f8, 61 },
    { 0, 0 },
    { 0x58a400fa, 40 },
    { 0xc25400fb, 85 },
    { 0x890400fc, 93 },
    { 0x813400fd, 23 },
    { 0x1fd400fe, 61 },
    { 0, 0 },
    { 0xe1140100, 55 },
    { 0xff140100, 34 },
    { 0x3d440102, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20040109, 98 },
    { 0, 0 },
    { 0x4124010b, 75 },
    { 0x2e24010b, 98 },
    { 0x8ca4010d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9640110, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30840114, 90 },
    { 0xd3b40114, 33 },
    { 0xfd940115, 95 },
    { 0x1b440116, 34 },
    { 0, 0 },
    { 0x44140119, 46 },
    { 0xa9e4011a, 33 },
    { 0x79f4011b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e40120, 23 },
    { 0x82e40121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4840124, 33 },
    { 0, 0 },
    { 0xa2b40126, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d40130, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d40134, 95 },
    { 0, 0 },
    { 0x41240136, 61 },
    { 0, 0 },
    { 0x14840138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f44013f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0440142, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46540149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa014014d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8940154, 33 },
    { 0xc0140154, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab40159, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea40161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e740168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d40171, 46 },
    { 0x70640172, 96 },
    { 0x3840173, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2840179, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1244017c, 34 },
    { 0x384017c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf840185, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e4018b, 61 },
    { 0xd824018c, 91 },
    { 0x3114018d, 61 },
    { 0x1db4018e, 90 },
    { 0x8cf4018f, 90 },
    { 0x4934018f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f140193, 43 },
    { 0xbd640194, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe614019b, 23 },
    { 0, 0 },
    { 0x99d4019d, 66 },
    { 0x6e04019d, 23 },
    { 0x2304019d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5401ac, 95 },
    { 0xf8e401ad, 33 },
    { 0xd5a401ae, 43 },
    { 0x53e401ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6401b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac401b6, 33 },
    { 0x1b9401b7, 87 },
    { 0x6e4401b7, 100 },
    { 0xb77401b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be401c1, 93 },
    { 0, 0 },
    { 0xc13401c3, 76 },
    { 0xb96401c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb401c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x287401cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f401d1, 76 },
    { 0, 0 },
    { 0xb58401d3, 33 },
    { 0x14b401d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937401dc, 76 },
    { 0, 0 },
    { 0xf3e401de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f401e1, 33 },
    { 0x3b8401e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb401e8, 95 },
    { 0, 0 },
    { 0xf58401ea, 90 },
    { 0x193401eb, 99 },
    { 0x5ab401ec, 95 },
    { 0x370401ec, 90 },
    { 0x6f3401ee, 95 },
    { 0xf41401ef, 19 },
    { 0xeb6401ef, 33 },
    { 0x322401f1, 90 },
    { 0x722401ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4401f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4401fa, 23 },
    { 0, 0 },
    { 0xb66401fc, 12 },
    { 0xe5d401fd, 93 },
    { 0xabe401fd, 23 },
    { 0xfd0401ff, 100 },
    { 0, 0 },
    { 0x1d440201, 76 },
    { 0x29840202, 23 },
    { 0x11440203, 90 },
    { 0x71140204, 100 },
    { 0xe0a40204, 55 },
    { 0x43c40203, 76 },
    { 0x8d340205, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4020d, 46 },
    { 0x6eb4020e, 99 },
    { 0, 0 },
    { 0x7d040210, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c40214, 84 },
    { 0, 0 },
    { 0x24a40216, 98 },
    { 0xf3e40217, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed4021c, 43 },
    { 0x11d4021d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b40220, 61 },
    { 0xb8c40221, 67 },
    { 0x8fa40221, 33 },
    { 0x4d240222, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d40226, 98 },
    { 0, 0 },
    { 0x1e340228, 90 },
    { 0x58f40229, 90 },
    { 0x78040228, 78 },
    { 0xe8d4022a, 78 },
    { 0, 0 },
    { 0xd294022d, 95 },
    { 0x7b4022e, 95 },
    { 0, 0 },
    { 0xde840230, 98 },
    { 0x68a40231, 66 },
    { 0xa3f40231, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b40236, 93 },
    { 0, 0 },
    { 0xfa840238, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e84023e, 66 },
    { 0, 0 },
    { 0xca640240, 95 },
    { 0x3b640240, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6540246, 77 },
    { 0x93340246, 55 },
    { 0xee840247, 95 },
    { 0xaff40249, 90 },
    { 0x2db4024a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb34024e, 90 },
    { 0x5ab4024f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7740252, 90 },
    { 0x5b140252, 33 },
    { 0x90140254, 84 },
    { 0xa240253, 95 },
    { 0, 0 },
    { 0x2d440257, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa944025a, 90 },
    { 0, 0 },
    { 0x50c4025c, 87 },
    { 0, 0 },
    { 0x3a14025e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48640267, 88 },
    { 0x48c40267, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d4026b, 87 },
    { 0, 0 },
    { 0x97c4026d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d40270, 90 },
    { 0x52040271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df40275, 98 },
    { 0xd4540276, 85 },
    { 0, 0 },
    { 0x1a140278, 55 },
    { 0, 0 },
    { 0xbd64027a, 61 },
    { 0x6204027b, 95 },
    { 0x6484027a, 75 },
    { 0x7794027d, 46 },
    { 0x3c84027a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca40282, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c40285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb4028a, 84 },
    { 0x17e4028b, 90 },
    { 0x66f4028b, 95 },
    { 0x41f4028d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f40290, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e40294, 91 },
    { 0x9cf40295, 90 },
    { 0xc8340295, 47 },
    { 0x2e640297, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de4029b, 90 },
    { 0x97b4029b, 43 },
    { 0x5f14029c, 90 },
    { 0, 0 },
    { 0x214029f, 43 },
    { 0x516402a0, 90 },
    { 0x2d3402a1, 90 },
    { 0x4d44029f, 76 },
    { 0x1c4402a3, 90 },
    { 0xdfe402a4, 76 },
    { 0x30b4029f, 33 },
    { 0, 0 },
    { 0x5c2402a7, 95 },
    { 0xa16402a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x334402ab, 98 },
    { 0, 0 },
    { 0x408402ad, 95 },
    { 0x66c402ae, 81 },
    { 0x263402ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a402b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433402b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e402b9, 93 },
    { 0x157402ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e402be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb402c3, 100 },
    { 0xe0402c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca402c7, 33 },
    { 0x403402c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a402cc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90402d1, 81 },
    { 0xbc6402d2, 76 },
    { 0x9e2402d2, 76 },
    { 0x67e402d1, 90 },
    { 0x17402d5, 66 },
    { 0xcae402d5, 46 },
    { 0xd20402d6, 100 },
    { 0, 0 },
    { 0xa58402d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc402dc, 23 },
    { 0xcb2402dd, 65 },
    { 0, 0 },
    { 0x697402df, 88 },
    { 0x524402df, 78 },
    { 0xa6b402e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f402e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff402e8, 78 },
    { 0x99a402e9, 23 },
    { 0xe0e402ea, 91 },
    { 0x182402eb, 78 },
    { 0x354402ea, 43 },
    { 0xf7a402ea, 36 },
    { 0xb21402ee, 78 },
    { 0x300402ef, 33 },
    { 0xf9a402eb, 61 },
    { 0x459402ef, 90 },
    { 0x1f2402f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f402f5, 98 },
    { 0, 0 },
    { 0x112402f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba402fc, 31 },
    { 0x5f5402fc, 91 },
    { 0x634402fe, 23 },
    { 0x9ff402fe, 95 },
    { 0x1fc40300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x65140303, 61 },
    { 0x97940304, 33 },
    { 0x55740304, 93 },
    { 0, 0 },
    { 0xdf440307, 23 },
    { 0xbb140307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb794030b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39140310, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9740313, 76 },
    { 0x32b40314, 95 },
    { 0, 0 },
    { 0x1ea40316, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7840319, 67 },
    { 0x3b84031a, 78 },
    { 0x2924031b, 90 },
    { 0x5474031c, 93 },
    { 0xbf64031c, 76 },
    { 0xa264031c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6340323, 76 },
    { 0xa8540324, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed40327, 23 },
    { 0x1740328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14032c, 33 },
    { 0x7b4032c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc40330, 90 },
    { 0x87940330, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e40336, 76 },
    { 0, 0 },
    { 0xb1a40338, 84 },
    { 0x68040339, 17 },
    { 0xf5e40338, 55 },
    { 0xb2b4033b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69140340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3440344, 77 },
    { 0x7f640345, 78 },
    { 0, 0 },
    { 0x21f40347, 23 },
    { 0, 0 },
    { 0xe6940349, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4014034e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59140355, 90 },
    { 0, 0 },
    { 0xc9840357, 33 },
    { 0xe5f40358, 33 },
    { 0x2e840359, 17 },
    { 0x7514035a, 90 },
    { 0x35a40358, 98 },
    { 0, 0 },
    { 0xbbf4035d, 76 },
    { 0xa704035e, 33 },
    { 0xc6d4035f, 95 },
    { 0, 0 },
    { 0xf5240361, 95 },
    { 0xb240361, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1584036f, 23 },
    { 0xf1340370, 90 },
    { 0x71e40371, 23 },
    { 0xab540372, 90 },
    { 0x1f640373, 74 },
    { 0xb5340373, 90 },
    { 0x9f140374, 68 },
    { 0x96640376, 100 },
    { 0x94d40377, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5174037a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb04037e, 98 },
    { 0, 0 },
    { 0x10240380, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d740384, 90 },
    { 0xf3640385, 34 },
    { 0x25f40386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18840389, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af40396, 95 },
    { 0xcb40396, 87 },
    { 0xacb40398, 33 },
    { 0, 0 },
    { 0xeb64039a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c34039d, 23 },
    { 0x1e84039e, 8 },
    { 0x8344039e, 99 },
    { 0, 0 },
    { 0x48e403a1, 44 },
    { 0, 0 },
    { 0x86a403a3, 33 },
    { 0, 0 },
    { 0x101403a5, 90 },
    { 0x9c8403a6, 34 },
    { 0x5d5403a7, 95 },
    { 0xffa403a8, 33 },
    { 0xed1403a8, 84 },
    { 0x35c403a6, 46 },
    { 0, 0 },
    { 0x75e403ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87403b0, 33 },
    { 0xf57403b0, 76 },
    { 0x942403b0, 24 },
    { 0x4a7403b3, 76 },
    { 0, 0 },
    { 0xb70403b5, 37 },
    { 0x3e3403b6, 88 },
    { 0x5c5403b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8403bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637403c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538403c4, 55 },
    { 0x4db403c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6403c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd403cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8403cf, 46 },
    { 0, 0 },
    { 0xdfd403d1, 61 },
    { 0x935403d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0403d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77403de, 91 },
    { 0x2c7403de, 33 },
    { 0xcc8403df, 93 },
    { 0xd70403df, 43 },
    { 0xa7403df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c403e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67403f0, 23 },
    { 0xacb403f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab403f4, 64 },
    { 0xcdc403f5, 61 },
    { 0x4ba403f4, 93 },
    { 0xc7b403f4, 33 },
    { 0, 0 },
    { 0x360403f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1403fd, 91 },
    { 0x31d403fe, 61 },
    { 0, 0 },
    { 0xd9e40400, 88 },
    { 0x1940401, 76 },
    { 0x47d40402, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e40405, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7004040b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1754040e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83140416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc4041c, 90 },
    { 0, 0 },
    { 0xddb4041e, 23 },
    { 0x8314041f, 98 },
    { 0xf6f4041f, 67 },
    { 0x3b740420, 46 },
    { 0xdd84041e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d940429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b04042e, 43 },
    { 0x3514042f, 52 },
    { 0x62240430, 33 },
    { 0x4eb40430, 95 },
    { 0x6fc40432, 55 },
    { 0x8a140432, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa724043a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d40440, 95 },
    { 0xe6b40441, 58 },
    { 0x35140441, 23 },
    { 0, 0 },
    { 0x89240444, 36 },
    { 0xebe40445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63640452, 33 },
    { 0x4f640453, 90 },
    { 0x8d140454, 98 },
    { 0x1da40455, 94 },
    { 0x80b40456, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4045f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e40463, 90 },
    { 0x59840463, 90 },
    { 0, 0 },
    { 0x91240466, 95 },
    { 0x8e240466, 43 },
    { 0xabf40468, 66 },
    { 0, 0 },
    { 0xe0b4046a, 61 },
    { 0, 0 },
    { 0x9e84046c, 55 },
    { 0xec94046c, 98 },
    { 0xaa74046e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d40473, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc24047d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f40480, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7540488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc174048c, 33 },
    { 0xe374048d, 52 },
    { 0x3784048d, 95 },
    { 0xe3c4048f, 100 },
    { 0xd134048e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a40497, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e4049c, 81 },
    { 0xab04049d, 61 },
    { 0x2b44049c, 33 },
    { 0x8d4049c, 76 },
    { 0xff4404a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0404a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7404a8, 77 },
    { 0, 0 },
    { 0xa46404aa, 61 },
    { 0x89e404ab, 40 },
    { 0x93f404ac, 95 },
    { 0, 0 },
    { 0x4cb404ae, 34 },
    { 0, 0 },
    { 0x84d404b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0404b5, 23 },
    { 0x108404b5, 55 },
    { 0x17a404b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e404bc, 90 },
    { 0, 0 },
    { 0xd04404be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x519404c1, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54404c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff404ca, 66 },
    { 0, 0 },
    { 0x9d4404cc, 95 },
    { 0xd1f404cd, 33 },
    { 0x93f404ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5404df, 34 },
    { 0, 0 },
    { 0xfe1404e1, 98 },
    { 0, 0 },
    { 0x423404e3, 90 },
    { 0x9dd404e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x651404e7, 90 },
    { 0, 0 },
    { 0x201404e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727404f5, 93 },
    { 0xa4f404f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26404fa, 33 },
    { 0x6d2404fb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x227404fe, 90 },
    { 0x2cf404fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e40502, 33 },
    { 0x5d840503, 81 },
    { 0xfaf40503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d440507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a4050c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214050f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9140512, 24 },
    { 0x1b740513, 90 },
    { 0x34c40513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45140517, 76 },
    { 0xc5040518, 55 },
    { 0x1dd40518, 99 },
    { 0x79d4051a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3140520, 55 },
    { 0x2f40520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70040525, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5254052c, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e40531, 43 },
    { 0, 0 },
    { 0xc3d40533, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb474053a, 85 },
    { 0xfc14053b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e4053e, 61 },
    { 0x9084053e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b40543, 98 },
    { 0x85840544, 46 },
    { 0xc540545, 57 },
    { 0x31740546, 34 },
    { 0, 0 },
    { 0xc0a40548, 86 },
    { 0xe3940549, 100 },
    { 0, 0 },
    { 0x3284054b, 93 },
    { 0x2f74054c, 98 },
    { 0xdda4054d, 64 },
    { 0x14e4054d, 17 },
    { 0, 0 },
    { 0xda340550, 76 },
    { 0, 0 },
    { 0x2e40552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5540557, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f4055b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d4055e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4240565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae4056a, 90 },
    { 0x1134056b, 23 },
    { 0xd4d4056c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb124056f, 76 },
    { 0x41240570, 33 },
    { 0xc8a40571, 61 },
    { 0xc7940571, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d40575, 33 },
    { 0x33240576, 91 },
    { 0, 0 },
    { 0x9b040578, 33 },
    { 0, 0 },
    { 0xd174057a, 65 },
    { 0x27f4057b, 61 },
    { 0, 0 },
    { 0xacf4057d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe140583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4240586, 23 },
    { 0, 0 },
    { 0xd9940588, 99 },
    { 0x6ec40589, 33 },
    { 0, 0 },
    { 0xca4058b, 95 },
    { 0xf6f4058c, 23 },
    { 0xd6b4058b, 46 },
    { 0x6034058e, 90 },
    { 0x3de4058e, 76 },
    { 0x88c40590, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22040594, 43 },
    { 0x9c540595, 66 },
    { 0, 0 },
    { 0x7b340597, 90 },
    { 0x7bd40598, 8 },
    { 0x60040599, 98 },
    { 0xad740598, 84 },
    { 0xdba4059b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8405a0, 33 },
    { 0, 0 },
    { 0x503405a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3405a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608405ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375405b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b405b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1405bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b405c0, 95 },
    { 0x76c405c1, 61 },
    { 0, 0 },
    { 0x408405c3, 93 },
    { 0x3a8405c3, 65 },
    { 0xe61405c5, 61 },
    { 0x266405c4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07405cb, 87 },
    { 0x173405cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb405d4, 90 },
    { 0xd89405d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a405e3, 100 },
    { 0xa38405e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7405e8, 90 },
    { 0xf1b405e9, 88 },
    { 0xeb6405ea, 66 },
    { 0xea9405eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e405ee, 90 },
    { 0xe28405ee, 33 },
    { 0, 0 },
    { 0x1c6405f1, 33 },
    { 0xd79405f2, 67 },
    { 0, 0 },
    { 0x380405f4, 95 },
    { 0, 0 },
    { 0x58d405f6, 93 },
    { 0, 0 },
    { 0xb30405f8, 23 },
    { 0x595405f9, 34 },
    { 0x456405f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8405ff, 33 },
    { 0x70c40600, 33 },
    { 0x81540601, 61 },
    { 0x2b040601, 91 },
    { 0xdd540603, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d40609, 95 },
    { 0, 0 },
    { 0xf844060b, 33 },
    { 0xd24060c, 69 },
    { 0x6ab4060d, 84 },
    { 0x2034060e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f140614, 90 },
    { 0x97f40615, 98 },
    { 0x8c940616, 33 },
    { 0, 0 },
    { 0xcbc40618, 61 },
    { 0x10640619, 34 },
    { 0, 0 },
    { 0x324061b, 76 },
    { 0xeec4061c, 33 },
    { 0xcf44061c, 85 },
    { 0, 0 },
    { 0x3f34061f, 88 },
    { 0, 0 },
    { 0x30840621, 34 },
    { 0x26b40622, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d40628, 33 },
    { 0xd340629, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f4062d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40631, 90 },
    { 0xffd40632, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1140639, 23 },
    { 0xc7c40639, 46 },
    { 0x3dc4063a, 43 },
    { 0xc4040639, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b40641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e94064a, 55 },
    { 0, 0 },
    { 0xdad4064c, 98 },
    { 0x9294064c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce4065a, 84 },
    { 0xb14065b, 90 },
    { 0, 0 },
    { 0x2d64065d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1940662, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd40667, 90 },
    { 0, 0 },
    { 0x7f540669, 55 },
    { 0, 0 },
    { 0x5a14066b, 88 },
    { 0xa044066b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf684066f, 85 },
    { 0, 0 },
    { 0xd8640671, 98 },
    { 0x52640671, 46 },
    { 0x4d240671, 93 },
    { 0x42540674, 61 },
    { 0x1c740675, 100 },
    { 0x93e40676, 98 },
    { 0xe240671, 42 },
    { 0x2df40678, 51 },
    { 0xa7740679, 61 },
    { 0x12540677, 90 },
    { 0x9ee4067b, 98 },
    { 0x7f74067b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8364067f, 66 },
    { 0, 0 },
    { 0x45040681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41140685, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2340688, 99 },
    { 0, 0 },
    { 0xa424068a, 87 },
    { 0xc454068a, 99 },
    { 0x1304068a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a40695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd94069a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90406a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26406a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb406a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6406aa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb406af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0406b4, 90 },
    { 0, 0 },
    { 0x6c8406b6, 68 },
    { 0, 0 },
    { 0xb0a406b8, 69 },
    { 0x1b0406b9, 85 },
    { 0xf3d406ba, 55 },
    { 0x5e5406bb, 23 },
    { 0, 0 },
    { 0x8ea406bd, 34 },
    { 0x61406be, 33 },
    { 0x8c9406be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6406c2, 23 },
    { 0x924406c3, 61 },
    { 0x45b406c2, 100 },
    { 0x811406c5, 90 },
    { 0xaa8406c6, 83 },
    { 0x768406c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x506406ca, 46 },
    { 0, 0 },
    { 0x8dc406cc, 90 },
    { 0, 0 },
    { 0x620406ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123406d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7406d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0406dd, 55 },
    { 0x801406de, 61 },
    { 0xd63406df, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca406e7, 33 },
    { 0x731406e7, 95 },
    { 0xb19406e8, 95 },
    { 0xec2406ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b406ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914406fb, 90 },
    { 0x4d8406fb, 98 },
    { 0x96406fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf040700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b4070a, 77 },
    { 0xd104070b, 93 },
    { 0xf054070c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4540712, 43 },
    { 0xc2640713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c4071a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f240721, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91540726, 61 },
    { 0x40740727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3704072b, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90340730, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32040738, 84 },
    { 0, 0 },
    { 0x2fd4073a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2194073d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40940743, 52 },
    { 0xd1f40744, 77 },
    { 0x5c940744, 85 },
    { 0xd0640746, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4074c, 93 },
    { 0x6e24074c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a940752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a140757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4075c, 65 },
    { 0x9f74075d, 81 },
    { 0, 0 },
    { 0x434075f, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c40763, 76 },
    { 0x42c40764, 23 },
    { 0, 0 },
    { 0xf8940766, 23 },
    { 0x77640767, 68 },
    { 0xadb40768, 64 },
    { 0xb1c40769, 76 },
    { 0x9b84076a, 90 },
    { 0xb1e4076a, 55 },
    { 0xa4d4076b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f34076f, 98 },
    { 0x22540770, 80 },
    { 0x4884076f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4240774, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec40777, 77 },
    { 0xc0f40777, 61 },
    { 0, 0 },
    { 0xa0b4077a, 90 },
    { 0x9024077b, 80 },
    { 0x7c04077c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42440784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc424078b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x974078e, 90 },
    { 0xac74078f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e40792, 33 },
    { 0, 0 },
    { 0xfc340794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a54079a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f4079d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc407a1, 68 },
    { 0, 0 },
    { 0x9a2407a3, 67 },
    { 0, 0 },
    { 0xa9a407a5, 33 },
    { 0x211407a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804407aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b407ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31407b1, 19 },
    { 0xe32407b2, 98 },
    { 0xc62407b3, 93 },
    { 0, 0 },
    { 0x9a7407b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd407c0, 33 },
    { 0, 0 },
    { 0x6d8407c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c407c5, 33 },
    { 0x4a7407c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d407cb, 61 },
    { 0x24a407cc, 67 },
    { 0x9a0407cd, 64 },
    { 0x664407cc, 99 },
    { 0xa8e407cf, 66 },
    { 0, 0 },
    { 0xb75407d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c407d4, 33 },
    { 0xe3407d5, 33 },
    { 0x8bd407d4, 98 },
    { 0, 0 },
    { 0x1a9407d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60407de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x349407e1, 81 },
    { 0x69b407e1, 33 },
    { 0, 0 },
    { 0xc1407e4, 33 },
    { 0xebb407e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448407ea, 55 },
    { 0x631407eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d407ef, 90 },
    { 0xff1407ef, 33 },
    { 0x34407ef, 90 },
    { 0xc64407f2, 55 },
    { 0xec2407f2, 76 },
    { 0x3cb407f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de407f7, 34 },
    { 0x6e7407f7, 33 },
    { 0x97c407f9, 87 },
    { 0x613407f8, 87 },
    { 0x1b6407f7, 95 },
    { 0x26407f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91407ff, 95 },
    { 0xe5840800, 33 },
    { 0xa1040800, 33 },
    { 0, 0 },
    { 0x71040803, 33 },
    { 0xe4540804, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb14080a, 87 },
    { 0xa4f4080b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b4080f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2840812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30340815, 95 },
    { 0x34840816, 46 },
    { 0, 0 },
    { 0xd8540818, 61 },
    { 0, 0 },
    { 0x64f4081a, 61 },
    { 0xc244081b, 90 },
    { 0, 0 },
    { 0x5634081d, 76 },
    { 0, 0 },
    { 0x58f4081f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee740822, 95 },
    { 0, 0 },
    { 0x4a940824, 95 },
    { 0x5b140825, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4082b, 66 },
    { 0xc44082c, 94 },
    { 0x54f4082c, 61 },
    { 0, 0 },
    { 0x2b24082f, 87 },
    { 0x7774082f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e40837, 61 },
    { 0, 0 },
    { 0xc4c40839, 81 },
    { 0x96c40839, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4083d, 90 },
    { 0x8424083e, 33 },
    { 0x1834083f, 61 },
    { 0x59840840, 90 },
    { 0x5f940840, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1240844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d140847, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7634084c, 78 },
    { 0xe874084d, 98 },
    { 0x1244084e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36740853, 33 },
    { 0x18e40854, 99 },
    { 0x73240853, 76 },
    { 0, 0 },
    { 0x47040857, 38 },
    { 0, 0 },
    { 0x27540859, 98 },
    { 0xc194085a, 33 },
    { 0xec34085b, 90 },
    { 0x63e4085b, 98 },
    { 0xc194085d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c40862, 33 },
    { 0, 0 },
    { 0xa1940864, 52 },
    { 0xe4540865, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14086b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a540871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c40875, 61 },
    { 0x9c940875, 98 },
    { 0x10a40877, 66 },
    { 0xbd140877, 17 },
    { 0x2c840879, 95 },
    { 0x2c640877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe204087d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e440881, 33 },
    { 0x2d40882, 33 },
    { 0x73e40881, 95 },
    { 0, 0 },
    { 0x9d40885, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d40889, 95 },
    { 0, 0 },
    { 0x22e4088b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2140891, 100 },
    { 0, 0 },
    { 0x52e40893, 43 },
    { 0x65640893, 34 },
    { 0x4f340894, 95 },
    { 0x70240896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff4089c, 43 },
    { 0xdc74089d, 61 },
    { 0xeb84089e, 34 },
    { 0x3d84089f, 40 },
    { 0x1c4408a0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b408a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8408a9, 33 },
    { 0, 0 },
    { 0xcce408ab, 90 },
    { 0, 0 },
    { 0x248408ad, 40 },
    { 0, 0 },
    { 0xbf1408af, 90 },
    { 0, 0 },
    { 0xb6d408b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5408b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f408bc, 99 },
    { 0xb22408bc, 33 },
    { 0xd11408bd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7408c6, 52 },
    { 0x119408c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1408ce, 96 },
    { 0x92f408ce, 98 },
    { 0x139408d0, 84 },
    { 0xf32408d0, 36 },
    { 0x2f9408d0, 33 },
    { 0xc37408d0, 23 },
    { 0x122408d0, 55 },
    { 0xc18408d1, 90 },
    { 0x8a3408d5, 95 },
    { 0xb63408d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x702408da, 91 },
    { 0x6c3408da, 100 },
    { 0x5e6408dc, 52 },
    { 0, 0 },
    { 0x316408de, 33 },
    { 0xf74408de, 67 },
    { 0, 0 },
    { 0x312408e1, 52 },
    { 0xc94408e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc408e5, 93 },
    { 0x822408e5, 90 },
    { 0, 0 },
    { 0x106408e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e408ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994408f4, 90 },
    { 0x4e2408f5, 34 },
    { 0x4d408f5, 23 },
    { 0xb37408f6, 34 },
    { 0x7e7408f8, 76 },
    { 0x579408f8, 33 },
    { 0x76b408f6, 90 },
    { 0x67b408fb, 91 },
    { 0x3b9408fc, 61 },
    { 0x4ec408f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e40900, 95 },
    { 0xc0440901, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1140906, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d40909, 95 },
    { 0xdc140909, 69 },
    { 0xafa4090b, 76 },
    { 0x4b64090c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x104090f, 33 },
    { 0x25440910, 87 },
    { 0xf6c40911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2140915, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4091f, 81 },
    { 0, 0 },
    { 0xc840921, 61 },
    { 0x38440922, 33 },
    { 0x31c40923, 80 },
    { 0x9d040924, 95 },
    { 0x36c40924, 90 },
    { 0x6b440926, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e4092b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8264092e, 23 },
    { 0, 0 },
    { 0x33440930, 94 },
    { 0, 0 },
    { 0x26840932, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab340939, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f4093f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b940946, 34 },
    { 0x61340947, 76 },
    { 0, 0 },
    { 0x22340949, 46 },
    { 0, 0 },
    { 0xea4094b, 98 },
    { 0x1a04094b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a240950, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd40959, 93 },
    { 0x6824095a, 55 },
    { 0xf7640959, 34 },
    { 0xa704095b, 99 },
    { 0x9f44095a, 76 },
    { 0x8d64095e, 87 },
    { 0x2e64095b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90040966, 90 },
    { 0, 0 },
    { 0xeed40968, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda84096f, 33 },
    { 0xee140970, 80 },
    { 0x24d40970, 33 },
    { 0xb2a40972, 76 },
    { 0x4b740973, 52 },
    { 0x9c140972, 78 },
    { 0, 0 },
    { 0x1b640976, 61 },
    { 0, 0 },
    { 0xe5040978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf4097c, 95 },
    { 0x5b04097d, 98 },
    { 0, 0 },
    { 0xb854097f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36340983, 77 },
    { 0xf9340984, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954098c, 95 },
    { 0x73d4098d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d40993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e640996, 76 },
    { 0, 0 },
    { 0x40940998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f54099c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x994099f, 33 },
    { 0, 0 },
    { 0xe2c409a1, 90 },
    { 0x82f409a1, 87 },
    { 0x610409a3, 85 },
    { 0xbb8409a4, 43 },
    { 0x1f9409a5, 76 },
    { 0, 0 },
    { 0x103409a7, 33 },
    { 0x678409a8, 90 },
    { 0, 0 },
    { 0x433409aa, 61 },
    { 0xc19409ab, 66 },
    { 0, 0 },
    { 0x4fa409ad, 95 },
    { 0x4e6409ae, 23 },
    { 0x26f409ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776409b3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c409bd, 55 },
    { 0x4ea409bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365409c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30409c5, 61 },
    { 0xf27409c6, 84 },
    { 0xa46409c5, 97 },
    { 0xa0e409c7, 84 },
    { 0, 0 },
    { 0x681409ca, 94 },
    { 0, 0 },
    { 0x82409cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94409d1, 98 },
    { 0xd9e409d2, 90 },
    { 0xa05409d1, 68 },
    { 0, 0 },
    { 0x984409d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x945409d8, 34 },
    { 0, 0 },
    { 0x815409da, 53 },
    { 0xc5c409da, 85 },
    { 0x6c409dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49409df, 52 },
    { 0xeaf409df, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b409e8, 90 },
    { 0xd0d409e9, 33 },
    { 0x232409ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3409ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01409f5, 23 },
    { 0, 0 },
    { 0xecb409f7, 93 },
    { 0, 0 },
    { 0xe60409f9, 61 },
    { 0xb69409fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62409ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d40a02, 33 },
    { 0x3db40a02, 34 },
    { 0x10540a02, 61 },
    { 0xe7140a05, 90 },
    { 0, 0 },
    { 0x8d140a07, 100 },
    { 0, 0 },
    { 0x35040a09, 98 },
    { 0x58940a0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44540a0d, 95 },
    { 0x72640a0e, 76 },
    { 0xa7840a0f, 90 },
    { 0x84540a0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36640a14, 67 },
    { 0xab140a14, 76 },
    { 0xf8c40a16, 96 },
    { 0x69540a16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e40a1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3940a21, 90 },
    { 0, 0 },
    { 0x61f40a23, 61 },
    { 0x2ff40a24, 93 },
    { 0, 0 },
    { 0xa6c40a26, 88 },
    { 0xa4b40a26, 76 },
    { 0, 0 },
    { 0xf0040a29, 43 },
    { 0x7040a2a, 93 },
    { 0xe3e40a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41240a38, 55 },
    { 0, 0 },
    { 0xc2840a3a, 99 },
    { 0x2c740a3b, 87 },
    { 0xf3340a3c, 64 },
    { 0x93b40a3d, 76 },
    { 0x78540a3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a040a42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc040a4d, 90 },
    { 0x70440a4e, 76 },
    { 0x13c40a4f, 76 },
    { 0x99340a4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca40a58, 90 },
    { 0xcab40a59, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2840a5e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64740a63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f040a67, 34 },
    { 0xcf740a68, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c840a70, 90 },
    { 0, 0 },
    { 0x1c240a72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4440a75, 33 },
    { 0, 0 },
    { 0x8a140a77, 43 },
    { 0, 0 },
    { 0xd3640a79, 43 },
    { 0x57f40a7a, 33 },
    { 0x8f640a79, 67 },
    { 0, 0 },
    { 0x18540a7d, 90 },
    { 0xb6940a7e, 98 },
    { 0x8c840a7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c40a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e640a88, 81 },
    { 0x48f40a89, 91 },
    { 0, 0 },
    { 0x92240a8b, 33 },
    { 0x1f740a8b, 76 },
    { 0xd040a8c, 90 },
    { 0x20b40a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74640a92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50240a96, 55 },
    { 0xb3f40a97, 53 },
    { 0, 0 },
    { 0xc6540a99, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a40a9e, 85 },
    { 0x42740a9f, 59 },
    { 0xece40aa0, 34 },
    { 0xf9940aa1, 90 },
    { 0x8d240aa2, 52 },
    { 0x39840aa3, 95 },
    { 0xae240aa4, 33 },
    { 0xae340aa4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94640aa8, 90 },
    { 0xe3f40aa8, 84 },
    { 0xd1b40aa9, 98 },
    { 0x59f40aa8, 61 },
    { 0, 0 },
    { 0x2a740aad, 45 },
    { 0, 0 },
    { 0x6a140aaf, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x5340ab2, 61 },
    { 0x81240ab3, 61 },
    { 0, 0 },
    { 0x13840ab5, 67 },
    { 0xf3740ab5, 23 },
    { 0, 0 },
    { 0x45a40ab8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70240abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3940abe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4040ac1, 55 },
    { 0xada40ac2, 75 },
    { 0, 0 },
    { 0x71040ac4, 23 },
    { 0xbef40ac5, 61 },
    { 0xa6540ac4, 78 },
    { 0xcd440ac6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c540aca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25540acf, 34 },
    { 0xe8640acf, 43 },
    { 0, 0 },
    { 0x25240ad2, 61 },
    { 0x13640ad3, 55 },
    { 0x61d40ad4, 53 },
    { 0xbdd40ad4, 90 },
    { 0x94b40ad6, 61 },
    { 0x97040ad7, 85 },
    { 0, 0 },
    { 0x7cb40ad9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa740adc, 36 },
    { 0xe740add, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a40ae9, 43 },
    { 0, 0 },
    { 0xfd040aeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c40aee, 98 },
    { 0, 0 },
    { 0x8b40af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b40af4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f40af9, 65 },
    { 0xe7440afa, 93 },
    { 0x2fc40af9, 55 },
    { 0, 0 },
    { 0x26b40afd, 100 },
    { 0x12c40afe, 33 },
    { 0xbb640aff, 61 },
    { 0, 0 },
    { 0x4b940b01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e940b06, 93 },
    { 0x1b40b07, 95 },
    { 0xb0a40b07, 46 },
    { 0, 0 },
    { 0xfc840b0a, 90 },
    { 0, 0 },
    { 0x3ff40b0c, 33 },
    { 0x37240b0c, 95 },
    { 0xdd40b0c, 55 },
    { 0xb0740b0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f40b14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea40b17, 61 },
    { 0xd2140b18, 67 },
    { 0x46540b19, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92940b20, 17 },
    { 0xee540b21, 66 },
    { 0, 0 },
    { 0xfb240b23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66140b27, 90 },
    { 0, 0 },
    { 0xc6440b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb40b2e, 98 },
    { 0x4a740b2e, 77 },
    { 0, 0 },
    { 0x5240b31, 76 },
    { 0x5de40b32, 98 },
    { 0xf8a40b32, 43 },
    { 0xb5a40b32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd140b37, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f40b42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67740b4d, 90 },
    { 0x98540b4e, 33 },
    { 0xb40b4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a640b55, 33 },
    { 0, 0 },
    { 0x1640b57, 34 },
    { 0x5e840b58, 61 },
    { 0, 0 },
    { 0x65b40b5a, 33 },
    { 0xfdb40b5a, 55 },
    { 0x8b340b5b, 98 },
    { 0x67d40b5d, 23 },
    { 0xb8440b5d, 55 },
    { 0x46140b5f, 23 },
    { 0xf1740b5d, 61 },
    { 0x1d240b61, 33 },
    { 0xf1440b5f, 90 },
    { 0xf2540b5e, 90 },
    { 0, 0 },
    { 0x12140b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e40b6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd40b72, 33 },
    { 0, 0 },
    { 0xa0240b74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f40b79, 68 },
    { 0xf440b7a, 90 },
    { 0, 0 },
    { 0x4d440b7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e540b84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37240b92, 23 },
    { 0xad740b93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e40b99, 76 },
    { 0x57940b99, 95 },
    { 0xea740b9b, 95 },
    { 0xbc240b9c, 95 },
    { 0xbe040b9d, 33 },
    { 0xebb40b9e, 34 },
    { 0x78640b9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81240ba2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7540baa, 61 },
    { 0x67940bab, 88 },
    { 0xa9940bac, 61 },
    { 0x21e40bad, 61 },
    { 0x1fc40bae, 98 },
    { 0x17340bae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d40bb4, 61 },
    { 0x75b40bb4, 23 },
    { 0xe2240bb6, 61 },
    { 0x35a40bb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73440bba, 61 },
    { 0x9940bbb, 95 },
    { 0xd6940bbc, 33 },
    { 0xf2540bbd, 33 },
    { 0x5a840bbb, 23 },
    { 0xbef40bbf, 33 },
    { 0x8d540bbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb40bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf840bc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb40bd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c640bd9, 66 },
    { 0x79a40bda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a740bdd, 66 },
    { 0, 0 },
    { 0x2c640bdf, 95 },
    { 0xa0540bdf, 90 },
    { 0x8ca40be1, 93 },
    { 0x8f340be1, 52 },
    { 0x8e40be3, 90 },
    { 0xd2c40be4, 34 },
    { 0x9d240be4, 90 },
    { 0x88640be1, 96 },
    { 0x10440be6, 55 },
    { 0, 0 },
    { 0xc4940be9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78840bee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed40bf1, 66 },
    { 0x63340bf1, 87 },
    { 0, 0 },
    { 0xdbc40bf4, 23 },
    { 0x9ea40bf5, 33 },
    { 0, 0 },
    { 0x42440bf7, 34 },
    { 0, 0 },
    { 0xf3840bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91540bfd, 23 },
    { 0x1ef40bfe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f40c05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf340c08, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e240c10, 66 },
    { 0x9d040c11, 61 },
    { 0xa7740c10, 61 },
    { 0xda140c13, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93140c18, 99 },
    { 0x8c640c19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8240c1d, 17 },
    { 0xe540c1e, 98 },
    { 0x34a40c1f, 81 },
    { 0x80240c20, 90 },
    { 0, 0 },
    { 0x7d340c22, 100 },
    { 0, 0 },
    { 0xfd940c24, 33 },
    { 0, 0 },
    { 0xffc40c26, 98 },
    { 0x62840c26, 85 },
    { 0, 0 },
    { 0x33e40c29, 98 },
    { 0, 0 },
    { 0x40540c2b, 77 },
    { 0x54440c2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50040c2f, 80 },
    { 0x1e140c30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x53440c33, 34 },
    { 0x7bd40c34, 52 },
    { 0xc6040c35, 90 },
    { 0xfcd40c36, 90 },
    { 0x83f40c34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xef340c3a, 61 },
    { 0x53540c3a, 61 },
    { 0x1d40c3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e40c3f, 93 },
    { 0x57a40c40, 78 },
    { 0, 0 },
    { 0xb2a40c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde40c4a, 95 },
    { 0x3a640c4b, 83 },
    { 0xcec40c4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea40c4f, 84 },
    { 0x4cd40c50, 90 },
    { 0x8c640c51, 98 },
    { 0x47940c51, 55 },
    { 0xd1d40c53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c40c56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7840c5c, 99 },
    { 0xfcd40c5c, 61 },
    { 0x69440c5d, 33 },
    { 0xf2040c5e, 95 },
    { 0x17040c5d, 76 },
    { 0, 0 },
    { 0xdc240c62, 42 },
    { 0x59940c63, 76 },
    { 0xb2040c64, 56 },
    { 0x30740c63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc440c6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e40c72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b740c76, 99 },
    { 0, 0 },
    { 0x15740c78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f140c81, 87 },
    { 0xd6840c82, 83 },
    { 0xdc740c82, 67 },
    { 0xc8740c83, 23 },
    { 0xbf740c85, 87 },
    { 0xac340c86, 83 },
    { 0x10f40c85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f240c8f, 90 },
    { 0x27e40c90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d40c93, 78 },
    { 0x32740c93, 95 },
    { 0x4c440c95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55740ca0, 66 },
    { 0x14f40ca1, 90 },
    { 0, 0 },
    { 0x1ab40ca3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d40caa, 23 },
    { 0, 0 },
    { 0x31840cac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b40caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f840cb3, 33 },
    { 0xa4040cb3, 95 },
    { 0x97740cb3, 34 },
    { 0xd4240cb5, 76 },
    { 0xe0f40cb7, 33 },
    { 0xe140cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd40cbc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b40cbf, 34 },
    { 0, 0 },
    { 0x46340cc1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e40cc7, 94 },
    { 0xf3740cc8, 98 },
    { 0x63740cc9, 90 },
    { 0x33f40cc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa440cce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40040cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1440cd6, 23 },
    { 0x59040cd7, 99 },
    { 0xf3d40cd7, 98 },
    { 0, 0 },
    { 0x20140cda, 67 },
    { 0x38e40cda, 76 },
    { 0x7640cda, 76 },
    { 0x89740cdd, 61 },
    { 0xd8f40cda, 90 },
    { 0xbcc40cdf, 98 },
    { 0x74440ce0, 33 },
    { 0xb4040cdf, 61 },
    { 0, 0 },
    { 0x6a440ce3, 66 },
    { 0x2a140ce3, 61 },
    { 0x6940ce3, 33 },
    { 0x75b40ce3, 34 },
    { 0, 0 },
    { 0x1e140ce8, 95 },
    { 0xa6a40ce9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd40cef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda040cfa, 61 },
    { 0x6bf40cfb, 43 },
    { 0x61440cfc, 23 },
    { 0xc9f40cfc, 99 },
    { 0xe6140cfe, 91 },
    { 0xcef40cfb, 23 },
    { 0xf2640d00, 33 },
    { 0, 0 },
    { 0x7e940d02, 84 },
    { 0xe6040d02, 61 },
    { 0x77140d04, 90 },
    { 0xe3140d05, 87 },
    { 0x10740d02, 61 },
    { 0x71040d06, 95 },
    { 0xb7940d08, 34 },
    { 0, 0 },
    { 0xee40d0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe840d0f, 23 },
    { 0x1ea40d10, 46 },
    { 0x83f40d11, 76 },
    { 0, 0 },
    { 0xded40d13, 95 },
    { 0, 0 },
    { 0x140d15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd40d18, 93 },
    { 0xb8040d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee140d24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37740d28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a540d2e, 60 },
    { 0x92440d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb40d35, 55 },
    { 0, 0 },
    { 0xd9740d37, 99 },
    { 0x3d40d38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b540d3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d40d41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49640d44, 33 },
    { 0, 0 },
    { 0xd3640d46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb440d51, 76 },
    { 0x70a40d51, 33 },
    { 0x61840d51, 87 },
    { 0x29840d52, 90 },
    { 0xecd40d55, 23 },
    { 0xa7640d56, 90 },
    { 0x2ad40d56, 38 },
    { 0, 0 },
    { 0x4eb40d59, 61 },
    { 0x1b340d5a, 100 },
    { 0x3f540d5b, 55 },
    { 0x10640d5b, 33 },
    { 0xe6f40d5c, 23 },
    { 0xaf340d5e, 76 },
    { 0xb0c40d5f, 34 },
    { 0, 0 },
    { 0xc5e40d61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5640d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6040d67, 66 },
    { 0x28540d67, 55 },
    { 0x1c40d68, 61 },
    { 0x76340d68, 76 },
    { 0x59840d68, 93 },
    { 0x5f840d6c, 19 },
    { 0xb740d67, 93 },
    { 0xef940d6e, 77 },
    { 0xe4840d6a, 43 },
    { 0x7a040d70, 76 },
    { 0xa5340d71, 99 },
    { 0xa1b40d71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a340d76, 34 },
    { 0, 0 },
    { 0xb8340d78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f740d7c, 46 },
    { 0xb5c40d7d, 66 },
    { 0xdf640d7e, 23 },
    { 0x86940d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d240d85, 78 },
    { 0, 0 },
    { 0x58f40d87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b140d8d, 33 },
    { 0, 0 },
    { 0xdd940d8f, 90 },
    { 0, 0 },
    { 0xc5d40d91, 23 },
    { 0xa1140d92, 100 },
    { 0xc1040d93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe40d9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da40da0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc740da4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d40db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d40db3, 34 },
    { 0x8cd40db4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe40db9, 33 },
    { 0xbd140dba, 77 },
    { 0xdc540dbb, 76 },
    { 0, 0 },
    { 0x36840dbd, 98 },
    { 0x61f40dbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f640dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd40dc5, 61 },
    { 0x4fd40dc5, 87 },
    { 0x74840dc7, 61 },
    { 0, 0 },
    { 0xc5740dc9, 76 },
    { 0x43440dca, 61 },
    { 0, 0 },
    { 0xbfa40dcc, 33 },
    { 0x68240dcc, 76 },
    { 0x28c40dcc, 90 },
    { 0, 0 },
    { 0x85b40dd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5440dd3, 76 },
    { 0x86d40dd3, 61 },
    { 0xcf140dd5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5040dda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6640dde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e40de6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc940de9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d040ded, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10240df0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb40df3, 100 },
    { 0xb2d40df3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0640df9, 61 },
    { 0, 0 },
    { 0xd2f40dfb, 95 },
    { 0x59240dfc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd940dff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c140e09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c140e0c, 55 },
    { 0, 0 },
    { 0xd2040e0e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5240e11, 34 },
    { 0x59540e12, 50 },
    { 0x54e40e11, 100 },
    { 0x7ce40e14, 90 },
    { 0x82c40e15, 84 },
    { 0xfcf40e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f340e19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b40e1c, 95 },
    { 0xd6740e1c, 23 },
    { 0xce40e1e, 76 },
    { 0, 0 },
    { 0x6a440e20, 90 },
    { 0x6ea40e20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee440e25, 76 },
    { 0xbad40e26, 61 },
    { 0, 0 },
    { 0xcf140e28, 61 },
    { 0x15840e28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f40e2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67540e37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1340e3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38440e3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e40e45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9640e48, 33 },
    { 0x19e40e49, 82 },
    { 0xd2e40e4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae940e4d, 33 },
    { 0xcf040e4e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d40e51, 33 },
    { 0x6440e51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89340e55, 98 },
    { 0, 0 },
    { 0x82c40e57, 23 },
    { 0x47740e57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1340e5b, 93 },
    { 0xd2b40e5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af40e61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31240e64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x36340e67, 23 },
    { 0xf0640e67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11240e6d, 76 },
    { 0, 0 },
    { 0xca340e6f, 81 },
    { 0x3240e6f, 33 },
    { 0xd2a40e6f, 98 },
    { 0x87e40e72, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1440e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9740e7e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d40e88, 75 },
    { 0x16640e88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88840e8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e40e90, 34 },
    { 0x6640e90, 98 },
    { 0, 0 },
    { 0xcb40e93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0440e98, 43 },
    { 0, 0 },
    { 0xebf40e9a, 68 },
    { 0xef040e9a, 93 },
    { 0xcbd40e9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88540e9f, 55 },
    { 0x92f40e9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb40ea4, 95 },
    { 0xb6840ea5, 98 },
    { 0, 0 },
    { 0x65640ea7, 23 },
    { 0, 0 },
    { 0x1ee40ea9, 56 },
    { 0x3440ea9, 93 },
    { 0x1c040eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4840eb2, 34 },
    { 0xeea40eb3, 66 },
    { 0x22440eb2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88540ec0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2040ec4, 84 },
    { 0xd2e40ec4, 33 },
    { 0x8bf40ec5, 90 },
    { 0, 0 },
    { 0x83740ec8, 22 },
    { 0, 0 },
    { 0x69140eca, 69 },
    { 0x4ec40eca, 34 },
    { 0, 0 },
    { 0xf1140ecd, 55 },
    { 0xb3840ece, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f840ed1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e40ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c240edc, 17 },
    { 0xe2140edd, 98 },
    { 0, 0 },
    { 0xe1c40edf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b40ee4, 98 },
    { 0x1dd40ee5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee40ee9, 87 },
    { 0xf0040eea, 34 },
    { 0x53f40eeb, 76 },
    { 0x38440ee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f40ef0, 90 },
    { 0x5c440ef1, 78 },
    { 0x72240ef1, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e140ef5, 87 },
    { 0x58040ef6, 51 },
    { 0x44040ef5, 61 },
    { 0xd9d40ef8, 90 },
    { 0, 0 },
    { 0x35c40efa, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb40efe, 95 },
    { 0x56540efe, 90 },
    { 0, 0 },
    { 0x51940f01, 55 },
    { 0x28440f02, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b240f06, 100 },
    { 0, 0 },
    { 0xe9040f08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe440f0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6340f10, 90 },
    { 0x2e840f11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa40f14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50240f17, 95 },
    { 0, 0 },
    { 0x76c40f19, 95 },
    { 0x57240f1a, 33 },
    { 0, 0 },
    { 0x3dd40f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5340f24, 95 },
    { 0x21e40f25, 66 },
    { 0xf6940f26, 69 },
    { 0, 0 },
    { 0x89140f28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d040f34, 23 },
    { 0xb3940f34, 38 },
    { 0x3a940f36, 90 },
    { 0xa6c40f37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc940f40, 90 },
    { 0x3c40f40, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c40f45, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x59340f48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf40f4b, 93 },
    { 0xd3a40f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c240f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53440f56, 66 },
    { 0x7db40f57, 61 },
    { 0xb4e40f56, 83 },
    { 0, 0 },
    { 0xd4640f5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54340f6e, 52 },
    { 0xc2740f6e, 91 },
    { 0x88d40f6e, 90 },
    { 0x32b40f70, 90 },
    { 0xef340f71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d40f7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4140f7d, 61 },
    { 0, 0 },
    { 0x64740f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d40f84, 61 },
    { 0xc7540f85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66140f8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e40f8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe40f91, 56 },
    { 0x6a640f91, 95 },
    { 0x1fb40f93, 42 },
    { 0x13e40f94, 90 },
    { 0xb3a40f95, 34 },
    { 0xca040f95, 95 },
    { 0, 0 },
    { 0xea640f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6240f9b, 61 },
    { 0x9dd40f9c, 91 },
    { 0x82f40f9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a40fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b40fa4, 90 },
    { 0x3b940fa4, 95 },
    { 0, 0 },
    { 0x66e40fa7, 67 },
    { 0x53940fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e40fab, 52 },
    { 0x69a40fac, 38 },
    { 0x25940fad, 61 },
    { 0xcaf40fac, 33 },
    { 0xfc740fac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xec840fb2, 33 },
    { 0x98a40fb3, 17 },
    { 0, 0 },
    { 0xa540fb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf640fb8, 98 },
    { 0, 0 },
    { 0x31540fba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2540fbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e40fc2, 98 },
    { 0x20040fc3, 76 },
    { 0xb7840fc4, 33 },
    { 0x97140fc5, 23 },
    { 0x68e40fc5, 88 },
    { 0xd2c40fc7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab340fd8, 90 },
    { 0x40740fd9, 61 },
    { 0x28b40fd9, 61 },
    { 0x5c40fdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8740fde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74640fe2, 55 },
    { 0x88840fe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69340fe7, 61 },
    { 0xce440fe8, 95 },
    { 0x7f040fe9, 33 },
    { 0x22c40fea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb940fef, 61 },
    { 0, 0 },
    { 0xd5440ff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17340ff4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d40ffa, 43 },
    { 0x16440ffb, 77 },
    { 0xe340ffa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a40fff, 76 },
    { 0x91441000, 23 },
    { 0x17741001, 98 },
    { 0, 0 },
    { 0x3241003, 33 },
    { 0x3d041004, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf41007, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1564100a, 98 },
    { 0xd434100a, 87 },
    { 0x7764100b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e41010, 23 },
    { 0x14e41011, 61 },
    { 0xce841010, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed741016, 98 },
    { 0, 0 },
    { 0xd9241018, 98 },
    { 0xf1441019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4101c, 98 },
    { 0x66c4101c, 85 },
    { 0xbe94101e, 78 },
    { 0xb234101f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af41024, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d4102b, 33 },
    { 0x8a44102c, 76 },
    { 0, 0 },
    { 0x9b94102e, 67 },
    { 0xbda4102e, 33 },
    { 0x82c4102e, 33 },
    { 0x20f41031, 90 },
    { 0xfb141032, 49 },
    { 0x8db4102f, 98 },
    { 0x13141034, 61 },
    { 0x9e641035, 98 },
    { 0x2bd41036, 95 },
    { 0x31841034, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f4103a, 100 },
    { 0xb594103a, 93 },
    { 0, 0 },
    { 0xc9c4103d, 55 },
    { 0xbaa4103e, 61 },
    { 0xe564103f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9741044, 43 },
    { 0xc7d41045, 78 },
    { 0, 0 },
    { 0x6be41047, 55 },
    { 0, 0 },
    { 0x75141049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214104e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8641056, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x68641059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c94105d, 95 },
    { 0, 0 },
    { 0x97e4105f, 61 },
    { 0, 0 },
    { 0x8a41061, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb41064, 98 },
    { 0xd6e41065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef4106f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b41072, 61 },
    { 0x77741073, 90 },
    { 0x98741074, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254107b, 90 },
    { 0x8a84107b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24441080, 90 },
    { 0x6fb41081, 73 },
    { 0x18b41081, 46 },
    { 0x39a41083, 98 },
    { 0, 0 },
    { 0xc9941085, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79541089, 90 },
    { 0x1784108a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa4108f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2441098, 78 },
    { 0xd8841099, 34 },
    { 0xe6841099, 98 },
    { 0xde241099, 90 },
    { 0xa5c4109c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1704109f, 100 },
    { 0, 0 },
    { 0xcd0410a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122410a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09410a8, 23 },
    { 0x7e5410a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc410b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956410b5, 33 },
    { 0xe3410b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec410b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89410bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a410c3, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2410c6, 23 },
    { 0, 0 },
    { 0x4a8410c8, 55 },
    { 0x384410c9, 98 },
    { 0, 0 },
    { 0x1f0410cb, 98 },
    { 0x3c9410cc, 95 },
    { 0xd9d410cb, 33 },
    { 0xd36410cd, 100 },
    { 0x735410cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec410d3, 33 },
    { 0, 0 },
    { 0x96410d5, 61 },
    { 0xa45410d6, 33 },
    { 0x97d410d7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1410da, 86 },
    { 0, 0 },
    { 0xcfe410dc, 98 },
    { 0, 0 },
    { 0xb15410de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979410ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a410f4, 91 },
    { 0x199410f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf410fa, 33 },
    { 0xc8b410fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a410fe, 90 },
    { 0xa1a410ff, 66 },
    { 0x18041100, 98 },
    { 0x9aa41100, 23 },
    { 0x32841100, 90 },
    { 0x9d041103, 76 },
    { 0, 0 },
    { 0x7f941105, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5541108, 33 },
    { 0xe5a41108, 90 },
    { 0, 0 },
    { 0x40a4110b, 100 },
    { 0xfb74110b, 34 },
    { 0xb274110d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e41111, 95 },
    { 0, 0 },
    { 0x4d141113, 76 },
    { 0x42141114, 95 },
    { 0xc8c41115, 23 },
    { 0x67c41115, 61 },
    { 0x6c641117, 43 },
    { 0xc4e41117, 95 },
    { 0xf5b41116, 33 },
    { 0xb2941116, 61 },
    { 0x74d41117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2704111e, 23 },
    { 0x5ce4111f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e41122, 90 },
    { 0x9ea41122, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10341127, 90 },
    { 0xc1241128, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f4112e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c41133, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd41137, 33 },
    { 0xb2141138, 66 },
    { 0, 0 },
    { 0xc4f4113a, 9 },
    { 0x3944113a, 61 },
    { 0x6f74113b, 90 },
    { 0xa484113a, 93 },
    { 0x1d34113c, 33 },
    { 0xb324113f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb41143, 75 },
    { 0x3fa41143, 52 },
    { 0xe4141145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4114a, 61 },
    { 0xf544114b, 98 },
    { 0x4a14114c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d54114f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb241158, 90 },
    { 0x7ce41159, 98 },
    { 0xe5d41159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef4115d, 33 },
    { 0x54f4115e, 23 },
    { 0x404115e, 84 },
    { 0xc0941160, 34 },
    { 0x904115f, 34 },
    { 0x98c41162, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d41172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a41175, 98 },
    { 0x6b441176, 98 },
    { 0xe5f41177, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f24117c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2341180, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e41183, 95 },
    { 0, 0 },
    { 0x141185, 95 },
    { 0, 0 },
    { 0x6dc41187, 76 },
    { 0x62a41188, 98 },
    { 0x61c41189, 61 },
    { 0x2ba4118a, 90 },
    { 0x3b44118a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb4118e, 23 },
    { 0, 0 },
    { 0x51741190, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1504119e, 68 },
    { 0x3794119e, 90 },
    { 0, 0 },
    { 0xf91411a1, 98 },
    { 0x719411a2, 65 },
    { 0x9be411a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35411a7, 95 },
    { 0xde411a8, 98 },
    { 0xcb5411a9, 61 },
    { 0, 0 },
    { 0x9c3411ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead411af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a411b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd411c2, 95 },
    { 0x38411c3, 84 },
    { 0x464411c3, 91 },
    { 0xff9411c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b411cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f411ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58411d4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2411d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7411da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5411de, 33 },
    { 0x4a4411df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef411e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c411e7, 61 },
    { 0x41e411e8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f411eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x332411ee, 66 },
    { 0, 0 },
    { 0xaaa411f0, 98 },
    { 0x92b411f1, 81 },
    { 0, 0 },
    { 0xf60411f3, 95 },
    { 0, 0 },
    { 0xbd2411f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f411f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63841202, 76 },
    { 0xc9641202, 98 },
    { 0xd3b41202, 90 },
    { 0x6f941202, 23 },
    { 0x52441202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79441210, 90 },
    { 0x77141211, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1041214, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b41218, 98 },
    { 0x13741218, 90 },
    { 0x8824121a, 98 },
    { 0xab841219, 95 },
    { 0xa0441218, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb041220, 90 },
    { 0xa9041221, 98 },
    { 0xab641222, 99 },
    { 0x8a441222, 90 },
    { 0xaad41224, 77 },
    { 0x5aa41224, 33 },
    { 0x5d041226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7341229, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34122e, 90 },
    { 0x3c54122f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1441232, 90 },
    { 0, 0 },
    { 0x83f41234, 23 },
    { 0xdd441234, 61 },
    { 0x6aa41235, 95 },
    { 0xc7e41237, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3464123d, 66 },
    { 0, 0 },
    { 0xff04123f, 90 },
    { 0xc8a41240, 61 },
    { 0x5fd41241, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2641247, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c4124f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a841254, 90 },
    { 0x88141254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64125c, 33 },
    { 0x3fd4125c, 46 },
    { 0xacd4125e, 33 },
    { 0xed14125c, 98 },
    { 0x11a41260, 33 },
    { 0xae641260, 53 },
    { 0xc6c41262, 66 },
    { 0xa8741263, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1541266, 33 },
    { 0xde041267, 61 },
    { 0x69441268, 95 },
    { 0xc7b41269, 55 },
    { 0x8ba4126a, 61 },
    { 0x7014126b, 87 },
    { 0x2fd4126a, 95 },
    { 0x4e34126d, 95 },
    { 0xdcc41268, 90 },
    { 0x6ef4126b, 95 },
    { 0xdbd41268, 87 },
    { 0xcd441271, 34 },
    { 0, 0 },
    { 0xc9541273, 52 },
    { 0xa9f41274, 91 },
    { 0x68141275, 33 },
    { 0x8d741273, 100 },
    { 0xb7841277, 85 },
    { 0, 0 },
    { 0xe8e41279, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5354127e, 95 },
    { 0, 0 },
    { 0xef341280, 90 },
    { 0x9d941281, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d541288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c4128c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764128f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60941293, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e41296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf241299, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xba24129c, 61 },
    { 0x3654129d, 61 },
    { 0xf414129e, 43 },
    { 0, 0 },
    { 0x22a412a0, 66 },
    { 0, 0 },
    { 0xaea412a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549412a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x235412a9, 90 },
    { 0xbeb412a9, 33 },
    { 0x7a1412ab, 91 },
    { 0, 0 },
    { 0x8412ad, 95 },
    { 0x528412ad, 52 },
    { 0, 0 },
    { 0xca1412b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f412b9, 90 },
    { 0, 0 },
    { 0xa4e412bb, 46 },
    { 0x822412bb, 65 },
    { 0xa1d412bd, 90 },
    { 0x380412bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xab412c1, 55 },
    { 0, 0 },
    { 0x531412c3, 78 },
    { 0xfde412c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf412c7, 90 },
    { 0x311412c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c412cb, 23 },
    { 0x62a412cc, 90 },
    { 0xcc8412cd, 95 },
    { 0, 0 },
    { 0x2ea412cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b412d3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923412d7, 90 },
    { 0, 0 },
    { 0x74d412d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6412e0, 90 },
    { 0xb5c412e0, 98 },
    { 0x5e2412e2, 98 },
    { 0xde4412e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5412f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58412f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x74412f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d41302, 44 },
    { 0x4e641303, 55 },
    { 0xfc341304, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c41307, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4324130c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf4130f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a41313, 34 },
    { 0, 0 },
    { 0xf7e41315, 100 },
    { 0, 0 },
    { 0x3b641317, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b84131a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc4131f, 23 },
    { 0x61d41320, 33 },
    { 0xf8341321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54341325, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7614132a, 98 },
    { 0, 0 },
    { 0xa14132c, 98 },
    { 0xec84132d, 23 },
    { 0x6394132e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9341333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe941336, 43 },
    { 0xf8041337, 33 },
    { 0x2ba41338, 93 },
    { 0x4ac41339, 90 },
    { 0xeb84133a, 76 },
    { 0xbdb41339, 70 },
    { 0x28d41337, 33 },
    { 0x3a541339, 55 },
    { 0xb454133e, 61 },
    { 0x4ed41337, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb4134a, 40 },
    { 0xe64134b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b4134f, 61 },
    { 0x1204134f, 23 },
    { 0x5b41350, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c4135c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe41361, 78 },
    { 0, 0 },
    { 0xb3541363, 93 },
    { 0xbc241364, 55 },
    { 0x86541365, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b41369, 43 },
    { 0x9c54136a, 101 },
    { 0x26a4136b, 61 },
    { 0x9744136a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c41373, 93 },
    { 0x28f41374, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7144137a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a94137d, 90 },
    { 0xb684137e, 61 },
    { 0, 0 },
    { 0x7ad41380, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x78741383, 93 },
    { 0x80b41383, 88 },
    { 0xf3641385, 88 },
    { 0x13041386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad4138c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2324138f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff41392, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5541395, 95 },
    { 0xccf41395, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc534139c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130413a0, 91 },
    { 0x99e413a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709413a6, 55 },
    { 0x21d413a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe413b2, 61 },
    { 0xbfb413b3, 33 },
    { 0x962413b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da413b7, 52 },
    { 0xdb0413b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5413bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b413c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf413c5, 90 },
    { 0x5a9413c6, 55 },
    { 0, 0 },
    { 0x851413c8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8413cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x477413d0, 53 },
    { 0x70f413d1, 90 },
    { 0, 0 },
    { 0x8cd413d3, 90 },
    { 0xfb1413d4, 98 },
    { 0xa71413d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753413db, 85 },
    { 0xad3413dc, 98 },
    { 0xb5e413dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b413e1, 84 },
    { 0xee0413e2, 90 },
    { 0x9ba413e3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c413eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc413f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7413f6, 91 },
    { 0, 0 },
    { 0x581413f8, 23 },
    { 0, 0 },
    { 0x2c6413fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87413fd, 76 },
    { 0x87a413fd, 99 },
    { 0xd54413ff, 81 },
    { 0xda941400, 61 },
    { 0x52e41400, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d041407, 90 },
    { 0, 0 },
    { 0x73c41409, 90 },
    { 0x1674140a, 23 },
    { 0x3b74140a, 55 },
    { 0, 0 },
    { 0x3a44140d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd41410, 65 },
    { 0xade41410, 90 },
    { 0xcd141412, 44 },
    { 0x1a941413, 23 },
    { 0, 0 },
    { 0x68041415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5404141f, 68 },
    { 0, 0 },
    { 0xd1c41421, 90 },
    { 0, 0 },
    { 0xcdd41423, 90 },
    { 0x97741423, 75 },
    { 0, 0 },
    { 0xc9341426, 87 },
    { 0xea041427, 94 },
    { 0xdcc41427, 43 },
    { 0, 0 },
    { 0x66e4142a, 43 },
    { 0, 0 },
    { 0x4904142c, 76 },
    { 0, 0 },
    { 0x43f4142e, 23 },
    { 0xa474142e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d441434, 66 },
    { 0xb4c41435, 90 },
    { 0x2a541436, 52 },
    { 0, 0 },
    { 0x51541438, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d4143d, 77 },
    { 0x6b74143d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9541442, 81 },
    { 0, 0 },
    { 0xd7541444, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9441449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4144e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x52641451, 61 },
    { 0x76b41451, 34 },
    { 0x1a641452, 98 },
    { 0xdf441454, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xed741457, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a4145c, 95 },
    { 0x2bf4145c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6841461, 33 },
    { 0x7ea41462, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67341467, 23 },
    { 0, 0 },
    { 0x27041469, 90 },
    { 0x3d94146a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a4146e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6341475, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f4147b, 81 },
    { 0x3074147b, 95 },
    { 0x1234147d, 23 },
    { 0x9df4147d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c41485, 60 },
    { 0x86741486, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d4148b, 23 },
    { 0xde24148b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64148f, 84 },
    { 0x194148f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c41493, 98 },
    { 0x84941494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa574149c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d4149f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d414a3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14414a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb414ac, 34 },
    { 0x6d3414ad, 61 },
    { 0, 0 },
    { 0xda1414af, 90 },
    { 0x328414af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f414b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78414bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616414c4, 61 },
    { 0x92f414c5, 26 },
    { 0x587414c6, 52 },
    { 0x83d414c6, 34 },
    { 0xeb1414c7, 61 },
    { 0, 0 },
    { 0xd2d414ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed414cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c414d1, 33 },
    { 0xd82414d2, 90 },
    { 0xdba414d3, 85 },
    { 0xf0d414d4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac414d8, 76 },
    { 0, 0 },
    { 0x78e414da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf414dd, 78 },
    { 0xb14414dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1414e1, 98 },
    { 0xe39414e1, 19 },
    { 0x911414e3, 100 },
    { 0x7e414e2, 88 },
    { 0x27414e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621414ef, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac414f4, 84 },
    { 0xffe414f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233414fb, 98 },
    { 0x931414fc, 76 },
    { 0x495414fd, 35 },
    { 0x997414fc, 93 },
    { 0xed0414ff, 98 },
    { 0x48b41500, 66 },
    { 0x69341500, 95 },
    { 0x9a8414fc, 95 },
    { 0, 0 },
    { 0xbfc41504, 61 },
    { 0xb4c41504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb641509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x824150c, 61 },
    { 0xe544150d, 33 },
    { 0xb164150e, 87 },
    { 0x3374150f, 34 },
    { 0xf424150e, 34 },
    { 0x87641510, 66 },
    { 0x4d041510, 75 },
    { 0x6a241513, 23 },
    { 0, 0 },
    { 0xba141515, 85 },
    { 0xff841516, 88 },
    { 0x99941516, 34 },
    { 0x5db41518, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11241520, 100 },
    { 0, 0 },
    { 0xc6241522, 33 },
    { 0xf3041523, 61 },
    { 0x43e41522, 46 },
    { 0x58241522, 61 },
    { 0x9c041526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb4152d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74341531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8341534, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d41539, 95 },
    { 0xfa4153a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb204153d, 90 },
    { 0, 0 },
    { 0x74153f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d141545, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc41549, 93 },
    { 0x19f4154a, 77 },
    { 0x8064154a, 23 },
    { 0xbcf4154c, 23 },
    { 0x84e4154d, 90 },
    { 0x6d84154d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a841552, 93 },
    { 0x55841553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d84155d, 95 },
    { 0x8b64155e, 90 },
    { 0x4604155f, 90 },
    { 0, 0 },
    { 0x1f141561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1641566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e4156a, 78 },
    { 0x8bd4156b, 90 },
    { 0x3304156c, 61 },
    { 0, 0 },
    { 0xc6e4156e, 23 },
    { 0, 0 },
    { 0xd3a41570, 61 },
    { 0xc8d41571, 90 },
    { 0x4ae41571, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d41578, 61 },
    { 0x4fd41579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2294157c, 46 },
    { 0, 0 },
    { 0xaf44157e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a41581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c41585, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a4158a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3484158e, 48 },
    { 0x7ed4158f, 46 },
    { 0xb8f4158f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace41594, 61 },
    { 0x89341595, 23 },
    { 0x5e041596, 61 },
    { 0x73341597, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d4159c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd994159f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548415a6, 98 },
    { 0x92a415a7, 55 },
    { 0x37f415a8, 67 },
    { 0xf97415a8, 67 },
    { 0xff4415a7, 95 },
    { 0x816415a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d415bc, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138415c5, 34 },
    { 0xbb8415c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd415c9, 96 },
    { 0x6c415c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac415cd, 33 },
    { 0, 0 },
    { 0xdb7415cf, 95 },
    { 0, 0 },
    { 0x377415d1, 33 },
    { 0, 0 },
    { 0xd93415d3, 61 },
    { 0x317415d3, 88 },
    { 0xc6c415d5, 87 },
    { 0x81b415d6, 61 },
    { 0x6f4415d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674415db, 23 },
    { 0x943415db, 52 },
    { 0, 0 },
    { 0xd5415de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a415e4, 90 },
    { 0x7f415e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2415e8, 33 },
    { 0xd93415e8, 43 },
    { 0, 0 },
    { 0x3ee415eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73415ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22415f2, 83 },
    { 0, 0 },
    { 0xc5f415f4, 33 },
    { 0xf00415f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21415fd, 46 },
    { 0x3d8415fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f441602, 70 },
    { 0xd9e41603, 33 },
    { 0, 0 },
    { 0xced41605, 95 },
    { 0x7e441605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe41609, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c4160e, 90 },
    { 0x44d4160e, 99 },
    { 0xcc04160f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f441613, 91 },
    { 0, 0 },
    { 0xb941615, 95 },
    { 0x7f641616, 33 },
    { 0x3ad41616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4161e, 88 },
    { 0x52c4161f, 66 },
    { 0x3eb41620, 98 },
    { 0xd154161f, 55 },
    { 0x9304161f, 98 },
    { 0, 0 },
    { 0x5b041624, 93 },
    { 0x4b541624, 66 },
    { 0xd3441626, 17 },
    { 0xfde41625, 98 },
    { 0xd4041628, 53 },
    { 0x5c941628, 78 },
    { 0x61341629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b41631, 56 },
    { 0xcd41632, 90 },
    { 0x1f841633, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf41636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb41640, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1541646, 61 },
    { 0xf4741647, 98 },
    { 0xd8541648, 84 },
    { 0xf9b41646, 52 },
    { 0, 0 },
    { 0x38c4164b, 64 },
    { 0x6b4164c, 78 },
    { 0xfa44164b, 33 },
    { 0, 0 },
    { 0x4644164f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c41654, 23 },
    { 0x2b741655, 67 },
    { 0x39141654, 52 },
    { 0xea41656, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6864165a, 61 },
    { 0x3204165b, 33 },
    { 0x34e4165c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d41661, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99441666, 90 },
    { 0xadf41666, 61 },
    { 0, 0 },
    { 0xf0441669, 23 },
    { 0xfa841669, 61 },
    { 0xc14166b, 93 },
    { 0, 0 },
    { 0xdb4166d, 95 },
    { 0x4e14166d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f041672, 90 },
    { 0x2f241673, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24167a, 98 },
    { 0xa3e4167b, 33 },
    { 0x7804167a, 90 },
    { 0x4e24167d, 46 },
    { 0x2674167c, 90 },
    { 0xeca4167e, 78 },
    { 0x2ba41680, 61 },
    { 0, 0 },
    { 0x52541682, 95 },
    { 0, 0 },
    { 0xe4941684, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d54168b, 90 },
    { 0x9ae4168c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb4169a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c84169d, 95 },
    { 0x1f4169e, 83 },
    { 0xfd14169f, 33 },
    { 0xfee416a0, 88 },
    { 0x92c416a0, 98 },
    { 0xe54416a1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6416a5, 85 },
    { 0xa89416a5, 55 },
    { 0xb91416a7, 43 },
    { 0, 0 },
    { 0x18f416a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x487416ac, 36 },
    { 0x12c416ad, 61 },
    { 0x8d9416ae, 76 },
    { 0xc1f416af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b416b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d416be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x221416c1, 95 },
    { 0x50e416c2, 43 },
    { 0x8d7416c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f416c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a416cd, 23 },
    { 0x24416ce, 33 },
    { 0, 0 },
    { 0x714416d0, 34 },
    { 0xcb416d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8416d5, 43 },
    { 0xc43416d6, 23 },
    { 0x631416d7, 93 },
    { 0x152416d6, 95 },
    { 0x339416d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d416dd, 46 },
    { 0x36b416de, 46 },
    { 0xad2416de, 34 },
    { 0, 0 },
    { 0x274416e1, 55 },
    { 0x6fa416e2, 93 },
    { 0, 0 },
    { 0xfdf416e4, 90 },
    { 0, 0 },
    { 0xde1416e6, 95 },
    { 0x654416e7, 87 },
    { 0xc05416e8, 33 },
    { 0x85c416e7, 23 },
    { 0, 0 },
    { 0x15c416eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23416ee, 93 },
    { 0x46f416ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5416f4, 74 },
    { 0, 0 },
    { 0x5e2416f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf416f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x156416fc, 34 },
    { 0x97416fd, 34 },
    { 0xfc9416fe, 90 },
    { 0xc48416fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d41702, 78 },
    { 0x41341703, 77 },
    { 0, 0 },
    { 0x8a941705, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c4170b, 93 },
    { 0x2b04170b, 61 },
    { 0x2fb4170d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66841711, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6041714, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc4171b, 61 },
    { 0xa394171b, 95 },
    { 0xfb14171b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51541721, 90 },
    { 0x91641721, 90 },
    { 0x4ec41722, 43 },
    { 0xefb41723, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b4172b, 98 },
    { 0, 0 },
    { 0x54172d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee941733, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88441736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd41739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc4173c, 48 },
    { 0x28e4173d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf541744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3241749, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3141751, 78 },
    { 0xcf141751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae941756, 79 },
    { 0x66941757, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd964175b, 23 },
    { 0xdab4175b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93641761, 98 },
    { 0x62141762, 90 },
    { 0x4641763, 99 },
    { 0, 0 },
    { 0x67a41765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc4176a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b41771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b41777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f74177d, 86 },
    { 0x6c64177d, 90 },
    { 0, 0 },
    { 0x1f541780, 98 },
    { 0, 0 },
    { 0x9b641782, 61 },
    { 0, 0 },
    { 0x74241784, 98 },
    { 0x54041785, 77 },
    { 0xdec41786, 95 },
    { 0x16a41787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c4178e, 75 },
    { 0, 0 },
    { 0xe6241790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a41798, 43 },
    { 0xd9d41799, 95 },
    { 0xe974179a, 95 },
    { 0x1014179a, 90 },
    { 0xa5f4179c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5394179f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7417a2, 95 },
    { 0, 0 },
    { 0xbee417a4, 61 },
    { 0, 0 },
    { 0xe95417a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587417aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a417ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b417b2, 90 },
    { 0xca5417b3, 98 },
    { 0xa7c417b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9417b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187417bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e417c0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c417c3, 23 },
    { 0xc56417c4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb417c7, 34 },
    { 0xe2d417c7, 43 },
    { 0x1d8417c9, 66 },
    { 0xaf3417c8, 93 },
    { 0, 0 },
    { 0xe46417cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9417d0, 33 },
    { 0x81c417d1, 59 },
    { 0, 0 },
    { 0xe72417d3, 90 },
    { 0x86e417d4, 76 },
    { 0x55a417d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4417d9, 61 },
    { 0x154417da, 90 },
    { 0xfe8417d9, 33 },
    { 0x182417dc, 66 },
    { 0xc42417dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2417e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x667417e5, 23 },
    { 0xa09417e6, 95 },
    { 0xd08417e7, 23 },
    { 0xe9417e8, 61 },
    { 0x8d3417e9, 33 },
    { 0x7d7417ea, 81 },
    { 0xa31417eb, 90 },
    { 0xb0417ea, 98 },
    { 0x952417e7, 90 },
    { 0x2f3417ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99417f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6417f8, 90 },
    { 0, 0 },
    { 0x76c417fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553417fe, 52 },
    { 0xf03417ff, 76 },
    { 0x376417fe, 61 },
    { 0xb1d41800, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27441807, 88 },
    { 0xec041807, 90 },
    { 0x27341809, 55 },
    { 0xbd04180a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264180f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a941813, 100 },
    { 0xecc41814, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324181a, 61 },
    { 0xc1f4181b, 23 },
    { 0, 0 },
    { 0x6644181d, 98 },
    { 0x9684181e, 90 },
    { 0x7e64181e, 90 },
    { 0x3f341820, 55 },
    { 0x88a41821, 84 },
    { 0xe5941821, 33 },
    { 0, 0 },
    { 0x4ef41824, 78 },
    { 0x47941825, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f4182c, 67 },
    { 0x1734182d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7441830, 33 },
    { 0x34741830, 23 },
    { 0xebc41832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4541836, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e4183b, 36 },
    { 0xf844183c, 95 },
    { 0, 0 },
    { 0xa694183e, 98 },
    { 0x8a64183f, 61 },
    { 0x844183f, 33 },
    { 0xb1d4183f, 43 },
    { 0x86341842, 61 },
    { 0, 0 },
    { 0xab441844, 95 },
    { 0x70541845, 23 },
    { 0x12341846, 33 },
    { 0x32941847, 95 },
    { 0x2a441845, 90 },
    { 0, 0 },
    { 0xb884184a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32841851, 83 },
    { 0xe9f41852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7954185f, 90 },
    { 0, 0 },
    { 0xb3941861, 84 },
    { 0x37941861, 90 },
    { 0, 0 },
    { 0x57141864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71241868, 43 },
    { 0xfb941869, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4186c, 33 },
    { 0xe154186c, 95 },
    { 0, 0 },
    { 0xf64186f, 81 },
    { 0x8ed4186f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd741873, 78 },
    { 0xc1f41874, 88 },
    { 0, 0 },
    { 0x7a341876, 90 },
    { 0xb2641877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3464187a, 98 },
    { 0xfd54187a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f64187f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc41884, 97 },
    { 0, 0 },
    { 0x95e41886, 75 },
    { 0xd7d41887, 68 },
    { 0xe3f41888, 100 },
    { 0x6bb41888, 98 },
    { 0x1264188a, 90 },
    { 0x4a041889, 14 },
    { 0, 0 },
    { 0xfa64188d, 90 },
    { 0xbae4188e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b941892, 34 },
    { 0x6da41893, 99 },
    { 0xffc41894, 90 },
    { 0xe8941895, 95 },
    { 0xb1841895, 61 },
    { 0x53a41897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72418a1, 33 },
    { 0x9fa418a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864418a6, 90 },
    { 0x2db418a7, 33 },
    { 0x739418a6, 90 },
    { 0xb94418a9, 87 },
    { 0xda7418a6, 33 },
    { 0xf65418aa, 43 },
    { 0x45d418a7, 33 },
    { 0x103418ad, 46 },
    { 0xcb418ad, 33 },
    { 0xbff418af, 61 },
    { 0x746418ad, 40 },
    { 0xab3418b0, 61 },
    { 0x80c418b1, 69 },
    { 0, 0 },
    { 0xddd418b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe418bd, 33 },
    { 0x821418be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280418c3, 34 },
    { 0xd59418c4, 33 },
    { 0xcdc418c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3418c8, 33 },
    { 0x7d4418c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352418cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b418d8, 95 },
    { 0x207418d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29418e0, 23 },
    { 0x97f418e1, 84 },
    { 0x974418e2, 90 },
    { 0xccb418e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b418e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7418ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b418f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2418f7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8418fa, 33 },
    { 0xb8418fb, 56 },
    { 0x479418fb, 98 },
    { 0x32a418fb, 95 },
    { 0x729418fe, 33 },
    { 0x2d5418ff, 98 },
    { 0x9fe41900, 98 },
    { 0xb72418fe, 33 },
    { 0x4a941902, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58341908, 33 },
    { 0x8a041909, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43141912, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45341915, 95 },
    { 0xa0f41915, 83 },
    { 0x29041917, 98 },
    { 0, 0 },
    { 0x74a41919, 90 },
    { 0, 0 },
    { 0x5d04191b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04191e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48341923, 76 },
    { 0xf2041924, 61 },
    { 0, 0 },
    { 0x93141926, 33 },
    { 0x6641927, 23 },
    { 0, 0 },
    { 0x75441929, 76 },
    { 0, 0 },
    { 0x6d74192b, 68 },
    { 0xb874192c, 85 },
    { 0x89c4192c, 90 },
    { 0x3ae4192e, 95 },
    { 0x1b4192f, 76 },
    { 0x84b41930, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc41935, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9341940, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774194c, 95 },
    { 0x1814194d, 76 },
    { 0, 0 },
    { 0xacf4194f, 38 },
    { 0x46741950, 87 },
    { 0, 0 },
    { 0xfaf41952, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a641955, 90 },
    { 0, 0 },
    { 0xa0341957, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824195d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2941962, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d4196e, 23 },
    { 0, 0 },
    { 0xb6241970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d341973, 61 },
    { 0, 0 },
    { 0xaf941975, 40 },
    { 0x37941975, 52 },
    { 0x17d41977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b4197d, 90 },
    { 0x7204197d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43841986, 61 },
    { 0x8cb41986, 76 },
    { 0, 0 },
    { 0xec441989, 90 },
    { 0xa2b4198a, 61 },
    { 0xe9c4198b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2514198e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f41991, 99 },
    { 0, 0 },
    { 0x14241993, 33 },
    { 0xa0b41994, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c4199d, 91 },
    { 0xb944199d, 75 },
    { 0, 0 },
    { 0xb6c419a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35419ab, 98 },
    { 0x2d5419ac, 90 },
    { 0x421419ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb419b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b419b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9419bd, 90 },
    { 0xdac419be, 78 },
    { 0x487419bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76419c2, 61 },
    { 0xe6a419c2, 34 },
    { 0x20419c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78419cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd419ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef419d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c419d8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c419db, 90 },
    { 0, 0 },
    { 0x669419dd, 90 },
    { 0, 0 },
    { 0x38419df, 33 },
    { 0x4f8419e0, 98 },
    { 0x59a419e0, 33 },
    { 0x93c419e0, 61 },
    { 0, 0 },
    { 0x4b2419e4, 33 },
    { 0xbbf419e5, 23 },
    { 0xa1f419e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a419ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a419f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca419f4, 87 },
    { 0, 0 },
    { 0xb05419f6, 40 },
    { 0x5f419f7, 95 },
    { 0x434419f8, 34 },
    { 0xe11419f9, 34 },
    { 0xe2419fa, 23 },
    { 0xbd0419fb, 46 },
    { 0x275419fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57541a08, 33 },
    { 0x80a41a08, 61 },
    { 0, 0 },
    { 0x78d41a0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea241a0e, 98 },
    { 0xb8e41a0f, 61 },
    { 0x60d41a0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60541a19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be41a1c, 43 },
    { 0x9a341a1d, 98 },
    { 0xab041a1e, 76 },
    { 0, 0 },
    { 0xb1d41a20, 20 },
    { 0x25c41a20, 90 },
    { 0xa6f41a20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc41a27, 66 },
    { 0x2141a28, 61 },
    { 0, 0 },
    { 0x2a741a2a, 85 },
    { 0, 0 },
    { 0x94041a2c, 55 },
    { 0xf4d41a2d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24741a32, 76 },
    { 0x65641a32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b41a3c, 55 },
    { 0x46641a3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb041a41, 80 },
    { 0x1641a42, 46 },
    { 0, 0 },
    { 0x13a41a44, 52 },
    { 0xe8a41a44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a341a49, 33 },
    { 0, 0 },
    { 0xb1341a4b, 90 },
    { 0xd3041a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48241a4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e041a56, 61 },
    { 0xe041a56, 33 },
    { 0x59141a57, 93 },
    { 0xd2c41a59, 100 },
    { 0xb4a41a58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2441a61, 90 },
    { 0xca241a61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c141a65, 33 },
    { 0, 0 },
    { 0x2cb41a67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c41a6a, 98 },
    { 0x9041a6a, 61 },
    { 0xe1041a6a, 87 },
    { 0, 0 },
    { 0xbda41a6e, 90 },
    { 0xe3441a6e, 90 },
    { 0x57141a6e, 100 },
    { 0x7f341a70, 24 },
    { 0xaec41a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba341a75, 61 },
    { 0x3941a76, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6941a79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba41a7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7141a7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4141a82, 46 },
    { 0x91841a83, 46 },
    { 0, 0 },
    { 0xbd341a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44341a8b, 90 },
    { 0x2b341a8b, 66 },
    { 0x6b941a8d, 23 },
    { 0x85b41a8e, 90 },
    { 0, 0 },
    { 0x88c41a90, 55 },
    { 0xa3441a91, 78 },
    { 0, 0 },
    { 0x99041a93, 90 },
    { 0x3ca41a94, 95 },
    { 0x10941a95, 93 },
    { 0, 0 },
    { 0xdeb41a97, 34 },
    { 0x5ab41a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83641a9c, 90 },
    { 0xba41a9c, 94 },
    { 0xd8741a9c, 33 },
    { 0x36441a9d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e41aa2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63441aaa, 33 },
    { 0xc0f41aab, 85 },
    { 0x4741aaa, 77 },
    { 0xe3741aaa, 76 },
    { 0xcc41aae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe041ab1, 55 },
    { 0x71a41ab2, 66 },
    { 0, 0 },
    { 0x2ef41ab4, 85 },
    { 0x78a41ab5, 46 },
    { 0x15b41ab6, 95 },
    { 0, 0 },
    { 0x28441ab8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc441abb, 43 },
    { 0, 0 },
    { 0xe2041abd, 55 },
    { 0x2e941abe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52941ac2, 90 },
    { 0x5f641ac2, 84 },
    { 0x9bb41ac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d41ac7, 55 },
    { 0x37941ac8, 99 },
    { 0x1a041ac9, 95 },
    { 0x44941ac8, 90 },
    { 0xc0241acb, 95 },
    { 0x73241acb, 97 },
    { 0x99e41acd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7541ad0, 66 },
    { 0xbc741ad0, 95 },
    { 0xde041ad0, 68 },
    { 0x35241ad0, 55 },
    { 0, 0 },
    { 0x3db41ad5, 95 },
    { 0xd6941ad6, 90 },
    { 0x30e41ad7, 33 },
    { 0x1fd41ad7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd741add, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b41ae4, 99 },
    { 0x15441ae4, 33 },
    { 0x6fc41ae6, 65 },
    { 0, 0 },
    { 0xef941ae8, 90 },
    { 0, 0 },
    { 0xa0041aea, 33 },
    { 0xfae41aeb, 33 },
    { 0x60341aec, 95 },
    { 0x8441aeb, 34 },
    { 0x65641aee, 17 },
    { 0x40e41aed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa641af6, 90 },
    { 0, 0 },
    { 0x5ca41af8, 33 },
    { 0x72f41af9, 52 },
    { 0, 0 },
    { 0x94d41afb, 87 },
    { 0, 0 },
    { 0x1e441afd, 61 },
    { 0x37a41afe, 6 },
    { 0xc8e41afd, 95 },
    { 0xba941b00, 55 },
    { 0x6d741b01, 23 },
    { 0, 0 },
    { 0x1c441b03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd41b07, 85 },
    { 0x90c41b08, 90 },
    { 0, 0 },
    { 0x43a41b0a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea041b1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a41b1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2841b29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30341b2c, 23 },
    { 0, 0 },
    { 0x61841b2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e741b35, 61 },
    { 0x6e441b35, 90 },
    { 0, 0 },
    { 0x32141b38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b41b3b, 33 },
    { 0x4ec41b3b, 90 },
    { 0x5d841b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a41b40, 61 },
    { 0, 0 },
    { 0xd4c41b42, 98 },
    { 0xa7241b43, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb41b47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5441b4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c541b50, 76 },
    { 0, 0 },
    { 0x6f341b52, 76 },
    { 0xf8241b53, 95 },
    { 0xb5541b54, 90 },
    { 0xa2641b52, 98 },
    { 0x8d341b54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27641b5e, 61 },
    { 0, 0 },
    { 0x5b041b60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5841b69, 90 },
    { 0x4af41b6a, 80 },
    { 0xea241b69, 66 },
    { 0xd7c41b6b, 98 },
    { 0, 0 },
    { 0x12141b6e, 43 },
    { 0xe7241b6e, 40 },
    { 0, 0 },
    { 0x6ad41b71, 33 },
    { 0xf6041b72, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc41b79, 87 },
    { 0, 0 },
    { 0xdf741b7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b041b83, 33 },
    { 0, 0 },
    { 0x9db41b85, 93 },
    { 0x27141b85, 90 },
    { 0, 0 },
    { 0xc0a41b88, 78 },
    { 0xa1e41b88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca441b8d, 34 },
    { 0x2b41b8e, 90 },
    { 0xf5641b8f, 33 },
    { 0xac941b8e, 85 },
    { 0xb7541b91, 38 },
    { 0, 0 },
    { 0x85c41b93, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c41b97, 90 },
    { 0xf7c41b98, 55 },
    { 0xa5941b99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a41b9c, 61 },
    { 0, 0 },
    { 0x2dc41b9e, 61 },
    { 0xf0841b9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d41ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52441ba8, 76 },
    { 0x9141ba9, 33 },
    { 0x69e41baa, 52 },
    { 0, 0 },
    { 0x1a41bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e41bb0, 93 },
    { 0, 0 },
    { 0x2b741bb2, 33 },
    { 0x6ba41bb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d141bb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c41bbb, 76 },
    { 0x82141bbc, 55 },
    { 0xf6541bbd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f641bc2, 53 },
    { 0, 0 },
    { 0x5be41bc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a41bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e041bd1, 95 },
    { 0x20a41bd1, 23 },
    { 0x7c841bd3, 46 },
    { 0, 0 },
    { 0x7e341bd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f41bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae41bdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c41be4, 98 },
    { 0x68c41be4, 84 },
    { 0x38741be4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf41be9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d41bec, 34 },
    { 0, 0 },
    { 0x6bf41bee, 99 },
    { 0, 0 },
    { 0xb9441bf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19741bf9, 90 },
    { 0x4e241bfa, 61 },
    { 0xa0d41bfa, 92 },
    { 0x7e241bfb, 90 },
    { 0x47e41bfd, 61 },
    { 0x18a41bfa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6141c02, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c841c06, 61 },
    { 0xdc341c07, 100 },
    { 0, 0 },
    { 0xea741c09, 55 },
    { 0x78641c0a, 76 },
    { 0x55e41c09, 33 },
    { 0xd5641c0c, 76 },
    { 0, 0 },
    { 0xb541c0e, 61 },
    { 0, 0 },
    { 0x3c841c10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44941c13, 90 },
    { 0x2741c14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c41c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde341c1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee41c24, 66 },
    { 0x5be41c25, 93 },
    { 0x3bb41c25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40941c29, 90 },
    { 0x31841c2a, 98 },
    { 0xcdd41c2a, 90 },
    { 0xc1a41c2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd41c2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43141c36, 55 },
    { 0xea741c37, 66 },
    { 0, 0 },
    { 0xae141c39, 78 },
    { 0, 0 },
    { 0xcbe41c3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7441c40, 43 },
    { 0xfa241c40, 76 },
    { 0x89041c42, 98 },
    { 0xbd041c43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde041c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec941c4a, 66 },
    { 0xc7d41c4a, 33 },
    { 0x14241c4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e41c53, 33 },
    { 0x89041c53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e41c5d, 96 },
    { 0x4b241c5d, 61 },
    { 0x7f641c5f, 85 },
    { 0xe8a41c5f, 46 },
    { 0x5af41c5d, 95 },
    { 0xeae41c62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3941c66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b41c69, 23 },
    { 0xb8d41c6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e41c75, 55 },
    { 0xee241c75, 34 },
    { 0, 0 },
    { 0x41741c78, 95 },
    { 0x1d741c79, 46 },
    { 0, 0 },
    { 0xa7e41c7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac41c81, 95 },
    { 0x51e41c81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf141c87, 53 },
    { 0xffe41c88, 66 },
    { 0x61f41c88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d141c93, 90 },
    { 0x20b41c94, 90 },
    { 0x31c41c95, 33 },
    { 0x2bc41c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f741c99, 95 },
    { 0, 0 },
    { 0xf741c9b, 90 },
    { 0xaaa41c9c, 55 },
    { 0x22a41c9d, 23 },
    { 0x29341c9d, 67 },
    { 0x21741c9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d41ca5, 98 },
    { 0x25341ca5, 91 },
    { 0xdb441ca7, 56 },
    { 0xbb641ca8, 98 },
    { 0x68741ca8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec41caf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61141cb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2341cb8, 61 },
    { 0x8741cb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a41cbc, 87 },
    { 0, 0 },
    { 0xe4b41cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd841cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3841cc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e41cc8, 76 },
    { 0xaca41cc9, 99 },
    { 0xf5741cc9, 23 },
    { 0x81141ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e41cd0, 93 },
    { 0xd1b41cd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c41cd5, 80 },
    { 0x60741cd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6141cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f41cdf, 33 },
    { 0x56d41ce0, 75 },
    { 0, 0 },
    { 0x6ef41ce2, 43 },
    { 0, 0 },
    { 0xdd441ce4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6241ce8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba41cec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x26941cef, 23 },
    { 0xe4441cf0, 43 },
    { 0xafa41cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb641cf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27041cff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94141d07, 23 },
    { 0x53341d08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60441d0c, 91 },
    { 0x8f441d0d, 33 },
    { 0xcb241d0c, 33 },
    { 0x7ef41d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d41d12, 61 },
    { 0xbe241d13, 33 },
    { 0x4db41d14, 66 },
    { 0, 0 },
    { 0x72941d16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9541d1b, 69 },
    { 0xd2641d1c, 43 },
    { 0x4b441d1b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b941d23, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66841d26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd41d2c, 85 },
    { 0x97f41d2c, 90 },
    { 0, 0 },
    { 0x63841d2f, 34 },
    { 0, 0 },
    { 0x71341d31, 87 },
    { 0x8ad41d32, 55 },
    { 0xf5141d33, 88 },
    { 0x25d41d34, 61 },
    { 0xa6741d35, 23 },
    { 0, 0 },
    { 0x2c841d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca241d3a, 33 },
    { 0, 0 },
    { 0xead41d3c, 90 },
    { 0, 0 },
    { 0x7c441d3e, 64 },
    { 0xa4241d3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc41d43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3141d4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0441d56, 95 },
    { 0xbe541d57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7141d5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad41d5e, 61 },
    { 0xce641d5e, 92 },
    { 0, 0 },
    { 0x1f141d61, 76 },
    { 0, 0 },
    { 0xc5141d63, 95 },
    { 0, 0 },
    { 0x6d041d65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91341d69, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62441d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e41d71, 65 },
    { 0xeac41d71, 33 },
    { 0, 0 },
    { 0xb3e41d74, 90 },
    { 0x98141d75, 95 },
    { 0xb3f41d75, 33 },
    { 0, 0 },
    { 0x95741d78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0641d7c, 90 },
    { 0x5a741d7d, 36 },
    { 0x76541d7e, 33 },
    { 0xaae41d7e, 81 },
    { 0xe3841d80, 98 },
    { 0x1c641d81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0041d8b, 55 },
    { 0x57b41d8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae41d90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db41d94, 98 },
    { 0xfa941d95, 90 },
    { 0xd2441d96, 34 },
    { 0x50141d97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4241d9e, 33 },
    { 0xe6a41d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84541da3, 67 },
    { 0x39e41da4, 78 },
    { 0xeba41da4, 95 },
    { 0x67841da6, 87 },
    { 0xc1441da7, 81 },
    { 0x88241da8, 23 },
    { 0x56941da9, 23 },
    { 0xeb641daa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c41db1, 61 },
    { 0x94741db2, 23 },
    { 0x29f41db3, 23 },
    { 0x5241db4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d641db7, 61 },
    { 0xdf441db7, 34 },
    { 0x5ff41db9, 55 },
    { 0x8c241dba, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4241dbd, 66 },
    { 0, 0 },
    { 0xdc41dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37241dc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97041dc5, 23 },
    { 0xcca41dc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c41dc9, 90 },
    { 0x2cd41dca, 46 },
    { 0, 0 },
    { 0x9f041dcc, 98 },
    { 0x86941dcd, 97 },
    { 0, 0 },
    { 0xa2441dcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9941dd7, 90 },
    { 0x9ef41dd8, 61 },
    { 0, 0 },
    { 0x7941dda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d541de2, 90 },
    { 0, 0 },
    { 0xe1e41de4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e41de9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec41dee, 90 },
    { 0x5f41dee, 90 },
    { 0x8b841df0, 95 },
    { 0x2ef41df0, 68 },
    { 0xe241df2, 34 },
    { 0x94541df0, 33 },
    { 0x64641df4, 34 },
    { 0x1fe41df4, 100 },
    { 0x4a741df6, 90 },
    { 0x5d341df5, 87 },
    { 0x18941df1, 100 },
    { 0xba241df9, 90 },
    { 0xc4a41dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c41dfe, 61 },
    { 0, 0 },
    { 0xa7041e00, 23 },
    { 0x7f441e00, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b41e04, 76 },
    { 0xbf941e05, 90 },
    { 0x5d141e05, 66 },
    { 0xcc341e07, 90 },
    { 0, 0 },
    { 0x8b241e09, 43 },
    { 0xd6f41e0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd41e0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc841e12, 95 },
    { 0x68441e13, 66 },
    { 0xb2f41e14, 87 },
    { 0x3e941e15, 52 },
    { 0xc6941e16, 61 },
    { 0x89141e15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee41e1c, 33 },
    { 0, 0 },
    { 0xce341e1e, 61 },
    { 0x3341e1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a41e25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39841e29, 76 },
    { 0xe6441e2a, 61 },
    { 0, 0 },
    { 0xe1741e2c, 23 },
    { 0, 0 },
    { 0x26041e2e, 33 },
    { 0xbae41e2f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63841e36, 98 },
    { 0, 0 },
    { 0x46041e38, 76 },
    { 0xca841e39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be41e42, 61 },
    { 0xad441e42, 46 },
    { 0xc9041e44, 46 },
    { 0x1f641e44, 90 },
    { 0x69341e46, 76 },
    { 0, 0 },
    { 0x11341e48, 43 },
    { 0, 0 },
    { 0xd9741e4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf241e4e, 93 },
    { 0, 0 },
    { 0xa6b41e50, 33 },
    { 0, 0 },
    { 0x3de41e52, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e41e5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e41e5d, 34 },
    { 0, 0 },
    { 0xa0241e5f, 78 },
    { 0x33e41e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf641e63, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32441e6b, 81 },
    { 0x7c341e6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8541e71, 61 },
    { 0xd9941e71, 99 },
    { 0x35941e73, 61 },
    { 0x4ba41e74, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d441e77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3341e7b, 23 },
    { 0x1bf41e7c, 56 },
    { 0x25441e7b, 87 },
    { 0x1d841e7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34841e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70141e85, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd541e88, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b41e8c, 95 },
    { 0, 0 },
    { 0xf9c41e8e, 24 },
    { 0x28941e8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9341e92, 95 },
    { 0xcc941e92, 81 },
    { 0, 0 },
    { 0x26741e95, 61 },
    { 0, 0 },
    { 0x6c541e97, 43 },
    { 0, 0 },
    { 0xd9441e99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e41ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76641ea5, 100 },
    { 0, 0 },
    { 0xfbf41ea7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac441eae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a41eb4, 34 },
    { 0xd8641eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab41eba, 33 },
    { 0x8ef41ebb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d741ec0, 80 },
    { 0xb141ec1, 90 },
    { 0xf741ec1, 43 },
    { 0x17c41ec1, 95 },
    { 0xee541ec4, 61 },
    { 0, 0 },
    { 0x30841ec6, 84 },
    { 0x78f41ec7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf41ecc, 78 },
    { 0, 0 },
    { 0x70941ece, 55 },
    { 0x4b441ecf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f41ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d41edf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb41ee2, 90 },
    { 0, 0 },
    { 0xc2c41ee4, 33 },
    { 0, 0 },
    { 0x77f41ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c41eea, 98 },
    { 0xbb41eeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13741ef1, 55 },
    { 0xa3c41ef2, 78 },
    { 0xf5441ef1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x28841ef6, 46 },
    { 0, 0 },
    { 0xa7d41ef8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45041efe, 98 },
    { 0, 0 },
    { 0xaa41f00, 90 },
    { 0xd1141f01, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f441f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0841f08, 76 },
    { 0x11b41f08, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8341f0c, 76 },
    { 0xf4741f0d, 90 },
    { 0x13d41f0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec41f16, 23 },
    { 0xe2041f17, 90 },
    { 0, 0 },
    { 0x4f741f19, 52 },
    { 0x2f941f1a, 43 },
    { 0xf9241f19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71841f20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd341f24, 95 },
    { 0x1f741f25, 34 },
    { 0xd0141f24, 94 },
    { 0x28541f27, 34 },
    { 0x7ec41f24, 46 },
    { 0x45041f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e841f2c, 33 },
    { 0x99841f2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0641f30, 12 },
    { 0, 0 },
    { 0x79341f32, 84 },
    { 0x56241f33, 98 },
    { 0xe2941f33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76541f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a41f3c, 98 },
    { 0x73e41f3d, 87 },
    { 0, 0 },
    { 0x5dc41f3f, 55 },
    { 0x62a41f40, 98 },
    { 0x5f641f3f, 95 },
    { 0xddb41f3f, 76 },
    { 0x9e041f41, 46 },
    { 0x1041f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe541f4b, 33 },
    { 0, 0 },
    { 0x8641f4d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a41f51, 56 },
    { 0x9a941f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f41f56, 95 },
    { 0xdd641f57, 99 },
    { 0x78341f57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a41f5f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x63241f62, 33 },
    { 0, 0 },
    { 0x1e841f64, 100 },
    { 0xafa41f64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe741f6e, 90 },
    { 0x81e41f6e, 33 },
    { 0x10b41f6e, 76 },
    { 0x94a41f71, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe041f74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b41f78, 81 },
    { 0, 0 },
    { 0x37041f7a, 100 },
    { 0xb3341f7a, 76 },
    { 0, 0 },
    { 0xe1f41f7d, 71 },
    { 0x7ca41f7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53041f81, 98 },
    { 0xe3c41f82, 98 },
    { 0xee341f81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e141f87, 55 },
    { 0x17e41f87, 55 },
    { 0xfa641f89, 66 },
    { 0xe9b41f87, 33 },
    { 0x72a41f8b, 29 },
    { 0x26941f8c, 34 },
    { 0xf7941f8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae41f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a41f99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b41f9d, 85 },
    { 0xa8741f9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b41fa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e41fa4, 98 },
    { 0xdd741fa5, 98 },
    { 0x8c641fa6, 55 },
    { 0, 0 },
    { 0x49c41fa8, 98 },
    { 0, 0 },
    { 0x7a641faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b641fae, 90 },
    { 0, 0 },
    { 0x2ed41fb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5141fb3, 23 },
    { 0, 0 },
    { 0xbcf41fb5, 95 },
    { 0x39f41fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe341fbc, 52 },
    { 0x6f641fbd, 43 },
    { 0, 0 },
    { 0x38041fbf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b41fc2, 88 },
    { 0xe3c41fc2, 23 },
    { 0xc7641fc4, 95 },
    { 0x2be41fc5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27341fca, 55 },
    { 0x16e41fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce541fd2, 61 },
    { 0, 0 },
    { 0xcb41fd4, 90 },
    { 0xe8341fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72841fda, 33 },
    { 0xb1a41fdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b941fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea441fe3, 95 },
    { 0x7a541fe4, 34 },
    { 0x2fd41fe5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59341fe9, 43 },
    { 0xfe141fe9, 98 },
    { 0x2941fe9, 67 },
    { 0x51141fec, 46 },
    { 0xe8341fea, 100 },
    { 0xb1b41fee, 100 },
    { 0x28d41fef, 53 },
    { 0x67a41fee, 95 },
    { 0x8a841ff1, 87 },
    { 0xbd641ff2, 85 },
    { 0x23041ff2, 88 },
    { 0x36941ff0, 75 },
    { 0xec141ff5, 76 },
    { 0x8f941fed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba41ffe, 23 },
    { 0xa2841ffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd42003, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e42008, 76 },
    { 0x48342009, 98 },
    { 0x4c74200a, 61 },
    { 0x4aa4200a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2824200e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa42017, 95 },
    { 0x3a242017, 95 },
    { 0x81942017, 33 },
    { 0xd6a4201a, 33 },
    { 0x20f4201b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b4201e, 90 },
    { 0xf7d4201f, 91 },
    { 0x5c84201e, 77 },
    { 0xbce42020, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e42025, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b42028, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f54202b, 33 },
    { 0x6fa4202c, 87 },
    { 0x8154202d, 61 },
    { 0x3694202d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae42036, 99 },
    { 0x45d42037, 33 },
    { 0, 0 },
    { 0x29542039, 61 },
    { 0x3424203a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa4203e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9774204b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3374204e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c842051, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd642056, 33 },
    { 0, 0 },
    { 0x39942058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e4205b, 90 },
    { 0xf554205c, 17 },
    { 0x1254205c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa742060, 52 },
    { 0xa2642061, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43542067, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf4206d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf942071, 61 },
    { 0x84d42071, 23 },
    { 0x93742072, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c742078, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8914207b, 76 },
    { 0xd884207c, 87 },
    { 0xfb74207b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf42081, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f42084, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d42087, 33 },
    { 0, 0 },
    { 0x22b42089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43442091, 55 },
    { 0x60742092, 68 },
    { 0, 0 },
    { 0x3b842094, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea42097, 81 },
    { 0xb8242098, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a84209b, 52 },
    { 0x9b14209c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604420a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78420a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1420aa, 61 },
    { 0x2b9420ab, 66 },
    { 0xc04420ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4420b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815420bf, 91 },
    { 0xba5420c0, 33 },
    { 0x819420c1, 23 },
    { 0x29b420c2, 98 },
    { 0xa82420c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed420c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489420cc, 67 },
    { 0xa96420cc, 34 },
    { 0x8fc420cc, 78 },
    { 0x1f4420cf, 61 },
    { 0, 0 },
    { 0xd9c420d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd420d5, 34 },
    { 0x812420d6, 61 },
    { 0x3e420d7, 90 },
    { 0x4c7420d7, 90 },
    { 0x58a420d9, 43 },
    { 0x7bb420d8, 61 },
    { 0xf07420d6, 90 },
    { 0x559420dc, 98 },
    { 0x991420dd, 23 },
    { 0xb47420de, 55 },
    { 0, 0 },
    { 0x933420e0, 23 },
    { 0xa10420e1, 90 },
    { 0xe35420e1, 33 },
    { 0x641420e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x730420e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3420e9, 33 },
    { 0x98f420ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997420ee, 61 },
    { 0xc31420ee, 61 },
    { 0x318420f0, 61 },
    { 0x8ad420f1, 23 },
    { 0, 0 },
    { 0xaae420f3, 23 },
    { 0xaed420f3, 33 },
    { 0, 0 },
    { 0x54d420f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31420fe, 66 },
    { 0, 0 },
    { 0x34042100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa42103, 52 },
    { 0x89642103, 23 },
    { 0x64942105, 34 },
    { 0x14a42103, 23 },
    { 0xa7842107, 90 },
    { 0xef42107, 100 },
    { 0xf5e42108, 95 },
    { 0, 0 },
    { 0x5894210b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd44210f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce42117, 55 },
    { 0xf042117, 23 },
    { 0xb0c42117, 67 },
    { 0x5314211a, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5242122, 61 },
    { 0x4642123, 61 },
    { 0xd1642123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc142129, 55 },
    { 0, 0 },
    { 0x2444212b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e142131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf542137, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae94213b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa114213f, 90 },
    { 0x494213f, 33 },
    { 0x4ee42141, 93 },
    { 0xd542140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e42145, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72242149, 85 },
    { 0xc414214a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f4214d, 90 },
    { 0xfb94214e, 88 },
    { 0x7434214f, 99 },
    { 0x75d4214e, 75 },
    { 0xb904214d, 61 },
    { 0x23242152, 93 },
    { 0xe3f42150, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2142157, 90 },
    { 0, 0 },
    { 0x81542159, 33 },
    { 0xd0c4215a, 61 },
    { 0xfe74215b, 98 },
    { 0xd6f4215a, 90 },
    { 0, 0 },
    { 0x38a4215e, 55 },
    { 0xcaf4215f, 65 },
    { 0xac842160, 55 },
    { 0x7144215f, 55 },
    { 0x3ac42162, 61 },
    { 0xcd42160, 99 },
    { 0x98e42164, 55 },
    { 0xeb542165, 23 },
    { 0x60742166, 98 },
    { 0xf7b42167, 61 },
    { 0x7f042161, 23 },
    { 0x30f42169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb54216c, 34 },
    { 0, 0 },
    { 0x4354216e, 90 },
    { 0, 0 },
    { 0x19c42170, 61 },
    { 0x73742170, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed42178, 90 },
    { 0xc5a42178, 43 },
    { 0xe0942178, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f4217e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c542181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb42184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef42187, 49 },
    { 0xa0742188, 77 },
    { 0xc9642188, 90 },
    { 0, 0 },
    { 0xca34218b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f44218f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51442192, 55 },
    { 0xb7a42193, 55 },
    { 0, 0 },
    { 0xaef42195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11842199, 90 },
    { 0xfd54219a, 90 },
    { 0x6fc4219b, 90 },
    { 0xd784219c, 33 },
    { 0x7444219d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153421a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54421a6, 61 },
    { 0x104421a7, 98 },
    { 0x5fc421a6, 98 },
    { 0xe0e421a9, 80 },
    { 0xe37421aa, 90 },
    { 0x730421a7, 93 },
    { 0x752421a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4421b0, 98 },
    { 0, 0 },
    { 0xd19421b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d421b6, 43 },
    { 0, 0 },
    { 0x439421b8, 93 },
    { 0, 0 },
    { 0xfe4421ba, 46 },
    { 0x25b421ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63421be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397421c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0421ca, 91 },
    { 0xce6421cb, 80 },
    { 0x82c421ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e421cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc421d2, 61 },
    { 0, 0 },
    { 0xe82421d4, 33 },
    { 0x39a421d4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c421e0, 98 },
    { 0xd06421e1, 61 },
    { 0xe95421e2, 66 },
    { 0xc7c421e0, 95 },
    { 0x4e9421e4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01421f5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db421f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25421ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb42203, 55 },
    { 0x86b42204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd04220b, 23 },
    { 0x5bd4220b, 91 },
    { 0xff14220b, 23 },
    { 0x3194220e, 90 },
    { 0x6ed4220f, 23 },
    { 0, 0 },
    { 0xa0642211, 90 },
    { 0x15842212, 23 },
    { 0, 0 },
    { 0x2e442214, 61 },
    { 0x4ce42214, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0842219, 61 },
    { 0, 0 },
    { 0xe474221b, 100 },
    { 0x1b14221c, 33 },
    { 0x4944221d, 17 },
    { 0xe7c4221e, 84 },
    { 0x4444221e, 95 },
    { 0, 0 },
    { 0xabb42221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4242224, 61 },
    { 0xbf842225, 34 },
    { 0xc6a42226, 61 },
    { 0xc4e42227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d44222d, 76 },
    { 0x2e54222d, 90 },
    { 0xf334222f, 52 },
    { 0xfa142230, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8842234, 88 },
    { 0xdc42235, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2894223c, 61 },
    { 0xe304223d, 90 },
    { 0xc3d4223d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51642242, 90 },
    { 0x4cc42243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b742246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764224b, 90 },
    { 0x2ff4224b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3864224f, 90 },
    { 0, 0 },
    { 0xeea42251, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80842256, 93 },
    { 0x7de42257, 67 },
    { 0x68542257, 33 },
    { 0xd0b42259, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af4225f, 93 },
    { 0x9434225f, 93 },
    { 0x30542261, 90 },
    { 0xbd542261, 33 },
    { 0x31f42263, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e84226d, 93 },
    { 0x33b4226d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6942272, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe42275, 91 },
    { 0xa9d42275, 23 },
    { 0x7642277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb4227a, 61 },
    { 0, 0 },
    { 0x9be4227c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3242281, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a42285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e44228d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85542291, 23 },
    { 0, 0 },
    { 0x1e542293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf42296, 90 },
    { 0xd6b42296, 55 },
    { 0, 0 },
    { 0x73942299, 76 },
    { 0x2c742299, 95 },
    { 0xe624229b, 80 },
    { 0x294229b, 34 },
    { 0, 0 },
    { 0xb794229e, 99 },
    { 0, 0 },
    { 0xcee422a0, 15 },
    { 0xc69422a1, 94 },
    { 0, 0 },
    { 0xe59422a3, 95 },
    { 0, 0 },
    { 0xba422a5, 23 },
    { 0x175422a6, 76 },
    { 0x2d1422a7, 34 },
    { 0xd47422a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab422ac, 52 },
    { 0x203422ad, 74 },
    { 0xf77422ae, 23 },
    { 0x18422af, 34 },
    { 0x6d6422af, 95 },
    { 0, 0 },
    { 0xd5c422b2, 98 },
    { 0xf50422b2, 33 },
    { 0x2d1422b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b422b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67422bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1422c1, 99 },
    { 0xc73422c2, 15 },
    { 0, 0 },
    { 0x526422c4, 95 },
    { 0, 0 },
    { 0x913422c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x171422c9, 46 },
    { 0xeba422ca, 33 },
    { 0x4422cb, 33 },
    { 0, 0 },
    { 0x7bb422cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0422d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28422d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad422db, 55 },
    { 0x236422dc, 23 },
    { 0xd2e422dd, 61 },
    { 0xb4f422dd, 67 },
    { 0x761422df, 61 },
    { 0x8c4422dd, 61 },
    { 0xe6f422e0, 94 },
    { 0xb14422e2, 61 },
    { 0, 0 },
    { 0x836422e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c422e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79422ea, 90 },
    { 0x8b422eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1422f6, 95 },
    { 0xf0e422f7, 55 },
    { 0x702422f6, 33 },
    { 0x69d422f9, 55 },
    { 0, 0 },
    { 0xf19422fb, 90 },
    { 0, 0 },
    { 0x925422fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f42304, 40 },
    { 0, 0 },
    { 0xc4742306, 43 },
    { 0, 0 },
    { 0x26f42308, 23 },
    { 0x81f42309, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f4230c, 76 },
    { 0x9ad4230d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35242311, 67 },
    { 0xa1542312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17542318, 91 },
    { 0xa2142318, 100 },
    { 0x4994231a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3384231e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d42321, 66 },
    { 0x1f342321, 90 },
    { 0, 0 },
    { 0x3f842324, 75 },
    { 0xc0b42325, 76 },
    { 0xd3a42325, 33 },
    { 0x2e942327, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe874232b, 33 },
    { 0x5f94232b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90442331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50942335, 76 },
    { 0x7d542336, 33 },
    { 0x57842336, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d54233b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a42341, 61 },
    { 0, 0 },
    { 0x8db42343, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68342347, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8854234c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe294234f, 61 },
    { 0x4ea42350, 77 },
    { 0, 0 },
    { 0x89a42352, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e442355, 76 },
    { 0x9d942356, 23 },
    { 0, 0 },
    { 0x43742358, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc42360, 80 },
    { 0xa9042360, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d4236a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa942370, 34 },
    { 0, 0 },
    { 0x78a42372, 76 },
    { 0, 0 },
    { 0xb2242374, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf42377, 76 },
    { 0, 0 },
    { 0xc6e42379, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba4237d, 53 },
    { 0x1444237e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc542388, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf034238b, 98 },
    { 0, 0 },
    { 0x6c34238d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd42393, 61 },
    { 0x24842394, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda42398, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d4239b, 76 },
    { 0x4f94239c, 81 },
    { 0xd8a4239d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d423a0, 76 },
    { 0xd8e423a1, 65 },
    { 0x4ff423a2, 33 },
    { 0, 0 },
    { 0xe00423a4, 33 },
    { 0x96423a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35423aa, 90 },
    { 0, 0 },
    { 0x7c5423ac, 95 },
    { 0, 0 },
    { 0x524423ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee423b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a423b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79423bc, 78 },
    { 0xc1423bd, 34 },
    { 0x65a423be, 38 },
    { 0xd35423bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1423c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x218423cc, 76 },
    { 0, 0 },
    { 0x775423ce, 55 },
    { 0x5b5423ce, 98 },
    { 0x2ed423ce, 95 },
    { 0xdbf423d1, 90 },
    { 0xbdf423d2, 76 },
    { 0x243423d1, 23 },
    { 0xb2e423d2, 61 },
    { 0xdd9423d5, 55 },
    { 0x67c423d6, 76 },
    { 0xaef423d5, 90 },
    { 0x985423d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d423db, 91 },
    { 0x709423db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x978423df, 23 },
    { 0xc71423df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db423e6, 76 },
    { 0xce5423e7, 40 },
    { 0, 0 },
    { 0xbfe423e9, 90 },
    { 0x5cf423e9, 33 },
    { 0, 0 },
    { 0x168423ec, 99 },
    { 0xf09423ed, 23 },
    { 0x9f5423ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3423f3, 66 },
    { 0x7d9423f3, 33 },
    { 0xa63423f5, 98 },
    { 0x1ef423f6, 33 },
    { 0x6f0423f6, 23 },
    { 0x26a423f4, 93 },
    { 0x464423f9, 33 },
    { 0x21e423f9, 90 },
    { 0xe1f423f6, 98 },
    { 0x702423fc, 75 },
    { 0xeef423fc, 43 },
    { 0, 0 },
    { 0xa5b423ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f042405, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b742409, 95 },
    { 0xc924240a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3384240f, 78 },
    { 0xbdd42410, 95 },
    { 0x4c142410, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61842417, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f24241b, 36 },
    { 0, 0 },
    { 0xa3d4241d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21742420, 67 },
    { 0, 0 },
    { 0x7a942422, 76 },
    { 0xd0842422, 58 },
    { 0x61f42424, 90 },
    { 0x28e42424, 33 },
    { 0xc6b42423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6042429, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd4242c, 61 },
    { 0xc594242c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1342432, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92142435, 61 },
    { 0, 0 },
    { 0xaf042437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b94243e, 93 },
    { 0, 0 },
    { 0x22642440, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96042443, 33 },
    { 0xf2a42444, 81 },
    { 0, 0 },
    { 0x21c42446, 33 },
    { 0xf3742447, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb014244f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b42452, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e42456, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce4245a, 43 },
    { 0xb374245a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf42462, 61 },
    { 0xe3c42462, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b42466, 99 },
    { 0, 0 },
    { 0x4da42468, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294246c, 33 },
    { 0xae54246d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba42477, 87 },
    { 0xad142478, 91 },
    { 0, 0 },
    { 0x5614247a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94247e, 67 },
    { 0, 0 },
    { 0x29542480, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f842489, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6394248d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4642491, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe42497, 98 },
    { 0x10642498, 95 },
    { 0, 0 },
    { 0x954249a, 33 },
    { 0x6674249b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee4249e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88424a1, 80 },
    { 0x883424a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x812424a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a424a9, 95 },
    { 0xd31424aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc424ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709424b1, 40 },
    { 0, 0 },
    { 0xeca424b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a424b7, 43 },
    { 0xd8d424b8, 33 },
    { 0xdf9424b9, 43 },
    { 0x479424b9, 66 },
    { 0x7c7424b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x729424be, 55 },
    { 0xbc424bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x581424c2, 55 },
    { 0, 0 },
    { 0x2c8424c4, 99 },
    { 0, 0 },
    { 0xd76424c6, 52 },
    { 0x7d1424c6, 23 },
    { 0xf34424c8, 76 },
    { 0, 0 },
    { 0x88e424ca, 92 },
    { 0x7c4424cb, 76 },
    { 0, 0 },
    { 0xe28424cd, 23 },
    { 0, 0 },
    { 0x443424cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb424d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef424d6, 67 },
    { 0xf0424d6, 53 },
    { 0x824424d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5424db, 75 },
    { 0, 0 },
    { 0x1fd424dd, 90 },
    { 0x9f6424de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa424e2, 98 },
    { 0, 0 },
    { 0x531424e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8424ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x525424ed, 23 },
    { 0x95b424ee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9424f1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333424f5, 61 },
    { 0xc9b424f5, 77 },
    { 0x3e8424f7, 95 },
    { 0x981424f8, 43 },
    { 0x51f424f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3424fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9424ff, 90 },
    { 0, 0 },
    { 0xeb942501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb42505, 95 },
    { 0, 0 },
    { 0xf4642507, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d4250b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10142510, 33 },
    { 0x3ec42511, 98 },
    { 0, 0 },
    { 0x7f42513, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9342517, 87 },
    { 0xf3642517, 95 },
    { 0x17242519, 76 },
    { 0x8a64251a, 99 },
    { 0x84b4251a, 56 },
    { 0, 0 },
    { 0x5734251d, 55 },
    { 0, 0 },
    { 0x2034251f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49542522, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a442529, 98 },
    { 0, 0 },
    { 0xcd04252b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9342534, 33 },
    { 0x12842535, 67 },
    { 0xb2042536, 93 },
    { 0xad442537, 61 },
    { 0, 0 },
    { 0xf742539, 90 },
    { 0, 0 },
    { 0x3c94253b, 87 },
    { 0x1434253c, 85 },
    { 0xb2c4253d, 55 },
    { 0xe454253b, 95 },
    { 0x1b94253d, 81 },
    { 0xf164253f, 90 },
    { 0x28742541, 100 },
    { 0xac742541, 76 },
    { 0x1e342542, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40142546, 33 },
    { 0, 0 },
    { 0x15542548, 90 },
    { 0xf1b42548, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec14254d, 95 },
    { 0, 0 },
    { 0xcab4254f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3842552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784255c, 23 },
    { 0x9964255d, 82 },
    { 0x7134255e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b42565, 90 },
    { 0xcf842566, 83 },
    { 0x5f842567, 40 },
    { 0, 0 },
    { 0x10b42569, 23 },
    { 0, 0 },
    { 0x7d94256b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7114256e, 33 },
    { 0xa204256e, 67 },
    { 0x90142570, 95 },
    { 0xce842570, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed842574, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38242578, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4244257c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9342582, 90 },
    { 0xe4f42583, 43 },
    { 0x8e042584, 61 },
    { 0xa0142585, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c542588, 22 },
    { 0, 0 },
    { 0x1d4258a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb634258f, 33 },
    { 0x8b042590, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2204259b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa4259e, 53 },
    { 0x53b4259e, 90 },
    { 0x871425a0, 61 },
    { 0xd9d425a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05425a6, 43 },
    { 0, 0 },
    { 0xab1425a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7425ac, 95 },
    { 0x590425ad, 87 },
    { 0x68a425ad, 95 },
    { 0xe08425af, 85 },
    { 0x1b6425b0, 66 },
    { 0x256425b1, 61 },
    { 0x79b425b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed425b6, 23 },
    { 0, 0 },
    { 0xe4c425b8, 66 },
    { 0, 0 },
    { 0x3af425ba, 95 },
    { 0x513425bb, 52 },
    { 0x651425bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc425bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72425c4, 93 },
    { 0, 0 },
    { 0x7a6425c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0425ca, 46 },
    { 0x5c7425ca, 95 },
    { 0x436425cb, 90 },
    { 0xaf8425cc, 90 },
    { 0xf01425cc, 95 },
    { 0, 0 },
    { 0xe95425d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x857425d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6425dd, 61 },
    { 0xe5b425dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81425e3, 61 },
    { 0, 0 },
    { 0xd6b425e5, 90 },
    { 0, 0 },
    { 0x7b4425e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83425ed, 90 },
    { 0, 0 },
    { 0x541425ef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8425f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x370425f8, 61 },
    { 0x704425f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea425fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d42601, 84 },
    { 0xc2142602, 40 },
    { 0xc9142601, 90 },
    { 0, 0 },
    { 0x4d242605, 95 },
    { 0x4d342606, 66 },
    { 0x65b42606, 43 },
    { 0xf3a42605, 87 },
    { 0xa6d42609, 68 },
    { 0x4374260a, 90 },
    { 0x68742609, 34 },
    { 0x154260c, 61 },
    { 0x74242608, 76 },
    { 0x50142608, 55 },
    { 0, 0 },
    { 0xa3142610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22142615, 90 },
    { 0xefa42616, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e42619, 61 },
    { 0xc4942619, 90 },
    { 0x6de4261a, 33 },
    { 0x96d4261c, 98 },
    { 0, 0 },
    { 0xea64261e, 95 },
    { 0, 0 },
    { 0xf9642620, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb942623, 61 },
    { 0xa3e42624, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf514262f, 43 },
    { 0x69d42630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb142634, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c54263a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef4263d, 61 },
    { 0xa034263e, 100 },
    { 0x9524263e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d42644, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6524264a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e42654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6142658, 76 },
    { 0, 0 },
    { 0x3134265a, 65 },
    { 0xee54265a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e842661, 38 },
    { 0x20442662, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93042667, 95 },
    { 0x36c42667, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24266c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a4266f, 55 },
    { 0xabc42670, 34 },
    { 0, 0 },
    { 0x94042672, 95 },
    { 0x10f42673, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc434267b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9294267e, 23 },
    { 0, 0 },
    { 0x40842680, 78 },
    { 0xd9042681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e42685, 98 },
    { 0x5aa42686, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e4268d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c742692, 81 },
    { 0x35642693, 61 },
    { 0x7d342694, 95 },
    { 0xe8e42693, 75 },
    { 0xd3e42696, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d42699, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4269e, 99 },
    { 0xa464269f, 52 },
    { 0x5f04269f, 95 },
    { 0xd9426a1, 98 },
    { 0xc74426a2, 34 },
    { 0, 0 },
    { 0xf66426a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f426ab, 33 },
    { 0x9ee426ab, 76 },
    { 0, 0 },
    { 0xaa1426ae, 78 },
    { 0, 0 },
    { 0x4e3426b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8426b3, 90 },
    { 0xe93426b4, 67 },
    { 0xf57426b5, 76 },
    { 0x9af426b6, 98 },
    { 0x98b426b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9426c7, 61 },
    { 0xd45426c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74426cf, 78 },
    { 0xcce426cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993426d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920426d9, 23 },
    { 0x1a9426da, 61 },
    { 0xcdc426d9, 84 },
    { 0x1cb426db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d426df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45426e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51426e6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6426e9, 61 },
    { 0xea8426ea, 23 },
    { 0xc5b426e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73426ee, 66 },
    { 0x215426ef, 91 },
    { 0xb4a426f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6426f9, 98 },
    { 0xb27426fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41426fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c42701, 90 },
    { 0xffe42701, 90 },
    { 0x69642703, 23 },
    { 0, 0 },
    { 0x42742705, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234270a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854270d, 60 },
    { 0x4b94270e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e42715, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b84271c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d42720, 90 },
    { 0x3f42721, 85 },
    { 0x46942720, 23 },
    { 0, 0 },
    { 0x88042724, 90 },
    { 0xff742724, 61 },
    { 0, 0 },
    { 0x7042727, 75 },
    { 0xb342728, 95 },
    { 0x26c42728, 61 },
    { 0x5004272a, 90 },
    { 0x574272a, 61 },
    { 0, 0 },
    { 0x3e94272d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d42735, 23 },
    { 0, 0 },
    { 0x9c42737, 93 },
    { 0xc6f42737, 90 },
    { 0x13242739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a4273c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8364273f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35042744, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b642748, 93 },
    { 0x7f242749, 90 },
    { 0xe24274a, 90 },
    { 0xf664274a, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6714274f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7242756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8274275e, 55 },
    { 0x83b4275f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39342767, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c4276c, 76 },
    { 0x6944276c, 34 },
    { 0x8dd4276e, 33 },
    { 0x1884276f, 23 },
    { 0, 0 },
    { 0xd3942771, 95 },
    { 0, 0 },
    { 0xd0742773, 33 },
    { 0x87a42774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48642784, 23 },
    { 0, 0 },
    { 0xf6e42786, 90 },
    { 0, 0 },
    { 0xf1242788, 33 },
    { 0xb7a42789, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5524278e, 90 },
    { 0x1104278f, 90 },
    { 0, 0 },
    { 0xd3642791, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4279b, 82 },
    { 0, 0 },
    { 0x1534279d, 61 },
    { 0x8b74279e, 76 },
    { 0, 0 },
    { 0x290427a0, 90 },
    { 0xba3427a1, 61 },
    { 0x3dd427a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5427a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3427b0, 40 },
    { 0, 0 },
    { 0xe4c427b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d427b7, 23 },
    { 0xa6e427b7, 98 },
    { 0, 0 },
    { 0x7a0427ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db427c0, 90 },
    { 0, 0 },
    { 0x37427c2, 90 },
    { 0, 0 },
    { 0xa70427c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5427c9, 78 },
    { 0x839427c9, 90 },
    { 0x20e427cb, 33 },
    { 0, 0 },
    { 0xa02427cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90427d0, 76 },
    { 0, 0 },
    { 0x6e3427d2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953427da, 46 },
    { 0xf9a427da, 90 },
    { 0, 0 },
    { 0xb71427dd, 33 },
    { 0, 0 },
    { 0x786427df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3427e5, 66 },
    { 0x8e7427e6, 95 },
    { 0x3be427e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653427eb, 61 },
    { 0x556427eb, 61 },
    { 0x71c427ec, 23 },
    { 0xe57427ec, 95 },
    { 0xbf5427ef, 95 },
    { 0xead427ef, 33 },
    { 0x63427ee, 55 },
    { 0, 0 },
    { 0xf49427f3, 61 },
    { 0x368427f4, 76 },
    { 0, 0 },
    { 0xf92427f6, 61 },
    { 0x6ef427f6, 91 },
    { 0, 0 },
    { 0x7cd427f9, 95 },
    { 0xb78427f9, 95 },
    { 0, 0 },
    { 0xa88427fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b42801, 95 },
    { 0, 0 },
    { 0x1cb42803, 34 },
    { 0x95b42804, 33 },
    { 0x18442805, 90 },
    { 0, 0 },
    { 0x40e42807, 98 },
    { 0xabd42808, 93 },
    { 0x98342809, 23 },
    { 0xa504280a, 83 },
    { 0, 0 },
    { 0x91b4280c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b942810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd42817, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e54281d, 66 },
    { 0xe624281e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c42822, 33 },
    { 0, 0 },
    { 0xbda42824, 76 },
    { 0xcc42825, 100 },
    { 0, 0 },
    { 0xb5542827, 99 },
    { 0, 0 },
    { 0x9b542829, 55 },
    { 0x10d42829, 34 },
    { 0x56042829, 23 },
    { 0, 0 },
    { 0xb174282d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a442833, 93 },
    { 0, 0 },
    { 0x23e42835, 76 },
    { 0, 0 },
    { 0xf7e42837, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a4283a, 90 },
    { 0x5f4283b, 76 },
    { 0, 0 },
    { 0x8c44283d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c242843, 23 },
    { 0x2b442844, 84 },
    { 0x1de42843, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37042849, 55 },
    { 0xd344284a, 95 },
    { 0x6564284b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda42851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9842856, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7774285b, 33 },
    { 0x2dc4285c, 61 },
    { 0, 0 },
    { 0x79c4285e, 34 },
    { 0x79b4285f, 56 },
    { 0xadd4285e, 61 },
    { 0x25542860, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d342866, 33 },
    { 0, 0 },
    { 0x14942868, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c4286d, 33 },
    { 0xaab4286e, 84 },
    { 0, 0 },
    { 0xd4d42870, 40 },
    { 0, 0 },
    { 0x1642872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a4287e, 77 },
    { 0x8cd4287e, 91 },
    { 0xa454287f, 90 },
    { 0xe7442881, 40 },
    { 0x39442882, 95 },
    { 0, 0 },
    { 0xeab42884, 95 },
    { 0xf42885, 90 },
    { 0x3d42886, 83 },
    { 0x40a42887, 61 },
    { 0xa8b42887, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4288b, 90 },
    { 0, 0 },
    { 0xd6c4288d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff42891, 61 },
    { 0xb3b42892, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1842895, 33 },
    { 0, 0 },
    { 0xf9e42897, 95 },
    { 0x77042897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4289b, 66 },
    { 0xa424289c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6428a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0428a3, 33 },
    { 0x576428a3, 97 },
    { 0x254428a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57428ac, 84 },
    { 0xaa0428ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a428b4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3428b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8428ba, 66 },
    { 0, 0 },
    { 0x124428bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87428c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e428c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x919428c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36428ca, 33 },
    { 0, 0 },
    { 0x76a428cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22428d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7428d8, 56 },
    { 0x62e428d9, 95 },
    { 0, 0 },
    { 0x6e5428db, 88 },
    { 0xf9b428db, 52 },
    { 0x8e0428dd, 77 },
    { 0x8cd428dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983428e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414428e9, 90 },
    { 0xb03428ea, 98 },
    { 0, 0 },
    { 0x93428ec, 87 },
    { 0x3af428ec, 76 },
    { 0x41f428ee, 77 },
    { 0xe8428ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17428fc, 43 },
    { 0x65a428fd, 55 },
    { 0x31d428fc, 95 },
    { 0x54b428ff, 33 },
    { 0x8a5428fe, 55 },
    { 0xfba42901, 91 },
    { 0xef642901, 100 },
    { 0xa8242903, 98 },
    { 0xc742901, 33 },
    { 0, 0 },
    { 0x9a042906, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e94290a, 84 },
    { 0xe8c4290b, 95 },
    { 0xfb54290c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b4291d, 90 },
    { 0x7d4291e, 85 },
    { 0, 0 },
    { 0x96442920, 77 },
    { 0, 0 },
    { 0xcfc42922, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51442926, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3804292a, 23 },
    { 0xf4d4292b, 81 },
    { 0xa284292a, 96 },
    { 0, 0 },
    { 0x2074292e, 75 },
    { 0xc194292f, 98 },
    { 0x3a24292f, 98 },
    { 0x4ea42930, 52 },
    { 0x4b44292f, 33 },
    { 0x98242933, 33 },
    { 0x90a42934, 75 },
    { 0xe3142935, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9954293a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14293d, 85 },
    { 0x38f4293e, 93 },
    { 0xdf74293f, 77 },
    { 0x8eb42940, 75 },
    { 0x63442941, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8442944, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7542948, 95 },
    { 0, 0 },
    { 0x8ef4294a, 99 },
    { 0, 0 },
    { 0xddd4294c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xc344294f, 46 },
    { 0x4964294f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81442953, 84 },
    { 0x66a42953, 90 },
    { 0xfbc42955, 93 },
    { 0x18442956, 33 },
    { 0, 0 },
    { 0x2f242958, 80 },
    { 0x80a42959, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc824295d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f842963, 23 },
    { 0x37542964, 76 },
    { 0xecc42965, 76 },
    { 0x4d842965, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf024296c, 23 },
    { 0, 0 },
    { 0xfb04296e, 90 },
    { 0, 0 },
    { 0x93b42970, 90 },
    { 0xd8542971, 55 },
    { 0xcc42970, 51 },
    { 0xb6d42973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff42979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df4297e, 61 },
    { 0x8344297e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa042982, 76 },
    { 0x34342983, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e242986, 68 },
    { 0x44f42987, 95 },
    { 0x1f442986, 43 },
    { 0x7c42989, 95 },
    { 0x5164298a, 98 },
    { 0x1354298b, 99 },
    { 0x67a4298c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95842990, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2844299a, 90 },
    { 0xc974299b, 55 },
    { 0, 0 },
    { 0x8044299d, 23 },
    { 0x2234299e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01429a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67429a4, 55 },
    { 0, 0 },
    { 0xd71429a6, 33 },
    { 0xbf0429a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa429af, 95 },
    { 0, 0 },
    { 0x5ae429b1, 55 },
    { 0xef5429b2, 100 },
    { 0, 0 },
    { 0x2f6429b4, 69 },
    { 0xc96429b4, 76 },
    { 0x7f3429b6, 23 },
    { 0x9e3429b7, 34 },
    { 0x900429b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde429be, 87 },
    { 0x162429be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef429c4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf429c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c429cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152429d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942429d6, 87 },
    { 0x74f429d7, 76 },
    { 0xe02429d6, 55 },
    { 0x3b4429d8, 76 },
    { 0x1b5429d8, 87 },
    { 0xe9b429db, 99 },
    { 0xf98429dc, 100 },
    { 0x227429d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x328429e0, 84 },
    { 0, 0 },
    { 0x95429e2, 33 },
    { 0x9ed429e2, 52 },
    { 0, 0 },
    { 0x5de429e5, 23 },
    { 0x49c429e6, 17 },
    { 0xc01429e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x710429ea, 66 },
    { 0x295429eb, 95 },
    { 0, 0 },
    { 0x42f429ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321429f1, 98 },
    { 0xb6a429f2, 33 },
    { 0xc3f429f2, 96 },
    { 0x98c429f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8429f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9429fc, 90 },
    { 0x15a429fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c442a02, 90 },
    { 0, 0 },
    { 0x68742a04, 77 },
    { 0x1c542a05, 98 },
    { 0xec342a06, 95 },
    { 0x6142a07, 55 },
    { 0xafa42a06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b42a0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38042a10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c642a13, 23 },
    { 0, 0 },
    { 0x83742a15, 99 },
    { 0x6a742a15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46742a1b, 33 },
    { 0, 0 },
    { 0x8c042a1d, 78 },
    { 0xc1042a1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7442a21, 33 },
    { 0x11442a22, 55 },
    { 0, 0 },
    { 0xdc642a24, 61 },
    { 0x43e42a25, 77 },
    { 0, 0 },
    { 0xfcd42a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8942a2b, 65 },
    { 0x27242a2b, 95 },
    { 0x37342a2b, 43 },
    { 0x6b142a2d, 95 },
    { 0xf6242a2d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4342a33, 46 },
    { 0x38742a33, 61 },
    { 0x5b742a35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a42a39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe242a3c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6742a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8242a45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f42a4b, 67 },
    { 0, 0 },
    { 0xd5042a4d, 76 },
    { 0x6bf42a4e, 93 },
    { 0x9e942a4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe742a52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b42a57, 95 },
    { 0, 0 },
    { 0xa9342a59, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f42a60, 95 },
    { 0x9242a61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b942a66, 66 },
    { 0, 0 },
    { 0xbbe42a68, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63042a6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c42a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0842a72, 43 },
    { 0xcad42a72, 33 },
    { 0x65642a74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac342a78, 98 },
    { 0xa0a42a79, 70 },
    { 0xdfb42a7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a42a7d, 88 },
    { 0x43f42a7d, 23 },
    { 0x30b42a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc42a82, 55 },
    { 0x6c742a83, 34 },
    { 0xa6942a84, 33 },
    { 0, 0 },
    { 0xd6942a86, 33 },
    { 0, 0 },
    { 0x82242a88, 95 },
    { 0, 0 },
    { 0xac242a8a, 61 },
    { 0x52942a8b, 55 },
    { 0x32b42a8b, 91 },
    { 0x5d642a8b, 85 },
    { 0xfd642a8e, 61 },
    { 0x82642a8f, 98 },
    { 0, 0 },
    { 0xcd942a91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42a94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f242a98, 36 },
    { 0, 0 },
    { 0xc6742a9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2842a9e, 66 },
    { 0, 0 },
    { 0x3fb42aa0, 98 },
    { 0x5642aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f42aa6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe242aa9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042aac, 61 },
    { 0x94542aad, 81 },
    { 0x61b42aac, 95 },
    { 0xab142aac, 43 },
    { 0x9c142aaf, 43 },
    { 0x44242ab0, 83 },
    { 0x67d42ab2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f542ab7, 91 },
    { 0x9242ab8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0542ac0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f442ac3, 98 },
    { 0, 0 },
    { 0xb5642ac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e42aca, 66 },
    { 0, 0 },
    { 0xb6542acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e842acf, 98 },
    { 0, 0 },
    { 0xa0d42ad1, 17 },
    { 0x14442ad2, 67 },
    { 0xf7242ad2, 95 },
    { 0x9e442ad3, 90 },
    { 0xae542ad2, 61 },
    { 0, 0 },
    { 0x24842ad7, 95 },
    { 0xe0742ad7, 90 },
    { 0xaa042ad9, 33 },
    { 0x32b42ad7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e942adf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc142ae6, 33 },
    { 0, 0 },
    { 0x5c942ae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f42aeb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2042aee, 93 },
    { 0, 0 },
    { 0xa1142af0, 98 },
    { 0, 0 },
    { 0xfac42af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5542afb, 61 },
    { 0, 0 },
    { 0x84442afd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20642b01, 77 },
    { 0, 0 },
    { 0xa2542b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63442b06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66842b0b, 33 },
    { 0, 0 },
    { 0x16042b0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6842b13, 91 },
    { 0xc8542b14, 98 },
    { 0xaf042b13, 33 },
    { 0xf2d42b16, 76 },
    { 0xc1a42b13, 34 },
    { 0xb8442b13, 90 },
    { 0xe9d42b19, 100 },
    { 0x6f242b1a, 23 },
    { 0x7a42b1b, 34 },
    { 0xf5142b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48742b1f, 23 },
    { 0x49242b20, 33 },
    { 0xe6942b21, 65 },
    { 0x61842b22, 90 },
    { 0xc3342b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c42b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85742b2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5542b32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c42b39, 95 },
    { 0x9cf42b3a, 67 },
    { 0xf8542b3b, 76 },
    { 0x2742b3c, 33 },
    { 0xfa42b3b, 90 },
    { 0xafb42b3b, 33 },
    { 0xb1142b3c, 90 },
    { 0x5bb42b39, 61 },
    { 0xfa42b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86042b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d42b4f, 90 },
    { 0x7c442b50, 98 },
    { 0xb2742b51, 55 },
    { 0x5ca42b50, 34 },
    { 0xb4342b50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f942b57, 85 },
    { 0, 0 },
    { 0xe5d42b59, 55 },
    { 0, 0 },
    { 0x74342b5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee42b5f, 90 },
    { 0x99442b5f, 93 },
    { 0, 0 },
    { 0xb8342b62, 98 },
    { 0x50242b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c342b67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21142b6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e42b6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4042b76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a42b82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc42b90, 23 },
    { 0, 0 },
    { 0x97642b92, 33 },
    { 0xf1842b93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15942b97, 65 },
    { 0, 0 },
    { 0x4ca42b99, 61 },
    { 0x9f342b9a, 98 },
    { 0xc2242b9b, 93 },
    { 0xbad42b9b, 93 },
    { 0, 0 },
    { 0x4ca42b9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa242ba1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e842ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e42baa, 96 },
    { 0x9e542bab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b42bbb, 90 },
    { 0x7d742bbc, 23 },
    { 0xb2142bbb, 55 },
    { 0x61142bbe, 36 },
    { 0xc2a42bbd, 76 },
    { 0x4a242bbd, 78 },
    { 0x4ab42bc1, 65 },
    { 0x8e542bc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5842bc9, 23 },
    { 0, 0 },
    { 0x62342bcb, 52 },
    { 0, 0 },
    { 0xbab42bcd, 91 },
    { 0x38b42bce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c642bd2, 33 },
    { 0x3b042bd2, 61 },
    { 0xde842bd4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e42bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a542bde, 34 },
    { 0x6b742bdf, 43 },
    { 0x72942be0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37642be5, 77 },
    { 0, 0 },
    { 0x62a42be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b42bea, 90 },
    { 0x5a142beb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f42bf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1042bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9842bff, 97 },
    { 0x43542c00, 90 },
    { 0x9e342bff, 95 },
    { 0x48942c01, 68 },
    { 0xe3b42c03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd542c06, 34 },
    { 0x1f342c07, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e42c0d, 84 },
    { 0x3fa42c0e, 17 },
    { 0x70542c0f, 91 },
    { 0xaaf42c0e, 33 },
    { 0x64a42c11, 76 },
    { 0x9b542c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda242c1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82842c20, 46 },
    { 0x59e42c20, 61 },
    { 0xc4642c22, 61 },
    { 0x7f842c23, 43 },
    { 0, 0 },
    { 0x27842c25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a342c2c, 90 },
    { 0xbb142c2c, 93 },
    { 0x91942c2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da42c32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19542c36, 43 },
    { 0x36242c37, 34 },
    { 0xd1442c37, 55 },
    { 0xecb42c37, 61 },
    { 0xc8142c37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78642c45, 98 },
    { 0xa9942c46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f842c4e, 33 },
    { 0, 0 },
    { 0x88d42c50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e42c53, 43 },
    { 0xd3542c54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x85742c57, 55 },
    { 0x47442c58, 61 },
    { 0x8fe42c59, 100 },
    { 0xb0642c5a, 98 },
    { 0x1e542c5b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x26042c5e, 33 },
    { 0x6d342c5f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xed342c62, 98 },
    { 0, 0 },
    { 0xaf842c64, 90 },
    { 0x72542c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a42c6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f142c6e, 76 },
    { 0, 0 },
    { 0xdb842c70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52642c77, 98 },
    { 0xea042c78, 40 },
    { 0x3de42c79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33542c80, 43 },
    { 0x28c42c81, 91 },
    { 0x68742c82, 99 },
    { 0x2a742c82, 77 },
    { 0, 0 },
    { 0x9c242c85, 95 },
    { 0x2ce42c86, 33 },
    { 0x7ba42c86, 17 },
    { 0xed142c87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe42c8d, 81 },
    { 0xcb342c8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe242c92, 98 },
    { 0, 0 },
    { 0xddf42c94, 33 },
    { 0, 0 },
    { 0x67042c96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca42c9d, 68 },
    { 0xf8c42c9d, 34 },
    { 0xfb442c9f, 61 },
    { 0xd7442ca0, 90 },
    { 0, 0 },
    { 0x97a42ca2, 76 },
    { 0xb5e42ca3, 90 },
    { 0xb0542ca4, 61 },
    { 0xb542ca3, 43 },
    { 0xd6442ca6, 76 },
    { 0x99042ca7, 33 },
    { 0xa4a42ca8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a042cac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b42cb1, 61 },
    { 0x8e42cb2, 66 },
    { 0x64c42cb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa442cb6, 64 },
    { 0x5cf42cb7, 34 },
    { 0xfe342cb8, 95 },
    { 0x68b42cb7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9242cbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a42cc8, 68 },
    { 0, 0 },
    { 0x9d042cca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f842cd1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e42cd6, 61 },
    { 0x2a242cd7, 23 },
    { 0x11f42cd8, 23 },
    { 0x2642cd8, 61 },
    { 0x8f42cda, 90 },
    { 0x3aa42cdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1342cdf, 95 },
    { 0, 0 },
    { 0x1ab42ce1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x29242ce4, 23 },
    { 0x27d42ce5, 23 },
    { 0xeb842ce5, 91 },
    { 0x4542ce7, 91 },
    { 0x1c642ce8, 23 },
    { 0xd6542ce9, 95 },
    { 0x1b042cea, 98 },
    { 0x24e42ceb, 90 },
    { 0x39c42ce8, 34 },
    { 0xeea42ce7, 61 },
    { 0x3ee42cee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc842cf3, 95 },
    { 0xe2c42cf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0542cfd, 69 },
    { 0xdfa42cfe, 90 },
    { 0x48942cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c142d03, 90 },
    { 0, 0 },
    { 0x2f642d05, 23 },
    { 0, 0 },
    { 0x66342d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85542d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26142d0d, 56 },
    { 0x43742d0d, 90 },
    { 0, 0 },
    { 0xfdd42d10, 33 },
    { 0x2b942d11, 76 },
    { 0xaae42d12, 55 },
    { 0, 0 },
    { 0x76b42d14, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c42d1b, 76 },
    { 0x14742d1c, 76 },
    { 0xc9542d1d, 90 },
    { 0x5b642d1c, 46 },
    { 0x8c942d1f, 99 },
    { 0x84642d20, 76 },
    { 0x70c42d21, 23 },
    { 0, 0 },
    { 0x20342d23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68242d29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa42d2d, 76 },
    { 0x66b42d2d, 98 },
    { 0x57c42d2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a942d32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5742d37, 43 },
    { 0xb5d42d38, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a842d40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e42d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e42d4d, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d42d50, 90 },
    { 0x4942d51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e42d5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab942d5f, 35 },
    { 0, 0 },
    { 0xbb942d61, 76 },
    { 0, 0 },
    { 0x9f942d63, 100 },
    { 0x40e42d64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d42d67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9642d6f, 67 },
    { 0x11542d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d42d76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d42d79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff42d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58542d87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80442d8a, 66 },
    { 0xd4d42d8a, 61 },
    { 0x3a142d8c, 93 },
    { 0x96242d8a, 61 },
    { 0, 0 },
    { 0xb0242d8f, 95 },
    { 0xd3142d90, 101 },
    { 0x38e42d91, 52 },
    { 0, 0 },
    { 0x7f42d93, 17 },
    { 0xc442d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c42d9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7242d9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef42da1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8642da6, 85 },
    { 0xbcd42da7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32042db0, 77 },
    { 0, 0 },
    { 0x81642db2, 99 },
    { 0xb3b42db3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51542db8, 90 },
    { 0x86742db8, 90 },
    { 0xfaa42dba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d42dbd, 98 },
    { 0x5a942dbe, 61 },
    { 0xb7b42dbf, 33 },
    { 0x79742dc0, 23 },
    { 0, 0 },
    { 0x95d42dc2, 78 },
    { 0x12c42dc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd142dca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac42dce, 97 },
    { 0xee342dce, 33 },
    { 0x34142dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe42dd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e42dd9, 33 },
    { 0x25942dda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5942de3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb342de9, 61 },
    { 0, 0 },
    { 0xb9842deb, 76 },
    { 0x1e42deb, 90 },
    { 0x27642ded, 90 },
    { 0xbcf42dee, 23 },
    { 0x13342def, 90 },
    { 0x13c42def, 85 },
    { 0x32942def, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac42df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c42df8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb42dfe, 77 },
    { 0xb1d42dff, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59942e06, 100 },
    { 0x1642e07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2242e0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed42e0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a42e14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94642e17, 46 },
    { 0x83a42e18, 43 },
    { 0xbc442e19, 67 },
    { 0x1e942e19, 23 },
    { 0x53542e1b, 95 },
    { 0x69442e1b, 34 },
    { 0xd42e1c, 61 },
    { 0x3c642e1e, 33 },
    { 0x1ea42e1e, 76 },
    { 0xdb642e1e, 84 },
    { 0xce542e21, 33 },
    { 0x7242e1b, 55 },
    { 0x8e442e23, 98 },
    { 0xfd342e20, 56 },
    { 0xea442e25, 55 },
    { 0x34542e1c, 43 },
    { 0, 0 },
    { 0x22e42e28, 23 },
    { 0x18d42e29, 95 },
    { 0xa6942e2a, 61 },
    { 0xdc942e2a, 95 },
    { 0x45042e2c, 90 },
    { 0, 0 },
    { 0x8e42e2e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x25542e31, 33 },
    { 0x16742e32, 95 },
    { 0x7b942e32, 93 },
    { 0x7f342e34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab42e37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2842e3a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f142e40, 90 },
    { 0xdc542e41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2842e44, 90 },
    { 0, 0 },
    { 0x75642e46, 78 },
    { 0, 0 },
    { 0xc1c42e48, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43642e4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21042e4f, 98 },
    { 0, 0 },
    { 0x73642e51, 91 },
    { 0xa2142e51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b42e55, 91 },
    { 0x91e42e56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5142e59, 90 },
    { 0x1242e59, 65 },
    { 0x38f42e5a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b42e5e, 95 },
    { 0x3b742e5f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242e63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x39042e66, 99 },
    { 0x64e42e66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e42e6d, 17 },
    { 0x5b442e6e, 81 },
    { 0x2cb42e6f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17842e76, 91 },
    { 0, 0 },
    { 0x20f42e78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c42e7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca242e81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de42e86, 76 },
    { 0, 0 },
    { 0xd2942e88, 55 },
    { 0xd0242e88, 76 },
    { 0x6dd42e8a, 61 },
    { 0xf1542e89, 76 },
    { 0x7c042e8c, 91 },
    { 0x55942e89, 17 },
    { 0xdc42e8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e342e96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1842e9b, 95 },
    { 0x52142e9c, 84 },
    { 0x2a142e9c, 78 },
    { 0xb7b42e9e, 64 },
    { 0xdd442e9f, 65 },
    { 0xa9442ea0, 95 },
    { 0, 0 },
    { 0x4e242ea2, 61 },
    { 0xd3c42ea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3742eac, 52 },
    { 0x41542eac, 53 },
    { 0x4c542eae, 61 },
    { 0x8b342eae, 23 },
    { 0x7bf42eac, 85 },
    { 0, 0 },
    { 0x7bc42eb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75142eba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a42ebe, 76 },
    { 0xa1842ebf, 23 },
    { 0x98d42ec0, 98 },
    { 0x93542ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57042ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16442ec8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a442ed0, 61 },
    { 0, 0 },
    { 0x79942ed2, 23 },
    { 0x15542ed2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97742eda, 76 },
    { 0xd2242edb, 90 },
    { 0xd7842edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd242edf, 87 },
    { 0, 0 },
    { 0x8a842ee1, 46 },
    { 0, 0 },
    { 0x6ae42ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84942ee6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c42eea, 98 },
    { 0x9af42eea, 98 },
    { 0x50542eec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa742ef3, 81 },
    { 0x13442ef3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58542efa, 95 },
    { 0x80c42efa, 90 },
    { 0x8e342efc, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43442f01, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45842f08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63542f0c, 98 },
    { 0, 0 },
    { 0x4c242f0e, 33 },
    { 0xd5842f0f, 88 },
    { 0x54e42f10, 61 },
    { 0xd4a42f11, 23 },
    { 0x32c42f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd342f16, 66 },
    { 0, 0 },
    { 0x5742f18, 90 },
    { 0x58842f18, 33 },
    { 0x33142f19, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f242f1f, 33 },
    { 0xbe442f20, 90 },
    { 0x542f1f, 55 },
    { 0, 0 },
    { 0xb9542f23, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f42f26, 93 },
    { 0, 0 },
    { 0xb2d42f28, 90 },
    { 0xe3242f28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68342f2d, 23 },
    { 0, 0 },
    { 0x51242f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57942f35, 61 },
    { 0x13c42f35, 31 },
    { 0xd642f37, 33 },
    { 0x6ec42f38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b42f3c, 23 },
    { 0xbca42f3d, 100 },
    { 0x20e42f3c, 98 },
    { 0, 0 },
    { 0xe4542f40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb42f43, 95 },
    { 0x76442f44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff42f49, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e142f4e, 52 },
    { 0x9e42f4f, 33 },
    { 0x20a42f4f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14142f56, 95 },
    { 0x1be42f57, 90 },
    { 0x5cc42f57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f742f60, 75 },
    { 0x9c842f61, 67 },
    { 0x85a42f62, 77 },
    { 0x34e42f63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93742f67, 23 },
    { 0x72842f68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c42f6f, 33 },
    { 0xd2642f70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f42f77, 76 },
    { 0, 0 },
    { 0x26b42f79, 88 },
    { 0, 0 },
    { 0xbf842f7b, 90 },
    { 0xffd42f7c, 95 },
    { 0xc6442f7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a242f80, 90 },
    { 0xb9142f80, 88 },
    { 0x93a42f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3942f87, 82 },
    { 0x6d942f87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df42f91, 81 },
    { 0x6fc42f91, 58 },
    { 0x72642f93, 65 },
    { 0, 0 },
    { 0xa2042f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1042f9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44842f9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2142fa6, 33 },
    { 0x87442fa7, 90 },
    { 0xcd42fa7, 85 },
    { 0, 0 },
    { 0xec42faa, 43 },
    { 0xf1742fab, 33 },
    { 0x8e942fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d42faf, 96 },
    { 0x5fd42faf, 33 },
    { 0xf9e42fb1, 23 },
    { 0xc9042fb2, 46 },
    { 0xb3a42fb1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e42fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf42fbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3842fbe, 90 },
    { 0xa3f42fbf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f42fc2, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f442fc5, 23 },
    { 0xa6b42fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e42fd0, 92 },
    { 0x14242fd1, 95 },
    { 0x55042fd2, 78 },
    { 0x43e42fd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59742fd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66942fda, 95 },
    { 0, 0 },
    { 0x3bf42fdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd542fe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd42fe5, 76 },
    { 0, 0 },
    { 0xa5a42fe7, 90 },
    { 0, 0 },
    { 0x2f142fe9, 76 },
    { 0, 0 },
    { 0xa4d42feb, 46 },
    { 0, 0 },
    { 0xd1b42fed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e42ff2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11442ff9, 33 },
    { 0x20142ffa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90042fff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c443002, 76 },
    { 0x1143002, 33 },
    { 0, 0 },
    { 0xca943005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd143011, 78 },
    { 0x6e843012, 23 },
    { 0x56d43012, 90 },
    { 0x76843014, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa43019, 90 },
    { 0xa364301a, 90 },
    { 0x6464301a, 43 },
    { 0, 0 },
    { 0x1ab4301d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c043021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb43028, 55 },
    { 0x20643028, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554302d, 61 },
    { 0x8ac4302d, 90 },
    { 0x7874302f, 95 },
    { 0x4d243030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d43037, 66 },
    { 0, 0 },
    { 0x99343039, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d4303d, 55 },
    { 0, 0 },
    { 0xff04303f, 61 },
    { 0, 0 },
    { 0x8b743041, 90 },
    { 0, 0 },
    { 0xa7a43043, 91 },
    { 0x59a43044, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85143048, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d4304e, 61 },
    { 0xf904304e, 23 },
    { 0, 0 },
    { 0xe3f43051, 67 },
    { 0xd2343051, 42 },
    { 0x48143053, 90 },
    { 0x48343051, 33 },
    { 0x71e43055, 68 },
    { 0x55c43054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b143059, 90 },
    { 0x75d43059, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50943063, 55 },
    { 0x44d43063, 80 },
    { 0x61f43065, 61 },
    { 0x43a43064, 90 },
    { 0xcde43067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b4306a, 43 },
    { 0x9bf4306a, 33 },
    { 0, 0 },
    { 0xbe44306d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77243070, 98 },
    { 0, 0 },
    { 0xcbd43072, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e43078, 23 },
    { 0xf1443079, 55 },
    { 0x1784307a, 95 },
    { 0, 0 },
    { 0x6f04307c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a43080, 91 },
    { 0xaa343081, 95 },
    { 0xcf343082, 95 },
    { 0x12943083, 43 },
    { 0, 0 },
    { 0xbb043085, 98 },
    { 0, 0 },
    { 0x3743087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf74308e, 76 },
    { 0xc024308f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e43094, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e43097, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc4309a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df430a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43430aa, 87 },
    { 0x7c2430aa, 55 },
    { 0, 0 },
    { 0x211430ad, 23 },
    { 0x727430ae, 61 },
    { 0, 0 },
    { 0x9ab430b0, 76 },
    { 0x53a430b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x669430b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19430b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x784430bb, 69 },
    { 0x496430bc, 40 },
    { 0xd88430bd, 61 },
    { 0x3b430be, 91 },
    { 0xcf8430bc, 34 },
    { 0, 0 },
    { 0x2d5430c1, 81 },
    { 0x1b6430c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5430c7, 90 },
    { 0x18b430c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687430cc, 52 },
    { 0x50b430cc, 61 },
    { 0, 0 },
    { 0xa51430cf, 88 },
    { 0xced430d0, 23 },
    { 0x8ee430d1, 76 },
    { 0x2cf430cf, 90 },
    { 0xcc2430d2, 55 },
    { 0x20f430d4, 33 },
    { 0x424430d5, 33 },
    { 0x185430d5, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2430dc, 98 },
    { 0, 0 },
    { 0x20e430de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc430e6, 67 },
    { 0xe36430e7, 33 },
    { 0, 0 },
    { 0x7c5430e9, 61 },
    { 0x1e0430ea, 61 },
    { 0xcf5430eb, 95 },
    { 0x788430ec, 52 },
    { 0, 0 },
    { 0x257430ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd430f1, 53 },
    { 0, 0 },
    { 0xcb4430f3, 93 },
    { 0, 0 },
    { 0x1aa430f5, 96 },
    { 0, 0 },
    { 0xed430f7, 84 },
    { 0xa78430f8, 66 },
    { 0, 0 },
    { 0xa99430fa, 95 },
    { 0x26e430fb, 98 },
    { 0x8b8430fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c343100, 43 },
    { 0x51f43101, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb43104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee43107, 34 },
    { 0xe6743108, 43 },
    { 0xca443107, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c4310e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf943113, 33 },
    { 0x6ea43113, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd443117, 76 },
    { 0, 0 },
    { 0xe343119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6974311d, 52 },
    { 0x8794311d, 34 },
    { 0xa0a4311e, 90 },
    { 0x7a543120, 98 },
    { 0xc6c43120, 61 },
    { 0x64a43120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d43127, 95 },
    { 0x72b43128, 81 },
    { 0x23343129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e74312d, 98 },
    { 0, 0 },
    { 0x16a4312f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0743132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb543137, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4313b, 95 },
    { 0, 0 },
    { 0xb3a4313d, 33 },
    { 0xfaf4313e, 61 },
    { 0, 0 },
    { 0x53743140, 80 },
    { 0, 0 },
    { 0x7cf43142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9843145, 90 },
    { 0x85343146, 90 },
    { 0x7e043147, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64314c, 33 },
    { 0, 0 },
    { 0xfff4314e, 95 },
    { 0x1f64314e, 33 },
    { 0x6c54314f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c43158, 84 },
    { 0, 0 },
    { 0x7034315a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4315d, 46 },
    { 0, 0 },
    { 0xae64315f, 34 },
    { 0x8aa43160, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc843163, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab43167, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a4316b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3343171, 55 },
    { 0x1ff43172, 23 },
    { 0xf9b43173, 61 },
    { 0xda243174, 83 },
    { 0xb1d43175, 76 },
    { 0x69443172, 95 },
    { 0xb3443175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e4317a, 69 },
    { 0x1f04317b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe344317f, 91 },
    { 0, 0 },
    { 0x3d243181, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e43186, 61 },
    { 0, 0 },
    { 0xb3943188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5864318b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90643196, 90 },
    { 0xfc943197, 90 },
    { 0xef043198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5964319b, 61 },
    { 0x8094319c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b431a2, 91 },
    { 0xff0431a3, 76 },
    { 0, 0 },
    { 0x7bf431a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2431ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9431af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c431b2, 23 },
    { 0xd07431b2, 96 },
    { 0x240431b4, 93 },
    { 0x908431b5, 33 },
    { 0xf54431b6, 90 },
    { 0x8c3431b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c431bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x775431be, 95 },
    { 0x35431bf, 33 },
    { 0x49a431c0, 61 },
    { 0, 0 },
    { 0x321431c2, 23 },
    { 0x6e5431c2, 84 },
    { 0, 0 },
    { 0xf0a431c5, 85 },
    { 0x634431c6, 46 },
    { 0xba9431c7, 96 },
    { 0, 0 },
    { 0x784431c9, 88 },
    { 0xcb2431c9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330431ce, 34 },
    { 0x748431cf, 99 },
    { 0xf69431cf, 34 },
    { 0x301431cf, 95 },
    { 0x252431d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e431d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f431dc, 43 },
    { 0x318431dd, 90 },
    { 0xa6a431dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b431e4, 23 },
    { 0x7f431e5, 61 },
    { 0xbd5431e6, 93 },
    { 0, 0 },
    { 0x927431e8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721431ec, 61 },
    { 0xefe431ec, 34 },
    { 0, 0 },
    { 0xbaf431ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8431fa, 93 },
    { 0xa41431fa, 99 },
    { 0xb84431fc, 33 },
    { 0xdd8431fc, 66 },
    { 0, 0 },
    { 0x978431ff, 95 },
    { 0x83543200, 33 },
    { 0x26f43201, 61 },
    { 0, 0 },
    { 0x9e143203, 95 },
    { 0, 0 },
    { 0x9bb43205, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48143209, 33 },
    { 0x7924320a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d74320e, 90 },
    { 0x1984320e, 85 },
    { 0xde943210, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61643216, 55 },
    { 0x7a343217, 90 },
    { 0x62a43218, 61 },
    { 0x65043219, 55 },
    { 0, 0 },
    { 0xf9c4321b, 98 },
    { 0, 0 },
    { 0xfcf4321d, 96 },
    { 0x4fc4321d, 34 },
    { 0x5774321f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9443223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac143228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb474322d, 61 },
    { 0x89e4322d, 95 },
    { 0, 0 },
    { 0x92443230, 100 },
    { 0xdd843231, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb243239, 90 },
    { 0x8f44323a, 23 },
    { 0x6954323a, 34 },
    { 0, 0 },
    { 0xfcc4323d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc43240, 98 },
    { 0, 0 },
    { 0xfaa43242, 61 },
    { 0, 0 },
    { 0xad143244, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a64324f, 52 },
    { 0, 0 },
    { 0x28443251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b43257, 66 },
    { 0, 0 },
    { 0xc6c43259, 76 },
    { 0, 0 },
    { 0x6b74325b, 11 },
    { 0, 0 },
    { 0xd6a4325d, 33 },
    { 0x1d84325e, 98 },
    { 0x1f34325f, 98 },
    { 0xcbf43260, 33 },
    { 0xb9143261, 99 },
    { 0xec343260, 98 },
    { 0xe1e43263, 78 },
    { 0, 0 },
    { 0x2dd43265, 23 },
    { 0xedc43266, 55 },
    { 0x80843267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b4326f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f43273, 55 },
    { 0x65143274, 80 },
    { 0, 0 },
    { 0x52843276, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f443280, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49043286, 34 },
    { 0xb9043286, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa4328a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9094328d, 90 },
    { 0, 0 },
    { 0xb914328f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41743292, 43 },
    { 0, 0 },
    { 0x81743294, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d24329e, 76 },
    { 0x2ba4329f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13432a2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1432a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f432ab, 98 },
    { 0, 0 },
    { 0x6e6432ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x140432b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x973432b3, 90 },
    { 0, 0 },
    { 0x9e1432b5, 93 },
    { 0, 0 },
    { 0x87432b7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca432bd, 77 },
    { 0x392432be, 55 },
    { 0x287432be, 33 },
    { 0, 0 },
    { 0x930432c1, 61 },
    { 0x261432c2, 17 },
    { 0, 0 },
    { 0x7c9432c4, 76 },
    { 0x840432c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23432c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a432cf, 75 },
    { 0x5b2432d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b432d3, 66 },
    { 0, 0 },
    { 0xa5e432d5, 33 },
    { 0x900432d5, 43 },
    { 0x5c432d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff432df, 95 },
    { 0x867432e0, 61 },
    { 0, 0 },
    { 0xa85432e2, 61 },
    { 0, 0 },
    { 0x93f432e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f432e7, 35 },
    { 0xdd2432e8, 55 },
    { 0x819432e9, 34 },
    { 0x337432ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11432ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121432f1, 95 },
    { 0x2d7432f1, 43 },
    { 0xf5a432f3, 61 },
    { 0x4f1432f3, 85 },
    { 0xa1432f5, 43 },
    { 0xcf5432f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632432f9, 95 },
    { 0, 0 },
    { 0x664432fb, 23 },
    { 0x2f3432fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e43304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32043307, 70 },
    { 0, 0 },
    { 0x5e743309, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc264330d, 76 },
    { 0xcde4330d, 55 },
    { 0, 0 },
    { 0xec943310, 76 },
    { 0xb2543310, 43 },
    { 0, 0 },
    { 0x4e243313, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be43318, 78 },
    { 0x72943319, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc84331c, 90 },
    { 0, 0 },
    { 0xce84331e, 90 },
    { 0, 0 },
    { 0x46343320, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d843325, 52 },
    { 0, 0 },
    { 0xc0743327, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f4332c, 61 },
    { 0, 0 },
    { 0xdb74332e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69943337, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e24333c, 93 },
    { 0x8cd4333d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64343342, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc043353, 98 },
    { 0, 0 },
    { 0xdea43355, 43 },
    { 0xb5143356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf4335a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f4335d, 65 },
    { 0, 0 },
    { 0xdc34335f, 98 },
    { 0x4d243360, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e443377, 34 },
    { 0x5ff43378, 90 },
    { 0, 0 },
    { 0xfb44337a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97943380, 33 },
    { 0xdfb43380, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e24338a, 61 },
    { 0, 0 },
    { 0x1424338c, 46 },
    { 0x3e74338d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7843393, 23 },
    { 0xb6c43394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc134339d, 61 },
    { 0xd964339e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb433a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d433a5, 34 },
    { 0, 0 },
    { 0x29a433a7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6433ad, 90 },
    { 0x427433ae, 98 },
    { 0x27d433ad, 23 },
    { 0xce0433b0, 61 },
    { 0x8f3433b1, 61 },
    { 0x718433b2, 90 },
    { 0, 0 },
    { 0x263433b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x231433b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b433bd, 80 },
    { 0, 0 },
    { 0xa37433bf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6433c3, 61 },
    { 0x810433c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9433c8, 33 },
    { 0xbff433c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118433cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b433d0, 98 },
    { 0x9ee433d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811433d5, 75 },
    { 0xf6c433d6, 46 },
    { 0, 0 },
    { 0xc10433d8, 61 },
    { 0x66b433d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd433dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e433e2, 34 },
    { 0xa43433e2, 100 },
    { 0x8d6433e4, 88 },
    { 0x437433e2, 98 },
    { 0x61c433e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1433eb, 23 },
    { 0xfff433ec, 95 },
    { 0x97d433ec, 55 },
    { 0x94f433ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8433f1, 23 },
    { 0, 0 },
    { 0xb24433f3, 55 },
    { 0xdc6433f4, 61 },
    { 0, 0 },
    { 0x74433f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x356433f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x737433fc, 23 },
    { 0xd28433fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd43402, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7843405, 61 },
    { 0x84e43406, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f43409, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c143412, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9843417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e43423, 75 },
    { 0x1bb43423, 23 },
    { 0x1de43423, 90 },
    { 0, 0 },
    { 0xc1b43427, 95 },
    { 0xcaf43428, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf584342e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba43431, 95 },
    { 0, 0 },
    { 0xdc443433, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5364343c, 33 },
    { 0xc924343d, 90 },
    { 0x344343e, 81 },
    { 0x1684343c, 33 },
    { 0x2624343f, 23 },
    { 0, 0 },
    { 0xbbf43442, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c043446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64344b, 93 },
    { 0xae14344c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec43453, 90 },
    { 0x3d343454, 93 },
    { 0x1f843455, 91 },
    { 0x80943453, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2343459, 61 },
    { 0, 0 },
    { 0x2994345b, 90 },
    { 0, 0 },
    { 0xd984345d, 95 },
    { 0x3214345d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e43468, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe324346f, 33 },
    { 0x7b34346f, 95 },
    { 0x8e143471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf043477, 75 },
    { 0xf4f43478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc14347b, 34 },
    { 0, 0 },
    { 0x2d24347d, 33 },
    { 0x9d04347e, 55 },
    { 0xda64347f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe43482, 23 },
    { 0xf5943483, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d943486, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x74443489, 33 },
    { 0, 0 },
    { 0x2624348b, 90 },
    { 0x1544348c, 90 },
    { 0x274348c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64843492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3094349f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e434a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120434a6, 61 },
    { 0x235434a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x581434aa, 90 },
    { 0xc4d434ab, 33 },
    { 0, 0 },
    { 0xf90434ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61434b2, 100 },
    { 0x5c7434b2, 55 },
    { 0, 0 },
    { 0xdc3434b5, 98 },
    { 0x9fd434b6, 90 },
    { 0xe83434b7, 95 },
    { 0, 0 },
    { 0xc2d434b9, 33 },
    { 0x473434b9, 78 },
    { 0, 0 },
    { 0xe67434bc, 34 },
    { 0x3fc434bd, 87 },
    { 0x8c9434bc, 100 },
    { 0xa43434bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770434ce, 61 },
    { 0x7c4434ce, 43 },
    { 0x1b6434d0, 34 },
    { 0x464434d1, 90 },
    { 0xeae434cf, 98 },
    { 0, 0 },
    { 0xd6d434d4, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a434d7, 99 },
    { 0x599434d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805434dc, 90 },
    { 0xa2a434dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182434e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13434f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37434fe, 61 },
    { 0x87f434fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63143503, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea43508, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe4350b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc43513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3744351f, 90 },
    { 0x2dd43520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48643529, 78 },
    { 0xc474352a, 93 },
    { 0x3dc4352b, 33 },
    { 0xf7b4352b, 98 },
    { 0xdaa4352b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e543530, 98 },
    { 0xf6343530, 33 },
    { 0x9dc43532, 85 },
    { 0x2be43532, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc343536, 33 },
    { 0xa1b43537, 61 },
    { 0xfef43536, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39943542, 33 },
    { 0x34a43542, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc43546, 76 },
    { 0, 0 },
    { 0x8c543548, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6404354f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xba343552, 43 },
    { 0x97043553, 90 },
    { 0, 0 },
    { 0x45b43555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4534355d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc443562, 17 },
    { 0, 0 },
    { 0x4c343564, 84 },
    { 0xf5343565, 23 },
    { 0xda143566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd143577, 33 },
    { 0x5dd43578, 33 },
    { 0x73543578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b4357c, 91 },
    { 0xbfe4357d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b43582, 98 },
    { 0x41b43583, 33 },
    { 0, 0 },
    { 0x8a343585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8443589, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604358f, 90 },
    { 0xc6c43590, 23 },
    { 0xeaf43591, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab943595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf4359c, 99 },
    { 0, 0 },
    { 0x4ec4359e, 34 },
    { 0xe014359f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421435a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8435ab, 85 },
    { 0x648435ac, 61 },
    { 0x174435ac, 95 },
    { 0x4db435ae, 80 },
    { 0x874435af, 91 },
    { 0, 0 },
    { 0xf52435b1, 23 },
    { 0x500435b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d435b5, 46 },
    { 0, 0 },
    { 0x63a435b7, 77 },
    { 0x4dd435b8, 33 },
    { 0, 0 },
    { 0xef5435ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06435bf, 23 },
    { 0x982435c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5435c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6435c6, 61 },
    { 0, 0 },
    { 0xbf2435c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4435cd, 95 },
    { 0, 0 },
    { 0x188435cf, 98 },
    { 0xc4a435d0, 33 },
    { 0, 0 },
    { 0xe8c435d2, 61 },
    { 0xd74435d3, 84 },
    { 0x74b435d4, 95 },
    { 0x104435d4, 98 },
    { 0x204435d6, 33 },
    { 0, 0 },
    { 0x9ce435d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e435db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99435de, 96 },
    { 0xc96435de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375435e3, 90 },
    { 0, 0 },
    { 0x7a3435e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6435ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6435f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435f4, 46 },
    { 0, 0 },
    { 0x108435f6, 98 },
    { 0, 0 },
    { 0xfa2435f8, 93 },
    { 0xc61435f9, 85 },
    { 0, 0 },
    { 0xed1435fb, 17 },
    { 0x233435fc, 33 },
    { 0, 0 },
    { 0xf5b435fe, 23 },
    { 0x7b7435fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e43602, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c43606, 67 },
    { 0x29443607, 23 },
    { 0x36243608, 90 },
    { 0, 0 },
    { 0xf014360a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe374360e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf374361c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9214361f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f43622, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e243628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f4362c, 23 },
    { 0x19b4362c, 76 },
    { 0x3c84362d, 98 },
    { 0, 0 },
    { 0xd4643630, 55 },
    { 0, 0 },
    { 0x50443632, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee943636, 88 },
    { 0x8ca43636, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d14363c, 90 },
    { 0xf5d4363d, 95 },
    { 0xb7e4363e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24743644, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1443647, 55 },
    { 0xad643648, 76 },
    { 0x22243649, 47 },
    { 0xa934364a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5143650, 34 },
    { 0xba743651, 34 },
    { 0xeb043652, 23 },
    { 0xcd643653, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe243656, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd43659, 76 },
    { 0x4bf4365a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a4365e, 33 },
    { 0, 0 },
    { 0x18343660, 55 },
    { 0, 0 },
    { 0x66a43662, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643666, 99 },
    { 0xa843667, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca4366a, 33 },
    { 0, 0 },
    { 0xe534366c, 61 },
    { 0xdae4366d, 87 },
    { 0xcae4366d, 61 },
    { 0x3944366c, 90 },
    { 0xa343670, 90 },
    { 0xb8143670, 43 },
    { 0x3f943672, 76 },
    { 0x76d43673, 93 },
    { 0x92d43673, 84 },
    { 0xa5043673, 23 },
    { 0, 0 },
    { 0x83143677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b4367b, 33 },
    { 0x55b4367b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a94367f, 99 },
    { 0xbb4367f, 61 },
    { 0xd4943681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5143687, 87 },
    { 0x78443688, 61 },
    { 0xf7143689, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37643691, 61 },
    { 0, 0 },
    { 0x72243693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab843696, 90 },
    { 0x33643696, 43 },
    { 0x62d43698, 34 },
    { 0, 0 },
    { 0xbae4369a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240436a1, 61 },
    { 0x492436a2, 61 },
    { 0x332436a3, 23 },
    { 0x498436a3, 33 },
    { 0, 0 },
    { 0x4e436a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1436a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb436ad, 95 },
    { 0, 0 },
    { 0x18d436af, 23 },
    { 0, 0 },
    { 0xe2436b1, 61 },
    { 0x7a9436b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5436b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e436b9, 76 },
    { 0xa1436b9, 95 },
    { 0, 0 },
    { 0x351436bc, 98 },
    { 0, 0 },
    { 0x28e436be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd436c6, 81 },
    { 0xb5b436c7, 76 },
    { 0x49f436c6, 90 },
    { 0x86436c9, 61 },
    { 0, 0 },
    { 0x288436cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a436d1, 33 },
    { 0xdbb436d1, 34 },
    { 0, 0 },
    { 0xe42436d4, 90 },
    { 0xb32436d5, 66 },
    { 0, 0 },
    { 0x10d436d7, 55 },
    { 0xb8b436d8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2436db, 61 },
    { 0x17b436dc, 85 },
    { 0x61d436dc, 33 },
    { 0, 0 },
    { 0x5b8436df, 95 },
    { 0x8c7436e0, 90 },
    { 0x2b2436e1, 33 },
    { 0x946436e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a436e6, 87 },
    { 0x4ca436e6, 90 },
    { 0xc63436e6, 43 },
    { 0x352436e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e436f4, 77 },
    { 0, 0 },
    { 0xc9c436f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31436f9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b436ff, 61 },
    { 0x834436ff, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c43705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b64370a, 95 },
    { 0x27c4370b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4564370f, 78 },
    { 0x7d143710, 95 },
    { 0, 0 },
    { 0x14643712, 98 },
    { 0xf6743712, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d43717, 90 },
    { 0xb9d43718, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d4371d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b43722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b43728, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91943735, 23 },
    { 0xc7543736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10043741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f43749, 61 },
    { 0, 0 },
    { 0x7424374b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c43750, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73443754, 61 },
    { 0, 0 },
    { 0xe9443756, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf4375a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd54375d, 77 },
    { 0x46c4375e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d43762, 95 },
    { 0, 0 },
    { 0x1ac43764, 98 },
    { 0x8c943764, 61 },
    { 0xeef43766, 33 },
    { 0x7e243765, 90 },
    { 0, 0 },
    { 0xf8943769, 65 },
    { 0, 0 },
    { 0x6014376b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c64376f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17043773, 88 },
    { 0, 0 },
    { 0x8fd43775, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d4377a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe243782, 95 },
    { 0, 0 },
    { 0xdc943784, 95 },
    { 0xda543784, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6544378f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d43792, 98 },
    { 0x31d43793, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d43797, 33 },
    { 0x1d143798, 95 },
    { 0x26643798, 23 },
    { 0, 0 },
    { 0x2f84379b, 98 },
    { 0xa094379c, 97 },
    { 0, 0 },
    { 0x7504379e, 46 },
    { 0x6aa4379e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9437a6, 90 },
    { 0xb3437a7, 52 },
    { 0x2da437a8, 93 },
    { 0x42d437a8, 98 },
    { 0, 0 },
    { 0x6f8437ab, 90 },
    { 0, 0 },
    { 0x8db437ad, 61 },
    { 0xea3437ae, 90 },
    { 0, 0 },
    { 0xbdd437b0, 90 },
    { 0x860437b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89437b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45437b8, 55 },
    { 0x20437b9, 90 },
    { 0x955437ba, 23 },
    { 0x8d4437bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a437be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46437c6, 90 },
    { 0, 0 },
    { 0xc70437c8, 93 },
    { 0x887437c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe437cc, 76 },
    { 0, 0 },
    { 0xb96437ce, 48 },
    { 0xcf2437ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad437d2, 52 },
    { 0x5f0437d2, 90 },
    { 0x479437d4, 98 },
    { 0, 0 },
    { 0xfa8437d6, 23 },
    { 0x3f3437d7, 33 },
    { 0xd43437d6, 33 },
    { 0xd92437d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b437dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d437e0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0437e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd437ec, 24 },
    { 0x3e437ed, 43 },
    { 0xf34437ed, 98 },
    { 0xc27437ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b437f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93437fc, 61 },
    { 0, 0 },
    { 0xa9f437fe, 81 },
    { 0x986437fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa043808, 93 },
    { 0x7f943808, 33 },
    { 0x40c43808, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa474380d, 95 },
    { 0xe154380e, 78 },
    { 0x7b04380e, 90 },
    { 0x78543810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1043813, 33 },
    { 0x26d43814, 90 },
    { 0x63543813, 17 },
    { 0x57a43816, 61 },
    { 0x36143814, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f24381c, 95 },
    { 0, 0 },
    { 0x2594381e, 51 },
    { 0x4da4381f, 90 },
    { 0, 0 },
    { 0x3d343821, 95 },
    { 0x8a943822, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be4382a, 95 },
    { 0x4cf4382b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b43836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf4383a, 61 },
    { 0x7774383b, 98 },
    { 0x6894383a, 76 },
    { 0xad34383d, 33 },
    { 0xad04383e, 98 },
    { 0x2c44383f, 33 },
    { 0x51243840, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a143843, 87 },
    { 0, 0 },
    { 0x52b43845, 90 },
    { 0x96543845, 65 },
    { 0xfbb43847, 101 },
    { 0x3ef43847, 76 },
    { 0xaad43849, 43 },
    { 0, 0 },
    { 0x3174384b, 46 },
    { 0x9514384c, 95 },
    { 0, 0 },
    { 0x6fe4384e, 23 },
    { 0xcd4384e, 55 },
    { 0xb164384e, 78 },
    { 0xb3e43851, 23 },
    { 0xb6643852, 90 },
    { 0x4e743852, 34 },
    { 0xfef43853, 55 },
    { 0xd0043852, 95 },
    { 0xd643856, 66 },
    { 0, 0 },
    { 0x3bc43858, 98 },
    { 0xf6343859, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4804385c, 98 },
    { 0xa1d4385d, 55 },
    { 0, 0 },
    { 0x28b4385f, 33 },
    { 0, 0 },
    { 0x77443861, 55 },
    { 0, 0 },
    { 0x91b43863, 81 },
    { 0, 0 },
    { 0x47943865, 56 },
    { 0x18943865, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1143870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0643875, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a943879, 43 },
    { 0x4f343879, 99 },
    { 0xaf24387a, 46 },
    { 0, 0 },
    { 0x8164387d, 55 },
    { 0x5e84387e, 98 },
    { 0x8414387f, 86 },
    { 0x71a43880, 96 },
    { 0x24e43881, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d43885, 66 },
    { 0xd2f43885, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de43889, 90 },
    { 0, 0 },
    { 0x3b24388b, 61 },
    { 0, 0 },
    { 0x3924388d, 90 },
    { 0x3734388d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b743892, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd043897, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d4389b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8438a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63438a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7438ae, 61 },
    { 0xdde438af, 55 },
    { 0, 0 },
    { 0xa96438b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82438b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f438bc, 96 },
    { 0x9e9438bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb438c0, 68 },
    { 0xb67438c1, 23 },
    { 0x201438c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x184438c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b438cd, 55 },
    { 0x3aa438ce, 90 },
    { 0xb8438cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x404438d2, 98 },
    { 0x864438d3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e438d6, 55 },
    { 0xb36438d6, 33 },
    { 0xd33438d8, 43 },
    { 0x7d5438d8, 93 },
    { 0xb51438d8, 61 },
    { 0xac8438db, 23 },
    { 0x702438dc, 33 },
    { 0xbdd438dc, 61 },
    { 0, 0 },
    { 0xd14438df, 76 },
    { 0xaea438e0, 76 },
    { 0, 0 },
    { 0x6a2438e2, 87 },
    { 0xa59438e3, 43 },
    { 0xbc8438e2, 23 },
    { 0, 0 },
    { 0x7f4438e6, 52 },
    { 0, 0 },
    { 0xe3438e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e438ec, 43 },
    { 0xa90438ed, 95 },
    { 0xb64438ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97438f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59438f8, 91 },
    { 0x29a438f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d438fc, 90 },
    { 0xbfe438fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e543902, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52043905, 98 },
    { 0x73343905, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe884390f, 93 },
    { 0x26b4390f, 98 },
    { 0xb5f4390f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e343916, 33 },
    { 0, 0 },
    { 0x23643918, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc634391d, 95 },
    { 0x3d54391e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd43921, 61 },
    { 0x35f43922, 96 },
    { 0x38b43922, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb943928, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d4392c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7543937, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec4393b, 98 },
    { 0xcc34393b, 61 },
    { 0xfe04393d, 95 },
    { 0, 0 },
    { 0x7e24393f, 66 },
    { 0x8554393f, 76 },
    { 0xa7943941, 46 },
    { 0xfb043941, 95 },
    { 0xd1f43943, 34 },
    { 0, 0 },
    { 0xc4443945, 33 },
    { 0x15243946, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde34394c, 23 },
    { 0xe084394c, 98 },
    { 0xf004394c, 3 },
    { 0, 0 },
    { 0x46943950, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa943953, 93 },
    { 0x6243953, 93 },
    { 0xe6d43955, 61 },
    { 0xa6843953, 43 },
    { 0xd9543957, 93 },
    { 0x72643958, 56 },
    { 0, 0 },
    { 0x8d94395a, 76 },
    { 0xf344395b, 90 },
    { 0xbc94395b, 99 },
    { 0xb354395b, 55 },
    { 0xd44395d, 76 },
    { 0x1774395d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee43964, 98 },
    { 0x3c243964, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8443968, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3884397b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba4397f, 98 },
    { 0xb544397f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99143989, 55 },
    { 0xc3543989, 61 },
    { 0x2ae43989, 53 },
    { 0x4464398c, 91 },
    { 0xd2a4398c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e43996, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a4399d, 99 },
    { 0, 0 },
    { 0xd354399f, 23 },
    { 0, 0 },
    { 0xf12439a1, 95 },
    { 0xd95439a2, 98 },
    { 0, 0 },
    { 0xcff439a4, 46 },
    { 0xd8c439a5, 34 },
    { 0x263439a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820439ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5439af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04439b2, 55 },
    { 0xdab439b3, 23 },
    { 0xfd7439b2, 55 },
    { 0x20d439b5, 95 },
    { 0xb83439b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418439be, 93 },
    { 0xa08439bf, 98 },
    { 0xaf2439c0, 87 },
    { 0x2a8439c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef439c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be439cb, 33 },
    { 0x3a4439cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858439d4, 95 },
    { 0x20a439d5, 90 },
    { 0x273439d4, 61 },
    { 0x5a0439d7, 99 },
    { 0x26f439d8, 87 },
    { 0x82c439d7, 76 },
    { 0x241439da, 67 },
    { 0, 0 },
    { 0xbdd439dc, 90 },
    { 0xf7e439dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440439e1, 33 },
    { 0x9d0439e2, 61 },
    { 0x40d439e1, 98 },
    { 0x429439e4, 83 },
    { 0x933439e4, 55 },
    { 0xc2b439e5, 77 },
    { 0xde6439e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f439ef, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174439f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1439f9, 55 },
    { 0x216439fa, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x426439fd, 33 },
    { 0x680439fe, 76 },
    { 0x5e439fd, 93 },
    { 0x9db43a00, 90 },
    { 0x2243a01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7543a06, 90 },
    { 0x9ef43a07, 34 },
    { 0x62043a07, 34 },
    { 0x63543a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e43a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d043a12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50643a1c, 23 },
    { 0xed543a1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3143a22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e543a25, 98 },
    { 0xf3b43a26, 34 },
    { 0xd0c43a26, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d43a2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5043a36, 33 },
    { 0x66943a36, 90 },
    { 0x23f43a38, 95 },
    { 0xb8343a38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19343a3d, 98 },
    { 0x4e43a3e, 61 },
    { 0x3f443a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c43a44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4943a4a, 95 },
    { 0, 0 },
    { 0xd7a43a4c, 61 },
    { 0xb2543a4d, 34 },
    { 0xe2743a4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7443a56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc43a5d, 55 },
    { 0, 0 },
    { 0x26543a5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89743a64, 17 },
    { 0x40e43a65, 76 },
    { 0x7143a65, 95 },
    { 0xd8843a66, 33 },
    { 0x62143a68, 98 },
    { 0xc8443a69, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46043a71, 52 },
    { 0, 0 },
    { 0xa3a43a73, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d43a7c, 90 },
    { 0, 0 },
    { 0x9da43a7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd43a83, 90 },
    { 0, 0 },
    { 0x51843a85, 95 },
    { 0x1fd43a85, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb43a89, 65 },
    { 0, 0 },
    { 0xca943a8b, 69 },
    { 0x40343a8c, 34 },
    { 0xe2043a8b, 98 },
    { 0x90343a8c, 97 },
    { 0x56b43a8c, 90 },
    { 0xc7a43a90, 40 },
    { 0x5e843a91, 99 },
    { 0xe3843a92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f43a9d, 90 },
    { 0x99c43a9e, 61 },
    { 0, 0 },
    { 0x89a43aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac143aa3, 95 },
    { 0xe1a43aa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96243aab, 61 },
    { 0, 0 },
    { 0x75b43aad, 52 },
    { 0xe6943aae, 23 },
    { 0x64b43aaf, 95 },
    { 0x84343aae, 65 },
    { 0xb4543ab1, 61 },
    { 0x56743ab2, 33 },
    { 0xee443aad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e43ab7, 98 },
    { 0, 0 },
    { 0x16643ab9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8343abd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f43ac1, 33 },
    { 0, 0 },
    { 0x59143ac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8343ac6, 40 },
    { 0xbf343ac7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99043acc, 93 },
    { 0, 0 },
    { 0x4ed43ace, 33 },
    { 0x54543acf, 66 },
    { 0x7d043ad0, 85 },
    { 0x99b43ad1, 85 },
    { 0xb2443ad0, 68 },
    { 0x83443ad2, 67 },
    { 0, 0 },
    { 0x19c43ad5, 90 },
    { 0x9ce43ad5, 90 },
    { 0, 0 },
    { 0xaec43ad8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4143adb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9243ae0, 84 },
    { 0, 0 },
    { 0xd4643ae2, 33 },
    { 0x8e143ae3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44343aeb, 84 },
    { 0, 0 },
    { 0x44143aed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e743af0, 98 },
    { 0x2e43af1, 55 },
    { 0, 0 },
    { 0xcb443af3, 23 },
    { 0xdf43af4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5743af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74043afd, 84 },
    { 0, 0 },
    { 0xe4743aff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb543b02, 87 },
    { 0x5fb43b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef43b06, 55 },
    { 0x5dd43b07, 52 },
    { 0xf8543b07, 34 },
    { 0x85343b06, 66 },
    { 0xc4e43b08, 33 },
    { 0xbba43b07, 98 },
    { 0x2ec43b0c, 55 },
    { 0x1a543b0d, 33 },
    { 0, 0 },
    { 0xe7943b0f, 95 },
    { 0xa9143b10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d143b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23343b1a, 90 },
    { 0x3c943b1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda243b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a43b25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a243b28, 97 },
    { 0x6bf43b28, 95 },
    { 0xfb743b28, 76 },
    { 0xe2743b2b, 81 },
    { 0xdf143b2a, 90 },
    { 0xe7543b2d, 100 },
    { 0xf7f43b2e, 34 },
    { 0x5b643b2f, 91 },
    { 0xe5643b2f, 100 },
    { 0xa7e43b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d43b36, 95 },
    { 0xbc343b36, 61 },
    { 0, 0 },
    { 0x61b43b39, 90 },
    { 0, 0 },
    { 0x3b943b3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9643b41, 95 },
    { 0x91743b42, 34 },
    { 0x62043b42, 90 },
    { 0x33143b44, 66 },
    { 0x79043b41, 61 },
    { 0xf3343b46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9943b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4743b4e, 23 },
    { 0x90e43b4e, 91 },
    { 0xd4343b4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81443b54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca43b58, 61 },
    { 0x7be43b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f43b5f, 90 },
    { 0x86643b5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b743b65, 67 },
    { 0, 0 },
    { 0x15943b67, 23 },
    { 0x25c43b68, 43 },
    { 0x51543b69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85943b6d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f43b7a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4543b7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a43b8a, 90 },
    { 0x83743b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb43b91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb443b99, 66 },
    { 0x8dd43b9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96543b9d, 90 },
    { 0, 0 },
    { 0xd9643b9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d43ba4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5843ba8, 20 },
    { 0x27843ba9, 23 },
    { 0x91e43baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e43bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c043bb3, 40 },
    { 0, 0 },
    { 0x43343bb5, 87 },
    { 0, 0 },
    { 0x3e443bb7, 55 },
    { 0x74343bb7, 46 },
    { 0xa2e43bb9, 87 },
    { 0xc2543bb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73243bbf, 98 },
    { 0x2ea43bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc443bc3, 33 },
    { 0x41b43bc3, 23 },
    { 0x7f743bc4, 90 },
    { 0, 0 },
    { 0xe3c43bc7, 61 },
    { 0x22043bc8, 34 },
    { 0x1ff43bc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57543bcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1543bcf, 98 },
    { 0x91e43bcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a243bd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55143bd7, 46 },
    { 0, 0 },
    { 0xad243bd9, 33 },
    { 0x19b43bda, 61 },
    { 0xd5643bdb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44343be0, 76 },
    { 0xb7043be0, 91 },
    { 0x16343be1, 33 },
    { 0x42e43be3, 34 },
    { 0, 0 },
    { 0xd7e43be5, 61 },
    { 0xeae43be6, 33 },
    { 0xa3443be6, 75 },
    { 0x8d43be6, 66 },
    { 0x81843be9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81443bee, 90 },
    { 0x36443bee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28543bf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab543bfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac043c00, 78 },
    { 0xd0943c00, 95 },
    { 0x61c43c00, 90 },
    { 0x47c43c02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf43c06, 66 },
    { 0, 0 },
    { 0x97f43c08, 87 },
    { 0, 0 },
    { 0xcfa43c0a, 61 },
    { 0x5e143c0b, 68 },
    { 0x27b43c0c, 98 },
    { 0xbe643c0b, 90 },
    { 0x8e043c0d, 33 },
    { 0xad43c0f, 46 },
    { 0x9f143c0c, 77 },
    { 0x62143c11, 23 },
    { 0xd7043c12, 55 },
    { 0x80043c13, 90 },
    { 0xf843c13, 34 },
    { 0, 0 },
    { 0x4b643c16, 67 },
    { 0xe7a43c17, 95 },
    { 0xe1343c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f43c1d, 90 },
    { 0x62243c1e, 52 },
    { 0xfbe43c1d, 93 },
    { 0, 0 },
    { 0x76443c21, 33 },
    { 0, 0 },
    { 0x7bb43c23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e843c29, 33 },
    { 0xb3143c2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b043c2f, 90 },
    { 0, 0 },
    { 0x53543c31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc43c35, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x33243c38, 61 },
    { 0, 0 },
    { 0x1b843c3a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7643c3f, 78 },
    { 0xe5343c40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80143c49, 33 },
    { 0x19443c4a, 91 },
    { 0, 0 },
    { 0x38e43c4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db43c4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52643c54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c43c5b, 46 },
    { 0xeb643c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5343c60, 23 },
    { 0x76343c61, 90 },
    { 0xb6e43c62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e43c65, 46 },
    { 0xc0d43c66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd243c6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a643c70, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1443c75, 61 },
    { 0, 0 },
    { 0x66243c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a43c7b, 52 },
    { 0x5ef43c7c, 90 },
    { 0, 0 },
    { 0x5a043c7e, 46 },
    { 0, 0 },
    { 0x40743c80, 90 },
    { 0xf0843c81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde43c91, 91 },
    { 0xf4043c92, 61 },
    { 0x6a843c93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec43c97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b43c9a, 78 },
    { 0x9e443c9a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b43ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df43ca7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a43caf, 46 },
    { 0, 0 },
    { 0xd6543cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa43cba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a943cc0, 87 },
    { 0x1ee43cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4143cc7, 23 },
    { 0x8243cc7, 46 },
    { 0xdb143cc8, 61 },
    { 0, 0 },
    { 0x31c43ccb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff343cd0, 46 },
    { 0, 0 },
    { 0x1ee43cd2, 33 },
    { 0, 0 },
    { 0x46843cd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b43cda, 40 },
    { 0x26443cdb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7543cdf, 61 },
    { 0x44c43cdf, 43 },
    { 0x50a43ce1, 43 },
    { 0xac443ce2, 66 },
    { 0x4bb43ce3, 61 },
    { 0xcb143ce4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1643cea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x36543ced, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc343cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d443cf7, 33 },
    { 0, 0 },
    { 0x73943cf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f43cfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc43d00, 98 },
    { 0xba543d00, 46 },
    { 0, 0 },
    { 0xbdf43d03, 100 },
    { 0x31543d04, 33 },
    { 0x23943d05, 68 },
    { 0xf9743d04, 61 },
    { 0xd2243d07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de43d0c, 98 },
    { 0, 0 },
    { 0xb543d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc43d14, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e43d19, 55 },
    { 0xff743d1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a443d1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56243d21, 23 },
    { 0xba543d22, 33 },
    { 0, 0 },
    { 0x45043d24, 95 },
    { 0, 0 },
    { 0x6e743d26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc743d2a, 98 },
    { 0xf8143d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a43d2f, 24 },
    { 0x5643d30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b843d33, 78 },
    { 0xfa543d33, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69543d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2643d40, 43 },
    { 0, 0 },
    { 0x64f43d42, 98 },
    { 0xae643d43, 101 },
    { 0x9ab43d43, 90 },
    { 0x2b243d43, 78 },
    { 0x69c43d44, 98 },
    { 0, 0 },
    { 0x32843d48, 85 },
    { 0, 0 },
    { 0x40c43d4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8343d4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x71943d50, 34 },
    { 0xcc643d50, 33 },
    { 0x1ac43d52, 61 },
    { 0x78543d52, 55 },
    { 0xf943d53, 65 },
    { 0, 0 },
    { 0xaac43d56, 67 },
    { 0x64c43d56, 23 },
    { 0xae743d58, 61 },
    { 0x8c43d59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a43d60, 88 },
    { 0x21f43d61, 23 },
    { 0x11043d62, 61 },
    { 0x59243d63, 93 },
    { 0x5bd43d63, 23 },
    { 0x9ad43d60, 34 },
    { 0xade43d66, 61 },
    { 0x7a143d67, 101 },
    { 0xcd943d68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1243d6c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31743d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f43d75, 55 },
    { 0, 0 },
    { 0xc6743d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c43d7e, 55 },
    { 0x1fd43d7f, 68 },
    { 0x9cc43d7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd43d85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c343d89, 34 },
    { 0xac443d8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c43d8d, 87 },
    { 0xb9643d8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed643d99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e943d9d, 46 },
    { 0, 0 },
    { 0xdca43d9f, 33 },
    { 0xbd443da0, 67 },
    { 0x10e43da1, 98 },
    { 0xf8543da2, 76 },
    { 0x2ca43da2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5543da6, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x66343da9, 33 },
    { 0x3fe43da9, 58 },
    { 0x59a43dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc43daf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce943db5, 95 },
    { 0xb5e43db5, 98 },
    { 0, 0 },
    { 0xaa843db8, 61 },
    { 0, 0 },
    { 0xd1c43dba, 98 },
    { 0xda043dbb, 81 },
    { 0, 0 },
    { 0xfe543dbd, 33 },
    { 0xbe143dbe, 23 },
    { 0, 0 },
    { 0x14743dc0, 95 },
    { 0x28b43dc1, 23 },
    { 0, 0 },
    { 0xb7543dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc43dcc, 93 },
    { 0x34843dcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73843dd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3543dd9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x22443ddc, 95 },
    { 0xc4e43ddd, 95 },
    { 0, 0 },
    { 0xe8743ddf, 95 },
    { 0, 0 },
    { 0xfa343de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf243de5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16143def, 55 },
    { 0x95743df0, 95 },
    { 0x7ac43def, 90 },
    { 0xef43df2, 90 },
    { 0, 0 },
    { 0x1a843df4, 34 },
    { 0, 0 },
    { 0xdb743df6, 90 },
    { 0, 0 },
    { 0x51243df8, 92 },
    { 0x7b443df9, 90 },
    { 0x91943dfa, 99 },
    { 0, 0 },
    { 0xbab43dfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31643dff, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x53243e02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a443e06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4543e0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c43e12, 34 },
    { 0, 0 },
    { 0xda943e14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa043e1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb43e22, 17 },
    { 0x65143e22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd443e28, 91 },
    { 0, 0 },
    { 0xe9043e2a, 90 },
    { 0xa5943e2b, 98 },
    { 0xf0043e2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf943e31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf143e34, 61 },
    { 0xa0d43e34, 96 },
    { 0, 0 },
    { 0xadc43e37, 90 },
    { 0xa0543e38, 61 },
    { 0x24743e39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c43e3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f43e3f, 101 },
    { 0, 0 },
    { 0x40043e41, 23 },
    { 0x7fc43e42, 61 },
    { 0x5c443e43, 87 },
    { 0xe0843e41, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c043e4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e43e54, 33 },
    { 0x1b943e54, 46 },
    { 0x3d943e56, 34 },
    { 0x8cf43e57, 43 },
    { 0, 0 },
    { 0xf4c43e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc843e60, 98 },
    { 0x4fb43e60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b643e64, 98 },
    { 0xa7643e65, 46 },
    { 0x36143e65, 61 },
    { 0x45443e67, 61 },
    { 0, 0 },
    { 0xb4543e69, 68 },
    { 0, 0 },
    { 0x33643e6b, 46 },
    { 0, 0 },
    { 0xda043e6d, 90 },
    { 0x79043e6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d043e76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb243e79, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62143e7d, 84 },
    { 0, 0 },
    { 0x94043e7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5343e82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1243e85, 98 },
    { 0x38743e86, 90 },
    { 0, 0 },
    { 0x98c43e88, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c43e8d, 61 },
    { 0xf3b43e8e, 85 },
    { 0xaa43e8e, 33 },
    { 0xba343e8f, 61 },
    { 0x5c543e91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b43e97, 98 },
    { 0x89043e98, 77 },
    { 0, 0 },
    { 0xc0b43e9a, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b43e9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa43ea3, 98 },
    { 0xe4e43ea3, 90 },
    { 0, 0 },
    { 0xd1843ea6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c343eaa, 90 },
    { 0x75443eab, 91 },
    { 0, 0 },
    { 0x75543ead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68643eb0, 95 },
    { 0, 0 },
    { 0xc7343eb2, 61 },
    { 0x5f443eb2, 78 },
    { 0, 0 },
    { 0xeaa43eb5, 99 },
    { 0, 0 },
    { 0xd3743eb7, 33 },
    { 0, 0 },
    { 0xdc843eb9, 93 },
    { 0x6c743eba, 76 },
    { 0x1a943ebb, 91 },
    { 0x57643ebc, 23 },
    { 0x27243ebd, 90 },
    { 0, 0 },
    { 0x4ed43ebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f43ec4, 76 },
    { 0, 0 },
    { 0xb43ec6, 90 },
    { 0x1ba43ec6, 90 },
    { 0xac943ec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf43ecb, 90 },
    { 0, 0 },
    { 0xea43ecd, 78 },
    { 0xb4443ece, 85 },
    { 0xd9943ecd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70943ed3, 33 },
    { 0x76443ed3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70443ed9, 43 },
    { 0, 0 },
    { 0x75343edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8743ee0, 99 },
    { 0xd7643ee0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7343eec, 78 },
    { 0x52843eed, 23 },
    { 0xb0343eee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1443ef3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a443efd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d843f02, 33 },
    { 0, 0 },
    { 0x96e43f04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11443f07, 33 },
    { 0xf843f07, 90 },
    { 0xe5d43f09, 34 },
    { 0x9b443f08, 46 },
    { 0, 0 },
    { 0xd0343f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76743f10, 76 },
    { 0x88643f11, 98 },
    { 0x96343f12, 85 },
    { 0xccd43f13, 61 },
    { 0xf0c43f12, 61 },
    { 0xd3b43f13, 34 },
    { 0x31543f16, 98 },
    { 0x27143f17, 55 },
    { 0x28843f18, 100 },
    { 0xced43f18, 66 },
    { 0x8043f13, 96 },
    { 0, 0 },
    { 0x77243f1c, 98 },
    { 0, 0 },
    { 0x28f43f1e, 61 },
    { 0x48143f1f, 90 },
    { 0xad143f1f, 98 },
    { 0xdf43f1e, 98 },
    { 0, 0 },
    { 0xdcf43f23, 61 },
    { 0x75143f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf843f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe543f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47343f33, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50343f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff43f3d, 68 },
    { 0xece43f3d, 90 },
    { 0, 0 },
    { 0x6e143f40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6343f43, 90 },
    { 0x7e43f43, 17 },
    { 0xce543f45, 90 },
    { 0x41543f43, 98 },
    { 0xb5f43f47, 61 },
    { 0x57d43f48, 61 },
    { 0xad743f48, 83 },
    { 0xc2843f43, 33 },
    { 0xcdf43f48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba43f51, 78 },
    { 0x36143f52, 90 },
    { 0, 0 },
    { 0xb7c43f54, 95 },
    { 0x35443f55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e43f58, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a43f5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a943f60, 95 },
    { 0x58c43f60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44343f65, 33 },
    { 0, 0 },
    { 0x67343f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70943f6e, 34 },
    { 0xd4f43f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd443f77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87843f84, 34 },
    { 0, 0 },
    { 0x65843f86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10643f8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad143f8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e43f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5243f95, 67 },
    { 0x10643f96, 33 },
    { 0x74243f97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71343fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a43fa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d43fa8, 67 },
    { 0xcf643fa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f43fae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf243fb3, 61 },
    { 0xed143fb4, 52 },
    { 0, 0 },
    { 0xd2843fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c843fbd, 98 },
    { 0x7c43fbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1143fc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99243fc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e43fce, 23 },
    { 0, 0 },
    { 0x81e43fd0, 76 },
    { 0xcae43fd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e43fd6, 95 },
    { 0x15b43fd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2843fda, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e43fdd, 52 },
    { 0xac343fdd, 90 },
    { 0xef443fdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15943fe3, 95 },
    { 0xbfc43fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d43fe7, 46 },
    { 0, 0 },
    { 0x84443fe9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98243fee, 33 },
    { 0xe7543fef, 46 },
    { 0, 0 },
    { 0x82b43ff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x28143ff4, 84 },
    { 0x26943ff5, 40 },
    { 0xfc643ff4, 98 },
    { 0, 0 },
    { 0x68843ff8, 33 },
    { 0, 0 },
    { 0x40343ffa, 23 },
    { 0x99443ffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e44002, 90 },
    { 0xbb944003, 52 },
    { 0, 0 },
    { 0x76f44005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51944008, 84 },
    { 0x33144009, 90 },
    { 0x5e04400a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa634400d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d44010, 90 },
    { 0, 0 },
    { 0x98044012, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764401a, 98 },
    { 0x42e4401b, 76 },
    { 0, 0 },
    { 0x76b4401d, 43 },
    { 0x2e84401e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d244021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd44025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac4402b, 88 },
    { 0xdce4402c, 95 },
    { 0xf714402b, 66 },
    { 0x85c4402e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb44033, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac4403c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e44043, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f744046, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d4404a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba14404d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19644053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b844057, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f84405b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed444060, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4944069, 33 },
    { 0xd624406a, 77 },
    { 0x224406b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2844074, 99 },
    { 0xe8544075, 95 },
    { 0x33544075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde244079, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a4408c, 90 },
    { 0x69a4408d, 98 },
    { 0x7bb4408d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1944099, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345440a2, 33 },
    { 0x8fc440a3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50440a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff440ac, 90 },
    { 0, 0 },
    { 0x171440ae, 34 },
    { 0x752440af, 85 },
    { 0, 0 },
    { 0x62440b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08440b5, 90 },
    { 0, 0 },
    { 0xf5a440b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59440ba, 34 },
    { 0xe0c440ba, 90 },
    { 0xd10440bc, 90 },
    { 0x24c440bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x447440c0, 61 },
    { 0, 0 },
    { 0x7c7440c2, 61 },
    { 0xe1f440c3, 93 },
    { 0xcb1440c2, 81 },
    { 0, 0 },
    { 0xdf8440c6, 43 },
    { 0xd79440c6, 61 },
    { 0xd67440c7, 93 },
    { 0x45a440c6, 93 },
    { 0, 0 },
    { 0xe88440cb, 43 },
    { 0x5c8440cb, 90 },
    { 0xddd440cd, 87 },
    { 0x53f440ce, 23 },
    { 0xf75440cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849440d8, 61 },
    { 0x474440d9, 61 },
    { 0x382440da, 61 },
    { 0xe8f440db, 61 },
    { 0xff2440dc, 90 },
    { 0x45f440dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4440e1, 65 },
    { 0, 0 },
    { 0x814440e3, 33 },
    { 0, 0 },
    { 0xb9440e5, 76 },
    { 0xbc0440e6, 91 },
    { 0, 0 },
    { 0xca3440e8, 61 },
    { 0xcb6440e9, 76 },
    { 0xe03440e8, 7 },
    { 0xff4440eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e440ee, 81 },
    { 0xdc7440ef, 95 },
    { 0, 0 },
    { 0xf9f440f1, 76 },
    { 0xd63440f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x464440f5, 81 },
    { 0, 0 },
    { 0x42440f7, 52 },
    { 0xc24440f7, 98 },
    { 0, 0 },
    { 0xf5c440fa, 90 },
    { 0xe09440fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7440ff, 52 },
    { 0, 0 },
    { 0xd0444101, 78 },
    { 0x6b644101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f544105, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc44110, 61 },
    { 0, 0 },
    { 0x7a844112, 90 },
    { 0, 0 },
    { 0xcb144114, 33 },
    { 0xdeb44115, 84 },
    { 0, 0 },
    { 0xe4144117, 33 },
    { 0, 0 },
    { 0x77144119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58744125, 34 },
    { 0, 0 },
    { 0xa3244127, 23 },
    { 0xb8744128, 61 },
    { 0xf4444127, 43 },
    { 0x7c944127, 66 },
    { 0xa694412b, 23 },
    { 0x4334412c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8944131, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c44134, 31 },
    { 0, 0 },
    { 0xff744136, 90 },
    { 0x8e544137, 98 },
    { 0x3c544137, 33 },
    { 0, 0 },
    { 0xcbe4413a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4144149, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e24414c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd344150, 61 },
    { 0, 0 },
    { 0xdaa44152, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c344158, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc4415c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40644165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16044168, 23 },
    { 0, 0 },
    { 0xead4416a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f44170, 53 },
    { 0xbd844171, 90 },
    { 0, 0 },
    { 0xc6844173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50144176, 43 },
    { 0x21f44177, 76 },
    { 0xc7c44178, 90 },
    { 0x59244179, 55 },
    { 0xb1944178, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xda24417d, 88 },
    { 0xa1c4417e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44244181, 33 },
    { 0, 0 },
    { 0x3be44183, 33 },
    { 0, 0 },
    { 0xd2c44185, 88 },
    { 0xd2844185, 34 },
    { 0x99044185, 55 },
    { 0x13044187, 95 },
    { 0x1944188, 52 },
    { 0xbe344188, 40 },
    { 0xd824418a, 95 },
    { 0xe5c44188, 98 },
    { 0xd3444186, 84 },
    { 0x4b34418e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76744198, 33 },
    { 0x1c844198, 90 },
    { 0x51144199, 43 },
    { 0x39744199, 43 },
    { 0x8a34419c, 99 },
    { 0, 0 },
    { 0x9834419e, 76 },
    { 0xfcf4419e, 23 },
    { 0, 0 },
    { 0xe39441a1, 90 },
    { 0xb84441a2, 95 },
    { 0x632441a1, 95 },
    { 0xe9f441a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c441a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2441ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec441b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f441bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be441c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8441c6, 98 },
    { 0x4c441c6, 76 },
    { 0x8fa441c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b441ce, 98 },
    { 0, 0 },
    { 0xf7c441d0, 43 },
    { 0, 0 },
    { 0x5ff441d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545441d8, 69 },
    { 0x27a441d9, 23 },
    { 0xdb8441d9, 90 },
    { 0xa6f441d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d441e0, 77 },
    { 0xc0d441e1, 66 },
    { 0x49e441e2, 95 },
    { 0x5b3441e1, 91 },
    { 0x190441e2, 99 },
    { 0x4a7441e4, 61 },
    { 0x12441e2, 90 },
    { 0, 0 },
    { 0xa5e441e8, 78 },
    { 0x6f7441e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400441f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a441f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0441f9, 61 },
    { 0x55d441f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f44203, 98 },
    { 0x81244204, 55 },
    { 0xabe44204, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d44209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe334420f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe44212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b744215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d4421c, 34 },
    { 0x6634421c, 98 },
    { 0x6324421c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82244225, 43 },
    { 0, 0 },
    { 0xff944227, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e94422a, 61 },
    { 0xe424422a, 34 },
    { 0, 0 },
    { 0xfbe4422d, 98 },
    { 0x1ab4422e, 33 },
    { 0xf544422f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5244233, 55 },
    { 0x17444234, 88 },
    { 0x2d644234, 55 },
    { 0xddf44234, 33 },
    { 0x1e244233, 23 },
    { 0x17344236, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c24423b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb44241, 23 },
    { 0x74e44242, 81 },
    { 0xbd944242, 90 },
    { 0x8144244, 34 },
    { 0x37644245, 34 },
    { 0x29044243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25344249, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7944252, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4044425d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e44260, 33 },
    { 0x62244261, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e44266, 85 },
    { 0x67844267, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da4426c, 55 },
    { 0, 0 },
    { 0x63f4426e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e14427d, 91 },
    { 0x6244427e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6444281, 88 },
    { 0xa7244281, 76 },
    { 0, 0 },
    { 0x1ec44284, 23 },
    { 0xc644285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa074428a, 76 },
    { 0xe3d4428a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a34428f, 61 },
    { 0, 0 },
    { 0x21644291, 17 },
    { 0xcfa44292, 23 },
    { 0x4ef44293, 76 },
    { 0x78f44292, 55 },
    { 0x51b44294, 61 },
    { 0x3ca44294, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e44299, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3442a0, 93 },
    { 0x810442a1, 55 },
    { 0, 0 },
    { 0xd9f442a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x167442a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b442ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f442b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7442c3, 78 },
    { 0, 0 },
    { 0x820442c5, 38 },
    { 0x6dd442c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a442c9, 23 },
    { 0x1d7442c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71442d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657442d7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8442dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab442df, 76 },
    { 0, 0 },
    { 0x930442e1, 23 },
    { 0xe2a442e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c442e5, 90 },
    { 0xb81442e6, 95 },
    { 0x6b3442e7, 88 },
    { 0x941442e8, 43 },
    { 0x7ee442e9, 61 },
    { 0x180442ea, 90 },
    { 0, 0 },
    { 0x382442ec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570442f0, 95 },
    { 0, 0 },
    { 0xec2442f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8442f7, 98 },
    { 0, 0 },
    { 0xf9a442f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04442fd, 34 },
    { 0x766442fe, 90 },
    { 0xe4b442fe, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80544308, 33 },
    { 0, 0 },
    { 0x59d4430a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8544430d, 61 },
    { 0, 0 },
    { 0x94a4430f, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x22844312, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b44317, 84 },
    { 0x6c144317, 3 },
    { 0x23044319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d4431c, 66 },
    { 0, 0 },
    { 0x3e34431e, 55 },
    { 0xa544431e, 23 },
    { 0, 0 },
    { 0x7be44321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26844325, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f4432a, 93 },
    { 0xab34432a, 95 },
    { 0xb864432c, 76 },
    { 0x4af4432a, 65 },
    { 0xa304432e, 90 },
    { 0x8ef4432f, 99 },
    { 0x4b744330, 17 },
    { 0x53a4432e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e44335, 62 },
    { 0x85244335, 98 },
    { 0xcf944337, 55 },
    { 0xf8544337, 98 },
    { 0, 0 },
    { 0x1e24433a, 90 },
    { 0x2e84433a, 33 },
    { 0x5844433a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c044344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a844347, 92 },
    { 0x43d44348, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee4434e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d44351, 87 },
    { 0x73544352, 80 },
    { 0, 0 },
    { 0x17e44354, 61 },
    { 0x83844355, 95 },
    { 0x97444354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea4435a, 43 },
    { 0x9d74435a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf944435f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd444366, 95 },
    { 0x11d44366, 90 },
    { 0, 0 },
    { 0xef944369, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5394436f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb44437b, 100 },
    { 0xe34437b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb814437f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f444386, 65 },
    { 0, 0 },
    { 0x21244388, 95 },
    { 0x48944388, 33 },
    { 0xd6244389, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c44438d, 46 },
    { 0x5734438e, 95 },
    { 0xcf44438f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8244395, 90 },
    { 0, 0 },
    { 0xc3744397, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a4439c, 76 },
    { 0, 0 },
    { 0xeff4439e, 33 },
    { 0xd474439f, 90 },
    { 0x194439e, 52 },
    { 0, 0 },
    { 0x3a5443a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d443a6, 95 },
    { 0x5e0443a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb443ab, 98 },
    { 0x826443ac, 99 },
    { 0, 0 },
    { 0x43a443ae, 61 },
    { 0x8fe443af, 65 },
    { 0x86e443b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19443b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f443b8, 76 },
    { 0x473443b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45443bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9443bf, 95 },
    { 0xf76443c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x873443c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603443c8, 61 },
    { 0xdf6443c9, 33 },
    { 0xf1e443ca, 95 },
    { 0x558443cb, 33 },
    { 0x222443cc, 90 },
    { 0x3c0443cb, 98 },
    { 0xca4443c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80443d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127443dc, 84 },
    { 0, 0 },
    { 0x479443de, 61 },
    { 0, 0 },
    { 0x975443e0, 93 },
    { 0x8e1443e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6443e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1443ea, 90 },
    { 0xdc3443eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50443ee, 33 },
    { 0xdf0443ee, 33 },
    { 0xc73443f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5443f4, 51 },
    { 0xf93443f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f443fd, 33 },
    { 0xc12443fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e644403, 33 },
    { 0xe3644403, 77 },
    { 0x37444405, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d944408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd44414, 33 },
    { 0, 0 },
    { 0x3ba44416, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x95744419, 76 },
    { 0, 0 },
    { 0x7fd4441b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c44420, 78 },
    { 0x64144421, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde44426, 55 },
    { 0x8a644427, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce4442a, 95 },
    { 0, 0 },
    { 0x3164442c, 76 },
    { 0xaee4442d, 95 },
    { 0xbe34442e, 98 },
    { 0x7cf4442f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54244433, 33 },
    { 0xd3c44434, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c44438, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27744442, 100 },
    { 0xdc444443, 33 },
    { 0xf5e44444, 43 },
    { 0xef244444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee844448, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27044453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f44459, 33 },
    { 0x23a4445a, 100 },
    { 0, 0 },
    { 0xde44445c, 90 },
    { 0, 0 },
    { 0xb7b4445e, 36 },
    { 0x4d04445f, 98 },
    { 0, 0 },
    { 0xa3544461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd144464, 90 },
    { 0xf2d44465, 90 },
    { 0x5b644466, 91 },
    { 0x26c44467, 61 },
    { 0, 0 },
    { 0x41d44469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4446d, 98 },
    { 0xb244446e, 90 },
    { 0x7ac4446e, 99 },
    { 0xd44446e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d44476, 33 },
    { 0xe4c44477, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea64447b, 43 },
    { 0x36e4447b, 55 },
    { 0x6104447d, 95 },
    { 0xa04447c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3544484, 17 },
    { 0, 0 },
    { 0x50844486, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3244489, 23 },
    { 0xc574448a, 46 },
    { 0xb34448b, 61 },
    { 0xe0c4448b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa414448f, 91 },
    { 0x2a44490, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b44497, 90 },
    { 0x7144498, 33 },
    { 0xb3344499, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea4449f, 52 },
    { 0xa2d444a0, 87 },
    { 0x1ed444a1, 78 },
    { 0x204444a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81444a5, 76 },
    { 0xb3e444a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859444ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790444af, 55 },
    { 0xec444af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30444b6, 90 },
    { 0xe39444b7, 90 },
    { 0x26b444b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d444bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7444c3, 61 },
    { 0xfad444c4, 55 },
    { 0xb88444c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0444c8, 51 },
    { 0x22e444c9, 78 },
    { 0x609444ca, 90 },
    { 0, 0 },
    { 0x21f444cc, 99 },
    { 0x5f0444cd, 33 },
    { 0xb3d444ce, 87 },
    { 0x8ef444ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x432444d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d444da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91444de, 98 },
    { 0, 0 },
    { 0x57d444e0, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db444e3, 33 },
    { 0x222444e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763444e9, 100 },
    { 0x7e4444ea, 95 },
    { 0, 0 },
    { 0xcb0444ec, 69 },
    { 0x4a444ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd444fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0444fe, 64 },
    { 0xea5444ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe4450e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88644514, 23 },
    { 0, 0 },
    { 0x89844516, 75 },
    { 0, 0 },
    { 0x41144518, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91844520, 33 },
    { 0, 0 },
    { 0x90444522, 61 },
    { 0, 0 },
    { 0x90144524, 33 },
    { 0xf4b44525, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc644529, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294452d, 76 },
    { 0x2db4452d, 95 },
    { 0x97c4452f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea44534, 66 },
    { 0x6cb44535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d044539, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb4453f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c44543, 93 },
    { 0xaf344544, 61 },
    { 0xec544544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7114454a, 99 },
    { 0, 0 },
    { 0xe964454c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb54454f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45144559, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f644563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7744566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f74456a, 84 },
    { 0x3f94456b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac34456f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f244574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9644577, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6724457b, 75 },
    { 0, 0 },
    { 0xb964457d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9644582, 46 },
    { 0, 0 },
    { 0xc8344584, 61 },
    { 0x6fd44584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a44588, 34 },
    { 0x34344589, 88 },
    { 0, 0 },
    { 0x14c4458b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1304458f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a44592, 61 },
    { 0x49f44593, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b344596, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b24459e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f445a1, 99 },
    { 0, 0 },
    { 0xa85445a3, 61 },
    { 0x9d5445a4, 55 },
    { 0, 0 },
    { 0x119445a6, 78 },
    { 0, 0 },
    { 0x884445a8, 90 },
    { 0x7cc445a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5445ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25445b2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a445b9, 90 },
    { 0, 0 },
    { 0xe56445bb, 61 },
    { 0, 0 },
    { 0x171445bd, 95 },
    { 0xca2445be, 91 },
    { 0xd3c445be, 65 },
    { 0, 0 },
    { 0xb00445c1, 90 },
    { 0xb28445c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407445ca, 61 },
    { 0, 0 },
    { 0x37c445cc, 91 },
    { 0x1fc445cd, 78 },
    { 0, 0 },
    { 0x694445cf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f445d3, 23 },
    { 0xd7d445d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771445d9, 61 },
    { 0x7bc445da, 90 },
    { 0, 0 },
    { 0xd20445dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697445e0, 98 },
    { 0xf98445e1, 98 },
    { 0x572445e1, 98 },
    { 0x3cb445e3, 76 },
    { 0xfa9445e4, 61 },
    { 0x465445e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x870445e8, 94 },
    { 0, 0 },
    { 0x762445ea, 55 },
    { 0, 0 },
    { 0x136445ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495445f3, 47 },
    { 0x9ea445f4, 90 },
    { 0xfd9445f5, 84 },
    { 0, 0 },
    { 0x33e445f7, 65 },
    { 0xe75445f8, 83 },
    { 0x82445f9, 76 },
    { 0xe57445fa, 93 },
    { 0x10f445fb, 76 },
    { 0xbd8445fc, 98 },
    { 0x8cc445f8, 43 },
    { 0x77d445fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a844602, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f244606, 34 },
    { 0xe7644607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f74460a, 90 },
    { 0, 0 },
    { 0x65e4460c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9344611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa964461b, 33 },
    { 0, 0 },
    { 0x8cb4461d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54844620, 93 },
    { 0xb5544620, 43 },
    { 0xed444622, 34 },
    { 0, 0 },
    { 0xf7044624, 50 },
    { 0xd7244625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a744628, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea444630, 90 },
    { 0x5d244631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e344636, 76 },
    { 0xafb44637, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c4463a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x714463d, 23 },
    { 0, 0 },
    { 0x4d4463f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4044642, 43 },
    { 0x28244643, 90 },
    { 0xee544644, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b44647, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6244650, 98 },
    { 0x59d44651, 61 },
    { 0xf5144652, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f44659, 81 },
    { 0, 0 },
    { 0x25e4465b, 93 },
    { 0x9024465b, 84 },
    { 0, 0 },
    { 0x2f14465e, 76 },
    { 0x55d4465f, 90 },
    { 0x8a044660, 43 },
    { 0, 0 },
    { 0x2de44662, 34 },
    { 0, 0 },
    { 0x43f44664, 61 },
    { 0x1da44665, 88 },
    { 0xc1044666, 84 },
    { 0, 0 },
    { 0x56544668, 43 },
    { 0x42744669, 23 },
    { 0xce24466a, 98 },
    { 0x8da4466a, 76 },
    { 0x2184466c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c944670, 38 },
    { 0, 0 },
    { 0x86444672, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73244675, 23 },
    { 0x4bf44676, 76 },
    { 0, 0 },
    { 0x18b44678, 90 },
    { 0x35044678, 93 },
    { 0, 0 },
    { 0x9584467b, 90 },
    { 0x81a4467c, 95 },
    { 0x3534467b, 98 },
    { 0, 0 },
    { 0x43f4467f, 55 },
    { 0x62444680, 90 },
    { 0x3e844680, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c44687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8904468a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab44690, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf064469b, 95 },
    { 0, 0 },
    { 0x5144469d, 95 },
    { 0xa414469d, 90 },
    { 0x544469e, 90 },
    { 0x8a34469e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6446a3, 77 },
    { 0xaf8446a3, 43 },
    { 0xcdb446a5, 80 },
    { 0x567446a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb446a9, 84 },
    { 0x80b446aa, 33 },
    { 0, 0 },
    { 0x8a0446ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56446b1, 98 },
    { 0xad4446b2, 94 },
    { 0, 0 },
    { 0x579446b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e446ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd446be, 93 },
    { 0, 0 },
    { 0x100446c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b446c4, 76 },
    { 0xd5d446c5, 33 },
    { 0x8f0446c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72446ce, 43 },
    { 0x44b446cf, 78 },
    { 0xc18446cf, 61 },
    { 0x9f7446d1, 95 },
    { 0x656446ce, 98 },
    { 0x4b9446d3, 55 },
    { 0xe16446d1, 95 },
    { 0x168446cf, 61 },
    { 0, 0 },
    { 0x77446d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931446dd, 34 },
    { 0, 0 },
    { 0x97a446df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7446e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a446e5, 17 },
    { 0x63c446e6, 61 },
    { 0x1e7446e7, 76 },
    { 0x714446e8, 75 },
    { 0xde4446e9, 68 },
    { 0, 0 },
    { 0x125446eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f446f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10446f5, 33 },
    { 0, 0 },
    { 0xe05446f7, 33 },
    { 0xbfd446f7, 33 },
    { 0xbc3446f9, 95 },
    { 0, 0 },
    { 0x5c5446fb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c446fe, 23 },
    { 0xaed446ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a44704, 90 },
    { 0x57644705, 85 },
    { 0, 0 },
    { 0x91844707, 55 },
    { 0, 0 },
    { 0x86244709, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b64470f, 23 },
    { 0xdd944710, 55 },
    { 0x8764470f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef44718, 78 },
    { 0, 0 },
    { 0xa5a4471a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc454471d, 17 },
    { 0xc8b4471e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c44725, 76 },
    { 0xe4044726, 23 },
    { 0x68544727, 33 },
    { 0x4444728, 90 },
    { 0xa3d44729, 43 },
    { 0xe2744727, 52 },
    { 0x5d844729, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb4472f, 23 },
    { 0, 0 },
    { 0xfc944731, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98244735, 76 },
    { 0xdf944735, 90 },
    { 0, 0 },
    { 0xed444738, 61 },
    { 0xf2444739, 34 },
    { 0x6804473a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c24473d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8944741, 88 },
    { 0x40444741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b44745, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294474b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4944757, 66 },
    { 0xe0344757, 23 },
    { 0, 0 },
    { 0x1204475a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c44763, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86144768, 33 },
    { 0xec244769, 34 },
    { 0x63844769, 61 },
    { 0x60b4476b, 31 },
    { 0xdd04476b, 90 },
    { 0xf954476b, 61 },
    { 0xe754476b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db44773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb44777, 100 },
    { 0xa2c44777, 80 },
    { 0xa9544778, 23 },
    { 0xe644777, 33 },
    { 0x9f94477b, 34 },
    { 0x6004477a, 95 },
    { 0xc524477d, 90 },
    { 0x78b4477c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68244781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a144784, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa314478e, 61 },
    { 0, 0 },
    { 0x83e44790, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad344794, 55 },
    { 0x5ef44795, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68044799, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd674479c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2447a4, 90 },
    { 0x1b2447a5, 33 },
    { 0x877447a6, 90 },
    { 0x88e447a6, 33 },
    { 0x9d6447a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c447ac, 61 },
    { 0x913447ac, 75 },
    { 0, 0 },
    { 0x2b447af, 93 },
    { 0x4b7447af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af447b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f447c2, 23 },
    { 0, 0 },
    { 0xe54447c4, 98 },
    { 0xf9f447c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea447c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c447cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd447d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452447d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547447e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde447e8, 52 },
    { 0, 0 },
    { 0xa01447ea, 98 },
    { 0, 0 },
    { 0xd5a447ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559447f1, 100 },
    { 0x1df447f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1447f5, 55 },
    { 0xc6c447f6, 90 },
    { 0xfd7447f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc447fa, 90 },
    { 0x4d4447fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98444800, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd444803, 88 },
    { 0xbc644804, 95 },
    { 0xe9a44805, 46 },
    { 0xf144804, 66 },
    { 0x52d44807, 98 },
    { 0, 0 },
    { 0xf9144809, 61 },
    { 0x9b544809, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a144812, 90 },
    { 0x61c44813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41344818, 98 },
    { 0x29a44819, 34 },
    { 0x2c04481a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd44824, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f34482a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6514482e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade44834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4604483a, 33 },
    { 0, 0 },
    { 0xbef4483c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da44840, 76 },
    { 0, 0 },
    { 0xdb044842, 33 },
    { 0xa4144843, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e44847, 81 },
    { 0xeaa44847, 55 },
    { 0x1b644849, 23 },
    { 0xc6b44849, 95 },
    { 0xe2b4484a, 33 },
    { 0xca4484a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6574484f, 98 },
    { 0xaee44850, 85 },
    { 0xc6144851, 90 },
    { 0xfce4484f, 69 },
    { 0x804484f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804485b, 23 },
    { 0x9cc4485c, 98 },
    { 0x4894485c, 61 },
    { 0xd1b4485e, 91 },
    { 0, 0 },
    { 0xf1d44860, 90 },
    { 0, 0 },
    { 0xe6444862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94486b, 23 },
    { 0x9c54486c, 98 },
    { 0x3754486d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c544872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae44881, 33 },
    { 0x88044882, 23 },
    { 0x34b44883, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6644886, 95 },
    { 0x4bd44887, 61 },
    { 0xdb44888, 55 },
    { 0xcb844887, 90 },
    { 0, 0 },
    { 0x17b4488b, 98 },
    { 0xeac4488c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11844890, 90 },
    { 0x49244891, 33 },
    { 0, 0 },
    { 0x54d44893, 76 },
    { 0x51c44894, 95 },
    { 0xf6a44894, 88 },
    { 0xbba44893, 98 },
    { 0x15e44896, 96 },
    { 0x1f344896, 43 },
    { 0xbfe44899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf394489e, 96 },
    { 0xb24489e, 40 },
    { 0x1f74489f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64448a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x691448a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec448af, 33 },
    { 0x5e4448af, 80 },
    { 0, 0 },
    { 0x85e448b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24448b6, 43 },
    { 0x7fe448b7, 55 },
    { 0xc50448b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644448c1, 90 },
    { 0, 0 },
    { 0xd3c448c3, 76 },
    { 0, 0 },
    { 0xce0448c5, 96 },
    { 0, 0 },
    { 0xee5448c7, 61 },
    { 0, 0 },
    { 0xfb1448c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x688448cc, 52 },
    { 0, 0 },
    { 0xad448ce, 100 },
    { 0, 0 },
    { 0xf4b448d0, 98 },
    { 0x62c448d1, 61 },
    { 0, 0 },
    { 0xf6d448d3, 81 },
    { 0, 0 },
    { 0x695448d5, 99 },
    { 0xbc4448d5, 78 },
    { 0, 0 },
    { 0x658448d8, 46 },
    { 0xa65448d9, 90 },
    { 0x638448da, 64 },
    { 0, 0 },
    { 0x148448dc, 67 },
    { 0xcab448dc, 61 },
    { 0, 0 },
    { 0x75b448df, 68 },
    { 0xbc0448e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8448e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39448e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b448ea, 61 },
    { 0, 0 },
    { 0x82448ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd448ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80448f4, 87 },
    { 0, 0 },
    { 0xbd5448f6, 61 },
    { 0xf0f448f7, 52 },
    { 0x624448f6, 90 },
    { 0x761448f8, 67 },
    { 0xf84448fa, 33 },
    { 0xee0448fb, 95 },
    { 0, 0 },
    { 0x50d448fd, 61 },
    { 0xeee448fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b44903, 46 },
    { 0, 0 },
    { 0x16a44905, 84 },
    { 0xa6b44906, 33 },
    { 0x8cb44905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb4490a, 90 },
    { 0, 0 },
    { 0x64490c, 61 },
    { 0x5cc4490d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7144911, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48044914, 61 },
    { 0x26d44914, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c04491b, 90 },
    { 0xdba4491b, 98 },
    { 0, 0 },
    { 0x8454491e, 33 },
    { 0x6ca4491f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58744925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c4492a, 87 },
    { 0, 0 },
    { 0x5bc4492c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73244930, 69 },
    { 0x2bb44931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86744939, 61 },
    { 0x57d44939, 46 },
    { 0xdd04493b, 61 },
    { 0xc4493c, 98 },
    { 0x4e64493d, 93 },
    { 0xb9d4493d, 40 },
    { 0x85d4493f, 61 },
    { 0x7e344939, 100 },
    { 0x46244941, 55 },
    { 0x80944942, 78 },
    { 0x13044942, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a4494c, 90 },
    { 0x174494d, 52 },
    { 0x764494e, 64 },
    { 0x6414494f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd44953, 46 },
    { 0, 0 },
    { 0x72644955, 34 },
    { 0xe3844956, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd4495a, 90 },
    { 0x8d64495a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf864495e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5644961, 33 },
    { 0x7b44962, 61 },
    { 0x94744963, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8844496a, 77 },
    { 0, 0 },
    { 0x90a4496c, 65 },
    { 0x2734496d, 90 },
    { 0x8424496d, 90 },
    { 0x85e4496f, 55 },
    { 0x64a4496f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e944976, 53 },
    { 0, 0 },
    { 0x7e844978, 95 },
    { 0x3d944978, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12944983, 85 },
    { 0x36f44984, 90 },
    { 0x23d44985, 99 },
    { 0x68044986, 95 },
    { 0x4bb44987, 55 },
    { 0xa3a44988, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa974498c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf524498f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14499a, 66 },
    { 0x9854499b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e4499e, 46 },
    { 0, 0 },
    { 0x5a6449a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881449a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5449af, 93 },
    { 0x97d449b0, 33 },
    { 0xfba449b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637449b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19449ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xded449bd, 90 },
    { 0x94b449bd, 93 },
    { 0xfb1449bd, 76 },
    { 0xc99449c0, 90 },
    { 0xb80449c0, 61 },
    { 0x74d449bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x907449c5, 76 },
    { 0, 0 },
    { 0x621449c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea449ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63449d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95449d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4449d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c449db, 33 },
    { 0x7e6449dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78449df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4449e7, 90 },
    { 0x92d449e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268449ed, 98 },
    { 0x44449ee, 91 },
    { 0xdb3449ef, 82 },
    { 0x2be449f0, 40 },
    { 0x98f449f1, 98 },
    { 0x946449f0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213449f6, 98 },
    { 0x576449f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x137449fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8449ff, 61 },
    { 0x9c744a00, 43 },
    { 0, 0 },
    { 0x21f44a02, 98 },
    { 0x45c44a02, 61 },
    { 0, 0 },
    { 0x7e344a05, 77 },
    { 0x38044a06, 76 },
    { 0, 0 },
    { 0xf2c44a08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2544a10, 17 },
    { 0, 0 },
    { 0xeda44a12, 61 },
    { 0, 0 },
    { 0x53044a14, 33 },
    { 0xac144a14, 91 },
    { 0x8e44a16, 99 },
    { 0x33244a17, 68 },
    { 0x25144a18, 33 },
    { 0xeed44a19, 46 },
    { 0, 0 },
    { 0xdc44a1b, 90 },
    { 0xcfd44a1b, 90 },
    { 0xed44a1c, 40 },
    { 0, 0 },
    { 0xb1844a1f, 33 },
    { 0x78944a20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2944a25, 81 },
    { 0xf7144a25, 23 },
    { 0, 0 },
    { 0xf1944a28, 33 },
    { 0, 0 },
    { 0xc4744a2a, 33 },
    { 0xf3544a2b, 46 },
    { 0x40744a2c, 34 },
    { 0x2a444a2c, 61 },
    { 0, 0 },
    { 0x87344a2f, 76 },
    { 0xf0c44a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f044a34, 91 },
    { 0x1c944a35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53244a3b, 76 },
    { 0xd6144a3c, 61 },
    { 0x2b344a3d, 23 },
    { 0xbf644a3e, 33 },
    { 0x96544a3e, 98 },
    { 0x15d44a40, 55 },
    { 0, 0 },
    { 0xa3544a42, 90 },
    { 0x2ef44a42, 90 },
    { 0, 0 },
    { 0x26a44a45, 67 },
    { 0xa1344a45, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35944a4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1044a52, 33 },
    { 0, 0 },
    { 0x74a44a54, 23 },
    { 0, 0 },
    { 0x59644a56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff44a5a, 34 },
    { 0x18344a5b, 23 },
    { 0xe8444a5c, 93 },
    { 0, 0 },
    { 0xe1b44a5e, 90 },
    { 0, 0 },
    { 0x81b44a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41444a66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d44a69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f344a6e, 90 },
    { 0x8b44a6f, 61 },
    { 0x62744a6f, 98 },
    { 0x21944a6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f144a75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e44a81, 33 },
    { 0x8c244a82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab544a86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc044a8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed144a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd44a90, 34 },
    { 0, 0 },
    { 0xae44a92, 33 },
    { 0xd9f44a93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3844a96, 85 },
    { 0x8f944a97, 91 },
    { 0x64d44a98, 46 },
    { 0xdf544a97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b44a9d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85144aa2, 84 },
    { 0xfca44aa3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e44aa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae144aab, 61 },
    { 0xbb344aac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c44ab6, 98 },
    { 0, 0 },
    { 0xcd444ab8, 85 },
    { 0x89a44ab9, 23 },
    { 0x2da44aba, 76 },
    { 0x2e44abb, 55 },
    { 0, 0 },
    { 0xbca44abd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d544ac1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58444ac5, 95 },
    { 0xf0f44ac5, 55 },
    { 0, 0 },
    { 0x31144ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b344acb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf844ad0, 61 },
    { 0xa5f44ad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea544ad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36544adb, 90 },
    { 0x81044adb, 98 },
    { 0x30c44add, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a44ae8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba44aec, 61 },
    { 0, 0 },
    { 0x2dd44aee, 61 },
    { 0x6fe44aef, 93 },
    { 0, 0 },
    { 0x64a44af1, 97 },
    { 0x2c144af2, 52 },
    { 0x6f944af3, 84 },
    { 0xc3744af3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9744afb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa44b00, 85 },
    { 0, 0 },
    { 0x85444b02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee644b0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e944b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e44b16, 76 },
    { 0xfa544b17, 65 },
    { 0xbfc44b18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a44b1d, 98 },
    { 0x3af44b1d, 95 },
    { 0x16644b1f, 68 },
    { 0x4da44b1f, 17 },
    { 0x5cd44b20, 19 },
    { 0xf944b22, 55 },
    { 0x8a644b1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa44b28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc44b2d, 34 },
    { 0x8af44b2e, 61 },
    { 0x5eb44b2f, 33 },
    { 0xcde44b30, 92 },
    { 0xf1244b30, 61 },
    { 0xc2244b32, 33 },
    { 0x32644b33, 23 },
    { 0xdd044b2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c44b38, 46 },
    { 0, 0 },
    { 0x71b44b3a, 46 },
    { 0xd844b3b, 90 },
    { 0x75144b3b, 33 },
    { 0xa344b3c, 43 },
    { 0x18f44b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e44b41, 98 },
    { 0x80644b42, 90 },
    { 0xdb044b43, 90 },
    { 0x36144b43, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf044b47, 33 },
    { 0xd1944b48, 52 },
    { 0x38344b47, 85 },
    { 0x24d44b4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf344b4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d44b50, 76 },
    { 0xcde44b50, 99 },
    { 0xd6d44b52, 98 },
    { 0x69344b51, 55 },
    { 0xdb744b54, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28044b59, 33 },
    { 0, 0 },
    { 0x74544b5b, 95 },
    { 0x2e44b5b, 61 },
    { 0, 0 },
    { 0x7a944b5e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad44b61, 87 },
    { 0xbc144b62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2644b65, 68 },
    { 0x2c244b66, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9544b69, 55 },
    { 0, 0 },
    { 0xe9244b6b, 23 },
    { 0x9f344b6b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x22844b6f, 91 },
    { 0x5ba44b70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59244b74, 52 },
    { 0xba344b75, 95 },
    { 0, 0 },
    { 0xadb44b77, 90 },
    { 0x7dc44b78, 61 },
    { 0x82244b78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb44b7e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a44b84, 90 },
    { 0xf5344b85, 39 },
    { 0x50744b86, 43 },
    { 0, 0 },
    { 0x71c44b88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf844b93, 33 },
    { 0x70a44b94, 75 },
    { 0x9ce44b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e44b99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe44b9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a44ba2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f044ba7, 95 },
    { 0xec144ba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b44bac, 90 },
    { 0, 0 },
    { 0xa3844bae, 78 },
    { 0x5d144baf, 33 },
    { 0x70a44bb0, 91 },
    { 0x92944bb0, 91 },
    { 0xf344bb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9244bb6, 98 },
    { 0, 0 },
    { 0x87e44bb8, 78 },
    { 0xa0944bb8, 55 },
    { 0xd7a44bb9, 65 },
    { 0xd4044bbb, 55 },
    { 0xa1044bbc, 98 },
    { 0xfa44bbd, 68 },
    { 0x53544bbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd744bc4, 52 },
    { 0, 0 },
    { 0x58144bc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c44bc9, 93 },
    { 0, 0 },
    { 0x4dc44bcb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2744bce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8344bd5, 88 },
    { 0xfea44bd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c44bda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5044be0, 98 },
    { 0, 0 },
    { 0x10c44be2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d744be5, 33 },
    { 0xb1e44be6, 91 },
    { 0x7a944be7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad644bef, 23 },
    { 0xfc544bef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9644bf4, 85 },
    { 0, 0 },
    { 0x81f44bf6, 52 },
    { 0xcd544bf7, 98 },
    { 0x32344bf8, 90 },
    { 0x39344bf6, 90 },
    { 0x2d244bf6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae44bfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b244c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c544c03, 34 },
    { 0x68f44c04, 33 },
    { 0, 0 },
    { 0x2cb44c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa44c0b, 23 },
    { 0x9d844c0c, 90 },
    { 0x57d44c0d, 81 },
    { 0x1a344c0e, 33 },
    { 0x64644c0c, 17 },
    { 0xe5544c0f, 33 },
    { 0x35f44c11, 95 },
    { 0x27144c11, 83 },
    { 0xb2c44c13, 33 },
    { 0xe0a44c0e, 93 },
    { 0xe4344c15, 90 },
    { 0x77e44c16, 33 },
    { 0xb4444c16, 90 },
    { 0xaa344c17, 61 },
    { 0x9bc44c19, 100 },
    { 0x94644c1a, 95 },
    { 0xd1f44c18, 33 },
    { 0, 0 },
    { 0xb3e44c1d, 61 },
    { 0x1fb44c1e, 23 },
    { 0, 0 },
    { 0x71e44c20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf44c24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba44c29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6744c2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d44c31, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b544c34, 90 },
    { 0x39644c35, 95 },
    { 0x7fd44c36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed44c3d, 23 },
    { 0xcd644c3e, 23 },
    { 0xd5144c3d, 95 },
    { 0, 0 },
    { 0x1744c41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d044c44, 55 },
    { 0xd4344c45, 66 },
    { 0, 0 },
    { 0xfae44c47, 46 },
    { 0x77144c48, 78 },
    { 0xa6944c49, 61 },
    { 0x1044c4a, 61 },
    { 0xa1b44c4a, 76 },
    { 0, 0 },
    { 0xbc44c4d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f944c51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a44c55, 76 },
    { 0, 0 },
    { 0x19044c57, 98 },
    { 0x1b244c58, 95 },
    { 0x60944c59, 33 },
    { 0xa844c5a, 90 },
    { 0, 0 },
    { 0xd9d44c5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf844c5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0944c63, 98 },
    { 0x36244c64, 34 },
    { 0x5e144c64, 95 },
    { 0x33744c66, 34 },
    { 0xbd944c67, 95 },
    { 0x35844c68, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a44c70, 61 },
    { 0x67744c71, 69 },
    { 0xb1d44c71, 46 },
    { 0x5d144c72, 34 },
    { 0xabb44c74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944c7c, 90 },
    { 0x53044c7d, 46 },
    { 0, 0 },
    { 0x82544c7f, 23 },
    { 0x74c44c80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab44c84, 90 },
    { 0xb8344c85, 90 },
    { 0xd6f44c85, 25 },
    { 0xa6644c87, 90 },
    { 0, 0 },
    { 0x98844c89, 61 },
    { 0, 0 },
    { 0x39344c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d44c90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a44c94, 55 },
    { 0x97f44c94, 61 },
    { 0, 0 },
    { 0x7bb44c97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c944c9c, 65 },
    { 0x9bc44c9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c344ca0, 85 },
    { 0xc1d44ca1, 23 },
    { 0xf7844ca1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51544ca7, 90 },
    { 0xf44ca8, 99 },
    { 0x5a844ca9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0144caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b444cb4, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b044cb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77344cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad044cc2, 90 },
    { 0x34744cc3, 55 },
    { 0x1ef44cc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a144cca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a44ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde544cd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd44cd8, 95 },
    { 0, 0 },
    { 0xac944cda, 55 },
    { 0x54844cdb, 23 },
    { 0xbc544cdc, 23 },
    { 0xe2c44cdc, 98 },
    { 0, 0 },
    { 0xd2644cdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b144ce4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb44ce7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d44cec, 95 },
    { 0xc9944ced, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f44cf0, 90 },
    { 0x78e44cf1, 90 },
    { 0xfde44cf2, 75 },
    { 0x5a44cf3, 98 },
    { 0, 0 },
    { 0xeec44cf5, 98 },
    { 0, 0 },
    { 0xe1644cf7, 61 },
    { 0x87044cf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e44cfb, 61 },
    { 0xf3944cfc, 76 },
    { 0xd2644cfc, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80844d07, 85 },
    { 0, 0 },
    { 0x97544d09, 61 },
    { 0, 0 },
    { 0x1a844d0b, 17 },
    { 0xf1c44d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75244d11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc244d14, 61 },
    { 0, 0 },
    { 0x4bd44d16, 84 },
    { 0xaa644d16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f844d1b, 76 },
    { 0x40d44d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab544d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28244d23, 95 },
    { 0x1bd44d23, 90 },
    { 0x53e44d25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d44d2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1244d2d, 61 },
    { 0, 0 },
    { 0x27e44d2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x78944d32, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd44d36, 52 },
    { 0xaef44d36, 75 },
    { 0xa8044d38, 84 },
    { 0x40244d39, 100 },
    { 0x33044d39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x38044d3d, 84 },
    { 0xde144d3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c844d41, 33 },
    { 0x98144d42, 33 },
    { 0x2f344d43, 98 },
    { 0x1b844d43, 98 },
    { 0xf3c44d45, 98 },
    { 0, 0 },
    { 0xd8344d47, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac544d4b, 33 },
    { 0x20744d4c, 90 },
    { 0, 0 },
    { 0x5ed44d4e, 66 },
    { 0xd4c44d4f, 95 },
    { 0x17c44d50, 61 },
    { 0xd1544d51, 98 },
    { 0x15644d52, 29 },
    { 0x2e644d53, 55 },
    { 0x22944d53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12944d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a44d5a, 55 },
    { 0, 0 },
    { 0x51f44d5c, 23 },
    { 0x70444d5d, 95 },
    { 0xd9644d5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f844d68, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad44d70, 55 },
    { 0xcb644d71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb744d79, 76 },
    { 0x44344d7a, 34 },
    { 0x33644d7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee44d7e, 95 },
    { 0x46c44d7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94444d85, 90 },
    { 0x87f44d86, 78 },
    { 0x31844d87, 90 },
    { 0, 0 },
    { 0xb5a44d89, 90 },
    { 0x29044d8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd44d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43444d91, 61 },
    { 0x4e244d92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34044d95, 90 },
    { 0x2f944d96, 52 },
    { 0xee544d96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5644d9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe144da5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc444da9, 75 },
    { 0x65c44daa, 61 },
    { 0xe5644dab, 93 },
    { 0x4cf44dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f344daf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39444db4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f44dbb, 100 },
    { 0, 0 },
    { 0x74644dbd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51144dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d044dcc, 93 },
    { 0, 0 },
    { 0x6ba44dce, 33 },
    { 0xa2944dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59644dd2, 61 },
    { 0xcca44dd2, 90 },
    { 0, 0 },
    { 0xc3b44dd5, 90 },
    { 0x6cd44dd6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c44dd9, 61 },
    { 0, 0 },
    { 0x6ce44ddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6444ddf, 36 },
    { 0, 0 },
    { 0x9d444de1, 98 },
    { 0xea944de2, 17 },
    { 0xf2144de3, 23 },
    { 0x48f44de2, 90 },
    { 0x35044de2, 90 },
    { 0xb3344de2, 95 },
    { 0, 0 },
    { 0xba144de8, 61 },
    { 0xeef44de9, 92 },
    { 0, 0 },
    { 0x1d44deb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc44dee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d644df1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e744df5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6744df8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa144dfb, 55 },
    { 0, 0 },
    { 0x3fb44dfd, 84 },
    { 0, 0 },
    { 0x33344dff, 80 },
    { 0x2c144e00, 90 },
    { 0xdcb44e01, 78 },
    { 0x7ca44e02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb44e05, 23 },
    { 0x20244e05, 90 },
    { 0xb2944e07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c044e0b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b44e0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa244e15, 91 },
    { 0xdc144e15, 23 },
    { 0x6df44e16, 61 },
    { 0xd44e18, 90 },
    { 0xd9144e19, 76 },
    { 0xbfa44e15, 53 },
    { 0, 0 },
    { 0xec544e1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63944e21, 34 },
    { 0x83c44e22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c844e25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c44e29, 55 },
    { 0x77744e29, 33 },
    { 0x98644e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81044e36, 23 },
    { 0x8ea44e37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4344e3b, 77 },
    { 0xfb644e3b, 61 },
    { 0x25144e3d, 34 },
    { 0, 0 },
    { 0x9af44e3f, 66 },
    { 0x7444e40, 23 },
    { 0xd4144e41, 55 },
    { 0xcfb44e40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40144e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b744e4c, 33 },
    { 0x2ac44e4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22944e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c844e58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc044e5b, 36 },
    { 0x83144e5c, 88 },
    { 0x52f44e5d, 61 },
    { 0, 0 },
    { 0xa2344e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0144e63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f844e69, 80 },
    { 0xb144e69, 33 },
    { 0x64a44e6b, 87 },
    { 0x37344e6b, 55 },
    { 0xe5544e6a, 33 },
    { 0x59344e6d, 33 },
    { 0x78b44e6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef644e76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f44e7d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1244e84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5144e88, 49 },
    { 0, 0 },
    { 0x60b44e8a, 52 },
    { 0, 0 },
    { 0xabd44e8c, 100 },
    { 0x8e044e8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac44e91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49344e95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044e98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf44e9b, 91 },
    { 0, 0 },
    { 0xa0544e9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61444ea2, 90 },
    { 0, 0 },
    { 0xcf444ea4, 23 },
    { 0x1f744ea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15844eb0, 98 },
    { 0, 0 },
    { 0xf3944eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da44eb6, 67 },
    { 0x11f44eb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e44ec0, 76 },
    { 0, 0 },
    { 0x22744ec2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c44ec7, 93 },
    { 0, 0 },
    { 0x53844ec9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d044ece, 46 },
    { 0, 0 },
    { 0xf5244ed0, 95 },
    { 0, 0 },
    { 0x29344ed2, 66 },
    { 0xf9e44ed2, 90 },
    { 0xe4b44ed2, 100 },
    { 0x94244ed5, 76 },
    { 0x6744ed5, 53 },
    { 0xcc644ed6, 98 },
    { 0xc5844ed4, 90 },
    { 0xc0a44ed9, 55 },
    { 0x9e044eda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f44edd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc344ee1, 90 },
    { 0xb0344ee2, 90 },
    { 0xa2e44ee2, 90 },
    { 0xdf44ee3, 95 },
    { 0x9fb44ee3, 34 },
    { 0x60944ee6, 84 },
    { 0x94d44ee4, 46 },
    { 0xfd44ee8, 90 },
    { 0x9c144ee9, 43 },
    { 0x5ee44eea, 66 },
    { 0xc7944eea, 81 },
    { 0, 0 },
    { 0x6ec44eed, 95 },
    { 0xf2444eee, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4744ef6, 23 },
    { 0xcaf44ef7, 100 },
    { 0xfd44ef7, 33 },
    { 0, 0 },
    { 0x7c344efa, 52 },
    { 0x42b44efb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c44f00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7944f06, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d244f09, 67 },
    { 0xbc044f0a, 46 },
    { 0x21244f0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef544f0e, 98 },
    { 0xe0644f0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87244f14, 95 },
    { 0x37344f15, 97 },
    { 0x95144f16, 95 },
    { 0, 0 },
    { 0xab44f18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1144f1f, 95 },
    { 0, 0 },
    { 0x8a044f21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f44f24, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9144f29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x83044f2c, 99 },
    { 0xda044f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b244f32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a44f36, 76 },
    { 0x34944f37, 96 },
    { 0xe6644f38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76644f3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af44f43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38944f46, 66 },
    { 0x12c44f47, 99 },
    { 0x21b44f48, 66 },
    { 0x3df44f47, 34 },
    { 0, 0 },
    { 0xc4144f4b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea344f51, 34 },
    { 0, 0 },
    { 0x86844f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe144f57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b44f5a, 96 },
    { 0x93b44f5b, 23 },
    { 0, 0 },
    { 0xcb644f5d, 61 },
    { 0, 0 },
    { 0xd7044f5f, 23 },
    { 0x67344f5f, 76 },
    { 0, 0 },
    { 0x61f44f62, 66 },
    { 0, 0 },
    { 0x8c244f64, 98 },
    { 0xb7844f64, 46 },
    { 0x1c744f66, 23 },
    { 0xecd44f66, 55 },
    { 0x79844f66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec44f6b, 56 },
    { 0xf4844f6c, 90 },
    { 0x62044f6b, 76 },
    { 0, 0 },
    { 0xc4444f6f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x92344f72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f44f75, 23 },
    { 0xe5c44f75, 90 },
    { 0, 0 },
    { 0xaff44f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa44f7c, 90 },
    { 0, 0 },
    { 0x19444f7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45544f82, 61 },
    { 0x33044f83, 90 },
    { 0x30244f84, 66 },
    { 0x41444f83, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a444f89, 84 },
    { 0x99244f8a, 61 },
    { 0x62c44f8b, 55 },
    { 0x68444f8c, 61 },
    { 0xcf644f8d, 91 },
    { 0xc9c44f8c, 98 },
    { 0x44644f8e, 66 },
    { 0xd4544f8e, 95 },
    { 0xa3c44f91, 46 },
    { 0, 0 },
    { 0x18344f93, 87 },
    { 0x32144f94, 76 },
    { 0x4044f95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8944f9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50544f9f, 78 },
    { 0xee444f9f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a44fa3, 90 },
    { 0x7ff44fa4, 33 },
    { 0x12744fa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc444fad, 33 },
    { 0xcc144fae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad44fb2, 98 },
    { 0, 0 },
    { 0x24644fb4, 61 },
    { 0x86044fb4, 95 },
    { 0, 0 },
    { 0x22b44fb7, 52 },
    { 0x1d744fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f44fbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1844fc5, 90 },
    { 0, 0 },
    { 0xb4044fc7, 90 },
    { 0x70b44fc8, 17 },
    { 0x13544fc7, 33 },
    { 0x26344fc9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f444fcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c44fd7, 95 },
    { 0x62f44fd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5544fdd, 33 },
    { 0xf9444fde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb344fe1, 61 },
    { 0, 0 },
    { 0x39544fe3, 90 },
    { 0x11544fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8544fe7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba44fea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf644fed, 65 },
    { 0, 0 },
    { 0x29244fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f44ff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18844ff8, 33 },
    { 0, 0 },
    { 0x9af44ffa, 26 },
    { 0x5fb44ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15745003, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef45006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a54500d, 61 },
    { 0xc124500e, 23 },
    { 0x7514500f, 76 },
    { 0x5344500e, 61 },
    { 0x5845010, 33 },
    { 0x6d94500e, 23 },
    { 0xdc54500d, 55 },
    { 0x1b245014, 75 },
    { 0xd445015, 80 },
    { 0x4e945016, 98 },
    { 0x40545017, 82 },
    { 0x3c45010, 67 },
    { 0x77645019, 34 },
    { 0xfee4501a, 93 },
    { 0xc3b4501b, 95 },
    { 0x6b74501c, 46 },
    { 0x78f4501c, 90 },
    { 0xe974501e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b45022, 98 },
    { 0, 0 },
    { 0xb3e45024, 59 },
    { 0x88445025, 98 },
    { 0, 0 },
    { 0x5845027, 76 },
    { 0x6d245027, 100 },
    { 0x6c345027, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b4502e, 33 },
    { 0xb404502f, 90 },
    { 0x9e745030, 98 },
    { 0x8e345030, 66 },
    { 0, 0 },
    { 0xde445033, 55 },
    { 0, 0 },
    { 0x2e645035, 23 },
    { 0x46045035, 95 },
    { 0xaf745036, 90 },
    { 0, 0 },
    { 0xd2a45039, 90 },
    { 0x5af4503a, 33 },
    { 0xa4545039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb45041, 23 },
    { 0x6e145042, 61 },
    { 0xa2745043, 52 },
    { 0xe2045044, 23 },
    { 0xaa245041, 78 },
    { 0xb0845045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344504a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54504e, 23 },
    { 0, 0 },
    { 0x5d045050, 61 },
    { 0x1745051, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2245054, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f845057, 98 },
    { 0x7b45058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22845060, 55 },
    { 0, 0 },
    { 0xe2345062, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f45066, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e4506a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84506d, 33 },
    { 0, 0 },
    { 0x5114506f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2745073, 78 },
    { 0x7ce45073, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1845079, 95 },
    { 0x6bd4507a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54507f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25845082, 96 },
    { 0xdcc45083, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87145086, 55 },
    { 0xb7345087, 40 },
    { 0x2b645088, 90 },
    { 0, 0 },
    { 0xf264508a, 61 },
    { 0xe024508b, 43 },
    { 0, 0 },
    { 0x5b54508d, 78 },
    { 0x78f4508e, 43 },
    { 0xfcb4508f, 61 },
    { 0xaf45090, 33 },
    { 0, 0 },
    { 0xf9745092, 95 },
    { 0x32845093, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba45099, 98 },
    { 0x5de45099, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d04509d, 91 },
    { 0x80b4509e, 61 },
    { 0, 0 },
    { 0xe13450a0, 52 },
    { 0x1f3450a1, 91 },
    { 0x94450a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46450a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec450ad, 97 },
    { 0x1ed450ad, 23 },
    { 0, 0 },
    { 0x77450b0, 23 },
    { 0xbb6450b1, 52 },
    { 0x15e450b2, 96 },
    { 0x273450b2, 95 },
    { 0x980450b4, 91 },
    { 0x325450b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d450bd, 52 },
    { 0x657450be, 61 },
    { 0x1c9450bf, 76 },
    { 0x4f3450c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b450c3, 68 },
    { 0, 0 },
    { 0x1a5450c5, 33 },
    { 0x852450c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0450ca, 99 },
    { 0, 0 },
    { 0x451450cc, 85 },
    { 0x551450cd, 84 },
    { 0xb0f450ce, 53 },
    { 0xae8450cd, 34 },
    { 0x414450d0, 87 },
    { 0xc32450d1, 98 },
    { 0x3fd450ce, 91 },
    { 0xfa5450d3, 34 },
    { 0, 0 },
    { 0xb90450d5, 66 },
    { 0, 0 },
    { 0x84b450d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233450dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b450e4, 55 },
    { 0x6b4450e5, 68 },
    { 0xd88450e6, 35 },
    { 0x401450e6, 68 },
    { 0xb43450e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7450eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x809450ee, 90 },
    { 0, 0 },
    { 0x2c1450f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe450f6, 23 },
    { 0, 0 },
    { 0xa62450f8, 90 },
    { 0, 0 },
    { 0x8ee450fa, 87 },
    { 0x63a450fb, 33 },
    { 0, 0 },
    { 0xccb450fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d745103, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xca745106, 43 },
    { 0x74c45107, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc4510b, 43 },
    { 0x6664510b, 95 },
    { 0, 0 },
    { 0x2ad4510e, 75 },
    { 0x6954510e, 61 },
    { 0xf5c45110, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3345114, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea45120, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee45125, 33 },
    { 0, 0 },
    { 0x94745127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2834512e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x20445131, 61 },
    { 0x5ae45131, 90 },
    { 0xa7745133, 90 },
    { 0x56345132, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24445138, 23 },
    { 0x1fe45139, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b4513d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4045140, 61 },
    { 0, 0 },
    { 0x5f845142, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e045148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1784514c, 90 },
    { 0, 0 },
    { 0xa6f4514e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a45158, 33 },
    { 0, 0 },
    { 0xa404515a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a4515d, 52 },
    { 0, 0 },
    { 0x2624515f, 55 },
    { 0x3514515f, 95 },
    { 0xa4e45161, 95 },
    { 0x3c945162, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d45166, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa454516d, 98 },
    { 0xdd24516d, 61 },
    { 0x75b4516f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a945172, 98 },
    { 0x34045173, 93 },
    { 0x22545173, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d45183, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd45188, 55 },
    { 0x59c45189, 43 },
    { 0x66c4518a, 99 },
    { 0x5594518b, 90 },
    { 0x41c4518b, 43 },
    { 0x3384518a, 91 },
    { 0xe594518e, 61 },
    { 0, 0 },
    { 0x47445190, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc45193, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea45198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b4519d, 95 },
    { 0x7234519d, 90 },
    { 0x73b4519d, 55 },
    { 0, 0 },
    { 0xc77451a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x373451a4, 76 },
    { 0x75d451a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667451aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921451b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa451bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385451c6, 98 },
    { 0, 0 },
    { 0x8e8451c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7451cc, 77 },
    { 0, 0 },
    { 0x3f7451ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d451d3, 98 },
    { 0xfb5451d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc451d8, 88 },
    { 0x5d451d9, 33 },
    { 0x3b2451d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f451df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2451ea, 90 },
    { 0x6fd451ea, 90 },
    { 0x346451ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x499451ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8451f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555451f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8451fc, 33 },
    { 0x1ac451fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d45200, 61 },
    { 0, 0 },
    { 0x8c745202, 33 },
    { 0xdc645203, 55 },
    { 0xb2d45204, 92 },
    { 0xff845205, 85 },
    { 0x77445206, 34 },
    { 0x5ff45206, 76 },
    { 0x55245207, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b4520c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a45215, 95 },
    { 0x84445216, 98 },
    { 0xba845217, 23 },
    { 0, 0 },
    { 0xdec45219, 90 },
    { 0, 0 },
    { 0x1d54521b, 88 },
    { 0x5d14521c, 46 },
    { 0, 0 },
    { 0xe214521e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x47445221, 84 },
    { 0x4d345222, 93 },
    { 0x97d45222, 61 },
    { 0x66845224, 46 },
    { 0xb4245225, 46 },
    { 0, 0 },
    { 0xe3045227, 87 },
    { 0x99345227, 98 },
    { 0, 0 },
    { 0x53e4522a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8864522f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a45233, 98 },
    { 0x3a545233, 90 },
    { 0x54745234, 90 },
    { 0x9e145236, 91 },
    { 0xa8545237, 33 },
    { 0x3145237, 27 },
    { 0x7d745234, 98 },
    { 0, 0 },
    { 0xfce4523b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f54523f, 33 },
    { 0x21c45240, 55 },
    { 0x30845241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a545246, 66 },
    { 0x33845246, 34 },
    { 0, 0 },
    { 0xa0445249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b4524d, 85 },
    { 0, 0 },
    { 0x72c4524f, 84 },
    { 0, 0 },
    { 0xcc845251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c45258, 66 },
    { 0x2c245258, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc45262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4945269, 61 },
    { 0x2704526a, 61 },
    { 0x1cd45269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32045274, 46 },
    { 0, 0 },
    { 0xa7b45276, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf84527b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b145282, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90045285, 98 },
    { 0x62045285, 61 },
    { 0, 0 },
    { 0x6845288, 33 },
    { 0x21d45289, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f4528c, 68 },
    { 0x96f4528d, 61 },
    { 0xcf34528c, 90 },
    { 0, 0 },
    { 0x58845290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88452a1, 43 },
    { 0xf2452a2, 95 },
    { 0x2e4452a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860452ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf452b2, 61 },
    { 0, 0 },
    { 0x105452b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02452b7, 78 },
    { 0xb58452b8, 95 },
    { 0xcf1452b8, 66 },
    { 0xbc8452ba, 23 },
    { 0xec0452bb, 55 },
    { 0x119452bc, 87 },
    { 0xa0e452bd, 95 },
    { 0xdf2452bb, 73 },
    { 0xa2e452bf, 33 },
    { 0xda8452c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5452c3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a452c6, 90 },
    { 0x113452c6, 87 },
    { 0, 0 },
    { 0x78f452c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798452cd, 52 },
    { 0xd8d452ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc452d6, 55 },
    { 0, 0 },
    { 0xfc6452d8, 23 },
    { 0xf6e452d9, 90 },
    { 0x94b452d9, 98 },
    { 0x788452d9, 98 },
    { 0x53e452d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb452df, 23 },
    { 0xb43452e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf452e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70452ea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df452ed, 90 },
    { 0x1f452ee, 90 },
    { 0xdfb452ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x400452f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44452ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b445306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c44530a, 98 },
    { 0x9c04530b, 23 },
    { 0, 0 },
    { 0x9804530d, 17 },
    { 0x75a4530e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e45311, 68 },
    { 0x2eb45311, 87 },
    { 0xad45311, 17 },
    { 0xc0e45314, 61 },
    { 0xc1e45313, 17 },
    { 0, 0 },
    { 0x35145317, 95 },
    { 0xd6945318, 23 },
    { 0x46b45319, 95 },
    { 0, 0 },
    { 0x4934531b, 46 },
    { 0xbe04531c, 61 },
    { 0xda14531d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d045321, 33 },
    { 0, 0 },
    { 0x3d945323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a45329, 67 },
    { 0, 0 },
    { 0x8074532b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3745331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f945334, 34 },
    { 0, 0 },
    { 0xf1f45336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b4533d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6745341, 66 },
    { 0x92545341, 68 },
    { 0, 0 },
    { 0x37e45344, 95 },
    { 0, 0 },
    { 0x3ce45346, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd964534a, 33 },
    { 0x67c4534a, 23 },
    { 0x934534c, 95 },
    { 0x3374534b, 75 },
    { 0x2854534e, 98 },
    { 0, 0 },
    { 0x2a45350, 93 },
    { 0xc7245351, 55 },
    { 0x8ce45352, 76 },
    { 0x2e545352, 76 },
    { 0x87145354, 33 },
    { 0xd4e45353, 84 },
    { 0, 0 },
    { 0x3b945357, 81 },
    { 0xc9f45358, 33 },
    { 0x1b745358, 61 },
    { 0, 0 },
    { 0x7774535b, 33 },
    { 0x89a4535c, 95 },
    { 0xe24535d, 61 },
    { 0x2cf4535d, 95 },
    { 0x2cb4535c, 90 },
    { 0, 0 },
    { 0x9fc45361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b845364, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f45367, 61 },
    { 0xb6445368, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3234536d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a45370, 33 },
    { 0xba545371, 90 },
    { 0x24145371, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73545375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee4537b, 76 },
    { 0, 0 },
    { 0x6694537d, 61 },
    { 0x3ac4537e, 33 },
    { 0x8a04537d, 61 },
    { 0xa604537f, 90 },
    { 0x2df45380, 90 },
    { 0x32b4537d, 95 },
    { 0xd6d45383, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2445388, 76 },
    { 0x2c845389, 37 },
    { 0x8054538a, 23 },
    { 0, 0 },
    { 0x31a4538c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc545394, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e4539a, 90 },
    { 0xcb54539b, 65 },
    { 0xb694539c, 91 },
    { 0xb4539b, 100 },
    { 0, 0 },
    { 0x4214539f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb453a3, 85 },
    { 0, 0 },
    { 0x4c9453a5, 34 },
    { 0, 0 },
    { 0x4d7453a7, 78 },
    { 0x3ae453a8, 90 },
    { 0x280453a9, 43 },
    { 0xee9453a7, 95 },
    { 0xe2c453ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c453b1, 33 },
    { 0, 0 },
    { 0xc10453b3, 98 },
    { 0, 0 },
    { 0x48453b5, 61 },
    { 0x842453b6, 55 },
    { 0, 0 },
    { 0xc65453b8, 90 },
    { 0, 0 },
    { 0x1f8453ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b453be, 95 },
    { 0x49453bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1453c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9453cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30453cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa453d7, 52 },
    { 0x6a9453d8, 95 },
    { 0xa24453d9, 66 },
    { 0, 0 },
    { 0xf9f453db, 33 },
    { 0xd6b453db, 48 },
    { 0x206453dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x784453e0, 55 },
    { 0xb59453e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec453e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768453e8, 93 },
    { 0x796453e9, 61 },
    { 0x211453ea, 66 },
    { 0x432453eb, 90 },
    { 0xe24453ec, 66 },
    { 0xf86453ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09453f1, 93 },
    { 0xce2453f2, 97 },
    { 0xc45453f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf453f7, 93 },
    { 0xc3d453f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff45400, 55 },
    { 0, 0 },
    { 0x66645402, 61 },
    { 0x46045403, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace45408, 98 },
    { 0xa3a45408, 43 },
    { 0xd274540a, 93 },
    { 0, 0 },
    { 0x3a24540c, 85 },
    { 0xe0f4540d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89745410, 75 },
    { 0x81145410, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x85945414, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a45419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a4541c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99845420, 34 },
    { 0xd5845421, 98 },
    { 0x41645422, 23 },
    { 0x72345423, 95 },
    { 0x15445424, 95 },
    { 0x68e45424, 43 },
    { 0x2f645426, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x88345429, 91 },
    { 0, 0 },
    { 0x1c74542b, 98 },
    { 0x8b24542b, 33 },
    { 0x64c4542d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd45430, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda045438, 91 },
    { 0x9945439, 34 },
    { 0xed24543a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c4543d, 95 },
    { 0xbc84543e, 52 },
    { 0x9124543f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3145442, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae4544b, 46 },
    { 0x2bf4544c, 94 },
    { 0, 0 },
    { 0x8cf4544e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25345454, 95 },
    { 0xfb45455, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c845458, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf4545c, 91 },
    { 0, 0 },
    { 0x7604545e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d545461, 85 },
    { 0x68945462, 88 },
    { 0x76745461, 33 },
    { 0, 0 },
    { 0x5ff45465, 90 },
    { 0x33f45465, 43 },
    { 0xee945465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4546a, 87 },
    { 0, 0 },
    { 0xb754546c, 77 },
    { 0x8cc4546c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a45470, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x23645473, 76 },
    { 0, 0 },
    { 0xcac45475, 43 },
    { 0xc8745476, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa045479, 87 },
    { 0x7d45479, 33 },
    { 0x80e4547b, 55 },
    { 0xa5845479, 90 },
    { 0x48f4547d, 61 },
    { 0x6944547e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b245483, 81 },
    { 0x55745484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3845488, 66 },
    { 0x18f45488, 34 },
    { 0, 0 },
    { 0x2dd4548b, 91 },
    { 0, 0 },
    { 0x3b24548d, 33 },
    { 0x65b4548e, 95 },
    { 0x3314548d, 98 },
    { 0xff345490, 95 },
    { 0, 0 },
    { 0xc8d45492, 17 },
    { 0, 0 },
    { 0xc1445494, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab45497, 55 },
    { 0x51045497, 33 },
    { 0x42a45499, 93 },
    { 0x5494549a, 90 },
    { 0x4bf4549b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb414549f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8454a7, 78 },
    { 0x178454a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d454ae, 84 },
    { 0xfb8454af, 78 },
    { 0xb46454af, 61 },
    { 0xfa6454b1, 61 },
    { 0x940454b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2454b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e454b8, 43 },
    { 0x9ff454b8, 61 },
    { 0, 0 },
    { 0xe01454bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54454c0, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43454c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353454cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4454ce, 61 },
    { 0xe72454cf, 33 },
    { 0, 0 },
    { 0x61e454d1, 23 },
    { 0x861454d2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d454d6, 99 },
    { 0, 0 },
    { 0x5ae454d8, 23 },
    { 0x45d454d8, 77 },
    { 0x74c454d8, 33 },
    { 0xc3d454db, 23 },
    { 0x29b454d8, 34 },
    { 0x818454dd, 90 },
    { 0x445454de, 61 },
    { 0xdf7454d8, 95 },
    { 0x5b8454da, 66 },
    { 0, 0 },
    { 0x230454e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a454e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96454f4, 95 },
    { 0x5d4454f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84454fb, 98 },
    { 0x91e454fc, 100 },
    { 0xffe454fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a345500, 61 },
    { 0x6d245501, 33 },
    { 0x11545501, 34 },
    { 0, 0 },
    { 0x5a445504, 100 },
    { 0xe3545504, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074550e, 66 },
    { 0, 0 },
    { 0x78e45510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd045513, 69 },
    { 0xef045514, 23 },
    { 0x5b545515, 35 },
    { 0, 0 },
    { 0xfc245517, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a74551b, 56 },
    { 0xcab4551c, 95 },
    { 0x7524551d, 68 },
    { 0x9c64551e, 85 },
    { 0xed24551e, 61 },
    { 0xd424551e, 90 },
    { 0x7e54551e, 87 },
    { 0xe1045522, 95 },
    { 0x4df45523, 33 },
    { 0x75545523, 33 },
    { 0xd3c45525, 23 },
    { 0x46245523, 33 },
    { 0, 0 },
    { 0x94f45528, 23 },
    { 0, 0 },
    { 0xaf4552a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6224552e, 90 },
    { 0xa924552f, 91 },
    { 0x27045530, 90 },
    { 0xba94552f, 90 },
    { 0xe6045532, 62 },
    { 0x1f945533, 61 },
    { 0, 0 },
    { 0x3e245535, 78 },
    { 0x9d745536, 33 },
    { 0x33545536, 34 },
    { 0, 0 },
    { 0x7ae45539, 95 },
    { 0x7c74553a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a45540, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd45543, 76 },
    { 0xca845544, 98 },
    { 0x1b45544, 52 },
    { 0xd2345544, 76 },
    { 0x53545546, 91 },
    { 0xa1545547, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe544554b, 23 },
    { 0, 0 },
    { 0x3934554d, 61 },
    { 0, 0 },
    { 0x26b4554f, 87 },
    { 0x6924554f, 98 },
    { 0x5bf45551, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xded45554, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c45557, 55 },
    { 0xd3345558, 90 },
    { 0xbf45558, 88 },
    { 0, 0 },
    { 0x77a4555b, 95 },
    { 0xc454555c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4555f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc45565, 99 },
    { 0x8b445566, 97 },
    { 0, 0 },
    { 0xdf345568, 95 },
    { 0xcc545568, 95 },
    { 0x63b4556a, 33 },
    { 0x5c74556a, 76 },
    { 0x1624556b, 100 },
    { 0xb064556a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78445578, 43 },
    { 0x75845579, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04557d, 76 },
    { 0x19b4557e, 80 },
    { 0x1674557f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb45582, 76 },
    { 0, 0 },
    { 0xe8045584, 33 },
    { 0, 0 },
    { 0xdc545586, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6214558d, 33 },
    { 0, 0 },
    { 0xd024558f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1045592, 48 },
    { 0x6fe45593, 83 },
    { 0x73a45594, 99 },
    { 0x8c145595, 90 },
    { 0x5c345596, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55945599, 23 },
    { 0x4634559a, 23 },
    { 0x7044559a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4559e, 23 },
    { 0xb824559f, 33 },
    { 0, 0 },
    { 0x9ed455a1, 55 },
    { 0x9b6455a2, 60 },
    { 0xe0f455a2, 67 },
    { 0xefa455a4, 90 },
    { 0x7be455a2, 99 },
    { 0x8f8455a6, 33 },
    { 0x829455a6, 98 },
    { 0x52455a8, 98 },
    { 0x6f5455a8, 98 },
    { 0xe14455a7, 33 },
    { 0xd84455a7, 33 },
    { 0xe4b455ac, 55 },
    { 0x561455ad, 53 },
    { 0x753455ae, 55 },
    { 0x452455ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d455b4, 68 },
    { 0x7fb455b5, 61 },
    { 0x67f455b4, 34 },
    { 0, 0 },
    { 0x335455b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994455bd, 61 },
    { 0, 0 },
    { 0x8455bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f455c5, 91 },
    { 0, 0 },
    { 0xe5d455c7, 46 },
    { 0x342455c8, 33 },
    { 0x793455c8, 43 },
    { 0, 0 },
    { 0x4bb455cb, 95 },
    { 0x20c455cc, 98 },
    { 0xe09455cd, 91 },
    { 0x4f2455ce, 95 },
    { 0x6fe455cf, 61 },
    { 0x417455cd, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe455d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe455dc, 43 },
    { 0, 0 },
    { 0x27455de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f455e2, 90 },
    { 0, 0 },
    { 0x667455e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26455ef, 58 },
    { 0x29455ef, 76 },
    { 0, 0 },
    { 0x187455f2, 46 },
    { 0x784455f3, 84 },
    { 0x6ec455f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11455f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b455fb, 88 },
    { 0xaf7455fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd945603, 43 },
    { 0, 0 },
    { 0x88545605, 91 },
    { 0, 0 },
    { 0xef445607, 43 },
    { 0x63545608, 88 },
    { 0xed345607, 51 },
    { 0xf4c4560a, 34 },
    { 0x5e74560b, 93 },
    { 0, 0 },
    { 0x714560d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b45612, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87545617, 76 },
    { 0xf1f45618, 61 },
    { 0xe5545619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f04561d, 61 },
    { 0xfa14561e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c45621, 33 },
    { 0xe0645622, 98 },
    { 0, 0 },
    { 0x43345624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51645627, 23 },
    { 0, 0 },
    { 0x99745629, 61 },
    { 0x77c4562a, 23 },
    { 0xc4a4562b, 23 },
    { 0x1945629, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x114562f, 33 },
    { 0xdcf4562f, 91 },
    { 0x44745631, 55 },
    { 0x89645630, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d45635, 98 },
    { 0, 0 },
    { 0x6ce45637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b4563d, 100 },
    { 0x5ba4563e, 43 },
    { 0x3bd4563e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0645648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda64564f, 52 },
    { 0xc9b4564f, 33 },
    { 0xd745651, 3 },
    { 0x2a545652, 23 },
    { 0x32845652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5245656, 61 },
    { 0, 0 },
    { 0x74745658, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7324565b, 33 },
    { 0, 0 },
    { 0x5ba4565d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc145660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc745665, 88 },
    { 0xaa145665, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb444566b, 33 },
    { 0x4904566c, 76 },
    { 0, 0 },
    { 0x5334566e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92745671, 61 },
    { 0xe4545672, 90 },
    { 0, 0 },
    { 0x20845674, 33 },
    { 0, 0 },
    { 0x80545676, 95 },
    { 0xac145676, 99 },
    { 0, 0 },
    { 0xd8345679, 95 },
    { 0xb9345679, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e645684, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb134568a, 43 },
    { 0x15f4568b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa294568f, 98 },
    { 0, 0 },
    { 0xc4845691, 66 },
    { 0x4e945692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd545695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc534569a, 23 },
    { 0, 0 },
    { 0x5234569c, 33 },
    { 0xa304569d, 23 },
    { 0xdeb4569c, 90 },
    { 0x23c4569d, 95 },
    { 0xfbf4569c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80456a7, 77 },
    { 0, 0 },
    { 0x5ed456a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x767456ac, 94 },
    { 0xc5f456ad, 34 },
    { 0x953456ad, 95 },
    { 0xb9d456ad, 98 },
    { 0xb01456b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x129456b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b456b8, 66 },
    { 0xf1a456b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83456bc, 61 },
    { 0x55d456bd, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4456c2, 55 },
    { 0, 0 },
    { 0xd58456c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77456c7, 34 },
    { 0x30a456c8, 91 },
    { 0, 0 },
    { 0xb98456ca, 76 },
    { 0x679456cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be456d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01456d4, 33 },
    { 0x881456d5, 46 },
    { 0x267456d5, 95 },
    { 0x4ff456d7, 93 },
    { 0x447456d8, 61 },
    { 0xa83456d8, 46 },
    { 0xd22456d7, 97 },
    { 0xdc4456db, 93 },
    { 0, 0 },
    { 0x587456dd, 23 },
    { 0, 0 },
    { 0xfd6456df, 76 },
    { 0x4b9456e0, 61 },
    { 0x2b5456e0, 76 },
    { 0x75d456e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x181456e5, 33 },
    { 0xbf7456e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c456eb, 99 },
    { 0, 0 },
    { 0x7e7456ed, 76 },
    { 0, 0 },
    { 0x1b6456ef, 33 },
    { 0x668456ef, 98 },
    { 0, 0 },
    { 0x179456f2, 52 },
    { 0, 0 },
    { 0xdb456f4, 33 },
    { 0x371456f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x304456f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x275456fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14456ff, 33 },
    { 0, 0 },
    { 0x90545701, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x85045704, 100 },
    { 0xe45705, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x46045708, 36 },
    { 0x66545708, 53 },
    { 0xf6f4570a, 75 },
    { 0x2404570a, 23 },
    { 0xf564570c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb45712, 61 },
    { 0xe2345713, 78 },
    { 0, 0 },
    { 0x9ed45715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e45718, 33 },
    { 0, 0 },
    { 0x7594571a, 95 },
    { 0x4bc4571a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3104571e, 80 },
    { 0, 0 },
    { 0x10b45720, 78 },
    { 0xc2145721, 98 },
    { 0, 0 },
    { 0xf2745723, 78 },
    { 0x5c745723, 23 },
    { 0xb6945724, 33 },
    { 0, 0 },
    { 0x1d245727, 90 },
    { 0, 0 },
    { 0x3145729, 87 },
    { 0xe7745729, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a45730, 85 },
    { 0x97b45731, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8045736, 93 },
    { 0, 0 },
    { 0x43645738, 98 },
    { 0x59f45739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b4573c, 61 },
    { 0, 0 },
    { 0x10e4573e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36545743, 91 },
    { 0x6a945743, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca45747, 61 },
    { 0x4af45748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13045753, 68 },
    { 0x93045754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd104575a, 61 },
    { 0x1c34575b, 95 },
    { 0xe604575c, 78 },
    { 0, 0 },
    { 0xcc64575e, 67 },
    { 0, 0 },
    { 0xcbb45760, 98 },
    { 0x2b345760, 34 },
    { 0x5ed45761, 98 },
    { 0xeb745763, 98 },
    { 0xe3045763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d45767, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e04576a, 85 },
    { 0xe3d4576b, 23 },
    { 0, 0 },
    { 0xab24576d, 33 },
    { 0x8f54576e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e45772, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd45776, 80 },
    { 0, 0 },
    { 0x99145778, 98 },
    { 0xcf145779, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394577c, 95 },
    { 0xb094577d, 61 },
    { 0x1b04577e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3045783, 77 },
    { 0, 0 },
    { 0xd0145785, 34 },
    { 0, 0 },
    { 0x41045787, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba345796, 43 },
    { 0x8dd45796, 33 },
    { 0x48c45798, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56457a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952457ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb457af, 43 },
    { 0xdde457b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da457b5, 99 },
    { 0, 0 },
    { 0x472457b7, 84 },
    { 0x190457b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28457bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e457c0, 87 },
    { 0x11a457c1, 34 },
    { 0x22c457c2, 90 },
    { 0xe5b457c3, 67 },
    { 0x9a6457c4, 55 },
    { 0, 0 },
    { 0x38b457c6, 97 },
    { 0x6ea457c7, 33 },
    { 0x4cc457c8, 66 },
    { 0, 0 },
    { 0x134457ca, 34 },
    { 0xedc457cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b457d1, 98 },
    { 0xaa9457d2, 61 },
    { 0x429457d2, 87 },
    { 0, 0 },
    { 0x36d457d5, 33 },
    { 0x474457d6, 93 },
    { 0x473457d5, 99 },
    { 0x1f1457d8, 55 },
    { 0x483457d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297457de, 33 },
    { 0, 0 },
    { 0x7e3457e0, 90 },
    { 0xd18457e1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e457ed, 61 },
    { 0xbde457ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87945803, 55 },
    { 0x96045804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd245807, 98 },
    { 0x5c845807, 90 },
    { 0x12945809, 88 },
    { 0xe2a4580a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe14580d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c045812, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39945817, 90 },
    { 0x46845818, 66 },
    { 0xd4945817, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d4581d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b45824, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9145827, 35 },
    { 0x80845828, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb304582b, 67 },
    { 0, 0 },
    { 0x4cf4582d, 23 },
    { 0x2e04582e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc045833, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c54583d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa45842, 34 },
    { 0, 0 },
    { 0x2e445844, 65 },
    { 0, 0 },
    { 0x5d145846, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f945849, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd4584e, 61 },
    { 0xdbd4584e, 61 },
    { 0, 0 },
    { 0xf2a45851, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d45856, 91 },
    { 0xafb45856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f4585b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa784585e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52745864, 100 },
    { 0, 0 },
    { 0x55c45866, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae4586f, 23 },
    { 0x6a745870, 34 },
    { 0, 0 },
    { 0x5eb45872, 23 },
    { 0xcbd45873, 61 },
    { 0xffa45873, 85 },
    { 0xece45875, 90 },
    { 0x2df45875, 61 },
    { 0xa7045873, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca645880, 59 },
    { 0, 0 },
    { 0x8dd45882, 61 },
    { 0xc8845882, 93 },
    { 0xbc845884, 93 },
    { 0, 0 },
    { 0x16845886, 43 },
    { 0xee045887, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a45894, 33 },
    { 0x39145895, 95 },
    { 0x2ae45895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a4589a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd458a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d458a3, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb458aa, 76 },
    { 0xcbc458ab, 85 },
    { 0x4eb458ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9458b1, 83 },
    { 0x8f1458b2, 90 },
    { 0x9a2458b3, 90 },
    { 0x547458b4, 55 },
    { 0x3dd458b3, 33 },
    { 0x16f458b6, 83 },
    { 0x1e5458b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4458bb, 92 },
    { 0x14f458bc, 46 },
    { 0xbcc458bd, 95 },
    { 0x2e9458bb, 65 },
    { 0xbe458bb, 55 },
    { 0xa94458c0, 55 },
    { 0xdbd458c1, 33 },
    { 0xb20458c2, 45 },
    { 0x1c5458c3, 67 },
    { 0, 0 },
    { 0x9aa458c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f458c8, 61 },
    { 0, 0 },
    { 0x7b4458ca, 34 },
    { 0, 0 },
    { 0x937458cc, 33 },
    { 0xeef458cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1458d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6458d5, 59 },
    { 0xd9c458d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e458db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc458e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e458e6, 34 },
    { 0x10a458e7, 61 },
    { 0x3c1458e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1458eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae458ee, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46458f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93458f5, 81 },
    { 0, 0 },
    { 0x116458f7, 68 },
    { 0x80f458f8, 4 },
    { 0x304458f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa45900, 76 },
    { 0, 0 },
    { 0x8245902, 91 },
    { 0x9d745903, 87 },
    { 0xfee45904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f45907, 33 },
    { 0, 0 },
    { 0x2b145909, 100 },
    { 0x8e04590a, 61 },
    { 0x2f4590a, 95 },
    { 0, 0 },
    { 0xfcd4590d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea045913, 90 },
    { 0x66845914, 23 },
    { 0xd6245915, 99 },
    { 0x14a45916, 33 },
    { 0x4d345915, 23 },
    { 0x84e45917, 23 },
    { 0x53a45917, 34 },
    { 0x18645919, 76 },
    { 0, 0 },
    { 0x5854591c, 61 },
    { 0x66a4591d, 17 },
    { 0x9dd4591d, 90 },
    { 0x3da4591f, 90 },
    { 0x97145920, 43 },
    { 0xbed45920, 90 },
    { 0xe8a45922, 100 },
    { 0x17145923, 98 },
    { 0xce54591f, 93 },
    { 0x43545921, 55 },
    { 0x9e04591d, 46 },
    { 0x40c4591f, 55 },
    { 0x57545928, 43 },
    { 0, 0 },
    { 0x43b4592a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe4592d, 43 },
    { 0xc714592e, 34 },
    { 0x6cd4592f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad445935, 90 },
    { 0x3db45936, 23 },
    { 0xeca45937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c44593e, 76 },
    { 0x9cd4593f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c45943, 46 },
    { 0x5ff45944, 61 },
    { 0xec245945, 87 },
    { 0x93845946, 98 },
    { 0xb1b45947, 98 },
    { 0, 0 },
    { 0xd8345949, 33 },
    { 0, 0 },
    { 0x8854594b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2424594e, 80 },
    { 0x69d4594f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f45952, 90 },
    { 0, 0 },
    { 0xb8545954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c45957, 95 },
    { 0x5fc45958, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd45961, 76 },
    { 0, 0 },
    { 0xacb45963, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8904596d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b945973, 78 },
    { 0x56245974, 64 },
    { 0, 0 },
    { 0x8a445976, 95 },
    { 0x93645977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3274597a, 77 },
    { 0, 0 },
    { 0x8114597c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5445987, 100 },
    { 0x24145988, 100 },
    { 0x2e545987, 65 },
    { 0, 0 },
    { 0xa424598b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c4598f, 90 },
    { 0xf7845990, 88 },
    { 0x6b045990, 60 },
    { 0x49045992, 61 },
    { 0xba745991, 99 },
    { 0, 0 },
    { 0xa6245995, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1745999, 77 },
    { 0x6e04599a, 61 },
    { 0, 0 },
    { 0x7fe4599c, 98 },
    { 0xa054599d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae459a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb459a3, 76 },
    { 0, 0 },
    { 0x98e459a5, 90 },
    { 0x62e459a5, 97 },
    { 0xaff459a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758459b1, 40 },
    { 0xbe2459b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4459b6, 43 },
    { 0, 0 },
    { 0xf56459b8, 90 },
    { 0, 0 },
    { 0xaaf459ba, 78 },
    { 0, 0 },
    { 0xaa5459bc, 90 },
    { 0, 0 },
    { 0x918459be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d459c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa459c4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f459c7, 17 },
    { 0x542459c7, 43 },
    { 0x7a8459c9, 43 },
    { 0xfd0459c7, 75 },
    { 0x988459cb, 33 },
    { 0x321459cc, 90 },
    { 0, 0 },
    { 0xf33459ce, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248459d4, 93 },
    { 0, 0 },
    { 0x44f459d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab459db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b459e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972459ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc459ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8459f4, 61 },
    { 0xe54459f5, 84 },
    { 0x69a459f6, 95 },
    { 0, 0 },
    { 0xe0c459f8, 76 },
    { 0x82a459f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657459fd, 61 },
    { 0, 0 },
    { 0x635459ff, 90 },
    { 0xd5f45a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b45a07, 55 },
    { 0x62945a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc845a0f, 91 },
    { 0x5d645a0f, 33 },
    { 0xe6745a0f, 49 },
    { 0x6a645a12, 43 },
    { 0xb6745a13, 95 },
    { 0xb8d45a14, 33 },
    { 0x26045a14, 76 },
    { 0x97945a15, 95 },
    { 0x78d45a12, 90 },
    { 0xabe45a18, 43 },
    { 0xe6245a19, 90 },
    { 0, 0 },
    { 0x33045a1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77745a1f, 61 },
    { 0xd0e45a20, 95 },
    { 0xeea45a21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b45a25, 95 },
    { 0, 0 },
    { 0xb7f45a27, 23 },
    { 0x6cb45a28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8245a2b, 17 },
    { 0, 0 },
    { 0x94e45a2d, 90 },
    { 0x95f45a2e, 100 },
    { 0xff45a2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb945a33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f445a37, 33 },
    { 0xb0545a38, 24 },
    { 0x7c45a38, 75 },
    { 0x97a45a3a, 100 },
    { 0x11a45a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed945a41, 23 },
    { 0xc2745a41, 77 },
    { 0x61a45a42, 95 },
    { 0xcd145a41, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25345a49, 33 },
    { 0x4e945a49, 93 },
    { 0, 0 },
    { 0x9fa45a4c, 98 },
    { 0xc2245a4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d45a50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc45a58, 93 },
    { 0, 0 },
    { 0x82d45a5a, 76 },
    { 0x4a245a5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd145a5f, 43 },
    { 0x69845a5f, 76 },
    { 0x6d745a60, 43 },
    { 0x52245a60, 23 },
    { 0x92f45a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47145a66, 95 },
    { 0xa045a67, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c845a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be45a79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc45a7f, 87 },
    { 0x5b145a80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b45a83, 84 },
    { 0, 0 },
    { 0x74145a85, 61 },
    { 0x9045a85, 33 },
    { 0x48845a87, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x69045a8a, 76 },
    { 0x91845a8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c45a8f, 90 },
    { 0, 0 },
    { 0x33945a91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c045a95, 33 },
    { 0x25c45a95, 90 },
    { 0x19745a97, 95 },
    { 0xa7445a98, 100 },
    { 0x11245a98, 98 },
    { 0x68045a99, 43 },
    { 0, 0 },
    { 0x80b45a9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e45a9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11745aa5, 98 },
    { 0x88545aa6, 95 },
    { 0xde745aa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2345aaa, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c45aae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39245ab5, 100 },
    { 0x12245ab5, 90 },
    { 0, 0 },
    { 0x9f45ab8, 55 },
    { 0x4c045ab8, 98 },
    { 0, 0 },
    { 0x52345abb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84345abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d45ac5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f245ac8, 33 },
    { 0x52a45ac8, 91 },
    { 0x9d245aca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce45ad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4645ad4, 98 },
    { 0xe5845ad5, 23 },
    { 0xca045ad6, 61 },
    { 0, 0 },
    { 0xe1145ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf45ae1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb145ae4, 88 },
    { 0, 0 },
    { 0xd0d45ae6, 98 },
    { 0x6c345ae7, 90 },
    { 0x4a845ae7, 23 },
    { 0x4a945ae6, 95 },
    { 0x67945aea, 46 },
    { 0, 0 },
    { 0xf0f45aec, 55 },
    { 0xfb545aed, 90 },
    { 0xf2b45aee, 33 },
    { 0, 0 },
    { 0x13945af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac945af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c45af7, 65 },
    { 0x1a845af7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df45b04, 61 },
    { 0x96f45b04, 90 },
    { 0, 0 },
    { 0x49645b07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x48545b0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b545b0d, 55 },
    { 0x48145b0e, 90 },
    { 0x52b45b0f, 43 },
    { 0x58645b0f, 23 },
    { 0x8c245b11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b45b14, 95 },
    { 0, 0 },
    { 0xedd45b16, 90 },
    { 0, 0 },
    { 0x3a545b18, 99 },
    { 0x5b045b18, 52 },
    { 0x30b45b1a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c45b1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a145b23, 34 },
    { 0x94645b24, 55 },
    { 0, 0 },
    { 0xe1545b26, 98 },
    { 0x21745b27, 38 },
    { 0, 0 },
    { 0x13445b29, 90 },
    { 0, 0 },
    { 0xcc645b2b, 98 },
    { 0x5d545b2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b45b2f, 33 },
    { 0, 0 },
    { 0x3b745b31, 61 },
    { 0, 0 },
    { 0x4af45b33, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf45b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0145b3f, 78 },
    { 0x8de45b3f, 91 },
    { 0x86f45b41, 98 },
    { 0xeee45b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2445b47, 93 },
    { 0, 0 },
    { 0x58645b49, 43 },
    { 0x3d945b4a, 66 },
    { 0x55b45b4a, 55 },
    { 0x4cc45b4a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9645b52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e745b55, 99 },
    { 0xcaf45b55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f745b5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18045b61, 90 },
    { 0x71345b61, 90 },
    { 0x91a45b62, 67 },
    { 0x1bc45b62, 33 },
    { 0x86f45b65, 23 },
    { 0x51b45b66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d45b6a, 95 },
    { 0xbe045b6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b145b6e, 33 },
    { 0x33b45b6f, 90 },
    { 0x54445b6f, 66 },
    { 0, 0 },
    { 0x5e845b72, 23 },
    { 0x20545b73, 90 },
    { 0, 0 },
    { 0xc3b45b75, 23 },
    { 0xa5e45b76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e45b7f, 61 },
    { 0x65c45b80, 33 },
    { 0xc6745b81, 98 },
    { 0, 0 },
    { 0x1c345b83, 95 },
    { 0xe8745b84, 90 },
    { 0x7a045b83, 90 },
    { 0x9f445b86, 33 },
    { 0x86645b87, 98 },
    { 0xdff45b88, 95 },
    { 0xf4745b85, 33 },
    { 0x34d45b89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d345b8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d45b94, 84 },
    { 0, 0 },
    { 0xc6645b96, 91 },
    { 0xfca45b97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e45ba0, 75 },
    { 0x47b45ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40945ba5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a45ba9, 90 },
    { 0, 0 },
    { 0x4b445bab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xed745bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a645bb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c45bb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2045bbb, 33 },
    { 0xd4145bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38845bc1, 85 },
    { 0, 0 },
    { 0x69b45bc3, 33 },
    { 0, 0 },
    { 0x8ae45bc5, 76 },
    { 0x56d45bc6, 61 },
    { 0x3fa45bc7, 95 },
    { 0, 0 },
    { 0x82a45bc9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b45bcc, 68 },
    { 0xe4145bcd, 33 },
    { 0xca245bce, 98 },
    { 0xff45bcf, 17 },
    { 0x46045bd0, 23 },
    { 0, 0 },
    { 0x60a45bd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a145bd8, 61 },
    { 0x6c845bd9, 33 },
    { 0xdf945bd9, 68 },
    { 0xdf645bd8, 95 },
    { 0xcd745bda, 98 },
    { 0x1df45bdd, 22 },
    { 0x3eb45bdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75945be3, 95 },
    { 0xc0745be4, 98 },
    { 0xbcd45be4, 61 },
    { 0x9bc45be6, 61 },
    { 0xd0145be4, 23 },
    { 0x14345be8, 23 },
    { 0, 0 },
    { 0xa5445bea, 95 },
    { 0x49745beb, 85 },
    { 0, 0 },
    { 0x89c45bed, 51 },
    { 0x25b45bed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c845bf4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24845bfb, 61 },
    { 0, 0 },
    { 0x4f245bfd, 90 },
    { 0, 0 },
    { 0x5a445bff, 33 },
    { 0x41145c00, 100 },
    { 0xddd45c01, 90 },
    { 0, 0 },
    { 0x7f045c03, 23 },
    { 0, 0 },
    { 0xa8545c05, 43 },
    { 0x8f745c06, 76 },
    { 0xbbd45c07, 23 },
    { 0x45245c08, 84 },
    { 0xcd445c05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x54345c0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d45c10, 81 },
    { 0x4af45c11, 61 },
    { 0x59745c10, 33 },
    { 0xe8945c12, 76 },
    { 0x7c945c11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b45c17, 61 },
    { 0, 0 },
    { 0xeb645c19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab45c1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c1f, 61 },
    { 0, 0 },
    { 0x6545c21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74545c24, 90 },
    { 0, 0 },
    { 0x92b45c26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d45c32, 55 },
    { 0xfc845c33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f45c36, 76 },
    { 0x50a45c37, 61 },
    { 0xac245c37, 100 },
    { 0, 0 },
    { 0x94745c3a, 55 },
    { 0xe8745c3b, 33 },
    { 0x1fe45c3b, 55 },
    { 0x11245c3a, 98 },
    { 0x22845c3e, 33 },
    { 0xe2045c3b, 90 },
    { 0x8a445c40, 75 },
    { 0x56a45c41, 61 },
    { 0x97145c3c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d45c47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c45c4c, 90 },
    { 0x9a245c4d, 98 },
    { 0xbc145c4e, 98 },
    { 0x8845c4c, 43 },
    { 0, 0 },
    { 0xde45c51, 95 },
    { 0xc7b45c51, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43645c56, 67 },
    { 0, 0 },
    { 0xf8645c58, 77 },
    { 0xe3c45c58, 55 },
    { 0xab445c58, 93 },
    { 0x78345c58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae45c60, 98 },
    { 0, 0 },
    { 0x6ab45c62, 43 },
    { 0x25945c63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f45c66, 61 },
    { 0, 0 },
    { 0xa8245c68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf45c6b, 95 },
    { 0x33245c6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d45c6f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93645c73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a645c7a, 61 },
    { 0x5fc45c7b, 99 },
    { 0x79845c7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec45c7f, 55 },
    { 0x4b045c80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e45c84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45245c89, 98 },
    { 0xea745c8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9745c8f, 33 },
    { 0x86045c8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f645c99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc345c9c, 99 },
    { 0xa8245c9c, 90 },
    { 0x7b945c9e, 95 },
    { 0, 0 },
    { 0xbba45ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6845ca8, 95 },
    { 0x22545ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf45cac, 34 },
    { 0xc4345cad, 76 },
    { 0, 0 },
    { 0x43745caf, 99 },
    { 0xb1445caf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c945cb7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe45cbb, 23 },
    { 0x44045cbb, 61 },
    { 0xeaf45cbc, 93 },
    { 0xe4f45cbe, 84 },
    { 0, 0 },
    { 0x8a245cc0, 95 },
    { 0, 0 },
    { 0xd2545cc2, 99 },
    { 0, 0 },
    { 0xc0345cc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f45ccc, 33 },
    { 0xdf45ccd, 67 },
    { 0xb4245ccd, 87 },
    { 0x72845cce, 40 },
    { 0x94345cd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0045cd4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a645cdb, 23 },
    { 0x98745cdc, 98 },
    { 0xfa445cdd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a45cea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60345ced, 66 },
    { 0xc4f45cee, 77 },
    { 0, 0 },
    { 0x55245cf0, 78 },
    { 0, 0 },
    { 0x3d445cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42445cf5, 91 },
    { 0, 0 },
    { 0x65f45cf7, 98 },
    { 0, 0 },
    { 0x6d745cf9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87345d05, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba545d08, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59345d11, 88 },
    { 0x29545d11, 33 },
    { 0x3e245d11, 93 },
    { 0xa7f45d14, 98 },
    { 0xfc545d11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd45d19, 88 },
    { 0xdc345d19, 98 },
    { 0xd9045d1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be45d25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b45d29, 61 },
    { 0x15845d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b45d2d, 66 },
    { 0x30545d2d, 55 },
    { 0xad045d2f, 87 },
    { 0, 0 },
    { 0x8d745d31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e645d35, 78 },
    { 0xd8845d35, 90 },
    { 0xe2a45d35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca245d3a, 55 },
    { 0xa545d3b, 91 },
    { 0x91645d3c, 55 },
    { 0xf445d3b, 95 },
    { 0x8f045d3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd145d41, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f345d45, 85 },
    { 0x22845d46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d945d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f845d58, 33 },
    { 0x11445d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b45d5d, 95 },
    { 0x5ad45d5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb445d62, 93 },
    { 0, 0 },
    { 0x62c45d64, 33 },
    { 0x23245d65, 23 },
    { 0, 0 },
    { 0x63745d67, 52 },
    { 0x27145d68, 66 },
    { 0x82345d67, 69 },
    { 0xb445d67, 33 },
    { 0x15945d6b, 97 },
    { 0, 0 },
    { 0xb2f45d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b45d7b, 23 },
    { 0, 0 },
    { 0x46245d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f145d81, 98 },
    { 0, 0 },
    { 0xc9945d83, 33 },
    { 0xf2b45d83, 61 },
    { 0x29945d85, 34 },
    { 0xf9945d85, 95 },
    { 0xebe45d87, 76 },
    { 0x2b845d88, 91 },
    { 0x16a45d89, 33 },
    { 0x79b45d8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a45d8d, 90 },
    { 0, 0 },
    { 0x59945d8f, 98 },
    { 0x4c945d8f, 61 },
    { 0x23745d90, 61 },
    { 0, 0 },
    { 0xd1c45d93, 98 },
    { 0, 0 },
    { 0x19745d95, 91 },
    { 0xd5b45d96, 61 },
    { 0xe1745d96, 85 },
    { 0xb1345d97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a945d9f, 87 },
    { 0x38c45da0, 33 },
    { 0xeeb45da0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e45da5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84045da9, 52 },
    { 0xd5845daa, 90 },
    { 0xbc645dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90045dae, 95 },
    { 0xff645daf, 100 },
    { 0x4f245db0, 61 },
    { 0x7c145daf, 55 },
    { 0xae145db2, 100 },
    { 0x9345db3, 33 },
    { 0, 0 },
    { 0x48245db5, 85 },
    { 0x41d45db5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25645dba, 23 },
    { 0, 0 },
    { 0x39845dbc, 52 },
    { 0x73045dbd, 43 },
    { 0x44345dbe, 23 },
    { 0x2945dbe, 78 },
    { 0xdc445dc0, 34 },
    { 0x10845dc1, 75 },
    { 0xf345dc2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20845dc6, 100 },
    { 0xdb245dc6, 95 },
    { 0xeed45dc7, 23 },
    { 0, 0 },
    { 0xc2845dca, 98 },
    { 0x41a45dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e45dd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2845dd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac045ddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25045de1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16645de4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f945de9, 95 },
    { 0x39f45de9, 76 },
    { 0x89545deb, 82 },
    { 0, 0 },
    { 0xbc045ded, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8545df7, 90 },
    { 0x88045df8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d45dfc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x15145dff, 95 },
    { 0, 0 },
    { 0x58f45e01, 46 },
    { 0, 0 },
    { 0x80d45e03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce445e07, 61 },
    { 0x1445e08, 98 },
    { 0xe9d45e09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a45e0e, 100 },
    { 0xe7345e0f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de45e15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8845e23, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e545e27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d345e2b, 43 },
    { 0x7b345e2c, 43 },
    { 0x9d245e2d, 23 },
    { 0xf8945e2d, 69 },
    { 0x7d345e2c, 23 },
    { 0, 0 },
    { 0xc4b45e31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44145e39, 61 },
    { 0, 0 },
    { 0xb9345e3b, 95 },
    { 0, 0 },
    { 0x33245e3d, 33 },
    { 0x2e445e3e, 52 },
    { 0x10745e3e, 61 },
    { 0, 0 },
    { 0xc4945e41, 67 },
    { 0, 0 },
    { 0xde545e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9045e4d, 33 },
    { 0x84745e4e, 33 },
    { 0x7145e4f, 43 },
    { 0x94045e4e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a345e53, 95 },
    { 0x6f45e54, 55 },
    { 0x3ea45e54, 98 },
    { 0, 0 },
    { 0xc7d45e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda345e5d, 46 },
    { 0x5d945e5d, 95 },
    { 0, 0 },
    { 0xf1945e60, 61 },
    { 0xe6045e61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a845e69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c45e71, 85 },
    { 0x4a345e71, 38 },
    { 0x59a45e73, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x26445e76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4345e79, 66 },
    { 0xa4345e7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a245e7d, 90 },
    { 0x60c45e7e, 61 },
    { 0, 0 },
    { 0xf7c45e80, 23 },
    { 0xf8045e81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d445e86, 76 },
    { 0xf1b45e87, 81 },
    { 0x78045e88, 90 },
    { 0x4cd45e89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21945e8d, 76 },
    { 0x5ed45e8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36445e92, 33 },
    { 0xf4d45e93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7245e98, 33 },
    { 0x3b545e99, 98 },
    { 0x8c945e98, 34 },
    { 0, 0 },
    { 0x4c745e9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9245e9f, 34 },
    { 0x82d45e9f, 98 },
    { 0x84545ea1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc645ea5, 33 },
    { 0, 0 },
    { 0x7ff45ea7, 95 },
    { 0, 0 },
    { 0xf2945ea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71045eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee945eb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a45eba, 33 },
    { 0xec145ebb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d45ebe, 33 },
    { 0, 0 },
    { 0x79a45ec0, 61 },
    { 0, 0 },
    { 0xcc345ec2, 61 },
    { 0xb8445ec3, 76 },
    { 0x68d45ec3, 67 },
    { 0xb3745ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9745ecd, 99 },
    { 0, 0 },
    { 0x1ec45ecf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73145ed2, 60 },
    { 0xcde45ed3, 93 },
    { 0x28145ed4, 55 },
    { 0, 0 },
    { 0x40845ed6, 95 },
    { 0x51f45ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f45eda, 93 },
    { 0, 0 },
    { 0x1c445edc, 61 },
    { 0xb7845edd, 61 },
    { 0xd8c45ede, 95 },
    { 0xad845edf, 61 },
    { 0xa1445ede, 33 },
    { 0, 0 },
    { 0x2a745ee2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac445ee5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c45eeb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a245eee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2845ef2, 61 },
    { 0x6e845ef3, 98 },
    { 0x5e45ef3, 23 },
    { 0, 0 },
    { 0x7ee45ef6, 61 },
    { 0, 0 },
    { 0xc8e45ef8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b45efd, 33 },
    { 0x50b45efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a845f01, 96 },
    { 0xe7745f02, 90 },
    { 0xcef45f03, 23 },
    { 0x38b45f02, 61 },
    { 0xe2045f04, 93 },
    { 0, 0 },
    { 0x52145f07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f45f0a, 33 },
    { 0x36c45f0b, 76 },
    { 0x9c645f0b, 93 },
    { 0x7b245f0d, 76 },
    { 0x85d45f0a, 90 },
    { 0xbb045f0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e45f14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f45f18, 100 },
    { 0x81f45f18, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef45f21, 55 },
    { 0, 0 },
    { 0xf4345f23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8745f26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e45f29, 90 },
    { 0x44645f2a, 90 },
    { 0xb5945f2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46745f35, 95 },
    { 0, 0 },
    { 0x1e245f37, 91 },
    { 0xf3545f38, 61 },
    { 0, 0 },
    { 0x31145f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f45f3f, 23 },
    { 0xb7945f40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe945f43, 17 },
    { 0, 0 },
    { 0x43345f45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd945f48, 95 },
    { 0, 0 },
    { 0x73245f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d45f50, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81845f53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae845f57, 61 },
    { 0x37645f58, 33 },
    { 0x9c45f59, 95 },
    { 0x4f145f58, 23 },
    { 0xfbe45f57, 90 },
    { 0x2fe45f5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b345f62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c245f66, 61 },
    { 0x7b345f66, 34 },
    { 0x7d445f67, 77 },
    { 0xf6d45f66, 91 },
    { 0xd4045f67, 34 },
    { 0, 0 },
    { 0x5cd45f6c, 90 },
    { 0x40545f6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38045f70, 19 },
    { 0, 0 },
    { 0xaa845f72, 61 },
    { 0x8a545f73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec45f77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50745f7e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f745f81, 23 },
    { 0x8c345f82, 76 },
    { 0x1dd45f83, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b45f86, 93 },
    { 0xf3a45f87, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d945f8a, 61 },
    { 0x15345f8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb645f90, 76 },
    { 0xe2345f91, 55 },
    { 0x47145f90, 55 },
    { 0x8ab45f92, 92 },
    { 0, 0 },
    { 0x32c45f95, 90 },
    { 0, 0 },
    { 0x54645f97, 33 },
    { 0x7b645f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c745f9c, 84 },
    { 0xe6e45f9d, 64 },
    { 0x3f545f9c, 61 },
    { 0, 0 },
    { 0x70545fa0, 95 },
    { 0xa8145fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2645fa7, 66 },
    { 0x79045fa8, 95 },
    { 0x18745fa7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4145fac, 33 },
    { 0xd5945fad, 87 },
    { 0, 0 },
    { 0xbc245faf, 33 },
    { 0x6fe45fb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde45fb5, 17 },
    { 0x1d945fb5, 90 },
    { 0, 0 },
    { 0x64045fb8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8945fbd, 67 },
    { 0xbf745fbd, 43 },
    { 0xaa145fbe, 23 },
    { 0, 0 },
    { 0xfa545fc1, 85 },
    { 0x6a445fc2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7445fc7, 98 },
    { 0x36245fc8, 78 },
    { 0, 0 },
    { 0xfb645fca, 46 },
    { 0x8de45fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f45fd1, 95 },
    { 0, 0 },
    { 0x6d945fd3, 90 },
    { 0xb3f45fd4, 98 },
    { 0, 0 },
    { 0x79f45fd6, 23 },
    { 0x19a45fd7, 76 },
    { 0, 0 },
    { 0x30845fd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e45fdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb645fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3945fe9, 95 },
    { 0, 0 },
    { 0x26045feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26745ff1, 88 },
    { 0x2b945ff2, 90 },
    { 0, 0 },
    { 0x24445ff4, 55 },
    { 0, 0 },
    { 0x90245ff6, 90 },
    { 0x68a45ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae45fff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85646006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a4600a, 76 },
    { 0, 0 },
    { 0x8fa4600c, 84 },
    { 0x5e4600d, 61 },
    { 0, 0 },
    { 0x6894600f, 98 },
    { 0x6bc46010, 98 },
    { 0x43946011, 23 },
    { 0, 0 },
    { 0x11b46013, 33 },
    { 0x12346014, 90 },
    { 0x16046015, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41646019, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d4601e, 23 },
    { 0xe64601f, 98 },
    { 0xe2746020, 95 },
    { 0xb7a46020, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de46024, 52 },
    { 0, 0 },
    { 0x82f46026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35646029, 61 },
    { 0, 0 },
    { 0xf664602b, 46 },
    { 0x8234602c, 90 },
    { 0x5384602d, 55 },
    { 0x4034602b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf346036, 99 },
    { 0x35946037, 85 },
    { 0xa3146038, 78 },
    { 0xd8f46039, 93 },
    { 0x4e646039, 95 },
    { 0, 0 },
    { 0xd4d4603c, 61 },
    { 0, 0 },
    { 0x6964603e, 85 },
    { 0x1694603e, 33 },
    { 0x63d46040, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3546049, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e46050, 95 },
    { 0, 0 },
    { 0x5d746052, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5446055, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5546058, 76 },
    { 0x6746059, 23 },
    { 0x6604605a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a46060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e046065, 90 },
    { 0x95a46065, 90 },
    { 0, 0 },
    { 0xe3246068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5454606b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c46070, 61 },
    { 0xcc846070, 17 },
    { 0, 0 },
    { 0x9be46073, 83 },
    { 0xa4e46074, 33 },
    { 0xb8e46075, 34 },
    { 0x8f346076, 34 },
    { 0, 0 },
    { 0xbe546078, 66 },
    { 0, 0 },
    { 0xcb4607a, 43 },
    { 0xd284607b, 53 },
    { 0xd074607c, 76 },
    { 0x3474607d, 95 },
    { 0x1544607d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f46081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd46089, 61 },
    { 0x3cd46089, 43 },
    { 0xe814608b, 95 },
    { 0, 0 },
    { 0xfc4608d, 46 },
    { 0xcec4608e, 90 },
    { 0xb394608d, 40 },
    { 0x8284608f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18046096, 98 },
    { 0, 0 },
    { 0xd0946098, 93 },
    { 0xe2d46098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb334609d, 93 },
    { 0, 0 },
    { 0x61f4609f, 76 },
    { 0, 0 },
    { 0x5f8460a1, 46 },
    { 0x94e460a2, 61 },
    { 0xbb1460a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4460a8, 33 },
    { 0x3d0460a8, 23 },
    { 0x206460a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0460b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82460bb, 66 },
    { 0, 0 },
    { 0x5d7460bd, 50 },
    { 0xdec460be, 67 },
    { 0x192460bf, 69 },
    { 0, 0 },
    { 0xb89460c1, 90 },
    { 0x4d6460c2, 76 },
    { 0x28460c3, 34 },
    { 0x8e7460c1, 46 },
    { 0xa16460c3, 95 },
    { 0xe1c460c6, 95 },
    { 0xf9d460c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9460ce, 98 },
    { 0xc74460cf, 76 },
    { 0, 0 },
    { 0x73d460d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b460d6, 34 },
    { 0xd7f460d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0460dc, 98 },
    { 0x1d2460dd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad460e0, 78 },
    { 0xcc6460e0, 76 },
    { 0x4ef460e2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416460e6, 33 },
    { 0, 0 },
    { 0xced460e8, 95 },
    { 0x17e460e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x573460ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9460f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58460f5, 90 },
    { 0x94a460f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523460fa, 90 },
    { 0x5b5460fa, 33 },
    { 0, 0 },
    { 0x44e460fd, 55 },
    { 0, 0 },
    { 0xe49460ff, 23 },
    { 0, 0 },
    { 0x50546101, 98 },
    { 0x96646102, 95 },
    { 0, 0 },
    { 0x25746104, 33 },
    { 0x5ef46105, 46 },
    { 0, 0 },
    { 0x46546107, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a46115, 34 },
    { 0xfdb46116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c46119, 33 },
    { 0, 0 },
    { 0x5504611b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646127, 78 },
    { 0x32c46128, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5934612e, 23 },
    { 0, 0 },
    { 0x46046130, 52 },
    { 0x28746130, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c46137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c4613a, 83 },
    { 0x8614613b, 33 },
    { 0x33b4613c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b4613f, 51 },
    { 0x8474613f, 76 },
    { 0x50746141, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2364614d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46246154, 91 },
    { 0, 0 },
    { 0x9c646156, 91 },
    { 0x13546157, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e4615c, 55 },
    { 0, 0 },
    { 0x4b74615e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6546161, 65 },
    { 0xb4446162, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b46165, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a46168, 61 },
    { 0xeeb46169, 78 },
    { 0, 0 },
    { 0xf674616b, 78 },
    { 0xfe24616b, 34 },
    { 0x6324616b, 51 },
    { 0x8574616e, 98 },
    { 0x7934616c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a46174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3474617d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a46182, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4846186, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f46189, 61 },
    { 0, 0 },
    { 0xa6c4618b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4618f, 95 },
    { 0xe9a46190, 61 },
    { 0x35146190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb46196, 23 },
    { 0x24746197, 99 },
    { 0xaed46196, 100 },
    { 0xdb146199, 61 },
    { 0, 0 },
    { 0xfd64619b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30461a1, 78 },
    { 0xa83461a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372461a9, 88 },
    { 0x410461a9, 55 },
    { 0xbca461ab, 92 },
    { 0x9b3461ab, 33 },
    { 0x639461ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68461b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf461b5, 61 },
    { 0xcd6461b6, 23 },
    { 0x668461b6, 61 },
    { 0x301461b8, 34 },
    { 0x610461b9, 55 },
    { 0x4f3461ba, 96 },
    { 0xb3c461b8, 77 },
    { 0x41e461bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72461bf, 33 },
    { 0, 0 },
    { 0xe68461c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f461c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d461c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7461ce, 98 },
    { 0, 0 },
    { 0xac5461d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509461d8, 75 },
    { 0x62d461d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0461dc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9461e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3461ed, 33 },
    { 0xcc5461ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35461f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a461f5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404461f9, 46 },
    { 0, 0 },
    { 0x1cf461fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97946200, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d946205, 90 },
    { 0x53a46206, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xec246209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794620c, 98 },
    { 0, 0 },
    { 0xe0b4620e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32546213, 33 },
    { 0x9a846214, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7846217, 23 },
    { 0, 0 },
    { 0xe5946219, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f46221, 52 },
    { 0x65346222, 90 },
    { 0x68746223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246227, 33 },
    { 0, 0 },
    { 0xe9f46229, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7946231, 95 },
    { 0, 0 },
    { 0x59d46233, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f46238, 61 },
    { 0x67246239, 98 },
    { 0x3b44623a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6146240, 61 },
    { 0, 0 },
    { 0xb7946242, 64 },
    { 0, 0 },
    { 0xbbe46244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee46249, 33 },
    { 0x9764624a, 90 },
    { 0xbc44624a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec4624f, 87 },
    { 0, 0 },
    { 0xd7646251, 98 },
    { 0x48346252, 94 },
    { 0xd6446253, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf446256, 55 },
    { 0x8c246257, 90 },
    { 0xb6c46258, 101 },
    { 0xc4446258, 78 },
    { 0x2604625a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7714625f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70446266, 33 },
    { 0x46d46266, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb104626a, 90 },
    { 0x29a4626a, 90 },
    { 0x394626c, 34 },
    { 0xa6e4626d, 55 },
    { 0x91b4626a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd146271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac46274, 98 },
    { 0x75146275, 95 },
    { 0x54346276, 61 },
    { 0, 0 },
    { 0x1d946278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64627c, 95 },
    { 0xf314627d, 77 },
    { 0xc974627e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa46281, 33 },
    { 0xfab46281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c346285, 90 },
    { 0x97646286, 95 },
    { 0xa3b46287, 95 },
    { 0, 0 },
    { 0xff746289, 84 },
    { 0xbb84628a, 33 },
    { 0x5414628a, 98 },
    { 0x55f4628a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c46294, 61 },
    { 0, 0 },
    { 0x2ba46296, 46 },
    { 0xdff46297, 90 },
    { 0xe1b46296, 93 },
    { 0x61e46299, 51 },
    { 0, 0 },
    { 0x1514629b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f84629f, 23 },
    { 0x7bf4629f, 46 },
    { 0, 0 },
    { 0x65e462a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49462a5, 90 },
    { 0, 0 },
    { 0x35462a7, 85 },
    { 0xc29462a8, 33 },
    { 0x8f0462a9, 76 },
    { 0x1f462aa, 23 },
    { 0xa55462aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496462b0, 90 },
    { 0, 0 },
    { 0x8c6462b2, 46 },
    { 0xe1d462b3, 23 },
    { 0, 0 },
    { 0xf5c462b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b462b8, 95 },
    { 0x519462b9, 52 },
    { 0xb6e462b9, 17 },
    { 0xe46462b9, 23 },
    { 0xa57462bc, 23 },
    { 0xe03462bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b462c0, 85 },
    { 0x52d462c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745462c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e462cb, 46 },
    { 0x830462cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e462d0, 34 },
    { 0x9c9462d1, 84 },
    { 0x8d3462d0, 46 },
    { 0x2dd462d3, 33 },
    { 0x3fa462d2, 98 },
    { 0x2a7462d4, 84 },
    { 0, 0 },
    { 0xa70462d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce462da, 99 },
    { 0, 0 },
    { 0x5e7462dc, 98 },
    { 0xae1462dc, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5462e0, 98 },
    { 0x951462e1, 77 },
    { 0xc42462e2, 43 },
    { 0x318462e2, 23 },
    { 0, 0 },
    { 0x3a1462e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0462e9, 95 },
    { 0x834462ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5462ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30462f1, 90 },
    { 0x335462f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02462fc, 55 },
    { 0x2d2462fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a46301, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f46304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f4630d, 98 },
    { 0x3fe4630e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd46312, 33 },
    { 0x7be46313, 55 },
    { 0, 0 },
    { 0x7d846315, 100 },
    { 0x29f46316, 90 },
    { 0, 0 },
    { 0x1e46318, 52 },
    { 0x3f846318, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d46322, 33 },
    { 0xe9946323, 55 },
    { 0x92246324, 90 },
    { 0, 0 },
    { 0xc6e46326, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd746329, 46 },
    { 0x65f4632a, 33 },
    { 0x8b34632b, 90 },
    { 0x34e4632c, 95 },
    { 0, 0 },
    { 0xae24632e, 61 },
    { 0x1884632e, 98 },
    { 0x5d146330, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28446334, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f46338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7346346, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b04634a, 95 },
    { 0, 0 },
    { 0xea04634c, 91 },
    { 0x1c34634d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4046352, 90 },
    { 0, 0 },
    { 0x64f46354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e46358, 90 },
    { 0xc346359, 61 },
    { 0x6614635a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc54635d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84146360, 95 },
    { 0x25746360, 93 },
    { 0xa5a46362, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61546366, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e74636c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d46373, 95 },
    { 0x81346374, 43 },
    { 0xc5f46375, 34 },
    { 0xbb846376, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f94637a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c46389, 93 },
    { 0x3244638a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd4638d, 17 },
    { 0x6e54638e, 55 },
    { 0x55f4638d, 76 },
    { 0x23d4638f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9246393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c46397, 23 },
    { 0, 0 },
    { 0x28746399, 100 },
    { 0xf9a4639a, 90 },
    { 0x7ce46399, 33 },
    { 0xd494639b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c74639f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x287463a2, 91 },
    { 0, 0 },
    { 0x5d6463a4, 88 },
    { 0xc97463a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d463a8, 90 },
    { 0x779463a9, 90 },
    { 0, 0 },
    { 0x746463ab, 91 },
    { 0x889463ab, 95 },
    { 0x5f8463ac, 23 },
    { 0, 0 },
    { 0xc7f463af, 78 },
    { 0x87e463af, 90 },
    { 0xe39463b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96463b5, 95 },
    { 0, 0 },
    { 0x18c463b7, 67 },
    { 0x2cb463b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1463bb, 95 },
    { 0, 0 },
    { 0xaa8463bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c463c6, 43 },
    { 0x940463c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91463cc, 90 },
    { 0x25d463cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a463d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4463d6, 93 },
    { 0, 0 },
    { 0x5df463d8, 86 },
    { 0x8463d8, 61 },
    { 0, 0 },
    { 0x13e463db, 98 },
    { 0x308463dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4463ed, 99 },
    { 0, 0 },
    { 0x9a4463ef, 33 },
    { 0, 0 },
    { 0x847463f1, 52 },
    { 0x4c2463f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a463f7, 84 },
    { 0x5ce463f7, 93 },
    { 0x16b463f9, 23 },
    { 0x701463f9, 76 },
    { 0x646463f8, 67 },
    { 0xfab463fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37846404, 94 },
    { 0xd5546404, 90 },
    { 0x38e46406, 61 },
    { 0xed46407, 93 },
    { 0, 0 },
    { 0x12c46409, 61 },
    { 0x1f046409, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e4640d, 88 },
    { 0, 0 },
    { 0x65a4640f, 34 },
    { 0xa044640f, 75 },
    { 0, 0 },
    { 0x12546412, 34 },
    { 0x35346413, 61 },
    { 0x1b746414, 33 },
    { 0x63f46415, 68 },
    { 0x46e46416, 61 },
    { 0x57f46417, 77 },
    { 0, 0 },
    { 0x3e146419, 23 },
    { 0x84b4641a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9794641d, 23 },
    { 0x2634641e, 100 },
    { 0x3bc4641f, 76 },
    { 0xf7946420, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c46425, 33 },
    { 0x83746426, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b04642a, 91 },
    { 0, 0 },
    { 0xc6b4642c, 46 },
    { 0x9154642c, 33 },
    { 0x8114642e, 91 },
    { 0x3d04642f, 33 },
    { 0x9b84642d, 90 },
    { 0, 0 },
    { 0xb1946432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1294643b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d646444, 87 },
    { 0xff446445, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc46448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d4644d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c46452, 61 },
    { 0x53446452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65746459, 55 },
    { 0xf4346459, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a4645d, 33 },
    { 0xefb4645d, 33 },
    { 0x64f4645f, 98 },
    { 0x4fe46460, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4946464, 60 },
    { 0xcf046464, 33 },
    { 0, 0 },
    { 0x2cf46467, 33 },
    { 0x21c46468, 88 },
    { 0xa8346468, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa04646d, 76 },
    { 0x75d4646e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd46475, 61 },
    { 0x6e646475, 55 },
    { 0x3c846477, 46 },
    { 0xde146478, 61 },
    { 0xbbc46478, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa184647c, 76 },
    { 0x54a4647d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e46481, 90 },
    { 0x96a46482, 61 },
    { 0x8bd46482, 93 },
    { 0, 0 },
    { 0x1cd46485, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c4648a, 90 },
    { 0xc694648b, 61 },
    { 0, 0 },
    { 0x3114648d, 99 },
    { 0x7ac4648e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c46493, 95 },
    { 0xe4e46494, 33 },
    { 0x98b46494, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca4649f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29464a4, 43 },
    { 0x5d1464a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x746464a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x544464ab, 95 },
    { 0, 0 },
    { 0xe1c464ad, 61 },
    { 0xdf5464ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a464b3, 33 },
    { 0xb48464b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a464c3, 55 },
    { 0x718464c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737464cc, 95 },
    { 0, 0 },
    { 0x1ab464ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07464d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3464d8, 76 },
    { 0, 0 },
    { 0xeec464da, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6464dd, 86 },
    { 0xb3a464dd, 92 },
    { 0x414464dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a464e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23464e6, 46 },
    { 0x4fe464e6, 34 },
    { 0, 0 },
    { 0xeb464e9, 33 },
    { 0xab4464e9, 69 },
    { 0, 0 },
    { 0xdf1464ec, 90 },
    { 0, 0 },
    { 0xb0d464ee, 98 },
    { 0xbe8464ef, 81 },
    { 0x378464f0, 55 },
    { 0x153464ee, 33 },
    { 0, 0 },
    { 0xbde464f3, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb464f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc464f9, 46 },
    { 0x9eb464f9, 33 },
    { 0x83c464fb, 61 },
    { 0xdc9464fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4146500, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb846503, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea946508, 61 },
    { 0x37346509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd84650d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c146510, 23 },
    { 0xbd446511, 34 },
    { 0x68946512, 90 },
    { 0x68146512, 90 },
    { 0x18e46514, 61 },
    { 0x71246515, 33 },
    { 0, 0 },
    { 0x69d46517, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad4651a, 61 },
    { 0, 0 },
    { 0x1234651c, 95 },
    { 0, 0 },
    { 0x3a54651e, 77 },
    { 0x8744651f, 9 },
    { 0x3764651e, 33 },
    { 0xeb64651f, 76 },
    { 0xcba46522, 68 },
    { 0x23946523, 34 },
    { 0x65a46522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f46528, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0746540, 76 },
    { 0, 0 },
    { 0xb7b46542, 98 },
    { 0xd4546543, 46 },
    { 0xa0046544, 46 },
    { 0xb9246542, 61 },
    { 0x18e46544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4684654d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba46555, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5146559, 98 },
    { 0xfb44655a, 98 },
    { 0xdae4655b, 90 },
    { 0x7a44655c, 95 },
    { 0, 0 },
    { 0xaa64655e, 61 },
    { 0xec64655f, 95 },
    { 0, 0 },
    { 0xf9d46561, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e4656c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc324656f, 95 },
    { 0, 0 },
    { 0x12f46571, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c46577, 77 },
    { 0xe8046578, 84 },
    { 0xc2d46578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a4657d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd146586, 61 },
    { 0x12346587, 90 },
    { 0, 0 },
    { 0x2a946589, 34 },
    { 0xeb34658a, 66 },
    { 0xc5a46589, 90 },
    { 0, 0 },
    { 0xcc54658d, 52 },
    { 0x6744658d, 34 },
    { 0xab94658d, 91 },
    { 0xef646590, 77 },
    { 0x67a46590, 61 },
    { 0x86a46591, 95 },
    { 0xc746593, 90 },
    { 0xa1b46593, 33 },
    { 0x4f046595, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a4659d, 76 },
    { 0xa374659d, 98 },
    { 0x96b4659f, 43 },
    { 0xa89465a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3465a8, 23 },
    { 0x1df465a8, 61 },
    { 0x3b2465a8, 90 },
    { 0x589465ab, 34 },
    { 0x6b6465a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c465b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5465b8, 66 },
    { 0, 0 },
    { 0x65f465ba, 90 },
    { 0, 0 },
    { 0xc5f465bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd465c2, 61 },
    { 0x3a465c3, 23 },
    { 0x429465c3, 87 },
    { 0, 0 },
    { 0xa81465c6, 52 },
    { 0x205465c6, 61 },
    { 0x8a1465c8, 84 },
    { 0, 0 },
    { 0xc43465ca, 95 },
    { 0, 0 },
    { 0xe0d465cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d465cf, 90 },
    { 0x450465cf, 90 },
    { 0xdd3465d1, 68 },
    { 0x1d465d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c465da, 61 },
    { 0, 0 },
    { 0xa7e465dc, 23 },
    { 0, 0 },
    { 0x70d465de, 98 },
    { 0xbf8465df, 93 },
    { 0x472465e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71465e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x518465e6, 81 },
    { 0, 0 },
    { 0xc9b465e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0465ee, 90 },
    { 0, 0 },
    { 0x3bf465f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f465f4, 23 },
    { 0x74f465f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68465fc, 76 },
    { 0, 0 },
    { 0x8b1465fe, 76 },
    { 0, 0 },
    { 0xae446600, 61 },
    { 0x72546601, 64 },
    { 0xcf046602, 55 },
    { 0, 0 },
    { 0x3ff46604, 87 },
    { 0xa9b46604, 61 },
    { 0x91b46604, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf346609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d24660d, 22 },
    { 0xb684660e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5946612, 98 },
    { 0xa3b46613, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c54661b, 33 },
    { 0, 0 },
    { 0x6384661d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46146621, 23 },
    { 0x42546622, 90 },
    { 0xbcf46623, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab446629, 95 },
    { 0xdde4662a, 61 },
    { 0, 0 },
    { 0xfc94662c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8446630, 95 },
    { 0x30246631, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a24663a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa824663e, 98 },
    { 0, 0 },
    { 0x6546640, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9846647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf246654, 82 },
    { 0xead46655, 95 },
    { 0x6b746655, 33 },
    { 0x2246655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa814665e, 61 },
    { 0, 0 },
    { 0xb8846660, 55 },
    { 0x99046660, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56646668, 55 },
    { 0x5c946669, 52 },
    { 0x8a846669, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc834666d, 95 },
    { 0x7ba4666e, 98 },
    { 0, 0 },
    { 0x9546670, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d46673, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e46676, 90 },
    { 0x38046677, 76 },
    { 0, 0 },
    { 0xa2546679, 90 },
    { 0x1934667a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb4667f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6746685, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5944668b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf46691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa546694, 87 },
    { 0xcae46694, 8 },
    { 0, 0 },
    { 0x80e46697, 93 },
    { 0, 0 },
    { 0xc3e46699, 12 },
    { 0x86446699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e4669f, 90 },
    { 0, 0 },
    { 0x619466a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x978466a4, 95 },
    { 0, 0 },
    { 0x9a2466a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735466b3, 67 },
    { 0xf25466b4, 95 },
    { 0xb31466b5, 95 },
    { 0x1ac466b6, 61 },
    { 0x697466b7, 43 },
    { 0x804466b6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d466c2, 55 },
    { 0xda6466c2, 23 },
    { 0x927466c4, 33 },
    { 0x461466c4, 33 },
    { 0x85f466c6, 52 },
    { 0, 0 },
    { 0xd64466c8, 68 },
    { 0, 0 },
    { 0x294466ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95466ce, 61 },
    { 0x4fb466ce, 88 },
    { 0x2e8466d0, 90 },
    { 0, 0 },
    { 0x7d1466d2, 33 },
    { 0x1bd466d2, 98 },
    { 0x831466d4, 76 },
    { 0xa9a466d5, 33 },
    { 0x1e2466d6, 98 },
    { 0x706466d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc466e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10466e9, 43 },
    { 0, 0 },
    { 0xf02466eb, 55 },
    { 0, 0 },
    { 0x488466ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab466f1, 61 },
    { 0x8e4466f1, 61 },
    { 0, 0 },
    { 0x990466f4, 43 },
    { 0, 0 },
    { 0x1be466f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c466fa, 95 },
    { 0xbd3466fa, 90 },
    { 0, 0 },
    { 0xec466fd, 64 },
    { 0x3c5466fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a46702, 90 },
    { 0xa9a46703, 33 },
    { 0, 0 },
    { 0xa3e46705, 75 },
    { 0x8cc46706, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2384670a, 61 },
    { 0, 0 },
    { 0x3574670c, 56 },
    { 0x5644670d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e46711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4671b, 75 },
    { 0xd414671c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc4671f, 55 },
    { 0x8654671f, 99 },
    { 0x69246721, 98 },
    { 0x16a46720, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b046726, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a46729, 33 },
    { 0x2a546729, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad46732, 98 },
    { 0x1a346733, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa574673b, 95 },
    { 0, 0 },
    { 0xaaa4673d, 61 },
    { 0xd124673d, 76 },
    { 0xf4f4673f, 76 },
    { 0xa4a46740, 87 },
    { 0x7bf4673d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5346744, 34 },
    { 0xa5746744, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc446748, 98 },
    { 0x88d46748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a4674e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb46751, 43 },
    { 0, 0 },
    { 0x21746753, 52 },
    { 0, 0 },
    { 0xbfd46755, 23 },
    { 0x53646755, 61 },
    { 0x79246757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a84675b, 95 },
    { 0x3d24675b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9346760, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x14346763, 90 },
    { 0xf7d46763, 100 },
    { 0x9b746765, 77 },
    { 0x34346766, 33 },
    { 0xb8846765, 90 },
    { 0, 0 },
    { 0xe6446769, 23 },
    { 0, 0 },
    { 0xe034676b, 95 },
    { 0x2324676b, 61 },
    { 0x6f44676c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d46774, 23 },
    { 0x7c846774, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e4677a, 23 },
    { 0x4d14677a, 75 },
    { 0x2a44677c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9824677f, 55 },
    { 0xc194677f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a746784, 40 },
    { 0xde646784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f4678a, 33 },
    { 0xb904678a, 90 },
    { 0xb134678c, 76 },
    { 0xfb44678b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c46790, 55 },
    { 0x56e46791, 76 },
    { 0x4a346792, 76 },
    { 0xc3246793, 52 },
    { 0x41646794, 90 },
    { 0xfda46795, 66 },
    { 0x60546794, 61 },
    { 0x53d46797, 100 },
    { 0xa5f46798, 77 },
    { 0x7e046798, 33 },
    { 0, 0 },
    { 0x95b4679b, 76 },
    { 0x20b4679c, 99 },
    { 0xb024679d, 23 },
    { 0x794679e, 61 },
    { 0xb0f4679f, 98 },
    { 0xa954679f, 76 },
    { 0x7db4679b, 43 },
    { 0xe24679c, 46 },
    { 0xd3f467a3, 78 },
    { 0x233467a4, 43 },
    { 0, 0 },
    { 0xa40467a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9467a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61467b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a467b4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe467b7, 33 },
    { 0, 0 },
    { 0x456467b9, 77 },
    { 0x501467ba, 34 },
    { 0, 0 },
    { 0xe80467bc, 43 },
    { 0xb2c467bd, 43 },
    { 0xbb1467be, 23 },
    { 0, 0 },
    { 0xf63467c0, 90 },
    { 0, 0 },
    { 0xaf6467c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c467c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6467ca, 23 },
    { 0xd84467cb, 23 },
    { 0x14c467ca, 66 },
    { 0xff3467cd, 61 },
    { 0x3c0467ce, 33 },
    { 0x406467cf, 80 },
    { 0xbeb467ce, 43 },
    { 0, 0 },
    { 0x592467d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4467d5, 99 },
    { 0x5bb467d6, 93 },
    { 0xd09467d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df467df, 88 },
    { 0x82c467df, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf467e4, 48 },
    { 0, 0 },
    { 0x507467e6, 75 },
    { 0xf99467e6, 77 },
    { 0x435467e8, 77 },
    { 0, 0 },
    { 0xf7c467ea, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x484467ed, 55 },
    { 0, 0 },
    { 0x45467ef, 98 },
    { 0x1f6467ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd467f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1467fa, 99 },
    { 0x623467fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a46800, 76 },
    { 0x6b246800, 33 },
    { 0x14546800, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95546805, 100 },
    { 0xc4e46806, 61 },
    { 0xdb46807, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x104680a, 61 },
    { 0x8234680b, 95 },
    { 0x5ce4680c, 33 },
    { 0x4d34680d, 100 },
    { 0x7ff4680e, 23 },
    { 0x73d4680d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df46812, 55 },
    { 0x66a46813, 95 },
    { 0xba146812, 95 },
    { 0xd2e46815, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f946818, 33 },
    { 0xcaf46819, 80 },
    { 0xc1c4681a, 33 },
    { 0x2b4681b, 93 },
    { 0x5314681c, 95 },
    { 0, 0 },
    { 0x4404681e, 23 },
    { 0xdd24681f, 99 },
    { 0xfc54681e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4346826, 23 },
    { 0, 0 },
    { 0x84d46828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4334682e, 85 },
    { 0x2704682f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13346833, 66 },
    { 0, 0 },
    { 0x22f46835, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65746839, 52 },
    { 0x1146839, 75 },
    { 0x2cf4683b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4683e, 85 },
    { 0xfee4683f, 33 },
    { 0xf2346840, 90 },
    { 0xfc246841, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1724684d, 87 },
    { 0x2c44684d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd546852, 23 },
    { 0, 0 },
    { 0x1a246854, 83 },
    { 0x6fa46854, 34 },
    { 0xea446856, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b46859, 33 },
    { 0x6ea46859, 66 },
    { 0xa514685a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8764685f, 98 },
    { 0, 0 },
    { 0x3c546861, 87 },
    { 0, 0 },
    { 0x7046863, 68 },
    { 0x87c46864, 61 },
    { 0x7f346865, 87 },
    { 0x30846866, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f84686a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4686f, 46 },
    { 0xa1346870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f346873, 100 },
    { 0x8de46874, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe94687b, 34 },
    { 0x9614687b, 76 },
    { 0x3584687d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54746883, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28146888, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9344688d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe246894, 84 },
    { 0, 0 },
    { 0x86746896, 87 },
    { 0, 0 },
    { 0x16f46898, 23 },
    { 0, 0 },
    { 0x1dd4689a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c14689f, 23 },
    { 0xa73468a0, 23 },
    { 0xd5468a0, 87 },
    { 0x96f468a0, 83 },
    { 0x29d4689f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d468a6, 33 },
    { 0xabd468a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5468aa, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4468ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c468b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22468b4, 76 },
    { 0x8ca468b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc468bc, 98 },
    { 0, 0 },
    { 0xb62468be, 61 },
    { 0xb76468bf, 85 },
    { 0x1c9468be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b468c9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42468cd, 88 },
    { 0xb88468ce, 95 },
    { 0x46d468cd, 67 },
    { 0xca3468d0, 90 },
    { 0xd45468d1, 55 },
    { 0xae4468cf, 93 },
    { 0, 0 },
    { 0x1c468d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249468d9, 33 },
    { 0, 0 },
    { 0x138468db, 98 },
    { 0x8f5468dc, 90 },
    { 0x5ec468dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04468e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x147468e3, 95 },
    { 0x56b468e3, 61 },
    { 0, 0 },
    { 0x2a468e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b468e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905468ed, 98 },
    { 0xb45468ee, 100 },
    { 0x7cf468ee, 23 },
    { 0x66a468f0, 76 },
    { 0x2d7468ee, 76 },
    { 0xeb9468f2, 90 },
    { 0x352468f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6468f6, 100 },
    { 0, 0 },
    { 0xd7d468f8, 90 },
    { 0, 0 },
    { 0x1c4468fa, 98 },
    { 0xb63468fb, 33 },
    { 0x56c468fc, 61 },
    { 0x6d8468fc, 66 },
    { 0xc33468fd, 43 },
    { 0x7ff468ff, 61 },
    { 0x3f646900, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c46903, 52 },
    { 0x7d746903, 98 },
    { 0x19546905, 23 },
    { 0x7d946904, 33 },
    { 0, 0 },
    { 0xe0446908, 55 },
    { 0xae446909, 33 },
    { 0x2904690a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14690d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73546912, 90 },
    { 0xf4c46912, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e46919, 78 },
    { 0xa894691a, 23 },
    { 0x8d74691b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c246920, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43346923, 68 },
    { 0x91146924, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e46929, 90 },
    { 0x3fc4692a, 55 },
    { 0x3df4692b, 61 },
    { 0x2b44692c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1046931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d46934, 61 },
    { 0x25546934, 90 },
    { 0xe5446934, 52 },
    { 0x5bb46934, 43 },
    { 0xfe146938, 90 },
    { 0x74746936, 90 },
    { 0x8e4693a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde546940, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35146943, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b46948, 33 },
    { 0, 0 },
    { 0xff4694a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6094694e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7146951, 61 },
    { 0x89746952, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93646957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4695a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf134695d, 53 },
    { 0xdf44695e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d46963, 85 },
    { 0x11f46964, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e74696d, 61 },
    { 0xac74696d, 23 },
    { 0x48b4696f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec46972, 46 },
    { 0x32f46972, 61 },
    { 0x92d46973, 33 },
    { 0xdeb46975, 23 },
    { 0xb7046974, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8164697a, 55 },
    { 0xb5a4697b, 98 },
    { 0, 0 },
    { 0x4364697d, 23 },
    { 0, 0 },
    { 0xcf34697f, 84 },
    { 0xa2946980, 33 },
    { 0xbc546980, 23 },
    { 0x8d46981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6246988, 76 },
    { 0xb7046989, 66 },
    { 0x3ec46989, 64 },
    { 0x7ac4698b, 33 },
    { 0x34d4698c, 90 },
    { 0x45b4698d, 55 },
    { 0x5cc4698d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf46994, 99 },
    { 0xa4146995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f46999, 95 },
    { 0, 0 },
    { 0x7274699b, 23 },
    { 0x234699c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127469a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41469a5, 90 },
    { 0x8e1469a6, 91 },
    { 0xbca469a6, 61 },
    { 0x5a5469a7, 76 },
    { 0xce6469a9, 33 },
    { 0xbd0469a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5469ad, 90 },
    { 0, 0 },
    { 0x776469af, 34 },
    { 0x854469af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79469b3, 90 },
    { 0, 0 },
    { 0x412469b5, 33 },
    { 0, 0 },
    { 0x82d469b7, 61 },
    { 0xc66469b7, 61 },
    { 0x956469b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155469be, 61 },
    { 0xe65469bf, 98 },
    { 0x626469c0, 66 },
    { 0, 0 },
    { 0x3ed469c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63469c7, 33 },
    { 0, 0 },
    { 0x4df469c9, 33 },
    { 0x3ba469ca, 76 },
    { 0, 0 },
    { 0x32f469cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b469d0, 23 },
    { 0xefd469d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df469d6, 90 },
    { 0x11f469d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e469da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67469de, 61 },
    { 0, 0 },
    { 0x459469e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c469e6, 90 },
    { 0x91b469e7, 76 },
    { 0, 0 },
    { 0xc81469e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405469f0, 61 },
    { 0, 0 },
    { 0xf0a469f2, 93 },
    { 0xa0a469f3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee469f8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19469fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91646a03, 90 },
    { 0xabe46a04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1246a0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35246a17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98646a1b, 85 },
    { 0xd346a1b, 95 },
    { 0, 0 },
    { 0xd4046a1e, 83 },
    { 0x19a46a1f, 90 },
    { 0x8fc46a1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5546a24, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x80146a27, 17 },
    { 0, 0 },
    { 0xbef46a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa746a31, 33 },
    { 0xadc46a31, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76046a38, 76 },
    { 0x86146a39, 90 },
    { 0xb6e46a38, 65 },
    { 0xb746a38, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66146a3f, 76 },
    { 0, 0 },
    { 0x32046a41, 68 },
    { 0xaaf46a42, 67 },
    { 0x57c46a43, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c846a46, 90 },
    { 0x86646a46, 98 },
    { 0, 0 },
    { 0xe9146a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc246a4c, 90 },
    { 0x9346a4d, 98 },
    { 0, 0 },
    { 0xced46a4f, 46 },
    { 0xc146a50, 93 },
    { 0, 0 },
    { 0xc4b46a52, 55 },
    { 0, 0 },
    { 0xff046a54, 76 },
    { 0, 0 },
    { 0x74546a56, 90 },
    { 0x22846a57, 91 },
    { 0xb9646a57, 95 },
    { 0xf0346a59, 61 },
    { 0x4f446a59, 95 },
    { 0x65046a59, 95 },
    { 0xd7546a57, 23 },
    { 0xfcd46a56, 85 },
    { 0xaa346a5e, 95 },
    { 0x6c846a5a, 93 },
    { 0xd1046a5f, 90 },
    { 0xd8946a61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d346a64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28146a68, 90 },
    { 0, 0 },
    { 0x86d46a6a, 78 },
    { 0x32346a6a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c46a70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce546a73, 93 },
    { 0, 0 },
    { 0xa8346a75, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7946a80, 33 },
    { 0x10246a80, 75 },
    { 0xb346a82, 78 },
    { 0x43246a83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8546a88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20346a8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96246a95, 85 },
    { 0x9ec46a96, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f46a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3746a9d, 93 },
    { 0xac446a9d, 61 },
    { 0x81e46a9f, 90 },
    { 0, 0 },
    { 0x9c946aa1, 90 },
    { 0xb3846aa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20346aa5, 34 },
    { 0x6246aa5, 52 },
    { 0, 0 },
    { 0x7a946aa8, 23 },
    { 0, 0 },
    { 0x8fc46aaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6046ab6, 33 },
    { 0xde046ab7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f346abc, 84 },
    { 0xb7546abc, 55 },
    { 0, 0 },
    { 0x89546abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e46ac3, 66 },
    { 0xe9d46ac4, 91 },
    { 0, 0 },
    { 0xf1546ac6, 95 },
    { 0x23746ac7, 52 },
    { 0xec046ac8, 46 },
    { 0xddb46ac6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b046acc, 87 },
    { 0xbab46acd, 91 },
    { 0x65b46acc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45346ad4, 90 },
    { 0x42946ad4, 33 },
    { 0, 0 },
    { 0x30f46ad7, 33 },
    { 0x53546ad7, 90 },
    { 0x60346ad9, 99 },
    { 0x8fd46ada, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8246add, 66 },
    { 0x64546ade, 34 },
    { 0xa6646adf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45146ae2, 33 },
    { 0x49246ae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3546ae8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26346aed, 33 },
    { 0xf7246aee, 43 },
    { 0, 0 },
    { 0xab346af0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec46af5, 46 },
    { 0xd6846af5, 34 },
    { 0xe7746af7, 98 },
    { 0xbf346af8, 90 },
    { 0xc5c46af5, 55 },
    { 0, 0 },
    { 0x6d346afb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca346aff, 90 },
    { 0xb3a46b00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f46b05, 93 },
    { 0x6a246b06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec46b0a, 34 },
    { 0xde446b0b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c146b0f, 98 },
    { 0, 0 },
    { 0x4146b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df46b15, 55 },
    { 0, 0 },
    { 0x7db46b17, 87 },
    { 0, 0 },
    { 0x28e46b19, 76 },
    { 0xf9b46b19, 64 },
    { 0x1e646b1b, 23 },
    { 0xdf746b1c, 33 },
    { 0x34b46b1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd46b20, 53 },
    { 0, 0 },
    { 0xd846b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47046b26, 93 },
    { 0xe1a46b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f46b2f, 90 },
    { 0x78c46b30, 34 },
    { 0, 0 },
    { 0x44346b32, 90 },
    { 0xd0746b33, 95 },
    { 0xe446b34, 33 },
    { 0x52a46b33, 61 },
    { 0xfdc46b35, 61 },
    { 0x3446b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e446b3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9646b45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c46b49, 76 },
    { 0xb3446b4a, 84 },
    { 0x72546b4a, 61 },
    { 0xb5b46b4b, 95 },
    { 0x84346b4a, 23 },
    { 0xcef46b4e, 33 },
    { 0x15246b4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d46b52, 95 },
    { 0x1f46b52, 55 },
    { 0, 0 },
    { 0x75d46b55, 61 },
    { 0x58a46b56, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f46b59, 55 },
    { 0, 0 },
    { 0xe6146b5b, 90 },
    { 0x7e46b5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad46b69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e46b6c, 93 },
    { 0, 0 },
    { 0xd2446b6e, 77 },
    { 0xb9d46b6e, 55 },
    { 0, 0 },
    { 0xfea46b71, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae046b75, 33 },
    { 0x95c46b75, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf46b7d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d46b84, 90 },
    { 0xc5d46b85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2146b88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64246b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7546b8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59746b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846b98, 34 },
    { 0x9d646b99, 99 },
    { 0xa1646b99, 23 },
    { 0x1c646b9b, 23 },
    { 0x9fc46b9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0446b9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c46ba4, 78 },
    { 0x86b46ba5, 98 },
    { 0, 0 },
    { 0x5b546ba7, 98 },
    { 0x3a146ba8, 34 },
    { 0xb2846ba9, 34 },
    { 0xcf946baa, 90 },
    { 0xdd846baa, 33 },
    { 0x64f46bab, 76 },
    { 0x35646baa, 61 },
    { 0x2f546bae, 61 },
    { 0xa6846baf, 80 },
    { 0x62f46bb0, 95 },
    { 0x41946bb1, 61 },
    { 0x8e46bae, 100 },
    { 0x5e746bab, 90 },
    { 0xa1546bb4, 84 },
    { 0xc4446bb4, 33 },
    { 0x8f346bb4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32146bbb, 70 },
    { 0x58446bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5046bbf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede46bc4, 55 },
    { 0, 0 },
    { 0x19c46bc6, 66 },
    { 0, 0 },
    { 0x33846bc8, 23 },
    { 0x91946bc8, 55 },
    { 0xde546bca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff46bcd, 93 },
    { 0x28246bce, 66 },
    { 0x47646bce, 33 },
    { 0x4a346bcd, 33 },
    { 0, 0 },
    { 0xb1446bd2, 46 },
    { 0x28e46bd3, 98 },
    { 0x7d246bd2, 90 },
    { 0, 0 },
    { 0x57646bd6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f246bda, 84 },
    { 0x47646bda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38246be0, 23 },
    { 0x52b46be0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c46beb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e46bee, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2046bf1, 33 },
    { 0x2dc46bf2, 33 },
    { 0, 0 },
    { 0xf1f46bf4, 76 },
    { 0, 0 },
    { 0x83346bf6, 95 },
    { 0, 0 },
    { 0xac946bf8, 33 },
    { 0xf2f46bf9, 33 },
    { 0, 0 },
    { 0x4246bfb, 61 },
    { 0x90b46bfc, 33 },
    { 0x1046bfc, 23 },
    { 0xbe046bfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e46c08, 90 },
    { 0, 0 },
    { 0xfca46c0a, 65 },
    { 0x54946c0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb46c0e, 81 },
    { 0x46a46c0f, 84 },
    { 0x58246c10, 83 },
    { 0xa3f46c11, 66 },
    { 0x55f46c10, 23 },
    { 0x95046c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e46c17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e346c1d, 95 },
    { 0xea446c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b746c23, 95 },
    { 0xe3346c24, 23 },
    { 0x1f746c23, 64 },
    { 0xfb746c26, 69 },
    { 0x18246c27, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb46c2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18746c2f, 66 },
    { 0x28d46c30, 99 },
    { 0x8f346c31, 23 },
    { 0xb7646c32, 90 },
    { 0x4a646c33, 23 },
    { 0x7a546c34, 55 },
    { 0xd5e46c35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e46c3e, 76 },
    { 0, 0 },
    { 0x46446c40, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88946c45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b846c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa246c50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146c53, 52 },
    { 0, 0 },
    { 0xc3546c55, 61 },
    { 0, 0 },
    { 0xe8446c57, 61 },
    { 0, 0 },
    { 0x57c46c59, 23 },
    { 0x1e146c59, 76 },
    { 0x25146c59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be46c60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e546c63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37646c67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f46c6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22146c74, 90 },
    { 0xfaa46c75, 95 },
    { 0xe3e46c76, 90 },
    { 0, 0 },
    { 0xb3646c78, 90 },
    { 0x81c46c78, 76 },
    { 0xd7c46c78, 55 },
    { 0x5bc46c7b, 90 },
    { 0xe5a46c78, 65 },
    { 0, 0 },
    { 0x8c646c7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f46c84, 98 },
    { 0x4ee46c85, 76 },
    { 0xb3046c86, 34 },
    { 0xa3b46c86, 90 },
    { 0x37c46c88, 61 },
    { 0xc0146c86, 76 },
    { 0x47246c89, 61 },
    { 0xbc246c8b, 66 },
    { 0x7ad46c86, 23 },
    { 0, 0 },
    { 0x6ee46c8e, 61 },
    { 0, 0 },
    { 0x57746c90, 95 },
    { 0, 0 },
    { 0x9e146c92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85546c95, 98 },
    { 0x59646c96, 43 },
    { 0x62046c97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c446c9b, 66 },
    { 0, 0 },
    { 0x98946c9d, 100 },
    { 0x64846c9e, 99 },
    { 0x20a46c9e, 55 },
    { 0, 0 },
    { 0xbb046ca1, 90 },
    { 0xd7346ca2, 78 },
    { 0, 0 },
    { 0xf5446ca4, 43 },
    { 0xaea46ca4, 36 },
    { 0x89b46ca6, 95 },
    { 0x97646ca7, 76 },
    { 0xb3146ca8, 99 },
    { 0x1b246ca6, 90 },
    { 0x7e046ca6, 23 },
    { 0x4dd46cab, 84 },
    { 0x33246cab, 67 },
    { 0x4a146cad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd446cb2, 90 },
    { 0xeb946cb3, 53 },
    { 0x4b246cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d46cb9, 95 },
    { 0x55d46cb9, 46 },
    { 0x4b146cb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x18046cbe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7946cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40946cc6, 90 },
    { 0x13846cc7, 98 },
    { 0x5d546cc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4846cce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96346cd4, 23 },
    { 0x36146cd4, 52 },
    { 0x3a546cd6, 33 },
    { 0, 0 },
    { 0xdf046cd8, 91 },
    { 0x2d046cd9, 81 },
    { 0x19a46cda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a46ce0, 33 },
    { 0xdd946ce0, 61 },
    { 0, 0 },
    { 0x90646ce3, 55 },
    { 0x33246ce4, 55 },
    { 0, 0 },
    { 0xdb046ce6, 95 },
    { 0x6bb46ce7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d46cea, 33 },
    { 0, 0 },
    { 0x37846cec, 23 },
    { 0x53146ced, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x78646cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3246cf6, 88 },
    { 0, 0 },
    { 0x59e46cf8, 90 },
    { 0, 0 },
    { 0xf4346cfa, 75 },
    { 0x1f146cfb, 98 },
    { 0, 0 },
    { 0x38446cfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57746d03, 99 },
    { 0, 0 },
    { 0x30746d05, 46 },
    { 0, 0 },
    { 0xbf546d07, 61 },
    { 0xc4846d07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce46d0b, 95 },
    { 0x2b146d0b, 68 },
    { 0x63d46d0b, 33 },
    { 0xa5b46d0e, 17 },
    { 0x69f46d0b, 43 },
    { 0x71f46d10, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb046d1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1546d20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x79646d23, 66 },
    { 0, 0 },
    { 0xfcc46d25, 23 },
    { 0xb9946d26, 95 },
    { 0xd8d46d27, 38 },
    { 0x88b46d25, 43 },
    { 0x7f846d29, 36 },
    { 0x22e46d2a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5246d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25846d31, 46 },
    { 0, 0 },
    { 0xc0646d33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d046d39, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb46d3e, 23 },
    { 0x27f46d3f, 85 },
    { 0xf4a46d40, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af46d49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58946d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf46d56, 100 },
    { 0, 0 },
    { 0xdf846d58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30846d5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85046d68, 93 },
    { 0x52e46d69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4946d77, 66 },
    { 0xe246d78, 90 },
    { 0x7c846d79, 98 },
    { 0, 0 },
    { 0x6146d7b, 99 },
    { 0, 0 },
    { 0x38446d7d, 76 },
    { 0xbe946d7d, 33 },
    { 0, 0 },
    { 0x3d246d80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e946d8d, 46 },
    { 0x8da46d8e, 84 },
    { 0xf0546d8e, 95 },
    { 0xcfa46d90, 55 },
    { 0x81e46d90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f46d95, 83 },
    { 0x6d846d96, 76 },
    { 0x4ee46d96, 34 },
    { 0x73f46d98, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d46d9b, 23 },
    { 0xe4a46d9c, 23 },
    { 0, 0 },
    { 0xb7146d9e, 23 },
    { 0x22c46d9e, 61 },
    { 0, 0 },
    { 0xb0446da1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88646da8, 95 },
    { 0x3ae46da8, 90 },
    { 0x49b46daa, 95 },
    { 0xb6546daa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba46dae, 69 },
    { 0xdb946dae, 91 },
    { 0, 0 },
    { 0x4e146db1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7946db4, 56 },
    { 0, 0 },
    { 0x73246db6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47346dbb, 93 },
    { 0x21046dbc, 55 },
    { 0xa3c46dbc, 90 },
    { 0xd8946dbe, 33 },
    { 0x1d046dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8646dc4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be46dc7, 90 },
    { 0, 0 },
    { 0x8e346dc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7746dcf, 95 },
    { 0, 0 },
    { 0xa8746dd1, 68 },
    { 0, 0 },
    { 0x25d46dd3, 90 },
    { 0, 0 },
    { 0xe6546dd5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4146dd8, 98 },
    { 0xe5846dd9, 76 },
    { 0xb5e46dd8, 76 },
    { 0xadc46dd8, 90 },
    { 0xca046dd9, 96 },
    { 0xc9046ddd, 95 },
    { 0xa6c46dde, 33 },
    { 0x36f46ddb, 43 },
    { 0, 0 },
    { 0xc4c46de1, 76 },
    { 0x24f46de2, 55 },
    { 0x28c46de3, 98 },
    { 0x3d946de2, 23 },
    { 0x3146de2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91746de9, 84 },
    { 0xf3146de9, 98 },
    { 0xaf046deb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db46def, 34 },
    { 0x53146def, 98 },
    { 0xcca46df0, 43 },
    { 0xfd746df0, 23 },
    { 0x82d46df1, 90 },
    { 0x92646df4, 95 },
    { 0x87d46df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a46df8, 100 },
    { 0xdfe46df8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef846dfc, 52 },
    { 0x3aa46dfd, 33 },
    { 0xfe46dfe, 85 },
    { 0x11246dff, 77 },
    { 0xf9c46e00, 61 },
    { 0, 0 },
    { 0xdbc46e02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f46e0c, 99 },
    { 0xb5d46e0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9646e1c, 76 },
    { 0xe1546e1d, 98 },
    { 0, 0 },
    { 0x27946e1f, 46 },
    { 0x42146e20, 66 },
    { 0xd6846e1f, 61 },
    { 0, 0 },
    { 0xf7f46e23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40146e27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2846e2c, 90 },
    { 0x22846e2d, 33 },
    { 0, 0 },
    { 0xe2646e2f, 95 },
    { 0x77946e30, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd46e39, 46 },
    { 0xbe646e39, 90 },
    { 0x9d946e3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f46e43, 67 },
    { 0, 0 },
    { 0x49946e45, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a946e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3646e51, 90 },
    { 0x8d246e51, 43 },
    { 0xaaf46e53, 84 },
    { 0, 0 },
    { 0x28246e55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2946e5a, 90 },
    { 0, 0 },
    { 0x5bc46e5c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b946e60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b46e63, 73 },
    { 0xc1946e63, 23 },
    { 0x30a46e65, 23 },
    { 0xb646e65, 33 },
    { 0x3a246e67, 52 },
    { 0xf4946e68, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32646e79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22146e80, 55 },
    { 0xcbf46e81, 46 },
    { 0x11146e82, 90 },
    { 0xb5846e83, 90 },
    { 0x3b046e83, 23 },
    { 0x86946e81, 33 },
    { 0xc1f46e80, 23 },
    { 0, 0 },
    { 0x59d46e88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42346e93, 33 },
    { 0, 0 },
    { 0xe246e95, 99 },
    { 0, 0 },
    { 0xbf646e97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf46e9a, 91 },
    { 0xe8346e9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e46ea6, 76 },
    { 0x2c146ea7, 76 },
    { 0xd0e46ea6, 98 },
    { 0, 0 },
    { 0x50846eaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce646eb4, 34 },
    { 0xc8646eb5, 33 },
    { 0x85146eb6, 23 },
    { 0x3d446eb6, 88 },
    { 0x7a46eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12746ebb, 98 },
    { 0xff946ebc, 34 },
    { 0, 0 },
    { 0x56446ebe, 55 },
    { 0, 0 },
    { 0x55e46ec0, 90 },
    { 0, 0 },
    { 0x96f46ec2, 34 },
    { 0, 0 },
    { 0xe0346ec4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac46ec8, 90 },
    { 0x21446ec9, 75 },
    { 0x2cd46eca, 90 },
    { 0, 0 },
    { 0x2f246ecc, 90 },
    { 0xa4046ecd, 34 },
    { 0x88f46ecc, 33 },
    { 0x21646ecf, 61 },
    { 0xb2446ece, 90 },
    { 0x4a46ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47946ed6, 90 },
    { 0x30646ed7, 34 },
    { 0x78546ed7, 66 },
    { 0, 0 },
    { 0xb5a46eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5246edf, 67 },
    { 0, 0 },
    { 0xe3b46ee1, 33 },
    { 0, 0 },
    { 0x82846ee3, 93 },
    { 0x5046ee3, 90 },
    { 0xba246ee5, 33 },
    { 0, 0 },
    { 0x22146ee7, 67 },
    { 0, 0 },
    { 0xb7446ee9, 95 },
    { 0x97a46ee9, 55 },
    { 0x2bf46eeb, 55 },
    { 0xbbf46eec, 55 },
    { 0xf7146ee9, 34 },
    { 0x10646eee, 38 },
    { 0x8fe46ee9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60946ef6, 90 },
    { 0xf5046ef7, 67 },
    { 0, 0 },
    { 0xa8046ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc446f01, 98 },
    { 0xef646f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c46f05, 61 },
    { 0, 0 },
    { 0xb7246f07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58946f0d, 90 },
    { 0x5c246f0e, 33 },
    { 0x46d46f0e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x58546f12, 91 },
    { 0x18d46f12, 55 },
    { 0x1c146f14, 98 },
    { 0x2de46f15, 95 },
    { 0x2c346f13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f46f19, 46 },
    { 0xcdf46f19, 33 },
    { 0xf0646f1a, 33 },
    { 0x59846f1c, 98 },
    { 0xeb746f1d, 61 },
    { 0xb8e46f1e, 76 },
    { 0xb5046f1e, 23 },
    { 0xfe246f20, 98 },
    { 0xf946f20, 33 },
    { 0x71246f22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44946f26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8346f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14446f30, 33 },
    { 0, 0 },
    { 0xb5f46f32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45246f3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d46f3e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e46f42, 67 },
    { 0x82046f43, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e46f49, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50046f4d, 95 },
    { 0, 0 },
    { 0x14a46f4f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25446f52, 23 },
    { 0x5d946f53, 84 },
    { 0xc5046f54, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72546f59, 90 },
    { 0x83946f5a, 23 },
    { 0x19446f5a, 61 },
    { 0x88846f5b, 55 },
    { 0x2c346f5d, 61 },
    { 0, 0 },
    { 0x13846f5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db46f62, 95 },
    { 0x56b46f63, 77 },
    { 0x1fc46f64, 46 },
    { 0, 0 },
    { 0x17446f66, 90 },
    { 0xd4946f67, 90 },
    { 0xcaf46f68, 91 },
    { 0x93a46f67, 52 },
    { 0x23646f6a, 23 },
    { 0xb2a46f6b, 45 },
    { 0, 0 },
    { 0xdc046f6d, 52 },
    { 0x78246f6d, 95 },
    { 0xa2b46f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb46f78, 33 },
    { 0xfdf46f79, 61 },
    { 0xdaa46f7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d46f7f, 78 },
    { 0xaaa46f80, 76 },
    { 0xaf146f81, 99 },
    { 0, 0 },
    { 0x9146f83, 90 },
    { 0xbfc46f83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1346f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61046f8c, 90 },
    { 0, 0 },
    { 0x11246f8e, 23 },
    { 0, 0 },
    { 0x8ba46f90, 61 },
    { 0x87f46f90, 100 },
    { 0xee046f92, 33 },
    { 0xac846f92, 87 },
    { 0xb4046f93, 81 },
    { 0xee046f93, 33 },
    { 0xf1746f96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2546f9d, 23 },
    { 0x1646f9e, 23 },
    { 0x16d46f9e, 95 },
    { 0, 0 },
    { 0x5e146fa1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d46fa9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87846fae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b46fb1, 33 },
    { 0x7ba46fb1, 98 },
    { 0x22c46fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a46fb9, 33 },
    { 0x2ed46fb9, 84 },
    { 0x60746fb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba046fbf, 55 },
    { 0, 0 },
    { 0xd1a46fc1, 52 },
    { 0x74046fc2, 61 },
    { 0xca446fc2, 34 },
    { 0, 0 },
    { 0xdd046fc5, 95 },
    { 0xc8f46fc5, 25 },
    { 0, 0 },
    { 0x8a146fc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf046fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8646fd1, 93 },
    { 0, 0 },
    { 0x3e446fd3, 36 },
    { 0x13146fd4, 38 },
    { 0x1c346fd5, 43 },
    { 0xf9d46fd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda746fdf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95846fe3, 66 },
    { 0x5ba46fe3, 76 },
    { 0xdf346fe5, 34 },
    { 0xb4346fe6, 95 },
    { 0xd8c46fe6, 23 },
    { 0xebd46fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98146feb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be46ff0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf46ff5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b46ff8, 95 },
    { 0xa4246ff9, 61 },
    { 0xbb546ffa, 40 },
    { 0x78d46ffb, 33 },
    { 0xc9446ffb, 68 },
    { 0, 0 },
    { 0x6c046ffe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f47001, 76 },
    { 0x2747002, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea47007, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30747010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a47015, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf47018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284701c, 66 },
    { 0x2314701c, 33 },
    { 0x8d34701e, 90 },
    { 0xf3f4701f, 23 },
    { 0, 0 },
    { 0x84947021, 46 },
    { 0xa7047022, 98 },
    { 0, 0 },
    { 0x51e47024, 90 },
    { 0x22c47025, 34 },
    { 0x2be47026, 33 },
    { 0x28d47027, 85 },
    { 0x16247026, 61 },
    { 0xcb947025, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a47032, 61 },
    { 0, 0 },
    { 0x32847034, 95 },
    { 0xe947035, 84 },
    { 0x46447036, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b4703b, 61 },
    { 0x17f4703c, 98 },
    { 0x3bb4703d, 61 },
    { 0x48c4703d, 78 },
    { 0xe204703d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d247044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7747049, 34 },
    { 0x8ee4704a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a47053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63247058, 23 },
    { 0x3f847059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d34705c, 84 },
    { 0, 0 },
    { 0x9cf4705e, 33 },
    { 0xfb04705f, 90 },
    { 0x347060, 23 },
    { 0, 0 },
    { 0x61647062, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef47065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd4706d, 84 },
    { 0xf264706e, 93 },
    { 0, 0 },
    { 0x93747070, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd547076, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e64707b, 77 },
    { 0, 0 },
    { 0x3f94707d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68147080, 61 },
    { 0x69f47080, 76 },
    { 0xad847082, 61 },
    { 0x73c47083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb847087, 61 },
    { 0x71347088, 85 },
    { 0xbbe47089, 55 },
    { 0xa4147088, 23 },
    { 0x25d47089, 23 },
    { 0x1b44708c, 23 },
    { 0, 0 },
    { 0x8d14708e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf47093, 75 },
    { 0xea47094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a4709c, 33 },
    { 0x96e4709c, 36 },
    { 0, 0 },
    { 0xe974709f, 100 },
    { 0xd5b470a0, 76 },
    { 0xcc3470a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307470a9, 46 },
    { 0, 0 },
    { 0x430470ab, 46 },
    { 0x113470ab, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5470b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8470c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9470c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e470cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb470d0, 76 },
    { 0xdb4470d1, 23 },
    { 0x177470d2, 23 },
    { 0x944470d1, 91 },
    { 0x385470d3, 98 },
    { 0x91d470d1, 43 },
    { 0x8d6470d6, 76 },
    { 0x81c470d6, 76 },
    { 0x5ea470d8, 76 },
    { 0xea4470d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43470e5, 88 },
    { 0x635470e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x425470e9, 61 },
    { 0x21d470e9, 55 },
    { 0x240470eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d470f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13747100, 77 },
    { 0xa2847101, 95 },
    { 0, 0 },
    { 0xa2147103, 34 },
    { 0x3147104, 23 },
    { 0, 0 },
    { 0x73447106, 98 },
    { 0, 0 },
    { 0x34047108, 34 },
    { 0x99b47109, 23 },
    { 0xd0e4710a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b4710e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67347111, 48 },
    { 0x7c847112, 33 },
    { 0x43e47111, 90 },
    { 0x6ae47114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc47117, 75 },
    { 0, 0 },
    { 0xfb547119, 76 },
    { 0, 0 },
    { 0xc044711b, 34 },
    { 0, 0 },
    { 0xb664711d, 98 },
    { 0x5a14711e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13547123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4947132, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa47139, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3854713e, 95 },
    { 0, 0 },
    { 0x81747140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf147143, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a447149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca4714c, 90 },
    { 0xac94714d, 23 },
    { 0xf754714e, 77 },
    { 0x7e54714f, 33 },
    { 0, 0 },
    { 0xea047151, 46 },
    { 0x6c147151, 96 },
    { 0, 0 },
    { 0x2a847154, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a84715d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0647162, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae47166, 90 },
    { 0x50c47167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc774716b, 52 },
    { 0x9854716c, 95 },
    { 0x4da4716c, 90 },
    { 0x4b44716e, 61 },
    { 0xf854716e, 67 },
    { 0x1184716b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6447177, 99 },
    { 0x17b47178, 76 },
    { 0xe5b47177, 34 },
    { 0xaf947179, 76 },
    { 0xdd47179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044717e, 34 },
    { 0xe244717f, 66 },
    { 0xc124717f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70747186, 61 },
    { 0, 0 },
    { 0xdf647188, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe124718b, 88 },
    { 0x77d4718c, 90 },
    { 0x344718c, 90 },
    { 0x4624718c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f47193, 93 },
    { 0xcc747194, 94 },
    { 0x3b547194, 84 },
    { 0x75d47196, 90 },
    { 0x18647197, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2114719a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa454719f, 52 },
    { 0xc744719f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51471a6, 90 },
    { 0xf6a471a6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94471aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be471b7, 93 },
    { 0, 0 },
    { 0x8ae471b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x544471bc, 81 },
    { 0xb23471bc, 80 },
    { 0xdb9471bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa471c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702471c8, 33 },
    { 0x52471c9, 90 },
    { 0x92d471ca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x336471cd, 95 },
    { 0, 0 },
    { 0xc21471cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b471de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53471e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b471ea, 76 },
    { 0, 0 },
    { 0x327471ec, 90 },
    { 0x9a471ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2471f0, 85 },
    { 0, 0 },
    { 0x305471f2, 100 },
    { 0x91c471f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x11471f6, 55 },
    { 0x655471f7, 77 },
    { 0xd66471f7, 90 },
    { 0xb70471f9, 90 },
    { 0xac2471f9, 97 },
    { 0xe7e471fb, 65 },
    { 0xac2471fb, 90 },
    { 0x430471fd, 61 },
    { 0x93d471fb, 98 },
    { 0x48f471fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e47202, 55 },
    { 0xcc047202, 93 },
    { 0, 0 },
    { 0x60d47205, 43 },
    { 0xc5a47206, 88 },
    { 0xe6947207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14720b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71947211, 87 },
    { 0, 0 },
    { 0xce547213, 90 },
    { 0x79547214, 33 },
    { 0xac947215, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb747218, 61 },
    { 0xc4947219, 81 },
    { 0, 0 },
    { 0x3b64721b, 61 },
    { 0x1654721c, 61 },
    { 0, 0 },
    { 0x8f94721e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e647221, 93 },
    { 0xd1e47221, 95 },
    { 0x93247223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6864722a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c64722e, 84 },
    { 0xeda4722f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0247232, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f47238, 90 },
    { 0x31147239, 33 },
    { 0xc3b47239, 61 },
    { 0x7e147238, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d647241, 69 },
    { 0xe3447241, 37 },
    { 0x6d447243, 77 },
    { 0x5447244, 55 },
    { 0x7447244, 33 },
    { 0x27847246, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af4724a, 98 },
    { 0x60a4724a, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90547252, 23 },
    { 0xa9147252, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b47256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63547259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd24725c, 40 },
    { 0x1664725d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c547260, 61 },
    { 0xf8e47261, 98 },
    { 0xbc947262, 76 },
    { 0, 0 },
    { 0x76f47264, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6684726f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca47274, 55 },
    { 0xff47275, 98 },
    { 0, 0 },
    { 0xc7e47277, 67 },
    { 0xfd347278, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc64727c, 46 },
    { 0x7cd4727d, 90 },
    { 0xbbc4727d, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e847283, 96 },
    { 0, 0 },
    { 0x3fb47285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb14728b, 17 },
    { 0x8594728c, 90 },
    { 0x5514728d, 33 },
    { 0x6e74728e, 95 },
    { 0xc4d4728f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1047296, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074729e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a472a8, 33 },
    { 0, 0 },
    { 0x56d472aa, 55 },
    { 0, 0 },
    { 0x3f0472ac, 43 },
    { 0, 0 },
    { 0x86b472ae, 98 },
    { 0x3ba472af, 31 },
    { 0x125472b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b472b6, 78 },
    { 0xcde472b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a472bb, 52 },
    { 0xe6c472bb, 90 },
    { 0, 0 },
    { 0x3d1472be, 91 },
    { 0xdf5472be, 34 },
    { 0x890472c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61472c6, 90 },
    { 0xa6f472c6, 96 },
    { 0xe3d472c8, 23 },
    { 0xcf6472c9, 93 },
    { 0xcfd472ca, 23 },
    { 0xf35472c6, 76 },
    { 0x1fa472cc, 34 },
    { 0, 0 },
    { 0xa35472ce, 90 },
    { 0x84b472cf, 95 },
    { 0, 0 },
    { 0xadd472d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd472db, 69 },
    { 0x8a6472dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41472e0, 33 },
    { 0x2bd472e0, 91 },
    { 0xbe1472e2, 33 },
    { 0, 0 },
    { 0x2e472e4, 23 },
    { 0x912472e5, 33 },
    { 0x8f2472e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49147300, 23 },
    { 0, 0 },
    { 0x14747302, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6147309, 55 },
    { 0xcae47309, 40 },
    { 0xaeb4730a, 23 },
    { 0x56a4730c, 43 },
    { 0x124730d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a347310, 98 },
    { 0xcdb47310, 61 },
    { 0x9ed47312, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63847317, 52 },
    { 0, 0 },
    { 0xe4147319, 76 },
    { 0, 0 },
    { 0x3e24731b, 67 },
    { 0x4364731b, 55 },
    { 0xc964731d, 43 },
    { 0, 0 },
    { 0x2f14731f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21247323, 55 },
    { 0x32547324, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b47327, 68 },
    { 0xeb447328, 95 },
    { 0x30947328, 61 },
    { 0xa1947329, 98 },
    { 0, 0 },
    { 0x9ee4732c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79147331, 84 },
    { 0xdc047332, 90 },
    { 0x21447333, 33 },
    { 0xca047333, 23 },
    { 0, 0 },
    { 0x4b047336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14733a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944733d, 43 },
    { 0x8f64733d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16747345, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c147349, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6954734d, 23 },
    { 0x59b4734e, 23 },
    { 0x4904734e, 34 },
    { 0xa924734f, 90 },
    { 0x75047351, 87 },
    { 0xbd147352, 33 },
    { 0x8ff47353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c4735d, 33 },
    { 0, 0 },
    { 0x5bb4735f, 59 },
    { 0x23147360, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc347363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a147367, 23 },
    { 0x5747367, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4544736c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1047376, 23 },
    { 0x3e447377, 61 },
    { 0, 0 },
    { 0xa5447379, 61 },
    { 0xd164737a, 82 },
    { 0xd7b4737b, 61 },
    { 0, 0 },
    { 0xf4737d, 95 },
    { 0xdf14737e, 33 },
    { 0, 0 },
    { 0x92447380, 46 },
    { 0xf9547381, 95 },
    { 0xb0847382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc747386, 76 },
    { 0x2b247386, 93 },
    { 0xdeb47388, 98 },
    { 0x90247389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2964738d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e47391, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c47396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1304739a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5473a0, 93 },
    { 0, 0 },
    { 0x640473a2, 65 },
    { 0x955473a2, 61 },
    { 0xa4473a2, 33 },
    { 0, 0 },
    { 0x1e473a6, 75 },
    { 0x31f473a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb473ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545473b0, 40 },
    { 0xdcb473b1, 68 },
    { 0, 0 },
    { 0x2f6473b3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf473c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a473c6, 23 },
    { 0x92d473c7, 90 },
    { 0x84e473c8, 95 },
    { 0xcb5473c9, 46 },
    { 0xc4d473ca, 99 },
    { 0x4ed473ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7473ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508473d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3473d5, 23 },
    { 0, 0 },
    { 0xc473d7, 90 },
    { 0, 0 },
    { 0xa94473d9, 95 },
    { 0xff2473da, 98 },
    { 0x2ed473db, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53473e7, 61 },
    { 0x19a473e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412473ef, 34 },
    { 0, 0 },
    { 0x706473f1, 84 },
    { 0, 0 },
    { 0x805473f3, 95 },
    { 0x852473f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340473f8, 78 },
    { 0x401473f9, 98 },
    { 0xce2473f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea47402, 55 },
    { 0x80f47403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e047406, 90 },
    { 0, 0 },
    { 0x44347408, 90 },
    { 0x81c47409, 87 },
    { 0xbb647408, 43 },
    { 0xe054740b, 95 },
    { 0x6f34740c, 36 },
    { 0x5d34740d, 77 },
    { 0x6964740e, 23 },
    { 0x1b44740f, 23 },
    { 0x51447410, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d947419, 84 },
    { 0xb674741a, 74 },
    { 0x44c47419, 76 },
    { 0xec14741c, 76 },
    { 0x90f4741d, 85 },
    { 0xfdd4741c, 90 },
    { 0x7474741f, 61 },
    { 0x87547420, 84 },
    { 0xb464741e, 95 },
    { 0x3724741c, 43 },
    { 0x4ef4741b, 44 },
    { 0x64547424, 40 },
    { 0, 0 },
    { 0x31347426, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4144742a, 23 },
    { 0x1044742a, 55 },
    { 0, 0 },
    { 0x7334742d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc47431, 55 },
    { 0x39247431, 55 },
    { 0, 0 },
    { 0x7b747434, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea47437, 88 },
    { 0x30847438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef4743b, 93 },
    { 0x5624743b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b47441, 87 },
    { 0x2b347442, 33 },
    { 0x44047443, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb4744a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad4744f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a47456, 23 },
    { 0, 0 },
    { 0xaba47458, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d4745b, 96 },
    { 0xe2f4745c, 76 },
    { 0xa7e4745c, 61 },
    { 0, 0 },
    { 0x8a24745f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8147462, 98 },
    { 0x17847463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18247466, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb134746a, 61 },
    { 0xc544746b, 61 },
    { 0xcf24746b, 100 },
    { 0, 0 },
    { 0xcdd4746e, 98 },
    { 0xcbd4746e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x62247472, 99 },
    { 0x94947473, 98 },
    { 0, 0 },
    { 0xd6447475, 68 },
    { 0, 0 },
    { 0x71f47477, 78 },
    { 0x4447478, 98 },
    { 0x5e047479, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe904747d, 76 },
    { 0xdb54747e, 23 },
    { 0xfef4747d, 61 },
    { 0x78947480, 98 },
    { 0, 0 },
    { 0x83c47482, 68 },
    { 0, 0 },
    { 0xb5e47484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c47489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d54748c, 68 },
    { 0x3ff4748c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8547491, 33 },
    { 0x92047492, 61 },
    { 0x8b447492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d47496, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec47499, 46 },
    { 0, 0 },
    { 0xf074749b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6474a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5474a8, 90 },
    { 0xa13474a9, 90 },
    { 0, 0 },
    { 0x9a6474ab, 100 },
    { 0, 0 },
    { 0xfe1474ad, 38 },
    { 0xe26474ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7474b4, 34 },
    { 0x759474b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1474bd, 24 },
    { 0x621474be, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758474c3, 76 },
    { 0, 0 },
    { 0xec0474c5, 98 },
    { 0, 0 },
    { 0xc51474c7, 95 },
    { 0xda474c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88474cd, 90 },
    { 0, 0 },
    { 0x258474cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3474d3, 91 },
    { 0x41d474d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458474da, 33 },
    { 0, 0 },
    { 0x27474dc, 90 },
    { 0, 0 },
    { 0xb7474de, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x32474e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b474e5, 33 },
    { 0x4c474e6, 95 },
    { 0xce4474e7, 95 },
    { 0xeed474e8, 90 },
    { 0x4f3474e8, 33 },
    { 0x7b474ea, 78 },
    { 0xb10474eb, 33 },
    { 0x3a5474e7, 43 },
    { 0x47c474e5, 61 },
    { 0xab7474ee, 40 },
    { 0x884474ef, 43 },
    { 0x6f5474e8, 90 },
    { 0x91c474f1, 33 },
    { 0xa54474f1, 90 },
    { 0x9fc474f3, 93 },
    { 0xb10474f4, 36 },
    { 0x81a474f5, 90 },
    { 0x4df474f6, 61 },
    { 0xfb3474f5, 90 },
    { 0x177474f8, 93 },
    { 0xbda474f2, 90 },
    { 0x2b2474e7, 67 },
    { 0, 0 },
    { 0x923474fc, 78 },
    { 0x2ea474fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc47500, 55 },
    { 0xb047500, 33 },
    { 0xd3447502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f47505, 90 },
    { 0, 0 },
    { 0x72e47507, 80 },
    { 0x7a647508, 33 },
    { 0x6b547509, 68 },
    { 0, 0 },
    { 0x9da4750b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e847510, 81 },
    { 0x66247510, 52 },
    { 0, 0 },
    { 0x26747513, 33 },
    { 0, 0 },
    { 0x6b47515, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb447518, 88 },
    { 0x18647518, 90 },
    { 0xa9a47518, 61 },
    { 0x18c47519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be4751f, 98 },
    { 0xf0247520, 55 },
    { 0, 0 },
    { 0xaee47522, 76 },
    { 0xdbe47522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b047528, 99 },
    { 0, 0 },
    { 0x4cd4752a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394752e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f447532, 90 },
    { 0x18b47533, 46 },
    { 0, 0 },
    { 0x62847535, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf234753a, 33 },
    { 0, 0 },
    { 0xc824753c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a47540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a47543, 90 },
    { 0xeb147543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b47548, 76 },
    { 0, 0 },
    { 0x7ef4754a, 23 },
    { 0x3a34754b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc94754e, 52 },
    { 0xe704754f, 91 },
    { 0, 0 },
    { 0xd9747551, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x49947554, 46 },
    { 0, 0 },
    { 0x80347556, 23 },
    { 0, 0 },
    { 0xddf47558, 17 },
    { 0xcda47559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f4755c, 61 },
    { 0xd514755d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47566, 33 },
    { 0x82447567, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1574756c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b47576, 76 },
    { 0x12147576, 55 },
    { 0xa5847578, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a4757b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47347583, 95 },
    { 0x38447584, 98 },
    { 0xc7047585, 34 },
    { 0xccc47586, 90 },
    { 0x1ca47586, 33 },
    { 0x84c47588, 90 },
    { 0x9cf47589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5784758c, 95 },
    { 0, 0 },
    { 0x3784758e, 85 },
    { 0xe424758f, 98 },
    { 0x2ac47590, 90 },
    { 0xc9147591, 23 },
    { 0x1ed47591, 61 },
    { 0xd947590, 95 },
    { 0, 0 },
    { 0x8447595, 37 },
    { 0, 0 },
    { 0xac547597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3654759d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e475a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d475a6, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5475a9, 93 },
    { 0xa5f475aa, 43 },
    { 0x4d8475ab, 90 },
    { 0x95a475ab, 95 },
    { 0xffc475ab, 98 },
    { 0x102475ae, 95 },
    { 0xeba475af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0475b6, 52 },
    { 0x92d475b6, 95 },
    { 0x16c475b8, 46 },
    { 0x153475b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x809475bc, 95 },
    { 0, 0 },
    { 0x287475be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80475c1, 90 },
    { 0x1ad475c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x936475c5, 23 },
    { 0x61c475c6, 33 },
    { 0xb6f475c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc475ca, 66 },
    { 0xfdd475cb, 52 },
    { 0x746475cc, 23 },
    { 0x8d4475ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67475d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc475d4, 90 },
    { 0x8e8475d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699475db, 90 },
    { 0, 0 },
    { 0xfe5475dd, 90 },
    { 0x74b475de, 33 },
    { 0, 0 },
    { 0xe73475e0, 98 },
    { 0x137475e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c475e4, 34 },
    { 0xab6475e5, 98 },
    { 0x411475e6, 66 },
    { 0xb08475e7, 76 },
    { 0x8a9475e8, 95 },
    { 0xeb7475e7, 61 },
    { 0x7f1475e7, 34 },
    { 0xf58475eb, 95 },
    { 0x846475ec, 33 },
    { 0, 0 },
    { 0xd1f475ee, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x596475f1, 95 },
    { 0x736475f2, 75 },
    { 0xb02475f3, 36 },
    { 0x39475f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e475fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2547604, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa47607, 34 },
    { 0x17047608, 76 },
    { 0, 0 },
    { 0xeb4760a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac47611, 98 },
    { 0, 0 },
    { 0x5d047613, 43 },
    { 0x2db47614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4761a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb14761d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae47624, 23 },
    { 0x31b47625, 33 },
    { 0, 0 },
    { 0x6dc47627, 95 },
    { 0x6e547627, 33 },
    { 0x47747629, 90 },
    { 0xde54762a, 75 },
    { 0, 0 },
    { 0x8714762c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92947630, 76 },
    { 0, 0 },
    { 0x9c147632, 76 },
    { 0x64647633, 76 },
    { 0x43247634, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b4763a, 95 },
    { 0xad24763a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c04763f, 81 },
    { 0xe5c47640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba047643, 91 },
    { 0x18d47644, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb4764a, 55 },
    { 0, 0 },
    { 0xb414764c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a47650, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x45347653, 78 },
    { 0, 0 },
    { 0xa9247655, 23 },
    { 0x12047655, 100 },
    { 0, 0 },
    { 0x9e547658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf244765b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8647663, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4754766b, 69 },
    { 0x45c4766b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a4766f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4447673, 34 },
    { 0x97747673, 95 },
    { 0, 0 },
    { 0x34a47676, 61 },
    { 0x30e47677, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b4767a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9894767e, 46 },
    { 0xe34767f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d47682, 61 },
    { 0, 0 },
    { 0xba347684, 66 },
    { 0x24047684, 33 },
    { 0x92247685, 76 },
    { 0x84d47686, 90 },
    { 0x29147688, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014768b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41247691, 52 },
    { 0x7ca47692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47697, 61 },
    { 0xa4447698, 33 },
    { 0xd7f47699, 85 },
    { 0xc884769a, 33 },
    { 0x7fe4769b, 99 },
    { 0xeb74769c, 78 },
    { 0xb2f4769d, 95 },
    { 0x5264769e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4476a2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0476ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b476af, 33 },
    { 0x168476b0, 76 },
    { 0x8f6476b1, 81 },
    { 0xf67476b0, 67 },
    { 0xf6c476b1, 33 },
    { 0x879476b4, 43 },
    { 0, 0 },
    { 0xdfe476b6, 98 },
    { 0x96a476b6, 61 },
    { 0, 0 },
    { 0x8c8476b9, 93 },
    { 0xbcb476ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44476c1, 67 },
    { 0x69d476c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600476c6, 76 },
    { 0xfde476c7, 90 },
    { 0, 0 },
    { 0x8f5476c9, 95 },
    { 0x26b476ca, 98 },
    { 0x878476ca, 34 },
    { 0x80476cc, 95 },
    { 0x4c6476cd, 98 },
    { 0xc67476cc, 17 },
    { 0x841476cf, 87 },
    { 0, 0 },
    { 0x5be476d1, 81 },
    { 0xb82476d2, 33 },
    { 0x6b4476d1, 43 },
    { 0, 0 },
    { 0x156476d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87476d8, 76 },
    { 0, 0 },
    { 0x74d476da, 66 },
    { 0, 0 },
    { 0x23a476dc, 93 },
    { 0, 0 },
    { 0x332476de, 28 },
    { 0x981476de, 33 },
    { 0x8b9476de, 96 },
    { 0xd7476df, 67 },
    { 0, 0 },
    { 0x86b476e3, 33 },
    { 0x790476e4, 90 },
    { 0x3b1476e5, 23 },
    { 0xde9476e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6476ea, 76 },
    { 0x271476ea, 90 },
    { 0x7ec476ec, 99 },
    { 0xc63476eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749476f7, 98 },
    { 0x9f9476f8, 75 },
    { 0xbdb476f9, 99 },
    { 0xfcf476fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4476fd, 95 },
    { 0, 0 },
    { 0x6bc476ff, 98 },
    { 0xedb47700, 33 },
    { 0xdd047700, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c47706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a44770c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc4770f, 98 },
    { 0x5ae47710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc47718, 61 },
    { 0x41547718, 23 },
    { 0, 0 },
    { 0x8f4771b, 66 },
    { 0, 0 },
    { 0xbfc4771d, 91 },
    { 0, 0 },
    { 0x8854771f, 46 },
    { 0xe8947720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a47726, 90 },
    { 0xdc447727, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d4772b, 98 },
    { 0x17d4772b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2847730, 61 },
    { 0xf3d47731, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a47734, 33 },
    { 0xd1647735, 93 },
    { 0, 0 },
    { 0x49b47737, 46 },
    { 0xc747738, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb94773c, 33 },
    { 0x9b84773d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c247743, 73 },
    { 0, 0 },
    { 0xa9247745, 98 },
    { 0xba547746, 61 },
    { 0, 0 },
    { 0x52b47748, 61 },
    { 0x73c47749, 91 },
    { 0x2c547749, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db47752, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc447756, 81 },
    { 0, 0 },
    { 0xe2e47758, 95 },
    { 0xc8647758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6244775e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f47765, 33 },
    { 0, 0 },
    { 0xbde47767, 78 },
    { 0xb4b47767, 95 },
    { 0xbb747768, 85 },
    { 0xc5f47767, 33 },
    { 0xa254776b, 99 },
    { 0x4a747767, 23 },
    { 0x50c4776d, 33 },
    { 0, 0 },
    { 0xa5a4776f, 90 },
    { 0xf4547770, 93 },
    { 0x2ff47770, 96 },
    { 0x65147770, 61 },
    { 0x57347771, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f147779, 23 },
    { 0, 0 },
    { 0x1fc4777b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b347783, 33 },
    { 0x13147784, 23 },
    { 0x17347784, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x27047788, 33 },
    { 0, 0 },
    { 0x8764778a, 33 },
    { 0x4614778b, 90 },
    { 0xa9d4778b, 90 },
    { 0xa0e4778c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1147790, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8947794, 98 },
    { 0, 0 },
    { 0x5dd47796, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f4779b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3477a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5477a8, 90 },
    { 0xebe477a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59477ac, 90 },
    { 0xa3d477ac, 76 },
    { 0, 0 },
    { 0x71b477af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89477b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8477bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2477bf, 43 },
    { 0, 0 },
    { 0xad9477c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28477c5, 33 },
    { 0xb7c477c6, 88 },
    { 0x65e477c7, 87 },
    { 0x6f4477c7, 66 },
    { 0x29477c8, 23 },
    { 0x31d477ca, 23 },
    { 0x903477ca, 17 },
    { 0x53b477cb, 33 },
    { 0xc34477cd, 65 },
    { 0x576477c6, 46 },
    { 0, 0 },
    { 0xb9f477d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed477d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a477de, 76 },
    { 0xfe0477df, 100 },
    { 0, 0 },
    { 0x78d477e1, 90 },
    { 0x4a477e2, 65 },
    { 0x125477e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233477e7, 95 },
    { 0x482477e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f477ec, 33 },
    { 0x7f9477ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4477f1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3477f4, 90 },
    { 0xeb3477f5, 52 },
    { 0xa5b477f6, 67 },
    { 0xa02477f7, 90 },
    { 0xc06477f7, 23 },
    { 0x3ba477f9, 99 },
    { 0x5f8477fa, 100 },
    { 0xae0477fa, 23 },
    { 0xc50477fc, 46 },
    { 0xe66477fb, 99 },
    { 0x673477fe, 85 },
    { 0xf9e477fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58747806, 99 },
    { 0xed847806, 98 },
    { 0xea947806, 46 },
    { 0x5dd47809, 34 },
    { 0x45a47809, 52 },
    { 0xefb4780b, 90 },
    { 0, 0 },
    { 0x1d64780d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e47814, 61 },
    { 0x55547815, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1847819, 87 },
    { 0x954781a, 61 },
    { 0xfcf4781a, 95 },
    { 0x46c47819, 55 },
    { 0, 0 },
    { 0x7dc4781e, 65 },
    { 0x8d14781f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a47822, 91 },
    { 0x15f47823, 52 },
    { 0xb3c47824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23747829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd304782f, 34 },
    { 0xcd947830, 95 },
    { 0x7e047831, 85 },
    { 0x22547831, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb47836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e4783c, 90 },
    { 0, 0 },
    { 0x6564783e, 40 },
    { 0, 0 },
    { 0x49d47840, 76 },
    { 0xe8847841, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b47848, 90 },
    { 0xb5847849, 55 },
    { 0x9dd4784a, 96 },
    { 0xe334784b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca47853, 95 },
    { 0, 0 },
    { 0x56347855, 69 },
    { 0, 0 },
    { 0x7947857, 55 },
    { 0xb5947858, 87 },
    { 0, 0 },
    { 0x924785a, 33 },
    { 0x5384785a, 95 },
    { 0, 0 },
    { 0x8194785d, 52 },
    { 0, 0 },
    { 0xa204785f, 33 },
    { 0, 0 },
    { 0xa7b47861, 98 },
    { 0xa9247862, 67 },
    { 0, 0 },
    { 0x89947864, 76 },
    { 0xa9547865, 33 },
    { 0x1a247866, 95 },
    { 0x93e47867, 90 },
    { 0x31b47864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74786b, 43 },
    { 0x7d84786c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9647871, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2664787e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48147883, 23 },
    { 0x6ab47884, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d4788a, 46 },
    { 0xdc4788b, 33 },
    { 0x7ab4788c, 91 },
    { 0x6d4788c, 95 },
    { 0x1744788d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f47892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43047895, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64147899, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf844789d, 34 },
    { 0xedb4789e, 90 },
    { 0x1ee4789f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x543478a2, 100 },
    { 0xe38478a3, 95 },
    { 0, 0 },
    { 0x812478a5, 61 },
    { 0xa4a478a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb478a9, 95 },
    { 0x6e478aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51478ad, 88 },
    { 0xfed478ad, 61 },
    { 0x498478af, 84 },
    { 0x716478ae, 52 },
    { 0, 0 },
    { 0xea0478b2, 90 },
    { 0x1fe478b2, 76 },
    { 0x847478b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1478b8, 81 },
    { 0xc08478b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a478c2, 93 },
    { 0xb70478c3, 52 },
    { 0xa1b478c3, 23 },
    { 0x3f9478c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723478cf, 90 },
    { 0x4ee478cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754478d4, 94 },
    { 0, 0 },
    { 0xf72478d6, 90 },
    { 0xdcb478d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd478da, 76 },
    { 0x7f1478db, 95 },
    { 0x5df478db, 90 },
    { 0x478db, 76 },
    { 0x37478dc, 76 },
    { 0x90c478da, 23 },
    { 0, 0 },
    { 0x664478e1, 52 },
    { 0, 0 },
    { 0x1d8478e3, 98 },
    { 0x26f478e4, 98 },
    { 0x181478e4, 98 },
    { 0x1bc478e6, 46 },
    { 0, 0 },
    { 0x63478e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee478f3, 76 },
    { 0xd40478f4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x500478f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d478fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53347900, 61 },
    { 0x30947900, 61 },
    { 0x4d647900, 95 },
    { 0, 0 },
    { 0x4cb47904, 76 },
    { 0xc0b47904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2554790a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xab14790d, 43 },
    { 0, 0 },
    { 0x62d4790f, 33 },
    { 0x3184790f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88447916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe47919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb4791c, 78 },
    { 0x9d24791d, 90 },
    { 0, 0 },
    { 0xb7a4791f, 17 },
    { 0x6f547920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51247923, 78 },
    { 0, 0 },
    { 0x11e47925, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074792d, 61 },
    { 0x2c74792e, 76 },
    { 0, 0 },
    { 0x5fc47930, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8247934, 33 },
    { 0x94747934, 38 },
    { 0, 0 },
    { 0x19b47937, 23 },
    { 0, 0 },
    { 0x93a47939, 33 },
    { 0x43d4793a, 85 },
    { 0xfd04793b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8094793f, 23 },
    { 0x80747940, 95 },
    { 0, 0 },
    { 0x36047942, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11247947, 61 },
    { 0, 0 },
    { 0x47b47949, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9904794d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b47950, 76 },
    { 0, 0 },
    { 0xc2f47952, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8147956, 34 },
    { 0x11c47957, 33 },
    { 0x2d247957, 90 },
    { 0xd6647959, 76 },
    { 0, 0 },
    { 0xcb24795b, 61 },
    { 0x32a4795c, 55 },
    { 0xcf94795d, 55 },
    { 0x4794795e, 33 },
    { 0xa914795f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8047963, 76 },
    { 0xb4447964, 61 },
    { 0xdf847965, 38 },
    { 0x87947963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72547969, 46 },
    { 0x2004796a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8447970, 43 },
    { 0xcb547971, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8847974, 33 },
    { 0x63847975, 95 },
    { 0x90747976, 88 },
    { 0, 0 },
    { 0xd6847978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d4797e, 88 },
    { 0, 0 },
    { 0x67047980, 90 },
    { 0, 0 },
    { 0x9bd47982, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6147987, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c44798a, 43 },
    { 0, 0 },
    { 0x434798c, 100 },
    { 0x5ea4798d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f647993, 46 },
    { 0x65747994, 85 },
    { 0xb2847995, 55 },
    { 0x9c347996, 87 },
    { 0x6e647993, 23 },
    { 0, 0 },
    { 0x44f47999, 23 },
    { 0x7e047999, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4799f, 61 },
    { 0x459479a0, 23 },
    { 0x4f6479a0, 90 },
    { 0, 0 },
    { 0x5ae479a3, 34 },
    { 0x4e479a3, 17 },
    { 0xeef479a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6479a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154479ae, 95 },
    { 0x921479af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec479b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d479b9, 90 },
    { 0xfe9479b9, 95 },
    { 0x34d479b9, 33 },
    { 0x33b479bb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34479bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f479c6, 91 },
    { 0xe3479c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b479cb, 77 },
    { 0x80a479cc, 33 },
    { 0x85479cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d479d6, 90 },
    { 0x16a479d7, 90 },
    { 0xfca479d8, 53 },
    { 0x37479d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f479e1, 77 },
    { 0, 0 },
    { 0xd89479e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b479e6, 90 },
    { 0xd9a479e7, 33 },
    { 0x595479e8, 98 },
    { 0xb51479e8, 98 },
    { 0xec7479ea, 76 },
    { 0xd08479eb, 33 },
    { 0x750479eb, 23 },
    { 0xb54479ed, 76 },
    { 0x350479ed, 90 },
    { 0xe98479ea, 90 },
    { 0x8fd479e6, 90 },
    { 0xe5c479f1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00479f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c479fb, 55 },
    { 0xb27479fb, 55 },
    { 0, 0 },
    { 0x7db479fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d847a02, 75 },
    { 0x78447a02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff147a0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb47a0f, 46 },
    { 0xd5347a10, 90 },
    { 0x4b147a11, 55 },
    { 0x70247a12, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4447a15, 77 },
    { 0, 0 },
    { 0xf3247a17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e047a21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca247a25, 34 },
    { 0, 0 },
    { 0x76947a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14947a2a, 95 },
    { 0xbb947a2b, 76 },
    { 0x51247a2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c47a2f, 33 },
    { 0, 0 },
    { 0xc5747a31, 33 },
    { 0x9df47a31, 61 },
    { 0xcb347a33, 90 },
    { 0xd7a47a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b47a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53047a41, 52 },
    { 0x74947a42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3847a49, 100 },
    { 0x3b747a49, 90 },
    { 0x5aa47a4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2847a52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20947a55, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x25447a58, 17 },
    { 0, 0 },
    { 0xfbd47a5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36047a62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1947a66, 95 },
    { 0, 0 },
    { 0x35e47a68, 90 },
    { 0, 0 },
    { 0x66347a6a, 77 },
    { 0x35d47a6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e47a74, 33 },
    { 0xb1847a75, 87 },
    { 0x18247a75, 95 },
    { 0x4f647a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b47a7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f47a82, 90 },
    { 0, 0 },
    { 0x48247a84, 48 },
    { 0x92247a85, 36 },
    { 0x8247a86, 52 },
    { 0x66547a86, 33 },
    { 0x96247a85, 95 },
    { 0xefb47a89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47a8d, 96 },
    { 0xc8b47a8e, 75 },
    { 0xca547a8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70347a94, 34 },
    { 0x8f047a95, 77 },
    { 0x5ab47a95, 66 },
    { 0x5bd47a97, 98 },
    { 0, 0 },
    { 0x59747a99, 90 },
    { 0x1db47a9a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47aa0, 17 },
    { 0, 0 },
    { 0x78447aa2, 40 },
    { 0, 0 },
    { 0x77247aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e47aab, 33 },
    { 0xd9347aab, 96 },
    { 0, 0 },
    { 0x82347aae, 33 },
    { 0xd0e47aaf, 91 },
    { 0, 0 },
    { 0x7bb47ab1, 78 },
    { 0xe6d47ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a47ab9, 23 },
    { 0x5f847aba, 69 },
    { 0xe7e47abb, 98 },
    { 0xe6c47abb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad47ac5, 55 },
    { 0, 0 },
    { 0x68847ac7, 55 },
    { 0x1f847ac7, 67 },
    { 0xfb047ac8, 46 },
    { 0x4ab47aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4347ace, 90 },
    { 0xc9e47acf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b47ad2, 23 },
    { 0xa7e47ad2, 85 },
    { 0x6ab47ad4, 95 },
    { 0x77447ad4, 43 },
    { 0, 0 },
    { 0xa0f47ad7, 76 },
    { 0x86547ad8, 98 },
    { 0x87d47ad9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe947adc, 68 },
    { 0xda847add, 95 },
    { 0x4cc47ade, 69 },
    { 0, 0 },
    { 0xcb747ae0, 46 },
    { 0xeb847ae0, 95 },
    { 0x7bb47ae0, 61 },
    { 0xc6947ae3, 95 },
    { 0x70747ae3, 23 },
    { 0x3d947ae1, 33 },
    { 0x71147ae4, 33 },
    { 0x3fb47ae7, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xea347aea, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc47af0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c47af5, 55 },
    { 0x27e47af6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9847af9, 23 },
    { 0x5d47afa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3747b00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd447b04, 95 },
    { 0x83047b05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea47b0a, 76 },
    { 0, 0 },
    { 0x54447b0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf47b10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba47b14, 76 },
    { 0x17647b15, 76 },
    { 0x3ab47b16, 100 },
    { 0xd6947b15, 23 },
    { 0x5b247b15, 23 },
    { 0x6c47b19, 90 },
    { 0xf9d47b1a, 46 },
    { 0xdf747b19, 66 },
    { 0x7d447b16, 34 },
    { 0xada47b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e47b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f47b30, 95 },
    { 0x79e47b31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d47b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea47b3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a47b40, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c047b44, 47 },
    { 0xd5e47b45, 33 },
    { 0xa5e47b45, 57 },
    { 0xb2047b47, 95 },
    { 0, 0 },
    { 0x90747b49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac47b4c, 95 },
    { 0x82047b4c, 95 },
    { 0x2b147b4e, 83 },
    { 0x95347b4f, 55 },
    { 0xd2a47b4f, 95 },
    { 0, 0 },
    { 0x67b47b52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1947b56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d847b59, 23 },
    { 0x19147b5a, 98 },
    { 0xf3a47b59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3247b60, 76 },
    { 0, 0 },
    { 0x11247b62, 61 },
    { 0, 0 },
    { 0xc3e47b64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0647b6f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6947b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf847b79, 91 },
    { 0xdf247b7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547b7e, 55 },
    { 0, 0 },
    { 0xeef47b80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a47b8d, 33 },
    { 0xe2847b8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef47b91, 90 },
    { 0x77747b91, 46 },
    { 0x24847b92, 55 },
    { 0xee147b94, 77 },
    { 0xa7447b95, 77 },
    { 0xe5d47b94, 43 },
    { 0xd1b47b93, 90 },
    { 0x39547b93, 55 },
    { 0, 0 },
    { 0x6b547b9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6647b9f, 80 },
    { 0x78547ba0, 77 },
    { 0x2c047ba0, 76 },
    { 0xa5247ba2, 36 },
    { 0x11847ba3, 90 },
    { 0x20747ba4, 87 },
    { 0x70a47ba1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24647ba9, 98 },
    { 0, 0 },
    { 0xb2a47bab, 69 },
    { 0, 0 },
    { 0xf1a47bad, 95 },
    { 0x85147bad, 90 },
    { 0xf8247bae, 37 },
    { 0, 0 },
    { 0xed447bb1, 61 },
    { 0x7e047bb2, 55 },
    { 0xc1a47bb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62647bb7, 95 },
    { 0, 0 },
    { 0xee847bb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c47bbc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1547bc6, 67 },
    { 0xa5f47bc7, 90 },
    { 0, 0 },
    { 0x74a47bc9, 93 },
    { 0x3d447bca, 17 },
    { 0x10947bca, 98 },
    { 0x3d247bcb, 84 },
    { 0xd9a47bcd, 33 },
    { 0x7e547bcd, 98 },
    { 0xef747bcf, 68 },
    { 0x57347bd0, 88 },
    { 0x24c47bce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4147bd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65447bd9, 55 },
    { 0x8bb47bda, 43 },
    { 0x7bc47bdb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee47bdf, 90 },
    { 0xd447be0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7147be3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b47be6, 33 },
    { 0, 0 },
    { 0x15247be8, 79 },
    { 0x1ea47be9, 76 },
    { 0x89547bea, 61 },
    { 0xf0a47beb, 33 },
    { 0x15247be9, 90 },
    { 0x4ec47bed, 85 },
    { 0, 0 },
    { 0xad447bef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa747bf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0947bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d547bfc, 38 },
    { 0x92a47bfc, 55 },
    { 0xdde47bfe, 33 },
    { 0x8e547bfe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1247c06, 55 },
    { 0x98d47c07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c47c0e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7947c11, 33 },
    { 0, 0 },
    { 0x1c47c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c47c18, 34 },
    { 0xd5947c19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0847c1f, 23 },
    { 0x64f47c1f, 98 },
    { 0xe8b47c21, 52 },
    { 0x48647c22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f47c28, 66 },
    { 0x9e147c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b747c2f, 98 },
    { 0xd4a47c30, 98 },
    { 0xafe47c2f, 90 },
    { 0, 0 },
    { 0x8c47c33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec747c36, 95 },
    { 0, 0 },
    { 0x15c47c38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b47c40, 38 },
    { 0xbed47c40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42847c48, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae47c4b, 95 },
    { 0x1b247c4c, 65 },
    { 0xf8247c4c, 66 },
    { 0xaa247c4e, 90 },
    { 0x50947c4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c47c59, 84 },
    { 0, 0 },
    { 0x9cc47c5b, 23 },
    { 0x5f947c5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be47c5f, 96 },
    { 0xec47c5f, 34 },
    { 0xf5747c60, 61 },
    { 0, 0 },
    { 0xef447c63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49247c66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4647c69, 98 },
    { 0xfbb47c6a, 95 },
    { 0x62647c6b, 43 },
    { 0x76747c6a, 43 },
    { 0, 0 },
    { 0x36747c6e, 61 },
    { 0xdb547c6f, 34 },
    { 0x94747c6e, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30247c78, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8047c7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94347c81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d47c86, 61 },
    { 0x13347c87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b47c8e, 33 },
    { 0xe0647c8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b047c92, 95 },
    { 0xc6047c93, 88 },
    { 0, 0 },
    { 0xc3147c95, 90 },
    { 0x1b147c95, 85 },
    { 0xa4947c95, 61 },
    { 0x26247c98, 34 },
    { 0, 0 },
    { 0x21547c9a, 98 },
    { 0, 0 },
    { 0xef747c9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a647ca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b47ca5, 78 },
    { 0xb9747ca6, 98 },
    { 0xec947ca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86447caa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62147cad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc47cb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac47cb7, 98 },
    { 0x67d47cb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba47cbd, 77 },
    { 0x98947cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a47cc9, 98 },
    { 0xa8b47cca, 90 },
    { 0x38b47cca, 34 },
    { 0, 0 },
    { 0xcc447ccd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c47cd1, 67 },
    { 0x42b47cd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4547cd6, 95 },
    { 0x8c347cd6, 55 },
    { 0, 0 },
    { 0x5b147cd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c47cde, 90 },
    { 0x27047cdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2647ce8, 23 },
    { 0, 0 },
    { 0xe0047cea, 46 },
    { 0x8f747ceb, 98 },
    { 0xf7847cec, 33 },
    { 0x8647cec, 98 },
    { 0x50847cea, 61 },
    { 0xf4947cee, 66 },
    { 0x65847ceb, 64 },
    { 0xbea47cef, 90 },
    { 0, 0 },
    { 0xaf047cf3, 40 },
    { 0x3c447cf3, 52 },
    { 0, 0 },
    { 0xf9847cf6, 55 },
    { 0, 0 },
    { 0x56d47cf8, 60 },
    { 0x50447cf9, 95 },
    { 0x63747cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7347cfd, 90 },
    { 0, 0 },
    { 0x74c47cff, 90 },
    { 0x69a47d00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86247d08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0847d0b, 90 },
    { 0x64b47d0c, 77 },
    { 0, 0 },
    { 0x51c47d0e, 23 },
    { 0x9f147d0e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46647d17, 76 },
    { 0x4ec47d18, 33 },
    { 0xc2247d19, 33 },
    { 0xaff47d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d47d21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c47d25, 33 },
    { 0x4a647d25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2747d2c, 98 },
    { 0xd1e47d2d, 34 },
    { 0, 0 },
    { 0x4f047d2f, 93 },
    { 0x84347d2f, 33 },
    { 0x7fc47d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a47d35, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c847d38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3647d42, 96 },
    { 0xa9847d43, 95 },
    { 0x1a647d44, 64 },
    { 0, 0 },
    { 0x28a47d46, 23 },
    { 0, 0 },
    { 0x42e47d48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d47d4b, 61 },
    { 0xaed47d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9947d53, 91 },
    { 0x1e947d54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67147d5c, 90 },
    { 0x30047d5d, 90 },
    { 0x4ac47d5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e47d61, 85 },
    { 0xce747d62, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a47d65, 76 },
    { 0xbb947d65, 87 },
    { 0x6c447d67, 81 },
    { 0xa2447d67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x17247d6b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e47d6f, 61 },
    { 0, 0 },
    { 0xbed47d71, 55 },
    { 0x70047d72, 46 },
    { 0x70947d72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b547d77, 33 },
    { 0x24f47d77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf947d7d, 95 },
    { 0x59247d7e, 87 },
    { 0x47b47d7f, 90 },
    { 0x89747d80, 34 },
    { 0, 0 },
    { 0x24347d82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f47d87, 98 },
    { 0x97f47d87, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7847d8f, 98 },
    { 0, 0 },
    { 0xee247d91, 95 },
    { 0x7dd47d92, 55 },
    { 0x1d47d93, 61 },
    { 0xb8d47d94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af47d97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda47d9c, 61 },
    { 0x73347d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f47da1, 98 },
    { 0, 0 },
    { 0x2347da3, 76 },
    { 0, 0 },
    { 0xc1747da5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44947dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59447db6, 46 },
    { 0, 0 },
    { 0x39547db8, 84 },
    { 0xf2e47db9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e347dbe, 76 },
    { 0xc9947dbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7447dc2, 23 },
    { 0x16647dc2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d47dc8, 98 },
    { 0x4347dc9, 98 },
    { 0x14747dca, 98 },
    { 0x24347dc9, 91 },
    { 0xbfc47dcc, 81 },
    { 0xfba47dcd, 90 },
    { 0, 0 },
    { 0xc4247dcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb947dd2, 98 },
    { 0x12f47dd2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc847dd6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81247ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0147dde, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0447de1, 61 },
    { 0, 0 },
    { 0xe2947de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e47dea, 91 },
    { 0x9c047dea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1747dee, 100 },
    { 0, 0 },
    { 0xe2247df0, 66 },
    { 0xb1e47df1, 81 },
    { 0x2f047df0, 33 },
    { 0x9e647df1, 46 },
    { 0x1e347df0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b847df7, 76 },
    { 0, 0 },
    { 0x7e647df9, 68 },
    { 0x48947dfa, 90 },
    { 0x8e047dfb, 100 },
    { 0x4e347dfc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f047e02, 95 },
    { 0, 0 },
    { 0x1d047e04, 23 },
    { 0xa7e47e05, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e547e0b, 98 },
    { 0xca147e0c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff47e10, 33 },
    { 0, 0 },
    { 0x89447e12, 55 },
    { 0x78747e13, 90 },
    { 0xb4247e14, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7347e17, 68 },
    { 0xec447e17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c47e1b, 61 },
    { 0x9c947e1c, 23 },
    { 0xf6c47e1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c847e21, 43 },
    { 0x9f247e22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c47e25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a47e28, 66 },
    { 0x7da47e28, 90 },
    { 0x3c947e29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34447e32, 23 },
    { 0xfd547e33, 33 },
    { 0xf9f47e32, 84 },
    { 0xc0a47e34, 66 },
    { 0xa4147e36, 55 },
    { 0xc8747e37, 61 },
    { 0x8b147e35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1147e41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b247e45, 61 },
    { 0xff447e46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b047e49, 95 },
    { 0x5c647e49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa447e55, 80 },
    { 0x9dc47e56, 97 },
    { 0, 0 },
    { 0x86547e58, 55 },
    { 0, 0 },
    { 0x2a247e5a, 33 },
    { 0x5e247e5b, 90 },
    { 0xcd647e5b, 43 },
    { 0xbb147e5a, 95 },
    { 0xb1147e5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11047e61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa47e64, 95 },
    { 0x9b247e65, 46 },
    { 0x44847e66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be47e6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b47e6f, 76 },
    { 0, 0 },
    { 0x36e47e71, 81 },
    { 0xf5f47e72, 33 },
    { 0xae147e71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36047e76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35147e7b, 93 },
    { 0xb47e7b, 98 },
    { 0xc6447e7c, 33 },
    { 0, 0 },
    { 0x2ff47e7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2147e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3247e86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67547e89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd347e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91347e90, 33 },
    { 0x3c847e91, 96 },
    { 0x11147e90, 90 },
    { 0xe6a47e93, 81 },
    { 0x88f47e93, 75 },
    { 0x7f347e94, 90 },
    { 0x7ba47e90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f47e9a, 88 },
    { 0x69247e9b, 31 },
    { 0x8c847e9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e947ea0, 95 },
    { 0x86547ea1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69147ea5, 76 },
    { 0x43747ea6, 23 },
    { 0, 0 },
    { 0x2f47ea8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef47eac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7647eaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8347ebb, 78 },
    { 0x55447ebc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb47ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15847ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af47eca, 91 },
    { 0x87547ecb, 93 },
    { 0, 0 },
    { 0x1e147ecd, 78 },
    { 0xbe247ece, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d047ed3, 99 },
    { 0, 0 },
    { 0x9bf47ed5, 95 },
    { 0, 0 },
    { 0x11d47ed7, 43 },
    { 0x94647ed8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db47edf, 69 },
    { 0x6ab47edf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2547ee3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8547ee7, 55 },
    { 0x77347ee8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a47eeb, 46 },
    { 0, 0 },
    { 0xd2047eed, 88 },
    { 0xbe247eee, 85 },
    { 0, 0 },
    { 0xd4c47ef0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa47ef5, 80 },
    { 0xfc47ef6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28147efc, 61 },
    { 0x31d47efc, 43 },
    { 0x38547efe, 31 },
    { 0x2c347eff, 34 },
    { 0x56d47efe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547f06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b247f0a, 33 },
    { 0xd8c47f0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68847f0e, 43 },
    { 0xf2147f0e, 90 },
    { 0, 0 },
    { 0xf7047f11, 92 },
    { 0x29747f11, 69 },
    { 0, 0 },
    { 0x63347f14, 95 },
    { 0x19b47f15, 95 },
    { 0x6947f15, 98 },
    { 0x25647f15, 33 },
    { 0x35847f15, 95 },
    { 0, 0 },
    { 0x1e647f1a, 52 },
    { 0xa7947f1a, 90 },
    { 0, 0 },
    { 0x34747f1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f47f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f47f2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c47f2d, 55 },
    { 0, 0 },
    { 0xc6e47f2f, 76 },
    { 0, 0 },
    { 0xa6947f31, 98 },
    { 0xff047f31, 90 },
    { 0, 0 },
    { 0xfb847f34, 95 },
    { 0x7847f35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4047f40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca47f43, 61 },
    { 0x73a47f44, 52 },
    { 0xac347f45, 90 },
    { 0xc2a47f46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80547f4b, 98 },
    { 0, 0 },
    { 0xe47f4d, 33 },
    { 0, 0 },
    { 0x16b47f4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b47f52, 55 },
    { 0x12747f53, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d347f57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a47f5e, 68 },
    { 0, 0 },
    { 0xc5747f60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58047f64, 98 },
    { 0, 0 },
    { 0xf3147f66, 33 },
    { 0x9d47f67, 55 },
    { 0xae447f68, 98 },
    { 0x99b47f69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96747f72, 55 },
    { 0x25947f73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f347f76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e47f7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6947f81, 23 },
    { 0x1d547f82, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x21947f85, 66 },
    { 0xd3747f85, 93 },
    { 0xcc647f87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x12347f8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b47f90, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa47f9b, 95 },
    { 0xf7d47f9c, 87 },
    { 0, 0 },
    { 0x4ba47f9e, 55 },
    { 0x5247f9e, 93 },
    { 0, 0 },
    { 0x6bd47fa1, 80 },
    { 0xf347fa2, 90 },
    { 0x79b47fa3, 98 },
    { 0, 0 },
    { 0x7147fa5, 81 },
    { 0xaa047fa6, 95 },
    { 0xf1847fa6, 23 },
    { 0x92247fa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6947faf, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b47fb3, 40 },
    { 0, 0 },
    { 0x6c747fb5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e47fbb, 98 },
    { 0x8c47fbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb47fc0, 85 },
    { 0, 0 },
    { 0x13c47fc2, 90 },
    { 0xd6247fc2, 87 },
    { 0x34347fc4, 76 },
    { 0x7ef47fc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7547fd1, 43 },
    { 0xe1847fd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c47fd9, 95 },
    { 0xdef47fd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21047fdd, 23 },
    { 0xbdc47fde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e47fe4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc747fe8, 43 },
    { 0x54147fe8, 52 },
    { 0, 0 },
    { 0xcb947feb, 46 },
    { 0x18547fec, 61 },
    { 0x8f047fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c347ff1, 90 },
    { 0x89647ff2, 90 },
    { 0xd2847ff3, 90 },
    { 0, 0 },
    { 0xba547ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a47ff8, 90 },
    { 0xd2547ff8, 61 },
    { 0xc9c47ffa, 23 },
    { 0, 0 },
    { 0xc9847ffc, 34 },
    { 0xd8147ffd, 40 },
    { 0xa8d47ffe, 34 },
    { 0x6e947ffd, 61 },
    { 0x51447ffc, 55 },
    { 0x1b448001, 95 },
    { 0x99547ffd, 33 },
    { 0, 0 },
    { 0x46248004, 34 },
    { 0xf2048004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc48009, 55 },
    { 0, 0 },
    { 0xba64800b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e148012, 84 },
    { 0x80148012, 17 },
    { 0xc7348013, 33 },
    { 0x46448014, 61 },
    { 0xaaa48015, 91 },
    { 0xc1348017, 93 },
    { 0, 0 },
    { 0x84148019, 61 },
    { 0x93a4801a, 90 },
    { 0x7434801a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb48023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2848027, 43 },
    { 0, 0 },
    { 0x40448029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed84802c, 76 },
    { 0, 0 },
    { 0xfcd4802e, 98 },
    { 0x1094802f, 76 },
    { 0x3d248030, 33 },
    { 0xec948031, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79248035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69948039, 90 },
    { 0x8894803a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe4803d, 91 },
    { 0x5084803e, 78 },
    { 0x4f84803f, 76 },
    { 0x18648040, 98 },
    { 0x71548041, 55 },
    { 0x9324803e, 33 },
    { 0xb9548043, 34 },
    { 0x67148043, 34 },
    { 0x20f48045, 43 },
    { 0xb9c4803e, 61 },
    { 0, 0 },
    { 0xa8748048, 90 },
    { 0x80748049, 61 },
    { 0x8cb4804a, 55 },
    { 0x7384804b, 33 },
    { 0x9cd4804c, 85 },
    { 0xc194804b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d248050, 95 },
    { 0xdc748051, 95 },
    { 0x6c848052, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c948055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0848058, 90 },
    { 0x36b48058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734805e, 66 },
    { 0xdec4805e, 90 },
    { 0, 0 },
    { 0xbcb48061, 95 },
    { 0xeb848062, 55 },
    { 0, 0 },
    { 0x86c48064, 95 },
    { 0, 0 },
    { 0xa0f48066, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b4806b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e4806f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da48073, 91 },
    { 0x21f48074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca48078, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a48080, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef48084, 90 },
    { 0x73948085, 67 },
    { 0xcb648084, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2084808c, 95 },
    { 0xf804808d, 90 },
    { 0x6574808e, 95 },
    { 0xbe54808f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f48093, 95 },
    { 0, 0 },
    { 0xb7748095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa48098, 90 },
    { 0xc948098, 33 },
    { 0xe134809a, 76 },
    { 0, 0 },
    { 0x31f4809c, 61 },
    { 0, 0 },
    { 0x1674809e, 33 },
    { 0x3db4809f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c480a3, 33 },
    { 0x662480a4, 23 },
    { 0xe09480a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34480a8, 33 },
    { 0x14b480a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2480ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x657480b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4480b4, 61 },
    { 0, 0 },
    { 0x87d480b6, 82 },
    { 0xdc5480b7, 98 },
    { 0x94480b8, 43 },
    { 0xbb4480b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0480bc, 80 },
    { 0, 0 },
    { 0x803480be, 61 },
    { 0, 0 },
    { 0x162480c0, 61 },
    { 0xa8f480c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5480c7, 33 },
    { 0, 0 },
    { 0xc3d480c9, 46 },
    { 0, 0 },
    { 0x1d1480cb, 78 },
    { 0xa51480cb, 45 },
    { 0xeff480cc, 77 },
    { 0x9aa480ce, 90 },
    { 0, 0 },
    { 0x285480d0, 96 },
    { 0, 0 },
    { 0x1c2480d2, 23 },
    { 0, 0 },
    { 0x69d480d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x423480d7, 66 },
    { 0xf9e480d7, 65 },
    { 0x803480d9, 48 },
    { 0, 0 },
    { 0x79c480db, 95 },
    { 0, 0 },
    { 0xd51480dd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x303480e0, 90 },
    { 0xd89480e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10480e4, 95 },
    { 0xd11480e5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a480e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e480ec, 23 },
    { 0xb8e480ed, 98 },
    { 0x145480ee, 90 },
    { 0, 0 },
    { 0x8ce480f0, 93 },
    { 0xa06480f1, 36 },
    { 0x744480f2, 100 },
    { 0xb8e480f3, 95 },
    { 0, 0 },
    { 0x110480f5, 88 },
    { 0x5d2480f6, 90 },
    { 0x323480f6, 98 },
    { 0, 0 },
    { 0x400480f9, 90 },
    { 0xf35480fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2480ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45348102, 34 },
    { 0x29648103, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d848110, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b48118, 76 },
    { 0x82248119, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a4811c, 23 },
    { 0, 0 },
    { 0xd124811e, 90 },
    { 0xe224811e, 23 },
    { 0, 0 },
    { 0x8dc48121, 61 },
    { 0x71348122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda148126, 85 },
    { 0, 0 },
    { 0xfc948128, 66 },
    { 0xff648129, 90 },
    { 0x5584812a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c4812d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c48132, 61 },
    { 0x9bb48133, 99 },
    { 0, 0 },
    { 0xa0048135, 85 },
    { 0x83148135, 43 },
    { 0, 0 },
    { 0xcd148138, 91 },
    { 0xdd48138, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d84813d, 40 },
    { 0, 0 },
    { 0x1664813f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb48143, 61 },
    { 0xd6848144, 55 },
    { 0x63748145, 76 },
    { 0x91f48146, 33 },
    { 0xaa048145, 90 },
    { 0x12e48146, 61 },
    { 0x5fa48149, 90 },
    { 0x71a4814a, 23 },
    { 0x18b48146, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f04814e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d48151, 80 },
    { 0xfe348151, 33 },
    { 0xff248153, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b48158, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab4815b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78348161, 90 },
    { 0x81848162, 23 },
    { 0x96748163, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f4816a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f64816d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61348174, 66 },
    { 0xff848175, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea48179, 90 },
    { 0x2d548179, 33 },
    { 0x2264817b, 76 },
    { 0x3ed4817b, 90 },
    { 0xd164817d, 98 },
    { 0xd044817e, 95 },
    { 0xe954817e, 23 },
    { 0, 0 },
    { 0xec448181, 90 },
    { 0, 0 },
    { 0x59248183, 33 },
    { 0xfb848184, 61 },
    { 0x8dd48185, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d748188, 61 },
    { 0xadb48189, 90 },
    { 0xc014818a, 61 },
    { 0x72748189, 90 },
    { 0xff4818b, 76 },
    { 0x66f4818c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f348191, 96 },
    { 0xc4148191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58348197, 61 },
    { 0xb5d48197, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da4819e, 84 },
    { 0x814819f, 93 },
    { 0x8ac4819f, 90 },
    { 0, 0 },
    { 0x7df481a2, 23 },
    { 0x247481a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967481a8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3481ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc481b3, 61 },
    { 0x5fb481b4, 33 },
    { 0x8bc481b5, 90 },
    { 0xb3e481b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862481ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x774481bd, 90 },
    { 0, 0 },
    { 0x627481bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd481c5, 80 },
    { 0x1c9481c6, 98 },
    { 0x97d481c7, 77 },
    { 0x459481c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768481cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2481d3, 33 },
    { 0, 0 },
    { 0x7ac481d5, 90 },
    { 0x1c9481d6, 98 },
    { 0xce3481d7, 51 },
    { 0x72e481d7, 33 },
    { 0xa77481d9, 33 },
    { 0xebd481d6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109481de, 68 },
    { 0x730481df, 100 },
    { 0x600481e0, 90 },
    { 0x513481e1, 95 },
    { 0x2cc481e2, 34 },
    { 0xcef481e1, 61 },
    { 0, 0 },
    { 0x306481e5, 90 },
    { 0x9e1481e6, 17 },
    { 0x4b1481e7, 95 },
    { 0x9d9481e6, 55 },
    { 0x647481e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841481f0, 43 },
    { 0xdab481f0, 90 },
    { 0, 0 },
    { 0x372481f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x280481f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136481fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1481ff, 97 },
    { 0xa5d48200, 95 },
    { 0xa848200, 33 },
    { 0, 0 },
    { 0x33948203, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a34820b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e24820f, 91 },
    { 0x46248210, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf24821a, 61 },
    { 0, 0 },
    { 0xd584821c, 95 },
    { 0x4a04821c, 33 },
    { 0x3754821e, 43 },
    { 0xce14821f, 95 },
    { 0, 0 },
    { 0x52148221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf848227, 69 },
    { 0xd2648227, 90 },
    { 0x10d48227, 76 },
    { 0x60048227, 34 },
    { 0, 0 },
    { 0xb624822c, 23 },
    { 0x9ae4822d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e648230, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba48234, 55 },
    { 0, 0 },
    { 0x2b548236, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be48241, 90 },
    { 0xb1a48242, 84 },
    { 0xf1a48243, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35948247, 23 },
    { 0x58848247, 46 },
    { 0, 0 },
    { 0x3aa4824a, 33 },
    { 0, 0 },
    { 0x21d4824c, 17 },
    { 0xc6e4824d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6548250, 34 },
    { 0, 0 },
    { 0xc9b48252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf48255, 84 },
    { 0x53748255, 93 },
    { 0, 0 },
    { 0x1ba48258, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c4825c, 81 },
    { 0xc134825d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18648263, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92548267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab4826c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e4826f, 75 },
    { 0xb8948270, 36 },
    { 0x1b448270, 90 },
    { 0x53348272, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b48278, 84 },
    { 0x46048279, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4827e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d348281, 64 },
    { 0xc6a48281, 55 },
    { 0x3dd48282, 34 },
    { 0xcd348284, 66 },
    { 0x45d48282, 91 },
    { 0xc4f48286, 33 },
    { 0x8fc48287, 33 },
    { 0x3ea48288, 94 },
    { 0x48e48289, 98 },
    { 0xb2c48286, 90 },
    { 0, 0 },
    { 0xe3c4828c, 61 },
    { 0, 0 },
    { 0xd594828e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d48291, 43 },
    { 0x5b548291, 95 },
    { 0, 0 },
    { 0x31648294, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48297, 66 },
    { 0x57f48298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf482a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e482a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84482a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86482ad, 87 },
    { 0, 0 },
    { 0x4e7482af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752482b4, 52 },
    { 0x156482b5, 95 },
    { 0xe66482b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d482b9, 23 },
    { 0x4f5482b9, 34 },
    { 0x513482bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2482bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483482c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4482ca, 84 },
    { 0, 0 },
    { 0xece482cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x861482cf, 93 },
    { 0, 0 },
    { 0x77c482d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea482d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0482d9, 46 },
    { 0xca5482d9, 34 },
    { 0, 0 },
    { 0xa4b482dc, 88 },
    { 0x620482dd, 80 },
    { 0, 0 },
    { 0x907482df, 52 },
    { 0xace482df, 38 },
    { 0xc0a482e1, 93 },
    { 0xb63482e1, 55 },
    { 0xfca482e3, 61 },
    { 0, 0 },
    { 0x216482e5, 34 },
    { 0, 0 },
    { 0x5bf482e7, 66 },
    { 0x606482e8, 91 },
    { 0x837482e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba482ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x463482ef, 69 },
    { 0xfa2482ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49482f6, 23 },
    { 0, 0 },
    { 0x530482f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23482fc, 90 },
    { 0, 0 },
    { 0x83b482fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a48305, 23 },
    { 0x26e48305, 61 },
    { 0x42848305, 99 },
    { 0xd5b48308, 46 },
    { 0x46a48309, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d4830c, 64 },
    { 0xa554830d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a48311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c24831c, 76 },
    { 0x5b34831d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e48321, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc248327, 37 },
    { 0x82448327, 33 },
    { 0xcb348329, 34 },
    { 0x8cf4832a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7874832f, 43 },
    { 0x1fc48330, 33 },
    { 0xe2548331, 95 },
    { 0xe6348330, 61 },
    { 0x45048333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46148338, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e04833c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e48340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e48346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed048349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa74834c, 76 },
    { 0, 0 },
    { 0xda14834e, 76 },
    { 0x89c4834e, 33 },
    { 0xf648350, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b648353, 90 },
    { 0, 0 },
    { 0x6fc48355, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce348358, 90 },
    { 0x62e48359, 22 },
    { 0x18f4835a, 85 },
    { 0x9404835a, 66 },
    { 0x63c48358, 34 },
    { 0x9254835b, 95 },
    { 0xe3d4835a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae648364, 61 },
    { 0x7a048364, 75 },
    { 0, 0 },
    { 0x9ec48367, 34 },
    { 0xa4348368, 98 },
    { 0x53248368, 95 },
    { 0, 0 },
    { 0x3674836b, 64 },
    { 0, 0 },
    { 0xfe04836d, 84 },
    { 0xe474836e, 61 },
    { 0xaab4836f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f48377, 61 },
    { 0x6b748377, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa4837d, 95 },
    { 0xc1e4837d, 69 },
    { 0x7464837f, 90 },
    { 0x92c4837f, 55 },
    { 0x9ac48381, 33 },
    { 0, 0 },
    { 0x68248383, 100 },
    { 0xc8a48384, 93 },
    { 0xd0948385, 95 },
    { 0x6d748386, 98 },
    { 0xb9648387, 67 },
    { 0x6d348388, 66 },
    { 0x7df48387, 90 },
    { 0x2d64838a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a4838d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef948395, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4674839c, 95 },
    { 0x6944839d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2483a1, 61 },
    { 0, 0 },
    { 0x892483a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6483a7, 98 },
    { 0xf79483a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc483b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1483b9, 98 },
    { 0x461483ba, 76 },
    { 0, 0 },
    { 0x716483bc, 66 },
    { 0x62c483bc, 95 },
    { 0x41f483bd, 91 },
    { 0, 0 },
    { 0x5ea483c0, 33 },
    { 0, 0 },
    { 0x5df483c2, 91 },
    { 0x7b7483c3, 23 },
    { 0x31a483c4, 81 },
    { 0x38483c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3483c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x302483cc, 99 },
    { 0x3f9483cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6483d5, 78 },
    { 0xb61483d5, 66 },
    { 0xb6483d6, 33 },
    { 0xd78483d6, 75 },
    { 0xee0483d9, 84 },
    { 0x8f8483da, 23 },
    { 0x725483d5, 76 },
    { 0x482483dc, 61 },
    { 0x29e483dd, 23 },
    { 0xa92483de, 77 },
    { 0x426483de, 67 },
    { 0, 0 },
    { 0x41d483e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6483e7, 90 },
    { 0xde3483e8, 61 },
    { 0x55d483e9, 55 },
    { 0, 0 },
    { 0x818483eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8483f0, 66 },
    { 0xf8483f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce483f4, 61 },
    { 0xdaf483f4, 61 },
    { 0x556483f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1483fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd483ff, 33 },
    { 0, 0 },
    { 0xd1048401, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80748405, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8594840a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c4840d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d548410, 81 },
    { 0, 0 },
    { 0xfd648412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e348419, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae4841e, 34 },
    { 0x9df4841e, 95 },
    { 0x9d848420, 87 },
    { 0xad748420, 33 },
    { 0x11548421, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2548428, 91 },
    { 0, 0 },
    { 0x8484842a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc704842d, 33 },
    { 0, 0 },
    { 0xc304842f, 98 },
    { 0, 0 },
    { 0x33048431, 46 },
    { 0, 0 },
    { 0xf4048433, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a48436, 98 },
    { 0xa5748436, 90 },
    { 0xe4b48437, 90 },
    { 0xb9048438, 46 },
    { 0, 0 },
    { 0xb9b4843b, 61 },
    { 0xf1b4843c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2448446, 61 },
    { 0x9fd48447, 92 },
    { 0x30c48448, 91 },
    { 0xe7648446, 34 },
    { 0xa3e4844a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf748450, 90 },
    { 0xc6748451, 97 },
    { 0, 0 },
    { 0x1f548453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23248459, 23 },
    { 0xdc54845a, 33 },
    { 0, 0 },
    { 0xd5d4845c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e48460, 23 },
    { 0x23248461, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d48464, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94846f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef48476, 23 },
    { 0, 0 },
    { 0xf8648478, 85 },
    { 0x25448479, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb4847c, 90 },
    { 0x5604847d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d4848a, 53 },
    { 0xe384848b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e48494, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f48499, 90 },
    { 0x96d4849a, 43 },
    { 0xbe24849b, 33 },
    { 0xa514849b, 34 },
    { 0x4bb48499, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d484a1, 99 },
    { 0xeb6484a1, 95 },
    { 0x355484a2, 33 },
    { 0x93c484a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397484ac, 68 },
    { 0xbab484ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4484b0, 65 },
    { 0x639484b1, 98 },
    { 0x631484b2, 87 },
    { 0, 0 },
    { 0xdbb484b4, 23 },
    { 0x3e5484b5, 99 },
    { 0x8c4484b6, 90 },
    { 0x9ae484b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x790484ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89484be, 55 },
    { 0x9d1484be, 36 },
    { 0, 0 },
    { 0x44484c1, 98 },
    { 0, 0 },
    { 0x283484c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91484c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986484cc, 57 },
    { 0x29a484cc, 43 },
    { 0x507484ce, 61 },
    { 0xecf484cf, 61 },
    { 0x518484cf, 90 },
    { 0x6e0484d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5484da, 76 },
    { 0, 0 },
    { 0x948484dc, 98 },
    { 0, 0 },
    { 0x665484de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x661484e1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de484ea, 76 },
    { 0x661484ea, 76 },
    { 0x1ca484ea, 23 },
    { 0x52c484eb, 90 },
    { 0x51f484ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74484f6, 76 },
    { 0, 0 },
    { 0x494484f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3484fc, 52 },
    { 0x506484fd, 61 },
    { 0, 0 },
    { 0x869484ff, 43 },
    { 0x11e48500, 81 },
    { 0, 0 },
    { 0xad148502, 42 },
    { 0, 0 },
    { 0xc8148504, 23 },
    { 0, 0 },
    { 0xb5a48506, 33 },
    { 0x65048506, 61 },
    { 0x20048508, 34 },
    { 0, 0 },
    { 0x2b34850a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e648512, 46 },
    { 0x46648513, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb034851c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b4851f, 61 },
    { 0, 0 },
    { 0xc1f48521, 90 },
    { 0x1ba48521, 84 },
    { 0x71e48523, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff48528, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b4852d, 90 },
    { 0, 0 },
    { 0x4544852f, 88 },
    { 0x95548530, 52 },
    { 0xd1848531, 100 },
    { 0x5d48531, 90 },
    { 0, 0 },
    { 0xd3848534, 98 },
    { 0x59348535, 98 },
    { 0, 0 },
    { 0xa5048537, 53 },
    { 0xddb48538, 90 },
    { 0x11a48539, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4814853e, 53 },
    { 0xcf54853f, 77 },
    { 0, 0 },
    { 0x8fc48541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92148546, 99 },
    { 0xecf48547, 33 },
    { 0xc2048547, 23 },
    { 0, 0 },
    { 0xe134854a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe134854d, 99 },
    { 0xc4b4854e, 76 },
    { 0x8354854f, 95 },
    { 0x43948550, 55 },
    { 0x2f848550, 95 },
    { 0x1514854f, 34 },
    { 0x34d4854d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a48557, 68 },
    { 0xe1048557, 90 },
    { 0, 0 },
    { 0xa914855a, 55 },
    { 0xe584855b, 61 },
    { 0x4204855c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65748561, 67 },
    { 0, 0 },
    { 0xc6948563, 90 },
    { 0x5e548564, 61 },
    { 0x1e448565, 66 },
    { 0x5e748566, 90 },
    { 0xfc448567, 52 },
    { 0x68448568, 34 },
    { 0x15748569, 61 },
    { 0xb7a4856a, 61 },
    { 0x19f48567, 34 },
    { 0xc6d4856c, 61 },
    { 0xa6b4856d, 61 },
    { 0x7734856e, 98 },
    { 0xa394856f, 23 },
    { 0xf2c48569, 33 },
    { 0, 0 },
    { 0x67548572, 23 },
    { 0xb5048573, 90 },
    { 0x47e48574, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93748577, 43 },
    { 0, 0 },
    { 0xca848579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4794857f, 87 },
    { 0xa114857f, 91 },
    { 0x5d648581, 23 },
    { 0x81348582, 33 },
    { 0xafe48582, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4858a, 34 },
    { 0xf034858b, 100 },
    { 0x45d4858a, 17 },
    { 0, 0 },
    { 0x8024858e, 91 },
    { 0xda94858f, 38 },
    { 0xd1548590, 33 },
    { 0x6b948590, 78 },
    { 0x64d48590, 46 },
    { 0x9cf4858f, 87 },
    { 0x40c48594, 90 },
    { 0xb1e4858f, 98 },
    { 0xacc48591, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4859a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4485a0, 61 },
    { 0xc5c485a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee485a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4485a9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad485af, 61 },
    { 0, 0 },
    { 0xcfb485b1, 34 },
    { 0x50e485b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a485b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd485ba, 26 },
    { 0xd22485ba, 23 },
    { 0, 0 },
    { 0x13b485bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e485c0, 43 },
    { 0x388485c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda485c7, 34 },
    { 0xd0c485c8, 23 },
    { 0xc89485c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc485ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7485d4, 56 },
    { 0xe0485d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53485d9, 90 },
    { 0, 0 },
    { 0xd81485db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b485e1, 23 },
    { 0xd7d485e2, 95 },
    { 0x550485e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32485e6, 90 },
    { 0xb74485e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c485ec, 93 },
    { 0x1aa485ed, 76 },
    { 0, 0 },
    { 0x7e485ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9485f8, 90 },
    { 0xb26485f9, 90 },
    { 0x46b485f9, 61 },
    { 0xc3d485fa, 61 },
    { 0, 0 },
    { 0x9f9485fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d348601, 66 },
    { 0, 0 },
    { 0xeeb48603, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f4860c, 55 },
    { 0x24f4860c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a648612, 78 },
    { 0x98548613, 98 },
    { 0xb9448614, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67648617, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf324861a, 98 },
    { 0, 0 },
    { 0x3484861c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc44861f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c48624, 90 },
    { 0, 0 },
    { 0xd9248626, 61 },
    { 0x35d48627, 95 },
    { 0, 0 },
    { 0xd6248629, 90 },
    { 0x31a4862a, 93 },
    { 0xbdc4862a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf594862e, 33 },
    { 0x4ee4862f, 90 },
    { 0xb948630, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xce348633, 65 },
    { 0, 0 },
    { 0xa2748635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e48639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0348640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8934864d, 98 },
    { 0x93f4864d, 95 },
    { 0, 0 },
    { 0xa9848650, 33 },
    { 0xaf548651, 61 },
    { 0, 0 },
    { 0x3c148653, 46 },
    { 0, 0 },
    { 0x94e48655, 90 },
    { 0x24448656, 77 },
    { 0xd4248656, 34 },
    { 0x1df48658, 23 },
    { 0x2e848659, 98 },
    { 0x93f48657, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3154865d, 76 },
    { 0x9214865d, 33 },
    { 0xa0d4865d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6348662, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da48666, 33 },
    { 0, 0 },
    { 0xd8c48668, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5964866c, 33 },
    { 0, 0 },
    { 0x1d44866e, 95 },
    { 0, 0 },
    { 0xfe248670, 85 },
    { 0x18948671, 98 },
    { 0x37b48672, 99 },
    { 0x9ea48672, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8248677, 95 },
    { 0x91448677, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6444867b, 5 },
    { 0x47c4867b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e48681, 95 },
    { 0x18248682, 99 },
    { 0, 0 },
    { 0xc0c48684, 95 },
    { 0x17c48685, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd48688, 61 },
    { 0, 0 },
    { 0x514868a, 33 },
    { 0x93e4868a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35748695, 90 },
    { 0, 0 },
    { 0x9e848697, 33 },
    { 0x8b248697, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe024869d, 55 },
    { 0xaff4869e, 23 },
    { 0x7784869f, 52 },
    { 0, 0 },
    { 0x709486a1, 98 },
    { 0x39c486a1, 90 },
    { 0xcd2486a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855486ab, 76 },
    { 0, 0 },
    { 0xc4c486ad, 66 },
    { 0x7f9486ae, 95 },
    { 0, 0 },
    { 0x41f486b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa486b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b486b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb486ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x480486bd, 61 },
    { 0xa5e486be, 23 },
    { 0xb2b486bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38486c3, 78 },
    { 0xc24486c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce486c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84486cd, 23 },
    { 0x8a6486cd, 33 },
    { 0xda486ce, 95 },
    { 0, 0 },
    { 0xea1486d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90486d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b486d7, 84 },
    { 0xbc486d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b486db, 87 },
    { 0x5ca486dc, 90 },
    { 0xbec486dc, 88 },
    { 0xcc486db, 98 },
    { 0xe40486dc, 36 },
    { 0x568486e0, 23 },
    { 0, 0 },
    { 0x1d9486e2, 61 },
    { 0x8d5486e3, 38 },
    { 0x75b486e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x677486e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb486eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40486f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x886486f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea486fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c486ff, 87 },
    { 0xfd48700, 84 },
    { 0x21348701, 95 },
    { 0x8c248701, 33 },
    { 0, 0 },
    { 0xae548704, 33 },
    { 0, 0 },
    { 0x6aa48706, 46 },
    { 0x20248707, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc454870c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7948712, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4548719, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa204871f, 95 },
    { 0x8d148720, 90 },
    { 0, 0 },
    { 0xc5d48722, 61 },
    { 0, 0 },
    { 0x32b48724, 55 },
    { 0x2fb48724, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df4872a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44872e, 95 },
    { 0x6714872f, 43 },
    { 0x1af4872e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff48738, 95 },
    { 0x7fd48738, 81 },
    { 0, 0 },
    { 0xa4b4873b, 33 },
    { 0, 0 },
    { 0x7e4873d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d548744, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b48749, 76 },
    { 0x73248749, 90 },
    { 0, 0 },
    { 0x5714874c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab48754, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3648757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b4875b, 34 },
    { 0, 0 },
    { 0xbea4875d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a48760, 40 },
    { 0, 0 },
    { 0x20748762, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed448765, 23 },
    { 0x43948766, 78 },
    { 0xf2448767, 95 },
    { 0xd7148768, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb994876c, 55 },
    { 0xf3c4876d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d48770, 67 },
    { 0x44c48771, 33 },
    { 0xe5948770, 34 },
    { 0x40d48773, 90 },
    { 0xc6e48770, 99 },
    { 0x31a48775, 61 },
    { 0x67e48776, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18948779, 33 },
    { 0xd484877a, 98 },
    { 0x7014877b, 61 },
    { 0x20d4877c, 95 },
    { 0x8514877d, 93 },
    { 0x20d4877d, 90 },
    { 0x5354877e, 95 },
    { 0, 0 },
    { 0xd048781, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38748788, 95 },
    { 0xf3748788, 55 },
    { 0x23b4878a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5434878e, 76 },
    { 0x6bd4878f, 91 },
    { 0x2bf4878e, 33 },
    { 0x5b148791, 33 },
    { 0xbba48790, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x548795, 87 },
    { 0, 0 },
    { 0x6c248797, 100 },
    { 0x37148798, 98 },
    { 0, 0 },
    { 0xaf24879a, 55 },
    { 0, 0 },
    { 0x6374879c, 61 },
    { 0xd814879d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9487a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73487a6, 98 },
    { 0xae3487a7, 43 },
    { 0x547487a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc487ad, 76 },
    { 0, 0 },
    { 0x3bd487af, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372487b5, 95 },
    { 0xeff487b6, 23 },
    { 0x76c487b6, 76 },
    { 0x7c2487b8, 90 },
    { 0xa4e487b9, 95 },
    { 0x3d8487b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56487be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2487c5, 33 },
    { 0xe51487c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89487c9, 90 },
    { 0x414487ca, 34 },
    { 0x3ca487ca, 95 },
    { 0x8f8487cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654487d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d487dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f487e1, 77 },
    { 0xa61487e2, 17 },
    { 0, 0 },
    { 0x9b6487e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d487e8, 90 },
    { 0, 0 },
    { 0x2ae487ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877487f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x385487f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x763487f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f487fc, 34 },
    { 0xff9487fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7148803, 37 },
    { 0xeb248804, 33 },
    { 0, 0 },
    { 0x13548806, 90 },
    { 0, 0 },
    { 0x35d48808, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4880d, 90 },
    { 0x9854880e, 90 },
    { 0x7494880f, 33 },
    { 0, 0 },
    { 0x78d48811, 55 },
    { 0, 0 },
    { 0xf1448813, 23 },
    { 0x90a48814, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95348818, 98 },
    { 0, 0 },
    { 0x7ee4881a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb48823, 90 },
    { 0x64b48824, 95 },
    { 0xdb948823, 93 },
    { 0x26248826, 46 },
    { 0, 0 },
    { 0x3ac48828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f84882c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f648830, 34 },
    { 0x5e148831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11448834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde948837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6034883b, 98 },
    { 0x8514883c, 55 },
    { 0x5ab4883c, 90 },
    { 0x1c74883c, 90 },
    { 0x5eb4883c, 90 },
    { 0x99a48840, 61 },
    { 0xc5048841, 90 },
    { 0xf9548840, 95 },
    { 0x28148843, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e48847, 98 },
    { 0x86748848, 95 },
    { 0x4cf48849, 33 },
    { 0x49748848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f4884d, 34 },
    { 0, 0 },
    { 0xdc24884f, 33 },
    { 0, 0 },
    { 0x2348851, 33 },
    { 0xbcc48852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73748859, 34 },
    { 0, 0 },
    { 0x5bc4885b, 95 },
    { 0xcbc4885c, 75 },
    { 0x7ec4885d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73548861, 98 },
    { 0x53248862, 55 },
    { 0x3fc48861, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f48868, 33 },
    { 0x31f48868, 33 },
    { 0xb1748869, 33 },
    { 0xc5748869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb48874, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda848878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4748880, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e748883, 61 },
    { 0x13c48884, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6604888a, 23 },
    { 0x4bf4888b, 55 },
    { 0xb824888a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd48893, 33 },
    { 0, 0 },
    { 0x10248895, 33 },
    { 0xc7d48896, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f048899, 55 },
    { 0x9ef4889a, 87 },
    { 0x6484889b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35488a4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff488a8, 91 },
    { 0, 0 },
    { 0x716488aa, 52 },
    { 0xe4c488aa, 61 },
    { 0xc62488ac, 33 },
    { 0x12c488ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380488b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3488bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd488c0, 90 },
    { 0x74b488c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3488c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0488ce, 95 },
    { 0, 0 },
    { 0xb4a488d0, 90 },
    { 0, 0 },
    { 0x7d7488d2, 90 },
    { 0x3e9488d3, 66 },
    { 0x45a488d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x982488d7, 43 },
    { 0x216488d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e488db, 66 },
    { 0, 0 },
    { 0xc2b488dd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6488e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464488e5, 100 },
    { 0x66b488e6, 98 },
    { 0, 0 },
    { 0xbf3488e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989488ec, 98 },
    { 0, 0 },
    { 0x43a488ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x925488f1, 90 },
    { 0x172488f2, 52 },
    { 0, 0 },
    { 0x129488f4, 90 },
    { 0, 0 },
    { 0x3aa488f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498488fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9488fe, 77 },
    { 0x3f4488ff, 98 },
    { 0x4b148900, 90 },
    { 0, 0 },
    { 0x7f148902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e48906, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d48909, 98 },
    { 0x1414890a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24890f, 95 },
    { 0, 0 },
    { 0xd1348911, 43 },
    { 0, 0 },
    { 0x9c48913, 76 },
    { 0, 0 },
    { 0x95948915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1534891a, 90 },
    { 0, 0 },
    { 0x2c54891c, 43 },
    { 0, 0 },
    { 0xcf4891e, 95 },
    { 0xf904891e, 69 },
    { 0xd7e48920, 33 },
    { 0xa8d4891f, 33 },
    { 0xcaa4891f, 34 },
    { 0, 0 },
    { 0x2148924, 34 },
    { 0, 0 },
    { 0x66648926, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2048929, 100 },
    { 0, 0 },
    { 0x4284892b, 95 },
    { 0xbdb4892c, 61 },
    { 0, 0 },
    { 0xc4892e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf48931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e148936, 34 },
    { 0, 0 },
    { 0x90d48938, 98 },
    { 0x21548938, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa094893f, 93 },
    { 0, 0 },
    { 0xf48941, 33 },
    { 0x99148942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a148945, 90 },
    { 0x96848946, 33 },
    { 0xce048946, 23 },
    { 0x42148946, 95 },
    { 0x7b348949, 76 },
    { 0xc0b4894a, 61 },
    { 0x8084894b, 90 },
    { 0x6204894c, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c348950, 72 },
    { 0x23948951, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef48958, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade4895f, 88 },
    { 0x8b48960, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c448965, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d48968, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d04896b, 33 },
    { 0, 0 },
    { 0x68a4896d, 99 },
    { 0x5df4896e, 65 },
    { 0xf6e4896f, 76 },
    { 0xcc48970, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c48976, 93 },
    { 0, 0 },
    { 0xdfc48978, 91 },
    { 0xf5048978, 90 },
    { 0x70e48979, 85 },
    { 0xa94897a, 90 },
    { 0xdd648978, 23 },
    { 0xbf248979, 101 },
    { 0x7484897d, 33 },
    { 0xb94897f, 98 },
    { 0, 0 },
    { 0xefa48981, 77 },
    { 0xfdb48982, 76 },
    { 0xba948982, 34 },
    { 0xda848984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f948988, 55 },
    { 0, 0 },
    { 0x1dc4898a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4898e, 90 },
    { 0x9484898f, 83 },
    { 0xb6d4898e, 93 },
    { 0x1eb48991, 61 },
    { 0x7b348992, 95 },
    { 0x64c48993, 90 },
    { 0x99048994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c48997, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d4899e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x633489a1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5489a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0489a7, 76 },
    { 0, 0 },
    { 0x3e0489a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea489ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f489b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x105489b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7489bf, 23 },
    { 0xe34489bf, 61 },
    { 0x482489c1, 23 },
    { 0xe5d489c2, 61 },
    { 0, 0 },
    { 0xee8489c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e489ca, 84 },
    { 0, 0 },
    { 0x3a4489cc, 61 },
    { 0x3de489cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8489d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79489d3, 98 },
    { 0x191489d4, 78 },
    { 0, 0 },
    { 0xfab489d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29489d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343489dd, 95 },
    { 0x3fe489de, 66 },
    { 0xda0489df, 67 },
    { 0xfbc489de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849489e4, 99 },
    { 0x4a0489e5, 52 },
    { 0xc9c489e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6489ec, 90 },
    { 0xcb4489ed, 88 },
    { 0xe85489ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35489f1, 61 },
    { 0, 0 },
    { 0x1dc489f3, 76 },
    { 0xbf489f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf489f9, 76 },
    { 0xfdc489fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4489fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd248a01, 23 },
    { 0x4d648a02, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45848a08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe48a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d48a10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd48a18, 98 },
    { 0x81648a18, 90 },
    { 0xfc248a19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa648a1d, 90 },
    { 0xf2848a1d, 95 },
    { 0, 0 },
    { 0xe6248a20, 59 },
    { 0x8b948a21, 76 },
    { 0, 0 },
    { 0x92d48a23, 61 },
    { 0x74348a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84748a27, 98 },
    { 0x94148a28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7448a2b, 34 },
    { 0, 0 },
    { 0xa1a48a2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c48a36, 66 },
    { 0xbf948a37, 95 },
    { 0x94d48a38, 95 },
    { 0, 0 },
    { 0x6648a3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x14448a3d, 98 },
    { 0, 0 },
    { 0x97148a3f, 23 },
    { 0x83548a40, 76 },
    { 0, 0 },
    { 0xa2948a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d48a4a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f248a4f, 43 },
    { 0, 0 },
    { 0xc4a48a51, 23 },
    { 0, 0 },
    { 0xaa648a53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a48a58, 34 },
    { 0x3fd48a59, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b48a62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a048a66, 61 },
    { 0x5a748a67, 68 },
    { 0x70d48a67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10648a6d, 95 },
    { 0, 0 },
    { 0x87e48a6f, 34 },
    { 0x77648a70, 23 },
    { 0x44248a71, 17 },
    { 0x1c548a71, 23 },
    { 0x7ac48a6f, 85 },
    { 0xd8348a74, 81 },
    { 0x67e48a75, 90 },
    { 0x8c148a76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd148a7b, 76 },
    { 0xe2748a7b, 55 },
    { 0x46d48a7c, 90 },
    { 0xb4248a7d, 93 },
    { 0xfa448a7d, 33 },
    { 0, 0 },
    { 0xe3548a81, 53 },
    { 0x47e48a81, 34 },
    { 0xb2a48a83, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6148a89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11748a8d, 47 },
    { 0x6ad48a8d, 91 },
    { 0x4f748a8f, 52 },
    { 0x7a748a8f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e48a96, 61 },
    { 0x60b48a97, 33 },
    { 0x9da48a96, 55 },
    { 0x99b48a99, 34 },
    { 0x6d48a9a, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf248aa2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d48aa6, 87 },
    { 0, 0 },
    { 0x48048aa8, 43 },
    { 0xe2448aa9, 23 },
    { 0x41848aaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66648aae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5748ab1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64648ab7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9948ac0, 33 },
    { 0x93f48ac1, 55 },
    { 0x35448ac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b48ac5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12848ac9, 95 },
    { 0xe8248aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce48ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11348ad1, 34 },
    { 0, 0 },
    { 0xcf48ad3, 76 },
    { 0, 0 },
    { 0x3d748ad5, 34 },
    { 0, 0 },
    { 0x1e848ad7, 46 },
    { 0, 0 },
    { 0xd1348ad9, 90 },
    { 0xe0748ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4248ae1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4448ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28748ae9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d48af1, 76 },
    { 0x18f48af2, 36 },
    { 0x19048af2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48248af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb48afe, 95 },
    { 0, 0 },
    { 0xf2148b00, 98 },
    { 0x55f48b01, 98 },
    { 0, 0 },
    { 0x1b348b03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64348b06, 77 },
    { 0xecd48b06, 43 },
    { 0xa7a48b08, 33 },
    { 0, 0 },
    { 0x35d48b0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af48b0f, 61 },
    { 0xcda48b0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d48b15, 34 },
    { 0x6f48b15, 33 },
    { 0x7b548b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e648b1a, 40 },
    { 0, 0 },
    { 0xa0148b1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d48b2a, 98 },
    { 0xcfe48b2b, 67 },
    { 0, 0 },
    { 0x9f48b2d, 99 },
    { 0xfe48b2d, 66 },
    { 0x73f48b2f, 76 },
    { 0x12a48b2f, 43 },
    { 0x8cb48b31, 33 },
    { 0x21a48b32, 90 },
    { 0x69f48b33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb248b36, 66 },
    { 0, 0 },
    { 0xe9248b38, 98 },
    { 0x5a148b38, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e748b3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4948b43, 33 },
    { 0xd6548b44, 90 },
    { 0x4b48b45, 98 },
    { 0x54b48b45, 33 },
    { 0, 0 },
    { 0xd8e48b48, 33 },
    { 0, 0 },
    { 0xcf348b4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a48b50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b648b53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f48b59, 40 },
    { 0x9d748b59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b48b5d, 33 },
    { 0, 0 },
    { 0x5d948b5f, 76 },
    { 0x8a448b60, 90 },
    { 0x5f748b61, 61 },
    { 0x49448b62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0148b65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b748b68, 33 },
    { 0xc4f48b69, 90 },
    { 0, 0 },
    { 0x69248b6b, 55 },
    { 0, 0 },
    { 0x748b6d, 61 },
    { 0x81448b6d, 8 },
    { 0x1b848b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f48b76, 61 },
    { 0xb6648b77, 61 },
    { 0x3c748b77, 77 },
    { 0x24a48b78, 67 },
    { 0x62948b7a, 95 },
    { 0x59548b7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48b81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef448b86, 61 },
    { 0xf5948b86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c48b8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d48b91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd648b98, 33 },
    { 0x22548b98, 33 },
    { 0xdea48b9a, 95 },
    { 0x9e548b9b, 77 },
    { 0xfd48b9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb48b9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67048ba2, 80 },
    { 0x3cb48ba3, 77 },
    { 0xb3e48ba4, 99 },
    { 0x7f648ba5, 76 },
    { 0x71048ba6, 98 },
    { 0x13248ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb648bab, 90 },
    { 0, 0 },
    { 0x48148bad, 42 },
    { 0x2d148bae, 90 },
    { 0xf0748bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc048bb4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48bb8, 95 },
    { 0, 0 },
    { 0x3e748bba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51148bbe, 77 },
    { 0xfd848bbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f48bc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec248bc8, 23 },
    { 0, 0 },
    { 0xdbe48bca, 95 },
    { 0, 0 },
    { 0x3a348bcc, 61 },
    { 0x99e48bcd, 61 },
    { 0xd7b48bce, 78 },
    { 0x9d448bce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc48bd9, 43 },
    { 0, 0 },
    { 0xf7348bdb, 78 },
    { 0x97448bdc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348be3, 91 },
    { 0x1c448be3, 83 },
    { 0, 0 },
    { 0x12348be6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3448be9, 23 },
    { 0xe248bea, 23 },
    { 0x35748beb, 34 },
    { 0x78748beb, 23 },
    { 0x71a48bed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1848bf1, 43 },
    { 0xa1748bf1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59248bf8, 55 },
    { 0x7e148bf9, 23 },
    { 0x72848bf9, 43 },
    { 0xfcb48bfb, 90 },
    { 0xdc948bf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0748c01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf148c05, 77 },
    { 0x36c48c06, 84 },
    { 0xe9c48c07, 61 },
    { 0xde748c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a248c0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5948c13, 24 },
    { 0xf2848c14, 61 },
    { 0x7e48c13, 43 },
    { 0x29648c14, 61 },
    { 0, 0 },
    { 0xeda48c18, 95 },
    { 0xdc48c18, 34 },
    { 0x22d48c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9348c1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26948c26, 33 },
    { 0x2d648c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d48c2b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x14148c2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0948c32, 93 },
    { 0xa6b48c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2148c37, 61 },
    { 0xce348c38, 61 },
    { 0x96248c39, 43 },
    { 0x81c48c3a, 95 },
    { 0x49448c3b, 90 },
    { 0, 0 },
    { 0xdd848c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46748c40, 23 },
    { 0x67548c40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda148c48, 95 },
    { 0, 0 },
    { 0x64b48c4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97248c51, 46 },
    { 0x9ae48c52, 46 },
    { 0, 0 },
    { 0x8048c54, 77 },
    { 0, 0 },
    { 0x1f048c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b48c5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb648c61, 99 },
    { 0, 0 },
    { 0xb9848c63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa848c6a, 23 },
    { 0xc1448c6b, 46 },
    { 0x46e48c6b, 90 },
    { 0x20948c6a, 93 },
    { 0, 0 },
    { 0xab648c6f, 98 },
    { 0xe548c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e48c77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb748c87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a48c8a, 95 },
    { 0x49d48c8b, 98 },
    { 0xbd648c8c, 95 },
    { 0xabd48c8d, 90 },
    { 0, 0 },
    { 0x80748c8f, 98 },
    { 0xbb548c8f, 90 },
    { 0x8d448c91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c48c95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46348ca2, 23 },
    { 0, 0 },
    { 0x8f348ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85048ca8, 23 },
    { 0, 0 },
    { 0x6d548caa, 76 },
    { 0, 0 },
    { 0xaf448cac, 90 },
    { 0, 0 },
    { 0xdd148cae, 61 },
    { 0x6f148cae, 40 },
    { 0, 0 },
    { 0xede48cb1, 81 },
    { 0xc0948cb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd248cb5, 17 },
    { 0x94748cb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b48cb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb848cbe, 66 },
    { 0xc3b48cbf, 76 },
    { 0xe4b48cbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a48cc5, 93 },
    { 0x6b748cc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d48cc9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2948cce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18548cd6, 23 },
    { 0, 0 },
    { 0x19b48cd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0248cdb, 95 },
    { 0x5ad48cdc, 77 },
    { 0x36548cdc, 95 },
    { 0xf5e48cde, 43 },
    { 0x2b948cdc, 85 },
    { 0x99b48cde, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x10348ce3, 88 },
    { 0x10c48ce4, 34 },
    { 0x78e48ce5, 81 },
    { 0, 0 },
    { 0x8d248ce7, 78 },
    { 0, 0 },
    { 0x66d48ce9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61948cee, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa48cf2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x84848cf5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d48cfd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c48d00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f48d03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc48d06, 97 },
    { 0, 0 },
    { 0x28548d08, 90 },
    { 0, 0 },
    { 0x82548d0a, 33 },
    { 0, 0 },
    { 0xa6948d0c, 33 },
    { 0, 0 },
    { 0x13648d0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a48d11, 90 },
    { 0xc8c48d11, 33 },
    { 0, 0 },
    { 0xa1648d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69348d19, 90 },
    { 0, 0 },
    { 0x83948d1b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2648d21, 91 },
    { 0xa5a48d22, 81 },
    { 0x97048d22, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58848d28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d48d2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50648d2f, 55 },
    { 0xc5f48d2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff448d33, 76 },
    { 0xbf948d34, 85 },
    { 0, 0 },
    { 0x12448d36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1348d3e, 34 },
    { 0, 0 },
    { 0x3a948d40, 36 },
    { 0x89348d41, 90 },
    { 0xf1548d40, 98 },
    { 0x13f48d43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99148d46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37348d4c, 81 },
    { 0x6248d4d, 87 },
    { 0x8a548d4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d748d51, 55 },
    { 0, 0 },
    { 0x85948d53, 23 },
    { 0, 0 },
    { 0x79d48d55, 90 },
    { 0, 0 },
    { 0xb9748d57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e48d5a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a148d5d, 33 },
    { 0x99848d5e, 33 },
    { 0xf5a48d5d, 90 },
    { 0xb5e48d5f, 61 },
    { 0, 0 },
    { 0x3d548d62, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b248d67, 33 },
    { 0x10048d68, 55 },
    { 0, 0 },
    { 0x46848d6a, 61 },
    { 0xe0048d6b, 23 },
    { 0x90548d6b, 46 },
    { 0, 0 },
    { 0xdd948d6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb48d72, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff648d76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e48d79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44548d7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x18348d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f48d83, 85 },
    { 0x8d848d84, 67 },
    { 0, 0 },
    { 0x4dc48d86, 34 },
    { 0x24448d87, 77 },
    { 0x29048d88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3848d8b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c48d91, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f148d98, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23448da0, 55 },
    { 0xec548da0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57448da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40448da8, 34 },
    { 0xd0448da8, 65 },
    { 0x84748daa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa248db7, 33 },
    { 0, 0 },
    { 0xa7848db9, 98 },
    { 0, 0 },
    { 0x15c48dbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a48dbf, 61 },
    { 0x63548dc0, 33 },
    { 0x64048dc1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e648dc4, 75 },
    { 0x36248dc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b48dc8, 36 },
    { 0x35c48dc8, 33 },
    { 0xf48dca, 95 },
    { 0x24148dc8, 98 },
    { 0x38048dcb, 98 },
    { 0x85448dcb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8548dd0, 33 },
    { 0x54f48dd1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc48dd4, 90 },
    { 0xf4248dd5, 23 },
    { 0x33948dd5, 61 },
    { 0x83748dd7, 61 },
    { 0xc1848dd8, 61 },
    { 0xda648dd6, 23 },
    { 0x41548dd7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f48ddf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74548de3, 23 },
    { 0x1c448de3, 85 },
    { 0, 0 },
    { 0xf1a48de6, 95 },
    { 0, 0 },
    { 0x88948de8, 95 },
    { 0x8eb48de9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca48dec, 90 },
    { 0, 0 },
    { 0x9ab48dee, 61 },
    { 0x52548def, 95 },
    { 0x13d48dee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75548df6, 43 },
    { 0x2648df7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf448dfd, 98 },
    { 0, 0 },
    { 0x13748dff, 90 },
    { 0x9a348e00, 67 },
    { 0x46848e01, 99 },
    { 0x7d248e00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d48e06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd48e0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab548e12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48548e18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14248e1d, 81 },
    { 0, 0 },
    { 0x4d048e1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d48e22, 94 },
    { 0x23348e22, 55 },
    { 0, 0 },
    { 0x28a48e25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d48e2c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d848e30, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b348e34, 66 },
    { 0, 0 },
    { 0x2a348e36, 33 },
    { 0xa8848e37, 33 },
    { 0x78e48e37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b48e3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53648e40, 61 },
    { 0x3fa48e41, 64 },
    { 0x55d48e40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c48e46, 95 },
    { 0x7ea48e47, 61 },
    { 0xf0d48e47, 61 },
    { 0x2b848e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a848e50, 23 },
    { 0x38b48e50, 34 },
    { 0, 0 },
    { 0x1f948e53, 93 },
    { 0, 0 },
    { 0xb448e55, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81748e59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b48e5c, 33 },
    { 0, 0 },
    { 0x2fc48e5e, 34 },
    { 0, 0 },
    { 0x1848e60, 43 },
    { 0xf6c48e60, 85 },
    { 0x50348e62, 90 },
    { 0xd2e48e63, 61 },
    { 0x92648e60, 66 },
    { 0xdca48e65, 34 },
    { 0x44348e62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a848e6b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19348e71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d48e7a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64848e80, 76 },
    { 0x2b548e81, 23 },
    { 0xac748e81, 61 },
    { 0, 0 },
    { 0x2cd48e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff448e88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd48e8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd848e91, 23 },
    { 0x1ea48e92, 61 },
    { 0x58048e93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b48e9c, 33 },
    { 0, 0 },
    { 0x5048e9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe948eaa, 98 },
    { 0xac48eab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5548eb4, 23 },
    { 0xcf48eb5, 61 },
    { 0xa6548eb6, 85 },
    { 0xeda48eb7, 90 },
    { 0x4e48eb6, 61 },
    { 0x6bd48eb7, 95 },
    { 0x37048eb4, 23 },
    { 0, 0 },
    { 0xb4c48ebc, 91 },
    { 0x6e548ebd, 17 },
    { 0, 0 },
    { 0x74b48ebf, 90 },
    { 0, 0 },
    { 0x2e748ec1, 95 },
    { 0, 0 },
    { 0x8a448ec3, 23 },
    { 0xa8248ec3, 84 },
    { 0xc0448ec5, 100 },
    { 0, 0 },
    { 0x6a48ec7, 33 },
    { 0, 0 },
    { 0x6ba48ec9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab348ecd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d448ed1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3148ed6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36848edd, 34 },
    { 0x19648ede, 23 },
    { 0x7e748edf, 66 },
    { 0x3fd48ede, 23 },
    { 0, 0 },
    { 0xf1848ee2, 43 },
    { 0x43848ee2, 76 },
    { 0x3d148ee4, 90 },
    { 0xc4748ee5, 61 },
    { 0xe6b48ee6, 17 },
    { 0xc9a48ee6, 23 },
    { 0x99448ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b48eeb, 17 },
    { 0xa0848eeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd48eef, 23 },
    { 0xe448ef0, 78 },
    { 0xcd248ef0, 23 },
    { 0, 0 },
    { 0x10b48ef3, 38 },
    { 0x1af48ef4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88648ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33148efd, 23 },
    { 0x7f548efd, 33 },
    { 0x9c48eff, 43 },
    { 0x4a348eff, 90 },
    { 0xe0948f01, 23 },
    { 0xc1f48eff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8248f05, 61 },
    { 0, 0 },
    { 0xab248f07, 33 },
    { 0x4f148f08, 23 },
    { 0xef48f09, 90 },
    { 0, 0 },
    { 0x12e48f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43748f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc748f14, 23 },
    { 0x22948f15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a48f1e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7348f21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e948f27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df48f2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c48f2f, 34 },
    { 0x94048f2f, 33 },
    { 0xe2448f2f, 46 },
    { 0x7f648f31, 67 },
    { 0xcc748f2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d548f37, 36 },
    { 0xc5e48f38, 33 },
    { 0x2d148f39, 61 },
    { 0xbe248f3a, 76 },
    { 0xc1148f37, 93 },
    { 0x14f48f3c, 33 },
    { 0xb4748f39, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe48f40, 90 },
    { 0x50548f40, 23 },
    { 0, 0 },
    { 0x1e948f43, 95 },
    { 0, 0 },
    { 0x9148f45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b148f4f, 98 },
    { 0xdfd48f4f, 76 },
    { 0xe5348f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab48f56, 43 },
    { 0x34b48f57, 61 },
    { 0xe3148f58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a48f5b, 90 },
    { 0, 0 },
    { 0xcaa48f5d, 67 },
    { 0xce248f5e, 84 },
    { 0x30948f5f, 80 },
    { 0xda648f60, 90 },
    { 0, 0 },
    { 0x3eb48f62, 98 },
    { 0x3048f62, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d48f66, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27048f6b, 90 },
    { 0xcd748f6b, 90 },
    { 0xe1948f6d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a48f71, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e648f74, 90 },
    { 0x63c48f75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d48f78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc348f7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15748f83, 93 },
    { 0x62e48f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12748f87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb848f8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72648f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc348f92, 33 },
    { 0x7648f93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f48f96, 77 },
    { 0, 0 },
    { 0x71a48f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71548fa1, 69 },
    { 0, 0 },
    { 0x22248fa3, 46 },
    { 0xeec48fa3, 77 },
    { 0, 0 },
    { 0xbac48fa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39648faf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4048fb2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd148fb5, 61 },
    { 0, 0 },
    { 0xd0648fb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xded48fba, 66 },
    { 0x2f348fba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b48fc1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61348fca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f48fcd, 43 },
    { 0, 0 },
    { 0xaa048fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a148fd3, 55 },
    { 0, 0 },
    { 0x80c48fd5, 46 },
    { 0x51848fd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f948fe0, 87 },
    { 0x6c748fe1, 98 },
    { 0x16b48fe2, 98 },
    { 0x39448fe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62548feb, 23 },
    { 0x87148fec, 33 },
    { 0x75248feb, 61 },
    { 0, 0 },
    { 0x48e48fef, 100 },
    { 0xd6448ff0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f448ff5, 33 },
    { 0x26848ff6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd48ffe, 97 },
    { 0xa9248ffe, 68 },
    { 0x26e48ffe, 12 },
    { 0xfe549001, 98 },
    { 0x3d149002, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be4900a, 90 },
    { 0x2394900a, 90 },
    { 0xc5d4900a, 95 },
    { 0x5244900a, 90 },
    { 0, 0 },
    { 0xe594900f, 84 },
    { 0x2a34900f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37849013, 95 },
    { 0, 0 },
    { 0x5549015, 87 },
    { 0xa2c49016, 33 },
    { 0xee549017, 17 },
    { 0x12f49018, 61 },
    { 0xbf649018, 43 },
    { 0xbe14901a, 38 },
    { 0xa1b49019, 90 },
    { 0, 0 },
    { 0xc614901d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e949020, 99 },
    { 0, 0 },
    { 0x8b249022, 23 },
    { 0, 0 },
    { 0xd7f49024, 23 },
    { 0x5d949024, 55 },
    { 0xa8d49024, 98 },
    { 0, 0 },
    { 0x64449028, 76 },
    { 0x27549029, 100 },
    { 0x52b4902a, 23 },
    { 0x7d74902a, 76 },
    { 0x13449029, 33 },
    { 0x20449029, 84 },
    { 0x5214902b, 76 },
    { 0xebd4902a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e149037, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb204903a, 88 },
    { 0xb34903b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21049040, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31349043, 95 },
    { 0x93249044, 61 },
    { 0x38a49045, 80 },
    { 0x4cc49046, 33 },
    { 0x4f49046, 55 },
    { 0xf3149046, 23 },
    { 0xb6549045, 55 },
    { 0xf7b4904a, 23 },
    { 0x97f49045, 81 },
    { 0xafb49049, 100 },
    { 0xa3549046, 85 },
    { 0x80a4904e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d49052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c64905a, 77 },
    { 0x2b44905b, 65 },
    { 0x5084905c, 61 },
    { 0xbaf4905a, 69 },
    { 0x3704905e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf949061, 46 },
    { 0xe3549061, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa749065, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce049068, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c4906c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45449070, 61 },
    { 0x44249071, 83 },
    { 0xfd249071, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4907a, 67 },
    { 0x4ab4907a, 87 },
    { 0x39e4907c, 99 },
    { 0, 0 },
    { 0x86e4907e, 34 },
    { 0xba4907f, 66 },
    { 0x5bb4907e, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a49084, 95 },
    { 0x5aa49085, 36 },
    { 0x5ed49086, 52 },
    { 0xa3c49087, 90 },
    { 0x33049088, 95 },
    { 0x28049089, 88 },
    { 0x3b449088, 97 },
    { 0x6e4908a, 38 },
    { 0x5f24908b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d549090, 93 },
    { 0, 0 },
    { 0xf0249092, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67249095, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe64909a, 95 },
    { 0xcae4909b, 61 },
    { 0xc754909a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b64909f, 61 },
    { 0xb4f4909f, 55 },
    { 0xac7490a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9490a7, 98 },
    { 0, 0 },
    { 0x95e490a9, 76 },
    { 0xb3b490a9, 61 },
    { 0, 0 },
    { 0x3ee490ac, 99 },
    { 0, 0 },
    { 0x64b490ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8490b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95490b9, 76 },
    { 0x16a490ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76490bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf490c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba490ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd490cd, 95 },
    { 0xb15490ce, 43 },
    { 0x2fa490cf, 46 },
    { 0xeac490d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72490d4, 93 },
    { 0x3ba490d5, 87 },
    { 0x723490d6, 40 },
    { 0, 0 },
    { 0x5db490d8, 55 },
    { 0x3aa490d9, 23 },
    { 0xeb3490d9, 93 },
    { 0xe28490d9, 23 },
    { 0xc80490dc, 90 },
    { 0xb56490dd, 61 },
    { 0xec0490dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca490e5, 33 },
    { 0xca0490e6, 90 },
    { 0xfe490e7, 55 },
    { 0xc39490e8, 94 },
    { 0xa89490e6, 33 },
    { 0, 0 },
    { 0xbbd490eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x475490ee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f490f5, 93 },
    { 0xb64490f5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x145490f9, 78 },
    { 0xfc5490fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f49103, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd49106, 52 },
    { 0x80149106, 23 },
    { 0x7ee49107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4910d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6449113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f49117, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe984911a, 67 },
    { 0x7ce4911b, 93 },
    { 0x8b84911a, 61 },
    { 0, 0 },
    { 0xf034911e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b149124, 87 },
    { 0, 0 },
    { 0x9649126, 23 },
    { 0x77849127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd814912a, 76 },
    { 0x4004912b, 95 },
    { 0xb124912a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4874912f, 88 },
    { 0, 0 },
    { 0xec049131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc49138, 90 },
    { 0x64049139, 69 },
    { 0x71c49139, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa044913d, 85 },
    { 0, 0 },
    { 0x3374913f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb49144, 33 },
    { 0, 0 },
    { 0x20549146, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d249150, 93 },
    { 0x8bc49151, 90 },
    { 0x8a49152, 61 },
    { 0x20849150, 90 },
    { 0xe5249151, 23 },
    { 0xf2a49152, 76 },
    { 0xc7149156, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa254915c, 98 },
    { 0x64b4915d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff349160, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7149167, 46 },
    { 0x44e49168, 61 },
    { 0, 0 },
    { 0x7bb4916a, 95 },
    { 0x86f4916b, 87 },
    { 0x2d84916b, 98 },
    { 0xdfe4916d, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac49172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a149176, 34 },
    { 0xc3f49177, 99 },
    { 0, 0 },
    { 0x9b49179, 90 },
    { 0xc6c4917a, 81 },
    { 0x8da49179, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1549180, 33 },
    { 0, 0 },
    { 0xffb49182, 75 },
    { 0, 0 },
    { 0x92c49184, 95 },
    { 0xc0149185, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf314918a, 23 },
    { 0, 0 },
    { 0xa504918c, 23 },
    { 0xebe4918c, 66 },
    { 0x11c4918c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc49191, 23 },
    { 0x72e49191, 64 },
    { 0, 0 },
    { 0xe7749194, 65 },
    { 0xd549195, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa574919a, 23 },
    { 0xda24919b, 43 },
    { 0xee74919a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff491a4, 34 },
    { 0xaa3491a5, 66 },
    { 0xc8a491a6, 98 },
    { 0xb74491a6, 61 },
    { 0x745491a7, 95 },
    { 0xfbe491a7, 95 },
    { 0xa91491aa, 67 },
    { 0x6e491ab, 90 },
    { 0, 0 },
    { 0x893491ad, 95 },
    { 0x243491ad, 87 },
    { 0x167491af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60491b2, 61 },
    { 0xfdc491b2, 61 },
    { 0x1e3491b3, 80 },
    { 0, 0 },
    { 0x308491b6, 98 },
    { 0, 0 },
    { 0x8df491b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a491bc, 68 },
    { 0xb01491bc, 33 },
    { 0xd6f491bd, 34 },
    { 0, 0 },
    { 0x499491c0, 23 },
    { 0x878491c1, 61 },
    { 0x433491c2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x853491c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61491c8, 98 },
    { 0x2d4491c9, 33 },
    { 0xc6b491c8, 95 },
    { 0xf99491c9, 90 },
    { 0xeee491c8, 64 },
    { 0xed1491ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb491d3, 98 },
    { 0x570491d4, 98 },
    { 0x671491d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5491e5, 75 },
    { 0xa3491e6, 90 },
    { 0x28c491e7, 95 },
    { 0xb95491e6, 43 },
    { 0x8c0491e9, 83 },
    { 0xb0b491ea, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x196491ed, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49491f2, 90 },
    { 0, 0 },
    { 0x4bd491f4, 90 },
    { 0, 0 },
    { 0x38a491f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7491fa, 33 },
    { 0, 0 },
    { 0x3b2491fc, 33 },
    { 0x58b491fd, 93 },
    { 0, 0 },
    { 0xd22491ff, 33 },
    { 0xe9049200, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d449203, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x83949206, 61 },
    { 0, 0 },
    { 0x79349208, 80 },
    { 0, 0 },
    { 0x7294920a, 77 },
    { 0xa134920b, 77 },
    { 0xe954920b, 85 },
    { 0xf564920c, 93 },
    { 0xac74920e, 61 },
    { 0x4d54920e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6849212, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f49216, 93 },
    { 0, 0 },
    { 0xf9649218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc724921b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d49220, 93 },
    { 0x80f49220, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa149225, 43 },
    { 0xfc249226, 88 },
    { 0x6049225, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e749230, 23 },
    { 0x92d49230, 33 },
    { 0xe2549231, 90 },
    { 0, 0 },
    { 0xc3149234, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc49239, 95 },
    { 0, 0 },
    { 0x74b4923b, 98 },
    { 0, 0 },
    { 0x28b4923d, 76 },
    { 0, 0 },
    { 0x7f44923f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab49242, 33 },
    { 0xf9749243, 33 },
    { 0x63e49244, 23 },
    { 0, 0 },
    { 0x67749246, 76 },
    { 0x91d49247, 33 },
    { 0x69549247, 61 },
    { 0xa0749246, 34 },
    { 0xa3e4924a, 61 },
    { 0x2ca49248, 98 },
    { 0x82a49246, 66 },
    { 0xe964924c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb49252, 90 },
    { 0x7d449253, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb49256, 61 },
    { 0, 0 },
    { 0x8b249258, 99 },
    { 0x14649259, 97 },
    { 0, 0 },
    { 0x3a54925b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f049260, 23 },
    { 0, 0 },
    { 0x34f49262, 34 },
    { 0, 0 },
    { 0xb8c49264, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c649267, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f4926a, 67 },
    { 0x57f4926a, 46 },
    { 0xa2c4926a, 61 },
    { 0x7b64926d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea49271, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f49274, 23 },
    { 0, 0 },
    { 0xdac49276, 98 },
    { 0, 0 },
    { 0x24049278, 33 },
    { 0x26949278, 99 },
    { 0xf3d49279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c4927d, 33 },
    { 0xe664927e, 17 },
    { 0x1744927e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18149288, 33 },
    { 0x65849289, 90 },
    { 0x3004928a, 99 },
    { 0x15e4928b, 90 },
    { 0xab54928c, 61 },
    { 0x52449289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e49292, 33 },
    { 0, 0 },
    { 0xddf49294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8094929b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e492a0, 70 },
    { 0x55a492a1, 69 },
    { 0x1c6492a2, 33 },
    { 0xe4e492a3, 61 },
    { 0xfe6492a2, 90 },
    { 0, 0 },
    { 0x985492a6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0492b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9492b4, 34 },
    { 0x3df492b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x944492b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40492bf, 98 },
    { 0xd5e492c0, 95 },
    { 0, 0 },
    { 0x35d492c2, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80492c5, 95 },
    { 0xb0c492c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92492c9, 90 },
    { 0x834492c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8492cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e492d4, 46 },
    { 0x9f8492d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce492d8, 55 },
    { 0x228492d9, 81 },
    { 0xec492d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b492e1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99492e5, 17 },
    { 0xfd8492e6, 90 },
    { 0xf58492e7, 75 },
    { 0xc90492e8, 68 },
    { 0xfa0492e7, 68 },
    { 0x634492e7, 61 },
    { 0x871492eb, 95 },
    { 0xc56492e9, 78 },
    { 0xf02492ec, 66 },
    { 0x8b7492ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6492f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128492fa, 88 },
    { 0x70492fb, 78 },
    { 0xc0f492fc, 85 },
    { 0, 0 },
    { 0xa51492fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7649302, 91 },
    { 0xd8b49303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8549308, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed84930d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f49315, 52 },
    { 0, 0 },
    { 0x9e349317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd344931a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db4931d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d349322, 61 },
    { 0, 0 },
    { 0x66149324, 76 },
    { 0xf2b49325, 95 },
    { 0xab349326, 90 },
    { 0xf9649327, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e4932a, 91 },
    { 0xfbf4932b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b4932e, 90 },
    { 0xee74932f, 23 },
    { 0x5449330, 46 },
    { 0xe0449331, 90 },
    { 0x49749332, 52 },
    { 0x1e049332, 61 },
    { 0x5f49334, 98 },
    { 0xb4749333, 38 },
    { 0x8e649336, 90 },
    { 0x5549337, 65 },
    { 0xe149338, 23 },
    { 0x1f649339, 90 },
    { 0xba74933a, 61 },
    { 0x9c14933b, 33 },
    { 0xc3f4933a, 76 },
    { 0, 0 },
    { 0x4334933e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4449349, 90 },
    { 0, 0 },
    { 0xe5c4934b, 98 },
    { 0x8914934b, 98 },
    { 0, 0 },
    { 0xf244934e, 100 },
    { 0, 0 },
    { 0x7dd49350, 76 },
    { 0, 0 },
    { 0xc5849352, 33 },
    { 0, 0 },
    { 0x78b49354, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf49357, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4935a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf649363, 61 },
    { 0xbdf49364, 95 },
    { 0xf6049363, 33 },
    { 0x9249363, 98 },
    { 0, 0 },
    { 0x44949368, 90 },
    { 0x3e849369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754936e, 33 },
    { 0x24a4936e, 80 },
    { 0xa214936f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d49374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1949378, 52 },
    { 0x38949379, 76 },
    { 0x5c649379, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949380, 33 },
    { 0, 0 },
    { 0x2b949382, 23 },
    { 0, 0 },
    { 0xed49384, 90 },
    { 0xe9749385, 33 },
    { 0x95449386, 98 },
    { 0x31549387, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74938e, 90 },
    { 0x174938e, 90 },
    { 0xf1f4938e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2149398, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5324939b, 43 },
    { 0xaeb4939c, 90 },
    { 0x96b4939d, 98 },
    { 0x1164939e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503493a2, 95 },
    { 0, 0 },
    { 0x553493a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c493a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a493ad, 98 },
    { 0xbc6493ae, 23 },
    { 0x6f5493ad, 61 },
    { 0x659493af, 61 },
    { 0, 0 },
    { 0x2e0493b2, 43 },
    { 0, 0 },
    { 0xe493b4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47493b8, 33 },
    { 0x4af493b9, 93 },
    { 0x890493ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98493bd, 47 },
    { 0, 0 },
    { 0x57493bf, 61 },
    { 0, 0 },
    { 0xd8c493c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0493c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8493ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d493cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad493d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9493d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68493e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9493e5, 53 },
    { 0x418493e6, 34 },
    { 0, 0 },
    { 0xd06493e8, 69 },
    { 0xe53493e9, 99 },
    { 0xdc9493e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5493ef, 95 },
    { 0x1e4493f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19493f4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6493f7, 90 },
    { 0xbfb493f8, 66 },
    { 0x9c0493f7, 91 },
    { 0x596493fa, 66 },
    { 0xac3493fb, 84 },
    { 0x33b493fc, 61 },
    { 0x603493fd, 93 },
    { 0xd0c493fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4249401, 95 },
    { 0, 0 },
    { 0x7f649403, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5749407, 98 },
    { 0xe6649407, 98 },
    { 0x27149409, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6449410, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb449414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4941b, 87 },
    { 0x1104941b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d849424, 66 },
    { 0x8d449425, 91 },
    { 0, 0 },
    { 0xd8f49427, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b54942f, 34 },
    { 0xa4e49430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f49433, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc149436, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e849440, 43 },
    { 0xa5d49441, 61 },
    { 0x72649442, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e049446, 95 },
    { 0xab749447, 39 },
    { 0xf8349446, 33 },
    { 0xf249449, 66 },
    { 0x5f049449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2449450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf49454, 76 },
    { 0, 0 },
    { 0x9eb49456, 34 },
    { 0x7a149457, 76 },
    { 0x7c949458, 61 },
    { 0x19549459, 90 },
    { 0x8c849459, 61 },
    { 0, 0 },
    { 0xb74945c, 33 },
    { 0x73d4945d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd349460, 61 },
    { 0x96a49461, 33 },
    { 0x22049462, 34 },
    { 0, 0 },
    { 0x2d49464, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c4946a, 66 },
    { 0x6764946a, 46 },
    { 0x3fd4946a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd49470, 33 },
    { 0x23a49471, 90 },
    { 0x87149472, 99 },
    { 0x60249473, 33 },
    { 0xc1749471, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd49478, 97 },
    { 0, 0 },
    { 0xbc94947a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x714947d, 90 },
    { 0x6094947d, 90 },
    { 0xe7b4947f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca49483, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af49488, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3604948c, 61 },
    { 0xf454948d, 91 },
    { 0, 0 },
    { 0x804948f, 65 },
    { 0xab24948f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad49499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a64949c, 95 },
    { 0, 0 },
    { 0x7a74949e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e494aa, 52 },
    { 0xe6c494ab, 98 },
    { 0x732494aa, 33 },
    { 0xfdb494ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703494b4, 98 },
    { 0x3f0494b5, 77 },
    { 0, 0 },
    { 0xca8494b7, 61 },
    { 0x2e3494b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b494bb, 96 },
    { 0xaad494bc, 43 },
    { 0x537494bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466494c1, 90 },
    { 0, 0 },
    { 0x67b494c3, 61 },
    { 0xe1d494c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183494c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd494cb, 98 },
    { 0, 0 },
    { 0xf92494cd, 52 },
    { 0x1f3494ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b494d2, 76 },
    { 0x5a3494d3, 34 },
    { 0, 0 },
    { 0x51b494d5, 95 },
    { 0xa494d6, 33 },
    { 0xdd5494d7, 33 },
    { 0xca8494d7, 43 },
    { 0x370494d9, 100 },
    { 0xe82494d9, 87 },
    { 0x34a494db, 91 },
    { 0x8de494d9, 90 },
    { 0, 0 },
    { 0xe47494de, 91 },
    { 0x7494de, 95 },
    { 0x307494e0, 81 },
    { 0x476494e0, 66 },
    { 0x4d2494de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44494e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8494e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e494ec, 23 },
    { 0, 0 },
    { 0xa40494ee, 65 },
    { 0x34a494ef, 23 },
    { 0x63c494ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d494f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b494fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb494ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2549502, 52 },
    { 0x53c49502, 46 },
    { 0, 0 },
    { 0xb3949505, 17 },
    { 0x1449506, 91 },
    { 0, 0 },
    { 0xdcd49508, 43 },
    { 0, 0 },
    { 0x6d44950a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3749515, 23 },
    { 0, 0 },
    { 0x76549517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb394951c, 68 },
    { 0x8ab4951c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d49527, 68 },
    { 0xaa549528, 66 },
    { 0x74349528, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4144952c, 85 },
    { 0xba54952d, 95 },
    { 0, 0 },
    { 0x3f84952f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x18049532, 33 },
    { 0x4eb49533, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a249538, 93 },
    { 0xabe49539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b64953d, 23 },
    { 0xe54953e, 90 },
    { 0, 0 },
    { 0x54049540, 66 },
    { 0x64049541, 77 },
    { 0xad749542, 95 },
    { 0xa3f49543, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7049552, 95 },
    { 0x10249553, 77 },
    { 0, 0 },
    { 0xd1349555, 23 },
    { 0, 0 },
    { 0xa0049557, 100 },
    { 0x1aa49558, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7554955c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024955f, 23 },
    { 0x26d49560, 93 },
    { 0, 0 },
    { 0x86249562, 65 },
    { 0xd5c49563, 98 },
    { 0xe9f49564, 66 },
    { 0x5b349562, 33 },
    { 0xc2249562, 99 },
    { 0, 0 },
    { 0x45a49568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf4956f, 90 },
    { 0, 0 },
    { 0x3ed49571, 95 },
    { 0x21e49572, 23 },
    { 0, 0 },
    { 0x87749574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef49578, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf284957e, 95 },
    { 0xd344957f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d49582, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd49587, 96 },
    { 0x49c49587, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf104958b, 33 },
    { 0xbb64958c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e49590, 95 },
    { 0x40f49591, 55 },
    { 0xeec49591, 23 },
    { 0x1ca49593, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e64959a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc714959f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4495a4, 33 },
    { 0, 0 },
    { 0x594495a6, 93 },
    { 0xb32495a7, 81 },
    { 0, 0 },
    { 0x582495a9, 90 },
    { 0x668495a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a495ad, 61 },
    { 0x856495ae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c495b5, 61 },
    { 0x375495b6, 95 },
    { 0, 0 },
    { 0x837495b8, 90 },
    { 0x39495b9, 23 },
    { 0x668495ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10495bd, 90 },
    { 0, 0 },
    { 0x7d1495bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf495c2, 76 },
    { 0x51b495c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5495c7, 91 },
    { 0xecb495c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76495cb, 98 },
    { 0x4a0495cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2495d1, 65 },
    { 0x194495d2, 53 },
    { 0x82f495d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2495e0, 84 },
    { 0, 0 },
    { 0xc81495e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9495ea, 98 },
    { 0x434495eb, 76 },
    { 0x21495ec, 99 },
    { 0xd76495ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9495f0, 90 },
    { 0x834495f0, 33 },
    { 0xef7495f0, 84 },
    { 0x861495f3, 85 },
    { 0xa36495f2, 23 },
    { 0, 0 },
    { 0xa9e495f6, 33 },
    { 0, 0 },
    { 0xb9f495f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e495fc, 55 },
    { 0x2de495fd, 98 },
    { 0x35f495fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x62649601, 93 },
    { 0xba349601, 95 },
    { 0x5c149602, 90 },
    { 0xb3749604, 61 },
    { 0x66449604, 61 },
    { 0x94949601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394960b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee4960e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38649613, 33 },
    { 0x64c49614, 97 },
    { 0, 0 },
    { 0x74a49616, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc704961a, 33 },
    { 0, 0 },
    { 0x57e4961c, 87 },
    { 0, 0 },
    { 0xf494961e, 95 },
    { 0, 0 },
    { 0xc6a49620, 23 },
    { 0x32d49620, 97 },
    { 0x15b49622, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d549629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844962f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d749634, 61 },
    { 0x31949635, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59549639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b4963c, 90 },
    { 0x36c4963d, 87 },
    { 0x5d04963e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce849646, 76 },
    { 0xe5649647, 90 },
    { 0xa1149648, 33 },
    { 0x21449649, 93 },
    { 0x4154964a, 94 },
    { 0xb2d49649, 83 },
    { 0x6c14964c, 98 },
    { 0xfc34964d, 52 },
    { 0, 0 },
    { 0xf4b4964f, 98 },
    { 0, 0 },
    { 0x8ea49651, 93 },
    { 0xfad49652, 61 },
    { 0x22449653, 23 },
    { 0x91449653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97949657, 95 },
    { 0x1e149658, 90 },
    { 0xab949659, 90 },
    { 0xd4149657, 61 },
    { 0xad749657, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef94965f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d849662, 23 },
    { 0xcb49662, 98 },
    { 0, 0 },
    { 0x8a49665, 90 },
    { 0x5a349666, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd4966c, 45 },
    { 0x3314966d, 90 },
    { 0, 0 },
    { 0x7d64966f, 34 },
    { 0, 0 },
    { 0x69c49671, 84 },
    { 0x78b49672, 34 },
    { 0x2ff49673, 98 },
    { 0x6e049671, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e4967a, 91 },
    { 0x6c64967b, 23 },
    { 0xd9e4967a, 33 },
    { 0x9764967d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70149681, 78 },
    { 0x19849681, 98 },
    { 0, 0 },
    { 0x43a49684, 33 },
    { 0, 0 },
    { 0xa8349686, 33 },
    { 0xdfd49687, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794968c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3094968f, 78 },
    { 0x7484968f, 95 },
    { 0xf8849690, 90 },
    { 0x4c54968f, 64 },
    { 0x91c49693, 33 },
    { 0xef749694, 90 },
    { 0x36749690, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4249698, 61 },
    { 0x2a549699, 90 },
    { 0xc984969a, 61 },
    { 0x1ff4969b, 98 },
    { 0x4b149698, 91 },
    { 0x1794969d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15496a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa496a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x365496ab, 61 },
    { 0x5de496ac, 52 },
    { 0x1a8496ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340496bf, 100 },
    { 0, 0 },
    { 0x5a1496c1, 23 },
    { 0, 0 },
    { 0x78e496c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66496c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36496ca, 99 },
    { 0x68d496cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e496d0, 90 },
    { 0x5e9496d1, 17 },
    { 0xd71496d0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84496d6, 52 },
    { 0x246496d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72496db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a496de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56496e3, 23 },
    { 0, 0 },
    { 0xf60496e5, 23 },
    { 0x1a2496e6, 90 },
    { 0xa0e496e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32496eb, 81 },
    { 0x65c496ec, 100 },
    { 0x655496eb, 61 },
    { 0x790496eb, 55 },
    { 0, 0 },
    { 0x96496f0, 77 },
    { 0xe69496f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a496f6, 61 },
    { 0xdfe496f7, 43 },
    { 0, 0 },
    { 0x7ab496f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692496fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c49701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b49704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b14970b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf49711, 46 },
    { 0x60e49711, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e49716, 23 },
    { 0x64c49717, 93 },
    { 0xf8149718, 88 },
    { 0x66a49717, 33 },
    { 0xfb64971a, 46 },
    { 0x2214971b, 80 },
    { 0x724971c, 23 },
    { 0x294971a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d49724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5149729, 67 },
    { 0, 0 },
    { 0xfda4972b, 46 },
    { 0x6894972c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b4972f, 91 },
    { 0, 0 },
    { 0xb8649731, 95 },
    { 0xaa949732, 87 },
    { 0x5749733, 87 },
    { 0x8ff49731, 61 },
    { 0xa249732, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8649738, 52 },
    { 0, 0 },
    { 0xb904973a, 17 },
    { 0xc204973a, 98 },
    { 0, 0 },
    { 0x4a4973d, 90 },
    { 0x9bd4973e, 61 },
    { 0, 0 },
    { 0xbcc49740, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40349745, 95 },
    { 0x64e49746, 90 },
    { 0, 0 },
    { 0x37149748, 95 },
    { 0, 0 },
    { 0x3524974a, 23 },
    { 0, 0 },
    { 0xf0f4974c, 46 },
    { 0, 0 },
    { 0x51c4974e, 34 },
    { 0, 0 },
    { 0x85049750, 61 },
    { 0, 0 },
    { 0x1fa49752, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea549759, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe844975c, 33 },
    { 0, 0 },
    { 0x74d4975e, 12 },
    { 0xa1e4975f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8849762, 61 },
    { 0, 0 },
    { 0x13449764, 52 },
    { 0x6a049764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf534976c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45149772, 90 },
    { 0x1bc49773, 61 },
    { 0x82e49774, 61 },
    { 0xca049775, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14977c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa49780, 33 },
    { 0x95249781, 78 },
    { 0, 0 },
    { 0xc1f49783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c249786, 33 },
    { 0xb3d49787, 90 },
    { 0xcf049787, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d49792, 95 },
    { 0xb6049792, 61 },
    { 0xa2649794, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e49798, 85 },
    { 0xa0e49799, 90 },
    { 0x32749798, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1984979e, 76 },
    { 0, 0 },
    { 0x38f497a0, 90 },
    { 0x2e497a1, 33 },
    { 0xfcb497a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429497a7, 95 },
    { 0xaed497a8, 33 },
    { 0, 0 },
    { 0x570497aa, 33 },
    { 0x29c497ab, 99 },
    { 0xa80497aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa497b1, 65 },
    { 0, 0 },
    { 0x6e2497b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153497b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7497bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da497c4, 90 },
    { 0x82d497c5, 85 },
    { 0x137497c6, 34 },
    { 0x965497c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd497d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef497d7, 55 },
    { 0x91d497d8, 99 },
    { 0x43c497d9, 46 },
    { 0x6ad497d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63497e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6497e3, 33 },
    { 0, 0 },
    { 0x1b497e5, 23 },
    { 0, 0 },
    { 0x729497e7, 76 },
    { 0, 0 },
    { 0xc9497e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6497ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3497f1, 87 },
    { 0, 0 },
    { 0xd1a497f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104497fb, 78 },
    { 0x1af497fc, 61 },
    { 0x958497fd, 98 },
    { 0x677497fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa49802, 90 },
    { 0xcab49803, 99 },
    { 0, 0 },
    { 0xe6749805, 33 },
    { 0x8e449806, 98 },
    { 0x73549807, 55 },
    { 0xcfb49808, 33 },
    { 0x89b49805, 23 },
    { 0, 0 },
    { 0x1fe4980b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8824980f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9449815, 23 },
    { 0xcdf49815, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28249820, 67 },
    { 0xb9549820, 23 },
    { 0xe1249821, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28049826, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1949829, 91 },
    { 0x71e49829, 95 },
    { 0xdd74982b, 77 },
    { 0xc7649829, 80 },
    { 0x8414982d, 55 },
    { 0x9084982d, 34 },
    { 0xcfc4982d, 95 },
    { 0x5404982d, 43 },
    { 0xe8549829, 98 },
    { 0xf0e49832, 81 },
    { 0xcb049832, 23 },
    { 0x7894982b, 33 },
    { 0xc154982b, 76 },
    { 0x7dd4982d, 33 },
    { 0x59b4982f, 90 },
    { 0, 0 },
    { 0xe3449839, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b84983c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49843, 90 },
    { 0xf8f49844, 34 },
    { 0xc1d49845, 78 },
    { 0x70949845, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd4984b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f149852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95549855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f64985b, 91 },
    { 0x3d94985c, 52 },
    { 0x7744985d, 98 },
    { 0x8784985d, 23 },
    { 0, 0 },
    { 0xa9c49860, 98 },
    { 0xb949861, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b249868, 61 },
    { 0x49149869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2904986d, 33 },
    { 0, 0 },
    { 0x7064986f, 38 },
    { 0x91e49870, 61 },
    { 0x1e649871, 61 },
    { 0, 0 },
    { 0x5b849873, 98 },
    { 0, 0 },
    { 0xd0c49875, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c4987a, 98 },
    { 0, 0 },
    { 0xd854987c, 61 },
    { 0x76d4987d, 34 },
    { 0x3ff4987e, 61 },
    { 0xc04987d, 99 },
    { 0x81249880, 33 },
    { 0xda149881, 84 },
    { 0x52d49882, 52 },
    { 0x45649883, 27 },
    { 0x7304987e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1949889, 61 },
    { 0, 0 },
    { 0x9b4988b, 76 },
    { 0, 0 },
    { 0x1044988d, 38 },
    { 0x8d94988d, 66 },
    { 0x19a4988f, 91 },
    { 0xc0149890, 77 },
    { 0xcd449890, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49894, 61 },
    { 0x73049895, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef49898, 95 },
    { 0x8d049899, 23 },
    { 0x1e24989a, 33 },
    { 0xdea49899, 34 },
    { 0xa4989b, 95 },
    { 0x1164989d, 55 },
    { 0, 0 },
    { 0xe894989f, 98 },
    { 0xf3a498a0, 85 },
    { 0xd4a498a0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb498a5, 76 },
    { 0, 0 },
    { 0x71a498a7, 43 },
    { 0, 0 },
    { 0x737498a9, 33 },
    { 0, 0 },
    { 0xa63498ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda498b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4498b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d498be, 46 },
    { 0x3fe498bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655498c5, 69 },
    { 0, 0 },
    { 0xb50498c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949498d0, 95 },
    { 0x55f498d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6498d4, 99 },
    { 0, 0 },
    { 0xa55498d6, 90 },
    { 0xbd2498d7, 33 },
    { 0xa38498d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40498db, 34 },
    { 0x27a498db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x891498df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a498e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74498e8, 23 },
    { 0x9b6498e9, 80 },
    { 0, 0 },
    { 0x4b4498eb, 91 },
    { 0x841498ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366498f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50498f4, 87 },
    { 0xe5d498f5, 100 },
    { 0x907498f5, 55 },
    { 0xc5498f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35749900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14549906, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85149909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d4990c, 33 },
    { 0x4604990d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52349911, 46 },
    { 0x76e49912, 33 },
    { 0, 0 },
    { 0xbfc49914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c49918, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a49920, 33 },
    { 0xa1749920, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c49926, 95 },
    { 0xc8849927, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb74992a, 33 },
    { 0, 0 },
    { 0x61d4992c, 43 },
    { 0, 0 },
    { 0xb044992e, 23 },
    { 0x5b34992e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d49933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4993d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3049941, 61 },
    { 0x22b49942, 76 },
    { 0x88749943, 31 },
    { 0x7ab49944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a49947, 43 },
    { 0xf9549948, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b4994d, 91 },
    { 0x57f4994e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9749951, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a49955, 33 },
    { 0xb249956, 55 },
    { 0xb3849957, 98 },
    { 0xc9849956, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d14995d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d49960, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64549963, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x65449966, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4249969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af4996f, 52 },
    { 0, 0 },
    { 0x86749971, 66 },
    { 0, 0 },
    { 0x75049973, 85 },
    { 0x54349973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9224997a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7149980, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d49983, 91 },
    { 0xe9b49984, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b849987, 61 },
    { 0x74c49988, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b44998c, 61 },
    { 0x4244998c, 34 },
    { 0xe54998d, 85 },
    { 0x5d14998f, 93 },
    { 0xa834998c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed49993, 65 },
    { 0x8e249994, 98 },
    { 0xf2d49993, 76 },
    { 0x60749996, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8614999a, 61 },
    { 0xdff4999b, 76 },
    { 0xf1d4999c, 90 },
    { 0, 0 },
    { 0xfce4999e, 52 },
    { 0x7cb4999e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939499a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff499ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e499b1, 100 },
    { 0x64f499b1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602499b6, 33 },
    { 0x1b7499b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9499be, 61 },
    { 0, 0 },
    { 0xe3a499c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f499c7, 55 },
    { 0x3d0499c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd499cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135499cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a499d7, 43 },
    { 0x58b499d8, 96 },
    { 0x5c499d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04499de, 61 },
    { 0, 0 },
    { 0x1a499e0, 91 },
    { 0x32f499e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c499e6, 55 },
    { 0, 0 },
    { 0x344499e8, 61 },
    { 0xd0e499e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68499ed, 95 },
    { 0x1c5499ee, 99 },
    { 0, 0 },
    { 0xdf1499f0, 95 },
    { 0, 0 },
    { 0xe76499f2, 90 },
    { 0, 0 },
    { 0x7e4499f4, 61 },
    { 0, 0 },
    { 0xce4499f6, 55 },
    { 0x72c499f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51499fb, 98 },
    { 0xaf3499fb, 61 },
    { 0x3e8499fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe649a01, 2 },
    { 0, 0 },
    { 0x16c49a03, 23 },
    { 0x67649a04, 55 },
    { 0xb9749a05, 46 },
    { 0x2bb49a06, 90 },
    { 0x53549a07, 90 },
    { 0xf2949a04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b149a0c, 98 },
    { 0, 0 },
    { 0xdc649a0e, 76 },
    { 0xb6c49a0e, 33 },
    { 0x7c349a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f649a14, 81 },
    { 0, 0 },
    { 0xaba49a16, 31 },
    { 0x5d749a16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b449a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a449a21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75249a26, 98 },
    { 0x2d349a27, 93 },
    { 0xad849a28, 95 },
    { 0x6e549a29, 100 },
    { 0, 0 },
    { 0x68f49a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b49a2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc849a39, 68 },
    { 0x76049a39, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf049a3e, 98 },
    { 0xa0d49a3f, 90 },
    { 0x16e49a40, 78 },
    { 0, 0 },
    { 0x3749a42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f49a47, 61 },
    { 0x48f49a48, 99 },
    { 0, 0 },
    { 0xb9049a4a, 98 },
    { 0x3ad49a4b, 36 },
    { 0x3d249a4b, 33 },
    { 0xb4549a4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b49a51, 95 },
    { 0, 0 },
    { 0x19749a53, 66 },
    { 0x62949a54, 95 },
    { 0x68949a55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4149a58, 9 },
    { 0xfb649a59, 95 },
    { 0x68249a5a, 23 },
    { 0x9c149a58, 98 },
    { 0, 0 },
    { 0x1ac49a5d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c49a61, 90 },
    { 0xdf449a62, 76 },
    { 0x7749a63, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3149a69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d49a6e, 43 },
    { 0x52e49a6f, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a449a73, 90 },
    { 0x16349a74, 95 },
    { 0x9a349a74, 93 },
    { 0xa9949a74, 84 },
    { 0, 0 },
    { 0xcae49a78, 55 },
    { 0x3da49a79, 64 },
    { 0xd7749a78, 81 },
    { 0x50949a79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a80, 98 },
    { 0, 0 },
    { 0x59549a82, 95 },
    { 0x8b749a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b49a87, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75049a8f, 33 },
    { 0x93649a90, 61 },
    { 0x7af49a91, 55 },
    { 0xf2149a92, 34 },
    { 0xd6949a93, 68 },
    { 0x26a49a94, 38 },
    { 0x14a49a90, 93 },
    { 0xd8549a96, 99 },
    { 0xc8249a95, 34 },
    { 0x50c49a97, 90 },
    { 0xdf849a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c49a9e, 61 },
    { 0, 0 },
    { 0x78949aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a49aa3, 90 },
    { 0, 0 },
    { 0xb0d49aa5, 91 },
    { 0xd5549aa6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x50149aa9, 76 },
    { 0x6a249aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e049aae, 34 },
    { 0, 0 },
    { 0x2e949ab0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e49ab3, 95 },
    { 0x5b549ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a49aba, 67 },
    { 0xb2249abb, 99 },
    { 0x7349abc, 61 },
    { 0xd6b49abc, 90 },
    { 0x4749abe, 97 },
    { 0, 0 },
    { 0xfcc49ac0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f49ac7, 33 },
    { 0x5e849ac8, 33 },
    { 0xadb49ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e49ad2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a249ad8, 34 },
    { 0xd9849ad8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca649ae2, 76 },
    { 0, 0 },
    { 0xd7549ae4, 61 },
    { 0, 0 },
    { 0xd7c49ae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28549ae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19849af3, 85 },
    { 0x60349af3, 98 },
    { 0xddf49af5, 17 },
    { 0xb2749af5, 95 },
    { 0xbd249af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b49afe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7449b08, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb49b0c, 33 },
    { 0x15e49b0c, 91 },
    { 0xe5d49b0e, 90 },
    { 0x73849b0e, 91 },
    { 0xbc749b10, 55 },
    { 0x2ef49b11, 33 },
    { 0x58f49b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37849b15, 76 },
    { 0x1ae49b16, 55 },
    { 0, 0 },
    { 0x91749b18, 99 },
    { 0xe7549b19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5949b1d, 95 },
    { 0xf2a49b1e, 90 },
    { 0, 0 },
    { 0x58849b20, 76 },
    { 0x26d49b20, 55 },
    { 0, 0 },
    { 0x7bf49b23, 65 },
    { 0x4d449b24, 23 },
    { 0x98e49b24, 33 },
    { 0x26649b26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19649b29, 100 },
    { 0xd5249b2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f449b2d, 87 },
    { 0, 0 },
    { 0x4ac49b2f, 52 },
    { 0xa3549b30, 98 },
    { 0xadc49b2f, 23 },
    { 0xd2c49b2f, 98 },
    { 0xd3649b33, 68 },
    { 0x5e949b31, 43 },
    { 0, 0 },
    { 0xec249b36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83049b39, 98 },
    { 0x92649b3a, 90 },
    { 0x36349b3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc849b40, 90 },
    { 0x5ec49b40, 46 },
    { 0x2249b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a49b46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62349b49, 78 },
    { 0x5ee49b4a, 95 },
    { 0x4449b49, 87 },
    { 0xf1449b4b, 22 },
    { 0x6d249b4d, 77 },
    { 0x8d949b4e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9449b54, 33 },
    { 0, 0 },
    { 0x66f49b56, 55 },
    { 0x3df49b57, 90 },
    { 0x2c249b57, 98 },
    { 0x4e49b56, 33 },
    { 0x1cf49b57, 91 },
    { 0x7e649b56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83649b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1049b64, 61 },
    { 0x1b749b64, 98 },
    { 0x1ce49b65, 23 },
    { 0xeda49b67, 43 },
    { 0x94949b68, 90 },
    { 0x4a349b66, 23 },
    { 0, 0 },
    { 0xdeb49b6b, 90 },
    { 0, 0 },
    { 0x84f49b6d, 67 },
    { 0, 0 },
    { 0x59849b6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f149b74, 99 },
    { 0, 0 },
    { 0x70e49b76, 77 },
    { 0, 0 },
    { 0xd049b78, 98 },
    { 0x1e649b78, 87 },
    { 0, 0 },
    { 0xdd49b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd49b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a749b84, 98 },
    { 0, 0 },
    { 0x92649b86, 98 },
    { 0x15949b87, 33 },
    { 0x22049b88, 23 },
    { 0xfd049b89, 90 },
    { 0x95e49b89, 33 },
    { 0xc8e49b8a, 23 },
    { 0x7a349b87, 61 },
    { 0x14e49b8d, 36 },
    { 0x9d949b8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d49b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5249b99, 78 },
    { 0x18849b9a, 84 },
    { 0x3a349b9b, 76 },
    { 0x3e249b9c, 67 },
    { 0x99d49b9a, 98 },
    { 0, 0 },
    { 0x4e349b9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3049bae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66149bb5, 46 },
    { 0x20349bb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a49bba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f49bbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e349bc0, 43 },
    { 0x41049bc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1649bcb, 43 },
    { 0xb1849bcc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x57349bcf, 93 },
    { 0x14a49bcf, 33 },
    { 0x8649bd1, 95 },
    { 0x8a849bd2, 61 },
    { 0x3bb49bd3, 90 },
    { 0x4ea49bd1, 61 },
    { 0, 0 },
    { 0x449bd6, 61 },
    { 0xf8e49bd6, 91 },
    { 0x61b49bd6, 61 },
    { 0, 0 },
    { 0xfdb49bda, 36 },
    { 0, 0 },
    { 0xaf249bdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea949be1, 90 },
    { 0, 0 },
    { 0x2c849be3, 67 },
    { 0xd3349be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55849be7, 46 },
    { 0xa6f49be8, 33 },
    { 0xa9a49be7, 66 },
    { 0x54949bea, 61 },
    { 0, 0 },
    { 0x54949bec, 17 },
    { 0x5ff49bed, 87 },
    { 0xd249bec, 34 },
    { 0x2fd49bed, 17 },
    { 0, 0 },
    { 0xea749bf1, 65 },
    { 0xfc549bf1, 23 },
    { 0x1e749bf1, 34 },
    { 0x5fd49bf3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a49bf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71049bfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4249bff, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac49c02, 100 },
    { 0xd6649c03, 76 },
    { 0, 0 },
    { 0xbdf49c05, 76 },
    { 0xdaa49c06, 23 },
    { 0x5f449c07, 95 },
    { 0, 0 },
    { 0x9e249c09, 84 },
    { 0xea549c09, 84 },
    { 0xd8549c0b, 33 },
    { 0xbff49c0a, 76 },
    { 0, 0 },
    { 0xb749c0e, 61 },
    { 0x85349c0e, 90 },
    { 0x449c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35849c13, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3349c16, 98 },
    { 0xbf049c16, 34 },
    { 0, 0 },
    { 0xab549c19, 78 },
    { 0x3cb49c19, 23 },
    { 0x61d49c1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d49c1f, 76 },
    { 0x5ba49c20, 100 },
    { 0xddf49c21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75949c29, 46 },
    { 0x75c49c2a, 90 },
    { 0, 0 },
    { 0x3d949c2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5249c32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f149c36, 33 },
    { 0xe9849c37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87649c3b, 55 },
    { 0, 0 },
    { 0xc4949c3d, 33 },
    { 0xc1949c3e, 90 },
    { 0x9849c3e, 80 },
    { 0x13e49c40, 77 },
    { 0x33749c41, 66 },
    { 0xceb49c42, 53 },
    { 0x6c549c43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf49c47, 33 },
    { 0x3b49c47, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6849c4b, 90 },
    { 0x45f49c4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a749c50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73449c56, 23 },
    { 0x65949c57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec249c5a, 18 },
    { 0, 0 },
    { 0xa9749c5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6249c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e49c6c, 33 },
    { 0xf3349c6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a49c74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b49c7b, 98 },
    { 0x94e49c7b, 90 },
    { 0x2e549c7d, 90 },
    { 0, 0 },
    { 0x6e749c7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24149c82, 43 },
    { 0, 0 },
    { 0xe5249c84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0449c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e49c8c, 95 },
    { 0x3f349c8c, 46 },
    { 0, 0 },
    { 0x99049c8f, 98 },
    { 0x98049c90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc049c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa449c97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8549c9a, 100 },
    { 0xf3849c9b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28249ca0, 55 },
    { 0xc9a49ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba49ca4, 98 },
    { 0xbb649ca4, 61 },
    { 0x2fb49ca5, 90 },
    { 0x7dd49ca4, 23 },
    { 0x37d49ca7, 46 },
    { 0xc3849ca9, 75 },
    { 0xf9b49caa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f49caf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd49cb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d49cb6, 90 },
    { 0xaaf49cb7, 90 },
    { 0x2df49cb8, 23 },
    { 0xbb049cb9, 61 },
    { 0x52c49cba, 33 },
    { 0xe0b49cbb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d649cc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e49cc6, 33 },
    { 0, 0 },
    { 0xa1149cc8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x86349ccb, 100 },
    { 0, 0 },
    { 0x48f49ccd, 40 },
    { 0x6c649cce, 90 },
    { 0x68649ccf, 42 },
    { 0xec949cd0, 46 },
    { 0x36649cd0, 55 },
    { 0x57549cd1, 76 },
    { 0x59349cd1, 90 },
    { 0, 0 },
    { 0xbdd49cd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0049cd8, 78 },
    { 0x46e49cd9, 55 },
    { 0, 0 },
    { 0x1c549cdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88649cef, 8 },
    { 0x15e49cf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c449cf4, 84 },
    { 0x92b49cf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92149cfb, 95 },
    { 0x2e049cfc, 33 },
    { 0x59e49cfd, 76 },
    { 0x449cfc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b549d03, 55 },
    { 0xcf349d04, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb149d09, 34 },
    { 0, 0 },
    { 0xa2649d0b, 23 },
    { 0x7b149d0b, 95 },
    { 0xd7949d0d, 48 },
    { 0x9b449d0e, 61 },
    { 0x91d49d0f, 56 },
    { 0, 0 },
    { 0xd4f49d11, 66 },
    { 0xd9949d12, 61 },
    { 0xc6249d11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9249d20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7149d23, 90 },
    { 0, 0 },
    { 0x5f149d25, 90 },
    { 0x28949d26, 23 },
    { 0x3a149d27, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0349d2b, 33 },
    { 0x66049d2b, 23 },
    { 0x26649d2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96849d35, 95 },
    { 0, 0 },
    { 0xc3749d37, 43 },
    { 0xef349d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb49d40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5249d43, 90 },
    { 0xf3849d44, 98 },
    { 0xff349d44, 95 },
    { 0xb0249d46, 83 },
    { 0x1ff49d47, 98 },
    { 0x6a449d48, 33 },
    { 0x91549d47, 96 },
    { 0x5a549d4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31649d51, 91 },
    { 0, 0 },
    { 0x54949d53, 90 },
    { 0x96349d53, 95 },
    { 0x6e149d55, 61 },
    { 0x33049d56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4149d5a, 77 },
    { 0, 0 },
    { 0x1be49d5c, 90 },
    { 0x26149d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7649d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9949d65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9649d69, 90 },
    { 0, 0 },
    { 0x9dd49d6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a549d71, 61 },
    { 0, 0 },
    { 0x9fa49d73, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c49d76, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65649d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f949d81, 82 },
    { 0x5dc49d82, 43 },
    { 0, 0 },
    { 0xad49d84, 85 },
    { 0x99449d85, 55 },
    { 0, 0 },
    { 0xdb049d87, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd349d8c, 92 },
    { 0xbf049d8d, 95 },
    { 0x30f49d8d, 90 },
    { 0x1e349d8f, 23 },
    { 0xa0849d8f, 88 },
    { 0xcec49d8c, 61 },
    { 0x95949d92, 76 },
    { 0, 0 },
    { 0x2c549d94, 87 },
    { 0xfc149d95, 90 },
    { 0x16b49d96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb49d9a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e49d9f, 78 },
    { 0, 0 },
    { 0xe9b49da1, 33 },
    { 0xf9949da2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16049da6, 61 },
    { 0x3ba49da7, 95 },
    { 0x80349da8, 78 },
    { 0x3349da9, 88 },
    { 0x95149daa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e49db1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7949db6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73849dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4549dc0, 33 },
    { 0, 0 },
    { 0x3db49dc2, 61 },
    { 0xb7e49dc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc949dc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a49dc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf49dd0, 55 },
    { 0, 0 },
    { 0x44549dd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81349dd9, 95 },
    { 0, 0 },
    { 0x21049ddb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b49de0, 34 },
    { 0xf6b49de0, 61 },
    { 0x9d849de2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf49de7, 66 },
    { 0, 0 },
    { 0x73249de9, 91 },
    { 0, 0 },
    { 0xb9d49deb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed49def, 67 },
    { 0, 0 },
    { 0xf0149df1, 93 },
    { 0, 0 },
    { 0x40f49df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d49df6, 23 },
    { 0x1f749df6, 33 },
    { 0, 0 },
    { 0xaaa49df9, 61 },
    { 0x12649dfa, 66 },
    { 0xbd649dfb, 61 },
    { 0, 0 },
    { 0x9ed49dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd149e00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba49e04, 78 },
    { 0x16649e05, 98 },
    { 0x86a49e06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc49e0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5449e0e, 55 },
    { 0, 0 },
    { 0x35249e10, 66 },
    { 0, 0 },
    { 0x55049e12, 61 },
    { 0x62b49e12, 98 },
    { 0xef649e14, 90 },
    { 0x9a549e15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7649e1a, 37 },
    { 0x17849e1b, 90 },
    { 0, 0 },
    { 0x22949e1d, 67 },
    { 0, 0 },
    { 0x7aa49e1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b049e25, 90 },
    { 0xc7e49e26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b49e29, 68 },
    { 0x89b49e2a, 61 },
    { 0x85649e2a, 77 },
    { 0, 0 },
    { 0xae549e2d, 61 },
    { 0xb5749e2e, 90 },
    { 0x61949e2e, 61 },
    { 0x42049e2e, 98 },
    { 0x3a949e2d, 33 },
    { 0x9ae49e30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a49e37, 61 },
    { 0x9c249e38, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x95549e3b, 94 },
    { 0xa4f49e3b, 98 },
    { 0x90e49e3d, 90 },
    { 0, 0 },
    { 0xdda49e3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d49e42, 43 },
    { 0xc4749e42, 90 },
    { 0x66649e43, 61 },
    { 0xf6449e43, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd49e49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb049e54, 87 },
    { 0x8ce49e54, 52 },
    { 0xb0449e55, 33 },
    { 0x46b49e57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78049e5b, 76 },
    { 0, 0 },
    { 0x5549e5d, 95 },
    { 0, 0 },
    { 0x7b749e5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a49e63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2749e68, 43 },
    { 0x34949e69, 90 },
    { 0xab49e6a, 98 },
    { 0x7ae49e68, 61 },
    { 0x9a349e6c, 96 },
    { 0x93f49e69, 38 },
    { 0x60d49e6d, 99 },
    { 0, 0 },
    { 0xa3549e70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd049e79, 46 },
    { 0xc6849e7a, 33 },
    { 0x71649e79, 98 },
    { 0x42c49e7c, 85 },
    { 0xd8849e7d, 28 },
    { 0x18749e7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a49e84, 46 },
    { 0xcbb49e84, 46 },
    { 0, 0 },
    { 0x3bd49e87, 93 },
    { 0, 0 },
    { 0x88849e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f749e8f, 55 },
    { 0, 0 },
    { 0xcc149e91, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b049e95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb649e98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac49e9b, 46 },
    { 0xdc349e9c, 95 },
    { 0xaec49e9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d049ea0, 33 },
    { 0x3d49ea0, 95 },
    { 0xce249ea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9149ea7, 68 },
    { 0x6a849ea7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab749eab, 90 },
    { 0xc2549eac, 90 },
    { 0xb4d49eac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1849eb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b49eb4, 98 },
    { 0x3bb49eb5, 34 },
    { 0x86049eb6, 95 },
    { 0xd3c49eb6, 43 },
    { 0xd7849eb8, 76 },
    { 0x21549eb7, 34 },
    { 0, 0 },
    { 0xf8049ebb, 33 },
    { 0x79949ebc, 85 },
    { 0xf6849ebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47149ec4, 55 },
    { 0xa2049ec5, 47 },
    { 0xc8149ec6, 90 },
    { 0xf1c49ec6, 98 },
    { 0x50d49ec8, 17 },
    { 0x82149ec8, 61 },
    { 0x78f49ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3149ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb49ed1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e49ed4, 33 },
    { 0xa9d49ed5, 34 },
    { 0xf4c49ed4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649edf, 96 },
    { 0x75f49ee0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a49ee5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d49ee8, 61 },
    { 0xb1249ee8, 98 },
    { 0xabe49eea, 34 },
    { 0xe6849eea, 100 },
    { 0x84649eec, 98 },
    { 0x29b49eed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95249ef7, 55 },
    { 0, 0 },
    { 0xd2649ef9, 33 },
    { 0xb7149efa, 33 },
    { 0xb6249efb, 17 },
    { 0x70849efc, 33 },
    { 0x20e49efd, 88 },
    { 0, 0 },
    { 0xc5049eff, 91 },
    { 0xaac49f00, 52 },
    { 0xd5949f01, 33 },
    { 0xbfb49f02, 46 },
    { 0xd1c49f02, 90 },
    { 0x33349f04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95549f0a, 61 },
    { 0xec449f0a, 95 },
    { 0x55749f0c, 98 },
    { 0x3cd49f0d, 90 },
    { 0x4b049f0b, 62 },
    { 0x3fd49f0b, 90 },
    { 0xabd49f10, 34 },
    { 0x6e849f0f, 34 },
    { 0x9e649f0b, 90 },
    { 0x18349f13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b549f17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f49f1b, 88 },
    { 0x88949f1c, 90 },
    { 0, 0 },
    { 0x61049f1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a49f22, 55 },
    { 0xb8649f23, 81 },
    { 0, 0 },
    { 0x3af49f25, 90 },
    { 0xfa849f26, 33 },
    { 0, 0 },
    { 0x42c49f28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70949f30, 84 },
    { 0, 0 },
    { 0x31349f32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c49f37, 61 },
    { 0x14d49f38, 90 },
    { 0x43049f39, 78 },
    { 0x78049f39, 23 },
    { 0x7ad49f3a, 34 },
    { 0, 0 },
    { 0x58549f3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5349f42, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f049f46, 33 },
    { 0xe8249f46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb449f4c, 38 },
    { 0xc249f4c, 95 },
    { 0xa1d49f4e, 90 },
    { 0x9c549f4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19449f52, 95 },
    { 0x8f849f53, 33 },
    { 0xe0549f52, 84 },
    { 0, 0 },
    { 0x90c49f56, 34 },
    { 0x82549f57, 33 },
    { 0x61349f56, 34 },
    { 0x5d749f58, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14749f5d, 46 },
    { 0x8f649f5d, 81 },
    { 0xd7449f5f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5249f64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66249f67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38749f6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d649f70, 23 },
    { 0, 0 },
    { 0xd2149f72, 98 },
    { 0xafc49f72, 46 },
    { 0xd2749f73, 23 },
    { 0x44c49f75, 87 },
    { 0x64949f75, 90 },
    { 0x36649f75, 34 },
    { 0xe6649f78, 23 },
    { 0x7a749f78, 36 },
    { 0xe7049f7a, 51 },
    { 0xd149f75, 98 },
    { 0x9c249f72, 55 },
    { 0xf7549f7a, 90 },
    { 0xbb549f7e, 34 },
    { 0, 0 },
    { 0x6af49f80, 95 },
    { 0xf7449f81, 23 },
    { 0, 0 },
    { 0xd8149f83, 76 },
    { 0xce49f84, 90 },
    { 0x11e49f84, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a249f88, 100 },
    { 0xe8649f88, 34 },
    { 0x35249f8a, 23 },
    { 0x75049f8b, 98 },
    { 0x94a49f89, 98 },
    { 0, 0 },
    { 0x8d249f8e, 33 },
    { 0x97249f8f, 66 },
    { 0x3fc49f8f, 95 },
    { 0x68349f91, 23 },
    { 0xed249f91, 90 },
    { 0x6e349f92, 98 },
    { 0x4ae49f94, 61 },
    { 0xd5a49f95, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6949f9c, 90 },
    { 0x15949f9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5949fa0, 61 },
    { 0x40e49fa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be49fa4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc49fac, 40 },
    { 0x38c49fad, 90 },
    { 0x19349fae, 98 },
    { 0xa6b49fae, 33 },
    { 0x12e49fb0, 98 },
    { 0xdf349fb0, 76 },
    { 0, 0 },
    { 0x84949fb3, 18 },
    { 0xa6c49fb4, 80 },
    { 0, 0 },
    { 0x26349fb6, 76 },
    { 0x95849fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32549fba, 61 },
    { 0x7e549fbb, 81 },
    { 0x7c449fbb, 23 },
    { 0x3e549fbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a49fc0, 90 },
    { 0xc7b49fc0, 46 },
    { 0x89b49fc1, 34 },
    { 0x47549fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83649fc9, 61 },
    { 0xe0f49fca, 93 },
    { 0xa2049fca, 76 },
    { 0xa1a49fcc, 78 },
    { 0, 0 },
    { 0x5d449fce, 34 },
    { 0x3cf49fce, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f49fd4, 33 },
    { 0x77849fd5, 90 },
    { 0x9749fd6, 98 },
    { 0x8b049fd6, 95 },
    { 0x5aa49fd7, 46 },
    { 0, 0 },
    { 0x77b49fda, 93 },
    { 0x88749fdb, 90 },
    { 0x21849fda, 76 },
    { 0x6ba49fdb, 55 },
    { 0xa7549fde, 34 },
    { 0, 0 },
    { 0xaa049fe0, 61 },
    { 0x62449fe1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46949fe5, 85 },
    { 0, 0 },
    { 0x6dc49fe7, 61 },
    { 0xdfc49fe7, 33 },
    { 0x69549fe7, 98 },
    { 0x2949fe8, 76 },
    { 0x50149feb, 55 },
    { 0x14749fea, 95 },
    { 0, 0 },
    { 0x9449fee, 33 },
    { 0xad949fef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f49ff5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9949ffa, 33 },
    { 0, 0 },
    { 0xd0349ffc, 61 },
    { 0x4ad49ffd, 34 },
    { 0, 0 },
    { 0xec949fff, 95 },
    { 0x5fa4a000, 76 },
    { 0xa2c49fff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b04a00c, 53 },
    { 0x76d4a00d, 90 },
    { 0x50f4a00d, 61 },
    { 0x3614a00f, 66 },
    { 0xa5a4a00f, 98 },
    { 0, 0 },
    { 0xa814a012, 53 },
    { 0, 0 },
    { 0xe924a014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb54a017, 61 },
    { 0x5984a017, 33 },
    { 0, 0 },
    { 0xb504a01a, 43 },
    { 0x4964a01b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e4a01e, 33 },
    { 0x1584a01e, 52 },
    { 0xe7a4a020, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd694a024, 52 },
    { 0x21d4a025, 93 },
    { 0x4884a026, 65 },
    { 0, 0 },
    { 0xc5d4a028, 34 },
    { 0x90e4a028, 76 },
    { 0xc904a028, 23 },
    { 0, 0 },
    { 0xe924a02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba4a033, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed4a038, 98 },
    { 0, 0 },
    { 0x3ad4a03a, 91 },
    { 0, 0 },
    { 0x174a03c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea94a040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a44a043, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x774a046, 23 },
    { 0x6a94a047, 43 },
    { 0, 0 },
    { 0x1a4a049, 75 },
    { 0x8e64a04a, 68 },
    { 0x7fa4a04b, 34 },
    { 0xe554a04c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4574a04f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f04a052, 93 },
    { 0x9344a053, 23 },
    { 0xddd4a054, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9314a057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4a060, 66 },
    { 0xd44a060, 34 },
    { 0x4104a061, 90 },
    { 0x82d4a063, 90 },
    { 0x8714a064, 90 },
    { 0, 0 },
    { 0xdfb4a066, 43 },
    { 0x1214a067, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c54a06f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf764a072, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14a081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb374a084, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac64a08a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764a08d, 93 },
    { 0xa814a08d, 33 },
    { 0xa5a4a08f, 67 },
    { 0, 0 },
    { 0xf0c4a091, 53 },
    { 0, 0 },
    { 0x60c4a093, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d34a098, 90 },
    { 0, 0 },
    { 0x2604a09a, 61 },
    { 0x2434a09a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b4a09f, 93 },
    { 0x1244a0a0, 33 },
    { 0x6734a09f, 90 },
    { 0, 0 },
    { 0xc4c4a0a3, 98 },
    { 0xa144a0a4, 90 },
    { 0xc704a0a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f4a0b7, 34 },
    { 0x3e54a0b7, 17 },
    { 0, 0 },
    { 0xc624a0ba, 95 },
    { 0x2264a0bb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe354a0c1, 76 },
    { 0x11b4a0c2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1904a0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9484a0cb, 46 },
    { 0x2064a0cc, 95 },
    { 0x1f14a0cc, 90 },
    { 0xb394a0ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d4a0d4, 88 },
    { 0x2144a0d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d4a0dc, 23 },
    { 0, 0 },
    { 0x10c4a0de, 38 },
    { 0x5654a0df, 90 },
    { 0x784a0de, 48 },
    { 0x72c4a0e1, 90 },
    { 0, 0 },
    { 0x4714a0e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a4a0e6, 52 },
    { 0x42c4a0e7, 91 },
    { 0x47b4a0e8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd494a0ef, 34 },
    { 0xc84a0f0, 33 },
    { 0x4344a0f1, 98 },
    { 0xb9e4a0f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6024a0f5, 98 },
    { 0xd414a0f6, 98 },
    { 0, 0 },
    { 0x3744a0f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb4a0fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34a106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d84a10b, 88 },
    { 0xbba4a10b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc24a111, 76 },
    { 0x1ec4a111, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec74a117, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb04a11c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1744a122, 95 },
    { 0, 0 },
    { 0x3da4a124, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794a12b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8004a12f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4a132, 98 },
    { 0x1704a132, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac4a137, 91 },
    { 0, 0 },
    { 0xe854a139, 46 },
    { 0x5c94a139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b74a13d, 75 },
    { 0, 0 },
    { 0x6554a13f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a4a142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba84a145, 53 },
    { 0xe0b4a146, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9944a152, 34 },
    { 0x4f54a152, 90 },
    { 0, 0 },
    { 0xdfd4a155, 95 },
    { 0x9fb4a156, 61 },
    { 0, 0 },
    { 0x53e4a158, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f4a15c, 95 },
    { 0xc094a15d, 93 },
    { 0xde4a15d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee54a163, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd744a168, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd64a16e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3034a173, 23 },
    { 0, 0 },
    { 0xf984a175, 81 },
    { 0x7334a176, 52 },
    { 0x5d04a177, 64 },
    { 0x4924a177, 55 },
    { 0x1f74a176, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf4a17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd684a182, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f4a185, 90 },
    { 0x7ba4a186, 91 },
    { 0xc724a186, 33 },
    { 0x55f4a188, 93 },
    { 0x95a4a189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb494a18d, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0xad84a190, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a4a198, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f4a19f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc214a1a2, 55 },
    { 0, 0 },
    { 0xb04a1a4, 95 },
    { 0x6fe4a1a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf654a1b3, 93 },
    { 0x1c14a1b3, 55 },
    { 0x98d4a1b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f4a1ba, 52 },
    { 0xb444a1bb, 90 },
    { 0x1434a1bb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd754a1bf, 52 },
    { 0x4124a1c0, 95 },
    { 0x5344a1c0, 33 },
    { 0xb6e4a1c2, 67 },
    { 0xe264a1c0, 84 },
    { 0xd354a1c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe4a1c8, 76 },
    { 0x6294a1c9, 76 },
    { 0xdfc4a1c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a54a1cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db4a1d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b4a1d5, 84 },
    { 0xe734a1d6, 64 },
    { 0, 0 },
    { 0x6384a1d8, 46 },
    { 0, 0 },
    { 0x8154a1da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4394a1dd, 84 },
    { 0x794a1dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1264a1e2, 90 },
    { 0, 0 },
    { 0xd914a1e4, 77 },
    { 0x48a4a1e5, 61 },
    { 0x5ce4a1e6, 55 },
    { 0x3854a1e7, 33 },
    { 0x7f54a1e6, 90 },
    { 0x2ea4a1e9, 55 },
    { 0x28e4a1e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db4a1ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c4a1f5, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b4a1fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c34a201, 87 },
    { 0xe54a201, 85 },
    { 0, 0 },
    { 0x98d4a204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e54a20a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4a213, 23 },
    { 0xa1c4a213, 33 },
    { 0, 0 },
    { 0x7b04a216, 56 },
    { 0x76d4a217, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8114a21d, 77 },
    { 0x2ad4a21e, 46 },
    { 0x6924a21f, 34 },
    { 0x5104a21f, 34 },
    { 0x4c84a21e, 55 },
    { 0x8974a222, 61 },
    { 0x5764a223, 95 },
    { 0x574a224, 52 },
    { 0x8f94a222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c4a228, 61 },
    { 0x5c84a228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f14a22d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1314a231, 34 },
    { 0x134a232, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554a235, 23 },
    { 0xb4d4a235, 61 },
    { 0, 0 },
    { 0x1144a238, 87 },
    { 0x2654a239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc064a23d, 66 },
    { 0, 0 },
    { 0xb744a23f, 89 },
    { 0xee44a240, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2574a243, 64 },
    { 0, 0 },
    { 0x2e54a245, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6474a248, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f94a24b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1364a24f, 99 },
    { 0xd0a4a24f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a74a255, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec4a260, 90 },
    { 0x9eb4a260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a265, 23 },
    { 0xbc14a266, 90 },
    { 0x46b4a266, 76 },
    { 0xa094a265, 90 },
    { 0xa544a269, 46 },
    { 0xf904a269, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5084a26f, 76 },
    { 0x72c4a270, 61 },
    { 0, 0 },
    { 0x5324a272, 43 },
    { 0, 0 },
    { 0xf614a274, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8154a278, 90 },
    { 0xf444a279, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c24a27d, 90 },
    { 0x19a4a27d, 33 },
    { 0x3064a27f, 90 },
    { 0x6494a280, 23 },
    { 0xc804a281, 66 },
    { 0x8804a282, 84 },
    { 0x8a74a283, 33 },
    { 0x3124a284, 93 },
    { 0xd534a285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d4a288, 101 },
    { 0xa7d4a289, 95 },
    { 0x7a34a288, 81 },
    { 0x5134a28b, 95 },
    { 0x8ba4a28b, 34 },
    { 0xafd4a28c, 34 },
    { 0x9234a28e, 93 },
    { 0x80f4a28f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8824a292, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf304a296, 90 },
    { 0, 0 },
    { 0x1414a298, 91 },
    { 0, 0 },
    { 0x5ca4a29a, 78 },
    { 0xf234a29a, 78 },
    { 0, 0 },
    { 0x11c4a29d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa074a2a1, 33 },
    { 0, 0 },
    { 0xfc74a2a3, 95 },
    { 0x9394a2a4, 52 },
    { 0xd1f4a2a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e14a2aa, 61 },
    { 0x83d4a2aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a4a2b4, 33 },
    { 0x6e4a2b4, 52 },
    { 0x28c4a2b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa44a2b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7804a2be, 85 },
    { 0x9984a2bf, 100 },
    { 0x4e94a2be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e4a2c7, 34 },
    { 0xdc94a2c8, 34 },
    { 0x36b4a2c8, 98 },
    { 0xaca4a2c8, 98 },
    { 0, 0 },
    { 0x884a2cc, 61 },
    { 0xffc4a2cd, 23 },
    { 0xd884a2ce, 33 },
    { 0x7864a2cd, 23 },
    { 0x434a2cd, 33 },
    { 0xb014a2d0, 33 },
    { 0xd184a2d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x364a2d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf94a2d8, 93 },
    { 0xbea4a2d9, 96 },
    { 0, 0 },
    { 0x2214a2db, 51 },
    { 0xb934a2db, 33 },
    { 0x6744a2dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2654a2e1, 90 },
    { 0x9dc4a2e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2304a2e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c4a2ee, 85 },
    { 0, 0 },
    { 0x67a4a2f0, 98 },
    { 0x1404a2f0, 34 },
    { 0x1f34a2f2, 95 },
    { 0x9484a2f3, 33 },
    { 0, 0 },
    { 0x8834a2f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c64a2f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f14a2ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf564a305, 95 },
    { 0, 0 },
    { 0xd124a307, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a30a, 35 },
    { 0, 0 },
    { 0x50f4a30c, 33 },
    { 0x5b54a30d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9804a310, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd4a314, 76 },
    { 0xae24a315, 33 },
    { 0xa924a316, 55 },
    { 0xfc64a317, 98 },
    { 0x19a4a318, 90 },
    { 0xd034a318, 33 },
    { 0xa224a31a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea4a31d, 90 },
    { 0xb894a31e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74a324, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4a327, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e24a32d, 87 },
    { 0, 0 },
    { 0x2664a32f, 98 },
    { 0x2844a330, 33 },
    { 0x2db4a331, 66 },
    { 0xd744a331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2914a336, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c4a339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e4a33d, 92 },
    { 0xe2e4a33e, 85 },
    { 0x8ee4a33f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4a347, 67 },
    { 0, 0 },
    { 0x2ac4a349, 95 },
    { 0x6514a34a, 55 },
    { 0, 0 },
    { 0xbf24a34c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec4a350, 78 },
    { 0x2764a351, 67 },
    { 0xd854a350, 98 },
    { 0xb504a353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f4a357, 90 },
    { 0x6394a358, 88 },
    { 0x41b4a358, 90 },
    { 0x3a04a35a, 34 },
    { 0x55f4a35b, 76 },
    { 0xa734a358, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2714a360, 43 },
    { 0x9d34a361, 61 },
    { 0x6d64a360, 76 },
    { 0x3504a362, 81 },
    { 0x1ab4a360, 33 },
    { 0xf444a365, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d54a36d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c04a371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a4a37a, 33 },
    { 0x3584a37b, 93 },
    { 0xc2a4a37a, 17 },
    { 0xea04a37d, 61 },
    { 0, 0 },
    { 0x9d4a37f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd614a383, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4034a38a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae4a38d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24a396, 93 },
    { 0, 0 },
    { 0x5b84a398, 51 },
    { 0xdc24a399, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc644a39d, 76 },
    { 0xa8f4a39e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4a3a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d4a3a4, 43 },
    { 0xd174a3a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c4a3ac, 23 },
    { 0x9c84a3ad, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5204a3b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6244a3b9, 95 },
    { 0x8c14a3ba, 33 },
    { 0x8c84a3ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6414a3c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e94a3c3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d4a3c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a3cb, 61 },
    { 0, 0 },
    { 0x3614a3cd, 78 },
    { 0, 0 },
    { 0x1cb4a3cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684a3db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3454a3de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c4a3e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3074a3eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9304a3ef, 98 },
    { 0xfec4a3f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa4a3f3, 90 },
    { 0x7a64a3f4, 99 },
    { 0xe414a3f5, 33 },
    { 0, 0 },
    { 0xeb4a3f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb4a3fa, 78 },
    { 0, 0 },
    { 0x3fb4a3fc, 33 },
    { 0x2044a3fc, 95 },
    { 0x6e14a3fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8384a402, 46 },
    { 0x9f34a402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c4a40b, 85 },
    { 0x65e4a40b, 33 },
    { 0x6004a40c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f4a414, 77 },
    { 0x9564a414, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc364a421, 33 },
    { 0x77f4a421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4874a425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c14a432, 95 },
    { 0xcc74a433, 95 },
    { 0, 0 },
    { 0x3c34a435, 100 },
    { 0x3a44a436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da4a43b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c4a440, 90 },
    { 0, 0 },
    { 0xc794a442, 46 },
    { 0xaee4a443, 61 },
    { 0, 0 },
    { 0x9e64a445, 61 },
    { 0x65f4a445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed4a44a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b24a44d, 98 },
    { 0x21c4a44d, 99 },
    { 0, 0 },
    { 0x7c54a450, 61 },
    { 0x5e24a450, 95 },
    { 0x2684a452, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe64a458, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde54a45b, 98 },
    { 0x27c4a45c, 55 },
    { 0, 0 },
    { 0x2e74a45e, 100 },
    { 0x5a34a45f, 98 },
    { 0x3374a460, 33 },
    { 0x6704a461, 61 },
    { 0, 0 },
    { 0xbcc4a463, 52 },
    { 0x4934a464, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff84a467, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3124a471, 98 },
    { 0, 0 },
    { 0x1c44a473, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f4a477, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2954a47a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc34a47f, 75 },
    { 0x6394a480, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9604a483, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a4a48a, 17 },
    { 0x3d54a48a, 90 },
    { 0x99d4a48a, 90 },
    { 0xb084a48d, 46 },
    { 0x9614a48d, 61 },
    { 0x34c4a48f, 52 },
    { 0x5174a48f, 100 },
    { 0x7c94a48e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce4a498, 77 },
    { 0x64e4a498, 17 },
    { 0x794a498, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd64a4a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca4a4a3, 33 },
    { 0, 0 },
    { 0x85f4a4a5, 95 },
    { 0, 0 },
    { 0xf3e4a4a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb34a4af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa614a4b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b84a4b7, 61 },
    { 0x7004a4b7, 23 },
    { 0, 0 },
    { 0x34a4a4ba, 66 },
    { 0xd4a4a4bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84a4ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9014a4d5, 55 },
    { 0x75c4a4d6, 46 },
    { 0, 0 },
    { 0x8ce4a4d8, 66 },
    { 0, 0 },
    { 0xea14a4da, 90 },
    { 0, 0 },
    { 0x8bd4a4dc, 53 },
    { 0, 0 },
    { 0x1104a4de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4884a4e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6114a4f0, 61 },
    { 0xa1f4a4f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d4a4f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5054a4f7, 98 },
    { 0xc44a4f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7424a4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb174a4fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b44a502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa4a509, 78 },
    { 0xd4a50a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e34a50d, 98 },
    { 0x6314a50d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5044a512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1504a51c, 33 },
    { 0x1c64a51c, 101 },
    { 0x6c94a51e, 23 },
    { 0x9874a51f, 17 },
    { 0xef34a51c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9604a523, 33 },
    { 0, 0 },
    { 0x4f44a525, 61 },
    { 0x4564a526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf694a52a, 90 },
    { 0x2154a52a, 67 },
    { 0xecc4a52a, 90 },
    { 0xa014a52d, 98 },
    { 0, 0 },
    { 0xa224a52f, 95 },
    { 0xf184a52f, 76 },
    { 0, 0 },
    { 0xa1d4a532, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb14a535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e34a539, 95 },
    { 0, 0 },
    { 0x76b4a53b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f4a53f, 90 },
    { 0x8214a540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde64a544, 76 },
    { 0, 0 },
    { 0x56d4a546, 95 },
    { 0x90b4a547, 88 },
    { 0x9944a546, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f4a54e, 98 },
    { 0x3f4a54f, 66 },
    { 0x6ac4a550, 70 },
    { 0xddd4a551, 76 },
    { 0x7da4a54e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec4a558, 93 },
    { 0, 0 },
    { 0x4234a55a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b94a55f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b4a567, 23 },
    { 0x1884a567, 95 },
    { 0x7bf4a569, 100 },
    { 0x7bd4a568, 33 },
    { 0xa954a56a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a56e, 91 },
    { 0x5d54a56e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8524a572, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac4a575, 33 },
    { 0x2b54a576, 46 },
    { 0x8844a575, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa4a57b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4154a57e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc74a581, 98 },
    { 0xb214a582, 80 },
    { 0x90b4a583, 50 },
    { 0x70a4a584, 100 },
    { 0xf8c4a585, 95 },
    { 0xa774a584, 98 },
    { 0, 0 },
    { 0xebf4a588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d4a58b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e04a593, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3684a59b, 61 },
    { 0xac74a59c, 34 },
    { 0x2bc4a59c, 90 },
    { 0xaec4a59b, 84 },
    { 0x4614a59f, 61 },
    { 0xedf4a5a0, 23 },
    { 0, 0 },
    { 0xbb4a5a2, 90 },
    { 0x1484a5a3, 100 },
    { 0x6c54a5a3, 88 },
    { 0x1a84a5a2, 61 },
    { 0x9b04a5a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a44a5aa, 90 },
    { 0, 0 },
    { 0x6c74a5ac, 33 },
    { 0xe164a5ac, 33 },
    { 0x8984a5ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c34a5b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344a5b6, 33 },
    { 0x7074a5b7, 90 },
    { 0x1ce4a5b7, 95 },
    { 0x7f24a5b8, 66 },
    { 0x2b04a5ba, 95 },
    { 0x37b4a5ba, 65 },
    { 0x9ba4a5bb, 95 },
    { 0x93a4a5bd, 90 },
    { 0x7364a5be, 36 },
    { 0xa9b4a5bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e4a5c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e4a5cb, 46 },
    { 0x5bb4a5cc, 61 },
    { 0xca24a5cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a64a5d0, 43 },
    { 0, 0 },
    { 0x4ca4a5d2, 98 },
    { 0x1e94a5d3, 55 },
    { 0xa184a5d3, 84 },
    { 0xc254a5d4, 61 },
    { 0x2714a5d4, 43 },
    { 0x6664a5d7, 91 },
    { 0xac04a5d8, 92 },
    { 0x67e4a5d7, 66 },
    { 0x5294a5d5, 78 },
    { 0x5274a5d2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4a5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7484a5eb, 33 },
    { 0x6f34a5ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f4a5f3, 34 },
    { 0x7ee4a5f3, 61 },
    { 0xf144a5f3, 75 },
    { 0x9a04a5f6, 95 },
    { 0, 0 },
    { 0xf1c4a5f8, 23 },
    { 0xf544a5f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44a5fe, 61 },
    { 0x604a5ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e4a607, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f4a60b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf4a60e, 61 },
    { 0x9274a60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f4a61d, 23 },
    { 0x1144a61e, 61 },
    { 0, 0 },
    { 0xe304a620, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314a626, 84 },
    { 0xf534a627, 90 },
    { 0, 0 },
    { 0x6124a629, 23 },
    { 0, 0 },
    { 0x6e94a62b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf824a632, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb024a635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d4a638, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a63d, 99 },
    { 0x89a4a63d, 43 },
    { 0, 0 },
    { 0xf074a640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc94a644, 76 },
    { 0, 0 },
    { 0x90f4a646, 43 },
    { 0x30c4a647, 93 },
    { 0x99a4a646, 95 },
    { 0xc344a646, 91 },
    { 0xf1e4a64a, 66 },
    { 0xff74a64a, 65 },
    { 0xe1a4a64c, 61 },
    { 0xf6c4a64d, 95 },
    { 0x5724a64c, 23 },
    { 0x4664a64b, 55 },
    { 0x57c4a650, 78 },
    { 0x58e4a651, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f4a655, 76 },
    { 0x3f94a656, 61 },
    { 0xc364a656, 33 },
    { 0, 0 },
    { 0x84b4a659, 90 },
    { 0x4fe4a65a, 23 },
    { 0xac04a65b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa064a660, 46 },
    { 0xf4a660, 95 },
    { 0x6f94a662, 93 },
    { 0xfa54a661, 46 },
    { 0, 0 },
    { 0xc1f4a665, 78 },
    { 0, 0 },
    { 0x8a94a667, 90 },
    { 0x78d4a668, 23 },
    { 0, 0 },
    { 0xf24a66a, 92 },
    { 0, 0 },
    { 0x824a66c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a84a674, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4a678, 34 },
    { 0x9d34a679, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4154a67d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4a686, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc4a689, 23 },
    { 0xc794a68a, 61 },
    { 0x774a68b, 23 },
    { 0, 0 },
    { 0xbac4a68d, 87 },
    { 0xc754a68e, 43 },
    { 0x8e74a68f, 76 },
    { 0, 0 },
    { 0x7854a691, 98 },
    { 0, 0 },
    { 0x3714a693, 23 },
    { 0, 0 },
    { 0xa384a695, 61 },
    { 0, 0 },
    { 0x21b4a697, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e94a6a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f64a6ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f4a6b2, 90 },
    { 0xb324a6b3, 91 },
    { 0xee64a6b4, 34 },
    { 0xb904a6b2, 17 },
    { 0xa154a6b6, 66 },
    { 0xcb14a6b6, 99 },
    { 0xc084a6b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4204a6bc, 34 },
    { 0, 0 },
    { 0xb554a6be, 61 },
    { 0xd0f4a6bf, 67 },
    { 0xe644a6bf, 61 },
    { 0x6ef4a6c1, 33 },
    { 0xd8b4a6c1, 95 },
    { 0xed74a6c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de4a6c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa434a6c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa4a6cc, 93 },
    { 0xaa94a6cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9504a6d0, 78 },
    { 0x3d94a6d1, 90 },
    { 0xcec4a6d1, 93 },
    { 0xf6e4a6d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a6d8, 76 },
    { 0xd914a6d9, 98 },
    { 0, 0 },
    { 0xbc64a6db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4a6e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54a6ea, 90 },
    { 0, 0 },
    { 0x8974a6ec, 23 },
    { 0x5d44a6ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564a6f2, 98 },
    { 0, 0 },
    { 0x27d4a6f4, 55 },
    { 0x83c4a6f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f44a6fd, 90 },
    { 0x2264a6fe, 43 },
    { 0xf34a6ff, 90 },
    { 0xc824a6ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe4a703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed4a709, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e4a710, 34 },
    { 0xb7c4a711, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a71a, 55 },
    { 0x60c4a71b, 98 },
    { 0x4aa4a71c, 61 },
    { 0x8074a71d, 40 },
    { 0, 0 },
    { 0xc314a71f, 23 },
    { 0x4f84a71f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a4a728, 84 },
    { 0, 0 },
    { 0x5f64a72a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe384a72f, 52 },
    { 0x4224a730, 95 },
    { 0xdd24a731, 33 },
    { 0x9944a732, 78 },
    { 0xe934a732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd4a737, 90 },
    { 0, 0 },
    { 0xd374a739, 93 },
    { 0, 0 },
    { 0x1604a73b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9964a73e, 55 },
    { 0x1f84a73e, 23 },
    { 0xf3d4a73f, 90 },
    { 0, 0 },
    { 0xece4a742, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda44a745, 90 },
    { 0, 0 },
    { 0xe824a747, 61 },
    { 0, 0 },
    { 0x9214a749, 65 },
    { 0x60b4a749, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f4a74e, 61 },
    { 0x4544a74f, 23 },
    { 0xfa84a750, 23 },
    { 0xe824a74f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb574a754, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d4a75d, 23 },
    { 0xe6b4a75e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa074a762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa94a766, 60 },
    { 0xa34a767, 90 },
    { 0xa594a768, 95 },
    { 0, 0 },
    { 0x53c4a76a, 90 },
    { 0xa3b4a76a, 93 },
    { 0, 0 },
    { 0xc1f4a76d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd4a770, 43 },
    { 0, 0 },
    { 0x5df4a772, 33 },
    { 0x7d44a772, 95 },
    { 0xc9f4a772, 43 },
    { 0x5664a774, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64a77b, 61 },
    { 0, 0 },
    { 0x4274a77d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc4a785, 77 },
    { 0, 0 },
    { 0xf404a787, 55 },
    { 0x6974a788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d4a78b, 64 },
    { 0x9dc4a78b, 23 },
    { 0, 0 },
    { 0xea4a78e, 23 },
    { 0xec64a78e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7334a792, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b4a795, 48 },
    { 0, 0 },
    { 0x51e4a797, 76 },
    { 0x43b4a798, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd614a79b, 43 },
    { 0x7114a79c, 90 },
    { 0x3304a79c, 43 },
    { 0xaec4a79c, 43 },
    { 0xb244a79f, 46 },
    { 0xa2c4a79e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1114a7a8, 98 },
    { 0, 0 },
    { 0x1954a7aa, 67 },
    { 0, 0 },
    { 0xe524a7ac, 95 },
    { 0, 0 },
    { 0x3a84a7ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d4a7b2, 84 },
    { 0x4054a7b2, 33 },
    { 0x5a14a7b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae34a7bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa694a7c0, 46 },
    { 0x9874a7c0, 90 },
    { 0x1d34a7c2, 61 },
    { 0xe914a7c0, 56 },
    { 0xfd44a7c4, 46 },
    { 0xaa84a7c5, 33 },
    { 0x7374a7c5, 98 },
    { 0x44a4a7c7, 98 },
    { 0xb784a7c7, 33 },
    { 0x8954a7c3, 33 },
    { 0, 0 },
    { 0x9184a7cb, 52 },
    { 0xebd4a7cc, 55 },
    { 0xa4a7cc, 77 },
    { 0x8a34a7ce, 17 },
    { 0xc8c4a7ce, 76 },
    { 0xe694a7cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4974a7d5, 33 },
    { 0xc234a7d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc94a7e4, 36 },
    { 0x2594a7e4, 99 },
    { 0x4b64a7e6, 76 },
    { 0x8824a7e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc134a7ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a4a7f1, 55 },
    { 0x89f4a7f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa724a7f6, 98 },
    { 0x4344a7f6, 81 },
    { 0xb334a7f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc254a7fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e4a802, 76 },
    { 0, 0 },
    { 0x7884a804, 98 },
    { 0, 0 },
    { 0x2da4a806, 61 },
    { 0x71b4a807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7354a80c, 98 },
    { 0, 0 },
    { 0xad04a80e, 33 },
    { 0, 0 },
    { 0x89e4a810, 98 },
    { 0, 0 },
    { 0xce04a812, 95 },
    { 0x6d04a813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e4a81c, 55 },
    { 0x2b14a81d, 33 },
    { 0x5da4a81e, 90 },
    { 0, 0 },
    { 0x7fc4a820, 90 },
    { 0x8c94a821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc484a825, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4a829, 23 },
    { 0x5c64a82a, 61 },
    { 0, 0 },
    { 0xf124a82c, 78 },
    { 0x8924a82c, 98 },
    { 0x5634a82c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc494a832, 76 },
    { 0xece4a832, 43 },
    { 0x53f4a833, 90 },
    { 0x27f4a835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4a83b, 96 },
    { 0x1674a83c, 98 },
    { 0x7524a83c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a4a841, 52 },
    { 0x814a842, 78 },
    { 0x1024a843, 61 },
    { 0xd7f4a843, 55 },
    { 0xbbe4a844, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e4a84b, 95 },
    { 0, 0 },
    { 0x5d04a84d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc844a855, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c04a858, 33 },
    { 0, 0 },
    { 0x3c54a85a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9124a864, 84 },
    { 0x53a4a865, 84 },
    { 0x9764a866, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b4a86a, 23 },
    { 0x9824a86a, 33 },
    { 0, 0 },
    { 0x4114a86d, 43 },
    { 0x28d4a86e, 76 },
    { 0, 0 },
    { 0xfd4a870, 90 },
    { 0x91c4a870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd4a877, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef84a87b, 61 },
    { 0x1c84a87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a94a886, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae4a88a, 61 },
    { 0x9e24a88b, 65 },
    { 0xb8f4a88a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5054a891, 78 },
    { 0, 0 },
    { 0xcf84a893, 85 },
    { 0x3aa4a894, 34 },
    { 0, 0 },
    { 0x1d34a896, 52 },
    { 0x19b4a896, 17 },
    { 0, 0 },
    { 0xa7c4a899, 61 },
    { 0x3d64a899, 98 },
    { 0x9084a89b, 90 },
    { 0x8e34a89c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7224a89f, 34 },
    { 0x1564a8a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c64a8a3, 77 },
    { 0x4224a8a4, 61 },
    { 0x124a8a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b4a8a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec4a8ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a54a8af, 95 },
    { 0, 0 },
    { 0x3ef4a8b1, 75 },
    { 0xcd34a8b1, 61 },
    { 0x6ca4a8b3, 100 },
    { 0x8574a8b4, 95 },
    { 0, 0 },
    { 0x29d4a8b6, 95 },
    { 0x8ee4a8b7, 95 },
    { 0xca04a8b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f64a8bd, 33 },
    { 0x6d84a8be, 61 },
    { 0, 0 },
    { 0x7c94a8c0, 95 },
    { 0xa7a4a8c1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f4a8c4, 91 },
    { 0x4bb4a8c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224a8cc, 90 },
    { 0xce34a8cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3004a8d0, 90 },
    { 0x41a4a8d1, 100 },
    { 0x9df4a8d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3084a8d6, 76 },
    { 0, 0 },
    { 0xc8f4a8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5034a8db, 98 },
    { 0xe7f4a8dc, 98 },
    { 0x4a94a8dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4a8e1, 33 },
    { 0x4354a8e1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954a8e5, 95 },
    { 0x3984a8e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa4a8e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b4a8ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b04a8f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf84a8f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc854a8f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc594a8fc, 61 },
    { 0, 0 },
    { 0xe434a8fe, 75 },
    { 0, 0 },
    { 0xf3a4a900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3564a903, 84 },
    { 0xe604a904, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4754a909, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d4a90c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf194a90f, 90 },
    { 0, 0 },
    { 0x4de4a911, 33 },
    { 0x614a912, 76 },
    { 0, 0 },
    { 0x6604a914, 55 },
    { 0x90d4a915, 98 },
    { 0, 0 },
    { 0x4a64a917, 78 },
    { 0xd564a918, 33 },
    { 0x5b74a919, 91 },
    { 0xc034a91a, 90 },
    { 0x3934a91a, 43 },
    { 0xb614a91c, 23 },
    { 0x5674a91c, 93 },
    { 0x80f4a91e, 33 },
    { 0xaac4a91d, 43 },
    { 0xdb04a920, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54a92d, 93 },
    { 0, 0 },
    { 0x2464a92f, 76 },
    { 0x5844a930, 33 },
    { 0xfe54a930, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7094a934, 33 },
    { 0, 0 },
    { 0xa734a936, 98 },
    { 0xc4f4a937, 90 },
    { 0xcc84a938, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf4a93b, 68 },
    { 0, 0 },
    { 0x1ff4a93d, 76 },
    { 0x7c64a93e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb4a943, 90 },
    { 0x4f14a943, 90 },
    { 0x534a945, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf084a94b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf984a94e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054a951, 46 },
    { 0x6534a952, 33 },
    { 0xd4a951, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af4a95b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf74a95e, 66 },
    { 0x1b04a95f, 24 },
    { 0x6274a960, 98 },
    { 0x44d4a960, 85 },
    { 0x2c44a962, 76 },
    { 0xccc4a962, 84 },
    { 0, 0 },
    { 0xb504a965, 98 },
    { 0x7d04a965, 98 },
    { 0xd8f4a967, 23 },
    { 0xa384a968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8174a970, 34 },
    { 0xd6c4a971, 34 },
    { 0x1354a972, 91 },
    { 0xa814a972, 88 },
    { 0xb2b4a974, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca4a978, 33 },
    { 0xe1a4a979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f94a97f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b44a982, 98 },
    { 0, 0 },
    { 0x4ac4a984, 17 },
    { 0xf7b4a984, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b4a98a, 52 },
    { 0, 0 },
    { 0xe964a98c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df4a991, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c4a997, 98 },
    { 0xed4a998, 90 },
    { 0xd814a999, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a34a99e, 90 },
    { 0, 0 },
    { 0x8074a9a0, 98 },
    { 0x6914a9a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xca14a9a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f4a9a9, 55 },
    { 0x2d4a9a9, 33 },
    { 0xf844a9ab, 66 },
    { 0xbd64a9ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4a9af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e4a9b3, 93 },
    { 0, 0 },
    { 0x5a54a9b5, 33 },
    { 0x79d4a9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b4a9bb, 61 },
    { 0x34c4a9bc, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed04a9c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde4a9c5, 40 },
    { 0x1d34a9c6, 93 },
    { 0x7824a9c7, 95 },
    { 0x374a9c7, 96 },
    { 0, 0 },
    { 0x36a4a9ca, 61 },
    { 0, 0 },
    { 0xd554a9cc, 98 },
    { 0, 0 },
    { 0xfc64a9ce, 33 },
    { 0x6d54a9cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf554a9d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf204a9d7, 33 },
    { 0x8504a9d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b34a9db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d74a9e2, 33 },
    { 0xc644a9e3, 90 },
    { 0xfd4a9e4, 46 },
    { 0, 0 },
    { 0x90e4a9e6, 90 },
    { 0, 0 },
    { 0x2dc4a9e8, 76 },
    { 0xcb94a9e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a9f0, 90 },
    { 0xb504a9f1, 67 },
    { 0x75a4a9f2, 90 },
    { 0xc484a9f2, 33 },
    { 0x1ad4a9f0, 95 },
    { 0, 0 },
    { 0xe914a9f6, 95 },
    { 0x4b4a9f7, 23 },
    { 0x294a9f8, 33 },
    { 0, 0 },
    { 0xdc34a9fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4aa02, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc4aa05, 23 },
    { 0xcba4aa06, 76 },
    { 0x394aa07, 61 },
    { 0xcc64aa08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1584aa0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c4aa10, 34 },
    { 0x7564aa11, 61 },
    { 0x8454aa10, 61 },
    { 0x4574aa12, 33 },
    { 0x5c04aa14, 90 },
    { 0x5504aa15, 55 },
    { 0x7bd4aa16, 43 },
    { 0, 0 },
    { 0x4104aa18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab4aa1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e4aa26, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d4aa2b, 66 },
    { 0x8b14aa2c, 23 },
    { 0x8954aa2d, 52 },
    { 0x184aa2e, 95 },
    { 0xbdc4aa2f, 90 },
    { 0xfe4aa2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a4aa33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a74aa37, 55 },
    { 0xae24aa38, 34 },
    { 0xc8b4aa38, 33 },
    { 0x7174aa3a, 90 },
    { 0x8744aa3b, 81 },
    { 0xbe54aa3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4aa43, 33 },
    { 0xa5d4aa44, 90 },
    { 0x77e4aa45, 33 },
    { 0xef24aa46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e64aa4e, 23 },
    { 0x8514aa4f, 90 },
    { 0x3784aa50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9154aa54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4aa5e, 90 },
    { 0, 0 },
    { 0x7ad4aa60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf854aa63, 98 },
    { 0x3824aa64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac4aa68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb24aa6b, 98 },
    { 0xbcf4aa6c, 55 },
    { 0, 0 },
    { 0x65f4aa6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab4aa71, 61 },
    { 0, 0 },
    { 0x3c14aa73, 23 },
    { 0xd1f4aa74, 76 },
    { 0x5f04aa73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f44aa84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e4aa8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4aa91, 60 },
    { 0x3494aa92, 55 },
    { 0x5f44aa92, 55 },
    { 0xa514aa93, 90 },
    { 0xc5a4aa95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb4aa98, 34 },
    { 0, 0 },
    { 0xc114aa9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc94aa9f, 61 },
    { 0, 0 },
    { 0x384aaa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9104aaa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4574aaad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b4aab4, 67 },
    { 0x9f64aab5, 52 },
    { 0x83e4aab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4aab9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4aabc, 95 },
    { 0xc3d4aabd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4aac0, 33 },
    { 0xd6a4aac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7974aace, 93 },
    { 0, 0 },
    { 0xc8d4aad0, 98 },
    { 0x5154aad1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf644aad6, 23 },
    { 0xa2c4aad7, 67 },
    { 0xff34aad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4754aadc, 90 },
    { 0, 0 },
    { 0x2aa4aade, 61 },
    { 0x2634aadf, 68 },
    { 0xfc4aade, 95 },
    { 0x71e4aae1, 61 },
    { 0xd884aae2, 84 },
    { 0xe754aae3, 66 },
    { 0x5d4aae3, 76 },
    { 0xd294aadf, 81 },
    { 0, 0 },
    { 0xbb54aae7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe924aaeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc54aaf0, 90 },
    { 0, 0 },
    { 0x75f4aaf2, 23 },
    { 0x6d4aaf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4aaf6, 46 },
    { 0xa334aaf7, 66 },
    { 0xce64aaf8, 33 },
    { 0x1554aaf9, 90 },
    { 0xdc94aaf9, 52 },
    { 0x6934aaf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe4aafe, 61 },
    { 0x6684aaff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc984ab04, 84 },
    { 0, 0 },
    { 0xe074ab06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4ab09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f54ab0c, 43 },
    { 0x3734ab0d, 100 },
    { 0x2e74ab0e, 33 },
    { 0xc14ab0e, 90 },
    { 0x5274ab10, 100 },
    { 0x7aa4ab0f, 78 },
    { 0x3eb4ab12, 61 },
    { 0x5c44ab13, 34 },
    { 0xdf4ab13, 85 },
    { 0xd5b4ab15, 85 },
    { 0xb8f4ab0f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8544ab19, 40 },
    { 0, 0 },
    { 0x20a4ab1b, 34 },
    { 0xbc04ab1b, 33 },
    { 0xee14ab1d, 61 },
    { 0x614ab1e, 87 },
    { 0x14d4ab1f, 88 },
    { 0x99a4ab20, 95 },
    { 0x174ab1c, 95 },
    { 0x8ee4ab22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1354ab25, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d4ab28, 66 },
    { 0x93a4ab29, 91 },
    { 0x98e4ab28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b4ab2f, 52 },
    { 0xbd44ab2f, 43 },
    { 0x6654ab31, 33 },
    { 0x7e4ab2f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe4ab3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf834ab3e, 64 },
    { 0xc9e4ab3e, 61 },
    { 0, 0 },
    { 0x6d34ab41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff4ab44, 61 },
    { 0x6c54ab45, 98 },
    { 0x5f84ab44, 91 },
    { 0x5ba4ab47, 14 },
    { 0xc0e4ab48, 90 },
    { 0, 0 },
    { 0xf4b4ab4a, 100 },
    { 0xb9b4ab4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3404ab4e, 93 },
    { 0x1894ab4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a4ab53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014ab5c, 61 },
    { 0x1184ab5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de4ab60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa4ab66, 90 },
    { 0x5ea4ab66, 46 },
    { 0xcfb4ab68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f4ab6b, 33 },
    { 0xe054ab6b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7254ab72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe4ab77, 76 },
    { 0x3d54ab78, 76 },
    { 0xc894ab79, 81 },
    { 0xfe4ab7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf654ab83, 33 },
    { 0x6d64ab84, 95 },
    { 0, 0 },
    { 0xe0f4ab86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce4ab91, 61 },
    { 0x6744ab92, 98 },
    { 0x3974ab93, 23 },
    { 0xc664ab92, 23 },
    { 0x30c4ab91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c4ab9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa394aba0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b4aba5, 46 },
    { 0, 0 },
    { 0xfb34aba7, 90 },
    { 0x87b4aba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854abab, 61 },
    { 0xd0b4abac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34abaf, 76 },
    { 0x6ab4abb0, 98 },
    { 0, 0 },
    { 0x8594abb2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd954abb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564abbb, 90 },
    { 0, 0 },
    { 0x7cc4abbd, 33 },
    { 0, 0 },
    { 0xfa04abbf, 23 },
    { 0x5524abc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b4abc7, 91 },
    { 0x4654abc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d04abcb, 52 },
    { 0x8824abcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b4abd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a4abd3, 95 },
    { 0xd104abd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd204abd7, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0x3254abda, 61 },
    { 0, 0 },
    { 0xfa34abdc, 90 },
    { 0xbe34abdd, 55 },
    { 0x15d4abde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e4abe1, 98 },
    { 0x84b4abe2, 23 },
    { 0x54d4abe2, 95 },
    { 0x6cd4abe4, 88 },
    { 0x9534abe4, 61 },
    { 0, 0 },
    { 0xa414abe7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc4abeb, 88 },
    { 0xf0c4abeb, 93 },
    { 0xa674abed, 61 },
    { 0x3df4abec, 34 },
    { 0xef54abef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d4abf2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4714abf9, 55 },
    { 0x56a4abfa, 55 },
    { 0xa374abfb, 90 },
    { 0x27b4abfc, 43 },
    { 0, 0 },
    { 0x6a4abfe, 23 },
    { 0x4ca4abff, 95 },
    { 0x9184abff, 95 },
    { 0x5a94abfe, 95 },
    { 0, 0 },
    { 0x5734ac03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a04ac06, 34 },
    { 0xf9f4ac07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb64ac10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4ac13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x804ac16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd4ac19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af4ac1c, 98 },
    { 0x2134ac1c, 90 },
    { 0, 0 },
    { 0x314ac1f, 93 },
    { 0xbbd4ac20, 100 },
    { 0x1d84ac21, 33 },
    { 0x41d4ac1f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd944ac25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd4ac29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa4ac2e, 68 },
    { 0xf5c4ac2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ac34, 90 },
    { 0x8a34ac34, 61 },
    { 0, 0 },
    { 0x874ac37, 98 },
    { 0, 0 },
    { 0xb654ac39, 90 },
    { 0x8284ac3a, 23 },
    { 0x964ac3b, 61 },
    { 0x1d74ac3a, 95 },
    { 0x8f94ac3c, 52 },
    { 0xb4b4ac3e, 43 },
    { 0x4d64ac3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814ac4a, 23 },
    { 0, 0 },
    { 0xede4ac4c, 90 },
    { 0x3a4ac4d, 61 },
    { 0xed74ac4e, 78 },
    { 0xffa4ac4d, 90 },
    { 0x4184ac50, 98 },
    { 0, 0 },
    { 0xb304ac52, 87 },
    { 0x28e4ac52, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd004ac56, 90 },
    { 0, 0 },
    { 0xb8c4ac58, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb064ac5b, 55 },
    { 0, 0 },
    { 0x9944ac5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee64ac61, 95 },
    { 0x7b24ac62, 44 },
    { 0x6be4ac63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af4ac66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c4ac69, 90 },
    { 0xe424ac6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc4ac6e, 84 },
    { 0xfbc4ac6e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd4ac72, 81 },
    { 0x1a34ac73, 98 },
    { 0xd34ac74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a4ac77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd594ac7c, 34 },
    { 0xf84ac7d, 76 },
    { 0x15b4ac7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7194ac81, 84 },
    { 0x9d14ac81, 55 },
    { 0xa7c4ac83, 76 },
    { 0xc994ac83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7834ac89, 33 },
    { 0, 0 },
    { 0x21c4ac8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc4ac91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8674ac97, 95 },
    { 0, 0 },
    { 0x9034ac99, 98 },
    { 0, 0 },
    { 0x7814ac9b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1074aca0, 46 },
    { 0x9f54aca1, 90 },
    { 0x31c4aca2, 61 },
    { 0, 0 },
    { 0xb014aca4, 98 },
    { 0xef84aca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x484aca8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e4acab, 98 },
    { 0x8564acac, 61 },
    { 0x9624acad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a4acb1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b4acb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5994acb9, 95 },
    { 0xfa14acba, 98 },
    { 0x4fd4acbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914acbf, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e4acc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa854acc9, 40 },
    { 0xec44acca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc34accd, 76 },
    { 0x6924accd, 98 },
    { 0, 0 },
    { 0x8e54acd0, 55 },
    { 0, 0 },
    { 0xce4acd2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a4acd9, 23 },
    { 0, 0 },
    { 0x7e24acdb, 98 },
    { 0x2794acdc, 68 },
    { 0x4d54acdd, 65 },
    { 0xdc54acdc, 95 },
    { 0xc734acdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe114ace3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd614ace8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c4aceb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc444acef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f4acf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb724acf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c4acf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e4acfe, 95 },
    { 0, 0 },
    { 0xa94ad00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf834ad04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714ad08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2424ad0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5944ad0f, 23 },
    { 0x4a64ad0f, 23 },
    { 0xa7a4ad11, 90 },
    { 0xcc94ad12, 23 },
    { 0, 0 },
    { 0xa044ad14, 46 },
    { 0, 0 },
    { 0x6884ad16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b4ad19, 23 },
    { 0, 0 },
    { 0xb734ad1b, 33 },
    { 0, 0 },
    { 0xbfa4ad1d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb514ad22, 61 },
    { 0x1574ad23, 77 },
    { 0x5ab4ad22, 90 },
    { 0x3d24ad22, 33 },
    { 0xbcb4ad26, 33 },
    { 0, 0 },
    { 0x1cc4ad28, 93 },
    { 0x2784ad29, 61 },
    { 0xd664ad2a, 33 },
    { 0x9114ad28, 93 },
    { 0xcd84ad2c, 90 },
    { 0, 0 },
    { 0xdb44ad2e, 34 },
    { 0x42f4ad2f, 23 },
    { 0xb7e4ad2f, 100 },
    { 0, 0 },
    { 0x7a24ad32, 61 },
    { 0xc7e4ad32, 98 },
    { 0xd5d4ad34, 98 },
    { 0xe364ad35, 61 },
    { 0x7cc4ad33, 43 },
    { 0x1284ad35, 55 },
    { 0x7a34ad35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2734ad3b, 31 },
    { 0xe404ad3c, 23 },
    { 0xfec4ad3d, 90 },
    { 0x8d44ad3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3824ad42, 17 },
    { 0xbdc4ad42, 23 },
    { 0x2da4ad44, 95 },
    { 0x4b84ad45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb124ad4e, 46 },
    { 0x5194ad4e, 46 },
    { 0x63c4ad50, 90 },
    { 0xcff4ad50, 69 },
    { 0xf024ad52, 90 },
    { 0x6794ad51, 90 },
    { 0x7444ad54, 81 },
    { 0, 0 },
    { 0x86e4ad56, 95 },
    { 0x82f4ad57, 98 },
    { 0xbe84ad58, 100 },
    { 0, 0 },
    { 0x9824ad5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d4ad63, 93 },
    { 0xf04ad64, 90 },
    { 0xaec4ad65, 61 },
    { 0x9024ad66, 99 },
    { 0, 0 },
    { 0xff4ad68, 90 },
    { 0x3864ad69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a04ad6d, 66 },
    { 0x90a4ad6d, 33 },
    { 0x1a64ad6e, 19 },
    { 0x7754ad70, 88 },
    { 0xc1c4ad70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ad74, 91 },
    { 0, 0 },
    { 0xcd84ad76, 95 },
    { 0, 0 },
    { 0x80c4ad78, 17 },
    { 0xc414ad79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774ad7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe644ad81, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b4ad8d, 65 },
    { 0x3e4ad8e, 23 },
    { 0, 0 },
    { 0xfca4ad90, 93 },
    { 0xcfb4ad91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a04ad95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6654ad9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b4ada5, 46 },
    { 0x744ada5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8474adaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7094adae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d4adb2, 33 },
    { 0x9d44adb3, 33 },
    { 0x1f64adb4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f4adb7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5074adba, 61 },
    { 0x1434adba, 61 },
    { 0x2fe4adbb, 95 },
    { 0, 0 },
    { 0xac4adbe, 99 },
    { 0x7304adbf, 33 },
    { 0xf054adbe, 90 },
    { 0x5ab4adc1, 61 },
    { 0x1784adc2, 93 },
    { 0x6d14adc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e4adc6, 87 },
    { 0x6974adc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d04adca, 76 },
    { 0x68d4adca, 61 },
    { 0, 0 },
    { 0x63a4adcd, 84 },
    { 0, 0 },
    { 0x6544adcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2934add7, 23 },
    { 0x7c94add7, 23 },
    { 0x9cf4add8, 34 },
    { 0x7fb4add8, 43 },
    { 0xd684addb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ade3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2374ade9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da4adec, 33 },
    { 0, 0 },
    { 0x5944adee, 48 },
    { 0x2754adee, 98 },
    { 0, 0 },
    { 0x28e4adf1, 23 },
    { 0x4b64adf1, 67 },
    { 0xf4e4adf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5064adf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa084adfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb974ae00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4ae04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8234ae08, 33 },
    { 0xcfb4ae09, 93 },
    { 0x7ed4ae0a, 55 },
    { 0, 0 },
    { 0xfa94ae0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4244ae15, 98 },
    { 0xc5b4ae16, 23 },
    { 0x3ce4ae17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d4ae1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb4ae1d, 46 },
    { 0x90c4ae1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2764ae29, 88 },
    { 0xe7d4ae29, 95 },
    { 0, 0 },
    { 0x41e4ae2c, 100 },
    { 0x9894ae2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c94ae37, 61 },
    { 0xd84ae38, 61 },
    { 0xb74ae39, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7914ae3d, 34 },
    { 0x85e4ae3e, 84 },
    { 0, 0 },
    { 0xe3e4ae40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6154ae49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9054ae4c, 90 },
    { 0x9464ae4d, 98 },
    { 0, 0 },
    { 0x9804ae4f, 33 },
    { 0x67a4ae4f, 76 },
    { 0xbf54ae4f, 95 },
    { 0x7784ae51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e94ae5a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa694ae5d, 85 },
    { 0, 0 },
    { 0x6b94ae5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2984ae64, 61 },
    { 0x3d24ae65, 81 },
    { 0, 0 },
    { 0xf0d4ae67, 65 },
    { 0xcbc4ae68, 61 },
    { 0x7964ae69, 98 },
    { 0x5d74ae69, 33 },
    { 0, 0 },
    { 0xa124ae6c, 55 },
    { 0x8c84ae6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8884ae75, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c4ae78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d4ae7d, 94 },
    { 0xeda4ae7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a4ae81, 66 },
    { 0xcd24ae82, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b4ae86, 78 },
    { 0x3684ae87, 90 },
    { 0, 0 },
    { 0x72c4ae89, 52 },
    { 0x1d14ae89, 33 },
    { 0, 0 },
    { 0x5e74ae8c, 98 },
    { 0xd124ae8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654ae95, 90 },
    { 0, 0 },
    { 0xa934ae97, 75 },
    { 0x2144ae98, 61 },
    { 0x21c4ae98, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5014ae9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3974aea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8024aea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f64aead, 33 },
    { 0, 0 },
    { 0x9714aeaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac4aeb5, 23 },
    { 0xd104aeb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5834aeb9, 61 },
    { 0, 0 },
    { 0xce44aebb, 23 },
    { 0x6134aebc, 40 },
    { 0x9774aebd, 90 },
    { 0xa24aebe, 43 },
    { 0xb3e4aebc, 87 },
    { 0x1184aebe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e04aeca, 93 },
    { 0x23f4aecb, 47 },
    { 0xc9e4aecb, 11 },
    { 0, 0 },
    { 0xfac4aece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d94aed3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb064aed8, 93 },
    { 0xc674aed8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc874aedc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5334aee4, 77 },
    { 0xf2a4aee5, 43 },
    { 0xa6e4aee6, 23 },
    { 0x1b4aee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9814aef3, 33 },
    { 0, 0 },
    { 0xeb84aef5, 87 },
    { 0x6e64aef6, 43 },
    { 0x33f4aef7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8594aefb, 76 },
    { 0xb4e4aefc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa324af03, 23 },
    { 0x5044af03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34af07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a4af0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b4af10, 66 },
    { 0, 0 },
    { 0x59e4af12, 52 },
    { 0xfe94af13, 23 },
    { 0, 0 },
    { 0xd9e4af15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9634af19, 61 },
    { 0, 0 },
    { 0x2dd4af1b, 90 },
    { 0xf484af1c, 90 },
    { 0x9eb4af1c, 93 },
    { 0xb664af1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1944af24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e4af27, 98 },
    { 0x34af27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d4af2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044af30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4574af35, 95 },
    { 0, 0 },
    { 0xa224af37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf54af3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e44af3f, 55 },
    { 0xe3a4af3f, 66 },
    { 0xdba4af40, 33 },
    { 0xc134af42, 98 },
    { 0xe464af42, 34 },
    { 0x3774af44, 81 },
    { 0, 0 },
    { 0xdc14af46, 95 },
    { 0, 0 },
    { 0x5af4af48, 23 },
    { 0x2a24af49, 46 },
    { 0xe24af4a, 69 },
    { 0x11f4af4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad64af50, 93 },
    { 0xb884af50, 98 },
    { 0xa874af52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1894af59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf464af61, 95 },
    { 0, 0 },
    { 0x1b14af63, 34 },
    { 0xc0d4af64, 90 },
    { 0x1cc4af64, 95 },
    { 0x7144af66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4af69, 78 },
    { 0xb894af6a, 76 },
    { 0x9294af69, 34 },
    { 0x6754af6a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc04af70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d4af74, 95 },
    { 0xfa44af75, 25 },
    { 0x3204af76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d4af79, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94af7d, 61 },
    { 0, 0 },
    { 0xcb84af7f, 90 },
    { 0xfac4af7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe494af84, 61 },
    { 0x6064af84, 33 },
    { 0xef04af86, 92 },
    { 0x794af86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94af8a, 33 },
    { 0xc984af8b, 61 },
    { 0xf814af8c, 23 },
    { 0xfe04af8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64af92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3254af9b, 61 },
    { 0xc294af9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc724afa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4afa4, 66 },
    { 0xe0d4afa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944afa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c4afab, 61 },
    { 0x2494afac, 43 },
    { 0x98d4afab, 93 },
    { 0x3934afab, 99 },
    { 0x1264afaf, 61 },
    { 0xf464afb0, 90 },
    { 0xa8c4afb1, 33 },
    { 0x6094afb2, 61 },
    { 0x4a84afb2, 33 },
    { 0, 0 },
    { 0xca24afb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d54afc0, 100 },
    { 0x4a04afc1, 33 },
    { 0xa444afc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4744afc9, 33 },
    { 0, 0 },
    { 0x6324afcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4afd2, 92 },
    { 0x8334afd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea94afd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4afdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e4afe0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d54afe6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5574afe9, 88 },
    { 0, 0 },
    { 0x5ce4afeb, 47 },
    { 0x1444afec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e4afef, 66 },
    { 0x58c4afef, 98 },
    { 0x46f4aff1, 90 },
    { 0xbfe4aff2, 43 },
    { 0x18f4aff3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d64aff9, 95 },
    { 0xbc54affa, 33 },
    { 0x71f4affa, 75 },
    { 0x46b4affa, 90 },
    { 0xd154aff9, 98 },
    { 0x5954affc, 95 },
    { 0xd524aff9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a4b002, 55 },
    { 0x1904b003, 95 },
    { 0, 0 },
    { 0xcfd4b005, 95 },
    { 0, 0 },
    { 0x3fd4b007, 71 },
    { 0x1814b008, 90 },
    { 0x9444b009, 87 },
    { 0xf984b009, 55 },
    { 0x68f4b00b, 61 },
    { 0xc9e4b00c, 98 },
    { 0x4aa4b00c, 33 },
    { 0xf7c4b00e, 91 },
    { 0, 0 },
    { 0x8814b010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94b015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de4b01b, 46 },
    { 0x6e14b01c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7754b01f, 78 },
    { 0xb164b020, 46 },
    { 0, 0 },
    { 0x3c14b022, 61 },
    { 0, 0 },
    { 0xeb4b024, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd434b027, 23 },
    { 0, 0 },
    { 0xcc34b029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c4b02c, 34 },
    { 0xc5c4b02d, 95 },
    { 0x9a24b02e, 90 },
    { 0xb604b02f, 84 },
    { 0x9144b02e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f94b035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f74b03c, 76 },
    { 0x2874b03d, 33 },
    { 0x8134b03d, 100 },
    { 0x6424b03f, 33 },
    { 0xf6d4b03c, 95 },
    { 0xf044b03d, 40 },
    { 0xe4b042, 61 },
    { 0xf554b043, 61 },
    { 0x3e24b041, 90 },
    { 0x3bf4b045, 100 },
    { 0x73b4b046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3414b04a, 95 },
    { 0xff84b04b, 93 },
    { 0xa44b04b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf724b052, 23 },
    { 0xfd34b053, 98 },
    { 0x7b64b054, 66 },
    { 0x57b4b055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c4b058, 95 },
    { 0x5ee4b058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c04b05d, 53 },
    { 0, 0 },
    { 0x25e4b05f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa214b064, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e54b06a, 88 },
    { 0x8c84b06b, 50 },
    { 0xdf34b06c, 99 },
    { 0, 0 },
    { 0xca4b06e, 34 },
    { 0, 0 },
    { 0xfeb4b070, 76 },
    { 0, 0 },
    { 0xe6e4b072, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd74b07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a24b082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4b086, 40 },
    { 0x8c94b087, 90 },
    { 0, 0 },
    { 0x95b4b089, 90 },
    { 0, 0 },
    { 0xe424b08b, 95 },
    { 0xc0a4b08c, 93 },
    { 0xe124b08d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f94b090, 67 },
    { 0, 0 },
    { 0xaad4b092, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd514b095, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4b099, 55 },
    { 0x7a64b09a, 91 },
    { 0, 0 },
    { 0xd2b4b09c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef4b0a0, 17 },
    { 0x3674b0a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff4b0a7, 98 },
    { 0x8494b0a8, 95 },
    { 0, 0 },
    { 0xe794b0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa34b0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44b0b1, 33 },
    { 0xc6a4b0b2, 34 },
    { 0x904b0b3, 98 },
    { 0, 0 },
    { 0xbba4b0b5, 55 },
    { 0x9364b0b6, 67 },
    { 0, 0 },
    { 0xd4b4b0b8, 77 },
    { 0x4d44b0b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb914b0bc, 67 },
    { 0x4684b0bd, 85 },
    { 0xda14b0bd, 55 },
    { 0xd3e4b0bf, 90 },
    { 0, 0 },
    { 0x1704b0c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7984b0c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c4b0cd, 90 },
    { 0xb454b0cd, 76 },
    { 0x2204b0ce, 90 },
    { 0xcf44b0d0, 46 },
    { 0, 0 },
    { 0x70d4b0d2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d94b0d5, 76 },
    { 0x5c94b0d6, 78 },
    { 0x54a4b0d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4b0da, 95 },
    { 0xc564b0da, 91 },
    { 0x14c4b0dc, 33 },
    { 0xe634b0dc, 99 },
    { 0x4124b0de, 46 },
    { 0, 0 },
    { 0xa8e4b0e0, 65 },
    { 0x9c84b0e1, 66 },
    { 0xdeb4b0e0, 55 },
    { 0x2874b0e0, 95 },
    { 0x7ef4b0e3, 90 },
    { 0x77d4b0e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c94b0e8, 46 },
    { 0xc0d4b0e8, 66 },
    { 0x3be4b0ea, 43 },
    { 0xeee4b0e8, 61 },
    { 0x7ac4b0ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf094b0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe104b0f9, 55 },
    { 0, 0 },
    { 0x394b0fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd4b100, 66 },
    { 0xe564b101, 90 },
    { 0xc004b100, 55 },
    { 0x8664b103, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3534b107, 12 },
    { 0, 0 },
    { 0x5534b109, 23 },
    { 0, 0 },
    { 0x314b10b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3794b111, 98 },
    { 0, 0 },
    { 0x1aa4b113, 33 },
    { 0xcce4b114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b94b117, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb354b11b, 98 },
    { 0x6174b11b, 23 },
    { 0x5eb4b11b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1594b121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf44b125, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd4b128, 34 },
    { 0xa2c4b128, 85 },
    { 0xe194b12a, 52 },
    { 0x1f44b12a, 33 },
    { 0xec64b12a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7434b12f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4b132, 76 },
    { 0x3354b132, 76 },
    { 0, 0 },
    { 0x6284b135, 85 },
    { 0x6704b136, 43 },
    { 0xfa24b137, 52 },
    { 0x514b138, 82 },
    { 0xd764b139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4b13c, 55 },
    { 0xd424b13d, 90 },
    { 0xaba4b13e, 61 },
    { 0x6424b13f, 101 },
    { 0xef84b140, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8684b148, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc04b14b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc4b154, 23 },
    { 0, 0 },
    { 0xcb84b156, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3624b15b, 85 },
    { 0xbcd4b15c, 23 },
    { 0, 0 },
    { 0xe8b4b15e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda4b167, 76 },
    { 0x34b167, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9644b176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9054b179, 78 },
    { 0x2fc4b17a, 67 },
    { 0, 0 },
    { 0x2c14b17c, 33 },
    { 0x34f4b17d, 93 },
    { 0, 0 },
    { 0x4964b17f, 78 },
    { 0, 0 },
    { 0x8564b181, 98 },
    { 0xb974b181, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xb274b185, 100 },
    { 0xa544b186, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9714b189, 76 },
    { 0xf774b18a, 23 },
    { 0x8464b189, 100 },
    { 0xd7d4b18c, 23 },
    { 0xe6b4b18c, 75 },
    { 0xc1f4b18e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4b193, 61 },
    { 0, 0 },
    { 0xab64b195, 90 },
    { 0x704b196, 61 },
    { 0xc1b4b197, 98 },
    { 0x2984b198, 55 },
    { 0, 0 },
    { 0x6524b19a, 65 },
    { 0x3304b19b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e34b19e, 90 },
    { 0x5e84b19e, 61 },
    { 0x3324b1a0, 90 },
    { 0x964b1a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe34b1ab, 34 },
    { 0xcd54b1ac, 33 },
    { 0x3644b1ac, 98 },
    { 0, 0 },
    { 0xf6d4b1af, 55 },
    { 0xdc24b1b0, 92 },
    { 0x8f84b1b0, 100 },
    { 0x5b94b1b0, 23 },
    { 0xe6f4b1b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe204b1b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6034b1bd, 90 },
    { 0x7754b1bd, 95 },
    { 0x1784b1bd, 93 },
    { 0xe754b1c0, 90 },
    { 0xaeb4b1c1, 78 },
    { 0x7b74b1c2, 23 },
    { 0xa404b1be, 83 },
    { 0x6274b1bd, 23 },
    { 0x1e64b1c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae4b1cc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e4b1d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d34b1db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f44b1de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4034b1e5, 90 },
    { 0x3654b1e6, 33 },
    { 0, 0 },
    { 0x9f94b1e8, 75 },
    { 0xcb24b1e9, 23 },
    { 0xdd54b1e9, 98 },
    { 0x6694b1eb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024b1f7, 61 },
    { 0, 0 },
    { 0xac14b1f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4b1fc, 23 },
    { 0xd274b1fc, 52 },
    { 0x2514b1fe, 76 },
    { 0xbca4b1fe, 76 },
    { 0x3354b1ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd4b203, 43 },
    { 0xade4b204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc884b207, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9904b20a, 90 },
    { 0, 0 },
    { 0xe134b20c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b20f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5424b215, 61 },
    { 0xab04b216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b4b21b, 66 },
    { 0x4624b21c, 67 },
    { 0x5af4b21d, 84 },
    { 0x73c4b21d, 49 },
    { 0xbf4b21e, 90 },
    { 0x1eb4b220, 55 },
    { 0x9a44b21f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda74b225, 46 },
    { 0, 0 },
    { 0x6fc4b227, 95 },
    { 0xa404b228, 52 },
    { 0x6b84b228, 23 },
    { 0x7544b22a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d4b231, 53 },
    { 0, 0 },
    { 0xf304b233, 23 },
    { 0x7e74b234, 23 },
    { 0, 0 },
    { 0xf834b236, 61 },
    { 0x6c64b236, 17 },
    { 0, 0 },
    { 0x2164b239, 33 },
    { 0, 0 },
    { 0xf894b23b, 90 },
    { 0x18e4b23c, 84 },
    { 0xf1a4b23c, 55 },
    { 0xf694b23d, 98 },
    { 0, 0 },
    { 0xaf84b240, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xec64b243, 34 },
    { 0x8d74b244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9614b24b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b04b24e, 90 },
    { 0x62a4b24f, 40 },
    { 0, 0 },
    { 0x1264b251, 95 },
    { 0x9f34b252, 61 },
    { 0xd4f4b253, 76 },
    { 0, 0 },
    { 0x51f4b255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d4b258, 43 },
    { 0xc4a4b258, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a4b25c, 33 },
    { 0x6624b25d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3634b266, 34 },
    { 0x7c34b267, 90 },
    { 0, 0 },
    { 0x1804b269, 31 },
    { 0, 0 },
    { 0x3584b26b, 90 },
    { 0x5c74b26c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf94b270, 90 },
    { 0x2294b271, 98 },
    { 0x3c4b272, 90 },
    { 0x3824b271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e4b27e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe4b283, 99 },
    { 0, 0 },
    { 0x9c84b285, 34 },
    { 0, 0 },
    { 0x9724b287, 33 },
    { 0x6354b287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1944b28b, 33 },
    { 0xec74b28c, 33 },
    { 0x7804b28d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c4b294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7584b29e, 84 },
    { 0x1624b29f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9144b2a5, 66 },
    { 0x16d4b2a5, 23 },
    { 0xb124b2a7, 33 },
    { 0xde54b2a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf4b2ac, 90 },
    { 0xbd04b2ad, 90 },
    { 0xf7f4b2ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7674b2b2, 90 },
    { 0xd324b2b3, 84 },
    { 0x5e14b2b4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d74b2bc, 61 },
    { 0xcff4b2bc, 85 },
    { 0x6cf4b2be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad4b2c2, 69 },
    { 0x75e4b2c3, 85 },
    { 0, 0 },
    { 0x784b2c5, 91 },
    { 0xec24b2c5, 98 },
    { 0xcb64b2c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d74b2cb, 90 },
    { 0xe474b2cc, 33 },
    { 0xd524b2cc, 43 },
    { 0, 0 },
    { 0x2c34b2cf, 93 },
    { 0x42b4b2d0, 33 },
    { 0, 0 },
    { 0x1324b2d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xea54b2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb194b2df, 73 },
    { 0x8b94b2df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b2e4, 87 },
    { 0xa184b2e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f54b2e8, 85 },
    { 0x4574b2e8, 98 },
    { 0x93e4b2ea, 98 },
    { 0, 0 },
    { 0xca24b2ec, 98 },
    { 0x224b2ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c94b2f2, 95 },
    { 0xc304b2f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f04b300, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b84b303, 76 },
    { 0xd234b304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f34b307, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2624b30c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e4b314, 95 },
    { 0xf054b315, 33 },
    { 0x2eb4b314, 93 },
    { 0x8e54b317, 98 },
    { 0xdab4b318, 100 },
    { 0xcdd4b319, 91 },
    { 0, 0 },
    { 0xb804b31b, 23 },
    { 0x67d4b31b, 34 },
    { 0, 0 },
    { 0x5294b31e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4b326, 61 },
    { 0xfe44b327, 98 },
    { 0xc674b328, 98 },
    { 0x2d94b328, 76 },
    { 0x64b4b32a, 91 },
    { 0x2c74b32b, 100 },
    { 0xa164b329, 76 },
    { 0x90a4b32d, 90 },
    { 0x59f4b32c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2634b339, 61 },
    { 0x1514b33a, 76 },
    { 0x1354b33a, 48 },
    { 0x4534b33c, 36 },
    { 0xacd4b33c, 33 },
    { 0xda34b33c, 33 },
    { 0, 0 },
    { 0x8de4b340, 90 },
    { 0xcfe4b341, 33 },
    { 0, 0 },
    { 0x9574b343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a4b34d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f4b350, 88 },
    { 0x6734b350, 90 },
    { 0xb864b352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e4b358, 23 },
    { 0, 0 },
    { 0xd4d4b35a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f4b35e, 55 },
    { 0x3c64b35f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x574b362, 23 },
    { 0, 0 },
    { 0xc3d4b364, 91 },
    { 0x27d4b365, 90 },
    { 0x40e4b365, 33 },
    { 0, 0 },
    { 0x8724b368, 61 },
    { 0x8d84b369, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd524b36c, 61 },
    { 0x58f4b36d, 61 },
    { 0xbd74b36e, 78 },
    { 0x6e34b36d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc634b37b, 99 },
    { 0, 0 },
    { 0x534b37d, 23 },
    { 0, 0 },
    { 0xa544b37f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b384, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde4b387, 61 },
    { 0xaca4b387, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa714b38c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2894b390, 33 },
    { 0x8fa4b391, 84 },
    { 0, 0 },
    { 0x62d4b393, 40 },
    { 0xc894b394, 61 },
    { 0, 0 },
    { 0xbf14b396, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f4b39d, 91 },
    { 0x1e14b39e, 33 },
    { 0, 0 },
    { 0x8f44b3a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9494b3a6, 77 },
    { 0x244b3a6, 55 },
    { 0xb1f4b3a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d4b3ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e4b3af, 98 },
    { 0x36d4b3b0, 90 },
    { 0x8684b3b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214b3b8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb34b3bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc4b3bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4654b3c6, 52 },
    { 0, 0 },
    { 0xac84b3c8, 34 },
    { 0, 0 },
    { 0xd694b3ca, 100 },
    { 0x69b4b3cb, 90 },
    { 0xfcc4b3cc, 59 },
    { 0x3784b3cd, 33 },
    { 0x27b4b3ce, 98 },
    { 0xaf64b3cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c4b3d3, 84 },
    { 0x45a4b3d4, 23 },
    { 0x8774b3d5, 95 },
    { 0x24f4b3d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb884b3d9, 87 },
    { 0, 0 },
    { 0xcd04b3db, 61 },
    { 0x31e4b3dc, 56 },
    { 0x84e4b3dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d4b3e4, 61 },
    { 0, 0 },
    { 0xbb24b3e6, 80 },
    { 0, 0 },
    { 0xa264b3e8, 61 },
    { 0, 0 },
    { 0x5d64b3ea, 43 },
    { 0, 0 },
    { 0xdfa4b3ec, 33 },
    { 0xab64b3ed, 95 },
    { 0xf534b3ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa654b3f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a64b3f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x804b3f9, 33 },
    { 0xdc14b3f9, 61 },
    { 0xfed4b3fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54b401, 61 },
    { 0xd434b402, 88 },
    { 0x1e44b403, 61 },
    { 0x5994b401, 23 },
    { 0x6924b405, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d44b40a, 98 },
    { 0x35d4b40b, 33 },
    { 0, 0 },
    { 0x8454b40d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd4b410, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f4b413, 93 },
    { 0x4ee4b414, 76 },
    { 0x1914b415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d4b419, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e4b41f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f4b422, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8784b425, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd724b42a, 23 },
    { 0, 0 },
    { 0x10b4b42c, 46 },
    { 0x5f24b42c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da4b432, 77 },
    { 0, 0 },
    { 0xfc64b434, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b74b43c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24b43f, 34 },
    { 0, 0 },
    { 0x7734b441, 23 },
    { 0x39d4b442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064b448, 99 },
    { 0x2464b448, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc604b44c, 55 },
    { 0, 0 },
    { 0x7074b44e, 61 },
    { 0x1a44b44f, 23 },
    { 0xb674b450, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b4b456, 61 },
    { 0xe294b457, 36 },
    { 0x8724b458, 23 },
    { 0xa494b457, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f54b460, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4b465, 91 },
    { 0xbb24b466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5584b46e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814b472, 33 },
    { 0, 0 },
    { 0x6e04b474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f4b478, 100 },
    { 0xad04b479, 84 },
    { 0x6934b479, 93 },
    { 0x58a4b478, 23 },
    { 0xec4b47b, 61 },
    { 0x9134b47d, 46 },
    { 0, 0 },
    { 0x94a4b47f, 33 },
    { 0x7714b47f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c94b483, 66 },
    { 0x93c4b483, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2224b489, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb644b48c, 43 },
    { 0x5f44b48d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944b491, 33 },
    { 0x2dd4b492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2014b495, 99 },
    { 0x9f24b496, 65 },
    { 0, 0 },
    { 0x5be4b498, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde34b49f, 61 },
    { 0x84c4b4a0, 34 },
    { 0x18e4b4a1, 33 },
    { 0x3684b49f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554b4a8, 100 },
    { 0, 0 },
    { 0x96c4b4aa, 76 },
    { 0xad84b4aa, 61 },
    { 0x5ca4b4ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb24b4b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca54b4b5, 17 },
    { 0x84c4b4b5, 98 },
    { 0x19c4b4b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd74b4bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54b4c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494b4c5, 90 },
    { 0x5774b4c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd34b4cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2584b4d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9154b4d4, 43 },
    { 0x3774b4d5, 52 },
    { 0xfd54b4d6, 34 },
    { 0x8e74b4d7, 33 },
    { 0x9a64b4d8, 33 },
    { 0x5b44b4d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5474b4df, 77 },
    { 0x45f4b4df, 52 },
    { 0, 0 },
    { 0x44b4b4e2, 61 },
    { 0x9364b4e2, 81 },
    { 0xadc4b4e4, 95 },
    { 0xacc4b4e4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8194b4ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a4b4ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7234b4f2, 76 },
    { 0x4f54b4f3, 91 },
    { 0, 0 },
    { 0x32f4b4f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1084b4f9, 23 },
    { 0x6914b4fa, 52 },
    { 0xf5c4b4fb, 77 },
    { 0xd884b4fc, 93 },
    { 0xad34b4fd, 61 },
    { 0xaf84b4fb, 76 },
    { 0x8814b4fe, 32 },
    { 0, 0 },
    { 0x4434b501, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3104b507, 88 },
    { 0xc914b508, 23 },
    { 0x2144b507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a24b50c, 95 },
    { 0xfac4b50c, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a4b511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb4b514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34b517, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b4b51c, 33 },
    { 0x60f4b51d, 61 },
    { 0x3ce4b51c, 84 },
    { 0, 0 },
    { 0xa0a4b520, 78 },
    { 0xa064b521, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1674b524, 90 },
    { 0xe184b525, 65 },
    { 0x15f4b525, 33 },
    { 0x294b525, 67 },
    { 0x644b528, 46 },
    { 0xf4f4b529, 90 },
    { 0, 0 },
    { 0x954b52b, 60 },
    { 0, 0 },
    { 0x23f4b52d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4964b531, 98 },
    { 0x5684b532, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e94b535, 95 },
    { 0x5c74b535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a14b53a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b34b53e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9594b541, 52 },
    { 0xff34b541, 90 },
    { 0x16b4b542, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4564b546, 44 },
    { 0x3054b547, 46 },
    { 0xc654b548, 77 },
    { 0xe2c4b546, 93 },
    { 0, 0 },
    { 0xc2c4b54b, 98 },
    { 0x3aa4b54c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb334b551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e4b555, 61 },
    { 0, 0 },
    { 0x4484b557, 93 },
    { 0x7744b557, 66 },
    { 0x1974b559, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8624b569, 93 },
    { 0, 0 },
    { 0x64c4b56b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d4b571, 85 },
    { 0, 0 },
    { 0x5ad4b573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9024b579, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab4b57e, 34 },
    { 0, 0 },
    { 0xccd4b580, 61 },
    { 0x98e4b580, 46 },
    { 0xc834b582, 20 },
    { 0xbb64b582, 33 },
    { 0xf794b584, 47 },
    { 0, 0 },
    { 0x21e4b586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa4b58b, 66 },
    { 0, 0 },
    { 0xc704b58d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8374b594, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b4b598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd544b5a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a64b5ad, 55 },
    { 0x5a84b5ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f44b5b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x734b5b5, 90 },
    { 0xf4d4b5b5, 33 },
    { 0, 0 },
    { 0xe9e4b5b8, 95 },
    { 0x7e34b5b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f4b5c1, 90 },
    { 0xbeb4b5c2, 33 },
    { 0, 0 },
    { 0xd064b5c4, 81 },
    { 0x9bd4b5c4, 55 },
    { 0, 0 },
    { 0xe6d4b5c7, 98 },
    { 0, 0 },
    { 0x4b74b5c9, 90 },
    { 0x8ee4b5c9, 75 },
    { 0, 0 },
    { 0x7484b5cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab4b5d1, 98 },
    { 0x52d4b5d2, 99 },
    { 0xfc14b5d3, 91 },
    { 0xe54b5d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c74b5d9, 90 },
    { 0xe264b5d9, 95 },
    { 0xcca4b5db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc324b5e5, 52 },
    { 0x7244b5e6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74b5ea, 61 },
    { 0, 0 },
    { 0x71d4b5ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f34b5f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da4b5fb, 95 },
    { 0x29d4b5fb, 33 },
    { 0x42d4b5fd, 61 },
    { 0x5064b5fb, 90 },
    { 0x68c4b5fd, 90 },
    { 0, 0 },
    { 0x6e74b601, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd334b607, 98 },
    { 0x1404b608, 93 },
    { 0x62c4b609, 91 },
    { 0x1d34b60a, 23 },
    { 0x1984b607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff34b60f, 33 },
    { 0, 0 },
    { 0x844b611, 87 },
    { 0xf194b612, 95 },
    { 0, 0 },
    { 0xd264b614, 85 },
    { 0xcce4b615, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c64b618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194b620, 98 },
    { 0, 0 },
    { 0xbee4b622, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1674b626, 91 },
    { 0xbe44b626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc684b62c, 19 },
    { 0x5414b62d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7054b630, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8764b635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5314b638, 81 },
    { 0x4354b639, 98 },
    { 0xf944b63a, 90 },
    { 0x68c4b639, 33 },
    { 0x3814b63c, 61 },
    { 0x1a4b63d, 61 },
    { 0x9804b639, 46 },
    { 0x88e4b639, 66 },
    { 0xe634b63d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c4b644, 98 },
    { 0, 0 },
    { 0x7a14b646, 95 },
    { 0x9434b646, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca4b64d, 56 },
    { 0xe194b64e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e94b651, 78 },
    { 0, 0 },
    { 0xda4b653, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b4b656, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f4b659, 90 },
    { 0x3a54b65a, 34 },
    { 0, 0 },
    { 0x3214b65c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e4b661, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe654b665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a4b66e, 91 },
    { 0xa594b66f, 98 },
    { 0xfca4b670, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb64b676, 76 },
    { 0, 0 },
    { 0xa2e4b678, 98 },
    { 0, 0 },
    { 0x82e4b67a, 23 },
    { 0, 0 },
    { 0xf544b67c, 23 },
    { 0x1b14b67c, 95 },
    { 0xf104b67e, 66 },
    { 0, 0 },
    { 0xf594b680, 34 },
    { 0x2664b681, 90 },
    { 0xf414b681, 98 },
    { 0, 0 },
    { 0xc114b684, 90 },
    { 0xd004b684, 76 },
    { 0x57c4b686, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf924b689, 33 },
    { 0, 0 },
    { 0x7d04b68b, 85 },
    { 0xa3e4b68b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba4b68f, 90 },
    { 0x5064b68f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b4b695, 76 },
    { 0xf6d4b695, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e4b69a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc4b69f, 78 },
    { 0x9264b6a0, 91 },
    { 0x3a54b6a0, 61 },
    { 0, 0 },
    { 0x71b4b6a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd4b6aa, 95 },
    { 0, 0 },
    { 0x3404b6ac, 33 },
    { 0, 0 },
    { 0x4504b6ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9514b6b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94b6b4, 61 },
    { 0x25f4b6b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd4b6b8, 93 },
    { 0x4ee4b6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd074b6be, 90 },
    { 0, 0 },
    { 0x7854b6c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f4b6c8, 90 },
    { 0x1004b6c9, 67 },
    { 0x3194b6ca, 90 },
    { 0, 0 },
    { 0x9544b6cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b6d4, 65 },
    { 0, 0 },
    { 0x8ef4b6d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa014b6d9, 94 },
    { 0xb8c4b6da, 95 },
    { 0xed24b6da, 55 },
    { 0x5014b6d9, 95 },
    { 0xb5c4b6dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d4b6e3, 76 },
    { 0xb604b6e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9534b6e8, 61 },
    { 0, 0 },
    { 0x6ab4b6ea, 98 },
    { 0, 0 },
    { 0xbdc4b6ec, 33 },
    { 0, 0 },
    { 0x7934b6ee, 61 },
    { 0xde34b6ef, 90 },
    { 0x4cf4b6f0, 69 },
    { 0x1bb4b6f1, 95 },
    { 0x47a4b6f0, 98 },
    { 0x1094b6f3, 34 },
    { 0x3d54b6ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4734b6fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f4b6fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5004b703, 95 },
    { 0, 0 },
    { 0xdf94b705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc404b708, 93 },
    { 0, 0 },
    { 0xbdc4b70a, 23 },
    { 0, 0 },
    { 0x54c4b70c, 99 },
    { 0x8084b70d, 98 },
    { 0, 0 },
    { 0x3784b70f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e4b712, 90 },
    { 0x1314b713, 88 },
    { 0, 0 },
    { 0xe894b715, 17 },
    { 0xfb74b716, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14b719, 98 },
    { 0, 0 },
    { 0xaa34b71b, 81 },
    { 0, 0 },
    { 0x7594b71d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e4b721, 93 },
    { 0, 0 },
    { 0x13e4b723, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f4b729, 60 },
    { 0, 0 },
    { 0xc7e4b72b, 95 },
    { 0x824b72b, 90 },
    { 0, 0 },
    { 0x27e4b72e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d4b731, 46 },
    { 0, 0 },
    { 0xe2a4b733, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7194b738, 93 },
    { 0xa9d4b739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2884b73c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294b742, 33 },
    { 0xeb54b742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa334b746, 34 },
    { 0, 0 },
    { 0x71c4b748, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3884b74c, 90 },
    { 0, 0 },
    { 0xef24b74e, 34 },
    { 0x2d44b74f, 95 },
    { 0, 0 },
    { 0x8d64b751, 77 },
    { 0xc3f4b752, 90 },
    { 0x6a04b753, 90 },
    { 0x9154b751, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4094b75a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54b75d, 33 },
    { 0xd164b75d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9594b764, 61 },
    { 0x7684b765, 90 },
    { 0, 0 },
    { 0x36f4b767, 33 },
    { 0xf244b768, 23 },
    { 0x6374b768, 61 },
    { 0, 0 },
    { 0xa904b76b, 85 },
    { 0x8e74b76c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd584b76f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214b778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d4b77c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234b780, 65 },
    { 0, 0 },
    { 0xe544b782, 61 },
    { 0x8c74b783, 23 },
    { 0x7674b784, 90 },
    { 0, 0 },
    { 0x1534b786, 23 },
    { 0, 0 },
    { 0x30b4b788, 61 },
    { 0xa34b789, 76 },
    { 0x3ca4b78a, 23 },
    { 0x93e4b78b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b44b790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x914b793, 34 },
    { 0x2334b793, 98 },
    { 0xbdd4b795, 95 },
    { 0xb024b796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa474b79a, 91 },
    { 0, 0 },
    { 0x8d4b79c, 90 },
    { 0, 0 },
    { 0xf6c4b79e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4b7a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc4b7a7, 93 },
    { 0x3094b7a8, 75 },
    { 0xd304b7a9, 87 },
    { 0x3bd4b7aa, 98 },
    { 0, 0 },
    { 0x2d4b7ac, 34 },
    { 0xc364b7ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4b7b0, 34 },
    { 0, 0 },
    { 0xaa14b7b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa824b7b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade4b7ba, 23 },
    { 0x33a4b7ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e4b7be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e14b7c1, 23 },
    { 0, 0 },
    { 0xd14b7c3, 23 },
    { 0, 0 },
    { 0x4c84b7c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7054b7c8, 95 },
    { 0, 0 },
    { 0x94e4b7ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b4b7ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xef84b7d1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a4b7d6, 95 },
    { 0x8254b7d6, 33 },
    { 0x5574b7d6, 23 },
    { 0x1784b7d9, 33 },
    { 0x51d4b7da, 55 },
    { 0x2544b7db, 66 },
    { 0x5804b7dc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xef94b7df, 55 },
    { 0x9e4b7e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b4b7e4, 23 },
    { 0x25b4b7e5, 66 },
    { 0x13c4b7e4, 98 },
    { 0x5044b7e7, 95 },
    { 0xbbb4b7e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b84b7eb, 61 },
    { 0, 0 },
    { 0x7d94b7ed, 95 },
    { 0x94f4b7ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f4b7f4, 33 },
    { 0x5b44b7f5, 61 },
    { 0x3e94b7f4, 95 },
    { 0xe0e4b7f6, 34 },
    { 0, 0 },
    { 0xec84b7f9, 40 },
    { 0xb74b7fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab4b7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad4b803, 55 },
    { 0xbac4b804, 95 },
    { 0x6204b803, 76 },
    { 0x92f4b804, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad4b810, 93 },
    { 0, 0 },
    { 0x7f54b812, 23 },
    { 0xb8b4b813, 88 },
    { 0x8de4b813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6934b818, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c4b81d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca64b820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2044b82b, 84 },
    { 0xfe54b82c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf14b831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1554b835, 95 },
    { 0xe034b836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb064b839, 23 },
    { 0xf3d4b839, 90 },
    { 0xad54b83b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4534b83f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee4b842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f4b846, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe714b84b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d4b84f, 81 },
    { 0, 0 },
    { 0x7664b851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4b854, 90 },
    { 0xbb84b855, 17 },
    { 0x714b856, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa534b859, 52 },
    { 0xb534b85a, 98 },
    { 0x20d4b85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4394b85f, 33 },
    { 0xdc84b85f, 55 },
    { 0x27b4b860, 34 },
    { 0x2fb4b860, 61 },
    { 0xf8f4b863, 95 },
    { 0xafc4b863, 65 },
    { 0, 0 },
    { 0x52f4b866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d4b86a, 61 },
    { 0, 0 },
    { 0x73e4b86c, 76 },
    { 0xc8b4b86d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4b870, 34 },
    { 0xb8a4b871, 95 },
    { 0xd714b870, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6424b875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e4b878, 93 },
    { 0xf254b879, 99 },
    { 0x8304b879, 75 },
    { 0x9934b87b, 69 },
    { 0x73f4b87b, 61 },
    { 0xc754b87c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef4b880, 33 },
    { 0xe424b881, 87 },
    { 0x34d4b882, 100 },
    { 0x694b882, 97 },
    { 0xd9a4b884, 85 },
    { 0xc7f4b885, 61 },
    { 0xd604b886, 40 },
    { 0xda34b886, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a4b88a, 52 },
    { 0x6e34b88b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1114b890, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14b898, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb4b89e, 61 },
    { 0, 0 },
    { 0xbbe4b8a0, 33 },
    { 0, 0 },
    { 0x51f4b8a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xece4b8a5, 90 },
    { 0x9b24b8a6, 96 },
    { 0x8f44b8a5, 95 },
    { 0x2124b8a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9384b8ac, 61 },
    { 0x25e4b8ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb14b8b3, 85 },
    { 0, 0 },
    { 0xea94b8b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d4b8ba, 85 },
    { 0x814b8ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7584b8c3, 23 },
    { 0x37e4b8c4, 61 },
    { 0x3ad4b8c3, 43 },
    { 0xae4b8c6, 98 },
    { 0, 0 },
    { 0x4c54b8c8, 52 },
    { 0, 0 },
    { 0xcfc4b8ca, 101 },
    { 0x8a94b8cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b8ce, 43 },
    { 0xd2a4b8cf, 100 },
    { 0, 0 },
    { 0xd274b8d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1224b8d5, 43 },
    { 0x7614b8d6, 95 },
    { 0xa634b8d7, 90 },
    { 0x45a4b8d8, 90 },
    { 0x8c74b8d9, 43 },
    { 0x7294b8d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a34b8dd, 76 },
    { 0x8214b8dd, 90 },
    { 0x4a34b8df, 81 },
    { 0x6bf4b8e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc414b8e9, 95 },
    { 0x1a24b8ea, 23 },
    { 0x3bd4b8eb, 23 },
    { 0xcf04b8ec, 34 },
    { 0x3764b8ed, 87 },
    { 0x8a34b8eb, 61 },
    { 0xda74b8ec, 23 },
    { 0x1474b8f0, 91 },
    { 0xf6f4b8ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d4b8f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2184b8fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd4b8ff, 61 },
    { 0x1fe4b900, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xda34b903, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4b906, 43 },
    { 0x37f4b907, 85 },
    { 0xf5f4b906, 99 },
    { 0xc154b907, 90 },
    { 0x68f4b906, 91 },
    { 0xc6e4b90b, 100 },
    { 0x3e04b90c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c44b90f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44b912, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c4b91a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d04b91d, 76 },
    { 0, 0 },
    { 0x7b44b91f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf864b927, 98 },
    { 0x4174b928, 94 },
    { 0xa7b4b929, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c14b92f, 43 },
    { 0xf8c4b930, 88 },
    { 0x7d44b931, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7324b934, 66 },
    { 0xc94b934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b4b93a, 100 },
    { 0x3514b93b, 90 },
    { 0x7ed4b93c, 90 },
    { 0, 0 },
    { 0xbfe4b93e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234b942, 61 },
    { 0x26e4b943, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6534b947, 23 },
    { 0xaac4b947, 91 },
    { 0, 0 },
    { 0xf74b94a, 17 },
    { 0, 0 },
    { 0xf6b4b94c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb874b950, 43 },
    { 0x9344b950, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe934b956, 76 },
    { 0x6b74b956, 40 },
    { 0, 0 },
    { 0xc164b959, 98 },
    { 0, 0 },
    { 0x9bc4b95b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4b95e, 61 },
    { 0, 0 },
    { 0xac44b960, 98 },
    { 0x9ea4b960, 55 },
    { 0, 0 },
    { 0x95b4b963, 91 },
    { 0x7454b964, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4b969, 78 },
    { 0, 0 },
    { 0x4614b96b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5534b96e, 34 },
    { 0xe434b96f, 55 },
    { 0x6b04b96e, 36 },
    { 0x2134b971, 76 },
    { 0xe1b4b972, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4984b976, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a4b97c, 84 },
    { 0, 0 },
    { 0xd1f4b97e, 76 },
    { 0, 0 },
    { 0xfd84b980, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b4b983, 55 },
    { 0, 0 },
    { 0x85b4b985, 93 },
    { 0x6344b986, 33 },
    { 0x9494b987, 66 },
    { 0xf764b986, 65 },
    { 0xee24b989, 66 },
    { 0xa794b989, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e4b991, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b994, 76 },
    { 0xdac4b995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c4b99d, 33 },
    { 0x5444b99d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f34b9a3, 55 },
    { 0, 0 },
    { 0x5814b9a5, 76 },
    { 0xd964b9a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b4b9aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44b9ad, 46 },
    { 0x2954b9ad, 52 },
    { 0x38e4b9ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc4b9b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b9bc, 23 },
    { 0xe914b9bd, 33 },
    { 0x6be4b9be, 88 },
    { 0xeff4b9bf, 90 },
    { 0xcc84b9c0, 78 },
    { 0x49b4b9bf, 69 },
    { 0xb754b9c2, 43 },
    { 0x8994b9c3, 23 },
    { 0xe4d4b9c4, 80 },
    { 0xa54b9be, 55 },
    { 0, 0 },
    { 0x1714b9c7, 40 },
    { 0x8274b9c7, 23 },
    { 0, 0 },
    { 0x25d4b9ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f4b9cd, 76 },
    { 0x7ec4b9ce, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a44b9dd, 78 },
    { 0xe854b9de, 33 },
    { 0x7c24b9df, 68 },
    { 0xdae4b9dd, 33 },
    { 0x9744b9e1, 46 },
    { 0, 0 },
    { 0x43d4b9e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c94b9e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e94b9eb, 87 },
    { 0, 0 },
    { 0x8854b9ed, 23 },
    { 0x96d4b9ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce74b9f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74b9fa, 90 },
    { 0x5fc4b9fa, 91 },
    { 0xdc64b9fc, 90 },
    { 0xe454b9fa, 43 },
    { 0xc344b9fe, 33 },
    { 0x8684b9ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb144ba03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344ba07, 90 },
    { 0, 0 },
    { 0xd804ba09, 95 },
    { 0xcec4ba0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4ba0e, 66 },
    { 0, 0 },
    { 0x5154ba10, 34 },
    { 0x8eb4ba11, 46 },
    { 0x1804ba12, 23 },
    { 0xa5d4ba12, 75 },
    { 0x1fb4ba14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14ba1d, 95 },
    { 0x33d4ba1d, 85 },
    { 0xdd74ba1f, 98 },
    { 0, 0 },
    { 0xaa74ba21, 23 },
    { 0, 0 },
    { 0x53c4ba23, 23 },
    { 0xb984ba24, 90 },
    { 0xa064ba23, 78 },
    { 0xb7f4ba26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4ba2b, 46 },
    { 0xf8c4ba2b, 98 },
    { 0x8004ba2d, 33 },
    { 0x5d14ba2e, 95 },
    { 0, 0 },
    { 0xb774ba30, 43 },
    { 0, 0 },
    { 0x9004ba32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6294ba38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7444ba3f, 23 },
    { 0, 0 },
    { 0x11d4ba41, 95 },
    { 0x5874ba42, 95 },
    { 0x42b4ba43, 90 },
    { 0, 0 },
    { 0xea14ba45, 98 },
    { 0x87b4ba46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c54ba4c, 55 },
    { 0, 0 },
    { 0x8124ba4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a4ba51, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f94ba57, 55 },
    { 0x7ff4ba58, 46 },
    { 0x6b64ba59, 100 },
    { 0x5ce4ba58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2874ba5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b4ba62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24ba67, 33 },
    { 0, 0 },
    { 0xd3c4ba69, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa374ba71, 90 },
    { 0xafe4ba72, 61 },
    { 0xa164ba73, 95 },
    { 0xeb14ba72, 95 },
    { 0xd1b4ba72, 90 },
    { 0xf54ba76, 67 },
    { 0x8f64ba77, 99 },
    { 0x6cf4ba71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae4ba7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944ba7f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe494ba87, 95 },
    { 0x5714ba88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc4ba8e, 87 },
    { 0x4f54ba8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34ba92, 61 },
    { 0x2f44ba92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8114ba96, 44 },
    { 0x7964ba97, 95 },
    { 0x4844ba98, 23 },
    { 0x6d4ba99, 84 },
    { 0x1844ba96, 90 },
    { 0xeab4ba9a, 98 },
    { 0x2584ba9c, 33 },
    { 0, 0 },
    { 0xb904ba9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a94baa4, 61 },
    { 0, 0 },
    { 0xce14baa6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1284baa9, 90 },
    { 0x88a4baaa, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf34bab8, 43 },
    { 0x50a4bab9, 12 },
    { 0xb454baba, 95 },
    { 0xf9c4baba, 61 },
    { 0x60f4babb, 61 },
    { 0x7a74babc, 61 },
    { 0xd854baba, 43 },
    { 0x46e4babf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2484bac2, 46 },
    { 0xce24bac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4bac8, 98 },
    { 0, 0 },
    { 0x6244baca, 76 },
    { 0x4c44bacb, 98 },
    { 0, 0 },
    { 0x1264bacd, 55 },
    { 0x2f44bacd, 46 },
    { 0x9694bacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe4bad5, 95 },
    { 0, 0 },
    { 0x79f4bad7, 33 },
    { 0xb424bad8, 98 },
    { 0xc5f4bad7, 33 },
    { 0xf4f4bad8, 91 },
    { 0xe2d4badb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef94bade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4bae3, 90 },
    { 0x6564bae4, 93 },
    { 0x6594bae5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb4baeb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324baee, 90 },
    { 0xc9d4baee, 88 },
    { 0x1d34baee, 52 },
    { 0xfea4baf1, 66 },
    { 0xb34baef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd134baf7, 81 },
    { 0, 0 },
    { 0xd444baf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1694bb00, 52 },
    { 0, 0 },
    { 0x77a4bb02, 61 },
    { 0xbb14bb03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4224bb06, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd404bb09, 33 },
    { 0x9994bb0a, 33 },
    { 0x8554bb0b, 76 },
    { 0xbac4bb0b, 98 },
    { 0x3094bb0c, 90 },
    { 0x2104bb0d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba4bb11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac34bb14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b44bb19, 74 },
    { 0xfa64bb1a, 55 },
    { 0x1724bb1b, 61 },
    { 0, 0 },
    { 0x95a4bb1d, 66 },
    { 0xd804bb1e, 68 },
    { 0, 0 },
    { 0xee44bb20, 69 },
    { 0xd254bb20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc74bb29, 90 },
    { 0x5b4bb2a, 90 },
    { 0xefa4bb2b, 98 },
    { 0xfb24bb29, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4bb30, 98 },
    { 0, 0 },
    { 0x38a4bb32, 33 },
    { 0x87c4bb33, 75 },
    { 0xd94bb32, 65 },
    { 0, 0 },
    { 0xde34bb36, 33 },
    { 0x7f64bb37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4bb3d, 33 },
    { 0xace4bb3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1814bb41, 33 },
    { 0x7774bb41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f34bb47, 98 },
    { 0, 0 },
    { 0xfa94bb49, 46 },
    { 0x9454bb4a, 98 },
    { 0, 0 },
    { 0xc4b4bb4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b4bb50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc44bb57, 84 },
    { 0xa8d4bb58, 24 },
    { 0xf764bb58, 100 },
    { 0, 0 },
    { 0xfda4bb5b, 98 },
    { 0xe284bb5b, 43 },
    { 0xdab4bb5d, 33 },
    { 0x5444bb5e, 61 },
    { 0, 0 },
    { 0xbb84bb60, 90 },
    { 0x2984bb60, 90 },
    { 0x5864bb61, 90 },
    { 0x674bb63, 95 },
    { 0x9e54bb61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b4bb67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee44bb6a, 46 },
    { 0x3004bb6b, 61 },
    { 0x36f4bb6c, 66 },
    { 0, 0 },
    { 0xda34bb6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c94bb73, 95 },
    { 0, 0 },
    { 0x8484bb75, 98 },
    { 0, 0 },
    { 0xc604bb77, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a44bb7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5924bb81, 61 },
    { 0x7734bb81, 23 },
    { 0xc164bb83, 23 },
    { 0xfb34bb82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d4bb8a, 84 },
    { 0, 0 },
    { 0x6614bb8c, 76 },
    { 0xe7f4bb8d, 34 },
    { 0xc114bb8e, 52 },
    { 0xd834bb8e, 33 },
    { 0xd5c4bb8c, 98 },
    { 0xee24bb8c, 90 },
    { 0, 0 },
    { 0x1244bb93, 34 },
    { 0, 0 },
    { 0x2734bb95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa694bb99, 99 },
    { 0x5b94bb99, 98 },
    { 0x3964bb99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd4bb9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d4bba3, 98 },
    { 0x3df4bba3, 68 },
    { 0xc4a4bba3, 90 },
    { 0x4c4bba6, 61 },
    { 0x6e74bba7, 34 },
    { 0x5054bba7, 23 },
    { 0x3c04bba9, 78 },
    { 0xc6b4bba9, 61 },
    { 0xbf74bbab, 95 },
    { 0x6fb4bba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa4bbaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f4bbb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd4bbb6, 91 },
    { 0x4ff4bbb7, 61 },
    { 0, 0 },
    { 0xca54bbb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a74bbc6, 61 },
    { 0, 0 },
    { 0xa2f4bbc8, 52 },
    { 0, 0 },
    { 0x91c4bbca, 61 },
    { 0, 0 },
    { 0x8d84bbcc, 98 },
    { 0xd564bbcd, 95 },
    { 0, 0 },
    { 0x9144bbcf, 90 },
    { 0, 0 },
    { 0x17e4bbd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b4bbd6, 98 },
    { 0xa0c4bbd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064bbe6, 34 },
    { 0xe24bbe6, 90 },
    { 0x8704bbe8, 93 },
    { 0xea14bbe8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b24bbf0, 61 },
    { 0xcaa4bbf1, 55 },
    { 0xaa94bbf2, 95 },
    { 0xb944bbf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb4bbf8, 81 },
    { 0x45b4bbf9, 42 },
    { 0xa474bbf9, 91 },
    { 0x54d4bbf9, 23 },
    { 0x604bbfc, 98 },
    { 0x3394bbfd, 34 },
    { 0x2374bbfe, 46 },
    { 0x9af4bbfc, 90 },
    { 0xcc84bc00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6474bc06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7644bc0a, 66 },
    { 0xb434bc0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda4bc10, 87 },
    { 0x7cd4bc11, 99 },
    { 0x84a4bc12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd014bc15, 85 },
    { 0xac54bc15, 95 },
    { 0x2134bc17, 90 },
    { 0x7134bc15, 17 },
    { 0x4514bc16, 90 },
    { 0x52f4bc1a, 90 },
    { 0x61d4bc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5234bc1f, 90 },
    { 0, 0 },
    { 0x5264bc21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4bc26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e4bc29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c4bc2c, 35 },
    { 0, 0 },
    { 0x4f14bc2e, 61 },
    { 0xe364bc2f, 8 },
    { 0x6b44bc30, 34 },
    { 0x4ce4bc31, 34 },
    { 0, 0 },
    { 0x8bc4bc33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa544bc3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe174bc3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b4bc42, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f4bc46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8214bc53, 99 },
    { 0, 0 },
    { 0x7754bc55, 89 },
    { 0x14d4bc56, 23 },
    { 0xd684bc57, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c4bc5d, 82 },
    { 0x9e74bc5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6354bc62, 76 },
    { 0, 0 },
    { 0xc7c4bc64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c4bc68, 93 },
    { 0x6934bc68, 55 },
    { 0xbc34bc69, 61 },
    { 0xc144bc6b, 34 },
    { 0, 0 },
    { 0x98b4bc6d, 95 },
    { 0x7104bc6e, 87 },
    { 0, 0 },
    { 0x5434bc70, 96 },
    { 0, 0 },
    { 0xe334bc72, 61 },
    { 0x55e4bc73, 46 },
    { 0x6764bc73, 61 },
    { 0x3ad4bc75, 90 },
    { 0xfa34bc76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a54bc79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7374bc7c, 97 },
    { 0xef84bc7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e44bc83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a34bc88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f4bc8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3174bc91, 69 },
    { 0xe214bc92, 98 },
    { 0xc34bc92, 33 },
    { 0x43b4bc92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4bc99, 90 },
    { 0x1954bc99, 95 },
    { 0x2204bc9b, 67 },
    { 0xdaf4bc9b, 23 },
    { 0x3cb4bc9d, 78 },
    { 0x9084bc9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc4bca4, 95 },
    { 0x874bca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd204bca9, 66 },
    { 0x93d4bcaa, 23 },
    { 0x64c4bcab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d24bcb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4bcb3, 80 },
    { 0x3014bcb4, 100 },
    { 0, 0 },
    { 0x9074bcb6, 23 },
    { 0x3104bcb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e04bcbc, 23 },
    { 0, 0 },
    { 0xea94bcbe, 68 },
    { 0x2fe4bcbf, 61 },
    { 0x4764bcc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b4bcd2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad4bcd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa114bcdc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc34bce1, 33 },
    { 0x7bb4bce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf024bce5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6134bcea, 87 },
    { 0xc764bceb, 67 },
    { 0x7a94bcec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf64bcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c54bcf3, 90 },
    { 0, 0 },
    { 0xdee4bcf5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff4bcf8, 98 },
    { 0x70b4bcf8, 43 },
    { 0, 0 },
    { 0x6254bcfb, 95 },
    { 0, 0 },
    { 0xfa54bcfd, 95 },
    { 0xce54bcfe, 90 },
    { 0x47b4bcff, 75 },
    { 0xc9f4bd00, 55 },
    { 0x75e4bd00, 23 },
    { 0xf434bcfe, 23 },
    { 0xbb34bcff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b4bd06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe334bd0b, 98 },
    { 0x6674bd0b, 95 },
    { 0, 0 },
    { 0x3844bd0e, 61 },
    { 0x50c4bd0f, 90 },
    { 0x6d44bd0e, 23 },
    { 0, 0 },
    { 0xd244bd12, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd4bd18, 95 },
    { 0, 0 },
    { 0xa994bd1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f44bd1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e14bd20, 32 },
    { 0, 0 },
    { 0xeab4bd22, 55 },
    { 0x7664bd23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b04bd27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe284bd2c, 90 },
    { 0xdfe4bd2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb384bd34, 55 },
    { 0, 0 },
    { 0x7554bd36, 93 },
    { 0xc84bd36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xef54bd3a, 90 },
    { 0x2bf4bd3b, 98 },
    { 0x9954bd3b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf4bd3f, 23 },
    { 0x25a4bd3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3714bd51, 61 },
    { 0, 0 },
    { 0x72c4bd53, 90 },
    { 0, 0 },
    { 0xef04bd55, 100 },
    { 0xad84bd56, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c84bd5a, 84 },
    { 0xa434bd5a, 95 },
    { 0xa874bd5c, 61 },
    { 0xf014bd5d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8334bd66, 93 },
    { 0x6784bd67, 66 },
    { 0xae34bd68, 23 },
    { 0xcc94bd69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe364bd70, 95 },
    { 0xfc84bd70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5084bd75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a44bd78, 33 },
    { 0xba14bd79, 23 },
    { 0xb094bd7a, 65 },
    { 0x9e54bd79, 98 },
    { 0, 0 },
    { 0x6e24bd7d, 99 },
    { 0x1d14bd7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc4bd8b, 85 },
    { 0xcba4bd8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9384bd8f, 36 },
    { 0, 0 },
    { 0xede4bd91, 98 },
    { 0xcae4bd92, 55 },
    { 0x6994bd93, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc914bd97, 90 },
    { 0x37d4bd97, 61 },
    { 0, 0 },
    { 0xfb24bd9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e64bda2, 98 },
    { 0xfad4bda3, 80 },
    { 0, 0 },
    { 0x7d84bda5, 24 },
    { 0, 0 },
    { 0xb914bda7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa514bdaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c4bdad, 76 },
    { 0, 0 },
    { 0xf3e4bdaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b4bdb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe334bdb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d04bdbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f44bdc0, 90 },
    { 0xf6d4bdc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704bdc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d4bdca, 84 },
    { 0, 0 },
    { 0xe3a4bdcc, 23 },
    { 0x6f94bdcd, 61 },
    { 0xa804bdce, 55 },
    { 0x8c14bdcf, 78 },
    { 0x8154bdcf, 23 },
    { 0xd7a4bdd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6914bdd5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d4bdda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd234bddf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3534bde4, 61 },
    { 0x3e44bde5, 98 },
    { 0, 0 },
    { 0xa7c4bde7, 23 },
    { 0xab94bde8, 90 },
    { 0xb1d4bde9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d24bdec, 95 },
    { 0xacd4bded, 76 },
    { 0x41d4bdee, 76 },
    { 0xd964bded, 95 },
    { 0x8704bdf0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf564bdf4, 67 },
    { 0x2e64bdf4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614bdf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c4bdfd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54be0b, 46 },
    { 0, 0 },
    { 0x5104be0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd4be11, 33 },
    { 0, 0 },
    { 0x4354be13, 90 },
    { 0, 0 },
    { 0x5154be15, 33 },
    { 0x4b04be16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d4be1c, 90 },
    { 0x93b4be1c, 93 },
    { 0x1e24be1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4be25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f4be2b, 95 },
    { 0x3254be2c, 23 },
    { 0, 0 },
    { 0x51d4be2e, 34 },
    { 0xd424be2f, 34 },
    { 0x874be30, 90 },
    { 0, 0 },
    { 0x9844be32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354be36, 87 },
    { 0x89e4be36, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab14be40, 61 },
    { 0xdd64be41, 33 },
    { 0x6e54be40, 90 },
    { 0xd4e4be43, 90 },
    { 0xc4f4be44, 55 },
    { 0x3714be45, 46 },
    { 0x1774be45, 95 },
    { 0x9e44be47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc074be4c, 34 },
    { 0x5c4be4d, 67 },
    { 0, 0 },
    { 0xc634be4f, 76 },
    { 0x4444be50, 61 },
    { 0x3204be51, 90 },
    { 0x4514be52, 33 },
    { 0x7fa4be52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb664be57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf614be5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc14be62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4be66, 83 },
    { 0x5704be67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504be6a, 97 },
    { 0x8d04be6b, 66 },
    { 0x8d4be6c, 84 },
    { 0x68d4be6b, 61 },
    { 0x36d4be6e, 90 },
    { 0, 0 },
    { 0xb0e4be70, 61 },
    { 0x86d4be71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c4be74, 23 },
    { 0x4ed4be75, 77 },
    { 0x4a34be75, 66 },
    { 0x3c44be77, 61 },
    { 0x5a24be77, 33 },
    { 0xe104be77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804be7d, 90 },
    { 0xd5a4be7d, 46 },
    { 0xd2b4be7f, 61 },
    { 0x7a84be7f, 87 },
    { 0x2b94be81, 77 },
    { 0xfb54be7f, 87 },
    { 0xe9c4be83, 61 },
    { 0x93b4be84, 34 },
    { 0xa8e4be7d, 98 },
    { 0x1a14be81, 95 },
    { 0x3674be87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9754be90, 76 },
    { 0x2794be91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e24be97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc734bea0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a14bea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6864beae, 61 },
    { 0, 0 },
    { 0x2fc4beb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7234bebd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc4bec0, 61 },
    { 0xce24bec1, 33 },
    { 0, 0 },
    { 0x8e34bec3, 90 },
    { 0x4384bec4, 90 },
    { 0x5f04bec5, 87 },
    { 0, 0 },
    { 0x9924bec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b4becc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24becf, 23 },
    { 0x9404becf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e74bed4, 33 },
    { 0, 0 },
    { 0xd824bed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5324bedb, 76 },
    { 0xbdd4bedb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14bee6, 88 },
    { 0xd604bee7, 100 },
    { 0xb974bee7, 33 },
    { 0x4cb4bee9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b4bef0, 33 },
    { 0, 0 },
    { 0xc084bef2, 99 },
    { 0, 0 },
    { 0x5cf4bef4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa064beff, 76 },
    { 0xeb34bf00, 23 },
    { 0, 0 },
    { 0xebf4bf02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa204bf08, 95 },
    { 0xf4bf09, 95 },
    { 0x8864bf0a, 98 },
    { 0xabe4bf09, 46 },
    { 0, 0 },
    { 0x8ef4bf0d, 76 },
    { 0xd074bf0d, 61 },
    { 0, 0 },
    { 0x1284bf10, 95 },
    { 0x91a4bf10, 27 },
    { 0, 0 },
    { 0x8ac4bf13, 95 },
    { 0x2b64bf14, 61 },
    { 0, 0 },
    { 0xbfe4bf16, 61 },
    { 0xa24bf17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e64bf1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc474bf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3564bf29, 91 },
    { 0x2c14bf2a, 93 },
    { 0, 0 },
    { 0xe194bf2c, 33 },
    { 0x8644bf2d, 76 },
    { 0x4dc4bf2d, 76 },
    { 0x4c34bf2d, 61 },
    { 0xd9d4bf2c, 87 },
    { 0x604bf31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f4bf35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf794bf38, 46 },
    { 0xfc14bf38, 55 },
    { 0, 0 },
    { 0x85c4bf3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1344bf42, 34 },
    { 0, 0 },
    { 0xc924bf44, 33 },
    { 0xf434bf44, 85 },
    { 0, 0 },
    { 0x1334bf47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3624bf4a, 52 },
    { 0x5914bf4b, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c4bf51, 33 },
    { 0x2e54bf51, 98 },
    { 0, 0 },
    { 0xa104bf54, 99 },
    { 0, 0 },
    { 0x3ab4bf56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3274bf59, 55 },
    { 0x204bf59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7784bf5d, 43 },
    { 0x5864bf5e, 92 },
    { 0xa414bf5f, 95 },
    { 0x75e4bf5f, 76 },
    { 0x7104bf5f, 61 },
    { 0xb7b4bf62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa594bf6a, 81 },
    { 0, 0 },
    { 0x3d44bf6c, 95 },
    { 0, 0 },
    { 0x59f4bf6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf654bf74, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4bf77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484bf7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd974bf81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe484bf90, 98 },
    { 0, 0 },
    { 0x19c4bf92, 99 },
    { 0xe944bf93, 93 },
    { 0, 0 },
    { 0xaa74bf95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a4bf9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b4bf9e, 93 },
    { 0, 0 },
    { 0xebf4bfa0, 98 },
    { 0xe014bfa1, 23 },
    { 0xb7d4bfa2, 80 },
    { 0xcc54bfa0, 77 },
    { 0x11c4bfa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4bfa8, 33 },
    { 0, 0 },
    { 0x9154bfaa, 93 },
    { 0, 0 },
    { 0xc5d4bfac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c34bfaf, 91 },
    { 0x1df4bfaf, 61 },
    { 0x1b74bfb1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf714bfb5, 78 },
    { 0x8ed4bfb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba04bfbb, 34 },
    { 0x7844bfbb, 55 },
    { 0x5d24bfbd, 67 },
    { 0x54b4bfbe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce4bfc3, 66 },
    { 0xbe94bfc4, 91 },
    { 0, 0 },
    { 0xe284bfc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f4bfcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8774bfd0, 34 },
    { 0, 0 },
    { 0x92b4bfd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34bfd5, 76 },
    { 0, 0 },
    { 0x454bfd7, 33 },
    { 0x21a4bfd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd4bfdd, 43 },
    { 0x9744bfdd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb454bfe8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8904bfec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9324bff1, 33 },
    { 0x77a4bff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe954bff6, 87 },
    { 0xe154bff7, 98 },
    { 0x23b4bff8, 90 },
    { 0x5624bff8, 98 },
    { 0, 0 },
    { 0xc354bffb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe094c000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d14c004, 90 },
    { 0x5424c005, 90 },
    { 0x6cf4c006, 91 },
    { 0xe364c006, 98 },
    { 0, 0 },
    { 0xe204c009, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda4c00e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb044c019, 61 },
    { 0xa554c01a, 43 },
    { 0x22d4c01b, 93 },
    { 0x6a14c01c, 85 },
    { 0x70e4c01b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e54c021, 95 },
    { 0x1fb4c022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c94c02a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf14c034, 23 },
    { 0, 0 },
    { 0x59c4c036, 46 },
    { 0xf214c036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d4c03c, 61 },
    { 0x81f4c03c, 95 },
    { 0x7e64c03e, 34 },
    { 0x38c4c03f, 33 },
    { 0xc5a4c03f, 88 },
    { 0x9e54c041, 61 },
    { 0, 0 },
    { 0xe394c043, 43 },
    { 0xe8b4c044, 95 },
    { 0xbb24c044, 76 },
    { 0, 0 },
    { 0xfb24c047, 61 },
    { 0x5f04c047, 61 },
    { 0, 0 },
    { 0x1ee4c04a, 99 },
    { 0xf134c04b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc4c04e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac74c057, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b34c05b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb014c05f, 90 },
    { 0xaf54c060, 33 },
    { 0x6fa4c060, 90 },
    { 0xef64c060, 23 },
    { 0x9c44c063, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba14c069, 46 },
    { 0, 0 },
    { 0x5424c06b, 91 },
    { 0xa2e4c06b, 46 },
    { 0x7154c06d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf4c070, 93 },
    { 0, 0 },
    { 0xe614c072, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce4c077, 90 },
    { 0, 0 },
    { 0x9044c079, 78 },
    { 0x4b34c07a, 76 },
    { 0, 0 },
    { 0x7684c07c, 98 },
    { 0, 0 },
    { 0xc614c07e, 90 },
    { 0x5044c07f, 33 },
    { 0x4ed4c07f, 33 },
    { 0, 0 },
    { 0xc64c082, 90 },
    { 0x77b4c083, 43 },
    { 0x4724c084, 90 },
    { 0, 0 },
    { 0x9f14c086, 55 },
    { 0x4d54c086, 34 },
    { 0xaa74c088, 55 },
    { 0x99f4c088, 17 },
    { 0xa594c089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe84c095, 55 },
    { 0x69b4c096, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f4c09a, 64 },
    { 0xfa04c09b, 98 },
    { 0x6124c09b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce4c09f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604c0a5, 90 },
    { 0x434c0a5, 23 },
    { 0xaec4c0a7, 94 },
    { 0x3754c0a5, 90 },
    { 0, 0 },
    { 0x5ca4c0aa, 90 },
    { 0xb674c0ab, 90 },
    { 0, 0 },
    { 0x7924c0ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d4c0b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc044c0b4, 93 },
    { 0, 0 },
    { 0x7fa4c0b6, 84 },
    { 0xc974c0b6, 90 },
    { 0x5c04c0b6, 93 },
    { 0, 0 },
    { 0x9534c0ba, 33 },
    { 0x2de4c0bb, 95 },
    { 0x1674c0bb, 43 },
    { 0xf484c0ba, 33 },
    { 0, 0 },
    { 0x80c4c0bf, 23 },
    { 0, 0 },
    { 0x40b4c0c1, 66 },
    { 0xec44c0c2, 98 },
    { 0xdb94c0c3, 23 },
    { 0x8684c0c4, 23 },
    { 0x3a04c0c5, 23 },
    { 0x7494c0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c4c0ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc84c0d7, 98 },
    { 0xc294c0d8, 55 },
    { 0xcc44c0d7, 3 },
    { 0, 0 },
    { 0xcea4c0db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a24c0e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd4c0e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5374c0ea, 61 },
    { 0, 0 },
    { 0xaaa4c0ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb14c0f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd44c0f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb74c0f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb4c103, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff4c10e, 69 },
    { 0xf7a4c10e, 78 },
    { 0x67d4c10f, 43 },
    { 0x92c4c111, 93 },
    { 0, 0 },
    { 0xa004c113, 90 },
    { 0, 0 },
    { 0x5b4c115, 61 },
    { 0xe3a4c116, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7774c119, 23 },
    { 0, 0 },
    { 0x2104c11b, 91 },
    { 0x7f94c11b, 43 },
    { 0x89a4c11b, 23 },
    { 0, 0 },
    { 0x7bd4c11f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a24c123, 90 },
    { 0x1f14c124, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c4c127, 90 },
    { 0x58e4c128, 93 },
    { 0xbd64c129, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a74c132, 76 },
    { 0xf074c133, 90 },
    { 0x7494c133, 98 },
    { 0x1184c134, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4814c13a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb4c13d, 90 },
    { 0x30f4c13e, 68 },
    { 0x8ee4c13e, 20 },
    { 0x374c140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf704c143, 43 },
    { 0x4b34c143, 91 },
    { 0x4e34c145, 44 },
    { 0x5b64c146, 55 },
    { 0x3ba4c143, 75 },
    { 0, 0 },
    { 0x42d4c149, 33 },
    { 0, 0 },
    { 0x3e34c14b, 23 },
    { 0, 0 },
    { 0x8224c14d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a4c152, 90 },
    { 0, 0 },
    { 0x4414c154, 33 },
    { 0, 0 },
    { 0x7f84c156, 33 },
    { 0x3ec4c157, 33 },
    { 0x5024c157, 55 },
    { 0x2744c158, 33 },
    { 0x3374c15a, 95 },
    { 0x4284c159, 95 },
    { 0, 0 },
    { 0x2694c15d, 55 },
    { 0xf3e4c15e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c04c161, 66 },
    { 0xe654c161, 23 },
    { 0, 0 },
    { 0x37e4c164, 65 },
    { 0, 0 },
    { 0x5cd4c166, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd74c169, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6124c170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c175, 90 },
    { 0, 0 },
    { 0x46e4c177, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94c17c, 84 },
    { 0, 0 },
    { 0x96d4c17e, 95 },
    { 0x5584c17f, 95 },
    { 0x3f4c180, 85 },
    { 0, 0 },
    { 0x4c24c182, 93 },
    { 0, 0 },
    { 0x9334c184, 77 },
    { 0x2574c185, 2 },
    { 0x7a44c186, 90 },
    { 0xc114c185, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384c18b, 95 },
    { 0x3784c18c, 61 },
    { 0, 0 },
    { 0x3404c18e, 46 },
    { 0x2904c18e, 95 },
    { 0x22b4c190, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb4c196, 23 },
    { 0x74e4c196, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4c1a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a4c1a6, 34 },
    { 0x73d4c1a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5304c1ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d4c1b1, 76 },
    { 0, 0 },
    { 0x6344c1b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d94c1b7, 90 },
    { 0x6024c1b8, 23 },
    { 0, 0 },
    { 0x1754c1ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2874c1bf, 67 },
    { 0xcb24c1c0, 23 },
    { 0x6ee4c1c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b94c1cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4764c1d0, 36 },
    { 0xd794c1d0, 43 },
    { 0xa424c1d2, 23 },
    { 0x5624c1d3, 95 },
    { 0x4034c1d4, 95 },
    { 0x7114c1d5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6654c1d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14c1dc, 98 },
    { 0, 0 },
    { 0x7b84c1de, 78 },
    { 0x1d54c1df, 93 },
    { 0x45f4c1de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a4c1e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d4c1e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb54c1ec, 61 },
    { 0x6e24c1ed, 91 },
    { 0x7384c1ed, 61 },
    { 0, 0 },
    { 0xb0f4c1f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d4c1f5, 90 },
    { 0x92e4c1f6, 67 },
    { 0x1394c1f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe24c1fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f4c1fd, 98 },
    { 0xec04c1fe, 95 },
    { 0xbb34c1ff, 33 },
    { 0xca44c200, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f24c205, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c44c20a, 98 },
    { 0x2284c20b, 55 },
    { 0, 0 },
    { 0x1ee4c20d, 55 },
    { 0x7ac4c20e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f4c213, 90 },
    { 0x9274c214, 78 },
    { 0xda24c215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074c21e, 55 },
    { 0xfa94c21f, 46 },
    { 0x5eb4c21e, 61 },
    { 0x3514c221, 61 },
    { 0x85e4c222, 91 },
    { 0xf214c223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1094c229, 76 },
    { 0x5d94c229, 96 },
    { 0xbd54c22a, 90 },
    { 0xe4d4c22a, 90 },
    { 0xcf94c22d, 66 },
    { 0xd374c22e, 95 },
    { 0x3d34c22f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4284c232, 33 },
    { 0xc0a4c232, 95 },
    { 0x34c4c234, 91 },
    { 0xce34c234, 90 },
    { 0x5e14c233, 95 },
    { 0xc904c237, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd954c23b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de4c23e, 93 },
    { 0xa474c23f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714c244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4534c247, 33 },
    { 0xa274c248, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d4c24b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d4c24f, 90 },
    { 0, 0 },
    { 0xb4c251, 90 },
    { 0, 0 },
    { 0xa1d4c253, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04c256, 23 },
    { 0xc524c257, 90 },
    { 0x4cc4c258, 75 },
    { 0xd4c257, 90 },
    { 0xaea4c256, 33 },
    { 0, 0 },
    { 0x5624c25c, 90 },
    { 0xe544c25d, 33 },
    { 0x6244c25e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf44c267, 33 },
    { 0, 0 },
    { 0x8ce4c269, 76 },
    { 0, 0 },
    { 0x2af4c26b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb774c277, 34 },
    { 0xda54c278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec04c27f, 90 },
    { 0, 0 },
    { 0xc324c281, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea44c289, 98 },
    { 0x73f4c28a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba4c28f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe474c293, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3004c296, 33 },
    { 0x5a54c296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04c29f, 93 },
    { 0, 0 },
    { 0x8414c2a1, 61 },
    { 0xba84c2a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494c2a7, 38 },
    { 0, 0 },
    { 0x70f4c2a9, 46 },
    { 0x6174c2a9, 99 },
    { 0, 0 },
    { 0x8de4c2ac, 33 },
    { 0, 0 },
    { 0x7a14c2ae, 33 },
    { 0, 0 },
    { 0xc184c2b0, 90 },
    { 0, 0 },
    { 0x4054c2b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5864c2bb, 95 },
    { 0x6c84c2bc, 76 },
    { 0, 0 },
    { 0xbc04c2be, 77 },
    { 0x9784c2bf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf04c2c5, 100 },
    { 0x5c64c2c5, 23 },
    { 0xb7e4c2c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2544c2cb, 93 },
    { 0xb6a4c2cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd534c2cf, 66 },
    { 0x5d04c2d0, 61 },
    { 0, 0 },
    { 0x8f24c2d2, 90 },
    { 0x53f4c2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9974c2d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x944c2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4c2dc, 23 },
    { 0x2e34c2dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8634c2e1, 23 },
    { 0, 0 },
    { 0x3674c2e3, 34 },
    { 0x3794c2e4, 33 },
    { 0x9f54c2e3, 84 },
    { 0xce4c2e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb4c2e9, 23 },
    { 0xc974c2ea, 34 },
    { 0x9944c2eb, 77 },
    { 0xcac4c2ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334c2f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf4c2f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d4c2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c4c2fe, 61 },
    { 0x9794c2ff, 61 },
    { 0xcef4c300, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e94c305, 90 },
    { 0x5804c306, 98 },
    { 0x2904c307, 55 },
    { 0, 0 },
    { 0xcb24c309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c84c311, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f94c315, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa084c31c, 95 },
    { 0xc7d4c31d, 90 },
    { 0x9394c31e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c4c321, 68 },
    { 0x6b44c322, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc4c326, 40 },
    { 0xcf44c326, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6514c32a, 34 },
    { 0x2d14c32a, 76 },
    { 0xe484c32c, 33 },
    { 0x9cc4c32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5724c331, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e4c334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3414c337, 33 },
    { 0x1814c338, 78 },
    { 0x2594c338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc54c344, 90 },
    { 0xbc34c345, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4c348, 76 },
    { 0x69d4c349, 61 },
    { 0xec64c34a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd274c34f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e4c359, 55 },
    { 0, 0 },
    { 0x6ae4c35b, 95 },
    { 0, 0 },
    { 0x48b4c35d, 90 },
    { 0x6964c35e, 90 },
    { 0x9234c35e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574c364, 100 },
    { 0, 0 },
    { 0xd4f4c366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc14c36b, 98 },
    { 0xf714c36c, 61 },
    { 0x4e44c36c, 90 },
    { 0, 0 },
    { 0xe8e4c36f, 84 },
    { 0xb174c370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304c377, 52 },
    { 0x1214c378, 34 },
    { 0x1954c378, 76 },
    { 0xd044c37a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd294c37d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb804c381, 33 },
    { 0xf534c382, 61 },
    { 0, 0 },
    { 0x92e4c384, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a4c387, 43 },
    { 0, 0 },
    { 0x2ac4c389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c4c38d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb4c391, 46 },
    { 0x23e4c392, 75 },
    { 0x3854c393, 84 },
    { 0, 0 },
    { 0x9ea4c395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe4c399, 93 },
    { 0, 0 },
    { 0x1184c39b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e14c3a6, 43 },
    { 0, 0 },
    { 0xe5f4c3a8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf44c3ae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb4c3b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d74c3b4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b4c3b8, 33 },
    { 0x61b4c3b8, 75 },
    { 0x5be4c3ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5444c3bf, 98 },
    { 0xdbf4c3c0, 33 },
    { 0xf344c3bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7864c3c4, 61 },
    { 0x674c3c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a34c3cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3974c3cf, 55 },
    { 0, 0 },
    { 0xe444c3d1, 36 },
    { 0, 0 },
    { 0x8294c3d3, 61 },
    { 0x1054c3d4, 90 },
    { 0, 0 },
    { 0x9814c3d6, 91 },
    { 0, 0 },
    { 0x1b14c3d8, 76 },
    { 0x7804c3d8, 46 },
    { 0x86f4c3da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9474c3e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5264c3ed, 95 },
    { 0xe654c3ed, 95 },
    { 0xfa54c3ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab4c3f6, 90 },
    { 0, 0 },
    { 0x54c4c3f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd74c3fc, 90 },
    { 0, 0 },
    { 0x3d94c3fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe764c403, 93 },
    { 0xe7e4c404, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4324c408, 91 },
    { 0xc434c409, 23 },
    { 0xf584c40a, 23 },
    { 0, 0 },
    { 0x3c24c40c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4444c411, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e64c414, 36 },
    { 0xc5e4c414, 33 },
    { 0x82c4c416, 90 },
    { 0x9af4c415, 95 },
    { 0, 0 },
    { 0x6f54c419, 87 },
    { 0, 0 },
    { 0x2054c41b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b4c41e, 78 },
    { 0x2e4c41f, 23 },
    { 0, 0 },
    { 0xc024c421, 76 },
    { 0xc014c422, 33 },
    { 0, 0 },
    { 0x2e4c424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe4c42a, 66 },
    { 0x4c74c42b, 34 },
    { 0xc8e4c42a, 88 },
    { 0x3b44c42b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf4c431, 87 },
    { 0, 0 },
    { 0x2154c433, 34 },
    { 0xb764c433, 90 },
    { 0, 0 },
    { 0x2be4c436, 61 },
    { 0, 0 },
    { 0xd2b4c438, 92 },
    { 0x1554c439, 76 },
    { 0xd994c43a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf004c43d, 61 },
    { 0x2674c43e, 61 },
    { 0x6984c43f, 91 },
    { 0, 0 },
    { 0x7774c441, 98 },
    { 0x4f44c441, 95 },
    { 0, 0 },
    { 0x8634c444, 46 },
    { 0, 0 },
    { 0xac4c446, 78 },
    { 0xf8a4c447, 98 },
    { 0x7cf4c448, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd684c44e, 84 },
    { 0xac4c44e, 23 },
    { 0, 0 },
    { 0xda04c451, 85 },
    { 0, 0 },
    { 0x3ab4c453, 76 },
    { 0x17f4c454, 98 },
    { 0xddf4c454, 98 },
    { 0xf744c456, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b4c459, 43 },
    { 0x50a4c459, 43 },
    { 0, 0 },
    { 0x3c64c45c, 46 },
    { 0x65c4c45d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f4c464, 85 },
    { 0, 0 },
    { 0xfa94c466, 98 },
    { 0x53f4c466, 34 },
    { 0x6c84c466, 98 },
    { 0x7be4c469, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb454c46f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8384c479, 90 },
    { 0x6804c479, 93 },
    { 0x4d74c47a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3904c482, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba4c485, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f4c48c, 55 },
    { 0, 0 },
    { 0xfd04c48e, 90 },
    { 0x6754c48e, 75 },
    { 0xc6c4c48f, 90 },
    { 0xa9a4c491, 23 },
    { 0, 0 },
    { 0xda14c493, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc04c498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd4c49c, 61 },
    { 0x2e54c49d, 34 },
    { 0, 0 },
    { 0x8d34c49f, 33 },
    { 0, 0 },
    { 0x9074c4a1, 61 },
    { 0, 0 },
    { 0xd244c4a3, 61 },
    { 0xd204c4a4, 17 },
    { 0, 0 },
    { 0xd6f4c4a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea74c4ab, 98 },
    { 0x1ec4c4ac, 53 },
    { 0, 0 },
    { 0x8614c4ae, 55 },
    { 0, 0 },
    { 0x9f24c4b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3514c4b5, 91 },
    { 0x1d34c4b6, 80 },
    { 0x64f4c4b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde4c4bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b04c4bf, 61 },
    { 0xb84c4c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614c4c6, 55 },
    { 0x2114c4c7, 61 },
    { 0x49e4c4c6, 61 },
    { 0x4f54c4c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e4c4d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be4c4d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b4c4dd, 43 },
    { 0xbc4c4de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c84c4e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x804c4e5, 90 },
    { 0x2664c4e6, 33 },
    { 0xd2e4c4e7, 61 },
    { 0xfce4c4e8, 93 },
    { 0x9fc4c4e8, 85 },
    { 0, 0 },
    { 0xc964c4eb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c4c4ef, 90 },
    { 0xc404c4ef, 61 },
    { 0x4bc4c4f1, 100 },
    { 0x9ca4c4ef, 77 },
    { 0x3184c4ef, 95 },
    { 0xb8a4c4f4, 33 },
    { 0, 0 },
    { 0xf0c4c4f6, 76 },
    { 0x4ba4c4f6, 100 },
    { 0, 0 },
    { 0xf544c4f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c4fd, 88 },
    { 0, 0 },
    { 0xe274c4ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca4c505, 98 },
    { 0x7b74c506, 84 },
    { 0x324c506, 95 },
    { 0x7914c506, 61 },
    { 0xccc4c505, 95 },
    { 0x6c84c508, 98 },
    { 0, 0 },
    { 0xb3a4c50c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1274c50f, 34 },
    { 0x9e64c510, 23 },
    { 0xd6a4c511, 46 },
    { 0xfae4c510, 23 },
    { 0x63c4c513, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf754c519, 76 },
    { 0xd1a4c519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df4c51f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac24c524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da4c528, 61 },
    { 0, 0 },
    { 0x3f84c52a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea4c52d, 61 },
    { 0, 0 },
    { 0x1f24c52f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54c532, 33 },
    { 0x8a14c533, 98 },
    { 0xf0b4c534, 23 },
    { 0x24a4c535, 61 },
    { 0x2354c536, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab04c53a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c4c53d, 90 },
    { 0, 0 },
    { 0x7bd4c53f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b4c542, 23 },
    { 0x4814c543, 95 },
    { 0, 0 },
    { 0x9854c545, 98 },
    { 0x3ea4c546, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd34c54b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa434c550, 61 },
    { 0, 0 },
    { 0x2114c552, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd34c55b, 95 },
    { 0x2f44c55c, 85 },
    { 0xbd44c55d, 40 },
    { 0x9cb4c55d, 61 },
    { 0xdba4c55d, 52 },
    { 0x4364c55e, 93 },
    { 0x624c561, 76 },
    { 0xff54c55d, 95 },
    { 0xac94c55b, 23 },
    { 0xfa34c564, 95 },
    { 0x2d4c560, 90 },
    { 0x8294c55c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf4c56b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2394c56e, 55 },
    { 0xa24c56f, 33 },
    { 0x3534c56e, 98 },
    { 0x5ef4c56f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1054c574, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a74c579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d4c57c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb784c580, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c4c583, 61 },
    { 0, 0 },
    { 0x94e4c585, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe404c593, 85 },
    { 0xa504c594, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e4c597, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4154c59c, 33 },
    { 0x8d24c59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3764c5a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f4c5a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c04c5a8, 59 },
    { 0x1774c5a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a14c5ac, 96 },
    { 0x68f4c5ad, 34 },
    { 0xe844c5ad, 33 },
    { 0xc654c5ac, 33 },
    { 0x1dc4c5b0, 48 },
    { 0xc974c5b1, 76 },
    { 0x7224c5af, 99 },
    { 0xcd54c5b3, 93 },
    { 0x57f4c5b4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xac74c5b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604c5bb, 55 },
    { 0, 0 },
    { 0x5584c5bd, 76 },
    { 0, 0 },
    { 0x20c4c5bf, 34 },
    { 0xabd4c5bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef04c5c3, 93 },
    { 0x6f74c5c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6704c5cd, 66 },
    { 0xb474c5cd, 93 },
    { 0x81f4c5cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7534c5d2, 95 },
    { 0x2c14c5d2, 76 },
    { 0xeac4c5d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d94c5d8, 101 },
    { 0, 0 },
    { 0xe8e4c5da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2614c5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144c5e3, 46 },
    { 0, 0 },
    { 0xa184c5e5, 66 },
    { 0x6094c5e6, 90 },
    { 0, 0 },
    { 0x1d64c5e8, 46 },
    { 0x9164c5e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34c5ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a74c5f3, 33 },
    { 0xe784c5f3, 65 },
    { 0, 0 },
    { 0x9f94c5f6, 95 },
    { 0xb8b4c5f7, 55 },
    { 0x7c74c5f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d94c5fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14c601, 23 },
    { 0x5b34c602, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5874c606, 98 },
    { 0x6d24c607, 98 },
    { 0x4194c608, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff4c60c, 61 },
    { 0xfa44c60d, 61 },
    { 0xac74c60e, 46 },
    { 0xe434c60f, 45 },
    { 0x4314c60e, 61 },
    { 0x7574c60e, 88 },
    { 0, 0 },
    { 0x75e4c613, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b4c616, 33 },
    { 0xe854c617, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe384c61b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd04c620, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2174c62a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b4c62d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae04c635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde14c63d, 61 },
    { 0x1e54c63e, 94 },
    { 0xefc4c63f, 43 },
    { 0xb254c63f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a54c645, 91 },
    { 0x5c94c646, 90 },
    { 0xd9a4c647, 90 },
    { 0x814c647, 90 },
    { 0xa154c647, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2054c64e, 23 },
    { 0x58c4c64f, 93 },
    { 0x7af4c64e, 23 },
    { 0xab84c651, 46 },
    { 0, 0 },
    { 0xc524c653, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c4c659, 95 },
    { 0x9814c65a, 33 },
    { 0x1434c65b, 23 },
    { 0xbb54c659, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4c663, 90 },
    { 0, 0 },
    { 0x9174c665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce74c668, 33 },
    { 0, 0 },
    { 0x5a04c66a, 34 },
    { 0xc644c66b, 67 },
    { 0x4e74c66c, 60 },
    { 0, 0 },
    { 0xedb4c66e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb4c672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa344c677, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a4c67a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44c67e, 61 },
    { 0, 0 },
    { 0x1a94c680, 33 },
    { 0x8784c681, 95 },
    { 0, 0 },
    { 0x6544c683, 93 },
    { 0xcd34c683, 33 },
    { 0x5754c685, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f94c689, 55 },
    { 0, 0 },
    { 0x224c68b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb604c694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d4c697, 85 },
    { 0, 0 },
    { 0x3a94c699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa44c69e, 43 },
    { 0x5774c69f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd4c6a6, 40 },
    { 0x8b34c6a7, 99 },
    { 0x6594c6a7, 23 },
    { 0xd024c6a9, 61 },
    { 0xa554c6a9, 98 },
    { 0xa524c6ab, 52 },
    { 0xa824c6ac, 68 },
    { 0x6734c6ac, 91 },
    { 0, 0 },
    { 0xceb4c6af, 61 },
    { 0x40d4c6b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x364c6b3, 95 },
    { 0xae34c6b4, 90 },
    { 0x4824c6b5, 90 },
    { 0x38e4c6b6, 55 },
    { 0x5424c6b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2674c6ba, 90 },
    { 0x6c14c6bb, 66 },
    { 0x9194c6bb, 55 },
    { 0, 0 },
    { 0x1a14c6be, 95 },
    { 0x8d04c6be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c6c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb514c6c5, 33 },
    { 0, 0 },
    { 0x1f74c6c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7824c6cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c4c6ce, 69 },
    { 0x6874c6cf, 55 },
    { 0x37e4c6cf, 33 },
    { 0x55d4c6d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4c6d5, 46 },
    { 0x39b4c6d6, 90 },
    { 0, 0 },
    { 0xc4b4c6d8, 95 },
    { 0x88a4c6d8, 91 },
    { 0xa434c6d8, 100 },
    { 0x3244c6d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb094c6e0, 100 },
    { 0, 0 },
    { 0xfa44c6e2, 61 },
    { 0xcc24c6e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a64c6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd4c6ec, 46 },
    { 0x4bc4c6ec, 76 },
    { 0, 0 },
    { 0x55b4c6ef, 61 },
    { 0xfa94c6f0, 87 },
    { 0x6894c6f1, 68 },
    { 0xe014c6f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8434c6fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde4c6fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14c701, 90 },
    { 0, 0 },
    { 0x3754c703, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e4c708, 66 },
    { 0x7124c709, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664c70e, 98 },
    { 0, 0 },
    { 0xa84c710, 36 },
    { 0, 0 },
    { 0x7114c712, 61 },
    { 0, 0 },
    { 0xaf74c714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd4c71a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3354c71e, 33 },
    { 0x1584c71f, 33 },
    { 0, 0 },
    { 0xb4e4c721, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d4c725, 33 },
    { 0xad74c725, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf694c72a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c4c72d, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e4c733, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd724c739, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8804c73d, 81 },
    { 0x3ec4c73d, 98 },
    { 0x1054c73f, 61 },
    { 0x5ec4c73f, 61 },
    { 0x5014c73f, 91 },
    { 0, 0 },
    { 0x6094c743, 33 },
    { 0x5e54c744, 61 },
    { 0xbaa4c744, 100 },
    { 0x34c745, 91 },
    { 0, 0 },
    { 0xdb4c748, 76 },
    { 0xe754c749, 87 },
    { 0xc104c74a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a94c755, 61 },
    { 0x4b24c756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3344c75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9954c763, 33 },
    { 0x5504c764, 44 },
    { 0xef64c763, 98 },
    { 0x7a84c766, 55 },
    { 0x8bc4c766, 33 },
    { 0xed84c763, 23 },
    { 0xedb4c768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4c76c, 96 },
    { 0, 0 },
    { 0x2534c76e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b4c773, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd4c776, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1914c779, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb484c77d, 66 },
    { 0x63d4c77d, 95 },
    { 0x2864c77f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a4c78a, 78 },
    { 0xa5c4c78b, 95 },
    { 0xbe54c78b, 76 },
    { 0xfa4c78d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e4c791, 33 },
    { 0x1d54c792, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a4c797, 90 },
    { 0x7da4c798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf4c7a1, 53 },
    { 0xe8e4c7a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad4c7ab, 77 },
    { 0x58d4c7ab, 98 },
    { 0, 0 },
    { 0x1384c7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24c7b3, 88 },
    { 0x5c4c7b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f84c7b7, 90 },
    { 0x1234c7b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d4c7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb4c7bf, 23 },
    { 0x4524c7c0, 67 },
    { 0, 0 },
    { 0x7344c7c2, 46 },
    { 0xaa24c7c3, 68 },
    { 0xcff4c7c2, 90 },
    { 0x1ff4c7c3, 61 },
    { 0x4cc4c7c3, 43 },
    { 0x6e74c7c7, 61 },
    { 0xd84c7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e4c7cb, 85 },
    { 0x92b4c7cc, 23 },
    { 0x1314c7cd, 75 },
    { 0x24a4c7cd, 98 },
    { 0x5b14c7cf, 34 },
    { 0, 0 },
    { 0xa1b4c7d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a14c7d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c7d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494c7dd, 23 },
    { 0, 0 },
    { 0x74c7df, 76 },
    { 0x7434c7e0, 76 },
    { 0, 0 },
    { 0x5234c7e2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff4c7e5, 66 },
    { 0xbb44c7e5, 98 },
    { 0xe704c7e7, 76 },
    { 0x6f24c7e8, 90 },
    { 0x1484c7e9, 61 },
    { 0x2864c7ea, 90 },
    { 0x1884c7e9, 34 },
    { 0x7954c7ec, 84 },
    { 0x4164c7e8, 67 },
    { 0x7c44c7ed, 33 },
    { 0xc3e4c7ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb4c7f4, 61 },
    { 0, 0 },
    { 0xe274c7f6, 95 },
    { 0x4dd4c7f7, 33 },
    { 0, 0 },
    { 0xdf44c7f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5224c7fc, 55 },
    { 0xfe34c7fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7974c80c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774c80f, 33 },
    { 0x7934c810, 76 },
    { 0xab64c80f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5054c818, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5714c81c, 67 },
    { 0x2b64c81d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f44c821, 33 },
    { 0x15b4c822, 81 },
    { 0xf74c822, 33 },
    { 0x3024c822, 61 },
    { 0x9e34c821, 77 },
    { 0x59b4c826, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8584c82a, 23 },
    { 0x15e4c82b, 23 },
    { 0x4e84c82c, 33 },
    { 0x6154c82d, 61 },
    { 0x2be4c82e, 33 },
    { 0xa464c82d, 76 },
    { 0xc294c82b, 61 },
    { 0x5834c831, 74 },
    { 0xfea4c82e, 55 },
    { 0x61c4c833, 68 },
    { 0x53b4c834, 85 },
    { 0xac84c833, 61 },
    { 0xa894c82b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb4c83b, 99 },
    { 0, 0 },
    { 0xb7b4c83d, 90 },
    { 0xfcf4c83d, 78 },
    { 0xc074c83e, 95 },
    { 0xd784c840, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134c845, 61 },
    { 0, 0 },
    { 0xaea4c847, 90 },
    { 0, 0 },
    { 0xeca4c849, 95 },
    { 0x5d84c849, 76 },
    { 0xe484c84b, 56 },
    { 0xfeb4c84c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d4c852, 61 },
    { 0xb614c853, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc004c85a, 98 },
    { 0xf0a4c85b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4c85e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a4c861, 87 },
    { 0xc354c862, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d4c865, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c872, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5594c875, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc14c87c, 43 },
    { 0xd6e4c87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed4c880, 61 },
    { 0x2de4c881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4744c886, 33 },
    { 0, 0 },
    { 0x5f34c888, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb154c88f, 90 },
    { 0, 0 },
    { 0xbee4c891, 33 },
    { 0x2ac4c892, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4c8a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc4c8a5, 61 },
    { 0xd54c8a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb54c8aa, 91 },
    { 0, 0 },
    { 0xa714c8ac, 17 },
    { 0x7d44c8ad, 98 },
    { 0xf014c8ae, 90 },
    { 0x7384c8ae, 90 },
    { 0x7964c8af, 90 },
    { 0xf454c8b1, 76 },
    { 0x5624c8b2, 85 },
    { 0x5d34c8b3, 33 },
    { 0xac74c8b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb424c8c2, 43 },
    { 0x2774c8c3, 61 },
    { 0x2d14c8c4, 87 },
    { 0, 0 },
    { 0x3fb4c8c6, 90 },
    { 0xa1c4c8c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac4c8cb, 33 },
    { 0, 0 },
    { 0x65c4c8cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4c8d0, 23 },
    { 0xb364c8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb34c8d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2804c8df, 76 },
    { 0x2b34c8df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee4c8e3, 98 },
    { 0, 0 },
    { 0x2f74c8e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6274c8e9, 90 },
    { 0xb8f4c8ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2974c8ed, 99 },
    { 0, 0 },
    { 0xcc44c8ef, 88 },
    { 0xaa74c8ef, 61 },
    { 0xce14c8f1, 90 },
    { 0x274c8f2, 34 },
    { 0x374c8f3, 87 },
    { 0x4b94c8f4, 17 },
    { 0xa784c8f5, 90 },
    { 0x90e4c8f5, 90 },
    { 0x2064c8f7, 96 },
    { 0, 0 },
    { 0x7204c8f9, 23 },
    { 0x13d4c8fa, 61 },
    { 0x1f84c8fb, 90 },
    { 0xe064c8f9, 99 },
    { 0x93f4c8fd, 33 },
    { 0x8874c8fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1644c905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c4c908, 46 },
    { 0, 0 },
    { 0x4944c90a, 52 },
    { 0, 0 },
    { 0x2734c90c, 90 },
    { 0, 0 },
    { 0xf04c90e, 90 },
    { 0, 0 },
    { 0xbbd4c910, 98 },
    { 0x2fb4c911, 90 },
    { 0, 0 },
    { 0x7834c913, 55 },
    { 0xd994c914, 23 },
    { 0x8d94c915, 90 },
    { 0, 0 },
    { 0x94c917, 80 },
    { 0, 0 },
    { 0xc7d4c919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6854c91c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac14c91f, 33 },
    { 0xef34c91f, 64 },
    { 0xd1f4c921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf734c926, 90 },
    { 0x7374c926, 40 },
    { 0x8ff4c928, 85 },
    { 0x244c928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8484c92c, 33 },
    { 0x63e4c92d, 95 },
    { 0x2354c92e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a74c933, 33 },
    { 0xc094c934, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf4c937, 33 },
    { 0, 0 },
    { 0xa934c939, 52 },
    { 0x50b4c939, 61 },
    { 0x8864c93b, 33 },
    { 0x63e4c93c, 55 },
    { 0xb7e4c93d, 95 },
    { 0xaa74c93a, 95 },
    { 0xf324c93b, 90 },
    { 0x7464c93b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c944, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe554c947, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c94c952, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3134c955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f74c95b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc4c961, 61 },
    { 0x41d4c962, 61 },
    { 0xa134c961, 90 },
    { 0x95a4c964, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1764c967, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee94c96f, 34 },
    { 0xac94c970, 23 },
    { 0x3a64c971, 90 },
    { 0xfd04c972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d24c97a, 76 },
    { 0, 0 },
    { 0x2684c97c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb244c97f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f34c986, 68 },
    { 0x20c4c986, 95 },
    { 0x6794c988, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e54c98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce74c98f, 66 },
    { 0, 0 },
    { 0x5ec4c991, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4374c998, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a44c99b, 90 },
    { 0, 0 },
    { 0xd14c99d, 87 },
    { 0, 0 },
    { 0x44d4c99f, 55 },
    { 0xc154c9a0, 66 },
    { 0x1514c9a1, 95 },
    { 0x9634c9a1, 97 },
    { 0x2b84c9a1, 23 },
    { 0x2864c9a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f4c9a7, 34 },
    { 0x65a4c9a8, 68 },
    { 0x9d34c9a9, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe304c9ad, 93 },
    { 0, 0 },
    { 0x6304c9af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a04c9b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3314c9b7, 61 },
    { 0x8d04c9b7, 38 },
    { 0x5f64c9b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba4c9be, 55 },
    { 0x28e4c9bf, 91 },
    { 0, 0 },
    { 0xdff4c9c1, 98 },
    { 0, 0 },
    { 0xa3c4c9c3, 43 },
    { 0xd2e4c9c4, 23 },
    { 0x7294c9c4, 90 },
    { 0x59e4c9c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec4c9c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed34c9cc, 23 },
    { 0x1c54c9cd, 99 },
    { 0x1404c9cd, 95 },
    { 0x74b4c9cd, 93 },
    { 0x9664c9d0, 98 },
    { 0xc0e4c9d1, 100 },
    { 0x7e74c9d2, 78 },
    { 0x1cf4c9d2, 95 },
    { 0xf4b4c9d4, 46 },
    { 0x21a4c9d2, 90 },
    { 0xe5b4c9d6, 61 },
    { 0x4474c9d6, 95 },
    { 0xd5b4c9d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd534c9dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb324c9e4, 91 },
    { 0x8ae4c9e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c9e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d64c9ed, 61 },
    { 0x4954c9ee, 33 },
    { 0xb7a4c9ef, 78 },
    { 0x8884c9ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c14c9f5, 33 },
    { 0x6274c9f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e4c9f9, 43 },
    { 0xe784c9fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad94ca02, 90 },
    { 0x4634ca03, 34 },
    { 0, 0 },
    { 0x3834ca05, 95 },
    { 0xbf44ca06, 61 },
    { 0x1fb4ca07, 46 },
    { 0x8564ca05, 98 },
    { 0, 0 },
    { 0xbf64ca0a, 61 },
    { 0xbe64ca0b, 88 },
    { 0xa854ca0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14ca11, 95 },
    { 0x1964ca11, 61 },
    { 0xe334ca13, 55 },
    { 0x5b4ca13, 33 },
    { 0x7964ca11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1664ca21, 98 },
    { 0x63e4ca21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4ca29, 90 },
    { 0x2164ca2a, 76 },
    { 0xa664ca2b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa24ca2e, 46 },
    { 0x8254ca2f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6434ca3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e4ca40, 88 },
    { 0xaa84ca41, 66 },
    { 0x33e4ca42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb304ca49, 98 },
    { 0, 0 },
    { 0xa14ca4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4ca53, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b4ca59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f4ca5d, 61 },
    { 0x77b4ca5e, 78 },
    { 0x53c4ca5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b4ca62, 76 },
    { 0, 0 },
    { 0x5804ca64, 66 },
    { 0x2024ca65, 92 },
    { 0xdfb4ca65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4ca6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7534ca72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8554ca76, 23 },
    { 0, 0 },
    { 0x8aa4ca78, 95 },
    { 0xae84ca78, 17 },
    { 0x50a4ca7a, 75 },
    { 0x8464ca7b, 55 },
    { 0, 0 },
    { 0x2e04ca7d, 74 },
    { 0xc424ca7e, 61 },
    { 0, 0 },
    { 0x88d4ca80, 34 },
    { 0x4b84ca80, 61 },
    { 0x6a54ca80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4ca8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed4ca8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb4ca95, 84 },
    { 0, 0 },
    { 0xf274ca97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1134ca9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8844caa2, 23 },
    { 0x25d4caa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a04caa7, 90 },
    { 0, 0 },
    { 0xd6d4caa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e04caad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d64cab1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d34cab4, 33 },
    { 0x3834cab5, 61 },
    { 0xfda4cab6, 78 },
    { 0, 0 },
    { 0x1764cab8, 23 },
    { 0, 0 },
    { 0x194caba, 98 },
    { 0x9844caba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb134cabe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d4cac1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf74cac6, 95 },
    { 0x6464cac7, 66 },
    { 0x9f24cac7, 95 },
    { 0x8764cac6, 99 },
    { 0, 0 },
    { 0x8854cacb, 23 },
    { 0x5724cacc, 61 },
    { 0xc04cacc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3724cad2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9974cad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb964cadc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6214cadf, 65 },
    { 0x65d4cae0, 76 },
    { 0x4fd4cae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3934caef, 95 },
    { 0xb444caf0, 95 },
    { 0x2b24caef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf44caf6, 99 },
    { 0xc354caf7, 37 },
    { 0, 0 },
    { 0xfe64caf9, 91 },
    { 0xf914cafa, 68 },
    { 0xae84cafb, 33 },
    { 0, 0 },
    { 0xf6b4cafd, 91 },
    { 0xb234cafe, 43 },
    { 0xa194caff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5974cb02, 99 },
    { 0xbf64cb03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb394cb07, 95 },
    { 0x8014cb08, 33 },
    { 0xc754cb08, 61 },
    { 0x4224cb07, 84 },
    { 0, 0 },
    { 0x34d4cb0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324cb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf014cb16, 23 },
    { 0x7414cb17, 33 },
    { 0xdca4cb18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd4cb1e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf34cb26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc4cb29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f04cb32, 55 },
    { 0, 0 },
    { 0xaae4cb34, 95 },
    { 0, 0 },
    { 0xb104cb36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94cb44, 99 },
    { 0xa1e4cb44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d4cb49, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb54cb57, 90 },
    { 0xb574cb58, 61 },
    { 0x38d4cb57, 33 },
    { 0xb954cb59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d34cb5d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa904cb67, 43 },
    { 0, 0 },
    { 0x9a84cb69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x394cb6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b4cb70, 91 },
    { 0, 0 },
    { 0x66c4cb72, 33 },
    { 0, 0 },
    { 0x384cb74, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d4cb7a, 61 },
    { 0, 0 },
    { 0x9ec4cb7c, 98 },
    { 0x4ea4cb7d, 98 },
    { 0, 0 },
    { 0x1a24cb7f, 90 },
    { 0x22d4cb80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec64cb84, 76 },
    { 0x9c24cb85, 90 },
    { 0xacf4cb86, 34 },
    { 0xc4c4cb84, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9834cb91, 95 },
    { 0x1354cb92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb894cb95, 98 },
    { 0, 0 },
    { 0xc8a4cb97, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb4cb9c, 95 },
    { 0, 0 },
    { 0x56d4cb9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9774cba2, 97 },
    { 0, 0 },
    { 0xf984cba4, 33 },
    { 0xba24cba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d4cbae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7484cbb1, 34 },
    { 0x8f94cbb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4cbb5, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb34cbba, 90 },
    { 0x9f94cbba, 98 },
    { 0xb574cbbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5454cbc2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2484cbc9, 95 },
    { 0xff94cbc9, 33 },
    { 0, 0 },
    { 0xc134cbcc, 95 },
    { 0, 0 },
    { 0xbac4cbce, 66 },
    { 0, 0 },
    { 0xf64cbd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1324cbd3, 85 },
    { 0, 0 },
    { 0xe84cbd5, 98 },
    { 0x1bf4cbd6, 61 },
    { 0xc234cbd7, 90 },
    { 0x40c4cbd6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7804cbdc, 87 },
    { 0x49f4cbdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d4cbe1, 91 },
    { 0x5a84cbe1, 33 },
    { 0x8144cbe3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a94cbe6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f14cbeb, 33 },
    { 0xcc14cbec, 33 },
    { 0, 0 },
    { 0x8164cbee, 100 },
    { 0x5754cbef, 98 },
    { 0x8904cbf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee4cbf3, 52 },
    { 0, 0 },
    { 0xafd4cbf5, 34 },
    { 0, 0 },
    { 0x7304cbf7, 61 },
    { 0x8424cbf8, 55 },
    { 0x64c4cbf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df4cbfc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1164cbff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8554cc06, 90 },
    { 0x92f4cc07, 33 },
    { 0x48f4cc08, 90 },
    { 0x6334cc08, 67 },
    { 0x3fc4cc0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf4cc0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d4cc13, 64 },
    { 0x9d34cc14, 93 },
    { 0xcf64cc15, 33 },
    { 0x6764cc16, 23 },
    { 0x91b4cc17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9494cc1e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc44cc21, 90 },
    { 0xc114cc21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1464cc25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe4cc29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284cc2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5844cc35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9784cc38, 61 },
    { 0x1854cc39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc394cc3d, 98 },
    { 0x4c44cc3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b4cc41, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe494cc4b, 33 },
    { 0x43e4cc4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f4cc51, 96 },
    { 0x3794cc52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b64cc56, 23 },
    { 0, 0 },
    { 0xf864cc58, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f4cc66, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc194cc6e, 76 },
    { 0x15a4cc6e, 90 },
    { 0xee94cc70, 34 },
    { 0, 0 },
    { 0xbe74cc72, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b4cc75, 85 },
    { 0xe404cc76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f24cc79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf164cc7f, 95 },
    { 0, 0 },
    { 0x1d44cc81, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb4cc88, 95 },
    { 0x7004cc88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a4cc8e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e4cc92, 98 },
    { 0xd114cc93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe424cc98, 100 },
    { 0xd3c4cc99, 75 },
    { 0x6fc4cc99, 85 },
    { 0x6ac4cc99, 61 },
    { 0xa994cc9c, 34 },
    { 0x1364cc99, 98 },
    { 0, 0 },
    { 0xa154cc9f, 98 },
    { 0, 0 },
    { 0xd9a4cca1, 87 },
    { 0x7834cca1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc534ccb4, 66 },
    { 0x9624ccb5, 95 },
    { 0x8cf4ccb6, 90 },
    { 0, 0 },
    { 0x70d4ccb8, 76 },
    { 0x7a74ccb9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de4ccbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf4ccc0, 98 },
    { 0x494ccc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054ccc6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e4cccd, 100 },
    { 0, 0 },
    { 0x6c04cccf, 33 },
    { 0xaed4ccd0, 90 },
    { 0x3464ccd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb384ccd5, 61 },
    { 0, 0 },
    { 0x6a94ccd7, 52 },
    { 0x9254ccd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d4cce5, 68 },
    { 0x4c94cce5, 33 },
    { 0x6814cce5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ccea, 88 },
    { 0x9af4cceb, 76 },
    { 0x6cf4ccec, 61 },
    { 0xe334cced, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d4ccf2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d4ccfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d4cd01, 43 },
    { 0x5ad4cd02, 98 },
    { 0xa724cd02, 43 },
    { 0x3ae4cd04, 95 },
    { 0xb4c4cd05, 34 },
    { 0, 0 },
    { 0xdb94cd07, 61 },
    { 0, 0 },
    { 0x1a4cd09, 91 },
    { 0xf814cd09, 100 },
    { 0x1d84cd0b, 23 },
    { 0x4cc4cd0c, 98 },
    { 0xa44cd0c, 90 },
    { 0x2e24cd0e, 33 },
    { 0xc94cd0f, 33 },
    { 0x5fc4cd0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe524cd14, 90 },
    { 0, 0 },
    { 0x1de4cd16, 76 },
    { 0xd54cd16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd884cd1a, 91 },
    { 0, 0 },
    { 0xe884cd1c, 93 },
    { 0x3a14cd1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab24cd22, 98 },
    { 0xbc44cd22, 33 },
    { 0x42b4cd23, 93 },
    { 0x5264cd25, 95 },
    { 0, 0 },
    { 0xadc4cd27, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2374cd2a, 33 },
    { 0x7164cd2b, 33 },
    { 0, 0 },
    { 0x8a34cd2d, 87 },
    { 0xc264cd2e, 33 },
    { 0xc054cd2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf4cd35, 91 },
    { 0xd184cd36, 95 },
    { 0xb84cd36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde84cd3b, 23 },
    { 0, 0 },
    { 0xe1f4cd3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5594cd46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b4cd49, 90 },
    { 0xded4cd4a, 90 },
    { 0, 0 },
    { 0x9264cd4c, 76 },
    { 0, 0 },
    { 0xb544cd4e, 90 },
    { 0, 0 },
    { 0x7724cd50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2924cd5b, 95 },
    { 0xf584cd5c, 90 },
    { 0x93a4cd5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b4cd60, 90 },
    { 0x3064cd60, 34 },
    { 0x5c94cd62, 90 },
    { 0x9184cd62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xee34cd66, 33 },
    { 0, 0 },
    { 0xa924cd68, 87 },
    { 0xdf94cd68, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c4cd70, 33 },
    { 0x24a4cd71, 66 },
    { 0, 0 },
    { 0xfe64cd73, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d4cd78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b54cd7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf84cd80, 46 },
    { 0x4884cd81, 55 },
    { 0x4e14cd81, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a4cd8b, 17 },
    { 0x4074cd8c, 61 },
    { 0xa144cd8d, 96 },
    { 0xcce4cd8b, 55 },
    { 0xb4c4cd8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d84cd96, 90 },
    { 0x76f4cd97, 90 },
    { 0xbb54cd97, 100 },
    { 0x5404cd99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd84cd9d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e44cda1, 67 },
    { 0, 0 },
    { 0x7da4cda3, 33 },
    { 0x3834cda4, 95 },
    { 0x6c44cda5, 88 },
    { 0xe6c4cda6, 34 },
    { 0x8734cda7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c4cdad, 23 },
    { 0, 0 },
    { 0xa294cdaf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c4cdb2, 95 },
    { 0xc734cdb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5874cdb6, 23 },
    { 0xd794cdb7, 90 },
    { 0, 0 },
    { 0x23e4cdb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec4cdbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba4cdc5, 95 },
    { 0, 0 },
    { 0x9ff4cdc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84cdcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4584cdd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae24cdd8, 93 },
    { 0xce94cdd8, 40 },
    { 0xc4b4cdda, 90 },
    { 0xcc44cddb, 61 },
    { 0, 0 },
    { 0x94c4cddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd4cde1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9014cde8, 95 },
    { 0x34e4cde9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4cdee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f44cdfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb64ce00, 33 },
    { 0, 0 },
    { 0x84c4ce02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb74ce06, 93 },
    { 0x6b34ce06, 34 },
    { 0, 0 },
    { 0x8374ce09, 90 },
    { 0x9534ce0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a4ce0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf604ce14, 46 },
    { 0, 0 },
    { 0xadb4ce16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6954ce1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7244ce22, 98 },
    { 0xcb64ce23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b4ce26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc4ce29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b4ce31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654ce35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b74ce38, 46 },
    { 0xd454ce38, 61 },
    { 0x5844ce39, 98 },
    { 0x5dd4ce3b, 95 },
    { 0, 0 },
    { 0xda74ce3d, 90 },
    { 0xdcc4ce3e, 76 },
    { 0, 0 },
    { 0x60e4ce40, 93 },
    { 0xb6f4ce41, 34 },
    { 0x93b4ce41, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3134ce47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec4ce4d, 90 },
    { 0x75f4ce4e, 91 },
    { 0x3334ce4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654ce52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4ce57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea4ce5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd4ce65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf104ce6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd694ce6d, 98 },
    { 0xbde4ce6e, 95 },
    { 0, 0 },
    { 0x3f44ce70, 61 },
    { 0x23f4ce71, 61 },
    { 0, 0 },
    { 0xe064ce73, 90 },
    { 0x3894ce73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4ce78, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5954ce7d, 95 },
    { 0x9214ce7d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4ce82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2564ce85, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a4ce89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec4ce8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf714ce90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b4ce94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4444ce97, 98 },
    { 0xfb04ce98, 23 },
    { 0, 0 },
    { 0xb424ce9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa874ce9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e4cea1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2664cea5, 61 },
    { 0xeb34cea5, 93 },
    { 0, 0 },
    { 0xaa24cea8, 77 },
    { 0xdba4cea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f4ceaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294ceb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad44ceb6, 95 },
    { 0, 0 },
    { 0x744ceb8, 90 },
    { 0x92c4ceb9, 92 },
    { 0, 0 },
    { 0x8f84cebb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe864cebe, 84 },
    { 0x2f94cebf, 80 },
    { 0xd5d4cebf, 76 },
    { 0x1b24cebe, 55 },
    { 0x4e54cec1, 93 },
    { 0x414cec0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf674cec9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf094cecd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af4ced0, 52 },
    { 0xbe84ced0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3354ced4, 95 },
    { 0xb6f4ced5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704cedb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd4cee1, 95 },
    { 0x94b4cee1, 23 },
    { 0, 0 },
    { 0x32c4cee4, 76 },
    { 0, 0 },
    { 0x5c74cee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3814ceeb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3334ceef, 33 },
    { 0, 0 },
    { 0x6954cef1, 55 },
    { 0, 0 },
    { 0xde4cef3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7424cef7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd504cefa, 90 },
    { 0x10e4cefb, 98 },
    { 0x1544cefa, 61 },
    { 0xbd94cefd, 76 },
    { 0x94e4cefe, 90 },
    { 0, 0 },
    { 0x7b54cf00, 43 },
    { 0xa854cf00, 93 },
    { 0, 0 },
    { 0x41c4cf03, 91 },
    { 0x5f44cf03, 96 },
    { 0x4654cf03, 55 },
    { 0x9974cf06, 90 },
    { 0xe6c4cf07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714cf13, 76 },
    { 0, 0 },
    { 0xc654cf15, 68 },
    { 0xb404cf15, 95 },
    { 0xd234cf16, 61 },
    { 0x8114cf15, 95 },
    { 0xc794cf18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14cf1e, 88 },
    { 0x2ab4cf1e, 90 },
    { 0x1654cf20, 61 },
    { 0x5784cf21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2964cf25, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f4cf29, 90 },
    { 0, 0 },
    { 0x7c34cf2b, 61 },
    { 0x4c4cf2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e4cf2f, 55 },
    { 0xa8f4cf30, 76 },
    { 0x93c4cf31, 98 },
    { 0xe184cf32, 90 },
    { 0xdac4cf33, 90 },
    { 0x3824cf33, 91 },
    { 0x2814cf30, 95 },
    { 0x1c04cf36, 98 },
    { 0xa074cf37, 93 },
    { 0xdff4cf34, 90 },
    { 0x5dc4cf39, 95 },
    { 0x5f54cf3a, 33 },
    { 0, 0 },
    { 0xdd54cf3c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8414cf3f, 61 },
    { 0xd404cf3f, 90 },
    { 0xd024cf41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf4cf45, 33 },
    { 0xc4d4cf46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf474cf4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa684cf4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b4cf52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e4cf55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c14cf58, 90 },
    { 0, 0 },
    { 0x48a4cf5a, 90 },
    { 0xd804cf5a, 76 },
    { 0x3844cf5c, 88 },
    { 0, 0 },
    { 0xf3e4cf5e, 88 },
    { 0x4234cf5f, 87 },
    { 0, 0 },
    { 0xa224cf61, 61 },
    { 0xabc4cf62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a4cf69, 46 },
    { 0x6b4cf6a, 99 },
    { 0x7e14cf6b, 61 },
    { 0, 0 },
    { 0xb84cf6d, 34 },
    { 0, 0 },
    { 0x5654cf6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d54cf75, 76 },
    { 0, 0 },
    { 0x1364cf77, 55 },
    { 0x39f4cf78, 76 },
    { 0, 0 },
    { 0xb0a4cf7a, 17 },
    { 0xcdc4cf7b, 46 },
    { 0xea14cf7b, 52 },
    { 0xf584cf7a, 33 },
    { 0x5494cf7e, 23 },
    { 0, 0 },
    { 0x42e4cf80, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc074cf83, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4504cf87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c4cf8d, 75 },
    { 0x9b64cf8e, 66 },
    { 0, 0 },
    { 0x2674cf90, 98 },
    { 0xc224cf91, 77 },
    { 0x9694cf91, 95 },
    { 0xe824cf93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e04cf97, 33 },
    { 0x7f24cf98, 52 },
    { 0x1fb4cf99, 61 },
    { 0x2354cf98, 61 },
    { 0, 0 },
    { 0xb74cf9c, 52 },
    { 0x9514cf9d, 90 },
    { 0xf304cf9e, 98 },
    { 0x9474cf9c, 90 },
    { 0x4be4cf9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe304cfab, 95 },
    { 0, 0 },
    { 0xc5c4cfad, 33 },
    { 0x87b4cfad, 34 },
    { 0x9764cfaf, 55 },
    { 0x36b4cfb0, 61 },
    { 0x4284cfb0, 65 },
    { 0, 0 },
    { 0xb584cfb3, 55 },
    { 0, 0 },
    { 0x3d64cfb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b4cfbe, 23 },
    { 0x9604cfbf, 34 },
    { 0x5f44cfc0, 98 },
    { 0x3e74cfc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd4cfc6, 95 },
    { 0xf554cfc7, 55 },
    { 0xe154cfc8, 93 },
    { 0x6f04cfc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3234cfd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf044cfd3, 23 },
    { 0x2a84cfd3, 55 },
    { 0xa1a4cfd5, 95 },
    { 0xa084cfd5, 76 },
    { 0xa654cfd7, 55 },
    { 0x9594cfd8, 23 },
    { 0xa394cfd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3054cfde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84cfe1, 90 },
    { 0, 0 },
    { 0xc724cfe3, 46 },
    { 0xe474cfe3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9304cfe9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94cfec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc4cff1, 90 },
    { 0, 0 },
    { 0x39b4cff3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a4cff9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f64cffc, 61 },
    { 0xeaf4cffd, 65 },
    { 0xe074cffc, 76 },
    { 0x54cfff, 88 },
    { 0xa0c4cfff, 90 },
    { 0x2284d000, 95 },
    { 0x5164d000, 33 },
    { 0xb124d003, 55 },
    { 0, 0 },
    { 0x6cb4d005, 90 },
    { 0x5fd4d006, 33 },
    { 0, 0 },
    { 0x4f64d008, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe694d00b, 91 },
    { 0, 0 },
    { 0x1744d00d, 75 },
    { 0x9eb4d00d, 100 },
    { 0, 0 },
    { 0x1294d010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb184d016, 98 },
    { 0, 0 },
    { 0x28d4d018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe254d01d, 81 },
    { 0xde44d01e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284d021, 33 },
    { 0xa104d022, 67 },
    { 0, 0 },
    { 0x7a74d024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b34d02a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c14d031, 98 },
    { 0x8714d032, 36 },
    { 0, 0 },
    { 0xc5c4d034, 42 },
    { 0, 0 },
    { 0xd74d036, 90 },
    { 0xa944d037, 23 },
    { 0, 0 },
    { 0xc364d039, 76 },
    { 0x5c04d039, 23 },
    { 0, 0 },
    { 0x8214d03c, 75 },
    { 0xd134d03d, 23 },
    { 0xf9c4d03c, 23 },
    { 0, 0 },
    { 0xc654d040, 23 },
    { 0xbb94d040, 90 },
    { 0xed64d042, 61 },
    { 0x9c04d042, 76 },
    { 0x5bf4d040, 34 },
    { 0, 0 },
    { 0x1754d046, 34 },
    { 0xc264d046, 33 },
    { 0x4c74d047, 98 },
    { 0x3a4d049, 87 },
    { 0x78d4d049, 95 },
    { 0x7ca4d047, 90 },
    { 0xf454d04c, 95 },
    { 0, 0 },
    { 0xb8b4d04e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564d052, 98 },
    { 0x7954d053, 40 },
    { 0, 0 },
    { 0xade4d055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4d05b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc74d060, 91 },
    { 0, 0 },
    { 0x7a94d062, 33 },
    { 0, 0 },
    { 0x5a54d064, 61 },
    { 0x8eb4d065, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa704d068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff54d06c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb94d072, 43 },
    { 0xfec4d072, 23 },
    { 0x4ea4d074, 81 },
    { 0, 0 },
    { 0x3824d076, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6684d079, 23 },
    { 0x3ac4d07a, 33 },
    { 0xec14d07b, 100 },
    { 0xe424d07a, 95 },
    { 0, 0 },
    { 0xaa94d07e, 23 },
    { 0, 0 },
    { 0x2d74d080, 68 },
    { 0x9164d081, 40 },
    { 0, 0 },
    { 0x50f4d083, 61 },
    { 0xfe34d084, 76 },
    { 0x7384d084, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a4d08a, 61 },
    { 0x2cf4d08a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8714d092, 66 },
    { 0, 0 },
    { 0x4b4d094, 33 },
    { 0x8534d095, 95 },
    { 0x12d4d095, 85 },
    { 0xd3f4d096, 78 },
    { 0xb754d094, 23 },
    { 0x714d095, 61 },
    { 0x35d4d095, 33 },
    { 0x87a4d09b, 76 },
    { 0x1c24d09c, 98 },
    { 0x2354d09d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf324d0a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e4d0a6, 55 },
    { 0, 0 },
    { 0xae74d0a8, 93 },
    { 0x334d0a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e4d0ad, 90 },
    { 0x2704d0ad, 90 },
    { 0x6dc4d0ae, 33 },
    { 0x5894d0b0, 61 },
    { 0x4284d0b1, 55 },
    { 0xef04d0b0, 23 },
    { 0x5b44d0b3, 81 },
    { 0xfa94d0b1, 75 },
    { 0, 0 },
    { 0xea4d0b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74d0ba, 98 },
    { 0x51b4d0bb, 34 },
    { 0x9934d0bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c4d0bf, 91 },
    { 0, 0 },
    { 0x29b4d0c1, 55 },
    { 0, 0 },
    { 0x5cd4d0c3, 34 },
    { 0x244d0c4, 90 },
    { 0xe474d0c5, 55 },
    { 0x33d4d0c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a4d0ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d0cd, 55 },
    { 0x9684d0cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2704d0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf34d0d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe204d0d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7774d0dd, 98 },
    { 0, 0 },
    { 0xf744d0df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc4d0e2, 33 },
    { 0, 0 },
    { 0xc3e4d0e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44d0e7, 33 },
    { 0x3084d0e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc754d0ef, 23 },
    { 0x7584d0f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c4d0f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe4d0f9, 33 },
    { 0, 0 },
    { 0xed54d0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd084d0fe, 90 },
    { 0xc754d0ff, 61 },
    { 0, 0 },
    { 0xc8d4d101, 33 },
    { 0xc244d102, 55 },
    { 0x7b14d103, 90 },
    { 0, 0 },
    { 0xac44d105, 76 },
    { 0, 0 },
    { 0x7364d107, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e94d10f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f4d112, 90 },
    { 0x4024d113, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b4d116, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814d119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f4d121, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d4d127, 90 },
    { 0, 0 },
    { 0x354d129, 33 },
    { 0x99c4d12a, 80 },
    { 0xf394d12a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1224d131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074d136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba34d13a, 33 },
    { 0xb9d4d13b, 61 },
    { 0xd364d13b, 90 },
    { 0xfcf4d13b, 23 },
    { 0x2b4d13e, 90 },
    { 0x3064d13f, 78 },
    { 0xc334d13f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7464d144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7694d156, 23 },
    { 0xc014d157, 33 },
    { 0x8b4d157, 33 },
    { 0xc374d159, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb84d15c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a4d161, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f4d164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd454d167, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7434d16c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c4d172, 33 },
    { 0x2904d173, 61 },
    { 0x5cc4d174, 33 },
    { 0, 0 },
    { 0xb914d176, 95 },
    { 0x8534d177, 95 },
    { 0xd9d4d178, 23 },
    { 0xb8b4d178, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b44d17e, 90 },
    { 0x94d17f, 90 },
    { 0xf1e4d180, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704d184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c54d18b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa384d18e, 95 },
    { 0x73c4d18f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024d192, 76 },
    { 0x4904d193, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1254d196, 33 },
    { 0xa1d4d197, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d19b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1354d19e, 33 },
    { 0, 0 },
    { 0x88f4d1a0, 55 },
    { 0x97f4d1a1, 76 },
    { 0x6fc4d1a2, 85 },
    { 0x7f84d1a3, 98 },
    { 0x2594d1a4, 90 },
    { 0, 0 },
    { 0xd914d1a6, 76 },
    { 0x9b64d1a7, 88 },
    { 0x62e4d1a8, 96 },
    { 0x2464d1a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe794d1af, 90 },
    { 0xaa4d1af, 61 },
    { 0xc714d1b1, 76 },
    { 0x7954d1b2, 55 },
    { 0xe5a4d1b1, 43 },
    { 0x3334d1b4, 61 },
    { 0, 0 },
    { 0x6df4d1b6, 75 },
    { 0, 0 },
    { 0x1db4d1b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344d1bc, 33 },
    { 0, 0 },
    { 0x3504d1be, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x3284d1c1, 23 },
    { 0, 0 },
    { 0x1a14d1c3, 77 },
    { 0, 0 },
    { 0xed04d1c5, 33 },
    { 0, 0 },
    { 0x2c14d1c7, 100 },
    { 0x67b4d1c8, 85 },
    { 0x2064d1c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe984d1cf, 85 },
    { 0x2ca4d1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f4d1d3, 98 },
    { 0, 0 },
    { 0x95a4d1d5, 23 },
    { 0, 0 },
    { 0x8124d1d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab4d1da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e4d1e8, 23 },
    { 0x7224d1e8, 98 },
    { 0x2c94d1ea, 23 },
    { 0, 0 },
    { 0xc804d1ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a4d1f0, 61 },
    { 0, 0 },
    { 0x2024d1f2, 99 },
    { 0, 0 },
    { 0x6ef4d1f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a44d1f8, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee4d1fc, 78 },
    { 0x714d1fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7274d200, 95 },
    { 0x6d44d200, 66 },
    { 0xe064d202, 66 },
    { 0xd524d200, 23 },
    { 0x1c64d204, 99 },
    { 0x9b24d205, 95 },
    { 0xcbd4d205, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e44d20a, 61 },
    { 0xf774d20b, 100 },
    { 0x2424d20c, 76 },
    { 0x8094d20c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5704d218, 36 },
    { 0xcde4d219, 43 },
    { 0xf6b4d21a, 90 },
    { 0x6404d218, 98 },
    { 0x28f4d21c, 33 },
    { 0x3ce4d21c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7034d220, 98 },
    { 0x1a74d221, 87 },
    { 0, 0 },
    { 0x1e04d223, 95 },
    { 0xca14d224, 76 },
    { 0x5824d225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe584d228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c74d22b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f4d22e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4424d231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df4d234, 77 },
    { 0xfe54d234, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7684d23c, 90 },
    { 0xc924d23d, 87 },
    { 0x8494d23e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794d242, 93 },
    { 0xbf54d243, 90 },
    { 0x6264d244, 43 },
    { 0xc914d245, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae14d24a, 33 },
    { 0xb074d24a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb14d24e, 98 },
    { 0x78a4d24e, 23 },
    { 0, 0 },
    { 0x67c4d251, 61 },
    { 0x3194d251, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d24d257, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4d25c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e84d25f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d269, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa244d26d, 33 },
    { 0xf394d26d, 33 },
    { 0, 0 },
    { 0x1d64d270, 61 },
    { 0x51c4d270, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b4d275, 98 },
    { 0x754d276, 90 },
    { 0x9ff4d275, 95 },
    { 0, 0 },
    { 0x30f4d279, 90 },
    { 0xfa74d279, 95 },
    { 0xcaa4d27b, 61 },
    { 0x48c4d27c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da4d280, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8454d283, 95 },
    { 0, 0 },
    { 0xcb34d285, 33 },
    { 0xc184d286, 34 },
    { 0xab34d287, 33 },
    { 0xc764d288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6154d28c, 45 },
    { 0x1554d28d, 67 },
    { 0xe1a4d28d, 98 },
    { 0, 0 },
    { 0x1594d290, 100 },
    { 0x5de4d291, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4d298, 34 },
    { 0x9d24d299, 61 },
    { 0, 0 },
    { 0xd4e4d29b, 98 },
    { 0x8bd4d29c, 95 },
    { 0, 0 },
    { 0x6534d29e, 98 },
    { 0x2864d29e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c44d2a4, 33 },
    { 0, 0 },
    { 0x3ca4d2a6, 23 },
    { 0x3bb4d2a7, 55 },
    { 0, 0 },
    { 0x2c54d2a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b4d2ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc574d2b0, 17 },
    { 0, 0 },
    { 0x8fd4d2b2, 100 },
    { 0x7f94d2b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x754d2b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5994d2b9, 33 },
    { 0x86a4d2ba, 98 },
    { 0, 0 },
    { 0x2454d2bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d2bf, 61 },
    { 0x95e4d2c0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a4d2c7, 98 },
    { 0x3cd4d2c8, 101 },
    { 0x74c4d2c9, 81 },
    { 0xc394d2c8, 93 },
    { 0x49e4d2c9, 61 },
    { 0xcf24d2c7, 52 },
    { 0x7104d2cd, 93 },
    { 0, 0 },
    { 0x8f14d2cf, 81 },
    { 0x35d4d2cf, 76 },
    { 0, 0 },
    { 0x7f84d2d2, 90 },
    { 0x51d4d2d3, 46 },
    { 0xc564d2d3, 98 },
    { 0x32b4d2d5, 33 },
    { 0xe514d2d4, 100 },
    { 0x41c4d2d6, 61 },
    { 0xc64d2d7, 68 },
    { 0, 0 },
    { 0x3614d2da, 81 },
    { 0, 0 },
    { 0x1e44d2dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d2df, 33 },
    { 0x80b4d2e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b4d2e7, 100 },
    { 0x2dc4d2e8, 43 },
    { 0x7f94d2e7, 23 },
    { 0x2e04d2e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b4d2ed, 84 },
    { 0, 0 },
    { 0x4d24d2ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe4d2f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4d302, 43 },
    { 0x70a4d302, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d309, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604d30d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x574d310, 98 },
    { 0, 0 },
    { 0xdf24d312, 52 },
    { 0x7aa4d312, 90 },
    { 0, 0 },
    { 0x9544d315, 33 },
    { 0x9094d316, 61 },
    { 0x7b4d316, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9864d31c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c74d31f, 23 },
    { 0, 0 },
    { 0xc594d321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc4d324, 61 },
    { 0, 0 },
    { 0xc3f4d326, 90 },
    { 0x53e4d327, 95 },
    { 0xbd64d328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec84d32b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3814d32f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b74d333, 23 },
    { 0xd6c4d333, 33 },
    { 0x8924d335, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd4d33b, 95 },
    { 0x46b4d33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe854d340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6264d343, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6884d348, 23 },
    { 0x8154d348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb494d34e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf4d353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14d359, 77 },
    { 0, 0 },
    { 0x7064d35b, 98 },
    { 0x3874d35b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd004d360, 91 },
    { 0, 0 },
    { 0xea24d362, 91 },
    { 0, 0 },
    { 0xcfa4d364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb104d372, 81 },
    { 0x1544d373, 98 },
    { 0xfe74d374, 88 },
    { 0xdae4d375, 90 },
    { 0x7634d376, 76 },
    { 0, 0 },
    { 0x56d4d378, 23 },
    { 0xbb74d379, 34 },
    { 0x37e4d37a, 23 },
    { 0xd3e4d37a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4d37f, 98 },
    { 0x2334d37f, 90 },
    { 0, 0 },
    { 0xa2d4d382, 98 },
    { 0x8614d383, 33 },
    { 0x9634d384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394d38a, 87 },
    { 0, 0 },
    { 0x7804d38c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9914d393, 98 },
    { 0xef34d394, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64d398, 80 },
    { 0x24c4d399, 52 },
    { 0xcfe4d399, 66 },
    { 0xd124d399, 67 },
    { 0xedb4d398, 33 },
    { 0, 0 },
    { 0x5554d39e, 67 },
    { 0, 0 },
    { 0x40b4d3a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb874d3a4, 90 },
    { 0xdcb4d3a5, 23 },
    { 0x9214d3a5, 90 },
    { 0x32c4d3a4, 33 },
    { 0x15e4d3a8, 90 },
    { 0xce24d3a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b4d3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4954d3b1, 23 },
    { 0xc444d3b2, 98 },
    { 0x65e4d3b3, 78 },
    { 0xf864d3b1, 95 },
    { 0x1774d3b5, 33 },
    { 0, 0 },
    { 0x6254d3b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f74d3bb, 55 },
    { 0x9324d3bc, 100 },
    { 0, 0 },
    { 0x80c4d3be, 93 },
    { 0, 0 },
    { 0x6624d3c0, 95 },
    { 0xa634d3c1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee4d3c7, 43 },
    { 0x4694d3c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7134d3ce, 61 },
    { 0, 0 },
    { 0x66c4d3d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9874d3d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc4d3db, 99 },
    { 0, 0 },
    { 0xefd4d3dd, 99 },
    { 0x1374d3de, 93 },
    { 0xd084d3df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9474d3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa94d3e5, 78 },
    { 0x3814d3e6, 61 },
    { 0, 0 },
    { 0x8014d3e8, 95 },
    { 0xc054d3e9, 38 },
    { 0x6a24d3ea, 76 },
    { 0x8454d3eb, 17 },
    { 0xf944d3ec, 81 },
    { 0x19b4d3ec, 95 },
    { 0xfb44d3ed, 95 },
    { 0xecb4d3ef, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e4d3f3, 98 },
    { 0, 0 },
    { 0x98f4d3f5, 66 },
    { 0x8504d3f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf84d3f9, 84 },
    { 0x58b4d3f9, 61 },
    { 0, 0 },
    { 0x9094d3fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6414d3ff, 46 },
    { 0xf4d4d400, 90 },
    { 0, 0 },
    { 0x6824d402, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb054d408, 61 },
    { 0xee14d409, 78 },
    { 0x97b4d408, 90 },
    { 0, 0 },
    { 0x1c64d40c, 23 },
    { 0, 0 },
    { 0x4e44d40e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3924d412, 95 },
    { 0, 0 },
    { 0x3184d414, 76 },
    { 0, 0 },
    { 0xe884d416, 76 },
    { 0, 0 },
    { 0xf664d418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f44d41e, 33 },
    { 0x57c4d41f, 44 },
    { 0, 0 },
    { 0x7ed4d421, 61 },
    { 0xcbe4d422, 33 },
    { 0xdbf4d423, 81 },
    { 0xa4d424, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7704d429, 90 },
    { 0x43b4d42a, 98 },
    { 0x8ed4d42b, 91 },
    { 0xc514d42c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec4d42f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea4d432, 83 },
    { 0x7c4d432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f4d438, 98 },
    { 0, 0 },
    { 0xd3e4d43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2784d442, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e4d445, 98 },
    { 0x6c14d446, 90 },
    { 0, 0 },
    { 0xb664d448, 66 },
    { 0x7144d448, 68 },
    { 0x4284d44a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a04d44f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3974d453, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4974d457, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d4d465, 23 },
    { 0x7d24d466, 61 },
    { 0x9904d467, 98 },
    { 0xdb74d465, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1644d46c, 55 },
    { 0x2e94d46d, 95 },
    { 0x41e4d46d, 33 },
    { 0x13b4d46e, 61 },
    { 0xe884d470, 90 },
    { 0xf0c4d470, 61 },
    { 0xbdb4d472, 95 },
    { 0, 0 },
    { 0x3f64d474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d74d477, 83 },
    { 0x1624d478, 52 },
    { 0x5c04d479, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd4d47c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7354d481, 23 },
    { 0x6574d482, 77 },
    { 0x53e4d483, 61 },
    { 0x6a54d483, 76 },
    { 0x5964d485, 33 },
    { 0x1684d482, 95 },
    { 0x7a84d487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d84d48b, 33 },
    { 0xd6e4d48c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764d48f, 98 },
    { 0x1ff4d490, 81 },
    { 0x58d4d491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa54d498, 33 },
    { 0x3c24d499, 95 },
    { 0xb394d498, 66 },
    { 0x8a04d49a, 33 },
    { 0x5bd4d49c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb244d4a1, 61 },
    { 0xe384d4a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1294d4a5, 61 },
    { 0xb084d4a5, 76 },
    { 0, 0 },
    { 0x1bf4d4a8, 61 },
    { 0x2754d4a8, 76 },
    { 0x9354d4aa, 33 },
    { 0x4ca4d4ab, 78 },
    { 0x2324d4aa, 34 },
    { 0x7994d4ad, 99 },
    { 0x6ba4d4ae, 90 },
    { 0xebd4d4af, 66 },
    { 0, 0 },
    { 0x4dc4d4b1, 93 },
    { 0x6fc4d4b2, 65 },
    { 0xe114d4b1, 34 },
    { 0xe2e4d4b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a4d4b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc844d4ba, 99 },
    { 0, 0 },
    { 0x6134d4bc, 23 },
    { 0x5af4d4bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb4d4c6, 46 },
    { 0x8d74d4c6, 75 },
    { 0x8e04d4c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a4d4cc, 95 },
    { 0x6fc4d4cd, 97 },
    { 0x4974d4ce, 52 },
    { 0xa354d4cd, 33 },
    { 0x3864d4ce, 43 },
    { 0x4d54d4d0, 61 },
    { 0xcb74d4d2, 98 },
    { 0x1734d4d3, 46 },
    { 0xe5b4d4d2, 90 },
    { 0x2934d4d2, 34 },
    { 0xe3a4d4d0, 61 },
    { 0xbed4d4ce, 98 },
    { 0xf5c4d4d3, 98 },
    { 0xc084d4d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5214d4dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1354d4e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f94d4e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf4d4e7, 90 },
    { 0xd894d4e7, 52 },
    { 0xc664d4e9, 90 },
    { 0, 0 },
    { 0x72a4d4eb, 95 },
    { 0x89e4d4ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe554d4ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c4d4f4, 81 },
    { 0, 0 },
    { 0x8ff4d4f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d4d4fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa884d4ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a4d502, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb4d505, 87 },
    { 0, 0 },
    { 0x8644d507, 68 },
    { 0x5a84d508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a44d50c, 90 },
    { 0, 0 },
    { 0xa4a4d50e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f4d513, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3794d518, 33 },
    { 0, 0 },
    { 0x38f4d51a, 95 },
    { 0x1e4d51b, 95 },
    { 0x8a14d51c, 33 },
    { 0xecf4d51b, 33 },
    { 0, 0 },
    { 0xc94d51f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b04d524, 95 },
    { 0xc1e4d525, 23 },
    { 0xfdf4d525, 95 },
    { 0xdc34d527, 23 },
    { 0xedd4d527, 34 },
    { 0x6654d529, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd4d52e, 23 },
    { 0x3494d52f, 87 },
    { 0x12a4d52f, 34 },
    { 0x3684d531, 98 },
    { 0, 0 },
    { 0x9cc4d533, 33 },
    { 0x19e4d534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d4d537, 61 },
    { 0, 0 },
    { 0x924d539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c4d540, 90 },
    { 0xa2a4d541, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe34d545, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa4d551, 100 },
    { 0x9b64d552, 33 },
    { 0x4fd4d553, 55 },
    { 0x6164d554, 52 },
    { 0xbc34d554, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc4d55d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc134d560, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1004d563, 33 },
    { 0xe484d564, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc944d56c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4d570, 88 },
    { 0, 0 },
    { 0xdc04d572, 91 },
    { 0x4474d573, 99 },
    { 0x1634d574, 61 },
    { 0, 0 },
    { 0x4884d576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa414d57a, 88 },
    { 0, 0 },
    { 0x3e74d57c, 61 },
    { 0x4b34d57d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc44d583, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac4d589, 93 },
    { 0xa1e4d58a, 99 },
    { 0x84a4d58b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4d58e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa424d592, 90 },
    { 0xfea4d593, 100 },
    { 0x2524d594, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb94d59d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef4d5a0, 33 },
    { 0x4544d5a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3594d5a7, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4d5aa, 76 },
    { 0, 0 },
    { 0xd9f4d5ac, 23 },
    { 0xd554d5ad, 95 },
    { 0, 0 },
    { 0xa944d5af, 23 },
    { 0xf244d5af, 46 },
    { 0x3964d5af, 23 },
    { 0x2584d5b2, 87 },
    { 0x3ea4d5b3, 98 },
    { 0xf214d5b2, 98 },
    { 0xb334d5b3, 95 },
    { 0xe2c4d5b3, 43 },
    { 0xd484d5b7, 61 },
    { 0x3e4d5b8, 43 },
    { 0xf564d5b9, 61 },
    { 0, 0 },
    { 0x4154d5bb, 66 },
    { 0xe124d5bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd434d5c3, 55 },
    { 0xdb44d5c3, 23 },
    { 0xa424d5c5, 33 },
    { 0x50c4d5c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e4d5cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb74d5d0, 100 },
    { 0, 0 },
    { 0xb7c4d5d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x274d5d5, 33 },
    { 0xa834d5d5, 61 },
    { 0x394d5d7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4d5dc, 33 },
    { 0, 0 },
    { 0x25e4d5de, 33 },
    { 0x6a44d5de, 76 },
    { 0x4064d5e0, 23 },
    { 0x1ca4d5e0, 100 },
    { 0, 0 },
    { 0x7d34d5e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb04d5e7, 85 },
    { 0, 0 },
    { 0xeb74d5e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb684d5f1, 61 },
    { 0, 0 },
    { 0x9de4d5f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde94d5f8, 90 },
    { 0, 0 },
    { 0x5784d5fa, 90 },
    { 0xbcf4d5fa, 61 },
    { 0, 0 },
    { 0xaa64d5fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec4d606, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5104d609, 95 },
    { 0x2ed4d60a, 55 },
    { 0xc14d609, 93 },
    { 0xe214d60c, 78 },
    { 0x6154d60c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac44d619, 90 },
    { 0, 0 },
    { 0x1104d61b, 23 },
    { 0, 0 },
    { 0x5544d61d, 90 },
    { 0x8624d61e, 68 },
    { 0xd194d61d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6304d622, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b14d627, 61 },
    { 0xa144d627, 43 },
    { 0xe924d629, 8 },
    { 0x8c4d628, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b24d62d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4d632, 33 },
    { 0x2d84d633, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf174d637, 85 },
    { 0, 0 },
    { 0xbb24d639, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4d642, 90 },
    { 0, 0 },
    { 0xc8e4d644, 98 },
    { 0, 0 },
    { 0x6264d646, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a4d64a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a84d64d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb434d655, 95 },
    { 0xd6e4d656, 75 },
    { 0x3844d657, 90 },
    { 0x3d54d658, 66 },
    { 0x2fb4d657, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644d660, 90 },
    { 0, 0 },
    { 0x6884d662, 66 },
    { 0xb54d663, 95 },
    { 0, 0 },
    { 0xff04d665, 34 },
    { 0, 0 },
    { 0x6864d667, 76 },
    { 0, 0 },
    { 0x49c4d669, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xca54d66c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344d671, 99 },
    { 0xae34d671, 98 },
    { 0xd4f4d673, 61 },
    { 0x6554d674, 61 },
    { 0xfa44d675, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a34d679, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca4d67e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554d681, 33 },
    { 0, 0 },
    { 0xe224d683, 33 },
    { 0xda14d683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf794d687, 76 },
    { 0, 0 },
    { 0xf514d689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a24d68f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec4d694, 76 },
    { 0xb684d695, 61 },
    { 0xe194d695, 98 },
    { 0, 0 },
    { 0xf634d698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c4d6a1, 88 },
    { 0, 0 },
    { 0x1ea4d6a3, 23 },
    { 0, 0 },
    { 0x4224d6a5, 23 },
    { 0, 0 },
    { 0x8604d6a7, 98 },
    { 0x96a4d6a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4d6ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a54d6b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb4d6b6, 95 },
    { 0x72e4d6b7, 46 },
    { 0x6364d6b8, 34 },
    { 0x6974d6b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2174d6bc, 23 },
    { 0x9134d6bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb584d6c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4494d6ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2134d6cd, 33 },
    { 0, 0 },
    { 0xf54d6cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a04d6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd064d6e3, 61 },
    { 0xf494d6e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e04d6ea, 65 },
    { 0x82e4d6eb, 90 },
    { 0xc3f4d6ea, 61 },
    { 0x424d6ed, 81 },
    { 0xf094d6ed, 98 },
    { 0x90c4d6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d6f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4d6f6, 55 },
    { 0xf434d6f7, 55 },
    { 0xa7b4d6f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a4d6fb, 93 },
    { 0, 0 },
    { 0x60f4d6fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4d709, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef4d70c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44d712, 61 },
    { 0x8584d713, 52 },
    { 0, 0 },
    { 0xa084d715, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74d71d, 23 },
    { 0, 0 },
    { 0x9a64d71f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac4d723, 90 },
    { 0x27a4d724, 76 },
    { 0x9a44d725, 100 },
    { 0x71d4d725, 91 },
    { 0xf404d723, 95 },
    { 0x4e44d725, 23 },
    { 0xaf54d729, 23 },
    { 0xcef4d72a, 100 },
    { 0xc314d729, 90 },
    { 0xeb04d72a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c4d731, 33 },
    { 0, 0 },
    { 0x8284d733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf4d736, 66 },
    { 0, 0 },
    { 0xe144d738, 33 },
    { 0, 0 },
    { 0xa274d73a, 76 },
    { 0x67e4d73a, 100 },
    { 0x5df4d73a, 76 },
    { 0x50d4d73d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x8714d740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9724d744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc874d749, 91 },
    { 0xce74d749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8144d74e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4d751, 88 },
    { 0, 0 },
    { 0x3cc4d753, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4994d757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd584d75e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b24d764, 93 },
    { 0x8174d764, 90 },
    { 0, 0 },
    { 0xe674d767, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9124d76c, 55 },
    { 0xef34d76c, 61 },
    { 0, 0 },
    { 0x65f4d76f, 23 },
    { 0xf034d76f, 40 },
    { 0xabc4d76f, 78 },
    { 0xdfc4d770, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e04d778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f4d77e, 78 },
    { 0x6384d77e, 17 },
    { 0xccb4d780, 84 },
    { 0x2404d780, 90 },
    { 0x53b4d782, 98 },
    { 0x9bc4d783, 100 },
    { 0xf144d784, 100 },
    { 0xa844d77f, 66 },
    { 0xdca4d785, 23 },
    { 0x6a94d783, 46 },
    { 0x35d4d788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c4d792, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6234d795, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8824d799, 93 },
    { 0xa694d799, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b14d79e, 83 },
    { 0x474d79f, 91 },
    { 0xdf34d79f, 66 },
    { 0x4584d7a1, 33 },
    { 0x8404d79e, 76 },
    { 0x834d7a3, 84 },
    { 0, 0 },
    { 0x16c4d7a5, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4d7a8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654d7ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1914d7b6, 99 },
    { 0x1554d7b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb04d7ba, 100 },
    { 0xb154d7ba, 33 },
    { 0xaf34d7bc, 97 },
    { 0xabc4d7bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4704d7c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a24d7c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db4d7c6, 78 },
    { 0, 0 },
    { 0xcbd4d7c8, 95 },
    { 0, 0 },
    { 0x96b4d7ca, 46 },
    { 0x2c74d7cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e54d7cf, 81 },
    { 0, 0 },
    { 0xf9c4d7d1, 46 },
    { 0x33a4d7d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b4d7d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014d7e1, 98 },
    { 0, 0 },
    { 0xcf64d7e3, 33 },
    { 0x3f4d7e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xea54d7e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d4d7ea, 98 },
    { 0, 0 },
    { 0xc764d7ec, 52 },
    { 0x2e04d7ed, 90 },
    { 0x5394d7ee, 33 },
    { 0x3f14d7ed, 90 },
    { 0, 0 },
    { 0x6994d7f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d7f4, 90 },
    { 0xc14d7f5, 99 },
    { 0xa414d7f6, 98 },
    { 0x9fa4d7f4, 61 },
    { 0, 0 },
    { 0xe5a4d7f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb4d7ff, 95 },
    { 0xf1b4d800, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf24d803, 33 },
    { 0x6d74d804, 61 },
    { 0x1f14d805, 95 },
    { 0, 0 },
    { 0x2034d807, 55 },
    { 0xe5d4d808, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7074d811, 95 },
    { 0xae34d812, 90 },
    { 0x4fa4d813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe174d816, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed44d81c, 65 },
    { 0x8cb4d81c, 66 },
    { 0, 0 },
    { 0x2824d81f, 80 },
    { 0, 0 },
    { 0x15a4d821, 98 },
    { 0, 0 },
    { 0x6324d823, 98 },
    { 0xefc4d823, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8274d827, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af4d82a, 84 },
    { 0, 0 },
    { 0x2244d82c, 98 },
    { 0x15b4d82d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c44d831, 66 },
    { 0xc424d832, 66 },
    { 0xb454d831, 95 },
    { 0x7594d831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4d838, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe394d83b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c84d842, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f4d849, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f54d84d, 98 },
    { 0x9ba4d84e, 61 },
    { 0x7884d84f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d74d855, 95 },
    { 0xc0c4d856, 69 },
    { 0x55e4d857, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e4d85b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b4d85f, 78 },
    { 0xa5e4d860, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d94d864, 61 },
    { 0x3614d864, 100 },
    { 0xb1e4d864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f74d86c, 17 },
    { 0x1aa4d86d, 98 },
    { 0x9244d86d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b54d872, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1234d877, 46 },
    { 0, 0 },
    { 0xb264d879, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a4d87f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c4d887, 90 },
    { 0xdbe4d887, 34 },
    { 0, 0 },
    { 0xa754d88a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec4d88d, 93 },
    { 0x5004d88d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3134d891, 99 },
    { 0xafe4d891, 76 },
    { 0xfeb4d891, 96 },
    { 0xaba4d894, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d898, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf4d89c, 98 },
    { 0x5be4d89d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e4d8a1, 61 },
    { 0xc5e4d8a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f64d8a6, 55 },
    { 0xe94d8a7, 90 },
    { 0, 0 },
    { 0x4ce4d8a9, 34 },
    { 0x88b4d8aa, 100 },
    { 0xe894d8ab, 99 },
    { 0, 0 },
    { 0xa904d8ad, 95 },
    { 0x87c4d8ae, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d4d8b3, 61 },
    { 0, 0 },
    { 0xe6e4d8b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f94d8bb, 55 },
    { 0, 0 },
    { 0xd1a4d8bd, 46 },
    { 0xd44d8be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a4d8c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e4d8d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4d8dc, 23 },
    { 0x67d4d8dd, 23 },
    { 0x1404d8de, 90 },
    { 0, 0 },
    { 0xda24d8e0, 90 },
    { 0x3eb4d8e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b4d8e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2944d8ec, 55 },
    { 0, 0 },
    { 0x7694d8ee, 61 },
    { 0, 0 },
    { 0xcc34d8f0, 23 },
    { 0, 0 },
    { 0xa924d8f2, 33 },
    { 0, 0 },
    { 0x6614d8f4, 33 },
    { 0x68c4d8f4, 78 },
    { 0xb474d8f5, 76 },
    { 0xb274d8f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d4d8fb, 90 },
    { 0x62d4d8fc, 91 },
    { 0, 0 },
    { 0xb224d8fe, 33 },
    { 0xa74d8ff, 55 },
    { 0xad84d900, 52 },
    { 0x1004d8ff, 38 },
    { 0, 0 },
    { 0x94d4d903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e4d908, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1404d90e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7874d911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a4d916, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c74d919, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6564d91d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc4d923, 90 },
    { 0xe7e4d924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x814d927, 70 },
    { 0x5e24d928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc774d930, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7944d933, 61 },
    { 0x3dc4d934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9824d938, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd4d941, 98 },
    { 0xb294d942, 80 },
    { 0x4324d942, 33 },
    { 0xc644d944, 33 },
    { 0x6964d943, 33 },
    { 0xe374d946, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d4d950, 55 },
    { 0, 0 },
    { 0x4084d952, 90 },
    { 0xbe74d952, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf204d959, 33 },
    { 0xbee4d959, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7864d960, 98 },
    { 0, 0 },
    { 0xb0b4d962, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4d966, 61 },
    { 0x6444d967, 55 },
    { 0, 0 },
    { 0xf8c4d969, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd4d973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c4d976, 55 },
    { 0x514d977, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b54d981, 61 },
    { 0, 0 },
    { 0x26b4d983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe424d98d, 61 },
    { 0x6ca4d98d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774d991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb4d998, 50 },
    { 0, 0 },
    { 0xd0d4d99a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e4d99d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e4d9a4, 99 },
    { 0, 0 },
    { 0x2394d9a6, 23 },
    { 0x97b4d9a6, 84 },
    { 0xc264d9a7, 33 },
    { 0xcd84d9a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc4d9ac, 90 },
    { 0x7fa4d9ad, 18 },
    { 0x3ef4d9ae, 99 },
    { 0x9e74d9af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7414d9bc, 23 },
    { 0, 0 },
    { 0x5814d9be, 90 },
    { 0xee54d9be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5444d9c7, 90 },
    { 0, 0 },
    { 0xa9f4d9c9, 80 },
    { 0x4744d9c9, 91 },
    { 0x2f4d9ca, 93 },
    { 0x8a74d9cc, 95 },
    { 0xbe54d9cd, 76 },
    { 0x5c04d9cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff4d9d1, 23 },
    { 0, 0 },
    { 0xeb04d9d3, 33 },
    { 0, 0 },
    { 0xa594d9d5, 43 },
    { 0x274d9d6, 61 },
    { 0x6be4d9d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984d9db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a54d9e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa964d9e4, 55 },
    { 0, 0 },
    { 0xdb34d9e6, 69 },
    { 0x6194d9e7, 85 },
    { 0xb054d9e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6104d9f2, 98 },
    { 0, 0 },
    { 0xb174d9f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c44d9fa, 91 },
    { 0x1264d9fa, 34 },
    { 0x6fc4d9fb, 68 },
    { 0x8714d9fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f74da00, 61 },
    { 0x85f4da01, 33 },
    { 0xa344da02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb824da06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6894da0d, 33 },
    { 0x2b04da0d, 95 },
    { 0xdcb4da0f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb224da12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4da15, 100 },
    { 0, 0 },
    { 0x28c4da17, 33 },
    { 0x944da18, 91 },
    { 0xe3d4da19, 90 },
    { 0xe2c4da1a, 95 },
    { 0x1844da17, 90 },
    { 0x9604da1c, 99 },
    { 0xae24da1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a44da23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb4da27, 90 },
    { 0, 0 },
    { 0x5004da29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f4da2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc494da33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b44da37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b4da3d, 98 },
    { 0x3fc4da3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c4da44, 22 },
    { 0xa0d4da45, 65 },
    { 0xf704da46, 33 },
    { 0xc394da47, 33 },
    { 0x354da47, 33 },
    { 0, 0 },
    { 0xc6f4da4a, 61 },
    { 0, 0 },
    { 0x3984da4c, 55 },
    { 0xd554da4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d4da50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f4da53, 95 },
    { 0x2374da54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5714da5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf4da5d, 93 },
    { 0, 0 },
    { 0x15c4da5f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc4da64, 100 },
    { 0x8a44da64, 76 },
    { 0, 0 },
    { 0x2824da67, 23 },
    { 0x7164da68, 46 },
    { 0x7654da69, 43 },
    { 0x8354da68, 85 },
    { 0xf224da6b, 55 },
    { 0x1714da6b, 61 },
    { 0x9734da6b, 87 },
    { 0x29a4da6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4da71, 90 },
    { 0x8e44da72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4384da7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4da82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b4da86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9834da89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5534da8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64da92, 67 },
    { 0x6084da92, 90 },
    { 0x2b04da94, 75 },
    { 0x6314da95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a94da9b, 61 },
    { 0x6eb4da9c, 34 },
    { 0x69f4da9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4daaa, 90 },
    { 0x1834daab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd4daaf, 33 },
    { 0xcce4dab0, 61 },
    { 0, 0 },
    { 0xa64dab2, 95 },
    { 0, 0 },
    { 0xc084dab4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2334dab8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf294dabd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e4dac1, 55 },
    { 0x14c4dac1, 61 },
    { 0xbbd4dac3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad64dac6, 52 },
    { 0x7384dac6, 34 },
    { 0, 0 },
    { 0xd714dac9, 55 },
    { 0xd2d4dac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d94dacd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf4dad0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb904dad3, 68 },
    { 0xf24dad4, 98 },
    { 0x5c94dad4, 23 },
    { 0xe414dad6, 34 },
    { 0x684dad7, 99 },
    { 0x9c14dad8, 65 },
    { 0x4b44dad9, 55 },
    { 0x55f4dada, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd764dadd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4014dae2, 46 },
    { 0x5af4dae2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c4dae7, 23 },
    { 0xf644dae8, 90 },
    { 0x6c94dae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a4daec, 55 },
    { 0x11f4daed, 96 },
    { 0x4804daed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b4daf2, 84 },
    { 0xa6e4daf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7204daf6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4dafd, 95 },
    { 0x3784dafe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb764db01, 90 },
    { 0, 0 },
    { 0xbf04db03, 90 },
    { 0x6084db04, 95 },
    { 0, 0 },
    { 0xbc54db06, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4db0a, 23 },
    { 0x2894db0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf914db11, 34 },
    { 0, 0 },
    { 0x1574db13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c4db16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c54db19, 34 },
    { 0xd174db1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa94db1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb724db22, 90 },
    { 0xcb44db23, 98 },
    { 0x75e4db24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d94db27, 95 },
    { 0xa294db28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf244db32, 98 },
    { 0, 0 },
    { 0x1744db34, 33 },
    { 0x83b4db35, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd384db3b, 83 },
    { 0xd254db3c, 85 },
    { 0x6d24db3c, 34 },
    { 0xeef4db3d, 23 },
    { 0xa534db3c, 90 },
    { 0x6914db40, 98 },
    { 0xe604db3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e4db51, 87 },
    { 0x1014db51, 93 },
    { 0x1a64db51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb74db5b, 33 },
    { 0x2b34db5c, 95 },
    { 0, 0 },
    { 0x1bb4db5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d4db67, 33 },
    { 0x12c4db68, 17 },
    { 0xf314db69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d4db6d, 90 },
    { 0xedf4db6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf84db73, 98 },
    { 0xb124db74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b4db78, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db87, 23 },
    { 0, 0 },
    { 0x5c34db89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b4db91, 91 },
    { 0x6614db91, 34 },
    { 0x914db93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9714db97, 33 },
    { 0x64d4db97, 88 },
    { 0x4d94db99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d4db9c, 100 },
    { 0x7ba4db9c, 23 },
    { 0xb1d4db9e, 61 },
    { 0x2ed4db9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8354dba5, 90 },
    { 0x3384dba6, 90 },
    { 0x5d34dba7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934dbab, 93 },
    { 0x4a54dbac, 90 },
    { 0, 0 },
    { 0x70f4dbae, 66 },
    { 0x4654dbaf, 85 },
    { 0xcc84dbb0, 66 },
    { 0x7d4dbb1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b4dbb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f4dbb9, 76 },
    { 0xb5f4dbba, 100 },
    { 0x2124dbbb, 98 },
    { 0xf824dbb9, 95 },
    { 0xa504dbbb, 23 },
    { 0, 0 },
    { 0x6d54dbbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7604dbc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a74dbc7, 43 },
    { 0x94a4dbc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce4dbcc, 52 },
    { 0x3fe4dbcd, 23 },
    { 0, 0 },
    { 0x2374dbcf, 61 },
    { 0x72b4dbd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e4dbd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4dbde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4594dbe2, 61 },
    { 0x65a4dbe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c4dbeb, 61 },
    { 0x2e04dbec, 98 },
    { 0xfaf4dbed, 78 },
    { 0xa764dbed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e4dbf9, 61 },
    { 0, 0 },
    { 0x7224dbfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a44dc02, 76 },
    { 0x4304dc03, 19 },
    { 0xf34dc04, 46 },
    { 0xc974dc05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c64dc09, 90 },
    { 0, 0 },
    { 0x3514dc0b, 98 },
    { 0x8bd4dc0c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee04dc15, 23 },
    { 0, 0 },
    { 0xdd44dc17, 61 },
    { 0, 0 },
    { 0xc054dc19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d4dc1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d14dc22, 33 },
    { 0, 0 },
    { 0x4404dc24, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e4dc29, 78 },
    { 0x76f4dc29, 97 },
    { 0x1264dc2a, 100 },
    { 0x1b04dc2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c4dc30, 68 },
    { 0, 0 },
    { 0x86f4dc32, 75 },
    { 0x4584dc33, 34 },
    { 0, 0 },
    { 0xb514dc35, 98 },
    { 0x5644dc36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4194dc3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84dc3e, 33 },
    { 0, 0 },
    { 0x23a4dc40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d4dc43, 69 },
    { 0x42f4dc44, 98 },
    { 0, 0 },
    { 0xe624dc46, 61 },
    { 0x2984dc47, 33 },
    { 0x2684dc48, 46 },
    { 0, 0 },
    { 0x7534dc4a, 34 },
    { 0x3d24dc4a, 98 },
    { 0, 0 },
    { 0xe1f4dc4d, 95 },
    { 0, 0 },
    { 0x16a4dc4f, 100 },
    { 0, 0 },
    { 0xe884dc51, 43 },
    { 0xcc04dc52, 76 },
    { 0, 0 },
    { 0x1754dc54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda24dc57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde64dc5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe614dc60, 76 },
    { 0, 0 },
    { 0x2d74dc62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6104dc65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4dc6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec4dc6f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc4dc72, 67 },
    { 0xc364dc72, 98 },
    { 0x5874dc72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xace4dc77, 43 },
    { 0x3884dc78, 98 },
    { 0x89e4dc78, 33 },
    { 0, 0 },
    { 0xace4dc7b, 61 },
    { 0, 0 },
    { 0xa304dc7d, 46 },
    { 0, 0 },
    { 0xad64dc7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a84dc82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a4dc86, 76 },
    { 0, 0 },
    { 0x3324dc88, 78 },
    { 0xe8f4dc88, 95 },
    { 0, 0 },
    { 0x14c4dc8b, 98 },
    { 0x2aa4dc8b, 61 },
    { 0x9874dc8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a4dc91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a4dc98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a4dc9f, 95 },
    { 0xab74dca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc504dca4, 93 },
    { 0x6204dca5, 67 },
    { 0xe74dca4, 61 },
    { 0, 0 },
    { 0xba94dca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb764dcab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8514dcae, 33 },
    { 0, 0 },
    { 0x7084dcb0, 61 },
    { 0, 0 },
    { 0x3fd4dcb2, 23 },
    { 0x9de4dcb2, 97 },
    { 0x59e4dcb4, 33 },
    { 0x9144dcb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4014dcb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4964dcbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe444dcc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64dccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc294dccf, 33 },
    { 0x4794dcd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae84dcd5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2914dcda, 90 },
    { 0, 0 },
    { 0xcf44dcdc, 92 },
    { 0x46b4dcdd, 33 },
    { 0, 0 },
    { 0x1c24dcdf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7944dce2, 87 },
    { 0xc674dce3, 98 },
    { 0xa4dce4, 75 },
    { 0xbac4dce3, 61 },
    { 0xba54dce4, 76 },
    { 0x5474dce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4824dcee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b4dcf1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c4dcf4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae44dcf8, 23 },
    { 0, 0 },
    { 0x62e4dcfa, 66 },
    { 0, 0 },
    { 0xad44dcfc, 68 },
    { 0x4d84dcfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5064dd01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c4dd06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1474dd09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8754dd0d, 84 },
    { 0x6574dd0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d4dd13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f74dd17, 33 },
    { 0xf7d4dd18, 93 },
    { 0, 0 },
    { 0x84f4dd1a, 95 },
    { 0xc0d4dd1b, 76 },
    { 0xf5d4dd1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e04dd1f, 76 },
    { 0x2fa4dd1f, 76 },
    { 0xf964dd21, 76 },
    { 0x9284dd22, 98 },
    { 0, 0 },
    { 0x7104dd24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef4dd27, 33 },
    { 0xf294dd28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4dd2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c84dd31, 85 },
    { 0xf7d4dd32, 85 },
    { 0, 0 },
    { 0xf4c4dd34, 95 },
    { 0xce14dd34, 33 },
    { 0x1044dd36, 61 },
    { 0xbfc4dd36, 76 },
    { 0x36d4dd38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa4dd3c, 33 },
    { 0xb864dd3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e4dd40, 55 },
    { 0x77f4dd41, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae94dd47, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b4dd4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e4dd4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f4dd5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc934dd5f, 98 },
    { 0x7444dd5f, 23 },
    { 0xd444dd61, 23 },
    { 0x3d44dd62, 55 },
    { 0, 0 },
    { 0x62b4dd64, 23 },
    { 0xb2d4dd64, 90 },
    { 0x5364dd66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe294dd6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3934dd72, 61 },
    { 0xd7c4dd73, 33 },
    { 0, 0 },
    { 0x57b4dd75, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc24dd7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc4dd7f, 98 },
    { 0x90c4dd80, 67 },
    { 0, 0 },
    { 0x9f14dd82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f4dd8c, 90 },
    { 0xc924dd8d, 36 },
    { 0, 0 },
    { 0x5704dd8f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4914dd94, 95 },
    { 0, 0 },
    { 0x2164dd96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8744dd9a, 46 },
    { 0x5db4dd9a, 99 },
    { 0x7794dd9c, 90 },
    { 0x3e14dd9c, 98 },
    { 0x25f4dd9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d4dda2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c4dda5, 33 },
    { 0, 0 },
    { 0x93a4dda7, 61 },
    { 0, 0 },
    { 0xcd24dda9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2114ddb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc94ddb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b4ddba, 90 },
    { 0xb3f4ddbb, 76 },
    { 0x75f4ddbb, 36 },
    { 0x3644ddbd, 98 },
    { 0xd7e4ddbe, 87 },
    { 0xa444ddbf, 33 },
    { 0xd934ddbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf4ddc4, 33 },
    { 0x5b74ddc4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8744ddca, 34 },
    { 0, 0 },
    { 0xb444ddcc, 90 },
    { 0x3da4ddcd, 99 },
    { 0x784ddcd, 98 },
    { 0x3fb4ddcf, 88 },
    { 0, 0 },
    { 0xf1d4ddd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c94ddd7, 98 },
    { 0x9454ddd7, 93 },
    { 0x9164ddd9, 34 },
    { 0, 0 },
    { 0x4d04dddb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc034dde2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd254dde5, 95 },
    { 0x7ce4dde5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea94ddeb, 61 },
    { 0xe4d4ddec, 90 },
    { 0xd3d4dded, 33 },
    { 0x5fe4ddee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54ddf7, 98 },
    { 0x5244ddf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b4de01, 78 },
    { 0, 0 },
    { 0xc834de03, 100 },
    { 0x60a4de04, 23 },
    { 0x3504de05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be4de0b, 23 },
    { 0, 0 },
    { 0xd934de0d, 55 },
    { 0xd4b4de0e, 100 },
    { 0xd5b4de0e, 23 },
    { 0xd464de0f, 95 },
    { 0xa424de11, 52 },
    { 0xfda4de12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3724de15, 95 },
    { 0, 0 },
    { 0xb574de17, 61 },
    { 0, 0 },
    { 0x8634de19, 33 },
    { 0x6f44de1a, 76 },
    { 0xed24de1a, 61 },
    { 0xb624de1c, 90 },
    { 0xb2f4de1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f4de28, 68 },
    { 0, 0 },
    { 0xdd44de2a, 98 },
    { 0x9db4de2b, 33 },
    { 0x90f4de2b, 40 },
    { 0, 0 },
    { 0xc564de2e, 23 },
    { 0x2d14de2e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf4de35, 65 },
    { 0xd74de35, 23 },
    { 0, 0 },
    { 0xe354de38, 99 },
    { 0x84f4de38, 33 },
    { 0xb8b4de3a, 93 },
    { 0x7eb4de3b, 52 },
    { 0x2ee4de3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d4de41, 88 },
    { 0xf4d4de42, 90 },
    { 0xf574de43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c4de48, 98 },
    { 0xe474de48, 23 },
    { 0, 0 },
    { 0xdb14de4b, 66 },
    { 0, 0 },
    { 0x9e84de4d, 90 },
    { 0, 0 },
    { 0x65b4de4f, 95 },
    { 0, 0 },
    { 0xbf4de51, 93 },
    { 0xa9a4de51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84de55, 46 },
    { 0x1e84de55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe674de59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb4de5e, 95 },
    { 0, 0 },
    { 0x35a4de60, 90 },
    { 0, 0 },
    { 0x7104de62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74de65, 23 },
    { 0x2ea4de65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b34de6a, 91 },
    { 0x5b64de6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3634de6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4de73, 95 },
    { 0xe114de74, 76 },
    { 0xc2a4de75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe384de79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5694de80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3904de85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9844de8d, 83 },
    { 0, 0 },
    { 0x2af4de8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab4de93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1074de96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd54de99, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7884de9c, 55 },
    { 0xd974de9d, 85 },
    { 0x8ad4de9c, 95 },
    { 0xecc4de9e, 67 },
    { 0xbdc4dea0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d34dea3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2284dea6, 95 },
    { 0, 0 },
    { 0x2bd4dea8, 87 },
    { 0xbbf4dea9, 33 },
    { 0x7c94deaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc264deae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd414deb4, 83 },
    { 0, 0 },
    { 0xb4b4deb6, 61 },
    { 0, 0 },
    { 0x6b74deb8, 66 },
    { 0, 0 },
    { 0x13c4deba, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa394debf, 55 },
    { 0x3af4debf, 91 },
    { 0xb744dec1, 52 },
    { 0x1734dec2, 84 },
    { 0xc714dec2, 23 },
    { 0x2464dec3, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd4decd, 90 },
    { 0xef34dece, 90 },
    { 0x8214decd, 99 },
    { 0xe754decf, 23 },
    { 0, 0 },
    { 0x1714ded2, 92 },
    { 0x43e4ded3, 87 },
    { 0x6074ded3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae4ded8, 46 },
    { 0x354ded8, 90 },
    { 0xc034deda, 61 },
    { 0x7434dedb, 33 },
    { 0, 0 },
    { 0xb284dedd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9894dee9, 46 },
    { 0x9564deea, 46 },
    { 0, 0 },
    { 0x1274deec, 61 },
    { 0, 0 },
    { 0x44a4deee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c74def1, 69 },
    { 0, 0 },
    { 0x6f64def3, 33 },
    { 0xbc04def4, 33 },
    { 0, 0 },
    { 0x1194def6, 77 },
    { 0x1844def6, 61 },
    { 0x32a4def8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe4defe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a4df04, 90 },
    { 0x7434df05, 33 },
    { 0xf94df06, 33 },
    { 0xa914df06, 55 },
    { 0x8a4df08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a54df10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e4df16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa184df19, 34 },
    { 0xbca4df1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe624df24, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc854df2d, 98 },
    { 0x1074df2e, 23 },
    { 0xe964df2f, 65 },
    { 0, 0 },
    { 0xf354df31, 90 },
    { 0, 0 },
    { 0x50f4df33, 76 },
    { 0x7d14df34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5954df37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd384df3b, 90 },
    { 0x5d34df3c, 33 },
    { 0, 0 },
    { 0x11e4df3e, 23 },
    { 0xb834df3f, 76 },
    { 0xf2a4df40, 78 },
    { 0x1724df41, 23 },
    { 0x9a94df3f, 33 },
    { 0x5794df43, 91 },
    { 0xa544df44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e14df47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f24df4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e4df53, 90 },
    { 0x2b94df54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9034df58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf674df5d, 95 },
    { 0, 0 },
    { 0x9ea4df5f, 61 },
    { 0x4b34df60, 61 },
    { 0x4e54df5f, 55 },
    { 0x5cf4df62, 61 },
    { 0xebf4df61, 87 },
    { 0xcac4df5f, 90 },
    { 0, 0 },
    { 0x75f4df66, 33 },
    { 0xd644df67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb774df6b, 61 },
    { 0x30a4df6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8844df6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc4df72, 33 },
    { 0xc594df73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e74df77, 88 },
    { 0xbaa4df78, 91 },
    { 0xed74df78, 17 },
    { 0, 0 },
    { 0x4e4df7b, 23 },
    { 0x204df7c, 66 },
    { 0xa74df7d, 33 },
    { 0x2214df7e, 90 },
    { 0x7c74df7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c4df82, 78 },
    { 0xe214df82, 98 },
    { 0x8a74df83, 98 },
    { 0x2e44df83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba4df8a, 33 },
    { 0, 0 },
    { 0x27b4df8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2594df94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd624df99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb4df9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed54dfa9, 98 },
    { 0x6df4dfaa, 98 },
    { 0, 0 },
    { 0xf7b4dfac, 98 },
    { 0, 0 },
    { 0xa1a4dfae, 90 },
    { 0xbe34dfaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe354dfb3, 61 },
    { 0xdf84dfb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af4dfb8, 100 },
    { 0xb404dfb9, 52 },
    { 0x67a4dfba, 91 },
    { 0xa6e4dfbb, 61 },
    { 0xc8a4dfbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8694dfbf, 61 },
    { 0x64b4dfbf, 55 },
    { 0, 0 },
    { 0xbf04dfc2, 33 },
    { 0, 0 },
    { 0x1b4dfc4, 61 },
    { 0xaaf4dfc5, 93 },
    { 0x82b4dfc6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc124dfcb, 34 },
    { 0xac34dfcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda04dfd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf824dfdb, 90 },
    { 0x5b44dfdb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8024dfe2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3044dfe8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6324dfef, 95 },
    { 0, 0 },
    { 0x4464dff1, 98 },
    { 0x2c4dff2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa384dff6, 33 },
    { 0x48e4dff7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f4dffa, 34 },
    { 0, 0 },
    { 0xa704dffc, 98 },
    { 0, 0 },
    { 0xdb24dffe, 98 },
    { 0x7234dffe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d14e003, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf84e007, 61 },
    { 0x3c54e008, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c4e00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3584e00f, 98 },
    { 0xee54e010, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa984e014, 23 },
    { 0, 0 },
    { 0xd724e016, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd154e019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc4e01d, 76 },
    { 0x9224e01e, 66 },
    { 0x42e4e01f, 98 },
    { 0xbee4e020, 90 },
    { 0x9c04e01d, 65 },
    { 0xf464e022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854e026, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x614e029, 23 },
    { 0xd524e029, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b4e030, 61 },
    { 0xe7a4e031, 61 },
    { 0x1d54e032, 55 },
    { 0, 0 },
    { 0x1964e034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e54e039, 94 },
    { 0x3c84e03a, 90 },
    { 0x5e44e03b, 52 },
    { 0x2814e03c, 76 },
    { 0xe4c4e03d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4e040, 66 },
    { 0x5904e040, 23 },
    { 0, 0 },
    { 0xfcd4e043, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a4e04d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a4e050, 90 },
    { 0xe5e4e050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf924e055, 61 },
    { 0xfe54e056, 33 },
    { 0xc1b4e057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa494e05c, 33 },
    { 0x3544e05d, 61 },
    { 0x9654e05d, 95 },
    { 0xa684e05f, 61 },
    { 0xd774e05f, 78 },
    { 0x5b24e05f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x1694e064, 90 },
    { 0xa004e065, 23 },
    { 0xd794e066, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce4e06a, 55 },
    { 0, 0 },
    { 0x3db4e06c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c34e073, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4e078, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b4e07c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc014e084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f4e087, 61 },
    { 0x44f4e088, 77 },
    { 0x26e4e088, 90 },
    { 0, 0 },
    { 0x77e4e08b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe44e094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b4e098, 61 },
    { 0, 0 },
    { 0xf164e09a, 84 },
    { 0x7f04e09b, 61 },
    { 0, 0 },
    { 0x9634e09d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5944e0a2, 33 },
    { 0x2474e0a3, 77 },
    { 0xa294e0a2, 23 },
    { 0x3214e0a2, 61 },
    { 0, 0 },
    { 0x4094e0a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc654e0b4, 23 },
    { 0, 0 },
    { 0xd624e0b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4e0bc, 67 },
    { 0xd094e0bd, 85 },
    { 0x22b4e0be, 46 },
    { 0, 0 },
    { 0x54a4e0c0, 61 },
    { 0xcb04e0c1, 34 },
    { 0xc04e0c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b34e0c5, 90 },
    { 0x94e4e0c6, 91 },
    { 0x6ea4e0c5, 93 },
    { 0, 0 },
    { 0x2ed4e0c9, 23 },
    { 0x6c84e0ca, 55 },
    { 0xdbb4e0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4e0cf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc854e0d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a4e0d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a4e0db, 95 },
    { 0, 0 },
    { 0x47f4e0dd, 23 },
    { 0xd04e0de, 90 },
    { 0x10d4e0df, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c04e0e6, 23 },
    { 0xf0b4e0e7, 55 },
    { 0x5594e0e6, 99 },
    { 0xb8a4e0e6, 87 },
    { 0xebc4e0e9, 55 },
    { 0xa654e0e8, 61 },
    { 0xa074e0ec, 95 },
    { 0xef44e0ed, 34 },
    { 0x6594e0e6, 46 },
    { 0xdce4e0ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c4e0f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2344e0f8, 33 },
    { 0x4b04e0f9, 100 },
    { 0xde04e0fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b4e108, 95 },
    { 0x3904e109, 91 },
    { 0x9bb4e108, 95 },
    { 0xdd24e10a, 78 },
    { 0x7134e10a, 33 },
    { 0, 0 },
    { 0x59a4e10e, 90 },
    { 0x28b4e10f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484e117, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b84e11f, 23 },
    { 0x7f54e120, 98 },
    { 0x4b54e121, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb484e125, 33 },
    { 0, 0 },
    { 0xcc14e127, 90 },
    { 0, 0 },
    { 0x7a54e129, 81 },
    { 0, 0 },
    { 0xecb4e12b, 84 },
    { 0x9d14e12c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e84e12f, 82 },
    { 0xd604e130, 61 },
    { 0, 0 },
    { 0x90f4e132, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f14e136, 17 },
    { 0, 0 },
    { 0xddc4e138, 99 },
    { 0x1984e139, 61 },
    { 0, 0 },
    { 0xd144e13b, 93 },
    { 0x7c64e13c, 23 },
    { 0, 0 },
    { 0xa0b4e13e, 93 },
    { 0x6b44e13f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb94e142, 90 },
    { 0xa704e142, 23 },
    { 0xc704e144, 90 },
    { 0x2ec4e144, 61 },
    { 0xb684e142, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba4e14b, 61 },
    { 0xd0a4e14c, 55 },
    { 0x1aa4e14d, 98 },
    { 0xad14e14b, 90 },
    { 0x22c4e14f, 55 },
    { 0x3b44e150, 36 },
    { 0xdd24e151, 43 },
    { 0xf094e152, 95 },
    { 0, 0 },
    { 0x7da4e154, 33 },
    { 0x82e4e154, 33 },
    { 0x2554e156, 90 },
    { 0, 0 },
    { 0x9774e158, 76 },
    { 0xf7f4e159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b4e15f, 33 },
    { 0xbcd4e160, 33 },
    { 0, 0 },
    { 0x4364e162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4e16b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d64e172, 95 },
    { 0x5104e173, 95 },
    { 0x1d34e172, 90 },
    { 0, 0 },
    { 0x4164e176, 66 },
    { 0xcef4e177, 33 },
    { 0x7e04e178, 90 },
    { 0xb374e178, 61 },
    { 0, 0 },
    { 0x6cf4e17b, 23 },
    { 0x1cd4e17c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a04e17f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4024e182, 23 },
    { 0x24f4e182, 61 },
    { 0x2e74e184, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b94e18a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb214e18d, 90 },
    { 0, 0 },
    { 0x45e4e18f, 90 },
    { 0, 0 },
    { 0x10f4e191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a4e19c, 67 },
    { 0x53e4e19d, 90 },
    { 0, 0 },
    { 0xfd44e19f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe24e1a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa4e1a5, 90 },
    { 0x52c4e1a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1854e1ae, 87 },
    { 0, 0 },
    { 0xefd4e1b0, 99 },
    { 0x1224e1b1, 90 },
    { 0x5054e1b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d4e1b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa784e1bf, 33 },
    { 0x8b4e1bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff4e1c5, 33 },
    { 0x24d4e1c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3674e1cb, 75 },
    { 0xcc54e1cb, 95 },
    { 0x4684e1cb, 38 },
    { 0x7034e1cc, 98 },
    { 0x48e4e1ce, 56 },
    { 0x5624e1d0, 91 },
    { 0xb8b4e1cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284e1d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f4e1dc, 90 },
    { 0xeb54e1dd, 98 },
    { 0x8dc4e1dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4e1e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f54e1e5, 33 },
    { 0xa7a4e1e5, 76 },
    { 0xff74e1e7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9964e1eb, 88 },
    { 0x8154e1ec, 61 },
    { 0x1c54e1ec, 90 },
    { 0x5354e1eb, 33 },
    { 0xbd44e1ee, 94 },
    { 0, 0 },
    { 0xf64e1f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed4e1fe, 90 },
    { 0, 0 },
    { 0x3b44e200, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d64e203, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd214e207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d04e20a, 85 },
    { 0, 0 },
    { 0x9204e20c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a24e20f, 76 },
    { 0xdd64e20f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a04e21b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4e21f, 46 },
    { 0, 0 },
    { 0xf7c4e221, 61 },
    { 0xd064e221, 91 },
    { 0x32d4e223, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f4e227, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f4e22b, 46 },
    { 0xa114e22c, 46 },
    { 0, 0 },
    { 0x9ab4e22e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf4e233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea34e239, 33 },
    { 0x7664e239, 76 },
    { 0x1874e239, 91 },
    { 0x7654e23c, 95 },
    { 0x71d4e23a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8134e242, 85 },
    { 0, 0 },
    { 0xe584e244, 95 },
    { 0xe704e245, 46 },
    { 0x1a4e245, 90 },
    { 0, 0 },
    { 0x8404e248, 33 },
    { 0, 0 },
    { 0x8f64e24a, 76 },
    { 0xa284e24b, 81 },
    { 0x3b94e24c, 55 },
    { 0xd5d4e24b, 95 },
    { 0xa874e24e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64e253, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a4e257, 90 },
    { 0x2fd4e258, 89 },
    { 0x3b24e258, 61 },
    { 0x5954e258, 99 },
    { 0xca4e25b, 33 },
    { 0xb1c4e258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa154e260, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a4e26f, 43 },
    { 0x71b4e270, 33 },
    { 0xf564e271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd14e274, 46 },
    { 0, 0 },
    { 0x3d4e276, 61 },
    { 0xbd04e276, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd4e27c, 78 },
    { 0xd634e27d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6164e283, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5104e28a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f4e28f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc4e292, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd4e295, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f54e29b, 34 },
    { 0x1d34e29c, 55 },
    { 0x1674e29c, 95 },
    { 0x53a4e29d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca24e2a3, 80 },
    { 0x84a4e2a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf54e2aa, 90 },
    { 0xdb4e2aa, 55 },
    { 0x4954e2ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7474e2ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd4e2c2, 33 },
    { 0x3194e2c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb294e2cb, 61 },
    { 0x7e44e2cc, 76 },
    { 0x38c4e2cc, 98 },
    { 0, 0 },
    { 0x3b4e2cf, 64 },
    { 0, 0 },
    { 0xf384e2d1, 61 },
    { 0, 0 },
    { 0xfd94e2d3, 34 },
    { 0x124e2d4, 33 },
    { 0x85e4e2d3, 98 },
    { 0xd2e4e2d6, 89 },
    { 0xb7d4e2d6, 95 },
    { 0xfcb4e2d8, 77 },
    { 0x6b04e2d9, 76 },
    { 0xb5d4e2d9, 61 },
    { 0, 0 },
    { 0x1b24e2dc, 99 },
    { 0x1424e2dd, 76 },
    { 0x4f64e2dc, 100 },
    { 0x9e74e2df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c4e2e3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1264e2e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe094e2ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a4e2f1, 43 },
    { 0x3da4e2f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec4e2f5, 95 },
    { 0, 0 },
    { 0x1e04e2f7, 34 },
    { 0, 0 },
    { 0x2284e2f9, 90 },
    { 0x95e4e2f9, 66 },
    { 0xe2e4e2fb, 55 },
    { 0x9324e2fa, 95 },
    { 0x1404e2fa, 90 },
    { 0x6c94e2f9, 23 },
    { 0xbae4e2fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c94e306, 90 },
    { 0xb5b4e307, 55 },
    { 0xe3b4e307, 23 },
    { 0xfda4e308, 68 },
    { 0x6bc4e307, 90 },
    { 0, 0 },
    { 0x8754e30c, 95 },
    { 0x64e4e30c, 23 },
    { 0xf824e30d, 95 },
    { 0x7384e30f, 66 },
    { 0x4284e30e, 33 },
    { 0x2f74e30f, 81 },
    { 0xcaf4e30d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c34e317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a14e31a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd4e321, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf474e324, 100 },
    { 0x5324e325, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a4e32a, 76 },
    { 0xab04e32b, 33 },
    { 0, 0 },
    { 0x8694e32d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3344e331, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6724e337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e94e33a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174e33f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c4e343, 77 },
    { 0x3cc4e344, 90 },
    { 0x9334e344, 66 },
    { 0x6924e346, 17 },
    { 0, 0 },
    { 0xd534e348, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34e34f, 33 },
    { 0xa624e350, 32 },
    { 0xb384e350, 76 },
    { 0xcd54e350, 23 },
    { 0x9a14e352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4214e35c, 17 },
    { 0x34e4e35d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94e361, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b4e364, 90 },
    { 0x74d4e365, 77 },
    { 0x4284e365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf4e36a, 90 },
    { 0, 0 },
    { 0x5354e36c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af4e371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8654e375, 90 },
    { 0x77a4e376, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce4e37b, 78 },
    { 0xd534e37c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6064e388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed4e392, 19 },
    { 0x784e393, 61 },
    { 0x7614e392, 75 },
    { 0xd994e394, 76 },
    { 0x1df4e396, 61 },
    { 0x64e397, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc014e39d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1314e3a0, 61 },
    { 0x6ca4e3a1, 95 },
    { 0x8064e3a2, 23 },
    { 0x20a4e3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04e3a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1904e3ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5894e3b4, 90 },
    { 0, 0 },
    { 0x58f4e3b6, 76 },
    { 0xde44e3b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b84e3bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524e3c0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db4e3c5, 55 },
    { 0x8734e3c6, 34 },
    { 0x24c4e3c6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e4e3ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4924e3d3, 84 },
    { 0x7414e3d4, 90 },
    { 0, 0 },
    { 0x19e4e3d6, 46 },
    { 0, 0 },
    { 0x5cc4e3d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e34e3dd, 90 },
    { 0, 0 },
    { 0x7544e3df, 46 },
    { 0xdc24e3df, 61 },
    { 0, 0 },
    { 0xc8d4e3e2, 23 },
    { 0, 0 },
    { 0xaf24e3e4, 98 },
    { 0xc574e3e5, 90 },
    { 0x5b84e3e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb334e3eb, 99 },
    { 0, 0 },
    { 0xd484e3ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c4e3f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca4e3f6, 33 },
    { 0, 0 },
    { 0xafb4e3f8, 98 },
    { 0x14e4e3f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e401, 95 },
    { 0x14e4e402, 99 },
    { 0xc784e403, 61 },
    { 0xc404e401, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab4e408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe64e40f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7924e412, 61 },
    { 0x9094e413, 97 },
    { 0x49b4e413, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba4e419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2994e41c, 46 },
    { 0xad04e41c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4e420, 77 },
    { 0x96c4e420, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e4e425, 84 },
    { 0x4464e425, 33 },
    { 0xd394e426, 90 },
    { 0x56b4e428, 99 },
    { 0xaf24e427, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c4e433, 23 },
    { 0x4a84e434, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9274e439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44e440, 98 },
    { 0x4fc4e440, 98 },
    { 0x8d14e441, 55 },
    { 0, 0 },
    { 0xbf44e444, 90 },
    { 0, 0 },
    { 0x3944e446, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4484e44e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a4e453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3454e456, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14e459, 66 },
    { 0xdb64e45a, 23 },
    { 0x4d64e45a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04e460, 95 },
    { 0xd434e461, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa674e473, 23 },
    { 0, 0 },
    { 0x5d14e475, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e4e478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4e47b, 76 },
    { 0xe044e47b, 90 },
    { 0x9294e47d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b74e482, 46 },
    { 0, 0 },
    { 0xb9f4e484, 33 },
    { 0x4764e485, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f4e488, 23 },
    { 0xcd4e489, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d4e48d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df4e493, 90 },
    { 0, 0 },
    { 0x5014e495, 90 },
    { 0x7c94e496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1184e499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a4e49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9654e4a8, 98 },
    { 0x38a4e4a9, 23 },
    { 0, 0 },
    { 0xec4e4ab, 61 },
    { 0xd384e4ac, 33 },
    { 0, 0 },
    { 0xc544e4ae, 33 },
    { 0xcd84e4af, 67 },
    { 0x9874e4af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac4e4b4, 34 },
    { 0x2664e4b4, 33 },
    { 0x9b14e4b6, 93 },
    { 0x16d4e4b6, 33 },
    { 0x2bf4e4b7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614e4bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4604e4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f4e4c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd834e4c7, 90 },
    { 0, 0 },
    { 0x3774e4c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e4e4cd, 34 },
    { 0xbfe4e4ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed64e4d5, 17 },
    { 0x6574e4d6, 90 },
    { 0xa4d4e4d5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db4e4dd, 78 },
    { 0x6744e4de, 95 },
    { 0xefa4e4df, 90 },
    { 0x28b4e4e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4e4e5, 95 },
    { 0, 0 },
    { 0xdd04e4e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7734e4ea, 23 },
    { 0xfa44e4ea, 98 },
    { 0, 0 },
    { 0xd124e4ed, 95 },
    { 0x1f4e4ed, 98 },
    { 0x4bf4e4ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d4e4f5, 90 },
    { 0x1ee4e4f6, 100 },
    { 0x9324e4f7, 61 },
    { 0x14c4e4f8, 61 },
    { 0xb434e4f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd4e501, 76 },
    { 0xe7b4e501, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a14e505, 91 },
    { 0x634e506, 61 },
    { 0xaa24e507, 81 },
    { 0x4ac4e505, 93 },
    { 0x9ba4e505, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f74e50d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5254e515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5634e518, 76 },
    { 0xbbd4e519, 78 },
    { 0x5d64e51a, 81 },
    { 0, 0 },
    { 0xbd54e51c, 55 },
    { 0x50d4e51d, 77 },
    { 0x9414e51d, 76 },
    { 0, 0 },
    { 0x44a4e520, 33 },
    { 0, 0 },
    { 0x7454e522, 61 },
    { 0xbd14e523, 96 },
    { 0x6ad4e524, 99 },
    { 0x1c94e522, 78 },
    { 0x9844e526, 96 },
    { 0xcb44e526, 98 },
    { 0x9904e525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd4e52b, 90 },
    { 0x5864e52c, 88 },
    { 0x85d4e52c, 91 },
    { 0x6be4e52d, 90 },
    { 0xbc34e52f, 34 },
    { 0x6614e52f, 98 },
    { 0, 0 },
    { 0x4eb4e532, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd54e535, 55 },
    { 0x7034e536, 88 },
    { 0x7c64e537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d4e53b, 90 },
    { 0xcc34e53c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec4e53f, 76 },
    { 0x2a44e53f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d4e543, 61 },
    { 0x50b4e544, 61 },
    { 0, 0 },
    { 0x124e546, 98 },
    { 0x2ea4e547, 69 },
    { 0, 0 },
    { 0x2b34e549, 90 },
    { 0, 0 },
    { 0x25e4e54b, 92 },
    { 0xf9a4e54b, 46 },
    { 0xb704e54d, 90 },
    { 0, 0 },
    { 0x3574e54f, 93 },
    { 0xd114e550, 95 },
    { 0x2114e551, 93 },
    { 0x8344e552, 34 },
    { 0xbaf4e551, 95 },
    { 0x5974e554, 34 },
    { 0x5714e553, 84 },
    { 0xba04e556, 34 },
    { 0xf664e556, 33 },
    { 0, 0 },
    { 0xafd4e559, 66 },
    { 0x24e559, 77 },
    { 0x7ae4e55a, 98 },
    { 0x1c04e55c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a24e561, 77 },
    { 0, 0 },
    { 0x6854e563, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1914e56c, 84 },
    { 0x10a4e56d, 23 },
    { 0xf164e56e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d4e574, 98 },
    { 0x3424e575, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b4e578, 23 },
    { 0, 0 },
    { 0x12b4e57a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf384e57e, 23 },
    { 0xb304e57f, 61 },
    { 0x3534e57f, 33 },
    { 0xa5e4e581, 90 },
    { 0, 0 },
    { 0x1a84e583, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e34e58f, 90 },
    { 0x2154e58f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc594e595, 90 },
    { 0xca74e596, 76 },
    { 0xe094e597, 91 },
    { 0, 0 },
    { 0xa844e599, 23 },
    { 0xd824e59a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804e59f, 90 },
    { 0, 0 },
    { 0x53c4e5a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6704e5ae, 66 },
    { 0, 0 },
    { 0x5324e5b0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04e5b3, 91 },
    { 0x31d4e5b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d4e5b8, 23 },
    { 0x6924e5b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f14e5bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a4e5c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff4e5c5, 46 },
    { 0x6944e5c6, 90 },
    { 0, 0 },
    { 0x8614e5c8, 91 },
    { 0x69b4e5c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf94e5ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a4e5d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5644e5d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc4e5da, 84 },
    { 0x5f54e5db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5984e5df, 88 },
    { 0x2f14e5e0, 33 },
    { 0, 0 },
    { 0xe264e5e2, 97 },
    { 0, 0 },
    { 0xb194e5e4, 68 },
    { 0xf634e5e5, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e14e5eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3364e5f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa544e5f7, 90 },
    { 0, 0 },
    { 0x9514e5f9, 66 },
    { 0, 0 },
    { 0x5d44e5fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c4e600, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc04e603, 90 },
    { 0, 0 },
    { 0xbb34e605, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa984e608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb4e60b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d4e60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb4e612, 93 },
    { 0x3eb4e613, 34 },
    { 0x2e14e614, 93 },
    { 0x6584e615, 33 },
    { 0, 0 },
    { 0x7a94e617, 100 },
    { 0x3924e618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c54e620, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa4e624, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f4e62a, 40 },
    { 0x14c4e62a, 61 },
    { 0, 0 },
    { 0x5c94e62d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3744e63a, 61 },
    { 0x854e63a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d54e641, 97 },
    { 0, 0 },
    { 0x6344e643, 95 },
    { 0, 0 },
    { 0x96a4e645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b4e648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc204e64b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef34e651, 98 },
    { 0, 0 },
    { 0x8d24e653, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9654e665, 93 },
    { 0, 0 },
    { 0xc9d4e667, 78 },
    { 0x2be4e668, 55 },
    { 0, 0 },
    { 0x6894e66a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc444e66f, 84 },
    { 0xe5a4e66f, 33 },
    { 0x1a4e670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf64e678, 99 },
    { 0xaba4e679, 46 },
    { 0xb04e67a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd884e67e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc454e68d, 98 },
    { 0, 0 },
    { 0x79d4e68f, 90 },
    { 0, 0 },
    { 0xdaa4e691, 90 },
    { 0, 0 },
    { 0x24a4e693, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c24e696, 78 },
    { 0xae14e697, 36 },
    { 0x6fb4e698, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc4e6a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb04e6a9, 98 },
    { 0, 0 },
    { 0xb864e6ab, 33 },
    { 0x1ed4e6ac, 23 },
    { 0x22c4e6ac, 23 },
    { 0xfc84e6ae, 90 },
    { 0, 0 },
    { 0x764e6b0, 66 },
    { 0x6174e6b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef4e6b6, 95 },
    { 0x2564e6b6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544e6bb, 90 },
    { 0, 0 },
    { 0xe454e6bd, 88 },
    { 0xfd44e6bd, 46 },
    { 0x64f4e6bd, 61 },
    { 0, 0 },
    { 0xd074e6c1, 98 },
    { 0, 0 },
    { 0x98d4e6c3, 67 },
    { 0x7ae4e6c4, 23 },
    { 0x9de4e6c3, 81 },
    { 0x87e4e6c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4414e6cb, 95 },
    { 0xd374e6cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c4e6d3, 76 },
    { 0x2494e6d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b4e6dd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xab24e6e0, 40 },
    { 0xfdc4e6e0, 98 },
    { 0xa2b4e6e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc84e6e7, 33 },
    { 0xdc54e6e7, 33 },
    { 0x98d4e6e9, 52 },
    { 0x8924e6e8, 65 },
    { 0x8604e6e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b64e6ef, 66 },
    { 0, 0 },
    { 0xaeb4e6f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9184e6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e6fc, 95 },
    { 0, 0 },
    { 0x6654e6fe, 23 },
    { 0, 0 },
    { 0x7c44e700, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f64e703, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2384e70d, 61 },
    { 0, 0 },
    { 0x5804e70f, 95 },
    { 0x1894e710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a4e713, 98 },
    { 0xfd04e714, 33 },
    { 0x78a4e713, 23 },
    { 0x76c4e716, 33 },
    { 0x2064e717, 98 },
    { 0, 0 },
    { 0x4164e719, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1694e71e, 46 },
    { 0x4c44e71f, 95 },
    { 0xbc34e71e, 98 },
    { 0xdf94e71f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da4e727, 33 },
    { 0x21e4e728, 90 },
    { 0xf1c4e728, 87 },
    { 0x97e4e72a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2434e733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb034e738, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf24e73e, 66 },
    { 0, 0 },
    { 0xfc44e740, 66 },
    { 0x2904e740, 89 },
    { 0, 0 },
    { 0x58a4e743, 43 },
    { 0xe744e744, 33 },
    { 0x63d4e745, 61 },
    { 0xeaa4e746, 23 },
    { 0xec64e743, 61 },
    { 0x1f94e744, 95 },
    { 0x9374e749, 87 },
    { 0x5174e748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac4e74d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf54e753, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed4e757, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a4e75b, 33 },
    { 0xf844e75b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe474e75f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee4e762, 34 },
    { 0, 0 },
    { 0xc9a4e764, 98 },
    { 0x7c44e764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb884e76a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf64e76d, 95 },
    { 0x1784e76e, 33 },
    { 0, 0 },
    { 0x7fc4e770, 100 },
    { 0x8544e770, 34 },
    { 0, 0 },
    { 0x38e4e773, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8394e77a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f74e77f, 90 },
    { 0x34e780, 93 },
    { 0x9164e781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa04e78c, 34 },
    { 0, 0 },
    { 0xede4e78e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd454e794, 85 },
    { 0, 0 },
    { 0xb474e796, 87 },
    { 0xdf94e797, 23 },
    { 0x8ed4e798, 99 },
    { 0x1d14e797, 33 },
    { 0xc7f4e799, 95 },
    { 0x8694e798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f04e79e, 93 },
    { 0xcd44e79e, 95 },
    { 0x4914e7a0, 33 },
    { 0xc7c4e7a1, 23 },
    { 0x14f4e7a2, 43 },
    { 0, 0 },
    { 0x7af4e7a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3494e7a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4344e7af, 90 },
    { 0xa5d4e7af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c74e7b4, 95 },
    { 0, 0 },
    { 0x6084e7b6, 99 },
    { 0xb7f4e7b6, 34 },
    { 0xb054e7b8, 55 },
    { 0, 0 },
    { 0xf474e7ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8694e7bd, 55 },
    { 0x8084e7be, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd4e7ca, 46 },
    { 0, 0 },
    { 0x4aa4e7cc, 52 },
    { 0xdae4e7cd, 95 },
    { 0xf4b4e7ce, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2214e7d5, 61 },
    { 0x7a84e7d6, 92 },
    { 0xa884e7d7, 61 },
    { 0x6294e7d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac4e7de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab4e7e1, 36 },
    { 0xba4e7e2, 95 },
    { 0x5d4e7e1, 17 },
    { 0, 0 },
    { 0xef84e7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5404e7f1, 55 },
    { 0x1b74e7f2, 91 },
    { 0x7e34e7f3, 61 },
    { 0x50c4e7f4, 95 },
    { 0x58f4e7f5, 84 },
    { 0x2254e7f6, 95 },
    { 0x64f4e7f7, 23 },
    { 0x3214e7f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe984e7fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe854e806, 33 },
    { 0, 0 },
    { 0x8ec4e808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd4e80e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf4e817, 55 },
    { 0xa004e817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d4e81b, 33 },
    { 0x8f54e81c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a34e81f, 90 },
    { 0x2744e820, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d44e824, 52 },
    { 0x214e825, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054e829, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1824e82d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3794e830, 46 },
    { 0xd614e831, 95 },
    { 0x57f4e832, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f4e837, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c44e83f, 76 },
    { 0, 0 },
    { 0x1ff4e841, 90 },
    { 0xc054e841, 23 },
    { 0x9404e842, 87 },
    { 0xecf4e844, 30 },
    { 0, 0 },
    { 0xf844e846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa104e849, 95 },
    { 0x65c4e849, 76 },
    { 0, 0 },
    { 0xda64e84c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f14e858, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3874e85e, 68 },
    { 0x6484e85e, 95 },
    { 0, 0 },
    { 0x1b04e861, 90 },
    { 0xad84e862, 100 },
    { 0, 0 },
    { 0x7c94e864, 33 },
    { 0, 0 },
    { 0x3c14e866, 69 },
    { 0x254e867, 67 },
    { 0, 0 },
    { 0xbf74e869, 93 },
    { 0x88d4e86a, 90 },
    { 0xad34e86b, 33 },
    { 0xb314e86c, 95 },
    { 0xd4e86c, 98 },
    { 0xeb54e86e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4e873, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1884e877, 43 },
    { 0xf834e878, 24 },
    { 0x4934e878, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554e87c, 75 },
    { 0x34e4e87d, 91 },
    { 0xa454e87e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb884e883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5654e886, 33 },
    { 0x16e4e887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74e894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4e8a1, 95 },
    { 0x2d4e8a1, 76 },
    { 0x1ad4e8a2, 95 },
    { 0xf2a4e8a4, 88 },
    { 0xa204e8a5, 90 },
    { 0, 0 },
    { 0x2704e8a7, 95 },
    { 0x1ff4e8a8, 81 },
    { 0x2714e8a9, 91 },
    { 0xae84e8aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b24e8ae, 61 },
    { 0, 0 },
    { 0x5934e8b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb984e8bd, 76 },
    { 0, 0 },
    { 0x2f14e8bf, 76 },
    { 0, 0 },
    { 0xb4d4e8c1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6144e8c6, 93 },
    { 0, 0 },
    { 0x9e04e8c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea4e8cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f4e8d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d14e8d3, 90 },
    { 0x3414e8d3, 61 },
    { 0, 0 },
    { 0xab74e8d6, 23 },
    { 0x8de4e8d7, 95 },
    { 0xbd64e8d7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec4e8db, 78 },
    { 0xa2b4e8db, 23 },
    { 0x8384e8dd, 91 },
    { 0, 0 },
    { 0xaaf4e8df, 23 },
    { 0x59e4e8e0, 61 },
    { 0xfd84e8e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca4e8e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f4e8e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b64e8ed, 23 },
    { 0, 0 },
    { 0x2df4e8ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d34e8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc714e8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3454e8fb, 90 },
    { 0, 0 },
    { 0xcde4e8fd, 34 },
    { 0x4b24e8fe, 33 },
    { 0x3ce4e8ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac4e904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b24e90a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea4e910, 61 },
    { 0x7e44e910, 81 },
    { 0x5f4e912, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa554e915, 66 },
    { 0xdca4e916, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5164e919, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d44e91c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c84e929, 23 },
    { 0, 0 },
    { 0x8d64e92b, 23 },
    { 0x4bf4e92c, 84 },
    { 0x1204e92d, 53 },
    { 0xd9a4e92d, 40 },
    { 0x9694e92e, 33 },
    { 0x73d4e92e, 33 },
    { 0x9db4e931, 33 },
    { 0xdc14e932, 53 },
    { 0xf64e932, 90 },
    { 0xc954e934, 34 },
    { 0x94c4e934, 67 },
    { 0x4154e92c, 23 },
    { 0xe6c4e932, 90 },
    { 0xaa74e92f, 90 },
    { 0, 0 },
    { 0xef04e93a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe414e93f, 92 },
    { 0x6094e940, 34 },
    { 0, 0 },
    { 0x41e4e942, 46 },
    { 0x4734e943, 83 },
    { 0xe684e944, 78 },
    { 0x14d4e943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c4e948, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e4e94f, 23 },
    { 0xda74e950, 82 },
    { 0xf34e951, 93 },
    { 0xe814e951, 23 },
    { 0xe254e953, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5184e956, 52 },
    { 0xe094e957, 61 },
    { 0xe894e958, 33 },
    { 0xcef4e959, 81 },
    { 0xfaa4e95a, 65 },
    { 0xd3e4e95b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea4e962, 78 },
    { 0, 0 },
    { 0x6b74e964, 98 },
    { 0, 0 },
    { 0x9634e966, 33 },
    { 0x4194e967, 23 },
    { 0xfc04e966, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb44e96b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b4e973, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc484e977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa864e97a, 98 },
    { 0x504e97a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb04e980, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a64e984, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a4e987, 33 },
    { 0x8ff4e988, 43 },
    { 0, 0 },
    { 0xfdd4e98a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd4e98e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1384e993, 55 },
    { 0xbae4e994, 90 },
    { 0x9014e995, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd304e998, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe804e99d, 99 },
    { 0xb84e99e, 95 },
    { 0, 0 },
    { 0x7d64e9a0, 93 },
    { 0xf0e4e9a1, 43 },
    { 0x10e4e9a2, 90 },
    { 0x9264e9a2, 34 },
    { 0x5ed4e9a0, 99 },
    { 0xe734e9a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x774e9a8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b4e9ab, 67 },
    { 0x16c4e9ab, 61 },
    { 0x5894e9ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2514e9b0, 90 },
    { 0x7794e9b1, 99 },
    { 0, 0 },
    { 0x7fa4e9b3, 43 },
    { 0, 0 },
    { 0xcda4e9b5, 85 },
    { 0x8464e9b6, 76 },
    { 0xc254e9b5, 76 },
    { 0xcfb4e9b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa74e9ca, 95 },
    { 0, 0 },
    { 0xe404e9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8034e9d0, 33 },
    { 0xcc14e9d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4e9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac14e9da, 95 },
    { 0x384e9da, 33 },
    { 0xc584e9dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e44e9df, 85 },
    { 0, 0 },
    { 0x37b4e9e1, 23 },
    { 0xc274e9e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3154e9eb, 95 },
    { 0xf224e9ec, 23 },
    { 0, 0 },
    { 0x6fa4e9ee, 33 },
    { 0x41d4e9ef, 34 },
    { 0x3964e9f0, 33 },
    { 0x5b64e9f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd554e9f4, 90 },
    { 0xd714e9f4, 23 },
    { 0xaa24e9f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4ea02, 90 },
    { 0xb7d4ea02, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c4ea08, 61 },
    { 0x1774ea09, 75 },
    { 0x2a54ea08, 67 },
    { 0x8fa4ea09, 43 },
    { 0x3f94ea0c, 78 },
    { 0, 0 },
    { 0x5764ea0e, 55 },
    { 0xea94ea0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4344ea15, 23 },
    { 0x3164ea16, 61 },
    { 0, 0 },
    { 0x5704ea18, 56 },
    { 0x8d4ea19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7834ea1c, 55 },
    { 0xaf64ea1d, 33 },
    { 0, 0 },
    { 0xac24ea1f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f4ea23, 36 },
    { 0x99a4ea23, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0x7134ea27, 90 },
    { 0, 0 },
    { 0xc594ea29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b44ea2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b84ea32, 61 },
    { 0x2c94ea33, 90 },
    { 0xdbc4ea32, 98 },
    { 0xec74ea35, 61 },
    { 0, 0 },
    { 0x1c64ea37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe874ea44, 95 },
    { 0xaa34ea45, 95 },
    { 0x30c4ea46, 87 },
    { 0xf554ea47, 46 },
    { 0x2ca4ea48, 98 },
    { 0xfcd4ea49, 80 },
    { 0, 0 },
    { 0x85b4ea4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e4ea52, 33 },
    { 0x9254ea53, 61 },
    { 0, 0 },
    { 0x9a04ea55, 23 },
    { 0x7204ea55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4724ea5b, 43 },
    { 0x84e4ea5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f4ea60, 90 },
    { 0xe024ea61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c4ea6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e4ea6d, 17 },
    { 0, 0 },
    { 0x15d4ea6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc004ea72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6534ea79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f64ea7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94ea83, 23 },
    { 0x7af4ea84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4564ea8c, 23 },
    { 0, 0 },
    { 0xe004ea8e, 68 },
    { 0x21a4ea8f, 34 },
    { 0x5f24ea8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3754ea95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b64ea9e, 55 },
    { 0x76f4ea9f, 95 },
    { 0, 0 },
    { 0x9ba4eaa1, 76 },
    { 0x8f94eaa2, 55 },
    { 0, 0 },
    { 0x7864eaa4, 61 },
    { 0xe1b4eaa5, 90 },
    { 0x544eaa6, 61 },
    { 0x6264eaa7, 61 },
    { 0x1434eaa8, 98 },
    { 0x3e54eaa7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f4eaac, 23 },
    { 0xd804eaac, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814eab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d94eab9, 76 },
    { 0, 0 },
    { 0x1a54eabb, 34 },
    { 0, 0 },
    { 0x4584eabd, 76 },
    { 0xff64eabe, 34 },
    { 0xc494eabf, 61 },
    { 0, 0 },
    { 0xf884eac1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354eac5, 33 },
    { 0xf194eac6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb4eac9, 61 },
    { 0x3d4eaca, 95 },
    { 0x64e4eacb, 100 },
    { 0x4784eacb, 90 },
    { 0xaea4eacd, 61 },
    { 0x3484eace, 95 },
    { 0xf9c4eace, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf4ead2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f4ead6, 98 },
    { 0, 0 },
    { 0x3a14ead8, 76 },
    { 0x3e04ead9, 61 },
    { 0x9404eada, 46 },
    { 0x8724ead9, 95 },
    { 0xe34ead9, 90 },
    { 0x5724eadd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa64eae0, 90 },
    { 0xff4eae1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8454eaf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc654eaf8, 66 },
    { 0, 0 },
    { 0x6ab4eafa, 76 },
    { 0x2c24eafb, 23 },
    { 0, 0 },
    { 0x69a4eafd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a4eb03, 95 },
    { 0x3504eb04, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad14eb0c, 87 },
    { 0, 0 },
    { 0x3b94eb0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f4eb11, 23 },
    { 0, 0 },
    { 0x5774eb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd294eb16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f54eb19, 76 },
    { 0x5564eb1a, 34 },
    { 0x8914eb1a, 90 },
    { 0xbac4eb1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb4eb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee4eb2e, 87 },
    { 0x57c4eb2f, 23 },
    { 0x3134eb2f, 61 },
    { 0x2194eb2f, 92 },
    { 0, 0 },
    { 0xc1b4eb33, 35 },
    { 0x4484eb33, 66 },
    { 0xa834eb34, 100 },
    { 0xa364eb34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa874eb39, 64 },
    { 0xcd4eb3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6184eb3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c4eb44, 85 },
    { 0, 0 },
    { 0x81c4eb46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974eb4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e4eb51, 77 },
    { 0x5a94eb51, 95 },
    { 0xbf4eb53, 23 },
    { 0, 0 },
    { 0xf094eb55, 23 },
    { 0x23e4eb56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f84eb59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5344eb61, 95 },
    { 0, 0 },
    { 0xc914eb63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4eb67, 61 },
    { 0xa514eb68, 98 },
    { 0xd334eb69, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af4eb6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d4eb71, 33 },
    { 0xf6c4eb72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb64eb78, 91 },
    { 0xfb4eb79, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a4eb86, 61 },
    { 0xabc4eb86, 23 },
    { 0x50a4eb88, 98 },
    { 0xba74eb87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca4eb93, 61 },
    { 0xc1d4eb94, 75 },
    { 0x9734eb95, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4eb9b, 90 },
    { 0, 0 },
    { 0xd924eb9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef34eba2, 46 },
    { 0x2a14eba3, 95 },
    { 0, 0 },
    { 0xca94eba5, 33 },
    { 0x7aa4eba5, 43 },
    { 0xf0e4eba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef4ebab, 33 },
    { 0x5b4ebac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d4ebb2, 93 },
    { 0xe964ebb3, 100 },
    { 0, 0 },
    { 0xb4c4ebb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8354ebbd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a4ebc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9724ebc6, 98 },
    { 0, 0 },
    { 0xf444ebc8, 78 },
    { 0, 0 },
    { 0xffd4ebca, 34 },
    { 0xa434ebca, 34 },
    { 0x25a4ebcc, 17 },
    { 0, 0 },
    { 0x9b84ebce, 90 },
    { 0x8554ebcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa224ebd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d4ebdd, 33 },
    { 0x8424ebde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b4ebe2, 17 },
    { 0xe7a4ebe3, 90 },
    { 0x62c4ebe3, 33 },
    { 0x9fb4ebe3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc4ebe9, 95 },
    { 0xe574ebea, 84 },
    { 0x5054ebe9, 46 },
    { 0xefd4ebea, 33 },
    { 0xdb34ebec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed4ebf1, 33 },
    { 0x5474ebf1, 90 },
    { 0, 0 },
    { 0xbdf4ebf4, 98 },
    { 0xb084ebf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a24ebfa, 52 },
    { 0x3eb4ebfb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4744ebfe, 23 },
    { 0xe1f4ebff, 91 },
    { 0xcd14ec00, 52 },
    { 0, 0 },
    { 0x6fc4ec02, 61 },
    { 0x26e4ec03, 98 },
    { 0xd0d4ec03, 76 },
    { 0x3b64ec05, 93 },
    { 0xea64ec06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4ec0a, 95 },
    { 0, 0 },
    { 0xc524ec0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9134ec0f, 46 },
    { 0x3a44ec0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c4ec14, 90 },
    { 0, 0 },
    { 0x334ec16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524ec19, 61 },
    { 0x7814ec1a, 43 },
    { 0xa7a4ec1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a54ec1e, 90 },
    { 0xb2e4ec1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84ec23, 90 },
    { 0xd794ec24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944ec28, 95 },
    { 0, 0 },
    { 0x5974ec2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6464ec30, 97 },
    { 0, 0 },
    { 0x2784ec32, 98 },
    { 0xd2b4ec32, 23 },
    { 0x88d4ec32, 12 },
    { 0x3e34ec32, 34 },
    { 0x5844ec36, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe374ec3d, 90 },
    { 0xccc4ec3d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf044ec42, 95 },
    { 0x2a84ec43, 92 },
    { 0xaa54ec43, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1074ec4a, 76 },
    { 0xcc34ec4b, 95 },
    { 0, 0 },
    { 0x7f04ec4d, 66 },
    { 0x24b4ec4e, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a4ec51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b4ec57, 67 },
    { 0x29f4ec58, 23 },
    { 0xa414ec59, 43 },
    { 0, 0 },
    { 0x9374ec5b, 95 },
    { 0, 0 },
    { 0xd6e4ec5d, 91 },
    { 0, 0 },
    { 0xf1e4ec5f, 66 },
    { 0x95e4ec5f, 76 },
    { 0xc5c4ec5f, 34 },
    { 0, 0 },
    { 0x8774ec63, 23 },
    { 0x4f14ec64, 43 },
    { 0, 0 },
    { 0x9b34ec66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad4ec6b, 99 },
    { 0, 0 },
    { 0x9a84ec6d, 46 },
    { 0xf764ec6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3384ec72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4ec75, 98 },
    { 0, 0 },
    { 0x4194ec77, 98 },
    { 0, 0 },
    { 0xa04ec79, 55 },
    { 0x4e84ec7a, 91 },
    { 0x9fc4ec7b, 76 },
    { 0xa544ec79, 33 },
    { 0x9234ec7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f4ec81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814ec84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a4ec8a, 98 },
    { 0x37c4ec8b, 43 },
    { 0x72c4ec8b, 90 },
    { 0, 0 },
    { 0x50d4ec8e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab4ec95, 90 },
    { 0x5784ec96, 52 },
    { 0xec14ec97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814ec9a, 56 },
    { 0x7054ec9a, 98 },
    { 0x9f64ec9a, 93 },
    { 0xe994ec9d, 98 },
    { 0xeac4ec9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae64eca2, 90 },
    { 0x2314eca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb94eca8, 33 },
    { 0x98d4eca9, 87 },
    { 0x76f4eca9, 68 },
    { 0x4374eca8, 98 },
    { 0xb024ecac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1474ecb2, 90 },
    { 0xe2b4ecb3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d4ecb6, 61 },
    { 0x7144ecb6, 33 },
    { 0xfb94ecb8, 90 },
    { 0x6424ecb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2054ecbe, 90 },
    { 0xfc94ecbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd64ecc6, 95 },
    { 0xbbb4ecc6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2764ecca, 65 },
    { 0x7704eccb, 90 },
    { 0xeb44eccc, 77 },
    { 0xcea4ecca, 46 },
    { 0, 0 },
    { 0x53b4eccf, 100 },
    { 0, 0 },
    { 0xc094ecd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa454ecd5, 34 },
    { 0xb694ecd6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4ecd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc194ecdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c04ece5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d84ece8, 61 },
    { 0xcfd4ece8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe4ecec, 52 },
    { 0, 0 },
    { 0x4344ecee, 100 },
    { 0x47f4ecef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a4ecf4, 84 },
    { 0x1994ecf5, 33 },
    { 0xf734ecf5, 90 },
    { 0x3d24ecf5, 33 },
    { 0x7a54ecf8, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d4ecfb, 95 },
    { 0x24e4ecfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca4ed01, 17 },
    { 0xa804ed02, 90 },
    { 0x4a4ed03, 95 },
    { 0xe544ed04, 61 },
    { 0x1594ed03, 40 },
    { 0, 0 },
    { 0xfeb4ed07, 23 },
    { 0xd044ed08, 55 },
    { 0xbe44ed09, 95 },
    { 0x46d4ed0a, 78 },
    { 0xe864ed0b, 84 },
    { 0xab54ed0c, 38 },
    { 0, 0 },
    { 0xf6e4ed0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe54ed11, 98 },
    { 0xd5e4ed12, 88 },
    { 0x3cd4ed13, 98 },
    { 0, 0 },
    { 0xdaf4ed15, 85 },
    { 0, 0 },
    { 0xef74ed17, 95 },
    { 0, 0 },
    { 0xd864ed19, 46 },
    { 0x7be4ed1a, 23 },
    { 0x68d4ed1b, 95 },
    { 0, 0 },
    { 0x2364ed1d, 90 },
    { 0x5f74ed1d, 90 },
    { 0x90c4ed1d, 33 },
    { 0, 0 },
    { 0x7484ed21, 76 },
    { 0, 0 },
    { 0xfec4ed23, 33 },
    { 0x6594ed23, 98 },
    { 0, 0 },
    { 0x9464ed26, 61 },
    { 0xf294ed26, 33 },
    { 0, 0 },
    { 0x80f4ed29, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd124ed2c, 90 },
    { 0xdaa4ed2d, 61 },
    { 0, 0 },
    { 0xbe74ed2f, 84 },
    { 0, 0 },
    { 0xd74ed31, 95 },
    { 0, 0 },
    { 0x84a4ed33, 33 },
    { 0x2d54ed34, 23 },
    { 0xfad4ed34, 90 },
    { 0x3744ed36, 90 },
    { 0, 0 },
    { 0x4a04ed38, 23 },
    { 0x7794ed39, 87 },
    { 0x84f4ed38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d44ed41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74ed44, 95 },
    { 0, 0 },
    { 0x8714ed46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644ed4a, 34 },
    { 0, 0 },
    { 0x1084ed4c, 95 },
    { 0x1034ed4c, 100 },
    { 0x93c4ed4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb34ed52, 61 },
    { 0x5ea4ed53, 33 },
    { 0x8534ed54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8054ed57, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc4ed60, 81 },
    { 0, 0 },
    { 0x3f04ed62, 23 },
    { 0xf844ed63, 87 },
    { 0, 0 },
    { 0x9204ed65, 95 },
    { 0x7b44ed66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f4ed69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e64ed6c, 88 },
    { 0x594ed6c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9924ed70, 78 },
    { 0x204ed71, 95 },
    { 0x4684ed72, 61 },
    { 0, 0 },
    { 0xe844ed74, 61 },
    { 0, 0 },
    { 0x3824ed76, 88 },
    { 0x204ed77, 23 },
    { 0, 0 },
    { 0x1df4ed79, 95 },
    { 0x6a34ed79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7714ed82, 85 },
    { 0x75c4ed83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ed86, 95 },
    { 0x1e04ed87, 46 },
    { 0x2324ed88, 43 },
    { 0x414ed88, 61 },
    { 0x9a34ed8a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc94ed8e, 90 },
    { 0x1c94ed8f, 61 },
    { 0x224ed90, 87 },
    { 0x4b74ed91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c4ed96, 90 },
    { 0, 0 },
    { 0xec94ed98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5414eda1, 61 },
    { 0, 0 },
    { 0xa914eda3, 91 },
    { 0x29b4eda4, 43 },
    { 0xa4e4eda3, 48 },
    { 0x5cd4eda6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x244eda9, 90 },
    { 0x2ba4edaa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e94edb2, 33 },
    { 0x14edb3, 98 },
    { 0xebd4edb4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e64edb7, 84 },
    { 0x9c74edb8, 22 },
    { 0xa8d4edb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d4edc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db4edc6, 55 },
    { 0, 0 },
    { 0xc784edc8, 61 },
    { 0xf64edc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a54edcd, 76 },
    { 0, 0 },
    { 0xe864edcf, 61 },
    { 0xbca4edcf, 68 },
    { 0, 0 },
    { 0x53a4edd2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e74edd5, 81 },
    { 0x8164edd6, 33 },
    { 0xe614edd6, 90 },
    { 0x3fc4edd8, 61 },
    { 0x6fc4edd5, 23 },
    { 0, 0 },
    { 0x6a34eddb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf4edde, 99 },
    { 0x74c4eddf, 33 },
    { 0xe774ede0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d74ede9, 76 },
    { 0, 0 },
    { 0xc304edeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f24edf2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db4edf5, 90 },
    { 0xfe94edf5, 90 },
    { 0x2404edf7, 93 },
    { 0, 0 },
    { 0x434edf9, 68 },
    { 0x634edfa, 23 },
    { 0xe074edfb, 43 },
    { 0xbd54edf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e4ee01, 90 },
    { 0x1a54ee02, 90 },
    { 0x214ee02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd794ee08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a04ee0b, 61 },
    { 0x7cb4ee0b, 90 },
    { 0xbcf4ee0d, 84 },
    { 0x7074ee0e, 61 },
    { 0x5774ee0e, 93 },
    { 0, 0 },
    { 0x5ee4ee11, 99 },
    { 0, 0 },
    { 0xf654ee13, 34 },
    { 0x97c4ee14, 93 },
    { 0x9bf4ee15, 61 },
    { 0, 0 },
    { 0xa234ee17, 61 },
    { 0xc5f4ee18, 61 },
    { 0xaff4ee19, 33 },
    { 0x474ee17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f4ee23, 50 },
    { 0x1194ee24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1164ee29, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b4ee2d, 98 },
    { 0xce84ee2e, 99 },
    { 0, 0 },
    { 0x2954ee30, 91 },
    { 0xde74ee31, 90 },
    { 0xf3c4ee32, 90 },
    { 0xca44ee33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54ee37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4ee3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3604ee3d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4ee42, 88 },
    { 0x3284ee42, 98 },
    { 0x75a4ee44, 81 },
    { 0x884ee43, 23 },
    { 0, 0 },
    { 0x38d4ee47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef74ee51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd514ee56, 40 },
    { 0x1a84ee57, 33 },
    { 0xb9d4ee58, 61 },
    { 0xed24ee58, 76 },
    { 0, 0 },
    { 0x4eb4ee5b, 95 },
    { 0x8644ee5c, 80 },
    { 0xfda4ee5d, 23 },
    { 0xb9a4ee5b, 61 },
    { 0, 0 },
    { 0xfb44ee60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2644ee63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d4ee6b, 90 },
    { 0x84e4ee6b, 90 },
    { 0, 0 },
    { 0x7904ee6e, 33 },
    { 0xf734ee6e, 33 },
    { 0x2884ee70, 33 },
    { 0, 0 },
    { 0xe414ee72, 33 },
    { 0xe424ee72, 90 },
    { 0x9834ee73, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb04ee78, 61 },
    { 0, 0 },
    { 0xe074ee7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e4ee7d, 61 },
    { 0x2a64ee7d, 43 },
    { 0, 0 },
    { 0x2a74ee80, 84 },
    { 0x80f4ee81, 90 },
    { 0x3954ee80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ee86, 55 },
    { 0, 0 },
    { 0x2d44ee88, 33 },
    { 0, 0 },
    { 0xa964ee8a, 80 },
    { 0x4df4ee8b, 90 },
    { 0, 0 },
    { 0x3834ee8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d14ee91, 83 },
    { 0, 0 },
    { 0x36e4ee93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7814ee98, 98 },
    { 0x7c14ee99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf544ee9c, 33 },
    { 0xa644ee9c, 95 },
    { 0, 0 },
    { 0x1674ee9f, 23 },
    { 0x2894ee9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd84eeaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064eeb1, 24 },
    { 0x7d84eeb2, 46 },
    { 0x1f14eeb3, 91 },
    { 0, 0 },
    { 0x4904eeb5, 74 },
    { 0xc9d4eeb6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea34eebb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed4eebe, 98 },
    { 0xbe14eebf, 46 },
    { 0xd774eec0, 90 },
    { 0xaec4eec1, 90 },
    { 0x6b74eec2, 33 },
    { 0x41c4eec3, 77 },
    { 0x86b4eec0, 100 },
    { 0, 0 },
    { 0x6134eec6, 84 },
    { 0x5974eec6, 90 },
    { 0xfd44eec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d4eecb, 33 },
    { 0xfe34eecc, 34 },
    { 0, 0 },
    { 0x9574eece, 43 },
    { 0x3c64eece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a84eed2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f4eed6, 23 },
    { 0x9584eed7, 95 },
    { 0, 0 },
    { 0xe6b4eed9, 55 },
    { 0x1004eeda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b4eedd, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4eee6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1414eeea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1624eef1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf664eef4, 90 },
    { 0x8ed4eef5, 99 },
    { 0x3b34eef6, 92 },
    { 0, 0 },
    { 0x5164eef8, 100 },
    { 0x5f54eef8, 44 },
    { 0, 0 },
    { 0xba74eefb, 33 },
    { 0x2084eefc, 46 },
    { 0x1284eefc, 33 },
    { 0x5a4eefc, 33 },
    { 0x37e4eeff, 33 },
    { 0x3aa4eefe, 93 },
    { 0xe594eefb, 98 },
    { 0, 0 },
    { 0x8994ef03, 61 },
    { 0, 0 },
    { 0x5fe4ef05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb4ef0a, 61 },
    { 0xa9c4ef0b, 33 },
    { 0x8464ef0c, 23 },
    { 0x2f44ef0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xece4ef10, 95 },
    { 0xba34ef11, 95 },
    { 0xc444ef12, 33 },
    { 0x43a4ef13, 85 },
    { 0xcd44ef11, 23 },
    { 0xcc64ef15, 90 },
    { 0x1024ef15, 84 },
    { 0xe384ef17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e4ef1a, 23 },
    { 0x1d84ef1a, 40 },
    { 0, 0 },
    { 0xb814ef1d, 23 },
    { 0xf794ef1e, 33 },
    { 0, 0 },
    { 0x5f34ef20, 98 },
    { 0, 0 },
    { 0x7f74ef22, 85 },
    { 0xd374ef23, 95 },
    { 0xdd54ef24, 33 },
    { 0x1464ef25, 76 },
    { 0x77a4ef26, 87 },
    { 0x16d4ef27, 34 },
    { 0, 0 },
    { 0x4ab4ef29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce64ef2d, 61 },
    { 0x5f64ef2e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb234ef34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a4ef38, 38 },
    { 0xba64ef38, 95 },
    { 0x1824ef3a, 95 },
    { 0xd1a4ef38, 61 },
    { 0x1ee4ef3a, 46 },
    { 0xddb4ef3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa4ef46, 66 },
    { 0xf334ef47, 33 },
    { 0xf5d4ef47, 23 },
    { 0x39e4ef47, 33 },
    { 0x9cc4ef49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f4ef4f, 23 },
    { 0x4af4ef50, 87 },
    { 0, 0 },
    { 0x25d4ef52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc84ef56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e54ef59, 76 },
    { 0xe324ef5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa364ef5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf294ef61, 61 },
    { 0, 0 },
    { 0x40a4ef63, 90 },
    { 0x6a24ef64, 91 },
    { 0xefa4ef64, 33 },
    { 0xf6b4ef66, 61 },
    { 0x4984ef64, 100 },
    { 0x31d4ef65, 67 },
    { 0, 0 },
    { 0xa2c4ef6a, 99 },
    { 0xe914ef6b, 66 },
    { 0xc064ef6c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8264ef6f, 98 },
    { 0x8ba4ef70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b4ef7b, 98 },
    { 0x1f44ef7c, 90 },
    { 0xb3c4ef7b, 61 },
    { 0x5c4ef7e, 90 },
    { 0x2c74ef7f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4464ef83, 95 },
    { 0x414ef83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa934ef89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d4ef8f, 23 },
    { 0, 0 },
    { 0x1f34ef91, 23 },
    { 0x90e4ef91, 81 },
    { 0x13e4ef92, 34 },
    { 0, 0 },
    { 0xc4b4ef95, 100 },
    { 0xdb44ef96, 23 },
    { 0x1d64ef97, 55 },
    { 0, 0 },
    { 0x6364ef99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8084ef9d, 23 },
    { 0, 0 },
    { 0xd4a4ef9f, 98 },
    { 0, 0 },
    { 0x6b94efa1, 90 },
    { 0xb8f4efa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd4efa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8514efa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3984efab, 76 },
    { 0, 0 },
    { 0xe5d4efad, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8644efb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f4efb7, 61 },
    { 0, 0 },
    { 0xd154efb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794efc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7974efc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa874efc8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4084efcc, 23 },
    { 0xeae4efcd, 23 },
    { 0xcde4efcd, 23 },
    { 0xe5f4efcf, 76 },
    { 0xcd44efcf, 84 },
    { 0x6304efd0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b94efd4, 23 },
    { 0xf564efd5, 81 },
    { 0xd284efd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d94efda, 61 },
    { 0x36b4efdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9994efde, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e4efe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e4efe6, 33 },
    { 0xd6b4efe7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c04efea, 78 },
    { 0, 0 },
    { 0x3004efec, 43 },
    { 0x804efec, 61 },
    { 0x2884efee, 52 },
    { 0xd234efee, 46 },
    { 0, 0 },
    { 0x2754eff1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e4eff4, 95 },
    { 0xc8e4eff4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd064eff8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4effe, 98 },
    { 0x8b54effe, 98 },
    { 0x6bc4effe, 90 },
    { 0xbc34efff, 55 },
    { 0x39a4f002, 33 },
    { 0x48d4f000, 95 },
    { 0, 0 },
    { 0x3324f005, 61 },
    { 0, 0 },
    { 0xcd04f007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd94f00c, 46 },
    { 0, 0 },
    { 0x4264f00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2304f016, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9134f01a, 95 },
    { 0, 0 },
    { 0x12f4f01c, 61 },
    { 0, 0 },
    { 0xd514f01e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7554f023, 95 },
    { 0xc8c4f024, 90 },
    { 0x5e04f024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d84f032, 65 },
    { 0xc1f4f033, 23 },
    { 0, 0 },
    { 0xdd64f035, 93 },
    { 0xccd4f035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc384f039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc954f03d, 90 },
    { 0x4ba4f03e, 85 },
    { 0xe564f03f, 33 },
    { 0x78f4f040, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb4f043, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a04f048, 65 },
    { 0x5d54f049, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d4f050, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8814f054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f4f059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde04f05d, 90 },
    { 0x7b14f05d, 98 },
    { 0x3224f05e, 90 },
    { 0x9874f05f, 90 },
    { 0, 0 },
    { 0xb3a4f062, 52 },
    { 0x4194f063, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9864f067, 34 },
    { 0xca54f068, 61 },
    { 0x8eb4f068, 95 },
    { 0x99a4f06a, 34 },
    { 0x77b4f06b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b34f07c, 61 },
    { 0xe8e4f07c, 61 },
    { 0x2534f07e, 43 },
    { 0, 0 },
    { 0x41b4f080, 33 },
    { 0x2834f081, 85 },
    { 0x9424f081, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe884f08b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab4f092, 91 },
    { 0xf94f093, 99 },
    { 0x7b4f092, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c84f098, 33 },
    { 0xeb74f099, 65 },
    { 0, 0 },
    { 0xfb84f09b, 55 },
    { 0, 0 },
    { 0x4ba4f09d, 87 },
    { 0, 0 },
    { 0xd084f09f, 66 },
    { 0x4c64f0a0, 43 },
    { 0xc424f0a1, 34 },
    { 0xe624f0a0, 88 },
    { 0x7de4f0a0, 33 },
    { 0xc3b4f0a4, 23 },
    { 0xaf84f0a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a4f0a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9134f0ad, 55 },
    { 0x3774f0ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724f0b3, 87 },
    { 0xed84f0b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf194f0b8, 33 },
    { 0, 0 },
    { 0x224f0ba, 34 },
    { 0, 0 },
    { 0xb334f0bc, 81 },
    { 0xcc44f0bd, 55 },
    { 0, 0 },
    { 0x4504f0bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd94f0c8, 33 },
    { 0xcc44f0c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc4f0d0, 40 },
    { 0x2204f0d1, 33 },
    { 0, 0 },
    { 0xcac4f0d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb4f0d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7594f0dc, 90 },
    { 0xe604f0dc, 100 },
    { 0xd614f0de, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f0e3, 34 },
    { 0, 0 },
    { 0x4464f0e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd464f0e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9424f0ec, 98 },
    { 0, 0 },
    { 0xa4b4f0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc274f0f2, 95 },
    { 0xd1d4f0f3, 61 },
    { 0, 0 },
    { 0xa2a4f0f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2824f0fb, 23 },
    { 0x5bf4f0fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb54f102, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e4f106, 78 },
    { 0, 0 },
    { 0xb704f108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c4f10d, 33 },
    { 0, 0 },
    { 0xefc4f10f, 90 },
    { 0xbb24f110, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf414f117, 75 },
    { 0, 0 },
    { 0x16e4f119, 76 },
    { 0xbd24f119, 33 },
    { 0x47c4f11a, 90 },
    { 0, 0 },
    { 0x1c74f11d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf14f120, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa464f123, 98 },
    { 0x6794f123, 99 },
    { 0, 0 },
    { 0x4ae4f126, 33 },
    { 0x46d4f127, 66 },
    { 0, 0 },
    { 0x3754f129, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd4f12f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c4f135, 90 },
    { 0x7bd4f136, 91 },
    { 0, 0 },
    { 0xae74f138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c4f13b, 90 },
    { 0, 0 },
    { 0x59f4f13d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec74f141, 52 },
    { 0, 0 },
    { 0xefc4f143, 98 },
    { 0x7204f143, 90 },
    { 0xde74f144, 98 },
    { 0x5674f146, 100 },
    { 0xfda4f146, 17 },
    { 0x8f04f147, 90 },
    { 0xac04f149, 78 },
    { 0x7ee4f149, 56 },
    { 0x394f146, 95 },
    { 0, 0 },
    { 0xd414f14d, 91 },
    { 0x6dd4f14e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c94f151, 85 },
    { 0x7764f152, 43 },
    { 0, 0 },
    { 0x1a64f154, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e4f15a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da4f161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff4f167, 77 },
    { 0x83f4f167, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be4f16c, 76 },
    { 0x5424f16d, 98 },
    { 0xd0e4f16e, 52 },
    { 0x6044f16e, 34 },
    { 0x7704f16c, 23 },
    { 0x4ab4f171, 76 },
    { 0xcea4f171, 33 },
    { 0x69a4f16f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a4f178, 95 },
    { 0x37c4f178, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8394f17c, 98 },
    { 0x63e4f17d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb04f184, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab4f18a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca4f18e, 76 },
    { 0x95a4f18e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c94f192, 80 },
    { 0x1574f193, 53 },
    { 0, 0 },
    { 0x75b4f195, 55 },
    { 0xb4d4f195, 17 },
    { 0x6c34f197, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7364f19c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e4f1a4, 95 },
    { 0x7564f1a5, 90 },
    { 0, 0 },
    { 0x4f64f1a7, 43 },
    { 0, 0 },
    { 0xd444f1a9, 46 },
    { 0, 0 },
    { 0x85f4f1ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f24f1ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa924f1b6, 33 },
    { 0xf0c4f1b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd4f1c3, 34 },
    { 0xdc24f1c4, 98 },
    { 0xeae4f1c5, 61 },
    { 0x934f1c4, 23 },
    { 0x2a84f1c7, 33 },
    { 0xf754f1c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7384f1cb, 40 },
    { 0x25f4f1cc, 90 },
    { 0xf5f4f1cd, 76 },
    { 0x9324f1cd, 95 },
    { 0x1954f1cd, 76 },
    { 0x4c54f1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9934f1d3, 66 },
    { 0x2fd4f1d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f14f1dd, 61 },
    { 0xdae4f1de, 90 },
    { 0, 0 },
    { 0x3f44f1e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe864f1e6, 95 },
    { 0, 0 },
    { 0xe304f1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a4f1eb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc034f1ef, 90 },
    { 0x1734f1f0, 55 },
    { 0, 0 },
    { 0x27f4f1f2, 52 },
    { 0x9024f1f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7554f1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa54f1fd, 76 },
    { 0x1284f1fd, 33 },
    { 0xfa54f1ff, 33 },
    { 0xd1a4f200, 87 },
    { 0x6aa4f201, 33 },
    { 0xd654f202, 23 },
    { 0x5654f202, 61 },
    { 0x3974f204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4894f207, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524f20c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7024f211, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf794f218, 90 },
    { 0x6fe4f219, 98 },
    { 0x2ba4f21a, 67 },
    { 0x48b4f219, 33 },
    { 0x1194f21c, 61 },
    { 0, 0 },
    { 0x35d4f21e, 88 },
    { 0x4f14f21e, 23 },
    { 0, 0 },
    { 0x35d4f221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d34f22b, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504f232, 90 },
    { 0xff24f233, 34 },
    { 0x5ed4f234, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb4f237, 90 },
    { 0x99c4f238, 61 },
    { 0, 0 },
    { 0xa534f23a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4f23e, 65 },
    { 0, 0 },
    { 0xd6c4f240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4f246, 34 },
    { 0xfc14f247, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb474f24a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd4f24f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4f252, 52 },
    { 0x9e64f253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2774f256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa24f25a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504f260, 90 },
    { 0xbdf4f261, 23 },
    { 0x23d4f262, 66 },
    { 0x9c54f260, 34 },
    { 0xfbd4f264, 33 },
    { 0, 0 },
    { 0xff24f266, 23 },
    { 0, 0 },
    { 0x5ed4f268, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1154f26d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb4f273, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5044f278, 91 },
    { 0x3bb4f279, 91 },
    { 0x1754f27a, 43 },
    { 0xf774f27b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a4f284, 65 },
    { 0x2304f284, 98 },
    { 0x84b4f285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4934f28c, 53 },
    { 0, 0 },
    { 0x4a94f28e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d34f295, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde74f299, 98 },
    { 0x5764f299, 76 },
    { 0x30b4f29b, 95 },
    { 0, 0 },
    { 0x434f29d, 90 },
    { 0xa054f29e, 33 },
    { 0, 0 },
    { 0x35d4f2a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f64f2a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d4f2b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be4f2b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4904f2be, 33 },
    { 0xac04f2bf, 90 },
    { 0xa224f2c0, 23 },
    { 0xceb4f2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f4f2d2, 33 },
    { 0xc974f2d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2144f2d7, 61 },
    { 0, 0 },
    { 0x5e34f2d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe994f2e1, 33 },
    { 0, 0 },
    { 0x3814f2e3, 61 },
    { 0x5804f2e4, 90 },
    { 0x43e4f2e5, 77 },
    { 0, 0 },
    { 0xff74f2e7, 90 },
    { 0x13a4f2e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb4f2eb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4394f2ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f2f6, 34 },
    { 0, 0 },
    { 0xf584f2f8, 23 },
    { 0x5284f2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea4f2fc, 33 },
    { 0xfce4f2fd, 76 },
    { 0x7284f2fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f4f301, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd4f305, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94f30d, 34 },
    { 0x2834f30e, 91 },
    { 0x3e54f30e, 99 },
    { 0x99f4f30e, 64 },
    { 0xf534f311, 90 },
    { 0xf364f311, 55 },
    { 0xb2c4f311, 43 },
    { 0, 0 },
    { 0x6504f315, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd44f31a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd24f31d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b4f328, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d34f32b, 95 },
    { 0x3674f32c, 99 },
    { 0xd804f32d, 88 },
    { 0x5c74f32e, 65 },
    { 0, 0 },
    { 0xf8c4f330, 33 },
    { 0xd724f330, 98 },
    { 0, 0 },
    { 0x4aa4f333, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b4f338, 55 },
    { 0x89c4f339, 43 },
    { 0x6454f33a, 95 },
    { 0x4eb4f33a, 90 },
    { 0xd84f33a, 34 },
    { 0xeb44f33d, 24 },
    { 0, 0 },
    { 0xdfa4f33f, 43 },
    { 0x4e44f340, 93 },
    { 0x2d4f33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb4f346, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf984f34b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5504f34f, 46 },
    { 0xd2d4f350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6604f353, 23 },
    { 0, 0 },
    { 0x5174f355, 90 },
    { 0x3a94f356, 68 },
    { 0x5564f356, 75 },
    { 0x32f4f357, 90 },
    { 0x8934f358, 43 },
    { 0xe014f355, 98 },
    { 0x4274f356, 91 },
    { 0x7d94f35c, 69 },
    { 0, 0 },
    { 0x38f4f35e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b4f361, 77 },
    { 0xeec4f361, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e4f365, 83 },
    { 0xa464f365, 55 },
    { 0, 0 },
    { 0x22c4f368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de4f36b, 93 },
    { 0, 0 },
    { 0x65e4f36d, 34 },
    { 0x3414f36e, 76 },
    { 0x3044f36d, 33 },
    { 0, 0 },
    { 0x5534f371, 37 },
    { 0x2ed4f372, 91 },
    { 0x4144f372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc434f378, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a4f37c, 78 },
    { 0xfbe4f37d, 76 },
    { 0xd564f37e, 52 },
    { 0x7264f37d, 55 },
    { 0, 0 },
    { 0x244f381, 76 },
    { 0x9c74f382, 61 },
    { 0x6724f383, 87 },
    { 0x9124f384, 95 },
    { 0xbd44f382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d4f38a, 97 },
    { 0x5b54f38b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c4f391, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c64f39e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d04f3a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a54f3a4, 61 },
    { 0x9724f3a4, 90 },
    { 0xaa74f3a6, 95 },
    { 0, 0 },
    { 0x2624f3a8, 22 },
    { 0x7684f3a8, 98 },
    { 0x7c34f3aa, 90 },
    { 0x1bc4f3aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda34f3af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xea04f3b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf4f3b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d14f3ba, 9 },
    { 0xffc4f3ba, 66 },
    { 0x1834f3bc, 90 },
    { 0x57c4f3bc, 93 },
    { 0xa444f3bd, 98 },
    { 0x1e34f3bc, 33 },
    { 0xe284f3c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6614f3c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6224f3c6, 90 },
    { 0x93f4f3c7, 33 },
    { 0xc544f3c8, 61 },
    { 0xebf4f3c9, 90 },
    { 0x9774f3ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d14f3cd, 43 },
    { 0xb264f3ce, 61 },
    { 0x9b14f3cf, 95 },
    { 0xfc94f3d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a44f3d3, 99 },
    { 0x88f4f3d4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb244f3d7, 88 },
    { 0, 0 },
    { 0xd204f3d9, 34 },
    { 0x6e64f3d9, 98 },
    { 0x8c84f3db, 17 },
    { 0x47a4f3dc, 98 },
    { 0, 0 },
    { 0x4e64f3de, 33 },
    { 0xdb74f3df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf714f3e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed74f3e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf584f3e9, 91 },
    { 0xd554f3ea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b4f3f1, 61 },
    { 0x6d04f3f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa4f3f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c4f3f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a64f3ff, 93 },
    { 0x6b04f400, 65 },
    { 0x3934f401, 33 },
    { 0xe114f401, 98 },
    { 0x2c64f403, 87 },
    { 0xe344f404, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb94f408, 77 },
    { 0, 0 },
    { 0x81d4f40a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b4f415, 23 },
    { 0x21b4f416, 23 },
    { 0x6fa4f417, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f4f41a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3684f41d, 91 },
    { 0, 0 },
    { 0xa384f41f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf994f423, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e4f427, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8284f42a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac54f42d, 34 },
    { 0x63b4f42e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9234f433, 34 },
    { 0, 0 },
    { 0xa7d4f435, 76 },
    { 0xcde4f436, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e4f441, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9704f444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8254f44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e64f457, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a4f45c, 55 },
    { 0xdf94f45d, 43 },
    { 0x2184f45e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a4f465, 33 },
    { 0xfe24f466, 33 },
    { 0, 0 },
    { 0x6114f468, 33 },
    { 0x9714f469, 55 },
    { 0xf924f46a, 88 },
    { 0xac44f468, 55 },
    { 0xac34f469, 90 },
    { 0xace4f46d, 61 },
    { 0x3c94f468, 85 },
    { 0, 0 },
    { 0x3ae4f470, 23 },
    { 0x95d4f470, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f34f479, 100 },
    { 0xc314f47a, 61 },
    { 0xad84f47a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4384f47f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc814f484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f74f488, 88 },
    { 0xc914f488, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5524f490, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xff94f493, 61 },
    { 0x1c24f494, 33 },
    { 0, 0 },
    { 0xfbb4f496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2604f499, 90 },
    { 0x7644f49a, 90 },
    { 0xc9b4f49b, 43 },
    { 0, 0 },
    { 0x4364f49d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1664f4a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e44f4a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd4f4ac, 90 },
    { 0xfa4f4ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb64f4b0, 99 },
    { 0xa994f4b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf34f4b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c4f4be, 33 },
    { 0xc3a4f4be, 90 },
    { 0x4b24f4c0, 61 },
    { 0xe2e4f4c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4f4c8, 55 },
    { 0x9d24f4c8, 84 },
    { 0xf224f4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff4f4cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e44f4d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa914f4e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8354f4e6, 67 },
    { 0, 0 },
    { 0xf904f4e8, 98 },
    { 0, 0 },
    { 0x8464f4ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7734f4f2, 33 },
    { 0, 0 },
    { 0x89b4f4f4, 61 },
    { 0x1274f4f5, 33 },
    { 0x2144f4f6, 93 },
    { 0xeb94f4f7, 90 },
    { 0x9634f4f8, 55 },
    { 0xfd34f4f9, 97 },
    { 0x2cd4f4fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be4f509, 43 },
    { 0x3fa4f50a, 76 },
    { 0x4c94f50b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca24f511, 90 },
    { 0, 0 },
    { 0xb14f513, 93 },
    { 0, 0 },
    { 0xb894f515, 23 },
    { 0x6984f515, 33 },
    { 0, 0 },
    { 0x43e4f518, 95 },
    { 0xc784f519, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa524f521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd424f524, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe84f529, 90 },
    { 0, 0 },
    { 0x9064f52b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4944f533, 61 },
    { 0x2c94f534, 90 },
    { 0x29d4f535, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd504f538, 61 },
    { 0, 0 },
    { 0xd364f53a, 91 },
    { 0, 0 },
    { 0xc444f53c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc14f544, 90 },
    { 0x4744f545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c4f548, 76 },
    { 0x9224f549, 33 },
    { 0, 0 },
    { 0x3ff4f54b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc414f54f, 61 },
    { 0, 0 },
    { 0x4244f551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b94f554, 33 },
    { 0xe2a4f554, 90 },
    { 0x9b94f555, 32 },
    { 0xd404f557, 76 },
    { 0x6854f558, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd604f55b, 99 },
    { 0, 0 },
    { 0x61f4f55d, 34 },
    { 0, 0 },
    { 0x2114f55f, 99 },
    { 0, 0 },
    { 0x6804f561, 76 },
    { 0x2274f561, 34 },
    { 0x904f563, 76 },
    { 0x14c4f562, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf704f567, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f54f56c, 61 },
    { 0x3c94f56c, 33 },
    { 0, 0 },
    { 0xa374f56f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf324f578, 75 },
    { 0, 0 },
    { 0x9974f57a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d84f57e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5404f583, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb694f586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e4f58f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe914f597, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074f59b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9264f5a2, 76 },
    { 0, 0 },
    { 0x4bc4f5a4, 45 },
    { 0xa444f5a5, 100 },
    { 0xdef4f5a5, 66 },
    { 0x6514f5a7, 61 },
    { 0, 0 },
    { 0x8e4f5a9, 90 },
    { 0xc444f5aa, 33 },
    { 0x3724f5ab, 90 },
    { 0, 0 },
    { 0x6f64f5ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c4f5b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1374f5b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa674f5b6, 76 },
    { 0x7794f5b6, 65 },
    { 0, 0 },
    { 0x6614f5b9, 33 },
    { 0xea4f5b9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4f5be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb304f5c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7674f5c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb4f5c9, 61 },
    { 0x81e4f5ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74f5d1, 46 },
    { 0xe384f5d2, 33 },
    { 0xb9b4f5d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f4f5d6, 95 },
    { 0x7d74f5d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb4f5da, 90 },
    { 0, 0 },
    { 0xd474f5dc, 100 },
    { 0, 0 },
    { 0x1144f5de, 52 },
    { 0, 0 },
    { 0x6c24f5e0, 52 },
    { 0x4184f5e1, 98 },
    { 0xc194f5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd224f5e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc664f5eb, 81 },
    { 0xbbd4f5ec, 72 },
    { 0, 0 },
    { 0xe124f5ee, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe104f5f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd4f5f5, 98 },
    { 0, 0 },
    { 0xa5e4f5f7, 78 },
    { 0x9514f5f8, 90 },
    { 0xa8a4f5f7, 95 },
    { 0xa354f5f8, 66 },
    { 0, 0 },
    { 0xb904f5fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f4f604, 84 },
    { 0xcbe4f605, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d94f60c, 95 },
    { 0x8df4f60d, 46 },
    { 0, 0 },
    { 0x4694f60f, 76 },
    { 0xb2a4f610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7044f614, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d44f618, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc64f61f, 34 },
    { 0x2984f61f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b54f623, 52 },
    { 0x9e04f623, 52 },
    { 0x5e4f625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb4f628, 68 },
    { 0, 0 },
    { 0xccd4f62a, 85 },
    { 0x5e4f62b, 55 },
    { 0xdd24f62c, 68 },
    { 0x64b4f62d, 40 },
    { 0xa354f62b, 95 },
    { 0x8044f62f, 23 },
    { 0xa7c4f630, 61 },
    { 0, 0 },
    { 0x4c04f632, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e4f635, 93 },
    { 0x8614f635, 61 },
    { 0, 0 },
    { 0xfb54f638, 98 },
    { 0x42a4f638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94f63d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3324f642, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f646, 99 },
    { 0x8324f647, 85 },
    { 0x5ae4f648, 61 },
    { 0x1414f647, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf164f652, 99 },
    { 0, 0 },
    { 0x7f24f654, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf864f65a, 98 },
    { 0xcc94f65b, 97 },
    { 0xc7a4f65a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4f661, 33 },
    { 0, 0 },
    { 0xd994f663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f4f66b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44f66e, 68 },
    { 0, 0 },
    { 0xfa4f670, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f04f679, 46 },
    { 0, 0 },
    { 0xfea4f67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb4f67e, 95 },
    { 0x9cd4f67e, 95 },
    { 0, 0 },
    { 0x2724f681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7794f687, 98 },
    { 0xc24f687, 33 },
    { 0, 0 },
    { 0x844f68a, 90 },
    { 0xf564f68a, 75 },
    { 0x67e4f68c, 90 },
    { 0xb544f68d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5434f690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe574f693, 33 },
    { 0xf0a4f694, 90 },
    { 0, 0 },
    { 0xd074f696, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb4f69a, 36 },
    { 0, 0 },
    { 0x81a4f69c, 61 },
    { 0xc5e4f69c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce54f6a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa124f6a6, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34f6aa, 67 },
    { 0, 0 },
    { 0xc824f6ac, 34 },
    { 0x96e4f6ac, 34 },
    { 0xeb94f6ac, 17 },
    { 0x5614f6af, 98 },
    { 0xf234f6ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a4f6b4, 61 },
    { 0x7314f6b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f4f6b9, 99 },
    { 0x7984f6ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa434f6be, 33 },
    { 0, 0 },
    { 0x63c4f6c0, 66 },
    { 0xd714f6c0, 90 },
    { 0xee24f6c0, 53 },
    { 0, 0 },
    { 0x9784f6c4, 98 },
    { 0, 0 },
    { 0x1b94f6c6, 98 },
    { 0xad54f6c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd24f6cb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba4f6cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b24f6d2, 46 },
    { 0xa764f6d3, 61 },
    { 0, 0 },
    { 0x7e4f6d5, 34 },
    { 0xf414f6d5, 98 },
    { 0, 0 },
    { 0x1804f6d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf534f6db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x254f6de, 46 },
    { 0x4874f6df, 95 },
    { 0x7f74f6df, 76 },
    { 0x664f6e0, 46 },
    { 0x8074f6e2, 52 },
    { 0x70a4f6de, 43 },
    { 0x72a4f6e4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf544f6eb, 90 },
    { 0xb914f6ec, 90 },
    { 0, 0 },
    { 0x8e54f6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3354f6f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd4f6f6, 95 },
    { 0xc54f6f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf144f6fb, 98 },
    { 0xa944f6fc, 43 },
    { 0xd754f6fd, 88 },
    { 0x40f4f6fd, 46 },
    { 0xdc34f6fb, 61 },
    { 0, 0 },
    { 0x7774f701, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb4f70e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed4f711, 61 },
    { 0, 0 },
    { 0xb984f713, 56 },
    { 0, 0 },
    { 0x3094f715, 88 },
    { 0xe844f715, 34 },
    { 0x75e4f717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af4f71b, 52 },
    { 0xb904f71b, 34 },
    { 0x43f4f71c, 55 },
    { 0xf994f71e, 61 },
    { 0x7554f71e, 67 },
    { 0x4f4f71d, 95 },
    { 0xe94f721, 33 },
    { 0, 0 },
    { 0x6234f723, 33 },
    { 0x7124f724, 61 },
    { 0x1fa4f725, 95 },
    { 0x5da4f724, 93 },
    { 0x3824f723, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xc654f72a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab44f731, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a4f736, 90 },
    { 0xb274f737, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc4f73c, 91 },
    { 0x7494f73c, 61 },
    { 0x5704f73d, 98 },
    { 0x3314f73f, 43 },
    { 0x9254f73f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f4f747, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4f74d, 33 },
    { 0xe604f74d, 40 },
    { 0, 0 },
    { 0xb1a4f750, 84 },
    { 0x8cc4f751, 95 },
    { 0, 0 },
    { 0xee4f753, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c4f756, 84 },
    { 0, 0 },
    { 0x1714f758, 61 },
    { 0x9ee4f759, 76 },
    { 0x9654f75a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf24f75d, 98 },
    { 0, 0 },
    { 0xe1c4f75f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b4f767, 95 },
    { 0, 0 },
    { 0xb8b4f769, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c4f76f, 90 },
    { 0x90e4f76f, 95 },
    { 0, 0 },
    { 0x3d54f772, 43 },
    { 0xcab4f773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa4f778, 80 },
    { 0x1f14f778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f4f780, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe064f785, 33 },
    { 0x1624f786, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3064f789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f4f78c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f4f790, 90 },
    { 0x5774f791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24f794, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e4f79c, 34 },
    { 0x1e84f79c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f7a3, 100 },
    { 0xbbf4f7a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc4f7a7, 90 },
    { 0, 0 },
    { 0xb44f7a9, 34 },
    { 0x3fc4f7aa, 34 },
    { 0, 0 },
    { 0xd7f4f7ac, 43 },
    { 0x4414f7ac, 90 },
    { 0xe2c4f7ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c64f7b2, 46 },
    { 0x164f7b2, 98 },
    { 0xa4d4f7b2, 76 },
    { 0x7fd4f7b2, 90 },
    { 0x98d4f7b6, 52 },
    { 0x6d4f7b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6124f7bb, 36 },
    { 0x4d24f7bb, 90 },
    { 0x2eb4f7bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9554f7c0, 98 },
    { 0xd914f7c1, 23 },
    { 0xa184f7c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6284f7c5, 17 },
    { 0xbc4f7c6, 23 },
    { 0x23e4f7c6, 77 },
    { 0x7bb4f7c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb824f7cf, 88 },
    { 0xb664f7cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5664f7d3, 23 },
    { 0xd034f7d3, 55 },
    { 0x5474f7d5, 87 },
    { 0x73b4f7d5, 33 },
    { 0xaf84f7d6, 94 },
    { 0x49f4f7d4, 90 },
    { 0xb2a4f7d6, 33 },
    { 0xd1b4f7d4, 76 },
    { 0, 0 },
    { 0x71e4f7dc, 100 },
    { 0xbd64f7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb554f7e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd424f7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6484f7ed, 98 },
    { 0xdc84f7ee, 78 },
    { 0, 0 },
    { 0x94b4f7f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c4f7f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8134f7fc, 90 },
    { 0x1934f7fc, 90 },
    { 0x89d4f7fe, 98 },
    { 0x694f7fe, 95 },
    { 0x6144f7ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db4f807, 34 },
    { 0x6f4f808, 87 },
    { 0, 0 },
    { 0x5474f80a, 33 },
    { 0, 0 },
    { 0x244f80c, 33 },
    { 0xfa24f80d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb04f811, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e04f814, 8 },
    { 0, 0 },
    { 0xc454f816, 76 },
    { 0x8654f817, 33 },
    { 0x7de4f817, 65 },
    { 0xa9e4f818, 93 },
    { 0xb4a4f81a, 43 },
    { 0, 0 },
    { 0xea84f81c, 77 },
    { 0, 0 },
    { 0xbbe4f81e, 61 },
    { 0xf224f81e, 98 },
    { 0x9d34f81e, 98 },
    { 0, 0 },
    { 0x3134f822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b24f828, 75 },
    { 0, 0 },
    { 0x81f4f82a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb634f82f, 61 },
    { 0x6534f830, 23 },
    { 0xf2d4f831, 19 },
    { 0x1264f831, 78 },
    { 0x9824f833, 61 },
    { 0x5264f831, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef4f838, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef4f83d, 88 },
    { 0x55f4f83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa144f841, 90 },
    { 0xe754f842, 52 },
    { 0x55d4f843, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x704f846, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d4f849, 87 },
    { 0x98a4f84a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8914f84d, 40 },
    { 0x274f84e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e4f852, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f4f85c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d4f85f, 78 },
    { 0x58f4f860, 85 },
    { 0x1044f861, 61 },
    { 0, 0 },
    { 0x3194f863, 55 },
    { 0x2e04f863, 61 },
    { 0xa734f865, 95 },
    { 0x8744f866, 99 },
    { 0x9d64f863, 33 },
    { 0, 0 },
    { 0xc564f869, 100 },
    { 0, 0 },
    { 0x6624f86b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f54f86e, 90 },
    { 0xcc34f86f, 90 },
    { 0x604f870, 98 },
    { 0xdf34f86f, 61 },
    { 0x5714f870, 75 },
    { 0x4484f873, 99 },
    { 0x4d24f874, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f87d, 68 },
    { 0xabf4f87d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f4f888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d4f892, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d4f895, 76 },
    { 0x5214f895, 61 },
    { 0x9ac4f897, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd574f89d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f34f8a0, 98 },
    { 0xb7f4f8a1, 91 },
    { 0x1544f8a2, 98 },
    { 0, 0 },
    { 0xd1d4f8a4, 46 },
    { 0xd7b4f8a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754f8a9, 61 },
    { 0x5464f8a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b4f8ad, 100 },
    { 0, 0 },
    { 0x45e4f8af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5144f8b5, 98 },
    { 0xddd4f8b6, 23 },
    { 0xcf14f8b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1244f8bc, 76 },
    { 0x3434f8bd, 17 },
    { 0xd164f8bc, 95 },
    { 0, 0 },
    { 0x3324f8c0, 23 },
    { 0, 0 },
    { 0xf1f4f8c2, 46 },
    { 0x4214f8c3, 90 },
    { 0xb064f8c4, 40 },
    { 0xb394f8c5, 43 },
    { 0, 0 },
    { 0x6e74f8c7, 55 },
    { 0xae04f8c7, 34 },
    { 0, 0 },
    { 0xf94f8ca, 61 },
    { 0xeaa4f8cb, 38 },
    { 0xfe94f8cc, 61 },
    { 0x4614f8cd, 43 },
    { 0x1ae4f8ca, 23 },
    { 0x4dc4f8cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a74f8d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c74f8d8, 78 },
    { 0xa964f8d9, 55 },
    { 0x6614f8d8, 90 },
    { 0, 0 },
    { 0x4e04f8dc, 23 },
    { 0, 0 },
    { 0x75e4f8de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7944f8e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x774f8e5, 38 },
    { 0xdb94f8e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9894f8e9, 33 },
    { 0xe914f8ea, 33 },
    { 0xe964f8eb, 98 },
    { 0, 0 },
    { 0x5da4f8ed, 93 },
    { 0, 0 },
    { 0x4e34f8ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf124f8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4324f8f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f8fb, 75 },
    { 0x734f8fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c4f901, 100 },
    { 0, 0 },
    { 0xcbf4f903, 34 },
    { 0x1bd4f903, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c4f90a, 61 },
    { 0, 0 },
    { 0xe404f90c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b4f910, 43 },
    { 0xfc34f910, 61 },
    { 0x2e04f910, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954f91b, 23 },
    { 0x76f4f91c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3054f925, 23 },
    { 0x4cb4f926, 34 },
    { 0xcac4f927, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94f92c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4774f936, 43 },
    { 0xe954f937, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b54f93a, 46 },
    { 0x6234f93a, 91 },
    { 0x69c4f93a, 55 },
    { 0xdb94f93d, 90 },
    { 0x26a4f93d, 90 },
    { 0x9ec4f93f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4f942, 38 },
    { 0x9754f943, 33 },
    { 0x6164f944, 34 },
    { 0, 0 },
    { 0x5214f946, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4f94a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f34f94d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb34f950, 84 },
    { 0x2694f951, 61 },
    { 0xbd94f951, 33 },
    { 0x2484f951, 61 },
    { 0xdae4f954, 93 },
    { 0xca14f955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b84f958, 34 },
    { 0x7ea4f958, 76 },
    { 0x254f958, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf404f960, 78 },
    { 0x9e84f961, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e4f970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2074f974, 23 },
    { 0x9504f975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea4f97a, 95 },
    { 0, 0 },
    { 0xb524f97c, 95 },
    { 0x3504f97d, 23 },
    { 0x2c84f97d, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b34f987, 55 },
    { 0, 0 },
    { 0x73a4f989, 61 },
    { 0xe4a4f98a, 90 },
    { 0x92a4f98a, 36 },
    { 0x4cc4f98c, 98 },
    { 0xb954f98d, 90 },
    { 0xe274f98e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b84f991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f4f995, 52 },
    { 0, 0 },
    { 0x51b4f997, 43 },
    { 0, 0 },
    { 0x7004f999, 33 },
    { 0, 0 },
    { 0xb94f99b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5234f9a1, 56 },
    { 0xb1b4f9a2, 23 },
    { 0x3514f9a1, 98 },
    { 0xc0d4f9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f4f9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4f9ac, 76 },
    { 0xa384f9ac, 95 },
    { 0x5524f9ad, 34 },
    { 0x93c4f9af, 23 },
    { 0x89e4f9ad, 98 },
    { 0x29f4f9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb784f9b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa924f9ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1974f9bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7374f9c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d4f9c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4f9cb, 98 },
    { 0x8e24f9cc, 33 },
    { 0xd644f9cd, 43 },
    { 0x3f14f9ce, 90 },
    { 0, 0 },
    { 0x4924f9d0, 90 },
    { 0xeb24f9d0, 90 },
    { 0, 0 },
    { 0xd4b4f9d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee24f9d6, 61 },
    { 0xaf34f9d7, 95 },
    { 0x8d14f9d8, 33 },
    { 0xfc04f9d6, 34 },
    { 0, 0 },
    { 0xd7b4f9db, 61 },
    { 0, 0 },
    { 0x1e14f9dd, 43 },
    { 0, 0 },
    { 0x1f74f9df, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb54f9e2, 76 },
    { 0xade4f9e3, 61 },
    { 0x6914f9e4, 61 },
    { 0, 0 },
    { 0x8634f9e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d74f9ea, 100 },
    { 0, 0 },
    { 0xc054f9ec, 90 },
    { 0x1564f9ed, 23 },
    { 0xa0c4f9ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394f9f1, 67 },
    { 0x3574f9f1, 95 },
    { 0x254f9f3, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa594f9f8, 90 },
    { 0x2054f9f8, 33 },
    { 0xb6b4f9f8, 55 },
    { 0x94f9fb, 61 },
    { 0x9314f9fc, 33 },
    { 0x5754f9fb, 34 },
    { 0xd9b4f9f9, 92 },
    { 0, 0 },
    { 0x54a4fa00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6674fa04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b4fa07, 34 },
    { 0, 0 },
    { 0x6d84fa09, 95 },
    { 0xfef4fa0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e4fa0e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d4fa12, 61 },
    { 0xe804fa13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b4fa16, 33 },
    { 0x93d4fa17, 23 },
    { 0xf784fa17, 61 },
    { 0x3bc4fa19, 95 },
    { 0x48e4fa1a, 76 },
    { 0xa304fa1b, 61 },
    { 0, 0 },
    { 0xbb34fa1d, 33 },
    { 0x10e4fa1e, 66 },
    { 0xa594fa1f, 87 },
    { 0, 0 },
    { 0x3e04fa21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca74fa24, 88 },
    { 0x5ec4fa25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d04fa2b, 78 },
    { 0, 0 },
    { 0x334fa2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d24fa30, 100 },
    { 0, 0 },
    { 0x74d4fa32, 95 },
    { 0, 0 },
    { 0xa574fa34, 100 },
    { 0x23c4fa35, 61 },
    { 0xda94fa36, 76 },
    { 0xe964fa37, 23 },
    { 0, 0 },
    { 0x89c4fa39, 23 },
    { 0x28c4fa3a, 33 },
    { 0, 0 },
    { 0xcf94fa3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4fa3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a14fa42, 34 },
    { 0x8644fa42, 33 },
    { 0, 0 },
    { 0xa6a4fa45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc4fa48, 100 },
    { 0, 0 },
    { 0x7ea4fa4a, 66 },
    { 0, 0 },
    { 0x2044fa4c, 97 },
    { 0xd774fa4d, 76 },
    { 0xe504fa4d, 90 },
    { 0xfe64fa4f, 77 },
    { 0x3db4fa50, 23 },
    { 0xfee4fa4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2494fa57, 61 },
    { 0, 0 },
    { 0x6e94fa59, 23 },
    { 0, 0 },
    { 0x8024fa5b, 61 },
    { 0xcf54fa5b, 61 },
    { 0x9134fa5d, 61 },
    { 0x504fa5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf044fa61, 33 },
    { 0, 0 },
    { 0x78a4fa63, 75 },
    { 0, 0 },
    { 0x8d54fa65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa744fa69, 37 },
    { 0, 0 },
    { 0xad04fa6b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d4fa78, 81 },
    { 0xf084fa79, 90 },
    { 0x19c4fa79, 61 },
    { 0xac4fa79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94fa7e, 90 },
    { 0xc204fa7f, 23 },
    { 0xcf94fa7e, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda34fa89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c4fa8e, 61 },
    { 0, 0 },
    { 0x2ea4fa90, 46 },
    { 0x2a64fa91, 33 },
    { 0x6ad4fa92, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd994fa9a, 95 },
    { 0x1054fa9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f14fa9e, 98 },
    { 0, 0 },
    { 0x3764faa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9934faa4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe864faa9, 90 },
    { 0, 0 },
    { 0x4c44faab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb404fab0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2124fab8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4864fabc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca64fabf, 61 },
    { 0, 0 },
    { 0xa1b4fac1, 43 },
    { 0xb6f4fac1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b4fac7, 43 },
    { 0, 0 },
    { 0x8f34fac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde44fad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd4fad8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3774fadc, 55 },
    { 0x33f4fadd, 99 },
    { 0x8c34fade, 90 },
    { 0xc8f4fadd, 90 },
    { 0x24c4fae0, 81 },
    { 0x84c4fadd, 93 },
    { 0x40c4fadd, 98 },
    { 0, 0 },
    { 0x2b74fae4, 90 },
    { 0x7494fae4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc4fae8, 33 },
    { 0x8ae4fae8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf44faee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a4faf2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b74faf6, 64 },
    { 0x6094faf7, 61 },
    { 0x57b4faf8, 76 },
    { 0, 0 },
    { 0x5fc4fafa, 33 },
    { 0x2d64fafb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5824fb00, 61 },
    { 0x96a4fb01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b74fb04, 76 },
    { 0xe884fb04, 23 },
    { 0x1f44fb06, 23 },
    { 0xccc4fb05, 23 },
    { 0x8d34fb04, 93 },
    { 0, 0 },
    { 0x78c4fb0a, 67 },
    { 0, 0 },
    { 0x8144fb0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f4fb0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c4fb12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b4fb17, 23 },
    { 0, 0 },
    { 0x97c4fb19, 84 },
    { 0x91e4fb1a, 33 },
    { 0xa994fb19, 95 },
    { 0xf854fb1c, 90 },
    { 0xd484fb1d, 90 },
    { 0xab74fb19, 33 },
    { 0xe4c4fb1f, 66 },
    { 0x2a54fb1c, 100 },
    { 0x1ac4fb21, 90 },
    { 0, 0 },
    { 0x8714fb23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe444fb26, 23 },
    { 0, 0 },
    { 0x1bd4fb28, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb4fb33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b4fb3a, 93 },
    { 0xfb04fb3b, 55 },
    { 0, 0 },
    { 0x4474fb3d, 85 },
    { 0x7654fb3e, 66 },
    { 0x3414fb3e, 90 },
    { 0, 0 },
    { 0xba14fb41, 17 },
    { 0, 0 },
    { 0x45c4fb43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3054fb46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf054fb4e, 46 },
    { 0xe694fb4f, 55 },
    { 0, 0 },
    { 0xce4fb51, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3954fb54, 23 },
    { 0xa114fb54, 98 },
    { 0x7b94fb55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7134fb59, 75 },
    { 0x2ba4fb5a, 95 },
    { 0xf824fb5a, 83 },
    { 0x8e54fb5c, 55 },
    { 0x5b34fb5c, 81 },
    { 0xf714fb5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e84fb62, 90 },
    { 0x4da4fb63, 90 },
    { 0x51e4fb63, 87 },
    { 0x604fb65, 93 },
    { 0x39a4fb66, 98 },
    { 0xa404fb66, 98 },
    { 0x8694fb68, 93 },
    { 0, 0 },
    { 0x8c84fb6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb4fb6e, 55 },
    { 0x5874fb6e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf4fb72, 87 },
    { 0xe364fb72, 23 },
    { 0x6c64fb72, 33 },
    { 0, 0 },
    { 0x6244fb76, 76 },
    { 0xd184fb77, 61 },
    { 0xa664fb77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7634fb7c, 46 },
    { 0xecd4fb7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae4fb81, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9194fb8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f4fb9a, 99 },
    { 0xc214fb9b, 34 },
    { 0, 0 },
    { 0xf334fb9d, 77 },
    { 0xd654fb9e, 78 },
    { 0, 0 },
    { 0xac24fba0, 23 },
    { 0x1934fba1, 98 },
    { 0x2ba4fba0, 33 },
    { 0x8ca4fba3, 67 },
    { 0x9714fba4, 98 },
    { 0, 0 },
    { 0x7574fba6, 43 },
    { 0xcfe4fba7, 43 },
    { 0xf0e4fba8, 76 },
    { 0xc694fba9, 98 },
    { 0x7a4fbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc4fbae, 90 },
    { 0, 0 },
    { 0x32e4fbb0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5204fbb3, 67 },
    { 0x1574fbb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a4fbb7, 33 },
    { 0, 0 },
    { 0xa6f4fbb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd84fbc6, 76 },
    { 0, 0 },
    { 0x9b74fbc8, 75 },
    { 0xb0c4fbc9, 90 },
    { 0xf324fbca, 95 },
    { 0x1fa4fbcb, 61 },
    { 0xa404fbc9, 95 },
    { 0, 0 },
    { 0x4154fbce, 99 },
    { 0x37d4fbcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e4fbda, 33 },
    { 0, 0 },
    { 0x424fbdc, 88 },
    { 0xf04fbdd, 34 },
    { 0, 0 },
    { 0xdbc4fbdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c4fbe2, 99 },
    { 0x2924fbe2, 43 },
    { 0, 0 },
    { 0x3994fbe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9104fbed, 61 },
    { 0x2964fbee, 99 },
    { 0, 0 },
    { 0x9fb4fbf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb44fbf3, 94 },
    { 0x3d74fbf4, 33 },
    { 0xc334fbf5, 33 },
    { 0x7e44fbf6, 98 },
    { 0x5d44fbf7, 34 },
    { 0, 0 },
    { 0x5ba4fbf9, 33 },
    { 0xfca4fbf9, 23 },
    { 0x3c94fbfb, 67 },
    { 0, 0 },
    { 0x74fbfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b4fc01, 52 },
    { 0x2d24fc02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c24fc08, 34 },
    { 0x7b84fc09, 88 },
    { 0x5e4fc0a, 95 },
    { 0xcb84fc0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c54fc0f, 61 },
    { 0x1674fc10, 55 },
    { 0x9a94fc0f, 33 },
    { 0x90e4fc0f, 84 },
    { 0xd414fc0f, 52 },
    { 0xe744fc14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de4fc19, 93 },
    { 0xd1b4fc1a, 90 },
    { 0, 0 },
    { 0xf604fc1c, 66 },
    { 0, 0 },
    { 0x9cd4fc1e, 33 },
    { 0, 0 },
    { 0xd6e4fc20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe04fc24, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d84fc2b, 76 },
    { 0, 0 },
    { 0xcb94fc2d, 98 },
    { 0x24fc2e, 95 },
    { 0x56f4fc2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be4fc32, 99 },
    { 0x14f4fc33, 98 },
    { 0x97e4fc34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564fc39, 46 },
    { 0, 0 },
    { 0x67c4fc3b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f24fc3f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe004fc42, 85 },
    { 0, 0 },
    { 0xf994fc44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2014fc48, 84 },
    { 0x5044fc48, 61 },
    { 0xc6b4fc4a, 52 },
    { 0x1f04fc4b, 76 },
    { 0xe914fc4b, 23 },
    { 0x3a14fc4d, 90 },
    { 0x7554fc4e, 43 },
    { 0xb694fc4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb4fc55, 33 },
    { 0xb874fc56, 61 },
    { 0, 0 },
    { 0xd0e4fc58, 55 },
    { 0x3194fc59, 61 },
    { 0x3b34fc5a, 61 },
    { 0x57e4fc5b, 51 },
    { 0x33b4fc59, 46 },
    { 0xe634fc5d, 88 },
    { 0, 0 },
    { 0xc474fc5f, 43 },
    { 0x3f04fc60, 65 },
    { 0x4954fc61, 76 },
    { 0xece4fc60, 84 },
    { 0xc8d4fc63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea4fc66, 55 },
    { 0xa1d4fc67, 95 },
    { 0, 0 },
    { 0x2ef4fc69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a4fc6c, 61 },
    { 0x9964fc6d, 43 },
    { 0xc404fc6e, 76 },
    { 0x12c4fc6f, 42 },
    { 0, 0 },
    { 0x7654fc71, 90 },
    { 0xe734fc72, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db4fc76, 33 },
    { 0x49d4fc77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d94fc7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab44fc81, 61 },
    { 0x6254fc82, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a14fc85, 46 },
    { 0xe934fc85, 46 },
    { 0x42e4fc87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3604fc8d, 46 },
    { 0x654fc8e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf684fc91, 36 },
    { 0x6a24fc92, 76 },
    { 0xd5d4fc92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7354fc97, 46 },
    { 0x4dc4fc97, 90 },
    { 0x31f4fc99, 99 },
    { 0x99d4fc98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6774fc9e, 77 },
    { 0xb1f4fc9f, 67 },
    { 0x87a4fca0, 85 },
    { 0, 0 },
    { 0x9c54fca2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8444fca8, 61 },
    { 0x1424fca9, 61 },
    { 0x85b4fcaa, 98 },
    { 0x20b4fca9, 35 },
    { 0xf074fca9, 33 },
    { 0xf0c4fca8, 85 },
    { 0xa644fcae, 85 },
    { 0xec54fcae, 76 },
    { 0, 0 },
    { 0xca14fcb1, 76 },
    { 0xdfe4fcb1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f34fcb5, 95 },
    { 0x5c14fcb6, 43 },
    { 0x80a4fcb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc354fcbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b4fcc3, 90 },
    { 0, 0 },
    { 0xf74fcc5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc514fcce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d34fcd1, 99 },
    { 0x33c4fcd1, 34 },
    { 0x69b4fcd2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a24fcd6, 90 },
    { 0, 0 },
    { 0xf44fcd8, 76 },
    { 0xbd64fcd9, 76 },
    { 0x1944fcd8, 55 },
    { 0x8244fcdb, 33 },
    { 0xa524fcd9, 100 },
    { 0xe3a4fcd8, 90 },
    { 0x2fa4fcde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd4fce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5174fce4, 33 },
    { 0x8164fce5, 34 },
    { 0xc984fce5, 98 },
    { 0xdd4fce7, 98 },
    { 0xbff4fce8, 33 },
    { 0xeea4fce9, 90 },
    { 0, 0 },
    { 0x7634fceb, 85 },
    { 0x4bb4fcec, 23 },
    { 0x5344fced, 87 },
    { 0x4d04fced, 83 },
    { 0xab04fcef, 55 },
    { 0, 0 },
    { 0x5964fcf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4404fcf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e4fcfa, 91 },
    { 0, 0 },
    { 0x81e4fcfc, 92 },
    { 0, 0 },
    { 0x9d84fcfe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc14fd02, 34 },
    { 0x8784fd02, 90 },
    { 0x7c94fd04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9214fd07, 61 },
    { 0xb5f4fd08, 34 },
    { 0x75e4fd09, 90 },
    { 0xd1e4fd08, 93 },
    { 0x7af4fd0b, 43 },
    { 0x6554fd0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d4fd10, 91 },
    { 0xa4a4fd11, 87 },
    { 0xaa4fd12, 87 },
    { 0xafc4fd12, 81 },
    { 0x4294fd13, 33 },
    { 0x76d4fd11, 66 },
    { 0xc0c4fd13, 81 },
    { 0x52e4fd17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8494fd1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe394fd1f, 94 },
    { 0x4b54fd20, 96 },
    { 0x1fd4fd20, 43 },
    { 0xd1c4fd22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4174fd26, 52 },
    { 0, 0 },
    { 0x2bf4fd28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9104fd2f, 23 },
    { 0x734fd30, 61 },
    { 0x1354fd2f, 95 },
    { 0x9f34fd32, 33 },
    { 0, 0 },
    { 0xdd64fd34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d4fd38, 100 },
    { 0x61d4fd38, 90 },
    { 0x7e34fd3a, 90 },
    { 0x6f24fd3b, 23 },
    { 0xcbc4fd39, 99 },
    { 0xa6e4fd38, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a4fd40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3744fd48, 81 },
    { 0, 0 },
    { 0x6344fd4a, 46 },
    { 0x1a44fd4b, 23 },
    { 0x9564fd4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db4fd51, 55 },
    { 0, 0 },
    { 0x2814fd53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e4fd56, 23 },
    { 0xa594fd56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c4fd5a, 43 },
    { 0x7444fd5b, 90 },
    { 0xef44fd5b, 76 },
    { 0, 0 },
    { 0x5454fd5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f4fd61, 89 },
    { 0x9bd4fd62, 33 },
    { 0, 0 },
    { 0x7b14fd64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb974fd6b, 90 },
    { 0x52c4fd6b, 91 },
    { 0x8fa4fd6d, 34 },
    { 0x7184fd6e, 99 },
    { 0x2854fd6e, 55 },
    { 0x3314fd70, 60 },
    { 0xe614fd70, 23 },
    { 0xa1d4fd72, 77 },
    { 0x64b4fd72, 95 },
    { 0x4b04fd72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e4fd78, 66 },
    { 0, 0 },
    { 0x4ab4fd7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa4fd7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af4fd82, 75 },
    { 0x50b4fd82, 98 },
    { 0x74fd84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c4fd8b, 23 },
    { 0xc194fd8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab24fd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb4fd94, 90 },
    { 0xc744fd95, 92 },
    { 0, 0 },
    { 0xdef4fd97, 43 },
    { 0x7534fd97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e4fd9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e4fda7, 90 },
    { 0, 0 },
    { 0xdf44fda9, 23 },
    { 0x6a74fdaa, 87 },
    { 0xd8e4fdab, 46 },
    { 0xab84fdac, 82 },
    { 0x1004fdad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf74fdb2, 33 },
    { 0, 0 },
    { 0x3774fdb4, 95 },
    { 0x4634fdb5, 43 },
    { 0xc084fdb5, 90 },
    { 0, 0 },
    { 0x59d4fdb8, 34 },
    { 0xfa4fdb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa4fdbd, 33 },
    { 0xc094fdbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a4fdc8, 34 },
    { 0xa1f4fdc9, 55 },
    { 0x5884fdc9, 88 },
    { 0xd494fdc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a94fdce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef4fdd7, 23 },
    { 0xee54fdd8, 95 },
    { 0, 0 },
    { 0x9b64fdda, 76 },
    { 0x7cf4fddb, 80 },
    { 0x9d54fddc, 66 },
    { 0, 0 },
    { 0xb954fdde, 33 },
    { 0, 0 },
    { 0xed4fde0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed34fde6, 46 },
    { 0xed44fde6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d4fdf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b4fdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2314fdfb, 55 },
    { 0, 0 },
    { 0xd914fdfd, 98 },
    { 0x46b4fdfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9524fe03, 88 },
    { 0, 0 },
    { 0x9b04fe05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba4fe0c, 33 },
    { 0x13f4fe0d, 90 },
    { 0x2c94fe0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d54fe11, 55 },
    { 0, 0 },
    { 0x6e64fe13, 90 },
    { 0xb794fe14, 53 },
    { 0xc2b4fe13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e84fe20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe354fe2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c4fe2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b24fe32, 61 },
    { 0xea54fe33, 34 },
    { 0, 0 },
    { 0xa254fe35, 61 },
    { 0x1764fe36, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1514fe39, 69 },
    { 0x19c4fe3a, 43 },
    { 0x5714fe3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe414fe3e, 76 },
    { 0xd054fe3f, 34 },
    { 0x57a4fe3f, 95 },
    { 0, 0 },
    { 0x1e74fe42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4fe48, 92 },
    { 0xcd24fe48, 90 },
    { 0xd7f4fe48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4214fe4f, 55 },
    { 0x4974fe50, 23 },
    { 0x4344fe50, 33 },
    { 0x44d4fe50, 76 },
    { 0, 0 },
    { 0xe034fe54, 85 },
    { 0x3a4fe54, 61 },
    { 0x1534fe56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1124fe5b, 90 },
    { 0, 0 },
    { 0x4664fe5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4054fe63, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7404fe66, 91 },
    { 0x5be4fe66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc724fe6b, 43 },
    { 0x1a74fe6c, 34 },
    { 0xc8c4fe6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb174fe75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2884fe7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf154fe82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d4fe85, 61 },
    { 0xaeb4fe86, 95 },
    { 0, 0 },
    { 0x94fe88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c4fe96, 93 },
    { 0x4314fe96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e4fe9d, 85 },
    { 0xedb4fe9e, 98 },
    { 0x3bf4fe9f, 61 },
    { 0x1914fe9f, 23 },
    { 0x26c4fe9f, 98 },
    { 0x4074fea2, 33 },
    { 0xffc4fea2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d4fea6, 95 },
    { 0, 0 },
    { 0xeac4fea8, 95 },
    { 0, 0 },
    { 0xc464feaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3014feaf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f4feb8, 87 },
    { 0x2f34feb9, 43 },
    { 0, 0 },
    { 0x5894febb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f4fec4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d4fec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf014fecd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa4fed1, 100 },
    { 0x19d4fed2, 94 },
    { 0x2b84fed3, 46 },
    { 0x7fd4fed4, 59 },
    { 0x164fed5, 61 },
    { 0xa384fed6, 93 },
    { 0x3a54fed5, 93 },
    { 0xd224fed2, 33 },
    { 0x8764fed1, 17 },
    { 0x9a14fed5, 43 },
    { 0x4684fedb, 98 },
    { 0x8354fedc, 76 },
    { 0xe404fedb, 99 },
    { 0x4014fede, 34 },
    { 0x5bc4fed1, 61 },
    { 0, 0 },
    { 0x7e84fee1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea74fee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5544feeb, 84 },
    { 0, 0 },
    { 0x8e24feed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6964fef3, 23 },
    { 0xf0b4fef4, 55 },
    { 0x9ab4fef5, 67 },
    { 0xb274fef5, 97 },
    { 0x9f64fef4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad74ff02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe604ff06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf4ff09, 95 },
    { 0xd2b4ff0a, 61 },
    { 0x8814ff09, 52 },
    { 0x7724ff0c, 90 },
    { 0, 0 },
    { 0xde04ff0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd984ff13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9854ff16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe944ff1a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6854ff1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe554ff20, 90 },
    { 0x2824ff21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd144ff24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a74ff2d, 85 },
    { 0xfda4ff2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d4ff32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6274ff35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d4ff3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d14ff44, 23 },
    { 0, 0 },
    { 0xa104ff46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b4ff49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4ff51, 55 },
    { 0x3414ff52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa024ff57, 61 },
    { 0, 0 },
    { 0xa494ff59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf414ff5f, 33 },
    { 0x50c4ff60, 61 },
    { 0xc9b4ff60, 43 },
    { 0xba24ff62, 90 },
    { 0, 0 },
    { 0xbad4ff64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4954ff68, 23 },
    { 0, 0 },
    { 0x7264ff6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd994ff70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4ff73, 33 },
    { 0x4824ff74, 24 },
    { 0xe544ff74, 90 },
    { 0xfcb4ff73, 61 },
    { 0x40d4ff77, 43 },
    { 0x2a4ff78, 98 },
    { 0xe3a4ff73, 67 },
    { 0, 0 },
    { 0xb4b4ff7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc4ff7e, 75 },
    { 0x8ba4ff7f, 61 },
    { 0x3544ff80, 95 },
    { 0x2ba4ff81, 52 },
    { 0xc5b4ff7f, 43 },
    { 0xeb94ff83, 61 },
    { 0x8dd4ff84, 67 },
    { 0xb414ff83, 34 },
    { 0x1654ff86, 85 },
    { 0xf464ff81, 85 },
    { 0xe544ff83, 55 },
    { 0x63d4ff89, 90 },
    { 0x8de4ff8a, 95 },
    { 0xd104ff89, 98 },
    { 0, 0 },
    { 0x85b4ff8d, 90 },
    { 0, 0 },
    { 0x8484ff8f, 90 },
    { 0xbad4ff8f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a4ff93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf274ff96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae4ff99, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f4ff9e, 99 },
    { 0xdc84ff9f, 95 },
    { 0x3b34ffa0, 68 },
    { 0xbfb4ffa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c4ffa4, 98 },
    { 0xf034ffa5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc4ffa8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f24ffaf, 23 },
    { 0, 0 },
    { 0xd9f4ffb1, 42 },
    { 0x554ffb2, 90 },
    { 0, 0 },
    { 0x91b4ffb4, 61 },
    { 0x2404ffb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9184ffbe, 55 },
    { 0x8d14ffbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a54ffc7, 23 },
    { 0x1724ffc7, 33 },
    { 0x6794ffc8, 61 },
    { 0x4904ffca, 90 },
    { 0xb614ffca, 69 },
    { 0, 0 },
    { 0x6644ffcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de4ffd0, 90 },
    { 0, 0 },
    { 0x9e4ffd2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3444ffd5, 56 },
    { 0, 0 },
    { 0xb174ffd7, 23 },
    { 0, 0 },
    { 0x4c84ffd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b4ffdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4344ffdf, 90 },
    { 0x9a44ffdf, 23 },
    { 0, 0 },
    { 0xa2b4ffe2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324ffe8, 76 },
    { 0x99b4ffe9, 90 },
    { 0x8d34ffea, 90 },
    { 0x8db4ffe8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c4ffee, 61 },
    { 0x4e34ffef, 23 },
    { 0x20b4fff0, 46 },
    { 0xbff4fff0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf4fff8, 23 },
    { 0xa684fff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf594fffe, 95 },
    { 0x1284fffe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f50002, 33 },
    { 0xa7550003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee50007, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f5000d, 78 },
    { 0x5665000e, 90 },
    { 0xe3b5000e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc650012, 95 },
    { 0x88b50013, 90 },
    { 0xf6d50014, 61 },
    { 0, 0 },
    { 0xb2e50016, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32250021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e50024, 78 },
    { 0, 0 },
    { 0xfe950026, 76 },
    { 0x60450027, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69950033, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68550037, 84 },
    { 0, 0 },
    { 0x1b250039, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xef25003c, 90 },
    { 0xff15003c, 90 },
    { 0x22f5003d, 33 },
    { 0, 0 },
    { 0xcaa50040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32450045, 64 },
    { 0x72d50046, 98 },
    { 0, 0 },
    { 0x20250048, 95 },
    { 0xd0e50049, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5665004f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe155005a, 33 },
    { 0x5065005b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5005f, 95 },
    { 0x60a5005f, 40 },
    { 0, 0 },
    { 0xa1950062, 99 },
    { 0x80f50063, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5450068, 43 },
    { 0, 0 },
    { 0x3365006a, 23 },
    { 0x1e45006b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a50071, 93 },
    { 0xcf850071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca350075, 65 },
    { 0xa0250075, 90 },
    { 0x29a50077, 78 },
    { 0xd9950075, 33 },
    { 0x28250077, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea5007c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa250083, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d50087, 84 },
    { 0x70c50087, 61 },
    { 0xbc550087, 76 },
    { 0xa3c5008a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe50092, 33 },
    { 0xe8550093, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595009a, 67 },
    { 0x4ac5009b, 84 },
    { 0, 0 },
    { 0xe8d5009d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515500a1, 23 },
    { 0, 0 },
    { 0x47f500a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00500a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d500a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be500ae, 61 },
    { 0, 0 },
    { 0xc0500b0, 17 },
    { 0xc30500b1, 98 },
    { 0xe5c500b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0500b9, 33 },
    { 0, 0 },
    { 0x7e500bb, 23 },
    { 0x90a500bc, 76 },
    { 0xfbe500bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c500c0, 52 },
    { 0x409500c1, 23 },
    { 0xee2500c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8500c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15500ca, 87 },
    { 0x633500ca, 23 },
    { 0xb78500cc, 84 },
    { 0xa88500cb, 90 },
    { 0xdc2500ce, 61 },
    { 0x60e500cc, 93 },
    { 0x7cf500d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84500d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4500d7, 33 },
    { 0, 0 },
    { 0x96a500d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d500dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9500e0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2500e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e500eb, 95 },
    { 0x21500ec, 52 },
    { 0x5b6500ed, 33 },
    { 0, 0 },
    { 0xb03500ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f500f7, 90 },
    { 0xfc0500f8, 90 },
    { 0xda2500f8, 33 },
    { 0x3af500f8, 95 },
    { 0x7af500f8, 85 },
    { 0x43c500fc, 33 },
    { 0xc14500f9, 68 },
    { 0, 0 },
    { 0x48b500ff, 95 },
    { 0xb450100, 90 },
    { 0, 0 },
    { 0x4e450102, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8950109, 43 },
    { 0, 0 },
    { 0x5415010b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c950116, 90 },
    { 0xc6750117, 95 },
    { 0x67950116, 33 },
    { 0x97650119, 76 },
    { 0xaea50119, 34 },
    { 0x4f65011b, 23 },
    { 0x87050119, 98 },
    { 0xa9c5011a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e50120, 61 },
    { 0xd850121, 23 },
    { 0x4d850120, 34 },
    { 0, 0 },
    { 0x29f50124, 46 },
    { 0, 0 },
    { 0x21450126, 93 },
    { 0, 0 },
    { 0x76350128, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9515012d, 33 },
    { 0xf135012d, 23 },
    { 0, 0 },
    { 0xf1a50130, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98850135, 66 },
    { 0x83b50136, 69 },
    { 0x90e50136, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c95013a, 100 },
    { 0x4af5013b, 67 },
    { 0x2615013a, 43 },
    { 0x2495013b, 76 },
    { 0x9435013b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65450142, 43 },
    { 0x53250143, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe50147, 33 },
    { 0x1da50147, 90 },
    { 0xd1150149, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe5014f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe50152, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8250155, 90 },
    { 0x27950156, 98 },
    { 0, 0 },
    { 0x64b50158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15015f, 80 },
    { 0xfdd50160, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee50164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf5016a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd50170, 61 },
    { 0x79d50171, 90 },
    { 0x94950172, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f50175, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f5017c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac50180, 55 },
    { 0x46550180, 43 },
    { 0, 0 },
    { 0xeef50183, 33 },
    { 0, 0 },
    { 0xa4050185, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a50189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e65018c, 43 },
    { 0x5a65018d, 61 },
    { 0, 0 },
    { 0xb585018f, 23 },
    { 0, 0 },
    { 0x9e850191, 82 },
    { 0x50650192, 90 },
    { 0, 0 },
    { 0xcfb50194, 68 },
    { 0xf2750194, 76 },
    { 0x90950196, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a5019c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563501a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x368501a3, 75 },
    { 0x656501a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x259501a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf501aa, 78 },
    { 0x3fc501ab, 33 },
    { 0x729501ac, 34 },
    { 0x373501ad, 90 },
    { 0x2b8501ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5501b1, 81 },
    { 0, 0 },
    { 0xf6c501b3, 33 },
    { 0x59a501b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1501c0, 66 },
    { 0xa9e501c1, 93 },
    { 0x86e501c2, 61 },
    { 0, 0 },
    { 0xb65501c4, 95 },
    { 0xe69501c5, 91 },
    { 0x85b501c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e501ca, 95 },
    { 0x5c3501ca, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0501ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db501d2, 93 },
    { 0x6f501d2, 90 },
    { 0x8c6501d3, 61 },
    { 0xf6f501d2, 33 },
    { 0, 0 },
    { 0x852501d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee501db, 95 },
    { 0xc5c501dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e501df, 90 },
    { 0x9f6501e0, 90 },
    { 0x974501e1, 95 },
    { 0x8cf501e2, 34 },
    { 0xb22501df, 23 },
    { 0, 0 },
    { 0x90d501e5, 90 },
    { 0xa8e501e5, 55 },
    { 0x79d501e5, 17 },
    { 0xa8a501e6, 87 },
    { 0x2e3501e9, 87 },
    { 0, 0 },
    { 0x5b4501eb, 65 },
    { 0xb0f501ec, 23 },
    { 0xf3501ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae501f0, 98 },
    { 0, 0 },
    { 0x270501f2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b501fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff50201, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d50207, 61 },
    { 0x42b50208, 90 },
    { 0x4fc50207, 23 },
    { 0xf5f50208, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a45020d, 53 },
    { 0x4415020d, 78 },
    { 0, 0 },
    { 0x2d150210, 90 },
    { 0x84650211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6975021f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7350225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c750229, 93 },
    { 0xb2b5022a, 61 },
    { 0xb665022a, 61 },
    { 0x5022b, 52 },
    { 0x9a15022a, 19 },
    { 0, 0 },
    { 0x5655022f, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd50234, 55 },
    { 0, 0 },
    { 0x90a50236, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf5023b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5023f, 76 },
    { 0, 0 },
    { 0x6ef50241, 55 },
    { 0, 0 },
    { 0x39d50243, 97 },
    { 0, 0 },
    { 0x83950245, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c50248, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c50253, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f50257, 95 },
    { 0xb0e50257, 61 },
    { 0x6cc50259, 90 },
    { 0x10f5025a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5515026a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf15026d, 95 },
    { 0xdbc5026e, 61 },
    { 0, 0 },
    { 0x45950270, 67 },
    { 0, 0 },
    { 0xf3350272, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8350277, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf575027a, 90 },
    { 0x775027b, 95 },
    { 0x8ce5027c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1455027f, 34 },
    { 0x9ab50280, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8050285, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x63650288, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc50293, 90 },
    { 0xc5550293, 77 },
    { 0, 0 },
    { 0xde750296, 33 },
    { 0, 0 },
    { 0x1cd50298, 90 },
    { 0xab350299, 90 },
    { 0, 0 },
    { 0x915029b, 23 },
    { 0x98b5029c, 61 },
    { 0x4585029d, 23 },
    { 0x5ef5029b, 95 },
    { 0, 0 },
    { 0x1ff502a0, 33 },
    { 0xa75502a1, 85 },
    { 0x9fd502a1, 75 },
    { 0xb19502a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d502a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2502ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df502b2, 80 },
    { 0x340502b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69502b8, 34 },
    { 0, 0 },
    { 0x98b502ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd502c1, 23 },
    { 0, 0 },
    { 0xd2502c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304502c7, 90 },
    { 0xbb2502c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f502cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d502cf, 61 },
    { 0x1c7502d0, 65 },
    { 0x27e502d1, 90 },
    { 0x67c502d2, 90 },
    { 0xb6e502d3, 40 },
    { 0, 0 },
    { 0xab6502d5, 23 },
    { 0x95502d6, 61 },
    { 0x1b6502d5, 33 },
    { 0x6a1502d8, 95 },
    { 0x966502d9, 33 },
    { 0, 0 },
    { 0x105502db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f502de, 98 },
    { 0xa4b502df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01502e4, 85 },
    { 0xbdf502e4, 33 },
    { 0x77b502e6, 56 },
    { 0x4df502e7, 99 },
    { 0xf6f502e8, 46 },
    { 0xbc4502e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13502ee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2502f4, 68 },
    { 0xeac502f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e502f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1502fb, 98 },
    { 0xfe7502fb, 94 },
    { 0, 0 },
    { 0x8f5502fe, 91 },
    { 0xabd502ff, 61 },
    { 0, 0 },
    { 0x26b50301, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49650305, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a50309, 76 },
    { 0xde050309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f5030f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38950316, 98 },
    { 0x65150316, 91 },
    { 0x7b950316, 76 },
    { 0, 0 },
    { 0x2cf5031a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b65031e, 81 },
    { 0, 0 },
    { 0xf0750320, 66 },
    { 0xd2650321, 98 },
    { 0, 0 },
    { 0xae550323, 90 },
    { 0xaea50324, 34 },
    { 0, 0 },
    { 0xa8050326, 98 },
    { 0xcfa50326, 17 },
    { 0x72150328, 90 },
    { 0x7ca50329, 33 },
    { 0x8ef5032a, 23 },
    { 0x31050326, 98 },
    { 0x5325032c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e50331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96150337, 98 },
    { 0x5a050337, 61 },
    { 0x75350339, 91 },
    { 0xa1450339, 23 },
    { 0, 0 },
    { 0x8a5033c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b75033f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e50343, 33 },
    { 0xfb850344, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b50348, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e5034c, 33 },
    { 0x27f5034d, 95 },
    { 0x7e85034e, 90 },
    { 0x9ab5034e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed50352, 95 },
    { 0x67850352, 23 },
    { 0x68050353, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5445035e, 76 },
    { 0, 0 },
    { 0x43450360, 23 },
    { 0xfb450360, 95 },
    { 0x55c50361, 76 },
    { 0xfcc50362, 61 },
    { 0, 0 },
    { 0xea850365, 23 },
    { 0, 0 },
    { 0xd7b50367, 84 },
    { 0x61b50368, 78 },
    { 0x6ea50369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc5036d, 90 },
    { 0, 0 },
    { 0x5f85036f, 88 },
    { 0, 0 },
    { 0xa850371, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e50375, 93 },
    { 0xc4c50376, 98 },
    { 0xb4a50376, 90 },
    { 0xf050378, 76 },
    { 0x1d250375, 76 },
    { 0xa245037a, 55 },
    { 0x61050377, 33 },
    { 0x2b95037a, 55 },
    { 0, 0 },
    { 0xe565037e, 61 },
    { 0, 0 },
    { 0xfc050380, 99 },
    { 0xd2450380, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x92250384, 23 },
    { 0x62450385, 95 },
    { 0xcfb50386, 93 },
    { 0xbd850387, 61 },
    { 0xc1950385, 46 },
    { 0x4ef50384, 61 },
    { 0x25f5038a, 67 },
    { 0, 0 },
    { 0x76b5038c, 66 },
    { 0x5645038d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b250392, 68 },
    { 0, 0 },
    { 0x55850394, 52 },
    { 0, 0 },
    { 0xf5350396, 67 },
    { 0xc9350397, 95 },
    { 0xb5750398, 33 },
    { 0xe4550399, 53 },
    { 0, 0 },
    { 0x98d5039b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c95039f, 33 },
    { 0x9e6503a0, 98 },
    { 0x560503a0, 55 },
    { 0xdfc503a2, 84 },
    { 0x9e4503a2, 61 },
    { 0x4c9503a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d503b1, 90 },
    { 0x3f2503b2, 98 },
    { 0xce4503b1, 61 },
    { 0x91d503b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c503b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac503bf, 90 },
    { 0, 0 },
    { 0x682503c1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc503c5, 61 },
    { 0x4c9503c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14503d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92503d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0503dc, 76 },
    { 0, 0 },
    { 0x3e3503de, 91 },
    { 0x38d503de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94503e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86503ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da503f7, 93 },
    { 0xfbe503f8, 55 },
    { 0x6fa503f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de503fc, 66 },
    { 0x1e8503fd, 91 },
    { 0xaad503fd, 38 },
    { 0x31d503fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c50402, 98 },
    { 0x5ee50402, 23 },
    { 0xe9050403, 22 },
    { 0xc050405, 34 },
    { 0x14f50406, 84 },
    { 0xe9e50407, 78 },
    { 0x64a50407, 78 },
    { 0x1f050407, 23 },
    { 0x66650406, 52 },
    { 0, 0 },
    { 0x1e75040c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6650414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd50417, 33 },
    { 0xc7450418, 76 },
    { 0xbcd50417, 99 },
    { 0x2f650419, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46050421, 23 },
    { 0xe6350422, 23 },
    { 0xa650421, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f50429, 76 },
    { 0x39b5042a, 90 },
    { 0x6735042b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5042f, 83 },
    { 0, 0 },
    { 0x86450431, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b50434, 61 },
    { 0, 0 },
    { 0x53650436, 23 },
    { 0, 0 },
    { 0xb9950438, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa005043f, 53 },
    { 0, 0 },
    { 0x66150441, 38 },
    { 0xba850441, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c950445, 33 },
    { 0x14350446, 90 },
    { 0xc1750445, 90 },
    { 0, 0 },
    { 0xf7b50449, 33 },
    { 0xae450449, 90 },
    { 0x3325044a, 98 },
    { 0, 0 },
    { 0xa785044d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb850451, 78 },
    { 0xd1550452, 23 },
    { 0x90250453, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d5045a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5965045d, 46 },
    { 0x98f5045e, 88 },
    { 0x4e05045f, 90 },
    { 0xa3550460, 90 },
    { 0xfaa5045e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac050467, 23 },
    { 0xac250468, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1075046c, 99 },
    { 0, 0 },
    { 0xf245046e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc250478, 95 },
    { 0xc8550479, 17 },
    { 0, 0 },
    { 0x5835047b, 23 },
    { 0xf8a5047c, 91 },
    { 0x7f45047d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e50484, 33 },
    { 0x34050484, 98 },
    { 0, 0 },
    { 0xc8150487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf605048d, 36 },
    { 0xba35048d, 68 },
    { 0xd1c5048f, 67 },
    { 0x7c15048d, 23 },
    { 0, 0 },
    { 0x8ec50492, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9950497, 55 },
    { 0x9b950498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5155049d, 33 },
    { 0x17b5049e, 61 },
    { 0x3915049f, 95 },
    { 0, 0 },
    { 0x6cb504a1, 95 },
    { 0x649504a1, 61 },
    { 0xa8a504a2, 43 },
    { 0, 0 },
    { 0x522504a5, 90 },
    { 0xe1b504a6, 23 },
    { 0x4d504a7, 23 },
    { 0x292504a6, 84 },
    { 0x807504a9, 84 },
    { 0x576504a7, 91 },
    { 0x3cf504a9, 90 },
    { 0x8cc504a8, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f504b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610504b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1504bc, 76 },
    { 0x30a504bd, 61 },
    { 0xd11504bd, 61 },
    { 0x5e2504bd, 90 },
    { 0x46504bf, 93 },
    { 0x8ac504be, 98 },
    { 0x74504bd, 90 },
    { 0x82504c3, 98 },
    { 0x6fd504c1, 55 },
    { 0, 0 },
    { 0x23d504c6, 91 },
    { 0x8b3504c7, 90 },
    { 0x450504c8, 61 },
    { 0, 0 },
    { 0x9ab504ca, 43 },
    { 0x31504ca, 75 },
    { 0, 0 },
    { 0xe29504cd, 46 },
    { 0x67504cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0504d2, 61 },
    { 0x245504d3, 93 },
    { 0, 0 },
    { 0x415504d5, 98 },
    { 0, 0 },
    { 0x8ce504d7, 55 },
    { 0x84504d8, 55 },
    { 0x591504d7, 98 },
    { 0xc88504da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd504df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39504e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61504e8, 98 },
    { 0x6ce504e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6504f3, 46 },
    { 0xaa8504f3, 98 },
    { 0x6bd504f5, 77 },
    { 0x23a504f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7504fa, 55 },
    { 0x1a0504fa, 95 },
    { 0xe0c504fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c550500, 23 },
    { 0x92950501, 33 },
    { 0xca950502, 80 },
    { 0, 0 },
    { 0xdc650504, 55 },
    { 0, 0 },
    { 0x31c50506, 66 },
    { 0x5b650507, 90 },
    { 0, 0 },
    { 0x3c850509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca5050c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d85050f, 95 },
    { 0x15c5050f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52050515, 17 },
    { 0, 0 },
    { 0x3d650517, 46 },
    { 0x53b50517, 55 },
    { 0x90a50519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b5051e, 52 },
    { 0x9415051f, 90 },
    { 0, 0 },
    { 0xafb50521, 33 },
    { 0, 0 },
    { 0xc2950523, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f650527, 43 },
    { 0x93150527, 34 },
    { 0, 0 },
    { 0x3735052a, 33 },
    { 0, 0 },
    { 0x7d25052c, 33 },
    { 0xb265052c, 88 },
    { 0x3f65052c, 67 },
    { 0xa2f5052e, 98 },
    { 0x4b450530, 78 },
    { 0x52350531, 61 },
    { 0x94950532, 17 },
    { 0x76850532, 90 },
    { 0x57750534, 87 },
    { 0x20a50532, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47250539, 90 },
    { 0, 0 },
    { 0xbaf5053b, 61 },
    { 0x1c85053c, 55 },
    { 0xf455053d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90750541, 67 },
    { 0x71c50542, 87 },
    { 0, 0 },
    { 0xa0c50544, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c50548, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa635054c, 90 },
    { 0x7ce5054d, 90 },
    { 0xfc35054d, 90 },
    { 0x9015054f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea35055a, 67 },
    { 0x8e15055b, 43 },
    { 0xeb05055c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3850561, 61 },
    { 0, 0 },
    { 0x22e50563, 91 },
    { 0x1c750563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae450568, 46 },
    { 0, 0 },
    { 0xe105056a, 55 },
    { 0, 0 },
    { 0xc035056c, 83 },
    { 0x1175056d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac150571, 43 },
    { 0xd1750572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a50575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa085057a, 98 },
    { 0x45d5057b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4850581, 33 },
    { 0x22050581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77050589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d5058f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b550593, 95 },
    { 0x73b50594, 48 },
    { 0x23750594, 90 },
    { 0x47b50596, 87 },
    { 0xb1850594, 24 },
    { 0xdfb50598, 33 },
    { 0x26150598, 87 },
    { 0, 0 },
    { 0x7925059b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e505a0, 100 },
    { 0, 0 },
    { 0x5a6505a2, 34 },
    { 0, 0 },
    { 0x7d7505a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41505ab, 33 },
    { 0, 0 },
    { 0xe0a505ad, 65 },
    { 0, 0 },
    { 0x333505af, 95 },
    { 0xf8e505af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6505b4, 93 },
    { 0x6e6505b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa505bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413505c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a505c4, 95 },
    { 0x3fb505c5, 33 },
    { 0xd86505c6, 77 },
    { 0, 0 },
    { 0x753505c8, 98 },
    { 0x9a4505c8, 33 },
    { 0xfe1505c8, 95 },
    { 0, 0 },
    { 0x322505cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2505cf, 33 },
    { 0xdb4505cf, 66 },
    { 0xbe6505d1, 68 },
    { 0x8a3505d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298505d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1505da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x814505dd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8505e3, 76 },
    { 0x42a505e3, 85 },
    { 0x552505e4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x314505e8, 61 },
    { 0x4a7505e8, 33 },
    { 0x154505e9, 98 },
    { 0xa5f505e8, 61 },
    { 0x4bb505ec, 40 },
    { 0x740505ec, 90 },
    { 0x4d9505ee, 90 },
    { 0x4bc505ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d505f2, 90 },
    { 0, 0 },
    { 0xaf1505f4, 75 },
    { 0x5f505f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf505fa, 98 },
    { 0, 0 },
    { 0x60c505fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7050601, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c50608, 85 },
    { 0x5e850608, 90 },
    { 0, 0 },
    { 0x4875060b, 76 },
    { 0x735060c, 87 },
    { 0, 0 },
    { 0xbb35060e, 61 },
    { 0x6cf5060e, 33 },
    { 0, 0 },
    { 0xc0450611, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe850617, 88 },
    { 0, 0 },
    { 0x43250619, 23 },
    { 0xe875061a, 77 },
    { 0x6d75061b, 90 },
    { 0, 0 },
    { 0x5e45061d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f650623, 61 },
    { 0, 0 },
    { 0xc0c50625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30850628, 23 },
    { 0, 0 },
    { 0x7f15062a, 81 },
    { 0, 0 },
    { 0x4855062c, 95 },
    { 0, 0 },
    { 0x63a5062e, 98 },
    { 0, 0 },
    { 0x80950630, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a550633, 61 },
    { 0x58250634, 95 },
    { 0xb9950635, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d50638, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c5063c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd925063f, 61 },
    { 0x85950640, 23 },
    { 0x36e50641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d50646, 98 },
    { 0x4ba50646, 95 },
    { 0xe550648, 95 },
    { 0x14b50648, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575064e, 52 },
    { 0, 0 },
    { 0xfe850650, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a650653, 55 },
    { 0x3d650654, 33 },
    { 0xd9b50655, 65 },
    { 0x48c50656, 95 },
    { 0xb4850657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf5065b, 17 },
    { 0x29f5065c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5350663, 61 },
    { 0, 0 },
    { 0x20b50665, 46 },
    { 0x50150665, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a75066a, 33 },
    { 0, 0 },
    { 0xe115066c, 98 },
    { 0xdad5066d, 84 },
    { 0x3b65066d, 46 },
    { 0x67a5066d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1850673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23150676, 98 },
    { 0x3e250677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8765067a, 23 },
    { 0, 0 },
    { 0xf2c5067c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115067f, 93 },
    { 0x7b65067f, 33 },
    { 0xea150681, 85 },
    { 0x4c55067f, 95 },
    { 0x4ea50681, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4650688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5068e, 66 },
    { 0x9685068f, 61 },
    { 0x83a5068f, 97 },
    { 0x7650690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b50694, 46 },
    { 0, 0 },
    { 0xd9150696, 95 },
    { 0x3450697, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35069a, 55 },
    { 0x7d45069a, 23 },
    { 0xec45069a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754506a7, 17 },
    { 0x940506a8, 33 },
    { 0, 0 },
    { 0x17f506aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd506b2, 23 },
    { 0, 0 },
    { 0x147506b4, 90 },
    { 0xf52506b5, 90 },
    { 0x651506b4, 34 },
    { 0x2f3506b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x720506ba, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907506be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93506c3, 43 },
    { 0x888506c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac506c7, 46 },
    { 0x780506c8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94506cb, 85 },
    { 0x664506cc, 87 },
    { 0, 0 },
    { 0x7f3506ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6506d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1506d5, 99 },
    { 0x9e1506d5, 17 },
    { 0xeb7506d7, 33 },
    { 0, 0 },
    { 0xd0d506d9, 61 },
    { 0xecb506da, 66 },
    { 0xfa4506db, 76 },
    { 0, 0 },
    { 0x545506dd, 61 },
    { 0xaf6506de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f506e2, 23 },
    { 0, 0 },
    { 0x59c506e4, 61 },
    { 0x7ed506e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2506ed, 90 },
    { 0x826506ee, 23 },
    { 0x58c506ed, 81 },
    { 0x57f506ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213506f5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4506fc, 23 },
    { 0xd4506fd, 33 },
    { 0x684506fe, 43 },
    { 0, 0 },
    { 0x7b650700, 98 },
    { 0, 0 },
    { 0xbf250702, 61 },
    { 0, 0 },
    { 0xad450704, 95 },
    { 0, 0 },
    { 0xf2e50706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3745070c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6515070f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3350712, 33 },
    { 0x8d450713, 43 },
    { 0xd6d50712, 95 },
    { 0x43150715, 90 },
    { 0x72b50715, 61 },
    { 0x58550715, 46 },
    { 0xff750718, 34 },
    { 0x19950719, 55 },
    { 0x20b50719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a50720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd50727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3095072a, 90 },
    { 0xda85072b, 81 },
    { 0xf695072a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x105072f, 61 },
    { 0, 0 },
    { 0x1a150731, 46 },
    { 0, 0 },
    { 0x29450733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85073d, 61 },
    { 0xade5073e, 68 },
    { 0x3225073f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe850743, 34 },
    { 0x51850744, 95 },
    { 0xc0f50745, 76 },
    { 0x94050746, 33 },
    { 0xf0a50747, 81 },
    { 0, 0 },
    { 0x90850749, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e5074e, 90 },
    { 0x3df5074f, 23 },
    { 0x9e95074f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80450755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d850759, 77 },
    { 0x83450759, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a850764, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x92350767, 55 },
    { 0, 0 },
    { 0x9f350769, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba5076f, 23 },
    { 0xf1150770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63050774, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c850777, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9715077a, 80 },
    { 0xe625077a, 34 },
    { 0, 0 },
    { 0x89e5077d, 43 },
    { 0, 0 },
    { 0xb345077f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f50783, 90 },
    { 0x8d50784, 52 },
    { 0, 0 },
    { 0xa4750786, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x10050789, 76 },
    { 0, 0 },
    { 0x12b5078b, 65 },
    { 0xe885078c, 76 },
    { 0x6795078d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x32350790, 88 },
    { 0, 0 },
    { 0x65e50792, 61 },
    { 0xc1250793, 98 },
    { 0x99f50794, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce50798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc385079f, 61 },
    { 0, 0 },
    { 0xfb5507a1, 77 },
    { 0x4eb507a1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x290507a5, 90 },
    { 0x4a8507a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x742507a9, 95 },
    { 0x1aa507aa, 33 },
    { 0x8e7507ab, 98 },
    { 0, 0 },
    { 0xcfe507ad, 43 },
    { 0x44d507ad, 93 },
    { 0x1e0507af, 85 },
    { 0xf19507b0, 93 },
    { 0x564507af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32507b4, 88 },
    { 0, 0 },
    { 0xa75507b6, 61 },
    { 0xfd0507b7, 23 },
    { 0, 0 },
    { 0x3f2507b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22507bc, 55 },
    { 0, 0 },
    { 0x100507be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf507c2, 61 },
    { 0x986507c2, 84 },
    { 0xd8e507c4, 23 },
    { 0x64e507c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x266507c8, 43 },
    { 0, 0 },
    { 0xc10507ca, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169507ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6507d2, 34 },
    { 0xe04507d2, 34 },
    { 0, 0 },
    { 0xadf507d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b507d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3507db, 84 },
    { 0xc96507dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c507e9, 52 },
    { 0x604507ea, 98 },
    { 0, 0 },
    { 0x866507ec, 95 },
    { 0xb88507ed, 61 },
    { 0x272507ee, 76 },
    { 0, 0 },
    { 0xf5c507f0, 61 },
    { 0, 0 },
    { 0x54b507f2, 75 },
    { 0, 0 },
    { 0x4b0507f4, 90 },
    { 0xa01507f5, 95 },
    { 0, 0 },
    { 0xb3d507f7, 61 },
    { 0, 0 },
    { 0x578507f9, 67 },
    { 0x759507fa, 90 },
    { 0xef6507f9, 34 },
    { 0x91a507fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f450800, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4150803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde50808, 90 },
    { 0, 0 },
    { 0x1a15080a, 90 },
    { 0, 0 },
    { 0x8095080c, 95 },
    { 0, 0 },
    { 0xc795080e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b50813, 34 },
    { 0x70b50814, 33 },
    { 0xf3e50815, 33 },
    { 0x52c50816, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9345081d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab50821, 43 },
    { 0x32d50822, 90 },
    { 0x91050823, 100 },
    { 0x17450821, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d50827, 85 },
    { 0x7b950828, 75 },
    { 0, 0 },
    { 0xdfd5082a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a050831, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0950834, 48 },
    { 0xb4f50835, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3665083e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30750842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c850848, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa685084c, 46 },
    { 0, 0 },
    { 0xc865084e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b50851, 61 },
    { 0xdd250852, 61 },
    { 0x51c50853, 23 },
    { 0x73450853, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f50859, 100 },
    { 0x81b50859, 17 },
    { 0xda25085b, 90 },
    { 0x185085c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe5085f, 61 },
    { 0x35050860, 17 },
    { 0x3f50860, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x31550864, 77 },
    { 0, 0 },
    { 0x8e650866, 95 },
    { 0, 0 },
    { 0x86050868, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b5086b, 98 },
    { 0x4685086b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd5086f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e50875, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a5087a, 90 },
    { 0, 0 },
    { 0xf785087c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31750880, 90 },
    { 0, 0 },
    { 0x41c50882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c50885, 76 },
    { 0, 0 },
    { 0xa3550887, 57 },
    { 0xe1250888, 84 },
    { 0, 0 },
    { 0xa865088a, 33 },
    { 0x7315088a, 46 },
    { 0x7c15088a, 95 },
    { 0x9b85088b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c50894, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe575089b, 43 },
    { 0x4455089b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671508a0, 91 },
    { 0x553508a1, 95 },
    { 0x30e508a1, 33 },
    { 0xe7d508a3, 61 },
    { 0x119508a4, 76 },
    { 0, 0 },
    { 0xc82508a6, 61 },
    { 0xb1f508a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b508ab, 98 },
    { 0xfa3508ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48508af, 90 },
    { 0x9fc508b0, 61 },
    { 0x4c508b1, 99 },
    { 0, 0 },
    { 0xb21508b3, 90 },
    { 0, 0 },
    { 0x7d3508b5, 23 },
    { 0, 0 },
    { 0xb93508b7, 76 },
    { 0, 0 },
    { 0x844508b9, 98 },
    { 0, 0 },
    { 0x29b508bb, 55 },
    { 0, 0 },
    { 0x8b5508bd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e508c0, 95 },
    { 0xd8a508c1, 80 },
    { 0x539508c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c508c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731508c9, 34 },
    { 0xd508ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1508ce, 81 },
    { 0xb40508cf, 52 },
    { 0xbe0508d0, 69 },
    { 0x2a5508cf, 76 },
    { 0xa3a508d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331508d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321508db, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x641508de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85508e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1508e9, 61 },
    { 0xcf1508e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759508f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4508f9, 90 },
    { 0x597508fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58950900, 98 },
    { 0xe5b50901, 52 },
    { 0xf5550900, 55 },
    { 0xe4850903, 95 },
    { 0x97950904, 90 },
    { 0, 0 },
    { 0xa8250906, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98650909, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc50915, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25050918, 90 },
    { 0x4ce50919, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6905091c, 34 },
    { 0x33e5091d, 87 },
    { 0x34c5091e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f550921, 33 },
    { 0xffb50921, 55 },
    { 0xe5d50923, 95 },
    { 0x32850924, 34 },
    { 0x7e150925, 61 },
    { 0xae50926, 46 },
    { 0xf2a50927, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35092f, 91 },
    { 0x1075092f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26750933, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6750938, 43 },
    { 0x3ba50938, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda05093d, 55 },
    { 0, 0 },
    { 0x5605093f, 69 },
    { 0x5a55093f, 90 },
    { 0x39550940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c50945, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77650948, 91 },
    { 0xe6e50948, 17 },
    { 0x41c5094a, 95 },
    { 0x7f650948, 61 },
    { 0xca85094c, 90 },
    { 0xa905094d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5950952, 61 },
    { 0xa3750952, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e50956, 23 },
    { 0x47d50957, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c5095a, 95 },
    { 0, 0 },
    { 0xfa35095c, 100 },
    { 0, 0 },
    { 0xeec5095e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba50961, 46 },
    { 0x38e50961, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c50966, 61 },
    { 0xa4750966, 46 },
    { 0x45b50967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2005096c, 33 },
    { 0xce45096d, 23 },
    { 0xa325096c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e850971, 90 },
    { 0x99850971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d750979, 61 },
    { 0xa865097a, 33 },
    { 0x765097b, 66 },
    { 0x49e5097c, 68 },
    { 0x8985097a, 90 },
    { 0xfa05097e, 90 },
    { 0xd5097e, 61 },
    { 0xb650980, 80 },
    { 0x5b650981, 33 },
    { 0xbe15097f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f50986, 23 },
    { 0xfb50986, 91 },
    { 0x54950988, 66 },
    { 0, 0 },
    { 0xec35098a, 52 },
    { 0xeea5098b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd5098e, 77 },
    { 0, 0 },
    { 0x52050990, 76 },
    { 0, 0 },
    { 0xcae50992, 90 },
    { 0xd0450992, 34 },
    { 0, 0 },
    { 0xf9550995, 98 },
    { 0x30f50995, 67 },
    { 0x9a350995, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f5099c, 85 },
    { 0x7315099d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4509a0, 95 },
    { 0, 0 },
    { 0x62d509a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57509a6, 33 },
    { 0x34a509a6, 23 },
    { 0x1cc509a6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98509af, 88 },
    { 0xede509b0, 90 },
    { 0x6d5509b1, 52 },
    { 0x58509b2, 52 },
    { 0x488509b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab509ba, 91 },
    { 0, 0 },
    { 0x19b509bc, 61 },
    { 0, 0 },
    { 0x727509be, 90 },
    { 0x52d509be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0509c3, 55 },
    { 0, 0 },
    { 0x62b509c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5509cc, 90 },
    { 0, 0 },
    { 0x21509ce, 93 },
    { 0x161509cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f509d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416509d9, 61 },
    { 0xd04509da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db509e0, 90 },
    { 0x8f8509e1, 23 },
    { 0x49c509e0, 61 },
    { 0, 0 },
    { 0x264509e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae509e9, 23 },
    { 0xe94509ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f509ed, 43 },
    { 0x2f4509ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f509f1, 95 },
    { 0x3ce509f2, 55 },
    { 0xe1e509f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382509fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c50a01, 33 },
    { 0, 0 },
    { 0xb8a50a03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81550a08, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e50a0b, 61 },
    { 0x16250a0b, 55 },
    { 0x55550a0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31850a10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c450a14, 23 },
    { 0x4b550a14, 61 },
    { 0xce150a14, 61 },
    { 0x25a50a14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff50a1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a50a20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd050a23, 43 },
    { 0, 0 },
    { 0x85550a25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40750a28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db50a35, 76 },
    { 0xc9650a35, 77 },
    { 0xeb650a36, 95 },
    { 0x93550a38, 91 },
    { 0, 0 },
    { 0x250a3a, 95 },
    { 0x52850a3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb450a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58850a49, 98 },
    { 0xd4950a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea50a4d, 23 },
    { 0, 0 },
    { 0x56350a4f, 76 },
    { 0x85150a50, 76 },
    { 0x24e50a51, 23 },
    { 0x96950a51, 33 },
    { 0xaaf50a50, 61 },
    { 0x38f50a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab50a5f, 23 },
    { 0xc1750a5f, 55 },
    { 0, 0 },
    { 0x41250a62, 93 },
    { 0x32050a63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2050a67, 94 },
    { 0x10650a68, 76 },
    { 0xfd850a69, 43 },
    { 0xcc350a6a, 98 },
    { 0xaf350a6a, 90 },
    { 0, 0 },
    { 0x3c550a6d, 34 },
    { 0, 0 },
    { 0x5f250a6f, 34 },
    { 0x94a50a6f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d50a76, 98 },
    { 0x4f850a76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2450a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1150a7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f750a84, 23 },
    { 0x29850a85, 90 },
    { 0xbda50a85, 23 },
    { 0x9d150a85, 61 },
    { 0xf4650a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc50a8e, 55 },
    { 0, 0 },
    { 0x7550a90, 83 },
    { 0x41a50a91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33550a96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8250aa0, 61 },
    { 0, 0 },
    { 0x66850aa2, 61 },
    { 0, 0 },
    { 0x69350aa4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x90350aa7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd50aaa, 53 },
    { 0, 0 },
    { 0x4a450aac, 43 },
    { 0, 0 },
    { 0x95250aae, 61 },
    { 0, 0 },
    { 0x3d750ab0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b850ab3, 90 },
    { 0xf7950ab4, 93 },
    { 0, 0 },
    { 0xdfc50ab6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e50aba, 34 },
    { 0xff450abb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32150abf, 76 },
    { 0x46f50ac0, 90 },
    { 0, 0 },
    { 0x15450ac2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d50ac8, 33 },
    { 0, 0 },
    { 0xf4850aca, 98 },
    { 0x15650acb, 90 },
    { 0x45e50acc, 87 },
    { 0, 0 },
    { 0x57550ace, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1650ad1, 99 },
    { 0, 0 },
    { 0xc7050ad3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de50ad7, 78 },
    { 0xa9550ad8, 99 },
    { 0, 0 },
    { 0x67d50ada, 33 },
    { 0xaf250adb, 100 },
    { 0x6ae50adc, 95 },
    { 0xa7050add, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d50ae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35950ae4, 23 },
    { 0, 0 },
    { 0xc9350ae6, 65 },
    { 0xf1350ae6, 90 },
    { 0x40750ae6, 98 },
    { 0xce250ae9, 95 },
    { 0, 0 },
    { 0xf7350aeb, 55 },
    { 0xbbf50aec, 53 },
    { 0xe2550aed, 46 },
    { 0x51950aec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76050af2, 90 },
    { 0x66e50af2, 61 },
    { 0, 0 },
    { 0xe6850af5, 78 },
    { 0x3f550af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee250b02, 43 },
    { 0, 0 },
    { 0x85350b04, 78 },
    { 0x16450b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17050b0a, 88 },
    { 0x93550b0b, 23 },
    { 0x72d50b0c, 90 },
    { 0x98c50b0a, 90 },
    { 0x57b50b0e, 61 },
    { 0x14750b0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f50b1c, 93 },
    { 0, 0 },
    { 0x65f50b1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5050b26, 81 },
    { 0, 0 },
    { 0xdb350b28, 90 },
    { 0x8ee50b28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86750b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf450b32, 99 },
    { 0x9ab50b33, 23 },
    { 0xc5d50b33, 67 },
    { 0xcad50b35, 78 },
    { 0x24850b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2650b3a, 95 },
    { 0x19750b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a50b42, 66 },
    { 0x5c450b43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb350b49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40450b4d, 65 },
    { 0x83150b4e, 61 },
    { 0x74550b4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa550b53, 90 },
    { 0, 0 },
    { 0x78550b55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e50b5b, 52 },
    { 0, 0 },
    { 0xb8c50b5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2650b61, 98 },
    { 0x55c50b61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c450b65, 23 },
    { 0, 0 },
    { 0xe6a50b67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac150b72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a50b76, 98 },
    { 0, 0 },
    { 0x6e150b78, 98 },
    { 0xb8d50b79, 85 },
    { 0x25e50b7a, 99 },
    { 0x23850b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4450b7e, 23 },
    { 0xd7550b7f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d50b86, 97 },
    { 0xf5050b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f50b8a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb50b90, 88 },
    { 0x32550b90, 93 },
    { 0, 0 },
    { 0xaab50b93, 61 },
    { 0, 0 },
    { 0x1a950b95, 67 },
    { 0x2550b96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b550b9a, 84 },
    { 0, 0 },
    { 0xf2250b9c, 84 },
    { 0x2ad50b9c, 95 },
    { 0xe0550b9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a850ba4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75250ba9, 52 },
    { 0x28450baa, 33 },
    { 0, 0 },
    { 0x8d450bac, 78 },
    { 0x50850bad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4550bb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe50bba, 23 },
    { 0xbb450bbb, 95 },
    { 0x9f650bbc, 72 },
    { 0xd1d50bbc, 77 },
    { 0x6cd50bbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64450bc2, 96 },
    { 0, 0 },
    { 0xcf950bc4, 40 },
    { 0x10d50bc5, 66 },
    { 0, 0 },
    { 0x32e50bc7, 91 },
    { 0xdb150bc8, 55 },
    { 0x44850bc9, 33 },
    { 0x18350bc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa450bd7, 90 },
    { 0xcc350bd8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d450bdc, 88 },
    { 0, 0 },
    { 0x78a50bde, 61 },
    { 0, 0 },
    { 0x18a50be0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x75850be3, 33 },
    { 0, 0 },
    { 0xc4c50be5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32550bed, 100 },
    { 0xbcc50bed, 90 },
    { 0x25950bef, 95 },
    { 0x18150bef, 33 },
    { 0x4850bf1, 99 },
    { 0x71a50bf2, 23 },
    { 0x88a50bf3, 98 },
    { 0x92850bed, 55 },
    { 0xd7b50bf4, 52 },
    { 0, 0 },
    { 0xee750bf7, 98 },
    { 0xa9d50bf8, 98 },
    { 0xbd450bf9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f350bfc, 23 },
    { 0x7f050bfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2750c00, 52 },
    { 0xeb50c01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78650c0c, 90 },
    { 0, 0 },
    { 0x96d50c0e, 95 },
    { 0, 0 },
    { 0x70150c10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68850c16, 33 },
    { 0xc1450c17, 77 },
    { 0, 0 },
    { 0xf0350c19, 61 },
    { 0xdb750c1a, 47 },
    { 0, 0 },
    { 0x48250c1c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa50c25, 53 },
    { 0x7ae50c26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb850c29, 76 },
    { 0, 0 },
    { 0x87250c2b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x82350c2e, 46 },
    { 0x80c50c2f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a50c32, 100 },
    { 0x58c50c32, 33 },
    { 0xb1150c33, 61 },
    { 0, 0 },
    { 0x29c50c36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a050c3a, 34 },
    { 0, 0 },
    { 0xf5650c3c, 33 },
    { 0xb7450c3c, 33 },
    { 0x5e250c3c, 55 },
    { 0xcee50c3d, 23 },
    { 0x18650c40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6050c44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96950c48, 23 },
    { 0, 0 },
    { 0xf0f50c4a, 34 },
    { 0x67350c4a, 81 },
    { 0x22850c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26550c53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c50c5a, 61 },
    { 0, 0 },
    { 0x2a550c5c, 56 },
    { 0x1b950c5d, 66 },
    { 0, 0 },
    { 0x95d50c5f, 95 },
    { 0xf6750c60, 23 },
    { 0xbfa50c60, 98 },
    { 0x62150c60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c50c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45350c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77250c70, 23 },
    { 0x86b50c70, 76 },
    { 0xe2e50c72, 33 },
    { 0, 0 },
    { 0x89650c74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2650c79, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf50c7c, 76 },
    { 0xa2f50c7c, 33 },
    { 0, 0 },
    { 0x4d950c7f, 100 },
    { 0, 0 },
    { 0x52b50c81, 61 },
    { 0, 0 },
    { 0x41550c83, 23 },
    { 0xbe750c83, 98 },
    { 0, 0 },
    { 0x64250c86, 90 },
    { 0, 0 },
    { 0xd6e50c88, 95 },
    { 0xf2850c88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea450c8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a250c91, 99 },
    { 0x22c50c91, 46 },
    { 0xdbe50c91, 95 },
    { 0x3e450c92, 33 },
    { 0, 0 },
    { 0xe6450c96, 76 },
    { 0x9bd50c97, 46 },
    { 0x27a50c96, 34 },
    { 0x39a50c97, 76 },
    { 0xc1c50c98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a50ca1, 90 },
    { 0, 0 },
    { 0xa6e50ca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f50ca6, 68 },
    { 0x7b250ca7, 91 },
    { 0, 0 },
    { 0xf1650ca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc950cac, 46 },
    { 0x72050cad, 33 },
    { 0x73a50cac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad50cb3, 76 },
    { 0xee850cb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f50cb7, 67 },
    { 0x2ea50cb8, 76 },
    { 0, 0 },
    { 0x79b50cba, 52 },
    { 0xe0e50cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb650cc1, 95 },
    { 0, 0 },
    { 0x40250cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53750cc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae50ccf, 43 },
    { 0xcc550cd0, 76 },
    { 0, 0 },
    { 0x4db50cd2, 77 },
    { 0xe1450cd3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d50cd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef50cde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb50ce2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f50ce6, 66 },
    { 0x66c50ce6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98650cf0, 33 },
    { 0xb1450cf1, 61 },
    { 0xdcd50cf2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b450cf5, 80 },
    { 0x9ff50cf6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be50cf9, 34 },
    { 0x52f50cfa, 99 },
    { 0, 0 },
    { 0x55350cfc, 23 },
    { 0, 0 },
    { 0xdbc50cfe, 98 },
    { 0x57050cff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c850d03, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77750d07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2350d12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac50d17, 87 },
    { 0xd5c50d18, 82 },
    { 0x65150d19, 52 },
    { 0x7c050d19, 23 },
    { 0x85750d1b, 76 },
    { 0, 0 },
    { 0x5df50d1d, 82 },
    { 0xe5850d1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38650d22, 95 },
    { 0, 0 },
    { 0xe8950d24, 98 },
    { 0x11950d25, 40 },
    { 0xe4050d26, 55 },
    { 0, 0 },
    { 0xf650d28, 90 },
    { 0x41050d29, 90 },
    { 0x18c50d2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda150d30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf650d33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac50d38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39550d41, 98 },
    { 0, 0 },
    { 0x91c50d43, 33 },
    { 0, 0 },
    { 0x89f50d45, 23 },
    { 0x66250d46, 17 },
    { 0xd0150d46, 98 },
    { 0xc2350d47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de50d4e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b50d53, 92 },
    { 0x39850d53, 90 },
    { 0xc8550d53, 34 },
    { 0x1cd50d55, 61 },
    { 0x22c50d57, 100 },
    { 0xb6f50d57, 90 },
    { 0, 0 },
    { 0x30850d5a, 87 },
    { 0x4a50d5b, 43 },
    { 0x26850d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28750d63, 12 },
    { 0x4f850d64, 93 },
    { 0x78150d65, 23 },
    { 0xf2650d65, 61 },
    { 0xc5c50d63, 52 },
    { 0x55850d68, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc50d6b, 33 },
    { 0xf1b50d6b, 90 },
    { 0xe7250d6d, 23 },
    { 0xf3750d6e, 98 },
    { 0x57f50d6f, 34 },
    { 0x8aa50d6f, 87 },
    { 0x2050d71, 77 },
    { 0x40150d72, 98 },
    { 0x95750d6d, 52 },
    { 0x6e750d74, 76 },
    { 0x97d50d75, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9650d78, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e50d7b, 90 },
    { 0x8cb50d7c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b50d7f, 98 },
    { 0x98a50d80, 61 },
    { 0xa3750d81, 76 },
    { 0x5d650d80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1150d88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de50d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50050d90, 95 },
    { 0, 0 },
    { 0x9d150d92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50050d96, 95 },
    { 0xf1050d96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be50d9a, 52 },
    { 0, 0 },
    { 0x94d50d9c, 23 },
    { 0x69a50d9d, 95 },
    { 0xbdd50d9d, 61 },
    { 0x1150d9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97750da4, 36 },
    { 0, 0 },
    { 0xb4350da6, 53 },
    { 0x50da6, 34 },
    { 0, 0 },
    { 0x1a50da9, 98 },
    { 0, 0 },
    { 0x23850dab, 90 },
    { 0xcba50dab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e50db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda250db9, 93 },
    { 0, 0 },
    { 0xd0450dbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b950dc1, 23 },
    { 0x7ab50dc1, 33 },
    { 0xa50dc1, 52 },
    { 0x10d50dc4, 99 },
    { 0x38c50dc5, 93 },
    { 0x3ee50dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc50dce, 90 },
    { 0, 0 },
    { 0xf4950dd0, 95 },
    { 0xca950dd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9250dd4, 100 },
    { 0, 0 },
    { 0x2d550dd6, 61 },
    { 0xc6850dd7, 23 },
    { 0x18b50dd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d50ddb, 80 },
    { 0x17750ddb, 43 },
    { 0, 0 },
    { 0xef050dde, 93 },
    { 0, 0 },
    { 0x82850de0, 33 },
    { 0, 0 },
    { 0x8d650de2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb50de6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93550dea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4050ded, 84 },
    { 0, 0 },
    { 0xc7650def, 60 },
    { 0xd550def, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c50df3, 46 },
    { 0xee250df4, 55 },
    { 0xecc50df5, 81 },
    { 0, 0 },
    { 0x90d50df7, 33 },
    { 0x64250df8, 72 },
    { 0xbd650df7, 95 },
    { 0x53750dfa, 61 },
    { 0xf3350dfa, 23 },
    { 0x6c550dfa, 34 },
    { 0xa5050dfa, 33 },
    { 0, 0 },
    { 0x4a150dff, 84 },
    { 0, 0 },
    { 0xf9950e01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e50e0a, 100 },
    { 0, 0 },
    { 0xb4b50e0c, 55 },
    { 0, 0 },
    { 0xa1150e0e, 33 },
    { 0xc3950e0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35750e12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92550e16, 61 },
    { 0x4b750e17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea50e1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f50e1d, 38 },
    { 0xb0950e1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a50e21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7150e24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd50e28, 61 },
    { 0xa1e50e29, 80 },
    { 0x6c50e2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f50e2d, 78 },
    { 0, 0 },
    { 0x94250e2f, 90 },
    { 0x25250e2f, 33 },
    { 0x67750e31, 43 },
    { 0xdc950e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c50e3b, 46 },
    { 0x2af50e3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc50e42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x93350e45, 98 },
    { 0xa6e50e46, 46 },
    { 0x73150e46, 33 },
    { 0xd5b50e46, 55 },
    { 0xead50e49, 76 },
    { 0x88750e46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f750e50, 98 },
    { 0x5ba50e51, 99 },
    { 0x5450e51, 95 },
    { 0, 0 },
    { 0x6a850e54, 23 },
    { 0xe0a50e55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf350e59, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e050e62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a750e65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c950e69, 67 },
    { 0xbed50e6a, 61 },
    { 0xe3a50e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2650e73, 90 },
    { 0, 0 },
    { 0x72750e75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f350e7a, 93 },
    { 0x79c50e7b, 65 },
    { 0x1ca50e7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b950e7f, 34 },
    { 0xa0250e80, 92 },
    { 0x9a750e81, 90 },
    { 0x60850e82, 33 },
    { 0xafb50e83, 76 },
    { 0xd6150e83, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a050e87, 33 },
    { 0xbf150e87, 55 },
    { 0xf4750e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc950e8c, 90 },
    { 0x63150e8d, 90 },
    { 0xc2150e8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6650e93, 75 },
    { 0x33d50e94, 68 },
    { 0xfd450e93, 98 },
    { 0x7fe50e94, 90 },
    { 0x1df50e97, 75 },
    { 0, 0 },
    { 0x1d150e99, 83 },
    { 0, 0 },
    { 0x19050e9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e750ea5, 99 },
    { 0x4450ea6, 36 },
    { 0xcc650ea7, 90 },
    { 0, 0 },
    { 0x23850ea9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b50eb2, 90 },
    { 0, 0 },
    { 0xd3d50eb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a50eb8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c650ebb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5650ec0, 90 },
    { 0x2d50ec0, 95 },
    { 0x98650ec1, 33 },
    { 0, 0 },
    { 0x21450ec4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce150ec9, 61 },
    { 0x88850eca, 76 },
    { 0x78c50ecb, 95 },
    { 0, 0 },
    { 0xf8550ecd, 78 },
    { 0, 0 },
    { 0xc8050ecf, 33 },
    { 0x8e350ed0, 90 },
    { 0xaf350ecf, 61 },
    { 0, 0 },
    { 0x60c50ed3, 98 },
    { 0, 0 },
    { 0x63550ed5, 55 },
    { 0xe2850ed5, 66 },
    { 0xb8750ed7, 55 },
    { 0, 0 },
    { 0xef750ed9, 23 },
    { 0, 0 },
    { 0x3c450edb, 43 },
    { 0, 0 },
    { 0x95150edd, 93 },
    { 0, 0 },
    { 0xbd450edf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c350ee3, 33 },
    { 0xfb750ee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a650eee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be50ef1, 95 },
    { 0, 0 },
    { 0xb850ef3, 95 },
    { 0xdfc50ef3, 98 },
    { 0xd5f50ef3, 33 },
    { 0xbab50ef4, 76 },
    { 0xda950ef7, 90 },
    { 0, 0 },
    { 0xb1a50ef9, 61 },
    { 0, 0 },
    { 0x12350efb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d150eff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e350f04, 91 },
    { 0, 0 },
    { 0x1ac50f06, 95 },
    { 0xc3350f06, 61 },
    { 0x3e650f08, 95 },
    { 0xdef50f09, 33 },
    { 0x35650f0a, 23 },
    { 0x74950f0b, 95 },
    { 0, 0 },
    { 0xdef50f0d, 67 },
    { 0x9f50f0e, 61 },
    { 0x32050f0e, 76 },
    { 0xf4250f0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b50f17, 98 },
    { 0xf6850f18, 91 },
    { 0x7ad50f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9650f1f, 61 },
    { 0, 0 },
    { 0x5b50f21, 69 },
    { 0xe1050f22, 93 },
    { 0, 0 },
    { 0xf4450f24, 43 },
    { 0, 0 },
    { 0xfcc50f26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b850f29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f50f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26950f35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a50f39, 95 },
    { 0x4a50f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34550f3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c50f43, 98 },
    { 0xe5250f44, 85 },
    { 0x98f50f45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec50f48, 55 },
    { 0x30f50f49, 98 },
    { 0, 0 },
    { 0x37750f4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8250f4f, 100 },
    { 0xe0850f50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4650f57, 33 },
    { 0xe7e50f58, 65 },
    { 0xf9750f59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91550f62, 87 },
    { 0xa3250f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e50f6d, 78 },
    { 0x81550f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f250f75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd50f78, 33 },
    { 0x77c50f79, 61 },
    { 0x3a550f7a, 23 },
    { 0xd6450f7a, 69 },
    { 0x42c50f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b50f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d50f88, 43 },
    { 0, 0 },
    { 0x36450f8a, 61 },
    { 0x71450f8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5550f94, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x35850f97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0450f9e, 91 },
    { 0xc1c50f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12150fa2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb150fa5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0750fa8, 98 },
    { 0x10850fa8, 87 },
    { 0xb1450faa, 61 },
    { 0xa2050fab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb950fae, 61 },
    { 0xd4b50faf, 90 },
    { 0xb7c50faf, 46 },
    { 0x59450fb1, 52 },
    { 0, 0 },
    { 0x7e450fb3, 90 },
    { 0, 0 },
    { 0xbdf50fb5, 93 },
    { 0xa2450fb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e950fbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab650fca, 46 },
    { 0, 0 },
    { 0xab050fcc, 33 },
    { 0x8b350fcd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1850fd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e50fd6, 90 },
    { 0xb3150fd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e650fda, 33 },
    { 0x52d50fda, 76 },
    { 0xdb650fdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8450fdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80950fe7, 98 },
    { 0x6c750fe8, 76 },
    { 0x14d50fe9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72850ff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df50ff6, 99 },
    { 0xde850ff7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e950ffc, 33 },
    { 0xeb450ffd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da51000, 34 },
    { 0x87051001, 90 },
    { 0x5f051002, 61 },
    { 0x8cf51002, 92 },
    { 0xc8551004, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc25100c, 61 },
    { 0xf205100d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf51010, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc351015, 99 },
    { 0, 0 },
    { 0x48551017, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb5101d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef51022, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6575102a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e5102f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c51034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d5103b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91351040, 90 },
    { 0x1051040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd5104d, 95 },
    { 0x59d5104d, 55 },
    { 0x6a5104f, 87 },
    { 0xbaf51050, 78 },
    { 0xf6c51050, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d51057, 67 },
    { 0x15d51058, 34 },
    { 0x68c51059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c5105c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5851060, 95 },
    { 0xdbe51061, 61 },
    { 0xafd51062, 95 },
    { 0xeb651061, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa251068, 98 },
    { 0, 0 },
    { 0xd335106a, 90 },
    { 0xc675106b, 57 },
    { 0xa195106a, 61 },
    { 0x7b05106d, 90 },
    { 0x7e25106e, 34 },
    { 0, 0 },
    { 0xebe51070, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45551073, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44951079, 23 },
    { 0x9495107a, 77 },
    { 0, 0 },
    { 0xdb55107c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9251085, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe5108c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6151090, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58951093, 33 },
    { 0x63951094, 95 },
    { 0xa251095, 87 },
    { 0x44251096, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b5109d, 98 },
    { 0x81c5109e, 98 },
    { 0, 0 },
    { 0xd03510a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f510a8, 33 },
    { 0x3d4510a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98510ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a510b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822510b9, 93 },
    { 0x835510ba, 23 },
    { 0x921510bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4510be, 61 },
    { 0x8db510be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8510cc, 83 },
    { 0x8a2510cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b510d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e510d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753510e0, 98 },
    { 0x516510e1, 99 },
    { 0x85b510e2, 76 },
    { 0, 0 },
    { 0x6ac510e4, 84 },
    { 0x343510e5, 61 },
    { 0x267510e5, 33 },
    { 0xb70510e5, 33 },
    { 0x75510e8, 95 },
    { 0x2ef510e5, 95 },
    { 0x4a3510e7, 77 },
    { 0x9cc510eb, 95 },
    { 0xb13510eb, 90 },
    { 0x7bc510eb, 33 },
    { 0xd3f510e7, 98 },
    { 0, 0 },
    { 0x4e4510f0, 61 },
    { 0x208510f0, 99 },
    { 0, 0 },
    { 0xea8510f3, 95 },
    { 0x45510f4, 90 },
    { 0, 0 },
    { 0x855510f6, 55 },
    { 0x3c7510f7, 61 },
    { 0x833510f8, 61 },
    { 0, 0 },
    { 0x5ff510fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf510ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db51103, 87 },
    { 0, 0 },
    { 0xdb951105, 23 },
    { 0x93451105, 95 },
    { 0xe6e51106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc245110d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd251111, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9551114, 61 },
    { 0xfae51114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee551118, 98 },
    { 0x3de51119, 93 },
    { 0x16551119, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5111d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38851122, 55 },
    { 0, 0 },
    { 0x84651124, 90 },
    { 0xa2051125, 55 },
    { 0xd9c51124, 93 },
    { 0x77851127, 23 },
    { 0x7ad51127, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5005112f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38851140, 90 },
    { 0x15d51141, 52 },
    { 0x2b951140, 80 },
    { 0x7ff51143, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b51147, 55 },
    { 0x2d751148, 90 },
    { 0x12551147, 61 },
    { 0x4e651149, 95 },
    { 0xfeb51149, 90 },
    { 0, 0 },
    { 0x4065114d, 99 },
    { 0xf65114e, 78 },
    { 0xc35114d, 33 },
    { 0, 0 },
    { 0x41951151, 87 },
    { 0xbc651152, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb51155, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa5115f, 23 },
    { 0x5f451160, 33 },
    { 0x37c51161, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1751166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe715116a, 34 },
    { 0, 0 },
    { 0x1975116c, 98 },
    { 0xe945116d, 98 },
    { 0x5745116e, 90 },
    { 0x1d25116f, 76 },
    { 0, 0 },
    { 0x9351171, 93 },
    { 0x45d51172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd51176, 55 },
    { 0x55c51177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf351182, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45051186, 95 },
    { 0xacb51187, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5118d, 99 },
    { 0xffb5118e, 52 },
    { 0x1015118f, 23 },
    { 0, 0 },
    { 0xf4351191, 61 },
    { 0x1c951192, 90 },
    { 0, 0 },
    { 0xb3151194, 23 },
    { 0, 0 },
    { 0x3bb51196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af5119b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7235119f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439511a8, 93 },
    { 0xb9511a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d511ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0511bf, 90 },
    { 0, 0 },
    { 0x1c9511c1, 99 },
    { 0, 0 },
    { 0x787511c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb511c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x364511ca, 90 },
    { 0x6c3511ca, 52 },
    { 0x87511cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd511d0, 77 },
    { 0x6fe511d0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401511d9, 55 },
    { 0x4db511da, 46 },
    { 0x5d511d9, 17 },
    { 0x62b511dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8511e0, 66 },
    { 0, 0 },
    { 0x524511e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a511e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c511ea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906511ee, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b511f6, 99 },
    { 0x266511f6, 66 },
    { 0xbc3511f6, 95 },
    { 0, 0 },
    { 0x385511fa, 66 },
    { 0xba7511fa, 76 },
    { 0xc44511fc, 65 },
    { 0, 0 },
    { 0xb6e511fe, 76 },
    { 0, 0 },
    { 0x11551200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c851203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a651208, 43 },
    { 0xfde51209, 77 },
    { 0x45151209, 23 },
    { 0xe905120b, 95 },
    { 0, 0 },
    { 0xed05120d, 33 },
    { 0xd095120d, 75 },
    { 0x56e5120f, 61 },
    { 0, 0 },
    { 0x12851211, 85 },
    { 0x76451212, 55 },
    { 0x64151213, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d251219, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f15121d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea451223, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe51227, 66 },
    { 0xda751227, 38 },
    { 0x88451227, 23 },
    { 0xac15122a, 80 },
    { 0x7055122b, 90 },
    { 0, 0 },
    { 0x30c5122d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1351231, 61 },
    { 0xd7e51231, 91 },
    { 0xce851231, 95 },
    { 0xc7651233, 55 },
    { 0xe7e51235, 33 },
    { 0x48451236, 90 },
    { 0x4c851235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2185123e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0151244, 61 },
    { 0x9851245, 52 },
    { 0xafc51244, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x65951249, 55 },
    { 0, 0 },
    { 0x8a35124b, 90 },
    { 0xac55124b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b5124f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b51252, 77 },
    { 0xae251253, 90 },
    { 0xd051254, 33 },
    { 0x97351254, 55 },
    { 0, 0 },
    { 0x79851257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb515125b, 76 },
    { 0x1115125c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea51265, 33 },
    { 0, 0 },
    { 0xa651267, 34 },
    { 0x7da51267, 84 },
    { 0x37b51268, 34 },
    { 0xe225126a, 33 },
    { 0x3f51269, 33 },
    { 0x9a65126c, 76 },
    { 0xb215126d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72051270, 68 },
    { 0x9c451271, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1151277, 47 },
    { 0, 0 },
    { 0xc1a51279, 61 },
    { 0, 0 },
    { 0x62a5127b, 23 },
    { 0, 0 },
    { 0xf375127d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13651284, 33 },
    { 0, 0 },
    { 0x90151286, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91851289, 91 },
    { 0xdc851289, 61 },
    { 0x435128b, 68 },
    { 0xe7b5128b, 90 },
    { 0xa415128d, 90 },
    { 0, 0 },
    { 0x6175128f, 33 },
    { 0xea951290, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d51299, 33 },
    { 0x2695129a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6225129d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11512a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43512a7, 90 },
    { 0x76d512a8, 91 },
    { 0xd01512a7, 78 },
    { 0xb75512a7, 33 },
    { 0xc03512a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213512af, 84 },
    { 0, 0 },
    { 0x81b512b1, 90 },
    { 0xb95512b2, 23 },
    { 0, 0 },
    { 0x5e8512b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3512b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x235512bc, 69 },
    { 0x8a2512bd, 33 },
    { 0xf1f512bd, 95 },
    { 0x327512bf, 95 },
    { 0, 0 },
    { 0xa58512c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08512c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61512cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7512d0, 81 },
    { 0, 0 },
    { 0x99e512d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca512d6, 90 },
    { 0, 0 },
    { 0x250512d8, 23 },
    { 0, 0 },
    { 0x442512da, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c512e0, 76 },
    { 0, 0 },
    { 0x938512e2, 17 },
    { 0, 0 },
    { 0x211512e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80512ea, 61 },
    { 0, 0 },
    { 0x506512ec, 98 },
    { 0x28f512ed, 43 },
    { 0x583512ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db512f5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6512f9, 90 },
    { 0xda512f9, 95 },
    { 0x612512fb, 95 },
    { 0x86b512fc, 48 },
    { 0xa63512fd, 91 },
    { 0x502512fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b251302, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c51307, 23 },
    { 0, 0 },
    { 0xed751309, 90 },
    { 0x6e35130a, 90 },
    { 0xb6f5130a, 90 },
    { 0x4855130c, 78 },
    { 0, 0 },
    { 0x2b25130e, 93 },
    { 0, 0 },
    { 0x30551310, 77 },
    { 0x86151311, 100 },
    { 0, 0 },
    { 0xdd451313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d51318, 33 },
    { 0xdc551319, 78 },
    { 0x4385131a, 90 },
    { 0xfd551318, 55 },
    { 0xe9f5131c, 90 },
    { 0x4915131c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c851325, 77 },
    { 0x5e851325, 76 },
    { 0x5f151327, 76 },
    { 0x55851327, 98 },
    { 0x5ab51325, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b75132d, 95 },
    { 0xc6d5132e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3751332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e51336, 96 },
    { 0x97651336, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8415133c, 90 },
    { 0, 0 },
    { 0xb765133e, 85 },
    { 0xaee5133f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a51343, 98 },
    { 0x41051344, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e25134d, 61 },
    { 0, 0 },
    { 0x1665134f, 100 },
    { 0x7a451350, 90 },
    { 0xbd85134f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e51355, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe625135a, 23 },
    { 0, 0 },
    { 0x8085135c, 33 },
    { 0x8665135c, 33 },
    { 0, 0 },
    { 0xa555135f, 87 },
    { 0x17a51360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98451369, 98 },
    { 0, 0 },
    { 0x41b5136b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e51375, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2025137c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f551384, 61 },
    { 0, 0 },
    { 0x80951386, 90 },
    { 0x89751386, 90 },
    { 0xc1051387, 34 },
    { 0x1e051389, 46 },
    { 0, 0 },
    { 0xce85138b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e5138e, 90 },
    { 0x7475138e, 95 },
    { 0x36f51390, 34 },
    { 0x4de5138f, 66 },
    { 0xc1e51392, 33 },
    { 0xae851393, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30451397, 55 },
    { 0xe0d51398, 90 },
    { 0x36951399, 34 },
    { 0x4c95139a, 88 },
    { 0xa125139b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f55139e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea513a1, 55 },
    { 0x198513a2, 84 },
    { 0xbf4513a3, 76 },
    { 0x446513a2, 76 },
    { 0xfec513a1, 43 },
    { 0, 0 },
    { 0xc21513a7, 34 },
    { 0x199513a8, 61 },
    { 0xe27513a9, 33 },
    { 0, 0 },
    { 0xb5c513ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f513ae, 81 },
    { 0x49513ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12513b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c513be, 87 },
    { 0, 0 },
    { 0x881513c0, 23 },
    { 0xbbb513c0, 67 },
    { 0xaa1513c0, 33 },
    { 0x302513c3, 77 },
    { 0x2cf513c1, 55 },
    { 0xfa1513c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05513cb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610513d0, 56 },
    { 0x5b6513d1, 90 },
    { 0, 0 },
    { 0xf4a513d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20513d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5513df, 23 },
    { 0xdca513e0, 23 },
    { 0, 0 },
    { 0xf49513e2, 40 },
    { 0xb0a513e3, 65 },
    { 0x604513e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5513e8, 17 },
    { 0x33513e9, 23 },
    { 0, 0 },
    { 0xa46513eb, 55 },
    { 0, 0 },
    { 0xb2c513ed, 43 },
    { 0, 0 },
    { 0x216513ef, 90 },
    { 0x2e7513f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f513f4, 33 },
    { 0, 0 },
    { 0x490513f6, 95 },
    { 0x862513f7, 33 },
    { 0xd92513f7, 98 },
    { 0x26f513f9, 84 },
    { 0, 0 },
    { 0xbeb513fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac651400, 33 },
    { 0x11d51401, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1615140b, 91 },
    { 0, 0 },
    { 0x8db5140d, 43 },
    { 0x69b5140e, 33 },
    { 0x73f5140e, 43 },
    { 0xb9f5140d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f751413, 90 },
    { 0x3e251414, 33 },
    { 0xc5051413, 76 },
    { 0x49451416, 90 },
    { 0xa4151417, 76 },
    { 0xcfe51418, 66 },
    { 0xa7851413, 66 },
    { 0x1515141a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f51420, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92751425, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c51428, 98 },
    { 0xc9951429, 95 },
    { 0, 0 },
    { 0x6eb5142b, 23 },
    { 0x56a5142c, 33 },
    { 0, 0 },
    { 0x2bb5142e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20351436, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a51439, 23 },
    { 0, 0 },
    { 0xa125143b, 76 },
    { 0x5435143c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60951440, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef51444, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f51449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f5144e, 51 },
    { 0xa415144e, 76 },
    { 0xfa751450, 99 },
    { 0xe1a5144f, 81 },
    { 0, 0 },
    { 0xfbc51453, 67 },
    { 0x68151454, 100 },
    { 0xd1e51455, 33 },
    { 0x35151456, 61 },
    { 0x37b51457, 33 },
    { 0xb3351458, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9695145c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8785145f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59351462, 88 },
    { 0x95c51463, 43 },
    { 0x78851462, 23 },
    { 0xd0a51463, 34 },
    { 0xbc051462, 98 },
    { 0, 0 },
    { 0x91051468, 90 },
    { 0xf3b51469, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a251472, 99 },
    { 0, 0 },
    { 0xad151474, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb045147c, 88 },
    { 0x8f05147d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85251480, 90 },
    { 0x57851481, 95 },
    { 0, 0 },
    { 0x8a551483, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d951487, 33 },
    { 0x75c51488, 23 },
    { 0x47751489, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef5148e, 68 },
    { 0, 0 },
    { 0xf6051490, 88 },
    { 0x25151490, 90 },
    { 0x8e551490, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab51495, 23 },
    { 0x49451496, 23 },
    { 0x78451497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee5149a, 90 },
    { 0x8f25149a, 93 },
    { 0x3075149a, 98 },
    { 0xb5b5149d, 33 },
    { 0x21c5149d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee514a3, 61 },
    { 0x482514a4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3514a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x302514ab, 52 },
    { 0x1a8514ac, 78 },
    { 0xd84514ab, 34 },
    { 0xa37514ae, 98 },
    { 0xd40514ad, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x162514b2, 55 },
    { 0x59d514b3, 90 },
    { 0x112514b3, 33 },
    { 0xda4514b5, 100 },
    { 0, 0 },
    { 0x999514b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac514ba, 61 },
    { 0, 0 },
    { 0xbb1514bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e514bf, 61 },
    { 0x431514bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf514c6, 36 },
    { 0x844514c7, 90 },
    { 0, 0 },
    { 0x778514c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8514d0, 98 },
    { 0x534514d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9514d5, 99 },
    { 0x52e514d6, 23 },
    { 0x7fb514d7, 77 },
    { 0xa80514d6, 93 },
    { 0xbd7514d9, 95 },
    { 0x77a514da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759514e4, 88 },
    { 0xe60514e5, 61 },
    { 0xef0514e6, 55 },
    { 0, 0 },
    { 0x9d2514e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a514f2, 87 },
    { 0, 0 },
    { 0x759514f4, 98 },
    { 0x6ed514f5, 61 },
    { 0x469514f5, 95 },
    { 0x76e514f5, 61 },
    { 0x204514f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8514fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f51504, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b95150a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5551516, 100 },
    { 0x1f451516, 95 },
    { 0x74a51518, 98 },
    { 0x73c51517, 95 },
    { 0x88a51518, 61 },
    { 0x22a5151a, 55 },
    { 0x285151c, 93 },
    { 0x6445151d, 23 },
    { 0, 0 },
    { 0xba35151f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5152c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd5152f, 99 },
    { 0x7e251530, 90 },
    { 0, 0 },
    { 0x84551532, 23 },
    { 0xaf51532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f51538, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4385153c, 61 },
    { 0xdb75153c, 61 },
    { 0xf555153c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f51543, 66 },
    { 0xb7e51544, 78 },
    { 0x9f151544, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b751549, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd135154d, 93 },
    { 0, 0 },
    { 0xc9d5154f, 33 },
    { 0, 0 },
    { 0x90551551, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0751562, 90 },
    { 0, 0 },
    { 0xd3251564, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f51567, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5156b, 84 },
    { 0xf145156c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f951573, 95 },
    { 0, 0 },
    { 0x42051575, 93 },
    { 0, 0 },
    { 0xee351577, 61 },
    { 0xdfe51578, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5157b, 100 },
    { 0x45e5157c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31751580, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98851584, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd51587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b051590, 76 },
    { 0, 0 },
    { 0xfef51592, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada51596, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b15159e, 33 },
    { 0, 0 },
    { 0x8fc515a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2515a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f515a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca515a9, 91 },
    { 0x59a515aa, 100 },
    { 0x21e515ab, 33 },
    { 0, 0 },
    { 0xd10515ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c515b1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb515b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215515bc, 76 },
    { 0, 0 },
    { 0xb4f515be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd515c2, 33 },
    { 0xf9d515c2, 98 },
    { 0xd32515c4, 46 },
    { 0xbe0515c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6515ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69515d4, 33 },
    { 0xc75515d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305515e2, 95 },
    { 0, 0 },
    { 0x527515e4, 80 },
    { 0xb92515e5, 23 },
    { 0, 0 },
    { 0xb2a515e7, 91 },
    { 0, 0 },
    { 0x4ed515e9, 55 },
    { 0xc18515ea, 23 },
    { 0x78e515ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8515ee, 55 },
    { 0x20f515ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1515f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76851605, 95 },
    { 0, 0 },
    { 0xe9751607, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb95160e, 76 },
    { 0xb65160f, 55 },
    { 0x2a15160e, 23 },
    { 0, 0 },
    { 0x751612, 90 },
    { 0, 0 },
    { 0x7dc51614, 55 },
    { 0xeeb51615, 61 },
    { 0x7d151615, 85 },
    { 0xb0851617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6765161a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85161e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b151621, 98 },
    { 0xb9f51622, 33 },
    { 0x39b51623, 43 },
    { 0x87751621, 38 },
    { 0xe4c51625, 33 },
    { 0x53051621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b51630, 33 },
    { 0x32351631, 76 },
    { 0x39751631, 75 },
    { 0x35a51632, 52 },
    { 0x10351634, 77 },
    { 0xcb051635, 34 },
    { 0x59351635, 80 },
    { 0xc2651635, 61 },
    { 0x47051630, 33 },
    { 0xebd51630, 56 },
    { 0xdaa5163a, 90 },
    { 0, 0 },
    { 0xbf45163c, 98 },
    { 0x7ee5163d, 98 },
    { 0, 0 },
    { 0x68e5163f, 90 },
    { 0x3b65163f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d51645, 64 },
    { 0xc6251645, 91 },
    { 0x3ef51647, 100 },
    { 0x42c51648, 84 },
    { 0xcf551648, 100 },
    { 0, 0 },
    { 0x995164b, 98 },
    { 0, 0 },
    { 0xd5e5164d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72651650, 43 },
    { 0xe2b51651, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef51656, 52 },
    { 0x5c351657, 95 },
    { 0xad551658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd535165b, 38 },
    { 0xeb55165c, 52 },
    { 0xc1a5165c, 85 },
    { 0, 0 },
    { 0x715165f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d51662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55751665, 23 },
    { 0x20051666, 23 },
    { 0, 0 },
    { 0xf5851668, 61 },
    { 0x5d651668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a5166c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb51675, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab751680, 90 },
    { 0xf0151681, 23 },
    { 0xead51682, 52 },
    { 0xa2c51680, 65 },
    { 0, 0 },
    { 0xad751685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd5168b, 91 },
    { 0x2785168c, 76 },
    { 0, 0 },
    { 0xb315168e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41051695, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1305169f, 76 },
    { 0x45e516a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc516a8, 91 },
    { 0x1e6516a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e516ae, 81 },
    { 0, 0 },
    { 0x4c2516b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0516b3, 95 },
    { 0x92516b4, 90 },
    { 0x7bd516b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f516bb, 95 },
    { 0, 0 },
    { 0x58c516bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243516c5, 96 },
    { 0xed2516c6, 90 },
    { 0x25d516c5, 43 },
    { 0x85c516c6, 90 },
    { 0xae4516c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f516cc, 90 },
    { 0, 0 },
    { 0x8a0516ce, 40 },
    { 0xce7516cf, 61 },
    { 0, 0 },
    { 0x2da516d1, 84 },
    { 0, 0 },
    { 0xb75516d3, 87 },
    { 0x32c516d4, 23 },
    { 0x88f516d5, 61 },
    { 0, 0 },
    { 0x9d6516d7, 55 },
    { 0x4b1516d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f516dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305516e7, 67 },
    { 0x609516e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3516ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd516f5, 99 },
    { 0xa98516f6, 52 },
    { 0, 0 },
    { 0x61e516f8, 23 },
    { 0xe9b516f8, 61 },
    { 0x913516fa, 33 },
    { 0x871516f9, 66 },
    { 0x4516fc, 46 },
    { 0xb85516fd, 85 },
    { 0, 0 },
    { 0x7fe516ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec51702, 55 },
    { 0, 0 },
    { 0x57f51704, 46 },
    { 0x98e51705, 95 },
    { 0, 0 },
    { 0x3da51707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe15170b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da5170f, 84 },
    { 0x6e95170f, 75 },
    { 0, 0 },
    { 0x4f251712, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e51715, 90 },
    { 0xa0151716, 78 },
    { 0x79251716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5171a, 98 },
    { 0x7c5171b, 90 },
    { 0xf25171a, 76 },
    { 0xc015171d, 66 },
    { 0x4cf5171e, 78 },
    { 0xa225171d, 33 },
    { 0x59d5171b, 90 },
    { 0xabc51720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff51724, 46 },
    { 0x52651725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d51728, 66 },
    { 0, 0 },
    { 0x77f5172a, 23 },
    { 0x9ff5172a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f51738, 76 },
    { 0, 0 },
    { 0xd5b5173a, 70 },
    { 0xe865173a, 76 },
    { 0x6855173b, 23 },
    { 0x6bf5173d, 100 },
    { 0xcfc5173e, 100 },
    { 0, 0 },
    { 0xc6d51740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15551747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5195174a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3751751, 78 },
    { 0x8e651752, 46 },
    { 0x3af51751, 52 },
    { 0xaba51754, 46 },
    { 0xe751754, 75 },
    { 0, 0 },
    { 0xd2351757, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f5175c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb51760, 33 },
    { 0xf2d51761, 98 },
    { 0x12951762, 98 },
    { 0x22751762, 55 },
    { 0x7c451764, 77 },
    { 0x40951761, 88 },
    { 0x70651766, 87 },
    { 0xaec51766, 100 },
    { 0x7ac51763, 75 },
    { 0, 0 },
    { 0xb055176a, 23 },
    { 0x9625176b, 43 },
    { 0x27b5176c, 90 },
    { 0x5f85176d, 85 },
    { 0x2af5176e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6451771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea051774, 98 },
    { 0x22e51774, 98 },
    { 0x1aa51774, 95 },
    { 0x2d151776, 65 },
    { 0x50851778, 66 },
    { 0xdff51779, 98 },
    { 0, 0 },
    { 0x1065177b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd805177f, 61 },
    { 0xc4151780, 98 },
    { 0x3a751781, 91 },
    { 0, 0 },
    { 0x1c451783, 98 },
    { 0x8c051783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f251789, 61 },
    { 0xa3a5178a, 90 },
    { 0, 0 },
    { 0xd365178c, 95 },
    { 0x3455178d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60951795, 90 },
    { 0x4d851796, 99 },
    { 0, 0 },
    { 0x15651798, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4205179b, 99 },
    { 0xfb45179c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94517a1, 85 },
    { 0xacd517a1, 67 },
    { 0xddb517a1, 33 },
    { 0x7d2517a4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613517ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350517b0, 33 },
    { 0x9d8517b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209517b6, 55 },
    { 0, 0 },
    { 0x84f517b8, 61 },
    { 0x660517b9, 52 },
    { 0xffe517ba, 95 },
    { 0x2ad517bb, 61 },
    { 0, 0 },
    { 0x126517bd, 23 },
    { 0x3d4517be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94517c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481517c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec517ca, 55 },
    { 0, 0 },
    { 0x9bb517cc, 95 },
    { 0xb60517cc, 84 },
    { 0xc2e517ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x611517d1, 93 },
    { 0x507517d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7517d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4517dd, 61 },
    { 0xdea517de, 98 },
    { 0x256517de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206517e7, 98 },
    { 0x520517e8, 78 },
    { 0x2d517e9, 55 },
    { 0x5db517ea, 68 },
    { 0x2fa517e7, 33 },
    { 0xed3517ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811517f0, 90 },
    { 0, 0 },
    { 0x92e517f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705517f6, 33 },
    { 0xb88517f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a517fb, 23 },
    { 0x597517fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x116517ff, 100 },
    { 0xf9251800, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b251803, 100 },
    { 0x47051803, 87 },
    { 0x2e51804, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad51809, 95 },
    { 0xebf5180a, 46 },
    { 0x9455180a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73751813, 34 },
    { 0, 0 },
    { 0xc4351815, 75 },
    { 0x9ed51816, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c45181b, 81 },
    { 0, 0 },
    { 0x78c5181d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5851820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5551823, 78 },
    { 0xf8951823, 66 },
    { 0x52151825, 88 },
    { 0xbe351823, 85 },
    { 0xc6c51827, 99 },
    { 0xad051826, 33 },
    { 0x50451824, 95 },
    { 0xea51827, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9605182d, 66 },
    { 0xc7e5182e, 43 },
    { 0x84c5182e, 98 },
    { 0x8b451830, 23 },
    { 0xa4d5182d, 90 },
    { 0, 0 },
    { 0x8cc51833, 90 },
    { 0, 0 },
    { 0x6c51835, 90 },
    { 0x1a851835, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50151840, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62151844, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6751848, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9695184d, 33 },
    { 0xe145184d, 96 },
    { 0xb485184f, 33 },
    { 0, 0 },
    { 0x1c651851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c251854, 46 },
    { 0, 0 },
    { 0xe5951856, 43 },
    { 0xc5551857, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7735185c, 90 },
    { 0, 0 },
    { 0x7345185e, 90 },
    { 0xd7c5185f, 94 },
    { 0, 0 },
    { 0x91e51861, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8151867, 61 },
    { 0x83b51868, 34 },
    { 0x3d51868, 95 },
    { 0x9ba5186a, 18 },
    { 0, 0 },
    { 0x435186c, 81 },
    { 0x2cd5186d, 34 },
    { 0, 0 },
    { 0x3ee5186f, 76 },
    { 0x37051870, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5151876, 46 },
    { 0, 0 },
    { 0xcd651878, 90 },
    { 0xefd51879, 90 },
    { 0x7295187a, 76 },
    { 0xd885187b, 81 },
    { 0x87f5187b, 90 },
    { 0xa8d5187a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85751880, 84 },
    { 0xc9351880, 99 },
    { 0xf7951881, 55 },
    { 0xe4d51883, 100 },
    { 0x44e51884, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef51889, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba75188c, 33 },
    { 0xac55188d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad51897, 78 },
    { 0x53551898, 46 },
    { 0xd8b51899, 61 },
    { 0, 0 },
    { 0xb215189b, 46 },
    { 0xdfc5189c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873518a3, 94 },
    { 0x82f518a3, 95 },
    { 0x198518a5, 95 },
    { 0x5b5518a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268518ab, 23 },
    { 0, 0 },
    { 0x1b8518ad, 84 },
    { 0xf17518ae, 78 },
    { 0xb61518ae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73518b3, 95 },
    { 0xe7c518b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e518b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766518bb, 61 },
    { 0, 0 },
    { 0x71f518bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4518c7, 98 },
    { 0, 0 },
    { 0x9b6518c9, 33 },
    { 0, 0 },
    { 0x1af518cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0518d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x753518d6, 98 },
    { 0, 0 },
    { 0x5ba518d8, 75 },
    { 0, 0 },
    { 0x9b3518da, 17 },
    { 0xa0d518db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2518de, 23 },
    { 0, 0 },
    { 0xbb2518e0, 90 },
    { 0, 0 },
    { 0x2af518e2, 34 },
    { 0x1b9518e2, 38 },
    { 0, 0 },
    { 0xa9b518e5, 95 },
    { 0x43f518e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9518e9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f518ec, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235518f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181518fa, 68 },
    { 0, 0 },
    { 0xfdc518fc, 98 },
    { 0, 0 },
    { 0xd54518fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e51908, 98 },
    { 0x46351909, 88 },
    { 0, 0 },
    { 0x6045190b, 55 },
    { 0x4f5190c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3265190f, 81 },
    { 0x1435190f, 61 },
    { 0x46851911, 77 },
    { 0x9b35190f, 90 },
    { 0x49451913, 98 },
    { 0x1a951912, 75 },
    { 0xbee51915, 90 },
    { 0x4d451915, 23 },
    { 0xe6351917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25191d, 55 },
    { 0x8895191e, 87 },
    { 0, 0 },
    { 0x13751920, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf951923, 23 },
    { 0x11951923, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185192a, 23 },
    { 0x70a5192b, 90 },
    { 0x3c15192c, 85 },
    { 0x4d35192b, 98 },
    { 0x7745192b, 33 },
    { 0x1f65192f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd351933, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1351937, 34 },
    { 0x7bc51938, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda05193f, 33 },
    { 0xb0f51940, 90 },
    { 0xc9251941, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7251946, 34 },
    { 0x94c51946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6555194a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea5194f, 34 },
    { 0, 0 },
    { 0x3c851951, 95 },
    { 0x3d751952, 91 },
    { 0x26c51951, 55 },
    { 0x14351952, 34 },
    { 0x48e51951, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d51959, 93 },
    { 0xe5951959, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb885195d, 91 },
    { 0x4db5195d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc51964, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99951967, 95 },
    { 0xbc151968, 61 },
    { 0x54551969, 33 },
    { 0x8455196a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5196d, 90 },
    { 0x2255196d, 95 },
    { 0xa085196f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3451976, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e5197c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5198a, 38 },
    { 0, 0 },
    { 0xc605198c, 90 },
    { 0x3005198d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69351992, 98 },
    { 0x50d51993, 98 },
    { 0xc9b51994, 33 },
    { 0x3aa51995, 90 },
    { 0xd9e51996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd45199f, 33 },
    { 0xb2e519a0, 93 },
    { 0x92519a1, 34 },
    { 0xc54519a1, 43 },
    { 0x375519a1, 75 },
    { 0, 0 },
    { 0xeb2519a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8519ac, 87 },
    { 0x39f519ac, 98 },
    { 0x216519ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x418519b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd519b8, 33 },
    { 0, 0 },
    { 0x81f519ba, 98 },
    { 0, 0 },
    { 0x784519bc, 90 },
    { 0x209519bd, 100 },
    { 0, 0 },
    { 0x16f519bf, 33 },
    { 0x858519bf, 98 },
    { 0xf8d519bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95519c4, 100 },
    { 0x58c519c4, 34 },
    { 0xe9a519c5, 67 },
    { 0, 0 },
    { 0x38f519c8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9519cf, 23 },
    { 0x752519d0, 66 },
    { 0x900519d1, 89 },
    { 0xa38519d2, 23 },
    { 0xbaa519d0, 95 },
    { 0x3a6519d4, 95 },
    { 0xb0f519d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2519dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e519e0, 98 },
    { 0x17b519e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a519e6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49519ed, 88 },
    { 0, 0 },
    { 0xe8d519ef, 33 },
    { 0, 0 },
    { 0x5a519f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9519f4, 98 },
    { 0x325519f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83519f8, 65 },
    { 0x923519f8, 23 },
    { 0x4fb519f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c519fd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9751a01, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc51a06, 91 },
    { 0x48151a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67751a0b, 78 },
    { 0xf5951a0b, 76 },
    { 0x72a51a0b, 95 },
    { 0, 0 },
    { 0xc51a0f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8951a15, 33 },
    { 0xfe851a15, 90 },
    { 0x4a051a15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb51a1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f451a1d, 98 },
    { 0x76451a1e, 23 },
    { 0x46851a1e, 78 },
    { 0x92851a20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec651a24, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a851a29, 46 },
    { 0xbec51a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a851a30, 76 },
    { 0x3d251a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12551a35, 46 },
    { 0, 0 },
    { 0x1b451a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca51a3e, 83 },
    { 0xaf651a3f, 90 },
    { 0x5f351a3f, 87 },
    { 0, 0 },
    { 0x5e851a42, 95 },
    { 0, 0 },
    { 0xae851a44, 98 },
    { 0xdf651a44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f51a48, 61 },
    { 0x2a951a49, 93 },
    { 0x93d51a48, 33 },
    { 0, 0 },
    { 0xa4651a4c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf51a50, 61 },
    { 0, 0 },
    { 0x76651a52, 33 },
    { 0x1d751a53, 95 },
    { 0x85551a53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f51a57, 100 },
    { 0x5ff51a57, 33 },
    { 0, 0 },
    { 0xbee51a5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0951a5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb851a63, 61 },
    { 0x5551a64, 76 },
    { 0xeac51a65, 85 },
    { 0x2d51a66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b751a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b51a71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa251a77, 77 },
    { 0, 0 },
    { 0xd8851a79, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d051a7d, 98 },
    { 0xf4051a7e, 90 },
    { 0x9de51a7e, 23 },
    { 0xdc451a80, 76 },
    { 0xad251a81, 87 },
    { 0x73c51a82, 67 },
    { 0xdc451a83, 98 },
    { 0xfa151a84, 91 },
    { 0xea551a84, 6 },
    { 0x69e51a7d, 61 },
    { 0xdef51a87, 43 },
    { 0x86d51a88, 43 },
    { 0x18351a88, 33 },
    { 0x22551a86, 55 },
    { 0, 0 },
    { 0x20551a8c, 90 },
    { 0, 0 },
    { 0x91251a8e, 90 },
    { 0xaf851a8e, 78 },
    { 0x62e51a90, 46 },
    { 0x83d51a8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f51a94, 61 },
    { 0x74451a95, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e51a98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e851a9d, 23 },
    { 0xee851a9e, 95 },
    { 0x18151a9f, 77 },
    { 0, 0 },
    { 0x70851aa1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x33051aa4, 52 },
    { 0, 0 },
    { 0xf5f51aa6, 66 },
    { 0, 0 },
    { 0x13c51aa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c51aab, 76 },
    { 0x7ee51aab, 55 },
    { 0x14551aad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e51ab2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b351ab8, 61 },
    { 0x1bd51ab9, 78 },
    { 0x78251ab9, 91 },
    { 0x5e751ab8, 33 },
    { 0x37951abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d51abf, 90 },
    { 0x93951ac0, 34 },
    { 0, 0 },
    { 0x53a51ac2, 33 },
    { 0xcfd51ac2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b51ac8, 76 },
    { 0xed51ac9, 76 },
    { 0x72151aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a51ace, 38 },
    { 0x45c51acf, 90 },
    { 0xab851acf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda51ad5, 48 },
    { 0, 0 },
    { 0x22051ad7, 55 },
    { 0x4d51ad7, 76 },
    { 0, 0 },
    { 0x3e951ada, 61 },
    { 0xd0151adb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be51ade, 98 },
    { 0, 0 },
    { 0x5d051ae0, 76 },
    { 0xbdc51ae0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc51ae5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65951aea, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe051aee, 76 },
    { 0x9c351aee, 95 },
    { 0, 0 },
    { 0x8d451af1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f51af5, 98 },
    { 0, 0 },
    { 0xf5751af7, 76 },
    { 0xfa651af7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a851afd, 85 },
    { 0xd8e51afe, 67 },
    { 0x83051aff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f51b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45051b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea551b0a, 90 },
    { 0, 0 },
    { 0x8b51b0c, 23 },
    { 0x69751b0d, 99 },
    { 0xbf351b0e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xce51b11, 33 },
    { 0x8c251b12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e51b16, 61 },
    { 0x3e351b17, 66 },
    { 0, 0 },
    { 0xac451b19, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68651b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f51b20, 93 },
    { 0xfbc51b21, 98 },
    { 0xb6051b22, 90 },
    { 0, 0 },
    { 0x14e51b24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92151b2a, 34 },
    { 0x65b51b2a, 95 },
    { 0xf3b51b2c, 38 },
    { 0x79e51b2d, 61 },
    { 0x77051b2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c51b3a, 33 },
    { 0x20651b3b, 33 },
    { 0x68651b3c, 98 },
    { 0x82051b3d, 90 },
    { 0xabc51b3e, 23 },
    { 0x3fe51b3e, 99 },
    { 0xfb451b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e51b48, 95 },
    { 0xecf51b49, 46 },
    { 0x11e51b4a, 80 },
    { 0xa5551b4b, 89 },
    { 0xc4651b48, 95 },
    { 0x19851b4d, 61 },
    { 0x3ea51b4d, 95 },
    { 0xd6451b4f, 33 },
    { 0x11951b4c, 55 },
    { 0, 0 },
    { 0x9051b52, 98 },
    { 0, 0 },
    { 0x92651b54, 46 },
    { 0x3bd51b55, 66 },
    { 0x80551b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27951b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5651b60, 52 },
    { 0x53251b61, 33 },
    { 0x5f751b62, 33 },
    { 0x42c51b60, 87 },
    { 0xaaa51b63, 33 },
    { 0x1a851b65, 76 },
    { 0, 0 },
    { 0x7fd51b67, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b51b6b, 23 },
    { 0xb8151b6b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x16451b6f, 55 },
    { 0, 0 },
    { 0xde251b71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a251b75, 33 },
    { 0x33e51b76, 17 },
    { 0, 0 },
    { 0xb3c51b78, 100 },
    { 0xc2051b79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24151b7e, 61 },
    { 0, 0 },
    { 0xe5e51b80, 98 },
    { 0x8ef51b81, 43 },
    { 0xe3551b82, 23 },
    { 0xf7c51b83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab751b88, 33 },
    { 0, 0 },
    { 0xe2651b8a, 100 },
    { 0xeca51b8b, 61 },
    { 0x91351b8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f51b8f, 61 },
    { 0, 0 },
    { 0x14151b91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d51b94, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac51b9a, 95 },
    { 0xd8a51b9b, 95 },
    { 0x20e51b9b, 90 },
    { 0x8651b9b, 90 },
    { 0x86151b9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18151ba1, 33 },
    { 0x96151ba2, 33 },
    { 0xcf51ba3, 33 },
    { 0, 0 },
    { 0xf5a51ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b051ba8, 33 },
    { 0, 0 },
    { 0x24751baa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x75951bad, 33 },
    { 0x6a851bae, 33 },
    { 0x39551baf, 95 },
    { 0x2e751bb0, 76 },
    { 0xcd451bb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c51bb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68251bc1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851bc9, 61 },
    { 0xe1651bca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17151bcf, 90 },
    { 0xb1b51bcf, 33 },
    { 0, 0 },
    { 0x9ea51bd2, 95 },
    { 0, 0 },
    { 0x5e251bd4, 47 },
    { 0x90e51bd5, 40 },
    { 0x5ab51bd4, 61 },
    { 0xb651bd7, 91 },
    { 0x28151bd8, 100 },
    { 0xb5e51bd9, 61 },
    { 0xd9e51bd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e51be3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b51be6, 55 },
    { 0x94c51be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca751bec, 34 },
    { 0, 0 },
    { 0x12151bee, 91 },
    { 0x1cc51bef, 23 },
    { 0x63c51bf0, 52 },
    { 0x38451bef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa151bf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf51bfb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0751bfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7551c05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37351c09, 40 },
    { 0, 0 },
    { 0xe1f51c0b, 90 },
    { 0xbc651c0b, 61 },
    { 0, 0 },
    { 0x4ce51c0e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46951c14, 78 },
    { 0x52d51c15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe51c18, 55 },
    { 0x88451c19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x22651c1c, 77 },
    { 0x8b451c1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba51c25, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1851c28, 34 },
    { 0xca151c28, 76 },
    { 0x2e851c28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66551c36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa251c3b, 87 },
    { 0x33451c3c, 61 },
    { 0xc7f51c3c, 23 },
    { 0x77c51c3b, 61 },
    { 0x2bf51c3f, 95 },
    { 0x89751c3d, 33 },
    { 0x7f751c41, 85 },
    { 0xc7551c42, 40 },
    { 0xacc51c3c, 33 },
    { 0x21d51c44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e51c49, 99 },
    { 0, 0 },
    { 0x28d51c4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64251c51, 43 },
    { 0x7dc51c51, 98 },
    { 0xd1751c52, 85 },
    { 0x54f51c53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3651c57, 23 },
    { 0, 0 },
    { 0x3c451c59, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab051c5d, 43 },
    { 0, 0 },
    { 0x40851c5f, 34 },
    { 0, 0 },
    { 0xc1e51c61, 92 },
    { 0x4a451c62, 55 },
    { 0, 0 },
    { 0x16851c64, 99 },
    { 0xe6c51c65, 87 },
    { 0xdaa51c66, 52 },
    { 0x84451c67, 76 },
    { 0x7851c65, 76 },
    { 0x38e51c67, 43 },
    { 0x21751c6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6251c71, 90 },
    { 0xa1951c72, 61 },
    { 0xc4651c72, 90 },
    { 0x7d451c74, 76 },
    { 0x5fb51c75, 83 },
    { 0, 0 },
    { 0x7e351c77, 34 },
    { 0, 0 },
    { 0x62751c79, 95 },
    { 0xff51c7a, 34 },
    { 0xfad51c7b, 98 },
    { 0xbe051c7c, 43 },
    { 0xa3251c7d, 55 },
    { 0x5d251c7a, 95 },
    { 0xff251c7f, 90 },
    { 0x40551c7f, 80 },
    { 0xa9e51c7d, 55 },
    { 0xdcf51c82, 33 },
    { 0x8b351c83, 55 },
    { 0xd1c51c82, 33 },
    { 0x96e51c83, 55 },
    { 0x9f551c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a051c89, 98 },
    { 0x7a251c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d551c9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54351c9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac651ca6, 61 },
    { 0x52451ca7, 87 },
    { 0xb1051ca6, 90 },
    { 0x57851ca8, 95 },
    { 0xda651ca8, 66 },
    { 0xa8e51ca8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73251cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa51cb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c51cb9, 23 },
    { 0x81651cba, 61 },
    { 0x43351cbb, 95 },
    { 0, 0 },
    { 0x64351cbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1351cc1, 61 },
    { 0x20551cc2, 34 },
    { 0x5951cc3, 55 },
    { 0, 0 },
    { 0x59d51cc5, 99 },
    { 0x51151cc6, 90 },
    { 0, 0 },
    { 0x68751cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c51ccc, 99 },
    { 0x91b51ccd, 23 },
    { 0x19b51ccc, 34 },
    { 0, 0 },
    { 0xb0951cd0, 55 },
    { 0xbe351cd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x99351cd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe851cdd, 95 },
    { 0xb8f51cde, 87 },
    { 0x68351cdf, 33 },
    { 0xcc151cdd, 66 },
    { 0x7de51ce1, 33 },
    { 0x3ee51cdd, 96 },
    { 0xa3a51ce3, 87 },
    { 0xb8451ce3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e51ce7, 98 },
    { 0x17051ce8, 66 },
    { 0xbe151ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb851cec, 55 },
    { 0, 0 },
    { 0x5e451cee, 33 },
    { 0x64751cef, 95 },
    { 0x54451cf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11151cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2751cfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c51cff, 46 },
    { 0, 0 },
    { 0xe4f51d01, 98 },
    { 0x6cf51d02, 90 },
    { 0xc7f51d03, 81 },
    { 0x3f151d03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4651d08, 69 },
    { 0x14551d09, 81 },
    { 0x48951d08, 98 },
    { 0x67a51d0a, 61 },
    { 0x87751d0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b351d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4651d16, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x36951d19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed51d1e, 81 },
    { 0x3eb51d1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d051d27, 61 },
    { 0, 0 },
    { 0xae451d29, 23 },
    { 0x21051d2a, 43 },
    { 0x48e51d29, 87 },
    { 0x3d151d29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f51d2f, 23 },
    { 0, 0 },
    { 0x5e451d31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5751d36, 78 },
    { 0xa6051d37, 76 },
    { 0, 0 },
    { 0xa9751d39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44051d41, 76 },
    { 0x9d851d41, 75 },
    { 0, 0 },
    { 0xc6a51d44, 51 },
    { 0x7d51d45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55051d49, 67 },
    { 0x40b51d49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca951d4d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c51d50, 76 },
    { 0xb5651d50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21551d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc51d5f, 55 },
    { 0, 0 },
    { 0x7e151d61, 61 },
    { 0x5e751d61, 90 },
    { 0x47b51d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6851d6c, 33 },
    { 0xa6a51d6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c51d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c51d78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74051d7b, 34 },
    { 0x16d51d7c, 61 },
    { 0xede51d7d, 91 },
    { 0, 0 },
    { 0x82b51d7f, 23 },
    { 0, 0 },
    { 0xe7f51d81, 95 },
    { 0, 0 },
    { 0x4651d83, 88 },
    { 0, 0 },
    { 0xa2851d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba851d8a, 95 },
    { 0x1f151d8b, 85 },
    { 0xa9151d8c, 52 },
    { 0x6e551d8c, 23 },
    { 0xf1251d8e, 23 },
    { 0x1b651d8a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e651d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd51d9f, 66 },
    { 0, 0 },
    { 0x49c51da1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7451da4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab51da7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b251daa, 61 },
    { 0x44b51dab, 23 },
    { 0x6b851dac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a51db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81151db9, 61 },
    { 0xfee51dba, 33 },
    { 0x7a851dba, 98 },
    { 0x6f051db9, 98 },
    { 0x70951dba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b351dc5, 34 },
    { 0xf51dc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec51dce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9051dd4, 34 },
    { 0x75b51dd5, 61 },
    { 0x43351dd6, 46 },
    { 0xa1c51dd7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0451de0, 100 },
    { 0xc9851de1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5551de7, 55 },
    { 0x3a651de8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90151dec, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d351def, 93 },
    { 0x2d751df0, 90 },
    { 0x68451df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7651df4, 23 },
    { 0, 0 },
    { 0x7151df6, 95 },
    { 0, 0 },
    { 0x15051df8, 66 },
    { 0x69851df9, 99 },
    { 0xef351df9, 87 },
    { 0x7ab51dfb, 61 },
    { 0, 0 },
    { 0xfe551dfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7651e00, 93 },
    { 0, 0 },
    { 0xcbf51e02, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf251e09, 98 },
    { 0xb6c51e0a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e51e0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e751e1a, 69 },
    { 0, 0 },
    { 0x6e251e1c, 46 },
    { 0, 0 },
    { 0x60551e1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4151e22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec51e26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5751e2e, 23 },
    { 0, 0 },
    { 0x8eb51e30, 23 },
    { 0x8f151e31, 55 },
    { 0xde351e32, 88 },
    { 0xfb951e31, 43 },
    { 0xd7851e34, 95 },
    { 0xcd251e33, 100 },
    { 0, 0 },
    { 0xb7751e37, 23 },
    { 0x8ea51e38, 90 },
    { 0xc6251e39, 61 },
    { 0x8a851e39, 90 },
    { 0x70651e3a, 23 },
    { 0x72151e3b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0851e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38251e4c, 90 },
    { 0x35e51e4c, 100 },
    { 0, 0 },
    { 0xa0a51e4f, 78 },
    { 0, 0 },
    { 0xc0551e51, 90 },
    { 0x1b851e51, 61 },
    { 0x9e151e53, 61 },
    { 0, 0 },
    { 0xe2151e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7451e59, 61 },
    { 0x63f51e5a, 95 },
    { 0xb151e5a, 84 },
    { 0, 0 },
    { 0x91f51e5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2251e60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55151e66, 43 },
    { 0x49e51e66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94951e6c, 90 },
    { 0, 0 },
    { 0x6c351e6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc451e71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19951e75, 17 },
    { 0xfeb51e76, 33 },
    { 0x51d51e77, 76 },
    { 0xd1b51e78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b51e7b, 95 },
    { 0xd0151e7b, 33 },
    { 0, 0 },
    { 0xfe851e7e, 33 },
    { 0, 0 },
    { 0xe8f51e80, 33 },
    { 0xd0d51e80, 43 },
    { 0, 0 },
    { 0x37d51e83, 23 },
    { 0x2a451e84, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57151e8a, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb451e99, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7251e9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93751ea5, 95 },
    { 0x19051ea6, 68 },
    { 0xba151ea5, 55 },
    { 0x7f751ea8, 61 },
    { 0x6751ea9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10451eb2, 78 },
    { 0x9ab51eb3, 61 },
    { 0x2ed51eb4, 91 },
    { 0, 0 },
    { 0x29351eb6, 61 },
    { 0, 0 },
    { 0xad551eb8, 46 },
    { 0x27f51eb9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba51ec1, 95 },
    { 0, 0 },
    { 0xce651ec3, 90 },
    { 0, 0 },
    { 0x851ec5, 33 },
    { 0x24c51ec6, 66 },
    { 0x7e51ec7, 98 },
    { 0x6bf51ec7, 90 },
    { 0x7e651ec9, 95 },
    { 0x6b751eca, 98 },
    { 0xb5751ec9, 90 },
    { 0, 0 },
    { 0x70551ecd, 33 },
    { 0x3b851ece, 46 },
    { 0x55351ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc751ed4, 90 },
    { 0x87551ed4, 33 },
    { 0, 0 },
    { 0xd2e51ed7, 78 },
    { 0xbcf51ed8, 90 },
    { 0x21651ed9, 95 },
    { 0x7c251ed9, 43 },
    { 0x77051ed9, 36 },
    { 0x8b051edc, 98 },
    { 0xd1251edd, 55 },
    { 0x97851ede, 61 },
    { 0, 0 },
    { 0x24051ee0, 23 },
    { 0, 0 },
    { 0x19c51ee2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec51ee5, 61 },
    { 0x9b051ee6, 90 },
    { 0x50451ee7, 17 },
    { 0xa4351ee8, 98 },
    { 0, 0 },
    { 0x84351eea, 55 },
    { 0x59f51eeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14551eef, 23 },
    { 0xd7151ef0, 98 },
    { 0xa5d51ef0, 43 },
    { 0x4cf51ef2, 52 },
    { 0xb251ef3, 55 },
    { 0xc5751eef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89551ef9, 33 },
    { 0x89c51efa, 33 },
    { 0xbd751efb, 66 },
    { 0xd9d51efc, 23 },
    { 0x56a51efd, 52 },
    { 0xbe751efd, 23 },
    { 0xc951eff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd51f06, 23 },
    { 0, 0 },
    { 0xb1851f08, 90 },
    { 0, 0 },
    { 0x9ef51f0a, 88 },
    { 0xe7e51f0b, 69 },
    { 0x9651f0b, 90 },
    { 0x6b251f0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b51f10, 66 },
    { 0xdfe51f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e351f14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e751f17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac51f1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc851f1e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea51f22, 93 },
    { 0xfb451f22, 23 },
    { 0x89551f24, 61 },
    { 0x6e51f25, 90 },
    { 0x5fe51f25, 33 },
    { 0x32651f26, 33 },
    { 0, 0 },
    { 0x18851f29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90151f2e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5551f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0751f3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e51f42, 61 },
    { 0, 0 },
    { 0x69151f44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50551f4a, 33 },
    { 0, 0 },
    { 0x54a51f4c, 61 },
    { 0, 0 },
    { 0x7a551f4e, 33 },
    { 0x6f51f4e, 52 },
    { 0x9c251f50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0051f58, 23 },
    { 0, 0 },
    { 0x5df51f5a, 33 },
    { 0xf0351f5b, 33 },
    { 0xb3f51f5b, 33 },
    { 0x71151f5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb451f60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e51f63, 76 },
    { 0x55951f64, 85 },
    { 0x27b51f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c51f68, 33 },
    { 0xb7851f68, 55 },
    { 0x17e51f6a, 61 },
    { 0x93151f6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b751f70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4551f74, 34 },
    { 0xa4851f75, 43 },
    { 0, 0 },
    { 0x49e51f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a51f7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d51f7e, 77 },
    { 0x9a551f7f, 33 },
    { 0x69f51f7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35451f84, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a51f89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0951f8f, 33 },
    { 0, 0 },
    { 0x44f51f91, 90 },
    { 0x8e651f92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8051f9e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25851fa2, 23 },
    { 0x2f951fa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c851faa, 61 },
    { 0x33351faa, 33 },
    { 0, 0 },
    { 0xcc751fad, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a051fb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f151fbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6351fbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5051fc2, 98 },
    { 0, 0 },
    { 0xe8751fc4, 43 },
    { 0, 0 },
    { 0x20d51fc6, 77 },
    { 0, 0 },
    { 0x8e451fc8, 99 },
    { 0x1b551fc9, 93 },
    { 0x86151fc8, 23 },
    { 0, 0 },
    { 0xc3351fcc, 61 },
    { 0, 0 },
    { 0xcf451fce, 55 },
    { 0x97751fcf, 33 },
    { 0, 0 },
    { 0xb0e51fd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da51fd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48451fd7, 98 },
    { 0, 0 },
    { 0xb1f51fd9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xce651fdc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49951fdf, 85 },
    { 0, 0 },
    { 0xf5151fe1, 61 },
    { 0x16f51fe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd751fe5, 23 },
    { 0x4b051fe5, 90 },
    { 0xa1651fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96651fea, 90 },
    { 0x6e251feb, 89 },
    { 0x6c751fec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57851ff0, 33 },
    { 0x8a651ff1, 75 },
    { 0, 0 },
    { 0x72c51ff3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x95051ff6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd651ff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8851ffc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf52000, 43 },
    { 0x35452001, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b52004, 55 },
    { 0xb5152005, 90 },
    { 0x6f552005, 95 },
    { 0x3152005, 84 },
    { 0xc8452008, 90 },
    { 0xdff52009, 98 },
    { 0xfd65200a, 97 },
    { 0x9d05200b, 90 },
    { 0xb875200c, 80 },
    { 0x6ce5200b, 95 },
    { 0x6595200d, 91 },
    { 0, 0 },
    { 0x66952010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a52015, 99 },
    { 0xbb352016, 52 },
    { 0x3e52015, 90 },
    { 0x2dc52018, 93 },
    { 0xa8052017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4015201d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe52020, 95 },
    { 0xd6d52021, 33 },
    { 0x4ac52022, 61 },
    { 0xbdd52023, 84 },
    { 0x13552022, 95 },
    { 0, 0 },
    { 0xea752026, 66 },
    { 0xa9e52026, 61 },
    { 0xa8d52028, 33 },
    { 0xd8052028, 61 },
    { 0x70b52029, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f55202e, 34 },
    { 0x1b75202f, 87 },
    { 0xba552030, 38 },
    { 0x935202f, 93 },
    { 0x14f5202e, 90 },
    { 0xc9b52032, 61 },
    { 0x8d752034, 40 },
    { 0xcf652035, 61 },
    { 0xa9352036, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c5203b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6615203e, 61 },
    { 0xdfe5203e, 84 },
    { 0xf7452040, 95 },
    { 0x69952040, 41 },
    { 0x5dd52042, 55 },
    { 0xf5752043, 52 },
    { 0x91a52043, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d5204f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53352056, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5152059, 100 },
    { 0x6135205a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74852060, 61 },
    { 0x1ab52060, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe752067, 78 },
    { 0x29e52068, 23 },
    { 0x7fa52067, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605206e, 61 },
    { 0x9215206f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a52073, 67 },
    { 0, 0 },
    { 0x8b952075, 43 },
    { 0x20c52076, 76 },
    { 0x6a952075, 97 },
    { 0x28f52077, 34 },
    { 0x37852079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea52082, 87 },
    { 0xe5a52083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67052086, 61 },
    { 0x27952087, 61 },
    { 0xd3a52088, 23 },
    { 0xa9c52089, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb852091, 46 },
    { 0x2b352091, 95 },
    { 0xdcc52093, 33 },
    { 0xcc152094, 90 },
    { 0xdc752095, 95 },
    { 0xa5252091, 61 },
    { 0, 0 },
    { 0x9b752098, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d45209d, 90 },
    { 0xdc55209e, 33 },
    { 0x4565209d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6520a3, 34 },
    { 0x5a3520a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1520a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x911520ab, 78 },
    { 0xb17520ac, 95 },
    { 0x8f6520ad, 65 },
    { 0x21c520ae, 23 },
    { 0xf58520ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6520b2, 98 },
    { 0, 0 },
    { 0x63520b4, 98 },
    { 0xd7520b4, 76 },
    { 0xb95520b6, 98 },
    { 0xef4520b7, 91 },
    { 0x4b520b4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13520bc, 76 },
    { 0x3c3520bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657520c1, 83 },
    { 0xe4e520c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83520c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571520cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e520d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x539520d7, 61 },
    { 0x66b520d7, 93 },
    { 0xfc6520d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d520e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f520e5, 85 },
    { 0, 0 },
    { 0xa3a520e7, 76 },
    { 0xb66520e8, 23 },
    { 0x261520e7, 95 },
    { 0xbfa520ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e520ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f520f4, 46 },
    { 0x90520f5, 66 },
    { 0x36f520f6, 95 },
    { 0xf90520f5, 76 },
    { 0, 0 },
    { 0x1ef520f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c520fd, 33 },
    { 0x45e520fd, 52 },
    { 0xdd5520fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4152102, 99 },
    { 0x7052103, 23 },
    { 0x20c52103, 90 },
    { 0x96452105, 93 },
    { 0x56c52106, 84 },
    { 0x10a52107, 76 },
    { 0x3e052102, 95 },
    { 0xaa352109, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af5210c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5252113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61452117, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b5211a, 66 },
    { 0xb1c5211a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2195211f, 23 },
    { 0xc6052120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d52126, 55 },
    { 0x67152127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e55212a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe895212d, 95 },
    { 0x4a05212e, 85 },
    { 0xc0e5212e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc552134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe952139, 43 },
    { 0, 0 },
    { 0x3fa5213b, 55 },
    { 0, 0 },
    { 0x1b55213d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb52140, 95 },
    { 0x83452141, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f52146, 33 },
    { 0xa1452147, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8115214a, 43 },
    { 0x4a05214b, 87 },
    { 0xf1e5214c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd52150, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68252153, 51 },
    { 0, 0 },
    { 0x7452155, 99 },
    { 0x4d052155, 23 },
    { 0xfa952157, 84 },
    { 0, 0 },
    { 0x7652159, 33 },
    { 0, 0 },
    { 0x7615215b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73352160, 34 },
    { 0xc7652161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d52164, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e52167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2435216c, 98 },
    { 0x7a75216c, 33 },
    { 0x2d5216e, 90 },
    { 0x6635216f, 33 },
    { 0x67752170, 34 },
    { 0x63d52171, 99 },
    { 0x2b55216f, 61 },
    { 0x3ee52173, 33 },
    { 0xfd752174, 90 },
    { 0x8a52173, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195217f, 61 },
    { 0x84352180, 90 },
    { 0, 0 },
    { 0x53352182, 23 },
    { 0x8b452183, 33 },
    { 0xa1152182, 33 },
    { 0x54b52183, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc052188, 55 },
    { 0, 0 },
    { 0x57f5218a, 93 },
    { 0x48d5218b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9365218e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa5219f, 90 },
    { 0, 0 },
    { 0xc3c521a1, 23 },
    { 0, 0 },
    { 0x7e7521a3, 76 },
    { 0xb04521a4, 90 },
    { 0x4e7521a4, 99 },
    { 0, 0 },
    { 0x621521a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a521ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x175521ae, 55 },
    { 0x8e1521ae, 95 },
    { 0xb0f521af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af521b9, 61 },
    { 0, 0 },
    { 0x682521bb, 64 },
    { 0, 0 },
    { 0xd2a521bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9521c2, 33 },
    { 0xb42521c2, 98 },
    { 0x740521c4, 98 },
    { 0x177521c5, 90 },
    { 0xda7521c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3521cd, 61 },
    { 0xc53521cd, 34 },
    { 0, 0 },
    { 0x7c2521d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea521d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4521d9, 61 },
    { 0xf2521da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12521e2, 64 },
    { 0xc48521e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28521e6, 61 },
    { 0xa47521e7, 23 },
    { 0, 0 },
    { 0xc8f521e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14521ec, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7521f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6521f4, 96 },
    { 0x541521f5, 98 },
    { 0x54e521f5, 55 },
    { 0xe45521f7, 90 },
    { 0xc8d521f8, 33 },
    { 0xab2521f5, 100 },
    { 0xa0c521fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca52201, 95 },
    { 0x17452202, 33 },
    { 0x1f52203, 33 },
    { 0x4f252202, 23 },
    { 0x60452205, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed852209, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc935220e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa052212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305221c, 83 },
    { 0xfd15221d, 93 },
    { 0xad25221e, 61 },
    { 0x2f45221e, 33 },
    { 0x62152220, 55 },
    { 0, 0 },
    { 0xe7e52222, 90 },
    { 0, 0 },
    { 0xc1052224, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f52228, 95 },
    { 0x5b852228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d5222c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe52235, 55 },
    { 0xa2e52235, 33 },
    { 0x26652237, 91 },
    { 0xddc52236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf65223d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e52246, 95 },
    { 0x54452247, 98 },
    { 0, 0 },
    { 0x63852249, 68 },
    { 0x74f52249, 90 },
    { 0xde5224b, 61 },
    { 0xb005224c, 61 },
    { 0x1ca5224b, 33 },
    { 0xa8f5224d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d52252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d52255, 17 },
    { 0x18b52256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa465225d, 66 },
    { 0x6435225d, 90 },
    { 0xf4b5225f, 93 },
    { 0x65952260, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f5226d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf465227b, 33 },
    { 0, 0 },
    { 0x1a5227d, 90 },
    { 0xd8a5227e, 53 },
    { 0x435227f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7652282, 55 },
    { 0, 0 },
    { 0x5c852284, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d65228a, 84 },
    { 0x5cd5228b, 98 },
    { 0xa0a5228c, 76 },
    { 0xeee5228d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xac852290, 52 },
    { 0x5b752290, 100 },
    { 0, 0 },
    { 0x40952293, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98552296, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa235229c, 95 },
    { 0, 0 },
    { 0x1e45229e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68522a1, 34 },
    { 0xa5c522a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b522a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7522ad, 90 },
    { 0, 0 },
    { 0xa1a522af, 33 },
    { 0, 0 },
    { 0xeab522b1, 33 },
    { 0, 0 },
    { 0x733522b3, 65 },
    { 0xe4b522b3, 75 },
    { 0x39f522b5, 91 },
    { 0, 0 },
    { 0xf02522b7, 61 },
    { 0xc8a522b7, 95 },
    { 0xba2522b9, 95 },
    { 0x2cf522b7, 90 },
    { 0x852522b7, 84 },
    { 0xea522bc, 46 },
    { 0x651522bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283522c1, 46 },
    { 0x4b2522c1, 90 },
    { 0, 0 },
    { 0x420522c4, 33 },
    { 0, 0 },
    { 0xfbe522c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a522cc, 61 },
    { 0, 0 },
    { 0xad3522ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x401522d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03522d5, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db522da, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d522dd, 76 },
    { 0x1ee522de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912522e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a522e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce522ea, 90 },
    { 0xc0a522eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda522ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57522f1, 87 },
    { 0x9f5522f1, 52 },
    { 0x58b522f3, 90 },
    { 0x17d522f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c522f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f522fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf05230c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca352314, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c52319, 78 },
    { 0, 0 },
    { 0x8c45231b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35231e, 93 },
    { 0xb6f5231f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc652323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e5232c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde05232f, 98 },
    { 0xd1152330, 55 },
    { 0, 0 },
    { 0x25d52332, 96 },
    { 0, 0 },
    { 0x65652334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6252338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f5233b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb652340, 69 },
    { 0, 0 },
    { 0xff852342, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd352345, 76 },
    { 0, 0 },
    { 0xc1652347, 90 },
    { 0xac952347, 33 },
    { 0x21f52349, 33 },
    { 0x50a52349, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c5234d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4352353, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4652357, 90 },
    { 0, 0 },
    { 0x5f152359, 90 },
    { 0x89552359, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6135235d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9452363, 61 },
    { 0, 0 },
    { 0x2c152365, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae5236f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac52377, 65 },
    { 0x18152378, 88 },
    { 0xa5652379, 33 },
    { 0x72a52377, 98 },
    { 0x68e52377, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca52380, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2852384, 87 },
    { 0, 0 },
    { 0x1052386, 52 },
    { 0xf3a52387, 52 },
    { 0, 0 },
    { 0xdff52389, 69 },
    { 0, 0 },
    { 0x35e5238b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4545238f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e52393, 61 },
    { 0, 0 },
    { 0x9bb52395, 95 },
    { 0xd7c52396, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a523a0, 76 },
    { 0x1c0523a1, 90 },
    { 0x473523a2, 23 },
    { 0x6c4523a0, 90 },
    { 0x800523a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65523a7, 76 },
    { 0x190523a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x508523ab, 90 },
    { 0x4da523ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193523b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945523b8, 90 },
    { 0x434523b9, 90 },
    { 0, 0 },
    { 0x166523bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471523c0, 95 },
    { 0, 0 },
    { 0x345523c2, 34 },
    { 0xff2523c3, 90 },
    { 0x4e9523c3, 93 },
    { 0xb39523c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5523c9, 93 },
    { 0x639523ca, 61 },
    { 0x688523ca, 61 },
    { 0x444523ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x748523cf, 98 },
    { 0xfd0523d0, 61 },
    { 0, 0 },
    { 0x279523d2, 23 },
    { 0x6e5523d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31523d9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb523de, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873523e5, 90 },
    { 0, 0 },
    { 0xa64523e7, 91 },
    { 0x89d523e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0523ee, 66 },
    { 0x228523ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156523f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324523fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c52400, 61 },
    { 0, 0 },
    { 0x1d952402, 76 },
    { 0x24752403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29952411, 95 },
    { 0xa1352412, 93 },
    { 0x95b52413, 69 },
    { 0x8b652413, 47 },
    { 0xb6552412, 34 },
    { 0x2ff52416, 53 },
    { 0xfcf52416, 52 },
    { 0xe9d52414, 61 },
    { 0xa2d52419, 75 },
    { 0xc2c52419, 76 },
    { 0xbcf5241b, 90 },
    { 0, 0 },
    { 0x9a25241d, 93 },
    { 0, 0 },
    { 0x8645241f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7352422, 43 },
    { 0x23952423, 90 },
    { 0x2a52422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5242f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad52433, 33 },
    { 0xf3252434, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5243a, 90 },
    { 0, 0 },
    { 0xcca5243c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3452440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5052445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33252448, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef5244c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda52456, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd5245d, 33 },
    { 0xba15245d, 55 },
    { 0xa6c5245d, 55 },
    { 0, 0 },
    { 0x15752461, 95 },
    { 0x9de52461, 34 },
    { 0xb052463, 100 },
    { 0xc0752464, 61 },
    { 0, 0 },
    { 0xd5052466, 56 },
    { 0xe9b52466, 90 },
    { 0xa9752468, 90 },
    { 0x81052468, 98 },
    { 0xae252468, 99 },
    { 0x7df52469, 90 },
    { 0xf2052468, 23 },
    { 0, 0 },
    { 0x3425246e, 76 },
    { 0xfa35246f, 76 },
    { 0x5665246f, 95 },
    { 0x28852471, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed052474, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5652477, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b95247d, 98 },
    { 0, 0 },
    { 0xe595247f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45752482, 23 },
    { 0, 0 },
    { 0x45652484, 33 },
    { 0x13e52485, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6405248a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf52491, 95 },
    { 0x8ef52491, 90 },
    { 0, 0 },
    { 0x16552494, 90 },
    { 0, 0 },
    { 0x83452496, 76 },
    { 0xc6052497, 100 },
    { 0xeba52497, 23 },
    { 0, 0 },
    { 0x12b5249a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9855249f, 93 },
    { 0, 0 },
    { 0x4fe524a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1524a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af524ae, 43 },
    { 0, 0 },
    { 0xb10524b0, 76 },
    { 0xbc4524b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82524b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9524bd, 90 },
    { 0x11e524bd, 99 },
    { 0x3c0524bf, 93 },
    { 0xea0524bf, 100 },
    { 0, 0 },
    { 0x85a524c2, 95 },
    { 0xa27524c3, 90 },
    { 0xa5b524c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2524ca, 33 },
    { 0xc4f524ca, 33 },
    { 0x3524ca, 90 },
    { 0x17e524cd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e524d2, 23 },
    { 0, 0 },
    { 0xef8524d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d524d7, 81 },
    { 0x7bb524d8, 55 },
    { 0x87c524d9, 90 },
    { 0x66e524d9, 17 },
    { 0x5e5524db, 77 },
    { 0, 0 },
    { 0x55a524dd, 98 },
    { 0x2e5524dd, 40 },
    { 0, 0 },
    { 0xf18524e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64524e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df524ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4524f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1524f6, 76 },
    { 0, 0 },
    { 0x24f524f8, 98 },
    { 0, 0 },
    { 0xa35524fa, 95 },
    { 0xf75524fb, 33 },
    { 0xff0524fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f52500, 23 },
    { 0x7cd52501, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c52505, 95 },
    { 0, 0 },
    { 0xd9452507, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d5250b, 95 },
    { 0xbb85250c, 68 },
    { 0x3d55250c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8645251a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd315251d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17852523, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2935252a, 34 },
    { 0, 0 },
    { 0x8a65252c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7552532, 77 },
    { 0xd6e52533, 55 },
    { 0, 0 },
    { 0x19352535, 91 },
    { 0x4b952535, 93 },
    { 0, 0 },
    { 0xf1952538, 55 },
    { 0x96552539, 46 },
    { 0xf915253a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc895253e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c552542, 85 },
    { 0xa452542, 90 },
    { 0xba552543, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9652549, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf352550, 61 },
    { 0xeba52551, 90 },
    { 0, 0 },
    { 0x9bc52553, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x48652556, 95 },
    { 0, 0 },
    { 0x19652558, 34 },
    { 0x59652559, 90 },
    { 0x3b55255a, 66 },
    { 0x2ce5255b, 61 },
    { 0x70d5255a, 76 },
    { 0x7425255d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f52561, 98 },
    { 0xd9352562, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58652568, 95 },
    { 0x10952568, 88 },
    { 0x13f52569, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15256e, 60 },
    { 0, 0 },
    { 0xc2952570, 90 },
    { 0x49652571, 90 },
    { 0x98052572, 90 },
    { 0x78852573, 33 },
    { 0xe7152572, 23 },
    { 0x50752573, 46 },
    { 0xcf952576, 75 },
    { 0x51852574, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d5257a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84952580, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6452583, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f52587, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b5258a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d352591, 55 },
    { 0x69152592, 98 },
    { 0x5c352593, 98 },
    { 0x15d52591, 90 },
    { 0xe1152595, 88 },
    { 0xf9e52595, 98 },
    { 0, 0 },
    { 0x26952598, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b5259d, 98 },
    { 0, 0 },
    { 0xd9b5259f, 77 },
    { 0x1cf525a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01525a3, 98 },
    { 0, 0 },
    { 0xd97525a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849525a9, 33 },
    { 0xceb525aa, 90 },
    { 0x6c6525aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0525ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a525b4, 61 },
    { 0x319525b4, 77 },
    { 0xb7a525b6, 61 },
    { 0, 0 },
    { 0xe10525b8, 93 },
    { 0, 0 },
    { 0xe01525ba, 33 },
    { 0, 0 },
    { 0xdb6525bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e525c1, 90 },
    { 0x6bb525c2, 90 },
    { 0x71525c3, 61 },
    { 0xaf0525c4, 55 },
    { 0xb11525c4, 78 },
    { 0xa3f525c6, 85 },
    { 0xca7525c7, 61 },
    { 0x64f525c1, 98 },
    { 0, 0 },
    { 0x77e525ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385525d1, 23 },
    { 0, 0 },
    { 0xf2e525d3, 76 },
    { 0, 0 },
    { 0xc9525d5, 33 },
    { 0, 0 },
    { 0xc73525d7, 99 },
    { 0x98b525d8, 52 },
    { 0x373525d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc525dd, 95 },
    { 0, 0 },
    { 0x956525df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd525e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879525e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x985525ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0525f2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4525f5, 52 },
    { 0x76525f6, 61 },
    { 0, 0 },
    { 0xa4f525f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f525ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c352604, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc5260a, 95 },
    { 0xdd55260b, 97 },
    { 0x2475260b, 61 },
    { 0x6705260a, 23 },
    { 0x5d05260e, 100 },
    { 0xba15260b, 82 },
    { 0, 0 },
    { 0x1f852611, 76 },
    { 0x2a452611, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a752616, 90 },
    { 0xd552617, 61 },
    { 0x51152617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165261c, 66 },
    { 0x2275261d, 91 },
    { 0x5dc5261e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f352624, 76 },
    { 0xc5952625, 61 },
    { 0, 0 },
    { 0x43352627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1535262d, 33 },
    { 0x4715262e, 98 },
    { 0xcd95262f, 95 },
    { 0xcaf52630, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a352636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a52639, 76 },
    { 0x9da5263a, 99 },
    { 0x8505263a, 33 },
    { 0x10f52639, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11352641, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8452647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae5264b, 31 },
    { 0x1a65264c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d52650, 23 },
    { 0x39752651, 66 },
    { 0, 0 },
    { 0xc8152653, 43 },
    { 0xc8052654, 43 },
    { 0, 0 },
    { 0x81452656, 55 },
    { 0, 0 },
    { 0x26d52658, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb5265f, 85 },
    { 0x10f52660, 61 },
    { 0xe525265f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa852667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5315266d, 33 },
    { 0xbd45266e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab52673, 55 },
    { 0, 0 },
    { 0xaa052675, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d5267b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5267e, 61 },
    { 0x3f75267f, 23 },
    { 0x9375267f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a52683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93652689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45452692, 95 },
    { 0x57552693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a52698, 95 },
    { 0x15e52698, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d65269d, 84 },
    { 0xe45269e, 61 },
    { 0, 0 },
    { 0xb74526a0, 34 },
    { 0x4c8526a1, 95 },
    { 0x5fe526a2, 98 },
    { 0x148526a1, 33 },
    { 0x58e526a4, 98 },
    { 0x937526a2, 22 },
    { 0x9ef526a6, 52 },
    { 0x636526a6, 90 },
    { 0, 0 },
    { 0xb00526a9, 61 },
    { 0x4d526aa, 55 },
    { 0, 0 },
    { 0xdc3526ac, 87 },
    { 0xed2526ad, 85 },
    { 0x8d0526ad, 55 },
    { 0, 0 },
    { 0x6c8526b0, 90 },
    { 0x1d526b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219526b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9526bd, 95 },
    { 0xe0a526be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332526c4, 95 },
    { 0xaef526c5, 98 },
    { 0, 0 },
    { 0x214526c7, 88 },
    { 0, 0 },
    { 0xa07526c9, 46 },
    { 0xad2526c9, 23 },
    { 0x504526cb, 23 },
    { 0x6bd526cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b526d2, 87 },
    { 0xe01526d3, 23 },
    { 0, 0 },
    { 0xf2c526d5, 90 },
    { 0xbe3526d6, 61 },
    { 0x29c526d6, 43 },
    { 0x7c1526d8, 72 },
    { 0xf73526d6, 34 },
    { 0xc1d526da, 78 },
    { 0x50b526db, 74 },
    { 0x684526db, 61 },
    { 0xb90526dd, 61 },
    { 0, 0 },
    { 0x6f7526df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6526e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6526e6, 76 },
    { 0x29b526e6, 84 },
    { 0x57e526e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c526ef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7526f3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a452705, 33 },
    { 0, 0 },
    { 0x9ce52707, 95 },
    { 0, 0 },
    { 0x93452709, 67 },
    { 0xedf52709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f75270e, 90 },
    { 0xfbb5270f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc352715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a052724, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e552727, 91 },
    { 0xc9a52728, 55 },
    { 0x23452729, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x7475272c, 87 },
    { 0xc0b5272c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca52734, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55652738, 69 },
    { 0, 0 },
    { 0x8415273a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e852740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34352744, 80 },
    { 0x41b52745, 90 },
    { 0x73652746, 76 },
    { 0x2b852747, 77 },
    { 0x33c52748, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b5274f, 78 },
    { 0, 0 },
    { 0xc1452751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d52755, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f52758, 23 },
    { 0xfce52758, 95 },
    { 0xe7b5275a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5275d, 33 },
    { 0x90c5275e, 33 },
    { 0xfd55275e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c52762, 90 },
    { 0x77152763, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b52768, 98 },
    { 0xba052768, 95 },
    { 0x7745276a, 93 },
    { 0, 0 },
    { 0xe4a5276c, 95 },
    { 0x5f35276d, 66 },
    { 0, 0 },
    { 0x8835276f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6452775, 61 },
    { 0x84152775, 93 },
    { 0x5e952776, 98 },
    { 0x94552776, 55 },
    { 0, 0 },
    { 0xdd75277a, 100 },
    { 0xe8f5277a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d5277f, 85 },
    { 0, 0 },
    { 0x2ec52781, 87 },
    { 0xdeb52782, 66 },
    { 0xabc52783, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa52788, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6855278b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a5278f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a352799, 93 },
    { 0xe3d52799, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e527a1, 95 },
    { 0xf18527a2, 87 },
    { 0x937527a2, 55 },
    { 0x800527a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a527a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c527ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b527b1, 61 },
    { 0, 0 },
    { 0x8e6527b3, 95 },
    { 0, 0 },
    { 0xc07527b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde527b9, 78 },
    { 0x922527ba, 33 },
    { 0x97b527bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1527c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04527ca, 90 },
    { 0, 0 },
    { 0x781527cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24527cf, 90 },
    { 0xbfb527d0, 23 },
    { 0, 0 },
    { 0x91e527d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3527d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0527db, 23 },
    { 0, 0 },
    { 0xfc527dd, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4527e0, 23 },
    { 0, 0 },
    { 0xe15527e2, 46 },
    { 0xe07527e2, 42 },
    { 0xfca527e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6527f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f527f6, 100 },
    { 0x72d527f6, 23 },
    { 0xcfa527f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57527fb, 84 },
    { 0xe24527fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x38527ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab52804, 17 },
    { 0, 0 },
    { 0x55152806, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b5280a, 90 },
    { 0, 0 },
    { 0x7ef5280c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc55280f, 80 },
    { 0x4dc52810, 95 },
    { 0xb7a52811, 90 },
    { 0x54152810, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb5281a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c5281e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6152827, 33 },
    { 0xaa352827, 33 },
    { 0x1a552827, 61 },
    { 0x98d5282a, 93 },
    { 0xcaf52829, 90 },
    { 0x3335282c, 33 },
    { 0, 0 },
    { 0x26c5282e, 99 },
    { 0, 0 },
    { 0x6bc52830, 66 },
    { 0xc4252831, 95 },
    { 0xde952832, 34 },
    { 0x61c52830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c52836, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd5283a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb52842, 75 },
    { 0, 0 },
    { 0xf5652844, 90 },
    { 0x91052845, 33 },
    { 0xec52846, 34 },
    { 0x15952847, 61 },
    { 0x5f152848, 56 },
    { 0xccf52846, 76 },
    { 0x7785284a, 100 },
    { 0xb1652847, 33 },
    { 0x12652844, 75 },
    { 0xc8b52848, 93 },
    { 0x3fc52844, 81 },
    { 0x5425284f, 91 },
    { 0x9ed52850, 52 },
    { 0xd0252851, 91 },
    { 0xa3e52852, 76 },
    { 0x51f52852, 33 },
    { 0x43c5284a, 90 },
    { 0, 0 },
    { 0xc4c52856, 93 },
    { 0xc1252857, 83 },
    { 0x6f552858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e65285b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5285f, 76 },
    { 0xd985285f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4152865, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80452868, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa052876, 91 },
    { 0xf6d52877, 80 },
    { 0x45852878, 61 },
    { 0x52552877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc52880, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x25252883, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c452889, 33 },
    { 0xe3a5288a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf45288e, 33 },
    { 0x2d65288f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa52893, 34 },
    { 0xa6452894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a52899, 90 },
    { 0xb7b5289a, 23 },
    { 0x50452899, 61 },
    { 0x9175289c, 98 },
    { 0x9b25289c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41528a5, 65 },
    { 0x6de528a5, 66 },
    { 0x8cf528a5, 34 },
    { 0x68f528a6, 23 },
    { 0xf47528a9, 91 },
    { 0xa51528aa, 46 },
    { 0x9a8528a8, 55 },
    { 0, 0 },
    { 0xee4528ad, 90 },
    { 0xab6528ae, 90 },
    { 0, 0 },
    { 0x5cb528b0, 61 },
    { 0x4fe528b1, 33 },
    { 0x520528b1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3528ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462528be, 78 },
    { 0, 0 },
    { 0x45528c0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50528c4, 61 },
    { 0, 0 },
    { 0x1c528c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82528cc, 83 },
    { 0xed8528cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8528d0, 23 },
    { 0xaf7528d1, 80 },
    { 0x254528d1, 95 },
    { 0, 0 },
    { 0x552528d4, 75 },
    { 0x6ac528d5, 53 },
    { 0xfea528d5, 23 },
    { 0xab6528d7, 98 },
    { 0x83e528d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09528e1, 90 },
    { 0, 0 },
    { 0xc90528e3, 61 },
    { 0xf59528e4, 90 },
    { 0x289528e4, 23 },
    { 0xed8528e6, 90 },
    { 0x8bf528e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db528ec, 100 },
    { 0, 0 },
    { 0x318528ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4528f3, 84 },
    { 0, 0 },
    { 0xb26528f5, 80 },
    { 0x48a528f6, 55 },
    { 0xdf9528f6, 90 },
    { 0x384528f8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f528fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd652901, 55 },
    { 0x60352902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e052908, 36 },
    { 0, 0 },
    { 0x7e05290a, 23 },
    { 0, 0 },
    { 0xa325290c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d352913, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad52919, 34 },
    { 0xd265291a, 34 },
    { 0x64b5291b, 34 },
    { 0, 0 },
    { 0x58f5291d, 43 },
    { 0xec75291e, 33 },
    { 0, 0 },
    { 0xefd52920, 98 },
    { 0x1252920, 17 },
    { 0x6cf52922, 33 },
    { 0xd8552922, 75 },
    { 0xfa152924, 43 },
    { 0xf9e52925, 98 },
    { 0xe6452926, 46 },
    { 0x9c52925, 61 },
    { 0x92152922, 95 },
    { 0x68752929, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa485292f, 90 },
    { 0x8215292f, 55 },
    { 0xb7d52930, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe52934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb085293d, 46 },
    { 0x30d5293d, 69 },
    { 0x3ce5293f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38852946, 76 },
    { 0xc8652947, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf5294b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6915294e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7552958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf5295b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb5295e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec852961, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad52965, 99 },
    { 0x5c652965, 95 },
    { 0, 0 },
    { 0xe0152968, 33 },
    { 0x60252968, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f52973, 90 },
    { 0xc2f52974, 87 },
    { 0xa5252974, 33 },
    { 0xa3052976, 46 },
    { 0, 0 },
    { 0xb8f52978, 93 },
    { 0, 0 },
    { 0x3335297a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98752981, 98 },
    { 0, 0 },
    { 0xeb452983, 61 },
    { 0xbe252983, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x21852987, 78 },
    { 0x37352988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe552994, 33 },
    { 0xdb452995, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb52998, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde5299b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31529a2, 23 },
    { 0, 0 },
    { 0xf9c529a4, 46 },
    { 0xff4529a5, 33 },
    { 0xf1e529a6, 99 },
    { 0xc45529a7, 95 },
    { 0xae8529a8, 61 },
    { 0xe79529a7, 95 },
    { 0xf16529aa, 99 },
    { 0x29e529a8, 87 },
    { 0xe87529a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0529b3, 84 },
    { 0x82c529b4, 33 },
    { 0x897529b5, 61 },
    { 0x6b529b6, 33 },
    { 0xd37529b6, 55 },
    { 0x9b9529b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x383529bb, 33 },
    { 0xff9529bc, 34 },
    { 0x1b6529bd, 23 },
    { 0xa64529be, 61 },
    { 0x916529bb, 99 },
    { 0x14a529c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x179529c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6529c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f529d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50529d7, 61 },
    { 0x758529d8, 98 },
    { 0xd76529d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7529df, 43 },
    { 0, 0 },
    { 0xc28529e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75529e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a529e7, 33 },
    { 0, 0 },
    { 0x7fa529e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a529ed, 90 },
    { 0x516529ee, 88 },
    { 0, 0 },
    { 0x29f529f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36529f8, 90 },
    { 0x539529f9, 88 },
    { 0xb49529fa, 90 },
    { 0xca529fb, 90 },
    { 0x759529fc, 95 },
    { 0xf15529fc, 90 },
    { 0, 0 },
    { 0xb04529ff, 76 },
    { 0xcc752a00, 23 },
    { 0x6cc52a01, 46 },
    { 0x40b52a02, 23 },
    { 0xaf52a01, 12 },
    { 0x69d52a04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a652a08, 33 },
    { 0xbe252a09, 90 },
    { 0, 0 },
    { 0xed652a0b, 98 },
    { 0x95c52a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f52a10, 98 },
    { 0x26552a10, 31 },
    { 0x14452a12, 95 },
    { 0, 0 },
    { 0xdcd52a14, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c52a17, 69 },
    { 0xdac52a18, 90 },
    { 0x23f52a19, 33 },
    { 0xeb152a1a, 23 },
    { 0xad052a17, 17 },
    { 0xad852a1c, 55 },
    { 0xcd52a1d, 95 },
    { 0x5aa52a19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f52a21, 87 },
    { 0xb5f52a22, 95 },
    { 0xe0452a23, 34 },
    { 0xde052a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e52a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5452a2c, 93 },
    { 0x68e52a2d, 33 },
    { 0, 0 },
    { 0x4fb52a2f, 46 },
    { 0x74552a30, 33 },
    { 0, 0 },
    { 0x8b52a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb652a35, 69 },
    { 0x97952a36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c52a3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9952a45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb552a48, 23 },
    { 0xd1252a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e52a4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac952a51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47452a54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49852a5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34252a63, 76 },
    { 0, 0 },
    { 0xe9452a65, 33 },
    { 0x79c52a65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57052a6a, 97 },
    { 0, 0 },
    { 0x6c052a6c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d52a70, 96 },
    { 0xae852a70, 33 },
    { 0x50352a72, 95 },
    { 0, 0 },
    { 0x73c52a74, 67 },
    { 0xb2552a75, 95 },
    { 0, 0 },
    { 0x13e52a77, 98 },
    { 0xfdb52a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea52a80, 61 },
    { 0x11052a80, 95 },
    { 0x3f552a82, 61 },
    { 0, 0 },
    { 0x5ae52a84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af52a88, 65 },
    { 0, 0 },
    { 0x89b52a8a, 90 },
    { 0, 0 },
    { 0x43a52a8c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde352a93, 85 },
    { 0, 0 },
    { 0xff352a95, 90 },
    { 0x1c252a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd752a99, 69 },
    { 0, 0 },
    { 0x74452a9b, 43 },
    { 0xbb52a9b, 61 },
    { 0xaa52a9d, 100 },
    { 0, 0 },
    { 0x50f52a9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c252aa7, 80 },
    { 0xcfc52aa8, 43 },
    { 0x39452aa8, 38 },
    { 0xc3052aa8, 55 },
    { 0x11c52aab, 33 },
    { 0xa4652aab, 90 },
    { 0, 0 },
    { 0xe0952aae, 95 },
    { 0x8f552aae, 34 },
    { 0, 0 },
    { 0xd1852ab1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3952ab4, 23 },
    { 0, 0 },
    { 0xfaf52ab6, 87 },
    { 0x1d152ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd52ac4, 66 },
    { 0xa2352ac5, 90 },
    { 0x54d52ac5, 99 },
    { 0xe2952ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e052aca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb552acd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a52ad3, 23 },
    { 0x3f152ad4, 90 },
    { 0x2ab52ad3, 76 },
    { 0xfc952ad6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3152adb, 33 },
    { 0xc6352adb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb052ae0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d452ae6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf52ae9, 88 },
    { 0x84152aea, 53 },
    { 0xa552aea, 95 },
    { 0xaa152ae9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc852af1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff752af7, 90 },
    { 0xad552af8, 61 },
    { 0xe6252af7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb352afd, 46 },
    { 0x4e652afd, 96 },
    { 0x53952afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe652b02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50952b07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d52b10, 84 },
    { 0x6f252b10, 23 },
    { 0, 0 },
    { 0xa1d52b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8552b18, 88 },
    { 0xc6452b18, 61 },
    { 0xfe552b1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x27252b1d, 76 },
    { 0x13552b1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee52b21, 78 },
    { 0xee352b21, 43 },
    { 0, 0 },
    { 0x81152b24, 33 },
    { 0xe1952b25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b52b28, 95 },
    { 0xecb52b28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40852b2c, 34 },
    { 0x1ce52b2c, 34 },
    { 0xd9452b2d, 33 },
    { 0x40352b2d, 66 },
    { 0x49752b2f, 46 },
    { 0x3f852b31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4652b34, 23 },
    { 0xc3c52b34, 76 },
    { 0xf3652b36, 55 },
    { 0xc7252b37, 55 },
    { 0, 0 },
    { 0x26752b39, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0352b3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27352b42, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4052b49, 23 },
    { 0x88352b4a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee52b4f, 33 },
    { 0, 0 },
    { 0xcd352b51, 61 },
    { 0x28152b52, 98 },
    { 0x18652b53, 58 },
    { 0x7fb52b54, 33 },
    { 0x17f52b55, 90 },
    { 0xba952b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37952b59, 55 },
    { 0, 0 },
    { 0x65b52b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac52b64, 52 },
    { 0, 0 },
    { 0xba852b66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f852b6c, 90 },
    { 0x8c852b6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46252b71, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada52b77, 95 },
    { 0x3d452b78, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a952b7b, 23 },
    { 0x63752b7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc152b80, 33 },
    { 0xae952b81, 90 },
    { 0x47052b82, 17 },
    { 0xabf52b81, 23 },
    { 0x2cb52b84, 98 },
    { 0x54452b81, 34 },
    { 0xc6552b86, 52 },
    { 0x38852b87, 66 },
    { 0x76752b86, 33 },
    { 0x32052b87, 96 },
    { 0xa7852b8a, 43 },
    { 0xcb952b8b, 75 },
    { 0xf8452b82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46252b94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7052b97, 61 },
    { 0x7dc52b97, 55 },
    { 0x6d452b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8452b9d, 55 },
    { 0xbb152b9e, 84 },
    { 0, 0 },
    { 0x40752ba0, 90 },
    { 0xddd52ba0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad052ba5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac52ba9, 61 },
    { 0, 0 },
    { 0x64e52bab, 33 },
    { 0x3b752bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a52baf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf752bb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa952bba, 64 },
    { 0, 0 },
    { 0x1bc52bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7552bc3, 34 },
    { 0x33752bc4, 93 },
    { 0, 0 },
    { 0xe3a52bc6, 46 },
    { 0, 0 },
    { 0x33652bc8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e52bcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31952bd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3452bd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4552bda, 100 },
    { 0xa5052bdb, 76 },
    { 0, 0 },
    { 0x2c52bdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21452be2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f52be7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d352bee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85852bf5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92552bfb, 91 },
    { 0x6f452bfc, 61 },
    { 0x8cc52bfc, 19 },
    { 0xdaa52bfe, 33 },
    { 0, 0 },
    { 0xf9e52c00, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3252c0a, 36 },
    { 0x22452c0b, 93 },
    { 0xbe452c0a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x53852c0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa52c1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a52c1f, 34 },
    { 0, 0 },
    { 0xe6652c21, 90 },
    { 0, 0 },
    { 0xb2552c23, 90 },
    { 0xce652c24, 95 },
    { 0x59d52c24, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a452c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf952c3f, 90 },
    { 0, 0 },
    { 0xd4052c41, 66 },
    { 0xb7052c42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11652c48, 33 },
    { 0xd8752c49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0852c50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1452c56, 90 },
    { 0xf8e52c57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae52c5c, 43 },
    { 0x28a52c5d, 78 },
    { 0x37252c5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db52c61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28452c69, 93 },
    { 0, 0 },
    { 0x70852c6b, 99 },
    { 0, 0 },
    { 0x25352c6d, 68 },
    { 0xa252c6e, 76 },
    { 0x33152c6f, 67 },
    { 0xfd452c6e, 33 },
    { 0x54f52c71, 55 },
    { 0xcd352c72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea52c76, 95 },
    { 0x89552c77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc52c7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1352c7f, 85 },
    { 0xe3e52c80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c952c86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9452c89, 33 },
    { 0x5af52c8a, 34 },
    { 0x6e152c8a, 66 },
    { 0x21052c8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae52c8f, 23 },
    { 0x69452c90, 98 },
    { 0xf8a52c8f, 61 },
    { 0x4e52c92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f52c9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a52ca1, 90 },
    { 0, 0 },
    { 0x8e552ca3, 33 },
    { 0, 0 },
    { 0x9c452ca5, 75 },
    { 0xeb152ca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b52ca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e52cac, 55 },
    { 0, 0 },
    { 0xf2a52cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66552cb2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0552cb5, 33 },
    { 0x3ad52cb5, 23 },
    { 0, 0 },
    { 0xdc652cb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5752cc1, 91 },
    { 0xe3c52cc1, 93 },
    { 0, 0 },
    { 0x97652cc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e552cce, 93 },
    { 0xc752ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca52cd5, 76 },
    { 0x6ef52cd5, 61 },
    { 0xaa152cd7, 95 },
    { 0xbd452cd6, 67 },
    { 0, 0 },
    { 0xf0e52cda, 99 },
    { 0x4fb52cdb, 77 },
    { 0, 0 },
    { 0x20152cdd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c652ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6552ce8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea52ced, 68 },
    { 0x7d52cee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36352cf2, 66 },
    { 0, 0 },
    { 0x8c252cf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48552cf9, 90 },
    { 0, 0 },
    { 0xcb252cfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d352cff, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7752d03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c52d09, 95 },
    { 0xa2852d0a, 33 },
    { 0, 0 },
    { 0x85952d0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc52d0f, 33 },
    { 0x2bc52d10, 81 },
    { 0x6a952d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24352d15, 64 },
    { 0xf6752d15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73552d19, 98 },
    { 0x2fc52d19, 90 },
    { 0, 0 },
    { 0xefd52d1c, 33 },
    { 0x63a52d1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8552d20, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51752d26, 34 },
    { 0x36052d27, 95 },
    { 0x9b252d27, 95 },
    { 0x5af52d26, 61 },
    { 0xed252d2a, 90 },
    { 0xd0452d2b, 46 },
    { 0x6fe52d2c, 91 },
    { 0x36f52d2c, 76 },
    { 0, 0 },
    { 0xd4f52d2f, 100 },
    { 0, 0 },
    { 0xf9652d31, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x11952d34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b52d3d, 92 },
    { 0x2fb52d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4052d42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd52d4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb052d52, 33 },
    { 0x9f852d52, 24 },
    { 0, 0 },
    { 0x76f52d55, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1652d5b, 33 },
    { 0xa9c52d5c, 98 },
    { 0x3d652d5c, 43 },
    { 0x7df52d5d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa252d63, 33 },
    { 0, 0 },
    { 0x45452d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e52d6a, 90 },
    { 0x8f52d6a, 61 },
    { 0xa1952d6b, 90 },
    { 0, 0 },
    { 0x78252d6e, 95 },
    { 0x17052d6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1052d77, 61 },
    { 0x30552d78, 98 },
    { 0x6a652d79, 33 },
    { 0x6b52d79, 23 },
    { 0x8cd52d7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb52d7f, 90 },
    { 0x60652d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a252d88, 87 },
    { 0x5be52d89, 33 },
    { 0, 0 },
    { 0xae152d8b, 38 },
    { 0x7a852d8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb52d8f, 78 },
    { 0x41352d90, 96 },
    { 0xe8852d8f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0152d98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3852d9e, 33 },
    { 0xa1752d9f, 90 },
    { 0x71352da0, 43 },
    { 0xe4b52da0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12352da7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8052daa, 52 },
    { 0xe6552daa, 98 },
    { 0, 0 },
    { 0x68752dad, 33 },
    { 0, 0 },
    { 0xc8552daf, 33 },
    { 0x20f52daf, 55 },
    { 0xedb52db1, 77 },
    { 0x5b152db2, 90 },
    { 0xb9152db2, 46 },
    { 0xca452db2, 43 },
    { 0x5e252db3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b52db8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42452dbf, 90 },
    { 0, 0 },
    { 0x1a952dc1, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b52dc5, 33 },
    { 0x3fd52dc6, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94552dcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2452dd1, 90 },
    { 0xf5552dd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64452dd5, 55 },
    { 0x1cf52dd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2252dd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b52de0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb52de3, 34 },
    { 0x2a152de4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae52de8, 66 },
    { 0x1bf52de9, 98 },
    { 0x7df52de9, 33 },
    { 0x43752deb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e52dee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c52df1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a52df6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e52df9, 95 },
    { 0xbbe52df9, 61 },
    { 0, 0 },
    { 0x70f52dfc, 55 },
    { 0x80352dfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e52e02, 90 },
    { 0xb752e02, 75 },
    { 0xeac52e02, 95 },
    { 0x6e552e05, 43 },
    { 0x72b52e06, 61 },
    { 0x41852e02, 90 },
    { 0xfce52e08, 34 },
    { 0, 0 },
    { 0x26252e0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24552e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc452e15, 61 },
    { 0, 0 },
    { 0xa4452e17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3052e1f, 90 },
    { 0x86652e20, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a52e23, 90 },
    { 0xeff52e24, 91 },
    { 0x95852e25, 95 },
    { 0xc6252e26, 17 },
    { 0xf6f52e27, 87 },
    { 0x74f52e28, 55 },
    { 0, 0 },
    { 0x2652e2a, 33 },
    { 0x61552e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74252e2e, 95 },
    { 0x5452e2f, 23 },
    { 0xb7152e30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59752e33, 33 },
    { 0, 0 },
    { 0x3e052e35, 95 },
    { 0, 0 },
    { 0x79a52e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21152e3b, 33 },
    { 0x39452e3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xab752e3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57352e42, 75 },
    { 0, 0 },
    { 0x72252e44, 88 },
    { 0x84252e45, 85 },
    { 0, 0 },
    { 0x752e47, 33 },
    { 0xb1052e48, 61 },
    { 0x16852e48, 66 },
    { 0x51352e4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23152e51, 23 },
    { 0x85452e52, 52 },
    { 0x94c52e51, 90 },
    { 0, 0 },
    { 0x40052e55, 17 },
    { 0xa9c52e55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55052e5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c852e62, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad52e68, 95 },
    { 0, 0 },
    { 0x2bc52e6a, 61 },
    { 0x24552e6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c52e6e, 61 },
    { 0, 0 },
    { 0xcc52e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac52e73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f52e79, 61 },
    { 0x9bb52e7a, 93 },
    { 0, 0 },
    { 0xfc452e7c, 23 },
    { 0x23b52e7d, 33 },
    { 0x2a52e7e, 33 },
    { 0x6d952e7e, 90 },
    { 0xa8552e80, 90 },
    { 0, 0 },
    { 0x51452e82, 34 },
    { 0, 0 },
    { 0x9352e84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52e8a, 91 },
    { 0xd9a52e8a, 61 },
    { 0xe4052e8c, 100 },
    { 0x3f052e8c, 90 },
    { 0x53c52e8b, 76 },
    { 0xa7c52e8f, 61 },
    { 0, 0 },
    { 0x3db52e91, 55 },
    { 0, 0 },
    { 0x38e52e93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd52e97, 100 },
    { 0xefb52e98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3952e9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa352ea6, 68 },
    { 0, 0 },
    { 0x4752ea8, 95 },
    { 0, 0 },
    { 0xee952eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e752ead, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a452eb0, 95 },
    { 0x10052eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f952eb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b52eb8, 95 },
    { 0xd1852eb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1152ebd, 99 },
    { 0x7c752ebe, 33 },
    { 0, 0 },
    { 0x78652ec0, 33 },
    { 0x6dd52ec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38052ec6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b52ec9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be52ecd, 94 },
    { 0, 0 },
    { 0x48052ecf, 33 },
    { 0, 0 },
    { 0xb1c52ed1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd52ed4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c52ed7, 61 },
    { 0, 0 },
    { 0xe8152ed9, 93 },
    { 0x88d52ed9, 66 },
    { 0, 0 },
    { 0xfa652edc, 61 },
    { 0, 0 },
    { 0x76452ede, 32 },
    { 0xcb452ede, 33 },
    { 0xc4452ee0, 38 },
    { 0x2f852ee0, 96 },
    { 0x9b252ee2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c52ee7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a452eec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd52eef, 23 },
    { 0x8f752ef0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c52ef4, 23 },
    { 0xece52ef5, 85 },
    { 0xe752ef4, 34 },
    { 0, 0 },
    { 0xcdb52ef8, 23 },
    { 0, 0 },
    { 0x3e952efa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec952f01, 34 },
    { 0xbd252f02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1052f07, 90 },
    { 0, 0 },
    { 0x70052f09, 23 },
    { 0, 0 },
    { 0x73f52f0b, 77 },
    { 0, 0 },
    { 0x35352f0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9952f15, 76 },
    { 0x41f52f16, 46 },
    { 0, 0 },
    { 0x20f52f18, 76 },
    { 0xb552f18, 95 },
    { 0, 0 },
    { 0xc3452f1b, 34 },
    { 0, 0 },
    { 0xa3652f1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90052f25, 98 },
    { 0x7aa52f26, 23 },
    { 0, 0 },
    { 0x3852f28, 61 },
    { 0x94f52f28, 34 },
    { 0, 0 },
    { 0xddf52f2b, 76 },
    { 0, 0 },
    { 0xce652f2d, 43 },
    { 0x68e52f2e, 95 },
    { 0x1e652f2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4352f33, 46 },
    { 0x21b52f33, 33 },
    { 0x3ba52f33, 23 },
    { 0, 0 },
    { 0x22452f37, 52 },
    { 0, 0 },
    { 0xa5d52f39, 55 },
    { 0xe0852f3a, 82 },
    { 0x56e52f39, 68 },
    { 0x26052f3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a52f3f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e052f42, 95 },
    { 0x70952f43, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce52f49, 95 },
    { 0x3f952f49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec52f55, 98 },
    { 0x84e52f56, 83 },
    { 0xea752f57, 23 },
    { 0x10452f56, 95 },
    { 0, 0 },
    { 0xaf052f5a, 81 },
    { 0, 0 },
    { 0x97952f5c, 76 },
    { 0xd8152f5d, 93 },
    { 0, 0 },
    { 0x2b152f5f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0852f63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10852f6a, 99 },
    { 0, 0 },
    { 0xfe052f6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9752f71, 23 },
    { 0x74b52f72, 93 },
    { 0x20c52f73, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b52f76, 87 },
    { 0, 0 },
    { 0x4a852f78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a52f7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf52f7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef52f82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21652f85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15652f8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f98, 95 },
    { 0, 0 },
    { 0xeb652f9a, 33 },
    { 0x42852f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb52fa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c52fa4, 34 },
    { 0x11f52fa4, 52 },
    { 0x85452fa6, 61 },
    { 0x34052fa6, 95 },
    { 0xe3c52fa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79652fae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d052fb3, 33 },
    { 0x21c52fb4, 61 },
    { 0, 0 },
    { 0x5d52fb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5952fbb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb052fc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x44452fc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d752fc8, 90 },
    { 0xcb752fc9, 90 },
    { 0x25552fca, 93 },
    { 0x65f52fcb, 99 },
    { 0x16752fca, 33 },
    { 0x65f52fc9, 23 },
    { 0xd5f52fce, 76 },
    { 0x8e852fcd, 90 },
    { 0xac452fc8, 61 },
    { 0x2ae52fd1, 93 },
    { 0, 0 },
    { 0x7fb52fd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c452fd6, 66 },
    { 0, 0 },
    { 0xa7352fd8, 95 },
    { 0x94552fd8, 90 },
    { 0x37652fda, 61 },
    { 0, 0 },
    { 0x76952fdc, 98 },
    { 0x1ae52fdd, 76 },
    { 0xe3f52fde, 91 },
    { 0, 0 },
    { 0xbf052fe0, 95 },
    { 0, 0 },
    { 0x7f452fe2, 84 },
    { 0, 0 },
    { 0xba52fe4, 38 },
    { 0xc3952fe4, 61 },
    { 0x71552fe6, 52 },
    { 0, 0 },
    { 0x1d752fe8, 61 },
    { 0x8ba52fe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c52fef, 33 },
    { 0xeaf52ff0, 99 },
    { 0x81752fef, 98 },
    { 0x72652ff2, 91 },
    { 0xb4352ff2, 87 },
    { 0x48a52ff4, 78 },
    { 0x7b652ff5, 90 },
    { 0xb7b52ff6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b52ff9, 78 },
    { 0x1252ffa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d53002, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0453005, 34 },
    { 0x15f53006, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad75300b, 23 },
    { 0, 0 },
    { 0x6d15300d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf53016, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f5301a, 76 },
    { 0, 0 },
    { 0x5995301c, 90 },
    { 0xd3f5301d, 46 },
    { 0x2af5301c, 98 },
    { 0, 0 },
    { 0x5bf53020, 98 },
    { 0x4053021, 98 },
    { 0xd4d53022, 61 },
    { 0x36353022, 90 },
    { 0x10a53024, 33 },
    { 0xbc253021, 90 },
    { 0x77953026, 99 },
    { 0x12053020, 100 },
    { 0, 0 },
    { 0x8e953029, 98 },
    { 0, 0 },
    { 0x6355302b, 55 },
    { 0, 0 },
    { 0x1b45302d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12753032, 93 },
    { 0xc0053033, 88 },
    { 0x7ec53032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4905303b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70453041, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a853044, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce75304c, 76 },
    { 0, 0 },
    { 0x3ac5304e, 99 },
    { 0x5005304f, 77 },
    { 0xf8653050, 99 },
    { 0xe245304e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d53055, 100 },
    { 0x67153055, 88 },
    { 0x59953056, 33 },
    { 0, 0 },
    { 0x67653059, 98 },
    { 0xc4a5305a, 33 },
    { 0x4bd5305b, 33 },
    { 0x2005305c, 97 },
    { 0x2415305c, 33 },
    { 0xb235305d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c53064, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf53067, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975306d, 98 },
    { 0x9ff5306e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de53071, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b053074, 66 },
    { 0x1ea53074, 61 },
    { 0x97353075, 90 },
    { 0, 0 },
    { 0x54453078, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f45307f, 23 },
    { 0x7ee53080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe53087, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f5308a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d653095, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5795309a, 91 },
    { 0xd615309b, 91 },
    { 0, 0 },
    { 0xb945309d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae530a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c530a9, 43 },
    { 0xe07530aa, 52 },
    { 0xed6530aa, 95 },
    { 0x8be530ac, 95 },
    { 0x6e1530ac, 23 },
    { 0x883530ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b530b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257530bb, 95 },
    { 0x984530bc, 34 },
    { 0xbe9530bd, 34 },
    { 0xfdc530be, 61 },
    { 0x35b530be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457530c3, 23 },
    { 0xdd530c3, 23 },
    { 0x37b530c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x208530c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709530cd, 90 },
    { 0x91d530cd, 33 },
    { 0, 0 },
    { 0x9b8530d0, 90 },
    { 0xc1a530d1, 77 },
    { 0x685530d1, 76 },
    { 0xf06530d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e530d6, 61 },
    { 0xc39530d7, 23 },
    { 0x9c0530d7, 34 },
    { 0xed8530d9, 33 },
    { 0xd7530da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283530de, 46 },
    { 0xaa5530df, 61 },
    { 0x9e530e0, 25 },
    { 0x9b9530e0, 65 },
    { 0xf2d530e2, 23 },
    { 0x8e3530e3, 51 },
    { 0x6fa530e3, 33 },
    { 0x6ad530e5, 95 },
    { 0x115530e5, 90 },
    { 0x1a3530e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57530ee, 98 },
    { 0x372530ef, 90 },
    { 0x26b530ef, 23 },
    { 0xb88530ef, 67 },
    { 0xf66530ef, 34 },
    { 0xa68530ef, 93 },
    { 0x1aa530f4, 90 },
    { 0x629530f5, 23 },
    { 0, 0 },
    { 0x97b530f7, 50 },
    { 0x460530f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc53100, 55 },
    { 0x82853100, 23 },
    { 0xb1d53102, 77 },
    { 0x78153103, 100 },
    { 0x67753103, 77 },
    { 0x9753105, 55 },
    { 0x16353104, 80 },
    { 0x1153102, 48 },
    { 0x26d53108, 52 },
    { 0xc2653109, 55 },
    { 0xb9a53108, 98 },
    { 0xa715310b, 95 },
    { 0xa3153108, 62 },
    { 0xacb53101, 99 },
    { 0x2655310e, 96 },
    { 0, 0 },
    { 0x20b53110, 83 },
    { 0xdcf53111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab65311a, 34 },
    { 0, 0 },
    { 0xba45311c, 34 },
    { 0x8ab5311c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2653122, 43 },
    { 0x63b53123, 33 },
    { 0x9b653123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da53128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b15312c, 33 },
    { 0x5c05312d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae53135, 33 },
    { 0xfdb53136, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de53139, 95 },
    { 0xccf53139, 100 },
    { 0x7b5313b, 33 },
    { 0x1ed5313c, 81 },
    { 0x1b253139, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22653141, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c53149, 34 },
    { 0, 0 },
    { 0x6055314b, 23 },
    { 0, 0 },
    { 0x87e5314d, 65 },
    { 0xa6f5314e, 46 },
    { 0x69c5314d, 33 },
    { 0x3c25314e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a53157, 93 },
    { 0x40d53158, 90 },
    { 0xdb053157, 91 },
    { 0xf7d5315a, 95 },
    { 0x4f53157, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e5315e, 52 },
    { 0xfa85315f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c45316f, 91 },
    { 0xa5f53170, 88 },
    { 0xa9953170, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a753176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d5317b, 61 },
    { 0, 0 },
    { 0xe195317d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71053188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5275318c, 55 },
    { 0xd465318d, 23 },
    { 0x43b5318c, 87 },
    { 0x1f45318f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e853196, 43 },
    { 0, 0 },
    { 0xd9d53198, 95 },
    { 0, 0 },
    { 0x5b35319a, 95 },
    { 0, 0 },
    { 0x4be5319c, 33 },
    { 0xec65319c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2531a3, 33 },
    { 0x116531a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb531a7, 90 },
    { 0x94a531a8, 61 },
    { 0xcd7531a9, 23 },
    { 0x5ef531a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c531ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f531b3, 76 },
    { 0x695531b3, 68 },
    { 0xee3531b5, 61 },
    { 0xd1a531b6, 98 },
    { 0x913531b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e531ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419531bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f531c3, 43 },
    { 0xf74531c3, 95 },
    { 0xd38531c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3531cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39531d0, 33 },
    { 0xae4531d1, 46 },
    { 0x3f5531d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0531d8, 61 },
    { 0xcff531d9, 43 },
    { 0x4ec531da, 68 },
    { 0x24a531db, 23 },
    { 0xd69531dc, 90 },
    { 0x824531d8, 98 },
    { 0, 0 },
    { 0xe79531df, 43 },
    { 0, 0 },
    { 0x3be531e1, 33 },
    { 0, 0 },
    { 0x701531e3, 98 },
    { 0x958531e3, 98 },
    { 0xb58531e5, 99 },
    { 0, 0 },
    { 0x8b1531e7, 90 },
    { 0x7f1531e8, 34 },
    { 0xd9d531e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d531ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21531ef, 98 },
    { 0, 0 },
    { 0xd0e531f1, 23 },
    { 0, 0 },
    { 0xbfe531f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a531f8, 99 },
    { 0x2da531f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7531ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15953209, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5320d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x47853210, 46 },
    { 0x7cb53210, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94953217, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c55321a, 98 },
    { 0x39c5321a, 76 },
    { 0, 0 },
    { 0x2aa5321d, 46 },
    { 0x3395321e, 23 },
    { 0x955321e, 98 },
    { 0x32e53220, 34 },
    { 0xce753221, 85 },
    { 0xdce5321e, 95 },
    { 0x6685321e, 98 },
    { 0x54653220, 46 },
    { 0x28253224, 23 },
    { 0xf4453226, 75 },
    { 0xaa553227, 33 },
    { 0x72253228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4295322d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0053230, 31 },
    { 0x90d53231, 95 },
    { 0, 0 },
    { 0x81f53233, 78 },
    { 0xfe53234, 61 },
    { 0xd3f53235, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b53239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c35323c, 98 },
    { 0x8ef5323d, 36 },
    { 0xca35323c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd53242, 33 },
    { 0x2bd53242, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a253249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde15324f, 23 },
    { 0, 0 },
    { 0xc253251, 55 },
    { 0, 0 },
    { 0xf3353253, 43 },
    { 0, 0 },
    { 0x19e53255, 23 },
    { 0x39a53256, 85 },
    { 0xcbc53257, 77 },
    { 0xb5b53258, 23 },
    { 0xfe653257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5325d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc353262, 46 },
    { 0x4a753263, 23 },
    { 0x36453264, 46 },
    { 0x1a753264, 23 },
    { 0xfca53265, 98 },
    { 0xbd953266, 61 },
    { 0x26653268, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5326c, 33 },
    { 0, 0 },
    { 0x1525326e, 64 },
    { 0x5d25326f, 95 },
    { 0, 0 },
    { 0x4c353271, 90 },
    { 0x33953271, 93 },
    { 0x69253273, 43 },
    { 0xf8853274, 33 },
    { 0xf2053275, 46 },
    { 0x69653274, 24 },
    { 0xd6f53273, 33 },
    { 0x90f53278, 76 },
    { 0x553279, 61 },
    { 0x1fe53272, 76 },
    { 0, 0 },
    { 0x2095327c, 17 },
    { 0xb995327d, 33 },
    { 0, 0 },
    { 0xb3a5327f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e53282, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b53285, 67 },
    { 0x26b53285, 88 },
    { 0x2c853287, 95 },
    { 0x9553288, 100 },
    { 0x4f553289, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce453291, 100 },
    { 0, 0 },
    { 0x62753293, 23 },
    { 0x62353294, 34 },
    { 0x4d053295, 33 },
    { 0x2a53295, 76 },
    { 0x2fe53297, 76 },
    { 0x21a53298, 33 },
    { 0, 0 },
    { 0x2665329a, 91 },
    { 0xb335329b, 61 },
    { 0x5ea5329a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55329f, 33 },
    { 0x82f5329f, 55 },
    { 0x696532a1, 38 },
    { 0, 0 },
    { 0x5f2532a3, 90 },
    { 0xddb532a3, 33 },
    { 0, 0 },
    { 0xbe0532a6, 98 },
    { 0xceb532a7, 90 },
    { 0x86a532a8, 46 },
    { 0x746532a9, 83 },
    { 0x359532a8, 84 },
    { 0x386532ab, 76 },
    { 0, 0 },
    { 0x26c532ad, 43 },
    { 0, 0 },
    { 0x66f532af, 69 },
    { 0, 0 },
    { 0x7d9532b1, 93 },
    { 0, 0 },
    { 0x123532b3, 90 },
    { 0, 0 },
    { 0x39d532b5, 33 },
    { 0, 0 },
    { 0x739532b7, 33 },
    { 0x37532b8, 78 },
    { 0, 0 },
    { 0xf3f532ba, 98 },
    { 0x358532bb, 46 },
    { 0, 0 },
    { 0xd75532bd, 61 },
    { 0xdcf532be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60532c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed532c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218532d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7532d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff532db, 33 },
    { 0x64d532dc, 46 },
    { 0x676532dd, 91 },
    { 0x729532db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5532e2, 66 },
    { 0x60e532e3, 93 },
    { 0xb86532e4, 33 },
    { 0xde8532e5, 98 },
    { 0x60f532e4, 34 },
    { 0, 0 },
    { 0x671532e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x108532eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a532ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6532f2, 95 },
    { 0xd8c532f3, 67 },
    { 0xc12532f3, 61 },
    { 0xba1532f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75953304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e153309, 55 },
    { 0x57f5330a, 78 },
    { 0x7c65330b, 77 },
    { 0x72253309, 98 },
    { 0xf6c5330d, 33 },
    { 0, 0 },
    { 0xf7d5330f, 98 },
    { 0x4ef53310, 95 },
    { 0, 0 },
    { 0x1ea53312, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16153315, 93 },
    { 0xda553316, 98 },
    { 0, 0 },
    { 0xd0053318, 33 },
    { 0, 0 },
    { 0x9cc5331a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d53321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52553324, 33 },
    { 0x8b53325, 81 },
    { 0x92253326, 91 },
    { 0x31e53327, 90 },
    { 0x95353328, 34 },
    { 0x31353328, 95 },
    { 0xb0c5332a, 78 },
    { 0x9305332b, 61 },
    { 0x4ad5332c, 87 },
    { 0x9bf5332d, 84 },
    { 0, 0 },
    { 0x23a5332f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee53338, 51 },
    { 0x36153339, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa5333d, 17 },
    { 0xad05333e, 34 },
    { 0, 0 },
    { 0xe7053340, 98 },
    { 0xe5153340, 93 },
    { 0xe9153341, 76 },
    { 0xb4a53343, 33 },
    { 0xa8f53342, 23 },
    { 0xfb253343, 36 },
    { 0x6e53345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda53351, 17 },
    { 0x9ab53352, 90 },
    { 0xb7653352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b5335f, 55 },
    { 0, 0 },
    { 0xa2453361, 23 },
    { 0x19f53362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48953365, 33 },
    { 0, 0 },
    { 0x2e453367, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4653370, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd053374, 98 },
    { 0x64c53375, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x96753378, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd275337b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9105337f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea5338e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea53393, 68 },
    { 0, 0 },
    { 0xba853395, 33 },
    { 0x26d53396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a53399, 90 },
    { 0xb0e53399, 90 },
    { 0xcb553399, 61 },
    { 0x5875339b, 43 },
    { 0xc8b5339b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb533a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be533ad, 96 },
    { 0x4e4533ae, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4533b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405533bc, 23 },
    { 0, 0 },
    { 0xa8c533be, 76 },
    { 0, 0 },
    { 0xe23533c0, 61 },
    { 0, 0 },
    { 0xd8e533c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744533c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5533ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8533ce, 52 },
    { 0, 0 },
    { 0x777533d0, 43 },
    { 0x6c1533d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9533d4, 68 },
    { 0, 0 },
    { 0xde2533d6, 23 },
    { 0x42e533d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17533db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3533e1, 95 },
    { 0x957533e2, 61 },
    { 0x3a7533e1, 76 },
    { 0, 0 },
    { 0xd53533e5, 100 },
    { 0x7af533e6, 68 },
    { 0x452533e6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1533ec, 61 },
    { 0x714533ed, 95 },
    { 0x5d9533ed, 90 },
    { 0x948533ed, 23 },
    { 0, 0 },
    { 0x783533f1, 61 },
    { 0xdfd533f2, 46 },
    { 0x5d2533f3, 23 },
    { 0x3a5533f4, 23 },
    { 0xd0a533f5, 61 },
    { 0x30533f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f533f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca533fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4753402, 84 },
    { 0xf4d53402, 23 },
    { 0, 0 },
    { 0x8d53405, 78 },
    { 0, 0 },
    { 0x50d53407, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25340a, 23 },
    { 0, 0 },
    { 0x4025340c, 56 },
    { 0x1fb5340c, 76 },
    { 0x1905340d, 17 },
    { 0, 0 },
    { 0x80b53410, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c953415, 93 },
    { 0x15053416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b53419, 85 },
    { 0x9d55341a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f5341e, 46 },
    { 0x80d5341f, 90 },
    { 0xc6353420, 4 },
    { 0x8e553421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa53426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6215342b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a5342f, 40 },
    { 0, 0 },
    { 0x27953431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa653434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5153439, 68 },
    { 0xdd95343a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7505343f, 61 },
    { 0xc6c53440, 95 },
    { 0xf0653440, 95 },
    { 0xfa253441, 95 },
    { 0x16c53441, 90 },
    { 0xcf053444, 23 },
    { 0x3d53445, 98 },
    { 0x3c553446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f53449, 84 },
    { 0, 0 },
    { 0x9915344b, 81 },
    { 0xa075344c, 55 },
    { 0, 0 },
    { 0x185344e, 98 },
    { 0x8c65344e, 46 },
    { 0x86253450, 95 },
    { 0xa2953450, 90 },
    { 0xd1f53451, 98 },
    { 0, 0 },
    { 0x653454, 34 },
    { 0xe4e53455, 76 },
    { 0, 0 },
    { 0x95e53457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1953468, 61 },
    { 0, 0 },
    { 0xef35346a, 23 },
    { 0x5555346a, 90 },
    { 0, 0 },
    { 0xc9b5346d, 61 },
    { 0x1a35346e, 33 },
    { 0x2fb5346f, 46 },
    { 0x48b5346f, 98 },
    { 0xbfa53470, 85 },
    { 0, 0 },
    { 0xd2b53473, 23 },
    { 0xb8b53474, 90 },
    { 0, 0 },
    { 0x9353476, 33 },
    { 0x40b53476, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe75347a, 67 },
    { 0xd515347b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5653481, 23 },
    { 0x85953482, 76 },
    { 0x39753483, 90 },
    { 0x66b53483, 46 },
    { 0xf1353483, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65053488, 98 },
    { 0xa3f53489, 76 },
    { 0xa285348a, 55 },
    { 0x1dc5348b, 52 },
    { 0xb8f5348b, 93 },
    { 0xf05348d, 90 },
    { 0x2f5348d, 43 },
    { 0xd205348a, 98 },
    { 0xd675348d, 61 },
    { 0xf1f53491, 98 },
    { 0x34d53489, 95 },
    { 0xc553488, 61 },
    { 0, 0 },
    { 0x76953495, 93 },
    { 0, 0 },
    { 0xc5d53497, 98 },
    { 0x11153498, 69 },
    { 0x1f953498, 98 },
    { 0xf4c53498, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c5349f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa534a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881534a7, 33 },
    { 0x281534a7, 90 },
    { 0x3c5534a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5534b1, 43 },
    { 0x7d3534b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92534b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865534c2, 98 },
    { 0x2df534c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e534c6, 46 },
    { 0x4ed534c7, 33 },
    { 0x394534c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1534cc, 84 },
    { 0, 0 },
    { 0x640534ce, 55 },
    { 0, 0 },
    { 0x3e9534d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b534d3, 43 },
    { 0xbfb534d4, 76 },
    { 0x6c4534d5, 78 },
    { 0, 0 },
    { 0x644534d7, 85 },
    { 0x2f5534d7, 67 },
    { 0xab6534d9, 98 },
    { 0x84b534da, 36 },
    { 0, 0 },
    { 0x743534dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7534e0, 23 },
    { 0x6b1534e0, 34 },
    { 0xb6e534e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191534e6, 76 },
    { 0x3d0534e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8534f2, 36 },
    { 0xb0e534f3, 61 },
    { 0x86c534f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74534fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c53502, 96 },
    { 0, 0 },
    { 0x36053504, 78 },
    { 0x94953505, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305350a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa525350d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b53510, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7853519, 93 },
    { 0xb8a53519, 33 },
    { 0xf5c53519, 69 },
    { 0, 0 },
    { 0x4e95351d, 91 },
    { 0x4b45351e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4753521, 90 },
    { 0x79a53521, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0553527, 61 },
    { 0x36d53528, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5525352b, 33 },
    { 0x2005352b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53653531, 90 },
    { 0x9ea53532, 90 },
    { 0xd0c53531, 76 },
    { 0, 0 },
    { 0xe0b53535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f53539, 98 },
    { 0x9755353a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe485353d, 77 },
    { 0xafb5353e, 90 },
    { 0xb235353d, 23 },
    { 0xb0053540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3853543, 66 },
    { 0x13e53544, 88 },
    { 0x97753544, 95 },
    { 0x28a53546, 61 },
    { 0xb6c53546, 98 },
    { 0x6fa53545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb425354b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25553553, 90 },
    { 0, 0 },
    { 0x2ab53555, 33 },
    { 0x1353556, 98 },
    { 0x1c853557, 15 },
    { 0xd5953556, 95 },
    { 0xdcb53559, 90 },
    { 0xa453559, 46 },
    { 0x9525355a, 34 },
    { 0xd0853559, 61 },
    { 0xc225355d, 95 },
    { 0xa2e5355e, 90 },
    { 0x825355f, 52 },
    { 0x20153560, 33 },
    { 0x75b5355f, 33 },
    { 0x3f553560, 61 },
    { 0x40c5355d, 90 },
    { 0xe3b53564, 55 },
    { 0x6c5355f, 91 },
    { 0xe253566, 90 },
    { 0x8d153567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1945356b, 90 },
    { 0, 0 },
    { 0x1115356d, 61 },
    { 0x3e85356e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x51453571, 33 },
    { 0x9d153572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c353579, 23 },
    { 0x64253579, 23 },
    { 0xbc75357b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e5357e, 68 },
    { 0x93a5357f, 95 },
    { 0x85953580, 90 },
    { 0, 0 },
    { 0x23053582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d153585, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d53589, 23 },
    { 0x1c053589, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd53590, 55 },
    { 0, 0 },
    { 0x42553592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x39953595, 76 },
    { 0xa7c53595, 93 },
    { 0, 0 },
    { 0x84153598, 98 },
    { 0xea953599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7955359d, 23 },
    { 0xf215359e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9535a4, 95 },
    { 0, 0 },
    { 0xc97535a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45535ab, 43 },
    { 0xc7e535ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb535b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76535b6, 78 },
    { 0, 0 },
    { 0x173535b8, 33 },
    { 0x860535b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf535be, 33 },
    { 0, 0 },
    { 0xa6a535c0, 55 },
    { 0, 0 },
    { 0xb71535c2, 66 },
    { 0, 0 },
    { 0x35f535c4, 77 },
    { 0x1e6535c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f535c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958535cc, 90 },
    { 0xef2535cc, 93 },
    { 0x26a535ce, 98 },
    { 0xbdb535cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a535d3, 55 },
    { 0, 0 },
    { 0xaec535d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9535dd, 23 },
    { 0, 0 },
    { 0x423535df, 36 },
    { 0x8a1535e0, 52 },
    { 0xc61535e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36535e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3535e7, 55 },
    { 0xafd535e8, 61 },
    { 0, 0 },
    { 0x576535ea, 34 },
    { 0, 0 },
    { 0x7fc535ec, 91 },
    { 0, 0 },
    { 0xff4535ee, 34 },
    { 0x937535ef, 90 },
    { 0x1eb535ee, 46 },
    { 0xe12535f1, 95 },
    { 0x42d535ee, 66 },
    { 0xa50535f3, 55 },
    { 0, 0 },
    { 0xd04535f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc535f9, 23 },
    { 0x3b2535fa, 81 },
    { 0xa41535fa, 90 },
    { 0, 0 },
    { 0x4e7535fd, 40 },
    { 0xa16535fe, 78 },
    { 0x30b535ff, 55 },
    { 0x15753600, 95 },
    { 0x9da53601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29553604, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86653607, 90 },
    { 0x45453608, 33 },
    { 0, 0 },
    { 0x1235360a, 90 },
    { 0xa9b5360b, 43 },
    { 0x9265360a, 87 },
    { 0x16f5360d, 98 },
    { 0xb795360c, 36 },
    { 0x8cc5360f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e253613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55253616, 65 },
    { 0x9c353616, 33 },
    { 0xd853618, 52 },
    { 0x6ed53619, 78 },
    { 0xe295361a, 77 },
    { 0x96d5361b, 55 },
    { 0xfa35361c, 90 },
    { 0xdcc53617, 23 },
    { 0x3ab5361e, 76 },
    { 0xd855361f, 61 },
    { 0x6935361e, 23 },
    { 0x4025361f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe653625, 46 },
    { 0x42053626, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34153629, 76 },
    { 0x165362a, 33 },
    { 0xca5362b, 34 },
    { 0x8815362c, 33 },
    { 0xcfc5362d, 98 },
    { 0x2ee5362c, 90 },
    { 0, 0 },
    { 0xc3f53630, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60653634, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575363a, 90 },
    { 0xbb05363a, 95 },
    { 0xb3c5363a, 33 },
    { 0x9a35363d, 55 },
    { 0, 0 },
    { 0x8765363f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7153643, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d53647, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d75364b, 100 },
    { 0, 0 },
    { 0x7345364d, 43 },
    { 0x47c5364e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f653651, 67 },
    { 0x74353652, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91953657, 98 },
    { 0xcb753658, 46 },
    { 0x42e53657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c5365c, 95 },
    { 0x2795365d, 76 },
    { 0, 0 },
    { 0xe365365f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80953663, 76 },
    { 0x7be53664, 56 },
    { 0xbad53665, 98 },
    { 0x83553664, 34 },
    { 0xdbe53664, 52 },
    { 0x67f53668, 85 },
    { 0x26f53669, 95 },
    { 0xc9053666, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b5366f, 96 },
    { 0x2b253670, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c53674, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc925367b, 38 },
    { 0x7fe5367c, 81 },
    { 0, 0 },
    { 0xcdb5367e, 61 },
    { 0, 0 },
    { 0xbf453680, 90 },
    { 0, 0 },
    { 0x71353682, 91 },
    { 0xa5d53683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d953686, 98 },
    { 0xb2553687, 98 },
    { 0, 0 },
    { 0x8f453689, 88 },
    { 0xa9553689, 61 },
    { 0x9a35368b, 100 },
    { 0xeae5368c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad753693, 95 },
    { 0, 0 },
    { 0xbe353695, 33 },
    { 0xb7953696, 23 },
    { 0x8e153697, 61 },
    { 0, 0 },
    { 0x11e53699, 23 },
    { 0, 0 },
    { 0x5945369b, 23 },
    { 0xe485369c, 98 },
    { 0xcbe5369b, 87 },
    { 0x8fd5369e, 33 },
    { 0, 0 },
    { 0x995536a0, 98 },
    { 0x4b536a1, 61 },
    { 0xcc8536a1, 46 },
    { 0x699536a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1536a7, 33 },
    { 0x754536a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x15536ab, 76 },
    { 0, 0 },
    { 0x85b536ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766536b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb536b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x165536ba, 76 },
    { 0x692536bb, 98 },
    { 0x204536bc, 33 },
    { 0, 0 },
    { 0xcb3536be, 98 },
    { 0x325536be, 33 },
    { 0, 0 },
    { 0xf53536c1, 61 },
    { 0xe62536c2, 55 },
    { 0xb6f536c3, 91 },
    { 0x9f7536c1, 98 },
    { 0x71536c2, 55 },
    { 0x50c536c4, 43 },
    { 0xd69536c3, 100 },
    { 0x7e5536c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d536cc, 23 },
    { 0, 0 },
    { 0xb9e536ce, 34 },
    { 0xa0f536ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28536d5, 68 },
    { 0xda1536d6, 100 },
    { 0, 0 },
    { 0x254536d8, 52 },
    { 0x3a6536d9, 98 },
    { 0x40d536da, 90 },
    { 0xcab536d9, 61 },
    { 0xfc9536dc, 17 },
    { 0xc07536d9, 61 },
    { 0, 0 },
    { 0x6cc536df, 61 },
    { 0xc8c536e0, 77 },
    { 0x8a536e1, 58 },
    { 0x836536e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x976536e5, 95 },
    { 0x5bc536e5, 84 },
    { 0x442536e7, 61 },
    { 0xac4536e5, 98 },
    { 0, 0 },
    { 0x13b536ea, 55 },
    { 0x462536eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4536f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3536f6, 90 },
    { 0, 0 },
    { 0x58b536f8, 33 },
    { 0x3536f8, 90 },
    { 0xbef536fa, 99 },
    { 0, 0 },
    { 0x711536fc, 61 },
    { 0x33d536fc, 91 },
    { 0, 0 },
    { 0x835536ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e153703, 98 },
    { 0, 0 },
    { 0xaad53705, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b53708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67053710, 66 },
    { 0x34053711, 61 },
    { 0x1a253712, 23 },
    { 0xaa953713, 77 },
    { 0x15e53712, 76 },
    { 0x28b53713, 90 },
    { 0xcfb53714, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5371e, 90 },
    { 0xdf95371f, 55 },
    { 0x1ef53720, 98 },
    { 0x3253721, 23 },
    { 0xbd53720, 33 },
    { 0x4bb53723, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73253729, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec5372c, 95 },
    { 0x39e5372d, 61 },
    { 0x415372e, 43 },
    { 0, 0 },
    { 0xd2a53730, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b53733, 91 },
    { 0x25153733, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e53737, 33 },
    { 0xfa253738, 34 },
    { 0x7e953738, 88 },
    { 0xa7753739, 98 },
    { 0x80653738, 90 },
    { 0, 0 },
    { 0xb6e5373d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66853742, 75 },
    { 0x42e53743, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee5374b, 61 },
    { 0, 0 },
    { 0x92c5374d, 43 },
    { 0xbba5374e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd53754, 46 },
    { 0, 0 },
    { 0xa4e53756, 90 },
    { 0x7f453757, 55 },
    { 0x76953756, 95 },
    { 0x30753759, 90 },
    { 0x5f35375a, 55 },
    { 0xabe5375a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a53763, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37853774, 33 },
    { 0x96a53775, 95 },
    { 0x99853775, 90 },
    { 0x9a153777, 72 },
    { 0x36c53775, 55 },
    { 0x90153775, 95 },
    { 0xb1a5377a, 61 },
    { 0xea35377a, 82 },
    { 0x8d5377b, 55 },
    { 0x56253779, 90 },
    { 0, 0 },
    { 0x1b85377f, 90 },
    { 0xa8f5377f, 33 },
    { 0xac553781, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67553785, 77 },
    { 0x75553786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165378b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5378e, 34 },
    { 0x5de5378f, 46 },
    { 0x2465378e, 33 },
    { 0x14653791, 34 },
    { 0xa735378e, 85 },
    { 0, 0 },
    { 0x50b53794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8753797, 34 },
    { 0xb6053798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5379c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d537a0, 67 },
    { 0, 0 },
    { 0x9cb537a2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875537a8, 77 },
    { 0x921537a9, 23 },
    { 0xb3537a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa537af, 91 },
    { 0, 0 },
    { 0xdb2537b1, 61 },
    { 0xc79537b2, 98 },
    { 0x430537b2, 100 },
    { 0x7bf537b2, 90 },
    { 0x4ad537b3, 93 },
    { 0x6ea537b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f537b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b537bc, 90 },
    { 0x8b1537bd, 33 },
    { 0x44e537bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0537c1, 69 },
    { 0x431537c1, 98 },
    { 0, 0 },
    { 0x2f3537c4, 61 },
    { 0x285537c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b537c9, 67 },
    { 0xe02537c9, 98 },
    { 0xb35537cb, 23 },
    { 0, 0 },
    { 0xeac537cd, 61 },
    { 0x4b4537cd, 61 },
    { 0x910537cf, 90 },
    { 0, 0 },
    { 0xe34537d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd537d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31537da, 34 },
    { 0, 0 },
    { 0x2e5537dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8537e0, 33 },
    { 0xd8f537e1, 61 },
    { 0x86d537e2, 52 },
    { 0xe21537e3, 52 },
    { 0x60c537e4, 88 },
    { 0x553537e4, 33 },
    { 0xce8537e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682537eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x174537ee, 98 },
    { 0, 0 },
    { 0x550537f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464537fa, 78 },
    { 0x9ee537fb, 43 },
    { 0, 0 },
    { 0x1d537fd, 95 },
    { 0xf85537fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef53803, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b5380a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7785380d, 23 },
    { 0, 0 },
    { 0x9e65380f, 55 },
    { 0x65253810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61653814, 61 },
    { 0, 0 },
    { 0x14753816, 98 },
    { 0x62e53817, 90 },
    { 0, 0 },
    { 0xee753819, 34 },
    { 0xf1e5381a, 88 },
    { 0xd6c53819, 87 },
    { 0xce55381c, 33 },
    { 0xf5381d, 98 },
    { 0x5265381e, 33 },
    { 0x9405381d, 91 },
    { 0x7753820, 66 },
    { 0x51c53821, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8453826, 33 },
    { 0x87f53826, 98 },
    { 0xd9e53827, 90 },
    { 0x48f53829, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e25382c, 87 },
    { 0, 0 },
    { 0xc35382e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b53832, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9305383b, 99 },
    { 0x54c5383b, 33 },
    { 0xd965383b, 55 },
    { 0xe895383e, 84 },
    { 0xbf35383f, 88 },
    { 0, 0 },
    { 0x23653841, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c53845, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c5384b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a65384e, 76 },
    { 0x66c5384f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb53852, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d253855, 34 },
    { 0, 0 },
    { 0x58c53857, 34 },
    { 0, 0 },
    { 0xd3153859, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5385c, 90 },
    { 0xcc75385d, 33 },
    { 0x6a65385e, 98 },
    { 0x1c25385f, 93 },
    { 0x79e53860, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9553864, 68 },
    { 0, 0 },
    { 0xcc253866, 93 },
    { 0x6dd53867, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e5386c, 66 },
    { 0x5ae5386d, 80 },
    { 0x2635386d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82653871, 33 },
    { 0x6a653871, 91 },
    { 0x1a853871, 94 },
    { 0x9b753874, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93953877, 76 },
    { 0xe4853878, 61 },
    { 0xb5553878, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f5387d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8753882, 61 },
    { 0, 0 },
    { 0x8e53884, 61 },
    { 0x9b153884, 38 },
    { 0xcda53886, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b5388b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e253893, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c53898, 33 },
    { 0xb9a53898, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d5389c, 33 },
    { 0, 0 },
    { 0xf6b5389e, 65 },
    { 0, 0 },
    { 0xabd538a0, 55 },
    { 0x7bf538a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15538ab, 84 },
    { 0x333538ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da538af, 66 },
    { 0xd5a538b0, 85 },
    { 0x715538af, 98 },
    { 0xc46538b1, 81 },
    { 0xa60538b2, 76 },
    { 0x592538b4, 34 },
    { 0x36e538b5, 95 },
    { 0, 0 },
    { 0x347538b7, 95 },
    { 0x359538b8, 61 },
    { 0xe1f538b7, 69 },
    { 0, 0 },
    { 0xb0f538bb, 52 },
    { 0, 0 },
    { 0x827538bd, 68 },
    { 0xa15538be, 98 },
    { 0x48e538bd, 33 },
    { 0x4ea538c0, 100 },
    { 0xb78538c1, 96 },
    { 0x6aa538c2, 85 },
    { 0xf57538bd, 96 },
    { 0, 0 },
    { 0x496538c5, 61 },
    { 0x6a6538c6, 78 },
    { 0xf82538c7, 91 },
    { 0x4ed538c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d538ce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93538d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7538db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b538df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1538e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249538eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7538f8, 61 },
    { 0, 0 },
    { 0x596538fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c538fd, 39 },
    { 0x138538fd, 91 },
    { 0xed4538ff, 61 },
    { 0xdf053900, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2653903, 66 },
    { 0, 0 },
    { 0x65753905, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf653908, 53 },
    { 0, 0 },
    { 0x4905390a, 17 },
    { 0x6895390b, 100 },
    { 0xefd5390c, 90 },
    { 0x6b55390d, 33 },
    { 0x6185390d, 95 },
    { 0, 0 },
    { 0x62653910, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29353914, 66 },
    { 0, 0 },
    { 0xfa653916, 78 },
    { 0x5a853916, 33 },
    { 0xc1553917, 98 },
    { 0x72b53919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f05391d, 43 },
    { 0, 0 },
    { 0xcb25391f, 64 },
    { 0xf8e53920, 33 },
    { 0x8d453920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea53924, 90 },
    { 0xc453925, 34 },
    { 0x90453926, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb165392f, 99 },
    { 0, 0 },
    { 0x1cc53931, 43 },
    { 0, 0 },
    { 0xb4d53933, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365393b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2285393e, 33 },
    { 0x5db5393f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5053942, 33 },
    { 0, 0 },
    { 0x5b553944, 33 },
    { 0x43053945, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35394d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d53958, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd5395c, 87 },
    { 0xf305395d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91453964, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3305396c, 91 },
    { 0xacf5396d, 43 },
    { 0xc865396e, 93 },
    { 0xc25396f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5753978, 90 },
    { 0, 0 },
    { 0x9ff5397a, 61 },
    { 0x5f5397b, 34 },
    { 0x4ff5397c, 90 },
    { 0x1845397a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd53981, 17 },
    { 0, 0 },
    { 0x9d153983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e53988, 77 },
    { 0, 0 },
    { 0x61a5398a, 33 },
    { 0x2605398a, 52 },
    { 0xa915398a, 90 },
    { 0x9385398a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c53991, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af53996, 91 },
    { 0x99553997, 81 },
    { 0, 0 },
    { 0x57953999, 22 },
    { 0x29753999, 90 },
    { 0xa435399b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e65399f, 33 },
    { 0, 0 },
    { 0x68f539a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d539a7, 93 },
    { 0xe82539a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745539ad, 93 },
    { 0, 0 },
    { 0xc72539af, 98 },
    { 0x8c7539b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c539b3, 90 },
    { 0x7a4539b4, 98 },
    { 0xa1c539b4, 88 },
    { 0x397539b4, 85 },
    { 0x8ca539b5, 43 },
    { 0xe54539b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c539bf, 66 },
    { 0, 0 },
    { 0x1eb539c1, 98 },
    { 0x69e539c1, 98 },
    { 0x637539c2, 61 },
    { 0, 0 },
    { 0x771539c5, 61 },
    { 0xc44539c6, 23 },
    { 0x171539c7, 34 },
    { 0xc94539c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97539cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e539d2, 33 },
    { 0x30e539d2, 55 },
    { 0x1b3539d4, 90 },
    { 0x9ca539d5, 83 },
    { 0x935539d6, 55 },
    { 0x735539d2, 87 },
    { 0xe9b539d8, 61 },
    { 0x214539d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c539e0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a539e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19539ea, 99 },
    { 0, 0 },
    { 0x2b0539ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed539f1, 88 },
    { 0xed1539f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0539f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05539fb, 99 },
    { 0, 0 },
    { 0xfcc539fd, 61 },
    { 0xba539fd, 23 },
    { 0x1e7539fd, 93 },
    { 0x10a53a00, 52 },
    { 0x24753a00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c53a04, 84 },
    { 0xa1e53a05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c353a08, 88 },
    { 0x1f053a08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b53a0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86753a10, 88 },
    { 0, 0 },
    { 0x3c653a12, 43 },
    { 0x9ff53a12, 66 },
    { 0xbd153a12, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf253a1c, 90 },
    { 0, 0 },
    { 0x9953a1e, 78 },
    { 0xe0153a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2053a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb53a27, 88 },
    { 0xd5f53a27, 34 },
    { 0, 0 },
    { 0xe8553a2a, 87 },
    { 0, 0 },
    { 0x9c453a2c, 33 },
    { 0xdbd53a2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17653a31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e653a35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97753a38, 90 },
    { 0xc7453a39, 98 },
    { 0x34d53a38, 23 },
    { 0, 0 },
    { 0xaa353a3c, 91 },
    { 0xec253a3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7753a40, 80 },
    { 0xa8c53a41, 100 },
    { 0, 0 },
    { 0xb0a53a43, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x19953a46, 90 },
    { 0x2e553a47, 95 },
    { 0, 0 },
    { 0xc5f53a49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72853a4c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b53a53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2653a56, 61 },
    { 0x14c53a57, 85 },
    { 0x6b153a58, 95 },
    { 0xe6253a57, 99 },
    { 0, 0 },
    { 0x65c53a5b, 98 },
    { 0x3c953a5b, 90 },
    { 0xb7053a5c, 17 },
    { 0xc5153a5e, 90 },
    { 0xa6653a5c, 91 },
    { 0x33353a60, 61 },
    { 0x66e53a61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40553a68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb53a6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f53a70, 92 },
    { 0, 0 },
    { 0x95e53a72, 61 },
    { 0x47553a73, 98 },
    { 0xd0953a74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe553a79, 66 },
    { 0xfb353a7a, 90 },
    { 0xde353a7b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f53a7f, 76 },
    { 0x92153a80, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe53a85, 87 },
    { 0x16b53a85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e53a89, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf353a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8253a9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x27253a9d, 90 },
    { 0xa7b53a9e, 33 },
    { 0x84d53a9f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4453aa6, 90 },
    { 0, 0 },
    { 0xfd753aa8, 55 },
    { 0xea253aa9, 84 },
    { 0xbf553aaa, 98 },
    { 0xf5f53aab, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d353aae, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc553ab1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a53abb, 61 },
    { 0xc9e53abb, 43 },
    { 0xe8053abd, 77 },
    { 0x40f53abb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53853ac8, 95 },
    { 0xfbb53ac9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e53acf, 43 },
    { 0xd2253ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f753ad4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54353ad9, 95 },
    { 0xd8753ada, 23 },
    { 0, 0 },
    { 0xbd853adc, 33 },
    { 0x8c253add, 52 },
    { 0, 0 },
    { 0xdf53adf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d53ae3, 84 },
    { 0, 0 },
    { 0x27353ae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8153aec, 67 },
    { 0x2f453aec, 91 },
    { 0x34253aee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc53af1, 24 },
    { 0x4fa53af1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd53af6, 66 },
    { 0xbf253af7, 61 },
    { 0x38b53af8, 90 },
    { 0x7d53af6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb553afe, 55 },
    { 0x98c53aff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32853b08, 38 },
    { 0x88053b09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47853b0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd53b0f, 98 },
    { 0, 0 },
    { 0xbe453b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7053b14, 61 },
    { 0x60c53b15, 78 },
    { 0x4cf53b16, 66 },
    { 0x56d53b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef553b1f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39253b23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e253b29, 95 },
    { 0xaa853b2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da53b2e, 61 },
    { 0xe6253b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82353b32, 90 },
    { 0xbd753b33, 43 },
    { 0xb7553b34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c253b37, 55 },
    { 0xfce53b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b53b3f, 84 },
    { 0, 0 },
    { 0x9c753b41, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd53b4a, 61 },
    { 0xd5553b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3253b4e, 100 },
    { 0, 0 },
    { 0xe0153b50, 90 },
    { 0xd6453b50, 61 },
    { 0, 0 },
    { 0xda353b53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b353b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41453b5d, 95 },
    { 0xd5053b5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51453b61, 46 },
    { 0, 0 },
    { 0x47553b63, 41 },
    { 0xf8953b64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15653b6b, 100 },
    { 0x58753b6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e53b70, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92853b75, 91 },
    { 0x5bb53b75, 55 },
    { 0x88353b76, 61 },
    { 0, 0 },
    { 0x46853b79, 95 },
    { 0, 0 },
    { 0x4de53b7b, 43 },
    { 0, 0 },
    { 0x2d853b7d, 61 },
    { 0xa1d53b7d, 90 },
    { 0x7c753b7e, 93 },
    { 0xa253b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58953b84, 33 },
    { 0, 0 },
    { 0xd5953b86, 99 },
    { 0xaed53b86, 98 },
    { 0x63653b88, 61 },
    { 0, 0 },
    { 0x78353b8a, 90 },
    { 0x53353b8b, 66 },
    { 0, 0 },
    { 0xb2c53b8d, 95 },
    { 0xbb453b8d, 100 },
    { 0x11c53b8f, 88 },
    { 0, 0 },
    { 0xe1253b91, 95 },
    { 0x82b53b92, 98 },
    { 0xd6f53b92, 98 },
    { 0xfb453b93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9453b99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a53b9c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db53ba1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x44953ba4, 93 },
    { 0x48d53ba5, 33 },
    { 0, 0 },
    { 0x33e53ba7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b653bab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f53bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e753bb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a53bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf53bc3, 67 },
    { 0, 0 },
    { 0x41b53bc5, 90 },
    { 0xd0253bc6, 76 },
    { 0x23253bc7, 61 },
    { 0x88853bc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a553bcd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a53bd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36553bd8, 33 },
    { 0, 0 },
    { 0x92553bda, 98 },
    { 0x6f953bda, 52 },
    { 0xc2153bdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e53be0, 52 },
    { 0, 0 },
    { 0x9a653be2, 23 },
    { 0xc8453be2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57153be7, 93 },
    { 0x61f53be7, 78 },
    { 0x69e53be9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc53bed, 98 },
    { 0x54453bed, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb853bf3, 55 },
    { 0x87853bf3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22453bfc, 76 },
    { 0xd2453bfd, 61 },
    { 0, 0 },
    { 0x18c53bff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3753c03, 98 },
    { 0xf8c53c04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4453c07, 61 },
    { 0x83553c08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb653c0b, 98 },
    { 0x28053c0b, 95 },
    { 0xf8453c0c, 98 },
    { 0xf1f53c0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2953c21, 61 },
    { 0x7ca53c21, 55 },
    { 0x64953c23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e53c26, 33 },
    { 0x97853c26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30053c2a, 98 },
    { 0xd7353c2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c453c30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68453c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90853c3b, 34 },
    { 0xd1753c3c, 61 },
    { 0xd553c3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c53c43, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae53c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc53c4c, 77 },
    { 0x45c53c4d, 33 },
    { 0xf7f53c4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1053c53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f53c58, 90 },
    { 0x94d53c58, 90 },
    { 0x73353c5a, 65 },
    { 0, 0 },
    { 0x90753c5c, 100 },
    { 0, 0 },
    { 0x28753c5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe053c64, 88 },
    { 0, 0 },
    { 0x59553c66, 98 },
    { 0x65d53c66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd53c6b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8653c76, 61 },
    { 0xaab53c76, 76 },
    { 0x9a53c76, 90 },
    { 0xfbb53c78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6153c80, 33 },
    { 0xdc953c80, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb453c86, 78 },
    { 0xe6353c87, 61 },
    { 0xa5a53c86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1453c8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d953c90, 68 },
    { 0x6d253c90, 61 },
    { 0x49553c90, 91 },
    { 0x253c93, 35 },
    { 0xda853c94, 88 },
    { 0xd5653c92, 66 },
    { 0, 0 },
    { 0xdb53c97, 61 },
    { 0, 0 },
    { 0xf7753c99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd853c9d, 84 },
    { 0x76453c9e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x93853ca1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4553ca6, 98 },
    { 0xd9853ca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6753cab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57853caf, 43 },
    { 0x95753cb0, 61 },
    { 0, 0 },
    { 0x40253cb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c53cb7, 76 },
    { 0x9e553cb7, 61 },
    { 0x75753cb9, 76 },
    { 0x46853cba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd53cbd, 98 },
    { 0, 0 },
    { 0x9b53cbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a53cc3, 97 },
    { 0x44453cc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14853cc7, 78 },
    { 0x9e053cc8, 61 },
    { 0x35653cc9, 23 },
    { 0xe5c53cc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86353ccd, 33 },
    { 0, 0 },
    { 0xe1953ccf, 66 },
    { 0x79e53cd0, 84 },
    { 0, 0 },
    { 0xf7253cd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3753cd7, 23 },
    { 0, 0 },
    { 0x5a453cd9, 33 },
    { 0x75c53cda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef53ce0, 34 },
    { 0, 0 },
    { 0x7d653ce2, 99 },
    { 0x8de53ce3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca53ce9, 78 },
    { 0x4c253ce9, 90 },
    { 0x2fc53ceb, 90 },
    { 0x4c753cec, 76 },
    { 0xb3753ced, 34 },
    { 0, 0 },
    { 0xf9753cef, 34 },
    { 0, 0 },
    { 0x8a753cf1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86953cf7, 55 },
    { 0x8a653cf8, 93 },
    { 0xca53cf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4353d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f53d0d, 83 },
    { 0x9ab53d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a253d15, 95 },
    { 0xefa53d16, 90 },
    { 0x1e553d17, 43 },
    { 0x77353d18, 23 },
    { 0xfba53d19, 61 },
    { 0, 0 },
    { 0xf4b53d1b, 23 },
    { 0x8ee53d1b, 61 },
    { 0, 0 },
    { 0x5e853d1e, 33 },
    { 0, 0 },
    { 0x90b53d20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc653d26, 33 },
    { 0xea553d26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71453d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b253d32, 98 },
    { 0xc9953d32, 85 },
    { 0xde153d32, 76 },
    { 0, 0 },
    { 0x36d53d36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef53d3a, 23 },
    { 0, 0 },
    { 0xb8a53d3c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b53d3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52853d44, 61 },
    { 0x7dd53d45, 66 },
    { 0xb153d45, 61 },
    { 0, 0 },
    { 0x53b53d48, 23 },
    { 0x62353d48, 23 },
    { 0xa8b53d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17553d4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16053d55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab53d58, 80 },
    { 0xc1353d59, 49 },
    { 0, 0 },
    { 0xbfe53d5b, 67 },
    { 0xbfa53d5c, 76 },
    { 0xac453d5b, 75 },
    { 0, 0 },
    { 0xcdc53d5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb53d62, 99 },
    { 0x32553d63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66653d68, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b653d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97653d70, 23 },
    { 0xffb53d70, 33 },
    { 0x40a53d71, 34 },
    { 0, 0 },
    { 0x37553d74, 99 },
    { 0, 0 },
    { 0x5a553d76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0453d79, 46 },
    { 0x50a53d7a, 85 },
    { 0x9d253d7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49753d82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d53d89, 76 },
    { 0, 0 },
    { 0x54e53d8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7953d90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c53d93, 90 },
    { 0x49653d94, 76 },
    { 0, 0 },
    { 0x48753d96, 65 },
    { 0x36953d96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1653d9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca853d9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b53da4, 72 },
    { 0x7f253da5, 98 },
    { 0xb6053da6, 95 },
    { 0xdcf53da6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f53dab, 33 },
    { 0xa4f53dac, 90 },
    { 0xfeb53dac, 33 },
    { 0x90353dac, 33 },
    { 0xf0053dad, 78 },
    { 0xb4253daf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57153db4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17453dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98353dbf, 33 },
    { 0x19553dbf, 23 },
    { 0xca453dc1, 90 },
    { 0xd7e53dc2, 95 },
    { 0x20453dc1, 84 },
    { 0x80253dc4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e53dc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b653dcb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99953dcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c53dd2, 98 },
    { 0, 0 },
    { 0xcb953dd4, 23 },
    { 0x20653dd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50353ddb, 76 },
    { 0xa9953ddb, 99 },
    { 0x11353ddd, 100 },
    { 0x87753dde, 55 },
    { 0x5be53ddf, 61 },
    { 0, 0 },
    { 0x4e53de1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d53de4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c153de9, 67 },
    { 0x3c253dea, 55 },
    { 0, 0 },
    { 0xaca53dec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f653df7, 84 },
    { 0xd7453df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b53dfb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2553e00, 33 },
    { 0x5f53e01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x69253e04, 90 },
    { 0xfbd53e05, 95 },
    { 0x27d53e06, 61 },
    { 0x10d53e06, 33 },
    { 0x25e53e06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c053e0f, 100 },
    { 0xdf153e10, 90 },
    { 0xcb053e11, 33 },
    { 0x31c53e0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e53e18, 55 },
    { 0x4a053e18, 55 },
    { 0x6a153e1a, 33 },
    { 0x8af53e1a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d353e1f, 90 },
    { 0xd5c53e20, 55 },
    { 0x41253e21, 17 },
    { 0xede53e22, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58053e26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4053e2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab53e30, 52 },
    { 0x30353e31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf53e36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba53e3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d653e40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc253e47, 33 },
    { 0x51453e48, 75 },
    { 0x25553e48, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71853e4e, 78 },
    { 0xffd53e4f, 95 },
    { 0, 0 },
    { 0xd0e53e51, 95 },
    { 0x8a453e52, 78 },
    { 0x1953e51, 23 },
    { 0xd9a53e52, 61 },
    { 0x38453e51, 23 },
    { 0x85e53e56, 17 },
    { 0xe9e53e55, 98 },
    { 0, 0 },
    { 0xf3053e59, 61 },
    { 0, 0 },
    { 0xd9553e5b, 34 },
    { 0xd5053e5c, 61 },
    { 0x24853e5d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9953e63, 61 },
    { 0, 0 },
    { 0xc953e65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc53e6a, 77 },
    { 0xf2453e6b, 95 },
    { 0x23b53e6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93853e6f, 95 },
    { 0, 0 },
    { 0xd4e53e71, 95 },
    { 0, 0 },
    { 0x95853e73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5853e76, 98 },
    { 0x5ef53e77, 23 },
    { 0xd1653e77, 98 },
    { 0, 0 },
    { 0x3f153e7a, 46 },
    { 0x45853e7b, 23 },
    { 0, 0 },
    { 0xf9853e7d, 99 },
    { 0x29753e7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b53e81, 55 },
    { 0, 0 },
    { 0x62a53e83, 95 },
    { 0xcbd53e84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b453e8c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b053e8f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66553e92, 75 },
    { 0xa853e93, 61 },
    { 0x7e153e93, 90 },
    { 0x3a753e95, 93 },
    { 0xa753e92, 99 },
    { 0, 0 },
    { 0x39a53e98, 55 },
    { 0, 0 },
    { 0x44453e9a, 23 },
    { 0xbaa53e9b, 99 },
    { 0x3e453e9c, 68 },
    { 0xbf153e9b, 98 },
    { 0x45553e9e, 61 },
    { 0x87b53e9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1053ea3, 46 },
    { 0x34f53ea3, 99 },
    { 0xe4953ea5, 100 },
    { 0x3bf53ea6, 34 },
    { 0xcfc53ea7, 23 },
    { 0x17553ea6, 93 },
    { 0xe4953ea7, 84 },
    { 0xd7853eaa, 98 },
    { 0x1cc53eaa, 91 },
    { 0xe3b53eac, 95 },
    { 0x83c53eab, 55 },
    { 0, 0 },
    { 0xa6153eaf, 23 },
    { 0, 0 },
    { 0x8d753eb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48153eb7, 95 },
    { 0x64c53eb8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14153ebc, 23 },
    { 0xe5d53ebc, 90 },
    { 0x69553ebd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a53ec4, 87 },
    { 0, 0 },
    { 0x26e53ec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f553ec9, 23 },
    { 0x63453eca, 61 },
    { 0, 0 },
    { 0x53953ecc, 55 },
    { 0xeb53ecc, 33 },
    { 0x99853ece, 87 },
    { 0xfeb53ecf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af53ed7, 43 },
    { 0xd8f53ed7, 90 },
    { 0x63c53ed8, 61 },
    { 0, 0 },
    { 0x33353edb, 61 },
    { 0, 0 },
    { 0x48253edd, 61 },
    { 0x1bd53ede, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16653ee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b253eec, 77 },
    { 0x99353eec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2453ef0, 98 },
    { 0xf1053ef0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a53ef8, 61 },
    { 0, 0 },
    { 0x5f053efa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f453efd, 24 },
    { 0, 0 },
    { 0x97e53eff, 23 },
    { 0xc0a53eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e453f03, 33 },
    { 0x4a153f03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xade53f07, 23 },
    { 0xde353f08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4053f0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1253f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c653f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa553f1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4553f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53653f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb153f2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e53f2d, 33 },
    { 0, 0 },
    { 0x6d953f2f, 61 },
    { 0x2153f30, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd053f34, 91 },
    { 0x6cd53f35, 98 },
    { 0x14953f35, 99 },
    { 0, 0 },
    { 0xc253f38, 61 },
    { 0x9d553f39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d53f3c, 90 },
    { 0xa2c53f3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87153f41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83853f44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x55653f47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a53f51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc53f58, 90 },
    { 0, 0 },
    { 0x2f653f5a, 56 },
    { 0x3c853f5b, 61 },
    { 0xada53f5b, 33 },
    { 0, 0 },
    { 0x7c653f5e, 90 },
    { 0x1e653f5f, 90 },
    { 0xb7353f60, 33 },
    { 0x43a53f5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3053f65, 34 },
    { 0x19a53f66, 91 },
    { 0xe3653f67, 55 },
    { 0x16053f68, 34 },
    { 0, 0 },
    { 0x7853f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d753f6e, 92 },
    { 0xf6253f6e, 59 },
    { 0xfc553f6f, 90 },
    { 0x5c553f71, 61 },
    { 0x74953f71, 78 },
    { 0xccf53f72, 61 },
    { 0x7ca53f74, 43 },
    { 0x4353f74, 91 },
    { 0x27953f75, 95 },
    { 0xae953f74, 33 },
    { 0x61e53f70, 90 },
    { 0x2d053f6e, 95 },
    { 0x78753f7a, 61 },
    { 0x90453f7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d953f84, 93 },
    { 0, 0 },
    { 0xe6253f86, 64 },
    { 0x6aa53f87, 95 },
    { 0x9f753f86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb53f8c, 46 },
    { 0x51b53f8c, 61 },
    { 0x74f53f8e, 76 },
    { 0, 0 },
    { 0xde953f90, 33 },
    { 0, 0 },
    { 0x94853f92, 23 },
    { 0x62653f92, 95 },
    { 0xe8b53f94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb53f97, 43 },
    { 0x78c53f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de53f9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d53f9e, 100 },
    { 0x2fc53f9e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x43353fa2, 61 },
    { 0xcab53fa3, 90 },
    { 0, 0 },
    { 0x88e53fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a53fa8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8553fae, 76 },
    { 0xbc453fae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a53fb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7653fc5, 8 },
    { 0, 0 },
    { 0xcc553fc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17953fcf, 61 },
    { 0xaa853fcf, 38 },
    { 0xc3053fd1, 90 },
    { 0x2d453fcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22253fd6, 61 },
    { 0x9ef53fd7, 91 },
    { 0x94b53fd8, 61 },
    { 0xa5853fd9, 33 },
    { 0x69a53fda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e53fdd, 95 },
    { 0x92453fde, 52 },
    { 0, 0 },
    { 0x6a153fe0, 23 },
    { 0, 0 },
    { 0xa6953fe2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1253feb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e653ff1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b53ff6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6553fff, 61 },
    { 0x1553fff, 98 },
    { 0x1d354001, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c25400f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f85401d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a5402b, 88 },
    { 0x3ae5402c, 84 },
    { 0x7055402d, 86 },
    { 0xea45402e, 61 },
    { 0xc0f5402b, 84 },
    { 0x7895402c, 52 },
    { 0x97054030, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e54036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5403b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b554040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3554047, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef95404c, 85 },
    { 0x3735404d, 55 },
    { 0x91c5404e, 40 },
    { 0x39c5404e, 61 },
    { 0, 0 },
    { 0x3b954051, 77 },
    { 0x62354051, 33 },
    { 0xb4654053, 46 },
    { 0x6dd54054, 85 },
    { 0x26654051, 43 },
    { 0, 0 },
    { 0x4f854057, 68 },
    { 0x36f54057, 33 },
    { 0x3e154059, 46 },
    { 0, 0 },
    { 0xe8a5405b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f754065, 61 },
    { 0x76554065, 91 },
    { 0, 0 },
    { 0x67054068, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6895406c, 90 },
    { 0xdc65406d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1354071, 75 },
    { 0xa2754071, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb154078, 97 },
    { 0, 0 },
    { 0x25a5407a, 98 },
    { 0xac55407b, 66 },
    { 0x2505407c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3775407f, 83 },
    { 0x3a54080, 52 },
    { 0x99b54081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66254086, 90 },
    { 0, 0 },
    { 0x8554088, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe905408c, 90 },
    { 0, 0 },
    { 0x2075408e, 46 },
    { 0, 0 },
    { 0x6eb54090, 93 },
    { 0xed054091, 61 },
    { 0xb3654092, 76 },
    { 0, 0 },
    { 0x8b54094, 34 },
    { 0x5e954095, 90 },
    { 0, 0 },
    { 0xf4154097, 46 },
    { 0xdf654098, 90 },
    { 0, 0 },
    { 0xaab5409a, 34 },
    { 0xebf5409b, 83 },
    { 0x2705409c, 55 },
    { 0x4b95409c, 91 },
    { 0xa125409e, 90 },
    { 0x9b25409e, 33 },
    { 0x1285409e, 53 },
    { 0x9635409e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x852540a4, 99 },
    { 0x3a5540a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d540ae, 75 },
    { 0xf7d540af, 90 },
    { 0x95f540b0, 98 },
    { 0x7f8540b1, 98 },
    { 0x5f9540b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b540b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce540bc, 90 },
    { 0xb74540bc, 84 },
    { 0x72b540bd, 91 },
    { 0xd52540bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619540c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a540ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x417540cd, 90 },
    { 0xd37540ce, 99 },
    { 0, 0 },
    { 0xaa540d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1540d5, 23 },
    { 0xf7540d6, 76 },
    { 0xfda540d7, 90 },
    { 0xcd9540d8, 61 },
    { 0, 0 },
    { 0x841540da, 55 },
    { 0x302540da, 98 },
    { 0xcf0540db, 56 },
    { 0x27b540dd, 90 },
    { 0xb27540de, 75 },
    { 0x1d4540df, 33 },
    { 0xa90540df, 33 },
    { 0, 0 },
    { 0xea3540e2, 90 },
    { 0x239540e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746540e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c540f3, 23 },
    { 0x59c540f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b540fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b654105, 33 },
    { 0xd8e54105, 93 },
    { 0xb8754107, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc645410e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34654111, 66 },
    { 0x5a354111, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb54116, 95 },
    { 0x36454117, 78 },
    { 0x61b54116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25411d, 76 },
    { 0xa925411e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b54128, 33 },
    { 0, 0 },
    { 0x11e5412a, 61 },
    { 0x9cd5412b, 91 },
    { 0x6015412c, 87 },
    { 0xd7f5412d, 84 },
    { 0x8685412e, 90 },
    { 0, 0 },
    { 0xd2454130, 90 },
    { 0xc8254131, 43 },
    { 0x72354132, 77 },
    { 0x47854133, 33 },
    { 0x72c54134, 95 },
    { 0xa8754135, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb54138, 90 },
    { 0, 0 },
    { 0xab5413a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63054141, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef854145, 55 },
    { 0x5e354146, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a5414a, 52 },
    { 0, 0 },
    { 0xbc15414c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce85414f, 23 },
    { 0, 0 },
    { 0x60754151, 23 },
    { 0xe6854152, 34 },
    { 0xe8054153, 90 },
    { 0x94a54154, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73354158, 61 },
    { 0x1dd54159, 87 },
    { 0x65c5415a, 33 },
    { 0x585415a, 67 },
    { 0, 0 },
    { 0x2385415d, 84 },
    { 0, 0 },
    { 0x9ff5415f, 61 },
    { 0xbfc54160, 61 },
    { 0xd7c5415f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6354164, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a54167, 55 },
    { 0x3fb54167, 55 },
    { 0xe7154169, 75 },
    { 0x25a5416a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4854175, 95 },
    { 0x6154176, 23 },
    { 0xa8c54177, 34 },
    { 0xacb54178, 33 },
    { 0xdd854179, 46 },
    { 0x8a054176, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3395417e, 87 },
    { 0xfcd5417e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d854184, 90 },
    { 0, 0 },
    { 0xd8d54186, 95 },
    { 0xa9154187, 98 },
    { 0xc1154188, 61 },
    { 0, 0 },
    { 0x2ec5418a, 93 },
    { 0xd1a5418b, 43 },
    { 0x6645418b, 34 },
    { 0x9425418d, 61 },
    { 0x2775418b, 33 },
    { 0x1645418f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b254192, 33 },
    { 0xe2054193, 55 },
    { 0, 0 },
    { 0xa8a54195, 95 },
    { 0x5e354196, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2875419b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d5419f, 94 },
    { 0, 0 },
    { 0x3c5541a1, 33 },
    { 0x733541a1, 67 },
    { 0xd84541a2, 33 },
    { 0x2a541a1, 90 },
    { 0xac0541a5, 61 },
    { 0x8db541a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0541ab, 23 },
    { 0, 0 },
    { 0xb6541ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2541b1, 61 },
    { 0x56a541b1, 23 },
    { 0x953541b3, 91 },
    { 0xab7541b3, 23 },
    { 0x298541b4, 95 },
    { 0xe2a541b6, 66 },
    { 0x4dd541b6, 64 },
    { 0xd34541b8, 83 },
    { 0, 0 },
    { 0x343541ba, 46 },
    { 0xbfb541bb, 78 },
    { 0x9ec541ba, 33 },
    { 0x198541bd, 76 },
    { 0, 0 },
    { 0x380541bf, 34 },
    { 0x721541bf, 55 },
    { 0x9e7541c1, 33 },
    { 0x8bd541c2, 93 },
    { 0xcf2541c3, 90 },
    { 0xee7541c3, 90 },
    { 0x17f541c0, 17 },
    { 0xc2c541c5, 34 },
    { 0x8dc541c4, 76 },
    { 0x23b541c8, 40 },
    { 0x261541c9, 95 },
    { 0x37e541ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e541cd, 90 },
    { 0xb82541cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334541d8, 83 },
    { 0, 0 },
    { 0x109541da, 61 },
    { 0, 0 },
    { 0xb1b541dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x725541df, 33 },
    { 0, 0 },
    { 0x881541e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf541e9, 17 },
    { 0, 0 },
    { 0x937541eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239541f1, 98 },
    { 0x17a541f1, 33 },
    { 0, 0 },
    { 0x62c541f4, 77 },
    { 0x76f541f5, 90 },
    { 0xe5f541f5, 76 },
    { 0x9da541f7, 87 },
    { 0xef6541f4, 75 },
    { 0x1a1541f9, 90 },
    { 0, 0 },
    { 0xed6541fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a541ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a54202, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe685420c, 85 },
    { 0xe785420d, 98 },
    { 0, 0 },
    { 0x83c5420f, 61 },
    { 0x6765420f, 98 },
    { 0xc225420f, 98 },
    { 0x8fb54212, 33 },
    { 0xd0754213, 33 },
    { 0x3a054213, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7554217, 91 },
    { 0, 0 },
    { 0x7d654219, 76 },
    { 0xa9254219, 23 },
    { 0x79754219, 61 },
    { 0xe845421c, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde454220, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37954229, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c5422c, 95 },
    { 0x1285422c, 90 },
    { 0xb6d5422e, 76 },
    { 0xe205422d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d654232, 95 },
    { 0, 0 },
    { 0x5bf54234, 52 },
    { 0, 0 },
    { 0x73054236, 76 },
    { 0, 0 },
    { 0xc354238, 55 },
    { 0, 0 },
    { 0xce35423a, 100 },
    { 0x5065423b, 56 },
    { 0x8d25423b, 91 },
    { 0, 0 },
    { 0xa6c5423e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3154241, 90 },
    { 0x14254241, 90 },
    { 0, 0 },
    { 0xf3254244, 76 },
    { 0, 0 },
    { 0x5054246, 84 },
    { 0xb8954247, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4585424b, 66 },
    { 0xc4d5424b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b54253, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24154256, 88 },
    { 0, 0 },
    { 0xf6954258, 46 },
    { 0, 0 },
    { 0xd015425a, 98 },
    { 0, 0 },
    { 0xf265425c, 36 },
    { 0x37c5425c, 90 },
    { 0x98e5425c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c054265, 98 },
    { 0x8a54266, 61 },
    { 0xb8554267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1935426b, 46 },
    { 0x8a5426b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76854273, 69 },
    { 0x3a954274, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f454277, 76 },
    { 0xa854277, 98 },
    { 0x9f154279, 40 },
    { 0, 0 },
    { 0xfdd5427b, 23 },
    { 0, 0 },
    { 0xf875427d, 66 },
    { 0xd3b5427e, 61 },
    { 0, 0 },
    { 0x15e54280, 43 },
    { 0x30354280, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a754285, 87 },
    { 0xa9f54286, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d54289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab754290, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7854293, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c54297, 55 },
    { 0, 0 },
    { 0x89454299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3605429e, 98 },
    { 0, 0 },
    { 0x39b542a0, 98 },
    { 0xff4542a1, 46 },
    { 0, 0 },
    { 0xf0c542a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684542a7, 61 },
    { 0, 0 },
    { 0xfc2542a9, 87 },
    { 0x670542a9, 95 },
    { 0x8a9542ab, 95 },
    { 0, 0 },
    { 0x61542ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9542b5, 95 },
    { 0xcbd542b6, 61 },
    { 0xc77542b7, 34 },
    { 0xd86542b8, 88 },
    { 0xdc5542b9, 66 },
    { 0xce2542ba, 76 },
    { 0xa96542bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0542c5, 100 },
    { 0, 0 },
    { 0xf67542c7, 90 },
    { 0, 0 },
    { 0xd19542c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x235542cc, 33 },
    { 0xe62542cc, 61 },
    { 0x8fd542ce, 61 },
    { 0x86542cf, 77 },
    { 0xcdb542cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17542d4, 90 },
    { 0xfb2542d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920542db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x494542de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a542e6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76542ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c542f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca542f9, 66 },
    { 0, 0 },
    { 0x618542fb, 90 },
    { 0xeda542fb, 61 },
    { 0, 0 },
    { 0xf0f542fe, 81 },
    { 0xe5b542ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6654306, 76 },
    { 0x64a54306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe345430a, 33 },
    { 0x4485430a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea54310, 61 },
    { 0, 0 },
    { 0x48454312, 90 },
    { 0xb1854313, 43 },
    { 0, 0 },
    { 0x93854315, 23 },
    { 0x22f54316, 87 },
    { 0x2b54317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b5431d, 33 },
    { 0, 0 },
    { 0x9965431f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1754329, 66 },
    { 0, 0 },
    { 0xcf05432b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9535432e, 33 },
    { 0xcb25432f, 81 },
    { 0, 0 },
    { 0x7454331, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd5433a, 93 },
    { 0x23e5433b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96454341, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b54344, 76 },
    { 0, 0 },
    { 0x20254346, 95 },
    { 0, 0 },
    { 0x39954348, 61 },
    { 0xb4d54349, 98 },
    { 0xf035434a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca65434f, 77 },
    { 0x28b5434f, 33 },
    { 0, 0 },
    { 0xef454352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f654355, 78 },
    { 0x5c854355, 23 },
    { 0x57254355, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c54363, 90 },
    { 0, 0 },
    { 0x55e54365, 46 },
    { 0x3c154366, 55 },
    { 0xf0454367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae95436c, 52 },
    { 0x4985436d, 80 },
    { 0xcb25436e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb25437b, 85 },
    { 0x3db5437c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda54386, 55 },
    { 0x9b354387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb5438a, 23 },
    { 0, 0 },
    { 0x85c5438c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90454391, 33 },
    { 0x4b754391, 95 },
    { 0x6ae54393, 46 },
    { 0x9b854394, 95 },
    { 0xb3d54395, 76 },
    { 0x18354396, 88 },
    { 0xbd554397, 98 },
    { 0x58a54396, 34 },
    { 0x4154399, 98 },
    { 0, 0 },
    { 0x5ae5439b, 84 },
    { 0xe5d5439c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c543a3, 23 },
    { 0x376543a4, 91 },
    { 0x157543a5, 43 },
    { 0x12c543a5, 40 },
    { 0xb9c543a7, 95 },
    { 0xcd1543a7, 90 },
    { 0xe88543a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292543ad, 91 },
    { 0x873543ae, 43 },
    { 0x803543af, 61 },
    { 0x1de543ae, 33 },
    { 0x696543ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec543b7, 100 },
    { 0x8d7543b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde543bb, 90 },
    { 0x2d3543bc, 33 },
    { 0x7ee543bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7543c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba543c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447543cb, 49 },
    { 0x9e5543cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1543cf, 61 },
    { 0, 0 },
    { 0x604543d1, 23 },
    { 0x5dd543d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91543d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a543dd, 76 },
    { 0, 0 },
    { 0x450543df, 90 },
    { 0, 0 },
    { 0x4da543e1, 90 },
    { 0xde8543e2, 76 },
    { 0xc19543e2, 93 },
    { 0x3e6543e1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de543e7, 98 },
    { 0, 0 },
    { 0x398543e9, 47 },
    { 0x6ad543e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89543ef, 76 },
    { 0, 0 },
    { 0x2d7543f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3543f4, 34 },
    { 0xc85543f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89543fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x735543fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c54403, 23 },
    { 0, 0 },
    { 0xd8454405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a54409, 61 },
    { 0xea354409, 33 },
    { 0x3715440b, 98 },
    { 0x5545440c, 95 },
    { 0x3825440c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5654410, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d5441e, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d454421, 23 },
    { 0xc354422, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c754425, 75 },
    { 0xe9154426, 33 },
    { 0x3f054425, 38 },
    { 0x6fb54426, 98 },
    { 0x85254426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2115442f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70854435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46254438, 90 },
    { 0, 0 },
    { 0x8545443a, 64 },
    { 0, 0 },
    { 0x56b5443c, 4 },
    { 0xc665443d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87854440, 87 },
    { 0xee854441, 52 },
    { 0xb9e54441, 33 },
    { 0x33a54441, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5254446, 95 },
    { 0x5a654447, 75 },
    { 0x3c454448, 90 },
    { 0xa3454448, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4305444c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa54450, 33 },
    { 0x51754450, 90 },
    { 0x4a754450, 12 },
    { 0, 0 },
    { 0x8dd54454, 53 },
    { 0, 0 },
    { 0x92b54456, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a54459, 48 },
    { 0x2325445a, 38 },
    { 0x7a154459, 90 },
    { 0xe195445c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e54462, 61 },
    { 0, 0 },
    { 0xc3c54464, 90 },
    { 0x5a654464, 61 },
    { 0, 0 },
    { 0xb54467, 23 },
    { 0xda354468, 43 },
    { 0xb2954467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5446e, 12 },
    { 0xe485446f, 76 },
    { 0, 0 },
    { 0x87c54471, 23 },
    { 0xd4254472, 76 },
    { 0xf8754473, 69 },
    { 0x61554474, 95 },
    { 0x60354471, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4655447a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b5447f, 23 },
    { 0xc1d5447f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c54486, 38 },
    { 0, 0 },
    { 0x47154488, 33 },
    { 0xdf154489, 52 },
    { 0xee45448a, 85 },
    { 0x7b454489, 93 },
    { 0x3705448c, 46 },
    { 0x46d54488, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x78854490, 98 },
    { 0, 0 },
    { 0xc2c54492, 91 },
    { 0x3da54493, 91 },
    { 0xc8754492, 100 },
    { 0, 0 },
    { 0x62754496, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a544a0, 61 },
    { 0xae7544a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66544a6, 90 },
    { 0xc5d544a7, 91 },
    { 0x90b544a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8544ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9544af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0544b4, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d544b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5544bd, 38 },
    { 0x6ef544be, 65 },
    { 0xda4544bf, 76 },
    { 0xf5a544bd, 61 },
    { 0xc93544bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684544c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24544cd, 90 },
    { 0xf5e544ce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47544d1, 85 },
    { 0x5c6544d2, 88 },
    { 0xf38544d3, 67 },
    { 0x2d6544d4, 53 },
    { 0xd07544d1, 23 },
    { 0x402544d4, 82 },
    { 0xc04544d3, 68 },
    { 0x564544d8, 95 },
    { 0xab0544d8, 33 },
    { 0x3f6544d8, 33 },
    { 0x52e544db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22544df, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d544e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd544eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d544ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03544f3, 67 },
    { 0, 0 },
    { 0x147544f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d544fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba54500, 76 },
    { 0x43354501, 55 },
    { 0x80b54502, 90 },
    { 0x79454502, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6254506, 90 },
    { 0x60654506, 98 },
    { 0x99654508, 81 },
    { 0x76f54509, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d5450c, 90 },
    { 0xd4d5450d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f54515, 99 },
    { 0, 0 },
    { 0xe6354517, 95 },
    { 0x67c54518, 99 },
    { 0, 0 },
    { 0x3545451a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf454521, 33 },
    { 0x5aa54522, 66 },
    { 0xeaf54523, 90 },
    { 0, 0 },
    { 0xad654525, 80 },
    { 0x8be54526, 90 },
    { 0, 0 },
    { 0x74954528, 99 },
    { 0xf0754528, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac554534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b5453c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14654544, 61 },
    { 0, 0 },
    { 0xeab54546, 33 },
    { 0x98b54547, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e5454a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde954554, 34 },
    { 0, 0 },
    { 0x67754556, 90 },
    { 0x7e254557, 90 },
    { 0, 0 },
    { 0x69654559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c5455c, 61 },
    { 0x7135455c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c354564, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f54568, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f954570, 61 },
    { 0, 0 },
    { 0xee954572, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96954577, 61 },
    { 0, 0 },
    { 0x85b54579, 76 },
    { 0x5d35457a, 90 },
    { 0x13a5457b, 52 },
    { 0xfcc5457c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5954582, 93 },
    { 0x54c54582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d5458c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99654590, 33 },
    { 0xa6654591, 33 },
    { 0x16f54591, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf45459a, 98 },
    { 0x7f55459b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711545a4, 76 },
    { 0, 0 },
    { 0x2e1545a6, 76 },
    { 0x505545a6, 33 },
    { 0x41e545a8, 68 },
    { 0x347545a9, 33 },
    { 0, 0 },
    { 0x18f545ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563545b2, 90 },
    { 0x115545b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5545bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e545c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86545d0, 33 },
    { 0x846545d1, 61 },
    { 0, 0 },
    { 0x6ed545d3, 23 },
    { 0, 0 },
    { 0x12c545d5, 93 },
    { 0x570545d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243545da, 33 },
    { 0x9ec545db, 33 },
    { 0xf0f545dc, 76 },
    { 0x4f545db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea545e3, 90 },
    { 0xcc6545e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c545e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37545ef, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0545f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f545f5, 33 },
    { 0xe07545f5, 95 },
    { 0x6d9545f7, 95 },
    { 0x1e2545f8, 33 },
    { 0xbf4545f6, 33 },
    { 0xfa6545fa, 23 },
    { 0x315545fb, 91 },
    { 0x545545fc, 33 },
    { 0xc01545fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e54601, 76 },
    { 0xd8a54601, 17 },
    { 0x82c54602, 92 },
    { 0x2d254601, 61 },
    { 0xcca54602, 43 },
    { 0, 0 },
    { 0xc7e54607, 67 },
    { 0xd2d54608, 33 },
    { 0x13754608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b854610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a54618, 91 },
    { 0, 0 },
    { 0x1505461a, 23 },
    { 0, 0 },
    { 0x5b25461c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe54621, 33 },
    { 0, 0 },
    { 0xc9754623, 78 },
    { 0xcdd54623, 52 },
    { 0, 0 },
    { 0x74454626, 61 },
    { 0xb8754627, 95 },
    { 0, 0 },
    { 0x5c354629, 85 },
    { 0x8ae5462a, 95 },
    { 0xf985462b, 33 },
    { 0x9f55462c, 95 },
    { 0x8345462d, 33 },
    { 0x12f5462c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef54632, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e254635, 2 },
    { 0, 0 },
    { 0xdaa54637, 90 },
    { 0, 0 },
    { 0xcb154639, 23 },
    { 0, 0 },
    { 0x8e05463b, 81 },
    { 0, 0 },
    { 0x18d5463d, 23 },
    { 0, 0 },
    { 0xffb5463f, 23 },
    { 0x2c154640, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854645, 90 },
    { 0x70154645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90354649, 33 },
    { 0xc5d5464a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db5464d, 66 },
    { 0x2b35464d, 56 },
    { 0xec25464d, 95 },
    { 0x48c5464e, 33 },
    { 0x97e5464e, 23 },
    { 0, 0 },
    { 0xe5e54653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b35465d, 33 },
    { 0x8645465e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6154663, 98 },
    { 0, 0 },
    { 0xeae54665, 95 },
    { 0x2e754665, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6575466c, 77 },
    { 0x89f5466d, 61 },
    { 0xbc5466c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a54672, 61 },
    { 0, 0 },
    { 0xde554674, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd54678, 33 },
    { 0, 0 },
    { 0xd2b5467a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de5467d, 90 },
    { 0x75c5467e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac75468b, 61 },
    { 0x465468c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80254695, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b54698, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d5469c, 91 },
    { 0, 0 },
    { 0x2245469e, 76 },
    { 0xc6e5469e, 46 },
    { 0xbb4546a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee546a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552546ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939546b2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0546b5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x280546b8, 34 },
    { 0x720546b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4546bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0546c9, 23 },
    { 0xdc8546ca, 64 },
    { 0xe2b546ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2546cf, 88 },
    { 0x4b0546d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6546d4, 23 },
    { 0x216546d5, 33 },
    { 0x4b3546d5, 95 },
    { 0x7ee546d7, 95 },
    { 0x24a546d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x192546db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41546de, 77 },
    { 0xe4a546df, 34 },
    { 0x953546de, 61 },
    { 0x6a2546de, 98 },
    { 0xe8546e2, 76 },
    { 0x84e546e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78546e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167546ee, 40 },
    { 0, 0 },
    { 0x505546f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x710546f3, 55 },
    { 0, 0 },
    { 0xffc546f5, 76 },
    { 0, 0 },
    { 0x8a0546f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828546fb, 34 },
    { 0, 0 },
    { 0x2c0546fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc54700, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed54708, 52 },
    { 0, 0 },
    { 0x5ee5470a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95954713, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x77254716, 90 },
    { 0x6fb54716, 91 },
    { 0xecf54718, 61 },
    { 0xf8d54718, 61 },
    { 0, 0 },
    { 0x76b5471b, 75 },
    { 0x3965471b, 43 },
    { 0, 0 },
    { 0x6395471e, 90 },
    { 0x23a5471e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1954726, 55 },
    { 0, 0 },
    { 0x85654728, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2355472b, 98 },
    { 0x56b5472c, 33 },
    { 0, 0 },
    { 0xca45472e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5954735, 23 },
    { 0xfb554736, 76 },
    { 0xcc554737, 61 },
    { 0xdb054738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a5473b, 90 },
    { 0x4db5473c, 100 },
    { 0, 0 },
    { 0x1ac5473e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c654743, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a54746, 98 },
    { 0, 0 },
    { 0x41c54748, 98 },
    { 0x82b54749, 23 },
    { 0xd95474a, 78 },
    { 0x8254749, 33 },
    { 0, 0 },
    { 0xca75474d, 66 },
    { 0, 0 },
    { 0x3ad5474f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6854752, 23 },
    { 0x4ad54753, 56 },
    { 0xb9854752, 33 },
    { 0xb6a54755, 34 },
    { 0x8dc54756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3345475c, 99 },
    { 0xe995475c, 67 },
    { 0x87f5475c, 61 },
    { 0, 0 },
    { 0x4e654760, 55 },
    { 0, 0 },
    { 0x5cd54762, 67 },
    { 0x9de54763, 46 },
    { 0xcfd54762, 76 },
    { 0x7c354762, 93 },
    { 0x41754764, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc75476a, 90 },
    { 0x2a35476b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86954772, 90 },
    { 0x6cf54773, 96 },
    { 0x30954772, 98 },
    { 0x12854775, 33 },
    { 0, 0 },
    { 0x8da54777, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d5477b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b35478b, 95 },
    { 0xb335478c, 66 },
    { 0x19f5478c, 61 },
    { 0x2235478d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8154793, 61 },
    { 0xb3f54793, 33 },
    { 0, 0 },
    { 0x5f654796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b354799, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef5479c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d5479f, 98 },
    { 0x8c85479f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x284547a3, 91 },
    { 0xd3e547a4, 98 },
    { 0, 0 },
    { 0x418547a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d547b1, 67 },
    { 0, 0 },
    { 0x10f547b3, 61 },
    { 0xb8c547b4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5547b7, 95 },
    { 0xc71547b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e547bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1547c2, 91 },
    { 0xaa5547c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6547c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a547c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd547cd, 95 },
    { 0x5f2547ce, 23 },
    { 0, 0 },
    { 0xd14547d0, 43 },
    { 0, 0 },
    { 0x19e547d2, 55 },
    { 0xb76547d3, 90 },
    { 0xadb547d4, 46 },
    { 0x2e4547d5, 84 },
    { 0x9bd547d6, 33 },
    { 0, 0 },
    { 0xb60547d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b547e2, 61 },
    { 0x8ab547e3, 55 },
    { 0x5b9547e4, 40 },
    { 0xe8547e2, 95 },
    { 0xaf5547e6, 88 },
    { 0x62c547e5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x215547ea, 61 },
    { 0xb63547ea, 33 },
    { 0x7f547ec, 90 },
    { 0x8c7547ed, 90 },
    { 0, 0 },
    { 0x103547ef, 55 },
    { 0, 0 },
    { 0x6d9547f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2547f7, 61 },
    { 0, 0 },
    { 0xaa8547f9, 76 },
    { 0, 0 },
    { 0xdb1547fb, 33 },
    { 0x58c547fc, 61 },
    { 0x746547fd, 87 },
    { 0, 0 },
    { 0x214547ff, 74 },
    { 0, 0 },
    { 0x25954801, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b54804, 90 },
    { 0, 0 },
    { 0x5bd54806, 61 },
    { 0x96454807, 33 },
    { 0xa7554807, 90 },
    { 0x89054809, 98 },
    { 0, 0 },
    { 0xabe5480b, 90 },
    { 0xd365480c, 98 },
    { 0x9d15480c, 40 },
    { 0xa6c5480e, 65 },
    { 0x30e5480f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f54818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15481b, 90 },
    { 0xa345481c, 76 },
    { 0xfce5481d, 95 },
    { 0xe365481c, 98 },
    { 0xce25481f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2254822, 33 },
    { 0, 0 },
    { 0xedb54824, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c54827, 90 },
    { 0x95154827, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c45482b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6854830, 46 },
    { 0x59b54831, 33 },
    { 0xf354832, 87 },
    { 0x38854830, 84 },
    { 0, 0 },
    { 0x92754835, 76 },
    { 0, 0 },
    { 0x4db54837, 66 },
    { 0x26854837, 33 },
    { 0x37454839, 91 },
    { 0xe2954837, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c5483d, 93 },
    { 0xe6e5483e, 99 },
    { 0x8c65483f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x91654842, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f5484a, 38 },
    { 0x7e25484a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa815484e, 95 },
    { 0x8e25484e, 33 },
    { 0, 0 },
    { 0x62e54851, 67 },
    { 0xfd754851, 78 },
    { 0, 0 },
    { 0x16154854, 95 },
    { 0x39254854, 95 },
    { 0xd9854856, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495485b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2255485e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c054865, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a54869, 23 },
    { 0x5695486a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d15486d, 23 },
    { 0, 0 },
    { 0xc405486f, 90 },
    { 0xc4854870, 99 },
    { 0, 0 },
    { 0x17154872, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd754879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e75487c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa5487f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c254883, 90 },
    { 0, 0 },
    { 0x97f54885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd25488a, 95 },
    { 0x9de5488b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055488f, 98 },
    { 0x6b254890, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92454898, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e5489d, 66 },
    { 0x7ff5489d, 55 },
    { 0xf4c5489f, 81 },
    { 0x7f5548a0, 61 },
    { 0xa6e548a1, 33 },
    { 0x570548a2, 76 },
    { 0x780548a1, 43 },
    { 0x16f548a3, 33 },
    { 0xf23548a5, 67 },
    { 0x7d3548a3, 24 },
    { 0x899548a7, 85 },
    { 0x871548a8, 33 },
    { 0x4b7548a9, 55 },
    { 0x49b548a9, 61 },
    { 0xfa4548ab, 98 },
    { 0, 0 },
    { 0xa5d548ad, 68 },
    { 0xf06548ae, 55 },
    { 0x6d3548ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a548b7, 99 },
    { 0xab1548b7, 76 },
    { 0x27f548b9, 66 },
    { 0x88f548ba, 61 },
    { 0x25b548bb, 65 },
    { 0xf4c548bb, 34 },
    { 0, 0 },
    { 0xaec548be, 66 },
    { 0x27b548be, 95 },
    { 0x90e548be, 23 },
    { 0x6d8548c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f548c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56548d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8548d3, 85 },
    { 0x645548d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb548db, 34 },
    { 0xb2d548dc, 43 },
    { 0xbf8548db, 95 },
    { 0xa2e548de, 95 },
    { 0, 0 },
    { 0x475548e0, 33 },
    { 0xef548e1, 43 },
    { 0xdd2548e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec548e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x837548ea, 67 },
    { 0x4f4548eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8548f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x30548f3, 98 },
    { 0, 0 },
    { 0x545548f5, 90 },
    { 0x5f0548f5, 99 },
    { 0, 0 },
    { 0x399548f8, 99 },
    { 0x2ff548f9, 17 },
    { 0, 0 },
    { 0x64548fb, 85 },
    { 0x6548fc, 76 },
    { 0x3b8548fc, 93 },
    { 0xee5548fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab54902, 95 },
    { 0, 0 },
    { 0x7d754904, 91 },
    { 0x89154904, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5490a, 61 },
    { 0xc2b5490a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc5490f, 98 },
    { 0xef54910, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61254915, 95 },
    { 0x1cd54915, 43 },
    { 0xa1554917, 55 },
    { 0, 0 },
    { 0x1c854919, 95 },
    { 0, 0 },
    { 0x9175491b, 98 },
    { 0, 0 },
    { 0x1bc5491d, 52 },
    { 0xa025491d, 61 },
    { 0xe715491d, 33 },
    { 0x93c54920, 85 },
    { 0, 0 },
    { 0xf9154922, 90 },
    { 0xc3554923, 80 },
    { 0xad54923, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7225492b, 98 },
    { 0xfa35492b, 34 },
    { 0xd895492d, 43 },
    { 0, 0 },
    { 0x7ab5492f, 95 },
    { 0xb125492f, 90 },
    { 0xe8054931, 98 },
    { 0xd9154932, 90 },
    { 0x4da54932, 90 },
    { 0x8c654932, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b554939, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd54948, 65 },
    { 0x19854948, 75 },
    { 0, 0 },
    { 0xeed5494b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d254950, 23 },
    { 0x4a054951, 76 },
    { 0, 0 },
    { 0x55354953, 33 },
    { 0xb6054954, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a54958, 84 },
    { 0x18354959, 23 },
    { 0x46154958, 33 },
    { 0xa585495b, 61 },
    { 0x25e5495c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4775495f, 76 },
    { 0x6c45495f, 66 },
    { 0x3a654960, 23 },
    { 0x81554962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64254965, 90 },
    { 0, 0 },
    { 0x24d54967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde95496b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d054970, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35497c, 95 },
    { 0xa865497d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58754980, 52 },
    { 0, 0 },
    { 0xaaf54982, 33 },
    { 0xd8554983, 98 },
    { 0x90a54983, 55 },
    { 0xf0554985, 88 },
    { 0x16f54983, 61 },
    { 0, 0 },
    { 0x87454988, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f25498b, 33 },
    { 0x6b35498c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c54993, 98 },
    { 0x52f54994, 46 },
    { 0x3b754993, 87 },
    { 0, 0 },
    { 0xd4e54997, 98 },
    { 0, 0 },
    { 0x2f54999, 23 },
    { 0, 0 },
    { 0x49c5499b, 55 },
    { 0x7db5499c, 23 },
    { 0xef35499b, 55 },
    { 0, 0 },
    { 0xd35499f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf549a2, 95 },
    { 0, 0 },
    { 0x469549a4, 33 },
    { 0x8c549a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae549a8, 23 },
    { 0, 0 },
    { 0x64c549aa, 20 },
    { 0xbe549aa, 90 },
    { 0x8fa549ab, 90 },
    { 0xfc0549ad, 55 },
    { 0xbaf549ad, 76 },
    { 0x144549af, 90 },
    { 0, 0 },
    { 0x6e9549b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc549b9, 33 },
    { 0, 0 },
    { 0xe1c549bb, 33 },
    { 0, 0 },
    { 0x596549bd, 76 },
    { 0xc5b549be, 23 },
    { 0x689549be, 74 },
    { 0x23e549c0, 55 },
    { 0xe5e549bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771549c5, 43 },
    { 0, 0 },
    { 0x1c5549c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813549cc, 34 },
    { 0x710549cc, 98 },
    { 0x867549ce, 90 },
    { 0, 0 },
    { 0xbc6549d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1549d9, 23 },
    { 0xba8549da, 76 },
    { 0x727549db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e549ea, 34 },
    { 0xe8a549eb, 33 },
    { 0xc25549ec, 98 },
    { 0xa17549ed, 33 },
    { 0xb92549ed, 33 },
    { 0x6f6549ee, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x649549f2, 23 },
    { 0xcae549f3, 98 },
    { 0xc51549f2, 76 },
    { 0x4ba549f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x120549f8, 93 },
    { 0xbc2549f8, 76 },
    { 0, 0 },
    { 0x631549fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54549ff, 95 },
    { 0x89e54a00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1354a05, 90 },
    { 0xbc454a05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x97054a09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf54a0d, 85 },
    { 0xf1e54a0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x97254a11, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e354a14, 84 },
    { 0, 0 },
    { 0xb1154a16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x34554a19, 55 },
    { 0xee754a1a, 46 },
    { 0x25354a1b, 33 },
    { 0x5ae54a1b, 95 },
    { 0x90d54a1d, 99 },
    { 0xc4454a1e, 56 },
    { 0x96754a1e, 33 },
    { 0x9154a20, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56454a25, 90 },
    { 0x26154a26, 33 },
    { 0x31a54a26, 90 },
    { 0xe0b54a28, 23 },
    { 0x9f554a26, 61 },
    { 0x4554a26, 78 },
    { 0x59354a2b, 55 },
    { 0xd5554a2c, 61 },
    { 0xf9154a2d, 90 },
    { 0x30f54a2e, 55 },
    { 0xed054a2e, 61 },
    { 0, 0 },
    { 0xab254a31, 61 },
    { 0x7c654a31, 95 },
    { 0, 0 },
    { 0x2a354a34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa54a37, 61 },
    { 0xbe754a37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e54a3c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x36754a3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f54a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb154a4e, 43 },
    { 0xea154a4f, 61 },
    { 0x73f54a4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72654a54, 23 },
    { 0xebd54a55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4654a61, 76 },
    { 0x7b354a62, 94 },
    { 0x29754a62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b54a66, 34 },
    { 0xb6354a66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1954a6c, 52 },
    { 0x5f554a6d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6554a70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c854a73, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2754a7b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c54a7e, 98 },
    { 0, 0 },
    { 0x51654a80, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1354a83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab54a8d, 61 },
    { 0xda054a8e, 90 },
    { 0xef654a8f, 96 },
    { 0, 0 },
    { 0x3ac54a91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39754a98, 90 },
    { 0xa6154a99, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0054a9e, 61 },
    { 0x8c654a9f, 52 },
    { 0x93054a9f, 61 },
    { 0x37354aa1, 76 },
    { 0x76f54aa1, 61 },
    { 0xe8f54aa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe54aaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a854aad, 33 },
    { 0x41b54aad, 99 },
    { 0xe0854aad, 90 },
    { 0, 0 },
    { 0xf8d54ab1, 61 },
    { 0x6b354ab2, 87 },
    { 0, 0 },
    { 0x1e254ab4, 90 },
    { 0x38e54ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c54aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44654abd, 55 },
    { 0xcaf54abd, 76 },
    { 0xa6754abd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1154ac4, 61 },
    { 0x15c54ac5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82754ad0, 75 },
    { 0x76e54ad1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b54ad5, 76 },
    { 0x61554ad5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa54ad9, 98 },
    { 0x22d54ada, 90 },
    { 0xa9054adb, 83 },
    { 0x37454adc, 61 },
    { 0xd2f54add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb54ae7, 85 },
    { 0xb6554ae8, 33 },
    { 0x91154ae7, 52 },
    { 0x3ad54aea, 66 },
    { 0x8a954aeb, 61 },
    { 0x2b354aec, 23 },
    { 0xb0754aed, 19 },
    { 0x48854ae7, 23 },
    { 0x4b954aef, 43 },
    { 0x12354ae8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e54afa, 33 },
    { 0x21854afa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5454aff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b054b05, 76 },
    { 0xcb754b06, 55 },
    { 0x74c54b07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94554b12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e54b15, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a454b19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a54b1f, 65 },
    { 0, 0 },
    { 0x4ca54b21, 87 },
    { 0xe7354b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e854b2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3254b2e, 74 },
    { 0xceb54b2f, 23 },
    { 0x65c54b30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb54b33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81754b38, 78 },
    { 0x92954b39, 95 },
    { 0xecc54b38, 90 },
    { 0xa7054b38, 33 },
    { 0xa2c54b38, 95 },
    { 0, 0 },
    { 0x11354b3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f254b49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa54b50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a54b55, 98 },
    { 0, 0 },
    { 0x3e54b57, 98 },
    { 0, 0 },
    { 0xe0d54b59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef54b5c, 61 },
    { 0x11054b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c254b60, 95 },
    { 0, 0 },
    { 0x6b254b62, 77 },
    { 0, 0 },
    { 0x7b554b64, 33 },
    { 0, 0 },
    { 0x88154b66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe54b6b, 33 },
    { 0x8e54b6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35554b71, 90 },
    { 0x80854b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b054b77, 43 },
    { 0, 0 },
    { 0x53d54b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b54b7e, 76 },
    { 0, 0 },
    { 0xac454b80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f54b83, 67 },
    { 0x12654b84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc54b8c, 34 },
    { 0x12154b8d, 34 },
    { 0x24c54b8d, 43 },
    { 0xfd854b8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41054b95, 61 },
    { 0, 0 },
    { 0x91254b97, 98 },
    { 0, 0 },
    { 0xd3554b99, 61 },
    { 0xc9454b9a, 66 },
    { 0x1d54b9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d54b9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c854ba6, 93 },
    { 0xe2b54ba7, 33 },
    { 0xe1a54ba8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90254bad, 90 },
    { 0, 0 },
    { 0x1f154baf, 87 },
    { 0xd0154bb0, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c54bb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb54bba, 90 },
    { 0x4f054bbb, 80 },
    { 0, 0 },
    { 0x57654bbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c354bc7, 93 },
    { 0xa9654bc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4054bcb, 65 },
    { 0x12e54bcb, 23 },
    { 0x76354bcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b54bda, 61 },
    { 0, 0 },
    { 0xc4e54bdc, 90 },
    { 0, 0 },
    { 0x70254bde, 95 },
    { 0, 0 },
    { 0x4c954be0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d54be3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e54bec, 61 },
    { 0x40b54bec, 61 },
    { 0xb6654bec, 55 },
    { 0, 0 },
    { 0x354bf0, 61 },
    { 0x52c54bf0, 78 },
    { 0, 0 },
    { 0xb054bf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb954bfa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c54bfd, 76 },
    { 0, 0 },
    { 0x5de54bff, 61 },
    { 0, 0 },
    { 0x80554c01, 98 },
    { 0, 0 },
    { 0xbbc54c03, 100 },
    { 0xaa354c04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd54c0b, 91 },
    { 0xb2b54c0c, 43 },
    { 0x90254c0d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5954c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87354c14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2454c1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45554c1e, 60 },
    { 0xed154c1f, 76 },
    { 0xa8354c1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb54c28, 80 },
    { 0x4f754c28, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19654c2d, 33 },
    { 0x77854c2d, 33 },
    { 0, 0 },
    { 0x74f54c30, 33 },
    { 0, 0 },
    { 0x7f954c32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6354c35, 76 },
    { 0, 0 },
    { 0x43354c37, 95 },
    { 0x81154c38, 77 },
    { 0x6b954c39, 61 },
    { 0xac754c39, 90 },
    { 0x2a54c3b, 61 },
    { 0x6ea54c37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4154c3f, 95 },
    { 0x74f54c40, 98 },
    { 0x62654c40, 91 },
    { 0xe3654c40, 90 },
    { 0x3854c40, 55 },
    { 0x1fc54c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac854c4b, 90 },
    { 0xa7554c4c, 95 },
    { 0x6a254c4d, 98 },
    { 0x5c154c4b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b54c51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc54c54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf54c5b, 99 },
    { 0xc554c5c, 52 },
    { 0xa2454c5c, 98 },
    { 0, 0 },
    { 0x94054c5f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23554c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b54c6e, 90 },
    { 0, 0 },
    { 0x70354c70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d054c75, 23 },
    { 0x5eb54c76, 79 },
    { 0, 0 },
    { 0x46c54c78, 23 },
    { 0, 0 },
    { 0x27754c7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a854c80, 17 },
    { 0x6fa54c81, 33 },
    { 0, 0 },
    { 0x59154c83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20954c8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30854c8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c54c92, 33 },
    { 0xacd54c93, 90 },
    { 0xe9154c94, 61 },
    { 0x1f554c95, 23 },
    { 0, 0 },
    { 0x2ef54c97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd54c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac54ca0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x20254ca3, 76 },
    { 0x62154ca3, 33 },
    { 0, 0 },
    { 0xacc54ca6, 96 },
    { 0x15b54ca7, 33 },
    { 0xe6054ca7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5854cad, 68 },
    { 0, 0 },
    { 0xefd54caf, 33 },
    { 0xfb854cb0, 61 },
    { 0x4a654cb1, 98 },
    { 0x6f454caf, 90 },
    { 0xeda54cb3, 61 },
    { 0x3a454cb3, 23 },
    { 0, 0 },
    { 0x51054cb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f054cba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e54cbd, 76 },
    { 0x96054cbd, 33 },
    { 0x3a854cbe, 46 },
    { 0xc2154cbf, 93 },
    { 0x4ca54cbf, 77 },
    { 0xd6b54cc2, 85 },
    { 0, 0 },
    { 0x63254cc4, 33 },
    { 0xd0354cc5, 78 },
    { 0, 0 },
    { 0x6de54cc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33454cce, 90 },
    { 0x55854ccf, 33 },
    { 0x3654cd0, 68 },
    { 0x2f854cd1, 33 },
    { 0x9a854cd0, 93 },
    { 0x96554cd3, 76 },
    { 0x5b354cd4, 95 },
    { 0, 0 },
    { 0x5f954cd6, 33 },
    { 0x6954cd7, 61 },
    { 0x73754cd8, 90 },
    { 0x21954cd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e54cdd, 98 },
    { 0x39f54cdd, 37 },
    { 0x49e54cdd, 43 },
    { 0x7ea54ce0, 97 },
    { 0x79b54ce1, 90 },
    { 0, 0 },
    { 0x47a54ce3, 46 },
    { 0xeb254ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99354ce7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb554ceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a954cf1, 98 },
    { 0, 0 },
    { 0x31654cf3, 33 },
    { 0x70c54cf4, 37 },
    { 0, 0 },
    { 0x9cb54cf6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff54cfb, 61 },
    { 0x43f54cfc, 23 },
    { 0x50554cfd, 23 },
    { 0xb9554cfc, 77 },
    { 0, 0 },
    { 0x6af54d00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d54d08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa54d0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0454d0e, 98 },
    { 0x24254d0f, 87 },
    { 0x99054d10, 61 },
    { 0, 0 },
    { 0xc7554d12, 33 },
    { 0, 0 },
    { 0xf2154d14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e254d17, 33 },
    { 0x8c354d18, 33 },
    { 0x69e54d18, 87 },
    { 0, 0 },
    { 0xd1754d1b, 99 },
    { 0xb6e54d1c, 94 },
    { 0x3e154d1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d354d20, 56 },
    { 0xded54d20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3954d25, 66 },
    { 0xaf454d25, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff54d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f54d2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64454d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4154d38, 33 },
    { 0xa9054d39, 76 },
    { 0xea254d3a, 51 },
    { 0x24754d3b, 23 },
    { 0, 0 },
    { 0x6b154d3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f554d40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c54d45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc54d48, 68 },
    { 0xec154d48, 51 },
    { 0xc7154d4a, 90 },
    { 0x9d954d4b, 91 },
    { 0x22c54d4c, 23 },
    { 0, 0 },
    { 0x65d54d4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7154d51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9554d54, 98 },
    { 0xe0154d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40154d5a, 98 },
    { 0, 0 },
    { 0xebd54d5c, 98 },
    { 0, 0 },
    { 0xef554d5e, 61 },
    { 0xc6254d5f, 95 },
    { 0, 0 },
    { 0x53054d61, 64 },
    { 0, 0 },
    { 0x46654d63, 33 },
    { 0x86354d64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7954d67, 23 },
    { 0, 0 },
    { 0xf0754d69, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc54d70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e54d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39554d7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a854d82, 78 },
    { 0xb8354d83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe454d86, 95 },
    { 0x8a754d87, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e54d8a, 95 },
    { 0x6e554d8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da54d91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6954d95, 33 },
    { 0xc5854d96, 43 },
    { 0x2da54d97, 90 },
    { 0, 0 },
    { 0xfb854d99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a054d9c, 95 },
    { 0xbf354d9c, 78 },
    { 0x5e954d9e, 67 },
    { 0x65554d9f, 98 },
    { 0x7a554d9e, 33 },
    { 0, 0 },
    { 0x18354da2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c54da5, 61 },
    { 0xeb54da6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c54dab, 95 },
    { 0, 0 },
    { 0x56354dad, 55 },
    { 0x62f54dae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7154db1, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c254dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a54dc2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1854dcc, 66 },
    { 0, 0 },
    { 0xe7254dce, 66 },
    { 0xe2454dce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3154dd4, 88 },
    { 0xb3d54dd5, 61 },
    { 0x4cc54dd5, 76 },
    { 0, 0 },
    { 0x9d754dd8, 96 },
    { 0, 0 },
    { 0x17a54dda, 98 },
    { 0xa7c54dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a354ddf, 17 },
    { 0x9f54de0, 95 },
    { 0x7fd54de1, 34 },
    { 0x4ed54de2, 88 },
    { 0x46854de0, 55 },
    { 0xc8954de1, 95 },
    { 0xdb354de5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3054dec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94854df1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1054df4, 76 },
    { 0, 0 },
    { 0xef954df6, 23 },
    { 0, 0 },
    { 0x1d754df8, 55 },
    { 0x2dd54df8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a54dfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d54e04, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7654e07, 66 },
    { 0xff854e07, 65 },
    { 0x7a754e09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64254e0e, 61 },
    { 0x5d654e0f, 33 },
    { 0x55654e0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d454e15, 43 },
    { 0, 0 },
    { 0x51754e17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd54e1b, 33 },
    { 0xc1454e1c, 75 },
    { 0x90754e1d, 61 },
    { 0x8c254e1e, 96 },
    { 0x13754e1f, 43 },
    { 0x1de54e1f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd54e27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49854e2b, 23 },
    { 0, 0 },
    { 0xeb854e2d, 23 },
    { 0x80354e2d, 98 },
    { 0x41e54e2f, 43 },
    { 0x6754e30, 98 },
    { 0x3754e31, 90 },
    { 0x30954e32, 46 },
    { 0x9c354e30, 84 },
    { 0x5e754e34, 43 },
    { 0x76954e35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76754e3a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x30754e3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62054e41, 33 },
    { 0, 0 },
    { 0x9b854e43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce54e46, 66 },
    { 0x80454e47, 61 },
    { 0, 0 },
    { 0x9ea54e49, 76 },
    { 0xf754e49, 36 },
    { 0x1e554e49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2354e51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x85854e54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e54e57, 95 },
    { 0x47454e58, 23 },
    { 0x19c54e58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39754e65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4854e68, 87 },
    { 0, 0 },
    { 0x24854e6a, 91 },
    { 0, 0 },
    { 0xdeb54e6c, 90 },
    { 0x8f354e6d, 90 },
    { 0x69654e6e, 61 },
    { 0x64354e6f, 84 },
    { 0x35354e6f, 91 },
    { 0x49454e71, 85 },
    { 0xbb354e72, 80 },
    { 0x3d854e72, 90 },
    { 0x88e54e74, 23 },
    { 0xe2354e74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da54e7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99754e7f, 90 },
    { 0xbe754e80, 93 },
    { 0x1ec54e80, 75 },
    { 0xa5f54e80, 85 },
    { 0xcc954e83, 33 },
    { 0x72654e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89554e8f, 61 },
    { 0xc9954e90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81154e96, 33 },
    { 0, 0 },
    { 0xad754e98, 46 },
    { 0, 0 },
    { 0xfc754e9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0654e9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54554ea4, 34 },
    { 0x54254ea5, 23 },
    { 0x6ec54ea6, 84 },
    { 0xafe54ea7, 81 },
    { 0x53254ea6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a54ead, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb454eb6, 33 },
    { 0, 0 },
    { 0xbea54eb8, 46 },
    { 0x6d954eb9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c354ebc, 61 },
    { 0xe9f54ebd, 76 },
    { 0xbf854ebe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f54ec2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49554ec8, 23 },
    { 0x43954ec9, 46 },
    { 0x4eb54ec9, 43 },
    { 0, 0 },
    { 0x46854ecc, 65 },
    { 0x79f54ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c54ed0, 90 },
    { 0, 0 },
    { 0x83f54ed2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa54ed8, 90 },
    { 0xb8954ed9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1254edf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c754ee4, 23 },
    { 0xda54ee5, 100 },
    { 0x16754ee5, 95 },
    { 0, 0 },
    { 0x6ca54ee8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b54eec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c854ef1, 90 },
    { 0x90354ef1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a54ef7, 98 },
    { 0, 0 },
    { 0xfbe54ef9, 43 },
    { 0, 0 },
    { 0x17f54efb, 90 },
    { 0x4a454efc, 100 },
    { 0x18054efd, 90 },
    { 0xbd954efc, 90 },
    { 0xe6b54eff, 85 },
    { 0x61e54f00, 61 },
    { 0x7de54efd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd54f07, 23 },
    { 0xe6954f07, 43 },
    { 0x99754f09, 95 },
    { 0, 0 },
    { 0x3a254f0b, 95 },
    { 0x3ca54f0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a354f10, 88 },
    { 0xe9b54f10, 43 },
    { 0, 0 },
    { 0x5f854f13, 95 },
    { 0x6c054f13, 95 },
    { 0x6ff54f13, 93 },
    { 0xe4a54f14, 90 },
    { 0x72e54f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd554f1e, 99 },
    { 0x4ec54f1f, 90 },
    { 0xbb154f20, 61 },
    { 0, 0 },
    { 0xc6954f22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64354f28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95554f2c, 98 },
    { 0xc4854f2d, 95 },
    { 0x34a54f2e, 82 },
    { 0x4c254f2f, 17 },
    { 0xcae54f30, 76 },
    { 0xa1a54f2f, 90 },
    { 0x85954f32, 55 },
    { 0xa8654f2d, 34 },
    { 0xa2454f34, 55 },
    { 0xa2f54f35, 23 },
    { 0xcba54f35, 77 },
    { 0x10f54f37, 55 },
    { 0x25754f38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae54f3c, 98 },
    { 0, 0 },
    { 0x44b54f3e, 61 },
    { 0x62f54f3e, 61 },
    { 0, 0 },
    { 0x2d954f41, 98 },
    { 0x5aa54f41, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde54f50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16354f53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a54f5d, 76 },
    { 0x50754f5e, 95 },
    { 0x64b54f5f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x99154f62, 95 },
    { 0xc3c54f63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f54f67, 37 },
    { 0xf3754f68, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43954f6c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c54f72, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8654f7a, 69 },
    { 0x5054f7a, 34 },
    { 0xbac54f7a, 76 },
    { 0x97654f7b, 34 },
    { 0x9c954f7e, 61 },
    { 0x4cb54f7e, 98 },
    { 0, 0 },
    { 0xd0354f81, 23 },
    { 0, 0 },
    { 0x68154f83, 61 },
    { 0x7ae54f84, 75 },
    { 0, 0 },
    { 0x88954f86, 90 },
    { 0x73a54f87, 98 },
    { 0xd9754f88, 34 },
    { 0xcbf54f88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22154f8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d54f92, 61 },
    { 0x96554f92, 98 },
    { 0xa6754f94, 91 },
    { 0xda254f95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b54f9c, 55 },
    { 0x4c154f9d, 98 },
    { 0x97454f9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea54fa3, 23 },
    { 0xdb954fa4, 23 },
    { 0xdec54fa4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a754fa8, 81 },
    { 0x86954fa9, 69 },
    { 0xe1e54fa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c54fae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a54fb2, 91 },
    { 0x8b154fb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47954fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee854fc0, 90 },
    { 0xb9554fc1, 90 },
    { 0, 0 },
    { 0x1e554fc3, 90 },
    { 0, 0 },
    { 0x6e654fc5, 97 },
    { 0xece54fc5, 66 },
    { 0x67b54fc5, 95 },
    { 0, 0 },
    { 0xb3454fc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x42354fcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61054fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d854fd6, 90 },
    { 0, 0 },
    { 0x1c754fd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab54fdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x25854fdf, 33 },
    { 0, 0 },
    { 0x40754fe1, 61 },
    { 0x85654fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x37454fe5, 76 },
    { 0xaeb54fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d154feb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b54fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52954ff4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x49354ff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d854ffa, 33 },
    { 0xda454ffb, 95 },
    { 0xad254ffa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a55001, 23 },
    { 0x34b55002, 40 },
    { 0, 0 },
    { 0x37455004, 23 },
    { 0, 0 },
    { 0xf8055006, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd85500a, 33 },
    { 0xd0c5500b, 90 },
    { 0xef25500c, 33 },
    { 0x5135500d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0855013, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c355016, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435501a, 23 },
    { 0xae15501b, 23 },
    { 0, 0 },
    { 0x7a35501d, 66 },
    { 0, 0 },
    { 0xbb15501f, 77 },
    { 0xb55020, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb55026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e5502b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43055031, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32055036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf5503b, 87 },
    { 0xc3b5503c, 98 },
    { 0x8fc5503d, 61 },
    { 0, 0 },
    { 0xa835503f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c055042, 90 },
    { 0, 0 },
    { 0x13c55044, 100 },
    { 0xe2455045, 98 },
    { 0, 0 },
    { 0x34355047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a55051, 23 },
    { 0x24b55051, 95 },
    { 0, 0 },
    { 0xda55054, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b55058, 17 },
    { 0x79055059, 33 },
    { 0xc7355058, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c5505f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be55062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40955067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f5506e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d855071, 90 },
    { 0xa4a55071, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f55077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa55081, 84 },
    { 0, 0 },
    { 0x77c55083, 52 },
    { 0x2fa55084, 23 },
    { 0x2f155085, 95 },
    { 0x16955084, 76 },
    { 0x21b55086, 61 },
    { 0x7955088, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2155508b, 33 },
    { 0, 0 },
    { 0x46b5508d, 49 },
    { 0x6b75508e, 25 },
    { 0x2db5508f, 33 },
    { 0x5d55090, 98 },
    { 0xa6255090, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97855097, 91 },
    { 0x2d955098, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5509c, 46 },
    { 0xfd05509c, 46 },
    { 0x92e5509e, 23 },
    { 0, 0 },
    { 0xa89550a0, 90 },
    { 0x221550a1, 33 },
    { 0xe70550a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3550a9, 55 },
    { 0, 0 },
    { 0xcdc550ab, 95 },
    { 0, 0 },
    { 0x8a3550ad, 66 },
    { 0xa84550ae, 46 },
    { 0, 0 },
    { 0x8db550b0, 76 },
    { 0x7ad550b0, 33 },
    { 0x338550b2, 23 },
    { 0, 0 },
    { 0xb47550b4, 55 },
    { 0x72e550b5, 95 },
    { 0xf77550b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d550bc, 90 },
    { 0xd88550bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c550c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5550c3, 90 },
    { 0, 0 },
    { 0xc8550c5, 98 },
    { 0x21b550c5, 95 },
    { 0, 0 },
    { 0x243550c8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26550cb, 100 },
    { 0, 0 },
    { 0x313550cd, 98 },
    { 0x305550ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9550d1, 65 },
    { 0x5fd550d2, 23 },
    { 0x686550d3, 66 },
    { 0x94d550d3, 61 },
    { 0xbc550d5, 95 },
    { 0xda1550d6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d550d9, 76 },
    { 0x234550da, 100 },
    { 0x209550db, 23 },
    { 0x67e550da, 93 },
    { 0xb8f550db, 23 },
    { 0x584550d9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17550e4, 68 },
    { 0, 0 },
    { 0x197550e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe550ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e550f5, 33 },
    { 0x5f550f6, 48 },
    { 0x37a550f6, 17 },
    { 0x541550f8, 95 },
    { 0x966550f9, 23 },
    { 0x160550f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x630550fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5855101, 95 },
    { 0x8ee55102, 61 },
    { 0, 0 },
    { 0x19355104, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f5510a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51355110, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b55113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6655511c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46755123, 98 },
    { 0x86c55124, 61 },
    { 0, 0 },
    { 0x59055126, 95 },
    { 0, 0 },
    { 0x82e55128, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5512b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce55130, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24355137, 90 },
    { 0x5d455137, 33 },
    { 0x60055138, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7325513c, 33 },
    { 0, 0 },
    { 0x3145513e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75755147, 90 },
    { 0x46655148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4235514b, 93 },
    { 0x1c15514c, 98 },
    { 0x6585514c, 4 },
    { 0xde25514e, 61 },
    { 0, 0 },
    { 0xe9055150, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a55156, 90 },
    { 0xeb655157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa425515b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7535515e, 90 },
    { 0xa2e5515e, 78 },
    { 0xfae5515f, 94 },
    { 0x81055161, 100 },
    { 0, 0 },
    { 0x94955163, 33 },
    { 0, 0 },
    { 0xb4d55165, 98 },
    { 0x28855166, 46 },
    { 0xa1355167, 75 },
    { 0x94455165, 95 },
    { 0xfdb55165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b65516d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa55171, 93 },
    { 0, 0 },
    { 0x27b55173, 24 },
    { 0x35e55173, 43 },
    { 0, 0 },
    { 0xb5555176, 61 },
    { 0, 0 },
    { 0x25d55178, 76 },
    { 0x37f55179, 98 },
    { 0xbd05517a, 90 },
    { 0xe605517b, 33 },
    { 0, 0 },
    { 0xb635517d, 95 },
    { 0x2c05517e, 61 },
    { 0xd615517f, 93 },
    { 0x70655180, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb855183, 55 },
    { 0x3555184, 90 },
    { 0, 0 },
    { 0x95a55186, 61 },
    { 0x69855187, 90 },
    { 0x35c55187, 84 },
    { 0xfb355189, 90 },
    { 0x5865518a, 33 },
    { 0x5ce5518b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf485518e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de55192, 65 },
    { 0xe8555193, 78 },
    { 0x61455194, 34 },
    { 0x71355194, 33 },
    { 0x28a55196, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc5519b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95519f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20551a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4551a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720551ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c551af, 52 },
    { 0x9a7551af, 46 },
    { 0xd78551b1, 90 },
    { 0x373551b0, 56 },
    { 0x63b551b3, 92 },
    { 0x329551b4, 33 },
    { 0x502551b5, 95 },
    { 0, 0 },
    { 0x47e551b7, 58 },
    { 0x700551b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a551bd, 34 },
    { 0x923551be, 33 },
    { 0xfd2551bf, 23 },
    { 0xe78551c0, 100 },
    { 0x4ec551c1, 85 },
    { 0, 0 },
    { 0xa3e551c3, 34 },
    { 0, 0 },
    { 0xc27551c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b551c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4551ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815551d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc551d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e551da, 78 },
    { 0xf1e551db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac551de, 95 },
    { 0x3c6551df, 34 },
    { 0xa1551e0, 33 },
    { 0x4a5551df, 90 },
    { 0, 0 },
    { 0xcb4551e3, 33 },
    { 0xf3e551e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c551e8, 98 },
    { 0x43f551e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d551ec, 23 },
    { 0x192551ec, 95 },
    { 0x1fe551ee, 77 },
    { 0xd33551ef, 34 },
    { 0, 0 },
    { 0x2b0551f1, 77 },
    { 0, 0 },
    { 0x55e551f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da551f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff55202, 33 },
    { 0xdba55202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6655209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe55520d, 93 },
    { 0x3755520e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5855214, 98 },
    { 0xb6055215, 90 },
    { 0xa9355216, 90 },
    { 0x2b855217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f5521a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c45521e, 90 },
    { 0xb315521f, 98 },
    { 0xf845521f, 93 },
    { 0xd7655221, 34 },
    { 0x9cc55221, 90 },
    { 0x14855222, 90 },
    { 0xd6955224, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50455228, 33 },
    { 0x77955229, 93 },
    { 0x31c5522a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe55230, 90 },
    { 0x4d055231, 90 },
    { 0x4055231, 61 },
    { 0, 0 },
    { 0xa1155234, 95 },
    { 0, 0 },
    { 0x1bc55236, 93 },
    { 0x68855237, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9145523c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2355242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6455246, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac55249, 33 },
    { 0x4de5524a, 93 },
    { 0x8375524a, 100 },
    { 0x6fa5524c, 78 },
    { 0x7555524a, 90 },
    { 0x62d5524e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52555251, 23 },
    { 0xd0655251, 61 },
    { 0xc9055253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97255259, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e55260, 43 },
    { 0, 0 },
    { 0x43b55262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd55268, 84 },
    { 0, 0 },
    { 0x4185526a, 90 },
    { 0xee95526b, 81 },
    { 0x5175526b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd955274, 43 },
    { 0xc4e55275, 23 },
    { 0, 0 },
    { 0xbc255277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b5527a, 17 },
    { 0x1cf5527a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a455287, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda65528b, 90 },
    { 0x5b15528c, 61 },
    { 0xe7e5528d, 98 },
    { 0x8c45528d, 90 },
    { 0xf075528f, 46 },
    { 0x5b255290, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de55293, 34 },
    { 0, 0 },
    { 0x96655295, 90 },
    { 0, 0 },
    { 0x92455297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff85529b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd185529e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3552a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2552aa, 17 },
    { 0x1c5552ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a552af, 69 },
    { 0, 0 },
    { 0x523552b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f552b4, 56 },
    { 0xf65552b5, 33 },
    { 0x32c552b6, 52 },
    { 0x5a3552b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27552bc, 93 },
    { 0x24f552bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a552c3, 78 },
    { 0xe0a552c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822552cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x136552d0, 61 },
    { 0, 0 },
    { 0x235552d2, 52 },
    { 0, 0 },
    { 0x2c7552d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54552d7, 43 },
    { 0x7ba552d8, 95 },
    { 0xfb2552d8, 43 },
    { 0x720552d7, 34 },
    { 0, 0 },
    { 0x9f1552dc, 55 },
    { 0, 0 },
    { 0xd38552de, 98 },
    { 0xf52552de, 83 },
    { 0xc78552e0, 85 },
    { 0, 0 },
    { 0x695552e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb552e6, 61 },
    { 0x5e9552e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1552eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x942552ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b552f1, 92 },
    { 0, 0 },
    { 0xf7a552f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5552f7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a55308, 61 },
    { 0x1f455309, 93 },
    { 0x57a55309, 43 },
    { 0xdc155308, 75 },
    { 0x3675530c, 87 },
    { 0xd135530d, 61 },
    { 0x285530b, 98 },
    { 0x2f45530f, 98 },
    { 0x12f55310, 23 },
    { 0x6f155311, 61 },
    { 0xc0255312, 85 },
    { 0, 0 },
    { 0xf1c55314, 90 },
    { 0xf5955314, 22 },
    { 0x28f55316, 98 },
    { 0, 0 },
    { 0xa1f55318, 90 },
    { 0xfda55318, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24155321, 33 },
    { 0x57655322, 76 },
    { 0x45b55322, 98 },
    { 0xfea55324, 53 },
    { 0xbf055324, 98 },
    { 0xf6955326, 66 },
    { 0xfda55327, 90 },
    { 0x47b55321, 55 },
    { 0xd455329, 61 },
    { 0xbb35532a, 46 },
    { 0xb255328, 90 },
    { 0x8fc55328, 46 },
    { 0, 0 },
    { 0x7625532e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf155335, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56455339, 34 },
    { 0, 0 },
    { 0xe0f5533b, 95 },
    { 0, 0 },
    { 0x3eb5533d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f155340, 34 },
    { 0xff155341, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7555346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f55349, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1855352, 33 },
    { 0, 0 },
    { 0xcb355354, 23 },
    { 0x4c155354, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73755358, 78 },
    { 0x6ec55359, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad955361, 98 },
    { 0x83f55361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa155368, 33 },
    { 0x12455368, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc55376, 90 },
    { 0xc8555377, 100 },
    { 0x9c655376, 23 },
    { 0x77655378, 98 },
    { 0x1405537a, 87 },
    { 0xe5d5537a, 98 },
    { 0, 0 },
    { 0x2dc5537d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5955381, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c55384, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c55387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe245538b, 65 },
    { 0x3765538c, 43 },
    { 0x9a05538c, 76 },
    { 0x1255538e, 61 },
    { 0xece5538f, 85 },
    { 0xdcc5538f, 76 },
    { 0x32b5538b, 55 },
    { 0, 0 },
    { 0xb3255393, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3155396, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e55399, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7135539d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0553a0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0553a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20553a9, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07553b1, 33 },
    { 0xfd7553b2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5553b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8553b8, 40 },
    { 0x381553b9, 90 },
    { 0, 0 },
    { 0x9553bb, 95 },
    { 0, 0 },
    { 0x176553bd, 91 },
    { 0x945553bd, 34 },
    { 0xc1a553bf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c553c2, 23 },
    { 0x199553c3, 90 },
    { 0xff5553c3, 33 },
    { 0x154553c4, 67 },
    { 0, 0 },
    { 0xabb553c7, 98 },
    { 0, 0 },
    { 0xd32553c9, 61 },
    { 0, 0 },
    { 0xc35553cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a553d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8553d5, 95 },
    { 0, 0 },
    { 0x233553d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee553da, 76 },
    { 0x7f4553da, 95 },
    { 0x4dc553db, 99 },
    { 0xaca553dd, 76 },
    { 0xd3e553de, 33 },
    { 0x50f553dd, 95 },
    { 0xbc4553dc, 92 },
    { 0xedf553e1, 93 },
    { 0x152553e2, 100 },
    { 0x4f9553df, 33 },
    { 0x8fc553e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f553f1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d553f4, 84 },
    { 0x10c553f5, 61 },
    { 0, 0 },
    { 0xfd4553f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b55402, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23655405, 33 },
    { 0, 0 },
    { 0xbd155407, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a5540b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5555412, 33 },
    { 0, 0 },
    { 0x79d55414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5355418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e5541b, 93 },
    { 0, 0 },
    { 0xc2c5541d, 36 },
    { 0, 0 },
    { 0x3be5541f, 23 },
    { 0x63955420, 88 },
    { 0x58655420, 33 },
    { 0, 0 },
    { 0x29955423, 52 },
    { 0x74f55424, 95 },
    { 0xaf355425, 23 },
    { 0xa0b55426, 17 },
    { 0x2fc55425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a5542a, 61 },
    { 0xe0b5542a, 19 },
    { 0xe75542c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd55433, 99 },
    { 0x97255434, 87 },
    { 0x24d55435, 61 },
    { 0, 0 },
    { 0xdfc55437, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e5543a, 93 },
    { 0x9635543b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f855440, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f55445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c55448, 34 },
    { 0xdf255448, 33 },
    { 0x7135544a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe5544e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a55455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6955459, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee55464, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f255467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c5546a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c155473, 33 },
    { 0x43755474, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa5547a, 76 },
    { 0xddb5547a, 90 },
    { 0xc255547c, 34 },
    { 0x6185547d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8355485, 78 },
    { 0, 0 },
    { 0xa9a55487, 61 },
    { 0x77f55488, 23 },
    { 0xf1d55489, 36 },
    { 0xf0f5548a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9735548d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73555495, 90 },
    { 0, 0 },
    { 0xdaa55497, 34 },
    { 0, 0 },
    { 0x47355499, 90 },
    { 0, 0 },
    { 0x4725549b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d554b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3554b8, 33 },
    { 0xcd0554b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219554bd, 55 },
    { 0, 0 },
    { 0x45b554bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb554c4, 23 },
    { 0, 0 },
    { 0x33554c6, 34 },
    { 0xb1b554c7, 33 },
    { 0x8b0554c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f554cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588554d0, 55 },
    { 0x85e554d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8554d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0554dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441554e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e554e6, 52 },
    { 0x1ad554e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568554eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c554f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0554f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00554f7, 61 },
    { 0x61a554f8, 95 },
    { 0x66a554f9, 98 },
    { 0x4a7554f8, 23 },
    { 0xaa4554fa, 61 },
    { 0, 0 },
    { 0x3a5554fd, 46 },
    { 0x173554fe, 98 },
    { 0xf04554fe, 90 },
    { 0, 0 },
    { 0x2cc55501, 38 },
    { 0xa9555502, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f755505, 24 },
    { 0x20c55506, 33 },
    { 0xd2f55507, 34 },
    { 0x62f55505, 33 },
    { 0, 0 },
    { 0x1c45550a, 90 },
    { 0x9a25550b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63655511, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae355518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae5551c, 88 },
    { 0, 0 },
    { 0xceb5551e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd55525, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab55529, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb025552d, 95 },
    { 0x6025552d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53855532, 34 },
    { 0, 0 },
    { 0xe7155534, 34 },
    { 0xca955535, 78 },
    { 0x6bc55536, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb55539, 88 },
    { 0x32c5553a, 90 },
    { 0xac65553a, 90 },
    { 0xbe75553a, 95 },
    { 0x3155553c, 98 },
    { 0xc6d5553a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41855543, 90 },
    { 0x35555544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5855548, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf65554c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f55555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6755555b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a5555f, 90 },
    { 0xbcc55560, 95 },
    { 0x5625555f, 23 },
    { 0xe65555f, 61 },
    { 0x9b155562, 84 },
    { 0xd2a5555f, 90 },
    { 0, 0 },
    { 0x41755566, 34 },
    { 0, 0 },
    { 0x8b555568, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be55570, 95 },
    { 0, 0 },
    { 0x3c655572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455557e, 33 },
    { 0x8a5557f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44455588, 90 },
    { 0x35255588, 23 },
    { 0xb935558a, 93 },
    { 0x5b955589, 98 },
    { 0, 0 },
    { 0x1db5558d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea55596, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e55599, 68 },
    { 0, 0 },
    { 0x8e15559b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b5559e, 95 },
    { 0xaa5559f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761555a3, 61 },
    { 0xb11555a4, 90 },
    { 0x5a0555a3, 53 },
    { 0x2ea555a6, 76 },
    { 0xd07555a5, 90 },
    { 0, 0 },
    { 0x3f8555a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324555ad, 91 },
    { 0, 0 },
    { 0xb9c555af, 34 },
    { 0xd4f555b0, 78 },
    { 0x556555b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb555b9, 52 },
    { 0, 0 },
    { 0xdd555bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f555be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf555c2, 52 },
    { 0x79f555c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6555c9, 90 },
    { 0x3b0555c9, 33 },
    { 0x9c1555ca, 17 },
    { 0x797555cc, 95 },
    { 0, 0 },
    { 0xf13555ce, 61 },
    { 0, 0 },
    { 0x3f1555d0, 76 },
    { 0x815555d1, 90 },
    { 0, 0 },
    { 0xe69555d3, 90 },
    { 0xfd8555d4, 46 },
    { 0x508555d4, 61 },
    { 0x7b6555d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10555da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff555df, 90 },
    { 0x354555df, 90 },
    { 0x11d555e1, 95 },
    { 0xe4e555e2, 90 },
    { 0xff7555e3, 90 },
    { 0x2a4555e4, 98 },
    { 0x9b7555e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa555e8, 95 },
    { 0x3cf555e9, 98 },
    { 0, 0 },
    { 0xf30555eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888555f6, 66 },
    { 0x818555f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d555fb, 23 },
    { 0xd41555fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9555ff, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b55602, 76 },
    { 0xd9d55602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25055606, 98 },
    { 0xe6e55607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865560b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28755610, 23 },
    { 0xfe855611, 33 },
    { 0x6a055612, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa55615, 80 },
    { 0xb8355616, 61 },
    { 0xfaf55617, 90 },
    { 0xc8555618, 90 },
    { 0, 0 },
    { 0x98e5561a, 33 },
    { 0x45a5561b, 23 },
    { 0x44d5561b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc955620, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a45562c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff55631, 69 },
    { 0, 0 },
    { 0xb3655633, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc55637, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e5563a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa5563d, 23 },
    { 0, 0 },
    { 0x5f5563f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc265564a, 68 },
    { 0xaba5564b, 99 },
    { 0xe5f5564c, 90 },
    { 0x9f55564d, 55 },
    { 0xa065564b, 23 },
    { 0xb235564a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b155655, 98 },
    { 0x48655656, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a55565b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96755662, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a55665, 23 },
    { 0x37d55665, 46 },
    { 0xaf555667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e5566f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4255674, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85567b, 66 },
    { 0xee85567c, 55 },
    { 0x7d75567d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc755680, 76 },
    { 0x66055680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6355684, 98 },
    { 0x6f255685, 76 },
    { 0, 0 },
    { 0x93755687, 33 },
    { 0xad855688, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a5568b, 78 },
    { 0, 0 },
    { 0x455568d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c55690, 43 },
    { 0, 0 },
    { 0x20055692, 23 },
    { 0xc1055693, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55698, 34 },
    { 0x7f455698, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f556a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x999556a6, 98 },
    { 0xbdf556a7, 61 },
    { 0xbc7556a8, 55 },
    { 0xcfa556a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea556ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83556b1, 76 },
    { 0xf46556b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d556b5, 66 },
    { 0, 0 },
    { 0x161556b7, 36 },
    { 0, 0 },
    { 0x9d556b9, 61 },
    { 0x933556b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30556bd, 46 },
    { 0xd42556be, 90 },
    { 0x686556bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee556c2, 98 },
    { 0, 0 },
    { 0x6f5556c4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd556cc, 23 },
    { 0xee5556cc, 81 },
    { 0, 0 },
    { 0x665556cf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a556d3, 95 },
    { 0, 0 },
    { 0x7ec556d5, 90 },
    { 0x2da556d6, 84 },
    { 0x791556d7, 90 },
    { 0xdda556d8, 87 },
    { 0x69e556d9, 61 },
    { 0xa4c556da, 90 },
    { 0, 0 },
    { 0xf5c556dc, 23 },
    { 0x5d556dd, 56 },
    { 0x4db556de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5556e3, 100 },
    { 0, 0 },
    { 0x275556e5, 90 },
    { 0x9d4556e5, 43 },
    { 0x488556e7, 52 },
    { 0, 0 },
    { 0x391556e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc556ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43556ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6556f9, 98 },
    { 0x2d1556f9, 98 },
    { 0x9e6556fb, 90 },
    { 0, 0 },
    { 0x28c556fd, 61 },
    { 0x8c1556fd, 61 },
    { 0xde0556ff, 90 },
    { 0x17355700, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4555704, 99 },
    { 0x67355705, 46 },
    { 0x8c155705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f45570b, 66 },
    { 0x3ad5570b, 55 },
    { 0xa565570d, 38 },
    { 0x39b5570e, 93 },
    { 0xe7c5570f, 98 },
    { 0xc0055710, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf55714, 34 },
    { 0x4ed55715, 48 },
    { 0, 0 },
    { 0xea455717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd085571c, 77 },
    { 0x9c85571c, 23 },
    { 0xba85571e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xea455721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f55724, 43 },
    { 0x1d755725, 92 },
    { 0xabc55726, 33 },
    { 0xc0f55727, 66 },
    { 0, 0 },
    { 0xed755729, 17 },
    { 0x94d5572a, 66 },
    { 0, 0 },
    { 0xde35572c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6355731, 33 },
    { 0xd3f55732, 61 },
    { 0x46455733, 46 },
    { 0xbba55734, 52 },
    { 0xc0555735, 98 },
    { 0x5d055735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40055739, 33 },
    { 0, 0 },
    { 0xc6d5573b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5355740, 49 },
    { 0xace55740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf055744, 90 },
    { 0, 0 },
    { 0x9aa55746, 95 },
    { 0xaf455747, 90 },
    { 0, 0 },
    { 0x59b55749, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3155574f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71555752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045575a, 34 },
    { 0xc235575b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5095575e, 33 },
    { 0x4955575f, 90 },
    { 0x71c55760, 95 },
    { 0xf365575e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af55768, 23 },
    { 0, 0 },
    { 0x8665576a, 75 },
    { 0, 0 },
    { 0xfec5576c, 23 },
    { 0, 0 },
    { 0x6ac5576e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d55778, 98 },
    { 0, 0 },
    { 0xeaf5577a, 67 },
    { 0xef35577a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f5577e, 44 },
    { 0x7205577e, 33 },
    { 0xbbb55780, 61 },
    { 0, 0 },
    { 0xd5f55782, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8555785, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb955789, 61 },
    { 0x2175578a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5578d, 23 },
    { 0xf4a5578e, 33 },
    { 0x8a55578f, 91 },
    { 0x9715578f, 23 },
    { 0x73f55790, 33 },
    { 0, 0 },
    { 0x8f955793, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b755796, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe35579c, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee557a7, 88 },
    { 0xbac557a8, 90 },
    { 0, 0 },
    { 0x62d557aa, 95 },
    { 0xca2557ab, 100 },
    { 0x4d6557ac, 61 },
    { 0, 0 },
    { 0xbd5557ae, 98 },
    { 0xda1557ae, 8 },
    { 0x383557b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217557b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6557b9, 65 },
    { 0x7d5557ba, 75 },
    { 0, 0 },
    { 0xe0c557bc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a557bf, 33 },
    { 0x36c557bf, 90 },
    { 0x47b557c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164557c6, 93 },
    { 0, 0 },
    { 0xafe557c8, 76 },
    { 0, 0 },
    { 0xab3557ca, 99 },
    { 0, 0 },
    { 0xd8b557cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14557d2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x900557d5, 91 },
    { 0x68a557d6, 87 },
    { 0x1fd557d7, 98 },
    { 0xaf1557d8, 52 },
    { 0x8f8557d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5557dc, 84 },
    { 0xd6a557dd, 42 },
    { 0xa06557dc, 75 },
    { 0xb7f557df, 81 },
    { 0x36f557e0, 34 },
    { 0x248557df, 23 },
    { 0, 0 },
    { 0x281557e3, 34 },
    { 0, 0 },
    { 0xdee557e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e557ea, 43 },
    { 0, 0 },
    { 0x183557ec, 52 },
    { 0x322557ed, 61 },
    { 0x1a6557ee, 46 },
    { 0, 0 },
    { 0xbb8557f0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a557f9, 33 },
    { 0x368557f9, 43 },
    { 0, 0 },
    { 0xfe6557fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92855807, 33 },
    { 0xecd55808, 55 },
    { 0x60555809, 66 },
    { 0xf365580a, 76 },
    { 0x73555808, 46 },
    { 0x93e55809, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0455811, 85 },
    { 0, 0 },
    { 0xb9a55813, 95 },
    { 0x60155814, 61 },
    { 0xf9655815, 84 },
    { 0, 0 },
    { 0xa8755817, 93 },
    { 0x26455818, 46 },
    { 0x36955819, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f55821, 90 },
    { 0, 0 },
    { 0xd0c55823, 75 },
    { 0xb8a55823, 46 },
    { 0x97855824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76355829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5582f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68155835, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f55839, 77 },
    { 0xab155839, 46 },
    { 0x87e5583b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f5583e, 98 },
    { 0x78c5583f, 90 },
    { 0, 0 },
    { 0x3755841, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f5584b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c5584e, 46 },
    { 0x6c75584f, 23 },
    { 0, 0 },
    { 0x10655851, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955854, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec65585a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c5585e, 52 },
    { 0x3035585f, 90 },
    { 0xa6655860, 33 },
    { 0x89b55861, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a55864, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9925586c, 60 },
    { 0x3105586d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9555875, 23 },
    { 0x93155876, 61 },
    { 0x6fb55877, 98 },
    { 0xdfe55878, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5565587e, 90 },
    { 0, 0 },
    { 0xb8855880, 90 },
    { 0x12a55881, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12755886, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc5588a, 65 },
    { 0xd7f5588b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d55890, 76 },
    { 0, 0 },
    { 0x8c555892, 77 },
    { 0, 0 },
    { 0x50155894, 34 },
    { 0x80255895, 33 },
    { 0x97e55896, 69 },
    { 0xfad55896, 46 },
    { 0xa4f55896, 98 },
    { 0x21b55895, 90 },
    { 0x19f5589a, 65 },
    { 0x5815589a, 90 },
    { 0x4685589b, 55 },
    { 0x9e15589c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96558a6, 51 },
    { 0x959558a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f558aa, 61 },
    { 0x908558ab, 93 },
    { 0, 0 },
    { 0xf0d558ad, 93 },
    { 0, 0 },
    { 0xc50558af, 34 },
    { 0, 0 },
    { 0xddd558b1, 67 },
    { 0x6ce558b2, 98 },
    { 0xee4558b3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648558ba, 95 },
    { 0x53558ba, 85 },
    { 0xc0f558bc, 34 },
    { 0x324558ba, 95 },
    { 0x2bd558be, 34 },
    { 0x9cb558bf, 95 },
    { 0xd1a558bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1558c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52558cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34558d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61558d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x467558d7, 61 },
    { 0, 0 },
    { 0x87f558d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308558e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x595558e6, 98 },
    { 0x548558e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0558ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564558f8, 55 },
    { 0x161558f9, 99 },
    { 0x947558f9, 61 },
    { 0x4558fb, 90 },
    { 0x9a8558fc, 34 },
    { 0x6b9558fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92955900, 98 },
    { 0x89955901, 33 },
    { 0xa3f55902, 90 },
    { 0x87d55900, 88 },
    { 0x6ed55903, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c55907, 90 },
    { 0, 0 },
    { 0x4a855909, 95 },
    { 0, 0 },
    { 0xe465590b, 23 },
    { 0x46c5590c, 64 },
    { 0, 0 },
    { 0x77c5590e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef55917, 75 },
    { 0x32955918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475591b, 95 },
    { 0x64a5591c, 36 },
    { 0xa05591c, 76 },
    { 0xe615591d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd555922, 76 },
    { 0x2e855923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1955926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5025592a, 98 },
    { 0, 0 },
    { 0x70d5592c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36155935, 67 },
    { 0, 0 },
    { 0x41c55937, 38 },
    { 0x3a55938, 61 },
    { 0, 0 },
    { 0x6895593a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d95593f, 76 },
    { 0, 0 },
    { 0x20e55941, 66 },
    { 0xb3355942, 95 },
    { 0, 0 },
    { 0x51855944, 77 },
    { 0, 0 },
    { 0x8d455946, 95 },
    { 0x27455947, 85 },
    { 0x9655948, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b55952, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2955956, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5595d, 100 },
    { 0x4865595d, 55 },
    { 0xac5595f, 95 },
    { 0xeba55960, 55 },
    { 0x9ed55961, 95 },
    { 0x7f655962, 91 },
    { 0x2215595f, 93 },
    { 0xc0955964, 91 },
    { 0xd6355965, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a5596d, 99 },
    { 0xe6a5596d, 33 },
    { 0x48e5596d, 55 },
    { 0, 0 },
    { 0x52a55971, 52 },
    { 0, 0 },
    { 0x31655973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42555976, 85 },
    { 0xe7055977, 23 },
    { 0, 0 },
    { 0x4455979, 98 },
    { 0x4f75597a, 61 },
    { 0x78155979, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b55981, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d555984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90555987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd625598a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5598d, 61 },
    { 0x3e75598e, 95 },
    { 0x3865598e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67455992, 61 },
    { 0, 0 },
    { 0xd8355994, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f655997, 98 },
    { 0x65e55998, 98 },
    { 0x63755999, 90 },
    { 0, 0 },
    { 0x3ab5599b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb085599e, 48 },
    { 0x74f5599f, 90 },
    { 0xd705599f, 84 },
    { 0xa1d5599f, 61 },
    { 0, 0 },
    { 0xbd9559a3, 61 },
    { 0x16559a4, 76 },
    { 0xfec559a5, 98 },
    { 0xfc6559a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263559aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392559af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6559b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877559b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2559bb, 95 },
    { 0, 0 },
    { 0x810559bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64559c6, 95 },
    { 0x67d559c7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a559cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208559d0, 76 },
    { 0x30b559d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78559d4, 95 },
    { 0, 0 },
    { 0x66a559d6, 23 },
    { 0, 0 },
    { 0x75c559d8, 77 },
    { 0x7df559d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720559dc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372559e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c559e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c559f0, 76 },
    { 0x7dd559f1, 95 },
    { 0xe86559f1, 55 },
    { 0xb73559f3, 43 },
    { 0xa00559f4, 33 },
    { 0, 0 },
    { 0x695559f6, 85 },
    { 0xf6f559f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1559fa, 90 },
    { 0x83f559fb, 92 },
    { 0xa46559fc, 23 },
    { 0xa86559fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad55a00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c55a03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c755a06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a55a09, 78 },
    { 0, 0 },
    { 0x78c55a0b, 61 },
    { 0xda155a0b, 17 },
    { 0x61655a0b, 90 },
    { 0xafc55a0b, 46 },
    { 0x72d55a0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa55a12, 23 },
    { 0, 0 },
    { 0x1e755a14, 23 },
    { 0xe4655a15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3355a19, 81 },
    { 0x7c655a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69255a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e55a24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb055a28, 88 },
    { 0xcf355a29, 68 },
    { 0, 0 },
    { 0x6f655a2b, 90 },
    { 0, 0 },
    { 0x60355a2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce55a30, 95 },
    { 0x9f155a30, 96 },
    { 0, 0 },
    { 0xb6655a33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e755a36, 87 },
    { 0xa9055a37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac55a3e, 34 },
    { 0xed55a3e, 90 },
    { 0, 0 },
    { 0x3855a41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a55a46, 9 },
    { 0, 0 },
    { 0xf3355a48, 99 },
    { 0xb7455a49, 98 },
    { 0x6c555a4a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9455a4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf55a52, 33 },
    { 0x96b55a53, 91 },
    { 0, 0 },
    { 0x2655a55, 91 },
    { 0xd655a56, 34 },
    { 0x90355a57, 98 },
    { 0x3bf55a58, 91 },
    { 0x12055a59, 87 },
    { 0xdc955a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c155a60, 90 },
    { 0x34355a61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e55a67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94555a6c, 99 },
    { 0xf5655a6c, 90 },
    { 0xc9255a6e, 61 },
    { 0, 0 },
    { 0x49b55a70, 23 },
    { 0x55e55a71, 34 },
    { 0x3df55a71, 55 },
    { 0xac555a73, 76 },
    { 0x69b55a74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda55a77, 76 },
    { 0x54655a78, 33 },
    { 0x67855a77, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16055a80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa55a85, 61 },
    { 0x5f55a86, 69 },
    { 0, 0 },
    { 0x88a55a88, 61 },
    { 0, 0 },
    { 0x17055a8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb055a8d, 66 },
    { 0xb5c55a8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d955a92, 33 },
    { 0, 0 },
    { 0x355a94, 100 },
    { 0xbf955a94, 95 },
    { 0x1a955a96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a355a99, 65 },
    { 0x47c55a99, 56 },
    { 0, 0 },
    { 0x2cb55a9c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc055aa0, 90 },
    { 0x2da55aa1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a55aa6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c555aa9, 61 },
    { 0, 0 },
    { 0xf8c55aab, 95 },
    { 0x48955aac, 23 },
    { 0, 0 },
    { 0x3fe55aae, 61 },
    { 0x72555aaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64655ab2, 52 },
    { 0, 0 },
    { 0xd7655ab4, 66 },
    { 0, 0 },
    { 0x54655ab6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c55aba, 93 },
    { 0, 0 },
    { 0x99755abc, 91 },
    { 0x70055abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df55ac0, 23 },
    { 0xd4255ac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0855ac6, 98 },
    { 0x90355ac7, 76 },
    { 0, 0 },
    { 0x82f55ac9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c55ad0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36455ad5, 98 },
    { 0xa155ad6, 95 },
    { 0xe3255ad5, 55 },
    { 0x71a55ad7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3255adb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea55ade, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2955ae4, 17 },
    { 0x70655ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97855aeb, 46 },
    { 0, 0 },
    { 0x33b55aed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79455af1, 88 },
    { 0x59d55af1, 56 },
    { 0x88e55af3, 61 },
    { 0xd2655af2, 75 },
    { 0x4c855af5, 87 },
    { 0xff955af3, 95 },
    { 0xa8f55af7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3855afb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44055afe, 88 },
    { 0x9fc55aff, 33 },
    { 0x4555afe, 93 },
    { 0, 0 },
    { 0x2ba55b02, 33 },
    { 0x9b755b03, 91 },
    { 0x5a455b04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce955b08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d55b0f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8055b1b, 28 },
    { 0xed755b1c, 33 },
    { 0xaf055b1c, 90 },
    { 0, 0 },
    { 0x57255b1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35455b23, 61 },
    { 0x92155b24, 99 },
    { 0, 0 },
    { 0xf55b26, 90 },
    { 0, 0 },
    { 0x1d755b28, 90 },
    { 0xb5255b29, 40 },
    { 0x48a55b2a, 23 },
    { 0x32b55b2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57455b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b55b35, 46 },
    { 0x4c455b35, 90 },
    { 0, 0 },
    { 0x49255b38, 95 },
    { 0, 0 },
    { 0x25255b3a, 52 },
    { 0xb9855b3b, 85 },
    { 0xa0555b3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd755b40, 100 },
    { 0, 0 },
    { 0xb355b42, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf55b4a, 61 },
    { 0xb6955b4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e55b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6255b59, 90 },
    { 0x2b055b59, 93 },
    { 0xcb055b5a, 85 },
    { 0, 0 },
    { 0x11b55b5d, 34 },
    { 0x36255b5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41655b61, 43 },
    { 0xf1b55b62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb55b65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a155b68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8355b6b, 61 },
    { 0, 0 },
    { 0xf2355b6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb55b71, 98 },
    { 0xe1f55b71, 23 },
    { 0x71355b72, 76 },
    { 0, 0 },
    { 0x2c355b75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b255b7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e55b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3455b8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb55b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a55b97, 46 },
    { 0x56155b98, 46 },
    { 0x86355b98, 55 },
    { 0, 0 },
    { 0xf3255b9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2255b9e, 93 },
    { 0, 0 },
    { 0xdc655ba0, 23 },
    { 0x7af55ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb55ba5, 19 },
    { 0, 0 },
    { 0xc4e55ba7, 33 },
    { 0xf2655ba7, 38 },
    { 0x85455ba7, 95 },
    { 0, 0 },
    { 0x59355bab, 34 },
    { 0, 0 },
    { 0x23a55bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed755bb2, 80 },
    { 0, 0 },
    { 0x2e955bb4, 90 },
    { 0x3f755bb5, 33 },
    { 0xc4655bb6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x21255bb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea55bbc, 98 },
    { 0x16d55bbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac55bc0, 98 },
    { 0xdf455bc1, 95 },
    { 0xaf555bc1, 33 },
    { 0xf8555bc0, 33 },
    { 0, 0 },
    { 0x79c55bc5, 99 },
    { 0, 0 },
    { 0x8a555bc7, 55 },
    { 0, 0 },
    { 0x34b55bc9, 95 },
    { 0, 0 },
    { 0x9355bcb, 90 },
    { 0x7c055bcc, 33 },
    { 0, 0 },
    { 0x5b255bce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc455bd6, 23 },
    { 0xde055bd7, 81 },
    { 0x6d855bd8, 91 },
    { 0x9bb55bd9, 87 },
    { 0x97755bda, 70 },
    { 0x7f55bdb, 95 },
    { 0x82a55bd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56555be5, 23 },
    { 0xc8d55be5, 66 },
    { 0x61955be7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb255beb, 66 },
    { 0x47955beb, 61 },
    { 0xeee55bec, 33 },
    { 0, 0 },
    { 0x2a155bef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d55bf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x64855bf5, 33 },
    { 0x3f455bf6, 61 },
    { 0x26f55bf5, 33 },
    { 0xbe955bf5, 95 },
    { 0x4d155bf9, 59 },
    { 0x32355bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec55bfd, 61 },
    { 0x2e855bfe, 61 },
    { 0x655bff, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9655c02, 87 },
    { 0xed555c02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59955c06, 88 },
    { 0, 0 },
    { 0xc0b55c08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2355c0b, 43 },
    { 0x49255c0b, 95 },
    { 0x60b55c0b, 33 },
    { 0x66055c0b, 75 },
    { 0x85e55c0c, 52 },
    { 0x2a455c10, 46 },
    { 0x2055c11, 76 },
    { 0xbd055c12, 46 },
    { 0x87f55c13, 90 },
    { 0xe1c55c14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc55c17, 93 },
    { 0xac55c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48955c1d, 93 },
    { 0xb6355c1e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x28955c21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50455c24, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c55c2a, 90 },
    { 0xc6455c2a, 23 },
    { 0, 0 },
    { 0xf2255c2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97555c32, 23 },
    { 0xe5355c33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89355c39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc955c3c, 33 },
    { 0x4cb55c3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e55c45, 90 },
    { 0xf3f55c46, 90 },
    { 0x26755c47, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6355c4d, 33 },
    { 0x7cb55c4e, 90 },
    { 0, 0 },
    { 0xb1555c50, 36 },
    { 0xbaa55c51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55655c62, 23 },
    { 0xcd055c62, 90 },
    { 0xaea55c64, 33 },
    { 0xda455c65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42355c6c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab55c75, 76 },
    { 0xe2555c76, 61 },
    { 0xd9f55c77, 33 },
    { 0x2ef55c78, 91 },
    { 0x20955c79, 55 },
    { 0, 0 },
    { 0x59355c7b, 61 },
    { 0, 0 },
    { 0x67555c7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf55c82, 56 },
    { 0x86455c82, 90 },
    { 0, 0 },
    { 0x37855c85, 91 },
    { 0x6ad55c86, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd55c8c, 95 },
    { 0x3da55c8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc055c90, 93 },
    { 0xd6555c91, 90 },
    { 0, 0 },
    { 0xc9955c93, 75 },
    { 0x5fb55c94, 95 },
    { 0, 0 },
    { 0xef655c96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa55c99, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x85455c9c, 90 },
    { 0xbbc55c9d, 85 },
    { 0x93c55c9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e55ca1, 61 },
    { 0x1b155ca2, 33 },
    { 0x3af55ca3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa855caa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d55cae, 51 },
    { 0x4b855caf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37355cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47255cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e55cbf, 17 },
    { 0, 0 },
    { 0x36e55cc1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55cc6, 90 },
    { 0x4d855cc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f655cd1, 90 },
    { 0xdf55cd2, 90 },
    { 0x84855cd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc55cd7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20755cdb, 33 },
    { 0x9e555cdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f55cec, 78 },
    { 0, 0 },
    { 0x48055cee, 93 },
    { 0xdb255cee, 95 },
    { 0x39455cef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6455cfa, 90 },
    { 0x66a55cfb, 61 },
    { 0xb6955cfa, 98 },
    { 0x3f655cfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26055d01, 61 },
    { 0x6c155d02, 76 },
    { 0xf9a55d02, 83 },
    { 0x5bb55d02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76855d0c, 91 },
    { 0xb2355d0c, 61 },
    { 0x56255d0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf855d11, 90 },
    { 0, 0 },
    { 0x37d55d13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955d1f, 55 },
    { 0, 0 },
    { 0xf6955d21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a55d26, 66 },
    { 0x84a55d26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff55d2b, 33 },
    { 0xb055d2c, 98 },
    { 0, 0 },
    { 0x46455d2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac55d32, 61 },
    { 0xbf055d33, 93 },
    { 0xbac55d34, 61 },
    { 0xb7055d35, 76 },
    { 0, 0 },
    { 0x67255d37, 52 },
    { 0x5df55d38, 55 },
    { 0xd7055d39, 34 },
    { 0, 0 },
    { 0x89455d3b, 90 },
    { 0xbc555d3c, 90 },
    { 0x8a455d3d, 33 },
    { 0xda455d3c, 90 },
    { 0x9f255d3c, 61 },
    { 0x6f855d3b, 94 },
    { 0xc6d55d3c, 76 },
    { 0xd4d55d42, 90 },
    { 0x6d255d43, 61 },
    { 0, 0 },
    { 0x30455d45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2755d4a, 95 },
    { 0x86055d4a, 78 },
    { 0, 0 },
    { 0xd7855d4d, 61 },
    { 0, 0 },
    { 0xde255d4f, 95 },
    { 0xc0f55d4f, 23 },
    { 0xf2e55d50, 23 },
    { 0x9e655d51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce55d55, 36 },
    { 0x5d255d56, 61 },
    { 0x98a55d57, 43 },
    { 0, 0 },
    { 0x8255d59, 95 },
    { 0, 0 },
    { 0x1355d5b, 90 },
    { 0xc9355d5c, 90 },
    { 0x92955d5d, 61 },
    { 0x7b055d5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc055d61, 43 },
    { 0, 0 },
    { 0x3da55d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43855d66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3855d6b, 33 },
    { 0x31f55d6c, 81 },
    { 0x4fe55d6d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d55d70, 43 },
    { 0xa5955d71, 67 },
    { 0x1cf55d72, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32555d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd455d79, 67 },
    { 0xf7255d7a, 33 },
    { 0, 0 },
    { 0x95f55d7c, 77 },
    { 0x8ab55d7d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78055d81, 17 },
    { 0xf2855d82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5555d85, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2355d8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20555d91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c55d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48855d97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb755da4, 90 },
    { 0x2155da5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1255daa, 77 },
    { 0xaec55dab, 33 },
    { 0, 0 },
    { 0xf7a55dad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f255db2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe55dba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0155dc1, 43 },
    { 0x53d55dc2, 90 },
    { 0xe3b55dc1, 61 },
    { 0x12b55dc4, 95 },
    { 0x70f55dc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd55dc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb55dcd, 55 },
    { 0, 0 },
    { 0x70155dcf, 66 },
    { 0x5a755dd0, 76 },
    { 0xf0255dd0, 98 },
    { 0, 0 },
    { 0x26055dd3, 56 },
    { 0xe7555dd4, 93 },
    { 0xe8a55dd3, 34 },
    { 0, 0 },
    { 0x4df55dd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba55ddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec455de0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b455de3, 53 },
    { 0xffe55de4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc55de7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50355df5, 43 },
    { 0x7ef55df6, 77 },
    { 0x29255df6, 23 },
    { 0x83b55df8, 61 },
    { 0x6fe55df7, 98 },
    { 0x96055dfa, 9 },
    { 0, 0 },
    { 0xc855dfc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd55dff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e455e0b, 95 },
    { 0x20355e0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c55e14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d355e18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a55e1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34255e2e, 61 },
    { 0xbf155e2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd55e32, 98 },
    { 0xb6b55e33, 90 },
    { 0xa4355e33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17755e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54355e3b, 61 },
    { 0, 0 },
    { 0xf9555e3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd355e45, 61 },
    { 0xe3255e46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb55e4b, 23 },
    { 0x4d955e4b, 98 },
    { 0x80c55e4d, 38 },
    { 0x55e55e4e, 90 },
    { 0x24555e4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e55e57, 67 },
    { 0x51655e57, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1355e5c, 19 },
    { 0x4e155e5d, 33 },
    { 0x17155e5d, 98 },
    { 0xd6e55e5f, 95 },
    { 0x13255e5f, 95 },
    { 0xc2155e5f, 98 },
    { 0xd3355e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9855e68, 98 },
    { 0xb7755e69, 90 },
    { 0xe0255e68, 55 },
    { 0xdd055e6b, 53 },
    { 0, 0 },
    { 0x7f555e6d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d55e72, 46 },
    { 0x27655e73, 40 },
    { 0xadb55e74, 77 },
    { 0x3ff55e73, 95 },
    { 0x62d55e76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c955e79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34855e80, 76 },
    { 0x8ef55e81, 98 },
    { 0x47155e82, 90 },
    { 0xff155e83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c255e87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee55e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49055e91, 33 },
    { 0x59f55e91, 48 },
    { 0xdb955e92, 75 },
    { 0xef255e94, 95 },
    { 0xe6955e95, 95 },
    { 0x4c355e95, 23 },
    { 0x3d855e95, 99 },
    { 0x28755e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63155ea0, 99 },
    { 0, 0 },
    { 0x16555ea2, 43 },
    { 0, 0 },
    { 0xbbb55ea4, 93 },
    { 0xed355ea5, 43 },
    { 0x87e55ea4, 23 },
    { 0x9c255ea7, 34 },
    { 0x98d55ea7, 52 },
    { 0xb955ea9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f55ead, 98 },
    { 0xd8355eae, 33 },
    { 0x62f55eae, 95 },
    { 0xd4655eb0, 98 },
    { 0xf9155ead, 29 },
    { 0x9a755eb2, 61 },
    { 0, 0 },
    { 0xc7f55eb4, 46 },
    { 0, 0 },
    { 0xee155eb6, 98 },
    { 0xc0d55eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26255eba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd55ec4, 61 },
    { 0x35555ec4, 23 },
    { 0xd8055ec5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a55ecf, 33 },
    { 0, 0 },
    { 0x24c55ed1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c55ed5, 95 },
    { 0x4755ed5, 17 },
    { 0x32a55ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc55edd, 95 },
    { 0, 0 },
    { 0x7a655edf, 90 },
    { 0xcc555ee0, 61 },
    { 0xb5655ee1, 95 },
    { 0x96255ee2, 95 },
    { 0, 0 },
    { 0x43955ee4, 99 },
    { 0x13755ee5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a55ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4555eeb, 95 },
    { 0, 0 },
    { 0xbf55eed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa55ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd855ef7, 55 },
    { 0xf8655ef8, 65 },
    { 0x35a55ef7, 38 },
    { 0xcf355efa, 99 },
    { 0, 0 },
    { 0x5d55efc, 33 },
    { 0x8cc55efd, 33 },
    { 0xfd755efe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f55f06, 100 },
    { 0xe0955f06, 90 },
    { 0x3e055f08, 55 },
    { 0xc8d55f09, 23 },
    { 0x3ee55f07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955f10, 99 },
    { 0x92055f11, 33 },
    { 0, 0 },
    { 0xdaa55f13, 88 },
    { 0xe1555f13, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb55f17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad55f1a, 23 },
    { 0xbe055f1a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b855f21, 84 },
    { 0x8f655f22, 78 },
    { 0xd6855f22, 67 },
    { 0xb8f55f23, 91 },
    { 0x35855f25, 95 },
    { 0, 0 },
    { 0x57c55f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e55f2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f55f31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f55f3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b55f41, 91 },
    { 0, 0 },
    { 0xd8655f43, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4755f4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa55f4d, 33 },
    { 0xc6055f4d, 43 },
    { 0x26355f4f, 48 },
    { 0x10255f4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72455f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce55f56, 43 },
    { 0x9055f57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72255f5d, 98 },
    { 0xca55f5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455f63, 87 },
    { 0, 0 },
    { 0xa9255f65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68855f69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb955f6d, 61 },
    { 0xd5455f6e, 95 },
    { 0xb7955f6f, 84 },
    { 0, 0 },
    { 0x1955f71, 48 },
    { 0x4d955f72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b455f75, 95 },
    { 0x22455f76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15755f7c, 19 },
    { 0xa8c55f7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a355f81, 61 },
    { 0x24055f82, 61 },
    { 0x62855f83, 95 },
    { 0xca455f82, 43 },
    { 0xd1455f85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23755f89, 61 },
    { 0, 0 },
    { 0x1ea55f8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f55f8e, 61 },
    { 0x77055f8f, 33 },
    { 0, 0 },
    { 0xf0e55f91, 23 },
    { 0xc1455f91, 85 },
    { 0, 0 },
    { 0x1ad55f94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce755f97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda55f9a, 18 },
    { 0, 0 },
    { 0xb4055f9c, 23 },
    { 0, 0 },
    { 0x78855f9e, 76 },
    { 0x1a355f9f, 69 },
    { 0x4b355fa0, 93 },
    { 0, 0 },
    { 0x41755fa2, 36 },
    { 0x59b55fa3, 95 },
    { 0xa2b55fa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1555faf, 23 },
    { 0xaf555fb0, 33 },
    { 0x52f55faf, 75 },
    { 0xf8255fb2, 90 },
    { 0xbdb55fb3, 95 },
    { 0xdf055fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d55fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7655fbf, 23 },
    { 0xa5955fc0, 95 },
    { 0x51f55fc0, 92 },
    { 0, 0 },
    { 0x3b555fc3, 46 },
    { 0x5ce55fc4, 78 },
    { 0, 0 },
    { 0x83455fc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81755fcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b55fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb055fe3, 33 },
    { 0xa9555fe4, 90 },
    { 0xbdb55fe4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63155fee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16755ff4, 66 },
    { 0xac355ff4, 90 },
    { 0, 0 },
    { 0xfc455ff7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab55ffc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4756001, 23 },
    { 0x8ff56001, 100 },
    { 0x2356002, 33 },
    { 0xb5656004, 84 },
    { 0x73656005, 34 },
    { 0, 0 },
    { 0xbc456007, 98 },
    { 0xc0856007, 34 },
    { 0xfb756009, 93 },
    { 0xc2e5600a, 76 },
    { 0xc135600b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6775600e, 33 },
    { 0xa645600e, 33 },
    { 0xd6256010, 90 },
    { 0, 0 },
    { 0x81156012, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c56016, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb495601c, 90 },
    { 0xdd15601d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58356020, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d56027, 61 },
    { 0x69556028, 2 },
    { 0x26656029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925602c, 76 },
    { 0, 0 },
    { 0xa445602e, 23 },
    { 0x1865602f, 52 },
    { 0xf1556030, 90 },
    { 0x5465602e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda656038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3956041, 23 },
    { 0x31256041, 90 },
    { 0x1d056043, 17 },
    { 0xb2056042, 90 },
    { 0x41c56045, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e256049, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c15604c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc56055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e56060, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb556063, 61 },
    { 0x32256064, 87 },
    { 0xf4056065, 90 },
    { 0xbe656065, 24 },
    { 0x85556067, 61 },
    { 0x6db56068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe515606c, 23 },
    { 0x8df5606d, 61 },
    { 0, 0 },
    { 0x61b5606f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x94556072, 90 },
    { 0, 0 },
    { 0xa7956074, 93 },
    { 0xaf556074, 91 },
    { 0x2fe56076, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5607b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5607f, 93 },
    { 0, 0 },
    { 0x68956081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5956084, 33 },
    { 0xa9256085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f56089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b5608e, 61 },
    { 0xf35608e, 33 },
    { 0, 0 },
    { 0xfd856091, 55 },
    { 0x97456092, 61 },
    { 0, 0 },
    { 0xd1656094, 90 },
    { 0x95956095, 90 },
    { 0x9c656096, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb5609f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64560a3, 23 },
    { 0xf86560a4, 38 },
    { 0xf23560a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67560ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x77560af, 66 },
    { 0x93b560af, 85 },
    { 0xb8560b1, 90 },
    { 0x803560af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f560b6, 23 },
    { 0x9b2560b7, 43 },
    { 0xb34560b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e560bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5560c5, 67 },
    { 0x141560c6, 61 },
    { 0x584560c7, 33 },
    { 0xf95560c8, 66 },
    { 0x7af560c5, 69 },
    { 0xeda560ca, 61 },
    { 0xaed560c9, 17 },
    { 0, 0 },
    { 0x8d1560cd, 23 },
    { 0x865560ce, 55 },
    { 0x7cb560cf, 98 },
    { 0x916560ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22560da, 66 },
    { 0xf56560db, 61 },
    { 0x9af560da, 66 },
    { 0x66c560dc, 90 },
    { 0xd0560de, 23 },
    { 0xd29560df, 78 },
    { 0xb42560e0, 98 },
    { 0x2c8560e1, 23 },
    { 0x96f560e2, 23 },
    { 0x1f9560e1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19560e7, 61 },
    { 0x165560e7, 95 },
    { 0x8c2560e7, 84 },
    { 0, 0 },
    { 0x30b560eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x936560ee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7560f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c560fb, 84 },
    { 0x80c560fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a56100, 100 },
    { 0xb656100, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc956105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c556109, 33 },
    { 0, 0 },
    { 0x2235610b, 85 },
    { 0xc85610b, 23 },
    { 0x4c25610d, 65 },
    { 0, 0 },
    { 0x5605610f, 98 },
    { 0xf1056110, 90 },
    { 0, 0 },
    { 0xea56112, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8356117, 55 },
    { 0xa5c56118, 98 },
    { 0xe7856119, 23 },
    { 0xfb856119, 95 },
    { 0x9555611b, 33 },
    { 0xe705611c, 43 },
    { 0, 0 },
    { 0xa665611e, 61 },
    { 0x2fb5611f, 23 },
    { 0xbb356120, 52 },
    { 0x4915611e, 61 },
    { 0x93656122, 67 },
    { 0x52356122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f456126, 76 },
    { 0xab56126, 99 },
    { 0x56b56128, 85 },
    { 0xb2756128, 90 },
    { 0, 0 },
    { 0x37b5612b, 98 },
    { 0x7b15612c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5612f, 61 },
    { 0, 0 },
    { 0x3f456131, 91 },
    { 0, 0 },
    { 0xb3e56133, 23 },
    { 0x8a256133, 64 },
    { 0, 0 },
    { 0x45756136, 66 },
    { 0, 0 },
    { 0xad456138, 23 },
    { 0x74c56138, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f05613d, 76 },
    { 0x52c5613d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f56144, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93256147, 49 },
    { 0x5cf56148, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a75614c, 55 },
    { 0x3665614d, 91 },
    { 0xb15614e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60956154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd656159, 33 },
    { 0, 0 },
    { 0xff65615b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55615f, 90 },
    { 0, 0 },
    { 0xdac56161, 90 },
    { 0, 0 },
    { 0xe3656163, 76 },
    { 0x35f56164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12156167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615616d, 55 },
    { 0xe3b5616d, 95 },
    { 0x2015616d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd56172, 43 },
    { 0, 0 },
    { 0xb7156174, 17 },
    { 0x92a56175, 33 },
    { 0x35656176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9556179, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2056180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f56189, 90 },
    { 0xd9f5618a, 61 },
    { 0x2ef5618b, 93 },
    { 0, 0 },
    { 0x6915618d, 93 },
    { 0x4f45618e, 81 },
    { 0x57b5618e, 61 },
    { 0x33156190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e56196, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe5619a, 95 },
    { 0xcf5619b, 88 },
    { 0x24d5619a, 61 },
    { 0x4125619d, 55 },
    { 0x7105619a, 95 },
    { 0xd9f5619f, 84 },
    { 0, 0 },
    { 0x7e7561a1, 52 },
    { 0, 0 },
    { 0x3c1561a3, 61 },
    { 0, 0 },
    { 0x20561a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5561a9, 95 },
    { 0x9c8561a9, 90 },
    { 0xef5561ab, 34 },
    { 0x3e561ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af561b3, 68 },
    { 0, 0 },
    { 0x401561b5, 34 },
    { 0x859561b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23561ba, 76 },
    { 0, 0 },
    { 0x489561bc, 91 },
    { 0xf07561bc, 93 },
    { 0x9e8561bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566561c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a561c5, 33 },
    { 0, 0 },
    { 0x398561c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367561d4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3561d8, 76 },
    { 0, 0 },
    { 0x915561da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75561e0, 75 },
    { 0x624561e0, 90 },
    { 0x6ab561e1, 76 },
    { 0, 0 },
    { 0x413561e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x337561e7, 83 },
    { 0xe13561e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2561ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf561ef, 98 },
    { 0, 0 },
    { 0x89b561f1, 61 },
    { 0x35d561f2, 68 },
    { 0x673561f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9561f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f561fa, 98 },
    { 0x866561fb, 33 },
    { 0x318561fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c56201, 98 },
    { 0x76d56202, 33 },
    { 0x95056202, 55 },
    { 0x8f456204, 84 },
    { 0, 0 },
    { 0x8b356206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985620d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f56210, 23 },
    { 0, 0 },
    { 0x2fd56212, 23 },
    { 0xc856212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f5621f, 90 },
    { 0xd0156220, 98 },
    { 0x2dd56221, 65 },
    { 0x4b656221, 43 },
    { 0x4b656223, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83256226, 98 },
    { 0, 0 },
    { 0xf0656228, 87 },
    { 0x9756229, 61 },
    { 0x1d656229, 75 },
    { 0x4c65622b, 95 },
    { 0xe0456229, 33 },
    { 0xa4056228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55656233, 95 },
    { 0xa6056234, 52 },
    { 0x36f56235, 95 },
    { 0xdd56235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b456239, 33 },
    { 0, 0 },
    { 0xcf15623b, 85 },
    { 0, 0 },
    { 0xe835623d, 43 },
    { 0x7e35623e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3256241, 61 },
    { 0x7e656242, 33 },
    { 0, 0 },
    { 0xa9156244, 23 },
    { 0x30b56245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87456248, 90 },
    { 0xd4c56249, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91056250, 69 },
    { 0x84c56251, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a56256, 61 },
    { 0xb7156256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de56260, 81 },
    { 0, 0 },
    { 0x72856262, 100 },
    { 0xa3456263, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91156269, 23 },
    { 0xc615626a, 23 },
    { 0xb25626a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc635626f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d56272, 95 },
    { 0xe2056272, 34 },
    { 0x33c56272, 95 },
    { 0x58e56275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d56278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa525627f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d56285, 69 },
    { 0x5356286, 90 },
    { 0, 0 },
    { 0xff456288, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x645628b, 90 },
    { 0, 0 },
    { 0x6ce5628d, 93 },
    { 0x6885628e, 23 },
    { 0x69f5628f, 61 },
    { 0xdf05628f, 43 },
    { 0x2c85628d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81156297, 91 },
    { 0xc3d56297, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf175629f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f562a3, 81 },
    { 0x98562a4, 59 },
    { 0x4ec562a5, 80 },
    { 0x4e7562a5, 95 },
    { 0xb88562a7, 33 },
    { 0, 0 },
    { 0x26d562a9, 100 },
    { 0, 0 },
    { 0xdc2562ab, 81 },
    { 0xf50562ab, 90 },
    { 0x919562ad, 43 },
    { 0, 0 },
    { 0x1ea562af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a562b3, 47 },
    { 0x92b562b4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5562b9, 34 },
    { 0x348562b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3562bd, 81 },
    { 0, 0 },
    { 0x3f8562bf, 13 },
    { 0x41a562c0, 76 },
    { 0x35562c1, 99 },
    { 0x128562c2, 99 },
    { 0x3b0562c3, 68 },
    { 0xb06562c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a562c9, 78 },
    { 0x872562c9, 85 },
    { 0, 0 },
    { 0xc87562cc, 59 },
    { 0x5ac562cc, 66 },
    { 0x308562ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a562d2, 95 },
    { 0, 0 },
    { 0x35d562d4, 98 },
    { 0x456562d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4562dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776562e1, 42 },
    { 0x377562e2, 98 },
    { 0xc4e562e2, 88 },
    { 0xc4a562e2, 100 },
    { 0xf21562e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753562e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e562ee, 61 },
    { 0x863562ef, 33 },
    { 0, 0 },
    { 0x75b562f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff562f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18562f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6562fc, 87 },
    { 0x66f562fd, 85 },
    { 0xff8562fe, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5056301, 66 },
    { 0xc6056301, 90 },
    { 0x1b656303, 65 },
    { 0, 0 },
    { 0x23a56305, 46 },
    { 0, 0 },
    { 0xfe56307, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c05630a, 78 },
    { 0xc995630b, 90 },
    { 0x5a65630a, 33 },
    { 0xc745630b, 90 },
    { 0x885630e, 33 },
    { 0x4605630f, 17 },
    { 0, 0 },
    { 0xa8856311, 55 },
    { 0x22f56312, 99 },
    { 0x16956313, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5175631a, 100 },
    { 0, 0 },
    { 0xbab5631c, 99 },
    { 0, 0 },
    { 0xe3e5631e, 84 },
    { 0x5f65631f, 81 },
    { 0, 0 },
    { 0xd5156321, 17 },
    { 0xbc256322, 23 },
    { 0xb3e56323, 33 },
    { 0, 0 },
    { 0x2a656325, 97 },
    { 0x17156326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f5632b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05632e, 66 },
    { 0x2915632f, 87 },
    { 0x24156330, 88 },
    { 0x27e56331, 78 },
    { 0x3fd56332, 33 },
    { 0x4f356331, 23 },
    { 0, 0 },
    { 0xda156335, 90 },
    { 0x4e456335, 91 },
    { 0x19c56337, 100 },
    { 0x26f56338, 90 },
    { 0x9f56339, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf165633d, 23 },
    { 0, 0 },
    { 0xbe85633f, 33 },
    { 0xfa956340, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63156344, 90 },
    { 0, 0 },
    { 0x74f56346, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9655634a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a55634d, 23 },
    { 0x1c35634d, 95 },
    { 0xa25634f, 69 },
    { 0x4cf5634e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd56353, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2856357, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f35635a, 33 },
    { 0, 0 },
    { 0xf915635c, 90 },
    { 0x6535635d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a56360, 61 },
    { 0, 0 },
    { 0x47656362, 23 },
    { 0x9f56363, 90 },
    { 0x4ba56364, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff56367, 100 },
    { 0x2d056368, 76 },
    { 0xee356369, 66 },
    { 0xa6d56368, 51 },
    { 0xf225636b, 95 },
    { 0x99f5636a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec5636f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74156377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc435637a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb56381, 23 },
    { 0xf5556382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed56385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e256388, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d25638b, 33 },
    { 0xd445638c, 43 },
    { 0x2425638d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf256395, 99 },
    { 0x62956396, 95 },
    { 0x5b256397, 85 },
    { 0x6b856395, 90 },
    { 0, 0 },
    { 0xe5e5639a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb45639d, 98 },
    { 0x6915639e, 91 },
    { 0x4245639d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x997563a2, 36 },
    { 0x5bb563a3, 99 },
    { 0x50563a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294563af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35563b3, 76 },
    { 0, 0 },
    { 0x616563b5, 95 },
    { 0xa06563b6, 23 },
    { 0, 0 },
    { 0x43b563b8, 33 },
    { 0x668563b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35563bf, 43 },
    { 0xa2563c0, 78 },
    { 0xb91563c1, 33 },
    { 0xf24563c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x232563c5, 95 },
    { 0xa28563c5, 75 },
    { 0xa14563c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395563d2, 23 },
    { 0xbda563d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df563d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60563dc, 61 },
    { 0x88e563dd, 46 },
    { 0x282563dd, 46 },
    { 0x4d6563de, 61 },
    { 0xd25563e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1563e3, 90 },
    { 0, 0 },
    { 0x809563e5, 34 },
    { 0x535563e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae563eb, 33 },
    { 0, 0 },
    { 0x7a563ed, 95 },
    { 0xa7563ee, 95 },
    { 0x9e563ef, 76 },
    { 0x488563f0, 61 },
    { 0xe8d563f0, 95 },
    { 0xbd1563f2, 100 },
    { 0x55b563f3, 87 },
    { 0xe8a563f3, 65 },
    { 0xff4563f5, 100 },
    { 0xb0f563f4, 18 },
    { 0xb52563f6, 99 },
    { 0x79c563f8, 98 },
    { 0, 0 },
    { 0xc1c563fa, 33 },
    { 0xfbb563fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911563ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa656403, 100 },
    { 0xfd056404, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2156409, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8015640d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89956411, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f956414, 87 },
    { 0x8056415, 98 },
    { 0x60e56416, 98 },
    { 0xf2056417, 61 },
    { 0, 0 },
    { 0x60b56419, 90 },
    { 0xcbd5641a, 98 },
    { 0x4b956419, 98 },
    { 0x58f5641a, 34 },
    { 0x9575641c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f56420, 61 },
    { 0x9c356420, 34 },
    { 0xa1c56420, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c5642d, 91 },
    { 0xc125642e, 67 },
    { 0xc075642f, 90 },
    { 0xd6756430, 61 },
    { 0x26e5642d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a56435, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8165643e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee56443, 96 },
    { 0xe8f56444, 68 },
    { 0, 0 },
    { 0x71b56446, 76 },
    { 0xfd956447, 55 },
    { 0xb9256448, 33 },
    { 0xdff56447, 90 },
    { 0, 0 },
    { 0x6235644b, 33 },
    { 0x8475644c, 34 },
    { 0xc715644c, 84 },
    { 0xee95644e, 55 },
    { 0x3335644b, 23 },
    { 0xf0656450, 99 },
    { 0xbc056451, 98 },
    { 0xdd56451, 33 },
    { 0xb0d56453, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8456459, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9735645c, 61 },
    { 0xe9e5645d, 61 },
    { 0x4045645c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae56462, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7456465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8856468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db5646b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d5646e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2356471, 46 },
    { 0xc1356472, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37856478, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca5647f, 76 },
    { 0, 0 },
    { 0xe8256481, 23 },
    { 0, 0 },
    { 0x5c156483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d156487, 17 },
    { 0x6f656487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa825648b, 34 },
    { 0xbca5648c, 61 },
    { 0xd9d5648d, 17 },
    { 0xe1a5648e, 76 },
    { 0, 0 },
    { 0xd5956490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e5649c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd564a1, 90 },
    { 0xd63564a2, 61 },
    { 0x441564a3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x867564a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131564b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08564b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x677564bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8564be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156564c2, 90 },
    { 0, 0 },
    { 0x548564c4, 95 },
    { 0, 0 },
    { 0x7ee564c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb564ca, 23 },
    { 0x17f564cb, 81 },
    { 0x3ed564cc, 61 },
    { 0xfc3564cd, 98 },
    { 0x424564ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896564d9, 61 },
    { 0, 0 },
    { 0x4d4564db, 100 },
    { 0, 0 },
    { 0x370564dd, 100 },
    { 0, 0 },
    { 0x165564df, 95 },
    { 0, 0 },
    { 0xae8564e1, 65 },
    { 0xac5564e2, 98 },
    { 0, 0 },
    { 0x21b564e4, 98 },
    { 0x584564e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e564e9, 98 },
    { 0x394564ea, 85 },
    { 0xed9564ea, 90 },
    { 0x8ee564eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558564fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66356500, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72056504, 33 },
    { 0, 0 },
    { 0x2b56506, 90 },
    { 0x5b56506, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9295650a, 23 },
    { 0, 0 },
    { 0xb3d5650c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b56510, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3756513, 61 },
    { 0x1b256514, 23 },
    { 0xe2756515, 61 },
    { 0xa5056516, 91 },
    { 0x7a056515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae5651b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe555651e, 55 },
    { 0, 0 },
    { 0x3d356520, 91 },
    { 0, 0 },
    { 0x6c056522, 34 },
    { 0, 0 },
    { 0x20756524, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba56528, 98 },
    { 0xc1656528, 80 },
    { 0x2d456529, 55 },
    { 0, 0 },
    { 0x8d15652c, 23 },
    { 0x8425652d, 76 },
    { 0xee65652c, 90 },
    { 0, 0 },
    { 0xcd256530, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e56535, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd56539, 91 },
    { 0x7ea5653a, 91 },
    { 0x6375653b, 61 },
    { 0x8f65653c, 61 },
    { 0x3575653d, 93 },
    { 0x3565653e, 90 },
    { 0xb415653f, 23 },
    { 0x9d5653f, 34 },
    { 0xbb056541, 33 },
    { 0xec35653b, 101 },
    { 0, 0 },
    { 0x7d156544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff56549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a5654d, 46 },
    { 0, 0 },
    { 0x8d95654f, 34 },
    { 0xc9556550, 90 },
    { 0x2d356550, 17 },
    { 0, 0 },
    { 0xac956553, 100 },
    { 0xa2c56554, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee56557, 46 },
    { 0xaa56557, 87 },
    { 0x21f56559, 33 },
    { 0x3a35655a, 61 },
    { 0, 0 },
    { 0x1075655c, 76 },
    { 0x8585655d, 69 },
    { 0x7ea5655c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a56562, 76 },
    { 0x42956563, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a56568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4075656d, 87 },
    { 0xedb5656d, 66 },
    { 0x6915656f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67956573, 61 },
    { 0xc7056574, 55 },
    { 0x27356574, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15657d, 34 },
    { 0x1f5657e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf856583, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4856586, 90 },
    { 0x1e556586, 34 },
    { 0, 0 },
    { 0xc056589, 33 },
    { 0, 0 },
    { 0x33a5658b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a5658e, 61 },
    { 0x5cc5658f, 91 },
    { 0, 0 },
    { 0x1d456591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c56596, 98 },
    { 0x74456596, 99 },
    { 0, 0 },
    { 0x6de56599, 61 },
    { 0, 0 },
    { 0xc6a5659b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8565a1, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136565a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238565ac, 95 },
    { 0x9d565ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301565b6, 55 },
    { 0, 0 },
    { 0xac4565b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d565bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da565bf, 33 },
    { 0x8cb565c0, 90 },
    { 0, 0 },
    { 0xa3e565c2, 61 },
    { 0, 0 },
    { 0xce8565c4, 85 },
    { 0xd3b565c5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda565c8, 88 },
    { 0x85565c8, 93 },
    { 0x29565c9, 95 },
    { 0, 0 },
    { 0x294565cc, 61 },
    { 0x174565cd, 77 },
    { 0xb8c565cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde565d3, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba565e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80565e6, 34 },
    { 0xdfb565e6, 33 },
    { 0x3f7565e7, 76 },
    { 0x3ac565e9, 33 },
    { 0x24b565ea, 91 },
    { 0x96d565e7, 65 },
    { 0xd97565ec, 61 },
    { 0, 0 },
    { 0x672565ee, 68 },
    { 0xf2d565ef, 90 },
    { 0, 0 },
    { 0xd0d565f1, 43 },
    { 0xcd5565f2, 91 },
    { 0xd7d565f3, 98 },
    { 0x158565f3, 96 },
    { 0xce4565f5, 37 },
    { 0xa8f565f6, 33 },
    { 0, 0 },
    { 0xfec565f8, 33 },
    { 0x756565f9, 33 },
    { 0x8a0565fa, 95 },
    { 0, 0 },
    { 0xa56565fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c356606, 61 },
    { 0, 0 },
    { 0xf6056608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b5660d, 100 },
    { 0, 0 },
    { 0x94a5660f, 81 },
    { 0xa055660f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd456614, 66 },
    { 0x2e56615, 33 },
    { 0x5de56616, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d05661c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe56622, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf956626, 98 },
    { 0xc9256627, 33 },
    { 0xee856628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f25662d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1556631, 23 },
    { 0x7fe56632, 99 },
    { 0, 0 },
    { 0x72356634, 76 },
    { 0xfe356634, 33 },
    { 0xd1e56636, 61 },
    { 0x90a56635, 100 },
    { 0, 0 },
    { 0xc3a56639, 90 },
    { 0x4f95663a, 23 },
    { 0x1f85663a, 87 },
    { 0xd8a56639, 61 },
    { 0x3e25663d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a56640, 61 },
    { 0x52656641, 93 },
    { 0x75f56642, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98056646, 61 },
    { 0xc0c56646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7445664d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd456651, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2356654, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6556658, 43 },
    { 0x4556659, 98 },
    { 0xd7b5665a, 66 },
    { 0xf275665b, 76 },
    { 0x4b55665c, 99 },
    { 0, 0 },
    { 0xdee5665e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f56662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e356665, 19 },
    { 0x7a456666, 52 },
    { 0xdda56666, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e65666a, 33 },
    { 0x7a65666b, 55 },
    { 0x76b5666c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba05666f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d56672, 90 },
    { 0x96656672, 23 },
    { 0x92e56674, 81 },
    { 0x15256675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e256678, 52 },
    { 0x7ae56678, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c5667c, 61 },
    { 0x1015667c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a56684, 74 },
    { 0xc856685, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a5668c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c56690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13566a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d566a5, 33 },
    { 0x859566a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7566aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x968566ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff566b0, 23 },
    { 0x8b7566b1, 87 },
    { 0xac5566b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4566b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77566bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1566c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6566c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d566c6, 99 },
    { 0x793566c6, 46 },
    { 0x3d8566c7, 87 },
    { 0, 0 },
    { 0x4d8566ca, 83 },
    { 0x7f4566ca, 34 },
    { 0, 0 },
    { 0x456566cd, 93 },
    { 0x7f6566ce, 33 },
    { 0, 0 },
    { 0x352566d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3566d7, 90 },
    { 0x44566d7, 98 },
    { 0x8b3566d8, 84 },
    { 0x436566da, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92566e2, 66 },
    { 0x51c566e3, 93 },
    { 0x32f566e4, 66 },
    { 0xcca566e3, 33 },
    { 0xd95566e3, 33 },
    { 0x9f3566e7, 93 },
    { 0xfe3566e7, 100 },
    { 0, 0 },
    { 0x822566ea, 90 },
    { 0x684566eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44566f3, 23 },
    { 0x596566f4, 61 },
    { 0, 0 },
    { 0xadf566f6, 87 },
    { 0, 0 },
    { 0x519566f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4566fe, 76 },
    { 0x104566ff, 88 },
    { 0x6d756700, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0056706, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5670b, 90 },
    { 0xe265670c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc235670f, 100 },
    { 0, 0 },
    { 0xc4356711, 100 },
    { 0xfe356711, 61 },
    { 0x92356712, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b56718, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c65671c, 95 },
    { 0, 0 },
    { 0xe695671e, 33 },
    { 0xaba5671f, 34 },
    { 0x63156720, 98 },
    { 0xe6f56720, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a656724, 65 },
    { 0, 0 },
    { 0xa2456726, 90 },
    { 0x36f56727, 100 },
    { 0, 0 },
    { 0xe0556729, 55 },
    { 0xd1656729, 65 },
    { 0xe825672b, 90 },
    { 0x6e55672c, 46 },
    { 0x6735672c, 98 },
    { 0x3dd5672c, 61 },
    { 0xf5556729, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c56735, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1685673b, 91 },
    { 0xbcf5673b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9475673f, 81 },
    { 0x9355673f, 34 },
    { 0, 0 },
    { 0xc4f56742, 46 },
    { 0x3f456743, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a956749, 76 },
    { 0x1e656749, 33 },
    { 0x6985674b, 55 },
    { 0, 0 },
    { 0xaf85674d, 33 },
    { 0x8dd5674d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22056752, 90 },
    { 0, 0 },
    { 0xebc56754, 93 },
    { 0, 0 },
    { 0xe6a56756, 76 },
    { 0, 0 },
    { 0xa8856758, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c85675b, 95 },
    { 0x9dd5675c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b56763, 100 },
    { 0, 0 },
    { 0x89356765, 93 },
    { 0xf5e56766, 23 },
    { 0x70e56766, 34 },
    { 0x5c956767, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3625676b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd085676f, 33 },
    { 0x3965676f, 66 },
    { 0x3dc56770, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40156774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7156779, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5065677f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed56782, 61 },
    { 0xf7b56783, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc95678a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f05678e, 52 },
    { 0, 0 },
    { 0xfde56790, 76 },
    { 0x37656790, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e56795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce456799, 90 },
    { 0, 0 },
    { 0xec15679b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd735679f, 91 },
    { 0x37f567a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x646567a3, 23 },
    { 0xf74567a4, 99 },
    { 0, 0 },
    { 0xc0e567a6, 69 },
    { 0, 0 },
    { 0x194567a8, 90 },
    { 0, 0 },
    { 0x59c567aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd567ae, 33 },
    { 0x6a7567af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff567b4, 98 },
    { 0x4bd567b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd567b9, 55 },
    { 0xd4c567ba, 46 },
    { 0x12b567bb, 96 },
    { 0x640567bc, 98 },
    { 0xd34567bd, 46 },
    { 0xc20567be, 90 },
    { 0x1d8567bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab567c6, 64 },
    { 0, 0 },
    { 0x201567c8, 90 },
    { 0, 0 },
    { 0x143567ca, 23 },
    { 0xb44567ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577567d0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa567d6, 52 },
    { 0, 0 },
    { 0xcda567d8, 95 },
    { 0xacd567d9, 84 },
    { 0x39d567da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db567df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x346567e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70567e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda567e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7567eb, 88 },
    { 0x176567ec, 75 },
    { 0xcc9567ed, 98 },
    { 0xfca567ed, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2567f1, 77 },
    { 0xd70567f2, 90 },
    { 0x7fe567f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e567f6, 76 },
    { 0xd67567f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10567fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2556800, 95 },
    { 0x39e56800, 93 },
    { 0, 0 },
    { 0xa4b56803, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f56806, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a5680a, 90 },
    { 0x88a5680b, 76 },
    { 0, 0 },
    { 0x4295680d, 95 },
    { 0x83c5680e, 90 },
    { 0, 0 },
    { 0x87656810, 84 },
    { 0x4e656811, 53 },
    { 0x22256811, 61 },
    { 0xc6756813, 34 },
    { 0xdee56814, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af56817, 40 },
    { 0x92756818, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5681b, 93 },
    { 0x20e5681c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc56821, 55 },
    { 0x60956822, 33 },
    { 0x4e556822, 90 },
    { 0x4b56822, 76 },
    { 0x39e56822, 67 },
    { 0xee956826, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x90256829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa935682c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72256833, 76 },
    { 0x59856833, 95 },
    { 0, 0 },
    { 0x12e56836, 46 },
    { 0, 0 },
    { 0x3bd56838, 90 },
    { 0, 0 },
    { 0x7e5683a, 90 },
    { 0xc4e5683b, 98 },
    { 0xd855683c, 99 },
    { 0x2565683c, 95 },
    { 0x9d95683c, 100 },
    { 0x73b5683f, 84 },
    { 0, 0 },
    { 0xf4856841, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b56844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52456848, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585684d, 33 },
    { 0, 0 },
    { 0x2505684f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce256854, 66 },
    { 0x94a56855, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44556859, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a56861, 93 },
    { 0, 0 },
    { 0xdc256863, 87 },
    { 0x20756864, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff56868, 88 },
    { 0x6056869, 76 },
    { 0, 0 },
    { 0x9245686b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16156871, 23 },
    { 0x1c856871, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xce056875, 68 },
    { 0xbef56876, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5687b, 95 },
    { 0xa7d5687c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3856882, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e256885, 55 },
    { 0x2c456886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f5688c, 66 },
    { 0, 0 },
    { 0x4695688e, 98 },
    { 0, 0 },
    { 0x30356890, 33 },
    { 0x3a956891, 98 },
    { 0x81c56891, 90 },
    { 0x3ec56893, 90 },
    { 0x78056894, 55 },
    { 0x5d656892, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05689a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b05689d, 51 },
    { 0, 0 },
    { 0xc335689f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf568a4, 100 },
    { 0, 0 },
    { 0xb11568a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5568a9, 84 },
    { 0x59d568aa, 80 },
    { 0x8b568aa, 23 },
    { 0xdd5568aa, 33 },
    { 0x304568ad, 95 },
    { 0, 0 },
    { 0xce8568af, 100 },
    { 0x9e8568b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e568ba, 81 },
    { 0xcd1568bb, 94 },
    { 0x772568bc, 95 },
    { 0xe91568bc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f568c0, 95 },
    { 0, 0 },
    { 0xac9568c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48568c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9568c8, 98 },
    { 0xf08568c9, 61 },
    { 0xcfd568ca, 91 },
    { 0x9b1568cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de568d2, 34 },
    { 0x1a5568d3, 90 },
    { 0x359568d4, 90 },
    { 0, 0 },
    { 0x287568d6, 94 },
    { 0xf73568d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67568da, 76 },
    { 0xa16568db, 77 },
    { 0xfc5568dc, 95 },
    { 0x81f568dd, 87 },
    { 0x5c9568dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb568e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3568e6, 95 },
    { 0x604568e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238568ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e568f3, 34 },
    { 0xd2f568f4, 81 },
    { 0xf98568f4, 43 },
    { 0xb0c568f5, 23 },
    { 0x2b9568f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea568fb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x239568fe, 61 },
    { 0x835568ff, 67 },
    { 0, 0 },
    { 0x85456901, 95 },
    { 0x7eb56901, 90 },
    { 0, 0 },
    { 0x79656904, 76 },
    { 0x80056904, 90 },
    { 0x88856906, 95 },
    { 0xa956906, 61 },
    { 0xc4956908, 55 },
    { 0xa1856909, 90 },
    { 0xa8d56909, 17 },
    { 0xf0b5690b, 98 },
    { 0x84f5690c, 17 },
    { 0x705690d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d56912, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee56915, 90 },
    { 0x2b256915, 34 },
    { 0xd7156917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad35691f, 33 },
    { 0x67756920, 33 },
    { 0xe3856920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb56926, 95 },
    { 0x1ec56927, 93 },
    { 0x9b256928, 95 },
    { 0x1c656927, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5475692d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305693a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5425693e, 23 },
    { 0xc05693e, 55 },
    { 0, 0 },
    { 0xf7456941, 52 },
    { 0, 0 },
    { 0x86f56943, 23 },
    { 0, 0 },
    { 0xacd56945, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad5694e, 46 },
    { 0xc7b5694f, 23 },
    { 0xd6156950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d56954, 61 },
    { 0, 0 },
    { 0x74d56956, 76 },
    { 0x5e956957, 98 },
    { 0xd6b56958, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac65695d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce56967, 64 },
    { 0x5c556968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff56971, 84 },
    { 0xe6556972, 23 },
    { 0xff656971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda156977, 23 },
    { 0x50a56977, 23 },
    { 0x21b56977, 33 },
    { 0x96556978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e5697f, 93 },
    { 0x36f56980, 45 },
    { 0xfcc56981, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f56987, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef5698d, 33 },
    { 0xc045698d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0256994, 77 },
    { 0x69056994, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e05699f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600569a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8569a8, 61 },
    { 0x4f4569a9, 95 },
    { 0x3fe569a8, 23 },
    { 0x627569ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d569b1, 90 },
    { 0x3c1569b2, 90 },
    { 0, 0 },
    { 0x72e569b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f569b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x960569ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf569be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c569c1, 46 },
    { 0x8cb569c1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413569cd, 23 },
    { 0xfa9569ce, 55 },
    { 0xd7569cd, 33 },
    { 0x23f569d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5569d4, 90 },
    { 0x617569d4, 98 },
    { 0x816569d6, 67 },
    { 0xb08569d6, 76 },
    { 0x591569d8, 93 },
    { 0xd8d569d9, 61 },
    { 0x1d1569d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b569dd, 61 },
    { 0, 0 },
    { 0xae4569df, 75 },
    { 0x159569e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f569e4, 33 },
    { 0, 0 },
    { 0x900569e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05569e9, 22 },
    { 0xc83569e9, 76 },
    { 0, 0 },
    { 0x520569ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45569ef, 55 },
    { 0, 0 },
    { 0xe51569f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x772569f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da569f7, 76 },
    { 0, 0 },
    { 0x520569f9, 30 },
    { 0x186569fa, 61 },
    { 0x6c1569fb, 91 },
    { 0xd2569fc, 98 },
    { 0xff2569fd, 23 },
    { 0x964569fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45256a08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd56a0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd156a11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7256a14, 95 },
    { 0, 0 },
    { 0xbad56a16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90956a1a, 66 },
    { 0x71556a1b, 95 },
    { 0x6c456a1a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e256a20, 96 },
    { 0x3cb56a21, 100 },
    { 0xc6256a22, 55 },
    { 0x28a56a23, 8 },
    { 0x30156a24, 84 },
    { 0xeb056a24, 53 },
    { 0xebf56a26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f656a29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1156a2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3956a30, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e56a33, 34 },
    { 0, 0 },
    { 0x54e56a35, 95 },
    { 0xa8256a36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a56a39, 33 },
    { 0xf8f56a39, 98 },
    { 0x3eb56a3b, 33 },
    { 0xbc956a3b, 52 },
    { 0xf6256a3d, 33 },
    { 0x356a3c, 33 },
    { 0xac156a3f, 95 },
    { 0xa256a3b, 23 },
    { 0, 0 },
    { 0x57956a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61856a45, 23 },
    { 0, 0 },
    { 0xf6156a47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f56a4c, 90 },
    { 0xfd556a4d, 55 },
    { 0x61356a4e, 90 },
    { 0xd6356a4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc56a55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a56a59, 46 },
    { 0xc1656a59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a56a5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c56a62, 61 },
    { 0, 0 },
    { 0x56d56a64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e56a67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b956a6d, 40 },
    { 0x53a56a6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3256a79, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f56a7c, 33 },
    { 0x19756a7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43156a83, 90 },
    { 0x7a456a84, 95 },
    { 0x7f356a85, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8256a88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c56a8e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd156a94, 43 },
    { 0x99556a95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a656a9b, 76 },
    { 0, 0 },
    { 0x80956a9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7056aa1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69556aaa, 61 },
    { 0x59856aab, 46 },
    { 0x89a56aac, 61 },
    { 0xe2956aad, 55 },
    { 0x3a556aae, 43 },
    { 0x83d56aae, 90 },
    { 0x7f456ab0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa56ab8, 98 },
    { 0xaed56ab9, 33 },
    { 0, 0 },
    { 0x48b56abb, 90 },
    { 0xbdc56abc, 52 },
    { 0x2f156abc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3056ac2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70756ac7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb256aca, 68 },
    { 0x1ab56aca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e56ace, 90 },
    { 0x3556acf, 69 },
    { 0xf3556acf, 46 },
    { 0xa0056ad1, 34 },
    { 0xd2f56ad1, 98 },
    { 0xe0856ad1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68256ad8, 81 },
    { 0x11e56ad9, 85 },
    { 0xcce56ada, 96 },
    { 0xe0656ad9, 76 },
    { 0x87256ad8, 95 },
    { 0x33e56add, 33 },
    { 0, 0 },
    { 0xd3556adf, 97 },
    { 0xcdf56adf, 93 },
    { 0, 0 },
    { 0x4e056ae2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe056af0, 93 },
    { 0xd8356af1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf356af6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f56afd, 91 },
    { 0x2ff56afd, 55 },
    { 0x50e56aff, 43 },
    { 0, 0 },
    { 0xea456b01, 46 },
    { 0x71356b01, 61 },
    { 0xabb56b03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf656b06, 55 },
    { 0xa1556b07, 99 },
    { 0x38456b07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d56b0d, 23 },
    { 0xf3356b0e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x86156b11, 61 },
    { 0, 0 },
    { 0x4a356b13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1456b18, 90 },
    { 0x4dd56b19, 53 },
    { 0xde156b19, 99 },
    { 0x1f556b1b, 95 },
    { 0x9ab56b1b, 33 },
    { 0, 0 },
    { 0x6f256b1e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6256b22, 46 },
    { 0xbb356b23, 66 },
    { 0x78c56b24, 96 },
    { 0xd0f56b25, 33 },
    { 0x68756b26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce56b2b, 61 },
    { 0, 0 },
    { 0xdb356b2d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d56b34, 55 },
    { 0x5eb56b35, 23 },
    { 0, 0 },
    { 0xf8456b37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db56b3b, 34 },
    { 0xd9156b3c, 56 },
    { 0x23656b3d, 23 },
    { 0xf5456b3e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x36056b41, 61 },
    { 0, 0 },
    { 0x26256b43, 67 },
    { 0xc9b56b44, 68 },
    { 0xc356b44, 61 },
    { 0xf2756b43, 90 },
    { 0xa1156b45, 65 },
    { 0x5c456b48, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d256b4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a56b50, 98 },
    { 0xf2e56b51, 85 },
    { 0, 0 },
    { 0x8d756b53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3856b57, 23 },
    { 0x4ed56b58, 90 },
    { 0x2a356b58, 76 },
    { 0, 0 },
    { 0xa9a56b5b, 23 },
    { 0x20256b5c, 65 },
    { 0x60f56b5b, 90 },
    { 0x44f56b5b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a56b64, 34 },
    { 0xe7756b65, 61 },
    { 0, 0 },
    { 0xc5556b67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe556b6f, 53 },
    { 0, 0 },
    { 0xc6e56b71, 52 },
    { 0x29956b71, 33 },
    { 0x37656b71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf56b78, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc856b82, 23 },
    { 0x59756b83, 61 },
    { 0xdbe56b82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f56b87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd156b8a, 52 },
    { 0xa0956b8b, 33 },
    { 0x9c256b8c, 85 },
    { 0xc5656b8d, 98 },
    { 0, 0 },
    { 0xe7056b8f, 23 },
    { 0, 0 },
    { 0x10956b91, 76 },
    { 0x4e56b92, 61 },
    { 0x5e956b93, 90 },
    { 0, 0 },
    { 0xe9456b95, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55856b9b, 75 },
    { 0x1be56b9c, 66 },
    { 0xc4656b9c, 93 },
    { 0xb556b9e, 90 },
    { 0xf256b9e, 99 },
    { 0, 0 },
    { 0x66b56ba1, 78 },
    { 0, 0 },
    { 0xb0b56ba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37856ba6, 95 },
    { 0, 0 },
    { 0xa756ba8, 95 },
    { 0x15056ba9, 98 },
    { 0x4556baa, 33 },
    { 0xd2756bab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58256bb5, 90 },
    { 0x8c156bb6, 17 },
    { 0xe8f56bb5, 100 },
    { 0x8cb56bb5, 95 },
    { 0x19556bb5, 93 },
    { 0xc6a56bba, 61 },
    { 0, 0 },
    { 0x8c756bbc, 92 },
    { 0x73f56bbd, 23 },
    { 0x44456bbe, 85 },
    { 0xb6856bbf, 90 },
    { 0, 0 },
    { 0x9b256bc1, 34 },
    { 0, 0 },
    { 0x7c656bc3, 50 },
    { 0x13856bc3, 34 },
    { 0xc5556bc5, 84 },
    { 0x15c56bc3, 99 },
    { 0xad356bc7, 33 },
    { 0xf8a56bc4, 61 },
    { 0xc8356bc9, 66 },
    { 0x94856bca, 90 },
    { 0x21056bc3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x82756bce, 95 },
    { 0, 0 },
    { 0xeb856bd0, 90 },
    { 0x5c356bd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f156bd6, 64 },
    { 0xb8056bd7, 61 },
    { 0x18856bd8, 23 },
    { 0, 0 },
    { 0xd8356bda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a56be2, 43 },
    { 0x1a956be3, 55 },
    { 0, 0 },
    { 0x17a56be5, 90 },
    { 0x67956be5, 52 },
    { 0x3ab56be6, 96 },
    { 0x23d56be5, 23 },
    { 0x87f56be9, 98 },
    { 0x5b656bea, 90 },
    { 0xee756bea, 90 },
    { 0, 0 },
    { 0x36b56bed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab56bf2, 55 },
    { 0xd6056bf3, 84 },
    { 0xd4956bf2, 90 },
    { 0x7af56bf5, 39 },
    { 0x66456bf6, 78 },
    { 0x74d56bf7, 90 },
    { 0x9e656bf8, 68 },
    { 0x81956bf9, 99 },
    { 0, 0 },
    { 0x3f756bfb, 61 },
    { 0, 0 },
    { 0xfb156bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe56c01, 87 },
    { 0x86856c01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe156c06, 43 },
    { 0x65e56c06, 76 },
    { 0xde956c07, 33 },
    { 0x28556c09, 33 },
    { 0x94b56c0a, 53 },
    { 0x53c56c06, 95 },
    { 0x27c56c06, 33 },
    { 0x57f56c0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b56c10, 69 },
    { 0xe2656c11, 61 },
    { 0xb0d56c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18956c1b, 55 },
    { 0, 0 },
    { 0x7c856c1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c56c21, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b56c24, 33 },
    { 0x71a56c25, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef56c2b, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df56c2f, 66 },
    { 0xc4456c30, 73 },
    { 0x56556c31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a56c36, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde56c3a, 52 },
    { 0x81756c3b, 78 },
    { 0x69e56c3b, 90 },
    { 0xf1f56c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f56c41, 87 },
    { 0xe8e56c41, 69 },
    { 0x1cf56c41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a56c48, 85 },
    { 0x8e356c49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee456c4d, 98 },
    { 0, 0 },
    { 0xa2256c4f, 91 },
    { 0x2ef56c50, 23 },
    { 0xbdd56c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe56c56, 55 },
    { 0x8c856c56, 61 },
    { 0x7d156c57, 61 },
    { 0, 0 },
    { 0x38a56c5a, 46 },
    { 0xf8d56c5b, 43 },
    { 0, 0 },
    { 0x6a456c5d, 46 },
    { 0x73456c5e, 99 },
    { 0xc4f56c5f, 65 },
    { 0, 0 },
    { 0xe5b56c61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba856c65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b56c6f, 53 },
    { 0, 0 },
    { 0x8ec56c71, 91 },
    { 0, 0 },
    { 0x79856c73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6856c76, 43 },
    { 0x35856c77, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c556c7f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea156c83, 95 },
    { 0, 0 },
    { 0xf3356c85, 23 },
    { 0x91f56c86, 66 },
    { 0x4e656c87, 33 },
    { 0x56656c86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5456c8b, 33 },
    { 0xdf256c8c, 52 },
    { 0x2aa56c8c, 55 },
    { 0x8d256c8e, 65 },
    { 0, 0 },
    { 0x20156c90, 95 },
    { 0, 0 },
    { 0x18f56c92, 76 },
    { 0xb9856c93, 68 },
    { 0x8fc56c93, 98 },
    { 0xa4d56c94, 36 },
    { 0xcd956c94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e756c99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e56c9d, 95 },
    { 0, 0 },
    { 0x8a456c9f, 55 },
    { 0x2ac56ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f56ca5, 93 },
    { 0, 0 },
    { 0x9ff56ca7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb56caa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea56cb1, 90 },
    { 0x8fe56cb2, 98 },
    { 0x9b156cb3, 90 },
    { 0, 0 },
    { 0xe8d56cb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3056cb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32556cbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf56cc0, 33 },
    { 0xe6856cc1, 76 },
    { 0x2bd56cc1, 61 },
    { 0x64356cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14156cc7, 61 },
    { 0xae556cc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd56cd2, 61 },
    { 0, 0 },
    { 0x2d456cd4, 55 },
    { 0, 0 },
    { 0x4656cd6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x95156cd9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea56cdf, 23 },
    { 0, 0 },
    { 0x7a756ce1, 66 },
    { 0, 0 },
    { 0x72c56ce3, 67 },
    { 0, 0 },
    { 0x47056ce5, 91 },
    { 0, 0 },
    { 0x50256ce7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b56ceb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2856cef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41056cf2, 74 },
    { 0, 0 },
    { 0x67256cf4, 33 },
    { 0xefc56cf5, 33 },
    { 0x95356cf6, 91 },
    { 0x9fc56cf6, 98 },
    { 0x57d56cf7, 52 },
    { 0x16c56cf9, 90 },
    { 0xf9d56cf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c56d03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f456d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c56d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0456d0d, 90 },
    { 0xa9356d0e, 87 },
    { 0xcbe56d0f, 93 },
    { 0x4ca56d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58256d1a, 95 },
    { 0x63756d1b, 61 },
    { 0xb1356d1c, 67 },
    { 0x3a856d1c, 90 },
    { 0x66f56d1b, 40 },
    { 0x63956d1d, 23 },
    { 0x8f056d1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a56d28, 84 },
    { 0x7a956d28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd956d2c, 95 },
    { 0x85e56d2d, 96 },
    { 0x8fe56d2d, 34 },
    { 0x3b056d2f, 66 },
    { 0, 0 },
    { 0xfe556d31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d56d34, 98 },
    { 0x81856d35, 23 },
    { 0x4cf56d35, 33 },
    { 0xf1056d36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d56d4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44356d55, 23 },
    { 0x5fc56d56, 23 },
    { 0xee056d56, 33 },
    { 0x1c256d57, 61 },
    { 0, 0 },
    { 0xaf956d5a, 23 },
    { 0x4ee56d5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18756d60, 75 },
    { 0, 0 },
    { 0x55956d62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d856d66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a56d6a, 23 },
    { 0x4a956d6b, 23 },
    { 0x9b256d6c, 90 },
    { 0x80956d6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa56d72, 33 },
    { 0, 0 },
    { 0xec056d74, 68 },
    { 0, 0 },
    { 0x68156d76, 84 },
    { 0x66e56d77, 95 },
    { 0x3b56d77, 61 },
    { 0x72456d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc56d7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7556d80, 95 },
    { 0x2e556d81, 98 },
    { 0, 0 },
    { 0xd8c56d83, 95 },
    { 0x81256d84, 33 },
    { 0x27e56d84, 95 },
    { 0x68156d85, 95 },
    { 0x67656d87, 81 },
    { 0x61856d86, 61 },
    { 0x62456d86, 85 },
    { 0x66f56d8a, 100 },
    { 0x3cf56d84, 33 },
    { 0x69c56d87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5656d9a, 33 },
    { 0x8256d9b, 93 },
    { 0xae356d9c, 98 },
    { 0x15656d9d, 33 },
    { 0xdaa56d9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa56da2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2756da9, 84 },
    { 0xa7956daa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce56dad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27056db8, 90 },
    { 0x5ae56db9, 23 },
    { 0x33156dba, 33 },
    { 0xde956dbb, 98 },
    { 0x2be56dbc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0056dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa56dc5, 33 },
    { 0, 0 },
    { 0xc7156dc7, 91 },
    { 0, 0 },
    { 0x86656dc9, 23 },
    { 0xc5956dca, 76 },
    { 0xa8456dcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36856dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d456dd6, 55 },
    { 0x17456dd6, 98 },
    { 0xe8556dd8, 76 },
    { 0x39956dd9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f56de2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da56de5, 64 },
    { 0x92956de6, 23 },
    { 0x27456de7, 76 },
    { 0xacb56de7, 33 },
    { 0x28356de9, 34 },
    { 0x42f56dea, 66 },
    { 0xd4d56dea, 98 },
    { 0xa2956dec, 91 },
    { 0x96256de7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1756df3, 98 },
    { 0, 0 },
    { 0xd2a56df5, 65 },
    { 0x32056df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29956df9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a56dfd, 33 },
    { 0x71656dfe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad256e03, 81 },
    { 0x21a56e03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36056e10, 23 },
    { 0xf3656e11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f56e14, 98 },
    { 0xeb756e15, 87 },
    { 0x35f56e16, 91 },
    { 0, 0 },
    { 0x5e756e18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a56e1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd656e20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac56e26, 33 },
    { 0x1e756e27, 61 },
    { 0, 0 },
    { 0x55056e29, 81 },
    { 0, 0 },
    { 0x43a56e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c56e31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb56e34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3456e37, 95 },
    { 0x13e56e38, 33 },
    { 0x9bd56e39, 90 },
    { 0x4c56e3a, 36 },
    { 0x70356e39, 90 },
    { 0x3dd56e3c, 66 },
    { 0, 0 },
    { 0x9f56e3e, 33 },
    { 0, 0 },
    { 0x4f056e40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead56e4a, 90 },
    { 0, 0 },
    { 0x13c56e4c, 43 },
    { 0x77356e4c, 90 },
    { 0x68256e4e, 93 },
    { 0xa1256e4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3156e55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24856e5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce56e5d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c56e67, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad56e6b, 98 },
    { 0x8056e6b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d56e6f, 43 },
    { 0x17d56e70, 61 },
    { 0, 0 },
    { 0x3c856e72, 98 },
    { 0x54956e73, 76 },
    { 0x83b56e72, 95 },
    { 0xc8356e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16056e78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e56e7d, 77 },
    { 0, 0 },
    { 0x44556e7f, 43 },
    { 0xca856e80, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2656e85, 90 },
    { 0x4f56e86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1856e8a, 65 },
    { 0x35356e8b, 34 },
    { 0x2756e8a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e56e94, 23 },
    { 0, 0 },
    { 0x7c456e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad56e9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e56e9d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6756ea4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56356ea7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d56eaa, 65 },
    { 0xe0d56eab, 76 },
    { 0xd7256eac, 33 },
    { 0x6f56eac, 90 },
    { 0x31756eae, 90 },
    { 0, 0 },
    { 0xa0a56eb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef56eb3, 23 },
    { 0x68356eb4, 61 },
    { 0xc2d56eb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca56ebb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd256ebe, 76 },
    { 0xe9e56ebe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e56ec2, 90 },
    { 0x25b56ec3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c356ec6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c256ec9, 98 },
    { 0x5656ec9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf956ecf, 90 },
    { 0, 0 },
    { 0xe8f56ed1, 46 },
    { 0x2b956ed1, 33 },
    { 0x60056ed3, 33 },
    { 0x51a56ed4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5256eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd56edd, 99 },
    { 0, 0 },
    { 0xe7c56edf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf56ee5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f56ee9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b56eed, 85 },
    { 0x70456eee, 90 },
    { 0xb4a56eef, 99 },
    { 0, 0 },
    { 0x4c356ef1, 55 },
    { 0x43256ef2, 23 },
    { 0xc4856ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63756ef7, 33 },
    { 0x43656ef8, 90 },
    { 0x2ed56ef9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25256f02, 90 },
    { 0, 0 },
    { 0xf8556f04, 52 },
    { 0, 0 },
    { 0x2e456f06, 23 },
    { 0, 0 },
    { 0x67156f08, 55 },
    { 0xf3f56f09, 23 },
    { 0, 0 },
    { 0x79656f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b856f10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c256f13, 98 },
    { 0, 0 },
    { 0x72856f15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc56f18, 95 },
    { 0, 0 },
    { 0xc0856f1a, 33 },
    { 0, 0 },
    { 0x90756f1c, 86 },
    { 0xf4556f1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac56f21, 95 },
    { 0xc6456f22, 43 },
    { 0, 0 },
    { 0x95e56f24, 78 },
    { 0x84056f25, 90 },
    { 0, 0 },
    { 0xce056f27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd56f2d, 91 },
    { 0xf8356f2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d856f31, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f356f40, 61 },
    { 0, 0 },
    { 0x5756f42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27456f46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77256f4a, 90 },
    { 0x65856f4b, 38 },
    { 0x7e156f4c, 61 },
    { 0, 0 },
    { 0xcca56f4e, 61 },
    { 0x2d356f4f, 90 },
    { 0xdd356f4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d56f53, 34 },
    { 0xd4056f53, 46 },
    { 0xe9456f55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56f63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a56f66, 95 },
    { 0xef356f67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48856f6a, 76 },
    { 0xd7656f6b, 33 },
    { 0x7cb56f6c, 46 },
    { 0x64a56f6c, 33 },
    { 0xab956f6e, 23 },
    { 0xca456f6f, 65 },
    { 0x60356f70, 99 },
    { 0xace56f71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c56f7f, 94 },
    { 0xb8a56f7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b856f86, 95 },
    { 0x2ee56f87, 90 },
    { 0x2d456f87, 95 },
    { 0, 0 },
    { 0x68f56f8a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e156f90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1256f94, 95 },
    { 0xce956f94, 68 },
    { 0xa0556f96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f656fa3, 88 },
    { 0, 0 },
    { 0xe8f56fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a56fa8, 75 },
    { 0x1d756fa9, 61 },
    { 0x5ff56fa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6956fb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d56fb8, 90 },
    { 0xdc556fb8, 34 },
    { 0x13556fb8, 88 },
    { 0x7fd56fbb, 43 },
    { 0x21e56fb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd756fc4, 33 },
    { 0x4f56fc4, 33 },
    { 0, 0 },
    { 0xdc156fc7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2356fcd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e56fd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93756fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13156fdc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4656fe2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e156fe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b56feb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b556ff0, 98 },
    { 0xda256ff1, 55 },
    { 0, 0 },
    { 0x33c56ff3, 55 },
    { 0xcfd56ff4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64056ffa, 95 },
    { 0x556ffb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb756ffe, 23 },
    { 0x71c56ffe, 85 },
    { 0x45157000, 33 },
    { 0xdc957001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba57005, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e457009, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a55700e, 55 },
    { 0xee95700e, 33 },
    { 0xc0557010, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b57014, 95 },
    { 0xd5157015, 97 },
    { 0xcda57015, 46 },
    { 0, 0 },
    { 0xb2d57018, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b5701d, 90 },
    { 0x66f5701d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13457023, 98 },
    { 0x54b57024, 55 },
    { 0x95157025, 52 },
    { 0xe3557024, 41 },
    { 0x6257025, 33 },
    { 0, 0 },
    { 0x42f57029, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b5702e, 23 },
    { 0x8e45702e, 91 },
    { 0xe975702e, 95 },
    { 0xbfa57030, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91357036, 55 },
    { 0, 0 },
    { 0xc3257038, 55 },
    { 0xe0b57039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4757049, 23 },
    { 0x6fc57049, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5704f, 95 },
    { 0xad65704f, 52 },
    { 0x20757051, 90 },
    { 0x77a57052, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9357056, 77 },
    { 0, 0 },
    { 0x91c57058, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca5705c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4957061, 89 },
    { 0x4ca57062, 61 },
    { 0xf8f57061, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35706a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf5706d, 91 },
    { 0xc1a5706e, 52 },
    { 0, 0 },
    { 0x11d57070, 33 },
    { 0, 0 },
    { 0x76357072, 23 },
    { 0x16057073, 77 },
    { 0xf6b57073, 61 },
    { 0x4fa57074, 84 },
    { 0xf6d57076, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475707b, 90 },
    { 0, 0 },
    { 0xe895707d, 61 },
    { 0xe645707e, 33 },
    { 0xcd55707f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d57083, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47357087, 90 },
    { 0xdeb57087, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775708d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30857092, 84 },
    { 0x9eb57093, 55 },
    { 0x17e57093, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac457097, 90 },
    { 0, 0 },
    { 0xf6a57099, 55 },
    { 0, 0 },
    { 0x7b35709b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286570a5, 34 },
    { 0x8de570a6, 76 },
    { 0xb9c570a6, 23 },
    { 0x3df570a8, 90 },
    { 0x48e570a9, 91 },
    { 0x1f0570a8, 33 },
    { 0xf71570a6, 95 },
    { 0x906570ac, 33 },
    { 0x636570ad, 33 },
    { 0x32d570ad, 90 },
    { 0xbb2570ac, 90 },
    { 0xaf570ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf570b3, 78 },
    { 0, 0 },
    { 0x1e570b5, 98 },
    { 0xc41570b5, 95 },
    { 0xe99570b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4570bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97570c7, 33 },
    { 0xd91570c7, 28 },
    { 0x8fd570c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88570d2, 61 },
    { 0x960570d3, 93 },
    { 0x7fd570d2, 23 },
    { 0xc43570d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50570db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x867570de, 98 },
    { 0xca1570de, 95 },
    { 0x44c570df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2570e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1570e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c570f2, 61 },
    { 0xa26570f2, 43 },
    { 0, 0 },
    { 0xe97570f5, 66 },
    { 0xb06570f5, 76 },
    { 0, 0 },
    { 0x125570f8, 95 },
    { 0x97e570f8, 23 },
    { 0, 0 },
    { 0x1f1570fb, 40 },
    { 0, 0 },
    { 0x781570fd, 92 },
    { 0, 0 },
    { 0x25d570ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb57102, 33 },
    { 0, 0 },
    { 0x80c57104, 23 },
    { 0x5ec57104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2445710d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44457112, 98 },
    { 0, 0 },
    { 0x2f957114, 23 },
    { 0, 0 },
    { 0xd9057116, 90 },
    { 0x97257117, 66 },
    { 0xd8d57116, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6485711e, 34 },
    { 0, 0 },
    { 0xeb357120, 90 },
    { 0x81657120, 90 },
    { 0x70357122, 91 },
    { 0, 0 },
    { 0xea57124, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5712a, 93 },
    { 0xc05712b, 93 },
    { 0x7fe5712c, 75 },
    { 0, 0 },
    { 0xb495712e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f57133, 33 },
    { 0x51557133, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce5713e, 90 },
    { 0, 0 },
    { 0x88457140, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf757143, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc757147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a5714b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5714e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd57158, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5715b, 90 },
    { 0xfe45715b, 81 },
    { 0x85a5715d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb57162, 98 },
    { 0xd1357162, 95 },
    { 0x2fe57164, 95 },
    { 0x95c57164, 33 },
    { 0, 0 },
    { 0xd1157167, 98 },
    { 0xd1657168, 34 },
    { 0x30357168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5935716f, 33 },
    { 0x7585716f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce357175, 77 },
    { 0, 0 },
    { 0x6f057177, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c5717b, 34 },
    { 0x8be5717b, 93 },
    { 0xe865717d, 91 },
    { 0x43a5717d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa57184, 66 },
    { 0x45b57185, 33 },
    { 0x7bd57186, 61 },
    { 0x15157186, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1675718e, 23 },
    { 0, 0 },
    { 0xec57190, 100 },
    { 0xc9d57191, 66 },
    { 0, 0 },
    { 0x20157193, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e57199, 23 },
    { 0x48d5719a, 90 },
    { 0, 0 },
    { 0x7725719c, 96 },
    { 0, 0 },
    { 0x55f5719e, 42 },
    { 0x89a5719f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab571a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5571a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e571ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e571af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d571b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b571b8, 80 },
    { 0xd80571b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7571be, 48 },
    { 0xc81571bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x562571c2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7571d3, 90 },
    { 0, 0 },
    { 0x9ef571d5, 95 },
    { 0xd1571d6, 65 },
    { 0xd55571d7, 68 },
    { 0, 0 },
    { 0xb99571d9, 75 },
    { 0, 0 },
    { 0xf06571db, 52 },
    { 0x51d571db, 98 },
    { 0x1af571dd, 76 },
    { 0xb34571dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc571e2, 34 },
    { 0xab1571e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c571e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd571eb, 84 },
    { 0x7f4571ec, 75 },
    { 0xf44571ed, 84 },
    { 0xf4e571ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0571f3, 90 },
    { 0xd5b571f3, 75 },
    { 0x20f571f5, 95 },
    { 0xf3c571f4, 98 },
    { 0x68f571f7, 61 },
    { 0x622571f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463571fe, 88 },
    { 0x404571ff, 69 },
    { 0xfdc57200, 23 },
    { 0x65057201, 95 },
    { 0x665571ff, 43 },
    { 0xac857203, 66 },
    { 0xa0557204, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2957207, 61 },
    { 0xf3857208, 43 },
    { 0, 0 },
    { 0xa4c5720a, 23 },
    { 0x56b5720b, 46 },
    { 0xb745720c, 23 },
    { 0xf595720d, 78 },
    { 0, 0 },
    { 0xb835720f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e57212, 93 },
    { 0, 0 },
    { 0xabf57214, 90 },
    { 0, 0 },
    { 0x31657216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f57220, 33 },
    { 0xd1c57221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa57227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b5722b, 90 },
    { 0, 0 },
    { 0xeaa5722d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52857230, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a57235, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8857238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec5723c, 70 },
    { 0x605723d, 33 },
    { 0, 0 },
    { 0x4ac5723f, 90 },
    { 0x79057240, 98 },
    { 0xe8257241, 90 },
    { 0x51357242, 34 },
    { 0, 0 },
    { 0x78757244, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74657249, 33 },
    { 0, 0 },
    { 0xb2b5724b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec5724e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2557251, 76 },
    { 0x42d57252, 66 },
    { 0xf1357253, 43 },
    { 0x7f457252, 95 },
    { 0xde657255, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e157258, 81 },
    { 0xd5357259, 23 },
    { 0x30b5725a, 90 },
    { 0xd35725b, 46 },
    { 0x19c57258, 38 },
    { 0, 0 },
    { 0x6c05725e, 84 },
    { 0xf435725e, 33 },
    { 0xb2457260, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad57263, 95 },
    { 0x91d57264, 55 },
    { 0, 0 },
    { 0xeb457266, 98 },
    { 0, 0 },
    { 0x60157268, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e5726b, 91 },
    { 0x9d25726c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9157274, 100 },
    { 0x1bf57275, 61 },
    { 0x6bb57276, 66 },
    { 0xb4357276, 51 },
    { 0x12a57276, 43 },
    { 0x3d957276, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa135727f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c757288, 33 },
    { 0, 0 },
    { 0xa025728a, 98 },
    { 0x2775728b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c5728e, 61 },
    { 0x59a5728f, 99 },
    { 0xe725728e, 34 },
    { 0xde957291, 77 },
    { 0xa5557292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d57295, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9057298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa035729e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9572ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe572b4, 98 },
    { 0, 0 },
    { 0xb08572b6, 95 },
    { 0x25d572b7, 33 },
    { 0, 0 },
    { 0x9ee572b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd572bd, 98 },
    { 0xc5d572be, 95 },
    { 0x7dc572bf, 98 },
    { 0x9e7572be, 100 },
    { 0xfc8572c1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7572c6, 93 },
    { 0x460572c7, 99 },
    { 0xceb572c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476572cc, 55 },
    { 0xee4572cc, 95 },
    { 0xc38572cd, 61 },
    { 0xcce572cf, 40 },
    { 0xded572d0, 76 },
    { 0, 0 },
    { 0x538572d2, 23 },
    { 0, 0 },
    { 0x461572d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3572da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac572e1, 76 },
    { 0xce7572e2, 100 },
    { 0xfca572e1, 33 },
    { 0xf35572e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483572e8, 33 },
    { 0x4ae572e8, 87 },
    { 0x7a0572e9, 34 },
    { 0x583572eb, 33 },
    { 0xf0b572e9, 36 },
    { 0xa93572ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x969572f0, 90 },
    { 0xf36572f1, 76 },
    { 0x269572f2, 75 },
    { 0, 0 },
    { 0x9a3572f4, 23 },
    { 0x401572f5, 91 },
    { 0x471572f5, 100 },
    { 0x913572f6, 90 },
    { 0xda0572f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b572fb, 75 },
    { 0x778572fc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257302, 83 },
    { 0, 0 },
    { 0xc0357304, 33 },
    { 0x2fa57305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4955730c, 34 },
    { 0xda35730d, 92 },
    { 0x65a5730d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec57313, 98 },
    { 0xca357314, 61 },
    { 0x5fd57315, 43 },
    { 0xaaa57316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3415731a, 43 },
    { 0x2855731b, 55 },
    { 0, 0 },
    { 0xf345731d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc857321, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1057324, 98 },
    { 0x2df57325, 53 },
    { 0xd257326, 55 },
    { 0x93257327, 76 },
    { 0x38157326, 17 },
    { 0xb6e57329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6057330, 90 },
    { 0x57657330, 87 },
    { 0xeec57332, 99 },
    { 0xf2057332, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f057337, 87 },
    { 0, 0 },
    { 0xdf457339, 76 },
    { 0x985733a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf35733f, 61 },
    { 0x29757340, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb57344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a957349, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d95734c, 17 },
    { 0, 0 },
    { 0x7f55734e, 33 },
    { 0, 0 },
    { 0x58657350, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96657355, 81 },
    { 0, 0 },
    { 0x53f57357, 98 },
    { 0x86e57358, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b5735b, 43 },
    { 0, 0 },
    { 0x3e15735d, 34 },
    { 0x79e5735e, 76 },
    { 0xa5c5735e, 43 },
    { 0, 0 },
    { 0x76a57361, 61 },
    { 0x92a57361, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf5736d, 33 },
    { 0x5955736e, 87 },
    { 0xf5a5736e, 43 },
    { 0, 0 },
    { 0xb4b57371, 98 },
    { 0, 0 },
    { 0x60c57373, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b457379, 90 },
    { 0x3a757379, 46 },
    { 0x60357379, 55 },
    { 0xb645737c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73557381, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e57385, 90 },
    { 0, 0 },
    { 0x45657387, 97 },
    { 0x58d57387, 76 },
    { 0x7b857387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5945738d, 61 },
    { 0xf885738e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7757396, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d75739b, 34 },
    { 0, 0 },
    { 0xbf45739d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff573a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a573aa, 55 },
    { 0x58b573aa, 98 },
    { 0, 0 },
    { 0xd46573ad, 61 },
    { 0xd32573ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7573b1, 61 },
    { 0xd60573b2, 55 },
    { 0xd11573b3, 76 },
    { 0x597573b4, 23 },
    { 0x9de573b4, 43 },
    { 0x9d6573b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8573bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb573bf, 61 },
    { 0, 0 },
    { 0x201573c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7573c6, 61 },
    { 0x2e3573c7, 61 },
    { 0xc54573c7, 33 },
    { 0x1c573c9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574573ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be573d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b573db, 87 },
    { 0xa3c573db, 23 },
    { 0x730573dc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27573e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b573e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x199573e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9573e9, 33 },
    { 0x763573ea, 90 },
    { 0xe64573eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f573f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3573f8, 46 },
    { 0x9a9573f8, 23 },
    { 0, 0 },
    { 0xbd8573fb, 61 },
    { 0, 0 },
    { 0x1c1573fd, 76 },
    { 0xe67573fe, 46 },
    { 0x95e573fe, 98 },
    { 0, 0 },
    { 0x13457401, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3357408, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xea35740b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54457410, 34 },
    { 0x7b557411, 23 },
    { 0x68657412, 61 },
    { 0x1ab57413, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f557418, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc35741e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8857422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d57425, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x67757428, 90 },
    { 0, 0 },
    { 0x34d5742a, 81 },
    { 0, 0 },
    { 0x1b35742c, 66 },
    { 0, 0 },
    { 0xdb25742e, 56 },
    { 0xc115742e, 53 },
    { 0x5805742e, 66 },
    { 0xd0c5742e, 90 },
    { 0xa2657430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8957436, 76 },
    { 0x7ab57437, 98 },
    { 0x41157437, 66 },
    { 0xfb957437, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6385743d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf757440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4157448, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb895744c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede57453, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35745a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5745d, 75 },
    { 0, 0 },
    { 0x6935745f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f5746f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b357472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa975747c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90657481, 98 },
    { 0x1f057482, 98 },
    { 0, 0 },
    { 0x19a57484, 33 },
    { 0x8a657485, 34 },
    { 0x78057486, 98 },
    { 0x2257487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc945748c, 34 },
    { 0xc1b5748c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22657490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d5749c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe574a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4574a9, 61 },
    { 0x9574a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf574ad, 52 },
    { 0x7e1574ae, 61 },
    { 0xdc574af, 33 },
    { 0x551574b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6574b4, 98 },
    { 0, 0 },
    { 0xf25574b6, 33 },
    { 0xf6b574b6, 34 },
    { 0x6c574b7, 98 },
    { 0x558574b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375574c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781574c7, 90 },
    { 0x93f574c8, 33 },
    { 0x61f574c8, 60 },
    { 0x8cc574ca, 93 },
    { 0xa1a574cb, 87 },
    { 0xb46574cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2574d5, 90 },
    { 0, 0 },
    { 0x6fe574d7, 85 },
    { 0xeff574d8, 76 },
    { 0x18e574d9, 33 },
    { 0, 0 },
    { 0xda7574db, 55 },
    { 0x13d574dc, 33 },
    { 0xed3574dd, 98 },
    { 0x590574de, 96 },
    { 0xe7c574dc, 95 },
    { 0xe3f574e0, 33 },
    { 0x14b574df, 61 },
    { 0x3e4574e2, 67 },
    { 0x248574e3, 76 },
    { 0xffb574e3, 23 },
    { 0xfa2574e5, 52 },
    { 0xc94574e2, 95 },
    { 0x25c574e7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9574ea, 90 },
    { 0xc74574eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9574ee, 46 },
    { 0x5d1574ef, 88 },
    { 0x72b574ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a574f3, 77 },
    { 0, 0 },
    { 0xbf3574f5, 52 },
    { 0x99d574f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607574fa, 33 },
    { 0xe6c574fb, 84 },
    { 0xc4f574fa, 34 },
    { 0, 0 },
    { 0x369574fe, 65 },
    { 0xe8e574ff, 17 },
    { 0, 0 },
    { 0x3f557501, 93 },
    { 0xb2457502, 76 },
    { 0x13657503, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff57509, 84 },
    { 0x29e57509, 25 },
    { 0xb25750b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d57512, 76 },
    { 0xb9c57513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e95751a, 91 },
    { 0, 0 },
    { 0xf905751c, 76 },
    { 0, 0 },
    { 0x3955751e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d57523, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8085752b, 33 },
    { 0xc535752c, 55 },
    { 0x67b5752d, 90 },
    { 0, 0 },
    { 0xf605752f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93757533, 98 },
    { 0x14357534, 95 },
    { 0xa0157535, 61 },
    { 0x87257534, 42 },
    { 0, 0 },
    { 0x6b557538, 43 },
    { 0x21857539, 23 },
    { 0x1115753a, 90 },
    { 0xd005753b, 95 },
    { 0x11557538, 61 },
    { 0x7d85753c, 76 },
    { 0xc065753e, 98 },
    { 0xbe85753d, 61 },
    { 0x47357540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33357544, 61 },
    { 0xe9557545, 80 },
    { 0, 0 },
    { 0xa6957547, 93 },
    { 0, 0 },
    { 0x74657549, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb5754c, 85 },
    { 0, 0 },
    { 0x7555754e, 23 },
    { 0, 0 },
    { 0x6cc57550, 98 },
    { 0x3e957551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf457556, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d35755d, 53 },
    { 0, 0 },
    { 0xf8c5755f, 33 },
    { 0, 0 },
    { 0xdb157561, 23 },
    { 0x65c57561, 90 },
    { 0x2a357563, 33 },
    { 0xe3257564, 90 },
    { 0xc457564, 43 },
    { 0x49157565, 33 },
    { 0xac757561, 46 },
    { 0xded57568, 90 },
    { 0, 0 },
    { 0xf935756a, 84 },
    { 0xb685756b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34657571, 61 },
    { 0xf9457572, 61 },
    { 0xecc57572, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb865757a, 33 },
    { 0, 0 },
    { 0x7905757c, 98 },
    { 0x8885757d, 61 },
    { 0x3475757e, 52 },
    { 0, 0 },
    { 0x48857580, 33 },
    { 0x9a957580, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9457585, 53 },
    { 0x84357585, 52 },
    { 0x70d57585, 34 },
    { 0xddf57586, 95 },
    { 0xd357589, 95 },
    { 0x26d5758a, 23 },
    { 0, 0 },
    { 0xd525758c, 95 },
    { 0, 0 },
    { 0xbbc5758e, 98 },
    { 0x3305758f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0557593, 23 },
    { 0, 0 },
    { 0xf6257595, 88 },
    { 0x25d57595, 23 },
    { 0, 0 },
    { 0x36e57598, 33 },
    { 0, 0 },
    { 0x7355759a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f5759e, 43 },
    { 0x8835759f, 23 },
    { 0, 0 },
    { 0x8ab575a1, 61 },
    { 0, 0 },
    { 0x7ca575a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69575a6, 8 },
    { 0xbaf575a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3575ae, 43 },
    { 0xc65575af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124575b3, 76 },
    { 0, 0 },
    { 0x1c2575b5, 87 },
    { 0, 0 },
    { 0x1bd575b7, 65 },
    { 0x9a8575b8, 95 },
    { 0xbfd575b9, 99 },
    { 0x575575ba, 61 },
    { 0xa9a575bb, 61 },
    { 0x67e575bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff575c1, 90 },
    { 0x1e4575c2, 95 },
    { 0xac4575c1, 85 },
    { 0xf80575c2, 66 },
    { 0x5a4575c5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc575c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147575da, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d575e4, 23 },
    { 0xf86575e5, 75 },
    { 0xdab575e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78575e9, 87 },
    { 0xe23575ea, 66 },
    { 0x375575e9, 61 },
    { 0x5ef575ea, 90 },
    { 0x584575ed, 100 },
    { 0x2fe575e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c575f2, 66 },
    { 0, 0 },
    { 0x1be575f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x847575f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e575fc, 90 },
    { 0xf70575fd, 87 },
    { 0x3d7575fd, 95 },
    { 0xe4d575ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f757602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88057605, 61 },
    { 0x27157606, 98 },
    { 0x44d57606, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b65760b, 95 },
    { 0x4bb5760c, 90 },
    { 0x8ec5760c, 88 },
    { 0, 0 },
    { 0x4585760f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9357617, 95 },
    { 0, 0 },
    { 0x8b557619, 95 },
    { 0, 0 },
    { 0x9945761b, 48 },
    { 0xa475761b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4757622, 23 },
    { 0x40a57622, 76 },
    { 0xe9457624, 34 },
    { 0xd0c57624, 98 },
    { 0x50b57626, 23 },
    { 0, 0 },
    { 0x41f57628, 90 },
    { 0xcf657629, 23 },
    { 0x6275762a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f5762f, 90 },
    { 0x7a857630, 75 },
    { 0, 0 },
    { 0xa3557632, 61 },
    { 0, 0 },
    { 0xd7a57634, 100 },
    { 0xc7157635, 95 },
    { 0x91957636, 90 },
    { 0x76257637, 66 },
    { 0xf8857634, 33 },
    { 0x94657635, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae5763c, 55 },
    { 0xe4a5763c, 61 },
    { 0x5e35763e, 91 },
    { 0x9a85763f, 75 },
    { 0x25b57640, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f57643, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d57647, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b15764a, 87 },
    { 0, 0 },
    { 0x12a5764c, 90 },
    { 0xc8c5764d, 99 },
    { 0x815764e, 23 },
    { 0xddf5764d, 55 },
    { 0, 0 },
    { 0xef857651, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1557654, 46 },
    { 0xd4157654, 23 },
    { 0xf5357654, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9157659, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e5765e, 61 },
    { 0x2ab5765e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b57663, 98 },
    { 0xce657664, 33 },
    { 0x21757665, 67 },
    { 0x45257665, 33 },
    { 0x91857667, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d5766c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc57670, 23 },
    { 0, 0 },
    { 0x69957672, 93 },
    { 0, 0 },
    { 0xb3c57674, 17 },
    { 0x7de57675, 88 },
    { 0x63857675, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d65767b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e05767f, 93 },
    { 0xc3b57680, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57557684, 23 },
    { 0xe7c57685, 33 },
    { 0, 0 },
    { 0x6aa57687, 78 },
    { 0, 0 },
    { 0xa7557689, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37157690, 43 },
    { 0x67657691, 87 },
    { 0xd2357690, 85 },
    { 0x29457693, 33 },
    { 0x7ea57694, 33 },
    { 0x85557695, 33 },
    { 0xa6157691, 66 },
    { 0xefa57690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5769c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314576a1, 87 },
    { 0xf0c576a2, 33 },
    { 0x9ed576a3, 55 },
    { 0x594576a4, 90 },
    { 0x1576a1, 33 },
    { 0xe4c576a6, 91 },
    { 0x238576a7, 98 },
    { 0xe0f576a8, 36 },
    { 0x375576a8, 34 },
    { 0, 0 },
    { 0xb61576ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82576ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb576b8, 85 },
    { 0x441576b9, 23 },
    { 0x949576b9, 55 },
    { 0x80a576b8, 90 },
    { 0x359576ba, 61 },
    { 0x6c7576b8, 82 },
    { 0xb6d576be, 46 },
    { 0x844576bf, 55 },
    { 0x104576bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890576c4, 36 },
    { 0x61b576c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3576ca, 90 },
    { 0x605576cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f576ce, 81 },
    { 0xebf576cf, 99 },
    { 0x774576d0, 98 },
    { 0, 0 },
    { 0xf31576d2, 85 },
    { 0, 0 },
    { 0xd1576d4, 98 },
    { 0x562576d5, 69 },
    { 0x155576d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b576d9, 65 },
    { 0, 0 },
    { 0x5f7576db, 40 },
    { 0x2dd576dc, 33 },
    { 0xba4576dd, 33 },
    { 0x9eb576db, 69 },
    { 0x6f576dd, 90 },
    { 0xa9f576dd, 99 },
    { 0x495576e0, 90 },
    { 0x660576dd, 48 },
    { 0xb95576db, 76 },
    { 0, 0 },
    { 0xc9a576e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x487576e8, 52 },
    { 0xd5a576e9, 90 },
    { 0xaf6576e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc576ed, 99 },
    { 0xb50576ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730576f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63576f8, 90 },
    { 0x198576f9, 76 },
    { 0xc13576fa, 95 },
    { 0xf8576fb, 23 },
    { 0xa0f576f8, 98 },
    { 0x831576f8, 33 },
    { 0, 0 },
    { 0x42f576ff, 95 },
    { 0, 0 },
    { 0xcc857701, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93457705, 98 },
    { 0x7c557706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf545770d, 95 },
    { 0x3dd5770d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf657714, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e57717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8035771b, 59 },
    { 0x2fa5771c, 90 },
    { 0xabe5771d, 52 },
    { 0x8d85771b, 93 },
    { 0x94f5771b, 23 },
    { 0, 0 },
    { 0xdf657721, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45157725, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70957729, 90 },
    { 0, 0 },
    { 0x9345772b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be5772f, 95 },
    { 0xa835772f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e25773b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7225773e, 33 },
    { 0x5165773f, 61 },
    { 0x6cf5773f, 61 },
    { 0x10a57740, 69 },
    { 0xb8957742, 95 },
    { 0xbba5773f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba357747, 98 },
    { 0, 0 },
    { 0x7f757749, 78 },
    { 0, 0 },
    { 0x3665774b, 76 },
    { 0x89b5774b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf035774f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94457752, 40 },
    { 0, 0 },
    { 0xe9e57754, 33 },
    { 0x83f57754, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2557760, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5957764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d57769, 46 },
    { 0x8ac57769, 94 },
    { 0xe805776b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61457770, 33 },
    { 0, 0 },
    { 0x74357772, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f57775, 101 },
    { 0xe0d57776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66957779, 98 },
    { 0xca05777a, 33 },
    { 0, 0 },
    { 0xba05777c, 98 },
    { 0, 0 },
    { 0x315777e, 76 },
    { 0xbde5777f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d57785, 67 },
    { 0x79d57786, 100 },
    { 0xff857786, 76 },
    { 0, 0 },
    { 0xb3f57789, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16057790, 95 },
    { 0x61357790, 66 },
    { 0, 0 },
    { 0xf2257793, 55 },
    { 0, 0 },
    { 0xe7c57795, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f5779b, 23 },
    { 0xb285779c, 55 },
    { 0x6e05779d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79577a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756577ab, 34 },
    { 0, 0 },
    { 0x9d2577ad, 90 },
    { 0x6f1577ae, 42 },
    { 0x22577ad, 35 },
    { 0xf1a577b0, 33 },
    { 0x360577b1, 33 },
    { 0x720577b1, 84 },
    { 0x1de577b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a577b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52577c1, 34 },
    { 0, 0 },
    { 0x65e577c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0577cc, 33 },
    { 0x9a7577cd, 91 },
    { 0x6c8577ce, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c577d2, 98 },
    { 0xc8b577d3, 28 },
    { 0, 0 },
    { 0xfb6577d5, 90 },
    { 0xbb3577d6, 98 },
    { 0x345577d7, 61 },
    { 0x2f4577d8, 33 },
    { 0x837577d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e577dc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e577e0, 100 },
    { 0, 0 },
    { 0xd08577e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6577ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d577f0, 90 },
    { 0xbd5577f0, 95 },
    { 0x3a4577f2, 98 },
    { 0x200577f3, 95 },
    { 0x8de577f3, 98 },
    { 0, 0 },
    { 0xf9f577f6, 23 },
    { 0x4d6577f7, 55 },
    { 0xd21577f7, 23 },
    { 0xe00577f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac577fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e57805, 84 },
    { 0x2f757805, 70 },
    { 0xe457807, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3345780c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f857810, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e57819, 67 },
    { 0x2735781a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455781d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a657820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6257824, 88 },
    { 0xfcc57824, 90 },
    { 0xe2a57825, 66 },
    { 0x1ee57827, 23 },
    { 0x6b257828, 90 },
    { 0xde557824, 90 },
    { 0, 0 },
    { 0x7355782b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c57830, 87 },
    { 0, 0 },
    { 0x1057832, 91 },
    { 0, 0 },
    { 0x37457834, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d57837, 90 },
    { 0xa8a57838, 52 },
    { 0x3f257837, 43 },
    { 0x1bf5783a, 90 },
    { 0, 0 },
    { 0x8cf5783c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec57841, 98 },
    { 0x25c57842, 55 },
    { 0x8c557843, 85 },
    { 0x6ef57843, 99 },
    { 0x5557845, 77 },
    { 0, 0 },
    { 0xac757847, 61 },
    { 0x9ea57848, 40 },
    { 0xf8b57847, 55 },
    { 0xd7557848, 61 },
    { 0xad55784b, 90 },
    { 0x4df5784b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f57850, 33 },
    { 0x72757851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe645785f, 100 },
    { 0x7f957860, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd957863, 17 },
    { 0x93d57864, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a357867, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795786a, 68 },
    { 0, 0 },
    { 0x7835786c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa457872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c157878, 34 },
    { 0xd6c57878, 33 },
    { 0x6885787a, 95 },
    { 0xdd65787b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1357883, 61 },
    { 0xccb57884, 23 },
    { 0x53357885, 60 },
    { 0xee957885, 95 },
    { 0xce057884, 93 },
    { 0x4d557888, 17 },
    { 0x69a57888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d5788d, 90 },
    { 0x42d5788e, 52 },
    { 0, 0 },
    { 0xabf57890, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8357895, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d5789c, 38 },
    { 0xdbd5789d, 88 },
    { 0xe495789d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb578a1, 23 },
    { 0x66578a2, 67 },
    { 0xa12578a1, 90 },
    { 0x211578a4, 66 },
    { 0x815578a4, 23 },
    { 0x9e7578a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc578a9, 100 },
    { 0, 0 },
    { 0xd67578ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91578af, 61 },
    { 0x8cf578af, 56 },
    { 0x41e578b1, 33 },
    { 0x762578b2, 85 },
    { 0x6af578b3, 33 },
    { 0xe93578b4, 68 },
    { 0x513578b0, 33 },
    { 0, 0 },
    { 0x4d6578b7, 95 },
    { 0, 0 },
    { 0xf9f578b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b578c3, 90 },
    { 0x403578c4, 95 },
    { 0x965578c5, 33 },
    { 0x63578c6, 95 },
    { 0, 0 },
    { 0x3f5578c8, 95 },
    { 0xbcc578c9, 33 },
    { 0xd02578ca, 34 },
    { 0x6c3578cb, 90 },
    { 0x211578cc, 98 },
    { 0xcc1578cc, 93 },
    { 0x486578c8, 78 },
    { 0xb21578ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29578d2, 23 },
    { 0, 0 },
    { 0x687578d4, 33 },
    { 0x881578d5, 90 },
    { 0, 0 },
    { 0x587578d7, 91 },
    { 0x121578d7, 87 },
    { 0, 0 },
    { 0x8a578da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56578df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc578e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7578f1, 90 },
    { 0x1fd578f2, 23 },
    { 0x3bd578f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28578fc, 91 },
    { 0xfba578fd, 33 },
    { 0xae2578fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9357906, 84 },
    { 0, 0 },
    { 0x7f857908, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b85790e, 66 },
    { 0x6465790f, 61 },
    { 0xe285790e, 90 },
    { 0x4fe57910, 55 },
    { 0x34d5790e, 61 },
    { 0xff257913, 33 },
    { 0x2b057913, 43 },
    { 0xcf757915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8457919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd57920, 97 },
    { 0xbef57921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79057928, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc5792c, 84 },
    { 0, 0 },
    { 0x2795792e, 96 },
    { 0, 0 },
    { 0x36857930, 53 },
    { 0x1ad57931, 55 },
    { 0x1d857930, 99 },
    { 0x88957932, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed57936, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d57939, 61 },
    { 0x4db5793a, 87 },
    { 0x2d95793a, 81 },
    { 0x69f5793c, 90 },
    { 0x65c5793d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f57941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2657944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe135794b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd657951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e57957, 61 },
    { 0, 0 },
    { 0x6bc57959, 46 },
    { 0xcf5795a, 77 },
    { 0x80b5795b, 100 },
    { 0xcea57959, 94 },
    { 0, 0 },
    { 0xc525795e, 85 },
    { 0, 0 },
    { 0xd7b57960, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b57964, 95 },
    { 0, 0 },
    { 0x43657966, 90 },
    { 0x65857966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd655796d, 61 },
    { 0x2dd5796d, 78 },
    { 0, 0 },
    { 0x8b857970, 95 },
    { 0, 0 },
    { 0xa8757972, 90 },
    { 0, 0 },
    { 0x47b57974, 76 },
    { 0x58b57975, 23 },
    { 0xf3657976, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0657979, 92 },
    { 0xe165797a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a5797d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c557982, 33 },
    { 0x99257982, 90 },
    { 0, 0 },
    { 0xe1257985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b57989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5798c, 67 },
    { 0, 0 },
    { 0xc8a5798e, 66 },
    { 0, 0 },
    { 0x97f57990, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e857993, 84 },
    { 0x85557993, 33 },
    { 0x8cc57995, 61 },
    { 0x6e157996, 61 },
    { 0x2b357997, 98 },
    { 0xa0157994, 61 },
    { 0x13057993, 95 },
    { 0x41a57997, 33 },
    { 0xdef5799b, 33 },
    { 0, 0 },
    { 0x5515799d, 23 },
    { 0xa655799e, 53 },
    { 0xefc5799e, 93 },
    { 0x562579a0, 68 },
    { 0xc815799f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda579a6, 46 },
    { 0x32f579a7, 43 },
    { 0x46a579a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20579ae, 43 },
    { 0xfdd579af, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd579b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e579bc, 61 },
    { 0, 0 },
    { 0xf56579be, 91 },
    { 0x518579bf, 85 },
    { 0, 0 },
    { 0x876579c1, 90 },
    { 0, 0 },
    { 0x572579c3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4579c6, 90 },
    { 0, 0 },
    { 0x86c579c8, 99 },
    { 0xcf9579c8, 43 },
    { 0, 0 },
    { 0xb2a579cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c579ce, 91 },
    { 0xd36579cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659579da, 85 },
    { 0xecd579db, 87 },
    { 0x87f579db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c579df, 61 },
    { 0x285579e0, 90 },
    { 0, 0 },
    { 0x8d579e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509579e6, 98 },
    { 0, 0 },
    { 0x2a8579e8, 81 },
    { 0x982579e9, 93 },
    { 0x91579e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc579ee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66579f3, 33 },
    { 0x81f579f4, 33 },
    { 0xfa8579f4, 43 },
    { 0, 0 },
    { 0xe72579f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b579fa, 61 },
    { 0, 0 },
    { 0x2aa579fc, 77 },
    { 0xf20579fd, 55 },
    { 0x36579fe, 90 },
    { 0, 0 },
    { 0x4db57a00, 78 },
    { 0, 0 },
    { 0x24257a02, 91 },
    { 0xa5357a03, 23 },
    { 0x56e57a04, 4 },
    { 0, 0 },
    { 0xdf257a06, 23 },
    { 0xff157a07, 90 },
    { 0xfac57a08, 33 },
    { 0x2ab57a09, 95 },
    { 0x10b57a09, 100 },
    { 0x6257a08, 23 },
    { 0x20c57a0c, 99 },
    { 0xf157a0c, 98 },
    { 0x11b57a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22257a1c, 33 },
    { 0x14757a1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d757a21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a57a24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa57a2d, 90 },
    { 0x9e957a2e, 34 },
    { 0x5f857a2e, 43 },
    { 0xf2557a30, 56 },
    { 0xfbf57a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d57a34, 33 },
    { 0, 0 },
    { 0x7b557a36, 95 },
    { 0x36d57a36, 100 },
    { 0xca57a38, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b57a40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6557a45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c057a49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f57a51, 46 },
    { 0xc7857a52, 91 },
    { 0, 0 },
    { 0x4c057a54, 61 },
    { 0xae57a55, 76 },
    { 0xe5857a56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3357a5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a57a60, 33 },
    { 0, 0 },
    { 0xa8a57a62, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68657a6b, 61 },
    { 0x18b57a6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1157a70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f57a7b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x94957a7e, 95 },
    { 0, 0 },
    { 0x56057a80, 95 },
    { 0xf8357a81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62257a87, 100 },
    { 0, 0 },
    { 0x76e57a89, 23 },
    { 0xce857a8a, 90 },
    { 0, 0 },
    { 0xd7857a8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f57a90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92957a93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d57a98, 34 },
    { 0xf2257a99, 43 },
    { 0xf0257a98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d757a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93057aa4, 66 },
    { 0x31757aa5, 91 },
    { 0, 0 },
    { 0x6cb57aa7, 98 },
    { 0xb8757aa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc57aad, 61 },
    { 0x71757aae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a57ab1, 95 },
    { 0, 0 },
    { 0x68957ab3, 90 },
    { 0, 0 },
    { 0xb9557ab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6857abc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca57abf, 67 },
    { 0x1e57ac0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0057ac3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9157ac8, 43 },
    { 0xf5757ac9, 98 },
    { 0xca857ac9, 75 },
    { 0xe457acb, 61 },
    { 0x47e57acb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d57ad0, 93 },
    { 0x34057ad0, 61 },
    { 0xf0d57ad2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e57ad5, 90 },
    { 0, 0 },
    { 0xffc57ad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed357ae3, 98 },
    { 0x19e57ae4, 93 },
    { 0x2f157ae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3157aea, 97 },
    { 0xcfa57aeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37757aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9357af3, 61 },
    { 0x28057af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9157afb, 46 },
    { 0, 0 },
    { 0x5f757afd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe257b01, 77 },
    { 0xb4e57b02, 46 },
    { 0x32a57b03, 55 },
    { 0, 0 },
    { 0xe4257b05, 34 },
    { 0x35e57b05, 98 },
    { 0x82d57b07, 61 },
    { 0, 0 },
    { 0xb3157b09, 95 },
    { 0x93157b09, 90 },
    { 0x4057b09, 61 },
    { 0x9657b0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea57b11, 98 },
    { 0x6e857b11, 90 },
    { 0x3f357b13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d57b17, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7957b1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98757b22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c57b2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84957b2e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c57b34, 90 },
    { 0x86857b34, 33 },
    { 0x82257b34, 33 },
    { 0x77f57b37, 99 },
    { 0, 0 },
    { 0x80457b39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2157b3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac57b3f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73557b43, 61 },
    { 0, 0 },
    { 0x1c57b45, 19 },
    { 0x10e57b45, 100 },
    { 0x52557b47, 38 },
    { 0, 0 },
    { 0x7ec57b49, 80 },
    { 0x4a057b49, 40 },
    { 0xf0857b49, 33 },
    { 0xa9857b49, 69 },
    { 0x7ba57b4b, 95 },
    { 0xac757b4e, 95 },
    { 0x82557b4e, 33 },
    { 0xfa157b50, 61 },
    { 0xfee57b50, 87 },
    { 0x20557b4c, 61 },
    { 0xc2857b53, 95 },
    { 0, 0 },
    { 0xc9157b55, 61 },
    { 0x62757b56, 61 },
    { 0xf3657b56, 90 },
    { 0x82457b56, 95 },
    { 0x20457b59, 93 },
    { 0x23e57b57, 76 },
    { 0xede57b5b, 76 },
    { 0x5fd57b5b, 43 },
    { 0x19357b5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56257b61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c57b64, 95 },
    { 0xe4f57b65, 76 },
    { 0xaf857b64, 52 },
    { 0, 0 },
    { 0xa6a57b68, 98 },
    { 0xacd57b69, 23 },
    { 0x98d57b6a, 66 },
    { 0x5ac57b6a, 91 },
    { 0xf0557b6b, 98 },
    { 0xcc557b6d, 46 },
    { 0x9b557b69, 61 },
    { 0xe2c57b69, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3057b72, 61 },
    { 0x9db57b73, 98 },
    { 0x94657b74, 52 },
    { 0x66257b72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4257b78, 78 },
    { 0, 0 },
    { 0x69157b7a, 90 },
    { 0, 0 },
    { 0x80b57b7c, 61 },
    { 0, 0 },
    { 0x3c257b7e, 34 },
    { 0xe6e57b7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24357b84, 95 },
    { 0xf0a57b84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf57b89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8657b8c, 34 },
    { 0, 0 },
    { 0xc3757b8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb357b98, 61 },
    { 0, 0 },
    { 0x25757b9a, 95 },
    { 0, 0 },
    { 0x29357b9c, 90 },
    { 0xf8f57b9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e857ba2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc57ba6, 98 },
    { 0xebb57ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5557bab, 55 },
    { 0, 0 },
    { 0x3fa57bad, 51 },
    { 0x66457bae, 95 },
    { 0x3757baf, 90 },
    { 0x52957baf, 90 },
    { 0xdd657baf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c657bb5, 90 },
    { 0, 0 },
    { 0x96357bb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad57bba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c57bbf, 90 },
    { 0, 0 },
    { 0xba757bc1, 23 },
    { 0x38357bc1, 95 },
    { 0x25057bc1, 84 },
    { 0xa2257bc2, 34 },
    { 0, 0 },
    { 0x9d157bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d57bc9, 33 },
    { 0x17a57bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac57bd0, 90 },
    { 0xd8757bd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe57bd4, 61 },
    { 0x98c57bd5, 61 },
    { 0x44157bd5, 43 },
    { 0x1b557bd4, 23 },
    { 0x28157bd8, 33 },
    { 0xa8757bd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56257be3, 36 },
    { 0x8bf57be4, 83 },
    { 0x19657be3, 23 },
    { 0xc2c57be6, 84 },
    { 0, 0 },
    { 0x57757be8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf57beb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a57bf1, 55 },
    { 0x4f357bf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc757c01, 33 },
    { 0x46557c02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f57c06, 98 },
    { 0xd9957c07, 84 },
    { 0, 0 },
    { 0x5e57c09, 81 },
    { 0x78657c0a, 59 },
    { 0, 0 },
    { 0xc9057c0c, 93 },
    { 0, 0 },
    { 0x93257c0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb257c15, 90 },
    { 0x21e57c16, 90 },
    { 0x70057c17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac57c1a, 33 },
    { 0, 0 },
    { 0xec857c1c, 52 },
    { 0xfe357c1d, 68 },
    { 0, 0 },
    { 0xac457c1f, 95 },
    { 0, 0 },
    { 0x3c357c21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91357c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4457c31, 66 },
    { 0x3c657c32, 33 },
    { 0x99d57c33, 98 },
    { 0xd8757c34, 90 },
    { 0xf9257c34, 80 },
    { 0xb2857c34, 90 },
    { 0xf0357c37, 40 },
    { 0x13657c37, 66 },
    { 0x41657c37, 23 },
    { 0, 0 },
    { 0xa57c3b, 91 },
    { 0xa9657c3b, 40 },
    { 0x1b057c3d, 52 },
    { 0xd357c3d, 76 },
    { 0x17857c3d, 23 },
    { 0x93e57c40, 66 },
    { 0xba257c40, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae57c44, 81 },
    { 0, 0 },
    { 0xf5f57c46, 33 },
    { 0, 0 },
    { 0xf257c48, 87 },
    { 0x88057c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc57c4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e57c53, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54257c57, 61 },
    { 0xf5b57c57, 56 },
    { 0x33757c59, 90 },
    { 0, 0 },
    { 0xe8157c5b, 75 },
    { 0xaa157c5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0157c5f, 99 },
    { 0xc5b57c60, 61 },
    { 0x10557c60, 33 },
    { 0x4ed57c60, 95 },
    { 0x14357c63, 78 },
    { 0xb6357c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50557c69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd757c70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b57c73, 90 },
    { 0, 0 },
    { 0x8f857c75, 93 },
    { 0xc5357c76, 90 },
    { 0, 0 },
    { 0xe0257c78, 61 },
    { 0xd0f57c79, 98 },
    { 0x52457c79, 23 },
    { 0x8cd57c78, 85 },
    { 0x77c57c7c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xea957c7f, 99 },
    { 0x6b157c80, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0157c83, 61 },
    { 0x67f57c84, 90 },
    { 0xdac57c83, 55 },
    { 0xdde57c86, 95 },
    { 0xed457c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf457c8c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31957c92, 52 },
    { 0xfd157c93, 97 },
    { 0, 0 },
    { 0xbbd57c95, 33 },
    { 0xa7657c96, 93 },
    { 0x25c57c97, 23 },
    { 0x1e457c98, 61 },
    { 0xa6b57c99, 90 },
    { 0xc7657c98, 76 },
    { 0x60057c97, 61 },
    { 0, 0 },
    { 0x9cc57c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13557ca2, 43 },
    { 0xce657ca3, 99 },
    { 0x64357ca4, 95 },
    { 0xd9357ca5, 90 },
    { 0x90c57ca6, 52 },
    { 0x7cd57ca6, 84 },
    { 0, 0 },
    { 0x1d557ca9, 33 },
    { 0x85e57ca9, 98 },
    { 0x89e57cab, 33 },
    { 0x31c57cac, 88 },
    { 0x5ed57cad, 91 },
    { 0xe4257cab, 95 },
    { 0x32e57caf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd057cb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e57cb8, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55457cd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48257cd6, 90 },
    { 0, 0 },
    { 0x7bd57cd8, 52 },
    { 0xdee57cd9, 61 },
    { 0xc8e57cd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb057cde, 90 },
    { 0x3c757cde, 95 },
    { 0, 0 },
    { 0xe2d57ce1, 23 },
    { 0, 0 },
    { 0x53857ce3, 61 },
    { 0x1557ce3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9057ce8, 90 },
    { 0x48657ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x60457cec, 61 },
    { 0x7857ced, 100 },
    { 0x89457cee, 90 },
    { 0x5d357cef, 61 },
    { 0xdf457cf0, 23 },
    { 0, 0 },
    { 0x6e957cf2, 90 },
    { 0xe557cf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1657cf6, 84 },
    { 0, 0 },
    { 0x38b57cf8, 90 },
    { 0, 0 },
    { 0x17a57cfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b57cff, 41 },
    { 0, 0 },
    { 0xfa757d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99257d05, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1457d13, 46 },
    { 0x81157d14, 3 },
    { 0x8e57d15, 52 },
    { 0, 0 },
    { 0x45f57d17, 98 },
    { 0xfa757d18, 87 },
    { 0xbed57d18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad57d1e, 98 },
    { 0x28057d1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x93157d22, 87 },
    { 0, 0 },
    { 0x5e157d24, 76 },
    { 0x4e157d25, 43 },
    { 0, 0 },
    { 0x12157d27, 34 },
    { 0, 0 },
    { 0xc6f57d29, 43 },
    { 0, 0 },
    { 0xd3d57d2b, 76 },
    { 0x8b257d2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b57d31, 66 },
    { 0, 0 },
    { 0x65757d33, 34 },
    { 0, 0 },
    { 0x19f57d35, 65 },
    { 0xbeb57d36, 100 },
    { 0xfd857d37, 85 },
    { 0xad857d38, 23 },
    { 0xe5657d35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32357d43, 87 },
    { 0x40557d44, 98 },
    { 0x14557d45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb57d48, 19 },
    { 0x40e57d48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa857d4c, 61 },
    { 0, 0 },
    { 0x13c57d4e, 76 },
    { 0x8f57d4f, 34 },
    { 0x42557d50, 98 },
    { 0, 0 },
    { 0x6c857d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c57d56, 92 },
    { 0x54e57d56, 43 },
    { 0xbef57d58, 33 },
    { 0xb4957d59, 78 },
    { 0x88057d57, 43 },
    { 0x7057d5a, 61 },
    { 0xd6c57d59, 93 },
    { 0x11157d59, 33 },
    { 0x4ef57d5e, 90 },
    { 0xcd357d5c, 33 },
    { 0xd6357d60, 68 },
    { 0x9b057d61, 67 },
    { 0x8c357d61, 33 },
    { 0xb8b57d61, 95 },
    { 0xb4157d61, 34 },
    { 0x8ed57d65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57357d70, 61 },
    { 0xb2f57d71, 61 },
    { 0xc857d71, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39557d75, 61 },
    { 0xda357d76, 33 },
    { 0xfd757d77, 95 },
    { 0xa9457d77, 99 },
    { 0x3af57d79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9557d7c, 90 },
    { 0xba857d7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e57d85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c57d88, 55 },
    { 0, 0 },
    { 0xdfa57d8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c857d8d, 95 },
    { 0x3257d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16657d91, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d257d96, 84 },
    { 0xb8d57d97, 34 },
    { 0x9dc57d97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b57d9b, 98 },
    { 0, 0 },
    { 0x22757d9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a257da3, 33 },
    { 0xe9057da3, 90 },
    { 0x43857da3, 99 },
    { 0xe6b57da5, 33 },
    { 0xd257da3, 90 },
    { 0xaa157da8, 98 },
    { 0xaeb57da9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae57dac, 32 },
    { 0x3cb57dad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a57db0, 61 },
    { 0x7a857db1, 78 },
    { 0x10e57db2, 98 },
    { 0x3ec57db3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49557dba, 90 },
    { 0x40457dbb, 43 },
    { 0xb6957dbb, 84 },
    { 0x61457dbc, 98 },
    { 0x5a657dbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17157dc2, 76 },
    { 0xb2d57dc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b57dce, 33 },
    { 0, 0 },
    { 0x96b57dd0, 98 },
    { 0x49d57dd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49057dd4, 90 },
    { 0x55457dd4, 90 },
    { 0x4b357dd5, 76 },
    { 0xb9a57dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1757ddb, 66 },
    { 0x89357ddc, 88 },
    { 0x38c57ddc, 61 },
    { 0xbcf57dde, 43 },
    { 0x4c657ddf, 95 },
    { 0xe7f57ddf, 95 },
    { 0, 0 },
    { 0x32357de2, 90 },
    { 0, 0 },
    { 0xde957de4, 61 },
    { 0x35c57de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f57dea, 23 },
    { 0x49c57dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1057df0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d57df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98157df9, 90 },
    { 0xb5157df9, 100 },
    { 0, 0 },
    { 0xd4557dfc, 91 },
    { 0x98e57dfd, 33 },
    { 0x7bc57dfe, 76 },
    { 0x80a57dfe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x95557e02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd157e05, 56 },
    { 0, 0 },
    { 0x4ed57e07, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x78357e0a, 34 },
    { 0xebe57e0b, 34 },
    { 0, 0 },
    { 0x5d757e0d, 61 },
    { 0, 0 },
    { 0x35057e0f, 90 },
    { 0xd1557e10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9157e13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42457e1c, 23 },
    { 0x5aa57e1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b957e23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a057e28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8057e2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e57e31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x68757e34, 66 },
    { 0xe7657e35, 93 },
    { 0x5ab57e36, 84 },
    { 0xf8857e36, 33 },
    { 0x86457e35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f57e3d, 33 },
    { 0x1d357e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5757e41, 78 },
    { 0xbef57e41, 52 },
    { 0x19a57e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d457e49, 84 },
    { 0xf7457e49, 91 },
    { 0xa9157e4b, 90 },
    { 0, 0 },
    { 0x84e57e4d, 90 },
    { 0x25c57e4e, 23 },
    { 0x21457e4f, 93 },
    { 0xf3f57e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85957e54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94557e57, 68 },
    { 0, 0 },
    { 0xed357e59, 98 },
    { 0xa7f57e59, 34 },
    { 0, 0 },
    { 0x1657e5c, 78 },
    { 0x7cd57e5d, 90 },
    { 0x77657e5d, 34 },
    { 0xb4757e5f, 76 },
    { 0, 0 },
    { 0x64c57e61, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9057e69, 81 },
    { 0, 0 },
    { 0xecb57e6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71657e6e, 95 },
    { 0x79557e6f, 76 },
    { 0x5fc57e6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac57e73, 43 },
    { 0x6ad57e74, 55 },
    { 0x3d957e74, 23 },
    { 0xe1057e73, 33 },
    { 0x36757e77, 76 },
    { 0x8a357e78, 90 },
    { 0, 0 },
    { 0xc0e57e7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6957e7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5557e83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46457e87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18157e8a, 61 },
    { 0, 0 },
    { 0x6a357e8c, 91 },
    { 0x2b657e8c, 98 },
    { 0xdd657e8d, 61 },
    { 0x8e57e8f, 87 },
    { 0, 0 },
    { 0x77857e91, 61 },
    { 0x2e757e92, 52 },
    { 0xec957e92, 23 },
    { 0xd9757e92, 33 },
    { 0x3c257e91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5157e9c, 23 },
    { 0xcfa57e9d, 23 },
    { 0, 0 },
    { 0xef257e9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa57ea3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b257ea7, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54957eae, 90 },
    { 0x6f857eaf, 61 },
    { 0x78057eaf, 46 },
    { 0x8b257eb0, 33 },
    { 0x21457eb2, 40 },
    { 0xc4257eb0, 90 },
    { 0x7ef57eb0, 43 },
    { 0x5c057eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e57ec4, 52 },
    { 0x40e57ec5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8557ec8, 43 },
    { 0xf057ec8, 99 },
    { 0x83b57eca, 90 },
    { 0x85657ecb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19757ed2, 95 },
    { 0, 0 },
    { 0x8f357ed4, 23 },
    { 0xf8c57ed4, 33 },
    { 0xf6b57ed4, 100 },
    { 0x1fa57ed7, 69 },
    { 0xd3e57ed8, 95 },
    { 0, 0 },
    { 0x5b057eda, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa157ede, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd457ee4, 91 },
    { 0x68e57ee5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657eea, 52 },
    { 0xf0257eeb, 100 },
    { 0, 0 },
    { 0xb857eed, 33 },
    { 0x58557eee, 55 },
    { 0, 0 },
    { 0xf1657ef0, 95 },
    { 0x23357ef1, 93 },
    { 0x1b157ef2, 61 },
    { 0, 0 },
    { 0xade57ef4, 95 },
    { 0x96d57ef5, 100 },
    { 0x81157ef6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x28657ef9, 61 },
    { 0, 0 },
    { 0x46357efb, 23 },
    { 0, 0 },
    { 0xb6b57efd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba57f0d, 43 },
    { 0x16957f0e, 44 },
    { 0, 0 },
    { 0x6b157f10, 88 },
    { 0xd3e57f11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca57f14, 91 },
    { 0x71857f14, 61 },
    { 0xda657f14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c57f1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c457f1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd857f25, 23 },
    { 0, 0 },
    { 0x7257f27, 76 },
    { 0, 0 },
    { 0x94157f29, 17 },
    { 0x24a57f2a, 33 },
    { 0x30257f2b, 23 },
    { 0x59257f2b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd257f31, 43 },
    { 0x80c57f32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c57f35, 63 },
    { 0x2ac57f36, 33 },
    { 0x46d57f37, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27157f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb957f3f, 43 },
    { 0x9b157f40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8157f43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2757f46, 84 },
    { 0xd1c57f46, 90 },
    { 0xa7a57f48, 93 },
    { 0xff857f48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce57f51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a57f55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e57f5a, 61 },
    { 0x28257f5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea57f60, 55 },
    { 0x8e357f61, 64 },
    { 0x53a57f62, 95 },
    { 0x98257f63, 36 },
    { 0xff057f64, 61 },
    { 0xd9e57f63, 33 },
    { 0, 0 },
    { 0x72a57f67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66257f6d, 42 },
    { 0, 0 },
    { 0xd7357f6f, 23 },
    { 0, 0 },
    { 0x6757f71, 23 },
    { 0, 0 },
    { 0xcc057f73, 33 },
    { 0, 0 },
    { 0x94357f75, 94 },
    { 0, 0 },
    { 0x85b57f77, 95 },
    { 0, 0 },
    { 0xe3157f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda057f7c, 99 },
    { 0xfc357f7c, 23 },
    { 0, 0 },
    { 0x80b57f7f, 23 },
    { 0, 0 },
    { 0xc5157f81, 23 },
    { 0x4f57f82, 95 },
    { 0xce057f82, 23 },
    { 0xd6157f84, 50 },
    { 0x87f57f84, 93 },
    { 0x4b257f86, 100 },
    { 0x7257f87, 98 },
    { 0xc7957f85, 55 },
    { 0x59957f89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c657f8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2757f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa57f98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xba57f9b, 23 },
    { 0xbd257f9c, 61 },
    { 0x51757f9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4057fa3, 90 },
    { 0x1a157fa4, 61 },
    { 0x7f657fa5, 95 },
    { 0, 0 },
    { 0xcdd57fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9457faa, 95 },
    { 0x3e057fab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7857fb2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8657fb5, 55 },
    { 0, 0 },
    { 0x9f657fb7, 90 },
    { 0x2e057fb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c57fbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af57fbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb657fc1, 75 },
    { 0, 0 },
    { 0x6c457fc3, 61 },
    { 0, 0 },
    { 0xbd57fc5, 33 },
    { 0x97a57fc6, 23 },
    { 0, 0 },
    { 0x53457fc8, 95 },
    { 0, 0 },
    { 0xf6457fca, 95 },
    { 0x7257fca, 55 },
    { 0xb0f57fcc, 61 },
    { 0xffa57fcb, 95 },
    { 0, 0 },
    { 0x13057fcf, 64 },
    { 0x8f657fd0, 23 },
    { 0xa9c57fd0, 43 },
    { 0, 0 },
    { 0x43557fd3, 61 },
    { 0xd5457fd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5157fd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b57fdc, 80 },
    { 0, 0 },
    { 0xbf657fde, 90 },
    { 0, 0 },
    { 0xabd57fe0, 95 },
    { 0x74857fe0, 43 },
    { 0xf6657fe2, 61 },
    { 0x17557fe2, 61 },
    { 0xff557fe3, 52 },
    { 0, 0 },
    { 0x29257fe6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9257fea, 90 },
    { 0xb8257fea, 43 },
    { 0xb1057fea, 68 },
    { 0xaa757fea, 33 },
    { 0x5657fee, 95 },
    { 0, 0 },
    { 0x75c57ff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757ff4, 76 },
    { 0, 0 },
    { 0xb3657ff6, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b57ffa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe457ffe, 34 },
    { 0, 0 },
    { 0x76058000, 46 },
    { 0xcc158000, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc858004, 81 },
    { 0x99458004, 90 },
    { 0x3a858006, 23 },
    { 0, 0 },
    { 0x99558008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8855800d, 95 },
    { 0x3d95800d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3758011, 61 },
    { 0x12b58012, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa025801a, 46 },
    { 0x72d5801b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a5801e, 42 },
    { 0, 0 },
    { 0x84a58020, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e58024, 46 },
    { 0x87558025, 46 },
    { 0x49b58025, 32 },
    { 0x39458026, 87 },
    { 0x4ba58027, 95 },
    { 0xc3258026, 55 },
    { 0x1305802a, 23 },
    { 0xe7b5802b, 81 },
    { 0x89b58025, 33 },
    { 0, 0 },
    { 0xab55802e, 23 },
    { 0, 0 },
    { 0xaf58030, 98 },
    { 0x35058031, 66 },
    { 0, 0 },
    { 0x8458033, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a58039, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25803e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea58043, 93 },
    { 0x24458044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97158047, 95 },
    { 0, 0 },
    { 0xd5658049, 85 },
    { 0x4465804a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd5804e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a058053, 23 },
    { 0x10058054, 98 },
    { 0x41558054, 61 },
    { 0x44c58054, 90 },
    { 0x1d658054, 76 },
    { 0x27e58057, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded58061, 23 },
    { 0x16258062, 43 },
    { 0xf2f58061, 90 },
    { 0xb3358064, 78 },
    { 0, 0 },
    { 0xd3158066, 76 },
    { 0xc8958067, 67 },
    { 0x5b558068, 98 },
    { 0xba258068, 51 },
    { 0xef05806a, 93 },
    { 0x2bc5806b, 61 },
    { 0x9495806c, 76 },
    { 0x4035806b, 55 },
    { 0x59d58068, 95 },
    { 0xd855806f, 100 },
    { 0, 0 },
    { 0x98258071, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f58078, 90 },
    { 0x19a58079, 51 },
    { 0x7bc5807a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065807f, 33 },
    { 0, 0 },
    { 0x1a458081, 23 },
    { 0xdd058081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d58085, 90 },
    { 0x10458086, 76 },
    { 0x92658087, 91 },
    { 0x7fa58086, 95 },
    { 0x18e58089, 66 },
    { 0, 0 },
    { 0x9705808b, 95 },
    { 0xfca5808c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb5808f, 90 },
    { 0x3055808f, 33 },
    { 0xb115808f, 76 },
    { 0, 0 },
    { 0xe2b58093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0358096, 46 },
    { 0xd2658096, 33 },
    { 0xa6c58097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8925809d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5580a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7580aa, 55 },
    { 0, 0 },
    { 0x357580ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881580b0, 61 },
    { 0xbcd580b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x364580b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3580b8, 76 },
    { 0xf8580b9, 23 },
    { 0, 0 },
    { 0x4c2580bb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d580be, 61 },
    { 0x5b0580bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf580c5, 76 },
    { 0xab2580c6, 92 },
    { 0xcb4580c6, 34 },
    { 0, 0 },
    { 0xbc9580c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3580ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441580d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de580d5, 76 },
    { 0, 0 },
    { 0x272580d7, 98 },
    { 0x7d1580d8, 55 },
    { 0x9fb580d9, 76 },
    { 0, 0 },
    { 0x8fd580db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f580e6, 100 },
    { 0xf16580e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717580ec, 84 },
    { 0x5d4580ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53580f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28580f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df580f9, 99 },
    { 0xc7f580fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b58104, 98 },
    { 0x54758105, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa58109, 68 },
    { 0xe5658109, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5035810f, 95 },
    { 0x1fa58110, 23 },
    { 0x2d75810f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e58115, 4 },
    { 0x1d858115, 33 },
    { 0, 0 },
    { 0xc2c58118, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb5811f, 95 },
    { 0, 0 },
    { 0x7a958121, 33 },
    { 0xc2458122, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1775812b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a5812e, 98 },
    { 0, 0 },
    { 0x87b58130, 87 },
    { 0, 0 },
    { 0xd158132, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f58135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe385813b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc315813e, 99 },
    { 0x71f5813f, 87 },
    { 0x2a65813e, 99 },
    { 0, 0 },
    { 0x33e58142, 33 },
    { 0, 0 },
    { 0xbf958144, 66 },
    { 0x42e58145, 93 },
    { 0xd5e58146, 81 },
    { 0x50258147, 93 },
    { 0, 0 },
    { 0xd7258149, 76 },
    { 0, 0 },
    { 0xd55814b, 99 },
    { 0, 0 },
    { 0x7205814d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d58152, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef558156, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea58159, 52 },
    { 0, 0 },
    { 0xad65815b, 23 },
    { 0, 0 },
    { 0x44f5815d, 90 },
    { 0xe585815e, 59 },
    { 0x7dc5815d, 43 },
    { 0xf4558160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de58163, 95 },
    { 0x58258164, 90 },
    { 0, 0 },
    { 0x4d258166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5816b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a958174, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a58177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a5817a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67158182, 23 },
    { 0xfaa58183, 95 },
    { 0x4ee58184, 23 },
    { 0x71c58184, 55 },
    { 0x56658184, 90 },
    { 0, 0 },
    { 0x96358188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d55818b, 98 },
    { 0, 0 },
    { 0x6765818d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2958191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37958194, 33 },
    { 0x96158194, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb158198, 95 },
    { 0x61658199, 23 },
    { 0x9765819a, 90 },
    { 0, 0 },
    { 0x3365819c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771581a3, 61 },
    { 0x3b5581a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228581aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1581b1, 58 },
    { 0x1ba581b2, 95 },
    { 0xc8b581b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a581b6, 90 },
    { 0x4b0581b7, 98 },
    { 0, 0 },
    { 0x59a581b9, 46 },
    { 0xb41581b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1581c5, 46 },
    { 0, 0 },
    { 0x486581c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71581ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a581cd, 90 },
    { 0, 0 },
    { 0x80f581cf, 68 },
    { 0, 0 },
    { 0x45a581d1, 61 },
    { 0, 0 },
    { 0x3c8581d3, 90 },
    { 0xa6a581d4, 23 },
    { 0xfa581d4, 43 },
    { 0xab581d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4581db, 61 },
    { 0, 0 },
    { 0x7cb581dd, 33 },
    { 0, 0 },
    { 0x652581df, 90 },
    { 0xfe581df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b581e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162581ed, 76 },
    { 0x4fd581ed, 61 },
    { 0xb93581ee, 77 },
    { 0xb0581ed, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9581f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120581f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7581fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e58201, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e758209, 88 },
    { 0x2fa58209, 40 },
    { 0x6cd5820a, 33 },
    { 0x5705820c, 61 },
    { 0xcfa5820d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015821a, 46 },
    { 0x4455821a, 90 },
    { 0xac85821c, 92 },
    { 0x6305821b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96158224, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd58229, 90 },
    { 0xbb65822a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc445822e, 87 },
    { 0x9c65822e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f58232, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e658239, 91 },
    { 0, 0 },
    { 0x3e35823b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6558247, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5265824b, 23 },
    { 0, 0 },
    { 0x44f5824d, 95 },
    { 0, 0 },
    { 0xf745824f, 23 },
    { 0, 0 },
    { 0x72658251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3558255, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2585825e, 43 },
    { 0xf415825f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b58263, 95 },
    { 0x97258263, 93 },
    { 0, 0 },
    { 0xd158266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c558269, 23 },
    { 0x1ce5826a, 90 },
    { 0, 0 },
    { 0x9545826c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c458270, 33 },
    { 0, 0 },
    { 0x59858272, 69 },
    { 0xc7058272, 100 },
    { 0, 0 },
    { 0xc7458275, 23 },
    { 0, 0 },
    { 0x72858277, 95 },
    { 0, 0 },
    { 0x57358279, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b5827e, 90 },
    { 0, 0 },
    { 0x58058280, 81 },
    { 0xc0658281, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3258289, 90 },
    { 0, 0 },
    { 0x2405828b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d85828f, 76 },
    { 0, 0 },
    { 0x5a258291, 90 },
    { 0, 0 },
    { 0xc5458293, 61 },
    { 0, 0 },
    { 0xf6a58295, 43 },
    { 0xa4058295, 91 },
    { 0xa5058296, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5582a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277582a5, 95 },
    { 0, 0 },
    { 0x80e582a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d582aa, 76 },
    { 0, 0 },
    { 0x1f3582ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528582b2, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370582ba, 34 },
    { 0x24b582bb, 34 },
    { 0, 0 },
    { 0x8e7582bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e582c3, 46 },
    { 0, 0 },
    { 0x5f2582c5, 40 },
    { 0xee8582c6, 23 },
    { 0xd16582c7, 93 },
    { 0xbca582c8, 95 },
    { 0x40b582c9, 90 },
    { 0x94e582c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0582ce, 87 },
    { 0x173582cf, 91 },
    { 0, 0 },
    { 0xdc7582d1, 78 },
    { 0x4e0582d1, 33 },
    { 0, 0 },
    { 0x92a582d4, 33 },
    { 0, 0 },
    { 0x37e582d6, 91 },
    { 0, 0 },
    { 0x7f4582d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x701582db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411582e0, 100 },
    { 0xb56582e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x627582e4, 33 },
    { 0xc6e582e4, 90 },
    { 0x6c8582e6, 61 },
    { 0, 0 },
    { 0x6d0582e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31582ec, 55 },
    { 0xa68582ed, 52 },
    { 0x39c582ed, 23 },
    { 0, 0 },
    { 0x37c582f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c582f6, 83 },
    { 0x4f582f6, 95 },
    { 0, 0 },
    { 0x648582f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a582fd, 23 },
    { 0xff4582fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4758301, 23 },
    { 0, 0 },
    { 0x60158303, 61 },
    { 0x4bb58304, 84 },
    { 0, 0 },
    { 0x8ef58306, 76 },
    { 0x65658307, 55 },
    { 0x95d58308, 90 },
    { 0xe8858309, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c5830c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x505830f, 23 },
    { 0x9d358310, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a258314, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17058317, 88 },
    { 0x30558317, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd85831e, 95 },
    { 0x86d5831f, 23 },
    { 0x8045831f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc58323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c758327, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade5832c, 95 },
    { 0x5a15832d, 61 },
    { 0x9fc5832c, 34 },
    { 0, 0 },
    { 0x86058330, 66 },
    { 0x19858331, 95 },
    { 0, 0 },
    { 0x70458333, 34 },
    { 0xae658334, 23 },
    { 0xc758335, 61 },
    { 0xdfe58336, 61 },
    { 0x70158334, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff55833c, 33 },
    { 0, 0 },
    { 0x60b5833e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e58341, 90 },
    { 0, 0 },
    { 0x3258343, 61 },
    { 0, 0 },
    { 0xad958345, 33 },
    { 0x81f58346, 55 },
    { 0x16658347, 23 },
    { 0x10b58347, 52 },
    { 0xbb58345, 99 },
    { 0, 0 },
    { 0xe735834b, 65 },
    { 0xd515834c, 76 },
    { 0, 0 },
    { 0xbfc5834e, 61 },
    { 0xcec5834f, 90 },
    { 0xdcb5834e, 95 },
    { 0, 0 },
    { 0xbd158352, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d58359, 34 },
    { 0xcf5835a, 55 },
    { 0, 0 },
    { 0xbac5835c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4345835f, 81 },
    { 0xe5c58360, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e5836f, 90 },
    { 0xd3158370, 98 },
    { 0x35b5836f, 66 },
    { 0x8de58372, 84 },
    { 0x14e58373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af58377, 76 },
    { 0, 0 },
    { 0x63f58379, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c5837c, 66 },
    { 0xb145837d, 90 },
    { 0x7995837c, 55 },
    { 0x85c5837c, 34 },
    { 0x42e58380, 98 },
    { 0x8af58380, 38 },
    { 0, 0 },
    { 0x69f58383, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005838a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad58391, 90 },
    { 0, 0 },
    { 0x89058393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50583a1, 100 },
    { 0, 0 },
    { 0xf95583a3, 85 },
    { 0x14a583a4, 95 },
    { 0x4e2583a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f583a8, 23 },
    { 0x602583a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7583ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x457583af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c583b2, 61 },
    { 0, 0 },
    { 0xc55583b4, 23 },
    { 0, 0 },
    { 0xf9c583b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4583ba, 80 },
    { 0, 0 },
    { 0xf65583bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e583bf, 75 },
    { 0x82e583bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72583c5, 23 },
    { 0, 0 },
    { 0x9e8583c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137583cc, 98 },
    { 0xa91583cc, 90 },
    { 0x5fa583ce, 76 },
    { 0xb73583cf, 61 },
    { 0xb9a583d0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x236583d3, 23 },
    { 0x57e583d3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x499583d7, 35 },
    { 0, 0 },
    { 0x7c3583d9, 76 },
    { 0, 0 },
    { 0x468583db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d583e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7583e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6583ec, 69 },
    { 0xd3e583ec, 23 },
    { 0x358583ee, 68 },
    { 0x196583ee, 61 },
    { 0x5ec583ef, 93 },
    { 0x49b583f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422583f6, 95 },
    { 0, 0 },
    { 0xcb7583f8, 66 },
    { 0x435583f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1583ff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd58404, 53 },
    { 0xd8b58405, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1165840a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb458412, 34 },
    { 0xa858412, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79058416, 90 },
    { 0, 0 },
    { 0xddb58418, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x3125841b, 33 },
    { 0, 0 },
    { 0xc015841d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a558420, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6758426, 93 },
    { 0x32d58427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d58431, 76 },
    { 0xc4758431, 90 },
    { 0x94058433, 85 },
    { 0x22058433, 75 },
    { 0, 0 },
    { 0x9f158436, 90 },
    { 0x27958437, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15843a, 61 },
    { 0x3035843b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd5843e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf58445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c05844a, 46 },
    { 0xb8a5844b, 61 },
    { 0xc075844a, 76 },
    { 0x77b5844d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16458451, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb58454, 33 },
    { 0, 0 },
    { 0x7f858456, 52 },
    { 0xe9458456, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3215845a, 83 },
    { 0xb585845b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3058461, 3 },
    { 0, 0 },
    { 0x65f58463, 46 },
    { 0xfcd58464, 46 },
    { 0xbde58465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e358468, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5325846d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c358471, 33 },
    { 0xb5b58472, 95 },
    { 0, 0 },
    { 0x75a58474, 98 },
    { 0x7b858475, 33 },
    { 0x9f658476, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd25847e, 23 },
    { 0, 0 },
    { 0x2d158480, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f458488, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795848f, 23 },
    { 0, 0 },
    { 0x86b58491, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b858498, 61 },
    { 0, 0 },
    { 0x47c5849a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481584a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x712584a3, 61 },
    { 0xc13584a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f584af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc584b2, 43 },
    { 0xf32584b3, 33 },
    { 0x4de584b4, 87 },
    { 0, 0 },
    { 0x861584b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9584b9, 95 },
    { 0x376584b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c584be, 23 },
    { 0x976584bf, 98 },
    { 0x33b584bf, 33 },
    { 0, 0 },
    { 0xdb2584c2, 98 },
    { 0x5b9584c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274584c8, 91 },
    { 0, 0 },
    { 0x2d0584ca, 61 },
    { 0x1d0584ca, 95 },
    { 0xf4a584ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x921584cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07584d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a584d5, 43 },
    { 0x77e584d6, 61 },
    { 0, 0 },
    { 0xd4e584d8, 61 },
    { 0x32c584d9, 66 },
    { 0xd35584d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4584e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5584eb, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f584ee, 95 },
    { 0, 0 },
    { 0xec5584f0, 95 },
    { 0x3c3584f1, 34 },
    { 0xe01584f2, 90 },
    { 0, 0 },
    { 0x93584f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd584f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4584ff, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d158509, 98 },
    { 0x3875850a, 87 },
    { 0, 0 },
    { 0xa875850c, 90 },
    { 0, 0 },
    { 0x4e85850e, 84 },
    { 0x6185850f, 61 },
    { 0x62658510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85458518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7155851f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62458522, 40 },
    { 0x1e058523, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df58528, 90 },
    { 0x7d558529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b5852d, 99 },
    { 0x8405852e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97358534, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f158538, 90 },
    { 0, 0 },
    { 0xede5853a, 93 },
    { 0x3b35853b, 55 },
    { 0, 0 },
    { 0x97a5853d, 95 },
    { 0xe575853d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac58541, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf58544, 90 },
    { 0x9a058545, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd95854c, 84 },
    { 0x4005854d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958550, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab058554, 61 },
    { 0x4c358555, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2258559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa5855c, 43 },
    { 0, 0 },
    { 0xa5c5855e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16858565, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71358568, 84 },
    { 0xc9958569, 46 },
    { 0xaa858568, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd855856f, 98 },
    { 0x6ca5856f, 66 },
    { 0, 0 },
    { 0x3e358572, 90 },
    { 0, 0 },
    { 0xaa58574, 90 },
    { 0x22558575, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d58578, 61 },
    { 0x71458578, 75 },
    { 0, 0 },
    { 0xd115857b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8665857e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a858583, 61 },
    { 0x61e58584, 98 },
    { 0, 0 },
    { 0x3f758586, 33 },
    { 0, 0 },
    { 0x4d058588, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35158590, 52 },
    { 0xcaa58590, 95 },
    { 0, 0 },
    { 0x24058593, 100 },
    { 0x9ff58594, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44058597, 90 },
    { 0x6cc58598, 61 },
    { 0x47c58599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c15859c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971585a0, 98 },
    { 0x455585a0, 23 },
    { 0xbeb585a2, 33 },
    { 0xaa585a3, 61 },
    { 0x67a585a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c585a7, 66 },
    { 0x9d8585a7, 23 },
    { 0x672585a8, 61 },
    { 0x7df585aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b585b3, 33 },
    { 0x5ff585b3, 84 },
    { 0xeaf585b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da585b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16585bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x359585bf, 84 },
    { 0xcb3585c0, 33 },
    { 0x951585c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9585c4, 61 },
    { 0, 0 },
    { 0x96c585c6, 85 },
    { 0x630585c6, 67 },
    { 0x54585c7, 69 },
    { 0xaa1585c7, 66 },
    { 0xbad585ca, 90 },
    { 0, 0 },
    { 0xfe9585cc, 76 },
    { 0x6b0585cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25585d1, 23 },
    { 0xebc585d2, 33 },
    { 0, 0 },
    { 0xbe585d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x771585d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61585dc, 43 },
    { 0x93585dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7585e2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655585e6, 23 },
    { 0xec9585e7, 61 },
    { 0x867585e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8585ee, 36 },
    { 0, 0 },
    { 0xc16585f0, 88 },
    { 0xc22585f1, 90 },
    { 0x982585f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7585f8, 46 },
    { 0xdd5585f9, 33 },
    { 0x8d585fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b585fd, 90 },
    { 0x2be585fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58258602, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf58609, 55 },
    { 0x7695860a, 55 },
    { 0x5945860b, 66 },
    { 0x53158609, 23 },
    { 0x1495860d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b358610, 33 },
    { 0x8e58610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1565861d, 33 },
    { 0x64d5861e, 34 },
    { 0x76e5861d, 76 },
    { 0, 0 },
    { 0x29958621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab58629, 23 },
    { 0xacc5862a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac15862e, 76 },
    { 0x57c5862f, 85 },
    { 0x66958630, 95 },
    { 0x75c58631, 43 },
    { 0xab958632, 33 },
    { 0xae658633, 88 },
    { 0xfb45862f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77358639, 95 },
    { 0x3b55863a, 52 },
    { 0x5db5863b, 98 },
    { 0, 0 },
    { 0x2345863d, 52 },
    { 0x8885863e, 90 },
    { 0x2d5863d, 90 },
    { 0x775863d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8358644, 88 },
    { 0, 0 },
    { 0x63d58646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8335864b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9585864f, 52 },
    { 0x44e58650, 66 },
    { 0xd7858651, 76 },
    { 0, 0 },
    { 0x41258653, 61 },
    { 0xc6558654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f58659, 98 },
    { 0x1b858659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2835865f, 66 },
    { 0, 0 },
    { 0x6fb58661, 61 },
    { 0, 0 },
    { 0xcc358663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef58666, 93 },
    { 0x4ad58666, 33 },
    { 0x19458668, 95 },
    { 0, 0 },
    { 0xc505866a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e85866f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5758675, 22 },
    { 0xce258675, 55 },
    { 0xc6458676, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a25867a, 55 },
    { 0x4ba5867b, 33 },
    { 0x975867b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6295867f, 98 },
    { 0x47558680, 90 },
    { 0, 0 },
    { 0xc6e58682, 99 },
    { 0x62158683, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8958687, 76 },
    { 0x56358688, 90 },
    { 0xd9b58688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e5868c, 33 },
    { 0xf775868d, 98 },
    { 0x3985868e, 95 },
    { 0x7af5868f, 98 },
    { 0x40058690, 33 },
    { 0x11658691, 43 },
    { 0, 0 },
    { 0x3d958693, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf765869c, 90 },
    { 0x8475869d, 33 },
    { 0, 0 },
    { 0x2f85869f, 52 },
    { 0x195586a0, 52 },
    { 0xf15586a0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829586a5, 95 },
    { 0xe27586a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00586a9, 85 },
    { 0xf32586aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae586af, 90 },
    { 0, 0 },
    { 0xd0f586b1, 61 },
    { 0xace586b1, 66 },
    { 0x7e4586b1, 87 },
    { 0x557586b2, 98 },
    { 0xffc586b2, 52 },
    { 0x232586b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1586c6, 87 },
    { 0xde2586c6, 76 },
    { 0x2f1586c8, 98 },
    { 0, 0 },
    { 0x6bc586ca, 88 },
    { 0, 0 },
    { 0x5e1586cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96586d4, 33 },
    { 0xff9586d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c586d9, 77 },
    { 0xa8a586da, 34 },
    { 0xb93586da, 23 },
    { 0x38f586dc, 23 },
    { 0x385586dd, 61 },
    { 0x991586da, 93 },
    { 0xe04586dd, 100 },
    { 0xef1586df, 100 },
    { 0x254586e1, 90 },
    { 0, 0 },
    { 0xf17586e3, 33 },
    { 0x9e0586e4, 61 },
    { 0x353586e5, 90 },
    { 0xff4586e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7586e9, 90 },
    { 0, 0 },
    { 0xdc4586eb, 52 },
    { 0xd4f586eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb586f0, 43 },
    { 0x89e586f1, 95 },
    { 0x97f586f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7586f6, 76 },
    { 0, 0 },
    { 0xedd586f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25586fc, 76 },
    { 0xa8d586fc, 95 },
    { 0xa73586fe, 95 },
    { 0xbb9586ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86558702, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab58707, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc58710, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e58717, 95 },
    { 0, 0 },
    { 0xd9e58719, 61 },
    { 0xff65871a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e5871e, 46 },
    { 0x29d5871e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf158723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1658727, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f65872a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd75872e, 95 },
    { 0xc915872e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24158733, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1558736, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4205873b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3858741, 23 },
    { 0xbdc58741, 95 },
    { 0xde358743, 33 },
    { 0x60758744, 76 },
    { 0x9cc58745, 23 },
    { 0x2d658742, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d5874c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1565874f, 68 },
    { 0, 0 },
    { 0xb9f58751, 38 },
    { 0x4c358752, 33 },
    { 0x95958753, 95 },
    { 0, 0 },
    { 0xf2558755, 23 },
    { 0x29058756, 95 },
    { 0, 0 },
    { 0xc3558758, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e58761, 90 },
    { 0, 0 },
    { 0x82e58763, 67 },
    { 0xda358764, 67 },
    { 0x46458763, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f858769, 46 },
    { 0xb5f58769, 98 },
    { 0xc7a5876b, 23 },
    { 0x4645876c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca58770, 61 },
    { 0x50258771, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f58777, 61 },
    { 0xd9c58778, 90 },
    { 0xbd958778, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a15877c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7515877f, 55 },
    { 0xb9158780, 90 },
    { 0xa5958781, 98 },
    { 0x74958782, 95 },
    { 0xeda58780, 95 },
    { 0xc9c58784, 43 },
    { 0xdf058785, 76 },
    { 0x65758785, 90 },
    { 0xcc258787, 101 },
    { 0x7bb58788, 90 },
    { 0x55758789, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd885878e, 95 },
    { 0, 0 },
    { 0x82f58790, 83 },
    { 0xec958791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d58795, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9258799, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a5879c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc855879f, 61 },
    { 0x508587a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd587a9, 67 },
    { 0xf36587aa, 93 },
    { 0, 0 },
    { 0xc50587ac, 38 },
    { 0, 0 },
    { 0xe14587ae, 61 },
    { 0x767587af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce587b6, 95 },
    { 0xee7587b7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e587bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4587be, 90 },
    { 0x41b587bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b587c3, 61 },
    { 0xf8b587c4, 33 },
    { 0x252587c5, 90 },
    { 0x6bc587c6, 100 },
    { 0x293587c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8587cd, 91 },
    { 0x24a587cd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e587d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f587d7, 76 },
    { 0xef0587d8, 76 },
    { 0x3d9587d9, 23 },
    { 0x6be587d9, 87 },
    { 0, 0 },
    { 0x7c1587dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841587e2, 90 },
    { 0, 0 },
    { 0xd68587e4, 43 },
    { 0xf35587e5, 88 },
    { 0x488587e6, 36 },
    { 0x318587e7, 88 },
    { 0xa7c587e5, 43 },
    { 0, 0 },
    { 0x6fc587ea, 99 },
    { 0xa0a587eb, 61 },
    { 0x1a6587ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61587f1, 33 },
    { 0x24c587f2, 90 },
    { 0x362587f2, 34 },
    { 0x26e587f4, 95 },
    { 0x1c2587f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886587fb, 55 },
    { 0x80d587fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27587ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e58805, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec958809, 61 },
    { 0x41b58809, 61 },
    { 0x2595880b, 90 },
    { 0x3dc5880a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d458811, 3 },
    { 0x37458812, 90 },
    { 0xa6b58813, 33 },
    { 0xf7a58814, 87 },
    { 0xaee58812, 76 },
    { 0xaed58816, 43 },
    { 0, 0 },
    { 0x27c58818, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70958828, 34 },
    { 0x66d58829, 53 },
    { 0x2d85882a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365882d, 95 },
    { 0x37f5882e, 76 },
    { 0x8755882f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf558836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e58841, 33 },
    { 0, 0 },
    { 0x88d58843, 61 },
    { 0, 0 },
    { 0xf1b58845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db5884b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6558851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b358856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2675885a, 91 },
    { 0x6ca5885b, 96 },
    { 0, 0 },
    { 0xc865885d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f258864, 95 },
    { 0x7e58865, 76 },
    { 0xa0558866, 40 },
    { 0xe558866, 40 },
    { 0x38e58868, 34 },
    { 0, 0 },
    { 0x8ae5886a, 33 },
    { 0x98a5886a, 40 },
    { 0, 0 },
    { 0xe435886d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c58871, 66 },
    { 0, 0 },
    { 0x43b58873, 55 },
    { 0, 0 },
    { 0xced58875, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93558879, 61 },
    { 0, 0 },
    { 0x9e65887b, 61 },
    { 0, 0 },
    { 0xabf5887d, 90 },
    { 0xd265887e, 68 },
    { 0, 0 },
    { 0xd5158880, 80 },
    { 0xa1158880, 66 },
    { 0x4ed58881, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68458888, 61 },
    { 0xcf858889, 98 },
    { 0x155888a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db5888e, 52 },
    { 0, 0 },
    { 0x9d358890, 76 },
    { 0, 0 },
    { 0xe4d58892, 53 },
    { 0, 0 },
    { 0x15f58894, 52 },
    { 0, 0 },
    { 0xcf858896, 34 },
    { 0, 0 },
    { 0xf8458898, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5889d, 95 },
    { 0xb485889e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2588a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc588a4, 97 },
    { 0x4fb588a4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d588a9, 90 },
    { 0, 0 },
    { 0x7cd588ab, 98 },
    { 0x25b588ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb588af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b588b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc588b5, 84 },
    { 0x6d8588b5, 84 },
    { 0x908588b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8588ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50588bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea588c0, 65 },
    { 0x9c2588c1, 90 },
    { 0xbf2588c0, 23 },
    { 0x19588c2, 99 },
    { 0xbfe588c4, 94 },
    { 0x947588c5, 90 },
    { 0xf2c588c6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6588c9, 43 },
    { 0x3d8588c9, 83 },
    { 0, 0 },
    { 0xb40588cc, 43 },
    { 0xf42588cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25588d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588d6, 93 },
    { 0x9b1588d6, 90 },
    { 0x150588d8, 34 },
    { 0xcd9588d8, 95 },
    { 0xa4f588da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0588dd, 61 },
    { 0xf9c588de, 90 },
    { 0x901588dd, 100 },
    { 0x3c588de, 98 },
    { 0xaa588dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00588e6, 84 },
    { 0, 0 },
    { 0x31a588e8, 100 },
    { 0xb13588e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae588ec, 23 },
    { 0x920588ed, 67 },
    { 0, 0 },
    { 0xb8b588ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe588f2, 61 },
    { 0x72b588f2, 43 },
    { 0xa27588f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37588f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29588fc, 90 },
    { 0x4d0588fd, 76 },
    { 0x729588fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14958901, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8075890a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9325890f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71458913, 46 },
    { 0, 0 },
    { 0x9af58915, 84 },
    { 0, 0 },
    { 0xff58917, 91 },
    { 0x17658918, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd625891b, 90 },
    { 0, 0 },
    { 0x9a25891d, 67 },
    { 0, 0 },
    { 0x9cc5891f, 39 },
    { 0xffb58920, 23 },
    { 0xdc65891f, 55 },
    { 0x77858920, 61 },
    { 0x10e58923, 61 },
    { 0x60e58924, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b58931, 42 },
    { 0xb7d58931, 55 },
    { 0x8e458933, 81 },
    { 0xff258934, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69158941, 99 },
    { 0x7c58942, 98 },
    { 0x50358942, 76 },
    { 0xa8b58944, 76 },
    { 0, 0 },
    { 0x35758946, 34 },
    { 0x9c858947, 99 },
    { 0, 0 },
    { 0xfb758949, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15958951, 23 },
    { 0, 0 },
    { 0x21958953, 61 },
    { 0xded58954, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1458957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6435895a, 92 },
    { 0, 0 },
    { 0xbf05895c, 23 },
    { 0, 0 },
    { 0x5995895e, 84 },
    { 0, 0 },
    { 0xe1e58960, 98 },
    { 0x99358961, 33 },
    { 0x17a58962, 91 },
    { 0x31858963, 88 },
    { 0x258964, 34 },
    { 0, 0 },
    { 0xfbd58966, 55 },
    { 0x90a58967, 90 },
    { 0x75d58968, 98 },
    { 0, 0 },
    { 0x4cd5896a, 98 },
    { 0xa7b5896b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63458973, 95 },
    { 0xa6758973, 90 },
    { 0xb3e58973, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2458978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf805897d, 98 },
    { 0x94e5897e, 76 },
    { 0x70f5897f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62558984, 23 },
    { 0, 0 },
    { 0x94158986, 61 },
    { 0x5bc58987, 33 },
    { 0x7dc58986, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b45898b, 23 },
    { 0x8515898c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce58991, 90 },
    { 0xeed58992, 90 },
    { 0x53858992, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d5899e, 17 },
    { 0xd705899e, 43 },
    { 0x401589a0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf589a5, 52 },
    { 0x558589a5, 43 },
    { 0x285589a7, 99 },
    { 0, 0 },
    { 0x6d8589a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544589ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a589b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2589b8, 100 },
    { 0x7d9589b9, 43 },
    { 0, 0 },
    { 0xe0a589bb, 33 },
    { 0x970589bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3589c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca589d0, 90 },
    { 0, 0 },
    { 0x311589d2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13589d5, 100 },
    { 0x48589d6, 61 },
    { 0xd7b589d7, 33 },
    { 0, 0 },
    { 0xa0e589d9, 66 },
    { 0, 0 },
    { 0xa589db, 61 },
    { 0x877589dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f589e6, 76 },
    { 0x7f0589e7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c589ee, 33 },
    { 0x64a589ef, 95 },
    { 0x938589f0, 56 },
    { 0x2ec589ef, 85 },
    { 0xba0589f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae589f6, 76 },
    { 0x40c589f7, 33 },
    { 0x28589f7, 33 },
    { 0xddb589f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba589fd, 67 },
    { 0xb96589fd, 61 },
    { 0x46f589fe, 17 },
    { 0, 0 },
    { 0x28158a01, 33 },
    { 0x12b58a02, 55 },
    { 0x69c58a02, 98 },
    { 0x69858a04, 98 },
    { 0xf5f58a05, 5 },
    { 0x93958a06, 98 },
    { 0xdb758a07, 90 },
    { 0, 0 },
    { 0x9f758a09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xca558a0c, 98 },
    { 0x89c58a0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d558a11, 34 },
    { 0xfda58a12, 91 },
    { 0, 0 },
    { 0x2c758a14, 61 },
    { 0xbec58a15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a858a19, 33 },
    { 0, 0 },
    { 0xdc658a1b, 90 },
    { 0x97e58a1c, 43 },
    { 0xaa358a1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb558a21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1558a28, 90 },
    { 0x80f58a28, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd58a2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be58a30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e658a33, 90 },
    { 0x9f858a33, 61 },
    { 0xb7658a33, 90 },
    { 0x58158a36, 76 },
    { 0x47d58a37, 33 },
    { 0xf9658a38, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4058a3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0358a3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24558a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79758a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca58a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb258a4f, 52 },
    { 0, 0 },
    { 0x18a58a51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c58a54, 33 },
    { 0x34e58a55, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d58a58, 90 },
    { 0xb9258a59, 33 },
    { 0x15758a5a, 95 },
    { 0x41558a5b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5558a60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c58a63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2558a67, 67 },
    { 0xd2858a68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8058a6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58a70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28958a74, 68 },
    { 0x74e58a75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7658a78, 91 },
    { 0x58458a79, 55 },
    { 0x1d758a7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c58a82, 26 },
    { 0xd7058a82, 61 },
    { 0xd2d58a83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f958a8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b658a99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc58a9c, 55 },
    { 0x81158a9d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82358aa3, 98 },
    { 0xfa458aa4, 55 },
    { 0, 0 },
    { 0xbb858aa6, 84 },
    { 0x63f58aa7, 68 },
    { 0x78358aa7, 52 },
    { 0x94a58aa9, 33 },
    { 0xa4f58aa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc58aae, 78 },
    { 0xaf158aaf, 34 },
    { 0x51558aaf, 90 },
    { 0, 0 },
    { 0x96a58ab2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b958ab5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c58aba, 23 },
    { 0x62b58aba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec858abe, 23 },
    { 0x29e58abf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79258ac3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e58ac8, 95 },
    { 0xbcb58ac9, 95 },
    { 0, 0 },
    { 0xdc958acb, 88 },
    { 0xd7758acc, 33 },
    { 0x7058acd, 95 },
    { 0, 0 },
    { 0x1c958acf, 61 },
    { 0xd5558ad0, 95 },
    { 0x30c58ad0, 99 },
    { 0, 0 },
    { 0x79058ad3, 84 },
    { 0xf7258ad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a58adb, 90 },
    { 0, 0 },
    { 0xb1a58add, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d058ae0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17658ae3, 98 },
    { 0xbb058ae3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd58ae7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a158aea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af58aee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc58af6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd58afb, 90 },
    { 0x5a158afc, 43 },
    { 0x1b358afd, 90 },
    { 0xbe158afe, 95 },
    { 0x47458aff, 76 },
    { 0, 0 },
    { 0x42b58b01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d658b04, 46 },
    { 0xf558b04, 93 },
    { 0x1c058b05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd58b0c, 76 },
    { 0x80f58b0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13258b10, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca58b13, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4458b18, 95 },
    { 0x50c58b19, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f158b1c, 90 },
    { 0x5d758b1c, 98 },
    { 0, 0 },
    { 0x83e58b1f, 23 },
    { 0xf4a58b20, 95 },
    { 0xc5d58b21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce358b28, 98 },
    { 0xff858b29, 69 },
    { 0, 0 },
    { 0x6ec58b2b, 17 },
    { 0x72d58b2c, 78 },
    { 0xe2158b2d, 51 },
    { 0x4358b2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e458b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66858b3d, 76 },
    { 0x98858b3e, 61 },
    { 0x15958b3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41058b47, 33 },
    { 0x4f158b48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4158b4c, 23 },
    { 0xfec58b4c, 33 },
    { 0xbff58b4d, 61 },
    { 0xb9158b4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b758b61, 61 },
    { 0, 0 },
    { 0x9ef58b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4658b66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb558b6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da58b72, 38 },
    { 0xd0c58b73, 46 },
    { 0xf4a58b73, 98 },
    { 0x36c58b74, 61 },
    { 0x75a58b76, 95 },
    { 0xf7a58b74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b58b7b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e358b7f, 55 },
    { 0xb9b58b7f, 98 },
    { 0x9db58b81, 61 },
    { 0x4c058b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c858b88, 61 },
    { 0xe1d58b89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84158b8d, 95 },
    { 0x2e558b8e, 98 },
    { 0x61358b8f, 91 },
    { 0, 0 },
    { 0xaea58b91, 68 },
    { 0x2a858b91, 33 },
    { 0x65958b93, 52 },
    { 0, 0 },
    { 0x20258b95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9358b9e, 68 },
    { 0xc3758b9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6958ba4, 56 },
    { 0, 0 },
    { 0x8558ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b458ba9, 76 },
    { 0xb358ba9, 98 },
    { 0x56058baa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30758bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17858bb5, 46 },
    { 0x3e858bb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x36258bb9, 23 },
    { 0xe2858bba, 98 },
    { 0, 0 },
    { 0x32858bbc, 95 },
    { 0, 0 },
    { 0x5eb58bbe, 46 },
    { 0, 0 },
    { 0x9d558bc0, 61 },
    { 0, 0 },
    { 0xfd658bc2, 23 },
    { 0x3a158bc2, 91 },
    { 0x9e658bc2, 98 },
    { 0, 0 },
    { 0x3af58bc6, 98 },
    { 0xdf958bc6, 34 },
    { 0x78658bc8, 55 },
    { 0xc8558bc8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34058bcd, 34 },
    { 0x5d58bce, 80 },
    { 0xfa158bcf, 34 },
    { 0x9ac58bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47558bd4, 90 },
    { 0x8fa58bd5, 61 },
    { 0xb0358bd4, 45 },
    { 0x6ad58bd6, 33 },
    { 0, 0 },
    { 0x9a58bd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30858bdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d58be0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd858be4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e958be8, 34 },
    { 0xc5258be9, 90 },
    { 0xce358bea, 87 },
    { 0xeb058beb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca358bee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d58bf1, 98 },
    { 0, 0 },
    { 0xf6c58bf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e58bf8, 53 },
    { 0, 0 },
    { 0xfe358bfa, 90 },
    { 0xe5058bfb, 90 },
    { 0x8ee58bfc, 93 },
    { 0x9e558bfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb58c02, 90 },
    { 0x7ca58c03, 98 },
    { 0, 0 },
    { 0x2b658c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b58c0b, 95 },
    { 0, 0 },
    { 0x9e458c0d, 33 },
    { 0, 0 },
    { 0x5a58c0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59758c13, 55 },
    { 0xdd358c14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad058c17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee58c1b, 66 },
    { 0x2c958c1c, 23 },
    { 0, 0 },
    { 0x7c658c1e, 77 },
    { 0xc5258c1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e58c26, 83 },
    { 0, 0 },
    { 0x5fe58c28, 84 },
    { 0x2a958c28, 46 },
    { 0, 0 },
    { 0x65058c2b, 91 },
    { 0, 0 },
    { 0x558c2d, 76 },
    { 0x20158c2e, 98 },
    { 0x71958c2f, 92 },
    { 0xb4158c30, 77 },
    { 0xa9058c30, 85 },
    { 0xe0158c30, 23 },
    { 0xa7858c33, 55 },
    { 0x91e58c34, 61 },
    { 0x41e58c34, 81 },
    { 0xc0658c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be58c39, 90 },
    { 0x46058c3a, 100 },
    { 0x77858c39, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f58c41, 90 },
    { 0xd6458c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28558c46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f58c57, 78 },
    { 0, 0 },
    { 0x2fd58c59, 98 },
    { 0x18158c5a, 33 },
    { 0x63658c59, 52 },
    { 0xdd158c5c, 40 },
    { 0xa2f58c5b, 23 },
    { 0x72b58c5a, 61 },
    { 0xea58c5e, 55 },
    { 0x96658c60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c458c63, 23 },
    { 0xb6058c64, 93 },
    { 0xb8058c65, 101 },
    { 0xe5758c66, 61 },
    { 0, 0 },
    { 0x55158c68, 98 },
    { 0xed658c68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68458c6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a58c70, 43 },
    { 0x66e58c71, 33 },
    { 0xf6258c71, 90 },
    { 0x6b758c71, 33 },
    { 0, 0 },
    { 0x5e458c75, 90 },
    { 0xbee58c75, 55 },
    { 0xe8e58c77, 93 },
    { 0x89e58c77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d158c7c, 46 },
    { 0x29e58c7d, 33 },
    { 0x2fc58c7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec558c8d, 90 },
    { 0xad358c8e, 23 },
    { 0, 0 },
    { 0x51a58c90, 34 },
    { 0x86a58c91, 95 },
    { 0, 0 },
    { 0x23758c93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9058c96, 43 },
    { 0x8758c96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe58c9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7658ca3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd358ca7, 66 },
    { 0, 0 },
    { 0x71358ca9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa58cac, 23 },
    { 0x8c358cad, 52 },
    { 0x61858cae, 95 },
    { 0x16658caf, 43 },
    { 0x41a58caf, 43 },
    { 0x94c58cb1, 97 },
    { 0xb9c58cb1, 76 },
    { 0, 0 },
    { 0x63f58cb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a58cbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e58cbf, 95 },
    { 0x2a958cc0, 33 },
    { 0x4d158cc1, 72 },
    { 0xf4158cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44058cc6, 90 },
    { 0, 0 },
    { 0x59158cc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a58cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3558cda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3358cdd, 91 },
    { 0xd1458cde, 23 },
    { 0x75058cdd, 61 },
    { 0xbb658ce0, 61 },
    { 0, 0 },
    { 0x2c858ce2, 98 },
    { 0, 0 },
    { 0x6e158ce4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86458ceb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b58cf6, 90 },
    { 0xc5258cf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40158cfb, 88 },
    { 0xcbb58cfb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf258cff, 76 },
    { 0x3cb58d00, 66 },
    { 0xc858d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf558d06, 98 },
    { 0x35f58d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e58d10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde358d15, 76 },
    { 0, 0 },
    { 0x66458d17, 43 },
    { 0, 0 },
    { 0xf1f58d19, 33 },
    { 0xd1d58d19, 85 },
    { 0xdea58d1b, 76 },
    { 0x78e58d19, 43 },
    { 0x54358d1d, 95 },
    { 0x23b58d1e, 33 },
    { 0xf4c58d1e, 90 },
    { 0xf6a58d1d, 33 },
    { 0xff458d1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce58d2e, 95 },
    { 0xfc858d2f, 33 },
    { 0, 0 },
    { 0x3fd58d31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22158d34, 90 },
    { 0, 0 },
    { 0x6d958d36, 33 },
    { 0x7b658d37, 64 },
    { 0xe7958d36, 70 },
    { 0xbab58d38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad858d3d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48158d43, 85 },
    { 0xab758d44, 90 },
    { 0x46758d44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d858d48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d358d4b, 55 },
    { 0xd7c58d4c, 56 },
    { 0x7a158d4d, 33 },
    { 0x2b858d4b, 23 },
    { 0, 0 },
    { 0xa6d58d50, 46 },
    { 0, 0 },
    { 0x6b258d52, 90 },
    { 0, 0 },
    { 0x90358d54, 23 },
    { 0, 0 },
    { 0xfe058d56, 23 },
    { 0, 0 },
    { 0xc5858d58, 77 },
    { 0, 0 },
    { 0xc7358d5a, 90 },
    { 0xe5658d5b, 95 },
    { 0, 0 },
    { 0x74158d5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a258d62, 93 },
    { 0, 0 },
    { 0x10258d64, 52 },
    { 0xbe658d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58158d6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb558d6d, 23 },
    { 0, 0 },
    { 0x25958d6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57258d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde358d77, 23 },
    { 0x25958d78, 84 },
    { 0x58d58d78, 88 },
    { 0x758d78, 87 },
    { 0x27258d7a, 98 },
    { 0x9ce58d7a, 23 },
    { 0x1158d7d, 40 },
    { 0x9b58d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41658d85, 34 },
    { 0, 0 },
    { 0xe7a58d87, 93 },
    { 0x84f58d88, 96 },
    { 0xfd258d89, 2 },
    { 0x45758d87, 75 },
    { 0x7e158d88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e958d90, 23 },
    { 0x97a58d91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11958d97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d658d9b, 90 },
    { 0, 0 },
    { 0xf2758d9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a58da7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a58dab, 90 },
    { 0xe8258dac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba58db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb58db5, 33 },
    { 0, 0 },
    { 0x22558db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83358dbc, 100 },
    { 0xd1958dbc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61458dc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb158dc7, 61 },
    { 0x1f658dc8, 46 },
    { 0, 0 },
    { 0x29658dca, 90 },
    { 0x92e58dca, 93 },
    { 0x8e458dcb, 84 },
    { 0x6cf58dcd, 17 },
    { 0xfa758dce, 98 },
    { 0x9c258dcd, 34 },
    { 0x57e58dcf, 43 },
    { 0xe0058dd1, 95 },
    { 0x6d358dd2, 33 },
    { 0x43c58dd3, 55 },
    { 0x50e58dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d758dda, 76 },
    { 0xd8158ddb, 90 },
    { 0xe0658dda, 11 },
    { 0x7e658ddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5458de2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc558de8, 23 },
    { 0x67958de9, 61 },
    { 0xc6258de9, 34 },
    { 0x4a558dea, 36 },
    { 0x7fe58de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e758def, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea58df2, 98 },
    { 0x96258df3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a858dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa158dff, 91 },
    { 0x67758dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e58e05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7758e0c, 23 },
    { 0x5df58e0d, 95 },
    { 0x1bc58e0d, 23 },
    { 0x7e58e0f, 76 },
    { 0x53658e0f, 100 },
    { 0x1b158e0d, 23 },
    { 0, 0 },
    { 0xa5558e13, 33 },
    { 0x70e58e14, 76 },
    { 0x9fd58e13, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f058e1c, 61 },
    { 0, 0 },
    { 0xd9e58e1e, 52 },
    { 0, 0 },
    { 0x3f558e20, 80 },
    { 0xc9d58e20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0958e26, 23 },
    { 0x12258e27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f958e2a, 55 },
    { 0x47158e2b, 95 },
    { 0x5ac58e2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b058e30, 61 },
    { 0, 0 },
    { 0xd5058e32, 90 },
    { 0x31758e32, 76 },
    { 0xb4e58e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e158e38, 55 },
    { 0xce558e38, 98 },
    { 0xee358e39, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2758e3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41358e40, 61 },
    { 0x65358e41, 46 },
    { 0x1cf58e41, 33 },
    { 0xd3058e43, 87 },
    { 0x29058e43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc58e47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26658e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab58e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2358e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b458e5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e58e5e, 95 },
    { 0x1ee58e5f, 55 },
    { 0x3af58e60, 90 },
    { 0x1e358e61, 66 },
    { 0x5e258e60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc58e65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca58e6a, 90 },
    { 0xfef58e6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b758e6f, 33 },
    { 0x53358e70, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e758e73, 98 },
    { 0x24558e74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee058e77, 99 },
    { 0x6d558e77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc58e7b, 95 },
    { 0xcb358e7c, 98 },
    { 0x1a158e7d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b458e80, 61 },
    { 0, 0 },
    { 0xc558e82, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce58e85, 85 },
    { 0, 0 },
    { 0xdd58e87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97458e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a558e94, 46 },
    { 0xe6358e95, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a58e98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d58e9e, 36 },
    { 0xa7a58e9f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62058ea7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c58eaf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb258eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41958eb6, 43 },
    { 0x75358eb6, 95 },
    { 0x70358eb7, 43 },
    { 0, 0 },
    { 0x32858eba, 33 },
    { 0x31958eba, 33 },
    { 0x47658ebc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81458ebf, 90 },
    { 0x2f558ebf, 93 },
    { 0x83458ec1, 42 },
    { 0x16958ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42758ec7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25758ecb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45158ed8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48958edd, 68 },
    { 0, 0 },
    { 0xc6e58edf, 61 },
    { 0, 0 },
    { 0x9ba58ee1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2758ee5, 98 },
    { 0x11658ee6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf58eea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93358eee, 23 },
    { 0, 0 },
    { 0xc2a58ef0, 98 },
    { 0xb8158ef1, 55 },
    { 0xe6d58ef1, 84 },
    { 0, 0 },
    { 0x23a58ef4, 55 },
    { 0x9a958ef5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d258ef9, 78 },
    { 0x6c858efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37758efe, 55 },
    { 0x62458eff, 98 },
    { 0xa4058eff, 23 },
    { 0xc0d58eff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c258f04, 81 },
    { 0x3d058f04, 34 },
    { 0xdc558f06, 93 },
    { 0x5ec58f07, 76 },
    { 0x6ac58f06, 61 },
    { 0x17f58f06, 61 },
    { 0xfd658f05, 48 },
    { 0xac058f07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10158f13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b258f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e58f1c, 99 },
    { 0, 0 },
    { 0x40b58f1e, 76 },
    { 0x57358f1e, 91 },
    { 0, 0 },
    { 0xc9e58f21, 93 },
    { 0xedb58f22, 91 },
    { 0x8fb58f23, 90 },
    { 0x33358f24, 95 },
    { 0x6eb58f25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28358f29, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4058f2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3258f37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63858f3e, 93 },
    { 0, 0 },
    { 0x9f558f40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e58f45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88258f49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b58f4d, 43 },
    { 0x26858f4e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b58f54, 61 },
    { 0x43b58f55, 68 },
    { 0, 0 },
    { 0xd4f58f57, 61 },
    { 0x95b58f58, 23 },
    { 0x9fe58f59, 78 },
    { 0x3d158f5a, 68 },
    { 0, 0 },
    { 0x8c458f5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a258f61, 33 },
    { 0, 0 },
    { 0x79e58f63, 76 },
    { 0x45e58f63, 91 },
    { 0, 0 },
    { 0xd8258f66, 61 },
    { 0x78058f66, 85 },
    { 0x37858f66, 67 },
    { 0, 0 },
    { 0xe5958f6a, 76 },
    { 0xdff58f6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e58f72, 33 },
    { 0xa4458f73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8458f78, 90 },
    { 0xa8d58f79, 33 },
    { 0x24c58f7a, 68 },
    { 0x11c58f7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9658f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1458f88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa158f8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0058f8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e58f94, 61 },
    { 0x14158f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f58f98, 98 },
    { 0xc8f58f99, 34 },
    { 0, 0 },
    { 0x4da58f9b, 95 },
    { 0xadf58f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22558f9f, 95 },
    { 0, 0 },
    { 0x52558fa1, 93 },
    { 0, 0 },
    { 0x1ae58fa3, 90 },
    { 0, 0 },
    { 0x21258fa5, 23 },
    { 0, 0 },
    { 0x72958fa7, 23 },
    { 0xdaa58fa7, 33 },
    { 0x32758fa9, 90 },
    { 0x1e258faa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5958fae, 33 },
    { 0, 0 },
    { 0x75358fb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82158fba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7158fbe, 23 },
    { 0, 0 },
    { 0xc9e58fc0, 93 },
    { 0x9d58fc0, 91 },
    { 0x76958fc1, 33 },
    { 0xf3958fc3, 95 },
    { 0xdce58fc4, 52 },
    { 0x2cd58fc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d58fce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d58fd1, 33 },
    { 0x38b58fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a158fd5, 33 },
    { 0, 0 },
    { 0x5e58fd7, 33 },
    { 0x9458fd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77258fdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd58fdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb558fe6, 61 },
    { 0x72358fe7, 98 },
    { 0xae258fe8, 38 },
    { 0xcf358fe9, 98 },
    { 0x13358fea, 36 },
    { 0x2fe58feb, 33 },
    { 0xffb58feb, 61 },
    { 0, 0 },
    { 0xe8158fee, 61 },
    { 0x52e58fee, 78 },
    { 0xc4158ff0, 55 },
    { 0x73c58fef, 46 },
    { 0xde658ff2, 95 },
    { 0xbda58ff3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9558ff8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c58ffd, 90 },
    { 0x24058ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f59006, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60359009, 61 },
    { 0xf945900a, 95 },
    { 0xa575900b, 23 },
    { 0, 0 },
    { 0xec35900d, 100 },
    { 0x4bf5900d, 78 },
    { 0x8c25900f, 98 },
    { 0xaf759010, 100 },
    { 0x8d559011, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a59017, 76 },
    { 0xc2e59018, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65901b, 95 },
    { 0x8cc5901c, 61 },
    { 0x6825901d, 43 },
    { 0, 0 },
    { 0x22b5901f, 52 },
    { 0, 0 },
    { 0x6d59021, 76 },
    { 0xc3559021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6559026, 33 },
    { 0xa7b59027, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a5902a, 98 },
    { 0x8365902b, 23 },
    { 0x4255902b, 80 },
    { 0x6d05902a, 23 },
    { 0x4365902e, 43 },
    { 0x3dc5902f, 34 },
    { 0, 0 },
    { 0x3259031, 76 },
    { 0, 0 },
    { 0xf0f59033, 61 },
    { 0x3d159034, 33 },
    { 0x89659035, 55 },
    { 0x51559036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5565903c, 38 },
    { 0, 0 },
    { 0x9db5903e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b59045, 88 },
    { 0, 0 },
    { 0x36b59047, 91 },
    { 0, 0 },
    { 0xbc059049, 76 },
    { 0, 0 },
    { 0x1cb5904b, 46 },
    { 0x5655904c, 61 },
    { 0xe4a5904d, 76 },
    { 0x3705904e, 76 },
    { 0x3b55904f, 95 },
    { 0x1845904c, 61 },
    { 0xe5d5904d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d59056, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a95905a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca59060, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9159064, 77 },
    { 0xc59064, 76 },
    { 0x32e59064, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a35906c, 23 },
    { 0x7a25906d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f59070, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7659073, 55 },
    { 0xe7859073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad15907e, 23 },
    { 0, 0 },
    { 0x31759080, 95 },
    { 0x22559080, 95 },
    { 0xe5359080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa775908d, 85 },
    { 0, 0 },
    { 0xd905908f, 61 },
    { 0x4d59090, 55 },
    { 0, 0 },
    { 0xe2659092, 76 },
    { 0x16b59093, 43 },
    { 0x44b59094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d590a8, 43 },
    { 0, 0 },
    { 0xc73590aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b590ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1590b1, 52 },
    { 0x6a2590b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x859590b5, 98 },
    { 0xdb1590b6, 23 },
    { 0, 0 },
    { 0xbbc590b8, 23 },
    { 0x3fc590b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0590bf, 68 },
    { 0xca7590c0, 90 },
    { 0xc7e590bf, 33 },
    { 0x583590c2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7590c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x170590c8, 33 },
    { 0, 0 },
    { 0x7c8590ca, 33 },
    { 0x563590ca, 38 },
    { 0, 0 },
    { 0x4f590cd, 34 },
    { 0, 0 },
    { 0xd87590cf, 77 },
    { 0x953590cf, 76 },
    { 0xb4a590d0, 90 },
    { 0xd25590d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592590d7, 99 },
    { 0xc49590d8, 61 },
    { 0x252590d9, 85 },
    { 0x476590d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59590dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743590e2, 90 },
    { 0x417590e3, 87 },
    { 0x4fe590e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d590ea, 33 },
    { 0x263590ea, 61 },
    { 0xb13590ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea590ef, 33 },
    { 0x4590f0, 61 },
    { 0x858590f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee590f4, 95 },
    { 0xfe3590f4, 55 },
    { 0, 0 },
    { 0x549590f7, 95 },
    { 0xdb6590f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e359100, 46 },
    { 0x559101, 95 },
    { 0, 0 },
    { 0xb9859103, 61 },
    { 0x31259104, 91 },
    { 0x4f459105, 43 },
    { 0x89859104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf95910d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb59110, 83 },
    { 0xdc159111, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5911a, 78 },
    { 0, 0 },
    { 0x3d55911c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4759120, 84 },
    { 0xec159121, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b59124, 98 },
    { 0xdd959124, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f59130, 76 },
    { 0xcac59131, 95 },
    { 0xc5c59132, 46 },
    { 0x74759130, 23 },
    { 0x58d59134, 91 },
    { 0x6fb59135, 33 },
    { 0x48d59134, 95 },
    { 0x58959137, 55 },
    { 0, 0 },
    { 0x2ed59139, 93 },
    { 0, 0 },
    { 0xde15913b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89359143, 91 },
    { 0x14e59144, 95 },
    { 0, 0 },
    { 0xe8359146, 76 },
    { 0, 0 },
    { 0x82659148, 76 },
    { 0x88059149, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe465914d, 98 },
    { 0, 0 },
    { 0xbd95914f, 34 },
    { 0x91e59150, 90 },
    { 0xfbc59151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7959159, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5915c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed25916d, 90 },
    { 0x18f5916e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c159171, 33 },
    { 0x7c159172, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f59175, 55 },
    { 0, 0 },
    { 0xb3659177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b59188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee5918d, 67 },
    { 0xe955918e, 23 },
    { 0xfd5918f, 90 },
    { 0x1825918f, 48 },
    { 0x56459191, 55 },
    { 0x3b159191, 55 },
    { 0, 0 },
    { 0x16a59194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76759197, 61 },
    { 0, 0 },
    { 0x83e59199, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195919c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244591a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3591a4, 91 },
    { 0x53c591a5, 90 },
    { 0, 0 },
    { 0x6b4591a7, 91 },
    { 0, 0 },
    { 0x29e591a9, 96 },
    { 0, 0 },
    { 0xeb5591ab, 34 },
    { 0x103591ac, 93 },
    { 0xec9591ac, 90 },
    { 0x510591ab, 61 },
    { 0xf78591af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86591b4, 23 },
    { 0, 0 },
    { 0xe9b591b6, 40 },
    { 0x885591b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee591bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7591c4, 55 },
    { 0, 0 },
    { 0xca9591c6, 99 },
    { 0xeae591c7, 33 },
    { 0x383591c7, 98 },
    { 0, 0 },
    { 0x807591ca, 55 },
    { 0x148591ca, 85 },
    { 0xf8e591cc, 90 },
    { 0x547591cd, 23 },
    { 0xfc591cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3591d1, 90 },
    { 0xcae591d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839591d9, 61 },
    { 0xc25591da, 33 },
    { 0, 0 },
    { 0xf68591dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x146591df, 75 },
    { 0x671591e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8591ec, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b591f1, 33 },
    { 0xc40591f2, 77 },
    { 0xf1b591f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e591f7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d591ff, 90 },
    { 0, 0 },
    { 0xe3c59201, 72 },
    { 0, 0 },
    { 0x98759203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c45920b, 61 },
    { 0x8fb5920b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e159211, 33 },
    { 0, 0 },
    { 0x64359213, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce59217, 33 },
    { 0, 0 },
    { 0x1f959219, 90 },
    { 0x3915921a, 52 },
    { 0x20359219, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba5921f, 76 },
    { 0, 0 },
    { 0xac559221, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf659226, 90 },
    { 0x7e59227, 76 },
    { 0x9a159227, 90 },
    { 0x52259229, 95 },
    { 0x84a59227, 69 },
    { 0xba55922b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d5922e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16759233, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e659237, 23 },
    { 0x1f459238, 61 },
    { 0xdf159239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8945923d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b5924d, 93 },
    { 0x68e5924d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3659252, 33 },
    { 0x18659253, 46 },
    { 0xcd559254, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d759258, 61 },
    { 0x85a59259, 33 },
    { 0xfbe5925a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f5925e, 61 },
    { 0x3a25925f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d559263, 23 },
    { 0xf6759264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae5926b, 33 },
    { 0x9a95926b, 95 },
    { 0xaed5926d, 36 },
    { 0xdf95926e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77259271, 90 },
    { 0x1ba59272, 90 },
    { 0x35c59271, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30359278, 43 },
    { 0x8f459279, 95 },
    { 0xe1e59278, 23 },
    { 0x9355927b, 90 },
    { 0x755927c, 75 },
    { 0xbfb5927d, 98 },
    { 0xd645927d, 90 },
    { 0x1a55927d, 52 },
    { 0, 0 },
    { 0x81d59281, 61 },
    { 0xf2559282, 52 },
    { 0, 0 },
    { 0x88859284, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3459287, 55 },
    { 0, 0 },
    { 0xc9a59289, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32859291, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a459296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205929c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b592a0, 95 },
    { 0x194592a1, 98 },
    { 0xbd9592a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d592a6, 34 },
    { 0, 0 },
    { 0x6e4592a8, 61 },
    { 0xe0f592a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f592ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d592b7, 81 },
    { 0x80592b8, 91 },
    { 0, 0 },
    { 0xe98592ba, 33 },
    { 0xf15592bb, 90 },
    { 0, 0 },
    { 0xc29592bd, 61 },
    { 0x37a592be, 93 },
    { 0xa64592be, 61 },
    { 0xce0592c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66592c6, 98 },
    { 0x19b592c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f592cd, 23 },
    { 0x69a592ce, 43 },
    { 0xb71592cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf592d2, 98 },
    { 0x972592d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0592d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0592e0, 98 },
    { 0xe75592e0, 76 },
    { 0xc04592e0, 55 },
    { 0x17c592e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c592e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x338592e9, 98 },
    { 0x998592ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8592ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e592f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338592ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da59303, 93 },
    { 0x39759304, 100 },
    { 0xeb59305, 77 },
    { 0xc3759306, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b15930e, 90 },
    { 0x2f15930f, 84 },
    { 0x44959310, 61 },
    { 0, 0 },
    { 0x63d59312, 91 },
    { 0xb4b59313, 66 },
    { 0xa2259314, 46 },
    { 0, 0 },
    { 0x4bc59316, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9855931c, 95 },
    { 0x7fa5931d, 95 },
    { 0xb525931e, 77 },
    { 0x5335931f, 88 },
    { 0x70e59320, 98 },
    { 0xd5c5931f, 52 },
    { 0xafb59321, 55 },
    { 0x38559320, 95 },
    { 0, 0 },
    { 0x40059325, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec059329, 46 },
    { 0x71759329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed59335, 91 },
    { 0xd4459335, 34 },
    { 0, 0 },
    { 0xfc859338, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa325933d, 55 },
    { 0x7d55933e, 98 },
    { 0xe8e5933f, 61 },
    { 0xb9059340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e55934c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb555934f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d59356, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0659359, 90 },
    { 0xde85935a, 33 },
    { 0x8605935b, 43 },
    { 0x24359359, 91 },
    { 0, 0 },
    { 0x8975935e, 90 },
    { 0, 0 },
    { 0xc2559360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2059368, 33 },
    { 0xffc59368, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a5936d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e59370, 33 },
    { 0xf8e59371, 61 },
    { 0, 0 },
    { 0xb0c59373, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d59376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x95359379, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b5937c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a5937f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1259384, 98 },
    { 0xde559385, 33 },
    { 0x59559386, 95 },
    { 0, 0 },
    { 0xfe959388, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb125938b, 61 },
    { 0x5b45938c, 76 },
    { 0xcb45938d, 43 },
    { 0xeff5938d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee759393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b59396, 65 },
    { 0x1e359397, 84 },
    { 0x48359396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad5939b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5939f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x339593a2, 95 },
    { 0x552593a3, 95 },
    { 0, 0 },
    { 0x821593a5, 61 },
    { 0x3bc593a6, 23 },
    { 0, 0 },
    { 0x23d593a8, 87 },
    { 0x6bf593a9, 33 },
    { 0xfc2593a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a593af, 98 },
    { 0xa2d593b0, 46 },
    { 0xaf0593af, 95 },
    { 0x38c593b2, 90 },
    { 0xf5b593b0, 46 },
    { 0x9e4593b4, 56 },
    { 0, 0 },
    { 0x4e2593b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d593b9, 91 },
    { 0, 0 },
    { 0x8c4593bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e593c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7593c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b593ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4593d1, 61 },
    { 0xdbf593d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e593dd, 78 },
    { 0, 0 },
    { 0x90593df, 98 },
    { 0x3ed593e0, 87 },
    { 0xe6593e0, 90 },
    { 0, 0 },
    { 0x8593e3, 40 },
    { 0, 0 },
    { 0x15d593e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22593e9, 34 },
    { 0, 0 },
    { 0x5c7593eb, 95 },
    { 0xdcc593eb, 85 },
    { 0, 0 },
    { 0xd6d593ee, 23 },
    { 0, 0 },
    { 0xb06593f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff593f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9593fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a59400, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde15940b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a59411, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22459416, 23 },
    { 0, 0 },
    { 0xb9459418, 91 },
    { 0x33e59419, 100 },
    { 0xb775941a, 76 },
    { 0xa85941a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47559422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53359427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f59432, 84 },
    { 0x1ce59432, 98 },
    { 0x4d859432, 33 },
    { 0xeaa59433, 98 },
    { 0x12959436, 68 },
    { 0x56559436, 76 },
    { 0x11f59437, 33 },
    { 0xfd759439, 33 },
    { 0x5a959438, 67 },
    { 0x21e5943b, 84 },
    { 0x42e5943c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c5944a, 43 },
    { 0x605944b, 52 },
    { 0x1725944b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3965945f, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe359465, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af5946a, 98 },
    { 0x9915946b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40959471, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b59474, 61 },
    { 0x24659475, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc59480, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c59483, 33 },
    { 0, 0 },
    { 0xe8759485, 23 },
    { 0x45d59486, 53 },
    { 0xb159486, 76 },
    { 0xd7359488, 66 },
    { 0xc2d59489, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc5948c, 43 },
    { 0x3315948c, 23 },
    { 0xb65948c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xda159491, 95 },
    { 0, 0 },
    { 0x42759493, 83 },
    { 0xe3559494, 98 },
    { 0x33759495, 69 },
    { 0x6d759496, 61 },
    { 0, 0 },
    { 0x9a659498, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a55949b, 34 },
    { 0xb565949c, 17 },
    { 0xc9e5949d, 95 },
    { 0x5955949e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80594a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f594a8, 93 },
    { 0x765594a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420594ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13594b0, 61 },
    { 0, 0 },
    { 0xf9a594b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5594b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f594b9, 91 },
    { 0, 0 },
    { 0x4e3594bb, 85 },
    { 0x53b594bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f594c9, 78 },
    { 0, 0 },
    { 0x735594cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d594ce, 48 },
    { 0xe39594cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701594d7, 90 },
    { 0xbef594d8, 17 },
    { 0x31c594d8, 61 },
    { 0, 0 },
    { 0xe23594db, 91 },
    { 0x2b0594dc, 46 },
    { 0xc7b594db, 23 },
    { 0, 0 },
    { 0xc18594df, 43 },
    { 0xc00594e0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x229594e3, 33 },
    { 0x292594e3, 61 },
    { 0xb1a594e4, 60 },
    { 0x299594e6, 33 },
    { 0x291594e7, 90 },
    { 0x9d4594e4, 61 },
    { 0x2e594e8, 75 },
    { 0xd50594ea, 77 },
    { 0x4f4594eb, 90 },
    { 0x946594ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389594f1, 23 },
    { 0x9e6594f1, 100 },
    { 0, 0 },
    { 0x492594f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7594ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba59502, 90 },
    { 0, 0 },
    { 0x2b559504, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d5950b, 77 },
    { 0x4865950c, 55 },
    { 0xc4d5950d, 76 },
    { 0x57e5950e, 100 },
    { 0xef75950f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5759512, 66 },
    { 0x49e59513, 55 },
    { 0x59959514, 33 },
    { 0xa9d59515, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b5951d, 61 },
    { 0xd615951e, 33 },
    { 0xafd5951d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e459522, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a59529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc59530, 93 },
    { 0x97f59531, 93 },
    { 0xe6e59532, 55 },
    { 0, 0 },
    { 0x22859534, 95 },
    { 0xd759535, 33 },
    { 0xcb59536, 33 },
    { 0xfe359537, 38 },
    { 0xe8159538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a5953c, 93 },
    { 0, 0 },
    { 0x5015953e, 88 },
    { 0x1365953f, 81 },
    { 0xefb59540, 90 },
    { 0x59559541, 93 },
    { 0xd9a59540, 95 },
    { 0x6455953e, 76 },
    { 0xb9b59544, 23 },
    { 0x5db59545, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5954a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925954d, 61 },
    { 0, 0 },
    { 0xdc45954f, 33 },
    { 0xcd959550, 87 },
    { 0xe8459550, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9659554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5955a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd95955e, 90 },
    { 0x4795955e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa59562, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27159565, 34 },
    { 0, 0 },
    { 0x88859567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b5956e, 95 },
    { 0xacb5956f, 61 },
    { 0x1fa59570, 68 },
    { 0xc0d5956e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60159575, 85 },
    { 0xa1759576, 76 },
    { 0x55b59577, 23 },
    { 0, 0 },
    { 0x5fa59579, 100 },
    { 0x6605957a, 61 },
    { 0xd235957b, 61 },
    { 0x77a5957a, 90 },
    { 0, 0 },
    { 0xbf75957e, 40 },
    { 0, 0 },
    { 0xad759580, 90 },
    { 0xef859581, 98 },
    { 0xa7f59582, 40 },
    { 0, 0 },
    { 0x1c459584, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8459590, 76 },
    { 0x88459591, 68 },
    { 0x40759590, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f59598, 61 },
    { 0, 0 },
    { 0x7135959a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c595a1, 95 },
    { 0xb0e595a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e595a6, 99 },
    { 0xd7c595a6, 81 },
    { 0x753595a8, 77 },
    { 0xb80595a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86595ac, 76 },
    { 0xdde595ac, 98 },
    { 0x944595ae, 23 },
    { 0, 0 },
    { 0x113595b0, 33 },
    { 0xce8595b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df595b5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be595b8, 98 },
    { 0, 0 },
    { 0xf7b595ba, 75 },
    { 0, 0 },
    { 0x87595bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0595c3, 23 },
    { 0x58a595c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3595c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0595cf, 98 },
    { 0xfdb595cf, 33 },
    { 0xd17595d1, 23 },
    { 0x516595d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67595d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3595df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c595ea, 33 },
    { 0, 0 },
    { 0x27d595ec, 33 },
    { 0x3d8595ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1595f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453595f4, 23 },
    { 0xf23595f4, 61 },
    { 0x25f595f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1595ff, 33 },
    { 0, 0 },
    { 0x65059601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f59605, 33 },
    { 0, 0 },
    { 0x7bf59607, 68 },
    { 0x8e259608, 46 },
    { 0x6ea59609, 90 },
    { 0x1145960a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e65960d, 23 },
    { 0, 0 },
    { 0x85960f, 66 },
    { 0xfb959610, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e59617, 23 },
    { 0, 0 },
    { 0x50f59619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f65961c, 99 },
    { 0x9145961d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c59623, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5962d, 52 },
    { 0, 0 },
    { 0x1355962f, 98 },
    { 0x65d59630, 52 },
    { 0xf6f59631, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc59634, 90 },
    { 0x36059634, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8495963f, 90 },
    { 0x4ef5963f, 90 },
    { 0x1865963f, 90 },
    { 0x3b5963f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e59645, 33 },
    { 0x20159645, 85 },
    { 0, 0 },
    { 0x67559648, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa835964c, 33 },
    { 0, 0 },
    { 0x3995964e, 23 },
    { 0x7b15964e, 33 },
    { 0x2b75964e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e59654, 61 },
    { 0x2ac59655, 43 },
    { 0, 0 },
    { 0xce59657, 90 },
    { 0, 0 },
    { 0xef959659, 95 },
    { 0x30e59659, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4005965e, 33 },
    { 0xa065965e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec59669, 98 },
    { 0x6955966a, 95 },
    { 0x3d05966b, 61 },
    { 0, 0 },
    { 0x21e5966d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4959679, 95 },
    { 0xfb65967a, 46 },
    { 0xde25967b, 99 },
    { 0x9225967c, 87 },
    { 0x4dd5967b, 61 },
    { 0x5585967e, 90 },
    { 0x8565967f, 64 },
    { 0xfa45967f, 23 },
    { 0x3b159681, 80 },
    { 0x70259681, 34 },
    { 0xf9e59683, 52 },
    { 0xccd59683, 61 },
    { 0xb8b59684, 33 },
    { 0, 0 },
    { 0xd2459687, 68 },
    { 0xf0959687, 61 },
    { 0x87259689, 98 },
    { 0xae5968a, 43 },
    { 0x9d15968b, 66 },
    { 0xf6a5968c, 95 },
    { 0xbc05968c, 93 },
    { 0x14e5968c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a59698, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4596a3, 61 },
    { 0x48a596a3, 23 },
    { 0x4e6596a5, 88 },
    { 0xd12596a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x491596a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa596ac, 90 },
    { 0, 0 },
    { 0x249596ae, 98 },
    { 0, 0 },
    { 0xeb9596b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931596b4, 78 },
    { 0, 0 },
    { 0x489596b6, 43 },
    { 0xaeb596b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110596c0, 34 },
    { 0x2a6596c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35596c4, 33 },
    { 0x789596c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389596c9, 85 },
    { 0, 0 },
    { 0x622596cb, 88 },
    { 0x8b8596cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221596d4, 93 },
    { 0x624596d5, 101 },
    { 0, 0 },
    { 0xa7596d7, 98 },
    { 0xdc8596d8, 87 },
    { 0, 0 },
    { 0xaf0596da, 76 },
    { 0, 0 },
    { 0xa35596dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30596df, 91 },
    { 0xbe3596e0, 98 },
    { 0x6f7596e1, 90 },
    { 0x94a596e1, 55 },
    { 0x792596df, 55 },
    { 0, 0 },
    { 0x88d596e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d596e8, 98 },
    { 0xf3596e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60596ef, 90 },
    { 0, 0 },
    { 0x4d6596f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb596f4, 87 },
    { 0, 0 },
    { 0xa4c596f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f596f9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef596fc, 76 },
    { 0x483596fd, 95 },
    { 0, 0 },
    { 0x1d8596ff, 98 },
    { 0xb6059700, 78 },
    { 0xe5959701, 90 },
    { 0x97759700, 67 },
    { 0x70659703, 33 },
    { 0x7da59704, 55 },
    { 0x71f59704, 90 },
    { 0x34b59706, 98 },
    { 0, 0 },
    { 0x6d059708, 33 },
    { 0, 0 },
    { 0x3135970a, 78 },
    { 0, 0 },
    { 0x2845970c, 76 },
    { 0x9385970d, 98 },
    { 0xc835970d, 43 },
    { 0xaef5970f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c859714, 34 },
    { 0x99859715, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d59719, 95 },
    { 0x7a45971a, 95 },
    { 0x8c45971a, 90 },
    { 0x1695971a, 100 },
    { 0, 0 },
    { 0x97e5971e, 98 },
    { 0x3915971f, 61 },
    { 0, 0 },
    { 0xbe059721, 52 },
    { 0x66b59722, 61 },
    { 0, 0 },
    { 0x76e59724, 33 },
    { 0xb4659725, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2365972a, 85 },
    { 0x7cf5972b, 85 },
    { 0xfb55972c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7255972f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce25973b, 95 },
    { 0, 0 },
    { 0x135973d, 90 },
    { 0x4985973e, 77 },
    { 0x1395973d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f059748, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x6005974b, 90 },
    { 0x3085974b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4245974f, 22 },
    { 0, 0 },
    { 0x43259751, 46 },
    { 0xd2659752, 98 },
    { 0, 0 },
    { 0x41359754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8245975c, 83 },
    { 0x66d5975c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0859761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab759765, 76 },
    { 0, 0 },
    { 0xb4459767, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf5976c, 17 },
    { 0xda5976d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d59770, 98 },
    { 0x44459771, 87 },
    { 0x6659772, 95 },
    { 0x56559770, 88 },
    { 0, 0 },
    { 0x21359775, 90 },
    { 0xa6c59776, 31 },
    { 0x3a359777, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee5977b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe995977f, 78 },
    { 0xe7859780, 77 },
    { 0xd2e59781, 95 },
    { 0xc7e5977f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9059785, 23 },
    { 0x9fa59786, 90 },
    { 0x4e59787, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25978d, 52 },
    { 0x5535978e, 77 },
    { 0x8285978f, 60 },
    { 0x78759790, 99 },
    { 0xbac59791, 23 },
    { 0x28959791, 61 },
    { 0x8da59792, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c5979a, 90 },
    { 0x6215979b, 34 },
    { 0x5695979c, 33 },
    { 0x6285979b, 33 },
    { 0xa3d5979e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53597a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4597a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6597ab, 90 },
    { 0xa52597ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1597b1, 33 },
    { 0, 0 },
    { 0x450597b3, 33 },
    { 0xd0e597b4, 95 },
    { 0x308597b5, 95 },
    { 0xa24597b5, 46 },
    { 0xf51597b7, 56 },
    { 0x727597b8, 90 },
    { 0xe43597b9, 33 },
    { 0xefa597ba, 99 },
    { 0x31a597b5, 85 },
    { 0, 0 },
    { 0xa2c597bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94597c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10597c4, 99 },
    { 0xb6f597c5, 90 },
    { 0x309597c4, 61 },
    { 0x500597c7, 95 },
    { 0, 0 },
    { 0x42a597c9, 98 },
    { 0, 0 },
    { 0x23b597cb, 65 },
    { 0x90597cb, 34 },
    { 0xb1d597cd, 23 },
    { 0x900597ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21597d2, 33 },
    { 0xdd5597d3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a597d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225597db, 60 },
    { 0, 0 },
    { 0xf7597dd, 85 },
    { 0, 0 },
    { 0x38b597df, 55 },
    { 0x781597df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f597e9, 23 },
    { 0xa43597ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x446597ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7597f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86597f8, 52 },
    { 0x4b3597f8, 61 },
    { 0xa78597f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab59801, 33 },
    { 0x3b59801, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d59806, 90 },
    { 0, 0 },
    { 0x21359808, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe645980e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d59812, 98 },
    { 0xe0059813, 61 },
    { 0x9da59814, 90 },
    { 0x91759815, 35 },
    { 0x6c259814, 33 },
    { 0x75c59815, 76 },
    { 0x13459817, 95 },
    { 0xb259819, 95 },
    { 0xc1e5981a, 90 },
    { 0x1e659815, 61 },
    { 0, 0 },
    { 0x735981d, 90 },
    { 0, 0 },
    { 0xf995981f, 61 },
    { 0x6b759820, 33 },
    { 0, 0 },
    { 0x7a459822, 61 },
    { 0xdda59822, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16959827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c5982b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42459830, 90 },
    { 0x4d359831, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a15983c, 33 },
    { 0, 0 },
    { 0xbe45983e, 91 },
    { 0, 0 },
    { 0x6d459840, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e59844, 99 },
    { 0, 0 },
    { 0x2b259846, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5705984c, 76 },
    { 0x4b55984c, 90 },
    { 0x8545984c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x64359851, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x72959854, 91 },
    { 0x51159855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f559858, 55 },
    { 0xc2959859, 98 },
    { 0, 0 },
    { 0xacf5985b, 90 },
    { 0xb0a5985b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6259861, 61 },
    { 0x43859861, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad059867, 23 },
    { 0x33c59868, 85 },
    { 0x57e59869, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3485986f, 76 },
    { 0x31659870, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf959874, 90 },
    { 0xe2859875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc955987a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee59880, 33 },
    { 0xbe459880, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2159884, 23 },
    { 0xe9b59884, 55 },
    { 0x53059885, 75 },
    { 0xf1259884, 78 },
    { 0x30559887, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d15988c, 76 },
    { 0xd8f5988d, 98 },
    { 0xe785988e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa359891, 33 },
    { 0, 0 },
    { 0x6d459893, 76 },
    { 0xbd359894, 84 },
    { 0xf4459895, 95 },
    { 0xafd59893, 95 },
    { 0x1a59897, 95 },
    { 0x7d259898, 90 },
    { 0xa6d59898, 95 },
    { 0x1a459895, 43 },
    { 0x47159897, 98 },
    { 0xa759898, 78 },
    { 0xcd75989c, 90 },
    { 0xa225989d, 95 },
    { 0x71a5989a, 95 },
    { 0x9355989b, 93 },
    { 0, 0 },
    { 0x585598a2, 98 },
    { 0xd55598a3, 33 },
    { 0x17f598a4, 87 },
    { 0x357598a2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0598ad, 99 },
    { 0, 0 },
    { 0xef5598af, 61 },
    { 0x60c598b0, 76 },
    { 0x660598b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67598b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db598be, 98 },
    { 0, 0 },
    { 0x93598c0, 51 },
    { 0, 0 },
    { 0xfd2598c2, 93 },
    { 0xd18598c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x303598c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75598ca, 33 },
    { 0, 0 },
    { 0xce1598cc, 91 },
    { 0x3ef598cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56598d0, 98 },
    { 0, 0 },
    { 0x24c598d2, 23 },
    { 0x99d598d2, 88 },
    { 0xb29598d4, 61 },
    { 0xfab598d4, 84 },
    { 0x628598d6, 93 },
    { 0, 0 },
    { 0x3b2598d8, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93598dc, 90 },
    { 0x389598dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc598e2, 95 },
    { 0, 0 },
    { 0x4d598e4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x875598e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12598eb, 99 },
    { 0, 0 },
    { 0x799598ed, 98 },
    { 0xd1c598ee, 55 },
    { 0, 0 },
    { 0x41b598f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b598f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d598f9, 43 },
    { 0xe9c598fa, 90 },
    { 0xeb3598fb, 33 },
    { 0x79a598fc, 61 },
    { 0, 0 },
    { 0x88e598fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d659905, 90 },
    { 0, 0 },
    { 0x37e59907, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d55990b, 95 },
    { 0x50a5990c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb59914, 98 },
    { 0x1c559915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53159918, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d75991e, 95 },
    { 0x67d5991f, 61 },
    { 0xec159920, 23 },
    { 0x8f359921, 93 },
    { 0x42859920, 95 },
    { 0, 0 },
    { 0x84459924, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55259927, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265992a, 78 },
    { 0x8465992a, 34 },
    { 0x3525992c, 33 },
    { 0, 0 },
    { 0xc9f5992e, 96 },
    { 0x4335992e, 95 },
    { 0, 0 },
    { 0xa4a59931, 95 },
    { 0x59759932, 34 },
    { 0, 0 },
    { 0xe0b59934, 85 },
    { 0, 0 },
    { 0x3bc59936, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef5993a, 33 },
    { 0xee95993b, 76 },
    { 0, 0 },
    { 0xa0e5993d, 61 },
    { 0x4445993d, 69 },
    { 0x6065993e, 33 },
    { 0xede59940, 34 },
    { 0x1b259940, 95 },
    { 0x68a59940, 55 },
    { 0x7dd59943, 98 },
    { 0x18059944, 76 },
    { 0x8e759943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96359949, 55 },
    { 0x7485994a, 84 },
    { 0x8fe5994a, 33 },
    { 0xea95994a, 55 },
    { 0xbad5994d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f59952, 87 },
    { 0x24159952, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65259958, 78 },
    { 0x43059958, 61 },
    { 0xd3a5995a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b5995f, 91 },
    { 0xcb359960, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb459968, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d5996c, 95 },
    { 0, 0 },
    { 0x8e65996e, 61 },
    { 0x95f5996e, 33 },
    { 0xe015996f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22059973, 78 },
    { 0xe2859973, 23 },
    { 0, 0 },
    { 0x42659976, 76 },
    { 0, 0 },
    { 0x90d59978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad05997c, 17 },
    { 0xb915997d, 52 },
    { 0x4585997d, 95 },
    { 0xaa05997d, 95 },
    { 0, 0 },
    { 0x56759981, 98 },
    { 0xda559982, 81 },
    { 0, 0 },
    { 0x82859984, 78 },
    { 0x5d859985, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa59989, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2805998c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5459991, 85 },
    { 0xb3a59992, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64759999, 68 },
    { 0, 0 },
    { 0xdc65999b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245999e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c599a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a599a6, 98 },
    { 0x9f7599a7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f599b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75599b6, 99 },
    { 0x819599b7, 98 },
    { 0xc0b599b7, 95 },
    { 0x927599b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123599bf, 34 },
    { 0x8a1599bf, 67 },
    { 0x3e9599c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e599c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d599c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1599cd, 85 },
    { 0x436599cd, 95 },
    { 0xd0d599cd, 33 },
    { 0, 0 },
    { 0x189599d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984599d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd599da, 87 },
    { 0xfd8599db, 33 },
    { 0, 0 },
    { 0x4f3599dd, 91 },
    { 0x777599dd, 55 },
    { 0xe1d599de, 98 },
    { 0, 0 },
    { 0x6b599e1, 95 },
    { 0x366599e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35599e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b599e9, 33 },
    { 0x1ef599ea, 61 },
    { 0xabf599eb, 68 },
    { 0x24c599ec, 84 },
    { 0x696599ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x952599f0, 90 },
    { 0x4e1599f0, 33 },
    { 0, 0 },
    { 0x740599f3, 90 },
    { 0xb78599f4, 61 },
    { 0x8c0599f5, 98 },
    { 0x684599f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42599fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1599fe, 33 },
    { 0, 0 },
    { 0x99259a00, 61 },
    { 0x62459a00, 61 },
    { 0x60859a02, 98 },
    { 0xca759a03, 100 },
    { 0xc2b59a03, 90 },
    { 0x4d659a03, 65 },
    { 0, 0 },
    { 0x26459a07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1259a0b, 33 },
    { 0x49459a0c, 23 },
    { 0x34e59a0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa459a10, 95 },
    { 0, 0 },
    { 0xd5b59a12, 33 },
    { 0, 0 },
    { 0x29d59a14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc659a1b, 98 },
    { 0x9ea59a1c, 33 },
    { 0xaf759a1c, 85 },
    { 0x6be59a1b, 80 },
    { 0, 0 },
    { 0x70b59a20, 90 },
    { 0xf3c59a20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc59a26, 87 },
    { 0, 0 },
    { 0xfec59a28, 61 },
    { 0x4fa59a29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3959a2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad59a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b59a37, 24 },
    { 0x56259a38, 52 },
    { 0xc8d59a39, 23 },
    { 0xd8a59a3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23959a3d, 38 },
    { 0x6d059a3d, 23 },
    { 0x9c759a3f, 43 },
    { 0x27859a3f, 98 },
    { 0x71959a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96659a44, 85 },
    { 0, 0 },
    { 0xd8959a46, 98 },
    { 0xe7859a47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44859a4a, 76 },
    { 0, 0 },
    { 0xa4a59a4c, 90 },
    { 0x42d59a4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0359a50, 90 },
    { 0xa6d59a51, 75 },
    { 0xf2459a52, 90 },
    { 0, 0 },
    { 0x86859a54, 90 },
    { 0, 0 },
    { 0xd7859a56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a59a5a, 64 },
    { 0, 0 },
    { 0xf6359a5c, 43 },
    { 0x13959a5c, 46 },
    { 0, 0 },
    { 0xf0759a5f, 76 },
    { 0xd8759a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7959a68, 61 },
    { 0xbc259a69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d459a6f, 90 },
    { 0xd1059a70, 33 },
    { 0x21e59a70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99959a75, 46 },
    { 0x62e59a76, 33 },
    { 0xfcd59a77, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3259a7d, 84 },
    { 0xbf459a7d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9559a83, 43 },
    { 0x6b759a84, 90 },
    { 0x36259a85, 56 },
    { 0xeec59a86, 23 },
    { 0xc59a83, 23 },
    { 0x80c59a88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c659a8d, 43 },
    { 0xec59a8d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba959a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f759a96, 43 },
    { 0xd9d59a96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0259a9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c259a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5759aa0, 77 },
    { 0xe2259aa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa59aa4, 61 },
    { 0x80359aa5, 90 },
    { 0x4d659aa4, 90 },
    { 0, 0 },
    { 0xf9b59aa8, 90 },
    { 0x6ce59aa8, 33 },
    { 0x55859aaa, 90 },
    { 0x4cd59aab, 17 },
    { 0, 0 },
    { 0x3a259aad, 61 },
    { 0xa8359aae, 88 },
    { 0x95e59aae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d859ab3, 88 },
    { 0xa0759ab4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90559ab9, 98 },
    { 0, 0 },
    { 0xffe59abb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6559abe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8959ac1, 90 },
    { 0xbb459ac1, 93 },
    { 0, 0 },
    { 0x9be59ac4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda659ac7, 98 },
    { 0x17959ac8, 90 },
    { 0xf6d59ac9, 33 },
    { 0xe2b59ac9, 77 },
    { 0, 0 },
    { 0x57c59acc, 33 },
    { 0x5b359acc, 76 },
    { 0x33459acd, 76 },
    { 0, 0 },
    { 0xbe59ad0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x7959ad3, 78 },
    { 0xef059ad3, 23 },
    { 0xcba59ad4, 90 },
    { 0, 0 },
    { 0x6759ad7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96759adc, 66 },
    { 0x1d859add, 76 },
    { 0x8e959ade, 55 },
    { 0xa559adc, 23 },
    { 0xad959ae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf59ae6, 23 },
    { 0, 0 },
    { 0xf5e59ae8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f359aeb, 91 },
    { 0xd0159aec, 91 },
    { 0, 0 },
    { 0x45859aee, 90 },
    { 0x7ee59aef, 55 },
    { 0xf1459af0, 84 },
    { 0, 0 },
    { 0x42459af2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c59af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f59af8, 95 },
    { 0x58859af8, 61 },
    { 0x96659afa, 33 },
    { 0, 0 },
    { 0xc6159afc, 33 },
    { 0xad759afc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e659b01, 98 },
    { 0, 0 },
    { 0x3ad59b03, 34 },
    { 0x2a259b04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b959b07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f59b0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39959b0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b759b18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5959b1d, 88 },
    { 0x5cf59b1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e359b22, 61 },
    { 0x4aa59b23, 46 },
    { 0xca159b24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d59b2a, 96 },
    { 0xacf59b2b, 98 },
    { 0, 0 },
    { 0x92e59b2d, 33 },
    { 0, 0 },
    { 0x85059b2f, 98 },
    { 0, 0 },
    { 0xda359b31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b59b35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb859b3a, 90 },
    { 0xeb259b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81759b3e, 34 },
    { 0xfbd59b3f, 46 },
    { 0x7c259b3e, 98 },
    { 0xb7159b41, 61 },
    { 0, 0 },
    { 0xb3e59b43, 90 },
    { 0, 0 },
    { 0x4ff59b45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94559b48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e59b4b, 75 },
    { 0xb7c59b4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2359b4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e59b56, 23 },
    { 0, 0 },
    { 0xf3c59b58, 91 },
    { 0, 0 },
    { 0xb4859b5a, 93 },
    { 0xfb259b5b, 75 },
    { 0xea659b5c, 95 },
    { 0x5a059b5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52959b62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b859b6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c459b73, 93 },
    { 0, 0 },
    { 0x1359b75, 34 },
    { 0xbe359b76, 55 },
    { 0x62e59b76, 90 },
    { 0x6b559b78, 95 },
    { 0x2fb59b79, 84 },
    { 0x62a59b79, 60 },
    { 0x26559b7b, 17 },
    { 0, 0 },
    { 0x69c59b7d, 98 },
    { 0, 0 },
    { 0xa5759b7f, 23 },
    { 0x32559b80, 98 },
    { 0x5b759b81, 90 },
    { 0xe6359b82, 83 },
    { 0x75959b7f, 90 },
    { 0, 0 },
    { 0xd7e59b85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34559b8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf59b93, 53 },
    { 0, 0 },
    { 0xe2259b95, 33 },
    { 0, 0 },
    { 0x1f059b97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce59b9a, 69 },
    { 0x6c459b9b, 93 },
    { 0xad059b9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f59ba1, 95 },
    { 0x42059ba1, 98 },
    { 0xec859ba1, 95 },
    { 0x24059ba4, 98 },
    { 0, 0 },
    { 0x94b59ba6, 66 },
    { 0x1b659ba6, 43 },
    { 0, 0 },
    { 0x6ab59ba9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e259bb1, 93 },
    { 0, 0 },
    { 0xcb859bb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43559bb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x17759bb9, 91 },
    { 0, 0 },
    { 0xccd59bbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b759bc0, 93 },
    { 0x15059bc1, 23 },
    { 0, 0 },
    { 0xd9e59bc3, 33 },
    { 0x73f59bc4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc59bca, 33 },
    { 0, 0 },
    { 0x4b259bcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa859bd1, 90 },
    { 0x69559bd2, 61 },
    { 0x54c59bd2, 33 },
    { 0xecb59bd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50959bd8, 48 },
    { 0x6dd59bd9, 97 },
    { 0x9f159bd8, 23 },
    { 0xd7359bdb, 99 },
    { 0xd4e59bda, 55 },
    { 0xbcc59bdb, 23 },
    { 0x5c659bde, 23 },
    { 0x39759bdf, 95 },
    { 0x86859bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a459be8, 87 },
    { 0xa0d59be8, 33 },
    { 0, 0 },
    { 0xb59beb, 46 },
    { 0xfc159bec, 52 },
    { 0xb3e59bed, 98 },
    { 0x16359bec, 36 },
    { 0xdc059bed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7059bf7, 66 },
    { 0x65959bf8, 33 },
    { 0xe3159bf9, 76 },
    { 0x45659bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e59c01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6459c05, 12 },
    { 0, 0 },
    { 0xd4c59c07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea59c15, 55 },
    { 0xfe459c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a59c19, 33 },
    { 0x88459c1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58759c1e, 66 },
    { 0xa0e59c1e, 52 },
    { 0x159c1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf659c25, 33 },
    { 0x19059c26, 46 },
    { 0, 0 },
    { 0xd6059c28, 90 },
    { 0xb1859c29, 40 },
    { 0x65d59c2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63059c2f, 93 },
    { 0xc8659c30, 98 },
    { 0xf1e59c31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3259c38, 88 },
    { 0, 0 },
    { 0x1b359c3a, 97 },
    { 0xc1b59c3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38859c43, 23 },
    { 0x5f759c43, 98 },
    { 0x2d059c44, 55 },
    { 0, 0 },
    { 0x4f759c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df59c4e, 34 },
    { 0xd1459c4f, 33 },
    { 0x96a59c4e, 100 },
    { 0x22659c50, 78 },
    { 0xcf259c52, 61 },
    { 0xf5759c53, 99 },
    { 0, 0 },
    { 0x89959c55, 33 },
    { 0xf8c59c56, 98 },
    { 0, 0 },
    { 0x11259c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da59c5b, 90 },
    { 0, 0 },
    { 0xe1859c5d, 66 },
    { 0x9a059c5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50359c64, 90 },
    { 0x27259c65, 55 },
    { 0xa6659c66, 95 },
    { 0x29159c67, 52 },
    { 0x26d59c68, 52 },
    { 0xc3159c69, 90 },
    { 0, 0 },
    { 0xc059c6b, 95 },
    { 0x7fc59c6b, 95 },
    { 0xb0b59c6d, 90 },
    { 0, 0 },
    { 0x16059c6f, 36 },
    { 0x77d59c70, 75 },
    { 0xa2b59c6f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab59c75, 23 },
    { 0, 0 },
    { 0xfe059c77, 43 },
    { 0, 0 },
    { 0x14359c79, 17 },
    { 0x4e859c7a, 33 },
    { 0x3d59c7b, 76 },
    { 0, 0 },
    { 0x71a59c7d, 33 },
    { 0x9159c7e, 90 },
    { 0xc0e59c7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad759c8d, 90 },
    { 0xb7059c8e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1259c91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89659c94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc259c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef259ca4, 76 },
    { 0xbe159ca5, 91 },
    { 0xd8259ca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e59cac, 90 },
    { 0xb4659cad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53859cb0, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f59cb3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b59cb6, 34 },
    { 0, 0 },
    { 0xa9659cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45959cbc, 46 },
    { 0x86b59cbc, 55 },
    { 0xa859cbe, 61 },
    { 0xfe959cbf, 76 },
    { 0xfcd59cc0, 23 },
    { 0xc4759cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a59cc5, 68 },
    { 0x2cd59cc5, 55 },
    { 0xca959cc6, 36 },
    { 0xd7859cc5, 65 },
    { 0x49059cc9, 66 },
    { 0xd3959cca, 55 },
    { 0xb9f59cc7, 91 },
    { 0x54a59cc7, 55 },
    { 0, 0 },
    { 0xd3259cce, 98 },
    { 0x75a59cce, 33 },
    { 0xed959cd0, 76 },
    { 0xc6b59cd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7659cd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8459cdc, 43 },
    { 0x1df59cdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74859ce6, 90 },
    { 0xad559ce6, 61 },
    { 0x4a559ce8, 34 },
    { 0xac759ce9, 91 },
    { 0xba359cea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb59cf0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a59cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa59cf8, 43 },
    { 0xcd159cf8, 67 },
    { 0, 0 },
    { 0x5a859cfb, 33 },
    { 0xafb59cfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79559d00, 17 },
    { 0x6b559d00, 33 },
    { 0xe3b59d02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d59d06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf59d0d, 38 },
    { 0xa6459d0e, 95 },
    { 0xed459d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a59d13, 33 },
    { 0, 0 },
    { 0x20559d15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd59d18, 90 },
    { 0xd759d19, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a59d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f59d24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59d28, 23 },
    { 0x4dc59d29, 77 },
    { 0, 0 },
    { 0xb0f59d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19259d2f, 33 },
    { 0x88859d30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a59d35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f59d39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea559d3d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6559d44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f59d4b, 100 },
    { 0x66159d4c, 98 },
    { 0x2c59d4d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7959d51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8759d56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6859d5a, 55 },
    { 0, 0 },
    { 0xc4359d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f59d5f, 61 },
    { 0x6959d60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db59d69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d859d6c, 33 },
    { 0x73e59d6d, 98 },
    { 0xcbf59d6e, 33 },
    { 0xa9a59d6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a59d73, 76 },
    { 0, 0 },
    { 0xaed59d75, 83 },
    { 0x79b59d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14359d79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1359d7c, 95 },
    { 0x41259d7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c59d84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d59d89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99259d8c, 90 },
    { 0x17459d8c, 34 },
    { 0, 0 },
    { 0x5b059d8f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe059d92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86859d9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a59d9f, 78 },
    { 0x2a359d9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c159da6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f259daa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x10459dad, 90 },
    { 0x9ee59dad, 34 },
    { 0x68e59daf, 90 },
    { 0xf4e59daf, 43 },
    { 0xdd559db1, 55 },
    { 0xcc459db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c59db6, 23 },
    { 0xaa959db7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea59dba, 76 },
    { 0xf5359dbb, 90 },
    { 0x25759dbc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d59dc0, 91 },
    { 0xd559dc1, 98 },
    { 0, 0 },
    { 0x40559dc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x90659dc6, 61 },
    { 0x89159dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc59dd1, 61 },
    { 0x14759dd1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac59dd9, 33 },
    { 0, 0 },
    { 0xe459ddb, 85 },
    { 0x33859ddb, 43 },
    { 0, 0 },
    { 0xa0159dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98259de3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b59de9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd59dec, 90 },
    { 0x8c659dec, 90 },
    { 0xa5d59dec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf759df1, 43 },
    { 0, 0 },
    { 0xa759df3, 61 },
    { 0, 0 },
    { 0xe7d59df5, 65 },
    { 0x1fd59df5, 33 },
    { 0x71359df6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73259dfc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81659e00, 93 },
    { 0xdb459e00, 34 },
    { 0x5a959e02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28859e06, 90 },
    { 0, 0 },
    { 0xb3b59e08, 61 },
    { 0x55859e09, 33 },
    { 0xd4459e0a, 90 },
    { 0xbeb59e0a, 23 },
    { 0xd7659e09, 69 },
    { 0x6b759e0d, 67 },
    { 0, 0 },
    { 0x8e659e0f, 33 },
    { 0x57559e10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa59e13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad059e1a, 33 },
    { 0, 0 },
    { 0xc8c59e1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef959e1f, 55 },
    { 0x1ef59e20, 81 },
    { 0x9ee59e20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d59e24, 61 },
    { 0x7d359e24, 90 },
    { 0x1c159e26, 23 },
    { 0, 0 },
    { 0x55e59e28, 90 },
    { 0x37c59e29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99959e31, 98 },
    { 0, 0 },
    { 0x4c559e33, 87 },
    { 0xc9959e34, 68 },
    { 0xe8c59e33, 100 },
    { 0x7fd59e35, 61 },
    { 0x9ea59e37, 90 },
    { 0x81059e33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66859e3d, 93 },
    { 0x69659e3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b459e45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c359e49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78059e4d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab59e64, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4259e68, 58 },
    { 0xc2659e69, 100 },
    { 0, 0 },
    { 0x37f59e6b, 33 },
    { 0xf6c59e6b, 76 },
    { 0x4b759e6b, 52 },
    { 0xe8459e6e, 85 },
    { 0x19259e6f, 65 },
    { 0x8eb59e6e, 23 },
    { 0x54f59e6d, 23 },
    { 0x82759e71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36459e75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8559e78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0959e7f, 33 },
    { 0x61659e80, 23 },
    { 0x90e59e7f, 33 },
    { 0xc7159e82, 81 },
    { 0, 0 },
    { 0x42459e84, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75359e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e59e8f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x82159e92, 23 },
    { 0x9f59e93, 61 },
    { 0xb7159e93, 98 },
    { 0x59959e95, 17 },
    { 0xef959e94, 84 },
    { 0x31959e97, 90 },
    { 0x34f59e96, 23 },
    { 0, 0 },
    { 0xe3659e9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62559ea0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee759ea6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3359ea9, 46 },
    { 0, 0 },
    { 0x96559eab, 95 },
    { 0xc5f59eac, 76 },
    { 0x4b859ead, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda59eb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84259ec0, 33 },
    { 0x90c59ec1, 76 },
    { 0xea259ec2, 10 },
    { 0, 0 },
    { 0xd2159ec4, 87 },
    { 0, 0 },
    { 0x3fd59ec6, 23 },
    { 0x85459ec7, 93 },
    { 0, 0 },
    { 0x64b59ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e59ece, 61 },
    { 0, 0 },
    { 0x2ed59ed0, 98 },
    { 0xe0859ed1, 23 },
    { 0xc6659ed1, 61 },
    { 0xbcd59ed1, 23 },
    { 0x30459ed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc859ed7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d59eda, 34 },
    { 0x9fb59edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c359edf, 90 },
    { 0xb3e59ee0, 90 },
    { 0x8d259ee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8859ee9, 33 },
    { 0, 0 },
    { 0xc4c59eeb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b59ef2, 100 },
    { 0x8b759ef2, 93 },
    { 0xf3d59ef4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a159ef8, 33 },
    { 0x4cb59ef9, 90 },
    { 0x3f959efa, 99 },
    { 0xa9f59efb, 52 },
    { 0x14c59efb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11159f01, 90 },
    { 0, 0 },
    { 0x7cf59f03, 46 },
    { 0x18b59f04, 34 },
    { 0xc9759f05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c59f0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e159f0e, 76 },
    { 0, 0 },
    { 0x21459f10, 33 },
    { 0x38759f10, 76 },
    { 0, 0 },
    { 0x60559f13, 33 },
    { 0xc2b59f13, 61 },
    { 0, 0 },
    { 0x27f59f16, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73559f1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96859f21, 95 },
    { 0x41d59f22, 90 },
    { 0x18259f23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf59f26, 40 },
    { 0x42959f26, 76 },
    { 0x31559f27, 90 },
    { 0x53959f28, 33 },
    { 0x31459f2a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c59f32, 93 },
    { 0, 0 },
    { 0x7f859f34, 61 },
    { 0, 0 },
    { 0x9da59f36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89459f39, 23 },
    { 0, 0 },
    { 0xb7159f3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14959f3f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af59f42, 67 },
    { 0xa2259f43, 35 },
    { 0xd5a59f42, 90 },
    { 0xa9a59f45, 90 },
    { 0x60859f42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e59f49, 23 },
    { 0x29159f49, 76 },
    { 0xd3859f4b, 76 },
    { 0xe8859f49, 61 },
    { 0x5cf59f4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc59f5d, 43 },
    { 0xdef59f5e, 23 },
    { 0xeaf59f5f, 75 },
    { 0xdfa59f5e, 66 },
    { 0, 0 },
    { 0x99d59f62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd59f68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0859f6b, 44 },
    { 0, 0 },
    { 0xb8c59f6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f59f72, 56 },
    { 0xdf859f73, 78 },
    { 0x38259f74, 17 },
    { 0x7ee59f74, 66 },
    { 0x57e59f73, 61 },
    { 0x7f459f75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1359f7d, 55 },
    { 0xe9359f7d, 90 },
    { 0x25559f7e, 36 },
    { 0x6d159f80, 66 },
    { 0x52159f7f, 43 },
    { 0, 0 },
    { 0xbea59f83, 85 },
    { 0xd4159f83, 46 },
    { 0, 0 },
    { 0x6b059f86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62359f89, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x49459f8c, 43 },
    { 0xdef59f8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30659f91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a59f95, 90 },
    { 0xdc659f96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd59f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78359f9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76959f9f, 43 },
    { 0, 0 },
    { 0xc7a59fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d59fa8, 23 },
    { 0, 0 },
    { 0x6b159faa, 90 },
    { 0, 0 },
    { 0xeb159fac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x51859faf, 43 },
    { 0, 0 },
    { 0xeb959fb1, 76 },
    { 0, 0 },
    { 0xa5d59fb3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b59fb7, 76 },
    { 0x9a459fb7, 90 },
    { 0, 0 },
    { 0x59259fba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c659fca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84459fcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff59fd8, 90 },
    { 0x2e59fd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe059fdd, 90 },
    { 0, 0 },
    { 0x32d59fdf, 76 },
    { 0, 0 },
    { 0x54059fe1, 95 },
    { 0x57159fe2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86359fe5, 61 },
    { 0x79459fe5, 34 },
    { 0x12359fe5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4559fec, 52 },
    { 0x2c59fed, 76 },
    { 0x8c159fed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba59ff4, 33 },
    { 0x68a59ff4, 90 },
    { 0x14159ff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59ffb, 90 },
    { 0x78959ffc, 94 },
    { 0xb3959ffd, 40 },
    { 0xe9359ffd, 61 },
    { 0xb7c59fff, 93 },
    { 0x2d59ffc, 93 },
    { 0x2aa5a001, 67 },
    { 0xfa559ffd, 61 },
    { 0x91b5a003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115a006, 46 },
    { 0x5cf5a007, 95 },
    { 0x96f5a008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc5a00c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c5a00f, 33 },
    { 0x56c5a010, 90 },
    { 0x3b5a00f, 90 },
    { 0xb1f5a012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc615a018, 61 },
    { 0x68a5a019, 90 },
    { 0x8e45a01a, 76 },
    { 0xc445a01a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf625a022, 66 },
    { 0x7b75a023, 95 },
    { 0xad25a022, 90 },
    { 0, 0 },
    { 0x6745a026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba5a029, 76 },
    { 0, 0 },
    { 0xc115a02b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b5a032, 23 },
    { 0, 0 },
    { 0x3325a034, 76 },
    { 0xbfe5a034, 95 },
    { 0, 0 },
    { 0xb505a037, 93 },
    { 0x9085a038, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6725a03c, 33 },
    { 0x8d35a03d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a5a041, 23 },
    { 0, 0 },
    { 0x64c5a043, 84 },
    { 0xf3e5a044, 93 },
    { 0xb75a044, 98 },
    { 0, 0 },
    { 0x3905a047, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x145a04a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e35a04d, 98 },
    { 0x95a5a04e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4375a051, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d5a05f, 33 },
    { 0, 0 },
    { 0xd935a061, 90 },
    { 0, 0 },
    { 0x7b45a063, 23 },
    { 0x7095a063, 34 },
    { 0xa455a063, 69 },
    { 0x7855a065, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c5a070, 95 },
    { 0xbeb5a071, 34 },
    { 0x1b95a072, 33 },
    { 0xebe5a072, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a5a076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f5a080, 98 },
    { 0x1335a080, 98 },
    { 0xbea5a082, 88 },
    { 0x4a95a082, 90 },
    { 0xa195a084, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5a089, 98 },
    { 0xa935a08a, 61 },
    { 0x2415a08b, 61 },
    { 0xa945a08c, 66 },
    { 0, 0 },
    { 0x25a5a08e, 98 },
    { 0, 0 },
    { 0xb275a090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa065a095, 61 },
    { 0x31a5a096, 95 },
    { 0x2235a096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7095a09a, 33 },
    { 0, 0 },
    { 0xa415a09c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb815a0a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf85a0a7, 95 },
    { 0xfa55a0a8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xec75a0ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda15a0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf975a0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd485a0b8, 61 },
    { 0, 0 },
    { 0xbeb5a0ba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d5a0bd, 33 },
    { 0x5825a0be, 23 },
    { 0x81e5a0bd, 61 },
    { 0xb585a0c0, 98 },
    { 0x1b55a0bf, 34 },
    { 0xb675a0c2, 76 },
    { 0xe0b5a0bf, 95 },
    { 0, 0 },
    { 0x44b5a0c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a5a0c8, 95 },
    { 0xde25a0c9, 34 },
    { 0xb335a0c9, 90 },
    { 0xe945a0cb, 98 },
    { 0xf055a0cc, 88 },
    { 0x61e5a0ca, 91 },
    { 0x3565a0cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd5a0dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df5a0e2, 100 },
    { 0xaa75a0e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef5a0e6, 90 },
    { 0, 0 },
    { 0x64d5a0e8, 33 },
    { 0xc525a0e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9295a0ed, 94 },
    { 0xf285a0ee, 90 },
    { 0xe105a0ed, 46 },
    { 0x7c95a0f0, 33 },
    { 0x90c5a0f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b55a0f4, 98 },
    { 0xd215a0f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6255a0f8, 91 },
    { 0x8785a0f9, 76 },
    { 0xdb75a0fa, 61 },
    { 0xac45a0fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5a102, 31 },
    { 0, 0 },
    { 0xd255a104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce5a107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d5a10b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc5a10e, 90 },
    { 0xb1a5a10f, 87 },
    { 0x78d5a110, 98 },
    { 0xea95a10e, 76 },
    { 0x10e5a10f, 23 },
    { 0x6815a10f, 77 },
    { 0xe1a5a114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7045a117, 52 },
    { 0xa215a118, 61 },
    { 0xcff5a119, 95 },
    { 0x7955a11a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c5a11d, 76 },
    { 0xecb5a11e, 98 },
    { 0x7b85a11f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f75a125, 33 },
    { 0x9885a125, 34 },
    { 0x1925a127, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a35a12d, 88 },
    { 0x8925a12e, 98 },
    { 0xa7f5a12f, 95 },
    { 0x4ce5a12e, 61 },
    { 0x5985a131, 87 },
    { 0xcf45a131, 55 },
    { 0, 0 },
    { 0x1805a134, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6635a142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb15a145, 46 },
    { 0x9475a146, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa815a14a, 90 },
    { 0x5415a14b, 33 },
    { 0xe085a14c, 90 },
    { 0, 0 },
    { 0x5505a14e, 23 },
    { 0xd1f5a14f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d5a155, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875a15b, 52 },
    { 0, 0 },
    { 0xe735a15d, 76 },
    { 0xbc15a15e, 90 },
    { 0x3dd5a15f, 33 },
    { 0x9305a160, 34 },
    { 0, 0 },
    { 0xdd05a162, 70 },
    { 0, 0 },
    { 0x1325a164, 81 },
    { 0xf975a164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d5a169, 87 },
    { 0x7955a16a, 55 },
    { 0, 0 },
    { 0x12c5a16c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905a16f, 90 },
    { 0x9645a170, 23 },
    { 0xe9a5a171, 34 },
    { 0xe75a172, 77 },
    { 0x8165a16f, 61 },
    { 0xfc75a174, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6645a17e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b95a181, 23 },
    { 0, 0 },
    { 0xcce5a183, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef05a187, 90 },
    { 0, 0 },
    { 0x8695a189, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8725a18c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25a18f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135a194, 90 },
    { 0x53a5a195, 90 },
    { 0xf455a194, 85 },
    { 0x7a75a197, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a85a19a, 33 },
    { 0, 0 },
    { 0x4595a19c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d35a1a4, 61 },
    { 0x87d5a1a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf015a1a9, 83 },
    { 0x51b5a1aa, 23 },
    { 0x11f5a1ab, 61 },
    { 0, 0 },
    { 0xb995a1ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715a1b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd5a1b4, 61 },
    { 0xf115a1b5, 23 },
    { 0x5395a1b6, 33 },
    { 0x8065a1b7, 65 },
    { 0xed15a1b6, 23 },
    { 0, 0 },
    { 0xe9c5a1ba, 60 },
    { 0x3e25a1bb, 87 },
    { 0xbbc5a1bc, 61 },
    { 0, 0 },
    { 0xa785a1be, 85 },
    { 0, 0 },
    { 0x26a5a1c0, 55 },
    { 0xdbe5a1c1, 95 },
    { 0xeea5a1c2, 67 },
    { 0xe0e5a1c3, 55 },
    { 0x7625a1c4, 90 },
    { 0, 0 },
    { 0xa105a1c6, 43 },
    { 0, 0 },
    { 0x21b5a1c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b5a1cb, 67 },
    { 0x3ba5a1cc, 76 },
    { 0x1d05a1cc, 90 },
    { 0xeff5a1cc, 84 },
    { 0x9525a1cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b5a1d3, 91 },
    { 0x13a5a1d3, 88 },
    { 0x38b5a1d5, 90 },
    { 0xc545a1d5, 95 },
    { 0x2585a1d7, 33 },
    { 0, 0 },
    { 0xcb95a1d9, 17 },
    { 0, 0 },
    { 0x6655a1db, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5a1e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a15a1e5, 33 },
    { 0, 0 },
    { 0x8de5a1e7, 34 },
    { 0, 0 },
    { 0xa225a1e9, 61 },
    { 0, 0 },
    { 0xb505a1eb, 90 },
    { 0xa095a1eb, 33 },
    { 0xd4e5a1ed, 52 },
    { 0x8e95a1ed, 90 },
    { 0xec15a1ef, 81 },
    { 0xe4c5a1f0, 61 },
    { 0x78d5a1ef, 52 },
    { 0, 0 },
    { 0xe7d5a1f3, 93 },
    { 0x1195a1f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d5a1f7, 93 },
    { 0x75e5a1f8, 67 },
    { 0x8cd5a1f7, 33 },
    { 0x8b35a1fa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5a1fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e5a202, 61 },
    { 0x9fd5a203, 93 },
    { 0x3b15a203, 98 },
    { 0x9a05a204, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5a208, 95 },
    { 0, 0 },
    { 0xa4c5a20a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475a20e, 55 },
    { 0x6295a20f, 17 },
    { 0xf265a210, 23 },
    { 0x5825a211, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a5a21a, 90 },
    { 0x5c55a21b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85a21e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c5a222, 76 },
    { 0, 0 },
    { 0x1695a224, 90 },
    { 0xb705a224, 90 },
    { 0xb705a225, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd545a229, 78 },
    { 0xe575a22a, 98 },
    { 0x6865a22b, 55 },
    { 0x41f5a229, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f25a231, 95 },
    { 0x41a5a232, 90 },
    { 0x1955a233, 77 },
    { 0xcc85a231, 17 },
    { 0, 0 },
    { 0x7d85a236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2025a239, 61 },
    { 0xce55a239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9165a23e, 98 },
    { 0xf075a23f, 76 },
    { 0x3e45a23e, 33 },
    { 0xc605a23f, 98 },
    { 0x66f5a242, 85 },
    { 0x2465a243, 76 },
    { 0x4d05a244, 87 },
    { 0xf315a240, 100 },
    { 0x2935a240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d5a249, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f5a24c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db5a24f, 98 },
    { 0x2ee5a24f, 19 },
    { 0x985a251, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4475a254, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb465a257, 33 },
    { 0, 0 },
    { 0x2e95a259, 78 },
    { 0, 0 },
    { 0x3f85a25b, 33 },
    { 0xecf5a25c, 23 },
    { 0x84c5a25d, 33 },
    { 0, 0 },
    { 0xa85a25f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d5a269, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce5a270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f5a273, 88 },
    { 0, 0 },
    { 0xb025a275, 36 },
    { 0xbb75a275, 94 },
    { 0xa65a277, 95 },
    { 0, 0 },
    { 0xaea5a279, 33 },
    { 0, 0 },
    { 0xd635a27b, 91 },
    { 0x8185a27c, 78 },
    { 0xd1c5a27d, 95 },
    { 0x2f5a27e, 34 },
    { 0x59d5a27f, 90 },
    { 0x2175a27c, 98 },
    { 0xa565a281, 52 },
    { 0, 0 },
    { 0x8425a283, 46 },
    { 0x675a284, 81 },
    { 0x1d05a285, 61 },
    { 0x625a286, 61 },
    { 0xb005a283, 76 },
    { 0, 0 },
    { 0x7165a289, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf555a291, 37 },
    { 0x4c75a291, 90 },
    { 0x55a293, 95 },
    { 0x275a293, 23 },
    { 0x8825a293, 33 },
    { 0xa65a296, 76 },
    { 0x6dc5a296, 33 },
    { 0xd9a5a296, 23 },
    { 0xe7a5a299, 90 },
    { 0x955a29a, 90 },
    { 0xf65a293, 23 },
    { 0x3685a294, 90 },
    { 0xe75a295, 23 },
    { 0, 0 },
    { 0x9885a29f, 95 },
    { 0x6ce5a29f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd5a2a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6455a2a9, 61 },
    { 0, 0 },
    { 0xd365a2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1295a2b2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd675a2b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5a2b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a5a2bb, 95 },
    { 0xe385a2bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa15a2bf, 98 },
    { 0xc305a2c0, 23 },
    { 0xa5a2c1, 33 },
    { 0xe6f5a2c0, 95 },
    { 0, 0 },
    { 0x335a2c4, 61 },
    { 0, 0 },
    { 0xfc65a2c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb5a2cc, 23 },
    { 0x1585a2cd, 91 },
    { 0xaea5a2ce, 33 },
    { 0xdac5a2cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a2d3, 61 },
    { 0x3a15a2d3, 61 },
    { 0, 0 },
    { 0xfde5a2d6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365a2d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd95a2dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc5a2e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5a2e9, 33 },
    { 0, 0 },
    { 0x825a2eb, 66 },
    { 0x6ca5a2eb, 40 },
    { 0, 0 },
    { 0x8995a2ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d5a2f2, 33 },
    { 0x5d05a2f3, 61 },
    { 0xa255a2f4, 33 },
    { 0xd975a2f4, 34 },
    { 0, 0 },
    { 0x2295a2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d15a305, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4995a308, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c5a30b, 34 },
    { 0x6555a30b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb725a313, 98 },
    { 0x8e35a313, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa855a31a, 97 },
    { 0x1c55a31b, 95 },
    { 0xb205a31b, 33 },
    { 0x57e5a31c, 98 },
    { 0xf045a31e, 95 },
    { 0, 0 },
    { 0xdf5a320, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3675a330, 90 },
    { 0x38c5a331, 59 },
    { 0x38d5a331, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa5a335, 85 },
    { 0x36e5a336, 55 },
    { 0, 0 },
    { 0xd8a5a338, 95 },
    { 0x6205a339, 33 },
    { 0xaee5a33a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b5a346, 17 },
    { 0x7625a347, 33 },
    { 0x79c5a348, 85 },
    { 0x34b5a349, 66 },
    { 0xc1f5a347, 33 },
    { 0xe735a34b, 90 },
    { 0xcad5a34b, 91 },
    { 0x445a34b, 98 },
    { 0xd735a34d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc725a353, 34 },
    { 0xa745a354, 90 },
    { 0x59f5a355, 98 },
    { 0x70f5a356, 65 },
    { 0xf815a357, 90 },
    { 0x6a85a358, 100 },
    { 0x8745a355, 90 },
    { 0xca15a35a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd725a360, 95 },
    { 0x54c5a361, 100 },
    { 0x3325a362, 100 },
    { 0x79c5a363, 33 },
    { 0xe525a362, 94 },
    { 0x9495a365, 95 },
    { 0x6d45a365, 95 },
    { 0x10e5a367, 76 },
    { 0xbe15a368, 98 },
    { 0, 0 },
    { 0xe6a5a36a, 46 },
    { 0, 0 },
    { 0xf2f5a36c, 33 },
    { 0xbee5a36d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d5a371, 75 },
    { 0, 0 },
    { 0xdaf5a373, 98 },
    { 0x1065a374, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5385a37c, 76 },
    { 0x8fd5a37d, 33 },
    { 0x5055a37e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf725a382, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c15a385, 76 },
    { 0xafc5a386, 90 },
    { 0xa885a385, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b5a38a, 100 },
    { 0, 0 },
    { 0x1b35a38c, 81 },
    { 0x3da5a38d, 98 },
    { 0x785a38d, 100 },
    { 0xa825a38f, 91 },
    { 0xeef5a390, 61 },
    { 0xcf85a390, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac5a39b, 98 },
    { 0xdda5a39c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe615a39f, 33 },
    { 0x9cc5a3a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55a3a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd285a3ab, 52 },
    { 0, 0 },
    { 0x8875a3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6405a3b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f5a3b6, 23 },
    { 0x1485a3b6, 61 },
    { 0x6bd5a3b8, 46 },
    { 0, 0 },
    { 0xbc05a3ba, 75 },
    { 0, 0 },
    { 0xdfc5a3bc, 33 },
    { 0xb365a3bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2605a3c1, 68 },
    { 0x10f5a3c2, 95 },
    { 0, 0 },
    { 0x1c95a3c4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f5a3c7, 85 },
    { 0x8d95a3c8, 93 },
    { 0, 0 },
    { 0xf0c5a3ca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8975a3cd, 33 },
    { 0x1f75a3ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f5a3d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab25a3d4, 23 },
    { 0x1ed5a3d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb015a3da, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5a3e0, 95 },
    { 0, 0 },
    { 0xaa65a3e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e5a3eb, 66 },
    { 0xb5d5a3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5a3f0, 76 },
    { 0xe3c5a3f1, 90 },
    { 0x3895a3f0, 69 },
    { 0xeca5a3f3, 33 },
    { 0xcfb5a3f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9105a3f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4395a3fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c5a3fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305a404, 33 },
    { 0, 0 },
    { 0x6b75a406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1795a40b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f15a412, 23 },
    { 0, 0 },
    { 0x975a414, 93 },
    { 0, 0 },
    { 0xce55a416, 95 },
    { 0x72f5a417, 90 },
    { 0x6f65a418, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf335a41d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c5a420, 36 },
    { 0xe7e5a421, 55 },
    { 0xa125a422, 75 },
    { 0xd3b5a423, 90 },
    { 0xdb95a423, 65 },
    { 0, 0 },
    { 0xbee5a426, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b5a429, 66 },
    { 0xb445a42a, 95 },
    { 0xa295a42a, 95 },
    { 0, 0 },
    { 0x93c5a42d, 98 },
    { 0xa5d5a42e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1545a437, 23 },
    { 0x15d5a438, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d55a43c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d35a441, 93 },
    { 0, 0 },
    { 0x4aa5a443, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c75a446, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5665a44b, 61 },
    { 0x8765a44c, 23 },
    { 0x1325a44d, 90 },
    { 0, 0 },
    { 0xfca5a44f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c45a454, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1015a457, 61 },
    { 0xcc95a457, 23 },
    { 0, 0 },
    { 0x23a5a45a, 98 },
    { 0xf795a45b, 98 },
    { 0xa2e5a45a, 98 },
    { 0x6b35a45b, 90 },
    { 0x5ff5a45e, 55 },
    { 0x3905a45f, 61 },
    { 0, 0 },
    { 0x2b45a461, 90 },
    { 0x7275a462, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf5a465, 98 },
    { 0x78e5a466, 43 },
    { 0x69f5a467, 76 },
    { 0, 0 },
    { 0x9635a469, 90 },
    { 0x5fe5a469, 40 },
    { 0x5e5a46b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8005a473, 46 },
    { 0, 0 },
    { 0x6ca5a475, 91 },
    { 0xe215a475, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be5a47a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf885a47e, 98 },
    { 0x2315a47f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa775a486, 61 },
    { 0xb675a487, 61 },
    { 0x5475a488, 93 },
    { 0xc095a488, 90 },
    { 0x9235a48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf595a48f, 100 },
    { 0x4005a490, 61 },
    { 0xe3a5a490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5a494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd5a497, 90 },
    { 0, 0 },
    { 0xf965a499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5a4a4, 98 },
    { 0xb0e5a4a5, 98 },
    { 0xb65a4a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad5a4a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905a4ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1745a4b0, 34 },
    { 0xec65a4b1, 95 },
    { 0xbc85a4b0, 95 },
    { 0, 0 },
    { 0xb705a4b4, 61 },
    { 0, 0 },
    { 0xd15a4b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d5a4b9, 33 },
    { 0xde65a4b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd85a4bf, 76 },
    { 0x7785a4c0, 52 },
    { 0, 0 },
    { 0x5b75a4c2, 61 },
    { 0xe0c5a4c3, 43 },
    { 0x36d5a4c4, 61 },
    { 0x3b05a4c5, 77 },
    { 0xd2e5a4c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2195a4c9, 90 },
    { 0x3f15a4ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25a4cd, 93 },
    { 0xeaf5a4cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded5a4d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a05a4db, 61 },
    { 0, 0 },
    { 0x25a5a4dd, 95 },
    { 0x7095a4de, 55 },
    { 0xa095a4df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef65a4e3, 95 },
    { 0, 0 },
    { 0xd5f5a4e5, 98 },
    { 0x50e5a4e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9465a4eb, 98 },
    { 0x83a5a4ec, 98 },
    { 0x9995a4ed, 76 },
    { 0x8855a4ec, 95 },
    { 0x1b05a4ef, 61 },
    { 0x2835a4ee, 43 },
    { 0, 0 },
    { 0x4d95a4f2, 69 },
    { 0, 0 },
    { 0x5605a4f4, 95 },
    { 0xffc5a4f5, 95 },
    { 0xbec5a4f4, 55 },
    { 0, 0 },
    { 0xf0d5a4f8, 84 },
    { 0, 0 },
    { 0x94c5a4fa, 33 },
    { 0x89d5a4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d5a4ff, 88 },
    { 0x8ae5a500, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c5a503, 90 },
    { 0x55f5a503, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc5a507, 61 },
    { 0, 0 },
    { 0xe5f5a509, 99 },
    { 0x8375a50a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435a50d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c85a512, 76 },
    { 0xe855a512, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5a516, 55 },
    { 0x1b75a516, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf955a51b, 55 },
    { 0, 0 },
    { 0x9705a51d, 33 },
    { 0xfda5a51e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb515a524, 98 },
    { 0xb425a525, 77 },
    { 0, 0 },
    { 0x9625a527, 56 },
    { 0x3765a527, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e55a52d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8905a530, 90 },
    { 0xc625a531, 36 },
    { 0x8255a530, 93 },
    { 0, 0 },
    { 0x395a534, 90 },
    { 0, 0 },
    { 0x8605a536, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925a53b, 91 },
    { 0x30d5a53c, 94 },
    { 0xb485a53d, 90 },
    { 0x805a53e, 61 },
    { 0x1ff5a53f, 34 },
    { 0x3405a53e, 93 },
    { 0x9535a53e, 52 },
    { 0x7715a542, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8145a545, 100 },
    { 0xdbf5a545, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f5a54f, 52 },
    { 0x4995a54f, 90 },
    { 0, 0 },
    { 0x1e75a552, 52 },
    { 0x1375a552, 90 },
    { 0xfcc5a553, 93 },
    { 0x1075a554, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a55a, 91 },
    { 0x5355a55b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f5a55e, 33 },
    { 0x39d5a55e, 23 },
    { 0x8845a55e, 23 },
    { 0, 0 },
    { 0xa55a562, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c65a567, 95 },
    { 0xfff5a568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8185a56b, 52 },
    { 0x6b45a56b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff5a571, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215a574, 65 },
    { 0, 0 },
    { 0x9f15a576, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec55a57f, 61 },
    { 0x2215a580, 98 },
    { 0x4485a581, 34 },
    { 0xb6e5a582, 66 },
    { 0x4595a583, 33 },
    { 0, 0 },
    { 0xe45a585, 90 },
    { 0x8e55a585, 55 },
    { 0xb645a587, 55 },
    { 0x99c5a587, 78 },
    { 0, 0 },
    { 0xb215a58a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d75a594, 23 },
    { 0x6825a595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd5a59a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c5a59d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba5a5aa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db5a5af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6575a5b2, 98 },
    { 0x7565a5b3, 76 },
    { 0x4e85a5b3, 23 },
    { 0xa915a5b3, 99 },
    { 0, 0 },
    { 0x9a05a5b7, 33 },
    { 0, 0 },
    { 0x3ec5a5b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a5a5bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad75a5c2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5a5c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d5a5d1, 61 },
    { 0x8555a5d2, 93 },
    { 0xed05a5d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a5a5d8, 87 },
    { 0x8f75a5d8, 23 },
    { 0x7125a5da, 33 },
    { 0, 0 },
    { 0x8635a5dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf05a5e1, 55 },
    { 0xd485a5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa545a5e5, 76 },
    { 0x8c45a5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa065a5ea, 90 },
    { 0x8055a5ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3935a5ee, 95 },
    { 0xc815a5ef, 33 },
    { 0x19e5a5ef, 90 },
    { 0x2315a5f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e65a5f6, 95 },
    { 0x5b85a5f7, 46 },
    { 0, 0 },
    { 0xd3e5a5f9, 53 },
    { 0xe35a5fa, 75 },
    { 0x9795a5f9, 23 },
    { 0, 0 },
    { 0x5b5a5fd, 61 },
    { 0xa365a5fd, 43 },
    { 0x17f5a5ff, 90 },
    { 0x4015a5fe, 75 },
    { 0, 0 },
    { 0x98d5a602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9235a60a, 55 },
    { 0xdcd5a60b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7385a60f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b5a614, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d5a617, 98 },
    { 0x9d25a618, 61 },
    { 0, 0 },
    { 0xb115a61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd05a61d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a622, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c75a625, 33 },
    { 0x57c5a626, 85 },
    { 0x10d5a627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da5a62b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b65a62f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b15a637, 23 },
    { 0x7d65a638, 98 },
    { 0xfa05a638, 55 },
    { 0, 0 },
    { 0xc7c5a63b, 83 },
    { 0x6ab5a63b, 55 },
    { 0, 0 },
    { 0xaff5a63e, 95 },
    { 0, 0 },
    { 0x7c95a640, 61 },
    { 0, 0 },
    { 0x89e5a642, 51 },
    { 0xddd5a643, 43 },
    { 0x8875a642, 68 },
    { 0x69b5a645, 85 },
    { 0xc165a642, 61 },
    { 0, 0 },
    { 0x7335a648, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3205a64c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6645a64f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c5a653, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15a659, 99 },
    { 0x84d5a65a, 95 },
    { 0x4105a65b, 95 },
    { 0xb595a65c, 23 },
    { 0x26d5a65b, 32 },
    { 0x7f65a65e, 88 },
    { 0x60a5a65e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b15a665, 43 },
    { 0, 0 },
    { 0x2c45a667, 61 },
    { 0x6455a668, 67 },
    { 0x5e55a669, 43 },
    { 0, 0 },
    { 0xfc05a66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e5a674, 87 },
    { 0x8535a675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee5a678, 98 },
    { 0, 0 },
    { 0xd515a67a, 84 },
    { 0xb4b5a67b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8915a682, 95 },
    { 0x8b95a683, 95 },
    { 0xea95a684, 61 },
    { 0x74b5a685, 61 },
    { 0x56d5a686, 98 },
    { 0x5475a682, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db5a68c, 75 },
    { 0, 0 },
    { 0x99f5a68e, 23 },
    { 0, 0 },
    { 0x1825a690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9965a699, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f85a69c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1965a69f, 55 },
    { 0x5ab5a69f, 61 },
    { 0x7cf5a6a1, 52 },
    { 0xe905a6a0, 34 },
    { 0xe295a6a1, 23 },
    { 0xfa25a6a4, 61 },
    { 0x7715a6a4, 98 },
    { 0, 0 },
    { 0xe8a5a6a7, 61 },
    { 0, 0 },
    { 0x9095a6a9, 61 },
    { 0, 0 },
    { 0xcfe5a6ab, 95 },
    { 0x865a6ac, 78 },
    { 0x7ac5a6ad, 55 },
    { 0xfe35a6ae, 66 },
    { 0xad75a6ac, 33 },
    { 0x77b5a6ac, 87 },
    { 0xc085a6ad, 90 },
    { 0, 0 },
    { 0xd325a6b3, 33 },
    { 0xd855a6b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa35a6b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb35a6bf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e5a6c7, 86 },
    { 0xd585a6c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5a6cd, 90 },
    { 0xd565a6ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8425a6d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd535a6d7, 90 },
    { 0, 0 },
    { 0x79d5a6d9, 90 },
    { 0xe405a6d9, 93 },
    { 0x58a5a6d9, 88 },
    { 0, 0 },
    { 0xc425a6dd, 95 },
    { 0xa845a6de, 90 },
    { 0x43b5a6de, 68 },
    { 0x29d5a6de, 90 },
    { 0xd8e5a6df, 88 },
    { 0x6b75a6e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9995a6e8, 44 },
    { 0xd4c5a6e9, 53 },
    { 0x7e15a6ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb45a6f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab15a6f3, 95 },
    { 0x5d05a6f4, 90 },
    { 0, 0 },
    { 0x7075a6f6, 98 },
    { 0, 0 },
    { 0xb645a6f8, 61 },
    { 0xf625a6f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc05a6fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf445a700, 67 },
    { 0x1155a701, 95 },
    { 0x6e65a702, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a35a707, 52 },
    { 0, 0 },
    { 0x2095a709, 23 },
    { 0, 0 },
    { 0x6315a70b, 23 },
    { 0, 0 },
    { 0xba35a70d, 90 },
    { 0x8ab5a70e, 98 },
    { 0, 0 },
    { 0xe575a710, 34 },
    { 0, 0 },
    { 0x8195a712, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc425a716, 34 },
    { 0x3275a717, 61 },
    { 0xf905a718, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a5a71e, 90 },
    { 0x4985a71e, 84 },
    { 0x1185a720, 76 },
    { 0xd035a71e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7195a724, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305a727, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa415a72f, 96 },
    { 0x7705a72f, 55 },
    { 0, 0 },
    { 0x9265a732, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4415a735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e5a73c, 34 },
    { 0, 0 },
    { 0x9e65a73e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d55a741, 61 },
    { 0, 0 },
    { 0xccc5a743, 59 },
    { 0, 0 },
    { 0xc635a745, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7925a748, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5a74e, 23 },
    { 0xd395a74f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6935a753, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a95a756, 95 },
    { 0xbea5a757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5a75a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e5a75f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5a763, 99 },
    { 0, 0 },
    { 0x1845a765, 99 },
    { 0xa935a765, 23 },
    { 0x9095a765, 97 },
    { 0, 0 },
    { 0x60d5a769, 95 },
    { 0xd145a769, 90 },
    { 0, 0 },
    { 0x7105a76c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d5a770, 23 },
    { 0xbcf5a771, 95 },
    { 0x59f5a770, 23 },
    { 0xb7c5a770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3115a777, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f85a77d, 34 },
    { 0x9195a77e, 55 },
    { 0, 0 },
    { 0x8135a780, 98 },
    { 0x73c5a781, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e5a788, 76 },
    { 0x7605a788, 81 },
    { 0xfa75a78a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a65a791, 90 },
    { 0, 0 },
    { 0xc305a793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae5a798, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee25a79d, 33 },
    { 0xa9a5a79d, 47 },
    { 0x17a5a79f, 36 },
    { 0x5a95a79e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb775a7a8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f75a7ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba5a7af, 95 },
    { 0xdb35a7b0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd615a7b7, 61 },
    { 0x1fa5a7b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e5a7bf, 66 },
    { 0x39e5a7bf, 34 },
    { 0xd935a7c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6415a7c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb75a7c9, 88 },
    { 0, 0 },
    { 0xa3f5a7cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8085a7d0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35a7d3, 76 },
    { 0, 0 },
    { 0x3aa5a7d5, 23 },
    { 0xeb35a7d6, 37 },
    { 0x5fa5a7d6, 33 },
    { 0xc285a7d5, 55 },
    { 0xe5c5a7d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a5a7df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb5a7e2, 61 },
    { 0, 0 },
    { 0xb4c5a7e4, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85a7e9, 81 },
    { 0x4265a7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5a7ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f25a7f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf095a7f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b5a7fc, 90 },
    { 0xf425a7fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1685a806, 90 },
    { 0, 0 },
    { 0x5ab5a808, 34 },
    { 0xbc15a808, 85 },
    { 0xc895a809, 95 },
    { 0xc705a80b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a75a816, 61 },
    { 0xeb15a816, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb5a81a, 85 },
    { 0x1e05a81b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5a81e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba65a824, 76 },
    { 0x1835a824, 95 },
    { 0x5b75a825, 33 },
    { 0x1bc5a827, 80 },
    { 0, 0 },
    { 0x9045a829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da5a830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1975a834, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f5a83a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb685a83d, 95 },
    { 0x6de5a83e, 93 },
    { 0x69e5a83f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca5a842, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb665a847, 90 },
    { 0, 0 },
    { 0x6795a849, 98 },
    { 0xe655a84a, 23 },
    { 0x6985a84b, 93 },
    { 0xf695a849, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc5a850, 23 },
    { 0xc1f5a851, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4325a854, 34 },
    { 0, 0 },
    { 0x6445a856, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a85a85c, 43 },
    { 0, 0 },
    { 0xc8c5a85e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5a861, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7205a867, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb45a86c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb715a871, 18 },
    { 0x7225a872, 95 },
    { 0x48e5a872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6415a876, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a5a87c, 61 },
    { 0, 0 },
    { 0x87d5a87e, 90 },
    { 0x6025a87f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b5a884, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55a889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5a88c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5715a88f, 34 },
    { 0xf0a5a890, 84 },
    { 0, 0 },
    { 0xfc75a892, 90 },
    { 0xc375a893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e5a897, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa5a89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c5a8a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5a8a3, 99 },
    { 0xfb15a8a4, 43 },
    { 0x4be5a8a5, 67 },
    { 0x6d75a8a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f5a8b3, 100 },
    { 0xa995a8b4, 61 },
    { 0xc915a8b3, 24 },
    { 0x90e5a8b6, 61 },
    { 0x1315a8b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e75a8bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6375a8bf, 98 },
    { 0xc205a8bf, 93 },
    { 0x2a15a8bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd365a8c4, 98 },
    { 0x3125a8c4, 69 },
    { 0x2cb5a8c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef5a8ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d95a8d1, 33 },
    { 0, 0 },
    { 0x5555a8d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6925a8d9, 95 },
    { 0, 0 },
    { 0x13e5a8db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c5a8de, 52 },
    { 0x2f85a8df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd5a8e2, 55 },
    { 0x2725a8e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e5a8e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe805a8ec, 95 },
    { 0, 0 },
    { 0xe655a8ee, 66 },
    { 0x65a8ef, 87 },
    { 0xa025a8ef, 90 },
    { 0, 0 },
    { 0x3075a8f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a5a8f5, 61 },
    { 0x8a45a8f6, 33 },
    { 0, 0 },
    { 0xce5a8f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2695a8fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2485a901, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa355a90a, 61 },
    { 0, 0 },
    { 0xa165a90c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af5a912, 23 },
    { 0, 0 },
    { 0x8595a914, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb25a919, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe265a91d, 95 },
    { 0xfa25a91d, 36 },
    { 0, 0 },
    { 0xef55a920, 93 },
    { 0xe445a921, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf95a925, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe315a928, 61 },
    { 0x64a5a929, 98 },
    { 0xa665a92a, 91 },
    { 0x6e05a92b, 81 },
    { 0x285a92c, 95 },
    { 0x5af5a92d, 98 },
    { 0, 0 },
    { 0x3575a92f, 61 },
    { 0xf385a930, 93 },
    { 0x5905a931, 98 },
    { 0x1bf5a92f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a5a938, 23 },
    { 0xfd15a938, 61 },
    { 0xb665a938, 69 },
    { 0xb3c5a938, 61 },
    { 0xd055a93b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d65a941, 33 },
    { 0x67d5a942, 98 },
    { 0, 0 },
    { 0x43d5a944, 33 },
    { 0x59d5a945, 100 },
    { 0xbef5a945, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5a949, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2755a94c, 66 },
    { 0, 0 },
    { 0x86f5a94e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb5a951, 33 },
    { 0x8ef5a952, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275a95f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c45a965, 23 },
    { 0x54d5a966, 33 },
    { 0x38a5a966, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1835a96b, 61 },
    { 0, 0 },
    { 0x2635a96d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de5a970, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4505a973, 34 },
    { 0xc7c5a974, 55 },
    { 0x7195a975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc125a97c, 100 },
    { 0, 0 },
    { 0x8635a97e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a5a986, 76 },
    { 0x2935a987, 52 },
    { 0x6475a987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f5a98b, 78 },
    { 0xeb75a98c, 61 },
    { 0xa655a98d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5a996, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c25a99d, 88 },
    { 0xcde5a99e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d5a9a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf415a9a7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a55a9b1, 90 },
    { 0xde75a9b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf835a9b5, 98 },
    { 0, 0 },
    { 0x8b65a9b7, 48 },
    { 0x375a9b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1975a9bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c15a9c0, 76 },
    { 0, 0 },
    { 0xe2c5a9c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc5a9c7, 61 },
    { 0, 0 },
    { 0x9955a9c9, 61 },
    { 0x7765a9c9, 90 },
    { 0xeaf5a9cb, 61 },
    { 0x8ab5a9cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf835a9cf, 33 },
    { 0xc865a9cf, 98 },
    { 0x4e15a9d1, 46 },
    { 0x32b5a9cf, 33 },
    { 0x1595a9d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9105a9d6, 45 },
    { 0x9285a9d7, 90 },
    { 0x44f5a9d8, 90 },
    { 0x89b5a9d9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7215a9dc, 100 },
    { 0x8c25a9dd, 90 },
    { 0, 0 },
    { 0x8ef5a9df, 34 },
    { 0xc5a5a9e0, 97 },
    { 0x5f25a9e0, 90 },
    { 0xc395a9e0, 17 },
    { 0, 0 },
    { 0x9a05a9e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a5a9e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc5a9f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6595a9fa, 75 },
    { 0xb1b5a9fb, 61 },
    { 0x8475a9fc, 93 },
    { 0x7eb5a9fd, 55 },
    { 0xa315a9fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb65aa01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435aa04, 61 },
    { 0xdd85aa05, 76 },
    { 0, 0 },
    { 0xec5aa07, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f5aa0a, 84 },
    { 0x47b5aa0b, 99 },
    { 0x3425aa0c, 33 },
    { 0x9f05aa0c, 76 },
    { 0x6d95aa0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5aa14, 95 },
    { 0xc345aa15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c5aa1a, 90 },
    { 0xe225aa1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f5aa20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb075aa25, 84 },
    { 0xeaf5aa25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865aa29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7145aa2f, 95 },
    { 0x51b5aa2f, 61 },
    { 0x3015aa31, 43 },
    { 0x3485aa32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d55aa37, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca25aa3b, 33 },
    { 0xb165aa3c, 52 },
    { 0xed65aa3c, 33 },
    { 0x6a5aa3c, 55 },
    { 0x7685aa3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1435aa44, 61 },
    { 0x9125aa45, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f35aa48, 55 },
    { 0x6765aa48, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa5aa50, 90 },
    { 0xc425aa51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc5aa55, 100 },
    { 0x39c5aa55, 34 },
    { 0, 0 },
    { 0x9325aa58, 34 },
    { 0xe785aa59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb5aa5c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715aa62, 95 },
    { 0, 0 },
    { 0x91c5aa64, 17 },
    { 0xdf65aa65, 52 },
    { 0x5225aa66, 33 },
    { 0x4355aa67, 43 },
    { 0xdda5aa67, 65 },
    { 0x1205aa69, 61 },
    { 0, 0 },
    { 0x1315aa6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1505aa71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065aa74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba55aa78, 61 },
    { 0xf575aa79, 53 },
    { 0xf5c5aa78, 91 },
    { 0x87a5aa7b, 46 },
    { 0xa685aa7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5aa85, 34 },
    { 0x7285aa85, 90 },
    { 0xd2e5aa87, 98 },
    { 0x9b45aa88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb95aa8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae35aa97, 43 },
    { 0xd595aa97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2205aa9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b45aa9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c95aaa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7685aaa8, 90 },
    { 0x5e45aaa9, 78 },
    { 0xd35aaa9, 87 },
    { 0x7b75aaab, 52 },
    { 0, 0 },
    { 0xac65aaad, 93 },
    { 0x5145aaad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695aabc, 76 },
    { 0, 0 },
    { 0x2535aabe, 77 },
    { 0x5a55aabe, 23 },
    { 0xf4b5aac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b55aac4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b5aac8, 55 },
    { 0, 0 },
    { 0xf925aaca, 55 },
    { 0x385aaca, 94 },
    { 0x47d5aacc, 90 },
    { 0xf115aacd, 43 },
    { 0, 0 },
    { 0x7c45aacf, 23 },
    { 0, 0 },
    { 0xf205aad1, 90 },
    { 0, 0 },
    { 0xbf5aad3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd185aad6, 90 },
    { 0, 0 },
    { 0x6c15aad8, 48 },
    { 0xf175aad9, 17 },
    { 0x5075aada, 98 },
    { 0xee05aadb, 66 },
    { 0x9de5aadc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95aadf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac5aae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf635aae8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd5aaeb, 98 },
    { 0xb185aaec, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a05aaef, 90 },
    { 0x3805aaef, 67 },
    { 0, 0 },
    { 0xbf95aaf2, 98 },
    { 0xdf5aaf3, 17 },
    { 0xe065aaf4, 90 },
    { 0, 0 },
    { 0xb595aaf6, 91 },
    { 0, 0 },
    { 0xee15aaf8, 95 },
    { 0, 0 },
    { 0x8815aafa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55aafd, 90 },
    { 0xc905aafe, 33 },
    { 0, 0 },
    { 0x5b65ab00, 98 },
    { 0xb085ab01, 90 },
    { 0x5965ab02, 78 },
    { 0xc485ab02, 93 },
    { 0x1255ab03, 43 },
    { 0xe9c5ab05, 66 },
    { 0xd4a5ab06, 93 },
    { 0x2bb5ab06, 33 },
    { 0xd275ab08, 61 },
    { 0x2905ab03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8795ab0c, 95 },
    { 0xbb15ab0d, 91 },
    { 0xf6f5ab0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b75ab11, 99 },
    { 0x5e75ab12, 23 },
    { 0xed25ab13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c65ab1b, 55 },
    { 0x1495ab1b, 55 },
    { 0xb025ab1d, 36 },
    { 0, 0 },
    { 0x5bf5ab1f, 91 },
    { 0, 0 },
    { 0xc1c5ab21, 34 },
    { 0x1255ab21, 93 },
    { 0x5495ab23, 95 },
    { 0x4d05ab23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa385ab27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca35ab2b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x485ab2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe895ab33, 24 },
    { 0x8755ab34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb115ab38, 95 },
    { 0x7775ab38, 98 },
    { 0xc325ab38, 55 },
    { 0xef85ab3a, 23 },
    { 0x4be5ab39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e5ab46, 90 },
    { 0x3115ab46, 93 },
    { 0xfb95ab47, 98 },
    { 0xb635ab47, 99 },
    { 0xdb75ab49, 61 },
    { 0, 0 },
    { 0xc575ab4c, 90 },
    { 0x7365ab4d, 33 },
    { 0x2935ab4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5ab54, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f5ab57, 87 },
    { 0x6465ab58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed5ab5c, 98 },
    { 0, 0 },
    { 0x5d5ab5e, 33 },
    { 0xc4c5ab5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f5ab6a, 100 },
    { 0xa735ab6b, 34 },
    { 0xccb5ab6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb25ab71, 43 },
    { 0, 0 },
    { 0xa0f5ab73, 98 },
    { 0, 0 },
    { 0x57a5ab75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc775ab7a, 55 },
    { 0, 0 },
    { 0x83e5ab7c, 34 },
    { 0x4235ab7d, 90 },
    { 0xd0d5ab7d, 85 },
    { 0x64c5ab7e, 43 },
    { 0xa45ab7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb225ab84, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6945ab87, 76 },
    { 0x1725ab88, 43 },
    { 0xc675ab87, 87 },
    { 0x7d85ab8a, 76 },
    { 0x3735ab8a, 61 },
    { 0xbd45ab8c, 33 },
    { 0x30c5ab8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae5ab93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445ab97, 90 },
    { 0x7a65ab98, 23 },
    { 0, 0 },
    { 0x9655ab9a, 33 },
    { 0x18f5ab9a, 75 },
    { 0x55d5ab9a, 98 },
    { 0x1445ab9d, 33 },
    { 0x6415ab9a, 33 },
    { 0xd245ab9f, 61 },
    { 0x4605ab9a, 90 },
    { 0xb175aba1, 52 },
    { 0, 0 },
    { 0x5475aba3, 100 },
    { 0x5545aba3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5aba8, 78 },
    { 0xf225aba8, 61 },
    { 0x5e85abaa, 23 },
    { 0x2ca5abab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7095abaf, 84 },
    { 0, 0 },
    { 0xfc15abb1, 90 },
    { 0x43c5abb2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d15abb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1305abbb, 95 },
    { 0x91e5abbc, 90 },
    { 0x7995abbd, 33 },
    { 0x62a5abbc, 82 },
    { 0, 0 },
    { 0x6225abc0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c15abc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05abce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb925abd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a75abd6, 23 },
    { 0xd5b5abd7, 46 },
    { 0x8325abd8, 95 },
    { 0x42e5abd7, 55 },
    { 0xd205abda, 90 },
    { 0x9345abdb, 46 },
    { 0x7645abdb, 43 },
    { 0x8bb5abda, 33 },
    { 0xa2b5abda, 67 },
    { 0x1815abdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd715abe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac5abf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf5abf8, 81 },
    { 0xa7e5abf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b5ac00, 88 },
    { 0xdba5ac01, 61 },
    { 0, 0 },
    { 0x3e05ac03, 87 },
    { 0, 0 },
    { 0xac25ac05, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b45ac0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf805ac11, 23 },
    { 0xdb5ac12, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ac15, 61 },
    { 0x575ac16, 90 },
    { 0xb235ac17, 33 },
    { 0x9345ac16, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe595ac20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff5ac25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea5ac28, 100 },
    { 0, 0 },
    { 0xf275ac2a, 61 },
    { 0, 0 },
    { 0xd9e5ac2c, 33 },
    { 0xb215ac2d, 43 },
    { 0xa1f5ac2e, 33 },
    { 0xbd05ac2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4415ac38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a5ac3c, 98 },
    { 0x38e5ac3d, 93 },
    { 0x8785ac3d, 95 },
    { 0x2835ac3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5ac42, 84 },
    { 0, 0 },
    { 0x64a5ac44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7055ac47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff25ac4b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c5ac50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc85ac53, 95 },
    { 0xa0d5ac54, 64 },
    { 0x4c85ac55, 61 },
    { 0x7845ac54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7215ac59, 33 },
    { 0xccf5ac5a, 66 },
    { 0x2c05ac59, 95 },
    { 0, 0 },
    { 0xbcc5ac5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265ac61, 61 },
    { 0x5995ac62, 98 },
    { 0x6965ac61, 98 },
    { 0x6695ac63, 84 },
    { 0xf605ac65, 90 },
    { 0xcbb5ac66, 95 },
    { 0xe655ac63, 90 },
    { 0xd1e5ac65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d5ac6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db5ac6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca5ac72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3375ac75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef15ac79, 95 },
    { 0x8655ac79, 95 },
    { 0x2f35ac7a, 100 },
    { 0, 0 },
    { 0xbc45ac7d, 98 },
    { 0xd845ac7e, 61 },
    { 0xd105ac7f, 100 },
    { 0xaf45ac7e, 33 },
    { 0xa9a5ac7e, 93 },
    { 0x71f5ac82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf5ac91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf75ac96, 90 },
    { 0x3535ac97, 55 },
    { 0, 0 },
    { 0xc265ac99, 90 },
    { 0x6b95ac9a, 91 },
    { 0x535ac99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f5ac9f, 95 },
    { 0x6635aca0, 67 },
    { 0x70f5aca1, 60 },
    { 0xd4c5aca2, 33 },
    { 0xb115aca2, 100 },
    { 0xc815aca4, 90 },
    { 0x3e05aca5, 95 },
    { 0, 0 },
    { 0x4f15aca7, 90 },
    { 0x5af5aca8, 90 },
    { 0, 0 },
    { 0x7415acaa, 23 },
    { 0x48c5acab, 99 },
    { 0x1a95acac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd515acb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf225acb6, 61 },
    { 0xb8b5acb7, 55 },
    { 0x78e5acb6, 76 },
    { 0x1a15acb8, 61 },
    { 0x89f5acba, 33 },
    { 0xc2a5acba, 46 },
    { 0x2695acb7, 36 },
    { 0, 0 },
    { 0xfe85acbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe5acc1, 81 },
    { 0xb75acc2, 55 },
    { 0x7555acc3, 78 },
    { 0, 0 },
    { 0xfde5acc5, 33 },
    { 0, 0 },
    { 0x2e45acc7, 34 },
    { 0x19c5acc8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd5accb, 23 },
    { 0xe3d5accc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615accf, 55 },
    { 0, 0 },
    { 0x7e5acd1, 95 },
    { 0x795acd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f75acd5, 76 },
    { 0xf855acd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5045acdc, 98 },
    { 0x16b5acdd, 87 },
    { 0x74f5acde, 23 },
    { 0xa75acdf, 46 },
    { 0x5365ace0, 90 },
    { 0x7d65acdd, 23 },
    { 0x9535acde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea5ace5, 88 },
    { 0xa0e5ace6, 78 },
    { 0x9865ace6, 93 },
    { 0x8275ace8, 17 },
    { 0x48a5ace9, 34 },
    { 0xef65ace5, 95 },
    { 0xcc25ace8, 61 },
    { 0xf415aceb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1405acef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a5acf4, 98 },
    { 0x6535acf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f5acf9, 95 },
    { 0xdc15acfa, 90 },
    { 0x61c5acfa, 95 },
    { 0x4785acfc, 55 },
    { 0x85f5acfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce35ad00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f35ad05, 61 },
    { 0x2355ad05, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e5ad0a, 33 },
    { 0xab15ad0a, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xb855ad0e, 61 },
    { 0xf545ad0f, 90 },
    { 0x9805ad10, 91 },
    { 0x49d5ad0f, 90 },
    { 0x8a45ad12, 91 },
    { 0x55a5ad12, 99 },
    { 0, 0 },
    { 0x3d55ad15, 33 },
    { 0x5a55ad15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5515ad1a, 55 },
    { 0x7aa5ad1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc515ad1f, 55 },
    { 0x22b5ad20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf815ad23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4575ad29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c5ad2e, 93 },
    { 0xd5e5ad2f, 37 },
    { 0, 0 },
    { 0x9d65ad31, 61 },
    { 0x8125ad32, 99 },
    { 0xca05ad31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8295ad37, 95 },
    { 0, 0 },
    { 0x1eb5ad39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25ad3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a45ad43, 33 },
    { 0, 0 },
    { 0xaee5ad45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda15ad4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8125ad4e, 99 },
    { 0, 0 },
    { 0x69d5ad50, 68 },
    { 0xe515ad50, 61 },
    { 0xaf95ad50, 52 },
    { 0xd15ad53, 33 },
    { 0x1365ad54, 95 },
    { 0xbe65ad54, 33 },
    { 0x6dd5ad56, 75 },
    { 0x5105ad57, 23 },
    { 0x7225ad56, 23 },
    { 0xdc5ad59, 95 },
    { 0x11b5ad59, 23 },
    { 0xc9a5ad5a, 93 },
    { 0, 0 },
    { 0x3835ad5d, 90 },
    { 0xcae5ad5e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2145ad63, 95 },
    { 0xa015ad64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8005ad67, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9285ad6d, 65 },
    { 0xad85ad6e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e95ad71, 84 },
    { 0x5d15ad71, 95 },
    { 0, 0 },
    { 0x2bf5ad74, 23 },
    { 0x2565ad75, 43 },
    { 0, 0 },
    { 0xacd5ad77, 81 },
    { 0, 0 },
    { 0x12a5ad79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7495ad7d, 95 },
    { 0xd325ad7e, 23 },
    { 0x97a5ad7f, 55 },
    { 0x67d5ad7e, 76 },
    { 0x1dc5ad7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ad8c, 90 },
    { 0x45e5ad8d, 90 },
    { 0x76e5ad8d, 33 },
    { 0, 0 },
    { 0xb315ad90, 34 },
    { 0xa5e5ad91, 66 },
    { 0x4335ad92, 76 },
    { 0, 0 },
    { 0x7255ad94, 84 },
    { 0xce05ad95, 69 },
    { 0x21b5ad96, 33 },
    { 0x4645ad95, 90 },
    { 0xa035ad97, 33 },
    { 0x6ad5ad99, 98 },
    { 0xdff5ad96, 61 },
    { 0x88e5ad9b, 23 },
    { 0x48b5ad95, 95 },
    { 0, 0 },
    { 0xf45ad9e, 23 },
    { 0x3ec5ad9f, 95 },
    { 0, 0 },
    { 0xe5e5ada1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5ada4, 93 },
    { 0xc915ada4, 46 },
    { 0x8a85ada6, 98 },
    { 0x8315ada7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc505adab, 23 },
    { 0, 0 },
    { 0xf255adad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5adb0, 90 },
    { 0x6a65adb0, 91 },
    { 0, 0 },
    { 0xe4b5adb3, 95 },
    { 0x40e5adb4, 98 },
    { 0x6125adb5, 61 },
    { 0xa595adb6, 33 },
    { 0x7f95adb7, 61 },
    { 0x6125adb8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd5adc2, 90 },
    { 0x6d35adc3, 66 },
    { 0x2795adc4, 90 },
    { 0x3ff5adc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb995adc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2615adcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2495add7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa835adde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5405ade1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865ade5, 93 },
    { 0xc9f5ade6, 90 },
    { 0xe315ade7, 65 },
    { 0x9f65ade8, 34 },
    { 0x3885ade9, 90 },
    { 0x8145adea, 33 },
    { 0xb705adeb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15adef, 98 },
    { 0x8bd5adf0, 99 },
    { 0xbb25adf1, 67 },
    { 0x8cf5adf2, 65 },
    { 0xd3a5adf1, 98 },
    { 0x6ec5adf4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e5adf7, 23 },
    { 0, 0 },
    { 0xd105adf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6905adfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3415ae08, 90 },
    { 0xa235ae09, 33 },
    { 0x1ec5ae0a, 99 },
    { 0xc3e5ae0a, 99 },
    { 0xcc15ae0a, 98 },
    { 0x9b55ae0d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb5ae13, 95 },
    { 0xae75ae13, 87 },
    { 0xa175ae13, 33 },
    { 0xee35ae13, 93 },
    { 0x4e05ae14, 58 },
    { 0x4925ae18, 51 },
    { 0x765ae18, 98 },
    { 0x12b5ae19, 90 },
    { 0, 0 },
    { 0x2ed5ae1c, 76 },
    { 0xaea5ae1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e45ae20, 33 },
    { 0xe625ae20, 98 },
    { 0xfe15ae22, 90 },
    { 0, 0 },
    { 0x9c65ae24, 23 },
    { 0xaf45ae25, 61 },
    { 0xff85ae26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e5ae35, 33 },
    { 0x1685ae36, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed5ae3b, 93 },
    { 0x85ae3c, 75 },
    { 0x3875ae3c, 33 },
    { 0x6e85ae3d, 85 },
    { 0xce15ae3b, 100 },
    { 0xd565ae40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a5ae43, 55 },
    { 0, 0 },
    { 0x44c5ae45, 90 },
    { 0, 0 },
    { 0x1565ae47, 55 },
    { 0x8e35ae47, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8935ae4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7035ae51, 77 },
    { 0, 0 },
    { 0xa0e5ae53, 34 },
    { 0xca15ae54, 90 },
    { 0, 0 },
    { 0x7195ae56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa475ae59, 61 },
    { 0x9c85ae5a, 33 },
    { 0, 0 },
    { 0x67e5ae5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f5ae5f, 90 },
    { 0x2205ae5f, 90 },
    { 0x92e5ae61, 93 },
    { 0x27c5ae5f, 23 },
    { 0x5785ae62, 33 },
    { 0x2915ae62, 98 },
    { 0x31e5ae65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a5ae69, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d5ae6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b5ae72, 99 },
    { 0, 0 },
    { 0x2845ae74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f5ae77, 87 },
    { 0x7fa5ae77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5165ae7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5ae81, 77 },
    { 0x2975ae81, 33 },
    { 0, 0 },
    { 0xb875ae84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3235ae88, 93 },
    { 0, 0 },
    { 0xd955ae8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635ae90, 90 },
    { 0x855ae91, 95 },
    { 0xc4d5ae90, 88 },
    { 0, 0 },
    { 0x5305ae94, 40 },
    { 0xe8e5ae95, 23 },
    { 0x3c95ae96, 90 },
    { 0xb945ae97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe605ae9c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb865aea2, 90 },
    { 0, 0 },
    { 0xf6a5aea4, 34 },
    { 0xb875aea5, 33 },
    { 0x89e5aea4, 81 },
    { 0xc9f5aea6, 46 },
    { 0x3015aea4, 76 },
    { 0x4de5aea9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4255aeb4, 23 },
    { 0xf165aeb5, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb5aeb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3655aebb, 95 },
    { 0xb795aebc, 100 },
    { 0xbb25aebd, 90 },
    { 0xb725aebd, 85 },
    { 0x1945aebb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd315aec3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d15aec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c15aece, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b5aed2, 61 },
    { 0, 0 },
    { 0x33b5aed4, 43 },
    { 0, 0 },
    { 0x3255aed6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195aedb, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25aede, 90 },
    { 0x21e5aede, 61 },
    { 0xc8f5aedf, 90 },
    { 0xa135aee1, 90 },
    { 0xb705aee2, 23 },
    { 0x7655aee3, 90 },
    { 0xaf35aee4, 99 },
    { 0, 0 },
    { 0x2255aee6, 90 },
    { 0x9965aee7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c35aeea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795aef1, 95 },
    { 0x6795aef2, 17 },
    { 0x2e65aef3, 95 },
    { 0x7f65aef4, 17 },
    { 0, 0 },
    { 0x7bb5aef6, 95 },
    { 0, 0 },
    { 0x765aef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb5af00, 67 },
    { 0xff65af01, 33 },
    { 0, 0 },
    { 0xc215af03, 61 },
    { 0xb835af04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8925af07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe65af0c, 55 },
    { 0xa145af0d, 61 },
    { 0xae25af0e, 95 },
    { 0xe135af0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5af12, 66 },
    { 0x42d5af13, 61 },
    { 0, 0 },
    { 0xc0e5af15, 95 },
    { 0x3355af16, 55 },
    { 0xb4c5af16, 88 },
    { 0x815af18, 98 },
    { 0x1365af18, 34 },
    { 0, 0 },
    { 0xe905af1b, 90 },
    { 0x63f5af1c, 90 },
    { 0x5bc5af1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4635af20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715af25, 93 },
    { 0, 0 },
    { 0xd445af27, 55 },
    { 0xdf35af27, 93 },
    { 0, 0 },
    { 0x3f95af2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c5af2d, 100 },
    { 0x39d5af2e, 81 },
    { 0xf245af2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe995af33, 17 },
    { 0xf355af34, 93 },
    { 0x3ff5af35, 61 },
    { 0x6375af36, 90 },
    { 0x8ec5af35, 93 },
    { 0x9bf5af38, 95 },
    { 0x3e25af34, 95 },
    { 0xc2a5af34, 90 },
    { 0x3355af37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f35af3e, 55 },
    { 0, 0 },
    { 0xd325af40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5af43, 67 },
    { 0x7115af44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db5af52, 33 },
    { 0x6ac5af52, 76 },
    { 0, 0 },
    { 0x4785af55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2385af5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a5af5e, 43 },
    { 0x3cc5af5f, 98 },
    { 0xa1c5af60, 90 },
    { 0, 0 },
    { 0x5ea5af62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac5af6c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa895af6f, 43 },
    { 0x5115af70, 61 },
    { 0, 0 },
    { 0x9cb5af72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe055af76, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae15af7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c5af7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec5af85, 98 },
    { 0, 0 },
    { 0xe25af87, 88 },
    { 0x2fd5af87, 61 },
    { 0x9ca5af87, 33 },
    { 0x2dd5af8a, 61 },
    { 0xe0a5af8b, 88 },
    { 0xee45af8c, 85 },
    { 0x1115af8b, 66 },
    { 0x1c75af87, 90 },
    { 0x7eb5af8f, 68 },
    { 0x9ab5af8e, 98 },
    { 0x1c35af91, 33 },
    { 0xd975af89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe315af95, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xba75af98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c65af9e, 95 },
    { 0, 0 },
    { 0x8b15afa0, 84 },
    { 0xd9e5afa1, 61 },
    { 0xda95afa2, 98 },
    { 0xe1f5afa3, 23 },
    { 0xdbe5afa3, 36 },
    { 0, 0 },
    { 0x875afa6, 81 },
    { 0xb165afa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a5afab, 90 },
    { 0x20a5afac, 33 },
    { 0xd3f5afac, 55 },
    { 0x2695afae, 46 },
    { 0xbd65afaf, 3 },
    { 0xb485afb0, 90 },
    { 0, 0 },
    { 0x43f5afb2, 95 },
    { 0, 0 },
    { 0x8e25afb4, 29 },
    { 0, 0 },
    { 0xc985afb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee75afb9, 98 },
    { 0xe0a5afba, 33 },
    { 0, 0 },
    { 0x3975afbc, 76 },
    { 0x125afbc, 55 },
    { 0x4c55afbe, 95 },
    { 0x94c5afbf, 95 },
    { 0, 0 },
    { 0x61a5afc1, 23 },
    { 0x7325afc2, 23 },
    { 0xdb55afc2, 90 },
    { 0xe9a5afc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c95afca, 95 },
    { 0x35a5afcb, 78 },
    { 0xde85afcb, 76 },
    { 0x1245afcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7285afd6, 33 },
    { 0x64a5afd7, 81 },
    { 0, 0 },
    { 0x2885afd9, 33 },
    { 0, 0 },
    { 0x2f5afdb, 33 },
    { 0, 0 },
    { 0xf895afdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb945afe1, 24 },
    { 0, 0 },
    { 0xe295afe3, 98 },
    { 0x2d55afe4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd105afe7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5585afee, 34 },
    { 0, 0 },
    { 0xecd5aff0, 95 },
    { 0xd285aff1, 33 },
    { 0, 0 },
    { 0x825aff3, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005aff6, 78 },
    { 0xed15aff7, 23 },
    { 0x82e5aff7, 100 },
    { 0x27d5aff7, 33 },
    { 0x9bb5affa, 94 },
    { 0x4d5affa, 46 },
    { 0xdfc5affb, 33 },
    { 0xad35affb, 90 },
    { 0x4315affe, 61 },
    { 0x8225afff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca5b006, 85 },
    { 0x52c5b007, 98 },
    { 0x2525b007, 43 },
    { 0xd985b009, 55 },
    { 0x8b85b006, 87 },
    { 0x6595b00b, 90 },
    { 0xef95b00c, 77 },
    { 0x3395b00d, 55 },
    { 0xe845b00d, 90 },
    { 0x3005b00e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db5b015, 33 },
    { 0x8e5b015, 95 },
    { 0xc415b017, 23 },
    { 0x25d5b018, 95 },
    { 0x2cc5b019, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c35b01c, 50 },
    { 0x2825b01d, 55 },
    { 0x5e05b01e, 33 },
    { 0xbd15b01f, 34 },
    { 0, 0 },
    { 0x995b021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5b024, 93 },
    { 0xaac5b024, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb945b028, 90 },
    { 0, 0 },
    { 0x28f5b02a, 23 },
    { 0, 0 },
    { 0x5335b02c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e5b033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a5b038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed65b03b, 98 },
    { 0xc9d5b03c, 20 },
    { 0x5925b03d, 95 },
    { 0, 0 },
    { 0x6825b03f, 61 },
    { 0x80e5b040, 76 },
    { 0xa375b041, 45 },
    { 0, 0 },
    { 0x75b5b043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d25b047, 91 },
    { 0, 0 },
    { 0xea75b049, 17 },
    { 0, 0 },
    { 0xc935b04b, 87 },
    { 0, 0 },
    { 0x7795b04d, 61 },
    { 0x9555b04e, 61 },
    { 0x1f85b04f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f55b055, 81 },
    { 0x8345b056, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f75b05b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff5b05f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9095b062, 46 },
    { 0, 0 },
    { 0xbfe5b064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf265b067, 91 },
    { 0x1c55b067, 95 },
    { 0xa975b068, 85 },
    { 0xf3f5b06a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd255b06d, 78 },
    { 0, 0 },
    { 0x9df5b06f, 66 },
    { 0xdcc5b06f, 55 },
    { 0x7195b070, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x585b074, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d5b07a, 95 },
    { 0, 0 },
    { 0xa655b07c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5b07f, 66 },
    { 0xd7b5b080, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a5b087, 85 },
    { 0xcd45b088, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305b08f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf015b092, 90 },
    { 0x90c5b093, 95 },
    { 0x7e85b094, 33 },
    { 0x9815b095, 95 },
    { 0x9ea5b094, 98 },
    { 0x635b095, 61 },
    { 0x8565b092, 61 },
    { 0xb3b5b099, 90 },
    { 0x3e05b099, 90 },
    { 0, 0 },
    { 0xeb05b09c, 61 },
    { 0xd115b09c, 23 },
    { 0xb2a5b09e, 85 },
    { 0x325b09f, 90 },
    { 0, 0 },
    { 0xab55b0a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3215b0a5, 46 },
    { 0x1da5b0a5, 34 },
    { 0xbd25b0a5, 99 },
    { 0x8975b0a8, 93 },
    { 0x67e5b0a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f65b0ac, 23 },
    { 0x1e25b0ad, 90 },
    { 0xf7b5b0ad, 88 },
    { 0x13c5b0ac, 23 },
    { 0, 0 },
    { 0x6a55b0b1, 34 },
    { 0xc085b0b1, 84 },
    { 0x705b0b3, 85 },
    { 0x355b0b3, 90 },
    { 0x4665b0b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed5b0bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa345b0c0, 85 },
    { 0x2d45b0c1, 87 },
    { 0, 0 },
    { 0x4135b0c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7265b0c6, 80 },
    { 0x3a45b0c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b5b0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa855b0d2, 98 },
    { 0x3f35b0d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335b0d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c5b0da, 95 },
    { 0x60a5b0db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c5b0de, 98 },
    { 0, 0 },
    { 0x9ec5b0e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd5b0ea, 85 },
    { 0xf45b0ea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2645b0ee, 93 },
    { 0x36d5b0ee, 34 },
    { 0x12d5b0ee, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85b0f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb5b0fb, 90 },
    { 0, 0 },
    { 0xda45b0fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b104, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc65b10d, 81 },
    { 0x1655b10e, 23 },
    { 0xc285b10e, 90 },
    { 0, 0 },
    { 0xd35b111, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb5b116, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4275b119, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4615b11c, 46 },
    { 0xfb35b11d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee5b120, 61 },
    { 0x3bc5b121, 98 },
    { 0xf6b5b122, 95 },
    { 0, 0 },
    { 0xf0d5b124, 90 },
    { 0xea05b125, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5b12c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c85b132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec5b13b, 61 },
    { 0, 0 },
    { 0xc975b13d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad5b143, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b5b148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd415b14b, 98 },
    { 0, 0 },
    { 0xdaa5b14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a5b152, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c5b15a, 66 },
    { 0x6a55b15a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2985b166, 33 },
    { 0x97f5b166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc235b16d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4335b171, 65 },
    { 0xd1b5b171, 99 },
    { 0x6b55b173, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad5b178, 76 },
    { 0x43f5b179, 90 },
    { 0x9f5b179, 23 },
    { 0x52d5b17b, 99 },
    { 0xd045b17c, 90 },
    { 0xb0d5b178, 75 },
    { 0x835b179, 48 },
    { 0, 0 },
    { 0x7685b180, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9525b187, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05b18d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c5b193, 43 },
    { 0xd315b194, 52 },
    { 0, 0 },
    { 0xbc25b196, 77 },
    { 0xc4c5b196, 33 },
    { 0x5ff5b198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7915b19d, 90 },
    { 0, 0 },
    { 0x35f5b19f, 67 },
    { 0xf2c5b1a0, 88 },
    { 0x4265b19f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe5b1a4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c5b1a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd5b1ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e55b1b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5b1ba, 95 },
    { 0, 0 },
    { 0x4af5b1bc, 95 },
    { 0x4665b1bd, 17 },
    { 0x8a05b1bc, 33 },
    { 0x5ce5b1bf, 95 },
    { 0xeb35b1c0, 95 },
    { 0xf965b1c1, 76 },
    { 0x8a25b1c1, 98 },
    { 0, 0 },
    { 0xd715b1c4, 55 },
    { 0, 0 },
    { 0x9915b1c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb905b1cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2815b1d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a5b1d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935b1db, 34 },
    { 0x57a5b1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5b1e1, 87 },
    { 0xbdd5b1e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8425b1eb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7485b1f0, 33 },
    { 0xbaa5b1f1, 93 },
    { 0xe2e5b1f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd545b1f5, 61 },
    { 0, 0 },
    { 0x3805b1f7, 33 },
    { 0xcc45b1f8, 87 },
    { 0xbb75b1f7, 84 },
    { 0, 0 },
    { 0x1555b1fb, 100 },
    { 0, 0 },
    { 0x2195b1fd, 23 },
    { 0xd735b1fe, 61 },
    { 0, 0 },
    { 0x7fd5b200, 90 },
    { 0xf4d5b201, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a5b205, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575b20e, 66 },
    { 0x6735b20f, 61 },
    { 0xa035b210, 34 },
    { 0x2c25b211, 95 },
    { 0, 0 },
    { 0x83e5b213, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d05b219, 33 },
    { 0xe4a5b21a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8625b224, 55 },
    { 0xee25b225, 61 },
    { 0x6b95b226, 87 },
    { 0xd015b226, 61 },
    { 0x8475b226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa95b22b, 56 },
    { 0x10e5b22b, 46 },
    { 0xe9a5b22d, 77 },
    { 0xe485b22b, 17 },
    { 0xcd25b22d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c35b232, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd5b239, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635b23c, 52 },
    { 0xc3d5b23d, 87 },
    { 0x9c75b23e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c65b245, 95 },
    { 0, 0 },
    { 0x99e5b247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5b24f, 33 },
    { 0x7285b24f, 90 },
    { 0xa495b250, 55 },
    { 0xb365b252, 55 },
    { 0xb715b250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba5b258, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8685b25b, 99 },
    { 0xfad5b25c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd55b266, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e5b275, 90 },
    { 0, 0 },
    { 0xeba5b277, 76 },
    { 0xaea5b278, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe5b27e, 90 },
    { 0x94b5b27e, 34 },
    { 0xead5b27f, 23 },
    { 0x6cb5b281, 61 },
    { 0x2185b282, 52 },
    { 0x9405b283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb5b289, 23 },
    { 0x5925b289, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6985b28d, 90 },
    { 0x6d5b28e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7965b294, 88 },
    { 0x3f25b294, 95 },
    { 0xc595b296, 84 },
    { 0x7b55b295, 61 },
    { 0xf6b5b298, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35b29d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f5b2a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455b2a5, 23 },
    { 0, 0 },
    { 0x1565b2a7, 98 },
    { 0, 0 },
    { 0xad05b2a9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x9295b2ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e5b2b1, 90 },
    { 0x7ab5b2b2, 61 },
    { 0, 0 },
    { 0xa005b2b4, 61 },
    { 0, 0 },
    { 0xf465b2b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8975b2ba, 94 },
    { 0xfb85b2bb, 85 },
    { 0xe915b2bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c5b2c2, 98 },
    { 0xe575b2c3, 33 },
    { 0x8635b2c4, 100 },
    { 0x7b45b2c2, 95 },
    { 0xeec5b2c2, 61 },
    { 0xb4e5b2c6, 76 },
    { 0x9e35b2c7, 76 },
    { 0xe8f5b2c9, 34 },
    { 0x4245b2c9, 69 },
    { 0x6695b2c9, 90 },
    { 0x4e5b2cc, 100 },
    { 0xdec5b2c5, 90 },
    { 0xe135b2ce, 95 },
    { 0xc65b2cc, 33 },
    { 0x7c65b2d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5b2d5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da5b2da, 95 },
    { 0x7d85b2db, 33 },
    { 0, 0 },
    { 0xc365b2dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2705b2e1, 33 },
    { 0x7945b2e1, 23 },
    { 0xb2f5b2e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a5b2e6, 91 },
    { 0xa295b2e6, 23 },
    { 0x4f5b2e6, 100 },
    { 0, 0 },
    { 0xcab5b2ea, 61 },
    { 0x3f45b2eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9945b2f0, 46 },
    { 0x5f25b2f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa795b2f4, 68 },
    { 0xa835b2f5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a5b2f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c5b301, 99 },
    { 0xf75b302, 33 },
    { 0, 0 },
    { 0xdf35b304, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa715b307, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225b30d, 55 },
    { 0x8c35b30e, 33 },
    { 0x8ff5b30e, 76 },
    { 0x4e15b30e, 61 },
    { 0x6af5b310, 33 },
    { 0xef5b30f, 90 },
    { 0x3725b311, 95 },
    { 0x9ae5b314, 76 },
    { 0x2465b315, 61 },
    { 0x625b316, 80 },
    { 0xaf05b310, 76 },
    { 0x2b25b318, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a5b31b, 46 },
    { 0x7275b31b, 89 },
    { 0, 0 },
    { 0x44a5b31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815b322, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045b325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a55b329, 90 },
    { 0x8cd5b329, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc5b32e, 55 },
    { 0x9f95b32f, 43 },
    { 0xecd5b330, 23 },
    { 0, 0 },
    { 0x9f5b332, 90 },
    { 0xa025b332, 61 },
    { 0x7db5b334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d5b338, 33 },
    { 0x9c05b339, 91 },
    { 0xa905b33a, 84 },
    { 0x3e55b33b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a5b341, 61 },
    { 0x5b05b342, 90 },
    { 0xbfe5b342, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf165b346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d5b34a, 68 },
    { 0xea5b34b, 100 },
    { 0x9385b34a, 98 },
    { 0x4645b34b, 34 },
    { 0xdd35b34e, 95 },
    { 0xc705b34a, 33 },
    { 0x50e5b350, 90 },
    { 0x43a5b351, 34 },
    { 0x20a5b351, 23 },
    { 0xb545b353, 61 },
    { 0x22b5b354, 66 },
    { 0x885b352, 66 },
    { 0x3c85b350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4235b35a, 61 },
    { 0xa9c5b35b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c65b361, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb615b364, 33 },
    { 0x7035b365, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5995b371, 90 },
    { 0x365b372, 33 },
    { 0, 0 },
    { 0xe3a5b374, 76 },
    { 0xb585b375, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5b37b, 23 },
    { 0x4175b37b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed5b380, 95 },
    { 0, 0 },
    { 0x7375b382, 76 },
    { 0, 0 },
    { 0x3e15b384, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995b388, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb85b394, 34 },
    { 0x8125b395, 66 },
    { 0, 0 },
    { 0x765b397, 78 },
    { 0xdb85b398, 95 },
    { 0x9fa5b398, 33 },
    { 0xad15b39a, 33 },
    { 0x2df5b398, 76 },
    { 0x5385b398, 90 },
    { 0xccd5b39d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x375b3a0, 90 },
    { 0, 0 },
    { 0x62a5b3a2, 33 },
    { 0xeae5b3a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b5b3a8, 98 },
    { 0x5d05b3a9, 23 },
    { 0xdd15b3aa, 93 },
    { 0x9915b3aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d5b3ae, 81 },
    { 0xad55b3af, 43 },
    { 0xa125b3af, 55 },
    { 0x7ef5b3af, 90 },
    { 0xbb35b3b2, 95 },
    { 0xa8e5b3b3, 81 },
    { 0x7ad5b3b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d5b3b8, 90 },
    { 0, 0 },
    { 0x7005b3ba, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5b3bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba5b3c0, 97 },
    { 0x62e5b3c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd5b3c4, 23 },
    { 0x7595b3c5, 100 },
    { 0, 0 },
    { 0xb765b3c7, 78 },
    { 0, 0 },
    { 0xd635b3c9, 61 },
    { 0, 0 },
    { 0xd535b3cb, 89 },
    { 0x2835b3cc, 23 },
    { 0xb3e5b3cd, 98 },
    { 0x6ae5b3ce, 34 },
    { 0xbb15b3ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a5b3da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6125b3de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7925b3e1, 17 },
    { 0x81d5b3e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5b3eb, 68 },
    { 0, 0 },
    { 0x1ec5b3ed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5b3f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c85b3f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f85b3fa, 85 },
    { 0x7d25b3fa, 43 },
    { 0x1eb5b3fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc135b400, 33 },
    { 0xa025b401, 61 },
    { 0x84c5b402, 46 },
    { 0, 0 },
    { 0x9bb5b404, 95 },
    { 0xfe15b405, 98 },
    { 0x5a05b406, 78 },
    { 0x1895b405, 52 },
    { 0x7d35b408, 90 },
    { 0xba35b408, 78 },
    { 0x9d25b408, 21 },
    { 0x66f5b405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3825b414, 74 },
    { 0x7d75b415, 55 },
    { 0x905b414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab5b419, 85 },
    { 0x94f5b41a, 36 },
    { 0, 0 },
    { 0x93b5b41c, 66 },
    { 0x6125b41c, 90 },
    { 0x6565b41e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325b422, 90 },
    { 0x6755b423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae55b426, 78 },
    { 0x7bc5b426, 43 },
    { 0x1bb5b428, 55 },
    { 0x2c95b429, 17 },
    { 0x505b428, 98 },
    { 0, 0 },
    { 0x5335b42c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275b42f, 90 },
    { 0x9535b42f, 33 },
    { 0x47b5b431, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba15b435, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e25b439, 33 },
    { 0x1465b43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce5b440, 76 },
    { 0x2285b440, 23 },
    { 0xdf55b442, 70 },
    { 0x8e95b443, 43 },
    { 0xe265b444, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5435b449, 90 },
    { 0xf535b449, 23 },
    { 0x8c05b44b, 68 },
    { 0xc9e5b44b, 67 },
    { 0, 0 },
    { 0xf4c5b44e, 93 },
    { 0, 0 },
    { 0xab95b450, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf565b453, 91 },
    { 0xf445b454, 90 },
    { 0x5125b455, 90 },
    { 0x5055b453, 90 },
    { 0xb665b454, 87 },
    { 0xb405b457, 90 },
    { 0xd635b459, 90 },
    { 0x9345b45a, 23 },
    { 0x83b5b45b, 77 },
    { 0xb155b456, 95 },
    { 0, 0 },
    { 0xe195b45e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb85b463, 61 },
    { 0x3ef5b463, 84 },
    { 0x5e65b465, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b55b46a, 76 },
    { 0, 0 },
    { 0xf865b46c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95b475, 88 },
    { 0x2b05b476, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f5b47b, 61 },
    { 0x9bd5b47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b35b47f, 61 },
    { 0x6d35b480, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xff35b483, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b5b487, 88 },
    { 0, 0 },
    { 0xbc75b489, 98 },
    { 0x3045b48a, 43 },
    { 0xc4b5b48a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa425b490, 98 },
    { 0x5c95b491, 78 },
    { 0x3745b491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575b496, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb955b49c, 43 },
    { 0x19b5b49d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe505b4a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd5b4a8, 98 },
    { 0xbd05b4a9, 55 },
    { 0, 0 },
    { 0x9075b4ab, 67 },
    { 0, 0 },
    { 0xe65b4ad, 48 },
    { 0x87b5b4ae, 33 },
    { 0x88c5b4af, 98 },
    { 0, 0 },
    { 0x2c25b4b1, 87 },
    { 0x7555b4b2, 34 },
    { 0, 0 },
    { 0x93c5b4b4, 43 },
    { 0, 0 },
    { 0x17f5b4b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4095b4b9, 90 },
    { 0x6be5b4b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab5b4be, 90 },
    { 0, 0 },
    { 0x7385b4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3175b4c9, 67 },
    { 0, 0 },
    { 0x7535b4cb, 23 },
    { 0xaa55b4cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc5b4cf, 43 },
    { 0xbab5b4d0, 33 },
    { 0xa525b4d1, 95 },
    { 0xeb15b4d2, 66 },
    { 0xc075b4d3, 90 },
    { 0xe9e5b4d4, 65 },
    { 0xc975b4d2, 84 },
    { 0x70e5b4d6, 33 },
    { 0xee15b4d7, 87 },
    { 0x3ec5b4d7, 34 },
    { 0, 0 },
    { 0xd845b4da, 61 },
    { 0x7555b4db, 34 },
    { 0x7fd5b4dc, 41 },
    { 0x5355b4db, 90 },
    { 0x4455b4dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5955b4e1, 91 },
    { 0x5a45b4e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e85b4e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x535b4eb, 23 },
    { 0, 0 },
    { 0x3415b4ed, 95 },
    { 0x1585b4ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f5b4f1, 95 },
    { 0x9b35b4f2, 23 },
    { 0x58b5b4f3, 90 },
    { 0xe5e5b4f3, 76 },
    { 0, 0 },
    { 0x5bb5b4f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f5b4fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca85b505, 33 },
    { 0, 0 },
    { 0x8585b507, 99 },
    { 0, 0 },
    { 0x7195b509, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25b50e, 33 },
    { 0x6905b50f, 84 },
    { 0xde45b50f, 93 },
    { 0x7465b50e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8625b51a, 33 },
    { 0, 0 },
    { 0xc8d5b51c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5b52a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2665b532, 33 },
    { 0xb95b533, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef55b536, 95 },
    { 0x3035b536, 91 },
    { 0xc65b538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb525b53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e5b546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275b54b, 61 },
    { 0x19f5b54b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c5b552, 61 },
    { 0, 0 },
    { 0x4b25b554, 98 },
    { 0x5ef5b555, 93 },
    { 0xd035b555, 61 },
    { 0xb235b557, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a5b55c, 23 },
    { 0x7d05b55d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa5b561, 95 },
    { 0, 0 },
    { 0xd3a5b563, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85b56c, 76 },
    { 0x56b5b56d, 90 },
    { 0x49d5b56e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc675b574, 67 },
    { 0, 0 },
    { 0xe175b576, 95 },
    { 0x3335b576, 99 },
    { 0x1c55b578, 98 },
    { 0x6bd5b579, 98 },
    { 0, 0 },
    { 0x64c5b57b, 52 },
    { 0xc395b57b, 61 },
    { 0x12f5b57d, 90 },
    { 0x8675b57e, 95 },
    { 0x92e5b57f, 17 },
    { 0x93d5b580, 90 },
    { 0xf655b57f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9775b588, 33 },
    { 0, 0 },
    { 0xc065b58a, 78 },
    { 0, 0 },
    { 0xab15b58c, 72 },
    { 0x9475b58d, 90 },
    { 0, 0 },
    { 0x8b55b58f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb105b592, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5b598, 91 },
    { 0, 0 },
    { 0x8025b59a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc605b59d, 90 },
    { 0x5dc5b59e, 98 },
    { 0x30a5b59e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e5b5a2, 87 },
    { 0, 0 },
    { 0xf495b5a4, 95 },
    { 0xbfd5b5a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc5b5a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1015b5ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd95b5b6, 61 },
    { 0x425b5b7, 90 },
    { 0x35b5b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d5b5bc, 23 },
    { 0xb475b5bd, 90 },
    { 0, 0 },
    { 0xbb05b5bf, 98 },
    { 0x4c25b5bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa635b5c6, 46 },
    { 0x5355b5c7, 90 },
    { 0xad85b5c7, 90 },
    { 0xd705b5c9, 93 },
    { 0x93f5b5ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f5b5ce, 52 },
    { 0xbb25b5cf, 98 },
    { 0x1485b5cf, 76 },
    { 0, 0 },
    { 0x9ff5b5d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f5b5d6, 23 },
    { 0xdf15b5d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8805b5dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a05b5e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e5b5e4, 55 },
    { 0x3eb5b5e4, 93 },
    { 0, 0 },
    { 0xbb05b5e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf075b5ec, 33 },
    { 0, 0 },
    { 0x32c5b5ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a5b5f4, 66 },
    { 0, 0 },
    { 0xd0d5b5f6, 81 },
    { 0x4c45b5f6, 95 },
    { 0x71a5b5f8, 93 },
    { 0x6d45b5f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb45b606, 90 },
    { 0x26c5b607, 23 },
    { 0xf6d5b608, 95 },
    { 0x61d5b606, 33 },
    { 0x5405b607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5b60d, 69 },
    { 0x4c5b60d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f75b612, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa185b617, 90 },
    { 0x8945b617, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa5b61f, 33 },
    { 0, 0 },
    { 0xab5b621, 88 },
    { 0, 0 },
    { 0x2665b623, 51 },
    { 0x4815b624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f05b62b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3535b633, 46 },
    { 0, 0 },
    { 0xadc5b635, 23 },
    { 0x39c5b636, 91 },
    { 0xfcd5b637, 48 },
    { 0, 0 },
    { 0x9025b639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f5b643, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e5b646, 33 },
    { 0, 0 },
    { 0xce55b648, 61 },
    { 0x4735b649, 42 },
    { 0x14e5b64a, 90 },
    { 0x3545b649, 80 },
    { 0x825b64c, 76 },
    { 0x9055b64c, 90 },
    { 0, 0 },
    { 0x49e5b64f, 33 },
    { 0, 0 },
    { 0x10c5b651, 98 },
    { 0xea55b652, 55 },
    { 0xfa85b653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf535b656, 75 },
    { 0x1e55b657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1525b65a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e95b65f, 61 },
    { 0, 0 },
    { 0x5fb5b661, 55 },
    { 0, 0 },
    { 0x67f5b663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c85b666, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf525b66a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d5b66e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e5b671, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455b677, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5b67b, 17 },
    { 0x2a95b67b, 61 },
    { 0xde95b67d, 36 },
    { 0, 0 },
    { 0x30c5b67f, 98 },
    { 0x4945b680, 98 },
    { 0xae05b67f, 90 },
    { 0x1715b682, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4775b68e, 95 },
    { 0, 0 },
    { 0x2165b690, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2545b693, 36 },
    { 0xd9c5b694, 95 },
    { 0x99f5b695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb5b69a, 34 },
    { 0xe7b5b69b, 84 },
    { 0xaeb5b69b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745b6a1, 61 },
    { 0x9d35b6a2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe315b6a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5b6b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe305b6b4, 46 },
    { 0, 0 },
    { 0x7da5b6b6, 67 },
    { 0, 0 },
    { 0xb6e5b6b8, 80 },
    { 0xfbe5b6b9, 43 },
    { 0x4365b6b9, 95 },
    { 0x78d5b6bb, 33 },
    { 0xcc95b6b9, 33 },
    { 0, 0 },
    { 0xb815b6be, 90 },
    { 0xc915b6bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d45b6c4, 95 },
    { 0, 0 },
    { 0xae5b6c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f5b6cb, 99 },
    { 0x3b65b6cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb525b6d1, 55 },
    { 0x2075b6d2, 61 },
    { 0xbd15b6d3, 33 },
    { 0xc415b6d4, 93 },
    { 0x8585b6d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e25b6dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7905b6e1, 91 },
    { 0xbbe5b6e1, 76 },
    { 0, 0 },
    { 0xbde5b6e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e75b6e8, 33 },
    { 0, 0 },
    { 0x9dc5b6ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255b6f1, 95 },
    { 0x8a65b6f2, 33 },
    { 0, 0 },
    { 0x2d75b6f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f5b6f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b5b6fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5115b703, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa5b706, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5b70b, 66 },
    { 0x7525b70b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x925b70f, 76 },
    { 0, 0 },
    { 0x4495b711, 34 },
    { 0, 0 },
    { 0x8cd5b713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a5b718, 95 },
    { 0xbcd5b719, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde75b71d, 90 },
    { 0x1055b71d, 33 },
    { 0x55b5b71f, 98 },
    { 0xf3f5b71f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd005b724, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1395b728, 90 },
    { 0, 0 },
    { 0xc45b72a, 90 },
    { 0, 0 },
    { 0x72a5b72c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe5b72f, 77 },
    { 0x9ff5b730, 88 },
    { 0x6bf5b730, 90 },
    { 0x6e55b72f, 61 },
    { 0x3385b732, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf95b739, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c05b743, 43 },
    { 0, 0 },
    { 0x28c5b745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f25b749, 90 },
    { 0x8e85b74a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5b74d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c5b752, 90 },
    { 0x3ed5b752, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8255b757, 33 },
    { 0, 0 },
    { 0xf905b759, 76 },
    { 0x72c5b75a, 33 },
    { 0x72d5b75b, 33 },
    { 0xe6f5b759, 34 },
    { 0, 0 },
    { 0x57e5b75e, 98 },
    { 0, 0 },
    { 0xc725b760, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3025b764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf825b767, 95 },
    { 0x155b768, 33 },
    { 0xa385b768, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd235b76d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd5b773, 98 },
    { 0x4965b773, 95 },
    { 0, 0 },
    { 0x4175b776, 66 },
    { 0x245b776, 90 },
    { 0x72f5b778, 61 },
    { 0x7585b779, 52 },
    { 0x7895b779, 23 },
    { 0x6cb5b777, 86 },
    { 0x9415b77c, 85 },
    { 0x4935b77d, 98 },
    { 0, 0 },
    { 0xca85b77f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3925b784, 23 },
    { 0x8675b785, 98 },
    { 0xf365b785, 93 },
    { 0xf975b784, 23 },
    { 0, 0 },
    { 0xe25b789, 67 },
    { 0xe665b78a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e5b78e, 95 },
    { 0, 0 },
    { 0xe005b790, 34 },
    { 0, 0 },
    { 0xf605b792, 90 },
    { 0x7ba5b793, 75 },
    { 0x44f5b794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3045b79b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb95b7a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b5b7a7, 46 },
    { 0, 0 },
    { 0x5f95b7a9, 95 },
    { 0x9695b7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd55b7ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d5b7b1, 90 },
    { 0xde75b7b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875b7b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f95b7b9, 33 },
    { 0xe485b7ba, 90 },
    { 0, 0 },
    { 0xcc45b7bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc95b7bf, 75 },
    { 0, 0 },
    { 0xcac5b7c1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x685b7c4, 100 },
    { 0xe355b7c5, 76 },
    { 0, 0 },
    { 0x2855b7c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc975b7ce, 34 },
    { 0xb8c5b7ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb5b7d3, 78 },
    { 0xf95b7d4, 55 },
    { 0xf7f5b7d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a5b7db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f75b7df, 61 },
    { 0xaab5b7e0, 95 },
    { 0xad45b7e1, 78 },
    { 0x3305b7e0, 33 },
    { 0xe2d5b7e3, 98 },
    { 0xddc5b7e0, 95 },
    { 0x2835b7e3, 98 },
    { 0xc6f5b7e0, 90 },
    { 0xb695b7e1, 67 },
    { 0xacd5b7e4, 38 },
    { 0x70f5b7e4, 90 },
    { 0x62e5b7ea, 90 },
    { 0xe3a5b7eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d75b7f0, 67 },
    { 0x4a55b7f0, 76 },
    { 0xf725b7f2, 46 },
    { 0x37c5b7f2, 55 },
    { 0xeaa5b7f2, 76 },
    { 0x2925b7f5, 61 },
    { 0xfbe5b7f5, 34 },
    { 0xf655b7f0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a5b7fa, 52 },
    { 0xae15b7fa, 60 },
    { 0, 0 },
    { 0x26b5b7fd, 99 },
    { 0, 0 },
    { 0x3315b7ff, 87 },
    { 0xda15b800, 90 },
    { 0x7d75b801, 100 },
    { 0x5875b800, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5b805, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5b808, 33 },
    { 0, 0 },
    { 0x2575b80a, 100 },
    { 0x2545b80a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b5b80e, 87 },
    { 0xf335b80e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6345b816, 91 },
    { 0, 0 },
    { 0x2d65b818, 88 },
    { 0, 0 },
    { 0xa125b81a, 17 },
    { 0x915b81a, 38 },
    { 0, 0 },
    { 0x3855b81d, 65 },
    { 0, 0 },
    { 0x2ca5b81f, 95 },
    { 0x6de5b820, 23 },
    { 0x635b821, 51 },
    { 0xa995b822, 95 },
    { 0xacb5b822, 90 },
    { 0x7e45b824, 61 },
    { 0xd195b825, 43 },
    { 0x8045b820, 52 },
    { 0xc085b827, 97 },
    { 0, 0 },
    { 0x6855b829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5b82d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f15b831, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955b834, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb245b838, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f5b83b, 90 },
    { 0xfd15b83b, 98 },
    { 0x7b65b83d, 61 },
    { 0xec85b83d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e45b845, 100 },
    { 0x3e45b846, 40 },
    { 0xf275b847, 90 },
    { 0xe935b847, 95 },
    { 0xaf15b849, 55 },
    { 0x1d5b847, 90 },
    { 0, 0 },
    { 0x4ae5b84c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf5b855, 76 },
    { 0x8315b856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff85b85b, 93 },
    { 0x5995b85c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a5b85f, 38 },
    { 0xa405b85f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe845b866, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff55b869, 55 },
    { 0xdc55b86a, 90 },
    { 0x2935b86a, 98 },
    { 0x9f05b86c, 98 },
    { 0x21e5b86c, 76 },
    { 0xfb65b86c, 84 },
    { 0x7a15b86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe5b874, 78 },
    { 0x3b25b874, 99 },
    { 0, 0 },
    { 0x6dd5b877, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf555b87b, 95 },
    { 0x9695b87c, 87 },
    { 0x775b87c, 98 },
    { 0, 0 },
    { 0x6495b87f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda45b883, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a5b887, 76 },
    { 0, 0 },
    { 0x5085b889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5b88c, 43 },
    { 0x4ee5b88d, 76 },
    { 0xfb35b88e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4715b891, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5905b895, 46 },
    { 0, 0 },
    { 0xa8e5b897, 8 },
    { 0, 0 },
    { 0xc565b899, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a5b89d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa85b8a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7205b8a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b8af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e15b8b3, 77 },
    { 0xf815b8b3, 76 },
    { 0xef65b8b5, 87 },
    { 0x42e5b8b5, 38 },
    { 0x38a5b8b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba5b8ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de5b8c4, 33 },
    { 0x5be5b8c5, 98 },
    { 0xd455b8c5, 98 },
    { 0, 0 },
    { 0x5925b8c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc5b8cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9695b8ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe955b8d1, 23 },
    { 0xde85b8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f5b8e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa95b8ed, 61 },
    { 0x6bf5b8ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf795b8fd, 90 },
    { 0xaea5b8fe, 33 },
    { 0x8765b8fe, 96 },
    { 0xd465b8ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde35b905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5b908, 36 },
    { 0, 0 },
    { 0x5455b90a, 90 },
    { 0x8355b90b, 95 },
    { 0xd735b90a, 33 },
    { 0xe0d5b90d, 98 },
    { 0xe675b90e, 64 },
    { 0xcd55b90d, 95 },
    { 0x31f5b910, 43 },
    { 0, 0 },
    { 0xe935b912, 90 },
    { 0, 0 },
    { 0xc6f5b914, 46 },
    { 0xdfd5b915, 33 },
    { 0xcc15b916, 99 },
    { 0, 0 },
    { 0x9b25b918, 33 },
    { 0x6d5b919, 61 },
    { 0x4285b91a, 98 },
    { 0xc85b91a, 95 },
    { 0x4a5b91c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4955b91f, 93 },
    { 0xd375b920, 90 },
    { 0x8cd5b921, 95 },
    { 0xbe55b922, 93 },
    { 0xd6c5b920, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc615b926, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5b92f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95b932, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf5b935, 61 },
    { 0x3845b936, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc395b939, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf495b93f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d5b947, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25b94a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d5b94d, 88 },
    { 0x4b05b94e, 90 },
    { 0x9f85b94f, 98 },
    { 0x16a5b950, 90 },
    { 0xab15b951, 85 },
    { 0, 0 },
    { 0x2f05b953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8195b956, 90 },
    { 0xdd25b957, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605b95d, 90 },
    { 0, 0 },
    { 0x5fd5b95f, 60 },
    { 0xd105b95f, 76 },
    { 0x8f65b95f, 91 },
    { 0x3d75b961, 90 },
    { 0, 0 },
    { 0x65f5b964, 33 },
    { 0x265b964, 90 },
    { 0x3d95b966, 95 },
    { 0x2ae5b966, 33 },
    { 0x905b968, 33 },
    { 0xf625b964, 76 },
    { 0xdeb5b96a, 61 },
    { 0xa3c5b96b, 100 },
    { 0x7f15b96b, 99 },
    { 0x6d55b969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be5b972, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7675b975, 100 },
    { 0xc315b976, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2135b984, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d5b989, 85 },
    { 0xd2b5b98a, 33 },
    { 0, 0 },
    { 0x5e75b98c, 34 },
    { 0, 0 },
    { 0xa15b98e, 23 },
    { 0, 0 },
    { 0x1e5b990, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb605b998, 23 },
    { 0, 0 },
    { 0x66d5b99a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d5b9a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e35b9a9, 55 },
    { 0, 0 },
    { 0x2925b9ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff5b9b1, 34 },
    { 0, 0 },
    { 0xb2a5b9b3, 66 },
    { 0x31b5b9b4, 23 },
    { 0xc525b9b3, 46 },
    { 0xeda5b9b5, 77 },
    { 0, 0 },
    { 0x33d5b9b8, 33 },
    { 0xe005b9b9, 78 },
    { 0x4f05b9b9, 77 },
    { 0xc105b9b8, 66 },
    { 0, 0 },
    { 0xc685b9bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d75b9c2, 34 },
    { 0x3fd5b9c3, 42 },
    { 0xe695b9c3, 63 },
    { 0, 0 },
    { 0xc115b9c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef5b9c9, 61 },
    { 0x22d5b9ca, 98 },
    { 0, 0 },
    { 0x84c5b9cc, 90 },
    { 0xc5b9cd, 52 },
    { 0xe65b9cd, 69 },
    { 0x1f75b9cf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a45b9d2, 90 },
    { 0, 0 },
    { 0xc235b9d4, 80 },
    { 0x99b5b9d4, 90 },
    { 0, 0 },
    { 0x46d5b9d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb15b9dd, 98 },
    { 0x8255b9dd, 43 },
    { 0x8bf5b9df, 90 },
    { 0x97c5b9e0, 98 },
    { 0, 0 },
    { 0x3145b9e2, 90 },
    { 0, 0 },
    { 0xa525b9e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9865b9e7, 61 },
    { 0xb375b9e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5b9eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a5b9f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7875b9fd, 67 },
    { 0xb8d5b9fe, 23 },
    { 0x6655b9ff, 99 },
    { 0xd095ba00, 61 },
    { 0, 0 },
    { 0x2ac5ba02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d75ba05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf5ba0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf885ba11, 90 },
    { 0, 0 },
    { 0xc145ba13, 65 },
    { 0xada5ba13, 90 },
    { 0x4195ba15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5ba1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2035ba23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec15ba28, 98 },
    { 0, 0 },
    { 0x80b5ba2a, 85 },
    { 0xdc85ba2a, 46 },
    { 0x2505ba2c, 70 },
    { 0, 0 },
    { 0xd1b5ba2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385ba35, 66 },
    { 0, 0 },
    { 0x5675ba37, 98 },
    { 0x8f55ba38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f65ba3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1655ba3f, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a5ba42, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865ba45, 98 },
    { 0, 0 },
    { 0xeba5ba47, 84 },
    { 0x5605ba48, 90 },
    { 0xc8e5ba47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e65ba4c, 33 },
    { 0x36c5ba4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5015ba50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa945ba59, 67 },
    { 0, 0 },
    { 0xd2a5ba5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f5ba63, 90 },
    { 0xd455ba64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a5ba6e, 34 },
    { 0, 0 },
    { 0x2785ba70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6365ba77, 33 },
    { 0xdfd5ba78, 95 },
    { 0xdd25ba79, 61 },
    { 0xabd5ba7a, 90 },
    { 0, 0 },
    { 0x3545ba7c, 93 },
    { 0xb745ba7c, 76 },
    { 0x2d75ba7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5935ba87, 34 },
    { 0xe225ba88, 99 },
    { 0xa895ba87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe105ba8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a55ba98, 58 },
    { 0, 0 },
    { 0x6a5ba9a, 33 },
    { 0x9a05ba9b, 76 },
    { 0, 0 },
    { 0x9935ba9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f5baa0, 52 },
    { 0xf825baa1, 33 },
    { 0x4125baa0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b35baa5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b75baab, 34 },
    { 0xadd5baac, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab15bab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c45bab4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa905bab8, 61 },
    { 0, 0 },
    { 0x1045baba, 95 },
    { 0x4cc5babb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe715bac1, 43 },
    { 0x7dd5bac2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc5bac5, 61 },
    { 0, 0 },
    { 0x2665bac7, 61 },
    { 0xf7b5bac8, 33 },
    { 0, 0 },
    { 0x26c5baca, 33 },
    { 0xb445baca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da5bacf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae5bad3, 76 },
    { 0x7f75bad3, 55 },
    { 0xb535bad3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e5bada, 76 },
    { 0x5885badb, 23 },
    { 0, 0 },
    { 0x9f05badd, 43 },
    { 0x2535bade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb65bae2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5bae6, 95 },
    { 0x8bc5bae7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35baec, 66 },
    { 0xe65baed, 65 },
    { 0, 0 },
    { 0xec95baef, 78 },
    { 0xb7c5baf0, 90 },
    { 0xb45baf1, 23 },
    { 0x52a5baf2, 34 },
    { 0xcfa5baf3, 64 },
    { 0x3795baf2, 17 },
    { 0xfd65baf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc35baf8, 98 },
    { 0, 0 },
    { 0xa8c5bafa, 33 },
    { 0xfe05bafb, 23 },
    { 0, 0 },
    { 0x1c15bafd, 90 },
    { 0, 0 },
    { 0xcbb5baff, 61 },
    { 0, 0 },
    { 0x3295bb01, 90 },
    { 0x3f35bb02, 76 },
    { 0x8fc5bb03, 78 },
    { 0x7cd5bb04, 23 },
    { 0xfdc5bb03, 87 },
    { 0x1c45bb04, 33 },
    { 0xc475bb07, 76 },
    { 0x4a75bb07, 100 },
    { 0x205bb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e65bb0e, 98 },
    { 0xaf75bb0f, 55 },
    { 0, 0 },
    { 0xb25bb11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad5bb17, 91 },
    { 0x8685bb18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2985bb1d, 91 },
    { 0xfb65bb1e, 33 },
    { 0, 0 },
    { 0x45b5bb20, 17 },
    { 0x2a95bb21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7315bb25, 91 },
    { 0, 0 },
    { 0x2655bb27, 80 },
    { 0xf035bb27, 90 },
    { 0xee05bb28, 98 },
    { 0, 0 },
    { 0xdd55bb2b, 84 },
    { 0x5945bb2c, 76 },
    { 0x6875bb2d, 61 },
    { 0, 0 },
    { 0x6525bb2f, 90 },
    { 0xa955bb2f, 34 },
    { 0xf235bb31, 23 },
    { 0, 0 },
    { 0x29d5bb33, 53 },
    { 0xf2b5bb33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f65bb39, 61 },
    { 0x5805bb3a, 95 },
    { 0, 0 },
    { 0x93b5bb3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d5bb41, 43 },
    { 0xf545bb42, 23 },
    { 0xb825bb43, 95 },
    { 0xd15bb42, 90 },
    { 0x9cb5bb45, 90 },
    { 0x5065bb46, 53 },
    { 0x3d45bb46, 87 },
    { 0, 0 },
    { 0x26f5bb49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f5bb4d, 90 },
    { 0xc8c5bb4e, 33 },
    { 0xe895bb4f, 90 },
    { 0, 0 },
    { 0xdf75bb51, 99 },
    { 0xffb5bb51, 98 },
    { 0, 0 },
    { 0x1825bb54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c5bb59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae45bb5c, 98 },
    { 0x3185bb5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc55bb65, 98 },
    { 0xd335bb66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5bb69, 95 },
    { 0, 0 },
    { 0xf7c5bb6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca5bb70, 67 },
    { 0xb2f5bb70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf5bb76, 34 },
    { 0, 0 },
    { 0x805bb78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6645bb7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad5bb81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc95bb85, 65 },
    { 0, 0 },
    { 0x3185bb87, 76 },
    { 0x85d5bb88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c85bb8c, 33 },
    { 0x6795bb8d, 67 },
    { 0xb375bb8c, 95 },
    { 0x6605bb8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe305bb98, 84 },
    { 0x7d5bb99, 65 },
    { 0x7a95bb9a, 91 },
    { 0, 0 },
    { 0x9055bb9c, 95 },
    { 0x7625bb9d, 95 },
    { 0xafa5bb9e, 23 },
    { 0x7855bb9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265bba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2045bbac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a85bbaf, 98 },
    { 0, 0 },
    { 0x1ca5bbb1, 90 },
    { 0x7885bbb1, 90 },
    { 0x58f5bbb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce5bbb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9955bbb9, 81 },
    { 0xdf45bbb9, 88 },
    { 0xa985bbb9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa75bbc0, 76 },
    { 0, 0 },
    { 0x4d95bbc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b5bbc5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b5bbcb, 55 },
    { 0x795bbcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2345bbcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865bbd4, 23 },
    { 0, 0 },
    { 0xf9f5bbd6, 76 },
    { 0x82d5bbd6, 77 },
    { 0x9db5bbd6, 95 },
    { 0x3655bbd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c5bbdc, 98 },
    { 0x9335bbdc, 99 },
    { 0xcaf5bbdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b5bbe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7845bbee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5bbf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275bbf9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d35bbfe, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6255bc0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c05bc17, 33 },
    { 0xfd45bc18, 66 },
    { 0xc55bc17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8135bc1e, 61 },
    { 0xab65bc1f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc5bc22, 43 },
    { 0, 0 },
    { 0x135bc24, 33 },
    { 0xcc95bc25, 76 },
    { 0xc865bc26, 98 },
    { 0x6aa5bc25, 87 },
    { 0xe125bc28, 98 },
    { 0xbb55bc29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb565bc2e, 55 },
    { 0xeef5bc2f, 99 },
    { 0x3715bc30, 90 },
    { 0xa195bc30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5bc34, 99 },
    { 0, 0 },
    { 0x83a5bc36, 33 },
    { 0x7f25bc37, 90 },
    { 0x12d5bc37, 55 },
    { 0xadf5bc36, 55 },
    { 0x2655bc3a, 34 },
    { 0x5bf5bc38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6545bc40, 95 },
    { 0x59a5bc41, 90 },
    { 0x6ba5bc42, 98 },
    { 0xed05bc40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5725bc4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d5bc4e, 46 },
    { 0, 0 },
    { 0x18a5bc50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf5bc55, 23 },
    { 0x43d5bc55, 95 },
    { 0, 0 },
    { 0xf9a5bc58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d5bc5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8295bc5e, 33 },
    { 0xb665bc5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc335bc64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f55bc6e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea5bc71, 6 },
    { 0, 0 },
    { 0xb215bc73, 34 },
    { 0x9595bc73, 83 },
    { 0x9165bc75, 87 },
    { 0x9ea5bc76, 90 },
    { 0x4c35bc77, 61 },
    { 0xc945bc77, 84 },
    { 0x6b45bc79, 95 },
    { 0xdee5bc79, 33 },
    { 0xd0b5bc7b, 33 },
    { 0xc495bc7c, 33 },
    { 0x2f05bc79, 90 },
    { 0xcb35bc7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc5bc83, 84 },
    { 0x50b5bc84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605bc89, 90 },
    { 0xe185bc8a, 87 },
    { 0x2345bc8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa05bc92, 68 },
    { 0x3c75bc93, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b5bc96, 87 },
    { 0xfba5bc97, 67 },
    { 0x2815bc98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8105bca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f5bca3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8565bca9, 91 },
    { 0, 0 },
    { 0xe465bcab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a15bcb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305bcbb, 98 },
    { 0x6bd5bcbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa565bcbf, 33 },
    { 0xf065bcc0, 66 },
    { 0x2ca5bcc0, 78 },
    { 0xdfd5bcc1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c35bcc7, 95 },
    { 0xdda5bcc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5bccf, 33 },
    { 0x2285bcd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb275bcd4, 76 },
    { 0x3455bcd5, 100 },
    { 0x6625bcd6, 98 },
    { 0xdc45bcd5, 33 },
    { 0xc4d5bcd5, 23 },
    { 0xdb85bcd7, 90 },
    { 0, 0 },
    { 0x70f5bcdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5bcdf, 69 },
    { 0, 0 },
    { 0x8815bce1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe75bce5, 90 },
    { 0xba05bce5, 76 },
    { 0xe045bce7, 87 },
    { 0xd1a5bce7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5bceb, 61 },
    { 0xbda5bcec, 84 },
    { 0x9f05bced, 90 },
    { 0x9285bcee, 85 },
    { 0x64e5bcef, 61 },
    { 0x2165bcf0, 95 },
    { 0x4465bcf1, 98 },
    { 0x7905bcf1, 23 },
    { 0x1d15bcf3, 43 },
    { 0x6f85bcf4, 66 },
    { 0xcf15bcf4, 33 },
    { 0x3c25bcf4, 90 },
    { 0x1685bcf7, 42 },
    { 0x34f5bcf5, 90 },
    { 0, 0 },
    { 0x5d85bcfa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe5bcfd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6205bd04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4785bd07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5bd0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e15bd11, 99 },
    { 0xa55bd11, 67 },
    { 0xcfc5bd13, 90 },
    { 0x2f25bd14, 34 },
    { 0x9415bd15, 88 },
    { 0xfc5bd16, 24 },
    { 0xc1a5bd15, 77 },
    { 0xe375bd14, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2785bd1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4525bd24, 33 },
    { 0, 0 },
    { 0x9c45bd26, 97 },
    { 0xfcf5bd27, 95 },
    { 0, 0 },
    { 0x16a5bd29, 52 },
    { 0xd315bd29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e5bd35, 46 },
    { 0xf885bd35, 99 },
    { 0, 0 },
    { 0x5245bd38, 76 },
    { 0x6f65bd38, 46 },
    { 0x5ff5bd3a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe585bd3d, 61 },
    { 0, 0 },
    { 0x94d5bd3f, 95 },
    { 0x7b95bd40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb85bd46, 23 },
    { 0, 0 },
    { 0xc615bd48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb415bd4b, 55 },
    { 0, 0 },
    { 0x29e5bd4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd25bd52, 68 },
    { 0xdab5bd53, 33 },
    { 0x3475bd54, 23 },
    { 0x2395bd55, 90 },
    { 0x3815bd56, 98 },
    { 0xcbc5bd57, 65 },
    { 0xa005bd58, 90 },
    { 0x78f5bd59, 95 },
    { 0, 0 },
    { 0x9df5bd5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5455bd60, 75 },
    { 0, 0 },
    { 0xa7e5bd62, 78 },
    { 0, 0 },
    { 0x7105bd64, 98 },
    { 0, 0 },
    { 0x785bd66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5485bd6b, 90 },
    { 0xc965bd6c, 90 },
    { 0x8025bd6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6855bd72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5bd76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee75bd7c, 76 },
    { 0x5075bd7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5bd81, 34 },
    { 0, 0 },
    { 0xffe5bd83, 90 },
    { 0x7155bd84, 52 },
    { 0xa2c5bd85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e15bd8f, 55 },
    { 0xf465bd90, 33 },
    { 0xcf25bd91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45bd97, 61 },
    { 0x4215bd97, 33 },
    { 0, 0 },
    { 0xe3c5bd9a, 93 },
    { 0, 0 },
    { 0xde35bd9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c5bd9f, 33 },
    { 0x5955bd9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1665bda7, 23 },
    { 0, 0 },
    { 0x6a55bda9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa915bdb1, 34 },
    { 0, 0 },
    { 0xb9a5bdb3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe55bdbe, 76 },
    { 0x4f85bdbe, 90 },
    { 0x1eb5bdc0, 43 },
    { 0x50d5bdc1, 77 },
    { 0, 0 },
    { 0xbb45bdc3, 95 },
    { 0x9975bdc4, 55 },
    { 0xadd5bdc5, 52 },
    { 0x8325bdc5, 66 },
    { 0xc355bdc7, 99 },
    { 0xe5d5bdc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf185bdcb, 90 },
    { 0x47a5bdcb, 98 },
    { 0xbca5bdcd, 55 },
    { 0x7de5bdcb, 33 },
    { 0x4f45bdce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6515bdd4, 55 },
    { 0x2055bdd5, 61 },
    { 0, 0 },
    { 0xae25bdd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee95bddb, 77 },
    { 0x38e5bddc, 33 },
    { 0x1f65bddb, 34 },
    { 0, 0 },
    { 0xae25bddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab5bde2, 33 },
    { 0, 0 },
    { 0xac75bde4, 65 },
    { 0x8505bde5, 33 },
    { 0xe295bde5, 75 },
    { 0xb8c5bde5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe175bdeb, 61 },
    { 0, 0 },
    { 0x7b45bded, 77 },
    { 0x4015bded, 46 },
    { 0, 0 },
    { 0x6905bdf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2145bdf4, 84 },
    { 0, 0 },
    { 0xcc15bdf6, 33 },
    { 0x53b5bdf7, 76 },
    { 0x745bdf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c5bdfb, 33 },
    { 0x5335bdfc, 91 },
    { 0, 0 },
    { 0xa685bdfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb445be02, 77 },
    { 0xb4c5be02, 33 },
    { 0xfcc5be04, 91 },
    { 0x4fb5be03, 66 },
    { 0xd3d5be04, 95 },
    { 0x73c5be07, 98 },
    { 0x6115be05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45be0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6315be10, 33 },
    { 0, 0 },
    { 0xe055be12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc25be15, 61 },
    { 0x6985be15, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa5be1e, 33 },
    { 0, 0 },
    { 0xb205be20, 95 },
    { 0, 0 },
    { 0xbe85be22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc5be25, 33 },
    { 0xce35be25, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5be2a, 52 },
    { 0xe8b5be2b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7165be2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05be38, 33 },
    { 0xda85be39, 95 },
    { 0xaa35be3a, 61 },
    { 0x63c5be3a, 84 },
    { 0x1725be38, 33 },
    { 0, 0 },
    { 0xf1f5be3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885be41, 98 },
    { 0, 0 },
    { 0x8055be43, 87 },
    { 0, 0 },
    { 0xa9d5be45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc55be48, 46 },
    { 0, 0 },
    { 0xd025be4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf5be51, 61 },
    { 0x50f5be52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd5be57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb5be5a, 46 },
    { 0xd8a5be5a, 100 },
    { 0xccf5be5c, 95 },
    { 0x75be5d, 34 },
    { 0x4bf5be5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce85be65, 76 },
    { 0xd1f5be65, 43 },
    { 0, 0 },
    { 0xd5c5be68, 34 },
    { 0, 0 },
    { 0x5cd5be6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7955be6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5be71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae5be79, 23 },
    { 0x9a75be79, 43 },
    { 0x7b65be7b, 23 },
    { 0xa225be7c, 95 },
    { 0xf005be7d, 95 },
    { 0xb475be7e, 23 },
    { 0x9795be7f, 55 },
    { 0xbe45be7d, 23 },
    { 0xd2e5be79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac85be84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed5be8a, 23 },
    { 0x7de5be8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd55be90, 93 },
    { 0x4255be91, 43 },
    { 0xfc05be92, 93 },
    { 0x7ee5be90, 100 },
    { 0x365be94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe765be9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3035bea1, 46 },
    { 0x7c25bea2, 100 },
    { 0x2795bea2, 90 },
    { 0xeff5bea4, 100 },
    { 0x9c05bea5, 98 },
    { 0, 0 },
    { 0xadb5bea7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb45bead, 66 },
    { 0x38f5bead, 87 },
    { 0x2c45beaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca5beb3, 98 },
    { 0, 0 },
    { 0x3f25beb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee5bec2, 81 },
    { 0, 0 },
    { 0x9ef5bec4, 33 },
    { 0x1195bec5, 97 },
    { 0x2125bec5, 98 },
    { 0, 0 },
    { 0xb245bec8, 69 },
    { 0, 0 },
    { 0x7735beca, 61 },
    { 0x6905beca, 90 },
    { 0, 0 },
    { 0xaad5becd, 91 },
    { 0x6e15becd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb645bed3, 23 },
    { 0, 0 },
    { 0xd345bed5, 23 },
    { 0, 0 },
    { 0xbcb5bed7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc5bede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a5bee1, 100 },
    { 0xb895bee2, 84 },
    { 0x2515bee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4275bee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3985beec, 88 },
    { 0xaad5beec, 90 },
    { 0x9995beee, 99 },
    { 0, 0 },
    { 0x26a5bef0, 66 },
    { 0xecd5bef0, 90 },
    { 0xfe5bef2, 23 },
    { 0, 0 },
    { 0x9b45bef4, 90 },
    { 0x2d75bef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc5bef9, 33 },
    { 0x2c85befa, 80 },
    { 0, 0 },
    { 0xdaf5befc, 93 },
    { 0x9bc5befd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c5bf02, 34 },
    { 0x1ff5bf03, 98 },
    { 0, 0 },
    { 0xd515bf05, 33 },
    { 0, 0 },
    { 0xc485bf07, 49 },
    { 0x3ce5bf08, 43 },
    { 0x9f5bf07, 58 },
    { 0, 0 },
    { 0x2d35bf0b, 33 },
    { 0, 0 },
    { 0xd695bf0d, 90 },
    { 0xf035bf0d, 98 },
    { 0, 0 },
    { 0x5d85bf10, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf5bf13, 85 },
    { 0x1d45bf14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bf1a, 34 },
    { 0x84d5bf1b, 33 },
    { 0x2595bf1c, 61 },
    { 0x5765bf1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e5bf24, 84 },
    { 0, 0 },
    { 0x23c5bf26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd5bf29, 23 },
    { 0xe525bf2a, 98 },
    { 0x9495bf2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e55bf2f, 84 },
    { 0, 0 },
    { 0xcd65bf31, 23 },
    { 0, 0 },
    { 0x1955bf33, 66 },
    { 0x1b55bf34, 98 },
    { 0x9805bf35, 33 },
    { 0x3505bf36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa5bf39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a5bf3e, 93 },
    { 0x8e95bf3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd5bf42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9425bf46, 78 },
    { 0xcbb5bf46, 61 },
    { 0, 0 },
    { 0xe4d5bf49, 95 },
    { 0, 0 },
    { 0x1ad5bf4b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f5bf4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3635bf53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6385bf56, 90 },
    { 0x4fd5bf57, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5815bf5a, 23 },
    { 0x5bf5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6735bf5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3745bf64, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bf69, 33 },
    { 0xb795bf6a, 55 },
    { 0x8aa5bf6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a55bf6e, 95 },
    { 0, 0 },
    { 0xd335bf70, 90 },
    { 0xf3c5bf71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4555bf74, 99 },
    { 0xcd5bf75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895bf79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f5bf80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8435bf87, 91 },
    { 0x6b65bf87, 55 },
    { 0, 0 },
    { 0x3da5bf8a, 33 },
    { 0, 0 },
    { 0x2855bf8c, 76 },
    { 0xde85bf8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9085bf94, 23 },
    { 0x7b5bf94, 55 },
    { 0x2965bf95, 98 },
    { 0xc6b5bf97, 61 },
    { 0x2cb5bf98, 76 },
    { 0x2145bf97, 95 },
    { 0x1c75bf9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5095bf9d, 95 },
    { 0, 0 },
    { 0xa3e5bf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb315bfa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5bfa7, 43 },
    { 0, 0 },
    { 0xffb5bfa9, 76 },
    { 0, 0 },
    { 0x6fd5bfab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275bfb0, 23 },
    { 0x8985bfb0, 75 },
    { 0, 0 },
    { 0xd945bfb3, 61 },
    { 0xd715bfb3, 23 },
    { 0x4415bfb5, 91 },
    { 0, 0 },
    { 0x4bc5bfb7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1875bfba, 55 },
    { 0xef65bfbb, 23 },
    { 0x8905bfba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb5bfc1, 98 },
    { 0xea45bfc2, 93 },
    { 0x70d5bfc2, 23 },
    { 0, 0 },
    { 0x2445bfc5, 98 },
    { 0xa0e5bfc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5bfc9, 95 },
    { 0x7485bfc9, 90 },
    { 0xa545bfcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e5bfce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b5bfd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd805bfd7, 89 },
    { 0x79c5bfd7, 90 },
    { 0, 0 },
    { 0xccb5bfda, 90 },
    { 0, 0 },
    { 0xc8c5bfdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5bfe0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda5bfe3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd705bfea, 23 },
    { 0x8c75bfea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3445bfee, 95 },
    { 0x5305bfee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b5bff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bff9, 61 },
    { 0xbaa5bffa, 90 },
    { 0, 0 },
    { 0x21e5bffc, 76 },
    { 0, 0 },
    { 0x1645bffe, 93 },
    { 0, 0 },
    { 0xc385c000, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305c004, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4925c010, 95 },
    { 0, 0 },
    { 0x6805c012, 98 },
    { 0, 0 },
    { 0x8bf5c014, 61 },
    { 0x9025c015, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb975c01e, 96 },
    { 0, 0 },
    { 0xd795c020, 23 },
    { 0xa275c021, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb25c024, 33 },
    { 0, 0 },
    { 0x2425c026, 99 },
    { 0xd965c027, 34 },
    { 0, 0 },
    { 0x4955c029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf5c02c, 55 },
    { 0x93c5c02d, 90 },
    { 0x67e5c02e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x905c031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd5c03a, 83 },
    { 0x94d5c03a, 66 },
    { 0, 0 },
    { 0xb975c03d, 33 },
    { 0xc245c03d, 95 },
    { 0, 0 },
    { 0x41f5c040, 93 },
    { 0x6d15c041, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d45c046, 23 },
    { 0, 0 },
    { 0x47c5c048, 91 },
    { 0x76d5c049, 93 },
    { 0, 0 },
    { 0x1d55c04b, 95 },
    { 0xa2a5c04b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f5c051, 95 },
    { 0x8875c051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb35c05a, 98 },
    { 0, 0 },
    { 0x6895c05c, 33 },
    { 0x6d05c05c, 75 },
    { 0x8955c05c, 90 },
    { 0xf785c05d, 91 },
    { 0xe8a5c060, 66 },
    { 0x5f85c060, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d55c064, 23 },
    { 0x3d55c064, 98 },
    { 0xbba5c065, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2235c06c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e5c06f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe085c072, 76 },
    { 0, 0 },
    { 0x38d5c074, 90 },
    { 0x5ea5c075, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc865c078, 76 },
    { 0xc3b5c079, 46 },
    { 0xe8c5c07a, 34 },
    { 0xa8d5c07a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4025c07e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5055c083, 85 },
    { 0xec25c084, 33 },
    { 0x47c5c085, 95 },
    { 0xc3f5c086, 95 },
    { 0x3f55c083, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf55c08c, 23 },
    { 0xfb55c08d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d5c094, 34 },
    { 0x60b5c094, 67 },
    { 0xbe35c096, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc55c099, 23 },
    { 0x72c5c09a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec85c0a0, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1165c0a4, 23 },
    { 0x2275c0a4, 23 },
    { 0x8665c0a4, 76 },
    { 0, 0 },
    { 0x6a25c0a8, 52 },
    { 0, 0 },
    { 0x3c25c0aa, 85 },
    { 0x9805c0ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e5c0af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a5c0b2, 76 },
    { 0x7d5c0b3, 43 },
    { 0, 0 },
    { 0xc075c0b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b5c0b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c95c0bd, 85 },
    { 0x78b5c0be, 98 },
    { 0xe425c0bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6755c0c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff55c0d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8645c0d4, 61 },
    { 0, 0 },
    { 0xd4d5c0d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e55c0d9, 46 },
    { 0, 0 },
    { 0xa205c0db, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d5c0df, 90 },
    { 0, 0 },
    { 0xb175c0e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795c0e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e95c0ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c85c0ef, 90 },
    { 0xae15c0f0, 34 },
    { 0x8b5c0f0, 33 },
    { 0, 0 },
    { 0x7be5c0f3, 98 },
    { 0xcc95c0f4, 93 },
    { 0x4485c0f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c5c0f8, 90 },
    { 0x9c25c0f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc885c0ff, 90 },
    { 0xdb95c100, 100 },
    { 0, 0 },
    { 0x36a5c102, 55 },
    { 0x36d5c102, 95 },
    { 0xb715c103, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf5c107, 61 },
    { 0xdf45c108, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c5c111, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7415c116, 46 },
    { 0xac5c117, 93 },
    { 0, 0 },
    { 0xcdd5c119, 61 },
    { 0x6575c119, 23 },
    { 0, 0 },
    { 0x3435c11c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae5c11f, 95 },
    { 0xbac5c11f, 46 },
    { 0x4f25c11f, 95 },
    { 0xdc95c122, 33 },
    { 0x3e45c123, 23 },
    { 0, 0 },
    { 0x3455c125, 87 },
    { 0xf095c125, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d35c12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe655c12f, 100 },
    { 0xec55c130, 98 },
    { 0x54a5c130, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd245c134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d5c137, 85 },
    { 0x9755c138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x365c13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db5c13e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f75c142, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5c148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f5c14d, 66 },
    { 0x30d5c14e, 76 },
    { 0, 0 },
    { 0x5b65c150, 34 },
    { 0xcd85c150, 76 },
    { 0, 0 },
    { 0x6da5c153, 95 },
    { 0xd765c154, 52 },
    { 0x8715c154, 34 },
    { 0xfa25c153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd5c15b, 90 },
    { 0xd225c15c, 90 },
    { 0xcda5c15d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a5c160, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1535c165, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8795c169, 95 },
    { 0xc035c169, 17 },
    { 0x81f5c16b, 78 },
    { 0x7615c16b, 46 },
    { 0x84e5c16d, 69 },
    { 0, 0 },
    { 0xf125c16f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5795c172, 90 },
    { 0x39f5c173, 90 },
    { 0, 0 },
    { 0x5875c175, 98 },
    { 0xc7f5c176, 52 },
    { 0xf0d5c177, 33 },
    { 0x8ca5c178, 90 },
    { 0x5e05c178, 98 },
    { 0x5f5c17a, 40 },
    { 0x1ac5c17b, 78 },
    { 0xe4d5c176, 33 },
    { 0x2305c17b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6415c180, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115c183, 93 },
    { 0xb135c184, 33 },
    { 0xf4e5c185, 55 },
    { 0xe325c185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f5c189, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d5c18d, 90 },
    { 0, 0 },
    { 0x8375c18f, 95 },
    { 0xa015c190, 77 },
    { 0, 0 },
    { 0x2a95c192, 77 },
    { 0xb3c5c193, 25 },
    { 0x395c194, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215c197, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5495c19b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d5c19f, 90 },
    { 0, 0 },
    { 0x91d5c1a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb95c1a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d05c1a8, 76 },
    { 0x1195c1a9, 95 },
    { 0, 0 },
    { 0xa535c1ab, 76 },
    { 0, 0 },
    { 0x10a5c1ad, 45 },
    { 0xc65c1ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3055c1b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea5c1b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc555c1b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d85c1bf, 61 },
    { 0x6cb5c1bf, 61 },
    { 0x64d5c1c1, 98 },
    { 0x73b5c1c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2795c1c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e75c1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f5c1d0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad05c1da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc345c1df, 46 },
    { 0x48e5c1df, 66 },
    { 0x2775c1e0, 96 },
    { 0x61c5c1e2, 84 },
    { 0x4975c1e3, 90 },
    { 0, 0 },
    { 0x9c45c1e5, 23 },
    { 0x1a55c1e6, 98 },
    { 0xaa05c1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e05c1ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb5c1ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c1f2, 34 },
    { 0, 0 },
    { 0xcf45c1f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d45c1f7, 43 },
    { 0, 0 },
    { 0xfa55c1f9, 90 },
    { 0xdd5c1fa, 90 },
    { 0x2f15c1fb, 90 },
    { 0x91a5c1fc, 76 },
    { 0x97d5c1fd, 98 },
    { 0x1af5c1fc, 61 },
    { 0, 0 },
    { 0x3f55c200, 34 },
    { 0xda5c201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff05c206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9635c20a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5385c20e, 98 },
    { 0, 0 },
    { 0x41c5c210, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca5c216, 84 },
    { 0, 0 },
    { 0x44b5c218, 100 },
    { 0, 0 },
    { 0xf215c21a, 98 },
    { 0x9615c21b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb5c220, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e5c225, 95 },
    { 0, 0 },
    { 0xbc95c227, 61 },
    { 0x6bd5c228, 55 },
    { 0x3d45c227, 98 },
    { 0x6e75c22a, 61 },
    { 0x80a5c229, 55 },
    { 0x1ee5c22c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a5c230, 33 },
    { 0x29a5c231, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe395c235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe215c239, 95 },
    { 0xa185c239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a5c23d, 87 },
    { 0, 0 },
    { 0xd15c23f, 61 },
    { 0x4945c240, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f5c243, 91 },
    { 0, 0 },
    { 0x1935c245, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8275c249, 33 },
    { 0, 0 },
    { 0x3da5c24b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c5c24e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad95c251, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a85c258, 53 },
    { 0x7235c259, 90 },
    { 0, 0 },
    { 0xf135c25b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8555c25f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde95c265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755c273, 98 },
    { 0, 0 },
    { 0x66e5c275, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5c27c, 78 },
    { 0x2a95c27d, 90 },
    { 0xfd65c27d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd15c286, 99 },
    { 0x2475c286, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f5c28a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5c28f, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae5c295, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e5c29b, 66 },
    { 0, 0 },
    { 0x2945c29d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7265c2a0, 100 },
    { 0, 0 },
    { 0x8095c2a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc85c2a7, 90 },
    { 0x8b05c2a8, 34 },
    { 0x6ea5c2a8, 100 },
    { 0x21d5c2aa, 61 },
    { 0x4585c2ab, 52 },
    { 0xd6a5c2aa, 61 },
    { 0x53d5c2a7, 67 },
    { 0x8935c2a8, 76 },
    { 0xc505c2af, 98 },
    { 0xe025c2b0, 93 },
    { 0xe7a5c2b0, 61 },
    { 0x76d5c2ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b65c2b5, 39 },
    { 0xd1b5c2b6, 23 },
    { 0x4b85c2b7, 61 },
    { 0x31e5c2b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4355c2bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9185c2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf5c2c4, 76 },
    { 0x305c2c4, 84 },
    { 0, 0 },
    { 0x565c2c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb5c2ca, 95 },
    { 0x785c2ca, 23 },
    { 0x3fe5c2cb, 91 },
    { 0xaf35c2cd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f5c2d0, 98 },
    { 0x3d65c2d1, 46 },
    { 0xf255c2d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25c2d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5c2dc, 40 },
    { 0, 0 },
    { 0x2f05c2de, 87 },
    { 0x4635c2df, 61 },
    { 0, 0 },
    { 0x51d5c2e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1995c2e4, 100 },
    { 0x34c5c2e5, 61 },
    { 0, 0 },
    { 0x8145c2e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9755c2ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15c2f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe995c2f5, 95 },
    { 0xfa5c2f6, 98 },
    { 0, 0 },
    { 0x1345c2f8, 87 },
    { 0, 0 },
    { 0x92a5c2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe5c2fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa715c301, 34 },
    { 0, 0 },
    { 0x6625c303, 76 },
    { 0xe5d5c304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a5c307, 61 },
    { 0xafd5c308, 23 },
    { 0x1015c307, 98 },
    { 0x6475c30a, 33 },
    { 0xc8c5c30a, 33 },
    { 0x3fa5c309, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2675c312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c85c317, 90 },
    { 0xd05c317, 95 },
    { 0x37f5c319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c31e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd85c325, 76 },
    { 0xc955c326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b65c32a, 61 },
    { 0x30f5c32a, 61 },
    { 0, 0 },
    { 0x8325c32d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f5c330, 95 },
    { 0x8315c331, 52 },
    { 0xdf55c332, 69 },
    { 0xb175c333, 98 },
    { 0xb875c334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4895c338, 55 },
    { 0xfd45c338, 74 },
    { 0xe3c5c339, 61 },
    { 0, 0 },
    { 0x6165c33c, 90 },
    { 0x16a5c33d, 33 },
    { 0xc385c33e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5c343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c348, 33 },
    { 0xa7f5c349, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e5c34d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1105c351, 33 },
    { 0xeef5c351, 51 },
    { 0x3185c353, 98 },
    { 0, 0 },
    { 0xf475c355, 100 },
    { 0x5595c356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb985c35a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc25c35e, 46 },
    { 0, 0 },
    { 0x80b5c360, 61 },
    { 0, 0 },
    { 0x955c362, 98 },
    { 0x5495c363, 33 },
    { 0, 0 },
    { 0x7b55c365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435c368, 100 },
    { 0x4e45c369, 33 },
    { 0x8905c36a, 66 },
    { 0xdcb5c369, 85 },
    { 0x5865c36c, 98 },
    { 0, 0 },
    { 0xd805c36e, 61 },
    { 0xa9e5c36e, 90 },
    { 0, 0 },
    { 0xa985c371, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f5c374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d25c378, 98 },
    { 0, 0 },
    { 0x2145c37a, 40 },
    { 0x84b5c37b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1635c37e, 95 },
    { 0x5215c37e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b25c384, 23 },
    { 0xb575c385, 52 },
    { 0x41c5c386, 88 },
    { 0x9655c387, 33 },
    { 0xefa5c388, 34 },
    { 0x93e5c388, 90 },
    { 0xd3e5c38a, 61 },
    { 0x3ae5c38b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a5c38e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f65c391, 33 },
    { 0x2e95c391, 23 },
    { 0xc885c393, 40 },
    { 0xbd25c393, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5c397, 61 },
    { 0x8ae5c397, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9865c3a0, 33 },
    { 0x3ee5c3a1, 76 },
    { 0x8ab5c3a0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a5c3a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8445c3a8, 58 },
    { 0xd7e5c3a9, 61 },
    { 0x5765c3aa, 61 },
    { 0x4795c3aa, 23 },
    { 0x8925c3ac, 65 },
    { 0x2615c3ad, 96 },
    { 0xc425c3ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7555c3b2, 98 },
    { 0xa825c3b3, 95 },
    { 0x81e5c3b4, 87 },
    { 0xb065c3b5, 76 },
    { 0x6e35c3b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd785c3bc, 55 },
    { 0, 0 },
    { 0x52e5c3be, 95 },
    { 0, 0 },
    { 0xbf5c3c0, 80 },
    { 0xcfa5c3c1, 95 },
    { 0, 0 },
    { 0xd555c3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5225c3c6, 33 },
    { 0xf565c3c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc5c3d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d35c3d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635c3dc, 90 },
    { 0, 0 },
    { 0xbb05c3de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e5c3e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec15c3e7, 90 },
    { 0xec5c3e7, 95 },
    { 0, 0 },
    { 0x475c3ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc445c3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6635c3f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb065c3f6, 23 },
    { 0xc565c3f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf525c3fb, 61 },
    { 0, 0 },
    { 0x32b5c3fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5035c404, 66 },
    { 0, 0 },
    { 0xe8f5c406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d5c40c, 23 },
    { 0x2705c40d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea5c413, 33 },
    { 0, 0 },
    { 0x245c415, 52 },
    { 0xe765c416, 101 },
    { 0, 0 },
    { 0xa5c418, 100 },
    { 0xb415c418, 43 },
    { 0x3965c418, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6665c422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5755c425, 66 },
    { 0x9905c426, 95 },
    { 0x8865c427, 33 },
    { 0xa405c427, 46 },
    { 0, 0 },
    { 0xe775c42a, 85 },
    { 0x745c42a, 64 },
    { 0xbab5c42c, 90 },
    { 0xc555c42d, 90 },
    { 0, 0 },
    { 0xc435c42f, 100 },
    { 0x7145c430, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd435c433, 61 },
    { 0, 0 },
    { 0x8615c435, 90 },
    { 0, 0 },
    { 0x5385c437, 90 },
    { 0, 0 },
    { 0x8ce5c439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2335c444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a5c44a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b5c44f, 55 },
    { 0x73c5c450, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd625c454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b5c45a, 76 },
    { 0, 0 },
    { 0xed85c45c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5c460, 76 },
    { 0x3775c461, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9595c467, 91 },
    { 0, 0 },
    { 0x5be5c469, 23 },
    { 0xc3e5c46a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f95c473, 34 },
    { 0, 0 },
    { 0xdd95c475, 90 },
    { 0, 0 },
    { 0x7bd5c477, 23 },
    { 0, 0 },
    { 0x3125c479, 46 },
    { 0, 0 },
    { 0x2465c47b, 33 },
    { 0xe1c5c47c, 33 },
    { 0x3f05c47d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5c480, 76 },
    { 0x1f45c480, 90 },
    { 0, 0 },
    { 0xd5a5c483, 85 },
    { 0xc55c483, 76 },
    { 0x8125c483, 46 },
    { 0x7ed5c484, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb25c48c, 70 },
    { 0xfe25c48d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b15c495, 33 },
    { 0, 0 },
    { 0xc265c497, 67 },
    { 0x8125c498, 45 },
    { 0xab35c499, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1935c49f, 76 },
    { 0x6075c4a0, 90 },
    { 0x4085c4a1, 33 },
    { 0x75c5c4a2, 46 },
    { 0, 0 },
    { 0xdd55c4a4, 33 },
    { 0xf2d5c4a5, 75 },
    { 0, 0 },
    { 0xc95c4a7, 33 },
    { 0x695c4a8, 23 },
    { 0, 0 },
    { 0xb775c4aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb5c4ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa445c4b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd525c4b4, 93 },
    { 0xe0c5c4b5, 95 },
    { 0x6fa5c4b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9245c4b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515c4c3, 98 },
    { 0x2875c4c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa255c4c8, 33 },
    { 0, 0 },
    { 0xafc5c4ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f5c4cf, 36 },
    { 0, 0 },
    { 0xfbb5c4d1, 43 },
    { 0x9d95c4d1, 82 },
    { 0xa165c4d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4945c4d6, 95 },
    { 0x9735c4d6, 76 },
    { 0xd835c4d8, 56 },
    { 0, 0 },
    { 0x6895c4da, 46 },
    { 0, 0 },
    { 0xd605c4dc, 33 },
    { 0x3825c4dd, 90 },
    { 0x2e5c4de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d5c4e2, 23 },
    { 0x4b85c4e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7375c4ea, 90 },
    { 0x4e05c4eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df5c4ee, 95 },
    { 0xe015c4ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c5c4f2, 52 },
    { 0x2d25c4f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e85c4f6, 33 },
    { 0xda45c4f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd5c4ff, 87 },
    { 0, 0 },
    { 0xfa05c501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d5c504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85c509, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35c50c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8285c514, 84 },
    { 0x2745c514, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda5c51a, 46 },
    { 0x4085c51b, 33 },
    { 0xbf35c51c, 76 },
    { 0x41b5c51a, 95 },
    { 0x46f5c51e, 33 },
    { 0x4745c51f, 90 },
    { 0x70e5c520, 90 },
    { 0x93a5c521, 80 },
    { 0, 0 },
    { 0x9b5c523, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd865c526, 66 },
    { 0xba75c527, 88 },
    { 0x1745c527, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1445c52e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c5c531, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca5c534, 23 },
    { 0xf195c535, 95 },
    { 0x96a5c536, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce5c53d, 93 },
    { 0x6245c53e, 95 },
    { 0xa915c53e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d5c543, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5c548, 97 },
    { 0xf05c549, 90 },
    { 0x2ea5c54a, 33 },
    { 0x695c549, 95 },
    { 0, 0 },
    { 0x5a15c54d, 33 },
    { 0xd9e5c54e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a5c551, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3295c557, 33 },
    { 0xd8b5c557, 95 },
    { 0x22d5c558, 95 },
    { 0, 0 },
    { 0x665c55b, 90 },
    { 0xadc5c55c, 43 },
    { 0xff65c55c, 33 },
    { 0x4525c55c, 59 },
    { 0x9135c55c, 95 },
    { 0xba5c560, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c5c568, 90 },
    { 0, 0 },
    { 0xc585c56a, 23 },
    { 0xe0e5c56b, 98 },
    { 0, 0 },
    { 0xca95c56d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea95c577, 33 },
    { 0x305c578, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f45c585, 55 },
    { 0x9225c586, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c5c58a, 98 },
    { 0xa2e5c58a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5c591, 55 },
    { 0x8e35c592, 90 },
    { 0, 0 },
    { 0x415c594, 90 },
    { 0, 0 },
    { 0x6a45c596, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8635c599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf5c59d, 87 },
    { 0x4d15c59e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad5c5a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6605c5a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6825c5b2, 93 },
    { 0x8215c5b3, 90 },
    { 0xe9c5c5b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef5c5b7, 33 },
    { 0xcd65c5b7, 23 },
    { 0x4905c5b9, 66 },
    { 0x9115c5b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f05c5bd, 85 },
    { 0x90d5c5be, 43 },
    { 0x6085c5bf, 95 },
    { 0xcca5c5c0, 100 },
    { 0x69b5c5c1, 98 },
    { 0xb7c5c5c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5c5c5, 76 },
    { 0xc2f5c5c6, 98 },
    { 0, 0 },
    { 0xe9e5c5c8, 84 },
    { 0, 0 },
    { 0xe915c5ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea15c5d3, 100 },
    { 0x5a65c5d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd55c5d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b5c5dd, 85 },
    { 0x1aa5c5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd5c5e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa575c5e7, 90 },
    { 0x9115c5e7, 36 },
    { 0, 0 },
    { 0x2e75c5ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4805c5ed, 84 },
    { 0x7ed5c5ed, 34 },
    { 0, 0 },
    { 0x84f5c5f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce5c5f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055c5fb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xca55c5fe, 90 },
    { 0x15e5c5ff, 55 },
    { 0x8575c5fe, 34 },
    { 0x27b5c601, 61 },
    { 0xfb15c602, 23 },
    { 0xf7e5c602, 66 },
    { 0xf975c604, 33 },
    { 0, 0 },
    { 0x8625c606, 66 },
    { 0x7345c607, 61 },
    { 0xcbb5c607, 98 },
    { 0x3965c609, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f5c60e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa955c611, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd935c614, 61 },
    { 0x8f55c615, 61 },
    { 0x31e5c615, 90 },
    { 0, 0 },
    { 0x97e5c618, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2435c61b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe165c61e, 61 },
    { 0x44b5c61e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf085c622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df5c625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635c629, 33 },
    { 0, 0 },
    { 0x14a5c62b, 61 },
    { 0xe8e5c62c, 61 },
    { 0, 0 },
    { 0x2315c62e, 85 },
    { 0, 0 },
    { 0xa985c630, 55 },
    { 0, 0 },
    { 0x15c632, 81 },
    { 0, 0 },
    { 0xf735c634, 23 },
    { 0xd3e5c635, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9945c638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a5c63b, 22 },
    { 0xdbd5c63c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9255c63f, 90 },
    { 0, 0 },
    { 0xf925c641, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd65c644, 90 },
    { 0x8b05c645, 23 },
    { 0xc915c646, 90 },
    { 0, 0 },
    { 0x1f05c648, 87 },
    { 0xdbf5c649, 95 },
    { 0xcd45c648, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f95c64d, 90 },
    { 0xf4e5c64e, 68 },
    { 0x8ff5c64d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6795c652, 61 },
    { 0x1e55c652, 78 },
    { 0x5dd5c654, 98 },
    { 0, 0 },
    { 0x39e5c656, 23 },
    { 0, 0 },
    { 0x5cc5c658, 23 },
    { 0, 0 },
    { 0xf475c65a, 52 },
    { 0x645c65b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb245c660, 23 },
    { 0xb255c661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3125c664, 61 },
    { 0, 0 },
    { 0x5dc5c666, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f05c669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f35c66d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a05c672, 87 },
    { 0x7395c672, 98 },
    { 0x2de5c673, 23 },
    { 0, 0 },
    { 0x3a5c676, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365c67d, 17 },
    { 0, 0 },
    { 0x56f5c67f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da5c683, 40 },
    { 0xf075c684, 90 },
    { 0xd25c684, 33 },
    { 0, 0 },
    { 0x68e5c687, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd695c68b, 49 },
    { 0x3425c68c, 87 },
    { 0x8e75c68c, 98 },
    { 0x6565c68d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e5c696, 68 },
    { 0x4a75c696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3995c69a, 90 },
    { 0x8a35c69b, 61 },
    { 0x2be5c69a, 33 },
    { 0x5b95c69d, 33 },
    { 0x1395c69d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8445c6a2, 87 },
    { 0x5795c6a3, 67 },
    { 0xc035c6a4, 52 },
    { 0xe9a5c6a5, 66 },
    { 0, 0 },
    { 0xd0b5c6a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8375c6aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5895c6ad, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c15c6b0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055c6b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5c6b7, 23 },
    { 0x1c35c6b8, 33 },
    { 0, 0 },
    { 0xcd15c6ba, 61 },
    { 0xc715c6bb, 65 },
    { 0, 0 },
    { 0xd735c6bd, 100 },
    { 0x5495c6be, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff5c6c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d5c6cf, 9 },
    { 0xcf85c6cf, 98 },
    { 0xf025c6d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb5c6d6, 99 },
    { 0x5775c6d7, 95 },
    { 0x61b5c6d7, 85 },
    { 0x9595c6d9, 88 },
    { 0x5a25c6d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf645c6de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7045c6e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5315c6e9, 61 },
    { 0x6d85c6ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7785c6ed, 23 },
    { 0, 0 },
    { 0xf735c6ef, 33 },
    { 0, 0 },
    { 0xb6f5c6f1, 98 },
    { 0x93e5c6f2, 93 },
    { 0xddd5c6f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b35c6f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e5c6fa, 78 },
    { 0, 0 },
    { 0x9405c6fc, 33 },
    { 0x5475c6fd, 98 },
    { 0xe305c6fe, 98 },
    { 0xae35c6ff, 46 },
    { 0x8375c6fe, 55 },
    { 0x7555c701, 98 },
    { 0, 0 },
    { 0xce15c703, 91 },
    { 0xb3b5c704, 90 },
    { 0, 0 },
    { 0xe6a5c706, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6545c709, 76 },
    { 0xe125c70a, 98 },
    { 0x5e05c70b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a5c716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865c71f, 61 },
    { 0x2f05c720, 76 },
    { 0x1d65c721, 90 },
    { 0x6665c722, 93 },
    { 0x1b75c723, 23 },
    { 0xaf5c724, 98 },
    { 0x3bb5c721, 90 },
    { 0x97d5c726, 33 },
    { 0xec25c726, 94 },
    { 0xc405c722, 76 },
    { 0xd8c5c729, 76 },
    { 0xa825c72a, 95 },
    { 0x9e65c72b, 90 },
    { 0xad25c72c, 55 },
    { 0xf2a5c722, 43 },
    { 0x6035c72e, 33 },
    { 0x3965c72f, 33 },
    { 0x1035c72e, 33 },
    { 0x9eb5c731, 100 },
    { 0xd5f5c732, 90 },
    { 0x9f75c728, 33 },
    { 0xecc5c734, 76 },
    { 0xa225c735, 99 },
    { 0x8715c721, 46 },
    { 0x8d5c730, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed35c73d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa465c747, 52 },
    { 0xba65c748, 33 },
    { 0, 0 },
    { 0x7045c74a, 67 },
    { 0x2a5c74a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2865c74e, 95 },
    { 0, 0 },
    { 0x4ba5c750, 52 },
    { 0xe835c751, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb5c757, 64 },
    { 0x7925c758, 34 },
    { 0x18f5c759, 90 },
    { 0x3615c759, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb65c75e, 99 },
    { 0x3405c75f, 90 },
    { 0x4fb5c75e, 55 },
    { 0xb865c760, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8985c766, 33 },
    { 0xf9b5c767, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba5c76a, 95 },
    { 0x66e5c76a, 33 },
    { 0x26a5c76c, 52 },
    { 0xa495c76c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff75c773, 78 },
    { 0, 0 },
    { 0x9205c775, 17 },
    { 0, 0 },
    { 0xf645c777, 77 },
    { 0, 0 },
    { 0xb565c779, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2125c77e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb405c784, 33 },
    { 0xf235c785, 81 },
    { 0x5575c784, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7975c78e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d25c791, 99 },
    { 0xa8b5c791, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2055c795, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f5c798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b45c79c, 95 },
    { 0, 0 },
    { 0x13c5c79e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c5c7a1, 61 },
    { 0x2425c7a2, 55 },
    { 0xd185c7a1, 75 },
    { 0xe595c7a1, 90 },
    { 0xfba5c7a5, 40 },
    { 0xc85c7a5, 91 },
    { 0xfbf5c7a3, 55 },
    { 0, 0 },
    { 0x7625c7a9, 43 },
    { 0x95c5c7aa, 90 },
    { 0, 0 },
    { 0xb8f5c7ac, 95 },
    { 0xac65c7ac, 34 },
    { 0x3a25c7ac, 90 },
    { 0xa865c7ac, 90 },
    { 0xadf5c7b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3165c7b9, 95 },
    { 0xfc85c7ba, 33 },
    { 0x1425c7bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb45c7c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc815c7c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba5c7cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xac35c7cf, 68 },
    { 0, 0 },
    { 0xf2d5c7d1, 33 },
    { 0xc795c7d2, 90 },
    { 0x5c65c7d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d95c7da, 61 },
    { 0, 0 },
    { 0xe905c7dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f5c7df, 61 },
    { 0xbf65c7e0, 95 },
    { 0x55e5c7df, 67 },
    { 0, 0 },
    { 0x105c7e3, 90 },
    { 0x6575c7e4, 67 },
    { 0x18e5c7e5, 93 },
    { 0x8f55c7e3, 78 },
    { 0x3ff5c7e6, 75 },
    { 0x77c5c7e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e5c7ec, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4625c7f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c5c7fb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa75c7fe, 78 },
    { 0x62e5c7fe, 40 },
    { 0x6395c7ff, 34 },
    { 0xbc15c801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe5c804, 90 },
    { 0x9be5c805, 38 },
    { 0, 0 },
    { 0x3bb5c807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695c80e, 78 },
    { 0, 0 },
    { 0x6495c810, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3205c817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d15c81a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2695c81d, 17 },
    { 0x8b45c81e, 77 },
    { 0xffc5c81f, 90 },
    { 0x3d55c81d, 91 },
    { 0x5d75c821, 33 },
    { 0x80b5c822, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3565c825, 90 },
    { 0xadd5c826, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b5c82c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c5c831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa275c837, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xde55c83a, 23 },
    { 0, 0 },
    { 0xa275c83c, 85 },
    { 0x1cf5c83c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b85c841, 33 },
    { 0x9a15c842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe325c84a, 90 },
    { 0x3c95c84a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca5c850, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2545c854, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b5c859, 95 },
    { 0x14c5c85a, 23 },
    { 0x5605c85b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4575c85f, 98 },
    { 0x6615c85f, 55 },
    { 0, 0 },
    { 0xd565c862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d5c866, 23 },
    { 0x8535c867, 55 },
    { 0, 0 },
    { 0x7d15c869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea5c86e, 55 },
    { 0, 0 },
    { 0xc865c870, 55 },
    { 0xd595c871, 23 },
    { 0x3ab5c872, 95 },
    { 0xd425c872, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6205c87a, 43 },
    { 0x9075c87a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf15c881, 88 },
    { 0x6025c882, 91 },
    { 0xfd5c882, 67 },
    { 0x7475c884, 61 },
    { 0x1615c885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa405c88e, 78 },
    { 0x1a55c88f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef5c892, 52 },
    { 0xffc5c893, 78 },
    { 0x2095c893, 100 },
    { 0x3615c895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85c89c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa605c89f, 90 },
    { 0xba75c8a0, 90 },
    { 0x3e25c89f, 90 },
    { 0x3c15c8a2, 23 },
    { 0, 0 },
    { 0x2035c8a4, 33 },
    { 0, 0 },
    { 0x7c85c8a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5255c8ac, 85 },
    { 0, 0 },
    { 0x6355c8ae, 40 },
    { 0, 0 },
    { 0xcb5c8b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf145c8b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c8ba, 61 },
    { 0x7935c8bb, 43 },
    { 0x62f5c8bc, 61 },
    { 0x635c8bd, 84 },
    { 0xce75c8bd, 34 },
    { 0, 0 },
    { 0x53b5c8c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e5c8c4, 98 },
    { 0, 0 },
    { 0x6445c8c6, 100 },
    { 0x33a5c8c6, 68 },
    { 0x9e15c8c6, 34 },
    { 0, 0 },
    { 0x2815c8ca, 33 },
    { 0x6195c8cb, 90 },
    { 0x23e5c8cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f5c8d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee65c8d5, 23 },
    { 0x4185c8d5, 95 },
    { 0xae65c8d7, 68 },
    { 0, 0 },
    { 0x82b5c8d9, 95 },
    { 0x6dd5c8d9, 52 },
    { 0xf955c8d9, 95 },
    { 0x4c75c8d9, 98 },
    { 0x2705c8dd, 43 },
    { 0xa955c8de, 33 },
    { 0x2145c8df, 88 },
    { 0, 0 },
    { 0xdfb5c8e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d5c8e4, 32 },
    { 0x29a5c8e5, 23 },
    { 0x80d5c8e6, 23 },
    { 0x7165c8e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f5c8ec, 23 },
    { 0x2095c8ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2755c900, 33 },
    { 0x3195c900, 76 },
    { 0, 0 },
    { 0xbe75c903, 90 },
    { 0x14a5c904, 33 },
    { 0x8c5c905, 90 },
    { 0x77e5c906, 46 },
    { 0xd45c906, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e5c90b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed35c90f, 46 },
    { 0xa835c90f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd175c915, 76 },
    { 0x4975c915, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf45c91d, 85 },
    { 0xa3b5c91d, 43 },
    { 0xba75c91f, 98 },
    { 0xd205c920, 93 },
    { 0, 0 },
    { 0x8255c922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8455c925, 85 },
    { 0x8445c926, 99 },
    { 0, 0 },
    { 0xc225c928, 90 },
    { 0, 0 },
    { 0x1b05c92a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b35c92e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6785c939, 76 },
    { 0x8cb5c93a, 100 },
    { 0xdc05c93b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc5c93e, 23 },
    { 0xa015c93f, 23 },
    { 0xbd55c940, 46 },
    { 0, 0 },
    { 0x6d35c942, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a5c947, 55 },
    { 0, 0 },
    { 0x8cc5c949, 78 },
    { 0x5985c94a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd925c953, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf255c956, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8045c95b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe365c95f, 98 },
    { 0, 0 },
    { 0x4a35c961, 90 },
    { 0x5015c961, 61 },
    { 0xbcd5c963, 95 },
    { 0xd2e5c964, 55 },
    { 0x4715c965, 98 },
    { 0x9825c966, 84 },
    { 0x45c5c967, 43 },
    { 0xbe05c966, 61 },
    { 0x17e5c966, 33 },
    { 0x93d5c964, 95 },
    { 0x4c35c968, 98 },
    { 0, 0 },
    { 0x11d5c96d, 100 },
    { 0, 0 },
    { 0x48d5c96f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd5c972, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d75c976, 78 },
    { 0xc635c977, 66 },
    { 0x9a5c978, 33 },
    { 0x5775c977, 95 },
    { 0x86e5c97a, 61 },
    { 0xd2b5c976, 33 },
    { 0xeb95c97a, 100 },
    { 0xd815c97c, 91 },
    { 0, 0 },
    { 0x1fb5c97f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b5c982, 90 },
    { 0x8ae5c982, 64 },
    { 0xc415c982, 33 },
    { 0xfde5c984, 85 },
    { 0, 0 },
    { 0x185c987, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd835c98a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c55c98f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f5c994, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb5c99a, 33 },
    { 0, 0 },
    { 0x4d65c99c, 98 },
    { 0x1375c99c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4995c9a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f5c9a7, 23 },
    { 0x86e5c9a7, 33 },
    { 0x5445c9a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb165c9b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c9b3, 90 },
    { 0x8495c9b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb35c9be, 68 },
    { 0xb605c9bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa375c9c5, 90 },
    { 0xb1d5c9c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d5c9c9, 90 },
    { 0, 0 },
    { 0x60e5c9cb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d5c9ce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f5c9d7, 55 },
    { 0, 0 },
    { 0x3dc5c9d9, 33 },
    { 0xeab5c9d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd275c9de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec5c9e1, 76 },
    { 0xf665c9e2, 98 },
    { 0xd865c9e3, 33 },
    { 0xfe55c9e4, 78 },
    { 0x5a15c9e5, 75 },
    { 0x5e65c9e6, 55 },
    { 0x2c35c9e7, 61 },
    { 0xbf55c9e7, 22 },
    { 0xf695c9e3, 34 },
    { 0x18e5c9ea, 99 },
    { 0x2f15c9e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d5c9ee, 34 },
    { 0xbc85c9ef, 42 },
    { 0, 0 },
    { 0x295c9f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b45c9f4, 23 },
    { 0, 0 },
    { 0xecf5c9f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc5c9fb, 55 },
    { 0x9295c9fb, 90 },
    { 0xdb15c9fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d75ca01, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb775ca07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5ca0b, 23 },
    { 0, 0 },
    { 0x4ab5ca0d, 34 },
    { 0, 0 },
    { 0xdf05ca0f, 90 },
    { 0, 0 },
    { 0x27e5ca11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7645ca14, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c5ca17, 98 },
    { 0, 0 },
    { 0x6625ca19, 33 },
    { 0, 0 },
    { 0xce05ca1b, 87 },
    { 0x8a35ca1c, 65 },
    { 0, 0 },
    { 0xbd35ca1e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f5ca24, 23 },
    { 0xdf35ca24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5ca29, 87 },
    { 0, 0 },
    { 0x8bc5ca2b, 90 },
    { 0xaa75ca2c, 77 },
    { 0x63a5ca2c, 95 },
    { 0x2065ca2e, 93 },
    { 0, 0 },
    { 0x6a55ca30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d25ca34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5ca38, 61 },
    { 0, 0 },
    { 0x9e85ca3a, 33 },
    { 0x6435ca3b, 91 },
    { 0xa185ca3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2455ca3f, 55 },
    { 0, 0 },
    { 0xa65ca41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ca44, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b95ca4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd5ca4f, 38 },
    { 0x1c05ca4f, 76 },
    { 0xfb55ca4f, 55 },
    { 0xc025ca52, 90 },
    { 0x76b5ca52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e75ca57, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa05ca5c, 85 },
    { 0x6295ca5d, 34 },
    { 0xe825ca5d, 61 },
    { 0x765ca5f, 23 },
    { 0, 0 },
    { 0xe435ca61, 61 },
    { 0x20d5ca62, 34 },
    { 0x66d5ca62, 55 },
    { 0xcc65ca64, 98 },
    { 0xde95ca62, 90 },
    { 0x31b5ca66, 33 },
    { 0, 0 },
    { 0xca85ca68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b65ca6b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada5ca72, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635ca78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685ca7c, 61 },
    { 0xf0c5ca7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b5ca86, 98 },
    { 0x23e5ca87, 66 },
    { 0x7005ca88, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd425ca8f, 95 },
    { 0, 0 },
    { 0x1975ca91, 90 },
    { 0xe765ca91, 55 },
    { 0x3305ca92, 90 },
    { 0x8cb5ca91, 77 },
    { 0x77f5ca92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b5ca99, 23 },
    { 0, 0 },
    { 0xcd75ca9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae65ca9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c5caa5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5745caa9, 61 },
    { 0x8865caaa, 78 },
    { 0x65e5caab, 84 },
    { 0xd8c5caac, 49 },
    { 0, 0 },
    { 0x7f65caae, 64 },
    { 0, 0 },
    { 0x32e5cab0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e5cab3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f45cab6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c5caba, 90 },
    { 0, 0 },
    { 0xadc5cabc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c85cac5, 23 },
    { 0xca05cac5, 67 },
    { 0, 0 },
    { 0xc305cac8, 55 },
    { 0x9a5cac9, 78 },
    { 0x5295cac9, 33 },
    { 0x2ea5cacb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b5cad3, 43 },
    { 0x92e5cad3, 98 },
    { 0, 0 },
    { 0x99f5cad6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb15cae2, 23 },
    { 0x5575cae3, 90 },
    { 0x195cae3, 90 },
    { 0xeaf5cae5, 97 },
    { 0x7a95cae3, 93 },
    { 0x5015cae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195caea, 90 },
    { 0x19a5caea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc835caef, 61 },
    { 0x5115caef, 34 },
    { 0x6615caf0, 81 },
    { 0x2775caf1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a5cafa, 50 },
    { 0x6175cafa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb185cb00, 61 },
    { 0, 0 },
    { 0x8155cb02, 61 },
    { 0xe7e5cb03, 33 },
    { 0x9cb5cb04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3625cb08, 95 },
    { 0, 0 },
    { 0x6795cb0a, 66 },
    { 0x93d5cb0b, 98 },
    { 0, 0 },
    { 0xe05cb0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325cb14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de5cb19, 55 },
    { 0, 0 },
    { 0x7995cb1b, 23 },
    { 0xaeb5cb1b, 95 },
    { 0xa375cb1c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9135cb21, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xef65cb24, 98 },
    { 0xc865cb25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa5cb2c, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda5cb2f, 61 },
    { 0, 0 },
    { 0xe235cb31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a75cb3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa955cb3d, 52 },
    { 0x9d35cb3e, 77 },
    { 0x6cf5cb3f, 98 },
    { 0x6815cb3d, 99 },
    { 0x58b5cb41, 83 },
    { 0xf675cb3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb735cb4b, 33 },
    { 0x7675cb4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd745cb51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa5cb54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a5cb5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff5cb5e, 33 },
    { 0, 0 },
    { 0xce5cb60, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf5cb63, 90 },
    { 0xa5f5cb64, 98 },
    { 0xf455cb65, 23 },
    { 0x3a5cb66, 56 },
    { 0x7165cb67, 23 },
    { 0xf8b5cb67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7515cb6d, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25cb73, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b85cb76, 87 },
    { 0x2985cb77, 98 },
    { 0x9ae5cb76, 98 },
    { 0, 0 },
    { 0x4825cb7a, 33 },
    { 0x33b5cb7b, 78 },
    { 0x2a75cb7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e5cb7f, 94 },
    { 0xf05cb80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c5cb85, 76 },
    { 0, 0 },
    { 0x4935cb87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d5cb90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5cb94, 95 },
    { 0xd235cb94, 43 },
    { 0x9305cb94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1845cb9d, 33 },
    { 0x9965cb9e, 43 },
    { 0xad35cb9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb565cba3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c35cba6, 100 },
    { 0xb685cba7, 90 },
    { 0, 0 },
    { 0x235cba9, 90 },
    { 0x8295cbaa, 61 },
    { 0, 0 },
    { 0x5925cbac, 33 },
    { 0x34f5cbad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb45cbb0, 90 },
    { 0, 0 },
    { 0xe045cbb2, 45 },
    { 0x52e5cbb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab5cbb8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d5cbbe, 53 },
    { 0x5f5cbbf, 33 },
    { 0xa6d5cbbf, 33 },
    { 0x3e35cbbe, 84 },
    { 0xa985cbc2, 33 },
    { 0x6735cbc3, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5cbc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c5cbcc, 23 },
    { 0x8c25cbcc, 61 },
    { 0, 0 },
    { 0xb0d5cbcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8085cbd7, 98 },
    { 0xcc45cbd8, 17 },
    { 0, 0 },
    { 0xdf65cbda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65cbdd, 98 },
    { 0, 0 },
    { 0xce65cbdf, 43 },
    { 0xebc5cbe0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x225cbe3, 90 },
    { 0x8cd5cbe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad25cbe7, 55 },
    { 0x5715cbe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb65cbec, 78 },
    { 0xc765cbed, 66 },
    { 0, 0 },
    { 0x6125cbef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba5cbf2, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c5cbf8, 78 },
    { 0xfff5cbf8, 55 },
    { 0x27b5cbfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d5cbfd, 90 },
    { 0, 0 },
    { 0x23d5cbff, 78 },
    { 0, 0 },
    { 0x4305cc01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf5cc0a, 33 },
    { 0x8325cc0b, 95 },
    { 0xbb65cc0c, 55 },
    { 0, 0 },
    { 0x86a5cc0e, 61 },
    { 0, 0 },
    { 0x4455cc10, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3625cc17, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05cc1a, 61 },
    { 0x94d5cc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda55cc1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c45cc24, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b65cc28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec5cc2b, 76 },
    { 0xe9e5cc2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd5cc32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5255cc35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce05cc38, 55 },
    { 0, 0 },
    { 0x45f5cc3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc95cc3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c95cc42, 23 },
    { 0xcc5cc43, 61 },
    { 0x1335cc43, 33 },
    { 0x36c5cc44, 90 },
    { 0, 0 },
    { 0xe5cc47, 23 },
    { 0, 0 },
    { 0x4585cc49, 61 },
    { 0xa2b5cc4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c65cc56, 23 },
    { 0x4425cc57, 100 },
    { 0, 0 },
    { 0xfbc5cc59, 85 },
    { 0x8cc5cc5a, 61 },
    { 0, 0 },
    { 0x7b45cc5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75cc67, 34 },
    { 0x9065cc68, 34 },
    { 0, 0 },
    { 0x69d5cc6a, 98 },
    { 0xd8a5cc6b, 80 },
    { 0, 0 },
    { 0x7c25cc6d, 90 },
    { 0xd985cc6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2475cc72, 34 },
    { 0xb525cc73, 23 },
    { 0, 0 },
    { 0x3a05cc75, 88 },
    { 0, 0 },
    { 0x4ad5cc77, 95 },
    { 0xfde5cc77, 95 },
    { 0, 0 },
    { 0x3e5cc7a, 61 },
    { 0xc8d5cc7b, 34 },
    { 0x1385cc7a, 95 },
    { 0x1c05cc7c, 99 },
    { 0xe6b5cc7e, 98 },
    { 0xd695cc7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d75cc83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5cc86, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35cc89, 23 },
    { 0, 0 },
    { 0x3705cc8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15cc8e, 33 },
    { 0xd0a5cc8e, 23 },
    { 0, 0 },
    { 0x64d5cc91, 55 },
    { 0, 0 },
    { 0xa725cc93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac5cc9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb485cc9d, 93 },
    { 0, 0 },
    { 0xed35cc9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b5cca2, 33 },
    { 0xd9a5cca3, 95 },
    { 0xcdc5cca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85cca7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd5ccad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d65ccb3, 33 },
    { 0, 0 },
    { 0x2b35ccb5, 80 },
    { 0, 0 },
    { 0x9165ccb7, 100 },
    { 0x7575ccb8, 53 },
    { 0x5925ccb8, 34 },
    { 0x3f25ccba, 90 },
    { 0x96a5ccbb, 96 },
    { 0x2235ccbc, 66 },
    { 0xf475ccbd, 43 },
    { 0x6b15ccbe, 66 },
    { 0x3695ccbf, 23 },
    { 0x4675ccbd, 33 },
    { 0x67b5ccb8, 34 },
    { 0x2885ccbe, 55 },
    { 0, 0 },
    { 0x13c5ccc4, 76 },
    { 0xdd55ccc4, 95 },
    { 0x3315ccc5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f55ccca, 95 },
    { 0x8f45cccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d5ccce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b5ccd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad5ccd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xae35ccdb, 33 },
    { 0x22f5ccdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda45cce0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe735cce4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b5cce9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc985ccef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d75ccf4, 34 },
    { 0xcb95ccf4, 67 },
    { 0xee55ccf6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb565ccf9, 61 },
    { 0, 0 },
    { 0x12f5ccfb, 91 },
    { 0x3a45ccfc, 78 },
    { 0x3af5ccfd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c5cd00, 61 },
    { 0x4225cd00, 67 },
    { 0x69c5cd01, 61 },
    { 0xa695cd03, 98 },
    { 0xc7e5cd03, 34 },
    { 0x9f85cd04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3055cd0e, 43 },
    { 0xd2b5cd0f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725cd16, 61 },
    { 0x5d45cd17, 90 },
    { 0x9eb5cd16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f45cd20, 90 },
    { 0x3015cd21, 23 },
    { 0, 0 },
    { 0xa035cd23, 76 },
    { 0x4745cd23, 90 },
    { 0, 0 },
    { 0xc165cd26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc905cd29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d5cd30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8675cd35, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8325cd3b, 95 },
    { 0x1105cd3b, 93 },
    { 0xdf05cd3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece5cd40, 95 },
    { 0, 0 },
    { 0x5415cd42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2275cd4d, 98 },
    { 0x5cb5cd4d, 23 },
    { 0x96d5cd4e, 95 },
    { 0xaf5cd50, 76 },
    { 0xf035cd50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9975cd5b, 90 },
    { 0xbff5cd5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d55cd60, 23 },
    { 0xe3f5cd61, 90 },
    { 0x18e5cd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e05cd65, 76 },
    { 0, 0 },
    { 0xb575cd67, 33 },
    { 0, 0 },
    { 0x6965cd69, 33 },
    { 0x9e35cd6a, 75 },
    { 0, 0 },
    { 0x1e85cd6c, 61 },
    { 0x33b5cd6d, 100 },
    { 0x56f5cd6c, 23 },
    { 0x5b75cd6f, 33 },
    { 0x31b5cd70, 90 },
    { 0x3545cd70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b5cd74, 33 },
    { 0x1be5cd74, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5cd7c, 90 },
    { 0xdbb5cd7d, 87 },
    { 0x7aa5cd7e, 100 },
    { 0x4d5cd7f, 67 },
    { 0x36d5cd7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b5cd89, 95 },
    { 0, 0 },
    { 0x65e5cd8b, 98 },
    { 0, 0 },
    { 0x68a5cd8d, 90 },
    { 0xb865cd8e, 66 },
    { 0xb2c5cd8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a15cd92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c5cd95, 77 },
    { 0xd365cd96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c5cd9c, 33 },
    { 0, 0 },
    { 0x7065cd9e, 76 },
    { 0x5a85cd9e, 23 },
    { 0x2205cd9f, 33 },
    { 0, 0 },
    { 0xffa5cda2, 23 },
    { 0xcd05cda2, 34 },
    { 0x3985cda4, 95 },
    { 0x7765cda5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a5cdaa, 76 },
    { 0x3405cdaa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d5cdb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b5cdb4, 81 },
    { 0x74b5cdb4, 90 },
    { 0x86c5cdb6, 43 },
    { 0, 0 },
    { 0x7345cdb8, 98 },
    { 0x8725cdb8, 76 },
    { 0, 0 },
    { 0x8fe5cdbb, 90 },
    { 0x2685cdbb, 101 },
    { 0x465cdbd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c5cdc3, 69 },
    { 0, 0 },
    { 0xa275cdc5, 61 },
    { 0x1ed5cdc6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b5cdcd, 61 },
    { 0x2805cdce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55cdd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd5cdd9, 33 },
    { 0, 0 },
    { 0x4885cddb, 98 },
    { 0x2885cddb, 66 },
    { 0x1d55cddc, 43 },
    { 0x5125cdde, 95 },
    { 0xdfc5cdde, 84 },
    { 0xbf45cddf, 46 },
    { 0xbab5cde1, 33 },
    { 0xae35cde2, 34 },
    { 0x7f25cde3, 38 },
    { 0x5135cde3, 90 },
    { 0xe605cddc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a5cde9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f5cdec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c5cdef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2905ce03, 23 },
    { 0xebf5ce04, 52 },
    { 0, 0 },
    { 0xddf5ce06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b5ce0e, 23 },
    { 0x26b5ce0e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c5ce12, 90 },
    { 0, 0 },
    { 0xdaa5ce14, 34 },
    { 0xada5ce15, 55 },
    { 0, 0 },
    { 0x3045ce17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7385ce1c, 23 },
    { 0x1635ce1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f65ce22, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695ce28, 98 },
    { 0x67d5ce28, 34 },
    { 0, 0 },
    { 0x9c55ce2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb85ce31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf75ce34, 95 },
    { 0x33f5ce35, 86 },
    { 0x7a35ce36, 95 },
    { 0x6b15ce37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f5ce3a, 33 },
    { 0x4d15ce3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc825ce42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe5ce4e, 33 },
    { 0xc295ce4f, 93 },
    { 0x3dd5ce50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f15ce53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf5ce59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5ce5f, 61 },
    { 0xb1e5ce60, 93 },
    { 0xcf75ce61, 88 },
    { 0xacd5ce62, 90 },
    { 0x3a95ce62, 95 },
    { 0, 0 },
    { 0x8c95ce65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d5ce71, 33 },
    { 0, 0 },
    { 0xd065ce73, 52 },
    { 0xadd5ce74, 95 },
    { 0x74f5ce75, 33 },
    { 0x75d5ce76, 55 },
    { 0, 0 },
    { 0x1145ce78, 46 },
    { 0xb175ce78, 91 },
    { 0x1fe5ce78, 61 },
    { 0x7025ce7b, 61 },
    { 0x8525ce7c, 97 },
    { 0xc7f5ce7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e85ce82, 33 },
    { 0, 0 },
    { 0x6cd5ce84, 95 },
    { 0, 0 },
    { 0x6e25ce86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc85ce8b, 85 },
    { 0x8c45ce8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a5ce8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8105ce92, 68 },
    { 0x82f5ce92, 55 },
    { 0, 0 },
    { 0x1ce5ce95, 23 },
    { 0x2ff5ce95, 17 },
    { 0x4cf5ce97, 61 },
    { 0xb55ce98, 67 },
    { 0x9fe5ce96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9065ce9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xde65cea1, 61 },
    { 0, 0 },
    { 0xca35cea3, 34 },
    { 0, 0 },
    { 0xc6f5cea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9285ceac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5ceb1, 98 },
    { 0x3925ceb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4265ceb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace5cebb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1595cebf, 61 },
    { 0, 0 },
    { 0x455cec1, 90 },
    { 0, 0 },
    { 0xde05cec3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1585cec6, 36 },
    { 0x9c05cec7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd5cecb, 94 },
    { 0xec95cecc, 33 },
    { 0xcb35cecb, 90 },
    { 0, 0 },
    { 0x8765cecf, 98 },
    { 0x64a5ced0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5ced8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd85cedb, 43 },
    { 0x99d5cedc, 23 },
    { 0xaa75cedb, 90 },
    { 0, 0 },
    { 0xd695cedf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f95cee3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae5ceeb, 90 },
    { 0xf535ceec, 33 },
    { 0, 0 },
    { 0x2e75ceee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7075cef2, 75 },
    { 0, 0 },
    { 0x4795cef4, 94 },
    { 0, 0 },
    { 0x415cef6, 33 },
    { 0xa5a5cef7, 90 },
    { 0xf0c5cef8, 43 },
    { 0xd8f5cef9, 76 },
    { 0x85d5cefa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f75cefe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8525cf03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7665cf07, 55 },
    { 0x5f45cf08, 66 },
    { 0xb335cf09, 84 },
    { 0x70e5cf0a, 90 },
    { 0x7cb5cf08, 98 },
    { 0xdb95cf08, 61 },
    { 0x5005cf09, 87 },
    { 0x86b5cf0e, 87 },
    { 0xf425cf0f, 95 },
    { 0xf05cf10, 91 },
    { 0xac05cf0f, 61 },
    { 0x7205cf10, 95 },
    { 0, 0 },
    { 0xc3a5cf14, 98 },
    { 0xd145cf15, 90 },
    { 0xaa05cf14, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5cf1f, 95 },
    { 0x185cf20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe875cf25, 23 },
    { 0x1ed5cf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd635cf2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc5cf31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e5cf36, 90 },
    { 0xbd15cf36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b5cf3b, 55 },
    { 0xa2c5cf3b, 23 },
    { 0xb1c5cf3d, 55 },
    { 0x64a5cf3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f15cf4e, 98 },
    { 0xa255cf4f, 66 },
    { 0x2345cf50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a5cf58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac15cf5b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9985cf62, 90 },
    { 0x5845cf63, 33 },
    { 0, 0 },
    { 0x3035cf65, 23 },
    { 0x35a5cf66, 66 },
    { 0xd975cf67, 90 },
    { 0xc615cf67, 95 },
    { 0x6bd5cf65, 98 },
    { 0xe615cf68, 33 },
    { 0x52c5cf68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4385cf6e, 78 },
    { 0xba85cf6e, 34 },
    { 0, 0 },
    { 0x4a85cf71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945cf78, 46 },
    { 0x9d95cf79, 64 },
    { 0, 0 },
    { 0x93d5cf7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d5cf7e, 98 },
    { 0, 0 },
    { 0xa9e5cf80, 90 },
    { 0xfe55cf81, 95 },
    { 0xad55cf80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe235cf87, 33 },
    { 0x3045cf88, 36 },
    { 0, 0 },
    { 0xe4b5cf8a, 95 },
    { 0xf525cf8a, 81 },
    { 0x6fe5cf8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e95cf98, 61 },
    { 0x6c35cf99, 96 },
    { 0x4c45cf9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5cf9e, 33 },
    { 0xf3a5cf9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5125cfa2, 84 },
    { 0xb3d5cfa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb665cfa6, 94 },
    { 0x5795cfa7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2395cfae, 90 },
    { 0, 0 },
    { 0xa6b5cfb0, 99 },
    { 0x6fc5cfb1, 68 },
    { 0xf255cfb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5cfb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe5cfba, 96 },
    { 0xd405cfbb, 93 },
    { 0x8815cfba, 90 },
    { 0, 0 },
    { 0xda85cfbe, 23 },
    { 0x4cb5cfbf, 90 },
    { 0x1bf5cfbf, 67 },
    { 0xd585cfc1, 90 },
    { 0x4845cfc2, 99 },
    { 0x7c85cfc2, 76 },
    { 0x5d05cfc3, 98 },
    { 0xf425cfc5, 87 },
    { 0x2265cfc2, 61 },
    { 0xfd75cfc7, 69 },
    { 0x84d5cfbf, 33 },
    { 0xb665cfc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd075cfcc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x435cfcf, 87 },
    { 0x6965cfcf, 69 },
    { 0x9145cfcf, 90 },
    { 0x6515cfd1, 33 },
    { 0, 0 },
    { 0x13b5cfd4, 84 },
    { 0, 0 },
    { 0xc9e5cfd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd85cfda, 23 },
    { 0xd575cfda, 90 },
    { 0x9795cfdc, 93 },
    { 0x2b05cfdc, 43 },
    { 0xf2e5cfdd, 33 },
    { 0xb285cfdf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6315cfe6, 46 },
    { 0xfa55cfe7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6245cfed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb15cff0, 90 },
    { 0xd465cff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d5cff8, 76 },
    { 0, 0 },
    { 0xd325cffa, 33 },
    { 0, 0 },
    { 0x5d35cffc, 85 },
    { 0, 0 },
    { 0x1995cffe, 33 },
    { 0xe3a5cfff, 61 },
    { 0, 0 },
    { 0xc805d001, 90 },
    { 0x39c5d002, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a85d00a, 98 },
    { 0, 0 },
    { 0x1355d00c, 23 },
    { 0x8e35d00d, 88 },
    { 0x73d5d00e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7515d012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435d019, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c5d01f, 95 },
    { 0, 0 },
    { 0x9e25d021, 55 },
    { 0xdf45d021, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d5d026, 52 },
    { 0xa345d026, 55 },
    { 0x8e75d028, 98 },
    { 0x2ba5d029, 90 },
    { 0x5295d029, 66 },
    { 0, 0 },
    { 0x1ec5d02c, 55 },
    { 0, 0 },
    { 0x7805d02e, 98 },
    { 0x2205d02f, 98 },
    { 0xc315d030, 43 },
    { 0x9ac5d031, 90 },
    { 0xcc45d032, 34 },
    { 0x4d55d033, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a25d03b, 67 },
    { 0x6c25d03b, 69 },
    { 0, 0 },
    { 0x8415d03e, 90 },
    { 0x9a35d03f, 95 },
    { 0x9bc5d03e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935d048, 61 },
    { 0x1eb5d048, 43 },
    { 0, 0 },
    { 0xf065d04b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba55d04e, 52 },
    { 0x4b25d04f, 61 },
    { 0xccd5d050, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605d053, 55 },
    { 0x5045d054, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5d063, 66 },
    { 0xe1d5d063, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb5d06e, 33 },
    { 0xddc5d06e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6755d073, 34 },
    { 0, 0 },
    { 0x6e35d075, 34 },
    { 0x5b15d075, 76 },
    { 0xf395d077, 33 },
    { 0, 0 },
    { 0xc505d079, 55 },
    { 0x915d079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e65d07e, 33 },
    { 0x2b55d07f, 90 },
    { 0, 0 },
    { 0x8be5d081, 93 },
    { 0, 0 },
    { 0x3a35d083, 23 },
    { 0xb505d084, 90 },
    { 0xe6d5d083, 85 },
    { 0x9195d085, 90 },
    { 0xec55d086, 55 },
    { 0x9685d088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1215d08b, 93 },
    { 0, 0 },
    { 0x7185d08d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d5d098, 61 },
    { 0x8375d099, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5d09c, 87 },
    { 0x32a5d09d, 90 },
    { 0x8035d09c, 87 },
    { 0x35e5d09f, 55 },
    { 0x9085d0a0, 61 },
    { 0x95f5d09c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa985d0aa, 78 },
    { 0x6345d0aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc5d0ae, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xde65d0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35d0b6, 61 },
    { 0xdcf5d0b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e5d0bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c75d0c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b85d0c6, 33 },
    { 0xb3a5d0c7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f5d0cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e5d0d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1405d0d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d5d0d9, 100 },
    { 0xa015d0da, 61 },
    { 0x2d75d0da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a05d0de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa255d0e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5325d0e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d5d0ea, 40 },
    { 0xd935d0eb, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xec25d0ee, 90 },
    { 0x675d0ee, 23 },
    { 0xe475d0ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe765d0f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f65d0fc, 78 },
    { 0, 0 },
    { 0xb95d0fe, 61 },
    { 0x6a65d0ff, 90 },
    { 0x5925d0ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4325d106, 61 },
    { 0, 0 },
    { 0xac95d108, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e5d10d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5d115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea5d11a, 46 },
    { 0x3fd5d11b, 76 },
    { 0, 0 },
    { 0x3a05d11d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe525d121, 85 },
    { 0xc6e5d121, 61 },
    { 0, 0 },
    { 0xbb45d124, 100 },
    { 0x4f45d125, 66 },
    { 0x74d5d126, 46 },
    { 0x8945d126, 34 },
    { 0x3c25d128, 23 },
    { 0x7795d129, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c5d12c, 33 },
    { 0xc7b5d12d, 33 },
    { 0xc3a5d12c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d135, 23 },
    { 0x1fb5d135, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2255d13a, 61 },
    { 0, 0 },
    { 0xe05d13c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c5d140, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1515d143, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f55d148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695d14d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa515d153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415d159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b75d15c, 93 },
    { 0xfa95d15c, 33 },
    { 0x1c15d15e, 66 },
    { 0xb265d15c, 76 },
    { 0, 0 },
    { 0x1185d161, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb395d16b, 55 },
    { 0, 0 },
    { 0xc625d16d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5d173, 55 },
    { 0x155d173, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa235d179, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc55d17c, 68 },
    { 0x5005d17c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb5d182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef5d188, 34 },
    { 0x1235d188, 33 },
    { 0x9075d18a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1855d18d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5d195, 33 },
    { 0, 0 },
    { 0x9635d197, 95 },
    { 0x3c05d198, 33 },
    { 0xffc5d199, 33 },
    { 0x1635d197, 78 },
    { 0x2a35d19a, 98 },
    { 0xeb25d199, 90 },
    { 0, 0 },
    { 0x26c5d19e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7315d1a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d1a7, 33 },
    { 0x84a5d1a7, 96 },
    { 0x4515d1a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a5d1af, 90 },
    { 0, 0 },
    { 0x7a5d1b1, 91 },
    { 0xbba5d1b1, 75 },
    { 0xc045d1b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c5d1b6, 78 },
    { 0x4135d1b7, 55 },
    { 0x1d45d1b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa545d1bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5d1c1, 91 },
    { 0, 0 },
    { 0x2235d1c3, 43 },
    { 0, 0 },
    { 0x7f35d1c5, 98 },
    { 0xb4a5d1c6, 95 },
    { 0xac45d1c7, 95 },
    { 0x8175d1c7, 98 },
    { 0x1895d1c8, 90 },
    { 0x3f35d1c5, 34 },
    { 0x1525d1cb, 91 },
    { 0, 0 },
    { 0xea25d1cd, 95 },
    { 0xa65d1cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6355d1da, 66 },
    { 0x55d5d1db, 95 },
    { 0x8565d1dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7805d1e0, 36 },
    { 0x5f75d1e0, 98 },
    { 0x14f5d1e0, 81 },
    { 0x2a55d1e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe795d1e7, 23 },
    { 0, 0 },
    { 0xfec5d1e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5d1ec, 61 },
    { 0x31c5d1ed, 91 },
    { 0, 0 },
    { 0xf5c5d1ef, 95 },
    { 0xc465d1f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e5d1f3, 59 },
    { 0xafb5d1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc5d1f7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb445d1fc, 66 },
    { 0xae55d1fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf75d200, 43 },
    { 0xc405d200, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b75d204, 23 },
    { 0x7de5d204, 61 },
    { 0xc7c5d204, 93 },
    { 0, 0 },
    { 0xa775d208, 61 },
    { 0xa25d209, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8295d212, 66 },
    { 0x62e5d212, 80 },
    { 0x64e5d214, 61 },
    { 0xab55d214, 33 },
    { 0xd655d214, 95 },
    { 0xb2f5d215, 87 },
    { 0, 0 },
    { 0x8b85d219, 33 },
    { 0, 0 },
    { 0x7275d21b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d5d223, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xac85d226, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5285d229, 23 },
    { 0x3be5d22a, 52 },
    { 0x4605d22b, 61 },
    { 0xdd5d22c, 98 },
    { 0xbd95d22c, 47 },
    { 0, 0 },
    { 0x8235d22f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa5d233, 61 },
    { 0x5d234, 90 },
    { 0xdaf5d235, 61 },
    { 0x7a35d233, 43 },
    { 0, 0 },
    { 0x475d238, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd895d23b, 83 },
    { 0xfa65d23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd335d241, 90 },
    { 0, 0 },
    { 0xe2d5d243, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef85d253, 61 },
    { 0xf3d5d254, 61 },
    { 0, 0 },
    { 0x31e5d256, 95 },
    { 0x5e15d257, 87 },
    { 0, 0 },
    { 0x6f5d259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e5d25d, 85 },
    { 0x6a55d25e, 93 },
    { 0, 0 },
    { 0xdfa5d260, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb15d269, 95 },
    { 0x6c45d26a, 51 },
    { 0, 0 },
    { 0xc645d26c, 33 },
    { 0xc515d26d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5095d271, 61 },
    { 0xa485d272, 33 },
    { 0, 0 },
    { 0x1d65d274, 91 },
    { 0x5c75d275, 33 },
    { 0, 0 },
    { 0xa465d277, 90 },
    { 0xd4a5d277, 90 },
    { 0, 0 },
    { 0x63a5d27a, 43 },
    { 0, 0 },
    { 0x75b5d27c, 90 },
    { 0x4e65d27c, 84 },
    { 0x84a5d27c, 99 },
    { 0x58a5d27c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc15d282, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5d28b, 76 },
    { 0xef75d28b, 76 },
    { 0, 0 },
    { 0x9165d28e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf945d291, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb5d299, 43 },
    { 0xc375d29a, 90 },
    { 0xdcb5d29b, 95 },
    { 0x86b5d29c, 61 },
    { 0x1425d29d, 90 },
    { 0xb2e5d29e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a05d2a3, 61 },
    { 0, 0 },
    { 0x9325d2a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd45d2a8, 43 },
    { 0, 0 },
    { 0x7f05d2aa, 34 },
    { 0x6e75d2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6945d2b0, 61 },
    { 0xa765d2b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c15d2b5, 95 },
    { 0xdf75d2b6, 90 },
    { 0, 0 },
    { 0xd75d2b8, 76 },
    { 0xeaa5d2b9, 68 },
    { 0xcd35d2b8, 53 },
    { 0x58b5d2bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f85d2c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8315d2c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca45d2ca, 97 },
    { 0x4155d2cb, 61 },
    { 0, 0 },
    { 0x755d2cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4495d2d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd35d2d8, 85 },
    { 0x61c5d2d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d5d2dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3765d2e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd5d2e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd635d2e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b75d2ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325d2f0, 33 },
    { 0, 0 },
    { 0xaaf5d2f2, 95 },
    { 0x38f5d2f3, 90 },
    { 0x8de5d2f4, 33 },
    { 0xa835d2f3, 67 },
    { 0x65b5d2f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e5d2fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8105d2fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb5d302, 85 },
    { 0x6f25d303, 23 },
    { 0x1005d302, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae5d308, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25d30c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef5d30f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc185d314, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9705d317, 66 },
    { 0x2bb5d318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5225d320, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f5d323, 68 },
    { 0, 0 },
    { 0x7125d325, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7615d328, 91 },
    { 0, 0 },
    { 0x5595d32a, 84 },
    { 0xf475d32a, 33 },
    { 0, 0 },
    { 0x1ab5d32d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2665d332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245d335, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe185d341, 98 },
    { 0x7645d342, 90 },
    { 0x3555d343, 33 },
    { 0xdb75d344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7405d349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6455d351, 90 },
    { 0xd555d352, 61 },
    { 0xef65d353, 98 },
    { 0x92c5d354, 85 },
    { 0x1d55d353, 64 },
    { 0x255d356, 90 },
    { 0x72a5d356, 33 },
    { 0x1365d358, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d35c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5d363, 90 },
    { 0xbf85d363, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da5d36b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc035d371, 90 },
    { 0, 0 },
    { 0xbd05d373, 55 },
    { 0x3d35d374, 56 },
    { 0x265d375, 33 },
    { 0x8925d376, 90 },
    { 0xde05d377, 76 },
    { 0x3a95d378, 23 },
    { 0x4e05d373, 90 },
    { 0xa985d379, 55 },
    { 0x7e95d374, 61 },
    { 0, 0 },
    { 0xc535d37d, 52 },
    { 0, 0 },
    { 0xd505d37f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5905d383, 34 },
    { 0xce95d383, 61 },
    { 0x10e5d385, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd5d389, 33 },
    { 0, 0 },
    { 0x9f75d38b, 34 },
    { 0x205d38c, 91 },
    { 0, 0 },
    { 0x8655d38e, 40 },
    { 0x42b5d38f, 98 },
    { 0xeab5d390, 55 },
    { 0x3f45d391, 98 },
    { 0x1445d392, 55 },
    { 0x4575d38f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e5d398, 100 },
    { 0x54b5d399, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa5d3a0, 46 },
    { 0x2bd5d3a0, 90 },
    { 0xb165d3a2, 33 },
    { 0xdde5d3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b5d3aa, 61 },
    { 0x4b25d3ab, 95 },
    { 0xeb45d3ac, 93 },
    { 0xd5f5d3ad, 95 },
    { 0x655d3ac, 90 },
    { 0x29f5d3ad, 33 },
    { 0x57a5d3b0, 33 },
    { 0xfe95d3b1, 93 },
    { 0xa2d5d3b2, 90 },
    { 0x7775d3b3, 61 },
    { 0x7665d3b2, 23 },
    { 0xe795d3b4, 43 },
    { 0x825d3b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d3bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7605d3c0, 84 },
    { 0x2945d3c1, 93 },
    { 0xb635d3c2, 23 },
    { 0x18e5d3c3, 95 },
    { 0x6a75d3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1175d3c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f5d3cd, 61 },
    { 0, 0 },
    { 0x4a15d3cf, 76 },
    { 0x5f85d3d0, 87 },
    { 0xcaa5d3cf, 90 },
    { 0, 0 },
    { 0xadd5d3d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e5d3d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce45d3da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d5d3de, 90 },
    { 0xbaa5d3df, 55 },
    { 0x48c5d3e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5545d3e4, 34 },
    { 0x7705d3e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd95d3e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf35d3eb, 75 },
    { 0x5f85d3ec, 90 },
    { 0xd1f5d3ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda75d3f0, 33 },
    { 0, 0 },
    { 0x8b45d3f2, 33 },
    { 0x125d3f3, 90 },
    { 0xdf45d3f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d65d3f8, 23 },
    { 0x1675d3f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e5d3fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4215d400, 33 },
    { 0x6a45d401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c5d40a, 55 },
    { 0, 0 },
    { 0x5255d40c, 95 },
    { 0, 0 },
    { 0x9b15d40e, 98 },
    { 0xe55d40e, 78 },
    { 0xd205d410, 65 },
    { 0x6ee5d411, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e5d417, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc5d420, 33 },
    { 0x4d15d421, 98 },
    { 0x11f5d421, 75 },
    { 0xb1c5d423, 23 },
    { 0x70c5d424, 85 },
    { 0, 0 },
    { 0x4b35d426, 85 },
    { 0x5da5d427, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7265d42a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd35d42e, 17 },
    { 0xb6a5d42f, 43 },
    { 0x8ff5d430, 93 },
    { 0x1935d431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5d43e, 55 },
    { 0x9d95d43f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde35d444, 34 },
    { 0x27f5d445, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b85d44c, 90 },
    { 0, 0 },
    { 0x2a25d44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5415d451, 23 },
    { 0x3d5d452, 66 },
    { 0x57c5d453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3545d459, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1825d461, 90 },
    { 0x6a35d461, 43 },
    { 0x3385d463, 93 },
    { 0, 0 },
    { 0x1a65d465, 95 },
    { 0x29d5d465, 46 },
    { 0x58d5d466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe825d46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05d472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe625d477, 95 },
    { 0xc1e5d478, 67 },
    { 0x9d15d478, 93 },
    { 0, 0 },
    { 0x1685d47b, 33 },
    { 0x7145d47c, 23 },
    { 0x6e5d47c, 90 },
    { 0x8955d47d, 95 },
    { 0, 0 },
    { 0xfe75d480, 90 },
    { 0xe835d481, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd235d489, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065d48e, 55 },
    { 0x64f5d48f, 43 },
    { 0x16d5d490, 33 },
    { 0, 0 },
    { 0xce85d492, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a5d49b, 95 },
    { 0x915d49c, 92 },
    { 0x4745d49d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b95d4a1, 76 },
    { 0xc365d4a2, 33 },
    { 0x14d5d4a2, 30 },
    { 0xcdb5d4a4, 33 },
    { 0x1495d4a4, 33 },
    { 0x1b85d4a4, 34 },
    { 0x9715d4a7, 90 },
    { 0x4085d4a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5d4ab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d4ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c5d4b2, 34 },
    { 0x6d95d4b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3585d4b6, 100 },
    { 0xe8c5d4b6, 55 },
    { 0xb4d5d4b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8925d4bb, 61 },
    { 0x35a5d4bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e5d4c2, 67 },
    { 0x76d5d4c3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5625d4cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7285d4cf, 55 },
    { 0x3985d4d0, 78 },
    { 0x3555d4d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb265d4d6, 77 },
    { 0x7fb5d4d7, 78 },
    { 0x76a5d4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb05d4dc, 76 },
    { 0x6e35d4dd, 23 },
    { 0x5495d4de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb735d4e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e5d4e6, 76 },
    { 0x125d4e7, 90 },
    { 0xac75d4e8, 88 },
    { 0xc275d4e7, 90 },
    { 0xa3c5d4ea, 46 },
    { 0, 0 },
    { 0xf395d4ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e5d4ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf825d4f7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5d4fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc25d4ff, 77 },
    { 0, 0 },
    { 0x76a5d501, 95 },
    { 0x74e5d502, 99 },
    { 0xe635d503, 98 },
    { 0xe825d502, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f25d507, 76 },
    { 0x4f75d507, 91 },
    { 0x9935d507, 61 },
    { 0x5c75d50a, 33 },
    { 0xe645d50b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6855d512, 90 },
    { 0x1285d513, 34 },
    { 0x24b5d514, 69 },
    { 0, 0 },
    { 0x4605d516, 76 },
    { 0xf45d517, 33 },
    { 0, 0 },
    { 0x55d5d519, 55 },
    { 0, 0 },
    { 0x6ab5d51b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb665d51e, 93 },
    { 0, 0 },
    { 0x5795d520, 84 },
    { 0x60e5d521, 61 },
    { 0x6575d522, 43 },
    { 0x27c5d521, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f05d527, 52 },
    { 0x725d528, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad5d52b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7465d52e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6285d531, 91 },
    { 0x6195d531, 55 },
    { 0x4f35d533, 34 },
    { 0, 0 },
    { 0xac85d535, 90 },
    { 0x6795d535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9735d53b, 76 },
    { 0, 0 },
    { 0xe2b5d53d, 61 },
    { 0xde75d53e, 81 },
    { 0x69c5d53e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e55d548, 51 },
    { 0xf4e5d549, 43 },
    { 0xbd75d548, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15d551, 23 },
    { 0x92e5d552, 23 },
    { 0, 0 },
    { 0xfcb5d554, 95 },
    { 0x30b5d555, 91 },
    { 0, 0 },
    { 0xc0f5d557, 66 },
    { 0x69c5d557, 33 },
    { 0, 0 },
    { 0x4eb5d55a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b85d55d, 23 },
    { 0, 0 },
    { 0xbad5d55f, 78 },
    { 0, 0 },
    { 0x3145d561, 66 },
    { 0xc145d561, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd5d565, 52 },
    { 0xf845d566, 91 },
    { 0x2225d566, 83 },
    { 0x9295d567, 90 },
    { 0x1ce5d569, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5d56f, 91 },
    { 0xea55d570, 78 },
    { 0xb1f5d56f, 55 },
    { 0x5a05d572, 91 },
    { 0x1995d571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d5d576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2205d57a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9415d57e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a5d584, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc465d588, 43 },
    { 0xfae5d588, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa65d58c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f95d591, 40 },
    { 0, 0 },
    { 0x325d593, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35d59a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3175d59e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e45d5a3, 95 },
    { 0, 0 },
    { 0xae15d5a5, 23 },
    { 0xda5d5a5, 76 },
    { 0x25a5d5a7, 65 },
    { 0xec55d5a7, 84 },
    { 0xc4c5d5a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca5d5af, 90 },
    { 0xb905d5b0, 93 },
    { 0x26e5d5b0, 90 },
    { 0x78c5d5b1, 33 },
    { 0x2d55d5b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a5d5b6, 23 },
    { 0x2335d5b7, 55 },
    { 0x805d5b8, 43 },
    { 0x6db5d5b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a15d5c4, 52 },
    { 0xa85d5c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3015d5cb, 87 },
    { 0x37d5d5cc, 77 },
    { 0x4b55d5cc, 33 },
    { 0x20b5d5cd, 34 },
    { 0, 0 },
    { 0xe985d5d0, 33 },
    { 0x3165d5d1, 23 },
    { 0x8155d5d0, 81 },
    { 0x625d5d3, 46 },
    { 0x4285d5d1, 81 },
    { 0, 0 },
    { 0x5e85d5d6, 78 },
    { 0x2d5d5d7, 23 },
    { 0x18e5d5d8, 91 },
    { 0x1955d5d9, 34 },
    { 0x6795d5da, 69 },
    { 0x7cb5d5db, 66 },
    { 0x4b55d5da, 33 },
    { 0x52c5d5db, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a35d5e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9945d5ec, 99 },
    { 0xbff5d5ed, 46 },
    { 0x9935d5ee, 88 },
    { 0x2485d5ec, 34 },
    { 0, 0 },
    { 0xa0e5d5f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a5d5f8, 55 },
    { 0, 0 },
    { 0x9bb5d5fa, 90 },
    { 0x8d5d5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa135d600, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85d60c, 61 },
    { 0xf585d60d, 98 },
    { 0x82d5d60e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2535d612, 81 },
    { 0xd275d613, 87 },
    { 0x77a5d614, 23 },
    { 0xabc5d615, 55 },
    { 0x98b5d615, 98 },
    { 0x4f45d613, 95 },
    { 0x2845d618, 33 },
    { 0x6b15d614, 84 },
    { 0x5cf5d613, 84 },
    { 0x43a5d61b, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc795d620, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f5d628, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca05d62d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1375d630, 95 },
    { 0x8095d631, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b5d634, 85 },
    { 0x35d635, 90 },
    { 0xca05d636, 61 },
    { 0x1505d634, 95 },
    { 0xa705d636, 61 },
    { 0x6a55d639, 90 },
    { 0xa665d639, 90 },
    { 0, 0 },
    { 0x7765d63c, 90 },
    { 0, 0 },
    { 0x9de5d63e, 33 },
    { 0, 0 },
    { 0x3075d640, 33 },
    { 0, 0 },
    { 0x78b5d642, 87 },
    { 0xcf55d643, 66 },
    { 0x8a75d644, 61 },
    { 0x1a95d645, 33 },
    { 0x9965d646, 95 },
    { 0xda55d647, 61 },
    { 0, 0 },
    { 0x9a55d649, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc5d654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a5d657, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c45d65b, 67 },
    { 0, 0 },
    { 0x97f5d65d, 76 },
    { 0, 0 },
    { 0x6bc5d65f, 76 },
    { 0xaa25d660, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d5d669, 33 },
    { 0xef45d66a, 33 },
    { 0, 0 },
    { 0xd565d66c, 87 },
    { 0x6d05d66d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2025d670, 95 },
    { 0x7ee5d671, 88 },
    { 0x6e45d670, 90 },
    { 0x2ed5d673, 88 },
    { 0, 0 },
    { 0x7a55d675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae5d678, 90 },
    { 0x7615d679, 23 },
    { 0x10c5d679, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5335d67e, 98 },
    { 0x4e05d67f, 55 },
    { 0x665d680, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba45d68e, 99 },
    { 0x5585d68e, 33 },
    { 0x7385d690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7605d694, 95 },
    { 0xe175d695, 38 },
    { 0x17d5d695, 78 },
    { 0x4035d694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c5d6a3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b25d6a7, 33 },
    { 0xf1d5d6a7, 77 },
    { 0xc1a5d6a7, 98 },
    { 0x78a5d6a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae5d6ad, 23 },
    { 0xac85d6ae, 23 },
    { 0x7d05d6ad, 46 },
    { 0, 0 },
    { 0x14c5d6b1, 52 },
    { 0x94a5d6b1, 52 },
    { 0xb2d5d6b3, 98 },
    { 0xc935d6b3, 33 },
    { 0x3115d6b5, 80 },
    { 0, 0 },
    { 0x2305d6b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2535d6ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a5d6c0, 23 },
    { 0x5885d6c1, 34 },
    { 0, 0 },
    { 0x1845d6c3, 61 },
    { 0, 0 },
    { 0xb915d6c5, 90 },
    { 0x49b5d6c6, 46 },
    { 0xef5d6c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a75d6cb, 90 },
    { 0xac45d6cc, 75 },
    { 0, 0 },
    { 0x40a5d6ce, 33 },
    { 0x9d95d6cf, 46 },
    { 0x4a55d6d0, 55 },
    { 0x7a95d6cf, 33 },
    { 0x3375d6d0, 90 },
    { 0xebc5d6d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf165d6d6, 61 },
    { 0x2935d6d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c5d6e0, 33 },
    { 0x1f45d6e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf875d6e6, 23 },
    { 0, 0 },
    { 0x8c55d6e8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5d6ee, 43 },
    { 0x7005d6ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a95d6f2, 55 },
    { 0, 0 },
    { 0x2285d6f4, 23 },
    { 0x3115d6f4, 76 },
    { 0x8825d6f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325d6f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c5d6fe, 33 },
    { 0, 0 },
    { 0x1275d700, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1905d705, 90 },
    { 0, 0 },
    { 0x7095d707, 90 },
    { 0, 0 },
    { 0x5995d709, 88 },
    { 0x3765d70a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6105d714, 23 },
    { 0, 0 },
    { 0x9f25d716, 90 },
    { 0xff05d716, 98 },
    { 0xc4e5d718, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3675d71e, 46 },
    { 0xa925d71e, 43 },
    { 0xdb5d720, 98 },
    { 0xeca5d720, 46 },
    { 0, 0 },
    { 0xb185d723, 23 },
    { 0, 0 },
    { 0xf3e5d725, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a45d728, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b5d72b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd5d72f, 90 },
    { 0, 0 },
    { 0xfd45d731, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5d734, 90 },
    { 0, 0 },
    { 0xc675d736, 90 },
    { 0x88c5d737, 55 },
    { 0xb955d738, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d73c, 33 },
    { 0xfcb5d73d, 76 },
    { 0, 0 },
    { 0xf2e5d73f, 95 },
    { 0x53a5d73f, 67 },
    { 0, 0 },
    { 0xcad5d742, 90 },
    { 0x3ac5d742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca35d746, 95 },
    { 0x6d65d747, 90 },
    { 0x1335d747, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7865d74d, 33 },
    { 0x8e45d74d, 66 },
    { 0, 0 },
    { 0x9555d750, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa5d753, 95 },
    { 0xddb5d753, 98 },
    { 0, 0 },
    { 0x27a5d756, 95 },
    { 0x9d35d756, 40 },
    { 0, 0 },
    { 0x5365d759, 77 },
    { 0x6be5d75a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f5d75e, 61 },
    { 0x17d5d75e, 95 },
    { 0x33a5d75f, 61 },
    { 0x8815d75e, 90 },
    { 0, 0 },
    { 0x9245d763, 90 },
    { 0xf985d764, 98 },
    { 0, 0 },
    { 0xce25d766, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca5d76b, 95 },
    { 0xf575d76c, 33 },
    { 0x92c5d76d, 55 },
    { 0, 0 },
    { 0x2065d76f, 61 },
    { 0x2a85d770, 90 },
    { 0xbfa5d771, 40 },
    { 0, 0 },
    { 0x3695d773, 40 },
    { 0xaa75d774, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe5d780, 99 },
    { 0, 0 },
    { 0x3455d782, 91 },
    { 0x4805d783, 84 },
    { 0, 0 },
    { 0xf9c5d785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c85d78c, 66 },
    { 0x9015d78d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d5d791, 99 },
    { 0, 0 },
    { 0x5c55d793, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3405d799, 65 },
    { 0x85f5d79a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65d79e, 33 },
    { 0x1a45d79f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7015d7a5, 55 },
    { 0x45d7a6, 43 },
    { 0x8b05d7a5, 98 },
    { 0xb325d7a8, 78 },
    { 0xe985d7a9, 58 },
    { 0x36e5d7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc35d7b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a5d7b4, 33 },
    { 0xccd5d7b5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb315d7bc, 100 },
    { 0xbfe5d7bd, 55 },
    { 0x9445d7bc, 55 },
    { 0x6525d7bf, 93 },
    { 0x7675d7be, 75 },
    { 0x455d7c1, 76 },
    { 0xfc25d7c2, 98 },
    { 0xc005d7c1, 23 },
    { 0x3975d7c3, 76 },
    { 0x3135d7c1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a35d7c8, 98 },
    { 0x2185d7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5735d7cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f05d7d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685d7d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b5d7d7, 33 },
    { 0, 0 },
    { 0xc1c5d7d9, 90 },
    { 0x8df5d7da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd335d7e0, 43 },
    { 0x6175d7e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5535d7ec, 95 },
    { 0x68e5d7ed, 95 },
    { 0xf315d7ed, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xd765d7f1, 23 },
    { 0x6395d7f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d5d7fa, 33 },
    { 0x8065d7fb, 23 },
    { 0xf645d7fb, 76 },
    { 0x16b5d7fc, 33 },
    { 0x7f45d7fa, 100 },
    { 0xf415d7ff, 98 },
    { 0xb35d800, 95 },
    { 0x5155d800, 91 },
    { 0xb2e5d802, 55 },
    { 0xf4a5d802, 61 },
    { 0xb6d5d804, 61 },
    { 0xaf75d805, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e5d808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d80b, 95 },
    { 0x26a5d80c, 33 },
    { 0, 0 },
    { 0x22b5d80e, 61 },
    { 0xe225d80f, 95 },
    { 0, 0 },
    { 0xae45d811, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe5d81b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d5d81e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5d821, 90 },
    { 0, 0 },
    { 0x6e35d823, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1455d830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be5d83b, 23 },
    { 0, 0 },
    { 0x7c15d83d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6715d846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca5d849, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c5d84e, 78 },
    { 0xbdf5d84e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec5d852, 76 },
    { 0, 0 },
    { 0x1a75d854, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5d858, 33 },
    { 0x9295d859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f5d85c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d5d861, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8095d864, 61 },
    { 0x5f55d864, 76 },
    { 0, 0 },
    { 0xc535d867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4845d86c, 84 },
    { 0x9495d86c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3495d871, 90 },
    { 0x95b5d871, 55 },
    { 0xda65d871, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f5d878, 61 },
    { 0x3b25d879, 95 },
    { 0, 0 },
    { 0x4a05d87b, 64 },
    { 0x7495d87b, 76 },
    { 0xa8f5d87d, 90 },
    { 0x7665d87e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef5d881, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9255d885, 61 },
    { 0x11d5d886, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a95d88b, 33 },
    { 0xa035d88c, 61 },
    { 0, 0 },
    { 0xa805d88e, 84 },
    { 0x8225d88f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9805d892, 33 },
    { 0xa4c5d893, 61 },
    { 0x6775d894, 93 },
    { 0x6615d895, 88 },
    { 0, 0 },
    { 0xcc65d897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb5d89b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc5d8a6, 76 },
    { 0x8f85d8a7, 90 },
    { 0x3b35d8a6, 76 },
    { 0x1c75d8a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb265d8b4, 46 },
    { 0x79b5d8b5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5d8bb, 98 },
    { 0x7f75d8bc, 55 },
    { 0x5975d8bd, 91 },
    { 0x7e5d8be, 61 },
    { 0xb395d8be, 46 },
    { 0, 0 },
    { 0x6105d8c1, 95 },
    { 0, 0 },
    { 0x1775d8c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad5d8c6, 17 },
    { 0xbac5d8c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a5d8cb, 52 },
    { 0x7bf5d8cb, 90 },
    { 0, 0 },
    { 0x6e05d8ce, 33 },
    { 0x2665d8cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7535d8d3, 23 },
    { 0x1fe5d8d4, 91 },
    { 0xa455d8d5, 4 },
    { 0x8aa5d8d5, 87 },
    { 0, 0 },
    { 0x3835d8d8, 84 },
    { 0xd945d8d9, 95 },
    { 0x9d05d8da, 24 },
    { 0x3a75d8d9, 91 },
    { 0xf1e5d8d8, 55 },
    { 0x9c75d8dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef85d8e6, 66 },
    { 0x3245d8e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d25d8ed, 33 },
    { 0, 0 },
    { 0xfe45d8ef, 95 },
    { 0x4c55d8f0, 95 },
    { 0x45e5d8f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd655d8fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a5d8ff, 33 },
    { 0x1725d900, 66 },
    { 0, 0 },
    { 0xde75d902, 87 },
    { 0xb0d5d903, 61 },
    { 0x69c5d904, 68 },
    { 0x3e75d905, 61 },
    { 0x5f45d904, 87 },
    { 0x9ff5d907, 90 },
    { 0xfa45d904, 61 },
    { 0xb2a5d905, 98 },
    { 0x3125d90a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e5d90d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf5d911, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8645d918, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe5d91b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195d91e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045d926, 55 },
    { 0xcc45d927, 61 },
    { 0xcee5d928, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f5d92e, 46 },
    { 0x4b25d92e, 90 },
    { 0, 0 },
    { 0x9885d931, 40 },
    { 0, 0 },
    { 0xb985d933, 77 },
    { 0x5d15d933, 91 },
    { 0x2fd5d933, 98 },
    { 0xee65d936, 55 },
    { 0xd035d937, 33 },
    { 0, 0 },
    { 0x195d939, 90 },
    { 0, 0 },
    { 0x5035d93b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf5d941, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa825d948, 91 },
    { 0, 0 },
    { 0xadd5d94a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a05d950, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615d953, 98 },
    { 0xeef5d954, 78 },
    { 0xe8b5d955, 43 },
    { 0xd1a5d956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe215d95a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa555d95d, 33 },
    { 0x4ef5d95e, 84 },
    { 0x2435d95d, 23 },
    { 0xe785d960, 98 },
    { 0, 0 },
    { 0x1e5d962, 64 },
    { 0, 0 },
    { 0x66a5d964, 50 },
    { 0x1a95d965, 34 },
    { 0xdde5d966, 95 },
    { 0xa295d967, 90 },
    { 0, 0 },
    { 0x38f5d969, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd5d972, 95 },
    { 0x1645d973, 61 },
    { 0x66f5d974, 87 },
    { 0xf915d975, 90 },
    { 0, 0 },
    { 0x2d05d977, 38 },
    { 0x64d5d978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa815d987, 34 },
    { 0x2b55d987, 90 },
    { 0, 0 },
    { 0x24d5d98a, 98 },
    { 0xae55d98a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f15d990, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55d995, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a5d999, 23 },
    { 0x6395d999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a5d9a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d5d9a5, 98 },
    { 0x4645d9a6, 88 },
    { 0, 0 },
    { 0x71e5d9a8, 95 },
    { 0x365d9a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b55d9af, 43 },
    { 0, 0 },
    { 0x7b95d9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a65d9b4, 91 },
    { 0xbda5d9b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5d9b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb65d9bf, 33 },
    { 0, 0 },
    { 0xafc5d9c1, 95 },
    { 0x3325d9c2, 87 },
    { 0x9ac5d9c3, 33 },
    { 0x4335d9c1, 55 },
    { 0, 0 },
    { 0xd8e5d9c6, 23 },
    { 0x4645d9c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf5d9cb, 66 },
    { 0xa465d9cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac5d9d0, 23 },
    { 0x82a5d9d0, 61 },
    { 0xf025d9d2, 33 },
    { 0x2a15d9d3, 91 },
    { 0x2e35d9d3, 23 },
    { 0, 0 },
    { 0x8be5d9d6, 34 },
    { 0xd5f5d9d7, 98 },
    { 0x6465d9d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee55d9df, 85 },
    { 0xb815d9df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9445d9e5, 23 },
    { 0xb145d9e5, 95 },
    { 0x90d5d9e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5005d9ef, 33 },
    { 0, 0 },
    { 0x8f85d9f1, 95 },
    { 0x2b45d9f2, 98 },
    { 0x11f5d9f2, 66 },
    { 0x5995d9f4, 90 },
    { 0x5925d9f5, 33 },
    { 0, 0 },
    { 0x9855d9f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a5d9fa, 93 },
    { 0x34e5d9fb, 33 },
    { 0, 0 },
    { 0xb3d5d9fd, 78 },
    { 0xdd45d9fe, 55 },
    { 0, 0 },
    { 0xe325da00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df5da07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa5da0a, 90 },
    { 0x8e05da0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5da0f, 95 },
    { 0, 0 },
    { 0xc2f5da11, 61 },
    { 0x5db5da11, 98 },
    { 0, 0 },
    { 0xdd55da14, 95 },
    { 0x38f5da15, 76 },
    { 0xe125da15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4925da19, 75 },
    { 0x21a5da1a, 91 },
    { 0xc875da1b, 66 },
    { 0xcae5da1b, 98 },
    { 0, 0 },
    { 0xfaf5da1e, 90 },
    { 0, 0 },
    { 0x61f5da20, 34 },
    { 0, 0 },
    { 0x1465da22, 95 },
    { 0x4725da22, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf065da28, 65 },
    { 0x5eb5da29, 55 },
    { 0x98b5da2a, 23 },
    { 0xccc5da2b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf75da2e, 53 },
    { 0, 0 },
    { 0xc405da30, 84 },
    { 0xe6e5da31, 55 },
    { 0, 0 },
    { 0xd675da33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3675da37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de5da3b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff5da3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1705da48, 46 },
    { 0x8cd5da48, 61 },
    { 0x89e5da4a, 33 },
    { 0xb855da4b, 34 },
    { 0xfb55da4b, 33 },
    { 0xd675da4d, 33 },
    { 0xd875da4c, 90 },
    { 0xaaa5da4f, 52 },
    { 0xe555da4b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa05da59, 93 },
    { 0xb995da5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c5da5e, 33 },
    { 0xf735da5f, 23 },
    { 0x765da60, 90 },
    { 0x66a5da60, 34 },
    { 0, 0 },
    { 0xc8d5da63, 77 },
    { 0x9a5da63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7235da68, 33 },
    { 0xa15da69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7595da71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c5da74, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a35da7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b5da7f, 55 },
    { 0xa3b5da80, 23 },
    { 0xc5c5da80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe95da85, 99 },
    { 0x74a5da85, 33 },
    { 0, 0 },
    { 0xef95da88, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c5da8e, 93 },
    { 0xc6c5da8f, 46 },
    { 0xa275da90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25da93, 55 },
    { 0x6bc5da94, 55 },
    { 0xf505da94, 95 },
    { 0x6e55da96, 61 },
    { 0xc455da93, 90 },
    { 0xfa75da97, 90 },
    { 0xa805da99, 61 },
    { 0xf125da9a, 88 },
    { 0xad95da9a, 43 },
    { 0x9015da9a, 90 },
    { 0x81c5da9d, 91 },
    { 0x6105da95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc5daa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a5daa8, 80 },
    { 0x2655daa8, 100 },
    { 0, 0 },
    { 0x4785daab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1645daae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d5dab4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5dac1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5dac4, 100 },
    { 0, 0 },
    { 0xfa35dac6, 98 },
    { 0xaaa5dac7, 90 },
    { 0x4015dac8, 95 },
    { 0xfca5dac8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6215dacd, 98 },
    { 0x6ac5dace, 33 },
    { 0x9dc5dacd, 24 },
    { 0xe915dad0, 90 },
    { 0x8015dad1, 33 },
    { 0xae45dad1, 17 },
    { 0xf475dad3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875dad6, 98 },
    { 0, 0 },
    { 0xe365dad8, 66 },
    { 0x9a25dad9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d5dade, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2895dae3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1185daed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d85daf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425dafc, 61 },
    { 0xf25dafd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf735db02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a45db05, 46 },
    { 0, 0 },
    { 0x9565db07, 90 },
    { 0x6155db08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115db15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8535db18, 46 },
    { 0x9235db19, 95 },
    { 0x4505db18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f55db1f, 49 },
    { 0, 0 },
    { 0xea35db21, 43 },
    { 0, 0 },
    { 0x1065db23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4995db28, 76 },
    { 0x3225db29, 76 },
    { 0, 0 },
    { 0x48f5db2b, 61 },
    { 0, 0 },
    { 0xed15db2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e5db32, 55 },
    { 0x2a55db33, 87 },
    { 0, 0 },
    { 0x9995db35, 34 },
    { 0, 0 },
    { 0x8d5db37, 90 },
    { 0x9335db38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f5db3b, 61 },
    { 0, 0 },
    { 0x36b5db3d, 55 },
    { 0xf965db3d, 76 },
    { 0x7d5db3d, 87 },
    { 0xa6a5db40, 76 },
    { 0xa25db3f, 43 },
    { 0x6fb5db3f, 98 },
    { 0x155db43, 40 },
    { 0x5eb5db40, 67 },
    { 0x45d5db45, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3915db48, 33 },
    { 0x2d85db49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf25db4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b5db52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6095db5b, 90 },
    { 0, 0 },
    { 0x4615db5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245db61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e25db64, 84 },
    { 0, 0 },
    { 0x4a15db66, 78 },
    { 0x4b85db67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba5db6f, 76 },
    { 0xc1b5db70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f5db73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc635db79, 61 },
    { 0, 0 },
    { 0xce15db7b, 95 },
    { 0x3ad5db7c, 23 },
    { 0x6a35db7b, 90 },
    { 0x44a5db7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa5db81, 69 },
    { 0x6fb5db81, 93 },
    { 0x4105db81, 100 },
    { 0x8e45db82, 61 },
    { 0, 0 },
    { 0x8be5db86, 76 },
    { 0x4c35db86, 33 },
    { 0x82b5db86, 34 },
    { 0x3b65db87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda85db8d, 61 },
    { 0xa355db8e, 33 },
    { 0xf935db8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e5db92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1225db97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e5db9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5db9d, 93 },
    { 0x4c15db9e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca5dba6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a5dba9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa5dbae, 34 },
    { 0x1df5dbae, 98 },
    { 0, 0 },
    { 0x2bd5dbb1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875dbb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c35dbb9, 99 },
    { 0x26d5dbba, 90 },
    { 0x3a65dbba, 23 },
    { 0x7c55dbbc, 52 },
    { 0xfc55dbba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f5dbc1, 90 },
    { 0, 0 },
    { 0x35e5dbc3, 91 },
    { 0xb005dbc3, 77 },
    { 0xd295dbc5, 90 },
    { 0xccd5dbc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb475dbd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b75dbd4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda85dbd9, 61 },
    { 0xa9a5dbda, 61 },
    { 0xe9e5dbda, 33 },
    { 0x5a45dbdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2755dbdf, 95 },
    { 0x20f5dbdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f5dbe4, 98 },
    { 0, 0 },
    { 0x25d5dbe6, 23 },
    { 0x8e5dbe7, 76 },
    { 0x8575dbe8, 68 },
    { 0x9245dbe7, 76 },
    { 0x9995dbea, 76 },
    { 0x4d5dbea, 33 },
    { 0x2f65dbe6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c5dbf0, 95 },
    { 0x3915dbf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e5dbf5, 99 },
    { 0xaca5dbf6, 33 },
    { 0, 0 },
    { 0xc5e5dbf8, 55 },
    { 0xfdd5dbf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c5dbfd, 78 },
    { 0xdbb5dbfe, 68 },
    { 0x89c5dbff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd085dc06, 55 },
    { 0, 0 },
    { 0x77b5dc08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c5dc0d, 43 },
    { 0, 0 },
    { 0xf4d5dc0f, 34 },
    { 0xd75dc0f, 98 },
    { 0x3525dc11, 76 },
    { 0xd7f5dc12, 69 },
    { 0, 0 },
    { 0x5505dc14, 95 },
    { 0x6615dc15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b5dc1d, 66 },
    { 0x6075dc1d, 98 },
    { 0, 0 },
    { 0x1815dc20, 100 },
    { 0xc305dc21, 88 },
    { 0x51b5dc20, 90 },
    { 0, 0 },
    { 0x1e25dc24, 90 },
    { 0x6365dc24, 90 },
    { 0x4615dc25, 33 },
    { 0, 0 },
    { 0xcfa5dc28, 93 },
    { 0xf025dc28, 61 },
    { 0xac25dc2a, 84 },
    { 0, 0 },
    { 0xc2f5dc2c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5dc2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5dc37, 85 },
    { 0x3245dc37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee5dc3b, 33 },
    { 0xf75dc3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4555dc41, 98 },
    { 0x5d45dc42, 45 },
    { 0xff45dc43, 75 },
    { 0x7c35dc42, 23 },
    { 0xf735dc45, 90 },
    { 0xdc65dc46, 65 },
    { 0x9f05dc47, 90 },
    { 0xda05dc47, 77 },
    { 0x68f5dc49, 33 },
    { 0x7a35dc44, 93 },
    { 0xfe95dc44, 46 },
    { 0x4f35dc4c, 33 },
    { 0xb115dc4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3235dc51, 95 },
    { 0x5ab5dc52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2335dc55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5dc58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115dc5c, 23 },
    { 0, 0 },
    { 0x80c5dc5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa225dc64, 55 },
    { 0x305dc65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5dc6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5965dc70, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d5dc74, 95 },
    { 0, 0 },
    { 0x6f45dc76, 61 },
    { 0, 0 },
    { 0x25f5dc78, 90 },
    { 0, 0 },
    { 0xfff5dc7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7565dc7d, 90 },
    { 0x35d5dc7e, 46 },
    { 0x1d85dc7e, 55 },
    { 0x8ca5dc80, 53 },
    { 0, 0 },
    { 0x3315dc82, 61 },
    { 0xc035dc82, 61 },
    { 0x6445dc84, 34 },
    { 0x8275dc84, 93 },
    { 0xb4c5dc86, 90 },
    { 0x7935dc85, 76 },
    { 0x87a5dc88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f5dc8d, 99 },
    { 0, 0 },
    { 0x8a95dc8f, 90 },
    { 0x9435dc90, 91 },
    { 0x5e45dc90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec95dc95, 51 },
    { 0x86f5dc96, 93 },
    { 0xffb5dc97, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc5dc9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5dc9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a5dca5, 91 },
    { 0x5295dca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec5dca9, 95 },
    { 0xa035dca9, 90 },
    { 0xed5dcab, 33 },
    { 0x7985dcab, 90 },
    { 0xe705dcaa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b5dcb4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac15dcb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd15dcbc, 43 },
    { 0, 0 },
    { 0x6c5dcbe, 61 },
    { 0, 0 },
    { 0x41b5dcc0, 93 },
    { 0xe675dcc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb425dcc4, 90 },
    { 0xdcd5dcc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05dccb, 76 },
    { 0, 0 },
    { 0x1995dccd, 90 },
    { 0xf4c5dccd, 101 },
    { 0x3d15dccf, 53 },
    { 0xd185dccf, 77 },
    { 0, 0 },
    { 0x6095dcd2, 66 },
    { 0x9165dcd2, 67 },
    { 0, 0 },
    { 0xe885dcd5, 33 },
    { 0x83b5dcd6, 33 },
    { 0, 0 },
    { 0xda15dcd8, 61 },
    { 0xf8e5dcd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a45dcdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf865dce2, 55 },
    { 0x4d5dce3, 34 },
    { 0x24e5dce4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905dcea, 88 },
    { 0x8c75dceb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd5dcee, 95 },
    { 0, 0 },
    { 0x4d65dcf0, 76 },
    { 0x6ad5dcf1, 34 },
    { 0xf865dcf2, 68 },
    { 0, 0 },
    { 0xbf85dcf4, 23 },
    { 0, 0 },
    { 0xd755dcf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e5dcf9, 17 },
    { 0x825dcfa, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c65dd03, 88 },
    { 0, 0 },
    { 0xe145dd05, 67 },
    { 0xdcb5dd06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc85dd0a, 46 },
    { 0x3545dd0a, 90 },
    { 0xbf65dd0c, 87 },
    { 0xe9b5dd0d, 95 },
    { 0x7e05dd0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea5dd11, 61 },
    { 0x4415dd11, 95 },
    { 0x8e15dd13, 78 },
    { 0, 0 },
    { 0xa1f5dd15, 91 },
    { 0x3755dd16, 100 },
    { 0, 0 },
    { 0x2c25dd18, 99 },
    { 0, 0 },
    { 0x1e05dd1a, 33 },
    { 0xdfe5dd1b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8255dd21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35dd27, 33 },
    { 0, 0 },
    { 0xe615dd29, 93 },
    { 0x2925dd2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7465dd2d, 61 },
    { 0, 0 },
    { 0xb455dd2f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a65dd32, 89 },
    { 0x49e5dd32, 90 },
    { 0x1f25dd34, 76 },
    { 0xbd05dd35, 83 },
    { 0x63e5dd35, 61 },
    { 0xd7f5dd32, 90 },
    { 0x74f5dd32, 98 },
    { 0x45e5dd32, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe85dd3e, 99 },
    { 0x2625dd3e, 81 },
    { 0x4b85dd40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c5dd4a, 61 },
    { 0, 0 },
    { 0x8985dd4c, 40 },
    { 0, 0 },
    { 0xd9b5dd4e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5dd59, 90 },
    { 0xae65dd59, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc5dd61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd45dd66, 98 },
    { 0x80d5dd67, 33 },
    { 0x9e85dd68, 88 },
    { 0xef5dd69, 53 },
    { 0xb4c5dd6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755dd6e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5995dd73, 84 },
    { 0xda65dd74, 61 },
    { 0x8345dd75, 33 },
    { 0x98f5dd76, 98 },
    { 0xc955dd77, 77 },
    { 0xbdf5dd75, 98 },
    { 0, 0 },
    { 0x19e5dd7a, 95 },
    { 0x30d5dd7b, 33 },
    { 0xa8f5dd7b, 66 },
    { 0x6425dd7d, 95 },
    { 0x28f5dd7c, 33 },
    { 0xa175dd7c, 17 },
    { 0x9475dd7d, 34 },
    { 0xff75dd81, 55 },
    { 0x38a5dd82, 95 },
    { 0x8dd5dd81, 90 },
    { 0x6fb5dd84, 95 },
    { 0xa85dd85, 88 },
    { 0xfb85dd85, 98 },
    { 0x7b75dd85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f65dd91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa965dd96, 34 },
    { 0xc8b5dd97, 33 },
    { 0, 0 },
    { 0x8b95dd99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba5dd9d, 66 },
    { 0, 0 },
    { 0x1215dd9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba5dda4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b5ddaa, 52 },
    { 0x1f95ddab, 33 },
    { 0x1e15ddab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe695ddb1, 23 },
    { 0x8485ddb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf415ddb6, 76 },
    { 0, 0 },
    { 0xbf35ddb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b5ddbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc5ddbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da5ddc2, 90 },
    { 0x7875ddc3, 61 },
    { 0x2e15ddc4, 55 },
    { 0, 0 },
    { 0x80e5ddc6, 85 },
    { 0x325ddc7, 68 },
    { 0xbac5ddc6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf5ddcc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9125ddcf, 23 },
    { 0x32f5ddd0, 85 },
    { 0x5b05ddd1, 61 },
    { 0x6d15ddd2, 43 },
    { 0x9335ddd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe735ddd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9445dde1, 23 },
    { 0x5825dde2, 76 },
    { 0xf225dde3, 23 },
    { 0, 0 },
    { 0x1b65dde5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe305ddea, 76 },
    { 0, 0 },
    { 0x3b35ddec, 45 },
    { 0xe355dded, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c5ddf3, 90 },
    { 0x9d45ddf4, 95 },
    { 0xaad5ddf4, 96 },
    { 0x5645ddf5, 34 },
    { 0x9b25ddf7, 53 },
    { 0x7455ddf7, 98 },
    { 0x5195ddf4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f5ddfc, 23 },
    { 0xba25ddfd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935de02, 98 },
    { 0xe3a5de03, 23 },
    { 0xde65de04, 76 },
    { 0xcb65de05, 75 },
    { 0xe845de04, 90 },
    { 0xb45de04, 55 },
    { 0x1b15de05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75de0b, 65 },
    { 0xab35de0b, 43 },
    { 0x9265de0c, 99 },
    { 0xe0c5de0e, 61 },
    { 0x7a5de0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa95de15, 33 },
    { 0xa935de16, 67 },
    { 0xe8b5de17, 23 },
    { 0x9795de17, 55 },
    { 0x6c45de19, 98 },
    { 0x8ce5de19, 95 },
    { 0x9905de15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6405de25, 90 },
    { 0x4fc5de26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5de29, 23 },
    { 0, 0 },
    { 0x31b5de2b, 43 },
    { 0xb6f5de2c, 23 },
    { 0xf745de2d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1155de34, 34 },
    { 0, 0 },
    { 0xaec5de36, 64 },
    { 0, 0 },
    { 0xa975de38, 75 },
    { 0xd1b5de38, 90 },
    { 0, 0 },
    { 0x9a55de3b, 52 },
    { 0xc2d5de3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2975de3f, 100 },
    { 0x5635de40, 90 },
    { 0, 0 },
    { 0x9565de42, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685de46, 78 },
    { 0, 0 },
    { 0x28f5de48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e5de50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1215de53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb575de5c, 84 },
    { 0x1255de5d, 90 },
    { 0, 0 },
    { 0xabf5de5f, 61 },
    { 0, 0 },
    { 0x6a25de61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe785de64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e85de6d, 90 },
    { 0xfb55de6e, 93 },
    { 0x6c75de6d, 75 },
    { 0x1dc5de70, 90 },
    { 0, 0 },
    { 0x7c35de72, 46 },
    { 0, 0 },
    { 0xddd5de74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d95de77, 23 },
    { 0xc4d5de78, 61 },
    { 0xdea5de79, 95 },
    { 0x6885de7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x245de7d, 65 },
    { 0xb255de7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95de82, 90 },
    { 0xd4b5de82, 95 },
    { 0, 0 },
    { 0xa875de85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7615de91, 95 },
    { 0, 0 },
    { 0x1135de93, 88 },
    { 0xb205de94, 23 },
    { 0xee55de95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e65de9b, 98 },
    { 0x7e45de9c, 61 },
    { 0x7c05de9d, 40 },
    { 0xdb25de9e, 85 },
    { 0, 0 },
    { 0x1c45dea0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5925dea3, 95 },
    { 0x6905dea4, 91 },
    { 0x1c65dea5, 90 },
    { 0, 0 },
    { 0x9705dea7, 100 },
    { 0, 0 },
    { 0xee75dea9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc35deac, 77 },
    { 0x8a5dead, 98 },
    { 0, 0 },
    { 0x7ab5deaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c5deb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc35deb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75deb9, 98 },
    { 0x56e5deb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745dec0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e5dec5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7555dec8, 66 },
    { 0, 0 },
    { 0xbc85deca, 76 },
    { 0x2a75decb, 88 },
    { 0x5a5decc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c5ded0, 66 },
    { 0x3725ded0, 43 },
    { 0xdc25ded2, 90 },
    { 0, 0 },
    { 0x2f55ded4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d5ded8, 53 },
    { 0xeee5ded9, 33 },
    { 0x6255deda, 61 },
    { 0xa9f5ded9, 93 },
    { 0xa505ded9, 34 },
    { 0x96e5dedb, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b5dee0, 99 },
    { 0xcd45dee1, 90 },
    { 0xea35dee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c5dee6, 33 },
    { 0, 0 },
    { 0xb75dee8, 67 },
    { 0xc445dee9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb5deef, 61 },
    { 0x9355deef, 46 },
    { 0xf255def1, 33 },
    { 0x34d5def2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd5def5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35def8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe465df00, 61 },
    { 0, 0 },
    { 0xf105df02, 43 },
    { 0, 0 },
    { 0xdda5df04, 61 },
    { 0, 0 },
    { 0x6235df06, 81 },
    { 0, 0 },
    { 0xafb5df08, 90 },
    { 0xdf25df08, 61 },
    { 0x9135df0a, 90 },
    { 0, 0 },
    { 0xe3a5df0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4415df18, 95 },
    { 0x8255df19, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff15df1e, 95 },
    { 0xb6e5df1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb5df2a, 61 },
    { 0xaf75df2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b5df2f, 90 },
    { 0, 0 },
    { 0x5275df31, 76 },
    { 0x8f35df32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd345df37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e5df3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c5df41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2295df44, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d35df4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab5df51, 84 },
    { 0x3655df51, 33 },
    { 0xe265df52, 84 },
    { 0x7fa5df54, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6145df58, 90 },
    { 0, 0 },
    { 0xb515df5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd065df5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc5df61, 98 },
    { 0xc155df62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8365df65, 55 },
    { 0x8db5df66, 55 },
    { 0x99a5df66, 100 },
    { 0xa6d5df67, 23 },
    { 0x9c15df66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75df71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a95df7c, 95 },
    { 0x2645df7c, 23 },
    { 0xfc5df7e, 76 },
    { 0x8465df7f, 23 },
    { 0x4525df7c, 61 },
    { 0x4375df81, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e5df8b, 91 },
    { 0x8045df8c, 43 },
    { 0x1c25df8d, 95 },
    { 0x1635df8c, 93 },
    { 0, 0 },
    { 0xc485df90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8785df97, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6315df9a, 68 },
    { 0xf7b5df9b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe695df9f, 34 },
    { 0x15f5df9f, 99 },
    { 0x4445dfa1, 66 },
    { 0xbc65dfa2, 93 },
    { 0xb9d5dfa3, 61 },
    { 0xd8f5df9f, 95 },
    { 0xf575dfa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15dfa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6005dfab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a25dfb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7065dfb6, 90 },
    { 0xdbc5dfb7, 33 },
    { 0xd8f5dfb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c5dfc0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2525dfc6, 95 },
    { 0, 0 },
    { 0x7b25dfc8, 76 },
    { 0x3915dfc9, 46 },
    { 0xcd95dfca, 99 },
    { 0x4685dfcb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8095dfcf, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c5dfd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5dfd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d5dfe0, 33 },
    { 0, 0 },
    { 0xff85dfe2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa365dfe7, 95 },
    { 0xa795dfe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb765dff5, 91 },
    { 0x9595dff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5dff9, 93 },
    { 0, 0 },
    { 0xda65dffb, 90 },
    { 0xce85dffc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba5e000, 98 },
    { 0xa0f5e001, 55 },
    { 0xa2e5e002, 66 },
    { 0xc925e000, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4055e007, 61 },
    { 0x4935e008, 61 },
    { 0xa3c5e007, 90 },
    { 0x11c5e00a, 46 },
    { 0xce65e00a, 61 },
    { 0, 0 },
    { 0x1225e00d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9925e015, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e75e01a, 33 },
    { 0x6775e01b, 68 },
    { 0x1575e01a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b5e01f, 55 },
    { 0xe345e01f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e25e029, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe295e034, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb355e03b, 88 },
    { 0x29c5e03b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7225e040, 98 },
    { 0, 0 },
    { 0x6d75e042, 100 },
    { 0x5ac5e043, 43 },
    { 0x6955e042, 61 },
    { 0x2c15e045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9775e04b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5e050, 33 },
    { 0, 0 },
    { 0x7985e052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b5e055, 56 },
    { 0x3695e056, 94 },
    { 0, 0 },
    { 0xd3d5e058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd785e05b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd5e05e, 87 },
    { 0x30a5e05e, 61 },
    { 0x4f15e05e, 90 },
    { 0xda55e05e, 93 },
    { 0xd045e061, 84 },
    { 0xb7d5e063, 81 },
    { 0, 0 },
    { 0xc3c5e065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4195e06b, 95 },
    { 0x16a5e06c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a5e06f, 23 },
    { 0xb505e070, 95 },
    { 0xf345e071, 100 },
    { 0x6f25e072, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e5e07f, 90 },
    { 0xfb75e080, 76 },
    { 0x87d5e081, 23 },
    { 0xa4c5e082, 95 },
    { 0, 0 },
    { 0xc565e084, 90 },
    { 0xd975e085, 78 },
    { 0xe635e086, 43 },
    { 0x3095e087, 33 },
    { 0, 0 },
    { 0xf8f5e089, 98 },
    { 0xa965e08a, 80 },
    { 0x8465e08a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b5e08f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db5e093, 61 },
    { 0, 0 },
    { 0x8bb5e095, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5675e098, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2625e09e, 61 },
    { 0xde95e09f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c85e0a2, 61 },
    { 0x16b5e0a3, 33 },
    { 0xbcb5e0a2, 33 },
    { 0, 0 },
    { 0x92b5e0a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd145e0ab, 98 },
    { 0, 0 },
    { 0x83c5e0ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d5e0b4, 55 },
    { 0x85a5e0b4, 95 },
    { 0xaf45e0b6, 33 },
    { 0xcc55e0b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7835e0ba, 61 },
    { 0, 0 },
    { 0x6095e0bc, 34 },
    { 0xfc95e0bd, 95 },
    { 0xc245e0be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee5e0c3, 95 },
    { 0xf305e0c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b85e0c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f5e0ca, 34 },
    { 0x8af5e0cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5e0d2, 61 },
    { 0, 0 },
    { 0x8635e0d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1075e0d8, 23 },
    { 0, 0 },
    { 0xdde5e0da, 36 },
    { 0x4c35e0da, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc5e0df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b5e0e5, 98 },
    { 0xcc45e0e6, 99 },
    { 0xfdc5e0e7, 76 },
    { 0xb895e0e8, 23 },
    { 0xcdc5e0e9, 55 },
    { 0x6445e0e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5e0f1, 17 },
    { 0xe895e0f2, 95 },
    { 0x4815e0f3, 98 },
    { 0xc345e0f3, 66 },
    { 0xb385e0f4, 99 },
    { 0, 0 },
    { 0x5e25e0f7, 98 },
    { 0x1125e0f8, 33 },
    { 0xa6f5e0f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6145e0fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb615e0ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c5e104, 61 },
    { 0, 0 },
    { 0x11f5e106, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b5e10b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa845e10e, 91 },
    { 0xeb65e10e, 100 },
    { 0x56c5e110, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce5e116, 84 },
    { 0x8675e116, 33 },
    { 0x9225e118, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e11b, 61 },
    { 0xad45e11c, 33 },
    { 0, 0 },
    { 0xe655e11e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee5e123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af5e12c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f75e135, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb25e13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f85e13f, 90 },
    { 0x8bc5e13f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a5e146, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f15e152, 33 },
    { 0, 0 },
    { 0x1595e154, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e5e15a, 55 },
    { 0xad95e15b, 33 },
    { 0, 0 },
    { 0xf0f5e15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a75e163, 99 },
    { 0x8685e163, 88 },
    { 0x1ef5e164, 69 },
    { 0x7b75e165, 98 },
    { 0xbc35e167, 87 },
    { 0xe235e164, 61 },
    { 0xaf05e166, 90 },
    { 0xaac5e16a, 55 },
    { 0xf0c5e16b, 76 },
    { 0x7ad5e16a, 93 },
    { 0x4745e166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b95e172, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e75e175, 33 },
    { 0, 0 },
    { 0x8835e177, 33 },
    { 0xad5e178, 98 },
    { 0x7105e179, 61 },
    { 0xbca5e179, 17 },
    { 0x97a5e17b, 33 },
    { 0x7745e17b, 55 },
    { 0x3195e17a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd5e186, 61 },
    { 0x7195e186, 33 },
    { 0x7d85e188, 84 },
    { 0xf25e189, 33 },
    { 0xa4a5e188, 90 },
    { 0, 0 },
    { 0xd6a5e18c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a95e190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd345e194, 61 },
    { 0x94d5e195, 98 },
    { 0x20b5e196, 61 },
    { 0xf735e194, 95 },
    { 0xb875e198, 90 },
    { 0x6795e199, 52 },
    { 0xc1a5e19a, 61 },
    { 0x1515e19b, 23 },
    { 0, 0 },
    { 0xb0f5e19d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd155e1a1, 98 },
    { 0xafc5e1a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7995e1a7, 95 },
    { 0xed65e1a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe5e1ab, 95 },
    { 0x47e5e1ac, 85 },
    { 0xc655e1ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb825e1b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe5e1ba, 95 },
    { 0, 0 },
    { 0x2ea5e1bc, 33 },
    { 0, 0 },
    { 0x4725e1be, 55 },
    { 0x9af5e1bf, 33 },
    { 0x5cb5e1c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5e1c4, 88 },
    { 0x6405e1c5, 85 },
    { 0x8645e1c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5e1cb, 23 },
    { 0x34b5e1cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e5e1d4, 78 },
    { 0x9035e1d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b5e1de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475e1e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2125e1e6, 23 },
    { 0x64a5e1e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5e1ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7695e1f2, 61 },
    { 0xe9b5e1f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5e1f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9285e1fa, 46 },
    { 0xaf5e1fb, 55 },
    { 0x1935e1fc, 61 },
    { 0, 0 },
    { 0xb945e1fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad5e209, 23 },
    { 0x6de5e20a, 95 },
    { 0xda95e20a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d5e210, 84 },
    { 0x91c5e211, 100 },
    { 0xce85e212, 23 },
    { 0x92e5e210, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e35e218, 23 },
    { 0xa6c5e218, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa305e21c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc175e21f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff45e223, 95 },
    { 0, 0 },
    { 0xb6c5e225, 98 },
    { 0xead5e225, 66 },
    { 0x78b5e225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf55e22a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35e237, 43 },
    { 0, 0 },
    { 0x68f5e239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5e240, 52 },
    { 0, 0 },
    { 0xf435e242, 61 },
    { 0x8885e242, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c95e246, 33 },
    { 0x4bb5e247, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e95e24d, 33 },
    { 0, 0 },
    { 0xead5e24f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe515e252, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe95e25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3095e25f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5e265, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a85e26c, 99 },
    { 0x1f75e26c, 33 },
    { 0, 0 },
    { 0x7e15e26f, 78 },
    { 0x7545e26f, 76 },
    { 0x5a5e271, 98 },
    { 0xb245e272, 55 },
    { 0, 0 },
    { 0xa3e5e274, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7705e277, 23 },
    { 0xa425e278, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa85e27c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc965e27f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd805e284, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb075e28a, 90 },
    { 0xc655e28b, 95 },
    { 0, 0 },
    { 0x5e65e28d, 98 },
    { 0x8f65e28d, 46 },
    { 0x5325e28f, 98 },
    { 0x7585e290, 33 },
    { 0, 0 },
    { 0x1165e292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5065e299, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb225e29c, 43 },
    { 0xd945e29d, 100 },
    { 0xccb5e29e, 91 },
    { 0xfe25e29f, 90 },
    { 0x1805e29f, 95 },
    { 0x62b5e2a1, 67 },
    { 0x8be5e2a2, 23 },
    { 0, 0 },
    { 0x435e2a4, 95 },
    { 0x8065e2a5, 98 },
    { 0, 0 },
    { 0xb4a5e2a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a5e2ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c5e2b0, 43 },
    { 0xe895e2b1, 55 },
    { 0x3db5e2b2, 76 },
    { 0xc025e2b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005e2b6, 34 },
    { 0, 0 },
    { 0x1935e2b8, 34 },
    { 0x35b5e2b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2165e2c9, 90 },
    { 0, 0 },
    { 0x18d5e2cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f5e2cf, 46 },
    { 0, 0 },
    { 0xd1c5e2d1, 88 },
    { 0x8fe5e2d2, 88 },
    { 0xc115e2d2, 95 },
    { 0xfb85e2d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7795e2d7, 90 },
    { 0x7f15e2d7, 61 },
    { 0xa5e5e2d9, 90 },
    { 0, 0 },
    { 0x4d15e2db, 52 },
    { 0, 0 },
    { 0x6045e2dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5755e2e1, 33 },
    { 0, 0 },
    { 0x8685e2e3, 33 },
    { 0, 0 },
    { 0x37b5e2e5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe05e2e9, 23 },
    { 0, 0 },
    { 0xc6c5e2eb, 98 },
    { 0x4675e2ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3205e2ef, 24 },
    { 0xaea5e2f0, 78 },
    { 0x7755e2f1, 34 },
    { 0x9e45e2f1, 98 },
    { 0, 0 },
    { 0x3725e2f4, 98 },
    { 0xc0f5e2f4, 95 },
    { 0xac05e2f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe865e2fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe25e2fd, 23 },
    { 0xc75e2fe, 33 },
    { 0x4b75e2fd, 100 },
    { 0xee75e300, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb495e309, 77 },
    { 0x90f5e309, 23 },
    { 0, 0 },
    { 0xbc65e30c, 95 },
    { 0x5325e30d, 61 },
    { 0x1bd5e30c, 90 },
    { 0, 0 },
    { 0xe765e310, 61 },
    { 0x6f5e311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb5e315, 52 },
    { 0, 0 },
    { 0xfb15e317, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e31a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da5e31e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a5e322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e35e32b, 98 },
    { 0x1d05e32c, 33 },
    { 0x15a5e32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f5e330, 76 },
    { 0x9dd5e331, 77 },
    { 0xfde5e331, 23 },
    { 0xabe5e333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf35e337, 55 },
    { 0, 0 },
    { 0x1075e339, 98 },
    { 0xdd55e33a, 23 },
    { 0, 0 },
    { 0x5195e33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe625e341, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f5e345, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4625e349, 61 },
    { 0, 0 },
    { 0x18e5e34b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d75e34f, 87 },
    { 0xd775e34f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a45e353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a45e35b, 84 },
    { 0xe735e35c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc275e362, 23 },
    { 0, 0 },
    { 0x5e85e364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5e367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e36b, 76 },
    { 0xfd75e36b, 61 },
    { 0, 0 },
    { 0xda05e36e, 95 },
    { 0, 0 },
    { 0x2815e370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada5e379, 38 },
    { 0xbe65e37a, 66 },
    { 0x7925e37b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355e381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f5e386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6395e389, 98 },
    { 0x4195e38a, 95 },
    { 0, 0 },
    { 0x1b5e38c, 77 },
    { 0, 0 },
    { 0xce75e38e, 34 },
    { 0, 0 },
    { 0x2745e390, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b5e393, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa5e396, 76 },
    { 0, 0 },
    { 0x3675e398, 88 },
    { 0x93c5e399, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c5e39c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa975e39f, 77 },
    { 0xa705e39f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6025e3a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd045e3a6, 90 },
    { 0xe225e3a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f95e3aa, 61 },
    { 0xb5d5e3ab, 33 },
    { 0xb395e3ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf045e3b0, 43 },
    { 0, 0 },
    { 0x5a75e3b2, 61 },
    { 0x73e5e3b3, 23 },
    { 0x8e25e3b4, 34 },
    { 0, 0 },
    { 0xd295e3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe765e3bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e5e3c0, 68 },
    { 0x4845e3c1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85e3c8, 95 },
    { 0x67f5e3c9, 51 },
    { 0, 0 },
    { 0xa25e3cb, 95 },
    { 0x75f5e3cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9385e3cf, 33 },
    { 0, 0 },
    { 0xa545e3d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e5e3d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec5e3d9, 61 },
    { 0x2255e3d9, 90 },
    { 0x8b65e3db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e55e3e5, 84 },
    { 0xb595e3e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x105e3e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe715e3f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d85e3f3, 33 },
    { 0, 0 },
    { 0x6875e3f5, 55 },
    { 0x6685e3f6, 84 },
    { 0x61b5e3f6, 36 },
    { 0x4335e3f6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e5e3fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1005e400, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf45e404, 95 },
    { 0xfd45e405, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe5e408, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8525e40d, 78 },
    { 0x1895e40e, 87 },
    { 0xaa85e40f, 61 },
    { 0xf185e410, 76 },
    { 0x8bc5e40d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b5e417, 76 },
    { 0x5e05e417, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd5e41e, 100 },
    { 0xdbe5e41e, 68 },
    { 0x8505e41e, 34 },
    { 0x5585e421, 61 },
    { 0, 0 },
    { 0xb315e423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde55e429, 61 },
    { 0, 0 },
    { 0x9525e42b, 95 },
    { 0x9845e42b, 93 },
    { 0, 0 },
    { 0x3fc5e42e, 95 },
    { 0x82a5e42f, 85 },
    { 0x55e42f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd765e440, 99 },
    { 0xe935e440, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2985e446, 95 },
    { 0x7545e446, 33 },
    { 0x17e5e448, 90 },
    { 0xd645e446, 90 },
    { 0xc5b5e446, 23 },
    { 0x95e5e44b, 61 },
    { 0xe085e44c, 76 },
    { 0x31e5e44d, 76 },
    { 0x6ca5e44d, 23 },
    { 0x8f25e44e, 93 },
    { 0x7b45e450, 81 },
    { 0x9c75e450, 95 },
    { 0x8585e451, 91 },
    { 0, 0 },
    { 0x7da5e454, 66 },
    { 0, 0 },
    { 0xb675e456, 23 },
    { 0xd6c5e457, 66 },
    { 0, 0 },
    { 0xca75e459, 93 },
    { 0x6835e45a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3865e45f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4185e464, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c95e469, 90 },
    { 0, 0 },
    { 0x2ac5e46b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9815e46f, 90 },
    { 0x1b5e46f, 83 },
    { 0, 0 },
    { 0xe7c5e472, 98 },
    { 0xbb25e473, 88 },
    { 0, 0 },
    { 0xadf5e475, 90 },
    { 0, 0 },
    { 0xc3a5e477, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3835e481, 98 },
    { 0xe445e481, 33 },
    { 0x2cd5e483, 95 },
    { 0xa9a5e481, 90 },
    { 0, 0 },
    { 0x7885e486, 55 },
    { 0x8665e487, 90 },
    { 0, 0 },
    { 0x7d45e489, 91 },
    { 0, 0 },
    { 0x4d75e48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d15e491, 91 },
    { 0, 0 },
    { 0xcec5e493, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6965e496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa525e49c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8335e4a1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf525e4a7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e5e4ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a85e4ae, 33 },
    { 0x8595e4ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5e4b2, 61 },
    { 0xa0f5e4b2, 61 },
    { 0xd055e4b2, 76 },
    { 0, 0 },
    { 0xf245e4b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c5e4b9, 61 },
    { 0x6bd5e4ba, 43 },
    { 0, 0 },
    { 0x72a5e4bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef35e4c0, 76 },
    { 0xf915e4c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15e4c4, 17 },
    { 0x8395e4c5, 33 },
    { 0, 0 },
    { 0x6d75e4c7, 90 },
    { 0x3035e4c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885e4cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef5e4ce, 33 },
    { 0x79f5e4cf, 90 },
    { 0xa795e4d0, 95 },
    { 0x7e35e4ce, 33 },
    { 0xaa65e4ce, 93 },
    { 0xd2c5e4d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205e4d6, 23 },
    { 0x9ec5e4d7, 61 },
    { 0x25e4d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8695e4db, 61 },
    { 0x3b5e4dc, 98 },
    { 0x8195e4dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d5e4e0, 23 },
    { 0xd5a5e4e1, 98 },
    { 0x5215e4e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805e4e8, 46 },
    { 0x5b55e4e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f5e4f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e5e4fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2395e4fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c45e502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe85e50a, 33 },
    { 0x7eb5e50a, 84 },
    { 0x6d55e50c, 68 },
    { 0x1c95e50d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e5e513, 90 },
    { 0xa7c5e513, 98 },
    { 0, 0 },
    { 0xb575e516, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f5e519, 90 },
    { 0, 0 },
    { 0x6455e51b, 33 },
    { 0x2355e51c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf5e51f, 55 },
    { 0, 0 },
    { 0x9355e521, 90 },
    { 0x7175e522, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d5e525, 90 },
    { 0x9b35e525, 55 },
    { 0xb095e526, 43 },
    { 0x65f5e525, 84 },
    { 0xa875e529, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e52f, 95 },
    { 0, 0 },
    { 0xbd95e531, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd215e535, 33 },
    { 0x3165e536, 76 },
    { 0x2565e535, 90 },
    { 0xc855e538, 61 },
    { 0x4e55e539, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb5e53e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a5e542, 55 },
    { 0x7ea5e543, 23 },
    { 0x4b25e544, 66 },
    { 0xa695e543, 100 },
    { 0xbc45e542, 61 },
    { 0xaee5e547, 33 },
    { 0xcb15e546, 46 },
    { 0xc435e543, 23 },
    { 0x2c75e54a, 98 },
    { 0xb675e54a, 96 },
    { 0xc9d5e54b, 43 },
    { 0x2a5e54d, 23 },
    { 0x655e54e, 76 },
    { 0x15e549, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb65e552, 98 },
    { 0, 0 },
    { 0xf2c5e554, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d5e557, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac5e55a, 52 },
    { 0xc4e5e55b, 100 },
    { 0x7ad5e55b, 55 },
    { 0xc295e55d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe065e561, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f45e56a, 76 },
    { 0x2e75e56b, 33 },
    { 0xc0a5e56b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd05e572, 88 },
    { 0, 0 },
    { 0x14f5e574, 33 },
    { 0, 0 },
    { 0x3895e576, 46 },
    { 0x3815e576, 33 },
    { 0x47c5e576, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e57c, 33 },
    { 0xd215e57d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc215e581, 95 },
    { 0, 0 },
    { 0xc0a5e583, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5e58f, 55 },
    { 0x6ae5e590, 93 },
    { 0x7195e590, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a5e596, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a5e599, 52 },
    { 0x31f5e59a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c5e59e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd445e5a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d25e5a7, 33 },
    { 0x6885e5a8, 61 },
    { 0x3c05e5a8, 33 },
    { 0x2af5e5aa, 90 },
    { 0xbf95e5ab, 90 },
    { 0x8c65e5aa, 95 },
    { 0x3785e5ad, 87 },
    { 0x3065e5aa, 23 },
    { 0x1115e5af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8865e5b2, 34 },
    { 0x94c5e5b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e5e5bb, 55 },
    { 0, 0 },
    { 0xf925e5bd, 43 },
    { 0xe5e5e5be, 93 },
    { 0xc2c5e5bf, 90 },
    { 0x6a35e5bf, 99 },
    { 0, 0 },
    { 0x2195e5c2, 46 },
    { 0xe035e5c2, 61 },
    { 0xa2d5e5c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd5e5c9, 90 },
    { 0, 0 },
    { 0x32d5e5cb, 61 },
    { 0, 0 },
    { 0xd275e5cd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa505e5d4, 95 },
    { 0xc955e5d5, 90 },
    { 0xb965e5d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f5e5dd, 33 },
    { 0x50d5e5de, 23 },
    { 0x1535e5de, 83 },
    { 0, 0 },
    { 0x8ee5e5e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd775e5e5, 55 },
    { 0, 0 },
    { 0xa645e5e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5585e5f1, 23 },
    { 0x645e5f2, 33 },
    { 0x1e65e5f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe35e5f9, 61 },
    { 0xcfb5e5f9, 95 },
    { 0x9505e5f9, 61 },
    { 0x7d15e5fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a5e604, 93 },
    { 0, 0 },
    { 0x9b15e606, 43 },
    { 0x8375e607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f5e60e, 91 },
    { 0x9585e60f, 52 },
    { 0xa875e60e, 61 },
    { 0x3b65e60f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef5e614, 90 },
    { 0, 0 },
    { 0x7c15e616, 34 },
    { 0, 0 },
    { 0x8da5e618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc45e623, 92 },
    { 0x79b5e624, 43 },
    { 0x8ee5e625, 81 },
    { 0xe0d5e626, 33 },
    { 0x9cf5e625, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f5e62b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865e62e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f5e632, 23 },
    { 0xfa95e633, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf85e636, 100 },
    { 0x425e636, 95 },
    { 0, 0 },
    { 0x9345e639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6275e642, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c15e645, 34 },
    { 0, 0 },
    { 0xbdc5e647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a5e64a, 23 },
    { 0xac05e64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde05e64e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b5e651, 93 },
    { 0x6045e652, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6735e659, 98 },
    { 0x3855e65a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b5e65d, 98 },
    { 0xcb75e65e, 90 },
    { 0x7515e65e, 23 },
    { 0xf095e660, 15 },
    { 0xc1d5e661, 55 },
    { 0x14a5e662, 100 },
    { 0x4865e663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1365e666, 90 },
    { 0, 0 },
    { 0x67a5e668, 90 },
    { 0xe85e669, 43 },
    { 0xb4b5e66a, 55 },
    { 0xb6c5e66b, 90 },
    { 0xec95e66c, 98 },
    { 0x72b5e66b, 33 },
    { 0xbab5e66e, 98 },
    { 0, 0 },
    { 0xba55e670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf595e673, 90 },
    { 0xb965e673, 90 },
    { 0xa125e674, 31 },
    { 0x5635e676, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc035e679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155e67d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5e681, 80 },
    { 0, 0 },
    { 0x3ea5e683, 55 },
    { 0x6ad5e684, 23 },
    { 0x70e5e684, 19 },
    { 0x4f65e686, 43 },
    { 0xf8c5e685, 98 },
    { 0xc005e687, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd345e693, 67 },
    { 0x3cb5e694, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205e697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e5e69c, 66 },
    { 0x9a95e69d, 101 },
    { 0x3555e69c, 43 },
    { 0x4825e69f, 33 },
    { 0x7b75e6a0, 52 },
    { 0x47f5e69f, 33 },
    { 0, 0 },
    { 0x4e65e6a3, 76 },
    { 0x5e5e6a4, 98 },
    { 0xd285e6a4, 43 },
    { 0xccb5e6a5, 90 },
    { 0x2a75e6a7, 78 },
    { 0x32e5e6a7, 84 },
    { 0xb9f5e6a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c5e6b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad5e6b6, 90 },
    { 0xb265e6b7, 95 },
    { 0, 0 },
    { 0x5615e6b9, 33 },
    { 0xece5e6b9, 91 },
    { 0xafe5e6bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e6c0, 34 },
    { 0xf665e6c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5e6c6, 55 },
    { 0, 0 },
    { 0x1c55e6c8, 23 },
    { 0xb915e6c9, 17 },
    { 0xed65e6c9, 76 },
    { 0x2c75e6cb, 61 },
    { 0xd9b5e6cc, 98 },
    { 0x7ac5e6cd, 95 },
    { 0, 0 },
    { 0x4c25e6cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6195e6d2, 84 },
    { 0x125e6d3, 90 },
    { 0xb005e6d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3845e6e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d5e6e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db5e6e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a5e6ed, 33 },
    { 0x1355e6ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e5e6f2, 98 },
    { 0x5b55e6f2, 90 },
    { 0x79c5e6f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9535e706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b75e70a, 76 },
    { 0, 0 },
    { 0xe3c5e70c, 23 },
    { 0x1955e70d, 61 },
    { 0x4555e70d, 91 },
    { 0x14b5e70c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b5e718, 33 },
    { 0, 0 },
    { 0x43f5e71a, 95 },
    { 0xa305e71a, 52 },
    { 0x5e5e71a, 99 },
    { 0xf185e71d, 67 },
    { 0x9d55e71e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf05e722, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc625e725, 85 },
    { 0x2565e726, 46 },
    { 0x4e35e725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615e72b, 95 },
    { 0x6a25e72c, 85 },
    { 0x87b5e72d, 98 },
    { 0xf495e72e, 42 },
    { 0, 0 },
    { 0x1905e730, 98 },
    { 0x2aa5e731, 52 },
    { 0xd715e730, 33 },
    { 0xefe5e732, 95 },
    { 0x3205e734, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a85e738, 61 },
    { 0x68c5e739, 67 },
    { 0x9955e739, 88 },
    { 0xf635e73b, 61 },
    { 0x2d65e73b, 76 },
    { 0xdac5e73b, 34 },
    { 0x5b35e73e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd5e741, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5e746, 90 },
    { 0x5e15e747, 33 },
    { 0x8005e748, 95 },
    { 0xfe15e749, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5e750, 90 },
    { 0x7ae5e751, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf55e754, 91 },
    { 0xcf45e755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f15e758, 2 },
    { 0xaad5e759, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc325e75d, 85 },
    { 0, 0 },
    { 0x1625e75f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5e764, 23 },
    { 0xa5e5e765, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a5e76f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8675e773, 95 },
    { 0x9eb5e774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b05e779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e77e, 61 },
    { 0x7825e77f, 88 },
    { 0, 0 },
    { 0x3a35e781, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7545e785, 55 },
    { 0, 0 },
    { 0x3655e787, 66 },
    { 0, 0 },
    { 0x155e789, 23 },
    { 0xa5f5e789, 46 },
    { 0xcbb5e78a, 46 },
    { 0x6b85e78c, 98 },
    { 0x6275e78d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d35e791, 23 },
    { 0xa905e792, 93 },
    { 0x2105e792, 55 },
    { 0x3c25e794, 43 },
    { 0xc815e793, 85 },
    { 0x6be5e796, 61 },
    { 0x8305e791, 95 },
    { 0x2e85e798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e5e79c, 43 },
    { 0xb435e79d, 85 },
    { 0x8675e79c, 90 },
    { 0, 0 },
    { 0xdaf5e7a0, 33 },
    { 0x2625e7a1, 78 },
    { 0x3eb5e7a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c5e7a8, 23 },
    { 0x4c05e7a9, 95 },
    { 0xb3c5e7a9, 23 },
    { 0xdc95e7ab, 95 },
    { 0x76d5e7a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd5e7c0, 76 },
    { 0, 0 },
    { 0x7605e7c2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05e7c8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d85e7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9955e7d7, 23 },
    { 0x91e5e7d7, 91 },
    { 0x385e7d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f5e7df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6985e7e3, 84 },
    { 0x93d5e7e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5925e7f8, 52 },
    { 0x5b85e7f8, 34 },
    { 0x9d05e7fa, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d85e7fd, 90 },
    { 0, 0 },
    { 0x28d5e7ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a5e807, 90 },
    { 0x97b5e808, 93 },
    { 0xd6d5e808, 93 },
    { 0xb3f5e80a, 61 },
    { 0x3015e80a, 46 },
    { 0x1bc5e80c, 23 },
    { 0x5aa5e80d, 99 },
    { 0x56a5e807, 34 },
    { 0x4115e80f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c5e812, 34 },
    { 0x8ee5e812, 98 },
    { 0xf0b5e814, 81 },
    { 0xb7d5e812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1815e819, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f5e820, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8185e823, 55 },
    { 0x7cd5e824, 33 },
    { 0x8865e825, 95 },
    { 0xb705e825, 85 },
    { 0x5d55e824, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f5e82d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb25e834, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a65e837, 90 },
    { 0x2c5e837, 90 },
    { 0xfd15e839, 98 },
    { 0x4b25e838, 69 },
    { 0x845e83b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb5e83e, 68 },
    { 0x4d85e83f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af5e842, 61 },
    { 0xfac5e843, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a5e84c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1475e853, 33 },
    { 0x1285e854, 75 },
    { 0x8bf5e855, 33 },
    { 0, 0 },
    { 0x61a5e857, 90 },
    { 0x9425e857, 33 },
    { 0x3d85e859, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb175e85c, 90 },
    { 0xf785e85c, 61 },
    { 0x4845e85d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc5e866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b5e869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885e86d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd115e873, 99 },
    { 0x6e15e874, 61 },
    { 0x7675e875, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f5e879, 46 },
    { 0x6bc5e879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c55e87e, 93 },
    { 0xaf25e87f, 43 },
    { 0x5005e880, 23 },
    { 0, 0 },
    { 0xc475e882, 91 },
    { 0xb695e883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a5e88a, 46 },
    { 0x3c85e88a, 34 },
    { 0xba95e88a, 98 },
    { 0, 0 },
    { 0x5ff5e88e, 33 },
    { 0x3045e88f, 98 },
    { 0, 0 },
    { 0x2e75e891, 48 },
    { 0xbf55e892, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4745e899, 85 },
    { 0xefa5e89a, 98 },
    { 0x2215e89b, 23 },
    { 0, 0 },
    { 0x445e89d, 61 },
    { 0x7eb5e89e, 100 },
    { 0xe1f5e89f, 68 },
    { 0x5435e8a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3935e8af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5e8b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d5e8b7, 33 },
    { 0x62c5e8b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6905e8be, 98 },
    { 0x3d45e8bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6635e8c2, 68 },
    { 0xe3d5e8c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5675e8c8, 100 },
    { 0x1ec5e8c9, 61 },
    { 0x54f5e8c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe695e8ce, 92 },
    { 0xa715e8ce, 98 },
    { 0x1f05e8d0, 33 },
    { 0, 0 },
    { 0x4575e8d2, 61 },
    { 0x9b5e8d2, 98 },
    { 0x1d75e8d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff75e8d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e15e8dc, 55 },
    { 0, 0 },
    { 0x9dd5e8de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc15e8e1, 33 },
    { 0x8405e8e2, 61 },
    { 0x8c05e8e3, 61 },
    { 0x8db5e8e2, 98 },
    { 0, 0 },
    { 0xa305e8e6, 55 },
    { 0, 0 },
    { 0xafb5e8e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a5e8ed, 91 },
    { 0x65b5e8ee, 93 },
    { 0, 0 },
    { 0xd3d5e8f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea5e8f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d5e8f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e8fc, 61 },
    { 0, 0 },
    { 0xc025e8fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae05e904, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x235e907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a5e90e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4465e916, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb5e923, 64 },
    { 0x3565e924, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935e927, 81 },
    { 0x81a5e927, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f55e92d, 34 },
    { 0, 0 },
    { 0x2f85e92f, 47 },
    { 0, 0 },
    { 0x9075e931, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a95e936, 51 },
    { 0, 0 },
    { 0x7565e938, 83 },
    { 0, 0 },
    { 0xa975e93a, 93 },
    { 0, 0 },
    { 0x23d5e93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195e940, 52 },
    { 0xae65e941, 76 },
    { 0xc4f5e940, 33 },
    { 0x9255e943, 52 },
    { 0, 0 },
    { 0x5975e945, 33 },
    { 0, 0 },
    { 0x6965e947, 96 },
    { 0x31c5e948, 90 },
    { 0xe065e949, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa825e94c, 69 },
    { 0x8e5e94d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd635e951, 100 },
    { 0xe035e952, 93 },
    { 0x51d5e952, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d5e956, 33 },
    { 0, 0 },
    { 0x29e5e958, 61 },
    { 0xcd55e958, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6215e95f, 98 },
    { 0xe825e960, 61 },
    { 0, 0 },
    { 0xff95e962, 61 },
    { 0xfac5e963, 52 },
    { 0x22a5e964, 52 },
    { 0xf9b5e963, 61 },
    { 0x7475e966, 23 },
    { 0, 0 },
    { 0xbff5e968, 55 },
    { 0, 0 },
    { 0x12f5e96a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe135e96d, 61 },
    { 0x6d05e96e, 61 },
    { 0x9df5e96f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd445e972, 38 },
    { 0x6605e973, 75 },
    { 0x1c75e974, 90 },
    { 0xfa35e974, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a5e978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd525e981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f55e984, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe5e987, 46 },
    { 0, 0 },
    { 0xb955e989, 77 },
    { 0, 0 },
    { 0x8f45e98b, 98 },
    { 0x3475e98c, 95 },
    { 0x8225e98d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa5e990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf895e994, 95 },
    { 0x3585e995, 90 },
    { 0xd755e994, 98 },
    { 0, 0 },
    { 0xd875e998, 52 },
    { 0x4255e999, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4295e9a0, 90 },
    { 0x40e5e9a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5e9a4, 77 },
    { 0x745e9a5, 99 },
    { 0x5835e9a6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7505e9ab, 87 },
    { 0xe4e5e9ac, 23 },
    { 0x2355e9ac, 91 },
    { 0, 0 },
    { 0xbd45e9af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5155e9b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd5e9bd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd605e9c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a5e9c6, 55 },
    { 0x6cf5e9c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d55e9cd, 98 },
    { 0, 0 },
    { 0x3c05e9cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc65e9db, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e9e1, 43 },
    { 0, 0 },
    { 0x9485e9e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5e9ea, 76 },
    { 0, 0 },
    { 0x33e5e9ec, 61 },
    { 0x4e65e9ed, 44 },
    { 0, 0 },
    { 0xa615e9ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5e9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d5e9fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb825e9fe, 33 },
    { 0xaa75e9ff, 93 },
    { 0, 0 },
    { 0xa65ea01, 55 },
    { 0x7fd5ea01, 95 },
    { 0x9375ea03, 34 },
    { 0x7715ea04, 46 },
    { 0xfb95ea05, 33 },
    { 0x4ed5ea06, 95 },
    { 0xfaf5ea01, 95 },
    { 0xfd35ea08, 88 },
    { 0x3885ea09, 33 },
    { 0, 0 },
    { 0x7985ea0b, 98 },
    { 0x84f5ea0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce5ea0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc115ea12, 43 },
    { 0, 0 },
    { 0xbc5ea14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b5ea19, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd955ea1e, 52 },
    { 0xc175ea1e, 99 },
    { 0, 0 },
    { 0xa1b5ea21, 90 },
    { 0x5155ea22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1155ea29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf05ea2c, 90 },
    { 0x9325ea2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5ea36, 55 },
    { 0xaf5ea37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6175ea3b, 84 },
    { 0xf475ea3b, 34 },
    { 0x8b5ea3d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7525ea46, 61 },
    { 0x86d5ea47, 91 },
    { 0x2a15ea48, 98 },
    { 0, 0 },
    { 0x8035ea4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab5ea4f, 78 },
    { 0xb4f5ea4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f5ea54, 78 },
    { 0xf455ea55, 55 },
    { 0x985ea54, 85 },
    { 0xa695ea54, 77 },
    { 0x1145ea58, 98 },
    { 0x9e45ea56, 83 },
    { 0xeb15ea5a, 23 },
    { 0xa55ea5a, 55 },
    { 0xa525ea5a, 55 },
    { 0xb685ea5d, 99 },
    { 0xe5f5ea5d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c75ea63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4045ea66, 24 },
    { 0xda5ea67, 61 },
    { 0x6385ea67, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d5ea6b, 84 },
    { 0x4865ea6b, 23 },
    { 0, 0 },
    { 0xa285ea6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e5ea71, 84 },
    { 0xffc5ea71, 77 },
    { 0xb045ea73, 34 },
    { 0x91b5ea74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a35ea77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd15ea7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf555ea7e, 90 },
    { 0x18d5ea7f, 81 },
    { 0x6615ea7f, 33 },
    { 0xc155ea81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e55ea84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5485ea8b, 61 },
    { 0xa505ea8c, 33 },
    { 0x5cc5ea8d, 61 },
    { 0x4635ea8d, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5ea91, 55 },
    { 0xc315ea92, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d95ea9b, 78 },
    { 0xf8f5ea9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa5ea9f, 95 },
    { 0x8335eaa0, 90 },
    { 0xb385eaa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b55eaa8, 90 },
    { 0x8d65eaa8, 33 },
    { 0, 0 },
    { 0x5015eaab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf105eaae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd415eab1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55eabc, 80 },
    { 0x76a5eabc, 90 },
    { 0xf4f5eabe, 95 },
    { 0xf615eabf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4945eac2, 43 },
    { 0x5855eac3, 43 },
    { 0x8d05eac2, 95 },
    { 0xa325eac3, 98 },
    { 0x8d5eac3, 55 },
    { 0, 0 },
    { 0xe595eac8, 23 },
    { 0x2d25eac9, 91 },
    { 0xeff5eac9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2035ead9, 33 },
    { 0x4015eada, 100 },
    { 0x7165eadb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb395eae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b75eae4, 96 },
    { 0, 0 },
    { 0x1545eae6, 84 },
    { 0x3905eae7, 100 },
    { 0xd575eae6, 61 },
    { 0x6d5eae9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a5eaed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6485eaf0, 91 },
    { 0x62b5eaf1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5eaf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2545eb02, 34 },
    { 0x7ac5eb02, 34 },
    { 0, 0 },
    { 0x33e5eb05, 76 },
    { 0x575eb05, 23 },
    { 0x5d5eb07, 76 },
    { 0x20f5eb08, 98 },
    { 0, 0 },
    { 0xda65eb0a, 46 },
    { 0x1ac5eb0b, 95 },
    { 0xe555eb0b, 17 },
    { 0xcda5eb0d, 75 },
    { 0xfc85eb0a, 98 },
    { 0x2275eb0f, 91 },
    { 0x5b45eb0b, 43 },
    { 0xa835eb11, 33 },
    { 0x3de5eb12, 46 },
    { 0x6695eb13, 95 },
    { 0xb8b5eb0f, 87 },
    { 0, 0 },
    { 0xb935eb16, 33 },
    { 0x75e5eb17, 33 },
    { 0xbc95eb17, 100 },
    { 0x48b5eb17, 61 },
    { 0, 0 },
    { 0x1365eb1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf925eb21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a15eb27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435eb2b, 33 },
    { 0, 0 },
    { 0x87f5eb2d, 43 },
    { 0x1235eb2d, 95 },
    { 0xf1b5eb2f, 34 },
    { 0x7ac5eb30, 90 },
    { 0x4745eb31, 23 },
    { 0x1ce5eb2f, 75 },
    { 0xf785eb31, 34 },
    { 0x2795eb34, 90 },
    { 0xca05eb34, 34 },
    { 0xeed5eb36, 33 },
    { 0xcf75eb2e, 93 },
    { 0, 0 },
    { 0x45e5eb39, 91 },
    { 0x9965eb39, 75 },
    { 0xe515eb3b, 55 },
    { 0, 0 },
    { 0xe195eb3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe265eb40, 61 },
    { 0x7685eb41, 55 },
    { 0xbee5eb42, 90 },
    { 0xb4f5eb43, 90 },
    { 0x1365eb44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d5eb47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a5eb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e5eb51, 61 },
    { 0x4715eb52, 87 },
    { 0x485eb53, 90 },
    { 0, 0 },
    { 0xfa35eb55, 98 },
    { 0x19a5eb56, 33 },
    { 0x9445eb56, 98 },
    { 0xa135eb57, 98 },
    { 0xe625eb59, 74 },
    { 0x1005eb5a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c35eb60, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5eb63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d95eb67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5eb6c, 52 },
    { 0xd8c5eb6d, 34 },
    { 0x8bd5eb6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f5eb72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a5eb75, 61 },
    { 0, 0 },
    { 0xf605eb77, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd025eb7a, 61 },
    { 0x75eb7b, 75 },
    { 0x74c5eb7a, 33 },
    { 0xe705eb7b, 34 },
    { 0x3275eb7e, 87 },
    { 0x8915eb7b, 95 },
    { 0x69c5eb7c, 33 },
    { 0xf365eb81, 43 },
    { 0xef45eb82, 66 },
    { 0x3215eb83, 91 },
    { 0x89a5eb83, 55 },
    { 0, 0 },
    { 0x3c35eb86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb55eb89, 76 },
    { 0xe435eb8a, 23 },
    { 0xfd5eb8a, 76 },
    { 0x3505eb8a, 84 },
    { 0xb545eb8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e45eb90, 99 },
    { 0xf0e5eb91, 77 },
    { 0x20e5eb92, 98 },
    { 0x5365eb93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b5eb97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b15eb9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7425eba2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f05eba8, 91 },
    { 0xf125eba8, 40 },
    { 0xf15eba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6815ebae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb865ebb4, 61 },
    { 0, 0 },
    { 0x3a05ebb6, 61 },
    { 0xfa15ebb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2145ebbc, 85 },
    { 0xb965ebbd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7975ebc2, 90 },
    { 0x7d45ebc3, 90 },
    { 0, 0 },
    { 0x6d05ebc5, 93 },
    { 0, 0 },
    { 0x2c5ebc7, 98 },
    { 0x71c5ebc8, 33 },
    { 0x7405ebc8, 61 },
    { 0, 0 },
    { 0xaeb5ebcb, 95 },
    { 0x3fa5ebcb, 67 },
    { 0x8205ebcc, 61 },
    { 0, 0 },
    { 0xbe05ebcf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa195ebd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2525ebdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d5ebe1, 84 },
    { 0, 0 },
    { 0x8e95ebe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf305ebe6, 78 },
    { 0x9db5ebe6, 95 },
    { 0xf765ebe8, 61 },
    { 0, 0 },
    { 0xa875ebea, 61 },
    { 0xec35ebea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c15ebee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec55ebf3, 90 },
    { 0x7485ebf4, 55 },
    { 0x9665ebf4, 66 },
    { 0x31c5ebf4, 61 },
    { 0xe915ebf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9595ebfa, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x5395ebfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e5ec02, 90 },
    { 0xa735ec02, 33 },
    { 0, 0 },
    { 0xbb85ec05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3275ec09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1885ec0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa585ec10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a5ec14, 95 },
    { 0, 0 },
    { 0x155ec16, 95 },
    { 0x28d5ec16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4665ec1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb5ec27, 78 },
    { 0, 0 },
    { 0x2e85ec29, 100 },
    { 0x51b5ec2a, 75 },
    { 0, 0 },
    { 0xade5ec2c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c35ec31, 98 },
    { 0x74b5ec31, 90 },
    { 0xe615ec33, 23 },
    { 0x8885ec34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba5ec3c, 76 },
    { 0x3c25ec3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4815ec40, 46 },
    { 0xb655ec41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe465ec4a, 98 },
    { 0xc745ec4b, 95 },
    { 0, 0 },
    { 0x9b45ec4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f95ec52, 76 },
    { 0, 0 },
    { 0xc485ec54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c5ec58, 33 },
    { 0x3b85ec58, 90 },
    { 0xdf95ec59, 84 },
    { 0xf2e5ec5b, 33 },
    { 0, 0 },
    { 0x3d15ec5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c05ec62, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc835ec67, 61 },
    { 0, 0 },
    { 0x7515ec69, 33 },
    { 0xea65ec69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc135ec6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05ec70, 61 },
    { 0x3595ec70, 90 },
    { 0x8585ec72, 90 },
    { 0x20a5ec73, 61 },
    { 0, 0 },
    { 0x3e85ec75, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc225ec78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85ec7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a5ec80, 23 },
    { 0xdcb5ec81, 61 },
    { 0xc4e5ec82, 61 },
    { 0xf0e5ec83, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d5ec87, 67 },
    { 0xf765ec87, 34 },
    { 0xbe05ec88, 76 },
    { 0, 0 },
    { 0x58b5ec8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1105ec93, 90 },
    { 0x3185ec94, 99 },
    { 0x21b5ec95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb145ec98, 46 },
    { 0x3b85ec99, 34 },
    { 0x3865ec99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2105eca0, 76 },
    { 0x7705eca0, 33 },
    { 0x6325eca2, 98 },
    { 0, 0 },
    { 0x4825eca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e05ecaa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e15ecae, 98 },
    { 0x88a5ecae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb945ecb4, 90 },
    { 0xa095ecb4, 100 },
    { 0xe7b5ecb6, 33 },
    { 0x2315ecb4, 95 },
    { 0, 0 },
    { 0xd115ecb9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf445ecbe, 23 },
    { 0, 0 },
    { 0xd895ecc0, 93 },
    { 0x6fc5ecc1, 61 },
    { 0, 0 },
    { 0x1275ecc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f95ecc7, 43 },
    { 0xc8d5ecc8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1805eccb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc305ecce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f5ecd1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d5ecd7, 55 },
    { 0xfe05ecd7, 61 },
    { 0, 0 },
    { 0xbe85ecda, 23 },
    { 0, 0 },
    { 0xae25ecdc, 37 },
    { 0x4405ecdd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6535ece1, 90 },
    { 0x9bb5ece2, 98 },
    { 0x92e5ece2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b5ece7, 43 },
    { 0xd655ece7, 93 },
    { 0xc9e5ece9, 55 },
    { 0, 0 },
    { 0x8dd5eceb, 85 },
    { 0x9ce5ecec, 33 },
    { 0x5ab5eced, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc025ecf6, 34 },
    { 0xed45ecf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de5ecfa, 98 },
    { 0x90c5ecfb, 23 },
    { 0x62d5ecfc, 43 },
    { 0xb7c5ecfc, 95 },
    { 0x1ca5ecfa, 90 },
    { 0x5385ecfe, 23 },
    { 0x65a5ed00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2465ed05, 98 },
    { 0x8715ed06, 46 },
    { 0xaf95ed07, 61 },
    { 0x5685ed07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1365ed0b, 90 },
    { 0x37f5ed0c, 98 },
    { 0, 0 },
    { 0xe715ed0e, 90 },
    { 0xe3d5ed0f, 98 },
    { 0xf255ed10, 90 },
    { 0, 0 },
    { 0xda85ed12, 85 },
    { 0, 0 },
    { 0xa685ed14, 61 },
    { 0xfb75ed15, 33 },
    { 0, 0 },
    { 0xd2d5ed17, 77 },
    { 0, 0 },
    { 0x7005ed19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c5ed1e, 61 },
    { 0, 0 },
    { 0x9ba5ed20, 33 },
    { 0, 0 },
    { 0x7d55ed22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c5ed29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc5ed2c, 75 },
    { 0x1695ed2c, 33 },
    { 0x88c5ed2d, 61 },
    { 0x8555ed2f, 61 },
    { 0, 0 },
    { 0x1ec5ed31, 98 },
    { 0x77c5ed31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8965ed35, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c5ed3a, 23 },
    { 0, 0 },
    { 0x26d5ed3c, 34 },
    { 0, 0 },
    { 0xde75ed3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4645ed43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba95ed4b, 98 },
    { 0, 0 },
    { 0x95f5ed4d, 34 },
    { 0x565ed4e, 85 },
    { 0x2005ed4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e5ed53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1385ed57, 33 },
    { 0, 0 },
    { 0x715ed59, 22 },
    { 0x7cb5ed5a, 76 },
    { 0x6185ed5a, 85 },
    { 0x5365ed5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb335ed60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe5ed64, 100 },
    { 0x93f5ed64, 65 },
    { 0x9f5ed66, 23 },
    { 0, 0 },
    { 0x4925ed68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf5ed6b, 95 },
    { 0, 0 },
    { 0xc85ed6d, 77 },
    { 0, 0 },
    { 0x39b5ed6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd5ed72, 84 },
    { 0xd6e5ed72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7545ed7c, 83 },
    { 0, 0 },
    { 0x2215ed7e, 34 },
    { 0x8705ed7f, 61 },
    { 0xe355ed80, 98 },
    { 0x2dc5ed81, 34 },
    { 0x3645ed81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf205ed85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd645ed89, 61 },
    { 0x66f5ed8a, 61 },
    { 0x6c85ed8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85ed91, 90 },
    { 0x3635ed92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd055ed95, 55 },
    { 0xcf45ed96, 90 },
    { 0x5ed97, 98 },
    { 0x48d5ed98, 55 },
    { 0x4c75ed99, 87 },
    { 0, 0 },
    { 0xa755ed9b, 23 },
    { 0x98f5ed9c, 55 },
    { 0x69b5ed9b, 33 },
    { 0xc765ed9b, 83 },
    { 0x1255ed9f, 95 },
    { 0, 0 },
    { 0x9155eda1, 87 },
    { 0, 0 },
    { 0x4f5eda3, 53 },
    { 0xd145eda4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5475edaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7915edad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3855edb3, 91 },
    { 0xc7a5edb4, 90 },
    { 0xad75edb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e5edb9, 90 },
    { 0x1725edba, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb5edbe, 77 },
    { 0xe915edbe, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7755edc5, 76 },
    { 0x8ae5edc5, 84 },
    { 0xb8a5edc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d45edca, 94 },
    { 0, 0 },
    { 0x5165edcc, 34 },
    { 0x88f5edcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4045edd0, 90 },
    { 0xf305edd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c55edd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e5edd9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d15eddc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d35ede0, 33 },
    { 0x585ede1, 90 },
    { 0xd845ede2, 23 },
    { 0xd315ede2, 98 },
    { 0x93e5ede4, 88 },
    { 0xf6f5ede0, 61 },
    { 0, 0 },
    { 0x7c65ede7, 33 },
    { 0xa8d5ede7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6245edeb, 87 },
    { 0, 0 },
    { 0xbf65eded, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d15edf3, 75 },
    { 0, 0 },
    { 0xab25edf5, 85 },
    { 0x4385edf6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9765edfb, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3875ee00, 46 },
    { 0xbec5ee00, 61 },
    { 0x7195ee02, 33 },
    { 0x1855ee03, 90 },
    { 0xf615ee04, 76 },
    { 0, 0 },
    { 0x1cb5ee06, 34 },
    { 0, 0 },
    { 0x1a35ee08, 33 },
    { 0x1475ee09, 99 },
    { 0xa1c5ee08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7475ee10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e45ee13, 34 },
    { 0xc995ee14, 34 },
    { 0x1a95ee15, 34 },
    { 0x9125ee15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f85ee19, 76 },
    { 0x7a95ee1a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15ee1e, 23 },
    { 0x3375ee1f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a5ee23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4105ee26, 61 },
    { 0xf265ee27, 23 },
    { 0x5415ee27, 99 },
    { 0, 0 },
    { 0x2005ee2a, 95 },
    { 0x4a35ee2b, 23 },
    { 0x1675ee2b, 99 },
    { 0x7a75ee2b, 90 },
    { 0x1c45ee2e, 90 },
    { 0x59c5ee2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc185ee33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac05ee36, 68 },
    { 0x2715ee37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d5ee3c, 43 },
    { 0x5035ee3c, 90 },
    { 0xaf55ee3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a5ee41, 90 },
    { 0xadd5ee42, 95 },
    { 0x6b95ee43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e85ee47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace5ee5a, 90 },
    { 0xdaa5ee5b, 64 },
    { 0x9365ee5b, 95 },
    { 0x7a35ee5b, 95 },
    { 0xfc65ee5e, 33 },
    { 0, 0 },
    { 0xf875ee60, 98 },
    { 0x9cb5ee60, 55 },
    { 0, 0 },
    { 0x3835ee63, 77 },
    { 0, 0 },
    { 0x8ca5ee65, 68 },
    { 0x4485ee66, 34 },
    { 0x61d5ee67, 84 },
    { 0xfbe5ee68, 34 },
    { 0xa55ee69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa795ee6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e5ee71, 46 },
    { 0xdba5ee71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc25ee76, 98 },
    { 0x1265ee77, 61 },
    { 0, 0 },
    { 0x9385ee79, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd5ee7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc155ee7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x395ee82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb605ee88, 23 },
    { 0, 0 },
    { 0xc475ee8a, 91 },
    { 0x1425ee8b, 95 },
    { 0, 0 },
    { 0x8465ee8d, 77 },
    { 0x3b65ee8e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff15ee97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d85ee9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4305ee9f, 23 },
    { 0, 0 },
    { 0xa275eea1, 90 },
    { 0x3945eea2, 98 },
    { 0x6885eea1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a5eea6, 98 },
    { 0xa175eea7, 55 },
    { 0x33c5eea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615eeab, 53 },
    { 0x45b5eeab, 98 },
    { 0x4fb5eeac, 95 },
    { 0, 0 },
    { 0x6425eeaf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e05eeb3, 95 },
    { 0, 0 },
    { 0x81a5eeb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c5eeba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf895eebd, 90 },
    { 0, 0 },
    { 0x7db5eebf, 33 },
    { 0, 0 },
    { 0x48f5eec1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4175eec4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba65eec8, 87 },
    { 0x3f35eec9, 61 },
    { 0xc195eeca, 84 },
    { 0xa185eec9, 61 },
    { 0, 0 },
    { 0x6f25eecd, 33 },
    { 0, 0 },
    { 0xa9b5eecf, 69 },
    { 0x34f5eecf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e15eed3, 61 },
    { 0x9e35eed4, 23 },
    { 0xc3a5eed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7635eee4, 90 },
    { 0x1185eee5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe585eeeb, 43 },
    { 0xf445eeec, 76 },
    { 0x1b05eeeb, 40 },
    { 0, 0 },
    { 0x29d5eeef, 17 },
    { 0x5695eef0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4785eef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e5eef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2105eef9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda95eefd, 90 },
    { 0x8065eefe, 90 },
    { 0xcad5eeff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc55ef06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9875ef0d, 91 },
    { 0x29f5ef0e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb875ef19, 76 },
    { 0xac05ef1a, 33 },
    { 0xd695ef1b, 23 },
    { 0, 0 },
    { 0xe095ef1d, 33 },
    { 0, 0 },
    { 0x5705ef1f, 90 },
    { 0x7e05ef20, 23 },
    { 0xfc35ef1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3325ef25, 76 },
    { 0, 0 },
    { 0x4355ef27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b5ef2b, 66 },
    { 0, 0 },
    { 0xa2c5ef2d, 90 },
    { 0x685ef2e, 33 },
    { 0x3d5ef2d, 55 },
    { 0xbec5ef30, 33 },
    { 0x9105ef2d, 95 },
    { 0, 0 },
    { 0x85b5ef33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45ef37, 55 },
    { 0xd625ef38, 78 },
    { 0xe2a5ef39, 76 },
    { 0xe405ef39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d5ef3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f5ef42, 93 },
    { 0xc1c5ef42, 90 },
    { 0x90c5ef43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a35ef48, 78 },
    { 0x265ef49, 55 },
    { 0x45d5ef4a, 99 },
    { 0xe445ef4b, 23 },
    { 0xa625ef4b, 95 },
    { 0x3fa5ef4d, 33 },
    { 0x3ef5ef4d, 17 },
    { 0xde65ef4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd695ef52, 76 },
    { 0, 0 },
    { 0x7925ef54, 46 },
    { 0x1925ef55, 52 },
    { 0xbd45ef56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc865ef60, 61 },
    { 0x9d85ef61, 17 },
    { 0x3945ef62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3815ef65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f55ef6a, 95 },
    { 0x185ef6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b75ef6f, 33 },
    { 0, 0 },
    { 0x6b35ef71, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5ef76, 90 },
    { 0, 0 },
    { 0xd025ef78, 61 },
    { 0xde35ef79, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb755ef7e, 33 },
    { 0x9b45ef7e, 95 },
    { 0x82a5ef7f, 61 },
    { 0, 0 },
    { 0x7fc5ef82, 23 },
    { 0x4235ef83, 23 },
    { 0x615ef84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe665ef8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf295ef90, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc5ef9e, 61 },
    { 0xdf85ef9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5efa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef65efac, 98 },
    { 0xd3d5efad, 55 },
    { 0xd9b5efae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa085efb2, 61 },
    { 0, 0 },
    { 0xd235efb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e5efbc, 84 },
    { 0, 0 },
    { 0x2e45efbe, 23 },
    { 0xcf85efbf, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5815efc6, 87 },
    { 0, 0 },
    { 0xabf5efc8, 34 },
    { 0x9655efc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85efd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b55efd4, 56 },
    { 0xff75efd4, 46 },
    { 0xdbd5efd6, 95 },
    { 0, 0 },
    { 0xd9a5efd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc5efdf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce55efe3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f15efe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3885efea, 33 },
    { 0x6bb5efeb, 33 },
    { 0x4055efeb, 95 },
    { 0, 0 },
    { 0x44c5efee, 23 },
    { 0xe5e5efef, 95 },
    { 0x11e5efef, 95 },
    { 0x4835efee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf335eff6, 95 },
    { 0x9455eff6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc475effb, 98 },
    { 0, 0 },
    { 0x6045effd, 90 },
    { 0x73e5effe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f5f00a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b5f017, 90 },
    { 0x8285f017, 64 },
    { 0, 0 },
    { 0xcd75f01a, 90 },
    { 0x7ba5f01b, 98 },
    { 0x9325f01c, 61 },
    { 0xca15f01d, 84 },
    { 0x5405f01c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445f022, 98 },
    { 0x7065f023, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5485f028, 95 },
    { 0x8d45f029, 76 },
    { 0xea95f028, 99 },
    { 0x6c75f02b, 33 },
    { 0x7d65f02c, 90 },
    { 0x7285f02d, 76 },
    { 0x6765f02c, 93 },
    { 0x7be5f02e, 66 },
    { 0x1d55f030, 66 },
    { 0xdad5f031, 76 },
    { 0x46f5f032, 90 },
    { 0xad55f033, 66 },
    { 0, 0 },
    { 0xb445f035, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d5f039, 93 },
    { 0x4595f03a, 33 },
    { 0, 0 },
    { 0x99f5f03c, 95 },
    { 0xbe65f03d, 55 },
    { 0xf9a5f03e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe445f041, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c5f045, 96 },
    { 0xc75f046, 61 },
    { 0x1c15f046, 76 },
    { 0x1615f048, 85 },
    { 0xceb5f049, 33 },
    { 0, 0 },
    { 0xad35f04b, 91 },
    { 0, 0 },
    { 0x1465f04d, 61 },
    { 0xcfe5f04e, 95 },
    { 0x44d5f04e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab5f052, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3505f056, 61 },
    { 0xf595f057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6975f05d, 61 },
    { 0, 0 },
    { 0x6ea5f05f, 39 },
    { 0, 0 },
    { 0x405f061, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b5f065, 23 },
    { 0x7085f065, 36 },
    { 0, 0 },
    { 0x6f45f068, 34 },
    { 0, 0 },
    { 0x665f06a, 91 },
    { 0x785f06a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b5f06e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb825f072, 93 },
    { 0x43c5f072, 78 },
    { 0, 0 },
    { 0x7825f075, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5f078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc075f07d, 33 },
    { 0, 0 },
    { 0xe9f5f07f, 97 },
    { 0x7d15f080, 93 },
    { 0x9dd5f081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1795f08a, 90 },
    { 0xcc45f08a, 98 },
    { 0x805f08c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c5f08f, 52 },
    { 0, 0 },
    { 0x6f45f091, 90 },
    { 0x9ba5f091, 43 },
    { 0x3665f093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3985f096, 90 },
    { 0x6fc5f097, 61 },
    { 0x365f096, 80 },
    { 0x1ea5f099, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b25f09c, 90 },
    { 0, 0 },
    { 0x1d35f09e, 99 },
    { 0, 0 },
    { 0xba65f0a0, 69 },
    { 0x2235f0a1, 33 },
    { 0, 0 },
    { 0x7215f0a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2815f0a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3325f0aa, 90 },
    { 0x2f5f0ab, 81 },
    { 0x82f5f0aa, 76 },
    { 0, 0 },
    { 0x215f0ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275f0b2, 81 },
    { 0, 0 },
    { 0x88e5f0b4, 23 },
    { 0, 0 },
    { 0x97d5f0b6, 93 },
    { 0xb905f0b7, 100 },
    { 0xebc5f0b7, 95 },
    { 0, 0 },
    { 0x3cd5f0ba, 33 },
    { 0, 0 },
    { 0x2535f0bc, 38 },
    { 0x2985f0bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e15f0c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a5f0c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d5f0cc, 90 },
    { 0, 0 },
    { 0xd415f0ce, 90 },
    { 0x4e85f0cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab5f0d4, 98 },
    { 0x1df5f0d5, 46 },
    { 0, 0 },
    { 0xc765f0d7, 61 },
    { 0, 0 },
    { 0xb3c5f0d9, 78 },
    { 0x6975f0da, 40 },
    { 0xfd75f0da, 95 },
    { 0xf365f0da, 23 },
    { 0xcc5f0dd, 91 },
    { 0xfe55f0dd, 65 },
    { 0x5d5f0df, 34 },
    { 0x2785f0dd, 44 },
    { 0x7a5f0e1, 33 },
    { 0x9445f0e2, 33 },
    { 0x8425f0e2, 77 },
    { 0xd125f0e4, 78 },
    { 0, 0 },
    { 0xa8f5f0e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb65f0e9, 59 },
    { 0xe3c5f0ea, 23 },
    { 0x2e55f0e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4505f0f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9395f0f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8765f100, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9045f103, 76 },
    { 0x6825f103, 61 },
    { 0, 0 },
    { 0x4f65f106, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa5f109, 33 },
    { 0xe085f109, 55 },
    { 0x8f5f109, 95 },
    { 0x8155f10c, 98 },
    { 0, 0 },
    { 0x93f5f10e, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe925f111, 43 },
    { 0xe965f112, 61 },
    { 0x3a15f113, 33 },
    { 0x1d55f113, 46 },
    { 0xaea5f113, 96 },
    { 0, 0 },
    { 0xd4f5f117, 46 },
    { 0, 0 },
    { 0xd3a5f119, 67 },
    { 0, 0 },
    { 0x6055f11b, 90 },
    { 0, 0 },
    { 0x81f5f11d, 56 },
    { 0, 0 },
    { 0x97f5f11f, 87 },
    { 0, 0 },
    { 0xa395f121, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95f124, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf5f127, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff85f12b, 66 },
    { 0xe925f12b, 76 },
    { 0, 0 },
    { 0x6b45f12e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b5f133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3505f136, 90 },
    { 0x7415f137, 34 },
    { 0, 0 },
    { 0x8fd5f139, 75 },
    { 0x1d55f13a, 90 },
    { 0, 0 },
    { 0x4e15f13c, 90 },
    { 0x9ba5f13c, 61 },
    { 0x9895f13d, 98 },
    { 0xdc85f13f, 43 },
    { 0xafb5f140, 76 },
    { 0x7cb5f141, 33 },
    { 0, 0 },
    { 0x5f35f143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6545f146, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5f14b, 90 },
    { 0xb835f14c, 83 },
    { 0xac15f14d, 98 },
    { 0x6845f14b, 23 },
    { 0x9cf5f14d, 90 },
    { 0, 0 },
    { 0x715f151, 33 },
    { 0x2a25f151, 76 },
    { 0, 0 },
    { 0xce85f154, 43 },
    { 0xe835f154, 90 },
    { 0x93d5f156, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e5f15b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec75f166, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd5f169, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f5f16e, 95 },
    { 0xbf95f16f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e5f175, 76 },
    { 0x8175f176, 33 },
    { 0, 0 },
    { 0xc9b5f178, 23 },
    { 0x1be5f179, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5f17c, 34 },
    { 0x5475f17c, 61 },
    { 0xfe75f17c, 33 },
    { 0xf6f5f17f, 67 },
    { 0x4935f17f, 84 },
    { 0xb055f181, 17 },
    { 0xf735f182, 76 },
    { 0xef5f183, 42 },
    { 0, 0 },
    { 0xcfa5f185, 84 },
    { 0, 0 },
    { 0x2535f187, 33 },
    { 0x30d5f188, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a5f18b, 95 },
    { 0, 0 },
    { 0xb865f18d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c65f196, 33 },
    { 0x9275f197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b5f19c, 23 },
    { 0, 0 },
    { 0xc4e5f19e, 90 },
    { 0xab35f19f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc035f1a2, 33 },
    { 0x8435f1a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee5f1a7, 17 },
    { 0, 0 },
    { 0x5265f1a9, 95 },
    { 0xfcf5f1a9, 61 },
    { 0x1165f1ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475f1ae, 99 },
    { 0xd105f1af, 33 },
    { 0x2b95f1b0, 43 },
    { 0x60b5f1b1, 99 },
    { 0x4c15f1b1, 90 },
    { 0x76f5f1af, 87 },
    { 0x2ad5f1b4, 61 },
    { 0x6fc5f1b4, 95 },
    { 0x2bb5f1b5, 95 },
    { 0, 0 },
    { 0xabc5f1b8, 99 },
    { 0, 0 },
    { 0xd445f1ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd55f1c3, 55 },
    { 0, 0 },
    { 0xcf75f1c5, 33 },
    { 0, 0 },
    { 0x1905f1c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc35f1ce, 78 },
    { 0xc595f1cf, 76 },
    { 0xa445f1d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8095f1d5, 55 },
    { 0, 0 },
    { 0x9575f1d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a75f1da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7565f1de, 55 },
    { 0, 0 },
    { 0x1125f1e0, 95 },
    { 0x6255f1e1, 33 },
    { 0x1505f1e0, 33 },
    { 0, 0 },
    { 0x7405f1e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf935f1e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3255f1eb, 61 },
    { 0x6a5f1ec, 55 },
    { 0xf155f1ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25f1f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe205f1f8, 61 },
    { 0x24e5f1f8, 99 },
    { 0x80a5f1fa, 55 },
    { 0xdba5f1fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1915f1fe, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55f203, 61 },
    { 0x36b5f204, 98 },
    { 0xd615f204, 90 },
    { 0xe855f206, 98 },
    { 0xa15f203, 95 },
    { 0x14f5f204, 61 },
    { 0x5ac5f203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5025f20d, 61 },
    { 0x6f65f20e, 90 },
    { 0x9665f20e, 90 },
    { 0x24c5f20e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb055f217, 34 },
    { 0x5905f218, 96 },
    { 0, 0 },
    { 0x8375f21a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad5f21e, 23 },
    { 0x8935f21f, 77 },
    { 0xc5d5f220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c85f226, 34 },
    { 0x5ac5f227, 75 },
    { 0xf9d5f227, 78 },
    { 0x89f5f228, 76 },
    { 0, 0 },
    { 0xf325f22b, 98 },
    { 0, 0 },
    { 0xd715f22d, 33 },
    { 0xf675f22e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe165f232, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f236, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5f23b, 33 },
    { 0xb9c5f23b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2435f240, 69 },
    { 0x5435f240, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25f245, 84 },
    { 0x8bf5f246, 23 },
    { 0xaf05f246, 23 },
    { 0xb765f248, 33 },
    { 0x9a5f249, 90 },
    { 0xf25f245, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d5f24d, 33 },
    { 0xd535f24d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5f251, 77 },
    { 0xe5c5f251, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9015f255, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b5f259, 23 },
    { 0, 0 },
    { 0xb025f25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6235f25e, 78 },
    { 0x3ad5f25e, 61 },
    { 0, 0 },
    { 0x7525f261, 61 },
    { 0x5b15f262, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b95f265, 78 },
    { 0xb615f266, 95 },
    { 0xd585f267, 55 },
    { 0x765f265, 93 },
    { 0x62e5f265, 93 },
    { 0, 0 },
    { 0x7b35f26b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c5f26e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55f272, 76 },
    { 0x68d5f272, 93 },
    { 0xee15f274, 87 },
    { 0x6ab5f275, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe435f283, 78 },
    { 0x7c95f283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9585f28b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7385f29b, 61 },
    { 0x51d5f29b, 61 },
    { 0x3d75f29b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da5f2a4, 68 },
    { 0x7015f2a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5255f2a8, 66 },
    { 0x8d55f2a8, 31 },
    { 0, 0 },
    { 0x5a5f2ab, 46 },
    { 0x3845f2ac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2975f2b1, 46 },
    { 0, 0 },
    { 0xa0a5f2b3, 90 },
    { 0xd345f2b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1135f2c7, 23 },
    { 0, 0 },
    { 0xd675f2c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8015f2ce, 61 },
    { 0xa3a5f2cf, 23 },
    { 0xcde5f2cf, 98 },
    { 0x2345f2d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b55f2d8, 98 },
    { 0x9b55f2d9, 90 },
    { 0xfd25f2da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe445f2e5, 95 },
    { 0, 0 },
    { 0xcbe5f2e7, 96 },
    { 0, 0 },
    { 0xfd85f2e9, 90 },
    { 0, 0 },
    { 0xe1d5f2eb, 61 },
    { 0x8495f2ec, 90 },
    { 0xb1b5f2ed, 76 },
    { 0xa305f2ed, 98 },
    { 0x3fb5f2ee, 33 },
    { 0xf1a5f2f0, 95 },
    { 0xdaa5f2f0, 55 },
    { 0x5e15f2ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb65f2f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd5f301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca5f305, 19 },
    { 0xe765f306, 55 },
    { 0, 0 },
    { 0xb505f308, 66 },
    { 0xab45f309, 76 },
    { 0x1a65f308, 76 },
    { 0x2485f308, 33 },
    { 0, 0 },
    { 0xa845f30d, 100 },
    { 0x715f30e, 61 },
    { 0xd1b5f30d, 98 },
    { 0xe705f30f, 76 },
    { 0x3905f311, 23 },
    { 0x75f5f311, 76 },
    { 0x56f5f311, 61 },
    { 0x8425f314, 93 },
    { 0x6e85f315, 90 },
    { 0x29d5f314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f5f32b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a55f331, 61 },
    { 0x9495f331, 91 },
    { 0x3525f333, 76 },
    { 0xd375f334, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf515f344, 29 },
    { 0xabe5f345, 93 },
    { 0x92a5f346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a5f34b, 40 },
    { 0x7925f34c, 46 },
    { 0xc015f34b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb5f352, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba15f358, 61 },
    { 0xfa85f358, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8515f35e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a5f361, 43 },
    { 0xe95f362, 61 },
    { 0, 0 },
    { 0x72d5f364, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5f367, 55 },
    { 0, 0 },
    { 0xcaa5f369, 46 },
    { 0x3095f369, 93 },
    { 0x5715f36b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e55f36e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f5f372, 23 },
    { 0x58f5f372, 98 },
    { 0, 0 },
    { 0xccf5f375, 61 },
    { 0x48d5f375, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb675f37b, 78 },
    { 0x67e5f37b, 33 },
    { 0xe5f37c, 80 },
    { 0xfc05f37e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e5f382, 43 },
    { 0xb825f382, 90 },
    { 0, 0 },
    { 0xe665f385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8355f38e, 98 },
    { 0, 0 },
    { 0xda05f390, 43 },
    { 0xd275f391, 23 },
    { 0, 0 },
    { 0x2f15f393, 93 },
    { 0x2d65f394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9935f39b, 84 },
    { 0x79f5f39c, 91 },
    { 0xe415f39c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f35f3a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5f3a4, 33 },
    { 0, 0 },
    { 0x2bc5f3a6, 90 },
    { 0x61e5f3a7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe25f3ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a95f3b5, 23 },
    { 0xa865f3b6, 90 },
    { 0x6a35f3b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce35f3bf, 90 },
    { 0, 0 },
    { 0x8925f3c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca5f3c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9745f3ce, 95 },
    { 0x5965f3cf, 61 },
    { 0x6005f3d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f55f3d5, 55 },
    { 0xbca5f3d6, 46 },
    { 0x6805f3d6, 55 },
    { 0x2b5f3d6, 90 },
    { 0, 0 },
    { 0xbb95f3da, 90 },
    { 0x8f5f3da, 87 },
    { 0x89b5f3da, 95 },
    { 0xf125f3dd, 91 },
    { 0x9b55f3de, 98 },
    { 0x7b75f3df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c75f3e9, 90 },
    { 0x3265f3e9, 75 },
    { 0x52d5f3e9, 95 },
    { 0xf465f3ec, 76 },
    { 0x6c45f3ea, 95 },
    { 0xff85f3e9, 98 },
    { 0xa455f3ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4105f3f2, 95 },
    { 0, 0 },
    { 0x5b5f3f4, 69 },
    { 0x9d95f3f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd135f3fb, 84 },
    { 0xecd5f3fc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb405f402, 66 },
    { 0xff5f403, 76 },
    { 0x94a5f402, 55 },
    { 0, 0 },
    { 0x4c65f406, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef5f409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd5f40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9025f412, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9785f416, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab35f41a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe195f420, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2115f425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf5f429, 17 },
    { 0x7b95f429, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee5f430, 90 },
    { 0x72f5f431, 90 },
    { 0x4295f431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae85f439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad5f43c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed5f442, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc5f445, 90 },
    { 0xb6b5f446, 23 },
    { 0, 0 },
    { 0x76b5f448, 61 },
    { 0x7695f449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c5f44f, 90 },
    { 0, 0 },
    { 0xc615f451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a65f455, 33 },
    { 0xe9d5f456, 77 },
    { 0x7f35f456, 33 },
    { 0x87a5f455, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9175f45b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x8205f45e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc205f461, 33 },
    { 0x6be5f462, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7435f467, 98 },
    { 0xd0e5f468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf195f46b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a45f47c, 90 },
    { 0x88f5f47c, 65 },
    { 0x8c55f47e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f483, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b5f488, 2 },
    { 0xb7c5f489, 61 },
    { 0x9565f48a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f5f48f, 61 },
    { 0x4ce5f490, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb35f498, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe35f49f, 98 },
    { 0xb775f4a0, 33 },
    { 0xf955f49f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e5f4a7, 76 },
    { 0xec35f4a8, 33 },
    { 0xa2c5f4a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x395f4ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745f4b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f25f4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb85f4be, 91 },
    { 0, 0 },
    { 0xe205f4c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c5f4c4, 87 },
    { 0xd085f4c4, 95 },
    { 0xa0e5f4c6, 33 },
    { 0xf9f5f4c6, 98 },
    { 0xdba5f4c7, 33 },
    { 0x5e05f4c9, 61 },
    { 0x7255f4ca, 90 },
    { 0, 0 },
    { 0x6305f4cc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e5f4d0, 61 },
    { 0xb895f4d1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e5f4d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce45f4d8, 40 },
    { 0x445f4d9, 98 },
    { 0xbab5f4da, 78 },
    { 0x1b65f4db, 61 },
    { 0x7c05f4dc, 33 },
    { 0x81f5f4db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6155f4e2, 65 },
    { 0, 0 },
    { 0x255f4e4, 76 },
    { 0x9d05f4e5, 78 },
    { 0, 0 },
    { 0x8bb5f4e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a5f4ec, 93 },
    { 0xb505f4ec, 61 },
    { 0, 0 },
    { 0xa615f4ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad5f4f2, 76 },
    { 0x7b15f4f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf835f4f9, 33 },
    { 0, 0 },
    { 0xf795f4fb, 61 },
    { 0xd275f4fb, 85 },
    { 0x55c5f4fc, 33 },
    { 0, 0 },
    { 0x96a5f4ff, 90 },
    { 0, 0 },
    { 0xf8f5f501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa585f50c, 76 },
    { 0xa325f50d, 23 },
    { 0x9355f50e, 90 },
    { 0, 0 },
    { 0x2305f510, 33 },
    { 0xd4a5f511, 33 },
    { 0xda15f512, 95 },
    { 0xce5f510, 61 },
    { 0, 0 },
    { 0xbf15f515, 32 },
    { 0x22b5f516, 90 },
    { 0, 0 },
    { 0xd2c5f518, 93 },
    { 0x7ae5f519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b5f51e, 33 },
    { 0, 0 },
    { 0x7e25f520, 88 },
    { 0xeeb5f521, 55 },
    { 0x7ee5f520, 33 },
    { 0, 0 },
    { 0x7185f524, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d5f528, 76 },
    { 0, 0 },
    { 0x46d5f52a, 77 },
    { 0x4405f52b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e15f52e, 61 },
    { 0, 0 },
    { 0xd105f530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af5f533, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4285f536, 55 },
    { 0xc2d5f536, 91 },
    { 0x6c5f536, 75 },
    { 0, 0 },
    { 0x14a5f53a, 33 },
    { 0x29e5f53a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d5f540, 81 },
    { 0xc6f5f541, 61 },
    { 0, 0 },
    { 0x595f543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d5f546, 61 },
    { 0x4bc5f546, 23 },
    { 0x5145f548, 77 },
    { 0x6225f549, 34 },
    { 0x9725f549, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9375f54d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3695f550, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5f555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f5f55a, 33 },
    { 0, 0 },
    { 0x3b45f55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc5f562, 77 },
    { 0x2695f563, 100 },
    { 0, 0 },
    { 0xb365f565, 99 },
    { 0, 0 },
    { 0x2ef5f567, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9975f56b, 81 },
    { 0x6d15f56c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3745f56f, 33 },
    { 0x9ca5f56f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b15f574, 67 },
    { 0xfcb5f574, 33 },
    { 0xa985f574, 23 },
    { 0, 0 },
    { 0x9c85f578, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a35f57c, 78 },
    { 0x2b15f57c, 23 },
    { 0, 0 },
    { 0xf5a5f57f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d65f585, 66 },
    { 0x5e95f585, 33 },
    { 0x1b15f585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7015f58a, 85 },
    { 0xbb95f58a, 46 },
    { 0x935f58c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa145f58f, 55 },
    { 0, 0 },
    { 0xecb5f591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2285f598, 90 },
    { 0xb5f5f599, 53 },
    { 0xf4e5f598, 43 },
    { 0xda75f59b, 61 },
    { 0xb8c5f59c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b5f59f, 33 },
    { 0, 0 },
    { 0x70a5f5a1, 90 },
    { 0x8ea5f5a2, 61 },
    { 0x3a75f5a1, 23 },
    { 0, 0 },
    { 0x5fe5f5a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc5f5a9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b5f5b0, 84 },
    { 0x8d45f5b1, 76 },
    { 0x3a85f5b1, 90 },
    { 0, 0 },
    { 0x6ac5f5b4, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff5f5bb, 61 },
    { 0x5475f5bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a5f5bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc955f5c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5795f5ca, 97 },
    { 0xde75f5cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3755f5ce, 90 },
    { 0x9ed5f5cf, 55 },
    { 0x2b85f5d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb675f5d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b25f5d9, 61 },
    { 0xd2a5f5da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e5f5df, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b5f5e2, 83 },
    { 0x8625f5e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e85f5e9, 95 },
    { 0, 0 },
    { 0xb915f5eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125f5f1, 75 },
    { 0, 0 },
    { 0x42b5f5f3, 76 },
    { 0xfab5f5f3, 69 },
    { 0xf8c5f5f3, 90 },
    { 0, 0 },
    { 0xc7c5f5f7, 90 },
    { 0x87a5f5f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda5f5fe, 33 },
    { 0, 0 },
    { 0xf2a5f600, 55 },
    { 0, 0 },
    { 0x3b5f602, 34 },
    { 0xf445f602, 91 },
    { 0xfe65f604, 17 },
    { 0, 0 },
    { 0xec5f606, 23 },
    { 0, 0 },
    { 0xa205f608, 90 },
    { 0x8875f609, 98 },
    { 0x85e5f608, 98 },
    { 0, 0 },
    { 0xb45f60c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc025f610, 98 },
    { 0x1a75f611, 43 },
    { 0x285f611, 95 },
    { 0x4ec5f613, 33 },
    { 0x6915f613, 23 },
    { 0xb4d5f615, 76 },
    { 0, 0 },
    { 0x5765f617, 100 },
    { 0x81d5f617, 90 },
    { 0x2f75f617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3165f61d, 98 },
    { 0, 0 },
    { 0x1f85f61f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e5f626, 78 },
    { 0xa725f627, 99 },
    { 0xd0c5f628, 88 },
    { 0x2195f628, 98 },
    { 0xa195f626, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d75f630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b5f633, 61 },
    { 0xbcc5f634, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7925f638, 46 },
    { 0xea75f638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf285f644, 84 },
    { 0xca85f645, 23 },
    { 0, 0 },
    { 0x12a5f647, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d35f64a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f5f650, 89 },
    { 0xa345f651, 33 },
    { 0x6fd5f651, 90 },
    { 0, 0 },
    { 0x6bd5f654, 55 },
    { 0xd2b5f655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6895f658, 23 },
    { 0, 0 },
    { 0xa525f65a, 55 },
    { 0x95d5f65b, 23 },
    { 0, 0 },
    { 0xa7b5f65d, 98 },
    { 0xeb75f65d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f5f664, 33 },
    { 0x8345f664, 98 },
    { 0x5395f666, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05f669, 33 },
    { 0x5a5f669, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9905f66d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a5f670, 61 },
    { 0xa3e5f671, 66 },
    { 0xf735f672, 95 },
    { 0x23b5f673, 34 },
    { 0x3e05f673, 61 },
    { 0x63c5f675, 61 },
    { 0x7745f673, 90 },
    { 0xc405f677, 98 },
    { 0xe3c5f678, 95 },
    { 0x785f675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd5f67c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe935f680, 52 },
    { 0x5375f680, 23 },
    { 0x2b25f682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd5f687, 99 },
    { 0xeb25f688, 90 },
    { 0x30f5f689, 100 },
    { 0xa045f68a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3925f68e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde45f691, 43 },
    { 0xde95f692, 87 },
    { 0x2335f693, 98 },
    { 0x3a85f693, 75 },
    { 0x785f693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6045f69a, 76 },
    { 0xce85f69b, 33 },
    { 0x2cc5f69c, 100 },
    { 0, 0 },
    { 0xbf15f69e, 93 },
    { 0x7fa5f69f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c5f6a2, 90 },
    { 0, 0 },
    { 0xef5f6a4, 61 },
    { 0xeee5f6a4, 55 },
    { 0, 0 },
    { 0xadb5f6a7, 33 },
    { 0x455f6a8, 33 },
    { 0x2385f6a9, 95 },
    { 0xed75f6aa, 90 },
    { 0x6e15f6aa, 100 },
    { 0, 0 },
    { 0xe8f5f6ad, 61 },
    { 0, 0 },
    { 0xaa95f6af, 90 },
    { 0x6c65f6b0, 23 },
    { 0x2cc5f6b1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c5f6b4, 34 },
    { 0, 0 },
    { 0x13c5f6b6, 90 },
    { 0, 0 },
    { 0xdbf5f6b8, 76 },
    { 0, 0 },
    { 0x2425f6ba, 98 },
    { 0x7295f6ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa5f6be, 33 },
    { 0x1485f6be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb5f6c4, 90 },
    { 0x6dd5f6c5, 90 },
    { 0, 0 },
    { 0x9465f6c7, 39 },
    { 0, 0 },
    { 0x9b5f6c9, 98 },
    { 0xedb5f6c9, 55 },
    { 0, 0 },
    { 0x2135f6cc, 90 },
    { 0x84c5f6cd, 90 },
    { 0x9085f6ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d05f6d3, 23 },
    { 0, 0 },
    { 0xae75f6d5, 33 },
    { 0x2e55f6d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7055f6e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab85f6e7, 33 },
    { 0, 0 },
    { 0xb775f6e9, 95 },
    { 0x1265f6ea, 61 },
    { 0x2b05f6eb, 80 },
    { 0x9905f6eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde95f6ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5f6f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd795f6f9, 98 },
    { 0, 0 },
    { 0xf295f6fb, 95 },
    { 0x19e5f6fc, 88 },
    { 0, 0 },
    { 0xcb35f6fe, 93 },
    { 0x9615f6ff, 43 },
    { 0x7ec5f6ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d45f705, 23 },
    { 0x85f706, 88 },
    { 0xa7e5f706, 33 },
    { 0x3825f707, 4 },
    { 0x2595f709, 80 },
    { 0, 0 },
    { 0xea5f70b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce25f712, 78 },
    { 0, 0 },
    { 0xd9f5f714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb555f71a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c5f71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb5f720, 52 },
    { 0x16b5f721, 23 },
    { 0x26b5f722, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795f727, 61 },
    { 0, 0 },
    { 0xfd35f729, 91 },
    { 0x49c5f72a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd5f72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b5f735, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7175f738, 90 },
    { 0x1215f739, 23 },
    { 0x2b5f73a, 95 },
    { 0x7b5f73b, 90 },
    { 0x1795f73c, 61 },
    { 0x7be5f739, 55 },
    { 0, 0 },
    { 0x2db5f73f, 76 },
    { 0x1f5f740, 66 },
    { 0, 0 },
    { 0x6ce5f742, 55 },
    { 0, 0 },
    { 0xa0c5f744, 90 },
    { 0x5735f744, 95 },
    { 0, 0 },
    { 0xb9e5f747, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575f74b, 61 },
    { 0x1cc5f74b, 33 },
    { 0xd765f74d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a35f750, 55 },
    { 0x2a95f751, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xea65f754, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8415f759, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf5f75c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a5f760, 76 },
    { 0x9f75f760, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605f768, 33 },
    { 0x55e5f769, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e45f76f, 68 },
    { 0x82e5f770, 98 },
    { 0x3bc5f76f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6605f774, 98 },
    { 0x1605f774, 90 },
    { 0x9bf5f775, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf15f77e, 75 },
    { 0xe955f77e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8605f782, 67 },
    { 0x5e5f782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f5f786, 64 },
    { 0x2805f786, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4695f78a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef75f791, 33 },
    { 0, 0 },
    { 0xec05f793, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa705f798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9585f7a0, 33 },
    { 0, 0 },
    { 0xbbf5f7a2, 33 },
    { 0xe315f7a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5f7a6, 34 },
    { 0, 0 },
    { 0x6595f7a8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9745f7ac, 90 },
    { 0, 0 },
    { 0xc3a5f7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d35f7b9, 82 },
    { 0xfc75f7ba, 61 },
    { 0x6305f7ba, 53 },
    { 0x1b5f7bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd375f7c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4485f7c6, 76 },
    { 0xdec5f7c7, 98 },
    { 0x4e55f7c8, 84 },
    { 0x7c95f7c8, 76 },
    { 0xc3b5f7c8, 33 },
    { 0x3b25f7cb, 43 },
    { 0x7b35f7cc, 90 },
    { 0xfd15f7c7, 90 },
    { 0x9465f7ce, 33 },
    { 0xacb5f7cf, 67 },
    { 0xe5c5f7cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c5f7d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe325f7db, 100 },
    { 0x4055f7dc, 58 },
    { 0x2cf5f7dc, 38 },
    { 0xf0c5f7de, 90 },
    { 0x1165f7df, 76 },
    { 0xc6e5f7df, 96 },
    { 0xe1c5f7e1, 98 },
    { 0, 0 },
    { 0x8225f7e3, 100 },
    { 0xc875f7e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a85f7e7, 99 },
    { 0x8615f7e8, 23 },
    { 0xcdd5f7e9, 90 },
    { 0xc475f7ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c25f7f1, 95 },
    { 0x7d85f7f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9675f7f6, 52 },
    { 0x4dc5f7f7, 78 },
    { 0x375f7f8, 84 },
    { 0x5b05f7f8, 99 },
    { 0x9b45f7fa, 78 },
    { 0x1985f7f7, 96 },
    { 0x7425f7fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff5f800, 99 },
    { 0xc1a5f801, 100 },
    { 0xbdb5f802, 78 },
    { 0x5725f801, 81 },
    { 0x4b45f803, 85 },
    { 0xb055f802, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa175f808, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc775f80b, 23 },
    { 0xd195f80c, 33 },
    { 0, 0 },
    { 0x48e5f80e, 34 },
    { 0x5a45f80e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb5f813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc65f816, 87 },
    { 0x10b5f817, 98 },
    { 0xa3b5f817, 98 },
    { 0x7085f819, 33 },
    { 0x6f35f81a, 91 },
    { 0, 0 },
    { 0xa1f5f81c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd165f820, 61 },
    { 0, 0 },
    { 0x7b35f822, 91 },
    { 0, 0 },
    { 0xbbb5f824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd5f831, 98 },
    { 0x8f75f832, 34 },
    { 0, 0 },
    { 0xd515f834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf5f83e, 33 },
    { 0x4415f83f, 95 },
    { 0x505f840, 52 },
    { 0xa7c5f83f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3145f847, 98 },
    { 0x6955f847, 66 },
    { 0x2e85f849, 61 },
    { 0x2765f849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3435f84d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a5f854, 33 },
    { 0xea55f855, 61 },
    { 0xe45f854, 76 },
    { 0, 0 },
    { 0xae65f858, 66 },
    { 0x3f65f859, 99 },
    { 0xafe5f858, 90 },
    { 0x44e5f85b, 90 },
    { 0xf125f859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab5f860, 95 },
    { 0xe595f861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2355f867, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa5f86a, 90 },
    { 0x40b5f86a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9185f870, 98 },
    { 0, 0 },
    { 0xf8d5f872, 90 },
    { 0xbf15f872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b5f878, 33 },
    { 0x1795f879, 90 },
    { 0x4605f87a, 87 },
    { 0x3a15f879, 33 },
    { 0x54a5f87c, 26 },
    { 0xfd55f87d, 90 },
    { 0, 0 },
    { 0x4865f87f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1445f884, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a5f887, 55 },
    { 0xb155f888, 93 },
    { 0x4715f889, 33 },
    { 0xe755f889, 95 },
    { 0x7715f889, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d5f896, 69 },
    { 0x63c5f897, 34 },
    { 0xe995f898, 46 },
    { 0x445f899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a65f89d, 33 },
    { 0, 0 },
    { 0x76c5f89f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c85f8a4, 98 },
    { 0xd6f5f8a5, 77 },
    { 0x4b45f8a6, 23 },
    { 0x4c55f8a4, 61 },
    { 0x9df5f8a8, 55 },
    { 0x9ac5f8a9, 98 },
    { 0x2b75f8a9, 95 },
    { 0, 0 },
    { 0x8895f8ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4535f8af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d5f8b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35f8b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab5f8bd, 34 },
    { 0x1075f8be, 33 },
    { 0xae05f8bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7845f8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2855f8c7, 61 },
    { 0x9f5f8c7, 23 },
    { 0, 0 },
    { 0xa1c5f8ca, 34 },
    { 0xa065f8cb, 91 },
    { 0, 0 },
    { 0x5b85f8cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9035f8d0, 87 },
    { 0xea05f8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d5f8d4, 95 },
    { 0x5735f8d5, 95 },
    { 0x2ff5f8d6, 36 },
    { 0x5c55f8d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b5f8dd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be5f8e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd5f8e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5225f8e7, 43 },
    { 0x9e25f8e7, 90 },
    { 0x8175f8e9, 43 },
    { 0x4155f8ea, 34 },
    { 0xdfb5f8e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685f8f1, 61 },
    { 0xb0b5f8f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f5f8f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac35f8f8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb5f8fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa5f8ff, 61 },
    { 0, 0 },
    { 0x9135f901, 61 },
    { 0x3f25f902, 76 },
    { 0, 0 },
    { 0x3b45f904, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2245f90a, 77 },
    { 0xd5b5f90b, 77 },
    { 0x3265f90c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e75f90f, 95 },
    { 0xb655f90f, 23 },
    { 0x28d5f90f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x755f914, 34 },
    { 0x3035f915, 85 },
    { 0x8a65f916, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2065f929, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc785f92d, 33 },
    { 0, 0 },
    { 0x13f5f92f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3835f935, 33 },
    { 0x97d5f936, 33 },
    { 0, 0 },
    { 0x9d05f938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f5f940, 34 },
    { 0x3715f940, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb5f944, 76 },
    { 0x6e85f945, 52 },
    { 0x2795f945, 61 },
    { 0x6b95f947, 95 },
    { 0xfb5f948, 33 },
    { 0x1295f949, 76 },
    { 0x9515f94a, 61 },
    { 0x6805f949, 75 },
    { 0x6ed5f94a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f5f953, 43 },
    { 0xf385f954, 76 },
    { 0, 0 },
    { 0x2505f956, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd475f95b, 61 },
    { 0x5b75f95c, 91 },
    { 0x3535f95c, 43 },
    { 0xb465f95e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a5f963, 52 },
    { 0, 0 },
    { 0x8545f965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a5f968, 100 },
    { 0, 0 },
    { 0x9fa5f96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d65f96e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff5f975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa185f979, 33 },
    { 0, 0 },
    { 0xf55f97b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f5f982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed75f985, 84 },
    { 0xb2f5f986, 43 },
    { 0, 0 },
    { 0x10a5f988, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2155f990, 55 },
    { 0xa895f991, 90 },
    { 0x36a5f991, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb05f99d, 95 },
    { 0x38e5f99e, 95 },
    { 0x9cc5f99f, 88 },
    { 0xd555f99f, 75 },
    { 0x3675f9a1, 95 },
    { 0x8645f9a0, 90 },
    { 0x84d5f99e, 93 },
    { 0x76b5f9a4, 85 },
    { 0x4ad5f9a2, 47 },
    { 0xb535f9a6, 33 },
    { 0xc25f99e, 95 },
    { 0x7ce5f99e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6135f9b1, 34 },
    { 0x8c55f9b2, 23 },
    { 0x1335f9b2, 67 },
    { 0, 0 },
    { 0x24f5f9b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5105f9b9, 34 },
    { 0, 0 },
    { 0xec75f9bb, 95 },
    { 0xc715f9bc, 68 },
    { 0x3985f9bb, 61 },
    { 0xbfb5f9bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1115f9c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe285f9c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df5f9cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8275f9d0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c5f9d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a5f9da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c5f9dd, 98 },
    { 0xf8b5f9de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc165f9e1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae5f9e7, 98 },
    { 0x7375f9e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb5f9ef, 61 },
    { 0x4ff5f9f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5175f9f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd525fa13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9235fa1a, 76 },
    { 0xfa55fa1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd15fa28, 90 },
    { 0x92e5fa29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d5fa2e, 87 },
    { 0x4c25fa2f, 78 },
    { 0x1485fa2f, 33 },
    { 0x1575fa2f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb775fa34, 52 },
    { 0x4a55fa35, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b5fa38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3975fa3b, 33 },
    { 0xeff5fa3c, 55 },
    { 0x3785fa3d, 98 },
    { 0xea15fa3e, 61 },
    { 0xb535fa3f, 23 },
    { 0, 0 },
    { 0xafe5fa41, 33 },
    { 0, 0 },
    { 0x7ed5fa43, 76 },
    { 0x3005fa44, 95 },
    { 0x2ae5fa45, 90 },
    { 0x69b5fa43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba5fa53, 46 },
    { 0xb515fa54, 76 },
    { 0x1275fa54, 76 },
    { 0, 0 },
    { 0x2c65fa57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8615fa5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb5fa5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1335fa61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5855fa64, 87 },
    { 0x5ec5fa65, 91 },
    { 0x51d5fa66, 98 },
    { 0xd0b5fa67, 95 },
    { 0, 0 },
    { 0x89f5fa69, 34 },
    { 0xb055fa6a, 61 },
    { 0xde05fa69, 98 },
    { 0xb9e5fa6c, 98 },
    { 0, 0 },
    { 0x35c5fa6e, 93 },
    { 0x6a75fa6e, 98 },
    { 0x8a55fa6e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd5fa74, 13 },
    { 0, 0 },
    { 0xaaf5fa76, 76 },
    { 0, 0 },
    { 0x17c5fa78, 55 },
    { 0, 0 },
    { 0xe125fa7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2965fa7d, 95 },
    { 0, 0 },
    { 0x1135fa7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5245fa83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8175fa8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd85fa93, 34 },
    { 0x7cb5fa94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b5fa9b, 33 },
    { 0, 0 },
    { 0x1a75fa9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc285faa2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c85faa7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6515faab, 20 },
    { 0x8cb5faac, 98 },
    { 0xd145faad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad55fab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc665fabd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa205fac2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595fac9, 34 },
    { 0xec65faca, 76 },
    { 0xa625facb, 33 },
    { 0x1a5faca, 76 },
    { 0x1695facd, 84 },
    { 0x6b15facd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe535fad6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f5fada, 90 },
    { 0, 0 },
    { 0xe1e5fadc, 61 },
    { 0, 0 },
    { 0x3d75fade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8865fae3, 33 },
    { 0, 0 },
    { 0x3695fae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8915fae9, 95 },
    { 0xe635faea, 23 },
    { 0x8565faeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f95faee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e85faf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5305fafe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9905fb02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a55fb0a, 36 },
    { 0x41d5fb0b, 52 },
    { 0x6465fb0b, 90 },
    { 0x2175fb0b, 46 },
    { 0x3a45fb0c, 93 },
    { 0x9f65fb0d, 91 },
    { 0xd3c5fb10, 84 },
    { 0x7bb5fb11, 46 },
    { 0x6d5fb11, 23 },
    { 0x9825fb13, 98 },
    { 0x3b75fb14, 33 },
    { 0xf0b5fb14, 33 },
    { 0x5185fb16, 66 },
    { 0xbac5fb15, 95 },
    { 0x47b5fb18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a85fb1b, 46 },
    { 0x7d65fb1c, 95 },
    { 0x7a75fb1c, 98 },
    { 0xfd25fb1e, 76 },
    { 0x8045fb1f, 98 },
    { 0xf8f5fb1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d5fb23, 84 },
    { 0x865fb24, 95 },
    { 0, 0 },
    { 0x55a5fb26, 55 },
    { 0x6065fb27, 85 },
    { 0, 0 },
    { 0xa875fb29, 76 },
    { 0xb0d5fb2a, 23 },
    { 0x2f85fb2b, 88 },
    { 0xa6d5fb29, 55 },
    { 0x1ca5fb2d, 61 },
    { 0xd055fb2e, 46 },
    { 0xb5b5fb2f, 88 },
    { 0x75e5fb30, 52 },
    { 0x9e5fb31, 90 },
    { 0xf8e5fb32, 66 },
    { 0x83b5fb2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3445fb39, 91 },
    { 0x50c5fb3a, 61 },
    { 0x5805fb3b, 61 },
    { 0x2bd5fb3b, 95 },
    { 0, 0 },
    { 0xd295fb3e, 84 },
    { 0, 0 },
    { 0x4c75fb40, 90 },
    { 0x765fb41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d35fb44, 42 },
    { 0x29c5fb44, 98 },
    { 0x8585fb46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9275fb4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5fb58, 33 },
    { 0, 0 },
    { 0x4635fb5a, 95 },
    { 0xbad5fb5b, 90 },
    { 0x3965fb5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d5fb63, 95 },
    { 0x8fa5fb64, 68 },
    { 0xcf5fb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff5fb68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb5fb6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1905fb70, 98 },
    { 0x2bc5fb70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f95fb76, 33 },
    { 0xf295fb76, 98 },
    { 0, 0 },
    { 0x5235fb79, 91 },
    { 0x47c5fb7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f5fb7d, 68 },
    { 0x9785fb7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a5fb83, 98 },
    { 0x12a5fb84, 95 },
    { 0, 0 },
    { 0xa55fb86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5fb8b, 95 },
    { 0, 0 },
    { 0xed5fb8d, 90 },
    { 0, 0 },
    { 0xa2d5fb8f, 90 },
    { 0, 0 },
    { 0x8305fb91, 87 },
    { 0, 0 },
    { 0x20d5fb93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf855fb96, 95 },
    { 0x7e25fb96, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf795fb9b, 78 },
    { 0x2725fb9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec5fb9f, 99 },
    { 0, 0 },
    { 0xde35fba1, 85 },
    { 0x1c25fba2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd725fba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd075fbae, 23 },
    { 0x22b5fbae, 33 },
    { 0, 0 },
    { 0x5b95fbb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c5fbb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c5fbbc, 98 },
    { 0xde05fbbc, 23 },
    { 0xb965fbbe, 88 },
    { 0xb15fbbf, 43 },
    { 0x74b5fbbe, 76 },
    { 0x1115fbc1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea45fbc5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee65fbcc, 85 },
    { 0xd55fbcd, 33 },
    { 0x5ab5fbcd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb15fbd2, 95 },
    { 0, 0 },
    { 0x5435fbd4, 33 },
    { 0, 0 },
    { 0x3b35fbd6, 90 },
    { 0, 0 },
    { 0x2f05fbd8, 61 },
    { 0, 0 },
    { 0xcd45fbda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b5fbdd, 86 },
    { 0xc5d5fbde, 76 },
    { 0xf815fbdf, 34 },
    { 0x7e65fbde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9995fbe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af5fbe8, 98 },
    { 0xa225fbe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a65fbed, 59 },
    { 0, 0 },
    { 0xfc05fbef, 61 },
    { 0xfb35fbf0, 88 },
    { 0xa0d5fbf1, 55 },
    { 0xc175fbf0, 61 },
    { 0x9db5fbf0, 23 },
    { 0x33f5fbf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c5fbfc, 97 },
    { 0xc55fbfd, 85 },
    { 0xaee5fbfc, 88 },
    { 0xc225fbff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4955fc02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf85fc05, 61 },
    { 0xf55fc06, 67 },
    { 0, 0 },
    { 0x7185fc08, 52 },
    { 0x2d65fc09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a85fc11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c25fc16, 61 },
    { 0x1145fc16, 33 },
    { 0, 0 },
    { 0x3805fc19, 23 },
    { 0, 0 },
    { 0x5225fc1b, 95 },
    { 0xa5f5fc1c, 100 },
    { 0xca65fc1c, 76 },
    { 0xfbf5fc1c, 52 },
    { 0, 0 },
    { 0xc525fc20, 90 },
    { 0xf715fc21, 61 },
    { 0xcea5fc22, 23 },
    { 0xe805fc23, 90 },
    { 0x4135fc23, 46 },
    { 0x75e5fc25, 55 },
    { 0, 0 },
    { 0xbdb5fc27, 83 },
    { 0xeda5fc28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e5fc2b, 90 },
    { 0x5bc5fc2c, 90 },
    { 0x6395fc2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd5fc30, 98 },
    { 0, 0 },
    { 0xd2a5fc32, 23 },
    { 0xefb5fc33, 23 },
    { 0xd4d5fc34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe605fc3a, 66 },
    { 0, 0 },
    { 0xcc95fc3c, 90 },
    { 0, 0 },
    { 0x5eb5fc3e, 87 },
    { 0, 0 },
    { 0x8cf5fc40, 95 },
    { 0xf545fc41, 76 },
    { 0, 0 },
    { 0x1985fc43, 95 },
    { 0, 0 },
    { 0x3a55fc45, 77 },
    { 0, 0 },
    { 0x9ed5fc47, 64 },
    { 0xb945fc48, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5565fc4f, 33 },
    { 0, 0 },
    { 0x8ec5fc51, 100 },
    { 0xe3b5fc51, 90 },
    { 0x1bf5fc53, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e5fc57, 43 },
    { 0xbef5fc58, 90 },
    { 0xb3d5fc57, 98 },
    { 0x3eb5fc57, 61 },
    { 0xf635fc5b, 78 },
    { 0x3865fc59, 90 },
    { 0x4ca5fc59, 98 },
    { 0xc435fc5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7475fc61, 90 },
    { 0x60f5fc61, 61 },
    { 0x2975fc62, 90 },
    { 0x8375fc64, 67 },
    { 0xe655fc65, 55 },
    { 0x49b5fc66, 61 },
    { 0x8245fc67, 76 },
    { 0, 0 },
    { 0xaf55fc69, 98 },
    { 0, 0 },
    { 0xdbd5fc6b, 77 },
    { 0, 0 },
    { 0xc9a5fc6d, 61 },
    { 0xefb5fc6e, 84 },
    { 0x82c5fc6f, 61 },
    { 0xefb5fc70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa495fc75, 98 },
    { 0, 0 },
    { 0x7655fc77, 91 },
    { 0x7a55fc77, 55 },
    { 0, 0 },
    { 0xc515fc7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd465fc7e, 56 },
    { 0x74e5fc7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e5fc85, 33 },
    { 0x78c5fc85, 61 },
    { 0xd735fc87, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec5fc90, 55 },
    { 0, 0 },
    { 0x8395fc92, 98 },
    { 0, 0 },
    { 0x5ef5fc94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb5fc97, 52 },
    { 0, 0 },
    { 0xeec5fc99, 75 },
    { 0x5c55fc9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd5fca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e35fca6, 52 },
    { 0xf645fca6, 95 },
    { 0xa725fca8, 61 },
    { 0xc175fca7, 46 },
    { 0x10b5fcaa, 81 },
    { 0xbd75fca8, 33 },
    { 0x2775fcac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc5fcaf, 98 },
    { 0, 0 },
    { 0xde45fcb1, 33 },
    { 0x7c85fcb2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e85fcb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d85fcbe, 46 },
    { 0, 0 },
    { 0x1965fcc0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b45fcc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3475fcc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b35fccc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5055fccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6915fcd6, 23 },
    { 0x7c45fcd6, 52 },
    { 0, 0 },
    { 0x1f35fcd9, 33 },
    { 0xfa85fcda, 90 },
    { 0xe75fcdb, 90 },
    { 0x3ff5fcdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a55fcdf, 61 },
    { 0x6cf5fce0, 23 },
    { 0x8095fce1, 55 },
    { 0x3155fce1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee5fce6, 65 },
    { 0, 0 },
    { 0x7345fce8, 46 },
    { 0, 0 },
    { 0x3465fcea, 23 },
    { 0x9685fcea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5185fcee, 34 },
    { 0x1ac5fcee, 98 },
    { 0, 0 },
    { 0xa455fcf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce55fcf6, 98 },
    { 0xa3b5fcf6, 95 },
    { 0, 0 },
    { 0x515fcf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc45fd01, 85 },
    { 0x86c5fd02, 87 },
    { 0, 0 },
    { 0xbb25fd04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b5fd08, 85 },
    { 0xeb25fd09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf55fd0e, 90 },
    { 0x5955fd0e, 26 },
    { 0xc1e5fd10, 43 },
    { 0xa9d5fd0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b15fd15, 67 },
    { 0xa855fd16, 95 },
    { 0, 0 },
    { 0xbfb5fd18, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9495fd1b, 76 },
    { 0x26e5fd1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a95fd23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d25fd2a, 33 },
    { 0, 0 },
    { 0x13c5fd2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff35fd30, 85 },
    { 0xf7a5fd31, 95 },
    { 0x8885fd32, 33 },
    { 0x9f05fd32, 61 },
    { 0xdb85fd34, 23 },
    { 0x1ee5fd30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6605fd38, 23 },
    { 0xb035fd38, 90 },
    { 0, 0 },
    { 0x6f25fd3b, 90 },
    { 0, 0 },
    { 0xe935fd3d, 84 },
    { 0xaa35fd3d, 23 },
    { 0xeac5fd3d, 67 },
    { 0xed25fd40, 23 },
    { 0, 0 },
    { 0x8ee5fd42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4785fd45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3275fd4d, 46 },
    { 0xa445fd4e, 17 },
    { 0x8b45fd4e, 93 },
    { 0, 0 },
    { 0x54f5fd51, 40 },
    { 0x5b15fd52, 95 },
    { 0xb275fd53, 81 },
    { 0x3fe5fd53, 98 },
    { 0x4565fd55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb435fd5f, 98 },
    { 0x6d35fd60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe365fd72, 97 },
    { 0, 0 },
    { 0xb535fd74, 33 },
    { 0xb25fd75, 90 },
    { 0x18d5fd76, 76 },
    { 0x8ee5fd76, 33 },
    { 0, 0 },
    { 0xbea5fd79, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa535fd7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c95fd83, 76 },
    { 0x3045fd83, 95 },
    { 0, 0 },
    { 0x83c5fd86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b85fd8d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff25fd92, 90 },
    { 0xa35fd92, 17 },
    { 0, 0 },
    { 0x49d5fd95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad5fd9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd5fda1, 52 },
    { 0x3f35fda2, 23 },
    { 0x5df5fda3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d35fdaa, 95 },
    { 0xdb85fdaa, 90 },
    { 0, 0 },
    { 0x4c05fdad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45fdb1, 76 },
    { 0x1cb5fdb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8365fdb6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5805fdbc, 42 },
    { 0, 0 },
    { 0x58c5fdbe, 19 },
    { 0x5795fdbf, 90 },
    { 0xd885fdc0, 33 },
    { 0x5305fdc1, 90 },
    { 0x6f75fdc2, 84 },
    { 0xbdf5fdc3, 23 },
    { 0x9095fdc2, 33 },
    { 0x3545fdc5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc435fdc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf35fdd3, 33 },
    { 0, 0 },
    { 0xa345fdd5, 61 },
    { 0x67c5fdd6, 100 },
    { 0xd9c5fdd6, 23 },
    { 0x31a5fdd6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c65fddd, 33 },
    { 0x9945fdde, 69 },
    { 0xc225fddf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd5fde2, 55 },
    { 0, 0 },
    { 0x9dc5fde4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5885fdea, 34 },
    { 0x48f5fdeb, 90 },
    { 0, 0 },
    { 0xcd35fded, 61 },
    { 0xd765fdee, 84 },
    { 0, 0 },
    { 0xa405fdf0, 76 },
    { 0x8635fdf0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae5fdf6, 95 },
    { 0xfc75fdf6, 95 },
    { 0, 0 },
    { 0xc8b5fdf9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae25fdfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e5fe01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac75fe04, 78 },
    { 0xd5fe04, 67 },
    { 0x1415fe06, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5335fe09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5905fe0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1755fe12, 23 },
    { 0x1505fe13, 100 },
    { 0x34f5fe13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e5fe18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4965fe21, 75 },
    { 0x3e15fe22, 61 },
    { 0, 0 },
    { 0xd545fe24, 90 },
    { 0x1725fe25, 99 },
    { 0x9f65fe26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c5fe2a, 80 },
    { 0x3595fe2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b85fe2e, 55 },
    { 0, 0 },
    { 0x1df5fe30, 98 },
    { 0xeb25fe31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a35fe36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e5fe3a, 81 },
    { 0xcdf5fe3b, 95 },
    { 0, 0 },
    { 0x8295fe3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2975fe43, 65 },
    { 0x3105fe44, 33 },
    { 0, 0 },
    { 0x6565fe46, 90 },
    { 0xfa15fe47, 98 },
    { 0, 0 },
    { 0x2f75fe49, 61 },
    { 0xcea5fe49, 55 },
    { 0xa0a5fe4b, 96 },
    { 0x6505fe4c, 77 },
    { 0x8c95fe4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba35fe50, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4235fe53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2875fe58, 33 },
    { 0x8035fe59, 90 },
    { 0xc4d5fe58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa245fe5f, 76 },
    { 0x6b05fe60, 95 },
    { 0xdd15fe60, 76 },
    { 0x275fe62, 87 },
    { 0x9ee5fe61, 34 },
    { 0x7355fe60, 34 },
    { 0, 0 },
    { 0xca55fe66, 33 },
    { 0x60d5fe67, 55 },
    { 0xfd25fe68, 23 },
    { 0xd815fe69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a35fe6e, 95 },
    { 0xe135fe6f, 55 },
    { 0x1645fe70, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca5fe75, 98 },
    { 0x5e95fe76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba5fe7e, 34 },
    { 0x80e5fe7e, 69 },
    { 0xac25fe7f, 99 },
    { 0, 0 },
    { 0x4965fe82, 61 },
    { 0x4805fe83, 91 },
    { 0x8b15fe82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a5fe92, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c5fe95, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f5fe9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3965fe9e, 33 },
    { 0x7765fe9e, 61 },
    { 0x47f5fea0, 93 },
    { 0xe125fea1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da5fea4, 67 },
    { 0x4d35fea5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca95fea9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd075feb0, 76 },
    { 0, 0 },
    { 0xdfa5feb2, 17 },
    { 0x2fa5feb2, 88 },
    { 0x9525feb4, 61 },
    { 0x5d95feb4, 98 },
    { 0xf555feb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d05febb, 98 },
    { 0, 0 },
    { 0x3405febd, 90 },
    { 0xc2d5febd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa145fec5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df5fece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f5fed3, 40 },
    { 0xbce5fed3, 56 },
    { 0x8375fed5, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1215fed9, 91 },
    { 0, 0 },
    { 0xae35fedb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd5fede, 64 },
    { 0x6215fedf, 90 },
    { 0x47a5fee0, 81 },
    { 0x4275fee1, 46 },
    { 0x6475fee2, 100 },
    { 0xa9a5fee3, 67 },
    { 0x45e5fee3, 61 },
    { 0, 0 },
    { 0xbd65fee6, 90 },
    { 0xce85fee7, 90 },
    { 0, 0 },
    { 0x6d85fee9, 46 },
    { 0x35fee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6195fef0, 33 },
    { 0x4da5fef1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5205fefc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a5ff03, 61 },
    { 0x92a5ff03, 95 },
    { 0xa7b5ff05, 98 },
    { 0xa25ff06, 43 },
    { 0xa6f5ff07, 33 },
    { 0x6125ff07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c5ff0b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6035ff10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe525ff13, 61 },
    { 0xccd5ff14, 55 },
    { 0xb8e5ff14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8185ff18, 88 },
    { 0x90c5ff19, 56 },
    { 0, 0 },
    { 0x61a5ff1b, 95 },
    { 0xaa25ff1c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e95ff20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed85ff28, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6955ff2b, 43 },
    { 0x3f15ff2c, 99 },
    { 0x1755ff2d, 78 },
    { 0xbd45ff2e, 90 },
    { 0x2a35ff2e, 23 },
    { 0xea05ff2f, 95 },
    { 0x2335ff2c, 95 },
    { 0xad45ff31, 90 },
    { 0x9ea5ff2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a45ff36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1475ff3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7395ff53, 90 },
    { 0, 0 },
    { 0x4aa5ff55, 98 },
    { 0x5be5ff56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc065ff62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca85ff67, 23 },
    { 0, 0 },
    { 0x5cc5ff69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5ff6d, 95 },
    { 0x145ff6e, 95 },
    { 0x43c5ff6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce25ff77, 76 },
    { 0x5f85ff78, 95 },
    { 0xb775ff79, 33 },
    { 0xef65ff7a, 99 },
    { 0xd6c5ff7b, 61 },
    { 0x18d5ff7a, 99 },
    { 0x7e15ff7d, 98 },
    { 0x9355ff7e, 95 },
    { 0xd1a5ff7f, 93 },
    { 0xcc5ff7f, 87 },
    { 0x6b5ff81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b5ff84, 43 },
    { 0x8495ff85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb895ff89, 98 },
    { 0x5105ff8a, 43 },
    { 0xd4b5ff8b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1345ff90, 23 },
    { 0x1355ff90, 76 },
    { 0x7f85ff92, 88 },
    { 0x9a95ff93, 100 },
    { 0xac5ff93, 77 },
    { 0x4a5ff95, 98 },
    { 0xec5ff96, 100 },
    { 0x9735ff97, 98 },
    { 0xa0f5ff98, 90 },
    { 0x4305ff95, 76 },
    { 0x3355ff9a, 90 },
    { 0x1b15ff9b, 66 },
    { 0xb825ff9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5395ff9f, 98 },
    { 0xb2a5ff9f, 90 },
    { 0, 0 },
    { 0x7e45ffa2, 90 },
    { 0xf065ffa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf615ffa6, 90 },
    { 0, 0 },
    { 0x325ffa8, 82 },
    { 0x4395ffa8, 23 },
    { 0x41f5ffaa, 17 },
    { 0x8805ffab, 98 },
    { 0x47f5ffab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa25ffb0, 23 },
    { 0x7345ffb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d55ffb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c5ffbb, 98 },
    { 0x4f25ffbc, 23 },
    { 0xaec5ffbb, 61 },
    { 0xd725ffbe, 90 },
    { 0x2845ffbf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f5ffc2, 43 },
    { 0xeb55ffc3, 26 },
    { 0xe295ffc2, 90 },
    { 0x5cf5ffc5, 98 },
    { 0x48c5ffc5, 87 },
    { 0x8d5ffc5, 33 },
    { 0x1f85ffc2, 95 },
    { 0x9785ffc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa875ffd3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e5ffd6, 90 },
    { 0xc865ffd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e5ffdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb385ffdf, 61 },
    { 0x49d5ffdf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d5ffe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb5ffe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d5fff3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1255fff6, 61 },
    { 0x4995fff7, 98 },
    { 0, 0 },
    { 0x5d75fff9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea35fffd, 88 },
    { 0x11e5fffe, 66 },
    { 0x1335ffff, 43 },
    { 0xbd35fffe, 34 },
    { 0xe875ffff, 99 },
    { 0x8d160002, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4460007, 55 },
    { 0x87960008, 61 },
    { 0xaf360008, 91 },
    { 0xdcb6000a, 46 },
    { 0x7316000a, 90 },
    { 0x8086000c, 61 },
    { 0, 0 },
    { 0xdd06000e, 61 },
    { 0, 0 },
    { 0x2a760010, 99 },
    { 0, 0 },
    { 0x27660012, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda660015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e56001b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc326001f, 90 },
    { 0, 0 },
    { 0xb8760021, 23 },
    { 0x3f360022, 55 },
    { 0x62760023, 53 },
    { 0xf4060023, 93 },
    { 0x2f760024, 76 },
    { 0, 0 },
    { 0x92260027, 61 },
    { 0x69460027, 34 },
    { 0x5b660029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b6002f, 46 },
    { 0x8c360030, 77 },
    { 0x4f060031, 61 },
    { 0, 0 },
    { 0xaf360033, 85 },
    { 0, 0 },
    { 0xb9e60035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96360038, 100 },
    { 0, 0 },
    { 0x5186003a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x9836003d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde60041, 52 },
    { 0, 0 },
    { 0x82b60043, 33 },
    { 0, 0 },
    { 0x6a660045, 76 },
    { 0x4fa60046, 99 },
    { 0xad460046, 33 },
    { 0x9d160048, 33 },
    { 0x36d60047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab6004c, 100 },
    { 0x19c6004d, 64 },
    { 0x8fc6004d, 95 },
    { 0, 0 },
    { 0x5db60050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b960053, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a60056, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c96005a, 46 },
    { 0, 0 },
    { 0xf686005c, 61 },
    { 0x5b06005d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a60064, 46 },
    { 0x82360065, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4596006b, 95 },
    { 0x22d6006b, 61 },
    { 0xeeb6006d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab360072, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e60075, 33 },
    { 0x9d960075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e60080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23160085, 23 },
    { 0, 0 },
    { 0x26760087, 55 },
    { 0, 0 },
    { 0xb1960089, 90 },
    { 0xda460089, 66 },
    { 0xa9b6008b, 88 },
    { 0x1d360089, 77 },
    { 0, 0 },
    { 0x6b86008e, 96 },
    { 0xdcf6008f, 33 },
    { 0x20260090, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c60096, 99 },
    { 0, 0 },
    { 0x5f360098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d6009d, 95 },
    { 0x9c6009e, 81 },
    { 0xfef6009f, 77 },
    { 0, 0 },
    { 0x248600a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc600a7, 95 },
    { 0, 0 },
    { 0xe39600a9, 98 },
    { 0xf5d600aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x610600ad, 33 },
    { 0x369600ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f600b5, 98 },
    { 0x57b600b5, 77 },
    { 0xd5e600b5, 100 },
    { 0x268600b6, 98 },
    { 0x8ea600b9, 33 },
    { 0x421600b5, 43 },
    { 0x260600b7, 61 },
    { 0x76a600bc, 87 },
    { 0x74d600bc, 76 },
    { 0x57b600bb, 52 },
    { 0xc5f600bf, 17 },
    { 0xd34600c0, 90 },
    { 0xd54600bf, 33 },
    { 0x274600c0, 98 },
    { 0xd52600c3, 87 },
    { 0x9bf600c4, 95 },
    { 0x285600c3, 98 },
    { 0x30a600b5, 33 },
    { 0xf67600c7, 61 },
    { 0x3f7600c8, 34 },
    { 0xba1600c9, 33 },
    { 0x32b600ca, 33 },
    { 0x898600cb, 61 },
    { 0x1e5600cc, 33 },
    { 0x2d0600ca, 61 },
    { 0x123600c5, 100 },
    { 0x95e600cf, 34 },
    { 0x2d600d0, 81 },
    { 0xf16600d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582600db, 90 },
    { 0x625600dc, 33 },
    { 0xdf5600dd, 87 },
    { 0x7fc600dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17600e4, 34 },
    { 0, 0 },
    { 0x77b600e6, 33 },
    { 0, 0 },
    { 0xe28600e8, 93 },
    { 0x153600e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3600ec, 91 },
    { 0, 0 },
    { 0x134600ee, 90 },
    { 0x224600ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3600f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c600f6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x550600f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f60100, 61 },
    { 0x56060100, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0260108, 70 },
    { 0, 0 },
    { 0x77f6010a, 99 },
    { 0x51a6010b, 90 },
    { 0, 0 },
    { 0xc386010d, 61 },
    { 0, 0 },
    { 0x1a86010f, 34 },
    { 0x6ad6010f, 95 },
    { 0xd2360110, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99560116, 91 },
    { 0x41e60117, 90 },
    { 0xd5960116, 33 },
    { 0xd8560118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef760121, 70 },
    { 0x37360122, 95 },
    { 0xbf260123, 90 },
    { 0x4a960124, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f60127, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b6012a, 98 },
    { 0, 0 },
    { 0x1b26012c, 98 },
    { 0x6716012d, 34 },
    { 0xcd86012c, 90 },
    { 0, 0 },
    { 0xa8360130, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa796013b, 100 },
    { 0x7be6013c, 55 },
    { 0x366013d, 91 },
    { 0xf86013d, 61 },
    { 0x7ef6013f, 11 },
    { 0xf1a60140, 90 },
    { 0xd5b60141, 33 },
    { 0x3d560141, 61 },
    { 0, 0 },
    { 0x48160144, 23 },
    { 0x36960145, 91 },
    { 0, 0 },
    { 0xd8c60147, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a6014c, 77 },
    { 0xdf56014d, 61 },
    { 0, 0 },
    { 0x2f56014f, 90 },
    { 0x2cc6014f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6560153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb676015b, 67 },
    { 0x4766015c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46015f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26560163, 76 },
    { 0x48260164, 33 },
    { 0xf6260165, 52 },
    { 0x6160164, 43 },
    { 0, 0 },
    { 0x88460168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf660176, 93 },
    { 0x46060177, 84 },
    { 0xf8260177, 76 },
    { 0x7860176, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3716017c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce460181, 99 },
    { 0xcbd60182, 61 },
    { 0, 0 },
    { 0x27460184, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d6018a, 46 },
    { 0x29f6018a, 75 },
    { 0x8996018c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5156018f, 90 },
    { 0xc1d60190, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7360194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6460197, 33 },
    { 0x97560198, 98 },
    { 0x1ee60198, 95 },
    { 0, 0 },
    { 0x2d86019b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362601a3, 90 },
    { 0, 0 },
    { 0x508601a5, 78 },
    { 0x81e601a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30601a9, 34 },
    { 0, 0 },
    { 0x432601ab, 33 },
    { 0x7f7601ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0601b0, 90 },
    { 0x571601b1, 87 },
    { 0xa3601b2, 43 },
    { 0xd15601b3, 23 },
    { 0x691601b4, 91 },
    { 0xff9601b0, 78 },
    { 0x34c601b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22601ba, 46 },
    { 0x42f601bb, 46 },
    { 0xd61601ba, 98 },
    { 0, 0 },
    { 0xb18601be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5601c4, 95 },
    { 0x163601c5, 90 },
    { 0, 0 },
    { 0x861601c7, 43 },
    { 0, 0 },
    { 0x40601c9, 55 },
    { 0x658601ca, 99 },
    { 0xf2601c9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e601ce, 100 },
    { 0xde6601cf, 88 },
    { 0x270601d0, 55 },
    { 0, 0 },
    { 0x678601d2, 87 },
    { 0x2c2601d3, 33 },
    { 0xca5601d2, 100 },
    { 0xeb0601d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee601da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57601e4, 93 },
    { 0x2bd601e5, 76 },
    { 0xf54601e5, 95 },
    { 0xe42601e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x227601ea, 34 },
    { 0, 0 },
    { 0x73f601ec, 96 },
    { 0x2c1601ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d601f3, 23 },
    { 0xebb601f4, 76 },
    { 0xe35601f5, 76 },
    { 0, 0 },
    { 0x3ed601f7, 52 },
    { 0x28a601f7, 95 },
    { 0x70d601f9, 90 },
    { 0xfa2601fa, 99 },
    { 0, 0 },
    { 0x310601fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc601ff, 61 },
    { 0xc8260200, 23 },
    { 0, 0 },
    { 0x39d60202, 75 },
    { 0xe5860203, 55 },
    { 0x4f560203, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9960208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f66020c, 90 },
    { 0, 0 },
    { 0x6a36020e, 99 },
    { 0x3686020f, 76 },
    { 0x956020e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b960215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d56021a, 78 },
    { 0x1326021b, 98 },
    { 0, 0 },
    { 0x9146021d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5660220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7160224, 33 },
    { 0, 0 },
    { 0x4fa60226, 33 },
    { 0xf5e60227, 23 },
    { 0x6bb60227, 33 },
    { 0x85e60229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8236022d, 91 },
    { 0xccc6022e, 61 },
    { 0x7fd6022e, 23 },
    { 0x7e60230, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee60238, 34 },
    { 0, 0 },
    { 0x4d06023a, 90 },
    { 0, 0 },
    { 0xa286023c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39260240, 100 },
    { 0, 0 },
    { 0x74c60242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7760246, 98 },
    { 0, 0 },
    { 0x61a60248, 34 },
    { 0, 0 },
    { 0xd536024a, 98 },
    { 0, 0 },
    { 0xabf6024c, 61 },
    { 0, 0 },
    { 0x59d6024e, 95 },
    { 0x64a6024f, 55 },
    { 0, 0 },
    { 0x5e60251, 98 },
    { 0, 0 },
    { 0xde060253, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b260257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546025b, 33 },
    { 0xe296025c, 95 },
    { 0x5c6025d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e560264, 99 },
    { 0x80c60264, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d160268, 33 },
    { 0x54360269, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a6026f, 61 },
    { 0x34e60270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb6027a, 95 },
    { 0x1d16027b, 93 },
    { 0x886027c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad60287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6028a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a60294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f60298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce16029b, 95 },
    { 0, 0 },
    { 0x6916029d, 61 },
    { 0x6a6029d, 98 },
    { 0xc566029d, 84 },
    { 0xddf6029e, 90 },
    { 0xa82602a1, 33 },
    { 0x7886029f, 90 },
    { 0, 0 },
    { 0x534602a4, 67 },
    { 0xf5e602a4, 90 },
    { 0xf17602a6, 74 },
    { 0x663602a7, 95 },
    { 0x7f7602a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275602b3, 90 },
    { 0x938602b3, 99 },
    { 0x6f0602b5, 61 },
    { 0x5b3602b4, 78 },
    { 0x20b602b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x290602ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a602bd, 91 },
    { 0x547602bd, 76 },
    { 0xf6b602bf, 56 },
    { 0, 0 },
    { 0x384602c1, 98 },
    { 0, 0 },
    { 0x84b602c3, 90 },
    { 0xaaa602c4, 95 },
    { 0xd43602c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e602cb, 61 },
    { 0, 0 },
    { 0xed1602cd, 61 },
    { 0, 0 },
    { 0x82b602cf, 23 },
    { 0xf3c602cf, 77 },
    { 0x671602cf, 90 },
    { 0x312602d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682602d6, 33 },
    { 0x429602d7, 91 },
    { 0, 0 },
    { 0xe8e602d9, 67 },
    { 0x293602d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f602dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x629602e0, 67 },
    { 0x531602e1, 40 },
    { 0x6a4602e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e602e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a602ea, 33 },
    { 0x41e602eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd602f2, 76 },
    { 0x188602f2, 90 },
    { 0, 0 },
    { 0x90a602f5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2602f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf602fe, 33 },
    { 0x268602ff, 61 },
    { 0xd1660300, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1060303, 99 },
    { 0x62f60304, 90 },
    { 0xf1e60304, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa866030a, 100 },
    { 0x5f66030b, 61 },
    { 0x3806030a, 95 },
    { 0xf276030b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce60310, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d60313, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4960319, 78 },
    { 0xa726031a, 90 },
    { 0xc996031b, 84 },
    { 0x9c60319, 33 },
    { 0x9376031d, 81 },
    { 0x63a60319, 61 },
    { 0x8606031f, 98 },
    { 0x82d6031b, 55 },
    { 0, 0 },
    { 0xa060322, 90 },
    { 0xa4a60323, 68 },
    { 0xc6860322, 61 },
    { 0xebc60325, 34 },
    { 0, 0 },
    { 0x5eb60327, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc086032b, 66 },
    { 0xaee6032c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e60335, 33 },
    { 0x1960336, 33 },
    { 0x1f760336, 43 },
    { 0x84860336, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316033c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f260340, 68 },
    { 0, 0 },
    { 0x54760342, 90 },
    { 0, 0 },
    { 0x8d160344, 76 },
    { 0x75560345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14960348, 33 },
    { 0, 0 },
    { 0xa106034a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6034d, 46 },
    { 0x816034d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb060351, 33 },
    { 0x2c60352, 40 },
    { 0x4f760352, 95 },
    { 0xa5860354, 23 },
    { 0xa9560352, 80 },
    { 0x4e060356, 34 },
    { 0x5cc60357, 46 },
    { 0x12060358, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c96035d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67860361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a560365, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8460369, 90 },
    { 0x19e6036a, 85 },
    { 0xa66036b, 95 },
    { 0xa7f60369, 17 },
    { 0, 0 },
    { 0xb8c6036e, 23 },
    { 0x7076036e, 34 },
    { 0xea86036f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49960373, 84 },
    { 0xd2760374, 33 },
    { 0xe6e60373, 95 },
    { 0xf8260376, 98 },
    { 0, 0 },
    { 0x44860378, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b6037c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41960380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b60383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f360386, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe560389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6276038d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7860391, 98 },
    { 0x1660391, 90 },
    { 0xb2f60393, 95 },
    { 0xba960394, 61 },
    { 0x4d860393, 23 },
    { 0xec960396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad860399, 34 },
    { 0x1e66039a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc266039e, 90 },
    { 0, 0 },
    { 0xc50603a0, 95 },
    { 0xc33603a0, 76 },
    { 0x69f603a2, 98 },
    { 0x373603a1, 38 },
    { 0xd63603a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d603a7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4603aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb603b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c603b4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93603b8, 68 },
    { 0x8e9603b8, 90 },
    { 0x1f7603b8, 23 },
    { 0xdfc603b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0603c0, 93 },
    { 0, 0 },
    { 0xaf5603c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e603c6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e603ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce603cd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c603d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958603d5, 95 },
    { 0, 0 },
    { 0x5ea603d7, 90 },
    { 0, 0 },
    { 0x3cc603d9, 46 },
    { 0, 0 },
    { 0x203603db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329603e5, 80 },
    { 0xb50603e6, 61 },
    { 0x655603e7, 93 },
    { 0xc41603e7, 52 },
    { 0x7cf603e9, 91 },
    { 0xdff603e7, 34 },
    { 0x234603e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae603ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763603f2, 99 },
    { 0xa8a603f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75603f7, 33 },
    { 0x7603f8, 33 },
    { 0x9b603f9, 91 },
    { 0x471603fa, 43 },
    { 0x31d603f7, 23 },
    { 0xfa4603f7, 61 },
    { 0xbbc603fd, 61 },
    { 0x51e603fe, 23 },
    { 0, 0 },
    { 0x6cb60400, 76 },
    { 0x1b60400, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21360404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76460408, 65 },
    { 0x2ea60409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f96040c, 93 },
    { 0x43b6040d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25560411, 95 },
    { 0x3f60412, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72560417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6041a, 61 },
    { 0xdab6041b, 17 },
    { 0, 0 },
    { 0x76e6041d, 75 },
    { 0x3326041d, 34 },
    { 0xd6041f, 90 },
    { 0x7da60420, 93 },
    { 0xde260420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52760424, 66 },
    { 0x57f60425, 17 },
    { 0x2c460425, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b76042a, 90 },
    { 0, 0 },
    { 0x6b06042c, 95 },
    { 0x12c6042d, 90 },
    { 0x3a46042d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7660431, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c960434, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d60437, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a6043b, 78 },
    { 0x2576043c, 37 },
    { 0, 0 },
    { 0xeac6043e, 76 },
    { 0, 0 },
    { 0x80760440, 91 },
    { 0x64d60441, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3260444, 84 },
    { 0x49b60445, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2376044d, 99 },
    { 0x4f86044e, 66 },
    { 0x2d76044f, 46 },
    { 0x33d60450, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x35660453, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97060457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f36045b, 90 },
    { 0, 0 },
    { 0x1d36045d, 61 },
    { 0x33f6045e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb960462, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6436046c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d60473, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf60476, 77 },
    { 0, 0 },
    { 0x9f960478, 93 },
    { 0x12c60479, 61 },
    { 0x33060478, 34 },
    { 0x5996047b, 98 },
    { 0x51b6047c, 68 },
    { 0x3036047d, 95 },
    { 0x51b6047e, 61 },
    { 0x20e6047e, 90 },
    { 0x1cb60480, 75 },
    { 0x9f660478, 95 },
    { 0x7bd60482, 81 },
    { 0xf2660483, 23 },
    { 0x19960484, 46 },
    { 0x9b260484, 95 },
    { 0x9c260486, 64 },
    { 0x26660485, 75 },
    { 0x57660488, 23 },
    { 0xac860489, 90 },
    { 0x3db6048a, 61 },
    { 0xc9a6048a, 43 },
    { 0x5f060482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d760490, 90 },
    { 0xcdd60491, 33 },
    { 0x1c360491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c60499, 23 },
    { 0, 0 },
    { 0x4586049b, 61 },
    { 0x6726049c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5604a0, 98 },
    { 0xf79604a1, 61 },
    { 0x70f604a2, 76 },
    { 0xbcf604a3, 98 },
    { 0x6d4604a3, 90 },
    { 0xbd4604a5, 98 },
    { 0x10b604a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49604aa, 55 },
    { 0, 0 },
    { 0xa23604ac, 23 },
    { 0xf2b604ac, 52 },
    { 0x2be604ae, 75 },
    { 0xa49604ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8604b6, 95 },
    { 0, 0 },
    { 0xc7604b8, 43 },
    { 0xb32604b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6604bd, 36 },
    { 0xf65604bd, 76 },
    { 0, 0 },
    { 0xaaa604c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6604c6, 33 },
    { 0x28e604c7, 90 },
    { 0x1ee604c8, 23 },
    { 0x7ef604c8, 95 },
    { 0x124604c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b604cf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x210604d2, 61 },
    { 0, 0 },
    { 0x26b604d4, 90 },
    { 0x74b604d4, 90 },
    { 0x6de604d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22604d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4604dc, 95 },
    { 0xfad604dd, 98 },
    { 0, 0 },
    { 0xbca604df, 52 },
    { 0x8de604e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638604e7, 33 },
    { 0x2a7604e8, 61 },
    { 0xa2604e9, 23 },
    { 0x1e604ea, 33 },
    { 0, 0 },
    { 0x259604ec, 52 },
    { 0xb93604ed, 90 },
    { 0x1bc604ee, 81 },
    { 0x55d604ef, 90 },
    { 0, 0 },
    { 0x26604f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd604f8, 90 },
    { 0x56604f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e604fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c604ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd60506, 95 },
    { 0x56b60507, 98 },
    { 0x4db60508, 75 },
    { 0xae60509, 95 },
    { 0xb9c60509, 98 },
    { 0x5a26050b, 95 },
    { 0x6876050c, 23 },
    { 0xe1a6050c, 92 },
    { 0x6ed60508, 61 },
    { 0xa4f6050f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd760516, 33 },
    { 0x14360517, 98 },
    { 0x1560518, 55 },
    { 0xdfb60519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9066051e, 61 },
    { 0, 0 },
    { 0x7ea60520, 95 },
    { 0, 0 },
    { 0x96b60522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc60528, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9186052c, 46 },
    { 0x70d6052c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa960531, 33 },
    { 0, 0 },
    { 0x8d560533, 95 },
    { 0x37e60534, 81 },
    { 0, 0 },
    { 0xa260536, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a260539, 33 },
    { 0x7026053a, 84 },
    { 0x3ad60539, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a6053f, 91 },
    { 0x79e60540, 33 },
    { 0x79a60540, 43 },
    { 0x15460541, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead6054b, 90 },
    { 0x71a6054b, 95 },
    { 0x8406054b, 43 },
    { 0xbea6054e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b60553, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8146055b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94260564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b6056a, 61 },
    { 0x5d96056a, 46 },
    { 0, 0 },
    { 0x6b66056d, 91 },
    { 0xbf16056e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d60572, 87 },
    { 0xb3f60573, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee76057d, 93 },
    { 0x4ea6057d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a260586, 80 },
    { 0xc2560586, 23 },
    { 0, 0 },
    { 0xfce60589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a6058c, 100 },
    { 0xb376058d, 90 },
    { 0xf226058d, 61 },
    { 0xc076058e, 23 },
    { 0, 0 },
    { 0xee460591, 77 },
    { 0x61d60591, 34 },
    { 0, 0 },
    { 0x95160594, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x57660597, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b76059b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601605a4, 46 },
    { 0x2e6605a5, 33 },
    { 0xbc8605a5, 80 },
    { 0xbb4605a7, 23 },
    { 0x4db605a7, 33 },
    { 0x365605a9, 61 },
    { 0xf69605aa, 37 },
    { 0x9af605aa, 33 },
    { 0, 0 },
    { 0xc82605ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4605b0, 34 },
    { 0x19e605b1, 90 },
    { 0, 0 },
    { 0xff0605b3, 95 },
    { 0x1f5605b3, 33 },
    { 0xd50605b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4605b8, 78 },
    { 0, 0 },
    { 0xb83605ba, 91 },
    { 0x195605bb, 55 },
    { 0xad1605bb, 33 },
    { 0xe85605ba, 33 },
    { 0, 0 },
    { 0xefc605bf, 55 },
    { 0, 0 },
    { 0xb87605c1, 99 },
    { 0x9a6605c2, 77 },
    { 0, 0 },
    { 0x512605c4, 55 },
    { 0, 0 },
    { 0xfad605c6, 33 },
    { 0xb43605c6, 23 },
    { 0x555605c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3605cb, 23 },
    { 0, 0 },
    { 0x6a5605cd, 97 },
    { 0x73c605cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716605db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93605e5, 93 },
    { 0, 0 },
    { 0x772605e7, 77 },
    { 0, 0 },
    { 0x95f605e9, 91 },
    { 0, 0 },
    { 0x4d9605eb, 81 },
    { 0xe64605ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb605ef, 90 },
    { 0, 0 },
    { 0xd28605f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a605f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387605ff, 37 },
    { 0x12360600, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e60604, 90 },
    { 0, 0 },
    { 0x78c60606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726060c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d6060f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7460614, 90 },
    { 0x25d60615, 76 },
    { 0x39e60614, 61 },
    { 0xc7260617, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf826061e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c60621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e60626, 78 },
    { 0x36c60627, 90 },
    { 0, 0 },
    { 0x10b60629, 43 },
    { 0, 0 },
    { 0xa26062b, 76 },
    { 0x5976062b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10860631, 77 },
    { 0x79060632, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2560639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c26063d, 85 },
    { 0xa166063e, 100 },
    { 0x8396063f, 90 },
    { 0x31c6063f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e60644, 61 },
    { 0xc4d60645, 46 },
    { 0xf6260645, 90 },
    { 0xf8260647, 34 },
    { 0, 0 },
    { 0x3ef60649, 65 },
    { 0x80360649, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe496064f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb860655, 90 },
    { 0xf3560656, 33 },
    { 0x7260657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b6065b, 90 },
    { 0x8646065b, 46 },
    { 0x3d76065d, 33 },
    { 0x5366065d, 76 },
    { 0x56b6065c, 76 },
    { 0x71060660, 55 },
    { 0x22e6065f, 90 },
    { 0, 0 },
    { 0x61760663, 95 },
    { 0x8a60664, 93 },
    { 0x4ba60665, 23 },
    { 0xaf260666, 52 },
    { 0x23060663, 91 },
    { 0xdc560666, 34 },
    { 0x84f60667, 17 },
    { 0xbf260665, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7976066f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10560673, 33 },
    { 0xb0860673, 34 },
    { 0x5b60675, 98 },
    { 0x4e060675, 23 },
    { 0xf2060675, 75 },
    { 0, 0 },
    { 0xae060679, 84 },
    { 0, 0 },
    { 0x46f6067b, 23 },
    { 0, 0 },
    { 0x1e46067d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb60680, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c60683, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa460686, 76 },
    { 0x9aa60687, 90 },
    { 0xe8060688, 46 },
    { 0x5fd60689, 98 },
    { 0xf6b6068a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c660692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f60695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd706069b, 93 },
    { 0xe936069b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52606a1, 33 },
    { 0xb5606a1, 23 },
    { 0x2ad606a1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a606a8, 76 },
    { 0xb83606a9, 90 },
    { 0, 0 },
    { 0x497606ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e606b0, 66 },
    { 0x4ce606b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e606b4, 53 },
    { 0x698606b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41606b9, 91 },
    { 0xa40606b9, 52 },
    { 0, 0 },
    { 0xe87606bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366606c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d606c3, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb606cf, 88 },
    { 0, 0 },
    { 0x1c7606d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c606d8, 23 },
    { 0x52b606d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c606df, 90 },
    { 0x1ae606e0, 90 },
    { 0x828606df, 61 },
    { 0xaea606e0, 81 },
    { 0x8ee606df, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8606e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db606ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23606ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc606f5, 55 },
    { 0x3ec606f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a606fb, 52 },
    { 0x4a7606fc, 55 },
    { 0x26d606fd, 90 },
    { 0xbca606fb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3860701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30160705, 95 },
    { 0xf6360706, 91 },
    { 0x1c860707, 98 },
    { 0xc1d60708, 68 },
    { 0xd1b60708, 33 },
    { 0, 0 },
    { 0xdea6070b, 46 },
    { 0xab76070c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18260712, 33 },
    { 0, 0 },
    { 0xb4960714, 76 },
    { 0x48d60714, 55 },
    { 0xdc660716, 34 },
    { 0x88b60716, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a6071c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e60720, 55 },
    { 0, 0 },
    { 0x5ed60722, 66 },
    { 0, 0 },
    { 0x32960724, 33 },
    { 0xa8f60725, 66 },
    { 0xf760726, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a6072a, 76 },
    { 0xe306072b, 90 },
    { 0, 0 },
    { 0x6686072d, 76 },
    { 0x8f06072d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f60735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec60739, 95 },
    { 0x1c16073a, 99 },
    { 0, 0 },
    { 0x2706073c, 87 },
    { 0, 0 },
    { 0xebe6073e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf696074b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f260757, 90 },
    { 0, 0 },
    { 0x2dc60759, 23 },
    { 0xb956075a, 93 },
    { 0x5146075a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b60761, 90 },
    { 0, 0 },
    { 0x12b60763, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68460767, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36076f, 95 },
    { 0, 0 },
    { 0xb5f60771, 61 },
    { 0, 0 },
    { 0x47e60773, 55 },
    { 0xeab60774, 95 },
    { 0x8c460773, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3960779, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6786077d, 84 },
    { 0x6846077d, 95 },
    { 0x3706077f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c60782, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb560785, 91 },
    { 0xa1860786, 98 },
    { 0xf5e60787, 23 },
    { 0x13160787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4360795, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa46079b, 61 },
    { 0x33f6079b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4646079f, 90 },
    { 0x95c607a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428607a6, 95 },
    { 0, 0 },
    { 0xe8d607a8, 17 },
    { 0x93f607a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a607ad, 83 },
    { 0x32607ad, 84 },
    { 0x4a7607af, 90 },
    { 0, 0 },
    { 0x8d607b1, 23 },
    { 0x28c607b2, 90 },
    { 0, 0 },
    { 0xbd7607b4, 81 },
    { 0x5f0607b4, 33 },
    { 0xe75607b6, 87 },
    { 0, 0 },
    { 0xda2607b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8607bb, 33 },
    { 0xf85607bb, 98 },
    { 0x681607bd, 52 },
    { 0xbd607bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09607c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14607c4, 90 },
    { 0, 0 },
    { 0x845607c6, 98 },
    { 0x738607c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473607cb, 55 },
    { 0x73e607cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d607d0, 95 },
    { 0xf42607d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b607d4, 90 },
    { 0x809607d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a607d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61607dd, 35 },
    { 0, 0 },
    { 0x9cc607df, 33 },
    { 0xe6f607e0, 76 },
    { 0x866607e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6607e4, 77 },
    { 0x8af607e4, 68 },
    { 0x6ca607e4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b607eb, 98 },
    { 0xe2b607ec, 90 },
    { 0xeee607ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8607f4, 34 },
    { 0x360607f5, 90 },
    { 0x8af607f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4607f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x856607fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b60803, 81 },
    { 0xfdd60804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71660807, 34 },
    { 0, 0 },
    { 0x96d60809, 85 },
    { 0xaa260809, 33 },
    { 0xd36080b, 23 },
    { 0x1666080c, 46 },
    { 0x5126080d, 90 },
    { 0x5cd6080d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d460812, 76 },
    { 0x9ae60813, 78 },
    { 0xf7460813, 90 },
    { 0xa7660815, 56 },
    { 0xe0860815, 99 },
    { 0x92760817, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6436081b, 100 },
    { 0xfe06081c, 61 },
    { 0xf8a6081d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96260821, 93 },
    { 0, 0 },
    { 0xcd560823, 46 },
    { 0x3b360824, 100 },
    { 0x4dd60825, 68 },
    { 0x64860826, 61 },
    { 0, 0 },
    { 0xc0860828, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa496082b, 78 },
    { 0xae96082b, 40 },
    { 0x6d6082d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe660830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b60836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a760841, 90 },
    { 0xbc260842, 90 },
    { 0xdb060843, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae60846, 98 },
    { 0x41960847, 55 },
    { 0, 0 },
    { 0x35660849, 61 },
    { 0x7456084a, 23 },
    { 0xbb56084a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d60851, 43 },
    { 0xa5f60852, 68 },
    { 0xccf60853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c66085d, 46 },
    { 0x6be6085d, 99 },
    { 0x9976085e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87360863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c60869, 87 },
    { 0xe6a60869, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f6086e, 100 },
    { 0, 0 },
    { 0xf2660870, 23 },
    { 0x54160871, 98 },
    { 0x5d660872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab60875, 23 },
    { 0x18d60875, 33 },
    { 0, 0 },
    { 0xeab60878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b6087b, 98 },
    { 0xd0e6087b, 33 },
    { 0xd266087b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f60880, 33 },
    { 0x53660881, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3860885, 98 },
    { 0x92060885, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6660890, 23 },
    { 0, 0 },
    { 0x90260892, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6089b, 65 },
    { 0, 0 },
    { 0xd306089d, 76 },
    { 0x44d6089e, 90 },
    { 0xf766089f, 75 },
    { 0x6dc608a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0608a4, 95 },
    { 0, 0 },
    { 0x6cf608a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f608ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3608b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5608b6, 93 },
    { 0x98c608b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de608ba, 61 },
    { 0x11608bb, 90 },
    { 0x69b608bc, 65 },
    { 0, 0 },
    { 0x4a608be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e608c3, 98 },
    { 0x5d9608c4, 99 },
    { 0xa02608c4, 23 },
    { 0x198608c3, 33 },
    { 0, 0 },
    { 0x1dc608c8, 24 },
    { 0, 0 },
    { 0xc17608ca, 76 },
    { 0x2db608ca, 76 },
    { 0, 0 },
    { 0x3e1608cd, 90 },
    { 0xfd3608ce, 100 },
    { 0, 0 },
    { 0x61d608d0, 55 },
    { 0x175608d1, 55 },
    { 0x29608d2, 90 },
    { 0, 0 },
    { 0xcab608d4, 99 },
    { 0xa18608d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66608e1, 17 },
    { 0, 0 },
    { 0xdc4608e3, 90 },
    { 0x1a6608e4, 55 },
    { 0xb6b608e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b608ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e608f3, 84 },
    { 0, 0 },
    { 0x741608f5, 23 },
    { 0, 0 },
    { 0x7d4608f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f608fa, 98 },
    { 0xa4b608fa, 61 },
    { 0, 0 },
    { 0x953608fd, 95 },
    { 0xd97608fe, 80 },
    { 0xb6608ff, 91 },
    { 0xedd60900, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19360904, 91 },
    { 0x26260904, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc436090a, 90 },
    { 0x7956090b, 95 },
    { 0xc366090c, 99 },
    { 0, 0 },
    { 0xb466090e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38060911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e60919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb86091d, 61 },
    { 0x1d26091e, 61 },
    { 0xb9f6091f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91660924, 90 },
    { 0, 0 },
    { 0x27160926, 23 },
    { 0x91e60927, 23 },
    { 0x4cf60927, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf516092b, 90 },
    { 0, 0 },
    { 0x3486092d, 55 },
    { 0, 0 },
    { 0x7f66092f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c60933, 43 },
    { 0x40c60933, 34 },
    { 0x3a60935, 78 },
    { 0x63460933, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89260940, 76 },
    { 0x74360940, 90 },
    { 0x52060941, 55 },
    { 0xdef60940, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0260947, 62 },
    { 0x59260947, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f60951, 67 },
    { 0xc8d60952, 39 },
    { 0x98b60951, 33 },
    { 0x28160953, 85 },
    { 0xf4c60955, 33 },
    { 0x76360956, 90 },
    { 0x5cf60952, 98 },
    { 0xe2460958, 56 },
    { 0x10060952, 87 },
    { 0x4ed60958, 76 },
    { 0x30260951, 43 },
    { 0xee6095c, 90 },
    { 0xcd56095c, 33 },
    { 0x49e60951, 98 },
    { 0xa226095f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1560962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a60966, 46 },
    { 0xbd860966, 84 },
    { 0x33f60968, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab86096d, 23 },
    { 0xcbf6096e, 76 },
    { 0, 0 },
    { 0xfe360970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61460976, 98 },
    { 0x7d060977, 64 },
    { 0x20360977, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d6097c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b76097f, 90 },
    { 0x7d960980, 84 },
    { 0x97260981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc60985, 90 },
    { 0x7660985, 33 },
    { 0x7a760985, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c06098c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18160993, 33 },
    { 0, 0 },
    { 0x5e860995, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d60999, 98 },
    { 0, 0 },
    { 0xe676099b, 58 },
    { 0, 0 },
    { 0xf516099d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4609a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54609a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18609aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x162609ad, 23 },
    { 0x646609ad, 84 },
    { 0x872609af, 53 },
    { 0, 0 },
    { 0xb81609b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0609b5, 33 },
    { 0x7d8609b5, 93 },
    { 0x963609b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x870609ba, 33 },
    { 0, 0 },
    { 0x609bc, 61 },
    { 0x2f3609bd, 90 },
    { 0x4d1609bd, 100 },
    { 0xf4f609be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff609c4, 17 },
    { 0x3e0609c5, 33 },
    { 0xc13609c4, 90 },
    { 0xe95609c5, 84 },
    { 0xe84609c4, 43 },
    { 0x6ad609c9, 23 },
    { 0, 0 },
    { 0xb26609cb, 34 },
    { 0, 0 },
    { 0x33c609cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e609d0, 33 },
    { 0x4c6609d1, 61 },
    { 0x75d609d1, 43 },
    { 0xa39609d3, 55 },
    { 0, 0 },
    { 0xe9c609d5, 90 },
    { 0x36f609d6, 23 },
    { 0x10c609d7, 61 },
    { 0xf8f609d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8609db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296609e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4609ec, 23 },
    { 0x28e609ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597609f6, 33 },
    { 0, 0 },
    { 0x36f609f8, 90 },
    { 0x654609f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c609fd, 81 },
    { 0x837609fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f60a11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5460a14, 43 },
    { 0, 0 },
    { 0x1f160a16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31260a19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c60a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e60a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5360a26, 66 },
    { 0x26560a27, 76 },
    { 0xd3660a26, 90 },
    { 0x89560a29, 34 },
    { 0xde660a2a, 90 },
    { 0, 0 },
    { 0x30c60a2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7160a30, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf460a38, 81 },
    { 0, 0 },
    { 0xcce60a3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72160a41, 43 },
    { 0x6560a42, 93 },
    { 0xb8d60a42, 90 },
    { 0x84760a44, 76 },
    { 0, 0 },
    { 0x85d60a46, 77 },
    { 0x3ac60a47, 95 },
    { 0, 0 },
    { 0xbbf60a49, 36 },
    { 0x9bd60a4a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81460a4f, 70 },
    { 0xd0760a50, 99 },
    { 0x97f60a51, 61 },
    { 0x80e60a4f, 61 },
    { 0x66560a53, 46 },
    { 0xb5a60a54, 90 },
    { 0x4060a54, 90 },
    { 0, 0 },
    { 0xdec60a57, 95 },
    { 0x84260a57, 98 },
    { 0xa9660a59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a60a62, 100 },
    { 0x95e60a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b60a66, 36 },
    { 0, 0 },
    { 0x82660a68, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x49360a6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4160a72, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x20860a75, 66 },
    { 0, 0 },
    { 0x2a660a77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b60a7a, 90 },
    { 0x67260a7b, 76 },
    { 0xe1d60a7b, 61 },
    { 0x10c60a7d, 98 },
    { 0, 0 },
    { 0xceb60a7f, 61 },
    { 0, 0 },
    { 0xf8d60a81, 84 },
    { 0xd0660a81, 34 },
    { 0, 0 },
    { 0xfae60a84, 76 },
    { 0x59b60a85, 80 },
    { 0x10a60a86, 91 },
    { 0x38760a87, 38 },
    { 0x17160a88, 61 },
    { 0x42360a89, 36 },
    { 0x7a460a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7060a8d, 98 },
    { 0x5cf60a8d, 98 },
    { 0xe3f60a8f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d60a93, 90 },
    { 0x3ce60a93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9560a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda460a9c, 33 },
    { 0x7ad60a9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3860aa0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1860aa5, 23 },
    { 0x9360aa6, 32 },
    { 0x22d60aa7, 61 },
    { 0x8660aa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16260ab1, 66 },
    { 0x1cc60ab1, 33 },
    { 0x9b660ab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f760abb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20060abf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62060ac4, 40 },
    { 0xd7a60ac4, 76 },
    { 0xb7560ac6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8660aca, 98 },
    { 0, 0 },
    { 0x2e060acc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda60ad3, 34 },
    { 0xc2b60ad4, 75 },
    { 0xe7460ad3, 43 },
    { 0x72860ad4, 90 },
    { 0xede60ad7, 23 },
    { 0xa7660ad8, 75 },
    { 0x87260ad6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc360ade, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e560ae1, 61 },
    { 0, 0 },
    { 0xaa560ae3, 34 },
    { 0x91260ae4, 67 },
    { 0x83f60ae5, 91 },
    { 0xa0360ae6, 23 },
    { 0x80660ae5, 33 },
    { 0x46460ae8, 33 },
    { 0xba060ae6, 61 },
    { 0xd3f60aea, 90 },
    { 0xd3e60aea, 55 },
    { 0xed960ae5, 90 },
    { 0xa6560ae7, 43 },
    { 0, 0 },
    { 0x45060aef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff160af3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55960af8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10760afc, 23 },
    { 0x3ea60afd, 91 },
    { 0x67160afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c760b03, 61 },
    { 0x42b60b04, 34 },
    { 0, 0 },
    { 0x1fc60b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a60b0c, 76 },
    { 0x8760b0d, 23 },
    { 0x89860b0e, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5860b13, 33 },
    { 0xd0160b14, 68 },
    { 0x93c60b14, 61 },
    { 0xacc60b14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9060b1a, 90 },
    { 0, 0 },
    { 0xc5560b1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6960b20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a060b25, 88 },
    { 0x2f960b26, 85 },
    { 0x3e160b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb60b30, 43 },
    { 0x61260b31, 23 },
    { 0x7d60b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76260b35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8960b3d, 95 },
    { 0x29560b3e, 84 },
    { 0xc5e60b3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30060b46, 93 },
    { 0, 0 },
    { 0x22860b48, 100 },
    { 0, 0 },
    { 0xfc160b4a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34360b51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec60b55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a560b58, 34 },
    { 0, 0 },
    { 0xa5060b5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee60b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf60b63, 66 },
    { 0x97c60b63, 90 },
    { 0x23860b65, 100 },
    { 0, 0 },
    { 0x95660b67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xed860b6a, 78 },
    { 0x47260b6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e060b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e760b73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9160b7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc960b7f, 23 },
    { 0x42e60b80, 91 },
    { 0x4b560b81, 34 },
    { 0xd0a60b80, 61 },
    { 0, 0 },
    { 0x2b060b84, 93 },
    { 0, 0 },
    { 0xfc560b86, 52 },
    { 0x22560b87, 76 },
    { 0xa8a60b86, 95 },
    { 0x3ae60b87, 95 },
    { 0xed760b8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c060b91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca60b94, 33 },
    { 0, 0 },
    { 0x6b960b96, 90 },
    { 0x3b460b97, 90 },
    { 0, 0 },
    { 0x7b260b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7760b9d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c760ba0, 33 },
    { 0x79760ba1, 61 },
    { 0x7c260ba2, 43 },
    { 0x89560ba0, 98 },
    { 0xef060ba4, 90 },
    { 0xe6c60ba5, 90 },
    { 0xca60ba2, 33 },
    { 0x67e60ba4, 61 },
    { 0x79e60ba8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d60bab, 90 },
    { 0, 0 },
    { 0x17460bad, 85 },
    { 0xa3e60bae, 98 },
    { 0x3ab60bae, 61 },
    { 0x69c60bad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c60bb4, 61 },
    { 0xe3c60bb4, 81 },
    { 0, 0 },
    { 0x6db60bb7, 34 },
    { 0x36e60bb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a060bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d460bc6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c360bc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63960bd1, 78 },
    { 0, 0 },
    { 0x72560bd3, 90 },
    { 0, 0 },
    { 0x7e460bd5, 61 },
    { 0x4dd60bd6, 95 },
    { 0x53060bd5, 61 },
    { 0x16760bd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3760be0, 23 },
    { 0xf4260be0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63760be9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0660bee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a60bf7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc360bfe, 90 },
    { 0x24760bff, 98 },
    { 0xa0860c00, 33 },
    { 0x4bf60bff, 90 },
    { 0x6de60c01, 91 },
    { 0xc0360c03, 91 },
    { 0x5a660c03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf60c09, 36 },
    { 0xb0e60c0a, 65 },
    { 0x61260c0b, 61 },
    { 0x77360c0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2760c0f, 98 },
    { 0x2fb60c10, 33 },
    { 0xdb460c11, 94 },
    { 0, 0 },
    { 0x6b260c13, 93 },
    { 0, 0 },
    { 0x10c60c15, 52 },
    { 0, 0 },
    { 0x8da60c17, 46 },
    { 0, 0 },
    { 0xd660c19, 33 },
    { 0x85660c1a, 45 },
    { 0x97960c1a, 61 },
    { 0x1a260c1b, 69 },
    { 0x4b360c1d, 87 },
    { 0xa7460c1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77760c21, 43 },
    { 0x60a60c22, 90 },
    { 0x99360c23, 90 },
    { 0x73b60c23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41360c2b, 17 },
    { 0x59b60c2c, 65 },
    { 0x16c60c2c, 76 },
    { 0xdf360c2e, 52 },
    { 0xa1360c2b, 98 },
    { 0, 0 },
    { 0x1960c31, 33 },
    { 0, 0 },
    { 0x90160c33, 33 },
    { 0xd8d60c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3260c37, 90 },
    { 0xe1d60c37, 22 },
    { 0xb1660c39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c60c3c, 43 },
    { 0, 0 },
    { 0x6cf60c3e, 76 },
    { 0, 0 },
    { 0x18360c40, 43 },
    { 0x45f60c40, 61 },
    { 0x90960c42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e60c46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e560c4b, 90 },
    { 0xc8b60c4b, 33 },
    { 0xcda60c4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50160c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a60c54, 93 },
    { 0x7160c54, 33 },
    { 0x5b160c56, 33 },
    { 0x9e60c56, 61 },
    { 0x61c60c55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91160c5b, 76 },
    { 0xd0260c5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc60c5f, 61 },
    { 0, 0 },
    { 0x2fc60c61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae60c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70060c69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91960c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec60c71, 95 },
    { 0xc8a60c72, 66 },
    { 0, 0 },
    { 0xe8a60c74, 34 },
    { 0xf2160c75, 34 },
    { 0x54b60c76, 23 },
    { 0, 0 },
    { 0xc3160c78, 95 },
    { 0, 0 },
    { 0xe560c7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e460c7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2960c80, 46 },
    { 0x2e560c80, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb60c84, 23 },
    { 0x30e60c85, 46 },
    { 0x76f60c86, 99 },
    { 0x61360c87, 84 },
    { 0xc5960c88, 55 },
    { 0x7ec60c87, 99 },
    { 0, 0 },
    { 0x30860c8b, 98 },
    { 0xb3f60c8b, 23 },
    { 0x96760c8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45060c93, 76 },
    { 0xf9660c94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5960c97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x52460c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5060ca0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe260ca5, 90 },
    { 0xa6c60ca6, 61 },
    { 0x3d760ca7, 61 },
    { 0x85160ca5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57460cac, 52 },
    { 0x87d60cad, 90 },
    { 0, 0 },
    { 0x4bd60caf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff60cb2, 33 },
    { 0x8e460cb3, 69 },
    { 0xeb060cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7960cbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2060cc0, 95 },
    { 0, 0 },
    { 0xe260cc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38460cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4060ccb, 23 },
    { 0x41a60ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a60cd4, 56 },
    { 0, 0 },
    { 0xf1560cd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6360ce7, 97 },
    { 0x22360ce8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf60ceb, 34 },
    { 0, 0 },
    { 0x2a460ced, 95 },
    { 0x79d60cee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c560cf2, 77 },
    { 0x44060cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9760cf8, 90 },
    { 0xde760cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0160cfd, 23 },
    { 0, 0 },
    { 0x74a60cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa960d06, 36 },
    { 0xe3e60d07, 61 },
    { 0xb8260d08, 33 },
    { 0xcc060d09, 23 },
    { 0x5a60d0a, 66 },
    { 0x38260d0b, 90 },
    { 0xb5960d0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c60d11, 98 },
    { 0, 0 },
    { 0x4360d13, 61 },
    { 0xa0360d14, 98 },
    { 0x9a060d13, 90 },
    { 0x38e60d13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99560d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51160d1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c60d23, 98 },
    { 0, 0 },
    { 0x20960d25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe960d29, 84 },
    { 0, 0 },
    { 0x35760d2b, 98 },
    { 0, 0 },
    { 0x47360d2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5960d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47860d38, 90 },
    { 0xb5360d39, 55 },
    { 0, 0 },
    { 0x12c60d3b, 34 },
    { 0x62560d3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32260d3f, 33 },
    { 0x2dc60d3f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86660d45, 66 },
    { 0xa7660d45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20360d49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8660d50, 55 },
    { 0x69860d50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45460d57, 95 },
    { 0xd6160d57, 99 },
    { 0x10960d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e260d5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c160d62, 76 },
    { 0, 0 },
    { 0xa2f60d64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6860d67, 76 },
    { 0xae860d67, 98 },
    { 0, 0 },
    { 0xfc60d6a, 76 },
    { 0x77460d6b, 33 },
    { 0, 0 },
    { 0x9c460d6d, 66 },
    { 0, 0 },
    { 0xc6d60d6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c60d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc60d75, 61 },
    { 0, 0 },
    { 0xa5f60d77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e60d7b, 91 },
    { 0x43360d7b, 61 },
    { 0xd5e60d7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea60d80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60060d84, 90 },
    { 0x11460d85, 90 },
    { 0, 0 },
    { 0xeb060d87, 61 },
    { 0x2ee60d88, 85 },
    { 0xef460d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44160d8d, 84 },
    { 0xc1760d8d, 33 },
    { 0xc1e60d8f, 55 },
    { 0x9dd60d90, 33 },
    { 0x53260d91, 33 },
    { 0x50c60d8d, 33 },
    { 0x4cb60d93, 90 },
    { 0x14760d91, 43 },
    { 0x77d60d8d, 76 },
    { 0xb0760d8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e760d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74460da3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c60da8, 23 },
    { 0, 0 },
    { 0x39b60daa, 23 },
    { 0, 0 },
    { 0xfff60dac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea60daf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d560db2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4760db5, 98 },
    { 0x2fe60db6, 78 },
    { 0xaed60db6, 90 },
    { 0xcd560db8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17960dbf, 78 },
    { 0, 0 },
    { 0x3c160dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e60dc4, 34 },
    { 0x91360dc5, 55 },
    { 0xd8860dc4, 33 },
    { 0x8d860dc7, 84 },
    { 0x3ea60dc4, 99 },
    { 0x6c060dc9, 46 },
    { 0x8de60dc4, 91 },
    { 0xea460dcb, 95 },
    { 0x27760dcb, 61 },
    { 0x9b560dcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb60dd4, 81 },
    { 0x3a60dd4, 61 },
    { 0x3c60dd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd60dda, 23 },
    { 0xc6360ddb, 76 },
    { 0, 0 },
    { 0x65b60ddd, 78 },
    { 0xbfd60dde, 90 },
    { 0, 0 },
    { 0x9ba60de0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a260dea, 46 },
    { 0, 0 },
    { 0xf1160dec, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3760df1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef060df6, 33 },
    { 0, 0 },
    { 0x3ed60df8, 75 },
    { 0x47360df9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c60dfd, 90 },
    { 0, 0 },
    { 0xb9560dff, 55 },
    { 0, 0 },
    { 0x84260e01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda60e07, 33 },
    { 0x7c960e08, 76 },
    { 0x89760e09, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20560e0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d060e11, 95 },
    { 0xed60e11, 76 },
    { 0x94060e13, 61 },
    { 0x84260e14, 95 },
    { 0, 0 },
    { 0x47560e16, 33 },
    { 0, 0 },
    { 0xf3260e18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53760e1e, 23 },
    { 0, 0 },
    { 0x55660e20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2160e24, 33 },
    { 0xd5660e25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c60e29, 76 },
    { 0, 0 },
    { 0x25060e2b, 33 },
    { 0x4760e2c, 90 },
    { 0, 0 },
    { 0x87860e2e, 66 },
    { 0xb2c60e2f, 23 },
    { 0xa1d60e2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d60e34, 23 },
    { 0, 0 },
    { 0xf9360e36, 38 },
    { 0x4b60e37, 55 },
    { 0xbeb60e38, 53 },
    { 0x1eb60e38, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x81160e3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e660e42, 84 },
    { 0xe0060e42, 33 },
    { 0x83760e43, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a060e47, 56 },
    { 0xaa460e48, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e60e4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e60e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd460e57, 23 },
    { 0x6f660e58, 98 },
    { 0, 0 },
    { 0xc0060e5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c60e5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd760e66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec760e69, 61 },
    { 0x4c760e6a, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b60e6e, 98 },
    { 0x7c960e6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f860e76, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d60e7b, 76 },
    { 0xdf60e7c, 23 },
    { 0xe4660e7c, 33 },
    { 0, 0 },
    { 0x18060e7f, 46 },
    { 0, 0 },
    { 0x8fd60e81, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e560e84, 76 },
    { 0, 0 },
    { 0x40d60e86, 99 },
    { 0, 0 },
    { 0x5aa60e88, 85 },
    { 0x48a60e89, 93 },
    { 0, 0 },
    { 0xfc260e8b, 52 },
    { 0x5fd60e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac060e91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4560e94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da60e98, 69 },
    { 0x86c60e98, 17 },
    { 0x4b460e9a, 98 },
    { 0x9d660e99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa60e9f, 61 },
    { 0xc5360ea0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e60ea3, 23 },
    { 0x58460ea4, 69 },
    { 0x3bf60ea5, 23 },
    { 0x50160ea6, 98 },
    { 0, 0 },
    { 0x58160ea8, 61 },
    { 0x12c60ea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf360eb0, 46 },
    { 0x80a60eb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf60eb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3560eb8, 85 },
    { 0x7a060eb9, 81 },
    { 0x74f60eb9, 90 },
    { 0xae760ebb, 91 },
    { 0, 0 },
    { 0x9f60ebd, 98 },
    { 0x6ea60ebe, 90 },
    { 0xab860ebf, 97 },
    { 0x33260ec0, 52 },
    { 0x21b60ebd, 100 },
    { 0x76060ec2, 61 },
    { 0x76460ec3, 33 },
    { 0x75e60ec4, 77 },
    { 0x97d60ec1, 76 },
    { 0x5e960ec6, 88 },
    { 0xb7760ebf, 61 },
    { 0x9cf60ec4, 61 },
    { 0, 0 },
    { 0x91f60eca, 23 },
    { 0x75360eca, 61 },
    { 0xadf60ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f60ed0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a60ed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d60edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c60edf, 43 },
    { 0xb8160edf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf60ee3, 95 },
    { 0x22a60ee3, 90 },
    { 0x75560ee5, 61 },
    { 0x1cc60ee3, 67 },
    { 0x97f60ee3, 33 },
    { 0x6cc60ee8, 98 },
    { 0xb2a60ee9, 90 },
    { 0x1e260eea, 91 },
    { 0xd5b60eeb, 23 },
    { 0x51d60eec, 76 },
    { 0xb8f60ee9, 23 },
    { 0, 0 },
    { 0xa60eef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29960ef3, 61 },
    { 0xdd460ef3, 95 },
    { 0, 0 },
    { 0x24060ef6, 46 },
    { 0xbfb60ef7, 91 },
    { 0x5b960ef6, 98 },
    { 0x31c60ef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b60efc, 46 },
    { 0x6eb60efd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df60f05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff160f0a, 95 },
    { 0xe60f0b, 98 },
    { 0, 0 },
    { 0x57360f0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dd60f12, 85 },
    { 0xa1a60f13, 43 },
    { 0x69460f14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e60f1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc160f20, 23 },
    { 0x9c960f20, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b60f28, 90 },
    { 0x53660f29, 87 },
    { 0xe0860f29, 93 },
    { 0xf2560f2a, 98 },
    { 0, 0 },
    { 0x4e460f2d, 90 },
    { 0, 0 },
    { 0x5bd60f2f, 95 },
    { 0x45360f30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e60f38, 90 },
    { 0x98460f39, 61 },
    { 0x38960f3a, 87 },
    { 0x70560f3b, 98 },
    { 0x4360f3c, 65 },
    { 0, 0 },
    { 0x26360f3e, 84 },
    { 0x50a60f3f, 98 },
    { 0x5c060f40, 33 },
    { 0x5af60f41, 76 },
    { 0x5fe60f42, 46 },
    { 0x36960f43, 76 },
    { 0x3ff60f41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb360f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87560f4c, 90 },
    { 0, 0 },
    { 0x31460f4e, 90 },
    { 0, 0 },
    { 0xe2e60f50, 55 },
    { 0xdb260f51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e60f59, 33 },
    { 0x3bc60f59, 33 },
    { 0x31860f5a, 53 },
    { 0x7d360f5c, 61 },
    { 0x21a60f5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab60f62, 95 },
    { 0xae560f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35760f68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c60f6c, 90 },
    { 0xf8c60f6d, 43 },
    { 0x9af60f6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c60f71, 61 },
    { 0x7b360f72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6360f77, 23 },
    { 0xb3560f77, 95 },
    { 0x51060f79, 90 },
    { 0, 0 },
    { 0x54360f7b, 99 },
    { 0xe6d60f7c, 93 },
    { 0x72c60f7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd860f80, 66 },
    { 0, 0 },
    { 0xbb760f82, 99 },
    { 0xc0c60f83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81660f89, 81 },
    { 0, 0 },
    { 0xc7760f8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc060f8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89960f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b060fa2, 98 },
    { 0x25760fa2, 98 },
    { 0xb6d60fa4, 34 },
    { 0x82c60fa5, 76 },
    { 0, 0 },
    { 0xb5f60fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e060fab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84460faf, 100 },
    { 0x20b60fb0, 95 },
    { 0x8660fb0, 98 },
    { 0xc9660fb2, 95 },
    { 0x3e760faf, 61 },
    { 0x5dc60fb4, 90 },
    { 0x2fd60fb3, 34 },
    { 0xe2360fb5, 61 },
    { 0xb260fb2, 100 },
    { 0xcb360fb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f60fbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb560fc1, 78 },
    { 0x5c060fc2, 98 },
    { 0x8d560fc1, 48 },
    { 0x2cb60fc4, 61 },
    { 0, 0 },
    { 0x23e60fc6, 87 },
    { 0xa6d60fc7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd060fce, 68 },
    { 0, 0 },
    { 0x43a60fd0, 33 },
    { 0x29f60fd1, 61 },
    { 0xc8760fd2, 46 },
    { 0, 0 },
    { 0xac260fd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a60fdc, 85 },
    { 0x3760fdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a60fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc560fed, 33 },
    { 0x75960fed, 78 },
    { 0x39e60fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c60ff9, 55 },
    { 0, 0 },
    { 0x7a660ffb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf60fff, 90 },
    { 0, 0 },
    { 0xf3a61001, 78 },
    { 0x63f61001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a61005, 99 },
    { 0, 0 },
    { 0x2c761007, 91 },
    { 0xb5361008, 33 },
    { 0xdab61009, 90 },
    { 0x3926100a, 23 },
    { 0x6c26100b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226100e, 55 },
    { 0x1ed6100f, 33 },
    { 0, 0 },
    { 0x5bb61011, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a261017, 61 },
    { 0x9461018, 33 },
    { 0x13061019, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef56101d, 23 },
    { 0, 0 },
    { 0x25f6101f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xda561022, 33 },
    { 0x39b61023, 43 },
    { 0xf9061024, 23 },
    { 0xb5b61025, 34 },
    { 0x29e61026, 75 },
    { 0x28661027, 88 },
    { 0x9ac61027, 98 },
    { 0x6bb61029, 66 },
    { 0x2e361022, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e56102f, 61 },
    { 0x4a361030, 88 },
    { 0, 0 },
    { 0x17c61032, 34 },
    { 0, 0 },
    { 0x94c61034, 98 },
    { 0x4c961035, 95 },
    { 0xbd361034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776103f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad61043, 90 },
    { 0xcca61044, 66 },
    { 0, 0 },
    { 0x55c61046, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c76104e, 95 },
    { 0x3e16104f, 77 },
    { 0x7c76104e, 48 },
    { 0xecb61051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38661054, 61 },
    { 0x1eb61054, 23 },
    { 0, 0 },
    { 0x30061057, 34 },
    { 0x71a61058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e6105b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a56105e, 91 },
    { 0xd526105f, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff61062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b61066, 76 },
    { 0xf6761067, 99 },
    { 0x35b61068, 34 },
    { 0, 0 },
    { 0x3a26106a, 67 },
    { 0x74b6106b, 93 },
    { 0x29a6106c, 99 },
    { 0x4d76106c, 46 },
    { 0, 0 },
    { 0x746106f, 23 },
    { 0, 0 },
    { 0x4c161071, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f61075, 95 },
    { 0x68561076, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96107c, 52 },
    { 0x3fb6107d, 95 },
    { 0xc206107e, 23 },
    { 0x7676107f, 90 },
    { 0x4b26107e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da61084, 90 },
    { 0xec861085, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c761088, 90 },
    { 0x21361088, 33 },
    { 0x71d6108a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd6108e, 55 },
    { 0, 0 },
    { 0x39761090, 34 },
    { 0x3ff61091, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4561095, 76 },
    { 0x4bb61096, 95 },
    { 0xe9861096, 43 },
    { 0, 0 },
    { 0xdd361099, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa756109c, 23 },
    { 0xb7e6109d, 55 },
    { 0xf756109e, 67 },
    { 0x6686109e, 90 },
    { 0, 0 },
    { 0xecd610a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f610a5, 84 },
    { 0x6b7610a5, 34 },
    { 0x917610a7, 92 },
    { 0, 0 },
    { 0x63610a9, 61 },
    { 0x56a610aa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb610b1, 100 },
    { 0, 0 },
    { 0xf6d610b3, 67 },
    { 0x9a0610b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76610b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d610ba, 93 },
    { 0xe2610bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75610be, 76 },
    { 0x49f610bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41610c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e610ca, 78 },
    { 0xc48610cb, 93 },
    { 0x7e7610cc, 33 },
    { 0x28e610cb, 33 },
    { 0, 0 },
    { 0xc0c610cf, 46 },
    { 0x804610d0, 95 },
    { 0x8e3610d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc610de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1610e1, 85 },
    { 0xa84610e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05610e7, 90 },
    { 0xf43610e8, 95 },
    { 0, 0 },
    { 0x3b6610ea, 55 },
    { 0, 0 },
    { 0x50d610ec, 52 },
    { 0xa8c610ed, 66 },
    { 0xa6b610ec, 98 },
    { 0, 0 },
    { 0xc83610f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff610f3, 90 },
    { 0xd4a610f3, 99 },
    { 0x514610f5, 90 },
    { 0, 0 },
    { 0xc610f7, 98 },
    { 0x1e610f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7610fe, 90 },
    { 0, 0 },
    { 0x56861100, 90 },
    { 0x55161101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8561106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de61109, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1756110c, 90 },
    { 0x51c6110d, 33 },
    { 0x7b76110d, 34 },
    { 0xac46110d, 55 },
    { 0x49b61110, 100 },
    { 0xda861110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f561117, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed56111b, 61 },
    { 0, 0 },
    { 0x7176111d, 23 },
    { 0x3b06111e, 95 },
    { 0x4876111f, 55 },
    { 0x4dd61120, 34 },
    { 0xec361121, 69 },
    { 0x9cc61122, 61 },
    { 0xb56111e, 8 },
    { 0x84861124, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e06112b, 46 },
    { 0, 0 },
    { 0x4056112d, 90 },
    { 0, 0 },
    { 0x5666112f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0861133, 93 },
    { 0, 0 },
    { 0xbe061135, 90 },
    { 0, 0 },
    { 0x47d61137, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e6113a, 61 },
    { 0x9a76113a, 33 },
    { 0xfe6113b, 98 },
    { 0x316113b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3661140, 33 },
    { 0x45761141, 55 },
    { 0xc5e61141, 33 },
    { 0, 0 },
    { 0x22061144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d461148, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec26114f, 61 },
    { 0x62d6114f, 77 },
    { 0x7d46114f, 90 },
    { 0xdb761152, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d61156, 40 },
    { 0x73161156, 23 },
    { 0xdf761158, 78 },
    { 0, 0 },
    { 0x5656115a, 98 },
    { 0, 0 },
    { 0x9716115c, 61 },
    { 0x9e16115d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e61164, 90 },
    { 0xfec61165, 23 },
    { 0x19661164, 99 },
    { 0xfdc61167, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf36116a, 90 },
    { 0, 0 },
    { 0x4f96116c, 61 },
    { 0x98d6116d, 23 },
    { 0xbdb6116d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a361172, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb61175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad261179, 23 },
    { 0, 0 },
    { 0x30b6117b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd61183, 85 },
    { 0xa5861184, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b61189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d6118e, 83 },
    { 0xca36118e, 95 },
    { 0, 0 },
    { 0x8e561191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20561194, 55 },
    { 0xa0261194, 33 },
    { 0x5ac61196, 98 },
    { 0x81861197, 61 },
    { 0xfdc61198, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f6119d, 43 },
    { 0, 0 },
    { 0xedc6119f, 99 },
    { 0xc8d611a0, 33 },
    { 0x142611a1, 85 },
    { 0, 0 },
    { 0xba2611a3, 76 },
    { 0xf6d611a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64611ac, 34 },
    { 0x837611ac, 76 },
    { 0x24d611ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x435611b1, 90 },
    { 0xfb3611b2, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c611b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a611bb, 90 },
    { 0x635611bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8611c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8611cb, 87 },
    { 0, 0 },
    { 0x8ce611cd, 33 },
    { 0xcab611ce, 95 },
    { 0, 0 },
    { 0x77e611d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b611d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6611dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09611ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1611f8, 23 },
    { 0x737611f8, 66 },
    { 0x833611fa, 90 },
    { 0x863611fb, 91 },
    { 0x3ba611fc, 90 },
    { 0x4b611fd, 55 },
    { 0, 0 },
    { 0xe7c611ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f61203, 40 },
    { 0, 0 },
    { 0xa2e61205, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab61208, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd36120d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x80061210, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61214, 23 },
    { 0, 0 },
    { 0x7e61216, 90 },
    { 0xe7c61217, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9176121a, 43 },
    { 0xe786121b, 78 },
    { 0x9fe6121b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91461222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6961225, 34 },
    { 0x71f61226, 90 },
    { 0x3f161225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b86122b, 88 },
    { 0xa196122b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a61230, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a561233, 95 },
    { 0x29c61233, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116123a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26123e, 98 },
    { 0xc146123e, 54 },
    { 0xb2e61240, 46 },
    { 0x8d261241, 38 },
    { 0x7796123e, 95 },
    { 0x1c461243, 90 },
    { 0, 0 },
    { 0xf0961245, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad86124a, 23 },
    { 0xf156124a, 98 },
    { 0x5456124c, 33 },
    { 0x6d36124d, 61 },
    { 0xb96124e, 23 },
    { 0xb576124c, 33 },
    { 0xa0061250, 90 },
    { 0x1bb61250, 76 },
    { 0x39461251, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b61255, 17 },
    { 0, 0 },
    { 0x2b761257, 33 },
    { 0xc3061258, 85 },
    { 0x4a61258, 33 },
    { 0x83161258, 46 },
    { 0x3c06125a, 46 },
    { 0x1ff6125b, 76 },
    { 0xdf66125a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29561260, 76 },
    { 0x52a61261, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e361266, 66 },
    { 0xf7461266, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb586126a, 85 },
    { 0x7916126a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a36126f, 61 },
    { 0x8fb6126f, 90 },
    { 0, 0 },
    { 0xcef61272, 84 },
    { 0xbd61272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e261277, 91 },
    { 0xffa61278, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9646127b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb261280, 23 },
    { 0xb0961281, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e61285, 88 },
    { 0x22961285, 87 },
    { 0x6161287, 90 },
    { 0x9dc61288, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4346128b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba161290, 34 },
    { 0xe4961290, 76 },
    { 0, 0 },
    { 0xe0f61293, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d61297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa526129c, 90 },
    { 0x4bf6129c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c612a4, 88 },
    { 0xd5b612a4, 67 },
    { 0xd16612a6, 91 },
    { 0, 0 },
    { 0xdb612a8, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf612ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc612b2, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb612b7, 34 },
    { 0x9ed612b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251612bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18612c2, 90 },
    { 0x1b7612c3, 90 },
    { 0xdfd612c4, 95 },
    { 0, 0 },
    { 0xa94612c6, 98 },
    { 0x7c612c7, 76 },
    { 0x98c612c7, 55 },
    { 0xca3612c9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d612cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee612d2, 55 },
    { 0x364612d3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db612d6, 46 },
    { 0xb74612d7, 99 },
    { 0x1b8612d8, 82 },
    { 0xefd612d6, 99 },
    { 0, 0 },
    { 0xe07612db, 95 },
    { 0x6f2612dc, 61 },
    { 0, 0 },
    { 0xaba612de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d612e8, 98 },
    { 0xddd612e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12612ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83612f4, 23 },
    { 0x4bf612f5, 98 },
    { 0xf2c612f6, 23 },
    { 0, 0 },
    { 0x88f612f8, 90 },
    { 0x9612f9, 94 },
    { 0xd80612f8, 48 },
    { 0, 0 },
    { 0x531612fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3612ff, 55 },
    { 0x82861300, 93 },
    { 0x5a4612ff, 95 },
    { 0, 0 },
    { 0xdbc61303, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e61307, 81 },
    { 0x44161308, 61 },
    { 0xffa61309, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276130c, 61 },
    { 0x73a6130d, 61 },
    { 0x36d6130c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x19561311, 33 },
    { 0x1dd61312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc61316, 67 },
    { 0, 0 },
    { 0xc2c61318, 100 },
    { 0xdfa61319, 76 },
    { 0, 0 },
    { 0x61a6131b, 76 },
    { 0xacb6131b, 61 },
    { 0, 0 },
    { 0x3ae6131e, 77 },
    { 0xf6f6131f, 17 },
    { 0x4f61320, 99 },
    { 0x9196131f, 95 },
    { 0x4456131f, 33 },
    { 0x3ed61321, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e61326, 61 },
    { 0x1f461327, 17 },
    { 0x9b861327, 43 },
    { 0x7e861328, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e06132f, 91 },
    { 0, 0 },
    { 0xd6761331, 98 },
    { 0xced61332, 77 },
    { 0x4b961333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d6133f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c961342, 75 },
    { 0x8f861342, 90 },
    { 0xe5761343, 60 },
    { 0xbeb61345, 33 },
    { 0, 0 },
    { 0xb9661347, 52 },
    { 0x1e361348, 90 },
    { 0xbd561349, 33 },
    { 0x9f66134a, 46 },
    { 0xf9c6134b, 36 },
    { 0x5e36134c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86134f, 23 },
    { 0xf6a61350, 90 },
    { 0, 0 },
    { 0x69c61352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd261357, 90 },
    { 0xb2661358, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466135d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b61361, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b261369, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb946136c, 23 },
    { 0x39d6136d, 90 },
    { 0, 0 },
    { 0xb9b6136f, 55 },
    { 0x53461370, 52 },
    { 0x2456136f, 33 },
    { 0xcbe6136f, 40 },
    { 0x2f761372, 33 },
    { 0x7e161372, 76 },
    { 0x25061374, 33 },
    { 0, 0 },
    { 0x9c761377, 40 },
    { 0x19a61378, 87 },
    { 0xc1b61378, 33 },
    { 0x13f61379, 46 },
    { 0x3d6137b, 95 },
    { 0x4ab6137b, 61 },
    { 0xd376137d, 97 },
    { 0, 0 },
    { 0x7cc6137f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c61383, 90 },
    { 0x41861384, 55 },
    { 0x2b661384, 95 },
    { 0xc7961386, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36138b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b76138e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d561391, 80 },
    { 0xb4561392, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa61395, 33 },
    { 0xeed61396, 55 },
    { 0x11961395, 88 },
    { 0x64b61398, 33 },
    { 0x8df61398, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6139e, 93 },
    { 0xeef6139f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83613a2, 61 },
    { 0x186613a2, 67 },
    { 0x14c613a4, 90 },
    { 0x458613a5, 98 },
    { 0x2e7613a5, 90 },
    { 0, 0 },
    { 0x6b1613a8, 76 },
    { 0x9f613a9, 90 },
    { 0x80e613aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e613ad, 23 },
    { 0xd4a613ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21613b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c613bb, 76 },
    { 0, 0 },
    { 0xeff613bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935613c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a613cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb613d0, 55 },
    { 0xaa0613d1, 40 },
    { 0, 0 },
    { 0xd70613d3, 81 },
    { 0x250613d4, 41 },
    { 0x27d613d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f613da, 93 },
    { 0x293613db, 23 },
    { 0x3ae613db, 98 },
    { 0x90b613db, 33 },
    { 0x174613de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87613e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2613e7, 90 },
    { 0x1f3613e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94613eb, 78 },
    { 0x4d3613ec, 66 },
    { 0x5e5613ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874613f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd613f7, 69 },
    { 0xad7613f8, 90 },
    { 0, 0 },
    { 0xc52613fa, 98 },
    { 0xabe613fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e613fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d61407, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd606140a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a61410, 61 },
    { 0xafb61411, 90 },
    { 0x2a261412, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0861419, 61 },
    { 0xcfb6141a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc6141e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19961421, 33 },
    { 0, 0 },
    { 0x9da61423, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58461429, 95 },
    { 0x7e66142a, 23 },
    { 0x5bd6142a, 34 },
    { 0x3496142b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a61435, 90 },
    { 0x6c261435, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc6143a, 55 },
    { 0, 0 },
    { 0x7636143c, 90 },
    { 0, 0 },
    { 0x3126143e, 85 },
    { 0xa6e6143f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18261448, 81 },
    { 0x5e061449, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8086144d, 61 },
    { 0x5066144e, 55 },
    { 0x60f6144e, 82 },
    { 0x15561450, 61 },
    { 0, 0 },
    { 0xba061452, 90 },
    { 0, 0 },
    { 0x5a861454, 78 },
    { 0x5ef61455, 61 },
    { 0, 0 },
    { 0x52f61457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d6145b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5661460, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d61467, 61 },
    { 0x5f561468, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb361472, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac61477, 85 },
    { 0xae961478, 61 },
    { 0x9c161479, 33 },
    { 0xe9d61477, 93 },
    { 0x6a6147b, 33 },
    { 0x70b61477, 23 },
    { 0, 0 },
    { 0x7b66147e, 90 },
    { 0x7b16147f, 95 },
    { 0x5126147f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65261484, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb761488, 91 },
    { 0xb4b61489, 75 },
    { 0x2a76148a, 95 },
    { 0xbd461489, 40 },
    { 0x9e161489, 23 },
    { 0xe096148d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb61490, 76 },
    { 0xa8e61491, 33 },
    { 0x85861492, 61 },
    { 0, 0 },
    { 0xfa61494, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b61498, 87 },
    { 0, 0 },
    { 0x8bb6149a, 98 },
    { 0, 0 },
    { 0x6bf6149c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c36149f, 17 },
    { 0x288614a0, 65 },
    { 0x59b6149f, 85 },
    { 0x6a1614a2, 23 },
    { 0x2eb614a2, 95 },
    { 0x4d0614a4, 90 },
    { 0x372614a4, 98 },
    { 0xe6b614a2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe614aa, 91 },
    { 0, 0 },
    { 0xafd614ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61614af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x271614b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5614b5, 85 },
    { 0, 0 },
    { 0x219614b7, 90 },
    { 0xef3614b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x349614bb, 95 },
    { 0x11e614bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f614bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6614c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b614cb, 90 },
    { 0, 0 },
    { 0x4c6614cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3614d3, 43 },
    { 0x8f3614d4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99614e0, 33 },
    { 0x41614e0, 61 },
    { 0x813614e0, 53 },
    { 0xf5d614e3, 87 },
    { 0xc20614e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb614e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad614ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb614ef, 23 },
    { 0xb06614ef, 98 },
    { 0x629614f1, 61 },
    { 0x2b7614f2, 84 },
    { 0x1b5614f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1614f9, 76 },
    { 0, 0 },
    { 0xc3a614fb, 98 },
    { 0xc0e614fc, 55 },
    { 0x7f4614fd, 98 },
    { 0, 0 },
    { 0x8cc614ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d6150b, 84 },
    { 0xeaa6150c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa461510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90661518, 64 },
    { 0x61961519, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5686151d, 42 },
    { 0xd456151e, 23 },
    { 0x70f6151f, 76 },
    { 0xfce61520, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b61527, 33 },
    { 0x50061528, 33 },
    { 0x51061529, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9336152c, 61 },
    { 0x31c6152c, 74 },
    { 0x5f96152c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0261531, 90 },
    { 0, 0 },
    { 0xa6561533, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a61536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc61539, 78 },
    { 0xadf6153a, 85 },
    { 0x44e6153b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11961540, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x24661543, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7856154f, 73 },
    { 0xc4b61550, 43 },
    { 0x78c61550, 43 },
    { 0x14461551, 84 },
    { 0xe7261551, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5461566, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc56156f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde61575, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a6157a, 23 },
    { 0, 0 },
    { 0xff66157c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a61585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c61588, 56 },
    { 0x4ea61589, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46158c, 12 },
    { 0x1fb6158d, 23 },
    { 0x2d06158e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae61593, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af61596, 76 },
    { 0xb0f61597, 95 },
    { 0x87b61597, 55 },
    { 0x5e961597, 90 },
    { 0x86c6159a, 61 },
    { 0x2bc6159b, 64 },
    { 0x70161599, 61 },
    { 0xd1f6159d, 87 },
    { 0xa9b6159e, 55 },
    { 0xabc6159f, 98 },
    { 0x135615a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba615a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881615ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411615b2, 33 },
    { 0, 0 },
    { 0xf7d615b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9615b7, 77 },
    { 0x36d615b8, 33 },
    { 0x8de615b9, 61 },
    { 0x825615ba, 95 },
    { 0x648615bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1615be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833615c6, 23 },
    { 0xa30615c6, 24 },
    { 0x827615c8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267615cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0615d0, 23 },
    { 0, 0 },
    { 0x718615d2, 33 },
    { 0xfcf615d2, 33 },
    { 0xf3e615d3, 85 },
    { 0x806615d3, 70 },
    { 0x2c9615d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c615d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54615e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5615eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13615ef, 33 },
    { 0xabc615ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x555615f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93615f6, 90 },
    { 0, 0 },
    { 0xae4615f8, 95 },
    { 0x51615f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3615fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1461600, 95 },
    { 0x4be61600, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a61605, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86161608, 95 },
    { 0, 0 },
    { 0xf576160a, 92 },
    { 0xd9b6160b, 81 },
    { 0, 0 },
    { 0x6386160d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c61610, 93 },
    { 0xeb561610, 43 },
    { 0xe2861612, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed861618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe346161c, 90 },
    { 0x1b6161d, 23 },
    { 0x4c26161d, 90 },
    { 0xaba6161c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58461624, 61 },
    { 0x85161624, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba6162a, 85 },
    { 0, 0 },
    { 0xacb6162c, 33 },
    { 0, 0 },
    { 0x5fd6162e, 76 },
    { 0x2fb6162e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9061634, 36 },
    { 0xe3861635, 99 },
    { 0, 0 },
    { 0x79b61637, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c6163e, 95 },
    { 0x1326163f, 23 },
    { 0x92c6163f, 98 },
    { 0x6661641, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f661645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f061648, 84 },
    { 0, 0 },
    { 0xa996164a, 53 },
    { 0, 0 },
    { 0xf846164c, 17 },
    { 0x3f66164d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba61650, 92 },
    { 0, 0 },
    { 0x1bc61652, 95 },
    { 0x2f461652, 33 },
    { 0x90a61654, 55 },
    { 0x66061655, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba261659, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4846165f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac61666, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca6166a, 23 },
    { 0x5a6166b, 61 },
    { 0, 0 },
    { 0x6aa6166d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f61673, 95 },
    { 0x39c61674, 90 },
    { 0xe8c61673, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d61678, 90 },
    { 0xc0a61678, 87 },
    { 0x44261679, 55 },
    { 0xd366167a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd061681, 98 },
    { 0x99461682, 61 },
    { 0x6bf61683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90861689, 100 },
    { 0xf0a61689, 90 },
    { 0, 0 },
    { 0x38c6168c, 33 },
    { 0x9076168c, 62 },
    { 0xa226168e, 46 },
    { 0xc866168f, 99 },
    { 0x6bd61690, 33 },
    { 0xc1461690, 40 },
    { 0x2f661691, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d61695, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e61699, 34 },
    { 0xe8b61699, 98 },
    { 0x8ed6169b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab616a2, 61 },
    { 0x3aa616a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x143616a6, 67 },
    { 0x9ce616a7, 33 },
    { 0xfd9616a6, 43 },
    { 0, 0 },
    { 0xb34616aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7616af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33616b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be616bb, 81 },
    { 0x623616bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x828616bf, 46 },
    { 0xa0c616c0, 93 },
    { 0x6a5616c1, 95 },
    { 0x298616c2, 61 },
    { 0x3ec616bf, 76 },
    { 0, 0 },
    { 0xc0c616c5, 61 },
    { 0, 0 },
    { 0xe01616c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x776616ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6616ce, 98 },
    { 0xbaf616ce, 95 },
    { 0x63d616ce, 61 },
    { 0, 0 },
    { 0xcf616d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e616d6, 93 },
    { 0x906616d7, 23 },
    { 0xba2616d8, 98 },
    { 0x82c616d9, 87 },
    { 0xd8c616d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25616de, 76 },
    { 0x564616de, 61 },
    { 0xcc6616de, 87 },
    { 0x121616e1, 81 },
    { 0, 0 },
    { 0xbbc616e3, 61 },
    { 0x87616e4, 23 },
    { 0x1c3616e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661616e9, 23 },
    { 0x38b616ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07616f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7616f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b616f8, 33 },
    { 0x97b616f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354616fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb661706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53361709, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d6170f, 84 },
    { 0xe8c6170f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b61713, 61 },
    { 0xcac61713, 98 },
    { 0, 0 },
    { 0x9d761716, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06171d, 90 },
    { 0, 0 },
    { 0xaa76171f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24461722, 95 },
    { 0, 0 },
    { 0x8b761724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c36172a, 98 },
    { 0, 0 },
    { 0xea56172c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5036172f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61733, 24 },
    { 0, 0 },
    { 0x8361735, 90 },
    { 0, 0 },
    { 0x27e61737, 75 },
    { 0, 0 },
    { 0xc6b61739, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a61740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366174a, 46 },
    { 0, 0 },
    { 0x90e6174c, 33 },
    { 0, 0 },
    { 0xe606174e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f361751, 47 },
    { 0x661752, 95 },
    { 0x32961752, 23 },
    { 0x7b161754, 78 },
    { 0, 0 },
    { 0x8a861756, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2636175f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e761766, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb516176b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6176176f, 95 },
    { 0, 0 },
    { 0xabc61771, 23 },
    { 0xfb161772, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xee961775, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa146177d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa61781, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9161786, 76 },
    { 0xd7261786, 61 },
    { 0x7ef61788, 81 },
    { 0x76161786, 61 },
    { 0xfd261786, 93 },
    { 0x1b26178b, 33 },
    { 0xd816178c, 91 },
    { 0xdbe6178c, 55 },
    { 0x6e66178c, 34 },
    { 0xe286178d, 61 },
    { 0x8d261790, 56 },
    { 0xae66178e, 89 },
    { 0, 0 },
    { 0x84c61793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49561797, 33 },
    { 0, 0 },
    { 0xb5061799, 98 },
    { 0xca61799, 100 },
    { 0x3b36179b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107617a3, 69 },
    { 0, 0 },
    { 0x337617a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88617aa, 98 },
    { 0x47f617ab, 90 },
    { 0x5ad617ac, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a617bc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d617bf, 61 },
    { 0, 0 },
    { 0xab7617c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884617c5, 46 },
    { 0x7fb617c5, 95 },
    { 0xc65617c6, 33 },
    { 0, 0 },
    { 0x7c617c9, 34 },
    { 0, 0 },
    { 0x826617cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120617d1, 91 },
    { 0xf97617d2, 43 },
    { 0, 0 },
    { 0x321617d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x904617d7, 61 },
    { 0x96b617d8, 76 },
    { 0xe43617d9, 93 },
    { 0xbc9617d9, 61 },
    { 0, 0 },
    { 0xca8617dc, 99 },
    { 0, 0 },
    { 0xfb2617de, 78 },
    { 0x66a617df, 81 },
    { 0, 0 },
    { 0xd67617e1, 95 },
    { 0, 0 },
    { 0xbc0617e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293617e7, 95 },
    { 0xa9617e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad617ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69617f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869617f4, 33 },
    { 0x629617f5, 52 },
    { 0xa24617f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9617f9, 95 },
    { 0x14617fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a617fd, 61 },
    { 0, 0 },
    { 0xdc8617ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed61804, 33 },
    { 0, 0 },
    { 0xed361806, 95 },
    { 0, 0 },
    { 0x83661808, 23 },
    { 0xf8461809, 88 },
    { 0xfbb61809, 33 },
    { 0xfc361808, 84 },
    { 0xa266180a, 55 },
    { 0, 0 },
    { 0xa556180e, 65 },
    { 0xda56180f, 98 },
    { 0, 0 },
    { 0x3bd61811, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b6181d, 95 },
    { 0xe236181e, 98 },
    { 0x39b6181f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd61822, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4161829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb6182c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e961834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586183a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad16183e, 76 },
    { 0x4dc6183f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d61842, 65 },
    { 0, 0 },
    { 0x19161844, 78 },
    { 0xbc561845, 33 },
    { 0x97061846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2606184a, 99 },
    { 0, 0 },
    { 0x8e96184c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f6184f, 90 },
    { 0, 0 },
    { 0x52761851, 98 },
    { 0xb0e61852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3761856, 100 },
    { 0x36361856, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6185d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d661863, 61 },
    { 0, 0 },
    { 0x84c61865, 23 },
    { 0x4b361866, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b361869, 33 },
    { 0xed861869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d61870, 100 },
    { 0xd1861871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d61875, 23 },
    { 0x6ce61876, 33 },
    { 0x8ff61877, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6916187c, 23 },
    { 0xa6f6187d, 33 },
    { 0xde76187d, 87 },
    { 0x576187d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b761884, 95 },
    { 0, 0 },
    { 0xdd161886, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb6188a, 90 },
    { 0x2a66188a, 95 },
    { 0xccf6188c, 46 },
    { 0xe2c6188c, 98 },
    { 0x7136188b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d861891, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf61894, 78 },
    { 0xaa261895, 81 },
    { 0xd7d61895, 84 },
    { 0xe2861896, 87 },
    { 0xb9761898, 95 },
    { 0x70b61899, 93 },
    { 0xa96189a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c66189d, 76 },
    { 0x1796189d, 67 },
    { 0x6236189f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe618a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2618a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946618ab, 78 },
    { 0x2dd618ac, 90 },
    { 0xd53618ac, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a618b1, 61 },
    { 0x98c618b2, 86 },
    { 0x207618b3, 31 },
    { 0xf63618b4, 33 },
    { 0x499618b1, 33 },
    { 0x465618b6, 33 },
    { 0x41e618b4, 90 },
    { 0x39d618b4, 43 },
    { 0xb67618b9, 23 },
    { 0x8b8618ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e618be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5618cb, 23 },
    { 0x832618cc, 34 },
    { 0x131618cd, 88 },
    { 0x472618ce, 98 },
    { 0x29f618cd, 61 },
    { 0, 0 },
    { 0x5b0618d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b618d4, 95 },
    { 0x3b2618d4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7618da, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa618e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x427618e4, 66 },
    { 0x36c618e4, 46 },
    { 0x577618e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6618e9, 98 },
    { 0, 0 },
    { 0x883618eb, 55 },
    { 0x8f5618ec, 57 },
    { 0xbe0618ed, 46 },
    { 0x971618ed, 66 },
    { 0x2ee618ef, 52 },
    { 0x366618ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1618fa, 98 },
    { 0, 0 },
    { 0x6f6618fc, 76 },
    { 0x3c9618fd, 90 },
    { 0xeb6618fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd61902, 76 },
    { 0x50061903, 24 },
    { 0xd2261904, 34 },
    { 0x3d261904, 99 },
    { 0x11e61906, 90 },
    { 0x17c61907, 76 },
    { 0x52461908, 90 },
    { 0xcf061907, 76 },
    { 0x6d16190a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b61911, 23 },
    { 0, 0 },
    { 0x4e961913, 43 },
    { 0, 0 },
    { 0xeb061915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c61919, 52 },
    { 0, 0 },
    { 0x90c6191b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d561921, 55 },
    { 0x69761921, 87 },
    { 0x4ad61922, 33 },
    { 0xefc61924, 98 },
    { 0xa2261925, 61 },
    { 0x4bf61926, 76 },
    { 0xf6661926, 23 },
    { 0, 0 },
    { 0x2ba61929, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f6192d, 46 },
    { 0x3946192e, 95 },
    { 0x23c6192d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e461932, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57261936, 65 },
    { 0xdd861937, 55 },
    { 0xd7161938, 90 },
    { 0xa5861938, 61 },
    { 0, 0 },
    { 0xcc66193b, 91 },
    { 0, 0 },
    { 0xa56193d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc661940, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e61945, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff6194d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e61956, 61 },
    { 0x35f61957, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016195b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda46195e, 43 },
    { 0x2466195f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561964, 43 },
    { 0x93b61965, 76 },
    { 0x4b161965, 87 },
    { 0, 0 },
    { 0xbfc61968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2336196e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6261971, 66 },
    { 0xc4761972, 40 },
    { 0x3dd61973, 66 },
    { 0x4ea61973, 33 },
    { 0xf7261973, 98 },
    { 0x65461976, 68 },
    { 0xd661977, 66 },
    { 0xab161973, 98 },
    { 0x1a861976, 23 },
    { 0, 0 },
    { 0x6076197b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c461981, 23 },
    { 0x29a61982, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f66198a, 84 },
    { 0xd8b6198b, 66 },
    { 0x3fb6198a, 90 },
    { 0, 0 },
    { 0x6db6198e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d61994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78461997, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb66199e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a619a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e619aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93619ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d619b1, 55 },
    { 0xeeb619b2, 90 },
    { 0x62619b3, 66 },
    { 0xae4619b4, 61 },
    { 0x524619b3, 23 },
    { 0, 0 },
    { 0xb3c619b7, 87 },
    { 0, 0 },
    { 0x6af619b9, 76 },
    { 0xa69619ba, 38 },
    { 0x1b1619ba, 52 },
    { 0, 0 },
    { 0x753619bd, 76 },
    { 0x3a5619be, 61 },
    { 0xf4c619bf, 95 },
    { 0xa3b619bd, 100 },
    { 0, 0 },
    { 0x7bd619c2, 95 },
    { 0, 0 },
    { 0xcc2619c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602619c8, 61 },
    { 0, 0 },
    { 0x8cc619ca, 87 },
    { 0x6a9619ca, 9 },
    { 0xceb619cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f619d0, 65 },
    { 0xef8619d1, 43 },
    { 0x8d3619d0, 61 },
    { 0x930619d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e619d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348619db, 100 },
    { 0, 0 },
    { 0xc88619dd, 90 },
    { 0x9f7619de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x730619e1, 92 },
    { 0, 0 },
    { 0x9bc619e3, 33 },
    { 0, 0 },
    { 0xd97619e5, 90 },
    { 0, 0 },
    { 0xe34619e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5619ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15619ed, 98 },
    { 0x1d3619ed, 90 },
    { 0, 0 },
    { 0x376619f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35619f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e619fa, 91 },
    { 0, 0 },
    { 0xd8e619fc, 52 },
    { 0x114619fc, 98 },
    { 0x366619fc, 43 },
    { 0x2b619fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa61a03, 23 },
    { 0x33561a03, 91 },
    { 0xbd261a05, 23 },
    { 0xd7461a05, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a261a0a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd561a12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9061a1d, 61 },
    { 0xbf061a1d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e61a25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0861a2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe61a30, 90 },
    { 0xa9961a30, 61 },
    { 0xf0061a30, 95 },
    { 0, 0 },
    { 0xdbe61a34, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb761a38, 61 },
    { 0xc061a39, 34 },
    { 0x86e61a3a, 33 },
    { 0x6f561a3b, 33 },
    { 0x3c361a39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd261a3f, 47 },
    { 0, 0 },
    { 0x14c61a41, 33 },
    { 0xe7261a42, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x99961a45, 101 },
    { 0, 0 },
    { 0xa9f61a47, 90 },
    { 0, 0 },
    { 0x24461a49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90761a53, 98 },
    { 0x22661a53, 85 },
    { 0x5c61a54, 98 },
    { 0x39261a56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22161a59, 43 },
    { 0, 0 },
    { 0x41a61a5b, 98 },
    { 0, 0 },
    { 0x3b161a5d, 46 },
    { 0, 0 },
    { 0xa7461a5f, 43 },
    { 0x75161a60, 23 },
    { 0xb3261a61, 55 },
    { 0, 0 },
    { 0xe2f61a63, 52 },
    { 0x6d661a63, 93 },
    { 0x3d161a65, 98 },
    { 0x34c61a63, 55 },
    { 0xa561a67, 23 },
    { 0, 0 },
    { 0xce61a69, 61 },
    { 0xe1f61a6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6961a71, 81 },
    { 0, 0 },
    { 0xaa561a73, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb61a77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d61a7d, 90 },
    { 0x74e61a7e, 76 },
    { 0x80c61a7f, 98 },
    { 0x47c61a7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f61a83, 43 },
    { 0xc4361a84, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0061a8c, 90 },
    { 0xe2961a8d, 55 },
    { 0xee261a8e, 43 },
    { 0xe961a8f, 98 },
    { 0x32061a8c, 84 },
    { 0xdd261a8e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d61a97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9761a9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4061aa4, 76 },
    { 0xa0461aa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b61aaa, 90 },
    { 0, 0 },
    { 0xeca61aac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5161ab2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e61ab9, 85 },
    { 0x4561ab9, 34 },
    { 0, 0 },
    { 0xb8d61abc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb61abf, 33 },
    { 0, 0 },
    { 0xcd361ac1, 23 },
    { 0, 0 },
    { 0x4861ac3, 23 },
    { 0x2e861ac4, 93 },
    { 0x89061ac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1561ace, 33 },
    { 0x62e61ace, 99 },
    { 0x6cc61ace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5561ad3, 61 },
    { 0, 0 },
    { 0x10f61ad5, 23 },
    { 0xfd61ad6, 90 },
    { 0x28b61ad6, 98 },
    { 0xa6a61ad6, 95 },
    { 0x8b61ad7, 98 },
    { 0, 0 },
    { 0x8f561adb, 90 },
    { 0x50161adb, 52 },
    { 0xcee61adb, 53 },
    { 0x7f261ade, 99 },
    { 0x86e61adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c561ae4, 23 },
    { 0, 0 },
    { 0x38661ae6, 99 },
    { 0x24e61ae6, 66 },
    { 0x39461ae6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a61aef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63761af5, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98861af9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d61afd, 65 },
    { 0, 0 },
    { 0x66a61aff, 46 },
    { 0x8f561aff, 55 },
    { 0x73661b01, 61 },
    { 0x58361b01, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d661b06, 33 },
    { 0xf5c61b07, 23 },
    { 0, 0 },
    { 0x99561b09, 23 },
    { 0xdae61b0a, 76 },
    { 0, 0 },
    { 0xec461b0c, 90 },
    { 0x1161b0d, 33 },
    { 0, 0 },
    { 0x4cb61b0f, 91 },
    { 0x5a61b10, 95 },
    { 0, 0 },
    { 0x5de61b12, 34 },
    { 0x79761b13, 34 },
    { 0, 0 },
    { 0xf0e61b15, 91 },
    { 0x4eb61b16, 98 },
    { 0xa3161b15, 33 },
    { 0x8f061b18, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab61b1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab061b23, 95 },
    { 0xe9261b24, 34 },
    { 0x5bd61b23, 90 },
    { 0, 0 },
    { 0xca961b27, 90 },
    { 0x37c61b28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d61b2c, 55 },
    { 0x73661b2d, 95 },
    { 0xd4a61b2e, 99 },
    { 0xf0e61b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e61b32, 87 },
    { 0x73561b33, 98 },
    { 0, 0 },
    { 0xc7961b35, 98 },
    { 0x8d861b36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10961b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6261b3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b61b42, 91 },
    { 0, 0 },
    { 0x80761b44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83161b49, 33 },
    { 0x21d61b4a, 33 },
    { 0xac61b4b, 55 },
    { 0, 0 },
    { 0xa3e61b4d, 23 },
    { 0x59161b4e, 91 },
    { 0, 0 },
    { 0xd4361b50, 78 },
    { 0xdea61b51, 33 },
    { 0xc1261b52, 87 },
    { 0x5b861b52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf61b57, 98 },
    { 0x34b61b57, 61 },
    { 0, 0 },
    { 0x66961b5a, 76 },
    { 0, 0 },
    { 0x77861b5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd61b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee61b63, 61 },
    { 0, 0 },
    { 0xbeb61b65, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e61b6b, 17 },
    { 0xc4d61b6c, 98 },
    { 0, 0 },
    { 0xea161b6e, 98 },
    { 0, 0 },
    { 0x13b61b70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a261b75, 90 },
    { 0x57f61b75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd61b7c, 100 },
    { 0xd9261b7c, 55 },
    { 0xb1561b7e, 33 },
    { 0x54761b7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9461b83, 55 },
    { 0xc4a61b83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e61b89, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9461b90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f61b97, 95 },
    { 0xaa061b97, 98 },
    { 0x5b961b97, 66 },
    { 0, 0 },
    { 0xdda61b9b, 99 },
    { 0xc6961b9c, 55 },
    { 0x30a61b9d, 98 },
    { 0xd4761b9e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e61ba5, 76 },
    { 0x7b461ba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f61ba9, 95 },
    { 0xd0461baa, 90 },
    { 0xa6661bab, 34 },
    { 0, 0 },
    { 0xf3861bad, 34 },
    { 0x5c361bae, 61 },
    { 0x62661baf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e561bb4, 75 },
    { 0, 0 },
    { 0x42d61bb6, 34 },
    { 0x8fa61bb7, 33 },
    { 0, 0 },
    { 0xbdb61bb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2761bc0, 33 },
    { 0, 0 },
    { 0x28f61bc2, 71 },
    { 0xfdc61bc3, 99 },
    { 0xf5861bc4, 98 },
    { 0xa2761bc3, 98 },
    { 0, 0 },
    { 0x59b61bc7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6061bca, 78 },
    { 0x2b561bcb, 68 },
    { 0xe8661bcc, 80 },
    { 0xe8b61bcd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf161bd4, 52 },
    { 0xa261bd5, 76 },
    { 0x8cf61bd5, 23 },
    { 0x4e561bd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82261bda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc61bde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c61be1, 90 },
    { 0x10261be2, 69 },
    { 0xbf461be3, 33 },
    { 0x7261be4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df61bee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c61bf1, 91 },
    { 0x7bb61bf2, 76 },
    { 0x83261bf3, 98 },
    { 0, 0 },
    { 0x17161bf5, 55 },
    { 0x5eb61bf5, 87 },
    { 0xb6861bf7, 36 },
    { 0xbd261bf5, 34 },
    { 0xd8261bf9, 98 },
    { 0x87461bf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec61bfd, 90 },
    { 0xf8561bfe, 90 },
    { 0x82061bfd, 99 },
    { 0xfaf61c00, 17 },
    { 0xc4e61c01, 46 },
    { 0x8b661c01, 23 },
    { 0x21161c03, 33 },
    { 0xc2161c03, 90 },
    { 0, 0 },
    { 0xc9f61c06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xab761c09, 90 },
    { 0, 0 },
    { 0x12361c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81761c12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc561c15, 55 },
    { 0x89d61c16, 90 },
    { 0xe2c61c16, 90 },
    { 0xe0661c18, 43 },
    { 0x49b61c18, 61 },
    { 0x3061c1a, 49 },
    { 0x36b61c1b, 43 },
    { 0x8e461c1b, 23 },
    { 0xa3e61c1b, 34 },
    { 0xbc661c1e, 77 },
    { 0x55d61c1a, 99 },
    { 0x8cf61c20, 95 },
    { 0x86861c19, 90 },
    { 0x69061c22, 82 },
    { 0x21b61c22, 84 },
    { 0xe8761c1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c61c2c, 33 },
    { 0xbee61c2d, 55 },
    { 0xe561c2d, 76 },
    { 0, 0 },
    { 0xb61c30, 55 },
    { 0xd2761c31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32861c35, 43 },
    { 0x6c761c36, 91 },
    { 0xd6f61c37, 43 },
    { 0xb3761c38, 98 },
    { 0xd4161c37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50261c45, 98 },
    { 0x5bf61c46, 33 },
    { 0x8c061c47, 43 },
    { 0, 0 },
    { 0x20161c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3961c4e, 43 },
    { 0xdd861c4f, 61 },
    { 0, 0 },
    { 0xf9361c51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67661c54, 90 },
    { 0x2c261c55, 76 },
    { 0x4d061c55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40061c5a, 90 },
    { 0xcbd61c5b, 93 },
    { 0xd5d61c5c, 46 },
    { 0x3c361c5c, 75 },
    { 0, 0 },
    { 0x5b61c5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63561c64, 33 },
    { 0xf1161c65, 61 },
    { 0x9361c65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea361c6d, 66 },
    { 0x33961c6d, 33 },
    { 0x84f61c6f, 76 },
    { 0x35561c70, 99 },
    { 0xa3e61c71, 55 },
    { 0xc3561c72, 33 },
    { 0x9d061c6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d161c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21261c7a, 61 },
    { 0x6961c7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6861c7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34061c82, 65 },
    { 0x18861c83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe61c86, 61 },
    { 0x61b61c87, 95 },
    { 0, 0 },
    { 0xc3561c89, 90 },
    { 0x48961c89, 90 },
    { 0, 0 },
    { 0xc5c61c8c, 91 },
    { 0x8561c8d, 33 },
    { 0, 0 },
    { 0x6a161c8f, 33 },
    { 0xf0861c90, 90 },
    { 0xa2a61c90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de61c99, 90 },
    { 0, 0 },
    { 0x7a661c9b, 61 },
    { 0xea161c9b, 43 },
    { 0xf7461c9d, 76 },
    { 0x4ca61c9e, 90 },
    { 0xd261c9d, 95 },
    { 0xa8161c9d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d161ca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d61ca8, 34 },
    { 0xd4e61ca9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81461cbb, 91 },
    { 0x1f161cbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36861cc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd61cc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b61ccb, 55 },
    { 0x83e61ccb, 98 },
    { 0, 0 },
    { 0x9b861cce, 78 },
    { 0, 0 },
    { 0x89661cd0, 61 },
    { 0x12761cd0, 95 },
    { 0xa5d61cd0, 52 },
    { 0x99061cd1, 95 },
    { 0x14061cd4, 76 },
    { 0x8a61cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e161cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b061ce1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e61ceb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9461cf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a61cf4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed161cfc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee61d00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b61d05, 98 },
    { 0xda061d06, 55 },
    { 0x58661d07, 61 },
    { 0x67161d08, 66 },
    { 0x91161d08, 98 },
    { 0x53d61d0a, 77 },
    { 0xdb861d0b, 87 },
    { 0x6f661d0c, 33 },
    { 0x70e61d08, 99 },
    { 0xf2061d0e, 33 },
    { 0xd4b61d0f, 65 },
    { 0x21161d0e, 23 },
    { 0x66061d10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e61d1c, 90 },
    { 0, 0 },
    { 0x7f561d1e, 90 },
    { 0xae061d1f, 23 },
    { 0, 0 },
    { 0xcb861d21, 69 },
    { 0x3961d21, 65 },
    { 0xac161d23, 43 },
    { 0, 0 },
    { 0xe4d61d25, 91 },
    { 0x48861d26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb61d29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60261d32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf61d35, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a61d3b, 75 },
    { 0xd3d61d3c, 61 },
    { 0x9a661d3c, 93 },
    { 0xcb861d3d, 90 },
    { 0x1a861d3e, 65 },
    { 0, 0 },
    { 0x3f161d41, 90 },
    { 0xc7661d41, 61 },
    { 0xfba61d43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee761d46, 52 },
    { 0x35661d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c61d4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b61d4e, 51 },
    { 0, 0 },
    { 0x57661d50, 98 },
    { 0xb9e61d50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe061d58, 61 },
    { 0, 0 },
    { 0xe6a61d5a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b61d5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec661d61, 52 },
    { 0, 0 },
    { 0x1561d63, 87 },
    { 0xe6e61d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12961d67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93761d6c, 33 },
    { 0x33161d6d, 76 },
    { 0, 0 },
    { 0x78f61d6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba61d74, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8761d77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70261d7e, 90 },
    { 0, 0 },
    { 0xbe761d80, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc61d84, 67 },
    { 0, 0 },
    { 0xd8961d86, 95 },
    { 0x73161d87, 33 },
    { 0xcab61d88, 95 },
    { 0xd4661d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f61d8e, 34 },
    { 0xa9761d8f, 78 },
    { 0x82d61d90, 34 },
    { 0x61f61d91, 52 },
    { 0x3a961d92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d61d95, 90 },
    { 0xcbd61d95, 87 },
    { 0x76e61d97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92861d9b, 61 },
    { 0, 0 },
    { 0x64c61d9d, 98 },
    { 0xbe461d9e, 76 },
    { 0x2bc61d9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef361da3, 33 },
    { 0xae861da4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4861dab, 52 },
    { 0, 0 },
    { 0x52d61dad, 87 },
    { 0, 0 },
    { 0x11a61daf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b61db2, 91 },
    { 0, 0 },
    { 0x8d261db4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x90761db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9361dbb, 87 },
    { 0x2361dbc, 33 },
    { 0xc6561dbd, 99 },
    { 0xc8061dbe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd61dc1, 88 },
    { 0x9a261dc2, 48 },
    { 0, 0 },
    { 0xc7e61dc4, 99 },
    { 0x31561dc5, 90 },
    { 0x4f561dc6, 46 },
    { 0xefe61dc7, 85 },
    { 0x5bc61dc6, 34 },
    { 0xee161dc8, 99 },
    { 0x60561dca, 33 },
    { 0xbf661dca, 90 },
    { 0, 0 },
    { 0xaa361dcd, 46 },
    { 0x3c561dcd, 65 },
    { 0xbb861dce, 61 },
    { 0x74961dd0, 33 },
    { 0x92f61dd0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f61dd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32861dd7, 33 },
    { 0xf0161dd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89161ddc, 99 },
    { 0, 0 },
    { 0x5fb61dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc61de1, 100 },
    { 0xb7261de1, 33 },
    { 0xa4a61de3, 33 },
    { 0x28561de4, 90 },
    { 0xabc61de5, 33 },
    { 0x76c61de6, 67 },
    { 0x70d61de6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca061dea, 75 },
    { 0x4e761deb, 33 },
    { 0x7ab61dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a761def, 34 },
    { 0xbf361df0, 52 },
    { 0, 0 },
    { 0xb9a61df2, 100 },
    { 0x21761df3, 99 },
    { 0, 0 },
    { 0xa6d61df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c61dfb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d661e00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24661e03, 93 },
    { 0, 0 },
    { 0xe0a61e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb561e0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c61e11, 23 },
    { 0, 0 },
    { 0x4a361e13, 90 },
    { 0xfe61e14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60661e17, 61 },
    { 0, 0 },
    { 0xb4a61e19, 23 },
    { 0x62f61e1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc61e1d, 76 },
    { 0xf0861e1e, 95 },
    { 0xe6b61e1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2461e25, 23 },
    { 0xcfa61e25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff661e2a, 75 },
    { 0, 0 },
    { 0xb9761e2c, 34 },
    { 0xeae61e2d, 33 },
    { 0xa3361e2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79161e37, 52 },
    { 0x6bb61e37, 33 },
    { 0x15b61e39, 65 },
    { 0, 0 },
    { 0x5ce61e3b, 61 },
    { 0x48161e3c, 33 },
    { 0x82561e3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f261e43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb261e47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd61e4a, 49 },
    { 0x88a61e4b, 43 },
    { 0x12261e4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4961e50, 90 },
    { 0xcd861e51, 100 },
    { 0xb7e61e52, 95 },
    { 0xb7361e53, 33 },
    { 0xab761e54, 93 },
    { 0x38d61e55, 46 },
    { 0x87261e56, 95 },
    { 0xb8e61e57, 98 },
    { 0xc2561e57, 66 },
    { 0, 0 },
    { 0x5a61e5a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x25061e5d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e561e64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d61e67, 76 },
    { 0x3a961e68, 33 },
    { 0xddb61e69, 33 },
    { 0, 0 },
    { 0x65661e6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d61e6e, 93 },
    { 0x2be61e6e, 56 },
    { 0x14861e6e, 90 },
    { 0x58361e71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b61e7a, 76 },
    { 0x32661e7b, 99 },
    { 0xaa761e7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc361e81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d161e86, 33 },
    { 0x1bd61e87, 100 },
    { 0x77a61e87, 93 },
    { 0xa4361e89, 43 },
    { 0x19d61e8a, 33 },
    { 0x21d61e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b61e8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d61e92, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce961e9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a861e9f, 33 },
    { 0x46561ea0, 46 },
    { 0, 0 },
    { 0x30d61ea2, 95 },
    { 0xed961ea3, 78 },
    { 0xa4261ea3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d261eaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b161eae, 34 },
    { 0, 0 },
    { 0x5d861eb0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76861eb3, 68 },
    { 0, 0 },
    { 0xde661eb5, 90 },
    { 0x2db61eb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f61ebd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1161ec0, 65 },
    { 0, 0 },
    { 0x7c961ec2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51661ec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b61ece, 95 },
    { 0xc2661ecf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe61ed2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x44061ed5, 66 },
    { 0, 0 },
    { 0x79961ed7, 23 },
    { 0, 0 },
    { 0xb7561ed9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe61ee1, 67 },
    { 0x35361ee1, 87 },
    { 0, 0 },
    { 0x78f61ee4, 98 },
    { 0x1ff61ee4, 84 },
    { 0x61661ee6, 61 },
    { 0, 0 },
    { 0x4de61ee8, 33 },
    { 0x63e61ee8, 68 },
    { 0, 0 },
    { 0xd7861eeb, 95 },
    { 0x7b761eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c561ef0, 87 },
    { 0x51361ef1, 91 },
    { 0, 0 },
    { 0xbf461ef3, 77 },
    { 0, 0 },
    { 0xbc961ef5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a61efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f461efd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8261f02, 85 },
    { 0x28461f03, 98 },
    { 0x3e661f04, 90 },
    { 0x6d261f04, 66 },
    { 0xc5861f06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b61f09, 98 },
    { 0xbfb61f0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f461f0e, 33 },
    { 0x6e161f0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d61f17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2661f1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c61f1e, 46 },
    { 0xfab61f1f, 95 },
    { 0x89f61f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9461f24, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6061f27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82361f2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf861f31, 90 },
    { 0x9de61f31, 33 },
    { 0xfce61f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a261f38, 33 },
    { 0x47e61f39, 96 },
    { 0x57f61f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e61f3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a61f40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0661f47, 55 },
    { 0, 0 },
    { 0x71d61f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e61f4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33061f52, 87 },
    { 0xedf61f53, 84 },
    { 0, 0 },
    { 0x8e461f55, 90 },
    { 0x2bd61f56, 95 },
    { 0, 0 },
    { 0x59c61f58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1861f5d, 44 },
    { 0x45761f5e, 77 },
    { 0xb2161f5f, 100 },
    { 0x961f60, 23 },
    { 0x81561f61, 95 },
    { 0, 0 },
    { 0x71461f63, 88 },
    { 0xaa661f64, 91 },
    { 0xb1261f65, 55 },
    { 0x53261f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a61f6a, 95 },
    { 0x2e061f6b, 95 },
    { 0x37861f6a, 90 },
    { 0x9fc61f6a, 33 },
    { 0x26d61f6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6561f73, 88 },
    { 0x39b61f74, 100 },
    { 0x8be61f74, 69 },
    { 0x6fa61f73, 33 },
    { 0, 0 },
    { 0x4fb61f78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83361f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50261f84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f261f89, 90 },
    { 0xdf661f89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf061f90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56461f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9261f97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9961f9b, 78 },
    { 0x7f361f9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad061fa0, 98 },
    { 0x2a361fa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8861fa7, 85 },
    { 0x94f61fa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc961fac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e461faf, 91 },
    { 0x3e661fb0, 43 },
    { 0xa7b61fb1, 61 },
    { 0, 0 },
    { 0x90d61fb3, 66 },
    { 0xcdd61fb3, 95 },
    { 0x19661fb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86261fb9, 91 },
    { 0x4af61fb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f361fbd, 33 },
    { 0x6761fbe, 76 },
    { 0, 0 },
    { 0xf4461fc0, 34 },
    { 0xb0d61fc1, 33 },
    { 0, 0 },
    { 0x44e61fc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe961fcb, 98 },
    { 0, 0 },
    { 0x41b61fcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e61fd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad61fd5, 55 },
    { 0xbc061fd6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e861fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3361fde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a61fe1, 91 },
    { 0x62161fe2, 33 },
    { 0xb3161fe3, 85 },
    { 0x2fb61fe1, 40 },
    { 0xa0261fe4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f661fec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd661fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b761ff4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d62004, 61 },
    { 0x12d62004, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9062009, 61 },
    { 0x3bd62009, 33 },
    { 0xdc6200b, 77 },
    { 0xfd56200c, 90 },
    { 0xacf6200a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0862014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3386201a, 23 },
    { 0, 0 },
    { 0x2d36201c, 75 },
    { 0xb76201c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c62025, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f162029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce6202c, 75 },
    { 0, 0 },
    { 0x2e26202e, 88 },
    { 0x6a66202f, 61 },
    { 0x5ae62030, 90 },
    { 0xcda62030, 66 },
    { 0x1e162031, 33 },
    { 0xe4762033, 61 },
    { 0x4bd6202e, 61 },
    { 0xd9062031, 77 },
    { 0xc1b62036, 23 },
    { 0, 0 },
    { 0xb1762038, 34 },
    { 0x3f862039, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db6203e, 90 },
    { 0x5326203f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8262043, 61 },
    { 0xb3662044, 99 },
    { 0, 0 },
    { 0x88062046, 69 },
    { 0, 0 },
    { 0xa4662048, 78 },
    { 0xa1462049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f6204c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb56204f, 23 },
    { 0x59062050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1962055, 90 },
    { 0x31c62055, 95 },
    { 0x57d62057, 23 },
    { 0xd0562058, 76 },
    { 0, 0 },
    { 0xda96205a, 100 },
    { 0, 0 },
    { 0x3da6205c, 46 },
    { 0, 0 },
    { 0x6006205e, 84 },
    { 0xa606205f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1662062, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b62066, 34 },
    { 0xe1a62067, 94 },
    { 0x1db62068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a6206b, 98 },
    { 0x59d6206c, 32 },
    { 0, 0 },
    { 0x8b06206e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10662071, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49362074, 33 },
    { 0x5a962075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1862078, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7706207b, 23 },
    { 0x92a6207b, 93 },
    { 0x1336207d, 77 },
    { 0x1096207c, 34 },
    { 0, 0 },
    { 0x8c362080, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f62083, 46 },
    { 0xa2162083, 100 },
    { 0, 0 },
    { 0xb2a62086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4416208c, 64 },
    { 0x4c86208d, 55 },
    { 0x486208e, 95 },
    { 0x1d66208e, 24 },
    { 0, 0 },
    { 0xd8c62091, 99 },
    { 0, 0 },
    { 0x1f662093, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b62099, 92 },
    { 0, 0 },
    { 0x8cc6209b, 91 },
    { 0, 0 },
    { 0x9d66209d, 78 },
    { 0x9966209d, 52 },
    { 0xd16209f, 98 },
    { 0xd356209f, 46 },
    { 0, 0 },
    { 0xfd2620a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94620a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7620ad, 91 },
    { 0, 0 },
    { 0x482620af, 95 },
    { 0x695620b0, 98 },
    { 0x1eb620af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f620b5, 85 },
    { 0x68f620b6, 59 },
    { 0, 0 },
    { 0xada620b8, 95 },
    { 0, 0 },
    { 0xab2620ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x828620bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902620c9, 99 },
    { 0xbfc620ca, 33 },
    { 0xb86620cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e620cf, 98 },
    { 0xa1e620cf, 23 },
    { 0x34e620d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96620d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21620da, 66 },
    { 0x363620db, 98 },
    { 0x44e620dc, 93 },
    { 0x721620da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477620e7, 75 },
    { 0x69d620e7, 46 },
    { 0x2cd620e7, 66 },
    { 0xf9e620ea, 66 },
    { 0xedf620ea, 61 },
    { 0xbf3620e7, 81 },
    { 0x7bc620ed, 61 },
    { 0, 0 },
    { 0xedd620ef, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73620f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d620fb, 33 },
    { 0, 0 },
    { 0xa8d620fd, 24 },
    { 0x36a620fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb62105, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b562108, 33 },
    { 0xd8a62109, 78 },
    { 0, 0 },
    { 0x69e6210b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d26210e, 33 },
    { 0xf646210f, 98 },
    { 0xb946210e, 33 },
    { 0, 0 },
    { 0xa9262112, 33 },
    { 0, 0 },
    { 0x20d62114, 90 },
    { 0xfa262115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa276211d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5162123, 90 },
    { 0x6db62124, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5462127, 78 },
    { 0, 0 },
    { 0xe7362129, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74662135, 68 },
    { 0xaf762136, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf656213b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a562141, 90 },
    { 0x6a362142, 95 },
    { 0xdc62142, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9662148, 23 },
    { 0, 0 },
    { 0x7886214a, 85 },
    { 0x1716214b, 90 },
    { 0xd4f6214c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53362150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2862153, 99 },
    { 0x76b62153, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4062157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e62163, 95 },
    { 0x94f62164, 98 },
    { 0x66162164, 80 },
    { 0x8fc62166, 61 },
    { 0xfde62165, 93 },
    { 0xd6162163, 95 },
    { 0xeb962169, 90 },
    { 0x91762169, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58062173, 95 },
    { 0, 0 },
    { 0xeb262175, 98 },
    { 0xfd162176, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d06217c, 34 },
    { 0x86b6217d, 43 },
    { 0x6556217c, 23 },
    { 0xfe66217f, 90 },
    { 0x92f6217c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62183, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1562187, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe326218f, 78 },
    { 0xc9762190, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x64262193, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x44662196, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f56219e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x450621a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc621b0, 43 },
    { 0, 0 },
    { 0x75d621b2, 76 },
    { 0, 0 },
    { 0xc7e621b4, 90 },
    { 0xb61621b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5621b9, 33 },
    { 0, 0 },
    { 0x7f6621bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d621bf, 100 },
    { 0x6fb621c0, 84 },
    { 0, 0 },
    { 0x992621c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0621c7, 23 },
    { 0, 0 },
    { 0xe55621c9, 90 },
    { 0xe95621ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d621ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x736621d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493621d5, 55 },
    { 0x11621d5, 76 },
    { 0xf55621d7, 74 },
    { 0x5a3621d8, 95 },
    { 0x27621d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x737621dc, 34 },
    { 0x81621dd, 95 },
    { 0xf14621dd, 45 },
    { 0x373621df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd621e6, 90 },
    { 0, 0 },
    { 0x9a5621e8, 91 },
    { 0, 0 },
    { 0x8a6621ea, 77 },
    { 0, 0 },
    { 0x116621ec, 61 },
    { 0x20f621ed, 43 },
    { 0x61f621ee, 46 },
    { 0x45b621ef, 61 },
    { 0x23621ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b621fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e6220a, 33 },
    { 0x8436220b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc6220e, 33 },
    { 0x5646220f, 93 },
    { 0, 0 },
    { 0x34e62211, 77 },
    { 0, 0 },
    { 0x9de62213, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d62217, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1366221a, 95 },
    { 0x826221a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56362220, 33 },
    { 0x61862220, 90 },
    { 0xefc62222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed62228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646222d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a62232, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12062237, 90 },
    { 0xa4f62237, 55 },
    { 0x3ae62238, 61 },
    { 0, 0 },
    { 0x5c56223b, 100 },
    { 0, 0 },
    { 0xed76223d, 93 },
    { 0, 0 },
    { 0x82c6223f, 61 },
    { 0x7d562240, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b62247, 90 },
    { 0x31862248, 98 },
    { 0x85d62247, 33 },
    { 0x8616224a, 99 },
    { 0xda862249, 77 },
    { 0x67f6224c, 88 },
    { 0x1736224c, 95 },
    { 0xeb96224c, 95 },
    { 0xec26224f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c62252, 61 },
    { 0x33c62252, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d6225b, 77 },
    { 0x2356225c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc6225f, 100 },
    { 0x7a262260, 91 },
    { 0x9a462261, 46 },
    { 0x43762260, 95 },
    { 0, 0 },
    { 0xaa662264, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c66226b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a62275, 34 },
    { 0xd2d62275, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6227a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6227d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce62280, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a362287, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56228d, 95 },
    { 0xc386228e, 33 },
    { 0x2c06228e, 55 },
    { 0x6b662290, 61 },
    { 0, 0 },
    { 0x20c62292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef62295, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d62298, 90 },
    { 0x5ce62298, 90 },
    { 0xb36229a, 33 },
    { 0x9066229b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8622a2, 61 },
    { 0xd76622a2, 90 },
    { 0xe49622a2, 98 },
    { 0, 0 },
    { 0x142622a6, 77 },
    { 0x9b2622a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63622aa, 61 },
    { 0xa7d622aa, 23 },
    { 0x8f1622ac, 71 },
    { 0x5e4622ac, 90 },
    { 0xa52622ae, 76 },
    { 0xa0f622af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6622b4, 33 },
    { 0x771622b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491622bf, 23 },
    { 0xd9c622c0, 61 },
    { 0x79622c0, 76 },
    { 0x759622c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42622c6, 90 },
    { 0x8e8622c7, 61 },
    { 0xcd5622c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97622cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b622d0, 93 },
    { 0x379622d1, 93 },
    { 0, 0 },
    { 0x440622d3, 61 },
    { 0x96d622d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d622dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1622e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52622e8, 46 },
    { 0, 0 },
    { 0x34622ea, 55 },
    { 0x405622eb, 90 },
    { 0xad622eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c622ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x622f2, 34 },
    { 0x3b0622f2, 23 },
    { 0x406622f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d622f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97362300, 46 },
    { 0x8af62300, 95 },
    { 0xcf162301, 95 },
    { 0, 0 },
    { 0x60462304, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc6230b, 61 },
    { 0, 0 },
    { 0x4926230d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23662311, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f62319, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8656231c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c62320, 90 },
    { 0x18b62321, 61 },
    { 0x79962322, 99 },
    { 0xa2262322, 90 },
    { 0x8b62323, 40 },
    { 0x63762325, 55 },
    { 0xd7962322, 101 },
    { 0xc4062327, 91 },
    { 0x1162328, 33 },
    { 0xf262322, 96 },
    { 0xfb762327, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e62330, 95 },
    { 0x68562330, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f62337, 34 },
    { 0, 0 },
    { 0x13062339, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6233e, 98 },
    { 0x3496233e, 33 },
    { 0xa8f6233f, 33 },
    { 0, 0 },
    { 0xa3062342, 33 },
    { 0x3c962342, 98 },
    { 0, 0 },
    { 0xacb62345, 46 },
    { 0x7b562346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf856234a, 77 },
    { 0, 0 },
    { 0xba56234c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc6234f, 46 },
    { 0x1646234f, 68 },
    { 0x14062350, 78 },
    { 0x83e62350, 55 },
    { 0x8fd6234f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d62359, 87 },
    { 0, 0 },
    { 0xa946235b, 99 },
    { 0, 0 },
    { 0xc306235d, 95 },
    { 0, 0 },
    { 0x4a16235f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c262363, 90 },
    { 0xde162364, 90 },
    { 0x99162364, 46 },
    { 0xd8d62363, 95 },
    { 0, 0 },
    { 0x41d62368, 67 },
    { 0xdd562368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee66236e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b62373, 33 },
    { 0xdc962374, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b6237b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e16237e, 95 },
    { 0x79a6237e, 95 },
    { 0x4546237e, 61 },
    { 0xf6062381, 93 },
    { 0, 0 },
    { 0x42362383, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6238b, 95 },
    { 0xdcf6238c, 52 },
    { 0x706238b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21562394, 90 },
    { 0x72a62395, 33 },
    { 0x3f362396, 61 },
    { 0x26362397, 61 },
    { 0x36c62398, 90 },
    { 0, 0 },
    { 0x9206239a, 91 },
    { 0xf66239b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e623a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2623a3, 90 },
    { 0x25a623a4, 33 },
    { 0xb6e623a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba623a8, 87 },
    { 0x789623a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37623ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x856623af, 43 },
    { 0x1a1623b0, 43 },
    { 0x6cb623b0, 23 },
    { 0, 0 },
    { 0x4a9623b3, 76 },
    { 0, 0 },
    { 0x6f4623b5, 90 },
    { 0xac0623b6, 90 },
    { 0xc71623b7, 90 },
    { 0, 0 },
    { 0xa2d623b9, 33 },
    { 0x3e1623ba, 90 },
    { 0x761623bb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367623bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7623c2, 55 },
    { 0x1d6623c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e623c8, 66 },
    { 0x8db623c9, 33 },
    { 0x3dc623ca, 23 },
    { 0, 0 },
    { 0x5cf623cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7623cf, 93 },
    { 0x531623cf, 34 },
    { 0x4f7623d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6623d4, 90 },
    { 0x441623d5, 90 },
    { 0x711623d6, 33 },
    { 0, 0 },
    { 0x7ef623d8, 90 },
    { 0, 0 },
    { 0xe9a623da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f623dd, 55 },
    { 0x548623de, 61 },
    { 0x988623df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e623e3, 61 },
    { 0xddf623e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7623e7, 33 },
    { 0x5a2623e7, 95 },
    { 0xef2623e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e623ed, 98 },
    { 0, 0 },
    { 0x245623ef, 33 },
    { 0xce1623ef, 75 },
    { 0x246623f1, 90 },
    { 0x4be623f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6623f7, 97 },
    { 0x2ea623f8, 33 },
    { 0xfd5623f7, 90 },
    { 0, 0 },
    { 0xa3623fb, 33 },
    { 0xc13623fc, 61 },
    { 0, 0 },
    { 0x15b623fe, 33 },
    { 0x2fa623fe, 43 },
    { 0xf2662400, 77 },
    { 0, 0 },
    { 0x98562402, 33 },
    { 0xf4b62402, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2362406, 95 },
    { 0, 0 },
    { 0xa1d62408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca6240c, 66 },
    { 0x4906240d, 98 },
    { 0xd636240e, 34 },
    { 0x7af6240e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xee762412, 61 },
    { 0, 0 },
    { 0xa6762414, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a62417, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa562422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16662425, 90 },
    { 0xfb962426, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d62429, 95 },
    { 0xed762429, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c6242d, 61 },
    { 0x5096242e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2862431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b62437, 52 },
    { 0x7af62437, 33 },
    { 0xdaf62439, 90 },
    { 0x8d862437, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be6243f, 96 },
    { 0x4c462440, 23 },
    { 0xd5662441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e62444, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd6244f, 88 },
    { 0, 0 },
    { 0x11c62451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f62455, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf762459, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56262462, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e62465, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16246a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf462476, 36 },
    { 0x32c62476, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed6247b, 90 },
    { 0x84f6247b, 23 },
    { 0x6316247c, 98 },
    { 0, 0 },
    { 0x9976247f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c62482, 76 },
    { 0xcc062483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac62486, 23 },
    { 0xf1c62487, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f6248b, 76 },
    { 0xb096248c, 99 },
    { 0x7526248d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87562491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66962494, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e62497, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d6249b, 87 },
    { 0, 0 },
    { 0x7776249d, 84 },
    { 0, 0 },
    { 0xfff6249f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce624a2, 78 },
    { 0, 0 },
    { 0x779624a4, 95 },
    { 0, 0 },
    { 0xdaf624a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d624a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46624ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c624af, 90 },
    { 0xe68624af, 98 },
    { 0, 0 },
    { 0x2cd624b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc624b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5624bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181624c4, 53 },
    { 0x896624c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d624c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0624cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4624d2, 23 },
    { 0, 0 },
    { 0xaaf624d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92624d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0624df, 90 },
    { 0x3c3624df, 90 },
    { 0xbb624e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a624e5, 55 },
    { 0x12d624e6, 40 },
    { 0, 0 },
    { 0x41b624e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a624ec, 33 },
    { 0xe05624ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a624f0, 90 },
    { 0xb38624f1, 61 },
    { 0xacf624f2, 95 },
    { 0xf2f624f3, 61 },
    { 0, 0 },
    { 0x801624f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6624f8, 88 },
    { 0xdad624f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb624fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe162506, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd6250a, 61 },
    { 0x5f86250b, 23 },
    { 0, 0 },
    { 0x11d6250d, 99 },
    { 0, 0 },
    { 0xe7c6250f, 66 },
    { 0x2a962510, 90 },
    { 0xac62511, 93 },
    { 0x73c62510, 61 },
    { 0x50162511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa62516, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9962519, 99 },
    { 0x95662519, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac6251e, 98 },
    { 0, 0 },
    { 0xef362520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3862523, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78762528, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb062532, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f6253a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92062540, 90 },
    { 0xa1862541, 95 },
    { 0xff962542, 43 },
    { 0xf9862540, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2966254b, 90 },
    { 0, 0 },
    { 0xaac6254d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14762552, 55 },
    { 0, 0 },
    { 0xa2b62554, 84 },
    { 0x46962555, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f962558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac06255f, 55 },
    { 0, 0 },
    { 0x44062561, 17 },
    { 0x40a62562, 65 },
    { 0, 0 },
    { 0x5b162564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42962568, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c76256d, 78 },
    { 0x4456256d, 93 },
    { 0, 0 },
    { 0x32f62570, 76 },
    { 0x4ac62571, 38 },
    { 0x6a962572, 23 },
    { 0x59762572, 23 },
    { 0xa5c62573, 66 },
    { 0x8b462575, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x51862578, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606257e, 90 },
    { 0x4f66257f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31962585, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe636258a, 61 },
    { 0xef46258b, 90 },
    { 0x90d6258c, 23 },
    { 0x1736258d, 90 },
    { 0xf126258e, 95 },
    { 0x9246258f, 98 },
    { 0xb3d62590, 34 },
    { 0, 0 },
    { 0x7aa62592, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87562596, 95 },
    { 0xc6262596, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3836259a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf826259d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e625a0, 93 },
    { 0x50a625a1, 99 },
    { 0x40a625a2, 34 },
    { 0x38f625a3, 61 },
    { 0xc86625a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3625af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc625b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8625bc, 55 },
    { 0xc76625bd, 85 },
    { 0x205625be, 95 },
    { 0x869625bf, 69 },
    { 0xb4a625bc, 61 },
    { 0, 0 },
    { 0x59c625c2, 61 },
    { 0, 0 },
    { 0x897625c4, 99 },
    { 0xa0b625c5, 98 },
    { 0x8cf625c6, 90 },
    { 0xfb6625c5, 90 },
    { 0x8c5625c8, 98 },
    { 0x670625c9, 33 },
    { 0x1ed625c9, 95 },
    { 0x750625c8, 98 },
    { 0xb39625cc, 76 },
    { 0, 0 },
    { 0x5f1625ce, 33 },
    { 0x21625cf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e625d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244625d8, 61 },
    { 0xdbd625d9, 95 },
    { 0xc7a625d9, 61 },
    { 0, 0 },
    { 0x6be625dc, 33 },
    { 0xcd2625dd, 33 },
    { 0xcbe625dd, 52 },
    { 0, 0 },
    { 0x90f625e0, 65 },
    { 0x3b0625e1, 82 },
    { 0x560625e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29625e5, 81 },
    { 0xa7d625e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442625ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x453625ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843625f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad625f7, 43 },
    { 0xcab625f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b625fc, 55 },
    { 0x83a625fc, 80 },
    { 0xf68625fe, 33 },
    { 0xed0625fe, 46 },
    { 0xf6e625fc, 33 },
    { 0x3fc62601, 61 },
    { 0xd2d62601, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x63962605, 81 },
    { 0x20062605, 90 },
    { 0xa4f62607, 61 },
    { 0xa9962608, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe762612, 23 },
    { 0, 0 },
    { 0x3a062614, 98 },
    { 0x6562615, 88 },
    { 0xe4962615, 90 },
    { 0x29562617, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d6261a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24262625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5866262a, 66 },
    { 0x66b6262b, 68 },
    { 0xb796262b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9862630, 23 },
    { 0xdec62630, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb762634, 61 },
    { 0x13e62635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e6263a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df62646, 76 },
    { 0xf6f62647, 23 },
    { 0, 0 },
    { 0x3d762649, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf576264c, 55 },
    { 0xcf06264c, 34 },
    { 0x6586264d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1962654, 98 },
    { 0, 0 },
    { 0x7b662656, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba36265b, 90 },
    { 0, 0 },
    { 0x7b46265d, 33 },
    { 0xbe96265e, 91 },
    { 0, 0 },
    { 0x13462660, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61962665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46266b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6266e, 61 },
    { 0xde56266f, 52 },
    { 0x43f62670, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4362674, 93 },
    { 0xf3562675, 61 },
    { 0x74062676, 90 },
    { 0, 0 },
    { 0xa1262678, 33 },
    { 0xc4b62679, 23 },
    { 0, 0 },
    { 0xcd76267b, 20 },
    { 0x7146267c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc56268b, 33 },
    { 0, 0 },
    { 0xe696268d, 34 },
    { 0x8576268d, 90 },
    { 0x9c96268e, 78 },
    { 0xcc56268d, 66 },
    { 0x9936268d, 61 },
    { 0x60262692, 88 },
    { 0x77262693, 98 },
    { 0, 0 },
    { 0x2f762695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad362699, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1476269f, 88 },
    { 0, 0 },
    { 0xcf3626a1, 23 },
    { 0xd9b626a1, 23 },
    { 0x3aa626a1, 40 },
    { 0, 0 },
    { 0x799626a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66626aa, 95 },
    { 0, 0 },
    { 0x5f5626ac, 59 },
    { 0, 0 },
    { 0x77f626ae, 90 },
    { 0xe56626ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f626b2, 61 },
    { 0, 0 },
    { 0x51e626b4, 61 },
    { 0x8d1626b5, 76 },
    { 0xc88626b4, 98 },
    { 0, 0 },
    { 0xd0e626b8, 33 },
    { 0, 0 },
    { 0xd3c626ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7626be, 93 },
    { 0x8fe626be, 90 },
    { 0x7a7626be, 52 },
    { 0, 0 },
    { 0x10f626c2, 90 },
    { 0xe2a626c2, 98 },
    { 0x77f626c4, 90 },
    { 0xf93626c5, 90 },
    { 0xfa9626c4, 90 },
    { 0xc13626c4, 90 },
    { 0xc52626c6, 37 },
    { 0x59c626c3, 33 },
    { 0x45b626ca, 85 },
    { 0x27c626ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2626d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x738626d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6626d7, 98 },
    { 0xe7e626d8, 98 },
    { 0, 0 },
    { 0x912626da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1626df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab626ec, 96 },
    { 0, 0 },
    { 0xe89626ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4626f3, 68 },
    { 0, 0 },
    { 0x83a626f5, 76 },
    { 0, 0 },
    { 0xafc626f7, 85 },
    { 0xe27626f8, 93 },
    { 0x101626f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d626ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x61362702, 78 },
    { 0xd3162702, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e6270a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf986270f, 97 },
    { 0xcaf62710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62715, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x12362718, 52 },
    { 0x5e262719, 91 },
    { 0xc3a6271a, 67 },
    { 0x40f62719, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf6271e, 85 },
    { 0xcb66271e, 98 },
    { 0, 0 },
    { 0xd4062721, 84 },
    { 0xf0f62722, 95 },
    { 0, 0 },
    { 0x7462724, 95 },
    { 0, 0 },
    { 0x54c62726, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b62729, 33 },
    { 0x276272a, 95 },
    { 0, 0 },
    { 0x1826272c, 33 },
    { 0x13b6272d, 34 },
    { 0, 0 },
    { 0x4066272f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95962739, 17 },
    { 0x5162739, 34 },
    { 0x5666273b, 84 },
    { 0xb236273b, 98 },
    { 0xe686273d, 90 },
    { 0x87a6273e, 61 },
    { 0xe196273f, 98 },
    { 0x62462740, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b62743, 98 },
    { 0xa3f62744, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc286274b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf6274f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0462754, 78 },
    { 0x1cf62754, 33 },
    { 0xa0062754, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb162759, 33 },
    { 0xcee6275a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc046275f, 90 },
    { 0, 0 },
    { 0x2fe62761, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd766276b, 95 },
    { 0x56e6276c, 83 },
    { 0, 0 },
    { 0x9086276e, 98 },
    { 0, 0 },
    { 0x73962770, 95 },
    { 0xe1f62771, 91 },
    { 0x77662772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a62775, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c62778, 99 },
    { 0xe1b62778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7966277c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d62781, 61 },
    { 0xdc262782, 58 },
    { 0xdee62782, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c62788, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e62790, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a62795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b62798, 88 },
    { 0xca662798, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be6279c, 33 },
    { 0, 0 },
    { 0x2fe6279e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa627a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df627a9, 90 },
    { 0, 0 },
    { 0x798627ab, 80 },
    { 0x1a5627ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8627b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f627bd, 98 },
    { 0xf81627bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67627c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a627c8, 99 },
    { 0xb9627c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x772627cc, 90 },
    { 0x77627cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d627d0, 47 },
    { 0x1d3627d0, 98 },
    { 0x6f2627d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x898627d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699627de, 78 },
    { 0x1fe627de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x695627e2, 85 },
    { 0x3bc627e2, 90 },
    { 0x87a627e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703627e8, 90 },
    { 0xa37627e9, 33 },
    { 0x510627e8, 99 },
    { 0, 0 },
    { 0xbc0627ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0627f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x249627f7, 90 },
    { 0xe6a627f8, 33 },
    { 0x799627f7, 55 },
    { 0, 0 },
    { 0x9b7627fb, 88 },
    { 0, 0 },
    { 0x825627fd, 33 },
    { 0xfc4627fd, 33 },
    { 0x40c627fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd262802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4462807, 52 },
    { 0x9ae62808, 33 },
    { 0xe7962809, 48 },
    { 0x72c6280a, 68 },
    { 0x4d062809, 23 },
    { 0x73c62808, 23 },
    { 0x5836280d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda162817, 95 },
    { 0xed262818, 46 },
    { 0x4cd62818, 46 },
    { 0, 0 },
    { 0x72c6281b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc006281f, 61 },
    { 0x7ac6281f, 33 },
    { 0x6856281f, 95 },
    { 0xc2a62822, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a362827, 90 },
    { 0x8a662828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba6282d, 48 },
    { 0xed36282d, 33 },
    { 0xf16282d, 75 },
    { 0xf4762830, 2 },
    { 0x83f62831, 34 },
    { 0x96162831, 61 },
    { 0xe96282d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd62836, 23 },
    { 0xdbc62837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8046283e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad962841, 61 },
    { 0, 0 },
    { 0xc4162843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed662847, 23 },
    { 0, 0 },
    { 0x41162849, 64 },
    { 0x1ba6284a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc6284e, 34 },
    { 0xd1f6284f, 84 },
    { 0xc826284e, 90 },
    { 0x5536284f, 100 },
    { 0xdd06284f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82162858, 55 },
    { 0, 0 },
    { 0x8386285a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e62862, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff6286b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f62871, 23 },
    { 0x54462872, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd62876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4506287d, 61 },
    { 0x9656287e, 68 },
    { 0xc4d6287e, 90 },
    { 0x3d962880, 90 },
    { 0x37062881, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb62884, 23 },
    { 0x23a62885, 61 },
    { 0xd5f62886, 33 },
    { 0xb3b62887, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae6288d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67262891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81962895, 100 },
    { 0, 0 },
    { 0xa2862897, 78 },
    { 0x23362898, 90 },
    { 0x66f62897, 76 },
    { 0x35a6289a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b86289f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d628a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c628aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x931628ad, 61 },
    { 0, 0 },
    { 0x699628af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6628b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a628bf, 61 },
    { 0xf15628c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6628c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb628c7, 93 },
    { 0x148628c8, 33 },
    { 0, 0 },
    { 0x8e1628ca, 53 },
    { 0, 0 },
    { 0x76a628cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8628d3, 95 },
    { 0x239628d4, 34 },
    { 0xf1f628d5, 55 },
    { 0x727628d5, 33 },
    { 0x171628d7, 33 },
    { 0, 0 },
    { 0x98b628d9, 76 },
    { 0x8a5628d9, 61 },
    { 0, 0 },
    { 0xabb628dc, 98 },
    { 0x255628dd, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88628e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x884628e6, 88 },
    { 0, 0 },
    { 0x1e8628e8, 64 },
    { 0x217628e9, 23 },
    { 0x5f9628ea, 61 },
    { 0xde6628eb, 95 },
    { 0x67c628ec, 75 },
    { 0x230628ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5628f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c628f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6628ff, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xab862902, 67 },
    { 0xce062903, 67 },
    { 0xe3a62902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e62907, 91 },
    { 0, 0 },
    { 0xfbe62909, 23 },
    { 0, 0 },
    { 0x72b6290b, 77 },
    { 0, 0 },
    { 0x3596290d, 93 },
    { 0, 0 },
    { 0x49a6290f, 61 },
    { 0x6aa62910, 76 },
    { 0x5f86290f, 59 },
    { 0x3d462912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f06291a, 23 },
    { 0x1356291a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf6291f, 98 },
    { 0xe826291f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc62923, 61 },
    { 0, 0 },
    { 0xac362925, 17 },
    { 0x5e262926, 61 },
    { 0x33062927, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa056292c, 33 },
    { 0x6292d, 33 },
    { 0xc1b6292c, 90 },
    { 0xe576292f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68162932, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db62935, 90 },
    { 0, 0 },
    { 0xe4e62937, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9862942, 23 },
    { 0xbe562943, 76 },
    { 0x90662944, 100 },
    { 0xd0462943, 90 },
    { 0xf662946, 84 },
    { 0, 0 },
    { 0x5c462948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71762951, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54562957, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e86295b, 46 },
    { 0xfbf6295c, 98 },
    { 0xbc86295c, 66 },
    { 0x46f6295d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa62964, 91 },
    { 0x61b62965, 33 },
    { 0, 0 },
    { 0xf2662967, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16296a, 91 },
    { 0x9896296b, 23 },
    { 0x7556296c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa262973, 55 },
    { 0x74e62974, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42862979, 88 },
    { 0, 0 },
    { 0xc3c6297b, 61 },
    { 0x84c6297c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb62980, 67 },
    { 0x9b962981, 96 },
    { 0x64962981, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a62985, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c6298d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f062991, 34 },
    { 0x39462992, 90 },
    { 0x5e162991, 33 },
    { 0x4662994, 46 },
    { 0xc8362991, 95 },
    { 0x9da62996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3626299a, 33 },
    { 0xc2f6299a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a629a1, 90 },
    { 0xf25629a2, 90 },
    { 0, 0 },
    { 0x893629a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2629aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc629b2, 93 },
    { 0, 0 },
    { 0x11b629b4, 84 },
    { 0x238629b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x347629b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598629be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405629c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d629ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a629d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35629d7, 61 },
    { 0x98c629d8, 75 },
    { 0x4fd629d8, 77 },
    { 0, 0 },
    { 0xb09629db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5629de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4629e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397629f0, 90 },
    { 0x177629f0, 90 },
    { 0x99b629f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8629f5, 90 },
    { 0x4d1629f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b629f9, 95 },
    { 0xb5f629f9, 17 },
    { 0, 0 },
    { 0xeae629fc, 33 },
    { 0, 0 },
    { 0x147629fe, 61 },
    { 0, 0 },
    { 0x3a762a00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b62a03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99562a07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c62a0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a662a12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x17562a15, 33 },
    { 0x3e162a16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f62a1d, 90 },
    { 0, 0 },
    { 0x38d62a1f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df62a25, 66 },
    { 0x9ea62a25, 55 },
    { 0xd8562a26, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c62a2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8062a32, 91 },
    { 0, 0 },
    { 0x69a62a34, 92 },
    { 0, 0 },
    { 0xe5862a36, 61 },
    { 0x72562a37, 46 },
    { 0, 0 },
    { 0x10562a39, 101 },
    { 0, 0 },
    { 0xd8362a3b, 23 },
    { 0x7db62a3c, 76 },
    { 0x8be62a3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e262a41, 61 },
    { 0x42e62a41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70162a47, 90 },
    { 0, 0 },
    { 0xd5d62a49, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef62a4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea62a59, 61 },
    { 0xc4e62a5a, 61 },
    { 0xe2662a5b, 61 },
    { 0x27b62a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc62a5f, 90 },
    { 0x7262a60, 90 },
    { 0xb1862a61, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2762a66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75662a6c, 55 },
    { 0, 0 },
    { 0xedc62a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0262a71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b262a75, 67 },
    { 0x662a75, 99 },
    { 0x62462a77, 90 },
    { 0, 0 },
    { 0xf3c62a79, 76 },
    { 0, 0 },
    { 0xe4762a7b, 90 },
    { 0x44262a7b, 61 },
    { 0x58262a7d, 33 },
    { 0x96662a7e, 33 },
    { 0x85262a7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f62a82, 33 },
    { 0x4f562a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd62a86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d62a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc62a8d, 66 },
    { 0, 0 },
    { 0x7c762a8f, 52 },
    { 0x70162a8f, 95 },
    { 0x24f62a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f62a94, 98 },
    { 0, 0 },
    { 0x9fd62a96, 55 },
    { 0, 0 },
    { 0x71c62a98, 66 },
    { 0x35d62a99, 41 },
    { 0xe3c62a9a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54262aab, 78 },
    { 0x31962aac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d62ab4, 61 },
    { 0, 0 },
    { 0x45662ab6, 90 },
    { 0x3ef62ab6, 23 },
    { 0x96d62ab8, 95 },
    { 0x26b62ab6, 55 },
    { 0, 0 },
    { 0x1f362abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1362ac1, 61 },
    { 0xf8d62ac2, 95 },
    { 0, 0 },
    { 0x56b62ac4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83062ace, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90662ad3, 34 },
    { 0, 0 },
    { 0xe6262ad5, 81 },
    { 0xe5062ad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d762ad9, 77 },
    { 0x50f62ad9, 40 },
    { 0xc0962ada, 23 },
    { 0x35562adc, 23 },
    { 0xea862ada, 23 },
    { 0x7762ade, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf62ae5, 95 },
    { 0, 0 },
    { 0xa5662ae7, 66 },
    { 0x5162ae7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362aed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a562af0, 55 },
    { 0, 0 },
    { 0x9f862af2, 95 },
    { 0, 0 },
    { 0x7dc62af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d762af7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd262afd, 90 },
    { 0x64462afe, 91 },
    { 0xbf962aff, 46 },
    { 0, 0 },
    { 0x8a562b01, 84 },
    { 0xf3d62b02, 46 },
    { 0xa2662b02, 90 },
    { 0, 0 },
    { 0xc9462b05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e262b09, 96 },
    { 0xe2962b0a, 61 },
    { 0, 0 },
    { 0x9362b0c, 95 },
    { 0, 0 },
    { 0x80f62b0e, 98 },
    { 0x52462b0f, 76 },
    { 0x6862b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5862b14, 43 },
    { 0, 0 },
    { 0xe0362b16, 23 },
    { 0xb7362b17, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49662b1b, 61 },
    { 0xd0e62b1c, 90 },
    { 0xdf162b1d, 17 },
    { 0x19962b1e, 33 },
    { 0x5e862b1f, 76 },
    { 0x54b62b20, 90 },
    { 0xec162b1d, 90 },
    { 0x3dc62b22, 66 },
    { 0x41d62b22, 34 },
    { 0x94062b24, 61 },
    { 0x92862b25, 55 },
    { 0x53762b26, 90 },
    { 0x7962b27, 81 },
    { 0x53662b1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf62b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e62b35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac62b39, 76 },
    { 0xfff62b3a, 46 },
    { 0x82e62b3b, 95 },
    { 0xd162b3c, 99 },
    { 0xc3462b3d, 46 },
    { 0x25862b3d, 33 },
    { 0x8da62b3f, 83 },
    { 0x24f62b40, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38162b44, 84 },
    { 0, 0 },
    { 0x30362b46, 23 },
    { 0xa1d62b47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d62b4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa662b51, 90 },
    { 0x8bb62b52, 90 },
    { 0x35f62b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd62b57, 90 },
    { 0, 0 },
    { 0xade62b59, 38 },
    { 0xdd662b5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d62b5d, 85 },
    { 0x20662b5d, 91 },
    { 0, 0 },
    { 0x58662b60, 90 },
    { 0xc9e62b61, 33 },
    { 0x46362b60, 23 },
    { 0x68b62b63, 76 },
    { 0, 0 },
    { 0x8e962b65, 81 },
    { 0x13962b66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc62b6f, 17 },
    { 0x90962b6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e62b73, 91 },
    { 0xa2962b74, 33 },
    { 0x47062b75, 55 },
    { 0xd1b62b76, 33 },
    { 0x93262b75, 33 },
    { 0xa0c62b76, 33 },
    { 0x4f62b73, 61 },
    { 0x3cd62b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a62b7e, 46 },
    { 0x8de62b7f, 84 },
    { 0xc2562b80, 95 },
    { 0x8e962b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe62b8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5962b92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda762b9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4062ba0, 76 },
    { 0xad662ba1, 43 },
    { 0xa8362ba2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a62baa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb62bad, 87 },
    { 0x52e62bad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97862bb6, 91 },
    { 0x5a962bb7, 67 },
    { 0x8cb62bb8, 33 },
    { 0, 0 },
    { 0x8af62bba, 78 },
    { 0x9ce62bba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68062bbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d62bc4, 61 },
    { 0x6ce62bc5, 33 },
    { 0x25b62bc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1062bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e362bcd, 33 },
    { 0xa4f62bce, 23 },
    { 0, 0 },
    { 0xd1b62bd0, 43 },
    { 0xbc862bd0, 23 },
    { 0x72762bd2, 77 },
    { 0x48662bd3, 61 },
    { 0xb2962bd2, 66 },
    { 0x8be62bd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53562be8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a62bf1, 90 },
    { 0xea262bf1, 38 },
    { 0x83062bf1, 33 },
    { 0xad162bf2, 90 },
    { 0xc9262bf5, 78 },
    { 0xbd462bf6, 98 },
    { 0x8a62bf3, 99 },
    { 0xc2862bf8, 90 },
    { 0x64662bf9, 23 },
    { 0x20662bfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a62bff, 33 },
    { 0, 0 },
    { 0x53862c01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15562c04, 85 },
    { 0x93362c04, 99 },
    { 0, 0 },
    { 0x13762c07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82962c0c, 90 },
    { 0, 0 },
    { 0xb7262c0e, 43 },
    { 0, 0 },
    { 0x9f762c10, 55 },
    { 0x59662c11, 78 },
    { 0, 0 },
    { 0x72062c13, 78 },
    { 0x28262c14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45362c19, 61 },
    { 0x34a62c1a, 33 },
    { 0x5b362c1b, 98 },
    { 0xf7d62c1c, 98 },
    { 0xcb62c1b, 66 },
    { 0xe0362c1e, 98 },
    { 0x4ba62c1f, 90 },
    { 0xe7b62c20, 90 },
    { 0xeff62c21, 93 },
    { 0xdeb62c22, 76 },
    { 0x1862c23, 33 },
    { 0xd6b62c24, 98 },
    { 0, 0 },
    { 0x49762c26, 23 },
    { 0x96062c27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f62c30, 93 },
    { 0, 0 },
    { 0xbf062c32, 61 },
    { 0x7f62c33, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13462c36, 66 },
    { 0, 0 },
    { 0x1f462c38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98962c43, 66 },
    { 0xe2162c44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb962c4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b62c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d862c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb462c53, 98 },
    { 0xb8262c54, 84 },
    { 0x2f462c55, 95 },
    { 0x89b62c55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a62c65, 33 },
    { 0x85c62c66, 67 },
    { 0xbb762c67, 87 },
    { 0x23062c68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a62c76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea62c7b, 90 },
    { 0, 0 },
    { 0x7bf62c7d, 98 },
    { 0xe162c7e, 66 },
    { 0, 0 },
    { 0x19462c80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15762c85, 76 },
    { 0, 0 },
    { 0x33562c87, 95 },
    { 0, 0 },
    { 0xecd62c89, 93 },
    { 0x3f462c8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b62c90, 85 },
    { 0xdf062c91, 41 },
    { 0xf7a62c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e862c97, 23 },
    { 0, 0 },
    { 0xbe162c99, 98 },
    { 0xb962c99, 55 },
    { 0x9a562c9b, 90 },
    { 0x6dc62c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83462ca4, 17 },
    { 0xf2762ca5, 46 },
    { 0, 0 },
    { 0x9ea62ca7, 61 },
    { 0xa1e62ca8, 24 },
    { 0, 0 },
    { 0x4a262caa, 52 },
    { 0xd0962cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56062cb3, 23 },
    { 0x71362cb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df62cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb62cbe, 90 },
    { 0x35e62cbf, 76 },
    { 0, 0 },
    { 0xb5d62cc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d62cc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d62ccd, 78 },
    { 0xb6262ccd, 95 },
    { 0xa6e62ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe62cd3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1962cd6, 88 },
    { 0xaaa62cd6, 76 },
    { 0x5ff62cd8, 76 },
    { 0x51762cd7, 78 },
    { 0x98c62cda, 23 },
    { 0x8dc62cdb, 98 },
    { 0, 0 },
    { 0x2c362cdd, 52 },
    { 0x95862cde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a62ce2, 61 },
    { 0x8c662ce2, 55 },
    { 0, 0 },
    { 0x6c962ce5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a62ced, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13762cf1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a62cf5, 95 },
    { 0x31c62cf6, 61 },
    { 0x5f162cf7, 76 },
    { 0x58462cf8, 76 },
    { 0x3dd62cf9, 61 },
    { 0xba862cf9, 90 },
    { 0xb6e62cf8, 33 },
    { 0xb4962cf9, 90 },
    { 0x18f62cfc, 98 },
    { 0x6a462cfc, 85 },
    { 0x66662cff, 81 },
    { 0x86062d00, 90 },
    { 0xd8962cff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6162d08, 98 },
    { 0xdbe62d09, 98 },
    { 0, 0 },
    { 0x8da62d0b, 84 },
    { 0x67e62d0c, 90 },
    { 0xfb262d0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb62d10, 33 },
    { 0xf6162d11, 95 },
    { 0xbf562d11, 98 },
    { 0x24f62d11, 33 },
    { 0xeac62d10, 43 },
    { 0, 0 },
    { 0x96b62d16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5162d1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3662d20, 99 },
    { 0, 0 },
    { 0x98362d22, 98 },
    { 0x5ce62d23, 98 },
    { 0xd862d22, 99 },
    { 0, 0 },
    { 0x82c62d26, 23 },
    { 0x6bf62d27, 33 },
    { 0x81162d27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3562d2b, 61 },
    { 0x1d962d2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a462d31, 90 },
    { 0, 0 },
    { 0xd1462d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae862d36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48162d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71062d45, 78 },
    { 0x6b462d46, 55 },
    { 0xd9d62d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d62d54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe62d57, 33 },
    { 0x18962d57, 95 },
    { 0x96f62d58, 52 },
    { 0xf4162d59, 23 },
    { 0x88c62d57, 93 },
    { 0, 0 },
    { 0x80462d5d, 46 },
    { 0x3e262d5e, 98 },
    { 0xe6162d5f, 43 },
    { 0, 0 },
    { 0x4d262d61, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae62d66, 13 },
    { 0, 0 },
    { 0x8d62d68, 33 },
    { 0xa8862d69, 95 },
    { 0x5d562d6a, 98 },
    { 0x62962d6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d62d70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a62d74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58562d7a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a562d7e, 95 },
    { 0x1ae62d7f, 61 },
    { 0x98162d7e, 23 },
    { 0, 0 },
    { 0xace62d82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa62d86, 67 },
    { 0x7bb62d86, 86 },
    { 0xce462d86, 76 },
    { 0, 0 },
    { 0x9ff62d8a, 67 },
    { 0x8dd62d8b, 61 },
    { 0, 0 },
    { 0x12f62d8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51062d91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4962d97, 85 },
    { 0xb9b62d97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac62d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e262d9f, 88 },
    { 0, 0 },
    { 0xb1462da1, 33 },
    { 0x59d62da1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb762da6, 55 },
    { 0x9dc62da7, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c62daa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81262dad, 81 },
    { 0x82762dad, 77 },
    { 0, 0 },
    { 0xc5962db0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39462db5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2262db8, 90 },
    { 0x47062db9, 98 },
    { 0x9ca62dba, 90 },
    { 0x27162db9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e62dc4, 95 },
    { 0, 0 },
    { 0x6062dc6, 90 },
    { 0x40f62dc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12562dcb, 95 },
    { 0x13862dcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8762dd1, 90 },
    { 0, 0 },
    { 0xfb062dd3, 90 },
    { 0, 0 },
    { 0xa2362dd5, 40 },
    { 0x7d062dd5, 23 },
    { 0x88b62dd7, 98 },
    { 0x13562dd8, 85 },
    { 0, 0 },
    { 0x2ef62dda, 85 },
    { 0x1f562ddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f62ddf, 93 },
    { 0x762ddf, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8462de3, 78 },
    { 0x2ef62de4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda62de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13362dfe, 66 },
    { 0xa1e62dff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23262e04, 34 },
    { 0xaee62e05, 85 },
    { 0xed862e06, 23 },
    { 0x6f562e07, 84 },
    { 0xbc62e06, 34 },
    { 0x93c62e06, 23 },
    { 0xadc62e0a, 55 },
    { 0x46d62e0b, 98 },
    { 0x74962e0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67162e10, 61 },
    { 0x62062e10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef62e15, 98 },
    { 0x87c62e15, 75 },
    { 0x2962e17, 23 },
    { 0, 0 },
    { 0x4c162e19, 90 },
    { 0x3d362e19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6462e1f, 98 },
    { 0x3a862e1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97562e25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce62e2c, 98 },
    { 0x68662e2d, 61 },
    { 0xcf762e2d, 34 },
    { 0, 0 },
    { 0xa3462e30, 55 },
    { 0xdd962e30, 34 },
    { 0xdd362e31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9662e36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0862e39, 61 },
    { 0xf4862e3a, 33 },
    { 0, 0 },
    { 0x58362e3c, 93 },
    { 0x21962e3c, 61 },
    { 0xca662e3e, 61 },
    { 0xc1a62e3f, 90 },
    { 0x55462e3e, 33 },
    { 0xba162e40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb862e47, 95 },
    { 0, 0 },
    { 0xafe62e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd62e4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76262e50, 55 },
    { 0xf0362e51, 33 },
    { 0xf8562e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c62e55, 76 },
    { 0, 0 },
    { 0x69f62e57, 100 },
    { 0xbd462e57, 95 },
    { 0xe0362e57, 51 },
    { 0x54062e5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c262e5e, 33 },
    { 0x67962e5f, 33 },
    { 0xd7d62e5f, 91 },
    { 0xa7362e5f, 95 },
    { 0x76162e62, 61 },
    { 0xfad62e63, 90 },
    { 0x36b62e61, 90 },
    { 0x5d462e65, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25962e6b, 23 },
    { 0x3ea62e6c, 99 },
    { 0, 0 },
    { 0xcf562e6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed62e74, 55 },
    { 0x46862e75, 33 },
    { 0x24b62e76, 23 },
    { 0x4362e76, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65462e7e, 61 },
    { 0, 0 },
    { 0x3c262e80, 61 },
    { 0xebb62e81, 68 },
    { 0, 0 },
    { 0xd5b62e83, 94 },
    { 0x92362e83, 61 },
    { 0xd7162e84, 100 },
    { 0x65062e84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e962e89, 98 },
    { 0x45e62e89, 61 },
    { 0, 0 },
    { 0x2f762e8c, 55 },
    { 0x23462e8c, 98 },
    { 0, 0 },
    { 0x40162e8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59662e99, 90 },
    { 0, 0 },
    { 0xfcb62e9b, 95 },
    { 0, 0 },
    { 0xe2462e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0362ea0, 90 },
    { 0xacd62ea1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d062ea8, 87 },
    { 0xc4f62ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b62eac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64562eb0, 85 },
    { 0, 0 },
    { 0x7c362eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36862eb6, 80 },
    { 0, 0 },
    { 0xf4b62eb8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3062ebe, 61 },
    { 0x7ec62ebf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f262ec2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x61262ec5, 95 },
    { 0xb5d62ec6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a62eca, 23 },
    { 0, 0 },
    { 0xb4c62ecc, 93 },
    { 0x80e62ecd, 66 },
    { 0x1e062ece, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94362ed4, 76 },
    { 0x7fa62ed5, 61 },
    { 0, 0 },
    { 0xf9762ed7, 33 },
    { 0, 0 },
    { 0xfb862ed9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7962eeb, 95 },
    { 0x37562eeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b062eef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e62ef3, 61 },
    { 0x46d62ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde62ef7, 95 },
    { 0, 0 },
    { 0x3f062ef9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d62f00, 53 },
    { 0xcd162f00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd762f07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72462f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b862f16, 55 },
    { 0x9f962f17, 34 },
    { 0xf9662f17, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7662f1e, 55 },
    { 0, 0 },
    { 0xacf62f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a62f24, 95 },
    { 0, 0 },
    { 0x4bf62f26, 23 },
    { 0xc2062f27, 98 },
    { 0x9d862f28, 77 },
    { 0xb362f29, 76 },
    { 0, 0 },
    { 0x10d62f2b, 90 },
    { 0x7d362f2b, 99 },
    { 0x10462f2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8662f32, 23 },
    { 0x9f862f33, 34 },
    { 0xa5262f34, 33 },
    { 0, 0 },
    { 0x36b62f36, 98 },
    { 0x6e962f37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e662f41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1362f44, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab62f47, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd062f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f62f52, 65 },
    { 0xaaf62f53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f362f5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c662f5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63762f62, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c62f65, 23 },
    { 0xf6662f66, 34 },
    { 0x4a662f66, 17 },
    { 0x29f62f68, 23 },
    { 0x60062f69, 87 },
    { 0x59b62f6a, 61 },
    { 0, 0 },
    { 0x15a62f6c, 90 },
    { 0, 0 },
    { 0xe9d62f6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a62f71, 33 },
    { 0, 0 },
    { 0x35662f73, 99 },
    { 0, 0 },
    { 0xa6162f75, 98 },
    { 0x55262f75, 67 },
    { 0x99a62f77, 61 },
    { 0xe2d62f78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0462f7b, 90 },
    { 0xf6062f7b, 95 },
    { 0x3ed62f7d, 33 },
    { 0x77962f7e, 81 },
    { 0x3fd62f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78862f85, 98 },
    { 0x94862f86, 99 },
    { 0x9c162f87, 33 },
    { 0xb1562f87, 36 },
    { 0xdf862f89, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b662f8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6762f8f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc62f92, 34 },
    { 0x79e62f92, 61 },
    { 0x25162f94, 76 },
    { 0xb1262f92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0762f98, 76 },
    { 0xf6062f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59062f9d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a862fa3, 90 },
    { 0x21762fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c062faa, 33 },
    { 0xbd962faa, 95 },
    { 0x5362fac, 33 },
    { 0xfb562fad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b62fb3, 23 },
    { 0x18062fb4, 90 },
    { 0x34062fb5, 55 },
    { 0xcf62fb6, 95 },
    { 0xf1962fb7, 77 },
    { 0x32b62fb8, 100 },
    { 0x68262fb9, 33 },
    { 0x29462fb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2462fbe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b662fc4, 98 },
    { 0, 0 },
    { 0x71862fc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c62fcb, 93 },
    { 0, 0 },
    { 0xc6262fcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c62fd3, 90 },
    { 0xb3862fd4, 78 },
    { 0xd5c62fd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a762fde, 100 },
    { 0xfe862fdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c62fe3, 61 },
    { 0, 0 },
    { 0x51162fe5, 91 },
    { 0, 0 },
    { 0xd7662fe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2762fea, 33 },
    { 0xccc62fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc162fee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc62ff2, 23 },
    { 0x12562ff3, 90 },
    { 0, 0 },
    { 0x54e62ff5, 99 },
    { 0, 0 },
    { 0x88762ff7, 95 },
    { 0, 0 },
    { 0xe5762ff9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5962ffe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58763004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e6300f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e163015, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5056301a, 33 },
    { 0x5986301b, 91 },
    { 0, 0 },
    { 0x89a6301d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35963023, 55 },
    { 0x4263024, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18563028, 61 },
    { 0x3ed63029, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4726302f, 40 },
    { 0x7df63030, 33 },
    { 0xcf263031, 76 },
    { 0x34a63031, 77 },
    { 0xd6963031, 55 },
    { 0xda763034, 95 },
    { 0x89c63032, 90 },
    { 0xfae63036, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1816303e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc63044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e6304b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3263053, 23 },
    { 0x2a063054, 99 },
    { 0xf9763055, 55 },
    { 0x38363056, 32 },
    { 0x5a363057, 77 },
    { 0x89963058, 91 },
    { 0x14d63058, 98 },
    { 0xfa363059, 43 },
    { 0xce06305b, 96 },
    { 0x426305c, 67 },
    { 0xf136305d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f63062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8163065, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a263068, 34 },
    { 0xf2c63069, 67 },
    { 0, 0 },
    { 0xbc96306b, 99 },
    { 0x1716306b, 61 },
    { 0x1a76306c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c763072, 90 },
    { 0x87a63073, 23 },
    { 0, 0 },
    { 0x95a63075, 33 },
    { 0, 0 },
    { 0x8ab63077, 23 },
    { 0x6bf63078, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6426307b, 55 },
    { 0x6216307c, 98 },
    { 0xe5e6307c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf63084, 98 },
    { 0, 0 },
    { 0xbe963086, 86 },
    { 0x26363087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e96308b, 76 },
    { 0, 0 },
    { 0x9336308d, 61 },
    { 0, 0 },
    { 0xf966308f, 40 },
    { 0x55063090, 23 },
    { 0, 0 },
    { 0x663092, 77 },
    { 0, 0 },
    { 0x2b763094, 90 },
    { 0x78363095, 61 },
    { 0x6a863094, 76 },
    { 0, 0 },
    { 0x69c63098, 34 },
    { 0xe3a63099, 11 },
    { 0x8156309a, 33 },
    { 0xaf26309b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b630a0, 76 },
    { 0xee2630a1, 95 },
    { 0, 0 },
    { 0xf04630a3, 98 },
    { 0x41e630a4, 90 },
    { 0x1d7630a4, 46 },
    { 0xd6f630a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x278630a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7630ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64630b1, 90 },
    { 0, 0 },
    { 0x7ea630b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844630c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b630c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38630c6, 95 },
    { 0, 0 },
    { 0x94f630c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4630cc, 93 },
    { 0x8c0630cc, 43 },
    { 0x33d630ce, 34 },
    { 0xe78630cf, 95 },
    { 0xd2e630d0, 43 },
    { 0xade630ce, 33 },
    { 0x874630d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3630d8, 95 },
    { 0x36b630d8, 98 },
    { 0xa98630da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b630de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475630e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6630e6, 33 },
    { 0x97d630e7, 33 },
    { 0x71b630e7, 55 },
    { 0x2fb630e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d630ee, 77 },
    { 0x2a3630ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783630f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718630fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f263100, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c63103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca63106, 95 },
    { 0, 0 },
    { 0x57763108, 46 },
    { 0, 0 },
    { 0x3596310a, 98 },
    { 0, 0 },
    { 0x21f6310c, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e63112, 95 },
    { 0x4c163113, 78 },
    { 0x9ac63114, 76 },
    { 0, 0 },
    { 0xa5f63116, 33 },
    { 0, 0 },
    { 0xc9963118, 23 },
    { 0xf8063119, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7926311f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c563123, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a63127, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2606312d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b63131, 76 },
    { 0x6ce63131, 91 },
    { 0xd6e63133, 99 },
    { 0x43463132, 84 },
    { 0x74963135, 56 },
    { 0x8e263136, 33 },
    { 0x4f763136, 47 },
    { 0x9ee63133, 33 },
    { 0, 0 },
    { 0x3ff6313a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc006313d, 98 },
    { 0x90a6313e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e263141, 33 },
    { 0, 0 },
    { 0x9c663143, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62363147, 91 },
    { 0xe8663148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4406314c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a163151, 95 },
    { 0x8fd63151, 40 },
    { 0x28463153, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x90963156, 41 },
    { 0x66963156, 61 },
    { 0x1063158, 76 },
    { 0x96d63159, 76 },
    { 0xe5a63159, 91 },
    { 0, 0 },
    { 0xfab6315c, 88 },
    { 0, 0 },
    { 0xd9a6315e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a63163, 95 },
    { 0x1ca63164, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd63167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f76316b, 55 },
    { 0xaec6316c, 90 },
    { 0x166316d, 67 },
    { 0x93b6316e, 78 },
    { 0x68a6316f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0463172, 66 },
    { 0x4c763172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb6317a, 33 },
    { 0x4c96317b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa066317f, 98 },
    { 0xfae63180, 90 },
    { 0xd2763181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f63188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74863193, 76 },
    { 0, 0 },
    { 0xb1663195, 90 },
    { 0xa3963196, 74 },
    { 0xe1963197, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe76319e, 90 },
    { 0x2b26319f, 34 },
    { 0xf10631a0, 33 },
    { 0x4c3631a1, 100 },
    { 0x749631a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b631a5, 90 },
    { 0x196631a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5631ac, 66 },
    { 0x6d631ac, 55 },
    { 0xd2d631ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f631b1, 38 },
    { 0x77c631b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc631bd, 33 },
    { 0xb4d631bd, 90 },
    { 0x13c631bf, 61 },
    { 0x237631bf, 23 },
    { 0, 0 },
    { 0x90631c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8631c9, 33 },
    { 0x4e8631c9, 53 },
    { 0, 0 },
    { 0x5d6631cc, 90 },
    { 0xd31631cd, 85 },
    { 0x12f631ce, 67 },
    { 0, 0 },
    { 0xcd9631d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba631d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf631da, 61 },
    { 0xbc6631db, 61 },
    { 0xdc6631dc, 33 },
    { 0, 0 },
    { 0xd76631de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c631e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x415631e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4631eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55631ee, 91 },
    { 0xa4c631ee, 33 },
    { 0x754631ef, 98 },
    { 0, 0 },
    { 0x989631f2, 90 },
    { 0x121631f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c631f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a631fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f631ff, 55 },
    { 0x4d263200, 61 },
    { 0xb5e631ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34063204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf63208, 61 },
    { 0x7df63208, 61 },
    { 0x66e63209, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf616320d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d63212, 42 },
    { 0, 0 },
    { 0x5263214, 55 },
    { 0x8963215, 98 },
    { 0x70563214, 33 },
    { 0, 0 },
    { 0xf4363218, 33 },
    { 0xbae63219, 90 },
    { 0, 0 },
    { 0xbf46321b, 76 },
    { 0, 0 },
    { 0x1f06321d, 33 },
    { 0xf896321d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38063222, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a63226, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c63232, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f63236, 90 },
    { 0, 0 },
    { 0x4dd63238, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f6323b, 33 },
    { 0xb946323b, 100 },
    { 0x6346323d, 33 },
    { 0, 0 },
    { 0x6f46323f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13163243, 55 },
    { 0xcb963244, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c63247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd116324b, 95 },
    { 0x5426324c, 43 },
    { 0xb2c6324d, 17 },
    { 0xa416324e, 90 },
    { 0x8c86324e, 43 },
    { 0xcbb63250, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a63253, 46 },
    { 0xf7963254, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a6325f, 78 },
    { 0xe9c63260, 43 },
    { 0x65063261, 81 },
    { 0x6d663262, 90 },
    { 0xe5563263, 33 },
    { 0x9ca63262, 98 },
    { 0, 0 },
    { 0xa4f63266, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6326d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xab263270, 61 },
    { 0x31863271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa63276, 46 },
    { 0, 0 },
    { 0xd8e63278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056327e, 85 },
    { 0, 0 },
    { 0xba63280, 33 },
    { 0x5c163281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda363285, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c56328a, 23 },
    { 0xc2d6328b, 87 },
    { 0x42e6328a, 61 },
    { 0x40d6328d, 98 },
    { 0, 0 },
    { 0xfba6328f, 59 },
    { 0x58663290, 78 },
    { 0x98063291, 23 },
    { 0, 0 },
    { 0x57263293, 85 },
    { 0, 0 },
    { 0x6263295, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec6329b, 55 },
    { 0xf936329c, 90 },
    { 0x8186329c, 80 },
    { 0, 0 },
    { 0x8496329f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c632a5, 77 },
    { 0x650632a6, 90 },
    { 0x137632a7, 77 },
    { 0xe50632a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758632ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a632b0, 95 },
    { 0, 0 },
    { 0x606632b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94632ba, 76 },
    { 0xcca632bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67632c1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910632c6, 90 },
    { 0x1e8632c6, 66 },
    { 0xffa632c8, 98 },
    { 0xb57632c6, 33 },
    { 0x4f9632ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11632cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6632d0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1632d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5632db, 100 },
    { 0xc58632dc, 95 },
    { 0xece632dc, 23 },
    { 0xedd632de, 100 },
    { 0x428632de, 33 },
    { 0x273632e0, 76 },
    { 0x9b9632e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x924632e4, 90 },
    { 0x64a632e4, 23 },
    { 0x619632e5, 87 },
    { 0xae632e5, 81 },
    { 0, 0 },
    { 0x9cc632e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29632ec, 23 },
    { 0x548632ec, 90 },
    { 0xe04632ee, 23 },
    { 0, 0 },
    { 0xbfd632f0, 99 },
    { 0xcfb632f1, 61 },
    { 0xcef632f2, 33 },
    { 0xeb632f3, 95 },
    { 0xef632f4, 23 },
    { 0x25632f5, 90 },
    { 0xf05632f5, 33 },
    { 0xe71632f7, 98 },
    { 0xada632f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a632fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be63301, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x37163304, 61 },
    { 0, 0 },
    { 0x76963306, 95 },
    { 0x4c963307, 55 },
    { 0xdfb63307, 90 },
    { 0xc5663309, 61 },
    { 0x2b363309, 33 },
    { 0, 0 },
    { 0x336330c, 61 },
    { 0x6496330c, 34 },
    { 0x9f46330e, 88 },
    { 0, 0 },
    { 0x4db63310, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b63313, 85 },
    { 0xd5063314, 33 },
    { 0x47b63315, 77 },
    { 0xe3e63316, 76 },
    { 0xa1a63317, 61 },
    { 0xf1263318, 91 },
    { 0x7e963319, 90 },
    { 0x5863314, 76 },
    { 0xe6463313, 77 },
    { 0xdd16331c, 34 },
    { 0, 0 },
    { 0xc386331e, 76 },
    { 0, 0 },
    { 0x17963320, 43 },
    { 0xbe063320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2563329, 99 },
    { 0x8c36332a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7386332f, 33 },
    { 0x9406332f, 95 },
    { 0x3b563331, 52 },
    { 0x47663331, 61 },
    { 0xe2163333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a963336, 61 },
    { 0xdf763337, 98 },
    { 0, 0 },
    { 0x68263339, 91 },
    { 0xaed63339, 66 },
    { 0x5eb6333b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e86333e, 61 },
    { 0x96333f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c63342, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d63346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e16334b, 90 },
    { 0xa666334c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4636334f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8263352, 81 },
    { 0, 0 },
    { 0xb4663354, 52 },
    { 0, 0 },
    { 0xcde63356, 100 },
    { 0, 0 },
    { 0x2b463358, 61 },
    { 0xd1763359, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a6335e, 65 },
    { 0, 0 },
    { 0x9cb63360, 23 },
    { 0, 0 },
    { 0x7a363362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2396336a, 93 },
    { 0x14a6336b, 76 },
    { 0x2bd6336c, 23 },
    { 0x2de6336d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c63375, 91 },
    { 0, 0 },
    { 0x70a63377, 33 },
    { 0x1963377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x256337b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe586337f, 66 },
    { 0, 0 },
    { 0xe5163381, 91 },
    { 0x9c963382, 90 },
    { 0x60263381, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3563387, 61 },
    { 0x3f463387, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6338b, 93 },
    { 0, 0 },
    { 0x5856338d, 46 },
    { 0xfb56338d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d63395, 55 },
    { 0x67863396, 61 },
    { 0x2bf63395, 66 },
    { 0xb6963398, 95 },
    { 0, 0 },
    { 0x2556339a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa646339e, 85 },
    { 0x3d46339f, 61 },
    { 0, 0 },
    { 0xc71633a1, 90 },
    { 0x328633a2, 76 },
    { 0xdba633a3, 33 },
    { 0x97f633a4, 76 },
    { 0, 0 },
    { 0xf10633a6, 55 },
    { 0xce5633a6, 95 },
    { 0xab3633a6, 66 },
    { 0x413633a9, 23 },
    { 0x12d633a9, 80 },
    { 0xc33633a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616633af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0633b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9633b5, 76 },
    { 0xacc633b5, 23 },
    { 0x280633b7, 23 },
    { 0x48a633b8, 90 },
    { 0x10e633b9, 61 },
    { 0xab633b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa633bd, 95 },
    { 0x227633be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409633c8, 98 },
    { 0, 0 },
    { 0xb68633ca, 90 },
    { 0x34e633cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce633d1, 90 },
    { 0, 0 },
    { 0xe00633d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7633d6, 67 },
    { 0, 0 },
    { 0x91a633d8, 76 },
    { 0, 0 },
    { 0xcdd633da, 61 },
    { 0x7ea633db, 76 },
    { 0xb66633dc, 90 },
    { 0xf7633db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c633e0, 90 },
    { 0, 0 },
    { 0x118633e2, 76 },
    { 0, 0 },
    { 0x3d9633e4, 61 },
    { 0xb54633e5, 47 },
    { 0xd40633e6, 17 },
    { 0x827633e7, 61 },
    { 0x177633e8, 95 },
    { 0xbcb633e6, 90 },
    { 0xee7633ea, 68 },
    { 0x3a633e6, 95 },
    { 0, 0 },
    { 0x23b633ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5633f2, 34 },
    { 0xce7633f2, 95 },
    { 0x222633f2, 52 },
    { 0xbb7633f4, 61 },
    { 0xa5b633f6, 23 },
    { 0x81633f7, 99 },
    { 0x6a633f7, 33 },
    { 0xf32633f9, 100 },
    { 0xa14633f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0633fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91163408, 76 },
    { 0, 0 },
    { 0x3956340a, 90 },
    { 0, 0 },
    { 0xdb6340c, 33 },
    { 0x82e6340d, 91 },
    { 0x5c96340e, 33 },
    { 0xcff6340d, 23 },
    { 0xc526340e, 46 },
    { 0, 0 },
    { 0x96563412, 90 },
    { 0x15363413, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec63416, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59763420, 55 },
    { 0xe1863420, 90 },
    { 0x47363422, 98 },
    { 0xca663422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e63426, 55 },
    { 0xd4d63427, 95 },
    { 0, 0 },
    { 0xaf663429, 98 },
    { 0x5aa63429, 23 },
    { 0, 0 },
    { 0x5166342c, 77 },
    { 0xac6342c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e63430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49463437, 95 },
    { 0x9aa63438, 23 },
    { 0x4ae63439, 76 },
    { 0xee46343a, 40 },
    { 0x30863439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4263442, 90 },
    { 0x3f363442, 95 },
    { 0x9863444, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb216344a, 85 },
    { 0, 0 },
    { 0x4c76344c, 66 },
    { 0xc7e6344d, 98 },
    { 0, 0 },
    { 0xb9e6344f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a63453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca63456, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa296345a, 53 },
    { 0, 0 },
    { 0x8d56345c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b63460, 91 },
    { 0xae263461, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8063465, 98 },
    { 0x38163466, 95 },
    { 0x42763465, 90 },
    { 0xdee63467, 90 },
    { 0xa3363469, 23 },
    { 0xe976346a, 61 },
    { 0x8c663467, 99 },
    { 0, 0 },
    { 0xea06346d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f63470, 33 },
    { 0xb7963471, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63476, 25 },
    { 0xc1e63477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e6347a, 33 },
    { 0, 0 },
    { 0xad66347c, 23 },
    { 0x8c6347c, 90 },
    { 0x6d76347e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84763483, 33 },
    { 0x5e763484, 98 },
    { 0, 0 },
    { 0x50963486, 33 },
    { 0x7ac63486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2656348a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c6348d, 76 },
    { 0x7766348d, 43 },
    { 0x8246348f, 55 },
    { 0x61a63490, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96163496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef6349c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de6349f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860634a8, 34 },
    { 0xe0c634a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536634ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4634b0, 65 },
    { 0x5ae634b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df634b7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317634bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f634c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d634c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c634c7, 66 },
    { 0, 0 },
    { 0x210634c9, 33 },
    { 0x6da634ca, 76 },
    { 0xd35634ca, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3634ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f634d2, 95 },
    { 0xb75634d3, 90 },
    { 0x1c634d2, 61 },
    { 0x356634d3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe634d8, 91 },
    { 0x226634d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x582634dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9634e1, 98 },
    { 0, 0 },
    { 0x76c634e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9634e8, 90 },
    { 0xc93634e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c634ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85634f2, 95 },
    { 0x877634f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3634f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0763501, 61 },
    { 0xa3763502, 55 },
    { 0x5f863502, 55 },
    { 0x763502, 43 },
    { 0x26163505, 43 },
    { 0x20b63506, 76 },
    { 0xdd963506, 40 },
    { 0x16863508, 93 },
    { 0x7d963508, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a6350e, 55 },
    { 0, 0 },
    { 0x52b63510, 85 },
    { 0, 0 },
    { 0x55863512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53663515, 90 },
    { 0xc7263515, 98 },
    { 0x36163515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd236351f, 90 },
    { 0, 0 },
    { 0x74663521, 34 },
    { 0, 0 },
    { 0x2c163523, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91263527, 34 },
    { 0xc9563528, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b6352b, 93 },
    { 0x1a96352b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc63531, 90 },
    { 0x3ea63531, 33 },
    { 0x8b163533, 53 },
    { 0xbe763533, 61 },
    { 0xeb63531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91663538, 33 },
    { 0x80363539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a06353f, 90 },
    { 0x6a26353f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28563543, 34 },
    { 0, 0 },
    { 0xb6d63545, 46 },
    { 0xbd263546, 43 },
    { 0, 0 },
    { 0x38c63548, 34 },
    { 0xa3163549, 61 },
    { 0x4e26354a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef6354d, 98 },
    { 0xfda6354e, 33 },
    { 0xbaf6354e, 76 },
    { 0, 0 },
    { 0xdfe63551, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca6355a, 33 },
    { 0xd6355b, 90 },
    { 0x8266355c, 87 },
    { 0xab26355a, 87 },
    { 0x3906355c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced63565, 76 },
    { 0, 0 },
    { 0xb8d63567, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3296356b, 23 },
    { 0x946356c, 23 },
    { 0xcd76356b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad363570, 55 },
    { 0xa2663571, 91 },
    { 0xcc163570, 98 },
    { 0x1363573, 43 },
    { 0, 0 },
    { 0x4c763575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea46357b, 33 },
    { 0xf0b6357c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28363580, 84 },
    { 0x63663580, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf63589, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46358c, 52 },
    { 0xbf16358c, 100 },
    { 0, 0 },
    { 0xb066358f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf63592, 90 },
    { 0, 0 },
    { 0xeba63594, 90 },
    { 0xd763595, 22 },
    { 0, 0 },
    { 0x23363597, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed635a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03635a4, 98 },
    { 0xfcc635a4, 55 },
    { 0x1f6635a6, 90 },
    { 0, 0 },
    { 0x945635a8, 90 },
    { 0, 0 },
    { 0x193635aa, 78 },
    { 0, 0 },
    { 0x1d5635ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81635af, 91 },
    { 0xb01635af, 95 },
    { 0xef0635b0, 90 },
    { 0x636635b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6635b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43635b9, 95 },
    { 0, 0 },
    { 0x939635bb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388635bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee635c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x552635c6, 66 },
    { 0x219635c7, 65 },
    { 0xfd6635c8, 61 },
    { 0x323635c9, 76 },
    { 0, 0 },
    { 0x726635cb, 34 },
    { 0x284635cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae635d1, 33 },
    { 0xdf3635d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a635d5, 34 },
    { 0, 0 },
    { 0x18c635d7, 90 },
    { 0xf1635d8, 77 },
    { 0xab635d8, 93 },
    { 0x4f7635d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7635de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858635e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92635e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02635ec, 76 },
    { 0, 0 },
    { 0xe8a635ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x718635f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5635fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90363600, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e63605, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d6360b, 90 },
    { 0x4ab6360c, 33 },
    { 0x4296360b, 90 },
    { 0xbd86360e, 61 },
    { 0x18d6360b, 99 },
    { 0, 0 },
    { 0xbe563611, 56 },
    { 0xbf963612, 65 },
    { 0x68163611, 46 },
    { 0xb2163614, 100 },
    { 0x32963613, 43 },
    { 0, 0 },
    { 0xf1b63617, 33 },
    { 0x2f163617, 90 },
    { 0x18163619, 95 },
    { 0xab6361a, 34 },
    { 0x1786361a, 90 },
    { 0x90f63619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b76361f, 46 },
    { 0, 0 },
    { 0x40363621, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64263625, 43 },
    { 0, 0 },
    { 0x66463627, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456362e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20663631, 33 },
    { 0, 0 },
    { 0xaf563633, 90 },
    { 0, 0 },
    { 0xc3363635, 33 },
    { 0, 0 },
    { 0xe3a63637, 90 },
    { 0x34763637, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd676363c, 66 },
    { 0, 0 },
    { 0xf866363e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e463643, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e6364d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c63652, 98 },
    { 0x2f463652, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d663656, 17 },
    { 0x2963657, 90 },
    { 0, 0 },
    { 0x24d63659, 55 },
    { 0xf006365a, 98 },
    { 0x26363659, 34 },
    { 0x3086365c, 90 },
    { 0xaed6365a, 93 },
    { 0x2206365e, 93 },
    { 0, 0 },
    { 0xb5463660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f763664, 66 },
    { 0x63163665, 33 },
    { 0, 0 },
    { 0xe7963667, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d6366d, 76 },
    { 0x27f6366d, 95 },
    { 0, 0 },
    { 0x63d63670, 95 },
    { 0, 0 },
    { 0x3a263672, 99 },
    { 0, 0 },
    { 0x88f63674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1926367b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33063680, 98 },
    { 0xe1363680, 43 },
    { 0, 0 },
    { 0x8a863683, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd706368e, 23 },
    { 0x8c16368f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6763694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14663698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb06369c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6346369f, 86 },
    { 0, 0 },
    { 0xb00636a1, 34 },
    { 0x5fa636a2, 90 },
    { 0x69b636a3, 95 },
    { 0x526636a4, 76 },
    { 0xc56636a4, 34 },
    { 0x3b0636a5, 75 },
    { 0, 0 },
    { 0x7fb636a8, 68 },
    { 0, 0 },
    { 0x1af636aa, 76 },
    { 0xf1c636ab, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1636b3, 90 },
    { 0x874636b3, 87 },
    { 0xa5e636b4, 90 },
    { 0xf09636b6, 90 },
    { 0x8dc636b4, 90 },
    { 0xe5d636b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2636be, 90 },
    { 0x9d5636bf, 55 },
    { 0xb00636c0, 90 },
    { 0x887636c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d636d2, 46 },
    { 0, 0 },
    { 0x8e1636d4, 61 },
    { 0xa85636d5, 95 },
    { 0xfb7636d6, 98 },
    { 0xdb9636d7, 34 },
    { 0, 0 },
    { 0xcf4636d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731636de, 33 },
    { 0xf94636df, 55 },
    { 0x338636df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f636e4, 23 },
    { 0x90f636e4, 61 },
    { 0x3d4636e6, 93 },
    { 0x7d7636e6, 99 },
    { 0xabe636e7, 98 },
    { 0x7fa636e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f636ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c636f2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07636f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3636fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3636fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b63706, 46 },
    { 0, 0 },
    { 0xb7063708, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd6370d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c63714, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e6371b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26371f, 33 },
    { 0, 0 },
    { 0x4f363721, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd63726, 33 },
    { 0, 0 },
    { 0x8ff63728, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3616372c, 95 },
    { 0xe166372c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7363730, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa63734, 46 },
    { 0x45263735, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf336373a, 85 },
    { 0, 0 },
    { 0x9d26373c, 23 },
    { 0x7ff6373d, 61 },
    { 0x8af6373c, 33 },
    { 0x7d16373f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17163743, 98 },
    { 0, 0 },
    { 0xe7263745, 33 },
    { 0, 0 },
    { 0x6cb63747, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd876374a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb6374f, 78 },
    { 0, 0 },
    { 0x1ed63751, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f063754, 78 },
    { 0x5b163755, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226375e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23963761, 90 },
    { 0xd6b63762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b63766, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1226376b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b063770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b563773, 91 },
    { 0, 0 },
    { 0x32c63775, 23 },
    { 0, 0 },
    { 0x55563777, 61 },
    { 0x4d663778, 33 },
    { 0, 0 },
    { 0x8996377a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x576377d, 93 },
    { 0x6d76377e, 34 },
    { 0x2f66377e, 76 },
    { 0, 0 },
    { 0x8f763781, 91 },
    { 0x2de63781, 55 },
    { 0x46163781, 99 },
    { 0xd7d63784, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82863787, 68 },
    { 0x4af63788, 99 },
    { 0, 0 },
    { 0xa7e6378a, 90 },
    { 0x9bb6378b, 33 },
    { 0x6f66378a, 93 },
    { 0x3996378b, 61 },
    { 0xbf56378e, 90 },
    { 0xc0a6378b, 95 },
    { 0x53b6378a, 90 },
    { 0xabc63791, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d63796, 84 },
    { 0xfee63796, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46379d, 90 },
    { 0, 0 },
    { 0xa386379f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689637a7, 23 },
    { 0, 0 },
    { 0xc1637a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b637b0, 90 },
    { 0x432637b1, 94 },
    { 0xc0d637b0, 61 },
    { 0x524637b3, 95 },
    { 0x155637b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5637b9, 61 },
    { 0xc19637ba, 98 },
    { 0, 0 },
    { 0x9ac637bc, 75 },
    { 0x4dd637bd, 23 },
    { 0x27a637bd, 81 },
    { 0x41c637bf, 98 },
    { 0, 0 },
    { 0x598637c1, 40 },
    { 0xc14637c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62637c5, 17 },
    { 0x273637c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04637ca, 90 },
    { 0xc9637cb, 23 },
    { 0x1dd637cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1637d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x107637d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51637dc, 61 },
    { 0, 0 },
    { 0x837637de, 83 },
    { 0xe6c637de, 98 },
    { 0xcfa637df, 88 },
    { 0x924637e1, 90 },
    { 0x90e637e0, 98 },
    { 0xe74637e3, 34 },
    { 0x709637e4, 46 },
    { 0x4d5637e4, 90 },
    { 0x19f637e6, 33 },
    { 0x57f637e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x722637ea, 61 },
    { 0, 0 },
    { 0xc04637ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x418637ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc637f6, 77 },
    { 0x3d4637f6, 33 },
    { 0, 0 },
    { 0x562637f9, 33 },
    { 0, 0 },
    { 0x788637fb, 91 },
    { 0xb6b637fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b063804, 55 },
    { 0x88463805, 98 },
    { 0x56c63806, 69 },
    { 0x7c263805, 95 },
    { 0x83563807, 90 },
    { 0, 0 },
    { 0x8796380a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82563812, 85 },
    { 0x9cd63813, 61 },
    { 0xfcc63812, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13363819, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b6381e, 61 },
    { 0x7346381f, 84 },
    { 0, 0 },
    { 0xf6a63821, 23 },
    { 0xee663822, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea86382b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d6383c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb263841, 95 },
    { 0, 0 },
    { 0x56563843, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d663849, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad63850, 100 },
    { 0xf6b63851, 66 },
    { 0x76463851, 90 },
    { 0x53e63852, 33 },
    { 0, 0 },
    { 0x26063855, 33 },
    { 0x22a63856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93863859, 68 },
    { 0xa056385a, 61 },
    { 0, 0 },
    { 0x8476385c, 33 },
    { 0, 0 },
    { 0x7396385e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed63861, 43 },
    { 0, 0 },
    { 0x56163863, 100 },
    { 0xd3163864, 33 },
    { 0x88763865, 80 },
    { 0x64763866, 95 },
    { 0xb5963864, 43 },
    { 0xd6263868, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7996386d, 66 },
    { 0xe226386d, 98 },
    { 0x2f86386f, 55 },
    { 0xd326386f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49863875, 61 },
    { 0xb8063875, 90 },
    { 0x83963876, 23 },
    { 0x23f63878, 90 },
    { 0x8f63879, 93 },
    { 0xb5b63878, 23 },
    { 0, 0 },
    { 0xaaa6387c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead63880, 90 },
    { 0, 0 },
    { 0x9ec63882, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b63885, 61 },
    { 0, 0 },
    { 0x31d63887, 93 },
    { 0xe163887, 67 },
    { 0xa0963887, 87 },
    { 0x32d6388a, 90 },
    { 0x3d963889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be6388e, 69 },
    { 0xc9a6388e, 90 },
    { 0x6fc63890, 85 },
    { 0xa4a63890, 90 },
    { 0x8d063892, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d563895, 95 },
    { 0xe2863896, 93 },
    { 0x3ab63897, 95 },
    { 0xd0b63898, 98 },
    { 0, 0 },
    { 0xd266389a, 100 },
    { 0x8406389b, 81 },
    { 0x5c56389a, 95 },
    { 0x7c6389c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6638a1, 98 },
    { 0, 0 },
    { 0x4fe638a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x672638a6, 34 },
    { 0x232638a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5638ab, 76 },
    { 0xcc8638ab, 33 },
    { 0x7aa638ad, 61 },
    { 0x4be638ae, 90 },
    { 0x7cd638af, 98 },
    { 0, 0 },
    { 0x4da638b1, 53 },
    { 0, 0 },
    { 0x4b1638b3, 78 },
    { 0, 0 },
    { 0xa93638b5, 77 },
    { 0x882638b5, 98 },
    { 0x184638b7, 76 },
    { 0x45b638b6, 85 },
    { 0x6b8638b9, 95 },
    { 0x4e4638ba, 38 },
    { 0, 0 },
    { 0x627638bc, 43 },
    { 0xbb1638bd, 95 },
    { 0, 0 },
    { 0xed1638bf, 95 },
    { 0, 0 },
    { 0x785638c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3638c8, 33 },
    { 0x33f638c9, 91 },
    { 0, 0 },
    { 0x151638cb, 23 },
    { 0x7f4638cb, 95 },
    { 0x81a638cb, 95 },
    { 0, 0 },
    { 0x8b638cf, 43 },
    { 0x572638cf, 61 },
    { 0x8ad638d0, 43 },
    { 0, 0 },
    { 0x179638d3, 93 },
    { 0x2b2638d3, 34 },
    { 0x86f638d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54638db, 34 },
    { 0xec9638dc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4638df, 53 },
    { 0xf4c638df, 56 },
    { 0x2cd638e0, 76 },
    { 0, 0 },
    { 0xaeb638e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0638ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x815638ef, 43 },
    { 0x693638f0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11638f3, 99 },
    { 0xd2d638f4, 95 },
    { 0xc9b638f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x405638f8, 46 },
    { 0xe1c638f9, 95 },
    { 0x96b638fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7638fd, 43 },
    { 0x944638fe, 43 },
    { 0x5b1638fe, 90 },
    { 0x20963900, 98 },
    { 0, 0 },
    { 0x95963902, 61 },
    { 0x3563903, 23 },
    { 0x25063903, 33 },
    { 0x28b63905, 33 },
    { 0x82b63906, 95 },
    { 0xc3463906, 23 },
    { 0xfac63906, 33 },
    { 0, 0 },
    { 0x5df6390a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9663913, 61 },
    { 0xdbf63913, 90 },
    { 0xc4d63913, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4686391a, 98 },
    { 0xf376391b, 52 },
    { 0x75d6391b, 98 },
    { 0x4b26391d, 99 },
    { 0, 0 },
    { 0x206391f, 43 },
    { 0xe7e63920, 33 },
    { 0, 0 },
    { 0xa6763922, 34 },
    { 0x25c63923, 61 },
    { 0xa8363923, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c63927, 76 },
    { 0, 0 },
    { 0xceb63929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b6392e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46263933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf63939, 98 },
    { 0x85d6393a, 90 },
    { 0xaf76393a, 98 },
    { 0xad36393b, 77 },
    { 0, 0 },
    { 0x29a6393e, 95 },
    { 0, 0 },
    { 0xa1c63940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e63943, 76 },
    { 0xf7763944, 90 },
    { 0x2ff63945, 93 },
    { 0xbd563946, 64 },
    { 0x9e963943, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e46394c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6394f, 95 },
    { 0xfc76394f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc063956, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3163959, 33 },
    { 0, 0 },
    { 0xe656395b, 67 },
    { 0xadb6395c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7163960, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e463964, 61 },
    { 0x33d63965, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f63970, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90463974, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b63979, 98 },
    { 0, 0 },
    { 0xd746397b, 90 },
    { 0x1486397c, 91 },
    { 0, 0 },
    { 0xc3e6397e, 23 },
    { 0x1f16397f, 98 },
    { 0x5b663980, 98 },
    { 0x57063981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd763991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e763995, 91 },
    { 0xfc63996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75363999, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c6399e, 90 },
    { 0x9946399f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19639a4, 59 },
    { 0, 0 },
    { 0x9c0639a6, 23 },
    { 0x83d639a6, 90 },
    { 0x46639a8, 23 },
    { 0xf50639a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8639b1, 94 },
    { 0, 0 },
    { 0xb20639b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9639b9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20639bd, 34 },
    { 0xb7a639bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452639c2, 100 },
    { 0, 0 },
    { 0xca2639c4, 95 },
    { 0, 0 },
    { 0xac0639c6, 85 },
    { 0x33639c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02639cf, 55 },
    { 0x616639cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252639d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32639d8, 33 },
    { 0, 0 },
    { 0xd4d639da, 66 },
    { 0x55b639da, 98 },
    { 0x668639dc, 33 },
    { 0xf99639dc, 66 },
    { 0x46b639de, 98 },
    { 0x2e639dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8639e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0639e7, 90 },
    { 0x14b639e8, 55 },
    { 0, 0 },
    { 0x94c639ea, 46 },
    { 0x694639eb, 75 },
    { 0x1be639eb, 61 },
    { 0x349639ec, 95 },
    { 0xfbb639ed, 64 },
    { 0, 0 },
    { 0x1a4639f0, 90 },
    { 0x4c5639f1, 98 },
    { 0, 0 },
    { 0xfe8639f3, 43 },
    { 0x467639f4, 90 },
    { 0x7d4639f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb639fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f63a02, 95 },
    { 0xc7263a03, 23 },
    { 0x2d163a04, 61 },
    { 0x99863a05, 76 },
    { 0x94863a04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb63a09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd63a0c, 23 },
    { 0, 0 },
    { 0x16163a0e, 2 },
    { 0xef063a0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5263a15, 93 },
    { 0xdd263a16, 99 },
    { 0x72363a17, 90 },
    { 0, 0 },
    { 0x26863a19, 95 },
    { 0xe1763a1a, 95 },
    { 0xb6863a1a, 23 },
    { 0, 0 },
    { 0x53963a1d, 68 },
    { 0x69363a1d, 88 },
    { 0xb6e63a1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d63a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd963a28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95963a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac63a38, 80 },
    { 0, 0 },
    { 0x27463a3a, 34 },
    { 0, 0 },
    { 0xb6d63a3c, 83 },
    { 0x29363a3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x59363a40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15163a43, 91 },
    { 0, 0 },
    { 0xa4463a45, 50 },
    { 0x3d863a46, 93 },
    { 0xa1663a46, 24 },
    { 0x5e363a46, 33 },
    { 0x16d63a46, 53 },
    { 0xc1263a4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38063a4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89363a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d63a56, 52 },
    { 0, 0 },
    { 0x79f63a58, 23 },
    { 0, 0 },
    { 0x31763a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb63a5f, 93 },
    { 0, 0 },
    { 0xf0963a61, 34 },
    { 0xc8663a62, 65 },
    { 0x51863a62, 33 },
    { 0x91663a64, 78 },
    { 0x14b63a62, 95 },
    { 0xff563a66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc63a6a, 95 },
    { 0, 0 },
    { 0x70463a6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b63a6f, 95 },
    { 0xce063a6f, 87 },
    { 0x4c63a70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b63a74, 90 },
    { 0x38163a74, 47 },
    { 0x1a763a76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30163a79, 84 },
    { 0, 0 },
    { 0x45563a7b, 83 },
    { 0xc5c63a7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3163a7f, 95 },
    { 0xfd763a80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d63a83, 34 },
    { 0, 0 },
    { 0x81b63a85, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c963a88, 55 },
    { 0x99d63a89, 95 },
    { 0x3bc63a8a, 95 },
    { 0x32463a8b, 87 },
    { 0x6b663a8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34263a8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe363a94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7963a9a, 95 },
    { 0xfc563a9b, 61 },
    { 0, 0 },
    { 0xa3e63a9d, 23 },
    { 0x14163a9d, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x29963aa1, 68 },
    { 0x19e63aa2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f63ac1, 76 },
    { 0xd0e63ac1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73363ac5, 66 },
    { 0, 0 },
    { 0x2ad63ac7, 33 },
    { 0, 0 },
    { 0xcdc63ac9, 90 },
    { 0x2d663aca, 61 },
    { 0x2f163aca, 90 },
    { 0x6a063ac9, 90 },
    { 0xc6963acd, 43 },
    { 0, 0 },
    { 0x3f163acf, 90 },
    { 0xea863ad0, 98 },
    { 0xc1563ad1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f363ad4, 34 },
    { 0x99963ad4, 65 },
    { 0xf3363ad6, 93 },
    { 0x74c63ad7, 55 },
    { 0, 0 },
    { 0x1e363ad9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d563ae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86063ae9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x56763aec, 55 },
    { 0x7e363aed, 61 },
    { 0, 0 },
    { 0xac763aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f963af8, 90 },
    { 0x77363af9, 61 },
    { 0x3e063afa, 95 },
    { 0, 0 },
    { 0x7da63afc, 93 },
    { 0x59663afc, 98 },
    { 0x9b363afe, 90 },
    { 0x8d63afe, 61 },
    { 0, 0 },
    { 0xa4963b01, 67 },
    { 0x29163b01, 85 },
    { 0x81863b03, 68 },
    { 0x34963b03, 61 },
    { 0xa4b63b05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe263b09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f563b0d, 99 },
    { 0, 0 },
    { 0x5a763b0f, 81 },
    { 0x29763b10, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c63b18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48663b1c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc63b23, 76 },
    { 0, 0 },
    { 0xe4563b25, 76 },
    { 0x90363b26, 43 },
    { 0x72963b26, 33 },
    { 0xf0f63b25, 100 },
    { 0, 0 },
    { 0x91963b2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1463b31, 66 },
    { 0x2ef63b32, 46 },
    { 0x3863b33, 43 },
    { 0x5cd63b34, 61 },
    { 0xf3e63b31, 90 },
    { 0, 0 },
    { 0x8cf63b37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf963b40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab63b43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9063b4b, 70 },
    { 0x6ce63b4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40163b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64063b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d63b5c, 90 },
    { 0x65063b5c, 87 },
    { 0, 0 },
    { 0xda363b5f, 92 },
    { 0, 0 },
    { 0x26563b61, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd63b69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d63b6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd63b73, 61 },
    { 0x1df63b74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c63b7b, 98 },
    { 0xac63b7c, 61 },
    { 0x3d563b7d, 99 },
    { 0x7f863b7e, 76 },
    { 0xa0663b7f, 61 },
    { 0, 0 },
    { 0xb863b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a963b84, 90 },
    { 0xd3563b84, 61 },
    { 0xc6b63b86, 76 },
    { 0xac863b87, 33 },
    { 0x56763b87, 90 },
    { 0x36f63b89, 78 },
    { 0xfcf63b8a, 33 },
    { 0x43f63b88, 46 },
    { 0xa0c63b8c, 90 },
    { 0x71c63b8c, 78 },
    { 0xa0f63b88, 90 },
    { 0x16663b88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21063b94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87263b99, 61 },
    { 0x1dd63b9a, 61 },
    { 0xb563b9b, 65 },
    { 0xf8a63b9a, 17 },
    { 0xcc063b9d, 84 },
    { 0xa7263b9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e663ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c863baa, 95 },
    { 0xacd63bab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf63bae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f63bb2, 33 },
    { 0x22263bb2, 34 },
    { 0x14463bb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c63bb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e063bbb, 33 },
    { 0, 0 },
    { 0x78963bbd, 61 },
    { 0x6763bbe, 43 },
    { 0x24563bbf, 46 },
    { 0xf4c63bbe, 67 },
    { 0xd8a63bc1, 61 },
    { 0x77463bc1, 76 },
    { 0x18163bc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a63bc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd663bdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64663bdf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa163be2, 34 },
    { 0x15763be3, 93 },
    { 0, 0 },
    { 0x82e63be5, 66 },
    { 0x8ce63be6, 78 },
    { 0x93063be5, 65 },
    { 0xa3b63be8, 67 },
    { 0x4ef63be9, 76 },
    { 0xfc863be9, 23 },
    { 0, 0 },
    { 0xd8e63bec, 91 },
    { 0x17363bed, 76 },
    { 0xb9d63bed, 90 },
    { 0, 0 },
    { 0x97963bf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66763bf3, 33 },
    { 0, 0 },
    { 0x74863bf5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29763bf9, 61 },
    { 0xb9063bfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a63c04, 90 },
    { 0, 0 },
    { 0x6c463c06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20463c0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f63c12, 91 },
    { 0x30563c13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47163c16, 77 },
    { 0xf8a63c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26763c20, 23 },
    { 0, 0 },
    { 0x23263c22, 84 },
    { 0x9d263c22, 95 },
    { 0x4dc63c24, 33 },
    { 0xb1263c24, 33 },
    { 0xa8263c26, 36 },
    { 0xdcf63c24, 95 },
    { 0xe0363c28, 95 },
    { 0x6c163c29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e63c2c, 33 },
    { 0xf0963c2d, 90 },
    { 0x40e63c2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963c32, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e663c36, 90 },
    { 0x83363c37, 46 },
    { 0xadd63c38, 55 },
    { 0x56663c39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39763c3d, 98 },
    { 0xe1463c3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa63c42, 55 },
    { 0x2ee63c43, 33 },
    { 0, 0 },
    { 0x83963c45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d63c4b, 98 },
    { 0x10c63c4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36663c4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4063c56, 78 },
    { 0x65d63c57, 43 },
    { 0xffb63c58, 76 },
    { 0x1f663c59, 40 },
    { 0xea963c5a, 95 },
    { 0x4c663c5b, 61 },
    { 0xe3f63c5b, 93 },
    { 0xece63c5d, 23 },
    { 0, 0 },
    { 0x90a63c5f, 33 },
    { 0, 0 },
    { 0xc5c63c61, 90 },
    { 0, 0 },
    { 0x55463c63, 34 },
    { 0x8df63c64, 52 },
    { 0x8663c65, 81 },
    { 0x8fb63c66, 61 },
    { 0x54963c67, 17 },
    { 0x22d63c67, 95 },
    { 0x33f63c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b463c6c, 77 },
    { 0xe8263c6d, 76 },
    { 0xe3363c6e, 90 },
    { 0xfec63c6e, 46 },
    { 0x16863c70, 33 },
    { 0xc7363c71, 24 },
    { 0xb9b63c72, 61 },
    { 0xaae63c73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee63c77, 91 },
    { 0xb5d63c77, 93 },
    { 0x77c63c79, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a763c7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c63c85, 98 },
    { 0, 0 },
    { 0x85763c87, 95 },
    { 0, 0 },
    { 0x95d63c89, 61 },
    { 0, 0 },
    { 0xe3a63c8b, 52 },
    { 0x763c8b, 98 },
    { 0x61463c8d, 61 },
    { 0x36463c8e, 34 },
    { 0x90963c8c, 92 },
    { 0xd4863c8d, 75 },
    { 0x41f63c91, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9563c94, 85 },
    { 0, 0 },
    { 0x2ad63c96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85063c9a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7663c9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a163ca2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3863ca7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2163caa, 52 },
    { 0, 0 },
    { 0x87663cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b63cb5, 80 },
    { 0, 0 },
    { 0x21063cb7, 99 },
    { 0xc6963cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc863cbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4163cbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b263cc2, 61 },
    { 0x6c763cc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d63cc6, 87 },
    { 0xe7663cc7, 34 },
    { 0, 0 },
    { 0x9cc63cc9, 61 },
    { 0xd3663cc9, 76 },
    { 0x62763cc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e63cd2, 92 },
    { 0xf0e63cd3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7963cd6, 67 },
    { 0xd1263cd7, 43 },
    { 0, 0 },
    { 0xd4563cd9, 53 },
    { 0x38f63cda, 61 },
    { 0xef563cdb, 33 },
    { 0, 0 },
    { 0xe8c63cdd, 77 },
    { 0, 0 },
    { 0x2463cdf, 34 },
    { 0x99563cdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa63ce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74463ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d63ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0763cf0, 91 },
    { 0x50e63cf1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16363cf6, 67 },
    { 0, 0 },
    { 0xfcc63cf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf163cfb, 61 },
    { 0xa8163cfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7263d02, 78 },
    { 0xc8563d02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90763d07, 87 },
    { 0x91463d07, 23 },
    { 0, 0 },
    { 0x89d63d0a, 66 },
    { 0, 0 },
    { 0x1af63d0c, 34 },
    { 0xc8163d0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c963d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd63d16, 56 },
    { 0x36463d16, 67 },
    { 0, 0 },
    { 0xdee63d19, 95 },
    { 0x35963d1a, 88 },
    { 0x20963d19, 64 },
    { 0x10c63d1c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44963d22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b63d25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0163d30, 61 },
    { 0x98063d31, 76 },
    { 0x963d32, 88 },
    { 0xcf363d33, 81 },
    { 0x1d763d33, 90 },
    { 0x8fd63d35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d63d38, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c063d3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c63d4a, 23 },
    { 0x55d63d4b, 61 },
    { 0x8ed63d4a, 68 },
    { 0x68363d4a, 33 },
    { 0xadf63d4e, 61 },
    { 0, 0 },
    { 0xda563d50, 88 },
    { 0, 0 },
    { 0x1f563d52, 100 },
    { 0x9c463d53, 95 },
    { 0xd5c63d53, 95 },
    { 0, 0 },
    { 0x5b063d56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9763d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5263d5d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7063d65, 55 },
    { 0x8e863d66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5763d6a, 76 },
    { 0xa8963d6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7663d6e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8763d71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x80963d74, 78 },
    { 0xe9163d75, 33 },
    { 0xbba63d76, 23 },
    { 0xb6063d76, 90 },
    { 0x79963d77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a63d7f, 90 },
    { 0, 0 },
    { 0x39763d81, 61 },
    { 0x2eb63d81, 98 },
    { 0xed763d82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad63d8b, 55 },
    { 0x28f63d8b, 23 },
    { 0, 0 },
    { 0x8e63d8e, 88 },
    { 0x38063d8f, 37 },
    { 0x7d563d90, 95 },
    { 0x7b663d90, 23 },
    { 0xf1263d90, 95 },
    { 0x78f63d93, 33 },
    { 0xa7d63d92, 95 },
    { 0xc7f63d8e, 90 },
    { 0xb9b63d96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf63d9b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xad463d9e, 78 },
    { 0, 0 },
    { 0x1ca63da0, 65 },
    { 0xe3163da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e63dae, 61 },
    { 0xca463daf, 98 },
    { 0, 0 },
    { 0x5bc63db1, 33 },
    { 0x58663db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23363db5, 87 },
    { 0xbbf63db6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead63dbb, 46 },
    { 0x15c63dbc, 34 },
    { 0xf263dbd, 76 },
    { 0x72663dbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29863dc1, 95 },
    { 0x17763dc2, 76 },
    { 0x16c63dc3, 95 },
    { 0, 0 },
    { 0xb3763dc5, 95 },
    { 0x8a263dc5, 61 },
    { 0x26b63dc5, 85 },
    { 0, 0 },
    { 0x29663dc9, 98 },
    { 0x10763dca, 34 },
    { 0, 0 },
    { 0x67163dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf63dcf, 34 },
    { 0, 0 },
    { 0x6db63dd1, 33 },
    { 0x80f63dd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b63dd5, 46 },
    { 0xf0f63dd6, 88 },
    { 0x6a063dd6, 95 },
    { 0x5b063dd7, 82 },
    { 0x16663dd9, 61 },
    { 0xbaa63dd9, 93 },
    { 0xe0463ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2663de2, 90 },
    { 0x28e63de2, 77 },
    { 0xaf563de4, 81 },
    { 0xc4263de5, 33 },
    { 0, 0 },
    { 0x15a63de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa763ded, 61 },
    { 0x55563dee, 76 },
    { 0x1f663def, 77 },
    { 0xa4063def, 95 },
    { 0x58763df1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a63df5, 93 },
    { 0x8463df6, 33 },
    { 0x40b63df7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a63dfc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d63e01, 34 },
    { 0x2f763e02, 23 },
    { 0x8c663e02, 85 },
    { 0x20563e02, 90 },
    { 0xc3d63e05, 98 },
    { 0x32163e06, 82 },
    { 0xde263e07, 33 },
    { 0xd7963e07, 87 },
    { 0xf8063e08, 93 },
    { 0x64e63e0a, 61 },
    { 0, 0 },
    { 0x81363e0c, 61 },
    { 0x72663e0d, 88 },
    { 0xb0963e0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab063e15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a263e18, 95 },
    { 0x89a63e18, 90 },
    { 0x72363e1a, 98 },
    { 0x4fc63e1b, 83 },
    { 0x27e63e1b, 43 },
    { 0x3bb63e1a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c63e23, 100 },
    { 0x6dc63e23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba63e28, 68 },
    { 0x9b363e29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36263e2f, 47 },
    { 0, 0 },
    { 0x42263e31, 61 },
    { 0, 0 },
    { 0x863e33, 24 },
    { 0, 0 },
    { 0xb7563e35, 46 },
    { 0x74a63e35, 87 },
    { 0, 0 },
    { 0xcd863e38, 66 },
    { 0xbbd63e38, 46 },
    { 0x67e63e3a, 98 },
    { 0x2763e3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x93263e3e, 84 },
    { 0, 0 },
    { 0x4a563e40, 61 },
    { 0x58563e41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b63e47, 68 },
    { 0x9b163e48, 61 },
    { 0x9a663e49, 90 },
    { 0xea763e48, 90 },
    { 0xabc63e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef263e4f, 61 },
    { 0xa3d63e50, 69 },
    { 0x75563e51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa563e54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d63e57, 91 },
    { 0xd9a63e58, 33 },
    { 0x23863e58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97963e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe163e6a, 46 },
    { 0xac63e6a, 98 },
    { 0x1d763e6c, 33 },
    { 0x86563e6c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42563e75, 90 },
    { 0x43663e76, 91 },
    { 0, 0 },
    { 0xf7663e78, 61 },
    { 0, 0 },
    { 0x57663e7a, 76 },
    { 0x6c063e7b, 90 },
    { 0, 0 },
    { 0x50663e7d, 61 },
    { 0x7f663e7e, 33 },
    { 0x2f963e7d, 46 },
    { 0x6f863e7d, 76 },
    { 0x81e63e81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85063e85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf63e8b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f63e96, 95 },
    { 0x3b463e97, 90 },
    { 0, 0 },
    { 0x5b163e99, 91 },
    { 0x68663e9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b63ea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24363ea3, 34 },
    { 0x28b63ea4, 90 },
    { 0, 0 },
    { 0x24b63ea6, 61 },
    { 0, 0 },
    { 0xbfe63ea8, 61 },
    { 0, 0 },
    { 0xf5f63eaa, 38 },
    { 0xdeb63eab, 23 },
    { 0x4063eac, 23 },
    { 0x18263eac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17163eb4, 77 },
    { 0xdec63eb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61163eb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6263ec0, 90 },
    { 0x31c63ec0, 61 },
    { 0x30563ec1, 43 },
    { 0x70863ec2, 93 },
    { 0xc4d63ec1, 64 },
    { 0x54463ec5, 85 },
    { 0x5b063ec6, 93 },
    { 0xc0b63ec6, 98 },
    { 0xd0963ec3, 90 },
    { 0xe5063ec3, 80 },
    { 0x1e563eca, 55 },
    { 0x6af63ecb, 94 },
    { 0x99863eca, 95 },
    { 0x55563ecd, 88 },
    { 0xa6063eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4663ed1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95763ed4, 52 },
    { 0xda463ed4, 90 },
    { 0x35f63ed6, 83 },
    { 0x2763ed4, 76 },
    { 0, 0 },
    { 0x73d63ed9, 55 },
    { 0x5b763eda, 90 },
    { 0xad763edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1363ee0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a463ee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22163eeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e63eee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e63ef6, 34 },
    { 0xf4863ef7, 19 },
    { 0xfcf63ef6, 61 },
    { 0, 0 },
    { 0xe9d63efa, 90 },
    { 0x61b63efb, 33 },
    { 0xf663efa, 90 },
    { 0, 0 },
    { 0xb7d63efe, 68 },
    { 0x5863eff, 66 },
    { 0x53363eff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33163f04, 96 },
    { 0xd6f63f04, 93 },
    { 0x15a63f05, 76 },
    { 0, 0 },
    { 0xe2a63f08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x65363f0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c63f0e, 76 },
    { 0xb9b63f0e, 93 },
    { 0x9a563f10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x94763f13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba763f16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db63f1b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c63f1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92063f23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d63f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29863f2d, 98 },
    { 0, 0 },
    { 0xcc963f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9963f36, 33 },
    { 0, 0 },
    { 0xe9163f38, 90 },
    { 0xd8b63f38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7863f3f, 55 },
    { 0x20663f3f, 76 },
    { 0xda763f40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee063f44, 90 },
    { 0x56063f45, 67 },
    { 0x1c663f45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d063f4c, 43 },
    { 0x6a463f4d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd863f62, 98 },
    { 0xe3e63f62, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92663f68, 76 },
    { 0xa0463f68, 93 },
    { 0x81663f6a, 80 },
    { 0, 0 },
    { 0x10a63f6c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef63f81, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f963f85, 98 },
    { 0xb4163f86, 93 },
    { 0x46063f86, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70563f8b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8463f95, 91 },
    { 0, 0 },
    { 0x99d63f97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf463f9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc163f9f, 90 },
    { 0x22b63fa0, 90 },
    { 0x29163fa0, 100 },
    { 0, 0 },
    { 0x48c63fa3, 55 },
    { 0, 0 },
    { 0xfb463fa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17663fad, 33 },
    { 0x35663fad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63fbb, 64 },
    { 0, 0 },
    { 0x4ab63fbd, 95 },
    { 0x4763fbd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0563fc2, 98 },
    { 0x71163fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3563fc7, 90 },
    { 0x2ea63fc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca963fcf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc063fd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa763fe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4263fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb063fec, 95 },
    { 0, 0 },
    { 0xa8363fee, 90 },
    { 0, 0 },
    { 0x64563ff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf063ff5, 52 },
    { 0x3ce63ff6, 61 },
    { 0x6dd63ff7, 34 },
    { 0xb6363ff8, 90 },
    { 0x69863ff7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6663ffd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e64002, 90 },
    { 0, 0 },
    { 0x9df64004, 43 },
    { 0x48964005, 33 },
    { 0, 0 },
    { 0x8ae64007, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd6400a, 98 },
    { 0xceb6400a, 46 },
    { 0, 0 },
    { 0x36c6400d, 99 },
    { 0xf5c6400e, 95 },
    { 0x7aa6400f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xba64012, 90 },
    { 0, 0 },
    { 0xa4364014, 90 },
    { 0x4dd64015, 23 },
    { 0x9564016, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d46401d, 69 },
    { 0xe416401d, 91 },
    { 0xa426401f, 88 },
    { 0xe106401f, 95 },
    { 0xd5e6401d, 76 },
    { 0x58d6401f, 86 },
    { 0xafb64023, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df64028, 95 },
    { 0x1464029, 95 },
    { 0x4326402a, 91 },
    { 0xbdd6402a, 43 },
    { 0x5b26402b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9964030, 98 },
    { 0xd9c64031, 61 },
    { 0x9cc64032, 33 },
    { 0, 0 },
    { 0x92a64034, 46 },
    { 0x1af64034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b64038, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66064041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2164044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b64047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a6404a, 61 },
    { 0, 0 },
    { 0x6f46404c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd686404f, 38 },
    { 0xef264050, 55 },
    { 0x55e64050, 23 },
    { 0, 0 },
    { 0x16364053, 55 },
    { 0x3be64054, 84 },
    { 0x62664054, 95 },
    { 0, 0 },
    { 0xf9164057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb6405a, 17 },
    { 0x29a6405a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe6405e, 33 },
    { 0x12b6405f, 77 },
    { 0xbe364060, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91164066, 61 },
    { 0xbf964067, 33 },
    { 0x70e64067, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db6406e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x46864071, 33 },
    { 0x7c764071, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d64075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046408a, 33 },
    { 0x5b66408b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa596408e, 43 },
    { 0xfd46408e, 90 },
    { 0xaea64090, 76 },
    { 0xeaf64091, 95 },
    { 0x7464092, 91 },
    { 0xf2f64093, 90 },
    { 0, 0 },
    { 0x5f064095, 61 },
    { 0x58c64096, 55 },
    { 0xae064096, 90 },
    { 0, 0 },
    { 0xc0864099, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5816409d, 52 },
    { 0, 0 },
    { 0x3b26409f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576640a7, 61 },
    { 0x5b0640a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18640ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8640b2, 78 },
    { 0x12b640b2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64640b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c640b9, 33 },
    { 0x57b640b9, 98 },
    { 0, 0 },
    { 0x120640bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5640c1, 52 },
    { 0x4ae640c1, 90 },
    { 0xebc640c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d640c7, 24 },
    { 0x954640c8, 85 },
    { 0x39d640c7, 23 },
    { 0, 0 },
    { 0xc44640cb, 52 },
    { 0xc7b640cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x651640cf, 98 },
    { 0xdbe640cf, 17 },
    { 0xc96640cf, 43 },
    { 0, 0 },
    { 0xef5640d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838640d8, 85 },
    { 0x496640d9, 90 },
    { 0x975640da, 47 },
    { 0xdf0640db, 90 },
    { 0x2d8640dc, 52 },
    { 0x747640db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf640e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6640ea, 33 },
    { 0x865640ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d640f8, 55 },
    { 0x4d4640f8, 95 },
    { 0xf3a640f9, 33 },
    { 0xbc2640fb, 61 },
    { 0x52c640f9, 61 },
    { 0x6a8640fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b64102, 46 },
    { 0xaed64102, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c64107, 76 },
    { 0x12064108, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb626410c, 61 },
    { 0xbcb6410c, 33 },
    { 0x3876410d, 90 },
    { 0xb646410f, 95 },
    { 0xe0c64110, 85 },
    { 0xf6164111, 33 },
    { 0x6ab64112, 33 },
    { 0x99064113, 90 },
    { 0x1fb64113, 66 },
    { 0x51464110, 76 },
    { 0, 0 },
    { 0xf3864117, 100 },
    { 0x4b864118, 38 },
    { 0, 0 },
    { 0xf0c6411a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18664123, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9596412f, 61 },
    { 0, 0 },
    { 0xa1664131, 61 },
    { 0x7a864132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe716413a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af6413f, 90 },
    { 0x2cc64140, 55 },
    { 0, 0 },
    { 0xd5864142, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c164145, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc6414b, 68 },
    { 0, 0 },
    { 0xe686414d, 90 },
    { 0xefc6414e, 66 },
    { 0x8106414f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81564155, 85 },
    { 0xbb964155, 77 },
    { 0xb5e64157, 88 },
    { 0x1fb64155, 61 },
    { 0xc4a64159, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6486415d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be64160, 95 },
    { 0x89464161, 65 },
    { 0x42a64162, 67 },
    { 0xbf464163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60464166, 61 },
    { 0x66f64167, 64 },
    { 0, 0 },
    { 0x66464169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d16416c, 76 },
    { 0xff06416c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f64173, 80 },
    { 0, 0 },
    { 0x57464175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb16417b, 76 },
    { 0x8cb6417b, 99 },
    { 0xec76417b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4764180, 23 },
    { 0xf0464181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb964184, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e6418a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4896418d, 77 },
    { 0, 0 },
    { 0x4f76418f, 95 },
    { 0x2fc6418f, 82 },
    { 0xc5364191, 90 },
    { 0xc1564190, 61 },
    { 0, 0 },
    { 0xf2964194, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d464198, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e6419b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b6419f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49641a4, 90 },
    { 0, 0 },
    { 0xb641a6, 90 },
    { 0, 0 },
    { 0x185641a8, 61 },
    { 0x19641a8, 88 },
    { 0x33f641a8, 67 },
    { 0, 0 },
    { 0xbad641ac, 76 },
    { 0xf07641ad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50641b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8641ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x843641bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3641c3, 8 },
    { 0, 0 },
    { 0xa4e641c5, 90 },
    { 0xf11641c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5641c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6641cc, 23 },
    { 0xfba641cd, 89 },
    { 0x212641ce, 34 },
    { 0, 0 },
    { 0xf3e641d0, 69 },
    { 0xdc3641d0, 98 },
    { 0x180641d2, 67 },
    { 0xa4641d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7641d7, 90 },
    { 0x217641d8, 99 },
    { 0, 0 },
    { 0x752641da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb641e0, 100 },
    { 0x9e0641e1, 34 },
    { 0xf67641e2, 95 },
    { 0x434641e0, 23 },
    { 0xc6c641e2, 69 },
    { 0xc10641e2, 43 },
    { 0xeed641e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68641e9, 100 },
    { 0x7ff641ea, 85 },
    { 0x98a641ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e641f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41641f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9641fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb764200, 23 },
    { 0, 0 },
    { 0xdeb64202, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd64205, 55 },
    { 0, 0 },
    { 0x52064207, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d6420b, 33 },
    { 0, 0 },
    { 0xec86420d, 76 },
    { 0x6d06420d, 59 },
    { 0, 0 },
    { 0x65464210, 90 },
    { 0x31364211, 78 },
    { 0xcd64211, 90 },
    { 0x69964213, 93 },
    { 0x3a964214, 88 },
    { 0x60764215, 85 },
    { 0xf7864213, 23 },
    { 0x9e164217, 90 },
    { 0x79864215, 38 },
    { 0x2f964213, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68464220, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a364225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7036422a, 98 },
    { 0x5c66422b, 99 },
    { 0x3fc6422c, 68 },
    { 0x4886422d, 76 },
    { 0x2636422c, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75664234, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876423b, 33 },
    { 0x4cd6423c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc164243, 23 },
    { 0xb0a64244, 33 },
    { 0, 0 },
    { 0xf6564246, 66 },
    { 0x11564247, 90 },
    { 0xab464248, 98 },
    { 0xf2c64249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8046424c, 87 },
    { 0xaaf6424d, 90 },
    { 0x61d6424e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf564253, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73464256, 98 },
    { 0x8c264256, 84 },
    { 0x2c164256, 87 },
    { 0xd0e64256, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8856425c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85664260, 33 },
    { 0x69f64261, 23 },
    { 0xb8e64262, 78 },
    { 0x80464263, 90 },
    { 0xee64264, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e6426a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c66426d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f664270, 55 },
    { 0x1e64270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6156427a, 55 },
    { 0x9706427a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc64282, 65 },
    { 0x8c764283, 23 },
    { 0x44864284, 76 },
    { 0, 0 },
    { 0x56e64286, 81 },
    { 0x16f64286, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea6428a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c26428d, 34 },
    { 0, 0 },
    { 0x5a86428f, 76 },
    { 0x5dc6428f, 98 },
    { 0, 0 },
    { 0xd2664292, 46 },
    { 0, 0 },
    { 0xf1464294, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0964297, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b06429e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a642a6, 98 },
    { 0, 0 },
    { 0xa0a642a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3642af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9642b2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5642b6, 90 },
    { 0x72c642b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873642bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc642be, 95 },
    { 0x143642bf, 65 },
    { 0xa04642c0, 43 },
    { 0x801642c1, 61 },
    { 0x373642bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8642c7, 90 },
    { 0x3d7642c7, 91 },
    { 0x528642c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e642cd, 33 },
    { 0x3a7642cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x284642d1, 93 },
    { 0xf23642d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88642d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c642d8, 87 },
    { 0xfe4642d9, 87 },
    { 0x6f2642da, 90 },
    { 0xc90642d8, 85 },
    { 0x647642da, 33 },
    { 0xc3b642db, 55 },
    { 0x29a642dd, 61 },
    { 0x5e6642dc, 90 },
    { 0xf13642db, 66 },
    { 0x179642da, 17 },
    { 0, 0 },
    { 0x5cc642e3, 99 },
    { 0, 0 },
    { 0x1c4642e5, 90 },
    { 0x620642e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9642eb, 66 },
    { 0x4dd642ec, 98 },
    { 0xa41642ec, 95 },
    { 0xf0f642ee, 43 },
    { 0x3db642eb, 33 },
    { 0xba6642eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x648642f3, 95 },
    { 0x395642f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e642f9, 90 },
    { 0xa5642fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36642ff, 55 },
    { 0xdf264300, 23 },
    { 0xba564301, 33 },
    { 0xad0642ff, 76 },
    { 0, 0 },
    { 0x3f064304, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed66430a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7876430f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab64314, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd64318, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee26431b, 33 },
    { 0, 0 },
    { 0x25d6431d, 90 },
    { 0xf176431e, 34 },
    { 0x4fc6431f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56864323, 43 },
    { 0x4e164323, 61 },
    { 0, 0 },
    { 0xffb64326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd6432b, 95 },
    { 0, 0 },
    { 0x7d6432d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3764332, 34 },
    { 0xc6164333, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c6433b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e86433e, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70264343, 33 },
    { 0, 0 },
    { 0xa1e64345, 95 },
    { 0x78064346, 87 },
    { 0, 0 },
    { 0x7cd64348, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a86434c, 76 },
    { 0xe6e6434d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27664351, 76 },
    { 0xdee64352, 42 },
    { 0x4264351, 61 },
    { 0, 0 },
    { 0x4064355, 33 },
    { 0, 0 },
    { 0xc2e64357, 95 },
    { 0xbdd64357, 90 },
    { 0x6fb64358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b264362, 91 },
    { 0xb5264362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e664368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d6436f, 90 },
    { 0, 0 },
    { 0xaaa64371, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x40264374, 95 },
    { 0x7e164375, 90 },
    { 0, 0 },
    { 0xaea64377, 64 },
    { 0x1c864378, 84 },
    { 0x7a264378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f64382, 90 },
    { 0x9da64383, 98 },
    { 0x27a64384, 98 },
    { 0x95264382, 61 },
    { 0x91a64386, 90 },
    { 0x44464387, 91 },
    { 0x1a064387, 95 },
    { 0xd2964388, 37 },
    { 0x8ba64386, 76 },
    { 0x4e6438b, 83 },
    { 0x1dc6438c, 23 },
    { 0xd626438c, 23 },
    { 0, 0 },
    { 0x5006438f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf164393, 77 },
    { 0xa8664394, 98 },
    { 0x4e364393, 90 },
    { 0x8ca64395, 61 },
    { 0x69864394, 61 },
    { 0x40f64398, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2643a0, 98 },
    { 0x205643a1, 100 },
    { 0, 0 },
    { 0x8c8643a3, 61 },
    { 0x9643a4, 33 },
    { 0xe21643a4, 95 },
    { 0xecb643a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79643ad, 23 },
    { 0xb82643ae, 23 },
    { 0xcbe643ae, 34 },
    { 0xea7643b0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36643b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1643b8, 77 },
    { 0x795643b8, 23 },
    { 0, 0 },
    { 0x3b1643bb, 43 },
    { 0xc38643bb, 61 },
    { 0, 0 },
    { 0x667643be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1643c5, 95 },
    { 0x63643c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f643cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9643d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d643d6, 90 },
    { 0x1e8643d7, 43 },
    { 0xbac643d8, 23 },
    { 0xb45643d9, 76 },
    { 0xbf643da, 84 },
    { 0x256643d9, 55 },
    { 0xc5b643db, 90 },
    { 0x4e5643dd, 90 },
    { 0x6ea643de, 90 },
    { 0xbe7643df, 33 },
    { 0xfa2643df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793643e9, 34 },
    { 0, 0 },
    { 0x4d5643eb, 65 },
    { 0xed3643ec, 90 },
    { 0, 0 },
    { 0x6a5643ee, 91 },
    { 0x951643ef, 33 },
    { 0, 0 },
    { 0x2bb643f1, 33 },
    { 0, 0 },
    { 0xe31643f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a643f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28643ff, 90 },
    { 0, 0 },
    { 0x3e464401, 23 },
    { 0x80c64402, 33 },
    { 0, 0 },
    { 0x32764404, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea64407, 98 },
    { 0x19464408, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe64411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91064414, 17 },
    { 0xc5b64415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9964418, 66 },
    { 0x4d564419, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e6441d, 87 },
    { 0, 0 },
    { 0x8e56441f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5764422, 33 },
    { 0xbbe64422, 61 },
    { 0xbe864424, 98 },
    { 0x19164425, 61 },
    { 0xb0c64425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc64430, 87 },
    { 0x41764431, 90 },
    { 0, 0 },
    { 0xafd64433, 55 },
    { 0xf4a64434, 23 },
    { 0, 0 },
    { 0xdeb64436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71764439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f6443f, 55 },
    { 0x39264440, 76 },
    { 0x42164441, 55 },
    { 0xe2864441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc64445, 34 },
    { 0xfeb64445, 61 },
    { 0, 0 },
    { 0xde064448, 95 },
    { 0x53f64449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9764451, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf964457, 61 },
    { 0x49764457, 90 },
    { 0x50364459, 61 },
    { 0, 0 },
    { 0x5c66445b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c64460, 17 },
    { 0, 0 },
    { 0x3b564462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b364465, 93 },
    { 0x85264466, 84 },
    { 0x83a64467, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c6446d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c64470, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x42064473, 61 },
    { 0, 0 },
    { 0x8fb64475, 33 },
    { 0, 0 },
    { 0x70464477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f36447a, 34 },
    { 0, 0 },
    { 0xc46447c, 75 },
    { 0, 0 },
    { 0x6906447e, 61 },
    { 0xe326447f, 66 },
    { 0x24f64480, 23 },
    { 0xa3e6447f, 55 },
    { 0, 0 },
    { 0x90f64483, 55 },
    { 0xc6c64484, 90 },
    { 0x62b64484, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd226448a, 53 },
    { 0, 0 },
    { 0xa836448c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c64490, 34 },
    { 0x87264491, 95 },
    { 0x8b864492, 75 },
    { 0x44a64492, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c64498, 76 },
    { 0x52a64499, 98 },
    { 0x6666449a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb66449f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f644a2, 34 },
    { 0xb47644a3, 61 },
    { 0x91a644a2, 90 },
    { 0x593644a5, 61 },
    { 0, 0 },
    { 0xada644a7, 76 },
    { 0, 0 },
    { 0x4df644a9, 90 },
    { 0, 0 },
    { 0x851644ab, 90 },
    { 0, 0 },
    { 0x121644ad, 98 },
    { 0x370644ae, 51 },
    { 0x836644af, 76 },
    { 0, 0 },
    { 0x982644b1, 76 },
    { 0xfd0644b1, 61 },
    { 0x582644b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a644b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90644c1, 61 },
    { 0x3dc644c2, 95 },
    { 0xf40644c2, 90 },
    { 0xc1d644c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30644c7, 61 },
    { 0, 0 },
    { 0x120644c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412644ce, 95 },
    { 0, 0 },
    { 0xe2644d0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc644d5, 84 },
    { 0xaa7644d5, 76 },
    { 0x65d644d5, 34 },
    { 0xaaf644d8, 90 },
    { 0x35d644d7, 75 },
    { 0xce1644da, 90 },
    { 0, 0 },
    { 0x489644dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa644e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b644ea, 80 },
    { 0xfd5644eb, 76 },
    { 0xec9644ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23644f1, 23 },
    { 0, 0 },
    { 0x414644f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f644f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946644fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e64500, 98 },
    { 0xfb564501, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95964506, 34 },
    { 0, 0 },
    { 0xae264508, 17 },
    { 0x83764508, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9026450d, 55 },
    { 0xf5a6450e, 66 },
    { 0xfa26450e, 33 },
    { 0xca164510, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a64515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7564518, 76 },
    { 0, 0 },
    { 0xb2b6451a, 99 },
    { 0xa416451a, 98 },
    { 0x6f46451b, 99 },
    { 0, 0 },
    { 0xede6451e, 84 },
    { 0, 0 },
    { 0x49164520, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb464525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6452b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6766452f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa64533, 33 },
    { 0x60c64534, 61 },
    { 0x10664535, 90 },
    { 0x7f064535, 98 },
    { 0, 0 },
    { 0xa2064538, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef6453d, 43 },
    { 0, 0 },
    { 0xb866453f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xac464542, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6454a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3596454d, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0xab564550, 76 },
    { 0x1b064551, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d64554, 23 },
    { 0xc3764555, 17 },
    { 0x7ee64554, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc6455b, 66 },
    { 0xffd6455c, 95 },
    { 0x586455d, 90 },
    { 0x5136455b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa564565, 52 },
    { 0, 0 },
    { 0xd1464567, 90 },
    { 0xba264568, 66 },
    { 0x3e364569, 85 },
    { 0x91d6456a, 33 },
    { 0x7ef64567, 67 },
    { 0xb7c6456c, 85 },
    { 0x8606456c, 34 },
    { 0x6366456e, 33 },
    { 0, 0 },
    { 0xa0364570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b264574, 95 },
    { 0xa7f64575, 61 },
    { 0x8eb64574, 75 },
    { 0xd6064577, 66 },
    { 0, 0 },
    { 0x91e64579, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b6457c, 85 },
    { 0xea86457c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58464580, 100 },
    { 0x49d64581, 33 },
    { 0x5dd64582, 88 },
    { 0x23c64583, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3564588, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf64591, 33 },
    { 0xfec64592, 33 },
    { 0x1e464593, 90 },
    { 0x66a64593, 61 },
    { 0, 0 },
    { 0x3d964596, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c264599, 66 },
    { 0, 0 },
    { 0x4fb6459b, 55 },
    { 0xc926459b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb766459f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3645a8, 61 },
    { 0, 0 },
    { 0x3e5645aa, 81 },
    { 0, 0 },
    { 0xbe3645ac, 23 },
    { 0, 0 },
    { 0x2b3645ae, 76 },
    { 0xe63645af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d645b2, 55 },
    { 0, 0 },
    { 0xbf8645b4, 36 },
    { 0x8f7645b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc645b9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4645bc, 55 },
    { 0xf52645bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b645c0, 33 },
    { 0xfd3645c1, 90 },
    { 0x37f645c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73645cb, 80 },
    { 0, 0 },
    { 0x851645cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2645d8, 61 },
    { 0x4a0645d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2645dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a645e0, 23 },
    { 0x509645e1, 61 },
    { 0x1e4645e2, 80 },
    { 0xcfd645e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea645ec, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4645ef, 87 },
    { 0x894645f0, 34 },
    { 0x1dc645f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144645f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d645fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d64600, 33 },
    { 0xd5a64601, 46 },
    { 0x30364601, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6764607, 43 },
    { 0, 0 },
    { 0x73f64609, 95 },
    { 0, 0 },
    { 0x83a6460b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f64610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64064613, 78 },
    { 0, 0 },
    { 0x30a64615, 23 },
    { 0, 0 },
    { 0xd7e64617, 34 },
    { 0x84064618, 34 },
    { 0xdab64619, 98 },
    { 0xae664619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b26461d, 95 },
    { 0xe2b6461d, 61 },
    { 0x5756461f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4726462c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac36462f, 33 },
    { 0, 0 },
    { 0x92764631, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x78164634, 33 },
    { 0x67364635, 80 },
    { 0x64664636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a64639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd976463e, 90 },
    { 0x1976463f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a64643, 90 },
    { 0x68864643, 23 },
    { 0, 0 },
    { 0x5fd64646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93864649, 98 },
    { 0x3b46464a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276464e, 90 },
    { 0, 0 },
    { 0x72864650, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6164653, 90 },
    { 0xb9064654, 46 },
    { 0x37264655, 91 },
    { 0xfdb64656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a64659, 90 },
    { 0x8666465a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e6465d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b64660, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3464663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b64667, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9536466d, 100 },
    { 0x8206466e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc064674, 33 },
    { 0, 0 },
    { 0xf9964676, 23 },
    { 0x5664677, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb6467a, 88 },
    { 0x7f46467a, 98 },
    { 0x4fe6467b, 33 },
    { 0x7bd6467a, 67 },
    { 0x68b6467b, 34 },
    { 0x4d46467f, 81 },
    { 0xa5f64680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e64683, 61 },
    { 0x11464684, 66 },
    { 0x78364685, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c6468b, 100 },
    { 0xb836468c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f56468f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f64693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3396469d, 61 },
    { 0, 0 },
    { 0x46d6469f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8646a5, 55 },
    { 0xb9a646a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0646af, 33 },
    { 0x142646af, 95 },
    { 0xd03646b1, 43 },
    { 0xaff646b2, 87 },
    { 0x842646b3, 95 },
    { 0x10b646b0, 83 },
    { 0, 0 },
    { 0x531646b6, 32 },
    { 0x5ad646b6, 68 },
    { 0xf9a646b8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12646be, 66 },
    { 0, 0 },
    { 0x2fc646c0, 88 },
    { 0x35d646c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a646c8, 78 },
    { 0xaac646c8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f646cc, 55 },
    { 0x628646cd, 46 },
    { 0xaf1646cd, 67 },
    { 0xcd3646cf, 66 },
    { 0x5f9646cf, 76 },
    { 0xadd646ce, 90 },
    { 0, 0 },
    { 0x887646d3, 61 },
    { 0xeae646d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53646dd, 76 },
    { 0xb01646de, 90 },
    { 0x977646df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x315646e2, 98 },
    { 0xa16646e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01646ed, 76 },
    { 0x13646ed, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df646f3, 52 },
    { 0x114646f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31646f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161646fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f964701, 23 },
    { 0xdcc64701, 23 },
    { 0x4bf64703, 33 },
    { 0x7ad64704, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db64708, 23 },
    { 0xba764708, 78 },
    { 0xe3d64708, 33 },
    { 0x28e6470b, 23 },
    { 0x35864709, 83 },
    { 0, 0 },
    { 0xc886470e, 97 },
    { 0x7076470e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a364717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4576471b, 61 },
    { 0x12e6471c, 99 },
    { 0xeb06471c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd64724, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f264728, 87 },
    { 0x70064729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c6472e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98664732, 93 },
    { 0x19e64733, 98 },
    { 0, 0 },
    { 0x35664735, 95 },
    { 0x2eb64736, 88 },
    { 0xfce64736, 69 },
    { 0x21864737, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9916473d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72064740, 76 },
    { 0x6bf64741, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed64746, 99 },
    { 0x98964746, 99 },
    { 0x6c064748, 34 },
    { 0xfea64748, 8 },
    { 0x8f26474a, 46 },
    { 0x2166474a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab64750, 61 },
    { 0, 0 },
    { 0xcbf64752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6475e, 43 },
    { 0xe896475e, 90 },
    { 0x9376475e, 90 },
    { 0, 0 },
    { 0xa0464762, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29864765, 61 },
    { 0, 0 },
    { 0x45064767, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf326476a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6476d, 76 },
    { 0x64e6476e, 95 },
    { 0, 0 },
    { 0x54564770, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46764773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88064779, 46 },
    { 0x7b86477a, 80 },
    { 0xe8a6477b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b64783, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d64787, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b6478b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c06478e, 61 },
    { 0xa356478f, 100 },
    { 0xff64790, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1164795, 61 },
    { 0, 0 },
    { 0x4ac64797, 76 },
    { 0, 0 },
    { 0xdea64799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2647a1, 90 },
    { 0x9ff647a2, 34 },
    { 0x19d647a3, 23 },
    { 0x87647a4, 61 },
    { 0x66647a3, 61 },
    { 0xc2a647a6, 61 },
    { 0x26a647a7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12647aa, 52 },
    { 0, 0 },
    { 0xb0a647ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6647b0, 61 },
    { 0x940647b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e647be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66647c3, 52 },
    { 0xc29647c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0647ca, 95 },
    { 0x4bc647cb, 77 },
    { 0x427647cc, 82 },
    { 0xd0d647cd, 87 },
    { 0x615647cb, 23 },
    { 0, 0 },
    { 0x9df647d0, 34 },
    { 0x59b647d0, 81 },
    { 0xce2647d1, 33 },
    { 0xb17647d3, 61 },
    { 0xd59647d3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6647de, 61 },
    { 0x374647de, 99 },
    { 0, 0 },
    { 0x1f8647e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db647e7, 95 },
    { 0x61e647e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe647eb, 23 },
    { 0xec9647eb, 33 },
    { 0x259647eb, 95 },
    { 0xcaf647ee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16647f6, 23 },
    { 0x94b647f7, 85 },
    { 0xe01647f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x287647fb, 99 },
    { 0xc7b647fc, 55 },
    { 0x275647fd, 43 },
    { 0x7a5647fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48664804, 98 },
    { 0, 0 },
    { 0x43464806, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d64809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c6480c, 98 },
    { 0x91f6480d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b64810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea964813, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68464819, 87 },
    { 0x5ea6481a, 42 },
    { 0x7c66481a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6481e, 23 },
    { 0xc5d6481e, 90 },
    { 0x43d6481e, 99 },
    { 0, 0 },
    { 0xd4a64822, 61 },
    { 0xd5a64823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2564826, 95 },
    { 0x49d64826, 55 },
    { 0, 0 },
    { 0x64164829, 69 },
    { 0x6df6482a, 61 },
    { 0, 0 },
    { 0xe796482c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14964831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76483a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e6483f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d764842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a364847, 88 },
    { 0x6ec64847, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3236484c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13164850, 90 },
    { 0xd5f64851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d164856, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f964861, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a64866, 55 },
    { 0x82264866, 34 },
    { 0x71164868, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x596486b, 93 },
    { 0xe56486b, 76 },
    { 0, 0 },
    { 0xbb6486e, 98 },
    { 0xf936486f, 2 },
    { 0, 0 },
    { 0x1e264871, 23 },
    { 0, 0 },
    { 0xecf64873, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f64879, 90 },
    { 0, 0 },
    { 0xb9a6487b, 43 },
    { 0xe2c6487c, 33 },
    { 0, 0 },
    { 0x5606487e, 100 },
    { 0, 0 },
    { 0x9fb64880, 65 },
    { 0xb8e64881, 40 },
    { 0, 0 },
    { 0x26b64883, 61 },
    { 0xe64884, 61 },
    { 0x67c64885, 90 },
    { 0x47564886, 36 },
    { 0x6d64886, 43 },
    { 0xc4464884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b64893, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55464898, 95 },
    { 0, 0 },
    { 0x7576489a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87648a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20648a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa648a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73648ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf648b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x954648b5, 64 },
    { 0xa02648b6, 98 },
    { 0x51a648b7, 99 },
    { 0x17e648b8, 61 },
    { 0xd7b648b9, 96 },
    { 0x135648ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44648c2, 33 },
    { 0x7f0648c3, 44 },
    { 0x208648c2, 95 },
    { 0x8d7648c4, 33 },
    { 0xfe0648c2, 46 },
    { 0x67c648c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209648cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e648d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27648d6, 87 },
    { 0x843648d7, 95 },
    { 0x401648d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d648de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339648e2, 23 },
    { 0x18648e3, 61 },
    { 0, 0 },
    { 0xc99648e5, 46 },
    { 0xcdc648e6, 81 },
    { 0x99e648e6, 34 },
    { 0, 0 },
    { 0x65a648e9, 90 },
    { 0x6cb648ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e648f2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0648fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22648fd, 98 },
    { 0x320648fd, 34 },
    { 0x882648ff, 61 },
    { 0x100648ff, 90 },
    { 0xf3e64901, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c364910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1464915, 95 },
    { 0x49564916, 99 },
    { 0, 0 },
    { 0xdb964918, 90 },
    { 0, 0 },
    { 0xe366491a, 52 },
    { 0xdf66491a, 23 },
    { 0xb846491c, 90 },
    { 0xbb36491d, 33 },
    { 0, 0 },
    { 0x9dd6491f, 55 },
    { 0x28664920, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e264924, 97 },
    { 0xe4c64925, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166492a, 93 },
    { 0xdb66492b, 33 },
    { 0x1ec6492a, 43 },
    { 0xdb06492b, 95 },
    { 0x5e56492e, 33 },
    { 0x15a6492f, 43 },
    { 0xb186492f, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c864934, 61 },
    { 0x7b264935, 99 },
    { 0x8dd64936, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e6493b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da6493e, 90 },
    { 0, 0 },
    { 0xc8364940, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd64944, 33 },
    { 0, 0 },
    { 0xf0f64946, 61 },
    { 0x2664946, 33 },
    { 0x3cb64948, 23 },
    { 0x33e64949, 61 },
    { 0xad964948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe064950, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa964955, 95 },
    { 0x55c64956, 9 },
    { 0, 0 },
    { 0x46764958, 24 },
    { 0x22f64959, 33 },
    { 0x9326495a, 61 },
    { 0, 0 },
    { 0x8276495c, 87 },
    { 0x3be6495d, 98 },
    { 0, 0 },
    { 0x5106495f, 61 },
    { 0x8326495f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e6496d, 98 },
    { 0x31c6496e, 33 },
    { 0x46d6496f, 33 },
    { 0x3e464970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a064973, 23 },
    { 0x12264973, 90 },
    { 0xc1664973, 23 },
    { 0x3a764976, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf46497b, 61 },
    { 0xef6497c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d66497f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a64985, 95 },
    { 0xa4b64985, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a56498c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f64993, 55 },
    { 0xea164994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6964997, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a6499c, 33 },
    { 0x3d56499c, 23 },
    { 0x5226499c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7649a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85649ac, 85 },
    { 0, 0 },
    { 0xe83649ae, 88 },
    { 0xff9649ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x612649b2, 55 },
    { 0x726649b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381649bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569649c0, 65 },
    { 0, 0 },
    { 0xb8b649c2, 61 },
    { 0, 0 },
    { 0x5a8649c4, 76 },
    { 0x8cd649c4, 17 },
    { 0x39c649c6, 98 },
    { 0, 0 },
    { 0x222649c8, 93 },
    { 0xdb2649c8, 55 },
    { 0x29649ca, 33 },
    { 0x755649cb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae649d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6649d4, 66 },
    { 0x78649d5, 84 },
    { 0x816649d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e649dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada649e1, 46 },
    { 0, 0 },
    { 0xde0649e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d649e8, 75 },
    { 0, 0 },
    { 0x238649ea, 75 },
    { 0xb0a649eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739649f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e649f5, 66 },
    { 0x96a649f6, 90 },
    { 0xc4a649f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196649fc, 65 },
    { 0x728649fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d64a00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a464a05, 66 },
    { 0xdf64a06, 61 },
    { 0xb2c64a06, 91 },
    { 0xdc464a06, 46 },
    { 0xd8364a09, 34 },
    { 0xa4964a0a, 61 },
    { 0xce364a0a, 23 },
    { 0x37d64a0c, 66 },
    { 0x5d264a0c, 90 },
    { 0, 0 },
    { 0xe8064a0f, 83 },
    { 0x80064a0f, 43 },
    { 0x32364a11, 98 },
    { 0x4fd64a11, 90 },
    { 0xca264a11, 61 },
    { 0x78b64a14, 90 },
    { 0xc5764a13, 76 },
    { 0x69364a16, 85 },
    { 0xc8264a17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c664a1b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34764a20, 34 },
    { 0xfb464a21, 95 },
    { 0, 0 },
    { 0x6d064a23, 46 },
    { 0, 0 },
    { 0xba64a25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0664a29, 43 },
    { 0x764a29, 67 },
    { 0xc1a64a2b, 99 },
    { 0, 0 },
    { 0x8c364a2d, 90 },
    { 0, 0 },
    { 0x14964a2f, 59 },
    { 0x3a64a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d964a34, 93 },
    { 0x4b164a35, 61 },
    { 0x1e364a36, 61 },
    { 0x24864a36, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a964a3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81064a41, 98 },
    { 0x74864a41, 95 },
    { 0xe5f64a43, 66 },
    { 0x31564a44, 23 },
    { 0xde364a43, 43 },
    { 0x86664a43, 84 },
    { 0x9a164a43, 59 },
    { 0xe164a48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1064a4d, 99 },
    { 0x69564a4e, 90 },
    { 0xaf464a4f, 99 },
    { 0xa5064a50, 95 },
    { 0x12e64a51, 88 },
    { 0x33a64a4f, 33 },
    { 0x1064a53, 93 },
    { 0x7ce64a50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e364a5e, 90 },
    { 0x7ea64a5f, 98 },
    { 0, 0 },
    { 0x9e64a61, 65 },
    { 0xd0c64a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23264a72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x45064a75, 98 },
    { 0x50464a75, 61 },
    { 0, 0 },
    { 0x4ec64a78, 90 },
    { 0xd7864a78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a64a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d64a82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a64a85, 23 },
    { 0xdf064a86, 98 },
    { 0xbaa64a85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6064a90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb964a94, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0364a9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42364a9f, 33 },
    { 0, 0 },
    { 0xd5264aa1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e64aa5, 33 },
    { 0, 0 },
    { 0x7f664aa7, 61 },
    { 0, 0 },
    { 0xa8f64aa9, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2664aae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8464ab3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6364ab6, 81 },
    { 0xd6a64ab7, 77 },
    { 0xe3164ab8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d64abb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59264ac4, 90 },
    { 0x2eb64ac5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78564ac9, 70 },
    { 0, 0 },
    { 0x64acb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db64ad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d064ad4, 23 },
    { 0xb1a64ad5, 90 },
    { 0x2bd64ad4, 91 },
    { 0x96764ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8264ada, 61 },
    { 0x57964ada, 98 },
    { 0x68664adc, 33 },
    { 0x51464add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e64ae0, 98 },
    { 0x8b864ae1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d64ae4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1764aed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7864af0, 90 },
    { 0x8a664af1, 90 },
    { 0xabd64af2, 67 },
    { 0, 0 },
    { 0x7564af4, 34 },
    { 0xb2d64af5, 43 },
    { 0x68164af6, 90 },
    { 0x44164af6, 78 },
    { 0x5a164af8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c64afb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62464afe, 100 },
    { 0xc8c64aff, 43 },
    { 0x79864b00, 33 },
    { 0, 0 },
    { 0xd6e64b02, 80 },
    { 0, 0 },
    { 0xdc64b04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d64b08, 90 },
    { 0xcb064b08, 91 },
    { 0xdfd64b08, 85 },
    { 0x5b564b0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7964b0f, 76 },
    { 0xea664b10, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47964b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e64b19, 34 },
    { 0xc7264b1a, 98 },
    { 0x9d64b1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7564b20, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd64b25, 61 },
    { 0, 0 },
    { 0x40864b27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b64b2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35764b2e, 98 },
    { 0xf1764b2f, 61 },
    { 0x1de64b2f, 33 },
    { 0x5a064b31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16564b35, 98 },
    { 0xa2364b35, 34 },
    { 0, 0 },
    { 0x91664b38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5564b3e, 46 },
    { 0x3d764b3f, 33 },
    { 0xd564b40, 55 },
    { 0xe3a64b3f, 98 },
    { 0xc2064b42, 76 },
    { 0xde264b3e, 69 },
    { 0, 0 },
    { 0x6d364b45, 43 },
    { 0, 0 },
    { 0x4ca64b47, 95 },
    { 0, 0 },
    { 0x14064b49, 76 },
    { 0x94864b49, 61 },
    { 0xb8364b4b, 98 },
    { 0xaad64b4a, 33 },
    { 0, 0 },
    { 0xbf464b4e, 52 },
    { 0x42764b4e, 80 },
    { 0, 0 },
    { 0xf7064b51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5064b57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6064b5a, 65 },
    { 0x16864b5b, 100 },
    { 0xaff64b5a, 61 },
    { 0x6c264b5c, 23 },
    { 0x71664b5e, 43 },
    { 0x1c764b5f, 33 },
    { 0, 0 },
    { 0xf2c64b61, 95 },
    { 0xfc864b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93764b67, 36 },
    { 0, 0 },
    { 0xe7764b69, 85 },
    { 0, 0 },
    { 0x32664b6b, 90 },
    { 0, 0 },
    { 0xf6364b6d, 92 },
    { 0x85c64b6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc964b79, 52 },
    { 0x86c64b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7064b7f, 33 },
    { 0x80a64b7f, 95 },
    { 0xb1b64b81, 99 },
    { 0x43f64b82, 61 },
    { 0x5a364b83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f64b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad964b8d, 90 },
    { 0xd1164b8e, 76 },
    { 0xd8b64b8e, 81 },
    { 0x95864b90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3464b94, 90 },
    { 0x38964b95, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60264b9a, 98 },
    { 0x36b64b9b, 61 },
    { 0x43964b9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba864b9f, 33 },
    { 0x4d964ba0, 23 },
    { 0xe0e64ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70464ba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef564baa, 33 },
    { 0xc8264baa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc64bae, 81 },
    { 0x2c064baf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2064bb4, 90 },
    { 0x37f64bb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e364bb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35864bbd, 61 },
    { 0x3f664bbe, 33 },
    { 0x3ae64bbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d64bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8264bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19964bce, 43 },
    { 0xc9364bce, 33 },
    { 0xda564bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd64bd5, 90 },
    { 0x86f64bd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38364bdb, 36 },
    { 0, 0 },
    { 0x11264bdd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4264bed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c64bf4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e364bfb, 76 },
    { 0, 0 },
    { 0x39964bfd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf464c00, 95 },
    { 0, 0 },
    { 0xd9664c02, 95 },
    { 0x4bc64c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd664c06, 95 },
    { 0xefd64c07, 67 },
    { 0xf0a64c08, 93 },
    { 0xed864c09, 43 },
    { 0x87e64c0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e64c14, 90 },
    { 0, 0 },
    { 0x17964c16, 33 },
    { 0xce064c17, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c64c1a, 61 },
    { 0, 0 },
    { 0x5a264c1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b64c20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b64c25, 98 },
    { 0xbb664c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86164c29, 33 },
    { 0, 0 },
    { 0x7764c2b, 61 },
    { 0x2c664c2c, 85 },
    { 0x28d64c2c, 98 },
    { 0, 0 },
    { 0xc9564c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe64c34, 33 },
    { 0xa2964c35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1364c3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61564c41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e64c44, 52 },
    { 0x56064c45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c064c4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22864c4e, 23 },
    { 0, 0 },
    { 0x4cb64c50, 33 },
    { 0xa9964c50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e364c55, 95 },
    { 0xdaf64c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9764c59, 93 },
    { 0x1b264c59, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e764c5e, 33 },
    { 0xa8464c5e, 98 },
    { 0, 0 },
    { 0xae664c61, 66 },
    { 0x57764c62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63564c68, 61 },
    { 0x99b64c69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a64c70, 92 },
    { 0xb6b64c70, 34 },
    { 0x2fd64c72, 33 },
    { 0xae964c71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd64c7a, 23 },
    { 0x88c64c7a, 98 },
    { 0, 0 },
    { 0xeed64c7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8564c81, 65 },
    { 0, 0 },
    { 0x8a364c83, 87 },
    { 0, 0 },
    { 0x4ca64c85, 33 },
    { 0x5a864c86, 66 },
    { 0x32e64c87, 90 },
    { 0, 0 },
    { 0x9ab64c89, 55 },
    { 0x32364c89, 90 },
    { 0x87d64c8b, 90 },
    { 0x16e64c8c, 61 },
    { 0xee864c8d, 33 },
    { 0xeca64c8e, 93 },
    { 0xd1664c8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76064c92, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda64c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62664ca0, 95 },
    { 0x3e464ca1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6164ca4, 66 },
    { 0xa4764ca5, 33 },
    { 0xec764ca5, 23 },
    { 0xc5164ca6, 61 },
    { 0x73664ca4, 90 },
    { 0xbb964ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20664cb0, 95 },
    { 0x1764cb1, 98 },
    { 0x73c64cb2, 95 },
    { 0x2b964cb1, 34 },
    { 0x96e64cb0, 61 },
    { 0xb4564cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67464cb9, 87 },
    { 0, 0 },
    { 0x97264cbb, 61 },
    { 0x77f64cbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb364cbf, 46 },
    { 0xf864cc0, 66 },
    { 0xe0e64cc1, 40 },
    { 0xab664cc2, 90 },
    { 0x32764cc3, 91 },
    { 0x7ba64cc4, 78 },
    { 0, 0 },
    { 0xa2f64cc6, 43 },
    { 0, 0 },
    { 0xb9464cc8, 33 },
    { 0xbaf64cc9, 33 },
    { 0xbfd64cca, 90 },
    { 0xd5f64ccb, 33 },
    { 0x75664ccc, 58 },
    { 0x1c764ccd, 90 },
    { 0, 0 },
    { 0x61964ccf, 61 },
    { 0x99e64cd0, 90 },
    { 0x54264cd1, 80 },
    { 0x10364cd2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28064cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc64cdf, 43 },
    { 0, 0 },
    { 0x1d64ce1, 61 },
    { 0x24064ce1, 93 },
    { 0x98c64ce1, 55 },
    { 0x72b64ce4, 76 },
    { 0, 0 },
    { 0x1f764ce6, 46 },
    { 0, 0 },
    { 0x43b64ce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d664ceb, 95 },
    { 0x8f264ceb, 93 },
    { 0xc2464ceb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d64cf1, 23 },
    { 0x61d64cf2, 43 },
    { 0x74c64cf1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb64cf7, 95 },
    { 0xe0f64cf8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x76164cfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4264cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8864d02, 52 },
    { 0xe164d03, 98 },
    { 0, 0 },
    { 0xeae64d05, 52 },
    { 0xb0864d05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8764d0b, 23 },
    { 0xa1464d0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda964d0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d764d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f64d18, 97 },
    { 0, 0 },
    { 0x53664d1a, 43 },
    { 0, 0 },
    { 0x4d464d1c, 33 },
    { 0xa9a64d1d, 61 },
    { 0x7cb64d1e, 33 },
    { 0, 0 },
    { 0x42564d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8764d24, 33 },
    { 0, 0 },
    { 0xf3764d26, 99 },
    { 0, 0 },
    { 0x6ed64d28, 61 },
    { 0xcd164d29, 61 },
    { 0x4ac64d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62964d32, 67 },
    { 0x6fa64d32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da64d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9864d3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2664d40, 98 },
    { 0x50664d40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de64d44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef64d47, 90 },
    { 0, 0 },
    { 0x3c064d49, 87 },
    { 0, 0 },
    { 0x6bc64d4b, 76 },
    { 0xc0464d4c, 40 },
    { 0x75064d4d, 23 },
    { 0x59e64d4e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db64d51, 78 },
    { 0x8b164d52, 61 },
    { 0xfca64d53, 95 },
    { 0xeb764d54, 56 },
    { 0x5b064d54, 98 },
    { 0x62364d53, 42 },
    { 0xba364d51, 33 },
    { 0x23164d57, 23 },
    { 0x36b64d58, 76 },
    { 0xa264d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd64d63, 77 },
    { 0xe7264d63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7564d67, 61 },
    { 0x12064d68, 100 },
    { 0x4e064d69, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xae564d6c, 23 },
    { 0xd5964d6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb64d71, 90 },
    { 0, 0 },
    { 0x50d64d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7064d7b, 90 },
    { 0, 0 },
    { 0xe7f64d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32364d84, 84 },
    { 0, 0 },
    { 0xb9b64d86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d64d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9664d95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7464d9b, 23 },
    { 0x1a964d9b, 85 },
    { 0x9dd64d9d, 88 },
    { 0, 0 },
    { 0x37364d9f, 90 },
    { 0xd8a64da0, 23 },
    { 0x4b964d9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70664da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38464dac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5264db3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be64dbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a64dc4, 61 },
    { 0x54664dc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68664dce, 98 },
    { 0x99064dcf, 61 },
    { 0, 0 },
    { 0xf0f64dd1, 91 },
    { 0xc1364dd1, 97 },
    { 0x25b64dd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32964dd6, 43 },
    { 0x75264dd7, 23 },
    { 0x88664dd7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a64de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc64de5, 78 },
    { 0x1a864de6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27864de9, 33 },
    { 0, 0 },
    { 0x8c364deb, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xda864dee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5064df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4464dfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7664dfd, 48 },
    { 0x51b64dfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa64e03, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8564e08, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f64e0d, 81 },
    { 0xa7d64e0d, 23 },
    { 0x87064e0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c64e12, 33 },
    { 0xcd964e13, 46 },
    { 0x69f64e14, 68 },
    { 0xb9864e14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d564e18, 55 },
    { 0x54a64e19, 23 },
    { 0xd3f64e18, 61 },
    { 0xfec64e18, 61 },
    { 0x77d64e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29964e21, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf64e28, 81 },
    { 0xde564e29, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59664e32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d364e37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a464e3a, 61 },
    { 0xa5464e3b, 76 },
    { 0xffc64e3c, 85 },
    { 0xd5d64e3d, 93 },
    { 0x4e164e3c, 90 },
    { 0, 0 },
    { 0x7da64e40, 61 },
    { 0xb5564e41, 43 },
    { 0x3cb64e41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f764e46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b64e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb64e59, 33 },
    { 0, 0 },
    { 0xaef64e5b, 65 },
    { 0, 0 },
    { 0xb3d64e5d, 23 },
    { 0x21d64e5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77164e61, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65464e65, 43 },
    { 0, 0 },
    { 0x65064e67, 40 },
    { 0, 0 },
    { 0x13464e69, 90 },
    { 0xb5c64e69, 55 },
    { 0xbbb64e6b, 76 },
    { 0, 0 },
    { 0xe2a64e6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c164e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50064e76, 75 },
    { 0xa6e64e76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9064e7a, 33 },
    { 0xda764e7b, 95 },
    { 0xae864e7b, 90 },
    { 0x8a064e7d, 98 },
    { 0x6ec64e7e, 81 },
    { 0x76e64e7f, 76 },
    { 0x6ee64e7f, 66 },
    { 0x2c364e7e, 61 },
    { 0x17264e82, 55 },
    { 0x4bb64e83, 93 },
    { 0xf8364e82, 98 },
    { 0, 0 },
    { 0xaba64e86, 66 },
    { 0, 0 },
    { 0x7a564e88, 98 },
    { 0, 0 },
    { 0xa1f64e8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc64e97, 61 },
    { 0, 0 },
    { 0x73264e99, 90 },
    { 0x15164e9a, 61 },
    { 0x1864e9a, 33 },
    { 0xf5464e99, 61 },
    { 0x5bb64e9a, 90 },
    { 0xe9964e9e, 95 },
    { 0xc0664e9b, 95 },
    { 0x1d64ea0, 43 },
    { 0, 0 },
    { 0x42c64ea2, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66864eaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23864ead, 85 },
    { 0xc3d64eae, 61 },
    { 0xc0364eaf, 95 },
    { 0, 0 },
    { 0xc0764eb1, 95 },
    { 0x51f64eb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d64eb7, 99 },
    { 0x40164eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b964ebb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28564ebf, 90 },
    { 0xa3c64ec0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f64ec5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x86564ec8, 43 },
    { 0, 0 },
    { 0xac564eca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c64ecd, 90 },
    { 0xa1164ecd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8364ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac64ed6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1464edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d64edf, 61 },
    { 0x8af64ee0, 91 },
    { 0x39d64ee1, 76 },
    { 0x10864ee2, 33 },
    { 0x46064ee3, 33 },
    { 0xae664ee3, 23 },
    { 0xb9764ee5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc364ee8, 67 },
    { 0, 0 },
    { 0x6964eea, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a64eee, 61 },
    { 0, 0 },
    { 0xca564ef0, 85 },
    { 0x59264ef1, 55 },
    { 0xb8464ef2, 90 },
    { 0x6b564ef2, 33 },
    { 0x17164ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77164ef7, 59 },
    { 0x1b64ef7, 90 },
    { 0x59364ef9, 78 },
    { 0x7ad64efa, 23 },
    { 0xd0364efb, 55 },
    { 0xa564efc, 23 },
    { 0x48664efd, 77 },
    { 0x33164efc, 98 },
    { 0x25264efc, 99 },
    { 0x48664efc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde864f04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65664f0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e464f14, 98 },
    { 0, 0 },
    { 0xb5f64f16, 78 },
    { 0x99264f17, 91 },
    { 0x71c64f18, 90 },
    { 0, 0 },
    { 0x66564f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68464f1f, 76 },
    { 0, 0 },
    { 0x99d64f21, 81 },
    { 0xf6664f21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8664f25, 76 },
    { 0, 0 },
    { 0xf5d64f27, 55 },
    { 0x1d464f28, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f964f2b, 66 },
    { 0xb0f64f2c, 78 },
    { 0, 0 },
    { 0x26a64f2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2764f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c64f38, 98 },
    { 0x8964f38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7164f3c, 46 },
    { 0x75b64f3c, 33 },
    { 0, 0 },
    { 0xe4464f3f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d564f4c, 95 },
    { 0xac164f4c, 76 },
    { 0x53464f4d, 90 },
    { 0x30d64f4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43964f53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9164f57, 87 },
    { 0xeb664f57, 61 },
    { 0x39164f59, 33 },
    { 0x13e64f5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b64f60, 95 },
    { 0x55764f61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x40964f64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48864f6b, 90 },
    { 0, 0 },
    { 0x24e64f6d, 43 },
    { 0xdb64f6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56564f78, 67 },
    { 0xcd064f79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c064f7c, 61 },
    { 0xffc64f7d, 87 },
    { 0, 0 },
    { 0xb9064f7f, 84 },
    { 0xa1564f80, 99 },
    { 0xd6e64f81, 98 },
    { 0, 0 },
    { 0xd7464f83, 84 },
    { 0xa9364f84, 69 },
    { 0xe1964f85, 98 },
    { 0x1e164f86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e664f8a, 76 },
    { 0xec964f8b, 87 },
    { 0, 0 },
    { 0xf8664f8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2464f95, 95 },
    { 0xd3364f95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f364f99, 34 },
    { 0x18064f99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73764f9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5164fb8, 69 },
    { 0xc5764fb9, 78 },
    { 0x5164fb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70164fc0, 23 },
    { 0x33c64fc1, 76 },
    { 0, 0 },
    { 0x7a964fc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a664fc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb64fcc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb464fcf, 36 },
    { 0x95164fd0, 95 },
    { 0x5d564fd1, 91 },
    { 0x34064fd2, 34 },
    { 0x1ba64fd3, 61 },
    { 0x2f64fd4, 98 },
    { 0xe1e64fd5, 52 },
    { 0xc664fd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf64fd9, 88 },
    { 0x69964fda, 55 },
    { 0x5d164fdb, 93 },
    { 0x25c64fdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee764fea, 90 },
    { 0, 0 },
    { 0xd1764fec, 61 },
    { 0xbd864fed, 43 },
    { 0x19764fee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a64ff4, 77 },
    { 0x7cc64ff5, 90 },
    { 0, 0 },
    { 0x70264ff7, 33 },
    { 0x1bc64ff8, 95 },
    { 0x3c964ff9, 46 },
    { 0xe9364ff7, 99 },
    { 0, 0 },
    { 0xaf164ffc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6565001, 23 },
    { 0, 0 },
    { 0xde965003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa65006, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9165009, 55 },
    { 0, 0 },
    { 0x4bf6500b, 93 },
    { 0, 0 },
    { 0x3696500d, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc265014, 98 },
    { 0xf3465015, 23 },
    { 0, 0 },
    { 0x5e265017, 23 },
    { 0, 0 },
    { 0x1e265019, 43 },
    { 0, 0 },
    { 0xbaa6501b, 87 },
    { 0x3406501c, 90 },
    { 0xef66501d, 78 },
    { 0xcd36501d, 33 },
    { 0x1d46501b, 33 },
    { 0x9b465020, 93 },
    { 0x7e165021, 23 },
    { 0x8c66501b, 33 },
    { 0, 0 },
    { 0x4f165024, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4065028, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe86502b, 75 },
    { 0xfe76502c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd556502f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a65033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79765039, 34 },
    { 0xe0065039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23665040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe65043, 98 },
    { 0, 0 },
    { 0x2ff65045, 43 },
    { 0x10865046, 90 },
    { 0, 0 },
    { 0x54265048, 43 },
    { 0xc765049, 98 },
    { 0, 0 },
    { 0xd126504b, 61 },
    { 0xb486504c, 33 },
    { 0x5616504b, 33 },
    { 0xf806504e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc65051, 76 },
    { 0x52365051, 61 },
    { 0, 0 },
    { 0x3c265054, 33 },
    { 0, 0 },
    { 0xf9865056, 90 },
    { 0x5d965057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f6505b, 61 },
    { 0, 0 },
    { 0x7cc6505d, 43 },
    { 0, 0 },
    { 0x8af6505f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f165063, 33 },
    { 0x64965064, 77 },
    { 0x7c65063, 95 },
    { 0x69565066, 43 },
    { 0x2ea65065, 76 },
    { 0, 0 },
    { 0x36c65069, 98 },
    { 0x89065069, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff46506f, 100 },
    { 0x8cc65070, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28765076, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee6507b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2346507e, 34 },
    { 0xbb26507f, 61 },
    { 0x6a665080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17165083, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75565088, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef46508c, 55 },
    { 0, 0 },
    { 0x8556508e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa465098, 98 },
    { 0xaae65099, 33 },
    { 0, 0 },
    { 0x2106509b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956509e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e650a3, 34 },
    { 0xfbc650a4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e650a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae650ab, 95 },
    { 0xa09650ac, 43 },
    { 0x793650ac, 23 },
    { 0x1cc650ac, 23 },
    { 0, 0 },
    { 0xe11650b0, 23 },
    { 0xb7b650b1, 98 },
    { 0x476650b2, 90 },
    { 0x159650b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0650b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39650bb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb650be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf650c9, 23 },
    { 0x5a6650ca, 81 },
    { 0x877650ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9650cf, 98 },
    { 0xf29650d0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d650d4, 98 },
    { 0x19e650d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8650da, 33 },
    { 0, 0 },
    { 0x921650dc, 90 },
    { 0x28f650dd, 46 },
    { 0, 0 },
    { 0x97b650df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f650e4, 61 },
    { 0, 0 },
    { 0x414650e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96650ec, 33 },
    { 0x27f650ed, 90 },
    { 0x459650ee, 34 },
    { 0xccd650ed, 23 },
    { 0xbba650ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe650f5, 90 },
    { 0x26b650f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x834650f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b650fd, 76 },
    { 0xb4e650fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a465105, 33 },
    { 0, 0 },
    { 0xa5f65107, 61 },
    { 0x1ec65108, 33 },
    { 0x7d165109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6510c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d6510f, 95 },
    { 0xcc16510f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db6511c, 98 },
    { 0x1ee6511d, 55 },
    { 0, 0 },
    { 0x5516511f, 76 },
    { 0x1ab65120, 23 },
    { 0xf5865121, 24 },
    { 0, 0 },
    { 0x7f165123, 98 },
    { 0xc6365124, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc65129, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9086512c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x46512f, 68 },
    { 0x5c06512f, 90 },
    { 0, 0 },
    { 0xbd665132, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69765136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd076513e, 55 },
    { 0x9016513f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c65144, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2465148, 61 },
    { 0x5d865149, 90 },
    { 0xe56514a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6514f, 46 },
    { 0, 0 },
    { 0x20665151, 98 },
    { 0xa9865152, 98 },
    { 0x65965152, 33 },
    { 0x3e965154, 55 },
    { 0xbed65155, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2946515a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa36515d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a65162, 53 },
    { 0xfae65163, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a65169, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6516e, 76 },
    { 0xa106516f, 66 },
    { 0xe1765170, 55 },
    { 0x88665171, 23 },
    { 0x2065171, 34 },
    { 0x7e765172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad065177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8056517a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9696517e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f65181, 95 },
    { 0xdd265182, 33 },
    { 0x13165181, 93 },
    { 0, 0 },
    { 0xa0165185, 61 },
    { 0, 0 },
    { 0xfc065187, 98 },
    { 0, 0 },
    { 0x56165189, 89 },
    { 0x8ec65189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde06518f, 90 },
    { 0, 0 },
    { 0xf1f65191, 66 },
    { 0xa2265192, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x74065195, 52 },
    { 0x8165195, 61 },
    { 0x86c65197, 23 },
    { 0xc5e65196, 77 },
    { 0, 0 },
    { 0xdc76519a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5266519d, 23 },
    { 0xedb6519d, 93 },
    { 0, 0 },
    { 0xe43651a0, 98 },
    { 0x147651a1, 95 },
    { 0x50a651a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6651aa, 91 },
    { 0xf05651ab, 78 },
    { 0x39f651ab, 95 },
    { 0xcad651ab, 85 },
    { 0x391651ae, 61 },
    { 0xa9651af, 22 },
    { 0xdf1651ae, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d651b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462651bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f651c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61651c5, 90 },
    { 0xd88651c5, 78 },
    { 0xf81651c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd651cd, 95 },
    { 0, 0 },
    { 0x57651cf, 66 },
    { 0xb6f651d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a651d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7651d9, 99 },
    { 0xc0b651da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69651e6, 55 },
    { 0x779651e7, 85 },
    { 0x31d651e8, 90 },
    { 0x475651e9, 95 },
    { 0x313651ea, 39 },
    { 0xdf6651eb, 69 },
    { 0x66a651ec, 90 },
    { 0x902651e9, 91 },
    { 0xe3b651e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5651f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95651f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd651fe, 90 },
    { 0x48c651fe, 23 },
    { 0x516651ff, 99 },
    { 0, 0 },
    { 0xb8865202, 76 },
    { 0x10665203, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce65206, 98 },
    { 0xeaa65207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xece6520a, 90 },
    { 0xb706520b, 95 },
    { 0x4bb6520c, 55 },
    { 0xcdb6520d, 91 },
    { 0x4a76520a, 23 },
    { 0x1b36520e, 90 },
    { 0x1796520b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb165214, 100 },
    { 0x92065215, 95 },
    { 0x53565214, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5965222, 98 },
    { 0x32065223, 77 },
    { 0xa2f65224, 95 },
    { 0, 0 },
    { 0x45b65226, 90 },
    { 0xcea65226, 90 },
    { 0xbf465228, 90 },
    { 0x49265229, 95 },
    { 0, 0 },
    { 0xf1a6522b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80865230, 67 },
    { 0x9bc65231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de6523b, 55 },
    { 0x6d56523c, 90 },
    { 0xc726523b, 98 },
    { 0x7896523e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8565242, 33 },
    { 0xe4265242, 97 },
    { 0x48965242, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a265257, 95 },
    { 0xfa765258, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5146525b, 66 },
    { 0, 0 },
    { 0xd596525d, 33 },
    { 0, 0 },
    { 0xf346525f, 33 },
    { 0xa1365260, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20865265, 43 },
    { 0, 0 },
    { 0xf2565267, 40 },
    { 0xd0665267, 33 },
    { 0x46365269, 61 },
    { 0, 0 },
    { 0xee16526b, 90 },
    { 0, 0 },
    { 0x2796526d, 91 },
    { 0xfd16526e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae65275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2016527e, 95 },
    { 0x5f16527f, 34 },
    { 0x7cf65280, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a65285, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88665289, 61 },
    { 0, 0 },
    { 0xbe26528b, 98 },
    { 0, 0 },
    { 0xf86528d, 52 },
    { 0x9cb6528e, 90 },
    { 0xf276528e, 66 },
    { 0, 0 },
    { 0x45e65291, 34 },
    { 0, 0 },
    { 0xe2c65293, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc565296, 23 },
    { 0x9ee65297, 98 },
    { 0x16e65297, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf6529c, 91 },
    { 0, 0 },
    { 0xf886529e, 88 },
    { 0x9116529f, 52 },
    { 0xe68652a0, 95 },
    { 0, 0 },
    { 0xd56652a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3652a7, 34 },
    { 0x9d1652a8, 34 },
    { 0xa39652a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38652ad, 87 },
    { 0x103652ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2652b1, 55 },
    { 0xfa1652b2, 65 },
    { 0, 0 },
    { 0xa50652b4, 43 },
    { 0, 0 },
    { 0xca2652b6, 95 },
    { 0x792652b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b652bc, 33 },
    { 0x83c652bd, 61 },
    { 0x4bc652be, 61 },
    { 0x5db652bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1652c8, 33 },
    { 0, 0 },
    { 0xcc4652ca, 95 },
    { 0x7c652ca, 55 },
    { 0x853652ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b652d6, 75 },
    { 0x5a9652d7, 90 },
    { 0, 0 },
    { 0xcd3652d9, 34 },
    { 0x2c3652d9, 33 },
    { 0x2e652da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519652e4, 98 },
    { 0, 0 },
    { 0xd7c652e6, 95 },
    { 0xde9652e6, 74 },
    { 0x2d9652e7, 99 },
    { 0x275652e6, 46 },
    { 0, 0 },
    { 0xcd4652eb, 46 },
    { 0, 0 },
    { 0x802652ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89652f0, 90 },
    { 0x408652f1, 43 },
    { 0x22652f2, 76 },
    { 0x775652f3, 33 },
    { 0x3c652f4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d652f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8652fb, 90 },
    { 0x982652fc, 23 },
    { 0x126652fd, 61 },
    { 0xaf7652fd, 61 },
    { 0, 0 },
    { 0x46b65300, 24 },
    { 0xbfd65301, 90 },
    { 0xfb565302, 98 },
    { 0x1b665300, 33 },
    { 0, 0 },
    { 0x7665305, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3286530a, 78 },
    { 0xc6f6530a, 64 },
    { 0x5906530c, 61 },
    { 0, 0 },
    { 0x2ea6530e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f65311, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce65314, 81 },
    { 0x6be65314, 33 },
    { 0x65f65316, 23 },
    { 0xbe65317, 56 },
    { 0x17665314, 95 },
    { 0x49165318, 99 },
    { 0xe186531a, 61 },
    { 0, 0 },
    { 0x57a6531c, 90 },
    { 0x1e76531d, 23 },
    { 0x6496531e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe065322, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a65326, 90 },
    { 0x53f65326, 33 },
    { 0xfd465328, 34 },
    { 0x30565329, 61 },
    { 0x9be65328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1246532d, 95 },
    { 0xfaf6532e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab65332, 36 },
    { 0x7ec65333, 98 },
    { 0, 0 },
    { 0x6e665335, 61 },
    { 0, 0 },
    { 0xa2065337, 43 },
    { 0, 0 },
    { 0x3ea65339, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b16533e, 75 },
    { 0x2f66533f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78465344, 43 },
    { 0xe6b65344, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e665349, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626534e, 61 },
    { 0x41a6534e, 55 },
    { 0, 0 },
    { 0x14965351, 93 },
    { 0xac565351, 90 },
    { 0, 0 },
    { 0x1d965354, 93 },
    { 0xb9565355, 23 },
    { 0xbd565356, 46 },
    { 0x4e965356, 33 },
    { 0x86165356, 90 },
    { 0xd465357, 99 },
    { 0xaa6535a, 23 },
    { 0xa4e65356, 75 },
    { 0, 0 },
    { 0xdd16535d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d965361, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75465368, 93 },
    { 0, 0 },
    { 0x3186536a, 23 },
    { 0, 0 },
    { 0x4e86536c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c765372, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e65375, 78 },
    { 0, 0 },
    { 0x54c65377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae6537b, 98 },
    { 0xaff6537b, 98 },
    { 0x8eb6537d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2365386, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd376538f, 93 },
    { 0xb0e65390, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a065397, 97 },
    { 0x4ca65398, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d6539d, 95 },
    { 0x6636539e, 33 },
    { 0, 0 },
    { 0xc55653a0, 100 },
    { 0x1653a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x484653a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b653a8, 75 },
    { 0xf4b653a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b653af, 33 },
    { 0x5ff653b0, 33 },
    { 0x122653af, 98 },
    { 0x142653af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef653c1, 55 },
    { 0x936653c1, 61 },
    { 0, 0 },
    { 0x56a653c4, 66 },
    { 0x72653c5, 99 },
    { 0x184653c6, 55 },
    { 0, 0 },
    { 0x1ce653c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171653cd, 33 },
    { 0x507653ce, 61 },
    { 0x86b653cf, 90 },
    { 0xf2e653d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08653d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501653d8, 55 },
    { 0xde4653d8, 33 },
    { 0, 0 },
    { 0x45d653db, 34 },
    { 0x8bd653dc, 75 },
    { 0xa7d653db, 95 },
    { 0x16d653de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d653e1, 61 },
    { 0x92c653e1, 43 },
    { 0xcc0653e3, 55 },
    { 0x588653e3, 100 },
    { 0x6a9653e4, 65 },
    { 0x954653e6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba653ef, 20 },
    { 0x88d653f0, 93 },
    { 0x396653f1, 88 },
    { 0, 0 },
    { 0x4b8653f3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce653f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb653fb, 23 },
    { 0x5c3653fc, 90 },
    { 0, 0 },
    { 0x1a3653fe, 55 },
    { 0xa73653ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d65405, 46 },
    { 0x74b65406, 61 },
    { 0xa8165407, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e6540b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d65411, 95 },
    { 0x19065411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c65416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68165419, 65 },
    { 0x7026541a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xea86541d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42665423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b16542a, 53 },
    { 0x9386542b, 90 },
    { 0x9c26542a, 43 },
    { 0, 0 },
    { 0x14f6542e, 61 },
    { 0, 0 },
    { 0x4ad65430, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1465438, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c6543c, 95 },
    { 0x4646543c, 91 },
    { 0xb116543e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f6544a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4886544d, 33 },
    { 0x30c6544d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b665451, 33 },
    { 0, 0 },
    { 0x50a65453, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f65458, 91 },
    { 0x64665459, 66 },
    { 0x4e65459, 90 },
    { 0x5cd65458, 43 },
    { 0xc5a65459, 33 },
    { 0, 0 },
    { 0xf876545e, 76 },
    { 0x2b16545f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd65466, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc765469, 88 },
    { 0x22b65469, 55 },
    { 0xa5b6546b, 40 },
    { 0, 0 },
    { 0x2616546d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b565471, 33 },
    { 0x30265472, 99 },
    { 0x4fa65473, 55 },
    { 0x5c365474, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac65478, 95 },
    { 0, 0 },
    { 0xd376547a, 98 },
    { 0xf466547b, 98 },
    { 0xf4f6547c, 33 },
    { 0x61f6547b, 23 },
    { 0x4ca6547d, 98 },
    { 0x5d56547f, 40 },
    { 0x4ca6547f, 61 },
    { 0xe7265480, 93 },
    { 0x2d965482, 88 },
    { 0x22b65483, 52 },
    { 0x45065484, 90 },
    { 0x3cf65485, 61 },
    { 0x2dd6547d, 33 },
    { 0x27665487, 76 },
    { 0x77f6547c, 75 },
    { 0x8aa65485, 90 },
    { 0xf466548a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd476548d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e765497, 98 },
    { 0x7b665498, 95 },
    { 0, 0 },
    { 0x4696549a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86549f, 95 },
    { 0x2ea654a0, 99 },
    { 0, 0 },
    { 0xcf2654a2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b654ad, 90 },
    { 0xe9d654ae, 66 },
    { 0x6c3654ae, 99 },
    { 0, 0 },
    { 0x77654b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7654b4, 61 },
    { 0x96654b4, 90 },
    { 0, 0 },
    { 0x572654b7, 67 },
    { 0xc35654b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb654bd, 80 },
    { 0, 0 },
    { 0xe80654bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e654c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677654ca, 61 },
    { 0, 0 },
    { 0x671654cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c654d1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db654d8, 23 },
    { 0x83f654d8, 61 },
    { 0xb8b654da, 33 },
    { 0, 0 },
    { 0x9fe654dc, 98 },
    { 0x4a5654dc, 76 },
    { 0x9df654de, 85 },
    { 0, 0 },
    { 0x60d654e0, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882654ec, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd654f2, 91 },
    { 0x237654f3, 90 },
    { 0, 0 },
    { 0x8e0654f5, 96 },
    { 0xc47654f5, 95 },
    { 0x9aa654f5, 76 },
    { 0xd87654f8, 23 },
    { 0x41654f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f654fe, 90 },
    { 0, 0 },
    { 0x7f465500, 91 },
    { 0x28865500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8665504, 99 },
    { 0xcce65505, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x51065508, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106550d, 95 },
    { 0xe416550e, 23 },
    { 0xe3f6550d, 33 },
    { 0x3456550f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65665513, 33 },
    { 0, 0 },
    { 0x11365515, 93 },
    { 0, 0 },
    { 0x8565517, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c6551a, 23 },
    { 0xcea6551b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8706551f, 99 },
    { 0x15965520, 82 },
    { 0xe1c65521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f865525, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76552b, 83 },
    { 0x67c6552b, 68 },
    { 0xc2c6552b, 33 },
    { 0x3a16552b, 33 },
    { 0x9026552f, 55 },
    { 0x9a165530, 33 },
    { 0xab765531, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x87065534, 34 },
    { 0x55765535, 90 },
    { 0, 0 },
    { 0xb5265537, 61 },
    { 0xe0465538, 90 },
    { 0x7fd65537, 98 },
    { 0, 0 },
    { 0x5746553b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c46553f, 61 },
    { 0x8bb65540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde65548, 90 },
    { 0x59265549, 27 },
    { 0, 0 },
    { 0xf106554b, 33 },
    { 0x15a6554b, 74 },
    { 0x8c96554d, 33 },
    { 0, 0 },
    { 0x90f6554f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35765554, 91 },
    { 0x65865555, 77 },
    { 0x40d65555, 78 },
    { 0, 0 },
    { 0x4665558, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b6555e, 98 },
    { 0, 0 },
    { 0xd0a65560, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf6556b, 90 },
    { 0xdcd6556c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc65571, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b65574, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b65577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec76557d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8265581, 90 },
    { 0xab465582, 52 },
    { 0x52665583, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad65586, 76 },
    { 0x9f165587, 76 },
    { 0x75b65588, 95 },
    { 0x7e965588, 61 },
    { 0xb6d6558a, 36 },
    { 0xe326558b, 98 },
    { 0x43365589, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90065590, 98 },
    { 0, 0 },
    { 0x2165592, 17 },
    { 0x73c65593, 61 },
    { 0xd8365593, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa356559f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0655a4, 66 },
    { 0x3cf655a5, 33 },
    { 0x6de655a6, 81 },
    { 0, 0 },
    { 0xe23655a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c655ad, 46 },
    { 0, 0 },
    { 0x319655af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a655bc, 77 },
    { 0x91c655bd, 55 },
    { 0x178655be, 95 },
    { 0x7ce655be, 70 },
    { 0xf15655bf, 90 },
    { 0xe52655c1, 23 },
    { 0, 0 },
    { 0x6d4655c3, 98 },
    { 0, 0 },
    { 0x44c655c5, 96 },
    { 0xeeb655c5, 81 },
    { 0, 0 },
    { 0xb7f655c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e655cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93655ce, 95 },
    { 0x6c1655cf, 87 },
    { 0, 0 },
    { 0xb33655d1, 55 },
    { 0x290655d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34655d8, 23 },
    { 0xe71655d9, 90 },
    { 0x655da, 61 },
    { 0x6e5655db, 66 },
    { 0, 0 },
    { 0xd2e655dd, 33 },
    { 0x8ce655de, 78 },
    { 0x665655de, 98 },
    { 0x17a655df, 98 },
    { 0, 0 },
    { 0x4f4655e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e655e6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362655ea, 23 },
    { 0x132655eb, 65 },
    { 0, 0 },
    { 0x741655ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb655f1, 61 },
    { 0xb2e655f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f655f9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac655fc, 90 },
    { 0xf0a655fd, 95 },
    { 0x4e8655fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd565601, 76 },
    { 0x32d65602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5256560b, 90 },
    { 0xd876560c, 66 },
    { 0xbfb6560d, 67 },
    { 0xdbc6560e, 98 },
    { 0, 0 },
    { 0x59965610, 33 },
    { 0xf7965611, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d65614, 76 },
    { 0xc7765615, 23 },
    { 0, 0 },
    { 0x2ce65617, 90 },
    { 0xe5e65617, 17 },
    { 0x49c65618, 96 },
    { 0xfe06561a, 68 },
    { 0x67465617, 76 },
    { 0x4a665618, 95 },
    { 0x5346561d, 91 },
    { 0xeec6561d, 90 },
    { 0x3106561f, 95 },
    { 0xc0d6561b, 78 },
    { 0xf3e65621, 46 },
    { 0x21e65621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba65629, 99 },
    { 0, 0 },
    { 0xe316562b, 44 },
    { 0x36c6562b, 61 },
    { 0, 0 },
    { 0xffd6562e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf65635, 98 },
    { 0x4b865636, 90 },
    { 0, 0 },
    { 0xa7365638, 90 },
    { 0, 0 },
    { 0x7516563a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c6563d, 81 },
    { 0, 0 },
    { 0x4ba6563f, 33 },
    { 0, 0 },
    { 0x7a265641, 34 },
    { 0, 0 },
    { 0x88665643, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa96564c, 88 },
    { 0, 0 },
    { 0x7fd6564e, 90 },
    { 0x7466564f, 78 },
    { 0x5706564e, 95 },
    { 0, 0 },
    { 0x30265652, 66 },
    { 0x7365653, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c6565d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x49665660, 46 },
    { 0xbe965661, 90 },
    { 0x47c65660, 43 },
    { 0xb4b65662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b765666, 34 },
    { 0x8a965666, 81 },
    { 0xafa65668, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd046566e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b65674, 98 },
    { 0, 0 },
    { 0x62865676, 95 },
    { 0, 0 },
    { 0x34265678, 33 },
    { 0, 0 },
    { 0xb6567a, 61 },
    { 0xc556567b, 93 },
    { 0x7956567c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b96567f, 61 },
    { 0x3f86567f, 46 },
    { 0x35665680, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd326568a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2086568d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2965691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e265695, 61 },
    { 0, 0 },
    { 0xbcd65697, 76 },
    { 0x73665697, 95 },
    { 0x4c065698, 33 },
    { 0x6dc65699, 55 },
    { 0x6ee65698, 33 },
    { 0, 0 },
    { 0x9886569d, 33 },
    { 0x3076569e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401656a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x78656a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94656ac, 87 },
    { 0x8dc656ad, 34 },
    { 0xcae656ad, 61 },
    { 0xa47656ad, 34 },
    { 0, 0 },
    { 0x110656b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68656b6, 90 },
    { 0, 0 },
    { 0xf1a656b8, 66 },
    { 0xb59656b8, 67 },
    { 0, 0 },
    { 0x7e1656bb, 33 },
    { 0x96e656bc, 33 },
    { 0, 0 },
    { 0x422656be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a656c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b656cd, 84 },
    { 0x500656cd, 23 },
    { 0x3f8656cf, 34 },
    { 0xd87656cf, 95 },
    { 0, 0 },
    { 0x4bf656d2, 87 },
    { 0x373656d3, 93 },
    { 0, 0 },
    { 0xe5656d5, 90 },
    { 0x9d0656d6, 33 },
    { 0, 0 },
    { 0xa3f656d8, 12 },
    { 0, 0 },
    { 0x641656da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb656dd, 34 },
    { 0, 0 },
    { 0x1c656df, 67 },
    { 0x4ff656e0, 65 },
    { 0x150656e0, 90 },
    { 0x27e656e0, 34 },
    { 0xf24656e2, 90 },
    { 0xd21656e4, 88 },
    { 0x288656e3, 76 },
    { 0x682656e6, 61 },
    { 0x598656e5, 23 },
    { 0x321656e8, 77 },
    { 0, 0 },
    { 0x3c656ea, 61 },
    { 0x409656eb, 76 },
    { 0x516656eb, 78 },
    { 0x993656ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28656f0, 88 },
    { 0, 0 },
    { 0x726656f2, 85 },
    { 0x737656f2, 80 },
    { 0x862656f4, 85 },
    { 0xad6656f5, 34 },
    { 0x799656f4, 98 },
    { 0xb3e656f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57656fd, 67 },
    { 0xf70656fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed65705, 23 },
    { 0x84065706, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb265709, 46 },
    { 0xc8e6570a, 33 },
    { 0xfe76570b, 95 },
    { 0xe926570b, 26 },
    { 0x81d6570d, 95 },
    { 0x4146570e, 23 },
    { 0xa8765709, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18765714, 61 },
    { 0x6ce65715, 78 },
    { 0, 0 },
    { 0x9fc65717, 95 },
    { 0xb3965718, 33 },
    { 0, 0 },
    { 0x4b16571a, 90 },
    { 0xb8e6571a, 85 },
    { 0x49c6571a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce65720, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9065724, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1765729, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6572f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c765734, 90 },
    { 0xf7e65734, 99 },
    { 0xd3465735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99765739, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab65742, 17 },
    { 0, 0 },
    { 0xbb265744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51065748, 99 },
    { 0x66c65749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6574c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65750, 90 },
    { 0, 0 },
    { 0x3e165752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54565755, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd526575c, 33 },
    { 0, 0 },
    { 0x97e6575e, 68 },
    { 0xe696575e, 98 },
    { 0, 0 },
    { 0xf3465761, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4065765, 33 },
    { 0x4af65766, 76 },
    { 0x7165765, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f36576b, 98 },
    { 0, 0 },
    { 0xa1b6576d, 90 },
    { 0x8f26576e, 90 },
    { 0x7416576f, 95 },
    { 0xddd65770, 87 },
    { 0x1516576f, 95 },
    { 0xf2f65772, 61 },
    { 0x1b86576d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b65779, 33 },
    { 0x1486577a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b6577d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55165780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2365784, 43 },
    { 0x10d65785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e6578d, 91 },
    { 0, 0 },
    { 0xb436578f, 95 },
    { 0x1d65790, 98 },
    { 0, 0 },
    { 0x34a65792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d65797, 78 },
    { 0xa9a65798, 33 },
    { 0x95265798, 56 },
    { 0x20d65799, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e6579e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc657a3, 90 },
    { 0, 0 },
    { 0xeae657a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e657af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa657b2, 95 },
    { 0x9e0657b3, 95 },
    { 0x930657b3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186657b8, 95 },
    { 0x897657b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf657c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30657c8, 87 },
    { 0, 0 },
    { 0x7657ca, 95 },
    { 0, 0 },
    { 0x7bf657cc, 43 },
    { 0, 0 },
    { 0x8d7657ce, 98 },
    { 0, 0 },
    { 0xaa0657d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x731657d3, 52 },
    { 0x4d0657d3, 43 },
    { 0x8e8657d5, 61 },
    { 0xdea657d4, 85 },
    { 0, 0 },
    { 0xd26657d8, 33 },
    { 0x31f657d8, 43 },
    { 0xd3d657d9, 100 },
    { 0xf65657d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3657e0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2657e6, 99 },
    { 0x1c3657e6, 34 },
    { 0x48c657e8, 33 },
    { 0xc4f657e9, 80 },
    { 0x5a8657e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34657f0, 33 },
    { 0, 0 },
    { 0xf0b657f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc657f5, 87 },
    { 0xf03657f6, 90 },
    { 0xfd657f7, 100 },
    { 0, 0 },
    { 0xf60657f9, 55 },
    { 0xf1e657f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71765804, 76 },
    { 0xb9065804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1865809, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8416580d, 61 },
    { 0x6c76580e, 61 },
    { 0x22a6580f, 61 },
    { 0xf3a65810, 91 },
    { 0xbdf6580e, 61 },
    { 0, 0 },
    { 0xa5065813, 61 },
    { 0xce265814, 76 },
    { 0x4c565815, 98 },
    { 0xd1365815, 23 },
    { 0xa7a65817, 23 },
    { 0, 0 },
    { 0xde565819, 93 },
    { 0, 0 },
    { 0x4916581b, 93 },
    { 0x4b66581c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5156581f, 46 },
    { 0x52065820, 33 },
    { 0xf0465820, 95 },
    { 0xa336581f, 95 },
    { 0xf9c65823, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f565826, 88 },
    { 0x61765827, 87 },
    { 0x18e65828, 67 },
    { 0, 0 },
    { 0xb816582a, 85 },
    { 0x8406582a, 66 },
    { 0xb556582c, 90 },
    { 0xdfe6582c, 81 },
    { 0xda96582c, 90 },
    { 0xa166582f, 75 },
    { 0x4486582b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c65836, 34 },
    { 0x76565836, 95 },
    { 0xbb865836, 23 },
    { 0xddc65836, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6583f, 33 },
    { 0xa965840, 33 },
    { 0xa9765841, 61 },
    { 0xb565842, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c65845, 76 },
    { 0x7d65846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc66584d, 61 },
    { 0x7336584e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d65851, 23 },
    { 0xad65852, 95 },
    { 0x4c165853, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f26585a, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb66585f, 76 },
    { 0x98d65860, 88 },
    { 0xc1965860, 93 },
    { 0x75465862, 98 },
    { 0, 0 },
    { 0xc0465864, 93 },
    { 0, 0 },
    { 0xd7e65866, 76 },
    { 0x47865866, 90 },
    { 0, 0 },
    { 0x9ac65869, 90 },
    { 0x88b6586a, 23 },
    { 0, 0 },
    { 0xad66586c, 85 },
    { 0xadc6586d, 90 },
    { 0xb3f6586d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93265871, 98 },
    { 0x4c565872, 84 },
    { 0xae865873, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f6587a, 84 },
    { 0, 0 },
    { 0xc466587c, 98 },
    { 0x9f26587d, 76 },
    { 0x7b76587e, 46 },
    { 0xcf76587c, 52 },
    { 0xfd265880, 34 },
    { 0x70165880, 33 },
    { 0x15b65882, 84 },
    { 0x30265882, 95 },
    { 0x5f065883, 34 },
    { 0xe3665885, 90 },
    { 0x8a665882, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c65889, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336588f, 90 },
    { 0x54c65890, 88 },
    { 0xea565891, 33 },
    { 0x3af65892, 52 },
    { 0x1b165893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3165897, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc666589e, 40 },
    { 0xfa06589e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39658a6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a658ae, 98 },
    { 0xb6e658ae, 43 },
    { 0x717658b0, 61 },
    { 0x3dd658b0, 23 },
    { 0xf4d658b2, 84 },
    { 0xb45658b3, 34 },
    { 0xd7e658b4, 23 },
    { 0xc57658b5, 12 },
    { 0x2a7658b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7658b9, 98 },
    { 0x28f658ba, 64 },
    { 0x479658bb, 33 },
    { 0x60b658ba, 61 },
    { 0xcbf658bd, 55 },
    { 0, 0 },
    { 0x6ed658bf, 90 },
    { 0, 0 },
    { 0xbc1658c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9658c5, 95 },
    { 0, 0 },
    { 0x84d658c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5658ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc658d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58658d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4658d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8658dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13658e0, 90 },
    { 0, 0 },
    { 0x8f9658e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787658e9, 90 },
    { 0x113658e9, 67 },
    { 0x8cf658e9, 95 },
    { 0xdc0658ec, 34 },
    { 0xf1b658eb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08658f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f658f9, 67 },
    { 0x327658fa, 90 },
    { 0x66d658fb, 90 },
    { 0x918658fb, 61 },
    { 0x4c6658fd, 90 },
    { 0xaf658fe, 33 },
    { 0x3f1658ff, 84 },
    { 0x83965900, 98 },
    { 0xc82658fe, 90 },
    { 0x7a8658ff, 33 },
    { 0xdc65903, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe65906, 84 },
    { 0x3ba65907, 17 },
    { 0xc7065908, 61 },
    { 0x4ba65908, 61 },
    { 0, 0 },
    { 0x4a16590b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7365911, 95 },
    { 0, 0 },
    { 0x8b165913, 46 },
    { 0x5c665914, 95 },
    { 0x66765914, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c65920, 55 },
    { 0xdc865921, 95 },
    { 0x67765921, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21865927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4436592d, 90 },
    { 0, 0 },
    { 0xea86592f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a65933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f65937, 76 },
    { 0, 0 },
    { 0xe0865939, 69 },
    { 0, 0 },
    { 0x8a06593b, 90 },
    { 0xc8e6593b, 85 },
    { 0x57a6593b, 34 },
    { 0x72a6593e, 34 },
    { 0xa596593f, 67 },
    { 0x94e65940, 33 },
    { 0, 0 },
    { 0xbc665942, 55 },
    { 0x85265943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a65947, 23 },
    { 0, 0 },
    { 0x91265949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd6594e, 78 },
    { 0xcb96594e, 66 },
    { 0, 0 },
    { 0x37665951, 90 },
    { 0x19565952, 61 },
    { 0xfe165953, 76 },
    { 0xf8965953, 90 },
    { 0xd6e65954, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde865959, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb466595d, 90 },
    { 0, 0 },
    { 0xf596595f, 90 },
    { 0x3e76595f, 80 },
    { 0xf9865960, 46 },
    { 0x6d265962, 43 },
    { 0x32e65963, 99 },
    { 0xfe565964, 90 },
    { 0xcc86595f, 33 },
    { 0x19465966, 55 },
    { 0x24e65967, 33 },
    { 0x52665968, 94 },
    { 0, 0 },
    { 0xb7b6596a, 33 },
    { 0x72d6596b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa665970, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6865974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c6597f, 93 },
    { 0xe9165980, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d565984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d565989, 36 },
    { 0xe4265989, 34 },
    { 0xcfc6598b, 81 },
    { 0, 0 },
    { 0x6a26598d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f65991, 98 },
    { 0xf2465992, 76 },
    { 0x23765992, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2065996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe266599a, 55 },
    { 0, 0 },
    { 0xed96599c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6599f, 91 },
    { 0xb31659a0, 61 },
    { 0xa8659a1, 67 },
    { 0x16c659a2, 69 },
    { 0x366659a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb659a9, 34 },
    { 0, 0 },
    { 0x7b659ab, 33 },
    { 0xe82659ab, 52 },
    { 0xc1f659ac, 100 },
    { 0xaf2659ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d659b3, 95 },
    { 0xd41659b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73659b9, 93 },
    { 0x323659ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf659be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf659c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7659d0, 87 },
    { 0xe82659d1, 43 },
    { 0xfee659d2, 98 },
    { 0x8ea659d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5659dc, 33 },
    { 0x99d659dc, 42 },
    { 0xfd3659dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8659e2, 90 },
    { 0xaa6659e2, 61 },
    { 0, 0 },
    { 0xacd659e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf659eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46659ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7659f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9659f8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6659fc, 61 },
    { 0xb41659fd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad65a06, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65a09, 98 },
    { 0x36765a09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea165a0f, 90 },
    { 0x57165a10, 88 },
    { 0, 0 },
    { 0x4f565a12, 34 },
    { 0x64765a12, 61 },
    { 0xee865a14, 98 },
    { 0x6df65a15, 90 },
    { 0x16765a15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c65a1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x82665a1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25665a26, 34 },
    { 0xeca65a26, 55 },
    { 0, 0 },
    { 0x3b665a29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c65a2c, 33 },
    { 0xeb765a2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed765a31, 43 },
    { 0x59c65a32, 77 },
    { 0xda165a31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd65a39, 98 },
    { 0x98865a3a, 90 },
    { 0, 0 },
    { 0x64d65a3c, 61 },
    { 0x97d65a3d, 90 },
    { 0xbdb65a3e, 68 },
    { 0x4fd65a3f, 90 },
    { 0xf6865a40, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d65a44, 90 },
    { 0xa5765a45, 33 },
    { 0, 0 },
    { 0xeb865a47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e465a4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64965a53, 33 },
    { 0xb3365a53, 90 },
    { 0xbea65a55, 76 },
    { 0xa8c65a56, 33 },
    { 0, 0 },
    { 0x4d065a58, 61 },
    { 0x7ca65a59, 93 },
    { 0xe3465a59, 77 },
    { 0, 0 },
    { 0x93f65a5c, 76 },
    { 0, 0 },
    { 0x14565a5e, 88 },
    { 0, 0 },
    { 0x59565a60, 55 },
    { 0x16a65a61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b65a65, 98 },
    { 0xf2865a65, 95 },
    { 0x28d65a67, 90 },
    { 0x86765a68, 33 },
    { 0x29d65a66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e565a71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b65a74, 99 },
    { 0, 0 },
    { 0xfa265a76, 90 },
    { 0, 0 },
    { 0xf8265a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae865a7d, 69 },
    { 0x77565a7e, 98 },
    { 0xa6065a7e, 95 },
    { 0x69265a80, 23 },
    { 0x21d65a7e, 98 },
    { 0xf265a7e, 95 },
    { 0x38765a83, 75 },
    { 0x66b65a84, 90 },
    { 0x86965a84, 91 },
    { 0xa065a86, 34 },
    { 0x1e665a87, 23 },
    { 0x6e265a85, 61 },
    { 0xfa765a80, 100 },
    { 0, 0 },
    { 0x65265a8b, 46 },
    { 0x6bc65a8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a65a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db65a93, 33 },
    { 0, 0 },
    { 0x2ee65a95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a265a98, 55 },
    { 0x34165a99, 95 },
    { 0xd6e65a9a, 76 },
    { 0, 0 },
    { 0x3d265a9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94465aa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f665aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d865aae, 90 },
    { 0x9ec65aae, 90 },
    { 0x20a65ab0, 46 },
    { 0, 0 },
    { 0xae65ab2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac65abd, 55 },
    { 0x61f65abe, 90 },
    { 0x3bf65abf, 93 },
    { 0x84565ac0, 61 },
    { 0x32765ac0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86265ac6, 100 },
    { 0, 0 },
    { 0x4a165ac8, 55 },
    { 0x21165ac8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25465ace, 95 },
    { 0xc9665acf, 66 },
    { 0xfe65ace, 52 },
    { 0xed265ad1, 66 },
    { 0x34865ad1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2665ad5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf65adc, 43 },
    { 0x2ab65add, 93 },
    { 0x7ea65ade, 77 },
    { 0xba65adc, 34 },
    { 0xc6a65ae0, 67 },
    { 0, 0 },
    { 0x1b865ae2, 90 },
    { 0x9765ae3, 61 },
    { 0xebf65ae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f65aed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9865af6, 36 },
    { 0, 0 },
    { 0x3c365af8, 33 },
    { 0, 0 },
    { 0x1665afa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1065afd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83965b01, 33 },
    { 0x6c565b02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b565b05, 23 },
    { 0xd4c65b06, 90 },
    { 0, 0 },
    { 0x3f065b08, 61 },
    { 0x3b865b09, 76 },
    { 0, 0 },
    { 0x10265b0b, 92 },
    { 0x89665b0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba65b11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3465b14, 34 },
    { 0x70765b15, 33 },
    { 0, 0 },
    { 0x63f65b17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb665b1c, 49 },
    { 0xebe65b1c, 61 },
    { 0xbd65b1e, 35 },
    { 0, 0 },
    { 0x2c465b20, 90 },
    { 0xb4d65b20, 24 },
    { 0, 0 },
    { 0x37165b23, 34 },
    { 0x9d665b24, 46 },
    { 0x8fc65b24, 23 },
    { 0xe6665b26, 61 },
    { 0xd2f65b27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2365b2a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5665b2d, 93 },
    { 0xd1665b2d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87965b32, 65 },
    { 0x89c65b33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea365b37, 78 },
    { 0x84165b38, 55 },
    { 0xccb65b39, 43 },
    { 0x7b465b3a, 90 },
    { 0x83565b3b, 69 },
    { 0xe8a65b39, 99 },
    { 0, 0 },
    { 0xa6d65b3e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64765b42, 76 },
    { 0, 0 },
    { 0x17565b44, 23 },
    { 0x7c865b44, 61 },
    { 0x49365b46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d465b4f, 34 },
    { 0xa5965b50, 55 },
    { 0, 0 },
    { 0x80f65b52, 90 },
    { 0x46565b53, 68 },
    { 0x33c65b54, 34 },
    { 0, 0 },
    { 0x95965b56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c65b5a, 90 },
    { 0, 0 },
    { 0x7565b5c, 96 },
    { 0x6bb65b5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde65b69, 33 },
    { 0, 0 },
    { 0xbc965b6b, 78 },
    { 0, 0 },
    { 0x54a65b6d, 76 },
    { 0, 0 },
    { 0xe4f65b6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74265b76, 61 },
    { 0, 0 },
    { 0x80965b78, 34 },
    { 0x29b65b79, 87 },
    { 0, 0 },
    { 0xbc765b7b, 76 },
    { 0x27d65b7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd65b7f, 61 },
    { 0, 0 },
    { 0x14765b81, 94 },
    { 0xa6565b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c65b89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca65b8d, 98 },
    { 0xf2465b8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc265b95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6765b98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90865b9c, 76 },
    { 0x8bd65b9c, 40 },
    { 0x7e965b9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2165ba3, 33 },
    { 0, 0 },
    { 0x98c65ba5, 33 },
    { 0xd0c65ba6, 90 },
    { 0x76565ba6, 90 },
    { 0, 0 },
    { 0xf3d65ba9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a065bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3265bb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x95665bb8, 33 },
    { 0, 0 },
    { 0xc9b65bba, 61 },
    { 0x39265bba, 90 },
    { 0, 0 },
    { 0x79765bbd, 34 },
    { 0, 0 },
    { 0xc9265bbf, 33 },
    { 0x33e65bc0, 99 },
    { 0xad365bc1, 55 },
    { 0x6cc65bc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf865bc7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a865bce, 90 },
    { 0, 0 },
    { 0x1e165bd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a465bda, 80 },
    { 0x20565bdb, 33 },
    { 0x28665bdc, 95 },
    { 0xe1965bdd, 23 },
    { 0x8e365bde, 95 },
    { 0x5d665bde, 98 },
    { 0xc8365be0, 90 },
    { 0x43165bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d65be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x89565be9, 95 },
    { 0x7165bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b465bed, 90 },
    { 0, 0 },
    { 0x15865bef, 93 },
    { 0, 0 },
    { 0x8ff65bf1, 33 },
    { 0x2f465bf2, 95 },
    { 0, 0 },
    { 0xd8b65bf4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d65bf8, 95 },
    { 0xaaa65bf8, 90 },
    { 0x87165bfa, 45 },
    { 0xfa265bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e65c00, 61 },
    { 0, 0 },
    { 0x91565c02, 33 },
    { 0x45b65c03, 98 },
    { 0x15a65c02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4865c07, 61 },
    { 0xe6765c08, 33 },
    { 0xa0d65c09, 95 },
    { 0xf4c65c0a, 95 },
    { 0xfe665c07, 61 },
    { 0x17665c07, 90 },
    { 0xc8665c0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed65c10, 98 },
    { 0, 0 },
    { 0x34165c12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba65c21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de65c24, 48 },
    { 0x55065c24, 61 },
    { 0xb9f65c24, 23 },
    { 0x72d65c27, 25 },
    { 0x69465c28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b65c2b, 69 },
    { 0x8cb65c2c, 90 },
    { 0, 0 },
    { 0x52e65c2e, 23 },
    { 0x10065c2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41065c34, 90 },
    { 0x85965c35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b965c38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b65c3b, 66 },
    { 0x5b65c3b, 85 },
    { 0x62365c3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d865c41, 93 },
    { 0, 0 },
    { 0x82965c43, 33 },
    { 0xc0065c44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f065c47, 77 },
    { 0xfce65c48, 33 },
    { 0x7ef65c49, 51 },
    { 0xf465c4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0465c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a65c54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85165c58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58765c5b, 61 },
    { 0x60e65c5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4265c5f, 93 },
    { 0, 0 },
    { 0x54865c61, 46 },
    { 0x9a065c62, 90 },
    { 0x9ba65c63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61965c68, 55 },
    { 0x8b65c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e865c6d, 75 },
    { 0xacb65c6e, 90 },
    { 0x66e65c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd165c72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8365c79, 23 },
    { 0x3f365c7a, 95 },
    { 0x28565c7b, 95 },
    { 0x89265c7c, 43 },
    { 0x9cf65c7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a965c80, 100 },
    { 0, 0 },
    { 0x1ed65c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db65c85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc65c8b, 33 },
    { 0x22765c8c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5965c8f, 42 },
    { 0x27865c90, 98 },
    { 0xda165c91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79665c95, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e465c9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e765ca3, 33 },
    { 0xd7165ca3, 69 },
    { 0, 0 },
    { 0xca865ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90065ca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b65cac, 95 },
    { 0x2d465cad, 90 },
    { 0x3d665cae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e165cb1, 66 },
    { 0x99a65cb2, 33 },
    { 0x61f65cb2, 53 },
    { 0x70165cb4, 33 },
    { 0x21a65cb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda365cba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2665cbd, 98 },
    { 0x56965cbd, 95 },
    { 0x13665cbf, 90 },
    { 0xd3f65cbe, 43 },
    { 0, 0 },
    { 0xdbc65cc2, 61 },
    { 0xbf565cc3, 76 },
    { 0xa6c65cc3, 80 },
    { 0x17d65cc5, 61 },
    { 0xe9965cc3, 90 },
    { 0xa8c65cc7, 84 },
    { 0x35a65cc8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf65ccb, 99 },
    { 0x7d465ccc, 85 },
    { 0xf6465ccc, 98 },
    { 0, 0 },
    { 0x7f065ccf, 100 },
    { 0xf2765cd0, 80 },
    { 0xda865cd1, 90 },
    { 0, 0 },
    { 0xd9365cd3, 52 },
    { 0, 0 },
    { 0xc1a65cd5, 61 },
    { 0xd5765cd6, 81 },
    { 0xafd65cd5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a365ce0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f65ce5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7665cea, 100 },
    { 0x73165ceb, 43 },
    { 0, 0 },
    { 0x5ad65ced, 43 },
    { 0, 0 },
    { 0x66165cef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49865cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2265cfa, 76 },
    { 0xca265cfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c65cfe, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4965d01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf765d04, 95 },
    { 0x27265d05, 90 },
    { 0x5ae65d06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1965d10, 98 },
    { 0x49665d10, 33 },
    { 0xa7365d12, 90 },
    { 0, 0 },
    { 0x44465d14, 33 },
    { 0x92665d15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f65d1a, 95 },
    { 0, 0 },
    { 0xcbf65d1c, 85 },
    { 0x37e65d1d, 33 },
    { 0x25765d1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35565d22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b65d26, 100 },
    { 0x68365d26, 43 },
    { 0x67065d28, 77 },
    { 0xb2265d29, 98 },
    { 0xc665d2a, 26 },
    { 0x44165d2a, 95 },
    { 0x22365d26, 95 },
    { 0, 0 },
    { 0x26d65d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee65d36, 61 },
    { 0x5b65d36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d365d3a, 41 },
    { 0xe565d3b, 98 },
    { 0x1365d3c, 61 },
    { 0, 0 },
    { 0xe5865d3e, 90 },
    { 0xd0265d3f, 61 },
    { 0, 0 },
    { 0x20f65d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6865d4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e65d4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e365d54, 33 },
    { 0xdc465d54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9565d58, 78 },
    { 0x31865d59, 93 },
    { 0xe1765d59, 88 },
    { 0xffa65d5a, 98 },
    { 0xa5d65d59, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x67165d5f, 23 },
    { 0xfb465d5f, 90 },
    { 0x4a465d60, 43 },
    { 0x92a65d61, 76 },
    { 0xc8865d63, 34 },
    { 0xa9065d63, 90 },
    { 0x6a865d65, 78 },
    { 0, 0 },
    { 0x62465d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37365d6b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d65d73, 75 },
    { 0x62365d74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81865d79, 90 },
    { 0, 0 },
    { 0xbe865d7b, 33 },
    { 0, 0 },
    { 0xaf065d7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90865d86, 98 },
    { 0, 0 },
    { 0x43965d88, 61 },
    { 0xd7365d88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97965d96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc65d9b, 87 },
    { 0, 0 },
    { 0x79a65d9d, 23 },
    { 0xa5765d9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c65da2, 55 },
    { 0x2d665da3, 96 },
    { 0xe5065da4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44265da8, 61 },
    { 0xab65da9, 34 },
    { 0, 0 },
    { 0x26f65dab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b365dae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7765db2, 69 },
    { 0xacc65db3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a265db6, 99 },
    { 0, 0 },
    { 0xb0065db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c65dbd, 80 },
    { 0, 0 },
    { 0xe9265dbf, 23 },
    { 0, 0 },
    { 0xa1e65dc1, 33 },
    { 0x4c665dc2, 78 },
    { 0xff265dc1, 88 },
    { 0x67665dc4, 33 },
    { 0x6f965dc5, 95 },
    { 0x61065dc4, 96 },
    { 0x8d665dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b865dca, 90 },
    { 0xf8965dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa165dd2, 90 },
    { 0, 0 },
    { 0xa7565dd4, 90 },
    { 0x4e565dd4, 90 },
    { 0x39065dd6, 40 },
    { 0x94365dd5, 40 },
    { 0xbd65dd8, 33 },
    { 0x3865dd9, 90 },
    { 0x9d665dd8, 23 },
    { 0x2c065ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6065ddf, 90 },
    { 0x4e865de0, 61 },
    { 0, 0 },
    { 0xd9a65de2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c65de5, 91 },
    { 0xf8565de5, 61 },
    { 0xc5765de7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac865df1, 43 },
    { 0x29e65df1, 61 },
    { 0, 0 },
    { 0x99665df4, 33 },
    { 0, 0 },
    { 0x1b065df6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d165df9, 84 },
    { 0xa5465df9, 61 },
    { 0, 0 },
    { 0xda165dfc, 98 },
    { 0x35865dfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78865e01, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f165e04, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37065e08, 33 },
    { 0x84d65e09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9265e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d65e14, 95 },
    { 0xa4065e15, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d65e18, 46 },
    { 0x72565e19, 78 },
    { 0, 0 },
    { 0x39465e1b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13665e1e, 24 },
    { 0xf3f65e1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d365e22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a965e27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a65e2c, 90 },
    { 0x6f765e2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9965e32, 34 },
    { 0xdcd65e32, 80 },
    { 0x25e65e34, 90 },
    { 0xd0c65e34, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f65e3a, 34 },
    { 0x42165e3a, 98 },
    { 0, 0 },
    { 0x43465e3d, 46 },
    { 0x6c165e3e, 90 },
    { 0x63065e3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65e4b, 23 },
    { 0xc9f65e4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4065e51, 100 },
    { 0, 0 },
    { 0xdb365e53, 90 },
    { 0xb3c65e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6265e57, 91 },
    { 0xb4f65e58, 33 },
    { 0xc1565e57, 90 },
    { 0, 0 },
    { 0x53665e5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e965e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda65e65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16365e68, 93 },
    { 0xbc65e69, 98 },
    { 0xa2b65e6a, 95 },
    { 0x9a165e6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad65e6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89365e74, 43 },
    { 0x17965e75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a65e78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b65e7c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda765e7f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8865e83, 90 },
    { 0x3df65e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e65e8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b265e90, 36 },
    { 0, 0 },
    { 0x75265e92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad65e96, 76 },
    { 0x50d65e96, 67 },
    { 0xefe65e98, 98 },
    { 0, 0 },
    { 0x50265e9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f865ea6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc65ea9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf365eac, 98 },
    { 0x48965ead, 33 },
    { 0xfc765ead, 34 },
    { 0xe5b65eaf, 98 },
    { 0x70365eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d65eb3, 90 },
    { 0xc7b65eb4, 90 },
    { 0xfa465eb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee65eb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b165ebb, 61 },
    { 0x34365ebc, 87 },
    { 0, 0 },
    { 0x46965ebe, 55 },
    { 0xc665ebf, 61 },
    { 0, 0 },
    { 0x9a565ec1, 67 },
    { 0x3ae65ec2, 98 },
    { 0xbc665ec3, 98 },
    { 0x83165ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f65ec9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64065ed3, 33 },
    { 0x18365ed4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d65ed7, 93 },
    { 0, 0 },
    { 0x2765ed9, 34 },
    { 0x99a65ed9, 53 },
    { 0x71f65edb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c965ee1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6165ee5, 76 },
    { 0x1e265ee5, 83 },
    { 0xa7265ee7, 61 },
    { 0, 0 },
    { 0xbe565ee9, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd465eed, 66 },
    { 0x18465eee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4165efb, 81 },
    { 0, 0 },
    { 0xe1765efd, 90 },
    { 0x54965efe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40665f05, 86 },
    { 0x7a65f06, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3765f0a, 98 },
    { 0, 0 },
    { 0x9cf65f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82765f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2965f21, 76 },
    { 0xf1e65f22, 76 },
    { 0, 0 },
    { 0xc2465f24, 61 },
    { 0xc5665f25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d65f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb65f30, 61 },
    { 0xd9d65f31, 61 },
    { 0x58b65f32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6065f39, 90 },
    { 0x2f265f39, 61 },
    { 0x31f65f3b, 33 },
    { 0x82665f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b65f42, 46 },
    { 0x5bb65f43, 90 },
    { 0xfd765f44, 95 },
    { 0x75165f45, 33 },
    { 0x76a65f44, 88 },
    { 0x9b65f43, 90 },
    { 0xb7665f44, 55 },
    { 0x6d465f46, 43 },
    { 0, 0 },
    { 0xbc465f4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c65f50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c165f55, 52 },
    { 0x1c65f55, 46 },
    { 0x9a865f57, 64 },
    { 0x8a965f55, 23 },
    { 0x24865f57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27065f5c, 61 },
    { 0x59965f5d, 90 },
    { 0x45e65f5c, 90 },
    { 0x20f65f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb65f6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d65f6e, 95 },
    { 0, 0 },
    { 0x31c65f70, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f65f74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7565f78, 90 },
    { 0x45e65f78, 99 },
    { 0x78065f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42765f7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a65f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76965f88, 90 },
    { 0xbbf65f88, 90 },
    { 0, 0 },
    { 0xc5665f8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9565f92, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d065f95, 23 },
    { 0xb8965f96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b65f9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5065fa1, 33 },
    { 0x8c865fa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11565fa7, 43 },
    { 0x6bb65fa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a65fab, 61 },
    { 0x1b165fac, 90 },
    { 0x9db65fac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c565fb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4665fb5, 38 },
    { 0x1cb65fb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c765fbb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae65fc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7565fc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d865fca, 95 },
    { 0x4bc65fcb, 61 },
    { 0x36d65fcb, 98 },
    { 0xc8765fcb, 33 },
    { 0x6bd65fce, 76 },
    { 0xf9665fce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7565fd7, 95 },
    { 0x59065fd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6665fdd, 33 },
    { 0x73c65fde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc165fe2, 90 },
    { 0, 0 },
    { 0xbaf65fe4, 76 },
    { 0, 0 },
    { 0xfba65fe6, 90 },
    { 0, 0 },
    { 0x60665fe8, 33 },
    { 0xbcf65fe9, 76 },
    { 0xa8f65fe9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1865fed, 98 },
    { 0xa8265fee, 46 },
    { 0, 0 },
    { 0x67a65ff0, 52 },
    { 0xe1465ff0, 17 },
    { 0x38465ff2, 61 },
    { 0x97e65ff3, 96 },
    { 0, 0 },
    { 0xdca65ff5, 67 },
    { 0xb4565ff6, 95 },
    { 0x26665ff7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df66002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91266005, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4366008, 95 },
    { 0xa3966009, 97 },
    { 0x17d66009, 61 },
    { 0x1236600b, 98 },
    { 0xb766009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f666012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c6601b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e66021, 90 },
    { 0xea866022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e6602d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c66031, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a66035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46766039, 33 },
    { 0x7816603a, 61 },
    { 0x2ff6603b, 33 },
    { 0xfe66603c, 100 },
    { 0xc526603d, 90 },
    { 0x6786603d, 90 },
    { 0xd976603d, 98 },
    { 0x61866040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1366046, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066049, 34 },
    { 0, 0 },
    { 0xaae6604b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d56604e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20266055, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee66059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7736605f, 55 },
    { 0x3ba66060, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26566064, 23 },
    { 0, 0 },
    { 0xaf366066, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a866069, 90 },
    { 0x43366069, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf646606d, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb66072, 87 },
    { 0, 0 },
    { 0x4c66074, 33 },
    { 0, 0 },
    { 0xffa66076, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c66079, 95 },
    { 0xb046607a, 90 },
    { 0x3166607b, 86 },
    { 0, 0 },
    { 0xb426607d, 23 },
    { 0, 0 },
    { 0x6a6607f, 61 },
    { 0, 0 },
    { 0xf6a66081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56566087, 33 },
    { 0xdf166087, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76608b, 23 },
    { 0, 0 },
    { 0x5b26608d, 97 },
    { 0, 0 },
    { 0x3cc6608f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a66092, 90 },
    { 0xacc66093, 61 },
    { 0, 0 },
    { 0xea766095, 65 },
    { 0xb5866096, 33 },
    { 0xe3a66097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb6609d, 77 },
    { 0, 0 },
    { 0x116609f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed660a9, 95 },
    { 0xd73660a9, 90 },
    { 0x90e660aa, 52 },
    { 0x4fc660ac, 46 },
    { 0x645660a9, 87 },
    { 0x788660a9, 84 },
    { 0xabc660af, 98 },
    { 0x58d660b0, 90 },
    { 0xb42660af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73660b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x625660bc, 90 },
    { 0xfa3660bd, 95 },
    { 0x157660be, 33 },
    { 0x8f660bc, 90 },
    { 0xd2d660bd, 76 },
    { 0x562660c1, 76 },
    { 0xccb660c2, 61 },
    { 0x5ca660c3, 53 },
    { 0x376660c4, 84 },
    { 0x568660c5, 23 },
    { 0x947660c5, 88 },
    { 0, 0 },
    { 0xdb9660c8, 90 },
    { 0xa9a660c9, 81 },
    { 0x99f660ca, 100 },
    { 0, 0 },
    { 0xb34660cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c660cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x350660d2, 74 },
    { 0xa8d660d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5660da, 55 },
    { 0x765660db, 33 },
    { 0x3be660db, 46 },
    { 0xb0d660dd, 52 },
    { 0xdd0660de, 93 },
    { 0x70e660df, 98 },
    { 0, 0 },
    { 0xfe660e1, 95 },
    { 0, 0 },
    { 0xcab660e3, 55 },
    { 0x5d4660e4, 78 },
    { 0x204660e4, 98 },
    { 0x30e660e6, 95 },
    { 0xdaf660e6, 43 },
    { 0x943660e8, 23 },
    { 0xba7660e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86660ef, 23 },
    { 0, 0 },
    { 0x91e660f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701660f7, 90 },
    { 0, 0 },
    { 0x1d660f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a660fd, 99 },
    { 0x5d5660fe, 95 },
    { 0xf2e660ff, 51 },
    { 0, 0 },
    { 0x30266101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c6610c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8386610f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca566112, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc966116, 61 },
    { 0xee566117, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb6611a, 95 },
    { 0xf246611b, 90 },
    { 0, 0 },
    { 0x3e36611d, 23 },
    { 0x1946611e, 95 },
    { 0x6996611d, 95 },
    { 0x2da66120, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a66123, 33 },
    { 0, 0 },
    { 0x88a66125, 66 },
    { 0x8b166126, 33 },
    { 0xcba66127, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6612b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8766131, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf166137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106613b, 23 },
    { 0x1f76613c, 23 },
    { 0, 0 },
    { 0x3496613e, 101 },
    { 0x93e6613f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb666149, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6614c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb66150, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49066153, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59666159, 52 },
    { 0xc266159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4666164, 76 },
    { 0x2f366165, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe866168, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f6616c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26966175, 87 },
    { 0xcd066176, 43 },
    { 0xad466175, 95 },
    { 0, 0 },
    { 0xcbe66179, 90 },
    { 0xdd166179, 76 },
    { 0xe286617a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f66182, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6618a, 61 },
    { 0xbc86618b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5606618e, 66 },
    { 0x34a6618e, 76 },
    { 0xb9866190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe86619c, 90 },
    { 0xc0b6619c, 34 },
    { 0xa006619e, 81 },
    { 0x1a96619d, 46 },
    { 0, 0 },
    { 0x9a3661a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4661a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f661ac, 90 },
    { 0x72c661ad, 34 },
    { 0xf98661ae, 61 },
    { 0x596661af, 33 },
    { 0x4a661b0, 76 },
    { 0x506661af, 23 },
    { 0, 0 },
    { 0x9b661b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc661b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720661ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef661bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81661c3, 88 },
    { 0xf9b661c3, 84 },
    { 0, 0 },
    { 0xb7c661c6, 33 },
    { 0x81f661c7, 85 },
    { 0, 0 },
    { 0xe6e661c9, 46 },
    { 0x4a9661ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe661cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a661d1, 76 },
    { 0xf7a661d2, 80 },
    { 0xe0a661d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7661d7, 61 },
    { 0xab0661d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x677661db, 61 },
    { 0xe98661dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4661df, 52 },
    { 0, 0 },
    { 0x4a8661e1, 90 },
    { 0x50f661e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6661e9, 93 },
    { 0, 0 },
    { 0xff4661eb, 99 },
    { 0, 0 },
    { 0x9f3661ed, 52 },
    { 0, 0 },
    { 0x94b661ef, 76 },
    { 0x7cc661f0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4661f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f661f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7661fb, 98 },
    { 0, 0 },
    { 0xfec661fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1966204, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d6620b, 98 },
    { 0xb956620c, 23 },
    { 0, 0 },
    { 0x7396620e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x82266211, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8866218, 33 },
    { 0x3a466219, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f96621e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0366229, 75 },
    { 0xf346622a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf6622d, 81 },
    { 0xf076622e, 95 },
    { 0, 0 },
    { 0xe7066230, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8066235, 76 },
    { 0x7bd66236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce46623a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e6623f, 33 },
    { 0x26066240, 76 },
    { 0xcdd6623f, 34 },
    { 0xb8866241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f66247, 61 },
    { 0xc8a66248, 95 },
    { 0, 0 },
    { 0xad86624a, 33 },
    { 0x9be6624b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69666252, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda266255, 90 },
    { 0, 0 },
    { 0xf3b66257, 98 },
    { 0x7e166258, 61 },
    { 0x90766259, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a6625e, 43 },
    { 0x8db6625f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5266263, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4166267, 52 },
    { 0xc666268, 90 },
    { 0xe9366269, 61 },
    { 0xeae66267, 96 },
    { 0xc3666268, 87 },
    { 0, 0 },
    { 0x8226626d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x19866270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb866273, 66 },
    { 0xbe966273, 77 },
    { 0x9a666275, 34 },
    { 0xb2a66275, 61 },
    { 0xd566276, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9916627c, 33 },
    { 0, 0 },
    { 0xe856627e, 23 },
    { 0x3806627f, 91 },
    { 0, 0 },
    { 0xb5a66281, 95 },
    { 0xb6c66281, 88 },
    { 0x10866283, 87 },
    { 0, 0 },
    { 0xd2d66285, 61 },
    { 0x3ae66286, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566628c, 67 },
    { 0, 0 },
    { 0x66f6628e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c66295, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91966298, 66 },
    { 0xe6266299, 23 },
    { 0x1946629a, 61 },
    { 0xde96629b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6629e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9662a2, 43 },
    { 0x2f3662a3, 66 },
    { 0x6ae662a4, 23 },
    { 0x2c9662a5, 100 },
    { 0, 0 },
    { 0x614662a7, 90 },
    { 0x178662a7, 34 },
    { 0x69f662a8, 68 },
    { 0x9f3662aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4662b0, 95 },
    { 0xd29662b1, 61 },
    { 0xce0662b2, 95 },
    { 0xc1662b3, 76 },
    { 0x81662b4, 23 },
    { 0xbfc662b5, 23 },
    { 0x7f1662b3, 23 },
    { 0, 0 },
    { 0xd87662b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac662bf, 46 },
    { 0x17a662c0, 34 },
    { 0x256662c1, 43 },
    { 0xcb5662c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc662c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4662cc, 33 },
    { 0xe40662cd, 77 },
    { 0, 0 },
    { 0xdef662cf, 55 },
    { 0xd45662d0, 98 },
    { 0x7ba662cf, 95 },
    { 0x1f4662d2, 76 },
    { 0xc79662d2, 61 },
    { 0x457662d1, 43 },
    { 0xe72662d5, 90 },
    { 0x85c662d6, 100 },
    { 0xdff662d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84662db, 98 },
    { 0x52b662dc, 33 },
    { 0, 0 },
    { 0x746662de, 95 },
    { 0x64d662df, 23 },
    { 0, 0 },
    { 0x713662e1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19662e7, 40 },
    { 0, 0 },
    { 0xd6a662e9, 95 },
    { 0, 0 },
    { 0x22e662eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc662f9, 93 },
    { 0, 0 },
    { 0x3b8662fb, 95 },
    { 0x30b662fc, 76 },
    { 0x869662fb, 96 },
    { 0, 0 },
    { 0xe6f662ff, 61 },
    { 0x99e66300, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71066305, 90 },
    { 0x78266306, 90 },
    { 0x9be66307, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6976630f, 96 },
    { 0x14166310, 98 },
    { 0, 0 },
    { 0x70766312, 33 },
    { 0x8f566312, 61 },
    { 0xb9866314, 90 },
    { 0x22c66315, 34 },
    { 0x95c66312, 95 },
    { 0xe4b66317, 95 },
    { 0, 0 },
    { 0xae966319, 33 },
    { 0x83e6631a, 95 },
    { 0x3ca6631a, 90 },
    { 0x4326631a, 77 },
    { 0x19566319, 98 },
    { 0x6ed6631e, 88 },
    { 0, 0 },
    { 0xeb666320, 98 },
    { 0, 0 },
    { 0x4fc66322, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f66326, 23 },
    { 0, 0 },
    { 0x96f66328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a6632b, 55 },
    { 0xf966632b, 33 },
    { 0x5516632c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46266336, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e6633a, 36 },
    { 0, 0 },
    { 0x2e26633c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c166342, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d66348, 90 },
    { 0x19166349, 98 },
    { 0x3b46634a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de66351, 61 },
    { 0xb4466352, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf166356, 61 },
    { 0x50366357, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1746635a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d26635d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f66361, 23 },
    { 0, 0 },
    { 0x64566363, 91 },
    { 0x30d66364, 98 },
    { 0x51f66364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5596636a, 91 },
    { 0x3ad6636b, 90 },
    { 0x2cc6636b, 67 },
    { 0x9ff6636c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e66373, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d166379, 98 },
    { 0xcaf66379, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d6637f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89566385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf566392, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf386639b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd666639e, 65 },
    { 0, 0 },
    { 0x7d9663a0, 78 },
    { 0, 0 },
    { 0x84a663a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b663a7, 90 },
    { 0xfc1663a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb663ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43663b0, 23 },
    { 0x5b1663b1, 34 },
    { 0x369663b2, 100 },
    { 0xfa9663b0, 17 },
    { 0x766663b4, 98 },
    { 0xea1663b0, 61 },
    { 0x59e663b6, 98 },
    { 0x569663b7, 95 },
    { 0xd4b663b7, 95 },
    { 0, 0 },
    { 0xcca663ba, 8 },
    { 0x159663bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5663c3, 95 },
    { 0x80c663c3, 36 },
    { 0, 0 },
    { 0x30d663c6, 98 },
    { 0, 0 },
    { 0x2fd663c8, 76 },
    { 0x710663c9, 46 },
    { 0, 0 },
    { 0x7f2663cb, 46 },
    { 0x16a663cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438663d3, 76 },
    { 0, 0 },
    { 0x54a663d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6663e5, 65 },
    { 0xe9a663e6, 90 },
    { 0xe70663e7, 98 },
    { 0, 0 },
    { 0x4aa663e9, 95 },
    { 0xfe3663e9, 43 },
    { 0xf92663eb, 78 },
    { 0x8f3663eb, 76 },
    { 0xf28663ec, 61 },
    { 0x4b5663ee, 90 },
    { 0xd36663ef, 98 },
    { 0xbb7663ef, 34 },
    { 0x9d663ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4663f4, 95 },
    { 0, 0 },
    { 0xf09663f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0663fa, 90 },
    { 0x3c6663fa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76866403, 98 },
    { 0, 0 },
    { 0x2166405, 43 },
    { 0x30466406, 90 },
    { 0x13366407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b6640e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x81766411, 34 },
    { 0xf9e66412, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff66415, 33 },
    { 0, 0 },
    { 0x25b66417, 76 },
    { 0x16766417, 99 },
    { 0, 0 },
    { 0xf676641a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49266421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76366424, 95 },
    { 0x45866424, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b6642c, 95 },
    { 0x4556642c, 43 },
    { 0x5026642e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b566431, 90 },
    { 0, 0 },
    { 0x26a66433, 23 },
    { 0x2be66434, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e766441, 98 },
    { 0xdbd66442, 98 },
    { 0xfb666443, 90 },
    { 0xc9366444, 90 },
    { 0x86466445, 99 },
    { 0, 0 },
    { 0x51666447, 61 },
    { 0xfbf66448, 68 },
    { 0x78d66447, 34 },
    { 0xbdf6644a, 98 },
    { 0x5a466448, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e66454, 95 },
    { 0xb4266454, 61 },
    { 0, 0 },
    { 0xf1366457, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda96645c, 17 },
    { 0x5476645d, 34 },
    { 0xd96645d, 38 },
    { 0, 0 },
    { 0xed266460, 75 },
    { 0, 0 },
    { 0xe9166462, 67 },
    { 0xf6066463, 23 },
    { 0, 0 },
    { 0xdfc66465, 100 },
    { 0x8ad66465, 90 },
    { 0, 0 },
    { 0x9a066468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846646e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d66471, 95 },
    { 0x89366472, 33 },
    { 0x12666472, 33 },
    { 0x71566474, 99 },
    { 0, 0 },
    { 0xb8a66476, 95 },
    { 0x96466477, 87 },
    { 0xa8266478, 84 },
    { 0xede66476, 100 },
    { 0x8f866477, 76 },
    { 0x3ae6647b, 38 },
    { 0x3ea66477, 95 },
    { 0, 0 },
    { 0xbcf6647e, 88 },
    { 0, 0 },
    { 0xd3e66480, 76 },
    { 0xb3f66481, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b366484, 95 },
    { 0, 0 },
    { 0x1a66486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de6648b, 96 },
    { 0x4926648c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x756648f, 78 },
    { 0x1a266490, 99 },
    { 0x64666490, 76 },
    { 0x39c66490, 98 },
    { 0, 0 },
    { 0x74566494, 98 },
    { 0xa7166494, 90 },
    { 0xd7566496, 23 },
    { 0x15d66494, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe606649d, 61 },
    { 0, 0 },
    { 0x8236649f, 95 },
    { 0xaab6649f, 90 },
    { 0xd29664a0, 43 },
    { 0, 0 },
    { 0x4c664a3, 77 },
    { 0xf9d664a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5664a8, 95 },
    { 0xb0c664a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b664ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed664b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a664b5, 61 },
    { 0x308664b5, 90 },
    { 0, 0 },
    { 0x974664b8, 98 },
    { 0x6e8664b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3664bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1664bf, 61 },
    { 0xd47664c0, 33 },
    { 0xca8664c1, 76 },
    { 0xee9664c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x557664c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab664cc, 96 },
    { 0, 0 },
    { 0x986664ce, 23 },
    { 0xd664cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43664d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91664d9, 76 },
    { 0x438664da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a664de, 98 },
    { 0x8ee664de, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0664e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430664e6, 61 },
    { 0xa61664e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd664ea, 82 },
    { 0xfd8664ea, 34 },
    { 0xa84664ec, 61 },
    { 0x77c664ed, 8 },
    { 0xff2664ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29664f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e664f4, 95 },
    { 0, 0 },
    { 0x60b664f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54664f9, 55 },
    { 0x659664fa, 33 },
    { 0x7b5664fb, 65 },
    { 0x422664fc, 55 },
    { 0x86b664fb, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b6650a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99766515, 37 },
    { 0x18f66516, 91 },
    { 0, 0 },
    { 0xc8266518, 23 },
    { 0x31e66519, 61 },
    { 0xf876651a, 90 },
    { 0, 0 },
    { 0x1f46651c, 52 },
    { 0x3306651c, 23 },
    { 0xaf16651d, 43 },
    { 0xb5b6651f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef66526, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83866529, 85 },
    { 0x2196652a, 87 },
    { 0x10966529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8466533, 98 },
    { 0x25b66533, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5776653d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17266540, 80 },
    { 0x81466541, 77 },
    { 0, 0 },
    { 0xf9866543, 33 },
    { 0x8cd66544, 34 },
    { 0x4b466544, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc36654a, 98 },
    { 0, 0 },
    { 0x476654c, 33 },
    { 0x5666654d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a866550, 98 },
    { 0, 0 },
    { 0xfe466552, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46366555, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae66560, 61 },
    { 0x70d66561, 23 },
    { 0, 0 },
    { 0x66e66563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e66566, 81 },
    { 0xd1b66566, 43 },
    { 0xcf66568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe66570, 34 },
    { 0, 0 },
    { 0x4e66572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18366575, 66 },
    { 0xa8066576, 90 },
    { 0xdf766576, 33 },
    { 0, 0 },
    { 0x2366579, 46 },
    { 0x8f36657a, 98 },
    { 0x46266579, 90 },
    { 0xa2b6657c, 33 },
    { 0xd086657b, 95 },
    { 0xd8866579, 55 },
    { 0x3876657f, 98 },
    { 0xf2d6657f, 100 },
    { 0x3c56657c, 43 },
    { 0x1de66582, 61 },
    { 0x29c66582, 82 },
    { 0x4b266584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca166587, 93 },
    { 0xac466588, 75 },
    { 0xb8666589, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5346658f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a66594, 46 },
    { 0xf9666595, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25665a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914665a8, 98 },
    { 0xb3665a9, 55 },
    { 0xe70665aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b665b0, 76 },
    { 0x1b4665b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0665b5, 23 },
    { 0xd59665b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0665b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2665bc, 61 },
    { 0x5ca665bd, 91 },
    { 0x846665be, 61 },
    { 0, 0 },
    { 0x8bd665c0, 87 },
    { 0, 0 },
    { 0xa14665c2, 100 },
    { 0x3fd665c3, 95 },
    { 0x25a665c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47665c7, 90 },
    { 0x465665c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46665cb, 100 },
    { 0x279665cc, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd665d0, 66 },
    { 0x47d665d1, 90 },
    { 0xf55665d0, 33 },
    { 0x31a665d3, 55 },
    { 0, 0 },
    { 0x8c5665d5, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886665da, 100 },
    { 0, 0 },
    { 0x5ce665dc, 61 },
    { 0xf0e665dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300665e1, 98 },
    { 0x63e665e2, 33 },
    { 0x194665e2, 77 },
    { 0x1e1665e4, 87 },
    { 0x98e665e2, 23 },
    { 0xcc4665e6, 33 },
    { 0xa83665e7, 90 },
    { 0x2f6665e2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b665ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a665f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640665fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2665ff, 76 },
    { 0xc5166600, 52 },
    { 0x53d665ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc566604, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb636660b, 33 },
    { 0xae46660b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0966612, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a66615, 66 },
    { 0, 0 },
    { 0x2366617, 87 },
    { 0, 0 },
    { 0x70f66619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd636661c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73666623, 98 },
    { 0x6f666624, 76 },
    { 0x99e66624, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b666628, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16662b, 80 },
    { 0xc2d6662c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e66632, 46 },
    { 0x34d66633, 12 },
    { 0, 0 },
    { 0x9bf66635, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18666639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f96663d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d66640, 60 },
    { 0x2c166641, 88 },
    { 0xd7366641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec66645, 100 },
    { 0, 0 },
    { 0xf4b66647, 23 },
    { 0, 0 },
    { 0x93d66649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb396664c, 90 },
    { 0, 0 },
    { 0xcc86664e, 55 },
    { 0x1686664f, 55 },
    { 0xf216664f, 90 },
    { 0x57366651, 100 },
    { 0, 0 },
    { 0x18966653, 95 },
    { 0xde766653, 95 },
    { 0, 0 },
    { 0xf1b66656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd6665e, 33 },
    { 0xbe06665f, 55 },
    { 0, 0 },
    { 0x91266661, 91 },
    { 0, 0 },
    { 0xfd566663, 61 },
    { 0x81766663, 93 },
    { 0xfc066665, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe746666d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8166678, 66 },
    { 0xc1166679, 34 },
    { 0x1336667a, 90 },
    { 0x20666678, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf66667f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b66682, 98 },
    { 0, 0 },
    { 0x28c66684, 55 },
    { 0x4e066684, 90 },
    { 0xe3e66686, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8346668c, 33 },
    { 0x3256668d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd66690, 80 },
    { 0, 0 },
    { 0x1d466692, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b6669a, 65 },
    { 0, 0 },
    { 0xba96669c, 90 },
    { 0, 0 },
    { 0x4036669e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6666a1, 33 },
    { 0, 0 },
    { 0x75c666a3, 100 },
    { 0, 0 },
    { 0x606666a5, 90 },
    { 0xb2f666a6, 33 },
    { 0xf71666a7, 23 },
    { 0x10a666a7, 55 },
    { 0xf77666a7, 65 },
    { 0x25d666aa, 87 },
    { 0xc38666a9, 90 },
    { 0, 0 },
    { 0xa54666ad, 23 },
    { 0x49f666ad, 90 },
    { 0, 0 },
    { 0x9d6666b0, 83 },
    { 0x8b0666b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x419666b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc666ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27666be, 76 },
    { 0xf51666bf, 52 },
    { 0, 0 },
    { 0xc74666c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3666c6, 88 },
    { 0xf5f666c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc666ca, 67 },
    { 0x7aa666ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2666ce, 90 },
    { 0, 0 },
    { 0xf30666d0, 55 },
    { 0, 0 },
    { 0x608666d2, 61 },
    { 0, 0 },
    { 0x6da666d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe666da, 46 },
    { 0xe2b666db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b666e1, 88 },
    { 0x7eb666e2, 61 },
    { 0xe0e666e3, 76 },
    { 0x879666e2, 81 },
    { 0xf48666e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568666ea, 90 },
    { 0xf59666eb, 83 },
    { 0xd3c666ea, 61 },
    { 0, 0 },
    { 0x922666ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61666f6, 43 },
    { 0, 0 },
    { 0xbab666f8, 52 },
    { 0, 0 },
    { 0xf32666fa, 93 },
    { 0x4ed666fa, 99 },
    { 0x161666fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef66702, 33 },
    { 0xbed66703, 66 },
    { 0xc4066702, 98 },
    { 0xc0a66703, 99 },
    { 0xd6866706, 98 },
    { 0x8666705, 91 },
    { 0x94866704, 98 },
    { 0, 0 },
    { 0x31d6670a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2976670e, 90 },
    { 0x4746670f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a66712, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86671e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c866723, 85 },
    { 0x55466724, 34 },
    { 0x5ae66725, 61 },
    { 0xd6766726, 98 },
    { 0xd6666727, 61 },
    { 0x5e466728, 90 },
    { 0x45b66728, 90 },
    { 0x7f566725, 17 },
    { 0x9d56672b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b6672e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a266731, 23 },
    { 0x86c66731, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57166735, 52 },
    { 0, 0 },
    { 0x45d66737, 33 },
    { 0, 0 },
    { 0x29866739, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe696673d, 61 },
    { 0x4206673e, 66 },
    { 0x4eb6673f, 99 },
    { 0xb3b66740, 95 },
    { 0xf7a66741, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e366745, 61 },
    { 0x5d166745, 34 },
    { 0x4ac66746, 23 },
    { 0, 0 },
    { 0xf8c66749, 43 },
    { 0x1d166749, 84 },
    { 0xe6d66749, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f76674e, 46 },
    { 0, 0 },
    { 0xc9266750, 17 },
    { 0x26d66751, 67 },
    { 0x1c166751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad66755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe566758, 46 },
    { 0x5d666758, 90 },
    { 0xe406675a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d6675d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb566761, 90 },
    { 0xfe166761, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa466765, 93 },
    { 0, 0 },
    { 0x43266767, 61 },
    { 0xa1666768, 43 },
    { 0x91066768, 98 },
    { 0, 0 },
    { 0x65f6676b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66676e, 99 },
    { 0x4b16676f, 55 },
    { 0xc2e6676e, 90 },
    { 0x7a366771, 85 },
    { 0x70866771, 76 },
    { 0x42e66773, 61 },
    { 0xc9f66774, 98 },
    { 0x3c666775, 46 },
    { 0x63066775, 61 },
    { 0xde766776, 76 },
    { 0x95966775, 61 },
    { 0x1b466779, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e6677e, 61 },
    { 0x9a56677f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e66782, 43 },
    { 0x6e466782, 94 },
    { 0x20466784, 98 },
    { 0x35366784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe366788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9986678c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac66791, 77 },
    { 0xfc666791, 77 },
    { 0x5f466793, 66 },
    { 0x35c66793, 23 },
    { 0x73266795, 61 },
    { 0xd3666791, 98 },
    { 0x38166797, 81 },
    { 0x42366798, 23 },
    { 0x12266799, 52 },
    { 0x3a86679a, 98 },
    { 0x8836679b, 98 },
    { 0x7e26679a, 61 },
    { 0xdb566793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30667a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7667aa, 66 },
    { 0xbd7667aa, 98 },
    { 0xe1c667ac, 100 },
    { 0, 0 },
    { 0xdcf667ae, 91 },
    { 0xfed667af, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736667b3, 55 },
    { 0xeb3667b4, 52 },
    { 0xed7667b3, 55 },
    { 0x56e667b6, 34 },
    { 0xe6e667b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff667bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd667c3, 84 },
    { 0xf6f667c3, 23 },
    { 0xcfc667c5, 33 },
    { 0x95a667c6, 90 },
    { 0xd33667c6, 33 },
    { 0x38c667c8, 95 },
    { 0, 0 },
    { 0x7f4667ca, 82 },
    { 0x79e667cb, 33 },
    { 0xd5a667cb, 23 },
    { 0x856667cc, 61 },
    { 0xa90667cb, 64 },
    { 0, 0 },
    { 0x3e5667d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e667d3, 93 },
    { 0x1e4667d4, 46 },
    { 0xf74667d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5667d8, 33 },
    { 0xafe667d8, 80 },
    { 0, 0 },
    { 0x838667db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6667de, 90 },
    { 0x70e667df, 98 },
    { 0, 0 },
    { 0x6fd667e1, 91 },
    { 0xa37667e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64667e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff667ec, 98 },
    { 0x197667ec, 84 },
    { 0x9e1667ee, 53 },
    { 0x2a3667ef, 55 },
    { 0xf7c667f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b667f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2667f9, 91 },
    { 0x761667fa, 100 },
    { 0xd68667fb, 98 },
    { 0x692667fa, 98 },
    { 0xb89667fb, 61 },
    { 0xeb4667fb, 67 },
    { 0, 0 },
    { 0xea066800, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca66803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a66808, 93 },
    { 0xe8266809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b26680d, 23 },
    { 0x3ed6680d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed66812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc66816, 23 },
    { 0x9b766816, 90 },
    { 0x30f66817, 87 },
    { 0xbe66817, 61 },
    { 0, 0 },
    { 0x8d86681b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb6681f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda166824, 23 },
    { 0x69866825, 85 },
    { 0x47e66824, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xef866829, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4936682d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce66833, 100 },
    { 0x8d566833, 90 },
    { 0x89266835, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51266844, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e66848, 51 },
    { 0, 0 },
    { 0xdf76684a, 81 },
    { 0xc8f6684a, 61 },
    { 0x1f96684c, 61 },
    { 0xadf6684d, 90 },
    { 0x8f36684a, 33 },
    { 0xc476684f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c766852, 23 },
    { 0xbde66853, 23 },
    { 0, 0 },
    { 0x48c66855, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a166860, 43 },
    { 0, 0 },
    { 0x33166862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49166866, 91 },
    { 0, 0 },
    { 0x9e366868, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e6686e, 61 },
    { 0xc876686f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e66875, 91 },
    { 0x66c66876, 90 },
    { 0, 0 },
    { 0xffa66878, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa386687c, 36 },
    { 0, 0 },
    { 0xd906687e, 61 },
    { 0xdfc6687e, 61 },
    { 0x46666880, 76 },
    { 0xae866880, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f6688f, 66 },
    { 0, 0 },
    { 0x56f66891, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b866896, 61 },
    { 0x4f866897, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb346689a, 98 },
    { 0xa3d6689a, 46 },
    { 0, 0 },
    { 0x72e6689d, 66 },
    { 0xbed6689e, 90 },
    { 0x18f6689f, 93 },
    { 0xf5a668a0, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec668af, 90 },
    { 0xbc2668af, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5668b3, 33 },
    { 0, 0 },
    { 0xc76668b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375668bd, 98 },
    { 0x72d668be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1668c3, 76 },
    { 0x533668c3, 76 },
    { 0xa8c668c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9668c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e668d1, 90 },
    { 0x174668d2, 33 },
    { 0x5d2668d2, 69 },
    { 0x1a668d4, 95 },
    { 0x472668d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80668e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f668e4, 65 },
    { 0, 0 },
    { 0x94668e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912668ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd668ee, 55 },
    { 0xac3668ef, 55 },
    { 0, 0 },
    { 0x913668f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4668fc, 23 },
    { 0, 0 },
    { 0xbd9668fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f466902, 95 },
    { 0, 0 },
    { 0x3e166904, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead6690c, 90 },
    { 0x7566690d, 43 },
    { 0, 0 },
    { 0x90d6690f, 95 },
    { 0, 0 },
    { 0x5ee66911, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72766915, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be66919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d6691c, 85 },
    { 0xc6a6691c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f66926, 87 },
    { 0, 0 },
    { 0xe4f66928, 55 },
    { 0xe1266928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e6692c, 98 },
    { 0x91c6692c, 78 },
    { 0x9e56692d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8366931, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45866935, 85 },
    { 0, 0 },
    { 0x7c066937, 90 },
    { 0xf2466937, 98 },
    { 0xe6b66939, 34 },
    { 0, 0 },
    { 0x9706693b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f66940, 95 },
    { 0x39066941, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5266946, 33 },
    { 0, 0 },
    { 0x4ff66948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f6695b, 88 },
    { 0, 0 },
    { 0x2286695d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x91666960, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a466964, 65 },
    { 0x51166965, 55 },
    { 0xe6d66966, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c366973, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df66976, 95 },
    { 0, 0 },
    { 0xd2f66978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7296697f, 93 },
    { 0x446697f, 56 },
    { 0xe0c66980, 90 },
    { 0x7b566980, 34 },
    { 0x8e266983, 65 },
    { 0x26f66984, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c66989, 93 },
    { 0xf6766989, 80 },
    { 0xc746698b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba866990, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81366995, 90 },
    { 0x94e66996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84466999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a76699f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653669a5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47669a8, 68 },
    { 0x657669a9, 88 },
    { 0x9d7669aa, 55 },
    { 0x430669ab, 61 },
    { 0xb95669ac, 75 },
    { 0x8c8669ad, 33 },
    { 0xe88669ac, 33 },
    { 0x98669af, 90 },
    { 0x9f8669af, 33 },
    { 0, 0 },
    { 0x361669b2, 98 },
    { 0xfd3669b2, 43 },
    { 0, 0 },
    { 0xd32669b5, 24 },
    { 0xa1c669b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x229669b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3669bf, 99 },
    { 0x5c669bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc669c4, 43 },
    { 0xd3b669c4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d669cc, 98 },
    { 0, 0 },
    { 0xaa9669ce, 88 },
    { 0x45c669cf, 95 },
    { 0, 0 },
    { 0xb52669d1, 23 },
    { 0x265669d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1669d5, 96 },
    { 0x73c669d5, 76 },
    { 0x91c669d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987669db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f669df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409669e7, 52 },
    { 0x31b669e8, 76 },
    { 0x446669e9, 52 },
    { 0x364669e8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6669ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee669f3, 61 },
    { 0, 0 },
    { 0x64e669f5, 78 },
    { 0x2e7669f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f669f9, 33 },
    { 0, 0 },
    { 0xe50669fb, 23 },
    { 0xb5b669fc, 95 },
    { 0xfc5669fd, 23 },
    { 0, 0 },
    { 0x830669ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb66a09, 61 },
    { 0, 0 },
    { 0x3c66a0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c66a0e, 98 },
    { 0x47a66a0f, 76 },
    { 0xca866a10, 98 },
    { 0x6fd66a0f, 90 },
    { 0x53366a0e, 46 },
    { 0xd7366a13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35466a18, 76 },
    { 0x86466a19, 72 },
    { 0, 0 },
    { 0xcdf66a1b, 61 },
    { 0, 0 },
    { 0xb6766a1d, 90 },
    { 0x6e366a1d, 43 },
    { 0x15e66a1f, 98 },
    { 0x31666a1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f66a23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70666a26, 76 },
    { 0xecf66a27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5766a2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc966a35, 33 },
    { 0xe066a35, 95 },
    { 0, 0 },
    { 0x50966a38, 98 },
    { 0xba366a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9066a40, 61 },
    { 0x80e66a40, 55 },
    { 0, 0 },
    { 0xb2366a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b166a46, 33 },
    { 0xf1e66a46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b66a4a, 55 },
    { 0x52166a4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55466a4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d66a54, 91 },
    { 0, 0 },
    { 0x72466a56, 78 },
    { 0xbaf66a57, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de66a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c66a5e, 84 },
    { 0x69366a5f, 76 },
    { 0xadd66a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d466a63, 43 },
    { 0x1aa66a64, 76 },
    { 0xffe66a64, 90 },
    { 0x35f66a66, 66 },
    { 0xad366a67, 83 },
    { 0, 0 },
    { 0x9e766a69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94266a6f, 95 },
    { 0, 0 },
    { 0x84866a71, 33 },
    { 0x99966a72, 91 },
    { 0xc3266a73, 90 },
    { 0xdf166a74, 66 },
    { 0x64166a71, 91 },
    { 0, 0 },
    { 0xbdc66a77, 90 },
    { 0x93966a77, 95 },
    { 0x1dd66a79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a66a7c, 91 },
    { 0x31666a7c, 90 },
    { 0x6866a7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8166a81, 95 },
    { 0xec566a81, 34 },
    { 0x93b66a83, 90 },
    { 0xed966a82, 90 },
    { 0, 0 },
    { 0x17666a86, 98 },
    { 0, 0 },
    { 0x64d66a88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9666a8d, 95 },
    { 0x45566a8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666a95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43766a99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59866aa5, 76 },
    { 0, 0 },
    { 0xfbd66aa7, 95 },
    { 0xf4f66aa8, 95 },
    { 0x89f66aa9, 95 },
    { 0x93666aaa, 34 },
    { 0xeb966aab, 87 },
    { 0x49066aaa, 33 },
    { 0x1d966aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41866ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7766ab7, 33 },
    { 0xb1166ab8, 100 },
    { 0xc7266ab7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5666abf, 81 },
    { 0, 0 },
    { 0x11f66ac1, 90 },
    { 0x10f66ac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87566ac9, 43 },
    { 0x9db66aca, 61 },
    { 0, 0 },
    { 0x42e66acc, 36 },
    { 0x59e66acd, 43 },
    { 0x1a266ace, 55 },
    { 0x63b66acf, 23 },
    { 0x2666acf, 46 },
    { 0xffb66ad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1366ad4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa066ad8, 61 },
    { 0x7e666ad9, 98 },
    { 0, 0 },
    { 0xc0866adb, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x72966ade, 23 },
    { 0, 0 },
    { 0x58c66ae0, 46 },
    { 0, 0 },
    { 0x5ab66ae2, 98 },
    { 0x75766ae2, 55 },
    { 0x2ff66ae3, 55 },
    { 0xe8266ae4, 98 },
    { 0xf6d66ae6, 33 },
    { 0xc2766ae2, 99 },
    { 0x31066ae3, 90 },
    { 0x6a366ae9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50666aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74766af7, 8 },
    { 0, 0 },
    { 0x2d366af9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc66aff, 87 },
    { 0x99f66aff, 87 },
    { 0, 0 },
    { 0x96a66b02, 61 },
    { 0xbe266b03, 93 },
    { 0, 0 },
    { 0x43266b05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a66b08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83566b0b, 34 },
    { 0x4e466b0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b366b18, 23 },
    { 0xd0b66b18, 23 },
    { 0, 0 },
    { 0x9fc66b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf566b22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81466b25, 66 },
    { 0xf5f66b26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62466b2b, 92 },
    { 0xb8666b2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d66b31, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b66b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b66b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b66b3f, 96 },
    { 0x72e66b40, 33 },
    { 0, 0 },
    { 0x58c66b42, 33 },
    { 0x23566b43, 87 },
    { 0x2b466b43, 75 },
    { 0x6e866b45, 76 },
    { 0xac666b46, 90 },
    { 0xece66b45, 90 },
    { 0x86d66b48, 33 },
    { 0xf6666b46, 78 },
    { 0x74366b4a, 23 },
    { 0x50a66b48, 98 },
    { 0x16d66b4c, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73966b52, 23 },
    { 0xa7466b52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf66b5b, 99 },
    { 0x70966b5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b64, 90 },
    { 0, 0 },
    { 0xfde66b66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9366b69, 98 },
    { 0x3e166b69, 66 },
    { 0xca466b6b, 93 },
    { 0xe366b6c, 90 },
    { 0x66b66b6b, 90 },
    { 0xeb766b6e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a66b72, 96 },
    { 0xf466b72, 95 },
    { 0x10266b74, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea66b78, 24 },
    { 0, 0 },
    { 0xc5466b7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba466b7e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf66b82, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b566b89, 84 },
    { 0x48b66b89, 95 },
    { 0, 0 },
    { 0x79466b8c, 61 },
    { 0x6cf66b8c, 94 },
    { 0, 0 },
    { 0x21066b8f, 74 },
    { 0x7e266b90, 61 },
    { 0xbf66b91, 66 },
    { 0, 0 },
    { 0x4ec66b93, 77 },
    { 0xe2066b94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff66b9a, 66 },
    { 0x77466b9b, 93 },
    { 0, 0 },
    { 0x67166b9d, 97 },
    { 0x7bc66b9e, 55 },
    { 0, 0 },
    { 0xbf566ba0, 95 },
    { 0x65c66ba1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8166ba4, 76 },
    { 0x8e066ba5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b466baa, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa766baf, 95 },
    { 0, 0 },
    { 0x4ae66bb1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83366bb8, 78 },
    { 0x9cc66bb8, 23 },
    { 0x52f66bb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a066bbe, 93 },
    { 0xd6866bbe, 84 },
    { 0xc9d66bc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81466bc6, 91 },
    { 0x43566bc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f66bcb, 33 },
    { 0x84e66bcc, 90 },
    { 0x12566bcc, 98 },
    { 0xfc066bce, 33 },
    { 0x90c66bcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21966bd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a366bdb, 23 },
    { 0x8a166bdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47866bf0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca966bf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b866bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c366c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab66c08, 23 },
    { 0x26766c08, 61 },
    { 0x91a66c09, 67 },
    { 0x53766c0b, 98 },
    { 0x33466c0c, 23 },
    { 0, 0 },
    { 0x1a866c0e, 98 },
    { 0, 0 },
    { 0xf4166c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62666c13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb266c17, 76 },
    { 0x31866c18, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d66c1b, 76 },
    { 0xa0166c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8366c22, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee66c26, 90 },
    { 0x5266c27, 90 },
    { 0x6c466c28, 76 },
    { 0, 0 },
    { 0xc9f66c2a, 84 },
    { 0xa866c2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf066c2e, 61 },
    { 0xfd766c2f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60966c34, 33 },
    { 0x75766c35, 90 },
    { 0, 0 },
    { 0x25066c37, 98 },
    { 0x91166c37, 68 },
    { 0xa9a66c39, 95 },
    { 0xf4b66c39, 19 },
    { 0x36366c3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5666c3e, 17 },
    { 0x18966c3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58366c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3966c4e, 68 },
    { 0x75666c4e, 61 },
    { 0, 0 },
    { 0x7c966c51, 93 },
    { 0xde866c52, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b66c55, 67 },
    { 0xbc566c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a66c59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd466c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb966c65, 78 },
    { 0, 0 },
    { 0xa1766c67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc366c72, 33 },
    { 0xf8566c73, 76 },
    { 0x5c266c74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28466c7a, 78 },
    { 0x57966c7b, 76 },
    { 0x84b66c7c, 23 },
    { 0x95366c7b, 23 },
    { 0x3a866c7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81866c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b566c87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd66c8b, 100 },
    { 0xef266c8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b66c94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xec366c97, 55 },
    { 0x4b566c97, 61 },
    { 0x54166c99, 23 },
    { 0, 0 },
    { 0xec566c9b, 101 },
    { 0xdc966c9b, 55 },
    { 0x9e166c9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c66ca1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa366ca8, 95 },
    { 0x4ff66ca9, 91 },
    { 0x30f66ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1466cad, 78 },
    { 0, 0 },
    { 0x8c666caf, 34 },
    { 0x4fb66caf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92666cb4, 78 },
    { 0x3fa66cb5, 99 },
    { 0, 0 },
    { 0xe7366cb7, 33 },
    { 0x1cd66cb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b66cbb, 87 },
    { 0x79366cbc, 64 },
    { 0x87866cbd, 34 },
    { 0, 0 },
    { 0x5a466cbf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e66cc2, 17 },
    { 0, 0 },
    { 0xbbb66cc4, 95 },
    { 0xcfe66cc5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd766cc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65566ccd, 88 },
    { 0xf1266ccd, 40 },
    { 0xf0866cce, 95 },
    { 0, 0 },
    { 0x28c66cd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c66cd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2066ce1, 33 },
    { 0, 0 },
    { 0x84066ce3, 33 },
    { 0x98266ce4, 52 },
    { 0x35266ce5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff766ce9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a566cee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb766cf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e66cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d66cfe, 43 },
    { 0x18c66cff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9266d04, 23 },
    { 0x83d66d05, 98 },
    { 0x2b766d05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b866d0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f66d0f, 100 },
    { 0xd3766d10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1666d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca66d17, 90 },
    { 0x26566d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8166d1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67866d23, 77 },
    { 0x2c966d23, 91 },
    { 0xaae66d25, 46 },
    { 0x3a066d23, 100 },
    { 0x61766d24, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55566d2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac466d3f, 90 },
    { 0xc8466d40, 100 },
    { 0x50666d3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b766d45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f366d4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a66d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff66d59, 93 },
    { 0x1b166d5a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x58166d5d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d866d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb566d65, 43 },
    { 0, 0 },
    { 0x3f566d67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9966d6a, 61 },
    { 0x52a66d6b, 76 },
    { 0, 0 },
    { 0xf6466d6d, 76 },
    { 0x46466d6d, 98 },
    { 0, 0 },
    { 0x4c666d70, 33 },
    { 0xbfc66d70, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2666d77, 96 },
    { 0x41f66d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15466d81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37666d88, 61 },
    { 0x2a766d88, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b66d91, 95 },
    { 0x56a66d92, 90 },
    { 0x40066d92, 33 },
    { 0, 0 },
    { 0xb1e66d95, 33 },
    { 0, 0 },
    { 0xb5066d97, 84 },
    { 0xcbf66d97, 43 },
    { 0x19d66d97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31466d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d66da3, 46 },
    { 0, 0 },
    { 0xb1366da5, 90 },
    { 0xf5d66da6, 90 },
    { 0x84e66da5, 12 },
    { 0, 0 },
    { 0xa7f66da9, 93 },
    { 0x45a66da9, 96 },
    { 0, 0 },
    { 0x9f066dac, 33 },
    { 0, 0 },
    { 0x30666dae, 33 },
    { 0xb8e66dae, 95 },
    { 0x38066db0, 96 },
    { 0, 0 },
    { 0x38d66db2, 98 },
    { 0xcbc66db2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90566dbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b166dbe, 61 },
    { 0, 0 },
    { 0xa7666dc0, 61 },
    { 0xb6566dc0, 23 },
    { 0, 0 },
    { 0x9b166dc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f66dc9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x90666dcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b66dcf, 34 },
    { 0x98966dcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f66dd3, 61 },
    { 0xf4f66dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e66de1, 93 },
    { 0x6cb66de2, 95 },
    { 0x1d166de3, 90 },
    { 0x89c66de3, 34 },
    { 0xf0766de2, 61 },
    { 0x8a366de6, 23 },
    { 0, 0 },
    { 0x6c666de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb66deb, 76 },
    { 0x9e566deb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa466def, 98 },
    { 0, 0 },
    { 0x65466df1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c266df6, 84 },
    { 0x6d66df7, 78 },
    { 0x52e66df8, 43 },
    { 0x8b866df8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f066dff, 40 },
    { 0xb3e66e00, 95 },
    { 0x1fc66e01, 90 },
    { 0x6de66e00, 76 },
    { 0x27b66e01, 98 },
    { 0, 0 },
    { 0x2e066e05, 98 },
    { 0x7bf66e06, 66 },
    { 0x2ed66e06, 17 },
    { 0, 0 },
    { 0x59966e09, 98 },
    { 0xc6566e0a, 60 },
    { 0xaf866e0b, 23 },
    { 0, 0 },
    { 0x9c366e0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa66e12, 83 },
    { 0x31d66e13, 90 },
    { 0xe1166e14, 90 },
    { 0xaf66e13, 61 },
    { 0x95d66e14, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95166e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61066e27, 84 },
    { 0xe0e66e27, 23 },
    { 0x50f66e27, 90 },
    { 0xde766e27, 80 },
    { 0xd3466e2b, 34 },
    { 0xa4f66e2b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d66e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46566e33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96066e36, 95 },
    { 0, 0 },
    { 0xfb166e38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf066e3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3466e3e, 40 },
    { 0x46766e3f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9766e42, 96 },
    { 0, 0 },
    { 0x1166e44, 76 },
    { 0x24366e45, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a66e4b, 61 },
    { 0x2b66e4b, 46 },
    { 0, 0 },
    { 0x5e766e4e, 55 },
    { 0x76f66e4f, 23 },
    { 0x69166e4f, 33 },
    { 0xa6766e51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a66e54, 33 },
    { 0, 0 },
    { 0xffd66e56, 66 },
    { 0, 0 },
    { 0x2a866e58, 98 },
    { 0x4c766e59, 91 },
    { 0xdff66e5a, 80 },
    { 0x55a66e59, 81 },
    { 0, 0 },
    { 0x9ab66e5d, 99 },
    { 0x94b66e5d, 95 },
    { 0, 0 },
    { 0x65266e60, 77 },
    { 0x8ba66e60, 91 },
    { 0x3ec66e62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a66e65, 55 },
    { 0xbcd66e65, 90 },
    { 0x36d66e67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a66e6a, 90 },
    { 0, 0 },
    { 0xe4c66e6c, 93 },
    { 0x20966e6c, 75 },
    { 0x62666e6e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b766e71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc66e74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5566e79, 80 },
    { 0x8b066e7a, 17 },
    { 0xe2366e7b, 90 },
    { 0xfe666e7c, 100 },
    { 0, 0 },
    { 0xf8366e7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4366e82, 78 },
    { 0, 0 },
    { 0x72866e84, 33 },
    { 0x70166e84, 88 },
    { 0x56e66e86, 33 },
    { 0x5ea66e87, 90 },
    { 0x68566e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb866e8e, 90 },
    { 0x8f966e8e, 90 },
    { 0, 0 },
    { 0x44c66e91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31266e94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1166e98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36866e9e, 95 },
    { 0xedc66e9f, 61 },
    { 0x2fc66e9e, 33 },
    { 0x15566ea1, 90 },
    { 0, 0 },
    { 0x3e166ea3, 61 },
    { 0x76466ea4, 55 },
    { 0, 0 },
    { 0x55866ea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d66ea9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f66eac, 95 },
    { 0x40466eac, 85 },
    { 0x1c066eae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9966eb4, 98 },
    { 0xa6266eb5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5066ebd, 54 },
    { 0, 0 },
    { 0x9a866ebf, 46 },
    { 0x11c66ec0, 33 },
    { 0, 0 },
    { 0x8f966ec2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac66ec5, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0066eca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e66ecd, 76 },
    { 0, 0 },
    { 0xd3266ecf, 90 },
    { 0x48166ed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23066ed4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d66ed8, 34 },
    { 0, 0 },
    { 0x77766eda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34466ee5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f66eea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc66eed, 90 },
    { 0x67c66eee, 61 },
    { 0x83866eee, 46 },
    { 0xc3266eef, 61 },
    { 0xe4566eed, 95 },
    { 0x4c466ef2, 94 },
    { 0x55366ef3, 33 },
    { 0xa9c66ef2, 93 },
    { 0, 0 },
    { 0xb1766ef6, 23 },
    { 0x9b266ef7, 33 },
    { 0x38d66ef7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf566efe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f66f01, 61 },
    { 0, 0 },
    { 0x1e066f03, 33 },
    { 0xc2566f04, 98 },
    { 0, 0 },
    { 0x80e66f06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5966f0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c66f16, 23 },
    { 0x18966f16, 100 },
    { 0x91e66f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d366f1c, 61 },
    { 0xdd566f1d, 77 },
    { 0x1cc66f1e, 55 },
    { 0x7fe66f1f, 95 },
    { 0xc1d66f20, 61 },
    { 0, 0 },
    { 0xf1466f22, 90 },
    { 0x97366f22, 95 },
    { 0xd3466f22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa566f2c, 33 },
    { 0xa366f2c, 33 },
    { 0, 0 },
    { 0x6f566f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f66f39, 66 },
    { 0, 0 },
    { 0xd5c66f3b, 33 },
    { 0x8166f3c, 90 },
    { 0xed566f3b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066f43, 90 },
    { 0x49566f43, 34 },
    { 0x63166f45, 98 },
    { 0x2266f46, 43 },
    { 0x3f266f46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d66f55, 61 },
    { 0x20a66f56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57166f59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32466f5c, 90 },
    { 0xf2b66f5d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef166f60, 95 },
    { 0xc3866f61, 98 },
    { 0, 0 },
    { 0x4e066f63, 33 },
    { 0x4e366f63, 90 },
    { 0x20966f65, 42 },
    { 0x67666f64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2666f6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46066f70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc266f74, 33 },
    { 0, 0 },
    { 0xbc366f76, 43 },
    { 0xf1b66f77, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65766f7e, 90 },
    { 0, 0 },
    { 0x3866f80, 55 },
    { 0xc3166f81, 100 },
    { 0xda066f81, 23 },
    { 0x90e66f83, 33 },
    { 0x50066f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff66f89, 43 },
    { 0, 0 },
    { 0xe1566f8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39266f95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d66f98, 90 },
    { 0x68e66f99, 61 },
    { 0x97f66f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5666fa1, 76 },
    { 0x49966fa1, 83 },
    { 0xb3566fa3, 95 },
    { 0xd166fa4, 98 },
    { 0x1ee66fa5, 93 },
    { 0xf0a66fa4, 100 },
    { 0x34a66fa7, 46 },
    { 0, 0 },
    { 0x59666fa9, 98 },
    { 0x27866faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85466fb2, 85 },
    { 0x40966fb2, 55 },
    { 0, 0 },
    { 0x22566fb5, 61 },
    { 0x39466fb5, 33 },
    { 0x15066fb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6466fba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd66fbe, 23 },
    { 0x11666fbe, 43 },
    { 0x8c666fbe, 61 },
    { 0xcb666fc1, 61 },
    { 0, 0 },
    { 0x2b366fc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf66fc6, 76 },
    { 0, 0 },
    { 0x6ac66fc8, 33 },
    { 0xd0b66fc9, 90 },
    { 0x9e366fca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4566fcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a66fd5, 95 },
    { 0x6b266fd6, 46 },
    { 0x78666fd7, 33 },
    { 0x4c266fd8, 98 },
    { 0xd0b66fd6, 33 },
    { 0x1d066fd5, 55 },
    { 0x1466fdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a66fdf, 93 },
    { 0xa1266fe0, 87 },
    { 0, 0 },
    { 0x78966fe2, 85 },
    { 0x65666fe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c66fe6, 43 },
    { 0xca466fe6, 80 },
    { 0xf8166fe6, 76 },
    { 0, 0 },
    { 0x3eb66fea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1866fef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf66ff6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b366ffc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8466fff, 55 },
    { 0, 0 },
    { 0xfaf67001, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x17867004, 76 },
    { 0, 0 },
    { 0x1c667006, 78 },
    { 0xf5767006, 76 },
    { 0xca267008, 55 },
    { 0x6467008, 23 },
    { 0, 0 },
    { 0xddc6700b, 55 },
    { 0, 0 },
    { 0xeb66700d, 80 },
    { 0x1df6700e, 23 },
    { 0x3206700d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f767012, 76 },
    { 0x57d67012, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06701a, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c36701e, 33 },
    { 0x5076701e, 33 },
    { 0, 0 },
    { 0x60667021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9267029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a6702c, 98 },
    { 0x7266702d, 23 },
    { 0x2b46702d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b67031, 93 },
    { 0x8cc67032, 43 },
    { 0, 0 },
    { 0xfba67034, 48 },
    { 0x9dd67034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1026703b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e67049, 95 },
    { 0, 0 },
    { 0x3996704b, 93 },
    { 0x80f6704c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde467057, 46 },
    { 0xeca67058, 99 },
    { 0x97a67059, 61 },
    { 0, 0 },
    { 0x6346705b, 76 },
    { 0x91d6705c, 93 },
    { 0x5686705d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a67060, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18667067, 76 },
    { 0x2467068, 53 },
    { 0x25667068, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec6706c, 36 },
    { 0xb456706d, 55 },
    { 0x5426706c, 65 },
    { 0, 0 },
    { 0x11367070, 23 },
    { 0xc0367070, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e367077, 52 },
    { 0x93667077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87067080, 99 },
    { 0, 0 },
    { 0xc9867082, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e67086, 23 },
    { 0xb9167087, 64 },
    { 0x7df67086, 34 },
    { 0xd867089, 65 },
    { 0x55167086, 52 },
    { 0x4926708b, 23 },
    { 0x99a6708b, 61 },
    { 0xd576708c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xae667090, 23 },
    { 0xa0667090, 23 },
    { 0xa1067092, 76 },
    { 0xd3d67090, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4106709b, 33 },
    { 0x7f6709c, 98 },
    { 0x4c76709d, 90 },
    { 0, 0 },
    { 0x9486709f, 95 },
    { 0x97a670a0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63670a4, 90 },
    { 0, 0 },
    { 0x61f670a6, 93 },
    { 0, 0 },
    { 0x44b670a8, 61 },
    { 0x2e8670a8, 98 },
    { 0xac1670aa, 98 },
    { 0x219670a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f670af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701670b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x614670b8, 61 },
    { 0, 0 },
    { 0xce8670ba, 63 },
    { 0x5ce670ba, 33 },
    { 0xe1d670bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x337670bf, 90 },
    { 0x995670bf, 98 },
    { 0x145670c1, 68 },
    { 0x61f670c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617670c6, 93 },
    { 0xec7670c6, 61 },
    { 0x2c4670c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500670d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba670d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2670d9, 61 },
    { 0, 0 },
    { 0xef3670db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264670e2, 90 },
    { 0xeb0670e2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807670e7, 43 },
    { 0x8ee670e8, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941670ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165670f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e670fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4767100, 98 },
    { 0xdfe67100, 65 },
    { 0xe2367102, 56 },
    { 0xd3467102, 98 },
    { 0, 0 },
    { 0x43e67105, 95 },
    { 0x8ef67106, 43 },
    { 0x73667106, 61 },
    { 0xfd167108, 91 },
    { 0, 0 },
    { 0xab6710a, 99 },
    { 0x8296710b, 46 },
    { 0xcae6710b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9967116, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d6711e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4267121, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376712d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90967135, 90 },
    { 0, 0 },
    { 0x3a767137, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa6713a, 95 },
    { 0xa856713b, 94 },
    { 0x9046713c, 61 },
    { 0x64e6713b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a67142, 95 },
    { 0x5de67143, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae67148, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c96714c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23467150, 61 },
    { 0, 0 },
    { 0xa2967152, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf66715c, 43 },
    { 0xbe16715c, 55 },
    { 0xf66715c, 34 },
    { 0x80f6715f, 98 },
    { 0x4cf67160, 46 },
    { 0xdac67160, 98 },
    { 0xa876715f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80767166, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c6716a, 90 },
    { 0, 0 },
    { 0x5c36716c, 62 },
    { 0xf1d6716c, 33 },
    { 0xc206716d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32967172, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f267176, 95 },
    { 0, 0 },
    { 0x1c67178, 61 },
    { 0x25567178, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b66717c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a6717f, 66 },
    { 0, 0 },
    { 0x64867181, 61 },
    { 0x18c67182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e167187, 101 },
    { 0xd6367188, 90 },
    { 0, 0 },
    { 0x8c46718a, 77 },
    { 0xfa86718b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c867191, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac567194, 33 },
    { 0x7b767194, 90 },
    { 0x93a67196, 46 },
    { 0x15367196, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da6719a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5806719d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5671a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x186671a9, 52 },
    { 0xcd1671aa, 76 },
    { 0x4bd671a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c671b7, 9 },
    { 0, 0 },
    { 0x170671b9, 91 },
    { 0xc7b671ba, 33 },
    { 0x760671bb, 34 },
    { 0xfec671bc, 95 },
    { 0xf60671bd, 46 },
    { 0, 0 },
    { 0x3cf671bf, 98 },
    { 0x88c671bf, 61 },
    { 0x7d1671bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a671c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d671c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147671cc, 61 },
    { 0, 0 },
    { 0x662671ce, 90 },
    { 0x74c671ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f671d4, 23 },
    { 0, 0 },
    { 0x4f1671d6, 61 },
    { 0xb49671d6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302671dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764671e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x804671e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191671e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8671eb, 61 },
    { 0x76b671ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04671f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d671f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6671fa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc67203, 34 },
    { 0, 0 },
    { 0x26767205, 65 },
    { 0, 0 },
    { 0x5dc67207, 93 },
    { 0xcf267208, 23 },
    { 0xe4067209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e96720d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7267210, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24267219, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9176721c, 33 },
    { 0, 0 },
    { 0x5d36721e, 87 },
    { 0, 0 },
    { 0x86467220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed67223, 33 },
    { 0, 0 },
    { 0x89a67225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6256722c, 91 },
    { 0xe4d6722d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7467230, 98 },
    { 0xc0967231, 34 },
    { 0, 0 },
    { 0x81267233, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c6723b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf86723e, 91 },
    { 0x5636723f, 90 },
    { 0x4406723f, 98 },
    { 0, 0 },
    { 0x22267242, 90 },
    { 0x7f267243, 33 },
    { 0x92667243, 43 },
    { 0, 0 },
    { 0xb9667246, 55 },
    { 0x2f67247, 90 },
    { 0xb5a67248, 76 },
    { 0x5a267249, 90 },
    { 0xa367248, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e6724f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23267256, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c067259, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30367268, 98 },
    { 0xd4267268, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb6726c, 76 },
    { 0xcf66726d, 33 },
    { 0, 0 },
    { 0x8ff6726f, 98 },
    { 0, 0 },
    { 0xc2367271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6067274, 76 },
    { 0x9a367275, 81 },
    { 0, 0 },
    { 0x8ab67277, 33 },
    { 0, 0 },
    { 0xfe167279, 93 },
    { 0xd196727a, 98 },
    { 0, 0 },
    { 0x3ba6727c, 61 },
    { 0xb1e6727d, 93 },
    { 0, 0 },
    { 0x8dd6727f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc867282, 76 },
    { 0xf6267283, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566728b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1256728e, 46 },
    { 0xceb6728f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb367294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e66729f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576672a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c672ab, 98 },
    { 0x787672ab, 33 },
    { 0x27c672ad, 90 },
    { 0xc4a672ae, 61 },
    { 0x10c672af, 23 },
    { 0, 0 },
    { 0x645672b1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8672b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2672bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x950672c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399672c4, 33 },
    { 0x2e4672c5, 23 },
    { 0, 0 },
    { 0x8e1672c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b672ca, 33 },
    { 0x7da672cb, 90 },
    { 0, 0 },
    { 0xded672cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59672d6, 43 },
    { 0, 0 },
    { 0x6cf672d8, 33 },
    { 0, 0 },
    { 0xcde672da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe672df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708672e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03672e9, 90 },
    { 0x51f672ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de672ee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac672f4, 90 },
    { 0xd45672f5, 84 },
    { 0, 0 },
    { 0xf6672f7, 52 },
    { 0xe85672f7, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb672fc, 95 },
    { 0x38e672fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2367300, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38967305, 90 },
    { 0xc5d67305, 55 },
    { 0x8a367307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da6730a, 66 },
    { 0x256730a, 80 },
    { 0x696730c, 66 },
    { 0xa76730d, 23 },
    { 0x2706730d, 76 },
    { 0, 0 },
    { 0x4267310, 96 },
    { 0x49e67311, 100 },
    { 0xe2967311, 46 },
    { 0xf3267313, 67 },
    { 0xe3e67311, 78 },
    { 0xe5067313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc6731a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81567320, 94 },
    { 0, 0 },
    { 0x77667322, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb86732d, 90 },
    { 0xd726732e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x98067331, 23 },
    { 0, 0 },
    { 0xe6867333, 69 },
    { 0, 0 },
    { 0x45f67335, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7376733b, 61 },
    { 0, 0 },
    { 0x9ee6733d, 90 },
    { 0xf466733e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xef067341, 90 },
    { 0xda67342, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d67345, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be67349, 24 },
    { 0x5516734a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa56734d, 95 },
    { 0x1df6734e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b367352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81567359, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21867360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78267364, 43 },
    { 0x8a167365, 66 },
    { 0xf467366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe767369, 90 },
    { 0xf5e6736a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2836736d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b67375, 61 },
    { 0, 0 },
    { 0x7267377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e6737a, 23 },
    { 0xb536737a, 33 },
    { 0xcd16737c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56737f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90667384, 98 },
    { 0, 0 },
    { 0x27567386, 33 },
    { 0xeb167387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816738b, 93 },
    { 0, 0 },
    { 0xc666738d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7867390, 23 },
    { 0x77567391, 33 },
    { 0x94467392, 90 },
    { 0, 0 },
    { 0x8bb67394, 61 },
    { 0x48c67395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9536739a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436739d, 23 },
    { 0xa606739d, 98 },
    { 0xadb6739f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x609673a2, 90 },
    { 0xd53673a3, 55 },
    { 0x99d673a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45673a8, 95 },
    { 0, 0 },
    { 0xb8d673aa, 67 },
    { 0x739673ab, 68 },
    { 0x950673ab, 34 },
    { 0x624673aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d673b2, 99 },
    { 0xbf0673b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2673b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47673be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723673c2, 17 },
    { 0x8b9673c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9673cc, 66 },
    { 0x7b7673cc, 95 },
    { 0x623673cd, 98 },
    { 0xe39673cf, 100 },
    { 0, 0 },
    { 0x4ac673d1, 34 },
    { 0x9b7673d2, 93 },
    { 0, 0 },
    { 0xdea673d4, 76 },
    { 0, 0 },
    { 0xf9673d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5673db, 34 },
    { 0, 0 },
    { 0x719673dd, 87 },
    { 0x745673de, 23 },
    { 0x51673df, 90 },
    { 0x99673dd, 43 },
    { 0xc36673df, 46 },
    { 0, 0 },
    { 0x9a673e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75673e8, 90 },
    { 0xcb8673e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80673ec, 77 },
    { 0, 0 },
    { 0x949673ee, 43 },
    { 0x3de673ef, 95 },
    { 0, 0 },
    { 0x96673f1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726673f9, 61 },
    { 0x742673f9, 23 },
    { 0x828673fb, 95 },
    { 0, 0 },
    { 0x2e3673fd, 98 },
    { 0xce1673fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x84167401, 77 },
    { 0xbc67401, 86 },
    { 0, 0 },
    { 0xedd67404, 87 },
    { 0x69a67404, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f67409, 100 },
    { 0x92a6740a, 95 },
    { 0x69667409, 55 },
    { 0x9396740c, 43 },
    { 0, 0 },
    { 0x2346740e, 78 },
    { 0x8e46740f, 95 },
    { 0x81b6740f, 66 },
    { 0x69967411, 90 },
    { 0xc7767412, 78 },
    { 0xebe67412, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb67418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef26741b, 76 },
    { 0xbec6741c, 81 },
    { 0x4516741c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f567420, 61 },
    { 0x19367421, 55 },
    { 0xb7967421, 98 },
    { 0x25667421, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab67426, 61 },
    { 0, 0 },
    { 0xabb67428, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d76742c, 23 },
    { 0xc576742c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11667431, 33 },
    { 0, 0 },
    { 0xe9367433, 23 },
    { 0x44967434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73267439, 55 },
    { 0xe5f67439, 66 },
    { 0x87a6743b, 78 },
    { 0x4846743c, 87 },
    { 0xe246743d, 23 },
    { 0x3356743d, 33 },
    { 0xc2b6743c, 85 },
    { 0, 0 },
    { 0x36c67441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63767444, 61 },
    { 0xa7567445, 99 },
    { 0xb3067446, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e56744c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90767452, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16967459, 99 },
    { 0xecd6745a, 23 },
    { 0, 0 },
    { 0x1ab6745c, 91 },
    { 0x9956745d, 76 },
    { 0xff56745c, 61 },
    { 0, 0 },
    { 0x9c667460, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba67467, 90 },
    { 0xf5467468, 98 },
    { 0, 0 },
    { 0x6916746a, 55 },
    { 0x5566746b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6746f, 90 },
    { 0, 0 },
    { 0x51a67471, 97 },
    { 0, 0 },
    { 0xfaa67473, 76 },
    { 0xd4967473, 95 },
    { 0, 0 },
    { 0x46067476, 81 },
    { 0xd1d67476, 99 },
    { 0, 0 },
    { 0xe0e67479, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c06747c, 78 },
    { 0x7056747d, 88 },
    { 0, 0 },
    { 0x246747f, 61 },
    { 0xb9867480, 61 },
    { 0, 0 },
    { 0xd7e67482, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73467486, 98 },
    { 0xa3067486, 90 },
    { 0xdad67488, 90 },
    { 0x267488, 43 },
    { 0xb9667487, 33 },
    { 0xe7e6748b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9566748f, 90 },
    { 0x13c6748f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c67496, 99 },
    { 0x53a67497, 34 },
    { 0xa267498, 46 },
    { 0x9967499, 67 },
    { 0x1d467499, 43 },
    { 0xfab6749b, 76 },
    { 0x2516749b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b674a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d674a4, 61 },
    { 0x19e674a4, 100 },
    { 0x931674a6, 37 },
    { 0xd6a674a6, 34 },
    { 0xcf8674a7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43674ac, 52 },
    { 0, 0 },
    { 0x9fe674ae, 61 },
    { 0xd84674ae, 23 },
    { 0xf5b674b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25674b3, 67 },
    { 0x34b674b4, 55 },
    { 0x211674b5, 17 },
    { 0xd83674b5, 90 },
    { 0, 0 },
    { 0xbb7674b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6674bb, 90 },
    { 0x6d1674bc, 88 },
    { 0xb6f674bd, 90 },
    { 0x3c1674be, 53 },
    { 0x57e674bd, 55 },
    { 0xa8f674c0, 33 },
    { 0x533674c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68674c6, 87 },
    { 0, 0 },
    { 0x743674c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b674cf, 33 },
    { 0x3ab674d0, 34 },
    { 0x985674d1, 90 },
    { 0xdf5674d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5674d5, 23 },
    { 0x384674d6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1674da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x78674dd, 95 },
    { 0, 0 },
    { 0xd08674df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42674e4, 78 },
    { 0x36f674e5, 55 },
    { 0xce3674e6, 100 },
    { 0x445674e7, 95 },
    { 0xca674e8, 61 },
    { 0x798674e4, 33 },
    { 0x48b674e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2674f1, 46 },
    { 0xa2674f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46674f7, 85 },
    { 0, 0 },
    { 0x8d6674f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526674ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5467502, 52 },
    { 0x63e67503, 67 },
    { 0, 0 },
    { 0x9f067505, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd886750a, 77 },
    { 0, 0 },
    { 0x1c36750c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb36750f, 90 },
    { 0xef867510, 76 },
    { 0xb6467511, 52 },
    { 0x71767512, 84 },
    { 0xea367513, 85 },
    { 0xd7767513, 55 },
    { 0xbc167515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16667519, 99 },
    { 0x6f66751a, 77 },
    { 0xb5d6751a, 75 },
    { 0x70c6751b, 61 },
    { 0x3486751d, 68 },
    { 0x8ce6751d, 98 },
    { 0x6c56751e, 98 },
    { 0x66a67520, 61 },
    { 0, 0 },
    { 0x71b67522, 100 },
    { 0x12067522, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x27467526, 75 },
    { 0x72267527, 33 },
    { 0x74b67527, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d96752e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b67534, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb67537, 33 },
    { 0x43f67538, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe346753c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e167541, 76 },
    { 0, 0 },
    { 0xc1167543, 90 },
    { 0x77267544, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd67554, 33 },
    { 0xd6e67555, 23 },
    { 0x51367556, 23 },
    { 0xe5e67557, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1836755f, 35 },
    { 0x15367560, 90 },
    { 0, 0 },
    { 0xca967562, 90 },
    { 0xc1c67562, 90 },
    { 0x48167564, 61 },
    { 0x52067565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f6756b, 100 },
    { 0, 0 },
    { 0x7ab6756d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0467572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c67576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa386757a, 87 },
    { 0, 0 },
    { 0x8d36757c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab767585, 77 },
    { 0, 0 },
    { 0x98767587, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf6758a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2967592, 33 },
    { 0x34667593, 90 },
    { 0x41f67594, 43 },
    { 0xec67595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83867598, 55 },
    { 0x83667599, 90 },
    { 0xf896759a, 34 },
    { 0xbb56759a, 90 },
    { 0, 0 },
    { 0x1bb6759d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a675a4, 95 },
    { 0x1eb675a5, 90 },
    { 0xc7e675a6, 99 },
    { 0x131675a7, 46 },
    { 0xbf1675a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d675ac, 93 },
    { 0xf84675ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69675b3, 88 },
    { 0, 0 },
    { 0x505675b5, 99 },
    { 0xb5675b6, 90 },
    { 0, 0 },
    { 0xd83675b8, 23 },
    { 0xe1b675b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306675bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85675c0, 90 },
    { 0x5d9675c1, 75 },
    { 0x2f8675c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1675ca, 75 },
    { 0x497675ca, 75 },
    { 0xf9d675cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743675d1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0675d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0675de, 43 },
    { 0, 0 },
    { 0xff1675e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c675e9, 76 },
    { 0x141675ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x817675ed, 76 },
    { 0, 0 },
    { 0x6dc675ef, 95 },
    { 0x92f675f0, 94 },
    { 0, 0 },
    { 0xc0e675f2, 43 },
    { 0, 0 },
    { 0x3c9675f4, 84 },
    { 0, 0 },
    { 0xa28675f6, 61 },
    { 0x740675f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410675fe, 33 },
    { 0, 0 },
    { 0xb0167600, 19 },
    { 0, 0 },
    { 0xe2867602, 55 },
    { 0xfbd67603, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c67608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd6760b, 87 },
    { 0xd5e6760c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b67610, 61 },
    { 0x18f67611, 66 },
    { 0x8fc67610, 23 },
    { 0x10767612, 90 },
    { 0x99667614, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47067621, 98 },
    { 0x85967622, 95 },
    { 0x2967623, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267629, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d16762d, 33 },
    { 0, 0 },
    { 0x5066762f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e467634, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4716763b, 34 },
    { 0x5d96763c, 87 },
    { 0x6ff6763c, 33 },
    { 0x56a6763b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x96567641, 34 },
    { 0xf1167642, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f67647, 53 },
    { 0xc4567648, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd556764b, 33 },
    { 0xc0a6764b, 8 },
    { 0xdb86764d, 23 },
    { 0x176764b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a67652, 90 },
    { 0, 0 },
    { 0x7e567654, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee67658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f36765b, 61 },
    { 0xe006765b, 23 },
    { 0x5266765d, 93 },
    { 0xd2a6765c, 76 },
    { 0x5d16765f, 78 },
    { 0xa486765f, 61 },
    { 0x2636765c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced6766a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f6766e, 90 },
    { 0x1eb6766e, 98 },
    { 0xc7f6766f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4267675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5336767a, 92 },
    { 0xc776767b, 87 },
    { 0x4066767c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe906767f, 33 },
    { 0, 0 },
    { 0xd4667681, 33 },
    { 0x4d767681, 52 },
    { 0x89667681, 33 },
    { 0, 0 },
    { 0x56167685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82867689, 90 },
    { 0x96a67689, 33 },
    { 0x66768a, 33 },
    { 0, 0 },
    { 0xd1b6768d, 95 },
    { 0x1226768e, 81 },
    { 0, 0 },
    { 0xb9367690, 17 },
    { 0x36f67691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd667698, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995676a2, 46 },
    { 0x496676a3, 95 },
    { 0xd9d676a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730676a8, 55 },
    { 0x414676a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522676ad, 61 },
    { 0, 0 },
    { 0x39f676af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a676b2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554676b8, 78 },
    { 0x264676b9, 65 },
    { 0x6b3676b9, 17 },
    { 0, 0 },
    { 0xf90676bc, 90 },
    { 0xf54676bd, 84 },
    { 0xc75676be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2676c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x833676c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x957676c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f676cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54676d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc676d4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a676e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353676e7, 76 },
    { 0, 0 },
    { 0xf85676e9, 38 },
    { 0x7dc676ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693676f0, 90 },
    { 0xb29676f0, 100 },
    { 0, 0 },
    { 0x659676f3, 52 },
    { 0xaf4676f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x830676f7, 61 },
    { 0xd7b676f7, 43 },
    { 0, 0 },
    { 0xb74676fa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x347676fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44267703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92867707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c66770b, 43 },
    { 0x5fd6770c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb67712, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf367715, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5626771b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23167725, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b26772c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f567731, 95 },
    { 0, 0 },
    { 0x25867733, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b767737, 90 },
    { 0x9df67738, 76 },
    { 0x98b67739, 61 },
    { 0x7346773a, 34 },
    { 0xf826773a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a6773e, 98 },
    { 0x9a26773e, 100 },
    { 0xa0b6773e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14867744, 23 },
    { 0xed267745, 53 },
    { 0xdda67746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f67749, 61 },
    { 0x3f267749, 76 },
    { 0xefb67749, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb536774e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5267752, 67 },
    { 0, 0 },
    { 0x9867754, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d67758, 98 },
    { 0, 0 },
    { 0x4056775a, 98 },
    { 0x1fc6775b, 95 },
    { 0x2736775c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93467762, 61 },
    { 0, 0 },
    { 0xe7067764, 90 },
    { 0, 0 },
    { 0x7f767766, 43 },
    { 0, 0 },
    { 0xc5267768, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e46776b, 33 },
    { 0xcb6776c, 61 },
    { 0xc526776b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b67771, 98 },
    { 0x83c67771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6067775, 33 },
    { 0x8ca67776, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf767779, 99 },
    { 0xe6a6777a, 90 },
    { 0xa226777a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff67782, 65 },
    { 0, 0 },
    { 0xd4667784, 95 },
    { 0x60c67785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c67789, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa266778f, 98 },
    { 0xb6b6778f, 93 },
    { 0x63267791, 61 },
    { 0xfb967790, 93 },
    { 0, 0 },
    { 0x37167794, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18767799, 76 },
    { 0x5e267799, 61 },
    { 0xaf567799, 91 },
    { 0x28d6779a, 87 },
    { 0x3f56779d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7677a0, 95 },
    { 0, 0 },
    { 0x127677a2, 61 },
    { 0xb06677a2, 90 },
    { 0x74c677a3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b677a7, 90 },
    { 0x26c677a7, 93 },
    { 0, 0 },
    { 0x5cf677aa, 95 },
    { 0x712677aa, 61 },
    { 0, 0 },
    { 0x181677ad, 78 },
    { 0x5cf677ae, 95 },
    { 0x829677af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd677b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b677b5, 99 },
    { 0xed1677b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00677b9, 84 },
    { 0xec4677ba, 90 },
    { 0, 0 },
    { 0xedc677bc, 98 },
    { 0, 0 },
    { 0x82a677be, 99 },
    { 0, 0 },
    { 0x99e677c0, 98 },
    { 0xc9b677c1, 23 },
    { 0, 0 },
    { 0xc92677c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77677cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b677d3, 61 },
    { 0xcc2677d4, 98 },
    { 0x3a6677d5, 55 },
    { 0x5ba677d6, 95 },
    { 0x73d677d6, 43 },
    { 0x6bd677d8, 95 },
    { 0x996677d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8677e4, 93 },
    { 0xc31677e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3677e8, 90 },
    { 0xbe2677e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3677ec, 77 },
    { 0xb86677ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8677f1, 76 },
    { 0x82a677f2, 95 },
    { 0, 0 },
    { 0x698677f4, 90 },
    { 0x517677f5, 84 },
    { 0xb4b677f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85677fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72677fd, 53 },
    { 0xe5e677fe, 23 },
    { 0xdf5677fe, 76 },
    { 0, 0 },
    { 0xd0f67801, 98 },
    { 0x25a67802, 23 },
    { 0, 0 },
    { 0xb8867804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7086780e, 95 },
    { 0xf516780f, 98 },
    { 0x33667810, 34 },
    { 0x98a6780e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd67815, 43 },
    { 0, 0 },
    { 0xe5467817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae6781d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b67821, 90 },
    { 0x5c267822, 34 },
    { 0x48e67821, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb67827, 23 },
    { 0, 0 },
    { 0x85167829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2306782c, 81 },
    { 0xbcf6782c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a67830, 61 },
    { 0, 0 },
    { 0x2ab67832, 90 },
    { 0xb4067833, 23 },
    { 0, 0 },
    { 0x2c467835, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4476783f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f67846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9246784b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6784f, 76 },
    { 0, 0 },
    { 0x87767851, 48 },
    { 0x13067852, 87 },
    { 0xd0567852, 40 },
    { 0x2e367854, 75 },
    { 0xf1367855, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a867858, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c6785b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec167861, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81967864, 84 },
    { 0x65f67864, 77 },
    { 0, 0 },
    { 0x31067867, 84 },
    { 0, 0 },
    { 0x9ab67869, 93 },
    { 0, 0 },
    { 0x4e16786b, 55 },
    { 0x1c86786c, 33 },
    { 0xb866786d, 33 },
    { 0x5d26786d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b67880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15867883, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf416788b, 91 },
    { 0x776788b, 98 },
    { 0x9ca6788b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8767890, 77 },
    { 0x9b267890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d6789c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777678a2, 52 },
    { 0, 0 },
    { 0xf53678a4, 55 },
    { 0, 0 },
    { 0x91e678a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x713678a9, 43 },
    { 0xd87678aa, 17 },
    { 0x15e678ab, 77 },
    { 0xde0678ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f678b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb678b3, 61 },
    { 0xb1a678b3, 15 },
    { 0x7b678b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a678b8, 61 },
    { 0, 0 },
    { 0x4a8678ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d678bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5678c2, 61 },
    { 0, 0 },
    { 0xdcf678c4, 4 },
    { 0x71f678c5, 77 },
    { 0x65a678c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5678ca, 67 },
    { 0, 0 },
    { 0x4ca678cc, 61 },
    { 0xaab678cd, 55 },
    { 0x41d678ce, 100 },
    { 0xea678cf, 55 },
    { 0, 0 },
    { 0xa88678d1, 61 },
    { 0xed7678d2, 98 },
    { 0, 0 },
    { 0xd07678d4, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x674678d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14678da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23678e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a678e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09678e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12678e9, 34 },
    { 0x418678e9, 85 },
    { 0x4ac678e9, 88 },
    { 0xb3f678ec, 87 },
    { 0x685678ec, 76 },
    { 0xa05678ee, 66 },
    { 0xa71678ec, 78 },
    { 0x84f678f0, 90 },
    { 0x67d678ef, 90 },
    { 0, 0 },
    { 0x9df678f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2678f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce67901, 44 },
    { 0, 0 },
    { 0x7c167903, 95 },
    { 0x38867904, 76 },
    { 0, 0 },
    { 0x62267906, 23 },
    { 0x70867907, 61 },
    { 0x2f67908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf6790b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d86790f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14267926, 61 },
    { 0, 0 },
    { 0x4e867928, 93 },
    { 0x79567928, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d86792c, 98 },
    { 0x7d96792d, 23 },
    { 0xefc6792d, 52 },
    { 0x1096792f, 95 },
    { 0, 0 },
    { 0x78d67931, 23 },
    { 0xaa067931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a67937, 76 },
    { 0x33867938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f6793c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a06793f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc67942, 93 },
    { 0x83867943, 33 },
    { 0x70e67944, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c67948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1726794b, 43 },
    { 0, 0 },
    { 0xc46794d, 33 },
    { 0, 0 },
    { 0xcb26794f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c67959, 33 },
    { 0xc056795a, 91 },
    { 0xf896795a, 96 },
    { 0xde96795c, 66 },
    { 0x3456795a, 33 },
    { 0xa586795e, 85 },
    { 0x8e96795f, 76 },
    { 0, 0 },
    { 0x20c67961, 76 },
    { 0, 0 },
    { 0x1fb67963, 90 },
    { 0x68567964, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a67967, 90 },
    { 0x30767968, 98 },
    { 0x58e67969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1867971, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26797a, 23 },
    { 0xa5a6797a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60967988, 19 },
    { 0x79667988, 34 },
    { 0xaec6798a, 33 },
    { 0, 0 },
    { 0x2356798c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf567995, 61 },
    { 0xd6d67996, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2066799a, 98 },
    { 0x3c66799a, 90 },
    { 0, 0 },
    { 0xb406799d, 23 },
    { 0, 0 },
    { 0xd4e6799f, 33 },
    { 0x4d5679a0, 91 },
    { 0xd22679a1, 83 },
    { 0, 0 },
    { 0x875679a3, 90 },
    { 0x26679a3, 95 },
    { 0x6f679a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd679af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa679b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f679ba, 98 },
    { 0xee9679bb, 76 },
    { 0x89d679bb, 69 },
    { 0xdda679bb, 52 },
    { 0x2eb679be, 43 },
    { 0x91f679bb, 95 },
    { 0xeb4679bb, 100 },
    { 0, 0 },
    { 0xc40679c2, 76 },
    { 0xef679c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7679c6, 33 },
    { 0xbb7679c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a679cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5679d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34679d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a679d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed679e3, 46 },
    { 0x462679e3, 67 },
    { 0x9ac679e5, 90 },
    { 0x3e8679e3, 33 },
    { 0xe8d679e5, 98 },
    { 0x495679e8, 90 },
    { 0xde6679e8, 33 },
    { 0x9d6679ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8679ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06679f0, 55 },
    { 0, 0 },
    { 0xd27679f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8679f9, 76 },
    { 0xde8679fa, 95 },
    { 0x8b3679f9, 95 },
    { 0xa1c679f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c467a02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4867a06, 61 },
    { 0x6b267a07, 61 },
    { 0xaac67a08, 90 },
    { 0xbc667a09, 76 },
    { 0xd9167a0a, 23 },
    { 0x10e67a0b, 88 },
    { 0xf7567a06, 76 },
    { 0xa4867a0d, 98 },
    { 0x46267a0d, 43 },
    { 0xfe467a0d, 85 },
    { 0xa2267a0c, 84 },
    { 0x35a67a11, 23 },
    { 0, 0 },
    { 0x25067a13, 46 },
    { 0x63d67a14, 90 },
    { 0xcc667a13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97067a19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9467a1f, 76 },
    { 0xe0c67a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea967a23, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8167a28, 52 },
    { 0xc8f67a29, 33 },
    { 0xbc467a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b67a31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77867a38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a67a43, 33 },
    { 0x4f267a44, 90 },
    { 0xb0167a45, 78 },
    { 0x9db67a45, 95 },
    { 0x85667a45, 98 },
    { 0x23567a44, 55 },
    { 0xf5567a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91267a50, 23 },
    { 0x90067a51, 33 },
    { 0xc0967a52, 33 },
    { 0x2367a53, 46 },
    { 0xe3d67a50, 83 },
    { 0xe8f67a51, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc67a60, 95 },
    { 0xf6867a61, 90 },
    { 0, 0 },
    { 0x2e867a63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed067a6a, 90 },
    { 0x9b267a6a, 33 },
    { 0xca767a6b, 55 },
    { 0x56067a6a, 23 },
    { 0xf0567a6d, 61 },
    { 0xd6067a6f, 61 },
    { 0xb7e67a70, 98 },
    { 0xb067a71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e167a74, 100 },
    { 0, 0 },
    { 0x54367a76, 33 },
    { 0, 0 },
    { 0x20d67a78, 76 },
    { 0x99967a78, 90 },
    { 0x1bd67a7a, 61 },
    { 0x48967a7b, 98 },
    { 0x3dd67a7c, 91 },
    { 0, 0 },
    { 0xfd167a7e, 95 },
    { 0, 0 },
    { 0x34c67a80, 76 },
    { 0, 0 },
    { 0xe5767a82, 76 },
    { 0x82a67a83, 95 },
    { 0x3db67a82, 17 },
    { 0x9bd67a85, 61 },
    { 0xa7e67a85, 55 },
    { 0x55367a87, 98 },
    { 0x30567a88, 23 },
    { 0xcf567a89, 81 },
    { 0, 0 },
    { 0x10f67a8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0767a92, 76 },
    { 0, 0 },
    { 0xf6167a94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da67a97, 52 },
    { 0x51067a97, 90 },
    { 0, 0 },
    { 0x71f67a9a, 90 },
    { 0, 0 },
    { 0xe6667a9c, 95 },
    { 0xcc167a9c, 82 },
    { 0x3f367a9c, 95 },
    { 0x8bd67a9f, 90 },
    { 0x58a67a9f, 33 },
    { 0x54067aa1, 93 },
    { 0x70b67aa2, 52 },
    { 0xd9667aa2, 66 },
    { 0x10a67aa3, 95 },
    { 0xce767aa2, 23 },
    { 0xa7b67aa6, 40 },
    { 0x29a67aa7, 43 },
    { 0xbca67aa8, 33 },
    { 0xb3767aa9, 98 },
    { 0x6767aa9, 99 },
    { 0xd1067aa9, 85 },
    { 0x47767aac, 23 },
    { 0x50b67aac, 34 },
    { 0x35f67aa9, 76 },
    { 0, 0 },
    { 0x46467ab0, 34 },
    { 0x72767ab1, 34 },
    { 0xee867ab2, 95 },
    { 0xf5967ab3, 61 },
    { 0x87467ab1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67aba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7367ac0, 95 },
    { 0, 0 },
    { 0x7de67ac2, 34 },
    { 0x62867ac2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c67ac6, 81 },
    { 0x48a67ac6, 85 },
    { 0x9e067ac8, 38 },
    { 0x4bb67ac9, 95 },
    { 0x9c867ac7, 98 },
    { 0x31b67aca, 67 },
    { 0x1c667acc, 90 },
    { 0, 0 },
    { 0x1af67ace, 65 },
    { 0, 0 },
    { 0x8f467ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a767ada, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce167ae4, 90 },
    { 0, 0 },
    { 0xc3867ae6, 90 },
    { 0x75b67ae7, 53 },
    { 0, 0 },
    { 0x1e667ae9, 66 },
    { 0x4b367ae9, 99 },
    { 0, 0 },
    { 0x2c767aec, 98 },
    { 0, 0 },
    { 0x2ea67aee, 48 },
    { 0, 0 },
    { 0xd4a67af0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae267af4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69867af9, 61 },
    { 0, 0 },
    { 0xbe467afb, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b67b01, 34 },
    { 0x5f767b01, 98 },
    { 0x49267b03, 46 },
    { 0xdf767b02, 100 },
    { 0, 0 },
    { 0xb1867b06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de67b0a, 61 },
    { 0, 0 },
    { 0x3b167b0c, 17 },
    { 0x3f467b0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb67b10, 61 },
    { 0xe5e67b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb67b16, 66 },
    { 0x19c67b17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x95467b1a, 61 },
    { 0, 0 },
    { 0x3c167b1c, 98 },
    { 0xb3867b1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e67b23, 78 },
    { 0x85867b24, 51 },
    { 0, 0 },
    { 0xec267b26, 90 },
    { 0x12867b27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea367b2a, 93 },
    { 0, 0 },
    { 0x30367b2c, 52 },
    { 0x5a67b2c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e67b34, 99 },
    { 0xee267b35, 23 },
    { 0x5b767b36, 33 },
    { 0x7da67b36, 95 },
    { 0, 0 },
    { 0x29c67b39, 61 },
    { 0xa6667b39, 53 },
    { 0x5df67b39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e867b3f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b67b42, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6367b46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x38267b49, 76 },
    { 0x15767b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf67b4f, 33 },
    { 0x38a67b50, 95 },
    { 0, 0 },
    { 0x4a767b52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78967b59, 66 },
    { 0xdad67b5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd67b5d, 23 },
    { 0, 0 },
    { 0x77e67b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe967b63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42967b69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38167b6d, 93 },
    { 0x69167b6d, 23 },
    { 0, 0 },
    { 0xcac67b70, 33 },
    { 0x86867b71, 95 },
    { 0xb6767b71, 33 },
    { 0x53167b73, 95 },
    { 0, 0 },
    { 0xdfd67b75, 76 },
    { 0x18767b76, 61 },
    { 0xa7867b77, 23 },
    { 0xdaf67b78, 52 },
    { 0x27d67b78, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2167b7e, 95 },
    { 0xc3667b7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc67b84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d567b89, 33 },
    { 0x76767b8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1867b8d, 46 },
    { 0xce467b8d, 33 },
    { 0x86767b8f, 23 },
    { 0xf6b67b90, 81 },
    { 0xd5a67b90, 23 },
    { 0, 0 },
    { 0x8e367b93, 77 },
    { 0x5e967b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5567b9c, 90 },
    { 0x39167b9d, 90 },
    { 0, 0 },
    { 0x58767b9f, 76 },
    { 0, 0 },
    { 0xe3b67ba1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4567ba7, 93 },
    { 0, 0 },
    { 0x7c467ba9, 88 },
    { 0, 0 },
    { 0xd6867bab, 61 },
    { 0x28f67bac, 66 },
    { 0x66d67bac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d67bb5, 100 },
    { 0, 0 },
    { 0x9ab67bb7, 43 },
    { 0x6a367bb7, 33 },
    { 0x9267bb9, 77 },
    { 0x11c67bba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68067bbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d467bc7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95267bca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56067bcf, 61 },
    { 0xae667bd0, 61 },
    { 0xa4767bd0, 88 },
    { 0x50567bd0, 95 },
    { 0x43067bd2, 69 },
    { 0, 0 },
    { 0xf6e67bd5, 100 },
    { 0, 0 },
    { 0x90d67bd7, 34 },
    { 0xbde67bd7, 100 },
    { 0x7b967bd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6767bde, 84 },
    { 0x2367bdf, 98 },
    { 0x6f967bde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11367be3, 33 },
    { 0xc1367be4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21967be9, 93 },
    { 0xd8367bea, 88 },
    { 0xda967bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31767bf7, 90 },
    { 0, 0 },
    { 0xdd767bf9, 95 },
    { 0xe2d67bf9, 65 },
    { 0x8b767bfa, 84 },
    { 0x63a67bfc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a67c02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9567c05, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc167c0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60267c0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf767c12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc067c16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae467c1a, 90 },
    { 0, 0 },
    { 0x5b67c1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48367c20, 61 },
    { 0x32d67c21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b67c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90967c2c, 33 },
    { 0xc8567c2d, 90 },
    { 0xa6a67c2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e67c31, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc67c3c, 95 },
    { 0x44f67c3d, 40 },
    { 0xe3967c3e, 34 },
    { 0x9c267c3f, 43 },
    { 0x8b867c40, 55 },
    { 0xc9c67c3f, 23 },
    { 0, 0 },
    { 0x42c67c43, 87 },
    { 0xf7967c43, 61 },
    { 0, 0 },
    { 0xe7167c46, 68 },
    { 0, 0 },
    { 0xc1667c48, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67c4b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b067c58, 77 },
    { 0x4a267c59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46967c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78467c61, 100 },
    { 0xb4667c61, 91 },
    { 0x7ac67c63, 99 },
    { 0x28867c63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b67c6e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20867c75, 93 },
    { 0x40c67c76, 23 },
    { 0x2dd67c77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b67c84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2267c8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59767c90, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb67c94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a367c97, 96 },
    { 0, 0 },
    { 0xfdc67c99, 36 },
    { 0xa5c67c99, 23 },
    { 0, 0 },
    { 0xfbf67c9c, 90 },
    { 0x7c067c9d, 80 },
    { 0x53067c9e, 76 },
    { 0xa8b67c9f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d67ca2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d67ca8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1467cab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92767cb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae67cb4, 34 },
    { 0x7b067cb5, 61 },
    { 0xa2f67cb5, 90 },
    { 0xfc067cb5, 23 },
    { 0x87667cb6, 93 },
    { 0x38667cb8, 34 },
    { 0x1e367cb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe067cbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66967cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd67cc5, 68 },
    { 0x4ee67cc6, 98 },
    { 0x19c67cc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14767cd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6467cd9, 23 },
    { 0x5bd67cda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5767ce0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb67ce5, 55 },
    { 0xcf367ce5, 43 },
    { 0x48967ce7, 95 },
    { 0x7e167ce7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c67ced, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3767cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5767d02, 78 },
    { 0x8c167d02, 52 },
    { 0xac467d04, 95 },
    { 0x15667d05, 90 },
    { 0, 0 },
    { 0xc7067d07, 33 },
    { 0xdb867d08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe67d10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15967d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8167d1c, 36 },
    { 0x43e67d1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b67d23, 90 },
    { 0xb2967d24, 84 },
    { 0x16567d25, 23 },
    { 0x65967d26, 33 },
    { 0xad67d27, 55 },
    { 0x83167d28, 51 },
    { 0x93167d29, 95 },
    { 0, 0 },
    { 0xa9b67d2b, 55 },
    { 0x42067d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9367d2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a467d36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c67d3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe367d3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f67d49, 33 },
    { 0x7bd67d4a, 17 },
    { 0, 0 },
    { 0xc3c67d4c, 98 },
    { 0xb3567d4c, 33 },
    { 0, 0 },
    { 0x49367d4f, 98 },
    { 0x1a067d50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93267d58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd67d5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23067d60, 88 },
    { 0x76167d61, 90 },
    { 0xbad67d60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f667d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab867d6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8667d72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93967d75, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf67d7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e67d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef67d89, 43 },
    { 0x5a467d89, 76 },
    { 0, 0 },
    { 0xcab67d8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15167d92, 95 },
    { 0x9c67d93, 33 },
    { 0xd2f67d94, 91 },
    { 0x6f67d95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd967d9a, 61 },
    { 0xa6667d9b, 90 },
    { 0x99367d9c, 91 },
    { 0xbd967d9d, 90 },
    { 0xab467d9d, 98 },
    { 0x4bd67d9c, 100 },
    { 0x44c67d9b, 17 },
    { 0x27767d9e, 100 },
    { 0x91767da2, 33 },
    { 0x58d67d9a, 84 },
    { 0x63767da4, 96 },
    { 0xed867da5, 46 },
    { 0x96667da6, 34 },
    { 0x1567da6, 100 },
    { 0x11867da8, 75 },
    { 0x9e067d9b, 95 },
    { 0xf4867da1, 90 },
    { 0xd967dab, 33 },
    { 0x3a667dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c67db1, 97 },
    { 0x89367db2, 90 },
    { 0x9a67db3, 33 },
    { 0x61367db3, 23 },
    { 0xced67db5, 90 },
    { 0x7fe67db6, 43 },
    { 0, 0 },
    { 0x50c67db8, 90 },
    { 0, 0 },
    { 0x1f67dba, 73 },
    { 0, 0 },
    { 0xc4067dbc, 34 },
    { 0, 0 },
    { 0xa1467dbe, 95 },
    { 0xf4267dbf, 23 },
    { 0xb9667dc0, 46 },
    { 0xb9067dc1, 68 },
    { 0xdb767dc2, 93 },
    { 0x17667dc2, 61 },
    { 0, 0 },
    { 0x93967dc5, 66 },
    { 0x62b67dc6, 90 },
    { 0x8ec67dc6, 33 },
    { 0, 0 },
    { 0x2667dc9, 61 },
    { 0x17567dca, 85 },
    { 0x61c67dca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xac567dce, 33 },
    { 0x9e367dcf, 23 },
    { 0x3e867dcf, 46 },
    { 0xe2567dce, 90 },
    { 0x9267dce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e67dd9, 66 },
    { 0, 0 },
    { 0xbca67ddb, 61 },
    { 0x66367ddb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d367de0, 61 },
    { 0x30967de1, 61 },
    { 0, 0 },
    { 0xd3667de3, 17 },
    { 0xe5c67de3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d67de7, 90 },
    { 0x2c167de8, 80 },
    { 0xced67de9, 52 },
    { 0, 0 },
    { 0x7ae67deb, 61 },
    { 0x1f967dec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1267df2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0467df6, 84 },
    { 0, 0 },
    { 0xe0e67df8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3567dfb, 61 },
    { 0xa3867dfc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a67dff, 90 },
    { 0, 0 },
    { 0x44367e01, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63467e06, 61 },
    { 0x61b67e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5367e0f, 33 },
    { 0xe3167e10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c67e13, 87 },
    { 0xfe567e14, 90 },
    { 0, 0 },
    { 0xf5d67e16, 55 },
    { 0x4867e16, 95 },
    { 0xc6467e16, 75 },
    { 0, 0 },
    { 0x66267e1a, 81 },
    { 0xda467e1b, 77 },
    { 0xfa267e1c, 23 },
    { 0x5ae67e1d, 84 },
    { 0, 0 },
    { 0x4f567e1f, 84 },
    { 0xd9b67e1f, 61 },
    { 0x30267e21, 85 },
    { 0x8e767e21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c67e2c, 34 },
    { 0x5cc67e2d, 33 },
    { 0xd5b67e2e, 23 },
    { 0xca867e2f, 85 },
    { 0, 0 },
    { 0x8c867e31, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d67e39, 90 },
    { 0, 0 },
    { 0x27967e3b, 61 },
    { 0, 0 },
    { 0xe0067e3d, 23 },
    { 0x6aa67e3d, 61 },
    { 0xe9a67e3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51367e42, 99 },
    { 0xefc67e43, 65 },
    { 0xab467e44, 34 },
    { 0x16867e45, 43 },
    { 0xbe867e46, 84 },
    { 0xca367e47, 90 },
    { 0xb8967e48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff67e4b, 61 },
    { 0, 0 },
    { 0x64f67e4d, 61 },
    { 0xc7567e4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48667e52, 33 },
    { 0x92867e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec67e59, 87 },
    { 0, 0 },
    { 0x1c967e5b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd67e5f, 33 },
    { 0xe9267e60, 24 },
    { 0, 0 },
    { 0xd2367e62, 43 },
    { 0x6a367e63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x60767e66, 66 },
    { 0xfa267e67, 78 },
    { 0xa3a67e68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46367e71, 23 },
    { 0xf1c67e71, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d67e78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18867e7b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b667e7f, 23 },
    { 0xb7067e7f, 90 },
    { 0xea967e7f, 17 },
    { 0x99267e80, 23 },
    { 0xbea67e81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c767e86, 40 },
    { 0, 0 },
    { 0xe9567e88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3667e8b, 78 },
    { 0x6d467e8b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0467e98, 90 },
    { 0, 0 },
    { 0xd7967e9a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e467ea8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad67eab, 61 },
    { 0x4e067eab, 84 },
    { 0x2eb67ead, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a67eb0, 81 },
    { 0, 0 },
    { 0x22667eb2, 98 },
    { 0xe9067eb2, 17 },
    { 0, 0 },
    { 0xac67eb5, 61 },
    { 0, 0 },
    { 0xc2567eb7, 23 },
    { 0xcf767eb8, 88 },
    { 0xf8067eb8, 95 },
    { 0x26667eb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce567ec1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd67ec6, 28 },
    { 0x36167ec7, 78 },
    { 0, 0 },
    { 0x10267ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4067ecf, 55 },
    { 0, 0 },
    { 0xa0867ed1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55767ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1867ee0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4267ee3, 61 },
    { 0x42667ee3, 23 },
    { 0, 0 },
    { 0x9bb67ee6, 33 },
    { 0, 0 },
    { 0x67d67ee8, 34 },
    { 0x2cb67ee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a67eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2667ef0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b767ef4, 95 },
    { 0, 0 },
    { 0x9c767ef6, 77 },
    { 0x9c67ef7, 52 },
    { 0x6e567ef8, 90 },
    { 0x33467ef9, 61 },
    { 0x41467efa, 61 },
    { 0x32267ef8, 33 },
    { 0x2f067efa, 76 },
    { 0xec567efb, 95 },
    { 0x67967efe, 17 },
    { 0x36967eff, 97 },
    { 0x4dd67eff, 53 },
    { 0xff667f01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f467f09, 69 },
    { 0x59167f0a, 98 },
    { 0xe0867f0b, 46 },
    { 0xb1367f0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff967f10, 61 },
    { 0, 0 },
    { 0x2e167f12, 69 },
    { 0x9c067f13, 95 },
    { 0x5ab67f13, 93 },
    { 0, 0 },
    { 0x9b467f16, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0x11267f19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca67f20, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e67f26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f67f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a67f2e, 88 },
    { 0x2b967f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab367f35, 75 },
    { 0, 0 },
    { 0xa8d67f37, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb67f3d, 56 },
    { 0, 0 },
    { 0x40867f3f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa967f43, 23 },
    { 0x75167f43, 23 },
    { 0xbce67f44, 76 },
    { 0x2a367f46, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc767f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff367f4c, 33 },
    { 0x33167f4d, 61 },
    { 0x82067f4e, 90 },
    { 0, 0 },
    { 0x47e67f50, 76 },
    { 0x9a267f51, 87 },
    { 0x17a67f51, 90 },
    { 0x81267f51, 76 },
    { 0x5dc67f53, 33 },
    { 0xe3167f50, 33 },
    { 0xd8567f56, 34 },
    { 0x5c767f54, 93 },
    { 0xcc867f58, 61 },
    { 0x3ab67f59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa67f5c, 90 },
    { 0xb7567f5d, 67 },
    { 0x9d267f5d, 61 },
    { 0, 0 },
    { 0xcd967f60, 81 },
    { 0x6aa67f61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b67f65, 98 },
    { 0xb0e67f66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79667f6d, 98 },
    { 0xb2a67f6d, 33 },
    { 0x91967f6f, 33 },
    { 0x8c667f6f, 90 },
    { 0, 0 },
    { 0xac667f72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc67f75, 84 },
    { 0xe767f75, 98 },
    { 0x44c67f76, 95 },
    { 0, 0 },
    { 0x3f067f79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2767f81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b467f88, 95 },
    { 0x8867f88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4367f8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d467f90, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb67f96, 23 },
    { 0xdd67f96, 23 },
    { 0x55267f96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x90167f9b, 90 },
    { 0, 0 },
    { 0x667f9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb167fa3, 17 },
    { 0x4ed67fa4, 98 },
    { 0, 0 },
    { 0x93e67fa6, 90 },
    { 0, 0 },
    { 0xb2a67fa8, 76 },
    { 0xcb567fa8, 99 },
    { 0xc6267faa, 77 },
    { 0xc2c67fab, 87 },
    { 0xffb67fac, 23 },
    { 0x3fd67fab, 98 },
    { 0x6ff67fae, 68 },
    { 0xdd367fa9, 23 },
    { 0, 0 },
    { 0x8f767fb1, 91 },
    { 0, 0 },
    { 0x5cd67fb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c167fb6, 23 },
    { 0xf767fb6, 75 },
    { 0, 0 },
    { 0x4b867fb9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64967fbd, 98 },
    { 0, 0 },
    { 0xf2567fbf, 87 },
    { 0xeac67fc0, 46 },
    { 0xc8067fbf, 90 },
    { 0x42d67fc2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0067fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a67fcb, 98 },
    { 0, 0 },
    { 0x8a767fcd, 23 },
    { 0xbcf67fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba867fd1, 95 },
    { 0xd1a67fd2, 95 },
    { 0, 0 },
    { 0x90967fd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a67fd8, 95 },
    { 0, 0 },
    { 0x7e567fda, 23 },
    { 0x66967fdb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f667fdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b67fe3, 55 },
    { 0, 0 },
    { 0x48867fe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf67fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab67fef, 33 },
    { 0xc067ff0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d67ff6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad167ff9, 45 },
    { 0xb8d67ff9, 33 },
    { 0x4e167ff9, 84 },
    { 0xb3c67ffc, 61 },
    { 0xe0267ffd, 23 },
    { 0xda767ffd, 23 },
    { 0x59e67ffc, 61 },
    { 0, 0 },
    { 0xeed68001, 81 },
    { 0xcfa68001, 77 },
    { 0xf2468001, 93 },
    { 0x94a68004, 61 },
    { 0x51868005, 95 },
    { 0x52868006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2168009, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e568014, 90 },
    { 0xb4e68015, 61 },
    { 0xb7068016, 90 },
    { 0, 0 },
    { 0x73f68018, 91 },
    { 0x74f68019, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a568021, 99 },
    { 0x8c68022, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43068029, 93 },
    { 0x4c06802a, 91 },
    { 0x8236802a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6768030, 85 },
    { 0x9a468031, 95 },
    { 0x9df68030, 95 },
    { 0, 0 },
    { 0xb6768034, 90 },
    { 0x25068035, 77 },
    { 0xf5d68035, 61 },
    { 0xbf568037, 99 },
    { 0xdfc68035, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a68040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2268049, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546804d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4768051, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e968055, 33 },
    { 0xfdf68056, 88 },
    { 0x60c68057, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a36805a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b68066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd6806e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b68071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa68075, 61 },
    { 0, 0 },
    { 0x1e68077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a868082, 33 },
    { 0x9a368082, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb68087, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6808a, 33 },
    { 0x8a26808a, 33 },
    { 0x69a6808c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec6808f, 68 },
    { 0xe296808f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae68096, 95 },
    { 0x2168097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596809d, 61 },
    { 0x6fa6809e, 77 },
    { 0x7d6809d, 100 },
    { 0, 0 },
    { 0x222680a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6680a4, 43 },
    { 0xf64680a4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90680a9, 34 },
    { 0x439680aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71680ad, 95 },
    { 0x271680ae, 95 },
    { 0x5fe680af, 74 },
    { 0xbea680b0, 78 },
    { 0x199680b1, 91 },
    { 0, 0 },
    { 0x5f6680b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319680b7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b680ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119680bf, 88 },
    { 0, 0 },
    { 0x2c7680c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff680c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d680ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89680d0, 97 },
    { 0, 0 },
    { 0xdb2680d2, 93 },
    { 0x6d0680d2, 76 },
    { 0, 0 },
    { 0x526680d5, 46 },
    { 0xcf5680d5, 95 },
    { 0x738680d7, 90 },
    { 0x8a680d6, 61 },
    { 0xa83680d9, 23 },
    { 0x757680d9, 60 },
    { 0x289680db, 61 },
    { 0x7c2680dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89680e1, 23 },
    { 0xbe1680e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3680e8, 23 },
    { 0xb6a680e9, 33 },
    { 0, 0 },
    { 0xf4f680eb, 90 },
    { 0xe0f680eb, 33 },
    { 0, 0 },
    { 0x134680ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac680f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb680f8, 52 },
    { 0x32e680f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x249680fc, 98 },
    { 0x377680fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1868105, 34 },
    { 0x4768105, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x77968109, 68 },
    { 0xb9f68109, 90 },
    { 0, 0 },
    { 0xcd56810c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a68115, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c36811d, 90 },
    { 0x68a6811e, 33 },
    { 0, 0 },
    { 0xc8a68120, 90 },
    { 0x4b168120, 98 },
    { 0xe4068122, 23 },
    { 0, 0 },
    { 0x60b68124, 55 },
    { 0xacd68125, 76 },
    { 0xa8668126, 96 },
    { 0xec768127, 33 },
    { 0x6b468127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6286812c, 33 },
    { 0x73b6812d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f168133, 90 },
    { 0, 0 },
    { 0xe1368135, 61 },
    { 0xf3b68136, 95 },
    { 0x9a268135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa936813a, 23 },
    { 0, 0 },
    { 0x4206813c, 95 },
    { 0, 0 },
    { 0xa6b6813e, 61 },
    { 0xb766813f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6068144, 23 },
    { 0x3ce68145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d568148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af6814d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb268150, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b68154, 33 },
    { 0, 0 },
    { 0x24968156, 33 },
    { 0xdf868157, 53 },
    { 0, 0 },
    { 0x1a268159, 76 },
    { 0, 0 },
    { 0xfa26815b, 34 },
    { 0x1a6815c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc68163, 95 },
    { 0x82668164, 46 },
    { 0xb3668165, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24668168, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e6816b, 23 },
    { 0x3566816c, 90 },
    { 0xa136816c, 33 },
    { 0xed26816c, 98 },
    { 0xac46816d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e68175, 76 },
    { 0xe3868175, 90 },
    { 0, 0 },
    { 0xd5968178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa06817b, 97 },
    { 0, 0 },
    { 0x186817d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c68181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b368184, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a68187, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d46818a, 90 },
    { 0xbce6818b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6818e, 61 },
    { 0xd056818f, 33 },
    { 0x1a068190, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d68197, 46 },
    { 0, 0 },
    { 0xa5568199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe6819c, 98 },
    { 0x54e6819d, 99 },
    { 0xe596819d, 95 },
    { 0xe2e6819f, 61 },
    { 0xbd76819f, 100 },
    { 0x9ef6819e, 90 },
    { 0xb21681a2, 90 },
    { 0x2f7681a3, 43 },
    { 0xd3681a0, 93 },
    { 0x563681a0, 95 },
    { 0x7e1681a6, 95 },
    { 0xdf681a2, 93 },
    { 0xa076819e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a681ad, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4681b3, 65 },
    { 0x58a681b4, 33 },
    { 0xa45681b4, 76 },
    { 0x367681b6, 33 },
    { 0x1a8681b6, 34 },
    { 0x2aa681b8, 43 },
    { 0x52c681b9, 52 },
    { 0xbf0681b6, 90 },
    { 0, 0 },
    { 0xc7681bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x781681bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82681c2, 90 },
    { 0xe1f681c3, 49 },
    { 0xad6681c4, 46 },
    { 0x885681c5, 98 },
    { 0x2e5681c5, 43 },
    { 0xf51681c7, 61 },
    { 0xddc681c8, 61 },
    { 0x1da681c5, 91 },
    { 0x2d7681ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915681cf, 23 },
    { 0, 0 },
    { 0x58a681d1, 46 },
    { 0, 0 },
    { 0xdf0681d3, 34 },
    { 0xeaf681d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef681da, 34 },
    { 0xc2681db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8681df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f681e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9681e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08681ec, 33 },
    { 0, 0 },
    { 0x6bf681ee, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c681f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc681f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec681fc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c68201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa768205, 95 },
    { 0, 0 },
    { 0xadd68207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed86820c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d6820f, 43 },
    { 0xdb368210, 66 },
    { 0x7516820f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1768219, 43 },
    { 0x95e6821a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56821e, 85 },
    { 0xc136821f, 61 },
    { 0x9e068220, 95 },
    { 0x81068221, 90 },
    { 0x3f868222, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc668225, 76 },
    { 0x6c68226, 99 },
    { 0xf1168226, 81 },
    { 0xeec68226, 33 },
    { 0x80468226, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf6822c, 33 },
    { 0x5936822d, 76 },
    { 0x6586822d, 90 },
    { 0x5d66822f, 95 },
    { 0, 0 },
    { 0x37268231, 90 },
    { 0x6a968232, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc368235, 90 },
    { 0, 0 },
    { 0x4d568237, 33 },
    { 0, 0 },
    { 0x77968239, 55 },
    { 0x59c68239, 90 },
    { 0x9996823a, 75 },
    { 0xc276823c, 99 },
    { 0x5c56823d, 91 },
    { 0x1046823d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9168241, 61 },
    { 0x2ed68241, 61 },
    { 0x49868241, 55 },
    { 0x42e68244, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b768247, 95 },
    { 0x3668247, 23 },
    { 0x23168247, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd726824e, 55 },
    { 0x2836824f, 90 },
    { 0x7336824f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4468253, 55 },
    { 0x1ee68253, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2768257, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc76825a, 61 },
    { 0x5c86825b, 98 },
    { 0, 0 },
    { 0xc2b6825d, 23 },
    { 0x8bc6825d, 34 },
    { 0x8ec6825f, 98 },
    { 0xc7568260, 91 },
    { 0xec46825e, 90 },
    { 0x29868262, 33 },
    { 0, 0 },
    { 0xe3868264, 85 },
    { 0x89e68264, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19668268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf56826b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16668271, 55 },
    { 0x55868272, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a68278, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6586827e, 84 },
    { 0xad86827e, 66 },
    { 0xfa468280, 100 },
    { 0xc4c6827e, 95 },
    { 0, 0 },
    { 0x2c68283, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8168288, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f568290, 76 },
    { 0x64b68291, 61 },
    { 0, 0 },
    { 0xd2068293, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x60268296, 40 },
    { 0x84d68297, 34 },
    { 0, 0 },
    { 0xdfe68299, 90 },
    { 0x176829a, 90 },
    { 0, 0 },
    { 0x28b6829c, 87 },
    { 0x6ed6829d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae682a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e682a5, 84 },
    { 0xcbf682a5, 52 },
    { 0x14a682a7, 95 },
    { 0x1fc682a8, 67 },
    { 0x898682a9, 90 },
    { 0xd0f682aa, 90 },
    { 0x326682a9, 52 },
    { 0x8e9682a6, 61 },
    { 0x947682ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c682b2, 33 },
    { 0xb9b682b2, 23 },
    { 0xb65682b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04682b7, 93 },
    { 0, 0 },
    { 0x10d682b9, 78 },
    { 0xb6b682ba, 78 },
    { 0x206682b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a682c1, 95 },
    { 0xa1d682c2, 38 },
    { 0x12f682c1, 34 },
    { 0x14e682c4, 93 },
    { 0xdd1682c2, 33 },
    { 0x11682c6, 43 },
    { 0xded682c6, 33 },
    { 0x2b9682c5, 61 },
    { 0x7a4682c9, 88 },
    { 0x465682ca, 61 },
    { 0x86e682c9, 33 },
    { 0x9ac682cc, 23 },
    { 0xb3a682cd, 95 },
    { 0xa43682cd, 75 },
    { 0x94d682cf, 17 },
    { 0x701682ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38682d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36682d7, 52 },
    { 0x185682d7, 33 },
    { 0x7ab682d9, 61 },
    { 0, 0 },
    { 0x221682db, 33 },
    { 0xac9682dc, 23 },
    { 0, 0 },
    { 0x24e682de, 61 },
    { 0, 0 },
    { 0xec1682e0, 69 },
    { 0, 0 },
    { 0xd3c682e2, 95 },
    { 0x2a1682e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea682e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe682ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f682f0, 90 },
    { 0xc10682f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358682f7, 33 },
    { 0x9f5682f7, 90 },
    { 0, 0 },
    { 0xc01682fa, 61 },
    { 0xdd9682fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed682fe, 78 },
    { 0xa50682ff, 4 },
    { 0xade682ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae368305, 84 },
    { 0xa1f68306, 61 },
    { 0xcd468307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4166830a, 43 },
    { 0x6dc6830b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5596830f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a468317, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1168321, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd68324, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf468327, 84 },
    { 0x9d868327, 23 },
    { 0x50468329, 90 },
    { 0, 0 },
    { 0x3da6832b, 33 },
    { 0x6cc6832c, 55 },
    { 0x7a66832d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee68330, 61 },
    { 0, 0 },
    { 0x81868332, 23 },
    { 0x36568333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29568336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4826833c, 95 },
    { 0x55d6833d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906834a, 76 },
    { 0, 0 },
    { 0xb796834c, 90 },
    { 0xf156834d, 33 },
    { 0x3056834d, 76 },
    { 0, 0 },
    { 0x6c068350, 61 },
    { 0x67a68350, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5768355, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c6835e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a68369, 55 },
    { 0x8ec6836a, 46 },
    { 0xbd46836a, 100 },
    { 0xb6b6836a, 46 },
    { 0x5fe6836d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62868373, 91 },
    { 0, 0 },
    { 0xf8468375, 90 },
    { 0xae768376, 55 },
    { 0, 0 },
    { 0x59168378, 75 },
    { 0, 0 },
    { 0x9fa6837a, 52 },
    { 0x9716837a, 23 },
    { 0, 0 },
    { 0x58b6837d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e968382, 56 },
    { 0x6c668382, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a68387, 90 },
    { 0x8ab68387, 100 },
    { 0x81768387, 95 },
    { 0x5668389, 43 },
    { 0x3066838b, 76 },
    { 0xb416838a, 95 },
    { 0x666838d, 95 },
    { 0xea96838d, 90 },
    { 0x2ba6838f, 84 },
    { 0x65b68390, 61 },
    { 0x77668391, 76 },
    { 0xe9c68390, 95 },
    { 0, 0 },
    { 0x44368394, 23 },
    { 0x1aa68394, 76 },
    { 0x99068394, 55 },
    { 0x6e068394, 77 },
    { 0x54668398, 98 },
    { 0x73b68398, 76 },
    { 0x2a66839a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd6839e, 90 },
    { 0x82b6839e, 95 },
    { 0xa0c6839f, 67 },
    { 0, 0 },
    { 0x24d683a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930683a9, 99 },
    { 0xf92683a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e683ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74683b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1683b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58683b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0683ba, 64 },
    { 0x837683ba, 76 },
    { 0, 0 },
    { 0x7f5683bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e683c4, 33 },
    { 0x52e683c4, 46 },
    { 0x3a1683c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a683cd, 95 },
    { 0xf42683ce, 55 },
    { 0x78683cf, 95 },
    { 0xc05683cf, 95 },
    { 0x44f683d1, 33 },
    { 0, 0 },
    { 0x695683d3, 23 },
    { 0, 0 },
    { 0xd683d5, 33 },
    { 0x1e6683d6, 76 },
    { 0xa19683d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145683df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155683e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4683ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab683ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab683f0, 61 },
    { 0, 0 },
    { 0x8a3683f2, 43 },
    { 0x8bd683f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510683fa, 90 },
    { 0x613683fa, 52 },
    { 0, 0 },
    { 0x8b8683fd, 61 },
    { 0x707683fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3868403, 65 },
    { 0, 0 },
    { 0xa2268405, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee56840c, 61 },
    { 0x85a6840c, 34 },
    { 0x7ba6840d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c68412, 68 },
    { 0x3f168412, 33 },
    { 0, 0 },
    { 0x4c868415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8246841a, 61 },
    { 0xf6b6841b, 90 },
    { 0x10e6841c, 85 },
    { 0, 0 },
    { 0xb896841e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb68422, 33 },
    { 0x20e68422, 46 },
    { 0x88568424, 61 },
    { 0x90468422, 61 },
    { 0xc6768426, 34 },
    { 0xe2d68426, 76 },
    { 0, 0 },
    { 0x4eb68429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c68431, 78 },
    { 0x9b068432, 23 },
    { 0x2a068433, 35 },
    { 0, 0 },
    { 0x6cc68435, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24268439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9868440, 23 },
    { 0xa2b68441, 100 },
    { 0xcec68441, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b68445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd746844b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e68450, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6868454, 43 },
    { 0, 0 },
    { 0x68068456, 33 },
    { 0x23468457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd976845b, 34 },
    { 0x6fd6845c, 23 },
    { 0xcdc6845c, 98 },
    { 0xfa76845d, 95 },
    { 0x2ed6845e, 95 },
    { 0x65568460, 77 },
    { 0xf1168461, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11768467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc396846a, 90 },
    { 0, 0 },
    { 0xeeb6846c, 61 },
    { 0x61f6846d, 90 },
    { 0xe416846e, 90 },
    { 0xe8d6846f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf768473, 95 },
    { 0xf2768474, 98 },
    { 0x62968475, 61 },
    { 0x66a68474, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x60968479, 98 },
    { 0x95c6847a, 23 },
    { 0, 0 },
    { 0xd0e6847c, 34 },
    { 0x8fe6847d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40868481, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd768485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16868489, 90 },
    { 0x136848a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14468491, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb868495, 95 },
    { 0x3a268496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db6849a, 23 },
    { 0x8646849b, 90 },
    { 0, 0 },
    { 0xa556849d, 33 },
    { 0xcd16849e, 23 },
    { 0xe156849e, 66 },
    { 0x950684a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x484684a3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x604684a6, 61 },
    { 0x5b4684a7, 90 },
    { 0, 0 },
    { 0x726684a9, 23 },
    { 0x765684a9, 61 },
    { 0x789684aa, 61 },
    { 0xd24684a9, 95 },
    { 0x973684ad, 67 },
    { 0xf7684ae, 55 },
    { 0x352684af, 69 },
    { 0xa22684ad, 61 },
    { 0x7cc684b1, 76 },
    { 0xda5684b0, 76 },
    { 0x7c9684b3, 99 },
    { 0x7f2684b4, 95 },
    { 0xe0f684aa, 68 },
    { 0x5ba684b6, 68 },
    { 0, 0 },
    { 0xc5c684b8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7684c8, 90 },
    { 0x260684c8, 52 },
    { 0, 0 },
    { 0xd5684cb, 97 },
    { 0, 0 },
    { 0xca3684cd, 76 },
    { 0x80f684cd, 95 },
    { 0x205684cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f684d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2684db, 43 },
    { 0xff1684db, 76 },
    { 0x968684dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4684e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf684e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c684e8, 78 },
    { 0x6b1684e9, 88 },
    { 0xc13684e9, 90 },
    { 0x8d2684e9, 76 },
    { 0x29e684eb, 75 },
    { 0, 0 },
    { 0xbcf684ee, 90 },
    { 0x541684ee, 91 },
    { 0x5a6684f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d684f3, 23 },
    { 0x3e6684f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88684f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x655684fb, 36 },
    { 0xaa0684fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x790684ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c68502, 90 },
    { 0x5968502, 23 },
    { 0x8a68504, 43 },
    { 0x61a68505, 76 },
    { 0xf0968506, 90 },
    { 0xd6468504, 93 },
    { 0x6c468507, 90 },
    { 0xb9168507, 95 },
    { 0x61c68504, 93 },
    { 0x5eb6850b, 37 },
    { 0x8526850c, 55 },
    { 0x5e468509, 96 },
    { 0x5646850c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e668512, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b36851b, 23 },
    { 0, 0 },
    { 0x4276851d, 93 },
    { 0x59e6851d, 33 },
    { 0x5ae6851f, 68 },
    { 0xd3e68520, 87 },
    { 0, 0 },
    { 0xad168522, 33 },
    { 0, 0 },
    { 0xf3e68524, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4868528, 98 },
    { 0, 0 },
    { 0x2886852a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36852f, 76 },
    { 0, 0 },
    { 0xda268531, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5936853a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9336853e, 100 },
    { 0x8d76853f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b68542, 92 },
    { 0, 0 },
    { 0x69f68544, 64 },
    { 0x58368544, 61 },
    { 0xd9568546, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae6854a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea6854e, 23 },
    { 0x2a46854f, 61 },
    { 0, 0 },
    { 0x25568551, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3068555, 43 },
    { 0xcc068556, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2686855d, 67 },
    { 0x83d6855e, 95 },
    { 0x2b76855f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f68562, 98 },
    { 0x8e668563, 90 },
    { 0x1ce68564, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6068567, 46 },
    { 0x56b68568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd16856b, 33 },
    { 0x8586856c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15568571, 23 },
    { 0x6c768571, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b68577, 76 },
    { 0xbdb68578, 23 },
    { 0, 0 },
    { 0x9956857a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1976857d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6168587, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac6858c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6858f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b68592, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6859b, 55 },
    { 0xb1a6859b, 90 },
    { 0x24f6859c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5685a2, 98 },
    { 0, 0 },
    { 0x74e685a4, 46 },
    { 0xbcd685a4, 76 },
    { 0, 0 },
    { 0xb82685a7, 93 },
    { 0x54e685a7, 76 },
    { 0x11f685a9, 90 },
    { 0, 0 },
    { 0xd9f685ab, 90 },
    { 0xc1a685ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966685b1, 90 },
    { 0x6c685b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db685bb, 23 },
    { 0xca8685bb, 99 },
    { 0x835685bb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64685c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6685c6, 87 },
    { 0x557685c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c685cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5685d2, 90 },
    { 0x854685d3, 33 },
    { 0, 0 },
    { 0x4f9685d5, 98 },
    { 0x431685d5, 56 },
    { 0x753685d7, 95 },
    { 0, 0 },
    { 0x2fb685d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a685de, 100 },
    { 0x79b685de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9685e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af685e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b685ef, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0685f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e685f5, 33 },
    { 0x477685f6, 98 },
    { 0, 0 },
    { 0x6c5685f8, 98 },
    { 0x35d685f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d685fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca685ff, 90 },
    { 0x18368600, 33 },
    { 0x27568601, 61 },
    { 0x9bb68601, 58 },
    { 0, 0 },
    { 0x68168604, 90 },
    { 0x9fe68605, 34 },
    { 0xe7668606, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae56860b, 55 },
    { 0x5bb6860c, 95 },
    { 0xbb96860d, 90 },
    { 0x7886860d, 43 },
    { 0xed46860f, 43 },
    { 0xd136860d, 98 },
    { 0xba26860f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7968617, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a66861b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe506861e, 23 },
    { 0, 0 },
    { 0xf768620, 33 },
    { 0x99868621, 33 },
    { 0x3ef68622, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f6862a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc416862d, 95 },
    { 0, 0 },
    { 0x1e76862f, 43 },
    { 0x60468630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a968633, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e468636, 98 },
    { 0x1f468637, 84 },
    { 0, 0 },
    { 0xdb668639, 90 },
    { 0x46f6863a, 17 },
    { 0, 0 },
    { 0x3086863c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b76863f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x31068642, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78368645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e068649, 98 },
    { 0xd1c6864a, 61 },
    { 0xdcb6864b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d26864f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a68657, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4676865c, 43 },
    { 0xc8c6865d, 76 },
    { 0xc846865e, 61 },
    { 0, 0 },
    { 0xee668660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d368668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd368670, 95 },
    { 0x47868671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86868674, 61 },
    { 0, 0 },
    { 0xf6a68676, 19 },
    { 0x1da68676, 99 },
    { 0x50a68676, 67 },
    { 0x44068678, 98 },
    { 0x9d96867a, 76 },
    { 0xba66867a, 93 },
    { 0xe026867c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d6867f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df68682, 33 },
    { 0xe568683, 38 },
    { 0xf0168683, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c068688, 95 },
    { 0x52b68689, 76 },
    { 0x8a56868a, 68 },
    { 0xdf36868b, 52 },
    { 0x6cd6868b, 31 },
    { 0, 0 },
    { 0xa996868e, 67 },
    { 0, 0 },
    { 0x91f68690, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a468693, 61 },
    { 0x98a68694, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd06869c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f686a2, 9 },
    { 0, 0 },
    { 0xd62686a4, 90 },
    { 0, 0 },
    { 0xe37686a6, 95 },
    { 0, 0 },
    { 0xc33686a8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f686ac, 88 },
    { 0x703686ad, 61 },
    { 0x122686ad, 33 },
    { 0x981686af, 61 },
    { 0xcd8686ae, 46 },
    { 0x22686b1, 61 },
    { 0x3be686b2, 40 },
    { 0x584686b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc686b6, 81 },
    { 0x887686b7, 43 },
    { 0, 0 },
    { 0xf5b686b9, 23 },
    { 0, 0 },
    { 0x559686bb, 98 },
    { 0xf5d686bb, 33 },
    { 0, 0 },
    { 0x40e686be, 38 },
    { 0, 0 },
    { 0xc4c686c0, 76 },
    { 0, 0 },
    { 0x4e9686c2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1686c5, 65 },
    { 0xd68686c5, 90 },
    { 0xda2686c7, 33 },
    { 0xd9d686c8, 68 },
    { 0x4ec686c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725686cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba686d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278686d7, 33 },
    { 0x994686d8, 61 },
    { 0xe32686d9, 95 },
    { 0, 0 },
    { 0x529686db, 98 },
    { 0x3c7686dc, 90 },
    { 0xc4a686dd, 23 },
    { 0, 0 },
    { 0x76b686df, 93 },
    { 0x6f8686e0, 33 },
    { 0xa33686e0, 61 },
    { 0xca8686e2, 43 },
    { 0x338686e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d686e9, 33 },
    { 0, 0 },
    { 0xb7f686eb, 17 },
    { 0x2e3686eb, 95 },
    { 0x9bb686ed, 78 },
    { 0, 0 },
    { 0x20c686ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3686f5, 84 },
    { 0, 0 },
    { 0x790686f7, 22 },
    { 0, 0 },
    { 0x96a686f9, 23 },
    { 0xff6686fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f68701, 99 },
    { 0xe0468702, 55 },
    { 0xfbe68703, 90 },
    { 0x1af68702, 90 },
    { 0x81a68704, 100 },
    { 0xab968706, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x668709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566870d, 99 },
    { 0xce06870e, 78 },
    { 0x8236870d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83068715, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb6871b, 33 },
    { 0x8ce6871b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89568720, 61 },
    { 0xc3468721, 55 },
    { 0, 0 },
    { 0xbb868723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a68726, 61 },
    { 0, 0 },
    { 0xccd68728, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79968733, 17 },
    { 0xa4f68733, 90 },
    { 0xe4068735, 23 },
    { 0x97268736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8556873b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c06873e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4568741, 55 },
    { 0x1e168741, 76 },
    { 0, 0 },
    { 0xff068744, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6668749, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa906874d, 61 },
    { 0x5d96874d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e168752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa836875a, 33 },
    { 0xa636875b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f68763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f468766, 23 },
    { 0x28168767, 95 },
    { 0x1cd68767, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb856876c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a668770, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b268773, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26468776, 99 },
    { 0x16e68777, 76 },
    { 0xd6168777, 90 },
    { 0xa9568776, 43 },
    { 0, 0 },
    { 0xa416877b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b68780, 23 },
    { 0xaaa68780, 61 },
    { 0, 0 },
    { 0xb9068783, 33 },
    { 0xb2b68783, 23 },
    { 0xb5f68784, 33 },
    { 0xc1e68786, 43 },
    { 0x82f68786, 33 },
    { 0xcbe68788, 65 },
    { 0xff268788, 55 },
    { 0, 0 },
    { 0x47a6878b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf066878f, 33 },
    { 0xc0868790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62668796, 81 },
    { 0xfd568796, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec16879b, 33 },
    { 0x3fd6879c, 87 },
    { 0x6776879d, 90 },
    { 0x1036879e, 33 },
    { 0, 0 },
    { 0x76c687a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779687a4, 33 },
    { 0xf30687a5, 85 },
    { 0x762687a6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e687a9, 76 },
    { 0x65b687aa, 61 },
    { 0x673687ab, 33 },
    { 0x9ac687ac, 33 },
    { 0xfd6687ac, 55 },
    { 0x732687ae, 61 },
    { 0xa23687af, 95 },
    { 0xb96687b0, 67 },
    { 0xa50687b1, 95 },
    { 0x77a687ad, 43 },
    { 0x1b687b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19687b7, 100 },
    { 0, 0 },
    { 0xb60687b9, 90 },
    { 0x8ff687ba, 76 },
    { 0x312687bb, 95 },
    { 0, 0 },
    { 0xf51687bd, 90 },
    { 0xc61687bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04687c6, 76 },
    { 0, 0 },
    { 0x22e687c8, 61 },
    { 0x6ff687c9, 55 },
    { 0xcae687c9, 84 },
    { 0x7ba687c8, 95 },
    { 0xd5d687cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x985687cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765687d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2687dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82687e1, 23 },
    { 0, 0 },
    { 0x391687e3, 78 },
    { 0xbd3687e4, 33 },
    { 0, 0 },
    { 0xa4a687e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64687ed, 78 },
    { 0, 0 },
    { 0xe6a687ef, 90 },
    { 0x25687f0, 90 },
    { 0xe9687ef, 61 },
    { 0, 0 },
    { 0xa25687f3, 46 },
    { 0xe56687f4, 61 },
    { 0, 0 },
    { 0x1b1687f6, 90 },
    { 0x5aa687f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0468802, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1168805, 34 },
    { 0x27e68806, 98 },
    { 0x7da68807, 90 },
    { 0x64168808, 90 },
    { 0x13168809, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae86880e, 61 },
    { 0, 0 },
    { 0x3aa68810, 90 },
    { 0x48868811, 93 },
    { 0xb1a68810, 61 },
    { 0x1fa68813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42668816, 93 },
    { 0, 0 },
    { 0x38f68818, 95 },
    { 0x50668819, 23 },
    { 0x10468819, 38 },
    { 0, 0 },
    { 0xa1f6881c, 55 },
    { 0x4f6881c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9468821, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c68826, 55 },
    { 0, 0 },
    { 0xe6768828, 80 },
    { 0x99b68829, 95 },
    { 0, 0 },
    { 0x4a6882b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df68832, 33 },
    { 0, 0 },
    { 0xc4168834, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc568838, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b68840, 33 },
    { 0, 0 },
    { 0x9d968842, 76 },
    { 0xc6168843, 99 },
    { 0, 0 },
    { 0xf7c68845, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f68849, 59 },
    { 0x3bc6884a, 23 },
    { 0, 0 },
    { 0x2076884c, 90 },
    { 0xfee6884d, 76 },
    { 0x2376884d, 100 },
    { 0, 0 },
    { 0x2a068850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b68853, 81 },
    { 0, 0 },
    { 0xfcf68855, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe68859, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b6885c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb476885f, 33 },
    { 0x7636885f, 99 },
    { 0x63868860, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6468865, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde86886a, 95 },
    { 0x36886b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d368870, 23 },
    { 0xba268871, 33 },
    { 0, 0 },
    { 0xe2c68873, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1268876, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d68879, 61 },
    { 0x34b6887a, 61 },
    { 0x19a6887b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8068880, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b68883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27168887, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc576888a, 93 },
    { 0x9a76888a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13668894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d26889b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2688a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5688aa, 37 },
    { 0, 0 },
    { 0x278688ac, 23 },
    { 0, 0 },
    { 0x7ef688ae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833688b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde688b7, 90 },
    { 0xe90688b7, 55 },
    { 0, 0 },
    { 0xf54688ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc688bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291688c2, 80 },
    { 0xb41688c3, 76 },
    { 0, 0 },
    { 0x36b688c5, 67 },
    { 0x772688c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265688d0, 90 },
    { 0, 0 },
    { 0xb0f688d2, 98 },
    { 0xada688d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731688d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab688e2, 90 },
    { 0x1b2688e3, 34 },
    { 0, 0 },
    { 0x1d1688e5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec688e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d688eb, 90 },
    { 0x388688eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0688f9, 98 },
    { 0, 0 },
    { 0x941688fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f68906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6136890a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a6890e, 33 },
    { 0x63d6890f, 33 },
    { 0x96b68910, 90 },
    { 0xa8b68910, 67 },
    { 0xf9568910, 46 },
    { 0, 0 },
    { 0xb8168914, 76 },
    { 0x50168915, 67 },
    { 0, 0 },
    { 0x98068917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd426891b, 34 },
    { 0, 0 },
    { 0x79d6891d, 81 },
    { 0x7cd6891e, 100 },
    { 0, 0 },
    { 0x8e968920, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd768923, 72 },
    { 0x4ca68923, 34 },
    { 0xd3468925, 76 },
    { 0, 0 },
    { 0x4e568927, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e36892a, 67 },
    { 0x51c6892b, 33 },
    { 0x31c6892c, 90 },
    { 0, 0 },
    { 0xcc06892e, 95 },
    { 0x9e76892f, 87 },
    { 0xe936892f, 60 },
    { 0x16968931, 91 },
    { 0xafc68931, 61 },
    { 0x14468933, 100 },
    { 0x7ca6892f, 95 },
    { 0, 0 },
    { 0x14368936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25068939, 91 },
    { 0x3ec68939, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6893d, 34 },
    { 0, 0 },
    { 0xef46893f, 90 },
    { 0x31168940, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7368944, 93 },
    { 0xc9068945, 61 },
    { 0x77b68946, 98 },
    { 0, 0 },
    { 0xa0d68948, 90 },
    { 0xbab68948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c46894e, 95 },
    { 0x9696894e, 98 },
    { 0xd4d6894f, 34 },
    { 0xaad6894f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67168955, 76 },
    { 0x70d68956, 55 },
    { 0, 0 },
    { 0x8ce68958, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa066895c, 55 },
    { 0x556895c, 34 },
    { 0, 0 },
    { 0x4a36895f, 33 },
    { 0xca068960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5068967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9736896a, 68 },
    { 0xb796896b, 33 },
    { 0x1816896b, 55 },
    { 0, 0 },
    { 0xd2c6896e, 90 },
    { 0, 0 },
    { 0x55068970, 66 },
    { 0, 0 },
    { 0x6ef68972, 23 },
    { 0, 0 },
    { 0xa5568974, 78 },
    { 0x6a368975, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7568979, 95 },
    { 0, 0 },
    { 0x9376897b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db68981, 95 },
    { 0xeca68982, 69 },
    { 0x64068983, 99 },
    { 0xb5a68983, 90 },
    { 0x21c68985, 38 },
    { 0, 0 },
    { 0x24768987, 67 },
    { 0xcf68987, 69 },
    { 0x21f68989, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7566898c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e66899a, 33 },
    { 0x5546899a, 34 },
    { 0x9bb6899a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147689a0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3689a3, 93 },
    { 0, 0 },
    { 0x763689a5, 25 },
    { 0x9b3689a6, 78 },
    { 0x1d3689a7, 76 },
    { 0, 0 },
    { 0x78a689a9, 98 },
    { 0x2c7689aa, 23 },
    { 0x35d689ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8689af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d689b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8689b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735689be, 23 },
    { 0x9a1689bf, 61 },
    { 0xb2b689c0, 80 },
    { 0x718689c1, 76 },
    { 0x633689c2, 67 },
    { 0x79d689c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f689c6, 98 },
    { 0x901689c7, 55 },
    { 0xa46689c6, 33 },
    { 0x2bc689c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e689cc, 76 },
    { 0xc27689cd, 90 },
    { 0x255689cc, 65 },
    { 0, 0 },
    { 0x3e5689d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2689d7, 47 },
    { 0xf21689d8, 95 },
    { 0xfb9689d9, 90 },
    { 0x359689da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa689de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5689e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0689e6, 90 },
    { 0x646689e7, 81 },
    { 0xf40689e8, 95 },
    { 0, 0 },
    { 0x15f689ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4689f1, 98 },
    { 0x6f3689f1, 90 },
    { 0x83d689f1, 33 },
    { 0x140689f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90689f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3689ff, 33 },
    { 0, 0 },
    { 0x55768a01, 93 },
    { 0x4f368a02, 95 },
    { 0x23b68a01, 34 },
    { 0x61d68a04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50568a07, 23 },
    { 0, 0 },
    { 0x90068a09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e68a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4268a11, 91 },
    { 0, 0 },
    { 0x5aa68a13, 34 },
    { 0, 0 },
    { 0xaa968a15, 90 },
    { 0x44a68a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb68a19, 90 },
    { 0x33b68a1a, 23 },
    { 0xe7968a1b, 90 },
    { 0, 0 },
    { 0xbec68a1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22968a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b68a24, 98 },
    { 0xd1168a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9968a2b, 23 },
    { 0xa6768a2b, 34 },
    { 0xc2268a2c, 90 },
    { 0x27968a2e, 23 },
    { 0xb4c68a2f, 90 },
    { 0x9b168a30, 33 },
    { 0x5bc68a2d, 95 },
    { 0x29668a32, 98 },
    { 0xc2e68a2c, 43 },
    { 0x97f68a34, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5668a37, 56 },
    { 0x57e68a38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b68a3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea68a3e, 55 },
    { 0xc9b68a3e, 98 },
    { 0xf0d68a40, 78 },
    { 0xa5a68a41, 34 },
    { 0x3fa68a3e, 61 },
    { 0xc5168a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68168a4b, 98 },
    { 0xd2b68a4c, 99 },
    { 0x69c68a4d, 91 },
    { 0x3fe68a4d, 40 },
    { 0xda268a4f, 33 },
    { 0x93768a4f, 85 },
    { 0xa4468a51, 33 },
    { 0xbe368a52, 90 },
    { 0xca068a51, 23 },
    { 0xd3468a4c, 66 },
    { 0x18668a55, 90 },
    { 0x3d868a55, 52 },
    { 0x30168a53, 33 },
    { 0xc9268a58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c368a5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd568a61, 58 },
    { 0xcf968a61, 84 },
    { 0, 0 },
    { 0x6fd68a64, 55 },
    { 0, 0 },
    { 0x5a68a66, 33 },
    { 0xc5a68a67, 90 },
    { 0x6d068a68, 23 },
    { 0x13468a69, 95 },
    { 0x71568a68, 95 },
    { 0x75e68a6b, 33 },
    { 0xe6668a68, 43 },
    { 0x76868a68, 78 },
    { 0x90468a6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a668a71, 23 },
    { 0x2fc68a72, 33 },
    { 0x12868a72, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e268a77, 53 },
    { 0x3168a78, 33 },
    { 0x93f68a78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c68a7d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7968a81, 76 },
    { 0xdc368a82, 46 },
    { 0xf3668a83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e68a97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a568a9d, 66 },
    { 0, 0 },
    { 0xc7268a9f, 33 },
    { 0xdc768aa0, 46 },
    { 0, 0 },
    { 0xff568aa2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a68aa8, 98 },
    { 0, 0 },
    { 0xeb68aaa, 98 },
    { 0x68f68aaa, 68 },
    { 0, 0 },
    { 0x86068aad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c668ab1, 95 },
    { 0xbcd68ab2, 98 },
    { 0xc5168ab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb68ab8, 98 },
    { 0xe3468ab8, 78 },
    { 0x4d868ab9, 84 },
    { 0, 0 },
    { 0x80568abc, 66 },
    { 0x24168abd, 23 },
    { 0xf2968abe, 90 },
    { 0x9a468abd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17468ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9768ace, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d68ad3, 34 },
    { 0x6dc68ad4, 61 },
    { 0x78568ad5, 33 },
    { 0, 0 },
    { 0x8368ad7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4368ada, 76 },
    { 0x88e68adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63268adf, 55 },
    { 0xd6f68ae0, 61 },
    { 0x18d68ae1, 76 },
    { 0xa3668ae0, 83 },
    { 0x58568ae1, 100 },
    { 0x95368ae4, 68 },
    { 0x13768ae0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42868ae9, 90 },
    { 0x6ef68aea, 90 },
    { 0xfaa68aea, 64 },
    { 0x92d68aec, 61 },
    { 0xc5968aeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xad968af0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0068af5, 67 },
    { 0x16568af6, 90 },
    { 0x24168af7, 33 },
    { 0xf2868af7, 90 },
    { 0x91168af7, 91 },
    { 0x9af68af7, 46 },
    { 0x94e68afb, 95 },
    { 0xda468af8, 23 },
    { 0, 0 },
    { 0x6a368afe, 33 },
    { 0, 0 },
    { 0x66d68b00, 55 },
    { 0x3d68b00, 33 },
    { 0, 0 },
    { 0x7e368b03, 66 },
    { 0, 0 },
    { 0x4e468b05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c68b0b, 61 },
    { 0x27d68b0c, 95 },
    { 0x62368b0d, 33 },
    { 0x91f68b0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7068b11, 23 },
    { 0x6d168b12, 55 },
    { 0x2af68b11, 100 },
    { 0x99368b14, 90 },
    { 0x73468b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab268b19, 98 },
    { 0xcdc68b19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e68b1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda68b21, 33 },
    { 0xde868b22, 98 },
    { 0, 0 },
    { 0xa1c68b24, 95 },
    { 0x6a568b25, 52 },
    { 0x5b368b26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a68b2d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60468b32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9468b35, 76 },
    { 0xa7568b35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1168b43, 90 },
    { 0, 0 },
    { 0x50868b45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb68b48, 61 },
    { 0, 0 },
    { 0x51068b4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d68b50, 61 },
    { 0xf0268b51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c68b54, 61 },
    { 0x1df68b54, 33 },
    { 0xd6968b56, 56 },
    { 0xed968b57, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e68b5e, 90 },
    { 0xff168b5f, 98 },
    { 0x2e668b60, 91 },
    { 0xa2468b61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b68b64, 23 },
    { 0, 0 },
    { 0xa8568b66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf668b6b, 76 },
    { 0, 0 },
    { 0x6e568b6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd668b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a468b86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e68b89, 85 },
    { 0x36b68b8a, 85 },
    { 0, 0 },
    { 0x6bc68b8c, 34 },
    { 0x4b768b8c, 61 },
    { 0xb5168b8c, 98 },
    { 0, 0 },
    { 0x12168b90, 23 },
    { 0xd5768b91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f168b95, 88 },
    { 0xeef68b95, 88 },
    { 0x98668b96, 23 },
    { 0x48668b98, 90 },
    { 0x2b568b99, 61 },
    { 0xd7468b9a, 95 },
    { 0xfe868b97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12868b9e, 61 },
    { 0x7ec68b9f, 91 },
    { 0xde668b9e, 33 },
    { 0x70768ba0, 61 },
    { 0x48668ba2, 34 },
    { 0xc2d68ba0, 33 },
    { 0x2e768b9e, 33 },
    { 0x51f68ba5, 33 },
    { 0x53d68ba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62968bab, 95 },
    { 0x16768bab, 98 },
    { 0, 0 },
    { 0x2bc68bae, 23 },
    { 0xc7068baf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59968bb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb668bb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb068bba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4568bbf, 34 },
    { 0, 0 },
    { 0x69a68bc1, 90 },
    { 0, 0 },
    { 0x57e68bc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e68bcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a68bd3, 81 },
    { 0, 0 },
    { 0x6868bd5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb368bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e68bde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e68be6, 98 },
    { 0x1db68be7, 95 },
    { 0, 0 },
    { 0x27468be9, 23 },
    { 0, 0 },
    { 0x96f68beb, 33 },
    { 0x14568bec, 33 },
    { 0x71e68beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e68bf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e68bf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b68c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c68c05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d68c08, 65 },
    { 0x58768c09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b968c15, 95 },
    { 0x88868c16, 61 },
    { 0xcc068c17, 61 },
    { 0x36168c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd568c1b, 55 },
    { 0x7a68c1c, 87 },
    { 0, 0 },
    { 0x33668c1e, 61 },
    { 0, 0 },
    { 0x10668c20, 66 },
    { 0xe9d68c20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0768c2a, 95 },
    { 0x35968c2b, 52 },
    { 0xab68c2b, 23 },
    { 0, 0 },
    { 0x5d668c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2668c31, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab68c36, 46 },
    { 0xc7168c37, 55 },
    { 0xd0568c36, 90 },
    { 0, 0 },
    { 0x25a68c3a, 98 },
    { 0x57e68c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d168c3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25368c44, 88 },
    { 0, 0 },
    { 0xc8b68c46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc668c49, 77 },
    { 0, 0 },
    { 0xfff68c4b, 81 },
    { 0, 0 },
    { 0xe8a68c4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6668c51, 33 },
    { 0x64d68c52, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e268c58, 61 },
    { 0, 0 },
    { 0x78968c5a, 33 },
    { 0xca168c5a, 23 },
    { 0xd2c68c5c, 33 },
    { 0x76868c5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf668c60, 61 },
    { 0x2ac68c61, 61 },
    { 0x7268c60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e68c67, 23 },
    { 0xc468c68, 61 },
    { 0xea68c68, 55 },
    { 0x1f168c68, 95 },
    { 0xfee68c6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8168c6e, 33 },
    { 0x36868c6f, 98 },
    { 0x2e68c70, 77 },
    { 0, 0 },
    { 0xafe68c72, 23 },
    { 0x1de68c73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd968c76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0468c79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c768c7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c68c82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3468c85, 76 },
    { 0x14468c85, 33 },
    { 0x28268c87, 75 },
    { 0x8cc68c88, 95 },
    { 0xa7068c89, 84 },
    { 0x42e68c8a, 75 },
    { 0x57868c8b, 23 },
    { 0x94a68c8c, 100 },
    { 0, 0 },
    { 0x5c968c8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e568c92, 55 },
    { 0xc8968c93, 61 },
    { 0x31368c92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68068c97, 98 },
    { 0, 0 },
    { 0x76268c99, 55 },
    { 0xe8d68c9a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96068c9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f68ca4, 100 },
    { 0xc8868ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39568ca8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f68cad, 98 },
    { 0xe6468cad, 61 },
    { 0x60568caf, 84 },
    { 0x6a468cb0, 61 },
    { 0x62e68cb1, 65 },
    { 0xa5368cb2, 55 },
    { 0x9bf68cb3, 78 },
    { 0xa6868cb4, 98 },
    { 0, 0 },
    { 0x4aa68cb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa68cb9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc868cbd, 76 },
    { 0xd2b68cbd, 90 },
    { 0xf1668cbd, 61 },
    { 0, 0 },
    { 0x47068cc1, 93 },
    { 0, 0 },
    { 0xc5368cc3, 55 },
    { 0, 0 },
    { 0x22b68cc5, 100 },
    { 0, 0 },
    { 0x20c68cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27968ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da68cd1, 95 },
    { 0, 0 },
    { 0x31668cd3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16268cd9, 90 },
    { 0x53768cda, 90 },
    { 0, 0 },
    { 0x96f68cdc, 98 },
    { 0x22b68cdd, 90 },
    { 0, 0 },
    { 0x94a68cdf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f68ce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88468ce5, 48 },
    { 0, 0 },
    { 0x62668ce7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b368ced, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf68cf1, 93 },
    { 0xfd668cf2, 93 },
    { 0x1d968cf3, 23 },
    { 0x96968cf4, 66 },
    { 0x90a68cf5, 100 },
    { 0x19968cf6, 33 },
    { 0, 0 },
    { 0x46868cf8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c668d00, 23 },
    { 0x3fe68d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6668d09, 46 },
    { 0x9868d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b68d11, 90 },
    { 0, 0 },
    { 0x5a068d13, 61 },
    { 0xfaf68d14, 36 },
    { 0, 0 },
    { 0xe6068d16, 61 },
    { 0xd4268d17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b68d1d, 23 },
    { 0x10868d1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4068d23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f168d29, 23 },
    { 0, 0 },
    { 0xae868d2b, 90 },
    { 0x72768d2c, 90 },
    { 0xca168d2c, 55 },
    { 0x4a868d2e, 69 },
    { 0, 0 },
    { 0xff968d30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc168d37, 34 },
    { 0, 0 },
    { 0x44568d39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f68d3d, 76 },
    { 0x4a968d3e, 100 },
    { 0xf3668d3f, 95 },
    { 0x8668d3f, 98 },
    { 0x24768d3e, 33 },
    { 0x5d668d3d, 87 },
    { 0x92568d43, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d468d48, 95 },
    { 0xf8668d48, 90 },
    { 0x8ea68d4a, 95 },
    { 0xedb68d4a, 95 },
    { 0xbde68d4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d68d4f, 64 },
    { 0, 0 },
    { 0xf6668d51, 61 },
    { 0x2cd68d52, 33 },
    { 0x6a068d53, 36 },
    { 0x1a468d54, 40 },
    { 0xca68d52, 88 },
    { 0xe9368d56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce68d5d, 43 },
    { 0x72768d5e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6468d63, 61 },
    { 0x8068d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc68d67, 61 },
    { 0xf4b68d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268d6b, 23 },
    { 0x48768d6c, 66 },
    { 0xfae68d6c, 17 },
    { 0xbe668d6e, 44 },
    { 0x93568d6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c68d74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc68d83, 99 },
    { 0xaa568d84, 61 },
    { 0x79968d84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56968d96, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91268d9d, 23 },
    { 0xe4868d9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde168da3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f68da7, 61 },
    { 0xd1d68da7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4968dab, 23 },
    { 0x9e668dac, 46 },
    { 0xa9468dad, 34 },
    { 0, 0 },
    { 0x42368daf, 90 },
    { 0x94668db0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d468db3, 76 },
    { 0xf3668db4, 55 },
    { 0xb1568db5, 90 },
    { 0xb7a68db3, 95 },
    { 0xfa768db6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc668dbe, 95 },
    { 0x74968dbf, 99 },
    { 0x5968dc0, 90 },
    { 0xf5768dbe, 76 },
    { 0, 0 },
    { 0x59368dc3, 98 },
    { 0, 0 },
    { 0x9e068dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b68dcb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8868dd2, 100 },
    { 0xe2b68dd3, 88 },
    { 0x3db68dd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b68ddb, 61 },
    { 0x41668ddc, 53 },
    { 0, 0 },
    { 0x4ec68dde, 76 },
    { 0xed468dde, 90 },
    { 0, 0 },
    { 0x13768de1, 76 },
    { 0x9c768de2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d68de5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d68de8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x31768deb, 95 },
    { 0x77f68deb, 84 },
    { 0x7ea68ded, 94 },
    { 0x75c68ded, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c68df4, 78 },
    { 0xe4368df4, 76 },
    { 0xe0f68df5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc868df9, 43 },
    { 0, 0 },
    { 0x6ea68dfb, 43 },
    { 0x4fd68dfb, 61 },
    { 0, 0 },
    { 0xb0168dfe, 78 },
    { 0x2d768dff, 95 },
    { 0xf1e68e00, 76 },
    { 0x69268e00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0268e09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6868e0c, 93 },
    { 0, 0 },
    { 0x8a968e0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef568e1c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab868e22, 36 },
    { 0x26668e23, 90 },
    { 0x51c68e24, 93 },
    { 0x1a468e25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4068e28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54068e2c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e368e2f, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8068e32, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46168e3a, 61 },
    { 0x5d968e3b, 95 },
    { 0x4d068e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c468e4d, 93 },
    { 0, 0 },
    { 0xab268e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e968e54, 98 },
    { 0xbc068e55, 23 },
    { 0x9ad68e56, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6968e5a, 66 },
    { 0x4f768e5a, 90 },
    { 0xc6d68e5c, 76 },
    { 0xc5f68e5c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19268e61, 68 },
    { 0x4468e62, 94 },
    { 0, 0 },
    { 0xa5a68e64, 33 },
    { 0, 0 },
    { 0x85368e66, 23 },
    { 0x27b68e67, 81 },
    { 0, 0 },
    { 0x65f68e69, 95 },
    { 0xa1c68e6a, 84 },
    { 0x62068e6a, 99 },
    { 0xa3068e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a968e75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a68e7a, 84 },
    { 0x2f668e7a, 88 },
    { 0xd0f68e7a, 95 },
    { 0xe7668e7c, 98 },
    { 0xab68e7a, 34 },
    { 0x13368e7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83868e86, 87 },
    { 0xc5768e87, 76 },
    { 0xa1768e88, 87 },
    { 0x60468e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a68e8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f68e99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f968e9c, 77 },
    { 0x17368e9c, 78 },
    { 0x9fd68e9e, 23 },
    { 0xb5d68e9f, 24 },
    { 0xfd768ea0, 33 },
    { 0x58968ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2368ea5, 23 },
    { 0x4cb68ea6, 90 },
    { 0, 0 },
    { 0x8f268ea8, 98 },
    { 0x4e068ea9, 91 },
    { 0x66b68eaa, 61 },
    { 0x31668eab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2568eb0, 90 },
    { 0x2f868eb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e68eb8, 77 },
    { 0x4ac68eb8, 61 },
    { 0x6c768eb8, 61 },
    { 0, 0 },
    { 0x61568ebc, 34 },
    { 0xc7f68ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7068ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70168ec6, 46 },
    { 0, 0 },
    { 0xde268ec8, 61 },
    { 0x71168ec8, 95 },
    { 0x41468eca, 52 },
    { 0, 0 },
    { 0xf9368ecc, 66 },
    { 0, 0 },
    { 0xe6068ece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a68ed6, 34 },
    { 0xc8868ed7, 33 },
    { 0x1a668ed7, 33 },
    { 0x32668ed8, 98 },
    { 0xc8468eda, 40 },
    { 0x4e168ed8, 17 },
    { 0x7fd68edc, 33 },
    { 0x70468edd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53768ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b68ef2, 93 },
    { 0x3b868ef2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb68ef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b68f03, 85 },
    { 0xa5668f03, 29 },
    { 0xc5668f05, 87 },
    { 0x95168f05, 100 },
    { 0x87868f07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61268f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8568f17, 90 },
    { 0x1b68f17, 80 },
    { 0x56468f19, 66 },
    { 0xa1268f1a, 76 },
    { 0x5e568f1b, 33 },
    { 0xa9a68f1c, 34 },
    { 0x22b68f1d, 98 },
    { 0xe0b68f1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7468f22, 77 },
    { 0x56768f22, 61 },
    { 0x19f68f24, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c68f27, 90 },
    { 0, 0 },
    { 0xc3d68f29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93668f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c68f31, 43 },
    { 0, 0 },
    { 0xee068f33, 100 },
    { 0xd068f33, 33 },
    { 0xa3b68f34, 34 },
    { 0, 0 },
    { 0x90e68f37, 95 },
    { 0x6e568f38, 33 },
    { 0x49968f38, 34 },
    { 0x8f968f37, 40 },
    { 0, 0 },
    { 0xa8468f3c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f768f42, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed568f48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5768f4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c68f53, 98 },
    { 0x67a68f54, 23 },
    { 0, 0 },
    { 0x6dd68f56, 91 },
    { 0, 0 },
    { 0xe3668f58, 55 },
    { 0x7c368f59, 99 },
    { 0xc2e68f5a, 61 },
    { 0xcf468f59, 61 },
    { 0x8db68f5c, 98 },
    { 0x2a968f5d, 66 },
    { 0xf4068f5c, 61 },
    { 0x9c868f5f, 98 },
    { 0xda368f60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39368f64, 66 },
    { 0xb9468f64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f68f68, 100 },
    { 0xb0668f68, 34 },
    { 0x63b68f68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa268f70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e068f7a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d68f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d68f82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec68f86, 98 },
    { 0, 0 },
    { 0x19268f88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71668f8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a568f90, 90 },
    { 0xd4068f91, 90 },
    { 0, 0 },
    { 0xb768f93, 93 },
    { 0, 0 },
    { 0xc7568f95, 23 },
    { 0xb2f68f95, 95 },
    { 0x53968f96, 66 },
    { 0, 0 },
    { 0x4db68f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b68f9c, 65 },
    { 0x87568f9d, 87 },
    { 0x39f68f9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d68fa3, 33 },
    { 0xbdd68fa4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0168fa7, 61 },
    { 0, 0 },
    { 0x3e368fa9, 90 },
    { 0x94b68faa, 67 },
    { 0x89e68fa9, 61 },
    { 0xada68fac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf68faf, 81 },
    { 0xceb68faf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3968fbc, 23 },
    { 0x62568fbc, 17 },
    { 0, 0 },
    { 0xc0768fbf, 61 },
    { 0xf068fbf, 46 },
    { 0x2cd68fc1, 76 },
    { 0x2b68fc1, 34 },
    { 0xb1f68fc1, 33 },
    { 0x86168fc4, 99 },
    { 0x9ce68fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed68fc8, 34 },
    { 0xd3468fc9, 90 },
    { 0, 0 },
    { 0xfc68fcb, 89 },
    { 0x33068fcc, 65 },
    { 0x26c68fcd, 33 },
    { 0, 0 },
    { 0x95568fcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b68fd5, 43 },
    { 0, 0 },
    { 0xc6d68fd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0868fe3, 23 },
    { 0x7c268fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c468fe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc68fea, 61 },
    { 0, 0 },
    { 0x2e568fec, 84 },
    { 0x53e68fed, 81 },
    { 0xb368fee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81168ff3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf368ff7, 98 },
    { 0xeeb68ff8, 76 },
    { 0, 0 },
    { 0x94a68ffa, 23 },
    { 0x79668ffb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c68fff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4869003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b69007, 85 },
    { 0, 0 },
    { 0x3f469009, 46 },
    { 0xb666900a, 90 },
    { 0x20f6900a, 76 },
    { 0x32f6900c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed269010, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd269014, 95 },
    { 0, 0 },
    { 0x3969016, 98 },
    { 0, 0 },
    { 0xdc869018, 34 },
    { 0x2c469019, 95 },
    { 0x3a96901a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d36901f, 41 },
    { 0xde269020, 93 },
    { 0x9036901f, 66 },
    { 0x99369020, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc69026, 77 },
    { 0x40b69027, 80 },
    { 0, 0 },
    { 0xef769029, 23 },
    { 0, 0 },
    { 0xf5a6902b, 85 },
    { 0, 0 },
    { 0x29f6902d, 23 },
    { 0x5d66902e, 61 },
    { 0, 0 },
    { 0x1b869030, 95 },
    { 0x2d469031, 76 },
    { 0x14b69030, 33 },
    { 0x52069033, 61 },
    { 0xc1669031, 33 },
    { 0xb0d69030, 61 },
    { 0x6c169036, 100 },
    { 0x2ec69035, 61 },
    { 0x93c69038, 43 },
    { 0xd7369039, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f69041, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b469044, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa669048, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85769050, 23 },
    { 0, 0 },
    { 0xa5269052, 75 },
    { 0x69b69052, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8669056, 90 },
    { 0x44369057, 23 },
    { 0x15269056, 34 },
    { 0xdf169056, 90 },
    { 0xe2969059, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4769061, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44669068, 87 },
    { 0xb5069069, 93 },
    { 0xe286906a, 76 },
    { 0xfb56906b, 76 },
    { 0x49069069, 95 },
    { 0x7c969068, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca69073, 90 },
    { 0x51269074, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b69078, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48469083, 90 },
    { 0xf7969083, 90 },
    { 0x36a69085, 65 },
    { 0, 0 },
    { 0xf2c69087, 78 },
    { 0x30269088, 61 },
    { 0xf469089, 23 },
    { 0x11e6908a, 91 },
    { 0x9316908b, 101 },
    { 0xc536908c, 90 },
    { 0xa9c6908d, 76 },
    { 0xa46908a, 90 },
    { 0xd0c69088, 98 },
    { 0x1f669090, 98 },
    { 0x81169091, 95 },
    { 0x59769089, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d369097, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266909a, 58 },
    { 0, 0 },
    { 0x1b66909c, 93 },
    { 0, 0 },
    { 0xc516909e, 98 },
    { 0x3f96909e, 90 },
    { 0x87e6909f, 55 },
    { 0x339690a1, 90 },
    { 0x2b1690a2, 23 },
    { 0xc47690a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0690a7, 34 },
    { 0xd39690a8, 61 },
    { 0xeb5690a9, 33 },
    { 0xab6690aa, 90 },
    { 0x579690ab, 34 },
    { 0x3fa690ac, 93 },
    { 0x414690a8, 40 },
    { 0xeca690ae, 65 },
    { 0x97a690ae, 68 },
    { 0x6a9690ae, 34 },
    { 0x3ac690b1, 43 },
    { 0, 0 },
    { 0x926690b3, 76 },
    { 0x423690b3, 93 },
    { 0xd9b690b4, 40 },
    { 0x192690b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x265690b9, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0690bf, 83 },
    { 0, 0 },
    { 0xd3b690c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1690c6, 61 },
    { 0, 0 },
    { 0x6c6690c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3690cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e690d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a690d9, 43 },
    { 0xec0690d9, 90 },
    { 0x566690da, 87 },
    { 0, 0 },
    { 0xb7c690dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0690e0, 90 },
    { 0x465690e1, 76 },
    { 0x737690e2, 33 },
    { 0xfd0690e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a690e8, 90 },
    { 0xd31690e9, 55 },
    { 0xf54690e9, 98 },
    { 0x755690eb, 98 },
    { 0x82c690ec, 33 },
    { 0x782690ea, 19 },
    { 0, 0 },
    { 0x21e690ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76690f5, 90 },
    { 0, 0 },
    { 0x10b690f7, 81 },
    { 0x448690f7, 46 },
    { 0x8c0690f7, 98 },
    { 0xc3b690fa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce690ff, 95 },
    { 0xfab690ff, 98 },
    { 0, 0 },
    { 0xff969102, 95 },
    { 0x4af69102, 24 },
    { 0, 0 },
    { 0xc6969105, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1369108, 95 },
    { 0, 0 },
    { 0xb056910a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f169110, 76 },
    { 0, 0 },
    { 0xfd669112, 90 },
    { 0, 0 },
    { 0xe1069114, 61 },
    { 0, 0 },
    { 0x62269116, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce69119, 34 },
    { 0x5786911a, 88 },
    { 0, 0 },
    { 0x8cf6911c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd036911f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2369126, 55 },
    { 0, 0 },
    { 0xa7169128, 95 },
    { 0xa169129, 99 },
    { 0x6bc6912a, 90 },
    { 0, 0 },
    { 0xa786912c, 90 },
    { 0x4ac6912d, 93 },
    { 0x3b56912e, 55 },
    { 0x4856912f, 33 },
    { 0x5a69130, 55 },
    { 0x2d469130, 95 },
    { 0xfd16912d, 85 },
    { 0x2169131, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94269137, 33 },
    { 0, 0 },
    { 0x78d69139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd869142, 61 },
    { 0x4a69143, 43 },
    { 0x92469144, 55 },
    { 0x70e69142, 33 },
    { 0x85269146, 90 },
    { 0xd0469146, 55 },
    { 0, 0 },
    { 0x3ba69149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb16914d, 46 },
    { 0, 0 },
    { 0x1776914f, 23 },
    { 0xc369150, 90 },
    { 0, 0 },
    { 0xc3f69152, 66 },
    { 0x58369153, 23 },
    { 0, 0 },
    { 0x2f869155, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b69159, 84 },
    { 0x77a6915a, 55 },
    { 0, 0 },
    { 0x99a6915c, 23 },
    { 0x6d46915c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x64469160, 52 },
    { 0, 0 },
    { 0x70369162, 76 },
    { 0x39d69163, 76 },
    { 0x22869164, 91 },
    { 0xd1a69164, 66 },
    { 0x9e869165, 23 },
    { 0x8e069165, 34 },
    { 0x43969168, 43 },
    { 0x5f769169, 61 },
    { 0xc0669169, 98 },
    { 0, 0 },
    { 0x92b6916c, 33 },
    { 0xff6916c, 100 },
    { 0x19a6916e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1569179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f6917c, 33 },
    { 0, 0 },
    { 0x1a96917e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc769185, 43 },
    { 0x5d869185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7176918f, 98 },
    { 0x8656918f, 81 },
    { 0, 0 },
    { 0xa2169192, 23 },
    { 0x73c69192, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d69197, 95 },
    { 0xfa069198, 61 },
    { 0xd7a69198, 23 },
    { 0, 0 },
    { 0x1b06919b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a691a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62691a7, 95 },
    { 0x886691a7, 95 },
    { 0x3f4691a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2691ac, 81 },
    { 0, 0 },
    { 0xf3f691ae, 33 },
    { 0, 0 },
    { 0x5e6691b0, 61 },
    { 0x909691b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x413691b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab691bd, 90 },
    { 0xcc4691be, 65 },
    { 0xf5b691bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44691c4, 65 },
    { 0, 0 },
    { 0x7a8691c6, 23 },
    { 0xc46691c6, 46 },
    { 0, 0 },
    { 0x6b691c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3691cc, 61 },
    { 0x8fe691cc, 61 },
    { 0x4f1691ce, 33 },
    { 0xad8691ce, 61 },
    { 0xe28691d0, 52 },
    { 0x32e691cf, 34 },
    { 0xd1691cd, 95 },
    { 0x60c691d3, 55 },
    { 0x221691d3, 84 },
    { 0x8fb691d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3691d8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x507691db, 33 },
    { 0x303691dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa691e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c691eb, 34 },
    { 0x62b691eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397691f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e691fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375691ff, 91 },
    { 0x27c69200, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d669206, 67 },
    { 0, 0 },
    { 0x1c869208, 91 },
    { 0xbeb69209, 76 },
    { 0x84a6920a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xba66920d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9169210, 33 },
    { 0, 0 },
    { 0x7c169212, 97 },
    { 0x63e69212, 77 },
    { 0x67b69213, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34969218, 96 },
    { 0xc8969219, 99 },
    { 0x7e66921a, 55 },
    { 0x6bb6921b, 33 },
    { 0, 0 },
    { 0x9c36921d, 61 },
    { 0x40f6921e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc69224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c269228, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2569231, 34 },
    { 0xf1169232, 98 },
    { 0x47369232, 61 },
    { 0, 0 },
    { 0x87569235, 76 },
    { 0x3a769235, 33 },
    { 0, 0 },
    { 0x3c669238, 90 },
    { 0, 0 },
    { 0xe6923a, 33 },
    { 0x3a66923a, 84 },
    { 0xb2c6923c, 84 },
    { 0, 0 },
    { 0x8306923e, 90 },
    { 0x1396923f, 69 },
    { 0x3756923f, 93 },
    { 0x12b6923f, 90 },
    { 0xb1269242, 34 },
    { 0x9a69242, 33 },
    { 0, 0 },
    { 0xcaa69245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78469251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e069257, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a36925b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976925f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8769262, 33 },
    { 0, 0 },
    { 0x2569264, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569268, 91 },
    { 0x65769269, 36 },
    { 0xe9769269, 95 },
    { 0x8486926b, 65 },
    { 0xf426926c, 55 },
    { 0x29b6926d, 95 },
    { 0x5fb6926e, 33 },
    { 0x10c6926b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede69278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa6927e, 61 },
    { 0, 0 },
    { 0x10c69280, 99 },
    { 0, 0 },
    { 0x23e69282, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede69286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef86928e, 32 },
    { 0xfe16928e, 33 },
    { 0x5d06928f, 95 },
    { 0xfe969291, 90 },
    { 0xbde69292, 67 },
    { 0, 0 },
    { 0xc9e69294, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe169297, 61 },
    { 0x6169298, 90 },
    { 0, 0 },
    { 0xa266929a, 90 },
    { 0, 0 },
    { 0xa9e6929c, 17 },
    { 0xd706929d, 23 },
    { 0, 0 },
    { 0x3cb6929f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951692a6, 51 },
    { 0, 0 },
    { 0x5f3692a8, 90 },
    { 0x5d0692a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480692ae, 34 },
    { 0xf0f692af, 34 },
    { 0xd1692af, 90 },
    { 0x718692ae, 46 },
    { 0, 0 },
    { 0x9c2692b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xede692b6, 61 },
    { 0xff2692b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71692bd, 95 },
    { 0xe7b692be, 61 },
    { 0xe58692bf, 23 },
    { 0x523692c0, 40 },
    { 0xd4c692c0, 33 },
    { 0x9a9692c2, 56 },
    { 0, 0 },
    { 0xb6c692c4, 61 },
    { 0, 0 },
    { 0xc6692c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776692ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae692cf, 52 },
    { 0x7c692d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f692d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd692d9, 90 },
    { 0x915692da, 66 },
    { 0xfc1692db, 23 },
    { 0x83d692da, 34 },
    { 0, 0 },
    { 0xfda692de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34692e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d692e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x587692ec, 23 },
    { 0x44c692ed, 90 },
    { 0x3fe692ee, 55 },
    { 0x7bf692ee, 90 },
    { 0xd7c692ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18692f4, 77 },
    { 0x4c6692f4, 76 },
    { 0x961692f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03692f9, 61 },
    { 0xc8692f9, 61 },
    { 0, 0 },
    { 0xe2f692fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a69301, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x69169304, 91 },
    { 0xeec69305, 61 },
    { 0x6ae69304, 80 },
    { 0xab169307, 100 },
    { 0x19869308, 43 },
    { 0xbde69307, 84 },
    { 0xd3569309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8626930e, 22 },
    { 0x6ba6930f, 23 },
    { 0, 0 },
    { 0x96969311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93269315, 61 },
    { 0x4eb69316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a6931d, 55 },
    { 0x2326931e, 90 },
    { 0x8756931f, 96 },
    { 0x77a69320, 34 },
    { 0xf946931f, 43 },
    { 0xed569320, 23 },
    { 0, 0 },
    { 0x71669324, 76 },
    { 0x65969324, 90 },
    { 0xee869326, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf926932a, 61 },
    { 0x4326932b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9966932f, 95 },
    { 0x7c469330, 94 },
    { 0, 0 },
    { 0xdea69332, 76 },
    { 0, 0 },
    { 0x8a69334, 98 },
    { 0x46869335, 65 },
    { 0xf2469336, 46 },
    { 0x12669337, 90 },
    { 0xbba69338, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e16933b, 90 },
    { 0xe66933c, 90 },
    { 0x2eb6933c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e369340, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88969348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe416934e, 65 },
    { 0xd356934f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb69354, 43 },
    { 0x36c69355, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26935b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f6935e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd569363, 23 },
    { 0x3869364, 93 },
    { 0, 0 },
    { 0x7e069366, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6936b, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6869372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1169375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33969378, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a6937c, 23 },
    { 0xde46937d, 76 },
    { 0x8266937c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75669387, 23 },
    { 0xbc469387, 23 },
    { 0x94b69389, 53 },
    { 0xdb56938a, 33 },
    { 0x66069389, 99 },
    { 0, 0 },
    { 0xf126938d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd069393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b69397, 23 },
    { 0x16369398, 61 },
    { 0xca269399, 95 },
    { 0xde669399, 23 },
    { 0xa6169399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f6939e, 55 },
    { 0, 0 },
    { 0x932693a0, 43 },
    { 0, 0 },
    { 0x822693a2, 76 },
    { 0x978693a2, 46 },
    { 0x20d693a4, 52 },
    { 0x3e9693a2, 98 },
    { 0x84d693a6, 85 },
    { 0x682693a6, 33 },
    { 0, 0 },
    { 0xd9f693a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b693ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c693b5, 61 },
    { 0xe47693b5, 98 },
    { 0x7b7693b6, 46 },
    { 0xe03693b7, 87 },
    { 0x2e8693b7, 61 },
    { 0x289693ba, 61 },
    { 0xb7e693bb, 84 },
    { 0xcf6693bc, 87 },
    { 0x2a4693bd, 90 },
    { 0x6d8693bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf693c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570693c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c693cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x147693cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1693d3, 33 },
    { 0, 0 },
    { 0x232693d5, 87 },
    { 0xded693d6, 33 },
    { 0x79c693d7, 33 },
    { 0xc49693d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983693df, 90 },
    { 0x998693e0, 33 },
    { 0xb32693e1, 23 },
    { 0x803693e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6693e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb693ee, 90 },
    { 0xae8693ee, 93 },
    { 0, 0 },
    { 0xcd9693f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129693f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41693fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af693fd, 33 },
    { 0x9d0693fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc69402, 61 },
    { 0x89269403, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1169408, 70 },
    { 0, 0 },
    { 0xd9a6940a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4016940d, 34 },
    { 0x69f6940e, 87 },
    { 0x48a6940f, 90 },
    { 0xafc69410, 95 },
    { 0, 0 },
    { 0x37969412, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d69418, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9106941d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75769421, 78 },
    { 0x65269421, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f69425, 90 },
    { 0xc0669426, 88 },
    { 0x2dc69425, 66 },
    { 0xf1169425, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f36942f, 23 },
    { 0xba669430, 68 },
    { 0x94f69431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c69435, 93 },
    { 0x4169435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b6943d, 23 },
    { 0x73d6943e, 95 },
    { 0xa556943e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8169448, 46 },
    { 0, 0 },
    { 0xe746944a, 61 },
    { 0x38d6944a, 46 },
    { 0, 0 },
    { 0x8d86944d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81069450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac869457, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb69462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80969465, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34569469, 95 },
    { 0, 0 },
    { 0x3d46946b, 95 },
    { 0x9956946c, 78 },
    { 0x96b6946c, 95 },
    { 0x3756946c, 90 },
    { 0x9256946f, 55 },
    { 0xd306946d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816947d, 33 },
    { 0x2ad6947e, 64 },
    { 0xc4d6947f, 90 },
    { 0x1af69480, 33 },
    { 0, 0 },
    { 0x8af69482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda169488, 100 },
    { 0xe2469488, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf206948c, 98 },
    { 0, 0 },
    { 0xd676948e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13669491, 76 },
    { 0xff569492, 61 },
    { 0xba169492, 61 },
    { 0x59269494, 95 },
    { 0xc6e69495, 98 },
    { 0x7ac69496, 98 },
    { 0x8f769496, 23 },
    { 0, 0 },
    { 0xa6a69499, 100 },
    { 0, 0 },
    { 0xd466949b, 90 },
    { 0, 0 },
    { 0x8386949d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a694a0, 77 },
    { 0, 0 },
    { 0xf1f694a2, 61 },
    { 0x50a694a3, 33 },
    { 0xb47694a3, 76 },
    { 0x109694a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06694a8, 98 },
    { 0x1e2694a9, 98 },
    { 0x4f2694a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0694af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f694b3, 91 },
    { 0x435694b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5694bb, 95 },
    { 0xac1694bc, 95 },
    { 0, 0 },
    { 0x12f694be, 93 },
    { 0xb9d694bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a694ca, 33 },
    { 0xecc694cb, 97 },
    { 0xd84694ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8694d0, 55 },
    { 0, 0 },
    { 0xe73694d2, 95 },
    { 0x239694d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9694d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111694dd, 84 },
    { 0x5ab694de, 33 },
    { 0xebc694de, 33 },
    { 0xfab694df, 25 },
    { 0x188694de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab694e5, 98 },
    { 0xd7694e5, 33 },
    { 0x91f694e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x738694ea, 75 },
    { 0x1de694eb, 46 },
    { 0x127694eb, 36 },
    { 0x126694ea, 23 },
    { 0x683694ee, 76 },
    { 0x4eb694ef, 91 },
    { 0, 0 },
    { 0xf68694f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde694f5, 95 },
    { 0, 0 },
    { 0xd0f694f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880694fc, 78 },
    { 0, 0 },
    { 0x949694fe, 23 },
    { 0, 0 },
    { 0x99a69500, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4669504, 61 },
    { 0xe3b69505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40669509, 98 },
    { 0xdcd6950a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52869513, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a69517, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0369520, 23 },
    { 0x5b769521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84469524, 43 },
    { 0xaac69525, 61 },
    { 0xaf769526, 34 },
    { 0, 0 },
    { 0x15c69528, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c6952c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f469535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd69538, 33 },
    { 0x5e969539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd266953d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6869545, 65 },
    { 0, 0 },
    { 0x11e69547, 33 },
    { 0x61b69547, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3156954d, 68 },
    { 0x4a46954e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f69551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b69556, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16869559, 61 },
    { 0, 0 },
    { 0xd286955b, 78 },
    { 0, 0 },
    { 0xa8d6955d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d269560, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569567, 90 },
    { 0x55169568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3236956b, 90 },
    { 0x866956c, 34 },
    { 0xb16956c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5469571, 51 },
    { 0x85269571, 61 },
    { 0xcb669573, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e69578, 46 },
    { 0, 0 },
    { 0xe756957a, 93 },
    { 0, 0 },
    { 0x8206957c, 95 },
    { 0x7b96957c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c69580, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f469587, 100 },
    { 0x4869588, 33 },
    { 0x4f369589, 87 },
    { 0xc096958a, 90 },
    { 0x3f169589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a56958f, 61 },
    { 0x81869590, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4969593, 98 },
    { 0x99169593, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f69599, 52 },
    { 0xfdc6959a, 52 },
    { 0xcce69599, 97 },
    { 0x2386959c, 95 },
    { 0xd886959d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871695a3, 34 },
    { 0xd16695a4, 61 },
    { 0, 0 },
    { 0x55d695a6, 34 },
    { 0x5aa695a7, 23 },
    { 0x46d695a7, 90 },
    { 0, 0 },
    { 0xecf695aa, 85 },
    { 0x80d695ab, 66 },
    { 0, 0 },
    { 0xfd3695ad, 33 },
    { 0, 0 },
    { 0xbcd695af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd695b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2695bb, 93 },
    { 0, 0 },
    { 0x2af695bd, 61 },
    { 0x16d695be, 76 },
    { 0xd35695bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03695c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1695c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb695cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8695cf, 46 },
    { 0, 0 },
    { 0xbd1695d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4695d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2695d9, 74 },
    { 0xc64695da, 52 },
    { 0xe88695db, 77 },
    { 0xce5695dc, 95 },
    { 0xe98695dd, 93 },
    { 0xae1695de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf695e1, 91 },
    { 0xc2695e2, 93 },
    { 0x80f695e1, 23 },
    { 0, 0 },
    { 0x72695e5, 33 },
    { 0, 0 },
    { 0x7f3695e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108695eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f695ef, 61 },
    { 0x806695f0, 91 },
    { 0, 0 },
    { 0x457695f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c695f7, 91 },
    { 0, 0 },
    { 0xb8d695f9, 67 },
    { 0xa81695f9, 33 },
    { 0x776695fb, 61 },
    { 0x305695fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4769600, 95 },
    { 0x1f369600, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74869608, 34 },
    { 0x3ff69609, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f569610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce169613, 98 },
    { 0xb2569614, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52969618, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a6961d, 95 },
    { 0x1306961e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b69621, 90 },
    { 0x10a69622, 95 },
    { 0x6ce69623, 33 },
    { 0x8be69624, 80 },
    { 0xa5169624, 98 },
    { 0x77e69626, 95 },
    { 0x95369627, 90 },
    { 0x8c769623, 46 },
    { 0xad069628, 65 },
    { 0xaf969629, 90 },
    { 0x6ca6962b, 34 },
    { 0xe036962c, 90 },
    { 0xfc86962d, 77 },
    { 0x5ec6962e, 100 },
    { 0xa446962e, 77 },
    { 0x20169628, 93 },
    { 0x20969627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50169635, 90 },
    { 0xc6a69636, 90 },
    { 0, 0 },
    { 0x3e669638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f069643, 43 },
    { 0xf4b69643, 55 },
    { 0x30969645, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce669648, 52 },
    { 0x49869649, 76 },
    { 0x4f06964a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f69651, 95 },
    { 0, 0 },
    { 0x3169653, 91 },
    { 0, 0 },
    { 0x70f69655, 17 },
    { 0, 0 },
    { 0xd6369657, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f46965e, 33 },
    { 0x43d6965f, 33 },
    { 0xa2e69660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2069665, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e46966d, 99 },
    { 0, 0 },
    { 0xa536966f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c69672, 85 },
    { 0x22869673, 95 },
    { 0x53369673, 76 },
    { 0x40c69675, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a86967c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c69680, 98 },
    { 0, 0 },
    { 0xbfd69682, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b69685, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b69688, 95 },
    { 0, 0 },
    { 0x3b76968a, 34 },
    { 0x54b6968b, 93 },
    { 0x6b6968c, 52 },
    { 0, 0 },
    { 0x2276968e, 33 },
    { 0xf9a6968f, 43 },
    { 0, 0 },
    { 0x90069691, 46 },
    { 0, 0 },
    { 0xd5369693, 23 },
    { 0xd4569694, 17 },
    { 0xcde69695, 95 },
    { 0x5969695, 33 },
    { 0x26269697, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd076969b, 90 },
    { 0x9686969c, 100 },
    { 0xc8a6969d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40696a1, 12 },
    { 0x399696a2, 76 },
    { 0xa0a696a3, 95 },
    { 0x94f696a4, 33 },
    { 0, 0 },
    { 0x15b696a6, 55 },
    { 0xa76696a7, 61 },
    { 0x436696a7, 33 },
    { 0x371696a9, 33 },
    { 0x2fe696aa, 34 },
    { 0x5d2696ab, 61 },
    { 0xd1a696ab, 81 },
    { 0x9a1696ab, 88 },
    { 0, 0 },
    { 0xa37696af, 46 },
    { 0x8d0696af, 90 },
    { 0x1fe696b1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332696b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a696b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66696bc, 76 },
    { 0x6f7696bd, 53 },
    { 0, 0 },
    { 0xa6a696bf, 81 },
    { 0, 0 },
    { 0x9a3696c1, 34 },
    { 0x508696c1, 95 },
    { 0, 0 },
    { 0x56d696c4, 69 },
    { 0x58a696c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134696cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7696ce, 99 },
    { 0xf63696ce, 76 },
    { 0x946696d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e696d7, 90 },
    { 0x212696d8, 98 },
    { 0x751696d9, 33 },
    { 0x1dc696d9, 33 },
    { 0xec696db, 68 },
    { 0x8c8696d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30696e1, 33 },
    { 0xd29696e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1696e5, 43 },
    { 0x5e6696e5, 93 },
    { 0xc85696e6, 61 },
    { 0x370696e8, 61 },
    { 0x90e696e9, 81 },
    { 0xd7f696e8, 61 },
    { 0xdd696eb, 90 },
    { 0xb74696ec, 23 },
    { 0x1bd696eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54696f0, 61 },
    { 0, 0 },
    { 0xdbc696f2, 95 },
    { 0x6ba696f2, 100 },
    { 0xdf0696f4, 95 },
    { 0x162696f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9696fd, 33 },
    { 0x6a9696fd, 61 },
    { 0xae2696ff, 33 },
    { 0x821696ff, 95 },
    { 0xe1069701, 93 },
    { 0x25669702, 90 },
    { 0x9e069703, 95 },
    { 0xdfa69700, 93 },
    { 0x23f69705, 88 },
    { 0x57e69705, 95 },
    { 0xc9569707, 90 },
    { 0xc7f69708, 46 },
    { 0x7b669707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6970d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd69711, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29569718, 93 },
    { 0x55e69719, 46 },
    { 0xe1e69719, 95 },
    { 0xd3069719, 23 },
    { 0x3f06971c, 43 },
    { 0xd156971c, 90 },
    { 0, 0 },
    { 0xcc26971f, 23 },
    { 0xc426971f, 90 },
    { 0x3e969721, 34 },
    { 0x2f569722, 33 },
    { 0x74e69723, 52 },
    { 0xc0d69723, 33 },
    { 0xb8269724, 98 },
    { 0x20169726, 61 },
    { 0, 0 },
    { 0x1d869728, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b69731, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56069734, 90 },
    { 0x6ce69735, 56 },
    { 0x1a969736, 43 },
    { 0x24869735, 78 },
    { 0x4a69735, 90 },
    { 0xed669739, 95 },
    { 0, 0 },
    { 0x87f6973b, 90 },
    { 0x2c56973b, 90 },
    { 0x2de6973d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37769742, 98 },
    { 0x5b69743, 46 },
    { 0xf8c69742, 92 },
    { 0x94569742, 90 },
    { 0x54369746, 33 },
    { 0xd7969743, 90 },
    { 0x2f069748, 33 },
    { 0, 0 },
    { 0x4bf6974a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb436974e, 77 },
    { 0xae86974f, 52 },
    { 0x14669750, 90 },
    { 0x2c86974f, 76 },
    { 0x6869751, 55 },
    { 0x6f569753, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81069756, 23 },
    { 0xb0069756, 61 },
    { 0xd6269758, 90 },
    { 0x8c169756, 99 },
    { 0xba86975a, 98 },
    { 0, 0 },
    { 0x92d6975c, 34 },
    { 0x8286975d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec869763, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f469767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456976c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12369770, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f69779, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe366977c, 36 },
    { 0x35c6977d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf69782, 87 },
    { 0, 0 },
    { 0xae169784, 33 },
    { 0x95069785, 88 },
    { 0x10c69786, 78 },
    { 0, 0 },
    { 0x69e69788, 55 },
    { 0, 0 },
    { 0x4ea6978a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6896978e, 90 },
    { 0xa7c6978f, 34 },
    { 0x6636978f, 95 },
    { 0xa8469791, 98 },
    { 0x81069792, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee69799, 33 },
    { 0, 0 },
    { 0xb7e6979b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d697a2, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x650697a5, 33 },
    { 0, 0 },
    { 0xcd4697a7, 98 },
    { 0x537697a8, 23 },
    { 0x4a8697a9, 60 },
    { 0xad0697a9, 100 },
    { 0, 0 },
    { 0x9dd697ac, 90 },
    { 0xbf697ad, 61 },
    { 0xd03697ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e697b2, 46 },
    { 0xa65697b3, 43 },
    { 0x9cb697b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe697b7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443697c0, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c697c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e697cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621697d3, 55 },
    { 0x2d7697d3, 23 },
    { 0xaf2697d5, 61 },
    { 0x731697d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7697db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9697de, 90 },
    { 0xb5c697de, 95 },
    { 0, 0 },
    { 0xbeb697e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a697e6, 98 },
    { 0x2cc697e7, 90 },
    { 0, 0 },
    { 0xf21697e9, 95 },
    { 0xf33697ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a697ef, 55 },
    { 0, 0 },
    { 0xd8d697f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4697f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0697f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5697fc, 55 },
    { 0, 0 },
    { 0x90697fe, 61 },
    { 0xbfc697ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069807, 23 },
    { 0xf1e69808, 85 },
    { 0x3c469809, 90 },
    { 0x9ed6980a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06980d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa69812, 23 },
    { 0xd2d69813, 95 },
    { 0x3bb69814, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe586981b, 90 },
    { 0, 0 },
    { 0xe5f6981d, 90 },
    { 0xdfb6981e, 33 },
    { 0x12f6981f, 91 },
    { 0x7d96981e, 75 },
    { 0x25f6981e, 23 },
    { 0x31a69820, 95 },
    { 0x6ac69823, 43 },
    { 0x17f69823, 33 },
    { 0x3769823, 33 },
    { 0xf9969822, 61 },
    { 0xa7c69827, 23 },
    { 0x82b69827, 76 },
    { 0x78a69827, 66 },
    { 0, 0 },
    { 0xffc6982b, 95 },
    { 0x1a6982c, 33 },
    { 0x7e46982d, 98 },
    { 0, 0 },
    { 0xa996982f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6769836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38369839, 99 },
    { 0x3756983a, 76 },
    { 0x7ae6983b, 23 },
    { 0x3126983c, 87 },
    { 0x60b6983b, 23 },
    { 0xd786983a, 93 },
    { 0, 0 },
    { 0x6fd69840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e69845, 95 },
    { 0, 0 },
    { 0x72669847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6984b, 61 },
    { 0xd926984b, 23 },
    { 0xd4f6984b, 76 },
    { 0x40c6984e, 78 },
    { 0xad86984d, 34 },
    { 0xb626984c, 33 },
    { 0x43369851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b69854, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4469858, 23 },
    { 0x39b69859, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee66985d, 95 },
    { 0x8f76985e, 78 },
    { 0, 0 },
    { 0x6e969860, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a46986a, 90 },
    { 0x496986b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d06986e, 61 },
    { 0x80d6986f, 61 },
    { 0x8c96986e, 98 },
    { 0x6a36986e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa769874, 61 },
    { 0, 0 },
    { 0xe7769876, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b869879, 55 },
    { 0xd4069879, 95 },
    { 0x5dc6987b, 76 },
    { 0, 0 },
    { 0xb966987d, 23 },
    { 0x5596987e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10669888, 76 },
    { 0x8f669889, 34 },
    { 0xd7b6988a, 61 },
    { 0x756988b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d069891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc369894, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d369897, 34 },
    { 0x8c369898, 23 },
    { 0, 0 },
    { 0xd126989a, 33 },
    { 0xc426989a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc186989e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6698a6, 33 },
    { 0, 0 },
    { 0x434698a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b698b0, 90 },
    { 0xc6698b1, 33 },
    { 0, 0 },
    { 0x1fa698b3, 23 },
    { 0x143698b4, 61 },
    { 0x404698b5, 76 },
    { 0x86698b6, 90 },
    { 0xe74698b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd698bc, 33 },
    { 0x685698bc, 94 },
    { 0x9d0698be, 33 },
    { 0, 0 },
    { 0x380698c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x278698c3, 95 },
    { 0xa11698c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da698c7, 23 },
    { 0xbe7698c8, 77 },
    { 0xb16698c8, 33 },
    { 0x576698c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b698d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54698db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac698de, 95 },
    { 0xcd5698de, 23 },
    { 0x474698e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e698e5, 93 },
    { 0, 0 },
    { 0xb97698e7, 52 },
    { 0x97d698e8, 55 },
    { 0x50698e9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83698ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc698f1, 88 },
    { 0, 0 },
    { 0x87b698f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70698f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb698ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e869902, 23 },
    { 0x35b69903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b69907, 23 },
    { 0, 0 },
    { 0xa6d69909, 98 },
    { 0xccb69909, 95 },
    { 0, 0 },
    { 0x2ad6990c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd69913, 65 },
    { 0xd6c69913, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c69919, 36 },
    { 0xb416991a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6666991e, 46 },
    { 0x8716991f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8469923, 95 },
    { 0x8ab69924, 95 },
    { 0x15469925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb69928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc26992b, 95 },
    { 0, 0 },
    { 0x71d6992d, 55 },
    { 0x8076992e, 23 },
    { 0x49e6992e, 33 },
    { 0, 0 },
    { 0xd6e69931, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87769939, 78 },
    { 0x22f69939, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca069940, 90 },
    { 0, 0 },
    { 0x63f69942, 23 },
    { 0, 0 },
    { 0xda569944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa756994a, 55 },
    { 0x1bb6994b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1796994f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8469954, 61 },
    { 0, 0 },
    { 0x2e869956, 61 },
    { 0, 0 },
    { 0x87269958, 33 },
    { 0x7c569958, 33 },
    { 0x4a969959, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a6995e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd69962, 53 },
    { 0xb6f69962, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41869966, 61 },
    { 0x57169966, 76 },
    { 0, 0 },
    { 0x5da69969, 90 },
    { 0xcc969969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b6996f, 23 },
    { 0xa606996f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35369974, 90 },
    { 0x80669975, 77 },
    { 0, 0 },
    { 0x7da69977, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe426997b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926997f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d69982, 98 },
    { 0xa4b69982, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26469988, 98 },
    { 0xe8769989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d6998d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e69994, 23 },
    { 0x90769994, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636999b, 8 },
    { 0x4476999c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab6999f, 98 },
    { 0x56d699a0, 61 },
    { 0x748699a1, 33 },
    { 0x5aa699a1, 33 },
    { 0x56f699a3, 33 },
    { 0x7d0699a4, 33 },
    { 0xd2b699a4, 34 },
    { 0xdc3699a6, 93 },
    { 0x13699a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x840699aa, 23 },
    { 0x403699aa, 91 },
    { 0x630699ac, 17 },
    { 0x71699ac, 23 },
    { 0x11d699ae, 87 },
    { 0x794699aa, 95 },
    { 0xefa699b0, 65 },
    { 0xe8e699b1, 23 },
    { 0x735699ad, 61 },
    { 0x2c1699b0, 55 },
    { 0, 0 },
    { 0xcf5699b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89699b8, 93 },
    { 0x8f0699b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5699bc, 61 },
    { 0xefb699bc, 34 },
    { 0x83e699bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28699c5, 87 },
    { 0, 0 },
    { 0x994699c7, 61 },
    { 0, 0 },
    { 0xb70699c9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce699ce, 90 },
    { 0x63f699cf, 52 },
    { 0xb6699d0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20699d4, 90 },
    { 0xec0699d5, 90 },
    { 0xc14699d6, 66 },
    { 0x1f1699d7, 98 },
    { 0x821699d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28699db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5699de, 61 },
    { 0, 0 },
    { 0xba0699e0, 61 },
    { 0xd13699e1, 76 },
    { 0, 0 },
    { 0xf96699e3, 98 },
    { 0, 0 },
    { 0x4f9699e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe699e9, 33 },
    { 0x69a699e9, 52 },
    { 0x434699eb, 90 },
    { 0xdd5699ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x604699ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2699f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965699f7, 83 },
    { 0xa60699f8, 98 },
    { 0x398699f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c699fd, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1269a03, 81 },
    { 0, 0 },
    { 0x83369a05, 43 },
    { 0xd2969a06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f069a14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75969a18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57369a1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55069a24, 97 },
    { 0xef769a25, 61 },
    { 0x22a69a26, 100 },
    { 0, 0 },
    { 0xc6e69a28, 66 },
    { 0xacf69a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd69a2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c569a33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62569a36, 52 },
    { 0x99a69a37, 90 },
    { 0, 0 },
    { 0x1d969a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a69a3c, 75 },
    { 0x51469a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65669a40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee69a44, 90 },
    { 0xb9969a44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42169a51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7269a56, 94 },
    { 0xb2069a57, 61 },
    { 0xf4269a57, 76 },
    { 0, 0 },
    { 0x52169a5a, 33 },
    { 0x66569a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c969a5e, 61 },
    { 0, 0 },
    { 0x4da69a60, 98 },
    { 0, 0 },
    { 0x60b69a62, 55 },
    { 0, 0 },
    { 0xf5069a64, 33 },
    { 0xe3669a65, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be69a6b, 90 },
    { 0x9b569a6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e569a71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e69a74, 76 },
    { 0, 0 },
    { 0x9ed69a76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80269a79, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf669a7f, 98 },
    { 0x90869a80, 78 },
    { 0xda969a81, 91 },
    { 0xa0469a82, 61 },
    { 0, 0 },
    { 0x5cb69a84, 65 },
    { 0xc1069a85, 61 },
    { 0x9cc69a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11869a89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97869a99, 66 },
    { 0x10869a99, 99 },
    { 0x61c69a9b, 34 },
    { 0xe1a69a9c, 91 },
    { 0xdfa69a9b, 90 },
    { 0xfcb69a9e, 33 },
    { 0, 0 },
    { 0x41169aa0, 68 },
    { 0xb4969aa0, 61 },
    { 0xa5e69aa2, 23 },
    { 0x2eb69aa1, 98 },
    { 0x7dc69aa3, 76 },
    { 0x87969aa0, 99 },
    { 0x38869aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4769aaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e69ab2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2469abb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa69ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d69ac9, 68 },
    { 0x24169aca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf269adb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57969adf, 55 },
    { 0, 0 },
    { 0xefd69ae1, 33 },
    { 0x3ea69ae2, 55 },
    { 0x50c69ae2, 61 },
    { 0x6b869ae2, 55 },
    { 0x19769ae5, 33 },
    { 0xfb069ae6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf69aef, 33 },
    { 0xfa469aef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469af3, 36 },
    { 0xef269af4, 90 },
    { 0xcb269af5, 52 },
    { 0x3e69af5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21069af9, 61 },
    { 0xed869afa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd969afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0769b04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c269b07, 61 },
    { 0x8a469b08, 65 },
    { 0xd2769b08, 96 },
    { 0xb1669b09, 76 },
    { 0x46769b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa69b0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2569b12, 85 },
    { 0, 0 },
    { 0x89369b14, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f69b17, 76 },
    { 0x88669b18, 23 },
    { 0, 0 },
    { 0x11f69b1a, 76 },
    { 0, 0 },
    { 0x8fa69b1c, 84 },
    { 0x5869b1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0269b21, 34 },
    { 0x8d169b22, 43 },
    { 0x53569b22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f69b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f69b2c, 17 },
    { 0xb9e69b2c, 100 },
    { 0, 0 },
    { 0xd5569b2f, 76 },
    { 0x43a69b2f, 91 },
    { 0, 0 },
    { 0x81369b32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac869b35, 61 },
    { 0x5269b36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c969b3a, 87 },
    { 0xbb69b3b, 62 },
    { 0xe8869b3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e069b3f, 61 },
    { 0x57a69b40, 65 },
    { 0x95169b41, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x97369b44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87169b49, 76 },
    { 0x9e569b49, 33 },
    { 0, 0 },
    { 0x99a69b4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff969b4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff069b52, 66 },
    { 0xa1869b53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55969b61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f669b64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d69b6a, 55 },
    { 0x24a69b6b, 61 },
    { 0x84569b6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d769b70, 100 },
    { 0xa4a69b70, 23 },
    { 0x71a69b72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43869b77, 17 },
    { 0, 0 },
    { 0xf0d69b79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c69b80, 90 },
    { 0xa3e69b81, 52 },
    { 0x9a069b82, 68 },
    { 0x3fc69b82, 98 },
    { 0xceb69b82, 43 },
    { 0xce469b85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec769b88, 90 },
    { 0, 0 },
    { 0xa8269b8a, 61 },
    { 0, 0 },
    { 0xcdb69b8c, 33 },
    { 0x95169b8c, 33 },
    { 0xd5569b8d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669b91, 90 },
    { 0xcf369b91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e569b96, 33 },
    { 0, 0 },
    { 0xdb369b98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b69b9b, 90 },
    { 0x4c269b9b, 66 },
    { 0xd4d69b9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56169ba1, 61 },
    { 0x6a669ba2, 46 },
    { 0x66f69ba3, 95 },
    { 0xdee69ba2, 78 },
    { 0x15969ba2, 23 },
    { 0x1e069ba6, 23 },
    { 0xdfd69ba6, 95 },
    { 0x4bf69ba8, 67 },
    { 0, 0 },
    { 0xff269baa, 99 },
    { 0x84669bab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d069baf, 23 },
    { 0, 0 },
    { 0x4a869bb1, 95 },
    { 0xa9a69bb2, 66 },
    { 0, 0 },
    { 0x55969bb4, 95 },
    { 0xdf869bb5, 61 },
    { 0x1f269bb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d969bba, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca69bc3, 65 },
    { 0, 0 },
    { 0x2f769bc5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13769bc9, 61 },
    { 0x9c069bc9, 100 },
    { 0xc1d69bc9, 43 },
    { 0, 0 },
    { 0x7f469bcd, 95 },
    { 0, 0 },
    { 0xeee69bcf, 95 },
    { 0xcb269bd0, 17 },
    { 0x66469bd1, 98 },
    { 0, 0 },
    { 0xad469bd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca169bd8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2169bdc, 66 },
    { 0, 0 },
    { 0x29d69bde, 98 },
    { 0xf9969bdf, 93 },
    { 0, 0 },
    { 0x77c69be1, 61 },
    { 0xbe169be2, 46 },
    { 0xd8969be3, 61 },
    { 0, 0 },
    { 0x58169be5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67369be9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e69bed, 23 },
    { 0xb0a69bed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b69bf4, 75 },
    { 0, 0 },
    { 0xbb169bf6, 97 },
    { 0xb9869bf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a69bfc, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f69c08, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3669c0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20769c11, 98 },
    { 0, 0 },
    { 0x31169c13, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c69c18, 17 },
    { 0, 0 },
    { 0x9169c1a, 46 },
    { 0, 0 },
    { 0xffe69c1c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a69c20, 29 },
    { 0, 0 },
    { 0x82a69c22, 95 },
    { 0xbaf69c23, 85 },
    { 0x6f69c24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3369c27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39769c2b, 95 },
    { 0xef069c2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c869c30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b69c35, 90 },
    { 0xd0b69c35, 55 },
    { 0, 0 },
    { 0xf2769c38, 46 },
    { 0x8fe69c39, 61 },
    { 0, 0 },
    { 0xb6369c3b, 95 },
    { 0xb3b69c3c, 90 },
    { 0xd7569c3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47369c48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db69c4b, 85 },
    { 0, 0 },
    { 0x41a69c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b69c51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f69c57, 98 },
    { 0xec369c58, 93 },
    { 0xd8b69c59, 55 },
    { 0xf9369c5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb569c60, 90 },
    { 0x8ac69c60, 33 },
    { 0x3d169c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd69c66, 95 },
    { 0x4c669c67, 34 },
    { 0, 0 },
    { 0xb0a69c69, 76 },
    { 0x14f69c6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e69c6e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31669c77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e769c7f, 90 },
    { 0xc4c69c80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4269c83, 66 },
    { 0x2c869c84, 88 },
    { 0x6bb69c85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a69c8a, 33 },
    { 0x63969c8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb69c8e, 55 },
    { 0x43069c8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda369c98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c9d, 97 },
    { 0, 0 },
    { 0xe1569c9f, 81 },
    { 0, 0 },
    { 0xade69ca1, 90 },
    { 0, 0 },
    { 0xd8669ca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97769caa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef69cae, 61 },
    { 0xea769cae, 55 },
    { 0x81c69cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40869cb6, 23 },
    { 0x84169cb7, 95 },
    { 0x25469cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d669cbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe269cc2, 23 },
    { 0x58269cc3, 98 },
    { 0x8e569cc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc369cc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d69cca, 93 },
    { 0, 0 },
    { 0xafa69ccc, 98 },
    { 0xc2e69ccd, 98 },
    { 0x83969ccc, 85 },
    { 0x92969ccf, 33 },
    { 0xb3569cd0, 98 },
    { 0xe4f69cd1, 33 },
    { 0x4369ccd, 61 },
    { 0x6ba69cd3, 90 },
    { 0xfb769cd4, 46 },
    { 0x25e69cd5, 98 },
    { 0x85969cd6, 61 },
    { 0xcbd69cd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65469cdf, 90 },
    { 0, 0 },
    { 0x16f69ce1, 95 },
    { 0x83469ce2, 90 },
    { 0, 0 },
    { 0xb0a69ce4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49069ce9, 95 },
    { 0xe1a69ce9, 61 },
    { 0, 0 },
    { 0x9ee69cec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b269cf0, 78 },
    { 0xd5269cf1, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d69cf4, 23 },
    { 0xd4069cf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af69cf9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c69d04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86069d11, 52 },
    { 0xe8d69d11, 61 },
    { 0xf8569d11, 90 },
    { 0xcd669d14, 92 },
    { 0xb1069d14, 90 },
    { 0, 0 },
    { 0x93169d17, 98 },
    { 0x70969d18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb669d1b, 55 },
    { 0x1b669d1b, 100 },
    { 0x83069d1d, 78 },
    { 0xda569d1c, 34 },
    { 0x98d69d1d, 91 },
    { 0x14969d20, 47 },
    { 0, 0 },
    { 0x47669d22, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f69d26, 74 },
    { 0x11e69d27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f69d2a, 98 },
    { 0xb2b69d2b, 46 },
    { 0, 0 },
    { 0xdf969d2d, 61 },
    { 0xa9769d2e, 77 },
    { 0x27369d2f, 23 },
    { 0x3c669d2f, 98 },
    { 0xa6269d31, 43 },
    { 0x39b69d2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e69d3b, 66 },
    { 0xced69d3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0969d40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a69d43, 85 },
    { 0xd1f69d43, 90 },
    { 0, 0 },
    { 0xf69d46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b69d49, 36 },
    { 0x9169d4a, 17 },
    { 0, 0 },
    { 0x8d69d4c, 33 },
    { 0x9cd69d4d, 76 },
    { 0xc3669d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d969d52, 95 },
    { 0x13969d52, 91 },
    { 0x80569d53, 91 },
    { 0x5bf69d54, 90 },
    { 0xb2d69d56, 66 },
    { 0x90569d56, 69 },
    { 0x55169d54, 33 },
    { 0x66b69d52, 95 },
    { 0, 0 },
    { 0x169d5b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b69d5f, 62 },
    { 0xdb969d60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf69d67, 43 },
    { 0xf2069d67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c69d6d, 76 },
    { 0, 0 },
    { 0x5a069d6f, 61 },
    { 0xe2f69d6f, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe669d73, 99 },
    { 0x74f69d74, 33 },
    { 0, 0 },
    { 0x1fc69d76, 90 },
    { 0x8569d77, 33 },
    { 0x95a69d78, 36 },
    { 0x7d869d79, 95 },
    { 0, 0 },
    { 0xa2d69d7b, 34 },
    { 0xd6d69d7c, 43 },
    { 0xc4469d7b, 84 },
    { 0x6169d7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c69d85, 93 },
    { 0, 0 },
    { 0xf2969d87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b69d8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f69d8f, 19 },
    { 0x9de69d8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f369d96, 61 },
    { 0xf1669d97, 95 },
    { 0xb8269d96, 23 },
    { 0, 0 },
    { 0xaaf69d9a, 34 },
    { 0, 0 },
    { 0xe769d9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b69d9f, 61 },
    { 0, 0 },
    { 0x54a69da1, 23 },
    { 0xef869da2, 55 },
    { 0x9dc69da1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10469daa, 95 },
    { 0x7f169dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca769db0, 33 },
    { 0, 0 },
    { 0x9d269db2, 33 },
    { 0x1aa69db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce69db8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f69dbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c69dc3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec69dc6, 98 },
    { 0xdae69dc7, 67 },
    { 0xb1569dc8, 46 },
    { 0, 0 },
    { 0xb569dca, 52 },
    { 0x31969dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c69dd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f869dd7, 78 },
    { 0x1ae69dd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f69ddd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb969de3, 77 },
    { 0x70569de4, 99 },
    { 0, 0 },
    { 0x6fb69de6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c69de9, 77 },
    { 0x4c869de9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52869def, 55 },
    { 0x9d269df0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x14569df3, 100 },
    { 0xec169df4, 37 },
    { 0, 0 },
    { 0x4a269df6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa269df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be69e04, 85 },
    { 0, 0 },
    { 0x9e269e06, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75969e0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e69e0d, 61 },
    { 0x37f69e0e, 33 },
    { 0xf5469e0e, 97 },
    { 0x98469e10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15369e14, 33 },
    { 0, 0 },
    { 0x1469e16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2269e19, 99 },
    { 0, 0 },
    { 0x53569e1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da69e25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61469e34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f69e3b, 17 },
    { 0x21e69e3b, 90 },
    { 0, 0 },
    { 0x32969e3e, 34 },
    { 0x26a69e3e, 78 },
    { 0xe7d69e3e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d869e4a, 90 },
    { 0x99769e4b, 34 },
    { 0xa2669e4c, 23 },
    { 0x2d69e4d, 61 },
    { 0, 0 },
    { 0x2ba69e4f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b169e52, 34 },
    { 0, 0 },
    { 0xc3669e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad69e57, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a269e5b, 61 },
    { 0, 0 },
    { 0x43a69e5d, 66 },
    { 0, 0 },
    { 0x96569e5f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea69e62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf569e68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2669e6b, 91 },
    { 0x2c969e6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a69e6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde369e75, 61 },
    { 0xd6869e76, 43 },
    { 0x2c069e77, 23 },
    { 0xfb869e78, 66 },
    { 0x32e69e78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3369e7e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80669e83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba69e8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b69e8e, 65 },
    { 0xf7869e8e, 33 },
    { 0xc3569e90, 23 },
    { 0, 0 },
    { 0x7c69e92, 90 },
    { 0, 0 },
    { 0xcf369e94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f069e98, 85 },
    { 0xf8c69e99, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6869e9c, 33 },
    { 0x10c69e9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f69ea0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98369ea3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a269ea6, 95 },
    { 0xcfc69ea7, 95 },
    { 0x4ed69ea7, 95 },
    { 0xc2469ea7, 33 },
    { 0x4b69eaa, 33 },
    { 0, 0 },
    { 0x55869eac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9269eaf, 90 },
    { 0, 0 },
    { 0xc4a69eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0669ebb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb69ec2, 95 },
    { 0, 0 },
    { 0xb7369ec4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82669ecd, 34 },
    { 0x4d469ece, 66 },
    { 0x2d169ecf, 33 },
    { 0xdf69ed0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5669ed4, 61 },
    { 0x21e69ed5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f69ed8, 95 },
    { 0x8569ed8, 61 },
    { 0x76269ed8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be69edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8369ee3, 61 },
    { 0x28969ee3, 61 },
    { 0x1a969ee5, 23 },
    { 0x90c69ee4, 23 },
    { 0xb0769ee7, 98 },
    { 0x3dd69ee7, 78 },
    { 0x8f69ee4, 87 },
    { 0xade69ee6, 76 },
    { 0x34569ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f69ef3, 46 },
    { 0xd8d69ef3, 17 },
    { 0, 0 },
    { 0xbb469ef6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb369efa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f869f01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d69f04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e069f0b, 95 },
    { 0x5b569f0b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1069f10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa769f13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9269f1a, 23 },
    { 0x4a869f1b, 33 },
    { 0, 0 },
    { 0x5c369f1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b69f20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc769f23, 55 },
    { 0, 0 },
    { 0xc6369f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed69f31, 75 },
    { 0x64069f32, 55 },
    { 0x23569f33, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7069f36, 84 },
    { 0, 0 },
    { 0x9db69f38, 78 },
    { 0, 0 },
    { 0x62669f3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f69f43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a69f49, 23 },
    { 0x5669f4a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1969f52, 77 },
    { 0xbfe69f53, 33 },
    { 0x7ea69f54, 23 },
    { 0, 0 },
    { 0x1d169f56, 95 },
    { 0, 0 },
    { 0x87369f58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d69f5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6669f61, 98 },
    { 0, 0 },
    { 0x68169f63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d569f66, 68 },
    { 0, 0 },
    { 0x5f869f68, 76 },
    { 0, 0 },
    { 0xe8969f6a, 93 },
    { 0xa3669f6b, 78 },
    { 0x92469f6b, 95 },
    { 0, 0 },
    { 0x22c69f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22169f71, 90 },
    { 0xb0d69f72, 95 },
    { 0x55f69f73, 76 },
    { 0x3f969f74, 33 },
    { 0x39d69f73, 46 },
    { 0x87969f76, 98 },
    { 0x2aa69f77, 98 },
    { 0xa5369f77, 43 },
    { 0x7ff69f74, 68 },
    { 0xac269f74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b69f7e, 23 },
    { 0xd1b69f7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2769f82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b69f86, 87 },
    { 0x98369f87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c69f8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da69f8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57169f96, 90 },
    { 0x54d69f97, 33 },
    { 0x5d769f97, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf69f9b, 61 },
    { 0x6ff69f9b, 90 },
    { 0x65e69f9c, 91 },
    { 0, 0 },
    { 0x8b669f9f, 99 },
    { 0x37469f9f, 33 },
    { 0x5e769fa1, 90 },
    { 0xce369f9f, 61 },
    { 0, 0 },
    { 0x5f569fa4, 90 },
    { 0xbfa69fa4, 95 },
    { 0, 0 },
    { 0x5de69fa7, 55 },
    { 0x18269fa7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25969fae, 94 },
    { 0x50d69faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee69fb3, 33 },
    { 0, 0 },
    { 0xa3969fb5, 90 },
    { 0x76869fb6, 33 },
    { 0xc6369fb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f69fba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d69fbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28969fcb, 56 },
    { 0x80169fcb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf69fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f69fd4, 61 },
    { 0xb2b69fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c669fdb, 34 },
    { 0xb9d69fdc, 97 },
    { 0xb3169fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec569fe2, 90 },
    { 0xcca69fe3, 90 },
    { 0x12e69fe3, 33 },
    { 0x5df69fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba69fe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9769ff0, 33 },
    { 0, 0 },
    { 0xba869ff2, 98 },
    { 0x26e69ff3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8769ff9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d69ffd, 61 },
    { 0x3df69ffe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b6a008, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6a00f, 43 },
    { 0, 0 },
    { 0xe96a011, 23 },
    { 0, 0 },
    { 0xb496a013, 75 },
    { 0x3f16a013, 10 },
    { 0x9c06a015, 76 },
    { 0xb266a015, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe036a01b, 91 },
    { 0x4356a01b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c06a020, 55 },
    { 0xfc86a020, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e36a025, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5216a02a, 34 },
    { 0x2da6a02a, 84 },
    { 0x2676a02c, 56 },
    { 0x8f16a02c, 68 },
    { 0x696a02e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf6a033, 23 },
    { 0, 0 },
    { 0x1896a035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d6a03b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b66a03f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa256a043, 90 },
    { 0x48a6a044, 23 },
    { 0, 0 },
    { 0xd5a6a046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce06a04b, 99 },
    { 0x53a6a04c, 90 },
    { 0x7266a04b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff6a052, 95 },
    { 0, 0 },
    { 0xf5e6a054, 90 },
    { 0, 0 },
    { 0x61b6a056, 85 },
    { 0xc156a057, 55 },
    { 0x9486a058, 55 },
    { 0, 0 },
    { 0xbc86a05a, 33 },
    { 0xe776a05a, 34 },
    { 0xc226a05c, 33 },
    { 0xa2b6a05d, 83 },
    { 0xe336a05c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a6a06a, 34 },
    { 0xb1a6a06b, 66 },
    { 0x8ca6a06c, 61 },
    { 0, 0 },
    { 0x446a06e, 90 },
    { 0, 0 },
    { 0xb726a070, 33 },
    { 0xab26a070, 90 },
    { 0, 0 },
    { 0x8506a073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d86a079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5966a07f, 98 },
    { 0x8286a080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646a086, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d6a08a, 99 },
    { 0, 0 },
    { 0x17c6a08c, 33 },
    { 0, 0 },
    { 0x6a26a08e, 33 },
    { 0x68b6a08e, 80 },
    { 0xd466a090, 95 },
    { 0x3126a08f, 55 },
    { 0x4b86a08f, 90 },
    { 0xe366a093, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16a097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f76a0a1, 98 },
    { 0x1316a0a2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac6a0ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2226a0b4, 46 },
    { 0, 0 },
    { 0xad26a0b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe986a0ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6a0bd, 33 },
    { 0, 0 },
    { 0x1356a0bf, 33 },
    { 0x3126a0bf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb336a0c6, 33 },
    { 0, 0 },
    { 0xb516a0c8, 43 },
    { 0x85d6a0c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7496a0cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd616a0d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6566a0d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c6a0db, 76 },
    { 0xfd26a0dc, 93 },
    { 0xfba6a0dd, 90 },
    { 0xdc66a0de, 98 },
    { 0x87c6a0de, 43 },
    { 0, 0 },
    { 0x8dd6a0e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1386a0e6, 33 },
    { 0, 0 },
    { 0xf086a0e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a6a0eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e6a0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3296a0f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac6a0ff, 90 },
    { 0x7e16a100, 93 },
    { 0xfb96a100, 33 },
    { 0xb736a102, 90 },
    { 0x30b6a103, 33 },
    { 0x5916a104, 76 },
    { 0x67f6a105, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6a108, 91 },
    { 0xef36a109, 33 },
    { 0x3a66a10a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde96a10e, 90 },
    { 0x8fd6a10f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906a11a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd306a11f, 93 },
    { 0xbd96a120, 23 },
    { 0, 0 },
    { 0xf4b6a122, 95 },
    { 0x2f86a123, 61 },
    { 0x9ff6a124, 93 },
    { 0xc696a125, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad6a128, 34 },
    { 0x6566a129, 66 },
    { 0x70b6a12a, 33 },
    { 0xed06a12b, 33 },
    { 0x3e76a12b, 84 },
    { 0x336a12c, 97 },
    { 0xe8c6a12d, 66 },
    { 0, 0 },
    { 0x80d6a130, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef6a134, 61 },
    { 0x27a6a134, 98 },
    { 0x2196a136, 76 },
    { 0x6086a137, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd076a13a, 76 },
    { 0xed46a13b, 99 },
    { 0x6fd6a13c, 90 },
    { 0x85e6a13d, 87 },
    { 0xaff6a13b, 93 },
    { 0, 0 },
    { 0xa2e6a140, 76 },
    { 0x27a6a141, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea86a145, 23 },
    { 0xc0b6a146, 55 },
    { 0, 0 },
    { 0x6a26a148, 99 },
    { 0x3726a149, 52 },
    { 0, 0 },
    { 0x67e6a14b, 76 },
    { 0x5106a14c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8076a155, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb46a158, 33 },
    { 0x6246a159, 52 },
    { 0xfd66a15a, 97 },
    { 0xc946a15b, 76 },
    { 0, 0 },
    { 0x216a15d, 34 },
    { 0, 0 },
    { 0xe1e6a15f, 23 },
    { 0x5706a15f, 83 },
    { 0xf166a15f, 69 },
    { 0, 0 },
    { 0x5a86a163, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6a166, 90 },
    { 0x5576a166, 43 },
    { 0, 0 },
    { 0x4686a169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4976a16d, 33 },
    { 0xe076a16e, 61 },
    { 0x7fd6a16f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d6a173, 91 },
    { 0x4a46a174, 84 },
    { 0xca86a175, 34 },
    { 0xd766a176, 91 },
    { 0xaf26a175, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd6a17f, 33 },
    { 0xda6a180, 33 },
    { 0x3256a181, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe786a18a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf6a190, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf476a196, 95 },
    { 0x3036a196, 33 },
    { 0xd796a196, 33 },
    { 0x6336a198, 23 },
    { 0x42d6a197, 36 },
    { 0xcc86a19b, 90 },
    { 0, 0 },
    { 0xd516a19d, 46 },
    { 0x2456a19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e6a1a2, 90 },
    { 0xed76a1a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9646a1a6, 97 },
    { 0xe416a1a6, 98 },
    { 0xc216a1a7, 90 },
    { 0, 0 },
    { 0x35c6a1aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5276a1af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8956a1b4, 98 },
    { 0, 0 },
    { 0x43f6a1b6, 90 },
    { 0x3316a1b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f96a1ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e86a1bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d76a1c3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb156a1c7, 75 },
    { 0x766a1c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6a1cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd6a1d3, 61 },
    { 0x5336a1d3, 33 },
    { 0, 0 },
    { 0xc3b6a1d6, 61 },
    { 0, 0 },
    { 0xf936a1d8, 76 },
    { 0x3bd6a1d8, 33 },
    { 0, 0 },
    { 0xaf06a1db, 56 },
    { 0x10e6a1db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd686a1e1, 55 },
    { 0x6886a1e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf06a1e6, 33 },
    { 0x7b26a1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2446a1f5, 78 },
    { 0, 0 },
    { 0x87d6a1f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d26a1fa, 61 },
    { 0x8736a1fb, 90 },
    { 0x7226a1fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1566a201, 61 },
    { 0, 0 },
    { 0xa8e6a203, 43 },
    { 0xee56a204, 98 },
    { 0x4ea6a203, 36 },
    { 0, 0 },
    { 0x3366a207, 90 },
    { 0xa016a208, 61 },
    { 0x4e96a209, 98 },
    { 0xf6b6a209, 46 },
    { 0x6986a20b, 98 },
    { 0x80f6a20a, 55 },
    { 0x5246a20d, 98 },
    { 0, 0 },
    { 0xb396a20f, 55 },
    { 0x1486a210, 95 },
    { 0xc1a6a211, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc6a214, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe106a21a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f6a21f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf066a223, 90 },
    { 0xa666a223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c6a234, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2396a23d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f6a241, 61 },
    { 0, 0 },
    { 0x4c86a243, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb896a24b, 98 },
    { 0x2926a24c, 23 },
    { 0, 0 },
    { 0x776a24e, 90 },
    { 0x2066a24f, 55 },
    { 0x5026a250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf6a255, 43 },
    { 0xa856a256, 98 },
    { 0x9976a256, 87 },
    { 0, 0 },
    { 0x2626a259, 90 },
    { 0, 0 },
    { 0xbad6a25b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab6a25e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd6a263, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe286a266, 90 },
    { 0x5e36a267, 23 },
    { 0xb566a268, 66 },
    { 0x6e26a269, 33 },
    { 0, 0 },
    { 0xf606a26b, 93 },
    { 0xe366a26b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x5176a26f, 90 },
    { 0, 0 },
    { 0xe736a271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb6a275, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb486a27a, 61 },
    { 0, 0 },
    { 0xa226a27c, 77 },
    { 0xfa56a27d, 33 },
    { 0xb076a27d, 96 },
    { 0xf76a27f, 99 },
    { 0x5896a280, 55 },
    { 0xe6d6a281, 95 },
    { 0xc826a282, 23 },
    { 0x93c6a27f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe6a288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c6a28e, 87 },
    { 0x8e6a28f, 55 },
    { 0xd836a290, 17 },
    { 0x8036a291, 61 },
    { 0xc736a292, 23 },
    { 0x2346a292, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326a29a, 55 },
    { 0xa436a29b, 75 },
    { 0x146a29c, 90 },
    { 0xf6e6a29d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5046a2a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc6a2aa, 34 },
    { 0, 0 },
    { 0xf526a2ac, 61 },
    { 0x2036a2ad, 77 },
    { 0xa46a2ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146a2b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6a2b5, 33 },
    { 0xacd6a2b6, 61 },
    { 0, 0 },
    { 0xe136a2b8, 33 },
    { 0, 0 },
    { 0xc096a2ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9826a2bd, 61 },
    { 0x5956a2be, 75 },
    { 0, 0 },
    { 0x3fd6a2c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa616a2c8, 52 },
    { 0xc8f6a2c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c6a2cc, 61 },
    { 0, 0 },
    { 0xd246a2ce, 52 },
    { 0xa0c6a2cf, 100 },
    { 0xc496a2d0, 33 },
    { 0xeb56a2d1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9336a2d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab6a2d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde6a2da, 93 },
    { 0x70e6a2db, 83 },
    { 0x9476a2db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf6a2e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7816a2e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826a2e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a86a2ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab6a2f2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646a2f5, 93 },
    { 0x12f6a2f6, 55 },
    { 0x6cd6a2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e6a2fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf676a301, 40 },
    { 0xfc56a302, 95 },
    { 0x5f06a302, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c6a307, 90 },
    { 0, 0 },
    { 0x33a6a309, 46 },
    { 0, 0 },
    { 0x4836a30b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef76a30f, 95 },
    { 0xac26a310, 100 },
    { 0x9f46a310, 90 },
    { 0x7fd6a312, 33 },
    { 0x21b6a312, 23 },
    { 0xea16a314, 23 },
    { 0x8786a315, 34 },
    { 0, 0 },
    { 0xb3b6a317, 90 },
    { 0, 0 },
    { 0x71d6a319, 75 },
    { 0, 0 },
    { 0x8206a31b, 96 },
    { 0x2db6a31c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3406a31f, 33 },
    { 0xbc56a320, 95 },
    { 0xe426a321, 98 },
    { 0, 0 },
    { 0x4446a323, 33 },
    { 0x1dd6a324, 46 },
    { 0, 0 },
    { 0x1716a326, 68 },
    { 0, 0 },
    { 0x9d16a328, 90 },
    { 0, 0 },
    { 0x6c76a32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf56a32f, 34 },
    { 0x78d6a330, 23 },
    { 0x5656a331, 55 },
    { 0x1916a331, 40 },
    { 0xf4a6a331, 17 },
    { 0xd3c6a32f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a338, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb6a33b, 81 },
    { 0, 0 },
    { 0x9ac6a33d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7096a343, 77 },
    { 0x6906a343, 90 },
    { 0, 0 },
    { 0x6836a346, 33 },
    { 0x8c6a347, 90 },
    { 0x9b96a346, 61 },
    { 0, 0 },
    { 0xd266a34a, 33 },
    { 0x4e26a34a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a6a351, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e6a359, 23 },
    { 0xb566a359, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c76a35d, 90 },
    { 0, 0 },
    { 0x6666a35f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c6a362, 84 },
    { 0x9cd6a363, 90 },
    { 0x5d96a364, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6a367, 33 },
    { 0x3796a368, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a66a36f, 95 },
    { 0, 0 },
    { 0x8476a371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c6a377, 90 },
    { 0x6226a377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c76a37d, 95 },
    { 0x54f6a37e, 76 },
    { 0x656a37e, 55 },
    { 0x93c6a380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3526a383, 101 },
    { 0x8676a383, 43 },
    { 0x3e36a383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8616a38d, 87 },
    { 0x6b36a38e, 33 },
    { 0xe9e6a38d, 61 },
    { 0x8956a38d, 90 },
    { 0x5276a390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6a395, 90 },
    { 0x4b36a396, 90 },
    { 0, 0 },
    { 0x6606a398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e6a39c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6a39f, 75 },
    { 0xb716a39f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a6a3a6, 95 },
    { 0x1f16a3a6, 65 },
    { 0xc486a3a8, 98 },
    { 0, 0 },
    { 0xa336a3aa, 90 },
    { 0, 0 },
    { 0x3546a3ac, 91 },
    { 0, 0 },
    { 0x7266a3ae, 81 },
    { 0x2416a3ae, 75 },
    { 0xe756a3b0, 95 },
    { 0, 0 },
    { 0x3c66a3b2, 88 },
    { 0, 0 },
    { 0xe886a3b4, 95 },
    { 0x7ab6a3b5, 38 },
    { 0x47f6a3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a6a3bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a3c0, 2 },
    { 0, 0 },
    { 0x216a3c2, 75 },
    { 0, 0 },
    { 0xbf66a3c4, 95 },
    { 0x686a3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5606a3cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e6a3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb6a3d7, 90 },
    { 0, 0 },
    { 0xc126a3d9, 53 },
    { 0xa526a3da, 90 },
    { 0xabc6a3db, 23 },
    { 0x8c6a3dc, 40 },
    { 0x3756a3dc, 100 },
    { 0, 0 },
    { 0x7876a3df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f6a3e9, 33 },
    { 0xa146a3ea, 55 },
    { 0x2696a3eb, 33 },
    { 0x85d6a3ea, 98 },
    { 0xb186a3ea, 23 },
    { 0, 0 },
    { 0xb906a3ef, 78 },
    { 0xf766a3ef, 33 },
    { 0x7176a3f0, 90 },
    { 0xd3c6a3f2, 77 },
    { 0x7016a3ef, 93 },
    { 0xf336a3f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc826a3f8, 91 },
    { 0, 0 },
    { 0xd1b6a3fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2866a408, 61 },
    { 0, 0 },
    { 0x50b6a40a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1656a40e, 90 },
    { 0xeaa6a40e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d6a41d, 90 },
    { 0x7276a41e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6396a42a, 66 },
    { 0x3fc6a42a, 93 },
    { 0, 0 },
    { 0x4456a42d, 33 },
    { 0x9536a42e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c6a432, 33 },
    { 0, 0 },
    { 0x296a434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad6a437, 61 },
    { 0, 0 },
    { 0xa796a439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c66a441, 90 },
    { 0x5126a442, 34 },
    { 0xac66a441, 34 },
    { 0, 0 },
    { 0x506a445, 55 },
    { 0x5d56a445, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e66a44a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6a44d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d6a456, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e26a45e, 61 },
    { 0x4286a45e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4486a462, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8796a468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c66a46b, 33 },
    { 0xc66a46c, 97 },
    { 0, 0 },
    { 0x1016a46e, 23 },
    { 0xc8f6a46e, 90 },
    { 0x4e36a470, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a6a474, 98 },
    { 0, 0 },
    { 0xacd6a476, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8946a479, 90 },
    { 0, 0 },
    { 0x66b6a47b, 34 },
    { 0, 0 },
    { 0x4746a47d, 98 },
    { 0x75d6a47e, 66 },
    { 0, 0 },
    { 0xd4f6a480, 90 },
    { 0x3c6a481, 76 },
    { 0x78a6a482, 23 },
    { 0x8a86a483, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956a486, 33 },
    { 0x6596a486, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca6a48a, 64 },
    { 0, 0 },
    { 0x2d16a48c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e6a48f, 88 },
    { 0xf766a490, 55 },
    { 0x67c6a48f, 98 },
    { 0xb86a491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c6a495, 94 },
    { 0x8e66a496, 23 },
    { 0x3cc6a497, 77 },
    { 0x2a36a496, 33 },
    { 0x1196a499, 91 },
    { 0xd806a497, 98 },
    { 0xc976a498, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac06a4a2, 87 },
    { 0, 0 },
    { 0xf516a4a4, 34 },
    { 0x92b6a4a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b6a4aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d6a4af, 78 },
    { 0x19c6a4b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a36a4bc, 23 },
    { 0x7b76a4bd, 98 },
    { 0, 0 },
    { 0xbfa6a4bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f6a4c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d16a4c7, 67 },
    { 0, 0 },
    { 0xb6e6a4c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d6a4d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd36a4d9, 23 },
    { 0xf9d6a4da, 90 },
    { 0x2b86a4d9, 40 },
    { 0x17d6a4dc, 98 },
    { 0xbab6a4dd, 90 },
    { 0x8a6a4dc, 90 },
    { 0x1d76a4dc, 87 },
    { 0xe0b6a4da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f6a4e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7136a4e6, 78 },
    { 0x3226a4e7, 43 },
    { 0x8346a4e8, 33 },
    { 0x5836a4e9, 95 },
    { 0x5b26a4ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a46a4ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b36a4f1, 90 },
    { 0xe256a4f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6a4f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d6a4fb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c86a518, 23 },
    { 0, 0 },
    { 0x75d6a51a, 90 },
    { 0xd5a6a51a, 91 },
    { 0, 0 },
    { 0xfe06a51d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a6a522, 33 },
    { 0x4916a523, 93 },
    { 0x7c76a524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6a527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb6a52d, 90 },
    { 0xf616a52e, 96 },
    { 0x5a06a52f, 61 },
    { 0x71c6a530, 90 },
    { 0x39a6a52f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f66a537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab56a53d, 90 },
    { 0, 0 },
    { 0xa2b6a53f, 98 },
    { 0, 0 },
    { 0x28d6a541, 87 },
    { 0, 0 },
    { 0xa9b6a543, 33 },
    { 0xea66a544, 76 },
    { 0x37c6a545, 93 },
    { 0x7356a543, 90 },
    { 0x4ff6a544, 34 },
    { 0xf956a548, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f6a553, 76 },
    { 0xd4b6a554, 87 },
    { 0x386a554, 68 },
    { 0x1b66a556, 46 },
    { 0xbbd6a557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf6a55b, 98 },
    { 0x54b6a55c, 61 },
    { 0x1e36a55d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe696a562, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf96a566, 33 },
    { 0xc256a566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266a56c, 33 },
    { 0x4246a56d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9106a570, 76 },
    { 0xac06a571, 93 },
    { 0xb796a570, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6a575, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456a578, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed6a57e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406a582, 23 },
    { 0x2bb6a582, 100 },
    { 0xdcf6a584, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed6a592, 85 },
    { 0xd8a6a592, 90 },
    { 0x4b56a594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c6a598, 100 },
    { 0x8846a598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b66a59c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d06a5a2, 33 },
    { 0x98f6a5a3, 87 },
    { 0, 0 },
    { 0x4dc6a5a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a86a5a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec6a5ac, 88 },
    { 0xaf76a5ad, 90 },
    { 0x2636a5ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f6a5b9, 43 },
    { 0x4bc6a5ba, 61 },
    { 0x6416a5ba, 90 },
    { 0, 0 },
    { 0x4e16a5bd, 95 },
    { 0, 0 },
    { 0xa396a5bf, 90 },
    { 0x1276a5c0, 56 },
    { 0x8656a5c1, 98 },
    { 0x6326a5bf, 98 },
    { 0x4266a5c0, 76 },
    { 0x8c16a5bf, 61 },
    { 0x66a6a5c1, 33 },
    { 0, 0 },
    { 0x4436a5c7, 67 },
    { 0x2706a5c8, 43 },
    { 0x29e6a5c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f6a5cf, 33 },
    { 0x3da6a5d0, 23 },
    { 0x43f6a5d1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x296a5d4, 33 },
    { 0x1006a5d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8056a5da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a6a5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf6a5e2, 23 },
    { 0x2686a5e2, 76 },
    { 0x7696a5e4, 23 },
    { 0xaf46a5e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a6a5e9, 88 },
    { 0x7296a5ea, 43 },
    { 0xfe86a5eb, 95 },
    { 0x3346a5ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1836a5ef, 33 },
    { 0, 0 },
    { 0x91e6a5f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a6a5f5, 33 },
    { 0, 0 },
    { 0x1cd6a5f7, 99 },
    { 0x39d6a5f8, 43 },
    { 0xefb6a5f7, 34 },
    { 0xbab6a5fa, 100 },
    { 0xfdb6a5f8, 55 },
    { 0, 0 },
    { 0x25f6a5fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd26a602, 95 },
    { 0xda16a603, 76 },
    { 0x3786a604, 33 },
    { 0, 0 },
    { 0xfe86a606, 95 },
    { 0x5896a606, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f6a60c, 76 },
    { 0xf06a60d, 23 },
    { 0x40c6a60e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096a613, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd36a617, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4516a61b, 98 },
    { 0, 0 },
    { 0xe436a61d, 61 },
    { 0, 0 },
    { 0x99d6a61f, 90 },
    { 0xd536a620, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa946a623, 98 },
    { 0x6846a624, 43 },
    { 0xbf06a624, 75 },
    { 0xc8f6a626, 52 },
    { 0x7366a624, 98 },
    { 0x4a46a628, 33 },
    { 0xfa26a624, 98 },
    { 0, 0 },
    { 0x6146a62b, 90 },
    { 0xa596a62c, 23 },
    { 0xe256a62d, 90 },
    { 0xa06a62c, 90 },
    { 0xb2c6a62f, 61 },
    { 0xbb06a62e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e6a634, 76 },
    { 0x5526a634, 95 },
    { 0x1b76a636, 91 },
    { 0xe8a6a637, 23 },
    { 0xfd46a638, 80 },
    { 0xbcc6a637, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c6a63e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6a646, 61 },
    { 0xbe46a647, 87 },
    { 0x6776a648, 46 },
    { 0, 0 },
    { 0xc66a64a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb076a64d, 76 },
    { 0, 0 },
    { 0x4a6a64f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd196a653, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a6a659, 98 },
    { 0x99d6a65a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe116a663, 76 },
    { 0, 0 },
    { 0x366a665, 33 },
    { 0x6486a665, 95 },
    { 0xda56a665, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a6a66b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046a678, 90 },
    { 0x3466a679, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee56a681, 95 },
    { 0xfac6a682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe66a689, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736a691, 90 },
    { 0x7816a692, 90 },
    { 0xd096a693, 98 },
    { 0, 0 },
    { 0x3096a695, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b6a699, 90 },
    { 0xced6a69a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec6a69d, 61 },
    { 0x5606a69e, 61 },
    { 0x26f6a69d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa6a6a7, 33 },
    { 0x6a6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a6ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae6a6ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f96a6b1, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc426a6b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1556a6ba, 61 },
    { 0x5bc6a6bb, 48 },
    { 0, 0 },
    { 0xef6a6bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e6a6c0, 33 },
    { 0x6a96a6c1, 84 },
    { 0xa336a6c2, 98 },
    { 0x2376a6c3, 95 },
    { 0, 0 },
    { 0x6f76a6c5, 76 },
    { 0, 0 },
    { 0x2246a6c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e6a6ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd76a6cd, 43 },
    { 0, 0 },
    { 0x1eb6a6cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e6a6d6, 90 },
    { 0x1c26a6d7, 66 },
    { 0x2bb6a6d8, 90 },
    { 0xdb06a6d9, 55 },
    { 0xe366a6d9, 95 },
    { 0xc166a6d7, 55 },
    { 0xb9b6a6db, 93 },
    { 0xf4b6a6db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc386a6e0, 91 },
    { 0x1826a6e1, 61 },
    { 0, 0 },
    { 0x1c86a6e3, 34 },
    { 0, 0 },
    { 0xd186a6e5, 90 },
    { 0x6016a6e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2386a6ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6a6f2, 90 },
    { 0, 0 },
    { 0x27c6a6f4, 87 },
    { 0, 0 },
    { 0x55f6a6f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad86a6fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266a701, 33 },
    { 0x33b6a701, 23 },
    { 0xeca6a701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a6a706, 90 },
    { 0x5866a706, 76 },
    { 0x1b56a708, 91 },
    { 0x8b96a706, 99 },
    { 0x7ef6a70a, 43 },
    { 0x6cc6a708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a6a70e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee86a714, 90 },
    { 0xee76a715, 76 },
    { 0x2aa6a716, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd56a719, 40 },
    { 0x9246a71a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a71d, 87 },
    { 0xc386a71e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b6a728, 90 },
    { 0, 0 },
    { 0xeb86a72a, 75 },
    { 0xe5f6a72a, 33 },
    { 0, 0 },
    { 0xce66a72d, 95 },
    { 0xc116a72e, 34 },
    { 0x7416a72e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a6a739, 33 },
    { 0, 0 },
    { 0x8e26a73b, 90 },
    { 0, 0 },
    { 0x74a6a73d, 93 },
    { 0, 0 },
    { 0x61a6a73f, 61 },
    { 0xb556a740, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2946a746, 95 },
    { 0x9006a747, 91 },
    { 0x8296a748, 40 },
    { 0x24a6a747, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b16a751, 17 },
    { 0x5f46a752, 93 },
    { 0x3cc6a753, 61 },
    { 0xdc6a753, 34 },
    { 0x5366a754, 90 },
    { 0, 0 },
    { 0xf946a757, 61 },
    { 0x31b6a758, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6276a75d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a6a762, 90 },
    { 0, 0 },
    { 0x8f76a764, 91 },
    { 0xc506a764, 23 },
    { 0xb696a766, 83 },
    { 0, 0 },
    { 0x82e6a768, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6166a779, 33 },
    { 0x21d6a77a, 78 },
    { 0, 0 },
    { 0x7a76a77c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xef96a77f, 33 },
    { 0x6eb6a780, 17 },
    { 0x1446a780, 81 },
    { 0x2ff6a780, 33 },
    { 0xe166a783, 98 },
    { 0xfed6a781, 76 },
    { 0xcaa6a785, 33 },
    { 0xafb6a786, 34 },
    { 0x6f16a787, 90 },
    { 0xd356a788, 95 },
    { 0xee6a788, 98 },
    { 0xf0b6a789, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d36a78f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc216a794, 52 },
    { 0x6e36a795, 33 },
    { 0x5206a795, 98 },
    { 0x2386a797, 61 },
    { 0x2546a798, 52 },
    { 0xece6a797, 93 },
    { 0, 0 },
    { 0x6c26a79b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc706a7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4446a7a6, 23 },
    { 0xb406a7a7, 33 },
    { 0xc2e6a7a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe626a7ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb6a7b3, 93 },
    { 0x6636a7b3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6916a7b8, 93 },
    { 0x5076a7b9, 17 },
    { 0x51c6a7ba, 90 },
    { 0xa006a7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff6a7be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab16a7c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e06a7c5, 90 },
    { 0, 0 },
    { 0x34d6a7c7, 90 },
    { 0xcdc6a7c8, 23 },
    { 0, 0 },
    { 0xf6e6a7ca, 61 },
    { 0x6006a7ca, 76 },
    { 0x5346a7cb, 93 },
    { 0x13a6a7cc, 85 },
    { 0x75f6a7ce, 33 },
    { 0xf996a7cf, 90 },
    { 0x73d6a7d0, 23 },
    { 0x2b86a7d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2386a7d6, 66 },
    { 0x30e6a7d7, 62 },
    { 0xf6a7d7, 90 },
    { 0x87c6a7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd66a7df, 93 },
    { 0x5006a7df, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8366a7e7, 61 },
    { 0x6a6a7e7, 91 },
    { 0xe876a7e9, 9 },
    { 0xcc86a7e8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xf616a7ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8976a7f1, 61 },
    { 0xab56a7f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6a806, 93 },
    { 0x54e6a806, 33 },
    { 0, 0 },
    { 0x766a809, 98 },
    { 0xa3a6a80a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe136a80d, 61 },
    { 0xc286a80e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd786a813, 33 },
    { 0xad26a814, 99 },
    { 0xf036a815, 61 },
    { 0x3db6a815, 33 },
    { 0x4826a814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc036a81d, 91 },
    { 0, 0 },
    { 0x3d46a81f, 90 },
    { 0x7186a81f, 90 },
    { 0, 0 },
    { 0x1856a822, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096a826, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e76a82c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a46a839, 23 },
    { 0, 0 },
    { 0xe206a83b, 75 },
    { 0xceb6a83b, 61 },
    { 0xe346a83d, 90 },
    { 0xae46a83e, 33 },
    { 0xc586a83e, 23 },
    { 0xe586a83d, 23 },
    { 0x9d56a841, 33 },
    { 0x3c66a83d, 61 },
    { 0x5836a843, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb6a846, 56 },
    { 0, 0 },
    { 0xfb96a848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b56a850, 88 },
    { 0, 0 },
    { 0xfb26a852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f6a855, 77 },
    { 0, 0 },
    { 0x76b6a857, 85 },
    { 0xe826a858, 88 },
    { 0x8336a859, 33 },
    { 0xf226a85a, 99 },
    { 0xb896a858, 98 },
    { 0xff56a857, 33 },
    { 0xa2f6a85c, 43 },
    { 0xa256a85e, 95 },
    { 0x28a6a85f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e06a866, 34 },
    { 0, 0 },
    { 0x4b56a868, 23 },
    { 0, 0 },
    { 0xb8f6a86a, 61 },
    { 0xbd6a86b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd286a870, 43 },
    { 0xe396a871, 55 },
    { 0x87c6a872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a06a87d, 81 },
    { 0xf9a6a87e, 55 },
    { 0x4856a87f, 85 },
    { 0xd0a6a880, 34 },
    { 0xfd86a881, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb346a884, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e6a88c, 76 },
    { 0x6d6a88c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c6a890, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e6a899, 38 },
    { 0, 0 },
    { 0xaf06a89b, 98 },
    { 0, 0 },
    { 0x32a6a89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa206a8a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa06a8ac, 52 },
    { 0, 0 },
    { 0x4446a8ae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db6a8b1, 93 },
    { 0x79a6a8b1, 90 },
    { 0, 0 },
    { 0x6876a8b4, 80 },
    { 0x61c6a8b4, 55 },
    { 0, 0 },
    { 0xe726a8b7, 98 },
    { 0xed86a8b8, 55 },
    { 0x8b96a8b9, 90 },
    { 0xf056a8ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb126a8c0, 23 },
    { 0xb626a8c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8606a8c6, 33 },
    { 0xf9b6a8c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c6a8ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c6a8cd, 90 },
    { 0x1cb6a8cd, 81 },
    { 0, 0 },
    { 0xd36a8d0, 43 },
    { 0x5176a8d1, 95 },
    { 0, 0 },
    { 0xdd36a8d3, 77 },
    { 0, 0 },
    { 0x6bd6a8d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa86a8d8, 67 },
    { 0x2376a8d9, 85 },
    { 0x7116a8d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f76a8dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6a8e4, 93 },
    { 0xce56a8e5, 90 },
    { 0x6246a8e6, 80 },
    { 0xd486a8e6, 55 },
    { 0xdf16a8e8, 84 },
    { 0x7516a8e9, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb696a8ef, 61 },
    { 0xe386a8ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd286a8f3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df6a8fa, 85 },
    { 0xf226a8fb, 15 },
    { 0x3cb6a8fc, 33 },
    { 0x256a8fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e86a900, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6a907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146a90f, 88 },
    { 0xf896a910, 98 },
    { 0x816a911, 95 },
    { 0xc4c6a912, 93 },
    { 0, 0 },
    { 0x8d26a914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc906a91c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3726a91f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb86a925, 84 },
    { 0x5826a926, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f6a929, 66 },
    { 0x5c96a929, 66 },
    { 0xd626a92b, 91 },
    { 0x4496a92c, 69 },
    { 0xa8b6a92d, 98 },
    { 0x4156a92e, 98 },
    { 0xcc86a92f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf846a932, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde6a939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316a93c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76a942, 43 },
    { 0, 0 },
    { 0xfc56a944, 23 },
    { 0x6386a944, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd66a948, 87 },
    { 0xfeb6a948, 33 },
    { 0, 0 },
    { 0x32e6a94b, 61 },
    { 0x3cc6a94b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee6a950, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b6a953, 33 },
    { 0, 0 },
    { 0xb1b6a955, 33 },
    { 0x9836a956, 90 },
    { 0x49d6a955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed56a960, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c6a963, 33 },
    { 0, 0 },
    { 0x1cd6a965, 33 },
    { 0x7076a965, 90 },
    { 0, 0 },
    { 0x1c76a968, 95 },
    { 0x8246a969, 99 },
    { 0xebb6a96a, 85 },
    { 0x1fc6a968, 90 },
    { 0x6076a96a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5006a972, 98 },
    { 0, 0 },
    { 0xa8b6a974, 23 },
    { 0xd276a975, 61 },
    { 0x9ab6a976, 95 },
    { 0, 0 },
    { 0xf0a6a978, 100 },
    { 0xbee6a979, 78 },
    { 0x1c6a979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7446a980, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e6a983, 33 },
    { 0, 0 },
    { 0x38c6a985, 88 },
    { 0x3b76a986, 33 },
    { 0xe446a987, 33 },
    { 0xbee6a988, 55 },
    { 0xb7e6a989, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a6a98e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb126a992, 43 },
    { 0, 0 },
    { 0x1616a994, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26a997, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb156a99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe086a9a1, 98 },
    { 0x7ca6a9a2, 98 },
    { 0x1516a9a3, 23 },
    { 0x1266a9a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x526a9a7, 55 },
    { 0xe306a9a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba6a9ac, 46 },
    { 0xc076a9ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec6a9b0, 49 },
    { 0, 0 },
    { 0x1c06a9b2, 33 },
    { 0xd7b6a9b3, 98 },
    { 0x4fd6a9b4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c46a9ba, 43 },
    { 0, 0 },
    { 0xedf6a9bc, 98 },
    { 0x70e6a9bd, 90 },
    { 0x6bf6a9be, 33 },
    { 0x9956a9be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f6a9c4, 99 },
    { 0, 0 },
    { 0x1b6a9c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c6a9cd, 33 },
    { 0x6036a9cd, 46 },
    { 0xb786a9cf, 23 },
    { 0x6476a9cd, 91 },
    { 0, 0 },
    { 0x3b46a9d2, 61 },
    { 0x62e6a9d3, 93 },
    { 0x5206a9d3, 33 },
    { 0, 0 },
    { 0x18c6a9d6, 61 },
    { 0xd8a6a9d7, 34 },
    { 0xc916a9d8, 93 },
    { 0xf9e6a9d9, 48 },
    { 0xf5c6a9da, 78 },
    { 0x51e6a9d9, 90 },
    { 0x7126a9db, 95 },
    { 0, 0 },
    { 0x2156a9de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe106a9e2, 98 },
    { 0, 0 },
    { 0xda16a9e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6a9e7, 34 },
    { 0, 0 },
    { 0x8bd6a9e9, 98 },
    { 0xbcd6a9ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc416a9f0, 61 },
    { 0, 0 },
    { 0xe206a9f2, 55 },
    { 0x6106a9f3, 95 },
    { 0x4ba6a9f3, 98 },
    { 0x5bf6a9f5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e6a9fa, 61 },
    { 0xc9e6a9fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xba16a9fe, 76 },
    { 0x826a9fe, 95 },
    { 0xffe6aa00, 53 },
    { 0x9a66aa00, 55 },
    { 0, 0 },
    { 0x3166aa03, 43 },
    { 0, 0 },
    { 0x5386aa05, 17 },
    { 0, 0 },
    { 0xafe6aa07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb36aa0a, 61 },
    { 0xdc36aa0b, 61 },
    { 0x9126aa0c, 33 },
    { 0, 0 },
    { 0xe3f6aa0e, 61 },
    { 0x2f56aa0f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3786aa13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb6aa16, 33 },
    { 0x6ad6aa17, 55 },
    { 0, 0 },
    { 0x3316aa19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6aa26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516aa2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2696aa30, 61 },
    { 0x4736aa31, 77 },
    { 0x286aa31, 66 },
    { 0x82a6aa33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee46aa38, 90 },
    { 0, 0 },
    { 0x78e6aa3a, 33 },
    { 0xbce6aa3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc6aa3f, 91 },
    { 0xd2c6aa3f, 95 },
    { 0xe4b6aa41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d6aa44, 33 },
    { 0, 0 },
    { 0xf016aa46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1206aa49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2396aa4e, 91 },
    { 0x9a36aa4e, 34 },
    { 0x2c6aa4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2996aa54, 95 },
    { 0x67b6aa55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b36aa58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6aa5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486aa65, 99 },
    { 0xa8b6aa66, 25 },
    { 0xef16aa67, 71 },
    { 0x3e16aa68, 55 },
    { 0, 0 },
    { 0x826aa6a, 85 },
    { 0, 0 },
    { 0x9106aa6c, 61 },
    { 0x64f6aa6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e6aa74, 61 },
    { 0, 0 },
    { 0x7dc6aa76, 81 },
    { 0xac16aa77, 81 },
    { 0x1596aa76, 61 },
    { 0x6886aa79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe6aa7c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa396aa82, 98 },
    { 0xa346aa83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1326aa8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7276aa8d, 52 },
    { 0x70f6aa8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf416aa93, 90 },
    { 0x6326aa94, 38 },
    { 0x4966aa93, 55 },
    { 0x7786aa94, 95 },
    { 0xbc56aa95, 99 },
    { 0x5116aa98, 33 },
    { 0x966aa98, 23 },
    { 0x68a6aa9a, 99 },
    { 0x2146aa9b, 90 },
    { 0x1246aa96, 55 },
    { 0x1996aa9d, 23 },
    { 0x5476aa9e, 55 },
    { 0x2c96aa97, 98 },
    { 0x94a6aaa0, 52 },
    { 0xda6aaa0, 61 },
    { 0x7966aaa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a6aaa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf6aaae, 98 },
    { 0x9c86aaaf, 61 },
    { 0x5246aab0, 95 },
    { 0xedd6aab1, 94 },
    { 0xfa26aab2, 33 },
    { 0x8ac6aab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f96aab9, 34 },
    { 0xe876aab9, 93 },
    { 0xde76aabb, 93 },
    { 0, 0 },
    { 0x576aabd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c6aac1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b6aac7, 61 },
    { 0x3db6aac8, 90 },
    { 0x3a96aac9, 100 },
    { 0, 0 },
    { 0x7a26aacb, 93 },
    { 0, 0 },
    { 0xf7a6aacd, 61 },
    { 0xa86aacd, 85 },
    { 0xa596aacf, 61 },
    { 0x12c6aad0, 90 },
    { 0x7516aacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf6aad4, 99 },
    { 0, 0 },
    { 0xffd6aad6, 98 },
    { 0x7c86aad7, 61 },
    { 0x7e66aad8, 95 },
    { 0xc516aad9, 81 },
    { 0x3ea6aada, 95 },
    { 0x6236aada, 84 },
    { 0x6366aad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e86aae0, 98 },
    { 0, 0 },
    { 0xf1f6aae2, 43 },
    { 0xb806aae3, 81 },
    { 0xc6c6aae3, 33 },
    { 0, 0 },
    { 0xe1b6aae6, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba6aae9, 61 },
    { 0x90b6aae9, 87 },
    { 0x8b26aaeb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d6aaee, 61 },
    { 0xf896aaef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3326aaf2, 78 },
    { 0x6c66aaf3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe966aaff, 98 },
    { 0xec26ab00, 95 },
    { 0xe446ab01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726ab07, 23 },
    { 0xa5c6ab07, 84 },
    { 0xdb06ab09, 98 },
    { 0xb5f6ab08, 93 },
    { 0xb296ab0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf6ab0f, 81 },
    { 0x2676ab10, 33 },
    { 0, 0 },
    { 0xbc26ab12, 23 },
    { 0, 0 },
    { 0x5c56ab14, 90 },
    { 0xd896ab15, 52 },
    { 0x6ab16, 93 },
    { 0x6586ab16, 75 },
    { 0xcdd6ab18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae6ab1c, 95 },
    { 0x6716ab1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8396ab20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456ab24, 34 },
    { 0xdcc6ab24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056ab2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce66ab2d, 2 },
    { 0, 0 },
    { 0xf4f6ab2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e26ab32, 34 },
    { 0, 0 },
    { 0x3ce6ab34, 33 },
    { 0x4f6ab34, 33 },
    { 0x47f6ab34, 33 },
    { 0xd756ab37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f86ab3c, 76 },
    { 0x1d86ab3d, 61 },
    { 0x32c6ab3e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056ab41, 61 },
    { 0xdbd6ab42, 17 },
    { 0x6ba6ab43, 95 },
    { 0x53c6ab43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f66ab4c, 98 },
    { 0xe186ab4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6186ab57, 98 },
    { 0x6a86ab57, 98 },
    { 0x6f76ab57, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6ab5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586ab66, 23 },
    { 0, 0 },
    { 0xe4c6ab68, 33 },
    { 0, 0 },
    { 0x5886ab6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276ab6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb516ab72, 61 },
    { 0, 0 },
    { 0x7376ab74, 55 },
    { 0x2bb6ab75, 87 },
    { 0xe316ab76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa706ab79, 92 },
    { 0, 0 },
    { 0xb996ab7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466ab80, 23 },
    { 0x9706ab80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc596ab88, 84 },
    { 0x1f26ab89, 33 },
    { 0, 0 },
    { 0x70d6ab8b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd316ab8e, 53 },
    { 0xceb6ab8e, 61 },
    { 0x2556ab8f, 95 },
    { 0x6236ab91, 55 },
    { 0, 0 },
    { 0xf66ab93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1206ab97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2936ab9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5886aba2, 77 },
    { 0x76f6aba2, 23 },
    { 0x36f6aba2, 23 },
    { 0x136aba3, 23 },
    { 0xfd06aba6, 61 },
    { 0xc726aba7, 77 },
    { 0xb2b6aba7, 95 },
    { 0xddc6aba2, 95 },
    { 0xd406abaa, 95 },
    { 0x4146abab, 33 },
    { 0x5c76abab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce46abaf, 95 },
    { 0, 0 },
    { 0x4c36abb1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6abb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a6abc1, 76 },
    { 0xa2f6abc2, 100 },
    { 0xc8f6abc3, 90 },
    { 0x7ab6abc2, 83 },
    { 0x8cb6abc5, 76 },
    { 0xa206abc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff86abcc, 90 },
    { 0x4536abcd, 95 },
    { 0x7f66abce, 87 },
    { 0xde76abcf, 33 },
    { 0xa2b6abcc, 61 },
    { 0x97c6abcc, 99 },
    { 0x9146abd2, 95 },
    { 0x3916abd2, 68 },
    { 0xc5a6abd2, 98 },
    { 0xe356abce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6abd9, 46 },
    { 0x71f6abda, 66 },
    { 0x7d6abd9, 33 },
    { 0xf726abd9, 33 },
    { 0xfac6abd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6abe1, 33 },
    { 0, 0 },
    { 0x4106abe3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1016abe6, 61 },
    { 0x63f6abe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2296abec, 34 },
    { 0x16c6abed, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6abf9, 55 },
    { 0, 0 },
    { 0xcc76abfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa976ac00, 95 },
    { 0, 0 },
    { 0x43e6ac02, 98 },
    { 0x2ea6ac02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd6ac07, 34 },
    { 0, 0 },
    { 0x9716ac09, 78 },
    { 0xd886ac0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5656ac0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa376ac12, 76 },
    { 0x8b6ac12, 75 },
    { 0, 0 },
    { 0x1976ac15, 90 },
    { 0x3a46ac15, 23 },
    { 0x1006ac15, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866ac1d, 90 },
    { 0xfeb6ac1e, 33 },
    { 0xfd56ac1e, 90 },
    { 0x97d6ac20, 75 },
    { 0xb056ac20, 98 },
    { 0x35c6ac21, 46 },
    { 0x696ac20, 85 },
    { 0xa8e6ac1f, 23 },
    { 0xf706ac20, 23 },
    { 0xc996ac1d, 33 },
    { 0x406ac20, 93 },
    { 0x9ca6ac28, 61 },
    { 0x1c66ac29, 90 },
    { 0x1316ac29, 93 },
    { 0x3c66ac2b, 88 },
    { 0x9846ac2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a6ac2f, 100 },
    { 0xb8a6ac30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6ac33, 34 },
    { 0xb266ac34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc736ac37, 98 },
    { 0x25b6ac38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2936ac43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed36ac47, 43 },
    { 0xa536ac47, 23 },
    { 0xd036ac49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb576ac51, 75 },
    { 0, 0 },
    { 0x5766ac53, 61 },
    { 0x9586ac53, 61 },
    { 0x88a6ac55, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb6ac58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6ac5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb726ac65, 98 },
    { 0x3be6ac66, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56ac6a, 76 },
    { 0x1056ac6a, 85 },
    { 0x7b66ac6c, 90 },
    { 0xfc96ac6c, 78 },
    { 0xedc6ac6e, 76 },
    { 0xe0b6ac6f, 98 },
    { 0x3f6ac70, 87 },
    { 0xadd6ac71, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ac74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf56ac7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7606ac7e, 33 },
    { 0x4b86ac7f, 90 },
    { 0xfca6ac7e, 95 },
    { 0, 0 },
    { 0xe8f6ac82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f26ac88, 76 },
    { 0, 0 },
    { 0xddd6ac8a, 90 },
    { 0, 0 },
    { 0xf396ac8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9176ac90, 33 },
    { 0, 0 },
    { 0xd2c6ac92, 95 },
    { 0x34d6ac93, 90 },
    { 0x79d6ac94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6ac98, 43 },
    { 0, 0 },
    { 0x8016ac9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac6ac9f, 90 },
    { 0x5e6aca0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b66aca5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e36acac, 67 },
    { 0xa606acad, 34 },
    { 0xc736acad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c26acb2, 84 },
    { 0xd786acb2, 23 },
    { 0xed56acb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x916acb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746acbc, 61 },
    { 0, 0 },
    { 0x4d56acbe, 95 },
    { 0x3066acbf, 90 },
    { 0, 0 },
    { 0xf8b6acc1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf6acc5, 85 },
    { 0xd176acc5, 33 },
    { 0xe186acc7, 61 },
    { 0xfef6acc7, 55 },
    { 0, 0 },
    { 0x556acca, 52 },
    { 0x7f96accb, 95 },
    { 0xb586accb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6acd0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5376acd5, 90 },
    { 0, 0 },
    { 0xc226acd7, 64 },
    { 0x19d6acd8, 34 },
    { 0, 0 },
    { 0x2796acda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2206acde, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a6ace2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7726ace5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4726ace8, 98 },
    { 0x6a66ace8, 91 },
    { 0x1c56ace8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb986acef, 61 },
    { 0x3a06acef, 90 },
    { 0x3b6acf1, 23 },
    { 0x6cb6acf1, 95 },
    { 0x53e6acf1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8216acf7, 61 },
    { 0x1586acf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d6acfc, 99 },
    { 0, 0 },
    { 0xc496acfe, 61 },
    { 0x3146acff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2756ad04, 67 },
    { 0x99c6ad04, 90 },
    { 0xa7f6ad06, 23 },
    { 0x5f26ad04, 95 },
    { 0xe3c6ad08, 23 },
    { 0x8f56ad08, 34 },
    { 0xb5f6ad0a, 64 },
    { 0x2006ad0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c6ad0f, 90 },
    { 0xd4a6ad0f, 23 },
    { 0xabb6ad11, 23 },
    { 0x3cf6ad11, 61 },
    { 0xd2f6ad13, 45 },
    { 0, 0 },
    { 0x44f6ad15, 91 },
    { 0xc2b6ad16, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae6ad19, 61 },
    { 0x4a36ad1a, 95 },
    { 0x62e6ad1b, 76 },
    { 0x2976ad1a, 61 },
    { 0x7c66ad1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5146ad20, 95 },
    { 0, 0 },
    { 0x8626ad22, 77 },
    { 0x2796ad23, 76 },
    { 0x2cb6ad23, 33 },
    { 0xe1e6ad23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6166ad28, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e6ad2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c6ad34, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc6ad38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7746ad3e, 23 },
    { 0, 0 },
    { 0xfad6ad40, 61 },
    { 0x9006ad40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496ad45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a6ad4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466ad4d, 67 },
    { 0xa746ad4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9426ad57, 85 },
    { 0, 0 },
    { 0x7ad6ad59, 95 },
    { 0x72c6ad5a, 55 },
    { 0x2cd6ad5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2556ad5e, 95 },
    { 0xcf36ad5e, 46 },
    { 0x2d26ad60, 99 },
    { 0x4276ad61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd426ad65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8886ad6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3986ad72, 98 },
    { 0xe3a6ad73, 61 },
    { 0x1146ad74, 95 },
    { 0x5036ad74, 90 },
    { 0x6946ad75, 92 },
    { 0x4ac6ad74, 99 },
    { 0xfa66ad77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b26ad7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8306ad87, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116ad8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ad97, 95 },
    { 0xf126ad98, 95 },
    { 0x5116ad99, 23 },
    { 0x5ff6ad9a, 66 },
    { 0xb806ad9b, 33 },
    { 0x38d6ad9c, 33 },
    { 0xf3f6ad9d, 77 },
    { 0x9dc6ad9d, 33 },
    { 0, 0 },
    { 0xa9c6ada0, 61 },
    { 0x5d86ada1, 66 },
    { 0x3486ada2, 61 },
    { 0xe386ada3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b6ada6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd186adab, 95 },
    { 0xff36adab, 61 },
    { 0x10c6adad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c16adb5, 46 },
    { 0xccd6adb5, 98 },
    { 0x7b46adb5, 93 },
    { 0xfec6adb8, 84 },
    { 0x9a46adb9, 68 },
    { 0x4d66adba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1116adc0, 93 },
    { 0x2886adc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c6adc8, 61 },
    { 0, 0 },
    { 0x4476adca, 24 },
    { 0xb886adcb, 61 },
    { 0x9016adcb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda6adcf, 23 },
    { 0x84b6adcf, 33 },
    { 0xfa26add1, 68 },
    { 0, 0 },
    { 0xf406add3, 90 },
    { 0, 0 },
    { 0x3ab6add5, 77 },
    { 0, 0 },
    { 0xb2c6add7, 66 },
    { 0, 0 },
    { 0x4b06add9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1146ade0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016ade3, 61 },
    { 0, 0 },
    { 0xcc96ade5, 61 },
    { 0x3e56ade5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5046ade9, 90 },
    { 0xd9d6adea, 90 },
    { 0x3706ade9, 40 },
    { 0xc806ade9, 23 },
    { 0xd6c6aded, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f6adf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3176adf8, 95 },
    { 0xdbd6adf9, 90 },
    { 0x7996adfa, 23 },
    { 0, 0 },
    { 0xbdb6adfc, 42 },
    { 0xd366adfc, 85 },
    { 0xa206adfd, 98 },
    { 0xfdb6adff, 90 },
    { 0x99d6ae00, 90 },
    { 0xcc26ae01, 90 },
    { 0x6286ae02, 96 },
    { 0x2f66ae02, 23 },
    { 0, 0 },
    { 0xb7d6ae05, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976ae0a, 90 },
    { 0x2d06ae0b, 46 },
    { 0, 0 },
    { 0xe7f6ae0d, 90 },
    { 0xafe6ae0d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b46ae17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3936ae1a, 88 },
    { 0xaad6ae1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a6ae20, 81 },
    { 0x9e56ae20, 85 },
    { 0xe8a6ae22, 78 },
    { 0x7d66ae23, 91 },
    { 0xf216ae21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5676ae28, 76 },
    { 0x6c76ae28, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6126ae2c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f66ae30, 91 },
    { 0xd076ae31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7506ae35, 55 },
    { 0xeac6ae36, 90 },
    { 0x72a6ae37, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be6ae3c, 84 },
    { 0, 0 },
    { 0xdc66ae3e, 33 },
    { 0xb256ae3e, 33 },
    { 0x48c6ae40, 61 },
    { 0x42f6ae41, 68 },
    { 0x1bd6ae42, 53 },
    { 0xc4c6ae43, 85 },
    { 0x9c26ae44, 55 },
    { 0xdfa6ae45, 52 },
    { 0xce16ae46, 61 },
    { 0x4a46ae47, 95 },
    { 0x1c56ae45, 100 },
    { 0x8766ae49, 98 },
    { 0xa816ae4a, 77 },
    { 0xde06ae4b, 61 },
    { 0x9996ae4b, 33 },
    { 0x5526ae49, 61 },
    { 0xe396ae4e, 66 },
    { 0x9476ae46, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8106ae52, 90 },
    { 0, 0 },
    { 0xfc96ae54, 34 },
    { 0x6cd6ae54, 11 },
    { 0xe476ae54, 65 },
    { 0xd436ae55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af6ae5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9226ae5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf6ae62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3526ae6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8646ae6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc6ae71, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e6ae79, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ae80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb766ae84, 98 },
    { 0x6406ae84, 85 },
    { 0x1886ae86, 76 },
    { 0x2d86ae87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb86ae8f, 99 },
    { 0x53c6ae90, 23 },
    { 0x7236ae91, 90 },
    { 0x87a6ae92, 96 },
    { 0, 0 },
    { 0xbcd6ae94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb226ae99, 43 },
    { 0xbfa6ae9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca6ae9f, 76 },
    { 0x7d36ae9f, 40 },
    { 0, 0 },
    { 0x6bd6aea2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4426aeb2, 77 },
    { 0x3e26aeb3, 61 },
    { 0x6976aeb4, 61 },
    { 0, 0 },
    { 0x5f46aeb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e66aeba, 100 },
    { 0x4486aebb, 23 },
    { 0, 0 },
    { 0xfa06aebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c16aec0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc6aec6, 93 },
    { 0x7d36aec6, 75 },
    { 0x4606aec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x716aecb, 52 },
    { 0x8e96aecc, 95 },
    { 0x1f6aecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa6aed1, 81 },
    { 0x6dc6aed2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9056aed5, 98 },
    { 0x9526aed6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c96aedb, 98 },
    { 0x8456aedb, 95 },
    { 0x2876aedd, 76 },
    { 0, 0 },
    { 0x116aedf, 12 },
    { 0, 0 },
    { 0x2e86aee1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e6aee6, 98 },
    { 0xcf46aee6, 90 },
    { 0, 0 },
    { 0xd546aee9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3386aeef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d56aef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1146aef8, 85 },
    { 0, 0 },
    { 0xc9a6aefa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6aefd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9306af00, 96 },
    { 0x556af01, 95 },
    { 0x2ef6af02, 98 },
    { 0x4396af02, 23 },
    { 0x346af00, 95 },
    { 0, 0 },
    { 0xb7b6af06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46af0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf986af0e, 55 },
    { 0, 0 },
    { 0x78c6af10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d6af14, 90 },
    { 0x9836af14, 43 },
    { 0x8a6af16, 43 },
    { 0, 0 },
    { 0xe876af18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be6af21, 34 },
    { 0, 0 },
    { 0x95b6af23, 95 },
    { 0x2046af24, 90 },
    { 0, 0 },
    { 0x8906af26, 95 },
    { 0x1e6af27, 78 },
    { 0x4756af28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5836af2e, 56 },
    { 0x596af2f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd6af34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9836af3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4866af4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd876af5e, 33 },
    { 0xd066af5f, 61 },
    { 0x906af5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa46af66, 90 },
    { 0xfcb6af66, 33 },
    { 0xd3a6af68, 66 },
    { 0xc4f6af69, 23 },
    { 0x5836af6a, 90 },
    { 0x6b26af67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd6af73, 98 },
    { 0x9f76af74, 40 },
    { 0x7bf6af74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336af7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc36af80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a36af83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6146af86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d6af89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e6af8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc36af94, 61 },
    { 0x1a16af94, 99 },
    { 0x45d6af96, 61 },
    { 0xa596af96, 90 },
    { 0x3e96af97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066af9b, 53 },
    { 0x7026af9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6afa0, 97 },
    { 0xe1a6afa1, 61 },
    { 0x4446afa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb556afa5, 34 },
    { 0x13d6afa6, 90 },
    { 0xcb16afa6, 52 },
    { 0x3476afa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d36afad, 56 },
    { 0x3c06afad, 61 },
    { 0x1486afae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db6afb2, 43 },
    { 0x5d06afb3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a6afb6, 23 },
    { 0xbad6afb7, 17 },
    { 0xd366afb8, 23 },
    { 0x2d46afb6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5046afc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa696afd3, 33 },
    { 0x4ba6afd4, 66 },
    { 0, 0 },
    { 0x3156afd6, 93 },
    { 0x92c6afd7, 85 },
    { 0x4db6afd7, 90 },
    { 0x4cf6afd9, 17 },
    { 0, 0 },
    { 0x4746afdb, 90 },
    { 0xae06afdb, 98 },
    { 0x9df6afdb, 61 },
    { 0, 0 },
    { 0x7c56afdf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b06afe3, 23 },
    { 0xbb26afe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b26afe9, 78 },
    { 0x336afea, 68 },
    { 0x5386afea, 95 },
    { 0x4a46afec, 17 },
    { 0xaa06afed, 66 },
    { 0xd226afec, 33 },
    { 0, 0 },
    { 0xcbc6aff0, 90 },
    { 0, 0 },
    { 0xe1a6aff2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b56aff7, 61 },
    { 0, 0 },
    { 0x1576aff9, 23 },
    { 0, 0 },
    { 0x5926affb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f6b003, 55 },
    { 0x4ef6b003, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4786b00a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d6b00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176b012, 95 },
    { 0xeee6b013, 52 },
    { 0x44a6b014, 52 },
    { 0xc6e6b015, 61 },
    { 0, 0 },
    { 0x2fc6b017, 23 },
    { 0xd6b018, 90 },
    { 0, 0 },
    { 0x21f6b01a, 90 },
    { 0, 0 },
    { 0xdf06b01c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf6b026, 95 },
    { 0x85b6b026, 87 },
    { 0x5256b027, 75 },
    { 0x5ed6b029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a6b02c, 33 },
    { 0x9fc6b02d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d6b030, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb366b034, 91 },
    { 0x92a6b035, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566b039, 55 },
    { 0xdd6b03a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b6b03e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a66b044, 96 },
    { 0x55b6b045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e96b049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5156b04d, 91 },
    { 0x2716b04d, 33 },
    { 0, 0 },
    { 0x9ff6b050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c6b060, 90 },
    { 0, 0 },
    { 0x7b26b062, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6b065, 55 },
    { 0x5e86b066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456b06b, 33 },
    { 0xe0e6b06b, 90 },
    { 0, 0 },
    { 0x5d46b06e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff6b072, 85 },
    { 0x56b6b073, 23 },
    { 0xde16b072, 95 },
    { 0xd166b075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126b078, 33 },
    { 0x89e6b079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb096b07d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b081, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9436b08a, 23 },
    { 0, 0 },
    { 0x8816b08c, 90 },
    { 0xe436b08c, 77 },
    { 0xe276b08e, 93 },
    { 0, 0 },
    { 0xb816b090, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4186b094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6b099, 17 },
    { 0xa526b09a, 17 },
    { 0x2826b09b, 67 },
    { 0x79b6b09c, 76 },
    { 0xd7c6b09d, 93 },
    { 0xef96b09e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc16b0a5, 76 },
    { 0x6986b0a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf386b0af, 90 },
    { 0xfc96b0b0, 98 },
    { 0x3726b0af, 75 },
    { 0xccb6b0b2, 33 },
    { 0xf576b0b3, 34 },
    { 0xa806b0b2, 46 },
    { 0, 0 },
    { 0x7b36b0b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe506b0bf, 100 },
    { 0xb9f6b0c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa066b0c6, 95 },
    { 0xdbc6b0c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e6b0ca, 84 },
    { 0x4106b0cb, 66 },
    { 0x7516b0cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff6b0d2, 42 },
    { 0xa2d6b0d2, 23 },
    { 0x1016b0d3, 93 },
    { 0, 0 },
    { 0x95c6b0d6, 33 },
    { 0x2056b0d7, 95 },
    { 0xada6b0d6, 84 },
    { 0xa9a6b0d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a6b0de, 33 },
    { 0x146b0df, 33 },
    { 0xcb16b0e0, 98 },
    { 0xf1b6b0e1, 43 },
    { 0x1ee6b0e1, 33 },
    { 0xdf46b0e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6b0ea, 90 },
    { 0xff6b0eb, 33 },
    { 0xd936b0ec, 90 },
    { 0x20c6b0ec, 91 },
    { 0xbfe6b0ea, 99 },
    { 0x67e6b0ee, 98 },
    { 0, 0 },
    { 0x8e56b0f1, 61 },
    { 0xef66b0f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6566b0f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f8, 100 },
    { 0x43a6b0f9, 33 },
    { 0, 0 },
    { 0xc9d6b0fb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa356b0fe, 37 },
    { 0, 0 },
    { 0xb3e6b100, 81 },
    { 0xbdc6b100, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b6b104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8466b107, 52 },
    { 0x6c26b108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf6b10e, 23 },
    { 0, 0 },
    { 0x3316b110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9966b115, 23 },
    { 0, 0 },
    { 0xdb96b117, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb6b11d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4396b120, 90 },
    { 0x2786b121, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106b127, 33 },
    { 0x2c86b127, 91 },
    { 0, 0 },
    { 0x5ed6b12a, 33 },
    { 0x6a86b12b, 61 },
    { 0, 0 },
    { 0x9236b12d, 91 },
    { 0x80b6b12e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa6b131, 23 },
    { 0, 0 },
    { 0xdc96b133, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb286b136, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f6b13f, 41 },
    { 0x9746b140, 80 },
    { 0x7b96b141, 61 },
    { 0x2676b142, 65 },
    { 0x5656b140, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c6b149, 76 },
    { 0x93e6b14a, 91 },
    { 0x4686b14b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca6b14e, 93 },
    { 0x7916b14f, 90 },
    { 0xcd06b14f, 90 },
    { 0x2946b14f, 66 },
    { 0x5726b14f, 95 },
    { 0xbb46b153, 66 },
    { 0xa286b153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc6b159, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc376b15c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f6b15f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3766b162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6b16c, 99 },
    { 0x49a6b16d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd006b174, 95 },
    { 0x6e76b174, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5876b178, 76 },
    { 0x1486b179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5956b17e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6b185, 33 },
    { 0xe106b186, 75 },
    { 0, 0 },
    { 0xd1a6b188, 76 },
    { 0, 0 },
    { 0x3156b18a, 33 },
    { 0x8256b18a, 23 },
    { 0x1ef6b18c, 91 },
    { 0, 0 },
    { 0x8166b18e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f66b192, 61 },
    { 0x1f76b193, 91 },
    { 0xf7b6b192, 46 },
    { 0x1106b195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd6b199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x106b19c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec6b19f, 61 },
    { 0, 0 },
    { 0x7646b1a1, 61 },
    { 0x5086b1a2, 81 },
    { 0xe76b1a3, 33 },
    { 0xaeb6b1a4, 90 },
    { 0xfb26b1a5, 34 },
    { 0xbb36b1a6, 43 },
    { 0xe8f6b1a7, 95 },
    { 0, 0 },
    { 0xeee6b1a9, 93 },
    { 0x5366b1a9, 33 },
    { 0xdd26b1ab, 61 },
    { 0xd6b6b1ac, 98 },
    { 0x9466b1ab, 98 },
    { 0, 0 },
    { 0xb6c6b1af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb6b1b5, 90 },
    { 0xb9d6b1b6, 76 },
    { 0xd836b1b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f46b1bc, 76 },
    { 0xf2b6b1bd, 98 },
    { 0, 0 },
    { 0x38b6b1bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e36b1c5, 90 },
    { 0x8f66b1c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4066b1c9, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a26b1d2, 91 },
    { 0x4026b1d2, 59 },
    { 0, 0 },
    { 0x1d56b1d5, 90 },
    { 0, 0 },
    { 0xcf06b1d7, 95 },
    { 0xf46b1d7, 88 },
    { 0xef86b1d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c76b1dd, 93 },
    { 0x9e76b1de, 100 },
    { 0xaa86b1df, 93 },
    { 0x3806b1dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6b1e6, 55 },
    { 0x1ae6b1e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6b1f0, 85 },
    { 0x1c96b1f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1586b1f4, 33 },
    { 0, 0 },
    { 0x87f6b1f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd6b1fa, 61 },
    { 0x76a6b1fa, 91 },
    { 0, 0 },
    { 0xf3b6b1fd, 77 },
    { 0x98d6b1fd, 61 },
    { 0xb6e6b1fd, 95 },
    { 0, 0 },
    { 0x7d96b201, 53 },
    { 0x8ef6b202, 90 },
    { 0x7856b203, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2196b206, 98 },
    { 0x5686b206, 78 },
    { 0xc776b208, 90 },
    { 0x4076b207, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26b20c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6b20f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc56b212, 61 },
    { 0, 0 },
    { 0x35a6b214, 86 },
    { 0x10c6b214, 90 },
    { 0xd096b216, 77 },
    { 0, 0 },
    { 0x41e6b218, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe926b21b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc156b221, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb6b227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf6b22d, 65 },
    { 0x8d86b22e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b6b232, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e86b236, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf936b239, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf6b23d, 93 },
    { 0x3f86b23d, 90 },
    { 0, 0 },
    { 0xb46b240, 46 },
    { 0xb5d6b240, 43 },
    { 0x7286b242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e6b245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8636b24d, 98 },
    { 0, 0 },
    { 0xa8c6b24f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf966b252, 23 },
    { 0, 0 },
    { 0x6a86b254, 87 },
    { 0x326b255, 90 },
    { 0, 0 },
    { 0x6d36b257, 90 },
    { 0, 0 },
    { 0xc056b259, 23 },
    { 0xe2e6b25a, 90 },
    { 0xc5d6b25b, 61 },
    { 0, 0 },
    { 0xfe96b25d, 52 },
    { 0xb526b25e, 76 },
    { 0x33c6b25e, 77 },
    { 0x2006b260, 43 },
    { 0xf576b25e, 90 },
    { 0xedb6b262, 33 },
    { 0, 0 },
    { 0x6e26b264, 43 },
    { 0x34a6b265, 78 },
    { 0, 0 },
    { 0xf256b267, 95 },
    { 0, 0 },
    { 0x5e56b269, 33 },
    { 0, 0 },
    { 0xd476b26b, 90 },
    { 0x9b56b26b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4236b273, 52 },
    { 0x74d6b274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb606b277, 23 },
    { 0, 0 },
    { 0x5d26b279, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b36b283, 90 },
    { 0, 0 },
    { 0x4eb6b285, 90 },
    { 0xbc96b286, 76 },
    { 0x486b285, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9286b28b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06b28e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956b292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a16b295, 95 },
    { 0, 0 },
    { 0x1736b297, 33 },
    { 0x37d6b298, 93 },
    { 0x3656b299, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9986b29c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a6b29f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b6b2a3, 78 },
    { 0x9dc6b2a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb6b2a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456b2ab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf156b2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1646b2b3, 33 },
    { 0xfeb6b2b3, 90 },
    { 0, 0 },
    { 0x1886b2b6, 98 },
    { 0, 0 },
    { 0x786b2b8, 34 },
    { 0xde06b2b9, 33 },
    { 0xcd66b2ba, 90 },
    { 0x19d6b2ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4736b2bf, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c6b2c3, 61 },
    { 0, 0 },
    { 0x386b2c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae06b2ca, 81 },
    { 0x51a6b2cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f86b2d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba36b2d6, 23 },
    { 0xea36b2d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f46b2db, 99 },
    { 0x5e6b2dc, 90 },
    { 0, 0 },
    { 0x5596b2de, 61 },
    { 0x8826b2de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9186b2e3, 33 },
    { 0x7fd6b2e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106b2e7, 43 },
    { 0, 0 },
    { 0x9f6b2e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b6b2ec, 77 },
    { 0x9cd6b2ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb96b2f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5436b2f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae6b2fa, 78 },
    { 0x8206b2fb, 38 },
    { 0xbc26b2fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4886b2ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b304, 76 },
    { 0, 0 },
    { 0xbe6b306, 40 },
    { 0, 0 },
    { 0x7b66b308, 98 },
    { 0, 0 },
    { 0x1c76b30a, 78 },
    { 0, 0 },
    { 0x4856b30c, 43 },
    { 0xa416b30d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a76b310, 90 },
    { 0, 0 },
    { 0x7486b312, 23 },
    { 0, 0 },
    { 0xe676b314, 95 },
    { 0xeab6b314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd96b318, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa356b31d, 90 },
    { 0x3b26b31e, 46 },
    { 0, 0 },
    { 0x3ee6b320, 33 },
    { 0, 0 },
    { 0x5676b322, 76 },
    { 0x2b06b322, 90 },
    { 0, 0 },
    { 0xc056b325, 33 },
    { 0x9e76b326, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf426b32c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5356b330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd6b334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb066b337, 33 },
    { 0, 0 },
    { 0x49e6b339, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a56b33c, 95 },
    { 0, 0 },
    { 0x8bb6b33e, 93 },
    { 0, 0 },
    { 0x87e6b340, 90 },
    { 0x7026b341, 75 },
    { 0xbe86b342, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e96b34d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2796b352, 66 },
    { 0, 0 },
    { 0x1fa6b354, 98 },
    { 0xe196b355, 23 },
    { 0, 0 },
    { 0x9416b357, 91 },
    { 0, 0 },
    { 0x78c6b359, 95 },
    { 0x5ef6b359, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea6b35f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46b363, 76 },
    { 0xcf56b364, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f06b369, 23 },
    { 0x57a6b36a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16b36d, 43 },
    { 0x4216b36e, 61 },
    { 0x4ad6b36f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca6b380, 46 },
    { 0x88d6b381, 55 },
    { 0x7286b381, 91 },
    { 0x8546b383, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686b387, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af6b38b, 95 },
    { 0xabb6b38c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326b39a, 78 },
    { 0x7566b39a, 33 },
    { 0, 0 },
    { 0x45a6b39d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c86b3a7, 64 },
    { 0, 0 },
    { 0x5236b3a9, 33 },
    { 0x4906b3aa, 33 },
    { 0x9c96b3ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc686b3ae, 76 },
    { 0xb226b3ae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9986b3b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca6b3b6, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee56b3ba, 33 },
    { 0x8f76b3bb, 53 },
    { 0xbd66b3ba, 95 },
    { 0xf976b3bb, 87 },
    { 0x6616b3be, 34 },
    { 0x2636b3bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f6b3c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe376b3c5, 99 },
    { 0, 0 },
    { 0x7b06b3c7, 88 },
    { 0x1d96b3c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46b3cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f96b3d1, 34 },
    { 0xd3d6b3d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf596b3dd, 76 },
    { 0x506b3de, 23 },
    { 0x98e6b3de, 98 },
    { 0xc7b6b3e0, 46 },
    { 0x7da6b3e1, 33 },
    { 0x9ba6b3e0, 93 },
    { 0xaec6b3e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6b3ea, 91 },
    { 0, 0 },
    { 0x37c6b3ec, 61 },
    { 0xd6f6b3ec, 75 },
    { 0, 0 },
    { 0x7946b3ef, 68 },
    { 0x2266b3ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a6b3f6, 90 },
    { 0xe436b3f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c86b3fa, 61 },
    { 0x5386b3fa, 96 },
    { 0x5cb6b3fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba6b402, 23 },
    { 0xef06b402, 98 },
    { 0xc946b403, 88 },
    { 0x8926b405, 55 },
    { 0x6cf6b406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6b40a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2306b40f, 23 },
    { 0xfde6b40f, 46 },
    { 0x4de6b411, 82 },
    { 0xd706b411, 66 },
    { 0, 0 },
    { 0x13a6b414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d6b417, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc496b41c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d96b41f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4426b423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d6b42b, 76 },
    { 0x3766b42c, 67 },
    { 0, 0 },
    { 0xe0f6b42e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6b431, 43 },
    { 0xbdc6b432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f6b43c, 33 },
    { 0xcf96b43c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96b448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c6b44b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb646b451, 23 },
    { 0x1ed6b451, 46 },
    { 0x91d6b453, 52 },
    { 0x1f16b453, 66 },
    { 0x1926b455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c6b458, 61 },
    { 0x9dd6b459, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216b45d, 33 },
    { 0x8036b45d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e6b461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86b464, 61 },
    { 0, 0 },
    { 0xdf46b466, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa116b46b, 61 },
    { 0xbfd6b46c, 55 },
    { 0, 0 },
    { 0x8a16b46e, 33 },
    { 0, 0 },
    { 0xa0d6b470, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd816b473, 55 },
    { 0x8046b474, 43 },
    { 0xb66b475, 98 },
    { 0x7016b474, 90 },
    { 0x41e6b477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d26b47b, 23 },
    { 0xe166b47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd16b481, 101 },
    { 0x74f6b481, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6b485, 95 },
    { 0xc686b486, 43 },
    { 0, 0 },
    { 0x8c86b488, 90 },
    { 0, 0 },
    { 0x54c6b48a, 53 },
    { 0x71d6b48a, 43 },
    { 0xb756b48c, 61 },
    { 0x8ea6b48c, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f36b492, 76 },
    { 0, 0 },
    { 0x5996b494, 23 },
    { 0xe906b495, 43 },
    { 0x84b6b496, 33 },
    { 0x3276b497, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906b49c, 33 },
    { 0x426b49d, 33 },
    { 0x6976b49c, 76 },
    { 0x5746b49d, 90 },
    { 0x8e86b49d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc566b4a3, 23 },
    { 0x3456b4a3, 61 },
    { 0xc466b4a5, 76 },
    { 0xe0c6b4a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d6b4b3, 61 },
    { 0, 0 },
    { 0x7606b4b5, 61 },
    { 0, 0 },
    { 0x7776b4b7, 95 },
    { 0xe616b4b8, 61 },
    { 0, 0 },
    { 0x5c76b4ba, 66 },
    { 0, 0 },
    { 0x82c6b4bc, 61 },
    { 0x3f56b4bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe6b4c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc626b4c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a6b4cb, 91 },
    { 0x7a26b4cc, 95 },
    { 0xbb86b4cd, 85 },
    { 0x3426b4ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e6b4d3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x996b4d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b6b4da, 33 },
    { 0, 0 },
    { 0x86c6b4dc, 90 },
    { 0, 0 },
    { 0x2fd6b4de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff06b4e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2296b4f1, 61 },
    { 0x6466b4f2, 23 },
    { 0x9b46b4f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7386b4f7, 61 },
    { 0xd636b4f7, 34 },
    { 0xeb66b4f9, 95 },
    { 0xf3d6b4fa, 33 },
    { 0, 0 },
    { 0x16c6b4fc, 33 },
    { 0, 0 },
    { 0x93b6b4fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6b501, 90 },
    { 0, 0 },
    { 0xc836b503, 33 },
    { 0, 0 },
    { 0x736b505, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b36b508, 17 },
    { 0x60b6b509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b50d, 98 },
    { 0x2666b50e, 85 },
    { 0xad76b50e, 34 },
    { 0x60e6b50e, 33 },
    { 0x1956b50f, 96 },
    { 0x66c6b512, 84 },
    { 0xb566b513, 66 },
    { 0x7d6b513, 33 },
    { 0, 0 },
    { 0xd2d6b516, 95 },
    { 0, 0 },
    { 0x7276b518, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a16b51b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c6b51f, 77 },
    { 0x2a86b520, 76 },
    { 0x5c6b51f, 61 },
    { 0x70b6b522, 87 },
    { 0x46f6b522, 86 },
    { 0, 0 },
    { 0x9046b525, 87 },
    { 0x9666b526, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb6b52b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd526b53b, 98 },
    { 0x236b53c, 61 },
    { 0xac86b53d, 33 },
    { 0x5e66b53e, 61 },
    { 0xc966b53f, 61 },
    { 0, 0 },
    { 0x33a6b541, 33 },
    { 0x21c6b542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1446b545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef6b548, 90 },
    { 0x2636b549, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca6b550, 98 },
    { 0x8c56b550, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1716b558, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c6b55c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd806b560, 46 },
    { 0x3ad6b561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e56b565, 78 },
    { 0x5986b566, 87 },
    { 0x2036b566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd6b56c, 34 },
    { 0x20f6b56d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5226b572, 95 },
    { 0, 0 },
    { 0x7aa6b574, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a16b578, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa146b57c, 23 },
    { 0, 0 },
    { 0x27b6b57e, 90 },
    { 0, 0 },
    { 0x7726b580, 96 },
    { 0x4436b580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec6b588, 61 },
    { 0x9c6b589, 36 },
    { 0xa0d6b589, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef6b58d, 98 },
    { 0x7ad6b58e, 23 },
    { 0x67c6b58d, 90 },
    { 0xe416b58f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b16b597, 90 },
    { 0x5d76b598, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6b59c, 91 },
    { 0xbf6b59d, 93 },
    { 0x556b59c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa76b5a1, 99 },
    { 0xd096b5a2, 90 },
    { 0, 0 },
    { 0x19f6b5a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c86b5aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b6b5ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a6b5b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d6b5b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b96b5be, 90 },
    { 0xecb6b5be, 55 },
    { 0x8456b5be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf6b5c3, 55 },
    { 0xe516b5c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x106b5c7, 90 },
    { 0x2c56b5c8, 84 },
    { 0, 0 },
    { 0x7fb6b5ca, 55 },
    { 0, 0 },
    { 0x7056b5cc, 78 },
    { 0xa426b5cd, 55 },
    { 0x39c6b5ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb46b5d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3606b5d9, 36 },
    { 0x1cb6b5da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8876b5dd, 45 },
    { 0xe816b5dd, 61 },
    { 0x9ab6b5df, 90 },
    { 0x9426b5e0, 23 },
    { 0xd26b5e1, 55 },
    { 0x9466b5e1, 23 },
    { 0x9836b5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd26b5e9, 23 },
    { 0x8cc6b5e9, 61 },
    { 0, 0 },
    { 0xddd6b5ec, 61 },
    { 0, 0 },
    { 0xe806b5ee, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a6b5f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3626b5fa, 90 },
    { 0, 0 },
    { 0xb4a6b5fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c6b5ff, 93 },
    { 0x42e6b600, 90 },
    { 0x7ef6b601, 33 },
    { 0x1a06b602, 90 },
    { 0xbd86b603, 90 },
    { 0xa3b6b603, 92 },
    { 0x5c46b602, 77 },
    { 0x5206b606, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4476b60b, 67 },
    { 0x1ec6b60c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2766b611, 87 },
    { 0, 0 },
    { 0x1f46b613, 90 },
    { 0x55d6b614, 23 },
    { 0xf9f6b615, 33 },
    { 0xa8e6b616, 90 },
    { 0, 0 },
    { 0xf686b618, 100 },
    { 0, 0 },
    { 0x5e36b61a, 61 },
    { 0x8ae6b61a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d6b620, 23 },
    { 0xdf86b620, 81 },
    { 0x3c96b622, 61 },
    { 0x93f6b621, 91 },
    { 0xc826b620, 61 },
    { 0x2d76b625, 91 },
    { 0xc0d6b622, 90 },
    { 0x9256b627, 87 },
    { 0, 0 },
    { 0x1c76b629, 52 },
    { 0, 0 },
    { 0xd4a6b62b, 90 },
    { 0xffa6b62c, 95 },
    { 0xfef6b62c, 23 },
    { 0, 0 },
    { 0x3e66b62f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e6b637, 88 },
    { 0xd356b638, 66 },
    { 0xf746b637, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe606b63d, 69 },
    { 0x7186b63d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f46b647, 55 },
    { 0, 0 },
    { 0xb226b649, 81 },
    { 0, 0 },
    { 0x3cb6b64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb936b64e, 33 },
    { 0, 0 },
    { 0x45a6b650, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa16b653, 91 },
    { 0, 0 },
    { 0xa826b655, 17 },
    { 0xdba6b656, 76 },
    { 0x5316b657, 100 },
    { 0, 0 },
    { 0xc036b659, 61 },
    { 0x9556b659, 33 },
    { 0xb5d6b65a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c6b663, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b6b666, 46 },
    { 0xe156b667, 61 },
    { 0x75f6b668, 52 },
    { 0xdbc6b668, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xe556b66c, 38 },
    { 0x8b66b66c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4306b673, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2316b678, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f6b680, 33 },
    { 0, 0 },
    { 0xacb6b682, 23 },
    { 0xd1a6b683, 90 },
    { 0x6756b684, 23 },
    { 0x42f6b685, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe376b68b, 75 },
    { 0x70e6b68b, 77 },
    { 0x4366b68c, 100 },
    { 0xe416b68b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1496b698, 87 },
    { 0, 0 },
    { 0x92a6b69a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5166b69e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca6b6a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b46b6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4866b6a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf06b6ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6b6b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da6b6b3, 95 },
    { 0xc1b6b6b4, 90 },
    { 0x1ee6b6b4, 87 },
    { 0x2d16b6b6, 95 },
    { 0x6896b6b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b46b6bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a36b6c0, 23 },
    { 0x2dc6b6c1, 84 },
    { 0, 0 },
    { 0xed86b6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9806b6c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1396b6cb, 77 },
    { 0x9d6b6cc, 90 },
    { 0x3e46b6cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4876b6d1, 100 },
    { 0xea6b6d2, 75 },
    { 0x9a86b6d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6b6d8, 100 },
    { 0x5e66b6d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a16b6de, 46 },
    { 0xf346b6df, 90 },
    { 0x4f96b6e0, 55 },
    { 0x3a16b6e0, 93 },
    { 0x7c76b6e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c26b6e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa6b6ec, 46 },
    { 0x1e6b6ec, 76 },
    { 0x78f6b6ee, 78 },
    { 0x43f6b6ee, 61 },
    { 0xcd26b6ec, 23 },
    { 0x2526b6ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec86b6f4, 61 },
    { 0x8116b6f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b6b6f8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386b6ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe766b702, 61 },
    { 0xee46b703, 57 },
    { 0x2636b703, 33 },
    { 0, 0 },
    { 0x1cd6b706, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb6b70c, 85 },
    { 0, 0 },
    { 0xd546b70e, 55 },
    { 0x7676b70f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb66b713, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f6b718, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8556b71c, 98 },
    { 0xfea6b71d, 95 },
    { 0x8ed6b71e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e6b723, 61 },
    { 0x96b6b723, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc6b736, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d46b73c, 90 },
    { 0x84a6b73c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc046b746, 33 },
    { 0xc266b747, 98 },
    { 0, 0 },
    { 0xe6a6b749, 33 },
    { 0xc916b74a, 23 },
    { 0xc186b74b, 46 },
    { 0xf506b74c, 80 },
    { 0xa3f6b74d, 90 },
    { 0, 0 },
    { 0x79d6b74f, 87 },
    { 0x9e6b750, 33 },
    { 0xb346b751, 90 },
    { 0x8a76b751, 43 },
    { 0x8c86b752, 33 },
    { 0xc846b754, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836b75c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7216b761, 43 },
    { 0, 0 },
    { 0x8976b763, 33 },
    { 0x1546b763, 91 },
    { 0xae96b763, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb6b774, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7126b777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e6b77b, 33 },
    { 0x3b66b77b, 67 },
    { 0xc76b77d, 76 },
    { 0xa056b77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f6b786, 76 },
    { 0xaa6b787, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1096b78d, 68 },
    { 0xe906b78e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56b795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6b79b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646b79f, 93 },
    { 0xcf56b7a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3836b7a3, 23 },
    { 0x1276b7a4, 43 },
    { 0x21e6b7a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb6b7a9, 95 },
    { 0xb0d6b7a9, 84 },
    { 0x12c6b7ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c76b7ae, 76 },
    { 0, 0 },
    { 0x4b66b7b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c16b7b3, 52 },
    { 0x216b7b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4096b7c2, 95 },
    { 0x8466b7c3, 95 },
    { 0xf506b7c4, 90 },
    { 0, 0 },
    { 0xaf46b7c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe536b7ca, 23 },
    { 0x4b86b7cb, 55 },
    { 0x9986b7cb, 90 },
    { 0x55f6b7cd, 46 },
    { 0x21f6b7cd, 33 },
    { 0x8ce6b7cf, 78 },
    { 0xe6c6b7cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6b7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe656b7d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe46b7dd, 98 },
    { 0x3696b7dd, 85 },
    { 0x6cb6b7df, 61 },
    { 0x6816b7e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8476b7e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc126b7ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc6b7f4, 23 },
    { 0, 0 },
    { 0xe196b7f6, 76 },
    { 0x6006b7f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8476b7fa, 67 },
    { 0xb46b7fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb6b801, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f6b809, 87 },
    { 0x7ce6b809, 35 },
    { 0, 0 },
    { 0xf586b80c, 80 },
    { 0, 0 },
    { 0xa116b80e, 95 },
    { 0, 0 },
    { 0x7666b810, 100 },
    { 0xc316b811, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f6b818, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b81b, 95 },
    { 0x8a06b81c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5696b81f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2866b823, 33 },
    { 0x2726b824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7716b829, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa746b82c, 98 },
    { 0, 0 },
    { 0x9776b82e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a86b833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b83b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e6b843, 33 },
    { 0, 0 },
    { 0x2346b845, 98 },
    { 0x38f6b846, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4296b84b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d6b852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b26b856, 100 },
    { 0x3fa6b857, 58 },
    { 0, 0 },
    { 0x9cd6b859, 98 },
    { 0, 0 },
    { 0xc156b85b, 98 },
    { 0x4766b85c, 34 },
    { 0xd816b85b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa686b864, 46 },
    { 0xea56b865, 98 },
    { 0, 0 },
    { 0x7746b867, 33 },
    { 0, 0 },
    { 0x7c66b869, 34 },
    { 0, 0 },
    { 0xe06b86b, 98 },
    { 0x34b6b86b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a86b86f, 93 },
    { 0xab96b870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb296b877, 76 },
    { 0x9b6b877, 95 },
    { 0x1436b879, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2596b87c, 61 },
    { 0x6f6b87d, 33 },
    { 0xcfa6b87e, 33 },
    { 0x4786b87f, 90 },
    { 0x9ca6b87c, 83 },
    { 0, 0 },
    { 0xe4c6b882, 23 },
    { 0xccb6b883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf96b886, 76 },
    { 0x83d6b887, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3836b88f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc906b896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b6b89a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e6b89d, 33 },
    { 0, 0 },
    { 0x6286b89f, 90 },
    { 0xa8e6b8a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd6b8a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea6b8a8, 90 },
    { 0xa576b8a8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb146b8ac, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb6b8b1, 33 },
    { 0xfbd6b8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa936b8b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a96b8b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6666b8bd, 46 },
    { 0, 0 },
    { 0x4096b8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a76b8c2, 100 },
    { 0x43e6b8c3, 61 },
    { 0, 0 },
    { 0x91b6b8c5, 100 },
    { 0, 0 },
    { 0xd7b6b8c7, 61 },
    { 0, 0 },
    { 0xbc26b8c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d36b8d5, 83 },
    { 0x306b8d5, 43 },
    { 0xeec6b8d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb26b8dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a6b8e1, 36 },
    { 0x4e86b8e2, 84 },
    { 0x2616b8e2, 95 },
    { 0xd3e6b8e4, 55 },
    { 0xa776b8e4, 98 },
    { 0, 0 },
    { 0x2fa6b8e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b46b8ea, 46 },
    { 0xab06b8eb, 33 },
    { 0xe076b8ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6b8ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86b8f2, 34 },
    { 0x8d06b8f2, 23 },
    { 0xd926b8f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d6b8f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1466b8fc, 98 },
    { 0xde26b8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6b901, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6b905, 93 },
    { 0x46d6b905, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b6b90d, 61 },
    { 0x36f6b90e, 46 },
    { 0xceb6b90f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a26b913, 80 },
    { 0x67c6b913, 76 },
    { 0xe6d6b915, 78 },
    { 0, 0 },
    { 0xabc6b917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a6b91b, 87 },
    { 0xd766b91c, 95 },
    { 0, 0 },
    { 0x20d6b91e, 95 },
    { 0xcea6b91e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf6b922, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e6b925, 33 },
    { 0xc9f6b925, 84 },
    { 0x6b46b927, 33 },
    { 0x5496b927, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc096b92c, 52 },
    { 0xe296b92d, 23 },
    { 0x5126b92d, 33 },
    { 0xad06b92e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2796b932, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56b935, 33 },
    { 0, 0 },
    { 0x58d6b937, 33 },
    { 0xe896b938, 90 },
    { 0, 0 },
    { 0x82b6b93a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9936b93d, 66 },
    { 0x88c6b93d, 61 },
    { 0x66c6b93f, 93 },
    { 0x8986b93e, 34 },
    { 0x1d26b93d, 61 },
    { 0, 0 },
    { 0xae96b943, 90 },
    { 0, 0 },
    { 0xac76b945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb266b948, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e56b94c, 76 },
    { 0, 0 },
    { 0x8766b94e, 90 },
    { 0xb9d6b94f, 98 },
    { 0x98d6b950, 53 },
    { 0, 0 },
    { 0x79d6b952, 33 },
    { 0, 0 },
    { 0x5326b954, 85 },
    { 0x7ea6b955, 61 },
    { 0, 0 },
    { 0x35d6b957, 95 },
    { 0x7bc6b958, 100 },
    { 0x8796b959, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e96b95e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3456b962, 61 },
    { 0x5116b963, 95 },
    { 0x72e6b963, 52 },
    { 0x8c56b964, 93 },
    { 0xadd6b966, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb816b96b, 90 },
    { 0, 0 },
    { 0x5c06b96d, 90 },
    { 0x33c6b96e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316b973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8406b976, 52 },
    { 0x68d6b977, 34 },
    { 0x33f6b977, 95 },
    { 0x5186b979, 33 },
    { 0x58f6b977, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc156b97f, 33 },
    { 0xc696b980, 43 },
    { 0x1e26b981, 33 },
    { 0x8246b980, 90 },
    { 0, 0 },
    { 0x6de6b984, 100 },
    { 0xc956b985, 95 },
    { 0x8e6b984, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06b98d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e6b990, 61 },
    { 0xb156b991, 61 },
    { 0, 0 },
    { 0x8d96b993, 93 },
    { 0xc386b993, 17 },
    { 0, 0 },
    { 0x1fe6b996, 76 },
    { 0x9b96b997, 66 },
    { 0x2c76b998, 55 },
    { 0xa876b997, 90 },
    { 0x8076b998, 94 },
    { 0x8f76b99b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac6b9a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b9a7, 98 },
    { 0xabb6b9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b9ab, 33 },
    { 0, 0 },
    { 0xbb56b9ad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de6b9b2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a26b9b7, 87 },
    { 0x8896b9b7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef6b9bc, 43 },
    { 0xc096b9bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d6b9c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d56b9c8, 34 },
    { 0xe396b9c8, 91 },
    { 0x8bd6b9ca, 23 },
    { 0x45c6b9cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f6b9ce, 23 },
    { 0, 0 },
    { 0xca46b9d0, 61 },
    { 0x5ba6b9d1, 33 },
    { 0xa386b9d2, 38 },
    { 0xad56b9d3, 84 },
    { 0xa4b6b9d4, 23 },
    { 0, 0 },
    { 0x31d6b9d6, 23 },
    { 0x6e16b9d6, 76 },
    { 0xa5c6b9d8, 23 },
    { 0, 0 },
    { 0xced6b9da, 90 },
    { 0x9916b9db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c6b9de, 33 },
    { 0xb5a6b9de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab6b9e2, 100 },
    { 0xa716b9e2, 55 },
    { 0xa7f6b9e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5296b9ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe526b9ed, 75 },
    { 0xd566b9ed, 23 },
    { 0x9b86b9ef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa16b9f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7576b9f6, 98 },
    { 0, 0 },
    { 0xc996b9f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb896b9fb, 34 },
    { 0, 0 },
    { 0xf266b9fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2686ba08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb606ba0b, 90 },
    { 0, 0 },
    { 0x88e6ba0d, 61 },
    { 0x2fa6ba0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6376ba12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a96ba15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5746ba19, 90 },
    { 0x4cd6ba1a, 78 },
    { 0xc836ba19, 23 },
    { 0x6316ba1c, 33 },
    { 0x3666ba19, 46 },
    { 0x1566ba1e, 61 },
    { 0xc576ba1e, 23 },
    { 0x3386ba20, 33 },
    { 0xdce6ba21, 95 },
    { 0x9726ba21, 61 },
    { 0x7966ba20, 95 },
    { 0x2056ba23, 61 },
    { 0xf956ba22, 75 },
    { 0xd706ba26, 90 },
    { 0x636ba24, 33 },
    { 0x5826ba1e, 55 },
    { 0x6b06ba29, 66 },
    { 0x6606ba2a, 90 },
    { 0xfbb6ba28, 61 },
    { 0x4566ba2c, 33 },
    { 0x6886ba2d, 90 },
    { 0xb4b6ba2e, 99 },
    { 0xc3b6ba2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a6ba35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf26ba3a, 90 },
    { 0xe726ba3b, 93 },
    { 0, 0 },
    { 0x946ba3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c6ba40, 99 },
    { 0xfef6ba41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca6ba46, 87 },
    { 0, 0 },
    { 0xcb26ba48, 93 },
    { 0, 0 },
    { 0xd216ba4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a6ba4f, 97 },
    { 0, 0 },
    { 0xd4c6ba51, 90 },
    { 0xd036ba52, 95 },
    { 0xc626ba53, 100 },
    { 0xeb66ba54, 95 },
    { 0xe4a6ba55, 61 },
    { 0x9096ba56, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc6ba5b, 93 },
    { 0x8db6ba5c, 85 },
    { 0, 0 },
    { 0xb376ba5e, 68 },
    { 0x2936ba5e, 34 },
    { 0xedf6ba60, 91 },
    { 0, 0 },
    { 0x6bf6ba62, 61 },
    { 0xd976ba63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6ba6b, 95 },
    { 0xe936ba6b, 33 },
    { 0x1a56ba6c, 76 },
    { 0xf356ba6c, 61 },
    { 0x1736ba6c, 92 },
    { 0x34d6ba6c, 43 },
    { 0xb0d6ba71, 61 },
    { 0x2b6ba6c, 55 },
    { 0, 0 },
    { 0xc56ba74, 87 },
    { 0, 0 },
    { 0x1696ba76, 5 },
    { 0, 0 },
    { 0xa766ba78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1056ba7b, 61 },
    { 0, 0 },
    { 0x48f6ba7d, 33 },
    { 0xad26ba7e, 95 },
    { 0x63b6ba7d, 93 },
    { 0xc226ba7f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf256ba85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd896ba8e, 55 },
    { 0xc156ba8f, 61 },
    { 0, 0 },
    { 0x3dc6ba91, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f6ba94, 77 },
    { 0x3466ba95, 90 },
    { 0x7746ba94, 65 },
    { 0xb8f6ba97, 76 },
    { 0x7e16ba98, 90 },
    { 0x4886ba97, 66 },
    { 0, 0 },
    { 0x5d36ba9b, 98 },
    { 0, 0 },
    { 0x2c26ba9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5506baa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c6baaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc6bab0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc6bab5, 33 },
    { 0xebe6bab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736bab9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x546babc, 61 },
    { 0x4d6babc, 23 },
    { 0, 0 },
    { 0xf5b6babf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8416bac3, 90 },
    { 0x17d6bac4, 61 },
    { 0xc406bac3, 95 },
    { 0xe7d6bac6, 80 },
    { 0x886bac6, 76 },
    { 0x87c6bac8, 90 },
    { 0x4e06bac9, 61 },
    { 0x2876baca, 34 },
    { 0x8c36bacb, 90 },
    { 0x9626bacc, 90 },
    { 0x57a6baca, 55 },
    { 0, 0 },
    { 0x1406bacf, 90 },
    { 0, 0 },
    { 0x7666bad1, 43 },
    { 0xc966bad2, 61 },
    { 0, 0 },
    { 0x65d6bad4, 95 },
    { 0x7e46bad5, 67 },
    { 0xebc6bad6, 98 },
    { 0xc396bad6, 61 },
    { 0x6416bad7, 90 },
    { 0x1276bad9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f86bae4, 33 },
    { 0xdf46bae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f6bae9, 90 },
    { 0x6f06baea, 93 },
    { 0x9506baea, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d36baf2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d6baf5, 93 },
    { 0xf766baf5, 43 },
    { 0, 0 },
    { 0x3296baf8, 23 },
    { 0xafd6baf9, 33 },
    { 0x976bafa, 98 },
    { 0xce96bafb, 98 },
    { 0x31c6bafb, 52 },
    { 0, 0 },
    { 0x2186bafe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2526bb01, 66 },
    { 0xb746bb02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a36bb0a, 95 },
    { 0x25b6bb0b, 90 },
    { 0, 0 },
    { 0xf6b6bb0d, 48 },
    { 0xb266bb0d, 90 },
    { 0xcda6bb0f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa686bb12, 55 },
    { 0xf196bb13, 78 },
    { 0x9ae6bb13, 98 },
    { 0, 0 },
    { 0x6b6bb16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb466bb19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a6bb1d, 90 },
    { 0x84e6bb1d, 99 },
    { 0xdaf6bb1f, 92 },
    { 0, 0 },
    { 0x2f76bb21, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5896bb26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a36bb2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6566bb36, 91 },
    { 0, 0 },
    { 0xb9e6bb38, 46 },
    { 0xcba6bb38, 95 },
    { 0xcd26bb39, 23 },
    { 0xffe6bb38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6436bb41, 93 },
    { 0xfff6bb42, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xed16bb45, 97 },
    { 0, 0 },
    { 0x37e6bb47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea6bb4a, 33 },
    { 0x2876bb4a, 61 },
    { 0x2bf6bb4c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc126bb50, 55 },
    { 0x1656bb50, 33 },
    { 0xb706bb52, 98 },
    { 0xa276bb53, 43 },
    { 0xb816bb52, 33 },
    { 0x5856bb55, 23 },
    { 0x9446bb55, 34 },
    { 0x4136bb57, 98 },
    { 0xaed6bb58, 90 },
    { 0, 0 },
    { 0xd896bb5a, 56 },
    { 0x6c6bb5b, 52 },
    { 0x24c6bb5a, 98 },
    { 0x8d36bb5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe356bb62, 53 },
    { 0xdeb6bb62, 33 },
    { 0, 0 },
    { 0x7c6bb65, 89 },
    { 0, 0 },
    { 0xd056bb67, 90 },
    { 0, 0 },
    { 0x1a36bb69, 61 },
    { 0xd276bb6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6bb6e, 61 },
    { 0x6176bb6e, 90 },
    { 0x8556bb70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d6bb73, 61 },
    { 0xb376bb74, 90 },
    { 0, 0 },
    { 0x9e56bb76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b66bb7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d6bb7e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1226bb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176bb8a, 53 },
    { 0, 0 },
    { 0xe5e6bb8c, 95 },
    { 0x8f36bb8c, 55 },
    { 0xb906bb8c, 61 },
    { 0x11c6bb8f, 61 },
    { 0x16e6bb8c, 95 },
    { 0xea16bb91, 76 },
    { 0xb996bb92, 43 },
    { 0x6186bb93, 34 },
    { 0xac6bb94, 90 },
    { 0x366bb95, 48 },
    { 0xb7b6bb95, 33 },
    { 0x9a66bb96, 95 },
    { 0x5af6bb98, 95 },
    { 0, 0 },
    { 0xb6c6bb9a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6bb9d, 23 },
    { 0, 0 },
    { 0x9ba6bb9f, 98 },
    { 0, 0 },
    { 0x4b6bba1, 90 },
    { 0x2526bba2, 55 },
    { 0xebd6bba3, 95 },
    { 0x27d6bba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad6bba9, 33 },
    { 0, 0 },
    { 0xa276bbab, 84 },
    { 0, 0 },
    { 0x6246bbad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de6bbb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea6bbb5, 23 },
    { 0xd8d6bbb5, 95 },
    { 0xcfb6bbb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4616bbba, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d6bbc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6bbc5, 40 },
    { 0x4946bbc6, 55 },
    { 0x7d66bbc7, 52 },
    { 0x6196bbc6, 43 },
    { 0xc66bbc6, 91 },
    { 0xffb6bbc6, 61 },
    { 0xf8e6bbcb, 61 },
    { 0, 0 },
    { 0x4d36bbcd, 100 },
    { 0xd0e6bbcd, 90 },
    { 0, 0 },
    { 0x5ef6bbd0, 61 },
    { 0x9e36bbd0, 36 },
    { 0, 0 },
    { 0xa786bbd3, 98 },
    { 0x9976bbd4, 98 },
    { 0x5146bbd5, 95 },
    { 0xf0e6bbd6, 33 },
    { 0xe726bbd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc906bbdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6bbde, 34 },
    { 0xd836bbdf, 34 },
    { 0xe106bbe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846bbe3, 95 },
    { 0xa386bbe4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c56bbed, 23 },
    { 0x8876bbed, 88 },
    { 0, 0 },
    { 0x196bbf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436bbf6, 61 },
    { 0xef26bbf7, 90 },
    { 0, 0 },
    { 0xb626bbf9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196bbff, 17 },
    { 0xa986bc00, 87 },
    { 0, 0 },
    { 0x8556bc02, 78 },
    { 0xa426bc02, 93 },
    { 0, 0 },
    { 0x466bc05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b6bc0e, 9 },
    { 0x5836bc0e, 90 },
    { 0, 0 },
    { 0x3366bc11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b46bc14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be6bc1b, 89 },
    { 0x8066bc1c, 38 },
    { 0, 0 },
    { 0xf0c6bc1e, 98 },
    { 0xd416bc1f, 23 },
    { 0xd636bc20, 65 },
    { 0x2df6bc21, 78 },
    { 0x94d6bc22, 46 },
    { 0x35d6bc22, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a96bc28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe886bc32, 77 },
    { 0x2916bc33, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf996bc38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf246bc3c, 23 },
    { 0x8586bc3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6bc43, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe76bc47, 76 },
    { 0x3a36bc48, 48 },
    { 0x2876bc48, 90 },
    { 0xaa36bc4a, 33 },
    { 0x916bc4b, 33 },
    { 0x876bc4c, 99 },
    { 0xbf16bc48, 91 },
    { 0xf576bc49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb6bc57, 90 },
    { 0, 0 },
    { 0x36b6bc59, 33 },
    { 0x1aa6bc5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc06bc5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe706bc62, 66 },
    { 0x28f6bc62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f6bc66, 46 },
    { 0, 0 },
    { 0x88f6bc68, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d6bc6c, 43 },
    { 0, 0 },
    { 0xfd26bc6e, 23 },
    { 0x84f6bc6e, 95 },
    { 0x4646bc70, 95 },
    { 0xeea6bc6e, 91 },
    { 0xf3b6bc72, 98 },
    { 0xe196bc72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d6bc7c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb816bc81, 88 },
    { 0x8026bc82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2866bc88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d6bc8b, 90 },
    { 0xfd36bc8c, 98 },
    { 0xb2a6bc8d, 45 },
    { 0xf536bc8e, 52 },
    { 0xbb66bc8c, 90 },
    { 0xd3a6bc8c, 90 },
    { 0x4fe6bc8b, 76 },
    { 0, 0 },
    { 0x2b16bc93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2626bc9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba6bca3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc86bca6, 90 },
    { 0xc9c6bca7, 43 },
    { 0x85a6bca8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3056bcae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x956bcb1, 99 },
    { 0xe886bcb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c6bcba, 43 },
    { 0x6c86bcbb, 77 },
    { 0x17b6bcbc, 23 },
    { 0, 0 },
    { 0x6726bcbe, 33 },
    { 0, 0 },
    { 0xc4e6bcc0, 90 },
    { 0, 0 },
    { 0xb926bcc2, 55 },
    { 0x2616bcc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd956bcc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b6bcc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c6bccd, 97 },
    { 0xc9b6bcce, 33 },
    { 0x3066bcce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176bcd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe066bcda, 65 },
    { 0xae06bcdb, 46 },
    { 0, 0 },
    { 0x1b36bcdd, 75 },
    { 0xc456bcdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f6bce4, 87 },
    { 0, 0 },
    { 0x4776bce6, 79 },
    { 0x6ae6bce6, 100 },
    { 0x5ad6bce6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56bcf2, 80 },
    { 0xad36bcf3, 23 },
    { 0x3306bcf2, 95 },
    { 0xd686bcf5, 61 },
    { 0xca06bcf6, 80 },
    { 0xf946bcf6, 34 },
    { 0x916bcf8, 90 },
    { 0, 0 },
    { 0xbb46bcfa, 95 },
    { 0xe326bcfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee86bd06, 88 },
    { 0, 0 },
    { 0xe376bd08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7846bd0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde56bd0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb986bd16, 95 },
    { 0xcaa6bd17, 78 },
    { 0xb3f6bd17, 95 },
    { 0, 0 },
    { 0xc026bd1a, 23 },
    { 0x7186bd1a, 55 },
    { 0, 0 },
    { 0xd3e6bd1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1466bd20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a6bd24, 95 },
    { 0x3746bd24, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe6bd2b, 90 },
    { 0, 0 },
    { 0xa9b6bd2d, 23 },
    { 0, 0 },
    { 0x3326bd2f, 61 },
    { 0xb2d6bd30, 56 },
    { 0x62a6bd31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9956bd35, 61 },
    { 0, 0 },
    { 0x5fc6bd37, 90 },
    { 0, 0 },
    { 0x7426bd39, 91 },
    { 0x1f76bd3a, 98 },
    { 0x33f6bd3b, 42 },
    { 0x7f6bd3b, 61 },
    { 0x3166bd3a, 25 },
    { 0xd2e6bd3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4886bd44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf36bd48, 33 },
    { 0x87c6bd49, 66 },
    { 0x97c6bd4a, 67 },
    { 0xc66bd4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a66bd4f, 95 },
    { 0xe646bd4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e6bd58, 23 },
    { 0x67d6bd59, 85 },
    { 0x3bd6bd5a, 90 },
    { 0x796bd59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6bd62, 75 },
    { 0xd86bd62, 61 },
    { 0, 0 },
    { 0x16b6bd65, 98 },
    { 0xde96bd66, 98 },
    { 0xc46bd67, 17 },
    { 0x6986bd68, 90 },
    { 0xd386bd69, 95 },
    { 0xe586bd69, 55 },
    { 0, 0 },
    { 0x2266bd6c, 90 },
    { 0xee16bd6c, 69 },
    { 0xeab6bd6e, 78 },
    { 0x9316bd6c, 36 },
    { 0x2fa6bd70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf566bd78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec6bd7f, 33 },
    { 0x7c76bd80, 77 },
    { 0, 0 },
    { 0x2b46bd82, 76 },
    { 0x4896bd83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c6bd86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1636bd89, 61 },
    { 0x936bd89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe156bd8f, 33 },
    { 0xe5c6bd90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6266bd97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a6bd9e, 55 },
    { 0x16e6bd9f, 66 },
    { 0x5406bda0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa266bdac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5726bdb3, 33 },
    { 0x106bdb3, 61 },
    { 0x9a86bdb5, 83 },
    { 0xc616bdb6, 23 },
    { 0, 0 },
    { 0x4d16bdb8, 90 },
    { 0, 0 },
    { 0xb466bdba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16bdbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216bdc5, 90 },
    { 0x96d6bdc6, 61 },
    { 0x5b6bdc7, 34 },
    { 0xd536bdc8, 98 },
    { 0x7c26bdc9, 55 },
    { 0x9fb6bdca, 23 },
    { 0x2556bdcb, 100 },
    { 0x57e6bdc8, 90 },
    { 0x5a96bdcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8076bdd2, 95 },
    { 0xfd26bdd3, 88 },
    { 0xf916bdd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c6bdd7, 61 },
    { 0xd5c6bdd8, 90 },
    { 0x6fd6bdd7, 66 },
    { 0, 0 },
    { 0x3c56bddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed46bdde, 76 },
    { 0xcb56bddf, 66 },
    { 0, 0 },
    { 0x3626bde1, 90 },
    { 0x5ab6bde2, 90 },
    { 0, 0 },
    { 0xe486bde4, 90 },
    { 0xc9f6bde4, 55 },
    { 0x1336bde6, 46 },
    { 0x6086bde7, 81 },
    { 0, 0 },
    { 0x1b56bde9, 98 },
    { 0xe0e6bdea, 33 },
    { 0x3896bdeb, 33 },
    { 0xefc6bdeb, 90 },
    { 0x72b6bdea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c6bdf5, 61 },
    { 0x1326bdf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4006bdf9, 34 },
    { 0xecf6bdfa, 88 },
    { 0xadc6bdfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd666bdfe, 98 },
    { 0x6c66bdfe, 69 },
    { 0x7466bdfe, 91 },
    { 0xdb26be01, 100 },
    { 0, 0 },
    { 0x25c6be03, 33 },
    { 0, 0 },
    { 0x8ea6be05, 61 },
    { 0x3ba6be06, 69 },
    { 0x3be6be07, 33 },
    { 0x3906be07, 67 },
    { 0xd116be09, 46 },
    { 0xc7c6be0a, 97 },
    { 0xc156be0b, 90 },
    { 0xc7e6be0b, 95 },
    { 0xec96be09, 46 },
    { 0x2c86be09, 90 },
    { 0x9ba6be0f, 33 },
    { 0x2d56be0d, 33 },
    { 0xaf26be09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec16be14, 43 },
    { 0x7116be14, 35 },
    { 0xce26be16, 93 },
    { 0xb436be16, 23 },
    { 0x6566be14, 95 },
    { 0xfbd6be19, 33 },
    { 0xec06be18, 83 },
    { 0x5136be1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4306be1e, 23 },
    { 0, 0 },
    { 0x5256be20, 76 },
    { 0xee46be21, 33 },
    { 0xf156be21, 23 },
    { 0x33a6be21, 34 },
    { 0xfac6be24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb526be2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a06be31, 95 },
    { 0, 0 },
    { 0x4b36be33, 61 },
    { 0xcab6be34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6be3a, 99 },
    { 0x5096be3a, 84 },
    { 0x1ac6be3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7366be40, 46 },
    { 0x9e16be41, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd86be44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8906be4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566be54, 23 },
    { 0x4116be55, 33 },
    { 0, 0 },
    { 0xff46be57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff36be5d, 90 },
    { 0x7d66be5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf6be61, 55 },
    { 0xd166be62, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4286be68, 61 },
    { 0, 0 },
    { 0xefa6be6a, 95 },
    { 0xe6d6be6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7066be6e, 66 },
    { 0, 0 },
    { 0x2e76be70, 95 },
    { 0x3f06be71, 61 },
    { 0xbaa6be72, 61 },
    { 0xcd76be73, 95 },
    { 0x3576be74, 55 },
    { 0x6756be75, 33 },
    { 0x2e26be75, 98 },
    { 0xa9a6be71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9346be7b, 75 },
    { 0x49d6be7c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a76be80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb76be84, 95 },
    { 0x8026be85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c6be8c, 93 },
    { 0x1686be8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf346be90, 33 },
    { 0x7b76be90, 98 },
    { 0, 0 },
    { 0x5026be93, 95 },
    { 0, 0 },
    { 0x9b86be95, 90 },
    { 0x18b6be96, 95 },
    { 0x4fd6be97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6be9a, 36 },
    { 0x8f36be9b, 84 },
    { 0, 0 },
    { 0xe596be9d, 76 },
    { 0, 0 },
    { 0xf786be9f, 61 },
    { 0x7a76bea0, 100 },
    { 0x5e46bea0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c86bea7, 95 },
    { 0x9076bea8, 55 },
    { 0xbfd6bea9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c6beac, 23 },
    { 0xf8c6bead, 93 },
    { 0x85d6bead, 52 },
    { 0x4356bead, 100 },
    { 0x44f6beb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf076beb5, 66 },
    { 0x6126beb5, 76 },
    { 0xd6b6beb5, 61 },
    { 0x3206beb8, 33 },
    { 0x7766beb8, 95 },
    { 0x9246beb7, 67 },
    { 0, 0 },
    { 0x48e6bebc, 61 },
    { 0x73a6bebd, 68 },
    { 0xb8d6bebe, 98 },
    { 0xe376bebf, 33 },
    { 0xa296bec0, 66 },
    { 0xeda6bebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd6bec8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a6becd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa056bed1, 99 },
    { 0x8616bed2, 90 },
    { 0x1076bed3, 33 },
    { 0x6336bed2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb6bed8, 77 },
    { 0x8006bed8, 43 },
    { 0x10a6bed8, 46 },
    { 0xd3e6bedb, 33 },
    { 0x9996bedc, 95 },
    { 0x7f56bedd, 91 },
    { 0x3606bedd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f76bee9, 99 },
    { 0x9826beea, 46 },
    { 0, 0 },
    { 0x28d6beec, 95 },
    { 0x4416beed, 99 },
    { 0x45e6beed, 19 },
    { 0, 0 },
    { 0x1b46bef0, 90 },
    { 0xcdc6bef1, 98 },
    { 0xc706bef2, 95 },
    { 0x6e36bef3, 78 },
    { 0xeb16bef0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546befc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f6beff, 76 },
    { 0x4dd6bf00, 90 },
    { 0x15a6bf00, 90 },
    { 0x5ac6bf02, 78 },
    { 0x68e6bf02, 76 },
    { 0x75d6bf04, 95 },
    { 0x2b66bf05, 90 },
    { 0x2e86bf03, 80 },
    { 0xd476bf07, 93 },
    { 0x9506bf08, 95 },
    { 0, 0 },
    { 0x8d36bf0a, 64 },
    { 0x7e76bf0b, 90 },
    { 0x1f36bf0b, 98 },
    { 0, 0 },
    { 0xa756bf0e, 33 },
    { 0xdf06bf0f, 88 },
    { 0xa8a6bf10, 33 },
    { 0x4746bf11, 90 },
    { 0x2996bf10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f56bf15, 77 },
    { 0, 0 },
    { 0x5916bf17, 61 },
    { 0x6e46bf18, 90 },
    { 0xe246bf19, 90 },
    { 0x1eb6bf19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae6bf1e, 36 },
    { 0, 0 },
    { 0xe466bf20, 97 },
    { 0x44f6bf20, 100 },
    { 0xb696bf21, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f36bf28, 33 },
    { 0xfb36bf29, 90 },
    { 0x9fe6bf2a, 98 },
    { 0x8786bf2a, 43 },
    { 0, 0 },
    { 0x8a46bf2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe826bf32, 98 },
    { 0xbba6bf33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x976bf36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb6bf3f, 87 },
    { 0xe066bf40, 33 },
    { 0x26b6bf41, 95 },
    { 0xeb76bf40, 67 },
    { 0xb656bf43, 90 },
    { 0xf4a6bf40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb6bf4b, 23 },
    { 0, 0 },
    { 0x9b66bf4d, 23 },
    { 0x19b6bf4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f6bf56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a86bf5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e6bf5e, 55 },
    { 0, 0 },
    { 0x8006bf60, 99 },
    { 0, 0 },
    { 0x826bf62, 43 },
    { 0xecd6bf63, 34 },
    { 0, 0 },
    { 0x2796bf65, 76 },
    { 0xf736bf65, 95 },
    { 0x6416bf67, 23 },
    { 0x4396bf66, 33 },
    { 0xe9e6bf69, 96 },
    { 0, 0 },
    { 0x9076bf6b, 61 },
    { 0, 0 },
    { 0x6d96bf6d, 33 },
    { 0x8586bf6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6286bf73, 90 },
    { 0, 0 },
    { 0xed56bf75, 67 },
    { 0xc276bf76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f16bf7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116bf7d, 95 },
    { 0, 0 },
    { 0xa246bf7f, 69 },
    { 0x24d6bf80, 34 },
    { 0x9e86bf81, 76 },
    { 0xece6bf81, 55 },
    { 0xbec6bf81, 98 },
    { 0x5716bf81, 33 },
    { 0xbe56bf81, 75 },
    { 0x1b26bf83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6616bf89, 65 },
    { 0x1d86bf8a, 100 },
    { 0x7c86bf8b, 67 },
    { 0xed76bf8a, 75 },
    { 0, 0 },
    { 0x14a6bf8e, 90 },
    { 0x57a6bf8f, 90 },
    { 0x28e6bf8e, 43 },
    { 0xb7d6bf91, 90 },
    { 0xa526bf90, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b56bf96, 55 },
    { 0x6656bf97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf726bf9b, 55 },
    { 0x8bd6bf9b, 87 },
    { 0, 0 },
    { 0x58b6bf9e, 61 },
    { 0xcf26bf9f, 34 },
    { 0x1f86bf9e, 81 },
    { 0x2546bf9f, 34 },
    { 0x4f06bf9e, 58 },
    { 0x6816bfa3, 43 },
    { 0xd306bfa3, 100 },
    { 0xa886bfa3, 90 },
    { 0xc876bfa6, 69 },
    { 0xfa36bfa7, 90 },
    { 0xdb6bfa8, 75 },
    { 0x2bc6bfa9, 60 },
    { 0x2a56bfaa, 98 },
    { 0x2396bfab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x796bfae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a16bfbc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc6bfbf, 83 },
    { 0x8e26bfbf, 90 },
    { 0, 0 },
    { 0x5b66bfc2, 33 },
    { 0, 0 },
    { 0x4a36bfc4, 93 },
    { 0x2ca6bfc5, 93 },
    { 0x5c56bfc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c6bfcf, 90 },
    { 0x17e6bfd0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab6bfd3, 55 },
    { 0, 0 },
    { 0xed66bfd5, 33 },
    { 0x5396bfd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6bfd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6bfdd, 55 },
    { 0xd726bfdd, 95 },
    { 0xd486bfdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236bfe4, 76 },
    { 0xadd6bfe5, 90 },
    { 0, 0 },
    { 0xe436bfe7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a6bfeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7516bff1, 61 },
    { 0x8976bff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd6bff7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda6bffc, 87 },
    { 0x6a6bffc, 55 },
    { 0, 0 },
    { 0xfe16bfff, 33 },
    { 0xf2e6bfff, 98 },
    { 0, 0 },
    { 0xb5d6c002, 23 },
    { 0xbec6c003, 34 },
    { 0x4256c003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc936c008, 19 },
    { 0, 0 },
    { 0x4406c00a, 95 },
    { 0xc1c6c00a, 99 },
    { 0x21e6c00c, 33 },
    { 0xd686c00c, 66 },
    { 0x74e6c00d, 55 },
    { 0x4766c00f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d6c016, 77 },
    { 0, 0 },
    { 0x39d6c018, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf406c020, 34 },
    { 0xb996c021, 101 },
    { 0x4236c022, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8486c026, 98 },
    { 0x1356c027, 43 },
    { 0xa2d6c027, 90 },
    { 0xbed6c029, 98 },
    { 0x7c6c02a, 61 },
    { 0x1566c02a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9516c030, 87 },
    { 0x6586c030, 95 },
    { 0, 0 },
    { 0x4176c033, 90 },
    { 0x8a16c033, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x506c037, 23 },
    { 0, 0 },
    { 0x5206c039, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3996c03c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6c044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb496c047, 67 },
    { 0x3136c048, 33 },
    { 0x6e56c047, 61 },
    { 0, 0 },
    { 0x84d6c04b, 55 },
    { 0xfa6c04c, 76 },
    { 0x4ae6c04b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d6c053, 61 },
    { 0x1a96c054, 95 },
    { 0xa236c055, 93 },
    { 0x8d96c056, 97 },
    { 0x4366c057, 101 },
    { 0, 0 },
    { 0x4fc6c059, 55 },
    { 0x5ff6c05a, 23 },
    { 0, 0 },
    { 0xa236c05c, 95 },
    { 0xd196c05d, 61 },
    { 0xce66c05d, 23 },
    { 0x8046c05f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac6c062, 33 },
    { 0x56f6c063, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e86c06a, 61 },
    { 0, 0 },
    { 0x4346c06c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c16c071, 93 },
    { 0xeb36c072, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56c077, 93 },
    { 0xc6d6c078, 95 },
    { 0, 0 },
    { 0x5ee6c07a, 90 },
    { 0x47c6c07a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd266c07e, 61 },
    { 0x8ad6c07e, 93 },
    { 0x9266c080, 77 },
    { 0, 0 },
    { 0x4016c082, 95 },
    { 0xfb76c083, 46 },
    { 0xc386c083, 43 },
    { 0, 0 },
    { 0xef66c086, 98 },
    { 0, 0 },
    { 0xf356c088, 34 },
    { 0x276c089, 55 },
    { 0x8c16c08a, 68 },
    { 0xa066c08b, 61 },
    { 0xcf6c08c, 90 },
    { 0xc4e6c08c, 84 },
    { 0xbac6c08b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c6c091, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586c094, 46 },
    { 0, 0 },
    { 0xec66c096, 85 },
    { 0, 0 },
    { 0x5296c098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe346c09b, 77 },
    { 0xcc6c09b, 90 },
    { 0, 0 },
    { 0x3a6c09e, 68 },
    { 0x6a26c09e, 34 },
    { 0xbfb6c0a0, 33 },
    { 0xf526c0a1, 90 },
    { 0x5876c09e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626c0a7, 33 },
    { 0, 0 },
    { 0x3106c0a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb556c0af, 91 },
    { 0x1b06c0af, 95 },
    { 0, 0 },
    { 0xb0d6c0b2, 98 },
    { 0, 0 },
    { 0xbfc6c0b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6076c0bf, 61 },
    { 0xbe86c0c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c6c0c4, 74 },
    { 0, 0 },
    { 0x5b96c0c6, 46 },
    { 0x7886c0c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd966c0ce, 61 },
    { 0, 0 },
    { 0x256c0d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4686c0d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6c0d9, 95 },
    { 0xf0a6c0da, 98 },
    { 0, 0 },
    { 0x9d06c0dc, 52 },
    { 0xfe66c0dd, 76 },
    { 0x19b6c0de, 36 },
    { 0x2486c0df, 88 },
    { 0x486c0dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8566c0e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6c0ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d56c0f1, 23 },
    { 0x2546c0f2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e6c0f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a6c0fb, 46 },
    { 0xb5b6c0fb, 76 },
    { 0xebb6c0fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4776c102, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc736c105, 60 },
    { 0xadf6c106, 66 },
    { 0, 0 },
    { 0xf756c108, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee06c10b, 33 },
    { 0x76f6c10c, 61 },
    { 0xed66c10d, 34 },
    { 0xbe46c10e, 46 },
    { 0x9c06c10f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf6c112, 75 },
    { 0x8a16c113, 76 },
    { 0xe326c114, 90 },
    { 0xe4d6c115, 76 },
    { 0xc456c116, 34 },
    { 0x40f6c114, 23 },
    { 0x3396c118, 85 },
    { 0x78c6c119, 90 },
    { 0x5a6c11a, 55 },
    { 0xf086c11b, 85 },
    { 0x9976c113, 61 },
    { 0x4b6c11d, 34 },
    { 0x8296c11e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a76c128, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4176c12c, 23 },
    { 0, 0 },
    { 0x9856c12e, 68 },
    { 0x60c6c12e, 78 },
    { 0x46e6c12e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5066c134, 33 },
    { 0xbcb6c135, 55 },
    { 0x3006c135, 33 },
    { 0x1ee6c134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa676c144, 95 },
    { 0, 0 },
    { 0xe816c146, 33 },
    { 0xaaf6c146, 95 },
    { 0, 0 },
    { 0x3ca6c149, 66 },
    { 0x1d06c14a, 87 },
    { 0x2236c149, 33 },
    { 0x6a46c14c, 67 },
    { 0x9da6c14d, 95 },
    { 0xe1c6c14b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc836c154, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c6c157, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6726c15a, 23 },
    { 0x1716c15b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5336c15e, 85 },
    { 0x3486c15f, 61 },
    { 0xc6c6c160, 36 },
    { 0xa5d6c15f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956c167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a6c16c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b06c170, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6c174, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e6c177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1316c17a, 33 },
    { 0, 0 },
    { 0xb006c17c, 90 },
    { 0, 0 },
    { 0xe776c17e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236c183, 36 },
    { 0xc3f6c184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x596c187, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec6c18b, 77 },
    { 0x1db6c18b, 90 },
    { 0x8de6c18d, 91 },
    { 0xeaf6c18b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af6c192, 97 },
    { 0x9b66c193, 80 },
    { 0x63a6c194, 78 },
    { 0x3946c195, 98 },
    { 0xf6c6c196, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b6c19d, 98 },
    { 0x26e6c19d, 91 },
    { 0x3f16c19f, 90 },
    { 0xac46c1a0, 78 },
    { 0xa026c19d, 90 },
    { 0x5ad6c1a2, 33 },
    { 0xb5c6c1a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a56c1a9, 33 },
    { 0, 0 },
    { 0xd016c1ab, 67 },
    { 0xebb6c1ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c6c1b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd506c1b4, 23 },
    { 0, 0 },
    { 0xcca6c1b6, 61 },
    { 0x9426c1b6, 23 },
    { 0xe1d6c1b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb566c1bb, 77 },
    { 0xe086c1bc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb746c1c1, 78 },
    { 0x7066c1c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6c1c7, 95 },
    { 0x4e06c1c7, 61 },
    { 0, 0 },
    { 0x1e76c1ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f66c1cf, 46 },
    { 0x1e6c1cf, 76 },
    { 0, 0 },
    { 0x6266c1d2, 23 },
    { 0x8db6c1d3, 91 },
    { 0x8586c1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336c1d7, 99 },
    { 0x1926c1d8, 46 },
    { 0xcda6c1d8, 98 },
    { 0, 0 },
    { 0x9cb6c1db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056c1e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb76c1e7, 46 },
    { 0xaa36c1e8, 61 },
    { 0x9976c1e9, 90 },
    { 0x1936c1ea, 98 },
    { 0, 0 },
    { 0xdbd6c1ec, 36 },
    { 0x78a6c1ed, 33 },
    { 0x9386c1ee, 46 },
    { 0x7456c1ef, 98 },
    { 0xdd76c1ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd766c1f7, 55 },
    { 0, 0 },
    { 0x19c6c1f9, 61 },
    { 0x2636c1fa, 23 },
    { 0xdd66c1fb, 46 },
    { 0x82d6c1fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c96c200, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b86c206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb16c20c, 34 },
    { 0, 0 },
    { 0xbb26c20e, 91 },
    { 0xa1c6c20f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d46c21a, 84 },
    { 0, 0 },
    { 0xa376c21c, 23 },
    { 0xc7f6c21c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b6c220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c6c223, 43 },
    { 0x7556c224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb456c229, 23 },
    { 0x50e6c22a, 61 },
    { 0xc706c229, 88 },
    { 0xbe26c22c, 91 },
    { 0x9fa6c22b, 85 },
    { 0x3476c229, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d6c231, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1846c234, 34 },
    { 0xa3f6c235, 61 },
    { 0xfb6c236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc176c239, 90 },
    { 0x6716c23a, 23 },
    { 0x4816c23a, 89 },
    { 0xefa6c23c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc876c242, 90 },
    { 0, 0 },
    { 0x2e86c244, 66 },
    { 0xb466c244, 33 },
    { 0xe916c246, 46 },
    { 0x8c6c246, 98 },
    { 0x58e6c248, 52 },
    { 0xb756c245, 95 },
    { 0xc326c249, 75 },
    { 0, 0 },
    { 0xc3a6c24c, 76 },
    { 0xd666c24c, 95 },
    { 0x1606c24e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7286c255, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c6c25c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e6c260, 81 },
    { 0xcda6c261, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e6c264, 90 },
    { 0xe4b6c264, 34 },
    { 0x8676c266, 61 },
    { 0x6836c266, 61 },
    { 0x6656c264, 90 },
    { 0, 0 },
    { 0x8556c26a, 98 },
    { 0x7736c26b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac6c26f, 95 },
    { 0x6fa6c270, 99 },
    { 0x2a46c271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa786c275, 46 },
    { 0xdc36c276, 61 },
    { 0xd6d6c277, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe196c27f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c56c285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4056c28b, 51 },
    { 0x5e16c28c, 68 },
    { 0xd6e6c28d, 90 },
    { 0, 0 },
    { 0x2e06c28f, 76 },
    { 0xed86c28f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a6c293, 75 },
    { 0x4926c294, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xff86c297, 46 },
    { 0x53c6c298, 23 },
    { 0xe286c299, 90 },
    { 0, 0 },
    { 0x2d86c29b, 66 },
    { 0x6286c29c, 23 },
    { 0xecf6c29b, 43 },
    { 0xe276c29c, 23 },
    { 0, 0 },
    { 0x3a36c2a0, 61 },
    { 0x166c2a1, 55 },
    { 0, 0 },
    { 0x9a16c2a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e6c2a8, 33 },
    { 0xeeb6c2a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a6c2af, 61 },
    { 0xb6f6c2b0, 76 },
    { 0x7f86c2b0, 53 },
    { 0x38a6c2b2, 33 },
    { 0x1ed6c2b0, 90 },
    { 0, 0 },
    { 0x526c2b5, 33 },
    { 0xd276c2b5, 43 },
    { 0, 0 },
    { 0x72c6c2b8, 46 },
    { 0xb0a6c2b9, 90 },
    { 0xab76c2b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef6c2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b46c2c3, 61 },
    { 0x9ee6c2c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd6c2c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9946c2cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f26c2cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab6c2d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd16c2d5, 55 },
    { 0xf376c2d5, 66 },
    { 0xd0c6c2d7, 23 },
    { 0, 0 },
    { 0x2746c2d9, 98 },
    { 0xa2d6c2da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd986c2df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8786c2ea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596c2ef, 34 },
    { 0xd256c2f0, 95 },
    { 0x95e6c2f1, 78 },
    { 0x4b16c2f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b86c2f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a6c2fc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc76c300, 83 },
    { 0, 0 },
    { 0x7c46c302, 98 },
    { 0, 0 },
    { 0xdd86c304, 34 },
    { 0xfba6c304, 43 },
    { 0x9e46c306, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f6c309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2056c30f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a6c313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496c318, 98 },
    { 0, 0 },
    { 0xff86c31a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6c31d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc656c322, 90 },
    { 0xb196c323, 78 },
    { 0x5796c324, 78 },
    { 0xc236c325, 43 },
    { 0x9b86c324, 33 },
    { 0x3a66c325, 61 },
    { 0, 0 },
    { 0xc9e6c329, 66 },
    { 0xb856c329, 61 },
    { 0, 0 },
    { 0xd606c32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d6c331, 98 },
    { 0xe936c331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f6c33a, 98 },
    { 0x7876c33b, 78 },
    { 0x5236c33c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7326c340, 69 },
    { 0xa9d6c341, 85 },
    { 0x2cd6c342, 90 },
    { 0, 0 },
    { 0x8b26c344, 61 },
    { 0x1e96c345, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e86c34b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd6c34e, 93 },
    { 0x73e6c34e, 23 },
    { 0, 0 },
    { 0x2b26c351, 98 },
    { 0xb566c351, 90 },
    { 0x4046c352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe366c35c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6526c362, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6c365, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a6c369, 95 },
    { 0x9cd6c36a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1336c36d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c6c370, 90 },
    { 0x7b66c371, 66 },
    { 0xa946c372, 34 },
    { 0xf706c373, 61 },
    { 0x7ec6c374, 23 },
    { 0, 0 },
    { 0x9306c376, 33 },
    { 0x4ed6c377, 76 },
    { 0x2b76c376, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f6c37b, 98 },
    { 0x5976c37c, 65 },
    { 0x3ab6c37c, 61 },
    { 0, 0 },
    { 0xc9a6c37f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d6c383, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316c388, 66 },
    { 0xd2b6c389, 100 },
    { 0, 0 },
    { 0xc996c38b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca6c38e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706c394, 90 },
    { 0, 0 },
    { 0xebb6c396, 81 },
    { 0x3606c397, 61 },
    { 0xf2e6c397, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f6c39c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e06c3a0, 23 },
    { 0, 0 },
    { 0x6536c3a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe736c3a6, 34 },
    { 0xba56c3a7, 88 },
    { 0xd446c3a7, 53 },
    { 0, 0 },
    { 0xb76c3aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816c3ad, 87 },
    { 0x2f36c3ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d86c3b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b66c3b9, 34 },
    { 0x7c56c3ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6c3bf, 98 },
    { 0, 0 },
    { 0x6606c3c1, 23 },
    { 0, 0 },
    { 0xf436c3c3, 75 },
    { 0x1ba6c3c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6706c3cb, 99 },
    { 0xac6c3cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3256c3d0, 99 },
    { 0, 0 },
    { 0xde46c3d2, 33 },
    { 0x9186c3d3, 76 },
    { 0x7826c3d4, 90 },
    { 0xae96c3d3, 34 },
    { 0x3c46c3d6, 99 },
    { 0x7916c3d7, 85 },
    { 0x196c3d8, 85 },
    { 0x5a36c3d8, 23 },
    { 0xdec6c3d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd856c3dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f6c3e0, 98 },
    { 0x18f6c3e1, 98 },
    { 0x26d6c3e2, 39 },
    { 0x8b56c3e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f6c3ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd836c3ef, 61 },
    { 0x2f86c3ef, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906c3f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3256c3fc, 87 },
    { 0xd866c3fd, 84 },
    { 0, 0 },
    { 0xf116c3ff, 92 },
    { 0xd2a6c400, 33 },
    { 0x2566c400, 61 },
    { 0xcd76c3ff, 98 },
    { 0x7c36c403, 95 },
    { 0x556c402, 98 },
    { 0xe616c405, 77 },
    { 0xb3b6c405, 43 },
    { 0xb616c405, 33 },
    { 0xb4b6c408, 61 },
    { 0xa2b6c409, 98 },
    { 0xb0a6c40a, 33 },
    { 0xe7b6c40a, 23 },
    { 0x3266c40c, 95 },
    { 0x9946c40c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c26c410, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e36c416, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c41d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb6c420, 87 },
    { 0x5656c421, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7806c427, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb216c42b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a6c431, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f16c435, 23 },
    { 0x4286c436, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e96c43b, 43 },
    { 0x6036c43c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5516c448, 93 },
    { 0xd336c448, 23 },
    { 0xcec6c44a, 90 },
    { 0xe4b6c44b, 98 },
    { 0, 0 },
    { 0x4546c44d, 23 },
    { 0xbe46c44d, 91 },
    { 0x8136c44d, 75 },
    { 0x98c6c44f, 34 },
    { 0x2df6c44d, 95 },
    { 0x8f16c452, 90 },
    { 0xa196c453, 98 },
    { 0x2a76c454, 33 },
    { 0x7026c454, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6656c45c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b6c461, 66 },
    { 0x516c461, 90 },
    { 0xa6e6c463, 87 },
    { 0x1d26c464, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3276c46f, 23 },
    { 0, 0 },
    { 0x52d6c471, 98 },
    { 0x4b06c471, 23 },
    { 0x4dd6c473, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6c477, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a06c47f, 90 },
    { 0xba6c480, 85 },
    { 0xc5c6c481, 40 },
    { 0x1ed6c482, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9816c48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c06c490, 77 },
    { 0, 0 },
    { 0x97b6c492, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb566c498, 23 },
    { 0, 0 },
    { 0xff36c49a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c46c4a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6c4a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3466c4a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e6c4ab, 84 },
    { 0, 0 },
    { 0xfe16c4ad, 76 },
    { 0xb566c4ae, 52 },
    { 0xf706c4ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe986c4b2, 99 },
    { 0x4d16c4b3, 33 },
    { 0, 0 },
    { 0x4966c4b5, 61 },
    { 0x8656c4b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce6c4c5, 34 },
    { 0x7936c4c6, 90 },
    { 0x4566c4c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f6c4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e6c4d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456c4d4, 87 },
    { 0, 0 },
    { 0x7196c4d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e6c4dc, 33 },
    { 0, 0 },
    { 0x71d6c4de, 52 },
    { 0xa5d6c4df, 40 },
    { 0xc906c4df, 61 },
    { 0x3d76c4df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6c4e4, 33 },
    { 0x8a46c4e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46c4e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c6c4ec, 33 },
    { 0xdce6c4ed, 90 },
    { 0x9166c4ec, 46 },
    { 0, 0 },
    { 0x2f16c4f0, 23 },
    { 0x6676c4f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea26c4f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d6c501, 53 },
    { 0x8446c502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06c508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe496c511, 90 },
    { 0x2786c511, 33 },
    { 0x2836c512, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee76c516, 33 },
    { 0x22b6c516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d6c51c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5016c51f, 88 },
    { 0, 0 },
    { 0xde16c521, 66 },
    { 0, 0 },
    { 0x37c6c523, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf706c526, 66 },
    { 0, 0 },
    { 0xca26c528, 23 },
    { 0x3ca6c529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4846c52c, 46 },
    { 0, 0 },
    { 0x1596c52e, 61 },
    { 0x1dd6c52e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb16c538, 98 },
    { 0, 0 },
    { 0x54b6c53a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2456c53d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd6c540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276c547, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e66c54a, 34 },
    { 0, 0 },
    { 0x9516c54c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f6c550, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb526c554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e6c559, 33 },
    { 0x3cf6c55a, 90 },
    { 0xb3d6c55a, 81 },
    { 0x206c55c, 66 },
    { 0xe406c55c, 23 },
    { 0x5536c55a, 95 },
    { 0xb956c55f, 90 },
    { 0, 0 },
    { 0xce46c561, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca6c564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe546c567, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6c56c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f6c571, 76 },
    { 0x7376c572, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e46c57b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2356c57e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc6c581, 33 },
    { 0, 0 },
    { 0xd2d6c583, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a26c587, 52 },
    { 0x906c588, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b6c591, 76 },
    { 0xd126c591, 76 },
    { 0x41f6c593, 33 },
    { 0xb646c594, 95 },
    { 0xfa66c595, 87 },
    { 0x4c36c596, 81 },
    { 0xf426c596, 55 },
    { 0x6616c598, 33 },
    { 0x6b26c596, 98 },
    { 0, 0 },
    { 0x80f6c59b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc916c5aa, 84 },
    { 0x3a76c5ab, 87 },
    { 0, 0 },
    { 0x61f6c5ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d86c5b1, 95 },
    { 0, 0 },
    { 0xae6c5b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed6c5b8, 78 },
    { 0xe346c5b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf6c5bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd6c5c5, 46 },
    { 0xd8d6c5c5, 23 },
    { 0x2836c5c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b6c5ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9576c5ce, 61 },
    { 0, 0 },
    { 0x1206c5d0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd6c5d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa816c5d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe016c5de, 75 },
    { 0xa726c5df, 77 },
    { 0x7d56c5e0, 23 },
    { 0, 0 },
    { 0xdff6c5e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda26c5e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec06c5ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e86c5f3, 98 },
    { 0xa666c5f4, 100 },
    { 0x5806c5f3, 55 },
    { 0x9526c5f4, 67 },
    { 0x84b6c5f7, 66 },
    { 0, 0 },
    { 0x7ab6c5f9, 98 },
    { 0xcab6c5f9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436c5fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb56c601, 90 },
    { 0x5796c601, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb806c605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e26c60a, 66 },
    { 0x3716c60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf96c60f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9556c615, 17 },
    { 0, 0 },
    { 0x9b76c617, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9216c61d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f36c622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d6c627, 98 },
    { 0, 0 },
    { 0x2906c629, 90 },
    { 0xc5f6c62a, 96 },
    { 0x3c16c62b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa966c631, 23 },
    { 0x8ab6c632, 98 },
    { 0, 0 },
    { 0x3236c634, 90 },
    { 0, 0 },
    { 0xcbd6c636, 90 },
    { 0, 0 },
    { 0x64c6c638, 91 },
    { 0xf7f6c639, 43 },
    { 0xc5b6c63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe6c63e, 33 },
    { 0x3066c63f, 98 },
    { 0, 0 },
    { 0xd9f6c641, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f46c646, 98 },
    { 0xbc36c647, 98 },
    { 0x3d86c648, 84 },
    { 0xae46c649, 95 },
    { 0x6336c64a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7586c64e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c6c652, 87 },
    { 0x6086c652, 61 },
    { 0x29a6c654, 65 },
    { 0x5836c652, 98 },
    { 0x6c46c656, 98 },
    { 0xdbd6c652, 98 },
    { 0x98e6c654, 90 },
    { 0xb5f6c654, 17 },
    { 0x60e6c65a, 61 },
    { 0x1206c65b, 61 },
    { 0x2d36c65c, 78 },
    { 0xe846c659, 61 },
    { 0x3a86c65e, 90 },
    { 0x3696c65f, 78 },
    { 0x26e6c660, 90 },
    { 0x1cf6c65f, 61 },
    { 0, 0 },
    { 0x4206c663, 95 },
    { 0x7ce6c664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6c668, 98 },
    { 0xfb66c669, 66 },
    { 0x67f6c66a, 25 },
    { 0xbc06c669, 65 },
    { 0x8956c66b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb6c670, 84 },
    { 0x6556c671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8936c674, 95 },
    { 0x7656c675, 33 },
    { 0x1286c676, 98 },
    { 0x40b6c677, 34 },
    { 0xa266c677, 84 },
    { 0, 0 },
    { 0x6cc6c67a, 23 },
    { 0, 0 },
    { 0xccd6c67c, 61 },
    { 0xe316c67d, 55 },
    { 0x67b6c67d, 34 },
    { 0x36b6c67c, 99 },
    { 0, 0 },
    { 0x2c26c681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8156c684, 39 },
    { 0, 0 },
    { 0xbb16c686, 23 },
    { 0, 0 },
    { 0xf9b6c688, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6c68d, 23 },
    { 0xa8f6c68e, 93 },
    { 0x1336c68f, 90 },
    { 0x456c68f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe556c693, 90 },
    { 0x5e36c694, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336c698, 90 },
    { 0x9a6c699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b46c6a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6526c6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb306c6a8, 77 },
    { 0x4926c6a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2116c6b1, 65 },
    { 0xe316c6b2, 46 },
    { 0xbac6c6b3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe636c6b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd186c6bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c46c6c3, 33 },
    { 0xbd56c6c4, 61 },
    { 0, 0 },
    { 0xe616c6c6, 34 },
    { 0x12a6c6c7, 98 },
    { 0, 0 },
    { 0x8476c6c9, 43 },
    { 0xe236c6ca, 76 },
    { 0x8c96c6cb, 33 },
    { 0x9b46c6ca, 76 },
    { 0x12e6c6cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df6c6d0, 98 },
    { 0, 0 },
    { 0xe236c6d2, 81 },
    { 0xf856c6d3, 46 },
    { 0x846c6d2, 100 },
    { 0x6fc6c6d5, 90 },
    { 0x6916c6d6, 30 },
    { 0x59a6c6d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f66c6da, 17 },
    { 0, 0 },
    { 0xc006c6dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1046c6e4, 61 },
    { 0x3e16c6e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c6c6e8, 33 },
    { 0x24a6c6e8, 34 },
    { 0xec06c6ea, 98 },
    { 0x2df6c6eb, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f56c6f3, 53 },
    { 0xc9d6c6f4, 33 },
    { 0xe626c6f5, 76 },
    { 0x8336c6f5, 23 },
    { 0, 0 },
    { 0x3bf6c6f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf56c6fb, 95 },
    { 0xaf16c6fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d16c702, 34 },
    { 0xa7a6c703, 61 },
    { 0x35c6c704, 84 },
    { 0, 0 },
    { 0xa416c706, 76 },
    { 0x6996c707, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b6c711, 23 },
    { 0x43b6c712, 90 },
    { 0x1da6c713, 33 },
    { 0, 0 },
    { 0x56e6c715, 93 },
    { 0, 0 },
    { 0x8336c717, 23 },
    { 0xb4e6c718, 55 },
    { 0xa2b6c719, 55 },
    { 0x1476c71a, 17 },
    { 0x8ff6c71a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb416c71e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8086c721, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6726c725, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea46c728, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee6c72b, 95 },
    { 0xb8d6c72c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c6c731, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b6c736, 75 },
    { 0x6796c736, 95 },
    { 0xdc66c738, 23 },
    { 0, 0 },
    { 0xeb56c73a, 95 },
    { 0, 0 },
    { 0xe776c73c, 61 },
    { 0, 0 },
    { 0xf716c73e, 90 },
    { 0xeb56c73f, 98 },
    { 0, 0 },
    { 0xb7b6c741, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf396c745, 23 },
    { 0, 0 },
    { 0x47d6c747, 98 },
    { 0, 0 },
    { 0x99d6c749, 23 },
    { 0, 0 },
    { 0x36b6c74b, 23 },
    { 0x5996c74c, 87 },
    { 0, 0 },
    { 0x8cf6c74e, 23 },
    { 0x57f6c74f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66c753, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566c762, 33 },
    { 0x8d66c763, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe746c766, 61 },
    { 0xfc6c767, 52 },
    { 0, 0 },
    { 0x7186c769, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6c770, 95 },
    { 0xe5d6c771, 78 },
    { 0x6486c772, 95 },
    { 0, 0 },
    { 0xe6c774, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a6c777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e6c77e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be6c786, 76 },
    { 0x8dc6c787, 95 },
    { 0xb526c788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc666c791, 23 },
    { 0, 0 },
    { 0xce06c793, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac46c798, 100 },
    { 0, 0 },
    { 0x9546c79a, 23 },
    { 0x7be6c79b, 61 },
    { 0xa046c79b, 34 },
    { 0xdbf6c79c, 61 },
    { 0x4cd6c79e, 78 },
    { 0x69b6c79f, 76 },
    { 0xf8d6c79b, 90 },
    { 0, 0 },
    { 0xe906c7a2, 98 },
    { 0xd336c7a3, 33 },
    { 0x41b6c7a4, 55 },
    { 0x1f36c7a5, 99 },
    { 0x38f6c7a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf06c7a9, 98 },
    { 0xccc6c7a9, 95 },
    { 0xb556c7aa, 95 },
    { 0x3866c7ac, 75 },
    { 0x8926c7ad, 23 },
    { 0xa986c7ae, 95 },
    { 0x3a6c7af, 85 },
    { 0x2ac6c7b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9506c7b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa46c7b7, 23 },
    { 0x23d6c7b7, 33 },
    { 0, 0 },
    { 0x7526c7ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f6c7bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16c7c2, 61 },
    { 0xee86c7c2, 97 },
    { 0, 0 },
    { 0x6376c7c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1976c7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d36c7cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b66c7ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf6c7d1, 43 },
    { 0xf9f6c7d1, 55 },
    { 0x9896c7d3, 90 },
    { 0x4976c7d4, 51 },
    { 0x3796c7d5, 95 },
    { 0x5c56c7d3, 38 },
    { 0, 0 },
    { 0xc9e6c7d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6c7df, 84 },
    { 0x1496c7e0, 77 },
    { 0x24c6c7e0, 34 },
    { 0x5826c7df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba06c7e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc076c7ed, 90 },
    { 0x2966c7ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf6c7f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb716c7fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa976c802, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b16c806, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a6c809, 77 },
    { 0, 0 },
    { 0x3286c80b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56c810, 67 },
    { 0x1296c811, 33 },
    { 0, 0 },
    { 0xeed6c813, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736c81b, 36 },
    { 0x6bb6c81c, 43 },
    { 0xe986c81d, 95 },
    { 0xa5a6c81e, 87 },
    { 0x5486c81d, 46 },
    { 0xd166c81d, 95 },
    { 0x89a6c821, 80 },
    { 0x92e6c821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3346c826, 61 },
    { 0x40b6c827, 90 },
    { 0xc426c828, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f6c82f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c6c833, 55 },
    { 0, 0 },
    { 0x6d36c835, 23 },
    { 0xe306c836, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c6c83c, 43 },
    { 0x2266c83c, 33 },
    { 0xc516c83c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e6c846, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b6c849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6c84e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1616c851, 90 },
    { 0x5736c852, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6c857, 55 },
    { 0, 0 },
    { 0xdd66c859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9756c85e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9646c862, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9216c86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c26c871, 67 },
    { 0x7896c871, 76 },
    { 0, 0 },
    { 0xd506c874, 95 },
    { 0, 0 },
    { 0xd96c876, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b6c879, 98 },
    { 0, 0 },
    { 0xaec6c87b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb56c886, 36 },
    { 0, 0 },
    { 0x5a66c888, 90 },
    { 0, 0 },
    { 0x316c88a, 66 },
    { 0, 0 },
    { 0x67d6c88c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7076c88f, 78 },
    { 0, 0 },
    { 0x2e56c891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3806c894, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f6c898, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde86c8a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb6c8ae, 52 },
    { 0xd676c8af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b6c8b2, 98 },
    { 0x3a06c8b3, 61 },
    { 0xbd56c8b4, 85 },
    { 0, 0 },
    { 0x6a96c8b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f6c8ba, 33 },
    { 0x7566c8bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c6c8c0, 23 },
    { 0xc4b6c8c1, 60 },
    { 0x9146c8c2, 100 },
    { 0x2e36c8c0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6776c8c6, 98 },
    { 0xf9c6c8c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa6c8d0, 56 },
    { 0x9816c8d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3736c8d4, 98 },
    { 0, 0 },
    { 0x81c6c8d6, 33 },
    { 0xa76c8d7, 95 },
    { 0xca26c8d6, 67 },
    { 0, 0 },
    { 0xb106c8da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16c8dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6356c8e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a26c8ea, 81 },
    { 0, 0 },
    { 0x9326c8ec, 76 },
    { 0xcac6c8ed, 90 },
    { 0x9ad6c8ee, 90 },
    { 0x1d06c8ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9086c8f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1226c8f7, 98 },
    { 0x926c8f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e6c902, 61 },
    { 0, 0 },
    { 0x3d76c904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5736c908, 78 },
    { 0x64d6c909, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa706c911, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a66c914, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe836c919, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56c921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc6c924, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d6c927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e6c92c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6866c936, 33 },
    { 0x9526c937, 95 },
    { 0xb8b6c938, 40 },
    { 0xac86c937, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456c93d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1006c940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa396c943, 33 },
    { 0, 0 },
    { 0xf886c945, 43 },
    { 0x9a6c946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd116c94a, 33 },
    { 0xdbb6c94b, 67 },
    { 0x3af6c94b, 90 },
    { 0xbb76c94a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb846c958, 61 },
    { 0xaff6c959, 33 },
    { 0, 0 },
    { 0x7276c95b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c56c960, 38 },
    { 0, 0 },
    { 0x50f6c962, 28 },
    { 0x5e66c962, 76 },
    { 0x21b6c964, 90 },
    { 0xf0a6c963, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be6c968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e6c96b, 33 },
    { 0xb066c96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b6c96f, 76 },
    { 0, 0 },
    { 0x6ed6c971, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3146c975, 76 },
    { 0xbee6c975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb26c979, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e6c97c, 91 },
    { 0x7c66c97d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad6c981, 95 },
    { 0, 0 },
    { 0x7726c983, 33 },
    { 0xc906c984, 81 },
    { 0x4d96c984, 52 },
    { 0x6216c986, 55 },
    { 0xe4e6c987, 33 },
    { 0xd646c987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc856c98c, 93 },
    { 0x6986c98d, 87 },
    { 0x7806c98e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8976c994, 23 },
    { 0xf556c995, 41 },
    { 0x5bf6c994, 33 },
    { 0x9136c996, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c26c99f, 99 },
    { 0xa236c99f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6536c9ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c66c9b0, 9 },
    { 0, 0 },
    { 0xa356c9b2, 43 },
    { 0xeb86c9b2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f16c9b7, 90 },
    { 0x8dd6c9b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb76c9bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa806c9be, 90 },
    { 0xd576c9bf, 46 },
    { 0x4e36c9bf, 33 },
    { 0xa0f6c9bf, 75 },
    { 0xeb16c9bf, 99 },
    { 0xab26c9c3, 98 },
    { 0, 0 },
    { 0x8a86c9c5, 76 },
    { 0x8d46c9c6, 75 },
    { 0x6136c9c7, 78 },
    { 0x8e96c9c7, 90 },
    { 0x6366c9c9, 46 },
    { 0x4406c9c9, 81 },
    { 0xd206c9c9, 90 },
    { 0x1e46c9c8, 73 },
    { 0x86f6c9cb, 95 },
    { 0xcbd6c9ce, 34 },
    { 0x7bf6c9ce, 95 },
    { 0xa826c9cb, 84 },
    { 0x39e6c9d1, 23 },
    { 0x6316c9d2, 91 },
    { 0x8296c9d3, 61 },
    { 0xccb6c9d4, 61 },
    { 0x9456c9cc, 90 },
    { 0xe496c9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd466c9e0, 95 },
    { 0x91b6c9e1, 90 },
    { 0, 0 },
    { 0x6616c9e3, 61 },
    { 0x5de6c9e4, 90 },
    { 0x4f96c9e4, 68 },
    { 0x7636c9e6, 46 },
    { 0x68b6c9e6, 43 },
    { 0x6ab6c9e5, 61 },
    { 0xd2e6c9e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee26c9ec, 33 },
    { 0x3506c9ed, 61 },
    { 0, 0 },
    { 0x8e36c9ef, 69 },
    { 0x866c9f0, 76 },
    { 0xb536c9f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa6c9f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326c9fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc146ca06, 93 },
    { 0, 0 },
    { 0xc86ca08, 76 },
    { 0, 0 },
    { 0xe5d6ca0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca6ca0d, 33 },
    { 0, 0 },
    { 0x7276ca0f, 90 },
    { 0xe9c6ca10, 84 },
    { 0, 0 },
    { 0x426ca12, 23 },
    { 0x6c6ca13, 33 },
    { 0, 0 },
    { 0xc956ca15, 85 },
    { 0x1d76ca16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4046ca21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6ca26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76ca2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ca33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc06ca37, 23 },
    { 0, 0 },
    { 0xf6b6ca39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2616ca3c, 33 },
    { 0x4326ca3d, 66 },
    { 0x2606ca3c, 61 },
    { 0x2916ca3f, 55 },
    { 0xce6ca40, 90 },
    { 0xdc86ca41, 34 },
    { 0, 0 },
    { 0xddf6ca43, 90 },
    { 0x17e6ca44, 61 },
    { 0x8e96ca43, 23 },
    { 0, 0 },
    { 0x26e6ca47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd26ca4b, 34 },
    { 0, 0 },
    { 0xb326ca4d, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36ca54, 95 },
    { 0xe1d6ca55, 75 },
    { 0xd36ca55, 68 },
    { 0x2146ca57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec6ca5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd6ca5e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f16ca61, 77 },
    { 0xc856ca62, 93 },
    { 0, 0 },
    { 0x46a6ca64, 61 },
    { 0x8846ca64, 61 },
    { 0x15c6ca66, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6ca6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb306ca72, 46 },
    { 0x8016ca72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956ca78, 61 },
    { 0x77f6ca79, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6366ca7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2836ca82, 61 },
    { 0x9de6ca82, 95 },
    { 0xafb6ca83, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6ca8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6ca90, 23 },
    { 0xb496ca91, 33 },
    { 0x1156ca92, 95 },
    { 0x1036ca91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f6ca96, 90 },
    { 0x9656ca97, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b6ca9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7116ca9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb636caa1, 46 },
    { 0x9ad6caa2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd086caa6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe746caaa, 17 },
    { 0xa116caab, 55 },
    { 0xf5b6caac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b6caaf, 23 },
    { 0x3b96cab0, 95 },
    { 0, 0 },
    { 0x9456cab2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f6cab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa876caba, 33 },
    { 0, 0 },
    { 0xcc6cabc, 90 },
    { 0xfe86cabc, 77 },
    { 0x80a6cabc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe676cac3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6cac6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a76cacb, 77 },
    { 0x7496cacc, 93 },
    { 0, 0 },
    { 0x79f6cace, 34 },
    { 0x4276cacf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe6cada, 23 },
    { 0x9f26cadb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a6cae3, 98 },
    { 0xc576cae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4586cae7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e6caea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86caee, 98 },
    { 0, 0 },
    { 0xbed6caf0, 100 },
    { 0, 0 },
    { 0x2876caf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2786cafa, 95 },
    { 0xfc66cafb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x846cafe, 77 },
    { 0, 0 },
    { 0x15c6cb00, 77 },
    { 0x49c6cb01, 55 },
    { 0x396cb01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de6cb06, 23 },
    { 0x7ce6cb07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4316cb0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a86cb11, 87 },
    { 0x4396cb11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd626cb15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6cb18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b6cb1b, 34 },
    { 0, 0 },
    { 0xd896cb1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6716cb32, 33 },
    { 0x5396cb33, 33 },
    { 0x2236cb33, 23 },
    { 0x226cb35, 98 },
    { 0x4056cb36, 91 },
    { 0xab16cb37, 43 },
    { 0xd2b6cb38, 33 },
    { 0xe176cb39, 61 },
    { 0xccb6cb34, 67 },
    { 0xaee6cb38, 96 },
    { 0xa426cb33, 38 },
    { 0, 0 },
    { 0xf416cb3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1936cb41, 93 },
    { 0x6916cb42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe6cb46, 35 },
    { 0x4d66cb47, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e6cb4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a36cb4e, 33 },
    { 0xb546cb4f, 33 },
    { 0xa356cb4e, 95 },
    { 0x3276cb50, 95 },
    { 0xcb16cb52, 34 },
    { 0, 0 },
    { 0x5576cb54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf46cb57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e76cb5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e26cb5f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa846cb64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b6cb67, 90 },
    { 0xb8e6cb67, 55 },
    { 0x6b66cb69, 75 },
    { 0x5146cb68, 23 },
    { 0x2426cb69, 98 },
    { 0x5066cb67, 76 },
    { 0x4716cb6d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade6cb72, 46 },
    { 0xdea6cb73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e6cb7a, 76 },
    { 0x116cb7a, 90 },
    { 0x10e6cb7c, 83 },
    { 0xcb76cb7d, 55 },
    { 0, 0 },
    { 0xbba6cb7f, 75 },
    { 0xf906cb80, 33 },
    { 0x1966cb81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8286cb84, 81 },
    { 0, 0 },
    { 0xbeb6cb86, 46 },
    { 0, 0 },
    { 0xca46cb88, 8 },
    { 0xe6d6cb89, 76 },
    { 0xc396cb8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d36cb8f, 61 },
    { 0xf1a6cb90, 23 },
    { 0, 0 },
    { 0xf5e6cb92, 33 },
    { 0, 0 },
    { 0xd256cb94, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de6cb98, 99 },
    { 0x5e96cb99, 61 },
    { 0, 0 },
    { 0xc566cb9b, 95 },
    { 0, 0 },
    { 0x1f96cb9d, 23 },
    { 0xb4d6cb9d, 33 },
    { 0xe546cb9d, 34 },
    { 0x5db6cb9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c6cba7, 96 },
    { 0, 0 },
    { 0xd296cba9, 67 },
    { 0xd036cbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb706cbad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af6cbb1, 85 },
    { 0x9a56cbb2, 84 },
    { 0xb546cbb2, 77 },
    { 0xa6c6cbb1, 84 },
    { 0x2a06cbb5, 23 },
    { 0, 0 },
    { 0x8f66cbb7, 90 },
    { 0, 0 },
    { 0xee56cbb9, 76 },
    { 0x2bd6cbb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2036cbbe, 33 },
    { 0xf76cbbe, 87 },
    { 0xf216cbbf, 100 },
    { 0, 0 },
    { 0x7056cbc2, 34 },
    { 0xe046cbc3, 23 },
    { 0, 0 },
    { 0xc3a6cbc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab6cbc9, 55 },
    { 0x1966cbca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd266cbd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316cbd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d6cbda, 55 },
    { 0xd256cbdb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f6cbdf, 52 },
    { 0x5ce6cbdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4026cbe8, 91 },
    { 0x8676cbe8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea6cbec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb796cbef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f6cbf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae6cbf5, 93 },
    { 0x9066cbf5, 99 },
    { 0xd6b6cbf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a66cbfb, 34 },
    { 0, 0 },
    { 0xbb46cbfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2676cc00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e46cc04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5566cc07, 98 },
    { 0, 0 },
    { 0x91b6cc09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa226cc0f, 78 },
    { 0xe8b6cc10, 61 },
    { 0x4d66cc11, 34 },
    { 0, 0 },
    { 0x52a6cc13, 87 },
    { 0, 0 },
    { 0x316cc15, 46 },
    { 0, 0 },
    { 0xc736cc17, 61 },
    { 0x4a46cc17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e46cc1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e86cc28, 67 },
    { 0, 0 },
    { 0xf9a6cc2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f6cc2d, 23 },
    { 0, 0 },
    { 0x8fe6cc2f, 76 },
    { 0x90a6cc2f, 43 },
    { 0x13c6cc30, 68 },
    { 0xa936cc31, 98 },
    { 0x4976cc33, 95 },
    { 0, 0 },
    { 0x1196cc35, 55 },
    { 0x5a6cc36, 90 },
    { 0x2116cc37, 23 },
    { 0x43e6cc36, 33 },
    { 0x8616cc35, 61 },
    { 0x99c6cc37, 46 },
    { 0, 0 },
    { 0xdc6cc3c, 40 },
    { 0x6026cc3d, 98 },
    { 0x7856cc3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376cc43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c6cc46, 98 },
    { 0xa816cc47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6776cc4a, 95 },
    { 0, 0 },
    { 0x1e66cc4c, 33 },
    { 0, 0 },
    { 0xa2f6cc4e, 33 },
    { 0x4536cc4f, 40 },
    { 0xbe26cc4f, 76 },
    { 0x4006cc51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6cc54, 43 },
    { 0x3a56cc54, 23 },
    { 0, 0 },
    { 0xe436cc57, 90 },
    { 0xb856cc57, 33 },
    { 0, 0 },
    { 0x9a36cc5a, 75 },
    { 0x8f46cc5b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc386cc61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb6cc6d, 52 },
    { 0x7b76cc6e, 52 },
    { 0x91d6cc6f, 76 },
    { 0x2e56cc70, 91 },
    { 0x43e6cc6e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd96cc74, 77 },
    { 0x34b6cc74, 90 },
    { 0xe746cc76, 85 },
    { 0xeb46cc74, 84 },
    { 0, 0 },
    { 0x7cb6cc79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf246cc7c, 66 },
    { 0x58f6cc7d, 90 },
    { 0, 0 },
    { 0x6516cc7f, 55 },
    { 0x8e76cc80, 33 },
    { 0x83a6cc80, 61 },
    { 0x8286cc82, 98 },
    { 0, 0 },
    { 0x2596cc84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a6cc89, 88 },
    { 0, 0 },
    { 0x5796cc8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f16cc96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b16cc99, 98 },
    { 0, 0 },
    { 0x5e56cc9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b6cc9f, 76 },
    { 0, 0 },
    { 0xbdc6cca1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac16cca5, 55 },
    { 0xcaa6cca6, 90 },
    { 0xdaa6cca6, 98 },
    { 0x8326cca8, 90 },
    { 0xc716cca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2946ccb1, 90 },
    { 0, 0 },
    { 0x8dc6ccb3, 95 },
    { 0, 0 },
    { 0xb706ccb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7776ccc9, 90 },
    { 0x67d6ccca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e6cccd, 33 },
    { 0, 0 },
    { 0xfb86cccf, 99 },
    { 0x8766ccd0, 61 },
    { 0xea96ccd1, 98 },
    { 0xa126ccd0, 33 },
    { 0xe7f6ccd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c6ccd8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8946ccdb, 33 },
    { 0, 0 },
    { 0x5cd6ccdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6cce0, 98 },
    { 0xc616cce0, 46 },
    { 0x33e6cce2, 80 },
    { 0, 0 },
    { 0x4956cce4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb6cce7, 90 },
    { 0x58a6cce8, 23 },
    { 0x37b6cce9, 44 },
    { 0x74a6cce7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e6ccf3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6ccf6, 34 },
    { 0x1956ccf6, 90 },
    { 0x25d6ccf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae16ccfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6cd02, 95 },
    { 0, 0 },
    { 0x7206cd04, 61 },
    { 0x8486cd04, 90 },
    { 0xdf96cd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea6cd09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826cd0e, 95 },
    { 0, 0 },
    { 0x7ef6cd10, 95 },
    { 0x3b26cd11, 46 },
    { 0, 0 },
    { 0xa596cd13, 90 },
    { 0, 0 },
    { 0xb8a6cd15, 91 },
    { 0xc676cd15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6cd1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f86cd22, 55 },
    { 0xcb46cd22, 23 },
    { 0x4c76cd24, 90 },
    { 0x3236cd25, 34 },
    { 0, 0 },
    { 0x9626cd27, 77 },
    { 0x5e26cd28, 100 },
    { 0, 0 },
    { 0x9a66cd2a, 81 },
    { 0x2fb6cd2b, 46 },
    { 0, 0 },
    { 0x1d6cd2d, 67 },
    { 0x79f6cd2d, 33 },
    { 0, 0 },
    { 0x6366cd30, 75 },
    { 0xd196cd31, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956cd35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be6cd3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe686cd42, 19 },
    { 0, 0 },
    { 0xd8f6cd44, 33 },
    { 0xa2d6cd44, 37 },
    { 0xbf86cd46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d96cd4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7256cd53, 23 },
    { 0, 0 },
    { 0x7096cd55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf6cd59, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9226cd66, 98 },
    { 0x9d66cd66, 23 },
    { 0x3846cd67, 95 },
    { 0xf486cd67, 61 },
    { 0x1f56cd6a, 78 },
    { 0x57d6cd6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a6cd72, 65 },
    { 0x7876cd72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1186cd76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca6cd80, 76 },
    { 0xb026cd80, 81 },
    { 0x3086cd82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba6cd86, 78 },
    { 0x4076cd87, 23 },
    { 0x2cd6cd86, 95 },
    { 0x4a6cd89, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc16cd8c, 56 },
    { 0x7c56cd8c, 90 },
    { 0xa76cd8c, 99 },
    { 0x5916cd8e, 90 },
    { 0, 0 },
    { 0xdf76cd91, 23 },
    { 0x1ce6cd92, 38 },
    { 0, 0 },
    { 0x1c86cd94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a66cd9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f6cda0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96cda4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956cda7, 99 },
    { 0x9db6cda8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e36cdab, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d6cdaf, 61 },
    { 0x6d6cdb0, 23 },
    { 0, 0 },
    { 0x8e96cdb2, 33 },
    { 0, 0 },
    { 0x5606cdb4, 61 },
    { 0x99b6cdb5, 61 },
    { 0x9286cdb5, 33 },
    { 0, 0 },
    { 0x5246cdb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6cdbb, 23 },
    { 0x8376cdbb, 33 },
    { 0x29f6cdbd, 61 },
    { 0x7c86cdbe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d86cdc1, 33 },
    { 0x48e6cdc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf956cdc6, 23 },
    { 0xe666cdc6, 61 },
    { 0x16a6cdc6, 23 },
    { 0x9a96cdc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef6cdcf, 90 },
    { 0xd2a6cdd0, 33 },
    { 0xa936cdcf, 101 },
    { 0x42b6cdd2, 66 },
    { 0x3266cdcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846cdd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56cdda, 95 },
    { 0x37c6cdda, 61 },
    { 0xda36cddc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff6cde4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6cdeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e6cdf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1026cdfb, 98 },
    { 0, 0 },
    { 0xf576cdfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4486ce01, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff6ce0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f6ce0d, 76 },
    { 0x1d56ce0e, 23 },
    { 0x2486ce0f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe6ce13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226ce18, 98 },
    { 0xb4d6ce19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc6ce1d, 46 },
    { 0, 0 },
    { 0xc406ce1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d6ce22, 90 },
    { 0, 0 },
    { 0xf396ce24, 46 },
    { 0, 0 },
    { 0x2a16ce26, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d06ce2d, 17 },
    { 0x45a6ce2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec6ce31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ce35, 93 },
    { 0, 0 },
    { 0x8cc6ce37, 43 },
    { 0, 0 },
    { 0x6056ce39, 55 },
    { 0, 0 },
    { 0x2c36ce3b, 81 },
    { 0x9ae6ce3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6ce42, 23 },
    { 0, 0 },
    { 0x88d6ce44, 90 },
    { 0, 0 },
    { 0x7176ce46, 43 },
    { 0xc826ce46, 90 },
    { 0x1ff6ce48, 90 },
    { 0x9c6ce49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e6ce4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7596ce50, 17 },
    { 0xa396ce51, 76 },
    { 0x5846ce51, 90 },
    { 0xcb6ce51, 98 },
    { 0, 0 },
    { 0xb576ce55, 90 },
    { 0xd016ce56, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d6ce59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ce5c, 34 },
    { 0, 0 },
    { 0xae06ce5e, 94 },
    { 0x6756ce5f, 83 },
    { 0x1b56ce5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8236ce64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf776ce67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea6ce6a, 17 },
    { 0x4356ce6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6646ce6e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516ce72, 90 },
    { 0, 0 },
    { 0x25b6ce74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe766ce79, 98 },
    { 0xeac6ce7a, 85 },
    { 0x88a6ce79, 33 },
    { 0xcdb6ce7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ce84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f6ce87, 76 },
    { 0, 0 },
    { 0x5e46ce89, 61 },
    { 0xcfb6ce89, 93 },
    { 0xf766ce8a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a56ce8e, 93 },
    { 0x8126ce8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b66ce92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc726ce95, 99 },
    { 0x5186ce96, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d6ce9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56ce9d, 76 },
    { 0x736ce9e, 69 },
    { 0x946ce9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06cea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586cea5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d46cea8, 75 },
    { 0x63d6cea8, 46 },
    { 0x8bc6cea9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8916ceae, 66 },
    { 0x8b26ceaf, 17 },
    { 0, 0 },
    { 0x296ceb1, 33 },
    { 0x5626ceb1, 90 },
    { 0x8bf6ceb3, 46 },
    { 0x7136ceb4, 52 },
    { 0x5896ceb2, 33 },
    { 0x9eb6ceb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb136ceba, 23 },
    { 0x9e26cebb, 33 },
    { 0, 0 },
    { 0x87f6cebd, 66 },
    { 0x7996cebe, 33 },
    { 0xe406cebf, 67 },
    { 0x8316cebf, 61 },
    { 0xb206cec1, 90 },
    { 0xf4f6cec1, 33 },
    { 0x1856cec3, 95 },
    { 0x38c6cec3, 90 },
    { 0x7d6cec2, 33 },
    { 0xfd16cec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6cec9, 84 },
    { 0x79b6ceca, 33 },
    { 0x1946cecb, 61 },
    { 0xcec6cec9, 90 },
    { 0x10d6cecd, 67 },
    { 0, 0 },
    { 0xdf46cecf, 95 },
    { 0xe586ced0, 93 },
    { 0, 0 },
    { 0xba46ced2, 23 },
    { 0x5276ced3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d6cedc, 33 },
    { 0xd046cedd, 95 },
    { 0x54b6cede, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f6cee2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa156cee8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd6ceed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc966cef1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe936cef5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6cefc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab6cf00, 98 },
    { 0x61b6cf01, 43 },
    { 0, 0 },
    { 0x1bc6cf03, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b6cf0c, 61 },
    { 0xe726cf0d, 84 },
    { 0x8c36cf0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c6cf11, 46 },
    { 0, 0 },
    { 0x4cb6cf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8586cf18, 61 },
    { 0x5506cf19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb076cf21, 55 },
    { 0x64c6cf21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6cf27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6116cf2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa246cf2f, 76 },
    { 0x8996cf30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc366cf33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf056cf37, 90 },
    { 0xb696cf38, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a96cf3c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b6cf41, 87 },
    { 0, 0 },
    { 0x88b6cf43, 76 },
    { 0x67b6cf43, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f66cf49, 98 },
    { 0xcc56cf4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4326cf4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf376cf50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f6cf53, 78 },
    { 0x9e86cf54, 61 },
    { 0x6206cf53, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa166cf5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x486cf61, 33 },
    { 0xecd6cf62, 23 },
    { 0x606cf63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb6cf66, 23 },
    { 0x4b06cf67, 75 },
    { 0xf6c6cf66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef6cf6d, 90 },
    { 0xac86cf6e, 76 },
    { 0x25e6cf6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c6cf72, 61 },
    { 0, 0 },
    { 0xdeb6cf74, 77 },
    { 0xb8c6cf75, 61 },
    { 0xe0e6cf74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7226cf7a, 33 },
    { 0x2056cf7b, 90 },
    { 0x8466cf7b, 98 },
    { 0xcec6cf7d, 23 },
    { 0xf276cf7e, 99 },
    { 0xc976cf7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7166cf83, 23 },
    { 0x1606cf83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe6cf88, 61 },
    { 0xeb56cf89, 90 },
    { 0xda76cf89, 90 },
    { 0x766cf89, 99 },
    { 0, 0 },
    { 0x52b6cf8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b6cf90, 90 },
    { 0x20a6cf91, 78 },
    { 0xb836cf90, 98 },
    { 0, 0 },
    { 0xd4d6cf94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e6cf9a, 33 },
    { 0x31d6cf9a, 99 },
    { 0x8ad6cf9a, 23 },
    { 0x7926cf9d, 61 },
    { 0x1c6cf9e, 52 },
    { 0x8c16cf9d, 23 },
    { 0xd9f6cfa0, 95 },
    { 0xd8a6cf9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a6cfa9, 23 },
    { 0x1336cfaa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f56cfb0, 34 },
    { 0x49d6cfb0, 77 },
    { 0x95d6cfb0, 93 },
    { 0xfe76cfb2, 98 },
    { 0x79e6cfb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6cfb8, 98 },
    { 0x9d56cfb9, 40 },
    { 0xb676cfb8, 3 },
    { 0xc336cfbb, 90 },
    { 0x7d16cfbc, 33 },
    { 0xc996cfbd, 94 },
    { 0xa2b6cfbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf86cfc1, 75 },
    { 0, 0 },
    { 0x59b6cfc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab26cfc6, 95 },
    { 0xdec6cfc7, 67 },
    { 0, 0 },
    { 0xe9e6cfc9, 36 },
    { 0x4de6cfc9, 61 },
    { 0x1516cfcb, 90 },
    { 0x8636cfcc, 67 },
    { 0xfd36cfcd, 33 },
    { 0xe9d6cfce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe556cfd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1576cfd7, 52 },
    { 0xa706cfd8, 95 },
    { 0, 0 },
    { 0x7656cfda, 98 },
    { 0, 0 },
    { 0x2d96cfdc, 52 },
    { 0xf9f6cfdc, 95 },
    { 0, 0 },
    { 0xdd96cfdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd46cfe3, 93 },
    { 0xc836cfe4, 90 },
    { 0xd396cfe4, 95 },
    { 0xfee6cfe6, 23 },
    { 0x2076cfe6, 55 },
    { 0, 0 },
    { 0xc1a6cfe9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd6cfed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4956cff0, 95 },
    { 0x3d76cff1, 90 },
    { 0x7666cff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6cff5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3446d002, 55 },
    { 0xe406d003, 70 },
    { 0x9786d004, 55 },
    { 0xe4d6d005, 23 },
    { 0, 0 },
    { 0xc4e6d007, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca6d00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6d010, 98 },
    { 0, 0 },
    { 0xbdf6d012, 99 },
    { 0xe3d6d012, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a6d01b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4936d020, 23 },
    { 0xa346d021, 90 },
    { 0, 0 },
    { 0x5026d023, 90 },
    { 0, 0 },
    { 0x6146d025, 95 },
    { 0, 0 },
    { 0xf666d027, 66 },
    { 0xbb46d027, 100 },
    { 0xf3c6d029, 98 },
    { 0xebb6d02a, 98 },
    { 0x4626d029, 33 },
    { 0x8576d02c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5426d030, 81 },
    { 0, 0 },
    { 0x8396d032, 55 },
    { 0xad36d033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5966d03a, 61 },
    { 0, 0 },
    { 0x7706d03c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff46d03f, 76 },
    { 0x76b6d040, 93 },
    { 0xb796d041, 99 },
    { 0x1da6d042, 48 },
    { 0xeba6d043, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6d046, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6796d049, 84 },
    { 0, 0 },
    { 0xe046d04b, 98 },
    { 0x1ec6d04c, 91 },
    { 0xaf86d04d, 98 },
    { 0x8d36d04e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1096d055, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee76d059, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f16d05d, 76 },
    { 0x526d05d, 46 },
    { 0, 0 },
    { 0x35e6d060, 43 },
    { 0x9396d061, 98 },
    { 0xd336d062, 90 },
    { 0xbfe6d063, 56 },
    { 0x6746d064, 46 },
    { 0x2336d063, 76 },
    { 0xcbc6d066, 76 },
    { 0x4916d067, 43 },
    { 0x6cd6d067, 95 },
    { 0x7766d066, 99 },
    { 0x3086d060, 90 },
    { 0x47a6d067, 55 },
    { 0x1516d06c, 67 },
    { 0xeb96d062, 33 },
    { 0x8016d06e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b6d072, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6d075, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x7026d078, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5746d07f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9356d085, 90 },
    { 0, 0 },
    { 0x6896d087, 33 },
    { 0xc5f6d087, 43 },
    { 0x6636d088, 55 },
    { 0xd7f6d08a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c6d093, 91 },
    { 0xd376d093, 98 },
    { 0x6946d095, 84 },
    { 0x52c6d093, 23 },
    { 0xbad6d095, 38 },
    { 0x676d097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf6d09e, 33 },
    { 0, 0 },
    { 0x9df6d0a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7646d0a5, 33 },
    { 0x9a46d0a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d0ac, 91 },
    { 0, 0 },
    { 0xddf6d0ae, 95 },
    { 0xedd6d0ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9906d0b2, 95 },
    { 0x3696d0b3, 81 },
    { 0x4c76d0b4, 23 },
    { 0xc7a6d0b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff76d0bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca96d0c1, 81 },
    { 0x52e6d0c2, 61 },
    { 0x7ee6d0c2, 92 },
    { 0x7896d0c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6d0c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6d0ca, 55 },
    { 0xbae6d0ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc06d0ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd36d0d1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa6d0d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746d0d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f6d0db, 93 },
    { 0x2336d0db, 85 },
    { 0x3f26d0dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be6d0e1, 90 },
    { 0, 0 },
    { 0x8926d0e3, 46 },
    { 0, 0 },
    { 0xf2e6d0e5, 67 },
    { 0xa956d0e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa6d0ed, 23 },
    { 0, 0 },
    { 0x63a6d0ef, 33 },
    { 0, 0 },
    { 0x3ef6d0f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646d0f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e6d0fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe756d0fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6746d101, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e6d106, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde66d10c, 66 },
    { 0, 0 },
    { 0x53b6d10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7606d111, 95 },
    { 0x7686d111, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d6d11a, 98 },
    { 0x8e46d11b, 23 },
    { 0x1ec6d11c, 84 },
    { 0x1706d11d, 66 },
    { 0xdcc6d11d, 43 },
    { 0x9e86d11e, 34 },
    { 0x726d11f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f6d123, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3506d128, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8606d12b, 61 },
    { 0xcb96d12c, 33 },
    { 0x60c6d12b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f6d131, 98 },
    { 0x236d132, 99 },
    { 0, 0 },
    { 0x1d06d134, 99 },
    { 0, 0 },
    { 0xf8f6d136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc6d145, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf46d14a, 46 },
    { 0x7216d14b, 85 },
    { 0xe666d14c, 98 },
    { 0, 0 },
    { 0x206d14e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b6d154, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846d15a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1656d165, 61 },
    { 0, 0 },
    { 0x90a6d167, 90 },
    { 0x63b6d168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc76d16c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe6d16f, 33 },
    { 0, 0 },
    { 0x7556d171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56d17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846d180, 100 },
    { 0xbb36d180, 98 },
    { 0xde06d180, 95 },
    { 0xf896d183, 34 },
    { 0xf2e6d184, 43 },
    { 0xda96d185, 90 },
    { 0x7fc6d186, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c56d18b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6186d18e, 55 },
    { 0, 0 },
    { 0xb746d190, 43 },
    { 0, 0 },
    { 0x3ec6d192, 61 },
    { 0x72e6d193, 87 },
    { 0x49f6d194, 95 },
    { 0xc166d195, 43 },
    { 0x586d196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b6d19b, 95 },
    { 0xd0a6d19c, 23 },
    { 0x61e6d19c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1026d1a1, 95 },
    { 0x8ac6d1a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06d1a7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf36d1b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd26d1bc, 98 },
    { 0x6de6d1bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c6d1c5, 33 },
    { 0, 0 },
    { 0x6416d1c7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316d1cc, 55 },
    { 0, 0 },
    { 0xd266d1ce, 90 },
    { 0, 0 },
    { 0xfcd6d1d0, 95 },
    { 0x27a6d1d1, 66 },
    { 0, 0 },
    { 0xd16d1d3, 61 },
    { 0x6ee6d1d3, 23 },
    { 0, 0 },
    { 0x5c66d1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616d1db, 98 },
    { 0, 0 },
    { 0x5b86d1dd, 34 },
    { 0, 0 },
    { 0x5a56d1df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5646d1e7, 43 },
    { 0xa76d1e8, 33 },
    { 0, 0 },
    { 0xf1f6d1ea, 99 },
    { 0xc796d1eb, 98 },
    { 0xe9c6d1eb, 78 },
    { 0xef26d1ea, 100 },
    { 0x6bc6d1ee, 55 },
    { 0, 0 },
    { 0xa5c6d1f0, 91 },
    { 0, 0 },
    { 0x93f6d1f2, 40 },
    { 0xbb86d1f3, 33 },
    { 0x53f6d1f4, 66 },
    { 0xbfa6d1f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336d1f8, 90 },
    { 0xcb76d1f8, 100 },
    { 0x7316d1fa, 91 },
    { 0x60d6d1fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac46d200, 67 },
    { 0, 0 },
    { 0x1986d202, 33 },
    { 0xa6c6d203, 52 },
    { 0, 0 },
    { 0xfce6d205, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b36d208, 98 },
    { 0x98a6d209, 95 },
    { 0x85b6d20a, 76 },
    { 0x3fc6d20b, 90 },
    { 0x1c96d20c, 76 },
    { 0x3486d20d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab76d21a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc6d21e, 95 },
    { 0, 0 },
    { 0xda76d220, 76 },
    { 0xb896d220, 33 },
    { 0xd46d220, 61 },
    { 0x8386d220, 100 },
    { 0, 0 },
    { 0x2d6d225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe86d22a, 93 },
    { 0x3346d22a, 95 },
    { 0x7b6d22c, 90 },
    { 0xe736d22a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc876d230, 33 },
    { 0x6fc6d230, 33 },
    { 0xfd6d230, 61 },
    { 0x3b6d231, 95 },
    { 0, 0 },
    { 0xe0c6d235, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec06d23a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26d23e, 98 },
    { 0x2be6d23e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c6d243, 55 },
    { 0, 0 },
    { 0xe426d245, 95 },
    { 0, 0 },
    { 0x1a86d247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad6d24b, 90 },
    { 0x2f96d24b, 23 },
    { 0xe356d24d, 90 },
    { 0xff86d24e, 40 },
    { 0x5216d24b, 34 },
    { 0, 0 },
    { 0x7f16d251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8406d257, 95 },
    { 0x7a96d257, 33 },
    { 0x4296d258, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e6d25c, 76 },
    { 0, 0 },
    { 0x266d25e, 69 },
    { 0, 0 },
    { 0xde36d260, 94 },
    { 0, 0 },
    { 0xc266d262, 99 },
    { 0, 0 },
    { 0x1e46d264, 34 },
    { 0, 0 },
    { 0x6816d266, 55 },
    { 0, 0 },
    { 0x6a36d268, 76 },
    { 0x4166d268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd596d26c, 100 },
    { 0x1b96d26c, 17 },
    { 0xe946d26e, 33 },
    { 0xc9d6d26e, 98 },
    { 0x3af6d270, 23 },
    { 0x5746d271, 77 },
    { 0x4806d26d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a6d278, 61 },
    { 0, 0 },
    { 0x95e6d27a, 55 },
    { 0xfab6d27b, 99 },
    { 0x1006d27c, 33 },
    { 0, 0 },
    { 0xaf96d27e, 98 },
    { 0x8d16d27f, 61 },
    { 0, 0 },
    { 0x3ef6d281, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe976d286, 95 },
    { 0, 0 },
    { 0x5c66d288, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba86d291, 23 },
    { 0, 0 },
    { 0x1866d293, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9956d297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f6d29a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a46d2a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e6d2a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b26d2b0, 93 },
    { 0x7816d2b1, 34 },
    { 0x9e96d2b2, 95 },
    { 0, 0 },
    { 0xbd66d2b4, 61 },
    { 0xc976d2b5, 95 },
    { 0x72f6d2b5, 61 },
    { 0x4346d2b7, 61 },
    { 0, 0 },
    { 0x6046d2b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4176d2bd, 55 },
    { 0, 0 },
    { 0xea66d2bf, 43 },
    { 0, 0 },
    { 0x94f6d2c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f86d2c5, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5376d2cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646d2ce, 55 },
    { 0x4896d2cf, 76 },
    { 0x8ab6d2d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb926d2d3, 33 },
    { 0, 0 },
    { 0xd2f6d2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed76d2da, 75 },
    { 0x53c6d2da, 40 },
    { 0, 0 },
    { 0x26d2dd, 99 },
    { 0xd8d6d2de, 90 },
    { 0xee26d2df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d6d2e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b86d2e6, 98 },
    { 0, 0 },
    { 0x7336d2e8, 61 },
    { 0, 0 },
    { 0x85c6d2ea, 61 },
    { 0x81a6d2ea, 88 },
    { 0x60d6d2ec, 77 },
    { 0x7a46d2ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc96d2f0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x1486d2f3, 46 },
    { 0x4bf6d2f4, 33 },
    { 0, 0 },
    { 0xe7c6d2f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e76d2fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6d2fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c6d301, 66 },
    { 0x8506d302, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb976d308, 43 },
    { 0xfbd6d309, 23 },
    { 0, 0 },
    { 0xff86d30b, 33 },
    { 0xd586d30b, 67 },
    { 0x9836d30c, 90 },
    { 0xfde6d30e, 90 },
    { 0xe2c6d30c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa336d314, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6d31d, 65 },
    { 0, 0 },
    { 0x4916d31f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd26d324, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2236d32b, 90 },
    { 0xbc26d32c, 23 },
    { 0x2d96d32c, 90 },
    { 0x96d6d32d, 80 },
    { 0x10c6d32f, 100 },
    { 0x5976d330, 98 },
    { 0x8a06d331, 76 },
    { 0x756d32f, 98 },
    { 0x6156d333, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b16d33b, 76 },
    { 0xc06d33b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f6d340, 91 },
    { 0xe056d341, 90 },
    { 0x7d36d341, 76 },
    { 0x6046d340, 75 },
    { 0x70b6d342, 67 },
    { 0, 0 },
    { 0xd176d346, 23 },
    { 0xfda6d347, 61 },
    { 0xda26d347, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d6d35d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a6d360, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd146d364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc6d369, 100 },
    { 0, 0 },
    { 0x72a6d36b, 98 },
    { 0xd476d36c, 76 },
    { 0, 0 },
    { 0x9686d36e, 100 },
    { 0x9a96d36f, 76 },
    { 0, 0 },
    { 0x77b6d371, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16d374, 93 },
    { 0xef86d374, 61 },
    { 0xd076d376, 97 },
    { 0x7816d374, 55 },
    { 0x3766d378, 98 },
    { 0xec6d379, 85 },
    { 0x1a86d376, 55 },
    { 0, 0 },
    { 0x7b06d37c, 90 },
    { 0xb066d37d, 95 },
    { 0, 0 },
    { 0x2bb6d37f, 90 },
    { 0x35b6d380, 52 },
    { 0xf016d380, 24 },
    { 0xcb76d380, 90 },
    { 0x41a6d380, 69 },
    { 0xb496d384, 90 },
    { 0, 0 },
    { 0x8576d386, 93 },
    { 0x2946d387, 90 },
    { 0x2356d388, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b6d38e, 33 },
    { 0xe746d38f, 61 },
    { 0x4706d38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6186d396, 34 },
    { 0, 0 },
    { 0xc1d6d398, 61 },
    { 0x28e6d399, 61 },
    { 0x5596d399, 77 },
    { 0x27d6d399, 33 },
    { 0xa106d39c, 98 },
    { 0xe0f6d398, 33 },
    { 0x2d6d398, 64 },
    { 0xb5e6d39c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6676d3a2, 90 },
    { 0x70f6d3a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab6d3a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a96d3ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da6d3b1, 93 },
    { 0x15d6d3b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4246d3b9, 23 },
    { 0x6026d3ba, 90 },
    { 0x7056d3ba, 46 },
    { 0, 0 },
    { 0xb0b6d3bd, 94 },
    { 0, 0 },
    { 0x2596d3bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9326d3c2, 61 },
    { 0x8166d3c3, 46 },
    { 0xce06d3c3, 8 },
    { 0xea16d3c5, 94 },
    { 0, 0 },
    { 0xc746d3c7, 98 },
    { 0x5ae6d3c8, 55 },
    { 0x1bc6d3c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266d3cd, 61 },
    { 0xb9a6d3cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126d3d3, 83 },
    { 0xbc46d3d4, 95 },
    { 0, 0 },
    { 0xa0c6d3d6, 95 },
    { 0, 0 },
    { 0x5ab6d3d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586d3e3, 100 },
    { 0, 0 },
    { 0xe056d3e5, 33 },
    { 0x5a46d3e6, 33 },
    { 0x5396d3e6, 33 },
    { 0x1d76d3e8, 98 },
    { 0, 0 },
    { 0x2626d3ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf946d3ed, 99 },
    { 0, 0 },
    { 0x5ca6d3ef, 87 },
    { 0x8276d3f0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e6d3f6, 93 },
    { 0xffe6d3f7, 33 },
    { 0x5746d3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6266d3fd, 84 },
    { 0, 0 },
    { 0x29c6d3ff, 76 },
    { 0x1f16d400, 99 },
    { 0x476d401, 90 },
    { 0xebc6d401, 90 },
    { 0x9ad6d3ff, 33 },
    { 0xe476d400, 95 },
    { 0xc036d401, 67 },
    { 0xc506d406, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce86d409, 76 },
    { 0xb656d40a, 66 },
    { 0x41a6d40b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb6d40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b66d412, 100 },
    { 0x1296d413, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96d416, 33 },
    { 0xe7e6d417, 65 },
    { 0x4f86d417, 33 },
    { 0xa196d416, 76 },
    { 0x4f16d41a, 90 },
    { 0xf026d41b, 65 },
    { 0x84e6d41b, 23 },
    { 0, 0 },
    { 0xe316d41e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3306d424, 78 },
    { 0x1066d424, 23 },
    { 0, 0 },
    { 0x4556d427, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b6d42a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf16d42e, 38 },
    { 0xb796d42f, 90 },
    { 0xe3c6d430, 76 },
    { 0x1176d431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc86d435, 93 },
    { 0x56b6d435, 34 },
    { 0x1556d437, 68 },
    { 0x6f76d438, 90 },
    { 0x5c46d437, 61 },
    { 0x8ad6d437, 55 },
    { 0xcad6d43b, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd6d43f, 90 },
    { 0, 0 },
    { 0x4416d441, 51 },
    { 0, 0 },
    { 0xb316d443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb876d446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1256d44a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f6d44e, 93 },
    { 0x6546d44f, 23 },
    { 0xe006d450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016d453, 90 },
    { 0, 0 },
    { 0x4576d455, 80 },
    { 0x7d36d456, 33 },
    { 0, 0 },
    { 0xc966d458, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6476d45c, 46 },
    { 0x6c56d45d, 34 },
    { 0x9ad6d45e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e06d461, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef66d469, 95 },
    { 0x8d56d46a, 98 },
    { 0x9406d46a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6d46e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad6d475, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf176d47b, 91 },
    { 0x8d96d47c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f86d482, 90 },
    { 0x6be6d483, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d06d489, 34 },
    { 0x2c26d48a, 48 },
    { 0xcd56d48b, 76 },
    { 0x6736d489, 55 },
    { 0x9b16d48c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b6d499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3596d49c, 98 },
    { 0, 0 },
    { 0x3726d49e, 55 },
    { 0, 0 },
    { 0x9566d4a0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956d4a4, 87 },
    { 0, 0 },
    { 0x31c6d4a6, 76 },
    { 0xddd6d4a7, 76 },
    { 0xbbc6d4a7, 85 },
    { 0, 0 },
    { 0xc2e6d4aa, 90 },
    { 0x16f6d4aa, 78 },
    { 0xba06d4ab, 34 },
    { 0x5e36d4ac, 23 },
    { 0x8ec6d4ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f6d4b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf6d4b6, 98 },
    { 0xaf6d4b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b56d4ba, 98 },
    { 0, 0 },
    { 0x1436d4bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826d4c1, 90 },
    { 0x9fb6d4c2, 95 },
    { 0xc3e6d4c2, 90 },
    { 0xc926d4c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa736d4c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d4cf, 76 },
    { 0xd866d4cf, 95 },
    { 0x48a6d4d1, 76 },
    { 0, 0 },
    { 0x66f6d4d3, 88 },
    { 0, 0 },
    { 0x2426d4d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e16d4de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46d4e1, 90 },
    { 0x6ed6d4e1, 33 },
    { 0x5ff6d4e3, 98 },
    { 0x17a6d4e4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xda96d4e7, 46 },
    { 0xc3a6d4e8, 95 },
    { 0x4456d4e9, 34 },
    { 0x3006d4ea, 61 },
    { 0x7946d4e7, 55 },
    { 0x986d4ec, 90 },
    { 0xcae6d4ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3696d4f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed16d4f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e36d4f9, 90 },
    { 0x1e96d4f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f6d4fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c6d509, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2196d50f, 93 },
    { 0, 0 },
    { 0x3616d511, 80 },
    { 0, 0 },
    { 0x9396d513, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab6d517, 60 },
    { 0x416d518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf566d51f, 95 },
    { 0x5a86d520, 98 },
    { 0xd686d520, 76 },
    { 0x7706d51f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d525, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c6d529, 90 },
    { 0x8086d529, 61 },
    { 0x5956d52b, 19 },
    { 0xb566d52c, 93 },
    { 0, 0 },
    { 0x1906d52e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6d539, 84 },
    { 0, 0 },
    { 0xf1a6d53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c76d53e, 33 },
    { 0xea06d53f, 76 },
    { 0x2e66d53f, 34 },
    { 0x14e6d541, 20 },
    { 0xc106d542, 33 },
    { 0xd86d543, 98 },
    { 0x2d6d540, 52 },
    { 0x6966d541, 98 },
    { 0x6b6d545, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd136d549, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c96d54c, 40 },
    { 0x9a66d54d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1736d550, 95 },
    { 0, 0 },
    { 0x2256d552, 90 },
    { 0xe3c6d553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d6d557, 95 },
    { 0xc376d558, 23 },
    { 0x1d36d559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8546d564, 47 },
    { 0xe2b6d565, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc6d569, 95 },
    { 0x9806d56a, 90 },
    { 0, 0 },
    { 0x97a6d56c, 61 },
    { 0, 0 },
    { 0x3346d56e, 55 },
    { 0xd26d56f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4416d573, 78 },
    { 0xe116d573, 95 },
    { 0x5d66d574, 78 },
    { 0x31c6d576, 33 },
    { 0x91c6d576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2626d57b, 76 },
    { 0, 0 },
    { 0x61e6d57d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b46d581, 98 },
    { 0xae6d582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe866d585, 91 },
    { 0, 0 },
    { 0x7a6d587, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb476d58a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1726d58d, 76 },
    { 0x1af6d58e, 33 },
    { 0xda46d58f, 98 },
    { 0x54f6d58e, 98 },
    { 0, 0 },
    { 0x4126d592, 90 },
    { 0x2cb6d592, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b6d596, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5706d59b, 33 },
    { 0x7326d59c, 55 },
    { 0, 0 },
    { 0x25d6d59e, 90 },
    { 0x416d59e, 33 },
    { 0, 0 },
    { 0x3a96d5a1, 33 },
    { 0, 0 },
    { 0x3756d5a3, 48 },
    { 0x4dc6d5a4, 77 },
    { 0xae16d5a3, 90 },
    { 0x4796d5a5, 99 },
    { 0x3676d5a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad26d5ab, 61 },
    { 0x5176d5ac, 90 },
    { 0x6126d5ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f6d5b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c6d5ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f16d5bf, 67 },
    { 0x8556d5c0, 23 },
    { 0, 0 },
    { 0x6556d5c2, 61 },
    { 0x4d96d5c2, 33 },
    { 0x8836d5c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456d5cd, 66 },
    { 0xe4f6d5ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf6d5d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc886d5d5, 84 },
    { 0x39a6d5d6, 81 },
    { 0xdab6d5d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb776d5db, 58 },
    { 0xdfa6d5db, 76 },
    { 0xbdd6d5dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76d5e1, 33 },
    { 0x9506d5e2, 61 },
    { 0xd9d6d5e3, 33 },
    { 0, 0 },
    { 0xe26d5e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb056d5e8, 43 },
    { 0, 0 },
    { 0x37b6d5ea, 34 },
    { 0, 0 },
    { 0xf836d5ec, 88 },
    { 0, 0 },
    { 0xf876d5ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266d5f1, 23 },
    { 0xb806d5f2, 90 },
    { 0x4676d5f2, 52 },
    { 0x8656d5f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a6d5f7, 33 },
    { 0x6606d5f7, 90 },
    { 0xf3b6d5f8, 55 },
    { 0x29c6d5fa, 66 },
    { 0xf036d5fb, 95 },
    { 0x6c96d5fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6d602, 90 },
    { 0, 0 },
    { 0x1e76d604, 95 },
    { 0, 0 },
    { 0xda66d606, 33 },
    { 0, 0 },
    { 0x6c56d608, 93 },
    { 0xca66d609, 17 },
    { 0x666d60a, 93 },
    { 0x3896d60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb6d60e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f6d612, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2716d619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6d61d, 61 },
    { 0xe126d61e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96d623, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca76d628, 90 },
    { 0x3976d628, 95 },
    { 0x92c6d62a, 90 },
    { 0x476d62b, 33 },
    { 0xe156d62b, 90 },
    { 0xb2a6d62d, 90 },
    { 0xac16d62b, 98 },
    { 0xbb36d62c, 33 },
    { 0xdcf6d62d, 85 },
    { 0x7846d631, 90 },
    { 0, 0 },
    { 0x7f56d633, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a56d636, 99 },
    { 0xef56d636, 84 },
    { 0, 0 },
    { 0x9886d639, 61 },
    { 0x5416d63a, 33 },
    { 0x2f76d63b, 33 },
    { 0x6396d63c, 90 },
    { 0xcb36d63a, 46 },
    { 0xb6d6d63b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d641, 33 },
    { 0x3206d642, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9416d645, 33 },
    { 0, 0 },
    { 0x1fb6d647, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b6d64a, 98 },
    { 0, 0 },
    { 0xa76d64c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc466d652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c6d655, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb676d65d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac6d661, 76 },
    { 0x2e56d662, 98 },
    { 0x5d26d661, 61 },
    { 0, 0 },
    { 0x45c6d665, 40 },
    { 0, 0 },
    { 0xe366d667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf176d66a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8366d671, 61 },
    { 0xe76d672, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7866d67c, 90 },
    { 0x2286d67d, 98 },
    { 0xdd56d67c, 95 },
    { 0, 0 },
    { 0xc7b6d680, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda56d688, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf476d68b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f6d68f, 98 },
    { 0xf966d690, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7216d695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f6d698, 85 },
    { 0x8366d699, 90 },
    { 0xdbd6d698, 93 },
    { 0xafa6d69a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae6d6a1, 97 },
    { 0, 0 },
    { 0x92c6d6a3, 95 },
    { 0x406d6a4, 52 },
    { 0xad06d6a5, 100 },
    { 0x30e6d6a6, 76 },
    { 0xdf36d6a7, 77 },
    { 0xad6d6a4, 61 },
    { 0xfde6d6a4, 76 },
    { 0x6de6d6aa, 90 },
    { 0x9d76d6aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a6d6af, 99 },
    { 0x5f26d6af, 61 },
    { 0x6546d6b1, 76 },
    { 0x2d26d6b2, 78 },
    { 0x8876d6b1, 55 },
    { 0, 0 },
    { 0x7316d6b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776d6b9, 98 },
    { 0, 0 },
    { 0xcb6d6bb, 33 },
    { 0x79c6d6bb, 55 },
    { 0xa286d6bd, 76 },
    { 0x65b6d6bd, 98 },
    { 0x9f6d6be, 98 },
    { 0xd166d6c0, 87 },
    { 0xc956d6c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad6d6c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4626d6ce, 23 },
    { 0x4126d6cf, 91 },
    { 0x16a6d6d0, 95 },
    { 0x82f6d6d1, 90 },
    { 0x3cd6d6d0, 33 },
    { 0x79b6d6d3, 67 },
    { 0xfe06d6d1, 33 },
    { 0, 0 },
    { 0x4ee6d6d6, 67 },
    { 0xc406d6d6, 95 },
    { 0x90e6d6d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f46d6dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b36d6e1, 38 },
    { 0, 0 },
    { 0xa6a6d6e3, 100 },
    { 0, 0 },
    { 0x9dc6d6e5, 85 },
    { 0xb626d6e5, 76 },
    { 0x7a56d6e7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb236d6ea, 75 },
    { 0x1df6d6eb, 95 },
    { 0xbf56d6ec, 33 },
    { 0xc576d6ed, 46 },
    { 0x5686d6ee, 90 },
    { 0xc5b6d6ef, 61 },
    { 0, 0 },
    { 0x2416d6f1, 23 },
    { 0x1cc6d6f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd86d6f6, 65 },
    { 0xced6d6f7, 90 },
    { 0xd3a6d6f8, 93 },
    { 0xc1e6d6f8, 33 },
    { 0xddb6d6f7, 33 },
    { 0x2d06d6f8, 98 },
    { 0x606d6fa, 87 },
    { 0x7fd6d6f7, 23 },
    { 0, 0 },
    { 0x29b6d6ff, 98 },
    { 0xbe06d700, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd656d704, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c6d708, 46 },
    { 0x9ed6d709, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df6d70d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe586d710, 61 },
    { 0x9506d711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6d715, 17 },
    { 0x196d715, 84 },
    { 0, 0 },
    { 0xef46d718, 23 },
    { 0xcea6d718, 98 },
    { 0xb396d71a, 33 },
    { 0x3e46d71a, 33 },
    { 0x8f46d719, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa916d722, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa386d727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba6d72a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b6d72d, 33 },
    { 0x99c6d72d, 60 },
    { 0x6e6d72f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e6d733, 90 },
    { 0, 0 },
    { 0xed96d735, 95 },
    { 0, 0 },
    { 0x45d6d737, 91 },
    { 0, 0 },
    { 0xf2e6d739, 77 },
    { 0x5786d739, 90 },
    { 0xe956d73b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f6d747, 78 },
    { 0xd726d748, 96 },
    { 0x9e16d749, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb196d74c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe386d755, 23 },
    { 0x1876d756, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6286d75b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe046d75e, 55 },
    { 0x3d16d75f, 81 },
    { 0x63a6d760, 43 },
    { 0x39e6d761, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a6d765, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d769, 76 },
    { 0x85a6d76a, 23 },
    { 0xf556d76b, 61 },
    { 0xfd56d76c, 98 },
    { 0xac86d76d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9656d770, 52 },
    { 0, 0 },
    { 0x7866d772, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2446d779, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd86d77f, 87 },
    { 0xe106d780, 95 },
    { 0xa9c6d781, 90 },
    { 0x4596d781, 43 },
    { 0x3e06d783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc986d788, 91 },
    { 0xb396d789, 61 },
    { 0x6b6d78a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc216d78d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a6d793, 95 },
    { 0, 0 },
    { 0x9326d795, 52 },
    { 0xcd16d795, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5036d79e, 33 },
    { 0, 0 },
    { 0x2366d7a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5016d7a3, 46 },
    { 0x4136d7a4, 55 },
    { 0x76f6d7a3, 23 },
    { 0x7d26d7a3, 34 },
    { 0x2ed6d7a6, 95 },
    { 0, 0 },
    { 0x5da6d7a9, 23 },
    { 0xd3f6d7aa, 65 },
    { 0x8f6d7ab, 23 },
    { 0, 0 },
    { 0xa2b6d7ad, 67 },
    { 0x1836d7ae, 38 },
    { 0xa666d7af, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe066d7b2, 23 },
    { 0xe606d7b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2726d7b6, 61 },
    { 0, 0 },
    { 0x72e6d7b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff06d7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e6d7c7, 90 },
    { 0x2886d7c8, 33 },
    { 0xf296d7c8, 43 },
    { 0xfbc6d7c7, 66 },
    { 0, 0 },
    { 0xc5c6d7cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e6d7d7, 23 },
    { 0x9546d7d8, 93 },
    { 0, 0 },
    { 0xd356d7da, 76 },
    { 0, 0 },
    { 0xe826d7dc, 46 },
    { 0, 0 },
    { 0x6466d7de, 95 },
    { 0x916d7df, 33 },
    { 0x30b6d7df, 90 },
    { 0xd206d7de, 51 },
    { 0, 0 },
    { 0x3a26d7e3, 43 },
    { 0, 0 },
    { 0x2716d7e5, 90 },
    { 0xfaa6d7e6, 61 },
    { 0xea56d7e7, 23 },
    { 0xc096d7e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd86d7ec, 61 },
    { 0x2266d7ed, 43 },
    { 0x7286d7ee, 87 },
    { 0xc1d6d7ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3576d7f6, 76 },
    { 0x9016d7f7, 95 },
    { 0xf116d7f8, 98 },
    { 0, 0 },
    { 0xd06d7fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66d7fd, 90 },
    { 0x3a66d7fe, 90 },
    { 0, 0 },
    { 0x7856d800, 34 },
    { 0, 0 },
    { 0xbbb6d802, 52 },
    { 0, 0 },
    { 0xc196d804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a6d807, 90 },
    { 0, 0 },
    { 0xd406d809, 90 },
    { 0, 0 },
    { 0x8106d80b, 23 },
    { 0x1a36d80c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc126d80f, 64 },
    { 0xc326d810, 61 },
    { 0x20a6d80f, 52 },
    { 0xfc46d810, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f6d815, 78 },
    { 0, 0 },
    { 0xcba6d817, 33 },
    { 0xb1f6d818, 61 },
    { 0x2c66d817, 90 },
    { 0xd236d81a, 91 },
    { 0x1db6d81b, 98 },
    { 0xb1c6d81c, 98 },
    { 0x39b6d81b, 76 },
    { 0xe556d81c, 98 },
    { 0xfb6d81c, 83 },
    { 0x83c6d820, 90 },
    { 0xcc76d821, 98 },
    { 0xa3a6d821, 93 },
    { 0x1a36d81b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6d828, 100 },
    { 0, 0 },
    { 0xba36d82a, 33 },
    { 0x7706d82b, 46 },
    { 0x1476d82a, 34 },
    { 0xe2c6d82d, 33 },
    { 0x10a6d82c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e6d832, 34 },
    { 0xcd56d832, 87 },
    { 0, 0 },
    { 0x4bb6d835, 33 },
    { 0x21d6d836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb796d839, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86d83e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f86d845, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b66d84a, 61 },
    { 0, 0 },
    { 0xa696d84c, 59 },
    { 0x3da6d84c, 23 },
    { 0x5226d84c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda6d858, 78 },
    { 0, 0 },
    { 0x5ce6d85a, 33 },
    { 0, 0 },
    { 0x1576d85c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4496d862, 52 },
    { 0xe4b6d863, 100 },
    { 0, 0 },
    { 0xf746d865, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6d86c, 100 },
    { 0xdbd6d86d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d6d876, 33 },
    { 0x1026d877, 90 },
    { 0x93c6d877, 52 },
    { 0xdd96d879, 33 },
    { 0xacd6d879, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb76d881, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a6d885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d76d88c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c06d88f, 81 },
    { 0x6786d890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce76d895, 84 },
    { 0, 0 },
    { 0xcd36d897, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6d89d, 96 },
    { 0xe946d89e, 66 },
    { 0x4706d89f, 76 },
    { 0x76c6d89f, 61 },
    { 0x58a6d8a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5676d8aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe126d8ae, 34 },
    { 0, 0 },
    { 0x2846d8b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd416d8b3, 23 },
    { 0x4986d8b4, 90 },
    { 0x50e6d8b5, 90 },
    { 0xeb16d8b6, 23 },
    { 0xb506d8b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd236d8bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956d8c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc6d8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb276d8cf, 23 },
    { 0xb0e6d8d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f36d8d4, 100 },
    { 0x9e66d8d5, 81 },
    { 0xfcc6d8d4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf6d8d9, 95 },
    { 0x7166d8da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a6d8de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa516d8ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf36d8ed, 61 },
    { 0x8736d8ed, 76 },
    { 0, 0 },
    { 0x9c36d8f0, 76 },
    { 0x4f76d8f0, 98 },
    { 0, 0 },
    { 0xc1b6d8f3, 52 },
    { 0, 0 },
    { 0xcff6d8f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b16d8fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2256d900, 90 },
    { 0, 0 },
    { 0x35e6d902, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7556d90e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16d911, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a6d917, 40 },
    { 0, 0 },
    { 0x2a26d919, 95 },
    { 0x4346d919, 76 },
    { 0, 0 },
    { 0xeac6d91c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7716d922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3076d929, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc316d92c, 77 },
    { 0x2406d92c, 55 },
    { 0x66d6d92e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8826d932, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8256d936, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36d93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe436d941, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8316d944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e6d949, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6d950, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2976d953, 67 },
    { 0x6766d954, 90 },
    { 0, 0 },
    { 0xdea6d956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866d95f, 67 },
    { 0x4f86d960, 24 },
    { 0x786d960, 98 },
    { 0xc626d962, 43 },
    { 0x34a6d963, 76 },
    { 0xcd66d964, 68 },
    { 0x7bd6d95f, 90 },
    { 0x3bf6d95f, 38 },
    { 0, 0 },
    { 0x4e26d968, 61 },
    { 0x35b6d969, 23 },
    { 0x8696d96a, 34 },
    { 0xc736d96b, 93 },
    { 0x4356d968, 33 },
    { 0x7886d96d, 91 },
    { 0xc0f6d96e, 66 },
    { 0x9e26d96f, 90 },
    { 0x6546d970, 95 },
    { 0x7ca6d971, 46 },
    { 0xa476d972, 84 },
    { 0x1c26d96f, 90 },
    { 0, 0 },
    { 0x2316d975, 52 },
    { 0x6036d976, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x8626d979, 23 },
    { 0x2c86d97a, 99 },
    { 0x5776d97a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb536d97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6d983, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1316d986, 90 },
    { 0xa26d987, 33 },
    { 0xfc6d986, 53 },
    { 0, 0 },
    { 0x7336d98a, 55 },
    { 0xf36d98b, 46 },
    { 0x7426d98c, 97 },
    { 0xa6d98b, 25 },
    { 0x1ec6d98e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe226d991, 95 },
    { 0xef66d991, 83 },
    { 0, 0 },
    { 0x2a26d994, 90 },
    { 0xe6d995, 55 },
    { 0x2216d995, 34 },
    { 0x7786d997, 68 },
    { 0x9dc6d998, 33 },
    { 0xa06d998, 99 },
    { 0xbf16d99a, 46 },
    { 0xe216d99a, 78 },
    { 0xc036d99c, 90 },
    { 0x19d6d99d, 87 },
    { 0x4196d99e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf626d9a4, 61 },
    { 0, 0 },
    { 0x8db6d9a6, 61 },
    { 0x18d6d9a7, 75 },
    { 0x5816d9a7, 55 },
    { 0x4296d9a6, 87 },
    { 0, 0 },
    { 0xe006d9ab, 87 },
    { 0xbe26d9ac, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c6d9b0, 64 },
    { 0x29f6d9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb6d9b4, 33 },
    { 0x6c6d9b4, 95 },
    { 0x7d6d9b6, 33 },
    { 0x45a6d9b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6666d9bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4236d9c4, 28 },
    { 0, 0 },
    { 0x7ee6d9c6, 95 },
    { 0x6ea6d9c7, 34 },
    { 0xd7e6d9c6, 33 },
    { 0xcc36d9c9, 90 },
    { 0, 0 },
    { 0xd076d9cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba6d9cf, 99 },
    { 0, 0 },
    { 0xb296d9d1, 90 },
    { 0x1d56d9d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66d9d5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b6d9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca6d9db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe16d9de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4666d9e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1346d9ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9126d9f4, 33 },
    { 0x64e6d9f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6d9f8, 66 },
    { 0xb286d9f8, 33 },
    { 0x6776d9f8, 87 },
    { 0xb976d9fb, 49 },
    { 0xd4f6d9fb, 76 },
    { 0x5186d9fd, 34 },
    { 0x76a6d9fd, 90 },
    { 0x57a6d9fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a66da03, 55 },
    { 0, 0 },
    { 0xaaa6da05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6da0a, 90 },
    { 0, 0 },
    { 0x4686da0c, 23 },
    { 0xb426da0d, 90 },
    { 0x1936da0e, 55 },
    { 0xac86da0f, 76 },
    { 0x9856da0f, 91 },
    { 0x2df6da11, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846da15, 99 },
    { 0xa676da16, 93 },
    { 0x7036da17, 90 },
    { 0, 0 },
    { 0x4b86da19, 76 },
    { 0, 0 },
    { 0xb806da1b, 88 },
    { 0x9a06da1b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba66da21, 85 },
    { 0, 0 },
    { 0x5216da23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316da2a, 90 },
    { 0x38b6da2a, 76 },
    { 0x5fd6da2c, 33 },
    { 0xd126da2c, 95 },
    { 0xd1e6da2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1506da38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b6da3b, 95 },
    { 0xe126da3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6da42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb76da46, 98 },
    { 0x20d6da47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5866da4f, 61 },
    { 0xac96da4f, 98 },
    { 0xedd6da4f, 81 },
    { 0x716da52, 40 },
    { 0x6286da52, 91 },
    { 0x76b6da52, 33 },
    { 0x2de6da52, 38 },
    { 0, 0 },
    { 0x8566da57, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a26da5b, 46 },
    { 0x5a46da5c, 33 },
    { 0x7896da5d, 68 },
    { 0xd286da5e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06da61, 61 },
    { 0, 0 },
    { 0xe486da63, 95 },
    { 0, 0 },
    { 0xf3c6da65, 33 },
    { 0xa446da65, 95 },
    { 0, 0 },
    { 0x2156da68, 95 },
    { 0x6f96da69, 52 },
    { 0, 0 },
    { 0xb7f6da6b, 43 },
    { 0xa356da6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc516da71, 90 },
    { 0x8306da72, 88 },
    { 0xc246da73, 33 },
    { 0xa696da73, 61 },
    { 0, 0 },
    { 0x7026da76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd6da7a, 90 },
    { 0x5946da7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa216da7f, 76 },
    { 0, 0 },
    { 0xc466da81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b6da84, 98 },
    { 0x7496da85, 98 },
    { 0, 0 },
    { 0x23a6da87, 98 },
    { 0, 0 },
    { 0x6ea6da89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e26da8e, 67 },
    { 0x5846da8f, 84 },
    { 0x1946da90, 61 },
    { 0xcf46da91, 91 },
    { 0x1386da91, 76 },
    { 0xb2f6da93, 66 },
    { 0x27b6da91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d6da97, 76 },
    { 0x1ab6da97, 33 },
    { 0xa16da98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b66da9c, 85 },
    { 0x62c6da9c, 98 },
    { 0xc3e6da9c, 95 },
    { 0, 0 },
    { 0xc06daa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b56daa4, 66 },
    { 0, 0 },
    { 0x67a6daa6, 43 },
    { 0x8da6daa7, 68 },
    { 0xe4c6daa6, 66 },
    { 0x5f76daa9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e6daac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6306dab5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6796dab9, 67 },
    { 0x3826dab9, 95 },
    { 0x3156dabb, 93 },
    { 0xbde6dabb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b6dac0, 33 },
    { 0x7706dac1, 43 },
    { 0xbef6dac2, 33 },
    { 0x1ac6dac1, 90 },
    { 0, 0 },
    { 0x8466dac5, 46 },
    { 0, 0 },
    { 0xb1b6dac7, 43 },
    { 0xe1f6dac8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a6dacb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f6dace, 37 },
    { 0xf0b6dacf, 33 },
    { 0x1b16dad0, 60 },
    { 0, 0 },
    { 0xdc26dad2, 33 },
    { 0x846dad3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8216dade, 95 },
    { 0x18e6dade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d6dae2, 55 },
    { 0x8096dae3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a26daed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f46daf0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4086daf5, 84 },
    { 0xde26daf6, 61 },
    { 0x7266daf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d6dafa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc06dafd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c6db04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea6db0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6946db11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f6db16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6db1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7466db22, 95 },
    { 0x18e6db23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c66db27, 34 },
    { 0xbf06db28, 17 },
    { 0x5f56db29, 23 },
    { 0x21e6db29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4186db3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1596db40, 78 },
    { 0xcb76db41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc296db45, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad56db49, 88 },
    { 0xcbc6db4a, 61 },
    { 0x666db4a, 90 },
    { 0x27b6db4c, 81 },
    { 0xd7f6db4c, 43 },
    { 0x126db4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6db54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1326db5a, 90 },
    { 0x80a6db5b, 85 },
    { 0, 0 },
    { 0x3706db5d, 78 },
    { 0xea86db5e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee6db61, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b6db64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc6db67, 33 },
    { 0x2416db68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b76db6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f6db76, 77 },
    { 0xc4a6db77, 78 },
    { 0x9b96db78, 55 },
    { 0x8706db79, 93 },
    { 0x6396db7a, 90 },
    { 0x496db77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b6db7f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b96db82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce6db86, 61 },
    { 0x9466db86, 90 },
    { 0, 0 },
    { 0x9b06db89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c76db8d, 23 },
    { 0x7d06db8e, 43 },
    { 0, 0 },
    { 0xcf66db90, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56db96, 98 },
    { 0, 0 },
    { 0x5a06db98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be6dbab, 90 },
    { 0xae56dbab, 87 },
    { 0xf0b6dbac, 34 },
    { 0, 0 },
    { 0xc546dbaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce86dbb2, 96 },
    { 0x26dbb3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb6dbb6, 55 },
    { 0, 0 },
    { 0xc336dbb8, 90 },
    { 0x3e46dbb8, 43 },
    { 0x1676dbb9, 90 },
    { 0, 0 },
    { 0x2426dbbc, 75 },
    { 0, 0 },
    { 0x2296dbbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56dbc1, 46 },
    { 0x6f86dbc2, 61 },
    { 0xb096dbc2, 100 },
    { 0xe046dbc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6836dbc7, 76 },
    { 0, 0 },
    { 0x7f76dbc9, 61 },
    { 0xd516dbca, 33 },
    { 0, 0 },
    { 0x7946dbcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f6dbd1, 87 },
    { 0, 0 },
    { 0xb426dbd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf6dbd7, 34 },
    { 0xf626dbd8, 55 },
    { 0xb1b6dbd9, 85 },
    { 0, 0 },
    { 0x79c6dbdb, 66 },
    { 0x8c66dbdc, 55 },
    { 0xfe46dbdd, 95 },
    { 0x4296dbde, 34 },
    { 0xad86dbdf, 78 },
    { 0, 0 },
    { 0xfe96dbe1, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b6dbe4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d16dbeb, 85 },
    { 0x1ca6dbec, 90 },
    { 0x8b06dbed, 90 },
    { 0x3e86dbed, 61 },
    { 0xbb16dbef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa096dbf3, 33 },
    { 0x66d6dbf3, 23 },
    { 0xc7a6dbf5, 23 },
    { 0x1e6dbf6, 55 },
    { 0x3c76dbf7, 90 },
    { 0, 0 },
    { 0x476dbf9, 76 },
    { 0x2e06dbf9, 61 },
    { 0, 0 },
    { 0x8c16dbfc, 95 },
    { 0x44a6dbfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f66dc05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb536dc0a, 83 },
    { 0x1696dc0a, 98 },
    { 0x2026dc0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6dc0f, 23 },
    { 0x83f6dc0f, 68 },
    { 0x5dd6dc11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c6dc14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd486dc1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3826dc1e, 23 },
    { 0, 0 },
    { 0xfad6dc20, 61 },
    { 0x50a6dc21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1236dc28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef16dc2b, 66 },
    { 0, 0 },
    { 0xf2a6dc2d, 33 },
    { 0, 0 },
    { 0x3ab6dc2f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c6dc34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2206dc37, 90 },
    { 0, 0 },
    { 0xf9e6dc39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6dc3c, 22 },
    { 0, 0 },
    { 0xc466dc3e, 95 },
    { 0x4c26dc3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf16dc43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf766dc47, 95 },
    { 0x3496dc48, 98 },
    { 0xf556dc49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6dc52, 87 },
    { 0xf236dc53, 61 },
    { 0x5b06dc53, 23 },
    { 0, 0 },
    { 0x52b6dc56, 95 },
    { 0x43d6dc56, 34 },
    { 0x7086dc57, 61 },
    { 0xc76dc59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd66dc5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa076dc61, 23 },
    { 0, 0 },
    { 0xe806dc63, 48 },
    { 0, 0 },
    { 0x2b06dc65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6dc69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1266dc6c, 33 },
    { 0xcf06dc6c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956dc73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06dc76, 95 },
    { 0xf306dc76, 99 },
    { 0x4e36dc78, 61 },
    { 0x4d46dc79, 61 },
    { 0xd696dc79, 90 },
    { 0xb8c6dc7b, 93 },
    { 0xd036dc7b, 98 },
    { 0xe776dc7c, 90 },
    { 0x1566dc7e, 33 },
    { 0x4c96dc7b, 100 },
    { 0x78f6dc78, 98 },
    { 0xc206dc81, 61 },
    { 0xfee6dc77, 87 },
    { 0, 0 },
    { 0xc846dc84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3576dc88, 95 },
    { 0xa396dc89, 23 },
    { 0xedb6dc8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf436dc8f, 76 },
    { 0x1606dc90, 23 },
    { 0x526dc91, 33 },
    { 0xb2d6dc92, 76 },
    { 0xe216dc91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa96dc98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6776dc9c, 76 },
    { 0x4646dc9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf6dca2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc076dca9, 33 },
    { 0, 0 },
    { 0xa426dcab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2356dcae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad26dcb2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c06dcb8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46dcbe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8336dcc3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4166dcc6, 43 },
    { 0xe0e6dcc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a6dcca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6dcd0, 34 },
    { 0x3c6dcd0, 95 },
    { 0xf616dcd0, 95 },
    { 0xa746dcd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b6dcd7, 98 },
    { 0, 0 },
    { 0xbb96dcd9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856dcdd, 43 },
    { 0, 0 },
    { 0x9d6dcdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8716dce2, 23 },
    { 0, 0 },
    { 0x98d6dce4, 61 },
    { 0xee36dce5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86dcec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6276dcef, 87 },
    { 0, 0 },
    { 0xc3d6dcf1, 95 },
    { 0x1df6dcf2, 90 },
    { 0x66e6dcf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff06dcf8, 67 },
    { 0x2c86dcf9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd966dcfd, 98 },
    { 0x1786dcfd, 90 },
    { 0, 0 },
    { 0x2036dd00, 95 },
    { 0x4766dd00, 33 },
    { 0x76dd01, 67 },
    { 0xe66dd03, 33 },
    { 0x7246dd03, 23 },
    { 0xb06dd05, 90 },
    { 0, 0 },
    { 0x1126dd07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586dd0d, 90 },
    { 0, 0 },
    { 0x8056dd0f, 67 },
    { 0x8516dd10, 65 },
    { 0, 0 },
    { 0x9c76dd12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd106dd1c, 52 },
    { 0xa8a6dd1d, 90 },
    { 0xb606dd1e, 90 },
    { 0xd7d6dd1c, 95 },
    { 0x7316dd20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc6dd2b, 66 },
    { 0xdd06dd2c, 76 },
    { 0x44c6dd2d, 75 },
    { 0xe3d6dd2d, 76 },
    { 0xad36dd2f, 91 },
    { 0x46c6dd2f, 55 },
    { 0, 0 },
    { 0xd0d6dd32, 23 },
    { 0x7836dd32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc336dd38, 90 },
    { 0, 0 },
    { 0x3a56dd3a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x5576dd3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe796dd40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16dd43, 23 },
    { 0x51b6dd43, 46 },
    { 0xfe46dd45, 84 },
    { 0xea56dd45, 55 },
    { 0xdc76dd47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe336dd4e, 76 },
    { 0xa16dd4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde6dd56, 61 },
    { 0, 0 },
    { 0x1226dd58, 33 },
    { 0x2666dd59, 95 },
    { 0xbf96dd58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7596dd62, 61 },
    { 0x2e66dd62, 46 },
    { 0, 0 },
    { 0x5d86dd65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f6dd68, 33 },
    { 0, 0 },
    { 0x5146dd6a, 61 },
    { 0x1746dd6a, 55 },
    { 0, 0 },
    { 0x1206dd6d, 23 },
    { 0xe2b6dd6d, 43 },
    { 0, 0 },
    { 0xdb6dd70, 33 },
    { 0xf906dd71, 17 },
    { 0x4796dd71, 52 },
    { 0xf486dd71, 76 },
    { 0x2346dd74, 33 },
    { 0x1026dd73, 84 },
    { 0xcca6dd76, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a6dd7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c06dd83, 90 },
    { 0x2236dd84, 43 },
    { 0x6296dd84, 76 },
    { 0x82b6dd86, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1156dd8e, 61 },
    { 0xb1b6dd8f, 76 },
    { 0x99f6dd90, 91 },
    { 0, 0 },
    { 0x3036dd92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c6dd95, 61 },
    { 0, 0 },
    { 0xf0a6dd97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb576dd9a, 43 },
    { 0x3146dd9a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8116dda2, 87 },
    { 0x7286dda3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f6dda7, 76 },
    { 0x4376dda7, 98 },
    { 0, 0 },
    { 0xc7d6ddaa, 80 },
    { 0xd386ddaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2846ddae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9006ddb4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b6ddb7, 81 },
    { 0xed86ddb8, 61 },
    { 0, 0 },
    { 0x71d6ddba, 85 },
    { 0x27f6ddba, 90 },
    { 0x4646ddbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6ddc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad16ddca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2486ddd1, 76 },
    { 0xc56ddd1, 99 },
    { 0, 0 },
    { 0xf366ddd4, 61 },
    { 0, 0 },
    { 0x4ac6ddd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec36ddda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb316dde1, 55 },
    { 0x8fe6dde2, 98 },
    { 0xd66dde2, 55 },
    { 0xda96dde3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed76ddea, 90 },
    { 0x8426ddeb, 98 },
    { 0x89f6ddec, 81 },
    { 0xed56ddeb, 33 },
    { 0x6886ddeb, 90 },
    { 0x2716ddec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd6ddf2, 23 },
    { 0xdd96ddf3, 46 },
    { 0xf376ddf4, 39 },
    { 0, 0 },
    { 0xae66ddf6, 43 },
    { 0, 0 },
    { 0x266ddf8, 90 },
    { 0xb2e6ddf8, 55 },
    { 0xf8b6ddfa, 34 },
    { 0x23c6ddfa, 55 },
    { 0x90b6ddfc, 75 },
    { 0x1de6ddfd, 77 },
    { 0xdb76ddfd, 33 },
    { 0x3946ddfe, 90 },
    { 0xae86de00, 46 },
    { 0, 0 },
    { 0x2f56de02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb276de05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb66de08, 90 },
    { 0xfaa6de09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9406de10, 95 },
    { 0, 0 },
    { 0x94e6de12, 76 },
    { 0x1836de12, 46 },
    { 0, 0 },
    { 0xa4d6de15, 61 },
    { 0, 0 },
    { 0x5eb6de17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe526de1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a6de21, 95 },
    { 0x5586de22, 23 },
    { 0x8006de23, 23 },
    { 0xa066de24, 34 },
    { 0xc636de24, 33 },
    { 0x5066de25, 23 },
    { 0x55c6de27, 23 },
    { 0x7776de27, 61 },
    { 0, 0 },
    { 0x8ab6de2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7886de2d, 33 },
    { 0, 0 },
    { 0xc836de2f, 101 },
    { 0xe6f6de30, 76 },
    { 0xd646de2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5876de34, 95 },
    { 0, 0 },
    { 0xf36de36, 40 },
    { 0x1dd6de36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a6de3d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb46de40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb6de44, 61 },
    { 0, 0 },
    { 0x6e76de46, 61 },
    { 0, 0 },
    { 0xa9c6de48, 95 },
    { 0xb316de49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c6de4c, 95 },
    { 0xc6f6de4d, 84 },
    { 0xbe26de4d, 34 },
    { 0x7a16de4d, 61 },
    { 0, 0 },
    { 0x4806de51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f96de54, 23 },
    { 0x4296de55, 99 },
    { 0x96a6de56, 33 },
    { 0x3456de57, 34 },
    { 0x8f36de57, 90 },
    { 0x6396de58, 95 },
    { 0xcd56de56, 98 },
    { 0, 0 },
    { 0xf46de5c, 32 },
    { 0xf4c6de5c, 90 },
    { 0xad26de5d, 23 },
    { 0, 0 },
    { 0x4316de60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb706de63, 61 },
    { 0x13b6de64, 95 },
    { 0x9ec6de65, 90 },
    { 0xf536de66, 61 },
    { 0xbbf6de66, 98 },
    { 0x2756de64, 61 },
    { 0x6a46de69, 95 },
    { 0x5c26de6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9976de71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd906de77, 77 },
    { 0, 0 },
    { 0x17d6de79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1176de7d, 99 },
    { 0, 0 },
    { 0x2aa6de7f, 68 },
    { 0xcaf6de7f, 90 },
    { 0x4676de80, 90 },
    { 0xf7e6de82, 95 },
    { 0x9196de82, 95 },
    { 0x7f36de82, 95 },
    { 0xe956de85, 46 },
    { 0, 0 },
    { 0xebe6de87, 90 },
    { 0, 0 },
    { 0x1816de89, 98 },
    { 0x21b6de8a, 33 },
    { 0xaa26de8b, 85 },
    { 0, 0 },
    { 0xc086de8d, 40 },
    { 0x7196de8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc56de97, 98 },
    { 0, 0 },
    { 0x5b36de99, 87 },
    { 0x3bc6de99, 98 },
    { 0x5ae6de9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5236de9f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1386deaa, 90 },
    { 0xeef6deab, 55 },
    { 0, 0 },
    { 0x4946dead, 95 },
    { 0xcd6dead, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab6deb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c96debb, 61 },
    { 0x6916debc, 61 },
    { 0xc956debb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3956dec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f6dec6, 67 },
    { 0xd616dec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9516decd, 61 },
    { 0, 0 },
    { 0xfec6decf, 98 },
    { 0, 0 },
    { 0x3686ded1, 95 },
    { 0x6996ded2, 23 },
    { 0x2a66ded2, 87 },
    { 0, 0 },
    { 0x8f96ded5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6ded8, 49 },
    { 0x3c26ded8, 91 },
    { 0, 0 },
    { 0x8486dedb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf836dee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x916dee5, 23 },
    { 0x3416dee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f76deea, 90 },
    { 0, 0 },
    { 0xf7d6deec, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe406deef, 33 },
    { 0x9f16def0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe296def6, 61 },
    { 0xd026def7, 34 },
    { 0xaa6def8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d06defe, 61 },
    { 0x3576defe, 55 },
    { 0x3536df00, 52 },
    { 0xc496df01, 90 },
    { 0x8f06defe, 84 },
    { 0, 0 },
    { 0x59f6df04, 77 },
    { 0, 0 },
    { 0x12a6df06, 68 },
    { 0, 0 },
    { 0xc016df08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd6df0d, 23 },
    { 0, 0 },
    { 0xb226df0f, 98 },
    { 0x1bd6df10, 84 },
    { 0x7ae6df10, 81 },
    { 0xc56df12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2106df15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd6df19, 33 },
    { 0, 0 },
    { 0xbeb6df1b, 23 },
    { 0x6d66df1c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd6df22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5536df29, 88 },
    { 0xc126df2a, 87 },
    { 0, 0 },
    { 0xa166df2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd6df36, 61 },
    { 0, 0 },
    { 0x1bc6df38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1996df3d, 33 },
    { 0xc426df3e, 87 },
    { 0x9156df3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e46df42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab66df45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126df4a, 90 },
    { 0x2d96df4b, 66 },
    { 0xcb36df4c, 56 },
    { 0x1956df4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f86df53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b6df5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e6df5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3566df61, 23 },
    { 0, 0 },
    { 0x98b6df63, 69 },
    { 0xf616df64, 46 },
    { 0x60c6df65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb166df6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2346df74, 95 },
    { 0x7886df75, 23 },
    { 0xc3b6df76, 23 },
    { 0xeee6df77, 43 },
    { 0, 0 },
    { 0x3ea6df79, 23 },
    { 0xc8d6df7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad6df7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c6df89, 33 },
    { 0, 0 },
    { 0x9a66df8b, 81 },
    { 0x4ba6df8c, 76 },
    { 0x8896df8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f6df94, 90 },
    { 0, 0 },
    { 0x8156df96, 98 },
    { 0xcbf6df97, 17 },
    { 0xfe16df97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe896df9b, 23 },
    { 0, 0 },
    { 0x82f6df9d, 33 },
    { 0xfa46df9e, 91 },
    { 0x1426df9d, 100 },
    { 0x27f6dfa0, 95 },
    { 0xed16dfa1, 23 },
    { 0x4046dfa2, 68 },
    { 0x1d36dfa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c6dfa6, 76 },
    { 0x5cf6dfa7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc76dfad, 90 },
    { 0xa46dfae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa776dfb4, 91 },
    { 0x95c6dfb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a6dfb8, 90 },
    { 0, 0 },
    { 0xe796dfba, 95 },
    { 0, 0 },
    { 0x8cc6dfbc, 23 },
    { 0, 0 },
    { 0x95a6dfbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c56dfc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b36dfc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e56dfce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe616dfd1, 91 },
    { 0xebb6dfd2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f36dfd7, 64 },
    { 0x236dfd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc6dfdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa26dfde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b16dfe8, 95 },
    { 0xd246dfe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c6dfec, 76 },
    { 0xb0c6dfed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b16dff5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9786dff8, 46 },
    { 0xb6d6dff8, 33 },
    { 0, 0 },
    { 0x97a6dffb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf966dffe, 33 },
    { 0xb5d6dfff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf6e007, 46 },
    { 0, 0 },
    { 0xc826e009, 91 },
    { 0x55b6e00a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe146e010, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b6e014, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5716e017, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276e01f, 90 },
    { 0xeec6e020, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9546e02a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa86e02e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8526e032, 95 },
    { 0xda66e033, 90 },
    { 0xb346e033, 33 },
    { 0xea96e035, 33 },
    { 0x46f6e036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa696e03c, 88 },
    { 0x5556e03c, 33 },
    { 0xfc46e03e, 23 },
    { 0xdf36e03d, 23 },
    { 0xb5b6e040, 34 },
    { 0, 0 },
    { 0xce56e042, 76 },
    { 0, 0 },
    { 0x1f46e044, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b6e047, 34 },
    { 0, 0 },
    { 0x44e6e049, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc76e04c, 43 },
    { 0, 0 },
    { 0x8fd6e04e, 33 },
    { 0x99e6e04f, 95 },
    { 0x3246e050, 33 },
    { 0x226e051, 95 },
    { 0x7ac6e04f, 34 },
    { 0xd896e053, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3126e056, 23 },
    { 0x7376e057, 52 },
    { 0, 0 },
    { 0x97e6e059, 34 },
    { 0x3356e05a, 92 },
    { 0x4556e05b, 90 },
    { 0, 0 },
    { 0xf906e05d, 90 },
    { 0, 0 },
    { 0x30e6e05f, 61 },
    { 0, 0 },
    { 0x5196e061, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136e065, 52 },
    { 0, 0 },
    { 0xefc6e067, 61 },
    { 0x6136e068, 100 },
    { 0x956e069, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6e06c, 90 },
    { 0x3e56e06c, 55 },
    { 0xaa46e06e, 23 },
    { 0xf6c6e06f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a56e079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7856e07c, 52 },
    { 0, 0 },
    { 0x9936e07e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd46e08a, 33 },
    { 0x9716e08a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a6e08e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7106e098, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6846e0a0, 34 },
    { 0x54d6e0a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af6e0a5, 90 },
    { 0, 0 },
    { 0x5e46e0a7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a6e0ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f96e0b4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a6e0b9, 33 },
    { 0xeb6e0b9, 23 },
    { 0xfb36e0b9, 33 },
    { 0x9186e0bb, 90 },
    { 0, 0 },
    { 0x7db6e0be, 93 },
    { 0xaf36e0be, 90 },
    { 0xd56e0c0, 52 },
    { 0x46e6e0c1, 55 },
    { 0xa3f6e0c2, 90 },
    { 0xf206e0c1, 43 },
    { 0x3226e0c4, 46 },
    { 0x926e0c5, 84 },
    { 0x77e6e0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7926e0c9, 98 },
    { 0x51f6e0ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c6e0ce, 61 },
    { 0x6386e0cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a56e0d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f6e0d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b56e0dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e6e0e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b56e0e4, 76 },
    { 0xf276e0e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6e0ee, 85 },
    { 0, 0 },
    { 0xe636e0f0, 88 },
    { 0xc476e0f1, 43 },
    { 0, 0 },
    { 0x7976e0f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb256e0fd, 43 },
    { 0, 0 },
    { 0xf126e0ff, 33 },
    { 0, 0 },
    { 0x1186e101, 61 },
    { 0xa386e102, 93 },
    { 0x5036e103, 95 },
    { 0, 0 },
    { 0x34a6e105, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6e10e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be6e113, 52 },
    { 0, 0 },
    { 0xe7b6e115, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e56e119, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b86e11e, 46 },
    { 0xbda6e11e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9686e122, 98 },
    { 0x5f06e123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6e126, 67 },
    { 0x2016e127, 43 },
    { 0, 0 },
    { 0x706e129, 87 },
    { 0, 0 },
    { 0x6756e12b, 52 },
    { 0x7da6e12b, 98 },
    { 0, 0 },
    { 0x4fc6e12e, 68 },
    { 0xf716e12f, 90 },
    { 0xa8e6e130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae66e134, 76 },
    { 0, 0 },
    { 0xc3b6e136, 91 },
    { 0, 0 },
    { 0x44d6e138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd156e13b, 76 },
    { 0x90d6e13b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee6e13f, 95 },
    { 0x6c76e13f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9356e144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb956e147, 90 },
    { 0x5206e148, 90 },
    { 0, 0 },
    { 0xc8d6e14a, 55 },
    { 0, 0 },
    { 0x9496e14c, 76 },
    { 0xcc86e14c, 23 },
    { 0, 0 },
    { 0x81d6e14f, 61 },
    { 0x4fe6e150, 61 },
    { 0x5d56e151, 23 },
    { 0xc2c6e14f, 78 },
    { 0x5ab6e153, 93 },
    { 0xb0d6e154, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c6e158, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f6e15d, 95 },
    { 0, 0 },
    { 0xa306e15f, 93 },
    { 0x2c66e160, 34 },
    { 0x9c6e160, 68 },
    { 0x6726e160, 55 },
    { 0, 0 },
    { 0x18f6e164, 69 },
    { 0xfc66e164, 90 },
    { 0x6116e166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe796e16a, 34 },
    { 0, 0 },
    { 0x9896e16c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2176e16f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd86e172, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d36e177, 90 },
    { 0x6bf6e177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6e17b, 61 },
    { 0x2856e17c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe826e180, 33 },
    { 0x5b56e181, 76 },
    { 0, 0 },
    { 0xbe36e183, 95 },
    { 0, 0 },
    { 0x3a86e185, 96 },
    { 0x9936e186, 95 },
    { 0x10a6e187, 33 },
    { 0xd056e188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe566e18d, 77 },
    { 0x8656e18e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8726e193, 98 },
    { 0xfde6e193, 61 },
    { 0, 0 },
    { 0x6866e196, 90 },
    { 0, 0 },
    { 0xd756e198, 98 },
    { 0x9696e199, 95 },
    { 0x5446e199, 23 },
    { 0xba6e19b, 78 },
    { 0x5826e19c, 90 },
    { 0xd876e19a, 99 },
    { 0x4006e19d, 90 },
    { 0x6426e19f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1706e1a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d66e1a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc76e1a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6736e1ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7696e1b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc436e1b7, 43 },
    { 0x5016e1b8, 95 },
    { 0xa4a6e1b9, 23 },
    { 0x21a6e1ba, 99 },
    { 0x6506e1ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8446e1c0, 90 },
    { 0, 0 },
    { 0x52d6e1c2, 34 },
    { 0, 0 },
    { 0xa556e1c4, 61 },
    { 0x6746e1c5, 61 },
    { 0, 0 },
    { 0x9cc6e1c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6e1ca, 91 },
    { 0xc376e1ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd516e1ce, 52 },
    { 0xb926e1cf, 33 },
    { 0x1586e1d0, 64 },
    { 0x4b96e1d0, 93 },
    { 0x3816e1d0, 95 },
    { 0x85b6e1d2, 90 },
    { 0xd6d6e1d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e06e1d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4826e1dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2556e1df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a86e1e2, 76 },
    { 0x7876e1e3, 67 },
    { 0xf6e6e1e2, 65 },
    { 0x6ed6e1e2, 90 },
    { 0x55a6e1e4, 98 },
    { 0xad86e1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b76e1ee, 61 },
    { 0x1a6e1ef, 88 },
    { 0x79d6e1ee, 93 },
    { 0, 0 },
    { 0x8fe6e1f2, 61 },
    { 0xfbd6e1f2, 76 },
    { 0, 0 },
    { 0x22b6e1f5, 98 },
    { 0, 0 },
    { 0xe9b6e1f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5216e1fb, 90 },
    { 0, 0 },
    { 0xcd96e1fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b96e202, 56 },
    { 0, 0 },
    { 0xac6e204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc766e207, 90 },
    { 0xa226e208, 43 },
    { 0, 0 },
    { 0x7cb6e20a, 88 },
    { 0x8f96e20a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b06e212, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8726e217, 33 },
    { 0x4f16e217, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6e21c, 87 },
    { 0xd286e21d, 34 },
    { 0xd376e21e, 55 },
    { 0, 0 },
    { 0x8056e220, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d6e22a, 55 },
    { 0x3e96e22b, 43 },
    { 0x4f76e22b, 90 },
    { 0x3dd6e22c, 55 },
    { 0x3526e22d, 90 },
    { 0x9da6e22d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1496e235, 98 },
    { 0x2706e235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f6e239, 23 },
    { 0x22e6e239, 88 },
    { 0x3c16e23b, 98 },
    { 0xd196e239, 78 },
    { 0x2d96e23d, 90 },
    { 0x15a6e23e, 33 },
    { 0x16d6e23f, 90 },
    { 0xb46e240, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5056e24b, 98 },
    { 0x3226e24c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad66e251, 67 },
    { 0xf856e251, 90 },
    { 0x2db6e253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496e256, 61 },
    { 0x1336e257, 87 },
    { 0x65e6e257, 34 },
    { 0xc166e259, 98 },
    { 0, 0 },
    { 0xf276e25b, 69 },
    { 0xdc36e25c, 36 },
    { 0x6356e25c, 98 },
    { 0xba86e25d, 91 },
    { 0, 0 },
    { 0x5666e260, 98 },
    { 0x5a36e260, 95 },
    { 0x6486e262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c46e266, 33 },
    { 0x7376e266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8276e26f, 98 },
    { 0xe8d6e270, 99 },
    { 0x8326e271, 66 },
    { 0xd816e270, 81 },
    { 0x9276e270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf266e276, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e6e27d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1996e282, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc6e28a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf256e28e, 33 },
    { 0x98f6e28e, 93 },
    { 0xf796e28e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86e298, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c66e29b, 98 },
    { 0x4216e29c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1386e2a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf6e2a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a56e2ad, 95 },
    { 0, 0 },
    { 0xffd6e2af, 99 },
    { 0, 0 },
    { 0x52f6e2b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e2b4, 90 },
    { 0xa7e6e2b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc136e2b9, 61 },
    { 0, 0 },
    { 0xc056e2bb, 93 },
    { 0, 0 },
    { 0x9386e2bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56e2cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d6e2cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6486e2d2, 43 },
    { 0xc096e2d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986e2df, 67 },
    { 0xb536e2df, 90 },
    { 0x9876e2e0, 33 },
    { 0x3cd6e2e2, 43 },
    { 0x4156e2e1, 55 },
    { 0, 0 },
    { 0x1cb6e2e5, 76 },
    { 0x1bb6e2e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7996e2e9, 33 },
    { 0xe986e2ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6e2f0, 68 },
    { 0, 0 },
    { 0xd006e2f2, 98 },
    { 0x1cc6e2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde26e2f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e6e2fa, 48 },
    { 0xf4e6e2fa, 33 },
    { 0xdf46e2fc, 90 },
    { 0x4ec6e2fa, 33 },
    { 0, 0 },
    { 0x9ea6e2ff, 85 },
    { 0x8cb6e300, 98 },
    { 0, 0 },
    { 0x7746e302, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f6e305, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f6e308, 52 },
    { 0x27e6e309, 61 },
    { 0, 0 },
    { 0x74b6e30b, 90 },
    { 0xc9e6e30c, 66 },
    { 0xbb66e30c, 33 },
    { 0, 0 },
    { 0xfba6e30f, 84 },
    { 0x7c06e310, 88 },
    { 0xdf86e30f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66e315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8996e31c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e6e320, 61 },
    { 0x356e321, 61 },
    { 0x5356e321, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6e32b, 66 },
    { 0xf156e32c, 43 },
    { 0xf926e32b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4126e331, 55 },
    { 0xcbe6e331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe26e336, 90 },
    { 0x1c6e336, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5126e33a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626e341, 90 },
    { 0x3a96e341, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb356e350, 55 },
    { 0xe496e350, 95 },
    { 0xbad6e351, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb6e361, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a66e368, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea66e36b, 100 },
    { 0x1ed6e36c, 55 },
    { 0x6186e36c, 61 },
    { 0x9c06e36d, 98 },
    { 0xcce6e36e, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456e379, 99 },
    { 0xcaf6e379, 43 },
    { 0x8776e37a, 84 },
    { 0x8526e37b, 84 },
    { 0x4076e37a, 98 },
    { 0x59c6e37e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb46e382, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb56e38a, 95 },
    { 0, 0 },
    { 0x7d6e38c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4236e38f, 98 },
    { 0, 0 },
    { 0xac06e391, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d16e396, 83 },
    { 0, 0 },
    { 0xdbb6e398, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e6e39c, 33 },
    { 0xb746e39c, 66 },
    { 0, 0 },
    { 0x4d56e39f, 98 },
    { 0xe06e3a0, 34 },
    { 0xdff6e3a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd6e3a4, 43 },
    { 0, 0 },
    { 0x12b6e3a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e6e3ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5066e3b0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b6e3b3, 87 },
    { 0, 0 },
    { 0x1636e3b5, 33 },
    { 0x5f96e3b6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f6e3bb, 100 },
    { 0x1e86e3bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c6e3c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16e3c7, 90 },
    { 0x96f6e3c8, 88 },
    { 0x20e6e3c7, 99 },
    { 0x5946e3ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d76e3d2, 46 },
    { 0x7fd6e3d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd46e3d7, 61 },
    { 0, 0 },
    { 0x8d26e3d9, 90 },
    { 0x7306e3da, 76 },
    { 0, 0 },
    { 0xc866e3dc, 96 },
    { 0x8ea6e3dc, 85 },
    { 0xcb96e3de, 87 },
    { 0x9f26e3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd6e3e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd6e3e9, 23 },
    { 0, 0 },
    { 0x7446e3eb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e6e3fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9836e3fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6e403, 55 },
    { 0, 0 },
    { 0x3fa6e405, 99 },
    { 0x766e405, 66 },
    { 0x7646e406, 75 },
    { 0xefe6e406, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf6e40b, 33 },
    { 0x2886e40c, 46 },
    { 0x806e40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c6e410, 33 },
    { 0x82a6e411, 77 },
    { 0xe976e411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xef66e415, 23 },
    { 0xae46e415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc576e422, 69 },
    { 0, 0 },
    { 0x9616e424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd226e42b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a6e42e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7306e436, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c76e43d, 93 },
    { 0x9a36e43d, 100 },
    { 0x5ec6e43f, 95 },
    { 0x6fa6e440, 33 },
    { 0xe116e440, 98 },
    { 0x9f66e440, 34 },
    { 0xb9d6e443, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d6e44a, 43 },
    { 0x9be6e44a, 52 },
    { 0x5f66e44c, 95 },
    { 0xca06e44c, 90 },
    { 0xc436e44e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f6e453, 99 },
    { 0xc736e454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b6e45f, 42 },
    { 0x57e6e460, 95 },
    { 0x81f6e45f, 99 },
    { 0, 0 },
    { 0xd826e463, 33 },
    { 0, 0 },
    { 0xde96e465, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf636e469, 23 },
    { 0x5646e46a, 61 },
    { 0xac26e46b, 95 },
    { 0, 0 },
    { 0x1046e46d, 95 },
    { 0xb5e6e46e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e06e471, 66 },
    { 0, 0 },
    { 0x98e6e473, 46 },
    { 0x18d6e474, 87 },
    { 0xd076e474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a6e478, 61 },
    { 0, 0 },
    { 0xa246e47a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e6e488, 95 },
    { 0xa9b6e489, 90 },
    { 0, 0 },
    { 0x4046e48b, 33 },
    { 0xda96e48c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e86e48f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606e492, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b6e495, 91 },
    { 0, 0 },
    { 0xa106e497, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea6e49a, 97 },
    { 0x6e56e49b, 61 },
    { 0xda56e49a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc326e4a0, 90 },
    { 0xa516e4a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a6e4a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e4aa, 95 },
    { 0x5196e4aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5176e4b4, 34 },
    { 0, 0 },
    { 0x50d6e4b6, 34 },
    { 0x2576e4b7, 85 },
    { 0xf2f6e4b7, 61 },
    { 0x6456e4b8, 43 },
    { 0xac36e4ba, 98 },
    { 0x63b6e4bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b76e4c1, 66 },
    { 0x726e4c2, 98 },
    { 0x8bd6e4c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b6e4c7, 53 },
    { 0xc8c6e4c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6e4cb, 77 },
    { 0xfb56e4cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc6e4cf, 33 },
    { 0xde6e4d0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e76e4d3, 55 },
    { 0x7886e4d4, 85 },
    { 0, 0 },
    { 0x65a6e4d6, 79 },
    { 0x1456e4d7, 90 },
    { 0xbc56e4d7, 61 },
    { 0x7486e4d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96e4df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a6e4e6, 85 },
    { 0xa9f6e4e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f06e4ea, 76 },
    { 0x7106e4eb, 61 },
    { 0x12c6e4ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd76e4f7, 66 },
    { 0x3556e4f8, 23 },
    { 0, 0 },
    { 0x6a96e4fa, 61 },
    { 0xa4a6e4fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac6e4fe, 84 },
    { 0, 0 },
    { 0xb86e500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586e506, 61 },
    { 0, 0 },
    { 0x7a06e508, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b96e50b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876e510, 91 },
    { 0xc966e510, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e514, 61 },
    { 0x6076e515, 90 },
    { 0xadf6e514, 90 },
    { 0xf196e517, 84 },
    { 0x8fd6e517, 33 },
    { 0x36e519, 88 },
    { 0, 0 },
    { 0xd696e51b, 76 },
    { 0, 0 },
    { 0x4426e51d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7996e528, 90 },
    { 0x70d6e529, 76 },
    { 0, 0 },
    { 0xdc86e52b, 91 },
    { 0, 0 },
    { 0x5626e52d, 61 },
    { 0x3976e52e, 23 },
    { 0x2036e52f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac6e532, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf576e537, 61 },
    { 0x5c6e537, 33 },
    { 0xcb16e538, 95 },
    { 0x40a6e53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3306e53e, 98 },
    { 0, 0 },
    { 0xa3a6e540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d6e543, 61 },
    { 0x49f6e544, 77 },
    { 0x9c06e544, 95 },
    { 0x4596e546, 55 },
    { 0x3b46e547, 67 },
    { 0x3a86e548, 61 },
    { 0x74e6e549, 34 },
    { 0, 0 },
    { 0x4546e54b, 34 },
    { 0x9586e54b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d6e551, 87 },
    { 0x316e552, 83 },
    { 0, 0 },
    { 0x3996e554, 87 },
    { 0x41c6e555, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5816e558, 94 },
    { 0x16a6e558, 53 },
    { 0x7cf6e55a, 61 },
    { 0x8936e55b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f6e561, 61 },
    { 0xe346e562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4116e566, 55 },
    { 0x5c76e567, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466e56b, 90 },
    { 0xebe6e56b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa246e570, 98 },
    { 0x946e571, 83 },
    { 0xc4c6e571, 67 },
    { 0x3146e571, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3926e57b, 95 },
    { 0x7db6e57c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc6e581, 23 },
    { 0xaa16e581, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3706e589, 75 },
    { 0, 0 },
    { 0xf736e58b, 61 },
    { 0, 0 },
    { 0x43e6e58d, 95 },
    { 0, 0 },
    { 0x2776e58f, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a6e594, 33 },
    { 0x7cb6e595, 90 },
    { 0x6056e595, 90 },
    { 0x1966e594, 61 },
    { 0, 0 },
    { 0x2906e599, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1416e59c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa6e5a1, 61 },
    { 0x9d36e5a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e6e5a7, 78 },
    { 0, 0 },
    { 0xca46e5a9, 90 },
    { 0x35b6e5aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6e5b3, 61 },
    { 0, 0 },
    { 0x31a6e5b5, 33 },
    { 0x9816e5b5, 61 },
    { 0x56e5b7, 61 },
    { 0xe96e5b7, 56 },
    { 0x366e5b8, 98 },
    { 0x61f6e5ba, 88 },
    { 0xf126e5b7, 90 },
    { 0x5af6e5bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1766e5bf, 98 },
    { 0, 0 },
    { 0xaa66e5c1, 90 },
    { 0x6026e5c2, 34 },
    { 0, 0 },
    { 0x686e5c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6136e5d0, 43 },
    { 0x5976e5d1, 33 },
    { 0x5cf6e5d1, 38 },
    { 0, 0 },
    { 0xe006e5d4, 61 },
    { 0, 0 },
    { 0x3db6e5d6, 66 },
    { 0xb326e5d6, 90 },
    { 0x5046e5d7, 85 },
    { 0xf826e5d8, 100 },
    { 0, 0 },
    { 0x4706e5db, 98 },
    { 0x59b6e5db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a6e5df, 88 },
    { 0xba66e5e0, 96 },
    { 0, 0 },
    { 0x51b6e5e2, 90 },
    { 0xb426e5e3, 69 },
    { 0x5366e5e4, 100 },
    { 0xafa6e5e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e6e5ee, 99 },
    { 0, 0 },
    { 0x8776e5f0, 33 },
    { 0xb066e5f1, 33 },
    { 0, 0 },
    { 0xa516e5f3, 23 },
    { 0, 0 },
    { 0xa206e5f5, 52 },
    { 0x6916e5f6, 61 },
    { 0, 0 },
    { 0xb96e5f8, 33 },
    { 0x2c36e5f9, 100 },
    { 0, 0 },
    { 0x1f56e5fb, 95 },
    { 0, 0 },
    { 0xd446e5fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a6e600, 61 },
    { 0, 0 },
    { 0x566e602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a6e60c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d86e612, 98 },
    { 0x5646e613, 61 },
    { 0x2696e614, 90 },
    { 0x976e615, 61 },
    { 0xd26e613, 46 },
    { 0xcee6e617, 61 },
    { 0xfe86e617, 43 },
    { 0xd4d6e619, 43 },
    { 0x66d6e61a, 90 },
    { 0x5db6e61a, 55 },
    { 0xc896e61b, 95 },
    { 0xaa76e61d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x196e620, 90 },
    { 0x5526e621, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe46e62a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1366e62e, 55 },
    { 0x7446e62f, 23 },
    { 0x7eb6e630, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd86e634, 86 },
    { 0xca06e635, 90 },
    { 0x4436e635, 23 },
    { 0xbf6e637, 43 },
    { 0x6536e636, 90 },
    { 0x2df6e639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5186e63c, 66 },
    { 0, 0 },
    { 0x38f6e63e, 68 },
    { 0, 0 },
    { 0x90c6e640, 96 },
    { 0xb856e640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda26e644, 55 },
    { 0, 0 },
    { 0x3ee6e646, 95 },
    { 0, 0 },
    { 0x4446e648, 42 },
    { 0x6e86e648, 95 },
    { 0xaaf6e649, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626e64d, 87 },
    { 0x5106e64d, 23 },
    { 0, 0 },
    { 0xc746e650, 23 },
    { 0x6726e651, 98 },
    { 0x5006e651, 98 },
    { 0x4526e653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36e656, 34 },
    { 0, 0 },
    { 0x7a66e658, 95 },
    { 0xb3d6e659, 98 },
    { 0x3cb6e65a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff66e65e, 76 },
    { 0, 0 },
    { 0x1656e660, 99 },
    { 0, 0 },
    { 0x5506e662, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3436e665, 65 },
    { 0x2a46e665, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e6e66f, 76 },
    { 0xc746e670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa6e674, 76 },
    { 0, 0 },
    { 0x456e676, 98 },
    { 0, 0 },
    { 0xc66e678, 95 },
    { 0x1dc6e678, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3856e681, 23 },
    { 0x1056e682, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb816e68b, 61 },
    { 0x5136e68c, 98 },
    { 0, 0 },
    { 0x296e68e, 27 },
    { 0x1df6e68f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716e695, 55 },
    { 0, 0 },
    { 0xc386e697, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06e69a, 34 },
    { 0xa556e69b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b96e69e, 61 },
    { 0xa896e69f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa16e6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc66e6b2, 90 },
    { 0x6236e6b3, 23 },
    { 0x5176e6b4, 43 },
    { 0xb296e6b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3286e6bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd786e6c1, 43 },
    { 0xd666e6c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf6e6c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6e6cd, 85 },
    { 0x6d86e6cd, 93 },
    { 0x8eb6e6ce, 23 },
    { 0x86e6d0, 33 },
    { 0, 0 },
    { 0xa226e6d2, 61 },
    { 0, 0 },
    { 0x1d46e6d4, 34 },
    { 0, 0 },
    { 0x4196e6d6, 98 },
    { 0x4426e6d7, 23 },
    { 0xb836e6d7, 46 },
    { 0xe326e6d9, 23 },
    { 0xff26e6d9, 23 },
    { 0x3326e6d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e16e6e4, 61 },
    { 0, 0 },
    { 0x8fa6e6e6, 61 },
    { 0x5056e6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8426e6ea, 61 },
    { 0, 0 },
    { 0x9856e6ec, 90 },
    { 0x85f6e6ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d66e6f8, 98 },
    { 0x4826e6f9, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b6e6ff, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9756e705, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d6e709, 95 },
    { 0, 0 },
    { 0x3e06e70b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee6e70f, 17 },
    { 0x5206e70f, 23 },
    { 0x6556e710, 95 },
    { 0xbc86e710, 34 },
    { 0xb716e713, 95 },
    { 0xeb26e714, 90 },
    { 0xc746e715, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf86e718, 56 },
    { 0xd8c6e718, 95 },
    { 0xbe76e718, 98 },
    { 0xd496e719, 90 },
    { 0x39b6e71c, 75 },
    { 0xc8a6e71d, 46 },
    { 0x3a36e71d, 53 },
    { 0xcc06e71f, 91 },
    { 0xcf86e71f, 61 },
    { 0x52b6e71d, 56 },
    { 0, 0 },
    { 0xe5c6e723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d6e727, 96 },
    { 0x7016e728, 61 },
    { 0x8cf6e729, 33 },
    { 0x9526e72a, 33 },
    { 0xf966e72a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc6e72e, 90 },
    { 0xda66e72e, 33 },
    { 0, 0 },
    { 0x64b6e731, 98 },
    { 0x9876e732, 61 },
    { 0x2d66e732, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6466e736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c6e73b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa846e73e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c96e742, 90 },
    { 0, 0 },
    { 0xc5c6e744, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4656e747, 90 },
    { 0x8786e748, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c6e74e, 90 },
    { 0, 0 },
    { 0xb0e6e750, 52 },
    { 0, 0 },
    { 0x6aa6e752, 87 },
    { 0, 0 },
    { 0x2656e754, 95 },
    { 0x1306e755, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc476e75b, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf156e760, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf6e766, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d26e76a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4516e76d, 76 },
    { 0xb456e76e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86e771, 46 },
    { 0, 0 },
    { 0x3076e773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e96e77b, 84 },
    { 0xbd96e77c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6e780, 90 },
    { 0x5a86e781, 33 },
    { 0xb3b6e782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5626e788, 87 },
    { 0xd5a6e788, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d36e78c, 43 },
    { 0x84a6e78d, 87 },
    { 0x8486e78e, 46 },
    { 0, 0 },
    { 0x4a96e790, 34 },
    { 0x2516e790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6e797, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276e79a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f6e79e, 61 },
    { 0x99d6e79f, 98 },
    { 0x93b6e79f, 98 },
    { 0x67e6e7a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc546e7a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2776e7ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7886e7b1, 46 },
    { 0xafa6e7b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2376e7ba, 69 },
    { 0x6c56e7bb, 90 },
    { 0xc036e7bc, 61 },
    { 0x2e96e7bd, 61 },
    { 0x17a6e7bd, 33 },
    { 0x46d6e7bf, 55 },
    { 0xeaf6e7bf, 43 },
    { 0x8936e7bf, 55 },
    { 0xa96e7be, 55 },
    { 0, 0 },
    { 0xe5a6e7c4, 43 },
    { 0, 0 },
    { 0x56c6e7c6, 23 },
    { 0xe096e7c7, 33 },
    { 0xbb06e7c8, 87 },
    { 0x29b6e7c8, 61 },
    { 0xd1a6e7ca, 95 },
    { 0xfb36e7ca, 76 },
    { 0x2ac6e7ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a6e7cf, 61 },
    { 0x4e56e7cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f6e7d3, 43 },
    { 0x63a6e7d4, 19 },
    { 0, 0 },
    { 0x5326e7d6, 61 },
    { 0x2e26e7d7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4036e7de, 61 },
    { 0, 0 },
    { 0x6c16e7e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda06e7e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa6e7e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd486e7ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6e7f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4906e7f4, 52 },
    { 0xf766e7f5, 90 },
    { 0x1d36e7f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e6e7fb, 93 },
    { 0x3fb6e7fb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c36e7ff, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a6e802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d6e80d, 23 },
    { 0x8286e80d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa186e812, 33 },
    { 0xc586e813, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd596e818, 98 },
    { 0x8c26e819, 95 },
    { 0xd1c6e81a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e86e81d, 98 },
    { 0, 0 },
    { 0x1556e81f, 95 },
    { 0x3466e820, 90 },
    { 0x1e26e821, 95 },
    { 0x3986e822, 87 },
    { 0xfb36e822, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc6e826, 99 },
    { 0xc346e827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd346e82a, 46 },
    { 0x61e6e82b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6076e834, 33 },
    { 0x3ba6e835, 76 },
    { 0, 0 },
    { 0xd076e837, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3456e83b, 91 },
    { 0x41d6e83c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56e844, 95 },
    { 0x2566e845, 61 },
    { 0xadd6e846, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d16e849, 58 },
    { 0, 0 },
    { 0x1056e84b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d6e852, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f6e86b, 33 },
    { 0x8a16e86b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2846e86f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c6e873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56e877, 23 },
    { 0xf946e878, 98 },
    { 0, 0 },
    { 0xb896e87a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe6e87d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b6e888, 33 },
    { 0xdb86e888, 51 },
    { 0x83b6e88a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7006e88f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1526e895, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d6e89c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4966e8a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab6e8ab, 35 },
    { 0x8e06e8ab, 90 },
    { 0x376e8ab, 33 },
    { 0x7cb6e8ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b6e8b6, 90 },
    { 0x1716e8b7, 52 },
    { 0, 0 },
    { 0xed46e8b9, 55 },
    { 0xfb26e8b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa106e8bd, 91 },
    { 0x1576e8be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f6e8c5, 99 },
    { 0, 0 },
    { 0xd936e8c7, 64 },
    { 0, 0 },
    { 0xa066e8c9, 55 },
    { 0, 0 },
    { 0xcf96e8cb, 78 },
    { 0x2bd6e8cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2326e8d1, 43 },
    { 0xad36e8d2, 87 },
    { 0xeb76e8d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a66e8d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4526e8d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e36e8de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe6e8e1, 98 },
    { 0xaae6e8e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e6e8e5, 87 },
    { 0xec26e8e6, 46 },
    { 0, 0 },
    { 0x2af6e8e8, 95 },
    { 0, 0 },
    { 0x6e6e8ea, 55 },
    { 0x5876e8eb, 90 },
    { 0xe3d6e8eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf396e8f1, 85 },
    { 0xd856e8f1, 67 },
    { 0x87d6e8f1, 95 },
    { 0, 0 },
    { 0x336e8f5, 95 },
    { 0x5f46e8f6, 67 },
    { 0xfc06e8f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2476e8fd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f6e900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6e904, 67 },
    { 0x53e6e905, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c06e909, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d6e90e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a6e915, 33 },
    { 0x2806e916, 76 },
    { 0xd6b6e917, 100 },
    { 0x55c6e917, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec6e923, 90 },
    { 0x46e924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba16e927, 76 },
    { 0xca6e928, 81 },
    { 0, 0 },
    { 0x9bc6e92a, 60 },
    { 0, 0 },
    { 0x2106e92c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb6e930, 85 },
    { 0x24f6e930, 77 },
    { 0x93d6e932, 95 },
    { 0, 0 },
    { 0x3b36e934, 43 },
    { 0x7446e935, 80 },
    { 0, 0 },
    { 0x3d26e937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f6e93b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2966e940, 55 },
    { 0, 0 },
    { 0xa816e942, 90 },
    { 0x4a36e943, 98 },
    { 0x7d96e944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9206e94d, 61 },
    { 0xb936e94d, 33 },
    { 0x49d6e94e, 90 },
    { 0x2936e94f, 67 },
    { 0, 0 },
    { 0x3fa6e952, 90 },
    { 0xec86e953, 23 },
    { 0, 0 },
    { 0x74b6e955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c86e958, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6e95b, 98 },
    { 0xb206e95c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e6e960, 90 },
    { 0, 0 },
    { 0x9f6e962, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec06e965, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2126e96a, 34 },
    { 0, 0 },
    { 0xb5f6e96c, 95 },
    { 0, 0 },
    { 0xfff6e96e, 33 },
    { 0x5616e96e, 91 },
    { 0x10d6e970, 33 },
    { 0x7d36e970, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b6e974, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc6e977, 43 },
    { 0xa606e978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7156e97d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca6e983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd846e986, 98 },
    { 0xfa46e986, 91 },
    { 0, 0 },
    { 0x2c36e989, 98 },
    { 0x6e6e98a, 77 },
    { 0x3546e98a, 99 },
    { 0x5576e98a, 43 },
    { 0xc1d6e98d, 34 },
    { 0x1926e98a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd6e993, 61 },
    { 0x5706e994, 43 },
    { 0x8b36e995, 33 },
    { 0xd3c6e996, 17 },
    { 0x86f6e995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8356e9a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d6e9a4, 63 },
    { 0x1aa6e9a5, 95 },
    { 0x12f6e9a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd046e9a9, 80 },
    { 0x3486e9aa, 61 },
    { 0, 0 },
    { 0x8776e9ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2706e9b2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5956e9b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6e9bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8866e9c0, 43 },
    { 0x3e76e9c1, 90 },
    { 0xc136e9c1, 98 },
    { 0x3d76e9c3, 43 },
    { 0x6a76e9c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6e9cb, 61 },
    { 0x6e6e9cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc6e9d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f6e9d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff6e9de, 78 },
    { 0xed56e9df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d6e9e3, 76 },
    { 0xdc26e9e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5296e9ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6986e9fa, 98 },
    { 0xe856e9fb, 61 },
    { 0xe806e9fb, 65 },
    { 0xfd46e9fc, 77 },
    { 0, 0 },
    { 0x6306e9ff, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6396ea06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed86ea0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd6ea10, 52 },
    { 0x8686ea11, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1376ea15, 93 },
    { 0, 0 },
    { 0x8506ea17, 33 },
    { 0x1fc6ea18, 90 },
    { 0, 0 },
    { 0x9bf6ea1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7426ea22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8466ea2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e16ea30, 66 },
    { 0xbc96ea31, 66 },
    { 0xd446ea32, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3126ea36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c6ea3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d6ea3d, 61 },
    { 0x17e6ea3e, 99 },
    { 0xa2a6ea3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd6ea43, 90 },
    { 0, 0 },
    { 0x866ea45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a6ea48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa306ea4b, 90 },
    { 0, 0 },
    { 0x9126ea4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc6ea51, 98 },
    { 0x4e86ea52, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e6ea58, 98 },
    { 0x2b66ea58, 52 },
    { 0xf6ea5a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c96ea5f, 99 },
    { 0xfdd6ea60, 88 },
    { 0x2d46ea5f, 96 },
    { 0x9cc6ea61, 33 },
    { 0, 0 },
    { 0xe4f6ea64, 90 },
    { 0xbd96ea65, 61 },
    { 0xc846ea66, 100 },
    { 0x8196ea67, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596ea6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf826ea6e, 98 },
    { 0, 0 },
    { 0x36d6ea70, 38 },
    { 0x7af6ea71, 23 },
    { 0x6786ea72, 61 },
    { 0xea96ea73, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc616ea79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb976ea7f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5646ea84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f6ea8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7266ea92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8206ea96, 98 },
    { 0x456ea97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd876ea9c, 90 },
    { 0, 0 },
    { 0x9116ea9e, 43 },
    { 0xcbc6ea9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d56eaa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd6eaa9, 61 },
    { 0xcaf6eaa9, 75 },
    { 0x2916eaab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec6eaae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9136eab3, 33 },
    { 0, 0 },
    { 0x5266eab5, 90 },
    { 0x8d96eab5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8386eabb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9726eac4, 19 },
    { 0x5b86eac4, 68 },
    { 0x8ec6eac6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb6eac9, 61 },
    { 0xfb66eaca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1626eacd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1946ead0, 61 },
    { 0xba86ead1, 90 },
    { 0x9276ead0, 52 },
    { 0x2d66ead3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e36ead6, 61 },
    { 0xfb86ead7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe036eada, 99 },
    { 0x5ae6eada, 87 },
    { 0x59d6eadb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1006eae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b6eae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb926eaed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216eaf0, 75 },
    { 0x9266eaf1, 61 },
    { 0xc346eaf2, 53 },
    { 0xc9d6eaf2, 91 },
    { 0x5756eaf3, 36 },
    { 0x14c6eaf5, 90 },
    { 0x11f6eaf1, 34 },
    { 0xdea6eaf5, 95 },
    { 0x31d6eaf6, 33 },
    { 0xc806eaf9, 93 },
    { 0x27a6eaf9, 33 },
    { 0xc826eaf4, 90 },
    { 0x7aa6eafc, 98 },
    { 0x2f06eafd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2996eb00, 90 },
    { 0x97f6eb00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde76eb05, 98 },
    { 0x1be6eb06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a6eb09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636eb0e, 65 },
    { 0x8426eb0f, 93 },
    { 0, 0 },
    { 0xd886eb11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc86eb16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6eb1d, 90 },
    { 0, 0 },
    { 0x8116eb1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a56eb33, 61 },
    { 0xb6f6eb34, 61 },
    { 0xd7d6eb34, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd6eb3a, 85 },
    { 0xd1b6eb3b, 33 },
    { 0x13a6eb3c, 76 },
    { 0x36a6eb3c, 18 },
    { 0x5b66eb3e, 76 },
    { 0x4e66eb3f, 85 },
    { 0x62a6eb3c, 91 },
    { 0xf926eb3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd6eb44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe96eb4d, 87 },
    { 0xd7d6eb4d, 33 },
    { 0x7db6eb4f, 95 },
    { 0, 0 },
    { 0x40f6eb51, 85 },
    { 0xdd06eb52, 61 },
    { 0xc6b6eb51, 55 },
    { 0x806eb54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be6eb57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4786eb5b, 33 },
    { 0xe5e6eb5c, 33 },
    { 0xf2b6eb5c, 93 },
    { 0xd376eb5e, 95 },
    { 0, 0 },
    { 0x8896eb60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8406eb68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa236eb6d, 95 },
    { 0x74a6eb6e, 99 },
    { 0xe856eb6f, 98 },
    { 0x6246eb70, 23 },
    { 0x1af6eb70, 40 },
    { 0x47e6eb6f, 90 },
    { 0xe246eb72, 23 },
    { 0, 0 },
    { 0x5916eb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb776eb7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb6eb81, 95 },
    { 0xe6c6eb82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e6eb89, 33 },
    { 0xa886eb8a, 66 },
    { 0x5d36eb8b, 80 },
    { 0x9086eb8a, 33 },
    { 0x5e16eb8d, 5 },
    { 0, 0 },
    { 0x8516eb8f, 91 },
    { 0xa946eb90, 36 },
    { 0x3096eb91, 90 },
    { 0x6826eb91, 61 },
    { 0x92f6eb91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe906eb96, 23 },
    { 0, 0 },
    { 0x2146eb98, 33 },
    { 0xc4d6eb99, 43 },
    { 0xfb76eb99, 90 },
    { 0x3716eb9b, 90 },
    { 0, 0 },
    { 0x1c46eb9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e46eba4, 33 },
    { 0x3a6eba5, 43 },
    { 0xf926eba5, 33 },
    { 0, 0 },
    { 0x8026eba8, 84 },
    { 0xd336eba9, 76 },
    { 0x3366ebaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc96ebb1, 84 },
    { 0, 0 },
    { 0x69c6ebb3, 55 },
    { 0, 0 },
    { 0xba86ebb5, 23 },
    { 0x4436ebb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b46ebba, 90 },
    { 0x4586ebba, 23 },
    { 0x7ba6ebba, 95 },
    { 0x5e06ebbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d6ebc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc6ebc5, 99 },
    { 0x6f76ebc6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d6ebcf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e6ebd2, 90 },
    { 0, 0 },
    { 0x36c6ebd4, 85 },
    { 0x1286ebd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2586ebd9, 55 },
    { 0xec6ebda, 93 },
    { 0x7a86ebdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b6ebe5, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf26ebe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b6ebee, 90 },
    { 0, 0 },
    { 0xec46ebf0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b6ebf3, 98 },
    { 0xc086ebf3, 98 },
    { 0xd606ebf3, 34 },
    { 0x3816ebf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6ebfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8506ec02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6ec05, 90 },
    { 0x7766ec06, 90 },
    { 0, 0 },
    { 0x6d96ec08, 33 },
    { 0xa446ec09, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9456ec0d, 85 },
    { 0x26b6ec0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46ec15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb196ec18, 33 },
    { 0, 0 },
    { 0xb716ec1a, 33 },
    { 0, 0 },
    { 0x3376ec1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5246ec1f, 52 },
    { 0x5826ec20, 55 },
    { 0xb186ec21, 61 },
    { 0x3426ec22, 84 },
    { 0x6ec23, 90 },
    { 0xbe46ec22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b6ec2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b6ec30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d6ec36, 43 },
    { 0, 0 },
    { 0xa6a6ec38, 90 },
    { 0xf766ec38, 95 },
    { 0x866ec3a, 43 },
    { 0xec6ec3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6ec40, 61 },
    { 0x7e86ec41, 33 },
    { 0, 0 },
    { 0xda16ec43, 81 },
    { 0, 0 },
    { 0xfbc6ec45, 81 },
    { 0, 0 },
    { 0x7e96ec47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f6ec4a, 55 },
    { 0xbc46ec4b, 59 },
    { 0x4bf6ec4b, 33 },
    { 0x4686ec4d, 90 },
    { 0x9f86ec4c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e76ec51, 43 },
    { 0, 0 },
    { 0xab26ec53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb6ec58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f6ec5b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd686ec62, 61 },
    { 0xb696ec63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c66ec6d, 61 },
    { 0, 0 },
    { 0x7ae6ec6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c6ec72, 33 },
    { 0xd146ec73, 91 },
    { 0, 0 },
    { 0xd8e6ec75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c6ec7f, 23 },
    { 0, 0 },
    { 0xf556ec81, 85 },
    { 0x7ea6ec81, 61 },
    { 0, 0 },
    { 0x73b6ec84, 93 },
    { 0x8b96ec85, 90 },
    { 0xd186ec86, 98 },
    { 0x3716ec87, 64 },
    { 0xf956ec86, 95 },
    { 0xf7c6ec89, 98 },
    { 0xec96ec89, 98 },
    { 0, 0 },
    { 0xba26ec8c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce6ec8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d6ec92, 76 },
    { 0, 0 },
    { 0x166ec94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f6ec97, 91 },
    { 0xce6ec98, 98 },
    { 0xaf36ec97, 90 },
    { 0x8a6ec9a, 96 },
    { 0xd696ec9b, 55 },
    { 0, 0 },
    { 0x7ca6ec9d, 46 },
    { 0, 0 },
    { 0xd766ec9f, 95 },
    { 0, 0 },
    { 0xc106eca1, 87 },
    { 0xc7b6eca2, 76 },
    { 0xcf6eca3, 55 },
    { 0xf2f6eca3, 56 },
    { 0x9fb6eca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4846ecab, 61 },
    { 0x2a26ecac, 95 },
    { 0, 0 },
    { 0x3526ecae, 98 },
    { 0x9776ecaf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4456ecb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb6ecb8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f6ecbb, 23 },
    { 0x8606ecbc, 91 },
    { 0, 0 },
    { 0x67e6ecbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a56ecc2, 95 },
    { 0xaf76ecc3, 23 },
    { 0x8536ecc2, 33 },
    { 0x6c66ecc5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a6ecca, 56 },
    { 0x3ad6ecca, 52 },
    { 0x4836eccc, 90 },
    { 0xaa36eccd, 87 },
    { 0xee76ecce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd6ecd1, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4436ecd9, 61 },
    { 0xa2c6ecda, 95 },
    { 0x4216ecda, 84 },
    { 0x28d6ecda, 65 },
    { 0, 0 },
    { 0xa906ecde, 95 },
    { 0x5ef6ecdf, 99 },
    { 0x61a6ece0, 84 },
    { 0x70d6ece1, 77 },
    { 0x6026ece0, 23 },
    { 0xffa6ece3, 65 },
    { 0xb4f6ece4, 85 },
    { 0xb126ece0, 93 },
    { 0, 0 },
    { 0xf3f6ece7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8376ecea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1616ecf0, 67 },
    { 0xdd56ecf1, 34 },
    { 0x7706ecf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc016ecf9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc676ecfe, 76 },
    { 0, 0 },
    { 0x1806ed00, 55 },
    { 0xee56ed01, 34 },
    { 0x44d6ed00, 98 },
    { 0xdf06ed03, 98 },
    { 0, 0 },
    { 0x8ee6ed05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb36ed08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a6ed0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b6ed0f, 46 },
    { 0x1b06ed10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf756ed1b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f6ed20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd06ed23, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6496ed28, 55 },
    { 0, 0 },
    { 0xa1f6ed2a, 91 },
    { 0x58a6ed2b, 33 },
    { 0xb846ed2a, 43 },
    { 0x9866ed2d, 33 },
    { 0, 0 },
    { 0x83f6ed2f, 33 },
    { 0xb126ed30, 34 },
    { 0x4df6ed31, 98 },
    { 0xfad6ed32, 52 },
    { 0x47b6ed30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6ed38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b6ed3c, 61 },
    { 0, 0 },
    { 0xf916ed3e, 23 },
    { 0xacf6ed3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3346ed44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa266ed47, 61 },
    { 0x7ca6ed48, 61 },
    { 0xb506ed49, 59 },
    { 0x9216ed48, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b6ed4d, 46 },
    { 0x2f26ed4e, 95 },
    { 0, 0 },
    { 0xa696ed50, 61 },
    { 0xfb56ed51, 46 },
    { 0x75b6ed52, 91 },
    { 0x6ba6ed51, 98 },
    { 0x976ed54, 33 },
    { 0x64b6ed55, 99 },
    { 0x30f6ed53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc26ed5c, 61 },
    { 0, 0 },
    { 0x86ed5e, 99 },
    { 0xdc16ed5f, 33 },
    { 0x81f6ed5f, 34 },
    { 0x1fb6ed61, 68 },
    { 0x3fa6ed5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8256ed65, 88 },
    { 0xf026ed65, 80 },
    { 0x1e16ed67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb36ed6a, 61 },
    { 0xaac6ed6b, 46 },
    { 0x1f76ed6c, 33 },
    { 0xc856ed6c, 95 },
    { 0xcde6ed6c, 33 },
    { 0x3616ed6f, 34 },
    { 0x7a66ed70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f6ed77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d6ed7f, 98 },
    { 0x96d6ed80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc66ed84, 98 },
    { 0, 0 },
    { 0xaf26ed86, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d6ed89, 98 },
    { 0x186ed89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb476ed8e, 90 },
    { 0xbb66ed8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd646ed93, 98 },
    { 0xde06ed94, 34 },
    { 0x7e16ed95, 24 },
    { 0, 0 },
    { 0x4876ed97, 90 },
    { 0, 0 },
    { 0x716ed99, 23 },
    { 0xe5e6ed9a, 87 },
    { 0, 0 },
    { 0x2ee6ed9c, 61 },
    { 0xd9c6ed9d, 90 },
    { 0xbce6ed9e, 33 },
    { 0x74b6ed9e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4326eda3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3196eda8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c6edab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe996edae, 88 },
    { 0x72f6edae, 46 },
    { 0xb486edb0, 23 },
    { 0, 0 },
    { 0x3536edb2, 76 },
    { 0x9106edb3, 46 },
    { 0, 0 },
    { 0x7fb6edb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6edbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6edbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5136edc1, 33 },
    { 0xce76edc2, 33 },
    { 0, 0 },
    { 0xcac6edc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa6edc9, 95 },
    { 0x8df6edca, 23 },
    { 0x636edcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8826edce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f6edd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c6edd5, 90 },
    { 0x6566edd6, 33 },
    { 0x7966edd7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e6edda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c6eddd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a46ede4, 99 },
    { 0x6226ede5, 53 },
    { 0xa356ede5, 85 },
    { 0x9f46ede7, 87 },
    { 0xd4f6ede8, 90 },
    { 0x5796ede4, 82 },
    { 0xd246edea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6edef, 77 },
    { 0xb996edf0, 23 },
    { 0, 0 },
    { 0x6b56edf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c6edf9, 64 },
    { 0xcf16edfa, 66 },
    { 0, 0 },
    { 0xdb06edfc, 61 },
    { 0x4326edfd, 99 },
    { 0x3ae6edfe, 95 },
    { 0xf266edfc, 76 },
    { 0, 0 },
    { 0x9a96ee01, 76 },
    { 0x8da6ee02, 68 },
    { 0x2146ee03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf146ee0a, 76 },
    { 0, 0 },
    { 0x4036ee0c, 90 },
    { 0, 0 },
    { 0x72a6ee0e, 43 },
    { 0, 0 },
    { 0xc326ee10, 95 },
    { 0xc0a6ee10, 23 },
    { 0xbc96ee12, 87 },
    { 0, 0 },
    { 0xba06ee14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab56ee1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5176ee24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8866ee2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c6ee32, 82 },
    { 0x6a06ee32, 76 },
    { 0xd336ee32, 34 },
    { 0x24a6ee34, 95 },
    { 0x5316ee36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd476ee3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd106ee42, 38 },
    { 0x7a66ee42, 23 },
    { 0xd3c6ee42, 95 },
    { 0xdd96ee44, 97 },
    { 0, 0 },
    { 0x9226ee47, 55 },
    { 0, 0 },
    { 0x1486ee49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d6ee51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae6ee54, 83 },
    { 0x4466ee55, 84 },
    { 0x51e6ee56, 95 },
    { 0x6556ee57, 87 },
    { 0, 0 },
    { 0x3326ee59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2386ee5d, 87 },
    { 0, 0 },
    { 0xb886ee5f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa6ee62, 46 },
    { 0x3f36ee63, 95 },
    { 0xce96ee64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3506ee68, 95 },
    { 0, 0 },
    { 0x9236ee6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776ee77, 90 },
    { 0xda46ee77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xad46ee7b, 23 },
    { 0x28f6ee7c, 23 },
    { 0x4026ee7c, 90 },
    { 0x82d6ee7c, 33 },
    { 0xb396ee7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f76ee83, 76 },
    { 0xf396ee84, 61 },
    { 0xd336ee85, 33 },
    { 0xd346ee86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7196ee8b, 33 },
    { 0, 0 },
    { 0x3c66ee8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd276ee93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6ee97, 23 },
    { 0x636ee98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd6eea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a6eead, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe216eeb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546eebd, 90 },
    { 0x92e6eebd, 52 },
    { 0, 0 },
    { 0xb8b6eec0, 98 },
    { 0x3916eec1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6eec4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e16eec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb56eecd, 90 },
    { 0x3f06eece, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66eed1, 56 },
    { 0x4e66eed2, 33 },
    { 0, 0 },
    { 0x5676eed4, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa6eed7, 90 },
    { 0, 0 },
    { 0xfb66eed9, 33 },
    { 0, 0 },
    { 0xd856eedb, 90 },
    { 0xc0c6eedc, 98 },
    { 0, 0 },
    { 0x5966eede, 95 },
    { 0xc906eedf, 75 },
    { 0x90a6eedf, 46 },
    { 0xe016eee1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b56eeeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff6eeee, 98 },
    { 0x7bc6eeef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb6eef2, 33 },
    { 0xdcf6eef2, 90 },
    { 0x5ee6eef4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6596eef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d6eefb, 93 },
    { 0xf506eefc, 99 },
    { 0x3b66eefd, 61 },
    { 0xeb56eefe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe566ef01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3336ef14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1056ef17, 90 },
    { 0xf016ef18, 91 },
    { 0x1fb6ef18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb516ef20, 55 },
    { 0xc0b6ef21, 23 },
    { 0x7a16ef22, 98 },
    { 0x76d6ef22, 65 },
    { 0xb5c6ef23, 90 },
    { 0x5de6ef25, 75 },
    { 0xfdc6ef26, 76 },
    { 0xe0e6ef27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb16ef2e, 99 },
    { 0x2126ef2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa6ef3f, 61 },
    { 0, 0 },
    { 0x8216ef41, 33 },
    { 0, 0 },
    { 0x7756ef43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a96ef4c, 93 },
    { 0, 0 },
    { 0x9fe6ef4e, 87 },
    { 0xfad6ef4f, 23 },
    { 0x4f46ef50, 55 },
    { 0x6d16ef51, 95 },
    { 0xc2c6ef52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3016ef55, 76 },
    { 0x8a66ef55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3886ef5a, 98 },
    { 0xf216ef5a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b26ef5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5476ef64, 61 },
    { 0, 0 },
    { 0x8a86ef66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6586ef6b, 76 },
    { 0xed76ef6c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f6ef6f, 90 },
    { 0x4ff6ef6f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a6ef73, 81 },
    { 0x4af6ef74, 99 },
    { 0x986ef75, 61 },
    { 0x31b6ef75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda26ef7e, 95 },
    { 0x5ad6ef7f, 27 },
    { 0x6af6ef7e, 33 },
    { 0x73a6ef81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa016ef86, 98 },
    { 0x94b6ef87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f06ef8b, 61 },
    { 0x1cd6ef8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc256ef99, 61 },
    { 0x33d6ef9a, 95 },
    { 0xa206ef9a, 17 },
    { 0xe416ef9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3436efa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6efa9, 52 },
    { 0x85b6efa9, 90 },
    { 0x33c6efaa, 46 },
    { 0x1b46efac, 95 },
    { 0, 0 },
    { 0x846efae, 78 },
    { 0xcff6efaf, 23 },
    { 0, 0 },
    { 0x6266efb1, 93 },
    { 0x4696efb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb96efc6, 95 },
    { 0x6856efc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6efcc, 46 },
    { 0x7e26efcd, 76 },
    { 0x60d6efce, 78 },
    { 0, 0 },
    { 0x24a6efd0, 61 },
    { 0xe1e6efd1, 98 },
    { 0x8cb6efd1, 93 },
    { 0x4216efd1, 95 },
    { 0xc316efd1, 95 },
    { 0x6f46efd1, 61 },
    { 0x22f6efd6, 76 },
    { 0, 0 },
    { 0xc096efd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f6efdb, 84 },
    { 0x6cd6efdc, 61 },
    { 0xa026efdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a6efe5, 83 },
    { 0, 0 },
    { 0x87b6efe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac6efec, 66 },
    { 0x7386efed, 90 },
    { 0, 0 },
    { 0xef56efef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5426eff2, 91 },
    { 0, 0 },
    { 0x9bf6eff4, 91 },
    { 0x4716eff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f36effb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a86effe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8046f001, 95 },
    { 0xaa26f001, 90 },
    { 0xff46f002, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d6f011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b6f014, 90 },
    { 0xa906f015, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2106f019, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc266f01f, 55 },
    { 0, 0 },
    { 0x3746f021, 61 },
    { 0, 0 },
    { 0xc2c6f023, 33 },
    { 0, 0 },
    { 0x48d6f025, 34 },
    { 0x3a06f025, 90 },
    { 0x31b6f027, 98 },
    { 0x15a6f028, 66 },
    { 0xaf86f025, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b6f030, 61 },
    { 0xff76f031, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f16f034, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a36f037, 17 },
    { 0xe6a6f038, 61 },
    { 0x4ab6f039, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc586f03f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5476f046, 76 },
    { 0, 0 },
    { 0x2bc6f048, 98 },
    { 0, 0 },
    { 0xb2d6f04a, 78 },
    { 0, 0 },
    { 0x92a6f04c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f6f04f, 34 },
    { 0xcfd6f050, 98 },
    { 0x7ee6f051, 98 },
    { 0xb106f052, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa86f056, 34 },
    { 0xe4b6f057, 98 },
    { 0xce56f057, 93 },
    { 0xa1c6f057, 23 },
    { 0xcfb6f05a, 94 },
    { 0x1856f056, 36 },
    { 0xff76f05c, 90 },
    { 0xafe6f05d, 98 },
    { 0x9ea6f05e, 33 },
    { 0x8be6f05f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3806f064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4396f068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9856f06b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176f073, 77 },
    { 0x59f6f074, 97 },
    { 0xe9c6f074, 90 },
    { 0x9b26f076, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2066f079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a6f07c, 55 },
    { 0, 0 },
    { 0xb986f07e, 90 },
    { 0, 0 },
    { 0xdba6f080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b86f083, 61 },
    { 0xd756f084, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b6f088, 33 },
    { 0x7a36f089, 100 },
    { 0xe4f6f08a, 66 },
    { 0x9426f08b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2806f08f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2736f096, 23 },
    { 0xe586f097, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96f09a, 33 },
    { 0, 0 },
    { 0xdb26f09c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a6f0ad, 99 },
    { 0x5736f0ae, 90 },
    { 0x5916f0af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7536f0b6, 76 },
    { 0xf5e6f0b7, 90 },
    { 0, 0 },
    { 0x3916f0b9, 98 },
    { 0x9ad6f0ba, 33 },
    { 0x6da6f0bb, 33 },
    { 0x8096f0bc, 61 },
    { 0x7d96f0bd, 90 },
    { 0x6816f0be, 33 },
    { 0x7396f0bd, 76 },
    { 0xbf86f0bf, 12 },
    { 0, 0 },
    { 0x5f26f0c2, 88 },
    { 0, 0 },
    { 0xb7b6f0c4, 87 },
    { 0x476f0c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc46f0cb, 23 },
    { 0, 0 },
    { 0x9f36f0cd, 17 },
    { 0xb716f0ce, 91 },
    { 0xa186f0ce, 90 },
    { 0x3676f0ce, 95 },
    { 0xc016f0d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b6f0d4, 90 },
    { 0, 0 },
    { 0xacf6f0d6, 34 },
    { 0xeba6f0d7, 98 },
    { 0xc226f0d8, 80 },
    { 0x5506f0d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3756f0de, 84 },
    { 0, 0 },
    { 0xc166f0e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f6f0e6, 98 },
    { 0xef86f0e7, 95 },
    { 0x2b76f0e8, 90 },
    { 0x7ab6f0e8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c6f0ee, 100 },
    { 0x8646f0ee, 61 },
    { 0x1206f0f0, 99 },
    { 0x5586f0f0, 61 },
    { 0, 0 },
    { 0xe66f0f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c6f0f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6f0fb, 33 },
    { 0xf196f0fc, 78 },
    { 0xf626f0fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6616f106, 61 },
    { 0xbcc6f107, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436f10b, 99 },
    { 0xeff6f10c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e86f114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb76f11d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a06f126, 55 },
    { 0, 0 },
    { 0xb1a6f128, 77 },
    { 0, 0 },
    { 0xd5b6f12a, 68 },
    { 0x4176f12b, 90 },
    { 0x5446f12b, 98 },
    { 0x58b6f12d, 51 },
    { 0x4906f12d, 98 },
    { 0xda26f12e, 64 },
    { 0x22b6f130, 99 },
    { 0x34d6f12f, 95 },
    { 0x25d6f12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f06f136, 90 },
    { 0x66b6f136, 93 },
    { 0, 0 },
    { 0x16f139, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46f13d, 95 },
    { 0, 0 },
    { 0xd526f13f, 100 },
    { 0xb646f13f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc466f144, 90 },
    { 0x4e16f144, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26f14b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7526f14f, 95 },
    { 0, 0 },
    { 0xad16f151, 33 },
    { 0, 0 },
    { 0x8576f153, 75 },
    { 0xe466f154, 34 },
    { 0xa3a6f155, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b6f160, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8146f168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a6f170, 23 },
    { 0x3106f170, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6756f17c, 77 },
    { 0xccc6f17d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed6f182, 55 },
    { 0x8cb6f183, 33 },
    { 0x8066f184, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7776f187, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa786f18a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06f190, 23 },
    { 0xe646f191, 64 },
    { 0x8016f192, 90 },
    { 0xd396f191, 90 },
    { 0xe2b6f194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f6f199, 36 },
    { 0xdd56f19a, 66 },
    { 0x7d06f199, 87 },
    { 0xf4d6f19c, 90 },
    { 0, 0 },
    { 0xf556f19e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c6f1a5, 61 },
    { 0xcd76f1a6, 34 },
    { 0x15b6f1a5, 34 },
    { 0xc676f1a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d6f1ab, 87 },
    { 0x95f6f1ac, 61 },
    { 0x1f96f1ad, 90 },
    { 0x90f6f1ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xba36f1b1, 55 },
    { 0xb8f6f1b1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b26f1b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876f1be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7646f1c1, 91 },
    { 0x50a6f1c1, 90 },
    { 0x1386f1c3, 95 },
    { 0x7096f1c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa506f1c8, 99 },
    { 0xf4a6f1c9, 91 },
    { 0, 0 },
    { 0x8d16f1cb, 90 },
    { 0x1cc6f1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf716f1d3, 95 },
    { 0x4ab6f1d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb26f1db, 99 },
    { 0x6d6f1dc, 61 },
    { 0xb1c6f1db, 90 },
    { 0xb886f1de, 77 },
    { 0x9056f1df, 43 },
    { 0x5a86f1db, 98 },
    { 0xf686f1e0, 33 },
    { 0xc366f1e2, 98 },
    { 0x14f6f1e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd6f1e6, 95 },
    { 0x2356f1e7, 55 },
    { 0xcf16f1e6, 94 },
    { 0x4f06f1e9, 23 },
    { 0xc186f1ea, 91 },
    { 0x65f6f1eb, 33 },
    { 0x8ac6f1ec, 52 },
    { 0xad36f1ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd506f1f1, 55 },
    { 0xb986f1f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2446f1fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7456f1fd, 95 },
    { 0xaea6f1fd, 91 },
    { 0x7ec6f1ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd6f202, 34 },
    { 0x4fc6f203, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d6f207, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c96f20b, 85 },
    { 0xd126f20c, 33 },
    { 0x6156f20d, 33 },
    { 0x2d56f20e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2706f212, 61 },
    { 0x41a6f212, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5966f219, 55 },
    { 0x1d46f21a, 66 },
    { 0x20e6f21a, 95 },
    { 0xd646f21a, 33 },
    { 0x4ee6f219, 33 },
    { 0xcfd6f21e, 55 },
    { 0x1fc6f21d, 33 },
    { 0x56f220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff76f225, 33 },
    { 0x7bf6f226, 55 },
    { 0x5ab6f227, 55 },
    { 0xe5b6f227, 90 },
    { 0x9bd6f229, 55 },
    { 0x3606f228, 95 },
    { 0x6016f22b, 17 },
    { 0, 0 },
    { 0x136f22d, 33 },
    { 0x4f36f22e, 98 },
    { 0, 0 },
    { 0xd246f230, 46 },
    { 0, 0 },
    { 0x45b6f232, 95 },
    { 0, 0 },
    { 0x5516f234, 90 },
    { 0x4446f235, 64 },
    { 0x5586f234, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e6f23b, 43 },
    { 0x4d96f23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc336f242, 33 },
    { 0, 0 },
    { 0x3436f244, 61 },
    { 0x9a16f245, 90 },
    { 0x22c6f244, 95 },
    { 0x7f76f247, 90 },
    { 0x5276f247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8306f24e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7396f252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a6f256, 90 },
    { 0, 0 },
    { 0x2ab6f258, 43 },
    { 0x69c6f259, 75 },
    { 0, 0 },
    { 0xbe16f25b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e96f262, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea6f265, 55 },
    { 0xa8d6f265, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef76f26a, 90 },
    { 0xde16f26b, 75 },
    { 0xad6f26c, 90 },
    { 0, 0 },
    { 0xb666f26e, 34 },
    { 0xbf46f26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b86f277, 90 },
    { 0xdec6f277, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa6f27b, 78 },
    { 0x5af6f27c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd6f27f, 43 },
    { 0, 0 },
    { 0xecd6f281, 90 },
    { 0x3a96f282, 90 },
    { 0x65d6f281, 61 },
    { 0, 0 },
    { 0x4bf6f285, 34 },
    { 0xcc76f286, 55 },
    { 0, 0 },
    { 0x8cb6f288, 78 },
    { 0xa536f289, 46 },
    { 0x8c26f28a, 55 },
    { 0xd3b6f28b, 76 },
    { 0x7716f28b, 33 },
    { 0, 0 },
    { 0x22c6f28e, 95 },
    { 0xce96f28e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be6f293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc346f29e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d6f2a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d6f2a4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a26f2a8, 90 },
    { 0, 0 },
    { 0x4626f2aa, 33 },
    { 0x76f2aa, 34 },
    { 0, 0 },
    { 0x7ef6f2ad, 90 },
    { 0, 0 },
    { 0x3326f2af, 95 },
    { 0xb6c6f2b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a6f2b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c6f2b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d6f2c6, 60 },
    { 0, 0 },
    { 0x1896f2c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d6f2d0, 48 },
    { 0x23e6f2d1, 90 },
    { 0xa3b6f2d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f2d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d6f2da, 83 },
    { 0xdf76f2da, 23 },
    { 0x95c6f2dc, 52 },
    { 0x9dd6f2da, 34 },
    { 0xdcf6f2dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1956f2e1, 90 },
    { 0, 0 },
    { 0x8bd6f2e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e36f2e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9066f2ef, 95 },
    { 0xfc16f2f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e6f2f3, 95 },
    { 0x6f96f2f4, 95 },
    { 0x5f36f2f5, 61 },
    { 0xa3d6f2f4, 95 },
    { 0, 0 },
    { 0xfb96f2f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e6f2fc, 33 },
    { 0, 0 },
    { 0x8fb6f2fe, 76 },
    { 0, 0 },
    { 0x63c6f300, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x556f303, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xae56f306, 63 },
    { 0x5206f306, 23 },
    { 0x91c6f308, 46 },
    { 0x82a6f307, 33 },
    { 0x2166f30a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a96f30d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7296f311, 91 },
    { 0xca46f311, 23 },
    { 0x7b56f313, 98 },
    { 0x3076f311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516f31e, 77 },
    { 0x7e6f31f, 23 },
    { 0xe46f320, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee6f323, 33 },
    { 0x6d66f324, 17 },
    { 0xf316f324, 87 },
    { 0xf186f324, 33 },
    { 0xa906f327, 34 },
    { 0x4666f326, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd316f32d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7886f330, 91 },
    { 0x4556f331, 76 },
    { 0xa466f332, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6f33f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea6f342, 61 },
    { 0, 0 },
    { 0x6f26f344, 61 },
    { 0xbe56f344, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe136f348, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c6f34c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e6f356, 36 },
    { 0x4de6f356, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f6f35a, 75 },
    { 0x3f46f35a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16f360, 76 },
    { 0, 0 },
    { 0xe186f362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4406f365, 23 },
    { 0xd106f366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a6f36a, 61 },
    { 0xb556f36b, 55 },
    { 0x1226f36b, 23 },
    { 0xc76f36c, 88 },
    { 0x18e6f36e, 33 },
    { 0x1476f36d, 33 },
    { 0, 0 },
    { 0x26f371, 46 },
    { 0x56f371, 98 },
    { 0xe366f371, 61 },
    { 0x3bf6f374, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf566f37c, 34 },
    { 0x296f37d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2966f380, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b6f387, 76 },
    { 0, 0 },
    { 0x5f16f389, 43 },
    { 0, 0 },
    { 0xaa96f38b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae6f390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc6f398, 61 },
    { 0, 0 },
    { 0xe706f39a, 78 },
    { 0xc1d6f39b, 66 },
    { 0x26b6f39a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae46f3a1, 23 },
    { 0, 0 },
    { 0xd316f3a3, 99 },
    { 0, 0 },
    { 0x9a56f3a5, 61 },
    { 0xb296f3a6, 61 },
    { 0xe06f3a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e6f3b2, 90 },
    { 0x3e86f3b2, 43 },
    { 0x9896f3b4, 61 },
    { 0x86a6f3b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7566f3bf, 90 },
    { 0xd146f3c0, 61 },
    { 0x6946f3c1, 55 },
    { 0xc366f3c1, 76 },
    { 0, 0 },
    { 0xa046f3c4, 91 },
    { 0xe196f3c4, 36 },
    { 0, 0 },
    { 0x1d06f3c7, 99 },
    { 0x1136f3c8, 33 },
    { 0xa0b6f3c9, 61 },
    { 0x706f3ca, 95 },
    { 0, 0 },
    { 0x6596f3cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436f3cf, 87 },
    { 0xf0e6f3d0, 33 },
    { 0x7ab6f3d1, 95 },
    { 0x9c36f3d1, 61 },
    { 0x7e86f3d2, 95 },
    { 0xb8b6f3d4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4906f3d7, 99 },
    { 0x4316f3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a6f3de, 98 },
    { 0x5846f3df, 95 },
    { 0x4106f3de, 95 },
    { 0xe8b6f3e0, 33 },
    { 0x8e76f3e2, 33 },
    { 0xaea6f3de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc6f3e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d6f3e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d56f3ed, 33 },
    { 0, 0 },
    { 0xa486f3ef, 84 },
    { 0x3146f3ef, 55 },
    { 0, 0 },
    { 0x6ac6f3f2, 52 },
    { 0x8246f3f3, 61 },
    { 0xf026f3f4, 90 },
    { 0xc9f6f3f3, 98 },
    { 0xfec6f3f6, 66 },
    { 0xfba6f3f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586f3fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6926f400, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf966f406, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9706f40a, 61 },
    { 0x2f56f40a, 90 },
    { 0xc9a6f40c, 81 },
    { 0xe346f40c, 90 },
    { 0x5c66f40a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8396f413, 93 },
    { 0xcb96f414, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca56f417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa296f41e, 93 },
    { 0xc096f41e, 43 },
    { 0x2b26f41f, 63 },
    { 0x5db6f421, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe6f424, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd746f427, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc566f42d, 85 },
    { 0x5a36f42e, 33 },
    { 0, 0 },
    { 0x82e6f430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e6f433, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f6f43b, 90 },
    { 0, 0 },
    { 0x2676f43d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06f444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x666f447, 95 },
    { 0, 0 },
    { 0x5f76f449, 23 },
    { 0x41a6f44a, 75 },
    { 0, 0 },
    { 0x2d46f44c, 66 },
    { 0, 0 },
    { 0x4616f44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3316f455, 61 },
    { 0, 0 },
    { 0x7476f457, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7626f45a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb966f45f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c6f465, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f468, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe896f46d, 100 },
    { 0, 0 },
    { 0x4e36f46f, 98 },
    { 0, 0 },
    { 0xc9f6f471, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066f475, 95 },
    { 0xb506f475, 66 },
    { 0x7996f477, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7636f47a, 85 },
    { 0xf6b6f47a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc956f481, 76 },
    { 0xc476f482, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f6f485, 37 },
    { 0x3916f485, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d66f48b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e96f48e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c6f492, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2186f496, 61 },
    { 0x55e6f497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c6f49a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff6f49d, 55 },
    { 0x9906f49e, 65 },
    { 0xb3c6f49f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf426f4a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4396f4a5, 76 },
    { 0xc26f4a5, 80 },
    { 0x2f26f4a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7796f4ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b6f4af, 85 },
    { 0, 0 },
    { 0xffc6f4b1, 95 },
    { 0x4436f4b1, 91 },
    { 0x2616f4b2, 95 },
    { 0x2506f4b4, 33 },
    { 0x2626f4b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb196f4c3, 76 },
    { 0x69f6f4c4, 46 },
    { 0xcb56f4c3, 85 },
    { 0, 0 },
    { 0xf346f4c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b56f4cb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76f4d8, 80 },
    { 0x8106f4d8, 87 },
    { 0xf866f4da, 53 },
    { 0x5ea6f4db, 33 },
    { 0xd526f4d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c06f4e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad6f4e4, 90 },
    { 0xde56f4e5, 98 },
    { 0x5256f4e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a06f4e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc776f4f0, 87 },
    { 0x356f4f1, 23 },
    { 0x65d6f4f1, 33 },
    { 0, 0 },
    { 0x62f6f4f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a6f4f7, 98 },
    { 0xdd76f4f8, 43 },
    { 0x2536f4f8, 90 },
    { 0x6916f4fa, 33 },
    { 0x94d6f4fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d6f500, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9056f505, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6506f509, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc56f512, 33 },
    { 0, 0 },
    { 0xb506f514, 34 },
    { 0xd1a6f514, 90 },
    { 0x3b06f514, 33 },
    { 0xa346f517, 95 },
    { 0x10f6f514, 98 },
    { 0, 0 },
    { 0x8806f51a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596f522, 91 },
    { 0xc916f522, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6f528, 33 },
    { 0x3fb6f529, 52 },
    { 0xa3a6f52a, 61 },
    { 0x8a56f529, 90 },
    { 0x81b6f52c, 61 },
    { 0x6036f52c, 90 },
    { 0xb506f52c, 94 },
    { 0x3e26f52f, 100 },
    { 0, 0 },
    { 0x2176f531, 90 },
    { 0, 0 },
    { 0x3306f533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb6f53b, 33 },
    { 0x6456f53c, 75 },
    { 0x84c6f53d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b6f548, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6f54b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3446f54e, 33 },
    { 0xe316f54f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f6f553, 23 },
    { 0x8be6f554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3496f55b, 93 },
    { 0xd036f55c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a6f560, 55 },
    { 0xb4c6f561, 61 },
    { 0x4b46f562, 90 },
    { 0x5cd6f562, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e6f569, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36f56c, 90 },
    { 0x3926f56d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6726f570, 88 },
    { 0, 0 },
    { 0x3476f572, 85 },
    { 0x45d6f573, 90 },
    { 0x4f06f574, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a6f577, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe796f57a, 46 },
    { 0xed66f57b, 88 },
    { 0x7466f57b, 46 },
    { 0x5fa6f57a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f6f584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6f58b, 61 },
    { 0x1096f58b, 59 },
    { 0xa906f58d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8036f590, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3216f597, 33 },
    { 0x1736f597, 55 },
    { 0x79f6f599, 98 },
    { 0, 0 },
    { 0x6336f59b, 33 },
    { 0xbe36f59b, 33 },
    { 0x1dd6f59d, 76 },
    { 0x1896f59c, 66 },
    { 0xa96f59f, 56 },
    { 0x5006f59f, 95 },
    { 0, 0 },
    { 0x84e6f5a2, 91 },
    { 0xfe16f5a2, 61 },
    { 0x5596f5a2, 60 },
    { 0xbaf6f5a5, 23 },
    { 0xeef6f5a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3696f5aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff56f5ae, 90 },
    { 0xbf16f5af, 95 },
    { 0xac06f5af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc26f5b6, 33 },
    { 0, 0 },
    { 0x3e6f5b8, 93 },
    { 0x5596f5b9, 90 },
    { 0x6f16f5b9, 55 },
    { 0, 0 },
    { 0xbcc6f5bc, 98 },
    { 0xfeb6f5bd, 93 },
    { 0x22b6f5be, 23 },
    { 0, 0 },
    { 0x8996f5c0, 34 },
    { 0x2f06f5c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf16f5c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe116f5ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e6f5cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6196f5d1, 95 },
    { 0xd956f5d2, 33 },
    { 0x31c6f5d3, 77 },
    { 0x8146f5d3, 90 },
    { 0xb486f5d5, 95 },
    { 0xe756f5d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9696f5da, 33 },
    { 0, 0 },
    { 0x75f6f5dc, 98 },
    { 0x626f5dd, 61 },
    { 0x7856f5dd, 43 },
    { 0x72c6f5df, 61 },
    { 0x4d36f5df, 69 },
    { 0xf3f6f5e1, 99 },
    { 0xa116f5e2, 90 },
    { 0x8b66f5e3, 23 },
    { 0xf3d6f5e2, 55 },
    { 0x16e6f5e2, 93 },
    { 0x6be6f5e6, 61 },
    { 0x4be6f5e6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d6f5ea, 98 },
    { 0x70d6f5ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6f5ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7876f5fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b6f5ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4976f605, 52 },
    { 0xd06f606, 33 },
    { 0x5806f606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456f60b, 91 },
    { 0x4cf6f60b, 98 },
    { 0x5486f60d, 40 },
    { 0xd3b6f60e, 98 },
    { 0x4906f60f, 95 },
    { 0xdbf6f610, 53 },
    { 0xaea6f611, 95 },
    { 0xe106f612, 35 },
    { 0x3de6f613, 90 },
    { 0xb7e6f614, 76 },
    { 0xb336f613, 93 },
    { 0xd716f616, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6f619, 98 },
    { 0, 0 },
    { 0x33e6f61b, 65 },
    { 0xbd46f61c, 93 },
    { 0xf3d6f61d, 33 },
    { 0x58c6f61e, 95 },
    { 0, 0 },
    { 0x2086f620, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e66f629, 43 },
    { 0, 0 },
    { 0xa386f62b, 33 },
    { 0x4e16f62b, 90 },
    { 0x5466f62b, 98 },
    { 0x5be6f62e, 34 },
    { 0xa236f62b, 55 },
    { 0xbf96f630, 55 },
    { 0x24a6f62f, 76 },
    { 0x9f86f62d, 95 },
    { 0x7c6f62e, 90 },
    { 0xefc6f631, 46 },
    { 0x9936f635, 87 },
    { 0x8906f636, 100 },
    { 0x1786f62f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f16f63a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b86f63e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5826f641, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee6f648, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb16f64b, 33 },
    { 0x4f76f64c, 84 },
    { 0, 0 },
    { 0xb486f64e, 34 },
    { 0xb636f64f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8016f653, 99 },
    { 0, 0 },
    { 0xe096f655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e6f65a, 84 },
    { 0x2aa6f65b, 46 },
    { 0, 0 },
    { 0xbfd6f65d, 66 },
    { 0x60c6f65d, 59 },
    { 0x8596f65e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116f662, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e6f66a, 76 },
    { 0, 0 },
    { 0x8466f66c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e6f675, 33 },
    { 0x3e86f676, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f6f67b, 55 },
    { 0x4cc6f67c, 95 },
    { 0xdd76f67d, 98 },
    { 0xa916f67e, 98 },
    { 0x8b46f67e, 95 },
    { 0, 0 },
    { 0x7b46f681, 90 },
    { 0, 0 },
    { 0x46e6f683, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6f688, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa96f68b, 33 },
    { 0xb2b6f68c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1756f692, 90 },
    { 0xf256f693, 91 },
    { 0xc226f694, 76 },
    { 0x3406f695, 93 },
    { 0x6886f693, 87 },
    { 0xba86f697, 33 },
    { 0xe26f696, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8366f69f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb6f6a5, 90 },
    { 0x25d6f6a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c6f6ab, 77 },
    { 0, 0 },
    { 0x1ff6f6ad, 43 },
    { 0xa206f6ae, 98 },
    { 0x65c6f6af, 34 },
    { 0x1246f6b0, 76 },
    { 0x9986f6b1, 34 },
    { 0xf276f6af, 84 },
    { 0, 0 },
    { 0xd866f6b4, 77 },
    { 0x6c96f6b5, 85 },
    { 0x816f6b5, 33 },
    { 0, 0 },
    { 0xe6b6f6b8, 33 },
    { 0xa5f6f6b9, 34 },
    { 0x59b6f6ba, 76 },
    { 0x59d6f6bb, 90 },
    { 0xd96f6ba, 43 },
    { 0xea56f6bd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c6f6c6, 76 },
    { 0, 0 },
    { 0xb426f6c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b6f6cc, 61 },
    { 0xc806f6cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe446f6d2, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xd386f6d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6f6d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db6f6dc, 98 },
    { 0x1316f6dc, 61 },
    { 0, 0 },
    { 0x8896f6df, 23 },
    { 0x646f6e0, 23 },
    { 0x7a66f6e1, 61 },
    { 0x8426f6e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe816f6e5, 40 },
    { 0xfa16f6e6, 33 },
    { 0x9136f6e7, 33 },
    { 0, 0 },
    { 0x6c6f6e9, 33 },
    { 0xf576f6ea, 81 },
    { 0xe3d6f6eb, 93 },
    { 0x3996f6ec, 55 },
    { 0x76c6f6ed, 76 },
    { 0x14c6f6ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3066f6f1, 95 },
    { 0x39c6f6f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b16f6f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6f6fe, 95 },
    { 0x89a6f6ff, 43 },
    { 0x8c46f700, 33 },
    { 0xd256f701, 90 },
    { 0x2036f701, 93 },
    { 0xe686f703, 90 },
    { 0x9de6f704, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac6f709, 76 },
    { 0x5036f70a, 98 },
    { 0x5d26f70b, 90 },
    { 0, 0 },
    { 0x7bf6f70d, 61 },
    { 0, 0 },
    { 0x396f70f, 61 },
    { 0, 0 },
    { 0xce26f711, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16f720, 46 },
    { 0x2b6f721, 81 },
    { 0x3e06f721, 23 },
    { 0x1036f723, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab36f729, 34 },
    { 0x1506f72a, 99 },
    { 0, 0 },
    { 0x2836f72c, 33 },
    { 0x4526f72d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8716f731, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d6f73a, 95 },
    { 0x9b16f73b, 91 },
    { 0x18a6f73b, 90 },
    { 0xc816f73d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f16f746, 61 },
    { 0x71a6f747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc936f74a, 95 },
    { 0x7866f74a, 23 },
    { 0x7e16f74c, 61 },
    { 0, 0 },
    { 0xc266f74e, 61 },
    { 0x8366f74f, 33 },
    { 0x9a66f74f, 55 },
    { 0x5516f751, 68 },
    { 0x3e6f752, 95 },
    { 0x54b6f753, 76 },
    { 0x8d6f752, 61 },
    { 0x7f6f754, 61 },
    { 0xc8f6f756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe66f75a, 98 },
    { 0, 0 },
    { 0x3096f75c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f86f75f, 95 },
    { 0, 0 },
    { 0x3456f761, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd6f768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6f76b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf6f76e, 93 },
    { 0xeaf6f76e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3096f777, 34 },
    { 0x6de6f778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1196f77b, 61 },
    { 0x5be6f77b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f6f783, 23 },
    { 0x5e86f784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe076f788, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b6f78b, 95 },
    { 0x6f46f78c, 85 },
    { 0, 0 },
    { 0xb296f78e, 33 },
    { 0, 0 },
    { 0x6fc6f790, 33 },
    { 0x9896f791, 88 },
    { 0x5a6f792, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a36f795, 76 },
    { 0x80f6f796, 44 },
    { 0, 0 },
    { 0x4466f798, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb756f79d, 43 },
    { 0, 0 },
    { 0x6626f79f, 55 },
    { 0x62d6f79f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa646f7a6, 100 },
    { 0x6d16f7a7, 85 },
    { 0x4476f7a7, 81 },
    { 0xb2e6f7a8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e96f7ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f7b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d96f7b4, 61 },
    { 0x3116f7b5, 61 },
    { 0x4576f7b6, 76 },
    { 0x9886f7b7, 100 },
    { 0x7726f7b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2286f7bb, 95 },
    { 0xe3b6f7bb, 33 },
    { 0xa0a6f7bd, 83 },
    { 0xe66f7bb, 33 },
    { 0x89c6f7bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9196f7c2, 98 },
    { 0x3db6f7c2, 33 },
    { 0xc826f7c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf86f7c9, 98 },
    { 0xe566f7ca, 40 },
    { 0x9a96f7cb, 43 },
    { 0xf8a6f7cc, 88 },
    { 0xe56f7cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe516f7d2, 40 },
    { 0x9e16f7d3, 85 },
    { 0x1f66f7d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8916f7dd, 61 },
    { 0, 0 },
    { 0x6f7df, 98 },
    { 0, 0 },
    { 0x4cb6f7e1, 90 },
    { 0, 0 },
    { 0x89e6f7e3, 87 },
    { 0x1656f7e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e6f7e7, 84 },
    { 0x396f7e7, 95 },
    { 0, 0 },
    { 0x88b6f7ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a6f7ed, 43 },
    { 0xde06f7ee, 34 },
    { 0xb5b6f7ef, 78 },
    { 0xbfc6f7f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3586f7f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d96f7fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe06f7ff, 33 },
    { 0x1de6f7ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef56f803, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f6f808, 98 },
    { 0x9976f808, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5946f80d, 33 },
    { 0x5526f80e, 34 },
    { 0, 0 },
    { 0x70a6f810, 90 },
    { 0xd086f810, 33 },
    { 0xfc66f812, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc646f81d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd436f821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e06f829, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2496f830, 90 },
    { 0, 0 },
    { 0x7376f832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb226f835, 61 },
    { 0xbfc6f835, 95 },
    { 0x35a6f837, 93 },
    { 0x2da6f838, 23 },
    { 0, 0 },
    { 0x9d46f83a, 90 },
    { 0, 0 },
    { 0xe856f83c, 90 },
    { 0, 0 },
    { 0x2886f83e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad6f845, 98 },
    { 0x5586f846, 23 },
    { 0xe8b6f845, 66 },
    { 0x93d6f848, 88 },
    { 0x3376f849, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d96f851, 61 },
    { 0xda46f852, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf96f857, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe196f85a, 61 },
    { 0, 0 },
    { 0xe176f85c, 40 },
    { 0, 0 },
    { 0x1d16f85e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd6f863, 96 },
    { 0x4996f864, 99 },
    { 0xa286f865, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb56f869, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe756f86e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a6f871, 61 },
    { 0, 0 },
    { 0xe5b6f873, 96 },
    { 0xe566f874, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb996f879, 99 },
    { 0x39d6f87a, 95 },
    { 0x6606f879, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf396f87e, 17 },
    { 0x3526f87f, 33 },
    { 0, 0 },
    { 0x2f46f881, 33 },
    { 0x6346f882, 34 },
    { 0, 0 },
    { 0xeb96f884, 78 },
    { 0xba6f885, 100 },
    { 0x6fe6f884, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf806f88c, 81 },
    { 0, 0 },
    { 0x6b56f88e, 90 },
    { 0x59b6f88e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6f893, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866f89a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c06f8a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea76f8a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb6f8ad, 91 },
    { 0, 0 },
    { 0x8fc6f8af, 33 },
    { 0, 0 },
    { 0xafa6f8b1, 98 },
    { 0, 0 },
    { 0x4fe6f8b3, 61 },
    { 0, 0 },
    { 0xf156f8b5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd66f8bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa996f8c3, 43 },
    { 0x7466f8c4, 55 },
    { 0, 0 },
    { 0x9896f8c6, 61 },
    { 0x5546f8c7, 90 },
    { 0x3b06f8c7, 41 },
    { 0x7ed6f8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8746f8cd, 33 },
    { 0x6836f8ce, 34 },
    { 0, 0 },
    { 0xacb6f8d0, 43 },
    { 0xe726f8d1, 23 },
    { 0xfa6f8d2, 33 },
    { 0x68f6f8d1, 43 },
    { 0xb386f8d2, 33 },
    { 0, 0 },
    { 0xfdc6f8d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6f8da, 97 },
    { 0, 0 },
    { 0x1b56f8dc, 61 },
    { 0, 0 },
    { 0xd0e6f8de, 90 },
    { 0x7dc6f8df, 85 },
    { 0x27a6f8df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6456f8e3, 97 },
    { 0x776f8e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa666f8e8, 61 },
    { 0, 0 },
    { 0x21e6f8ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6f8f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x966f8f6, 77 },
    { 0x2a36f8f6, 90 },
    { 0xa3a6f8f8, 43 },
    { 0x56c6f8f7, 23 },
    { 0x3916f8fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d6f8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf466f904, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f86f907, 33 },
    { 0x2d66f908, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b6f90b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c06f90f, 33 },
    { 0x4e06f910, 98 },
    { 0, 0 },
    { 0xce56f912, 90 },
    { 0, 0 },
    { 0x4186f914, 98 },
    { 0x8346f914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2436f918, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8556f91b, 90 },
    { 0x3ae6f91b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b6f921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc6f927, 98 },
    { 0x72e6f928, 49 },
    { 0x5096f929, 46 },
    { 0x9726f929, 78 },
    { 0x88f6f92b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b6f92e, 90 },
    { 0, 0 },
    { 0x4f16f930, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb6f93a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d6f93d, 33 },
    { 0x6b66f93e, 52 },
    { 0xd0d6f93d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf206f943, 98 },
    { 0xb0c6f943, 61 },
    { 0xbaf6f945, 75 },
    { 0xa496f946, 33 },
    { 0x3e26f944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd886f94a, 55 },
    { 0, 0 },
    { 0xa446f94c, 98 },
    { 0, 0 },
    { 0xaa06f94e, 23 },
    { 0x5166f94e, 81 },
    { 0xdbc6f950, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc6f954, 95 },
    { 0x22c6f955, 52 },
    { 0xe5a6f956, 61 },
    { 0x2776f954, 51 },
    { 0xec16f957, 35 },
    { 0xba6f957, 33 },
    { 0x9f66f95a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f6f95f, 33 },
    { 0xcee6f95f, 23 },
    { 0x99d6f961, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa96f964, 78 },
    { 0x1396f965, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e26f96c, 23 },
    { 0, 0 },
    { 0x4286f96e, 87 },
    { 0x1456f96f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f6f972, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e6f975, 38 },
    { 0xbc86f975, 61 },
    { 0x7466f977, 90 },
    { 0xa396f977, 61 },
    { 0x5de6f979, 95 },
    { 0x7176f977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766f986, 88 },
    { 0x2096f987, 99 },
    { 0xad06f987, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e6f990, 55 },
    { 0x4b26f991, 23 },
    { 0xe4e6f992, 23 },
    { 0x79f6f993, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d6f999, 42 },
    { 0x2ac6f99a, 98 },
    { 0x9f56f999, 19 },
    { 0, 0 },
    { 0x54f6f99d, 61 },
    { 0xf266f99e, 41 },
    { 0x11d6f99f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c86f9a2, 98 },
    { 0, 0 },
    { 0x4dc6f9a4, 33 },
    { 0x3506f9a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda46f9a8, 33 },
    { 0xa7c6f9a9, 99 },
    { 0x8166f9aa, 98 },
    { 0, 0 },
    { 0x3356f9ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9256f9b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f26f9b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb076f9be, 90 },
    { 0x8476f9bf, 34 },
    { 0x6ea6f9bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a6f9c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c56f9c7, 90 },
    { 0xb3c6f9c8, 87 },
    { 0x7d26f9c9, 99 },
    { 0x95a6f9ca, 90 },
    { 0x60e6f9ca, 66 },
    { 0x2e06f9c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd606f9cf, 40 },
    { 0, 0 },
    { 0x49e6f9d1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2816f9d7, 76 },
    { 0x39f6f9d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f9e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe196f9ec, 67 },
    { 0x3af6f9ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a6f9f0, 61 },
    { 0x5376f9f1, 61 },
    { 0, 0 },
    { 0x5bc6f9f3, 55 },
    { 0xb036f9f3, 26 },
    { 0x4f26f9f5, 90 },
    { 0xe9b6f9f6, 90 },
    { 0x2a66f9f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f6f9fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7176fa00, 56 },
    { 0x2836fa01, 95 },
    { 0x8cb6fa02, 33 },
    { 0, 0 },
    { 0xce46fa04, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf316fa08, 90 },
    { 0x52a6fa09, 49 },
    { 0, 0 },
    { 0x9ce6fa0b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe686fa10, 88 },
    { 0x6a16fa10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc6fa14, 52 },
    { 0, 0 },
    { 0x6a66fa16, 90 },
    { 0x13a6fa16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7706fa1b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a6fa21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1216fa25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd6fa28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa6fa2f, 55 },
    { 0xdda6fa30, 61 },
    { 0xef56fa2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc86fa38, 95 },
    { 0x69d6fa39, 33 },
    { 0x2596fa3a, 98 },
    { 0x8856fa3b, 46 },
    { 0x7756fa3c, 33 },
    { 0, 0 },
    { 0x8f6fa3e, 95 },
    { 0xb5a6fa3f, 23 },
    { 0x23e6fa3f, 52 },
    { 0x8266fa3f, 87 },
    { 0x8216fa3e, 90 },
    { 0, 0 },
    { 0x7b66fa44, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc846fa4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f06fa56, 91 },
    { 0x2696fa56, 68 },
    { 0x4186fa57, 90 },
    { 0x5046fa59, 90 },
    { 0xcd86fa57, 90 },
    { 0xc1b6fa57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa6fa5f, 33 },
    { 0x87d6fa60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3246fa64, 84 },
    { 0, 0 },
    { 0xa0d6fa66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a16fa69, 34 },
    { 0, 0 },
    { 0xc136fa6b, 76 },
    { 0x2246fa6c, 93 },
    { 0, 0 },
    { 0xcda6fa6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b6fa79, 90 },
    { 0xbd6fa79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f56fa7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b6fa81, 95 },
    { 0xe516fa81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa706fa88, 95 },
    { 0, 0 },
    { 0x9d06fa8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c6fa8d, 76 },
    { 0x1c76fa8e, 55 },
    { 0x74f6fa8f, 23 },
    { 0x65e6fa90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db6fa96, 90 },
    { 0xe036fa96, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b46fa9b, 53 },
    { 0x8bb6fa9b, 23 },
    { 0, 0 },
    { 0x7da6fa9e, 23 },
    { 0x1db6fa9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc066faa2, 77 },
    { 0xdba6faa2, 95 },
    { 0xeb66faa2, 98 },
    { 0x4726faa5, 33 },
    { 0, 0 },
    { 0x80a6faa7, 95 },
    { 0, 0 },
    { 0xc846faa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a6faac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6fab0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b06fab3, 100 },
    { 0xbef6fab3, 88 },
    { 0x4496fab5, 33 },
    { 0, 0 },
    { 0x4656fab7, 61 },
    { 0x9a96fab7, 95 },
    { 0xaf46fab9, 90 },
    { 0xef16faba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6fabe, 90 },
    { 0, 0 },
    { 0xf956fac0, 61 },
    { 0xea66fac1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1546fac5, 23 },
    { 0xf46fac6, 99 },
    { 0, 0 },
    { 0x4c06fac8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e6facd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c46fad0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f6fad4, 23 },
    { 0x5306fad4, 66 },
    { 0x5036fad6, 91 },
    { 0, 0 },
    { 0x43f6fad8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b6fade, 98 },
    { 0, 0 },
    { 0x39f6fae0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d6fae5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9266fae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc56faef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2516faf2, 75 },
    { 0xf3c6faf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c6faf6, 55 },
    { 0x5396faf7, 23 },
    { 0xf8f6faf7, 61 },
    { 0x9ed6faf9, 43 },
    { 0xf6e6faf9, 23 },
    { 0xf5c6fafb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4006fafe, 23 },
    { 0, 0 },
    { 0x1066fb00, 23 },
    { 0x35a6fb01, 35 },
    { 0, 0 },
    { 0xbc86fb03, 71 },
    { 0xce56fb04, 98 },
    { 0xec06fb05, 34 },
    { 0, 0 },
    { 0x5166fb07, 33 },
    { 0x5786fb08, 95 },
    { 0, 0 },
    { 0x9776fb0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d6fb0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e6fb15, 61 },
    { 0x86fb16, 85 },
    { 0, 0 },
    { 0xd556fb18, 33 },
    { 0, 0 },
    { 0xc2b6fb1a, 90 },
    { 0x16b6fb1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e86fb1e, 33 },
    { 0x5ea6fb1f, 93 },
    { 0x7986fb1f, 98 },
    { 0x4586fb21, 98 },
    { 0xcc6fb20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e56fb26, 87 },
    { 0, 0 },
    { 0xf7a6fb28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d6fb2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226fb2e, 67 },
    { 0, 0 },
    { 0xc2a6fb30, 52 },
    { 0xf666fb31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf6fb34, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd176fb39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb606fb3d, 17 },
    { 0x366fb3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd6fb41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe256fb4c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c96fb4f, 23 },
    { 0xfec6fb4f, 55 },
    { 0x256fb51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x466fb54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f66fb59, 93 },
    { 0x1b96fb59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d6fb5e, 85 },
    { 0x7056fb5f, 61 },
    { 0x69b6fb5f, 98 },
    { 0xf1a6fb5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6fb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe06fb69, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5526fb6e, 87 },
    { 0xe0b6fb6f, 95 },
    { 0xcd26fb70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7166fb74, 78 },
    { 0x7916fb75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f86fb79, 76 },
    { 0xb736fb7a, 76 },
    { 0, 0 },
    { 0x1c36fb7c, 34 },
    { 0x4ff6fb7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa96fb81, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a86fb87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe516fb8a, 47 },
    { 0xcc16fb8b, 61 },
    { 0x7166fb8c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6246fb90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc06fb93, 66 },
    { 0, 0 },
    { 0xc3d6fb95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf446fb98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac46fb9b, 84 },
    { 0x44f6fb9b, 85 },
    { 0, 0 },
    { 0x6516fb9e, 23 },
    { 0xed36fb9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da6fba4, 33 },
    { 0x6fba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5426fba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b6fbac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706fbb1, 95 },
    { 0x7116fbb2, 95 },
    { 0xa826fbb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f6fbb9, 46 },
    { 0x2346fbba, 98 },
    { 0xf516fbba, 33 },
    { 0xd526fbbc, 95 },
    { 0x1db6fbbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc16fbc2, 76 },
    { 0x5026fbc2, 55 },
    { 0xec86fbc4, 90 },
    { 0xe856fbc4, 12 },
    { 0x706fbc6, 98 },
    { 0xb406fbc7, 90 },
    { 0xcaf6fbc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b6fbcb, 33 },
    { 0x3a26fbcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb116fbd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe6fbd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d6fbd8, 91 },
    { 0x5cd6fbd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb046fbdc, 90 },
    { 0, 0 },
    { 0x11f6fbde, 61 },
    { 0xe2f6fbdf, 100 },
    { 0, 0 },
    { 0x4ed6fbe1, 61 },
    { 0xe3d6fbe2, 91 },
    { 0, 0 },
    { 0x4356fbe4, 23 },
    { 0x986fbe4, 76 },
    { 0x76fbe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f6fbe9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe216fbed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3746fbf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb26fbfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5636fc02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c6fc05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa546fc09, 88 },
    { 0xcbd6fc0a, 95 },
    { 0x1b56fc0b, 77 },
    { 0x9586fc0b, 98 },
    { 0xe686fc0d, 33 },
    { 0, 0 },
    { 0x63f6fc0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3436fc15, 43 },
    { 0x8f06fc16, 90 },
    { 0x89b6fc15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e6fc1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d6fc27, 42 },
    { 0x80c6fc27, 95 },
    { 0xb1e6fc28, 99 },
    { 0x62e6fc2a, 85 },
    { 0x1226fc2b, 61 },
    { 0x1a56fc2c, 33 },
    { 0x61a6fc2d, 66 },
    { 0x6616fc2a, 68 },
    { 0x6a16fc2e, 34 },
    { 0x9326fc2b, 69 },
    { 0xc896fc2a, 33 },
    { 0xb6a6fc2b, 61 },
    { 0x9e66fc27, 90 },
    { 0xf516fc33, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d6fc3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e6fc46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2456fc4a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4986fc4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a36fc55, 23 },
    { 0, 0 },
    { 0xc66fc57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6fc5a, 46 },
    { 0xe946fc5b, 90 },
    { 0x6a46fc5c, 90 },
    { 0x8606fc5c, 95 },
    { 0x7006fc5e, 98 },
    { 0x6e6fc5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6fc62, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf676fc68, 55 },
    { 0x4d66fc69, 95 },
    { 0, 0 },
    { 0xc066fc6b, 66 },
    { 0x4996fc6b, 95 },
    { 0x3ac6fc6d, 90 },
    { 0x8806fc6e, 61 },
    { 0xe696fc6e, 33 },
    { 0x7a46fc70, 45 },
    { 0xe1d6fc6d, 85 },
    { 0x1b86fc72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5536fc75, 99 },
    { 0x2ac6fc75, 61 },
    { 0xb4f6fc77, 33 },
    { 0xc076fc76, 95 },
    { 0, 0 },
    { 0x8b16fc7a, 63 },
    { 0, 0 },
    { 0x3656fc7c, 90 },
    { 0, 0 },
    { 0x2d36fc7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac6fc81, 93 },
    { 0xf216fc82, 100 },
    { 0x4ef6fc81, 75 },
    { 0, 0 },
    { 0x5eb6fc85, 23 },
    { 0xa536fc86, 61 },
    { 0, 0 },
    { 0xf3e6fc88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6fc8b, 98 },
    { 0x76c6fc8c, 52 },
    { 0x9f56fc8d, 23 },
    { 0, 0 },
    { 0xbaa6fc8f, 90 },
    { 0, 0 },
    { 0xc756fc91, 23 },
    { 0x5056fc91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a26fc9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc6fc9f, 34 },
    { 0xb796fca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca6fca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df6fcab, 33 },
    { 0xbdf6fcab, 90 },
    { 0, 0 },
    { 0x97f6fcae, 76 },
    { 0, 0 },
    { 0xc816fcb0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6fcb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1316fcc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd586fcc9, 61 },
    { 0x7f46fcca, 76 },
    { 0xe4a6fccb, 51 },
    { 0xd086fccb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b96fccf, 66 },
    { 0xa3a6fcd0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3656fcd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56fcd6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb6fcdc, 17 },
    { 0, 0 },
    { 0xa096fcde, 78 },
    { 0x23d6fcde, 84 },
    { 0xea26fcde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd326fce3, 33 },
    { 0xaa06fce4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa16fce7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a76fcea, 90 },
    { 0xdbc6fceb, 90 },
    { 0x61d6fcec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb526fcef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd06fcf2, 81 },
    { 0x1f06fcf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6876fcf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f6fcfd, 81 },
    { 0, 0 },
    { 0xc376fcff, 100 },
    { 0, 0 },
    { 0x5576fd01, 17 },
    { 0xc476fd02, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa6fd08, 98 },
    { 0xd446fd09, 85 },
    { 0, 0 },
    { 0x3d16fd0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb6fd11, 66 },
    { 0x4006fd12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb916fd15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe686fd18, 90 },
    { 0x6dd6fd19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4566fd22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e06fd25, 90 },
    { 0x4d06fd25, 91 },
    { 0x3b96fd27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e6fd2a, 33 },
    { 0x7b36fd2b, 61 },
    { 0xd736fd2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8386fd30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e6fd34, 69 },
    { 0xfa16fd35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b6fd3d, 76 },
    { 0xf566fd3d, 87 },
    { 0xf036fd3f, 90 },
    { 0x9ee6fd40, 96 },
    { 0xd206fd3d, 34 },
    { 0x31f6fd3f, 98 },
    { 0x9e26fd43, 93 },
    { 0x9166fd44, 76 },
    { 0xb4e6fd44, 61 },
    { 0x4a36fd45, 33 },
    { 0xdd86fd3d, 95 },
    { 0, 0 },
    { 0x5e86fd49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4226fd4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1666fd54, 88 },
    { 0x5e66fd55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a6fd5a, 56 },
    { 0x5966fd5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06fd5e, 68 },
    { 0x35f6fd5f, 90 },
    { 0xf0a6fd60, 84 },
    { 0x5816fd61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8456fd64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae6fd6e, 61 },
    { 0x7456fd6f, 34 },
    { 0xcfa6fd70, 90 },
    { 0x22f6fd71, 78 },
    { 0, 0 },
    { 0xc096fd73, 98 },
    { 0x73d6fd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6756fd77, 76 },
    { 0xdbb6fd77, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb6fd81, 69 },
    { 0, 0 },
    { 0x2626fd83, 66 },
    { 0xcd6fd84, 76 },
    { 0, 0 },
    { 0xb056fd86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa76fd8b, 95 },
    { 0x9386fd8b, 96 },
    { 0, 0 },
    { 0x7bb6fd8e, 43 },
    { 0xb446fd8e, 33 },
    { 0, 0 },
    { 0xa2d6fd91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1886fd95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee6fd98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe806fda1, 22 },
    { 0xeba6fda2, 23 },
    { 0xa666fda3, 76 },
    { 0xc956fda2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc716fdaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a6fdba, 90 },
    { 0x4616fdbb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab76fdc2, 61 },
    { 0xe8f6fdc2, 64 },
    { 0x3576fdc4, 46 },
    { 0xb256fdc5, 52 },
    { 0, 0 },
    { 0x4676fdc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9826fdca, 95 },
    { 0x62c6fdcb, 83 },
    { 0xe486fdcc, 98 },
    { 0xf916fdca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7666fdd1, 97 },
    { 0x76f6fdd2, 91 },
    { 0xf1a6fdd3, 40 },
    { 0xc6d6fdd1, 34 },
    { 0x38b6fdd5, 61 },
    { 0x4d66fdd5, 90 },
    { 0xd206fdd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2236fdda, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7736fddd, 23 },
    { 0x1da6fddd, 33 },
    { 0x7266fdde, 61 },
    { 0x9936fde0, 98 },
    { 0xd886fde1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4326fde8, 95 },
    { 0xd016fde9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a6fdec, 43 },
    { 0x2336fded, 23 },
    { 0x1cf6fded, 91 },
    { 0x4a06fdef, 90 },
    { 0, 0 },
    { 0x9f86fdf1, 23 },
    { 0xc136fdf2, 99 },
    { 0, 0 },
    { 0xe656fdf4, 90 },
    { 0, 0 },
    { 0x8b96fdf6, 95 },
    { 0x3946fdf7, 98 },
    { 0x42b6fdf7, 55 },
    { 0xdcd6fdf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1496fdfc, 61 },
    { 0, 0 },
    { 0x6e26fdfe, 76 },
    { 0xd556fdff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd136fe02, 90 },
    { 0, 0 },
    { 0x2226fe04, 77 },
    { 0x9eb6fe05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f6fe0b, 33 },
    { 0, 0 },
    { 0x13f6fe0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc466fe16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d6fe1f, 90 },
    { 0x766fe20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b96fe25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa046fe2a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4776fe2f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d86fe33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6fe3a, 15 },
    { 0, 0 },
    { 0xb96fe3c, 85 },
    { 0x1216fe3d, 33 },
    { 0x4596fe3e, 98 },
    { 0x6136fe3e, 33 },
    { 0xe696fe40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fe43, 66 },
    { 0xd346fe43, 33 },
    { 0, 0 },
    { 0x70a6fe46, 80 },
    { 0, 0 },
    { 0x9df6fe48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9596fe4e, 33 },
    { 0, 0 },
    { 0x2766fe50, 84 },
    { 0, 0 },
    { 0x2986fe52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x856fe55, 99 },
    { 0x9306fe55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb836fe5b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d6fe62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce6fe67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da6fe6b, 95 },
    { 0, 0 },
    { 0x2a06fe6d, 100 },
    { 0xbd66fe6e, 38 },
    { 0, 0 },
    { 0xc526fe70, 95 },
    { 0xd196fe71, 99 },
    { 0, 0 },
    { 0x26b6fe73, 99 },
    { 0, 0 },
    { 0x16c6fe75, 66 },
    { 0x74f6fe76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9826fe7d, 90 },
    { 0x2bb6fe7d, 38 },
    { 0xf696fe7f, 61 },
    { 0xbe26fe80, 66 },
    { 0x8476fe80, 24 },
    { 0x7826fe82, 90 },
    { 0, 0 },
    { 0x9716fe84, 33 },
    { 0x6de6fe85, 23 },
    { 0xea56fe84, 98 },
    { 0x5166fe87, 78 },
    { 0xc06fe88, 81 },
    { 0x2e26fe87, 34 },
    { 0xa236fe86, 87 },
    { 0, 0 },
    { 0x8bc6fe8c, 61 },
    { 0xc226fe8d, 98 },
    { 0x6aa6fe8e, 98 },
    { 0x1b66fe8e, 61 },
    { 0xf966fe8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2196fe93, 61 },
    { 0, 0 },
    { 0x4c16fe95, 34 },
    { 0xce26fe96, 85 },
    { 0x2a96fe96, 52 },
    { 0x6c06fe98, 90 },
    { 0xca16fe99, 76 },
    { 0xdc36fe99, 33 },
    { 0xa006fe96, 95 },
    { 0x2ee6fe96, 43 },
    { 0, 0 },
    { 0x5826fe9e, 95 },
    { 0, 0 },
    { 0xd1e6fea0, 90 },
    { 0x5c76fea0, 91 },
    { 0xb146fea2, 90 },
    { 0x6d76fea3, 23 },
    { 0xa5d6fea3, 43 },
    { 0x75d6fea5, 95 },
    { 0x3da6fea6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136feb5, 66 },
    { 0xf7e6feb6, 33 },
    { 0x4f66feb7, 43 },
    { 0x93a6feb7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d16febf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9666fec2, 55 },
    { 0, 0 },
    { 0x7416fec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f6feca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb456fecf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa346fed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa906fee4, 33 },
    { 0, 0 },
    { 0x4e16fee6, 75 },
    { 0, 0 },
    { 0xc3a6fee8, 55 },
    { 0, 0 },
    { 0xc696feea, 33 },
    { 0xd926feea, 95 },
    { 0, 0 },
    { 0xd536feed, 61 },
    { 0, 0 },
    { 0x3cb6feef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e6fef2, 23 },
    { 0xb9f6fef3, 87 },
    { 0x9d36fef4, 90 },
    { 0xdbc6fef5, 33 },
    { 0x69c6fef6, 40 },
    { 0x9376fef2, 98 },
    { 0xd9e6fef8, 43 },
    { 0x7446fef9, 95 },
    { 0x34b6fef9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fefd, 34 },
    { 0, 0 },
    { 0xbba6feff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a6ff02, 33 },
    { 0, 0 },
    { 0x4ad6ff04, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e6ff0b, 78 },
    { 0x25b6ff0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3126ff10, 23 },
    { 0, 0 },
    { 0x2ea6ff12, 61 },
    { 0x9a76ff13, 61 },
    { 0xcd6ff14, 84 },
    { 0xdc86ff15, 90 },
    { 0, 0 },
    { 0x636ff17, 33 },
    { 0x23e6ff18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b6ff1d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b6ff21, 90 },
    { 0, 0 },
    { 0x4e6ff23, 33 },
    { 0x7026ff23, 76 },
    { 0x2f56ff25, 98 },
    { 0x1e26ff24, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e6ff29, 34 },
    { 0x2e26ff2a, 91 },
    { 0, 0 },
    { 0x1156ff2c, 23 },
    { 0xf9c6ff2d, 52 },
    { 0x4916ff2d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea6ff33, 90 },
    { 0, 0 },
    { 0x36ff35, 91 },
    { 0, 0 },
    { 0x1386ff37, 61 },
    { 0, 0 },
    { 0x95a6ff39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a16ff41, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f56ff44, 61 },
    { 0, 0 },
    { 0xe136ff46, 43 },
    { 0x4996ff46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b6ff4c, 90 },
    { 0x83f6ff4d, 46 },
    { 0xde56ff4e, 61 },
    { 0x9dd6ff4f, 98 },
    { 0x3376ff4d, 90 },
    { 0xcbe6ff50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe356ff55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9396ff59, 43 },
    { 0, 0 },
    { 0xf5f6ff5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c6ff63, 95 },
    { 0, 0 },
    { 0x6566ff65, 81 },
    { 0xa6ff65, 61 },
    { 0, 0 },
    { 0xf416ff68, 93 },
    { 0, 0 },
    { 0x71e6ff6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ff6e, 95 },
    { 0, 0 },
    { 0x76ff70, 23 },
    { 0, 0 },
    { 0x9366ff72, 23 },
    { 0x7066ff73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7376ff76, 61 },
    { 0x4ad6ff76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3176ff7b, 55 },
    { 0x70f6ff7b, 46 },
    { 0xf326ff7d, 78 },
    { 0x42b6ff7d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d36ff81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd956ff86, 98 },
    { 0xa9e6ff86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea16ff8c, 46 },
    { 0x9aa6ff8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d16ff92, 61 },
    { 0, 0 },
    { 0x506ff94, 90 },
    { 0x9116ff94, 46 },
    { 0x51b6ff96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b76ff99, 98 },
    { 0x55d6ff9a, 65 },
    { 0x5386ff9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6406ff9f, 76 },
    { 0xbba6ff9f, 90 },
    { 0, 0 },
    { 0x2fd6ffa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc6ffa7, 61 },
    { 0, 0 },
    { 0x8286ffa9, 53 },
    { 0x1b46ffaa, 76 },
    { 0x536ffa9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa646ffaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b6ffb5, 90 },
    { 0x596ffb6, 90 },
    { 0x3336ffb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e6ffbd, 17 },
    { 0, 0 },
    { 0x536ffbf, 23 },
    { 0xede6ffc0, 90 },
    { 0x1f36ffc1, 100 },
    { 0, 0 },
    { 0x5326ffc3, 95 },
    { 0x5176ffc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6626ffc7, 90 },
    { 0xc866ffc8, 99 },
    { 0, 0 },
    { 0x5be6ffca, 76 },
    { 0x80e6ffca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e76ffcf, 68 },
    { 0x19f6ffd0, 91 },
    { 0x6506ffd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa506ffd8, 17 },
    { 0, 0 },
    { 0xb786ffda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb6ffe2, 95 },
    { 0, 0 },
    { 0xeda6ffe4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96ffed, 19 },
    { 0x67b6ffee, 85 },
    { 0x9ca6ffee, 84 },
    { 0xd16fff0, 78 },
    { 0xe2d6fff1, 90 },
    { 0xf986fff0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be6fff6, 46 },
    { 0x16d6fff7, 23 },
    { 0x5ec6fff6, 98 },
    { 0xed06fff7, 61 },
    { 0x7316fffa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1516fffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68770000, 98 },
    { 0x7dc70001, 98 },
    { 0, 0 },
    { 0x51070003, 84 },
    { 0, 0 },
    { 0x77070005, 34 },
    { 0x9d670006, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb987000b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4170011, 98 },
    { 0, 0 },
    { 0x96770013, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c170016, 90 },
    { 0, 0 },
    { 0xc7970018, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8157001b, 23 },
    { 0x8917001b, 64 },
    { 0, 0 },
    { 0x62f7001e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda870022, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3870028, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a7002e, 34 },
    { 0, 0 },
    { 0x3aa70030, 93 },
    { 0x2d370030, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d70034, 88 },
    { 0x75370034, 46 },
    { 0xd2370036, 55 },
    { 0x43a70036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac7003a, 95 },
    { 0xe8a7003b, 77 },
    { 0x7537003c, 43 },
    { 0x4677003b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8470043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24370046, 45 },
    { 0x3dc70046, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c7004b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff7004f, 33 },
    { 0x4747004f, 83 },
    { 0x7fa70051, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac70058, 90 },
    { 0, 0 },
    { 0x28a7005a, 90 },
    { 0, 0 },
    { 0xa967005c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc70060, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd470063, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc70067, 95 },
    { 0xf1170068, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a07006b, 81 },
    { 0x5d7006c, 77 },
    { 0xd5f7006d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99470071, 33 },
    { 0xbce70072, 76 },
    { 0, 0 },
    { 0xfa770074, 76 },
    { 0xb0e70075, 46 },
    { 0x47670076, 90 },
    { 0x72870077, 46 },
    { 0, 0 },
    { 0x76170079, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e7007d, 90 },
    { 0xafa7007e, 95 },
    { 0xd47007d, 55 },
    { 0x4bb70080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca170083, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f70086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c570089, 68 },
    { 0x57370089, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d7008d, 61 },
    { 0xf277008d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc70093, 34 },
    { 0x53870093, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd117009a, 90 },
    { 0, 0 },
    { 0xad57009c, 66 },
    { 0x2bd7009d, 95 },
    { 0, 0 },
    { 0x937009f, 61 },
    { 0x798700a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710700aa, 66 },
    { 0, 0 },
    { 0xd0c700ac, 43 },
    { 0x1fb700ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0700b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42700b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d700b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664700bc, 95 },
    { 0, 0 },
    { 0x1d0700be, 97 },
    { 0xa9b700bf, 76 },
    { 0x575700c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57700c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f700c9, 87 },
    { 0x3a9700ca, 95 },
    { 0xd49700ca, 33 },
    { 0x39e700ca, 90 },
    { 0xbd3700cd, 87 },
    { 0x498700ce, 98 },
    { 0, 0 },
    { 0x59a700d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad700d5, 33 },
    { 0xb2e700d5, 55 },
    { 0x60b700d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382700db, 33 },
    { 0, 0 },
    { 0xced700dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1700e0, 33 },
    { 0, 0 },
    { 0xf6a700e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x74700e5, 33 },
    { 0x9bb700e5, 95 },
    { 0x59b700e6, 98 },
    { 0x17b700e8, 43 },
    { 0x4a1700e9, 95 },
    { 0x727700e9, 98 },
    { 0, 0 },
    { 0x83a700ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79700f0, 82 },
    { 0, 0 },
    { 0x8b7700f2, 23 },
    { 0, 0 },
    { 0xd68700f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933700fa, 76 },
    { 0x99b700fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe700fe, 46 },
    { 0xda7700ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58170103, 23 },
    { 0x94570103, 95 },
    { 0xfb470105, 23 },
    { 0x5e370106, 65 },
    { 0xd7e70107, 23 },
    { 0x21a70107, 33 },
    { 0x7470107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53670114, 100 },
    { 0x25770114, 33 },
    { 0x24f70116, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c70119, 61 },
    { 0, 0 },
    { 0x4ce7011b, 95 },
    { 0x6997011c, 98 },
    { 0xd2c7011b, 43 },
    { 0xef17011c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca070126, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6670129, 75 },
    { 0x7db70129, 100 },
    { 0x5f7012a, 19 },
    { 0x7012c, 98 },
    { 0x22c7012d, 98 },
    { 0x5487012d, 90 },
    { 0xbe37012c, 40 },
    { 0x8807012c, 84 },
    { 0x8577012d, 95 },
    { 0x8e170129, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22970138, 91 },
    { 0xef570139, 95 },
    { 0x99c70138, 90 },
    { 0xb5a7013b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37013e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d70148, 98 },
    { 0xd1e70148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf497014c, 97 },
    { 0x15d7014d, 76 },
    { 0x7437014d, 91 },
    { 0xfbe7014d, 95 },
    { 0, 0 },
    { 0xc670151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56270154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4370157, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc7015a, 90 },
    { 0, 0 },
    { 0xbe97015c, 87 },
    { 0xfef7015d, 17 },
    { 0x9a87015d, 34 },
    { 0x3ad7015f, 98 },
    { 0x9a67015d, 98 },
    { 0x4ef70161, 23 },
    { 0x8f670162, 33 },
    { 0x36f70162, 61 },
    { 0x27270163, 61 },
    { 0xaa77015f, 95 },
    { 0xf8770161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e70169, 61 },
    { 0x35f7016a, 23 },
    { 0x51570169, 75 },
    { 0x1c77016c, 33 },
    { 0, 0 },
    { 0xa1f7016e, 66 },
    { 0, 0 },
    { 0x16c70170, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6070173, 90 },
    { 0, 0 },
    { 0xea270175, 97 },
    { 0x59470176, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a7017a, 66 },
    { 0, 0 },
    { 0x167017c, 91 },
    { 0x6f67017c, 23 },
    { 0x10b7017e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e70182, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc70185, 18 },
    { 0xb7770186, 95 },
    { 0xa2f70187, 90 },
    { 0xbb270188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7007018b, 84 },
    { 0x1217018c, 65 },
    { 0, 0 },
    { 0xfb87018e, 66 },
    { 0xe2e7018f, 100 },
    { 0x8fd7018f, 33 },
    { 0x5c370191, 61 },
    { 0x4aa7018f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23170196, 33 },
    { 0, 0 },
    { 0x8c870198, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb7019c, 93 },
    { 0xc67019d, 17 },
    { 0x1447019e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a701a4, 87 },
    { 0xb28701a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a701a8, 90 },
    { 0x427701a9, 95 },
    { 0xe3f701aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73701af, 47 },
    { 0xb76701b0, 76 },
    { 0x520701b1, 90 },
    { 0xa10701b1, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da701b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6701bf, 90 },
    { 0x506701c0, 66 },
    { 0xa95701c0, 67 },
    { 0x50d701c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2701cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851701cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc701d3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07701d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866701dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02701df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb701e2, 95 },
    { 0, 0 },
    { 0xcb6701e4, 33 },
    { 0x32701e4, 55 },
    { 0xde3701e5, 66 },
    { 0xa81701e6, 76 },
    { 0x232701e8, 33 },
    { 0x2ec701e8, 98 },
    { 0xbff701ea, 34 },
    { 0xab3701ea, 34 },
    { 0, 0 },
    { 0x6f5701ed, 77 },
    { 0, 0 },
    { 0xb77701ef, 77 },
    { 0x355701f0, 90 },
    { 0x536701f0, 46 },
    { 0xce7701ef, 95 },
    { 0xea7701f3, 34 },
    { 0xc62701f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26701f7, 100 },
    { 0x6cb701f8, 36 },
    { 0x51b701f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d70200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e470203, 98 },
    { 0x71a70204, 76 },
    { 0xcfc70205, 95 },
    { 0xddc70205, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a7020e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f70212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46670215, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f70218, 43 },
    { 0, 0 },
    { 0x7ac7021a, 98 },
    { 0xb6d7021a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84970222, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc70225, 23 },
    { 0x41370225, 98 },
    { 0x6ed70227, 14 },
    { 0x4bc70227, 66 },
    { 0x13570227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77022f, 76 },
    { 0x3a70230, 95 },
    { 0xa6670230, 93 },
    { 0, 0 },
    { 0x20f70233, 23 },
    { 0x86170234, 98 },
    { 0, 0 },
    { 0xb5270236, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a70239, 23 },
    { 0x3f97023a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137023f, 47 },
    { 0xd057023f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c70243, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe70246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e17024e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf70252, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf870256, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e17025c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf70261, 33 },
    { 0xbc570262, 61 },
    { 0x2de70262, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3070268, 34 },
    { 0x4a870269, 90 },
    { 0, 0 },
    { 0x6187026b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d7026f, 99 },
    { 0, 0 },
    { 0x4c70271, 33 },
    { 0, 0 },
    { 0x98d70273, 95 },
    { 0xcbb70274, 64 },
    { 0, 0 },
    { 0xb4e70276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb17027e, 90 },
    { 0, 0 },
    { 0xad70280, 76 },
    { 0x6b270280, 98 },
    { 0x19b70282, 33 },
    { 0, 0 },
    { 0xada70284, 96 },
    { 0x5a770284, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8470288, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d7028d, 90 },
    { 0xddf7028e, 98 },
    { 0, 0 },
    { 0xf7670290, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b470297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2607029a, 23 },
    { 0, 0 },
    { 0x3817029c, 23 },
    { 0x6657029d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x440702a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59702a3, 61 },
    { 0xff702a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9702aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87702b3, 61 },
    { 0xeff702b3, 61 },
    { 0x7ea702b5, 76 },
    { 0x69f702b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0702b9, 81 },
    { 0x540702b9, 87 },
    { 0x4fd702bb, 90 },
    { 0x7f4702bc, 61 },
    { 0xbfb702bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49702c1, 23 },
    { 0x6fa702c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5702c6, 87 },
    { 0, 0 },
    { 0x659702c8, 52 },
    { 0xde7702c9, 23 },
    { 0, 0 },
    { 0x95c702cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4702d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb702d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf702dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1702e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed702ec, 98 },
    { 0xb3702ed, 98 },
    { 0x3d702ee, 17 },
    { 0, 0 },
    { 0x5eb702f0, 33 },
    { 0xb09702f1, 99 },
    { 0, 0 },
    { 0x12b702f3, 85 },
    { 0, 0 },
    { 0xd70702f5, 33 },
    { 0xe24702f6, 76 },
    { 0xf59702f7, 67 },
    { 0x7a3702f6, 95 },
    { 0x539702f9, 52 },
    { 0, 0 },
    { 0x6a3702fb, 76 },
    { 0x41e702fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d270300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7170305, 69 },
    { 0, 0 },
    { 0xc0b70307, 76 },
    { 0x87d70307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a7030d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b70311, 33 },
    { 0x14570311, 90 },
    { 0x57670313, 93 },
    { 0xd4970314, 61 },
    { 0x7bf70315, 87 },
    { 0xb3370313, 98 },
    { 0xbd170311, 23 },
    { 0xc3870318, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa717031c, 55 },
    { 0xfec7031d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b70321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2570327, 98 },
    { 0xb1470328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7032c, 77 },
    { 0, 0 },
    { 0x7317032e, 61 },
    { 0, 0 },
    { 0xd1870330, 61 },
    { 0x5b670331, 33 },
    { 0, 0 },
    { 0xfa870333, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a970337, 43 },
    { 0x9e270338, 33 },
    { 0xda70339, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d670345, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4397034a, 23 },
    { 0, 0 },
    { 0x4467034c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f07034f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9970352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b070355, 90 },
    { 0x60c70356, 33 },
    { 0x61870357, 61 },
    { 0x79e70357, 90 },
    { 0xe8470355, 98 },
    { 0, 0 },
    { 0x9e57035b, 90 },
    { 0x9a77035c, 90 },
    { 0xd037035c, 90 },
    { 0, 0 },
    { 0xd367035f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd70364, 87 },
    { 0x76a70365, 53 },
    { 0x7dd70366, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db7036a, 52 },
    { 0xd807036b, 77 },
    { 0xc877036c, 90 },
    { 0x4b7036d, 23 },
    { 0x4f87036d, 84 },
    { 0xb827036f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29570374, 87 },
    { 0xd4f70375, 68 },
    { 0xbd770376, 97 },
    { 0xc6070376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e7037c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20570384, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7038a, 34 },
    { 0xdd97038a, 33 },
    { 0x3f17038a, 88 },
    { 0x3b97038b, 66 },
    { 0x4c47038e, 43 },
    { 0x8937038a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c70392, 78 },
    { 0x7dd70393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32870396, 90 },
    { 0, 0 },
    { 0x22670398, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5497039c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c703a2, 95 },
    { 0, 0 },
    { 0x8f5703a4, 96 },
    { 0, 0 },
    { 0x7b7703a6, 33 },
    { 0x61a703a7, 61 },
    { 0x236703a8, 46 },
    { 0xf3703a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e703ac, 61 },
    { 0xa21703ac, 23 },
    { 0x144703ae, 91 },
    { 0x914703af, 61 },
    { 0x24f703b0, 61 },
    { 0xf83703b0, 32 },
    { 0x6f9703b2, 91 },
    { 0xf0b703b3, 93 },
    { 0x44f703b4, 34 },
    { 0xbfa703b3, 85 },
    { 0xf6a703b5, 33 },
    { 0xeaa703ac, 33 },
    { 0xef2703b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7703bf, 64 },
    { 0x16a703c0, 85 },
    { 0x9d9703c1, 90 },
    { 0, 0 },
    { 0xe4703c3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7703c6, 23 },
    { 0x39b703c7, 91 },
    { 0x530703c8, 98 },
    { 0x435703c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x215703cc, 33 },
    { 0, 0 },
    { 0xced703ce, 61 },
    { 0, 0 },
    { 0xacb703d0, 55 },
    { 0, 0 },
    { 0xd1a703d2, 97 },
    { 0, 0 },
    { 0x142703d4, 96 },
    { 0x8de703d5, 98 },
    { 0, 0 },
    { 0x4a5703d7, 81 },
    { 0xb83703d7, 46 },
    { 0x6e703d9, 46 },
    { 0xc43703d9, 76 },
    { 0, 0 },
    { 0xe2a703dc, 65 },
    { 0, 0 },
    { 0xddf703de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2703eb, 61 },
    { 0, 0 },
    { 0x230703ed, 46 },
    { 0x431703ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08703f1, 90 },
    { 0x4e5703f2, 33 },
    { 0, 0 },
    { 0xb92703f4, 85 },
    { 0xe52703f5, 46 },
    { 0, 0 },
    { 0xbfd703f7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33703fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a370400, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b70403, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b70406, 33 },
    { 0x40270407, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217040b, 66 },
    { 0xed07040c, 56 },
    { 0xf9e7040d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3970410, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32070414, 61 },
    { 0x1e70415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c7041a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7041d, 33 },
    { 0, 0 },
    { 0x3fc7041f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d70425, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64970429, 55 },
    { 0xdb570429, 33 },
    { 0x91c7042b, 46 },
    { 0xdf97042c, 33 },
    { 0x9827042b, 96 },
    { 0x5447042e, 33 },
    { 0xaa87042e, 61 },
    { 0xff37042b, 90 },
    { 0x7cf70430, 90 },
    { 0x4387042c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c070438, 81 },
    { 0xccc70438, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9637043c, 99 },
    { 0xd507043d, 96 },
    { 0xf757043e, 55 },
    { 0x1497043f, 61 },
    { 0xcc77043f, 55 },
    { 0xdb470441, 52 },
    { 0xde77043f, 92 },
    { 0x76470442, 98 },
    { 0xd2f70443, 33 },
    { 0xbeb70445, 61 },
    { 0x9dd70446, 91 },
    { 0xd8870447, 66 },
    { 0x35570447, 95 },
    { 0xa9970443, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c77044c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d70450, 47 },
    { 0x8f870451, 66 },
    { 0, 0 },
    { 0x76f70453, 95 },
    { 0, 0 },
    { 0x69570455, 95 },
    { 0xfd070456, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa7045d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1870467, 90 },
    { 0x6e670468, 97 },
    { 0x78870469, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c7046f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270475, 76 },
    { 0x68b70475, 95 },
    { 0x4bc70475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67047b, 23 },
    { 0x1f27047b, 55 },
    { 0, 0 },
    { 0xeaf7047e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf570481, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65570487, 95 },
    { 0x30f70487, 95 },
    { 0x74470488, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d17048c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb570499, 33 },
    { 0x6a07049a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757049d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe704a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be704a8, 46 },
    { 0x906704a9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x917704ac, 64 },
    { 0x24d704ad, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2704b1, 98 },
    { 0, 0 },
    { 0x585704b3, 34 },
    { 0x705704b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f704b8, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd704bc, 55 },
    { 0xe94704bc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb704c1, 76 },
    { 0xab9704c1, 33 },
    { 0x7c9704c3, 33 },
    { 0xf84704c4, 43 },
    { 0x8e3704c4, 55 },
    { 0x4da704c6, 76 },
    { 0x34d704c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e704cc, 61 },
    { 0xd9d704cd, 61 },
    { 0, 0 },
    { 0xc66704cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7704d3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86704d6, 90 },
    { 0, 0 },
    { 0xaa7704d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14704db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4704e0, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x873704e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1704e7, 23 },
    { 0, 0 },
    { 0x26d704e9, 52 },
    { 0xd77704ea, 33 },
    { 0xd5d704eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4704f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402704f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f704fd, 95 },
    { 0, 0 },
    { 0xa92704ff, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf70504, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f7050b, 61 },
    { 0, 0 },
    { 0x6537050d, 93 },
    { 0x2e67050d, 85 },
    { 0x1517050d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a70514, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda470518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137051b, 34 },
    { 0xfbb7051b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a70520, 46 },
    { 0xa7370520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a970529, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4770530, 65 },
    { 0xd2470530, 90 },
    { 0x1b270530, 33 },
    { 0x41670533, 61 },
    { 0x21070534, 61 },
    { 0xa1670535, 90 },
    { 0xe9070536, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe7053a, 95 },
    { 0x2337053b, 98 },
    { 0x13e7053c, 33 },
    { 0x42f7053a, 98 },
    { 0x9647053e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2170543, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab70546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b70549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7317054c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287054f, 43 },
    { 0, 0 },
    { 0x75670551, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f70557, 95 },
    { 0, 0 },
    { 0x13a70559, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c17055d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58170565, 19 },
    { 0xa7170566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda370569, 23 },
    { 0xb8e70569, 33 },
    { 0, 0 },
    { 0x9e47056c, 67 },
    { 0, 0 },
    { 0xfc67056e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a70571, 33 },
    { 0x47b70572, 55 },
    { 0xd0c70572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a470577, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa7057b, 90 },
    { 0xea07057c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff7057f, 55 },
    { 0, 0 },
    { 0x33570581, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa7058a, 93 },
    { 0, 0 },
    { 0x1b37058c, 33 },
    { 0, 0 },
    { 0x3e47058e, 95 },
    { 0xc4a7058f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89270592, 95 },
    { 0x55e70592, 33 },
    { 0x4f570594, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175705a5, 88 },
    { 0x2f1705a6, 68 },
    { 0xb02705a6, 66 },
    { 0xb91705a8, 66 },
    { 0xd50705a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27705ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6705b0, 90 },
    { 0x68f705b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52705be, 87 },
    { 0x11f705bf, 55 },
    { 0x9d8705c0, 61 },
    { 0xa81705be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe705ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0705d5, 33 },
    { 0x6a3705d5, 87 },
    { 0xb6705d7, 43 },
    { 0x827705d8, 78 },
    { 0x20a705d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858705df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7705f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d705f5, 23 },
    { 0x787705f5, 85 },
    { 0x5df705f7, 90 },
    { 0x9d8705f8, 34 },
    { 0x226705f5, 85 },
    { 0x4cb705f6, 61 },
    { 0xd3d705fb, 66 },
    { 0xdf3705fa, 77 },
    { 0xa56705fd, 98 },
    { 0x9e705f9, 95 },
    { 0x7b705fc, 95 },
    { 0, 0 },
    { 0x46470601, 46 },
    { 0, 0 },
    { 0x6ae70603, 33 },
    { 0x43b70604, 86 },
    { 0x5d370604, 33 },
    { 0xe0770606, 43 },
    { 0x42c70606, 33 },
    { 0x6b070608, 90 },
    { 0x5a770604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767060f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42570613, 23 },
    { 0x5ae70614, 61 },
    { 0x42270615, 81 },
    { 0x5b170613, 34 },
    { 0xbf170617, 26 },
    { 0xd3670618, 55 },
    { 0x8be70615, 61 },
    { 0xe9a7061a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81170620, 90 },
    { 0x2ae70621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe370626, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe857062c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5270630, 59 },
    { 0xbd70631, 52 },
    { 0x68870632, 90 },
    { 0x23270633, 23 },
    { 0, 0 },
    { 0x66670635, 61 },
    { 0, 0 },
    { 0xb5470637, 23 },
    { 0x64d70637, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a870644, 90 },
    { 0, 0 },
    { 0xa4b70646, 90 },
    { 0x6a770647, 87 },
    { 0, 0 },
    { 0xdaf70649, 61 },
    { 0xac47064a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e070651, 99 },
    { 0x96670651, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64470657, 33 },
    { 0xf6c70658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd977065b, 100 },
    { 0x5ba7065c, 23 },
    { 0xdd57065c, 76 },
    { 0, 0 },
    { 0xa777065f, 98 },
    { 0, 0 },
    { 0xf8370661, 61 },
    { 0xb2770661, 87 },
    { 0xbe270662, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b70668, 34 },
    { 0x8d70669, 90 },
    { 0x7cd7066a, 23 },
    { 0x4f57066b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c7066e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9370673, 90 },
    { 0, 0 },
    { 0xc3170675, 84 },
    { 0xd0c70676, 61 },
    { 0xb0070677, 99 },
    { 0xbfc70678, 23 },
    { 0, 0 },
    { 0x1f37067a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8817067f, 23 },
    { 0x9a370680, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba070686, 75 },
    { 0x1c270687, 90 },
    { 0xa3070686, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47470694, 100 },
    { 0, 0 },
    { 0x80470696, 95 },
    { 0xb8a70697, 90 },
    { 0xe0070696, 17 },
    { 0x37570699, 91 },
    { 0x1ab7069a, 90 },
    { 0x5af7069b, 33 },
    { 0x3667069b, 33 },
    { 0xcc07069d, 33 },
    { 0x4be70699, 23 },
    { 0xee77069f, 93 },
    { 0x1f17069a, 95 },
    { 0x6a0706a1, 61 },
    { 0, 0 },
    { 0x404706a3, 33 },
    { 0xcfb706a4, 99 },
    { 0, 0 },
    { 0x69a706a6, 87 },
    { 0, 0 },
    { 0xdf3706a8, 17 },
    { 0xdd8706a9, 61 },
    { 0xf19706a9, 55 },
    { 0xe46706a8, 33 },
    { 0xdb9706ac, 46 },
    { 0x51a706ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d706b1, 61 },
    { 0, 0 },
    { 0x3ca706b3, 100 },
    { 0, 0 },
    { 0xd69706b5, 61 },
    { 0xabe706b6, 76 },
    { 0x735706b7, 85 },
    { 0x8c4706b7, 76 },
    { 0xea6706b6, 61 },
    { 0x755706ba, 90 },
    { 0x9fe706bb, 23 },
    { 0xb49706b6, 95 },
    { 0xc6f706ba, 90 },
    { 0x77d706bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d706c1, 90 },
    { 0x623706c2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6706c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23706ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e706d1, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6706d4, 91 },
    { 0xe49706d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36706e0, 17 },
    { 0x8f0706e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7706e5, 88 },
    { 0xf83706e6, 61 },
    { 0xedb706e5, 90 },
    { 0, 0 },
    { 0x652706e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e706f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19706f3, 46 },
    { 0xc32706f4, 90 },
    { 0xeca706f4, 90 },
    { 0xb3d706f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377706fa, 33 },
    { 0x629706fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7706ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d70708, 98 },
    { 0x46470709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3567070f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f70717, 90 },
    { 0x3c070718, 33 },
    { 0, 0 },
    { 0xf837071a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a7071e, 46 },
    { 0xa967071f, 55 },
    { 0xee770720, 33 },
    { 0x7c570720, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f270725, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b770728, 61 },
    { 0x78170729, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a97072f, 23 },
    { 0, 0 },
    { 0x98f70731, 61 },
    { 0, 0 },
    { 0x8770733, 95 },
    { 0x9d970734, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee970737, 81 },
    { 0x9e570738, 61 },
    { 0x8ea70739, 33 },
    { 0x37370738, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7073d, 67 },
    { 0xbd37073d, 33 },
    { 0x24d7073f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e70746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57074b, 23 },
    { 0x9c77074c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94070750, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75170755, 34 },
    { 0x47270755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e70759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7077075c, 33 },
    { 0, 0 },
    { 0xea97075e, 77 },
    { 0xddd7075e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8570767, 99 },
    { 0, 0 },
    { 0x49f70769, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a970773, 87 },
    { 0xf6770773, 85 },
    { 0, 0 },
    { 0xa1e70776, 98 },
    { 0x63170776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e7077c, 66 },
    { 0xf607077c, 43 },
    { 0, 0 },
    { 0x82d7077f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57570782, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d170785, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17078a, 93 },
    { 0, 0 },
    { 0x2307078c, 95 },
    { 0x8f67078d, 98 },
    { 0xd8d7078c, 93 },
    { 0, 0 },
    { 0x23870790, 34 },
    { 0xac570791, 81 },
    { 0xc5f70791, 43 },
    { 0xaed70792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c70796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b7079e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x205707a1, 46 },
    { 0x29b707a2, 61 },
    { 0xed4707a3, 53 },
    { 0, 0 },
    { 0xa68707a5, 87 },
    { 0x412707a6, 67 },
    { 0x667707a7, 91 },
    { 0x2c2707a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a707ab, 52 },
    { 0x9a9707ab, 34 },
    { 0x39707ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1707b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b707b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706707bc, 90 },
    { 0x67d707bd, 98 },
    { 0x670707be, 33 },
    { 0xec8707be, 55 },
    { 0xa53707c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7707c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee707c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c707e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a707e7, 90 },
    { 0xeea707e7, 61 },
    { 0xc94707e9, 61 },
    { 0, 0 },
    { 0x7ce707eb, 90 },
    { 0, 0 },
    { 0x1ac707ed, 76 },
    { 0xc2c707ed, 33 },
    { 0, 0 },
    { 0xf34707f0, 23 },
    { 0x626707f1, 23 },
    { 0xa46707f2, 23 },
    { 0x4f4707f2, 99 },
    { 0x23e707f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b707f8, 34 },
    { 0x21f707f8, 90 },
    { 0x676707fa, 75 },
    { 0xb8707f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98707fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab70807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6807080a, 61 },
    { 0, 0 },
    { 0x2f87080c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c70810, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0070817, 82 },
    { 0x3870818, 33 },
    { 0xdd770817, 90 },
    { 0, 0 },
    { 0x18a7081b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3670828, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1657082b, 33 },
    { 0, 0 },
    { 0x4ad7082d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0870833, 55 },
    { 0xccc70834, 90 },
    { 0, 0 },
    { 0x58070836, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2670840, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a70843, 34 },
    { 0, 0 },
    { 0xed870845, 93 },
    { 0, 0 },
    { 0x93170847, 93 },
    { 0, 0 },
    { 0x7c370849, 87 },
    { 0xc087084a, 61 },
    { 0, 0 },
    { 0x5717084c, 65 },
    { 0x5ee7084d, 98 },
    { 0x4387084d, 95 },
    { 0x2197084f, 98 },
    { 0x1a87084d, 98 },
    { 0xeac7084d, 96 },
    { 0xb7870852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b70856, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3770859, 91 },
    { 0xbf970859, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec70861, 33 },
    { 0xed770862, 91 },
    { 0xe8170862, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa70866, 59 },
    { 0xb1270867, 61 },
    { 0, 0 },
    { 0x87a70869, 100 },
    { 0x76f7086a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e17086f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66170872, 61 },
    { 0x6c470872, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x28970876, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca370879, 43 },
    { 0, 0 },
    { 0x1207087b, 76 },
    { 0xf787087c, 95 },
    { 0x1807087d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c70881, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc70885, 80 },
    { 0x50070886, 93 },
    { 0xfe170886, 87 },
    { 0x8c070888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd87088b, 75 },
    { 0x6e47088c, 61 },
    { 0, 0 },
    { 0xc737088e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3270896, 33 },
    { 0x28e70897, 95 },
    { 0x7aa70897, 46 },
    { 0x3ad70899, 91 },
    { 0x4267089a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae97089d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1708aa, 85 },
    { 0x484708ab, 17 },
    { 0xf96708ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db708b0, 97 },
    { 0x787708b1, 100 },
    { 0x892708b0, 82 },
    { 0, 0 },
    { 0x64e708b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28708b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800708bd, 46 },
    { 0xc8f708be, 61 },
    { 0, 0 },
    { 0xd0f708c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30708c3, 23 },
    { 0x684708c4, 93 },
    { 0, 0 },
    { 0xcfb708c6, 76 },
    { 0xb82708c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0708ca, 55 },
    { 0x488708ca, 90 },
    { 0x4d708cc, 98 },
    { 0x77a708cc, 55 },
    { 0, 0 },
    { 0xf3b708cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d708d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28708d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47708db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f708e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16708e3, 76 },
    { 0, 0 },
    { 0xf89708e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66708ec, 55 },
    { 0xdc2708ec, 55 },
    { 0xb1d708ee, 90 },
    { 0xf14708ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530708f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f708f8, 77 },
    { 0, 0 },
    { 0xdc3708fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60708fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25270904, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe170908, 98 },
    { 0x1b370908, 55 },
    { 0xe7370908, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a97090d, 61 },
    { 0xb1e7090e, 23 },
    { 0, 0 },
    { 0xc9370910, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac70913, 23 },
    { 0, 0 },
    { 0x44170915, 99 },
    { 0, 0 },
    { 0x78470917, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c7091b, 43 },
    { 0xaa57091c, 61 },
    { 0, 0 },
    { 0xe9c7091e, 75 },
    { 0, 0 },
    { 0x68b70920, 90 },
    { 0xf2d70921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14570925, 80 },
    { 0x29470926, 55 },
    { 0xebd70926, 90 },
    { 0xbb70928, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7092e, 17 },
    { 0, 0 },
    { 0x21f70930, 52 },
    { 0xc6970930, 80 },
    { 0, 0 },
    { 0xefb70933, 23 },
    { 0x15470934, 99 },
    { 0xaf870934, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87093c, 95 },
    { 0xb17093d, 75 },
    { 0x4867093d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7670941, 23 },
    { 0x6c070942, 90 },
    { 0xaeb70943, 61 },
    { 0x63b70943, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e70948, 90 },
    { 0x93e70948, 90 },
    { 0xb077094a, 95 },
    { 0x41b7094b, 90 },
    { 0x46570949, 93 },
    { 0xe97094c, 61 },
    { 0x2e57094c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b370952, 33 },
    { 0x3a770953, 84 },
    { 0x34870954, 61 },
    { 0xc2070952, 61 },
    { 0x33770955, 76 },
    { 0x53070957, 23 },
    { 0x7c70958, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf70962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad70965, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x16570968, 33 },
    { 0xae370968, 61 },
    { 0x53e7096a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9077096f, 98 },
    { 0x81c70970, 33 },
    { 0x6a27096f, 78 },
    { 0, 0 },
    { 0x45e70973, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d170979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd577097d, 90 },
    { 0, 0 },
    { 0x3f37097f, 66 },
    { 0x1407097f, 90 },
    { 0xd0670980, 55 },
    { 0x50f70982, 90 },
    { 0xebd70983, 55 },
    { 0, 0 },
    { 0x42170985, 75 },
    { 0xccc70986, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7098e, 76 },
    { 0xa137098e, 33 },
    { 0xf5070990, 59 },
    { 0, 0 },
    { 0x65f70992, 34 },
    { 0x54770993, 100 },
    { 0x8cd70993, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14370998, 90 },
    { 0x38670999, 34 },
    { 0, 0 },
    { 0x6e67099b, 61 },
    { 0x1d47099b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da7099f, 33 },
    { 0, 0 },
    { 0xcaf709a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61709a7, 76 },
    { 0xbfc709a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16709ac, 23 },
    { 0, 0 },
    { 0xb17709ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4709b1, 33 },
    { 0x81b709b1, 95 },
    { 0x4b7709b3, 90 },
    { 0x9b9709b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f709b7, 55 },
    { 0xcc4709b8, 96 },
    { 0x693709b9, 85 },
    { 0x458709b9, 87 },
    { 0xf32709ba, 69 },
    { 0xa2d709bb, 33 },
    { 0x586709bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9709c8, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3709cb, 43 },
    { 0, 0 },
    { 0x8c7709cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad709d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312709d5, 98 },
    { 0x984709d6, 98 },
    { 0x484709d7, 88 },
    { 0x269709d7, 98 },
    { 0xea8709d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445709e1, 33 },
    { 0, 0 },
    { 0x2ba709e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70709e7, 100 },
    { 0x846709e8, 90 },
    { 0, 0 },
    { 0xf8c709ea, 33 },
    { 0, 0 },
    { 0xae0709ec, 77 },
    { 0x4de709ed, 98 },
    { 0, 0 },
    { 0x15709ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc709f2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d709f5, 23 },
    { 0, 0 },
    { 0xef2709f7, 94 },
    { 0xbb6709f8, 90 },
    { 0xd40709f9, 67 },
    { 0x787709fa, 61 },
    { 0xc7d709f7, 55 },
    { 0xa0c709fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed70a02, 61 },
    { 0x51370a03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db70a06, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d70a09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d70a10, 84 },
    { 0, 0 },
    { 0xe0a70a12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9370a15, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b70a1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a70a1f, 95 },
    { 0x6ab70a1f, 91 },
    { 0, 0 },
    { 0x53770a22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf70a25, 90 },
    { 0xc9a70a26, 45 },
    { 0x9aa70a26, 55 },
    { 0x4df70a27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b70a2d, 90 },
    { 0x66070a2e, 23 },
    { 0x9d070a2e, 52 },
    { 0x73b70a30, 81 },
    { 0xeb870a30, 61 },
    { 0x98c70a32, 46 },
    { 0, 0 },
    { 0x68670a34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e70a38, 90 },
    { 0, 0 },
    { 0x3070a3a, 95 },
    { 0, 0 },
    { 0x9f670a3c, 61 },
    { 0xfee70a3d, 100 },
    { 0x3a170a3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec370a49, 64 },
    { 0x4af70a49, 61 },
    { 0x6d470a4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24670a4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc70a51, 6 },
    { 0, 0 },
    { 0x2de70a53, 23 },
    { 0xaf870a53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44370a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d70a5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a70a64, 55 },
    { 0, 0 },
    { 0xea070a66, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c70a69, 33 },
    { 0x4e870a6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e070a71, 52 },
    { 0x6d470a72, 33 },
    { 0, 0 },
    { 0x76270a74, 77 },
    { 0, 0 },
    { 0xbc970a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2870a7a, 17 },
    { 0x8dd70a7b, 90 },
    { 0xeab70a7c, 81 },
    { 0x1b170a7a, 80 },
    { 0xa9970a7c, 36 },
    { 0xcff70a7f, 90 },
    { 0x88c70a7a, 43 },
    { 0, 0 },
    { 0xa8d70a82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce170a85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x62570a88, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d370a8e, 95 },
    { 0, 0 },
    { 0x20270a90, 90 },
    { 0x65170a91, 61 },
    { 0xb4870a92, 76 },
    { 0x2fb70a92, 88 },
    { 0x67f70a92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85670a9c, 75 },
    { 0, 0 },
    { 0xf670a9e, 98 },
    { 0x76270a9f, 34 },
    { 0xd1970a9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55670aa5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df70aa9, 33 },
    { 0, 0 },
    { 0x6e270aab, 98 },
    { 0x7a570aac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf70ab0, 33 },
    { 0x4b670ab1, 93 },
    { 0x8bd70ab2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c70ab6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e70abe, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b70ac3, 89 },
    { 0, 0 },
    { 0xa9070ac5, 90 },
    { 0x85a70ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d70ad3, 55 },
    { 0, 0 },
    { 0xfcb70ad5, 33 },
    { 0xba270ad6, 90 },
    { 0x52c70ad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa070adc, 43 },
    { 0, 0 },
    { 0x6c370ade, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69170ae4, 100 },
    { 0x53f70ae5, 90 },
    { 0x9a670ae6, 90 },
    { 0xbd270ae7, 87 },
    { 0x5ac70ae8, 33 },
    { 0x27d70ae9, 95 },
    { 0x90770ae6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c70aed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74670af1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d70af8, 76 },
    { 0xc2270af9, 98 },
    { 0, 0 },
    { 0x14670afb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7170afe, 61 },
    { 0x44d70aff, 76 },
    { 0, 0 },
    { 0xd570b01, 87 },
    { 0x15670b02, 90 },
    { 0xdd70b01, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c70b06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe70b0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb470b0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d570b15, 75 },
    { 0xf5470b16, 23 },
    { 0x3c270b15, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6570b1c, 46 },
    { 0x22b70b1d, 33 },
    { 0x24c70b1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b70b22, 52 },
    { 0, 0 },
    { 0x91170b24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9170b28, 98 },
    { 0x35870b29, 84 },
    { 0xf2b70b2a, 52 },
    { 0, 0 },
    { 0x4bb70b2c, 75 },
    { 0x32270b2d, 68 },
    { 0x98170b2d, 33 },
    { 0x44470b2f, 90 },
    { 0x7e770b2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef70b38, 95 },
    { 0x3ac70b38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee70b3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf70b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1170b46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33370b4e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4070b52, 34 },
    { 0xb470b53, 95 },
    { 0xad170b52, 34 },
    { 0, 0 },
    { 0x90370b56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70970b5c, 91 },
    { 0x56b70b5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d70b64, 43 },
    { 0x6a270b65, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91170b69, 93 },
    { 0xab670b6a, 98 },
    { 0, 0 },
    { 0x9870b6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970b70, 43 },
    { 0x17d70b71, 33 },
    { 0xcbb70b71, 77 },
    { 0x14d70b73, 52 },
    { 0x10470b71, 93 },
    { 0, 0 },
    { 0x78f70b76, 67 },
    { 0xea970b77, 95 },
    { 0x9e070b77, 61 },
    { 0x7170b79, 78 },
    { 0, 0 },
    { 0x28c70b7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc70b80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0470b83, 60 },
    { 0, 0 },
    { 0x99c70b85, 87 },
    { 0x8170b86, 91 },
    { 0x61370b85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40170b8e, 61 },
    { 0x80b70b8e, 90 },
    { 0x1e270b90, 17 },
    { 0x2b70b8e, 61 },
    { 0x56670b92, 99 },
    { 0, 0 },
    { 0xcfd70b94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73070b9b, 66 },
    { 0xee270b9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d70ba0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d70ba9, 93 },
    { 0, 0 },
    { 0xae670bab, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f70bae, 33 },
    { 0xb2970bae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6470bb2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7470bb6, 90 },
    { 0, 0 },
    { 0xa0070bb8, 61 },
    { 0x91770bb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe970bbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71470bbf, 33 },
    { 0x38170bc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee70bc6, 61 },
    { 0x91c70bc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2870bca, 84 },
    { 0, 0 },
    { 0xbdd70bcc, 61 },
    { 0x29c70bcd, 90 },
    { 0x6b570bce, 87 },
    { 0xbd570bcd, 87 },
    { 0x83770bd0, 90 },
    { 0xb6070bd1, 95 },
    { 0, 0 },
    { 0xfc70bd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9370bd7, 90 },
    { 0, 0 },
    { 0x9d570bd9, 98 },
    { 0x7bb70bda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15970be0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x27470be3, 76 },
    { 0, 0 },
    { 0x7ab70be5, 98 },
    { 0xc1670be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e70be9, 61 },
    { 0, 0 },
    { 0xf5970beb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe770bef, 55 },
    { 0x17070bf0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c70bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0770bf6, 33 },
    { 0xcdd70bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd70bfa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f70bfd, 33 },
    { 0xab570bfe, 61 },
    { 0x5b570bff, 33 },
    { 0x5070c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2970c05, 23 },
    { 0x18470c06, 90 },
    { 0, 0 },
    { 0x3f170c08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75070c14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e70c17, 33 },
    { 0xc3470c18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30770c1d, 81 },
    { 0, 0 },
    { 0xe3070c1f, 95 },
    { 0x1d170c20, 43 },
    { 0, 0 },
    { 0x2b570c22, 52 },
    { 0x3a070c23, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47370c2d, 76 },
    { 0xaa170c2e, 95 },
    { 0, 0 },
    { 0xee670c30, 88 },
    { 0xe5f70c31, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x52170c34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63170c38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60870c47, 98 },
    { 0x7e570c48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1970c4b, 90 },
    { 0x70c4c, 95 },
    { 0xd6070c4c, 76 },
    { 0x3e370c4c, 100 },
    { 0x41b70c4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28970c55, 93 },
    { 0xcba70c56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45470c5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66370c60, 55 },
    { 0x7fa70c61, 23 },
    { 0xe3270c61, 90 },
    { 0x30870c63, 43 },
    { 0x50970c64, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5670c68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd70c6b, 55 },
    { 0x7bc70c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c70c71, 95 },
    { 0x71070c72, 90 },
    { 0x8370c73, 23 },
    { 0xb570c74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be70c7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b670c7d, 95 },
    { 0x46470c7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e70c85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed370c88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42870c8d, 99 },
    { 0x5e770c8e, 99 },
    { 0, 0 },
    { 0x65670c90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e770c94, 85 },
    { 0xa4670c95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5370c99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0070c9d, 33 },
    { 0, 0 },
    { 0xda870c9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f70ca2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b70ca5, 23 },
    { 0x93170ca6, 98 },
    { 0xb3170ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c70cb4, 66 },
    { 0x7f170cb4, 23 },
    { 0x3f970cb6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97070cba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e70cbd, 93 },
    { 0x23a70cbe, 17 },
    { 0, 0 },
    { 0xcd470cc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8770cc4, 95 },
    { 0x59870cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10570cc9, 67 },
    { 0x5ac70cc9, 33 },
    { 0x35a70cca, 98 },
    { 0, 0 },
    { 0x13c70ccd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x14070cd0, 23 },
    { 0x4b970cd1, 90 },
    { 0x45c70cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e70cdb, 43 },
    { 0xd4470cdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a70ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf70cee, 78 },
    { 0, 0 },
    { 0x24570cf0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a70cfa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f70d02, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5770d06, 87 },
    { 0x21870d07, 100 },
    { 0x7a670d06, 31 },
    { 0, 0 },
    { 0x5ee70d0a, 46 },
    { 0x57770d0b, 46 },
    { 0x36270d0a, 69 },
    { 0x54770d0d, 61 },
    { 0xa6570d0e, 98 },
    { 0x8e970d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6970d15, 52 },
    { 0xb8670d16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7470d19, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e70d24, 95 },
    { 0xc6d70d25, 33 },
    { 0xa0270d26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0370d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce70d2f, 95 },
    { 0x19370d30, 33 },
    { 0x39d70d30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17370d37, 98 },
    { 0x8fd70d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3570d3d, 46 },
    { 0xe870d3e, 55 },
    { 0x26b70d3f, 76 },
    { 0x3da70d40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e570d44, 36 },
    { 0x4d170d44, 33 },
    { 0x8d70d44, 23 },
    { 0xf3670d47, 52 },
    { 0, 0 },
    { 0x2bd70d49, 33 },
    { 0xfc870d4a, 33 },
    { 0xbea70d4b, 85 },
    { 0xc7170d49, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x88870d4f, 23 },
    { 0x89870d50, 76 },
    { 0x4f770d50, 23 },
    { 0, 0 },
    { 0x96a70d53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3570d58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37670d5b, 68 },
    { 0x11870d5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad570d63, 33 },
    { 0, 0 },
    { 0xace70d65, 34 },
    { 0xa8370d66, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9770d6c, 55 },
    { 0x4a870d6d, 95 },
    { 0xd4770d6d, 40 },
    { 0, 0 },
    { 0xfd570d70, 38 },
    { 0x76570d70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec70d77, 61 },
    { 0, 0 },
    { 0x23a70d79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda70d7c, 61 },
    { 0, 0 },
    { 0x9b70d7e, 61 },
    { 0, 0 },
    { 0x5dc70d80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16470d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d70d88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c70d8d, 90 },
    { 0, 0 },
    { 0xb5770d8f, 85 },
    { 0xf8570d90, 76 },
    { 0xfd170d90, 98 },
    { 0xd1470d92, 85 },
    { 0xcce70d93, 46 },
    { 0x83170d94, 95 },
    { 0, 0 },
    { 0xf4870d96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea70da2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2070da5, 90 },
    { 0x71470da5, 93 },
    { 0, 0 },
    { 0xc5c70da8, 52 },
    { 0xc0570da8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76770db3, 98 },
    { 0x4be70db4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e970db8, 33 },
    { 0, 0 },
    { 0xcb770dba, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4370dbf, 98 },
    { 0, 0 },
    { 0x44370dc1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6370dc4, 90 },
    { 0x34e70dc5, 90 },
    { 0xa1770dc6, 95 },
    { 0xfce70dc7, 65 },
    { 0xaa170dc4, 93 },
    { 0x8d070dc9, 90 },
    { 0xc9370dca, 61 },
    { 0x3f270dca, 61 },
    { 0x21270dc9, 33 },
    { 0xf6f70dcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b670dd2, 23 },
    { 0x30670dd2, 76 },
    { 0xc8170dd3, 95 },
    { 0x5fd70dd5, 91 },
    { 0x19070dd4, 55 },
    { 0x89a70dd7, 84 },
    { 0xec670dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e70ddb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24370de5, 95 },
    { 0, 0 },
    { 0x12c70de7, 90 },
    { 0x45170de7, 34 },
    { 0x7fa70de9, 95 },
    { 0x3fb70de9, 33 },
    { 0, 0 },
    { 0xd1670dec, 68 },
    { 0x29870ded, 33 },
    { 0xf3270dec, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51170df5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f670df8, 83 },
    { 0x34270df9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1670dff, 33 },
    { 0x7a370dff, 55 },
    { 0, 0 },
    { 0x1c170e02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f970e06, 17 },
    { 0x9670e06, 33 },
    { 0xb8770e08, 61 },
    { 0, 0 },
    { 0xbc870e0a, 90 },
    { 0, 0 },
    { 0xac070e0c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb70e13, 34 },
    { 0xeca70e14, 61 },
    { 0, 0 },
    { 0xbd570e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee470e19, 84 },
    { 0x36370e1a, 65 },
    { 0x83770e1a, 43 },
    { 0xd4a70e1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f870e25, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3470e2a, 98 },
    { 0xf6770e2a, 95 },
    { 0x9ba70e2c, 90 },
    { 0xe4670e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9570e35, 61 },
    { 0x9c170e36, 90 },
    { 0, 0 },
    { 0xd0e70e38, 61 },
    { 0x8c270e39, 98 },
    { 0xac170e39, 95 },
    { 0x45f70e3b, 61 },
    { 0x26c70e3c, 23 },
    { 0x62370e3d, 90 },
    { 0x13970e3d, 43 },
    { 0x64170e3e, 33 },
    { 0x50970e40, 90 },
    { 0xfcf70e40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52070e47, 24 },
    { 0xeb170e48, 95 },
    { 0, 0 },
    { 0x1d370e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c70e54, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x21970e57, 95 },
    { 0, 0 },
    { 0xa9670e59, 95 },
    { 0x5c470e59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2070e5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf570e62, 61 },
    { 0x87070e63, 78 },
    { 0, 0 },
    { 0xb1b70e65, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a70e6a, 98 },
    { 0, 0 },
    { 0xeff70e6c, 99 },
    { 0xc8e70e6d, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a770e76, 90 },
    { 0, 0 },
    { 0x8d370e78, 91 },
    { 0xb70e79, 23 },
    { 0xaf570e7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6970e7e, 23 },
    { 0xcfe70e7f, 23 },
    { 0, 0 },
    { 0x8ba70e81, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11070e88, 87 },
    { 0, 0 },
    { 0x6d070e8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf370e95, 67 },
    { 0xdd970e96, 34 },
    { 0x8f770e96, 33 },
    { 0, 0 },
    { 0x2e470e99, 61 },
    { 0xf1570e9a, 98 },
    { 0, 0 },
    { 0x45070e9c, 46 },
    { 0x40a70e9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19070ea1, 61 },
    { 0xdd570ea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd670eaa, 90 },
    { 0x9ff70eab, 88 },
    { 0xf2d70eaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16670eb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48770ebb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c770ebe, 52 },
    { 0, 0 },
    { 0xe3e70ec0, 33 },
    { 0x53e70ec1, 17 },
    { 0x26f70ec2, 55 },
    { 0x7b170ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe170ec6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1670eca, 95 },
    { 0xe3e70ecb, 55 },
    { 0xaf70ecc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23270ed1, 90 },
    { 0x14970ed2, 88 },
    { 0xc3470ed1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf470ed6, 76 },
    { 0, 0 },
    { 0xd1e70ed8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd70edc, 98 },
    { 0, 0 },
    { 0x99e70ede, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c270ee3, 33 },
    { 0x47c70ee4, 33 },
    { 0xc7970ee5, 98 },
    { 0xa7a70ee6, 67 },
    { 0x63470ee3, 76 },
    { 0x1e970ee8, 34 },
    { 0xdd770ee6, 98 },
    { 0, 0 },
    { 0x6a370eeb, 80 },
    { 0, 0 },
    { 0x69d70eed, 87 },
    { 0x1a70eee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7670ef1, 46 },
    { 0xb2470ef2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c70ef6, 23 },
    { 0x75c70ef6, 78 },
    { 0x47070ef8, 76 },
    { 0, 0 },
    { 0xbb670efa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe70efd, 33 },
    { 0xa1e70efe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f70f05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7570f09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1470f0f, 23 },
    { 0, 0 },
    { 0xaac70f11, 34 },
    { 0, 0 },
    { 0xdc870f13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f670f18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c470f1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1270f1f, 61 },
    { 0x95170f1f, 90 },
    { 0xa1270f21, 100 },
    { 0, 0 },
    { 0xd3770f23, 61 },
    { 0xb9770f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e70f2a, 34 },
    { 0xdf070f2b, 98 },
    { 0x41470f2c, 52 },
    { 0, 0 },
    { 0xa9e70f2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d870f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a70f35, 90 },
    { 0xb4370f36, 66 },
    { 0x14a70f37, 68 },
    { 0x99070f38, 61 },
    { 0, 0 },
    { 0xb570f3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52570f3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c70f47, 90 },
    { 0, 0 },
    { 0x12870f49, 98 },
    { 0, 0 },
    { 0xf4e70f4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf970f53, 61 },
    { 0xd8f70f54, 77 },
    { 0x96c70f55, 33 },
    { 0xc3770f54, 88 },
    { 0xe5270f53, 99 },
    { 0xbb670f58, 33 },
    { 0x98c70f58, 90 },
    { 0, 0 },
    { 0xea870f5b, 85 },
    { 0, 0 },
    { 0x4ca70f5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15570f63, 98 },
    { 0xed170f64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0770f69, 47 },
    { 0x45c70f6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff70f70, 90 },
    { 0, 0 },
    { 0xa5870f72, 99 },
    { 0x9a470f72, 76 },
    { 0x47670f74, 67 },
    { 0x2cc70f75, 90 },
    { 0x47c70f72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45370f7f, 66 },
    { 0xaa370f80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f070f86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6470f8a, 90 },
    { 0xb3570f8a, 55 },
    { 0xf0670f8c, 61 },
    { 0x9a470f8d, 34 },
    { 0x68270f8e, 55 },
    { 0xeb370f8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x98770f92, 100 },
    { 0, 0 },
    { 0x72970f94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2170f97, 34 },
    { 0xd6070f97, 55 },
    { 0x3f670f99, 98 },
    { 0, 0 },
    { 0xeaa70f9b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x55070f9e, 43 },
    { 0x3cf70f9f, 61 },
    { 0xbba70fa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20270fa9, 85 },
    { 0x45870fa9, 90 },
    { 0, 0 },
    { 0xc2a70fac, 67 },
    { 0, 0 },
    { 0x34170fae, 61 },
    { 0x15770fae, 55 },
    { 0, 0 },
    { 0xdb70fb1, 17 },
    { 0x47570fb2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd170fb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b70fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b70fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77870fc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf870fce, 90 },
    { 0xd8a70fce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0670fd2, 23 },
    { 0xd0370fd3, 68 },
    { 0x41a70fd4, 88 },
    { 0xc7a70fd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4570fd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b70fdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb870fe1, 88 },
    { 0, 0 },
    { 0x94b70fe3, 90 },
    { 0x50470fe4, 95 },
    { 0x84a70fe5, 87 },
    { 0, 0 },
    { 0x5a70fe7, 46 },
    { 0xd7370fe8, 85 },
    { 0xda270fe8, 84 },
    { 0xab170fe9, 33 },
    { 0x6e470fe9, 55 },
    { 0x1d770fec, 33 },
    { 0x85570fec, 33 },
    { 0, 0 },
    { 0x76070fef, 46 },
    { 0, 0 },
    { 0xda070ff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f970ffc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a71000, 90 },
    { 0, 0 },
    { 0x74e71002, 66 },
    { 0, 0 },
    { 0x4ad71004, 43 },
    { 0xcdc71005, 95 },
    { 0x65e71006, 23 },
    { 0x16071007, 66 },
    { 0, 0 },
    { 0xa9171009, 90 },
    { 0x26d7100a, 33 },
    { 0x76371009, 40 },
    { 0, 0 },
    { 0xda47100d, 95 },
    { 0x17f7100e, 93 },
    { 0xc927100f, 93 },
    { 0x6577100e, 55 },
    { 0xd5271010, 33 },
    { 0xf3a71012, 66 },
    { 0xdf071013, 61 },
    { 0x84771014, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a871017, 76 },
    { 0x51371018, 52 },
    { 0xd0671018, 76 },
    { 0, 0 },
    { 0x3a07101b, 61 },
    { 0xd867101c, 55 },
    { 0, 0 },
    { 0x4097101e, 33 },
    { 0, 0 },
    { 0xc1771020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b71023, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec7102a, 93 },
    { 0xc8c7102a, 95 },
    { 0, 0 },
    { 0x2ae7102d, 78 },
    { 0xcc27102d, 67 },
    { 0x9a7102d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd071034, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed071038, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a37103b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3417103e, 69 },
    { 0x857103f, 33 },
    { 0xd3e7103f, 100 },
    { 0, 0 },
    { 0x73471042, 95 },
    { 0xba71043, 23 },
    { 0x24771042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de71047, 23 },
    { 0xdcf71048, 90 },
    { 0x2c71048, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3471051, 61 },
    { 0, 0 },
    { 0x10f71053, 47 },
    { 0x61271054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37571058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7105c, 90 },
    { 0xa0a7105d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33671062, 33 },
    { 0xfa171062, 23 },
    { 0, 0 },
    { 0x9c771065, 90 },
    { 0x6a371065, 33 },
    { 0x5c571067, 55 },
    { 0x5b671066, 55 },
    { 0xc1271068, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf7106d, 98 },
    { 0x69d7106e, 34 },
    { 0xdf17106e, 90 },
    { 0x3ba71070, 34 },
    { 0xe1271070, 23 },
    { 0, 0 },
    { 0xb5d71073, 23 },
    { 0xf9a71073, 87 },
    { 0, 0 },
    { 0x65171076, 100 },
    { 0x9f571076, 95 },
    { 0x94b71078, 18 },
    { 0x2c871076, 61 },
    { 0x6227107a, 67 },
    { 0x287107b, 100 },
    { 0xa757107b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8557107f, 17 },
    { 0x37a71080, 84 },
    { 0x4cf71081, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5671086, 87 },
    { 0xc4671086, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb857108b, 33 },
    { 0x2427108c, 93 },
    { 0, 0 },
    { 0x3a57108e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7671092, 68 },
    { 0x5c071092, 23 },
    { 0x83371094, 33 },
    { 0x56471095, 98 },
    { 0, 0 },
    { 0xab71097, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf37109a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217109e, 43 },
    { 0x31a7109e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63710a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2710aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1710ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb710b2, 88 },
    { 0, 0 },
    { 0x9aa710b4, 90 },
    { 0x5fa710b5, 76 },
    { 0x97e710b5, 100 },
    { 0xf82710b5, 90 },
    { 0, 0 },
    { 0x2c1710b9, 61 },
    { 0x90e710ba, 23 },
    { 0xbf6710bb, 55 },
    { 0x3c3710bb, 66 },
    { 0xaff710ba, 100 },
    { 0x309710bc, 90 },
    { 0x78e710bf, 98 },
    { 0xade710bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673710c5, 55 },
    { 0x856710c6, 33 },
    { 0, 0 },
    { 0x101710c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a710cc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98710d0, 76 },
    { 0, 0 },
    { 0xe09710d2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2710d6, 23 },
    { 0, 0 },
    { 0x690710d8, 95 },
    { 0, 0 },
    { 0xa9710da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9710dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6710e4, 90 },
    { 0x37c710e5, 99 },
    { 0x725710e6, 33 },
    { 0xb10710e7, 36 },
    { 0x48c710e8, 34 },
    { 0x281710e9, 23 },
    { 0, 0 },
    { 0x2d4710eb, 91 },
    { 0xe1710ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f710f3, 90 },
    { 0x73e710f4, 99 },
    { 0x883710f4, 61 },
    { 0x2be710f6, 43 },
    { 0xf2b710f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970710fb, 33 },
    { 0, 0 },
    { 0xe9a710fd, 98 },
    { 0x8ba710fd, 33 },
    { 0x7ac710ff, 87 },
    { 0x7f671100, 85 },
    { 0x33071100, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70371104, 33 },
    { 0, 0 },
    { 0x80d71106, 97 },
    { 0x43e71107, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff7110a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1171110, 23 },
    { 0, 0 },
    { 0xbd171112, 90 },
    { 0x40871112, 53 },
    { 0xc8b71114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed671117, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb17111a, 61 },
    { 0, 0 },
    { 0xd427111c, 90 },
    { 0x3e67111d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a671120, 61 },
    { 0, 0 },
    { 0x8e871122, 90 },
    { 0x9e371122, 61 },
    { 0x75e71123, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc71127, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c7112b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba271133, 61 },
    { 0xfd371134, 98 },
    { 0xb8071135, 61 },
    { 0, 0 },
    { 0xb8371137, 52 },
    { 0, 0 },
    { 0x84571139, 88 },
    { 0x69871139, 61 },
    { 0xa777113a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf237113e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed171145, 90 },
    { 0x44571146, 43 },
    { 0x45b71147, 33 },
    { 0x31471148, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e67114c, 23 },
    { 0xde47114c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd971156, 95 },
    { 0xeeb71157, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b07115a, 90 },
    { 0xf947115b, 33 },
    { 0xdc97115c, 33 },
    { 0, 0 },
    { 0xdcb7115e, 97 },
    { 0x55a7115f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f771167, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc867116c, 33 },
    { 0x7bb7116d, 95 },
    { 0, 0 },
    { 0x4fe7116f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89571173, 95 },
    { 0x3b271174, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d71177, 61 },
    { 0x90071178, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b7117f, 46 },
    { 0, 0 },
    { 0xacf71181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc971188, 98 },
    { 0xb9071189, 54 },
    { 0x9e87118a, 95 },
    { 0xbb77118b, 33 },
    { 0x75d71188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb071190, 17 },
    { 0xc4071191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b7119e, 23 },
    { 0xa8f7119f, 95 },
    { 0, 0 },
    { 0x421711a1, 61 },
    { 0xef2711a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x131711a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799711ae, 90 },
    { 0x777711af, 77 },
    { 0x7b5711b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee711b3, 93 },
    { 0xfc0711b4, 81 },
    { 0, 0 },
    { 0xdba711b6, 90 },
    { 0xd6b711b7, 95 },
    { 0xb1a711b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b711bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1711c3, 76 },
    { 0x362711c4, 95 },
    { 0, 0 },
    { 0x4c9711c6, 99 },
    { 0, 0 },
    { 0xa94711c8, 33 },
    { 0x724711c9, 33 },
    { 0xec1711ca, 61 },
    { 0x1ad711cb, 33 },
    { 0, 0 },
    { 0xd3e711cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32711d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b711d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190711dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29711e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb711e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c711e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862711eb, 78 },
    { 0xc34711ec, 33 },
    { 0x3d7711ec, 43 },
    { 0x5d8711eb, 24 },
    { 0xeda711ee, 23 },
    { 0xf08711f0, 55 },
    { 0xace711f1, 93 },
    { 0xd26711f1, 90 },
    { 0xf12711f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1711f8, 61 },
    { 0xa7c711f9, 91 },
    { 0x85b711fa, 95 },
    { 0, 0 },
    { 0x63711fc, 98 },
    { 0x697711fd, 98 },
    { 0xfa1711fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38471203, 78 },
    { 0xc0f71204, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b71207, 23 },
    { 0xf1571208, 46 },
    { 0xa1d71209, 53 },
    { 0x70471209, 55 },
    { 0x4157120a, 87 },
    { 0xce07120c, 33 },
    { 0xd9d7120d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe771213, 33 },
    { 0xdd471213, 98 },
    { 0xf8671215, 23 },
    { 0x35971214, 23 },
    { 0xa7071217, 98 },
    { 0xa1571218, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77121b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9471221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7571224, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x82771227, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3747122d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f7123a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa587123d, 95 },
    { 0x7657123d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd371241, 61 },
    { 0, 0 },
    { 0x8b071243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6171248, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf987124c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a571250, 76 },
    { 0xe3d71251, 95 },
    { 0x26771252, 52 },
    { 0x63671253, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd71257, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc517125a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c7125d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b71267, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe727126c, 98 },
    { 0xfd17126c, 61 },
    { 0x807126d, 98 },
    { 0, 0 },
    { 0xb1d71270, 76 },
    { 0, 0 },
    { 0x20071272, 33 },
    { 0xecb71273, 98 },
    { 0xfeb71274, 76 },
    { 0xba071275, 93 },
    { 0xed471274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9371279, 81 },
    { 0xc5f7127a, 91 },
    { 0x9247127b, 23 },
    { 0xc7c7127c, 55 },
    { 0x75b7127a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c571281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52771287, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc637128b, 90 },
    { 0x74a7128b, 90 },
    { 0, 0 },
    { 0xc777128e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9371291, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f7129a, 90 },
    { 0x8c97129b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f712a1, 81 },
    { 0, 0 },
    { 0xad1712a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae712a7, 95 },
    { 0x412712a8, 99 },
    { 0x39b712a8, 23 },
    { 0xdeb712aa, 76 },
    { 0xeec712ab, 76 },
    { 0xd27712a7, 95 },
    { 0xe3712ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321712b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d712b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14712bb, 76 },
    { 0x194712bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97712c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6712cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde712d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91712d8, 77 },
    { 0x552712d8, 23 },
    { 0xfa7712da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234712de, 85 },
    { 0x21712df, 33 },
    { 0x89f712e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28712e3, 23 },
    { 0, 0 },
    { 0xfd5712e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee712eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751712f0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38712f3, 76 },
    { 0xa76712f4, 95 },
    { 0xcc2712f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2712f8, 33 },
    { 0x4f6712f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4712ff, 95 },
    { 0x82071300, 76 },
    { 0, 0 },
    { 0x7ee71302, 93 },
    { 0x85071303, 34 },
    { 0, 0 },
    { 0x3b371305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf71309, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c671310, 93 },
    { 0xc1e71311, 33 },
    { 0xb1671312, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7871318, 95 },
    { 0xa6f71318, 33 },
    { 0, 0 },
    { 0x7337131b, 98 },
    { 0x6a77131c, 46 },
    { 0, 0 },
    { 0x1ff7131e, 61 },
    { 0x2597131f, 90 },
    { 0, 0 },
    { 0x1fc71321, 55 },
    { 0x72971321, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4871327, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c67132a, 100 },
    { 0xbf67132a, 100 },
    { 0xce57132c, 40 },
    { 0x3327132c, 43 },
    { 0xb87132d, 43 },
    { 0xc1a7132d, 76 },
    { 0xc067132d, 93 },
    { 0xf727132f, 76 },
    { 0x81071332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3617133a, 65 },
    { 0xca47133b, 95 },
    { 0x8c07133c, 67 },
    { 0x54f7133b, 34 },
    { 0x3267133e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc971347, 23 },
    { 0x19f71348, 33 },
    { 0, 0 },
    { 0x9a47134a, 65 },
    { 0x9687134a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8371350, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe71357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9357135a, 77 },
    { 0x47c7135b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee71361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32071369, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7136e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d71375, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7457137c, 98 },
    { 0x7457137d, 61 },
    { 0x3947137e, 61 },
    { 0xcb77137f, 46 },
    { 0xeaf7137c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c971384, 64 },
    { 0x29e71385, 96 },
    { 0x11071386, 33 },
    { 0x7fc71386, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5627138c, 90 },
    { 0xdab7138c, 78 },
    { 0x6c7138e, 87 },
    { 0x5147138e, 52 },
    { 0, 0 },
    { 0x1b271391, 98 },
    { 0, 0 },
    { 0xf4271393, 33 },
    { 0xdb471394, 95 },
    { 0, 0 },
    { 0x29c71396, 25 },
    { 0, 0 },
    { 0x16871398, 61 },
    { 0x3d571398, 55 },
    { 0, 0 },
    { 0x83f7139b, 67 },
    { 0xc217139c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96713a2, 91 },
    { 0x344713a3, 96 },
    { 0, 0 },
    { 0xc49713a5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b713aa, 33 },
    { 0xca713ab, 93 },
    { 0x693713ac, 90 },
    { 0x873713aa, 23 },
    { 0x807713ae, 34 },
    { 0xad713af, 33 },
    { 0x82e713ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6713b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x537713ba, 43 },
    { 0xe4a713ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x181713be, 100 },
    { 0, 0 },
    { 0xc73713c0, 66 },
    { 0xfb0713c1, 90 },
    { 0, 0 },
    { 0x74c713c3, 61 },
    { 0x385713c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df713ca, 98 },
    { 0x7b7713cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793713cf, 23 },
    { 0, 0 },
    { 0x382713d1, 90 },
    { 0x432713d2, 61 },
    { 0xb95713d3, 76 },
    { 0xaf6713d3, 93 },
    { 0x487713d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3713d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b713db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a713e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b713e9, 33 },
    { 0xbd4713e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3713ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d713f1, 61 },
    { 0xcec713f1, 76 },
    { 0xf46713f3, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4713f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc713fa, 46 },
    { 0, 0 },
    { 0xead713fc, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4713ff, 23 },
    { 0x2cf71400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46071403, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc71406, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6071409, 43 },
    { 0x7017140a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74371410, 43 },
    { 0x8d471411, 34 },
    { 0x83f71412, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80871416, 33 },
    { 0, 0 },
    { 0x11971418, 23 },
    { 0xd2d71418, 52 },
    { 0, 0 },
    { 0xe647141b, 55 },
    { 0xb517141b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d7141f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe171424, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85871428, 66 },
    { 0xac771429, 78 },
    { 0x27d7142a, 33 },
    { 0x5437142b, 43 },
    { 0, 0 },
    { 0xce87142d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce71430, 33 },
    { 0x6af71431, 53 },
    { 0x4b971432, 34 },
    { 0, 0 },
    { 0xc8371434, 33 },
    { 0xb0d71435, 55 },
    { 0, 0 },
    { 0xbde71437, 33 },
    { 0, 0 },
    { 0xb4a71439, 66 },
    { 0x3d37143a, 17 },
    { 0x83a7143b, 55 },
    { 0xd947143c, 90 },
    { 0xbbe7143d, 98 },
    { 0xe2c7143e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1667144d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd71450, 80 },
    { 0x8d571451, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb137145a, 61 },
    { 0xe137145a, 98 },
    { 0x10b7145c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c17145f, 33 },
    { 0x4a571460, 61 },
    { 0xbde7145f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x93271464, 98 },
    { 0x99771465, 21 },
    { 0x6371466, 84 },
    { 0xf0371467, 100 },
    { 0xbc771468, 76 },
    { 0xd3771468, 77 },
    { 0x1c71467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b371472, 90 },
    { 0xbb471472, 66 },
    { 0xebc71473, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c471477, 95 },
    { 0x84f71478, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e07147b, 90 },
    { 0, 0 },
    { 0xd727147d, 33 },
    { 0x20a7147e, 90 },
    { 0xe7b7147e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18071484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92371489, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1147148c, 34 },
    { 0x92c7148c, 95 },
    { 0xcc47148c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4571492, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc71496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb7149b, 88 },
    { 0xd237149b, 23 },
    { 0xe747149d, 61 },
    { 0x73c7149e, 23 },
    { 0xd5e7149f, 90 },
    { 0, 0 },
    { 0xea3714a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f714a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a714ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1714b4, 76 },
    { 0xa64714b5, 76 },
    { 0x9d4714b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f714bd, 93 },
    { 0, 0 },
    { 0xbd3714bf, 95 },
    { 0, 0 },
    { 0x527714c1, 95 },
    { 0, 0 },
    { 0xcba714c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e714c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d714cb, 46 },
    { 0xbf2714cc, 23 },
    { 0x895714cd, 55 },
    { 0xa1a714ce, 99 },
    { 0xff1714cf, 98 },
    { 0, 0 },
    { 0xdbc714d1, 99 },
    { 0, 0 },
    { 0xe6b714d3, 97 },
    { 0x6c6714d4, 23 },
    { 0xb1c714d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986714d9, 52 },
    { 0, 0 },
    { 0xd19714db, 33 },
    { 0x38d714dc, 52 },
    { 0, 0 },
    { 0x124714de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155714e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854714e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b714ea, 17 },
    { 0x2ef714eb, 77 },
    { 0x96f714eb, 90 },
    { 0xc85714ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201714f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f714f4, 90 },
    { 0x832714f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a714f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa714fc, 23 },
    { 0xbb5714fd, 61 },
    { 0, 0 },
    { 0xcbe714ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b771506, 76 },
    { 0x2bb71507, 64 },
    { 0xaa371508, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5977150c, 90 },
    { 0xa787150d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xece71510, 61 },
    { 0x4f71511, 90 },
    { 0, 0 },
    { 0xf7a71513, 98 },
    { 0xdbb71513, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4777151d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a71520, 99 },
    { 0xdf671520, 90 },
    { 0x1c071520, 23 },
    { 0x57271521, 100 },
    { 0x81371520, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c671529, 23 },
    { 0x5d871529, 100 },
    { 0x28a7152b, 100 },
    { 0x7f471529, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a71530, 23 },
    { 0xa6871531, 76 },
    { 0x29e71532, 75 },
    { 0x85071532, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0x8971536, 84 },
    { 0xc771537, 95 },
    { 0, 0 },
    { 0x40c71539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31071540, 98 },
    { 0xdb871540, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b571547, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e7154b, 90 },
    { 0x597154c, 33 },
    { 0x2da7154d, 55 },
    { 0, 0 },
    { 0xdbc7154f, 90 },
    { 0, 0 },
    { 0xd171551, 90 },
    { 0xf2d71551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b971556, 94 },
    { 0, 0 },
    { 0xf7a71558, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7277155b, 59 },
    { 0x61c7155b, 55 },
    { 0, 0 },
    { 0x47f7155e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e171562, 95 },
    { 0xb2771562, 98 },
    { 0, 0 },
    { 0xad671565, 61 },
    { 0xd4771565, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe907156a, 85 },
    { 0xe057156b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf97156f, 95 },
    { 0x4871570, 76 },
    { 0x23c7156f, 17 },
    { 0, 0 },
    { 0x49a71573, 68 },
    { 0x3b271574, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a17157a, 90 },
    { 0x8d17157b, 43 },
    { 0x1c37157c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86371583, 33 },
    { 0x72171584, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb37158b, 84 },
    { 0xa767158c, 43 },
    { 0xed7158c, 87 },
    { 0xd327158e, 61 },
    { 0x1bd7158e, 66 },
    { 0xa71590, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb771593, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e71599, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8077159c, 61 },
    { 0x7c07159d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f715a0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d715a3, 95 },
    { 0, 0 },
    { 0x46e715a5, 68 },
    { 0x3e0715a6, 23 },
    { 0x75f715a6, 55 },
    { 0x931715a8, 98 },
    { 0xb1e715a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7715af, 98 },
    { 0, 0 },
    { 0xf7d715b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342715b5, 33 },
    { 0x486715b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845715bc, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x64715bf, 17 },
    { 0x8bd715c0, 85 },
    { 0, 0 },
    { 0x5da715c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35715c6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449715ca, 98 },
    { 0x9a6715cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x718715ce, 84 },
    { 0x4f4715cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb715d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3715d7, 90 },
    { 0, 0 },
    { 0x180715d9, 33 },
    { 0x15d715da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8715e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246715e6, 69 },
    { 0x73a715e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x714715ea, 90 },
    { 0x8ee715eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82715f1, 88 },
    { 0, 0 },
    { 0xf15715f3, 99 },
    { 0xc7715f4, 33 },
    { 0x5ba715f5, 78 },
    { 0x726715f6, 34 },
    { 0xb79715f4, 93 },
    { 0x2f715f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f715fb, 98 },
    { 0x291715fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad715ff, 98 },
    { 0x51e71600, 61 },
    { 0xef571601, 95 },
    { 0x18071602, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71605, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d571608, 90 },
    { 0, 0 },
    { 0x11b7160a, 99 },
    { 0x13d7160a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c7160e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c071611, 33 },
    { 0x17071612, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d271615, 90 },
    { 0xc6271615, 46 },
    { 0x11f71617, 33 },
    { 0xf7671617, 95 },
    { 0x74e71619, 34 },
    { 0, 0 },
    { 0xd1f7161b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f7161e, 90 },
    { 0x6307161f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce71623, 23 },
    { 0x36271624, 66 },
    { 0xc3971624, 98 },
    { 0x93671626, 95 },
    { 0xf6f71626, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7162a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8647162d, 90 },
    { 0, 0 },
    { 0xc697162f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b471639, 55 },
    { 0xde77163a, 81 },
    { 0xa857163b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4597163f, 77 },
    { 0xeaa71640, 61 },
    { 0x72471641, 23 },
    { 0x87d71640, 83 },
    { 0, 0 },
    { 0x4b771644, 75 },
    { 0, 0 },
    { 0x1a71646, 61 },
    { 0, 0 },
    { 0x12971648, 81 },
    { 0x8e071648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d47164c, 80 },
    { 0x8f67164d, 91 },
    { 0x2037164d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e71653, 76 },
    { 0xbda71654, 61 },
    { 0, 0 },
    { 0x25e71656, 33 },
    { 0xf2271657, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a47165a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d671665, 33 },
    { 0, 0 },
    { 0xe0171667, 88 },
    { 0, 0 },
    { 0x5a371669, 95 },
    { 0x517166a, 33 },
    { 0x16c7166a, 55 },
    { 0x71c7166c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d7167b, 95 },
    { 0x4047167c, 33 },
    { 0, 0 },
    { 0xd7f7167e, 95 },
    { 0x12d7167f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb071685, 38 },
    { 0xf5771686, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4647168a, 76 },
    { 0x64c7168b, 95 },
    { 0x38f7168c, 90 },
    { 0x18b7168d, 91 },
    { 0xe6e7168b, 98 },
    { 0xe167168f, 78 },
    { 0xf4171690, 90 },
    { 0xf227168a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1171694, 33 },
    { 0x5f171695, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57169d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584716a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a716a5, 98 },
    { 0x9bf716a6, 55 },
    { 0, 0 },
    { 0x5d3716a8, 23 },
    { 0xd77716a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x463716ac, 98 },
    { 0xa01716ad, 61 },
    { 0x774716ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda716b6, 87 },
    { 0xc3a716b7, 33 },
    { 0x736716b7, 95 },
    { 0xfa2716b7, 100 },
    { 0, 0 },
    { 0xa40716bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x791716be, 75 },
    { 0xa52716bf, 90 },
    { 0xd74716c0, 52 },
    { 0, 0 },
    { 0x444716c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da716c6, 90 },
    { 0x44b716c7, 46 },
    { 0, 0 },
    { 0x924716c9, 78 },
    { 0x583716ca, 72 },
    { 0, 0 },
    { 0xeba716cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82716d2, 33 },
    { 0xb2f716d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b716d7, 65 },
    { 0x65b716d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78716dd, 90 },
    { 0, 0 },
    { 0x74b716df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2716e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50716ee, 76 },
    { 0xe09716ee, 87 },
    { 0, 0 },
    { 0x72e716f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b716f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e716f8, 93 },
    { 0x125716f8, 95 },
    { 0, 0 },
    { 0xdf3716fb, 65 },
    { 0, 0 },
    { 0x795716fd, 61 },
    { 0, 0 },
    { 0x8d3716ff, 33 },
    { 0, 0 },
    { 0xb4a71701, 95 },
    { 0, 0 },
    { 0x7a171703, 92 },
    { 0xc7c71704, 91 },
    { 0x85171704, 34 },
    { 0xcbc71706, 61 },
    { 0x65771704, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e87170a, 33 },
    { 0xbaa7170b, 33 },
    { 0xcda7170c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a771711, 77 },
    { 0xa3b71711, 34 },
    { 0, 0 },
    { 0x8a971714, 75 },
    { 0, 0 },
    { 0x6c471716, 33 },
    { 0x9a371717, 78 },
    { 0x52571717, 95 },
    { 0x3aa71717, 87 },
    { 0xe957171a, 90 },
    { 0xad071719, 23 },
    { 0xfb97171c, 76 },
    { 0x6547171d, 76 },
    { 0, 0 },
    { 0xeea7171f, 61 },
    { 0xd071720, 88 },
    { 0xae471720, 33 },
    { 0x32271722, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae371725, 90 },
    { 0x38471726, 90 },
    { 0xa1871726, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d7172d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13771730, 52 },
    { 0, 0 },
    { 0x44971732, 39 },
    { 0, 0 },
    { 0x32671734, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e37173a, 90 },
    { 0x91c7173b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc867173e, 90 },
    { 0x5567173f, 23 },
    { 0, 0 },
    { 0xd0a71741, 33 },
    { 0xc4371741, 66 },
    { 0, 0 },
    { 0x4e071744, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7174f, 83 },
    { 0, 0 },
    { 0xf9e71751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d71754, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc71759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c7175c, 97 },
    { 0xc407175c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57371760, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3771763, 90 },
    { 0x91071763, 33 },
    { 0xd5b71765, 55 },
    { 0x8771766, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6271769, 61 },
    { 0xc4e71769, 33 },
    { 0xf5771769, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98871771, 90 },
    { 0xa0271772, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e71775, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb7177f, 33 },
    { 0, 0 },
    { 0x9bc71781, 61 },
    { 0xce271782, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1271788, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb7178b, 76 },
    { 0, 0 },
    { 0xecc7178d, 61 },
    { 0xcec7178e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf971799, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17179d, 90 },
    { 0, 0 },
    { 0xd957179f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b717a2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46717a7, 84 },
    { 0, 0 },
    { 0xe2b717a9, 34 },
    { 0xdf5717a9, 98 },
    { 0, 0 },
    { 0xb3c717ac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d717b3, 43 },
    { 0x4d9717b4, 61 },
    { 0xdb7717b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26717b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94717bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d717c1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e717c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785717ce, 78 },
    { 0xd96717cf, 75 },
    { 0x54e717d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76717d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92717e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7717e3, 23 },
    { 0x64f717e3, 61 },
    { 0x50717e5, 33 },
    { 0xc58717e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc717ec, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960717f0, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b717f6, 55 },
    { 0, 0 },
    { 0x95f717f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48771800, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3071804, 98 },
    { 0, 0 },
    { 0x3c371806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07180d, 61 },
    { 0xbd77180e, 98 },
    { 0x3cf7180f, 90 },
    { 0x2e97180e, 95 },
    { 0x94f7180f, 98 },
    { 0x86671812, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b571819, 65 },
    { 0xd8b7181a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63371821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50671824, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0171829, 95 },
    { 0xa1c7182a, 82 },
    { 0xaef7182b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c7182e, 84 },
    { 0, 0 },
    { 0xb0871830, 99 },
    { 0, 0 },
    { 0xbe071832, 98 },
    { 0, 0 },
    { 0x68871834, 90 },
    { 0x43d71835, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f7183f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a71845, 55 },
    { 0x4b271846, 90 },
    { 0x5c871847, 33 },
    { 0x16a71847, 76 },
    { 0xf8471849, 91 },
    { 0x1947184a, 78 },
    { 0x2d57184b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7184e, 78 },
    { 0x1517184e, 61 },
    { 0, 0 },
    { 0xe3471851, 52 },
    { 0xcfa71851, 40 },
    { 0, 0 },
    { 0xd3771854, 33 },
    { 0xebd71855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb87185c, 85 },
    { 0xbc87185d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf571860, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63671866, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce71869, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa7186c, 68 },
    { 0, 0 },
    { 0xdd37186e, 90 },
    { 0, 0 },
    { 0xdfa71870, 76 },
    { 0x1f971870, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde71877, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff47187b, 61 },
    { 0x86e7187c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7187f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db71888, 66 },
    { 0xeed71889, 84 },
    { 0xec271889, 90 },
    { 0xf1271888, 95 },
    { 0xfe7188c, 80 },
    { 0x6497188d, 23 },
    { 0xaf97188e, 61 },
    { 0xddd7188e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86271893, 81 },
    { 0x3c971893, 98 },
    { 0x18471893, 76 },
    { 0xad771896, 98 },
    { 0x3f671895, 25 },
    { 0xdb171898, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe637189b, 67 },
    { 0x95c7189c, 90 },
    { 0x57d7189d, 98 },
    { 0xc867189e, 95 },
    { 0, 0 },
    { 0xce2718a0, 65 },
    { 0x858718a1, 46 },
    { 0x69718a0, 95 },
    { 0x8bb718a3, 91 },
    { 0x6f3718a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x162718a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8718ab, 67 },
    { 0x9a718ac, 90 },
    { 0x6a4718ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7718b0, 46 },
    { 0x862718b1, 34 },
    { 0, 0 },
    { 0x9fc718b3, 61 },
    { 0x275718b3, 100 },
    { 0x799718b3, 100 },
    { 0x2d0718b6, 33 },
    { 0x664718b5, 67 },
    { 0xa6e718b8, 77 },
    { 0x7da718b3, 76 },
    { 0, 0 },
    { 0xb1f718bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356718c5, 95 },
    { 0xe7a718c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76718ca, 34 },
    { 0x434718ca, 81 },
    { 0x68a718cc, 76 },
    { 0x1e8718cd, 43 },
    { 0x9f9718ce, 23 },
    { 0xded718cb, 33 },
    { 0x37d718d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec718d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a718da, 33 },
    { 0x4bf718db, 33 },
    { 0x287718da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6718e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92718ea, 66 },
    { 0x3b8718eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0718f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5718f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe718f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0718fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0718ff, 52 },
    { 0x5f571900, 33 },
    { 0, 0 },
    { 0x4d771902, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a271908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb497191a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c7191d, 34 },
    { 0, 0 },
    { 0x9ee7191f, 77 },
    { 0xb3d7191f, 81 },
    { 0, 0 },
    { 0x77b71922, 43 },
    { 0x82071923, 90 },
    { 0xa6d71924, 61 },
    { 0x4a371924, 90 },
    { 0xf9871926, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a971929, 61 },
    { 0xa2b7192a, 33 },
    { 0, 0 },
    { 0x7ed7192c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96971930, 61 },
    { 0x4d771931, 85 },
    { 0, 0 },
    { 0xa6a71933, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30571937, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb7193a, 77 },
    { 0x8467193b, 33 },
    { 0x3a27193c, 89 },
    { 0xd3f7193a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd571944, 33 },
    { 0xe9871945, 33 },
    { 0xc1271946, 33 },
    { 0xbcc71946, 98 },
    { 0x59071948, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7194b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d71957, 100 },
    { 0x1a371958, 99 },
    { 0x93371959, 61 },
    { 0x4617195a, 76 },
    { 0x7337195b, 61 },
    { 0x9227195a, 34 },
    { 0xb47195d, 33 },
    { 0xa797195e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79871961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62071964, 78 },
    { 0, 0 },
    { 0x49d71966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f71969, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a57196c, 90 },
    { 0xd417196d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d71971, 90 },
    { 0, 0 },
    { 0x7ee71973, 65 },
    { 0x87c71973, 23 },
    { 0, 0 },
    { 0x27971976, 90 },
    { 0xfa771976, 76 },
    { 0xe4071978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f7197c, 98 },
    { 0xf427197d, 95 },
    { 0, 0 },
    { 0x8fb7197f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1471984, 33 },
    { 0x7d971984, 61 },
    { 0, 0 },
    { 0xd571987, 46 },
    { 0, 0 },
    { 0x75071989, 99 },
    { 0xbf17198a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7198d, 98 },
    { 0xc847198e, 98 },
    { 0xd7d7198f, 98 },
    { 0xf4571990, 78 },
    { 0x89371991, 55 },
    { 0x47d71992, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c71996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c87199d, 33 },
    { 0x9de7199e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc719a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475719a7, 90 },
    { 0xe75719a7, 90 },
    { 0x564719a8, 33 },
    { 0, 0 },
    { 0xad719ab, 33 },
    { 0, 0 },
    { 0x5cb719ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x602719b0, 90 },
    { 0x97c719b1, 98 },
    { 0x3c5719b2, 23 },
    { 0x75d719b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577719b9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6719bd, 23 },
    { 0x297719be, 93 },
    { 0xb1a719bf, 88 },
    { 0, 0 },
    { 0x9ec719c1, 61 },
    { 0x64f719c2, 78 },
    { 0xff719c3, 93 },
    { 0, 0 },
    { 0x189719c5, 55 },
    { 0, 0 },
    { 0x633719c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a719ca, 98 },
    { 0xe64719cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a719d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88719d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e719d9, 52 },
    { 0, 0 },
    { 0x4ed719db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875719e1, 61 },
    { 0, 0 },
    { 0x250719e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b719e7, 87 },
    { 0xd57719e8, 65 },
    { 0, 0 },
    { 0xd1d719ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877719ef, 100 },
    { 0x77e719f0, 23 },
    { 0, 0 },
    { 0xa0f719f2, 77 },
    { 0x912719f3, 98 },
    { 0xf9f719f2, 55 },
    { 0, 0 },
    { 0x9ab719f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae719fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a671a07, 98 },
    { 0x6f871a07, 23 },
    { 0x7f71a08, 61 },
    { 0, 0 },
    { 0x15d71a0b, 23 },
    { 0, 0 },
    { 0x3df71a0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3471a11, 76 },
    { 0xd6871a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7071a16, 95 },
    { 0, 0 },
    { 0x9d271a18, 84 },
    { 0, 0 },
    { 0xc0a71a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5371a1d, 93 },
    { 0x18771a1e, 17 },
    { 0x33071a1d, 95 },
    { 0, 0 },
    { 0xa9b71a21, 98 },
    { 0xb2971a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50771a26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb71a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed71a2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe971a31, 98 },
    { 0xfaf71a32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d771a37, 33 },
    { 0, 0 },
    { 0x39a71a39, 90 },
    { 0, 0 },
    { 0x53971a3b, 67 },
    { 0x60471a3c, 55 },
    { 0xc1971a3d, 33 },
    { 0x4f471a3e, 23 },
    { 0x2fe71a3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a771a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d71a48, 61 },
    { 0x2f571a49, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43071a50, 99 },
    { 0, 0 },
    { 0xc2471a52, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x78371a55, 98 },
    { 0xd9071a56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54371a59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b471a5c, 91 },
    { 0x20371a5d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b71a63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3871a67, 80 },
    { 0x58471a68, 33 },
    { 0xad971a69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0071a6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc71a71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x44271a74, 90 },
    { 0x66771a75, 99 },
    { 0x1a971a75, 33 },
    { 0x17171a76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36871a7a, 76 },
    { 0x6af71a7b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc871a7e, 98 },
    { 0, 0 },
    { 0xf2d71a80, 100 },
    { 0xb2e71a80, 84 },
    { 0xb6371a81, 76 },
    { 0x55871a83, 90 },
    { 0xa3a71a83, 66 },
    { 0x60471a85, 90 },
    { 0x62c71a86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48871a8a, 61 },
    { 0xcfe71a8a, 90 },
    { 0x21471a8c, 61 },
    { 0x66c71a8d, 66 },
    { 0xc9871a8d, 99 },
    { 0xc1d71a8e, 84 },
    { 0xbcf71a90, 23 },
    { 0x1db71a91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd71a97, 75 },
    { 0x48371a98, 43 },
    { 0, 0 },
    { 0x28371a9a, 95 },
    { 0x21d71a9b, 33 },
    { 0x3cc71a9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd71a9f, 13 },
    { 0x9671aa0, 23 },
    { 0xb571a9f, 90 },
    { 0, 0 },
    { 0xc9071aa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4571aa7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7771aac, 61 },
    { 0, 0 },
    { 0x20271aae, 98 },
    { 0xf3771aae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b471ab4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b871ab7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18571abd, 33 },
    { 0x9cf71abd, 98 },
    { 0, 0 },
    { 0x9c871ac0, 33 },
    { 0xf0771ac0, 90 },
    { 0x5c571ac2, 93 },
    { 0x5d571ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde771ac9, 98 },
    { 0x5eb71ac9, 55 },
    { 0xc0571aca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d71acf, 84 },
    { 0xe6971acf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f471ad6, 81 },
    { 0x9071ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a71ada, 95 },
    { 0x64771adb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7071ade, 68 },
    { 0x2ec71adf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde371ae2, 61 },
    { 0x60271ae2, 98 },
    { 0xb5f71ae4, 55 },
    { 0xe7071ae4, 75 },
    { 0x99371ae5, 61 },
    { 0x8e71ae6, 85 },
    { 0xd971ae8, 90 },
    { 0x6fb71ae8, 33 },
    { 0x4de71ae6, 33 },
    { 0x65571aeb, 91 },
    { 0xa6771ae4, 91 },
    { 0xc9771ae8, 95 },
    { 0, 0 },
    { 0xd2671aef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea71af4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c71afa, 95 },
    { 0xf7871afa, 99 },
    { 0, 0 },
    { 0xced71afd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd71b01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed671b05, 34 },
    { 0x3af71b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a371b09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4671b0c, 98 },
    { 0xa4b71b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75771b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee71b13, 78 },
    { 0, 0 },
    { 0x93c71b15, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f771b19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c271b1e, 95 },
    { 0, 0 },
    { 0x3b71b20, 75 },
    { 0x79771b21, 98 },
    { 0x4a271b22, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c71b26, 95 },
    { 0x67771b27, 23 },
    { 0, 0 },
    { 0xe7d71b29, 66 },
    { 0xf2a71b29, 61 },
    { 0xa7b71b2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda71b34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe671b38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea71b3b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec71b3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f71b41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a71b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3271b4e, 33 },
    { 0, 0 },
    { 0x2ba71b50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1671b55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d71b5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x61671b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe971b63, 34 },
    { 0, 0 },
    { 0x7a471b65, 88 },
    { 0xbe371b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc571b6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b071b72, 55 },
    { 0x9ea71b73, 23 },
    { 0x90571b73, 90 },
    { 0x1dc71b73, 23 },
    { 0x6cd71b75, 23 },
    { 0xfd971b77, 100 },
    { 0x8f371b78, 61 },
    { 0xa9971b76, 17 },
    { 0x7b371b7a, 66 },
    { 0xdca71b7b, 76 },
    { 0x26871b7c, 90 },
    { 0xec471b7d, 33 },
    { 0, 0 },
    { 0x4c271b7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f71b82, 33 },
    { 0, 0 },
    { 0x4c771b84, 43 },
    { 0, 0 },
    { 0xeb871b86, 95 },
    { 0xa2c71b87, 99 },
    { 0x52971b88, 61 },
    { 0xa6871b89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a71b8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea971b93, 98 },
    { 0, 0 },
    { 0x21171b95, 78 },
    { 0xd2971b96, 90 },
    { 0, 0 },
    { 0xb7271b98, 93 },
    { 0xd2771b99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99471b9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f771ba2, 95 },
    { 0, 0 },
    { 0x33471ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0471ba8, 61 },
    { 0xf8671ba9, 33 },
    { 0xf9271ba8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3771bae, 17 },
    { 0, 0 },
    { 0xfce71bb0, 23 },
    { 0x14471bb0, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd71bb5, 98 },
    { 0, 0 },
    { 0x36971bb7, 90 },
    { 0x82c71bb8, 33 },
    { 0x74a71bb8, 33 },
    { 0xecc71bba, 90 },
    { 0xd3771bba, 98 },
    { 0xf2c71bbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b471bbf, 95 },
    { 0xa8a71bbf, 84 },
    { 0, 0 },
    { 0xfa971bc2, 23 },
    { 0xbb671bc3, 78 },
    { 0x28271bc2, 90 },
    { 0x70a71bc5, 84 },
    { 0xb9371bc4, 90 },
    { 0, 0 },
    { 0x61371bc8, 61 },
    { 0, 0 },
    { 0x17771bca, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf371bd0, 55 },
    { 0xa7c71bd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e71bd4, 55 },
    { 0x52771bd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3371bda, 95 },
    { 0xd5d71bdb, 61 },
    { 0x11a71bdc, 90 },
    { 0, 0 },
    { 0xf6b71bde, 46 },
    { 0x40271bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d571be3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3271be9, 33 },
    { 0x50871bea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49571bf1, 33 },
    { 0xd4971bf2, 67 },
    { 0xddf71bf3, 98 },
    { 0, 0 },
    { 0x20d71bf5, 33 },
    { 0x3ba71bf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78671bf9, 76 },
    { 0x88571bfa, 23 },
    { 0x71871bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed371c06, 61 },
    { 0x6ae71c06, 95 },
    { 0, 0 },
    { 0xd5471c09, 55 },
    { 0xb8471c0a, 76 },
    { 0xd4b71c0b, 68 },
    { 0x1d771c0a, 52 },
    { 0x60a71c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c71c12, 77 },
    { 0xea371c13, 95 },
    { 0xea971c14, 33 },
    { 0xbed71c15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b871c1b, 95 },
    { 0xe9771c1c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x78271c1f, 90 },
    { 0, 0 },
    { 0x13871c21, 90 },
    { 0xb7b71c22, 68 },
    { 0x9ea71c23, 95 },
    { 0, 0 },
    { 0x58571c25, 55 },
    { 0x68871c26, 46 },
    { 0x25d71c26, 55 },
    { 0, 0 },
    { 0xab171c29, 76 },
    { 0x89871c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96571c2d, 61 },
    { 0x27971c2e, 61 },
    { 0x33771c2e, 77 },
    { 0x91071c2d, 33 },
    { 0xd5971c30, 33 },
    { 0x90d71c32, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37071c38, 52 },
    { 0x93671c39, 90 },
    { 0x7cc71c3a, 91 },
    { 0xb6b71c3b, 61 },
    { 0x45671c3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb471c48, 23 },
    { 0, 0 },
    { 0x8ba71c4a, 33 },
    { 0x54171c4b, 93 },
    { 0xaa471c4a, 61 },
    { 0, 0 },
    { 0x53971c4e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e671c58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21271c5c, 33 },
    { 0x7c671c5d, 66 },
    { 0xf1d71c5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f871c62, 34 },
    { 0, 0 },
    { 0xb0871c64, 61 },
    { 0x18b71c65, 93 },
    { 0x43e71c65, 75 },
    { 0x13071c64, 66 },
    { 0, 0 },
    { 0x69c71c69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e71c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f71c71, 33 },
    { 0xf8d71c72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8571c75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f171c7c, 90 },
    { 0x4671c7d, 23 },
    { 0xda071c7e, 88 },
    { 0xc4c71c7c, 88 },
    { 0x34471c7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c71c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89171c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf671c8d, 61 },
    { 0x1371c8e, 78 },
    { 0, 0 },
    { 0x22f71c90, 99 },
    { 0x1af71c90, 90 },
    { 0, 0 },
    { 0x6ed71c93, 23 },
    { 0x16571c94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2671c97, 33 },
    { 0xca871c98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89771ca0, 98 },
    { 0x72a71ca0, 68 },
    { 0x86d71ca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47071ca8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87871cac, 23 },
    { 0x271cac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a571cb2, 98 },
    { 0xaf071cb2, 43 },
    { 0xd0971cb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa71cb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce71cbd, 98 },
    { 0x94071cbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17671cc1, 61 },
    { 0xf1271cc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a371cc6, 99 },
    { 0x70471cc7, 34 },
    { 0x58071cc7, 67 },
    { 0xe1671cc8, 33 },
    { 0x77b71cca, 23 },
    { 0xf8371cc7, 33 },
    { 0x30d71cc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e71cd3, 65 },
    { 0x6e471cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34471cda, 78 },
    { 0xc7f71cda, 36 },
    { 0xd8071cdb, 23 },
    { 0xdb871cdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28371ce2, 90 },
    { 0xdf371ce2, 33 },
    { 0x88371ce4, 76 },
    { 0, 0 },
    { 0x59871ce6, 33 },
    { 0xc6871ce7, 66 },
    { 0x16671ce8, 61 },
    { 0xa4171ce6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca71ced, 95 },
    { 0x90e71ced, 23 },
    { 0, 0 },
    { 0xc8e71cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e371cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3471cf7, 46 },
    { 0xf8971cf8, 95 },
    { 0xff471cf7, 55 },
    { 0x12471cfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5271cff, 98 },
    { 0x67d71d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a71d06, 33 },
    { 0, 0 },
    { 0xb3571d08, 43 },
    { 0xa0571d09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0371d0f, 88 },
    { 0, 0 },
    { 0xe8271d11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf571d16, 99 },
    { 0, 0 },
    { 0x56271d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa071d1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f271d24, 84 },
    { 0x9bc71d24, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab071d2b, 66 },
    { 0xe9571d2c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a71d2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d071d32, 90 },
    { 0xb71d33, 55 },
    { 0x4e771d33, 33 },
    { 0xbe471d35, 99 },
    { 0xad671d35, 76 },
    { 0x3cb71d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85671d3d, 61 },
    { 0xa471d3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17271d41, 94 },
    { 0, 0 },
    { 0xfb871d43, 33 },
    { 0xe1371d43, 33 },
    { 0x80071d45, 93 },
    { 0, 0 },
    { 0x3f571d47, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e71d50, 91 },
    { 0xfc871d50, 93 },
    { 0, 0 },
    { 0xa4771d53, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0571d57, 33 },
    { 0x33e71d58, 33 },
    { 0xfd071d59, 98 },
    { 0, 0 },
    { 0xe0871d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd171d5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd371d62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x19471d65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd71d6a, 90 },
    { 0x43171d6a, 65 },
    { 0x58571d6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f71d7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac271d7d, 88 },
    { 0x9b271d7e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5271d81, 33 },
    { 0, 0 },
    { 0x14971d83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c871d87, 61 },
    { 0, 0 },
    { 0x49371d89, 98 },
    { 0xd4071d8a, 76 },
    { 0, 0 },
    { 0x89171d8c, 93 },
    { 0x95f71d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63771d90, 33 },
    { 0x4f071d91, 56 },
    { 0x7cd71d91, 37 },
    { 0x24a71d93, 87 },
    { 0xbc471d94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9671d9e, 99 },
    { 0x13371d9f, 52 },
    { 0xe8f71d9e, 33 },
    { 0xe6371da0, 90 },
    { 0xd2671da2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea571da5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e571da9, 52 },
    { 0xd4371da9, 43 },
    { 0xea371daa, 33 },
    { 0x10371da9, 33 },
    { 0, 0 },
    { 0x46271dae, 78 },
    { 0, 0 },
    { 0xac371db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0571db3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a71db7, 61 },
    { 0, 0 },
    { 0x74d71db9, 61 },
    { 0x28771dba, 56 },
    { 0x81171dbb, 23 },
    { 0x81f71dbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f71dc3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd71dc6, 61 },
    { 0xfda71dc7, 87 },
    { 0x971dc8, 23 },
    { 0xfc671dc9, 64 },
    { 0x3df71dc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f671dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b71dd4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7771dde, 61 },
    { 0, 0 },
    { 0xce671de0, 61 },
    { 0x6f571de1, 55 },
    { 0, 0 },
    { 0x1da71de3, 61 },
    { 0, 0 },
    { 0x36871de5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f71deb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e71dee, 90 },
    { 0, 0 },
    { 0xf5771df0, 33 },
    { 0, 0 },
    { 0x9e071df2, 98 },
    { 0xc071df2, 46 },
    { 0x51471df4, 61 },
    { 0x88b71df5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b71dfa, 91 },
    { 0x86b71dfb, 87 },
    { 0x89571dfc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa71e00, 98 },
    { 0x28571e00, 61 },
    { 0xea171e00, 98 },
    { 0x75371e00, 91 },
    { 0x4c71e00, 90 },
    { 0xbf971e05, 87 },
    { 0x98b71e02, 33 },
    { 0xdc671e04, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x21871e0a, 43 },
    { 0, 0 },
    { 0x1f371e0c, 95 },
    { 0, 0 },
    { 0xee971e0e, 75 },
    { 0xe7d71e0e, 8 },
    { 0x74471e0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba71e16, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9971e1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd871e21, 90 },
    { 0x97871e22, 99 },
    { 0, 0 },
    { 0x2f471e24, 23 },
    { 0xb7f71e25, 100 },
    { 0x39571e26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c71e2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a71e2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e671e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67371e35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e271e3b, 90 },
    { 0xf5471e3c, 91 },
    { 0x5c971e3c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a71e40, 90 },
    { 0, 0 },
    { 0x4b371e42, 17 },
    { 0x49f71e43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb71e48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63171e4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a71e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb71e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4471e5e, 93 },
    { 0x38f71e5f, 98 },
    { 0xc8e71e5e, 90 },
    { 0, 0 },
    { 0x24f71e62, 76 },
    { 0xe4e71e62, 91 },
    { 0xacc71e64, 34 },
    { 0xb1771e65, 33 },
    { 0xcfb71e63, 91 },
    { 0x2b171e67, 90 },
    { 0x14271e67, 33 },
    { 0x59d71e67, 34 },
    { 0, 0 },
    { 0x80671e6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d71e6e, 33 },
    { 0xc871e6f, 66 },
    { 0x87571e70, 90 },
    { 0x6b171e71, 33 },
    { 0x4c971e72, 61 },
    { 0xb9f71e73, 95 },
    { 0x91471e74, 34 },
    { 0, 0 },
    { 0xadb71e76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58171e7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76571e81, 98 },
    { 0xe0071e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed471e85, 33 },
    { 0x86a71e86, 98 },
    { 0x8a971e87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed71e8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a71e8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b71e9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba71e9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b71ea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d071ea8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b71ebf, 90 },
    { 0x3b871ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b71ec3, 23 },
    { 0xec771ec3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f71ec9, 33 },
    { 0xb2371eca, 55 },
    { 0xea471ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe971ece, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4171ed2, 90 },
    { 0x9f171ed2, 95 },
    { 0xb4671ed4, 46 },
    { 0, 0 },
    { 0x22271ed6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca171eda, 52 },
    { 0xad71edb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c71ede, 61 },
    { 0x67771edf, 23 },
    { 0x7bd71edf, 95 },
    { 0xa6b71edf, 43 },
    { 0, 0 },
    { 0xe3771ee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67271ee9, 33 },
    { 0xf1271ee9, 33 },
    { 0xe7d71eeb, 46 },
    { 0x21c71eeb, 61 },
    { 0xa7571eec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f171ef0, 98 },
    { 0xd9571ef1, 23 },
    { 0xe1571ef0, 55 },
    { 0xe0271ef3, 98 },
    { 0xfc071ef4, 34 },
    { 0x9d271ef3, 95 },
    { 0x93f71ef6, 75 },
    { 0x2771ef6, 19 },
    { 0x8f971ef8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f71efb, 44 },
    { 0xd0771efc, 91 },
    { 0x9f771efd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef71f00, 93 },
    { 0x91271f00, 98 },
    { 0, 0 },
    { 0x53271f03, 61 },
    { 0xe1b71f03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36571f09, 93 },
    { 0, 0 },
    { 0xad671f0b, 33 },
    { 0x3d71f0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b471f0f, 87 },
    { 0x1d771f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d471f13, 23 },
    { 0, 0 },
    { 0x41571f15, 46 },
    { 0x6ed71f15, 90 },
    { 0, 0 },
    { 0x1dd71f18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29971f20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x32871f23, 90 },
    { 0x19f71f24, 84 },
    { 0, 0 },
    { 0x7bd71f26, 78 },
    { 0, 0 },
    { 0xeab71f28, 90 },
    { 0xf9871f28, 23 },
    { 0xb9271f28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc71f2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f71f36, 23 },
    { 0x15471f36, 34 },
    { 0, 0 },
    { 0x4bb71f39, 33 },
    { 0x44f71f3a, 88 },
    { 0xdeb71f3a, 88 },
    { 0, 0 },
    { 0x62c71f3d, 61 },
    { 0x4d771f3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53571f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6871f4f, 76 },
    { 0xdfe71f50, 61 },
    { 0, 0 },
    { 0xa8571f52, 90 },
    { 0, 0 },
    { 0xb7171f54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef471f59, 33 },
    { 0x36d71f5a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3971f63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7771f66, 88 },
    { 0xa1371f66, 80 },
    { 0xac371f66, 87 },
    { 0, 0 },
    { 0x23a71f6a, 61 },
    { 0, 0 },
    { 0x33b71f6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9071f70, 90 },
    { 0, 0 },
    { 0x42a71f72, 61 },
    { 0x65d71f73, 98 },
    { 0x59c71f74, 95 },
    { 0x2e671f75, 33 },
    { 0x1bb71f75, 61 },
    { 0xf0671f77, 76 },
    { 0xee671f75, 67 },
    { 0, 0 },
    { 0x49071f7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b71f7e, 77 },
    { 0, 0 },
    { 0xd471f80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f871f83, 95 },
    { 0x7b871f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3871f8a, 33 },
    { 0x87e71f8b, 90 },
    { 0x4a571f8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64071f92, 61 },
    { 0xe671f93, 84 },
    { 0, 0 },
    { 0xa6071f95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7671f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce71f9b, 43 },
    { 0, 0 },
    { 0xd6871f9d, 91 },
    { 0x83371f9e, 66 },
    { 0xd4071f9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e571fa8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee371fad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d171fb0, 55 },
    { 0x4cb71fb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7471fb8, 91 },
    { 0x28071fb8, 61 },
    { 0x6cf71fb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d271fbe, 98 },
    { 0xfda71fbf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d271fc2, 90 },
    { 0xf1d71fc3, 95 },
    { 0x1c871fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac71fc7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef071fcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c71fd3, 90 },
    { 0x4d271fd4, 90 },
    { 0xb1671fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3371fdb, 91 },
    { 0, 0 },
    { 0x7e571fdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa971fe2, 90 },
    { 0xf7d71fe2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5971fe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a471feb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79471ff5, 52 },
    { 0x37371ff6, 91 },
    { 0xe2271ff7, 43 },
    { 0x35071ff8, 27 },
    { 0x6d471ff6, 23 },
    { 0xfbc71ffa, 33 },
    { 0xfc571ffb, 55 },
    { 0x7c671ffa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa671fff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32772003, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b72008, 90 },
    { 0xaa572009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b67200c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a72015, 98 },
    { 0xdd772016, 23 },
    { 0xcf572017, 23 },
    { 0x80472016, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d72022, 33 },
    { 0xb7772022, 88 },
    { 0x3cc72022, 90 },
    { 0xb1f72025, 55 },
    { 0xaf772023, 33 },
    { 0xf5372023, 55 },
    { 0, 0 },
    { 0x79672029, 23 },
    { 0, 0 },
    { 0x6fb7202b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57072031, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed72035, 55 },
    { 0xd3972036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e72039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d7203e, 80 },
    { 0, 0 },
    { 0x2aa72040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14272049, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b7204f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d37205b, 80 },
    { 0xa197205c, 95 },
    { 0x9017205d, 95 },
    { 0, 0 },
    { 0xd97205f, 98 },
    { 0x87d72060, 90 },
    { 0, 0 },
    { 0x75772062, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc172069, 99 },
    { 0xd5b72069, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3047206e, 61 },
    { 0, 0 },
    { 0xa9f72070, 61 },
    { 0, 0 },
    { 0x9fd72072, 76 },
    { 0, 0 },
    { 0x4e072074, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc7207b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d17207e, 61 },
    { 0xc4e7207e, 93 },
    { 0, 0 },
    { 0x38e72081, 91 },
    { 0x3d272081, 95 },
    { 0xa1a72081, 33 },
    { 0x2a372084, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x972087, 90 },
    { 0x76572088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297208b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6372091, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9172096, 66 },
    { 0xd4972096, 90 },
    { 0x23072098, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950720a4, 55 },
    { 0xa29720a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1720a8, 76 },
    { 0x73a720a9, 93 },
    { 0, 0 },
    { 0x9ab720ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4720b0, 77 },
    { 0x5a7720b1, 36 },
    { 0x7ff720b2, 61 },
    { 0x1ac720b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27720b8, 90 },
    { 0x88d720b9, 61 },
    { 0x236720ba, 46 },
    { 0x709720bb, 95 },
    { 0xe45720bc, 61 },
    { 0x1d7720b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c720c5, 33 },
    { 0x98a720c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x724720c9, 17 },
    { 0, 0 },
    { 0xdc4720cb, 90 },
    { 0xb8d720cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6720d9, 98 },
    { 0, 0 },
    { 0x70a720db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05720de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2720e1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810720e7, 42 },
    { 0, 0 },
    { 0x39b720e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4720ef, 23 },
    { 0, 0 },
    { 0x37b720f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8720f6, 33 },
    { 0x7ca720f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e720fd, 76 },
    { 0x57a720fd, 90 },
    { 0xc0d720fd, 95 },
    { 0xe7172100, 23 },
    { 0x8c720ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7972104, 78 },
    { 0x5da72105, 97 },
    { 0xd8872105, 95 },
    { 0x8b872107, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c7210b, 100 },
    { 0, 0 },
    { 0x85e7210d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e572111, 91 },
    { 0, 0 },
    { 0x19e72113, 95 },
    { 0x21672114, 55 },
    { 0xc3972114, 66 },
    { 0x9d72115, 47 },
    { 0xa7e72117, 87 },
    { 0x87572117, 33 },
    { 0x57872115, 98 },
    { 0xe947211a, 95 },
    { 0, 0 },
    { 0x53e7211c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff72122, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8872131, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a72137, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d67213a, 61 },
    { 0xcbc7213b, 99 },
    { 0, 0 },
    { 0x3df7213d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0272140, 87 },
    { 0x1be72141, 95 },
    { 0xb1472142, 76 },
    { 0xa9172142, 33 },
    { 0x6b172144, 77 },
    { 0x17972141, 90 },
    { 0, 0 },
    { 0xf672147, 46 },
    { 0xe5272148, 23 },
    { 0, 0 },
    { 0x8eb7214a, 34 },
    { 0x6de7214b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea7214e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e72155, 95 },
    { 0xd7072155, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce77215a, 43 },
    { 0, 0 },
    { 0x67d7215c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b72163, 98 },
    { 0xf2072164, 46 },
    { 0xe8772165, 77 },
    { 0xe4a72164, 90 },
    { 0x19c72167, 76 },
    { 0, 0 },
    { 0x84172169, 17 },
    { 0x1337216a, 91 },
    { 0x2af7216b, 34 },
    { 0x1ff7216b, 90 },
    { 0, 0 },
    { 0xbdf7216e, 90 },
    { 0x53e7216f, 78 },
    { 0xb8f7216f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0872173, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe7217a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a7217d, 90 },
    { 0x2fa7217e, 33 },
    { 0x7cf7217f, 98 },
    { 0, 0 },
    { 0x68072181, 76 },
    { 0, 0 },
    { 0xf5a72183, 33 },
    { 0x77672184, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f72187, 81 },
    { 0x14872188, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b7218d, 98 },
    { 0x8537218d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x872191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc72194, 33 },
    { 0, 0 },
    { 0x70772196, 90 },
    { 0, 0 },
    { 0x35772198, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc7219d, 33 },
    { 0xdb7219e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507721a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e721a8, 67 },
    { 0x861721a9, 81 },
    { 0x3e0721a9, 23 },
    { 0x574721ab, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb721ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459721b3, 23 },
    { 0xe47721b3, 33 },
    { 0x386721b5, 65 },
    { 0x3ca721b5, 52 },
    { 0x944721b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe721bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867721c1, 100 },
    { 0x73f721c2, 95 },
    { 0xfb9721c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736721ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89721cd, 98 },
    { 0, 0 },
    { 0xbcd721cf, 55 },
    { 0xadf721cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x211721d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773721d8, 90 },
    { 0x6bb721d9, 66 },
    { 0, 0 },
    { 0xfee721db, 55 },
    { 0, 0 },
    { 0xc69721dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539721e3, 90 },
    { 0, 0 },
    { 0xf1f721e5, 34 },
    { 0, 0 },
    { 0x4fe721e7, 34 },
    { 0xfde721e8, 78 },
    { 0x86f721e9, 77 },
    { 0x47f721e9, 34 },
    { 0x711721e9, 99 },
    { 0xc52721ec, 61 },
    { 0xfbe721ed, 34 },
    { 0, 0 },
    { 0xddc721ef, 61 },
    { 0xfc721f0, 76 },
    { 0x712721f1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x533721f4, 61 },
    { 0xac8721f5, 95 },
    { 0x7e6721f6, 61 },
    { 0x878721f5, 34 },
    { 0, 0 },
    { 0xe6a721f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d721fc, 33 },
    { 0xcab721fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce72201, 95 },
    { 0, 0 },
    { 0x2bc72203, 23 },
    { 0x4e72204, 38 },
    { 0xc5e72205, 85 },
    { 0x11f72206, 90 },
    { 0x11b72205, 23 },
    { 0xe0a72204, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa537220c, 55 },
    { 0x16c7220d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b72210, 23 },
    { 0xd6d72211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f72215, 23 },
    { 0, 0 },
    { 0x42b72217, 23 },
    { 0x73472218, 77 },
    { 0xce172218, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d77221c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad72220, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa72225, 43 },
    { 0x14472226, 85 },
    { 0x25472225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f27222a, 66 },
    { 0, 0 },
    { 0x31b7222c, 55 },
    { 0, 0 },
    { 0xbf07222e, 55 },
    { 0xadd7222f, 98 },
    { 0, 0 },
    { 0x75372231, 75 },
    { 0x20372232, 61 },
    { 0x89372231, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61272239, 89 },
    { 0, 0 },
    { 0xd017223b, 93 },
    { 0xe6a7223c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f7223f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e72248, 95 },
    { 0x84172249, 95 },
    { 0xe572249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c7224f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd872252, 93 },
    { 0, 0 },
    { 0xc1572254, 91 },
    { 0x30e72255, 90 },
    { 0, 0 },
    { 0xac972257, 95 },
    { 0xf6e72257, 99 },
    { 0xe872259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27225e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e72262, 55 },
    { 0, 0 },
    { 0x10872264, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c7226d, 98 },
    { 0xdab7226d, 90 },
    { 0x9447226d, 61 },
    { 0x1c172270, 23 },
    { 0xb1872271, 90 },
    { 0x8be72272, 78 },
    { 0xea07226d, 67 },
    { 0xea372274, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d77227a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc7227e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a72282, 49 },
    { 0x80372283, 33 },
    { 0x9d272284, 98 },
    { 0x26772285, 93 },
    { 0, 0 },
    { 0xf7572287, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9257228a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf947228f, 43 },
    { 0x2eb72290, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd72296, 95 },
    { 0, 0 },
    { 0x33272298, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757229b, 90 },
    { 0x5c97229c, 91 },
    { 0xecf7229c, 33 },
    { 0xf707229c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f722a6, 91 },
    { 0, 0 },
    { 0x4c7722a8, 98 },
    { 0xfcb722a8, 90 },
    { 0xe8a722aa, 84 },
    { 0xc46722ab, 61 },
    { 0x438722ac, 52 },
    { 0xf7c722ab, 95 },
    { 0x5a0722ad, 93 },
    { 0xcdb722af, 61 },
    { 0x687722b0, 77 },
    { 0, 0 },
    { 0x89722b2, 87 },
    { 0x659722b3, 33 },
    { 0x13b722b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7722b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65722bd, 90 },
    { 0, 0 },
    { 0x2c4722bf, 90 },
    { 0x2e5722c0, 95 },
    { 0, 0 },
    { 0xd30722c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c722cc, 98 },
    { 0xe65722cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c722d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2722d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376722da, 33 },
    { 0, 0 },
    { 0xacd722dc, 88 },
    { 0xbe6722dc, 34 },
    { 0xfbd722de, 98 },
    { 0x119722de, 23 },
    { 0, 0 },
    { 0x34a722e1, 87 },
    { 0x64f722e2, 52 },
    { 0x490722e2, 23 },
    { 0xc4b722e3, 61 },
    { 0xda722e5, 90 },
    { 0x371722e6, 33 },
    { 0xedc722e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x206722ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0722ed, 51 },
    { 0xe07722ee, 61 },
    { 0xc6722ee, 33 },
    { 0xcce722ee, 85 },
    { 0, 0 },
    { 0xb48722f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b722f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873722fe, 71 },
    { 0xafb722ff, 91 },
    { 0x2dc722ff, 33 },
    { 0xe5d722ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b72304, 23 },
    { 0x1bb72304, 33 },
    { 0xbc272306, 77 },
    { 0xb8472307, 95 },
    { 0xbda72308, 66 },
    { 0xd8072307, 90 },
    { 0xa2d7230a, 40 },
    { 0x5ea7230a, 98 },
    { 0, 0 },
    { 0x1037230d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa72311, 98 },
    { 0xb9572312, 85 },
    { 0, 0 },
    { 0x2072314, 90 },
    { 0x80f72314, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8672319, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f72324, 99 },
    { 0x46072325, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7407232a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8217232d, 98 },
    { 0, 0 },
    { 0xba87232f, 61 },
    { 0xe6f72330, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13572335, 33 },
    { 0x3d772336, 34 },
    { 0x3b72337, 95 },
    { 0xfde72335, 60 },
    { 0x5a572339, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2527233c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb57233f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c72345, 91 },
    { 0, 0 },
    { 0xf7d72347, 33 },
    { 0, 0 },
    { 0xeb872349, 52 },
    { 0, 0 },
    { 0xa967234b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe172351, 90 },
    { 0xcc472351, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73072356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd7235b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3447235f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c972367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a7236c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c372370, 90 },
    { 0x10c72371, 98 },
    { 0, 0 },
    { 0xf8e72373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3027237a, 23 },
    { 0, 0 },
    { 0xc3c7237c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de7237f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac72383, 61 },
    { 0, 0 },
    { 0xe6f72385, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b572389, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f27238f, 33 },
    { 0, 0 },
    { 0x1072391, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb372395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59772399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb797239d, 90 },
    { 0x1737239e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a723a1, 34 },
    { 0x54a723a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f723a5, 33 },
    { 0xc4e723a6, 90 },
    { 0x98b723a7, 98 },
    { 0, 0 },
    { 0x138723a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43723ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5723af, 98 },
    { 0x2b0723b0, 61 },
    { 0x381723b0, 61 },
    { 0, 0 },
    { 0xca6723b3, 33 },
    { 0, 0 },
    { 0x997723b5, 90 },
    { 0x4e9723b6, 90 },
    { 0xa06723b7, 46 },
    { 0x4b4723b8, 67 },
    { 0, 0 },
    { 0xff1723ba, 90 },
    { 0x9a723bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0723be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36723c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9723c5, 61 },
    { 0, 0 },
    { 0x1e3723c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37723ca, 61 },
    { 0x7c2723cb, 46 },
    { 0xaaa723cc, 93 },
    { 0xe33723ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce723d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3723d3, 34 },
    { 0xa9f723d4, 76 },
    { 0xa4f723d4, 98 },
    { 0x5bc723d5, 33 },
    { 0xdb1723d6, 76 },
    { 0x3e5723d8, 76 },
    { 0x934723d4, 55 },
    { 0xc21723da, 76 },
    { 0x8ee723db, 85 },
    { 0x799723dc, 98 },
    { 0xf8a723dd, 33 },
    { 0x67f723db, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474723e8, 38 },
    { 0, 0 },
    { 0x569723ea, 43 },
    { 0, 0 },
    { 0x40d723ec, 33 },
    { 0x6c0723ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73723f0, 95 },
    { 0, 0 },
    { 0xe5b723f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c723f6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93723fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba72400, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a72404, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x56472407, 98 },
    { 0x19572407, 55 },
    { 0x5e572408, 43 },
    { 0x1267240a, 91 },
    { 0x47f7240b, 98 },
    { 0, 0 },
    { 0x9997240d, 100 },
    { 0x4f47240e, 40 },
    { 0x2ca7240e, 66 },
    { 0x3a672410, 93 },
    { 0x5637240f, 61 },
    { 0x50472412, 76 },
    { 0x1c77240f, 33 },
    { 0xf1072414, 43 },
    { 0x3c72415, 78 },
    { 0x73372416, 23 },
    { 0, 0 },
    { 0x89872418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f07241c, 33 },
    { 0xaee7241c, 95 },
    { 0xcb97241e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8472421, 43 },
    { 0, 0 },
    { 0xc9172423, 96 },
    { 0xc4472424, 61 },
    { 0xd4272423, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f7242b, 95 },
    { 0xbd57242c, 90 },
    { 0, 0 },
    { 0x4a77242e, 92 },
    { 0x8f97242e, 34 },
    { 0x84a72430, 93 },
    { 0xe5472431, 43 },
    { 0xc9c72432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2087243a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c7243d, 68 },
    { 0, 0 },
    { 0x5c87243f, 34 },
    { 0x4a572440, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79772446, 23 },
    { 0x23272447, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7097244c, 61 },
    { 0x2767244d, 23 },
    { 0, 0 },
    { 0x5f07244f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6772453, 69 },
    { 0x22b72453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b672457, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x737245a, 95 },
    { 0x2b37245b, 95 },
    { 0xe7a7245c, 61 },
    { 0x6727245d, 67 },
    { 0xdbf7245e, 70 },
    { 0xae7245e, 93 },
    { 0x9b672460, 46 },
    { 0xcd37245c, 33 },
    { 0xa8572461, 84 },
    { 0x1f72461, 81 },
    { 0, 0 },
    { 0xb4d72465, 90 },
    { 0xaf272466, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56672469, 90 },
    { 0x9027246a, 43 },
    { 0x4d87246a, 91 },
    { 0x6757246c, 33 },
    { 0x10b7246c, 33 },
    { 0x8a87246b, 100 },
    { 0xcd37246f, 23 },
    { 0xeba7246b, 34 },
    { 0, 0 },
    { 0xe1d72472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b972476, 46 },
    { 0, 0 },
    { 0xc72478, 61 },
    { 0xa5f72479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6667247d, 99 },
    { 0x4767247e, 78 },
    { 0xcb47247d, 95 },
    { 0x69472480, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe72484, 43 },
    { 0, 0 },
    { 0xede72486, 23 },
    { 0xb6572487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3697248d, 43 },
    { 0x3ad7248e, 90 },
    { 0xb157248d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b72493, 61 },
    { 0, 0 },
    { 0x92272495, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a7249c, 98 },
    { 0xda77249c, 90 },
    { 0xa867249d, 34 },
    { 0xaad7249d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8724a3, 91 },
    { 0x8b8724a4, 98 },
    { 0, 0 },
    { 0x896724a6, 66 },
    { 0x5d0724a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9724af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47724b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70724b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae724bb, 61 },
    { 0, 0 },
    { 0x2d5724bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f724c0, 65 },
    { 0x28d724c1, 24 },
    { 0x957724c1, 33 },
    { 0xcdb724c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0724c8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5724cb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d724d6, 46 },
    { 0xc31724d7, 90 },
    { 0x4b7724d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4724dc, 90 },
    { 0xa36724dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a724e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34724ec, 90 },
    { 0x9d8724ed, 34 },
    { 0x940724ee, 23 },
    { 0, 0 },
    { 0x5a6724f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9724f9, 95 },
    { 0x6bd724fa, 75 },
    { 0xab0724fb, 46 },
    { 0x6c9724fb, 33 },
    { 0x218724fd, 88 },
    { 0xbac724fe, 91 },
    { 0xf86724ff, 43 },
    { 0x1cb724fa, 33 },
    { 0xeb872501, 81 },
    { 0xf5772502, 34 },
    { 0x4af72501, 95 },
    { 0x1ad72504, 75 },
    { 0xf8172505, 61 },
    { 0x7d272505, 61 },
    { 0x54c72507, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2427250a, 100 },
    { 0x727250a, 67 },
    { 0, 0 },
    { 0xc617250d, 95 },
    { 0, 0 },
    { 0x8c97250f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8972512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a872516, 98 },
    { 0, 0 },
    { 0x1db72518, 90 },
    { 0x35772519, 76 },
    { 0xda7251a, 23 },
    { 0xbd972518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977251f, 46 },
    { 0x2772520, 23 },
    { 0x37372520, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef72524, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x98472527, 98 },
    { 0, 0 },
    { 0xb6d72529, 76 },
    { 0x7787252a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe047252d, 95 },
    { 0x9907252e, 76 },
    { 0x6767252f, 55 },
    { 0x8f47252e, 67 },
    { 0x7e372530, 23 },
    { 0xa172530, 33 },
    { 0xc7472532, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a7253a, 4 },
    { 0xd5e7253b, 100 },
    { 0x87d7253c, 55 },
    { 0x9ba7253b, 33 },
    { 0xe057253d, 61 },
    { 0x38c7253f, 77 },
    { 0x3187253b, 61 },
    { 0, 0 },
    { 0xf3e72542, 76 },
    { 0x15c72542, 95 },
    { 0x43372542, 33 },
    { 0xc5e72542, 93 },
    { 0x25e72546, 17 },
    { 0x6a072547, 99 },
    { 0xa4c72546, 61 },
    { 0x20872549, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c7254c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9972554, 55 },
    { 0, 0 },
    { 0x6f172556, 66 },
    { 0xbad72556, 90 },
    { 0x2b572558, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d772565, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe372569, 76 },
    { 0, 0 },
    { 0x8937256b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c67256e, 23 },
    { 0, 0 },
    { 0x93272570, 95 },
    { 0x95c72570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb872575, 33 },
    { 0, 0 },
    { 0x85272577, 78 },
    { 0, 0 },
    { 0xb2b72579, 78 },
    { 0xf167257a, 90 },
    { 0x51872579, 61 },
    { 0, 0 },
    { 0xfb07257d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe72583, 90 },
    { 0x4fe72583, 23 },
    { 0x31772583, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b472588, 90 },
    { 0, 0 },
    { 0xddd7258a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87258e, 90 },
    { 0x6f87258f, 98 },
    { 0x9e7258f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e72597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107259b, 61 },
    { 0x7297259c, 90 },
    { 0x9fb7259d, 76 },
    { 0x6af7259c, 99 },
    { 0xe887259e, 66 },
    { 0x26e7259c, 90 },
    { 0xcaf7259e, 43 },
    { 0xc5e725a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933725a6, 75 },
    { 0xf02725a7, 78 },
    { 0xc53725a7, 95 },
    { 0x133725a9, 43 },
    { 0xba2725a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe725b5, 34 },
    { 0x4ac725b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f725bb, 98 },
    { 0, 0 },
    { 0x68a725bd, 96 },
    { 0xcef725be, 95 },
    { 0, 0 },
    { 0x44725c0, 90 },
    { 0, 0 },
    { 0x644725c2, 98 },
    { 0xf6d725c2, 90 },
    { 0, 0 },
    { 0x996725c5, 33 },
    { 0xd0d725c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69725cc, 76 },
    { 0xa74725cd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x277725d0, 17 },
    { 0x6e7725d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b725d7, 77 },
    { 0x799725d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x269725db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2725e0, 100 },
    { 0x7fe725e1, 98 },
    { 0xc55725e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42725e5, 90 },
    { 0x1e2725e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd725eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270725ee, 87 },
    { 0x6a4725ee, 90 },
    { 0xcd2725f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9725f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d725f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8725fb, 95 },
    { 0x923725fc, 85 },
    { 0xfd4725fd, 67 },
    { 0xca1725fe, 98 },
    { 0x3a5725ff, 55 },
    { 0x38472600, 33 },
    { 0x4f372601, 76 },
    { 0x68672600, 33 },
    { 0x6b172603, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f72606, 33 },
    { 0, 0 },
    { 0x50a72608, 90 },
    { 0x5c472609, 61 },
    { 0x5b57260a, 83 },
    { 0x24072608, 33 },
    { 0xed87260c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7557260f, 52 },
    { 0x34e7260f, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x24372613, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8172616, 76 },
    { 0xcc72617, 76 },
    { 0, 0 },
    { 0x87a72619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7637261c, 23 },
    { 0x7ed7261d, 100 },
    { 0x8407261d, 98 },
    { 0xcfb7261f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2472623, 90 },
    { 0x82572624, 91 },
    { 0x63472624, 98 },
    { 0xb6072623, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4872629, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b67262f, 52 },
    { 0xf2d7262f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2567263d, 33 },
    { 0x2387263d, 75 },
    { 0, 0 },
    { 0xedf72640, 85 },
    { 0xc8072641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c72647, 23 },
    { 0, 0 },
    { 0xd7d72649, 23 },
    { 0x1137264a, 99 },
    { 0xadc72649, 99 },
    { 0x22a7264c, 23 },
    { 0x4827264d, 77 },
    { 0x3a372649, 95 },
    { 0x53d7264d, 61 },
    { 0, 0 },
    { 0xff72651, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84872659, 61 },
    { 0x3c172659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a72663, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b72668, 95 },
    { 0xa7772669, 95 },
    { 0xfb472668, 33 },
    { 0, 0 },
    { 0x4207266c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd017266f, 23 },
    { 0xb8572670, 90 },
    { 0x84f72671, 91 },
    { 0x2a72670, 43 },
    { 0xd1672670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e672676, 91 },
    { 0, 0 },
    { 0x4ce72678, 34 },
    { 0xa9372679, 55 },
    { 0, 0 },
    { 0x1d67267b, 23 },
    { 0xe177267c, 61 },
    { 0xc407267b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb72680, 64 },
    { 0x92072681, 95 },
    { 0, 0 },
    { 0xd7472683, 61 },
    { 0x87872684, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa7268c, 33 },
    { 0, 0 },
    { 0x9ed7268e, 100 },
    { 0xaf47268e, 69 },
    { 0x1ca7268e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b7269a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4257269f, 95 },
    { 0xd04726a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x355726a3, 23 },
    { 0xc726a4, 100 },
    { 0xaa9726a5, 95 },
    { 0xfc7726a6, 81 },
    { 0xd5f726a5, 61 },
    { 0x61f726a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed726ab, 33 },
    { 0xd11726ab, 76 },
    { 0x2e0726ab, 90 },
    { 0x8ba726ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20726b5, 23 },
    { 0, 0 },
    { 0xd2d726b7, 76 },
    { 0x554726b8, 61 },
    { 0xa2726b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688726c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d726c4, 98 },
    { 0x802726c5, 99 },
    { 0x29d726c5, 61 },
    { 0x5fb726c6, 33 },
    { 0xb9f726c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3726d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81726d7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x573726da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c726dd, 46 },
    { 0, 0 },
    { 0x194726df, 33 },
    { 0x5d2726df, 90 },
    { 0, 0 },
    { 0x197726e2, 23 },
    { 0xdb0726e2, 33 },
    { 0x4cf726e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01726f3, 75 },
    { 0xba3726f4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438726f8, 95 },
    { 0, 0 },
    { 0x4ee726fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590726ff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26972703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8172708, 43 },
    { 0, 0 },
    { 0xd097270a, 33 },
    { 0xd507270a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d72711, 23 },
    { 0x66472712, 78 },
    { 0x7c672713, 90 },
    { 0x72f72712, 98 },
    { 0xb9772714, 85 },
    { 0x34872716, 23 },
    { 0x62072717, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5247271e, 84 },
    { 0xc627271f, 23 },
    { 0xcdc72720, 95 },
    { 0x6ec72721, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd72726, 23 },
    { 0xa1d72727, 23 },
    { 0, 0 },
    { 0x9eb72729, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf837272c, 33 },
    { 0x5837272d, 77 },
    { 0xf507272c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41972736, 23 },
    { 0, 0 },
    { 0x2dc72738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7577273c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c72746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe657274a, 90 },
    { 0x6f47274b, 33 },
    { 0x7ad7274c, 99 },
    { 0, 0 },
    { 0x8ad7274e, 55 },
    { 0x5597274f, 88 },
    { 0x7cc72750, 98 },
    { 0xf3772751, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a72755, 98 },
    { 0, 0 },
    { 0x58072757, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5767275c, 90 },
    { 0xd277275c, 33 },
    { 0, 0 },
    { 0xfe67275f, 98 },
    { 0, 0 },
    { 0xa272761, 23 },
    { 0xe8b72761, 78 },
    { 0x30972763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6572766, 95 },
    { 0, 0 },
    { 0x22e72768, 91 },
    { 0, 0 },
    { 0x70b7276a, 98 },
    { 0x8a37276b, 33 },
    { 0x66b7276c, 33 },
    { 0x68a7276d, 90 },
    { 0x39d7276d, 98 },
    { 0x20e7276d, 95 },
    { 0x41372770, 43 },
    { 0x46a7276e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed872778, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de7277c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4777277f, 33 },
    { 0x5d572780, 49 },
    { 0, 0 },
    { 0x2a672782, 90 },
    { 0xf9b72783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27972786, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b72789, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc187278c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2772791, 46 },
    { 0x8b372792, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7279a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47279f, 23 },
    { 0, 0 },
    { 0xff1727a1, 17 },
    { 0, 0 },
    { 0x93b727a3, 34 },
    { 0x2b1727a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d727a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619727ab, 33 },
    { 0, 0 },
    { 0x4a5727ad, 61 },
    { 0x680727ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e727b1, 77 },
    { 0xf1727b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d727b9, 77 },
    { 0, 0 },
    { 0x142727bb, 87 },
    { 0x87c727bc, 66 },
    { 0, 0 },
    { 0x9aa727be, 84 },
    { 0x7c1727bf, 19 },
    { 0x56727bf, 76 },
    { 0x297727c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7727c5, 33 },
    { 0xe50727c6, 98 },
    { 0xb44727c6, 90 },
    { 0xd2727c6, 61 },
    { 0xb3a727c9, 76 },
    { 0x6bf727c5, 34 },
    { 0, 0 },
    { 0xfd4727cc, 34 },
    { 0xa7f727cd, 33 },
    { 0, 0 },
    { 0x6fb727cf, 61 },
    { 0xf81727cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e727d3, 95 },
    { 0x330727d4, 90 },
    { 0, 0 },
    { 0xb7e727d6, 90 },
    { 0x7fc727d7, 90 },
    { 0x33f727d6, 61 },
    { 0xe26727d6, 23 },
    { 0, 0 },
    { 0x975727db, 93 },
    { 0x388727dc, 76 },
    { 0xf0c727db, 53 },
    { 0xfd7727de, 43 },
    { 0x797727df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a727e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e727e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750727ed, 33 },
    { 0xca9727ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0727fc, 23 },
    { 0xaa5727fd, 33 },
    { 0x2de727fc, 85 },
    { 0x266727ff, 83 },
    { 0x85e727ff, 100 },
    { 0x4a8727ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73872804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f72808, 95 },
    { 0, 0 },
    { 0xbd07280a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7472818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6547281e, 68 },
    { 0xb8a7281e, 95 },
    { 0x13b7281e, 61 },
    { 0x94472821, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x1372824, 24 },
    { 0x76872825, 33 },
    { 0x60672825, 88 },
    { 0xaee72827, 23 },
    { 0x30a72828, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6427282d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c72836, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a7283b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69572840, 81 },
    { 0, 0 },
    { 0x7ca72842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56972848, 93 },
    { 0x25a72849, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e72850, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec872855, 40 },
    { 0xd4372856, 23 },
    { 0xd1f72856, 93 },
    { 0xc2072856, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd737285b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26272860, 95 },
    { 0x30c72860, 76 },
    { 0xc5872862, 55 },
    { 0x57d72862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de7286a, 90 },
    { 0, 0 },
    { 0xaee7286c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2272872, 76 },
    { 0, 0 },
    { 0xd7172874, 90 },
    { 0, 0 },
    { 0x30172876, 23 },
    { 0xac772877, 84 },
    { 0x86772876, 61 },
    { 0, 0 },
    { 0xcd77287a, 74 },
    { 0x9187287b, 61 },
    { 0, 0 },
    { 0x3057287d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19572885, 100 },
    { 0x15672886, 67 },
    { 0, 0 },
    { 0x4a272888, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a7288b, 61 },
    { 0xbaa7288b, 34 },
    { 0xb07288b, 90 },
    { 0x1b07288c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66372893, 95 },
    { 0, 0 },
    { 0x98172895, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba72899, 98 },
    { 0xc567289a, 33 },
    { 0x1ab7289b, 90 },
    { 0x50e7289c, 95 },
    { 0xf1a7289b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc728a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3728a5, 66 },
    { 0x1fc728a5, 98 },
    { 0x6bb728a5, 90 },
    { 0x2c9728a5, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6728ad, 55 },
    { 0xd728ad, 46 },
    { 0, 0 },
    { 0xae2728b0, 90 },
    { 0, 0 },
    { 0xcf8728b2, 80 },
    { 0, 0 },
    { 0x728728b4, 33 },
    { 0xa91728b4, 90 },
    { 0x46a728b5, 55 },
    { 0x36d728b7, 61 },
    { 0xe08728b6, 93 },
    { 0x1dc728b8, 84 },
    { 0xed1728ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce728bd, 90 },
    { 0x9e6728be, 95 },
    { 0x37f728bf, 61 },
    { 0x211728c0, 61 },
    { 0xb21728bf, 34 },
    { 0xe16728c2, 90 },
    { 0x371728c1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245728c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738728ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b728d4, 55 },
    { 0, 0 },
    { 0x4b4728d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb728db, 90 },
    { 0xc79728db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x94728df, 84 },
    { 0xcbc728df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a728e5, 46 },
    { 0, 0 },
    { 0x1bf728e7, 32 },
    { 0x645728e8, 34 },
    { 0x220728e8, 91 },
    { 0xf97728e8, 87 },
    { 0x59c728e8, 98 },
    { 0xeba728ec, 65 },
    { 0x666728ed, 98 },
    { 0x2ff728ee, 93 },
    { 0x651728ef, 85 },
    { 0x7aa728ed, 46 },
    { 0xfa728e7, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450728f5, 88 },
    { 0x860728f5, 98 },
    { 0xffa728f7, 61 },
    { 0x64a728f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26728fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82372900, 98 },
    { 0x67b72901, 76 },
    { 0x16072901, 43 },
    { 0, 0 },
    { 0xbaf72904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4447290a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da7290d, 43 },
    { 0xbf47290e, 33 },
    { 0, 0 },
    { 0xb2172910, 90 },
    { 0, 0 },
    { 0xf9772912, 90 },
    { 0xf4b72913, 33 },
    { 0xc9d72913, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c372918, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f7291e, 61 },
    { 0, 0 },
    { 0xfde72920, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b72926, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c7292f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5572932, 98 },
    { 0x2aa72932, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf927293a, 46 },
    { 0x37f7293b, 78 },
    { 0xa847293a, 95 },
    { 0xb37293d, 90 },
    { 0x63e7293b, 34 },
    { 0x8b67293f, 99 },
    { 0, 0 },
    { 0x90872941, 34 },
    { 0x30672942, 98 },
    { 0xd1a72941, 90 },
    { 0, 0 },
    { 0x1a472945, 61 },
    { 0xaad72946, 90 },
    { 0, 0 },
    { 0xe5372948, 46 },
    { 0xf9d72949, 55 },
    { 0x3827294a, 33 },
    { 0xc937294b, 90 },
    { 0x73c7294a, 87 },
    { 0x6497294d, 52 },
    { 0x15d7294d, 33 },
    { 0xc2972948, 95 },
    { 0x81272950, 95 },
    { 0x7957294b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa72956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9472959, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8217295c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c72969, 76 },
    { 0xe4b7296a, 46 },
    { 0, 0 },
    { 0x1017296c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1772976, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2377297d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a72992, 23 },
    { 0, 0 },
    { 0x73972994, 76 },
    { 0x5a472995, 46 },
    { 0, 0 },
    { 0xb7b72997, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b7299d, 46 },
    { 0, 0 },
    { 0xce07299f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7729a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c729a9, 23 },
    { 0x9c9729aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9729b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff729b7, 93 },
    { 0, 0 },
    { 0x819729b9, 84 },
    { 0x431729ba, 33 },
    { 0x89c729bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328729c5, 99 },
    { 0x737729c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b729c9, 61 },
    { 0xcc2729ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5729d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc729da, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab729e2, 33 },
    { 0x6af729e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88729e8, 84 },
    { 0x152729e9, 33 },
    { 0xf57729e9, 61 },
    { 0, 0 },
    { 0x2b4729ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x674729ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a729f2, 95 },
    { 0, 0 },
    { 0xcba729f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93729f8, 95 },
    { 0x1d9729f9, 61 },
    { 0x68e729f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce729fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d72a00, 33 },
    { 0xd4572a00, 67 },
    { 0x44d72a02, 33 },
    { 0x3d072a03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c72a06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91572a09, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e72a0c, 98 },
    { 0x77872a0d, 34 },
    { 0x32472a0e, 33 },
    { 0x24b72a0c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f72a16, 66 },
    { 0x44d72a17, 61 },
    { 0xd7e72a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef472a1b, 23 },
    { 0xce872a1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x72572a1f, 90 },
    { 0xd9372a1f, 55 },
    { 0xf5572a21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32272a24, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0172a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0872a2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c572a2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1472a32, 98 },
    { 0xba572a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d72a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48172a3a, 90 },
    { 0x7a72a3a, 98 },
    { 0xa7972a3a, 76 },
    { 0xf3a72a3d, 90 },
    { 0xf1f72a3e, 95 },
    { 0x41d72a3b, 93 },
    { 0xc0a72a3e, 55 },
    { 0, 0 },
    { 0x61572a42, 61 },
    { 0xdf572a42, 17 },
    { 0x81f72a44, 95 },
    { 0, 0 },
    { 0x38672a46, 69 },
    { 0, 0 },
    { 0xcb972a48, 36 },
    { 0, 0 },
    { 0xa1372a4a, 66 },
    { 0x17472a4a, 87 },
    { 0x6a972a4c, 46 },
    { 0x1cb72a4d, 52 },
    { 0, 0 },
    { 0x74972a4f, 12 },
    { 0xf9072a50, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e272a56, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x47172a59, 17 },
    { 0xe9372a5a, 90 },
    { 0x26672a59, 55 },
    { 0xc4f72a5b, 93 },
    { 0x3a272a5d, 55 },
    { 0xdeb72a5e, 95 },
    { 0x32372a5c, 98 },
    { 0x1c72a60, 61 },
    { 0, 0 },
    { 0x30e72a62, 95 },
    { 0xfc072a63, 76 },
    { 0xb6972a64, 33 },
    { 0x9a472a63, 43 },
    { 0, 0 },
    { 0x2972a67, 90 },
    { 0x5d372a68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d72a6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36872a6f, 91 },
    { 0xe5272a70, 90 },
    { 0, 0 },
    { 0xbbb72a72, 33 },
    { 0xb172a72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xff572a76, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2172a7a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4072a81, 100 },
    { 0xeec72a82, 100 },
    { 0, 0 },
    { 0x64e72a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf372a89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea72a91, 98 },
    { 0, 0 },
    { 0x7b072a93, 90 },
    { 0xd4f72a94, 78 },
    { 0xbb572a94, 90 },
    { 0xb6572a96, 66 },
    { 0x86272a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972a9a, 77 },
    { 0, 0 },
    { 0x70472a9c, 67 },
    { 0xb4772a9d, 91 },
    { 0xc5a72a9e, 61 },
    { 0x28172a9f, 98 },
    { 0x872a9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e72aa3, 99 },
    { 0x14472aa3, 90 },
    { 0x7f972aa5, 61 },
    { 0, 0 },
    { 0x7d672aa7, 52 },
    { 0x38972aa8, 40 },
    { 0x47972aa7, 75 },
    { 0xa8572aaa, 90 },
    { 0x28072aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1772ab8, 90 },
    { 0x3c272ab9, 33 },
    { 0xba372aba, 95 },
    { 0x99872aba, 95 },
    { 0x7ab72ab8, 95 },
    { 0x24f72aba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84272ac3, 23 },
    { 0xac972ac4, 87 },
    { 0xf4872ac4, 34 },
    { 0xef772ac5, 93 },
    { 0x45672ac7, 23 },
    { 0, 0 },
    { 0xe672ac9, 69 },
    { 0x16372aca, 98 },
    { 0x4c572acb, 78 },
    { 0x19072acc, 23 },
    { 0xb8172ac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8872ad1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f72ad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c72adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d572adf, 23 },
    { 0, 0 },
    { 0xa8672ae1, 55 },
    { 0xb9072ae2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda72ae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1272aea, 43 },
    { 0x3d072aeb, 67 },
    { 0xf9f72aec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2472aef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83072af3, 91 },
    { 0x3d472af4, 90 },
    { 0xb2c72af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa72af9, 55 },
    { 0x89372afa, 100 },
    { 0x33172af9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91572aff, 94 },
    { 0, 0 },
    { 0x67f72b01, 23 },
    { 0x97d72b02, 61 },
    { 0, 0 },
    { 0x67172b04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc72b09, 61 },
    { 0xab972b0a, 66 },
    { 0xbe072b0b, 61 },
    { 0, 0 },
    { 0xb9372b0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd772b11, 33 },
    { 0x3be72b12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f72b15, 98 },
    { 0x39e72b16, 23 },
    { 0x6d672b17, 23 },
    { 0xf3b72b18, 98 },
    { 0xe2972b19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c72b1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92172b26, 33 },
    { 0x66b72b27, 68 },
    { 0xf8b72b28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5072b2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91872b30, 34 },
    { 0x74672b30, 90 },
    { 0x74772b32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc672b38, 23 },
    { 0x4aa72b39, 23 },
    { 0x94972b3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce072b3e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2772b41, 98 },
    { 0x84f72b42, 90 },
    { 0xc2a72b42, 34 },
    { 0x38472b44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d272b49, 95 },
    { 0x30472b4a, 23 },
    { 0, 0 },
    { 0x35c72b4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97072b52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98372b57, 96 },
    { 0x62472b57, 43 },
    { 0x6c772b59, 76 },
    { 0xdde72b5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b72b5d, 93 },
    { 0xf6472b5e, 56 },
    { 0, 0 },
    { 0x2f772b60, 33 },
    { 0x45272b61, 34 },
    { 0x2f272b62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae72b65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e272b6a, 33 },
    { 0, 0 },
    { 0x1fd72b6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e72b73, 23 },
    { 0x12c72b74, 23 },
    { 0xa272b75, 101 },
    { 0xa1072b76, 23 },
    { 0x57d72b74, 97 },
    { 0, 0 },
    { 0x76b72b79, 57 },
    { 0x1d272b79, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf372b81, 76 },
    { 0, 0 },
    { 0x9a772b83, 95 },
    { 0x3a572b84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb72b87, 33 },
    { 0, 0 },
    { 0xb8a72b89, 34 },
    { 0, 0 },
    { 0x1c372b8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85572b8e, 61 },
    { 0x92a72b8f, 90 },
    { 0, 0 },
    { 0x40872b91, 90 },
    { 0x1f872b91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d72b95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x24172b98, 87 },
    { 0x9a172b99, 23 },
    { 0x4d72b9a, 78 },
    { 0x78a72b9b, 34 },
    { 0, 0 },
    { 0xda272b9d, 76 },
    { 0xd0c72b9d, 75 },
    { 0, 0 },
    { 0x99472ba0, 23 },
    { 0, 0 },
    { 0xfd272ba2, 93 },
    { 0x5cf72ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e72ba8, 61 },
    { 0x21e72ba9, 93 },
    { 0, 0 },
    { 0xaab72bab, 76 },
    { 0, 0 },
    { 0x50d72bad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc72bb4, 33 },
    { 0x21e72bb5, 90 },
    { 0x76c72bb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be72bbc, 93 },
    { 0xd0e72bbd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af72bc5, 61 },
    { 0x54172bc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e872bc9, 90 },
    { 0xfbd72bca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b72bce, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98472bd2, 34 },
    { 0x9d372bd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52472bd7, 34 },
    { 0xb6d72bd8, 77 },
    { 0xc9f72bd9, 90 },
    { 0x7dc72bda, 23 },
    { 0xd2872bd9, 98 },
    { 0x1d872bdb, 61 },
    { 0, 0 },
    { 0x8c72bde, 23 },
    { 0xf5472bdf, 23 },
    { 0xd0e72be0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6072be4, 17 },
    { 0, 0 },
    { 0xc7a72be6, 95 },
    { 0xd0872be7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a72bf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5972c04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a872c09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b72c0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67972c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc72c15, 76 },
    { 0x63172c16, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa72c1e, 76 },
    { 0x8572c1f, 90 },
    { 0x2f272c1f, 95 },
    { 0x1c872c1e, 90 },
    { 0x87e72c21, 95 },
    { 0x29e72c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3772c2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd72c30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c72c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1272c3b, 100 },
    { 0xbde72c3c, 95 },
    { 0x1a772c3d, 66 },
    { 0x30472c3e, 52 },
    { 0x33472c3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c672c47, 95 },
    { 0, 0 },
    { 0xf9872c49, 61 },
    { 0x16372c4a, 75 },
    { 0xb4b72c49, 55 },
    { 0x29d72c4c, 95 },
    { 0xe4472c4d, 98 },
    { 0x91972c4d, 91 },
    { 0x4f872c4d, 33 },
    { 0xae072c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7472c5a, 98 },
    { 0xd7a72c5a, 87 },
    { 0x8672c5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad572c62, 95 },
    { 0x40472c62, 23 },
    { 0xa7d72c64, 78 },
    { 0x8e972c64, 75 },
    { 0x9b72c63, 90 },
    { 0x9be72c67, 55 },
    { 0xdd872c64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a072c6b, 90 },
    { 0xe8b72c6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c72c70, 98 },
    { 0x12a72c70, 91 },
    { 0x4f72c71, 76 },
    { 0xf3e72c73, 98 },
    { 0, 0 },
    { 0x28e72c75, 90 },
    { 0, 0 },
    { 0xf6572c77, 61 },
    { 0, 0 },
    { 0x72272c79, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f72c7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c72c87, 55 },
    { 0, 0 },
    { 0xdab72c89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c572c8f, 33 },
    { 0x872c8f, 76 },
    { 0xed772c90, 55 },
    { 0, 0 },
    { 0x84672c93, 23 },
    { 0, 0 },
    { 0x4d172c95, 76 },
    { 0, 0 },
    { 0xc5a72c97, 98 },
    { 0xfb572c97, 33 },
    { 0x4bf72c97, 90 },
    { 0x37672c9a, 98 },
    { 0x25472c9b, 61 },
    { 0xac972c9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62772ca0, 61 },
    { 0, 0 },
    { 0x36b72ca2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d72ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f572cab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96272caf, 90 },
    { 0xe8272cb0, 99 },
    { 0x16172cb0, 55 },
    { 0xb4572cb0, 23 },
    { 0, 0 },
    { 0x3bc72cb4, 61 },
    { 0, 0 },
    { 0x91972cb6, 34 },
    { 0xb2d72cb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8372cbf, 36 },
    { 0x10972cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab72cc3, 98 },
    { 0x5f872cc3, 78 },
    { 0, 0 },
    { 0xd0c72cc6, 90 },
    { 0x45b72cc6, 99 },
    { 0x13e72cc8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c72ccc, 90 },
    { 0xcb272ccd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d472cd3, 33 },
    { 0x41d72cd4, 78 },
    { 0, 0 },
    { 0x20f72cd6, 87 },
    { 0, 0 },
    { 0xa172cd8, 98 },
    { 0xfd372cd9, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8172cdc, 68 },
    { 0, 0 },
    { 0xc8972cde, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b72ce1, 61 },
    { 0, 0 },
    { 0x39d72ce3, 46 },
    { 0xce072ce3, 67 },
    { 0x45f72ce3, 61 },
    { 0x10e72ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44972ce9, 77 },
    { 0x7e572ce9, 90 },
    { 0xafc72ce9, 61 },
    { 0x5572ce9, 75 },
    { 0x30572ced, 23 },
    { 0xce772cee, 90 },
    { 0x59a72cef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94072cfd, 87 },
    { 0xfaa72cfe, 61 },
    { 0x7a872cfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a72d03, 23 },
    { 0x9c772d03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xea572d07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c72d0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb72d12, 95 },
    { 0xe1672d12, 33 },
    { 0xfe272d14, 95 },
    { 0, 0 },
    { 0x1072d16, 90 },
    { 0, 0 },
    { 0x50272d18, 46 },
    { 0, 0 },
    { 0x4a472d1a, 65 },
    { 0, 0 },
    { 0xa4172d1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d72d1f, 61 },
    { 0, 0 },
    { 0xee872d21, 85 },
    { 0, 0 },
    { 0x23972d23, 90 },
    { 0, 0 },
    { 0xc0472d25, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a72d30, 95 },
    { 0xbea72d31, 95 },
    { 0, 0 },
    { 0x7ca72d33, 93 },
    { 0xf272d34, 90 },
    { 0x4aa72d33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92972d38, 76 },
    { 0x17c72d39, 95 },
    { 0x8fd72d39, 78 },
    { 0x78972d3b, 98 },
    { 0x1fa72d3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb372d41, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e972d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df72d4f, 90 },
    { 0x68f72d50, 55 },
    { 0xef972d51, 68 },
    { 0xb5d72d4f, 69 },
    { 0x7d72d53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4672d59, 98 },
    { 0, 0 },
    { 0x68272d5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d972d5e, 85 },
    { 0, 0 },
    { 0x2e772d60, 55 },
    { 0x41272d61, 61 },
    { 0x3bd72d61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb72d66, 46 },
    { 0x3fa72d67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e72d6c, 61 },
    { 0x81b72d6d, 23 },
    { 0x49372d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53772d72, 95 },
    { 0, 0 },
    { 0xfd772d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f72d77, 90 },
    { 0x90172d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca772d7e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72672d87, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6372d8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a72d8e, 25 },
    { 0xbb272d8e, 33 },
    { 0, 0 },
    { 0x5ed72d91, 61 },
    { 0x90a72d92, 66 },
    { 0x5b972d92, 23 },
    { 0xf3072d94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e972d98, 91 },
    { 0, 0 },
    { 0x6e672d9a, 61 },
    { 0x50972d9b, 91 },
    { 0x67372d9a, 98 },
    { 0, 0 },
    { 0x8ba72d9e, 61 },
    { 0xdaa72d9f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15972da5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61372da8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac72dac, 76 },
    { 0x7ab72dac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f72db3, 75 },
    { 0x3b72db4, 43 },
    { 0xc1d72db5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea72dba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc72dc2, 95 },
    { 0xe0f72dc3, 55 },
    { 0x2fe72dc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60972dc8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7972dcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a72dd1, 34 },
    { 0x2e672dd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5172dd5, 33 },
    { 0, 0 },
    { 0x7ec72dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4472ddc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93772de2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad072de6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d72dea, 75 },
    { 0xd5d72deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1072dee, 98 },
    { 0xf2572dee, 84 },
    { 0x3af72def, 33 },
    { 0, 0 },
    { 0xee472df2, 80 },
    { 0xec772df2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b72df8, 87 },
    { 0xcb172df9, 33 },
    { 0x2f572dfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd72e05, 85 },
    { 0, 0 },
    { 0x63572e07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f172e0a, 95 },
    { 0, 0 },
    { 0x3f472e0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc72e10, 52 },
    { 0xcf172e10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1372e15, 98 },
    { 0x2d672e15, 95 },
    { 0xef272e17, 61 },
    { 0x2bb72e17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0872e1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8372e26, 34 },
    { 0, 0 },
    { 0x6a472e28, 95 },
    { 0, 0 },
    { 0xb2772e2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef72e30, 73 },
    { 0, 0 },
    { 0x3e372e32, 75 },
    { 0, 0 },
    { 0xe8672e34, 43 },
    { 0x71972e34, 91 },
    { 0, 0 },
    { 0x57472e37, 76 },
    { 0x88272e38, 77 },
    { 0xd8672e38, 99 },
    { 0x5d872e38, 87 },
    { 0, 0 },
    { 0x12272e3c, 85 },
    { 0, 0 },
    { 0xefd72e3e, 23 },
    { 0x6de72e3f, 61 },
    { 0, 0 },
    { 0xd2e72e41, 98 },
    { 0x7a672e42, 95 },
    { 0x8f772e42, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca72e47, 88 },
    { 0xf0a72e48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56872e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3672e55, 66 },
    { 0x7e72e55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc172e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d872e66, 95 },
    { 0xbf172e67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44372e6b, 61 },
    { 0x58072e6c, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca672e71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d72e76, 95 },
    { 0, 0 },
    { 0x1c272e78, 95 },
    { 0x1f372e78, 99 },
    { 0xa9672e78, 95 },
    { 0xff172e7b, 43 },
    { 0, 0 },
    { 0x65072e7d, 98 },
    { 0x50172e7d, 85 },
    { 0xe8772e7d, 99 },
    { 0x3d172e80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87372e84, 61 },
    { 0xcf972e85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e972e88, 23 },
    { 0x54472e89, 78 },
    { 0xd1272e8a, 40 },
    { 0x3f372e8b, 33 },
    { 0xbb072e8a, 95 },
    { 0x5f972e8d, 95 },
    { 0xb3872e8e, 76 },
    { 0xf0072e8f, 33 },
    { 0xb8372e90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec772e93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48172e97, 61 },
    { 0xdb172e98, 90 },
    { 0xb4d72e99, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec72ea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a672ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89472eaa, 98 },
    { 0x3f72eab, 55 },
    { 0, 0 },
    { 0xcdc72ead, 90 },
    { 0xd2072ead, 33 },
    { 0x67872eaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66072eb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87572eb6, 23 },
    { 0xe6972eb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93072ebc, 33 },
    { 0xe6672ebd, 23 },
    { 0x9a672ebc, 33 },
    { 0xa3672ebf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a72ec3, 34 },
    { 0x6a672ec3, 33 },
    { 0, 0 },
    { 0xd2572ec6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a72ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a72ecd, 52 },
    { 0xca972ece, 90 },
    { 0x5972ece, 33 },
    { 0x3772ed0, 77 },
    { 0x8872ed1, 98 },
    { 0x9072ecf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd672ed5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b572ee0, 76 },
    { 0x872ee0, 43 },
    { 0x34872ee2, 90 },
    { 0x3d872ee3, 34 },
    { 0x79772ee1, 93 },
    { 0x87472ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2572eea, 76 },
    { 0x8e72eea, 76 },
    { 0xc5172eea, 98 },
    { 0xf2372eed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c72ef1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98472ef8, 90 },
    { 0x77172ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d572efc, 98 },
    { 0xa5072efd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58972f00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda772f0e, 95 },
    { 0x8272f0f, 38 },
    { 0x2f472f10, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced72f15, 98 },
    { 0x45572f16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf172f1b, 34 },
    { 0x52f72f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51572f23, 12 },
    { 0x2e272f24, 33 },
    { 0, 0 },
    { 0x48872f26, 66 },
    { 0xf4b72f26, 33 },
    { 0xa2d72f28, 61 },
    { 0xc2272f28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6872f2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a72f34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db72f42, 52 },
    { 0x5cf72f42, 34 },
    { 0, 0 },
    { 0xac672f45, 33 },
    { 0x6cf72f45, 33 },
    { 0xd8072f46, 61 },
    { 0x61372f48, 100 },
    { 0xf7272f49, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2872f51, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5772f55, 66 },
    { 0x2e072f56, 76 },
    { 0xd8572f57, 67 },
    { 0x10d72f55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10872f5b, 76 },
    { 0, 0 },
    { 0x74c72f5d, 23 },
    { 0, 0 },
    { 0xe2072f5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c72f62, 43 },
    { 0x92372f63, 93 },
    { 0x4bf72f63, 95 },
    { 0x63072f63, 99 },
    { 0xa7872f66, 61 },
    { 0xc9772f66, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68872f6d, 23 },
    { 0x16d72f6e, 61 },
    { 0x41f72f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc72f72, 93 },
    { 0, 0 },
    { 0xdf872f74, 56 },
    { 0x48a72f75, 55 },
    { 0, 0 },
    { 0x44a72f77, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10472f7d, 61 },
    { 0, 0 },
    { 0xbad72f7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35372f83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e72f86, 43 },
    { 0xd3072f87, 67 },
    { 0xe9672f86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa72f8d, 93 },
    { 0xabd72f8e, 33 },
    { 0xc8b72f8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a072f94, 23 },
    { 0, 0 },
    { 0x59172f96, 85 },
    { 0, 0 },
    { 0x4b572f98, 95 },
    { 0xba372f99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80672f9e, 52 },
    { 0x15472f9f, 33 },
    { 0xef472fa0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1072fa4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f172faa, 90 },
    { 0x94272faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34272fae, 34 },
    { 0x1d872faf, 90 },
    { 0, 0 },
    { 0xf6472fb1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x31572fb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d72fb8, 61 },
    { 0x18172fb9, 38 },
    { 0x7fb72fba, 98 },
    { 0x9772fbb, 46 },
    { 0x79d72fba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f72fbf, 95 },
    { 0xe2e72fc0, 55 },
    { 0x2a672fc0, 93 },
    { 0, 0 },
    { 0x95472fc3, 98 },
    { 0, 0 },
    { 0xc7972fc5, 97 },
    { 0xb1772fc5, 46 },
    { 0x44d72fc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb72fca, 46 },
    { 0xa1372fcb, 33 },
    { 0, 0 },
    { 0x96e72fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c872fd7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e772fdd, 85 },
    { 0, 0 },
    { 0x34672fdf, 61 },
    { 0, 0 },
    { 0x60372fe1, 98 },
    { 0xe0e72fe2, 91 },
    { 0x16c72fe1, 61 },
    { 0x1b672fe4, 99 },
    { 0xfd872fe5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x46972fe8, 61 },
    { 0, 0 },
    { 0x6f572fea, 68 },
    { 0, 0 },
    { 0xbb772fec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d772ff7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb972ffd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f73003, 66 },
    { 0x63573004, 55 },
    { 0x5c73005, 76 },
    { 0x38473005, 95 },
    { 0xf3473007, 98 },
    { 0, 0 },
    { 0x3b973009, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce173011, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2387301d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75673024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36173028, 90 },
    { 0, 0 },
    { 0x2e27302a, 43 },
    { 0, 0 },
    { 0x1f67302c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4273031, 23 },
    { 0x12573032, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83373036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc7303c, 46 },
    { 0x91b7303d, 100 },
    { 0xe627303e, 76 },
    { 0, 0 },
    { 0x41473040, 98 },
    { 0xd0473041, 46 },
    { 0xa8f73040, 23 },
    { 0x80673041, 90 },
    { 0x5c573044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07304f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073052, 61 },
    { 0x30873053, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40473058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d7305f, 95 },
    { 0x4b673060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d673063, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6367306f, 68 },
    { 0x4773070, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c873074, 61 },
    { 0xa173075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f873078, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3247307c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c7307f, 98 },
    { 0, 0 },
    { 0x7ba73081, 98 },
    { 0xbd673081, 38 },
    { 0xadd73081, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7308a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a7308d, 90 },
    { 0x5077308e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d73091, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae73094, 98 },
    { 0x33a73095, 95 },
    { 0xb0273095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b7309b, 33 },
    { 0x6a47309c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf730a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0730a6, 81 },
    { 0, 0 },
    { 0x6c5730a8, 98 },
    { 0x9db730a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x843730ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6730b2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395730b6, 87 },
    { 0, 0 },
    { 0xeb1730b8, 76 },
    { 0x99730b9, 98 },
    { 0xe2b730b8, 65 },
    { 0xe13730bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f730be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b730c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d730c7, 90 },
    { 0xe5c730c7, 84 },
    { 0x6ec730c9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39730cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97730d1, 90 },
    { 0xec3730d1, 87 },
    { 0x58e730d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775730d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3730dc, 95 },
    { 0x7f3730dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65730e0, 95 },
    { 0, 0 },
    { 0xa42730e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364730e6, 85 },
    { 0, 0 },
    { 0x6c9730e8, 95 },
    { 0x933730e8, 61 },
    { 0x3db730e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e730ed, 96 },
    { 0xc98730ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5730f6, 95 },
    { 0, 0 },
    { 0xfe1730f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3730fb, 65 },
    { 0xdfd730fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f73102, 52 },
    { 0x9fc73102, 61 },
    { 0, 0 },
    { 0x1c573105, 23 },
    { 0x1a073106, 66 },
    { 0, 0 },
    { 0x2f673108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d17310b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe47310e, 75 },
    { 0x5667310f, 90 },
    { 0, 0 },
    { 0x1b573111, 33 },
    { 0, 0 },
    { 0xfed73113, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4273116, 76 },
    { 0x2a773117, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e7311c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d37311f, 91 },
    { 0xb2873120, 90 },
    { 0x1d773120, 93 },
    { 0xe0773120, 34 },
    { 0x2d273123, 33 },
    { 0x3d573124, 90 },
    { 0, 0 },
    { 0xfbc73126, 43 },
    { 0xdb973127, 78 },
    { 0xe7e73128, 67 },
    { 0xfd373128, 76 },
    { 0x7167312a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7573130, 90 },
    { 0, 0 },
    { 0xcf73132, 43 },
    { 0xa4c73133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54873137, 38 },
    { 0x13d73137, 99 },
    { 0xa8f73139, 61 },
    { 0x7477313a, 17 },
    { 0xa3c73137, 43 },
    { 0x69773139, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3287313f, 78 },
    { 0x4e77313f, 76 },
    { 0x97773141, 33 },
    { 0xe5173142, 61 },
    { 0, 0 },
    { 0x78d73144, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e473148, 95 },
    { 0x31073149, 33 },
    { 0x4237314a, 61 },
    { 0x8ff7314a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x117314e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a73152, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c973158, 23 },
    { 0x53f73159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f7315d, 77 },
    { 0x2507315e, 90 },
    { 0xfe27315f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab473166, 95 },
    { 0x1b973167, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6857316c, 36 },
    { 0, 0 },
    { 0x4517316e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda573174, 76 },
    { 0xbd473174, 87 },
    { 0x4cf73176, 90 },
    { 0, 0 },
    { 0x29d73178, 61 },
    { 0, 0 },
    { 0xb717317a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb57317d, 78 },
    { 0xd137317d, 98 },
    { 0x8b77317f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b73186, 90 },
    { 0x6e73187, 56 },
    { 0xbe273187, 55 },
    { 0xfab73189, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c7318e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc873191, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce73196, 76 },
    { 0x6ab73197, 8 },
    { 0, 0 },
    { 0x7b173199, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a7319c, 67 },
    { 0xec97319d, 61 },
    { 0, 0 },
    { 0x65b7319f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21731a2, 33 },
    { 0, 0 },
    { 0x442731a4, 93 },
    { 0x7fd731a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4731a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904731ad, 76 },
    { 0, 0 },
    { 0xef1731af, 84 },
    { 0, 0 },
    { 0x546731b1, 90 },
    { 0x92c731b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13731b9, 76 },
    { 0, 0 },
    { 0x90a731bb, 17 },
    { 0, 0 },
    { 0xadc731bd, 84 },
    { 0, 0 },
    { 0x9f2731bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8731c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff731d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2731d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x807731db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c731e3, 43 },
    { 0x20731e4, 43 },
    { 0xb21731e5, 55 },
    { 0xade731e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1731e9, 55 },
    { 0, 0 },
    { 0xfd0731eb, 90 },
    { 0, 0 },
    { 0xd73731ed, 52 },
    { 0xf94731ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea731f1, 95 },
    { 0, 0 },
    { 0x507731f3, 76 },
    { 0xafb731f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835731f8, 33 },
    { 0, 0 },
    { 0xd4d731fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23731fe, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d273205, 90 },
    { 0xe3b73206, 33 },
    { 0, 0 },
    { 0xa5f73208, 90 },
    { 0, 0 },
    { 0x9407320a, 61 },
    { 0xdfb7320b, 98 },
    { 0xbb7320c, 34 },
    { 0, 0 },
    { 0xb527320e, 58 },
    { 0x71d7320f, 93 },
    { 0x1e07320f, 76 },
    { 0xeed73211, 33 },
    { 0xd3173211, 33 },
    { 0x9c573212, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab573218, 61 },
    { 0xa3173219, 93 },
    { 0x45f73219, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17321f, 87 },
    { 0x3657321f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca73227, 100 },
    { 0, 0 },
    { 0x92573229, 55 },
    { 0x9b673229, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17322d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d73238, 61 },
    { 0x42173239, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4327323d, 17 },
    { 0x11c7323d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53373241, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49373246, 61 },
    { 0x97373246, 33 },
    { 0xb3b73248, 98 },
    { 0x44973249, 84 },
    { 0, 0 },
    { 0x5c67324b, 46 },
    { 0x8887324c, 85 },
    { 0x5857324d, 88 },
    { 0x437324d, 93 },
    { 0xcf87324f, 100 },
    { 0, 0 },
    { 0x4e973251, 34 },
    { 0x24873252, 61 },
    { 0xe5c73253, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc73256, 98 },
    { 0x1f573257, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe687325c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c73264, 40 },
    { 0, 0 },
    { 0x59c73266, 23 },
    { 0xe1973267, 95 },
    { 0, 0 },
    { 0xaca73269, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec7326f, 99 },
    { 0, 0 },
    { 0xdf873271, 93 },
    { 0x3bb73271, 64 },
    { 0, 0 },
    { 0xb1373274, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2373277, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7327d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42073286, 33 },
    { 0, 0 },
    { 0x373288, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee7328c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b7328f, 55 },
    { 0x1fa73290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c73293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1373296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea67329c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15732a0, 61 },
    { 0, 0 },
    { 0x282732a2, 95 },
    { 0xb89732a2, 95 },
    { 0xf41732a4, 94 },
    { 0x768732a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0732a8, 33 },
    { 0, 0 },
    { 0xede732aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e732ae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5732b5, 23 },
    { 0xd7e732b6, 34 },
    { 0x40d732b7, 95 },
    { 0, 0 },
    { 0x355732b9, 91 },
    { 0x7fc732ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8732bd, 95 },
    { 0x170732be, 61 },
    { 0xc81732bf, 61 },
    { 0, 0 },
    { 0xacd732c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633732c5, 33 },
    { 0x18a732c6, 23 },
    { 0x7fc732c5, 95 },
    { 0x431732c7, 33 },
    { 0xef9732c9, 95 },
    { 0, 0 },
    { 0xdfc732cb, 78 },
    { 0xbb4732cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb732cf, 95 },
    { 0xc5d732d0, 33 },
    { 0xcd6732d0, 33 },
    { 0x7bc732d1, 33 },
    { 0x679732d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781732d7, 95 },
    { 0xf08732d7, 23 },
    { 0xbef732d9, 61 },
    { 0, 0 },
    { 0x40c732db, 17 },
    { 0x5f7732dc, 95 },
    { 0x13c732dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9732e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3732e3, 43 },
    { 0, 0 },
    { 0x876732e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5732e8, 33 },
    { 0xe44732e9, 33 },
    { 0, 0 },
    { 0x707732eb, 98 },
    { 0, 0 },
    { 0x5f5732ed, 98 },
    { 0xcb1732ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d732f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd732fb, 100 },
    { 0x1e5732fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60732ff, 95 },
    { 0x4f0732ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe973305, 65 },
    { 0xf5573306, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4847330a, 90 },
    { 0x98b7330b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a73312, 84 },
    { 0x78273313, 76 },
    { 0x4fa73312, 76 },
    { 0, 0 },
    { 0x31b73316, 33 },
    { 0x81a73317, 90 },
    { 0xf3073317, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae7331c, 90 },
    { 0x5527331d, 34 },
    { 0x72a7331e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f773322, 93 },
    { 0x6c173322, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20873327, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa7332a, 66 },
    { 0x59b7332b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c7332e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1673333, 81 },
    { 0x79c73334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad77333a, 90 },
    { 0xb6a7333b, 76 },
    { 0x52a7333a, 33 },
    { 0x7a17333c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81073342, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5773346, 90 },
    { 0xe2273347, 33 },
    { 0x41f73346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37334b, 61 },
    { 0xc697334c, 65 },
    { 0x6107334d, 61 },
    { 0, 0 },
    { 0x5107334f, 34 },
    { 0, 0 },
    { 0xdee73351, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e73355, 100 },
    { 0, 0 },
    { 0xf4c73357, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f7335b, 46 },
    { 0x8fd7335b, 87 },
    { 0, 0 },
    { 0x3587335e, 46 },
    { 0x1ac7335f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c73366, 78 },
    { 0xadf73366, 34 },
    { 0, 0 },
    { 0x30c73369, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87336c, 77 },
    { 0, 0 },
    { 0x3927336e, 52 },
    { 0, 0 },
    { 0x3ea73370, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba173374, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d77337a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb7337d, 98 },
    { 0, 0 },
    { 0xdc67337f, 55 },
    { 0, 0 },
    { 0xe8973381, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b77338d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da73393, 33 },
    { 0xd5d73393, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a97339a, 83 },
    { 0, 0 },
    { 0xa717339c, 61 },
    { 0xfcc7339d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c733a0, 68 },
    { 0x2d3733a1, 76 },
    { 0xfe2733a2, 77 },
    { 0xea9733a2, 55 },
    { 0x9a0733a4, 52 },
    { 0x95733a2, 90 },
    { 0xb1b733a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e733ad, 75 },
    { 0xf05733ae, 46 },
    { 0xd0b733af, 75 },
    { 0xc2733b0, 95 },
    { 0x884733ae, 52 },
    { 0x754733b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c733b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b733bb, 43 },
    { 0x5a0733bc, 61 },
    { 0x2d7733bd, 98 },
    { 0xcb5733bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30733c1, 61 },
    { 0x366733c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1733c5, 33 },
    { 0xaed733c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b733ca, 43 },
    { 0, 0 },
    { 0xe02733cc, 95 },
    { 0x905733cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47733d5, 85 },
    { 0xdc8733d6, 85 },
    { 0xf57733d7, 33 },
    { 0x384733d8, 81 },
    { 0x745733d7, 76 },
    { 0x640733d5, 99 },
    { 0xdef733db, 33 },
    { 0xf5e733dc, 95 },
    { 0x24c733dc, 66 },
    { 0x7e5733de, 23 },
    { 0x10d733d5, 61 },
    { 0x6ad733e0, 61 },
    { 0xb97733d9, 61 },
    { 0x452733dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x338733e5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3733e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e733ed, 33 },
    { 0x320733ee, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53733f3, 46 },
    { 0xad6733f3, 76 },
    { 0x698733f5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686733fb, 90 },
    { 0x41f733fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad73408, 33 },
    { 0x3c373408, 76 },
    { 0x77573408, 85 },
    { 0xdfe7340a, 23 },
    { 0xbd573409, 61 },
    { 0x9d67340d, 33 },
    { 0x1457340c, 98 },
    { 0, 0 },
    { 0x1e873410, 95 },
    { 0x3f473411, 93 },
    { 0xf1e73410, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb673419, 90 },
    { 0x82b7341a, 90 },
    { 0xc237341b, 90 },
    { 0x6267341b, 99 },
    { 0x6137341d, 77 },
    { 0x52e7341c, 76 },
    { 0xe0f7341f, 91 },
    { 0x6a773420, 98 },
    { 0xf0f73421, 90 },
    { 0x5bb73422, 87 },
    { 0, 0 },
    { 0x49673424, 34 },
    { 0, 0 },
    { 0x50473426, 98 },
    { 0xf4a73427, 33 },
    { 0x1c673426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e7342b, 33 },
    { 0x8a67342c, 65 },
    { 0x7517342d, 34 },
    { 0, 0 },
    { 0x5a07342f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45173433, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56073437, 87 },
    { 0x7e673437, 95 },
    { 0xdc273437, 55 },
    { 0x16273438, 84 },
    { 0xed7343b, 76 },
    { 0x49073439, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa873440, 43 },
    { 0x1be73440, 23 },
    { 0, 0 },
    { 0xefa73443, 95 },
    { 0x7073444, 78 },
    { 0x13a73444, 46 },
    { 0xf5a73443, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca7344c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e73450, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d773459, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x67345c, 23 },
    { 0x4e47345c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe973461, 33 },
    { 0, 0 },
    { 0xc2c73463, 81 },
    { 0xfeb73463, 52 },
    { 0x68f73465, 87 },
    { 0x93b73465, 35 },
    { 0xdd573464, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f7346d, 87 },
    { 0, 0 },
    { 0xddf7346f, 19 },
    { 0x95a7346f, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b473476, 34 },
    { 0x39273476, 33 },
    { 0xe4a73478, 43 },
    { 0xff973479, 99 },
    { 0, 0 },
    { 0x6e87347b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c73485, 90 },
    { 0xb8a73486, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f73489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da7348d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc273495, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7349b, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe7349f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e734a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed734ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705734b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c734b7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355734bf, 23 },
    { 0xc69734c0, 33 },
    { 0x389734c1, 61 },
    { 0xb9c734c1, 90 },
    { 0, 0 },
    { 0x9fd734c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f734ca, 75 },
    { 0xe70734cb, 61 },
    { 0xb03734cb, 33 },
    { 0x489734cb, 93 },
    { 0, 0 },
    { 0x37a734cf, 33 },
    { 0, 0 },
    { 0xf71734d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df734d4, 55 },
    { 0, 0 },
    { 0x8ae734d6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5734db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e734e6, 84 },
    { 0x21734e6, 61 },
    { 0x257734e7, 61 },
    { 0x6d9734e9, 98 },
    { 0xad9734ea, 88 },
    { 0x6cc734eb, 33 },
    { 0x139734e6, 75 },
    { 0, 0 },
    { 0xbfc734ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39734f4, 23 },
    { 0xa68734f5, 64 },
    { 0x804734f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1734ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a73502, 91 },
    { 0x32473502, 87 },
    { 0, 0 },
    { 0x61073505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21373509, 95 },
    { 0x6577350a, 76 },
    { 0xef77350b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11973511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc07351c, 33 },
    { 0xd727351d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f73523, 46 },
    { 0, 0 },
    { 0xe3773525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df73529, 76 },
    { 0x6077352a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7352e, 95 },
    { 0x82e7352f, 33 },
    { 0x137352e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc73533, 100 },
    { 0, 0 },
    { 0x2f373535, 33 },
    { 0x2aa73535, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xde473539, 95 },
    { 0x62b7353a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7073544, 23 },
    { 0xf7273545, 100 },
    { 0, 0 },
    { 0x28873547, 68 },
    { 0x54173548, 33 },
    { 0x26673547, 33 },
    { 0xc9f73549, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e17354d, 90 },
    { 0, 0 },
    { 0x9837354f, 87 },
    { 0x3f873550, 81 },
    { 0xa017354f, 46 },
    { 0xf4c73550, 44 },
    { 0xecd73553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10073557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e7355a, 61 },
    { 0x767355a, 98 },
    { 0x7b67355c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a73560, 33 },
    { 0xea73561, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3173565, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc73569, 100 },
    { 0x73273569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff7356f, 67 },
    { 0x16c73570, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9173575, 77 },
    { 0x3f73576, 95 },
    { 0x72473577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8817357a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33173580, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d773583, 65 },
    { 0, 0 },
    { 0xa2173585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10773588, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5187358d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee473596, 90 },
    { 0, 0 },
    { 0xf5673598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6735a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6735a5, 67 },
    { 0, 0 },
    { 0xa9d735a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba735ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07735ae, 23 },
    { 0, 0 },
    { 0x232735b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664735b5, 34 },
    { 0x917735b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55735ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905735c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e735c6, 87 },
    { 0xfa9735c7, 78 },
    { 0xe3b735c8, 95 },
    { 0x5d9735c9, 98 },
    { 0x5b2735c8, 91 },
    { 0x62f735c6, 68 },
    { 0, 0 },
    { 0xd14735cd, 76 },
    { 0xef1735cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233735d3, 99 },
    { 0xb5b735d4, 61 },
    { 0x600735d5, 52 },
    { 0x7af735d5, 23 },
    { 0, 0 },
    { 0xcb9735d8, 95 },
    { 0x225735d8, 55 },
    { 0, 0 },
    { 0x1c7735db, 61 },
    { 0xa8d735dc, 23 },
    { 0x979735dd, 100 },
    { 0xfb8735de, 98 },
    { 0xcd2735df, 67 },
    { 0x1f7735de, 68 },
    { 0, 0 },
    { 0x473735e2, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d735e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80735ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7735ef, 47 },
    { 0xd7a735ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e735f6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191735fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d73601, 23 },
    { 0x5d473602, 23 },
    { 0, 0 },
    { 0x23973604, 55 },
    { 0, 0 },
    { 0x25c73606, 33 },
    { 0x9fb73607, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4817360a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea17360d, 95 },
    { 0x9f07360e, 38 },
    { 0, 0 },
    { 0xae73610, 90 },
    { 0x69d73610, 33 },
    { 0, 0 },
    { 0x4ee73613, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73618, 66 },
    { 0xe1273619, 34 },
    { 0x6ab73619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5173622, 23 },
    { 0, 0 },
    { 0x80073624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad473627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8087362c, 33 },
    { 0x3957362d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb73632, 64 },
    { 0xf4173633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94873636, 98 },
    { 0x30f73636, 23 },
    { 0x2a73638, 93 },
    { 0, 0 },
    { 0x3cd7363a, 61 },
    { 0xc9a7363b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2373640, 90 },
    { 0x3ab73640, 61 },
    { 0, 0 },
    { 0xfc773643, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e47364a, 61 },
    { 0xe107364b, 90 },
    { 0x4377364c, 90 },
    { 0x46c7364b, 98 },
    { 0xe1b7364c, 69 },
    { 0x23f7364a, 55 },
    { 0xf1a7364d, 98 },
    { 0xa2173651, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93173656, 55 },
    { 0xa7373657, 98 },
    { 0, 0 },
    { 0x41a73659, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb207365d, 95 },
    { 0, 0 },
    { 0xbab7365f, 55 },
    { 0x7767365f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1787366a, 33 },
    { 0, 0 },
    { 0x4a37366c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f73671, 98 },
    { 0xb9573672, 93 },
    { 0xdf173672, 84 },
    { 0xb1d73674, 98 },
    { 0x40673675, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a27367d, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c73683, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5873687, 33 },
    { 0x15373688, 76 },
    { 0, 0 },
    { 0x3397368a, 67 },
    { 0x2d97368b, 87 },
    { 0x7ae7368c, 83 },
    { 0x5b7368d, 98 },
    { 0xee27368e, 90 },
    { 0x26f7368e, 23 },
    { 0, 0 },
    { 0xdaa73691, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee73695, 33 },
    { 0, 0 },
    { 0x8773697, 98 },
    { 0x71073698, 33 },
    { 0, 0 },
    { 0x1977369a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17369f, 76 },
    { 0x403736a0, 76 },
    { 0xae27369f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2736a6, 76 },
    { 0xad2736a7, 33 },
    { 0, 0 },
    { 0xf4c736a9, 90 },
    { 0x577736aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a736b6, 78 },
    { 0x720736b6, 61 },
    { 0xa7b736b8, 34 },
    { 0x59c736b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825736be, 98 },
    { 0xc93736bf, 90 },
    { 0xffa736bf, 81 },
    { 0x93d736c1, 76 },
    { 0, 0 },
    { 0xaf736c3, 23 },
    { 0xa06736c3, 80 },
    { 0, 0 },
    { 0xf66736c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9736cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06736ce, 33 },
    { 0, 0 },
    { 0x944736d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3736d8, 76 },
    { 0, 0 },
    { 0x701736da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167736e3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6736ed, 90 },
    { 0, 0 },
    { 0xac4736ef, 33 },
    { 0x504736f0, 90 },
    { 0xef7736f1, 87 },
    { 0, 0 },
    { 0xfc5736f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf736f9, 61 },
    { 0x85a736f9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529736fe, 66 },
    { 0x9f7736ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3173704, 95 },
    { 0xdec73705, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3373708, 90 },
    { 0x73d73708, 23 },
    { 0x667370a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf273712, 61 },
    { 0x89b73712, 33 },
    { 0, 0 },
    { 0x9ab73715, 98 },
    { 0x8b873716, 93 },
    { 0xfe473717, 95 },
    { 0x78073718, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47371c, 95 },
    { 0x1527371d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec373723, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b73728, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e57372b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86073733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8873736, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c17373a, 94 },
    { 0, 0 },
    { 0xd77373c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e73743, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x45873746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c17374b, 17 },
    { 0x19d7374c, 76 },
    { 0xbc07374d, 34 },
    { 0x2777374b, 66 },
    { 0x9de7374f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf73754, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e73757, 76 },
    { 0x17d73757, 76 },
    { 0, 0 },
    { 0x19f7375a, 90 },
    { 0xc2d7375b, 81 },
    { 0xd577375b, 33 },
    { 0, 0 },
    { 0xe5d7375e, 23 },
    { 0xe257375f, 98 },
    { 0, 0 },
    { 0xa1673761, 61 },
    { 0xd8273761, 91 },
    { 0, 0 },
    { 0x6a473764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85873767, 65 },
    { 0, 0 },
    { 0xd4373769, 55 },
    { 0xb787376a, 23 },
    { 0, 0 },
    { 0x8b37376c, 34 },
    { 0, 0 },
    { 0xc767376e, 23 },
    { 0xb0d7376f, 66 },
    { 0x1f17376f, 67 },
    { 0x4ab7376f, 46 },
    { 0x4d173772, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74673776, 76 },
    { 0x72f73777, 87 },
    { 0x8b373776, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8777377c, 76 },
    { 0x9f27377c, 33 },
    { 0x2117377e, 90 },
    { 0, 0 },
    { 0x1f73780, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d373786, 52 },
    { 0x8f873787, 55 },
    { 0xfcd73788, 34 },
    { 0x3f073788, 75 },
    { 0, 0 },
    { 0xc517378b, 23 },
    { 0, 0 },
    { 0x7087378d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc73795, 99 },
    { 0x14f73795, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a573799, 91 },
    { 0x7217379a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb737a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f737ad, 68 },
    { 0x816737ae, 85 },
    { 0x305737af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x354737b2, 66 },
    { 0, 0 },
    { 0xd95737b4, 33 },
    { 0, 0 },
    { 0xc78737b6, 61 },
    { 0xc92737b7, 97 },
    { 0x3d1737b7, 99 },
    { 0x447737b9, 95 },
    { 0xa4737b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2737c1, 98 },
    { 0xa34737c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8737c7, 23 },
    { 0, 0 },
    { 0x2ff737c9, 81 },
    { 0xebf737c9, 84 },
    { 0x874737c9, 98 },
    { 0, 0 },
    { 0xf51737cd, 95 },
    { 0x87c737ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe737d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7737d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x568737d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6737dc, 55 },
    { 0xa2737dc, 52 },
    { 0, 0 },
    { 0x1ed737df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a737e2, 62 },
    { 0, 0 },
    { 0x9f1737e4, 46 },
    { 0xd04737e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde737e8, 34 },
    { 0, 0 },
    { 0x3dd737ea, 33 },
    { 0xdd6737eb, 98 },
    { 0xd2737eb, 61 },
    { 0xcf4737ec, 90 },
    { 0xf3c737ed, 61 },
    { 0xe27737ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1737f4, 97 },
    { 0x759737f4, 90 },
    { 0, 0 },
    { 0x4a1737f7, 61 },
    { 0xb73737f8, 98 },
    { 0, 0 },
    { 0x4a3737fa, 33 },
    { 0x1aa737fb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x550737fe, 64 },
    { 0, 0 },
    { 0xc6d73800, 66 },
    { 0, 0 },
    { 0xac573802, 90 },
    { 0xd6973803, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e73806, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6273809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c7380f, 91 },
    { 0x88373810, 34 },
    { 0xe6b73811, 95 },
    { 0x64c73812, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf7381a, 52 },
    { 0, 0 },
    { 0xeb17381c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b7381f, 90 },
    { 0x81673820, 81 },
    { 0xd107381f, 52 },
    { 0xb8073822, 76 },
    { 0x31473820, 90 },
    { 0x5c973820, 95 },
    { 0x96373824, 90 },
    { 0x1d073820, 19 },
    { 0x17a73821, 55 },
    { 0x18873821, 23 },
    { 0, 0 },
    { 0x17d7382a, 76 },
    { 0x98c7382b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91573832, 95 },
    { 0x3c873833, 75 },
    { 0x67973834, 61 },
    { 0xc7573834, 33 },
    { 0xa0f73836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc73839, 95 },
    { 0, 0 },
    { 0x77c7383b, 91 },
    { 0xfe47383c, 33 },
    { 0, 0 },
    { 0x1647383e, 99 },
    { 0x8657383f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c373847, 33 },
    { 0xe5b73848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8873850, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f173853, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41473858, 90 },
    { 0x6b773859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf7385f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc73866, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7427386e, 75 },
    { 0, 0 },
    { 0xb2973870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b273873, 33 },
    { 0x47c73874, 34 },
    { 0xad273875, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c473878, 82 },
    { 0x3f573879, 98 },
    { 0, 0 },
    { 0x4ae7387b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c73881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2773886, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd597388a, 33 },
    { 0x4517388a, 55 },
    { 0x66b7388a, 93 },
    { 0x7b27388d, 95 },
    { 0xe627388d, 61 },
    { 0x94c7388e, 88 },
    { 0x7eb73890, 75 },
    { 0x9477388f, 95 },
    { 0xd6e73892, 23 },
    { 0xd437388b, 43 },
    { 0x6d873894, 34 },
    { 0x1a37388c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e7389f, 95 },
    { 0x8a738a0, 24 },
    { 0x5bd738a1, 34 },
    { 0xb0b738a2, 95 },
    { 0x765738a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b738a9, 33 },
    { 0, 0 },
    { 0x38f738ab, 98 },
    { 0x60e738ac, 34 },
    { 0xb44738ac, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7738b0, 90 },
    { 0, 0 },
    { 0xe3f738b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c738b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10738bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571738c2, 90 },
    { 0xd32738c3, 90 },
    { 0xd52738c3, 78 },
    { 0, 0 },
    { 0x1c6738c6, 98 },
    { 0, 0 },
    { 0xb8a738c8, 95 },
    { 0x25d738c8, 65 },
    { 0xb3d738c8, 61 },
    { 0x807738ca, 95 },
    { 0x204738c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c738d7, 88 },
    { 0, 0 },
    { 0x4da738d9, 95 },
    { 0xd8c738da, 34 },
    { 0xf76738da, 67 },
    { 0, 0 },
    { 0x2738dd, 34 },
    { 0xbf7738de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff738e2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943738e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9738eb, 61 },
    { 0xe37738ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5738fb, 46 },
    { 0, 0 },
    { 0x269738fd, 93 },
    { 0x138738fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b973904, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb573908, 98 },
    { 0x8f73908, 61 },
    { 0xb3d7390a, 90 },
    { 0x7747390b, 81 },
    { 0x89d7390c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee97390f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac73914, 92 },
    { 0, 0 },
    { 0x25273916, 95 },
    { 0xc1a73917, 90 },
    { 0x3f773916, 87 },
    { 0x38a73919, 34 },
    { 0x4cb7391a, 98 },
    { 0xd2773918, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92773921, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe73925, 81 },
    { 0xbb873926, 46 },
    { 0, 0 },
    { 0x53273928, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa967392c, 46 },
    { 0, 0 },
    { 0xedc7392e, 43 },
    { 0xb837392f, 67 },
    { 0x1537392e, 100 },
    { 0, 0 },
    { 0x3dd73932, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a73937, 55 },
    { 0, 0 },
    { 0x45c73939, 23 },
    { 0x5657393a, 76 },
    { 0x5557393b, 61 },
    { 0, 0 },
    { 0x2287393d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5373945, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31073949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17394c, 91 },
    { 0xa267394d, 98 },
    { 0xde37394e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b73952, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d773956, 23 },
    { 0, 0 },
    { 0x33173958, 66 },
    { 0, 0 },
    { 0x6fa7395a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d7395f, 55 },
    { 0, 0 },
    { 0x83d73961, 76 },
    { 0xfcf73962, 93 },
    { 0, 0 },
    { 0x62073964, 66 },
    { 0x9d873964, 88 },
    { 0, 0 },
    { 0xc2273967, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07396b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x887396e, 90 },
    { 0xd3d7396e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f73972, 85 },
    { 0, 0 },
    { 0x18a73974, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e7397a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad673981, 52 },
    { 0xec173982, 98 },
    { 0x81273983, 17 },
    { 0, 0 },
    { 0x2e473985, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd7398e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a973991, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3373995, 23 },
    { 0x27073996, 33 },
    { 0x88d73995, 61 },
    { 0, 0 },
    { 0xf5073999, 93 },
    { 0x9ea7399a, 98 },
    { 0x77399b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e7399f, 90 },
    { 0, 0 },
    { 0x435739a1, 93 },
    { 0x7b0739a2, 90 },
    { 0xd08739a3, 38 },
    { 0, 0 },
    { 0x937739a5, 23 },
    { 0x27739a5, 34 },
    { 0x46739a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d739aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x436739ad, 33 },
    { 0x6ca739ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536739b2, 98 },
    { 0, 0 },
    { 0x2d8739b4, 76 },
    { 0x7fd739b4, 33 },
    { 0xc5a739b6, 61 },
    { 0xa93739b6, 98 },
    { 0x11b739b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e739bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478739c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3739d2, 61 },
    { 0x58d739d2, 76 },
    { 0xdde739d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8739d7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555739df, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd739e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455739e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7739ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239739f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b739fb, 98 },
    { 0xda2739fb, 76 },
    { 0, 0 },
    { 0x843739fe, 98 },
    { 0xa5739fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70973a04, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73173a08, 23 },
    { 0, 0 },
    { 0xab173a0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe173a0e, 23 },
    { 0xe1073a0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd173a12, 90 },
    { 0, 0 },
    { 0x14173a14, 61 },
    { 0, 0 },
    { 0xb5e73a16, 90 },
    { 0x2e873a16, 77 },
    { 0x1373a18, 34 },
    { 0x3b973a18, 61 },
    { 0, 0 },
    { 0x51973a1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e73a1e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b73a23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff373a26, 23 },
    { 0x5ea73a27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc273a2c, 61 },
    { 0xff373a2d, 23 },
    { 0x7f973a2e, 84 },
    { 0x3b173a2f, 90 },
    { 0x1a273a30, 84 },
    { 0x14c73a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa273a34, 55 },
    { 0, 0 },
    { 0x63873a36, 33 },
    { 0, 0 },
    { 0xdb73a38, 81 },
    { 0, 0 },
    { 0xe0373a3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1373a3d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9873a41, 23 },
    { 0, 0 },
    { 0xb3873a43, 90 },
    { 0xcc573a44, 84 },
    { 0, 0 },
    { 0xea473a46, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f373a4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be73a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50173a52, 95 },
    { 0x9ce73a52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e73a56, 23 },
    { 0, 0 },
    { 0xfd873a58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f73a5b, 23 },
    { 0, 0 },
    { 0x5eb73a5d, 90 },
    { 0x97073a5e, 33 },
    { 0, 0 },
    { 0xdc373a60, 59 },
    { 0, 0 },
    { 0xea873a62, 76 },
    { 0, 0 },
    { 0x3e973a64, 61 },
    { 0xa1973a65, 61 },
    { 0x73973a64, 98 },
    { 0x4e673a67, 90 },
    { 0xd9f73a66, 23 },
    { 0x19073a69, 42 },
    { 0x27073a6a, 98 },
    { 0xfbc73a6b, 95 },
    { 0xdfa73a6b, 23 },
    { 0, 0 },
    { 0x5e773a6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21273a75, 90 },
    { 0x58373a76, 55 },
    { 0x15173a77, 61 },
    { 0x12373a78, 61 },
    { 0, 0 },
    { 0xdd073a7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d573a7d, 95 },
    { 0x31073a7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93973a82, 23 },
    { 0x21773a83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73a88, 75 },
    { 0, 0 },
    { 0xe9f73a8a, 61 },
    { 0, 0 },
    { 0xc0773a8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52173a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf173a96, 61 },
    { 0x12673a97, 78 },
    { 0xa0773a96, 95 },
    { 0x93f73a97, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb373a9c, 34 },
    { 0xfec73a9d, 33 },
    { 0, 0 },
    { 0x82073a9f, 61 },
    { 0x6573aa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f73aa3, 46 },
    { 0x96273aa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90773aa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6373aab, 46 },
    { 0x4973aac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7573aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d673ab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51273aba, 99 },
    { 0xdb273abb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17373ac1, 24 },
    { 0x5373ac2, 95 },
    { 0x43773ac3, 33 },
    { 0xa5c73ac2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c73ace, 90 },
    { 0, 0 },
    { 0xa673ad0, 99 },
    { 0xbe073ad0, 61 },
    { 0x1e873ad1, 23 },
    { 0x47a73ad3, 33 },
    { 0x33e73ad1, 45 },
    { 0xc9573ad5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22473ad9, 90 },
    { 0xb8273ad9, 91 },
    { 0x5be73ad9, 90 },
    { 0x85b73ada, 95 },
    { 0xbe673add, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c73ae2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e73ae6, 90 },
    { 0, 0 },
    { 0x3b973ae8, 99 },
    { 0x46773ae9, 78 },
    { 0, 0 },
    { 0x7e573aeb, 90 },
    { 0xdd873aeb, 98 },
    { 0xc1973aed, 43 },
    { 0xac173aee, 76 },
    { 0x1ad73aee, 34 },
    { 0, 0 },
    { 0xc6373af1, 81 },
    { 0x37573af2, 61 },
    { 0xaf273af3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d73af8, 66 },
    { 0x11d73af9, 33 },
    { 0xca073afa, 77 },
    { 0x8e73af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a73afe, 33 },
    { 0xad273afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e73b05, 76 },
    { 0, 0 },
    { 0x4f773b07, 66 },
    { 0x1b273b07, 33 },
    { 0x8ca73b09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf973b0d, 99 },
    { 0x37873b0d, 90 },
    { 0xb6773b0e, 38 },
    { 0x68873b10, 46 },
    { 0x54a73b11, 95 },
    { 0x49b73b12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b673b15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc573b18, 65 },
    { 0xaa273b19, 52 },
    { 0x52473b1a, 91 },
    { 0x63c73b19, 99 },
    { 0x42773b1c, 98 },
    { 0, 0 },
    { 0x2bd73b1e, 87 },
    { 0, 0 },
    { 0xf0d73b20, 95 },
    { 0, 0 },
    { 0x49373b22, 93 },
    { 0x34f73b22, 98 },
    { 0xc673b24, 67 },
    { 0, 0 },
    { 0xd573b26, 90 },
    { 0xd2d73b27, 62 },
    { 0x88a73b28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2373b2d, 95 },
    { 0x57473b2e, 99 },
    { 0, 0 },
    { 0xf1173b30, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64973b35, 98 },
    { 0, 0 },
    { 0x5e473b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae73b3b, 33 },
    { 0xcd73b3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1173b42, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52473b4d, 90 },
    { 0x55f73b4e, 91 },
    { 0x14e73b4f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51373b54, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0573b5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a473b5d, 43 },
    { 0, 0 },
    { 0x77e73b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72573b63, 95 },
    { 0x97873b63, 87 },
    { 0xf6a73b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce73b71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c73b74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5473b77, 98 },
    { 0x87b73b78, 61 },
    { 0xad073b79, 23 },
    { 0x3973b79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f973b81, 88 },
    { 0xcc773b82, 90 },
    { 0, 0 },
    { 0xb7c73b84, 78 },
    { 0xcc873b85, 95 },
    { 0x82e73b84, 97 },
    { 0x12c73b85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8073b8b, 33 },
    { 0xe2373b8c, 43 },
    { 0, 0 },
    { 0xaef73b8e, 81 },
    { 0, 0 },
    { 0x43273b90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b73b94, 34 },
    { 0, 0 },
    { 0x57973b96, 75 },
    { 0x5c073b97, 76 },
    { 0, 0 },
    { 0xd0173b99, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f73b9c, 33 },
    { 0xa4773b9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d273ba1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x79173ba4, 94 },
    { 0xacf73ba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af73ba9, 23 },
    { 0x80773baa, 61 },
    { 0xf1a73bab, 61 },
    { 0xb2b73bac, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16773bb2, 61 },
    { 0, 0 },
    { 0xb7373bb4, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e73bb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd73bbc, 34 },
    { 0, 0 },
    { 0x57f73bbe, 95 },
    { 0xd2073bbf, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f73bc3, 95 },
    { 0, 0 },
    { 0x3773bc5, 78 },
    { 0xbcd73bc5, 46 },
    { 0, 0 },
    { 0x87b73bc8, 69 },
    { 0x15673bc9, 93 },
    { 0, 0 },
    { 0x1da73bcb, 75 },
    { 0x24673bcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6173bd0, 61 },
    { 0x94173bd1, 23 },
    { 0x32373bd1, 91 },
    { 0xf6173bd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1273bd8, 90 },
    { 0x1a973bd9, 90 },
    { 0x15a73bda, 98 },
    { 0x94973bd9, 55 },
    { 0x37273bdb, 98 },
    { 0x8ef73bdd, 61 },
    { 0x82973bde, 43 },
    { 0x56973bdf, 91 },
    { 0x92773bdd, 95 },
    { 0xa9e73be1, 90 },
    { 0x8a173be2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd373be8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff873beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb73bef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f173bf6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d73bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd673bfd, 98 },
    { 0xb6e73bfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71573c02, 98 },
    { 0x7773c02, 33 },
    { 0xcd473c04, 33 },
    { 0, 0 },
    { 0xb2e73c06, 76 },
    { 0, 0 },
    { 0x2d473c08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da73c0c, 98 },
    { 0, 0 },
    { 0x24073c0e, 93 },
    { 0, 0 },
    { 0x45c73c10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b673c13, 98 },
    { 0xe573c14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f173c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5273c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf73c21, 98 },
    { 0, 0 },
    { 0xbb273c23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25473c26, 55 },
    { 0x70b73c27, 99 },
    { 0xd0973c27, 34 },
    { 0x18773c29, 85 },
    { 0x13c73c2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9573c2e, 90 },
    { 0xa1b73c2f, 61 },
    { 0xd2673c2f, 90 },
    { 0x2c573c31, 78 },
    { 0xd6173c31, 23 },
    { 0x31873c2e, 66 },
    { 0x84e73c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c73c3a, 33 },
    { 0x1ce73c3b, 23 },
    { 0x2d373c3c, 99 },
    { 0x3fb73c3a, 94 },
    { 0, 0 },
    { 0x7973c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff173c43, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c73c49, 98 },
    { 0xad073c4a, 55 },
    { 0xbde73c4b, 90 },
    { 0xa9573c4c, 33 },
    { 0x8b773c4d, 90 },
    { 0x95573c4e, 84 },
    { 0, 0 },
    { 0x89b73c50, 23 },
    { 0, 0 },
    { 0x6d573c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d573c55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43473c5b, 76 },
    { 0xbf173c5c, 90 },
    { 0, 0 },
    { 0xa3d73c5e, 61 },
    { 0x56b73c5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a73c67, 90 },
    { 0x7e873c67, 23 },
    { 0, 0 },
    { 0x9dd73c6a, 55 },
    { 0x28773c6b, 40 },
    { 0xee973c6c, 68 },
    { 0x6c873c6c, 17 },
    { 0, 0 },
    { 0x2d573c6f, 95 },
    { 0x15a73c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48773c75, 90 },
    { 0xa6473c76, 78 },
    { 0, 0 },
    { 0xb4173c78, 23 },
    { 0x93b73c79, 33 },
    { 0xb673c7a, 33 },
    { 0x89573c7b, 52 },
    { 0xe2c73c7b, 61 },
    { 0x68c73c7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c973c80, 66 },
    { 0x7fc73c80, 52 },
    { 0x66b73c81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x31673c85, 33 },
    { 0xdca73c86, 76 },
    { 0, 0 },
    { 0x36573c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64773c8d, 43 },
    { 0xa2373c8e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51873c92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a73c96, 61 },
    { 0xc5a73c96, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3273c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a73c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea73ca3, 67 },
    { 0x5cd73ca4, 90 },
    { 0x5b73ca5, 66 },
    { 0, 0 },
    { 0x7fc73ca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7173cab, 78 },
    { 0x32b73cab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a73caf, 33 },
    { 0x26d73caf, 91 },
    { 0, 0 },
    { 0xca073cb2, 33 },
    { 0, 0 },
    { 0xb0d73cb4, 98 },
    { 0, 0 },
    { 0x96373cb6, 95 },
    { 0x6d673cb6, 90 },
    { 0xa7773cb8, 76 },
    { 0xfe473cb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b773cbd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc73cc1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8973cc5, 90 },
    { 0, 0 },
    { 0x6e73cc7, 95 },
    { 0, 0 },
    { 0x1b873cc9, 85 },
    { 0, 0 },
    { 0x3373ccb, 76 },
    { 0, 0 },
    { 0xc3a73ccd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48473cd6, 61 },
    { 0x73a73cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73973cda, 91 },
    { 0xf873cda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb73ce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24973ce5, 95 },
    { 0, 0 },
    { 0xf673ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c73cea, 98 },
    { 0x86473cea, 90 },
    { 0x89b73ceb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59973cf0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91573cf4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac73cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b573d04, 75 },
    { 0xb6c73d05, 68 },
    { 0x9673d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57873d09, 33 },
    { 0, 0 },
    { 0xe0673d0b, 43 },
    { 0xb6773d0c, 66 },
    { 0, 0 },
    { 0x5c773d0e, 99 },
    { 0, 0 },
    { 0xa2573d10, 61 },
    { 0, 0 },
    { 0xe0b73d12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5473d17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a673d1b, 33 },
    { 0xca173d1c, 84 },
    { 0xba273d1d, 91 },
    { 0, 0 },
    { 0xdfc73d1f, 90 },
    { 0, 0 },
    { 0xcd373d21, 95 },
    { 0, 0 },
    { 0x6473d23, 61 },
    { 0xb2373d23, 87 },
    { 0x32373d25, 95 },
    { 0xbe673d25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d73d2c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4573d2f, 98 },
    { 0x44873d30, 33 },
    { 0x81b73d30, 34 },
    { 0x79373d30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed73d36, 76 },
    { 0x25f73d37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8973d3a, 46 },
    { 0x6d873d3b, 33 },
    { 0xe4273d3b, 83 },
    { 0xa2573d3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf73d44, 90 },
    { 0x3a673d45, 61 },
    { 0xb2373d45, 97 },
    { 0x14273d47, 66 },
    { 0x87873d45, 33 },
    { 0xc4073d46, 95 },
    { 0xeab73d4a, 90 },
    { 0x87973d4b, 88 },
    { 0x51473d4c, 95 },
    { 0x81973d4d, 43 },
    { 0x50273d4e, 55 },
    { 0x9cf73d4f, 78 },
    { 0x88873d50, 17 },
    { 0xb7b73d50, 66 },
    { 0, 0 },
    { 0x69b73d53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0073d56, 23 },
    { 0x7b173d57, 36 },
    { 0, 0 },
    { 0xdf273d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1873d5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93873d62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d73d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe73d70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae073d77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e73d7a, 76 },
    { 0x4cf73d7a, 85 },
    { 0x45f73d7c, 34 },
    { 0x14a73d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e73d88, 40 },
    { 0x18073d89, 93 },
    { 0xbb773d8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4773d8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda73d92, 90 },
    { 0x4f373d92, 99 },
    { 0x5cb73d94, 17 },
    { 0, 0 },
    { 0x17b73d96, 61 },
    { 0, 0 },
    { 0xcc373d98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d773d9b, 95 },
    { 0xe6d73d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb473da7, 33 },
    { 0, 0 },
    { 0x5dd73da9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a173dad, 91 },
    { 0, 0 },
    { 0xaba73daf, 98 },
    { 0x47f73db0, 90 },
    { 0xa9173db1, 90 },
    { 0, 0 },
    { 0xdeb73db3, 85 },
    { 0, 0 },
    { 0xcfb73db5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39473dba, 55 },
    { 0x63673dba, 61 },
    { 0, 0 },
    { 0xd3f73dbd, 61 },
    { 0xfcd73dbd, 98 },
    { 0x50d73dbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da73dc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb473dc6, 43 },
    { 0xbfd73dc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee73dcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba73dcf, 95 },
    { 0x9ca73dd0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff73dd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c373dd9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca073de2, 76 },
    { 0xc6173de3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd373de6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3673dea, 85 },
    { 0xe2773deb, 95 },
    { 0x4d273dec, 87 },
    { 0x85d73deb, 95 },
    { 0x27a73dee, 61 },
    { 0xb5173def, 55 },
    { 0x1ec73df0, 33 },
    { 0xb8f73df1, 55 },
    { 0, 0 },
    { 0xceb73df3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb473df6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa573e00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xed473e03, 97 },
    { 0, 0 },
    { 0x16a73e05, 88 },
    { 0x1e073e05, 76 },
    { 0xdff73e05, 84 },
    { 0xa5f73e07, 78 },
    { 0xca373e05, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d73e0c, 98 },
    { 0xe1f73e0c, 98 },
    { 0x59b73e0e, 76 },
    { 0, 0 },
    { 0xbf273e10, 90 },
    { 0, 0 },
    { 0xce973e12, 61 },
    { 0x89e73e13, 90 },
    { 0x7dc73e14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf73e1a, 95 },
    { 0x73673e1b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4573e23, 55 },
    { 0, 0 },
    { 0x6c873e25, 98 },
    { 0, 0 },
    { 0x90973e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a73e2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d73e2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5873e30, 90 },
    { 0xc9473e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a73e35, 34 },
    { 0, 0 },
    { 0x89073e37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa73e3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92673e43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c673e46, 23 },
    { 0x4ef73e46, 84 },
    { 0, 0 },
    { 0x60f73e49, 90 },
    { 0, 0 },
    { 0x2cf73e4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c773e50, 98 },
    { 0xfcf73e51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d673e54, 23 },
    { 0xf9a73e55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1573e5b, 98 },
    { 0x99873e5c, 53 },
    { 0x8da73e5c, 93 },
    { 0x2e073e5e, 34 },
    { 0xef473e5e, 17 },
    { 0xec73e5f, 90 },
    { 0xcc973e5d, 99 },
    { 0x93e73e62, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc73e6c, 23 },
    { 0x12d73e6d, 85 },
    { 0x44373e6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a373e72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef73e78, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7773e7f, 95 },
    { 0, 0 },
    { 0x38273e81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13673e86, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a273e8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf73e95, 100 },
    { 0xe4973e96, 76 },
    { 0xda073e97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4273e9c, 52 },
    { 0xfa673e9c, 33 },
    { 0xcd273e9e, 43 },
    { 0x7bc73e9e, 90 },
    { 0x4e473e9e, 87 },
    { 0x8dd73ea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69173eaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70273eb3, 98 },
    { 0x3e973eb3, 95 },
    { 0xc8573eb5, 90 },
    { 0x47d73eb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f73ec0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x29173ec3, 76 },
    { 0x66573ec3, 95 },
    { 0x8b373ec5, 95 },
    { 0xfbc73ec5, 33 },
    { 0, 0 },
    { 0xe0973ec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb73ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b73ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5273ed6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88173ee0, 98 },
    { 0x46173ee1, 23 },
    { 0xd7773ee0, 90 },
    { 0x83c73ee3, 33 },
    { 0, 0 },
    { 0x48973ee5, 19 },
    { 0, 0 },
    { 0xedb73ee7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f73eed, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84373ef1, 24 },
    { 0x4c73ef1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b73ef5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76273ef9, 95 },
    { 0, 0 },
    { 0xee973efb, 55 },
    { 0, 0 },
    { 0x49973efd, 46 },
    { 0xac173efd, 76 },
    { 0x68b73eff, 66 },
    { 0x15a73f00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46373f07, 55 },
    { 0x46773f08, 98 },
    { 0x66e73f07, 95 },
    { 0, 0 },
    { 0x2dd73f0b, 76 },
    { 0, 0 },
    { 0xd5d73f0d, 61 },
    { 0x3ef73f0e, 76 },
    { 0, 0 },
    { 0x8773f10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b73f14, 90 },
    { 0x60173f15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24173f1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc73f1d, 23 },
    { 0, 0 },
    { 0xf6373f1f, 23 },
    { 0x5ee73f1f, 90 },
    { 0, 0 },
    { 0x21e73f22, 23 },
    { 0x48f73f23, 75 },
    { 0x5d173f24, 99 },
    { 0xc4b73f25, 33 },
    { 0x5b273f24, 90 },
    { 0x78873f23, 46 },
    { 0, 0 },
    { 0x3873f29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9373f2c, 55 },
    { 0xcf173f2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b73f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1673f34, 61 },
    { 0x37773f34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7573f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e573f3b, 95 },
    { 0x1f173f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd473f42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a73f45, 90 },
    { 0x2c673f46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a573f49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2973f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8573f55, 61 },
    { 0x15473f56, 90 },
    { 0x2c973f56, 95 },
    { 0xc0773f58, 33 },
    { 0x22d73f58, 76 },
    { 0xc9073f59, 55 },
    { 0x83873f5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e73f60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77773f63, 100 },
    { 0xa5573f63, 95 },
    { 0, 0 },
    { 0x9c973f66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6073f6b, 46 },
    { 0x5673f6c, 95 },
    { 0x3f573f6d, 61 },
    { 0x54273f6e, 95 },
    { 0x43b73f6f, 61 },
    { 0x2c73f6f, 90 },
    { 0x76873f70, 90 },
    { 0, 0 },
    { 0x6d873f73, 81 },
    { 0x77d73f74, 34 },
    { 0, 0 },
    { 0x94f73f76, 81 },
    { 0, 0 },
    { 0x6bb73f78, 98 },
    { 0x8b073f79, 61 },
    { 0xcfa73f7a, 95 },
    { 0x28f73f78, 33 },
    { 0x10273f7c, 98 },
    { 0x4173f7d, 76 },
    { 0xfa673f7d, 95 },
    { 0, 0 },
    { 0x23373f80, 95 },
    { 0x60b73f80, 98 },
    { 0, 0 },
    { 0x2b773f83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99373f8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6073f8e, 33 },
    { 0, 0 },
    { 0x52473f90, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5673f96, 87 },
    { 0x88073f96, 98 },
    { 0x80573f98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec73f9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60273fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8473fa7, 61 },
    { 0, 0 },
    { 0xb5673fa9, 98 },
    { 0x28173faa, 98 },
    { 0x24973fab, 95 },
    { 0, 0 },
    { 0xcce73fad, 64 },
    { 0, 0 },
    { 0x5aa73faf, 46 },
    { 0, 0 },
    { 0xe0c73fb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c73fb6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86373fbc, 24 },
    { 0xf0f73fbd, 95 },
    { 0xf0e73fbc, 33 },
    { 0x90473fbf, 98 },
    { 0x68473fc0, 52 },
    { 0xd4373fc1, 61 },
    { 0x65e73fc1, 91 },
    { 0x4dd73fc2, 76 },
    { 0x3373fc4, 66 },
    { 0x36c73fc2, 95 },
    { 0x47973fc3, 46 },
    { 0x9ae73fc2, 95 },
    { 0xc7073fc8, 78 },
    { 0x5e973fc7, 33 },
    { 0xf6273fc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb73fcd, 95 },
    { 0xe7b73fce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85973fd4, 61 },
    { 0xf2473fd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c73fdd, 23 },
    { 0x64773fde, 34 },
    { 0xef073fdf, 23 },
    { 0xba173fdf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89273fe6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b73fec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66173fef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34473ff5, 34 },
    { 0xab873ff5, 76 },
    { 0xe2873ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba73ffb, 33 },
    { 0, 0 },
    { 0xab973ffd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd74003, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097400f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f074012, 91 },
    { 0, 0 },
    { 0xff374014, 86 },
    { 0x4fd74015, 33 },
    { 0x4a774014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9057401b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6797401e, 68 },
    { 0xdd7401f, 99 },
    { 0, 0 },
    { 0x39374021, 34 },
    { 0, 0 },
    { 0xaf374023, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b87402c, 46 },
    { 0xa527402d, 90 },
    { 0xe897402e, 95 },
    { 0, 0 },
    { 0xb4b74030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d574033, 33 },
    { 0xe0874034, 23 },
    { 0x75974034, 33 },
    { 0, 0 },
    { 0xe1074037, 17 },
    { 0x81174038, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf727403d, 61 },
    { 0x8657403d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d74043, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff74049, 90 },
    { 0x93c7404a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef74051, 61 },
    { 0xdad74052, 88 },
    { 0xd8274052, 98 },
    { 0x2f474052, 23 },
    { 0xe3874054, 46 },
    { 0x71974056, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b74059, 76 },
    { 0x4274059, 99 },
    { 0xc347405b, 93 },
    { 0x6ce7405c, 61 },
    { 0xd1e7405b, 85 },
    { 0x24e7405d, 90 },
    { 0xe97405e, 33 },
    { 0xad27405f, 34 },
    { 0x9127405e, 65 },
    { 0xf0374062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb874066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d774070, 43 },
    { 0, 0 },
    { 0x8c974072, 66 },
    { 0xb6274073, 95 },
    { 0xa0174073, 95 },
    { 0x83274075, 33 },
    { 0x81a74076, 61 },
    { 0x5cf74076, 33 },
    { 0xa5574078, 55 },
    { 0xba874079, 90 },
    { 0xc2f74077, 23 },
    { 0xef374075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac174082, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb774085, 95 },
    { 0xafc74086, 98 },
    { 0xf6b74087, 95 },
    { 0x22274088, 33 },
    { 0x71474089, 52 },
    { 0xc774089, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c7408d, 61 },
    { 0x9657408e, 40 },
    { 0xe6b7408f, 23 },
    { 0, 0 },
    { 0x2dc74091, 23 },
    { 0x42374092, 33 },
    { 0x7c474093, 98 },
    { 0x45a74093, 92 },
    { 0xc74095, 23 },
    { 0xde074094, 90 },
    { 0x2ce74094, 61 },
    { 0, 0 },
    { 0xffb74099, 76 },
    { 0, 0 },
    { 0x9377409b, 61 },
    { 0x98f7409c, 90 },
    { 0x5517409c, 95 },
    { 0x41e7409e, 61 },
    { 0xea17409d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4740a6, 69 },
    { 0x5fe740a7, 85 },
    { 0xacf740a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6740ab, 61 },
    { 0, 0 },
    { 0xb2e740ad, 61 },
    { 0x2b2740ae, 99 },
    { 0, 0 },
    { 0x4bb740b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x136740b3, 55 },
    { 0, 0 },
    { 0xa48740b5, 98 },
    { 0xc1e740b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe740ba, 33 },
    { 0xf4c740bb, 33 },
    { 0, 0 },
    { 0x389740bd, 77 },
    { 0, 0 },
    { 0x819740bf, 76 },
    { 0xd7a740c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc740c3, 98 },
    { 0x6f4740c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x171740c7, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3740cd, 78 },
    { 0x52c740ce, 90 },
    { 0x731740cf, 90 },
    { 0x405740cf, 90 },
    { 0, 0 },
    { 0x226740d2, 66 },
    { 0xd68740d3, 85 },
    { 0x78a740d4, 93 },
    { 0xdb740d5, 33 },
    { 0, 0 },
    { 0x796740d7, 61 },
    { 0x852740d7, 93 },
    { 0xb97740d9, 61 },
    { 0x8c7740d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x441740dd, 99 },
    { 0x9fa740dd, 33 },
    { 0xa23740df, 90 },
    { 0x528740e0, 90 },
    { 0x972740e0, 90 },
    { 0, 0 },
    { 0xe59740e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3740e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08740ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164740f1, 77 },
    { 0, 0 },
    { 0xefd740f3, 91 },
    { 0, 0 },
    { 0x39d740f5, 23 },
    { 0x12e740f5, 61 },
    { 0xbae740f6, 90 },
    { 0xb56740f7, 90 },
    { 0x1dd740f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf74104, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa527410b, 90 },
    { 0xfe97410c, 76 },
    { 0x4967410d, 98 },
    { 0, 0 },
    { 0xf657410f, 98 },
    { 0x35474110, 95 },
    { 0x41974110, 33 },
    { 0, 0 },
    { 0x56674113, 23 },
    { 0xa8b74114, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9547411e, 61 },
    { 0, 0 },
    { 0x86974120, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c74123, 95 },
    { 0x30774124, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af74131, 78 },
    { 0xebc74132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb174135, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba7413a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc707413e, 46 },
    { 0xc257413f, 61 },
    { 0x87874140, 23 },
    { 0xb5474141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c74144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3474147, 78 },
    { 0, 0 },
    { 0x30974149, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea57414d, 77 },
    { 0, 0 },
    { 0x48e7414f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8417415b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e74165, 91 },
    { 0xc2974165, 90 },
    { 0x49174167, 23 },
    { 0xf7874166, 33 },
    { 0xbcb74168, 55 },
    { 0, 0 },
    { 0xd587416b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd407416f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d74173, 93 },
    { 0xd1274174, 55 },
    { 0xc6a74174, 98 },
    { 0x23c74176, 66 },
    { 0x90474174, 98 },
    { 0x8af74178, 61 },
    { 0, 0 },
    { 0x5187417a, 98 },
    { 0, 0 },
    { 0x6857417c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc77417f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d74185, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3574188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57418e, 43 },
    { 0x7a57418f, 61 },
    { 0x28c7418e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f674198, 61 },
    { 0xba274199, 65 },
    { 0xd3e7419a, 33 },
    { 0xfd67419b, 90 },
    { 0x3af7419c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366741a0, 33 },
    { 0x928741a1, 61 },
    { 0xfc0741a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e741a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625741aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd741ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa741b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x208741b3, 90 },
    { 0, 0 },
    { 0xaec741b5, 61 },
    { 0x2d3741b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae741b9, 95 },
    { 0, 0 },
    { 0x9d9741bb, 98 },
    { 0x86c741bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b741bf, 55 },
    { 0, 0 },
    { 0xbc0741c1, 85 },
    { 0xe56741c2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7741c5, 95 },
    { 0, 0 },
    { 0xfa5741c7, 78 },
    { 0x59741c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d741cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7741cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c741d3, 34 },
    { 0xf2e741d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7741d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851741e0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb741e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282741ee, 46 },
    { 0x3df741ef, 76 },
    { 0x31741f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35741f3, 23 },
    { 0, 0 },
    { 0x777741f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488741f9, 85 },
    { 0x6cf741f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x686741fd, 95 },
    { 0x264741fe, 90 },
    { 0x1a3741ff, 55 },
    { 0, 0 },
    { 0x7a74201, 98 },
    { 0, 0 },
    { 0xae274203, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb74209, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf107420e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d74214, 61 },
    { 0, 0 },
    { 0x46e74216, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae74219, 91 },
    { 0x3c74219, 55 },
    { 0xbc674219, 33 },
    { 0x91f7421c, 61 },
    { 0x4257421d, 64 },
    { 0x1937421a, 100 },
    { 0x2477421c, 61 },
    { 0, 0 },
    { 0xf4c74221, 90 },
    { 0x28174222, 61 },
    { 0, 0 },
    { 0x45b74224, 98 },
    { 0x5c74224, 98 },
    { 0x1bf74226, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f7422c, 90 },
    { 0xc807422d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba74231, 33 },
    { 0, 0 },
    { 0x84674233, 85 },
    { 0, 0 },
    { 0x81574235, 34 },
    { 0xf274235, 33 },
    { 0x95e74237, 90 },
    { 0x1ef74235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d7423f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c274244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa307424d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a74258, 100 },
    { 0x8aa74258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf7425d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca74262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c74265, 46 },
    { 0x6e974266, 33 },
    { 0x9aa74267, 51 },
    { 0xedb74266, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f7426b, 76 },
    { 0, 0 },
    { 0xccd7426d, 61 },
    { 0x8417426e, 23 },
    { 0x3cd7426f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d74274, 90 },
    { 0x4dd74274, 76 },
    { 0x1d174275, 90 },
    { 0xb6d74276, 46 },
    { 0xd374274, 76 },
    { 0, 0 },
    { 0xbae7427a, 23 },
    { 0, 0 },
    { 0xdf7427c, 55 },
    { 0xc427427c, 95 },
    { 0x2c87427e, 75 },
    { 0x6777427f, 95 },
    { 0xee274280, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb674285, 60 },
    { 0xbba74286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae7428a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb74291, 61 },
    { 0x45a74292, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf074295, 61 },
    { 0, 0 },
    { 0x6be74297, 33 },
    { 0x3b874298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f7429e, 33 },
    { 0xce37429f, 98 },
    { 0xa99742a0, 61 },
    { 0xed7742a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913742a7, 23 },
    { 0, 0 },
    { 0xa76742a9, 90 },
    { 0x69c742aa, 99 },
    { 0x2a0742ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6742b5, 90 },
    { 0x3ab742b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7742ba, 91 },
    { 0x705742bb, 33 },
    { 0, 0 },
    { 0xf73742bd, 34 },
    { 0, 0 },
    { 0xc8a742bf, 91 },
    { 0x397742c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba742cc, 78 },
    { 0x17f742cd, 23 },
    { 0, 0 },
    { 0x31d742cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9742d3, 68 },
    { 0x7f4742d3, 23 },
    { 0, 0 },
    { 0xf9a742d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8742d9, 17 },
    { 0x5d0742da, 99 },
    { 0, 0 },
    { 0x815742dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53742df, 98 },
    { 0x3f2742e0, 33 },
    { 0x58a742e1, 55 },
    { 0, 0 },
    { 0xfdb742e3, 65 },
    { 0, 0 },
    { 0x1ab742e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7742e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4742f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee742f8, 91 },
    { 0x6ce742f8, 87 },
    { 0xc59742f8, 91 },
    { 0x6ed742fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69742fe, 8 },
    { 0, 0 },
    { 0x2ab74300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a74303, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c74306, 77 },
    { 0, 0 },
    { 0x97474308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e374311, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b77431a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267431e, 33 },
    { 0xbe17431f, 23 },
    { 0x40d74320, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc74325, 55 },
    { 0, 0 },
    { 0x55074327, 76 },
    { 0x40974327, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd37432b, 61 },
    { 0, 0 },
    { 0x3b47432d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8174334, 61 },
    { 0, 0 },
    { 0xcfb74336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7433a, 90 },
    { 0x2b57433b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7407433f, 81 },
    { 0, 0 },
    { 0xf4c74341, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74349, 61 },
    { 0, 0 },
    { 0x22b7434b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d374355, 90 },
    { 0xeae74356, 46 },
    { 0x44f74355, 98 },
    { 0x97074358, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7435b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83974365, 90 },
    { 0x34174366, 33 },
    { 0, 0 },
    { 0x33b74368, 90 },
    { 0xfd074369, 33 },
    { 0x13574368, 98 },
    { 0xc077436b, 84 },
    { 0x1ad7436c, 46 },
    { 0x2627436c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4274370, 98 },
    { 0xf5174370, 33 },
    { 0xe74371, 95 },
    { 0x21f74372, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b74376, 84 },
    { 0xd6274377, 61 },
    { 0xb7374377, 98 },
    { 0x49b74376, 98 },
    { 0xb407437a, 24 },
    { 0xeb27437a, 61 },
    { 0xaad7437c, 23 },
    { 0x8d07437d, 61 },
    { 0x7b27437a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af74387, 95 },
    { 0, 0 },
    { 0x25674389, 61 },
    { 0x71b74389, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f7438f, 90 },
    { 0x81b74390, 93 },
    { 0xf2374391, 90 },
    { 0, 0 },
    { 0xf4c74393, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b7439a, 76 },
    { 0, 0 },
    { 0x84b7439c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9743a2, 40 },
    { 0, 0 },
    { 0x70c743a4, 87 },
    { 0xb37743a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3743ab, 55 },
    { 0x76a743ac, 46 },
    { 0, 0 },
    { 0x81a743ae, 93 },
    { 0xc36743af, 34 },
    { 0, 0 },
    { 0x6b0743b1, 34 },
    { 0xa66743b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9743b9, 33 },
    { 0, 0 },
    { 0x40e743bb, 43 },
    { 0xd0f743bc, 33 },
    { 0xba1743bd, 33 },
    { 0xda743be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743743c3, 76 },
    { 0, 0 },
    { 0xf2d743c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984743cd, 90 },
    { 0, 0 },
    { 0xb00743cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23743d2, 76 },
    { 0, 0 },
    { 0x43f743d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4743d7, 95 },
    { 0xe8b743d8, 43 },
    { 0, 0 },
    { 0x420743da, 95 },
    { 0x7f6743db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf743e5, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede743ec, 90 },
    { 0xa71743ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd743f7, 93 },
    { 0xc4c743f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f743fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f74400, 91 },
    { 0xbca74400, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd774405, 61 },
    { 0, 0 },
    { 0x90674407, 61 },
    { 0x37f74408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8457440d, 33 },
    { 0x5b87440e, 43 },
    { 0, 0 },
    { 0xdda74410, 33 },
    { 0xdf474410, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d74414, 75 },
    { 0, 0 },
    { 0x8df74416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31474419, 98 },
    { 0x9fc7441a, 77 },
    { 0x59f7441a, 76 },
    { 0x7d47441c, 46 },
    { 0xc437441a, 90 },
    { 0x4d7441e, 78 },
    { 0x7ee7441b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x77974422, 33 },
    { 0x40b74422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb74428, 47 },
    { 0xdd274428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb7442c, 23 },
    { 0x5e87442d, 61 },
    { 0xb977442c, 55 },
    { 0x1487442d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92574435, 66 },
    { 0, 0 },
    { 0x6c474437, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a7443a, 88 },
    { 0xb37443a, 34 },
    { 0x6c67443c, 34 },
    { 0, 0 },
    { 0x8697443e, 58 },
    { 0xf547443e, 33 },
    { 0x40774440, 23 },
    { 0x2f474440, 90 },
    { 0xd5474441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc974446, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa947444d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d7445a, 77 },
    { 0x3177445a, 46 },
    { 0xd707445a, 98 },
    { 0, 0 },
    { 0x2567445e, 99 },
    { 0x7d27445f, 36 },
    { 0xe3d74460, 99 },
    { 0x34c74460, 23 },
    { 0xdc74462, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917446c, 95 },
    { 0x8227446d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91474471, 90 },
    { 0x9ec74472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a74478, 33 },
    { 0x70574479, 33 },
    { 0x1167447a, 33 },
    { 0x9477447b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c7447e, 98 },
    { 0x67b7447e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d474485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e7448b, 66 },
    { 0x4057448c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a974490, 95 },
    { 0, 0 },
    { 0x76774492, 52 },
    { 0xdbe74493, 77 },
    { 0x36074494, 95 },
    { 0xf7674495, 87 },
    { 0, 0 },
    { 0x1a174497, 23 },
    { 0xd8074498, 98 },
    { 0xf3d74498, 95 },
    { 0x7657449a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4017449e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x168744a1, 33 },
    { 0xb72744a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e744ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6744b2, 34 },
    { 0, 0 },
    { 0xb2a744b4, 100 },
    { 0x356744b5, 61 },
    { 0x234744b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40744be, 33 },
    { 0xa84744be, 90 },
    { 0xd66744be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729744c9, 55 },
    { 0xba3744ca, 61 },
    { 0, 0 },
    { 0x8e6744cc, 76 },
    { 0, 0 },
    { 0x149744ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3744d2, 23 },
    { 0xdfa744d3, 61 },
    { 0x2f2744d2, 90 },
    { 0xc33744d4, 46 },
    { 0, 0 },
    { 0x406744d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54744e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f744e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x323744ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae744f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b744f6, 98 },
    { 0xb35744f7, 97 },
    { 0xe27744f7, 33 },
    { 0, 0 },
    { 0x916744fa, 91 },
    { 0xe08744fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x260744fe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af74502, 23 },
    { 0x69974502, 98 },
    { 0xf9274504, 98 },
    { 0x2a474505, 90 },
    { 0x93474506, 81 },
    { 0x63c74507, 55 },
    { 0, 0 },
    { 0x10c74509, 88 },
    { 0xf0e7450a, 67 },
    { 0x45c7450b, 61 },
    { 0x3fc7450b, 33 },
    { 0xa557450a, 95 },
    { 0xfce7450b, 98 },
    { 0x1447450e, 85 },
    { 0xc1674509, 61 },
    { 0x42f7450d, 33 },
    { 0xa6574512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a74516, 98 },
    { 0xd4474517, 34 },
    { 0xc5e74518, 76 },
    { 0xe6c74519, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4767451d, 46 },
    { 0x587451e, 98 },
    { 0x6a97451f, 17 },
    { 0, 0 },
    { 0x4f274521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1974526, 17 },
    { 0x3a474527, 98 },
    { 0, 0 },
    { 0xb3174529, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9367452e, 78 },
    { 0xe4c7452f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30474539, 55 },
    { 0x357453a, 46 },
    { 0xa127453b, 98 },
    { 0xbdb7453c, 100 },
    { 0x3527453a, 33 },
    { 0x3c57453a, 84 },
    { 0x9a67453f, 90 },
    { 0x50074540, 78 },
    { 0, 0 },
    { 0x87774542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc87454b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d74550, 76 },
    { 0, 0 },
    { 0x35074552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed74556, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e7455e, 23 },
    { 0, 0 },
    { 0xc4274560, 95 },
    { 0, 0 },
    { 0x7e274562, 88 },
    { 0x68e74563, 95 },
    { 0, 0 },
    { 0xc1b74565, 17 },
    { 0, 0 },
    { 0x69974567, 95 },
    { 0, 0 },
    { 0xc0f74569, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf417456c, 90 },
    { 0x95e7456d, 84 },
    { 0xd087456e, 98 },
    { 0x357456f, 95 },
    { 0, 0 },
    { 0x8f374571, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab74574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb74579, 23 },
    { 0xc2a7457a, 43 },
    { 0, 0 },
    { 0xc77457c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8097457f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627458b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd47458f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63374593, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d7459c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71745ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6745b3, 67 },
    { 0xc5a745b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4745ba, 55 },
    { 0x3ac745bb, 23 },
    { 0x6fb745bc, 55 },
    { 0x300745bd, 93 },
    { 0, 0 },
    { 0x138745bf, 93 },
    { 0xa29745c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659745c5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6745cb, 23 },
    { 0xd58745cc, 43 },
    { 0, 0 },
    { 0xbe3745ce, 61 },
    { 0x5be745ce, 93 },
    { 0xf62745d0, 95 },
    { 0xf745ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4745d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160745db, 76 },
    { 0xcf1745db, 95 },
    { 0xd4e745dc, 95 },
    { 0x21b745de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0745e4, 90 },
    { 0xef5745e4, 23 },
    { 0x5be745e6, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4745f2, 98 },
    { 0x7c745f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442745fe, 93 },
    { 0xac0745ff, 98 },
    { 0x5a5745ff, 52 },
    { 0, 0 },
    { 0xcba74602, 93 },
    { 0xc4a74602, 95 },
    { 0x3c174604, 95 },
    { 0, 0 },
    { 0x9c474606, 90 },
    { 0, 0 },
    { 0x68974608, 95 },
    { 0, 0 },
    { 0x25a7460a, 91 },
    { 0xd967460b, 55 },
    { 0xecd7460a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d7460f, 78 },
    { 0xb7d74610, 98 },
    { 0x7b574611, 46 },
    { 0, 0 },
    { 0xe5474613, 91 },
    { 0xb2d74614, 46 },
    { 0x59674614, 84 },
    { 0, 0 },
    { 0x3e474617, 65 },
    { 0x5d074618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4797461b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf467461f, 98 },
    { 0, 0 },
    { 0x88474621, 77 },
    { 0x5e74622, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f74625, 69 },
    { 0, 0 },
    { 0xf9174627, 51 },
    { 0, 0 },
    { 0x19574629, 46 },
    { 0x9627462a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07462f, 52 },
    { 0x93f7462f, 61 },
    { 0x50874630, 98 },
    { 0x1f674632, 55 },
    { 0x9c774630, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb74637, 90 },
    { 0x13074638, 98 },
    { 0xade74639, 100 },
    { 0x8017463a, 93 },
    { 0x9d77463b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb87463e, 66 },
    { 0x5627463f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a74648, 55 },
    { 0x75574649, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d7464d, 55 },
    { 0x8ef7464e, 90 },
    { 0xee77464d, 55 },
    { 0xd167464d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b74658, 55 },
    { 0x9c574659, 33 },
    { 0xabb7465a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3757465d, 99 },
    { 0x1177465d, 23 },
    { 0x3b47465f, 61 },
    { 0x6c77465f, 100 },
    { 0xd4674661, 90 },
    { 0xfb574662, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa74669, 85 },
    { 0x4377466a, 95 },
    { 0xc2f7466a, 33 },
    { 0, 0 },
    { 0x61a7466d, 95 },
    { 0, 0 },
    { 0x1197466f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc974673, 33 },
    { 0x4f974673, 64 },
    { 0xd7d74673, 95 },
    { 0xccf74676, 55 },
    { 0x9ca74677, 33 },
    { 0x2d74678, 61 },
    { 0x19774679, 66 },
    { 0xd07467a, 90 },
    { 0x48c7467a, 76 },
    { 0xb877467c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11374685, 55 },
    { 0, 0 },
    { 0xc4874687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657468b, 88 },
    { 0xa4f7468c, 66 },
    { 0xc737468d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5974692, 90 },
    { 0x6a574692, 33 },
    { 0x7e374694, 8 },
    { 0x44074692, 95 },
    { 0x63974695, 88 },
    { 0x5cf74697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547469a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb957469e, 99 },
    { 0x1a07469f, 76 },
    { 0x57a7469f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45746a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4746ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6746b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772746b8, 98 },
    { 0x4b7746b8, 90 },
    { 0x7746b9, 91 },
    { 0xbc0746ba, 33 },
    { 0xe57746b8, 33 },
    { 0x851746b8, 95 },
    { 0x850746b8, 46 },
    { 0x265746b9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0746c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146746ca, 95 },
    { 0x16a746cb, 68 },
    { 0xe1b746cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0746cf, 43 },
    { 0, 0 },
    { 0xc36746d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348746d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x896746d8, 90 },
    { 0, 0 },
    { 0x1c0746da, 90 },
    { 0, 0 },
    { 0x887746dc, 98 },
    { 0xdf9746dc, 100 },
    { 0, 0 },
    { 0xf8e746df, 23 },
    { 0, 0 },
    { 0x7c7746e1, 98 },
    { 0xe6a746e2, 23 },
    { 0x91a746e3, 88 },
    { 0x5746e2, 33 },
    { 0xd88746e4, 66 },
    { 0xbd4746e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28746ec, 61 },
    { 0, 0 },
    { 0x503746ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662746f4, 33 },
    { 0x9e746f5, 90 },
    { 0, 0 },
    { 0x9a6746f7, 98 },
    { 0, 0 },
    { 0xb00746f9, 66 },
    { 0, 0 },
    { 0x324746fb, 84 },
    { 0xc46746fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f746ff, 61 },
    { 0xb83746ff, 33 },
    { 0x97974701, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b74705, 65 },
    { 0x10a74706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b7470a, 61 },
    { 0x8d7470b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a574714, 90 },
    { 0x8c974715, 95 },
    { 0xc874714, 98 },
    { 0xa4074714, 61 },
    { 0, 0 },
    { 0xdbe74719, 33 },
    { 0, 0 },
    { 0x2757471b, 77 },
    { 0x43b7471c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f874725, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e7472a, 91 },
    { 0x4267472b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37574730, 34 },
    { 0x3e674731, 95 },
    { 0x32f74731, 95 },
    { 0, 0 },
    { 0xbc74734, 23 },
    { 0xe5574734, 61 },
    { 0x5c074734, 55 },
    { 0, 0 },
    { 0x7f74738, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17474740, 98 },
    { 0x75b74741, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d74746, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a7474b, 33 },
    { 0xcf17474c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10574751, 52 },
    { 0x14774751, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9417476b, 90 },
    { 0x7727476c, 93 },
    { 0x3ae7476c, 33 },
    { 0xe0b7476c, 51 },
    { 0, 0 },
    { 0xda574770, 33 },
    { 0xfdc74771, 98 },
    { 0x1a674770, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe74775, 100 },
    { 0, 0 },
    { 0xcc674777, 76 },
    { 0x3d474778, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f7477d, 38 },
    { 0, 0 },
    { 0x3b37477f, 48 },
    { 0x2f874780, 95 },
    { 0, 0 },
    { 0x1a674782, 61 },
    { 0, 0 },
    { 0xcf374784, 77 },
    { 0xa8574785, 33 },
    { 0xd4b74786, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea17478d, 95 },
    { 0xd7c7478e, 100 },
    { 0, 0 },
    { 0x25274790, 98 },
    { 0x2c174791, 81 },
    { 0x4d774791, 87 },
    { 0, 0 },
    { 0xcd074794, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4887479b, 43 },
    { 0x34c7479c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8747a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2747a7, 98 },
    { 0xd8b747a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x532747ab, 87 },
    { 0, 0 },
    { 0xed9747ad, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9747b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46747b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86747b9, 68 },
    { 0xe4a747ba, 61 },
    { 0x13e747b9, 43 },
    { 0x637747bc, 36 },
    { 0xdd747b9, 93 },
    { 0x705747b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e747cb, 17 },
    { 0x8cf747cb, 55 },
    { 0x26747cd, 34 },
    { 0x7f9747cd, 98 },
    { 0x410747cf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3747d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2747d6, 90 },
    { 0xd2b747d7, 88 },
    { 0x333747d8, 23 },
    { 0xc5747d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161747df, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed747e6, 93 },
    { 0x686747e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6747ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20747f0, 33 },
    { 0, 0 },
    { 0x458747f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f747f5, 61 },
    { 0xab3747f6, 67 },
    { 0, 0 },
    { 0xa00747f8, 90 },
    { 0x508747f9, 67 },
    { 0xbe2747fa, 75 },
    { 0x336747fb, 55 },
    { 0xfa6747fc, 66 },
    { 0x6bb747fb, 66 },
    { 0xd7e747fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa74801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b74804, 90 },
    { 0, 0 },
    { 0xc0274806, 90 },
    { 0x57774807, 98 },
    { 0, 0 },
    { 0x3e174809, 55 },
    { 0xcaf7480a, 95 },
    { 0, 0 },
    { 0x6f87480c, 61 },
    { 0xd0b7480d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6074810, 81 },
    { 0x3674811, 84 },
    { 0xdb874812, 98 },
    { 0, 0 },
    { 0x95174814, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117481a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a7481e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca574822, 91 },
    { 0x1e374822, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb357482a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf074830, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb874834, 93 },
    { 0, 0 },
    { 0xd0a74836, 95 },
    { 0x15174837, 55 },
    { 0xb8d74836, 33 },
    { 0xa7874839, 99 },
    { 0x86a7483a, 88 },
    { 0x2d74838, 61 },
    { 0, 0 },
    { 0xf077483d, 88 },
    { 0x3d7483d, 95 },
    { 0xae7483f, 100 },
    { 0xa3174840, 55 },
    { 0xedf74841, 33 },
    { 0, 0 },
    { 0xaa174843, 100 },
    { 0x7bd74843, 75 },
    { 0, 0 },
    { 0xda074846, 40 },
    { 0, 0 },
    { 0xcc774848, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e7484b, 95 },
    { 0x5c97484b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7484f, 33 },
    { 0xad974850, 93 },
    { 0, 0 },
    { 0x8c074852, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf74855, 61 },
    { 0x1ae74856, 61 },
    { 0xe7774857, 81 },
    { 0x38d74858, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4227485c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d974860, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df74865, 78 },
    { 0x27774866, 90 },
    { 0xd9b74866, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb7486c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b07486f, 48 },
    { 0, 0 },
    { 0x50e74871, 55 },
    { 0, 0 },
    { 0x26f74873, 55 },
    { 0, 0 },
    { 0x97774875, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9947487f, 33 },
    { 0x10b74880, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad874888, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a7488c, 23 },
    { 0x7517488d, 43 },
    { 0x45c7488e, 46 },
    { 0x6fe7488f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7274892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c74895, 81 },
    { 0x39674896, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc77489b, 76 },
    { 0xb6c7489c, 93 },
    { 0x2567489d, 100 },
    { 0xb667489e, 67 },
    { 0, 0 },
    { 0xbd3748a0, 78 },
    { 0x3d6748a0, 43 },
    { 0xdd6748a2, 98 },
    { 0x588748a2, 98 },
    { 0x893748a2, 55 },
    { 0, 0 },
    { 0x326748a6, 23 },
    { 0, 0 },
    { 0xf17748a8, 98 },
    { 0, 0 },
    { 0xe87748aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4748ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc748b0, 91 },
    { 0xbf0748b0, 99 },
    { 0x7c5748b0, 98 },
    { 0x57748b2, 87 },
    { 0xe10748b1, 23 },
    { 0xbbe748b5, 77 },
    { 0xe21748b6, 90 },
    { 0, 0 },
    { 0x447748b8, 98 },
    { 0, 0 },
    { 0x40f748ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293748c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c748c4, 97 },
    { 0x982748c5, 75 },
    { 0x503748c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2748c9, 61 },
    { 0x3d3748c9, 87 },
    { 0xed4748cb, 66 },
    { 0xeb5748cb, 90 },
    { 0x7ba748c9, 33 },
    { 0x244748ce, 76 },
    { 0x806748cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d748d5, 66 },
    { 0xb71748d6, 100 },
    { 0, 0 },
    { 0x27c748d8, 23 },
    { 0x8dd748d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94748dd, 85 },
    { 0x4c9748dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4748e2, 98 },
    { 0x2d748e3, 66 },
    { 0xb6c748e4, 46 },
    { 0xee3748e3, 55 },
    { 0x1d1748e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc748ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f748f6, 52 },
    { 0, 0 },
    { 0x55d748f8, 33 },
    { 0x77d748f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7748fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1274900, 65 },
    { 0xcb874901, 78 },
    { 0x6f674901, 33 },
    { 0x88f74901, 90 },
    { 0x40a74904, 33 },
    { 0xebc74904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54574908, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c67490c, 98 },
    { 0x7f87490d, 75 },
    { 0xb0c7490e, 52 },
    { 0xc9a7490f, 23 },
    { 0x79f74910, 95 },
    { 0x974910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb74918, 99 },
    { 0, 0 },
    { 0x5997491a, 50 },
    { 0x2487491b, 98 },
    { 0x4dd7491c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b274920, 33 },
    { 0x6f974920, 61 },
    { 0xc0074922, 81 },
    { 0x51a74923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63874929, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7007492c, 78 },
    { 0x92b7492d, 98 },
    { 0xa8a7492e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e474932, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e74937, 67 },
    { 0x73e74938, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a7493b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b7493f, 100 },
    { 0xe0e7493f, 100 },
    { 0, 0 },
    { 0xd0c74942, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e674945, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127494a, 61 },
    { 0x5ed7494b, 63 },
    { 0x45b7494c, 76 },
    { 0xe3f7494a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11274950, 33 },
    { 0x8fb74951, 100 },
    { 0x95274952, 33 },
    { 0xcf74951, 33 },
    { 0xade74954, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc874958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e37495b, 76 },
    { 0x5d47495c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7307495f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed74962, 34 },
    { 0xd2a74963, 40 },
    { 0xf2e74964, 90 },
    { 0x3c074965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8274968, 55 },
    { 0, 0 },
    { 0x7957496a, 61 },
    { 0x3207496a, 91 },
    { 0x6227496c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34374970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4674973, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb437497d, 52 },
    { 0xf397497e, 61 },
    { 0x55c7497f, 61 },
    { 0x43d7497d, 98 },
    { 0, 0 },
    { 0x6a874982, 100 },
    { 0x76f74982, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5674987, 84 },
    { 0, 0 },
    { 0xfa174989, 33 },
    { 0xb157498a, 34 },
    { 0x5827498a, 33 },
    { 0, 0 },
    { 0xf007498d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d874998, 34 },
    { 0, 0 },
    { 0xef27499a, 55 },
    { 0x37d7499b, 87 },
    { 0x6237499a, 90 },
    { 0, 0 },
    { 0x4327499e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad749a5, 80 },
    { 0x67d749a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0749a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c749ac, 61 },
    { 0x16b749ad, 55 },
    { 0, 0 },
    { 0x2bf749af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x378749b2, 23 },
    { 0xc4e749b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65749b7, 24 },
    { 0xf47749b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x606749bb, 81 },
    { 0x9c1749bc, 65 },
    { 0x901749bb, 46 },
    { 0x261749be, 43 },
    { 0x5f9749bf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3749c2, 90 },
    { 0x152749c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93749c9, 95 },
    { 0x7cc749c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653749d1, 99 },
    { 0xe6749d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8749d5, 85 },
    { 0, 0 },
    { 0x6a1749d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388749de, 66 },
    { 0x83c749df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e749e4, 33 },
    { 0xc63749e5, 98 },
    { 0x89749e6, 61 },
    { 0x340749e4, 61 },
    { 0x46b749e5, 68 },
    { 0x181749e6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b749ec, 43 },
    { 0, 0 },
    { 0xfc9749ee, 91 },
    { 0xc1749ef, 90 },
    { 0x1f2749f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1749f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3749fa, 61 },
    { 0x87749fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x261749fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b74a01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55774a05, 98 },
    { 0xe9b74a06, 61 },
    { 0, 0 },
    { 0x6c974a08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf674a0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10774a11, 65 },
    { 0, 0 },
    { 0xc9574a13, 46 },
    { 0xe9874a14, 33 },
    { 0xe8b74a15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd874a18, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57474a1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19974a20, 33 },
    { 0, 0 },
    { 0x9c674a22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20574a27, 46 },
    { 0, 0 },
    { 0xb3e74a29, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90574a2f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77574a33, 98 },
    { 0x2b474a34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d674a37, 99 },
    { 0xd8274a37, 86 },
    { 0xc5974a39, 100 },
    { 0x86f74a3a, 93 },
    { 0xa2174a3b, 87 },
    { 0xdae74a39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f074a3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca974a42, 84 },
    { 0, 0 },
    { 0x93474a44, 90 },
    { 0xb8174a45, 52 },
    { 0xb2674a46, 90 },
    { 0, 0 },
    { 0x38d74a48, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2874a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e974a4e, 46 },
    { 0, 0 },
    { 0x62574a50, 81 },
    { 0xb7074a51, 45 },
    { 0xc8074a52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68774a55, 34 },
    { 0x46f74a55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6274a5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce774a5d, 12 },
    { 0xa974a5e, 94 },
    { 0xbe574a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf874a68, 95 },
    { 0x2b374a69, 33 },
    { 0x78f74a6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27374a6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3574a70, 55 },
    { 0x72074a70, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x98074a74, 61 },
    { 0xda974a75, 95 },
    { 0xf4474a75, 84 },
    { 0x16174a76, 23 },
    { 0, 0 },
    { 0x8474a79, 90 },
    { 0xb5074a79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf74a80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c174a86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f74a8c, 61 },
    { 0xa5574a8d, 100 },
    { 0, 0 },
    { 0xa9774a8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e274a92, 95 },
    { 0xe1c74a93, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe174a98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82174aa0, 91 },
    { 0xbdd74aa0, 98 },
    { 0x19b74aa0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e074aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea74aab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2974aae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0474ab2, 34 },
    { 0, 0 },
    { 0x75674ab4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a374aba, 33 },
    { 0x3c274abb, 33 },
    { 0x15774abc, 98 },
    { 0x4fc74abd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x8174ac0, 23 },
    { 0, 0 },
    { 0x1b374ac2, 55 },
    { 0x86d74ac3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c74aca, 43 },
    { 0x7b374aca, 98 },
    { 0x72774aca, 95 },
    { 0xac774acd, 93 },
    { 0xfe674ace, 61 },
    { 0x3ed74aca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a74ad2, 76 },
    { 0xf5274ad2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b74ad8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4574add, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f74ae0, 33 },
    { 0x53874ae1, 61 },
    { 0x77874ae1, 62 },
    { 0xdf074ae3, 81 },
    { 0x62574ae4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x58574ae7, 76 },
    { 0xbf274ae7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43074aec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9074af1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6974afa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c74b06, 46 },
    { 0x78d74b07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c74b0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef74b0e, 66 },
    { 0, 0 },
    { 0x73b74b10, 66 },
    { 0xd2074b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d74b16, 95 },
    { 0x15f74b16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae74b1d, 76 },
    { 0xb7874b1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec574b21, 55 },
    { 0xa7574b22, 33 },
    { 0xc1774b21, 98 },
    { 0xab674b24, 76 },
    { 0, 0 },
    { 0xa5974b26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f274b2d, 33 },
    { 0, 0 },
    { 0xaf374b2f, 61 },
    { 0xaad74b30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2074b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a74b3b, 61 },
    { 0xf074b3c, 66 },
    { 0x8f74b3c, 85 },
    { 0, 0 },
    { 0xe8074b3f, 55 },
    { 0x8c074b3f, 90 },
    { 0xbb074b41, 95 },
    { 0x8b774b42, 97 },
    { 0xb2a74b42, 23 },
    { 0x24474b40, 33 },
    { 0x68a74b44, 98 },
    { 0, 0 },
    { 0x74674b47, 61 },
    { 0xfbc74b47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d74b4b, 33 },
    { 0x98a74b4b, 97 },
    { 0, 0 },
    { 0x9bf74b4e, 90 },
    { 0xa6474b4f, 55 },
    { 0x4a74b4f, 66 },
    { 0xc0174b51, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44074b5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a974b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78674b60, 98 },
    { 0, 0 },
    { 0x29774b62, 43 },
    { 0x2f974b63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36474b68, 61 },
    { 0x87874b69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49374b71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f74b77, 90 },
    { 0, 0 },
    { 0x1774b79, 33 },
    { 0, 0 },
    { 0x42274b7b, 61 },
    { 0, 0 },
    { 0x47774b7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8674b83, 2 },
    { 0xe1a74b84, 98 },
    { 0xd7274b84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d74b8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f374b8f, 61 },
    { 0, 0 },
    { 0x1ad74b91, 66 },
    { 0xe7b74b92, 67 },
    { 0, 0 },
    { 0x6f574b94, 68 },
    { 0x12374b94, 98 },
    { 0x9b974b95, 33 },
    { 0x1ad74b97, 55 },
    { 0, 0 },
    { 0x13174b99, 99 },
    { 0, 0 },
    { 0xf9674b9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46974ba0, 98 },
    { 0xe0d74ba0, 33 },
    { 0xa7974ba2, 43 },
    { 0x16774ba3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c274ba7, 81 },
    { 0x7dd74ba8, 95 },
    { 0x75b74ba9, 55 },
    { 0xf3c74ba8, 4 },
    { 0x6ee74bab, 95 },
    { 0x2bf74ba9, 52 },
    { 0x92d74bad, 33 },
    { 0xea74bad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f374bb5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d74bbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b74bc1, 66 },
    { 0xdd674bc1, 98 },
    { 0xa874bc1, 84 },
    { 0xd4074bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac674bc7, 33 },
    { 0, 0 },
    { 0x98674bc9, 99 },
    { 0x5ba74bc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a474bcd, 23 },
    { 0x42f74bce, 52 },
    { 0xd7974bce, 74 },
    { 0x2d874bce, 99 },
    { 0xa9774bd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f74bd4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2774bda, 61 },
    { 0x2c174bdb, 85 },
    { 0, 0 },
    { 0x9d874bdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b74be1, 55 },
    { 0x97074be2, 43 },
    { 0, 0 },
    { 0xad574be4, 55 },
    { 0xc3f74be4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea874be8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76574beb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a874bf7, 61 },
    { 0x41b74bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef74bfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7874c01, 90 },
    { 0x58e74c01, 95 },
    { 0x7f274c03, 23 },
    { 0xd4a74c04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d174c0d, 98 },
    { 0, 0 },
    { 0x53f74c0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea74c12, 99 },
    { 0x23674c12, 90 },
    { 0x84b74c12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b74c18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef74c22, 75 },
    { 0, 0 },
    { 0xe8874c24, 93 },
    { 0x57174c25, 95 },
    { 0x7a774c26, 61 },
    { 0, 0 },
    { 0x35374c28, 61 },
    { 0xcf274c28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb74c32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71974c35, 97 },
    { 0x69a74c36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d74c3b, 90 },
    { 0x95874c3c, 77 },
    { 0, 0 },
    { 0x7cf74c3e, 98 },
    { 0, 0 },
    { 0x37274c40, 90 },
    { 0, 0 },
    { 0x27e74c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf74c48, 61 },
    { 0x50074c49, 90 },
    { 0xc974c49, 76 },
    { 0xe5474c49, 99 },
    { 0x1b974c4b, 66 },
    { 0x86b74c4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9374c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d674c53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c174c58, 23 },
    { 0xf9774c58, 33 },
    { 0xd8c74c5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd74c5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d174c61, 95 },
    { 0x62674c62, 61 },
    { 0x7ac74c63, 34 },
    { 0, 0 },
    { 0xb3174c65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c74c69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d74c6d, 52 },
    { 0, 0 },
    { 0x9f174c6f, 90 },
    { 0, 0 },
    { 0x8a574c71, 43 },
    { 0, 0 },
    { 0x9674c73, 23 },
    { 0x42174c73, 99 },
    { 0x81374c75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22074c78, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7874c7d, 76 },
    { 0x17774c7d, 98 },
    { 0, 0 },
    { 0x9ef74c80, 68 },
    { 0x6bd74c81, 90 },
    { 0, 0 },
    { 0x2bb74c83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16474c87, 76 },
    { 0x70574c88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xed974c8b, 33 },
    { 0x8bb74c8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe274c93, 61 },
    { 0, 0 },
    { 0xf6774c95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b74c99, 95 },
    { 0x22b74c9a, 78 },
    { 0, 0 },
    { 0x20774c9c, 61 },
    { 0xedf74c9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c874ca5, 33 },
    { 0x47a74ca5, 75 },
    { 0x29c74ca6, 55 },
    { 0x7ff74ca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b74cab, 76 },
    { 0xd1e74cab, 23 },
    { 0x51b74cad, 61 },
    { 0xcd74cae, 100 },
    { 0x82374cad, 23 },
    { 0xd174cb0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0774cb6, 95 },
    { 0, 0 },
    { 0xfea74cb8, 76 },
    { 0xce074cb9, 33 },
    { 0xb4a74cb8, 90 },
    { 0x5b274cb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5574cc0, 97 },
    { 0, 0 },
    { 0x22774cc2, 55 },
    { 0xa9974cc3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e74cc7, 81 },
    { 0x2c174cc7, 43 },
    { 0xd5c74cc9, 90 },
    { 0, 0 },
    { 0x9cc74ccb, 66 },
    { 0, 0 },
    { 0xc7074ccd, 87 },
    { 0x9b74cce, 87 },
    { 0xcff74ccf, 75 },
    { 0, 0 },
    { 0x80374cd1, 52 },
    { 0xde674cd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6974cd5, 98 },
    { 0x74174cd5, 61 },
    { 0, 0 },
    { 0x97d74cd8, 90 },
    { 0, 0 },
    { 0x1c074cda, 95 },
    { 0xbc174cdb, 23 },
    { 0xc5874cdc, 91 },
    { 0xf8674cdd, 33 },
    { 0x8b674cdb, 98 },
    { 0x17274cdd, 23 },
    { 0x5f874cdf, 90 },
    { 0, 0 },
    { 0x5b574ce2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a74ce6, 46 },
    { 0, 0 },
    { 0x3874ce8, 77 },
    { 0, 0 },
    { 0xf1974cea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8474cf0, 97 },
    { 0x75f74cf1, 33 },
    { 0xcb274cf0, 52 },
    { 0xdb074cf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed274cf7, 52 },
    { 0, 0 },
    { 0xccd74cf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f74cfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7274d08, 61 },
    { 0, 0 },
    { 0xef674d0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1674d14, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a74d1b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x474d1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b374d22, 90 },
    { 0x6b674d22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a974d26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac174d2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8674d39, 33 },
    { 0x68e74d3a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25074d3d, 46 },
    { 0x5e574d3d, 98 },
    { 0xb8f74d3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80574d43, 61 },
    { 0, 0 },
    { 0x61574d45, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc74d4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62774d4e, 61 },
    { 0, 0 },
    { 0x68974d50, 55 },
    { 0xd1574d50, 90 },
    { 0x43a74d50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e74d58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63774d5e, 38 },
    { 0xb9974d5f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed74d63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd74d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66674d6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d574d6d, 96 },
    { 0x15874d6e, 95 },
    { 0xcff74d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d76, 23 },
    { 0x6b974d77, 23 },
    { 0xe8374d76, 95 },
    { 0, 0 },
    { 0xfde74d7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf74d7e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb274d83, 51 },
    { 0x20874d84, 98 },
    { 0x5ef74d84, 33 },
    { 0x1a774d84, 38 },
    { 0xb1674d86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4074d91, 23 },
    { 0, 0 },
    { 0xc374d93, 23 },
    { 0xf2f74d93, 43 },
    { 0x70b74d95, 23 },
    { 0xc774d96, 67 },
    { 0x78f74d97, 90 },
    { 0x7e374d98, 98 },
    { 0x28274d98, 90 },
    { 0, 0 },
    { 0x7874d9b, 90 },
    { 0xcbc74d9b, 90 },
    { 0x4a74d9d, 90 },
    { 0, 0 },
    { 0xe8d74d9f, 91 },
    { 0x2f874d9f, 23 },
    { 0x7674d9f, 81 },
    { 0x6974da2, 99 },
    { 0x9ba74da3, 90 },
    { 0x24174da2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec074da8, 61 },
    { 0x58374da9, 93 },
    { 0x34874daa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a74db1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e774db5, 23 },
    { 0, 0 },
    { 0x56674db7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff274dc4, 98 },
    { 0x22374dc5, 46 },
    { 0, 0 },
    { 0x75274dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69474dcc, 43 },
    { 0x97f74dcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb74dd1, 95 },
    { 0xda974dd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9874dd7, 98 },
    { 0xa2074dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77774ddd, 84 },
    { 0, 0 },
    { 0x18474ddf, 90 },
    { 0, 0 },
    { 0x88e74de1, 85 },
    { 0, 0 },
    { 0x70574de3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3774ded, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57974df1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a574df6, 65 },
    { 0x77a74df7, 61 },
    { 0x5d374df6, 66 },
    { 0, 0 },
    { 0x89c74dfa, 61 },
    { 0xdd974dfb, 90 },
    { 0x6c874dfc, 61 },
    { 0x96274dfd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5074e02, 81 },
    { 0xd6d74e03, 66 },
    { 0, 0 },
    { 0xf6e74e05, 90 },
    { 0, 0 },
    { 0x4f274e07, 34 },
    { 0, 0 },
    { 0x774e09, 61 },
    { 0, 0 },
    { 0xd7a74e0b, 23 },
    { 0xcb374e0c, 90 },
    { 0, 0 },
    { 0xbba74e0e, 85 },
    { 0xab974e0f, 34 },
    { 0x60b74e10, 76 },
    { 0, 0 },
    { 0xcd574e12, 68 },
    { 0x84e74e13, 68 },
    { 0, 0 },
    { 0x8e74e15, 33 },
    { 0xfed74e16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c74e1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27574e21, 90 },
    { 0x67174e22, 33 },
    { 0x66f74e23, 76 },
    { 0x17274e24, 90 },
    { 0x74374e24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf574e28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b674e2f, 33 },
    { 0x34174e2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf74e33, 33 },
    { 0, 0 },
    { 0x5fb74e35, 90 },
    { 0x70e74e35, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb74e39, 34 },
    { 0x51a74e39, 93 },
    { 0x48474e3b, 67 },
    { 0x38774e3b, 43 },
    { 0xa3674e3d, 90 },
    { 0, 0 },
    { 0x35c74e3f, 33 },
    { 0x1d674e40, 84 },
    { 0x1f574e41, 90 },
    { 0x2cf74e41, 95 },
    { 0xb8574e41, 46 },
    { 0x4c274e40, 43 },
    { 0, 0 },
    { 0xa1174e46, 76 },
    { 0x7c674e46, 34 },
    { 0x45374e48, 85 },
    { 0x1e274e48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b74e4c, 90 },
    { 0x6f174e4d, 90 },
    { 0x96774e4e, 61 },
    { 0xa9a74e4f, 23 },
    { 0x98c74e4d, 33 },
    { 0x65c74e4d, 90 },
    { 0x8be74e52, 90 },
    { 0x77574e53, 33 },
    { 0, 0 },
    { 0xae374e55, 34 },
    { 0x3f974e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d74e59, 33 },
    { 0xa4e74e5a, 23 },
    { 0, 0 },
    { 0x90974e5c, 93 },
    { 0xaaf74e5d, 99 },
    { 0x2ca74e5e, 33 },
    { 0xfed74e5f, 23 },
    { 0x5b674e5d, 98 },
    { 0x8d74e61, 78 },
    { 0xe2774e5e, 33 },
    { 0, 0 },
    { 0x5bc74e64, 101 },
    { 0x5af74e65, 55 },
    { 0, 0 },
    { 0xa6d74e67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c174e6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8974e73, 90 },
    { 0xff174e73, 95 },
    { 0xa3974e73, 90 },
    { 0x22774e73, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e74e7d, 87 },
    { 0, 0 },
    { 0xa3674e7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a74e83, 98 },
    { 0xdc074e84, 78 },
    { 0xd0674e85, 33 },
    { 0x30374e85, 23 },
    { 0x15a74e84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68774e8b, 61 },
    { 0xe5974e8c, 95 },
    { 0, 0 },
    { 0xc0f74e8e, 98 },
    { 0x6d374e8f, 61 },
    { 0, 0 },
    { 0xcda74e91, 78 },
    { 0, 0 },
    { 0xf0c74e93, 83 },
    { 0, 0 },
    { 0x8e574e95, 96 },
    { 0, 0 },
    { 0x9a974e97, 33 },
    { 0, 0 },
    { 0x8a74e99, 90 },
    { 0xf1374e9a, 61 },
    { 0xd9274e9a, 95 },
    { 0x3be74e9b, 98 },
    { 0x30674e9d, 23 },
    { 0x83574e9e, 81 },
    { 0x23174e9f, 34 },
    { 0x8cb74e9e, 98 },
    { 0x35c74ea1, 23 },
    { 0x43974e9e, 75 },
    { 0xc5f74ea3, 88 },
    { 0xa6a74ea4, 85 },
    { 0x1f774ea5, 55 },
    { 0xc2f74ea6, 90 },
    { 0xc4b74ea7, 55 },
    { 0x3b374e9d, 17 },
    { 0x3d274ea6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6474ead, 17 },
    { 0xb7274ead, 23 },
    { 0, 0 },
    { 0x68a74eb0, 90 },
    { 0x4cd74eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f574eb7, 34 },
    { 0xbdc74eb8, 98 },
    { 0x8b074eb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71874ebe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73074ec2, 36 },
    { 0, 0 },
    { 0x39c74ec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3574ec7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc074ecc, 98 },
    { 0x7bf74ecd, 98 },
    { 0xed974ecd, 40 },
    { 0x39274ecf, 90 },
    { 0x9b674ed0, 23 },
    { 0x9d174ed1, 90 },
    { 0x1c874ed2, 33 },
    { 0, 0 },
    { 0xd8574ed4, 95 },
    { 0x1bf74ed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe874ed8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64874ee2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a74ee6, 93 },
    { 0x90174ee7, 85 },
    { 0x17374ee8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc74eec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e74eef, 76 },
    { 0xad174eef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b074ef3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66674efa, 98 },
    { 0xb1174efa, 98 },
    { 0x6db74efc, 93 },
    { 0x47774efa, 46 },
    { 0x99e74efe, 61 },
    { 0x23f74eff, 98 },
    { 0x7c774f00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a74f08, 93 },
    { 0x48274f09, 90 },
    { 0x73874f0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd74f0e, 98 },
    { 0, 0 },
    { 0xdb074f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf74f14, 76 },
    { 0, 0 },
    { 0xc0c74f16, 90 },
    { 0xea274f16, 88 },
    { 0x99e74f16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4474f1f, 33 },
    { 0xbe774f20, 43 },
    { 0x90c74f21, 23 },
    { 0x7974f22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a74f25, 65 },
    { 0xb4974f26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd974f30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91574f33, 23 },
    { 0xc6474f34, 90 },
    { 0xa8874f34, 33 },
    { 0x62a74f34, 98 },
    { 0x82c74f37, 98 },
    { 0x274f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56674f3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4374f3e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e74f42, 66 },
    { 0, 0 },
    { 0x62b74f44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e574f47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d74f4a, 33 },
    { 0x7d774f4b, 95 },
    { 0, 0 },
    { 0x86374f4d, 61 },
    { 0xa9974f4e, 95 },
    { 0xb8574f4f, 93 },
    { 0xa3474f50, 52 },
    { 0, 0 },
    { 0xe1374f52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b74f57, 33 },
    { 0xbac74f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1074f5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c074f5e, 93 },
    { 0xdea74f5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4674f63, 23 },
    { 0xa5e74f63, 90 },
    { 0, 0 },
    { 0x76074f66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf74f69, 61 },
    { 0xd7174f69, 61 },
    { 0x8a774f6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d74f6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c074f78, 46 },
    { 0xdd174f79, 98 },
    { 0xc9f74f78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a574f80, 98 },
    { 0, 0 },
    { 0xdb674f82, 61 },
    { 0x61f74f83, 77 },
    { 0x97374f84, 61 },
    { 0xb6c74f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb74f89, 95 },
    { 0x69c74f89, 90 },
    { 0, 0 },
    { 0x48b74f8c, 76 },
    { 0, 0 },
    { 0xb0774f8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6774f95, 98 },
    { 0x81774f96, 95 },
    { 0xe1174f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a74f9f, 100 },
    { 0x13b74f9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99274fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8974fac, 61 },
    { 0xccd74fad, 66 },
    { 0x65a74fae, 98 },
    { 0x87374fae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d674fb2, 90 },
    { 0x42074fb3, 95 },
    { 0x6d74fb4, 33 },
    { 0x3e574fb5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c74fb8, 90 },
    { 0x9bc74fb8, 33 },
    { 0x50374fba, 98 },
    { 0x80774fb9, 52 },
    { 0, 0 },
    { 0xf6374fbd, 87 },
    { 0x94074fbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af74fc2, 34 },
    { 0x23874fc2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed174fc7, 85 },
    { 0x5bd74fc8, 76 },
    { 0x2d674fc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d74fcd, 93 },
    { 0x37574fcd, 46 },
    { 0, 0 },
    { 0xdaa74fd0, 33 },
    { 0, 0 },
    { 0x45e74fd2, 23 },
    { 0xbb074fd2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e574fe1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa74fe5, 23 },
    { 0, 0 },
    { 0xf3a74fe7, 98 },
    { 0xa2474fe7, 90 },
    { 0x7e74fe8, 91 },
    { 0x8474fe7, 23 },
    { 0x4fa74fea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56774ff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80274ff8, 55 },
    { 0, 0 },
    { 0x79074ffa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63674ffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c75002, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa075006, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc275009, 61 },
    { 0x8dc7500a, 66 },
    { 0x2087500b, 78 },
    { 0xd3c7500a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc267500f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46275012, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb775015, 95 },
    { 0xeb775016, 85 },
    { 0, 0 },
    { 0x30175018, 33 },
    { 0xa9c75019, 33 },
    { 0xf4475019, 98 },
    { 0x8207501b, 23 },
    { 0, 0 },
    { 0x14c7501d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa75021, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f75024, 95 },
    { 0x19c75025, 90 },
    { 0x37475026, 33 },
    { 0xfcc75027, 33 },
    { 0x31375028, 98 },
    { 0x30e75029, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd427502f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf175033, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x25175036, 95 },
    { 0x41075037, 91 },
    { 0xca575038, 55 },
    { 0x60475038, 33 },
    { 0x88875039, 55 },
    { 0x40c7503b, 85 },
    { 0x47c7503c, 33 },
    { 0x1d07503b, 90 },
    { 0xcf57503e, 33 },
    { 0xcd97503b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94175043, 95 },
    { 0x37075044, 55 },
    { 0, 0 },
    { 0xb9e75046, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4175049, 90 },
    { 0xa5d7504a, 33 },
    { 0xca37504b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d75055, 88 },
    { 0, 0 },
    { 0xf4e75057, 78 },
    { 0xd7f75058, 98 },
    { 0xa4975059, 100 },
    { 0, 0 },
    { 0x26d7505b, 33 },
    { 0xe2d7505c, 34 },
    { 0x1697505d, 9 },
    { 0x72d7505b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a75062, 55 },
    { 0, 0 },
    { 0xbcf75064, 90 },
    { 0, 0 },
    { 0xf475066, 46 },
    { 0xc3075067, 43 },
    { 0x8af75068, 55 },
    { 0xa0075066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a7506c, 76 },
    { 0x8c77506d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91875072, 34 },
    { 0, 0 },
    { 0xb75074, 55 },
    { 0, 0 },
    { 0xf8775076, 33 },
    { 0xc5d75077, 23 },
    { 0x15d75078, 91 },
    { 0x9ac75077, 90 },
    { 0x13f75077, 23 },
    { 0x5747507b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc937507e, 33 },
    { 0x1247507f, 90 },
    { 0x47475080, 33 },
    { 0x6e675080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64575084, 43 },
    { 0xd7b75085, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d7508b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6947508f, 23 },
    { 0x7c575090, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc375094, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d75098, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab37509b, 65 },
    { 0, 0 },
    { 0x16c7509d, 77 },
    { 0x94d7509e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4750a6, 100 },
    { 0xb4d750a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf750be, 35 },
    { 0xde5750bf, 33 },
    { 0xe89750c0, 98 },
    { 0, 0 },
    { 0x168750c2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08750c7, 87 },
    { 0xdd4750c8, 76 },
    { 0, 0 },
    { 0xff5750ca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157750ce, 55 },
    { 0, 0 },
    { 0x291750d0, 33 },
    { 0xc4c750d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x491750d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a750d7, 98 },
    { 0xb29750d7, 66 },
    { 0x95f750d9, 34 },
    { 0, 0 },
    { 0x448750db, 78 },
    { 0xcfa750dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0750df, 61 },
    { 0x8c3750df, 94 },
    { 0x158750e1, 23 },
    { 0xd9750e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984750e8, 90 },
    { 0xdf750e9, 52 },
    { 0xc08750ea, 98 },
    { 0xfe8750ea, 90 },
    { 0xc33750e9, 46 },
    { 0x68c750ed, 33 },
    { 0x65c750ec, 61 },
    { 0x731750ef, 78 },
    { 0x32b750f0, 85 },
    { 0xb05750f1, 61 },
    { 0xb80750f1, 90 },
    { 0xb53750ef, 94 },
    { 0xabb750f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245750fc, 87 },
    { 0, 0 },
    { 0x5dc750fe, 69 },
    { 0xd80750fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d75102, 95 },
    { 0, 0 },
    { 0x63f75104, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a175108, 76 },
    { 0, 0 },
    { 0xc357510a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc07510d, 95 },
    { 0xba87510d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea975113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e75119, 90 },
    { 0, 0 },
    { 0x2f47511b, 90 },
    { 0xeeb7511c, 98 },
    { 0x2057511c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48275120, 61 },
    { 0x17375121, 90 },
    { 0x19475121, 95 },
    { 0x31d75120, 34 },
    { 0xea475122, 33 },
    { 0xd075124, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf637512a, 90 },
    { 0x3f07512a, 78 },
    { 0, 0 },
    { 0x1b17512d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96175130, 98 },
    { 0x9ad75131, 34 },
    { 0, 0 },
    { 0x55e75133, 43 },
    { 0x4ef75134, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x34175137, 33 },
    { 0x84175138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1317513b, 90 },
    { 0, 0 },
    { 0x43b7513d, 61 },
    { 0, 0 },
    { 0x1a97513f, 98 },
    { 0xf1475140, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x87575143, 23 },
    { 0xe4475144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df75149, 43 },
    { 0, 0 },
    { 0xb8b7514b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b7514e, 98 },
    { 0, 0 },
    { 0xe2575150, 98 },
    { 0, 0 },
    { 0x6a575152, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59375155, 93 },
    { 0, 0 },
    { 0xad575157, 33 },
    { 0xca675158, 90 },
    { 0x68375158, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5087515c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b75160, 98 },
    { 0, 0 },
    { 0xb675162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab75166, 77 },
    { 0, 0 },
    { 0xd1c75168, 99 },
    { 0xd0175169, 91 },
    { 0x907516a, 33 },
    { 0xb267516b, 61 },
    { 0x28775169, 33 },
    { 0xa227516d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60975170, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d775174, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f475177, 61 },
    { 0x6cc75177, 33 },
    { 0x30d75179, 76 },
    { 0x9d7517a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f97517d, 90 },
    { 0xd97517e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e75184, 23 },
    { 0xed975185, 98 },
    { 0xba475185, 66 },
    { 0x91b75186, 81 },
    { 0xce175184, 76 },
    { 0xf1075187, 76 },
    { 0x2b47518a, 76 },
    { 0x7bb7518b, 76 },
    { 0, 0 },
    { 0x6117518d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c675194, 23 },
    { 0xe9175195, 43 },
    { 0xcb175196, 76 },
    { 0, 0 },
    { 0x7fd75198, 98 },
    { 0, 0 },
    { 0x5eb7519a, 61 },
    { 0, 0 },
    { 0xd757519c, 46 },
    { 0, 0 },
    { 0x5367519e, 23 },
    { 0, 0 },
    { 0x715751a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6751a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3751aa, 82 },
    { 0xa88751ab, 87 },
    { 0x7b9751ac, 61 },
    { 0xdfe751ad, 45 },
    { 0x25f751ae, 90 },
    { 0, 0 },
    { 0xcfa751b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8751b7, 88 },
    { 0xca4751b8, 55 },
    { 0xcd0751b7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e751c0, 95 },
    { 0x289751c1, 90 },
    { 0x924751c0, 78 },
    { 0x14b751c3, 23 },
    { 0, 0 },
    { 0x280751c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff751c9, 33 },
    { 0x69c751ca, 76 },
    { 0xbe7751cb, 33 },
    { 0, 0 },
    { 0xf5e751cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c751d3, 81 },
    { 0xbca751d4, 90 },
    { 0x74751d5, 23 },
    { 0x5bb751d6, 34 },
    { 0xe66751d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29751df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff751e4, 95 },
    { 0, 0 },
    { 0x248751e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea751ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5751ed, 52 },
    { 0xcd2751ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67751f2, 61 },
    { 0, 0 },
    { 0xb6c751f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77751f8, 87 },
    { 0x21d751f9, 78 },
    { 0xf69751fa, 33 },
    { 0xab5751fa, 53 },
    { 0x92a751f9, 33 },
    { 0, 0 },
    { 0xa8f751fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc75204, 66 },
    { 0, 0 },
    { 0xc7875206, 76 },
    { 0xf9675206, 98 },
    { 0, 0 },
    { 0xad275209, 66 },
    { 0xd547520a, 100 },
    { 0x82e75209, 88 },
    { 0x137520c, 40 },
    { 0, 0 },
    { 0xbb27520e, 90 },
    { 0xa297520f, 23 },
    { 0x35a75210, 33 },
    { 0, 0 },
    { 0xe5f75212, 90 },
    { 0x7f75212, 98 },
    { 0xa1c75214, 90 },
    { 0x12575212, 61 },
    { 0xe5775212, 90 },
    { 0x5d875217, 43 },
    { 0x9b675218, 81 },
    { 0x32a75217, 76 },
    { 0xb1475219, 100 },
    { 0x1cf7521b, 46 },
    { 0x8557521b, 95 },
    { 0xd407521b, 93 },
    { 0xee7521b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc75221, 76 },
    { 0xba575222, 98 },
    { 0xf3d75221, 90 },
    { 0x6ab75224, 67 },
    { 0xd5c75225, 95 },
    { 0, 0 },
    { 0x1675227, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a7522c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17522f, 46 },
    { 0x4f7522f, 78 },
    { 0, 0 },
    { 0x3de75232, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87775237, 93 },
    { 0, 0 },
    { 0xab575239, 93 },
    { 0x7107523a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee75241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8675245, 23 },
    { 0x79175246, 93 },
    { 0x57e75245, 23 },
    { 0xc0575246, 34 },
    { 0, 0 },
    { 0x4807524a, 84 },
    { 0, 0 },
    { 0x23c7524c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9197524f, 90 },
    { 0, 0 },
    { 0x5d175251, 98 },
    { 0xdc675252, 75 },
    { 0, 0 },
    { 0xb7d75254, 87 },
    { 0x6bc75254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a75259, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea175265, 23 },
    { 0x1e75266, 55 },
    { 0xc4975266, 55 },
    { 0x9ee75268, 77 },
    { 0xf5775269, 34 },
    { 0x5767526a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a07526e, 52 },
    { 0, 0 },
    { 0x56c75270, 90 },
    { 0, 0 },
    { 0xfb375272, 90 },
    { 0x8f175272, 43 },
    { 0x3e75273, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf067527b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd75284, 85 },
    { 0x64975284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c75288, 67 },
    { 0xe4075289, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e27528c, 67 },
    { 0x7777528d, 90 },
    { 0x35a7528e, 61 },
    { 0x65f7528f, 33 },
    { 0xb477528f, 92 },
    { 0, 0 },
    { 0x9f675292, 90 },
    { 0x4f875293, 33 },
    { 0, 0 },
    { 0x4c875295, 98 },
    { 0xaa575295, 77 },
    { 0x57f75296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37529d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1752a2, 90 },
    { 0xe40752a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4752ad, 33 },
    { 0x55a752ae, 34 },
    { 0x2cd752af, 65 },
    { 0xa4752b0, 23 },
    { 0xa85752b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x524752b4, 55 },
    { 0xfb752b5, 90 },
    { 0, 0 },
    { 0xd72752b7, 87 },
    { 0, 0 },
    { 0x280752b9, 55 },
    { 0x346752b9, 98 },
    { 0, 0 },
    { 0x6b9752bc, 77 },
    { 0xa15752bc, 83 },
    { 0, 0 },
    { 0xee5752bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f752c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x788752c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80752d1, 34 },
    { 0x26b752d2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0752d8, 98 },
    { 0xe63752d9, 99 },
    { 0, 0 },
    { 0x251752db, 52 },
    { 0, 0 },
    { 0x973752dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b752e1, 61 },
    { 0, 0 },
    { 0x904752e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303752e7, 55 },
    { 0x1a5752e8, 34 },
    { 0xfc3752e8, 55 },
    { 0, 0 },
    { 0xfc6752eb, 43 },
    { 0x101752ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95752f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45752fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a752fd, 78 },
    { 0xf1f752fe, 34 },
    { 0x64e752fe, 33 },
    { 0, 0 },
    { 0xe8d75301, 95 },
    { 0x24b75302, 33 },
    { 0xa2a75302, 33 },
    { 0xba175304, 84 },
    { 0x5d475302, 95 },
    { 0x98875306, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f27530b, 23 },
    { 0, 0 },
    { 0xa97530d, 77 },
    { 0, 0 },
    { 0x53e7530f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d75312, 90 },
    { 0x3b675313, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a675317, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f7531c, 61 },
    { 0x2f7531d, 34 },
    { 0x9947531d, 90 },
    { 0xb027531c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2075323, 61 },
    { 0x68f75323, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c375329, 46 },
    { 0, 0 },
    { 0xaf57532b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e75337, 33 },
    { 0, 0 },
    { 0x96875339, 33 },
    { 0x9077533a, 23 },
    { 0xdc17533b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee75340, 23 },
    { 0x4de75341, 95 },
    { 0x95075342, 33 },
    { 0x52275342, 33 },
    { 0x40775344, 91 },
    { 0x7f275345, 23 },
    { 0x18575343, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c87534a, 98 },
    { 0xff07534b, 93 },
    { 0x5c27534c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9975350, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47535a, 90 },
    { 0x7e97535b, 84 },
    { 0x6f17535c, 23 },
    { 0, 0 },
    { 0x2147535e, 38 },
    { 0xd367535f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f275362, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68975367, 93 },
    { 0x8a475367, 23 },
    { 0xa3575369, 33 },
    { 0xcd475369, 98 },
    { 0x17b75369, 76 },
    { 0, 0 },
    { 0xc2d7536d, 52 },
    { 0x5387536d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3575371, 34 },
    { 0xa3875371, 61 },
    { 0x82b75373, 33 },
    { 0x7d375374, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f37537a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c75380, 93 },
    { 0, 0 },
    { 0x9bd75382, 23 },
    { 0x55175383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca475387, 67 },
    { 0x3d775388, 68 },
    { 0x21275387, 33 },
    { 0x77b75387, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e7538d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48575399, 90 },
    { 0xa287539a, 98 },
    { 0x1a27539b, 33 },
    { 0x8767539c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5537539f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd753a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7753a8, 93 },
    { 0xe5d753a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x313753ac, 90 },
    { 0, 0 },
    { 0x68c753ae, 33 },
    { 0x883753af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f753b3, 88 },
    { 0x89d753b3, 61 },
    { 0, 0 },
    { 0x420753b6, 90 },
    { 0, 0 },
    { 0x5d753b8, 88 },
    { 0x126753b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83753bc, 88 },
    { 0, 0 },
    { 0x7f3753be, 34 },
    { 0, 0 },
    { 0xd3d753c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1753ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c753cd, 98 },
    { 0x907753ce, 23 },
    { 0xfa9753cd, 90 },
    { 0, 0 },
    { 0x201753d1, 32 },
    { 0, 0 },
    { 0x783753d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba753d6, 95 },
    { 0, 0 },
    { 0x2f3753d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668753df, 55 },
    { 0xcfd753df, 61 },
    { 0, 0 },
    { 0x582753e2, 33 },
    { 0x7e0753e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x47753e6, 61 },
    { 0x4ad753e6, 82 },
    { 0x8f3753e8, 95 },
    { 0x744753e8, 95 },
    { 0, 0 },
    { 0x31e753eb, 67 },
    { 0, 0 },
    { 0x2a5753ed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16753f5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843753fa, 33 },
    { 0, 0 },
    { 0xd6753fc, 98 },
    { 0, 0 },
    { 0xbb7753fe, 100 },
    { 0xe2b753fe, 61 },
    { 0x137753ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f275403, 77 },
    { 0, 0 },
    { 0x62375405, 61 },
    { 0x1bb75405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f7540a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd247540f, 90 },
    { 0xac77540f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c675414, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e77541c, 33 },
    { 0x4b97541d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4575422, 85 },
    { 0, 0 },
    { 0x7ea75424, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e375427, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a7542c, 90 },
    { 0, 0 },
    { 0x4727542e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba75432, 61 },
    { 0x66975433, 90 },
    { 0xd6775434, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8275437, 100 },
    { 0xb4d75437, 90 },
    { 0, 0 },
    { 0x997543a, 34 },
    { 0xeb87543b, 46 },
    { 0x6637543c, 66 },
    { 0xdc07543d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f75445, 43 },
    { 0x6c675446, 61 },
    { 0xd9f75446, 76 },
    { 0xbf375448, 76 },
    { 0xbef75447, 99 },
    { 0xb067544a, 67 },
    { 0x56a7544a, 33 },
    { 0x5797544c, 64 },
    { 0x20f75446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3775455, 34 },
    { 0xddf75456, 23 },
    { 0, 0 },
    { 0x32b75458, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb147545c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a75460, 81 },
    { 0xba775460, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b475464, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e57546d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c7547a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a7547d, 90 },
    { 0, 0 },
    { 0x1f57547f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9875483, 90 },
    { 0x4da75484, 52 },
    { 0x5e075484, 23 },
    { 0x36475484, 83 },
    { 0xffb75487, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff7548a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf757548d, 88 },
    { 0x31e7548e, 90 },
    { 0xc27548d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2075496, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b7549a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa7549d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5754a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a754a5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6754ad, 87 },
    { 0x124754ae, 33 },
    { 0x765754ad, 61 },
    { 0xfd1754b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x358754b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55754b7, 23 },
    { 0xabd754b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df754c3, 61 },
    { 0xbcc754c4, 33 },
    { 0xa6b754c3, 53 },
    { 0xf3d754c6, 23 },
    { 0xcc8754c6, 33 },
    { 0x1ed754c6, 98 },
    { 0x8e4754c9, 33 },
    { 0xe92754ca, 90 },
    { 0x9dc754c9, 33 },
    { 0xc57754cc, 61 },
    { 0xbe3754c7, 93 },
    { 0x589754ca, 67 },
    { 0, 0 },
    { 0xdfd754d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917754d6, 61 },
    { 0xbcc754d6, 98 },
    { 0xf44754d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f754df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931754e3, 43 },
    { 0xa66754e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7754e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d754eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa754ee, 68 },
    { 0xff754ef, 61 },
    { 0x525754ef, 87 },
    { 0x475754f1, 61 },
    { 0xc44754f2, 98 },
    { 0xe07754ee, 33 },
    { 0x1f0754f4, 23 },
    { 0xf9754ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5754f8, 88 },
    { 0xc92754f9, 33 },
    { 0xe82754fa, 61 },
    { 0xb27754fb, 33 },
    { 0x9f754fa, 61 },
    { 0, 0 },
    { 0xb27754fe, 23 },
    { 0, 0 },
    { 0xc1375500, 34 },
    { 0x90675500, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd75504, 61 },
    { 0xd2775504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58875508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58875510, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26275513, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b75516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc287551a, 90 },
    { 0x3307551a, 95 },
    { 0x2277551c, 98 },
    { 0, 0 },
    { 0x7aa7551e, 81 },
    { 0x8f27551f, 90 },
    { 0x497551e, 36 },
    { 0, 0 },
    { 0x78275522, 34 },
    { 0x10f75523, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb77552a, 95 },
    { 0xab17552a, 93 },
    { 0x3c67552c, 78 },
    { 0xffe7552c, 61 },
    { 0x8777552e, 33 },
    { 0x6da7552e, 33 },
    { 0x7517552f, 83 },
    { 0xb937552f, 65 },
    { 0xfd7552f, 76 },
    { 0x7517552a, 66 },
    { 0x7e875533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe87553a, 43 },
    { 0x9537553b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9375545, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a67554b, 61 },
    { 0x4d47554c, 34 },
    { 0, 0 },
    { 0xeee7554e, 98 },
    { 0x6c87554f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e75552, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da75556, 90 },
    { 0xd8575557, 67 },
    { 0, 0 },
    { 0xc3975559, 61 },
    { 0x9bf7555a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7555e, 78 },
    { 0, 0 },
    { 0x39f75560, 61 },
    { 0x48175560, 91 },
    { 0x63b75561, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x54775565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d75568, 76 },
    { 0x44475569, 61 },
    { 0x3277556a, 93 },
    { 0x9cb75568, 46 },
    { 0x6d27556c, 66 },
    { 0x17e7556c, 98 },
    { 0x417556d, 55 },
    { 0, 0 },
    { 0x5ba75570, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7807557d, 100 },
    { 0xf4e7557e, 95 },
    { 0x3b67557f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a975582, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd7558f, 95 },
    { 0, 0 },
    { 0xb2675591, 90 },
    { 0x82675592, 76 },
    { 0, 0 },
    { 0x97175594, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a075598, 99 },
    { 0x73e75599, 66 },
    { 0, 0 },
    { 0xf0e7559b, 99 },
    { 0, 0 },
    { 0xd127559d, 61 },
    { 0, 0 },
    { 0xa827559f, 34 },
    { 0xb0e755a0, 66 },
    { 0x26f755a1, 87 },
    { 0x7f755a0, 93 },
    { 0x5cf755a1, 61 },
    { 0xe4d755a4, 33 },
    { 0xd14755a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70755a9, 78 },
    { 0xc35755a9, 19 },
    { 0, 0 },
    { 0xb9a755ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef755af, 23 },
    { 0, 0 },
    { 0xf6c755b1, 98 },
    { 0x466755b1, 23 },
    { 0x313755b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91755ba, 61 },
    { 0xf22755ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1755be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x457755c1, 34 },
    { 0, 0 },
    { 0xca7755c3, 88 },
    { 0xa8c755c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x637755c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268755cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1755d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c755d4, 33 },
    { 0x879755d5, 78 },
    { 0xcf3755d6, 23 },
    { 0xd4b755d7, 43 },
    { 0x13755d8, 33 },
    { 0x469755d8, 23 },
    { 0x1bb755d4, 33 },
    { 0x2ac755d9, 98 },
    { 0xa13755dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d755e0, 33 },
    { 0, 0 },
    { 0x30f755e2, 90 },
    { 0xf63755e3, 23 },
    { 0x866755e3, 95 },
    { 0xe24755e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9755ea, 23 },
    { 0xd83755ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c755f4, 55 },
    { 0, 0 },
    { 0x234755f6, 23 },
    { 0x682755f6, 33 },
    { 0xfbc755f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f755fb, 98 },
    { 0x96e755fc, 61 },
    { 0xa1f755fd, 97 },
    { 0x88b755fc, 76 },
    { 0, 0 },
    { 0x21775600, 90 },
    { 0, 0 },
    { 0x52c75602, 99 },
    { 0x12075603, 95 },
    { 0x28475604, 33 },
    { 0xef175605, 95 },
    { 0xca075605, 76 },
    { 0xf3e75605, 95 },
    { 0x8ce75608, 87 },
    { 0xd6d75609, 23 },
    { 0x7b57560a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be75610, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1075615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc75619, 90 },
    { 0, 0 },
    { 0x417561b, 46 },
    { 0xec27561b, 98 },
    { 0x9177561b, 76 },
    { 0, 0 },
    { 0x1487561f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b075627, 33 },
    { 0, 0 },
    { 0x85d75629, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68375635, 87 },
    { 0, 0 },
    { 0x5d75637, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7563b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c75641, 90 },
    { 0, 0 },
    { 0x20175643, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd75647, 38 },
    { 0x39475648, 90 },
    { 0x39975648, 61 },
    { 0xeb375649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75975650, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0275655, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f75659, 95 },
    { 0x9af7565a, 90 },
    { 0, 0 },
    { 0x887565c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b575660, 61 },
    { 0, 0 },
    { 0x38e75662, 98 },
    { 0x52875663, 43 },
    { 0x6775663, 46 },
    { 0, 0 },
    { 0x9a275666, 33 },
    { 0xd2075666, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2627566a, 33 },
    { 0x53c7566b, 87 },
    { 0xba17566c, 34 },
    { 0, 0 },
    { 0xb777566e, 61 },
    { 0xd4f7566e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc275673, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c75678, 90 },
    { 0x1b975679, 98 },
    { 0x13e75678, 43 },
    { 0xf2e7567b, 95 },
    { 0xf5475679, 55 },
    { 0, 0 },
    { 0xa0e7567e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82375682, 90 },
    { 0xe5c75682, 61 },
    { 0xfe975683, 33 },
    { 0x84875685, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7997568f, 95 },
    { 0xe5275690, 98 },
    { 0xfdd75691, 85 },
    { 0x96c75692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44275697, 33 },
    { 0, 0 },
    { 0x4f175699, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246756a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663756a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3756ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f756b0, 84 },
    { 0, 0 },
    { 0xceb756b2, 98 },
    { 0x910756b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501756ba, 90 },
    { 0xcf8756bb, 55 },
    { 0x8cc756bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x135756bf, 55 },
    { 0xce0756bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x100756c3, 78 },
    { 0x43756c4, 33 },
    { 0xcee756c5, 85 },
    { 0xfc8756c6, 90 },
    { 0xfa5756c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f756cd, 76 },
    { 0xcf4756ce, 46 },
    { 0x4d756ce, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e756d3, 23 },
    { 0xda8756d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6756d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x921756db, 68 },
    { 0, 0 },
    { 0xe54756dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c756e1, 91 },
    { 0xc9e756e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b756e9, 98 },
    { 0xd05756ea, 78 },
    { 0x4cc756eb, 61 },
    { 0x121756ea, 98 },
    { 0x8a2756eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d756f0, 34 },
    { 0x9756f1, 75 },
    { 0xf4d756f2, 43 },
    { 0x999756f3, 90 },
    { 0x796756f3, 98 },
    { 0x5e0756f4, 23 },
    { 0x79d756f3, 90 },
    { 0x833756f7, 93 },
    { 0xb0b756f6, 33 },
    { 0x9d6756f9, 95 },
    { 0xb45756fa, 46 },
    { 0, 0 },
    { 0xcde756fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e756ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78175704, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8775708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437570b, 95 },
    { 0, 0 },
    { 0x7aa7570d, 55 },
    { 0, 0 },
    { 0xaf37570f, 23 },
    { 0x9a175710, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9075718, 55 },
    { 0xa1075719, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb7571e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe475724, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d275727, 90 },
    { 0x55475728, 65 },
    { 0xfb075728, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e7572f, 75 },
    { 0, 0 },
    { 0xc75731, 95 },
    { 0, 0 },
    { 0x40775733, 85 },
    { 0, 0 },
    { 0xcad75735, 95 },
    { 0xa4575736, 46 },
    { 0, 0 },
    { 0x6ce75738, 33 },
    { 0, 0 },
    { 0x1b87573a, 76 },
    { 0xc167573b, 55 },
    { 0x30b7573c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e75740, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce975744, 90 },
    { 0x4a775745, 98 },
    { 0xb5975746, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98d75756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c7575d, 23 },
    { 0xf77575e, 90 },
    { 0x7427575f, 33 },
    { 0x1777575d, 55 },
    { 0x2c775761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27375764, 90 },
    { 0, 0 },
    { 0xa6875766, 34 },
    { 0x16775767, 76 },
    { 0x3c175768, 95 },
    { 0x6175768, 95 },
    { 0x55375767, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9875774, 76 },
    { 0, 0 },
    { 0xfc675776, 17 },
    { 0x12875777, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f7577b, 23 },
    { 0x4817577c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5837577f, 43 },
    { 0, 0 },
    { 0x41d75781, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e75784, 52 },
    { 0xd9975785, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x11475788, 95 },
    { 0, 0 },
    { 0x5337578a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f7578d, 78 },
    { 0xd7b7578d, 95 },
    { 0x2f37578f, 36 },
    { 0, 0 },
    { 0xe8575791, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26975794, 68 },
    { 0x17b75794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d975798, 43 },
    { 0xeab75799, 46 },
    { 0x3577579a, 85 },
    { 0, 0 },
    { 0x4c37579c, 33 },
    { 0x2747579d, 90 },
    { 0xae07579d, 95 },
    { 0, 0 },
    { 0x124757a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b757a6, 33 },
    { 0xf0f757a7, 91 },
    { 0, 0 },
    { 0x450757a9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191757af, 61 },
    { 0x2f7757b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf757bc, 55 },
    { 0x5f757bd, 88 },
    { 0xa94757be, 33 },
    { 0xe39757bd, 87 },
    { 0xc54757bf, 90 },
    { 0, 0 },
    { 0x381757c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43757c5, 88 },
    { 0xaa9757c5, 61 },
    { 0x599757c5, 95 },
    { 0, 0 },
    { 0xfc8757c9, 61 },
    { 0xa5d757ca, 95 },
    { 0xe3f757cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61757cf, 95 },
    { 0x2d4757d0, 61 },
    { 0x768757cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa757d5, 66 },
    { 0x7ed757d6, 61 },
    { 0xbc0757d7, 61 },
    { 0x942757d8, 61 },
    { 0x1b7757d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9757df, 90 },
    { 0xc8c757e0, 33 },
    { 0xdc9757e1, 23 },
    { 0xf10757e2, 23 },
    { 0x350757e3, 33 },
    { 0x915757e3, 90 },
    { 0xbd757e1, 90 },
    { 0xc25757e6, 33 },
    { 0xfb0757e6, 33 },
    { 0x45e757e2, 47 },
    { 0xd96757e9, 90 },
    { 0, 0 },
    { 0xd5d757eb, 61 },
    { 0, 0 },
    { 0x3ef757ed, 60 },
    { 0x362757ee, 52 },
    { 0, 0 },
    { 0x64c757f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f757f3, 90 },
    { 0x5ed757f3, 85 },
    { 0xfc757f4, 76 },
    { 0xc14757f6, 46 },
    { 0xd65757f7, 61 },
    { 0xe85757f8, 91 },
    { 0, 0 },
    { 0xdc6757fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d675801, 23 },
    { 0x6b975801, 33 },
    { 0x53475802, 93 },
    { 0x94b75802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a7580b, 23 },
    { 0, 0 },
    { 0xaa37580d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e875816, 61 },
    { 0xe1875817, 91 },
    { 0xee475818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7581b, 61 },
    { 0x8b27581c, 76 },
    { 0x7587581d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d75822, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b575828, 33 },
    { 0xd2975829, 34 },
    { 0, 0 },
    { 0x3667582b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e75830, 93 },
    { 0, 0 },
    { 0xbc975832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb175839, 99 },
    { 0, 0 },
    { 0x97c7583b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2775842, 76 },
    { 0x2f975843, 75 },
    { 0xa5e75842, 33 },
    { 0x6c175842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf47584a, 33 },
    { 0x26d7584a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5737584e, 90 },
    { 0x7317584f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c75858, 77 },
    { 0xf5475858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46875860, 67 },
    { 0xd2375861, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1975865, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86575869, 90 },
    { 0x9c87586a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf75871, 90 },
    { 0xd1975872, 98 },
    { 0xfaa75873, 61 },
    { 0x50275874, 87 },
    { 0x47f75871, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a575881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2275884, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65975887, 95 },
    { 0x26b75888, 90 },
    { 0xcf875887, 90 },
    { 0x8b7588a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda7588d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e575891, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b75894, 90 },
    { 0x44075895, 33 },
    { 0xa8b75895, 17 },
    { 0, 0 },
    { 0x6d475898, 93 },
    { 0xb675899, 55 },
    { 0x1e97589a, 98 },
    { 0xdb07589a, 23 },
    { 0xa4c7589a, 34 },
    { 0, 0 },
    { 0xad77589e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f758a1, 23 },
    { 0x92c758a2, 46 },
    { 0x5ea758a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb758a6, 98 },
    { 0xd0a758a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e758aa, 67 },
    { 0xfcd758aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee758b0, 55 },
    { 0, 0 },
    { 0x6ef758b2, 76 },
    { 0, 0 },
    { 0xcc1758b4, 100 },
    { 0, 0 },
    { 0xb77758b6, 41 },
    { 0xa21758b7, 93 },
    { 0x150758b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36758bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60c758bf, 61 },
    { 0, 0 },
    { 0xa9e758c1, 90 },
    { 0xb1758c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8758c6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734758cd, 55 },
    { 0x56d758cd, 76 },
    { 0x784758cd, 95 },
    { 0, 0 },
    { 0x5dc758d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a758d6, 93 },
    { 0x140758d7, 43 },
    { 0x98a758d6, 23 },
    { 0, 0 },
    { 0x57e758da, 93 },
    { 0x776758da, 90 },
    { 0x617758dc, 33 },
    { 0x491758db, 61 },
    { 0xc97758de, 90 },
    { 0xcd3758de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd758e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0758e9, 32 },
    { 0xbc5758ea, 88 },
    { 0xe6e758eb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x781758ee, 66 },
    { 0x5ce758ef, 67 },
    { 0x93b758f0, 100 },
    { 0, 0 },
    { 0x233758f2, 90 },
    { 0, 0 },
    { 0xb8c758f4, 78 },
    { 0x695758f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77758fb, 90 },
    { 0x46f758fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c75902, 98 },
    { 0x8f075903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b75906, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b75909, 96 },
    { 0xd9275909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e175910, 35 },
    { 0x3a475910, 46 },
    { 0, 0 },
    { 0xd5775913, 66 },
    { 0x9e775914, 61 },
    { 0x4a775915, 33 },
    { 0x4ff75916, 98 },
    { 0xb3575917, 91 },
    { 0x85f75915, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92475920, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec75927, 23 },
    { 0xf8875927, 61 },
    { 0x81975929, 33 },
    { 0x89875929, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c75930, 33 },
    { 0, 0 },
    { 0x48a75932, 98 },
    { 0, 0 },
    { 0x13275934, 93 },
    { 0, 0 },
    { 0xc5f75936, 64 },
    { 0x28575937, 99 },
    { 0xcab75937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de7593b, 33 },
    { 0, 0 },
    { 0xbed7593d, 85 },
    { 0xc117593d, 98 },
    { 0xa037593f, 23 },
    { 0xc307593f, 33 },
    { 0xbb075940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45375946, 23 },
    { 0x52b75947, 97 },
    { 0, 0 },
    { 0xae775949, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e7594e, 33 },
    { 0xd1c7594e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f275955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6475958, 81 },
    { 0x32575958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df7595c, 99 },
    { 0, 0 },
    { 0x26c7595e, 90 },
    { 0x5d17595f, 55 },
    { 0, 0 },
    { 0xda075961, 76 },
    { 0, 0 },
    { 0xcc775963, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e75968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7596b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c7596f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36575975, 90 },
    { 0x7d975975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa267597a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe297597d, 33 },
    { 0x4f87597e, 87 },
    { 0xe6f7597e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43775988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2175991, 23 },
    { 0, 0 },
    { 0x13f75993, 23 },
    { 0, 0 },
    { 0x39575995, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db75998, 87 },
    { 0xb9075999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf7599c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475759a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c759a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae759ae, 43 },
    { 0x371759af, 64 },
    { 0xebe759b0, 66 },
    { 0x8fb759b0, 34 },
    { 0x71d759b2, 95 },
    { 0x622759b3, 55 },
    { 0x108759b3, 95 },
    { 0, 0 },
    { 0x17759b6, 98 },
    { 0, 0 },
    { 0x869759b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd759bf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553759c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211759d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728759dc, 23 },
    { 0xcc4759dc, 83 },
    { 0xc13759de, 61 },
    { 0x62d759df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45759e4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913759e8, 76 },
    { 0, 0 },
    { 0xc65759ea, 43 },
    { 0x8d3759eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1759ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301759f2, 76 },
    { 0, 0 },
    { 0xeca759f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x550759f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5759fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7759fe, 33 },
    { 0x963759fe, 95 },
    { 0x76875a00, 46 },
    { 0x7b775a01, 95 },
    { 0x2a675a00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6875a08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa575a0d, 95 },
    { 0xedb75a0e, 87 },
    { 0x88975a0e, 98 },
    { 0xb6475a10, 98 },
    { 0xfdc75a11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de75a15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc75a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6275a1c, 78 },
    { 0x52375a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62975a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8975a28, 98 },
    { 0xd375a28, 95 },
    { 0x7a175a2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73975a2f, 61 },
    { 0, 0 },
    { 0xf1f75a31, 98 },
    { 0x5f075a32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74775a36, 90 },
    { 0xf4275a36, 34 },
    { 0x17075a36, 90 },
    { 0x9b575a37, 90 },
    { 0x43875a3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96375a3e, 90 },
    { 0x38575a3f, 34 },
    { 0x9b175a3e, 98 },
    { 0x98575a40, 33 },
    { 0x6bd75a41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f75a45, 93 },
    { 0x36975a46, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4375a4b, 23 },
    { 0xb7575a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb75a51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a75a56, 91 },
    { 0x21275a56, 98 },
    { 0xc7775a58, 23 },
    { 0xfad75a59, 98 },
    { 0, 0 },
    { 0x35775a5b, 66 },
    { 0x32675a5c, 67 },
    { 0xc3575a5d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d75a66, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c75a69, 91 },
    { 0xa5e75a6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc975a6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c75a77, 95 },
    { 0x88275a77, 55 },
    { 0, 0 },
    { 0x6c175a7a, 95 },
    { 0xb0875a7a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78475a7f, 23 },
    { 0, 0 },
    { 0xdc175a81, 55 },
    { 0x5de75a81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a575a85, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f75a88, 61 },
    { 0, 0 },
    { 0xc8f75a8a, 34 },
    { 0, 0 },
    { 0xb8f75a8c, 95 },
    { 0xd7875a8d, 61 },
    { 0, 0 },
    { 0x6d575a8f, 67 },
    { 0x76375a8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc75a93, 33 },
    { 0x39075a94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35275a9c, 66 },
    { 0xd8575a9d, 90 },
    { 0x5e175a9c, 98 },
    { 0, 0 },
    { 0x26675aa0, 55 },
    { 0x53b75aa0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f75aa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96475aab, 95 },
    { 0x68f75aac, 34 },
    { 0x1b575aad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13675ab5, 78 },
    { 0, 0 },
    { 0x81c75ab7, 43 },
    { 0xe8475ab8, 75 },
    { 0x70175ab9, 91 },
    { 0x4e875aba, 61 },
    { 0xabe75abb, 95 },
    { 0x1675abc, 90 },
    { 0x8be75abd, 95 },
    { 0xdde75abe, 90 },
    { 0x8a575abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5675ac2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff75acf, 90 },
    { 0x1b175ad0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d75ad4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13475ada, 90 },
    { 0x70675adb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b775ae3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e075ae6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2475ae9, 43 },
    { 0x18b75ae9, 61 },
    { 0xd8e75aeb, 33 },
    { 0x7b875aec, 67 },
    { 0xab75aec, 23 },
    { 0, 0 },
    { 0x5da75aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca075af2, 33 },
    { 0xb0875af3, 61 },
    { 0xb4775af4, 98 },
    { 0, 0 },
    { 0x59575af6, 84 },
    { 0xa2375af7, 23 },
    { 0, 0 },
    { 0x90175af9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x53375afc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf75b00, 85 },
    { 0, 0 },
    { 0xbca75b02, 33 },
    { 0, 0 },
    { 0x6b175b04, 61 },
    { 0x10775b05, 95 },
    { 0x78f75b06, 23 },
    { 0x54475b07, 17 },
    { 0x7e75b07, 76 },
    { 0x64875b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9375b0e, 100 },
    { 0, 0 },
    { 0x17575b10, 100 },
    { 0x2b175b11, 84 },
    { 0xbda75b10, 88 },
    { 0, 0 },
    { 0xee475b14, 76 },
    { 0xa2f75b15, 46 },
    { 0, 0 },
    { 0x5e075b17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea575b20, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b75b2e, 61 },
    { 0xa2175b2e, 61 },
    { 0xedd75b2e, 55 },
    { 0, 0 },
    { 0x7075b32, 90 },
    { 0x6a275b33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5575b36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd75b39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e75b3f, 55 },
    { 0x95d75b40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36675b44, 90 },
    { 0xb6375b45, 90 },
    { 0, 0 },
    { 0xe5075b47, 90 },
    { 0xc5975b48, 61 },
    { 0xd3375b49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6475b4e, 61 },
    { 0xe675b4f, 33 },
    { 0x54a75b4e, 33 },
    { 0, 0 },
    { 0x4a375b52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf75b57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a75b5c, 95 },
    { 0, 0 },
    { 0xe0d75b5e, 98 },
    { 0xd3175b5f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea75b64, 90 },
    { 0x11075b65, 88 },
    { 0, 0 },
    { 0x5d275b67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b375b6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd75b6d, 98 },
    { 0, 0 },
    { 0xe6875b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f675b73, 23 },
    { 0, 0 },
    { 0xe4675b75, 90 },
    { 0x4d475b76, 66 },
    { 0x9f475b75, 33 },
    { 0x5b375b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f75b7c, 95 },
    { 0, 0 },
    { 0x15475b7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3875b81, 23 },
    { 0, 0 },
    { 0xa7a75b83, 23 },
    { 0, 0 },
    { 0xada75b85, 61 },
    { 0x61675b86, 66 },
    { 0xf9b75b86, 93 },
    { 0xb9d75b88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e75b8b, 61 },
    { 0, 0 },
    { 0x92575b8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc75b90, 34 },
    { 0, 0 },
    { 0xe4b75b92, 33 },
    { 0xf775b93, 98 },
    { 0x86c75b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45775b97, 98 },
    { 0x55375b97, 93 },
    { 0x5f775b98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f75b9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59175ba4, 83 },
    { 0, 0 },
    { 0x34e75ba6, 91 },
    { 0xc7c75ba7, 76 },
    { 0x6f275ba8, 34 },
    { 0x7ca75ba9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59675bae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e75bb3, 23 },
    { 0x4175bb4, 95 },
    { 0, 0 },
    { 0x59775bb6, 55 },
    { 0, 0 },
    { 0x78075bb8, 13 },
    { 0xdd275bb8, 34 },
    { 0xedb75bba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd75bbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6675bc1, 91 },
    { 0, 0 },
    { 0x61175bc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b375bc7, 77 },
    { 0x58f75bc8, 61 },
    { 0x4ba75bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9175bd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd75bd4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b75bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db75bdb, 23 },
    { 0xfa375bdc, 34 },
    { 0x14f75bdb, 23 },
    { 0, 0 },
    { 0x19075bdf, 67 },
    { 0xe7775be0, 95 },
    { 0xbf875be1, 66 },
    { 0x5475be1, 40 },
    { 0, 0 },
    { 0xcdf75be4, 33 },
    { 0, 0 },
    { 0x4a175be6, 98 },
    { 0x94475be6, 46 },
    { 0xcf75be7, 17 },
    { 0, 0 },
    { 0x84c75bea, 90 },
    { 0x7175beb, 98 },
    { 0x17e75bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba175bf4, 55 },
    { 0x67375bf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5875bf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10375bfc, 83 },
    { 0x73675bfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5675c02, 55 },
    { 0x21675c03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa375c07, 77 },
    { 0xbee75c08, 88 },
    { 0x3be75c07, 61 },
    { 0xa2875c09, 91 },
    { 0, 0 },
    { 0xac575c0c, 33 },
    { 0, 0 },
    { 0x1b575c0e, 43 },
    { 0x96775c0f, 67 },
    { 0x44775c0e, 95 },
    { 0xdc575c11, 93 },
    { 0xcda75c11, 98 },
    { 0x5ff75c0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c775c16, 33 },
    { 0xfdb75c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a775c1e, 46 },
    { 0xd9a75c1e, 99 },
    { 0xa4975c20, 65 },
    { 0x57975c1e, 95 },
    { 0xbab75c22, 46 },
    { 0xc9375c21, 55 },
    { 0x14675c24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca75c29, 87 },
    { 0xf3675c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe75c2e, 61 },
    { 0, 0 },
    { 0xaf575c30, 98 },
    { 0x55b75c31, 98 },
    { 0xb0875c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a475c35, 90 },
    { 0, 0 },
    { 0xea675c37, 93 },
    { 0x34675c38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63375c3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea175c41, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5575c45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf75c48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72275c50, 23 },
    { 0x2d875c51, 42 },
    { 0x8e975c51, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1575c55, 95 },
    { 0xfae75c56, 90 },
    { 0xf9775c57, 75 },
    { 0x5bf75c57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8775c61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e75c66, 23 },
    { 0x57675c67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c275c6d, 23 },
    { 0xd8e75c6e, 53 },
    { 0xd3075c6f, 23 },
    { 0x90575c70, 43 },
    { 0x6b575c70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f75c75, 95 },
    { 0x4d575c76, 76 },
    { 0xe7675c77, 66 },
    { 0xc6375c76, 61 },
    { 0x51275c79, 36 },
    { 0x41575c77, 66 },
    { 0xf5a75c7b, 99 },
    { 0xa7d75c7c, 88 },
    { 0x4ea75c7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5675c82, 33 },
    { 0xd7875c83, 33 },
    { 0x8bb75c84, 61 },
    { 0xabf75c82, 87 },
    { 0xd6675c82, 90 },
    { 0x4a975c87, 87 },
    { 0x7ee75c88, 90 },
    { 0x59975c88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77675c8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1975c8f, 61 },
    { 0x29875c90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf575c93, 33 },
    { 0x2d375c94, 95 },
    { 0, 0 },
    { 0xf8775c96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67675c99, 61 },
    { 0x47875c99, 91 },
    { 0xda375c9b, 43 },
    { 0xbe975c9c, 33 },
    { 0, 0 },
    { 0x86275c9e, 93 },
    { 0, 0 },
    { 0x56575ca0, 90 },
    { 0x9e075ca1, 76 },
    { 0x41375ca2, 33 },
    { 0x69e75ca2, 33 },
    { 0x575ca4, 17 },
    { 0, 0 },
    { 0x55a75ca6, 76 },
    { 0x69e75ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2175caa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e75cae, 61 },
    { 0x9475cae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55775cb7, 43 },
    { 0xcd275cb8, 98 },
    { 0xe5575cb8, 98 },
    { 0x71b75cb9, 34 },
    { 0x27775cbb, 61 },
    { 0xa5d75cbb, 66 },
    { 0x51675cb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43175cc6, 61 },
    { 0xac275cc7, 34 },
    { 0xd375cc8, 95 },
    { 0xa6075cc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd175ccd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9575cd2, 33 },
    { 0x9f275cd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe75cda, 76 },
    { 0xc7475cdb, 43 },
    { 0xb8c75cdc, 87 },
    { 0x1875cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e75ce2, 90 },
    { 0x84275ce3, 96 },
    { 0x55275ce4, 98 },
    { 0xb9a75ce2, 87 },
    { 0, 0 },
    { 0xd6f75ce7, 95 },
    { 0x7dd75ce8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce75ceb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa75cee, 95 },
    { 0xb8175cef, 61 },
    { 0x67475cf0, 95 },
    { 0x5975cf1, 43 },
    { 0x48875cf2, 55 },
    { 0x60175cf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e75cf6, 78 },
    { 0, 0 },
    { 0x79675cf8, 76 },
    { 0, 0 },
    { 0x53b75cfa, 33 },
    { 0x62275cfb, 34 },
    { 0xb6a75cfc, 75 },
    { 0xc4d75cfd, 90 },
    { 0, 0 },
    { 0x8e775cff, 87 },
    { 0, 0 },
    { 0xe9875d01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e75d07, 55 },
    { 0x4af75d08, 23 },
    { 0x73c75d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba875d0d, 95 },
    { 0x81b75d0d, 90 },
    { 0, 0 },
    { 0x79175d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa075d13, 43 },
    { 0x4d975d13, 46 },
    { 0x90b75d13, 23 },
    { 0xf7175d14, 99 },
    { 0, 0 },
    { 0xded75d18, 82 },
    { 0x1c75d19, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d75d1d, 90 },
    { 0x58175d1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76475d25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3175d28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1875d2d, 23 },
    { 0x4d275d2e, 23 },
    { 0x4f975d2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb075d33, 66 },
    { 0, 0 },
    { 0x29c75d35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace75d3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b75d48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c275d4d, 77 },
    { 0x44475d4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc975d51, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c575d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc75d5a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c775d5d, 100 },
    { 0x24c75d5e, 46 },
    { 0xcd275d5e, 61 },
    { 0, 0 },
    { 0x9b675d61, 90 },
    { 0, 0 },
    { 0x2ba75d63, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8675d67, 51 },
    { 0xc4375d68, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb575d6d, 43 },
    { 0, 0 },
    { 0x94375d6f, 90 },
    { 0, 0 },
    { 0x45375d71, 78 },
    { 0, 0 },
    { 0xf7d75d73, 33 },
    { 0, 0 },
    { 0x65f75d75, 81 },
    { 0, 0 },
    { 0x79c75d77, 33 },
    { 0xa6175d78, 90 },
    { 0x3a475d78, 90 },
    { 0xa1e75d7a, 43 },
    { 0x80475d77, 55 },
    { 0xd6775d7c, 67 },
    { 0x63075d79, 66 },
    { 0xfc875d7e, 90 },
    { 0, 0 },
    { 0x7c675d80, 55 },
    { 0, 0 },
    { 0x8c375d82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e75d87, 77 },
    { 0, 0 },
    { 0x90375d89, 61 },
    { 0x6b75d89, 84 },
    { 0x7cc75d8b, 88 },
    { 0, 0 },
    { 0xdba75d8d, 23 },
    { 0, 0 },
    { 0xe2575d8f, 90 },
    { 0, 0 },
    { 0xd2375d91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c75d95, 90 },
    { 0xf6775d95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1875d99, 76 },
    { 0x14c75d9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd75d9f, 46 },
    { 0xbff75da0, 85 },
    { 0, 0 },
    { 0x87375da2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c375da7, 95 },
    { 0, 0 },
    { 0x28e75da9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c75dac, 59 },
    { 0x9b675dac, 95 },
    { 0xaa475dae, 52 },
    { 0x12375dae, 55 },
    { 0x38c75db0, 76 },
    { 0x95c75db1, 52 },
    { 0x86875db2, 95 },
    { 0x53b75daf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f075db7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a75dbf, 55 },
    { 0xa375dc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69275dc4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a75dc8, 95 },
    { 0xa2875dc8, 95 },
    { 0, 0 },
    { 0x60075dcb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb775dce, 34 },
    { 0, 0 },
    { 0x28b75dd0, 93 },
    { 0xac975dd0, 33 },
    { 0xdfa75dd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec75dd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c75ddb, 76 },
    { 0x58375ddc, 81 },
    { 0x50d75ddd, 33 },
    { 0xce975dde, 90 },
    { 0, 0 },
    { 0x9e775de0, 98 },
    { 0x8de75de1, 76 },
    { 0x81675de0, 66 },
    { 0x79a75de3, 91 },
    { 0x92975de4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b175ded, 33 },
    { 0x55975dee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675df5, 52 },
    { 0x88175df6, 90 },
    { 0x675df7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8875dfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe275e00, 84 },
    { 0, 0 },
    { 0x4ae75e02, 95 },
    { 0xc8375e03, 93 },
    { 0x87a75e04, 33 },
    { 0x83d75e03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5675e09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d375e11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae875e15, 55 },
    { 0x1a775e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f75e1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x56975e1f, 84 },
    { 0x7e175e20, 33 },
    { 0x70575e21, 94 },
    { 0x3eb75e22, 61 },
    { 0xa4475e23, 67 },
    { 0, 0 },
    { 0x50775e25, 99 },
    { 0xd9375e26, 98 },
    { 0, 0 },
    { 0x12875e28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df75e2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3275e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2375e41, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc75e45, 98 },
    { 0x76e75e46, 83 },
    { 0, 0 },
    { 0x3f375e48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c875e4d, 68 },
    { 0, 0 },
    { 0xe3f75e4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae75e54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f75e57, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0575e5c, 43 },
    { 0, 0 },
    { 0x2e175e5e, 23 },
    { 0x5ce75e5f, 61 },
    { 0, 0 },
    { 0x18475e61, 23 },
    { 0x9a975e61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e75e67, 66 },
    { 0, 0 },
    { 0xb3a75e69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d475e7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c775e7e, 93 },
    { 0xd7d75e7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f75e82, 95 },
    { 0xbb175e82, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30475e87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d75e8a, 84 },
    { 0xfda75e8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e75e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a75e95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c75eac, 43 },
    { 0x2f375eac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87075eb0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c75eb3, 100 },
    { 0, 0 },
    { 0xf1c75eb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b75eba, 61 },
    { 0, 0 },
    { 0x4e775ebc, 85 },
    { 0xe8675ebc, 87 },
    { 0x53175ebc, 23 },
    { 0xdc775ebc, 95 },
    { 0x66d75ec0, 90 },
    { 0x57475ec1, 75 },
    { 0x95775ec2, 34 },
    { 0xeda75ec3, 52 },
    { 0xf1f75ebd, 90 },
    { 0x5f575ec5, 43 },
    { 0xdfe75ec2, 65 },
    { 0, 0 },
    { 0x79275ec8, 23 },
    { 0x65e75ec9, 90 },
    { 0xb4075ec8, 34 },
    { 0x78675ecb, 61 },
    { 0, 0 },
    { 0x98d75ecd, 90 },
    { 0, 0 },
    { 0xb3f75ecf, 95 },
    { 0x34e75ed0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a75ed3, 33 },
    { 0xe7775ed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60675edb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a75ee0, 93 },
    { 0xf4575ee1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a75ee5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d175ee8, 33 },
    { 0, 0 },
    { 0xc5175eea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6075eed, 95 },
    { 0, 0 },
    { 0x4d975eef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d175ef2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a875efa, 95 },
    { 0x92675efb, 95 },
    { 0x3e375efc, 34 },
    { 0, 0 },
    { 0x5f075efe, 23 },
    { 0, 0 },
    { 0x62a75f00, 100 },
    { 0x32b75f01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3775f08, 34 },
    { 0x27375f08, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb475f0c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8975f0f, 98 },
    { 0x4b775f10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c75f14, 98 },
    { 0xa5275f15, 33 },
    { 0x58475f16, 96 },
    { 0x16c75f16, 85 },
    { 0, 0 },
    { 0xec775f19, 94 },
    { 0xb9f75f1a, 99 },
    { 0, 0 },
    { 0x2bf75f1c, 95 },
    { 0x86375f1d, 34 },
    { 0x38875f1e, 76 },
    { 0x71e75f1f, 90 },
    { 0xd0775f20, 55 },
    { 0x71075f20, 95 },
    { 0x24b75f22, 90 },
    { 0x6dd75f23, 98 },
    { 0xb475f22, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a75f27, 23 },
    { 0x99775f27, 34 },
    { 0x2c575f27, 87 },
    { 0x62375f2a, 52 },
    { 0x50175f2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db75f36, 23 },
    { 0x87e75f36, 86 },
    { 0xc7375f38, 95 },
    { 0, 0 },
    { 0x50b75f3a, 23 },
    { 0xbd975f3b, 76 },
    { 0x43075f3a, 95 },
    { 0x44e75f3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f75f43, 76 },
    { 0xdc375f43, 66 },
    { 0, 0 },
    { 0x69f75f46, 61 },
    { 0, 0 },
    { 0xe2e75f48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49475f4b, 61 },
    { 0x88c75f4c, 33 },
    { 0xfd75f4d, 95 },
    { 0x37375f4e, 23 },
    { 0x6de75f4f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae75f55, 76 },
    { 0x29675f56, 93 },
    { 0xec175f56, 91 },
    { 0x3f675f56, 33 },
    { 0x1d575f56, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95275f5e, 90 },
    { 0xf4175f5e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c575f62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5675f66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc775f6c, 90 },
    { 0x6a475f6d, 90 },
    { 0x55a75f6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb475f72, 61 },
    { 0xa6a75f73, 98 },
    { 0x60d75f74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52775f7b, 43 },
    { 0, 0 },
    { 0x49675f7d, 61 },
    { 0x4ef75f7d, 90 },
    { 0, 0 },
    { 0xb5875f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66375f83, 76 },
    { 0x42e75f84, 91 },
    { 0x4f175f84, 68 },
    { 0, 0 },
    { 0x77d75f87, 76 },
    { 0, 0 },
    { 0x4ea75f89, 90 },
    { 0, 0 },
    { 0x94475f8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a75f8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b75f93, 61 },
    { 0, 0 },
    { 0x3f975f95, 65 },
    { 0x4fe75f96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a275f99, 94 },
    { 0xa0d75f99, 33 },
    { 0x29475f9b, 98 },
    { 0x49675f9c, 95 },
    { 0x16a75f9d, 61 },
    { 0xc3275f9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca75fa7, 87 },
    { 0x30375fa7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d75fac, 33 },
    { 0x82f75fad, 33 },
    { 0x9e475fae, 33 },
    { 0x31b75fac, 80 },
    { 0x98a75fb0, 33 },
    { 0, 0 },
    { 0xace75fb2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f75fb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5475fba, 33 },
    { 0xc2275fbb, 90 },
    { 0x97575fbc, 61 },
    { 0x49475fbb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f75fc3, 94 },
    { 0xe9875fc4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89475fc8, 95 },
    { 0xdec75fc9, 90 },
    { 0x90a75fc9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7975fd1, 34 },
    { 0xb9275fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f75fd9, 34 },
    { 0, 0 },
    { 0x8ac75fdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5075fde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb975fe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6175fe7, 89 },
    { 0, 0 },
    { 0x6b575fe9, 84 },
    { 0xaea75fea, 34 },
    { 0x2e675feb, 46 },
    { 0, 0 },
    { 0xe5a75fed, 77 },
    { 0, 0 },
    { 0x96075fef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac175ffd, 98 },
    { 0xd8b75ffe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f76006, 33 },
    { 0, 0 },
    { 0xb3a76008, 95 },
    { 0, 0 },
    { 0xbeb7600a, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b7600e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59176014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7601a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc357601d, 93 },
    { 0, 0 },
    { 0x57601f, 33 },
    { 0x3ce76020, 34 },
    { 0, 0 },
    { 0x2f576022, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf76025, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da7602a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc7602e, 90 },
    { 0xf7e7602e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f76032, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf7603e, 66 },
    { 0, 0 },
    { 0xaaf76040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3876048, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b07604b, 46 },
    { 0x5037604c, 95 },
    { 0x1ce7604d, 67 },
    { 0xd7d7604d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c76051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0676054, 55 },
    { 0xb276055, 95 },
    { 0xc7676056, 96 },
    { 0, 0 },
    { 0x7d876058, 46 },
    { 0x4f076058, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c7605d, 76 },
    { 0xbb77605e, 55 },
    { 0x9167605f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6476063, 90 },
    { 0x20376064, 38 },
    { 0xc7476065, 78 },
    { 0x25e76064, 33 },
    { 0x1cc76067, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd167606d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a676072, 34 },
    { 0x6d176073, 43 },
    { 0, 0 },
    { 0x1e276075, 23 },
    { 0x38076075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf676088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7608b, 95 },
    { 0xd457608b, 61 },
    { 0x5ff7608c, 95 },
    { 0xdf37608e, 55 },
    { 0xc127608b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98976092, 77 },
    { 0xe9d76093, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77976098, 33 },
    { 0x52f76099, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8807609e, 61 },
    { 0x5057609e, 33 },
    { 0xee0760a0, 81 },
    { 0x25b7609e, 55 },
    { 0xe6a760a2, 88 },
    { 0, 0 },
    { 0x13d760a4, 46 },
    { 0, 0 },
    { 0x58760a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7760ab, 67 },
    { 0x247760ac, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793760b7, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6760bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a760c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59760c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4760d5, 87 },
    { 0xfdb760d6, 33 },
    { 0, 0 },
    { 0x26c760d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06760db, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3760de, 99 },
    { 0x7a6760df, 66 },
    { 0xe58760e0, 76 },
    { 0, 0 },
    { 0x1b0760e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9760e5, 55 },
    { 0x785760e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1760e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567760ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806760f1, 90 },
    { 0x519760f2, 46 },
    { 0xf13760f3, 88 },
    { 0xc7f760f4, 88 },
    { 0, 0 },
    { 0x1e9760f6, 61 },
    { 0xc6e760f6, 61 },
    { 0xba9760f8, 23 },
    { 0x78e760f8, 58 },
    { 0, 0 },
    { 0x50b760fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5476100, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f76103, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c76106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae7610a, 33 },
    { 0x61e7610a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497610f, 91 },
    { 0, 0 },
    { 0x6af76111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc476117, 98 },
    { 0xe4876117, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47611b, 87 },
    { 0xb347611c, 90 },
    { 0, 0 },
    { 0x60f7611e, 95 },
    { 0, 0 },
    { 0xdb776120, 98 },
    { 0x1ed76120, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f176125, 65 },
    { 0x12a76126, 52 },
    { 0xfc076126, 40 },
    { 0xa7476128, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d87612c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d57612f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e76133, 93 },
    { 0, 0 },
    { 0xf076135, 98 },
    { 0x16776136, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda67613b, 34 },
    { 0, 0 },
    { 0x9ea7613d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b76140, 66 },
    { 0, 0 },
    { 0xfbe76142, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee76146, 39 },
    { 0xc1476146, 33 },
    { 0xc6176148, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb767614c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec476159, 99 },
    { 0x9f97615a, 90 },
    { 0xc367615a, 52 },
    { 0x4347615c, 90 },
    { 0x3c47615b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9176161, 33 },
    { 0, 0 },
    { 0xa0676163, 65 },
    { 0xd2d76163, 23 },
    { 0xde476165, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597616b, 66 },
    { 0xe407616c, 78 },
    { 0x9107616d, 17 },
    { 0xea57616d, 61 },
    { 0x1927616e, 61 },
    { 0x3137616b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde776174, 78 },
    { 0x2ec76175, 84 },
    { 0x8f776175, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7617c, 84 },
    { 0, 0 },
    { 0x4217617e, 80 },
    { 0x9627617f, 90 },
    { 0x85176180, 98 },
    { 0x39576181, 98 },
    { 0x75e76180, 23 },
    { 0x6276181, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa976187, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc7618a, 90 },
    { 0x68e7618a, 87 },
    { 0x8ee7618c, 90 },
    { 0, 0 },
    { 0x4ba7618e, 61 },
    { 0x6a7618f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58276192, 40 },
    { 0, 0 },
    { 0xefb76194, 90 },
    { 0x2bd76195, 55 },
    { 0xd2876196, 93 },
    { 0x88676197, 90 },
    { 0x95476198, 23 },
    { 0, 0 },
    { 0x3a37619a, 95 },
    { 0x7b7619a, 93 },
    { 0x2347619c, 90 },
    { 0, 0 },
    { 0x3de7619e, 23 },
    { 0x1fe7619e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c761a6, 67 },
    { 0x6fc761a7, 81 },
    { 0x464761a8, 34 },
    { 0, 0 },
    { 0x151761aa, 76 },
    { 0x1fb761ab, 23 },
    { 0x86e761ac, 23 },
    { 0, 0 },
    { 0xcb4761ae, 61 },
    { 0xd2e761ae, 33 },
    { 0x9de761af, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6761b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f761b7, 95 },
    { 0x4cc761b7, 90 },
    { 0xd2d761b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0761bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9761c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43761c4, 46 },
    { 0xaee761c4, 55 },
    { 0xf20761c6, 90 },
    { 0x373761c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce761cb, 95 },
    { 0, 0 },
    { 0x7cd761cd, 34 },
    { 0xd6e761ce, 33 },
    { 0x766761cd, 91 },
    { 0, 0 },
    { 0x369761d1, 33 },
    { 0, 0 },
    { 0xc4e761d3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231761d7, 95 },
    { 0, 0 },
    { 0x5c8761d9, 34 },
    { 0, 0 },
    { 0x2cd761db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47761e0, 88 },
    { 0, 0 },
    { 0xebd761e2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad761e7, 81 },
    { 0xe24761e7, 52 },
    { 0x6e3761e9, 96 },
    { 0xca6761e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb761ed, 99 },
    { 0x5d6761ee, 98 },
    { 0xb71761ed, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af761f2, 95 },
    { 0xad761f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x588761f6, 61 },
    { 0xb0b761f7, 98 },
    { 0x424761f7, 61 },
    { 0x3ab761f8, 90 },
    { 0xb72761f9, 33 },
    { 0x58d761f7, 87 },
    { 0, 0 },
    { 0x2e3761fd, 55 },
    { 0x86c761fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf76205, 76 },
    { 0, 0 },
    { 0x59d76207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9127620b, 98 },
    { 0x4797620b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1097620f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f76214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b76217, 95 },
    { 0, 0 },
    { 0x6776219, 99 },
    { 0x5337621a, 23 },
    { 0xacf7621b, 67 },
    { 0x1bc76219, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc067621f, 34 },
    { 0x6d876220, 33 },
    { 0x90376220, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3476227, 66 },
    { 0xc4276227, 23 },
    { 0xbaf76229, 51 },
    { 0xde276228, 55 },
    { 0, 0 },
    { 0x99f7622c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac476230, 90 },
    { 0x27776230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda776234, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf277623a, 75 },
    { 0xb667623b, 17 },
    { 0x53b7623c, 93 },
    { 0x7a57623c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36776240, 98 },
    { 0x81d76241, 67 },
    { 0, 0 },
    { 0x6a76243, 100 },
    { 0xf6d76244, 9 },
    { 0x7a176245, 52 },
    { 0xd7d76245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b7624b, 33 },
    { 0x5b7624c, 34 },
    { 0xcd17624d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93876258, 61 },
    { 0, 0 },
    { 0x3e17625a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1117625d, 66 },
    { 0x2f87625e, 46 },
    { 0x667625e, 36 },
    { 0x82476260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d76263, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7447626a, 81 },
    { 0xd5d7626a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4976270, 33 },
    { 0, 0 },
    { 0x25776272, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a976278, 58 },
    { 0xd4a76279, 78 },
    { 0x9e576278, 61 },
    { 0x72f76279, 43 },
    { 0xda87627c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3776281, 34 },
    { 0x2a976281, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc76286, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a87628f, 90 },
    { 0, 0 },
    { 0x2c176291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76276295, 33 },
    { 0xb1a76295, 23 },
    { 0x8e976297, 88 },
    { 0x8c276297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd297629e, 66 },
    { 0x4147629e, 33 },
    { 0x607762a0, 87 },
    { 0x56f762a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7762ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c762b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8762b3, 87 },
    { 0x73762b4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03762b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582762bc, 76 },
    { 0x15b762bd, 23 },
    { 0, 0 },
    { 0xb7e762bf, 90 },
    { 0xeb0762c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae762c3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55762c7, 95 },
    { 0, 0 },
    { 0xac1762c9, 68 },
    { 0x517762ca, 33 },
    { 0x3bf762cb, 95 },
    { 0x975762cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da762cf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf762d4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91762d7, 87 },
    { 0x67c762d8, 93 },
    { 0xd6c762d9, 87 },
    { 0, 0 },
    { 0x88e762db, 61 },
    { 0xebd762dc, 55 },
    { 0xfdf762dd, 78 },
    { 0xd26762db, 95 },
    { 0, 0 },
    { 0xa7f762e0, 76 },
    { 0, 0 },
    { 0xfba762e2, 34 },
    { 0xd40762e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad762e7, 98 },
    { 0, 0 },
    { 0xcf3762e9, 33 },
    { 0, 0 },
    { 0xc8f762eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906762ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bc762f2, 98 },
    { 0x296762f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1762f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d762fb, 33 },
    { 0, 0 },
    { 0x3f1762fd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0276308, 95 },
    { 0x3ad76309, 80 },
    { 0x6317630a, 98 },
    { 0, 0 },
    { 0x70d7630c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb677630f, 56 },
    { 0, 0 },
    { 0x42f76311, 55 },
    { 0x79576312, 61 },
    { 0x9e476312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f7631f, 33 },
    { 0xba576320, 34 },
    { 0xd1976320, 55 },
    { 0x9dc76322, 66 },
    { 0xb0b76322, 33 },
    { 0, 0 },
    { 0x78176325, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b476328, 81 },
    { 0xd6d76329, 78 },
    { 0x3f97632a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7632e, 33 },
    { 0, 0 },
    { 0xb2176330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38076337, 88 },
    { 0, 0 },
    { 0xf4576339, 76 },
    { 0, 0 },
    { 0xd387633b, 76 },
    { 0, 0 },
    { 0x677633d, 81 },
    { 0, 0 },
    { 0x56c7633f, 93 },
    { 0x55a76340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d76344, 81 },
    { 0xec76344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26276348, 52 },
    { 0, 0 },
    { 0xf807634a, 84 },
    { 0, 0 },
    { 0xfc67634c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a76352, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f76356, 90 },
    { 0xd0576357, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c77635c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6847635f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f76362, 84 },
    { 0, 0 },
    { 0x31a76364, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9276367, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7636e, 95 },
    { 0, 0 },
    { 0x76876370, 17 },
    { 0xafc76371, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e76375, 67 },
    { 0xfd176375, 90 },
    { 0x19576375, 98 },
    { 0x75576378, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77637c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15076380, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c976385, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87576388, 95 },
    { 0, 0 },
    { 0x9b7638a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a076390, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae76393, 87 },
    { 0xe876394, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a76397, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae7639b, 23 },
    { 0x3fe7639c, 98 },
    { 0x3f67639c, 23 },
    { 0x52c7639e, 23 },
    { 0x5687639d, 95 },
    { 0xebe763a0, 90 },
    { 0xdd9763a1, 95 },
    { 0xc36763a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc763a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb763a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6763ab, 95 },
    { 0xdcd763ac, 67 },
    { 0, 0 },
    { 0x884763ae, 85 },
    { 0, 0 },
    { 0x6c2763b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5763b3, 61 },
    { 0xbd763b4, 95 },
    { 0x815763b5, 23 },
    { 0xc83763b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6763b9, 95 },
    { 0xf7f763ba, 98 },
    { 0x833763bb, 85 },
    { 0xb06763bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd763c2, 77 },
    { 0, 0 },
    { 0x6a5763c4, 55 },
    { 0, 0 },
    { 0xf9e763c6, 90 },
    { 0xbd4763c7, 61 },
    { 0, 0 },
    { 0xe7f763c9, 17 },
    { 0xdfb763ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363763ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66763d2, 99 },
    { 0x25e763d3, 17 },
    { 0, 0 },
    { 0xba1763d5, 85 },
    { 0, 0 },
    { 0x1c0763d7, 40 },
    { 0, 0 },
    { 0xcb2763d9, 23 },
    { 0x8c3763d9, 76 },
    { 0xe18763db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe763de, 90 },
    { 0, 0 },
    { 0x720763e0, 33 },
    { 0x3d4763e1, 90 },
    { 0xf03763e1, 77 },
    { 0, 0 },
    { 0x31d763e4, 90 },
    { 0x39a763e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc763eb, 61 },
    { 0x76763ec, 95 },
    { 0x43e763eb, 82 },
    { 0xcf6763ee, 93 },
    { 0x11c763ef, 75 },
    { 0x200763eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238763f7, 34 },
    { 0, 0 },
    { 0x491763f9, 61 },
    { 0, 0 },
    { 0xa1a763fb, 33 },
    { 0x803763fc, 23 },
    { 0xb37763fd, 68 },
    { 0xd08763fd, 55 },
    { 0x60e763fb, 98 },
    { 0, 0 },
    { 0xb4c76401, 98 },
    { 0x2476402, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e776406, 52 },
    { 0, 0 },
    { 0xbec76408, 55 },
    { 0, 0 },
    { 0x6d97640a, 23 },
    { 0, 0 },
    { 0x16b7640c, 88 },
    { 0x3a97640d, 85 },
    { 0xc277640d, 96 },
    { 0x7307640e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6376412, 23 },
    { 0, 0 },
    { 0xd3476414, 23 },
    { 0xe1576414, 40 },
    { 0x37176416, 23 },
    { 0xda776417, 61 },
    { 0, 0 },
    { 0xc8c76419, 75 },
    { 0, 0 },
    { 0x6ee7641b, 33 },
    { 0x4477641c, 43 },
    { 0x1517641b, 95 },
    { 0xba17641e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89776424, 23 },
    { 0x80c76424, 61 },
    { 0x91a76424, 61 },
    { 0x67b76426, 61 },
    { 0x20476426, 66 },
    { 0xf4d76429, 84 },
    { 0, 0 },
    { 0x3c07642b, 61 },
    { 0xb107642c, 98 },
    { 0x2987642c, 90 },
    { 0xc1d7642e, 43 },
    { 0, 0 },
    { 0xd976430, 46 },
    { 0xc9476431, 23 },
    { 0x63976431, 76 },
    { 0xd1a76431, 33 },
    { 0x46876434, 87 },
    { 0xdf76435, 90 },
    { 0x32176430, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x48076439, 85 },
    { 0xa097643a, 78 },
    { 0x3fe7643b, 38 },
    { 0x88c7643c, 91 },
    { 0x96c7643b, 55 },
    { 0x227643c, 17 },
    { 0xd937643a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x14076442, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76076446, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07644b, 98 },
    { 0x6497644c, 90 },
    { 0x26f7644d, 95 },
    { 0, 0 },
    { 0x88f7644f, 90 },
    { 0, 0 },
    { 0x22376451, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89476456, 77 },
    { 0x99176457, 87 },
    { 0xbe576456, 85 },
    { 0, 0 },
    { 0xc3e7645a, 98 },
    { 0, 0 },
    { 0x94e7645c, 95 },
    { 0, 0 },
    { 0x4467645e, 98 },
    { 0x8937645f, 98 },
    { 0, 0 },
    { 0xb6976461, 98 },
    { 0, 0 },
    { 0xa9676463, 52 },
    { 0, 0 },
    { 0x3f776465, 76 },
    { 0, 0 },
    { 0xf7976467, 99 },
    { 0x27c76468, 34 },
    { 0xf8776469, 33 },
    { 0x82276469, 98 },
    { 0x7b17646b, 76 },
    { 0x41276469, 33 },
    { 0x27c7646a, 43 },
    { 0x4187646b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f776471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b76474, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7527647b, 61 },
    { 0x3c7647b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b7647f, 61 },
    { 0x8f376480, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf76486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b876489, 99 },
    { 0x2bc7648a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d7648d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8376495, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead76499, 90 },
    { 0xa8d76499, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a7649e, 43 },
    { 0, 0 },
    { 0x1aa764a0, 38 },
    { 0xc97764a1, 98 },
    { 0x3ba764a2, 33 },
    { 0x54764a2, 66 },
    { 0, 0 },
    { 0xb23764a5, 70 },
    { 0x9ad764a6, 23 },
    { 0x6c6764a7, 95 },
    { 0x876764a5, 55 },
    { 0x4b1764a9, 90 },
    { 0, 0 },
    { 0x6f7764ab, 95 },
    { 0x98764ab, 95 },
    { 0x477764ad, 34 },
    { 0xc42764ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc764b8, 46 },
    { 0xe22764b9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3764bc, 95 },
    { 0, 0 },
    { 0xe05764be, 9 },
    { 0xd65764be, 90 },
    { 0x132764c0, 55 },
    { 0xe68764c0, 76 },
    { 0x767764c2, 95 },
    { 0xf39764c3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173764cb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f764cf, 82 },
    { 0xb43764d0, 75 },
    { 0, 0 },
    { 0x188764d2, 98 },
    { 0x927764d3, 33 },
    { 0, 0 },
    { 0x5f6764d5, 61 },
    { 0x9e1764d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27764d9, 93 },
    { 0x771764da, 91 },
    { 0xe9b764da, 95 },
    { 0, 0 },
    { 0xb4b764dd, 23 },
    { 0xc91764de, 46 },
    { 0xb19764df, 43 },
    { 0x4da764df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10764f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40764f3, 98 },
    { 0xff8764f3, 33 },
    { 0x875764f5, 55 },
    { 0xad2764f6, 33 },
    { 0xe91764f6, 95 },
    { 0x523764f8, 33 },
    { 0xe39764f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba764fc, 33 },
    { 0, 0 },
    { 0x59d764fe, 33 },
    { 0xd0e764fe, 76 },
    { 0, 0 },
    { 0xea176501, 61 },
    { 0xfad76502, 33 },
    { 0x81876503, 95 },
    { 0x6cd76503, 33 },
    { 0, 0 },
    { 0xd1076506, 59 },
    { 0x8a376507, 43 },
    { 0, 0 },
    { 0xc4d76509, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad47650e, 23 },
    { 0, 0 },
    { 0xd8876510, 61 },
    { 0x57876511, 61 },
    { 0x47b76512, 93 },
    { 0x13676513, 91 },
    { 0x22476514, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e176517, 93 },
    { 0, 0 },
    { 0x4b576519, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97651f, 43 },
    { 0x30c76520, 85 },
    { 0xc2076521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9176529, 33 },
    { 0xd3776529, 55 },
    { 0x1167652a, 23 },
    { 0x5587652c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf227652f, 55 },
    { 0, 0 },
    { 0x23f76531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d77653a, 23 },
    { 0, 0 },
    { 0x9fd7653c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b76540, 77 },
    { 0, 0 },
    { 0xf9476542, 61 },
    { 0xdf876543, 93 },
    { 0x53476544, 55 },
    { 0x22776544, 75 },
    { 0x5cf76546, 17 },
    { 0x2e76546, 23 },
    { 0x1176545, 90 },
    { 0x53976549, 76 },
    { 0x9bb76549, 90 },
    { 0x80976549, 99 },
    { 0x64876544, 67 },
    { 0x5f476544, 95 },
    { 0x38f7654e, 24 },
    { 0xd2276548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35576552, 33 },
    { 0x5eb76553, 33 },
    { 0xee976552, 43 },
    { 0, 0 },
    { 0xdb576556, 55 },
    { 0, 0 },
    { 0xd1a76558, 66 },
    { 0xe0776558, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe76562, 78 },
    { 0xa4b76562, 98 },
    { 0, 0 },
    { 0x35b76565, 33 },
    { 0x5b76565, 93 },
    { 0xe9876566, 87 },
    { 0x75576565, 23 },
    { 0xc8a76567, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db7656d, 68 },
    { 0x6a47656d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3176571, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19776576, 100 },
    { 0x4ee76577, 55 },
    { 0, 0 },
    { 0xf8776579, 87 },
    { 0xee97657a, 88 },
    { 0x8507657b, 95 },
    { 0x2157657c, 66 },
    { 0x3647657c, 90 },
    { 0x9267657e, 33 },
    { 0x1357657c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74376584, 43 },
    { 0, 0 },
    { 0x85f76586, 33 },
    { 0x72e76587, 98 },
    { 0, 0 },
    { 0x22676589, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce7658f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3176593, 61 },
    { 0xdef76593, 55 },
    { 0x92a76595, 90 },
    { 0x35076595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3617659f, 61 },
    { 0x7367659f, 77 },
    { 0x4fe7659f, 93 },
    { 0, 0 },
    { 0xa93765a3, 23 },
    { 0, 0 },
    { 0x90765a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1765ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f765b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d765b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10765b7, 70 },
    { 0xdde765b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c765bb, 43 },
    { 0xc8f765bc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb765bf, 90 },
    { 0x972765c0, 76 },
    { 0xb26765c1, 78 },
    { 0x59765c0, 61 },
    { 0xeb9765c3, 55 },
    { 0, 0 },
    { 0x18e765c5, 55 },
    { 0, 0 },
    { 0x62765c7, 99 },
    { 0, 0 },
    { 0x9d5765c9, 23 },
    { 0x2e0765c9, 23 },
    { 0x990765cb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c765ce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb765d1, 65 },
    { 0, 0 },
    { 0x8b765d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x376765d6, 33 },
    { 0, 0 },
    { 0x34a765d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef765dd, 76 },
    { 0xc6a765dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a765e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x943765e6, 61 },
    { 0xa32765e7, 33 },
    { 0x2b765e7, 93 },
    { 0, 0 },
    { 0xf95765ea, 95 },
    { 0x1d1765eb, 46 },
    { 0, 0 },
    { 0x3dc765ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190765f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817765f6, 75 },
    { 0x5e1765f7, 55 },
    { 0x4f3765f7, 76 },
    { 0, 0 },
    { 0x4c2765fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca76602, 93 },
    { 0x8d976603, 100 },
    { 0xd0876603, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21176607, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287660b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e17660e, 55 },
    { 0x2a87660f, 67 },
    { 0x9f27660f, 34 },
    { 0x6e376611, 61 },
    { 0, 0 },
    { 0x71376613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7661d, 55 },
    { 0x19d7661e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f76621, 77 },
    { 0xf0576622, 77 },
    { 0, 0 },
    { 0xe1876624, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b076628, 90 },
    { 0, 0 },
    { 0x7ce7662a, 95 },
    { 0, 0 },
    { 0x5977662c, 81 },
    { 0x4767662d, 33 },
    { 0, 0 },
    { 0x5707662f, 61 },
    { 0, 0 },
    { 0x24276631, 95 },
    { 0x9ba76632, 61 },
    { 0x92b76633, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3476639, 33 },
    { 0, 0 },
    { 0x3b37663b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d76640, 87 },
    { 0xf476640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb76644, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8976648, 46 },
    { 0x8f476649, 98 },
    { 0xd576649, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb7664e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6776652, 61 },
    { 0x9f776653, 33 },
    { 0, 0 },
    { 0xfc576655, 95 },
    { 0, 0 },
    { 0xc2976657, 61 },
    { 0x9d676658, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4287665c, 61 },
    { 0x4b77665d, 33 },
    { 0xecb7665d, 52 },
    { 0xc507665e, 33 },
    { 0xaa776660, 91 },
    { 0x8d876661, 95 },
    { 0x3de76662, 46 },
    { 0xb1476663, 61 },
    { 0xa0b76664, 95 },
    { 0x26b76665, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8576669, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd7666e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62776671, 55 },
    { 0, 0 },
    { 0x2576673, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92476679, 98 },
    { 0x23f7667a, 98 },
    { 0x6947667b, 33 },
    { 0xca7667a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d676684, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x81276687, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db7668a, 93 },
    { 0, 0 },
    { 0x9ba7668c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b76694, 91 },
    { 0xeae76695, 90 },
    { 0xa7376696, 66 },
    { 0xe0476697, 75 },
    { 0x28a76698, 66 },
    { 0x76e76695, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d7669d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b766a1, 90 },
    { 0x87a766a1, 52 },
    { 0, 0 },
    { 0xa82766a4, 33 },
    { 0x3b6766a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58766a8, 91 },
    { 0, 0 },
    { 0x3fd766aa, 33 },
    { 0xa30766aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb766b2, 23 },
    { 0xf34766b2, 90 },
    { 0x71766b4, 40 },
    { 0x9ff766b4, 98 },
    { 0x916766b3, 76 },
    { 0x8bb766b4, 38 },
    { 0x935766b8, 17 },
    { 0xcfa766b9, 23 },
    { 0x5d0766ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed766be, 90 },
    { 0, 0 },
    { 0xef3766c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b766ca, 33 },
    { 0xc1d766ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f766d0, 95 },
    { 0xadd766d1, 17 },
    { 0x381766d2, 90 },
    { 0x586766d2, 46 },
    { 0x670766d3, 55 },
    { 0xac5766d1, 55 },
    { 0xd26766d6, 66 },
    { 0x734766d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x732766da, 48 },
    { 0x735766db, 37 },
    { 0x568766dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3766e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa766e5, 93 },
    { 0xd2766e6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a766ea, 80 },
    { 0x7af766ea, 100 },
    { 0, 0 },
    { 0xee5766ed, 61 },
    { 0, 0 },
    { 0x991766ef, 52 },
    { 0xd7d766f0, 90 },
    { 0x21f766f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e766f6, 87 },
    { 0x276766f6, 33 },
    { 0xa04766f6, 90 },
    { 0x4766f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a766fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d976701, 61 },
    { 0, 0 },
    { 0x9a076703, 90 },
    { 0x64c76704, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97670a, 90 },
    { 0x7747670b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3457670e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1376712, 46 },
    { 0xb5876713, 95 },
    { 0x61f76713, 65 },
    { 0, 0 },
    { 0x7d276716, 61 },
    { 0x2a276717, 43 },
    { 0xd3376718, 55 },
    { 0, 0 },
    { 0x8707671a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c7671e, 33 },
    { 0xb807671f, 23 },
    { 0x3cf76720, 87 },
    { 0x5a976720, 76 },
    { 0x2697671e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd76725, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38876733, 23 },
    { 0xbf76734, 23 },
    { 0x94f76734, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf476739, 35 },
    { 0, 0 },
    { 0x2887673b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee07673f, 76 },
    { 0x4cb76740, 90 },
    { 0xaa676740, 90 },
    { 0xf9a7673f, 90 },
    { 0x3a876743, 90 },
    { 0xe7876741, 90 },
    { 0, 0 },
    { 0xb0c76746, 95 },
    { 0x81776746, 84 },
    { 0, 0 },
    { 0xcbd76749, 76 },
    { 0x8e47674a, 87 },
    { 0x3557674b, 46 },
    { 0, 0 },
    { 0x56f7674d, 67 },
    { 0xe3c7674d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14476752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b76755, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2997675d, 90 },
    { 0, 0 },
    { 0xe97675f, 98 },
    { 0x99176760, 61 },
    { 0x82376760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac76764, 90 },
    { 0xa3a76765, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a276769, 33 },
    { 0x8827676a, 51 },
    { 0xb607676b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b76775, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca76778, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8487677b, 84 },
    { 0x2c37677c, 100 },
    { 0xe7f7677d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x376780, 90 },
    { 0, 0 },
    { 0x58b76782, 81 },
    { 0x2c176782, 43 },
    { 0xc9b76784, 76 },
    { 0, 0 },
    { 0x3de76786, 90 },
    { 0xab676786, 33 },
    { 0x22676787, 84 },
    { 0x38876788, 81 },
    { 0x4a976789, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e67678e, 99 },
    { 0xcc47678e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e76795, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d76799, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc7679e, 77 },
    { 0xf907679f, 55 },
    { 0, 0 },
    { 0x871767a1, 98 },
    { 0x4f9767a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7767a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3767b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544767b9, 95 },
    { 0x1cf767ba, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06767bd, 87 },
    { 0x76f767be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2767c2, 17 },
    { 0x893767c3, 100 },
    { 0, 0 },
    { 0x524767c5, 95 },
    { 0x405767c6, 90 },
    { 0, 0 },
    { 0x3de767c8, 23 },
    { 0x5c1767c9, 90 },
    { 0x885767c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32767cd, 33 },
    { 0x7be767ce, 43 },
    { 0, 0 },
    { 0xf9b767d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a767d3, 23 },
    { 0, 0 },
    { 0x859767d5, 96 },
    { 0, 0 },
    { 0xad2767d7, 90 },
    { 0x527767d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41767db, 61 },
    { 0x626767dc, 76 },
    { 0xa8e767dd, 61 },
    { 0xc76767de, 80 },
    { 0xb8f767df, 90 },
    { 0xb7e767df, 98 },
    { 0x7bb767e1, 90 },
    { 0, 0 },
    { 0x88e767e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4767e6, 33 },
    { 0x215767e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537767f1, 68 },
    { 0, 0 },
    { 0x49c767f3, 23 },
    { 0, 0 },
    { 0x48767f5, 90 },
    { 0x338767f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27767f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4767fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d376800, 95 },
    { 0xc4c76801, 77 },
    { 0x11076801, 66 },
    { 0xf9d76801, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xd576806, 61 },
    { 0x6b776807, 55 },
    { 0x70d76806, 26 },
    { 0, 0 },
    { 0xea87680a, 52 },
    { 0x7617680b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e76810, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e76818, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257681d, 76 },
    { 0x6c7681e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a976823, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47976827, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3807682c, 33 },
    { 0xa77682d, 98 },
    { 0x5e27682e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90976834, 55 },
    { 0x8e976835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a76838, 33 },
    { 0x9b476839, 98 },
    { 0x1bb7683a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f7683d, 61 },
    { 0x41c7683d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f976841, 23 },
    { 0xc3976842, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c276845, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c57684f, 76 },
    { 0, 0 },
    { 0x63976851, 52 },
    { 0x77276851, 100 },
    { 0x2a176853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a76858, 17 },
    { 0x4dd76858, 95 },
    { 0xb0376858, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7685e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e76862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29676866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f476869, 68 },
    { 0x2e87686a, 90 },
    { 0, 0 },
    { 0xc517686c, 88 },
    { 0xa0f7686d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85776873, 52 },
    { 0x6e876873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897687f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a76882, 84 },
    { 0xbd76883, 88 },
    { 0x90f76883, 33 },
    { 0x73576885, 33 },
    { 0xe5f76886, 85 },
    { 0x3c676887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a7688f, 61 },
    { 0xee17688f, 99 },
    { 0x46f76891, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x60676894, 55 },
    { 0x15276894, 23 },
    { 0, 0 },
    { 0x75a76897, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567689b, 98 },
    { 0, 0 },
    { 0x3287689d, 100 },
    { 0xe1c7689e, 76 },
    { 0x1297689f, 90 },
    { 0xe107689f, 76 },
    { 0x8837689e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7768a7, 95 },
    { 0x2f0768a8, 84 },
    { 0xc44768a7, 95 },
    { 0, 0 },
    { 0xe1e768ab, 98 },
    { 0x933768ab, 93 },
    { 0x924768ad, 46 },
    { 0x822768ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650768b2, 61 },
    { 0xb1c768b2, 23 },
    { 0xb8768b4, 34 },
    { 0xd0f768b5, 43 },
    { 0, 0 },
    { 0x3f7768b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9768bb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd768c0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16768c4, 23 },
    { 0xdc9768c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb768cb, 84 },
    { 0, 0 },
    { 0x9a9768cd, 78 },
    { 0, 0 },
    { 0x4ca768cf, 34 },
    { 0xf60768d0, 98 },
    { 0, 0 },
    { 0xce2768d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a768d5, 43 },
    { 0, 0 },
    { 0x9bf768d7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b768dc, 90 },
    { 0x824768dd, 93 },
    { 0xcc7768dd, 61 },
    { 0, 0 },
    { 0xb9e768e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x455768e3, 23 },
    { 0x21e768e4, 95 },
    { 0x1dc768e3, 46 },
    { 0x720768e6, 61 },
    { 0xa76768e5, 33 },
    { 0xa49768e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357768f1, 23 },
    { 0, 0 },
    { 0xec8768f3, 17 },
    { 0, 0 },
    { 0x23d768f5, 33 },
    { 0x8ff768f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a768f9, 46 },
    { 0xf84768fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8876905, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f07690b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f67690f, 90 },
    { 0x4fa7690f, 23 },
    { 0xaf676911, 34 },
    { 0xe7576911, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87691e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c876924, 100 },
    { 0x57276925, 66 },
    { 0, 0 },
    { 0x30e76927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1667692a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25576933, 98 },
    { 0x8e276934, 43 },
    { 0xf9876933, 38 },
    { 0, 0 },
    { 0x40376937, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097693a, 33 },
    { 0, 0 },
    { 0x98b7693c, 43 },
    { 0x1b37693c, 33 },
    { 0xac27693e, 98 },
    { 0x6607693f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c76942, 66 },
    { 0xca076942, 43 },
    { 0x69176944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e76947, 55 },
    { 0xe6576948, 33 },
    { 0x35476948, 98 },
    { 0xa9c7694a, 85 },
    { 0x29176949, 34 },
    { 0xef07694b, 80 },
    { 0x8347694a, 96 },
    { 0, 0 },
    { 0x3a7694f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde276952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b76955, 23 },
    { 0xa1c76955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d7695a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1327695f, 61 },
    { 0, 0 },
    { 0x55c76961, 91 },
    { 0x41e76962, 56 },
    { 0xbf876963, 34 },
    { 0xe276964, 98 },
    { 0x99f76965, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f7696b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe67696f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2476977, 61 },
    { 0x93c76978, 55 },
    { 0xf176977, 90 },
    { 0, 0 },
    { 0x29b7697b, 61 },
    { 0xd87697c, 61 },
    { 0, 0 },
    { 0x5617697e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d876988, 23 },
    { 0x6c276988, 23 },
    { 0x8d276989, 34 },
    { 0xdaa7698b, 84 },
    { 0xf407698b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2547698f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54176994, 95 },
    { 0xbf676995, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc76998, 61 },
    { 0, 0 },
    { 0xf37699a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d87699e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x160769a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe769a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x897769aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a769ae, 90 },
    { 0, 0 },
    { 0x8bd769b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d769b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62769b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2769c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4769c6, 95 },
    { 0, 0 },
    { 0x638769c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759769ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x560769d1, 99 },
    { 0x5fb769d2, 100 },
    { 0x82b769d3, 56 },
    { 0xef8769d3, 93 },
    { 0x12d769d2, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae769d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b769dd, 76 },
    { 0xb85769de, 99 },
    { 0xb3769dd, 23 },
    { 0x96c769dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a769e3, 90 },
    { 0x8b7769e4, 46 },
    { 0x462769e4, 23 },
    { 0xf56769e4, 23 },
    { 0x20f769e4, 33 },
    { 0xcbd769e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4769ec, 85 },
    { 0x5a1769ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5769f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06769f7, 23 },
    { 0x570769f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b769fb, 90 },
    { 0, 0 },
    { 0xff9769fd, 52 },
    { 0x55e769fe, 33 },
    { 0xcc5769fd, 61 },
    { 0x50b76a00, 76 },
    { 0xaae76a01, 80 },
    { 0x8e976a00, 61 },
    { 0x9cd76a01, 76 },
    { 0, 0 },
    { 0x7d476a05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc76a0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4576a14, 95 },
    { 0xbe776a15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf176a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e576a27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c76a2a, 90 },
    { 0x2af76a2a, 99 },
    { 0x1fe76a2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1876a2f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae976a36, 23 },
    { 0, 0 },
    { 0xae676a38, 46 },
    { 0x53376a39, 52 },
    { 0x1e876a3a, 23 },
    { 0xce076a3b, 33 },
    { 0x53176a3c, 96 },
    { 0x42276a3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e776a42, 53 },
    { 0xdbc76a43, 93 },
    { 0xf9d76a44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22076a4b, 33 },
    { 0xbb376a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e76a56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf176a59, 23 },
    { 0x9576a5a, 95 },
    { 0xd5876a5b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xce576a5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8576a63, 61 },
    { 0, 0 },
    { 0x5f376a65, 23 },
    { 0x93476a65, 84 },
    { 0x8ce76a65, 76 },
    { 0xa6e76a68, 40 },
    { 0x25676a68, 95 },
    { 0x30376a69, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76376a6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf76a75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea76a7a, 61 },
    { 0x80476a7b, 55 },
    { 0x6a276a7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39876a7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca76a86, 88 },
    { 0xe2a76a87, 90 },
    { 0x38076a88, 99 },
    { 0x1ad76a87, 95 },
    { 0x35b76a8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c76a8f, 33 },
    { 0xf4976a90, 90 },
    { 0x39376a90, 95 },
    { 0x9b376a90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0376a95, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e976a99, 76 },
    { 0x5f76a9a, 90 },
    { 0x7ff76a9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d376aa0, 17 },
    { 0x4d676aa1, 17 },
    { 0x32d76aa1, 43 },
    { 0xff676aa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51776aa7, 78 },
    { 0xb9776aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c76aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d476ab4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad376ab7, 90 },
    { 0x73776ab8, 90 },
    { 0x89e76ab9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e76abc, 59 },
    { 0x8b976abd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b76ac0, 33 },
    { 0x3b876ac1, 90 },
    { 0x5fb76ac2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x36476ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf376ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb876ad0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca76ad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f176adb, 95 },
    { 0x6ac76adb, 84 },
    { 0, 0 },
    { 0x21f76ade, 90 },
    { 0x5db76adf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd76aea, 78 },
    { 0x30676aea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5076aef, 98 },
    { 0xd5976af0, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f76af8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4176b02, 90 },
    { 0, 0 },
    { 0xf2676b04, 23 },
    { 0xc9076b05, 23 },
    { 0, 0 },
    { 0x74776b07, 23 },
    { 0, 0 },
    { 0xca976b09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e076b0e, 95 },
    { 0x41876b0e, 100 },
    { 0x10f76b0f, 66 },
    { 0, 0 },
    { 0xa0276b12, 72 },
    { 0x81976b12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab76b17, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x19376b1a, 77 },
    { 0, 0 },
    { 0x45876b1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa476b23, 33 },
    { 0x50476b23, 33 },
    { 0xdba76b25, 33 },
    { 0x1dd76b26, 67 },
    { 0xa2c76b27, 33 },
    { 0, 0 },
    { 0xea376b29, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f376b2c, 61 },
    { 0xab876b2d, 100 },
    { 0, 0 },
    { 0xa5a76b2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa976b32, 84 },
    { 0x3a76b33, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb976b3a, 66 },
    { 0xd7a76b3b, 99 },
    { 0, 0 },
    { 0x6a976b3d, 43 },
    { 0xe7776b3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81376b41, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d76b45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f76b4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b76b53, 46 },
    { 0, 0 },
    { 0x15076b55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d476b5a, 87 },
    { 0x12176b5b, 98 },
    { 0x14776b5c, 98 },
    { 0xb7a76b5d, 23 },
    { 0xd8f76b5e, 76 },
    { 0x7576b5e, 98 },
    { 0xeb776b5d, 90 },
    { 0xc9376b61, 33 },
    { 0x2a176b5f, 23 },
    { 0x7f776b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f476b67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68776b6d, 90 },
    { 0xc1676b6d, 81 },
    { 0, 0 },
    { 0x4d876b70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b76b75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e676b78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4676b7f, 78 },
    { 0x2f276b80, 90 },
    { 0xc6776b80, 61 },
    { 0x176b82, 24 },
    { 0, 0 },
    { 0xbcc76b84, 23 },
    { 0x57576b85, 76 },
    { 0xdd276b86, 23 },
    { 0, 0 },
    { 0x41176b88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a76b8e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83276b93, 23 },
    { 0x69c76b94, 55 },
    { 0, 0 },
    { 0xf3776b96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d76b9b, 61 },
    { 0, 0 },
    { 0x6b876b9d, 90 },
    { 0x41e76b9d, 43 },
    { 0, 0 },
    { 0x1a76ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc176ba3, 99 },
    { 0, 0 },
    { 0x77876ba5, 91 },
    { 0xb8076ba6, 76 },
    { 0x9c776ba7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac276bba, 61 },
    { 0xa976bbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5376bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a076bc2, 100 },
    { 0, 0 },
    { 0x57e76bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6576bc8, 23 },
    { 0, 0 },
    { 0xd8776bca, 34 },
    { 0x72f76bca, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8476bce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce76bd9, 52 },
    { 0x63576bd9, 87 },
    { 0x81676bda, 90 },
    { 0, 0 },
    { 0xf3276bdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8276be0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c076be3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e476be8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a76bec, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa76bef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8576bf2, 95 },
    { 0x81176bf3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e876bf9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc76bfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a576c06, 93 },
    { 0xa5476c07, 99 },
    { 0, 0 },
    { 0x3b376c09, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4676c0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d76c10, 68 },
    { 0x9a576c10, 67 },
    { 0x69076c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f076c1f, 98 },
    { 0, 0 },
    { 0x5976c21, 66 },
    { 0x69576c22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b76c28, 91 },
    { 0x20276c29, 61 },
    { 0xfa476c2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f76c32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b676c35, 52 },
    { 0x60276c36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92676c39, 33 },
    { 0x51476c3a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6576c3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17576c41, 23 },
    { 0xc3e76c41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57676c45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9876c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48676c54, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb76c58, 33 },
    { 0xd1776c58, 90 },
    { 0x54176c58, 95 },
    { 0, 0 },
    { 0x82f76c5c, 40 },
    { 0, 0 },
    { 0x11976c5e, 98 },
    { 0xa6f76c5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d76c62, 46 },
    { 0x40b76c62, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac676c67, 87 },
    { 0, 0 },
    { 0xe8776c69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b76c6e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0876c72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd76c75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48376c7a, 76 },
    { 0, 0 },
    { 0x9fa76c7c, 90 },
    { 0x11576c7d, 100 },
    { 0xfa876c7e, 73 },
    { 0x15176c7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5176c82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b76c85, 52 },
    { 0xc4676c86, 98 },
    { 0x9bc76c86, 34 },
    { 0xc5c76c88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b76c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a76c92, 47 },
    { 0x5ff76c93, 34 },
    { 0xe3d76c94, 34 },
    { 0, 0 },
    { 0x1df76c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b76c99, 91 },
    { 0xa9d76c9a, 43 },
    { 0x5da76c9b, 33 },
    { 0xfd176c9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea76ca1, 95 },
    { 0x98f76ca2, 90 },
    { 0x94e76ca1, 61 },
    { 0, 0 },
    { 0x9b476ca5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa76ca8, 91 },
    { 0x48776ca9, 23 },
    { 0x6a776caa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41776cb0, 66 },
    { 0xd0676cb1, 33 },
    { 0xb3b76cb1, 80 },
    { 0x7ab76cb3, 90 },
    { 0xdf676cb3, 76 },
    { 0xb9b76cb5, 91 },
    { 0, 0 },
    { 0xf276cb7, 98 },
    { 0x72d76cb7, 33 },
    { 0, 0 },
    { 0x69576cba, 65 },
    { 0xe0876cbb, 95 },
    { 0x9e476cbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad76cc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d676cc8, 13 },
    { 0x43d76cc9, 33 },
    { 0x51876cc9, 91 },
    { 0x4e776cc8, 95 },
    { 0, 0 },
    { 0xafa76ccd, 61 },
    { 0xdfe76cce, 88 },
    { 0, 0 },
    { 0x85e76cd0, 90 },
    { 0xa4776cd1, 81 },
    { 0x9d576cd2, 95 },
    { 0xd4076cd0, 90 },
    { 0xf5f76cd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d776cd8, 33 },
    { 0xae576cd8, 33 },
    { 0x7f76cd9, 76 },
    { 0x78676cdb, 43 },
    { 0xde776cdc, 99 },
    { 0x64676cdc, 66 },
    { 0xbb876cde, 100 },
    { 0xff776cd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6376ce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58476ce8, 85 },
    { 0xd5976ce8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c76cef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x75676cf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd76cf8, 90 },
    { 0, 0 },
    { 0x27876cfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f76d00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0476d07, 95 },
    { 0, 0 },
    { 0xf5d76d09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda76d0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c76d1d, 43 },
    { 0x2df76d1e, 98 },
    { 0, 0 },
    { 0xec276d20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a76d24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63576d27, 34 },
    { 0xaa976d28, 33 },
    { 0x66d76d29, 92 },
    { 0x5d776d27, 23 },
    { 0x16176d27, 85 },
    { 0, 0 },
    { 0xf0876d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16176d34, 80 },
    { 0xd8176d34, 33 },
    { 0x2ce76d36, 99 },
    { 0x42776d37, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29176d3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e76d3e, 23 },
    { 0x41876d3e, 95 },
    { 0xf6676d40, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f776d43, 33 },
    { 0xeaf76d43, 43 },
    { 0x62976d45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92576d49, 95 },
    { 0x29b76d4a, 23 },
    { 0xbe76d4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e876d4f, 90 },
    { 0, 0 },
    { 0xc1976d51, 88 },
    { 0x4e076d52, 23 },
    { 0x85176d53, 61 },
    { 0x26e76d52, 95 },
    { 0, 0 },
    { 0xe1d76d56, 38 },
    { 0x83976d57, 67 },
    { 0xf6d76d56, 34 },
    { 0x1f76d59, 90 },
    { 0x24476d59, 98 },
    { 0x82c76d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c076d5f, 46 },
    { 0, 0 },
    { 0x42e76d61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3476d64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a076d6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f876d6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b76d71, 95 },
    { 0x65176d71, 95 },
    { 0x42c76d73, 76 },
    { 0xb6476d73, 77 },
    { 0x2a276d73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63476d7d, 23 },
    { 0xc3476d7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6776d82, 90 },
    { 0x46376d82, 52 },
    { 0, 0 },
    { 0x16876d85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d76d8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda576d9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f676d9f, 96 },
    { 0x17676da0, 90 },
    { 0x22376da1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce976da6, 97 },
    { 0, 0 },
    { 0xa0776da8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa76dad, 87 },
    { 0x5a376dae, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24076db5, 46 },
    { 0, 0 },
    { 0xf7d76db7, 55 },
    { 0xe8d76db7, 55 },
    { 0xa976db7, 55 },
    { 0xdfb76dba, 100 },
    { 0x3f076db9, 93 },
    { 0xbda76dbc, 61 },
    { 0, 0 },
    { 0x2ee76dbe, 95 },
    { 0x48476dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a276dc2, 46 },
    { 0, 0 },
    { 0xee276dc4, 40 },
    { 0, 0 },
    { 0x6d076dc6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x29476dc9, 95 },
    { 0, 0 },
    { 0x6eb76dcb, 76 },
    { 0x6d176dcb, 90 },
    { 0x39376dcd, 40 },
    { 0x1c376dce, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed76dd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81876dd8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f76ddd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b76de2, 76 },
    { 0, 0 },
    { 0xe976de4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2576dea, 34 },
    { 0x41876dea, 93 },
    { 0, 0 },
    { 0x6ce76ded, 95 },
    { 0, 0 },
    { 0xbd676def, 24 },
    { 0xa7c76df0, 98 },
    { 0x3cb76def, 90 },
    { 0xdfc76df2, 87 },
    { 0x22276df3, 75 },
    { 0x4f076def, 83 },
    { 0x5ed76df0, 90 },
    { 0, 0 },
    { 0xe1276df7, 61 },
    { 0x69e76df8, 90 },
    { 0x34076df7, 81 },
    { 0x3c376dfa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee76e00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb76e04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65076e08, 76 },
    { 0x26e76e09, 98 },
    { 0x3fc76e09, 23 },
    { 0, 0 },
    { 0x24076e0c, 90 },
    { 0, 0 },
    { 0x96276e0e, 52 },
    { 0x49276e0f, 49 },
    { 0, 0 },
    { 0xca776e11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d676e15, 85 },
    { 0, 0 },
    { 0x38276e17, 98 },
    { 0, 0 },
    { 0x53176e19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92476e1d, 98 },
    { 0, 0 },
    { 0x56576e1f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee076e2b, 98 },
    { 0x24276e2b, 98 },
    { 0xcbc76e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a76e33, 43 },
    { 0xab876e34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb76e38, 98 },
    { 0xc0976e39, 84 },
    { 0, 0 },
    { 0x4af76e3b, 33 },
    { 0x6a076e3c, 100 },
    { 0x47576e3d, 98 },
    { 0x32276e3e, 90 },
    { 0, 0 },
    { 0x27f76e40, 90 },
    { 0xb5176e41, 91 },
    { 0, 0 },
    { 0xae576e43, 61 },
    { 0x69576e44, 61 },
    { 0, 0 },
    { 0xd776e46, 52 },
    { 0, 0 },
    { 0x69f76e48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c76e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf276e53, 98 },
    { 0x55c76e53, 46 },
    { 0x9f576e55, 68 },
    { 0xc5276e56, 33 },
    { 0xe7976e57, 95 },
    { 0x79b76e58, 90 },
    { 0x9f176e58, 33 },
    { 0x7db76e56, 55 },
    { 0x12376e55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e776e67, 78 },
    { 0x6276e68, 68 },
    { 0, 0 },
    { 0x4676e6a, 55 },
    { 0xbe776e6b, 61 },
    { 0xff576e6b, 94 },
    { 0xf4a76e6d, 98 },
    { 0x8a876e6b, 52 },
    { 0x3e176e6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6676e72, 84 },
    { 0x4c276e73, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10576e77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65876e7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82976e80, 90 },
    { 0x56376e81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a76e88, 75 },
    { 0, 0 },
    { 0xfff76e8a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef76e8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9776e92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e176e95, 99 },
    { 0xa4676e95, 43 },
    { 0x22776e95, 34 },
    { 0, 0 },
    { 0xa6476e99, 33 },
    { 0xc9376e9a, 90 },
    { 0, 0 },
    { 0xc9e76e9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82176ea0, 33 },
    { 0xee76ea0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f076ea8, 100 },
    { 0x3ca76ea9, 66 },
    { 0x75276eaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a76eaf, 85 },
    { 0, 0 },
    { 0xa6476eb1, 95 },
    { 0, 0 },
    { 0x1ec76eb3, 95 },
    { 0, 0 },
    { 0xfb76eb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4176eb9, 34 },
    { 0x17676eba, 76 },
    { 0x9b776eba, 95 },
    { 0x56c76eb9, 33 },
    { 0x90776ebc, 77 },
    { 0x90276ebe, 98 },
    { 0xa2476ebf, 33 },
    { 0, 0 },
    { 0x476ec1, 33 },
    { 0x84d76ec2, 55 },
    { 0, 0 },
    { 0x84576ec4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1276ec9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88176ecc, 93 },
    { 0xeb676ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f76ed0, 90 },
    { 0x81976ed1, 87 },
    { 0, 0 },
    { 0xd9b76ed3, 61 },
    { 0xe5f76ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40976ed8, 43 },
    { 0, 0 },
    { 0x55176eda, 34 },
    { 0, 0 },
    { 0x8d476edc, 51 },
    { 0xa7276edc, 90 },
    { 0xb8976edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a076ee3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde976ee9, 66 },
    { 0, 0 },
    { 0x4b476eeb, 23 },
    { 0x53476eec, 99 },
    { 0x27376eec, 76 },
    { 0, 0 },
    { 0xa8276eef, 90 },
    { 0, 0 },
    { 0x17176ef1, 33 },
    { 0x6eb76ef2, 33 },
    { 0xdf876ef1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41376ef6, 99 },
    { 0x29676ef7, 75 },
    { 0x61b76ef8, 23 },
    { 0, 0 },
    { 0x7376efa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8676efe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24376f03, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97976f0a, 95 },
    { 0xc3676f0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b76f10, 33 },
    { 0xfe76f11, 23 },
    { 0x29576f12, 98 },
    { 0x5da76f13, 100 },
    { 0xc9b76f14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9976f18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x16776f1b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa776f1f, 98 },
    { 0, 0 },
    { 0xe976f21, 80 },
    { 0xf5876f22, 55 },
    { 0x7d776f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62576f2a, 34 },
    { 0x67876f2a, 23 },
    { 0x76576f2c, 95 },
    { 0xbda76f2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54876f31, 61 },
    { 0xc6076f31, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f76f39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe76f3d, 61 },
    { 0, 0 },
    { 0x6c76f3f, 90 },
    { 0xfdf76f40, 95 },
    { 0x8d976f40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53676f44, 95 },
    { 0xd2a76f44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42976f48, 90 },
    { 0xd9776f48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd76f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34376f54, 67 },
    { 0xfcf76f54, 61 },
    { 0x17d76f55, 43 },
    { 0x89976f56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd76f5a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb76f5d, 77 },
    { 0, 0 },
    { 0x49876f5f, 23 },
    { 0x85376f60, 90 },
    { 0x50b76f60, 93 },
    { 0xbe776f61, 61 },
    { 0, 0 },
    { 0x24f76f64, 67 },
    { 0xe6276f65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc76f6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4276f71, 90 },
    { 0, 0 },
    { 0x7ca76f73, 90 },
    { 0, 0 },
    { 0x43976f75, 78 },
    { 0, 0 },
    { 0x41576f77, 43 },
    { 0x37e76f77, 33 },
    { 0x24476f79, 23 },
    { 0xaed76f7a, 52 },
    { 0xeb76f79, 61 },
    { 0xd4476f79, 33 },
    { 0xe3e76f7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46576f80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e176f83, 95 },
    { 0, 0 },
    { 0xc3376f85, 33 },
    { 0xaa876f86, 90 },
    { 0, 0 },
    { 0xbe076f88, 23 },
    { 0xd5e76f89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79476f8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e76f90, 87 },
    { 0x78676f90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b576f98, 93 },
    { 0, 0 },
    { 0x7d276f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f76f9d, 55 },
    { 0xa8276f9d, 33 },
    { 0, 0 },
    { 0x10d76fa0, 55 },
    { 0, 0 },
    { 0x84076fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d76fa6, 98 },
    { 0xb5076fa7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce776fac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf876faf, 67 },
    { 0xfd576fb0, 98 },
    { 0x62876fb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28076fb5, 90 },
    { 0xb0176fb6, 40 },
    { 0, 0 },
    { 0x46176fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b76fbb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb976fbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3876fc5, 38 },
    { 0xc5676fc6, 98 },
    { 0, 0 },
    { 0xd5476fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47776fcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e876fd3, 98 },
    { 0, 0 },
    { 0x7f976fd5, 100 },
    { 0, 0 },
    { 0x8a476fd7, 90 },
    { 0x86e76fd8, 90 },
    { 0x5e576fd9, 76 },
    { 0xa9b76fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5076fe1, 66 },
    { 0, 0 },
    { 0x2b176fe3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb176fe7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f76fea, 91 },
    { 0x64f76feb, 90 },
    { 0xaf276fec, 91 },
    { 0, 0 },
    { 0x6c676fee, 95 },
    { 0, 0 },
    { 0x90276ff0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3476ff3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1676ff7, 33 },
    { 0x24676ff8, 55 },
    { 0x276ff9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x32776ffc, 85 },
    { 0xdf976ffc, 33 },
    { 0xf0e76ffe, 95 },
    { 0, 0 },
    { 0x13477000, 76 },
    { 0xaf977000, 23 },
    { 0, 0 },
    { 0xc2b77003, 93 },
    { 0, 0 },
    { 0xabf77005, 41 },
    { 0x63577006, 96 },
    { 0x50c77006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd7700a, 61 },
    { 0x5d97700a, 34 },
    { 0, 0 },
    { 0x69c7700d, 33 },
    { 0x8537700d, 90 },
    { 0x7057700f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b77013, 34 },
    { 0x9f777014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca77019, 69 },
    { 0, 0 },
    { 0xb007701b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec077020, 85 },
    { 0x74e77021, 98 },
    { 0, 0 },
    { 0x8a377023, 93 },
    { 0, 0 },
    { 0xbf177025, 95 },
    { 0, 0 },
    { 0x61c77027, 23 },
    { 0x8c477028, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb907702c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c877033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48277038, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6437703b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf77040, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21277044, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e077049, 33 },
    { 0, 0 },
    { 0x6f67704b, 66 },
    { 0, 0 },
    { 0x5e57704d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e77051, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xde277054, 44 },
    { 0x8d077055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467705b, 91 },
    { 0x3227705c, 81 },
    { 0x2237705d, 34 },
    { 0, 0 },
    { 0x8637705f, 84 },
    { 0, 0 },
    { 0xd5777061, 90 },
    { 0x20377062, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91977067, 100 },
    { 0, 0 },
    { 0x13677069, 99 },
    { 0xdcd7706a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27477078, 23 },
    { 0xb4577079, 33 },
    { 0xecc7707a, 61 },
    { 0x9c677079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6677082, 61 },
    { 0xfc477082, 55 },
    { 0x8277082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d77089, 61 },
    { 0x4a977089, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7127708d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec77094, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6417709a, 90 },
    { 0x5a57709a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7709f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c770a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4770aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde770b4, 66 },
    { 0x8cf770b5, 61 },
    { 0x948770b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61770b9, 23 },
    { 0x13d770ba, 61 },
    { 0x996770bb, 23 },
    { 0x134770bb, 61 },
    { 0x2c770bd, 100 },
    { 0x933770bb, 90 },
    { 0, 0 },
    { 0x435770c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d770c3, 23 },
    { 0, 0 },
    { 0x189770c5, 52 },
    { 0x575770c6, 43 },
    { 0xe29770c6, 98 },
    { 0xd30770c8, 90 },
    { 0xdb4770c9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c770d4, 77 },
    { 0xf14770d5, 27 },
    { 0x4e5770d6, 90 },
    { 0xf21770d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93770da, 85 },
    { 0xe87770db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994770e0, 33 },
    { 0x41a770e1, 90 },
    { 0x806770e2, 98 },
    { 0xc56770e0, 61 },
    { 0xc6c770e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c770f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f770f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145770fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e77100, 76 },
    { 0xc4477100, 93 },
    { 0, 0 },
    { 0xa9177103, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4677106, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a977110, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff77115, 91 },
    { 0xd4177115, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb7711b, 95 },
    { 0x6477711b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a77121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a7712a, 76 },
    { 0x9e77712a, 76 },
    { 0x7af7712a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd677130, 34 },
    { 0, 0 },
    { 0xa5577132, 100 },
    { 0, 0 },
    { 0x57c77134, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d777137, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2897713e, 93 },
    { 0x1687713f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f177145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac977148, 46 },
    { 0x7eb77149, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e7714d, 95 },
    { 0, 0 },
    { 0xedc7714f, 23 },
    { 0x15477150, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f477154, 55 },
    { 0, 0 },
    { 0xd5677156, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db7715b, 78 },
    { 0x3fe7715c, 90 },
    { 0xe647715b, 90 },
    { 0xa2c7715e, 55 },
    { 0xfca7715f, 95 },
    { 0xb407715d, 90 },
    { 0xfb777161, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de77164, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23777168, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f7716d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x277170, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62777178, 61 },
    { 0x20e77179, 90 },
    { 0x40b77178, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c7717e, 66 },
    { 0xc057717f, 17 },
    { 0xa507717f, 43 },
    { 0xab977181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38577184, 76 },
    { 0x18577185, 90 },
    { 0xf0477185, 98 },
    { 0x8db77187, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d7718d, 90 },
    { 0xc8f7718e, 47 },
    { 0, 0 },
    { 0x69e77190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c177195, 95 },
    { 0x3ef77196, 17 },
    { 0xe2577197, 33 },
    { 0, 0 },
    { 0xa7477199, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1771a4, 90 },
    { 0xdb3771a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e771b0, 65 },
    { 0x26e771b0, 95 },
    { 0xb2e771b2, 80 },
    { 0, 0 },
    { 0xb7c771b4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e771b9, 56 },
    { 0xab6771ba, 95 },
    { 0xcf6771bb, 33 },
    { 0xda3771bc, 55 },
    { 0, 0 },
    { 0x3d9771be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753771c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4771c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f771c8, 61 },
    { 0xae3771c9, 52 },
    { 0xa79771c9, 95 },
    { 0x51b771cb, 33 },
    { 0x56771cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9771cf, 95 },
    { 0, 0 },
    { 0xdba771d1, 76 },
    { 0, 0 },
    { 0x9ef771d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3771d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb771dd, 61 },
    { 0xd85771de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1771e3, 99 },
    { 0xa53771e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f771e7, 91 },
    { 0x4c1771e8, 99 },
    { 0, 0 },
    { 0x5e7771ea, 33 },
    { 0, 0 },
    { 0x1cc771ec, 34 },
    { 0, 0 },
    { 0x82e771ee, 90 },
    { 0x159771ee, 90 },
    { 0xcdf771f0, 61 },
    { 0x16a771ee, 23 },
    { 0x1e3771f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x943771f5, 33 },
    { 0x781771f5, 87 },
    { 0x6dd771f5, 95 },
    { 0x96b771f8, 92 },
    { 0x876771f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c771fd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b77200, 88 },
    { 0x5eb77201, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77077209, 95 },
    { 0, 0 },
    { 0x4207720b, 52 },
    { 0xcff7720b, 98 },
    { 0x7507720d, 98 },
    { 0xf037720e, 33 },
    { 0xaeb7720e, 100 },
    { 0, 0 },
    { 0x27977211, 67 },
    { 0xc0577212, 95 },
    { 0, 0 },
    { 0xca977214, 33 },
    { 0xb1477215, 66 },
    { 0, 0 },
    { 0xd7677217, 77 },
    { 0x71277217, 87 },
    { 0xefc77219, 100 },
    { 0xab57721a, 23 },
    { 0xdcb7721b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7721f, 91 },
    { 0xd327721f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda077223, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e977227, 33 },
    { 0xe6b77228, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42477234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59877237, 97 },
    { 0xce977238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2967723b, 90 },
    { 0, 0 },
    { 0x91f7723d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58277240, 91 },
    { 0xafc77241, 84 },
    { 0x97577242, 43 },
    { 0xdb77240, 46 },
    { 0x48377241, 95 },
    { 0xe4277240, 33 },
    { 0, 0 },
    { 0x68377247, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d67724a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227724f, 95 },
    { 0, 0 },
    { 0x98d77251, 90 },
    { 0xef177251, 90 },
    { 0x3b377253, 52 },
    { 0x7e777254, 97 },
    { 0x71577255, 84 },
    { 0xa2f77253, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c77725c, 90 },
    { 0, 0 },
    { 0x1a47725e, 90 },
    { 0x5e27725f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b77266, 100 },
    { 0x5e277266, 90 },
    { 0x5dd77268, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6487726c, 33 },
    { 0, 0 },
    { 0xa3e7726e, 90 },
    { 0xdb07726f, 48 },
    { 0, 0 },
    { 0xc8777271, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7297727a, 98 },
    { 0xf547727b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26277281, 90 },
    { 0, 0 },
    { 0xecc77283, 98 },
    { 0, 0 },
    { 0x8c177285, 93 },
    { 0x63c77286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1547728c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c677299, 17 },
    { 0, 0 },
    { 0x4cf7729b, 23 },
    { 0, 0 },
    { 0x21f7729d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218772a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca772a9, 61 },
    { 0xa4b772a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9772b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98772b5, 90 },
    { 0x64a772b6, 68 },
    { 0xc03772b7, 93 },
    { 0xbd4772b7, 90 },
    { 0x73f772b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a772bd, 88 },
    { 0x40b772bd, 91 },
    { 0xdaa772bd, 78 },
    { 0x437772c0, 90 },
    { 0xcfa772c1, 90 },
    { 0xe9f772c1, 90 },
    { 0xd48772c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7772c7, 78 },
    { 0, 0 },
    { 0x911772c9, 90 },
    { 0, 0 },
    { 0x1bb772cb, 98 },
    { 0x36c772cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1772cf, 90 },
    { 0xe4a772cf, 84 },
    { 0x609772d1, 61 },
    { 0x8ac772d1, 23 },
    { 0x145772d3, 23 },
    { 0x87c772d1, 33 },
    { 0x39772d5, 65 },
    { 0x7ad772cf, 95 },
    { 0x450772d7, 66 },
    { 0xa2772d8, 33 },
    { 0x7bb772d7, 61 },
    { 0x5ca772da, 76 },
    { 0x5a772db, 98 },
    { 0x3a2772dc, 95 },
    { 0xefd772dc, 43 },
    { 0, 0 },
    { 0xb74772df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d772e2, 95 },
    { 0, 0 },
    { 0xbfa772e4, 55 },
    { 0x772e5, 52 },
    { 0x8be772e6, 55 },
    { 0x2fe772e7, 76 },
    { 0x63772e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849772ec, 98 },
    { 0, 0 },
    { 0xd93772ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b772f6, 23 },
    { 0x918772f7, 90 },
    { 0, 0 },
    { 0x137772f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f772fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc77305, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2797730a, 99 },
    { 0, 0 },
    { 0x6be7730c, 33 },
    { 0x5577730d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca77314, 100 },
    { 0xe9877315, 84 },
    { 0x94c77316, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a77731c, 98 },
    { 0, 0 },
    { 0x6827731e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb77321, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5377326, 90 },
    { 0xd8c77326, 95 },
    { 0, 0 },
    { 0x9a277329, 77 },
    { 0x5847732a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b7732d, 23 },
    { 0x2e27732e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72077333, 23 },
    { 0x90577334, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8597733a, 98 },
    { 0, 0 },
    { 0x9a7733c, 52 },
    { 0xab17733d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1877340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17177343, 90 },
    { 0x2cf77344, 93 },
    { 0x7e577344, 91 },
    { 0xbbe77346, 61 },
    { 0xb4c77347, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a97734a, 93 },
    { 0xa6c7734b, 43 },
    { 0x1fc7734c, 46 },
    { 0, 0 },
    { 0x5687734e, 95 },
    { 0xb887734e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d377355, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c7735a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc347735d, 98 },
    { 0x2607735e, 23 },
    { 0xded7735d, 90 },
    { 0xe6c77360, 95 },
    { 0, 0 },
    { 0xcd277362, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c77366, 98 },
    { 0x6c77366, 87 },
    { 0, 0 },
    { 0xc1d77369, 90 },
    { 0x3357736a, 95 },
    { 0x5267736b, 23 },
    { 0x2ff7736b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0077371, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a7737b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb147737e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d077382, 61 },
    { 0x9b777382, 84 },
    { 0xa9e77384, 95 },
    { 0x69077384, 88 },
    { 0x3d777384, 33 },
    { 0xe7577385, 33 },
    { 0x29e77383, 43 },
    { 0xd1177389, 23 },
    { 0x9a377389, 61 },
    { 0, 0 },
    { 0xe757738c, 61 },
    { 0xe247738d, 34 },
    { 0x2787738d, 83 },
    { 0x2ed7738d, 61 },
    { 0x87a77390, 90 },
    { 0xf4277391, 34 },
    { 0xf5e7738c, 46 },
    { 0xd097738e, 23 },
    { 0x79177394, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67739a, 95 },
    { 0x80c7739b, 66 },
    { 0xc147739c, 95 },
    { 0xdc7739d, 77 },
    { 0xcc17739e, 78 },
    { 0x82a7739f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1773a7, 76 },
    { 0, 0 },
    { 0xd24773a9, 23 },
    { 0, 0 },
    { 0xd64773ab, 93 },
    { 0xc25773ac, 98 },
    { 0, 0 },
    { 0x7cb773ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0773ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae773c4, 43 },
    { 0x43773c5, 85 },
    { 0xc5f773c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58773ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21773ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6773d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00773db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971773e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf773e3, 33 },
    { 0, 0 },
    { 0x8d3773e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d773e8, 76 },
    { 0xa42773e9, 23 },
    { 0, 0 },
    { 0x4c6773eb, 46 },
    { 0x201773ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea773f8, 90 },
    { 0xded773f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510773ff, 93 },
    { 0x4dc77400, 95 },
    { 0x42077401, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca677408, 52 },
    { 0x93477409, 95 },
    { 0xe457740a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac77411, 90 },
    { 0xdba77411, 61 },
    { 0x2ca77411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb77419, 87 },
    { 0xb7a77419, 75 },
    { 0, 0 },
    { 0x33c7741c, 57 },
    { 0xeb97741c, 61 },
    { 0x1737741e, 33 },
    { 0, 0 },
    { 0x6a277420, 99 },
    { 0, 0 },
    { 0xd1977422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26377429, 95 },
    { 0x29e7742a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa77433, 99 },
    { 0x7b977433, 88 },
    { 0x71e77435, 23 },
    { 0x96e77435, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5607743c, 55 },
    { 0x78b7743d, 46 },
    { 0x6b47743d, 96 },
    { 0xfc87743f, 33 },
    { 0, 0 },
    { 0xb2a77441, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x277444, 66 },
    { 0x6d877445, 84 },
    { 0xd4b77445, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4977744d, 95 },
    { 0x46a7744e, 76 },
    { 0xbfb7744d, 76 },
    { 0x20977450, 23 },
    { 0x8c277451, 98 },
    { 0xe0f77451, 76 },
    { 0xa6777451, 78 },
    { 0x70d77454, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb97745e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44677464, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa77468, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf57746d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1777473, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7747a, 43 },
    { 0, 0 },
    { 0x9b57747c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe607747f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf977483, 23 },
    { 0, 0 },
    { 0xb2c77485, 64 },
    { 0xcff77486, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f77495, 61 },
    { 0, 0 },
    { 0xa2277497, 81 },
    { 0xde277497, 91 },
    { 0x12a77497, 33 },
    { 0x3177749a, 90 },
    { 0x4797749b, 98 },
    { 0x8a07749c, 61 },
    { 0xdcc7749b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec774a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6774a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a774ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123774b4, 90 },
    { 0xdea774b5, 59 },
    { 0xb4c774b6, 23 },
    { 0x822774b7, 90 },
    { 0xd10774b7, 61 },
    { 0xabc774b9, 90 },
    { 0x6ba774b9, 33 },
    { 0x74a774b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916774c1, 90 },
    { 0x674774c2, 76 },
    { 0xbcf774c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5774cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c774d3, 95 },
    { 0xcb4774d4, 90 },
    { 0x68c774d4, 46 },
    { 0xca8774d6, 23 },
    { 0xabb774d7, 87 },
    { 0x4fd774d7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef774db, 90 },
    { 0, 0 },
    { 0x395774dd, 46 },
    { 0xc8b774dd, 17 },
    { 0x5b1774df, 95 },
    { 0x333774e0, 95 },
    { 0x443774e0, 33 },
    { 0x6f5774e2, 33 },
    { 0x9fb774df, 90 },
    { 0x2cb774e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9774ec, 80 },
    { 0x2c1774ed, 55 },
    { 0xa8f774ee, 33 },
    { 0x38f774ef, 90 },
    { 0x2fc774f0, 95 },
    { 0xb39774ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1774f8, 46 },
    { 0x416774f9, 98 },
    { 0xcfa774fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2377505, 33 },
    { 0, 0 },
    { 0x8f877507, 99 },
    { 0x5f377508, 100 },
    { 0xcc777509, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b87750d, 76 },
    { 0x9f77750e, 98 },
    { 0x2f07750f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e77518, 87 },
    { 0x3b377519, 76 },
    { 0xd4877519, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b97751d, 90 },
    { 0, 0 },
    { 0x1537751f, 46 },
    { 0x5b177520, 33 },
    { 0x27177521, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x17277524, 61 },
    { 0, 0 },
    { 0x8f177526, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3477529, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a7752e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8377533, 33 },
    { 0xbc177533, 95 },
    { 0x84177535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21277539, 43 },
    { 0, 0 },
    { 0x36e7753b, 61 },
    { 0xe857753c, 55 },
    { 0xa607753c, 34 },
    { 0xcce7753e, 97 },
    { 0xa947753f, 98 },
    { 0x9207753e, 95 },
    { 0xcb877541, 61 },
    { 0x1c877542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf277546, 98 },
    { 0, 0 },
    { 0xb4477548, 95 },
    { 0x9e177549, 95 },
    { 0x4687754a, 46 },
    { 0x8637754a, 90 },
    { 0x25a7754c, 90 },
    { 0x5ba7754d, 61 },
    { 0x5787754d, 33 },
    { 0x15b7754f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d577554, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f77557, 77 },
    { 0x7e077557, 23 },
    { 0, 0 },
    { 0x1957755a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc047755d, 33 },
    { 0, 0 },
    { 0xd337755f, 76 },
    { 0xf817755f, 34 },
    { 0x7f777561, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d77564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e77568, 95 },
    { 0xb3677569, 81 },
    { 0x7c777569, 90 },
    { 0, 0 },
    { 0x9a47756c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34877570, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46177574, 23 },
    { 0xda777574, 84 },
    { 0x86d77576, 23 },
    { 0x59777576, 17 },
    { 0x99977575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f57757c, 61 },
    { 0xdb77757d, 33 },
    { 0, 0 },
    { 0x4e37757f, 98 },
    { 0xe7e77580, 55 },
    { 0x8fb77580, 90 },
    { 0x42e77582, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a677589, 75 },
    { 0, 0 },
    { 0xfec7758b, 90 },
    { 0x8667758c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2477590, 90 },
    { 0xfb677591, 93 },
    { 0, 0 },
    { 0x77477593, 90 },
    { 0x81677593, 59 },
    { 0x40377594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa677759b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87759f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f775a2, 90 },
    { 0, 0 },
    { 0x15f775a4, 65 },
    { 0, 0 },
    { 0x184775a6, 95 },
    { 0x39e775a7, 90 },
    { 0x1e9775a8, 33 },
    { 0x8bc775a9, 55 },
    { 0x1a0775a9, 90 },
    { 0x2f0775a8, 61 },
    { 0x49f775ac, 90 },
    { 0xf07775ad, 76 },
    { 0xcb2775ac, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8775b3, 98 },
    { 0x1a9775b3, 98 },
    { 0x3de775b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db775b9, 33 },
    { 0, 0 },
    { 0xa4b775bb, 61 },
    { 0, 0 },
    { 0xffb775bd, 23 },
    { 0x2c1775be, 90 },
    { 0xc96775be, 76 },
    { 0x95b775bf, 87 },
    { 0x776775c1, 95 },
    { 0x3b0775c1, 52 },
    { 0xbdd775bd, 76 },
    { 0x4b5775c4, 95 },
    { 0xa71775c4, 90 },
    { 0xc53775bd, 43 },
    { 0x37b775c7, 33 },
    { 0xc4d775c8, 17 },
    { 0xf73775c8, 90 },
    { 0xbdc775ca, 66 },
    { 0x2e4775c3, 23 },
    { 0x73775cc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a775cf, 90 },
    { 0x6e6775d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a775d4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x262775d7, 90 },
    { 0, 0 },
    { 0x405775d9, 23 },
    { 0x13775d9, 95 },
    { 0x7d7775db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x427775de, 61 },
    { 0x6d0775de, 43 },
    { 0xcc6775e0, 98 },
    { 0, 0 },
    { 0x587775e2, 43 },
    { 0xac0775e3, 84 },
    { 0, 0 },
    { 0x368775e5, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a775e9, 55 },
    { 0, 0 },
    { 0x11c775eb, 68 },
    { 0xe8775ec, 98 },
    { 0x7c6775ed, 55 },
    { 0x788775eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e775fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6775ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb77603, 33 },
    { 0xe0477603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c77608, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d37760c, 17 },
    { 0x4667760d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x73877610, 55 },
    { 0, 0 },
    { 0xa4177612, 61 },
    { 0x7d977613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd7761e, 90 },
    { 0, 0 },
    { 0xe377620, 88 },
    { 0x58077621, 23 },
    { 0x7c577622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81277626, 98 },
    { 0x77077627, 99 },
    { 0x9d277628, 90 },
    { 0, 0 },
    { 0xea47762a, 61 },
    { 0xc477762b, 90 },
    { 0x23a7762c, 43 },
    { 0xf057762d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b77630, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e677633, 77 },
    { 0, 0 },
    { 0x64e77635, 84 },
    { 0x88677635, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed67763b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b07763e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d477644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75477648, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a7764d, 61 },
    { 0xf7764e, 95 },
    { 0xdab7764e, 90 },
    { 0x2fa7764e, 61 },
    { 0, 0 },
    { 0x65077652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62877655, 43 },
    { 0, 0 },
    { 0x29477657, 33 },
    { 0xf8f77658, 34 },
    { 0x40177659, 77 },
    { 0xabc77659, 33 },
    { 0x7ca7765b, 90 },
    { 0, 0 },
    { 0x1cf7765d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae77662, 90 },
    { 0x32f77663, 95 },
    { 0xa377662, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d77667, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87766a, 55 },
    { 0, 0 },
    { 0x4177766c, 95 },
    { 0x827766d, 43 },
    { 0, 0 },
    { 0xad37766f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e77675, 76 },
    { 0xbb277676, 84 },
    { 0xabb77677, 76 },
    { 0x7d077678, 76 },
    { 0x39677678, 64 },
    { 0x9cb77679, 33 },
    { 0xd7b7767a, 95 },
    { 0x96c77678, 23 },
    { 0x5877767a, 66 },
    { 0x36b7767e, 95 },
    { 0xa4077677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f97768a, 93 },
    { 0, 0 },
    { 0xdb97768c, 61 },
    { 0xd7768c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f077692, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11977696, 40 },
    { 0x27177696, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f7769b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3776a0, 55 },
    { 0x3b776a0, 76 },
    { 0x70a776a2, 61 },
    { 0, 0 },
    { 0xa88776a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405776ae, 75 },
    { 0xf3c776ae, 23 },
    { 0, 0 },
    { 0x7a9776b1, 46 },
    { 0x87f776b2, 90 },
    { 0, 0 },
    { 0xee7776b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d776ba, 55 },
    { 0, 0 },
    { 0xa18776bc, 92 },
    { 0xd6a776bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165776c2, 33 },
    { 0x239776c2, 46 },
    { 0, 0 },
    { 0x641776c5, 43 },
    { 0x6cf776c6, 90 },
    { 0x7f9776c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146776ce, 90 },
    { 0x9e8776cf, 34 },
    { 0, 0 },
    { 0x1d0776d1, 90 },
    { 0x745776d1, 98 },
    { 0x74f776d1, 43 },
    { 0, 0 },
    { 0xabb776d5, 90 },
    { 0xc49776d6, 90 },
    { 0xbd9776d6, 93 },
    { 0, 0 },
    { 0xad1776d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37776df, 91 },
    { 0xd47776e0, 33 },
    { 0x4fd776e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3776eb, 84 },
    { 0xc2a776eb, 77 },
    { 0x666776eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c776f4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329776f9, 98 },
    { 0x6ad776fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75277700, 34 },
    { 0, 0 },
    { 0x68d77702, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd77708, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa827770d, 91 },
    { 0x14d7770d, 76 },
    { 0x3ac7770e, 55 },
    { 0xbdd7770d, 95 },
    { 0x177711, 55 },
    { 0x99c77711, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8777715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c77718, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7771b, 33 },
    { 0xe2d7771c, 66 },
    { 0, 0 },
    { 0x7207771e, 34 },
    { 0xab67771e, 55 },
    { 0x54277720, 76 },
    { 0x96977721, 93 },
    { 0x1ab77720, 95 },
    { 0xfe977723, 33 },
    { 0xc8977722, 98 },
    { 0xdd77721, 23 },
    { 0xae677726, 55 },
    { 0, 0 },
    { 0x38b77728, 76 },
    { 0x6a577728, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2507772e, 95 },
    { 0x4b57772f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0677739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9097773c, 98 },
    { 0, 0 },
    { 0x9b27773e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de77742, 90 },
    { 0, 0 },
    { 0x4e577744, 33 },
    { 0xc7a77745, 99 },
    { 0xae177746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44077749, 95 },
    { 0x80e7774a, 99 },
    { 0xa5f77749, 23 },
    { 0x3cd77749, 100 },
    { 0, 0 },
    { 0x9507774e, 92 },
    { 0xef47774f, 34 },
    { 0xb3877750, 91 },
    { 0x9ba7774f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23277755, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e7775a, 91 },
    { 0x2987775b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9417775e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f677761, 95 },
    { 0x2777762, 95 },
    { 0x5e77761, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e477768, 17 },
    { 0xc0277768, 38 },
    { 0, 0 },
    { 0xfd07776b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557776f, 90 },
    { 0xeda77770, 76 },
    { 0x94677771, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79077777, 33 },
    { 0xae977778, 23 },
    { 0xddf77779, 34 },
    { 0, 0 },
    { 0x9777777b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f77780, 61 },
    { 0xea777780, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75377786, 43 },
    { 0x27a77786, 43 },
    { 0, 0 },
    { 0xeee77789, 61 },
    { 0, 0 },
    { 0xa487778b, 68 },
    { 0x5cd7778c, 61 },
    { 0x51d7778d, 80 },
    { 0xb0b7778b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17377792, 98 },
    { 0, 0 },
    { 0xe5877794, 76 },
    { 0xd8c77794, 55 },
    { 0xc3777794, 61 },
    { 0xb1b77794, 36 },
    { 0, 0 },
    { 0x17a77799, 81 },
    { 0xf4d7779a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7779e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1777a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c777a7, 55 },
    { 0xca8777a7, 90 },
    { 0, 0 },
    { 0x6de777aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4777b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515777bc, 90 },
    { 0x81f777bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a777c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda777c4, 55 },
    { 0, 0 },
    { 0xceb777c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x408777c9, 98 },
    { 0x947777c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16777cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x934777d0, 98 },
    { 0xc2e777d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b777d4, 78 },
    { 0, 0 },
    { 0xe55777d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab777db, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4777e2, 40 },
    { 0x5df777e3, 23 },
    { 0xc31777e3, 55 },
    { 0xcf777e2, 95 },
    { 0, 0 },
    { 0xfaa777e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72777ee, 52 },
    { 0, 0 },
    { 0xf06777f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29777f3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x198777f6, 84 },
    { 0x427777f7, 33 },
    { 0, 0 },
    { 0x3bf777f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80577801, 90 },
    { 0, 0 },
    { 0xe2c77803, 62 },
    { 0, 0 },
    { 0x21077805, 61 },
    { 0xba677805, 76 },
    { 0, 0 },
    { 0xcfd77808, 34 },
    { 0x6d77808, 95 },
    { 0xd927780a, 85 },
    { 0xa7677808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af7780f, 100 },
    { 0, 0 },
    { 0x80377811, 67 },
    { 0xac277812, 94 },
    { 0xc4177813, 98 },
    { 0x84677814, 23 },
    { 0xfb277815, 90 },
    { 0x84877814, 33 },
    { 0x44377817, 78 },
    { 0xdb177818, 90 },
    { 0x21777815, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1347781c, 85 },
    { 0x757781d, 85 },
    { 0, 0 },
    { 0x8477781f, 91 },
    { 0x69e7781f, 23 },
    { 0, 0 },
    { 0x25f77822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb647782c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c77830, 93 },
    { 0, 0 },
    { 0x42c77832, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30377838, 90 },
    { 0xadf77839, 76 },
    { 0x5267783a, 33 },
    { 0xd1c77838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa7783f, 90 },
    { 0x8a477840, 86 },
    { 0x1c077841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c77847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f7784c, 72 },
    { 0, 0 },
    { 0xf2f7784e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab277854, 23 },
    { 0x63877854, 33 },
    { 0xf3d77856, 33 },
    { 0x2f377856, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa487785a, 67 },
    { 0, 0 },
    { 0x6907785c, 55 },
    { 0xce07785c, 98 },
    { 0x95c7785e, 33 },
    { 0xd5e7785e, 93 },
    { 0xbc27785f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8377868, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1157786b, 17 },
    { 0x74b7786c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977870, 76 },
    { 0xd3877871, 61 },
    { 0, 0 },
    { 0x90577873, 61 },
    { 0x43a77874, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db77878, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657787b, 61 },
    { 0xe3a7787c, 43 },
    { 0xec97787d, 34 },
    { 0, 0 },
    { 0x8937787f, 98 },
    { 0xdbb7787f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e177883, 84 },
    { 0, 0 },
    { 0xf077885, 76 },
    { 0xf1777886, 23 },
    { 0, 0 },
    { 0x92977888, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2547788c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b7788f, 61 },
    { 0x79577890, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c77896, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497789b, 50 },
    { 0x4c67789c, 87 },
    { 0xba7789b, 98 },
    { 0, 0 },
    { 0x7517789f, 23 },
    { 0x227778a0, 40 },
    { 0x623778a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b778a4, 77 },
    { 0x742778a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478778ad, 61 },
    { 0x571778ad, 55 },
    { 0xd58778ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x686778b2, 23 },
    { 0xff1778b3, 69 },
    { 0x64b778b3, 95 },
    { 0x24c778b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c778b9, 40 },
    { 0x626778ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c778be, 43 },
    { 0x4a8778bf, 99 },
    { 0x4df778c0, 98 },
    { 0, 0 },
    { 0xb5c778c2, 61 },
    { 0xf27778c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab778c6, 23 },
    { 0x887778c7, 95 },
    { 0x7f2778c8, 55 },
    { 0xaf4778c9, 33 },
    { 0x4d4778ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc778d3, 33 },
    { 0x71778d4, 23 },
    { 0xf78778d5, 61 },
    { 0, 0 },
    { 0xb94778d7, 95 },
    { 0x57d778d8, 46 },
    { 0, 0 },
    { 0x54e778da, 61 },
    { 0, 0 },
    { 0x489778dc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6778df, 95 },
    { 0, 0 },
    { 0x960778e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174778eb, 95 },
    { 0xabd778ec, 98 },
    { 0, 0 },
    { 0xa8b778ee, 82 },
    { 0, 0 },
    { 0x55c778f0, 66 },
    { 0xcd778f0, 85 },
    { 0, 0 },
    { 0x8a6778f3, 95 },
    { 0x8da778f4, 56 },
    { 0, 0 },
    { 0xe7d778f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08778fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f77904, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f57790a, 100 },
    { 0, 0 },
    { 0x6d37790c, 90 },
    { 0x8997790d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28377913, 33 },
    { 0, 0 },
    { 0xabf77915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c7791d, 95 },
    { 0xdf37791e, 67 },
    { 0x1387791f, 76 },
    { 0, 0 },
    { 0x4d477921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d477927, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf47792c, 23 },
    { 0x8447792d, 33 },
    { 0x64c7792d, 76 },
    { 0x1717792f, 66 },
    { 0xd947792e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9877937, 43 },
    { 0, 0 },
    { 0xca477939, 76 },
    { 0, 0 },
    { 0xfd77793b, 93 },
    { 0x35b7793c, 90 },
    { 0xb707793d, 78 },
    { 0, 0 },
    { 0xd367793f, 33 },
    { 0, 0 },
    { 0xdcf77941, 98 },
    { 0x20277942, 88 },
    { 0x7e77943, 76 },
    { 0x74d77941, 93 },
    { 0x73877945, 90 },
    { 0xd8877945, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72777949, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a7794e, 100 },
    { 0xb487794f, 66 },
    { 0x7387794f, 55 },
    { 0xa2377951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5777958, 33 },
    { 0, 0 },
    { 0xfde7795a, 78 },
    { 0, 0 },
    { 0xa0d7795c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47795f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e777964, 95 },
    { 0, 0 },
    { 0x5f677966, 90 },
    { 0xbb77967, 61 },
    { 0x82177968, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf417796b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5377796e, 95 },
    { 0xaa77796f, 83 },
    { 0x1077970, 99 },
    { 0, 0 },
    { 0x2c777972, 61 },
    { 0x66277972, 66 },
    { 0xa1d77974, 93 },
    { 0, 0 },
    { 0xff77976, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x74977979, 91 },
    { 0xa0c77979, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4127797d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5477980, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b77983, 66 },
    { 0xa1877984, 55 },
    { 0x47d77985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d77988, 33 },
    { 0, 0 },
    { 0x74e7798a, 90 },
    { 0xc6d7798b, 76 },
    { 0x24f7798c, 90 },
    { 0xe267798d, 61 },
    { 0x39c7798c, 85 },
    { 0x1b77798f, 90 },
    { 0xe9b77990, 90 },
    { 0x1f77798e, 98 },
    { 0xe7777990, 84 },
    { 0xba57798b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f37799c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d779a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49779a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709779a7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f779aa, 90 },
    { 0x1bc779aa, 33 },
    { 0xef5779ab, 23 },
    { 0x253779ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1779b1, 61 },
    { 0, 0 },
    { 0x19f779b3, 90 },
    { 0x557779b4, 33 },
    { 0x1de779b3, 100 },
    { 0x1c7779b3, 90 },
    { 0x6c0779b3, 90 },
    { 0, 0 },
    { 0x6b0779b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8779bd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e779c0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d779c6, 84 },
    { 0x64e779c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd779cc, 34 },
    { 0xf0b779cc, 33 },
    { 0x9c0779ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x123779d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f779d7, 66 },
    { 0, 0 },
    { 0xa1d779d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x398779dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e779df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1779e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87779e7, 43 },
    { 0xc23779e8, 33 },
    { 0, 0 },
    { 0xfc7779ea, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1779ed, 66 },
    { 0x219779ed, 84 },
    { 0x7b0779ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b779f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc779f7, 88 },
    { 0xcf8779f7, 90 },
    { 0xe5e779f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d779fc, 90 },
    { 0x94779fd, 93 },
    { 0, 0 },
    { 0xfda779ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77a02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84177a08, 91 },
    { 0, 0 },
    { 0xaca77a0a, 33 },
    { 0x7db77a0a, 33 },
    { 0, 0 },
    { 0xb2777a0d, 98 },
    { 0xc8677a0e, 34 },
    { 0xad877a0f, 66 },
    { 0, 0 },
    { 0x82277a11, 90 },
    { 0x26a77a12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e77a18, 66 },
    { 0xd6677a18, 76 },
    { 0x23677a19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f377a20, 90 },
    { 0xcef77a21, 55 },
    { 0, 0 },
    { 0x65677a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd977a2e, 23 },
    { 0x88977a2f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf177a32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82477a39, 23 },
    { 0, 0 },
    { 0x3d577a3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73577a45, 90 },
    { 0xe5377a46, 33 },
    { 0x1f577a45, 61 },
    { 0xf9a77a48, 95 },
    { 0, 0 },
    { 0xcc177a4a, 52 },
    { 0x3f77a4b, 64 },
    { 0x56077a4c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d877a50, 90 },
    { 0, 0 },
    { 0xc5d77a52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177a56, 95 },
    { 0, 0 },
    { 0x3a877a58, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5877a5c, 100 },
    { 0x5f977a5c, 90 },
    { 0, 0 },
    { 0x12a77a5f, 95 },
    { 0xbf77a5f, 91 },
    { 0x51d77a60, 43 },
    { 0x43d77a61, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37477a69, 99 },
    { 0xdac77a69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7877a6e, 33 },
    { 0xe2677a6e, 55 },
    { 0, 0 },
    { 0xd7077a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d777a74, 95 },
    { 0x71577a75, 99 },
    { 0, 0 },
    { 0xb9577a77, 23 },
    { 0x61377a78, 99 },
    { 0xe3e77a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f177a7e, 23 },
    { 0x61e77a7f, 87 },
    { 0x68e77a80, 66 },
    { 0xfe977a81, 61 },
    { 0xed477a80, 90 },
    { 0xf8877a83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57077a87, 82 },
    { 0, 0 },
    { 0x21d77a89, 40 },
    { 0xbee77a8a, 28 },
    { 0, 0 },
    { 0xf077a8c, 61 },
    { 0x76077a8c, 75 },
    { 0x8fd77a8e, 33 },
    { 0xb2f77a8c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e77a92, 76 },
    { 0x5d977a92, 51 },
    { 0x45077a94, 33 },
    { 0, 0 },
    { 0x7877a96, 33 },
    { 0, 0 },
    { 0xaa577a98, 68 },
    { 0, 0 },
    { 0x6d277a9a, 85 },
    { 0, 0 },
    { 0x22877a9c, 34 },
    { 0, 0 },
    { 0x77177a9e, 90 },
    { 0x12a77a9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea77aa3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c77aa6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f77aae, 90 },
    { 0xf0d77aaf, 34 },
    { 0x75877aae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c377ab7, 55 },
    { 0x87877ab8, 52 },
    { 0, 0 },
    { 0x2d877aba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec77ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85177ac4, 34 },
    { 0x2d777ac4, 98 },
    { 0x69d77ac5, 91 },
    { 0xf2077ac5, 61 },
    { 0xc3c77ac8, 78 },
    { 0x7b877ac9, 90 },
    { 0x31777ac9, 66 },
    { 0xb1877ac8, 91 },
    { 0xf0177acc, 95 },
    { 0, 0 },
    { 0xf7e77ace, 33 },
    { 0xbae77ace, 90 },
    { 0x27e77ace, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f777ad3, 33 },
    { 0x65b77ad4, 84 },
    { 0, 0 },
    { 0x92977ad6, 68 },
    { 0, 0 },
    { 0xae977ad8, 65 },
    { 0x5cf77ad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d577adc, 61 },
    { 0x74d77add, 33 },
    { 0xf6d77ade, 17 },
    { 0xfa977adf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e77aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2677aed, 100 },
    { 0, 0 },
    { 0xa7277aef, 55 },
    { 0xadc77af0, 33 },
    { 0xb3577af0, 43 },
    { 0x3a077af1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x92277af5, 98 },
    { 0xde477af5, 84 },
    { 0x17f77af6, 34 },
    { 0xefa77af7, 38 },
    { 0x24c77af9, 55 },
    { 0, 0 },
    { 0xd6c77afb, 98 },
    { 0xfa377afc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x21177aff, 88 },
    { 0x2dc77aff, 90 },
    { 0, 0 },
    { 0xb3c77b02, 91 },
    { 0xe7377b03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb777b06, 100 },
    { 0xff777b07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4377b0a, 95 },
    { 0xb7577b0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c77b0e, 95 },
    { 0x9a977b0e, 43 },
    { 0x2a077b0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7877b1c, 55 },
    { 0xb4677b1d, 98 },
    { 0, 0 },
    { 0x76c77b1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14777b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f77b2a, 90 },
    { 0x42a77b2a, 98 },
    { 0x2ed77b2b, 34 },
    { 0x5fc77b2d, 55 },
    { 0, 0 },
    { 0x10277b2f, 91 },
    { 0x4177b30, 85 },
    { 0x5977b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e77b36, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e77b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a77b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6577b40, 33 },
    { 0x18f77b41, 61 },
    { 0xc4f77b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x30077b45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb77b49, 61 },
    { 0x2fb77b49, 78 },
    { 0xb9777b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa177b4e, 33 },
    { 0x5d777b4f, 23 },
    { 0, 0 },
    { 0x87a77b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2077b54, 68 },
    { 0x6f77b55, 23 },
    { 0xb0377b56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e177b59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d977b5d, 95 },
    { 0, 0 },
    { 0x62c77b5f, 76 },
    { 0x33877b60, 76 },
    { 0xe0177b61, 38 },
    { 0x9bf77b62, 23 },
    { 0xaf077b62, 91 },
    { 0xa3577b64, 90 },
    { 0x8af77b65, 23 },
    { 0xc2b77b61, 85 },
    { 0xdf777b65, 33 },
    { 0x12177b63, 76 },
    { 0x1d477b61, 61 },
    { 0x44c77b6a, 43 },
    { 0x3c177b66, 61 },
    { 0x7b277b6c, 61 },
    { 0x5a077b6d, 90 },
    { 0x77b77b6d, 33 },
    { 0xf0977b6f, 98 },
    { 0xb0c77b6f, 97 },
    { 0xcdb77b70, 84 },
    { 0x8ce77b72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e77b79, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd77b7d, 98 },
    { 0x80a77b7e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a377b83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7077b86, 34 },
    { 0xed977b86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee177b8a, 84 },
    { 0x35777b8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb677b8f, 23 },
    { 0xd9277b8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7477b93, 23 },
    { 0x7fd77b94, 77 },
    { 0xa4b77b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc77b98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70377b9d, 84 },
    { 0xb5e77b9e, 33 },
    { 0x66277b9d, 23 },
    { 0xfa577ba0, 98 },
    { 0, 0 },
    { 0xcce77ba2, 95 },
    { 0xbf377ba3, 33 },
    { 0, 0 },
    { 0x93a77ba5, 61 },
    { 0, 0 },
    { 0x53577ba7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac477bad, 98 },
    { 0x52177bae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28377bb5, 23 },
    { 0x96777bb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd77bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80177bbf, 98 },
    { 0, 0 },
    { 0x17f77bc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0577bc5, 55 },
    { 0xfbe77bc6, 34 },
    { 0x9ad77bc6, 33 },
    { 0x64577bc6, 76 },
    { 0x51177bc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88977bcc, 98 },
    { 0, 0 },
    { 0xedf77bce, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb77bd4, 95 },
    { 0xf8b77bd5, 98 },
    { 0x1f577bd6, 95 },
    { 0xeeb77bd4, 93 },
    { 0xe9577bd7, 84 },
    { 0xf1c77bd9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37877be3, 33 },
    { 0x66777be4, 99 },
    { 0xf2b77be5, 95 },
    { 0x7eb77be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c077bf1, 95 },
    { 0x21a77bf1, 43 },
    { 0x3c377bf1, 33 },
    { 0xe9077bf1, 33 },
    { 0xece77bf5, 23 },
    { 0x53477bf6, 46 },
    { 0xa8677bf7, 46 },
    { 0x6fc77bf8, 55 },
    { 0x59a77bf6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe77bfe, 61 },
    { 0x83877bff, 33 },
    { 0, 0 },
    { 0x5e077c01, 100 },
    { 0, 0 },
    { 0x7c077c03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e477c0a, 81 },
    { 0x9d177c0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b977c0f, 33 },
    { 0x8d177c0f, 98 },
    { 0x67977c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79777c14, 98 },
    { 0x80477c15, 33 },
    { 0x3f777c16, 34 },
    { 0xfca77c15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed277c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9177c20, 65 },
    { 0xc2a77c21, 66 },
    { 0x9a677c20, 23 },
    { 0x97677c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb277c26, 90 },
    { 0xc4877c27, 61 },
    { 0x71f77c28, 33 },
    { 0xc0777c28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b77c2c, 98 },
    { 0x33277c2c, 67 },
    { 0x86e77c2e, 77 },
    { 0x6ed77c2e, 95 },
    { 0xa8f77c2e, 87 },
    { 0xc977c30, 43 },
    { 0x6ab77c32, 46 },
    { 0xd3277c33, 23 },
    { 0xad277c33, 34 },
    { 0xb3477c34, 55 },
    { 0x63e77c2d, 34 },
    { 0, 0 },
    { 0x23c77c38, 76 },
    { 0, 0 },
    { 0xd9677c3a, 34 },
    { 0, 0 },
    { 0xcf77c3c, 95 },
    { 0x6cc77c3c, 36 },
    { 0xcc977c3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32777c43, 33 },
    { 0x81477c44, 85 },
    { 0x9df77c43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf477c48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f77c51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f77c54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x64977c57, 60 },
    { 0xaeb77c58, 61 },
    { 0xa1477c59, 100 },
    { 0x97777c59, 23 },
    { 0x26c77c5b, 23 },
    { 0xbf677c5c, 91 },
    { 0x28f77c5d, 98 },
    { 0, 0 },
    { 0x16877c5f, 33 },
    { 0xea977c60, 94 },
    { 0x54277c61, 90 },
    { 0x17a77c62, 98 },
    { 0x9b677c60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb977c66, 90 },
    { 0xd1f77c67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14877c6f, 90 },
    { 0xad477c6f, 87 },
    { 0, 0 },
    { 0x93b77c72, 77 },
    { 0xbc277c72, 61 },
    { 0xc8177c74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f77c77, 98 },
    { 0xd3877c78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d677c7b, 98 },
    { 0x87177c7b, 33 },
    { 0x1d777c7d, 90 },
    { 0xd1477c7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c77c82, 34 },
    { 0x6d077c83, 61 },
    { 0, 0 },
    { 0x1fb77c85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8977c91, 91 },
    { 0x20577c92, 95 },
    { 0x4a077c92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e77c96, 66 },
    { 0xc3a77c97, 98 },
    { 0xf8f77c96, 87 },
    { 0x3b077c99, 90 },
    { 0xdde77c99, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc77c9d, 90 },
    { 0xf3377c9e, 53 },
    { 0xf5a77c9f, 44 },
    { 0, 0 },
    { 0x47877ca1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7477ca8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x22077cab, 98 },
    { 0xab077cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed77cb4, 76 },
    { 0x9c77cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd77cba, 61 },
    { 0xa0a77cbb, 93 },
    { 0xed377cbc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea77cbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71177cc4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b277cc7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26377ccf, 90 },
    { 0x2f777cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d977cd5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c77cdb, 69 },
    { 0x97477cdb, 43 },
    { 0, 0 },
    { 0xc2b77cde, 98 },
    { 0, 0 },
    { 0x5c477ce0, 93 },
    { 0x4a77ce0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a77ce9, 40 },
    { 0x6f177cea, 84 },
    { 0, 0 },
    { 0x27677cec, 61 },
    { 0x34d77ced, 61 },
    { 0xac077cee, 54 },
    { 0xd1677cef, 78 },
    { 0xbb677cef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28977cf4, 33 },
    { 0x11b77cf5, 55 },
    { 0, 0 },
    { 0xe6577cf7, 52 },
    { 0xc9977cf8, 55 },
    { 0x40777cf9, 67 },
    { 0, 0 },
    { 0xa8477cfb, 98 },
    { 0x3fa77cfc, 61 },
    { 0x10b77cfd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2977d01, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca77d04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d77d0b, 95 },
    { 0, 0 },
    { 0xe8477d0d, 33 },
    { 0xef577d0e, 95 },
    { 0x33d77d0f, 46 },
    { 0xd2e77d10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf477d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3777d19, 90 },
    { 0xa3177d1a, 66 },
    { 0x49e77d1a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc77d1e, 61 },
    { 0xf5477d1e, 61 },
    { 0x44e77d20, 95 },
    { 0xc5f77d1e, 23 },
    { 0xf9477d22, 36 },
    { 0, 0 },
    { 0xc4177d24, 55 },
    { 0xecd77d25, 76 },
    { 0x81477d25, 52 },
    { 0xafc77d27, 76 },
    { 0xbfb77d28, 93 },
    { 0xec77d28, 77 },
    { 0x13477d26, 98 },
    { 0x80b77d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e77d30, 52 },
    { 0x26177d31, 61 },
    { 0x33077d32, 90 },
    { 0, 0 },
    { 0xfc477d34, 61 },
    { 0x75777d35, 33 },
    { 0x1e577d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe677d3b, 61 },
    { 0, 0 },
    { 0xce477d3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d77d42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d377d46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81177d4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3377d50, 65 },
    { 0xa3777d51, 98 },
    { 0, 0 },
    { 0xd7a77d53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f77d58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d77d5b, 61 },
    { 0x86877d5c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67877d62, 98 },
    { 0, 0 },
    { 0x88d77d64, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee877d68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb377d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4977d73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85277d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c677d86, 91 },
    { 0xfff77d87, 99 },
    { 0, 0 },
    { 0x6e977d89, 96 },
    { 0, 0 },
    { 0x9b577d8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8477d8f, 91 },
    { 0x42c77d8f, 34 },
    { 0x5f977d8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd77d95, 34 },
    { 0x4fc77d95, 95 },
    { 0x88777d95, 90 },
    { 0x72e77d98, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba077d9d, 90 },
    { 0xd7977d9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa77da2, 98 },
    { 0xc2477da2, 33 },
    { 0x53977da4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75277da8, 34 },
    { 0, 0 },
    { 0x3bc77daa, 33 },
    { 0xef77dab, 95 },
    { 0x9ab77dab, 90 },
    { 0xe7977dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a177db4, 76 },
    { 0, 0 },
    { 0xf0377db6, 33 },
    { 0x2a077db7, 23 },
    { 0x91977db8, 46 },
    { 0x4e277db9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f777dbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f77dc2, 76 },
    { 0, 0 },
    { 0x79c77dc4, 90 },
    { 0xed777dc5, 23 },
    { 0xfbf77dc6, 76 },
    { 0x69277dc5, 61 },
    { 0, 0 },
    { 0xa2077dc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e77dd6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c277dd9, 33 },
    { 0x68277dda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a77dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46177de2, 90 },
    { 0x3e177de3, 61 },
    { 0x42777de3, 76 },
    { 0x11177de2, 90 },
    { 0xe9a77de6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f77dec, 33 },
    { 0, 0 },
    { 0xd7577dee, 34 },
    { 0, 0 },
    { 0xc7477df0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f77df3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7777df9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd77dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc977e02, 99 },
    { 0xe2777e02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d77e06, 34 },
    { 0, 0 },
    { 0x1d477e08, 98 },
    { 0x5a977e09, 33 },
    { 0x16477e08, 23 },
    { 0, 0 },
    { 0x2cb77e0c, 46 },
    { 0xdb477e0c, 67 },
    { 0x68d77e0e, 55 },
    { 0x75a77e0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f77e1a, 78 },
    { 0x4ab77e1a, 90 },
    { 0, 0 },
    { 0x33177e1d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c377e28, 23 },
    { 0x79b77e29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf577e2f, 90 },
    { 0xf1e77e30, 76 },
    { 0x76277e31, 95 },
    { 0xd7b77e2f, 98 },
    { 0, 0 },
    { 0x41277e34, 76 },
    { 0xd3677e34, 33 },
    { 0x19277e36, 23 },
    { 0x20f77e37, 61 },
    { 0, 0 },
    { 0x33577e39, 95 },
    { 0x64b77e3a, 34 },
    { 0x7bb77e3b, 88 },
    { 0x6ad77e3b, 90 },
    { 0x5e177e39, 90 },
    { 0x4a777e3e, 78 },
    { 0x60277e3c, 43 },
    { 0x4277e3c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a477e46, 98 },
    { 0, 0 },
    { 0x43a77e48, 43 },
    { 0, 0 },
    { 0xaef77e4a, 17 },
    { 0xbff77e4a, 84 },
    { 0xdcb77e4a, 33 },
    { 0xc3f77e4d, 33 },
    { 0xabf77e4d, 33 },
    { 0x72277e4b, 58 },
    { 0, 0 },
    { 0x22a77e51, 95 },
    { 0x95877e52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf477e56, 43 },
    { 0x78477e57, 77 },
    { 0x74277e58, 55 },
    { 0x7f277e58, 90 },
    { 0xd9a77e58, 76 },
    { 0x93077e58, 95 },
    { 0x43277e59, 98 },
    { 0xb5077e5d, 23 },
    { 0x2c377e5e, 33 },
    { 0x3b377e5f, 90 },
    { 0xeb77e60, 78 },
    { 0xdec77e5d, 90 },
    { 0x95077e62, 95 },
    { 0x67d77e5a, 84 },
    { 0, 0 },
    { 0x9b177e65, 96 },
    { 0x25b77e66, 83 },
    { 0x4f577e67, 90 },
    { 0xc6c77e68, 95 },
    { 0x28b77e69, 61 },
    { 0xad977e69, 90 },
    { 0xf5b77e67, 98 },
    { 0x8b377e6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf677e72, 87 },
    { 0xa5e77e73, 98 },
    { 0x9177e73, 33 },
    { 0x28077e72, 46 },
    { 0, 0 },
    { 0x8b077e77, 75 },
    { 0x82077e78, 38 },
    { 0x45b77e79, 85 },
    { 0, 0 },
    { 0x9de77e7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38977e82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0977e85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4377e88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc77e8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82577e90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3677e94, 23 },
    { 0xcc77e95, 90 },
    { 0xa2077e96, 61 },
    { 0, 0 },
    { 0x5c277e98, 76 },
    { 0, 0 },
    { 0x89a77e9a, 76 },
    { 0x27777e9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc77e9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78677ea2, 98 },
    { 0xddb77ea2, 51 },
    { 0xfeb77ea4, 33 },
    { 0x4c277ea5, 77 },
    { 0, 0 },
    { 0x94477ea7, 90 },
    { 0, 0 },
    { 0x25177ea9, 100 },
    { 0x2ef77ea9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e77eae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f77eb5, 61 },
    { 0, 0 },
    { 0x9df77eb7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x95177eba, 61 },
    { 0x9ab77ebb, 95 },
    { 0xd1677ebc, 43 },
    { 0x6d477ebd, 76 },
    { 0x9bd77ebe, 90 },
    { 0x61077ebf, 98 },
    { 0xf1f77eba, 76 },
    { 0xd7877ec1, 61 },
    { 0x58a77ec2, 23 },
    { 0xa977ec2, 95 },
    { 0xd3c77ec0, 76 },
    { 0, 0 },
    { 0x4177ec6, 38 },
    { 0xad477ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38477ecc, 76 },
    { 0x77877ecd, 61 },
    { 0x58177ece, 36 },
    { 0x7b77ecf, 98 },
    { 0, 0 },
    { 0xc0577ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d77ed5, 84 },
    { 0, 0 },
    { 0x98777ed7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d77edc, 95 },
    { 0, 0 },
    { 0x1277ede, 100 },
    { 0xceb77edf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x69677ee2, 43 },
    { 0xb6777ee3, 52 },
    { 0xedd77ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb77eec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d577eef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df77ef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda577ef9, 78 },
    { 0xfbf77efa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b77eff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6277f04, 34 },
    { 0xf1277f05, 43 },
    { 0x3af77f06, 90 },
    { 0x8a777f04, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59877f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c577f19, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4877f1c, 93 },
    { 0xef677f1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d777f21, 43 },
    { 0, 0 },
    { 0x6c877f23, 61 },
    { 0xdc077f24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5377f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a977f3f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x90277f42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc77f46, 77 },
    { 0xa6e77f46, 99 },
    { 0xd9a77f48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e077f4e, 95 },
    { 0xc5977f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb77f54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8477f61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86477f64, 33 },
    { 0xd0377f64, 95 },
    { 0xaa077f64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0477f69, 65 },
    { 0x94877f6a, 34 },
    { 0xacb77f6a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be77f74, 12 },
    { 0xc3d77f75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1277f7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd77f80, 61 },
    { 0xbee77f81, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5177f89, 33 },
    { 0xa7477f89, 99 },
    { 0, 0 },
    { 0x9ae77f8c, 76 },
    { 0xdfc77f8d, 90 },
    { 0x2df77f8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb77f92, 93 },
    { 0x9f277f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63277f96, 81 },
    { 0xfb77f96, 78 },
    { 0xb4277f96, 46 },
    { 0, 0 },
    { 0xbf677f9a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a77f9e, 85 },
    { 0, 0 },
    { 0x5f077fa0, 61 },
    { 0x58877fa1, 55 },
    { 0x74a77fa2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e77fb2, 33 },
    { 0, 0 },
    { 0x86a77fb4, 33 },
    { 0x25277fb5, 91 },
    { 0, 0 },
    { 0xbe677fb7, 33 },
    { 0, 0 },
    { 0x61677fb9, 33 },
    { 0x3ac77fba, 66 },
    { 0x82577fbb, 98 },
    { 0x99677fba, 23 },
    { 0xcc577fbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f877fc0, 61 },
    { 0x37277fc1, 46 },
    { 0, 0 },
    { 0x14c77fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba677fcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec77fd3, 99 },
    { 0xf5477fd4, 23 },
    { 0, 0 },
    { 0xf3177fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c777fd9, 90 },
    { 0, 0 },
    { 0x5f977fdb, 95 },
    { 0, 0 },
    { 0x3af77fdd, 55 },
    { 0x9ca77fde, 43 },
    { 0, 0 },
    { 0xa377fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf77fe4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f377fe8, 43 },
    { 0xcab77fe9, 95 },
    { 0x38577fe9, 67 },
    { 0xbf577fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d77fee, 98 },
    { 0, 0 },
    { 0x24b77ff0, 34 },
    { 0, 0 },
    { 0xae077ff2, 76 },
    { 0xd7077ff3, 95 },
    { 0, 0 },
    { 0x3e177ff5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda677ffe, 56 },
    { 0, 0 },
    { 0x9af78000, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec878007, 87 },
    { 0xd4c78008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6677800c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd78012, 46 },
    { 0x1c278012, 95 },
    { 0xc1a78014, 46 },
    { 0, 0 },
    { 0xcb878016, 76 },
    { 0xc5278017, 95 },
    { 0x92878018, 34 },
    { 0x8eb78019, 95 },
    { 0xc2d7801a, 46 },
    { 0xabb78016, 98 },
    { 0x6d778019, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f78020, 98 },
    { 0, 0 },
    { 0xc3278022, 99 },
    { 0x23a78023, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4297802e, 33 },
    { 0xa587802f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3578032, 98 },
    { 0xe0d78032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98478039, 33 },
    { 0xbed7803a, 33 },
    { 0xc507803b, 85 },
    { 0xdb87803b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567803f, 33 },
    { 0x87478040, 78 },
    { 0x2a178041, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12078049, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee78051, 65 },
    { 0xf0378051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e27805d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4378060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687806a, 75 },
    { 0x6917806b, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7806f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a78072, 23 },
    { 0x8eb78072, 95 },
    { 0, 0 },
    { 0xddc78075, 93 },
    { 0x3b678076, 61 },
    { 0x76478077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7417807b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c7807e, 98 },
    { 0x2067807e, 98 },
    { 0, 0 },
    { 0x70a78081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc78085, 93 },
    { 0, 0 },
    { 0xfa478087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f57808d, 55 },
    { 0x9f07808e, 33 },
    { 0x747808d, 90 },
    { 0x3b27808e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e78093, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b78096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7697809a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a780a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60780b8, 93 },
    { 0xac5780b9, 85 },
    { 0xf1a780ba, 33 },
    { 0x5c0780bb, 23 },
    { 0, 0 },
    { 0xafa780bd, 33 },
    { 0, 0 },
    { 0xd11780bf, 84 },
    { 0x6b9780c0, 61 },
    { 0, 0 },
    { 0x28f780c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f780c6, 23 },
    { 0xd02780c7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d780ca, 90 },
    { 0x5a5780cb, 33 },
    { 0x6a3780cc, 90 },
    { 0xa3b780cd, 33 },
    { 0, 0 },
    { 0x9c2780cf, 90 },
    { 0xdd3780d0, 46 },
    { 0x64780d0, 61 },
    { 0xada780cf, 95 },
    { 0xe50780cf, 43 },
    { 0x344780d4, 98 },
    { 0, 0 },
    { 0x9a780d6, 90 },
    { 0x2be780d7, 95 },
    { 0x232780d8, 46 },
    { 0, 0 },
    { 0x5c8780da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3780e1, 99 },
    { 0xec9780e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f780e6, 98 },
    { 0xb0780e7, 100 },
    { 0xee6780e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9780ec, 77 },
    { 0, 0 },
    { 0xf45780ee, 87 },
    { 0xa8b780ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c780f8, 33 },
    { 0xb74780f9, 93 },
    { 0x5c8780fa, 66 },
    { 0, 0 },
    { 0x808780fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75780ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf78105, 93 },
    { 0, 0 },
    { 0x51178107, 98 },
    { 0x59378108, 53 },
    { 0x17f78108, 95 },
    { 0x69b7810a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9047810d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd278110, 78 },
    { 0xfbc78111, 23 },
    { 0xa0678112, 95 },
    { 0xfe678110, 43 },
    { 0, 0 },
    { 0x70778115, 92 },
    { 0, 0 },
    { 0x83478117, 33 },
    { 0x73978117, 33 },
    { 0x97e78118, 95 },
    { 0xf47811a, 87 },
    { 0, 0 },
    { 0xc957811c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097811f, 61 },
    { 0xb3378120, 61 },
    { 0, 0 },
    { 0xead78122, 46 },
    { 0, 0 },
    { 0xaf278124, 33 },
    { 0x8c178125, 98 },
    { 0x90278126, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a57812a, 46 },
    { 0xffd7812b, 46 },
    { 0xc4a7812c, 98 },
    { 0, 0 },
    { 0x3cd7812e, 23 },
    { 0xd817812e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82878138, 28 },
    { 0xf4778138, 33 },
    { 0x5e978138, 55 },
    { 0x1ee7813b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc167813e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53478143, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2678146, 100 },
    { 0, 0 },
    { 0x2e678148, 88 },
    { 0xe0578148, 33 },
    { 0, 0 },
    { 0x1ba7814b, 77 },
    { 0x6937814b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14278150, 61 },
    { 0x89878150, 46 },
    { 0x31778150, 33 },
    { 0xb8a78153, 61 },
    { 0x1c478154, 95 },
    { 0, 0 },
    { 0x3378156, 61 },
    { 0x6478157, 48 },
    { 0x9ba78157, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f97815e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e878163, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63778166, 34 },
    { 0, 0 },
    { 0x65a78168, 95 },
    { 0xcc078168, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617816c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a178172, 88 },
    { 0x4fa78172, 90 },
    { 0x4f478173, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a7817b, 90 },
    { 0x1667817b, 61 },
    { 0x167817d, 61 },
    { 0x1eb7817d, 88 },
    { 0xf027817d, 34 },
    { 0, 0 },
    { 0xb2e78181, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd78186, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77818a, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d7818e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92678195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc78199, 98 },
    { 0xcb27819a, 90 },
    { 0x68f7819b, 95 },
    { 0xdf77819b, 33 },
    { 0xd567819d, 93 },
    { 0x13b78199, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38781a2, 80 },
    { 0, 0 },
    { 0x1fd781a4, 98 },
    { 0xd23781a4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa781a8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8781ad, 91 },
    { 0x75e781ae, 85 },
    { 0x350781ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f781b3, 23 },
    { 0, 0 },
    { 0x86781b5, 90 },
    { 0, 0 },
    { 0x839781b7, 93 },
    { 0x5a6781b8, 100 },
    { 0x592781b8, 33 },
    { 0x289781ba, 23 },
    { 0x958781bb, 46 },
    { 0xd3b781ba, 55 },
    { 0, 0 },
    { 0x9781be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1781c3, 90 },
    { 0x8ec781c4, 61 },
    { 0, 0 },
    { 0xbdc781c6, 46 },
    { 0x8b4781c6, 99 },
    { 0, 0 },
    { 0xd40781c9, 33 },
    { 0x6d5781ca, 98 },
    { 0x16a781ca, 61 },
    { 0xa85781cb, 98 },
    { 0x6f0781cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff781d0, 80 },
    { 0, 0 },
    { 0x8b6781d2, 90 },
    { 0x9f781d2, 91 },
    { 0xf03781d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81781d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b781de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a781e3, 90 },
    { 0x421781e4, 90 },
    { 0, 0 },
    { 0x1f9781e6, 33 },
    { 0xa9c781e7, 76 },
    { 0xde9781e6, 61 },
    { 0x9be781e8, 90 },
    { 0xe87781ea, 88 },
    { 0xb3a781eb, 34 },
    { 0xb22781eb, 76 },
    { 0x212781ed, 33 },
    { 0x99c781ee, 61 },
    { 0x4cf781ef, 88 },
    { 0x771781ea, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602781f4, 46 },
    { 0, 0 },
    { 0x99a781f6, 55 },
    { 0, 0 },
    { 0xe05781f8, 98 },
    { 0xb94781f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42878201, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe177820e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7778212, 90 },
    { 0x66278212, 95 },
    { 0x3d678214, 33 },
    { 0, 0 },
    { 0x72878216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c7821b, 76 },
    { 0xc387821c, 100 },
    { 0xf707821c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b78225, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x42778228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6417822e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e78235, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf7823b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba78240, 95 },
    { 0, 0 },
    { 0xe3f78242, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1578246, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb67824b, 84 },
    { 0x5cd7824b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56078251, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x86978254, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a78259, 36 },
    { 0, 0 },
    { 0xe077825b, 87 },
    { 0x1937825c, 66 },
    { 0x5957825d, 43 },
    { 0, 0 },
    { 0xb607825f, 78 },
    { 0x8fb78260, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f7826b, 98 },
    { 0, 0 },
    { 0xa137826d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f78277, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4127827c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb878283, 95 },
    { 0, 0 },
    { 0xbca78285, 43 },
    { 0xd5378285, 33 },
    { 0x91d78285, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa087828a, 90 },
    { 0xb897828b, 33 },
    { 0xa417828c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68078291, 90 },
    { 0x4c978291, 98 },
    { 0, 0 },
    { 0x77678294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d78297, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc7829e, 23 },
    { 0x7567829f, 88 },
    { 0x5b97829f, 95 },
    { 0x5807829f, 61 },
    { 0x3b0782a2, 61 },
    { 0xc2b7829e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0782a7, 23 },
    { 0xf55782a7, 34 },
    { 0x1b0782a9, 61 },
    { 0x8bf782a9, 98 },
    { 0xd9b782a8, 76 },
    { 0, 0 },
    { 0x5fa782ad, 66 },
    { 0, 0 },
    { 0xc63782af, 90 },
    { 0, 0 },
    { 0x98c782b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac782b6, 77 },
    { 0x3fa782b7, 34 },
    { 0xc53782b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f782bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc782bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33782c2, 76 },
    { 0x798782c3, 90 },
    { 0x1f4782c4, 90 },
    { 0, 0 },
    { 0x216782c6, 76 },
    { 0xf14782c6, 95 },
    { 0, 0 },
    { 0x6fe782c9, 61 },
    { 0x251782c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2782d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce782d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13782d7, 87 },
    { 0x4ed782d8, 46 },
    { 0, 0 },
    { 0xf9a782da, 34 },
    { 0xbaa782da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd782de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2782e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24782e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x323782e8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172782ec, 85 },
    { 0xfd9782ed, 76 },
    { 0x7e9782ee, 61 },
    { 0, 0 },
    { 0x8de782f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56782f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x594782f7, 23 },
    { 0x4e5782f7, 23 },
    { 0, 0 },
    { 0x597782fa, 95 },
    { 0x8f7782fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e782fe, 99 },
    { 0xc51782ff, 33 },
    { 0xa2e782fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89878309, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe257830d, 67 },
    { 0xc467830e, 95 },
    { 0xb987830e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a078313, 76 },
    { 0x21978314, 90 },
    { 0xf0278314, 61 },
    { 0x56578316, 98 },
    { 0, 0 },
    { 0x1578318, 95 },
    { 0, 0 },
    { 0x21f7831a, 98 },
    { 0x18c7831b, 99 },
    { 0xc087831a, 43 },
    { 0x9977831d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded78321, 61 },
    { 0, 0 },
    { 0x7ff78323, 90 },
    { 0xa9d78323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d7832d, 90 },
    { 0, 0 },
    { 0xa0e7832f, 76 },
    { 0xa7578330, 87 },
    { 0xd5078330, 76 },
    { 0xdc078331, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f478335, 81 },
    { 0xd5978336, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3577833c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b78344, 90 },
    { 0, 0 },
    { 0x58878346, 87 },
    { 0xb4c78346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1277834a, 95 },
    { 0, 0 },
    { 0xa6c7834c, 95 },
    { 0xd597834d, 33 },
    { 0, 0 },
    { 0xa57834f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa78354, 33 },
    { 0x3c878355, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca78358, 98 },
    { 0xa4378359, 91 },
    { 0xa8d78359, 77 },
    { 0x68578359, 95 },
    { 0x5297835c, 90 },
    { 0x15078359, 100 },
    { 0x89778359, 17 },
    { 0, 0 },
    { 0x27d78360, 90 },
    { 0x5d978360, 61 },
    { 0x8f278362, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c078367, 61 },
    { 0xab878367, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf227836d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x96778370, 33 },
    { 0, 0 },
    { 0xb4c78372, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35278376, 23 },
    { 0, 0 },
    { 0x7d778378, 76 },
    { 0x79278379, 61 },
    { 0xfda78379, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff278384, 37 },
    { 0, 0 },
    { 0xf4f78386, 90 },
    { 0x5e578387, 95 },
    { 0x1c78386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d7838b, 61 },
    { 0, 0 },
    { 0xa337838d, 55 },
    { 0, 0 },
    { 0x4147838f, 61 },
    { 0xa6f7838f, 61 },
    { 0x3b978391, 61 },
    { 0x5678392, 23 },
    { 0x69178393, 61 },
    { 0xbc278394, 53 },
    { 0xa0b78392, 34 },
    { 0x33978396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10783a0, 34 },
    { 0, 0 },
    { 0x7b783a2, 61 },
    { 0x798783a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2783ab, 90 },
    { 0xb9a783ab, 43 },
    { 0xae783ad, 83 },
    { 0xe3a783ae, 90 },
    { 0xf4b783af, 98 },
    { 0xcdc783b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61783b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x228783b7, 95 },
    { 0, 0 },
    { 0xe3c783b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc783bc, 43 },
    { 0xf3f783bd, 61 },
    { 0, 0 },
    { 0x32f783bf, 61 },
    { 0, 0 },
    { 0xf81783c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x767783c4, 95 },
    { 0xd9f783c4, 80 },
    { 0x56a783c4, 77 },
    { 0, 0 },
    { 0x8a7783c8, 90 },
    { 0x7f783c9, 98 },
    { 0, 0 },
    { 0x476783cb, 61 },
    { 0x11b783cc, 75 },
    { 0x793783cd, 43 },
    { 0, 0 },
    { 0xdf0783cf, 65 },
    { 0xb7f783cf, 61 },
    { 0, 0 },
    { 0xb4d783d2, 33 },
    { 0, 0 },
    { 0x9af783d4, 98 },
    { 0x8be783d5, 23 },
    { 0x474783d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1783db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816783e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09783e5, 61 },
    { 0, 0 },
    { 0x27b783e7, 43 },
    { 0xcc1783e8, 78 },
    { 0x7d6783e9, 87 },
    { 0x78f783e9, 46 },
    { 0, 0 },
    { 0x806783ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb783f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f978406, 61 },
    { 0x26578406, 61 },
    { 0, 0 },
    { 0x69978409, 33 },
    { 0, 0 },
    { 0x25d7840b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a278413, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf878417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197841b, 90 },
    { 0xc2b7841b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3507841f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e378423, 76 },
    { 0x5d478424, 34 },
    { 0x83c78425, 78 },
    { 0xb4578424, 88 },
    { 0, 0 },
    { 0x35378428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c7842b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac7842f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d78432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b78435, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc78439, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f7843c, 90 },
    { 0x4ed7843c, 95 },
    { 0x7e47843e, 95 },
    { 0xf47843f, 97 },
    { 0x5ff78440, 90 },
    { 0x62978440, 98 },
    { 0x89b7843c, 93 },
    { 0, 0 },
    { 0xe4378444, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2097844d, 34 },
    { 0x2f77844d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a78454, 61 },
    { 0xe6178455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46078458, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b7845e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc978467, 76 },
    { 0, 0 },
    { 0x27b78469, 46 },
    { 0x52178469, 93 },
    { 0x5777846a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca78470, 90 },
    { 0x77c78471, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b78476, 66 },
    { 0xe8078476, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca57847d, 55 },
    { 0xb627847d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478481, 55 },
    { 0x46078481, 90 },
    { 0x5fb78483, 98 },
    { 0x71778484, 61 },
    { 0, 0 },
    { 0x30178486, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7378489, 98 },
    { 0x7077848a, 61 },
    { 0x8f17848b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7848f, 34 },
    { 0xb9078490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf878493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c37849d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3784ad, 90 },
    { 0xf70784ae, 33 },
    { 0xb28784af, 98 },
    { 0xf07784b0, 78 },
    { 0x64784b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d784b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be784bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00784be, 66 },
    { 0, 0 },
    { 0x365784c0, 95 },
    { 0, 0 },
    { 0x477784c2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94784cd, 46 },
    { 0, 0 },
    { 0xc02784cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3784d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9784d6, 87 },
    { 0x10e784d6, 98 },
    { 0x6d7784d8, 90 },
    { 0x9e3784d7, 33 },
    { 0x823784d6, 33 },
    { 0, 0 },
    { 0xe74784dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x786784df, 55 },
    { 0, 0 },
    { 0xe6b784e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d784e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x881784e9, 90 },
    { 0xf12784ea, 33 },
    { 0x9f9784eb, 90 },
    { 0x1e6784ea, 95 },
    { 0, 0 },
    { 0x26784ee, 90 },
    { 0x5ed784ef, 95 },
    { 0xab5784f0, 95 },
    { 0x55e784ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255784f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x753784f9, 90 },
    { 0xcf9784fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x608784fd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9978500, 69 },
    { 0, 0 },
    { 0x69c78502, 55 },
    { 0, 0 },
    { 0x8c178504, 95 },
    { 0xbf878504, 98 },
    { 0, 0 },
    { 0xd2178507, 100 },
    { 0x2c778508, 90 },
    { 0x2ce78507, 95 },
    { 0, 0 },
    { 0xc6b7850b, 33 },
    { 0x44a7850b, 90 },
    { 0, 0 },
    { 0xb97850e, 61 },
    { 0x9a07850f, 95 },
    { 0xd7f7850f, 61 },
    { 0xf527850e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd78516, 46 },
    { 0x74278516, 23 },
    { 0x24b78518, 98 },
    { 0x16278519, 91 },
    { 0, 0 },
    { 0xfb37851b, 23 },
    { 0xe57851c, 33 },
    { 0x4cb7851b, 61 },
    { 0x5c87851e, 77 },
    { 0x2f67851d, 61 },
    { 0, 0 },
    { 0x93f78521, 72 },
    { 0x5ad78521, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48578528, 95 },
    { 0, 0 },
    { 0x5947852a, 61 },
    { 0x6477852b, 95 },
    { 0x47d7852b, 33 },
    { 0xe217852d, 92 },
    { 0xe0d7852e, 61 },
    { 0xb77852e, 98 },
    { 0xa8b78530, 61 },
    { 0x4be7852f, 61 },
    { 0xe4f7852b, 90 },
    { 0x69078531, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x62178536, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21078539, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c7853c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b578542, 76 },
    { 0xfd478543, 90 },
    { 0x30d78544, 56 },
    { 0x62478544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d7854b, 90 },
    { 0xd077854c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73778551, 33 },
    { 0x9fc78552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74978557, 66 },
    { 0x61178557, 61 },
    { 0x91e78559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d07855e, 33 },
    { 0, 0 },
    { 0xf6878560, 33 },
    { 0x1e378561, 98 },
    { 0xd7a78562, 98 },
    { 0x4eb78563, 90 },
    { 0, 0 },
    { 0xbf278565, 61 },
    { 0x69778565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7856b, 61 },
    { 0xcc27856c, 95 },
    { 0xfc27856c, 81 },
    { 0xaed7856c, 87 },
    { 0x18a7856f, 85 },
    { 0x3207856c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5878573, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4178576, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a78579, 69 },
    { 0x3547857a, 78 },
    { 0x1fb78579, 61 },
    { 0x1eb7857b, 34 },
    { 0x9037857b, 90 },
    { 0xb627857e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1078581, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1078587, 33 },
    { 0x38978587, 93 },
    { 0, 0 },
    { 0xdeb7858a, 78 },
    { 0xb987858b, 33 },
    { 0, 0 },
    { 0xa4e7858d, 76 },
    { 0xaf57858e, 23 },
    { 0xfec7858e, 33 },
    { 0, 0 },
    { 0x95f78591, 77 },
    { 0x32a78592, 90 },
    { 0xfe278592, 23 },
    { 0x4eb78591, 56 },
    { 0x3c278595, 100 },
    { 0xae178596, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc807859a, 87 },
    { 0xb1a7859a, 43 },
    { 0, 0 },
    { 0xa3f7859d, 23 },
    { 0x76f7859e, 99 },
    { 0xfc07859f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1785a6, 83 },
    { 0, 0 },
    { 0x5ba785a8, 84 },
    { 0, 0 },
    { 0xcfb785aa, 61 },
    { 0xb24785aa, 91 },
    { 0x979785ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d785af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192785b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f785be, 34 },
    { 0xd12785be, 23 },
    { 0x242785c0, 53 },
    { 0, 0 },
    { 0xc58785c2, 23 },
    { 0xca8785c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x593785c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1785c9, 65 },
    { 0x7e0785ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f785ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x922785d1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce785df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f785eb, 61 },
    { 0, 0 },
    { 0xd4d785ed, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x862785f0, 78 },
    { 0, 0 },
    { 0x64e785f2, 90 },
    { 0x139785f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba785f6, 61 },
    { 0x8ce785f7, 55 },
    { 0x474785f7, 76 },
    { 0xa8f785f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3785fc, 68 },
    { 0x171785fc, 76 },
    { 0, 0 },
    { 0x45b785ff, 23 },
    { 0x78d785ff, 23 },
    { 0x4e78600, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee278605, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b7860c, 98 },
    { 0x9847860c, 65 },
    { 0xa907860d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2978611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3478615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45878619, 100 },
    { 0xdbe7861a, 98 },
    { 0, 0 },
    { 0xdb57861c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d178621, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b78627, 55 },
    { 0, 0 },
    { 0xc8f78629, 52 },
    { 0x3c678629, 23 },
    { 0, 0 },
    { 0x3ba7862c, 55 },
    { 0x1ef7862d, 80 },
    { 0x1b7862c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3678639, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa837863c, 88 },
    { 0x4c7863d, 69 },
    { 0xaf17863e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb7864a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1078651, 80 },
    { 0x6c678652, 90 },
    { 0x55a78653, 81 },
    { 0x92c78651, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b78657, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe967865a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0178664, 78 },
    { 0x3c278664, 52 },
    { 0x69f78666, 100 },
    { 0xeb978666, 67 },
    { 0x26b78668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1557866f, 88 },
    { 0, 0 },
    { 0xa8378671, 98 },
    { 0xc6f78671, 90 },
    { 0xeca78673, 90 },
    { 0x16178673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f78677, 33 },
    { 0x6a778678, 76 },
    { 0xd2178677, 33 },
    { 0xdd27867a, 99 },
    { 0x8797867b, 61 },
    { 0xedf7867b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a478683, 95 },
    { 0xe7778684, 90 },
    { 0, 0 },
    { 0x25c78686, 76 },
    { 0xc1378687, 90 },
    { 0x97c78686, 61 },
    { 0x9a278689, 36 },
    { 0x32478687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567868e, 33 },
    { 0, 0 },
    { 0x45978690, 78 },
    { 0, 0 },
    { 0xf8678692, 61 },
    { 0xff378693, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x37578696, 98 },
    { 0, 0 },
    { 0x1b278698, 23 },
    { 0x35878699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea786a6, 69 },
    { 0, 0 },
    { 0xa93786a8, 34 },
    { 0x294786a9, 95 },
    { 0xdc1786aa, 55 },
    { 0, 0 },
    { 0x192786ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff786bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6786c4, 95 },
    { 0x51f786c5, 23 },
    { 0xc51786c4, 93 },
    { 0x15d786c6, 61 },
    { 0, 0 },
    { 0x1f2786c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb786ce, 23 },
    { 0x569786cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0786d8, 96 },
    { 0xd78786d9, 33 },
    { 0xaa1786da, 98 },
    { 0x10786db, 33 },
    { 0x976786dc, 23 },
    { 0x689786dc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e786e1, 23 },
    { 0, 0 },
    { 0xe4a786e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37786e8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x341786eb, 61 },
    { 0x14f786ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754786fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f278705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e78709, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab7870d, 23 },
    { 0x6007870d, 90 },
    { 0, 0 },
    { 0x76f78710, 61 },
    { 0xcd978711, 98 },
    { 0x23678712, 98 },
    { 0, 0 },
    { 0x70978714, 23 },
    { 0x6ea78715, 81 },
    { 0xb5878714, 95 },
    { 0x1178716, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b27871c, 91 },
    { 0x73e7871c, 61 },
    { 0, 0 },
    { 0xe2e7871f, 76 },
    { 0xc0f78720, 23 },
    { 0xa5d78721, 95 },
    { 0xbd578722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a78726, 33 },
    { 0xf1578726, 61 },
    { 0x73578728, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc97872e, 78 },
    { 0xd447872e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db78732, 93 },
    { 0xb0b78733, 33 },
    { 0xf9378734, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb78737, 91 },
    { 0x3b078738, 95 },
    { 0, 0 },
    { 0x8cc7873a, 95 },
    { 0, 0 },
    { 0xa827873c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78742, 95 },
    { 0xafb78743, 100 },
    { 0xe3778744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e78747, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8977874a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd27874d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8678750, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe137875e, 55 },
    { 0x6df7875e, 90 },
    { 0xd447875f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f78763, 95 },
    { 0xf0378764, 43 },
    { 0xcb678763, 82 },
    { 0x3b78766, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee478769, 95 },
    { 0, 0 },
    { 0x9157876b, 33 },
    { 0xf1e7876b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef578770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21478775, 52 },
    { 0xdf678775, 99 },
    { 0x76478776, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a7877f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a78783, 33 },
    { 0x3f878783, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xce978787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8217878a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb87878f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83878794, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e578798, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d787a1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29787a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0787b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87787b7, 55 },
    { 0x905787b8, 33 },
    { 0xf787b9, 33 },
    { 0, 0 },
    { 0x4e5787bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74787c1, 85 },
    { 0x927787c2, 36 },
    { 0x79e787c3, 52 },
    { 0xda1787c2, 99 },
    { 0x48d787c2, 61 },
    { 0, 0 },
    { 0x1ad787c7, 91 },
    { 0xd1e787c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92787cd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63787d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2787d5, 33 },
    { 0, 0 },
    { 0xbdd787d7, 55 },
    { 0xf4787d8, 90 },
    { 0x556787d9, 98 },
    { 0xd17787da, 84 },
    { 0xf2d787db, 33 },
    { 0, 0 },
    { 0x2cf787dd, 33 },
    { 0x346787de, 33 },
    { 0x270787df, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c787e3, 23 },
    { 0, 0 },
    { 0x5bc787e5, 90 },
    { 0xef1787e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3787e9, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea787ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786787f2, 100 },
    { 0xa0787f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6787f6, 40 },
    { 0x39c787f7, 90 },
    { 0xefc787f7, 90 },
    { 0x521787f9, 90 },
    { 0xc76787fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0787ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5378803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa247880a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a97880d, 23 },
    { 0, 0 },
    { 0xb197880f, 61 },
    { 0, 0 },
    { 0x8dd78811, 78 },
    { 0, 0 },
    { 0x7378813, 66 },
    { 0x5e478813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b78817, 100 },
    { 0xc4478818, 98 },
    { 0x46f78819, 76 },
    { 0, 0 },
    { 0x9527881b, 46 },
    { 0x38c7881c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f178821, 34 },
    { 0x80278821, 66 },
    { 0x27f78823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24278828, 55 },
    { 0, 0 },
    { 0x3be7882a, 78 },
    { 0, 0 },
    { 0x88d7882c, 99 },
    { 0x1e37882d, 33 },
    { 0x1217882c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4678833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b378836, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c77883a, 90 },
    { 0xbd87883b, 90 },
    { 0, 0 },
    { 0xf527883d, 98 },
    { 0x5687883e, 87 },
    { 0xb147883f, 23 },
    { 0, 0 },
    { 0x5c578841, 94 },
    { 0, 0 },
    { 0x9e378843, 90 },
    { 0x9a078844, 61 },
    { 0xef578845, 33 },
    { 0x93f78846, 76 },
    { 0xa9c78847, 91 },
    { 0, 0 },
    { 0xa7378849, 46 },
    { 0, 0 },
    { 0x6207884b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c78855, 33 },
    { 0xd0f78856, 61 },
    { 0xca878857, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8867885c, 23 },
    { 0, 0 },
    { 0xa727885e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77878863, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd478867, 98 },
    { 0, 0 },
    { 0xa7478869, 65 },
    { 0x5217886a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c7886e, 33 },
    { 0xc937886f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8478872, 98 },
    { 0, 0 },
    { 0xee378874, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3078877, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3267887d, 98 },
    { 0xac57887e, 85 },
    { 0xa8b7887e, 98 },
    { 0, 0 },
    { 0xfdf78881, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b78885, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e7888a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d7888f, 76 },
    { 0x3d378890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65278893, 95 },
    { 0, 0 },
    { 0x6278895, 98 },
    { 0, 0 },
    { 0x14d78897, 33 },
    { 0x49678897, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8627889c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b7889f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0788a5, 46 },
    { 0, 0 },
    { 0x7b8788a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3788aa, 34 },
    { 0x85f788ab, 28 },
    { 0x566788ac, 76 },
    { 0x666788ac, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210788b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31788b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117788b8, 99 },
    { 0xc7e788b9, 91 },
    { 0x99b788b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d788c1, 43 },
    { 0xb0c788c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a788c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd788cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf788cf, 90 },
    { 0xbb788d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8788d5, 90 },
    { 0, 0 },
    { 0x2fd788d7, 90 },
    { 0, 0 },
    { 0xabb788d9, 33 },
    { 0xc31788da, 97 },
    { 0x705788db, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2788df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee788e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x183788e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e788e8, 98 },
    { 0, 0 },
    { 0x8e5788ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee788f5, 33 },
    { 0x421788f5, 87 },
    { 0x13b788f6, 55 },
    { 0, 0 },
    { 0x6db788f9, 76 },
    { 0x79a788f9, 99 },
    { 0xd54788fb, 87 },
    { 0, 0 },
    { 0x98d788fd, 85 },
    { 0, 0 },
    { 0xbb788ff, 33 },
    { 0xd9b78900, 34 },
    { 0xb1678901, 33 },
    { 0x54378902, 98 },
    { 0x6c578903, 98 },
    { 0xf4678903, 90 },
    { 0x69d78905, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1817890a, 23 },
    { 0x5f7890b, 98 },
    { 0x3d97890a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf7890f, 33 },
    { 0, 0 },
    { 0xc5278911, 68 },
    { 0, 0 },
    { 0xe1f78913, 67 },
    { 0x86678913, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c7891a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82178924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68978927, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc247892a, 90 },
    { 0xe327892b, 23 },
    { 0, 0 },
    { 0x78a7892d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf78934, 76 },
    { 0xadc78935, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8367893c, 99 },
    { 0x2287893d, 91 },
    { 0xf137893d, 43 },
    { 0x6257893d, 33 },
    { 0x3097893d, 61 },
    { 0x85678941, 23 },
    { 0x5877893f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x52878945, 67 },
    { 0x84578946, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7894a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe687894d, 81 },
    { 0x6ac7894e, 90 },
    { 0x3a47894f, 33 },
    { 0x61378950, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88678958, 61 },
    { 0x48978959, 36 },
    { 0x4dc7895a, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd178960, 33 },
    { 0, 0 },
    { 0x51678962, 95 },
    { 0x2d878962, 98 },
    { 0, 0 },
    { 0x80678965, 23 },
    { 0x9cf78966, 61 },
    { 0x7878967, 90 },
    { 0xb1678968, 55 },
    { 0x14d78969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae87896e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63378971, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5378976, 34 },
    { 0xdbe78977, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547897c, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2378985, 90 },
    { 0, 0 },
    { 0x64b78987, 55 },
    { 0x6a778987, 46 },
    { 0x66178989, 33 },
    { 0x4f67898a, 76 },
    { 0x71d7898b, 61 },
    { 0xb777898a, 90 },
    { 0, 0 },
    { 0x8847898e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d878993, 99 },
    { 0, 0 },
    { 0x7f578995, 90 },
    { 0x1a778996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8578999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a7899c, 87 },
    { 0, 0 },
    { 0xbe47899e, 56 },
    { 0, 0 },
    { 0xa33789a0, 33 },
    { 0x99b789a0, 98 },
    { 0xd65789a2, 90 },
    { 0x9a0789a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b789a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46789ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b789af, 65 },
    { 0, 0 },
    { 0x419789b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f789b5, 98 },
    { 0x6a2789b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11789bc, 34 },
    { 0, 0 },
    { 0x2eb789be, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d789c8, 95 },
    { 0, 0 },
    { 0x397789ca, 90 },
    { 0x890789cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c789d1, 95 },
    { 0x44b789d1, 90 },
    { 0, 0 },
    { 0xd1c789d4, 90 },
    { 0x3e789d5, 98 },
    { 0x7fb789d6, 90 },
    { 0x5d4789d6, 98 },
    { 0x7d5789d8, 66 },
    { 0, 0 },
    { 0xe77789da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x703789dd, 61 },
    { 0, 0 },
    { 0xa50789df, 61 },
    { 0, 0 },
    { 0x973789e1, 51 },
    { 0, 0 },
    { 0xe6a789e3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x195789e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f789ec, 100 },
    { 0xc9b789ed, 33 },
    { 0, 0 },
    { 0x6d4789ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476789f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472789f7, 98 },
    { 0x8cf789f7, 61 },
    { 0x57e789f8, 61 },
    { 0x829789fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f789fe, 77 },
    { 0xadd789ff, 87 },
    { 0x21b78a00, 65 },
    { 0xa4478a00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3978a04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5678a0a, 66 },
    { 0xd0978a0b, 33 },
    { 0xc078a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e178a0f, 85 },
    { 0, 0 },
    { 0xbee78a11, 76 },
    { 0x35678a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf978a15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69878a1a, 23 },
    { 0, 0 },
    { 0xd9778a1c, 90 },
    { 0x2eb78a1d, 34 },
    { 0xa4378a1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d78a23, 33 },
    { 0xeff78a24, 23 },
    { 0xb5e78a25, 84 },
    { 0xafb78a26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9978a2d, 61 },
    { 0, 0 },
    { 0x6e478a2f, 61 },
    { 0xc6b78a30, 33 },
    { 0, 0 },
    { 0xf8478a32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2778a35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a78a38, 90 },
    { 0xca978a38, 98 },
    { 0x9d378a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c578a3e, 61 },
    { 0x8f978a3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5778a42, 33 },
    { 0x44178a42, 33 },
    { 0xd6d78a43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb78a47, 97 },
    { 0x33f78a47, 99 },
    { 0x8c78a47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84678a4e, 98 },
    { 0, 0 },
    { 0xaca78a50, 61 },
    { 0x13678a50, 95 },
    { 0xac178a52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f878a57, 78 },
    { 0xa9878a57, 61 },
    { 0xef178a59, 34 },
    { 0x7d678a57, 95 },
    { 0xc6778a58, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66378a5f, 93 },
    { 0x6ea78a5f, 78 },
    { 0xa7a78a60, 23 },
    { 0xbc878a62, 95 },
    { 0, 0 },
    { 0xd9078a64, 98 },
    { 0xb4078a65, 76 },
    { 0x4978a66, 88 },
    { 0xe8b78a67, 90 },
    { 0x77a78a67, 90 },
    { 0x4e578a65, 95 },
    { 0xad378a6a, 90 },
    { 0x79f78a64, 75 },
    { 0xcc478a6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68478a70, 33 },
    { 0, 0 },
    { 0x5a378a72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df78a83, 61 },
    { 0xa7878a83, 98 },
    { 0x5bb78a85, 61 },
    { 0x1a978a84, 33 },
    { 0xc7278a87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb078a8f, 76 },
    { 0, 0 },
    { 0x34978a91, 55 },
    { 0, 0 },
    { 0xb1c78a93, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d78a96, 61 },
    { 0xe778a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a78a9a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c978a9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d78aa0, 23 },
    { 0x7978aa1, 77 },
    { 0xdbd78aa2, 33 },
    { 0xb5c78aa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43178aa8, 23 },
    { 0, 0 },
    { 0xc1578aaa, 43 },
    { 0, 0 },
    { 0x84f78aac, 55 },
    { 0xb1d78aad, 55 },
    { 0x96b78aad, 87 },
    { 0x8f678aac, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2478ab7, 90 },
    { 0xcda78ab7, 67 },
    { 0xa0078ab8, 90 },
    { 0x18c78aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92978abf, 69 },
    { 0, 0 },
    { 0xc7378ac1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16478acb, 90 },
    { 0, 0 },
    { 0x2b178acd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb78ad0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed78ada, 93 },
    { 0xb5b78adb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15678adf, 90 },
    { 0x68c78adf, 95 },
    { 0x45078ae0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48178ae9, 38 },
    { 0x56d78ae9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8378aee, 23 },
    { 0, 0 },
    { 0x4f678af0, 33 },
    { 0, 0 },
    { 0x85e78af2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c78afc, 23 },
    { 0xa978afd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2478b03, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a378b06, 55 },
    { 0, 0 },
    { 0xef478b08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed78b0d, 34 },
    { 0, 0 },
    { 0x57a78b0f, 85 },
    { 0xa0778b0f, 61 },
    { 0x2ba78b0f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x11378b14, 33 },
    { 0, 0 },
    { 0xa9978b16, 34 },
    { 0, 0 },
    { 0xf7278b18, 99 },
    { 0, 0 },
    { 0x39578b1a, 95 },
    { 0, 0 },
    { 0x47c78b1c, 95 },
    { 0x25178b1c, 90 },
    { 0, 0 },
    { 0x8c078b1f, 84 },
    { 0xf4778b20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e378b2a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e78b2e, 99 },
    { 0xbd178b2f, 90 },
    { 0x9b978b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f78b37, 76 },
    { 0xe6878b37, 94 },
    { 0, 0 },
    { 0xa9478b3a, 95 },
    { 0x8d078b3b, 76 },
    { 0x4bc78b3b, 34 },
    { 0, 0 },
    { 0x45978b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66778b43, 84 },
    { 0, 0 },
    { 0x50e78b45, 52 },
    { 0x56f78b46, 33 },
    { 0, 0 },
    { 0xb3a78b48, 66 },
    { 0xf2b78b49, 91 },
    { 0, 0 },
    { 0x47778b4b, 23 },
    { 0x3bc78b4c, 98 },
    { 0x73e78b4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a78b50, 33 },
    { 0x47478b50, 95 },
    { 0, 0 },
    { 0xa6c78b53, 23 },
    { 0x5f678b54, 90 },
    { 0x20e78b53, 90 },
    { 0, 0 },
    { 0xd2478b57, 23 },
    { 0xf1a78b58, 98 },
    { 0xe6378b57, 98 },
    { 0xb5578b5a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f78b5f, 23 },
    { 0x61078b60, 33 },
    { 0, 0 },
    { 0xa7378b62, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd78b65, 99 },
    { 0x2e178b66, 34 },
    { 0xa8978b67, 61 },
    { 0xda778b67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c78b6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce78b71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd78b75, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7078b78, 99 },
    { 0xc8578b79, 90 },
    { 0xd9e78b7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c278b7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50378b80, 66 },
    { 0x6eb78b81, 95 },
    { 0, 0 },
    { 0x4c078b83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33178b86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56478b8a, 95 },
    { 0x86a78b8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd778b8e, 61 },
    { 0, 0 },
    { 0xef478b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b578b94, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e78b98, 55 },
    { 0xd7a78b99, 76 },
    { 0x2c678b9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e78b9d, 66 },
    { 0x71d78b9d, 61 },
    { 0xcea78b9f, 99 },
    { 0xbdd78ba0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2978ba5, 78 },
    { 0xc3278ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8078ba9, 76 },
    { 0xf2578baa, 61 },
    { 0x57778bab, 33 },
    { 0x2a78bac, 34 },
    { 0xd7a78bab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8778bb2, 78 },
    { 0x84d78bb2, 90 },
    { 0x3a978bb2, 55 },
    { 0, 0 },
    { 0x9eb78bb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1078bb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2178bbd, 90 },
    { 0x89978bbe, 61 },
    { 0xd8678bbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66678bc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec78bc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb78bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d778bcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc78bd0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad78bd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb78bdd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10878be2, 17 },
    { 0xbbb78be2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf78be6, 34 },
    { 0x11578be7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc78bec, 98 },
    { 0, 0 },
    { 0x55578bee, 87 },
    { 0xd0c78bef, 90 },
    { 0x15b78bf0, 56 },
    { 0xcbf78bf0, 43 },
    { 0xe578bf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d878bf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb78bfa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e178c09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b78c0f, 100 },
    { 0x42a78c10, 23 },
    { 0x59978c11, 78 },
    { 0, 0 },
    { 0xa9c78c13, 95 },
    { 0, 0 },
    { 0x68578c15, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x97878c18, 77 },
    { 0xe5b78c19, 90 },
    { 0, 0 },
    { 0xc5878c1b, 55 },
    { 0, 0 },
    { 0x84678c1d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa178c22, 78 },
    { 0x21978c22, 90 },
    { 0x73278c22, 91 },
    { 0x30c78c25, 90 },
    { 0x65278c25, 90 },
    { 0, 0 },
    { 0x2c478c28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3778c2b, 61 },
    { 0x48c78c2b, 98 },
    { 0x42a78c2b, 43 },
    { 0x74178c2e, 90 },
    { 0x53978c2f, 56 },
    { 0xf1d78c2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a78c33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65778c39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b78c40, 23 },
    { 0x2a178c41, 33 },
    { 0x48178c41, 95 },
    { 0x44678c43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e978c46, 90 },
    { 0, 0 },
    { 0x45a78c48, 41 },
    { 0x59a78c49, 23 },
    { 0x4eb78c49, 61 },
    { 0xaaa78c4a, 98 },
    { 0, 0 },
    { 0x4d278c4d, 52 },
    { 0, 0 },
    { 0x48d78c4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x87978c52, 84 },
    { 0x9da78c52, 76 },
    { 0xbb378c54, 33 },
    { 0x55078c54, 98 },
    { 0x3ee78c56, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5078c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b78c5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5478c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0678c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67278c6f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18778c75, 76 },
    { 0x87e78c76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57378c79, 90 },
    { 0x16178c7a, 23 },
    { 0xec778c7b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17678c7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18178c88, 81 },
    { 0, 0 },
    { 0xef278c8a, 23 },
    { 0xe0878c8b, 90 },
    { 0, 0 },
    { 0x7e678c8d, 33 },
    { 0x5b378c8e, 90 },
    { 0x6378c8e, 52 },
    { 0xf8878c8d, 23 },
    { 0x8c178c91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b78c95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97778c99, 91 },
    { 0, 0 },
    { 0x86378c9b, 23 },
    { 0x12778c9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b278c9f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc78ca9, 97 },
    { 0, 0 },
    { 0x51378cab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e78cae, 33 },
    { 0, 0 },
    { 0x4e278cb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd78cb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed178cbc, 61 },
    { 0, 0 },
    { 0x76778cbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d78cc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f678cc4, 61 },
    { 0x8ed78cc5, 95 },
    { 0x1e378cc6, 90 },
    { 0, 0 },
    { 0x10f78cc8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2178cce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98878cd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2878cda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85578cde, 46 },
    { 0x7a578cdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3778ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed778cef, 93 },
    { 0xee978cf0, 33 },
    { 0xe5878cf1, 76 },
    { 0x2b078cf2, 83 },
    { 0, 0 },
    { 0x9a278cf4, 97 },
    { 0x44d78cf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea78cfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51978cfe, 93 },
    { 0, 0 },
    { 0x81b78d00, 97 },
    { 0xe1778d01, 90 },
    { 0xa4978d02, 95 },
    { 0xe3378d03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de78d07, 70 },
    { 0x28678d07, 95 },
    { 0xca078d08, 99 },
    { 0, 0 },
    { 0x84378d0b, 90 },
    { 0x5f378d0c, 52 },
    { 0x42c78d0d, 61 },
    { 0x478d0c, 91 },
    { 0x34a78d0c, 33 },
    { 0x35578d10, 33 },
    { 0xeed78d10, 33 },
    { 0x85978d12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14978d15, 55 },
    { 0xd5478d15, 55 },
    { 0, 0 },
    { 0x22378d18, 43 },
    { 0xece78d19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f978d1c, 100 },
    { 0, 0 },
    { 0xfa878d1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd178d21, 35 },
    { 0, 0 },
    { 0xfa678d23, 100 },
    { 0xa0078d24, 68 },
    { 0x17478d25, 93 },
    { 0x41378d24, 61 },
    { 0x7e978d24, 90 },
    { 0xd2478d28, 85 },
    { 0xe8e78d28, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d78d2d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba78d32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31378d38, 33 },
    { 0, 0 },
    { 0x60578d3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6678d41, 91 },
    { 0x1ce78d42, 87 },
    { 0xaff78d42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3178d48, 88 },
    { 0xb5e78d48, 55 },
    { 0xabf78d4a, 61 },
    { 0x8578d4a, 33 },
    { 0x7dd78d4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a78d50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc78d61, 61 },
    { 0x4cd78d61, 76 },
    { 0x34678d63, 55 },
    { 0x7d178d64, 43 },
    { 0x9f978d65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c78d69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf178d6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c78d6f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64778d73, 34 },
    { 0xb1f78d74, 90 },
    { 0x3c678d73, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46178d7b, 61 },
    { 0, 0 },
    { 0xc8078d7d, 98 },
    { 0, 0 },
    { 0xcaa78d7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76778d83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a78d89, 98 },
    { 0x34a78d8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x27478d8d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23478d91, 95 },
    { 0x4f578d92, 100 },
    { 0x4f078d91, 84 },
    { 0x9dc78d94, 99 },
    { 0x4cf78d92, 33 },
    { 0, 0 },
    { 0x81b78d97, 23 },
    { 0x66c78d98, 98 },
    { 0xbd478d99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c78d9e, 23 },
    { 0x46678d9f, 61 },
    { 0xd6e78da0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b78da3, 61 },
    { 0, 0 },
    { 0x2df78da5, 88 },
    { 0, 0 },
    { 0x77b78da7, 61 },
    { 0xc7d78da8, 90 },
    { 0, 0 },
    { 0xc0c78daa, 90 },
    { 0x62b78dab, 98 },
    { 0x9c178dac, 46 },
    { 0, 0 },
    { 0x5cc78dae, 100 },
    { 0xa4578daf, 98 },
    { 0x11d78db0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62278db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc78dba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc78dcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84978dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2478dd7, 98 },
    { 0, 0 },
    { 0x56978dd9, 33 },
    { 0, 0 },
    { 0x6df78ddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1178ddf, 98 },
    { 0x54778ddf, 61 },
    { 0xbc678de1, 90 },
    { 0x2b978de2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a678de6, 98 },
    { 0x3fe78de7, 33 },
    { 0, 0 },
    { 0xba678de9, 23 },
    { 0x17878dea, 95 },
    { 0, 0 },
    { 0xf2a78dec, 76 },
    { 0, 0 },
    { 0x68f78dee, 95 },
    { 0xc3e78def, 55 },
    { 0, 0 },
    { 0xc1378df1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678df5, 93 },
    { 0x7ac78df6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b678dfa, 33 },
    { 0xa3478dfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3478e0a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd78e10, 55 },
    { 0x12978e11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb78e1a, 98 },
    { 0, 0 },
    { 0xcc78e1c, 90 },
    { 0, 0 },
    { 0x53a78e1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e78e26, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x65878e29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15478e2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b78e30, 33 },
    { 0x81b78e31, 68 },
    { 0, 0 },
    { 0x21a78e33, 61 },
    { 0x21078e34, 23 },
    { 0, 0 },
    { 0xc8078e36, 95 },
    { 0x1b978e37, 90 },
    { 0x7ea78e36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9078e3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee78e3e, 33 },
    { 0xd3678e3f, 90 },
    { 0x65f78e3e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef78e47, 76 },
    { 0xbd178e48, 17 },
    { 0x6f78e47, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x878e4c, 95 },
    { 0x51078e4d, 80 },
    { 0x91078e4e, 96 },
    { 0, 0 },
    { 0x6df78e50, 95 },
    { 0, 0 },
    { 0x31c78e52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b078e58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a78e5e, 76 },
    { 0xf3d78e5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2078e72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48978e77, 90 },
    { 0xbdf78e78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e78e7c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89378e82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8578e87, 46 },
    { 0, 0 },
    { 0x7b978e89, 36 },
    { 0, 0 },
    { 0xec78e8b, 95 },
    { 0x8df78e8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c978e8f, 64 },
    { 0x7bf78e8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44378e94, 78 },
    { 0, 0 },
    { 0x64a78e96, 33 },
    { 0x9b78e97, 100 },
    { 0x1e978e96, 61 },
    { 0, 0 },
    { 0xd3978e9a, 23 },
    { 0, 0 },
    { 0x41778e9c, 88 },
    { 0, 0 },
    { 0xf5f78e9e, 90 },
    { 0xad678e9e, 61 },
    { 0xe8f78ea0, 61 },
    { 0x8b078ea1, 90 },
    { 0x5d578ea2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xff478ea5, 46 },
    { 0x6c978ea6, 90 },
    { 0x33978ea7, 33 },
    { 0xb3278ea5, 33 },
    { 0, 0 },
    { 0xa678eaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7578eb2, 26 },
    { 0x2778eb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf078eb9, 23 },
    { 0x91378eba, 52 },
    { 0, 0 },
    { 0xf5f78ebc, 61 },
    { 0x5e178ebc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c278ec4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3878ec7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3478ed0, 98 },
    { 0x87478ed0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb78ed6, 61 },
    { 0x11178ed7, 90 },
    { 0xfa78ed7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4678ee1, 76 },
    { 0x3b678ee2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75778ee7, 84 },
    { 0x80c78ee7, 33 },
    { 0x15678ee9, 33 },
    { 0, 0 },
    { 0x7e578eeb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a78ef1, 34 },
    { 0xb7778ef1, 23 },
    { 0xb5778ef3, 61 },
    { 0x36b78ef4, 77 },
    { 0xa6a78ef5, 90 },
    { 0x97278ef6, 77 },
    { 0xac378ef4, 93 },
    { 0x2b178ef6, 46 },
    { 0x34478ef4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9878f00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf78f03, 46 },
    { 0xa0e78f04, 88 },
    { 0, 0 },
    { 0x7cc78f06, 23 },
    { 0, 0 },
    { 0x4a578f08, 33 },
    { 0x3f978f09, 61 },
    { 0, 0 },
    { 0x66b78f0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4778f0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8478f11, 99 },
    { 0xce878f12, 43 },
    { 0xe9478f11, 34 },
    { 0x41a78f11, 98 },
    { 0, 0 },
    { 0x72a78f16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f78f19, 68 },
    { 0, 0 },
    { 0xa7478f1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b78f24, 61 },
    { 0x1fe78f25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b78f29, 80 },
    { 0, 0 },
    { 0x71978f2b, 33 },
    { 0x9a378f2c, 98 },
    { 0x36678f2d, 23 },
    { 0x8da78f2e, 66 },
    { 0xf6e78f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478f33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a778f38, 55 },
    { 0x84578f39, 81 },
    { 0x51a78f3a, 81 },
    { 0xaf678f39, 75 },
    { 0xea878f3c, 99 },
    { 0xeea78f39, 34 },
    { 0xa9478f3e, 23 },
    { 0x65478f3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8678f42, 95 },
    { 0, 0 },
    { 0x2b978f44, 76 },
    { 0x85278f45, 23 },
    { 0x16b78f45, 67 },
    { 0xb2978f47, 61 },
    { 0x33e78f47, 91 },
    { 0x24578f44, 90 },
    { 0, 0 },
    { 0x68078f4b, 76 },
    { 0xfb178f4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b178f53, 23 },
    { 0x76078f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62478f58, 23 },
    { 0, 0 },
    { 0x19278f5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37778f60, 61 },
    { 0xc7678f61, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be78f65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac278f69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc578f74, 55 },
    { 0, 0 },
    { 0x8b678f76, 55 },
    { 0, 0 },
    { 0x20978f78, 90 },
    { 0x93d78f78, 52 },
    { 0x91078f7a, 23 },
    { 0x5e978f7a, 90 },
    { 0x15e78f7c, 82 },
    { 0xecc78f79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb78f83, 33 },
    { 0xc8a78f83, 33 },
    { 0, 0 },
    { 0x50f78f86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c378f8a, 76 },
    { 0x3ec78f8b, 52 },
    { 0xb5c78f8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c78f8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7478f93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba878f97, 55 },
    { 0xbc078f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b178f9c, 33 },
    { 0x46b78f9d, 77 },
    { 0x12c78f9e, 61 },
    { 0, 0 },
    { 0xeea78fa0, 93 },
    { 0x39778fa0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f78fa5, 23 },
    { 0x1cc78fa6, 34 },
    { 0, 0 },
    { 0xcc978fa8, 33 },
    { 0xae678fa8, 23 },
    { 0xe3378faa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc378fad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd78fb0, 61 },
    { 0, 0 },
    { 0xcb578fb2, 61 },
    { 0, 0 },
    { 0x8be78fb4, 61 },
    { 0, 0 },
    { 0x38478fb6, 76 },
    { 0xea678fb6, 100 },
    { 0xb3e78fb8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34678fbd, 55 },
    { 0x8f378fbe, 77 },
    { 0x3bf78fbe, 66 },
    { 0x9e078fbf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd78fd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7578fd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed78fdb, 85 },
    { 0xe9278fdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49678fe2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x90878fe5, 33 },
    { 0x9578fe5, 90 },
    { 0xcc378fe5, 95 },
    { 0, 0 },
    { 0x87478fe9, 98 },
    { 0xcf378fea, 61 },
    { 0x88278feb, 78 },
    { 0x3f478fe9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5778fef, 98 },
    { 0xd6c78fef, 23 },
    { 0, 0 },
    { 0x23f78ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65679000, 83 },
    { 0xb6779001, 17 },
    { 0x6b679002, 33 },
    { 0x6e779000, 43 },
    { 0xa2c79002, 56 },
    { 0x28779001, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0679008, 75 },
    { 0x46e79008, 98 },
    { 0, 0 },
    { 0xf27900b, 2 },
    { 0xad17900c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83879011, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e579014, 23 },
    { 0x6f479014, 93 },
    { 0xc7479015, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4007901d, 93 },
    { 0xf617901e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0879021, 34 },
    { 0, 0 },
    { 0xfc179023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379026, 90 },
    { 0xfb779027, 76 },
    { 0x64b79028, 95 },
    { 0x77279029, 43 },
    { 0xaa87902a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc79033, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7903b, 23 },
    { 0xfae7903b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af79046, 17 },
    { 0, 0 },
    { 0x3a279048, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6667904d, 93 },
    { 0x50b7904e, 43 },
    { 0x5677904f, 55 },
    { 0, 0 },
    { 0x5ae79051, 95 },
    { 0x36879052, 23 },
    { 0x85279052, 87 },
    { 0xe7f79054, 61 },
    { 0x70c79052, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf7905b, 90 },
    { 0xb057905b, 77 },
    { 0x7dc7905d, 90 },
    { 0x8097905c, 33 },
    { 0x2007905d, 23 },
    { 0x62279060, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a79069, 77 },
    { 0, 0 },
    { 0xa147906b, 90 },
    { 0, 0 },
    { 0x39f7906d, 99 },
    { 0x1ad7906d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73679073, 95 },
    { 0x10379074, 90 },
    { 0xaca79074, 87 },
    { 0x50379076, 85 },
    { 0xe2279076, 14 },
    { 0, 0 },
    { 0x66279079, 61 },
    { 0, 0 },
    { 0xf627907b, 23 },
    { 0, 0 },
    { 0xde17907d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad79081, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f779085, 76 },
    { 0x41979086, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7908a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b579091, 33 },
    { 0, 0 },
    { 0xbf779093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7879099, 76 },
    { 0, 0 },
    { 0xed37909b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830790a0, 90 },
    { 0x419790a0, 95 },
    { 0xa2b790a2, 87 },
    { 0x3da790a3, 52 },
    { 0x8af790a3, 23 },
    { 0x24790a5, 61 },
    { 0x29a790a5, 33 },
    { 0x9cb790a5, 55 },
    { 0x77d790a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588790ac, 90 },
    { 0x939790ad, 90 },
    { 0x73e790ae, 100 },
    { 0x891790af, 66 },
    { 0x547790ae, 52 },
    { 0, 0 },
    { 0x4b1790b2, 33 },
    { 0, 0 },
    { 0x560790b4, 90 },
    { 0, 0 },
    { 0x1d4790b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b790bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96790bf, 33 },
    { 0x124790c0, 90 },
    { 0, 0 },
    { 0x9a4790c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224790ca, 23 },
    { 0x8ed790cb, 98 },
    { 0x4f6790cc, 84 },
    { 0x34a790cb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969790d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3790d7, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xc790da, 33 },
    { 0xc1e790db, 95 },
    { 0x5f790dc, 95 },
    { 0, 0 },
    { 0xa8a790de, 67 },
    { 0xffc790df, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4790e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648790e8, 33 },
    { 0xc0e790e9, 33 },
    { 0x3a6790e9, 61 },
    { 0x61d790eb, 33 },
    { 0x52f790e9, 95 },
    { 0x857790ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc790f1, 66 },
    { 0x2dd790f2, 34 },
    { 0, 0 },
    { 0xdc3790f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae790f7, 23 },
    { 0, 0 },
    { 0xd53790f9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5790fc, 76 },
    { 0x889790fd, 36 },
    { 0x86c790fe, 98 },
    { 0, 0 },
    { 0xf5879100, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b79103, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d79108, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a47910d, 61 },
    { 0x29e7910e, 68 },
    { 0, 0 },
    { 0x82f79110, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f79113, 91 },
    { 0xdb79113, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b79118, 98 },
    { 0, 0 },
    { 0xb577911a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c79126, 46 },
    { 0, 0 },
    { 0x33379128, 55 },
    { 0, 0 },
    { 0x14c7912a, 91 },
    { 0xaae7912a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9079130, 95 },
    { 0x30679131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc687913a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x547913d, 88 },
    { 0xf877913d, 80 },
    { 0xe7b7913f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c679142, 98 },
    { 0x72779143, 95 },
    { 0x7c79144, 43 },
    { 0x10d79145, 90 },
    { 0x1e379145, 61 },
    { 0x5f279144, 95 },
    { 0, 0 },
    { 0x3c779149, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e7914d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c779156, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da7915a, 33 },
    { 0x14c7915b, 95 },
    { 0xb057915c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17479169, 34 },
    { 0, 0 },
    { 0x6cb7916b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6857916f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2679173, 61 },
    { 0x39179173, 46 },
    { 0, 0 },
    { 0x8ab79176, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c79179, 46 },
    { 0xa0e7917a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23679182, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df79185, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20479189, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9579190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db79193, 67 },
    { 0xae079194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33791a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d791a8, 90 },
    { 0xce6791a9, 98 },
    { 0x50d791aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1791b7, 33 },
    { 0x33a791b8, 23 },
    { 0xb2e791b7, 91 },
    { 0x2de791ba, 23 },
    { 0xf6a791bb, 61 },
    { 0x81d791bb, 91 },
    { 0x8f6791b7, 87 },
    { 0x379791be, 95 },
    { 0x171791bc, 100 },
    { 0xdd791bd, 98 },
    { 0xdc8791b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c791c6, 36 },
    { 0xf3c791c7, 46 },
    { 0x74a791c6, 43 },
    { 0, 0 },
    { 0x33e791ca, 76 },
    { 0xa5e791cb, 90 },
    { 0xca9791cc, 90 },
    { 0x706791cd, 23 },
    { 0xdb2791cd, 66 },
    { 0x1f2791cd, 32 },
    { 0x69791d0, 88 },
    { 0, 0 },
    { 0xe48791d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c791dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474791e1, 23 },
    { 0x7ee791e1, 55 },
    { 0xbcf791e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5791e6, 76 },
    { 0xbbb791e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834791f0, 76 },
    { 0, 0 },
    { 0xf1f791f2, 90 },
    { 0x345791f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11791f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00791fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed79200, 40 },
    { 0xdc879200, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e79205, 52 },
    { 0x45579206, 33 },
    { 0x73d79205, 64 },
    { 0xbac79205, 95 },
    { 0x9b379208, 61 },
    { 0, 0 },
    { 0x4c97920b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7920f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd79212, 33 },
    { 0xd9579213, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7307921a, 76 },
    { 0, 0 },
    { 0x96f7921c, 75 },
    { 0xb7e7921d, 90 },
    { 0, 0 },
    { 0x6057921f, 69 },
    { 0, 0 },
    { 0xcb979221, 91 },
    { 0x31b79222, 23 },
    { 0x7079222, 90 },
    { 0xb979224, 33 },
    { 0x9f879222, 76 },
    { 0xc7a79226, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c07922c, 64 },
    { 0x1dc7922d, 23 },
    { 0xe97922e, 33 },
    { 0x54a7922f, 43 },
    { 0xf1879230, 46 },
    { 0x9b07922e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88679234, 23 },
    { 0xdf979235, 90 },
    { 0x99979235, 23 },
    { 0, 0 },
    { 0x7b779238, 93 },
    { 0xc5e79238, 85 },
    { 0xe2279238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb79240, 81 },
    { 0x8c679241, 98 },
    { 0x4da79241, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x44479245, 95 },
    { 0x35a79246, 23 },
    { 0x26879246, 23 },
    { 0xd9879248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a7924b, 8 },
    { 0xd167924b, 23 },
    { 0x9377924d, 46 },
    { 0xd987924c, 61 },
    { 0, 0 },
    { 0x62f79250, 95 },
    { 0, 0 },
    { 0x4a279252, 90 },
    { 0, 0 },
    { 0xb6b79254, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf579258, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c7925c, 95 },
    { 0xe5e7925d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a79265, 23 },
    { 0xf6279265, 84 },
    { 0, 0 },
    { 0xf4f79268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687926f, 90 },
    { 0x7e879270, 95 },
    { 0xbba79270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d79274, 90 },
    { 0x7dd79275, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd7927a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c79283, 95 },
    { 0, 0 },
    { 0x22b79285, 90 },
    { 0x52179286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f679289, 33 },
    { 0, 0 },
    { 0xcbf7928b, 90 },
    { 0xeb7928b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6307928f, 34 },
    { 0x47928f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1947929a, 23 },
    { 0x44a7929a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd657929e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43792a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24792a9, 95 },
    { 0, 0 },
    { 0x9c9792ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0792ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1792b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5792bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc792be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5792c2, 90 },
    { 0, 0 },
    { 0x287792c4, 76 },
    { 0x244792c5, 40 },
    { 0xc8792c6, 76 },
    { 0x6ea792c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6792cc, 61 },
    { 0xb30792cd, 85 },
    { 0, 0 },
    { 0xe60792cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46792d2, 53 },
    { 0xbb9792d3, 61 },
    { 0x792792d2, 33 },
    { 0x97e792d5, 90 },
    { 0xa0c792d6, 101 },
    { 0xc2f792d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db792e3, 98 },
    { 0xfb3792e4, 61 },
    { 0, 0 },
    { 0xe2d792e6, 46 },
    { 0xf82792e6, 23 },
    { 0x4c4792e7, 95 },
    { 0xd7792e9, 33 },
    { 0x63e792ea, 23 },
    { 0x86a792eb, 33 },
    { 0xea8792ea, 76 },
    { 0x456792ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x652792f0, 43 },
    { 0x6ad792f1, 46 },
    { 0, 0 },
    { 0xb81792f3, 91 },
    { 0, 0 },
    { 0x152792f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8792fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334792ff, 91 },
    { 0, 0 },
    { 0x13979301, 68 },
    { 0xc8e79301, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff79305, 46 },
    { 0x38679306, 23 },
    { 0x67e79307, 55 },
    { 0xf5e79306, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab79311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf379315, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c7931c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9979320, 61 },
    { 0xc6779321, 23 },
    { 0xd5279321, 84 },
    { 0, 0 },
    { 0xbb179324, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1879327, 98 },
    { 0, 0 },
    { 0x40279329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7932d, 90 },
    { 0xb327932e, 43 },
    { 0xb87932d, 90 },
    { 0x71e7932d, 88 },
    { 0x5d17932e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b179334, 43 },
    { 0x52079335, 76 },
    { 0, 0 },
    { 0x83979337, 61 },
    { 0, 0 },
    { 0xd6079339, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff779343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3679346, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6977934b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe737934e, 33 },
    { 0xee07934f, 61 },
    { 0, 0 },
    { 0x30779351, 61 },
    { 0x89a79351, 33 },
    { 0xbcc79353, 84 },
    { 0xbe379353, 46 },
    { 0, 0 },
    { 0x1ac79356, 46 },
    { 0x11a79356, 61 },
    { 0x8f979358, 66 },
    { 0xd5f79359, 98 },
    { 0, 0 },
    { 0xef17935b, 43 },
    { 0x5d07935c, 23 },
    { 0x7077935d, 94 },
    { 0xcb77935e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55779362, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf147936a, 43 },
    { 0, 0 },
    { 0x7397936c, 84 },
    { 0, 0 },
    { 0x3007936e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa079376, 95 },
    { 0xaf379376, 76 },
    { 0, 0 },
    { 0xb6e79379, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb7937c, 93 },
    { 0xf257937d, 33 },
    { 0xd5d7937e, 34 },
    { 0, 0 },
    { 0x8f279380, 61 },
    { 0x62a79381, 61 },
    { 0, 0 },
    { 0xd2479383, 55 },
    { 0, 0 },
    { 0x32f79385, 81 },
    { 0xd6e79386, 78 },
    { 0xd9879385, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877938b, 33 },
    { 0xce47938c, 34 },
    { 0, 0 },
    { 0xbcc7938e, 36 },
    { 0x2507938f, 55 },
    { 0xfaa79390, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8679397, 90 },
    { 0, 0 },
    { 0x2e679399, 68 },
    { 0x7a27939a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4147939e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383793a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f793a6, 96 },
    { 0x2a2793a7, 53 },
    { 0xd63793a8, 23 },
    { 0x274793a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b793ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb793b4, 93 },
    { 0xbc4793b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x884793b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819793c4, 95 },
    { 0x988793c5, 33 },
    { 0x88f793c5, 33 },
    { 0x2cf793c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55793ca, 67 },
    { 0, 0 },
    { 0x21e793cc, 61 },
    { 0, 0 },
    { 0x59e793ce, 90 },
    { 0xdc8793cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e793d2, 61 },
    { 0, 0 },
    { 0x309793d4, 33 },
    { 0xfa7793d5, 65 },
    { 0x79a793d6, 43 },
    { 0, 0 },
    { 0xd0793d8, 52 },
    { 0x9eb793d9, 95 },
    { 0xe1e793da, 90 },
    { 0x708793d9, 95 },
    { 0xc0b793dc, 90 },
    { 0, 0 },
    { 0x256793de, 23 },
    { 0x258793df, 90 },
    { 0x901793e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2793e4, 61 },
    { 0xc62793e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e793ec, 98 },
    { 0x57d793ed, 98 },
    { 0x946793ee, 55 },
    { 0xb04793ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d793f2, 90 },
    { 0xa7a793f3, 90 },
    { 0x89a793f2, 33 },
    { 0x7a5793f5, 90 },
    { 0, 0 },
    { 0xbc0793f7, 90 },
    { 0x79a793f8, 95 },
    { 0x213793f9, 100 },
    { 0xb1b793f7, 52 },
    { 0xf1f793fb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1793fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf679401, 90 },
    { 0, 0 },
    { 0xe5679403, 23 },
    { 0x4779404, 98 },
    { 0, 0 },
    { 0x67b79406, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f17940d, 68 },
    { 0, 0 },
    { 0x9a77940f, 90 },
    { 0x88e79410, 90 },
    { 0x23d79411, 99 },
    { 0, 0 },
    { 0xe0a79413, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe79418, 93 },
    { 0x76079418, 55 },
    { 0, 0 },
    { 0x1b17941b, 90 },
    { 0x9f77941c, 61 },
    { 0x9ce7941c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd679420, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce279423, 92 },
    { 0, 0 },
    { 0x47279425, 52 },
    { 0x8c79426, 61 },
    { 0x4d979426, 61 },
    { 0xea479426, 68 },
    { 0x2479428, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc727942c, 95 },
    { 0, 0 },
    { 0xf0b7942e, 90 },
    { 0xd147942f, 98 },
    { 0xc3879430, 68 },
    { 0x33279430, 85 },
    { 0xad879432, 46 },
    { 0x53e79432, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d379437, 78 },
    { 0xe0779438, 23 },
    { 0xeee79437, 87 },
    { 0, 0 },
    { 0xf2b7943b, 38 },
    { 0x9617943c, 33 },
    { 0xb87943c, 46 },
    { 0x7437943d, 76 },
    { 0xcbc7943b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0f79442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30279446, 90 },
    { 0x47979446, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2917944e, 90 },
    { 0x1db7944f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c979454, 43 },
    { 0, 0 },
    { 0xd0579456, 17 },
    { 0xc0f79457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf7945b, 55 },
    { 0x2ce7945c, 69 },
    { 0x28c7945d, 90 },
    { 0xf47945d, 85 },
    { 0xf4c7945f, 23 },
    { 0xceb7945e, 78 },
    { 0x34d79461, 95 },
    { 0x3347945d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4379465, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e879468, 90 },
    { 0, 0 },
    { 0xad27946a, 61 },
    { 0xfaa7946a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b7946f, 33 },
    { 0xe2379470, 34 },
    { 0x8779471, 95 },
    { 0xc7179472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce579475, 61 },
    { 0, 0 },
    { 0x23c79477, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb857947a, 34 },
    { 0x5bd7947b, 93 },
    { 0, 0 },
    { 0x5447947d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed79484, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597948b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff79491, 69 },
    { 0, 0 },
    { 0xab179493, 47 },
    { 0xe9679494, 52 },
    { 0x57b79494, 90 },
    { 0xefb79496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7949a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b794a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904794a6, 76 },
    { 0, 0 },
    { 0x4a5794a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674794ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640794b7, 33 },
    { 0x842794b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4794bb, 23 },
    { 0x74f794bc, 75 },
    { 0x278794bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712794c7, 76 },
    { 0x1d7794c7, 98 },
    { 0, 0 },
    { 0xf72794ca, 90 },
    { 0xe21794ca, 90 },
    { 0x9bd794cc, 33 },
    { 0, 0 },
    { 0x879794ce, 66 },
    { 0xbac794ce, 33 },
    { 0, 0 },
    { 0xa44794d1, 78 },
    { 0xff9794d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2794d7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650794db, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db794e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x733794e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2794e6, 95 },
    { 0x62d794e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab794ee, 46 },
    { 0x89e794ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4794f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec794fc, 98 },
    { 0, 0 },
    { 0xc36794fe, 90 },
    { 0, 0 },
    { 0xe6e79500, 95 },
    { 0xa2e79501, 33 },
    { 0, 0 },
    { 0xb2279503, 91 },
    { 0x8dd79503, 90 },
    { 0x96279503, 67 },
    { 0x2ac79506, 87 },
    { 0x47579507, 90 },
    { 0xae179507, 58 },
    { 0x8af79509, 90 },
    { 0, 0 },
    { 0x9a27950b, 33 },
    { 0, 0 },
    { 0xadf7950d, 93 },
    { 0, 0 },
    { 0xf87950f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2879519, 61 },
    { 0x88979519, 98 },
    { 0x6517951a, 78 },
    { 0x3157951c, 23 },
    { 0, 0 },
    { 0x5567951e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4479523, 76 },
    { 0, 0 },
    { 0x86279525, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3597952b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c579531, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae79534, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b479537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87953a, 33 },
    { 0x40d7953b, 23 },
    { 0, 0 },
    { 0xcfe7953d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab79541, 33 },
    { 0xe9d79542, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d679547, 61 },
    { 0xb4079548, 23 },
    { 0x28479548, 76 },
    { 0x62d7954a, 98 },
    { 0, 0 },
    { 0x6137954c, 68 },
    { 0x9f87954c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef79552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c579556, 83 },
    { 0x5d979557, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e7955a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df7955d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb079561, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a79566, 53 },
    { 0x6f379566, 43 },
    { 0x88f79567, 90 },
    { 0x83e79569, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3879573, 76 },
    { 0x5b779573, 65 },
    { 0x5e879574, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e379578, 96 },
    { 0, 0 },
    { 0xb527957a, 43 },
    { 0xe5c7957b, 68 },
    { 0x16a7957c, 90 },
    { 0x207957c, 43 },
    { 0, 0 },
    { 0xe4f7957f, 95 },
    { 0xd5279580, 23 },
    { 0x52779581, 61 },
    { 0, 0 },
    { 0xc6a79583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63879588, 23 },
    { 0, 0 },
    { 0xad97958a, 61 },
    { 0, 0 },
    { 0x1717958c, 43 },
    { 0, 0 },
    { 0x5d57958e, 36 },
    { 0x3db7958f, 34 },
    { 0, 0 },
    { 0x6d779591, 59 },
    { 0, 0 },
    { 0x9f779593, 90 },
    { 0, 0 },
    { 0xa7779595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9287959d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4795a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158795a5, 34 },
    { 0x4a0795a6, 93 },
    { 0x23f795a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58795ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba795b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd795b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e795bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322795c7, 61 },
    { 0x59f795c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73795cd, 46 },
    { 0, 0 },
    { 0x60d795cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415795d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf795d7, 45 },
    { 0xbe6795d8, 33 },
    { 0, 0 },
    { 0x4ee795da, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab795e2, 55 },
    { 0, 0 },
    { 0xfa4795e4, 55 },
    { 0, 0 },
    { 0xbcc795e6, 95 },
    { 0x5dd795e7, 23 },
    { 0x10b795e8, 90 },
    { 0xc51795e9, 76 },
    { 0x454795e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x927795ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd795f0, 46 },
    { 0x708795f1, 61 },
    { 0x973795f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7795f8, 56 },
    { 0, 0 },
    { 0xdd7795fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136795fe, 78 },
    { 0, 0 },
    { 0x26379600, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f479603, 77 },
    { 0xe2879603, 23 },
    { 0x1f079605, 17 },
    { 0x5bb79606, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd7960c, 47 },
    { 0x1507960d, 23 },
    { 0, 0 },
    { 0x4f57960f, 61 },
    { 0x6ee79610, 98 },
    { 0x89279611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf79614, 33 },
    { 0xf1379614, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d79618, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb077961b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99079621, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be79628, 99 },
    { 0x84779628, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5379632, 46 },
    { 0, 0 },
    { 0x20879634, 97 },
    { 0x51479635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc7963c, 76 },
    { 0x6937963d, 76 },
    { 0x99d7963c, 61 },
    { 0xdfa7963f, 85 },
    { 0x5d7963f, 90 },
    { 0x59379640, 55 },
    { 0x58a79642, 67 },
    { 0, 0 },
    { 0xc7a79644, 61 },
    { 0x2b279645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b7964a, 91 },
    { 0, 0 },
    { 0x3007964c, 76 },
    { 0x4297964d, 93 },
    { 0xfc07964e, 76 },
    { 0xeb07964f, 91 },
    { 0x1eb7964e, 33 },
    { 0, 0 },
    { 0x24c79652, 61 },
    { 0, 0 },
    { 0x3a779654, 33 },
    { 0x53579655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47965f, 61 },
    { 0, 0 },
    { 0xe079661, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x44279664, 33 },
    { 0x6879665, 43 },
    { 0x5e479666, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x49279669, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c27966e, 90 },
    { 0, 0 },
    { 0xd7f79670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5479673, 98 },
    { 0, 0 },
    { 0x5cb79675, 34 },
    { 0x2079676, 38 },
    { 0xe0579675, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7967d, 95 },
    { 0x2c47967e, 61 },
    { 0x7947967d, 23 },
    { 0xd3679680, 34 },
    { 0x77079680, 61 },
    { 0xac179682, 67 },
    { 0xfbb79683, 90 },
    { 0x79479684, 23 },
    { 0x1ce79684, 61 },
    { 0x96679684, 84 },
    { 0x11879687, 23 },
    { 0x52679682, 95 },
    { 0x13d79689, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c79691, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef79696, 87 },
    { 0x96d79696, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734796a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f796a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae796ae, 33 },
    { 0xc9e796af, 33 },
    { 0x1e2796b0, 90 },
    { 0xcc2796b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20796b6, 95 },
    { 0xa796b7, 61 },
    { 0xc76796b8, 61 },
    { 0x903796b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c796c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778796c7, 77 },
    { 0xe12796c8, 61 },
    { 0xa3d796c9, 90 },
    { 0xd96796c8, 93 },
    { 0x54d796cb, 40 },
    { 0x1f0796cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x904796cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f796d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82796d6, 17 },
    { 0xd0f796d7, 33 },
    { 0, 0 },
    { 0x1dd796d9, 33 },
    { 0xc18796da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90796e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277796e5, 77 },
    { 0x85d796e6, 90 },
    { 0xbc6796e7, 17 },
    { 0x9a0796e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b796ec, 67 },
    { 0, 0 },
    { 0x51d796ee, 95 },
    { 0, 0 },
    { 0x410796f0, 33 },
    { 0x510796f1, 23 },
    { 0, 0 },
    { 0x9e0796f3, 100 },
    { 0, 0 },
    { 0xb22796f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22796f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb796ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d79705, 61 },
    { 0, 0 },
    { 0xf7179707, 52 },
    { 0x64179708, 98 },
    { 0, 0 },
    { 0xb737970a, 67 },
    { 0x3397970a, 93 },
    { 0xb357970b, 90 },
    { 0xcb97970d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e879713, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56379718, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x917971b, 55 },
    { 0xbac7971c, 61 },
    { 0, 0 },
    { 0xdc27971e, 93 },
    { 0xb7f7971e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6479723, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4879726, 61 },
    { 0, 0 },
    { 0x74d79728, 61 },
    { 0, 0 },
    { 0xd4c7972a, 33 },
    { 0xc1c7972b, 33 },
    { 0x46a7972c, 76 },
    { 0, 0 },
    { 0xe17972e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d79732, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd79735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd79739, 95 },
    { 0, 0 },
    { 0x6d37973b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd987973e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c79743, 90 },
    { 0, 0 },
    { 0x3479745, 98 },
    { 0x5ed79746, 93 },
    { 0, 0 },
    { 0x38979748, 46 },
    { 0x2b379748, 90 },
    { 0x71079748, 95 },
    { 0, 0 },
    { 0x8d47974c, 61 },
    { 0x4e67974c, 52 },
    { 0xcaf7974e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b79751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24779754, 23 },
    { 0xec379754, 95 },
    { 0xccb79755, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd7975b, 98 },
    { 0, 0 },
    { 0x1357975d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47779761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f79765, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a279768, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb7976b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e79778, 68 },
    { 0xe3579778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6357977f, 90 },
    { 0xcab79780, 52 },
    { 0, 0 },
    { 0xfdb79782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8779785, 90 },
    { 0x80679786, 95 },
    { 0x12a79786, 90 },
    { 0x2aa79788, 33 },
    { 0xc0e79789, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x47978c, 91 },
    { 0xa7978d, 98 },
    { 0x5767978e, 17 },
    { 0xa1b7978f, 61 },
    { 0x67d7978c, 33 },
    { 0x4be7978d, 90 },
    { 0xdd17978f, 90 },
    { 0x97f79793, 95 },
    { 0xa0079794, 23 },
    { 0xbda79795, 85 },
    { 0x8c679795, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f979799, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56797a0, 93 },
    { 0xd9d797a0, 81 },
    { 0x6db797a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7797aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1797ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435797b7, 43 },
    { 0x549797b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8797c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2797cc, 95 },
    { 0xa92797cd, 77 },
    { 0x388797ce, 90 },
    { 0x8cd797cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x797797d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e797d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf797e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace797ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac797ef, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e797f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee797fb, 84 },
    { 0xc42797fc, 33 },
    { 0x8c2797fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79279802, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c079807, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f7980c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d979814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf179818, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae7981b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8379823, 17 },
    { 0xd6c79823, 33 },
    { 0x5f079824, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f7982c, 90 },
    { 0x11e7982c, 46 },
    { 0, 0 },
    { 0x17e7982f, 90 },
    { 0xb8979830, 68 },
    { 0x2e679830, 96 },
    { 0x9b79832, 77 },
    { 0xeeb79832, 95 },
    { 0x6e579834, 34 },
    { 0, 0 },
    { 0x6179836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5307983b, 55 },
    { 0xc767983c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e379841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22879847, 49 },
    { 0x10779848, 98 },
    { 0xbfc79849, 76 },
    { 0x95c7984a, 95 },
    { 0xb307984a, 43 },
    { 0x5357984b, 46 },
    { 0x5c97984a, 61 },
    { 0x2487984e, 33 },
    { 0, 0 },
    { 0x12c79850, 95 },
    { 0xe3b79851, 61 },
    { 0xc3579852, 95 },
    { 0, 0 },
    { 0x7ee79854, 90 },
    { 0xd3679855, 33 },
    { 0xda179856, 61 },
    { 0, 0 },
    { 0x55979858, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5627985d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1479860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd679866, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97179872, 61 },
    { 0, 0 },
    { 0x9d179874, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a79878, 98 },
    { 0x27479879, 61 },
    { 0x5417987a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8079884, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa79887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf857988a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa417988f, 90 },
    { 0, 0 },
    { 0x41f79891, 98 },
    { 0x7b879891, 98 },
    { 0x68379891, 55 },
    { 0xacb79894, 23 },
    { 0xf6d79891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd487989b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a7989f, 23 },
    { 0, 0 },
    { 0xd0798a1, 98 },
    { 0, 0 },
    { 0x809798a3, 33 },
    { 0xde798a4, 52 },
    { 0x7d798a5, 23 },
    { 0x97798a4, 33 },
    { 0xb25798a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7798aa, 95 },
    { 0x746798ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b798ae, 90 },
    { 0xc59798ae, 52 },
    { 0xb64798b0, 33 },
    { 0xd91798b1, 90 },
    { 0xe28798b0, 23 },
    { 0x211798b3, 61 },
    { 0x9a2798b4, 81 },
    { 0x9a5798b5, 43 },
    { 0xbe7798b4, 33 },
    { 0x941798b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500798bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19798c0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a798c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x277798c9, 95 },
    { 0, 0 },
    { 0x793798cb, 33 },
    { 0, 0 },
    { 0x18a798cd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x778798d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x375798d3, 90 },
    { 0x715798d4, 46 },
    { 0xc02798d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee798da, 61 },
    { 0xbba798db, 23 },
    { 0xa5b798da, 66 },
    { 0, 0 },
    { 0x296798de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c798e1, 33 },
    { 0x705798e1, 88 },
    { 0, 0 },
    { 0x8e0798e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c798e9, 89 },
    { 0x2f6798ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b798ee, 69 },
    { 0x6a2798ef, 93 },
    { 0x842798ee, 61 },
    { 0, 0 },
    { 0x35d798f2, 76 },
    { 0xfb1798f2, 98 },
    { 0x1cc798f4, 93 },
    { 0, 0 },
    { 0x8de798f6, 90 },
    { 0, 0 },
    { 0x70a798f8, 17 },
    { 0xeb7798f9, 66 },
    { 0xf95798fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49798ff, 98 },
    { 0x68579900, 98 },
    { 0xc2c79901, 90 },
    { 0xd8979902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b179906, 49 },
    { 0xa579907, 77 },
    { 0x92579907, 90 },
    { 0, 0 },
    { 0xfa7990a, 95 },
    { 0xc8d7990b, 55 },
    { 0x62d7990c, 85 },
    { 0, 0 },
    { 0x9ec7990e, 33 },
    { 0xc607990e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a079913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1379919, 67 },
    { 0, 0 },
    { 0xbd07991b, 87 },
    { 0xa687991c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7991f, 69 },
    { 0x3af79920, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4079923, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db79926, 23 },
    { 0xd1379926, 61 },
    { 0x55a79928, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f7992b, 33 },
    { 0x5937992b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f79930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb79936, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3379939, 55 },
    { 0x1387993a, 98 },
    { 0, 0 },
    { 0x85c7993c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe379948, 23 },
    { 0xb2579948, 61 },
    { 0, 0 },
    { 0x9f27994b, 33 },
    { 0, 0 },
    { 0xcbc7994d, 23 },
    { 0, 0 },
    { 0x26f7994f, 33 },
    { 0, 0 },
    { 0x1fb79951, 76 },
    { 0xb4979952, 23 },
    { 0, 0 },
    { 0xa2679954, 91 },
    { 0xe5579955, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc79959, 33 },
    { 0x91a7995a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe227995d, 23 },
    { 0, 0 },
    { 0xe347995f, 90 },
    { 0xba57995f, 46 },
    { 0xec979961, 33 },
    { 0x43779962, 55 },
    { 0, 0 },
    { 0x3279964, 91 },
    { 0x23979965, 66 },
    { 0, 0 },
    { 0xf8179967, 85 },
    { 0x5db79968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3347996e, 90 },
    { 0, 0 },
    { 0x6e079970, 95 },
    { 0xb3979970, 23 },
    { 0x58c79972, 93 },
    { 0x7d579972, 33 },
    { 0x57b79974, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e79978, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7997d, 61 },
    { 0x1d17997d, 90 },
    { 0xb837997f, 90 },
    { 0x36479980, 55 },
    { 0xb0579980, 23 },
    { 0, 0 },
    { 0x4a679983, 90 },
    { 0xc9279984, 23 },
    { 0xd679984, 67 },
    { 0xcc079986, 23 },
    { 0x6fb79984, 46 },
    { 0x43679985, 98 },
    { 0x8ef79984, 33 },
    { 0x81579989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3397998e, 33 },
    { 0x217998f, 46 },
    { 0x3e879990, 91 },
    { 0, 0 },
    { 0x9ac79992, 100 },
    { 0, 0 },
    { 0xaac79994, 90 },
    { 0x43c79995, 34 },
    { 0, 0 },
    { 0x9d579997, 90 },
    { 0x6d79997, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xff27999b, 61 },
    { 0, 0 },
    { 0xc7d7999d, 52 },
    { 0x3ef7999e, 98 },
    { 0x1067999e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae799a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0799a8, 99 },
    { 0, 0 },
    { 0xb82799aa, 90 },
    { 0, 0 },
    { 0x7ab799ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c799af, 93 },
    { 0, 0 },
    { 0xe1f799b1, 61 },
    { 0x8c4799b2, 55 },
    { 0x88a799b3, 90 },
    { 0x91c799b1, 23 },
    { 0xe31799b4, 33 },
    { 0xaff799b6, 23 },
    { 0x51b799b7, 55 },
    { 0x4f6799b6, 61 },
    { 0xf99799b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd799c0, 34 },
    { 0xd74799c1, 61 },
    { 0x7a7799c1, 33 },
    { 0xb48799c3, 98 },
    { 0x280799c2, 76 },
    { 0x33c799c5, 98 },
    { 0x5eb799c6, 90 },
    { 0x52799c7, 78 },
    { 0x876799c8, 95 },
    { 0x50d799c9, 40 },
    { 0x659799c0, 97 },
    { 0x15a799cb, 99 },
    { 0x3fe799cc, 52 },
    { 0x8b6799cd, 95 },
    { 0x285799ce, 28 },
    { 0x843799cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3799d3, 33 },
    { 0x6fb799d4, 23 },
    { 0x814799d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15799da, 76 },
    { 0, 0 },
    { 0x3f1799dc, 61 },
    { 0x45e799dc, 77 },
    { 0xcba799dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e799e6, 55 },
    { 0x819799e7, 67 },
    { 0x1cd799e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a799ed, 33 },
    { 0, 0 },
    { 0xd16799ef, 90 },
    { 0x99799f0, 33 },
    { 0x1f5799f1, 98 },
    { 0, 0 },
    { 0x2f0799f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e799f6, 33 },
    { 0, 0 },
    { 0xb7e799f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42799fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a79a02, 33 },
    { 0xb3c79a03, 23 },
    { 0x4eb79a04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53279a0c, 33 },
    { 0, 0 },
    { 0x79179a0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc479a11, 98 },
    { 0x63d79a12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c79a15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b679a1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9579a1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6079a21, 61 },
    { 0, 0 },
    { 0xb1d79a23, 90 },
    { 0xdf179a23, 61 },
    { 0, 0 },
    { 0xe3c79a26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c579a2f, 85 },
    { 0x78779a30, 76 },
    { 0x40979a31, 90 },
    { 0x46479a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46679a35, 17 },
    { 0, 0 },
    { 0x6b179a37, 90 },
    { 0, 0 },
    { 0x53d79a39, 78 },
    { 0, 0 },
    { 0xf1479a3b, 93 },
    { 0, 0 },
    { 0x8b479a3d, 52 },
    { 0x1fb79a3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x89479a41, 55 },
    { 0xe2379a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51179a46, 55 },
    { 0, 0 },
    { 0xb5d79a48, 61 },
    { 0x3dc79a48, 65 },
    { 0, 0 },
    { 0xa1379a4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61779a51, 75 },
    { 0xadb79a52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a79a55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b579a5b, 34 },
    { 0x56079a5c, 55 },
    { 0, 0 },
    { 0x2fe79a5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e979a66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b79a69, 90 },
    { 0x65579a6a, 78 },
    { 0x81079a69, 84 },
    { 0x2d579a6b, 95 },
    { 0xfea79a6d, 93 },
    { 0x86a79a6d, 33 },
    { 0x16679a6d, 98 },
    { 0x92179a6d, 69 },
    { 0xdb379a6d, 95 },
    { 0xb1879a72, 98 },
    { 0, 0 },
    { 0x93079a74, 85 },
    { 0xe1b79a74, 61 },
    { 0, 0 },
    { 0x14879a77, 23 },
    { 0xae279a78, 23 },
    { 0xe2279a79, 98 },
    { 0x79579a7a, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e79a7e, 43 },
    { 0, 0 },
    { 0xc1179a80, 23 },
    { 0x41979a81, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac79a86, 52 },
    { 0x99a79a87, 23 },
    { 0xfb279a88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d79a8c, 34 },
    { 0x94079a8c, 33 },
    { 0x1b879a8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93479a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6479a98, 55 },
    { 0x3e579a99, 61 },
    { 0x4ea79a99, 76 },
    { 0xe8079a9b, 43 },
    { 0xcc979a9b, 23 },
    { 0xfeb79a9c, 91 },
    { 0x14479a9c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d79aa6, 76 },
    { 0xd4679aa6, 33 },
    { 0xea379aa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae179aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6579aaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb79ab2, 61 },
    { 0x60679ab3, 88 },
    { 0xe8a79ab4, 61 },
    { 0x14079ab3, 61 },
    { 0, 0 },
    { 0xa7579ab7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc479aba, 55 },
    { 0x30f79abb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4779abf, 91 },
    { 0, 0 },
    { 0x78c79ac1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6679ac4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f79ac8, 90 },
    { 0x88679ac9, 90 },
    { 0xb5179aca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6379acd, 23 },
    { 0xfde79acd, 90 },
    { 0x17f79acd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd479ad2, 43 },
    { 0x94179ad3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d679ad9, 55 },
    { 0xe4079ad9, 77 },
    { 0, 0 },
    { 0xd3479adc, 92 },
    { 0x56079add, 23 },
    { 0xd3d79ade, 95 },
    { 0, 0 },
    { 0x14479ae0, 33 },
    { 0x6d579ae0, 95 },
    { 0x26879ae2, 95 },
    { 0xc8079ae3, 65 },
    { 0xfe879ae4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a79aeb, 55 },
    { 0x94979aec, 33 },
    { 0x1a179aed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71679af8, 95 },
    { 0x13d79af9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af79afc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc79aff, 88 },
    { 0x4de79aff, 90 },
    { 0, 0 },
    { 0x2d079b02, 61 },
    { 0, 0 },
    { 0x3ba79b04, 76 },
    { 0x5d879b04, 65 },
    { 0, 0 },
    { 0xd1179b07, 85 },
    { 0xc2a79b07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa879b0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79379b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c79b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f79b1e, 91 },
    { 0x6ac79b1f, 55 },
    { 0xc7279b1e, 33 },
    { 0x3c179b20, 55 },
    { 0xdd779b20, 100 },
    { 0x82779b23, 34 },
    { 0, 0 },
    { 0x7cc79b25, 95 },
    { 0x4e679b26, 36 },
    { 0, 0 },
    { 0xe5079b28, 85 },
    { 0, 0 },
    { 0x93e79b2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a979b2d, 61 },
    { 0x57279b2d, 61 },
    { 0x29979b2f, 61 },
    { 0x56e79b30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3179b33, 95 },
    { 0xc6979b34, 98 },
    { 0x9d279b35, 53 },
    { 0, 0 },
    { 0x27479b37, 98 },
    { 0, 0 },
    { 0xc8d79b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2879b3c, 90 },
    { 0x82379b3d, 61 },
    { 0x6479b3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11779b41, 47 },
    { 0xa1579b42, 90 },
    { 0x59979b43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8079b46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e679b4a, 61 },
    { 0, 0 },
    { 0x12379b4c, 40 },
    { 0x65279b4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e79b53, 23 },
    { 0, 0 },
    { 0xacc79b55, 61 },
    { 0x6f879b56, 90 },
    { 0x8df79b56, 85 },
    { 0xd6b79b58, 90 },
    { 0x99279b59, 61 },
    { 0x57f79b57, 67 },
    { 0x6279b5b, 23 },
    { 0, 0 },
    { 0x9b879b5d, 69 },
    { 0, 0 },
    { 0x76879b5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d079b65, 87 },
    { 0x5c979b66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb79b69, 17 },
    { 0x14579b6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58779b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1879b7b, 95 },
    { 0, 0 },
    { 0xd0679b7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da79b80, 66 },
    { 0xb1779b81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91579b85, 68 },
    { 0xa9c79b86, 61 },
    { 0xa0979b87, 95 },
    { 0x6c179b88, 33 },
    { 0x39f79b88, 95 },
    { 0x64e79b88, 55 },
    { 0, 0 },
    { 0x1779b8c, 76 },
    { 0x18679b8d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6779b91, 90 },
    { 0xf2a79b92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c79b97, 23 },
    { 0x3c479b98, 95 },
    { 0x71b79b99, 81 },
    { 0x26679b99, 90 },
    { 0x2b579b9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1679b9e, 85 },
    { 0x91179b9f, 84 },
    { 0x8679ba0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c179ba5, 75 },
    { 0x65479ba5, 90 },
    { 0x8a079ba7, 81 },
    { 0x5b879ba8, 46 },
    { 0x7379ba6, 67 },
    { 0x57c79ba6, 23 },
    { 0, 0 },
    { 0x3ff79bac, 34 },
    { 0xeb979bac, 80 },
    { 0x6a579bae, 90 },
    { 0xdc79baf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb79bb2, 61 },
    { 0, 0 },
    { 0xb9c79bb4, 61 },
    { 0xd4079bb5, 61 },
    { 0x5a479bb6, 33 },
    { 0x3b379bb7, 91 },
    { 0x1b279bb4, 90 },
    { 0x83a79bb9, 93 },
    { 0xbd979bba, 78 },
    { 0xd8379bbb, 98 },
    { 0xa5979bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c79bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f79bc3, 86 },
    { 0x61879bc4, 46 },
    { 0x1b979bc4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e79bcc, 55 },
    { 0x53079bcd, 78 },
    { 0x64379bcd, 52 },
    { 0xc2279bcd, 98 },
    { 0x7e679bd0, 95 },
    { 0, 0 },
    { 0xd9579bd2, 90 },
    { 0, 0 },
    { 0x97b79bd4, 95 },
    { 0, 0 },
    { 0x3479bd6, 61 },
    { 0x9d979bd6, 90 },
    { 0, 0 },
    { 0x6dd79bd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b79bdc, 99 },
    { 0x97379bdd, 91 },
    { 0xab79bdd, 46 },
    { 0x34079bde, 48 },
    { 0, 0 },
    { 0xcf879be1, 55 },
    { 0x54879be2, 33 },
    { 0xdea79be1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79079be7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db79bea, 68 },
    { 0x9ca79beb, 95 },
    { 0x8d79bea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf079bf0, 69 },
    { 0x94579bf0, 90 },
    { 0, 0 },
    { 0xc79bf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39679bf7, 93 },
    { 0x71979bf8, 95 },
    { 0x32579bf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9579bfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84279c00, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4279c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d79c07, 43 },
    { 0x7c879c08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c979c10, 23 },
    { 0x97679c11, 33 },
    { 0x95479c12, 52 },
    { 0x179c10, 34 },
    { 0xf2279c14, 66 },
    { 0x4b379c15, 55 },
    { 0x56279c10, 95 },
    { 0xde279c17, 33 },
    { 0xaff79c14, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57079c1d, 80 },
    { 0x7c379c1e, 90 },
    { 0xf9a79c1f, 91 },
    { 0x6b079c20, 93 },
    { 0x92879c21, 33 },
    { 0xb6779c1d, 34 },
    { 0xc8f79c21, 36 },
    { 0xf9f79c22, 61 },
    { 0, 0 },
    { 0x5f279c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b79c2d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb79c30, 93 },
    { 0xcfd79c31, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd79c36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c79c39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e579c45, 61 },
    { 0xaaf79c46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc079c4b, 98 },
    { 0xc5679c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab79c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a79c5b, 90 },
    { 0xc179c5c, 33 },
    { 0x12979c5d, 76 },
    { 0, 0 },
    { 0x74b79c5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39479c63, 61 },
    { 0xb6679c64, 98 },
    { 0x40379c63, 98 },
    { 0x89579c65, 81 },
    { 0, 0 },
    { 0x69c79c68, 93 },
    { 0xb8279c69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74779c6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41579c71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a079c75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c079c7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7079c7d, 55 },
    { 0x1c279c7e, 90 },
    { 0x84079c7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b779c82, 61 },
    { 0xfe879c83, 33 },
    { 0xa3079c84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90679c8f, 95 },
    { 0x93079c90, 46 },
    { 0xe8579c90, 98 },
    { 0xe0479c92, 33 },
    { 0x30079c93, 43 },
    { 0x60679c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca979c9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba79c9e, 90 },
    { 0xc4a79c9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec79ca8, 36 },
    { 0, 0 },
    { 0x59c79caa, 66 },
    { 0xaee79cab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e479caf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb79cb2, 76 },
    { 0xa8b79cb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51079cba, 90 },
    { 0, 0 },
    { 0x3d079cbc, 23 },
    { 0x32879cbd, 85 },
    { 0xc8979cbe, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab79cc8, 61 },
    { 0, 0 },
    { 0x5d79cca, 33 },
    { 0x7b979ccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d79cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69079cd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e79cd6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19679cdb, 34 },
    { 0, 0 },
    { 0xdf979cdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d79ce5, 95 },
    { 0xb3979ce5, 98 },
    { 0x73579ce7, 33 },
    { 0x5f579ce8, 98 },
    { 0xeea79ce9, 61 },
    { 0xaec79cea, 78 },
    { 0, 0 },
    { 0xd2779cec, 93 },
    { 0, 0 },
    { 0x20479cee, 90 },
    { 0x9c979cef, 33 },
    { 0xa0b79cf0, 98 },
    { 0x9a779cf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb179cf6, 81 },
    { 0x67f79cf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca79cfa, 90 },
    { 0xd2579cfb, 61 },
    { 0, 0 },
    { 0x5fc79cfd, 23 },
    { 0, 0 },
    { 0x83379cff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95279d03, 78 },
    { 0x67d79d04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3479d0b, 98 },
    { 0, 0 },
    { 0x48879d0d, 36 },
    { 0xaeb79d0d, 36 },
    { 0x1e479d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf79d12, 64 },
    { 0x14379d12, 46 },
    { 0x26479d13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38179d18, 23 },
    { 0xe4079d19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad79d1d, 95 },
    { 0, 0 },
    { 0x2fd79d1f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26079d24, 93 },
    { 0xf3279d25, 61 },
    { 0x60679d24, 98 },
    { 0xf9579d25, 93 },
    { 0x4e979d26, 80 },
    { 0, 0 },
    { 0x48979d2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df79d2d, 95 },
    { 0xf6f79d2e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a79d3c, 98 },
    { 0xdd879d3d, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14279d41, 90 },
    { 0, 0 },
    { 0x37279d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76079d47, 55 },
    { 0x89279d48, 90 },
    { 0x3cb79d49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a679d4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57579d4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd379d53, 84 },
    { 0xed279d54, 17 },
    { 0xfd79d55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b79d58, 23 },
    { 0x8c479d59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46579d60, 95 },
    { 0xf6a79d60, 90 },
    { 0, 0 },
    { 0x1379d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1279d69, 43 },
    { 0xce579d6a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8679d6d, 98 },
    { 0, 0 },
    { 0xc2279d6f, 98 },
    { 0x50f79d70, 33 },
    { 0x8a579d70, 100 },
    { 0x30879d70, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e79d77, 61 },
    { 0xa1979d78, 33 },
    { 0, 0 },
    { 0xc5779d7a, 33 },
    { 0xe3779d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da79d8c, 61 },
    { 0xd4c79d8d, 23 },
    { 0xfa79d8c, 52 },
    { 0, 0 },
    { 0x99879d90, 88 },
    { 0, 0 },
    { 0xff379d92, 61 },
    { 0x42f79d93, 77 },
    { 0x29779d93, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1979d97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb79d9d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80679da1, 100 },
    { 0, 0 },
    { 0xefb79da3, 33 },
    { 0, 0 },
    { 0xa6579da5, 61 },
    { 0xfd79da5, 84 },
    { 0x21e79da7, 93 },
    { 0, 0 },
    { 0x90079da9, 95 },
    { 0x20179daa, 34 },
    { 0x1eb79da9, 61 },
    { 0, 0 },
    { 0x54f79dad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x26579db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33179db3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb79dbe, 99 },
    { 0, 0 },
    { 0x1d779dc0, 55 },
    { 0, 0 },
    { 0x7a779dc2, 85 },
    { 0, 0 },
    { 0xa1479dc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f79dc7, 55 },
    { 0x56779dc8, 95 },
    { 0x98579dc9, 77 },
    { 0x5aa79dc7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x35379dcd, 43 },
    { 0x59079dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b579dd2, 33 },
    { 0xe5679dd3, 34 },
    { 0, 0 },
    { 0xe2c79dd5, 91 },
    { 0x23879dd6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e79dd9, 46 },
    { 0x12179dda, 23 },
    { 0x5d679ddb, 61 },
    { 0x8a279ddb, 90 },
    { 0, 0 },
    { 0x29679dde, 23 },
    { 0xb6779dde, 61 },
    { 0x5b779de0, 34 },
    { 0xd9179de0, 23 },
    { 0x8fb79dde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c79de5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee79dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f79def, 93 },
    { 0x3f179df0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2079df7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada79dfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd79e00, 87 },
    { 0, 0 },
    { 0xc5479e02, 91 },
    { 0, 0 },
    { 0x77379e04, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4579e07, 87 },
    { 0, 0 },
    { 0xd1a79e09, 93 },
    { 0, 0 },
    { 0xe879e0b, 76 },
    { 0x28e79e0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4779e0f, 17 },
    { 0x75e79e0f, 100 },
    { 0, 0 },
    { 0x91779e12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f79e16, 33 },
    { 0, 0 },
    { 0x21979e18, 33 },
    { 0x85579e19, 81 },
    { 0xdcd79e1a, 84 },
    { 0xc0879e1b, 87 },
    { 0xdeb79e1c, 95 },
    { 0x54279e1b, 91 },
    { 0x379e1e, 81 },
    { 0x4de79e1c, 43 },
    { 0xbe879e1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a179e23, 23 },
    { 0, 0 },
    { 0x44579e25, 95 },
    { 0xc3379e26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37279e32, 77 },
    { 0xcba79e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9779e37, 95 },
    { 0x3fd79e38, 68 },
    { 0x4a779e39, 34 },
    { 0xfa779e3a, 98 },
    { 0xc7079e38, 55 },
    { 0, 0 },
    { 0x41479e3d, 33 },
    { 0xe0a79e3e, 55 },
    { 0xa4c79e3f, 93 },
    { 0x179e3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d79e49, 33 },
    { 0x94379e4a, 61 },
    { 0xed279e4b, 23 },
    { 0xfc279e49, 55 },
    { 0x37e79e49, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa279e51, 23 },
    { 0xd1679e52, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4879e55, 33 },
    { 0x51179e56, 23 },
    { 0, 0 },
    { 0x6179e58, 90 },
    { 0x13879e58, 88 },
    { 0x5e79e5a, 75 },
    { 0x88379e5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c79e5e, 52 },
    { 0, 0 },
    { 0xd8579e60, 91 },
    { 0, 0 },
    { 0x1d379e62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4079e68, 67 },
    { 0, 0 },
    { 0xc3579e6a, 23 },
    { 0xa1a79e6b, 55 },
    { 0x5ec79e6c, 95 },
    { 0xed79e6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c79e70, 99 },
    { 0xd4f79e71, 69 },
    { 0xddd79e72, 23 },
    { 0x62179e70, 95 },
    { 0x2b679e70, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea79e7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4179e7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed79e81, 61 },
    { 0, 0 },
    { 0xcef79e83, 98 },
    { 0, 0 },
    { 0x91679e85, 68 },
    { 0x6eb79e86, 61 },
    { 0x2bc79e86, 23 },
    { 0x82d79e86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc79e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71279e92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda879e96, 33 },
    { 0, 0 },
    { 0x8d079e98, 23 },
    { 0xfeb79e99, 61 },
    { 0xc8279e9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41479ea0, 34 },
    { 0x5c479ea1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0879ea6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed79eab, 61 },
    { 0xcec79eac, 66 },
    { 0, 0 },
    { 0x24579eae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa979eb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc79eb6, 90 },
    { 0, 0 },
    { 0x64479eb8, 52 },
    { 0x99979eb8, 100 },
    { 0, 0 },
    { 0x30179ebb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1379ebf, 92 },
    { 0x13979ebf, 78 },
    { 0xa1c79ec1, 90 },
    { 0x42479ec2, 65 },
    { 0x5fb79ec2, 61 },
    { 0xd5279ec1, 33 },
    { 0xc6d79ec5, 55 },
    { 0x3279ec5, 78 },
    { 0x99b79ec4, 93 },
    { 0, 0 },
    { 0xd7379ec9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e79ece, 46 },
    { 0xfef79ece, 55 },
    { 0xe1679ecf, 61 },
    { 0x51079ed0, 23 },
    { 0x41f79ed2, 90 },
    { 0xda279ed2, 33 },
    { 0, 0 },
    { 0xd0779ed5, 78 },
    { 0xbfd79ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa79ed9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82179edd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3979ee2, 27 },
    { 0, 0 },
    { 0x97e79ee4, 98 },
    { 0x82b79ee5, 65 },
    { 0x37e79ee5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76579ee9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec79eee, 61 },
    { 0x13679eee, 90 },
    { 0x4cc79eef, 23 },
    { 0, 0 },
    { 0xd7679ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb279ef5, 81 },
    { 0xe6979ef5, 55 },
    { 0, 0 },
    { 0x48679ef8, 33 },
    { 0x78879ef9, 91 },
    { 0x5479efa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6479efe, 91 },
    { 0, 0 },
    { 0x95979f00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c79f03, 76 },
    { 0, 0 },
    { 0xaab79f05, 23 },
    { 0, 0 },
    { 0x3e579f07, 90 },
    { 0x60b79f07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9279f0b, 33 },
    { 0x2de79f0b, 36 },
    { 0xdfe79f0d, 28 },
    { 0xb2479f0d, 76 },
    { 0, 0 },
    { 0xef379f10, 90 },
    { 0xc4979f11, 66 },
    { 0x88d79f10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21679f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b79f1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8779f1d, 80 },
    { 0x6d579f1e, 53 },
    { 0, 0 },
    { 0x41a79f20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c79f25, 88 },
    { 0xe4b79f25, 90 },
    { 0xf4179f27, 66 },
    { 0xaaa79f27, 67 },
    { 0xe0679f29, 65 },
    { 0x4179f29, 87 },
    { 0x77879f2b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b79f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed79f33, 99 },
    { 0xf2c79f34, 61 },
    { 0x60379f34, 95 },
    { 0x8f579f36, 98 },
    { 0x4f279f37, 90 },
    { 0, 0 },
    { 0xc4979f39, 55 },
    { 0x5f979f39, 61 },
    { 0x49179f39, 75 },
    { 0xf979f3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e579f3f, 90 },
    { 0, 0 },
    { 0x58e79f41, 76 },
    { 0, 0 },
    { 0xa2779f43, 90 },
    { 0xf4179f44, 76 },
    { 0xbc679f45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd379f4b, 78 },
    { 0, 0 },
    { 0x2ab79f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c79f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0179f55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97679f59, 43 },
    { 0xb7579f5a, 68 },
    { 0xb3879f5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1279f5f, 76 },
    { 0xdba79f5f, 33 },
    { 0x27279f60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e79f67, 46 },
    { 0, 0 },
    { 0x89379f69, 46 },
    { 0x13079f6a, 99 },
    { 0xd0079f6b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7079f6f, 23 },
    { 0xdc279f70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c779f74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde79f78, 33 },
    { 0x7ba79f79, 90 },
    { 0x4ed79f7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a79f80, 93 },
    { 0, 0 },
    { 0xe5d79f82, 46 },
    { 0, 0 },
    { 0xbfc79f84, 34 },
    { 0xaa979f85, 33 },
    { 0x63479f86, 76 },
    { 0xd9179f85, 76 },
    { 0x85979f88, 91 },
    { 0, 0 },
    { 0x75f79f8a, 33 },
    { 0xfaa79f8a, 76 },
    { 0x33679f8a, 95 },
    { 0x2e979f8d, 99 },
    { 0x8df79f8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73579f96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb479f9a, 78 },
    { 0x16779f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8479f9e, 23 },
    { 0x37279f9f, 61 },
    { 0, 0 },
    { 0xc6e79fa1, 17 },
    { 0, 0 },
    { 0x4f179fa3, 78 },
    { 0, 0 },
    { 0x7c079fa5, 88 },
    { 0, 0 },
    { 0x80c79fa7, 95 },
    { 0x97d79fa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91279fac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba79faf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18679fb8, 90 },
    { 0x83f79fb9, 61 },
    { 0x8ac79fba, 93 },
    { 0xb2879fbb, 95 },
    { 0, 0 },
    { 0x5f579fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c279fc5, 61 },
    { 0x71a79fc6, 61 },
    { 0x12d79fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63179fcb, 87 },
    { 0, 0 },
    { 0xe0d79fcd, 66 },
    { 0xf5f79fce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc79fe1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e79fe8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2379feb, 78 },
    { 0x4e079feb, 33 },
    { 0x2d179fed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63579ff3, 61 },
    { 0xa2179ff4, 43 },
    { 0, 0 },
    { 0x92679ff6, 95 },
    { 0x11879ff7, 61 },
    { 0xe4579ff8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf579ffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4217a000, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe117a006, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1697a009, 69 },
    { 0x9477a00a, 76 },
    { 0, 0 },
    { 0x8be7a00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707a013, 98 },
    { 0xc1d7a014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e7a019, 81 },
    { 0x7957a019, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297a020, 68 },
    { 0x8d07a021, 90 },
    { 0, 0 },
    { 0xff37a023, 43 },
    { 0xfd57a023, 95 },
    { 0, 0 },
    { 0x807a026, 90 },
    { 0xc607a027, 95 },
    { 0, 0 },
    { 0x5817a029, 43 },
    { 0, 0 },
    { 0xb647a02b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c57a02e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd7a031, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f7a034, 34 },
    { 0x2f07a035, 23 },
    { 0x557a036, 61 },
    { 0x91d7a037, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1737a03b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7a042, 34 },
    { 0, 0 },
    { 0x5f37a044, 87 },
    { 0xa4a7a044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae27a04c, 77 },
    { 0x13e7a04c, 98 },
    { 0xa707a04e, 52 },
    { 0xb277a04f, 51 },
    { 0xd647a050, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e7a057, 95 },
    { 0, 0 },
    { 0x7ba7a059, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787a05c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b47a05f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f17a065, 61 },
    { 0xfe37a066, 61 },
    { 0x5447a067, 93 },
    { 0xdb57a067, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf57a06b, 99 },
    { 0xe747a06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d7a076, 95 },
    { 0x1e77a077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7147a07a, 64 },
    { 0, 0 },
    { 0x5487a07c, 76 },
    { 0, 0 },
    { 0x9787a07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b77a081, 87 },
    { 0xe7a082, 67 },
    { 0x2ee7a083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd187a086, 90 },
    { 0x42d7a087, 43 },
    { 0xc27a086, 23 },
    { 0, 0 },
    { 0x9f97a08a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5147a08f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f7a092, 33 },
    { 0x93f7a093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a7a098, 23 },
    { 0xaea7a098, 83 },
    { 0, 0 },
    { 0xafb7a09b, 61 },
    { 0xb047a09c, 93 },
    { 0x5467a09c, 55 },
    { 0x5477a09e, 23 },
    { 0xbff7a09d, 23 },
    { 0xaf17a0a0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a7a0a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17a0b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa7a0b4, 55 },
    { 0x6707a0b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x247a0b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1437a0bb, 55 },
    { 0, 0 },
    { 0xdfa7a0bd, 33 },
    { 0, 0 },
    { 0xebc7a0bf, 33 },
    { 0x3607a0c0, 66 },
    { 0xdfb7a0c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f27a0c7, 95 },
    { 0xe357a0c8, 61 },
    { 0x27c7a0c9, 33 },
    { 0x1557a0c7, 98 },
    { 0, 0 },
    { 0xbf07a0cc, 33 },
    { 0xbf77a0cc, 84 },
    { 0, 0 },
    { 0x9ec7a0cf, 61 },
    { 0x50e7a0d0, 61 },
    { 0, 0 },
    { 0xa0b7a0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5177a0df, 80 },
    { 0, 0 },
    { 0x84b7a0e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787a0e5, 23 },
    { 0xda07a0e6, 99 },
    { 0xd377a0e6, 76 },
    { 0xafb7a0e5, 33 },
    { 0, 0 },
    { 0x1be7a0ea, 33 },
    { 0xad37a0eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a7a0ee, 33 },
    { 0x5567a0ef, 75 },
    { 0, 0 },
    { 0x5bb7a0f1, 34 },
    { 0xf497a0f2, 87 },
    { 0, 0 },
    { 0xa3b7a0f4, 85 },
    { 0, 0 },
    { 0xd877a0f6, 43 },
    { 0x1837a0f7, 99 },
    { 0, 0 },
    { 0xdde7a0f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef7a0fc, 36 },
    { 0x5097a0fc, 61 },
    { 0x76e7a0fc, 90 },
    { 0xe427a0fc, 22 },
    { 0x5c7a0fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47a103, 55 },
    { 0x4db7a104, 76 },
    { 0xeb57a105, 100 },
    { 0x4bb7a105, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xda17a109, 33 },
    { 0, 0 },
    { 0x6567a10b, 61 },
    { 0x4d7a10c, 87 },
    { 0x7fa7a10c, 90 },
    { 0x22e7a10d, 90 },
    { 0xc4f7a10f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde27a115, 88 },
    { 0x5a57a115, 34 },
    { 0xa97a117, 38 },
    { 0xc857a115, 84 },
    { 0x1ec7a117, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7747a11d, 95 },
    { 0x4467a11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9367a128, 46 },
    { 0, 0 },
    { 0x6c57a12a, 65 },
    { 0x2cd7a12b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a12f, 76 },
    { 0x9057a130, 33 },
    { 0x6a47a131, 77 },
    { 0xfb7a132, 33 },
    { 0x5457a132, 90 },
    { 0xae07a130, 85 },
    { 0xe217a133, 61 },
    { 0x2867a130, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c97a139, 76 },
    { 0, 0 },
    { 0x8fd7a13b, 98 },
    { 0, 0 },
    { 0x8ee7a13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7a142, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d7a145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e7a14c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd247a152, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd527a158, 88 },
    { 0x16b7a159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd7a15c, 95 },
    { 0xb6a7a15d, 33 },
    { 0, 0 },
    { 0x9eb7a15f, 61 },
    { 0x9227a15f, 98 },
    { 0x2f17a161, 66 },
    { 0, 0 },
    { 0x1b37a163, 85 },
    { 0xd1a7a164, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e7a171, 90 },
    { 0x6087a172, 76 },
    { 0xd67a173, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea97a178, 33 },
    { 0x7067a179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b87a17c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d7a181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa57a184, 61 },
    { 0xb7e7a185, 23 },
    { 0, 0 },
    { 0xeb77a187, 61 },
    { 0x5757a188, 33 },
    { 0xcce7a189, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f67a18d, 34 },
    { 0, 0 },
    { 0xe127a18f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7a197, 61 },
    { 0x1de7a198, 81 },
    { 0x9607a199, 90 },
    { 0x117a19a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa857a19d, 94 },
    { 0x44a7a19e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d77a1ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd7a1b0, 90 },
    { 0, 0 },
    { 0xc067a1b2, 67 },
    { 0xc2b7a1b3, 95 },
    { 0xb267a1b4, 66 },
    { 0x3dd7a1b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd437a1b8, 61 },
    { 0, 0 },
    { 0xfdb7a1ba, 90 },
    { 0, 0 },
    { 0x2057a1bc, 75 },
    { 0x60c7a1bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07a1c0, 36 },
    { 0x6a87a1c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3007a1c4, 61 },
    { 0xcd07a1c4, 90 },
    { 0xf947a1c5, 91 },
    { 0x2f07a1c7, 61 },
    { 0xa567a1c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177a1cd, 55 },
    { 0, 0 },
    { 0xed47a1cf, 23 },
    { 0, 0 },
    { 0xe117a1d1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d7a1d8, 84 },
    { 0, 0 },
    { 0xe997a1da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa67a1dd, 98 },
    { 0, 0 },
    { 0xf1f7a1df, 78 },
    { 0, 0 },
    { 0xea7a1e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a1e7, 33 },
    { 0, 0 },
    { 0xeaf7a1e9, 61 },
    { 0xd3a7a1ea, 78 },
    { 0xfcc7a1e9, 52 },
    { 0, 0 },
    { 0x7ef7a1ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba7a1f0, 66 },
    { 0x28a7a1f0, 98 },
    { 0, 0 },
    { 0xe707a1f3, 33 },
    { 0xd677a1f4, 98 },
    { 0xb187a1f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf497a1f9, 98 },
    { 0x2707a1fa, 95 },
    { 0xdfd7a1f9, 80 },
    { 0, 0 },
    { 0x5b47a1fd, 90 },
    { 0x8eb7a1fd, 100 },
    { 0xc2e7a1ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7267a202, 55 },
    { 0, 0 },
    { 0xaa77a204, 66 },
    { 0xbf7a204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea7a208, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557a20b, 61 },
    { 0x9c87a20b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4697a20f, 75 },
    { 0, 0 },
    { 0x4037a211, 90 },
    { 0xa697a211, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b37a219, 61 },
    { 0xf207a21a, 90 },
    { 0x9be7a21b, 95 },
    { 0xb5b7a21c, 90 },
    { 0x807a21d, 55 },
    { 0xcda7a219, 55 },
    { 0, 0 },
    { 0x4a17a220, 99 },
    { 0, 0 },
    { 0x9367a222, 90 },
    { 0x40a7a223, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a7a226, 17 },
    { 0xf067a226, 55 },
    { 0xe967a226, 90 },
    { 0x6ee7a229, 33 },
    { 0xf1d7a22a, 68 },
    { 0xb1b7a22b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff7a22f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d7a235, 33 },
    { 0x3437a235, 98 },
    { 0x9f87a237, 78 },
    { 0xf457a237, 84 },
    { 0xcd77a237, 90 },
    { 0x4807a23a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7a23d, 95 },
    { 0, 0 },
    { 0xae67a23f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f07a242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff7a246, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d7a249, 90 },
    { 0, 0 },
    { 0x84e7a24b, 77 },
    { 0xd987a24b, 55 },
    { 0x9257a24c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277a252, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6617a25f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd017a263, 33 },
    { 0, 0 },
    { 0x6a7a265, 76 },
    { 0x9bc7a266, 95 },
    { 0xa577a267, 90 },
    { 0, 0 },
    { 0xb137a269, 87 },
    { 0xd597a26a, 61 },
    { 0x5087a26b, 100 },
    { 0x1dd7a26a, 55 },
    { 0, 0 },
    { 0x6b7a26e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197a272, 28 },
    { 0xa3b7a272, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2127a278, 8 },
    { 0, 0 },
    { 0x78e7a27a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7887a27d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d7a282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe307a28b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f37a292, 96 },
    { 0x47b7a293, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2837a297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d7a29b, 33 },
    { 0xd267a29b, 78 },
    { 0, 0 },
    { 0x7ff7a29e, 78 },
    { 0, 0 },
    { 0x9667a2a0, 90 },
    { 0, 0 },
    { 0x8fd7a2a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577a2a5, 90 },
    { 0xfea7a2a6, 90 },
    { 0x3a17a2a7, 90 },
    { 0xf0f7a2a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad7a2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4837a2b4, 33 },
    { 0x9327a2b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7a2be, 98 },
    { 0xdce7a2be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac7a2c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4057a2c7, 61 },
    { 0x5167a2c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87a2cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc57a2d9, 61 },
    { 0x4fe7a2da, 98 },
    { 0, 0 },
    { 0x1d17a2dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd087a2df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c7a2e2, 33 },
    { 0x9ad7a2e3, 98 },
    { 0x427a2e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707a2e8, 33 },
    { 0xc537a2e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f7a2ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237a2f4, 23 },
    { 0x75c7a2f4, 61 },
    { 0, 0 },
    { 0x2a77a2f7, 70 },
    { 0x3eb7a2f8, 23 },
    { 0xb7a7a2f9, 85 },
    { 0, 0 },
    { 0x9487a2fb, 99 },
    { 0x6d47a2fc, 33 },
    { 0xe447a2fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7a304, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a309, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c97a314, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea7a318, 23 },
    { 0x4f7a319, 46 },
    { 0, 0 },
    { 0x52e7a31b, 61 },
    { 0x4837a31c, 65 },
    { 0x3667a31d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e7a322, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f7a325, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e7a328, 56 },
    { 0, 0 },
    { 0x6fa7a32a, 100 },
    { 0x5af7a32a, 33 },
    { 0x8697a32c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b7a331, 95 },
    { 0x3427a331, 91 },
    { 0xd4c7a333, 88 },
    { 0xf1b7a334, 95 },
    { 0x7fe7a335, 90 },
    { 0xc37a336, 95 },
    { 0xb487a335, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb877a33a, 33 },
    { 0xc5e7a33b, 23 },
    { 0, 0 },
    { 0x7bf7a33d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda7a341, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e47a347, 55 },
    { 0x5e47a348, 34 },
    { 0xd737a349, 61 },
    { 0x5317a348, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2337a34f, 90 },
    { 0xf2a7a34f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f7a358, 23 },
    { 0, 0 },
    { 0xad77a35a, 33 },
    { 0, 0 },
    { 0x3ec7a35c, 78 },
    { 0xad47a35c, 95 },
    { 0x6e37a35e, 98 },
    { 0xe677a35f, 78 },
    { 0x6707a35e, 66 },
    { 0x4037a361, 87 },
    { 0, 0 },
    { 0x78c7a363, 66 },
    { 0x20b7a364, 23 },
    { 0xda87a365, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe937a368, 33 },
    { 0, 0 },
    { 0xbb57a36a, 23 },
    { 0x1237a36b, 90 },
    { 0x9f27a36c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3067a370, 76 },
    { 0xf2b7a371, 55 },
    { 0, 0 },
    { 0x6467a373, 90 },
    { 0xe847a374, 91 },
    { 0xeb27a374, 33 },
    { 0x7417a376, 81 },
    { 0x2ef7a376, 95 },
    { 0x1e27a378, 90 },
    { 0x3717a378, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf7a37e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7a381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd37a384, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe37a387, 78 },
    { 0xaba7a388, 76 },
    { 0x3997a389, 98 },
    { 0x5a07a389, 55 },
    { 0x5187a38b, 85 },
    { 0x7e97a387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc57a38f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c37a395, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa37a398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5267a39b, 33 },
    { 0, 0 },
    { 0x86f7a39d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e7a3a0, 90 },
    { 0, 0 },
    { 0x4bb7a3a2, 93 },
    { 0x87d7a3a3, 23 },
    { 0xd1e7a3a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6407a3aa, 98 },
    { 0, 0 },
    { 0xd457a3ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8187a3b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a97a3b6, 93 },
    { 0, 0 },
    { 0x7b97a3b8, 43 },
    { 0x4257a3b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c7a3bd, 61 },
    { 0, 0 },
    { 0x5207a3bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea7a3c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e7a3cf, 33 },
    { 0xeeb7a3d0, 87 },
    { 0, 0 },
    { 0x2e67a3d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce7a3d5, 61 },
    { 0x9aa7a3d6, 66 },
    { 0xd307a3d6, 98 },
    { 0xb677a3d8, 67 },
    { 0, 0 },
    { 0x9bd7a3da, 90 },
    { 0, 0 },
    { 0xbca7a3dc, 55 },
    { 0x5cf7a3dd, 95 },
    { 0, 0 },
    { 0xbd17a3df, 34 },
    { 0x95e7a3e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb7a3e7, 98 },
    { 0xa187a3e8, 76 },
    { 0, 0 },
    { 0x7377a3ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa7a3f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c7a3f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb307a3fe, 93 },
    { 0x1087a3ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907a403, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a17a406, 55 },
    { 0x5de7a407, 48 },
    { 0x6867a408, 33 },
    { 0xeb27a408, 12 },
    { 0x1d27a40a, 90 },
    { 0xc4b7a40b, 43 },
    { 0, 0 },
    { 0xb677a40d, 68 },
    { 0xc577a40e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd807a412, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e7a417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb77a41a, 90 },
    { 0x2d17a41b, 61 },
    { 0x5fb7a41a, 38 },
    { 0x5bd7a41c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f7a420, 90 },
    { 0, 0 },
    { 0x3fa7a422, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8777a426, 34 },
    { 0xf107a427, 98 },
    { 0, 0 },
    { 0x2007a429, 46 },
    { 0x7a97a429, 33 },
    { 0x3b67a42a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3777a42e, 66 },
    { 0x86e7a42f, 52 },
    { 0, 0 },
    { 0x8b37a431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb227a435, 90 },
    { 0x26a7a435, 90 },
    { 0, 0 },
    { 0x4e37a438, 33 },
    { 0xc757a439, 33 },
    { 0xd027a43a, 33 },
    { 0x4bc7a43a, 61 },
    { 0, 0 },
    { 0x1227a43d, 55 },
    { 0xdfb7a43e, 90 },
    { 0xb867a43e, 23 },
    { 0xc097a440, 61 },
    { 0, 0 },
    { 0xb217a442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb97a44a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6207a44e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2577a451, 43 },
    { 0x9c77a452, 95 },
    { 0xbb67a452, 43 },
    { 0, 0 },
    { 0x3bd7a455, 23 },
    { 0x9287a456, 85 },
    { 0, 0 },
    { 0x1fd7a458, 61 },
    { 0x1497a459, 95 },
    { 0x5887a458, 95 },
    { 0, 0 },
    { 0x3537a45c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a07a462, 83 },
    { 0x72a7a463, 43 },
    { 0xc287a464, 80 },
    { 0xa937a465, 76 },
    { 0, 0 },
    { 0x7427a467, 90 },
    { 0xdc67a467, 90 },
    { 0xda57a469, 52 },
    { 0x2307a469, 43 },
    { 0x5477a46b, 55 },
    { 0xab37a468, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2047a473, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557a477, 61 },
    { 0xbed7a478, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b7a47b, 90 },
    { 0x5c17a47c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa37a484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9057a487, 68 },
    { 0xccc7a487, 95 },
    { 0xf67a489, 50 },
    { 0, 0 },
    { 0xd037a48b, 52 },
    { 0x2f7a48c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a37a48f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3157a493, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7a497, 55 },
    { 0, 0 },
    { 0x7057a499, 33 },
    { 0x267a49a, 40 },
    { 0xe257a499, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5967a4a0, 90 },
    { 0x9757a4a1, 90 },
    { 0x2bd7a4a2, 55 },
    { 0x16b7a4a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f7a4a7, 43 },
    { 0xaab7a4a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1407a4ac, 66 },
    { 0, 0 },
    { 0xb787a4ae, 68 },
    { 0x7ad7a4af, 66 },
    { 0xd77a4ae, 95 },
    { 0x46e7a4ae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7547a4b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb967a4ba, 56 },
    { 0, 0 },
    { 0x7317a4bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc7a4c2, 90 },
    { 0, 0 },
    { 0x94f7a4c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb117a4c7, 46 },
    { 0, 0 },
    { 0xdbb7a4c9, 98 },
    { 0, 0 },
    { 0x1507a4cb, 91 },
    { 0x837a4cb, 61 },
    { 0xc7a7a4cd, 46 },
    { 0, 0 },
    { 0x67e7a4cf, 80 },
    { 0, 0 },
    { 0x83d7a4d1, 43 },
    { 0x79f7a4d2, 55 },
    { 0x6cb7a4d2, 46 },
    { 0, 0 },
    { 0x23a7a4d5, 95 },
    { 0x68a7a4d5, 23 },
    { 0x40b7a4d7, 33 },
    { 0xfc17a4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1007a4db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a7a4de, 77 },
    { 0, 0 },
    { 0xba37a4e0, 33 },
    { 0x17a4e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb7a4ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f87a4f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf077a4f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x717a4fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab7a4fe, 90 },
    { 0x6a87a4fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe287a504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8177a509, 81 },
    { 0, 0 },
    { 0x7e67a50b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda7a510, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287a519, 33 },
    { 0xf5e7a519, 33 },
    { 0x1e7a51b, 98 },
    { 0x987a51b, 100 },
    { 0xc9c7a51c, 61 },
    { 0x7a51e, 33 },
    { 0xb2b7a51a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8807a522, 85 },
    { 0x1b97a522, 77 },
    { 0x5ee7a523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4247a529, 91 },
    { 0, 0 },
    { 0x147a52b, 67 },
    { 0, 0 },
    { 0x1de7a52d, 90 },
    { 0x3e7a52d, 55 },
    { 0xe547a52f, 98 },
    { 0xea17a530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf7a535, 53 },
    { 0x8217a535, 43 },
    { 0, 0 },
    { 0x1517a538, 76 },
    { 0xd3d7a539, 44 },
    { 0x45b7a539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db7a53d, 43 },
    { 0, 0 },
    { 0xa607a53f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d7a549, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547a553, 61 },
    { 0xffc7a554, 98 },
    { 0x29e7a555, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3167a559, 61 },
    { 0xdcb7a55a, 93 },
    { 0x7397a55b, 84 },
    { 0x8297a55a, 90 },
    { 0, 0 },
    { 0x6167a55e, 46 },
    { 0, 0 },
    { 0x8ef7a560, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee7a563, 90 },
    { 0, 0 },
    { 0x7cd7a565, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b7a56b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9327a56e, 85 },
    { 0x6a77a56f, 76 },
    { 0xe87a570, 61 },
    { 0xc8d7a570, 61 },
    { 0xf4e7a56f, 61 },
    { 0x2a07a571, 34 },
    { 0x6987a56e, 98 },
    { 0x2377a575, 76 },
    { 0xd097a575, 55 },
    { 0xd57a577, 33 },
    { 0x3847a575, 61 },
    { 0xaa77a578, 26 },
    { 0x97d7a57a, 75 },
    { 0x2bc7a57b, 98 },
    { 0xc437a571, 43 },
    { 0, 0 },
    { 0x1fc7a57e, 67 },
    { 0, 0 },
    { 0xc497a580, 55 },
    { 0xadf7a580, 33 },
    { 0xd6a7a582, 95 },
    { 0xdbc7a583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f7a586, 43 },
    { 0, 0 },
    { 0xf7b7a588, 33 },
    { 0, 0 },
    { 0x4ed7a58a, 66 },
    { 0x8eb7a58b, 98 },
    { 0, 0 },
    { 0xc737a58d, 95 },
    { 0, 0 },
    { 0x6877a58f, 85 },
    { 0, 0 },
    { 0xe147a591, 52 },
    { 0x1eb7a592, 76 },
    { 0xf727a591, 95 },
    { 0x6757a594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c87a598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e27a59b, 90 },
    { 0, 0 },
    { 0x9437a59d, 61 },
    { 0x3707a59e, 90 },
    { 0xe17a59f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f07a5a3, 88 },
    { 0x357a5a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2667a5aa, 95 },
    { 0, 0 },
    { 0x9bc7a5ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7a5b2, 61 },
    { 0, 0 },
    { 0xe887a5b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x187a5b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5007a5ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8337a5be, 55 },
    { 0, 0 },
    { 0xa2a7a5c0, 90 },
    { 0xbe47a5c1, 90 },
    { 0x41b7a5c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5837a5c6, 85 },
    { 0x28b7a5c6, 95 },
    { 0xdf87a5c8, 68 },
    { 0xfa87a5c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5237a5cc, 8 },
    { 0x6557a5cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd47a5d0, 33 },
    { 0, 0 },
    { 0xddf7a5d2, 90 },
    { 0x7f07a5d3, 78 },
    { 0x3c87a5d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b37a5da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac07a5e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca37a5eb, 98 },
    { 0x9647a5ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa227a5f1, 52 },
    { 0xb8b7a5f2, 95 },
    { 0x73d7a5f3, 95 },
    { 0x8587a5f4, 90 },
    { 0x2897a5f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb7a5fe, 23 },
    { 0xb587a5ff, 69 },
    { 0, 0 },
    { 0x6d57a601, 90 },
    { 0x33f7a601, 100 },
    { 0x3e57a601, 80 },
    { 0, 0 },
    { 0x8917a605, 61 },
    { 0xc17a605, 95 },
    { 0xaa77a607, 90 },
    { 0x42e7a607, 52 },
    { 0x51f7a609, 95 },
    { 0x94c7a609, 61 },
    { 0x8527a609, 23 },
    { 0x1a7a60b, 34 },
    { 0xf4a7a605, 100 },
    { 0xce87a606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7a611, 43 },
    { 0xe4b7a612, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e27a616, 95 },
    { 0x57f7a616, 85 },
    { 0xb877a618, 100 },
    { 0xb6b7a619, 34 },
    { 0x6f17a618, 55 },
    { 0x53e7a61b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a7a61f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf7a629, 33 },
    { 0xf97a629, 61 },
    { 0xb7f7a62b, 90 },
    { 0xf117a62a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87a634, 76 },
    { 0, 0 },
    { 0x6517a636, 33 },
    { 0, 0 },
    { 0x9fc7a638, 43 },
    { 0, 0 },
    { 0x7007a63a, 90 },
    { 0, 0 },
    { 0xcf07a63c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee7a63f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1687a646, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c37a64e, 98 },
    { 0x8e87a64e, 95 },
    { 0x2c17a650, 90 },
    { 0x1547a651, 61 },
    { 0, 0 },
    { 0x5b7a653, 84 },
    { 0xc097a654, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe447a658, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee7a65b, 46 },
    { 0xc8b7a65b, 23 },
    { 0x6567a65d, 90 },
    { 0x7597a65d, 98 },
    { 0xc5e7a65d, 55 },
    { 0xb827a660, 33 },
    { 0, 0 },
    { 0x4a87a662, 33 },
    { 0x5cd7a663, 61 },
    { 0x50e7a664, 80 },
    { 0x68e7a662, 33 },
    { 0, 0 },
    { 0xe817a667, 98 },
    { 0x1607a667, 95 },
    { 0, 0 },
    { 0xb4d7a66a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b7a66d, 98 },
    { 0x3377a66d, 59 },
    { 0xd647a66f, 33 },
    { 0x9c37a66f, 81 },
    { 0xf567a671, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd907a674, 61 },
    { 0xcdb7a675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc027a679, 87 },
    { 0x9007a67a, 43 },
    { 0xfe87a679, 100 },
    { 0x2b97a679, 33 },
    { 0x1627a67d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a7a681, 75 },
    { 0xcaf7a682, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2037a686, 90 },
    { 0xe497a686, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda27a68b, 78 },
    { 0, 0 },
    { 0xa157a68d, 90 },
    { 0, 0 },
    { 0x5807a68f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe7a698, 33 },
    { 0x95f7a699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee7a69d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2697a6a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17a6a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd7a6a7, 33 },
    { 0xf497a6a8, 76 },
    { 0x8e67a6a8, 48 },
    { 0, 0 },
    { 0x19f7a6ab, 33 },
    { 0xdcf7a6ab, 46 },
    { 0x5357a6ad, 68 },
    { 0x47b7a6ad, 96 },
    { 0xdce7a6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a6b7, 90 },
    { 0, 0 },
    { 0x4e67a6b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77a6c7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b7a6cb, 33 },
    { 0x8c17a6cc, 91 },
    { 0x8fe7a6cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7a6d4, 33 },
    { 0, 0 },
    { 0x6e77a6d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec57a6d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5997a6e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe657a6e5, 76 },
    { 0xabb7a6e6, 33 },
    { 0x1137a6e6, 95 },
    { 0x5fd7a6e8, 33 },
    { 0xd757a6e9, 90 },
    { 0xe827a6e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9607a6f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d7a6fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc367a700, 95 },
    { 0x5c77a701, 90 },
    { 0xe1c7a702, 90 },
    { 0xa7f7a703, 43 },
    { 0xe617a704, 99 },
    { 0xe437a705, 95 },
    { 0xb917a706, 77 },
    { 0xab7a705, 55 },
    { 0x6637a706, 95 },
    { 0x3137a707, 99 },
    { 0x4cf7a70a, 95 },
    { 0x9e27a70a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557a713, 33 },
    { 0, 0 },
    { 0x22b7a715, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a7a722, 76 },
    { 0x8f17a723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7907a726, 98 },
    { 0, 0 },
    { 0x7f27a728, 90 },
    { 0x67b7a729, 90 },
    { 0x7207a729, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc477a72d, 91 },
    { 0xf3a7a72e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb657a737, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a7a73a, 33 },
    { 0, 0 },
    { 0x1c57a73c, 92 },
    { 0xf7f7a73d, 90 },
    { 0x957a73e, 33 },
    { 0, 0 },
    { 0x8647a740, 55 },
    { 0x7e07a741, 78 },
    { 0, 0 },
    { 0x7177a743, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e7a74a, 90 },
    { 0x1dd7a74b, 76 },
    { 0x927a74a, 67 },
    { 0x4e27a74d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb337a751, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5277a757, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2047a75d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c7a762, 91 },
    { 0x9707a763, 76 },
    { 0x79b7a764, 43 },
    { 0x4ed7a762, 61 },
    { 0x91a7a766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2307a76a, 34 },
    { 0xbf97a76b, 67 },
    { 0xaa37a76b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d57a76f, 90 },
    { 0x5c67a770, 81 },
    { 0xd707a771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7907a774, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf17a777, 100 },
    { 0xf497a778, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567a77d, 95 },
    { 0x8527a77e, 90 },
    { 0x2c67a77d, 43 },
    { 0x1457a780, 76 },
    { 0x12d7a781, 61 },
    { 0, 0 },
    { 0x36d7a783, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1747a786, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a7a78b, 34 },
    { 0x2f77a78c, 61 },
    { 0xd9c7a78c, 23 },
    { 0xd197a78e, 49 },
    { 0x47f7a78d, 98 },
    { 0x8a77a790, 38 },
    { 0, 0 },
    { 0x2eb7a792, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2597a795, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee7a798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4977a79c, 61 },
    { 0xaa87a79d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b47a7a0, 84 },
    { 0xd2b7a7a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9057a7a5, 67 },
    { 0, 0 },
    { 0x8177a7a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f7a7aa, 23 },
    { 0xbb27a7ab, 61 },
    { 0x687a7ac, 76 },
    { 0xab37a7aa, 95 },
    { 0xce97a7aa, 61 },
    { 0x56b7a7ab, 46 },
    { 0x89a7a7b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa7a7bc, 90 },
    { 0xc067a7bd, 81 },
    { 0x1b97a7bc, 61 },
    { 0xf067a7be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf697a7c3, 90 },
    { 0xf967a7c4, 78 },
    { 0, 0 },
    { 0xa5c7a7c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f27a7c9, 66 },
    { 0x5a27a7c9, 98 },
    { 0x46c7a7ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf167a7cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67a7d8, 84 },
    { 0, 0 },
    { 0xdca7a7da, 95 },
    { 0x6327a7da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c7a7e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da7a7ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f67a7ee, 90 },
    { 0, 0 },
    { 0xb647a7f0, 85 },
    { 0, 0 },
    { 0x29b7a7f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa7a7fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d87a805, 23 },
    { 0x7d37a806, 80 },
    { 0x9677a807, 87 },
    { 0x47d7a808, 95 },
    { 0x4f17a808, 91 },
    { 0x987a80a, 90 },
    { 0xdc7a806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7a811, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2307a816, 93 },
    { 0x24e7a817, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd87a81e, 34 },
    { 0x6107a81e, 34 },
    { 0xfe57a81e, 53 },
    { 0x4047a821, 98 },
    { 0x5c77a822, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7a826, 95 },
    { 0xc9e7a826, 52 },
    { 0, 0 },
    { 0xeac7a829, 90 },
    { 0, 0 },
    { 0xef47a82b, 100 },
    { 0xbb47a82c, 90 },
    { 0, 0 },
    { 0x5e97a82e, 98 },
    { 0x9ef7a82f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9707a832, 95 },
    { 0xcca7a833, 52 },
    { 0xd4f7a834, 91 },
    { 0x9f67a835, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367a839, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b7a83c, 61 },
    { 0, 0 },
    { 0x2027a83e, 23 },
    { 0xc407a83e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee7a842, 67 },
    { 0xf837a842, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6017a846, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87a84a, 61 },
    { 0x1427a84a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2507a84f, 23 },
    { 0, 0 },
    { 0x8d37a851, 76 },
    { 0xf2a7a852, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7a858, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc417a85d, 52 },
    { 0, 0 },
    { 0xe4a7a85f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a17a862, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8387a868, 61 },
    { 0xf587a869, 40 },
    { 0xca7a868, 36 },
    { 0, 0 },
    { 0xa1b7a86c, 67 },
    { 0xa57a86d, 100 },
    { 0xa37a86c, 96 },
    { 0x3fa7a86f, 17 },
    { 0x237a870, 23 },
    { 0x95a7a86d, 43 },
    { 0xc857a86e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07a876, 41 },
    { 0x7617a877, 61 },
    { 0, 0 },
    { 0x7187a879, 23 },
    { 0x2057a879, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e7a886, 40 },
    { 0x9317a887, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd7a88a, 95 },
    { 0x7ec7a88b, 86 },
    { 0xc0f7a88c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007a88f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf07a894, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf637a898, 95 },
    { 0, 0 },
    { 0x5e17a89a, 91 },
    { 0x4b87a89b, 61 },
    { 0x1267a89b, 88 },
    { 0, 0 },
    { 0x6a97a89e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e7a8a4, 66 },
    { 0, 0 },
    { 0x8bc7a8a6, 98 },
    { 0xf417a8a6, 61 },
    { 0x3977a8a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4017a8ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f7a8ae, 61 },
    { 0xedc7a8af, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a7a8b3, 46 },
    { 0x7477a8b4, 55 },
    { 0x5f27a8b5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb7a8b9, 23 },
    { 0xd417a8ba, 46 },
    { 0x6a37a8bb, 90 },
    { 0xec67a8bb, 67 },
    { 0x2a57a8bc, 90 },
    { 0x2e97a8bb, 95 },
    { 0xa477a8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c7a8c3, 76 },
    { 0x2b87a8c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a8c7, 90 },
    { 0xd447a8c8, 95 },
    { 0x3bc7a8c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d57a8ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf167a8d2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a8d8, 90 },
    { 0x9007a8d9, 17 },
    { 0, 0 },
    { 0x5437a8db, 67 },
    { 0x6817a8dc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4447a8df, 33 },
    { 0x38a7a8e0, 95 },
    { 0x55b7a8e1, 88 },
    { 0, 0 },
    { 0xc517a8e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7a8e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e27a8ed, 67 },
    { 0x8917a8ee, 95 },
    { 0xcc7a8ef, 90 },
    { 0x8af7a8ef, 46 },
    { 0, 0 },
    { 0x7cc7a8f2, 24 },
    { 0x8417a8f2, 33 },
    { 0, 0 },
    { 0x6d7a8f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc507a8f8, 34 },
    { 0, 0 },
    { 0x7f77a8fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7a8fd, 85 },
    { 0x2fb7a8fe, 55 },
    { 0x1bd7a8ff, 33 },
    { 0x747a8fd, 33 },
    { 0x88e7a8fe, 98 },
    { 0x4047a901, 95 },
    { 0, 0 },
    { 0x9377a904, 78 },
    { 0xefa7a905, 61 },
    { 0, 0 },
    { 0x4137a907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d7a90f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf627a913, 67 },
    { 0x8287a913, 68 },
    { 0x1fd7a915, 99 },
    { 0, 0 },
    { 0xff97a917, 90 },
    { 0xdc57a917, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7a91e, 90 },
    { 0, 0 },
    { 0xcc77a920, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc207a925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb7a92d, 52 },
    { 0x5c27a92e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17a933, 90 },
    { 0xcf67a933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8227a937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc7a93a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc7a93d, 33 },
    { 0x1a87a93e, 77 },
    { 0x3f07a93f, 95 },
    { 0x4847a940, 90 },
    { 0x8d7a93e, 52 },
    { 0x3e17a942, 61 },
    { 0x9227a943, 90 },
    { 0x6ff7a940, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3477a948, 34 },
    { 0, 0 },
    { 0xd347a94a, 33 },
    { 0x5d7a94a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2017a950, 95 },
    { 0x6e47a951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e7a954, 94 },
    { 0x6717a954, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d7a95a, 88 },
    { 0, 0 },
    { 0x4167a95c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca7a960, 98 },
    { 0, 0 },
    { 0xd717a962, 46 },
    { 0, 0 },
    { 0x2fa7a964, 95 },
    { 0, 0 },
    { 0xb677a966, 43 },
    { 0, 0 },
    { 0x3dc7a968, 23 },
    { 0, 0 },
    { 0xd7c7a96a, 61 },
    { 0, 0 },
    { 0xa447a96c, 33 },
    { 0, 0 },
    { 0x8ca7a96e, 99 },
    { 0xed67a96f, 61 },
    { 0x2c97a970, 93 },
    { 0x6e57a96e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47a975, 36 },
    { 0xdf17a976, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9527a97b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d7a97e, 33 },
    { 0xe327a97f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd857a984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7177a988, 55 },
    { 0, 0 },
    { 0xfde7a98a, 96 },
    { 0, 0 },
    { 0xa777a98c, 95 },
    { 0, 0 },
    { 0x9a27a98e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77a991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf187a996, 98 },
    { 0x9c47a996, 95 },
    { 0xf5e7a998, 33 },
    { 0xc1a7a999, 33 },
    { 0x2597a99a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c7a99d, 93 },
    { 0x4f47a99e, 61 },
    { 0x29e7a99f, 88 },
    { 0, 0 },
    { 0x4897a9a1, 76 },
    { 0x4ca7a9a2, 98 },
    { 0x3957a9a3, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xec67a9a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577a9a9, 90 },
    { 0xb1a7a9aa, 33 },
    { 0x7ca7a9ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897a9ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b7a9b7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d7a9bd, 61 },
    { 0x8177a9be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e37a9c1, 76 },
    { 0x7607a9c2, 98 },
    { 0x5a47a9c3, 43 },
    { 0xb5d7a9c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b7a9c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f7a9cd, 55 },
    { 0xa1f7a9cd, 98 },
    { 0, 0 },
    { 0x6457a9d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa587a9d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba7a9d8, 33 },
    { 0x347a9d9, 17 },
    { 0xb417a9da, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f7a9de, 90 },
    { 0xc637a9df, 95 },
    { 0xd1a7a9de, 23 },
    { 0x2517a9e1, 95 },
    { 0x69b7a9df, 33 },
    { 0xb027a9df, 66 },
    { 0, 0 },
    { 0x5fd7a9e5, 33 },
    { 0, 0 },
    { 0xe377a9e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1867a9eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec07a9f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5557a9fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c7aa00, 69 },
    { 0xc8a7aa01, 61 },
    { 0x45c7aa02, 93 },
    { 0, 0 },
    { 0x8367aa04, 95 },
    { 0xab17aa05, 23 },
    { 0x437aa04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c7aa09, 33 },
    { 0x1a77aa09, 93 },
    { 0x4447aa0b, 23 },
    { 0xcfb7aa0c, 33 },
    { 0, 0 },
    { 0x31c7aa0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7947aa14, 95 },
    { 0xdc57aa15, 98 },
    { 0x1ad7aa14, 99 },
    { 0x3527aa16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af7aa20, 90 },
    { 0x1b7aa21, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f7aa24, 67 },
    { 0x1857aa25, 84 },
    { 0, 0 },
    { 0xca87aa27, 69 },
    { 0xb017aa28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa7aa2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5147aa2e, 23 },
    { 0, 0 },
    { 0x32b7aa30, 66 },
    { 0xa207aa31, 95 },
    { 0xcf67aa32, 33 },
    { 0xd537aa33, 61 },
    { 0x4077aa34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe607aa38, 95 },
    { 0xc3b7aa39, 90 },
    { 0, 0 },
    { 0x3b7aa3b, 76 },
    { 0xd2a7aa3b, 95 },
    { 0x9197aa3d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe767aa42, 88 },
    { 0, 0 },
    { 0xec07aa44, 61 },
    { 0xca47aa44, 87 },
    { 0x2277aa46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1617aa4b, 87 },
    { 0xd3a7aa4b, 55 },
    { 0x4497aa4c, 90 },
    { 0x6da7aa4c, 90 },
    { 0x7417aa4d, 43 },
    { 0x3f97aa4c, 61 },
    { 0, 0 },
    { 0xf507aa52, 43 },
    { 0x767aa53, 90 },
    { 0x5e87aa52, 78 },
    { 0, 0 },
    { 0xa557aa56, 61 },
    { 0, 0 },
    { 0xd7b7aa58, 55 },
    { 0xad57aa59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2047aa5c, 90 },
    { 0xfe37aa5c, 23 },
    { 0, 0 },
    { 0x6e87aa5f, 43 },
    { 0x3a17aa60, 98 },
    { 0x5537aa60, 95 },
    { 0xe8d7aa62, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xc237aa65, 90 },
    { 0xd887aa66, 55 },
    { 0x5777aa67, 61 },
    { 0x3477aa66, 23 },
    { 0x3a47aa69, 98 },
    { 0xb847aa6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad7aa6d, 99 },
    { 0, 0 },
    { 0x6777aa6f, 36 },
    { 0, 0 },
    { 0xc607aa71, 23 },
    { 0x1a27aa72, 33 },
    { 0x6a77aa73, 33 },
    { 0x5d67aa72, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6497aa80, 98 },
    { 0, 0 },
    { 0xdc77aa82, 46 },
    { 0x7747aa82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007aa87, 91 },
    { 0, 0 },
    { 0x4617aa89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577aa8e, 23 },
    { 0xde17aa8e, 23 },
    { 0xab37aa90, 90 },
    { 0x287aa91, 84 },
    { 0x817aa91, 61 },
    { 0x1bd7aa92, 61 },
    { 0xb697aa90, 93 },
    { 0xf957aa95, 95 },
    { 0, 0 },
    { 0x4077aa97, 33 },
    { 0, 0 },
    { 0x6107aa99, 23 },
    { 0x59b7aa9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb227aa9d, 98 },
    { 0, 0 },
    { 0xd167aa9f, 76 },
    { 0, 0 },
    { 0xcba7aaa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf07aaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd7aaab, 90 },
    { 0x1167aaac, 17 },
    { 0x5637aaad, 34 },
    { 0x35e7aaab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a7aab1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d7aab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c7aab9, 65 },
    { 0x6717aab9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c7aac4, 33 },
    { 0, 0 },
    { 0x7de7aac6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e7aac9, 100 },
    { 0x6db7aaca, 43 },
    { 0, 0 },
    { 0xb4c7aacc, 99 },
    { 0xa337aacd, 68 },
    { 0xe8a7aace, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aad6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb657aada, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a7aadd, 91 },
    { 0xef7aade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e67aae1, 38 },
    { 0x3e67aae2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b7aae5, 61 },
    { 0x1287aae6, 95 },
    { 0x2a37aae5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2517aaeb, 33 },
    { 0x1f97aaec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77aaf3, 84 },
    { 0xa857aaf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf7aaf8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17aafb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f97aafe, 85 },
    { 0, 0 },
    { 0xb997ab00, 55 },
    { 0xe627ab01, 90 },
    { 0x3b7ab02, 95 },
    { 0x5b17ab02, 61 },
    { 0xb17ab04, 33 },
    { 0x31c7ab04, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6937ab0a, 100 },
    { 0, 0 },
    { 0x2f7ab0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9177ab0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547ab14, 91 },
    { 0x2a47ab14, 99 },
    { 0x8897ab16, 77 },
    { 0, 0 },
    { 0xbe67ab18, 23 },
    { 0xd0f7ab18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ab1d, 23 },
    { 0xcf47ab1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc907ab21, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d7ab28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297ab2d, 90 },
    { 0xe057ab2d, 90 },
    { 0, 0 },
    { 0x2b47ab30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5967ab39, 23 },
    { 0, 0 },
    { 0x4c17ab3b, 23 },
    { 0x4807ab3c, 53 },
    { 0xa337ab3c, 98 },
    { 0x227ab3e, 23 },
    { 0x3c47ab3f, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda67ab46, 76 },
    { 0xc6e7ab46, 33 },
    { 0x4c97ab47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a7ab51, 98 },
    { 0, 0 },
    { 0xd1d7ab53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b7ab57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f27ab5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb7ab62, 91 },
    { 0x8527ab63, 34 },
    { 0x22f7ab62, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd637ab68, 17 },
    { 0x3d47ab69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe597ab6d, 55 },
    { 0x3337ab6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9267ab73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2067ab78, 61 },
    { 0xa197ab79, 90 },
    { 0xc637ab79, 23 },
    { 0xd147ab7b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3827ab80, 52 },
    { 0, 0 },
    { 0x617ab82, 61 },
    { 0x51d7ab83, 68 },
    { 0x4aa7ab84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e7ab8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1407ab98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d7ab9c, 46 },
    { 0x3107ab9c, 90 },
    { 0, 0 },
    { 0xdb37ab9f, 88 },
    { 0x8917aba0, 61 },
    { 0xae97aba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb7aba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8897abab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd067abae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7347abb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f17abba, 90 },
    { 0x4507abbb, 61 },
    { 0x31c7abbb, 77 },
    { 0x9d37abbb, 98 },
    { 0x787abbe, 76 },
    { 0x1167abbd, 90 },
    { 0, 0 },
    { 0xc917abc1, 61 },
    { 0x5b67abc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df7abc7, 90 },
    { 0xead7abc7, 98 },
    { 0x3b67abc7, 52 },
    { 0, 0 },
    { 0x8e67abcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b37abd1, 33 },
    { 0x5407abd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f67abd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b7abda, 23 },
    { 0x33d7abdb, 23 },
    { 0x8ad7abda, 98 },
    { 0x7f47abdd, 88 },
    { 0, 0 },
    { 0xd517abdf, 33 },
    { 0xeeb7abe0, 98 },
    { 0xbe67abe0, 23 },
    { 0x1a77abe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa847abe7, 33 },
    { 0x9e77abe7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd927abfe, 33 },
    { 0xf417abff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd677ac06, 61 },
    { 0x2bf7ac07, 98 },
    { 0x17ac08, 76 },
    { 0, 0 },
    { 0x7657ac0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e27ac13, 90 },
    { 0x71b7ac14, 98 },
    { 0x6777ac15, 50 },
    { 0x7227ac16, 90 },
    { 0x7207ac17, 61 },
    { 0xbb97ac17, 55 },
    { 0xafa7ac19, 90 },
    { 0x5f37ac1a, 55 },
    { 0x6097ac1a, 68 },
    { 0xed27ac1b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a77ac21, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea7ac28, 46 },
    { 0x317ac29, 98 },
    { 0x8d7ac2a, 95 },
    { 0xd397ac2a, 75 },
    { 0xfae7ac2a, 95 },
    { 0, 0 },
    { 0x8e27ac2e, 91 },
    { 0x14f7ac2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6317ac32, 46 },
    { 0, 0 },
    { 0x8137ac34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b7ac39, 61 },
    { 0, 0 },
    { 0xfe27ac3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5417ac40, 90 },
    { 0, 0 },
    { 0xc87ac42, 100 },
    { 0x3a77ac43, 33 },
    { 0xc537ac43, 46 },
    { 0, 0 },
    { 0x9667ac46, 55 },
    { 0xc9b7ac47, 52 },
    { 0xeeb7ac46, 90 },
    { 0xae7ac49, 75 },
    { 0xa507ac49, 90 },
    { 0x9227ac4b, 23 },
    { 0x1377ac4c, 33 },
    { 0x96d7ac4d, 23 },
    { 0, 0 },
    { 0xe1a7ac4f, 68 },
    { 0x85c7ac50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5917ac59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc547ac5f, 61 },
    { 0, 0 },
    { 0xd087ac61, 59 },
    { 0, 0 },
    { 0x4b07ac63, 90 },
    { 0, 0 },
    { 0xe107ac65, 85 },
    { 0xcd17ac66, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d7ac6c, 90 },
    { 0, 0 },
    { 0x8207ac6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df7ac72, 95 },
    { 0, 0 },
    { 0x3aa7ac74, 90 },
    { 0, 0 },
    { 0x6107ac76, 61 },
    { 0x4337ac77, 33 },
    { 0xf607ac78, 81 },
    { 0x19a7ac79, 76 },
    { 0x5e77ac7a, 95 },
    { 0, 0 },
    { 0xef17ac7c, 67 },
    { 0, 0 },
    { 0xe027ac7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467ac82, 33 },
    { 0x2727ac82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7ac8a, 95 },
    { 0x6c17ac8b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb247ac8e, 17 },
    { 0x7897ac8f, 90 },
    { 0x7ba7ac90, 33 },
    { 0, 0 },
    { 0x1f87ac92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8057ac97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d77ac9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f87ac9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7567aca9, 55 },
    { 0, 0 },
    { 0xf9e7acab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7acaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe727acb3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x897acb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b7acb9, 43 },
    { 0x56b7acba, 61 },
    { 0x4ec7acba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c7acbf, 98 },
    { 0x8f97acbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b7acc4, 33 },
    { 0x46f7acc5, 55 },
    { 0x10c7acc6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7087acce, 76 },
    { 0xeb57accf, 78 },
    { 0xf0b7accf, 90 },
    { 0x3887acd1, 33 },
    { 0x7d47acd1, 23 },
    { 0xaba7acd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf47acd6, 90 },
    { 0, 0 },
    { 0x6787acd8, 23 },
    { 0xb57acd9, 95 },
    { 0xd437acda, 34 },
    { 0x1aa7acd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ace2, 43 },
    { 0, 0 },
    { 0xdfb7ace4, 76 },
    { 0, 0 },
    { 0x1a87ace6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4987ace9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5377acf4, 76 },
    { 0x2307acf5, 78 },
    { 0x5687acf6, 33 },
    { 0x3a97acf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf737ad00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd7ad08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47ad0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7ad0f, 76 },
    { 0x9067ad0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec7ad17, 61 },
    { 0xf017ad18, 69 },
    { 0x1637ad18, 33 },
    { 0xd237ad1a, 46 },
    { 0, 0 },
    { 0xeeb7ad1c, 91 },
    { 0, 0 },
    { 0x5e7ad1e, 98 },
    { 0xa967ad1f, 34 },
    { 0x2267ad20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f7ad23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e7ad2d, 90 },
    { 0x3af7ad2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb007ad39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa07ad43, 66 },
    { 0, 0 },
    { 0xab07ad45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d57ad48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa7ad51, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77ad5d, 98 },
    { 0x4b97ad5e, 34 },
    { 0x6947ad5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c17ad63, 90 },
    { 0, 0 },
    { 0x3ee7ad65, 87 },
    { 0xb337ad65, 61 },
    { 0, 0 },
    { 0x86e7ad68, 67 },
    { 0x8c37ad69, 84 },
    { 0xd767ad68, 93 },
    { 0x84d7ad6b, 90 },
    { 0x9cd7ad6b, 33 },
    { 0xda37ad6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4197ad7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb7ad7e, 33 },
    { 0x33a7ad7e, 55 },
    { 0, 0 },
    { 0x1d37ad81, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd27ad84, 33 },
    { 0x6837ad85, 33 },
    { 0xe7d7ad85, 23 },
    { 0x77a7ad85, 23 },
    { 0x6f07ad88, 43 },
    { 0, 0 },
    { 0xf2a7ad8a, 88 },
    { 0xf697ad8a, 33 },
    { 0xd347ad8c, 23 },
    { 0, 0 },
    { 0xa637ad8e, 55 },
    { 0x4807ad8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47ad96, 83 },
    { 0x777ad96, 95 },
    { 0, 0 },
    { 0x3447ad99, 34 },
    { 0x7837ad99, 87 },
    { 0x84e7ad9b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b87ad9e, 84 },
    { 0x8957ad9f, 90 },
    { 0, 0 },
    { 0xe057ada1, 55 },
    { 0x13d7ada2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee17ada7, 98 },
    { 0, 0 },
    { 0xf487ada9, 98 },
    { 0, 0 },
    { 0x9e27adab, 85 },
    { 0xc897adac, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d47adaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa7adbc, 76 },
    { 0, 0 },
    { 0x9b87adbe, 98 },
    { 0x37c7adbf, 66 },
    { 0x2937adc0, 34 },
    { 0xa8c7adbe, 33 },
    { 0x2d17adbe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2617adc5, 98 },
    { 0x5bb7adc6, 52 },
    { 0x93c7adc6, 95 },
    { 0x7917adc8, 95 },
    { 0xb217adc5, 55 },
    { 0xe2a7adc5, 33 },
    { 0xbf47adca, 43 },
    { 0x3f87adcc, 87 },
    { 0, 0 },
    { 0xe587adce, 34 },
    { 0x5e97adcf, 95 },
    { 0, 0 },
    { 0x1ad7add1, 90 },
    { 0xab27add2, 23 },
    { 0xe247add2, 61 },
    { 0xfbc7add3, 76 },
    { 0, 0 },
    { 0x1c17add6, 98 },
    { 0x1d87add7, 23 },
    { 0, 0 },
    { 0x9857add9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5297addc, 52 },
    { 0x23c7addd, 55 },
    { 0xed57adde, 34 },
    { 0x30a7addf, 90 },
    { 0x8a77ade0, 76 },
    { 0, 0 },
    { 0xe1f7ade2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb37ade5, 76 },
    { 0x2287ade6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8227ade9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e47adec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77adef, 33 },
    { 0, 0 },
    { 0x57b7adf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe637adf4, 95 },
    { 0xeb7adf4, 90 },
    { 0x1697adf6, 23 },
    { 0xd6c7adf6, 90 },
    { 0x1117adf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17adfb, 98 },
    { 0x367adfc, 61 },
    { 0x88f7adfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee7ae05, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee27ae0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2947ae0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f47ae13, 43 },
    { 0x18a7ae13, 85 },
    { 0x83f7ae15, 53 },
    { 0x6b67ae16, 72 },
    { 0xfc07ae16, 90 },
    { 0x4d27ae16, 100 },
    { 0xe317ae16, 67 },
    { 0x6d07ae1a, 46 },
    { 0xccd7ae1a, 61 },
    { 0, 0 },
    { 0x38d7ae1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e97ae20, 23 },
    { 0x5387ae20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7587ae2a, 61 },
    { 0xb6b7ae2b, 90 },
    { 0xaeb7ae2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf507ae30, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7577ae33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd7ae36, 46 },
    { 0, 0 },
    { 0x98b7ae38, 87 },
    { 0xec57ae38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567ae40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf707ae44, 76 },
    { 0xb697ae45, 33 },
    { 0x38b7ae44, 61 },
    { 0, 0 },
    { 0xc657ae48, 66 },
    { 0, 0 },
    { 0x507ae4a, 78 },
    { 0, 0 },
    { 0x427ae4c, 84 },
    { 0x4f57ae4d, 90 },
    { 0, 0 },
    { 0x9317ae4f, 55 },
    { 0x1677ae4f, 90 },
    { 0x3787ae50, 34 },
    { 0xa667ae52, 75 },
    { 0x627ae50, 34 },
    { 0x5177ae54, 65 },
    { 0, 0 },
    { 0xba17ae56, 55 },
    { 0, 0 },
    { 0xbe37ae58, 90 },
    { 0x8cb7ae59, 90 },
    { 0x97ae5a, 95 },
    { 0, 0 },
    { 0xf377ae5c, 52 },
    { 0xf647ae5c, 95 },
    { 0xcd7ae5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf37ae64, 76 },
    { 0x5267ae65, 23 },
    { 0xf187ae65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e67ae6b, 77 },
    { 0x917ae6b, 33 },
    { 0x7577ae6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b97ae70, 98 },
    { 0xf037ae71, 98 },
    { 0x2cd7ae71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc7ae76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf897ae7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97ae80, 98 },
    { 0, 0 },
    { 0x5397ae82, 49 },
    { 0xf797ae82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd27ae8d, 55 },
    { 0, 0 },
    { 0xdab7ae8f, 55 },
    { 0xccd7ae8f, 98 },
    { 0, 0 },
    { 0xe837ae92, 23 },
    { 0x6e77ae93, 61 },
    { 0x5487ae94, 61 },
    { 0x1157ae93, 99 },
    { 0x45e7ae96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf897ae9c, 3 },
    { 0, 0 },
    { 0x6e7ae9e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa957aea7, 99 },
    { 0, 0 },
    { 0x8587aea9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc207aeb0, 33 },
    { 0x40d7aeb1, 23 },
    { 0, 0 },
    { 0x8707aeb3, 90 },
    { 0x22d7aeb4, 23 },
    { 0xbd87aeb5, 90 },
    { 0x6277aeb4, 88 },
    { 0, 0 },
    { 0x4e07aeb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc107aebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf917aebf, 87 },
    { 0x1f17aebf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c7aec4, 87 },
    { 0, 0 },
    { 0x9f7aec6, 33 },
    { 0x2217aec6, 98 },
    { 0, 0 },
    { 0xcb27aec9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a87aecc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5067aed3, 65 },
    { 0xdc17aed3, 55 },
    { 0xd3b7aed5, 61 },
    { 0, 0 },
    { 0xe7e7aed7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c7aeda, 98 },
    { 0xb257aedb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7057aede, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf077aee2, 33 },
    { 0, 0 },
    { 0xc387aee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a07aeea, 33 },
    { 0x5757aeeb, 76 },
    { 0xc427aeec, 90 },
    { 0x1eb7aeed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c7aef2, 61 },
    { 0x4c57aef2, 98 },
    { 0xfeb7aef4, 90 },
    { 0xcdb7aef4, 85 },
    { 0xe07aef6, 100 },
    { 0x4e87aef7, 33 },
    { 0xc577aef8, 61 },
    { 0x76e7aef6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c7aefc, 55 },
    { 0x6a57aefd, 90 },
    { 0x4ae7aefe, 43 },
    { 0x8007aeff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd617af0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27af0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c7af13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd37af17, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d7af1b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb417af21, 90 },
    { 0x50d7af22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f27af28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x307af2b, 33 },
    { 0xfe27af2c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f7af30, 90 },
    { 0xdc27af31, 33 },
    { 0x877af32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147af43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a57af48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xced7af4b, 93 },
    { 0x21c7af4c, 90 },
    { 0xdd87af4d, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x4957af50, 33 },
    { 0x697af51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7af54, 68 },
    { 0, 0 },
    { 0xb3b7af56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc827af5c, 90 },
    { 0, 0 },
    { 0xa4e7af5e, 76 },
    { 0, 0 },
    { 0x77d7af60, 67 },
    { 0x5757af61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f07af69, 61 },
    { 0x7c57af6a, 55 },
    { 0x61d7af6a, 61 },
    { 0x627af6b, 76 },
    { 0xff97af6c, 75 },
    { 0x6527af6d, 33 },
    { 0, 0 },
    { 0x3817af70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a57af73, 23 },
    { 0x3b97af74, 55 },
    { 0x50e7af73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c7af80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e77af86, 84 },
    { 0xe7e7af86, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf807af8a, 23 },
    { 0xfe27af8b, 78 },
    { 0x6747af8c, 33 },
    { 0, 0 },
    { 0x2697af8e, 34 },
    { 0xdcf7af8f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe427af97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e37afa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe7afa3, 61 },
    { 0x6187afa4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef7afa9, 81 },
    { 0x3f47afa9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67afaf, 33 },
    { 0, 0 },
    { 0x3c47afb1, 23 },
    { 0xc4b7afb2, 90 },
    { 0x5c87afb2, 23 },
    { 0xfb37afb3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e77afbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe7afc4, 90 },
    { 0x5f7afc5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d57afc8, 93 },
    { 0x6637afc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa847afd4, 98 },
    { 0, 0 },
    { 0xee67afd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8537afd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa317afdd, 33 },
    { 0x85a7afdd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817afe3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd7afea, 46 },
    { 0, 0 },
    { 0xfca7afec, 73 },
    { 0x8d07afec, 95 },
    { 0xb1f7afee, 66 },
    { 0x7547afef, 33 },
    { 0x2a47aff0, 23 },
    { 0x3fd7aff0, 95 },
    { 0x51e7afec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x307aff5, 66 },
    { 0xb2f7aff6, 76 },
    { 0x6dd7aff7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa7affa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9687affd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e47b009, 101 },
    { 0xd5f7b00a, 61 },
    { 0xeea7b00b, 95 },
    { 0xb407b009, 61 },
    { 0xa117b00d, 33 },
    { 0x3f17b00e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7b016, 87 },
    { 0x1f97b017, 95 },
    { 0, 0 },
    { 0x9a7b019, 98 },
    { 0, 0 },
    { 0x5af7b01b, 90 },
    { 0x3177b01c, 95 },
    { 0, 0 },
    { 0x4b37b01e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7b021, 99 },
    { 0, 0 },
    { 0x4f17b023, 78 },
    { 0x9c07b024, 95 },
    { 0x6b37b025, 33 },
    { 0x4db7b026, 46 },
    { 0x7157b024, 61 },
    { 0x6b17b028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada7b02e, 33 },
    { 0, 0 },
    { 0x6657b030, 90 },
    { 0xdd87b031, 55 },
    { 0x7b032, 100 },
    { 0x2a37b033, 61 },
    { 0x4fb7b034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d7b037, 93 },
    { 0, 0 },
    { 0x35e7b039, 95 },
    { 0xca57b03a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907b041, 34 },
    { 0xeaa7b042, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf7b049, 33 },
    { 0x7237b04a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe747b04f, 38 },
    { 0, 0 },
    { 0xfcf7b051, 95 },
    { 0xc457b052, 95 },
    { 0x66f7b051, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd7b056, 66 },
    { 0, 0 },
    { 0xbcc7b058, 65 },
    { 0x6817b058, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c7b05e, 95 },
    { 0x9407b05e, 98 },
    { 0xe587b060, 95 },
    { 0, 0 },
    { 0xc587b062, 76 },
    { 0x4b7b063, 75 },
    { 0xa617b063, 68 },
    { 0xb57b063, 61 },
    { 0, 0 },
    { 0x4397b067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877b06a, 98 },
    { 0xf277b06b, 61 },
    { 0, 0 },
    { 0x9187b06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7b071, 9 },
    { 0xe117b071, 90 },
    { 0xcb47b073, 76 },
    { 0x9a77b071, 43 },
    { 0xc797b071, 90 },
    { 0, 0 },
    { 0x8087b077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc957b07b, 91 },
    { 0x9d27b07c, 23 },
    { 0, 0 },
    { 0x92d7b07e, 69 },
    { 0x1fd7b07f, 77 },
    { 0x7cf7b080, 55 },
    { 0xbb57b080, 61 },
    { 0xe17b082, 81 },
    { 0, 0 },
    { 0xe447b084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767b087, 61 },
    { 0x7717b088, 90 },
    { 0xc617b088, 95 },
    { 0xe6a7b088, 84 },
    { 0xdfa7b089, 95 },
    { 0, 0 },
    { 0x4f67b08d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa197b094, 85 },
    { 0, 0 },
    { 0x1ac7b096, 68 },
    { 0x7d37b096, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f57b09c, 66 },
    { 0, 0 },
    { 0x5307b09e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xed57b0a1, 23 },
    { 0xf5f7b0a2, 43 },
    { 0xaa97b0a3, 81 },
    { 0x4f77b0a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa977b0a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe237b0ac, 95 },
    { 0x8f87b0ad, 34 },
    { 0, 0 },
    { 0xb697b0af, 75 },
    { 0, 0 },
    { 0x4247b0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127b0b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77b0bd, 61 },
    { 0x3ec7b0bd, 33 },
    { 0xf867b0bd, 90 },
    { 0x3da7b0c0, 77 },
    { 0x53a7b0be, 42 },
    { 0x7307b0c2, 76 },
    { 0xe1a7b0c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b7b0c6, 99 },
    { 0x57f7b0c7, 90 },
    { 0, 0 },
    { 0x11e7b0c9, 78 },
    { 0x9d57b0ca, 98 },
    { 0, 0 },
    { 0xe4a7b0cc, 55 },
    { 0, 0 },
    { 0x2af7b0ce, 39 },
    { 0, 0 },
    { 0x93e7b0d0, 100 },
    { 0, 0 },
    { 0x2237b0d2, 46 },
    { 0x94a7b0d3, 93 },
    { 0xef97b0d2, 85 },
    { 0xf9d7b0d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1897b0d8, 95 },
    { 0x6297b0d9, 69 },
    { 0x4d97b0da, 95 },
    { 0x57e7b0d8, 91 },
    { 0xc317b0da, 55 },
    { 0x1087b0dc, 48 },
    { 0xd537b0de, 46 },
    { 0x5a97b0d8, 96 },
    { 0xe727b0e0, 78 },
    { 0, 0 },
    { 0x4aa7b0e2, 90 },
    { 0x4297b0e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4987b0e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e97b0e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe7b0ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f7b0f1, 85 },
    { 0x4277b0f2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e7b0f5, 98 },
    { 0xda87b0f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a77b0fb, 96 },
    { 0, 0 },
    { 0x3307b0fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1917b102, 23 },
    { 0x2f37b102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c7b106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3207b10c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa227b10f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df7b115, 95 },
    { 0x8927b116, 77 },
    { 0x5b67b117, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c77b11a, 33 },
    { 0x8b67b11a, 33 },
    { 0xa057b11a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb7b11f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd787b125, 34 },
    { 0, 0 },
    { 0x8277b127, 33 },
    { 0x1b97b128, 23 },
    { 0x6f47b129, 93 },
    { 0xfc77b12a, 81 },
    { 0x53b7b12b, 34 },
    { 0xb587b12c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877b12f, 84 },
    { 0x5377b130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c87b137, 61 },
    { 0x937b138, 33 },
    { 0x5f17b139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b141, 34 },
    { 0, 0 },
    { 0xa9f7b143, 97 },
    { 0, 0 },
    { 0x3e47b145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc67b149, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd357b14f, 33 },
    { 0x1c17b150, 93 },
    { 0x60b7b151, 61 },
    { 0, 0 },
    { 0xcba7b153, 76 },
    { 0x2087b154, 23 },
    { 0x69e7b153, 91 },
    { 0x7e07b156, 98 },
    { 0x877b155, 17 },
    { 0xbf7b154, 76 },
    { 0xe477b153, 76 },
    { 0, 0 },
    { 0xc107b15b, 87 },
    { 0xc587b15c, 90 },
    { 0x33b7b15b, 77 },
    { 0x6cc7b15e, 52 },
    { 0xfb37b15b, 23 },
    { 0x9587b15b, 61 },
    { 0xce37b15e, 95 },
    { 0xa0a7b162, 85 },
    { 0xb717b161, 33 },
    { 0x45b7b164, 61 },
    { 0x1907b15d, 95 },
    { 0xcaa7b166, 98 },
    { 0x1dc7b167, 55 },
    { 0x7477b164, 34 },
    { 0, 0 },
    { 0x6d97b16a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227b170, 98 },
    { 0, 0 },
    { 0x1c67b172, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd237b178, 33 },
    { 0, 0 },
    { 0x41b7b17a, 61 },
    { 0, 0 },
    { 0xd267b17c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5447b181, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1377b187, 46 },
    { 0xd7f7b188, 98 },
    { 0x5ef7b189, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6497b18c, 98 },
    { 0, 0 },
    { 0x27d7b18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8177b192, 80 },
    { 0x63f7b193, 95 },
    { 0xc5a7b193, 91 },
    { 0x7e17b195, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf27b198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc7b19d, 90 },
    { 0x9d77b19e, 33 },
    { 0x4ba7b19f, 98 },
    { 0x88d7b1a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e7b1a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d47b1a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437b1af, 33 },
    { 0, 0 },
    { 0xc217b1b1, 72 },
    { 0xb627b1b2, 48 },
    { 0, 0 },
    { 0x147b1b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d7b1b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7187b1bd, 98 },
    { 0xbd67b1be, 93 },
    { 0x7177b1bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7b1c7, 93 },
    { 0xabe7b1c8, 56 },
    { 0x1917b1c9, 40 },
    { 0xe1a7b1ca, 23 },
    { 0, 0 },
    { 0x6b07b1cc, 38 },
    { 0x89f7b1cd, 61 },
    { 0, 0 },
    { 0x22e7b1cf, 93 },
    { 0x2607b1cf, 61 },
    { 0, 0 },
    { 0x32e7b1d2, 99 },
    { 0x7687b1d3, 87 },
    { 0x1c7b1d4, 87 },
    { 0, 0 },
    { 0x2b57b1d6, 75 },
    { 0x2767b1d7, 78 },
    { 0, 0 },
    { 0x8ab7b1d9, 95 },
    { 0x6257b1d9, 23 },
    { 0x79e7b1db, 23 },
    { 0x7557b1dc, 69 },
    { 0, 0 },
    { 0x7757b1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe7b1e4, 88 },
    { 0, 0 },
    { 0x80f7b1e6, 90 },
    { 0x3f97b1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17b1ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b7b1f2, 23 },
    { 0x2ba7b1f3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e7b1f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f7b1fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8347b200, 87 },
    { 0xf6c7b201, 95 },
    { 0x47c7b201, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e37b20c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf27b20f, 75 },
    { 0, 0 },
    { 0xc597b211, 38 },
    { 0xafb7b212, 23 },
    { 0xbd27b213, 95 },
    { 0xf477b211, 23 },
    { 0x3347b212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7b21a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f7b21d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7b220, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8077b225, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6577b229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd7b22e, 93 },
    { 0, 0 },
    { 0xf6b7b230, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb337b235, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec97b23a, 98 },
    { 0, 0 },
    { 0xfa77b23c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd7b23f, 33 },
    { 0x5d7b240, 66 },
    { 0xd577b240, 93 },
    { 0x3707b240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6697b246, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b249, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8247b24c, 98 },
    { 0, 0 },
    { 0xac37b24e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c7b254, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4047b258, 8 },
    { 0x17b258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf47b25e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc7b264, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa87b267, 95 },
    { 0, 0 },
    { 0x2ad7b269, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27b26e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d7b275, 55 },
    { 0x7097b275, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577b27f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf787b282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9157b286, 91 },
    { 0x49e7b287, 46 },
    { 0x8047b287, 92 },
    { 0xc97b289, 98 },
    { 0, 0 },
    { 0xefb7b28b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8617b28f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c7b293, 61 },
    { 0xf9f7b294, 23 },
    { 0x8997b295, 33 },
    { 0xd657b295, 55 },
    { 0x3597b294, 34 },
    { 0, 0 },
    { 0xc597b299, 66 },
    { 0xb137b29a, 66 },
    { 0x14f7b299, 90 },
    { 0x8bc7b29a, 33 },
    { 0, 0 },
    { 0xaf77b29e, 95 },
    { 0xe247b29f, 33 },
    { 0xfba7b29f, 98 },
    { 0x2097b2a1, 90 },
    { 0x81a7b2a2, 46 },
    { 0xa597b2a2, 76 },
    { 0, 0 },
    { 0xe447b2a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd657b2a9, 76 },
    { 0, 0 },
    { 0x8847b2ab, 68 },
    { 0xc497b2ac, 34 },
    { 0xb3e7b2ac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be7b2b8, 95 },
    { 0, 0 },
    { 0xb757b2ba, 98 },
    { 0x1877b2bb, 84 },
    { 0x5da7b2bc, 100 },
    { 0x3547b2bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb37b2c0, 91 },
    { 0x10d7b2c1, 61 },
    { 0x29b7b2c1, 52 },
    { 0, 0 },
    { 0xda27b2c4, 90 },
    { 0xd387b2c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe87b2c8, 55 },
    { 0, 0 },
    { 0xb357b2ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9887b2d5, 100 },
    { 0x87f7b2d5, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7617b2da, 61 },
    { 0xebe7b2db, 23 },
    { 0xa297b2dc, 66 },
    { 0xadd7b2da, 43 },
    { 0xaf97b2de, 38 },
    { 0, 0 },
    { 0xdc87b2e0, 55 },
    { 0, 0 },
    { 0x6ed7b2e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa477b2e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f7b2f1, 84 },
    { 0xbe67b2f2, 61 },
    { 0x7947b2f2, 33 },
    { 0, 0 },
    { 0x99c7b2f5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d7b2ff, 94 },
    { 0x4f47b2ff, 34 },
    { 0, 0 },
    { 0x367b302, 23 },
    { 0xcca7b303, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07b306, 43 },
    { 0, 0 },
    { 0x23e7b308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b77b30c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7b310, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf427b323, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc7b326, 81 },
    { 0x4ac7b327, 67 },
    { 0xa77b327, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f7b331, 9 },
    { 0xe3c7b332, 85 },
    { 0x6787b333, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf7b336, 40 },
    { 0xb247b336, 93 },
    { 0, 0 },
    { 0xfb47b339, 90 },
    { 0x8317b33a, 98 },
    { 0, 0 },
    { 0xefd7b33c, 76 },
    { 0x1857b33d, 98 },
    { 0x45d7b33e, 95 },
    { 0xfba7b33f, 23 },
    { 0x99a7b33e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8607b344, 95 },
    { 0x5177b345, 98 },
    { 0xb7e7b346, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7917b34b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae97b34e, 33 },
    { 0, 0 },
    { 0xae77b350, 90 },
    { 0x47b351, 91 },
    { 0xd2a7b351, 33 },
    { 0, 0 },
    { 0x76c7b354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9977b358, 61 },
    { 0x8507b358, 23 },
    { 0x86c7b35a, 88 },
    { 0, 0 },
    { 0xdb67b35c, 90 },
    { 0x6d77b35d, 61 },
    { 0x1c37b35e, 95 },
    { 0xa77b35e, 68 },
    { 0x9a87b35e, 17 },
    { 0, 0 },
    { 0xeb17b362, 90 },
    { 0, 0 },
    { 0x1a47b364, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe057b368, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb7b36c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6067b371, 95 },
    { 0x8257b372, 33 },
    { 0x5517b373, 90 },
    { 0x9fd7b373, 98 },
    { 0, 0 },
    { 0x38c7b376, 23 },
    { 0, 0 },
    { 0x5d97b378, 99 },
    { 0x5827b378, 90 },
    { 0, 0 },
    { 0x1e37b37b, 84 },
    { 0x4cd7b37c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf7b382, 98 },
    { 0, 0 },
    { 0x4407b384, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9107b387, 34 },
    { 0, 0 },
    { 0x8c47b389, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137b38d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e7b395, 83 },
    { 0, 0 },
    { 0xf7f7b397, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7b39d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea7b3a3, 34 },
    { 0x8387b3a4, 98 },
    { 0xcb17b3a5, 95 },
    { 0x9027b3a6, 61 },
    { 0, 0 },
    { 0x6b47b3a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a77b3ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9027b3b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e7b3b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b7b3b8, 17 },
    { 0x5d57b3b8, 33 },
    { 0xfe07b3ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf7b3bd, 55 },
    { 0x9737b3bd, 95 },
    { 0x98b7b3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff07b3c3, 90 },
    { 0x7767b3c4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e7b3ca, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x967b3cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb07b3d1, 66 },
    { 0xa3c7b3d1, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a7b3d7, 43 },
    { 0xb577b3d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef7b3db, 23 },
    { 0, 0 },
    { 0xd037b3dd, 61 },
    { 0, 0 },
    { 0xec97b3df, 33 },
    { 0, 0 },
    { 0xb027b3e1, 66 },
    { 0, 0 },
    { 0x2367b3e3, 33 },
    { 0x6337b3e3, 95 },
    { 0x6b77b3e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde7b3ef, 93 },
    { 0xeca7b3f0, 46 },
    { 0, 0 },
    { 0x23e7b3f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d7b3f8, 53 },
    { 0x9057b3f9, 67 },
    { 0x88f7b3f8, 99 },
    { 0x77b7b3f8, 61 },
    { 0, 0 },
    { 0x4037b3fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b7b401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f17b405, 95 },
    { 0xd717b406, 52 },
    { 0x2587b407, 100 },
    { 0xdef7b408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea37b411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7387b414, 17 },
    { 0xf1f7b415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127b41d, 78 },
    { 0x4917b41e, 95 },
    { 0, 0 },
    { 0x537b420, 23 },
    { 0x8af7b420, 95 },
    { 0xa727b422, 17 },
    { 0, 0 },
    { 0x34d7b424, 85 },
    { 0xc907b424, 34 },
    { 0, 0 },
    { 0x4057b427, 43 },
    { 0xf2a7b428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd227b42e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e17b431, 52 },
    { 0x1dd7b431, 52 },
    { 0xf977b433, 76 },
    { 0xdb07b433, 76 },
    { 0xcc97b434, 93 },
    { 0, 0 },
    { 0xb727b437, 75 },
    { 0xb8a7b438, 46 },
    { 0, 0 },
    { 0x4a97b43a, 76 },
    { 0x1c27b43b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc497b43e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b442, 55 },
    { 0x8237b442, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf77b448, 33 },
    { 0xbaf7b448, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4157b44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd417b453, 90 },
    { 0x12f7b454, 87 },
    { 0, 0 },
    { 0x5157b456, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc927b45b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3487b460, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e87b463, 34 },
    { 0x5a07b463, 66 },
    { 0, 0 },
    { 0xbc47b466, 100 },
    { 0xfa17b466, 93 },
    { 0x7897b468, 88 },
    { 0x9677b469, 55 },
    { 0x74e7b46a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57b46d, 76 },
    { 0x1d37b46e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc7b473, 98 },
    { 0x8817b474, 33 },
    { 0x60e7b475, 87 },
    { 0xc577b473, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d7b47b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e7b47e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa77b481, 52 },
    { 0xf1b7b482, 55 },
    { 0x3aa7b483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d7b489, 95 },
    { 0, 0 },
    { 0x24a7b48b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af7b48f, 61 },
    { 0x4847b48f, 95 },
    { 0x12b7b491, 76 },
    { 0x2ec7b491, 93 },
    { 0x9547b490, 88 },
    { 0xeeb7b48f, 90 },
    { 0x3bb7b48f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea7b49a, 61 },
    { 0, 0 },
    { 0xe1b7b49c, 33 },
    { 0x41a7b49c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9687b4a2, 93 },
    { 0x9f57b4a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff17b4a9, 43 },
    { 0, 0 },
    { 0x1bd7b4ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f97b4af, 80 },
    { 0, 0 },
    { 0xc8f7b4b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7b4b6, 99 },
    { 0x9507b4b6, 52 },
    { 0xd2d7b4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa337b4bf, 69 },
    { 0xda47b4c0, 75 },
    { 0, 0 },
    { 0xa3c7b4c2, 66 },
    { 0x2907b4c2, 61 },
    { 0x9197b4c4, 88 },
    { 0xdba7b4c5, 52 },
    { 0x1817b4c3, 23 },
    { 0xe947b4c7, 98 },
    { 0, 0 },
    { 0x31a7b4c9, 98 },
    { 0x30c7b4ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed7b4cd, 43 },
    { 0x2ac7b4cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc067b4d7, 34 },
    { 0xf237b4d7, 95 },
    { 0x78d7b4d9, 33 },
    { 0x49f7b4da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab7b4de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa7b4e3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe887b4e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd477b4ee, 90 },
    { 0, 0 },
    { 0xcd77b4f0, 87 },
    { 0xc257b4f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a37b4fa, 61 },
    { 0x7127b4fb, 23 },
    { 0xc307b4fc, 73 },
    { 0xaf57b4fd, 23 },
    { 0xfc17b4fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab67b502, 90 },
    { 0xea27b503, 95 },
    { 0xf8c7b504, 95 },
    { 0x2367b502, 33 },
    { 0, 0 },
    { 0xf8a7b507, 98 },
    { 0xaeb7b507, 25 },
    { 0x55a7b509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8317b50c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7b510, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e7b514, 33 },
    { 0xb307b515, 55 },
    { 0xfa97b515, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7b51b, 43 },
    { 0, 0 },
    { 0xa5d7b51d, 95 },
    { 0x48e7b51e, 100 },
    { 0x2cf7b51f, 66 },
    { 0xabb7b520, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4927b523, 100 },
    { 0x49c7b524, 76 },
    { 0x3407b525, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed7b530, 53 },
    { 0x2307b531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d7b535, 33 },
    { 0xcf57b535, 91 },
    { 0x32b7b537, 34 },
    { 0x9997b535, 61 },
    { 0x1037b539, 36 },
    { 0x4ae7b539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa517b540, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae7b543, 34 },
    { 0, 0 },
    { 0x1ff7b545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4867b548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c27b550, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6837b553, 61 },
    { 0, 0 },
    { 0xf397b555, 76 },
    { 0x1dc7b556, 61 },
    { 0x9ed7b557, 66 },
    { 0x2997b557, 98 },
    { 0x767b557, 65 },
    { 0x3637b55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc897b562, 23 },
    { 0x3127b562, 61 },
    { 0xdfb7b564, 61 },
    { 0x6ab7b564, 23 },
    { 0xfce7b564, 55 },
    { 0x8ea7b567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec7b56a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8517b56e, 87 },
    { 0xc557b56f, 98 },
    { 0xa587b56e, 84 },
    { 0, 0 },
    { 0xd07b572, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5727b576, 52 },
    { 0xa557b577, 92 },
    { 0x8637b578, 33 },
    { 0x9677b577, 76 },
    { 0x22e7b57a, 98 },
    { 0x2507b57a, 98 },
    { 0x3a77b57a, 99 },
    { 0x257b57a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2657b580, 61 },
    { 0x7617b580, 55 },
    { 0xd187b582, 34 },
    { 0x7857b583, 33 },
    { 0xf787b584, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf917b58c, 84 },
    { 0xbf7b58d, 84 },
    { 0xa977b58e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b87b591, 34 },
    { 0xa047b592, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2547b59f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc77b5a4, 77 },
    { 0xdb7b5a5, 33 },
    { 0x7ec7b5a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c7b5a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c7b5ac, 61 },
    { 0x1fa7b5ad, 95 },
    { 0x9fd7b5ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97b5b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee7b5ba, 23 },
    { 0, 0 },
    { 0x4ad7b5bc, 95 },
    { 0, 0 },
    { 0x2027b5be, 90 },
    { 0, 0 },
    { 0x5f87b5c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e7b5c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce7b5cf, 90 },
    { 0xa827b5d0, 55 },
    { 0, 0 },
    { 0x50a7b5d2, 52 },
    { 0xdb17b5d2, 52 },
    { 0, 0 },
    { 0xec67b5d5, 90 },
    { 0xc927b5d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5517b5db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f27b5e0, 78 },
    { 0x1237b5e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d07b5e4, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe387b5ea, 23 },
    { 0x1987b5eb, 85 },
    { 0x53a7b5ea, 33 },
    { 0x64e7b5ea, 90 },
    { 0x3537b5ee, 77 },
    { 0xf237b5ee, 88 },
    { 0x7a7b5ec, 23 },
    { 0x8a37b5f1, 61 },
    { 0x1817b5f1, 34 },
    { 0xb467b5ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d7b5f7, 90 },
    { 0x7e87b5f7, 23 },
    { 0x1757b5f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec7b5fc, 23 },
    { 0x95d7b5fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8487b602, 17 },
    { 0xc237b603, 91 },
    { 0xa9e7b603, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc7b608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d7b60b, 87 },
    { 0, 0 },
    { 0xc4d7b60d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd347b611, 95 },
    { 0, 0 },
    { 0x6e97b613, 46 },
    { 0xb407b613, 99 },
    { 0xcf57b613, 90 },
    { 0xf797b616, 90 },
    { 0x5e97b617, 95 },
    { 0xcf17b614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c57b61e, 98 },
    { 0xecb7b61f, 33 },
    { 0, 0 },
    { 0x387b621, 98 },
    { 0x5af7b622, 98 },
    { 0, 0 },
    { 0x6787b624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7727b627, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd97b62c, 78 },
    { 0xc827b62c, 34 },
    { 0x3937b62d, 34 },
    { 0, 0 },
    { 0xb37b630, 90 },
    { 0xdb47b631, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab7b634, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e7b638, 90 },
    { 0, 0 },
    { 0x5df7b63a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef97b63d, 99 },
    { 0x1997b63e, 99 },
    { 0xbf97b63f, 61 },
    { 0x927b63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb407b643, 61 },
    { 0xf467b643, 94 },
    { 0xe47b645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b97b648, 23 },
    { 0x32c7b648, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7b64d, 91 },
    { 0xb567b64e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad17b652, 43 },
    { 0x88b7b652, 95 },
    { 0xb8c7b654, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb7b658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787b65e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787b666, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d7b669, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e7b66c, 66 },
    { 0x4057b66c, 91 },
    { 0xf597b66e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f7b679, 33 },
    { 0xa957b679, 95 },
    { 0xa5a7b67b, 92 },
    { 0xec7b67c, 61 },
    { 0x3257b67d, 95 },
    { 0x17e7b67c, 23 },
    { 0x3487b67f, 55 },
    { 0, 0 },
    { 0xb7c7b681, 45 },
    { 0xe147b681, 46 },
    { 0x3927b683, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd7b689, 34 },
    { 0x22f7b689, 34 },
    { 0xd257b68b, 95 },
    { 0xf567b68c, 23 },
    { 0xbae7b68b, 66 },
    { 0x1be7b68a, 90 },
    { 0xa1f7b68f, 98 },
    { 0x3f87b68f, 61 },
    { 0x53e7b691, 98 },
    { 0xdce7b692, 61 },
    { 0xffe7b693, 88 },
    { 0, 0 },
    { 0xd5c7b695, 46 },
    { 0x4cf7b696, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf7b69a, 33 },
    { 0xcae7b69a, 90 },
    { 0x7467b69a, 76 },
    { 0x35d7b69b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3037b6a2, 34 },
    { 0, 0 },
    { 0x5e7b6a4, 33 },
    { 0, 0 },
    { 0x2527b6a6, 99 },
    { 0, 0 },
    { 0x2e87b6a8, 98 },
    { 0xc0b7b6a8, 75 },
    { 0x5ab7b6aa, 46 },
    { 0x1557b6ab, 33 },
    { 0xa427b6a9, 52 },
    { 0x1f77b6ad, 55 },
    { 0xd7f7b6ac, 33 },
    { 0xf277b6ad, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2427b6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e7b6b9, 87 },
    { 0xd4b7b6ba, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b27b6bf, 90 },
    { 0, 0 },
    { 0x7737b6c1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd7b6c4, 98 },
    { 0x7ef7b6c4, 52 },
    { 0, 0 },
    { 0x9507b6c7, 76 },
    { 0x64f7b6c8, 61 },
    { 0xa567b6c9, 90 },
    { 0x4a57b6c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d17b6cd, 33 },
    { 0x6dd7b6cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb7b6d2, 34 },
    { 0x18f7b6d3, 95 },
    { 0, 0 },
    { 0x3fc7b6d5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557b6da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e7b6e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d87b6e5, 37 },
    { 0xf5e7b6e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c47b6eb, 23 },
    { 0xab97b6ec, 33 },
    { 0x6027b6ec, 99 },
    { 0x97e7b6eb, 33 },
    { 0xde17b6eb, 55 },
    { 0x4427b6f0, 90 },
    { 0xb067b6f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b27b6f4, 76 },
    { 0x4347b6f5, 90 },
    { 0x6a67b6f6, 43 },
    { 0xe357b6f7, 33 },
    { 0, 0 },
    { 0x5f47b6f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8697b6fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c07b703, 99 },
    { 0x2537b704, 90 },
    { 0xb9d7b705, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e57b708, 90 },
    { 0, 0 },
    { 0x5c77b70a, 90 },
    { 0, 0 },
    { 0x9257b70c, 23 },
    { 0x6d7b70d, 61 },
    { 0x7ae7b70d, 80 },
    { 0x63a7b70d, 85 },
    { 0xf277b710, 99 },
    { 0x47b70f, 46 },
    { 0, 0 },
    { 0xe027b713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d97b716, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a07b719, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7b71e, 61 },
    { 0, 0 },
    { 0x7ec7b720, 46 },
    { 0x85d7b720, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf17b724, 33 },
    { 0xd0f7b725, 66 },
    { 0x1d27b724, 88 },
    { 0x39e7b727, 95 },
    { 0x9a07b727, 46 },
    { 0xf667b727, 93 },
    { 0x9d37b72a, 100 },
    { 0x51f7b72a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa547b732, 61 },
    { 0, 0 },
    { 0x8ef7b734, 34 },
    { 0x1e57b734, 100 },
    { 0x5d57b735, 23 },
    { 0x6937b734, 76 },
    { 0x1f67b736, 22 },
    { 0, 0 },
    { 0x4b47b73a, 95 },
    { 0, 0 },
    { 0x10f7b73c, 90 },
    { 0, 0 },
    { 0xdfb7b73e, 98 },
    { 0x31d7b73f, 61 },
    { 0xe87b740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc147b747, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107b74e, 23 },
    { 0x1127b74f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf057b753, 95 },
    { 0x8e57b754, 76 },
    { 0, 0 },
    { 0xc667b756, 33 },
    { 0, 0 },
    { 0x7d17b758, 90 },
    { 0x3677b759, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b7b75d, 95 },
    { 0x4c57b75d, 90 },
    { 0, 0 },
    { 0x3d57b760, 33 },
    { 0x6b17b761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f67b766, 93 },
    { 0xfcd7b766, 76 },
    { 0x5e7b768, 93 },
    { 0xcd27b769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c7b76c, 87 },
    { 0xdbe7b76d, 98 },
    { 0, 0 },
    { 0x7177b76f, 23 },
    { 0x8b17b770, 98 },
    { 0x5ee7b770, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c7b775, 84 },
    { 0x1137b775, 33 },
    { 0, 0 },
    { 0xd5e7b778, 98 },
    { 0x2c07b779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17b780, 49 },
    { 0x4dd7b780, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf947b785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9287b789, 34 },
    { 0x2667b789, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf427b78f, 23 },
    { 0x1e27b78f, 46 },
    { 0xf6a7b791, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2277b794, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d7b798, 61 },
    { 0, 0 },
    { 0x4bf7b79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e67b79f, 46 },
    { 0, 0 },
    { 0x7cc7b7a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3827b7a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b77b7a7, 66 },
    { 0xd877b7a8, 61 },
    { 0x99e7b7a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e7b7b0, 95 },
    { 0x8bb7b7b1, 93 },
    { 0x407b7b2, 93 },
    { 0xbec7b7b3, 76 },
    { 0x1a87b7b3, 96 },
    { 0x52d7b7b4, 76 },
    { 0xad57b7b2, 53 },
    { 0xf527b7b2, 55 },
    { 0xde87b7b8, 87 },
    { 0x9ee7b7b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a77b7bc, 34 },
    { 0, 0 },
    { 0xbc27b7be, 98 },
    { 0, 0 },
    { 0x98f7b7c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a7b7c7, 61 },
    { 0xb4e7b7c7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2717b7cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb77b7d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3557b7d7, 61 },
    { 0xcb67b7d7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d67b7de, 23 },
    { 0, 0 },
    { 0x9ea7b7e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f7b7e3, 76 },
    { 0xacb7b7e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e7b7e8, 85 },
    { 0xa1e7b7e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07b7ec, 98 },
    { 0x2187b7ec, 23 },
    { 0xe207b7ee, 87 },
    { 0x9e07b7ed, 55 },
    { 0x1a57b7ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb27b7f4, 33 },
    { 0x4eb7b7f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b7b7f8, 95 },
    { 0x2ea7b7f9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2607b7fc, 76 },
    { 0, 0 },
    { 0x3727b7fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2717b804, 61 },
    { 0, 0 },
    { 0x4127b806, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547b809, 87 },
    { 0x45c7b809, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c7b80d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb17b815, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e7b81b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d7b81f, 59 },
    { 0, 0 },
    { 0x5c27b821, 90 },
    { 0x2f97b822, 23 },
    { 0x977b823, 24 },
    { 0x77d7b824, 100 },
    { 0x20f7b825, 95 },
    { 0xed27b826, 52 },
    { 0x9b57b827, 90 },
    { 0x5987b828, 95 },
    { 0, 0 },
    { 0xd1f7b82a, 52 },
    { 0xce7b82a, 69 },
    { 0x7217b82c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1647b830, 98 },
    { 0xf417b831, 23 },
    { 0xcb57b830, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87b835, 76 },
    { 0x4df7b835, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c7b839, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb67b83c, 90 },
    { 0x7ed7b83d, 99 },
    { 0x9267b83c, 90 },
    { 0x47f7b83f, 55 },
    { 0xca77b840, 61 },
    { 0x2f47b83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d57b844, 55 },
    { 0, 0 },
    { 0x5837b846, 77 },
    { 0x1fe7b846, 33 },
    { 0xb637b847, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b7b84d, 95 },
    { 0x4a77b84d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587b853, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7b85a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4567b861, 95 },
    { 0x5cc7b862, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff57b86a, 61 },
    { 0xafa7b86b, 90 },
    { 0x6177b86b, 75 },
    { 0x70a7b86c, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367b870, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3597b873, 33 },
    { 0xb497b874, 23 },
    { 0, 0 },
    { 0xfad7b876, 61 },
    { 0x2f67b877, 76 },
    { 0x6dd7b878, 95 },
    { 0xce17b879, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c7b87c, 52 },
    { 0xf777b87d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b7b880, 101 },
    { 0xc537b881, 100 },
    { 0xb5e7b881, 33 },
    { 0xb1c7b883, 43 },
    { 0, 0 },
    { 0xebf7b885, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe887b888, 98 },
    { 0, 0 },
    { 0xa4a7b88a, 95 },
    { 0xc8e7b88a, 99 },
    { 0xcdf7b88c, 90 },
    { 0, 0 },
    { 0x25a7b88e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27b891, 61 },
    { 0xa5a7b891, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba17b895, 61 },
    { 0x7417b896, 90 },
    { 0x3787b897, 76 },
    { 0x677b898, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6157b89f, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc477b8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c7b8a8, 61 },
    { 0, 0 },
    { 0xc427b8aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3737b8ae, 66 },
    { 0xcd97b8ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7b8b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167b8b6, 33 },
    { 0xc9f7b8b7, 98 },
    { 0, 0 },
    { 0x6da7b8b9, 43 },
    { 0xf437b8ba, 93 },
    { 0x61e7b8ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd57b8c6, 85 },
    { 0xab7b8c7, 61 },
    { 0xeb47b8c7, 33 },
    { 0xcad7b8c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7b8cc, 33 },
    { 0xeb97b8cd, 55 },
    { 0x7c47b8ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b7b8d3, 93 },
    { 0x86f7b8d4, 90 },
    { 0, 0 },
    { 0x86d7b8d6, 33 },
    { 0xd017b8d7, 61 },
    { 0x68d7b8d6, 34 },
    { 0xaa67b8d9, 34 },
    { 0x6e7b8d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577b8df, 95 },
    { 0xe117b8df, 98 },
    { 0x9f87b8e1, 98 },
    { 0x9f77b8e2, 85 },
    { 0xd8f7b8e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617b8e6, 98 },
    { 0, 0 },
    { 0x6997b8e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7127b8ed, 78 },
    { 0, 0 },
    { 0xaaa7b8ef, 98 },
    { 0, 0 },
    { 0xc8d7b8f1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8377b8f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7b8fd, 23 },
    { 0xbd77b8fe, 33 },
    { 0xde17b8ff, 34 },
    { 0x84a7b8fd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd87b903, 33 },
    { 0, 0 },
    { 0x2787b905, 90 },
    { 0, 0 },
    { 0x5487b907, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c77b90b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc017b90f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc7b914, 33 },
    { 0x6eb7b915, 90 },
    { 0xf3a7b915, 77 },
    { 0x6217b917, 55 },
    { 0x6987b918, 55 },
    { 0, 0 },
    { 0x7547b91a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd717b91e, 33 },
    { 0x4aa7b91e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec77b923, 87 },
    { 0xbc57b923, 34 },
    { 0x38e7b925, 52 },
    { 0xc677b923, 33 },
    { 0xb7b924, 55 },
    { 0xf17b925, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb7b92b, 91 },
    { 0x5d37b92b, 99 },
    { 0x6bc7b92c, 40 },
    { 0xfc77b92e, 90 },
    { 0xfa37b92c, 85 },
    { 0x9f67b930, 23 },
    { 0x6677b931, 55 },
    { 0x11b7b932, 33 },
    { 0xa657b92e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb837b937, 61 },
    { 0x1347b938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7787b93c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce97b944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7417b947, 46 },
    { 0xe947b948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1747b952, 66 },
    { 0, 0 },
    { 0xc6c7b954, 33 },
    { 0x76d7b954, 61 },
    { 0xef7b956, 66 },
    { 0x5d77b957, 43 },
    { 0, 0 },
    { 0x8827b959, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d7b95c, 100 },
    { 0x7bb7b95d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf337b964, 98 },
    { 0, 0 },
    { 0x2467b966, 98 },
    { 0xf37b967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b57b96b, 95 },
    { 0xcf07b96b, 93 },
    { 0x7067b96d, 77 },
    { 0xe977b96d, 90 },
    { 0x3337b96d, 34 },
    { 0x90f7b96f, 55 },
    { 0x12f7b971, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6157b977, 90 },
    { 0xfcb7b978, 99 },
    { 0xad77b978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x157b97c, 98 },
    { 0x6747b97c, 90 },
    { 0xabc7b97c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e67b985, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a97b98b, 91 },
    { 0xc037b98b, 90 },
    { 0x9517b98d, 61 },
    { 0xfef7b98e, 100 },
    { 0xf707b98f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf447b993, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c7b996, 95 },
    { 0, 0 },
    { 0xf567b998, 61 },
    { 0x91b7b998, 76 },
    { 0x92e7b999, 52 },
    { 0x9b77b999, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a7b9a3, 55 },
    { 0x2e27b9a4, 34 },
    { 0x2907b9a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367b9a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf217b9ae, 61 },
    { 0, 0 },
    { 0xb0c7b9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe7b9b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7367b9b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7b9ba, 33 },
    { 0x15d7b9ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e17b9bf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7b9c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c27b9c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d27b9ca, 90 },
    { 0x1c87b9cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa097b9d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc67b9df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa657b9e8, 95 },
    { 0, 0 },
    { 0x5d57b9ea, 95 },
    { 0x5d27b9eb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb7b9ef, 75 },
    { 0, 0 },
    { 0xf977b9f1, 91 },
    { 0x82a7b9f1, 76 },
    { 0x8607b9f1, 23 },
    { 0, 0 },
    { 0xd497b9f5, 34 },
    { 0xc277b9f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf247b9f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1127b9fd, 90 },
    { 0x6487b9fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9317ba03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf407ba07, 55 },
    { 0x5d27ba08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x967ba0b, 98 },
    { 0xdb67ba0c, 61 },
    { 0, 0 },
    { 0x6da7ba0e, 85 },
    { 0xbb37ba0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab37ba14, 43 },
    { 0x1067ba15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f97ba1b, 90 },
    { 0xd6e7ba1c, 23 },
    { 0, 0 },
    { 0x9b87ba1e, 34 },
    { 0x5fd7ba1f, 66 },
    { 0xa5f7ba20, 33 },
    { 0xc337ba21, 95 },
    { 0x54e7ba22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae7ba28, 85 },
    { 0xd5e7ba28, 99 },
    { 0x1dc7ba2a, 98 },
    { 0, 0 },
    { 0xff07ba2c, 78 },
    { 0x1d67ba2d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc957ba31, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd7ba36, 61 },
    { 0, 0 },
    { 0x18a7ba38, 90 },
    { 0x7e47ba39, 59 },
    { 0x8a47ba3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27ba40, 61 },
    { 0x7347ba41, 90 },
    { 0x8557ba41, 90 },
    { 0x5fe7ba43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba7ba46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e87ba50, 98 },
    { 0, 0 },
    { 0x6a17ba52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac47ba55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb77ba58, 87 },
    { 0x4bf7ba59, 85 },
    { 0x4837ba5a, 65 },
    { 0, 0 },
    { 0x16d7ba5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3547ba5f, 97 },
    { 0, 0 },
    { 0x4787ba61, 49 },
    { 0xf457ba62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd557ba65, 33 },
    { 0x6c27ba66, 61 },
    { 0xe337ba67, 34 },
    { 0x277ba66, 98 },
    { 0, 0 },
    { 0x8c77ba6a, 95 },
    { 0, 0 },
    { 0x1aa7ba6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c7ba76, 90 },
    { 0x89e7ba77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb7ba7b, 61 },
    { 0xc707ba7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb87ba7f, 61 },
    { 0x6f7ba80, 34 },
    { 0, 0 },
    { 0xabb7ba82, 46 },
    { 0xd297ba83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d7ba8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b7ba97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b7ba9a, 98 },
    { 0x6e27ba9b, 61 },
    { 0x82f7ba9a, 61 },
    { 0xece7ba9d, 61 },
    { 0x4797ba9e, 33 },
    { 0, 0 },
    { 0xc0a7baa0, 90 },
    { 0, 0 },
    { 0xf377baa2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc137baa5, 33 },
    { 0xac77baa5, 90 },
    { 0, 0 },
    { 0xbd67baa8, 76 },
    { 0x8ce7baa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d7baaf, 55 },
    { 0x6707bab0, 98 },
    { 0x1ad7bab1, 61 },
    { 0xb6a7baaf, 33 },
    { 0xe927bab3, 43 },
    { 0xe227bab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e7bab8, 61 },
    { 0, 0 },
    { 0x3f77baba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8897babe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba47bac9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc7bacc, 42 },
    { 0x2807bacd, 95 },
    { 0x867bacd, 23 },
    { 0, 0 },
    { 0xba77bad0, 67 },
    { 0, 0 },
    { 0xa9f7bad2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e7bad5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4387badb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c7bae3, 98 },
    { 0x4c77bae3, 67 },
    { 0, 0 },
    { 0x49f7bae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3687bae9, 12 },
    { 0xdfe7baea, 85 },
    { 0x6a57baeb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb7baee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67baf6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e7bafa, 33 },
    { 0xa617bafb, 64 },
    { 0xd557bafc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf7bb02, 90 },
    { 0x9327bb03, 33 },
    { 0x8bd7bb02, 93 },
    { 0x94d7bb02, 90 },
    { 0x28f7bb06, 90 },
    { 0, 0 },
    { 0x13f7bb08, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb117bb0b, 95 },
    { 0, 0 },
    { 0x3297bb0d, 91 },
    { 0xcce7bb0e, 43 },
    { 0x61c7bb0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77bb13, 33 },
    { 0x6187bb14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab7bb17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda87bb1c, 90 },
    { 0, 0 },
    { 0x9347bb1e, 43 },
    { 0xa387bb1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c7bb25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8367bb2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9677bb30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a7bb33, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0x3007bb36, 33 },
    { 0x3927bb36, 85 },
    { 0x4da7bb37, 55 },
    { 0xfb7bb37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf647bb48, 99 },
    { 0xcf27bb49, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd7bb4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8647bb50, 98 },
    { 0x2f57bb51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f7bb57, 33 },
    { 0xd2e7bb58, 23 },
    { 0x7967bb59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7117bb60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc7bb64, 55 },
    { 0xebc7bb65, 75 },
    { 0x4597bb66, 90 },
    { 0xfce7bb67, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6297bb71, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce07bb79, 33 },
    { 0, 0 },
    { 0x64b7bb7b, 87 },
    { 0, 0 },
    { 0x7cf7bb7d, 90 },
    { 0xa707bb7e, 61 },
    { 0x8bf7bb7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547bb89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b7bb8e, 76 },
    { 0x7587bb8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd607bb92, 34 },
    { 0x6c37bb93, 81 },
    { 0, 0 },
    { 0xe517bb95, 78 },
    { 0x2e27bb96, 77 },
    { 0, 0 },
    { 0x92e7bb98, 23 },
    { 0x2457bb98, 23 },
    { 0, 0 },
    { 0x2607bb9b, 24 },
    { 0x557bb9c, 61 },
    { 0xb2d7bb9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f7bba6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c7bbac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9987bbb3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f7bbb7, 52 },
    { 0x417bbb8, 23 },
    { 0, 0 },
    { 0x61f7bbba, 85 },
    { 0x2bf7bbbb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4537bbc2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a7bbcb, 77 },
    { 0xa2f7bbcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a7bbd2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b7bbd6, 55 },
    { 0x72a7bbd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5667bbda, 90 },
    { 0xda17bbda, 99 },
    { 0x5c57bbdc, 34 },
    { 0x54d7bbdd, 67 },
    { 0xcf87bbda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba27bbe2, 90 },
    { 0, 0 },
    { 0xe9b7bbe4, 90 },
    { 0x72d7bbe5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a7bbea, 60 },
    { 0x7227bbeb, 90 },
    { 0, 0 },
    { 0xafd7bbed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a7bbf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae7bbf6, 96 },
    { 0x1f37bbf7, 98 },
    { 0xcef7bbf8, 98 },
    { 0x9c67bbf6, 83 },
    { 0x9d57bbf7, 69 },
    { 0x44f7bbfb, 66 },
    { 0, 0 },
    { 0x39c7bbfd, 46 },
    { 0, 0 },
    { 0x3f87bbff, 98 },
    { 0, 0 },
    { 0xc317bc01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa97bc05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a57bc09, 46 },
    { 0, 0 },
    { 0xde57bc0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc337bc0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f7bc12, 23 },
    { 0x8887bc13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8457bc18, 87 },
    { 0, 0 },
    { 0x45e7bc1a, 98 },
    { 0, 0 },
    { 0xdbd7bc1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe847bc22, 34 },
    { 0x6b77bc23, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d7bc27, 77 },
    { 0x7417bc28, 33 },
    { 0xccc7bc27, 76 },
    { 0x25f7bc2a, 85 },
    { 0xa987bc28, 95 },
    { 0x9597bc2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc037bc34, 17 },
    { 0, 0 },
    { 0xe157bc36, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd027bc39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b7bc42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f7bc46, 98 },
    { 0x8797bc46, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e7bc4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d7bc53, 100 },
    { 0, 0 },
    { 0x5b17bc55, 55 },
    { 0, 0 },
    { 0x7757bc57, 90 },
    { 0x2f97bc57, 66 },
    { 0x8327bc57, 90 },
    { 0x197bc5a, 33 },
    { 0x5cb7bc5a, 76 },
    { 0x17bc5c, 90 },
    { 0, 0 },
    { 0x39e7bc5e, 63 },
    { 0x1967bc5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb7bc62, 95 },
    { 0x14c7bc63, 23 },
    { 0xfde7bc64, 55 },
    { 0, 0 },
    { 0x93f7bc66, 95 },
    { 0, 0 },
    { 0x5a7bc68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007bc6c, 65 },
    { 0x1597bc6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa397bc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad7bc74, 95 },
    { 0x427bc75, 33 },
    { 0x6397bc76, 90 },
    { 0xdcb7bc77, 33 },
    { 0, 0 },
    { 0xbed7bc79, 61 },
    { 0x9a07bc79, 33 },
    { 0x1fd7bc79, 90 },
    { 0xbbf7bc79, 76 },
    { 0, 0 },
    { 0x6267bc7e, 76 },
    { 0x3f27bc7f, 76 },
    { 0x6007bc80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef7bc83, 61 },
    { 0x9577bc84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d7bc8d, 90 },
    { 0, 0 },
    { 0x4937bc8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c07bc95, 46 },
    { 0xf577bc95, 93 },
    { 0x4797bc97, 87 },
    { 0x8ae7bc96, 93 },
    { 0, 0 },
    { 0xfb57bc9a, 90 },
    { 0, 0 },
    { 0x90a7bc9c, 61 },
    { 0, 0 },
    { 0x8477bc9e, 34 },
    { 0x4897bc9f, 100 },
    { 0x7f97bca0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e77bca3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9747bca7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2787bcab, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc957bcae, 88 },
    { 0x9ca7bcaf, 99 },
    { 0, 0 },
    { 0x7d07bcb1, 77 },
    { 0xcec7bcb2, 66 },
    { 0xc9e7bcb3, 34 },
    { 0xd2b7bcb1, 33 },
    { 0, 0 },
    { 0x8d17bcb6, 95 },
    { 0x12b7bcb7, 90 },
    { 0x65c7bcb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d47bcbe, 90 },
    { 0, 0 },
    { 0xfb57bcc0, 98 },
    { 0xbda7bcc1, 40 },
    { 0xd97bcc2, 61 },
    { 0x9a27bcc1, 61 },
    { 0, 0 },
    { 0xb307bcc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb737bcc9, 13 },
    { 0x6187bcca, 46 },
    { 0x7ce7bccb, 95 },
    { 0, 0 },
    { 0xe4c7bccd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7bcd0, 61 },
    { 0xbb7bcd1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd17bcd5, 55 },
    { 0, 0 },
    { 0x2927bcd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d67bcda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8527bcde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4927bce2, 17 },
    { 0xca7bce3, 43 },
    { 0, 0 },
    { 0x8167bce5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe037bce9, 90 },
    { 0xbb37bcea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb07bced, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c57bcf1, 98 },
    { 0xf757bcf2, 90 },
    { 0, 0 },
    { 0xb947bcf4, 61 },
    { 0, 0 },
    { 0x5097bcf6, 67 },
    { 0, 0 },
    { 0xe867bcf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8547bcfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc7bd00, 96 },
    { 0, 0 },
    { 0x5a37bd02, 61 },
    { 0xa5e7bd03, 95 },
    { 0x85c7bd04, 95 },
    { 0x9f67bd03, 55 },
    { 0x1757bd03, 76 },
    { 0x60d7bd04, 80 },
    { 0xa177bd08, 84 },
    { 0x3427bd09, 76 },
    { 0x35d7bd07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5877bd0d, 76 },
    { 0x7cc7bd0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77bd12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257bd18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb767bd21, 90 },
    { 0xde97bd22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3347bd25, 55 },
    { 0xc707bd25, 75 },
    { 0xb907bd27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb377bd2c, 61 },
    { 0x17f7bd2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae97bd31, 33 },
    { 0xc947bd32, 77 },
    { 0x3f97bd31, 61 },
    { 0x19a7bd31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a17bd37, 95 },
    { 0, 0 },
    { 0x7597bd39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2367bd3d, 98 },
    { 0, 0 },
    { 0x6b07bd3f, 95 },
    { 0x6ac7bd40, 8 },
    { 0x5397bd3f, 33 },
    { 0, 0 },
    { 0xf657bd43, 67 },
    { 0xec87bd44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c27bd4b, 55 },
    { 0xb557bd4b, 93 },
    { 0xda27bd4d, 98 },
    { 0xdd17bd4d, 98 },
    { 0x79f7bd4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1627bd54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa17bd57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c7bd5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7bd5d, 43 },
    { 0, 0 },
    { 0x3337bd5f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f7bd65, 98 },
    { 0, 0 },
    { 0x8d97bd67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f7bd6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d7bd74, 52 },
    { 0x6097bd75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc977bd78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9067bd7d, 34 },
    { 0, 0 },
    { 0xc717bd7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9587bd84, 95 },
    { 0x6077bd85, 85 },
    { 0xc77bd84, 33 },
    { 0x6867bd85, 90 },
    { 0xcc67bd88, 23 },
    { 0, 0 },
    { 0x9427bd8a, 78 },
    { 0, 0 },
    { 0x5557bd8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2537bd91, 88 },
    { 0x7537bd91, 90 },
    { 0, 0 },
    { 0x5d17bd94, 34 },
    { 0x5ae7bd94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2227bd99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817bd9e, 90 },
    { 0, 0 },
    { 0x15e7bda0, 98 },
    { 0, 0 },
    { 0xfa27bda2, 65 },
    { 0x1317bda3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf917bda6, 90 },
    { 0x5fe7bda7, 90 },
    { 0xad7bda8, 90 },
    { 0, 0 },
    { 0x8987bdaa, 93 },
    { 0, 0 },
    { 0x87b7bdac, 90 },
    { 0x9447bdad, 95 },
    { 0xfd97bdae, 23 },
    { 0x9c37bdac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7097bdb6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d7bdb9, 33 },
    { 0x4037bdba, 33 },
    { 0xf2d7bdba, 61 },
    { 0x7b27bdbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b7bdc6, 23 },
    { 0x407bdc7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d7bdcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5517bdd2, 61 },
    { 0, 0 },
    { 0x5167bdd4, 85 },
    { 0, 0 },
    { 0x1a77bdd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec7bddd, 84 },
    { 0x5207bdde, 90 },
    { 0xab97bddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d97bde3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5147bdee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb887bdf1, 43 },
    { 0xaa17bdf1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a7bdf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed7bdf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a7bdfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3907be02, 76 },
    { 0, 0 },
    { 0x7727be04, 55 },
    { 0, 0 },
    { 0x657be06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e7be09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf217be0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5617be15, 54 },
    { 0x1f77be16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd117be1a, 80 },
    { 0x8947be1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd7be23, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17be29, 33 },
    { 0, 0 },
    { 0x8577be2b, 46 },
    { 0x9177be2c, 61 },
    { 0xf477be2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe7be34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c7be39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8847be3c, 61 },
    { 0x9077be3d, 90 },
    { 0xd877be3e, 55 },
    { 0x2c27be3f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce97be42, 99 },
    { 0xbd87be43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5067be48, 61 },
    { 0xa6f7be49, 69 },
    { 0, 0 },
    { 0x2517be4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f7be4f, 55 },
    { 0xca67be50, 98 },
    { 0x2537be51, 67 },
    { 0x83d7be4f, 95 },
    { 0xd2a7be53, 23 },
    { 0x57c7be54, 34 },
    { 0x3ee7be54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc37be5b, 95 },
    { 0xdd07be5b, 76 },
    { 0xd617be5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a57be64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd597be68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6457be6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f7be77, 61 },
    { 0, 0 },
    { 0x9517be79, 93 },
    { 0, 0 },
    { 0x7d77be7b, 76 },
    { 0xdb47be7c, 95 },
    { 0xbee7be7d, 95 },
    { 0x6727be7e, 90 },
    { 0x2877be7d, 23 },
    { 0xb187be7f, 90 },
    { 0xba7be7d, 61 },
    { 0x8f27be80, 46 },
    { 0x9de7be7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c7be89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e7be90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f87be98, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e27be9c, 90 },
    { 0x1567be9c, 87 },
    { 0x81a7be9c, 76 },
    { 0x3cf7be9f, 90 },
    { 0x3377bea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e7bea5, 98 },
    { 0x2707bea6, 23 },
    { 0xe5a7bea7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe527beab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d7beae, 90 },
    { 0xb7c7beaf, 66 },
    { 0, 0 },
    { 0xff87beb1, 95 },
    { 0, 0 },
    { 0x1d77beb3, 2 },
    { 0, 0 },
    { 0x9907beb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b7bebc, 99 },
    { 0xb427bebc, 76 },
    { 0x8b57bebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb507bec1, 33 },
    { 0, 0 },
    { 0x197bec3, 61 },
    { 0, 0 },
    { 0xb6f7bec5, 76 },
    { 0x5797bec6, 33 },
    { 0xa4b7bec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac7beca, 99 },
    { 0, 0 },
    { 0x2b17becc, 46 },
    { 0, 0 },
    { 0xfb87bece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1607bed1, 75 },
    { 0xe257bed2, 90 },
    { 0xacf7bed1, 23 },
    { 0, 0 },
    { 0x5b37bed5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x947bed8, 55 },
    { 0x6d27bed9, 23 },
    { 0x5767bed8, 23 },
    { 0xa297bedb, 33 },
    { 0xcff7bed8, 91 },
    { 0, 0 },
    { 0x3ad7bede, 33 },
    { 0, 0 },
    { 0xf6e7bee0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f87bee6, 87 },
    { 0x55d7bee7, 33 },
    { 0x58d7bee8, 33 },
    { 0xf27bee7, 100 },
    { 0xeb7bee8, 90 },
    { 0xf77bee8, 61 },
    { 0x2aa7beec, 83 },
    { 0xd137beec, 23 },
    { 0x4ed7bee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc07bef1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf7bef5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc77bef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd7bf00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd857bf08, 44 },
    { 0xf3e7bf09, 95 },
    { 0xb357bf08, 90 },
    { 0x30f7bf09, 20 },
    { 0xe7e7bf0c, 93 },
    { 0xf4a7bf0c, 55 },
    { 0x6807bf0e, 91 },
    { 0, 0 },
    { 0x2397bf10, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb67bf13, 33 },
    { 0x3057bf14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7bf1b, 23 },
    { 0, 0 },
    { 0x22e7bf1d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a07bf20, 43 },
    { 0x1db7bf20, 90 },
    { 0x2f7bf22, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de7bf2b, 59 },
    { 0x2c7bf2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6147bf31, 100 },
    { 0, 0 },
    { 0x7c57bf33, 76 },
    { 0, 0 },
    { 0x23f7bf35, 95 },
    { 0xe267bf36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4667bf3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1477bf3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617bf49, 46 },
    { 0x9fe7bf49, 90 },
    { 0xaba7bf4b, 76 },
    { 0x2fd7bf49, 100 },
    { 0, 0 },
    { 0x8007bf4e, 61 },
    { 0, 0 },
    { 0xcd67bf50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d7bf53, 95 },
    { 0x36f7bf53, 55 },
    { 0, 0 },
    { 0xbb97bf56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb57bf5a, 61 },
    { 0x3877bf5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1057bf5f, 23 },
    { 0xb7bf60, 90 },
    { 0x5037bf61, 61 },
    { 0x8f17bf61, 90 },
    { 0xcd97bf63, 65 },
    { 0x9027bf63, 88 },
    { 0x27e7bf65, 78 },
    { 0xb187bf66, 61 },
    { 0xb2b7bf64, 61 },
    { 0x8937bf67, 61 },
    { 0xf017bf67, 43 },
    { 0xdda7bf6a, 93 },
    { 0x5d7bf64, 93 },
    { 0x5767bf6c, 32 },
    { 0xcbc7bf6c, 43 },
    { 0x807bf6e, 33 },
    { 0xcb17bf64, 90 },
    { 0x1a17bf70, 98 },
    { 0xf807bf60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577bf75, 23 },
    { 0x5737bf76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf79, 33 },
    { 0x9997bf79, 33 },
    { 0, 0 },
    { 0xfa57bf7c, 33 },
    { 0x26e7bf7d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d7bf81, 33 },
    { 0, 0 },
    { 0x3127bf83, 33 },
    { 0xd527bf83, 66 },
    { 0, 0 },
    { 0xe507bf86, 98 },
    { 0xf6b7bf87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2367bf8a, 43 },
    { 0, 0 },
    { 0x9d7bf8c, 95 },
    { 0, 0 },
    { 0x57a7bf8e, 34 },
    { 0, 0 },
    { 0xcee7bf90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb7bf95, 46 },
    { 0, 0 },
    { 0x37b7bf97, 95 },
    { 0x1d17bf98, 90 },
    { 0x57bf99, 23 },
    { 0xf957bf9a, 61 },
    { 0xe427bf99, 33 },
    { 0x3407bf97, 87 },
    { 0x99a7bf9d, 65 },
    { 0x9f27bf9a, 55 },
    { 0x91f7bf9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb7bfa3, 95 },
    { 0xf117bfa3, 90 },
    { 0x6c7bfa4, 67 },
    { 0x3427bfa6, 34 },
    { 0, 0 },
    { 0xe117bfa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7bfb0, 98 },
    { 0, 0 },
    { 0xc667bfb2, 85 },
    { 0x7cd7bfb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa717bfb8, 55 },
    { 0x5a97bfb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e7bfbd, 46 },
    { 0x9cc7bfbe, 33 },
    { 0, 0 },
    { 0x4317bfc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a7bfc3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9477bfc6, 33 },
    { 0xecb7bfc6, 84 },
    { 0, 0 },
    { 0x58e7bfc9, 43 },
    { 0xd027bfca, 90 },
    { 0xc1c7bfcb, 55 },
    { 0x4f67bfcc, 61 },
    { 0xbff7bfcb, 61 },
    { 0xd8a7bfce, 98 },
    { 0x10f7bfcf, 88 },
    { 0x3fb7bfcf, 33 },
    { 0x1267bfd1, 90 },
    { 0x4be7bfd0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4717bfd5, 43 },
    { 0xf837bfd6, 100 },
    { 0x77f7bfd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7bfda, 98 },
    { 0x4457bfda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc257bfde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87bfe2, 52 },
    { 0xc9e7bfe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f77bfe6, 23 },
    { 0, 0 },
    { 0xd67bfe8, 94 },
    { 0, 0 },
    { 0x9117bfea, 33 },
    { 0xa7bfeb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb797bff4, 23 },
    { 0, 0 },
    { 0x6357bff6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3047bffb, 33 },
    { 0, 0 },
    { 0x3a07bffd, 81 },
    { 0, 0 },
    { 0xee87bfff, 67 },
    { 0xa1c7bfff, 76 },
    { 0x3087bfff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4617c005, 76 },
    { 0x4d57c006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3337c00b, 33 },
    { 0x3ca7c00c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9357c012, 93 },
    { 0x7597c013, 17 },
    { 0xbf87c013, 55 },
    { 0x2c57c015, 93 },
    { 0x52c7c012, 43 },
    { 0xb457c017, 61 },
    { 0xee67c016, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba57c01c, 87 },
    { 0, 0 },
    { 0xecf7c01e, 61 },
    { 0xdee7c01e, 90 },
    { 0x9447c020, 53 },
    { 0xed7c020, 34 },
    { 0x5917c020, 61 },
    { 0xd6b7c023, 61 },
    { 0, 0 },
    { 0x12a7c025, 46 },
    { 0, 0 },
    { 0x20a7c027, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa387c02e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7137c032, 33 },
    { 0x4447c033, 90 },
    { 0xb9b7c033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c7c038, 98 },
    { 0x26d7c038, 23 },
    { 0xb547c038, 61 },
    { 0xa287c03b, 98 },
    { 0x8f97c03b, 55 },
    { 0x3917c03c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a7c041, 96 },
    { 0, 0 },
    { 0x1c17c043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87c048, 78 },
    { 0x32c7c049, 99 },
    { 0x6917c049, 52 },
    { 0xbd67c049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e7c050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87c054, 33 },
    { 0xb777c055, 93 },
    { 0x4667c056, 66 },
    { 0, 0 },
    { 0x1e7c058, 98 },
    { 0xb107c059, 52 },
    { 0x6367c05a, 98 },
    { 0xbb7c05b, 90 },
    { 0xd587c05b, 98 },
    { 0x2177c05d, 93 },
    { 0xd5e7c058, 90 },
    { 0x2737c05c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b7c066, 91 },
    { 0xef97c066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb257c06d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37c071, 67 },
    { 0x57f7c072, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa877c076, 90 },
    { 0xab47c077, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7c07e, 33 },
    { 0, 0 },
    { 0xa2e7c080, 76 },
    { 0, 0 },
    { 0x4197c082, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a7c08b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b7c090, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c094, 90 },
    { 0, 0 },
    { 0x52a7c096, 33 },
    { 0, 0 },
    { 0xf3d7c098, 90 },
    { 0xa987c099, 93 },
    { 0x73f7c098, 23 },
    { 0xbc47c098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1787c0a4, 55 },
    { 0x95b7c0a5, 55 },
    { 0x4577c0a6, 90 },
    { 0, 0 },
    { 0xbf67c0a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377c0ac, 93 },
    { 0x1d7c0ad, 95 },
    { 0x96b7c0ae, 94 },
    { 0x8fe7c0af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac57c0b3, 43 },
    { 0x1017c0b4, 76 },
    { 0xcc17c0b4, 95 },
    { 0xd757c0b6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f67c0b9, 33 },
    { 0xae27c0ba, 33 },
    { 0, 0 },
    { 0x2a07c0bc, 88 },
    { 0, 0 },
    { 0x4a57c0be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3997c0c1, 95 },
    { 0x6807c0c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67c0c7, 95 },
    { 0xafe7c0c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1997c0cd, 76 },
    { 0, 0 },
    { 0x42d7c0cf, 55 },
    { 0x9e87c0d0, 90 },
    { 0x5297c0d0, 46 },
    { 0x7167c0d2, 61 },
    { 0x9cb7c0cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe07c0e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7637c0ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6707c0fb, 90 },
    { 0xd8b7c0fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6017c102, 99 },
    { 0, 0 },
    { 0xb417c104, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9187c107, 52 },
    { 0xf2c7c108, 55 },
    { 0xd027c109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe527c10d, 46 },
    { 0x3a77c10e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe707c111, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f7c11f, 46 },
    { 0xe777c120, 93 },
    { 0x89d7c11f, 33 },
    { 0, 0 },
    { 0x7b87c123, 90 },
    { 0x5f67c124, 61 },
    { 0x89a7c125, 23 },
    { 0x7e27c126, 46 },
    { 0x15e7c127, 46 },
    { 0x9367c126, 33 },
    { 0x39e7c129, 76 },
    { 0x16a7c12a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd367c12d, 90 },
    { 0xee7c12e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7807c135, 23 },
    { 0xce17c136, 34 },
    { 0xefe7c137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2137c13a, 23 },
    { 0xe817c13b, 98 },
    { 0x3d47c13a, 95 },
    { 0, 0 },
    { 0x9627c13e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f7c145, 23 },
    { 0x96d7c146, 81 },
    { 0x8e27c147, 23 },
    { 0xdee7c148, 91 },
    { 0x1377c149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea7c14c, 95 },
    { 0, 0 },
    { 0x12c7c14e, 52 },
    { 0x64c7c14f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c153, 87 },
    { 0x60e7c153, 98 },
    { 0x7f37c155, 98 },
    { 0x2647c156, 90 },
    { 0xab47c155, 38 },
    { 0x4207c156, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea57c161, 84 },
    { 0xb947c162, 52 },
    { 0, 0 },
    { 0x5417c164, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa017c168, 98 },
    { 0x8797c168, 90 },
    { 0, 0 },
    { 0x7f77c16b, 90 },
    { 0x2e77c16b, 90 },
    { 0xeb57c16b, 23 },
    { 0x9237c16d, 84 },
    { 0xb17c16d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3047c174, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe7c17b, 61 },
    { 0, 0 },
    { 0xc9e7c17d, 98 },
    { 0x9247c17e, 99 },
    { 0xe6f7c17e, 23 },
    { 0x8717c17d, 55 },
    { 0, 0 },
    { 0x6037c182, 84 },
    { 0xca77c183, 99 },
    { 0x4827c184, 66 },
    { 0xa3e7c183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f7c188, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1357c18e, 95 },
    { 0xab47c18f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f7c193, 100 },
    { 0x73f7c193, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf507c197, 98 },
    { 0, 0 },
    { 0x1887c199, 61 },
    { 0x26e7c19a, 78 },
    { 0xe4b7c19a, 78 },
    { 0xb2a7c19c, 23 },
    { 0xa7b7c19b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd357c1a3, 77 },
    { 0, 0 },
    { 0xf5a7c1a5, 85 },
    { 0x7c07c1a6, 61 },
    { 0x3227c1a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb27c1ad, 55 },
    { 0x6077c1ae, 90 },
    { 0, 0 },
    { 0x7997c1b0, 61 },
    { 0xf7c7c1b1, 43 },
    { 0, 0 },
    { 0xf0b7c1b3, 88 },
    { 0x8cf7c1b3, 90 },
    { 0x6a37c1b4, 88 },
    { 0, 0 },
    { 0xf0c7c1b7, 90 },
    { 0x9517c1b7, 61 },
    { 0x3ff7c1b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa127c1c1, 55 },
    { 0, 0 },
    { 0xae07c1c3, 90 },
    { 0x6597c1c4, 91 },
    { 0xa827c1c3, 34 },
    { 0, 0 },
    { 0xbb37c1c7, 23 },
    { 0x4257c1c7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad47c1cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3367c1dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8527c1e1, 91 },
    { 0x6517c1e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda67c1e5, 99 },
    { 0x5847c1e6, 61 },
    { 0x50e7c1e7, 77 },
    { 0, 0 },
    { 0xd477c1e9, 95 },
    { 0x42e7c1ea, 66 },
    { 0x31a7c1ea, 40 },
    { 0x9457c1ea, 76 },
    { 0, 0 },
    { 0xf347c1ee, 98 },
    { 0, 0 },
    { 0xff77c1f0, 98 },
    { 0, 0 },
    { 0x2327c1f2, 34 },
    { 0, 0 },
    { 0x9277c1f4, 33 },
    { 0xdc47c1f4, 81 },
    { 0x2197c1f6, 90 },
    { 0x4a87c1f7, 76 },
    { 0, 0 },
    { 0xc5a7c1f9, 46 },
    { 0, 0 },
    { 0x7d17c1fb, 33 },
    { 0, 0 },
    { 0x17c7c1fd, 61 },
    { 0, 0 },
    { 0xef37c1ff, 43 },
    { 0x7107c200, 84 },
    { 0x8987c201, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df7c205, 76 },
    { 0x4a7c206, 98 },
    { 0x6ff7c206, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf907c210, 33 },
    { 0x8907c211, 23 },
    { 0xcca7c212, 87 },
    { 0x68e7c210, 90 },
    { 0xc477c214, 61 },
    { 0x46f7c215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc7c21e, 61 },
    { 0, 0 },
    { 0x177c220, 93 },
    { 0x9dc7c220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9007c225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e7c229, 68 },
    { 0x3e87c229, 76 },
    { 0xc0c7c22b, 90 },
    { 0x2307c22b, 90 },
    { 0x3a57c22b, 83 },
    { 0x83b7c22d, 90 },
    { 0x66a7c22f, 90 },
    { 0xead7c22e, 90 },
    { 0xe7b7c231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4917c235, 61 },
    { 0x8c37c236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2537c23a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b7c23d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e37c245, 23 },
    { 0x2b37c245, 68 },
    { 0xb247c247, 90 },
    { 0x8857c248, 90 },
    { 0x30b7c247, 55 },
    { 0xc727c24a, 81 },
    { 0xd777c246, 93 },
    { 0x1587c24c, 23 },
    { 0x9817c24c, 76 },
    { 0x9527c24e, 93 },
    { 0x6657c248, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa7c256, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e7c25b, 23 },
    { 0x6837c25c, 93 },
    { 0x2f7c25d, 98 },
    { 0xa407c25b, 55 },
    { 0x56c7c25d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe57c264, 97 },
    { 0xd107c265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a7c27a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5797c27e, 95 },
    { 0, 0 },
    { 0xc57c280, 61 },
    { 0x4d7c280, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7c284, 95 },
    { 0xd797c285, 93 },
    { 0x8bc7c286, 90 },
    { 0, 0 },
    { 0xbdb7c288, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b87c28c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe97c290, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf27c294, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9387c297, 55 },
    { 0x1557c298, 90 },
    { 0x97c298, 90 },
    { 0x4767c29a, 98 },
    { 0x57c7c29b, 55 },
    { 0x18b7c29c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd957c2a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x137c2a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe987c2b1, 52 },
    { 0x4e17c2b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca77c2b8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7c2bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7987c2be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f7c2c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a7c2c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b77c2ca, 61 },
    { 0x8e47c2cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4877c2d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b7c2d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc797c2d8, 43 },
    { 0xda17c2d9, 52 },
    { 0x3687c2da, 65 },
    { 0x6d7c2db, 46 },
    { 0x3e37c2db, 90 },
    { 0, 0 },
    { 0x2f7c2de, 99 },
    { 0, 0 },
    { 0x1fe7c2e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b27c2e3, 76 },
    { 0xad7c2e3, 25 },
    { 0x3167c2e5, 90 },
    { 0xe287c2e6, 77 },
    { 0, 0 },
    { 0x5107c2e8, 61 },
    { 0x4be7c2e8, 95 },
    { 0x2b47c2ea, 46 },
    { 0x6947c2eb, 98 },
    { 0x8e57c2ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff17c2ef, 84 },
    { 0, 0 },
    { 0x35d7c2f1, 23 },
    { 0x6987c2f2, 90 },
    { 0xb527c2f3, 98 },
    { 0xda47c2f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe7c2f8, 84 },
    { 0x47e7c2f8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf77c2fc, 66 },
    { 0x5097c2fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a7c302, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd7c305, 46 },
    { 0xd547c306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b97c30a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab7c30d, 40 },
    { 0x9d07c30e, 55 },
    { 0x6ce7c30f, 93 },
    { 0xd1b7c30f, 61 },
    { 0x5477c30f, 90 },
    { 0, 0 },
    { 0xcd17c313, 90 },
    { 0xed97c314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f97c31a, 90 },
    { 0xbed7c31a, 76 },
    { 0, 0 },
    { 0x15c7c31d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f7c320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7c325, 90 },
    { 0x2107c326, 98 },
    { 0, 0 },
    { 0xc347c328, 98 },
    { 0x3b87c329, 88 },
    { 0xc537c329, 90 },
    { 0x5767c32b, 76 },
    { 0x4a17c32c, 61 },
    { 0x5c57c329, 95 },
    { 0x29b7c32e, 33 },
    { 0, 0 },
    { 0xe787c330, 90 },
    { 0x4427c330, 95 },
    { 0xbd87c330, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047c336, 33 },
    { 0, 0 },
    { 0xd1f7c338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe977c341, 90 },
    { 0xd777c342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb847c345, 46 },
    { 0, 0 },
    { 0xbbd7c347, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf557c350, 93 },
    { 0x1157c351, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd887c356, 88 },
    { 0x5617c357, 61 },
    { 0, 0 },
    { 0xa987c359, 55 },
    { 0x6a67c359, 91 },
    { 0, 0 },
    { 0x25e7c35c, 61 },
    { 0xe5b7c35d, 68 },
    { 0, 0 },
    { 0xed77c35f, 95 },
    { 0x3b7c360, 90 },
    { 0x60d7c360, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc7c365, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917c36b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd7c36e, 85 },
    { 0, 0 },
    { 0x1e27c370, 90 },
    { 0xc9f7c371, 90 },
    { 0x1627c372, 76 },
    { 0x9187c373, 34 },
    { 0x267c370, 33 },
    { 0xaa37c375, 100 },
    { 0x7ae7c370, 98 },
    { 0, 0 },
    { 0xc6e7c378, 23 },
    { 0x1097c379, 65 },
    { 0x3447c37a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd7c37d, 68 },
    { 0xff77c37d, 76 },
    { 0x4f27c37d, 23 },
    { 0x2507c37d, 55 },
    { 0, 0 },
    { 0x8f57c382, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577c388, 66 },
    { 0, 0 },
    { 0xbd57c38a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9637c390, 61 },
    { 0x8c07c391, 33 },
    { 0, 0 },
    { 0xb017c393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c398, 33 },
    { 0xfb97c399, 23 },
    { 0x3567c39a, 61 },
    { 0, 0 },
    { 0x19a7c39c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f7c39f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x387c3a2, 23 },
    { 0xe677c3a3, 61 },
    { 0xde27c3a4, 98 },
    { 0x9e77c3a3, 33 },
    { 0xceb7c3a4, 90 },
    { 0xedb7c3a7, 55 },
    { 0xa9b7c3a7, 76 },
    { 0x5727c3a7, 33 },
    { 0, 0 },
    { 0xb0a7c3ab, 43 },
    { 0xb017c3ac, 55 },
    { 0x8177c3ac, 76 },
    { 0, 0 },
    { 0xed97c3af, 23 },
    { 0x2317c3b0, 55 },
    { 0xe697c3b1, 33 },
    { 0x5ba7c3b2, 90 },
    { 0xc1c7c3b3, 23 },
    { 0x9b37c3b4, 68 },
    { 0x2497c3b2, 76 },
    { 0x4607c3b3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e7c3bc, 43 },
    { 0, 0 },
    { 0x4eb7c3be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237c3c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7c3c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e7c3cf, 90 },
    { 0, 0 },
    { 0x7187c3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a27c3d5, 61 },
    { 0xd0c7c3d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa837c3da, 93 },
    { 0x3657c3db, 67 },
    { 0x1bf7c3dc, 23 },
    { 0, 0 },
    { 0x8527c3de, 33 },
    { 0x5657c3df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3407c3e5, 95 },
    { 0, 0 },
    { 0x40f7c3e7, 95 },
    { 0xc387c3e7, 98 },
    { 0, 0 },
    { 0xa007c3ea, 34 },
    { 0, 0 },
    { 0x4b17c3ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f7c3ef, 61 },
    { 0xb6b7c3f0, 90 },
    { 0xe3f7c3f1, 33 },
    { 0x7a37c3f2, 33 },
    { 0x4397c3f3, 98 },
    { 0x15f7c3f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b77c3fc, 90 },
    { 0, 0 },
    { 0xcf37c3fe, 95 },
    { 0x8c47c3fe, 76 },
    { 0x52b7c400, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9887c40c, 33 },
    { 0x29f7c40d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e7c412, 61 },
    { 0xb4d7c413, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd167c416, 76 },
    { 0x16a7c417, 77 },
    { 0xacc7c416, 55 },
    { 0x5497c419, 90 },
    { 0, 0 },
    { 0xc237c41b, 66 },
    { 0, 0 },
    { 0xfa57c41d, 84 },
    { 0x817c41e, 76 },
    { 0x98f7c41d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47c422, 43 },
    { 0xf587c423, 88 },
    { 0x5517c424, 95 },
    { 0x437c425, 43 },
    { 0xec77c423, 90 },
    { 0x717c422, 23 },
    { 0xc887c424, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4107c42b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde47c42f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5067c432, 33 },
    { 0x8847c433, 61 },
    { 0, 0 },
    { 0xf597c435, 100 },
    { 0x1bc7c436, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127c43b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b57c43e, 90 },
    { 0, 0 },
    { 0x717c440, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197c443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2167c44c, 76 },
    { 0x2a47c44d, 90 },
    { 0, 0 },
    { 0x8a37c44f, 34 },
    { 0x7e27c450, 23 },
    { 0x80d7c450, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8387c454, 91 },
    { 0xc057c455, 52 },
    { 0x9c57c455, 34 },
    { 0x7c97c457, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda17c45b, 98 },
    { 0xdd67c45c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7c460, 91 },
    { 0x68e7c461, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa7c469, 65 },
    { 0x1577c469, 34 },
    { 0x8867c46a, 23 },
    { 0xbdb7c46b, 64 },
    { 0x6a07c46a, 78 },
    { 0xfbb7c46e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd87c471, 61 },
    { 0, 0 },
    { 0xe1d7c473, 66 },
    { 0x3097c473, 98 },
    { 0, 0 },
    { 0x2887c476, 34 },
    { 0xeae7c477, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1057c47a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517c47f, 46 },
    { 0xec7c480, 83 },
    { 0xc77c481, 33 },
    { 0xa2e7c480, 33 },
    { 0xda37c481, 61 },
    { 0x7be7c484, 55 },
    { 0, 0 },
    { 0xa4e7c486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde7c48d, 23 },
    { 0xac37c48e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed97c492, 61 },
    { 0xac17c492, 90 },
    { 0x9847c493, 76 },
    { 0x17c7c492, 90 },
    { 0, 0 },
    { 0xa17c497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b07c49c, 98 },
    { 0, 0 },
    { 0xf857c49e, 76 },
    { 0x7167c49f, 43 },
    { 0, 0 },
    { 0x9ba7c4a1, 78 },
    { 0x9b47c4a2, 52 },
    { 0x6cf7c4a2, 99 },
    { 0, 0 },
    { 0x4e57c4a5, 98 },
    { 0, 0 },
    { 0xe367c4a7, 98 },
    { 0x4957c4a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7577c4ab, 61 },
    { 0x4537c4ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2777c4b0, 95 },
    { 0x6167c4b1, 34 },
    { 0x8847c4b2, 34 },
    { 0x6097c4b2, 33 },
    { 0x8487c4b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d7c4bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b07c4bf, 43 },
    { 0xc447c4bf, 91 },
    { 0x37a7c4c1, 98 },
    { 0xaa7c4c2, 76 },
    { 0xbae7c4c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa657c4cd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b27c4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5507c4d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f7c4dd, 91 },
    { 0, 0 },
    { 0x3357c4df, 66 },
    { 0x2d97c4df, 34 },
    { 0x2e77c4e0, 23 },
    { 0xc167c4e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e7c4e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a7c4e9, 61 },
    { 0x247c4ea, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d7c4ed, 90 },
    { 0x1c7c4ee, 90 },
    { 0x99b7c4ee, 98 },
    { 0xb2d7c4f0, 76 },
    { 0x81f7c4ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x937c4f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6377c4fc, 61 },
    { 0, 0 },
    { 0x8977c4fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437c502, 33 },
    { 0x9027c502, 61 },
    { 0x7e17c504, 90 },
    { 0x9a37c502, 61 },
    { 0x7947c505, 84 },
    { 0x7cd7c507, 78 },
    { 0xce57c504, 95 },
    { 0x2f97c506, 95 },
    { 0x95d7c506, 98 },
    { 0xc7c7c502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b7c510, 99 },
    { 0, 0 },
    { 0x3217c512, 66 },
    { 0x6357c512, 90 },
    { 0xc4c7c514, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa887c518, 33 },
    { 0xbfe7c519, 78 },
    { 0x85e7c51a, 93 },
    { 0xbfb7c51a, 95 },
    { 0x30a7c51b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb847c51f, 90 },
    { 0x1c17c520, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf27c526, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7c52c, 95 },
    { 0xb6c7c52d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a7c530, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457c535, 33 },
    { 0x6867c536, 33 },
    { 0x8ec7c536, 91 },
    { 0x8cc7c538, 76 },
    { 0x1b07c539, 33 },
    { 0x767c53a, 23 },
    { 0x7ae7c538, 98 },
    { 0xe9b7c53c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa157c542, 98 },
    { 0, 0 },
    { 0x1417c544, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7867c54a, 66 },
    { 0x9877c54b, 35 },
    { 0xeb67c54c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d7c54f, 34 },
    { 0x3867c54f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2057c554, 61 },
    { 0x11d7c554, 93 },
    { 0x8ba7c554, 33 },
    { 0x93d7c554, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec47c55a, 90 },
    { 0xebc7c55b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e97c563, 17 },
    { 0x4287c564, 17 },
    { 0xcad7c565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd717c569, 95 },
    { 0x6857c569, 88 },
    { 0x5f27c56a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1977c572, 77 },
    { 0, 0 },
    { 0xcf67c574, 55 },
    { 0x5e7c575, 90 },
    { 0x6957c575, 98 },
    { 0xbd37c577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07c57b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6397c57f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3847c582, 98 },
    { 0xc307c583, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7c589, 66 },
    { 0, 0 },
    { 0xfdb7c58b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9117c58f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c594, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6087c59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf627c59f, 55 },
    { 0xd997c5a0, 95 },
    { 0x4487c5a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1797c5a7, 55 },
    { 0x7c77c5a8, 84 },
    { 0xe6b7c5a9, 95 },
    { 0xd687c5a8, 61 },
    { 0xdf7c5a9, 23 },
    { 0, 0 },
    { 0x12f7c5ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd7c5b4, 55 },
    { 0xb9c7c5b5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e7c5c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc587c5c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3127c5cc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7c5cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc7c5d3, 65 },
    { 0, 0 },
    { 0xdd87c5d5, 76 },
    { 0, 0 },
    { 0xc3d7c5d7, 68 },
    { 0x4d07c5d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf707c5db, 90 },
    { 0, 0 },
    { 0xbb57c5dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa017c5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57c5e6, 90 },
    { 0, 0 },
    { 0xf097c5e8, 95 },
    { 0xd2a7c5e9, 90 },
    { 0, 0 },
    { 0xffa7c5eb, 90 },
    { 0, 0 },
    { 0x9437c5ed, 95 },
    { 0, 0 },
    { 0x2df7c5ef, 33 },
    { 0, 0 },
    { 0xe567c5f1, 90 },
    { 0x79f7c5f2, 91 },
    { 0xf167c5f2, 95 },
    { 0x6fe7c5f2, 23 },
    { 0x7037c5f2, 76 },
    { 0x21a7c5f6, 98 },
    { 0x8f87c5f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e7c5fe, 95 },
    { 0x7a17c5ff, 43 },
    { 0x687c5fe, 55 },
    { 0xb397c600, 55 },
    { 0x38e7c602, 99 },
    { 0x717c603, 43 },
    { 0xa4c7c602, 95 },
    { 0xe437c605, 61 },
    { 0xabb7c605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1457c609, 95 },
    { 0, 0 },
    { 0xf127c60b, 46 },
    { 0x6a97c60c, 23 },
    { 0, 0 },
    { 0x9817c60e, 42 },
    { 0x32b7c60f, 46 },
    { 0xf337c60f, 61 },
    { 0x8537c611, 23 },
    { 0xb097c611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c7c616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf87c61d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e7c624, 61 },
    { 0xd6b7c624, 90 },
    { 0, 0 },
    { 0x2507c627, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c87c62a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa937c62f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77c632, 61 },
    { 0xac77c632, 67 },
    { 0x1dc7c633, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x8707c637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007c63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e67c63d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd337c640, 33 },
    { 0xb4e7c641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd537c644, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5637c64b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147c650, 88 },
    { 0xfac7c651, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa467c655, 34 },
    { 0xf887c656, 96 },
    { 0x8867c657, 91 },
    { 0xc557c658, 46 },
    { 0, 0 },
    { 0x4fd7c65a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b7c660, 34 },
    { 0xbae7c661, 90 },
    { 0xf6b7c662, 81 },
    { 0xfe87c662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7c668, 91 },
    { 0xeda7c668, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd87c66d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca07c670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4907c673, 61 },
    { 0xbe67c673, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb37c67a, 43 },
    { 0xe207c67a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa287c67e, 90 },
    { 0, 0 },
    { 0x6767c680, 76 },
    { 0xf7b7c681, 61 },
    { 0, 0 },
    { 0xa087c683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab47c686, 23 },
    { 0xa767c687, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e97c68a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a7c68d, 96 },
    { 0xb8c7c68e, 61 },
    { 0, 0 },
    { 0x5257c690, 66 },
    { 0, 0 },
    { 0x5d17c692, 76 },
    { 0, 0 },
    { 0x85c7c694, 98 },
    { 0, 0 },
    { 0x39f7c696, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d7c699, 98 },
    { 0, 0 },
    { 0x9a17c69b, 17 },
    { 0x68f7c69c, 61 },
    { 0xc877c69d, 88 },
    { 0x27a7c69d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b7c6a4, 61 },
    { 0xffd7c6a5, 23 },
    { 0x70b7c6a5, 90 },
    { 0x81f7c6a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f7c6b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb517c6b3, 33 },
    { 0, 0 },
    { 0x6567c6b5, 76 },
    { 0x3c47c6b5, 61 },
    { 0x2c97c6b5, 23 },
    { 0, 0 },
    { 0x40a7c6b9, 98 },
    { 0xa337c6ba, 88 },
    { 0, 0 },
    { 0x6dc7c6bc, 90 },
    { 0, 0 },
    { 0x7757c6be, 61 },
    { 0xa817c6bf, 90 },
    { 0x9577c6c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc877c6c5, 67 },
    { 0x6877c6c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4337c6c9, 52 },
    { 0x3b07c6ca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf7c6d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4667c6d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd897c6d8, 84 },
    { 0xce57c6d9, 17 },
    { 0, 0 },
    { 0x6937c6db, 55 },
    { 0x57b7c6db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec7c6e0, 75 },
    { 0x8367c6e1, 75 },
    { 0xf5a7c6e2, 85 },
    { 0x5c57c6e2, 99 },
    { 0xe687c6e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67c6ea, 85 },
    { 0, 0 },
    { 0x54c7c6ec, 100 },
    { 0x2d97c6ec, 90 },
    { 0x96f7c6ee, 46 },
    { 0, 0 },
    { 0xbc07c6f0, 69 },
    { 0xd0f7c6f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b7c6f5, 33 },
    { 0xdc37c6f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8727c6fa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217c704, 84 },
    { 0xec67c704, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd7c708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7747c70f, 23 },
    { 0, 0 },
    { 0x10b7c711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab47c714, 76 },
    { 0, 0 },
    { 0xe487c716, 99 },
    { 0x30f7c716, 61 },
    { 0, 0 },
    { 0xfce7c719, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e7c71e, 23 },
    { 0, 0 },
    { 0x8067c720, 91 },
    { 0, 0 },
    { 0x32c7c722, 100 },
    { 0xb527c722, 33 },
    { 0, 0 },
    { 0xb607c725, 95 },
    { 0x70d7c726, 91 },
    { 0xe0b7c725, 61 },
    { 0xe927c728, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c17c72c, 61 },
    { 0x68f7c72d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9707c734, 76 },
    { 0x9867c735, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d7c738, 88 },
    { 0x6f37c738, 55 },
    { 0, 0 },
    { 0x747c73b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab7c741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a7c746, 90 },
    { 0x327c747, 95 },
    { 0xfd37c748, 23 },
    { 0xec97c747, 84 },
    { 0x22e7c746, 43 },
    { 0x3037c74b, 66 },
    { 0, 0 },
    { 0x4be7c74d, 98 },
    { 0, 0 },
    { 0x16c7c74f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2667c755, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5557c759, 55 },
    { 0x7197c75a, 23 },
    { 0xce67c75b, 61 },
    { 0xea77c75c, 52 },
    { 0x2b77c759, 78 },
    { 0, 0 },
    { 0xea97c75f, 52 },
    { 0x1157c760, 95 },
    { 0x21b7c761, 76 },
    { 0, 0 },
    { 0xc8a7c763, 95 },
    { 0xf687c763, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2897c768, 34 },
    { 0x15f7c769, 33 },
    { 0x1027c76a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8477c76e, 98 },
    { 0, 0 },
    { 0xe377c770, 33 },
    { 0, 0 },
    { 0xdba7c772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b67c77b, 52 },
    { 0xf957c77b, 34 },
    { 0, 0 },
    { 0x6f87c77e, 76 },
    { 0xb27c77e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb077c787, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af7c793, 90 },
    { 0x9957c793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f7c798, 95 },
    { 0x2147c799, 98 },
    { 0x9607c79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf827c7a1, 61 },
    { 0xac77c7a2, 90 },
    { 0, 0 },
    { 0xf427c7a4, 90 },
    { 0, 0 },
    { 0x2767c7a6, 98 },
    { 0x83d7c7a7, 33 },
    { 0x4317c7a8, 91 },
    { 0, 0 },
    { 0x3017c7aa, 33 },
    { 0x9b77c7aa, 23 },
    { 0x1667c7ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca07c7af, 61 },
    { 0, 0 },
    { 0x90c7c7b1, 66 },
    { 0xe347c7b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7617c7b7, 76 },
    { 0, 0 },
    { 0xcf47c7b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f67c7be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d67c7c3, 34 },
    { 0, 0 },
    { 0x1967c7c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7037c7cc, 90 },
    { 0x1767c7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe307c7d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b7c7d5, 34 },
    { 0x2337c7d6, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d17c7db, 95 },
    { 0xfd7c7dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7c7e2, 87 },
    { 0xe707c7e2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e7c7e6, 91 },
    { 0x6167c7e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f97c7ea, 76 },
    { 0xd567c7ea, 93 },
    { 0, 0 },
    { 0x3d67c7ed, 90 },
    { 0xa287c7ed, 95 },
    { 0xeaa7c7ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be7c7f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e37c7ff, 78 },
    { 0x9177c7ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb57c807, 95 },
    { 0, 0 },
    { 0x19f7c809, 98 },
    { 0, 0 },
    { 0xa977c80b, 66 },
    { 0xf777c80c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3427c811, 61 },
    { 0x21e7c812, 61 },
    { 0x4e87c813, 23 },
    { 0xd077c814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c7c817, 94 },
    { 0x50e7c817, 95 },
    { 0x9a57c819, 67 },
    { 0xf5e7c81a, 87 },
    { 0, 0 },
    { 0x1707c81c, 95 },
    { 0, 0 },
    { 0x9007c81e, 61 },
    { 0, 0 },
    { 0x2f37c820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc7c823, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e47c827, 61 },
    { 0x2d7c827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1307c831, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea27c835, 93 },
    { 0x49e7c836, 61 },
    { 0, 0 },
    { 0xa647c838, 61 },
    { 0x8ba7c839, 98 },
    { 0x4de7c83a, 66 },
    { 0xc437c83b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b77c83f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d7c843, 84 },
    { 0, 0 },
    { 0xab67c845, 90 },
    { 0, 0 },
    { 0xcc27c847, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf887c84e, 55 },
    { 0, 0 },
    { 0xaed7c850, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb57c854, 33 },
    { 0, 0 },
    { 0xb1b7c856, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe7c859, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd587c85c, 77 },
    { 0x10d7c85c, 61 },
    { 0x1637c85c, 67 },
    { 0xc5a7c85d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e37c864, 84 },
    { 0xaf67c865, 34 },
    { 0x47b7c866, 95 },
    { 0x8c07c867, 98 },
    { 0xfa77c868, 90 },
    { 0, 0 },
    { 0x3837c86a, 33 },
    { 0xf407c86b, 61 },
    { 0x1da7c86c, 76 },
    { 0x6647c86c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea7c873, 33 },
    { 0x5087c873, 34 },
    { 0, 0 },
    { 0x6387c876, 66 },
    { 0x517c877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed7c87a, 90 },
    { 0x38e7c87b, 90 },
    { 0x3807c87c, 90 },
    { 0x53f7c87d, 17 },
    { 0xfaa7c87d, 95 },
    { 0x2287c87f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f77c884, 33 },
    { 0, 0 },
    { 0x1677c886, 22 },
    { 0, 0 },
    { 0xcd47c888, 61 },
    { 0x4aa7c888, 80 },
    { 0x7577c88a, 90 },
    { 0xac17c88b, 100 },
    { 0, 0 },
    { 0x4e47c88d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5697c892, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e7c895, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e37c8a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c8a5, 90 },
    { 0xf707c8a5, 76 },
    { 0x2307c8a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed7c8ad, 81 },
    { 0x6dd7c8ae, 34 },
    { 0xb5c7c8af, 90 },
    { 0x1397c8b0, 87 },
    { 0xf7c7c8b0, 99 },
    { 0x9d37c8ae, 33 },
    { 0xad97c8ae, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b7c8b8, 85 },
    { 0x53d7c8b9, 77 },
    { 0x3677c8ba, 95 },
    { 0x7757c8ba, 90 },
    { 0, 0 },
    { 0xbda7c8bd, 61 },
    { 0, 0 },
    { 0x5c27c8bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7c8c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe7c8c8, 93 },
    { 0x8467c8c9, 76 },
    { 0x5f37c8c8, 77 },
    { 0x1287c8cb, 55 },
    { 0x1dd7c8cc, 65 },
    { 0, 0 },
    { 0x45e7c8ce, 90 },
    { 0x85e7c8cf, 98 },
    { 0x9707c8d0, 52 },
    { 0x2df7c8d0, 95 },
    { 0, 0 },
    { 0x5ef7c8d3, 34 },
    { 0x9c77c8d4, 90 },
    { 0, 0 },
    { 0x7b17c8d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3237c8d9, 55 },
    { 0xd2b7c8d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae7c8e5, 98 },
    { 0, 0 },
    { 0x8e87c8e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9607c8ea, 33 },
    { 0xc747c8eb, 43 },
    { 0xe077c8eb, 95 },
    { 0xb5e7c8ed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe547c8f1, 91 },
    { 0x9b17c8f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee7c8f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea57c8fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff67c901, 98 },
    { 0x6607c901, 98 },
    { 0xcff7c901, 34 },
    { 0xf967c902, 60 },
    { 0x7d47c905, 81 },
    { 0x18c7c906, 33 },
    { 0x9157c907, 61 },
    { 0xa607c907, 52 },
    { 0, 0 },
    { 0xde87c90a, 69 },
    { 0, 0 },
    { 0x6597c90c, 84 },
    { 0x2777c90d, 34 },
    { 0xa07c90d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07c911, 23 },
    { 0xe3a7c912, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc007c916, 90 },
    { 0x7697c917, 98 },
    { 0x25b7c918, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd847c91b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c7c921, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa077c927, 90 },
    { 0xe907c928, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e87c92b, 61 },
    { 0x8e47c92b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07c931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4367c935, 95 },
    { 0x6607c936, 98 },
    { 0x4a97c936, 95 },
    { 0x5137c936, 36 },
    { 0x7f7c938, 85 },
    { 0, 0 },
    { 0x92d7c93b, 23 },
    { 0, 0 },
    { 0x5db7c93d, 98 },
    { 0, 0 },
    { 0xde37c93f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7c947, 61 },
    { 0xb5b7c947, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c37c94c, 43 },
    { 0x9c37c94d, 61 },
    { 0x7157c94e, 23 },
    { 0xaa47c94f, 43 },
    { 0, 0 },
    { 0x1767c951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b7c954, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc87c958, 95 },
    { 0x5337c959, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b7c95f, 95 },
    { 0xb097c960, 31 },
    { 0, 0 },
    { 0x8777c962, 90 },
    { 0x9c7c963, 33 },
    { 0xa6b7c964, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb7c968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47c96b, 61 },
    { 0x8d87c96b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7c96f, 88 },
    { 0x5c07c970, 33 },
    { 0xc317c970, 2 },
    { 0x6b17c972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5127c975, 34 },
    { 0x3157c976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe837c983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb7c986, 95 },
    { 0, 0 },
    { 0xa867c988, 75 },
    { 0xc867c989, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b7c990, 17 },
    { 0x5a17c991, 93 },
    { 0x8f7c992, 55 },
    { 0x17b7c993, 34 },
    { 0x2c7c994, 90 },
    { 0xf797c993, 93 },
    { 0xcf17c990, 90 },
    { 0, 0 },
    { 0x5327c998, 33 },
    { 0, 0 },
    { 0x7817c99a, 95 },
    { 0, 0 },
    { 0x9157c99c, 90 },
    { 0xd2c7c99c, 23 },
    { 0x6957c99d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd7c9a1, 68 },
    { 0x6097c9a1, 19 },
    { 0, 0 },
    { 0x7fb7c9a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f37c9a8, 90 },
    { 0, 0 },
    { 0xdcd7c9aa, 90 },
    { 0, 0 },
    { 0xa217c9ac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad97c9b3, 90 },
    { 0, 0 },
    { 0xde87c9b5, 33 },
    { 0x1cf7c9b6, 85 },
    { 0x8397c9b6, 23 },
    { 0, 0 },
    { 0xd3d7c9b9, 98 },
    { 0, 0 },
    { 0x8b87c9bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2867c9be, 34 },
    { 0, 0 },
    { 0x85b7c9c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe757c9c3, 52 },
    { 0x4b27c9c3, 95 },
    { 0xe207c9c5, 23 },
    { 0xf807c9c3, 90 },
    { 0, 0 },
    { 0x1e17c9c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67c9ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe47c9d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b7c9d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b27c9dc, 98 },
    { 0, 0 },
    { 0xe7f7c9de, 93 },
    { 0xe737c9df, 61 },
    { 0x6377c9e0, 61 },
    { 0x9847c9df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a7c9e9, 23 },
    { 0x9707c9ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa047c9f5, 93 },
    { 0x217c9f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a7c9fa, 93 },
    { 0, 0 },
    { 0x2967c9fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f17ca01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea7ca05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a7ca08, 55 },
    { 0xd4e7ca09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237ca0d, 23 },
    { 0x4ce7ca0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc7ca11, 46 },
    { 0x5167ca12, 90 },
    { 0xda57ca13, 33 },
    { 0xffd7ca14, 55 },
    { 0x7f17ca15, 98 },
    { 0x9767ca16, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd337ca19, 95 },
    { 0, 0 },
    { 0x4437ca1b, 90 },
    { 0, 0 },
    { 0x89a7ca1d, 98 },
    { 0x43f7ca1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ca22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d7ca26, 80 },
    { 0x6577ca27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe7ca31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb567ca39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7187ca3c, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f7ca40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf7ca43, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6577ca46, 78 },
    { 0x5897ca47, 55 },
    { 0xc9f7ca48, 98 },
    { 0x9997ca49, 52 },
    { 0xcff7ca48, 33 },
    { 0xa677ca48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f7ca51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a7ca5c, 93 },
    { 0x9b07ca5d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd7ca62, 98 },
    { 0, 0 },
    { 0x3817ca64, 55 },
    { 0x8c87ca65, 98 },
    { 0x9917ca66, 75 },
    { 0x4137ca66, 78 },
    { 0xa707ca66, 93 },
    { 0, 0 },
    { 0x2027ca6a, 91 },
    { 0x81b7ca6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb587ca6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd7ca77, 46 },
    { 0, 0 },
    { 0x81f7ca79, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3387ca7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea7ca81, 76 },
    { 0x26e7ca81, 96 },
    { 0x27b7ca83, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9187ca88, 23 },
    { 0xa9c7ca88, 61 },
    { 0x9b37ca88, 84 },
    { 0x617ca8b, 90 },
    { 0x547ca8b, 90 },
    { 0x5057ca8c, 76 },
    { 0x3107ca8e, 75 },
    { 0x1ba7ca8f, 33 },
    { 0xb0d7ca8f, 33 },
    { 0, 0 },
    { 0xa8a7ca92, 61 },
    { 0, 0 },
    { 0xb527ca94, 61 },
    { 0, 0 },
    { 0x9457ca96, 61 },
    { 0xe667ca97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe337ca9d, 80 },
    { 0, 0 },
    { 0xecd7ca9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd787caa3, 97 },
    { 0x49e7caa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6037caa8, 43 },
    { 0xc6c7caa9, 38 },
    { 0x2377caaa, 55 },
    { 0x8227caa9, 91 },
    { 0, 0 },
    { 0x9d77caad, 37 },
    { 0, 0 },
    { 0x8767caaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d7cabc, 75 },
    { 0x3037cabc, 95 },
    { 0x1db7cabd, 90 },
    { 0x4b77cabd, 60 },
    { 0xbea7cac0, 67 },
    { 0x8bd7cabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9507cacd, 66 },
    { 0, 0 },
    { 0xfc7cacf, 61 },
    { 0x4db7cad0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b7cad3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda07cada, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe557cae1, 95 },
    { 0xe937cae2, 98 },
    { 0x33e7cae3, 34 },
    { 0xd7a7cae1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b47cae8, 34 },
    { 0, 0 },
    { 0x2757caea, 33 },
    { 0, 0 },
    { 0xfa17caec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2637caf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7cafc, 34 },
    { 0x96c7cafc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c7cb03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67cb08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b17cb0f, 77 },
    { 0x1c37cb0f, 85 },
    { 0xdfe7cb10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd547cb16, 33 },
    { 0x6af7cb16, 23 },
    { 0xaea7cb18, 55 },
    { 0x45d7cb16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb47cb1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c27cb20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9957cb25, 55 },
    { 0x9847cb26, 98 },
    { 0x9d37cb27, 36 },
    { 0x25e7cb28, 85 },
    { 0x5f7cb28, 90 },
    { 0x4e67cb28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6377cb2d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf607cb30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e67cb34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7cb38, 90 },
    { 0x5527cb38, 61 },
    { 0, 0 },
    { 0x1fe7cb3b, 53 },
    { 0, 0 },
    { 0x98f7cb3d, 61 },
    { 0x2d77cb3d, 90 },
    { 0x8a97cb3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657cb49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb7cb54, 93 },
    { 0x26c7cb55, 99 },
    { 0x7307cb55, 33 },
    { 0x4487cb55, 55 },
    { 0xd457cb56, 34 },
    { 0x3ca7cb59, 33 },
    { 0, 0 },
    { 0x287cb5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d7cb65, 95 },
    { 0, 0 },
    { 0xcfc7cb67, 76 },
    { 0x90c7cb68, 90 },
    { 0x7fb7cb67, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57cb6e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec7cb76, 95 },
    { 0, 0 },
    { 0x2ba7cb78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df7cb7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3337cb7e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7cb82, 78 },
    { 0xd657cb82, 76 },
    { 0xc757cb83, 90 },
    { 0xddc7cb83, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f7cb8a, 23 },
    { 0, 0 },
    { 0x63c7cb8c, 46 },
    { 0x17cb8d, 95 },
    { 0x3d07cb8d, 98 },
    { 0xeb87cb8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1087cb95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd07cb9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7cba0, 34 },
    { 0x2267cba1, 55 },
    { 0x9b47cba2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907cba5, 55 },
    { 0x50f7cba5, 61 },
    { 0x63d7cba7, 23 },
    { 0x44b7cba7, 90 },
    { 0xe067cba6, 90 },
    { 0xc0e7cbaa, 90 },
    { 0x4b17cbab, 76 },
    { 0x7937cbaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc7cbb7, 90 },
    { 0x69d7cbb7, 55 },
    { 0x62d7cbb9, 90 },
    { 0x62f7cbb8, 76 },
    { 0x36e7cbbb, 76 },
    { 0xa107cbbb, 84 },
    { 0xe997cbbc, 87 },
    { 0x43d7cbb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1127cbc1, 78 },
    { 0x1a27cbc2, 23 },
    { 0x2397cbc3, 100 },
    { 0xc447cbc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7cbc9, 67 },
    { 0, 0 },
    { 0xbdd7cbcb, 23 },
    { 0x70e7cbcb, 61 },
    { 0x46e7cbcd, 91 },
    { 0x7607cbce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb957cbd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4047cbd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf857cbda, 98 },
    { 0, 0 },
    { 0xf017cbdc, 61 },
    { 0xd957cbdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4277cbe0, 98 },
    { 0xb147cbe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf7cbe5, 34 },
    { 0x2187cbe6, 95 },
    { 0x8077cbe5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2247cbf1, 78 },
    { 0x6387cbf1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e77cbf5, 46 },
    { 0x6387cbf6, 95 },
    { 0x6b7cbf7, 33 },
    { 0x1437cbf7, 76 },
    { 0x2667cbf9, 90 },
    { 0xbdb7cbfa, 98 },
    { 0xc1f7cbfb, 95 },
    { 0xa977cbfb, 100 },
    { 0x5687cbfd, 81 },
    { 0xc2c7cbfe, 90 },
    { 0x6c67cbff, 93 },
    { 0x4c87cc00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d7cc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd717cc0a, 23 },
    { 0, 0 },
    { 0x7aa7cc0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73e7cc10, 23 },
    { 0x1f7cc11, 55 },
    { 0x71f7cc11, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e57cc1a, 55 },
    { 0x8677cc1b, 75 },
    { 0, 0 },
    { 0x9f97cc1d, 17 },
    { 0x9e7cc1e, 55 },
    { 0x62f7cc1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e7cc23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b87cc2d, 61 },
    { 0, 0 },
    { 0xe877cc2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3917cc33, 90 },
    { 0x5577cc33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f47cc37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7cc3a, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xd097cc3d, 68 },
    { 0xc7cc3e, 55 },
    { 0xa297cc3f, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7417cc43, 53 },
    { 0, 0 },
    { 0x9c17cc45, 33 },
    { 0x6c07cc46, 52 },
    { 0, 0 },
    { 0x14e7cc48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe247cc4e, 95 },
    { 0x8947cc4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b17cc52, 95 },
    { 0x8e67cc53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4957cc56, 91 },
    { 0, 0 },
    { 0xc1f7cc58, 61 },
    { 0x4297cc59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa17cc60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb147cc64, 75 },
    { 0xf917cc65, 52 },
    { 0x3f27cc66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707cc69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab7cc6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b7cc74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd267cc77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c87cc7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa037cc83, 34 },
    { 0x9ed7cc84, 90 },
    { 0x9a07cc85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c7cc8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c17cc91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a7cc99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb27cc9e, 95 },
    { 0x4eb7cc9e, 43 },
    { 0xe447cca0, 90 },
    { 0xf557cca0, 90 },
    { 0x2577cca1, 90 },
    { 0x3cb7cca0, 95 },
    { 0xee97cca3, 93 },
    { 0xcd97cca1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a97cca8, 93 },
    { 0x7607cca9, 90 },
    { 0x45b7ccaa, 61 },
    { 0x5937ccab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6227ccb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7ccb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b7ccbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x827ccbf, 90 },
    { 0x1da7ccc0, 75 },
    { 0xb337ccc1, 76 },
    { 0x38e7ccc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b7ccc5, 61 },
    { 0xe877ccc6, 46 },
    { 0x3157ccc7, 61 },
    { 0x2d17ccc8, 23 },
    { 0xede7ccc6, 52 },
    { 0x8d47ccca, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xde87cccd, 43 },
    { 0xb317ccce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e7ccd5, 33 },
    { 0x6c07ccd5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf557ccdb, 34 },
    { 0xbe87ccdc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe7ccdf, 43 },
    { 0xf757cce0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e57cce8, 23 },
    { 0xb377cce9, 33 },
    { 0x907ccea, 95 },
    { 0, 0 },
    { 0x48b7ccec, 33 },
    { 0, 0 },
    { 0x3b67ccee, 61 },
    { 0x42f7ccef, 34 },
    { 0, 0 },
    { 0x48a7ccf1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f7ccf4, 33 },
    { 0x4ed7ccf5, 33 },
    { 0x5367ccf6, 95 },
    { 0x6e97ccf7, 99 },
    { 0x6a7ccf6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe387ccfe, 99 },
    { 0x18a7ccfe, 32 },
    { 0xb637ccfe, 90 },
    { 0x5ee7cd01, 90 },
    { 0xdeb7cd02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed97cd08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6917cd0c, 67 },
    { 0x3a67cd0d, 33 },
    { 0x7d47cd0d, 90 },
    { 0xa197cd0d, 83 },
    { 0x7be7cd0c, 84 },
    { 0xa967cd0c, 55 },
    { 0, 0 },
    { 0xcd07cd13, 55 },
    { 0x79a7cd14, 46 },
    { 0, 0 },
    { 0x1dc7cd16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3517cd1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea7cd20, 61 },
    { 0, 0 },
    { 0x8f77cd22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e7cd25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b7cd2a, 85 },
    { 0, 0 },
    { 0x8347cd2c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b07cd2f, 95 },
    { 0x5b47cd30, 90 },
    { 0x3087cd30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9757cd34, 52 },
    { 0x62d7cd34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb57cd3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8907cd40, 98 },
    { 0, 0 },
    { 0x9837cd42, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae7cd47, 33 },
    { 0, 0 },
    { 0xda77cd49, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8607cd4e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77cd53, 99 },
    { 0xf3c7cd54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3987cd57, 61 },
    { 0xce87cd58, 61 },
    { 0x837cd59, 61 },
    { 0x9fc7cd59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f7cd5f, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d7cd64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f7cd6c, 34 },
    { 0, 0 },
    { 0x44f7cd6e, 95 },
    { 0, 0 },
    { 0xc957cd70, 23 },
    { 0x3767cd70, 84 },
    { 0x53e7cd70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd697cd77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a47cd7d, 34 },
    { 0, 0 },
    { 0x2877cd7f, 95 },
    { 0x87e7cd7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x607cd83, 95 },
    { 0, 0 },
    { 0x807cd85, 33 },
    { 0, 0 },
    { 0xb817cd87, 23 },
    { 0xc587cd87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a7cd8b, 90 },
    { 0xd147cd8c, 55 },
    { 0x77e7cd8c, 23 },
    { 0xd577cd8e, 34 },
    { 0x3307cd8f, 61 },
    { 0x277cd90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7cd94, 95 },
    { 0xa257cd95, 90 },
    { 0xda7cd96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd27cd9e, 87 },
    { 0, 0 },
    { 0x2a67cda0, 43 },
    { 0x93f7cda0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec7cda7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa7cdaf, 36 },
    { 0xe037cdb0, 55 },
    { 0, 0 },
    { 0xbcc7cdb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc7cdb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3437cdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cdc4, 61 },
    { 0x9a47cdc4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177cdcb, 67 },
    { 0xe377cdcc, 34 },
    { 0x8f87cdcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b87cdd2, 99 },
    { 0x6f67cdd3, 34 },
    { 0, 0 },
    { 0xcbc7cdd5, 76 },
    { 0, 0 },
    { 0x8f37cdd7, 33 },
    { 0x7797cdd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1807cddb, 90 },
    { 0xf667cddc, 33 },
    { 0xdf07cddc, 96 },
    { 0xb127cddd, 95 },
    { 0xeb47cddf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf157cde4, 95 },
    { 0, 0 },
    { 0x5947cde6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b67cde9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb687cdec, 34 },
    { 0, 0 },
    { 0xe97cdee, 82 },
    { 0, 0 },
    { 0x47b7cdf0, 33 },
    { 0x1d37cdf1, 90 },
    { 0x7cd7cdf2, 95 },
    { 0xcfa7cdf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7cdf9, 98 },
    { 0x17b7cdfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb417cdfd, 93 },
    { 0, 0 },
    { 0x3747cdff, 33 },
    { 0xbf97ce00, 90 },
    { 0xc937ce00, 61 },
    { 0xfc97ce02, 33 },
    { 0xd227cdff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57ce07, 90 },
    { 0x31e7ce07, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b7ce0b, 78 },
    { 0, 0 },
    { 0x647ce0d, 33 },
    { 0, 0 },
    { 0x8777ce0f, 87 },
    { 0x7f57ce10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6097ce19, 43 },
    { 0x58e7ce19, 98 },
    { 0x41d7ce1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7ce1f, 34 },
    { 0xe427ce20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147ce23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5237ce26, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5167ce30, 33 },
    { 0xe937ce31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff7ce35, 98 },
    { 0xc7e7ce35, 23 },
    { 0xc5d7ce37, 90 },
    { 0x7897ce36, 90 },
    { 0xa507ce37, 98 },
    { 0x67f7ce3a, 95 },
    { 0xa947ce3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b7ce40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7ce44, 95 },
    { 0x7017ce44, 90 },
    { 0xf9f7ce45, 33 },
    { 0x2f47ce47, 90 },
    { 0xf607ce45, 24 },
    { 0x8487ce49, 76 },
    { 0, 0 },
    { 0xf07ce4b, 76 },
    { 0, 0 },
    { 0x64a7ce4d, 23 },
    { 0x8067ce4e, 46 },
    { 0xd3b7ce4f, 95 },
    { 0x22e7ce4f, 98 },
    { 0, 0 },
    { 0xb9b7ce52, 87 },
    { 0x2ce7ce52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb87ce5a, 85 },
    { 0, 0 },
    { 0x1267ce5c, 98 },
    { 0xf9c7ce5c, 90 },
    { 0x2257ce5d, 93 },
    { 0xbf07ce5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3647ce63, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc537ce67, 88 },
    { 0, 0 },
    { 0x44a7ce69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7ce6e, 46 },
    { 0x5ec7ce6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b7ce72, 46 },
    { 0x6e67ce72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7ce7c, 43 },
    { 0x3af7ce7c, 95 },
    { 0x5f17ce7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8207ce82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb297ce87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc67ce8c, 87 },
    { 0, 0 },
    { 0xb4c7ce8e, 66 },
    { 0, 0 },
    { 0x6867ce90, 90 },
    { 0x4ae7ce90, 61 },
    { 0xe8c7ce90, 95 },
    { 0xeb37ce91, 61 },
    { 0x1527ce90, 46 },
    { 0xbd67ce95, 55 },
    { 0x6f87ce93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc817ce9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d7ceae, 67 },
    { 0x6cc7ceaf, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6127ceb3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad87cebb, 84 },
    { 0, 0 },
    { 0xb9e7cebd, 34 },
    { 0x6e87cebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47cec1, 33 },
    { 0, 0 },
    { 0x7cd7cec3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b7cec6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3397cec9, 90 },
    { 0x1a97ceca, 90 },
    { 0x1507cecb, 77 },
    { 0, 0 },
    { 0x8e27cecd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb97ced0, 23 },
    { 0x5c67ced0, 99 },
    { 0x307ced2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d7ced9, 34 },
    { 0xaf87ced9, 34 },
    { 0x7f27cedb, 98 },
    { 0xfc97cedb, 98 },
    { 0x4ad7cedd, 90 },
    { 0x4557cedd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257cee1, 98 },
    { 0xc3c7cee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d57cee5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e7ceea, 85 },
    { 0x5bd7ceea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7ceee, 34 },
    { 0, 0 },
    { 0x6b67cef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e57cef3, 90 },
    { 0x3cd7cef4, 76 },
    { 0x7b17cef5, 77 },
    { 0x7b67cef5, 93 },
    { 0x6237cef7, 49 },
    { 0x4d97cef7, 61 },
    { 0x2e47cef5, 90 },
    { 0xce27cef3, 55 },
    { 0x7657cefb, 95 },
    { 0, 0 },
    { 0x36f7cefd, 93 },
    { 0x9f97cefe, 76 },
    { 0xd2c7ceff, 98 },
    { 0xba7ceff, 90 },
    { 0x3837cf00, 61 },
    { 0x5b27cefe, 93 },
    { 0x7527cf03, 33 },
    { 0x4277cf04, 90 },
    { 0, 0 },
    { 0x7ba7cf06, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e7cf09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd287cf0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6137cf10, 61 },
    { 0x44d7cf11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7cf15, 56 },
    { 0xed77cf15, 95 },
    { 0xea07cf17, 33 },
    { 0x7c07cf16, 91 },
    { 0x9d27cf19, 40 },
    { 0x5a07cf19, 69 },
    { 0x34f7cf19, 33 },
    { 0xcac7cf1a, 95 },
    { 0xe797cf1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xac97cf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5207cf26, 95 },
    { 0x3827cf27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e57cf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb7cf4c, 93 },
    { 0x8b67cf4d, 33 },
    { 0x2c37cf4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f7cf54, 98 },
    { 0x7c7cf55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d7cf58, 100 },
    { 0x1007cf58, 33 },
    { 0, 0 },
    { 0x2607cf5b, 90 },
    { 0xb567cf5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd987cf61, 43 },
    { 0, 0 },
    { 0xde7cf63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b67cf69, 98 },
    { 0x8a67cf69, 43 },
    { 0x3d67cf6b, 23 },
    { 0x1a97cf6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf47cf6f, 90 },
    { 0, 0 },
    { 0x7f67cf71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a7cf76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c7cf7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7cf7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7627cf81, 76 },
    { 0xfbf7cf81, 34 },
    { 0xba47cf83, 61 },
    { 0xc2c7cf84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427cf8e, 34 },
    { 0x5ac7cf8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf197cf92, 34 },
    { 0, 0 },
    { 0xbb37cf94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57cf97, 90 },
    { 0, 0 },
    { 0x38d7cf99, 90 },
    { 0xe337cf9a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b7cf9e, 98 },
    { 0, 0 },
    { 0x4907cfa0, 55 },
    { 0xadf7cfa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f7cfa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6847cfa8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d7cfab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de7cfae, 98 },
    { 0x56b7cfaf, 76 },
    { 0, 0 },
    { 0xc227cfb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e17cfb4, 34 },
    { 0, 0 },
    { 0xc7c7cfb6, 91 },
    { 0x7177cfb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3037cfba, 93 },
    { 0, 0 },
    { 0xaac7cfbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c7cfbf, 61 },
    { 0x6b67cfc0, 84 },
    { 0x3797cfc1, 46 },
    { 0x2287cfc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277cfc7, 55 },
    { 0x6cc7cfc7, 95 },
    { 0xa797cfc9, 76 },
    { 0x7027cfc7, 98 },
    { 0x3ce7cfcb, 90 },
    { 0xd0f7cfcc, 46 },
    { 0, 0 },
    { 0x31b7cfce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d17cfd3, 33 },
    { 0x32e7cfd3, 91 },
    { 0xb187cfd5, 23 },
    { 0xe3b7cfd6, 98 },
    { 0xcc17cfd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd7cfda, 98 },
    { 0, 0 },
    { 0x8397cfdc, 90 },
    { 0x2d07cfdd, 99 },
    { 0, 0 },
    { 0x73e7cfdf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cfe3, 61 },
    { 0xc077cfe4, 76 },
    { 0x5c97cfe5, 90 },
    { 0x6d77cfe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf827cfe9, 46 },
    { 0, 0 },
    { 0xf207cfeb, 33 },
    { 0x597cfec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d37cff0, 66 },
    { 0x5587cff0, 69 },
    { 0xe167cff2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb507cffa, 91 },
    { 0x987cffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d7cfff, 64 },
    { 0xfbd7cfff, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27d009, 67 },
    { 0xa117d00a, 75 },
    { 0x3de7d00a, 76 },
    { 0x5037d009, 95 },
    { 0xcda7d00d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d77d017, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6427d01b, 85 },
    { 0x3f97d01c, 33 },
    { 0, 0 },
    { 0x1227d01e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d023, 95 },
    { 0, 0 },
    { 0xcfe7d025, 52 },
    { 0xf247d025, 61 },
    { 0xd197d027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f57d02b, 95 },
    { 0xbfc7d02c, 98 },
    { 0xba27d02d, 61 },
    { 0x4c07d02b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee7d031, 69 },
    { 0x2987d032, 98 },
    { 0x6337d032, 77 },
    { 0x67a7d033, 55 },
    { 0xea47d033, 61 },
    { 0x8777d036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb207d039, 33 },
    { 0x6997d03a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7d03f, 95 },
    { 0, 0 },
    { 0x29d7d041, 34 },
    { 0xaac7d042, 93 },
    { 0, 0 },
    { 0xf947d044, 81 },
    { 0x9987d045, 98 },
    { 0x6817d046, 33 },
    { 0x1337d046, 34 },
    { 0x46c7d048, 23 },
    { 0x53a7d048, 95 },
    { 0x39b7d044, 85 },
    { 0x4577d046, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb67d04e, 88 },
    { 0x61d7d04f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7d052, 98 },
    { 0x8047d053, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb817d057, 46 },
    { 0xe1f7d058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6747d05c, 76 },
    { 0xa067d05d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a7d064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9957d069, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd607d06d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2377d071, 34 },
    { 0xd9d7d071, 95 },
    { 0x5c47d073, 98 },
    { 0xfc97d072, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f7d077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c7d07d, 77 },
    { 0xf27d07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6017d081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8787d085, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c7d090, 98 },
    { 0x3317d091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b7d099, 61 },
    { 0x1047d09a, 65 },
    { 0x61a7d09a, 95 },
    { 0, 0 },
    { 0x39c7d09d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b7d0a2, 61 },
    { 0x3107d0a3, 98 },
    { 0x5d37d0a4, 87 },
    { 0x3427d0a2, 99 },
    { 0x2027d0a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1957d0b0, 52 },
    { 0, 0 },
    { 0x3167d0b2, 90 },
    { 0xecc7d0b3, 98 },
    { 0, 0 },
    { 0x6bd7d0b5, 93 },
    { 0, 0 },
    { 0x1537d0b7, 64 },
    { 0, 0 },
    { 0x4ba7d0b9, 61 },
    { 0x1087d0ba, 76 },
    { 0xb4d7d0bb, 95 },
    { 0, 0 },
    { 0x76f7d0bd, 90 },
    { 0x4be7d0bd, 76 },
    { 0xda27d0bd, 90 },
    { 0x4d67d0c0, 23 },
    { 0x8007d0c1, 90 },
    { 0x87c7d0bf, 95 },
    { 0, 0 },
    { 0x94a7d0c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f7d0c7, 34 },
    { 0x9fd7d0c7, 33 },
    { 0x7e57d0c9, 17 },
    { 0x8357d0ca, 65 },
    { 0, 0 },
    { 0xaf17d0cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd7d0d4, 82 },
    { 0x1057d0d5, 95 },
    { 0x21a7d0d5, 55 },
    { 0, 0 },
    { 0xa777d0d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a7d0db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4557d0de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b7d0e6, 90 },
    { 0xbaf7d0e7, 61 },
    { 0x1837d0e8, 90 },
    { 0x97c7d0e9, 23 },
    { 0, 0 },
    { 0x47e7d0eb, 33 },
    { 0xac27d0eb, 34 },
    { 0x6d27d0ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f17d0f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb357d0f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e7d0fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf407d0ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc677d102, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b7d107, 91 },
    { 0x6307d107, 55 },
    { 0, 0 },
    { 0xcba7d10a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d10d, 33 },
    { 0xc397d10d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047d112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe647d116, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d7d11c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4737d121, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c67d125, 43 },
    { 0x6cb7d126, 95 },
    { 0, 0 },
    { 0xe307d128, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4647d137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de7d13c, 95 },
    { 0x5a87d13c, 61 },
    { 0xc457d13e, 93 },
    { 0, 0 },
    { 0xe537d140, 55 },
    { 0xb037d140, 81 },
    { 0x61e7d142, 52 },
    { 0x3d17d143, 98 },
    { 0xda07d140, 76 },
    { 0xfdf7d145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e7d148, 90 },
    { 0, 0 },
    { 0xb247d14a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed7d14f, 33 },
    { 0xe007d150, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5497d153, 98 },
    { 0x58f7d154, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b87d157, 23 },
    { 0, 0 },
    { 0xd667d159, 90 },
    { 0xc727d15a, 81 },
    { 0xcb27d159, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4967d160, 23 },
    { 0x12c7d161, 88 },
    { 0x8807d162, 52 },
    { 0, 0 },
    { 0xa177d164, 34 },
    { 0x1727d165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce7d16b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9697d16f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe937d174, 90 },
    { 0xe507d174, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d17d, 98 },
    { 0, 0 },
    { 0xc427d17f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb7d184, 23 },
    { 0x18e7d184, 61 },
    { 0x2147d185, 66 },
    { 0x4887d186, 55 },
    { 0, 0 },
    { 0xd0d7d189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad97d194, 34 },
    { 0x54e7d194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8247d19a, 23 },
    { 0, 0 },
    { 0xa647d19c, 67 },
    { 0x9d17d19d, 85 },
    { 0, 0 },
    { 0x77e7d19f, 61 },
    { 0x4f17d1a0, 99 },
    { 0xdb47d1a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb037d1a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b07d1a8, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xac87d1ab, 84 },
    { 0x6b97d1ab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae57d1b0, 55 },
    { 0, 0 },
    { 0x6e47d1b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b7d1b5, 90 },
    { 0xd437d1b6, 95 },
    { 0xd877d1b7, 33 },
    { 0x8587d1b6, 61 },
    { 0x6627d1b9, 46 },
    { 0, 0 },
    { 0x92a7d1bb, 77 },
    { 0x3f97d1bc, 61 },
    { 0, 0 },
    { 0x2bc7d1be, 55 },
    { 0x8fe7d1be, 34 },
    { 0x2fc7d1c0, 76 },
    { 0x5337d1c1, 95 },
    { 0, 0 },
    { 0x5be7d1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d17d1c8, 90 },
    { 0x89a7d1c9, 95 },
    { 0x2617d1c8, 76 },
    { 0x1717d1c8, 88 },
    { 0x9027d1cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d1d0, 90 },
    { 0x2da7d1d0, 98 },
    { 0x54b7d1d2, 55 },
    { 0xfd87d1d1, 90 },
    { 0, 0 },
    { 0xb347d1d5, 23 },
    { 0xa2a7d1d6, 98 },
    { 0x6bb7d1d7, 90 },
    { 0x4c57d1d5, 34 },
    { 0x4a17d1d9, 33 },
    { 0xe4f7d1d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f7d1de, 46 },
    { 0, 0 },
    { 0xbac7d1e0, 42 },
    { 0x2be7d1e0, 33 },
    { 0xd377d1e2, 98 },
    { 0, 0 },
    { 0x4ee7d1e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4457d1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa677d1ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc97d1fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac7d1fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d07d201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d57d204, 99 },
    { 0x3b47d205, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7d208, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff37d20c, 55 },
    { 0x67d20d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2917d210, 33 },
    { 0x7b47d211, 43 },
    { 0, 0 },
    { 0xf437d213, 95 },
    { 0x78d7d214, 98 },
    { 0, 0 },
    { 0x7547d216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9887d21f, 90 },
    { 0xc8f7d220, 100 },
    { 0xe397d220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1677d228, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a57d231, 46 },
    { 0xf6f7d231, 23 },
    { 0x88a7d232, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7d238, 61 },
    { 0x8987d239, 98 },
    { 0x54e7d23a, 23 },
    { 0xa0e7d23b, 78 },
    { 0x1257d238, 81 },
    { 0x2237d23d, 43 },
    { 0xad67d23b, 95 },
    { 0x6fe7d23f, 61 },
    { 0xa107d240, 23 },
    { 0x7c57d241, 85 },
    { 0x83f7d241, 55 },
    { 0x7447d243, 33 },
    { 0, 0 },
    { 0x7207d245, 23 },
    { 0xdb57d246, 34 },
    { 0x7357d247, 66 },
    { 0xf057d245, 33 },
    { 0x7837d249, 99 },
    { 0xdea7d249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd7d24f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b7d252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a7d255, 98 },
    { 0x1da7d255, 90 },
    { 0x8a27d256, 98 },
    { 0xe837d257, 95 },
    { 0x9ae7d259, 43 },
    { 0x9f67d259, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c57d25e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27d263, 100 },
    { 0x307d264, 23 },
    { 0x8b17d265, 23 },
    { 0xac97d266, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c7d26e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe527d271, 93 },
    { 0x667d272, 90 },
    { 0xb137d272, 100 },
    { 0x5f7d274, 95 },
    { 0, 0 },
    { 0x19d7d276, 95 },
    { 0xcf47d276, 46 },
    { 0x16f7d276, 95 },
    { 0x4237d279, 33 },
    { 0x9017d279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a7d27d, 99 },
    { 0xe97d27e, 90 },
    { 0, 0 },
    { 0xb477d280, 55 },
    { 0xe607d281, 96 },
    { 0xb4e7d282, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa827d285, 55 },
    { 0x8db7d286, 76 },
    { 0, 0 },
    { 0x4a07d288, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c47d292, 96 },
    { 0x9197d292, 85 },
    { 0xc0a7d294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb377d297, 74 },
    { 0, 0 },
    { 0x5067d299, 33 },
    { 0, 0 },
    { 0x2e67d29b, 66 },
    { 0x4bc7d29b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1577d29f, 77 },
    { 0, 0 },
    { 0x9647d2a1, 43 },
    { 0, 0 },
    { 0xbaf7d2a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947d2aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc7d2ae, 55 },
    { 0x85f7d2af, 46 },
    { 0xaa97d2b0, 90 },
    { 0xb807d2b1, 61 },
    { 0x6c97d2b0, 98 },
    { 0xac87d2ae, 34 },
    { 0, 0 },
    { 0xfff7d2b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe207d2ba, 66 },
    { 0, 0 },
    { 0x1de7d2bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe197d2c0, 91 },
    { 0x1897d2c1, 61 },
    { 0x9e17d2c2, 23 },
    { 0, 0 },
    { 0xde87d2c4, 98 },
    { 0x9ab7d2c5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817d2cb, 33 },
    { 0xa4c7d2cb, 91 },
    { 0x2a87d2cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf7d2d4, 90 },
    { 0x1c97d2d5, 93 },
    { 0xbe97d2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf7d2d9, 65 },
    { 0x3e67d2d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1767d2df, 61 },
    { 0, 0 },
    { 0x2cd7d2e1, 90 },
    { 0, 0 },
    { 0x8587d2e3, 76 },
    { 0, 0 },
    { 0xaba7d2e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa437d2eb, 64 },
    { 0x23d7d2ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f47d2f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe187d2f4, 78 },
    { 0x5967d2f5, 91 },
    { 0xd087d2f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7d2f9, 33 },
    { 0x6ac7d2fa, 33 },
    { 0, 0 },
    { 0x65a7d2fc, 46 },
    { 0x5157d2fd, 53 },
    { 0xbca7d2fe, 98 },
    { 0x3f57d2ff, 90 },
    { 0x2c37d2fe, 95 },
    { 0, 0 },
    { 0x54a7d302, 33 },
    { 0x6ac7d303, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd7d30a, 23 },
    { 0, 0 },
    { 0x4fd7d30c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a7d311, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad27d314, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db7d318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e17d31c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d7d320, 98 },
    { 0x3237d320, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d7d32b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e7d335, 60 },
    { 0x5777d336, 95 },
    { 0xb157d337, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57d33d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a87d343, 33 },
    { 0xbc67d344, 98 },
    { 0, 0 },
    { 0xd27d346, 90 },
    { 0, 0 },
    { 0x6f77d348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd907d34d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6397d352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9097d355, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe137d359, 90 },
    { 0xdb47d35a, 95 },
    { 0x2527d35b, 95 },
    { 0x1b07d35c, 93 },
    { 0xdc27d359, 95 },
    { 0x6dc7d35d, 33 },
    { 0, 0 },
    { 0xe7a7d360, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a7d366, 76 },
    { 0, 0 },
    { 0x4177d368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7357d36b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe657d371, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7157d375, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5937d378, 67 },
    { 0x377d379, 33 },
    { 0xa877d378, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f07d37f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe167d386, 90 },
    { 0x1867d387, 33 },
    { 0x5d37d387, 52 },
    { 0x1497d387, 76 },
    { 0x2257d38a, 90 },
    { 0xf917d386, 93 },
    { 0xc157d387, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5587d391, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa817d39a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c7d39d, 91 },
    { 0x4277d39e, 23 },
    { 0xe347d39d, 76 },
    { 0x4e97d39e, 90 },
    { 0, 0 },
    { 0x60c7d3a2, 90 },
    { 0x3b97d3a3, 81 },
    { 0, 0 },
    { 0x8397d3a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b7d3a9, 93 },
    { 0, 0 },
    { 0x23d7d3ab, 23 },
    { 0x9d37d3ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4287d3b2, 34 },
    { 0x8f87d3b3, 34 },
    { 0x5a97d3b3, 67 },
    { 0, 0 },
    { 0x18b7d3b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b7d3ba, 64 },
    { 0xb77d3bb, 34 },
    { 0x76e7d3bb, 95 },
    { 0, 0 },
    { 0x76e7d3be, 33 },
    { 0, 0 },
    { 0x92f7d3c0, 95 },
    { 0xeab7d3c0, 87 },
    { 0x6457d3c2, 88 },
    { 0x7a07d3c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f7d3c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba67d3ce, 95 },
    { 0, 0 },
    { 0x4a67d3d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c7d3d3, 61 },
    { 0x5c67d3d4, 46 },
    { 0x2597d3d5, 33 },
    { 0x15c7d3d6, 90 },
    { 0xb1b7d3d6, 23 },
    { 0x67a7d3d4, 61 },
    { 0xd197d3d8, 90 },
    { 0xdc57d3da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3dd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef7d3e4, 55 },
    { 0, 0 },
    { 0x3c87d3e6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b47d3ed, 93 },
    { 0, 0 },
    { 0x9137d3ef, 33 },
    { 0x59f7d3f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1697d3f8, 33 },
    { 0x2ce7d3f9, 61 },
    { 0xb6c7d3fa, 90 },
    { 0x27a7d3fb, 33 },
    { 0x697d3f8, 23 },
    { 0x9977d3fd, 55 },
    { 0x2f37d3fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf527d402, 23 },
    { 0x8d27d403, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2007d406, 91 },
    { 0x3897d407, 90 },
    { 0xfed7d408, 90 },
    { 0, 0 },
    { 0xd087d40a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf7d411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe707d417, 95 },
    { 0, 0 },
    { 0x19e7d419, 84 },
    { 0x1c37d41a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5187d41d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7d420, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2997d425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf207d428, 99 },
    { 0xb77d429, 33 },
    { 0, 0 },
    { 0xec17d42b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c07d42f, 46 },
    { 0xd537d430, 98 },
    { 0x67a7d42f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897d436, 95 },
    { 0xe727d437, 55 },
    { 0xff27d436, 23 },
    { 0x4b67d438, 81 },
    { 0x9e17d43a, 33 },
    { 0x2f87d43b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6277d43e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc327d444, 23 },
    { 0, 0 },
    { 0x70d7d446, 33 },
    { 0xd3f7d446, 33 },
    { 0, 0 },
    { 0x7d17d449, 95 },
    { 0x2447d44a, 69 },
    { 0x5997d44b, 9 },
    { 0, 0 },
    { 0xd857d44d, 46 },
    { 0x2607d44e, 98 },
    { 0x46d7d44f, 66 },
    { 0x7c47d450, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f77d458, 61 },
    { 0x3787d459, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc7d45c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f67d461, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa7d465, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe347d46d, 23 },
    { 0x77e7d46e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f77d471, 61 },
    { 0xae7d472, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b17d476, 33 },
    { 0xbc37d477, 33 },
    { 0x8e87d478, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b7d47c, 55 },
    { 0, 0 },
    { 0xe807d47e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d7d487, 90 },
    { 0, 0 },
    { 0x8c57d489, 90 },
    { 0, 0 },
    { 0xc777d48b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4687d48f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d7d492, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa387d497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1247d49a, 84 },
    { 0x3e27d49b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7357d4a6, 66 },
    { 0xc5f7d4a7, 61 },
    { 0x52f7d4a7, 90 },
    { 0x5dd7d4a8, 33 },
    { 0x8eb7d4a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e17d4ad, 33 },
    { 0xd897d4ae, 93 },
    { 0x5357d4ae, 61 },
    { 0x7f97d4b0, 33 },
    { 0xd07d4b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb437d4b5, 91 },
    { 0xc277d4b6, 90 },
    { 0xcd67d4b7, 76 },
    { 0x9167d4b5, 98 },
    { 0xf187d4b6, 98 },
    { 0x1057d4ba, 77 },
    { 0x9967d4ba, 84 },
    { 0xeb77d4bc, 61 },
    { 0x5c37d4bd, 100 },
    { 0xa667d4be, 95 },
    { 0x8fa7d4bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df7d4c2, 23 },
    { 0x1157d4c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd127d4cd, 95 },
    { 0x5f37d4ce, 61 },
    { 0xad67d4cf, 43 },
    { 0xec47d4cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2987d4d3, 23 },
    { 0xdc67d4d4, 23 },
    { 0, 0 },
    { 0xa4c7d4d6, 98 },
    { 0x14f7d4d6, 99 },
    { 0x4377d4d8, 33 },
    { 0x46c7d4d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5787d4dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf627d4e3, 55 },
    { 0, 0 },
    { 0x7547d4e5, 34 },
    { 0xde47d4e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1597d4ec, 77 },
    { 0x4a37d4ed, 91 },
    { 0x5fe7d4ee, 95 },
    { 0, 0 },
    { 0x9697d4f0, 76 },
    { 0xaf07d4f1, 23 },
    { 0xea37d4f0, 95 },
    { 0x7987d4f3, 76 },
    { 0x35d7d4f4, 90 },
    { 0x81e7d4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a07d4fa, 98 },
    { 0, 0 },
    { 0xc2d7d4fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb7d500, 77 },
    { 0xd447d500, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9527d505, 23 },
    { 0, 0 },
    { 0x53f7d507, 78 },
    { 0x2c67d507, 90 },
    { 0x67e7d509, 34 },
    { 0x9aa7d509, 90 },
    { 0xf037d509, 93 },
    { 0x3fa7d507, 91 },
    { 0xd587d50d, 55 },
    { 0xab97d50d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada7d514, 76 },
    { 0x25f7d515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f7d51a, 55 },
    { 0, 0 },
    { 0xed67d51c, 78 },
    { 0xe6e7d51d, 54 },
    { 0, 0 },
    { 0x557d51f, 23 },
    { 0xa1f7d520, 95 },
    { 0xa8d7d521, 98 },
    { 0xd627d521, 95 },
    { 0x3c97d522, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c67d531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc097d534, 40 },
    { 0xa4d7d534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac7d538, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff67d53b, 33 },
    { 0x6ff7d53b, 100 },
    { 0xac57d53d, 43 },
    { 0x22b7d53e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd967d547, 99 },
    { 0x6687d548, 95 },
    { 0, 0 },
    { 0xa0a7d54a, 33 },
    { 0x9ef7d54a, 93 },
    { 0xe317d54c, 85 },
    { 0xfa37d54c, 60 },
    { 0x6167d54e, 55 },
    { 0x3947d54d, 98 },
    { 0x4717d550, 90 },
    { 0xa8a7d54d, 98 },
    { 0xb457d552, 61 },
    { 0x9c7d54e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2477d557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee7d55c, 95 },
    { 0, 0 },
    { 0xa057d55e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a67d561, 93 },
    { 0, 0 },
    { 0x2557d563, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d67d567, 90 },
    { 0, 0 },
    { 0x5e17d569, 46 },
    { 0x75f7d56a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87d56f, 33 },
    { 0, 0 },
    { 0x2b77d571, 100 },
    { 0x6f27d572, 43 },
    { 0x7ed7d573, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77d576, 93 },
    { 0x7787d577, 78 },
    { 0xec17d576, 52 },
    { 0x4ad7d579, 90 },
    { 0, 0 },
    { 0xd027d57b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c7d57e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf537d582, 33 },
    { 0, 0 },
    { 0xdf37d584, 43 },
    { 0, 0 },
    { 0xbdb7d586, 61 },
    { 0, 0 },
    { 0x3677d588, 85 },
    { 0, 0 },
    { 0x97f7d58a, 99 },
    { 0x4e17d58b, 61 },
    { 0x8457d58c, 34 },
    { 0xaa57d58b, 33 },
    { 0xbbd7d58e, 33 },
    { 0x3c47d58f, 55 },
    { 0, 0 },
    { 0x9357d591, 46 },
    { 0x5c47d591, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4257d598, 95 },
    { 0, 0 },
    { 0x2ba7d59a, 55 },
    { 0, 0 },
    { 0xace7d59c, 55 },
    { 0x7e7d59d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a07d5a1, 90 },
    { 0x4067d5a1, 61 },
    { 0, 0 },
    { 0x5ea7d5a4, 85 },
    { 0x1ae7d5a5, 90 },
    { 0x917d5a6, 33 },
    { 0x6697d5a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7897d5ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e7d5b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e7d5bc, 98 },
    { 0x2a47d5bc, 33 },
    { 0xb1a7d5bc, 87 },
    { 0x84d7d5bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc7d5c4, 98 },
    { 0xa107d5c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17d5cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7d5d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1387d5d8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe7d5db, 77 },
    { 0xdd77d5dc, 66 },
    { 0, 0 },
    { 0xe717d5de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d7d5e1, 93 },
    { 0x1237d5e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf7d5ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae7d5ed, 23 },
    { 0xaf87d5ed, 66 },
    { 0xa57d5ed, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a7d5f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b07d5fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2827d603, 55 },
    { 0, 0 },
    { 0xfce7d605, 34 },
    { 0xacc7d606, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc707d60f, 61 },
    { 0, 0 },
    { 0x6927d611, 17 },
    { 0x7757d612, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d7d615, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3827d61c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b7d61f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd617d622, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xded7d625, 93 },
    { 0x2037d626, 85 },
    { 0, 0 },
    { 0x3cf7d628, 98 },
    { 0, 0 },
    { 0x4697d62a, 23 },
    { 0x11a7d62b, 90 },
    { 0, 0 },
    { 0x2177d62d, 23 },
    { 0xc127d62d, 95 },
    { 0x8907d62f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a7d635, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f7d63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4957d63d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b7d640, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c87d649, 45 },
    { 0xb7e7d649, 90 },
    { 0x4de7d649, 84 },
    { 0xdb97d64c, 23 },
    { 0x5387d64d, 33 },
    { 0xece7d64e, 99 },
    { 0x71d7d64d, 23 },
    { 0x8447d64b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7d658, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4117d660, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4157d664, 34 },
    { 0xfa7d665, 53 },
    { 0xd467d666, 23 },
    { 0xbce7d667, 75 },
    { 0x2bc7d666, 76 },
    { 0x3cd7d668, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb457d66d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5707d674, 33 },
    { 0, 0 },
    { 0xb887d676, 43 },
    { 0, 0 },
    { 0x5c77d678, 43 },
    { 0x7197d678, 55 },
    { 0, 0 },
    { 0x46e7d67b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5637d684, 23 },
    { 0xe2a7d684, 95 },
    { 0xb27d685, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed7d68a, 90 },
    { 0, 0 },
    { 0xedc7d68c, 55 },
    { 0xc4a7d68d, 56 },
    { 0x7037d68e, 87 },
    { 0x22a7d68f, 61 },
    { 0xb927d690, 95 },
    { 0xa157d691, 90 },
    { 0, 0 },
    { 0x6d7d693, 90 },
    { 0xae37d694, 90 },
    { 0xf957d694, 23 },
    { 0x8ae7d695, 98 },
    { 0xab17d697, 55 },
    { 0, 0 },
    { 0xb707d699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf7d69d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef7d6a0, 76 },
    { 0, 0 },
    { 0x7647d6a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb17d6a7, 96 },
    { 0x4dd7d6a8, 34 },
    { 0xc4e7d6a9, 17 },
    { 0x157d6a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2757d6af, 23 },
    { 0x70e7d6af, 95 },
    { 0x8cc7d6b1, 33 },
    { 0x46e7d6b2, 46 },
    { 0x1b77d6b0, 23 },
    { 0xa5c7d6b4, 98 },
    { 0xa337d6b2, 40 },
    { 0, 0 },
    { 0xd2e7d6b7, 85 },
    { 0, 0 },
    { 0x82a7d6b9, 91 },
    { 0xc7d6ba, 84 },
    { 0x6d7d6bb, 34 },
    { 0x7657d6bc, 23 },
    { 0x6807d6bc, 55 },
    { 0, 0 },
    { 0xd317d6bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf787d6c4, 72 },
    { 0, 0 },
    { 0x24a7d6c6, 98 },
    { 0, 0 },
    { 0x4a27d6c8, 38 },
    { 0xc97d6c9, 98 },
    { 0, 0 },
    { 0x9617d6cb, 66 },
    { 0x12b7d6cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f77d6d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe047d6d5, 99 },
    { 0xe847d6d6, 61 },
    { 0xb707d6d7, 33 },
    { 0, 0 },
    { 0xf917d6d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9847d6dc, 33 },
    { 0x8397d6dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b7d6e6, 23 },
    { 0, 0 },
    { 0x2f07d6e8, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6617d6ec, 90 },
    { 0, 0 },
    { 0xe947d6ee, 36 },
    { 0, 0 },
    { 0x6b47d6f0, 93 },
    { 0xa837d6f1, 55 },
    { 0xe7d7d6f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde7d6f6, 90 },
    { 0, 0 },
    { 0x6627d6f8, 83 },
    { 0x33f7d6f8, 34 },
    { 0, 0 },
    { 0x7d6fb, 61 },
    { 0x63b7d6fc, 34 },
    { 0x11a7d6fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc7d704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b7d707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd7d70b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7667d713, 34 },
    { 0, 0 },
    { 0xe407d715, 68 },
    { 0x44d7d715, 90 },
    { 0x2507d717, 43 },
    { 0xcd17d718, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db7d720, 98 },
    { 0x17b7d721, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9257d724, 23 },
    { 0xd627d725, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e7d72c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e7d732, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe537d735, 17 },
    { 0x6c37d735, 38 },
    { 0, 0 },
    { 0xc887d738, 33 },
    { 0, 0 },
    { 0xfb27d73a, 76 },
    { 0, 0 },
    { 0xb97d73c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c7d744, 23 },
    { 0x58c7d745, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc257d74d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa027d750, 77 },
    { 0, 0 },
    { 0x6ab7d752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb737d75c, 19 },
    { 0, 0 },
    { 0xd6a7d75e, 91 },
    { 0xba47d75f, 23 },
    { 0xf9f7d760, 85 },
    { 0x8eb7d761, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc7d764, 61 },
    { 0, 0 },
    { 0x8e7d766, 98 },
    { 0, 0 },
    { 0x1bf7d768, 34 },
    { 0xfdc7d768, 91 },
    { 0, 0 },
    { 0x4f97d76b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3377d76f, 23 },
    { 0x3ce7d770, 90 },
    { 0xc47d770, 77 },
    { 0x7dd7d770, 76 },
    { 0xaa17d773, 33 },
    { 0xdb07d774, 55 },
    { 0x10a7d775, 85 },
    { 0xa2c7d771, 33 },
    { 0xbd67d777, 76 },
    { 0x1087d774, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9217d780, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc07d783, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2827d787, 90 },
    { 0xecf7d787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9557d78b, 67 },
    { 0x11a7d78b, 55 },
    { 0, 0 },
    { 0xd9b7d78e, 33 },
    { 0x5007d78f, 33 },
    { 0x47c7d78e, 96 },
    { 0x6f7d791, 76 },
    { 0x2ed7d791, 23 },
    { 0x1647d792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd947d797, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6707d79c, 99 },
    { 0, 0 },
    { 0x8117d79e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637d7a5, 77 },
    { 0x26e7d7a6, 23 },
    { 0x1447d7a7, 33 },
    { 0xc5a7d7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1317d7ad, 43 },
    { 0xe057d7ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f77d7b6, 43 },
    { 0xc6e7d7b7, 78 },
    { 0x5597d7b6, 43 },
    { 0xdc37d7b7, 40 },
    { 0x82f7d7ba, 88 },
    { 0x7087d7ba, 46 },
    { 0x96f7d7bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7d7c1, 98 },
    { 0xb5c7d7c1, 61 },
    { 0x94e7d7c3, 34 },
    { 0x76b7d7c3, 98 },
    { 0, 0 },
    { 0x96f7d7c6, 90 },
    { 0, 0 },
    { 0xa027d7c8, 98 },
    { 0xa4e7d7c8, 67 },
    { 0xbac7d7c9, 61 },
    { 0x76c7d7cb, 61 },
    { 0xf9e7d7c8, 33 },
    { 0x40c7d7cc, 78 },
    { 0, 0 },
    { 0x6707d7cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d7d7d2, 43 },
    { 0, 0 },
    { 0xb3a7d7d4, 34 },
    { 0, 0 },
    { 0x1d87d7d6, 23 },
    { 0, 0 },
    { 0xc777d7d8, 33 },
    { 0x2407d7d8, 98 },
    { 0x2af7d7d9, 98 },
    { 0, 0 },
    { 0xbed7d7dc, 33 },
    { 0xde97d7dd, 61 },
    { 0, 0 },
    { 0x8ab7d7df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c47d7e4, 34 },
    { 0x527d7e5, 55 },
    { 0xe6b7d7e6, 52 },
    { 0xb5c7d7e7, 88 },
    { 0xca7d7e7, 43 },
    { 0x4b07d7e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6247d7ef, 95 },
    { 0x2397d7ef, 95 },
    { 0xb317d7f1, 67 },
    { 0x4cf7d7f2, 61 },
    { 0xf2a7d7f3, 85 },
    { 0, 0 },
    { 0xfe67d7f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2737d7fd, 98 },
    { 0, 0 },
    { 0xaec7d7ff, 23 },
    { 0xcc97d7ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1537d80b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff7d811, 90 },
    { 0x407d812, 61 },
    { 0xbf37d813, 98 },
    { 0, 0 },
    { 0xc317d815, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b17d81c, 55 },
    { 0x4fb7d81d, 61 },
    { 0xe9b7d81d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d822, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa7d829, 55 },
    { 0xff87d829, 90 },
    { 0xf217d829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b7d831, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3127d836, 61 },
    { 0, 0 },
    { 0x7b87d838, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba7d83b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a7d83f, 95 },
    { 0xae07d840, 55 },
    { 0x7fe7d83f, 33 },
    { 0, 0 },
    { 0x8d57d843, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf57d847, 98 },
    { 0x6c17d847, 61 },
    { 0x6637d847, 95 },
    { 0x1997d847, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1287d851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c7d857, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x1447d85a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c87d85d, 90 },
    { 0x8817d85d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7d862, 90 },
    { 0x5b77d862, 91 },
    { 0x1077d863, 55 },
    { 0xae67d863, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5527d86b, 98 },
    { 0xde97d86c, 90 },
    { 0, 0 },
    { 0x5057d86e, 77 },
    { 0xbaa7d86f, 61 },
    { 0xd27d870, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xce17d873, 100 },
    { 0xf797d873, 61 },
    { 0xd777d875, 85 },
    { 0xbc87d873, 55 },
    { 0xd4a7d877, 55 },
    { 0xaa97d873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57d87e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be7d885, 68 },
    { 0, 0 },
    { 0x5da7d887, 65 },
    { 0x47d888, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6857d88b, 84 },
    { 0xfa97d88c, 76 },
    { 0xce77d88d, 55 },
    { 0xe447d88e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc7d891, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c7d896, 90 },
    { 0xd717d897, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f7d89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367d89f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda07d8a2, 85 },
    { 0, 0 },
    { 0xdd77d8a4, 34 },
    { 0x5b97d8a5, 61 },
    { 0x20f7d8a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4417d8a9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e7d8ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657d8af, 88 },
    { 0xf947d8af, 33 },
    { 0xc577d8b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc067d8b5, 33 },
    { 0, 0 },
    { 0xd1d7d8b7, 90 },
    { 0x3797d8b8, 95 },
    { 0, 0 },
    { 0xcbc7d8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a7d8c4, 43 },
    { 0, 0 },
    { 0xd1f7d8c6, 101 },
    { 0xdcf7d8c7, 55 },
    { 0x5137d8c8, 90 },
    { 0x9bb7d8c8, 78 },
    { 0x9207d8ca, 90 },
    { 0xd3e7d8ca, 90 },
    { 0x8a27d8c9, 90 },
    { 0x8e97d8cd, 34 },
    { 0x5f47d8ce, 90 },
    { 0xd0c7d8cf, 91 },
    { 0, 0 },
    { 0xdc57d8d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4217d8d5, 77 },
    { 0x237d8d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a7d8d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f7d8dc, 66 },
    { 0x7137d8dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b7d8e1, 75 },
    { 0x7c67d8e2, 46 },
    { 0xbf07d8e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada7d8e9, 80 },
    { 0x5447d8e9, 75 },
    { 0x4617d8ea, 100 },
    { 0xd77d8ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c7d8ef, 33 },
    { 0x71d7d8f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07d8f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b67d8f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd727d8fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6557d8fe, 76 },
    { 0xdbb7d8ff, 55 },
    { 0xdbe7d900, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c7d909, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3737d90e, 90 },
    { 0xa557d90f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a67d914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa707d919, 93 },
    { 0xd4a7d919, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a7d91f, 99 },
    { 0, 0 },
    { 0xc077d921, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c87d929, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba7d92d, 34 },
    { 0x2337d92e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a77d931, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7d93b, 23 },
    { 0xca17d93b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577d93f, 23 },
    { 0xefd7d93f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3887d947, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec87d94e, 84 },
    { 0x2007d94f, 76 },
    { 0xc2d7d950, 55 },
    { 0, 0 },
    { 0xe317d952, 55 },
    { 0x4d37d953, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f77d958, 55 },
    { 0xe027d959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc7d95e, 85 },
    { 0xf757d95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f7d969, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf67d96c, 98 },
    { 0x207d96d, 90 },
    { 0x5127d96e, 55 },
    { 0x7297d96c, 43 },
    { 0xd477d970, 68 },
    { 0xee47d970, 95 },
    { 0, 0 },
    { 0xcd87d973, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a77d978, 93 },
    { 0x1c67d978, 66 },
    { 0, 0 },
    { 0xf8f7d97b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a7d97e, 66 },
    { 0x92c7d97e, 90 },
    { 0x9517d980, 95 },
    { 0x2417d981, 23 },
    { 0x1dc7d982, 75 },
    { 0x76c7d981, 87 },
    { 0x1497d984, 23 },
    { 0x8097d984, 33 },
    { 0, 0 },
    { 0x7f87d987, 90 },
    { 0x95b7d987, 98 },
    { 0x67d7d988, 61 },
    { 0xe287d989, 43 },
    { 0xe437d98b, 91 },
    { 0xf27d98c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4407d991, 88 },
    { 0xd837d991, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac17d999, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e7d99c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc47d9a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb77d9a8, 61 },
    { 0x4057d9a8, 23 },
    { 0x3a77d9a9, 95 },
    { 0, 0 },
    { 0x7667d9ac, 95 },
    { 0xe567d9ac, 23 },
    { 0x1667d9ae, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd7d9b2, 19 },
    { 0, 0 },
    { 0x5197d9b4, 61 },
    { 0xeac7d9b4, 95 },
    { 0x2077d9b6, 99 },
    { 0, 0 },
    { 0x3db7d9b8, 34 },
    { 0, 0 },
    { 0xd6c7d9ba, 23 },
    { 0, 0 },
    { 0xb837d9bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe47d9c0, 95 },
    { 0x7697d9c1, 93 },
    { 0, 0 },
    { 0xffe7d9c3, 33 },
    { 0xe047d9c3, 98 },
    { 0x6b67d9c5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec7d9c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf77d9d0, 98 },
    { 0xa957d9d0, 33 },
    { 0x5f27d9d1, 46 },
    { 0x73c7d9d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb7d9d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3397d9da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d87d9dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae7d9e2, 93 },
    { 0x45a7d9e3, 23 },
    { 0x88f7d9e2, 98 },
    { 0xba27d9e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b7d9e8, 50 },
    { 0x2247d9e8, 61 },
    { 0x6f57d9ea, 90 },
    { 0xd2d7d9e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e7d9ee, 95 },
    { 0xc9b7d9ef, 43 },
    { 0x8257d9f0, 84 },
    { 0, 0 },
    { 0x69d7d9f2, 98 },
    { 0, 0 },
    { 0xc697d9f4, 90 },
    { 0xc517d9f4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c7d9fd, 95 },
    { 0, 0 },
    { 0x41f7d9ff, 34 },
    { 0, 0 },
    { 0x93d7da01, 76 },
    { 0xadd7da02, 61 },
    { 0x8b87da03, 61 },
    { 0x37da02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2597da08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd7da10, 95 },
    { 0, 0 },
    { 0x77da12, 33 },
    { 0x6fe7da13, 99 },
    { 0xffd7da14, 61 },
    { 0x2897da14, 33 },
    { 0x9677da16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77da1d, 90 },
    { 0x55d7da1e, 61 },
    { 0x2c37da1f, 90 },
    { 0xa387da1f, 99 },
    { 0xb5c7da21, 68 },
    { 0xa417da21, 76 },
    { 0xabc7da22, 52 },
    { 0xcbe7da21, 40 },
    { 0x1f77da25, 60 },
    { 0xe037da26, 90 },
    { 0x3ce7da27, 88 },
    { 0x6507da26, 33 },
    { 0xfd97da26, 90 },
    { 0xfda7da2a, 55 },
    { 0xcaa7da24, 33 },
    { 0x1df7da2c, 52 },
    { 0xf0a7da2d, 43 },
    { 0xaf37da1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa7da34, 43 },
    { 0x3387da35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6437da39, 76 },
    { 0x3bd7da39, 90 },
    { 0x1b87da3b, 23 },
    { 0x1c67da3a, 98 },
    { 0x49d7da3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb657da40, 91 },
    { 0, 0 },
    { 0x5167da42, 23 },
    { 0xff37da42, 23 },
    { 0x17da42, 98 },
    { 0, 0 },
    { 0xa657da46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d57da49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4567da4f, 91 },
    { 0x8477da4f, 55 },
    { 0x8f67da51, 91 },
    { 0x567da51, 95 },
    { 0xf87da53, 90 },
    { 0xd167da54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3647da58, 33 },
    { 0xdf67da59, 61 },
    { 0x7ae7da5a, 73 },
    { 0x3c07da59, 61 },
    { 0x6907da5c, 96 },
    { 0, 0 },
    { 0x7337da5e, 46 },
    { 0, 0 },
    { 0x8c77da60, 98 },
    { 0xc97da60, 93 },
    { 0x9667da62, 76 },
    { 0xaa27da63, 61 },
    { 0x3787da64, 55 },
    { 0x8e7da64, 98 },
    { 0, 0 },
    { 0xb147da67, 99 },
    { 0, 0 },
    { 0xa157da69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef7da6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f7da72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2447da7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a47da85, 34 },
    { 0, 0 },
    { 0x5f7da87, 97 },
    { 0x43a7da87, 61 },
    { 0x48e7da89, 66 },
    { 0x897da89, 81 },
    { 0x2c77da8b, 95 },
    { 0x9b87da8a, 19 },
    { 0x9b77da8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4737da90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba7da94, 23 },
    { 0, 0 },
    { 0xda77da96, 55 },
    { 0xfed7da97, 61 },
    { 0, 0 },
    { 0x61d7da99, 55 },
    { 0x16c7da9a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d7da9d, 78 },
    { 0xfe47da9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137daa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127daa7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2087daad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa7dab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7227dab6, 66 },
    { 0, 0 },
    { 0xc327dab8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd237dabc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb997dabf, 90 },
    { 0, 0 },
    { 0xf507dac1, 55 },
    { 0xab47dac2, 90 },
    { 0, 0 },
    { 0xddd7dac4, 23 },
    { 0xccd7dac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf277dad0, 46 },
    { 0x4647dad1, 99 },
    { 0x8a47dad2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d7dad7, 76 },
    { 0, 0 },
    { 0xddc7dad9, 98 },
    { 0xa8e7dada, 46 },
    { 0xcbe7dad9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd297dade, 61 },
    { 0xa297dadf, 98 },
    { 0, 0 },
    { 0x8707dae1, 76 },
    { 0, 0 },
    { 0xc0d7dae3, 61 },
    { 0xf257dae4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb97dae8, 76 },
    { 0x1ca7dae9, 61 },
    { 0xd3c7daea, 43 },
    { 0, 0 },
    { 0x30a7daec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47daef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6147daf4, 52 },
    { 0xf87daf4, 23 },
    { 0x42f7daf6, 55 },
    { 0, 0 },
    { 0xc337daf8, 33 },
    { 0x9ef7daf8, 95 },
    { 0x46e7dafa, 66 },
    { 0xe207daf8, 33 },
    { 0xe177daf8, 90 },
    { 0x4dd7dafd, 77 },
    { 0xbd57daf9, 23 },
    { 0xc487daf9, 90 },
    { 0xe157daff, 98 },
    { 0x777daf9, 38 },
    { 0xfb27db02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d47db05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c87db0e, 17 },
    { 0x8ac7db0f, 98 },
    { 0xaa27db10, 33 },
    { 0x6167db11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e07db1a, 46 },
    { 0, 0 },
    { 0xad77db1c, 81 },
    { 0x5af7db1d, 34 },
    { 0x10b7db1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c7db21, 33 },
    { 0xf047db22, 77 },
    { 0, 0 },
    { 0x3107db24, 61 },
    { 0xa757db24, 78 },
    { 0x39e7db24, 46 },
    { 0, 0 },
    { 0x1b37db28, 99 },
    { 0, 0 },
    { 0x9c17db2a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8517db2f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd837db39, 23 },
    { 0xcac7db3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f7db41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee7db47, 43 },
    { 0xaa67db47, 40 },
    { 0, 0 },
    { 0x9b37db4a, 98 },
    { 0x8a27db4b, 95 },
    { 0x55f7db4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507db53, 90 },
    { 0xf4c7db53, 61 },
    { 0xb007db55, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd187db5b, 95 },
    { 0x8fb7db5c, 78 },
    { 0, 0 },
    { 0xa9e7db5e, 96 },
    { 0x1467db5f, 95 },
    { 0, 0 },
    { 0x367db61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a47db65, 61 },
    { 0x1577db66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd747db69, 90 },
    { 0x7947db6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2627db6d, 95 },
    { 0, 0 },
    { 0x5397db6f, 66 },
    { 0xd5b7db70, 44 },
    { 0, 0 },
    { 0xf227db72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7db77, 43 },
    { 0xa1e7db78, 90 },
    { 0, 0 },
    { 0x1567db7a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9157db89, 81 },
    { 0x3397db89, 38 },
    { 0x4eb7db8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7887db94, 90 },
    { 0x5237db94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b7db9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf597dba6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x707dba9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a37dbaf, 95 },
    { 0, 0 },
    { 0x92d7dbb1, 98 },
    { 0x5e57dbb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7dbb5, 95 },
    { 0x2cb7dbb6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4797dbb9, 61 },
    { 0, 0 },
    { 0xa777dbbb, 34 },
    { 0, 0 },
    { 0x6f57dbbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff67dbc4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf87dbc8, 95 },
    { 0, 0 },
    { 0x737dbca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d77dbd0, 61 },
    { 0x4b37dbd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d7dbd5, 75 },
    { 0x49b7dbd5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c7dbe3, 66 },
    { 0x61d7dbe4, 98 },
    { 0xa957dbe5, 23 },
    { 0xfb67dbe5, 81 },
    { 0x8537dbe6, 52 },
    { 0x307dbe4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d7dbec, 43 },
    { 0x9497dbed, 77 },
    { 0x8b27dbec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a7dbf1, 76 },
    { 0x2767dbf2, 98 },
    { 0, 0 },
    { 0xfe7dbf4, 90 },
    { 0x28c7dbf5, 95 },
    { 0x1ec7dbf6, 69 },
    { 0x7017dbf6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5967dbfc, 55 },
    { 0x9ae7dbfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc7dc03, 43 },
    { 0, 0 },
    { 0x2aa7dc05, 93 },
    { 0, 0 },
    { 0x53d7dc07, 77 },
    { 0x637dc08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6867dc0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf597dc10, 34 },
    { 0xdc87dc11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca7dc17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a7dc1c, 61 },
    { 0x27f7dc1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a7dc23, 90 },
    { 0xa247dc23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde37dc29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf547dc2d, 43 },
    { 0xf427dc2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b37dc36, 23 },
    { 0, 0 },
    { 0xaae7dc38, 33 },
    { 0xfe17dc39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9707dc3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9127dc40, 90 },
    { 0, 0 },
    { 0x4e87dc42, 61 },
    { 0x4a07dc42, 76 },
    { 0x9be7dc44, 33 },
    { 0, 0 },
    { 0x48d7dc46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd37dc4a, 55 },
    { 0x1a07dc4b, 69 },
    { 0xbcc7dc4c, 61 },
    { 0x7c57dc4d, 95 },
    { 0xa077dc4d, 23 },
    { 0x8547dc4f, 88 },
    { 0xa6f7dc4c, 17 },
    { 0x91d7dc51, 98 },
    { 0x2017dc4d, 33 },
    { 0x7c7dc53, 88 },
    { 0xeb07dc4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec87dc57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd27dc5f, 33 },
    { 0, 0 },
    { 0xc687dc61, 23 },
    { 0, 0 },
    { 0x5ea7dc63, 66 },
    { 0x1357dc64, 33 },
    { 0xe077dc65, 98 },
    { 0, 0 },
    { 0x1637dc67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167dc6f, 98 },
    { 0xb9c7dc6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf7dc73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x667dc76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d7dc7a, 98 },
    { 0xadd7dc7a, 76 },
    { 0x3397dc7c, 91 },
    { 0x1b7dc7c, 61 },
    { 0x6c87dc7d, 61 },
    { 0x1f47dc7f, 61 },
    { 0x69b7dc7a, 96 },
    { 0x82f7dc7c, 100 },
    { 0x9367dc82, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a7dc87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a7dc8a, 95 },
    { 0x7277dc8b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b7dc91, 61 },
    { 0x1be7dc91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe847dc96, 90 },
    { 0, 0 },
    { 0x6b87dc98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc397dc9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd597dc9e, 43 },
    { 0, 0 },
    { 0xea67dca0, 23 },
    { 0xe327dca1, 90 },
    { 0xe6f7dca2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x7427dca5, 78 },
    { 0x4a77dca6, 33 },
    { 0xb7c7dca5, 90 },
    { 0, 0 },
    { 0x1ff7dca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa247dcac, 85 },
    { 0x42b7dcad, 77 },
    { 0, 0 },
    { 0x3597dcaf, 98 },
    { 0x26e7dcb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be7dcb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d77dcb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f47dcbc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa87dcc1, 99 },
    { 0xb397dcc2, 34 },
    { 0xbac7dcc1, 84 },
    { 0x3427dcc4, 95 },
    { 0x9967dcc5, 98 },
    { 0xe937dcc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9837dcca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7dcd0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67dcd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc437dcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de7dce1, 23 },
    { 0xf2c7dce2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2227dce8, 98 },
    { 0x3f37dce9, 93 },
    { 0, 0 },
    { 0xda77dceb, 84 },
    { 0x7397dcec, 23 },
    { 0x5c07dced, 61 },
    { 0xacf7dcec, 34 },
    { 0, 0 },
    { 0xd087dcf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b7dcf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf617dcf8, 90 },
    { 0xd597dcf8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c7dcfd, 43 },
    { 0xa5f7dcfe, 17 },
    { 0xd257dcff, 23 },
    { 0x88b7dd00, 76 },
    { 0xddb7dcfe, 33 },
    { 0x9cf7dcfe, 61 },
    { 0xc927dd03, 43 },
    { 0xf387dcff, 98 },
    { 0x5d7dcfd, 100 },
    { 0x5ec7dcff, 90 },
    { 0, 0 },
    { 0xddf7dd08, 83 },
    { 0, 0 },
    { 0x9087dd0a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb7dd0f, 36 },
    { 0, 0 },
    { 0x3747dd11, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3507dd14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb7dd19, 33 },
    { 0xb87dd19, 43 },
    { 0xd3c7dd1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3897dd1f, 98 },
    { 0xa127dd20, 84 },
    { 0x25e7dd21, 100 },
    { 0xede7dd22, 38 },
    { 0, 0 },
    { 0x1587dd24, 23 },
    { 0x3147dd24, 61 },
    { 0, 0 },
    { 0x72e7dd27, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3627dd2a, 87 },
    { 0x1e57dd2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3157dd2e, 87 },
    { 0x7ff7dd2e, 33 },
    { 0xf087dd2f, 61 },
    { 0xa7d7dd2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e77dd34, 95 },
    { 0x6747dd34, 33 },
    { 0x3b7dd36, 17 },
    { 0x9527dd36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567dd3a, 52 },
    { 0x6fe7dd3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c17dd3e, 55 },
    { 0x2d57dd3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4257dd42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b7dd47, 76 },
    { 0xea27dd48, 33 },
    { 0xc6d7dd48, 88 },
    { 0x42e7dd49, 99 },
    { 0x85c7dd4a, 95 },
    { 0x1567dd4c, 34 },
    { 0xc9c7dd4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7dd50, 61 },
    { 0, 0 },
    { 0x357dd52, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf227dd5a, 90 },
    { 0x6e7dd5b, 66 },
    { 0, 0 },
    { 0x11c7dd5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f7dd64, 46 },
    { 0xabc7dd65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557dd6b, 67 },
    { 0, 0 },
    { 0x7d87dd6d, 99 },
    { 0, 0 },
    { 0x2917dd6f, 33 },
    { 0xb017dd70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7dd73, 55 },
    { 0x4e67dd74, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d67dd78, 77 },
    { 0xb6a7dd79, 75 },
    { 0x8307dd7a, 61 },
    { 0x25c7dd7b, 34 },
    { 0x8a97dd79, 84 },
    { 0, 0 },
    { 0x3fa7dd7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf577dd8a, 61 },
    { 0x8f67dd8b, 98 },
    { 0x6557dd8a, 33 },
    { 0xd327dd8d, 69 },
    { 0, 0 },
    { 0x4c17dd8f, 77 },
    { 0, 0 },
    { 0x7487dd91, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c27dd98, 17 },
    { 0xec7dd98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6587dda1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8967dda6, 61 },
    { 0x21b7dda6, 76 },
    { 0, 0 },
    { 0xa687dda9, 84 },
    { 0, 0 },
    { 0xc0e7ddab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf387ddb0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe087ddb5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed27ddb9, 90 },
    { 0x5da7ddba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde17ddbf, 95 },
    { 0, 0 },
    { 0x137ddc1, 90 },
    { 0x1f47ddc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee7ddc8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1287ddcd, 84 },
    { 0xb147ddce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb17ddd1, 95 },
    { 0, 0 },
    { 0xd47ddd3, 61 },
    { 0x35d7ddd3, 66 },
    { 0x4c07ddd5, 77 },
    { 0, 0 },
    { 0xf457ddd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb947ddda, 34 },
    { 0, 0 },
    { 0x7c67dddc, 78 },
    { 0xba37dddc, 76 },
    { 0, 0 },
    { 0xd167dddf, 93 },
    { 0x6287dde0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67dde6, 87 },
    { 0x5ea7dde7, 61 },
    { 0x6b17dde7, 23 },
    { 0, 0 },
    { 0x6cc7ddea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e37ddef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf67ddf5, 33 },
    { 0xd157ddf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6797ddfa, 55 },
    { 0xc467ddfa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797de02, 95 },
    { 0x51d7de03, 78 },
    { 0x14d7de04, 33 },
    { 0x2927de05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde7de0c, 76 },
    { 0xe587de0d, 90 },
    { 0x2ce7de0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc597de12, 67 },
    { 0xb5f7de12, 4 },
    { 0xb5a7de14, 90 },
    { 0xfff7de15, 90 },
    { 0xc237de15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c7de1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b17de1e, 23 },
    { 0x4cb7de1f, 23 },
    { 0x43f7de20, 98 },
    { 0x86c7de21, 65 },
    { 0xbca7de21, 90 },
    { 0xfb7de23, 95 },
    { 0xd567de23, 33 },
    { 0, 0 },
    { 0xad47de26, 33 },
    { 0, 0 },
    { 0x8007de28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2397de2b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa77de2e, 98 },
    { 0x9e67de2f, 90 },
    { 0, 0 },
    { 0x96f7de31, 90 },
    { 0x9c77de31, 66 },
    { 0x7827de31, 98 },
    { 0xba87de32, 98 },
    { 0x5b07de33, 55 },
    { 0x6647de34, 96 },
    { 0xf567de37, 80 },
    { 0x9f07de37, 91 },
    { 0xfb77de37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5297de3f, 23 },
    { 0, 0 },
    { 0xc6c7de41, 61 },
    { 0, 0 },
    { 0xa337de43, 61 },
    { 0, 0 },
    { 0x177de45, 23 },
    { 0xfcb7de46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c7de4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b7de51, 93 },
    { 0x72e7de51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5727de57, 90 },
    { 0xfdc7de57, 76 },
    { 0xdba7de57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d7de5c, 33 },
    { 0x9547de5c, 76 },
    { 0, 0 },
    { 0x3347de5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf437de64, 77 },
    { 0x8c7de65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de7de68, 90 },
    { 0x3147de69, 61 },
    { 0x5877de6a, 61 },
    { 0xf3f7de69, 98 },
    { 0xce47de6b, 95 },
    { 0, 0 },
    { 0xe3d7de6e, 76 },
    { 0, 0 },
    { 0xaf97de70, 85 },
    { 0x8a97de71, 34 },
    { 0x6757de70, 66 },
    { 0x6ed7de72, 66 },
    { 0x4027de74, 43 },
    { 0, 0 },
    { 0xadb7de76, 33 },
    { 0x5507de76, 91 },
    { 0x3cd7de77, 67 },
    { 0x9897de79, 17 },
    { 0, 0 },
    { 0xfe97de7b, 69 },
    { 0xd327de7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9647de7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a7de82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4557de87, 46 },
    { 0, 0 },
    { 0xe6b7de89, 33 },
    { 0xb6f7de8a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f7de8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b37de91, 61 },
    { 0, 0 },
    { 0x3537de93, 90 },
    { 0x83d7de94, 93 },
    { 0x6227de95, 99 },
    { 0x9097de94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f7de9c, 85 },
    { 0xbc57de9d, 46 },
    { 0xe517de9e, 76 },
    { 0, 0 },
    { 0x8c57dea0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c67dea5, 84 },
    { 0x33c7dea6, 80 },
    { 0x2c27dea6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f7deaa, 92 },
    { 0xe597deab, 23 },
    { 0x4c97deab, 78 },
    { 0, 0 },
    { 0x837deae, 84 },
    { 0xa947deaf, 55 },
    { 0x12f7deaf, 33 },
    { 0, 0 },
    { 0x3a37deb2, 93 },
    { 0x2557deb2, 76 },
    { 0x1497deb4, 55 },
    { 0, 0 },
    { 0x3737deb6, 93 },
    { 0xafb7deb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec97debc, 90 },
    { 0, 0 },
    { 0x9bf7debe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8677dec2, 95 },
    { 0xc867dec2, 90 },
    { 0, 0 },
    { 0x1e27dec5, 90 },
    { 0xe3e7dec6, 100 },
    { 0x2637dec7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b7decb, 95 },
    { 0, 0 },
    { 0x9127decd, 65 },
    { 0x2727dece, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2107ded4, 61 },
    { 0x2d07ded5, 23 },
    { 0, 0 },
    { 0xdda7ded7, 46 },
    { 0, 0 },
    { 0xe387ded9, 88 },
    { 0x8877deda, 32 },
    { 0x4d77ded9, 34 },
    { 0xbc47dedb, 78 },
    { 0, 0 },
    { 0x187dede, 50 },
    { 0x7257dedf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c7dee5, 33 },
    { 0, 0 },
    { 0x6c97dee7, 89 },
    { 0xb77dee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc7deee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd677def2, 46 },
    { 0x1837def3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc7defa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f67df00, 33 },
    { 0xaaf7df01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe087df04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4147df12, 34 },
    { 0xea07df12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a87df17, 76 },
    { 0, 0 },
    { 0x99d7df19, 23 },
    { 0x6947df1a, 76 },
    { 0, 0 },
    { 0xa637df1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6657df1f, 81 },
    { 0x7337df20, 33 },
    { 0x54e7df21, 90 },
    { 0x88b7df20, 76 },
    { 0, 0 },
    { 0xd07df24, 53 },
    { 0, 0 },
    { 0xea47df26, 90 },
    { 0x26b7df27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547df2e, 23 },
    { 0, 0 },
    { 0x5657df30, 61 },
    { 0x7927df31, 61 },
    { 0, 0 },
    { 0x4797df33, 93 },
    { 0x6f47df34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc317df37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b27df3c, 34 },
    { 0, 0 },
    { 0xfb37df3e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb57df41, 93 },
    { 0x5ab7df42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf7df45, 68 },
    { 0xfde7df46, 88 },
    { 0xe897df46, 48 },
    { 0xc207df48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb947df4c, 90 },
    { 0xba17df4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5327df50, 52 },
    { 0xdd97df50, 98 },
    { 0xded7df52, 99 },
    { 0xc207df53, 33 },
    { 0, 0 },
    { 0x5fc7df55, 83 },
    { 0, 0 },
    { 0x1ed7df57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3027df5a, 64 },
    { 0, 0 },
    { 0x7757df5c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f7df5f, 90 },
    { 0x8037df60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d7df64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a7df6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda7df6e, 43 },
    { 0x3497df6f, 90 },
    { 0x1cb7df70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xab57df73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9827df76, 93 },
    { 0x5f07df77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda57df7c, 95 },
    { 0x6a97df7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1787df80, 87 },
    { 0xeef7df81, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba7df86, 78 },
    { 0xb917df86, 90 },
    { 0, 0 },
    { 0xc2b7df89, 76 },
    { 0x1347df8a, 99 },
    { 0x9337df8b, 98 },
    { 0xd4f7df8b, 23 },
    { 0, 0 },
    { 0xef7df8e, 55 },
    { 0x8de7df8e, 95 },
    { 0x54e7df90, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd767df94, 98 },
    { 0xc3e7df95, 43 },
    { 0x28c7df96, 99 },
    { 0x6497df97, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77df9b, 90 },
    { 0x94a7df9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf607df9f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe737dfa5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d7dfa9, 34 },
    { 0, 0 },
    { 0xe1c7dfab, 95 },
    { 0x5b37dfab, 76 },
    { 0x7007dfac, 94 },
    { 0, 0 },
    { 0xd187dfaf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1897dfb3, 81 },
    { 0x7a17dfb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac7dfb7, 17 },
    { 0, 0 },
    { 0x55e7dfb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3987dfbe, 93 },
    { 0xc227dfbf, 33 },
    { 0, 0 },
    { 0x2267dfc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd437dfc7, 98 },
    { 0x3f47dfc8, 61 },
    { 0, 0 },
    { 0xcfa7dfca, 55 },
    { 0xeaa7dfcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b7dfd1, 95 },
    { 0xae77dfd2, 61 },
    { 0xb067dfd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6947dfd8, 95 },
    { 0, 0 },
    { 0xf3c7dfda, 23 },
    { 0x4107dfdb, 46 },
    { 0x6007dfdc, 65 },
    { 0x9977dfdc, 98 },
    { 0xeee7dfde, 83 },
    { 0x3fd7dfdc, 61 },
    { 0, 0 },
    { 0xd747dfe1, 91 },
    { 0xbf37dfe2, 33 },
    { 0x8ce7dfe1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb07dfe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197dfea, 33 },
    { 0xa3c7dfeb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc7dfef, 33 },
    { 0x9947dff0, 95 },
    { 0xee77dfef, 23 },
    { 0, 0 },
    { 0x3447dff3, 90 },
    { 0x74f7dff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c7dff7, 23 },
    { 0x63b7dff8, 33 },
    { 0x42c7dff9, 17 },
    { 0x8d27dff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dfff, 90 },
    { 0x77c7dfff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c27e003, 23 },
    { 0x1847e004, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa457e007, 40 },
    { 0x7247e008, 95 },
    { 0, 0 },
    { 0xa817e00a, 33 },
    { 0, 0 },
    { 0x8837e00c, 55 },
    { 0xdbf7e00d, 55 },
    { 0xc9e7e00d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e7e014, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7667e018, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17e021, 51 },
    { 0x1367e022, 98 },
    { 0xa0f7e023, 98 },
    { 0, 0 },
    { 0x5cd7e025, 61 },
    { 0, 0 },
    { 0x717e027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f7e02d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2197e035, 48 },
    { 0x9df7e036, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef7e03c, 90 },
    { 0, 0 },
    { 0x9ae7e03e, 88 },
    { 0x9137e03f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1357e044, 95 },
    { 0xf147e045, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1507e04d, 55 },
    { 0xb337e04e, 55 },
    { 0, 0 },
    { 0x2fc7e050, 90 },
    { 0, 0 },
    { 0x72a7e052, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7077e056, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e7e059, 88 },
    { 0x4c67e05a, 61 },
    { 0x1787e059, 93 },
    { 0xa2e7e05c, 55 },
    { 0x6a87e05c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4997e061, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d7e065, 55 },
    { 0xbea7e066, 46 },
    { 0xb3e7e067, 90 },
    { 0, 0 },
    { 0x7717e069, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc057e072, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e7e076, 58 },
    { 0xcf87e077, 69 },
    { 0x5487e078, 98 },
    { 0x20e7e079, 90 },
    { 0xbba7e07a, 33 },
    { 0x1487e079, 76 },
    { 0x43e7e078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67e07f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2457e085, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9017e08a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c7e08e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda7e095, 33 },
    { 0, 0 },
    { 0xe757e097, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c7e09c, 61 },
    { 0xe027e09d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced7e0ab, 33 },
    { 0x3f47e0ab, 90 },
    { 0xcde7e0ad, 76 },
    { 0xf027e0ad, 90 },
    { 0x3537e0ae, 46 },
    { 0x61d7e0ad, 33 },
    { 0x7067e0b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe537e0b6, 61 },
    { 0, 0 },
    { 0x6157e0b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe827e0c3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da7e0c8, 23 },
    { 0xd267e0c9, 63 },
    { 0x717e0ca, 88 },
    { 0, 0 },
    { 0x2387e0cc, 98 },
    { 0, 0 },
    { 0xb147e0ce, 43 },
    { 0, 0 },
    { 0x8e67e0d0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d7e0d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b7e0d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf57e0dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c57e0e2, 33 },
    { 0xd357e0e2, 36 },
    { 0x93d7e0e4, 95 },
    { 0x2107e0e3, 98 },
    { 0x9507e0e6, 98 },
    { 0x2a17e0e5, 76 },
    { 0xc867e0e8, 87 },
    { 0xf3f7e0e9, 85 },
    { 0x8bc7e0e8, 55 },
    { 0xe727e0eb, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a7e0ee, 95 },
    { 0xe267e0ee, 61 },
    { 0x6467e0f0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf57e0f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c67e0f9, 85 },
    { 0, 0 },
    { 0x6707e0fb, 88 },
    { 0, 0 },
    { 0x4e47e0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e7e103, 55 },
    { 0x7ad7e104, 53 },
    { 0xcd37e105, 93 },
    { 0x7e106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a7e10b, 61 },
    { 0xa6f7e10b, 61 },
    { 0xed57e10c, 98 },
    { 0xa2a7e10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c7e111, 61 },
    { 0xc9f7e112, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1617e115, 33 },
    { 0, 0 },
    { 0x63d7e117, 76 },
    { 0x6ad7e118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b27e11b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe227e11f, 98 },
    { 0xd5f7e120, 65 },
    { 0x98c7e120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8197e125, 90 },
    { 0x15c7e126, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde77e12b, 33 },
    { 0xb317e12b, 99 },
    { 0xde67e12b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb37e131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb017e134, 76 },
    { 0xb497e135, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c7e138, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7e13c, 23 },
    { 0, 0 },
    { 0x3437e13e, 99 },
    { 0x4977e13e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7e144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe207e14c, 33 },
    { 0xc577e14c, 46 },
    { 0x4097e14c, 23 },
    { 0x4167e14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7e154, 61 },
    { 0xb617e154, 23 },
    { 0xea47e154, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5917e161, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e7e165, 81 },
    { 0xb57e165, 22 },
    { 0xaa7e166, 90 },
    { 0x5737e168, 98 },
    { 0, 0 },
    { 0xacd7e16a, 76 },
    { 0x12f7e16b, 55 },
    { 0x63d7e16a, 43 },
    { 0, 0 },
    { 0x2a07e16e, 33 },
    { 0xcf77e16f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57e172, 61 },
    { 0x4217e173, 46 },
    { 0x40c7e174, 90 },
    { 0x3ff7e175, 90 },
    { 0x2407e176, 88 },
    { 0x9cb7e176, 76 },
    { 0x637e178, 98 },
    { 0, 0 },
    { 0xa877e17a, 61 },
    { 0xb087e17a, 33 },
    { 0x3977e17c, 90 },
    { 0x4dc7e17d, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b7e181, 46 },
    { 0, 0 },
    { 0x9527e183, 60 },
    { 0x45f7e183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd787e188, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5827e18c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc627e192, 61 },
    { 0x29e7e193, 80 },
    { 0x46d7e193, 95 },
    { 0x1747e193, 61 },
    { 0, 0 },
    { 0x8647e197, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8477e19a, 100 },
    { 0xef27e19a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb517e19f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5677e1a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc107e1a9, 98 },
    { 0xc227e1aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce7e1ad, 48 },
    { 0, 0 },
    { 0xb147e1af, 90 },
    { 0xc6e7e1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767e1b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217e1ba, 33 },
    { 0xc97e1ba, 66 },
    { 0xbc77e1bc, 98 },
    { 0x1367e1bd, 23 },
    { 0, 0 },
    { 0x44f7e1bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557e1c3, 90 },
    { 0, 0 },
    { 0x3767e1c5, 61 },
    { 0xdd7e1c6, 95 },
    { 0xf27e1c6, 76 },
    { 0xac17e1c8, 84 },
    { 0x4e57e1c9, 98 },
    { 0, 0 },
    { 0xa7e7e1cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7e1d2, 46 },
    { 0x187e1d3, 46 },
    { 0x22a7e1d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f7e1da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b7e1de, 23 },
    { 0, 0 },
    { 0xa1a7e1e0, 79 },
    { 0, 0 },
    { 0x8ea7e1e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7e1e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4077e1ec, 91 },
    { 0, 0 },
    { 0x92d7e1ee, 98 },
    { 0x8837e1ef, 61 },
    { 0, 0 },
    { 0x29e7e1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47e1f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb7e1f8, 88 },
    { 0xe517e1f9, 33 },
    { 0xd937e1fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d17e1fd, 33 },
    { 0xa117e1fe, 95 },
    { 0xfb17e1fd, 95 },
    { 0x7137e1ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7e205, 76 },
    { 0x5bc7e206, 23 },
    { 0xc4a7e205, 93 },
    { 0xe917e208, 33 },
    { 0, 0 },
    { 0x95e7e20a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x867e20d, 80 },
    { 0xe7a7e20e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c7e212, 55 },
    { 0x3457e213, 90 },
    { 0xf6f7e214, 36 },
    { 0xe197e214, 23 },
    { 0xfc07e215, 33 },
    { 0x8be7e214, 90 },
    { 0, 0 },
    { 0xcac7e219, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b27e21f, 95 },
    { 0xc037e21f, 99 },
    { 0xdf47e221, 61 },
    { 0xddf7e222, 55 },
    { 0x4337e223, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b7e229, 84 },
    { 0x8957e22a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3897e22e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a7e232, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9067e236, 34 },
    { 0x62f7e237, 61 },
    { 0xb117e238, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea7e23b, 78 },
    { 0x2a67e23c, 90 },
    { 0, 0 },
    { 0x4ac7e23e, 87 },
    { 0x9157e23f, 95 },
    { 0x6847e240, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee7e245, 98 },
    { 0x5f87e246, 33 },
    { 0x1597e245, 98 },
    { 0xf9c7e248, 34 },
    { 0x8aa7e248, 90 },
    { 0xa287e24a, 90 },
    { 0x6ea7e24b, 33 },
    { 0xe887e249, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7817e24f, 95 },
    { 0xc3f7e250, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87e253, 84 },
    { 0xe967e253, 95 },
    { 0xc5a7e254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d7e25d, 90 },
    { 0xf07e25d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6927e263, 84 },
    { 0x5227e264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c7e267, 99 },
    { 0, 0 },
    { 0xb407e269, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497e26c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff77e273, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb37e27b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb7e27f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea7e282, 90 },
    { 0xd597e283, 91 },
    { 0x9777e282, 33 },
    { 0x81e7e285, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b77e28d, 61 },
    { 0x6267e28d, 75 },
    { 0xa277e28f, 34 },
    { 0xfcf7e28d, 23 },
    { 0x4a77e28f, 33 },
    { 0xa317e28d, 33 },
    { 0x447e293, 61 },
    { 0x50c7e293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f7e297, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e7e29a, 61 },
    { 0x9c7e29b, 90 },
    { 0x2a57e29a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6747e29f, 61 },
    { 0x6277e2a0, 90 },
    { 0xc047e29f, 87 },
    { 0xddc7e2a2, 78 },
    { 0x7987e2a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9817e2a7, 88 },
    { 0x30c7e2a8, 95 },
    { 0, 0 },
    { 0xe9a7e2aa, 93 },
    { 0x3bd7e2ab, 88 },
    { 0xf1f7e2ac, 33 },
    { 0x3027e2ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6267e2b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f7e2b3, 90 },
    { 0x3c47e2b3, 58 },
    { 0, 0 },
    { 0x8177e2b6, 67 },
    { 0x3d07e2b7, 23 },
    { 0xe7e7e2b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd917e2bb, 100 },
    { 0x4337e2bc, 98 },
    { 0xb147e2bd, 59 },
    { 0, 0 },
    { 0x9dd7e2bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c17e2c4, 95 },
    { 0x207e2c4, 61 },
    { 0xf7a7e2c5, 95 },
    { 0x86b7e2c5, 91 },
    { 0x5537e2c8, 76 },
    { 0xe677e2c9, 77 },
    { 0, 0 },
    { 0x1577e2cb, 88 },
    { 0xfa57e2cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee7e2d1, 52 },
    { 0x13f7e2d1, 23 },
    { 0xe1a7e2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7127e2d7, 66 },
    { 0x3c07e2d8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae7e2de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c7e2e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c77e2e8, 90 },
    { 0xbdc7e2e8, 95 },
    { 0x74e7e2ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f7e2ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a7e2f3, 61 },
    { 0x3537e2f4, 90 },
    { 0x3b37e2f5, 98 },
    { 0xe7a7e2f6, 95 },
    { 0x9607e2f4, 34 },
    { 0x4e97e2f4, 34 },
    { 0xb417e2f9, 98 },
    { 0xb397e2f9, 23 },
    { 0, 0 },
    { 0x2827e2fc, 90 },
    { 0x49d7e2fd, 95 },
    { 0x29c7e2fe, 61 },
    { 0xfaa7e2ff, 55 },
    { 0xd17e2fe, 43 },
    { 0x29e7e2fe, 33 },
    { 0, 0 },
    { 0xbfd7e303, 90 },
    { 0, 0 },
    { 0x5737e305, 75 },
    { 0xedc7e306, 33 },
    { 0xa467e306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd27e30a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817e30f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f7e313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7e318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa7e324, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec27e327, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b47e32f, 33 },
    { 0x7927e330, 61 },
    { 0xeed7e331, 90 },
    { 0xfcc7e330, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f7e336, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca7e33f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de7e342, 81 },
    { 0x3cd7e343, 46 },
    { 0xcad7e344, 43 },
    { 0, 0 },
    { 0x50f7e346, 43 },
    { 0xafc7e347, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb7e359, 76 },
    { 0, 0 },
    { 0x1227e35b, 87 },
    { 0x8367e35c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4437e361, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe597e364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec57e367, 23 },
    { 0, 0 },
    { 0x18a7e369, 90 },
    { 0xdbe7e36a, 95 },
    { 0, 0 },
    { 0xd017e36c, 33 },
    { 0x10c7e36c, 61 },
    { 0x61b7e36e, 68 },
    { 0, 0 },
    { 0x2da7e370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd227e374, 76 },
    { 0x3a27e375, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa7e378, 38 },
    { 0, 0 },
    { 0xb2e7e37a, 61 },
    { 0, 0 },
    { 0xefe7e37c, 95 },
    { 0xe727e37d, 17 },
    { 0xf827e37e, 90 },
    { 0xb897e37e, 33 },
    { 0xd237e37e, 61 },
    { 0, 0 },
    { 0x79a7e382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb617e385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa557e388, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5427e38c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d7e395, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe767e39b, 61 },
    { 0xd617e39c, 43 },
    { 0xc1b7e39d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb807e3a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e47e3a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c7e3ad, 90 },
    { 0x1c67e3ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4317e3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9147e3b5, 76 },
    { 0, 0 },
    { 0x8ff7e3b7, 61 },
    { 0x36d7e3b8, 98 },
    { 0, 0 },
    { 0x4fb7e3ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc7e3bd, 61 },
    { 0xaff7e3be, 91 },
    { 0x1f67e3bf, 90 },
    { 0xc647e3be, 99 },
    { 0xa217e3be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f7e3c4, 55 },
    { 0, 0 },
    { 0xe097e3c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd67e3c9, 90 },
    { 0x81c7e3c9, 23 },
    { 0xe627e3cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8597e3d0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc057e3db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9217e3df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f07e3e3, 52 },
    { 0xea97e3e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd257e3ea, 33 },
    { 0, 0 },
    { 0x7ff7e3ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5407e3ef, 33 },
    { 0, 0 },
    { 0xa467e3f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f7e3f6, 88 },
    { 0xa937e3f7, 77 },
    { 0x4417e3f8, 98 },
    { 0x7277e3f7, 23 },
    { 0, 0 },
    { 0x827e3fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf537e3ff, 33 },
    { 0xe4f7e400, 33 },
    { 0, 0 },
    { 0x7337e402, 68 },
    { 0, 0 },
    { 0x4057e404, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567e409, 90 },
    { 0x9b67e40a, 95 },
    { 0, 0 },
    { 0x2e17e40c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e7e40f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee57e412, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d27e415, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b7e420, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d7e423, 69 },
    { 0x86c7e423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e7e428, 33 },
    { 0x3b37e428, 91 },
    { 0, 0 },
    { 0x9817e42b, 66 },
    { 0x5ba7e42b, 93 },
    { 0x5467e42d, 61 },
    { 0xee47e42d, 95 },
    { 0xb9a7e42f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed27e435, 95 },
    { 0x34e7e436, 95 },
    { 0x8c7e437, 100 },
    { 0xb127e438, 55 },
    { 0xbfd7e436, 46 },
    { 0x3447e43a, 23 },
    { 0, 0 },
    { 0xd237e43c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e43f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd507e443, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e7e448, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4697e450, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27e453, 98 },
    { 0, 0 },
    { 0xaf77e455, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e7e464, 90 },
    { 0xb3e7e464, 43 },
    { 0xa667e465, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27e46a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c97e472, 33 },
    { 0, 0 },
    { 0x85d7e474, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5847e477, 95 },
    { 0, 0 },
    { 0x8187e479, 88 },
    { 0x567e479, 95 },
    { 0, 0 },
    { 0x29f7e47c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc827e47f, 43 },
    { 0, 0 },
    { 0x68b7e481, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7027e484, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xba47e487, 23 },
    { 0x9587e487, 33 },
    { 0, 0 },
    { 0x4947e48a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2987e496, 98 },
    { 0xa957e496, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab7e49e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e4ae, 98 },
    { 0, 0 },
    { 0xd817e4b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe837e4b4, 46 },
    { 0x6be7e4b5, 33 },
    { 0xaa87e4b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4467e4b9, 90 },
    { 0x75a7e4b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c7e4be, 90 },
    { 0, 0 },
    { 0x7087e4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a7e4c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3177e4c7, 99 },
    { 0x2367e4c8, 61 },
    { 0xac47e4c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b7e4cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea7e4d0, 64 },
    { 0, 0 },
    { 0x2027e4d2, 90 },
    { 0xd837e4d2, 23 },
    { 0xfae7e4d4, 34 },
    { 0x3947e4d5, 56 },
    { 0x7fe7e4d6, 76 },
    { 0x837e4d5, 90 },
    { 0, 0 },
    { 0x9207e4d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa927e4df, 67 },
    { 0, 0 },
    { 0xd5a7e4e1, 61 },
    { 0, 0 },
    { 0x5617e4e3, 90 },
    { 0, 0 },
    { 0xc4c7e4e5, 33 },
    { 0x9ab7e4e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d7e4e9, 61 },
    { 0, 0 },
    { 0x44f7e4eb, 66 },
    { 0xb1f7e4ec, 61 },
    { 0, 0 },
    { 0x24b7e4ee, 46 },
    { 0x5187e4ef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a7e4f3, 61 },
    { 0x2f07e4f3, 95 },
    { 0x8c07e4f3, 77 },
    { 0x6647e4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77e4fa, 96 },
    { 0x7a37e4fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc977e502, 33 },
    { 0x3b67e503, 91 },
    { 0, 0 },
    { 0x2bf7e505, 43 },
    { 0xb327e506, 61 },
    { 0x9d57e507, 98 },
    { 0xdec7e508, 98 },
    { 0xd8f7e507, 76 },
    { 0xf907e50a, 51 },
    { 0x2127e50b, 66 },
    { 0xc387e508, 98 },
    { 0x9a57e50d, 66 },
    { 0x9407e50c, 61 },
    { 0, 0 },
    { 0x6937e510, 33 },
    { 0x8847e511, 95 },
    { 0xe957e512, 95 },
    { 0, 0 },
    { 0xaf7e514, 61 },
    { 0x59d7e515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3497e518, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3137e51b, 95 },
    { 0xee07e51b, 34 },
    { 0, 0 },
    { 0x4e7e51e, 23 },
    { 0x6717e51e, 90 },
    { 0x6727e51e, 91 },
    { 0xee57e51f, 55 },
    { 0xb207e522, 94 },
    { 0xa2b7e521, 34 },
    { 0x8467e51f, 55 },
    { 0xc857e525, 90 },
    { 0, 0 },
    { 0xda77e527, 61 },
    { 0x5e97e527, 38 },
    { 0x15c7e528, 75 },
    { 0x83b7e52a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe97e531, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce17e536, 61 },
    { 0xcbd7e537, 99 },
    { 0, 0 },
    { 0x9387e539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d7e53f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e7e542, 33 },
    { 0x8207e543, 90 },
    { 0xedb7e544, 61 },
    { 0x79e7e544, 90 },
    { 0xab97e546, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba7e54c, 43 },
    { 0, 0 },
    { 0x96b7e54e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e07e556, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8667e559, 87 },
    { 0, 0 },
    { 0x86c7e55b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d7e561, 33 },
    { 0xc4c7e562, 33 },
    { 0, 0 },
    { 0x6127e564, 61 },
    { 0x8267e565, 43 },
    { 0xf3d7e566, 90 },
    { 0xeca7e566, 76 },
    { 0x70d7e566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b57e56b, 90 },
    { 0x4ce7e56c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce7e570, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3997e573, 90 },
    { 0xb2a7e574, 61 },
    { 0xc9f7e574, 23 },
    { 0x82b7e576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f7e57f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8617e586, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd57e589, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727e58e, 77 },
    { 0xf507e58f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd107e594, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b87e598, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77e59d, 84 },
    { 0xa4d7e59e, 68 },
    { 0xfcd7e59f, 55 },
    { 0, 0 },
    { 0x5537e5a1, 90 },
    { 0x8fb7e5a2, 95 },
    { 0xef17e5a3, 99 },
    { 0, 0 },
    { 0x927e5a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc47e5a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe867e5b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e7e5b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba7e5bd, 66 },
    { 0, 0 },
    { 0x5747e5bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9277e5c5, 37 },
    { 0, 0 },
    { 0x9f87e5c7, 57 },
    { 0xced7e5c7, 23 },
    { 0xa2f7e5c7, 98 },
    { 0xf927e5ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c97e5d0, 88 },
    { 0xebf7e5d1, 95 },
    { 0, 0 },
    { 0x8be7e5d3, 95 },
    { 0xfa17e5d4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd487e5d7, 90 },
    { 0x1007e5d8, 84 },
    { 0xbbd7e5d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7e5dc, 33 },
    { 0, 0 },
    { 0x917e5de, 61 },
    { 0xf877e5de, 84 },
    { 0x417e5e0, 77 },
    { 0, 0 },
    { 0xfcc7e5e2, 99 },
    { 0x6f67e5e3, 90 },
    { 0, 0 },
    { 0x36c7e5e5, 33 },
    { 0x2597e5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8707e5ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e7e5ed, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc767e5f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8577e5f5, 90 },
    { 0x8847e5f6, 61 },
    { 0x9197e5f5, 55 },
    { 0xfce7e5f8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc7e600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e7e604, 61 },
    { 0xd197e605, 87 },
    { 0x5897e606, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf447e60b, 23 },
    { 0x8b37e60c, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f7e613, 34 },
    { 0x9ac7e614, 61 },
    { 0x8b17e615, 90 },
    { 0xdb77e614, 98 },
    { 0, 0 },
    { 0xc657e618, 90 },
    { 0, 0 },
    { 0xb777e61a, 23 },
    { 0x6a07e61b, 96 },
    { 0, 0 },
    { 0xa4b7e61d, 85 },
    { 0xce57e61e, 61 },
    { 0x97e61f, 33 },
    { 0x15a7e61d, 66 },
    { 0, 0 },
    { 0x75c7e622, 90 },
    { 0x997e623, 33 },
    { 0x1da7e624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb647e629, 90 },
    { 0xf827e62a, 95 },
    { 0xb0b7e62b, 43 },
    { 0x5cc7e629, 87 },
    { 0x95b7e62a, 23 },
    { 0x6597e62e, 90 },
    { 0x1a37e62f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707e633, 75 },
    { 0xc827e634, 87 },
    { 0xbb37e635, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7e639, 61 },
    { 0xfe27e63a, 52 },
    { 0, 0 },
    { 0x10d7e63c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7e642, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa07e646, 84 },
    { 0, 0 },
    { 0x8c47e648, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e87e64b, 98 },
    { 0xbd87e64c, 66 },
    { 0xa897e64d, 95 },
    { 0x5cf7e64e, 61 },
    { 0x71c7e64d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a7e652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1487e656, 23 },
    { 0, 0 },
    { 0xa027e658, 55 },
    { 0x6f87e659, 88 },
    { 0x1e57e65a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef7e65d, 23 },
    { 0x6bc7e65d, 61 },
    { 0x1da7e65f, 90 },
    { 0, 0 },
    { 0xd9e7e661, 66 },
    { 0, 0 },
    { 0xb1c7e663, 34 },
    { 0x7127e663, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3757e667, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e47e66d, 90 },
    { 0, 0 },
    { 0xe4a7e66f, 78 },
    { 0xd47e66f, 90 },
    { 0xec77e670, 95 },
    { 0x9947e66f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a7e679, 90 },
    { 0xa417e679, 101 },
    { 0xa8d7e67b, 61 },
    { 0x5b27e679, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd17e67f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7e682, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4847e685, 95 },
    { 0x3a97e686, 93 },
    { 0, 0 },
    { 0xe837e688, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad7e68b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f57e68f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b27e693, 61 },
    { 0x4417e694, 67 },
    { 0x3477e695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f7e699, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1757e69c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d7e69f, 33 },
    { 0, 0 },
    { 0xd37e6a1, 23 },
    { 0xc657e6a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc7e6a5, 61 },
    { 0, 0 },
    { 0x5697e6a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7e6b4, 66 },
    { 0, 0 },
    { 0xe837e6b6, 90 },
    { 0x8777e6b7, 55 },
    { 0x3447e6b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae67e6bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c7e6c1, 95 },
    { 0x1ae7e6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8107e6c5, 67 },
    { 0xf857e6c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca7e6ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e17e6d2, 33 },
    { 0x727e6d3, 90 },
    { 0xb3b7e6d2, 33 },
    { 0xbf67e6d4, 90 },
    { 0x4167e6d2, 43 },
    { 0x3f07e6d6, 61 },
    { 0xb47e6d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa07e6dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa027e6e2, 97 },
    { 0xcdf7e6e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb7e6ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c7e6f1, 61 },
    { 0, 0 },
    { 0x3d97e6f3, 100 },
    { 0xc847e6f4, 90 },
    { 0x6ab7e6f5, 76 },
    { 0x5b07e6f6, 46 },
    { 0, 0 },
    { 0x2937e6f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa7e6fb, 61 },
    { 0, 0 },
    { 0x4477e6fd, 98 },
    { 0x1e7e6fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7e70b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b57e712, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f7e718, 88 },
    { 0x33f7e719, 90 },
    { 0xb127e71a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf77e722, 33 },
    { 0x1297e722, 33 },
    { 0xa587e722, 93 },
    { 0xfe67e725, 75 },
    { 0, 0 },
    { 0xf9d7e727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b7e72b, 55 },
    { 0, 0 },
    { 0xeb97e72d, 95 },
    { 0x89e7e72e, 90 },
    { 0xae47e72f, 61 },
    { 0, 0 },
    { 0x4be7e731, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea47e737, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb7e73a, 95 },
    { 0xcc87e73b, 46 },
    { 0, 0 },
    { 0xd577e73d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b7e741, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd567e74d, 69 },
    { 0xa927e74e, 85 },
    { 0x6627e74d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e77e754, 85 },
    { 0xcda7e755, 76 },
    { 0x95a7e756, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7e75d, 61 },
    { 0x7c57e75e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5537e762, 76 },
    { 0x5037e763, 91 },
    { 0, 0 },
    { 0xcb77e765, 75 },
    { 0xa5d7e766, 98 },
    { 0, 0 },
    { 0xc4c7e768, 61 },
    { 0xc4f7e768, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d87e770, 46 },
    { 0, 0 },
    { 0x3d67e772, 90 },
    { 0xc377e773, 90 },
    { 0x72c7e774, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a7e77b, 43 },
    { 0, 0 },
    { 0x1427e77d, 56 },
    { 0, 0 },
    { 0x68e7e77f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1937e782, 23 },
    { 0x4077e782, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7697e78b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb277e790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa97e79f, 61 },
    { 0, 0 },
    { 0x17a7e7a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7197e7a7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe587e7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3567e7af, 23 },
    { 0, 0 },
    { 0x8147e7b1, 90 },
    { 0, 0 },
    { 0x13b7e7b3, 83 },
    { 0x21e7e7b4, 43 },
    { 0xcdc7e7b5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a7e7bd, 33 },
    { 0xa9c7e7be, 78 },
    { 0x8ba7e7be, 98 },
    { 0x68b7e7c0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c7e7c4, 88 },
    { 0x7297e7c5, 92 },
    { 0, 0 },
    { 0x8f87e7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f47e7cd, 100 },
    { 0, 0 },
    { 0x47b7e7cf, 46 },
    { 0, 0 },
    { 0x1767e7d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e77e7da, 90 },
    { 0xa7f7e7da, 61 },
    { 0x5b07e7dc, 85 },
    { 0xbb37e7dc, 95 },
    { 0x5df7e7de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba7e7e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c7e7e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6367e7ec, 90 },
    { 0x8fe7e7ec, 55 },
    { 0x2d27e7ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a77e7f1, 90 },
    { 0, 0 },
    { 0x7647e7f3, 52 },
    { 0, 0 },
    { 0x2f37e7f5, 52 },
    { 0x62d7e7f5, 75 },
    { 0xc717e7f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb027e7fa, 66 },
    { 0x78f7e7fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff7e7ff, 78 },
    { 0, 0 },
    { 0x6557e801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b7e806, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed27e80e, 69 },
    { 0x8fd7e80f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab7e813, 33 },
    { 0, 0 },
    { 0x2e97e815, 52 },
    { 0xa4b7e816, 85 },
    { 0, 0 },
    { 0xa227e818, 95 },
    { 0xc1d7e819, 85 },
    { 0x6467e81a, 90 },
    { 0x8897e81b, 95 },
    { 0x75a7e81a, 67 },
    { 0x667e81d, 66 },
    { 0xd127e81d, 33 },
    { 0xb467e81f, 85 },
    { 0xb77e81f, 55 },
    { 0xc477e81f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c7e828, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c7e82c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e7e82f, 38 },
    { 0xf407e82f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef7e835, 61 },
    { 0, 0 },
    { 0x1067e837, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727e83c, 87 },
    { 0xe757e83d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee07e840, 67 },
    { 0, 0 },
    { 0xe07e842, 34 },
    { 0, 0 },
    { 0xd467e844, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c7e847, 34 },
    { 0x23e7e848, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b17e84e, 80 },
    { 0, 0 },
    { 0x63e7e850, 77 },
    { 0x37e7e851, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6517e854, 61 },
    { 0xd427e855, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da7e85e, 84 },
    { 0x93b7e85f, 84 },
    { 0x7137e85f, 85 },
    { 0, 0 },
    { 0x7fe7e862, 33 },
    { 0xac27e863, 69 },
    { 0, 0 },
    { 0x8027e865, 83 },
    { 0xe0b7e865, 33 },
    { 0x8647e867, 61 },
    { 0x2aa7e868, 34 },
    { 0xaa07e869, 98 },
    { 0xbaa7e866, 67 },
    { 0, 0 },
    { 0xe6e7e86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db7e872, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7e875, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad7e878, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a87e87b, 90 },
    { 0x9987e87c, 76 },
    { 0x8de7e87b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd867e880, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4167e887, 33 },
    { 0x8f77e888, 80 },
    { 0, 0 },
    { 0xd5a7e88a, 33 },
    { 0, 0 },
    { 0x8b7e88c, 90 },
    { 0x48b7e88d, 90 },
    { 0x8e97e88e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa747e899, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df7e89e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f57e8a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3157e8af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa887e8b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e57e8b7, 66 },
    { 0x7ea7e8b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef7e8c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5247e8c7, 90 },
    { 0, 0 },
    { 0xc557e8c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa207e8cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d67e8d0, 90 },
    { 0x1d7e8d1, 87 },
    { 0xea77e8d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f17e8da, 76 },
    { 0xff77e8da, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc377e8de, 46 },
    { 0xf677e8df, 61 },
    { 0x1a07e8e0, 33 },
    { 0xc977e8df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec57e8e8, 55 },
    { 0x5897e8e9, 78 },
    { 0, 0 },
    { 0xf3e7e8eb, 33 },
    { 0, 0 },
    { 0x60f7e8ed, 93 },
    { 0x9777e8ee, 76 },
    { 0, 0 },
    { 0x44a7e8f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb257e8fb, 23 },
    { 0, 0 },
    { 0x69d7e8fd, 84 },
    { 0x3dc7e8fd, 90 },
    { 0, 0 },
    { 0xe427e900, 23 },
    { 0xc207e901, 90 },
    { 0xfda7e902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef7e906, 98 },
    { 0x2367e906, 99 },
    { 0x7b87e907, 33 },
    { 0xf8e7e909, 93 },
    { 0xa677e909, 46 },
    { 0xe197e907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597e90e, 67 },
    { 0x21f7e90f, 93 },
    { 0x1307e90e, 65 },
    { 0, 0 },
    { 0xf187e912, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b7e917, 93 },
    { 0xaee7e918, 91 },
    { 0x59f7e919, 61 },
    { 0xf4c7e91a, 55 },
    { 0xafe7e91a, 24 },
    { 0x5237e917, 33 },
    { 0x38f7e91d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a57e925, 98 },
    { 0, 0 },
    { 0xd587e927, 75 },
    { 0, 0 },
    { 0x98b7e929, 93 },
    { 0x5647e92a, 34 },
    { 0, 0 },
    { 0xd797e92c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a7e930, 90 },
    { 0x2ac7e930, 61 },
    { 0, 0 },
    { 0x8e87e933, 46 },
    { 0x5887e934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d17e939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b7e93d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b47e946, 84 },
    { 0, 0 },
    { 0xd6e7e948, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b37e94b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8507e94e, 88 },
    { 0xb047e94f, 95 },
    { 0xd487e950, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c57e953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a7e957, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc57e960, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27e963, 77 },
    { 0, 0 },
    { 0xff97e965, 90 },
    { 0, 0 },
    { 0x7dc7e967, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf87e971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed7e976, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e87e97a, 34 },
    { 0, 0 },
    { 0x8a97e97c, 95 },
    { 0x4737e97d, 34 },
    { 0x6537e97e, 91 },
    { 0x9877e97f, 90 },
    { 0x6987e97c, 33 },
    { 0x6147e981, 99 },
    { 0x14e7e982, 76 },
    { 0, 0 },
    { 0xae97e984, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17e989, 69 },
    { 0, 0 },
    { 0x9907e98b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5187e990, 90 },
    { 0x1fb7e991, 23 },
    { 0xcf27e990, 78 },
    { 0x3197e992, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce7e99f, 46 },
    { 0, 0 },
    { 0x2617e9a1, 91 },
    { 0, 0 },
    { 0x2367e9a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5557e9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8007e9ad, 2 },
    { 0x5287e9ae, 61 },
    { 0, 0 },
    { 0xd9d7e9b0, 52 },
    { 0xf5a7e9b1, 90 },
    { 0xbea7e9b2, 61 },
    { 0x177e9b3, 98 },
    { 0x70d7e9b4, 76 },
    { 0x42b7e9b0, 61 },
    { 0x2f97e9b6, 17 },
    { 0xee17e9b7, 78 },
    { 0x3bc7e9b7, 90 },
    { 0, 0 },
    { 0x1097e9ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7e9be, 87 },
    { 0x53c7e9be, 43 },
    { 0x6ad7e9be, 61 },
    { 0x4a97e9c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb387e9c5, 33 },
    { 0x6927e9c6, 90 },
    { 0xd1d7e9c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387e9cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8827e9d2, 61 },
    { 0x11f7e9d3, 90 },
    { 0, 0 },
    { 0x8c7e9d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c7e9da, 43 },
    { 0xe267e9db, 67 },
    { 0x5877e9db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb797e9ea, 90 },
    { 0x7437e9ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a7e9f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7947e9fc, 33 },
    { 0, 0 },
    { 0x8a77e9fe, 90 },
    { 0x1f77e9fe, 98 },
    { 0x85d7ea00, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa727ea03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d7ea06, 78 },
    { 0xf937ea07, 78 },
    { 0x6d7ea08, 91 },
    { 0x94d7ea08, 76 },
    { 0xaf27ea0a, 55 },
    { 0x9927ea09, 34 },
    { 0xac47ea0c, 98 },
    { 0xfd57ea0b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c7ea12, 61 },
    { 0xef37ea13, 23 },
    { 0x5387ea13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc7ea17, 90 },
    { 0x8187ea18, 76 },
    { 0x6357ea17, 76 },
    { 0x9f27ea1a, 95 },
    { 0x52f7ea1a, 66 },
    { 0x4587ea1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae7ea20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a97ea23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3577ea27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4367ea31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1287ea35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7187ea39, 90 },
    { 0x1d77ea39, 33 },
    { 0, 0 },
    { 0x5867ea3c, 33 },
    { 0x34a7ea3d, 33 },
    { 0x9f7ea3e, 43 },
    { 0x8b27ea3f, 33 },
    { 0xcff7ea3f, 90 },
    { 0x3d67ea41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b7ea4a, 61 },
    { 0, 0 },
    { 0x8d97ea4c, 23 },
    { 0xa967ea4c, 95 },
    { 0xd677ea4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a7ea56, 61 },
    { 0, 0 },
    { 0xe027ea58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f7ea5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb7ea61, 46 },
    { 0x75d7ea61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc147ea69, 43 },
    { 0, 0 },
    { 0x2e07ea6b, 95 },
    { 0, 0 },
    { 0x7867ea6d, 65 },
    { 0, 0 },
    { 0xa057ea6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f37ea75, 55 },
    { 0, 0 },
    { 0x5b37ea77, 90 },
    { 0x3bf7ea78, 80 },
    { 0x4377ea78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d97ea7c, 33 },
    { 0xe897ea7d, 90 },
    { 0xd1b7ea7e, 76 },
    { 0x41f7ea7e, 90 },
    { 0x7577ea7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8497ea89, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b7ea8c, 76 },
    { 0xf0f7ea8d, 68 },
    { 0x4bd7ea8d, 61 },
    { 0x3117ea8e, 33 },
    { 0xb327ea90, 23 },
    { 0x3907ea91, 81 },
    { 0x5c97ea92, 90 },
    { 0x65b7ea91, 90 },
    { 0x2637ea94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf847ea9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe757ea9f, 64 },
    { 0, 0 },
    { 0xe0e7eaa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa27eaa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf147eaa8, 43 },
    { 0, 0 },
    { 0x29d7eaaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9917eab1, 95 },
    { 0, 0 },
    { 0xbb47eab3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb27eab7, 93 },
    { 0x3127eab7, 95 },
    { 0, 0 },
    { 0x19d7eaba, 95 },
    { 0x3f87eabb, 99 },
    { 0x92b7eabc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba27eabf, 90 },
    { 0x5927eac0, 34 },
    { 0xd407eac1, 33 },
    { 0xb3d7eac1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e7eac5, 33 },
    { 0, 0 },
    { 0x3337eac7, 76 },
    { 0xb5f7eac8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac7eacc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1647ead1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7ead4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4327ead9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e7eadf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f7eae2, 90 },
    { 0, 0 },
    { 0xbd47eae4, 61 },
    { 0x5197eae5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b87eaea, 33 },
    { 0x5937eaeb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b7eaf3, 76 },
    { 0xbd37eaf3, 76 },
    { 0, 0 },
    { 0x9357eaf6, 55 },
    { 0, 0 },
    { 0x4757eaf8, 84 },
    { 0x3f67eaf9, 55 },
    { 0, 0 },
    { 0xad87eafb, 68 },
    { 0x1097eafc, 61 },
    { 0xa3a7eafb, 93 },
    { 0xd027eafc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f27eb02, 23 },
    { 0, 0 },
    { 0xc867eb04, 52 },
    { 0xd257eb04, 95 },
    { 0x7c57eb06, 43 },
    { 0, 0 },
    { 0x61b7eb08, 51 },
    { 0xe7d7eb08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f7eb0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9177eb10, 33 },
    { 0, 0 },
    { 0x1407eb12, 43 },
    { 0xf517eb13, 61 },
    { 0x68d7eb14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab17eb18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b87eb1f, 90 },
    { 0xaa7eb20, 98 },
    { 0xbf67eb21, 100 },
    { 0, 0 },
    { 0x757eb23, 76 },
    { 0x8cc7eb24, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4497eb28, 98 },
    { 0, 0 },
    { 0x1ce7eb2a, 34 },
    { 0xeb17eb2b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a47eb2f, 85 },
    { 0x5bc7eb30, 90 },
    { 0, 0 },
    { 0xf9c7eb32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa747eb35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d7eb3b, 61 },
    { 0xe217eb3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6947eb40, 23 },
    { 0x7e67eb41, 52 },
    { 0xd687eb40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa77eb46, 43 },
    { 0, 0 },
    { 0x98c7eb48, 51 },
    { 0x7b77eb48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b67eb53, 33 },
    { 0xf347eb54, 90 },
    { 0, 0 },
    { 0xac27eb56, 17 },
    { 0xcaa7eb56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f7eb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d7eb62, 85 },
    { 0, 0 },
    { 0x7c57eb64, 55 },
    { 0x9367eb65, 33 },
    { 0x3257eb64, 80 },
    { 0xdca7eb67, 34 },
    { 0x1827eb67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b07eb6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf187eb6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657eb76, 90 },
    { 0x79c7eb77, 91 },
    { 0xc7eb77, 98 },
    { 0x9e77eb79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b7eb80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c07eb84, 99 },
    { 0x3807eb84, 98 },
    { 0, 0 },
    { 0xd37eb87, 61 },
    { 0, 0 },
    { 0x4ac7eb89, 61 },
    { 0x8217eb89, 98 },
    { 0xdd47eb89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4027eb90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed7eb96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc677eb9a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177eba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2607eba5, 33 },
    { 0x4de7eba5, 33 },
    { 0xf707eba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3767ebab, 66 },
    { 0xb627ebac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7ebb2, 90 },
    { 0x2bb7ebb2, 95 },
    { 0xf9f7ebb4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff7ebbb, 98 },
    { 0xbcb7ebbc, 85 },
    { 0x8887ebbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde97ebc1, 87 },
    { 0, 0 },
    { 0xebe7ebc3, 95 },
    { 0x9017ebc3, 95 },
    { 0xec47ebc5, 95 },
    { 0xb977ebc5, 93 },
    { 0xfb37ebc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d27ebcb, 88 },
    { 0xfc47ebcc, 23 },
    { 0, 0 },
    { 0xb827ebce, 34 },
    { 0x34a7ebcf, 61 },
    { 0x8227ebcf, 23 },
    { 0, 0 },
    { 0x657ebd2, 61 },
    { 0x1f67ebd3, 95 },
    { 0xeef7ebd4, 61 },
    { 0xc517ebd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x157ebd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f7ebdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e67ebdf, 76 },
    { 0xfb47ebe0, 23 },
    { 0x3c7ebe0, 90 },
    { 0x5ff7ebe0, 55 },
    { 0xefb7ebe3, 33 },
    { 0x5ea7ebe3, 100 },
    { 0xb987ebe5, 66 },
    { 0x8817ebe3, 61 },
    { 0xca47ebe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c7ebea, 55 },
    { 0, 0 },
    { 0xd867ebec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd7ebf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2627ebf4, 77 },
    { 0xd467ebf5, 98 },
    { 0x3747ebf6, 88 },
    { 0x3a97ebf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc7ebfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d7ebfd, 78 },
    { 0x5077ebfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae7ec09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa097ec0c, 85 },
    { 0xcac7ec0c, 34 },
    { 0x3577ec0e, 90 },
    { 0x4127ec0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b7ec14, 95 },
    { 0x5277ec14, 95 },
    { 0x9897ec16, 90 },
    { 0, 0 },
    { 0x4b57ec18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe617ec1d, 43 },
    { 0xe207ec1e, 61 },
    { 0xc337ec1f, 33 },
    { 0x9c87ec20, 90 },
    { 0, 0 },
    { 0x9507ec22, 2 },
    { 0x65a7ec23, 61 },
    { 0, 0 },
    { 0xdc37ec25, 61 },
    { 0, 0 },
    { 0x3967ec27, 34 },
    { 0, 0 },
    { 0x7387ec29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7ec2e, 87 },
    { 0xf6e7ec2e, 43 },
    { 0x1027ec30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc47ec33, 66 },
    { 0, 0 },
    { 0x3b07ec35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d7ec38, 93 },
    { 0x9d07ec38, 34 },
    { 0x8b27ec3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207ec40, 90 },
    { 0xb07ec41, 55 },
    { 0xe4f7ec42, 90 },
    { 0xe587ec43, 61 },
    { 0, 0 },
    { 0x5b67ec45, 23 },
    { 0x14b7ec46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227ec49, 93 },
    { 0x76f7ec4a, 90 },
    { 0xa207ec4a, 43 },
    { 0x4197ec4a, 90 },
    { 0x7f07ec4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee87ec53, 61 },
    { 0x9b7ec53, 88 },
    { 0xdee7ec55, 93 },
    { 0, 0 },
    { 0x7d27ec57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf37ec5a, 98 },
    { 0x3a77ec5b, 61 },
    { 0x8707ec5b, 33 },
    { 0x12e7ec5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc297ec62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257ec68, 33 },
    { 0x587ec68, 46 },
    { 0x4f37ec6a, 90 },
    { 0, 0 },
    { 0xad17ec6c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d7ec6f, 52 },
    { 0xc127ec6f, 43 },
    { 0x7a37ec71, 90 },
    { 0, 0 },
    { 0xa977ec73, 55 },
    { 0xb8c7ec74, 78 },
    { 0xe357ec75, 55 },
    { 0x2437ec76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4197ec7a, 40 },
    { 0x1427ec7a, 34 },
    { 0x5737ec7c, 33 },
    { 0, 0 },
    { 0x7d07ec7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d7ec83, 25 },
    { 0, 0 },
    { 0xa897ec85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b7ec8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d7ec8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c57ec94, 98 },
    { 0, 0 },
    { 0xb717ec96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4647ec9b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb7eca0, 76 },
    { 0, 0 },
    { 0xcbd7eca2, 94 },
    { 0x5127eca2, 98 },
    { 0xf597eca4, 78 },
    { 0xcc17eca5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2597ecb2, 34 },
    { 0xbd67ecb3, 46 },
    { 0, 0 },
    { 0xac07ecb5, 48 },
    { 0x4477ecb5, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7037ecbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd97ecbf, 55 },
    { 0xc9c7ecc0, 93 },
    { 0, 0 },
    { 0xc377ecc2, 33 },
    { 0xe797ecc2, 34 },
    { 0x6b7ecc4, 52 },
    { 0xbc97ecc4, 90 },
    { 0, 0 },
    { 0xa4c7ecc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c97ecca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf7eccd, 40 },
    { 0xdb17ecce, 76 },
    { 0x9bf7eccf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e7ecd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7ecdd, 61 },
    { 0xaec7ecde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5767ece6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c7eceb, 90 },
    { 0, 0 },
    { 0xd107eced, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a7ecf2, 46 },
    { 0xeb87ecf2, 46 },
    { 0xea77ecf4, 84 },
    { 0x8c67ecf5, 88 },
    { 0x3017ecf5, 84 },
    { 0x2fa7ecf7, 90 },
    { 0xd7f7ecf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec7ecfc, 23 },
    { 0, 0 },
    { 0xc797ecfe, 90 },
    { 0, 0 },
    { 0x4a57ed00, 95 },
    { 0, 0 },
    { 0x3667ed02, 33 },
    { 0x97d7ed02, 84 },
    { 0x8d77ed04, 93 },
    { 0x6517ed04, 55 },
    { 0x8297ed05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8537ed0a, 90 },
    { 0xd17ed0a, 98 },
    { 0, 0 },
    { 0xd227ed0d, 33 },
    { 0, 0 },
    { 0xbda7ed0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a7ed14, 88 },
    { 0, 0 },
    { 0x29e7ed16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf567ed1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d7ed23, 61 },
    { 0x3d37ed24, 95 },
    { 0x77a7ed23, 98 },
    { 0xf547ed26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2167ed2d, 64 },
    { 0xb4c7ed2e, 46 },
    { 0x4987ed2e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8927ed34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3427ed3a, 61 },
    { 0xfd27ed3a, 90 },
    { 0xead7ed3a, 23 },
    { 0x3537ed3d, 85 },
    { 0xfde7ed3c, 90 },
    { 0xede7ed3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d7ed4a, 90 },
    { 0x5bd7ed4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb7ed51, 90 },
    { 0xee47ed51, 55 },
    { 0xbef7ed53, 67 },
    { 0xac57ed54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc527ed5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e27ed60, 61 },
    { 0, 0 },
    { 0x9b37ed62, 95 },
    { 0xaf07ed63, 95 },
    { 0x30e7ed64, 23 },
    { 0x8757ed64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b7ed6d, 99 },
    { 0, 0 },
    { 0x2e87ed6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a47ed74, 90 },
    { 0, 0 },
    { 0x7077ed76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b77ed80, 76 },
    { 0x3b7ed80, 84 },
    { 0xe457ed80, 99 },
    { 0, 0 },
    { 0x9607ed84, 23 },
    { 0xfff7ed84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0b7ed8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff37ed8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd7ed96, 17 },
    { 0x5897ed97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf17ed9d, 23 },
    { 0, 0 },
    { 0x3b17ed9f, 61 },
    { 0xcc67eda0, 90 },
    { 0x3ee7eda0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea07eda7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae7edae, 76 },
    { 0xb8a7edaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa827edb5, 90 },
    { 0, 0 },
    { 0x4b97edb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a7edc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc7edc9, 33 },
    { 0xa887edca, 43 },
    { 0x5897edcb, 95 },
    { 0x5307edca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c7edd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc327edd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa007eddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e7eddf, 52 },
    { 0x87c7ede0, 90 },
    { 0xb8e7ede1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4327ede6, 93 },
    { 0x357ede6, 46 },
    { 0x6397ede8, 33 },
    { 0x2807ede6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb597edec, 98 },
    { 0, 0 },
    { 0x4817edee, 99 },
    { 0xe617edef, 61 },
    { 0xbaf7edf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67edf3, 33 },
    { 0xcf87edf4, 33 },
    { 0, 0 },
    { 0x5d27edf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a7edf9, 84 },
    { 0x7ea7edfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b7edfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a7ee02, 33 },
    { 0, 0 },
    { 0xc807ee04, 52 },
    { 0x55d7ee05, 64 },
    { 0xc917ee06, 76 },
    { 0, 0 },
    { 0xb07ee08, 90 },
    { 0x89d7ee09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f77ee0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557ee0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c7ee14, 95 },
    { 0xee57ee14, 61 },
    { 0xc7d7ee14, 90 },
    { 0x1087ee15, 61 },
    { 0x71b7ee16, 90 },
    { 0xed97ee19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7ee1c, 61 },
    { 0xeea7ee1d, 100 },
    { 0x6c07ee1d, 100 },
    { 0x5407ee1d, 34 },
    { 0x4577ee20, 95 },
    { 0, 0 },
    { 0xf027ee22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e27ee27, 10 },
    { 0x6a07ee27, 100 },
    { 0, 0 },
    { 0xc7f7ee2a, 78 },
    { 0, 0 },
    { 0x577ee2c, 34 },
    { 0x1b77ee2d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6757ee36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b27ee3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf7ee45, 66 },
    { 0x64b7ee45, 94 },
    { 0, 0 },
    { 0x8177ee48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe017ee4c, 66 },
    { 0x4ff7ee4d, 61 },
    { 0x1eb7ee4d, 78 },
    { 0x73e7ee4e, 93 },
    { 0x3ca7ee4e, 33 },
    { 0x5ee7ee4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e7ee57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f37ee5c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde07ee61, 68 },
    { 0xc7c7ee62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e57ee66, 93 },
    { 0x46f7ee67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe27ee6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc7ee6d, 33 },
    { 0xae87ee6e, 33 },
    { 0x67d7ee6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc737ee72, 34 },
    { 0xdbc7ee73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc7ee76, 88 },
    { 0xa327ee76, 90 },
    { 0, 0 },
    { 0x1f17ee79, 99 },
    { 0, 0 },
    { 0x43c7ee7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d7ee80, 53 },
    { 0x9067ee80, 17 },
    { 0, 0 },
    { 0x5897ee83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb157ee87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a17ee8b, 68 },
    { 0, 0 },
    { 0x8a77ee8d, 61 },
    { 0xa2c7ee8e, 90 },
    { 0x8037ee8f, 98 },
    { 0x9317ee8f, 43 },
    { 0, 0 },
    { 0x9ad7ee92, 98 },
    { 0xbec7ee93, 95 },
    { 0x5307ee94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a7ee97, 48 },
    { 0x5867ee97, 43 },
    { 0, 0 },
    { 0x337ee9a, 90 },
    { 0, 0 },
    { 0x7347ee9c, 93 },
    { 0xf447ee9c, 98 },
    { 0x3b37ee9e, 98 },
    { 0x4167ee9f, 43 },
    { 0, 0 },
    { 0xdc17eea1, 61 },
    { 0xa507eea1, 77 },
    { 0x3767eea2, 95 },
    { 0x4357eea4, 23 },
    { 0xc467eea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f7eea8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d7eeb0, 95 },
    { 0x86f7eeb1, 33 },
    { 0xce67eeb2, 66 },
    { 0xbf67eeb3, 98 },
    { 0x6a67eeb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7287eeb7, 56 },
    { 0xa1b7eeb8, 78 },
    { 0, 0 },
    { 0x9c27eeba, 77 },
    { 0x8087eebb, 61 },
    { 0, 0 },
    { 0x2c67eebd, 98 },
    { 0x32f7eebd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe57eec3, 46 },
    { 0x8fa7eec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc27eec7, 23 },
    { 0x49c7eec8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3837eecf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e67eed6, 34 },
    { 0x8687eed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4087eede, 90 },
    { 0, 0 },
    { 0x4647eee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc307eee8, 61 },
    { 0x4367eee8, 23 },
    { 0x12f7eeea, 4 },
    { 0xe097eeeb, 23 },
    { 0xa687eee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe887eef0, 61 },
    { 0, 0 },
    { 0xe1d7eef2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee7eef5, 98 },
    { 0x63a7eef6, 52 },
    { 0xb697eef7, 90 },
    { 0x6bb7eef8, 90 },
    { 0x5c27eef9, 95 },
    { 0x7137eef6, 61 },
    { 0, 0 },
    { 0xa267eefc, 95 },
    { 0, 0 },
    { 0x9787eefe, 46 },
    { 0, 0 },
    { 0xfcc7ef00, 93 },
    { 0, 0 },
    { 0x8ac7ef02, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd07ef06, 95 },
    { 0xd3f7ef06, 61 },
    { 0xd9c7ef08, 90 },
    { 0x9b87ef07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1217ef0c, 33 },
    { 0, 0 },
    { 0xba67ef0e, 78 },
    { 0, 0 },
    { 0x8127ef10, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc7ef21, 34 },
    { 0xc87ef22, 61 },
    { 0x4f27ef23, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f7ef27, 36 },
    { 0xe787ef28, 90 },
    { 0xf827ef29, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67ef39, 34 },
    { 0x59c7ef3a, 100 },
    { 0xc207ef39, 61 },
    { 0, 0 },
    { 0xb2f7ef3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de7ef41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae57ef44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb7ef49, 34 },
    { 0, 0 },
    { 0x1687ef4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c87ef50, 99 },
    { 0, 0 },
    { 0x5d37ef52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda7ef56, 36 },
    { 0xaa27ef57, 98 },
    { 0x1f07ef58, 41 },
    { 0x8067ef59, 61 },
    { 0x5bc7ef57, 90 },
    { 0x2337ef5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa7ef60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8817ef65, 98 },
    { 0x13d7ef66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4547ef6a, 90 },
    { 0xb3d7ef6a, 90 },
    { 0, 0 },
    { 0x7b77ef6d, 46 },
    { 0x2c07ef6d, 23 },
    { 0, 0 },
    { 0x8b07ef70, 61 },
    { 0, 0 },
    { 0xe657ef72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8127ef76, 34 },
    { 0x3a67ef77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc87ef7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6447ef82, 61 },
    { 0x5c37ef82, 98 },
    { 0, 0 },
    { 0x4e37ef85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1407ef89, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf797ef8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c7ef90, 98 },
    { 0, 0 },
    { 0xd637ef92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c7ef99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf197efab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe987efaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4397efb5, 33 },
    { 0xad67efb6, 78 },
    { 0xce67efb6, 43 },
    { 0xd4f7efb6, 76 },
    { 0xe097efb7, 90 },
    { 0x7cc7efb6, 55 },
    { 0xf357efb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb727efbe, 76 },
    { 0xe667efbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b7efc4, 61 },
    { 0xd287efc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b7efc9, 84 },
    { 0x1dd7efca, 80 },
    { 0xbc17efcb, 90 },
    { 0xeda7efcb, 65 },
    { 0x2977efcd, 52 },
    { 0x4327efce, 95 },
    { 0xd837efcf, 69 },
    { 0x96f7efce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a67efd6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc057efdb, 61 },
    { 0, 0 },
    { 0xa247efdd, 98 },
    { 0x99c7efde, 90 },
    { 0xb327efdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa287efe3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3767efe7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d7efed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a7eff0, 95 },
    { 0, 0 },
    { 0xc847eff2, 77 },
    { 0, 0 },
    { 0x3007eff4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7eff9, 90 },
    { 0x1467eff9, 33 },
    { 0x2a87effb, 52 },
    { 0, 0 },
    { 0x5d87effd, 91 },
    { 0x2e77effe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f7f003, 95 },
    { 0xe5f7f004, 95 },
    { 0x9f87f005, 84 },
    { 0x5ee7f004, 99 },
    { 0x16f7f005, 98 },
    { 0xf17f008, 17 },
    { 0x69a7f009, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07f010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c97f015, 78 },
    { 0xde77f015, 76 },
    { 0xff7f015, 98 },
    { 0, 0 },
    { 0x15e7f019, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c7f01c, 33 },
    { 0, 0 },
    { 0x65f7f01e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c77f027, 23 },
    { 0xfdd7f028, 52 },
    { 0xa2a7f029, 68 },
    { 0xfdb7f028, 38 },
    { 0x6a67f02b, 98 },
    { 0xc8f7f028, 33 },
    { 0xd127f02d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a7f032, 61 },
    { 0, 0 },
    { 0x1687f034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8317f037, 98 },
    { 0xd97f038, 90 },
    { 0xf77f039, 64 },
    { 0x8d17f03a, 90 },
    { 0, 0 },
    { 0xd047f03c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4147f03f, 90 },
    { 0xad77f03f, 34 },
    { 0x46f7f03f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2427f046, 66 },
    { 0xca17f046, 23 },
    { 0xfd97f047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d7f04b, 93 },
    { 0x5cd7f04c, 33 },
    { 0, 0 },
    { 0x5957f04e, 90 },
    { 0x3617f04e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9747f053, 84 },
    { 0x4947f054, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5897f061, 33 },
    { 0xde77f062, 61 },
    { 0x24c7f063, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc07f067, 61 },
    { 0x53f7f068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb7f06e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a47f071, 61 },
    { 0, 0 },
    { 0xd2e7f073, 34 },
    { 0, 0 },
    { 0x4b7f075, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2307f079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e7f07f, 98 },
    { 0, 0 },
    { 0x8eb7f081, 36 },
    { 0, 0 },
    { 0xa097f083, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd757f086, 23 },
    { 0x2fd7f086, 90 },
    { 0xa977f087, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc37f094, 33 },
    { 0x6747f095, 91 },
    { 0, 0 },
    { 0x3e87f097, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd487f09c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4017f09f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf7f0a6, 53 },
    { 0, 0 },
    { 0x4d87f0a8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b37f0ae, 3 },
    { 0x9c97f0af, 61 },
    { 0x11c7f0b0, 88 },
    { 0x13e7f0b1, 23 },
    { 0xdd87f0af, 67 },
    { 0x3717f0b1, 33 },
    { 0x7d57f0b0, 65 },
    { 0xe7f0b5, 90 },
    { 0x4467f0b4, 91 },
    { 0xede7f0b7, 30 },
    { 0, 0 },
    { 0xe8f7f0b9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd697f0bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd907f0c3, 90 },
    { 0xabc7f0c4, 100 },
    { 0xcd57f0c3, 98 },
    { 0, 0 },
    { 0x5487f0c7, 99 },
    { 0x8907f0c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2767f0cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7387f0cf, 55 },
    { 0x5a17f0cf, 81 },
    { 0xf7e7f0d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7757f0d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d7f0dd, 61 },
    { 0x7857f0de, 88 },
    { 0xc937f0de, 38 },
    { 0xc3a7f0dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b7f0e3, 90 },
    { 0x9bd7f0e4, 61 },
    { 0xb997f0e5, 61 },
    { 0x8ba7f0e6, 87 },
    { 0xa8b7f0e7, 76 },
    { 0xeec7f0e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce77f0eb, 95 },
    { 0x4fd7f0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d7f0f0, 23 },
    { 0x71a7f0f1, 33 },
    { 0x4957f0f1, 85 },
    { 0, 0 },
    { 0xf0d7f0f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9847f0f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c7f0fb, 55 },
    { 0x8f7f0fc, 17 },
    { 0, 0 },
    { 0x987f0fe, 46 },
    { 0, 0 },
    { 0xa347f100, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b7f103, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b7f10b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6417f110, 95 },
    { 0, 0 },
    { 0xe0c7f112, 78 },
    { 0xc767f113, 23 },
    { 0, 0 },
    { 0xb747f115, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf327f118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c67f11c, 100 },
    { 0x3b77f11c, 67 },
    { 0x87b7f11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797f121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e7f125, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f12a, 98 },
    { 0x62c7f12b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4507f12e, 68 },
    { 0x1467f12f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e7f134, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f57f13b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1277f140, 61 },
    { 0, 0 },
    { 0xbb57f142, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e37f148, 61 },
    { 0x83a7f149, 95 },
    { 0xd3f7f149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2557f152, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb7f157, 61 },
    { 0x8fa7f158, 17 },
    { 0xb947f159, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8987f15c, 23 },
    { 0x7f87f15d, 61 },
    { 0xea57f15e, 93 },
    { 0x3137f15d, 61 },
    { 0x56a7f15c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87f165, 76 },
    { 0xef7f165, 88 },
    { 0, 0 },
    { 0x7207f168, 93 },
    { 0x7077f169, 61 },
    { 0xbeb7f16a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7f16d, 23 },
    { 0, 0 },
    { 0x1e67f16f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47f173, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9767f178, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d77f17b, 99 },
    { 0x77b7f17b, 23 },
    { 0xd117f17c, 78 },
    { 0xdaf7f17e, 93 },
    { 0xc5e7f17e, 33 },
    { 0xddc7f180, 95 },
    { 0, 0 },
    { 0xcbf7f182, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3177f185, 90 },
    { 0x2fe7f186, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb127f18d, 98 },
    { 0x2277f18e, 90 },
    { 0x2487f18f, 34 },
    { 0xbd97f190, 67 },
    { 0xacc7f18d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb87f19a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa497f19d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9097f1a8, 61 },
    { 0x1a27f1a9, 67 },
    { 0x5a27f1a9, 55 },
    { 0x9eb7f1aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b07f1af, 78 },
    { 0x8747f1b0, 90 },
    { 0, 0 },
    { 0x1367f1b2, 33 },
    { 0xb227f1b3, 33 },
    { 0, 0 },
    { 0x9257f1b5, 90 },
    { 0x8707f1b5, 98 },
    { 0xcd47f1b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6967f1c0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227f1c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f7f1d1, 76 },
    { 0x7667f1d2, 85 },
    { 0x46d7f1d2, 33 },
    { 0x8767f1d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf767f1d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3667f1da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e7f1dd, 61 },
    { 0, 0 },
    { 0xaf17f1df, 58 },
    { 0xe8a7f1e0, 48 },
    { 0x7547f1e1, 5 },
    { 0xed77f1e1, 90 },
    { 0x8b07f1e3, 90 },
    { 0x7307f1e4, 90 },
    { 0x24d7f1e2, 23 },
    { 0xec57f1e6, 87 },
    { 0x4df7f1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba27f1ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357f1ef, 96 },
    { 0xfca7f1f0, 98 },
    { 0, 0 },
    { 0x5f67f1f2, 66 },
    { 0xad57f1f3, 95 },
    { 0, 0 },
    { 0x4ed7f1f5, 90 },
    { 0x1de7f1f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6377f1fb, 95 },
    { 0, 0 },
    { 0x2297f1fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb27f200, 81 },
    { 0x6ed7f200, 34 },
    { 0xa357f202, 23 },
    { 0xc337f203, 90 },
    { 0x7d27f202, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f7f207, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a7f20a, 33 },
    { 0x40b7f20a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817f20f, 23 },
    { 0, 0 },
    { 0x3c17f211, 75 },
    { 0x3b07f211, 93 },
    { 0x5067f213, 33 },
    { 0, 0 },
    { 0xf87f215, 23 },
    { 0xaf7f216, 66 },
    { 0xa6a7f216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb7f21c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a7f21f, 65 },
    { 0x1d07f220, 76 },
    { 0x1a87f220, 52 },
    { 0, 0 },
    { 0x78f7f223, 55 },
    { 0xe867f224, 90 },
    { 0x7987f225, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c7f22a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb287f22f, 75 },
    { 0xe847f230, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b7f233, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff97f237, 33 },
    { 0, 0 },
    { 0xe667f239, 33 },
    { 0, 0 },
    { 0x1697f23b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca7f241, 98 },
    { 0, 0 },
    { 0xfb37f243, 23 },
    { 0, 0 },
    { 0x7237f245, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9227f248, 85 },
    { 0x2f57f249, 75 },
    { 0, 0 },
    { 0xf447f24b, 23 },
    { 0x3827f24c, 93 },
    { 0x6267f24c, 76 },
    { 0x7a17f24e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a97f251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd237f25e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f17f265, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a17f269, 55 },
    { 0xb287f26a, 75 },
    { 0xa247f26b, 78 },
    { 0x77b7f26c, 95 },
    { 0xb267f26d, 91 },
    { 0, 0 },
    { 0xe2f7f26f, 55 },
    { 0x5ba7f26f, 66 },
    { 0x7cc7f271, 90 },
    { 0x9f7f26f, 61 },
    { 0x86d7f270, 90 },
    { 0x2a07f273, 98 },
    { 0, 0 },
    { 0x14f7f276, 23 },
    { 0xfc67f276, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd7f27b, 98 },
    { 0xb867f27c, 55 },
    { 0x6f77f27b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a7f282, 33 },
    { 0x1b67f283, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f28a, 61 },
    { 0x2d27f28b, 67 },
    { 0, 0 },
    { 0x8917f28d, 33 },
    { 0x27f7f28e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3417f293, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3877f296, 76 },
    { 0xf597f297, 90 },
    { 0, 0 },
    { 0xcf27f299, 76 },
    { 0, 0 },
    { 0x4297f29b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa67f2a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed7f2a6, 43 },
    { 0x1e7f2a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f57f2ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc747f2b2, 83 },
    { 0, 0 },
    { 0xdc07f2b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e7f2ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d7f2bf, 90 },
    { 0, 0 },
    { 0x36a7f2c1, 87 },
    { 0xcf7f2c1, 10 },
    { 0xca47f2c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a07f2c6, 55 },
    { 0x75c7f2c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca37f2cd, 93 },
    { 0x3d7f2ce, 61 },
    { 0, 0 },
    { 0x3647f2d0, 76 },
    { 0x25e7f2d1, 61 },
    { 0x5db7f2d2, 95 },
    { 0x5387f2d3, 66 },
    { 0xc9e7f2d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b7f2db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b7f2df, 90 },
    { 0x8417f2df, 61 },
    { 0x517f2e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5917f2e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d97f2e8, 95 },
    { 0x16d7f2e9, 61 },
    { 0x3c77f2ea, 90 },
    { 0, 0 },
    { 0xcfd7f2ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc807f2f0, 93 },
    { 0, 0 },
    { 0x5d77f2f2, 90 },
    { 0xaa37f2f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c7f2f6, 88 },
    { 0xab7f2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc27f2fb, 90 },
    { 0xe467f2fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d17f2ff, 23 },
    { 0xc977f2ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f308, 65 },
    { 0xb407f308, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef7f30e, 90 },
    { 0x8467f30f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2877f312, 90 },
    { 0x1317f313, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9657f318, 46 },
    { 0xfba7f319, 90 },
    { 0xf047f319, 90 },
    { 0, 0 },
    { 0x7557f31c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc7f322, 23 },
    { 0xd717f322, 96 },
    { 0x6877f324, 84 },
    { 0, 0 },
    { 0x2a07f326, 95 },
    { 0xa4d7f326, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797f330, 34 },
    { 0xc837f330, 72 },
    { 0x397f330, 67 },
    { 0, 0 },
    { 0x19a7f334, 33 },
    { 0x8e77f335, 33 },
    { 0xf5d7f334, 55 },
    { 0xa487f337, 33 },
    { 0x8867f338, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2957f33b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad7f33e, 33 },
    { 0x4207f33e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1067f343, 66 },
    { 0x4c57f344, 90 },
    { 0x8d07f344, 61 },
    { 0x3af7f346, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5887f34a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf87f34f, 95 },
    { 0x4fd7f350, 99 },
    { 0x9817f350, 33 },
    { 0xf597f352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb7f358, 43 },
    { 0x58f7f358, 34 },
    { 0x4737f35a, 33 },
    { 0xdd57f358, 90 },
    { 0, 0 },
    { 0x23c7f35d, 99 },
    { 0xc477f35d, 93 },
    { 0xfdf7f35f, 33 },
    { 0x6e37f35f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2647f368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8497f36b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b7f36f, 91 },
    { 0, 0 },
    { 0x9327f371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8047f379, 55 },
    { 0, 0 },
    { 0x2d37f37b, 98 },
    { 0x88a7f37c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7f37f, 98 },
    { 0x35f7f380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb7f38d, 46 },
    { 0xb57f38e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e7f393, 100 },
    { 0x77c7f393, 98 },
    { 0xabc7f394, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77f39a, 34 },
    { 0xf37f39b, 55 },
    { 0x45a7f39a, 38 },
    { 0x35f7f39d, 87 },
    { 0x9487f39e, 23 },
    { 0xd6b7f39e, 90 },
    { 0xb5e7f39d, 55 },
    { 0x527f39b, 98 },
    { 0xa327f39c, 35 },
    { 0, 0 },
    { 0xded7f3a4, 90 },
    { 0x6cf7f3a5, 98 },
    { 0xd5f7f3a5, 95 },
    { 0x18e7f3a7, 43 },
    { 0, 0 },
    { 0x4f17f3a9, 33 },
    { 0, 0 },
    { 0xd507f3ab, 34 },
    { 0xd9f7f3ac, 78 },
    { 0xc277f3ad, 76 },
    { 0xe7d7f3ac, 98 },
    { 0x6177f3af, 95 },
    { 0x6457f3b0, 93 },
    { 0x8ef7f3b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa17f3b4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e7f3b9, 42 },
    { 0, 0 },
    { 0x8257f3bb, 67 },
    { 0x60a7f3bb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e7f3c0, 88 },
    { 0, 0 },
    { 0x40f7f3c2, 98 },
    { 0xc937f3c2, 99 },
    { 0x6f7f3c4, 78 },
    { 0x5547f3c4, 52 },
    { 0xe7f7f3c6, 34 },
    { 0x8157f3c7, 90 },
    { 0xf767f3c8, 46 },
    { 0x6757f3c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd157f3cc, 88 },
    { 0xbbd7f3cc, 76 },
    { 0xcb07f3ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae37f3d1, 52 },
    { 0xd3e7f3d1, 95 },
    { 0, 0 },
    { 0x9c67f3d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca7f3d9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb137f3dd, 95 },
    { 0, 0 },
    { 0xe6c7f3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67f3e5, 33 },
    { 0xf727f3e5, 95 },
    { 0, 0 },
    { 0x4a7f3e8, 55 },
    { 0x8b7f3e8, 100 },
    { 0x11e7f3ea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc497f3ed, 90 },
    { 0xbe97f3ee, 95 },
    { 0xa6b7f3ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd417f3f6, 90 },
    { 0, 0 },
    { 0x5c77f3f8, 76 },
    { 0, 0 },
    { 0xe197f3fa, 95 },
    { 0x4e47f3fb, 66 },
    { 0x667f3fc, 75 },
    { 0x267f3fa, 43 },
    { 0x33e7f3fb, 76 },
    { 0, 0 },
    { 0xe247f400, 90 },
    { 0, 0 },
    { 0x9757f402, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b7f405, 76 },
    { 0, 0 },
    { 0x5ca7f407, 95 },
    { 0x5107f408, 61 },
    { 0, 0 },
    { 0xf047f40a, 84 },
    { 0, 0 },
    { 0xc7a7f40c, 33 },
    { 0, 0 },
    { 0x93e7f40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf927f411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd7f415, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea7f41b, 90 },
    { 0, 0 },
    { 0x3db7f41d, 90 },
    { 0xef67f41e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d7f422, 98 },
    { 0xdac7f423, 87 },
    { 0x39d7f424, 33 },
    { 0xd3f7f425, 33 },
    { 0xea37f423, 61 },
    { 0x6877f427, 43 },
    { 0xc617f426, 98 },
    { 0xdb87f427, 87 },
    { 0xe7b7f42a, 90 },
    { 0x3347f42a, 52 },
    { 0x7847f426, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd57f430, 23 },
    { 0xc267f431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe617f435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc577f43c, 55 },
    { 0, 0 },
    { 0xadd7f43e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8207f445, 33 },
    { 0xa367f445, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1737f449, 46 },
    { 0, 0 },
    { 0xb17f44b, 61 },
    { 0, 0 },
    { 0x85f7f44d, 23 },
    { 0x2df7f44e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b67f457, 66 },
    { 0x9957f457, 95 },
    { 0x2b27f459, 61 },
    { 0x2a67f459, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda7f460, 87 },
    { 0xd857f461, 99 },
    { 0x1157f461, 99 },
    { 0, 0 },
    { 0xd607f464, 81 },
    { 0x52c7f465, 90 },
    { 0xbde7f465, 94 },
    { 0, 0 },
    { 0xb197f468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e7f46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f7f475, 33 },
    { 0x9307f476, 34 },
    { 0x7067f477, 95 },
    { 0xd847f477, 66 },
    { 0x8aa7f479, 34 },
    { 0xf417f47a, 95 },
    { 0x1c7f478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e67f47e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd397f482, 73 },
    { 0, 0 },
    { 0x51a7f484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5367f48c, 33 },
    { 0x2217f48d, 95 },
    { 0x13b7f48e, 23 },
    { 0, 0 },
    { 0x2677f490, 23 },
    { 0x3407f491, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5167f495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb237f4a1, 95 },
    { 0x29f7f4a2, 90 },
    { 0x7867f4a1, 98 },
    { 0x6b7f4a3, 23 },
    { 0x2447f4a5, 90 },
    { 0xc127f4a6, 33 },
    { 0x8df7f4a2, 76 },
    { 0x6597f4a5, 61 },
    { 0xf7d7f4a7, 93 },
    { 0, 0 },
    { 0x2237f4ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7f4af, 95 },
    { 0x3ec7f4af, 61 },
    { 0x2c97f4b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e7f4b8, 76 },
    { 0, 0 },
    { 0x6287f4ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd7f4bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf407f4c0, 76 },
    { 0, 0 },
    { 0xce97f4c2, 90 },
    { 0xf67f4c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea7f4c9, 78 },
    { 0xe767f4c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1e7f4d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1647f4d7, 78 },
    { 0, 0 },
    { 0x21a7f4d9, 43 },
    { 0xb7f7f4d9, 80 },
    { 0x8417f4db, 76 },
    { 0xe117f4dc, 23 },
    { 0x39a7f4dc, 61 },
    { 0x9eb7f4db, 90 },
    { 0x9767f4df, 23 },
    { 0x2727f4de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa547f4e6, 33 },
    { 0x9cf7f4e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce7f4f3, 46 },
    { 0, 0 },
    { 0xfc57f4f5, 90 },
    { 0x4f57f4f6, 55 },
    { 0xa0d7f4f7, 23 },
    { 0xbc97f4f7, 95 },
    { 0xec77f4f5, 23 },
    { 0xdc87f4fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f7f4fd, 46 },
    { 0x2597f4fe, 81 },
    { 0, 0 },
    { 0x3cf7f500, 33 },
    { 0x7277f501, 90 },
    { 0x8017f500, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe77f506, 76 },
    { 0x4e97f507, 61 },
    { 0x5ff7f508, 90 },
    { 0xf087f509, 55 },
    { 0xbc17f50a, 66 },
    { 0xe8a7f50b, 95 },
    { 0x6767f508, 55 },
    { 0, 0 },
    { 0x49e7f50e, 76 },
    { 0x13c7f50f, 65 },
    { 0xb6b7f50e, 90 },
    { 0xf297f50e, 23 },
    { 0, 0 },
    { 0x3607f513, 95 },
    { 0x90c7f514, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb507f51b, 33 },
    { 0x1437f51c, 33 },
    { 0x5357f51d, 98 },
    { 0, 0 },
    { 0xf687f51f, 43 },
    { 0, 0 },
    { 0xdb07f521, 55 },
    { 0xd637f521, 76 },
    { 0x60e7f521, 84 },
    { 0xb117f524, 76 },
    { 0x23a7f523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc567f529, 33 },
    { 0, 0 },
    { 0x4367f52b, 34 },
    { 0x4a47f52c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9397f530, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb177f533, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d7f536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef77f53c, 33 },
    { 0, 0 },
    { 0x8f87f53e, 23 },
    { 0x9347f53e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc537f546, 55 },
    { 0, 0 },
    { 0xa357f548, 76 },
    { 0xb907f548, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa97f54e, 55 },
    { 0x3737f54f, 84 },
    { 0xe787f54f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb7f555, 92 },
    { 0x8c97f556, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb647f55f, 34 },
    { 0xdeb7f560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e27f563, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b7f568, 55 },
    { 0, 0 },
    { 0xcea7f56a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5107f56d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd7f570, 90 },
    { 0xe627f571, 66 },
    { 0xac7f572, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc547f576, 33 },
    { 0, 0 },
    { 0xc647f578, 91 },
    { 0, 0 },
    { 0xe487f57a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a87f586, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa7f589, 98 },
    { 0xa147f58a, 33 },
    { 0x6dc7f58b, 95 },
    { 0xb9f7f58c, 90 },
    { 0x70b7f58d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec87f591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c47f595, 55 },
    { 0xb9f7f596, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea7f5a2, 76 },
    { 0x8337f5a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f5a7, 92 },
    { 0x51b7f5a8, 46 },
    { 0xf667f5a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf57f5ae, 91 },
    { 0xca67f5ae, 33 },
    { 0x3ba7f5b0, 90 },
    { 0x3a47f5b1, 23 },
    { 0x9697f5b2, 55 },
    { 0, 0 },
    { 0xf077f5b4, 67 },
    { 0xf977f5b5, 98 },
    { 0x9c57f5b6, 90 },
    { 0x9ca7f5b6, 85 },
    { 0x3357f5b7, 90 },
    { 0xddc7f5b9, 33 },
    { 0x28e7f5ba, 34 },
    { 0x73e7f5b5, 90 },
    { 0xcc67f5b7, 90 },
    { 0x8e87f5bc, 33 },
    { 0, 0 },
    { 0x2b07f5bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd557f5c4, 90 },
    { 0xae57f5c5, 55 },
    { 0x4e97f5c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8437f5d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c87f5d3, 66 },
    { 0xb127f5d4, 23 },
    { 0xf647f5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f5e0, 23 },
    { 0xd477f5e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa057f5e4, 61 },
    { 0, 0 },
    { 0x3a07f5e6, 23 },
    { 0x5967f5e6, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf7f5eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d7f5f7, 61 },
    { 0xb4c7f5f7, 46 },
    { 0x9717f5f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc7f5ff, 90 },
    { 0xb5f7f600, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d47f603, 66 },
    { 0x5807f604, 61 },
    { 0x59e7f605, 43 },
    { 0x5b7f606, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7f60e, 46 },
    { 0xe4e7f60f, 23 },
    { 0x3737f60e, 55 },
    { 0x5bb7f611, 98 },
    { 0x21d7f60e, 96 },
    { 0xeea7f613, 34 },
    { 0x7c97f60e, 80 },
    { 0xf1e7f615, 90 },
    { 0x80c7f615, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e67f61a, 70 },
    { 0x3377f61a, 23 },
    { 0x86e7f61c, 78 },
    { 0x647f61d, 98 },
    { 0x7e07f61c, 23 },
    { 0xb3b7f61e, 52 },
    { 0x1237f620, 90 },
    { 0x2457f621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd7f625, 55 },
    { 0, 0 },
    { 0x7e97f627, 93 },
    { 0x22d7f628, 98 },
    { 0xc787f629, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7f62e, 98 },
    { 0x5ff7f62e, 100 },
    { 0x50e7f630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f47f633, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f17f637, 61 },
    { 0x54c7f638, 90 },
    { 0x6087f637, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba47f63c, 43 },
    { 0xb387f63c, 90 },
    { 0x1b67f63d, 75 },
    { 0xa687f63f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc757f642, 77 },
    { 0x6a07f642, 78 },
    { 0x8f37f644, 33 },
    { 0x3167f645, 61 },
    { 0x2037f644, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce7f64a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd77f64f, 61 },
    { 0x1847f650, 23 },
    { 0x1e17f64f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb97f655, 61 },
    { 0x4bd7f656, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3187f659, 81 },
    { 0, 0 },
    { 0x8d67f65b, 90 },
    { 0x10a7f65c, 90 },
    { 0x78e7f65d, 46 },
    { 0x66c7f65e, 98 },
    { 0x2ab7f65d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f97f664, 61 },
    { 0, 0 },
    { 0x56a7f666, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae7f669, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc397f66f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf7f673, 61 },
    { 0, 0 },
    { 0x6d97f675, 61 },
    { 0x987f676, 93 },
    { 0xd3f7f677, 33 },
    { 0x8f87f677, 98 },
    { 0x727f679, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe87f67f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a7f68e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc7f692, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87f695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b7f69f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf77f6a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1517f6a9, 95 },
    { 0x6b77f6aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e7f6ad, 75 },
    { 0, 0 },
    { 0xeb37f6af, 91 },
    { 0x7207f6af, 61 },
    { 0x1a7f6b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7f6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea7f6bc, 23 },
    { 0x8b87f6bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f7f6c0, 87 },
    { 0x9f7f6c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b7f6c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf847f6c7, 90 },
    { 0x4ca7f6c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc27f6cb, 90 },
    { 0xbc37f6cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc847f6d5, 33 },
    { 0x9637f6d6, 52 },
    { 0x4617f6d6, 95 },
    { 0x5e47f6d7, 76 },
    { 0x72b7f6d8, 23 },
    { 0x2ea7f6da, 93 },
    { 0, 0 },
    { 0xd877f6dc, 95 },
    { 0x7657f6dd, 46 },
    { 0xb867f6de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d7f6e1, 76 },
    { 0x9427f6e1, 61 },
    { 0, 0 },
    { 0x4e37f6e4, 76 },
    { 0x95a7f6e5, 95 },
    { 0x3b87f6e5, 33 },
    { 0xd9a7f6e5, 55 },
    { 0, 0 },
    { 0x4f57f6e9, 100 },
    { 0xeb07f6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d7f6ed, 90 },
    { 0x44c7f6ee, 55 },
    { 0xc2d7f6ef, 55 },
    { 0xc997f6f0, 33 },
    { 0xc0d7f6f0, 33 },
    { 0x6a57f6f2, 95 },
    { 0, 0 },
    { 0x6067f6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c7f6ff, 61 },
    { 0xeda7f6ff, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f7f704, 23 },
    { 0, 0 },
    { 0xeb67f706, 33 },
    { 0x37a7f707, 61 },
    { 0, 0 },
    { 0x8a37f709, 100 },
    { 0, 0 },
    { 0x547f70b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa687f712, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa337f715, 61 },
    { 0, 0 },
    { 0x5f57f717, 61 },
    { 0x5e07f718, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb197f71d, 61 },
    { 0, 0 },
    { 0x2797f71f, 95 },
    { 0, 0 },
    { 0xed67f721, 33 },
    { 0x3437f722, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9927f725, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e7f728, 66 },
    { 0xd777f728, 68 },
    { 0, 0 },
    { 0x31c7f72b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d7f733, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc227f73a, 100 },
    { 0xf957f73a, 99 },
    { 0, 0 },
    { 0xd417f73d, 90 },
    { 0xff87f73e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f7f741, 75 },
    { 0, 0 },
    { 0x95c7f743, 87 },
    { 0xd8c7f743, 76 },
    { 0x34d7f745, 95 },
    { 0, 0 },
    { 0x2ae7f747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf337f74c, 95 },
    { 0, 0 },
    { 0xdaa7f74e, 98 },
    { 0, 0 },
    { 0x4237f750, 90 },
    { 0xd0d7f750, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1257f758, 76 },
    { 0x4bd7f758, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb67f762, 55 },
    { 0xb4e7f762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb7f76a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab7f76e, 85 },
    { 0, 0 },
    { 0xa4a7f770, 98 },
    { 0, 0 },
    { 0x99c7f772, 65 },
    { 0x53f7f773, 90 },
    { 0x5d37f774, 87 },
    { 0, 0 },
    { 0x1bd7f776, 61 },
    { 0x15e7f777, 94 },
    { 0x1b27f778, 76 },
    { 0x16f7f779, 76 },
    { 0x6f17f779, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e17f784, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe997f787, 33 },
    { 0xe997f788, 91 },
    { 0, 0 },
    { 0xda47f78a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9817f78f, 91 },
    { 0x7c77f790, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5077f794, 90 },
    { 0x7f17f794, 97 },
    { 0x1787f794, 98 },
    { 0xc697f797, 61 },
    { 0x5207f798, 95 },
    { 0x88d7f797, 33 },
    { 0xe4f7f799, 66 },
    { 0xb707f79b, 100 },
    { 0x73a7f799, 98 },
    { 0, 0 },
    { 0x3697f79e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc117f7a1, 33 },
    { 0x40e7f7a2, 23 },
    { 0xc377f7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c67f7a6, 33 },
    { 0xde07f7a7, 93 },
    { 0x6527f7a8, 77 },
    { 0x24b7f7a9, 99 },
    { 0xe5d7f7a8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d87f7ad, 33 },
    { 0x6a47f7ae, 23 },
    { 0xda37f7af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8457f7b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe7f7b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd97f7b9, 81 },
    { 0, 0 },
    { 0xa7c7f7bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc947f7c1, 33 },
    { 0x86d7f7c1, 98 },
    { 0x2557f7c2, 61 },
    { 0xa477f7c4, 90 },
    { 0xddb7f7c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd7f7c8, 61 },
    { 0xb287f7c9, 40 },
    { 0, 0 },
    { 0xdcd7f7cb, 66 },
    { 0, 0 },
    { 0xe447f7cd, 88 },
    { 0x1547f7cd, 23 },
    { 0x55a7f7cf, 67 },
    { 0xc2b7f7d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd297f7d3, 23 },
    { 0x2c7f7d3, 61 },
    { 0xe917f7d3, 33 },
    { 0, 0 },
    { 0x7de7f7d7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb77f7da, 55 },
    { 0xf817f7db, 34 },
    { 0x6517f7dc, 98 },
    { 0x7ad7f7dc, 76 },
    { 0x5e87f7da, 88 },
    { 0x1f97f7df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef7f7e2, 90 },
    { 0x5fa7f7e3, 46 },
    { 0x5fb7f7e4, 43 },
    { 0xce7f7e4, 33 },
    { 0x45e7f7e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d67f7e9, 67 },
    { 0xcef7f7e9, 61 },
    { 0, 0 },
    { 0xe87f7ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e7f7ef, 43 },
    { 0, 0 },
    { 0x9b07f7f1, 34 },
    { 0, 0 },
    { 0xba37f7f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2417f7f8, 61 },
    { 0x3df7f7f9, 90 },
    { 0x9f27f7fa, 23 },
    { 0x7327f7fa, 95 },
    { 0x7b7f7f8, 43 },
    { 0x817f7fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb467f800, 55 },
    { 0xfef7f801, 23 },
    { 0x23c7f802, 95 },
    { 0x4e57f803, 95 },
    { 0, 0 },
    { 0xbd47f805, 98 },
    { 0x3727f805, 55 },
    { 0xe0a7f807, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7f80a, 33 },
    { 0, 0 },
    { 0x8ca7f80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3707f811, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd557f817, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f7f81b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef7f81f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1687f826, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc97f82b, 77 },
    { 0xb097f82c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc7f832, 61 },
    { 0xfdb7f832, 90 },
    { 0x3517f833, 33 },
    { 0x4487f834, 98 },
    { 0, 0 },
    { 0x1e87f837, 48 },
    { 0, 0 },
    { 0x85d7f839, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef7f840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd07f847, 95 },
    { 0, 0 },
    { 0xcec7f849, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3427f84c, 43 },
    { 0, 0 },
    { 0xab27f84e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1857f855, 61 },
    { 0, 0 },
    { 0x6487f857, 33 },
    { 0xd977f858, 52 },
    { 0, 0 },
    { 0x39a7f85a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f7f85e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf297f861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c07f869, 75 },
    { 0xb587f869, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2917f870, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc367f874, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe037f87b, 98 },
    { 0xb887f87c, 87 },
    { 0xa7f7f87b, 23 },
    { 0xf027f87e, 33 },
    { 0x2047f87d, 99 },
    { 0x4f7f87c, 76 },
    { 0x5de7f87b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f885, 90 },
    { 0xeb37f885, 84 },
    { 0x39b7f887, 33 },
    { 0, 0 },
    { 0xc507f889, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7347f88e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd37f891, 95 },
    { 0, 0 },
    { 0xcc77f893, 90 },
    { 0x987f894, 95 },
    { 0, 0 },
    { 0xd9b7f896, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6897f89b, 76 },
    { 0, 0 },
    { 0xf257f89d, 84 },
    { 0xf7a7f89e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c7f8a6, 91 },
    { 0x8bb7f8a6, 98 },
    { 0x3c87f8a7, 98 },
    { 0, 0 },
    { 0x1657f8aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8027f8ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d7f8b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a7f8b6, 33 },
    { 0x73e7f8b7, 90 },
    { 0x5437f8b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e7f8bc, 61 },
    { 0xc9b7f8bd, 61 },
    { 0x1737f8be, 93 },
    { 0x2487f8bf, 99 },
    { 0x8a67f8c0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f7f8c5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb7f8d0, 68 },
    { 0xc7f7f8d1, 67 },
    { 0xa327f8d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147f8d9, 67 },
    { 0xac77f8da, 95 },
    { 0, 0 },
    { 0xb587f8dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f7f8e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e97f8e8, 33 },
    { 0x4857f8e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9787f8ed, 98 },
    { 0, 0 },
    { 0x7c37f8ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7f8f3, 46 },
    { 0x6727f8f3, 33 },
    { 0x2c27f8f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a97f8f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7397f8fb, 43 },
    { 0x4a17f8fc, 23 },
    { 0x1be7f8fd, 95 },
    { 0xaf87f8fb, 33 },
    { 0x4517f8ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6617f911, 33 },
    { 0xc4d7f912, 66 },
    { 0x4717f911, 85 },
    { 0x4aa7f912, 17 },
    { 0xecd7f912, 23 },
    { 0xa527f913, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7f919, 33 },
    { 0x92c7f91a, 33 },
    { 0x5157f91a, 66 },
    { 0, 0 },
    { 0x7b77f91d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf7f922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa337f925, 55 },
    { 0xfd37f926, 33 },
    { 0xe157f927, 90 },
    { 0xd487f926, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd7f92c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b7f930, 95 },
    { 0x907f931, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b7f934, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf57f93a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd087f940, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca7f944, 99 },
    { 0, 0 },
    { 0x5407f946, 34 },
    { 0xc767f946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f7f94a, 33 },
    { 0x7db7f94b, 68 },
    { 0, 0 },
    { 0x1e17f94d, 87 },
    { 0x9167f94e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e7f953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d7f95c, 98 },
    { 0, 0 },
    { 0xaa37f95e, 95 },
    { 0, 0 },
    { 0xe317f960, 17 },
    { 0xa117f961, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc7f965, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d7f96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd7f970, 66 },
    { 0x7c07f971, 33 },
    { 0x1bf7f972, 34 },
    { 0x5387f972, 98 },
    { 0x3767f970, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff7f97b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad7f97f, 98 },
    { 0, 0 },
    { 0xe007f981, 55 },
    { 0x9057f982, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5787f989, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8037f98f, 61 },
    { 0x5d57f990, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae37f996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3377f999, 76 },
    { 0x3337f99a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7f99f, 33 },
    { 0xfb57f9a0, 55 },
    { 0xb6d7f99f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa7f9a6, 95 },
    { 0x92b7f9a7, 90 },
    { 0x8d57f9a7, 81 },
    { 0x2c97f9a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xde77f9ac, 95 },
    { 0x9e57f9ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e7f9b3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607f9b7, 65 },
    { 0xf117f9b8, 55 },
    { 0x1a17f9b7, 100 },
    { 0xaba7f9ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4587f9bd, 78 },
    { 0, 0 },
    { 0x2ee7f9bf, 95 },
    { 0xfdc7f9bf, 61 },
    { 0x3bd7f9c1, 76 },
    { 0x7667f9c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7f9ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e7f9d0, 67 },
    { 0, 0 },
    { 0x6f17f9d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4817f9d5, 61 },
    { 0xaba7f9d6, 95 },
    { 0x8127f9d7, 90 },
    { 0xb947f9d5, 77 },
    { 0xcfb7f9d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf7f9dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe177f9e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff87f9e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f7f9e9, 98 },
    { 0, 0 },
    { 0xcf27f9eb, 90 },
    { 0xcff7f9ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x707f9ef, 23 },
    { 0, 0 },
    { 0xa897f9f1, 61 },
    { 0, 0 },
    { 0xc807f9f3, 76 },
    { 0, 0 },
    { 0xb497f9f5, 34 },
    { 0x7327f9f5, 55 },
    { 0x1f97f9f7, 95 },
    { 0, 0 },
    { 0xf247f9f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2947f9fd, 76 },
    { 0, 0 },
    { 0x1767f9ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8157fa03, 98 },
    { 0xb767fa04, 38 },
    { 0x9d67fa04, 33 },
    { 0, 0 },
    { 0x1307fa07, 23 },
    { 0xc2e7fa07, 95 },
    { 0, 0 },
    { 0xd07fa0a, 61 },
    { 0x3d57fa0a, 33 },
    { 0xa5f7fa0a, 67 },
    { 0xa227fa0d, 34 },
    { 0x2b7fa0e, 23 },
    { 0xc297fa0d, 90 },
    { 0, 0 },
    { 0x1867fa11, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb7fa18, 33 },
    { 0, 0 },
    { 0x857fa1a, 61 },
    { 0, 0 },
    { 0x8787fa1c, 66 },
    { 0x2f37fa1c, 55 },
    { 0x9947fa1e, 33 },
    { 0x8477fa1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa7fa23, 95 },
    { 0, 0 },
    { 0xa757fa25, 23 },
    { 0, 0 },
    { 0x9b97fa27, 98 },
    { 0x9467fa28, 33 },
    { 0x9397fa28, 90 },
    { 0, 0 },
    { 0xd777fa2b, 78 },
    { 0xe687fa2c, 56 },
    { 0xcd97fa2b, 81 },
    { 0xc097fa2e, 91 },
    { 0xbf77fa2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde87fa34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd527fa38, 76 },
    { 0, 0 },
    { 0xea07fa3a, 95 },
    { 0, 0 },
    { 0x6dc7fa3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7fa3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3757fa42, 98 },
    { 0x28b7fa43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a37fa48, 55 },
    { 0xafd7fa48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4197fa50, 95 },
    { 0x107fa51, 98 },
    { 0x83d7fa52, 33 },
    { 0, 0 },
    { 0xc5f7fa54, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a7fa58, 98 },
    { 0, 0 },
    { 0x2517fa5a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c37fa5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef27fa62, 33 },
    { 0xeea7fa63, 84 },
    { 0xd9c7fa64, 43 },
    { 0xd2e7fa65, 61 },
    { 0xa7d7fa66, 55 },
    { 0x637fa67, 55 },
    { 0x4687fa68, 95 },
    { 0x3537fa69, 99 },
    { 0xfe07fa69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b7fa6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9557fa74, 84 },
    { 0, 0 },
    { 0xedc7fa76, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab7fa7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f7fa80, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf77fa84, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5047fa87, 76 },
    { 0x3aa7fa88, 46 },
    { 0x9147fa89, 76 },
    { 0, 0 },
    { 0x9cc7fa8b, 90 },
    { 0, 0 },
    { 0xfb67fa8d, 93 },
    { 0x7067fa8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a77fa92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6847fa98, 61 },
    { 0xe817fa99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7fa9d, 90 },
    { 0, 0 },
    { 0x4ab7fa9f, 78 },
    { 0, 0 },
    { 0x3d27faa1, 100 },
    { 0, 0 },
    { 0x14e7faa3, 98 },
    { 0x4397faa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da7faa9, 34 },
    { 0, 0 },
    { 0x3217faab, 61 },
    { 0xef97faac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1677fab3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc897fabc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de7fac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e7fac4, 84 },
    { 0, 0 },
    { 0x4037fac6, 61 },
    { 0, 0 },
    { 0xcd07fac8, 33 },
    { 0, 0 },
    { 0x6f67faca, 33 },
    { 0x9a17facb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb7fad0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5827fad3, 98 },
    { 0x1e57fad4, 88 },
    { 0x3b07fad5, 99 },
    { 0xe6a7fad6, 53 },
    { 0x57a7fad7, 34 },
    { 0xa0d7fad8, 88 },
    { 0xd57fad8, 75 },
    { 0x4137fada, 90 },
    { 0xc8e7fad9, 33 },
    { 0xd417fad5, 94 },
    { 0, 0 },
    { 0xfbc7fade, 76 },
    { 0xe577fadf, 77 },
    { 0x6997fae0, 78 },
    { 0x9f07fae1, 65 },
    { 0, 0 },
    { 0x54f7fae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa707faea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d17faee, 61 },
    { 0xd647faef, 46 },
    { 0x1f17faef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1837faf6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9507faf9, 93 },
    { 0xb807faf9, 33 },
    { 0x3cf7fafb, 52 },
    { 0x8247fafb, 76 },
    { 0x7147fafd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f7fb00, 76 },
    { 0, 0 },
    { 0xe9c7fb02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f7fb07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de7fb0d, 95 },
    { 0x64c7fb0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b7fb15, 23 },
    { 0xe667fb16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda47fb1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e37fb1d, 46 },
    { 0, 0 },
    { 0x71b7fb1f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d97fb28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf967fb2b, 90 },
    { 0xae57fb2c, 95 },
    { 0x38f7fb2b, 24 },
    { 0xaa07fb2e, 33 },
    { 0, 0 },
    { 0x1e67fb30, 68 },
    { 0xd837fb31, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6457fb34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f67fb3a, 55 },
    { 0x37c7fb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce7fb3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa517fb4b, 98 },
    { 0, 0 },
    { 0x18d7fb4d, 46 },
    { 0, 0 },
    { 0x71f7fb4f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf177fb55, 95 },
    { 0, 0 },
    { 0x6aa7fb57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27fb5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a97fb5f, 88 },
    { 0x8447fb5f, 91 },
    { 0x3c97fb61, 95 },
    { 0x3397fb62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2437fb69, 43 },
    { 0x5197fb69, 69 },
    { 0, 0 },
    { 0x1567fb6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3707fb6f, 61 },
    { 0xa5d7fb6f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e7fb75, 33 },
    { 0x1b37fb75, 34 },
    { 0x1e67fb77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f7fb7a, 55 },
    { 0xd5f7fb7b, 98 },
    { 0, 0 },
    { 0x80e7fb7d, 61 },
    { 0x9a27fb7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9457fb84, 33 },
    { 0xcca7fb84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea7fb8a, 55 },
    { 0x53e7fb8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd457fb8f, 77 },
    { 0x7e57fb90, 90 },
    { 0xdec7fb8f, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0x9407fb94, 90 },
    { 0, 0 },
    { 0x3f87fb96, 23 },
    { 0x9bb7fb97, 76 },
    { 0x4f07fb96, 43 },
    { 0xd3d7fb99, 68 },
    { 0x92c7fb9a, 96 },
    { 0xb7f7fb98, 90 },
    { 0x3907fb9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c97fba2, 66 },
    { 0x4e37fba2, 43 },
    { 0x18f7fba2, 99 },
    { 0xe4f7fba5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f07fbab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b27fbae, 66 },
    { 0, 0 },
    { 0x5ba7fbb0, 68 },
    { 0x9837fbb1, 33 },
    { 0xfeb7fbb2, 90 },
    { 0x3357fbb3, 61 },
    { 0x25c7fbb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a67fbb7, 61 },
    { 0, 0 },
    { 0x5847fbb9, 95 },
    { 0x7847fbba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea17fbbd, 33 },
    { 0xb347fbbe, 17 },
    { 0, 0 },
    { 0x7ac7fbc0, 87 },
    { 0x6c17fbc1, 76 },
    { 0, 0 },
    { 0xa867fbc3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd7fbc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef7fbc9, 90 },
    { 0xf6d7fbc9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc047fbcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4317fbd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f7fbd7, 61 },
    { 0x6897fbd7, 84 },
    { 0x9f7fbd9, 98 },
    { 0x127fbda, 99 },
    { 0x9117fbd8, 55 },
    { 0x6b57fbda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb707fbe2, 98 },
    { 0, 0 },
    { 0x7497fbe4, 99 },
    { 0, 0 },
    { 0xf447fbe6, 33 },
    { 0x37d7fbe6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea7fbeb, 90 },
    { 0, 0 },
    { 0x68b7fbed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d7fbf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf997fbf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd817fbf8, 98 },
    { 0, 0 },
    { 0x9e37fbfa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c67fbff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc7fc05, 90 },
    { 0, 0 },
    { 0xb1e7fc07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c57fc0a, 90 },
    { 0xd4e7fc0a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa197fc0e, 34 },
    { 0x6487fc0f, 33 },
    { 0x4e7fc10, 33 },
    { 0x8977fc11, 52 },
    { 0x7087fc12, 100 },
    { 0x9477fc13, 90 },
    { 0, 0 },
    { 0x4527fc15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b7fc1b, 33 },
    { 0xccc7fc1b, 33 },
    { 0, 0 },
    { 0x1ec7fc1e, 96 },
    { 0x22b7fc1f, 100 },
    { 0xdd77fc20, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe07fc29, 90 },
    { 0, 0 },
    { 0xc8e7fc2b, 61 },
    { 0xd687fc2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8657fc2f, 98 },
    { 0, 0 },
    { 0x19e7fc31, 61 },
    { 0xfed7fc32, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4497fc35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b97fc3a, 95 },
    { 0xdc47fc3b, 76 },
    { 0x1b87fc3a, 90 },
    { 0, 0 },
    { 0xed7fc3e, 80 },
    { 0x95c7fc3e, 55 },
    { 0x87c7fc3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d7fc46, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4657fc4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa647fc53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c7fc57, 76 },
    { 0x48d7fc58, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e7fc5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d7fc61, 90 },
    { 0x8927fc61, 99 },
    { 0xd157fc63, 23 },
    { 0, 0 },
    { 0xc237fc65, 33 },
    { 0x5347fc65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc717fc6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67fc71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2957fc75, 55 },
    { 0xe057fc76, 76 },
    { 0x6707fc77, 95 },
    { 0xdc17fc78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca7fc80, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a7fc87, 23 },
    { 0, 0 },
    { 0x2237fc89, 23 },
    { 0x3637fc8a, 90 },
    { 0x3347fc8b, 23 },
    { 0xeb37fc8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff7fc90, 80 },
    { 0xc887fc91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f7fc94, 55 },
    { 0, 0 },
    { 0xcf07fc96, 100 },
    { 0x13b7fc97, 43 },
    { 0x93a7fc98, 91 },
    { 0x4ab7fc98, 78 },
    { 0, 0 },
    { 0x5dd7fc9b, 33 },
    { 0, 0 },
    { 0x9ac7fc9d, 76 },
    { 0x5337fc9e, 91 },
    { 0, 0 },
    { 0x62f7fca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa67fca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf67fcab, 61 },
    { 0xdff7fcab, 17 },
    { 0, 0 },
    { 0x3c67fcae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd607fcb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f7fcb8, 61 },
    { 0x16a7fcb9, 66 },
    { 0xc7c7fcba, 61 },
    { 0, 0 },
    { 0xd27fcbc, 23 },
    { 0, 0 },
    { 0xb537fcbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1717fcc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb17fcca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb997fccd, 95 },
    { 0, 0 },
    { 0x77a7fccf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c7fcd3, 53 },
    { 0x8947fcd4, 34 },
    { 0x7f67fcd4, 23 },
    { 0x2297fcd3, 34 },
    { 0xe517fcd4, 66 },
    { 0x63f7fcd8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d7fcdb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff7fce0, 55 },
    { 0x7ef7fce0, 66 },
    { 0x29d7fce2, 61 },
    { 0xa147fce3, 90 },
    { 0x91d7fce4, 55 },
    { 0x48c7fce5, 76 },
    { 0xc637fce5, 90 },
    { 0x7957fce5, 61 },
    { 0x30e7fce2, 76 },
    { 0x1cb7fce9, 85 },
    { 0x8377fce9, 52 },
    { 0, 0 },
    { 0xd887fcec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd7fcef, 75 },
    { 0xda77fcf0, 46 },
    { 0x5947fcf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e27fcf4, 99 },
    { 0x5667fcf5, 93 },
    { 0xae97fcf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e07fcf9, 90 },
    { 0x2967fcfa, 61 },
    { 0x3467fcfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f7fcfe, 90 },
    { 0, 0 },
    { 0xc3a7fd00, 88 },
    { 0x45d7fd00, 43 },
    { 0, 0 },
    { 0x9117fd03, 43 },
    { 0, 0 },
    { 0x5447fd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c77fd09, 42 },
    { 0x8357fd0a, 61 },
    { 0x6a87fd0a, 95 },
    { 0xb977fd0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe877fd10, 76 },
    { 0x16c7fd11, 23 },
    { 0xc1e7fd11, 95 },
    { 0x6487fd11, 43 },
    { 0, 0 },
    { 0xa6c7fd15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d7fd1a, 95 },
    { 0xf5a7fd1b, 43 },
    { 0, 0 },
    { 0xbcd7fd1d, 61 },
    { 0xcbb7fd1e, 76 },
    { 0x46e7fd1f, 90 },
    { 0xefc7fd1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d7fd2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e7fd2e, 87 },
    { 0xb8d7fd2f, 61 },
    { 0, 0 },
    { 0xd4c7fd31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b57fd34, 99 },
    { 0x6877fd35, 95 },
    { 0x3757fd36, 56 },
    { 0x6fa7fd36, 95 },
    { 0x8d87fd38, 98 },
    { 0x1ca7fd39, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b47fd3c, 98 },
    { 0x6257fd3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f7fd46, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf547fd49, 33 },
    { 0, 0 },
    { 0xc0c7fd4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf7fd55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea7fd59, 88 },
    { 0x69d7fd5a, 90 },
    { 0x4177fd5b, 55 },
    { 0x2b17fd5c, 33 },
    { 0x8dd7fd5b, 74 },
    { 0x9147fd5e, 95 },
    { 0x48e7fd5f, 61 },
    { 0x7b97fd60, 17 },
    { 0x43b7fd5f, 95 },
    { 0xdb87fd60, 38 },
    { 0xad27fd5d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2767fd69, 23 },
    { 0xa3e7fd6a, 23 },
    { 0x6a57fd6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b7fd6f, 76 },
    { 0, 0 },
    { 0x78d7fd71, 76 },
    { 0xd587fd72, 90 },
    { 0xf1c7fd73, 33 },
    { 0x1d37fd74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7fd78, 46 },
    { 0xe4d7fd79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a7fd7e, 55 },
    { 0, 0 },
    { 0x1c47fd80, 61 },
    { 0xd427fd81, 55 },
    { 0, 0 },
    { 0xcf07fd83, 23 },
    { 0x9c17fd84, 63 },
    { 0x8aa7fd85, 65 },
    { 0xa6e7fd86, 90 },
    { 0x1497fd86, 91 },
    { 0x6f27fd86, 99 },
    { 0, 0 },
    { 0x68b7fd8a, 65 },
    { 0x77a7fd8b, 95 },
    { 0, 0 },
    { 0x49f7fd8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a7fd90, 93 },
    { 0, 0 },
    { 0x8bd7fd92, 90 },
    { 0, 0 },
    { 0x9b67fd94, 76 },
    { 0x37a7fd94, 33 },
    { 0, 0 },
    { 0xb557fd97, 61 },
    { 0, 0 },
    { 0x5ac7fd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5447fda5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be7fda9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6747fdad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce77fdb3, 23 },
    { 0x9947fdb4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf87fdb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce7fdbd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c87fdc0, 55 },
    { 0xb617fdc1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97fdc4, 40 },
    { 0x1727fdc5, 90 },
    { 0x2287fdc6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e47fdcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a77fdcf, 90 },
    { 0, 0 },
    { 0x45d7fdd1, 61 },
    { 0x7717fdd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea7fdda, 33 },
    { 0x1467fddb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047fde3, 90 },
    { 0, 0 },
    { 0xa097fde5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd677fdef, 46 },
    { 0, 0 },
    { 0x27fdf1, 90 },
    { 0x76d7fdf2, 90 },
    { 0x7127fdf2, 90 },
    { 0xad07fdf4, 89 },
    { 0x157fdf5, 90 },
    { 0x16f7fdf6, 33 },
    { 0x1b87fdf6, 43 },
    { 0x6107fdf8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5607fdfc, 55 },
    { 0xf137fdfc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f77fe06, 98 },
    { 0xde27fe07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d7fe0e, 98 },
    { 0, 0 },
    { 0xac47fe10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67fe15, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe18, 100 },
    { 0x1c7fe19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa587fe21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5277fe25, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c7fe28, 84 },
    { 0xc2d7fe28, 93 },
    { 0x65b7fe2a, 95 },
    { 0x3597fe2a, 43 },
    { 0x69a7fe28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b7fe2f, 98 },
    { 0x5807fe30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a7fe38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa667fe3d, 55 },
    { 0, 0 },
    { 0x2607fe3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e7fe42, 77 },
    { 0x89d7fe42, 33 },
    { 0x3167fe42, 43 },
    { 0x3f37fe42, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e27fe4b, 66 },
    { 0x70a7fe4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa7fe50, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9897fe54, 61 },
    { 0x89a7fe54, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x657fe58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f77fe5b, 61 },
    { 0x1f87fe5c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec7fe60, 55 },
    { 0x4ba7fe60, 75 },
    { 0x25b7fe61, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d77fe66, 23 },
    { 0xbc27fe67, 93 },
    { 0x89d7fe66, 34 },
    { 0x5e77fe69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2527fe72, 90 },
    { 0x90a7fe72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8877fe76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d7fe7d, 34 },
    { 0x957fe7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7fe81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7fe85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c7fe8f, 76 },
    { 0xa887fe90, 90 },
    { 0, 0 },
    { 0x91d7fe92, 33 },
    { 0x3577fe93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe77fe9a, 76 },
    { 0xd397fe9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf827fe9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd97fea7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b17feac, 17 },
    { 0, 0 },
    { 0xa0d7feae, 78 },
    { 0x84d7feaf, 88 },
    { 0x4e47feb0, 98 },
    { 0x1b07feb1, 90 },
    { 0x3907feae, 33 },
    { 0, 0 },
    { 0x24f7feb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc727febe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c7fec2, 90 },
    { 0xffc7fec3, 33 },
    { 0, 0 },
    { 0x517fec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a7fec8, 61 },
    { 0xe2f7fec9, 96 },
    { 0x8097feca, 23 },
    { 0, 0 },
    { 0xd6f7fecc, 76 },
    { 0xd4a7fecc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c7fed0, 43 },
    { 0, 0 },
    { 0xe617fed2, 61 },
    { 0, 0 },
    { 0xfc57fed4, 76 },
    { 0x1457fed5, 77 },
    { 0x3307fed6, 33 },
    { 0x2e37fed6, 61 },
    { 0xa4b7fed8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd437fedd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7fee3, 55 },
    { 0x6267fee4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd777fee7, 95 },
    { 0, 0 },
    { 0xafa7fee9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c7fef0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf77fef4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a7fef9, 75 },
    { 0xe3f7fefa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c17ff00, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c7ff03, 100 },
    { 0, 0 },
    { 0x8ca7ff05, 98 },
    { 0xb447ff06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d07ff0a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6797ff0e, 23 },
    { 0, 0 },
    { 0xf207ff10, 34 },
    { 0x6267ff11, 69 },
    { 0x23d7ff10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5997ff1f, 23 },
    { 0x9897ff1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f7ff23, 98 },
    { 0x7957ff24, 81 },
    { 0xbe27ff25, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa7ff28, 76 },
    { 0x4747ff29, 88 },
    { 0xa5b7ff29, 61 },
    { 0xa667ff2b, 91 },
    { 0x39e7ff2c, 95 },
    { 0, 0 },
    { 0xf5b7ff2e, 76 },
    { 0, 0 },
    { 0x5e7ff30, 90 },
    { 0, 0 },
    { 0x91a7ff32, 90 },
    { 0x157ff33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f77ff37, 95 },
    { 0x7b97ff37, 93 },
    { 0, 0 },
    { 0x6a47ff3a, 61 },
    { 0x8f67ff3b, 43 },
    { 0xf0c7ff3a, 61 },
    { 0xae57ff3d, 23 },
    { 0x31f7ff3d, 76 },
    { 0x18f7ff3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c7ff47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab37ff4b, 43 },
    { 0x72c7ff4c, 98 },
    { 0, 0 },
    { 0xf827ff4e, 98 },
    { 0xf797ff4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf27ff53, 43 },
    { 0xb207ff54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf277ff58, 99 },
    { 0xb97ff59, 34 },
    { 0, 0 },
    { 0xe257ff5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3317ff60, 43 },
    { 0x91e7ff61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db7ff65, 23 },
    { 0, 0 },
    { 0x14b7ff67, 43 },
    { 0xa037ff68, 76 },
    { 0, 0 },
    { 0x93c7ff6a, 33 },
    { 0xe347ff6b, 33 },
    { 0x5587ff6c, 33 },
    { 0, 0 },
    { 0x2957ff6e, 55 },
    { 0xd307ff6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c7ff73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c7ff76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa217ff7f, 33 },
    { 0xf127ff7f, 95 },
    { 0, 0 },
    { 0x1b7ff82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea27ff87, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5787ff90, 76 },
    { 0x47ff91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d47ff98, 34 },
    { 0, 0 },
    { 0xdfc7ff9a, 100 },
    { 0xec77ff9b, 40 },
    { 0, 0 },
    { 0x59e7ff9d, 33 },
    { 0x1187ff9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7ffa1, 55 },
    { 0xef37ffa2, 84 },
    { 0, 0 },
    { 0x1a07ffa4, 80 },
    { 0x4487ffa5, 76 },
    { 0xebc7ffa6, 61 },
    { 0xd8c7ffa6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7ffaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f7ffad, 17 },
    { 0x6377ffae, 33 },
    { 0xa987ffaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8297ffb4, 43 },
    { 0x8157ffb4, 98 },
    { 0, 0 },
    { 0xcf67ffb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9577ffbb, 76 },
    { 0xa797ffbc, 95 },
    { 0xb097ffbb, 61 },
    { 0x27e7ffbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd047ffc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe47ffc6, 23 },
    { 0, 0 },
    { 0xeab7ffc8, 95 },
    { 0x1417ffc9, 33 },
    { 0xeae7ffc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c7ffd2, 66 },
    { 0, 0 },
    { 0x7447ffd4, 46 },
    { 0x3587ffd5, 98 },
    { 0x91e7ffd4, 88 },
    { 0xb937ffd7, 76 },
    { 0xb707ffd5, 85 },
    { 0xb367ffd8, 23 },
    { 0xdcf7ffda, 34 },
    { 0xda47ffdb, 43 },
    { 0x7e87ffdc, 90 },
    { 0xb977ffdd, 76 },
    { 0x9107ffdb, 33 },
    { 0x6fa7ffdb, 90 },
    { 0x5387ffe0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c7ffe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5d7ffe8, 90 },
    { 0, 0 },
    { 0xdaf7ffea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d97ffef, 81 },
    { 0, 0 },
    { 0xd1e7fff1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f7fffd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e80000, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d80008, 61 },
    { 0x1a680008, 98 },
    { 0xd8b80009, 76 },
    { 0, 0 },
    { 0x7378000c, 90 },
    { 0, 0 },
    { 0xbdf8000e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b80012, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb480015, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd8001a, 85 },
    { 0x9788001b, 90 },
    { 0, 0 },
    { 0xa8e8001d, 76 },
    { 0x3d28001e, 33 },
    { 0, 0 },
    { 0xae680020, 55 },
    { 0x82980021, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d80024, 98 },
    { 0x13980025, 61 },
    { 0x61980025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb580029, 100 },
    { 0xfb8002a, 23 },
    { 0xa168002b, 95 },
    { 0x2fc8002a, 46 },
    { 0, 0 },
    { 0x4b98002e, 61 },
    { 0x468002f, 43 },
    { 0, 0 },
    { 0xbf680031, 36 },
    { 0, 0 },
    { 0x68880033, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea80037, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab8003b, 23 },
    { 0x6068003c, 76 },
    { 0xfb08003d, 61 },
    { 0x1cb8003d, 90 },
    { 0, 0 },
    { 0xf0880040, 23 },
    { 0xb4b80041, 33 },
    { 0xe680042, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30280047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe888004e, 61 },
    { 0, 0 },
    { 0xf4080050, 46 },
    { 0xee980051, 69 },
    { 0, 0 },
    { 0xc2480053, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede80058, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2888005f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2580064, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3780067, 34 },
    { 0x90380068, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee8006b, 99 },
    { 0, 0 },
    { 0xb298006d, 75 },
    { 0, 0 },
    { 0xbf8006f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda80072, 30 },
    { 0x69f80073, 90 },
    { 0x3c080073, 52 },
    { 0x9c880074, 90 },
    { 0xc1d80076, 55 },
    { 0x8da80077, 76 },
    { 0xcaa80078, 55 },
    { 0x7ad80076, 23 },
    { 0xbf98007a, 61 },
    { 0x91f8007b, 17 },
    { 0, 0 },
    { 0x7c58007d, 77 },
    { 0xd378007d, 61 },
    { 0xea8007f, 91 },
    { 0x97980080, 76 },
    { 0x9de80081, 55 },
    { 0x46480081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b80088, 23 },
    { 0xc880089, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e8008e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b680092, 95 },
    { 0, 0 },
    { 0xe9680094, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45800a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1800a4, 85 },
    { 0xc3800a4, 95 },
    { 0xec8800a4, 100 },
    { 0x113800a7, 90 },
    { 0xbaa800a7, 66 },
    { 0x673800a8, 23 },
    { 0xd0800a9, 78 },
    { 0xad3800a7, 95 },
    { 0x597800ac, 33 },
    { 0x243800a7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x934800b0, 67 },
    { 0, 0 },
    { 0xf7b800b2, 33 },
    { 0, 0 },
    { 0x911800b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a800ba, 99 },
    { 0xb94800bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f800bf, 98 },
    { 0x41a800c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd800c6, 53 },
    { 0xc9d800c7, 34 },
    { 0xcd4800c7, 93 },
    { 0x647800c9, 91 },
    { 0xdc1800ca, 88 },
    { 0x937800cb, 90 },
    { 0, 0 },
    { 0xadb800cd, 90 },
    { 0x282800ce, 95 },
    { 0x216800cd, 76 },
    { 0x38800d0, 90 },
    { 0, 0 },
    { 0x5b9800d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1800d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522800dc, 95 },
    { 0x261800dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b800e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb800e8, 33 },
    { 0x1d9800e9, 90 },
    { 0, 0 },
    { 0xe50800eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2800ef, 90 },
    { 0x8bc800f0, 33 },
    { 0x1a4800f1, 61 },
    { 0x36d800f1, 66 },
    { 0x49a800f1, 61 },
    { 0x2c6800f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11800fc, 77 },
    { 0x785800fc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b180102, 61 },
    { 0xe1d80103, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc280106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868010c, 95 },
    { 0, 0 },
    { 0xda18010e, 61 },
    { 0xc9d8010f, 78 },
    { 0, 0 },
    { 0xd0f80111, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94980118, 95 },
    { 0x18a80119, 91 },
    { 0, 0 },
    { 0xad58011b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e8011e, 93 },
    { 0x9338011e, 76 },
    { 0x4a780120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c80123, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f80129, 33 },
    { 0x6948012a, 78 },
    { 0x6fd8012a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf380130, 33 },
    { 0x85580131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c980135, 33 },
    { 0xa0980135, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b80140, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e80143, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe180146, 61 },
    { 0, 0 },
    { 0x71c80148, 33 },
    { 0x32180149, 67 },
    { 0x87680148, 61 },
    { 0x5580149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe208014e, 66 },
    { 0x1528014e, 90 },
    { 0x1fa8014e, 33 },
    { 0xfab80151, 88 },
    { 0x9ca80152, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728015b, 43 },
    { 0x88b8015c, 33 },
    { 0, 0 },
    { 0xbb18015e, 76 },
    { 0x9a88015f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f80168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf588016b, 33 },
    { 0x2ac8016c, 61 },
    { 0, 0 },
    { 0x9e18016e, 65 },
    { 0xfe18016e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a80174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e88017a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc208017d, 76 },
    { 0x7a98017e, 98 },
    { 0x5b18017e, 34 },
    { 0, 0 },
    { 0xa9f80181, 88 },
    { 0, 0 },
    { 0x43180183, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61180186, 33 },
    { 0, 0 },
    { 0xed880188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d28018b, 90 },
    { 0x8ea8018c, 95 },
    { 0, 0 },
    { 0x8258018e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87580191, 98 },
    { 0x6ba80192, 61 },
    { 0xbf180192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15801a1, 68 },
    { 0x1c4801a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0801a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461801ae, 55 },
    { 0, 0 },
    { 0x157801b0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29801b3, 76 },
    { 0x5a7801b3, 23 },
    { 0xd05801b4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f801b9, 95 },
    { 0x926801b9, 95 },
    { 0, 0 },
    { 0x175801bc, 87 },
    { 0xb32801bd, 90 },
    { 0, 0 },
    { 0xceb801bf, 23 },
    { 0, 0 },
    { 0x12a801c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5801c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e801ce, 95 },
    { 0xce7801ce, 33 },
    { 0, 0 },
    { 0xb2f801d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc801d4, 66 },
    { 0, 0 },
    { 0x665801d6, 55 },
    { 0, 0 },
    { 0x87f801d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x385801db, 90 },
    { 0, 0 },
    { 0xf32801dd, 95 },
    { 0xf88801dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef801e5, 90 },
    { 0xab0801e6, 98 },
    { 0x8bc801e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711801f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d801f6, 36 },
    { 0xcf6801f7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2801fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde180200, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7980206, 98 },
    { 0xb3480207, 52 },
    { 0, 0 },
    { 0xf2f80209, 61 },
    { 0x11c80209, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8020d, 23 },
    { 0x5458020e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d580212, 61 },
    { 0xef280212, 100 },
    { 0xe2d80214, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa880217, 68 },
    { 0x16280218, 91 },
    { 0x58580217, 55 },
    { 0xf528021a, 98 },
    { 0xa778021a, 93 },
    { 0, 0 },
    { 0x5078021d, 76 },
    { 0, 0 },
    { 0x8e18021f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76580226, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a8022b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76780235, 34 },
    { 0x28280236, 90 },
    { 0x69080237, 95 },
    { 0x6d080238, 34 },
    { 0, 0 },
    { 0xf938023a, 76 },
    { 0x3bb8023b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af8023f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x27080242, 99 },
    { 0x68880243, 95 },
    { 0xd880244, 33 },
    { 0, 0 },
    { 0xd0080246, 96 },
    { 0x4ab80246, 95 },
    { 0, 0 },
    { 0x52280249, 90 },
    { 0, 0 },
    { 0x8738024b, 98 },
    { 0, 0 },
    { 0xf1e8024d, 90 },
    { 0xb458024e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa580252, 98 },
    { 0, 0 },
    { 0xde080254, 32 },
    { 0xe7480255, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca8025b, 23 },
    { 0xfaa8025c, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb580267, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb8026f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8080272, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b38027d, 76 },
    { 0x138027d, 34 },
    { 0x5b68027d, 76 },
    { 0xefa8027f, 84 },
    { 0, 0 },
    { 0x67c80282, 88 },
    { 0x42480282, 90 },
    { 0x19a80282, 43 },
    { 0x54980285, 75 },
    { 0xa5880283, 23 },
    { 0xaee80287, 78 },
    { 0x67680286, 33 },
    { 0xf8880287, 90 },
    { 0x20e8028a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc78028e, 81 },
    { 0x9798028f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e80294, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5428029a, 87 },
    { 0xd8029b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e78029e, 91 },
    { 0x56f8029e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815802a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60802a6, 96 },
    { 0xe31802a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b802ae, 90 },
    { 0, 0 },
    { 0x71d802b0, 90 },
    { 0x50d802b0, 33 },
    { 0x9e9802b2, 61 },
    { 0x69802b3, 75 },
    { 0xbec802b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459802b8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31802bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159802bf, 90 },
    { 0x40c802bf, 95 },
    { 0xe43802c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105802c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf802d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a802db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed802df, 90 },
    { 0xf4d802e0, 95 },
    { 0, 0 },
    { 0x9e1802e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154802eb, 90 },
    { 0x38a802ec, 81 },
    { 0x472802eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2802f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba802f4, 61 },
    { 0xf64802f5, 90 },
    { 0, 0 },
    { 0x23a802f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a802fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec80301, 52 },
    { 0, 0 },
    { 0xf9e80303, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b58030b, 91 },
    { 0x2238030c, 95 },
    { 0xa6d8030b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb80313, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc80316, 76 },
    { 0xdfb80316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b8031a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c8031e, 95 },
    { 0x8d98031f, 33 },
    { 0, 0 },
    { 0xd3380321, 91 },
    { 0xab380322, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d580334, 98 },
    { 0x91880335, 61 },
    { 0x10c80336, 95 },
    { 0x28580337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf628033d, 33 },
    { 0, 0 },
    { 0xc0d8033f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc680345, 61 },
    { 0xf0280345, 90 },
    { 0x9ae80346, 33 },
    { 0x93780348, 61 },
    { 0x56e80349, 66 },
    { 0x5a28034a, 98 },
    { 0x8ee8034a, 55 },
    { 0x1758034c, 76 },
    { 0x2b38034d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece80352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8035c, 40 },
    { 0x9208035d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e280360, 93 },
    { 0xdfe80360, 34 },
    { 0x70480362, 76 },
    { 0x4dc80363, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31980366, 84 },
    { 0, 0 },
    { 0xfd680368, 87 },
    { 0, 0 },
    { 0x3288036a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38036e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33880379, 43 },
    { 0, 0 },
    { 0x7ca8037b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb28037e, 55 },
    { 0xb28037f, 93 },
    { 0, 0 },
    { 0x3e480381, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a80388, 33 },
    { 0, 0 },
    { 0x998038a, 33 },
    { 0, 0 },
    { 0xeea8038c, 84 },
    { 0xc8a8038d, 33 },
    { 0xae68038d, 55 },
    { 0, 0 },
    { 0x7f80390, 61 },
    { 0x9a780391, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d80394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d80399, 72 },
    { 0xd8a8039a, 95 },
    { 0, 0 },
    { 0x9478039c, 33 },
    { 0xb278039d, 87 },
    { 0, 0 },
    { 0x1a28039f, 33 },
    { 0x493803a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a803a3, 46 },
    { 0, 0 },
    { 0x39c803a5, 80 },
    { 0xb01803a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a803ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66803af, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca803b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4803b9, 40 },
    { 0x5bb803ba, 44 },
    { 0xf9e803bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec803bf, 85 },
    { 0xb5c803bf, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520803c4, 86 },
    { 0x227803c5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1803cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93803d0, 46 },
    { 0x101803d1, 67 },
    { 0, 0 },
    { 0x106803d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248803df, 90 },
    { 0xf6803e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9803e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7803e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7803f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c803f3, 95 },
    { 0x53f803f4, 98 },
    { 0xb90803f3, 23 },
    { 0x546803f6, 66 },
    { 0x70e803f7, 87 },
    { 0x88c803f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d803fd, 84 },
    { 0, 0 },
    { 0xd17803ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d580404, 52 },
    { 0x59c80405, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d38040c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e80412, 95 },
    { 0xf8180413, 23 },
    { 0xba880414, 33 },
    { 0x7580415, 33 },
    { 0xbbc80416, 61 },
    { 0, 0 },
    { 0x4dc80418, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b8041c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de80420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a80423, 61 },
    { 0x6a280424, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f80427, 93 },
    { 0x6d280427, 38 },
    { 0, 0 },
    { 0x2008042a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb68042d, 55 },
    { 0, 0 },
    { 0xc168042f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f680432, 91 },
    { 0x36a80433, 55 },
    { 0x10b80434, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a80437, 90 },
    { 0, 0 },
    { 0xd4d80439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf08043d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab880443, 76 },
    { 0xf3e80444, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e280447, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe08044b, 90 },
    { 0x8f78044c, 98 },
    { 0xfdb8044c, 43 },
    { 0xae48044e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41380451, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10480457, 33 },
    { 0x89e80458, 34 },
    { 0x1e280459, 10 },
    { 0x5aa80459, 61 },
    { 0x4398045b, 33 },
    { 0x2b98045c, 98 },
    { 0, 0 },
    { 0xeda8045e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b80463, 93 },
    { 0xe5180464, 99 },
    { 0xe5080465, 84 },
    { 0x71380466, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5280469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9958046d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f980473, 90 },
    { 0x1f880474, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c80478, 17 },
    { 0xd3980478, 33 },
    { 0xa9c80479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688047f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3880485, 98 },
    { 0xfe680485, 69 },
    { 0, 0 },
    { 0x7a580488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b80490, 76 },
    { 0xd6080490, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x85080494, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb8049b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d08049e, 90 },
    { 0x3208049f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9804a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03804aa, 95 },
    { 0x4ef804aa, 61 },
    { 0x972804ab, 61 },
    { 0x315804ad, 76 },
    { 0x458804ad, 61 },
    { 0x310804af, 90 },
    { 0x36e804b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16804b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73804b7, 23 },
    { 0x49a804b7, 34 },
    { 0xb35804b9, 23 },
    { 0xa34804b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e804bf, 23 },
    { 0x4cb804c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a804c5, 84 },
    { 0xb5a804c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e804ce, 76 },
    { 0x72e804cf, 43 },
    { 0xe61804cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89804d3, 33 },
    { 0x7a2804d4, 34 },
    { 0xcc7804d5, 65 },
    { 0, 0 },
    { 0x60c804d7, 83 },
    { 0x4a3804d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c804db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e804e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9804e9, 95 },
    { 0x8f8804e9, 55 },
    { 0x9c804eb, 23 },
    { 0, 0 },
    { 0x137804ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20804f2, 100 },
    { 0, 0 },
    { 0x3e7804f4, 43 },
    { 0xbc7804f4, 96 },
    { 0, 0 },
    { 0xee9804f7, 43 },
    { 0, 0 },
    { 0xfe9804f9, 33 },
    { 0, 0 },
    { 0x8ff804fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x809804fe, 68 },
    { 0, 0 },
    { 0x5f380500, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6980507, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2928050a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb418050d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x81480510, 78 },
    { 0x45280511, 95 },
    { 0x18b80512, 66 },
    { 0x42a80513, 46 },
    { 0x6f280512, 23 },
    { 0x92280512, 46 },
    { 0x4f80516, 61 },
    { 0xc6780516, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d8051e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8280521, 33 },
    { 0xe8e80522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa80526, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1180529, 95 },
    { 0x1928052a, 33 },
    { 0, 0 },
    { 0xb368052c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd280534, 90 },
    { 0x20f80535, 98 },
    { 0xd6180536, 50 },
    { 0x4e480536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c8053b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc778053e, 61 },
    { 0xfe38053e, 38 },
    { 0xe9a80540, 98 },
    { 0xc880541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd80549, 34 },
    { 0xd118054a, 87 },
    { 0x3e48054b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb8054e, 66 },
    { 0xeea8054f, 33 },
    { 0x388054f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef80553, 90 },
    { 0x9f180554, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1188055b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7980566, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b80569, 90 },
    { 0x5298056a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828056e, 23 },
    { 0xc598056f, 76 },
    { 0x8e080570, 33 },
    { 0x8f580570, 95 },
    { 0x4d280572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d280579, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a8057e, 93 },
    { 0, 0 },
    { 0x9b380580, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad80583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f68058a, 90 },
    { 0xe488058a, 33 },
    { 0, 0 },
    { 0x50c8058d, 95 },
    { 0x92c8058e, 34 },
    { 0, 0 },
    { 0xbc180590, 95 },
    { 0, 0 },
    { 0xda080592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a80597, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f8059c, 98 },
    { 0xbde8059d, 95 },
    { 0x2708059d, 61 },
    { 0, 0 },
    { 0xff5805a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f805a9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25805ac, 90 },
    { 0x996805ad, 76 },
    { 0x6dc805ae, 98 },
    { 0x1c4805ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6805b3, 87 },
    { 0, 0 },
    { 0xa5e805b5, 90 },
    { 0xb84805b6, 90 },
    { 0, 0 },
    { 0xe4f805b8, 61 },
    { 0xac5805b9, 46 },
    { 0, 0 },
    { 0x2af805bb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0805c3, 77 },
    { 0, 0 },
    { 0xf91805c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198805ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703805ce, 33 },
    { 0, 0 },
    { 0x32a805d0, 23 },
    { 0xcad805d0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3805d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372805db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75805de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e805e5, 76 },
    { 0xf1c805e6, 99 },
    { 0, 0 },
    { 0x42c805e8, 34 },
    { 0xba805e9, 90 },
    { 0, 0 },
    { 0x910805eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a805f0, 77 },
    { 0xa00805f0, 23 },
    { 0x6ad805f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5805f9, 95 },
    { 0xf70805f9, 43 },
    { 0, 0 },
    { 0x610805fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78805ff, 88 },
    { 0xfc680600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab80603, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79580609, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c8060f, 98 },
    { 0x3468060f, 99 },
    { 0, 0 },
    { 0x43b80612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5380615, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d80619, 87 },
    { 0x5be8061a, 81 },
    { 0x68061b, 76 },
    { 0, 0 },
    { 0xbce8061d, 43 },
    { 0, 0 },
    { 0xc38061f, 55 },
    { 0, 0 },
    { 0x8580621, 95 },
    { 0xa3680621, 76 },
    { 0x46080622, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128062a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1238062e, 17 },
    { 0x62f8062f, 33 },
    { 0x1a280630, 33 },
    { 0xce380631, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71080635, 23 },
    { 0x8c580636, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa080639, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3748063c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af80640, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf080643, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8180646, 94 },
    { 0x6d380646, 33 },
    { 0x3bc80648, 90 },
    { 0xbcf80646, 90 },
    { 0, 0 },
    { 0x8658064b, 95 },
    { 0x6c48064c, 23 },
    { 0, 0 },
    { 0xb4a8064e, 83 },
    { 0, 0 },
    { 0x84f80650, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5480657, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9058065a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae8065e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf80662, 33 },
    { 0, 0 },
    { 0xd0280664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8280669, 91 },
    { 0, 0 },
    { 0xc288066b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38066f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9180673, 33 },
    { 0x6fb80674, 33 },
    { 0x34380675, 65 },
    { 0, 0 },
    { 0x81380677, 99 },
    { 0xc6b80677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb8067c, 96 },
    { 0x1f08067d, 100 },
    { 0, 0 },
    { 0x2158067f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a80688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a8068c, 34 },
    { 0x8538068d, 61 },
    { 0x918068d, 90 },
    { 0xb538068f, 98 },
    { 0xab38068f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2580695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d80699, 90 },
    { 0x5cb8069a, 66 },
    { 0xa5c80699, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb58069e, 76 },
    { 0xb6a8069f, 99 },
    { 0x51806a0, 93 },
    { 0, 0 },
    { 0x5e1806a2, 88 },
    { 0xe76806a2, 40 },
    { 0xac8806a4, 33 },
    { 0x916806a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90806a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f806ab, 98 },
    { 0, 0 },
    { 0x8c806ad, 34 },
    { 0xe79806ae, 61 },
    { 0x35c806af, 23 },
    { 0, 0 },
    { 0x687806b1, 94 },
    { 0, 0 },
    { 0xda806b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0806bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b806c1, 95 },
    { 0xb37806c2, 90 },
    { 0x7db806c2, 93 },
    { 0, 0 },
    { 0xddb806c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x833806c8, 87 },
    { 0xcf9806c9, 33 },
    { 0xa5a806c9, 98 },
    { 0xf00806c9, 55 },
    { 0, 0 },
    { 0xbd6806cd, 99 },
    { 0xadf806ce, 98 },
    { 0x92806ce, 88 },
    { 0xeff806ce, 61 },
    { 0xd4806d1, 24 },
    { 0x597806d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507806d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x766806da, 98 },
    { 0, 0 },
    { 0x413806dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24806e0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x51806e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda806ea, 76 },
    { 0x7f6806eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd806f0, 23 },
    { 0x471806f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1806f4, 67 },
    { 0, 0 },
    { 0xa78806f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655806fe, 85 },
    { 0x95806ff, 95 },
    { 0, 0 },
    { 0xd2880701, 23 },
    { 0xde580702, 61 },
    { 0, 0 },
    { 0xde980704, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x26080707, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e80713, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb980716, 91 },
    { 0xd5580717, 90 },
    { 0x96a80716, 17 },
    { 0xc7b80718, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd68071d, 52 },
    { 0x3268071e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51680721, 43 },
    { 0x67080722, 85 },
    { 0xe8080723, 99 },
    { 0xe3b80724, 43 },
    { 0, 0 },
    { 0xb6580726, 34 },
    { 0xc0b80727, 23 },
    { 0xee980727, 98 },
    { 0x4dd80726, 33 },
    { 0, 0 },
    { 0x98072b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97580730, 23 },
    { 0, 0 },
    { 0x93080732, 99 },
    { 0, 0 },
    { 0x72280734, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2680739, 95 },
    { 0x13780739, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xed28073d, 36 },
    { 0x7688073e, 98 },
    { 0, 0 },
    { 0x78180740, 55 },
    { 0, 0 },
    { 0xd3880742, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7780746, 90 },
    { 0xf7180746, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08074c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8074f, 75 },
    { 0x4c80750, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1380757, 77 },
    { 0x5e780757, 90 },
    { 0x8d880757, 34 },
    { 0xd398075a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaca80766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e80769, 87 },
    { 0, 0 },
    { 0x91a8076b, 98 },
    { 0x60d8076c, 80 },
    { 0x4858076b, 61 },
    { 0x5688076e, 77 },
    { 0x5b48076d, 95 },
    { 0x8da80770, 33 },
    { 0x68076f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b80776, 61 },
    { 0xd9680777, 95 },
    { 0x88280778, 33 },
    { 0, 0 },
    { 0x6cf8077a, 61 },
    { 0x3c68077b, 95 },
    { 0xeff8077c, 23 },
    { 0x3ff8077a, 84 },
    { 0, 0 },
    { 0x9ee8077f, 55 },
    { 0xad380780, 98 },
    { 0, 0 },
    { 0x1c380782, 33 },
    { 0x77d80782, 23 },
    { 0x16980784, 34 },
    { 0x8dc80785, 93 },
    { 0x1fd80786, 43 },
    { 0x57080784, 98 },
    { 0xfda80784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b28078d, 90 },
    { 0xd108078e, 76 },
    { 0xa68078d, 33 },
    { 0x3b480790, 23 },
    { 0x94080791, 34 },
    { 0xddd80791, 95 },
    { 0xb3880791, 95 },
    { 0xf5c80790, 90 },
    { 0xb7e80791, 33 },
    { 0xb4880796, 80 },
    { 0xe8880797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be8079a, 46 },
    { 0, 0 },
    { 0x9508079c, 81 },
    { 0, 0 },
    { 0x2458079e, 95 },
    { 0xa8b8079f, 81 },
    { 0x3ee807a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7807a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595807aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3807b1, 99 },
    { 0xc08807b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936807b7, 99 },
    { 0xf6807b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77807be, 43 },
    { 0xad807bf, 67 },
    { 0xae1807c0, 88 },
    { 0xc68807c1, 87 },
    { 0, 0 },
    { 0x60c807c3, 61 },
    { 0x6e1807c4, 61 },
    { 0x291807c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be807c8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9807d0, 90 },
    { 0x13f807d1, 95 },
    { 0, 0 },
    { 0x79c807d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97807d6, 90 },
    { 0x8b807d6, 90 },
    { 0x210807d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b807de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f807e1, 67 },
    { 0x361807e1, 98 },
    { 0x68a807e2, 93 },
    { 0x558807e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf807e8, 33 },
    { 0x65a807e8, 34 },
    { 0x9b807ea, 46 },
    { 0xdb5807ea, 34 },
    { 0x1c2807eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36807f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c807fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae807ff, 81 },
    { 0x79f807ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c80803, 33 },
    { 0xde080804, 55 },
    { 0x26780805, 98 },
    { 0x8c580803, 61 },
    { 0x44180804, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b68080d, 91 },
    { 0x2d98080d, 98 },
    { 0xe0c8080f, 98 },
    { 0x46e8080e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2880813, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53380818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18081f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c80822, 61 },
    { 0xd4380823, 33 },
    { 0x18e80824, 90 },
    { 0xb2d80825, 61 },
    { 0x5d680823, 33 },
    { 0xdd380827, 95 },
    { 0x89f80828, 95 },
    { 0x99380829, 55 },
    { 0, 0 },
    { 0xf4f8082b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x428082e, 61 },
    { 0x18e8082f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a80832, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32980835, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3478083b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1080842, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce080845, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x15280848, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d8084d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0780850, 61 },
    { 0, 0 },
    { 0xd5b80852, 33 },
    { 0x7b680853, 90 },
    { 0x2b580854, 95 },
    { 0x60d80852, 56 },
    { 0x8ef80856, 90 },
    { 0xabe80856, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa538085a, 34 },
    { 0x9238085b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7680861, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e80865, 64 },
    { 0x6a480866, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d8086a, 98 },
    { 0xfef8086b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c480874, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63580878, 55 },
    { 0, 0 },
    { 0x58a8087a, 33 },
    { 0xc128087b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2748087f, 66 },
    { 0xa1980880, 46 },
    { 0xaff8087f, 61 },
    { 0x5980881, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b80885, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc80888, 98 },
    { 0, 0 },
    { 0x7ff8088a, 33 },
    { 0x3ce8088b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7278088f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45980894, 85 },
    { 0x70680894, 61 },
    { 0, 0 },
    { 0xad780897, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7468089d, 76 },
    { 0, 0 },
    { 0x78a8089f, 61 },
    { 0, 0 },
    { 0x2d6808a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0808a8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9808ac, 87 },
    { 0, 0 },
    { 0xfa5808ae, 76 },
    { 0xf43808af, 60 },
    { 0x1d9808ae, 34 },
    { 0x99c808b1, 66 },
    { 0, 0 },
    { 0x7b5808b3, 33 },
    { 0xd67808b4, 43 },
    { 0, 0 },
    { 0x146808b6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7808b9, 90 },
    { 0x6c4808ba, 98 },
    { 0, 0 },
    { 0x14808bc, 61 },
    { 0x6e808bc, 61 },
    { 0, 0 },
    { 0x7ea808bf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae808c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b808cb, 61 },
    { 0xeda808cb, 33 },
    { 0xf92808cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62808d2, 76 },
    { 0x2f7808d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e808d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d808e2, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37808eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a808f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0808f7, 76 },
    { 0xadf808f8, 61 },
    { 0x28c808f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53808fe, 98 },
    { 0x4bd808fe, 99 },
    { 0x744808fe, 61 },
    { 0xa8f80901, 65 },
    { 0x39480902, 76 },
    { 0x9480903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee680906, 52 },
    { 0, 0 },
    { 0x49980908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd198090c, 78 },
    { 0, 0 },
    { 0xbb88090e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3080912, 88 },
    { 0, 0 },
    { 0xce80914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58980917, 61 },
    { 0, 0 },
    { 0x91d80919, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e8091e, 95 },
    { 0x8b78091f, 87 },
    { 0, 0 },
    { 0x10880921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62980926, 33 },
    { 0xe5280927, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf58092d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a80932, 33 },
    { 0xa480932, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1080937, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8093a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b38093f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd480943, 90 },
    { 0x21180944, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a180947, 55 },
    { 0xd4080947, 33 },
    { 0x74380949, 26 },
    { 0xd898094a, 76 },
    { 0x5bc8094b, 91 },
    { 0xeaa8094a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f80956, 46 },
    { 0x7b580957, 98 },
    { 0x64080957, 23 },
    { 0xa9480958, 33 },
    { 0xecc80958, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e8095f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6780962, 98 },
    { 0xe7e80962, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea78096a, 76 },
    { 0x7a18096a, 78 },
    { 0xd618096c, 98 },
    { 0xb568096c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e980973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d80982, 23 },
    { 0, 0 },
    { 0x1ab80984, 43 },
    { 0x5e180985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88380989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a280991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b980994, 33 },
    { 0xbc180995, 61 },
    { 0x8c080995, 84 },
    { 0x34480995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4358099d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a809a3, 90 },
    { 0x184809a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda809a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f809ab, 66 },
    { 0x146809ac, 84 },
    { 0x7f5809ad, 69 },
    { 0xf5d809ae, 91 },
    { 0xab1809af, 23 },
    { 0xdbd809b0, 69 },
    { 0, 0 },
    { 0xfda809b2, 77 },
    { 0xb14809b3, 55 },
    { 0xade809b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0809ba, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48809be, 33 },
    { 0x1d7809bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x794809c2, 78 },
    { 0, 0 },
    { 0xf94809c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75809ca, 93 },
    { 0x8a1809cb, 33 },
    { 0xbcc809ca, 40 },
    { 0x9a5809cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414809d9, 34 },
    { 0xea6809da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508809e5, 48 },
    { 0x2bc809e6, 98 },
    { 0, 0 },
    { 0x89e809e8, 52 },
    { 0xc52809e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5809ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c809f1, 38 },
    { 0x6d2809f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960809f8, 61 },
    { 0x128809f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1809fd, 33 },
    { 0x16e809fe, 61 },
    { 0x49e809ff, 55 },
    { 0xa33809ff, 93 },
    { 0xe2880a01, 55 },
    { 0xe8280a02, 33 },
    { 0x55980a01, 81 },
    { 0xae680a01, 100 },
    { 0xeb780a01, 61 },
    { 0xe5c80a01, 80 },
    { 0x3e580a07, 98 },
    { 0x74080a08, 76 },
    { 0xb6e80a09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e80a0c, 33 },
    { 0xcad80a0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf580a13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b980a18, 55 },
    { 0, 0 },
    { 0xfc780a1a, 95 },
    { 0xc2880a1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5680a1f, 33 },
    { 0x5b180a20, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x62480a23, 23 },
    { 0xca080a23, 98 },
    { 0x95180a23, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a80a34, 90 },
    { 0xe8f80a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee80a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe980a3d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b80a41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac80a4d, 33 },
    { 0xa2680a4e, 98 },
    { 0xffa80a4d, 34 },
    { 0x62f80a50, 48 },
    { 0x12e80a51, 98 },
    { 0x6ff80a52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3780a57, 61 },
    { 0x4cd80a57, 33 },
    { 0, 0 },
    { 0x35a80a5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee80a5d, 95 },
    { 0xc5c80a5e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4080a61, 33 },
    { 0x6a580a62, 77 },
    { 0x2b80a63, 66 },
    { 0, 0 },
    { 0x2dd80a65, 61 },
    { 0x8be80a65, 61 },
    { 0xc6980a67, 52 },
    { 0x6d380a65, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3580a6c, 33 },
    { 0x16780a6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19980a77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c80a7b, 90 },
    { 0xf5480a7b, 33 },
    { 0xd280a7b, 33 },
    { 0xd1c80a7e, 40 },
    { 0x62180a7f, 95 },
    { 0xbee80a7f, 66 },
    { 0xd5580a80, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f80a85, 98 },
    { 0, 0 },
    { 0xe1280a87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d80a8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7480a91, 33 },
    { 0, 0 },
    { 0xbe880a93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac80a97, 90 },
    { 0x21580a98, 34 },
    { 0, 0 },
    { 0xb2080a9a, 66 },
    { 0, 0 },
    { 0x9c780a9c, 90 },
    { 0x96780a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd80aa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b380aa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af80aae, 90 },
    { 0, 0 },
    { 0x580ab0, 99 },
    { 0xce980ab1, 90 },
    { 0x67f80ab2, 61 },
    { 0xa4a80ab3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e80ab6, 61 },
    { 0xd480ab6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c80abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed780ac6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15780ace, 61 },
    { 0x63380acf, 98 },
    { 0xbb980ad0, 90 },
    { 0x24980ace, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad280ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c80ada, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5180adf, 23 },
    { 0, 0 },
    { 0x7e780ae1, 33 },
    { 0x2b680ae1, 33 },
    { 0, 0 },
    { 0x4480ae4, 61 },
    { 0x82680ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3380aea, 95 },
    { 0x5c080aea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1580aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f80af2, 61 },
    { 0xad80af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25180af6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c80aff, 46 },
    { 0, 0 },
    { 0xf0d80b01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a80b0e, 33 },
    { 0x1e380b0f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x31980b12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32480b16, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x35480b19, 55 },
    { 0, 0 },
    { 0xc9480b1b, 67 },
    { 0, 0 },
    { 0xf0580b1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7180b20, 33 },
    { 0, 0 },
    { 0x5c780b22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f080b26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0680b2d, 61 },
    { 0x13e80b2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9780b31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2080b3a, 77 },
    { 0x9ca80b3b, 101 },
    { 0xbef80b3c, 91 },
    { 0xba280b3b, 48 },
    { 0x24b80b3b, 61 },
    { 0, 0 },
    { 0x2ed80b40, 23 },
    { 0x85880b41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde80b45, 95 },
    { 0x97180b46, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf80b49, 93 },
    { 0x34080b4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b680b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7480b52, 90 },
    { 0xde380b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33480b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71380b5c, 90 },
    { 0xd4880b5c, 61 },
    { 0, 0 },
    { 0xd4f80b5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d80b64, 66 },
    { 0x7b580b65, 33 },
    { 0x11380b66, 100 },
    { 0x45580b67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83880b6f, 95 },
    { 0xda480b70, 43 },
    { 0x84680b70, 100 },
    { 0x80080b72, 95 },
    { 0x7b880b72, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42980b77, 95 },
    { 0x30480b78, 76 },
    { 0xec580b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc480b80, 94 },
    { 0xf0e80b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb880b87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d680b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0480b98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae580b9b, 61 },
    { 0xc4c80b9b, 33 },
    { 0, 0 },
    { 0x78c80b9e, 55 },
    { 0x43b80b9f, 33 },
    { 0x87680b9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e80ba3, 94 },
    { 0x4ce80ba4, 80 },
    { 0xf4880ba5, 23 },
    { 0xe2a80ba5, 90 },
    { 0x99b80ba4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef80baf, 33 },
    { 0x6c580bb0, 90 },
    { 0, 0 },
    { 0xeb180bb2, 61 },
    { 0xc7a80bb3, 43 },
    { 0x89080bb4, 87 },
    { 0xd2180bb5, 100 },
    { 0xa2780bb3, 90 },
    { 0x66180bb7, 95 },
    { 0xa4680bb5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd680bbd, 23 },
    { 0xaa180bbe, 23 },
    { 0, 0 },
    { 0x28b80bc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5480bc4, 68 },
    { 0, 0 },
    { 0x3a480bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86180bcd, 23 },
    { 0, 0 },
    { 0x65080bcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18880bd5, 17 },
    { 0x3d980bd5, 66 },
    { 0x18180bd7, 95 },
    { 0x60d80bd6, 90 },
    { 0x9f880bd7, 100 },
    { 0x83880bda, 37 },
    { 0xdd780bda, 46 },
    { 0xb3780bdc, 81 },
    { 0xd3a80bdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f80be0, 67 },
    { 0, 0 },
    { 0xf9080be2, 76 },
    { 0x6de80be3, 90 },
    { 0, 0 },
    { 0xfab80be5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d80be8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90280bed, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb280bf3, 95 },
    { 0xfe180bf3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e80bf9, 76 },
    { 0x81f80bfa, 76 },
    { 0x44c80bfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba80bfe, 61 },
    { 0xc280bff, 90 },
    { 0, 0 },
    { 0xe7e80c01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb80c04, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e80c07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a380c0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd080c0e, 52 },
    { 0, 0 },
    { 0x6780c10, 95 },
    { 0, 0 },
    { 0x89b80c12, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15280c1a, 38 },
    { 0x25380c1b, 90 },
    { 0x45780c1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf80c20, 23 },
    { 0xd8380c21, 61 },
    { 0x57680c20, 33 },
    { 0x87d80c23, 43 },
    { 0xcc380c23, 55 },
    { 0xa8980c25, 90 },
    { 0xb7e80c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4880c29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f80c2c, 61 },
    { 0, 0 },
    { 0x25c80c2e, 66 },
    { 0x42580c2e, 98 },
    { 0xa3780c30, 87 },
    { 0, 0 },
    { 0x79f80c32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e80c37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6780c3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0380c3f, 23 },
    { 0, 0 },
    { 0xd6880c41, 76 },
    { 0xd780c41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac80c45, 34 },
    { 0x5db80c46, 90 },
    { 0x2d680c47, 33 },
    { 0x53680c45, 34 },
    { 0x7ed80c48, 76 },
    { 0xe3480c47, 33 },
    { 0, 0 },
    { 0xe3f80c4c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f80c53, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9180c56, 23 },
    { 0xaee80c57, 90 },
    { 0, 0 },
    { 0xcea80c59, 90 },
    { 0, 0 },
    { 0x6a880c5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f80c64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7780c67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4580c6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56280c6e, 85 },
    { 0xd0980c6e, 90 },
    { 0, 0 },
    { 0x5fb80c71, 90 },
    { 0, 0 },
    { 0x1c980c73, 43 },
    { 0x80b80c74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0080c78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc80c7b, 95 },
    { 0, 0 },
    { 0xd1e80c7d, 95 },
    { 0x49d80c7e, 68 },
    { 0xaeb80c7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5180c83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e880c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1580c8b, 76 },
    { 0, 0 },
    { 0x25780c8d, 43 },
    { 0, 0 },
    { 0xb2980c8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xada80c92, 82 },
    { 0x5b580c92, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a80c98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf80ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb80ca3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9380ca6, 23 },
    { 0xd1d80ca6, 61 },
    { 0xcf480ca7, 98 },
    { 0x86280ca9, 76 },
    { 0x58980caa, 76 },
    { 0xda280caa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4380cb0, 34 },
    { 0x36380cb1, 95 },
    { 0x76b80cb1, 97 },
    { 0xeab80cb3, 90 },
    { 0x84d80cb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e80cb8, 46 },
    { 0, 0 },
    { 0xfb280cba, 90 },
    { 0xc1180cbb, 68 },
    { 0xd6180cbc, 76 },
    { 0x2aa80cbc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13380cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6080cc5, 95 },
    { 0x9ca80cc6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d880ccd, 33 },
    { 0x12a80cce, 23 },
    { 0x62180ccf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3080cd2, 93 },
    { 0, 0 },
    { 0xd480cd4, 97 },
    { 0, 0 },
    { 0x2080cd6, 67 },
    { 0x5a680cd7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x57980cda, 61 },
    { 0xb7080cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d780ce2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6680ce5, 77 },
    { 0x8bd80ce6, 98 },
    { 0xf8c80ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29280ced, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7080cf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9780cf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d80cfa, 33 },
    { 0x94e80cfb, 33 },
    { 0xffe80cfb, 98 },
    { 0x28d80cfd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5580d02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b80d05, 99 },
    { 0, 0 },
    { 0x67f80d07, 65 },
    { 0x69780d07, 90 },
    { 0, 0 },
    { 0xb9580d0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41880d0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec80d12, 8 },
    { 0, 0 },
    { 0x30180d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db80d17, 95 },
    { 0xa780d18, 61 },
    { 0x69680d19, 33 },
    { 0xf1a80d1a, 67 },
    { 0xd6880d1b, 87 },
    { 0, 0 },
    { 0x5d80d1d, 66 },
    { 0x4a680d1d, 61 },
    { 0x6f380d1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe380d22, 95 },
    { 0x5b280d23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd980d26, 43 },
    { 0x91b80d26, 90 },
    { 0x1ae80d28, 98 },
    { 0x37b80d27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e580d2e, 76 },
    { 0x51080d2f, 34 },
    { 0, 0 },
    { 0x66880d31, 61 },
    { 0x1b980d31, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa80d3a, 90 },
    { 0x68880d3b, 68 },
    { 0, 0 },
    { 0xee280d3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e80d43, 33 },
    { 0xc7b80d44, 43 },
    { 0xac080d45, 52 },
    { 0, 0 },
    { 0x68e80d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49880d4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b80d53, 76 },
    { 0, 0 },
    { 0x33a80d55, 23 },
    { 0xa2180d55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9180d63, 90 },
    { 0, 0 },
    { 0xd1b80d65, 33 },
    { 0xd5a80d66, 90 },
    { 0xe9a80d67, 75 },
    { 0x5480d66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f80d6b, 55 },
    { 0x13680d6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68580d71, 76 },
    { 0, 0 },
    { 0xe0c80d73, 33 },
    { 0x81e80d73, 43 },
    { 0xa4b80d73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a380d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d780d7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25480d7f, 43 },
    { 0x22c80d7f, 46 },
    { 0, 0 },
    { 0x6080d82, 94 },
    { 0xb2f80d83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9880d87, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60980d8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3480d91, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6880d99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34880d9c, 76 },
    { 0xbbd80d9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9780da0, 84 },
    { 0xe0f80da1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda380da5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83480da8, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c980dab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a380dae, 90 },
    { 0, 0 },
    { 0x82580db0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f80db3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc80db6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21580dbb, 23 },
    { 0x4780dbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99780dc2, 56 },
    { 0, 0 },
    { 0x3c280dc4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9680dca, 76 },
    { 0x2d980dcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff080dce, 34 },
    { 0xfec80dce, 100 },
    { 0xba480dd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2080dd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c080dd9, 61 },
    { 0, 0 },
    { 0x15c80ddb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39580ddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8780de3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a80de7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2280dea, 91 },
    { 0x2e380deb, 23 },
    { 0x86980deb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f80df0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f80df3, 77 },
    { 0x36080df4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c80dff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c80e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6780e07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b80e0b, 23 },
    { 0xb2e80e0c, 78 },
    { 0x3c680e0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30280e11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b980e15, 98 },
    { 0, 0 },
    { 0x6db80e17, 46 },
    { 0x7cf80e17, 98 },
    { 0x5c080e17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4080e22, 98 },
    { 0xd4780e22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc80e26, 55 },
    { 0, 0 },
    { 0xc2680e28, 99 },
    { 0, 0 },
    { 0x7cd80e2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b880e2d, 98 },
    { 0xb4f80e2e, 22 },
    { 0xb8f80e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac80e32, 34 },
    { 0x5ca80e32, 95 },
    { 0x76680e32, 75 },
    { 0x44180e35, 61 },
    { 0x7fd80e36, 33 },
    { 0x89480e37, 34 },
    { 0x88780e38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec80e3b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e980e3e, 87 },
    { 0xe0a80e3f, 95 },
    { 0, 0 },
    { 0x5480e41, 23 },
    { 0x68d80e42, 95 },
    { 0xe9680e41, 23 },
    { 0x96480e42, 95 },
    { 0x91580e43, 87 },
    { 0, 0 },
    { 0xb1580e47, 98 },
    { 0x31e80e47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1480e4c, 87 },
    { 0x5e180e4c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d80e55, 91 },
    { 0xaa980e56, 40 },
    { 0x7ab80e56, 33 },
    { 0, 0 },
    { 0x52c80e59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d80e5c, 91 },
    { 0xc1b80e5d, 98 },
    { 0x42e80e5c, 33 },
    { 0, 0 },
    { 0x69680e60, 78 },
    { 0, 0 },
    { 0xfd580e62, 100 },
    { 0x65280e62, 61 },
    { 0x67e80e64, 93 },
    { 0x49b80e63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0280e6b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57580e6f, 76 },
    { 0, 0 },
    { 0x84880e71, 98 },
    { 0x63980e72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e480e78, 91 },
    { 0, 0 },
    { 0x94c80e7a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73580e80, 77 },
    { 0x14280e81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c580e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e80e9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d580ea0, 90 },
    { 0x96280ea0, 78 },
    { 0x6e980ea0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83180ea5, 91 },
    { 0x43580ea6, 76 },
    { 0xe7780ea5, 34 },
    { 0x6b980ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb80eae, 52 },
    { 0, 0 },
    { 0x86480eb0, 95 },
    { 0xfa880eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71280eb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c080eb7, 100 },
    { 0xa8980eb7, 77 },
    { 0xa2180eb9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52280ebd, 46 },
    { 0, 0 },
    { 0x10080ebf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db80ec5, 90 },
    { 0, 0 },
    { 0xee80ec7, 98 },
    { 0, 0 },
    { 0x6de80ec9, 52 },
    { 0xedf80eca, 33 },
    { 0xb8880eca, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef780ed2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4480ed5, 46 },
    { 0, 0 },
    { 0x25e80ed7, 99 },
    { 0x19880ed8, 33 },
    { 0xed680ed9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d80ede, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe380ee1, 66 },
    { 0xe6980ee2, 55 },
    { 0, 0 },
    { 0x47080ee4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e780ee7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc680eea, 98 },
    { 0, 0 },
    { 0x51280eec, 55 },
    { 0, 0 },
    { 0x98a80eee, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd80ef1, 33 },
    { 0x6db80ef2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81580ef9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51980f01, 98 },
    { 0x7c80f02, 95 },
    { 0xbca80f02, 87 },
    { 0, 0 },
    { 0xd9f80f05, 40 },
    { 0x4d180f06, 61 },
    { 0xcdd80f06, 90 },
    { 0, 0 },
    { 0xd6580f09, 17 },
    { 0x2480f0a, 90 },
    { 0xeeb80f0b, 100 },
    { 0x59680f0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab80f0f, 90 },
    { 0x6e980f0f, 36 },
    { 0x96880f11, 93 },
    { 0x96280f12, 46 },
    { 0, 0 },
    { 0xd1780f14, 61 },
    { 0x68080f15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a80f19, 23 },
    { 0xb7980f1a, 85 },
    { 0, 0 },
    { 0xbc880f1c, 33 },
    { 0x28080f1c, 23 },
    { 0, 0 },
    { 0xea580f1f, 76 },
    { 0x85f80f20, 23 },
    { 0x65f80f21, 43 },
    { 0, 0 },
    { 0x15b80f23, 76 },
    { 0xc7080f23, 33 },
    { 0x61580f25, 98 },
    { 0x81280f26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a280f29, 93 },
    { 0x3e180f2a, 91 },
    { 0x17b80f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c80f2e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e80f33, 34 },
    { 0, 0 },
    { 0x1a580f35, 78 },
    { 0xa7880f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25880f3b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d80f41, 33 },
    { 0x97980f41, 90 },
    { 0xef080f43, 65 },
    { 0x72580f42, 76 },
    { 0xb4680f41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c80f4c, 33 },
    { 0x34a80f4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c80f54, 90 },
    { 0xfbf80f54, 55 },
    { 0x91a80f54, 34 },
    { 0x21f80f57, 95 },
    { 0xb2180f58, 34 },
    { 0x69680f59, 95 },
    { 0x91080f58, 61 },
    { 0xc6c80f54, 95 },
    { 0xa8880f5c, 98 },
    { 0, 0 },
    { 0x5e280f5e, 33 },
    { 0xef180f5f, 61 },
    { 0, 0 },
    { 0x6f380f61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e80f64, 77 },
    { 0x2c480f65, 90 },
    { 0, 0 },
    { 0xf0480f67, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c80f6c, 98 },
    { 0xc4980f6d, 98 },
    { 0x3b280f6e, 100 },
    { 0x25d80f6f, 33 },
    { 0xdce80f6f, 85 },
    { 0xcf880f71, 95 },
    { 0x7d980f71, 95 },
    { 0x4d280f70, 90 },
    { 0x7ff80f6f, 98 },
    { 0x80e80f75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa80f79, 52 },
    { 0, 0 },
    { 0xec480f7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f180f81, 64 },
    { 0, 0 },
    { 0xbd480f83, 46 },
    { 0x78380f84, 66 },
    { 0, 0 },
    { 0xfcf80f86, 55 },
    { 0, 0 },
    { 0xfac80f88, 52 },
    { 0, 0 },
    { 0x41880f8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c80f8d, 68 },
    { 0x24380f8d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d80f92, 61 },
    { 0x12380f92, 90 },
    { 0x67b80f92, 33 },
    { 0x45280f95, 98 },
    { 0x3380f95, 91 },
    { 0xd3d80f95, 95 },
    { 0xaf780f98, 53 },
    { 0x5b180f98, 84 },
    { 0xa8a80f9a, 34 },
    { 0x85c80f96, 23 },
    { 0x8c780f98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a480fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0180fa6, 23 },
    { 0xd5480fa7, 9 },
    { 0xf380fa8, 99 },
    { 0x2d580fa8, 90 },
    { 0, 0 },
    { 0xe4880fab, 49 },
    { 0xbfb80fac, 14 },
    { 0xb7b80fad, 95 },
    { 0xac080fae, 46 },
    { 0x46480fae, 77 },
    { 0x6ad80fb0, 61 },
    { 0x77680fae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b80fb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5880fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14980fbc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c80fc2, 55 },
    { 0x7ef80fc3, 95 },
    { 0x19480fc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f80fc7, 90 },
    { 0, 0 },
    { 0x3a480fc9, 88 },
    { 0xec080fc9, 34 },
    { 0x22c80fc9, 65 },
    { 0x85580fcc, 90 },
    { 0xb3780fcd, 90 },
    { 0x8c780fcd, 98 },
    { 0xa7380fcf, 61 },
    { 0x64380fcf, 95 },
    { 0xfd80fcf, 98 },
    { 0x2b80fd2, 23 },
    { 0x47e80fd3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9980fd7, 98 },
    { 0, 0 },
    { 0xda980fd9, 43 },
    { 0xa2a80fda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9280fe2, 33 },
    { 0xdc780fe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b980fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed80fea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec680fed, 61 },
    { 0, 0 },
    { 0x6d980fef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7080ff2, 98 },
    { 0xdd980ff3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb80ff8, 84 },
    { 0x8480ff8, 43 },
    { 0x2ad80ffa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae81002, 66 },
    { 0, 0 },
    { 0xf7a81004, 33 },
    { 0x62d81005, 90 },
    { 0x1f381004, 33 },
    { 0, 0 },
    { 0x67781008, 33 },
    { 0, 0 },
    { 0xf158100a, 80 },
    { 0x7d78100b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb8100e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b81012, 90 },
    { 0, 0 },
    { 0x72c81014, 33 },
    { 0xe0081015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8448101a, 95 },
    { 0x8338101b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d8101f, 98 },
    { 0x7a281020, 95 },
    { 0, 0 },
    { 0xf1c81022, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde081025, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x45681028, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e8102b, 95 },
    { 0x4a8102b, 95 },
    { 0x7458102d, 91 },
    { 0, 0 },
    { 0xaaf8102f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef681035, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf181038, 95 },
    { 0x92b81039, 43 },
    { 0, 0 },
    { 0x9b98103b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce8103e, 90 },
    { 0, 0 },
    { 0x66d81040, 40 },
    { 0x24b81041, 33 },
    { 0xe0781040, 33 },
    { 0xaa381043, 91 },
    { 0, 0 },
    { 0x8381045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc8104a, 66 },
    { 0xacc8104a, 75 },
    { 0, 0 },
    { 0xdb08104d, 61 },
    { 0x9698104e, 66 },
    { 0x39e8104e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d281053, 98 },
    { 0, 0 },
    { 0x8b381055, 76 },
    { 0, 0 },
    { 0x8fc81057, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3118105e, 40 },
    { 0, 0 },
    { 0x76681060, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f981064, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f81069, 43 },
    { 0x2d98106a, 90 },
    { 0x1658106a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b81070, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a481073, 90 },
    { 0, 0 },
    { 0xeab81075, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x84381078, 61 },
    { 0xbd281078, 33 },
    { 0xdf08107a, 98 },
    { 0x95e8107a, 90 },
    { 0xe398107c, 23 },
    { 0xd518107d, 23 },
    { 0xee28107e, 61 },
    { 0, 0 },
    { 0xeab81080, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef681084, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba81088, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac481095, 76 },
    { 0xc5281096, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8848109a, 52 },
    { 0x1508109b, 33 },
    { 0x8ec8109c, 34 },
    { 0x5d38109d, 95 },
    { 0x68e8109e, 33 },
    { 0x7be8109e, 33 },
    { 0x1bc8109e, 61 },
    { 0x6fe810a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d810a4, 90 },
    { 0x618810a5, 76 },
    { 0, 0 },
    { 0x486810a7, 23 },
    { 0xe14810a8, 76 },
    { 0xda810a9, 87 },
    { 0xd5e810a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895810b0, 61 },
    { 0xeb1810b1, 85 },
    { 0xd78810b0, 61 },
    { 0, 0 },
    { 0x12810b4, 55 },
    { 0, 0 },
    { 0xd89810b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985810bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280810c0, 100 },
    { 0, 0 },
    { 0x4c3810c2, 93 },
    { 0xb73810c3, 61 },
    { 0, 0 },
    { 0xd59810c5, 98 },
    { 0, 0 },
    { 0x105810c7, 34 },
    { 0, 0 },
    { 0x6b1810c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d810cc, 66 },
    { 0xe11810cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a810d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52810d6, 95 },
    { 0x53c810d7, 55 },
    { 0x189810d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63810dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199810e8, 87 },
    { 0xe1a810e9, 55 },
    { 0xddc810ea, 64 },
    { 0xfac810eb, 98 },
    { 0, 0 },
    { 0xe4a810ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x296810f0, 33 },
    { 0xc36810f0, 90 },
    { 0, 0 },
    { 0xce0810f3, 23 },
    { 0x1f2810f4, 76 },
    { 0x71e810f4, 78 },
    { 0, 0 },
    { 0xfe4810f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d810ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x82881102, 81 },
    { 0x7c681103, 91 },
    { 0xae481102, 40 },
    { 0x74b81103, 90 },
    { 0xc4081102, 68 },
    { 0x8b581107, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a18110e, 61 },
    { 0xcea8110f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6781112, 90 },
    { 0x7c081113, 90 },
    { 0x42481114, 93 },
    { 0x44e81115, 95 },
    { 0x14781115, 90 },
    { 0x6f381112, 84 },
    { 0, 0 },
    { 0xdbd81119, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9181120, 77 },
    { 0x57b81121, 23 },
    { 0xd2581122, 55 },
    { 0xea581121, 87 },
    { 0, 0 },
    { 0x84b81125, 90 },
    { 0x8a81125, 2 },
    { 0, 0 },
    { 0x40a81128, 27 },
    { 0, 0 },
    { 0x55d8112a, 33 },
    { 0x7c88112b, 76 },
    { 0x8768112b, 95 },
    { 0x9e18112a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d981131, 33 },
    { 0x99081132, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x92281135, 95 },
    { 0x281136, 61 },
    { 0xedb81137, 69 },
    { 0xea181136, 23 },
    { 0x4381139, 43 },
    { 0xf928113a, 17 },
    { 0, 0 },
    { 0x3498113c, 55 },
    { 0xb498113d, 77 },
    { 0x4b98113e, 20 },
    { 0x7f48113d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca81142, 91 },
    { 0x3b81142, 95 },
    { 0, 0 },
    { 0xf7f81145, 99 },
    { 0xa2781146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8114c, 99 },
    { 0x3d48114d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e981152, 90 },
    { 0, 0 },
    { 0xaa581154, 91 },
    { 0, 0 },
    { 0x58c81156, 62 },
    { 0xf3981157, 99 },
    { 0, 0 },
    { 0x59f81159, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc278115f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13881163, 23 },
    { 0x9a181163, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54681168, 46 },
    { 0x58681169, 90 },
    { 0xc268116a, 95 },
    { 0xc6881168, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c81170, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f81173, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb81177, 78 },
    { 0, 0 },
    { 0x7a781179, 90 },
    { 0, 0 },
    { 0x6938117b, 76 },
    { 0, 0 },
    { 0x6208117d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f81180, 76 },
    { 0x68081180, 98 },
    { 0xb6b81181, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3181187, 61 },
    { 0, 0 },
    { 0xaf181189, 95 },
    { 0x16e8118a, 81 },
    { 0xd188118b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac8118e, 90 },
    { 0x99c8118f, 90 },
    { 0x3381190, 33 },
    { 0x87d81191, 55 },
    { 0x2c981191, 98 },
    { 0, 0 },
    { 0x9c481194, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa138119d, 81 },
    { 0, 0 },
    { 0x8a18119f, 23 },
    { 0x6e38119f, 95 },
    { 0, 0 },
    { 0xe64811a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x694811a5, 78 },
    { 0xfcb811a5, 98 },
    { 0, 0 },
    { 0xd6811a8, 95 },
    { 0x830811a9, 33 },
    { 0x966811a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d811af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c811b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf811b6, 17 },
    { 0xdc4811b7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7811bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b811be, 61 },
    { 0xe71811be, 61 },
    { 0xa86811be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b811c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9811cd, 84 },
    { 0x3a3811ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9811d1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3811d4, 98 },
    { 0xcbb811d5, 87 },
    { 0, 0 },
    { 0xea5811d7, 98 },
    { 0x2ef811d8, 98 },
    { 0xc89811d7, 100 },
    { 0xfb7811d9, 98 },
    { 0x3a7811db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17811df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16811e2, 98 },
    { 0x320811e3, 90 },
    { 0, 0 },
    { 0xdca811e5, 90 },
    { 0x6c811e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e811e9, 33 },
    { 0x585811ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52811ef, 98 },
    { 0xb06811f0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4811f3, 90 },
    { 0, 0 },
    { 0x737811f5, 95 },
    { 0x95811f6, 95 },
    { 0xa3a811f5, 76 },
    { 0xe7b811f8, 76 },
    { 0, 0 },
    { 0x194811fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573811fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c81202, 98 },
    { 0x60981202, 95 },
    { 0, 0 },
    { 0x23481205, 76 },
    { 0xd681206, 78 },
    { 0x44d81205, 76 },
    { 0, 0 },
    { 0x59081209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c8120c, 85 },
    { 0xe3b8120c, 95 },
    { 0x6268120e, 43 },
    { 0x8f18120f, 90 },
    { 0x2908120f, 76 },
    { 0xbbf8120f, 98 },
    { 0xb898120d, 46 },
    { 0x3f88120e, 33 },
    { 0xb588120e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc208121a, 85 },
    { 0x6348121b, 76 },
    { 0x7008121c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f581220, 23 },
    { 0x4bd81220, 84 },
    { 0x3b781222, 55 },
    { 0xaa81221, 23 },
    { 0x9a881224, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60981227, 90 },
    { 0x71981228, 55 },
    { 0, 0 },
    { 0x9c58122a, 34 },
    { 0, 0 },
    { 0x21b8122c, 33 },
    { 0xe7f8122c, 61 },
    { 0xf698122d, 85 },
    { 0x21f8122c, 90 },
    { 0, 0 },
    { 0x7e81231, 78 },
    { 0x6b281232, 55 },
    { 0xf5d81232, 90 },
    { 0, 0 },
    { 0x38f81235, 76 },
    { 0, 0 },
    { 0x7ed81237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a8123b, 91 },
    { 0x52b8123c, 90 },
    { 0xecd8123b, 36 },
    { 0, 0 },
    { 0x7ba8123f, 90 },
    { 0x78581240, 69 },
    { 0, 0 },
    { 0x1be81242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94781247, 23 },
    { 0x6f981247, 52 },
    { 0x19481249, 91 },
    { 0xa838124a, 94 },
    { 0, 0 },
    { 0x4968124c, 95 },
    { 0xbb8124d, 61 },
    { 0xf908124d, 33 },
    { 0x6d8124f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15881252, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40581255, 98 },
    { 0, 0 },
    { 0x68f81257, 90 },
    { 0x2f381257, 90 },
    { 0xc8181258, 61 },
    { 0, 0 },
    { 0x23a8125b, 90 },
    { 0, 0 },
    { 0x7758125d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2281260, 40 },
    { 0x12981260, 95 },
    { 0x12c81262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad81267, 98 },
    { 0xa7e81267, 90 },
    { 0x52181269, 23 },
    { 0x2f98126a, 84 },
    { 0, 0 },
    { 0x3898126c, 75 },
    { 0xed58126d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e81270, 95 },
    { 0, 0 },
    { 0xf8d81272, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x48681275, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc878127c, 33 },
    { 0, 0 },
    { 0xb08127e, 61 },
    { 0xb8b8127f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8181287, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2768128b, 90 },
    { 0, 0 },
    { 0xd4b8128d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f81292, 98 },
    { 0x60c81293, 23 },
    { 0x8d781294, 90 },
    { 0x9f081295, 85 },
    { 0x89881293, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae38129f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x955812a2, 65 },
    { 0x219812a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c812a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb812b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5812b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89812bb, 95 },
    { 0x87f812bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c812c2, 33 },
    { 0x967812c3, 33 },
    { 0, 0 },
    { 0x177812c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0812c9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7812d4, 78 },
    { 0x6da812d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x152812d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da812db, 88 },
    { 0xc3f812dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c812df, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265812ea, 61 },
    { 0xcc3812eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9812ee, 43 },
    { 0x808812ef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c812f5, 98 },
    { 0x4c0812f5, 55 },
    { 0x10812f7, 90 },
    { 0x899812f8, 66 },
    { 0x34c812f7, 33 },
    { 0x97f812f8, 91 },
    { 0xe1d812fa, 95 },
    { 0xfe812fc, 23 },
    { 0x5e9812fc, 90 },
    { 0xf92812f6, 87 },
    { 0x509812ff, 61 },
    { 0x37f81300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4881303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df81307, 98 },
    { 0xe3481307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0281312, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681319, 90 },
    { 0, 0 },
    { 0xfc68131b, 43 },
    { 0, 0 },
    { 0x5968131d, 34 },
    { 0, 0 },
    { 0xac8131f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e81322, 76 },
    { 0x73e81323, 90 },
    { 0x47e81322, 98 },
    { 0x87d81323, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb918132b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff981330, 90 },
    { 0xfca81331, 90 },
    { 0, 0 },
    { 0xb881333, 52 },
    { 0x41281334, 90 },
    { 0, 0 },
    { 0x77081336, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8133c, 46 },
    { 0x9ca8133d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e81345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb81348, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd858134c, 17 },
    { 0x78134d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b81350, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc81357, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x9198135a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f381361, 98 },
    { 0x7c681362, 55 },
    { 0x30881363, 55 },
    { 0x92b81362, 33 },
    { 0xb1a81365, 33 },
    { 0, 0 },
    { 0x71581367, 90 },
    { 0, 0 },
    { 0x43a81369, 90 },
    { 0, 0 },
    { 0x2f78136b, 67 },
    { 0x47c8136c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b81372, 61 },
    { 0x5281372, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83581379, 98 },
    { 0x95c81379, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed8137d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7881381, 98 },
    { 0x7fd81381, 61 },
    { 0xb7581382, 98 },
    { 0, 0 },
    { 0x28a81385, 95 },
    { 0x2a681386, 61 },
    { 0x65e81387, 67 },
    { 0xd2d81387, 98 },
    { 0x89281388, 34 },
    { 0xfe081387, 76 },
    { 0xb3e81385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a781393, 23 },
    { 0x66a81393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7728139a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e813a6, 61 },
    { 0, 0 },
    { 0x86c813a8, 61 },
    { 0x1bc813a9, 95 },
    { 0x7c7813aa, 34 },
    { 0x571813ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x424813ae, 61 },
    { 0x542813af, 55 },
    { 0xdc813af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e813b5, 75 },
    { 0, 0 },
    { 0x98f813b7, 98 },
    { 0x8d6813b8, 98 },
    { 0, 0 },
    { 0x21b813ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83813be, 42 },
    { 0xe35813be, 84 },
    { 0xb3b813be, 43 },
    { 0x4df813c1, 56 },
    { 0x185813c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3813cb, 78 },
    { 0x423813cc, 69 },
    { 0x79a813cc, 75 },
    { 0x87c813cb, 98 },
    { 0xed3813cf, 85 },
    { 0, 0 },
    { 0x7c8813d1, 90 },
    { 0x6c6813d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee813d5, 43 },
    { 0, 0 },
    { 0x697813d7, 84 },
    { 0, 0 },
    { 0x3d8813d9, 61 },
    { 0x9cd813da, 76 },
    { 0x653813da, 96 },
    { 0x292813dc, 95 },
    { 0, 0 },
    { 0xb58813de, 87 },
    { 0x820813df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f813e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b813ea, 52 },
    { 0x2c9813ea, 33 },
    { 0x328813ec, 23 },
    { 0x4c2813ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07813f2, 90 },
    { 0xe7a813f3, 68 },
    { 0x3d813f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52813f9, 33 },
    { 0xb61813f9, 33 },
    { 0x78f813fa, 40 },
    { 0xccf813fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6381404, 55 },
    { 0xe7c81405, 93 },
    { 0xb7081406, 94 },
    { 0x2a481406, 76 },
    { 0x37881405, 87 },
    { 0xe1c81409, 23 },
    { 0x1ba81405, 93 },
    { 0, 0 },
    { 0xd308140c, 37 },
    { 0x5148140d, 61 },
    { 0x4908140e, 52 },
    { 0x29c8140e, 76 },
    { 0xe9b8140e, 95 },
    { 0xc4a8140e, 23 },
    { 0, 0 },
    { 0x71281413, 33 },
    { 0, 0 },
    { 0xd8381415, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a881418, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b8141e, 76 },
    { 0x6348141f, 33 },
    { 0x2738141f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x8381423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f81427, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1788142c, 98 },
    { 0xe978142d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce81431, 95 },
    { 0x92481432, 95 },
    { 0xbd381433, 80 },
    { 0, 0 },
    { 0xe0881435, 23 },
    { 0x13581435, 23 },
    { 0x1fe81437, 46 },
    { 0xa7a81438, 46 },
    { 0, 0 },
    { 0x4f58143a, 55 },
    { 0x7648143b, 33 },
    { 0, 0 },
    { 0x41f8143d, 56 },
    { 0xdf48143e, 65 },
    { 0x98e8143f, 93 },
    { 0, 0 },
    { 0xe5381441, 61 },
    { 0x3a581441, 99 },
    { 0xb2981443, 33 },
    { 0, 0 },
    { 0xc2081445, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b8144a, 97 },
    { 0xd0f8144a, 84 },
    { 0, 0 },
    { 0x9ec8144d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e681451, 95 },
    { 0x17281452, 90 },
    { 0xfa181452, 61 },
    { 0x77a81454, 33 },
    { 0, 0 },
    { 0x3c381456, 34 },
    { 0x9ce81457, 33 },
    { 0, 0 },
    { 0x8e381459, 90 },
    { 0x4428145a, 76 },
    { 0xfb68145b, 66 },
    { 0x5028145c, 90 },
    { 0xeb68145d, 76 },
    { 0xb28145d, 95 },
    { 0xd188145e, 95 },
    { 0xab681460, 47 },
    { 0x2881461, 43 },
    { 0, 0 },
    { 0x2a381463, 87 },
    { 0x54e81464, 23 },
    { 0, 0 },
    { 0x8c781466, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd18146b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9518146f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69281473, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be81476, 17 },
    { 0, 0 },
    { 0xcdc81478, 97 },
    { 0xfcb81479, 90 },
    { 0x9bf8147a, 77 },
    { 0xa5a8147b, 75 },
    { 0, 0 },
    { 0xbb98147d, 99 },
    { 0x8cc8147e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd81484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8148b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a81490, 61 },
    { 0x5a681491, 33 },
    { 0x4681491, 33 },
    { 0x92781493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7481496, 76 },
    { 0xd5e81497, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5218149a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1814a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f814a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31814ad, 66 },
    { 0, 0 },
    { 0x2cc814af, 55 },
    { 0, 0 },
    { 0xa8f814b1, 56 },
    { 0, 0 },
    { 0x236814b3, 33 },
    { 0xb8c814b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392814b8, 76 },
    { 0x9e3814b9, 78 },
    { 0x814814b9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc814bf, 93 },
    { 0, 0 },
    { 0xcb7814c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df814c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88814ca, 61 },
    { 0, 0 },
    { 0xbf8814cc, 87 },
    { 0x2f3814cd, 85 },
    { 0xdd7814cc, 33 },
    { 0xec2814cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91814d3, 17 },
    { 0, 0 },
    { 0x23e814d5, 80 },
    { 0, 0 },
    { 0xa7e814d7, 66 },
    { 0xd99814d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b814df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01814e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16814ea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0814ef, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8814f2, 72 },
    { 0x55a814f3, 34 },
    { 0x7c814f3, 77 },
    { 0x24d814f5, 34 },
    { 0, 0 },
    { 0x710814f7, 34 },
    { 0xc3814f8, 61 },
    { 0x1af814f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8281502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35681507, 23 },
    { 0, 0 },
    { 0x29681509, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4081510, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec81519, 33 },
    { 0x17681519, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4081522, 85 },
    { 0x5c281523, 66 },
    { 0x4b781523, 61 },
    { 0xce281525, 73 },
    { 0, 0 },
    { 0x91f81527, 46 },
    { 0x75d81528, 61 },
    { 0x9581528, 91 },
    { 0xd5e8152a, 67 },
    { 0xe138152b, 46 },
    { 0, 0 },
    { 0xb798152d, 90 },
    { 0x2928152e, 78 },
    { 0xff68152d, 61 },
    { 0, 0 },
    { 0x1ec81531, 61 },
    { 0x8c181531, 95 },
    { 0xd5a81533, 87 },
    { 0, 0 },
    { 0x59181535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e181538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a8153f, 95 },
    { 0x9bf8153f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5281543, 61 },
    { 0xf3581544, 33 },
    { 0x56781545, 90 },
    { 0xc0f81544, 93 },
    { 0x12281544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff68154f, 98 },
    { 0x7381550, 76 },
    { 0, 0 },
    { 0xffb81552, 33 },
    { 0x31281553, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f8155b, 90 },
    { 0x318155c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87381561, 66 },
    { 0x50281562, 52 },
    { 0, 0 },
    { 0x39481564, 33 },
    { 0, 0 },
    { 0x4a981566, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xded81569, 55 },
    { 0, 0 },
    { 0x33a8156b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f81571, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10381575, 23 },
    { 0x43281575, 87 },
    { 0x11281576, 77 },
    { 0xc481575, 95 },
    { 0x53b81579, 34 },
    { 0x34981579, 61 },
    { 0xcef81578, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2448157e, 33 },
    { 0x9d8157e, 95 },
    { 0, 0 },
    { 0x92081581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a81585, 98 },
    { 0x84881586, 55 },
    { 0xb0c81585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d98158b, 34 },
    { 0xe558158b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c81595, 98 },
    { 0, 0 },
    { 0x75781597, 46 },
    { 0x50581597, 56 },
    { 0xd4d81599, 93 },
    { 0, 0 },
    { 0xeba8159b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x518159e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f815a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab815aa, 90 },
    { 0, 0 },
    { 0xd90815ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3815b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c815b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6815b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47815bd, 95 },
    { 0xf40815be, 23 },
    { 0, 0 },
    { 0x17b815c0, 23 },
    { 0xd34815c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168815c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500815ca, 75 },
    { 0, 0 },
    { 0x646815cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e815cf, 66 },
    { 0x5b3815cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302815da, 78 },
    { 0, 0 },
    { 0x685815dc, 90 },
    { 0xd16815dd, 23 },
    { 0x6c0815de, 75 },
    { 0x52a815dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x58815e2, 90 },
    { 0, 0 },
    { 0x610815e4, 90 },
    { 0, 0 },
    { 0x14a815e6, 23 },
    { 0, 0 },
    { 0xf76815e8, 80 },
    { 0x9dc815e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad815ee, 95 },
    { 0, 0 },
    { 0xe10815f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71815f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2815f6, 33 },
    { 0x5ca815f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea815fa, 65 },
    { 0xbbd815fa, 61 },
    { 0, 0 },
    { 0xc34815fd, 94 },
    { 0, 0 },
    { 0x9fb815ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64981603, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1081607, 90 },
    { 0x8ab81608, 52 },
    { 0, 0 },
    { 0xce28160a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23681612, 81 },
    { 0x49181613, 98 },
    { 0, 0 },
    { 0x5d081615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb481619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd8161c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8381622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd181627, 76 },
    { 0x42681628, 33 },
    { 0x75e81629, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca88162d, 76 },
    { 0, 0 },
    { 0x4f38162f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e481632, 65 },
    { 0xb5981633, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x54881636, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af8163e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb81643, 90 },
    { 0, 0 },
    { 0x59581645, 46 },
    { 0xeaf81646, 84 },
    { 0, 0 },
    { 0x53981648, 93 },
    { 0xc7981648, 98 },
    { 0, 0 },
    { 0x4098164b, 78 },
    { 0, 0 },
    { 0x9168164d, 98 },
    { 0x9d28164e, 61 },
    { 0x9018164d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11881655, 95 },
    { 0xb9981656, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab8165f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc781664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78166a, 96 },
    { 0, 0 },
    { 0xfba8166c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9608166f, 98 },
    { 0, 0 },
    { 0xb3c81671, 33 },
    { 0x36481672, 99 },
    { 0, 0 },
    { 0x72081674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d81678, 46 },
    { 0, 0 },
    { 0x87a8167a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee8167e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc181682, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7781687, 98 },
    { 0, 0 },
    { 0xb9581689, 23 },
    { 0x7ee8168a, 67 },
    { 0x2bb81689, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28881691, 55 },
    { 0x4281692, 76 },
    { 0, 0 },
    { 0x7ec81694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd881698, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f78169b, 98 },
    { 0xb338169c, 95 },
    { 0, 0 },
    { 0x3228169e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e816a2, 90 },
    { 0x98e816a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8816a7, 61 },
    { 0x4b3816a8, 90 },
    { 0x85c816a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01816ac, 61 },
    { 0x2a3816ad, 23 },
    { 0x109816ae, 23 },
    { 0xbc4816af, 55 },
    { 0x217816b0, 90 },
    { 0x5e6816b1, 43 },
    { 0xf4d816ac, 34 },
    { 0x718816ae, 75 },
    { 0xae1816ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e816ba, 55 },
    { 0x94e816bb, 90 },
    { 0x89d816ba, 34 },
    { 0x90816bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df816c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23816c7, 69 },
    { 0xe1a816c7, 90 },
    { 0, 0 },
    { 0x55d816ca, 23 },
    { 0, 0 },
    { 0xaf9816cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92816d0, 90 },
    { 0x8816d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x311816d4, 76 },
    { 0xb46816d5, 78 },
    { 0xc7d816d5, 43 },
    { 0, 0 },
    { 0x99d816d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09816e2, 76 },
    { 0x28d816e3, 91 },
    { 0, 0 },
    { 0x15a816e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191816f1, 61 },
    { 0x3fc816f2, 17 },
    { 0x919816f2, 33 },
    { 0, 0 },
    { 0x5bf816f5, 96 },
    { 0x665816f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c816ff, 61 },
    { 0, 0 },
    { 0x36c81701, 33 },
    { 0x17681702, 93 },
    { 0, 0 },
    { 0xfb981704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f81707, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa988170b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c81713, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe18171c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe218171f, 61 },
    { 0, 0 },
    { 0x8b081721, 55 },
    { 0, 0 },
    { 0x881723, 90 },
    { 0x2a681723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72281727, 84 },
    { 0xe7681727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1488172b, 95 },
    { 0x10a8172c, 99 },
    { 0x9b28172b, 98 },
    { 0, 0 },
    { 0x9d38172f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3881732, 98 },
    { 0x93581732, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c581736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e681739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d8173c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b81747, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2288174d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa81759, 65 },
    { 0xb6181759, 98 },
    { 0xc888175b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74681764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b68176c, 76 },
    { 0, 0 },
    { 0xfbe8176e, 52 },
    { 0x18c8176f, 61 },
    { 0xe148176e, 61 },
    { 0x6348176e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa8177b, 98 },
    { 0, 0 },
    { 0x6fc8177d, 49 },
    { 0xac8177e, 97 },
    { 0x4ce8177d, 90 },
    { 0x3e581780, 78 },
    { 0x6c881781, 95 },
    { 0x30781782, 55 },
    { 0x9aa81782, 98 },
    { 0, 0 },
    { 0x6bb81785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4728178c, 95 },
    { 0, 0 },
    { 0x1fd8178e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa8179a, 95 },
    { 0xf188179a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235817a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728817aa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x847817ad, 98 },
    { 0, 0 },
    { 0xccf817af, 93 },
    { 0, 0 },
    { 0xcc817b1, 23 },
    { 0xd1817b2, 23 },
    { 0x70817b3, 61 },
    { 0x9e4817b4, 98 },
    { 0x456817b5, 23 },
    { 0, 0 },
    { 0x3c0817b7, 76 },
    { 0x80a817b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a817bb, 95 },
    { 0, 0 },
    { 0x39817bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878817c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9817c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e817cc, 98 },
    { 0xbb9817cd, 98 },
    { 0xf3e817ce, 90 },
    { 0x562817cf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a817d2, 61 },
    { 0x84e817d3, 99 },
    { 0x80c817d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb817da, 88 },
    { 0xf35817da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751817e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da817e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c817eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4817ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb81801, 85 },
    { 0xf3281802, 99 },
    { 0x84a81803, 100 },
    { 0xecb81804, 23 },
    { 0xf4881803, 90 },
    { 0xf0e81804, 90 },
    { 0xc681804, 33 },
    { 0xe7c81808, 34 },
    { 0xaf281809, 95 },
    { 0xc0781808, 23 },
    { 0x8e38180b, 78 },
    { 0xecb8180b, 67 },
    { 0x8518180c, 93 },
    { 0xd608180e, 40 },
    { 0x49d8180a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a81817, 76 },
    { 0xdbf81817, 90 },
    { 0x26181817, 90 },
    { 0xfed81817, 61 },
    { 0x3eb8181b, 90 },
    { 0x9d581818, 43 },
    { 0x3848181d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d281827, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad98182a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb881834, 95 },
    { 0xcff81835, 78 },
    { 0x58381836, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8258183a, 61 },
    { 0xd178183b, 61 },
    { 0xcb98183b, 52 },
    { 0xa08183a, 81 },
    { 0, 0 },
    { 0x4d38183f, 55 },
    { 0xbf681840, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9781843, 93 },
    { 0, 0 },
    { 0x2f581845, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be8184c, 90 },
    { 0, 0 },
    { 0xa588184e, 67 },
    { 0x44c8184f, 99 },
    { 0x44c8184e, 84 },
    { 0x1a381851, 95 },
    { 0x6a881851, 23 },
    { 0x2381853, 33 },
    { 0, 0 },
    { 0x14d81855, 67 },
    { 0xd1d81856, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb08185a, 91 },
    { 0, 0 },
    { 0xca58185c, 23 },
    { 0, 0 },
    { 0x5458185e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b981862, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a81865, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e81869, 23 },
    { 0xcc081869, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d08186d, 33 },
    { 0x3618186d, 65 },
    { 0, 0 },
    { 0x38781870, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11581876, 98 },
    { 0xdad81877, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a8187a, 67 },
    { 0x48f8187b, 87 },
    { 0xb048187c, 23 },
    { 0, 0 },
    { 0x8f78187e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f81881, 34 },
    { 0, 0 },
    { 0x2b681883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55881886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb981889, 23 },
    { 0, 0 },
    { 0x9a68188b, 65 },
    { 0xedb8188c, 84 },
    { 0x35c8188d, 52 },
    { 0x6918188e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4681892, 61 },
    { 0x6d681892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5281896, 46 },
    { 0x37c81896, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a78189b, 95 },
    { 0x4aa8189c, 98 },
    { 0xbc38189d, 95 },
    { 0x6d18189e, 83 },
    { 0x5488189f, 95 },
    { 0x81b8189e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b818ab, 90 },
    { 0, 0 },
    { 0x355818ad, 33 },
    { 0xba5818ae, 98 },
    { 0x79d818ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e818b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8818bc, 33 },
    { 0x240818bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a818c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f818cb, 34 },
    { 0, 0 },
    { 0xf0f818cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df818d0, 93 },
    { 0x9b7818d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c818d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x264818d8, 61 },
    { 0x5b9818d9, 93 },
    { 0xe0f818da, 33 },
    { 0x73f818db, 36 },
    { 0xb1f818dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c818e0, 33 },
    { 0, 0 },
    { 0xebb818e2, 23 },
    { 0x463818e3, 91 },
    { 0xced818e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646818e9, 66 },
    { 0xdd9818ea, 61 },
    { 0x7cd818ea, 77 },
    { 0x584818ec, 78 },
    { 0x681818ed, 17 },
    { 0xe2f818ec, 66 },
    { 0xeee818ef, 33 },
    { 0x6f8818f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3818f7, 23 },
    { 0, 0 },
    { 0xbe3818f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc281900, 90 },
    { 0, 0 },
    { 0x3bc81902, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13381905, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a81908, 99 },
    { 0x34181908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c18190c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb81911, 33 },
    { 0x5ec81912, 33 },
    { 0, 0 },
    { 0x34e81914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd81917, 91 },
    { 0, 0 },
    { 0x19181919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9508191d, 93 },
    { 0, 0 },
    { 0xf278191f, 95 },
    { 0, 0 },
    { 0xd7e81921, 67 },
    { 0x91481921, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90981926, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f81929, 95 },
    { 0, 0 },
    { 0xf398192b, 55 },
    { 0, 0 },
    { 0x9fc8192d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4481930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81381933, 46 },
    { 0x17c81934, 46 },
    { 0x6ea81933, 61 },
    { 0x13481936, 76 },
    { 0x92f81937, 77 },
    { 0x39181937, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c8193c, 61 },
    { 0xc338193d, 78 },
    { 0x3fc8193d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b781945, 33 },
    { 0x5bf81945, 23 },
    { 0xdbc81946, 98 },
    { 0, 0 },
    { 0xffd81949, 90 },
    { 0xcd88194a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded8194f, 76 },
    { 0xd381950, 61 },
    { 0, 0 },
    { 0xa6e81952, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1681957, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b8195a, 87 },
    { 0, 0 },
    { 0x66d8195c, 68 },
    { 0, 0 },
    { 0x698195e, 61 },
    { 0, 0 },
    { 0x24281960, 90 },
    { 0x1db81961, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb81964, 66 },
    { 0x66f81964, 61 },
    { 0x1c081965, 85 },
    { 0x98e81967, 99 },
    { 0x86181967, 95 },
    { 0x8d281969, 93 },
    { 0xe3181968, 33 },
    { 0x50b81966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc948196e, 34 },
    { 0xf118196f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36381974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a81978, 67 },
    { 0xdeb81979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e81982, 55 },
    { 0x47281983, 95 },
    { 0xb0881984, 55 },
    { 0xef881984, 34 },
    { 0xc1a81985, 67 },
    { 0, 0 },
    { 0xb0681988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e8198b, 99 },
    { 0xf748198c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0281997, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b8199c, 77 },
    { 0x4438199d, 69 },
    { 0, 0 },
    { 0x2cd8199f, 78 },
    { 0x228819a0, 90 },
    { 0x168819a0, 84 },
    { 0x156819a2, 46 },
    { 0, 0 },
    { 0x296819a4, 33 },
    { 0xee9819a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b819ac, 61 },
    { 0xdf1819ad, 61 },
    { 0x108819ad, 88 },
    { 0x81b819af, 23 },
    { 0x6a0819b0, 84 },
    { 0x35f819af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e819b6, 93 },
    { 0xb55819b7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e819bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x878819bf, 23 },
    { 0xae8819c0, 47 },
    { 0xd0a819c1, 61 },
    { 0, 0 },
    { 0x61b819c3, 88 },
    { 0, 0 },
    { 0x2c7819c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d819c9, 87 },
    { 0x4fb819c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd819d2, 69 },
    { 0, 0 },
    { 0x998819d4, 75 },
    { 0, 0 },
    { 0x57f819d6, 71 },
    { 0xdaf819d6, 61 },
    { 0xd7b819d7, 33 },
    { 0xd9e819d8, 33 },
    { 0x398819da, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74819de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f819e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56819e5, 93 },
    { 0xf61819e5, 90 },
    { 0xd15819e7, 90 },
    { 0xf5c819e8, 23 },
    { 0x329819e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe819ec, 23 },
    { 0, 0 },
    { 0xb0f819ee, 76 },
    { 0, 0 },
    { 0xe78819f0, 95 },
    { 0xc22819f1, 34 },
    { 0, 0 },
    { 0x34a819f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d819f9, 98 },
    { 0x13e819fa, 68 },
    { 0xba3819fa, 95 },
    { 0x6c5819fb, 95 },
    { 0xece819fd, 90 },
    { 0xf74819fe, 23 },
    { 0xed7819ff, 33 },
    { 0xdd2819fc, 90 },
    { 0x6ad819fc, 100 },
    { 0x86b81a02, 34 },
    { 0x799819fe, 61 },
    { 0x13d819fb, 61 },
    { 0xdf581a05, 33 },
    { 0xdf881a05, 93 },
    { 0, 0 },
    { 0x7dd81a08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3781a12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3681a15, 61 },
    { 0, 0 },
    { 0xa8981a17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc081a1f, 90 },
    { 0x53b81a20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd481a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc481a29, 99 },
    { 0xd5381a2a, 55 },
    { 0x99481a2b, 90 },
    { 0, 0 },
    { 0x8b281a2d, 76 },
    { 0x65f81a2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47681a36, 78 },
    { 0x11881a36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7581a3c, 84 },
    { 0xe7881a3d, 61 },
    { 0x28781a3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca81a44, 23 },
    { 0, 0 },
    { 0x15281a46, 95 },
    { 0xe4681a47, 90 },
    { 0, 0 },
    { 0x95281a49, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd381a4c, 61 },
    { 0xab81a4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e81a53, 100 },
    { 0xd8081a54, 95 },
    { 0xa6081a55, 90 },
    { 0xf7381a54, 33 },
    { 0x55581a57, 33 },
    { 0xc3581a58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8881a5b, 98 },
    { 0, 0 },
    { 0x8cb81a5d, 68 },
    { 0x7cf81a5e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb81a61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd81a64, 76 },
    { 0x45781a65, 95 },
    { 0xe1d81a66, 76 },
    { 0xaf81a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43081a6b, 90 },
    { 0, 0 },
    { 0x1cb81a6d, 34 },
    { 0x5681a6e, 33 },
    { 0x3d281a6e, 95 },
    { 0, 0 },
    { 0x9bc81a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7381a78, 96 },
    { 0xbc881a78, 76 },
    { 0xff481a78, 98 },
    { 0, 0 },
    { 0x2d881a7c, 33 },
    { 0xd2681a7d, 98 },
    { 0xf3f81a7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd481a81, 33 },
    { 0xc8581a82, 61 },
    { 0x4181a83, 87 },
    { 0x2e181a84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64781a8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c81a94, 76 },
    { 0xbc081a95, 61 },
    { 0, 0 },
    { 0x54e81a97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181a9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda281aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4781aa3, 77 },
    { 0x37281aa4, 95 },
    { 0x23481aa3, 55 },
    { 0, 0 },
    { 0x6af81aa7, 33 },
    { 0xe8381aa7, 23 },
    { 0x3fe81aa9, 23 },
    { 0, 0 },
    { 0x52f81aab, 33 },
    { 0, 0 },
    { 0xfb781aad, 23 },
    { 0, 0 },
    { 0x1881aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86981ab3, 84 },
    { 0x2df81ab3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee81ab9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f781abc, 100 },
    { 0xc9281abd, 33 },
    { 0x1f281abe, 66 },
    { 0xdbe81abe, 90 },
    { 0x14f81abf, 91 },
    { 0x15081ac1, 98 },
    { 0x8e581abc, 90 },
    { 0x7de81ac3, 85 },
    { 0x70a81ac4, 61 },
    { 0xe4b81ac4, 95 },
    { 0, 0 },
    { 0x86c81ac7, 34 },
    { 0x34681ac8, 88 },
    { 0x9c481ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32881acc, 98 },
    { 0x5a081acd, 23 },
    { 0xf6881ace, 52 },
    { 0x16a81acf, 95 },
    { 0, 0 },
    { 0x37981ad1, 34 },
    { 0, 0 },
    { 0xdb081ad3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f81ada, 78 },
    { 0, 0 },
    { 0xb0981adc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8281adf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a081ae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a81ae7, 61 },
    { 0, 0 },
    { 0x17481ae9, 90 },
    { 0x4fb81aea, 46 },
    { 0xb3a81aeb, 43 },
    { 0xef881aec, 82 },
    { 0x3d81aed, 55 },
    { 0x4ad81aed, 90 },
    { 0x3d881aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb81af3, 33 },
    { 0x3ac81af3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e681af7, 38 },
    { 0x7f581af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c81afc, 78 },
    { 0xd7881afd, 61 },
    { 0x77f81afe, 49 },
    { 0x81f81afe, 95 },
    { 0x9ce81b00, 88 },
    { 0xf1381afd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac081b06, 90 },
    { 0x7f81b07, 33 },
    { 0x5d681b08, 98 },
    { 0x7a881b08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be81b0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2881b11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87981b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65481b19, 95 },
    { 0x14f81b1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c81b26, 4 },
    { 0, 0 },
    { 0xaf681b28, 66 },
    { 0x65e81b28, 58 },
    { 0x58281b2a, 66 },
    { 0x75381b2b, 33 },
    { 0, 0 },
    { 0x3dc81b2d, 90 },
    { 0x65081b2d, 33 },
    { 0xbe481b2f, 93 },
    { 0, 0 },
    { 0x77a81b31, 33 },
    { 0x5db81b31, 76 },
    { 0x70a81b32, 43 },
    { 0, 0 },
    { 0x8c781b35, 34 },
    { 0, 0 },
    { 0xb5481b37, 61 },
    { 0xe9c81b38, 23 },
    { 0x9181b39, 51 },
    { 0xca681b39, 61 },
    { 0xa3381b39, 93 },
    { 0xc3281b3c, 90 },
    { 0xf8881b3d, 90 },
    { 0x7d581b39, 90 },
    { 0, 0 },
    { 0x54f81b40, 97 },
    { 0, 0 },
    { 0xa0a81b42, 77 },
    { 0x7bc81b43, 33 },
    { 0x2c981b43, 90 },
    { 0x1ab81b45, 66 },
    { 0, 0 },
    { 0x54e81b47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d81b4c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b81b4f, 45 },
    { 0x36c81b50, 33 },
    { 0x17681b50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91681b56, 90 },
    { 0, 0 },
    { 0x60181b58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a81b5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e81b62, 33 },
    { 0, 0 },
    { 0x65d81b64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x96081b67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8581b6a, 80 },
    { 0, 0 },
    { 0xce981b6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c81b73, 92 },
    { 0x95181b74, 90 },
    { 0xd9c81b75, 95 },
    { 0xf3781b73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86381b79, 33 },
    { 0xdfa81b7a, 100 },
    { 0xd5e81b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53781b7e, 90 },
    { 0x97181b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2581b83, 33 },
    { 0xe6481b84, 90 },
    { 0x43b81b84, 61 },
    { 0xdb581b86, 93 },
    { 0xf8c81b85, 43 },
    { 0x49f81b85, 90 },
    { 0xa2081b85, 34 },
    { 0xe5e81b8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c81b8d, 78 },
    { 0, 0 },
    { 0x2b981b8f, 99 },
    { 0xf2781b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee81b93, 61 },
    { 0x6ee81b94, 95 },
    { 0xfe281b95, 76 },
    { 0x5a781b96, 98 },
    { 0x43c81b97, 98 },
    { 0xf8481b97, 55 },
    { 0, 0 },
    { 0x74f81b9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf81b9d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf81ba0, 75 },
    { 0x1b781ba0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb81baa, 61 },
    { 0, 0 },
    { 0x8f681bac, 76 },
    { 0, 0 },
    { 0x49e81bae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c81bb1, 34 },
    { 0x2a681bb2, 90 },
    { 0xf7281bb2, 99 },
    { 0, 0 },
    { 0x16b81bb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb81bb8, 90 },
    { 0, 0 },
    { 0x1d381bba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7981bbf, 33 },
    { 0x1f081bc0, 90 },
    { 0x51c81bbf, 95 },
    { 0, 0 },
    { 0x55281bc3, 76 },
    { 0x6ac81bc3, 88 },
    { 0x25381bc4, 61 },
    { 0x19b81bc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c81bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c81bcc, 23 },
    { 0, 0 },
    { 0xb4a81bce, 33 },
    { 0x7a281bcf, 93 },
    { 0, 0 },
    { 0x69281bd1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e81bd4, 55 },
    { 0x44081bd4, 90 },
    { 0x60781bd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c81bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4081bde, 33 },
    { 0, 0 },
    { 0x7de81be0, 88 },
    { 0x3e281be0, 87 },
    { 0x35581be2, 55 },
    { 0x1c81be3, 67 },
    { 0, 0 },
    { 0x85881be5, 84 },
    { 0xcc481be6, 93 },
    { 0x1b181be6, 78 },
    { 0xcb081be6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb81beb, 78 },
    { 0xc5081bec, 98 },
    { 0xdd481bed, 61 },
    { 0x92781bee, 99 },
    { 0x9f81bef, 23 },
    { 0x44881bf0, 76 },
    { 0xc2881bef, 95 },
    { 0, 0 },
    { 0xf7581bf3, 90 },
    { 0xe2481bf3, 61 },
    { 0x79481bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc81bf8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92781c00, 33 },
    { 0x42681c01, 43 },
    { 0, 0 },
    { 0xa5081c03, 99 },
    { 0x99381c04, 33 },
    { 0x7d981c03, 23 },
    { 0, 0 },
    { 0x15881c07, 71 },
    { 0xb3681c08, 23 },
    { 0x84481c09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3381c0c, 46 },
    { 0, 0 },
    { 0x46c81c0e, 78 },
    { 0x4d81c0e, 84 },
    { 0x98081c0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c81c15, 61 },
    { 0, 0 },
    { 0xe0981c17, 84 },
    { 0, 0 },
    { 0x5ff81c19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4381c21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4381c25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12781c28, 76 },
    { 0, 0 },
    { 0x3d881c2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8181c30, 95 },
    { 0, 0 },
    { 0x45781c32, 95 },
    { 0x9cd81c33, 90 },
    { 0xa2381c34, 34 },
    { 0x31981c35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63181c3a, 90 },
    { 0, 0 },
    { 0x60681c3c, 90 },
    { 0x81f81c3c, 40 },
    { 0, 0 },
    { 0x87781c3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e81c45, 90 },
    { 0, 0 },
    { 0x84481c47, 61 },
    { 0, 0 },
    { 0x2fe81c49, 33 },
    { 0, 0 },
    { 0xb2281c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0681c56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a81c5a, 90 },
    { 0xa4781c5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c481c61, 95 },
    { 0xbfb81c62, 85 },
    { 0x55c81c61, 55 },
    { 0x55e81c63, 33 },
    { 0xcdb81c65, 61 },
    { 0xd6181c64, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa581c6c, 98 },
    { 0xc1481c6d, 95 },
    { 0xdb281c6e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac81c72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74581c75, 61 },
    { 0xc0781c76, 98 },
    { 0xac581c77, 33 },
    { 0x7b681c78, 98 },
    { 0x5ba81c76, 95 },
    { 0xe1381c7a, 95 },
    { 0x9b681c7b, 61 },
    { 0, 0 },
    { 0x2ab81c7d, 90 },
    { 0, 0 },
    { 0x51181c7f, 90 },
    { 0x50e81c7f, 90 },
    { 0x7ae81c7f, 34 },
    { 0xe6481c80, 34 },
    { 0x83e81c80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc81c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f781c8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9881c8d, 33 },
    { 0, 0 },
    { 0xd3181c8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb681c92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a81c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe081c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23281ca3, 95 },
    { 0x4f681ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a81cab, 33 },
    { 0x19481cac, 99 },
    { 0x52281cab, 33 },
    { 0xc1c81cad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a81cb2, 98 },
    { 0, 0 },
    { 0x97981cb4, 88 },
    { 0x1d281cb5, 76 },
    { 0, 0 },
    { 0xfb981cb7, 23 },
    { 0x47c81cb7, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde81cbd, 61 },
    { 0x74f81cbe, 76 },
    { 0x4eb81cbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a81cc6, 90 },
    { 0xdc981cc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef81cce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef381cd3, 61 },
    { 0xe6381cd3, 33 },
    { 0xf8f81cd4, 61 },
    { 0, 0 },
    { 0xe1a81cd7, 33 },
    { 0, 0 },
    { 0x46281cd9, 92 },
    { 0xb9181cd9, 23 },
    { 0, 0 },
    { 0xb4281cdc, 76 },
    { 0, 0 },
    { 0x3e081cde, 23 },
    { 0, 0 },
    { 0x90981ce0, 86 },
    { 0, 0 },
    { 0x7b81ce2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6781ce8, 80 },
    { 0x31d81ce9, 23 },
    { 0, 0 },
    { 0x3bd81ceb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64481cee, 33 },
    { 0, 0 },
    { 0x32a81cf0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481cf5, 55 },
    { 0, 0 },
    { 0xfb981cf7, 61 },
    { 0x32281cf7, 55 },
    { 0x9d81cf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb081cfc, 33 },
    { 0xd5c81cfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b481d03, 43 },
    { 0, 0 },
    { 0xeb481d05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a81d08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb81d12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa181d17, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3081d1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59381d23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca81d27, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36381d2e, 33 },
    { 0x74381d2f, 90 },
    { 0x1c181d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2581d39, 76 },
    { 0xad481d3a, 99 },
    { 0, 0 },
    { 0xacd81d3c, 33 },
    { 0, 0 },
    { 0x8b381d3e, 76 },
    { 0x42e81d3f, 76 },
    { 0x2d181d3f, 94 },
    { 0xbf581d3f, 84 },
    { 0x90981d42, 59 },
    { 0x92b81d43, 66 },
    { 0x56081d43, 90 },
    { 0, 0 },
    { 0x7b981d46, 88 },
    { 0xb4a81d46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a781d4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83881d52, 76 },
    { 0xce181d53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c81d5b, 76 },
    { 0, 0 },
    { 0x77881d5d, 87 },
    { 0x71d81d5e, 43 },
    { 0x26981d5e, 90 },
    { 0, 0 },
    { 0x4b981d61, 95 },
    { 0x7d981d62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6781d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa781d68, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd681d6c, 98 },
    { 0, 0 },
    { 0xc5f81d6e, 98 },
    { 0xe5d81d6f, 43 },
    { 0xe0e81d6f, 34 },
    { 0xf1881d70, 55 },
    { 0x85d81d72, 64 },
    { 0x6d081d72, 52 },
    { 0x85c81d74, 91 },
    { 0, 0 },
    { 0x12881d76, 85 },
    { 0x4d681d77, 78 },
    { 0xee81d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb981d7b, 33 },
    { 0, 0 },
    { 0xd6e81d7d, 78 },
    { 0x7e181d7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb381d81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed081d89, 95 },
    { 0x60d81d8a, 95 },
    { 0x29e81d8a, 34 },
    { 0x7d181d8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45581d8f, 90 },
    { 0xc2981d90, 76 },
    { 0x24c81d91, 90 },
    { 0x19081d8f, 23 },
    { 0x2d681d93, 95 },
    { 0x8c681d92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f81d9c, 98 },
    { 0x99381d9d, 93 },
    { 0xc1c81d9e, 90 },
    { 0x62481d9f, 61 },
    { 0xb2c81d9f, 64 },
    { 0xd8f81da1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3181da6, 23 },
    { 0, 0 },
    { 0x2f481da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d81dab, 90 },
    { 0x7e181dac, 34 },
    { 0x87481dad, 83 },
    { 0x6c081dad, 88 },
    { 0, 0 },
    { 0xdd181db0, 23 },
    { 0, 0 },
    { 0xf181db2, 90 },
    { 0x31c81db2, 76 },
    { 0xbcf81db3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce81db8, 98 },
    { 0, 0 },
    { 0x64a81dba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7381dbd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68281dc0, 61 },
    { 0x52781dc0, 53 },
    { 0x51881dc2, 61 },
    { 0xc1981dc3, 78 },
    { 0x22e81dc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef281dcc, 61 },
    { 0x26881dcd, 65 },
    { 0xe9081dcc, 90 },
    { 0x5a981dce, 61 },
    { 0, 0 },
    { 0xfca81dd1, 34 },
    { 0x16a81dd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e81dd5, 33 },
    { 0x68081dd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b181ddd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e81de0, 90 },
    { 0x91b81de1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90181de6, 23 },
    { 0, 0 },
    { 0x7d681de8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90481ded, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d81df5, 23 },
    { 0x96381df5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3881dfb, 100 },
    { 0x84681dfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb81e02, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52681e09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9681e0e, 43 },
    { 0, 0 },
    { 0x80481e10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4781e16, 61 },
    { 0, 0 },
    { 0x3a381e18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7581e1c, 52 },
    { 0x15f81e1c, 98 },
    { 0, 0 },
    { 0x3be81e1f, 91 },
    { 0x98781e20, 61 },
    { 0x9bd81e21, 33 },
    { 0x3e081e22, 61 },
    { 0, 0 },
    { 0x5e181e24, 76 },
    { 0, 0 },
    { 0xb4381e26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca81e29, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f81e2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x89381e31, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c081e3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4181e3e, 55 },
    { 0xdcf81e3f, 76 },
    { 0x39581e3f, 61 },
    { 0, 0 },
    { 0x29c81e42, 33 },
    { 0x29d81e43, 100 },
    { 0x5bd81e43, 23 },
    { 0x86d81e43, 66 },
    { 0x47581e46, 77 },
    { 0, 0 },
    { 0xa1881e48, 17 },
    { 0xb5081e48, 61 },
    { 0xc3881e48, 61 },
    { 0x27e81e4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e81e4e, 33 },
    { 0x23c81e4e, 33 },
    { 0, 0 },
    { 0xe781e51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x11881e54, 95 },
    { 0, 0 },
    { 0xc8981e56, 76 },
    { 0x8d681e56, 33 },
    { 0x33881e56, 84 },
    { 0xb2d81e56, 90 },
    { 0xae581e5a, 23 },
    { 0x6c881e59, 61 },
    { 0x6a381e5a, 34 },
    { 0x53781e5d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb681e63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54981e6c, 61 },
    { 0xd3f81e6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45081e75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db81e78, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3281e7c, 61 },
    { 0xca181e7d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d81e81, 46 },
    { 0, 0 },
    { 0x46581e83, 96 },
    { 0xada81e84, 98 },
    { 0x6bf81e85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7681e8c, 100 },
    { 0xc5a81e8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c81e90, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d981e94, 61 },
    { 0x69481e95, 90 },
    { 0, 0 },
    { 0xd7e81e97, 58 },
    { 0x8b081e98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb81e9c, 23 },
    { 0x1e981e9d, 90 },
    { 0xf2881e9e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c581ea2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be81ea6, 66 },
    { 0, 0 },
    { 0x8a381ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af81eac, 95 },
    { 0, 0 },
    { 0xd3781eae, 71 },
    { 0x1181eaf, 52 },
    { 0x87d81eaf, 93 },
    { 0x2db81eb0, 95 },
    { 0xd9681eb1, 95 },
    { 0xaaa81eb1, 66 },
    { 0x23381eb4, 90 },
    { 0xed81eb0, 33 },
    { 0x9ae81eb6, 90 },
    { 0x81881eb6, 91 },
    { 0x76e81eb8, 100 },
    { 0, 0 },
    { 0xb1a81eba, 90 },
    { 0x27c81eba, 55 },
    { 0x33c81eba, 43 },
    { 0, 0 },
    { 0x43d81ebe, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f081ec1, 19 },
    { 0, 0 },
    { 0xcc381ec3, 34 },
    { 0x12681ec3, 33 },
    { 0x9db81ec4, 66 },
    { 0xf2c81ec5, 90 },
    { 0x71981ec7, 90 },
    { 0x6d481ec8, 61 },
    { 0xe4981ec5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf981ecc, 61 },
    { 0, 0 },
    { 0x32c81ece, 55 },
    { 0xb2681ecf, 76 },
    { 0xf6881ed0, 77 },
    { 0, 0 },
    { 0x1d781ed2, 98 },
    { 0, 0 },
    { 0x2a081ed4, 48 },
    { 0x43981ed5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9381edb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a81ee5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d81ee9, 33 },
    { 0xd3481eea, 100 },
    { 0, 0 },
    { 0x80981eec, 90 },
    { 0, 0 },
    { 0x82e81eee, 95 },
    { 0, 0 },
    { 0x74681ef0, 100 },
    { 0xcac81ef0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0081ef6, 46 },
    { 0x1a181ef7, 95 },
    { 0x15c81ef6, 95 },
    { 0x7f481ef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f81efd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a81f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac81f06, 99 },
    { 0xf9b81f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c81f0b, 84 },
    { 0x3ba81f0c, 95 },
    { 0, 0 },
    { 0x55e81f0e, 98 },
    { 0, 0 },
    { 0xaa581f10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a81f14, 33 },
    { 0, 0 },
    { 0xbeb81f16, 76 },
    { 0x17981f16, 90 },
    { 0, 0 },
    { 0x32c81f19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e81f1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1181f23, 91 },
    { 0x37381f24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45381f29, 90 },
    { 0x20a81f2a, 61 },
    { 0x6c381f29, 34 },
    { 0x5ba81f2c, 95 },
    { 0, 0 },
    { 0xa3781f2e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc181f33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14481f37, 33 },
    { 0x88781f38, 100 },
    { 0xe9481f39, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb081f3e, 99 },
    { 0x1be81f3f, 23 },
    { 0x6d281f40, 85 },
    { 0xf9c81f40, 67 },
    { 0x13d81f40, 33 },
    { 0x6fd81f41, 98 },
    { 0xca981f44, 90 },
    { 0, 0 },
    { 0xc4281f46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa281f49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f781f4c, 90 },
    { 0xe1f81f4c, 33 },
    { 0, 0 },
    { 0x58481f4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74681f53, 98 },
    { 0x89f81f54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66681f59, 55 },
    { 0x1ec81f59, 65 },
    { 0x3b081f59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae981f5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc981f61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c81f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93581f6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54581f71, 87 },
    { 0xd781f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96781f7c, 67 },
    { 0, 0 },
    { 0x74081f7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d81f87, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d81f8c, 55 },
    { 0, 0 },
    { 0x55481f8e, 81 },
    { 0, 0 },
    { 0xa6981f90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82881f94, 66 },
    { 0xe5181f95, 100 },
    { 0x1ed81f96, 33 },
    { 0xeda81f96, 76 },
    { 0x87f81f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a81f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f281fa0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f081fa6, 95 },
    { 0x7ac81fa6, 34 },
    { 0x7981fa8, 77 },
    { 0x1bf81fa7, 98 },
    { 0xd1581fa9, 61 },
    { 0xac81fab, 33 },
    { 0x57b81fac, 43 },
    { 0xa7281fa7, 43 },
    { 0xb8b81fae, 98 },
    { 0, 0 },
    { 0xa0981fb0, 98 },
    { 0x86381fb0, 88 },
    { 0, 0 },
    { 0x7a581fb3, 78 },
    { 0xc7981fb3, 66 },
    { 0xfc581fb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89081fba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde981fbd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87081fc2, 55 },
    { 0x8c481fc3, 90 },
    { 0x39481fc4, 23 },
    { 0xf6a81fc5, 65 },
    { 0x14981fc5, 33 },
    { 0x4b681fc7, 23 },
    { 0x71d81fc6, 33 },
    { 0xa0181fc9, 39 },
    { 0, 0 },
    { 0xad581fcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35881fd7, 77 },
    { 0x38681fd8, 90 },
    { 0x78a81fd9, 85 },
    { 0x4e781fd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be81fdf, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac81fe2, 55 },
    { 0x45381fe3, 65 },
    { 0xde081fe4, 46 },
    { 0x88381fe5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28181fef, 33 },
    { 0, 0 },
    { 0xaf481ff1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f81ff5, 55 },
    { 0x3b281ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa081ffb, 76 },
    { 0x89681ffc, 46 },
    { 0x4fd81ffc, 75 },
    { 0x3b681ffd, 95 },
    { 0x90581fff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f82004, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b8200a, 61 },
    { 0, 0 },
    { 0xdb28200c, 90 },
    { 0x318200d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac82011, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78482015, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x69082018, 95 },
    { 0x75382019, 61 },
    { 0x4ea82018, 95 },
    { 0xd38201a, 98 },
    { 0x7818201b, 95 },
    { 0, 0 },
    { 0x1c18201e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c82026, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60482029, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f82032, 23 },
    { 0x26b82032, 33 },
    { 0x61382034, 61 },
    { 0x5dd82032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed38203a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa668203f, 23 },
    { 0x2ac82040, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf182044, 61 },
    { 0x24a82045, 85 },
    { 0x1ad82046, 55 },
    { 0x5ed82045, 33 },
    { 0x91482048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d18204b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c28205a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c8205d, 88 },
    { 0x7c08205d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61582062, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c82066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe782069, 61 },
    { 0xf4e82069, 66 },
    { 0xceb82069, 61 },
    { 0x82b8206c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868206f, 90 },
    { 0x35f82070, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xde482073, 33 },
    { 0x4be82074, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb808207c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0682083, 46 },
    { 0x91b82084, 23 },
    { 0x38682083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac68208a, 93 },
    { 0x42b8208b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff82092, 55 },
    { 0x1a482093, 90 },
    { 0x6f782094, 100 },
    { 0x9a882095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec8209a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b8209d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d820a0, 93 },
    { 0xc31820a1, 33 },
    { 0x254820a2, 61 },
    { 0, 0 },
    { 0xc25820a4, 90 },
    { 0, 0 },
    { 0x848820a6, 68 },
    { 0, 0 },
    { 0x594820a8, 78 },
    { 0xe3820a9, 95 },
    { 0x2d5820a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e820ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6820b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45820b5, 38 },
    { 0x3ee820b6, 23 },
    { 0xab5820b5, 90 },
    { 0x579820b8, 98 },
    { 0, 0 },
    { 0x9d0820ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d820bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda820c9, 43 },
    { 0xeeb820ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x481820cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968820d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b820d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1820db, 26 },
    { 0x767820db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e820e4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13820e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1820ed, 95 },
    { 0, 0 },
    { 0x6bc820ef, 98 },
    { 0x546820f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849820f4, 85 },
    { 0x579820f4, 34 },
    { 0x2c5820f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5820fa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d820fe, 93 },
    { 0x8c1820ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f082102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a82105, 95 },
    { 0, 0 },
    { 0x82582107, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a48210c, 90 },
    { 0xb728210d, 33 },
    { 0, 0 },
    { 0x9728210f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80382113, 98 },
    { 0, 0 },
    { 0x4f382115, 33 },
    { 0, 0 },
    { 0xdb282117, 46 },
    { 0, 0 },
    { 0x17a82119, 97 },
    { 0x52882119, 66 },
    { 0, 0 },
    { 0x9ab8211c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d982122, 52 },
    { 0x9d282123, 61 },
    { 0x8ac82123, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f82129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d8212c, 90 },
    { 0xe0d8212c, 78 },
    { 0xe388212e, 98 },
    { 0, 0 },
    { 0x9a882130, 33 },
    { 0x82b82131, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b82134, 46 },
    { 0x63182134, 44 },
    { 0, 0 },
    { 0xd4982137, 61 },
    { 0x1fd82138, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a18213c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87682140, 99 },
    { 0xa0582140, 81 },
    { 0, 0 },
    { 0x2982143, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd82146, 95 },
    { 0, 0 },
    { 0x35382148, 88 },
    { 0, 0 },
    { 0x6228214a, 52 },
    { 0, 0 },
    { 0x6738214c, 23 },
    { 0x8b98214d, 90 },
    { 0xba08214e, 90 },
    { 0xf938214f, 23 },
    { 0xf468214d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb382153, 95 },
    { 0xc5a82154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f682157, 23 },
    { 0x54182158, 95 },
    { 0x93c82157, 33 },
    { 0x11382158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f68215d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a82163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8982169, 33 },
    { 0, 0 },
    { 0x3718216b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9768216e, 23 },
    { 0xe3b8216e, 93 },
    { 0x9b382170, 90 },
    { 0x50982171, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8482178, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d8217c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92782188, 61 },
    { 0x80e82189, 95 },
    { 0x4d08218a, 90 },
    { 0xada82188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe038218e, 34 },
    { 0x69a8218f, 51 },
    { 0, 0 },
    { 0x2fe82191, 61 },
    { 0x81582192, 87 },
    { 0xbfd82193, 55 },
    { 0x98f82193, 95 },
    { 0x5fe82195, 88 },
    { 0xe7d82191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7882199, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408219d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41821a0, 55 },
    { 0x54e821a1, 78 },
    { 0x504821a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183821a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1821ab, 90 },
    { 0xd0f821ac, 61 },
    { 0, 0 },
    { 0x4af821ae, 59 },
    { 0xe4f821ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6821b2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a821b7, 83 },
    { 0, 0 },
    { 0x427821b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40821bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465821c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0821c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd821d0, 61 },
    { 0xd93821d0, 87 },
    { 0, 0 },
    { 0x2aa821d3, 100 },
    { 0x27b821d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32821d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x981821da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3821de, 93 },
    { 0x95821df, 23 },
    { 0, 0 },
    { 0x42a821e1, 84 },
    { 0xdb1821e1, 81 },
    { 0xb19821e2, 90 },
    { 0x54d821e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8821e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7821ea, 100 },
    { 0x3d7821eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae821f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f821f9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e82203, 90 },
    { 0x8a082204, 83 },
    { 0x35682203, 90 },
    { 0x5f182206, 23 },
    { 0x60982206, 93 },
    { 0xe8b82208, 93 },
    { 0x10c82209, 95 },
    { 0x35982203, 33 },
    { 0xb3582206, 37 },
    { 0x72c8220c, 33 },
    { 0, 0 },
    { 0x8718220e, 61 },
    { 0x3a8220f, 40 },
    { 0, 0 },
    { 0xa2882211, 100 },
    { 0, 0 },
    { 0xcec82213, 95 },
    { 0xf7a82214, 76 },
    { 0x44182215, 61 },
    { 0x85b82216, 23 },
    { 0xd2782216, 90 },
    { 0x85682214, 93 },
    { 0, 0 },
    { 0x6728221a, 68 },
    { 0x2678221a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c88221e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba82228, 61 },
    { 0, 0 },
    { 0x15a8222a, 61 },
    { 0x61b8222b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7768223e, 90 },
    { 0x70d8223e, 43 },
    { 0, 0 },
    { 0xb6a82241, 90 },
    { 0x17582242, 33 },
    { 0x52b82242, 75 },
    { 0xa3782241, 43 },
    { 0x49c82244, 98 },
    { 0, 0 },
    { 0x48182247, 87 },
    { 0, 0 },
    { 0xfff82249, 99 },
    { 0, 0 },
    { 0xd3a8224b, 98 },
    { 0xa668224c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7098224f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd82254, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2482258, 76 },
    { 0x1582259, 98 },
    { 0, 0 },
    { 0x9b08225b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe8225e, 44 },
    { 0x9608225f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac82262, 90 },
    { 0x9ff82262, 83 },
    { 0x99382263, 34 },
    { 0, 0 },
    { 0x63e82266, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a82269, 99 },
    { 0x53b82269, 33 },
    { 0x7188226b, 34 },
    { 0x6088226c, 46 },
    { 0xaf48226b, 90 },
    { 0x3d78226c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69482276, 98 },
    { 0xceb82277, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608227c, 81 },
    { 0, 0 },
    { 0xc2f8227e, 90 },
    { 0xd0e8227e, 100 },
    { 0, 0 },
    { 0x7c382281, 98 },
    { 0, 0 },
    { 0xd4c82283, 23 },
    { 0xaf282283, 93 },
    { 0x39282285, 90 },
    { 0x20582286, 81 },
    { 0x19682286, 34 },
    { 0x16082288, 76 },
    { 0, 0 },
    { 0xa188228a, 43 },
    { 0x8188228b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee8228f, 55 },
    { 0xd758228f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b28229a, 76 },
    { 0, 0 },
    { 0xaa88229c, 37 },
    { 0xf368229d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c822a2, 23 },
    { 0, 0 },
    { 0x90e822a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd822a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658822ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72822b4, 90 },
    { 0xd86822b5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9822b8, 95 },
    { 0x1e9822b9, 23 },
    { 0xa6822ba, 88 },
    { 0xe42822b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f822be, 90 },
    { 0xd6d822bf, 90 },
    { 0x153822c0, 33 },
    { 0, 0 },
    { 0x46a822c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de822c9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801822ce, 99 },
    { 0, 0 },
    { 0xed9822d0, 100 },
    { 0xa0822d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3822d6, 43 },
    { 0x22d822d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635822db, 98 },
    { 0x689822db, 98 },
    { 0x466822dd, 90 },
    { 0, 0 },
    { 0x6e8822df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07822e5, 36 },
    { 0xed4822e6, 88 },
    { 0x89f822e7, 95 },
    { 0x719822e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c822ec, 87 },
    { 0xf21822ed, 90 },
    { 0xf98822ed, 23 },
    { 0xdf8822ef, 91 },
    { 0x14b822ef, 61 },
    { 0x386822ef, 61 },
    { 0x591822f2, 33 },
    { 0xd88822f3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a822f9, 76 },
    { 0xa0822fa, 95 },
    { 0x846822fb, 99 },
    { 0xa0b822fc, 95 },
    { 0xb8f822fa, 75 },
    { 0, 0 },
    { 0x40822ff, 90 },
    { 0x5e282300, 78 },
    { 0x68382301, 75 },
    { 0x1f882302, 34 },
    { 0x62182300, 78 },
    { 0xc7182304, 33 },
    { 0xdce82304, 93 },
    { 0x4ce82301, 91 },
    { 0x25d82307, 88 },
    { 0x5de82307, 66 },
    { 0x5c382308, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af8230e, 33 },
    { 0xdb08230e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe82312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9382315, 95 },
    { 0xbae82316, 66 },
    { 0xfe282316, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818231d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15482320, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982326, 90 },
    { 0, 0 },
    { 0x33682328, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa708232d, 76 },
    { 0, 0 },
    { 0x8b38232f, 100 },
    { 0, 0 },
    { 0x46b82331, 95 },
    { 0x3dc82332, 98 },
    { 0, 0 },
    { 0xc7d82334, 33 },
    { 0, 0 },
    { 0x67382336, 76 },
    { 0, 0 },
    { 0x79f82338, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1438233e, 23 },
    { 0x46e8233e, 91 },
    { 0x35f8233f, 76 },
    { 0x8db8233e, 61 },
    { 0x8c082342, 33 },
    { 0xad82343, 55 },
    { 0x49782344, 95 },
    { 0xfc782345, 61 },
    { 0x5b382345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78234c, 23 },
    { 0x9a08234d, 34 },
    { 0, 0 },
    { 0x7bd8234f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66882353, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c8235b, 92 },
    { 0x6f68235b, 43 },
    { 0x6f88235d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe082360, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c82364, 43 },
    { 0x42282365, 90 },
    { 0x77882366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a8236c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee8236f, 61 },
    { 0x1d282370, 95 },
    { 0x67f82371, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66482375, 98 },
    { 0, 0 },
    { 0x40282377, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48237c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5482384, 33 },
    { 0x9de82384, 43 },
    { 0xe1e82386, 91 },
    { 0x5682387, 75 },
    { 0, 0 },
    { 0x87282389, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda8238f, 93 },
    { 0, 0 },
    { 0x90d82391, 55 },
    { 0x37882392, 90 },
    { 0x13c82392, 95 },
    { 0x94e82392, 95 },
    { 0x53782395, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c182399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf38239e, 66 },
    { 0, 0 },
    { 0xc8a823a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3823a8, 55 },
    { 0x25b823a8, 98 },
    { 0xe0d823a9, 78 },
    { 0, 0 },
    { 0x5eb823ac, 80 },
    { 0x9e6823ad, 84 },
    { 0x982823ae, 93 },
    { 0x50c823af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56823b7, 23 },
    { 0x8be823b8, 33 },
    { 0, 0 },
    { 0xd4b823ba, 61 },
    { 0xda5823bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746823bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b823c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1823c8, 33 },
    { 0, 0 },
    { 0x2ee823ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba823cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x333823d0, 61 },
    { 0x4a2823d1, 17 },
    { 0, 0 },
    { 0xd52823d3, 61 },
    { 0xa63823d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35823d9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4823df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168823e6, 90 },
    { 0xff2823e7, 98 },
    { 0x454823e8, 67 },
    { 0x9ac823e9, 77 },
    { 0x7b2823e8, 33 },
    { 0x857823ea, 76 },
    { 0, 0 },
    { 0xb28823ed, 100 },
    { 0, 0 },
    { 0xc49823ef, 84 },
    { 0, 0 },
    { 0x852823f1, 90 },
    { 0, 0 },
    { 0x140823f3, 23 },
    { 0, 0 },
    { 0x438823f5, 90 },
    { 0, 0 },
    { 0x8e8823f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b823fa, 90 },
    { 0, 0 },
    { 0x2c4823fc, 55 },
    { 0x475823fc, 98 },
    { 0xbd1823fe, 43 },
    { 0xfe5823ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0682403, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d8240a, 33 },
    { 0xa368240b, 52 },
    { 0x8e28240b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc68240f, 33 },
    { 0xcc582410, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638241a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd588241d, 99 },
    { 0x24d8241e, 87 },
    { 0x7bc8241f, 66 },
    { 0x7d28241d, 33 },
    { 0x8ba82420, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x67382424, 99 },
    { 0x58382424, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa82428, 46 },
    { 0x9cc82429, 90 },
    { 0xe0082429, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b8242f, 93 },
    { 0x13f82430, 33 },
    { 0x64082431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b82438, 90 },
    { 0x9ab82438, 55 },
    { 0xdbd82439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8198243e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b82442, 90 },
    { 0, 0 },
    { 0x18982444, 23 },
    { 0x2882444, 61 },
    { 0, 0 },
    { 0xad682447, 46 },
    { 0x99e82448, 76 },
    { 0, 0 },
    { 0x6018244a, 93 },
    { 0xc7d8244b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e582450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe382454, 23 },
    { 0, 0 },
    { 0x75482456, 61 },
    { 0, 0 },
    { 0x9ce82458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918245d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f82462, 90 },
    { 0, 0 },
    { 0x9182464, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b282467, 81 },
    { 0x8e282468, 43 },
    { 0, 0 },
    { 0xf328246a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b8246d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6882470, 33 },
    { 0, 0 },
    { 0xeac82472, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4582479, 98 },
    { 0x38c8247a, 99 },
    { 0xd298247b, 98 },
    { 0, 0 },
    { 0x4d98247d, 95 },
    { 0x3018247e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c82485, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede8248c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee82497, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf118249a, 72 },
    { 0x39c8249a, 33 },
    { 0x7a28249c, 53 },
    { 0x77b8249c, 90 },
    { 0xfa18249d, 33 },
    { 0xee78249e, 46 },
    { 0x6458249f, 38 },
    { 0, 0 },
    { 0x876824a2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c824a6, 33 },
    { 0xb75824a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f824aa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2824ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b824b2, 76 },
    { 0xcbe824b2, 95 },
    { 0xf78824b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40824b9, 61 },
    { 0x54a824ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1824be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577824c3, 98 },
    { 0x1c0824c4, 76 },
    { 0x8bb824c5, 34 },
    { 0x44d824c6, 23 },
    { 0x6ed824c5, 90 },
    { 0, 0 },
    { 0x391824c9, 33 },
    { 0x2b6824c9, 55 },
    { 0xd1b824ca, 85 },
    { 0x796824ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x991824cf, 90 },
    { 0, 0 },
    { 0xf25824d1, 90 },
    { 0x1b3824d1, 23 },
    { 0, 0 },
    { 0x4e4824d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac824de, 61 },
    { 0xb1a824de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x912824e2, 61 },
    { 0, 0 },
    { 0x687824e4, 66 },
    { 0x14824e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd824e8, 76 },
    { 0x11e824e9, 94 },
    { 0xa96824ea, 90 },
    { 0x218824ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f824f1, 90 },
    { 0xd9b824f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5824f9, 55 },
    { 0xd83824fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x771824fd, 33 },
    { 0xc1a824fe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a482502, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e8250b, 95 },
    { 0x318250b, 90 },
    { 0x5e48250d, 76 },
    { 0xcf28250b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc82512, 61 },
    { 0xafc82512, 34 },
    { 0, 0 },
    { 0x12582515, 55 },
    { 0, 0 },
    { 0x7982517, 52 },
    { 0x9ab82518, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd328251b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe8251e, 33 },
    { 0, 0 },
    { 0x48b82520, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf682523, 61 },
    { 0x95582524, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84882529, 88 },
    { 0x96982529, 90 },
    { 0x6658252b, 95 },
    { 0xe758252c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f82530, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d82533, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c08253c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c82545, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b8254b, 95 },
    { 0x2488254c, 100 },
    { 0, 0 },
    { 0xc338254e, 95 },
    { 0xfe38254f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c582552, 33 },
    { 0, 0 },
    { 0x21682554, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe682557, 77 },
    { 0x3f582557, 33 },
    { 0, 0 },
    { 0x75c8255a, 76 },
    { 0x4ea8255b, 100 },
    { 0x1ed8255c, 95 },
    { 0xe238255d, 61 },
    { 0xdf98255c, 33 },
    { 0, 0 },
    { 0xa6a82560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d482565, 76 },
    { 0xe4482565, 98 },
    { 0x81982567, 33 },
    { 0x6e582567, 98 },
    { 0xb8d82569, 85 },
    { 0x1908256a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a38256d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2082570, 91 },
    { 0xe9e82570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40382576, 33 },
    { 0x3ba82576, 98 },
    { 0x68082578, 76 },
    { 0x6cc82579, 38 },
    { 0x8f48257a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e8257f, 34 },
    { 0xc1b82580, 52 },
    { 0xb3282581, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e8258c, 33 },
    { 0xb9e8258c, 86 },
    { 0xacf8258e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4382593, 78 },
    { 0, 0 },
    { 0x62a82595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b482599, 33 },
    { 0xb0182599, 23 },
    { 0, 0 },
    { 0x1a98259c, 23 },
    { 0, 0 },
    { 0xfe68259e, 33 },
    { 0x1558259f, 93 },
    { 0x5628259e, 43 },
    { 0, 0 },
    { 0x4b6825a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02825aa, 75 },
    { 0x5aa825ab, 98 },
    { 0xe3f825ac, 61 },
    { 0x77d825ad, 77 },
    { 0, 0 },
    { 0x25825af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe825b2, 23 },
    { 0x1e6825b3, 43 },
    { 0x95c825b4, 90 },
    { 0x2e825b5, 95 },
    { 0xffe825b6, 61 },
    { 0xd0d825b4, 55 },
    { 0, 0 },
    { 0x805825b9, 76 },
    { 0, 0 },
    { 0x144825bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88825bf, 91 },
    { 0x8fd825bf, 91 },
    { 0x4f825c1, 81 },
    { 0, 0 },
    { 0xce5825c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698825c8, 23 },
    { 0, 0 },
    { 0xe43825ca, 55 },
    { 0x722825cb, 17 },
    { 0x885825cc, 98 },
    { 0, 0 },
    { 0x8c0825ce, 90 },
    { 0x603825ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4825d2, 61 },
    { 0, 0 },
    { 0x2ef825d4, 95 },
    { 0x5f7825d4, 33 },
    { 0xbd9825d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557825da, 98 },
    { 0x19d825da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x978825de, 76 },
    { 0x5fa825df, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd825e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d825ea, 38 },
    { 0xa38825eb, 84 },
    { 0x10e825ec, 98 },
    { 0x122825ed, 36 },
    { 0x204825ee, 90 },
    { 0xe17825ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd825f3, 64 },
    { 0x230825f4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4825f7, 33 },
    { 0, 0 },
    { 0x4ce825f9, 90 },
    { 0, 0 },
    { 0x56c825fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3825fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc82601, 90 },
    { 0, 0 },
    { 0xba282603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d82606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd82609, 68 },
    { 0x23682609, 90 },
    { 0xd348260b, 61 },
    { 0xd1d8260a, 95 },
    { 0x9e88260d, 95 },
    { 0xd378260e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88382611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47782614, 61 },
    { 0x4b482614, 34 },
    { 0xc8982616, 23 },
    { 0x5f982617, 76 },
    { 0x46182617, 48 },
    { 0xa8682619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48261d, 34 },
    { 0x8f38261e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8182621, 90 },
    { 0, 0 },
    { 0x4ec82623, 56 },
    { 0x9be82624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37682627, 98 },
    { 0xb1a82628, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a8262c, 90 },
    { 0, 0 },
    { 0xff48262e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27182632, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd82637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8263b, 80 },
    { 0x6b88263b, 23 },
    { 0, 0 },
    { 0x2b88263e, 91 },
    { 0, 0 },
    { 0x94182640, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99782648, 95 },
    { 0xa3282649, 34 },
    { 0xccc8264a, 85 },
    { 0, 0 },
    { 0xb028264c, 33 },
    { 0, 0 },
    { 0xfca8264e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x15182651, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d282658, 23 },
    { 0x96682659, 24 },
    { 0x1558265a, 90 },
    { 0x5a68265b, 100 },
    { 0x3c68265b, 90 },
    { 0xc168265d, 23 },
    { 0xb328265d, 11 },
    { 0x3aa8265f, 95 },
    { 0xbbf8265b, 66 },
    { 0xeb682661, 76 },
    { 0x7a082662, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b82670, 61 },
    { 0xe2e82671, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d82678, 99 },
    { 0xccf82679, 61 },
    { 0xd8c8267a, 98 },
    { 0x2fd8267b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3282682, 76 },
    { 0xc0982683, 55 },
    { 0, 0 },
    { 0x34f82685, 84 },
    { 0x1bf82686, 100 },
    { 0x8d082687, 61 },
    { 0x51e82686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e982695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d82699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c8269c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818269f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a826a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1826a6, 84 },
    { 0x5b4826a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f826b3, 55 },
    { 0xb76826b4, 33 },
    { 0, 0 },
    { 0xa5d826b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1826ba, 52 },
    { 0xb8a826ba, 33 },
    { 0, 0 },
    { 0x19826bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b826c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729826c9, 23 },
    { 0x2cb826c9, 85 },
    { 0x3a3826cb, 61 },
    { 0x953826cc, 55 },
    { 0x513826cc, 90 },
    { 0xcd7826ce, 90 },
    { 0xa60826cf, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e826d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e826d5, 43 },
    { 0x16d826d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d826db, 66 },
    { 0xd5c826dc, 84 },
    { 0x452826dd, 33 },
    { 0x90d826dc, 61 },
    { 0, 0 },
    { 0xf32826e0, 55 },
    { 0, 0 },
    { 0xe64826e2, 46 },
    { 0xbe826e3, 40 },
    { 0x9f7826e3, 76 },
    { 0x88b826e5, 76 },
    { 0x269826e4, 76 },
    { 0, 0 },
    { 0x2bc826e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72826ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d826f4, 90 },
    { 0x3d6826f5, 90 },
    { 0x766826f6, 55 },
    { 0xeee826f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a826fb, 76 },
    { 0xc50826fb, 90 },
    { 0x452826fd, 33 },
    { 0x1ff826fe, 98 },
    { 0x17a826ff, 23 },
    { 0xf46826fc, 66 },
    { 0xc5982700, 34 },
    { 0, 0 },
    { 0x7fe82703, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e82707, 98 },
    { 0x7e682708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a78270d, 61 },
    { 0x9c8270e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf282711, 33 },
    { 0xd5182711, 17 },
    { 0, 0 },
    { 0xe0882714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47a82718, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38271c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd882721, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc82729, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8272e, 91 },
    { 0xe678272f, 52 },
    { 0xbb18272f, 88 },
    { 0xddd8272e, 95 },
    { 0xa1f82732, 66 },
    { 0x7b282733, 90 },
    { 0x4ba82734, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4982737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908273e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb82749, 98 },
    { 0, 0 },
    { 0x74c8274b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86582750, 93 },
    { 0xaf482751, 33 },
    { 0, 0 },
    { 0x97882753, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d82757, 84 },
    { 0x7c882758, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca8275e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b82766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d8276a, 65 },
    { 0, 0 },
    { 0xbbe8276c, 98 },
    { 0x9748276d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61482771, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b82778, 95 },
    { 0, 0 },
    { 0x28d8277a, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xc858277d, 61 },
    { 0, 0 },
    { 0xbdf8277f, 61 },
    { 0xd0282780, 98 },
    { 0xa4f82781, 33 },
    { 0, 0 },
    { 0x3cd82783, 98 },
    { 0xaa482784, 33 },
    { 0xdad82785, 93 },
    { 0x1ec82783, 65 },
    { 0x5ad82787, 33 },
    { 0xe982786, 95 },
    { 0x33182787, 84 },
    { 0, 0 },
    { 0x6088278b, 98 },
    { 0x3e38278c, 90 },
    { 0xc508278d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6882795, 68 },
    { 0x1bf82796, 61 },
    { 0x4af82796, 90 },
    { 0xfef82798, 76 },
    { 0, 0 },
    { 0x26e8279a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe248279e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563827a2, 95 },
    { 0x24827a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9827a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d827ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3827b0, 43 },
    { 0, 0 },
    { 0xace827b2, 48 },
    { 0x585827b3, 91 },
    { 0, 0 },
    { 0xc99827b5, 46 },
    { 0, 0 },
    { 0xe3e827b7, 55 },
    { 0xbe8827b7, 95 },
    { 0x921827b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807827c5, 58 },
    { 0xef8827c6, 89 },
    { 0x9a1827c7, 47 },
    { 0xe73827c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9827d1, 66 },
    { 0, 0 },
    { 0x85b827d3, 90 },
    { 0x165827d4, 33 },
    { 0x428827d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb827d8, 33 },
    { 0, 0 },
    { 0xf3b827da, 66 },
    { 0, 0 },
    { 0x141827dc, 98 },
    { 0, 0 },
    { 0x5d827de, 76 },
    { 0x22d827df, 84 },
    { 0, 0 },
    { 0x8fc827e1, 100 },
    { 0x451827e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3827e5, 90 },
    { 0xe65827e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e827e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x132827ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5827f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec827f6, 90 },
    { 0x850827f7, 33 },
    { 0xd96827f6, 90 },
    { 0x235827f9, 55 },
    { 0xc0a827f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb827ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e82807, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3408280a, 55 },
    { 0, 0 },
    { 0x3ef8280c, 33 },
    { 0xe2c8280d, 87 },
    { 0, 0 },
    { 0x1fa8280f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df82812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2682816, 91 },
    { 0xd7e82816, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d78281a, 61 },
    { 0, 0 },
    { 0x16f8281c, 77 },
    { 0xdc78281d, 98 },
    { 0x4968281c, 33 },
    { 0, 0 },
    { 0x7a882820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b82826, 87 },
    { 0x45782826, 31 },
    { 0, 0 },
    { 0xe9d82829, 67 },
    { 0xe608282a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe078282f, 83 },
    { 0x5fc82830, 46 },
    { 0xa5982830, 98 },
    { 0x9298282f, 90 },
    { 0xa2d82831, 76 },
    { 0x9b982834, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8283a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad58283e, 100 },
    { 0xeaf8283f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1382843, 33 },
    { 0x62582844, 95 },
    { 0xebd82843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75182848, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7898284b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3118284f, 98 },
    { 0, 0 },
    { 0xf1b82851, 33 },
    { 0, 0 },
    { 0x4fa82853, 34 },
    { 0x81282854, 34 },
    { 0x1a982855, 90 },
    { 0, 0 },
    { 0xc4382857, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a8285c, 33 },
    { 0, 0 },
    { 0x5bc8285e, 61 },
    { 0x9628285f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3182863, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd682866, 93 },
    { 0xacf82867, 33 },
    { 0xe8a82868, 43 },
    { 0, 0 },
    { 0x55b8286a, 61 },
    { 0xb638286b, 23 },
    { 0x3a68286a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc82873, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2908287c, 98 },
    { 0, 0 },
    { 0xeb88287e, 77 },
    { 0, 0 },
    { 0x8d382880, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe182886, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe482889, 52 },
    { 0xe7682889, 90 },
    { 0x42a8288b, 98 },
    { 0x9028288c, 88 },
    { 0x7578288d, 95 },
    { 0xe9d82889, 98 },
    { 0x56982889, 95 },
    { 0x1f82890, 68 },
    { 0x2e082890, 100 },
    { 0x6598288e, 43 },
    { 0xe8a82893, 33 },
    { 0xb0d82894, 52 },
    { 0xd4282893, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb082898, 23 },
    { 0, 0 },
    { 0xeb08289a, 25 },
    { 0x5c18289b, 23 },
    { 0xd478289c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e8289f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1828a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba828a5, 98 },
    { 0, 0 },
    { 0xed2828a7, 61 },
    { 0x91c828a8, 90 },
    { 0, 0 },
    { 0xd55828aa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb828ad, 95 },
    { 0xd57828ae, 98 },
    { 0x2aa828ad, 85 },
    { 0x9f6828b0, 33 },
    { 0x51f828ae, 80 },
    { 0x7ba828b2, 95 },
    { 0, 0 },
    { 0x8d7828b4, 75 },
    { 0xf1b828b5, 87 },
    { 0xc02828b6, 95 },
    { 0, 0 },
    { 0xd2d828b8, 95 },
    { 0x3e1828b9, 46 },
    { 0x138828b9, 84 },
    { 0x352828bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff828be, 84 },
    { 0x634828be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c828c9, 90 },
    { 0x4cb828ca, 61 },
    { 0x97f828ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d828d1, 55 },
    { 0x41e828d1, 76 },
    { 0xa90828d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0828dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09828e9, 76 },
    { 0xf84828ea, 98 },
    { 0, 0 },
    { 0x530828ec, 78 },
    { 0, 0 },
    { 0xc6a828ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b828f2, 64 },
    { 0, 0 },
    { 0xb6828f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3828f8, 77 },
    { 0, 0 },
    { 0xb62828fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a82901, 90 },
    { 0xfca82902, 66 },
    { 0x52082903, 95 },
    { 0x96282901, 87 },
    { 0x58682905, 77 },
    { 0x17882902, 43 },
    { 0, 0 },
    { 0x24d82908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb588290d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b882910, 61 },
    { 0xadd82910, 23 },
    { 0, 0 },
    { 0x96482913, 93 },
    { 0, 0 },
    { 0xd0782915, 61 },
    { 0, 0 },
    { 0xe1182917, 33 },
    { 0, 0 },
    { 0x7c482919, 85 },
    { 0x7158291a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5582928, 90 },
    { 0x6b482929, 78 },
    { 0x1098292a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc8292f, 61 },
    { 0, 0 },
    { 0xcf782931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f82934, 67 },
    { 0x20582935, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a82939, 33 },
    { 0xfc58293a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf48293d, 98 },
    { 0x98c8293d, 61 },
    { 0xd038293e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d82942, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f98294b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f8294e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81182952, 76 },
    { 0x37582953, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26782959, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc8295e, 23 },
    { 0, 0 },
    { 0x72b82960, 90 },
    { 0, 0 },
    { 0xe8282962, 61 },
    { 0, 0 },
    { 0x9bb82964, 84 },
    { 0xc4c82965, 80 },
    { 0x2d782965, 33 },
    { 0x6cc82967, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf358296c, 98 },
    { 0xbd28296d, 34 },
    { 0x5898296e, 53 },
    { 0xe568296d, 90 },
    { 0x1d082970, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f482974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b82977, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9788297b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2958297f, 98 },
    { 0x9ef8297f, 76 },
    { 0, 0 },
    { 0xec82982, 19 },
    { 0xab682983, 43 },
    { 0x7d482984, 95 },
    { 0x19a82985, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae8298a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b48298d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f982990, 34 },
    { 0, 0 },
    { 0x95982992, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa182997, 98 },
    { 0xdca82997, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0829a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d829a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42829ab, 99 },
    { 0x38e829ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3829b2, 33 },
    { 0xd8f829b3, 55 },
    { 0, 0 },
    { 0x27d829b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e829bd, 17 },
    { 0, 0 },
    { 0x43d829bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c829c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8829ce, 67 },
    { 0xa95829ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737829d3, 69 },
    { 0x46d829d4, 61 },
    { 0x6d5829d3, 85 },
    { 0x15b829d5, 95 },
    { 0x10b829d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6829df, 33 },
    { 0, 0 },
    { 0xd38829e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d829e4, 23 },
    { 0x1cf829e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e829e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc829ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9829f0, 65 },
    { 0xd90829f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd482a01, 90 },
    { 0x48e82a02, 76 },
    { 0x1a382a03, 33 },
    { 0xe3582a01, 95 },
    { 0x21082a03, 98 },
    { 0, 0 },
    { 0x14982a07, 90 },
    { 0xb8782a08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e82a0c, 33 },
    { 0xb7682a0d, 76 },
    { 0xf5082a0c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb282a1c, 43 },
    { 0x49782a1d, 61 },
    { 0xd8982a1e, 90 },
    { 0x74582a1f, 90 },
    { 0xf0982a20, 78 },
    { 0xbea82a21, 90 },
    { 0xeee82a22, 33 },
    { 0, 0 },
    { 0x18682a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b282a28, 77 },
    { 0, 0 },
    { 0x65b82a2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97682a33, 61 },
    { 0x6fc82a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef82a39, 34 },
    { 0xdc582a3a, 95 },
    { 0x59582a3b, 33 },
    { 0x61582a3c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2582a42, 61 },
    { 0xa7b82a43, 61 },
    { 0xd0182a43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2282a49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d82a4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8082a51, 61 },
    { 0x83782a52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34182a5b, 67 },
    { 0, 0 },
    { 0x15782a5d, 98 },
    { 0, 0 },
    { 0xbf182a5f, 95 },
    { 0xaec82a60, 61 },
    { 0xfe282a5f, 90 },
    { 0xc2c82a62, 46 },
    { 0x3af82a63, 90 },
    { 0xe2b82a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c82a68, 46 },
    { 0x74c82a69, 75 },
    { 0x4c382a6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb282a6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e82a70, 84 },
    { 0, 0 },
    { 0x86a82a72, 95 },
    { 0x17082a73, 99 },
    { 0x56582a72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e82a78, 55 },
    { 0x45182a78, 95 },
    { 0x25282a79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b82a7d, 23 },
    { 0xe1482a7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17582a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61382a8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a82a8e, 87 },
    { 0x30382a8f, 78 },
    { 0x97d82a90, 33 },
    { 0x1f82a90, 23 },
    { 0x12282a92, 43 },
    { 0xbf482a91, 76 },
    { 0xdbc82a93, 76 },
    { 0x25782a95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a682a98, 98 },
    { 0xb2782a99, 76 },
    { 0x58282a9a, 90 },
    { 0xeac82a9b, 77 },
    { 0xafb82a9c, 33 },
    { 0x8fa82a9d, 55 },
    { 0x4d182a9d, 78 },
    { 0xc0182a9d, 95 },
    { 0, 0 },
    { 0x76f82aa1, 87 },
    { 0xc5882aa2, 34 },
    { 0x6f882aa1, 19 },
    { 0, 0 },
    { 0x2c882aa5, 85 },
    { 0xa7b82aa6, 90 },
    { 0xca282aa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c82aac, 23 },
    { 0xbf882aad, 33 },
    { 0xc1482aac, 33 },
    { 0xffa82aac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c082ab2, 90 },
    { 0x36d82ab3, 46 },
    { 0, 0 },
    { 0xa3982ab5, 93 },
    { 0x1f182ab6, 95 },
    { 0x6bd82ab7, 33 },
    { 0xd0182ab8, 95 },
    { 0, 0 },
    { 0xe4982aba, 90 },
    { 0, 0 },
    { 0xf2182abc, 61 },
    { 0, 0 },
    { 0x37082abe, 77 },
    { 0xe3782abf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b82ac5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd82acb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40382ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c82ad2, 34 },
    { 0xd0d82ad3, 61 },
    { 0x76d82ad3, 61 },
    { 0, 0 },
    { 0x45c82ad6, 43 },
    { 0xa6982ad6, 61 },
    { 0x1e382ad8, 76 },
    { 0x35082ad9, 98 },
    { 0xb8282ad8, 76 },
    { 0xe9882adb, 23 },
    { 0x9d382ad6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4182ae6, 90 },
    { 0x8882ae7, 43 },
    { 0xf4a82ae8, 17 },
    { 0xa0f82ae8, 95 },
    { 0xe0882ae6, 75 },
    { 0x7ad82aeb, 93 },
    { 0, 0 },
    { 0xc2582aed, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda82af2, 34 },
    { 0x3cd82af3, 87 },
    { 0, 0 },
    { 0xc1f82af5, 76 },
    { 0, 0 },
    { 0xa2a82af7, 33 },
    { 0xb7f82af8, 95 },
    { 0x75882af9, 99 },
    { 0, 0 },
    { 0x4182afb, 33 },
    { 0, 0 },
    { 0x6a882afd, 23 },
    { 0x56d82afe, 93 },
    { 0, 0 },
    { 0x51d82b00, 23 },
    { 0x79082b00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda282b04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa82b07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c582b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41382b13, 91 },
    { 0x9182b14, 17 },
    { 0xe1982b15, 55 },
    { 0, 0 },
    { 0x20182b17, 93 },
    { 0x27082b17, 95 },
    { 0x5f382b18, 95 },
    { 0, 0 },
    { 0x57982b1b, 95 },
    { 0x87182b1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3082b22, 43 },
    { 0xe9b82b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f182b2e, 46 },
    { 0, 0 },
    { 0x34b82b30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96482b41, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15682b46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7482b49, 23 },
    { 0x11182b49, 33 },
    { 0, 0 },
    { 0x9b882b4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xff682b4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d82b56, 46 },
    { 0x5c482b57, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e82b5a, 95 },
    { 0xf1182b5b, 90 },
    { 0xaef82b5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c782b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f82b65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8082b6d, 28 },
    { 0xdf682b6d, 98 },
    { 0xc6382b6d, 34 },
    { 0x77a82b70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a82b74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1682b7d, 55 },
    { 0x96882b7e, 43 },
    { 0x93c82b7d, 33 },
    { 0, 0 },
    { 0x7c782b81, 95 },
    { 0, 0 },
    { 0x83382b83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x39582b86, 43 },
    { 0x16882b87, 91 },
    { 0xd2582b88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8382b8c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b782b91, 78 },
    { 0x68a82b92, 91 },
    { 0x5ac82b93, 52 },
    { 0xd1682b94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d82b99, 87 },
    { 0, 0 },
    { 0xd7282b9b, 95 },
    { 0, 0 },
    { 0xa4182b9d, 77 },
    { 0xa2282b9e, 46 },
    { 0xeef82b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f82ba2, 78 },
    { 0, 0 },
    { 0x85282ba4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd82ba7, 76 },
    { 0x7eb82ba8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0582bac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c82bb3, 43 },
    { 0, 0 },
    { 0x64182bb5, 33 },
    { 0x12282bb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f82bba, 33 },
    { 0xdaf82bbb, 83 },
    { 0xdf082bbc, 76 },
    { 0xd2a82bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad82bc1, 43 },
    { 0, 0 },
    { 0xfd182bc3, 67 },
    { 0xd6582bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61782bc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d82bcc, 34 },
    { 0xbc982bcd, 87 },
    { 0x8dd82bce, 46 },
    { 0x98a82bcd, 33 },
    { 0, 0 },
    { 0x31082bd1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce82bd6, 43 },
    { 0, 0 },
    { 0x95c82bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef82bdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe082bdf, 33 },
    { 0x77082be0, 33 },
    { 0x69582be1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f82be5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb82beb, 38 },
    { 0xa5c82bec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd82bf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c82bf3, 33 },
    { 0xa6682bf4, 90 },
    { 0xad082bf5, 77 },
    { 0xa382bf4, 90 },
    { 0x83e82bf7, 46 },
    { 0xfdb82bf7, 61 },
    { 0x75682bf3, 55 },
    { 0x12982bfa, 93 },
    { 0x3de82bfb, 90 },
    { 0, 0 },
    { 0x64382bfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc82c07, 78 },
    { 0x27482c08, 23 },
    { 0, 0 },
    { 0x54882c0a, 61 },
    { 0xd5382c0a, 23 },
    { 0x1e182c0c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d682c12, 23 },
    { 0x9f82c13, 33 },
    { 0x2d182c14, 33 },
    { 0, 0 },
    { 0xc182c16, 95 },
    { 0x84f82c17, 34 },
    { 0, 0 },
    { 0x8f82c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96582c20, 27 },
    { 0, 0 },
    { 0x41582c22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e82c25, 33 },
    { 0xf1682c26, 78 },
    { 0xec882c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83082c2d, 98 },
    { 0xbfd82c2e, 90 },
    { 0x1c182c2f, 61 },
    { 0, 0 },
    { 0x4db82c31, 33 },
    { 0, 0 },
    { 0xd6482c33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd82c3a, 95 },
    { 0x6c582c3b, 23 },
    { 0, 0 },
    { 0x12082c3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1782c4b, 33 },
    { 0x5e182c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f82c4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc982c52, 76 },
    { 0x57d82c53, 34 },
    { 0x6cb82c54, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32482c63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff82c6a, 99 },
    { 0x1c82c6b, 87 },
    { 0x21f82c6c, 76 },
    { 0xb6582c6d, 61 },
    { 0x86082c6c, 23 },
    { 0x64482c6c, 43 },
    { 0x38f82c70, 93 },
    { 0x48782c6c, 66 },
    { 0x12282c72, 90 },
    { 0x8fd82c6d, 67 },
    { 0x19582c74, 85 },
    { 0x20f82c6e, 36 },
    { 0x83182c74, 33 },
    { 0, 0 },
    { 0xa1e82c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb82c7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e82c80, 81 },
    { 0, 0 },
    { 0x6982c82, 61 },
    { 0x6182c83, 85 },
    { 0x37a82c84, 66 },
    { 0, 0 },
    { 0x26b82c86, 90 },
    { 0x15c82c86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb382c8b, 55 },
    { 0xa2e82c8c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa382c91, 61 },
    { 0xae682c92, 84 },
    { 0xc9b82c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c982c9b, 100 },
    { 0x26082c9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e82ca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d82ca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a82cac, 23 },
    { 0xc9182cad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea82cb2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc982cb6, 34 },
    { 0, 0 },
    { 0xe0d82cb8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x69382cbb, 23 },
    { 0x63582cbc, 87 },
    { 0xb9a82cbd, 84 },
    { 0x97882cbe, 61 },
    { 0, 0 },
    { 0x43b82cc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2682cc3, 34 },
    { 0xa3982cc3, 98 },
    { 0x16d82cc5, 67 },
    { 0, 0 },
    { 0x35e82cc7, 33 },
    { 0x3f82cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41882cd1, 61 },
    { 0xcf682cd2, 61 },
    { 0, 0 },
    { 0x9e782cd4, 95 },
    { 0x66b82cd5, 9 },
    { 0x69882cd4, 97 },
    { 0x70582cd6, 33 },
    { 0, 0 },
    { 0x90082cd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4182cdf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b382ce2, 90 },
    { 0x1c282ce3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b82ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d082cf1, 90 },
    { 0x79a82cf1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf82cfb, 33 },
    { 0x9982cfc, 90 },
    { 0x1ee82cfd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42182d04, 61 },
    { 0, 0 },
    { 0xa3282d06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea82d09, 61 },
    { 0xc7c82d0a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86882d0f, 77 },
    { 0x94882d10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a82d14, 95 },
    { 0x89382d15, 90 },
    { 0x18782d16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba82d1e, 95 },
    { 0x19e82d1e, 61 },
    { 0, 0 },
    { 0x4d582d21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x91482d24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa182d27, 33 },
    { 0x90682d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e782d2c, 61 },
    { 0x69282d2d, 95 },
    { 0x87a82d2e, 95 },
    { 0x35882d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c282d36, 23 },
    { 0x73b82d37, 88 },
    { 0x84382d38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed682d42, 61 },
    { 0xcb182d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4182d47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83782d4b, 61 },
    { 0x64882d4b, 91 },
    { 0xef382d4b, 43 },
    { 0x35e82d4d, 61 },
    { 0xb5e82d4b, 23 },
    { 0x31882d4b, 33 },
    { 0x43282d4f, 76 },
    { 0x97482d52, 95 },
    { 0, 0 },
    { 0xb1582d54, 81 },
    { 0, 0 },
    { 0x19482d56, 90 },
    { 0x76a82d57, 95 },
    { 0x56482d57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc182d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8282d5f, 77 },
    { 0, 0 },
    { 0x22782d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d182d65, 23 },
    { 0, 0 },
    { 0x4b682d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18982d6a, 66 },
    { 0x44882d6b, 90 },
    { 0x78e82d6c, 33 },
    { 0, 0 },
    { 0x98c82d6e, 46 },
    { 0x7d882d6e, 90 },
    { 0, 0 },
    { 0xd3782d71, 76 },
    { 0x1fb82d72, 34 },
    { 0, 0 },
    { 0x9a782d74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef82d7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75982d84, 46 },
    { 0, 0 },
    { 0x82b82d86, 95 },
    { 0xd2382d87, 98 },
    { 0x2d182d88, 70 },
    { 0xde82d89, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d82d90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23182d96, 23 },
    { 0x45582d97, 76 },
    { 0x55782d97, 91 },
    { 0x14b82d97, 98 },
    { 0x30882d98, 81 },
    { 0xdc082d9b, 23 },
    { 0x36182d9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c882d9f, 55 },
    { 0x5c282d9f, 90 },
    { 0, 0 },
    { 0xc8782da2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f82da6, 100 },
    { 0x1d982da6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd82dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9682db8, 65 },
    { 0, 0 },
    { 0x82f82dba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d982dbd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62882dc2, 23 },
    { 0x2e582dc3, 33 },
    { 0, 0 },
    { 0x78c82dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58382dcb, 100 },
    { 0x59882dcc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x60682dcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5782dd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b82dd6, 76 },
    { 0, 0 },
    { 0x5e282dd8, 5 },
    { 0xe4882dd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37182ddd, 34 },
    { 0xf2182dde, 33 },
    { 0x94c82dde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde182de2, 98 },
    { 0x1ad82de3, 90 },
    { 0xacb82de3, 95 },
    { 0xf6e82de5, 34 },
    { 0x49082de6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e82dec, 61 },
    { 0x8ae82ded, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd82dfd, 100 },
    { 0, 0 },
    { 0x82a82dff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24882e07, 61 },
    { 0xa6382e08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b82e0b, 100 },
    { 0x30082e0c, 23 },
    { 0xb7c82e0c, 76 },
    { 0xab482e0d, 76 },
    { 0xca682e0e, 76 },
    { 0xe2482e0b, 34 },
    { 0, 0 },
    { 0xdb382e12, 61 },
    { 0x79d82e12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca82e1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58582e1f, 98 },
    { 0, 0 },
    { 0xa6c82e21, 61 },
    { 0xdfa82e21, 93 },
    { 0xfa982e23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8482e27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1782e2b, 93 },
    { 0x8a782e2b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c682e31, 61 },
    { 0x87d82e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b82e3e, 46 },
    { 0xf6382e3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c882e43, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d82e47, 100 },
    { 0x43582e48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34882e4c, 90 },
    { 0, 0 },
    { 0x96282e4e, 78 },
    { 0, 0 },
    { 0x6f82e50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1782e57, 77 },
    { 0x1fe82e57, 61 },
    { 0x24682e58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd082e5f, 33 },
    { 0xeeb82e60, 61 },
    { 0x52a82e61, 33 },
    { 0xe9b82e62, 61 },
    { 0x96182e63, 98 },
    { 0xac082e64, 87 },
    { 0x2382e65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x99882e68, 33 },
    { 0, 0 },
    { 0xd6382e6a, 87 },
    { 0x4a182e6a, 66 },
    { 0x78182e6a, 83 },
    { 0x65782e6c, 98 },
    { 0x93d82e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb82e72, 33 },
    { 0x7ef82e72, 61 },
    { 0x60b82e72, 98 },
    { 0xe9282e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5982e78, 33 },
    { 0, 0 },
    { 0x2ff82e7a, 66 },
    { 0, 0 },
    { 0xca882e7c, 90 },
    { 0x19f82e7d, 95 },
    { 0x54682e7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e982e82, 90 },
    { 0x96c82e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de82e86, 56 },
    { 0x8f582e87, 95 },
    { 0xed482e87, 76 },
    { 0xbd882e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e382e8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd182e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88582e95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e582e9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce082e9f, 23 },
    { 0, 0 },
    { 0xed582ea1, 55 },
    { 0xd4d82ea1, 61 },
    { 0, 0 },
    { 0x24682ea4, 46 },
    { 0x9b482ea4, 23 },
    { 0xd2282ea6, 46 },
    { 0xca582ea7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8182eaa, 23 },
    { 0x6c082eab, 98 },
    { 0x92c82eac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f682eb0, 52 },
    { 0x76782eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa182eb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90182eb7, 66 },
    { 0, 0 },
    { 0x1bb82eb9, 98 },
    { 0x17582eb9, 93 },
    { 0x68782ebb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df82ebe, 23 },
    { 0x73782ebf, 61 },
    { 0xb0782ebe, 55 },
    { 0, 0 },
    { 0x5bb82ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d982ec5, 52 },
    { 0x48d82ec5, 95 },
    { 0xf3982ec7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9482ed0, 61 },
    { 0x36782ed1, 90 },
    { 0x48082ed2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3782edb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xde482ede, 76 },
    { 0x62482edf, 34 },
    { 0x8e682ede, 23 },
    { 0x46582ee1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade82ee5, 33 },
    { 0xbc282ee6, 90 },
    { 0x69c82ee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5082eeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7082eee, 100 },
    { 0x83082eef, 98 },
    { 0x54482ef0, 52 },
    { 0x8f782eef, 43 },
    { 0xabf82ef2, 23 },
    { 0xfe282ef0, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35782ef7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf82efb, 61 },
    { 0x15482efb, 98 },
    { 0xcdd82efd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb482f03, 78 },
    { 0, 0 },
    { 0x6082f05, 61 },
    { 0x53082f05, 33 },
    { 0x71e82f07, 66 },
    { 0xec82f08, 90 },
    { 0, 0 },
    { 0xd0282f0a, 90 },
    { 0x37c82f0b, 53 },
    { 0x1fa82f0c, 97 },
    { 0, 0 },
    { 0xacc82f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae082f12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e082f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2782f19, 87 },
    { 0, 0 },
    { 0x6ce82f1b, 98 },
    { 0xbee82f1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8782f1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d82f25, 61 },
    { 0xc0d82f26, 33 },
    { 0, 0 },
    { 0x49f82f28, 90 },
    { 0x59982f29, 23 },
    { 0, 0 },
    { 0xa0782f2b, 98 },
    { 0x12582f2c, 33 },
    { 0x12882f2c, 61 },
    { 0xcca82f2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b82f32, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87682f36, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b82f3e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b82f47, 46 },
    { 0, 0 },
    { 0x11f82f49, 68 },
    { 0x17f82f49, 98 },
    { 0x76282f4b, 95 },
    { 0x2b182f4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37682f4f, 88 },
    { 0, 0 },
    { 0xef282f51, 81 },
    { 0, 0 },
    { 0x1cc82f53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a282f57, 90 },
    { 0x6f282f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a82f5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69282f5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1582f64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91182f6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9082f6f, 95 },
    { 0x43f82f6f, 55 },
    { 0x54b82f71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4182f78, 98 },
    { 0xa5282f78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80882f84, 95 },
    { 0, 0 },
    { 0xd7c82f86, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b82f89, 33 },
    { 0xa4382f89, 43 },
    { 0x3de82f8b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc82f8e, 33 },
    { 0x83e82f8f, 65 },
    { 0x8b482f8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58182f95, 95 },
    { 0x84782f96, 33 },
    { 0x8dc82f97, 91 },
    { 0x6c682f98, 33 },
    { 0xbf582f99, 23 },
    { 0x7a882f99, 100 },
    { 0xabb82f9b, 55 },
    { 0x73382f9b, 90 },
    { 0x36f82f99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27782fa9, 76 },
    { 0x23282fa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e82fad, 61 },
    { 0, 0 },
    { 0x99182faf, 90 },
    { 0xa382fb0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96782fb6, 68 },
    { 0, 0 },
    { 0x99682fb8, 98 },
    { 0x2a582fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c82fbf, 98 },
    { 0x3e582fc0, 98 },
    { 0x37782fc1, 78 },
    { 0x4eb82fc0, 34 },
    { 0x7ab82fc3, 66 },
    { 0xbe82fc4, 90 },
    { 0x11182fc4, 98 },
    { 0x4d882fc2, 33 },
    { 0, 0 },
    { 0x71882fc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba82fcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66182fd1, 23 },
    { 0x5e782fd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c82fd6, 46 },
    { 0x45c82fd7, 96 },
    { 0xa4982fd8, 76 },
    { 0x20882fd7, 93 },
    { 0xfd982fda, 34 },
    { 0x6fd82fda, 66 },
    { 0xb7182fdb, 40 },
    { 0xef982fdd, 95 },
    { 0, 0 },
    { 0x85c82fdf, 87 },
    { 0xa8d82fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb282fe7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c282fef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60982ff3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e482ff9, 23 },
    { 0x44b82ffa, 78 },
    { 0x35e82ffb, 90 },
    { 0x9b82ffc, 95 },
    { 0, 0 },
    { 0x36182ffe, 98 },
    { 0xa2182fff, 98 },
    { 0xe5682fff, 46 },
    { 0x5683001, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c183004, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd83007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5358300b, 33 },
    { 0xde38300c, 77 },
    { 0xd0b8300d, 90 },
    { 0x2918300e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df83012, 61 },
    { 0, 0 },
    { 0x13283014, 23 },
    { 0xa2a83014, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd883018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1728301c, 61 },
    { 0xc528301d, 55 },
    { 0, 0 },
    { 0x6cd8301f, 23 },
    { 0xdb783020, 55 },
    { 0x8bc83020, 33 },
    { 0x4d183022, 87 },
    { 0, 0 },
    { 0x96b83024, 52 },
    { 0x6dc83024, 95 },
    { 0, 0 },
    { 0x9f583027, 23 },
    { 0xda083028, 43 },
    { 0x5083029, 76 },
    { 0, 0 },
    { 0x3888302b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8302e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66583033, 19 },
    { 0x8cf83034, 23 },
    { 0, 0 },
    { 0x17e83036, 61 },
    { 0x5eb83036, 66 },
    { 0xd2a83038, 23 },
    { 0, 0 },
    { 0x84a8303a, 61 },
    { 0x4d28303b, 51 },
    { 0x90f8303a, 90 },
    { 0x4328303b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x30483040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2883043, 33 },
    { 0xb7883043, 33 },
    { 0, 0 },
    { 0x1a83046, 23 },
    { 0x94983046, 95 },
    { 0x4a283048, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc018304b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b83050, 90 },
    { 0x1d883051, 93 },
    { 0x46f83052, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca83055, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a83058, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e8305f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c483063, 33 },
    { 0, 0 },
    { 0xc1f83065, 98 },
    { 0xcc283066, 78 },
    { 0xa1783067, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d8306c, 55 },
    { 0x5988306d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33983077, 95 },
    { 0, 0 },
    { 0x66f83079, 61 },
    { 0x5648307a, 98 },
    { 0xd938307b, 95 },
    { 0, 0 },
    { 0x9ab8307d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb83083, 23 },
    { 0xbf883084, 90 },
    { 0xd3d83084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d83088, 80 },
    { 0x89983089, 33 },
    { 0, 0 },
    { 0xec98308b, 88 },
    { 0xc8b8308c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c8308f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d83092, 90 },
    { 0, 0 },
    { 0x3b983094, 68 },
    { 0x34983094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24783099, 76 },
    { 0x7fd8309a, 23 },
    { 0xa398309b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3830a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c830a7, 98 },
    { 0xe04830a7, 83 },
    { 0x129830a9, 90 },
    { 0x3cd830a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0830af, 90 },
    { 0x9a6830b0, 87 },
    { 0, 0 },
    { 0xa58830b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03830b5, 17 },
    { 0, 0 },
    { 0x507830b7, 98 },
    { 0x9bf830b8, 90 },
    { 0xe80830b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398830bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf830c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6830c5, 81 },
    { 0xec6830c5, 61 },
    { 0xef3830c5, 61 },
    { 0x511830c8, 23 },
    { 0, 0 },
    { 0x1cd830ca, 52 },
    { 0x886830ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b830ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a830d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d830da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a830e4, 90 },
    { 0x72a830e5, 100 },
    { 0x1ac830e5, 61 },
    { 0, 0 },
    { 0x689830e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e830ee, 33 },
    { 0xab830ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd830f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a830f7, 26 },
    { 0x64e830f8, 33 },
    { 0xd34830f8, 61 },
    { 0, 0 },
    { 0x615830fb, 90 },
    { 0xd2b830fb, 95 },
    { 0, 0 },
    { 0x3c0830fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30983103, 77 },
    { 0x4683103, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3983113, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea83118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f18311d, 33 },
    { 0xdac8311e, 98 },
    { 0, 0 },
    { 0xb7e83120, 85 },
    { 0x7a183120, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f183124, 33 },
    { 0x1b983125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd738312a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35783130, 76 },
    { 0, 0 },
    { 0x53183132, 34 },
    { 0x5b283133, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f683139, 55 },
    { 0, 0 },
    { 0xadd8313b, 23 },
    { 0, 0 },
    { 0x3928313d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92783142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f83146, 91 },
    { 0x6bb83147, 61 },
    { 0xbfd83148, 23 },
    { 0, 0 },
    { 0xa588314a, 67 },
    { 0x3ca8314b, 66 },
    { 0, 0 },
    { 0x3ee8314d, 78 },
    { 0x3ec8314e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a983154, 95 },
    { 0x4f883155, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee983158, 34 },
    { 0, 0 },
    { 0x6548315a, 85 },
    { 0xdeb8315a, 33 },
    { 0x6d28315b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69483162, 91 },
    { 0x42783163, 75 },
    { 0, 0 },
    { 0x6e583165, 76 },
    { 0xa5e83165, 93 },
    { 0x3e183166, 52 },
    { 0xe2d83165, 23 },
    { 0xc1883167, 34 },
    { 0x96183169, 46 },
    { 0x68c8316b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c8316f, 78 },
    { 0x4bc83170, 23 },
    { 0, 0 },
    { 0xe9c83172, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b383177, 91 },
    { 0x24a83177, 52 },
    { 0xa5183179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7518317d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43583180, 33 },
    { 0xffc83181, 90 },
    { 0, 0 },
    { 0x59c83183, 90 },
    { 0xa3383184, 98 },
    { 0x5b083185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8318c, 90 },
    { 0x13b8318d, 90 },
    { 0x9838318e, 90 },
    { 0, 0 },
    { 0xb583190, 78 },
    { 0, 0 },
    { 0x52783192, 34 },
    { 0x68483193, 100 },
    { 0xfbc83194, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48183197, 68 },
    { 0xe383197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a88319d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4831a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2831a7, 99 },
    { 0xd1831a8, 23 },
    { 0x510831a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c831ac, 88 },
    { 0xba5831ac, 85 },
    { 0xa45831ae, 98 },
    { 0, 0 },
    { 0xef8831b0, 50 },
    { 0, 0 },
    { 0xdb831b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632831b9, 34 },
    { 0x2f831ba, 76 },
    { 0xd5b831ba, 33 },
    { 0, 0 },
    { 0xc03831bd, 100 },
    { 0xc15831bd, 76 },
    { 0x1f5831be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8831c2, 90 },
    { 0, 0 },
    { 0x601831c4, 61 },
    { 0xe38831c5, 95 },
    { 0x7c6831c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248831cc, 52 },
    { 0x587831cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5831d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e831da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x598831dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884831e3, 98 },
    { 0x99d831e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9831e8, 33 },
    { 0x272831e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1831ed, 61 },
    { 0, 0 },
    { 0xf52831ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4831f3, 55 },
    { 0, 0 },
    { 0x399831f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f831fa, 75 },
    { 0x617831fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac831fe, 33 },
    { 0x858831ff, 61 },
    { 0x1c383200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78683209, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf483210, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1483216, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c483219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538321c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea48321f, 61 },
    { 0xd9483220, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19183225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c983229, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f8322c, 98 },
    { 0x1268322d, 81 },
    { 0xf938322e, 95 },
    { 0x2dc8322c, 52 },
    { 0, 0 },
    { 0x1d483231, 93 },
    { 0x39483231, 61 },
    { 0, 0 },
    { 0x65283234, 61 },
    { 0, 0 },
    { 0x17983236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6728323a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xab08323d, 23 },
    { 0, 0 },
    { 0xda8323f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e183246, 46 },
    { 0x48683247, 34 },
    { 0x2a783248, 43 },
    { 0xd4c83249, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9583254, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8648325b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b78325e, 67 },
    { 0x4508325e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79083263, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66483267, 98 },
    { 0, 0 },
    { 0x1bb83269, 43 },
    { 0, 0 },
    { 0xbd48326b, 43 },
    { 0x4488326b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed38326f, 98 },
    { 0x5318326f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff83273, 96 },
    { 0, 0 },
    { 0xfa783275, 90 },
    { 0, 0 },
    { 0x77383277, 43 },
    { 0x7a283277, 33 },
    { 0, 0 },
    { 0xdc68327a, 76 },
    { 0, 0 },
    { 0xce68327c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc8327f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea83285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57983288, 61 },
    { 0xf8483288, 90 },
    { 0xba283288, 98 },
    { 0x4d683288, 98 },
    { 0x85d8328c, 46 },
    { 0x40983288, 33 },
    { 0, 0 },
    { 0x4628328f, 69 },
    { 0xc383290, 98 },
    { 0x5c983290, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc83294, 55 },
    { 0, 0 },
    { 0x42983296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908329b, 78 },
    { 0, 0 },
    { 0xead8329d, 98 },
    { 0xa468329e, 91 },
    { 0, 0 },
    { 0x2f832a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1832a3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520832ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x301832ae, 61 },
    { 0xb61832af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07832b2, 67 },
    { 0, 0 },
    { 0xbf0832b4, 91 },
    { 0xdc4832b4, 34 },
    { 0x81832b6, 90 },
    { 0, 0 },
    { 0xea2832b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15832bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400832bf, 90 },
    { 0, 0 },
    { 0xde7832c1, 34 },
    { 0x910832c2, 33 },
    { 0, 0 },
    { 0x2f6832c4, 98 },
    { 0x62e832c4, 33 },
    { 0x741832c6, 36 },
    { 0xae9832c7, 87 },
    { 0x71d832c4, 55 },
    { 0x143832c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94832ce, 95 },
    { 0, 0 },
    { 0x4ef832d0, 33 },
    { 0x43a832d0, 69 },
    { 0x5a6832d2, 78 },
    { 0x703832d1, 98 },
    { 0x465832d1, 95 },
    { 0, 0 },
    { 0x938832d6, 43 },
    { 0x7c9832d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f832da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f832de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2832e2, 75 },
    { 0x5d3832e3, 76 },
    { 0x6ce832e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b832e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8832f0, 100 },
    { 0xc1d832f1, 70 },
    { 0, 0 },
    { 0xf52832f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e832f6, 99 },
    { 0, 0 },
    { 0x1cc832f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8832fd, 66 },
    { 0x29832fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac883301, 81 },
    { 0x6fb83301, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a83306, 98 },
    { 0xc1983306, 59 },
    { 0x28f83308, 55 },
    { 0x51f83309, 61 },
    { 0x8a38330a, 34 },
    { 0, 0 },
    { 0x7a28330c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd708330f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96183313, 62 },
    { 0x37a83313, 23 },
    { 0xd8683314, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead8331d, 93 },
    { 0x3068331e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a83326, 95 },
    { 0, 0 },
    { 0x23983328, 34 },
    { 0xc4183329, 55 },
    { 0x38d8332a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c08332e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa83332, 46 },
    { 0, 0 },
    { 0x7883334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf83339, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb8333c, 76 },
    { 0xb868333d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83344, 33 },
    { 0, 0 },
    { 0xa6e83346, 67 },
    { 0xfdf83347, 95 },
    { 0x5d783348, 23 },
    { 0x1c183347, 100 },
    { 0, 0 },
    { 0xb048334b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f8334e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d83352, 55 },
    { 0x8fa83353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90283357, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce18335a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e83366, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f8336c, 99 },
    { 0x4188336d, 101 },
    { 0xca38336c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f383374, 33 },
    { 0x40c83374, 90 },
    { 0xb583376, 90 },
    { 0x14a83377, 55 },
    { 0x9ed83377, 46 },
    { 0xc0d83376, 93 },
    { 0x15183378, 34 },
    { 0xe288337b, 85 },
    { 0x93783376, 95 },
    { 0x29c83378, 90 },
    { 0x7408337e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3483381, 85 },
    { 0, 0 },
    { 0xb5383383, 46 },
    { 0x57983384, 76 },
    { 0xae183384, 33 },
    { 0x86883386, 66 },
    { 0xe1383387, 52 },
    { 0x2cc83385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c8338e, 89 },
    { 0x6838338e, 100 },
    { 0, 0 },
    { 0x56583391, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa838339e, 46 },
    { 0xafd8339f, 34 },
    { 0x615833a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x347833a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f833a7, 33 },
    { 0xf7b833a7, 33 },
    { 0, 0 },
    { 0x90833aa, 98 },
    { 0, 0 },
    { 0x7c833ac, 78 },
    { 0, 0 },
    { 0xcd1833ae, 52 },
    { 0x9f4833ae, 99 },
    { 0, 0 },
    { 0xd3833b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xace833b4, 52 },
    { 0xb84833b4, 95 },
    { 0xa7e833b5, 23 },
    { 0x24e833b5, 77 },
    { 0, 0 },
    { 0xa29833b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e833bf, 56 },
    { 0, 0 },
    { 0xa15833c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a833c4, 61 },
    { 0xd77833c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7833ce, 23 },
    { 0x224833cf, 23 },
    { 0, 0 },
    { 0xfda833d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16833d6, 43 },
    { 0x2de833d7, 93 },
    { 0x7bb833d7, 98 },
    { 0x839833d9, 87 },
    { 0x2d7833da, 93 },
    { 0x5f2833db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b833ec, 33 },
    { 0, 0 },
    { 0x8f1833ee, 33 },
    { 0, 0 },
    { 0x570833f0, 61 },
    { 0x942833f1, 99 },
    { 0, 0 },
    { 0x493833f3, 66 },
    { 0xe5833f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac833f8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196833fc, 95 },
    { 0, 0 },
    { 0x99f833fe, 48 },
    { 0xe0b833fe, 100 },
    { 0xd5f83400, 55 },
    { 0x54483401, 61 },
    { 0, 0 },
    { 0xd5f83403, 17 },
    { 0x2e683404, 98 },
    { 0, 0 },
    { 0xca283406, 33 },
    { 0xf1283407, 90 },
    { 0xaf483408, 61 },
    { 0x88e83408, 23 },
    { 0x6b78340a, 52 },
    { 0xc083408, 23 },
    { 0xd7d8340a, 98 },
    { 0x55c8340c, 33 },
    { 0x4cd8340e, 98 },
    { 0xbdd8340c, 87 },
    { 0xc168340e, 90 },
    { 0x42983406, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c83415, 23 },
    { 0x52d83415, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64083426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed88342a, 61 },
    { 0xb308342b, 98 },
    { 0xffa8342a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef8342f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8983435, 90 },
    { 0, 0 },
    { 0x3fb83437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d68343b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc748343f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b83443, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4183447, 95 },
    { 0xb5983447, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd8344d, 90 },
    { 0xb298344d, 98 },
    { 0x1c48344f, 61 },
    { 0x82a8344f, 77 },
    { 0x8278344f, 95 },
    { 0x94b83452, 87 },
    { 0xdec83452, 23 },
    { 0, 0 },
    { 0x2aa83455, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17483459, 84 },
    { 0, 0 },
    { 0x8ab8345b, 43 },
    { 0x1918345c, 90 },
    { 0, 0 },
    { 0x5168345e, 98 },
    { 0xd208345f, 55 },
    { 0x9a383460, 90 },
    { 0xd7d83461, 61 },
    { 0x2e48345e, 23 },
    { 0, 0 },
    { 0xfc283464, 23 },
    { 0x1db83465, 23 },
    { 0x82383465, 90 },
    { 0x46283465, 34 },
    { 0, 0 },
    { 0x30583469, 33 },
    { 0xabd83469, 33 },
    { 0, 0 },
    { 0x1ec8346c, 76 },
    { 0x4998346d, 77 },
    { 0x5698346e, 95 },
    { 0, 0 },
    { 0x77683470, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8083473, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d183476, 68 },
    { 0xf3e83477, 98 },
    { 0x40e83478, 90 },
    { 0xa9983479, 88 },
    { 0x31483479, 81 },
    { 0, 0 },
    { 0x5708347c, 100 },
    { 0xb298347d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc83480, 99 },
    { 0xedc83481, 52 },
    { 0xb0e83482, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e783485, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4718348a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528348f, 64 },
    { 0xd4b83490, 23 },
    { 0xc7583490, 23 },
    { 0x52d83490, 83 },
    { 0, 0 },
    { 0xd3c83494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae83497, 33 },
    { 0xf6883498, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1818349e, 64 },
    { 0x9f28349f, 46 },
    { 0x20a8349e, 33 },
    { 0xb048349e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c834a8, 46 },
    { 0, 0 },
    { 0x7bc834aa, 95 },
    { 0x4b9834ab, 90 },
    { 0xe95834ac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e834af, 87 },
    { 0x855834b0, 90 },
    { 0xc5a834b0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255834b5, 46 },
    { 0x47e834b6, 34 },
    { 0xff9834b6, 23 },
    { 0xab2834b8, 40 },
    { 0x360834b9, 43 },
    { 0xc32834b7, 99 },
    { 0, 0 },
    { 0x41b834bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95834c6, 33 },
    { 0xc62834c6, 55 },
    { 0x624834c8, 93 },
    { 0, 0 },
    { 0x68834ca, 98 },
    { 0x2ed834cb, 88 },
    { 0xef0834cc, 58 },
    { 0x40f834cc, 61 },
    { 0xc93834cb, 33 },
    { 0xa8c834cd, 61 },
    { 0x687834cd, 33 },
    { 0xbee834ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c834db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0834df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef834e4, 99 },
    { 0, 0 },
    { 0x42834e6, 90 },
    { 0x60a834e7, 90 },
    { 0x5b4834e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa834eb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd834f0, 99 },
    { 0x337834f1, 33 },
    { 0xa08834f2, 90 },
    { 0x504834f2, 95 },
    { 0xb21834f4, 37 },
    { 0xcc0834f5, 55 },
    { 0xe58834f1, 33 },
    { 0, 0 },
    { 0x4dc834f8, 23 },
    { 0x8e6834f8, 55 },
    { 0xffd834f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c834fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd83501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd83509, 52 },
    { 0xb5d8350a, 55 },
    { 0x3928350b, 90 },
    { 0x3e28350c, 88 },
    { 0xb688350c, 23 },
    { 0xe638350d, 23 },
    { 0x8ab8350c, 98 },
    { 0xa688350f, 61 },
    { 0x44183509, 34 },
    { 0x4483511, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c183515, 43 },
    { 0, 0 },
    { 0x7da83517, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb8351b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa888351e, 33 },
    { 0xfb28351f, 99 },
    { 0, 0 },
    { 0x41f83521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb83524, 33 },
    { 0xbce83524, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f8352a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef8352d, 33 },
    { 0x6fb8352e, 50 },
    { 0x8888352f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58983533, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xda483536, 98 },
    { 0x69d83537, 23 },
    { 0xeda83538, 51 },
    { 0, 0 },
    { 0x78353a, 90 },
    { 0xde08353a, 42 },
    { 0, 0 },
    { 0x8ae8353d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3283545, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc8354a, 95 },
    { 0xbc68354b, 23 },
    { 0x6288354c, 85 },
    { 0x2f38354d, 76 },
    { 0, 0 },
    { 0x86c8354f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27183554, 90 },
    { 0, 0 },
    { 0xc7583556, 76 },
    { 0x52383557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e8355e, 99 },
    { 0x8428355e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa83563, 82 },
    { 0x33483564, 87 },
    { 0x88a83563, 90 },
    { 0x67283566, 90 },
    { 0x37f83566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc8356a, 33 },
    { 0, 0 },
    { 0xb058356c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x168356f, 52 },
    { 0x22f8356f, 76 },
    { 0, 0 },
    { 0x28383572, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e83577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab78357b, 66 },
    { 0x5898357c, 33 },
    { 0x49b8357d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0083580, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b83585, 100 },
    { 0, 0 },
    { 0x98083587, 77 },
    { 0x90e83587, 100 },
    { 0xc1c83589, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8508358c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e583592, 33 },
    { 0, 0 },
    { 0x7b83594, 95 },
    { 0, 0 },
    { 0xf5083596, 85 },
    { 0xcde83596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede8359e, 23 },
    { 0x2b28359e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05835a3, 95 },
    { 0xb3c835a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1835ae, 43 },
    { 0x4f1835af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71835b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345835b8, 76 },
    { 0, 0 },
    { 0x14e835ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c835c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21835c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533835cd, 95 },
    { 0xd64835ce, 61 },
    { 0xbf1835cd, 34 },
    { 0x280835d0, 98 },
    { 0, 0 },
    { 0x4f9835d2, 23 },
    { 0xeec835d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5835e0, 90 },
    { 0xed2835e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48835e7, 64 },
    { 0xe7d835e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8835ef, 33 },
    { 0x777835ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x209835f3, 33 },
    { 0xb7c835f4, 76 },
    { 0xa84835f4, 91 },
    { 0xd6f835f4, 91 },
    { 0xf08835f3, 33 },
    { 0xdd8835f8, 33 },
    { 0, 0 },
    { 0xb8835fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c835fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd83602, 34 },
    { 0, 0 },
    { 0x2fd83604, 76 },
    { 0x7ec83604, 61 },
    { 0x683604, 33 },
    { 0xa2d83607, 80 },
    { 0, 0 },
    { 0x3a283609, 61 },
    { 0xb9e8360a, 90 },
    { 0x6e8360b, 93 },
    { 0x1038360b, 33 },
    { 0x4e18360d, 98 },
    { 0, 0 },
    { 0x3318360f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a183612, 90 },
    { 0xf6e83612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1783616, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e8361a, 95 },
    { 0x9ff8361b, 34 },
    { 0xe3f8361c, 23 },
    { 0x59b8361d, 43 },
    { 0x71d8361e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6883621, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92083627, 100 },
    { 0, 0 },
    { 0x48683629, 67 },
    { 0x4f28362a, 46 },
    { 0x7ee83629, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad8362f, 75 },
    { 0x3d083630, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d83637, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8628363c, 74 },
    { 0x2a48363d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab83643, 43 },
    { 0xd8e83644, 34 },
    { 0xd1a83644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638364c, 98 },
    { 0xb438364d, 95 },
    { 0xe5a8364d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1383653, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f83659, 76 },
    { 0x208365a, 99 },
    { 0, 0 },
    { 0x9008365c, 77 },
    { 0xaff8365c, 52 },
    { 0x6fd8365e, 46 },
    { 0x79f8365d, 90 },
    { 0x5248365d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5383664, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6283668, 98 },
    { 0, 0 },
    { 0x9378366a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d83672, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6583679, 23 },
    { 0xb1183679, 81 },
    { 0xa018367a, 33 },
    { 0, 0 },
    { 0x5848367d, 95 },
    { 0xe858367d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde283683, 33 },
    { 0x5083684, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d8368f, 98 },
    { 0x9d183690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9383694, 61 },
    { 0xaaa83695, 33 },
    { 0x78183694, 95 },
    { 0xc8183697, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d8369f, 76 },
    { 0, 0 },
    { 0xee2836a1, 23 },
    { 0xb7e836a2, 91 },
    { 0x264836a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c836a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38836ac, 99 },
    { 0x783836ad, 27 },
    { 0x312836ae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2836b5, 95 },
    { 0x8c3836b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x918836b9, 61 },
    { 0, 0 },
    { 0xb61836bb, 19 },
    { 0x3b5836bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62836c0, 61 },
    { 0x6d0836c0, 85 },
    { 0xe6e836c2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426836c6, 43 },
    { 0, 0 },
    { 0x3c8836c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9836cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea836d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8836d3, 61 },
    { 0xf99836d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf836d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87836da, 61 },
    { 0, 0 },
    { 0x992836dc, 90 },
    { 0x4dd836dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4836e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894836e5, 33 },
    { 0, 0 },
    { 0xa5b836e7, 90 },
    { 0xf39836e7, 95 },
    { 0xaf6836e7, 100 },
    { 0x156836ea, 98 },
    { 0xd55836ea, 76 },
    { 0xc9d836e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd836f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7836f7, 61 },
    { 0, 0 },
    { 0xc39836f9, 90 },
    { 0x293836fa, 23 },
    { 0, 0 },
    { 0xa70836fc, 33 },
    { 0x810836fd, 61 },
    { 0xce5836fc, 38 },
    { 0x193836fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87483703, 52 },
    { 0x94c83704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8370b, 33 },
    { 0, 0 },
    { 0x7c48370d, 76 },
    { 0, 0 },
    { 0xf868370f, 34 },
    { 0x23c8370f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa983715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd8371e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36183721, 76 },
    { 0x4e183721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e483731, 92 },
    { 0, 0 },
    { 0xd9c83733, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1978373a, 95 },
    { 0x32a8373b, 34 },
    { 0x2c68373c, 33 },
    { 0xdcc8373c, 85 },
    { 0x4248373e, 33 },
    { 0x2408373f, 99 },
    { 0, 0 },
    { 0x6fe83741, 90 },
    { 0, 0 },
    { 0xab83743, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c583747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd78374a, 53 },
    { 0x7b58374b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb883750, 78 },
    { 0xc3f83750, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d083756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef48375b, 90 },
    { 0x85a8375b, 90 },
    { 0xc258375b, 95 },
    { 0x76e8375e, 52 },
    { 0xc418375f, 81 },
    { 0x5df83760, 2 },
    { 0xcf883760, 46 },
    { 0xc0c83760, 93 },
    { 0xb3a8375c, 76 },
    { 0x19383764, 77 },
    { 0xc6883765, 90 },
    { 0x97283764, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3478376b, 98 },
    { 0xe448376c, 61 },
    { 0xb658376d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f383771, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4483776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f8377a, 95 },
    { 0x4a18377b, 61 },
    { 0x9908377c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c83783, 90 },
    { 0xf3983784, 90 },
    { 0xd5783785, 33 },
    { 0xed083784, 66 },
    { 0xc1f83784, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc48378a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38378d, 90 },
    { 0x82d8378e, 98 },
    { 0, 0 },
    { 0xef483790, 23 },
    { 0x9c83791, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b83798, 90 },
    { 0, 0 },
    { 0x8e38379a, 55 },
    { 0, 0 },
    { 0x4b08379c, 33 },
    { 0x66a8379d, 90 },
    { 0x8be8379d, 23 },
    { 0x55b8379d, 43 },
    { 0xe21837a0, 75 },
    { 0x2ab837a1, 23 },
    { 0x689837a2, 75 },
    { 0, 0 },
    { 0xb71837a4, 75 },
    { 0xde1837a5, 33 },
    { 0xf59837a5, 100 },
    { 0x1a1837a4, 85 },
    { 0x357837a8, 68 },
    { 0xa3a837a7, 33 },
    { 0x581837aa, 95 },
    { 0x97837aa, 61 },
    { 0x8e837ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0837af, 33 },
    { 0x574837b0, 61 },
    { 0x745837b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f837b5, 33 },
    { 0x4dd837b5, 95 },
    { 0x936837b7, 68 },
    { 0x649837b8, 23 },
    { 0x1ce837b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1837c1, 17 },
    { 0x36b837c2, 66 },
    { 0, 0 },
    { 0x820837c4, 99 },
    { 0, 0 },
    { 0xef4837c6, 84 },
    { 0x9e9837c7, 61 },
    { 0x434837c8, 65 },
    { 0x8fb837c8, 43 },
    { 0, 0 },
    { 0xd6c837cb, 33 },
    { 0xd1c837cc, 74 },
    { 0xde6837cd, 85 },
    { 0x1b8837ce, 61 },
    { 0x7a6837cf, 101 },
    { 0xdd8837ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e837d3, 61 },
    { 0xd2f837d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4837da, 76 },
    { 0x3e3837db, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbf837e5, 61 },
    { 0xab1837e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c837ea, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941837f0, 33 },
    { 0x616837f0, 84 },
    { 0xa7f837f1, 80 },
    { 0xd6e837f3, 76 },
    { 0xb63837f4, 55 },
    { 0xd86837f3, 23 },
    { 0x1f8837f4, 95 },
    { 0xe5d837f7, 90 },
    { 0x2a8837f8, 23 },
    { 0xf10837f7, 95 },
    { 0x9f1837f9, 23 },
    { 0xf26837f8, 55 },
    { 0x178837f3, 99 },
    { 0xc9c837fc, 84 },
    { 0, 0 },
    { 0x3f6837ff, 85 },
    { 0, 0 },
    { 0xcc683801, 61 },
    { 0, 0 },
    { 0x1a683803, 91 },
    { 0, 0 },
    { 0x2a883805, 98 },
    { 0, 0 },
    { 0x2e883807, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da83810, 76 },
    { 0xf3483811, 96 },
    { 0xcd383812, 33 },
    { 0x7fb83810, 99 },
    { 0xab83814, 33 },
    { 0x8fd83815, 23 },
    { 0xddb83815, 34 },
    { 0, 0 },
    { 0x61883818, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77083822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef383827, 98 },
    { 0, 0 },
    { 0xe4f83829, 65 },
    { 0, 0 },
    { 0x3ea8382b, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9883832, 93 },
    { 0x41d83833, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b83837, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b8383c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa608383f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20983845, 90 },
    { 0x73b83846, 91 },
    { 0x63183847, 90 },
    { 0x9ef83847, 90 },
    { 0x6e483847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd8384d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3783852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a583855, 33 },
    { 0x62c83856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb383859, 90 },
    { 0, 0 },
    { 0x9818385b, 98 },
    { 0xf488385c, 46 },
    { 0x9bf8385b, 23 },
    { 0x33c8385e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e83861, 85 },
    { 0x7a183861, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad83867, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d8386d, 61 },
    { 0xc4a8386e, 33 },
    { 0x9f48386d, 34 },
    { 0x62e83870, 88 },
    { 0x5028386d, 76 },
    { 0xff18386f, 30 },
    { 0x14083873, 43 },
    { 0, 0 },
    { 0xcdc83875, 33 },
    { 0xc6d83876, 78 },
    { 0x55683877, 98 },
    { 0x31d83875, 90 },
    { 0xa8c83876, 90 },
    { 0x683879, 46 },
    { 0, 0 },
    { 0xc4f8387c, 90 },
    { 0, 0 },
    { 0x1198387e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56783889, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa483895, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b83898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5838a0, 33 },
    { 0, 0 },
    { 0xb94838a2, 98 },
    { 0x657838a2, 61 },
    { 0, 0 },
    { 0x5e4838a5, 98 },
    { 0xc21838a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1838af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x884838b2, 55 },
    { 0, 0 },
    { 0xe4d838b4, 59 },
    { 0, 0 },
    { 0x1f3838b6, 17 },
    { 0x59a838b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3838bd, 95 },
    { 0xf8a838bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e838c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b838d4, 33 },
    { 0xf63838d5, 61 },
    { 0x3a4838d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32838de, 46 },
    { 0, 0 },
    { 0x445838e0, 61 },
    { 0xcc5838e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82838e5, 85 },
    { 0, 0 },
    { 0x5bd838e7, 61 },
    { 0xa8b838e8, 66 },
    { 0x6fa838e8, 90 },
    { 0, 0 },
    { 0x3d5838eb, 33 },
    { 0, 0 },
    { 0x42f838ed, 76 },
    { 0x31d838ee, 55 },
    { 0xf28838ef, 76 },
    { 0x52c838ef, 23 },
    { 0, 0 },
    { 0xf01838f2, 93 },
    { 0x1a6838f3, 61 },
    { 0, 0 },
    { 0xbb0838f5, 93 },
    { 0x19f838f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f838fc, 95 },
    { 0xf6f838fc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec583902, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b583906, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8390c, 95 },
    { 0, 0 },
    { 0xcf08390e, 33 },
    { 0xc4b8390f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc83912, 95 },
    { 0, 0 },
    { 0xa9583914, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8983917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8228391b, 88 },
    { 0x4578391b, 90 },
    { 0, 0 },
    { 0xc768391e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b83922, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0d8392a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba18392f, 76 },
    { 0, 0 },
    { 0x4f583931, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a83935, 98 },
    { 0xabc83935, 83 },
    { 0, 0 },
    { 0x5a883938, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b18393b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e8393f, 52 },
    { 0, 0 },
    { 0x57f83941, 98 },
    { 0xf5a83942, 76 },
    { 0xc2283943, 61 },
    { 0xca783943, 76 },
    { 0xc4183944, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c8394d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c883950, 90 },
    { 0x23683950, 37 },
    { 0, 0 },
    { 0x2ff83953, 55 },
    { 0xd2a83954, 38 },
    { 0x67883954, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4883959, 100 },
    { 0x9258395a, 61 },
    { 0, 0 },
    { 0xb488395c, 95 },
    { 0xafb8395d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad83964, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3983967, 61 },
    { 0, 0 },
    { 0xdbb83969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb828396c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe788396f, 51 },
    { 0, 0 },
    { 0x83683971, 61 },
    { 0, 0 },
    { 0xe0583973, 66 },
    { 0, 0 },
    { 0xff383975, 46 },
    { 0xe8a83975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb808397a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab68397e, 98 },
    { 0x5808397e, 90 },
    { 0xfa48397e, 33 },
    { 0, 0 },
    { 0xfd683982, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa83985, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0783995, 61 },
    { 0, 0 },
    { 0xf0e83997, 23 },
    { 0x44b83998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1378399d, 93 },
    { 0xded8399d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a839a1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43839a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2839ac, 95 },
    { 0x84c839ad, 76 },
    { 0xc2a839ae, 33 },
    { 0x87f839af, 52 },
    { 0x9e3839b0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0839bc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6839c7, 61 },
    { 0, 0 },
    { 0xcc0839c9, 34 },
    { 0x131839ca, 95 },
    { 0xa75839c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5839ce, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea839d3, 33 },
    { 0xd0a839d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb839d7, 76 },
    { 0x72d839d8, 95 },
    { 0xe8b839d7, 61 },
    { 0, 0 },
    { 0x714839db, 55 },
    { 0, 0 },
    { 0xcf839dd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0839e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53839ee, 81 },
    { 0, 0 },
    { 0x33839f0, 90 },
    { 0x14839f1, 76 },
    { 0, 0 },
    { 0xd01839f3, 43 },
    { 0, 0 },
    { 0x79839f5, 95 },
    { 0xf81839f5, 61 },
    { 0xc1b839f6, 91 },
    { 0, 0 },
    { 0xc73839f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe983a00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5683a09, 87 },
    { 0x11a83a0a, 23 },
    { 0x26e83a0b, 76 },
    { 0x40d83a0b, 43 },
    { 0xa9d83a0d, 99 },
    { 0x31f83a0e, 33 },
    { 0x13c83a0f, 87 },
    { 0xc6383a10, 23 },
    { 0x7a383a11, 43 },
    { 0x4483a10, 61 },
    { 0, 0 },
    { 0x95a83a14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f83a18, 91 },
    { 0, 0 },
    { 0xecd83a1a, 85 },
    { 0x10483a1a, 90 },
    { 0xe6683a1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc083a26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8883a29, 33 },
    { 0x55883a2a, 87 },
    { 0x72b83a2b, 23 },
    { 0, 0 },
    { 0x48283a2d, 76 },
    { 0, 0 },
    { 0x3d783a2f, 85 },
    { 0x85a83a2f, 46 },
    { 0, 0 },
    { 0x9a983a32, 33 },
    { 0xbf283a33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53183a38, 34 },
    { 0, 0 },
    { 0x9fc83a3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe883a3d, 90 },
    { 0, 0 },
    { 0xa4083a3f, 23 },
    { 0xacf83a40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15283a44, 90 },
    { 0, 0 },
    { 0x32083a46, 55 },
    { 0x1ca83a47, 23 },
    { 0x25f83a47, 33 },
    { 0x97083a49, 23 },
    { 0x50d83a46, 90 },
    { 0x6d783a48, 95 },
    { 0, 0 },
    { 0x7fe83a4d, 76 },
    { 0xfc183a4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0283a51, 98 },
    { 0x63d83a51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa83a56, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec83a5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13483a60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d83a65, 23 },
    { 0x33583a66, 78 },
    { 0x34283a66, 43 },
    { 0x8583a67, 95 },
    { 0xc2a83a65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27783a6f, 76 },
    { 0x74183a70, 33 },
    { 0xbde83a70, 69 },
    { 0xcc283a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2983a77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b83a7e, 43 },
    { 0x13283a7f, 55 },
    { 0, 0 },
    { 0xad783a81, 94 },
    { 0, 0 },
    { 0xfa583a83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f983a87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3283a8a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b83a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d383a91, 23 },
    { 0, 0 },
    { 0xf7283a93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac83a98, 33 },
    { 0x31183a99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe283aa2, 88 },
    { 0x29383aa3, 55 },
    { 0, 0 },
    { 0xe9c83aa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c83aa9, 56 },
    { 0xf983aaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b283aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff783ab0, 76 },
    { 0xff483ab1, 61 },
    { 0x78783ab1, 23 },
    { 0, 0 },
    { 0xfe283ab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0383abe, 17 },
    { 0xd0783abf, 23 },
    { 0x63b83abf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b83ac9, 95 },
    { 0x7d483aca, 81 },
    { 0x8bb83acb, 61 },
    { 0xd6f83aca, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae83acf, 17 },
    { 0xd2a83ad0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c483ad4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97983adc, 90 },
    { 0x38a83adc, 55 },
    { 0x84e83ade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab83ae2, 76 },
    { 0xa8983ae3, 61 },
    { 0x3cc83ae3, 33 },
    { 0x95983ae4, 34 },
    { 0, 0 },
    { 0xdba83ae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70583aea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc483aef, 23 },
    { 0x1d783aef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe83af4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af83af7, 90 },
    { 0x19d83af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc83afc, 61 },
    { 0xa2d83afd, 61 },
    { 0, 0 },
    { 0xcce83aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d83b02, 95 },
    { 0x1183b03, 87 },
    { 0x4fa83b04, 33 },
    { 0x6fa83b03, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3883b09, 95 },
    { 0xee783b0a, 88 },
    { 0xc1983b0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3283b0e, 52 },
    { 0x96183b0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc83b12, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff83b15, 66 },
    { 0x4a083b16, 17 },
    { 0xc9a83b17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e83b20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d83b25, 98 },
    { 0xe0983b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e383b2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b83b30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d983b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab83b3d, 23 },
    { 0x1a783b3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde683b43, 95 },
    { 0, 0 },
    { 0xad383b45, 92 },
    { 0, 0 },
    { 0xad983b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6483b4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11183b50, 87 },
    { 0, 0 },
    { 0x7f983b52, 55 },
    { 0x57683b53, 66 },
    { 0x7d83b54, 85 },
    { 0x90483b54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52083b58, 98 },
    { 0x45983b59, 36 },
    { 0x4d83b5a, 67 },
    { 0, 0 },
    { 0x2b183b5c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c83b61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af83b65, 93 },
    { 0xa6783b66, 61 },
    { 0xc5683b65, 43 },
    { 0x1fb83b65, 90 },
    { 0xa6583b69, 61 },
    { 0xc6983b6a, 34 },
    { 0xe6e83b6b, 98 },
    { 0x3ca83b6c, 88 },
    { 0x8da83b6b, 90 },
    { 0x64d83b6e, 76 },
    { 0xd5883b6f, 46 },
    { 0, 0 },
    { 0x62283b71, 61 },
    { 0, 0 },
    { 0xf9b83b73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59383b78, 33 },
    { 0x97e83b78, 61 },
    { 0xc1883b7a, 33 },
    { 0x1f83b7a, 95 },
    { 0, 0 },
    { 0x71e83b7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34583b81, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc83b88, 95 },
    { 0xe4983b89, 23 },
    { 0, 0 },
    { 0xdff83b8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c83b91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37283b94, 95 },
    { 0, 0 },
    { 0x33683b96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc83ba0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92383ba4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a83ba7, 23 },
    { 0xbc283ba8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0883bad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a583bb2, 76 },
    { 0x71983bb3, 98 },
    { 0x8db83bb4, 84 },
    { 0x8c083bb3, 55 },
    { 0xbfb83bb4, 76 },
    { 0x5a883bb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e383bba, 61 },
    { 0x82d83bbb, 53 },
    { 0x24683bbb, 76 },
    { 0xcf883bbd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57783bc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d583bc6, 61 },
    { 0xf8383bc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec083bcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27983bd8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb483bdd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d83be1, 65 },
    { 0x8fa83be1, 90 },
    { 0x38083be3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b83be8, 46 },
    { 0x5da83be9, 34 },
    { 0x95d83bea, 90 },
    { 0xaa883beb, 43 },
    { 0xc0b83bec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x57883bef, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x93583bf2, 91 },
    { 0x1ce83bf2, 61 },
    { 0x9fb83bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff83bfd, 17 },
    { 0, 0 },
    { 0xac783bff, 98 },
    { 0x13683c00, 67 },
    { 0x28d83c01, 34 },
    { 0, 0 },
    { 0xe2483c03, 40 },
    { 0xa5983c04, 33 },
    { 0xf2783c05, 76 },
    { 0, 0 },
    { 0x24483c07, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5183c13, 98 },
    { 0xa3883c14, 46 },
    { 0x93c83c13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a783c18, 61 },
    { 0xae483c19, 98 },
    { 0, 0 },
    { 0xcf883c1b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81783c20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c83c2e, 81 },
    { 0x4e383c2f, 34 },
    { 0xab583c2f, 43 },
    { 0xe9183c31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c83c34, 90 },
    { 0, 0 },
    { 0xe4683c36, 88 },
    { 0x9e583c37, 52 },
    { 0x75083c38, 34 },
    { 0xa3483c36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff983c40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x94083c43, 23 },
    { 0xa0e83c43, 61 },
    { 0, 0 },
    { 0x92e83c46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b83c4a, 55 },
    { 0, 0 },
    { 0xd1083c4c, 95 },
    { 0xaf683c4d, 90 },
    { 0x56983c4e, 61 },
    { 0xbee83c4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64883c57, 95 },
    { 0xaed83c58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34883c5b, 98 },
    { 0x39483c5c, 33 },
    { 0x8dd83c5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd83c60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c83c6a, 33 },
    { 0xa4383c6b, 98 },
    { 0xbc583c6c, 90 },
    { 0xd5e83c6a, 90 },
    { 0, 0 },
    { 0x4c383c6f, 34 },
    { 0xcac83c70, 76 },
    { 0xb2283c71, 76 },
    { 0x24183c70, 69 },
    { 0x1a883c6f, 34 },
    { 0xdb283c72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e83c7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e83c82, 33 },
    { 0x96383c83, 95 },
    { 0x50e83c84, 98 },
    { 0x8583c85, 100 },
    { 0, 0 },
    { 0xb5283c87, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60983c8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad83c8f, 100 },
    { 0, 0 },
    { 0x9f383c91, 99 },
    { 0x55b83c92, 61 },
    { 0x81d83c91, 98 },
    { 0, 0 },
    { 0x2e883c95, 55 },
    { 0x51883c96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6483c9b, 90 },
    { 0, 0 },
    { 0x7cc83c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81083ca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xee883ca6, 100 },
    { 0x96083ca7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a83cae, 88 },
    { 0xa83cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb83cb5, 95 },
    { 0xeb583cb6, 55 },
    { 0x4d883cb5, 98 },
    { 0xeb883cb5, 33 },
    { 0, 0 },
    { 0xd6883cba, 33 },
    { 0, 0 },
    { 0xc5383cbc, 76 },
    { 0x3af83cbd, 93 },
    { 0xb7c83cbd, 98 },
    { 0x97783cbf, 33 },
    { 0x5bf83cbe, 55 },
    { 0x583cbf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe183cc4, 33 },
    { 0x46283cc5, 90 },
    { 0x2a783cc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d783cce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57483cd6, 78 },
    { 0x81883cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b83cda, 43 },
    { 0xe2483cdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae183cde, 34 },
    { 0xe4b83cdf, 91 },
    { 0, 0 },
    { 0xe3583ce1, 33 },
    { 0xab883ce1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0383ce7, 95 },
    { 0x9d583ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d83ceb, 33 },
    { 0xe7283cec, 98 },
    { 0x13d83cec, 23 },
    { 0x77e83cec, 76 },
    { 0x42083cef, 78 },
    { 0x9e83cef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da83cf6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f683cf9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4683cfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b83cff, 46 },
    { 0x7bf83cff, 46 },
    { 0, 0 },
    { 0xe8d83d02, 100 },
    { 0x27983d03, 98 },
    { 0xffa83d02, 87 },
    { 0x7e283d03, 33 },
    { 0xbd283d02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc83d09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96783d0c, 95 },
    { 0, 0 },
    { 0x13783d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30283d14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xad183d17, 23 },
    { 0xf2a83d18, 23 },
    { 0x20d83d19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x78783d1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11183d1f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6483d22, 95 },
    { 0x85d83d23, 75 },
    { 0x70b83d23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d83d2c, 61 },
    { 0x25583d2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b883d30, 90 },
    { 0, 0 },
    { 0x39f83d32, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x883d35, 52 },
    { 0, 0 },
    { 0x40483d37, 98 },
    { 0xd0183d37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f83d46, 91 },
    { 0xed683d47, 99 },
    { 0x21383d48, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1483d4e, 61 },
    { 0xe4e83d4f, 80 },
    { 0, 0 },
    { 0x68583d51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e683d55, 33 },
    { 0xb1f83d55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2483d5a, 98 },
    { 0x89c83d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff83d5e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad83d61, 93 },
    { 0, 0 },
    { 0x76283d63, 99 },
    { 0xa4583d63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d83d67, 34 },
    { 0xd7483d68, 61 },
    { 0, 0 },
    { 0xbce83d6a, 68 },
    { 0xc5583d6b, 99 },
    { 0, 0 },
    { 0x7e183d6d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9783d70, 66 },
    { 0xb4583d71, 100 },
    { 0xd8083d72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec83d76, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d7a, 23 },
    { 0xf8083d7a, 77 },
    { 0xc8f83d7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c83d7f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41583d83, 61 },
    { 0x26a83d84, 93 },
    { 0, 0 },
    { 0xafc83d86, 55 },
    { 0x8ba83d87, 80 },
    { 0x92383d88, 84 },
    { 0xdb583d89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf83d8d, 43 },
    { 0xee83d8e, 55 },
    { 0x29f83d8d, 98 },
    { 0xc0f83d8d, 61 },
    { 0, 0 },
    { 0x6783d92, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x21183d95, 40 },
    { 0, 0 },
    { 0x51483d97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15083d9c, 23 },
    { 0x25e83d9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65383da0, 61 },
    { 0xb883da1, 100 },
    { 0xf0583da0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d83da5, 33 },
    { 0x80e83da6, 91 },
    { 0xeb983da7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e83daf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75383db9, 61 },
    { 0xff083dba, 23 },
    { 0x4c983dbb, 95 },
    { 0xba983dba, 90 },
    { 0x93c83dbd, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0783dc0, 95 },
    { 0x1b483dc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83483dc5, 23 },
    { 0x7f783dc6, 52 },
    { 0x483dc7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e83dcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c83dd1, 90 },
    { 0xe4783dd1, 90 },
    { 0xa883dd3, 66 },
    { 0xd2983dd4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f083dd7, 52 },
    { 0xada83dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c83ddd, 90 },
    { 0xece83dde, 33 },
    { 0, 0 },
    { 0x5a583de0, 33 },
    { 0xe9383de1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e83de7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb783dec, 84 },
    { 0x92483dec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b083df4, 91 },
    { 0xd5583df4, 69 },
    { 0xff283df4, 99 },
    { 0xddd83df7, 76 },
    { 0xbd283df7, 61 },
    { 0xa2d83df8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9383dff, 76 },
    { 0xa7f83e00, 90 },
    { 0x21183dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a83e04, 76 },
    { 0x77f83e05, 46 },
    { 0xba383e06, 81 },
    { 0x70183e05, 66 },
    { 0, 0 },
    { 0xf0b83e09, 95 },
    { 0xf6983e0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1383e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc383e11, 61 },
    { 0x18483e12, 90 },
    { 0xd2883e13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a83e16, 33 },
    { 0, 0 },
    { 0x83b83e18, 87 },
    { 0, 0 },
    { 0x9f83e1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f83e1d, 33 },
    { 0, 0 },
    { 0xfa083e1f, 93 },
    { 0, 0 },
    { 0x56b83e21, 23 },
    { 0x2d083e21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1183e25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f83e28, 98 },
    { 0x85383e29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f983e2d, 61 },
    { 0xf8683e2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d83e31, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e83e34, 90 },
    { 0xb8983e35, 97 },
    { 0xf0a83e35, 55 },
    { 0xbb183e36, 23 },
    { 0, 0 },
    { 0xb7083e39, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6283e42, 98 },
    { 0x53883e42, 55 },
    { 0x18083e44, 46 },
    { 0x34f83e45, 93 },
    { 0x66083e44, 61 },
    { 0x41d83e47, 40 },
    { 0x83283e48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c83e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13883e55, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a83e58, 33 },
    { 0x9183e59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d83e5e, 66 },
    { 0x21483e5f, 33 },
    { 0x26b83e5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a83e64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba783e68, 66 },
    { 0x3ff83e69, 33 },
    { 0x46a83e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7083e6e, 61 },
    { 0, 0 },
    { 0xdb883e70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b483e77, 87 },
    { 0, 0 },
    { 0x23183e79, 84 },
    { 0, 0 },
    { 0x983e7b, 23 },
    { 0x41c83e7c, 98 },
    { 0xcd083e7c, 55 },
    { 0, 0 },
    { 0x6e083e7f, 65 },
    { 0, 0 },
    { 0x64083e81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb83e86, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb583e89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0883e95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b783e98, 81 },
    { 0xaa383e99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22083e9f, 23 },
    { 0x5fb83e9f, 33 },
    { 0xd4d83ea0, 55 },
    { 0x38783ea1, 33 },
    { 0xf5583ea1, 95 },
    { 0, 0 },
    { 0xc3083ea5, 33 },
    { 0, 0 },
    { 0x70583ea7, 98 },
    { 0x1ce83ea7, 61 },
    { 0xdb183ea9, 95 },
    { 0xf3583eaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f83eaf, 38 },
    { 0, 0 },
    { 0x4ff83eb1, 78 },
    { 0, 0 },
    { 0x1a983eb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e83eb6, 60 },
    { 0xac383eb7, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2383ec1, 98 },
    { 0, 0 },
    { 0xf1883ec3, 61 },
    { 0x13d83ec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c083ecb, 90 },
    { 0xd1583ecc, 93 },
    { 0, 0 },
    { 0x3fb83ece, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e83ed1, 19 },
    { 0x7c83ed2, 34 },
    { 0x10e83ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6483ed8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb783ee2, 100 },
    { 0xc9b83ee3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d83eea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb83eed, 95 },
    { 0xee883eee, 61 },
    { 0xc3483eed, 34 },
    { 0xebf83ef0, 93 },
    { 0xd1d83eee, 96 },
    { 0, 0 },
    { 0x8c483ef3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9583ef8, 84 },
    { 0x65783ef9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd83f00, 87 },
    { 0x51283f00, 23 },
    { 0xfa983f02, 40 },
    { 0, 0 },
    { 0x51383f04, 33 },
    { 0x66883f04, 46 },
    { 0xd7a83f05, 65 },
    { 0x5ea83f04, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d83f0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb83f17, 95 },
    { 0, 0 },
    { 0x19583f19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d83f1c, 40 },
    { 0x94383f1d, 33 },
    { 0x9ae83f1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a83f22, 33 },
    { 0x95f83f23, 61 },
    { 0, 0 },
    { 0x2f983f25, 90 },
    { 0x46983f25, 98 },
    { 0x3fc83f27, 17 },
    { 0x16f83f27, 98 },
    { 0x52783f28, 95 },
    { 0x58c83f2a, 43 },
    { 0, 0 },
    { 0x87483f2c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb883f31, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72883f35, 75 },
    { 0x5583f36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26483f3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae83f3e, 94 },
    { 0, 0 },
    { 0x91083f40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d383f44, 90 },
    { 0, 0 },
    { 0x54383f46, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x35983f49, 34 },
    { 0x63083f4a, 33 },
    { 0x3f583f49, 95 },
    { 0, 0 },
    { 0xfe83f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d83f50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56483f53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc83f5a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d283f5d, 90 },
    { 0xab383f5d, 90 },
    { 0, 0 },
    { 0x10983f60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3583f64, 17 },
    { 0xdd683f64, 90 },
    { 0x3fa83f66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44383f69, 97 },
    { 0, 0 },
    { 0x13283f6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73183f71, 61 },
    { 0x1dd83f71, 17 },
    { 0x1f283f73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa183f76, 76 },
    { 0x3db83f77, 23 },
    { 0xb3883f77, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3983f7c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82883f80, 90 },
    { 0, 0 },
    { 0x4c283f82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d83f86, 55 },
    { 0xb3383f87, 84 },
    { 0x77583f86, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a83f8b, 90 },
    { 0, 0 },
    { 0xe4883f8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe83f90, 90 },
    { 0, 0 },
    { 0xd7883f92, 43 },
    { 0xd9e83f93, 52 },
    { 0x24b83f94, 93 },
    { 0xf5983f95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34383f9c, 61 },
    { 0, 0 },
    { 0xd3883f9e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68283fa6, 88 },
    { 0, 0 },
    { 0x59083fa8, 90 },
    { 0, 0 },
    { 0x88283faa, 66 },
    { 0x6e283faa, 38 },
    { 0xc7d83fac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5483fb0, 33 },
    { 0, 0 },
    { 0x45783fb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c83fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf983fb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e83fc0, 64 },
    { 0xe1583fc1, 90 },
    { 0x8a083fc1, 67 },
    { 0, 0 },
    { 0xd283fc4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef83fca, 33 },
    { 0, 0 },
    { 0x81483fcc, 61 },
    { 0x73d83fcd, 33 },
    { 0x85f83fce, 23 },
    { 0x36a83fcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c83fd7, 85 },
    { 0, 0 },
    { 0x21383fd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96083fe0, 43 },
    { 0x1d83fe1, 8 },
    { 0x9e983fe2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80183fe5, 33 },
    { 0x53783fe6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b583fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3183fed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70083ff1, 40 },
    { 0, 0 },
    { 0x84183ff3, 90 },
    { 0, 0 },
    { 0x13e83ff5, 61 },
    { 0x89583ff6, 17 },
    { 0, 0 },
    { 0x52783ff8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba84002, 43 },
    { 0xe4a84003, 33 },
    { 0, 0 },
    { 0x9e784005, 78 },
    { 0x78184005, 23 },
    { 0x7c84007, 52 },
    { 0x81b84008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe58400b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de84011, 95 },
    { 0xee784011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec84015, 99 },
    { 0, 0 },
    { 0x36784017, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a8401e, 33 },
    { 0, 0 },
    { 0x8f984020, 83 },
    { 0, 0 },
    { 0x74d84022, 23 },
    { 0, 0 },
    { 0x52c84024, 61 },
    { 0, 0 },
    { 0xbd84026, 99 },
    { 0x31f84027, 61 },
    { 0, 0 },
    { 0x84029, 90 },
    { 0x17d8402a, 98 },
    { 0x1d68402b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8084031, 98 },
    { 0x70b84031, 98 },
    { 0, 0 },
    { 0xab84034, 76 },
    { 0xdbd84035, 99 },
    { 0x8ff84035, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb318403f, 98 },
    { 0x40a84040, 100 },
    { 0x6de84041, 91 },
    { 0xbc984041, 91 },
    { 0xc2084043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30f84047, 95 },
    { 0, 0 },
    { 0xdbe84049, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f88404c, 47 },
    { 0x98b8404d, 90 },
    { 0x7878404e, 99 },
    { 0xc368404d, 61 },
    { 0, 0 },
    { 0x53884051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0084054, 33 },
    { 0x28e84055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c78405b, 100 },
    { 0x2358405c, 66 },
    { 0xacb8405d, 98 },
    { 0, 0 },
    { 0x14b8405f, 55 },
    { 0, 0 },
    { 0x50184061, 90 },
    { 0xc7884062, 95 },
    { 0, 0 },
    { 0x2a784064, 90 },
    { 0, 0 },
    { 0x68a84066, 43 },
    { 0xdf584067, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d8406a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d8406d, 98 },
    { 0xeb38406d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b88407c, 90 },
    { 0x6fc8407d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54984084, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448408a, 61 },
    { 0x7508408b, 34 },
    { 0x6fb8408b, 65 },
    { 0x46d8408d, 87 },
    { 0, 0 },
    { 0xd798408f, 90 },
    { 0x7778408f, 84 },
    { 0x4e284091, 98 },
    { 0, 0 },
    { 0x1284093, 90 },
    { 0x4ee84094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d84098, 23 },
    { 0, 0 },
    { 0x2f38409a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae8409d, 55 },
    { 0x74d8409d, 90 },
    { 0x57e8409e, 68 },
    { 0x7b1840a0, 99 },
    { 0x225840a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f840a4, 100 },
    { 0x912840a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b840a8, 100 },
    { 0xa7840a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac840ad, 95 },
    { 0xa20840ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de840b1, 33 },
    { 0xc98840b1, 87 },
    { 0xee1840b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9840bb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9840bf, 55 },
    { 0x6b7840c0, 90 },
    { 0x4ea840c1, 91 },
    { 0x1e8840c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807840c6, 43 },
    { 0x1ad840c7, 91 },
    { 0x73b840c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3840cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d840d5, 69 },
    { 0xe46840d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4840db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e840de, 84 },
    { 0, 0 },
    { 0xe12840e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b840e7, 93 },
    { 0x76d840e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6840ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1840f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x459840f7, 76 },
    { 0x307840f8, 46 },
    { 0, 0 },
    { 0x40b840fa, 33 },
    { 0, 0 },
    { 0xe61840fc, 90 },
    { 0x663840fd, 67 },
    { 0x8c7840fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1484103, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2884106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a8410a, 95 },
    { 0x7648410b, 43 },
    { 0x8f18410c, 80 },
    { 0xe6e8410b, 34 },
    { 0, 0 },
    { 0x73c8410f, 90 },
    { 0, 0 },
    { 0x5984111, 98 },
    { 0x88484112, 36 },
    { 0x8d684113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb84116, 98 },
    { 0xc1b84117, 61 },
    { 0xd6584116, 95 },
    { 0x44d84119, 61 },
    { 0xcbb8411a, 90 },
    { 0xffe8411b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288411e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd84123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe284126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2958412c, 23 },
    { 0x2038412d, 90 },
    { 0xb788412c, 95 },
    { 0, 0 },
    { 0x56b84130, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41284134, 23 },
    { 0xad784135, 95 },
    { 0, 0 },
    { 0x16184137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe758413a, 90 },
    { 0xf898413b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d84143, 64 },
    { 0x56e84144, 86 },
    { 0x5df84145, 43 },
    { 0xd9484145, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b8414b, 59 },
    { 0x3968414c, 95 },
    { 0xf178414d, 93 },
    { 0, 0 },
    { 0xc7b8414f, 23 },
    { 0xaeb84150, 90 },
    { 0, 0 },
    { 0xaa484152, 90 },
    { 0, 0 },
    { 0x1da84154, 23 },
    { 0x6e84154, 55 },
    { 0, 0 },
    { 0xb8c84157, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd848415a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b8415e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd84161, 66 },
    { 0, 0 },
    { 0x55a84163, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x25384166, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2084173, 93 },
    { 0, 0 },
    { 0x81384175, 34 },
    { 0, 0 },
    { 0xfe184177, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a8417d, 98 },
    { 0xe6d8417e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f84181, 90 },
    { 0x1284181, 95 },
    { 0x25a84183, 46 },
    { 0xaca84184, 61 },
    { 0x83284185, 90 },
    { 0x5cf84184, 76 },
    { 0xa8e84187, 43 },
    { 0x22a84188, 85 },
    { 0x5be84187, 98 },
    { 0xb308418a, 43 },
    { 0x2b584186, 91 },
    { 0x10184182, 95 },
    { 0, 0 },
    { 0x5a88418e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0184192, 90 },
    { 0xa5684192, 77 },
    { 0x46d84194, 43 },
    { 0x88784195, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4584199, 41 },
    { 0, 0 },
    { 0x4e18419b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15841a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9841a5, 95 },
    { 0x542841a6, 78 },
    { 0xa66841a5, 23 },
    { 0, 0 },
    { 0x5ea841a9, 61 },
    { 0x98c841aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03841ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5841ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4841bd, 100 },
    { 0x333841be, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef841c1, 34 },
    { 0x300841c2, 91 },
    { 0, 0 },
    { 0x6eb841c4, 98 },
    { 0, 0 },
    { 0x5ca841c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f841c9, 52 },
    { 0, 0 },
    { 0x9df841cb, 23 },
    { 0x288841cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a841d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811841d5, 66 },
    { 0x824841d5, 34 },
    { 0, 0 },
    { 0x5da841d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b841db, 23 },
    { 0xf92841db, 55 },
    { 0xddd841dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5841e3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5841ea, 33 },
    { 0, 0 },
    { 0x6ef841ec, 96 },
    { 0x7fe841ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb841f0, 33 },
    { 0x571841f1, 98 },
    { 0x322841f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7841f5, 95 },
    { 0, 0 },
    { 0x604841f7, 85 },
    { 0xe6e841f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f841fb, 61 },
    { 0x6e6841fc, 95 },
    { 0, 0 },
    { 0xf95841fe, 90 },
    { 0, 0 },
    { 0x37c84200, 98 },
    { 0, 0 },
    { 0xeef84202, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c84205, 91 },
    { 0x63c84205, 76 },
    { 0, 0 },
    { 0x73084208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b8420b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d08420e, 23 },
    { 0x5648420f, 61 },
    { 0, 0 },
    { 0x1ab84211, 33 },
    { 0x6c84212, 95 },
    { 0xf6f84213, 66 },
    { 0x12984211, 85 },
    { 0, 0 },
    { 0x5f784216, 33 },
    { 0xa5184217, 61 },
    { 0x7a84218, 90 },
    { 0xc9c84217, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8421d, 91 },
    { 0, 0 },
    { 0x5778421f, 33 },
    { 0x2a584220, 43 },
    { 0, 0 },
    { 0xe4284222, 40 },
    { 0x4d584223, 61 },
    { 0x6e784224, 61 },
    { 0xc184223, 38 },
    { 0xc8c84226, 91 },
    { 0xbd684222, 95 },
    { 0, 0 },
    { 0xa6184229, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78422f, 55 },
    { 0x2de8422f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c84240, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a784243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d8424a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf8424f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6384256, 23 },
    { 0xdd384256, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f8425c, 34 },
    { 0xf4c8425d, 98 },
    { 0x21e8425e, 99 },
    { 0xa358425d, 90 },
    { 0x10b84260, 90 },
    { 0, 0 },
    { 0x22484262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a84267, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98426a, 23 },
    { 0, 0 },
    { 0xfeb8426c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7384273, 95 },
    { 0x7b84274, 85 },
    { 0x1fd84275, 33 },
    { 0, 0 },
    { 0x7984277, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe88427a, 95 },
    { 0xf038427b, 61 },
    { 0, 0 },
    { 0xd8f8427d, 77 },
    { 0x17e8427e, 52 },
    { 0, 0 },
    { 0x4b084280, 43 },
    { 0, 0 },
    { 0xf4a84282, 65 },
    { 0xc984282, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e284287, 78 },
    { 0xb4084288, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb678428e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab84293, 61 },
    { 0xddf84294, 55 },
    { 0xdf184295, 93 },
    { 0xdb384295, 90 },
    { 0xb0284297, 76 },
    { 0x3b984298, 98 },
    { 0x81384299, 67 },
    { 0x8db84295, 61 },
    { 0x1988429b, 90 },
    { 0x6448429b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be8429f, 95 },
    { 0x339842a0, 55 },
    { 0x368842a1, 78 },
    { 0xd4f842a1, 67 },
    { 0x692842a1, 46 },
    { 0x6958429f, 36 },
    { 0, 0 },
    { 0xd5b842a6, 23 },
    { 0, 0 },
    { 0x50842a8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371842ac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42842b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba842b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c842bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c842be, 43 },
    { 0xd79842bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5842c2, 34 },
    { 0, 0 },
    { 0xa44842c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8842c9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e842cd, 90 },
    { 0xc11842cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd842d1, 76 },
    { 0x7bc842d1, 90 },
    { 0x803842d1, 43 },
    { 0x9df842d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda842d8, 98 },
    { 0xe23842d8, 90 },
    { 0xf87842da, 98 },
    { 0x646842db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18842de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6842e1, 61 },
    { 0, 0 },
    { 0x8a1842e3, 78 },
    { 0x29f842e4, 61 },
    { 0x3c2842e4, 33 },
    { 0, 0 },
    { 0x820842e7, 66 },
    { 0xf1d842e7, 81 },
    { 0x9e4842e9, 66 },
    { 0x82e842e7, 23 },
    { 0x2b3842eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb842f2, 64 },
    { 0x981842f2, 78 },
    { 0x28e842f2, 90 },
    { 0x931842f2, 33 },
    { 0xee3842f6, 95 },
    { 0, 0 },
    { 0x32d842f8, 88 },
    { 0xe83842f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363842fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0984300, 43 },
    { 0xc9984301, 90 },
    { 0x35484301, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe8430a, 76 },
    { 0x3438430b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33584312, 61 },
    { 0x17084313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b84318, 98 },
    { 0, 0 },
    { 0xfc58431a, 61 },
    { 0xf998431b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2138431e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b084322, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f84326, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae98432a, 33 },
    { 0x9848432b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa84335, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f38433c, 90 },
    { 0x96b8433c, 80 },
    { 0x8a48433e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91384342, 33 },
    { 0xada84343, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b84346, 95 },
    { 0x9f384347, 33 },
    { 0x85484346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8434c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a984350, 90 },
    { 0, 0 },
    { 0x17d84352, 91 },
    { 0x98784352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec78435a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85084362, 61 },
    { 0xe5384362, 90 },
    { 0, 0 },
    { 0xf2284365, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3278436b, 33 },
    { 0x3978436b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd884372, 39 },
    { 0x3e984373, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb884378, 78 },
    { 0, 0 },
    { 0xf1b8437a, 98 },
    { 0, 0 },
    { 0x5948437c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c684385, 98 },
    { 0xb1184386, 76 },
    { 0x6d784386, 43 },
    { 0x30284385, 90 },
    { 0x7284386, 33 },
    { 0x31d84385, 84 },
    { 0, 0 },
    { 0x64a8438c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0184394, 98 },
    { 0x34f84395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce84398, 33 },
    { 0, 0 },
    { 0x1e8439a, 76 },
    { 0x5e28439a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28439e, 67 },
    { 0, 0 },
    { 0xd8843a0, 33 },
    { 0x697843a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa843a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9843aa, 33 },
    { 0x7d9843ab, 95 },
    { 0xf55843ac, 90 },
    { 0x156843ad, 33 },
    { 0x95d843aa, 23 },
    { 0x52e843af, 90 },
    { 0xb8e843b0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6843b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81843b8, 90 },
    { 0, 0 },
    { 0x461843ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77843bf, 33 },
    { 0x4a7843c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160843c5, 33 },
    { 0, 0 },
    { 0xaed843c7, 61 },
    { 0, 0 },
    { 0x1fc843c9, 67 },
    { 0xe05843c9, 76 },
    { 0xb6a843cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a843cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc843d3, 95 },
    { 0, 0 },
    { 0x43a843d5, 34 },
    { 0x457843d6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9843db, 76 },
    { 0x91b843db, 90 },
    { 0xf3d843dd, 23 },
    { 0x361843db, 55 },
    { 0x505843dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a843e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686843ea, 66 },
    { 0, 0 },
    { 0x589843ec, 43 },
    { 0, 0 },
    { 0xe76843ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa843f2, 98 },
    { 0, 0 },
    { 0xeee843f4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf843f7, 66 },
    { 0, 0 },
    { 0x975843f9, 23 },
    { 0, 0 },
    { 0x6b3843fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc84403, 95 },
    { 0x1b784404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78440b, 95 },
    { 0x87a8440c, 88 },
    { 0xf7d8440d, 87 },
    { 0xc38440e, 95 },
    { 0x4db8440d, 23 },
    { 0x57e8440c, 55 },
    { 0xcae84411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc08441a, 99 },
    { 0, 0 },
    { 0xa2e8441c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c84420, 55 },
    { 0, 0 },
    { 0x38b84422, 76 },
    { 0, 0 },
    { 0x34884424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1284427, 69 },
    { 0xaa784428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6268442d, 33 },
    { 0x118442d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b84433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d84436, 43 },
    { 0xaf584437, 23 },
    { 0xc8084438, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3458443d, 40 },
    { 0x7a18443e, 76 },
    { 0x3b18443e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c84442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17084446, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88444a, 40 },
    { 0x2868444b, 99 },
    { 0xad88444c, 52 },
    { 0x8ac8444b, 98 },
    { 0xc108444a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d384451, 17 },
    { 0x17384452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35184455, 91 },
    { 0x7f484456, 78 },
    { 0xc9584455, 95 },
    { 0xd0b84455, 85 },
    { 0x32e84459, 76 },
    { 0xacb84459, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7698445f, 61 },
    { 0x4d984460, 90 },
    { 0x5e084461, 95 },
    { 0x12e8445f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d84465, 84 },
    { 0xc2e84466, 17 },
    { 0, 0 },
    { 0xb3184468, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98446d, 99 },
    { 0x56f8446d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c84471, 33 },
    { 0x8bc84472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d084477, 98 },
    { 0xb5f84478, 91 },
    { 0, 0 },
    { 0xd5a8447a, 34 },
    { 0, 0 },
    { 0xf878447c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9988447f, 37 },
    { 0xd3384480, 90 },
    { 0x13284481, 95 },
    { 0x13584480, 90 },
    { 0x59184483, 90 },
    { 0x59684484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b84487, 90 },
    { 0xadd84488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf818448b, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a08448f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90184493, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f84497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f98449d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12844a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc844a3, 33 },
    { 0, 0 },
    { 0xc02844a5, 40 },
    { 0x8cc844a6, 61 },
    { 0x4a8844a7, 100 },
    { 0xa76844a8, 40 },
    { 0, 0 },
    { 0xe3c844aa, 99 },
    { 0xc6844ab, 90 },
    { 0, 0 },
    { 0x708844ad, 23 },
    { 0, 0 },
    { 0x792844af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167844b4, 95 },
    { 0, 0 },
    { 0xc5d844b6, 33 },
    { 0x6b9844b7, 84 },
    { 0x5de844b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50844bb, 61 },
    { 0x843844bc, 95 },
    { 0x880844bd, 99 },
    { 0x4bf844bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce844c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x473844c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed844c7, 34 },
    { 0xd5a844c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf844ce, 33 },
    { 0, 0 },
    { 0x586844d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e844d5, 90 },
    { 0x499844d5, 98 },
    { 0x927844d7, 33 },
    { 0x966844d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54844e0, 76 },
    { 0x657844e0, 90 },
    { 0x755844e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d844e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8844e8, 98 },
    { 0xd16844e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b844ec, 90 },
    { 0, 0 },
    { 0x864844ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc844f1, 95 },
    { 0xa83844f2, 33 },
    { 0x487844f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38844f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf844fe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb84508, 52 },
    { 0x89b84508, 84 },
    { 0xca38450a, 23 },
    { 0, 0 },
    { 0x3a38450c, 52 },
    { 0xb08450c, 98 },
    { 0x59c8450c, 61 },
    { 0xa218450f, 90 },
    { 0x1698450e, 33 },
    { 0x8384511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8584516, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1918451c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db84524, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd84527, 33 },
    { 0, 0 },
    { 0xdf684529, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf8452d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958453b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf8453e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db84545, 85 },
    { 0, 0 },
    { 0xb6184547, 23 },
    { 0xd2184548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71184551, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a84554, 23 },
    { 0x91a84554, 90 },
    { 0x92d84556, 52 },
    { 0x58b84556, 23 },
    { 0x7b184557, 61 },
    { 0xfb084558, 34 },
    { 0x65a84557, 65 },
    { 0xe418455b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb84560, 61 },
    { 0x5e784561, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b584564, 61 },
    { 0xf2c84564, 67 },
    { 0, 0 },
    { 0x2e384567, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e48456b, 61 },
    { 0x5c78456c, 90 },
    { 0x8898456b, 98 },
    { 0x5218456e, 90 },
    { 0x83f8456b, 90 },
    { 0x72f84570, 33 },
    { 0x41f8456b, 84 },
    { 0, 0 },
    { 0xcd184573, 55 },
    { 0xec584574, 46 },
    { 0x94b84575, 61 },
    { 0xaad84575, 23 },
    { 0xf4584577, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e28457b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e184582, 78 },
    { 0xb1884583, 95 },
    { 0x6b84582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33584589, 91 },
    { 0x63d84589, 61 },
    { 0xe738458b, 95 },
    { 0xd168458c, 61 },
    { 0xf108458d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c84590, 33 },
    { 0xb3b84591, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a84595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47084598, 90 },
    { 0xe9b84599, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d845a1, 98 },
    { 0x877845a2, 93 },
    { 0x561845a3, 93 },
    { 0, 0 },
    { 0x462845a5, 61 },
    { 0x149845a6, 76 },
    { 0, 0 },
    { 0xd80845a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8845ac, 46 },
    { 0xa6c845ad, 90 },
    { 0, 0 },
    { 0x6bc845af, 61 },
    { 0, 0 },
    { 0x13b845b1, 90 },
    { 0x725845b2, 33 },
    { 0x279845b3, 90 },
    { 0, 0 },
    { 0xa5a845b5, 75 },
    { 0x652845b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9845ba, 34 },
    { 0x394845bb, 83 },
    { 0xa8b845bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd845c1, 61 },
    { 0xda5845c2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734845c7, 23 },
    { 0x417845c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db845cc, 74 },
    { 0xaca845cc, 95 },
    { 0, 0 },
    { 0xc7b845cf, 23 },
    { 0xb11845cf, 100 },
    { 0xf7c845d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515845d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf845dc, 34 },
    { 0x8ce845dd, 90 },
    { 0xb97845de, 98 },
    { 0x810845df, 98 },
    { 0x5fe845e0, 98 },
    { 0x2cc845e0, 99 },
    { 0xa2d845dd, 85 },
    { 0x89845e0, 34 },
    { 0x891845e4, 33 },
    { 0x92b845e0, 23 },
    { 0x6a8845dd, 33 },
    { 0x3eb845e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16845ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8845f2, 33 },
    { 0x3ef845f3, 43 },
    { 0xe88845f3, 43 },
    { 0, 0 },
    { 0xdad845f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff845f9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x641845fc, 61 },
    { 0, 0 },
    { 0x51e845fe, 46 },
    { 0, 0 },
    { 0x11f84600, 66 },
    { 0x5d684601, 61 },
    { 0x15984602, 43 },
    { 0xb7784601, 55 },
    { 0x88884601, 34 },
    { 0x4eb84605, 33 },
    { 0x11c84606, 90 },
    { 0xe984607, 33 },
    { 0x88a84608, 90 },
    { 0xde784608, 46 },
    { 0x76e8460a, 23 },
    { 0x46a8460a, 90 },
    { 0xa5984607, 90 },
    { 0xa8784609, 99 },
    { 0, 0 },
    { 0x638460f, 82 },
    { 0x5b84610, 95 },
    { 0xcf884611, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0784619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2684620, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb84626, 88 },
    { 0x4b84626, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58462c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d884635, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db84643, 52 },
    { 0, 0 },
    { 0xc7784645, 34 },
    { 0xb8184646, 52 },
    { 0xaf84645, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b8464b, 84 },
    { 0, 0 },
    { 0x9e38464d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67184653, 55 },
    { 0, 0 },
    { 0x88284655, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf684659, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a84661, 98 },
    { 0xd1c84662, 85 },
    { 0, 0 },
    { 0xd484664, 84 },
    { 0xfe684665, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc8466f, 77 },
    { 0x8da8466f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2884677, 99 },
    { 0x73484678, 99 },
    { 0x6a384677, 87 },
    { 0xa1584679, 99 },
    { 0, 0 },
    { 0x5db8467c, 90 },
    { 0, 0 },
    { 0x1688467e, 83 },
    { 0xd628467f, 61 },
    { 0, 0 },
    { 0xd4f84681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c884686, 33 },
    { 0x6cc84687, 66 },
    { 0, 0 },
    { 0x5084689, 33 },
    { 0x50184689, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b8468d, 61 },
    { 0x49d8468d, 52 },
    { 0, 0 },
    { 0x18284690, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e84697, 83 },
    { 0xb284698, 55 },
    { 0x79784699, 95 },
    { 0x95d84697, 98 },
    { 0x99584698, 98 },
    { 0x7ad84698, 90 },
    { 0, 0 },
    { 0xe708469e, 90 },
    { 0, 0 },
    { 0x12a846a0, 90 },
    { 0x94e846a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d846a5, 90 },
    { 0xc68846a6, 49 },
    { 0x29f846a5, 90 },
    { 0x4df846a8, 98 },
    { 0x760846a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6846ac, 91 },
    { 0x4f5846ad, 90 },
    { 0x6f846ae, 90 },
    { 0, 0 },
    { 0xc94846b0, 81 },
    { 0, 0 },
    { 0x54e846b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb846b7, 93 },
    { 0x1eb846b8, 95 },
    { 0xca9846b8, 17 },
    { 0, 0 },
    { 0x8b846bb, 66 },
    { 0xfdf846bc, 33 },
    { 0x673846bd, 90 },
    { 0, 0 },
    { 0x9ca846bf, 46 },
    { 0xf7b846c0, 33 },
    { 0, 0 },
    { 0x607846c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f846cc, 78 },
    { 0, 0 },
    { 0x6e4846ce, 67 },
    { 0x523846ce, 61 },
    { 0xeec846d0, 67 },
    { 0, 0 },
    { 0xd45846d2, 61 },
    { 0x982846d3, 43 },
    { 0, 0 },
    { 0x94b846d5, 23 },
    { 0xe17846d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844846db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147846df, 36 },
    { 0x5dd846e0, 52 },
    { 0xe9a846df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27846e4, 98 },
    { 0xf7c846e5, 93 },
    { 0x12a846e4, 46 },
    { 0x3f5846e7, 61 },
    { 0x434846e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737846ec, 33 },
    { 0x5ae846ed, 90 },
    { 0xf81846ec, 76 },
    { 0, 0 },
    { 0x73846f0, 90 },
    { 0, 0 },
    { 0x981846f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4846f9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09846fd, 15 },
    { 0, 0 },
    { 0x7ef846ff, 43 },
    { 0x53b84700, 83 },
    { 0xa2584700, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468470c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce84715, 33 },
    { 0, 0 },
    { 0x39684717, 91 },
    { 0x1d284718, 34 },
    { 0x7b84717, 70 },
    { 0x4a78471a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af84721, 66 },
    { 0x5ac84721, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b684726, 90 },
    { 0, 0 },
    { 0x96084728, 61 },
    { 0x9f884729, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e38472c, 17 },
    { 0, 0 },
    { 0xddd8472e, 90 },
    { 0xb178472f, 56 },
    { 0, 0 },
    { 0x42284731, 23 },
    { 0, 0 },
    { 0x48d84733, 95 },
    { 0xf7b84734, 98 },
    { 0, 0 },
    { 0xe8e84736, 34 },
    { 0, 0 },
    { 0x18184738, 33 },
    { 0x76684739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158473c, 23 },
    { 0x6108473d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4884740, 33 },
    { 0, 0 },
    { 0x6e884742, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c84747, 95 },
    { 0xaad84748, 98 },
    { 0x67784748, 95 },
    { 0x4838474a, 90 },
    { 0, 0 },
    { 0x4448474c, 61 },
    { 0x63f8474d, 78 },
    { 0, 0 },
    { 0x6458474f, 95 },
    { 0x53c84750, 90 },
    { 0xe718474f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea84754, 90 },
    { 0xff84755, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478475a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb628475d, 52 },
    { 0, 0 },
    { 0x6c38475f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b884764, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd8476c, 55 },
    { 0xa48476d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b84771, 90 },
    { 0xa4284772, 33 },
    { 0x4aa84773, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc84779, 34 },
    { 0xdc88477a, 23 },
    { 0x6718477a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x848477e, 33 },
    { 0x7668477f, 95 },
    { 0, 0 },
    { 0x34984781, 55 },
    { 0x81d84782, 33 },
    { 0xb5084783, 90 },
    { 0xa2084784, 87 },
    { 0x76184784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f38478d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b8479b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb8479e, 90 },
    { 0, 0 },
    { 0x3e1847a0, 69 },
    { 0xe80847a1, 33 },
    { 0x73847a0, 95 },
    { 0x78f847a1, 85 },
    { 0x3f6847a4, 91 },
    { 0, 0 },
    { 0xe9d847a6, 90 },
    { 0, 0 },
    { 0x984847a8, 33 },
    { 0x1a3847a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc847ae, 90 },
    { 0x260847af, 90 },
    { 0, 0 },
    { 0xe63847b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede847b8, 55 },
    { 0, 0 },
    { 0x7db847ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a847be, 90 },
    { 0x45b847be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3847c4, 46 },
    { 0, 0 },
    { 0xb50847c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1847cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9847d1, 23 },
    { 0, 0 },
    { 0xa65847d3, 88 },
    { 0x522847d3, 23 },
    { 0x243847d5, 98 },
    { 0, 0 },
    { 0x419847d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60847dc, 90 },
    { 0xdfe847dd, 55 },
    { 0x202847de, 95 },
    { 0x41b847df, 76 },
    { 0x3cd847dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x578847e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af847e8, 46 },
    { 0, 0 },
    { 0x28d847ea, 31 },
    { 0xeab847eb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d847ef, 33 },
    { 0xd92847f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf847f4, 90 },
    { 0xa0847f5, 46 },
    { 0, 0 },
    { 0xd56847f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b847fa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d84800, 33 },
    { 0, 0 },
    { 0x17d84802, 91 },
    { 0, 0 },
    { 0x4b684804, 90 },
    { 0xeca84805, 33 },
    { 0x2ed84806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3118480a, 93 },
    { 0x9708480b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e584810, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f884818, 75 },
    { 0, 0 },
    { 0x9af8481a, 90 },
    { 0, 0 },
    { 0x5278481c, 95 },
    { 0x8e68481c, 23 },
    { 0x168481e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54984821, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe884824, 98 },
    { 0, 0 },
    { 0x9cb84826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa84829, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd18482e, 61 },
    { 0, 0 },
    { 0xb0c84830, 99 },
    { 0x25a84831, 34 },
    { 0xe7584830, 84 },
    { 0xf3584830, 33 },
    { 0x19e84834, 78 },
    { 0xff984834, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc778483a, 91 },
    { 0xeac8483b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4108483e, 33 },
    { 0xb9b8483e, 90 },
    { 0x2f384840, 98 },
    { 0x4e8483e, 90 },
    { 0, 0 },
    { 0xe0584843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b184848, 33 },
    { 0, 0 },
    { 0xf598484a, 76 },
    { 0xf3a8484b, 34 },
    { 0x6b08484c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8484f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef84856, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a08485c, 95 },
    { 0, 0 },
    { 0x96e8485e, 90 },
    { 0xb6c8485f, 40 },
    { 0x38d8485f, 23 },
    { 0x83d84861, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x35984864, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6884867, 55 },
    { 0, 0 },
    { 0xd5484869, 61 },
    { 0x2688486a, 61 },
    { 0x7738486b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63284870, 68 },
    { 0x4db84871, 33 },
    { 0, 0 },
    { 0xd2a84873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76784878, 46 },
    { 0xeb984879, 91 },
    { 0xd208487a, 77 },
    { 0, 0 },
    { 0x5268487c, 52 },
    { 0x11d8487d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad784880, 85 },
    { 0x78184881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61884884, 100 },
    { 0, 0 },
    { 0x87184886, 61 },
    { 0x4b384886, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3618488c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8984890, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38984895, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b84898, 93 },
    { 0, 0 },
    { 0xd5c8489a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9118489e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128848a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8848af, 90 },
    { 0xc63848b0, 67 },
    { 0xcb3848b1, 78 },
    { 0xb41848b1, 90 },
    { 0x1cb848b0, 95 },
    { 0xd28848b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da848b7, 61 },
    { 0x7d5848b7, 46 },
    { 0, 0 },
    { 0x923848ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691848be, 34 },
    { 0xb24848bf, 84 },
    { 0xe58848c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6848c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c848ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3848d3, 98 },
    { 0x20b848d4, 23 },
    { 0x75c848d5, 67 },
    { 0xb59848d5, 33 },
    { 0x7f1848d3, 90 },
    { 0xb0a848d8, 90 },
    { 0x680848d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2848dd, 34 },
    { 0xe47848dd, 43 },
    { 0x1b5848dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116848e3, 61 },
    { 0x353848e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7848e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648848ee, 99 },
    { 0x77e848ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7848f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x438848f7, 55 },
    { 0x540848f7, 34 },
    { 0x652848f9, 52 },
    { 0x6b4848f9, 98 },
    { 0, 0 },
    { 0xbfd848fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4584900, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c384903, 33 },
    { 0xaa184903, 23 },
    { 0xf5984903, 55 },
    { 0x8e284906, 91 },
    { 0xbf684907, 90 },
    { 0x42884908, 61 },
    { 0, 0 },
    { 0x80c8490a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b8490f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f584913, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc518491a, 90 },
    { 0x758491a, 78 },
    { 0xa128491b, 67 },
    { 0x588491d, 46 },
    { 0x168491c, 95 },
    { 0xe98491c, 95 },
    { 0x82e8491e, 65 },
    { 0x3b184921, 87 },
    { 0x40384922, 77 },
    { 0, 0 },
    { 0x8b684924, 95 },
    { 0xd6284925, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a684928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf084931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec84936, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb08493a, 90 },
    { 0, 0 },
    { 0xf7a8493c, 98 },
    { 0, 0 },
    { 0xb5a8493e, 64 },
    { 0x8b88493f, 55 },
    { 0x81184940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d84944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab84949, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e28494d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa484950, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d84954, 23 },
    { 0x6e784955, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa8495a, 55 },
    { 0, 0 },
    { 0xc738495c, 46 },
    { 0xd418495d, 95 },
    { 0x38f8495e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb84962, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1084965, 33 },
    { 0, 0 },
    { 0x40884967, 90 },
    { 0x8ba84968, 91 },
    { 0xb584969, 90 },
    { 0, 0 },
    { 0x7a8496b, 98 },
    { 0x9368496b, 90 },
    { 0x5a48496d, 61 },
    { 0x1a48496d, 77 },
    { 0x58b8496f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c684972, 98 },
    { 0xc3684973, 98 },
    { 0x54184972, 43 },
    { 0, 0 },
    { 0xb9f84976, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf78497c, 23 },
    { 0, 0 },
    { 0xae78497e, 76 },
    { 0, 0 },
    { 0x52384980, 77 },
    { 0x27084981, 36 },
    { 0x23184981, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b84987, 77 },
    { 0xd2884987, 66 },
    { 0x88b84988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf168498c, 52 },
    { 0, 0 },
    { 0x598498e, 95 },
    { 0x8938498e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7284992, 98 },
    { 0, 0 },
    { 0xa4c84994, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38384998, 82 },
    { 0xe4684999, 43 },
    { 0, 0 },
    { 0xff98499b, 76 },
    { 0, 0 },
    { 0x7078499d, 23 },
    { 0x9318499e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6849a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d849a8, 33 },
    { 0x34a849a9, 43 },
    { 0x530849aa, 55 },
    { 0xf6d849a9, 96 },
    { 0, 0 },
    { 0xe51849ad, 90 },
    { 0x367849ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865849b3, 55 },
    { 0xc95849b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e849b9, 76 },
    { 0, 0 },
    { 0x250849bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7849c1, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x302849c4, 61 },
    { 0x792849c4, 95 },
    { 0, 0 },
    { 0x549849c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181849cb, 95 },
    { 0x4f1849cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b849dd, 23 },
    { 0x7b2849de, 100 },
    { 0xa15849de, 76 },
    { 0x82d849e0, 98 },
    { 0, 0 },
    { 0x6dc849e2, 61 },
    { 0xcdf849e2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20849e6, 76 },
    { 0x35c849e7, 33 },
    { 0xd72849e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41849ee, 55 },
    { 0, 0 },
    { 0x50f849f0, 23 },
    { 0x3f849f0, 65 },
    { 0, 0 },
    { 0x77e849f3, 23 },
    { 0x4b5849f4, 55 },
    { 0, 0 },
    { 0xa8b849f6, 90 },
    { 0x33f849f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28849fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5849fd, 55 },
    { 0xaff849fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2284a08, 77 },
    { 0, 0 },
    { 0xad584a0a, 90 },
    { 0xa1584a0a, 84 },
    { 0xf1884a0c, 93 },
    { 0, 0 },
    { 0x7b84a0e, 23 },
    { 0xf9884a0f, 90 },
    { 0xb5884a10, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63484a14, 66 },
    { 0x92684a15, 33 },
    { 0x4f984a15, 66 },
    { 0xd2584a17, 90 },
    { 0xcdf84a15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a84a1d, 98 },
    { 0, 0 },
    { 0x12284a1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x55984a22, 95 },
    { 0xf0e84a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67384a29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74484a2e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c84a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b684a37, 34 },
    { 0, 0 },
    { 0xfb184a39, 55 },
    { 0x1eb84a3a, 76 },
    { 0x10e84a3a, 76 },
    { 0, 0 },
    { 0xc7984a3d, 92 },
    { 0x2e084a3d, 76 },
    { 0xd284a3e, 55 },
    { 0xf2684a3f, 23 },
    { 0x77b84a41, 36 },
    { 0x6d284a40, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e84a45, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f84a49, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab84a4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e84a51, 55 },
    { 0x9ba84a52, 93 },
    { 0x32b84a53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4684a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4184a5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46084a63, 76 },
    { 0xa5d84a64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9784a68, 93 },
    { 0, 0 },
    { 0x5e884a6a, 90 },
    { 0x21d84a6a, 85 },
    { 0, 0 },
    { 0x99c84a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6684a74, 61 },
    { 0xb1384a74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9484a7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f784a82, 17 },
    { 0x2e084a82, 33 },
    { 0x2b684a83, 40 },
    { 0xd9e84a83, 101 },
    { 0x63b84a86, 33 },
    { 0, 0 },
    { 0x6884a88, 77 },
    { 0xc9b84a88, 23 },
    { 0, 0 },
    { 0x5784a8b, 55 },
    { 0x2d684a8b, 52 },
    { 0xafd84a8d, 23 },
    { 0xa7984a8b, 95 },
    { 0x3e584a8e, 98 },
    { 0xc9184a90, 61 },
    { 0x46684a8d, 23 },
    { 0x16084a8b, 90 },
    { 0x78784a91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a384a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0684a9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31384aa3, 55 },
    { 0, 0 },
    { 0x4584aa5, 98 },
    { 0, 0 },
    { 0xdc684aa7, 93 },
    { 0xfab84aa8, 46 },
    { 0x5fc84aa9, 98 },
    { 0x3b284aa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c84ab6, 99 },
    { 0, 0 },
    { 0x3ba84ab8, 98 },
    { 0xf9084ab9, 17 },
    { 0xfc584aba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a484abe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac84ac1, 93 },
    { 0x1ee84ac2, 43 },
    { 0x19984ac3, 55 },
    { 0xddd84ac2, 98 },
    { 0x25784ac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb084aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87484acd, 93 },
    { 0, 0 },
    { 0x6b384acf, 61 },
    { 0xca284ad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac84ad3, 90 },
    { 0xa1684ad3, 98 },
    { 0xe8284ad5, 52 },
    { 0xdac84ad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c984ad9, 17 },
    { 0xcb284ad9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x49784add, 61 },
    { 0x12984ade, 44 },
    { 0x26684adf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3084ae4, 93 },
    { 0x3b384ae5, 90 },
    { 0x1e884ae4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45884aed, 90 },
    { 0xa1184aee, 34 },
    { 0, 0 },
    { 0xc4a84af0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43584afb, 34 },
    { 0xe5a84afc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac84aff, 55 },
    { 0xe3c84b00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab84b04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93884b08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1584b0c, 61 },
    { 0xe7784b0d, 76 },
    { 0x78584b0e, 98 },
    { 0x56f84b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c84b12, 90 },
    { 0xd2184b13, 23 },
    { 0xce84b14, 55 },
    { 0, 0 },
    { 0x36584b16, 90 },
    { 0xcc884b17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a84b1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b84b1f, 43 },
    { 0x6a584b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4884b25, 90 },
    { 0x16f84b26, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf584b2b, 77 },
    { 0, 0 },
    { 0x97f84b2d, 61 },
    { 0xff484b2e, 33 },
    { 0xa3d84b2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75684b36, 78 },
    { 0, 0 },
    { 0x52384b38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f984b3b, 55 },
    { 0xc2e84b3c, 96 },
    { 0xbf84b3c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75084b43, 96 },
    { 0xedd84b44, 90 },
    { 0x70484b45, 33 },
    { 0x98c84b46, 66 },
    { 0x95884b44, 23 },
    { 0, 0 },
    { 0xf2f84b49, 46 },
    { 0xa6784b4a, 23 },
    { 0x5084b4a, 78 },
    { 0, 0 },
    { 0x8f384b4d, 95 },
    { 0x2e084b4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb384b53, 90 },
    { 0, 0 },
    { 0xec584b55, 46 },
    { 0x2184b55, 55 },
    { 0xcc084b57, 55 },
    { 0, 0 },
    { 0x13b84b59, 43 },
    { 0x79e84b5a, 52 },
    { 0xb8384b59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62184b5e, 99 },
    { 0xea84b5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a84b66, 90 },
    { 0x61b84b67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee84b6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed584b70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f784b76, 52 },
    { 0, 0 },
    { 0x7ae84b78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f84b7b, 34 },
    { 0x4ea84b7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b484b80, 90 },
    { 0x6b684b81, 66 },
    { 0x9cf84b81, 23 },
    { 0x43d84b80, 33 },
    { 0x3b984b82, 33 },
    { 0x6c284b82, 66 },
    { 0x82584b86, 55 },
    { 0x27984b87, 95 },
    { 0xa3484b86, 90 },
    { 0, 0 },
    { 0x93784b8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb84b90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9484b9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb484b9e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x85984ba1, 43 },
    { 0xcea84ba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e84ba7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d84bac, 34 },
    { 0, 0 },
    { 0x74684bae, 90 },
    { 0xbe184bae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae84bb3, 33 },
    { 0, 0 },
    { 0x19f84bb5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22484bbb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae184bc2, 17 },
    { 0, 0 },
    { 0x8c584bc4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d84bc8, 23 },
    { 0x56884bc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e584bda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4984bdd, 91 },
    { 0, 0 },
    { 0x2c984bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9084be3, 98 },
    { 0x7f884be4, 46 },
    { 0x40484be5, 100 },
    { 0xf9184be6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d384bf2, 90 },
    { 0x3a984bf3, 95 },
    { 0x184bf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0284bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d84bfe, 40 },
    { 0x9a884bff, 33 },
    { 0xf8d84c00, 66 },
    { 0xd1984bfe, 95 },
    { 0x89584bff, 23 },
    { 0x62e84c01, 33 },
    { 0x2e084c04, 95 },
    { 0, 0 },
    { 0x52784c06, 55 },
    { 0x1ea84c06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2be84c0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f84c10, 76 },
    { 0xeca84c11, 23 },
    { 0x37184c11, 23 },
    { 0xf9684c13, 33 },
    { 0x36a84c14, 33 },
    { 0x3a384c11, 71 },
    { 0x1f284c16, 34 },
    { 0, 0 },
    { 0x11b84c18, 95 },
    { 0xb8384c18, 90 },
    { 0, 0 },
    { 0x9f484c1b, 95 },
    { 0x45384c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6084c22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb884c2c, 66 },
    { 0xb3e84c2d, 61 },
    { 0x3d684c2d, 61 },
    { 0xba684c2f, 66 },
    { 0xf6b84c2e, 34 },
    { 0xa0f84c31, 95 },
    { 0, 0 },
    { 0xb4b84c33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb84c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc84c3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc784c3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9284c43, 61 },
    { 0, 0 },
    { 0x88f84c45, 61 },
    { 0xa6d84c45, 95 },
    { 0xc4084c47, 23 },
    { 0xb4484c48, 98 },
    { 0xb7c84c47, 95 },
    { 0xc1884c49, 98 },
    { 0, 0 },
    { 0x1f484c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f84c50, 55 },
    { 0x50d84c50, 99 },
    { 0, 0 },
    { 0xe5984c53, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac84c57, 55 },
    { 0x94984c58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce84c5f, 90 },
    { 0, 0 },
    { 0x3c084c61, 98 },
    { 0x7bb84c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce084c67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d884c6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb584c72, 61 },
    { 0xc3684c72, 99 },
    { 0xca684c73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26684c79, 76 },
    { 0xaee84c7a, 90 },
    { 0, 0 },
    { 0x73184c7c, 95 },
    { 0xa2e84c7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda484c80, 99 },
    { 0, 0 },
    { 0xfa584c82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a84c85, 23 },
    { 0, 0 },
    { 0xb6a84c87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6284c8a, 90 },
    { 0, 0 },
    { 0x4584c8c, 67 },
    { 0x8f984c8d, 93 },
    { 0x41884c8e, 90 },
    { 0x7ac84c8e, 55 },
    { 0, 0 },
    { 0x9a184c91, 98 },
    { 0x4b684c91, 61 },
    { 0xd8e84c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb84c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5784c9c, 23 },
    { 0x8bb84c9c, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d084ca3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df84ca7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8584cac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf984caf, 33 },
    { 0x68484cb0, 23 },
    { 0, 0 },
    { 0x5f84cb2, 61 },
    { 0x94584cb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed84cb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a84cc1, 43 },
    { 0xa0c84cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12284cc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7084ccd, 78 },
    { 0x9e884cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95984cd4, 40 },
    { 0x1ba84cd4, 55 },
    { 0x8c584cd6, 61 },
    { 0x12684cd7, 90 },
    { 0xce284cd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce84cdd, 91 },
    { 0xa5f84cde, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb84ce1, 33 },
    { 0, 0 },
    { 0x3d484ce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1284ce8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2484cef, 100 },
    { 0xe3a84cf0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e84cf7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9184cfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86084d00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7284d04, 66 },
    { 0, 0 },
    { 0xef184d06, 81 },
    { 0x8f684d07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9584d0b, 95 },
    { 0x8c584d0c, 99 },
    { 0xec184d0d, 97 },
    { 0xca584d0b, 55 },
    { 0, 0 },
    { 0x85184d10, 40 },
    { 0, 0 },
    { 0xbc184d12, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e84d19, 98 },
    { 0, 0 },
    { 0xcb184d1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b284d1e, 95 },
    { 0xa6f84d1f, 90 },
    { 0x31184d20, 76 },
    { 0xb8f84d1e, 27 },
    { 0x13784d22, 33 },
    { 0xdee84d23, 95 },
    { 0x5a484d24, 61 },
    { 0x13e84d25, 76 },
    { 0xeb284d25, 43 },
    { 0xc8e84d20, 44 },
    { 0x15684d1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e784d2e, 23 },
    { 0x57c84d2f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9284d34, 95 },
    { 0x79784d35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6384d38, 76 },
    { 0, 0 },
    { 0x13084d3a, 67 },
    { 0xc9484d3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30184d40, 90 },
    { 0x9d084d40, 84 },
    { 0, 0 },
    { 0x22384d43, 61 },
    { 0x9dd84d44, 100 },
    { 0, 0 },
    { 0x6e084d46, 55 },
    { 0x58284d46, 98 },
    { 0xb9684d46, 33 },
    { 0x52e84d49, 34 },
    { 0xd5784d4a, 61 },
    { 0xa7c84d47, 76 },
    { 0xf2984d4c, 34 },
    { 0xe5c84d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0484d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7384d56, 78 },
    { 0xc2f84d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7484d5c, 61 },
    { 0xb84d5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c284d62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a84d65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b84d6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4484d70, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec84d76, 61 },
    { 0xd5584d77, 23 },
    { 0, 0 },
    { 0x34184d79, 52 },
    { 0x8784d7a, 91 },
    { 0x88184d7b, 61 },
    { 0x2e984d7b, 46 },
    { 0x88784d79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b84d82, 90 },
    { 0, 0 },
    { 0x6be84d84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x22484d87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a84d8b, 52 },
    { 0x6484d8c, 90 },
    { 0, 0 },
    { 0x6aa84d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc984d92, 76 },
    { 0, 0 },
    { 0xe9884d94, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e84d9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6684d9f, 95 },
    { 0xbad84da0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e84da7, 66 },
    { 0x9e584da8, 88 },
    { 0, 0 },
    { 0xaf784daa, 61 },
    { 0x70484dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea84dae, 67 },
    { 0xa1b84daf, 55 },
    { 0, 0 },
    { 0x47b84db1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x97384db4, 33 },
    { 0x4fc84db5, 55 },
    { 0xfb884db6, 77 },
    { 0xcac84db4, 23 },
    { 0xbe084db8, 52 },
    { 0x42384db9, 76 },
    { 0, 0 },
    { 0x85d84dbb, 23 },
    { 0x29f84dbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44784dc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f84dc5, 46 },
    { 0, 0 },
    { 0xfbc84dc7, 61 },
    { 0xd784dc8, 90 },
    { 0x96784dc8, 90 },
    { 0, 0 },
    { 0xb7384dcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97484dcf, 55 },
    { 0x60c84dcf, 33 },
    { 0xa1b84dd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf84dd4, 77 },
    { 0, 0 },
    { 0x35084dd6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be84dda, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64384dde, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b84de1, 43 },
    { 0xcca84de1, 90 },
    { 0x27584de3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b84de8, 55 },
    { 0, 0 },
    { 0x40984dea, 90 },
    { 0x7c984deb, 42 },
    { 0xf0884deb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab84df1, 61 },
    { 0, 0 },
    { 0xa2184df3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24484df6, 90 },
    { 0xc2084df7, 76 },
    { 0x12084df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b484dfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e84e00, 98 },
    { 0x53e84e01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1284e06, 78 },
    { 0x58384e07, 90 },
    { 0x88c84e08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c684e0c, 33 },
    { 0, 0 },
    { 0xdca84e0e, 46 },
    { 0x53784e0f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8384e12, 64 },
    { 0, 0 },
    { 0xaba84e14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6884e18, 67 },
    { 0xc3384e19, 95 },
    { 0x3f684e1a, 23 },
    { 0x42984e1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e20, 52 },
    { 0x25d84e21, 23 },
    { 0xa8284e20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea984e28, 93 },
    { 0, 0 },
    { 0xc4a84e2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b84e2d, 78 },
    { 0, 0 },
    { 0x8be84e2f, 78 },
    { 0x6984e30, 33 },
    { 0x11f84e31, 85 },
    { 0x30f84e32, 61 },
    { 0xfda84e32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd984e3a, 76 },
    { 0xdf584e3b, 95 },
    { 0, 0 },
    { 0xbac84e3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54784e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c84e45, 61 },
    { 0xe1084e46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d84e49, 61 },
    { 0, 0 },
    { 0x9f984e4b, 89 },
    { 0, 0 },
    { 0xbc84e4d, 65 },
    { 0x88084e4e, 23 },
    { 0, 0 },
    { 0x34f84e50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5684e57, 81 },
    { 0xf4384e58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83684e5d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15584e62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e84e65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x93984e68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7184e70, 70 },
    { 0x36684e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc84e75, 98 },
    { 0xa1b84e75, 61 },
    { 0, 0 },
    { 0x2d884e78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1784e7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25684e7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c084e81, 33 },
    { 0xba84e82, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b84e85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c84e88, 97 },
    { 0xc3984e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13584e8d, 90 },
    { 0xf5484e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a84e91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde384e97, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e84e9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c284e9e, 61 },
    { 0x2584e9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e84ea7, 33 },
    { 0x69284ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a84eab, 90 },
    { 0, 0 },
    { 0xf7184ead, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a84eb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc84eb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e84ebe, 93 },
    { 0, 0 },
    { 0xe2684ec0, 84 },
    { 0, 0 },
    { 0xd9584ec2, 76 },
    { 0x4ba84ec2, 90 },
    { 0x28884ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc484eca, 33 },
    { 0x12584ecb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69484ed1, 90 },
    { 0, 0 },
    { 0x40f84ed3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x54584ed6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe84edb, 85 },
    { 0xa8184edb, 33 },
    { 0x85a84edd, 85 },
    { 0x89584ede, 90 },
    { 0, 0 },
    { 0xe4684ee0, 33 },
    { 0x3a684ee0, 95 },
    { 0, 0 },
    { 0x81d84ee3, 55 },
    { 0, 0 },
    { 0x48f84ee5, 98 },
    { 0xaa884ee6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb784eec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a84ef5, 83 },
    { 0xdb284ef6, 66 },
    { 0, 0 },
    { 0x4f84ef8, 90 },
    { 0x4eb84ef8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31584efd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x56684f00, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e84f03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5884f06, 61 },
    { 0xd3684f06, 61 },
    { 0x4af84f08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f84f11, 43 },
    { 0x5c084f12, 68 },
    { 0xe884f13, 61 },
    { 0x7784f13, 61 },
    { 0x4cf84f12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f84f18, 95 },
    { 0x44584f19, 98 },
    { 0, 0 },
    { 0xc5784f1b, 90 },
    { 0x56784f1b, 23 },
    { 0xb3c84f1c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f84f20, 93 },
    { 0, 0 },
    { 0x75f84f22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b84f27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b84f2a, 34 },
    { 0xebb84f2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90984f30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc784f33, 66 },
    { 0x8a384f34, 90 },
    { 0xcd284f35, 65 },
    { 0x25884f36, 87 },
    { 0x48384f35, 90 },
    { 0xd9584f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c84f3b, 100 },
    { 0, 0 },
    { 0x92584f3d, 91 },
    { 0x76384f3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70484f41, 34 },
    { 0xdbe84f42, 34 },
    { 0x11484f41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48984f46, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c384f49, 61 },
    { 0x7c784f4a, 90 },
    { 0x7cd84f49, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1484f4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e884f52, 33 },
    { 0, 0 },
    { 0xa6984f54, 23 },
    { 0x4bb84f55, 33 },
    { 0, 0 },
    { 0x5a684f57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b84f5b, 65 },
    { 0xe2d84f5c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e84f61, 84 },
    { 0xbb484f62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6084f67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e584f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd84f71, 61 },
    { 0xb2984f72, 61 },
    { 0x12084f73, 43 },
    { 0x98284f71, 82 },
    { 0x46f84f72, 61 },
    { 0xa5784f74, 52 },
    { 0, 0 },
    { 0x2ad84f78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc84f81, 34 },
    { 0xe1e84f82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d84f87, 95 },
    { 0x4a184f87, 90 },
    { 0xd6e84f88, 33 },
    { 0x1e784f8a, 90 },
    { 0, 0 },
    { 0x45a84f8c, 46 },
    { 0x7ae84f8c, 93 },
    { 0, 0 },
    { 0x24984f8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34384f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a884f9a, 66 },
    { 0xee484f9b, 93 },
    { 0, 0 },
    { 0xd7084f9d, 77 },
    { 0xc6d84f9e, 64 },
    { 0xeb684f9f, 61 },
    { 0x90b84fa0, 55 },
    { 0x5d884f9f, 55 },
    { 0x9e984fa2, 84 },
    { 0x49c84fa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f84fa6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16084fa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19584fac, 76 },
    { 0x36a84fad, 98 },
    { 0x47684fac, 61 },
    { 0x67684faf, 61 },
    { 0xc3884fb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f84fb3, 95 },
    { 0x16284fb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x62984fb7, 90 },
    { 0x9a984fb8, 61 },
    { 0x39584fb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9684fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b84fc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e84fc9, 52 },
    { 0x10b84fc9, 61 },
    { 0xd7d84fcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7584fd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d584fd3, 95 },
    { 0x27484fd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b84fdc, 97 },
    { 0x43a84fdc, 33 },
    { 0xd8e84fdd, 61 },
    { 0, 0 },
    { 0xce884fe0, 33 },
    { 0, 0 },
    { 0x51784fe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c84fe6, 23 },
    { 0, 0 },
    { 0x51684fe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b84feb, 23 },
    { 0, 0 },
    { 0xbe084fed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6584ff3, 76 },
    { 0x7c84ff4, 67 },
    { 0x67484ff5, 61 },
    { 0, 0 },
    { 0x89d84ff7, 48 },
    { 0x7fc84ff7, 46 },
    { 0x22384ff7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10284ffc, 23 },
    { 0x5c884ffd, 91 },
    { 0x9f84ffe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34385005, 98 },
    { 0x61285006, 61 },
    { 0x91c85007, 95 },
    { 0, 0 },
    { 0x66b85009, 61 },
    { 0x6bb85009, 33 },
    { 0xfc48500b, 83 },
    { 0x1518500c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f85014, 98 },
    { 0x9c385015, 76 },
    { 0x83585016, 43 },
    { 0, 0 },
    { 0xfdd85018, 55 },
    { 0x7b885019, 95 },
    { 0, 0 },
    { 0x298501b, 76 },
    { 0xb328501c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa085020, 46 },
    { 0x46685021, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf185024, 33 },
    { 0xc2a85025, 96 },
    { 0xeb585026, 90 },
    { 0x62f85027, 55 },
    { 0, 0 },
    { 0x3f685029, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c38502d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca085031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e85034, 90 },
    { 0x2f985035, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6918503b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d88503f, 95 },
    { 0x5c85040, 99 },
    { 0x2238503f, 33 },
    { 0x33d85042, 90 },
    { 0x9e285040, 76 },
    { 0, 0 },
    { 0xaa085045, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a85049, 95 },
    { 0xdec85049, 61 },
    { 0, 0 },
    { 0xe738504c, 76 },
    { 0x4008504d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa685056, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64885059, 95 },
    { 0xf6285059, 46 },
    { 0xdcc8505a, 52 },
    { 0, 0 },
    { 0xa528505d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae85060, 23 },
    { 0, 0 },
    { 0xb0b85062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f85068, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca8506b, 23 },
    { 0x1fd8506c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae85072, 61 },
    { 0x83c85073, 76 },
    { 0x8d185072, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0085077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd658507f, 84 },
    { 0xe685080, 23 },
    { 0, 0 },
    { 0xb2485082, 43 },
    { 0x94185083, 23 },
    { 0xb9685084, 90 },
    { 0x10485083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8508b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a385095, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f85099, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf8509c, 23 },
    { 0xe1e8509d, 98 },
    { 0xc1c8509e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263850a2, 33 },
    { 0xfce850a2, 33 },
    { 0xf1b850a2, 99 },
    { 0x323850a4, 90 },
    { 0, 0 },
    { 0x226850a7, 55 },
    { 0x743850a8, 90 },
    { 0x994850a9, 98 },
    { 0, 0 },
    { 0x716850ab, 78 },
    { 0, 0 },
    { 0x5ac850ad, 33 },
    { 0xba8850ae, 34 },
    { 0xa3e850af, 90 },
    { 0x472850ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x958850b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55850b8, 98 },
    { 0xc9a850b8, 55 },
    { 0x600850ba, 93 },
    { 0xeac850bb, 95 },
    { 0xaaf850ba, 99 },
    { 0, 0 },
    { 0x3db850be, 23 },
    { 0xeb6850be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353850c6, 84 },
    { 0xbb850c7, 66 },
    { 0x532850c7, 93 },
    { 0x449850c9, 95 },
    { 0, 0 },
    { 0xdf3850cb, 33 },
    { 0x691850cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86850d0, 23 },
    { 0x3ea850d0, 33 },
    { 0x903850d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39850d6, 61 },
    { 0xeca850d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb850dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5850e0, 61 },
    { 0x600850e0, 23 },
    { 0xb24850e2, 43 },
    { 0x738850e3, 95 },
    { 0x5e0850e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29850eb, 78 },
    { 0xcc4850ec, 34 },
    { 0xf56850ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6850f2, 34 },
    { 0, 0 },
    { 0xaa1850f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d850f7, 90 },
    { 0x4b850f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e685101, 85 },
    { 0, 0 },
    { 0xa3785103, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99285107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd38510a, 87 },
    { 0, 0 },
    { 0xd6f8510c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5028510f, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8511a, 78 },
    { 0x2868511a, 98 },
    { 0x9578511a, 99 },
    { 0x8bb8511d, 33 },
    { 0x2d58511e, 67 },
    { 0x2f38511a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60485122, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f985126, 52 },
    { 0x89d85127, 98 },
    { 0xdc385127, 90 },
    { 0xf6885129, 98 },
    { 0xa8f85129, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe85130, 33 },
    { 0x4e285131, 61 },
    { 0xaa385130, 68 },
    { 0x5e985133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab38513d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37785140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98385146, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44585155, 46 },
    { 0xf7b85155, 84 },
    { 0xa5085157, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b8515c, 98 },
    { 0x5288515d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7685163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3385168, 33 },
    { 0, 0 },
    { 0x4b98516a, 23 },
    { 0x40c8516b, 95 },
    { 0xe818516a, 43 },
    { 0x3f68516d, 33 },
    { 0x47e8516a, 99 },
    { 0xb7a8516f, 90 },
    { 0xa6885170, 23 },
    { 0xddf8516f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a85179, 98 },
    { 0x4da85179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee78517e, 98 },
    { 0x2538517f, 78 },
    { 0x578517e, 84 },
    { 0xd3e85181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43685184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe758518c, 90 },
    { 0, 0 },
    { 0x4bd8518e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9785191, 34 },
    { 0x25085191, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f85195, 84 },
    { 0xc6985196, 90 },
    { 0, 0 },
    { 0xff285198, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe8519b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef8519e, 76 },
    { 0x9548519f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec851a2, 67 },
    { 0, 0 },
    { 0x55b851a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473851a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63851ad, 46 },
    { 0xf8b851ad, 87 },
    { 0xf97851af, 83 },
    { 0x82851b0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d851b3, 65 },
    { 0x9a1851b4, 91 },
    { 0x51f851b5, 78 },
    { 0xc60851b5, 61 },
    { 0x814851b6, 23 },
    { 0xbb4851b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61851c1, 90 },
    { 0x58f851c2, 34 },
    { 0x7b1851c3, 34 },
    { 0x92d851c1, 93 },
    { 0x75c851c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4851cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a851d0, 33 },
    { 0x797851d1, 84 },
    { 0xbd0851d2, 61 },
    { 0xbfa851d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x676851d6, 23 },
    { 0xc6851d7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a851da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x279851dd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40851e0, 100 },
    { 0x510851e1, 90 },
    { 0x254851e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52851ea, 61 },
    { 0, 0 },
    { 0xc88851ec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163851f0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6851f3, 33 },
    { 0x80851f4, 90 },
    { 0x29e851f5, 95 },
    { 0x8dd851f6, 100 },
    { 0xf3c851f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a851fc, 90 },
    { 0x36b851fd, 91 },
    { 0x4ff851fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d185204, 34 },
    { 0x36485205, 34 },
    { 0, 0 },
    { 0xf1985207, 59 },
    { 0x81c85208, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2685211, 87 },
    { 0xe2985212, 34 },
    { 0, 0 },
    { 0xed185214, 61 },
    { 0xaad85214, 91 },
    { 0, 0 },
    { 0x4df85217, 76 },
    { 0x47385218, 100 },
    { 0x38b85219, 90 },
    { 0x4618521a, 90 },
    { 0x9838521b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b85221, 98 },
    { 0, 0 },
    { 0xb1d85223, 90 },
    { 0x4c085224, 61 },
    { 0x2eb85225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7808522c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7685235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8523b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb85240, 90 },
    { 0, 0 },
    { 0x56f85242, 95 },
    { 0, 0 },
    { 0x83f85244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7885249, 90 },
    { 0xc618524a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b8524e, 46 },
    { 0xcf98524f, 33 },
    { 0, 0 },
    { 0x9dd85251, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9585254, 90 },
    { 0xcfd85255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca18525a, 36 },
    { 0xfa58525b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e28525e, 93 },
    { 0x50d8525f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5085262, 95 },
    { 0x61985263, 34 },
    { 0, 0 },
    { 0xa1085265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38526b, 40 },
    { 0x1ed8526c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71785271, 36 },
    { 0, 0 },
    { 0x4dd85273, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf85278, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e08527c, 99 },
    { 0x6ae8527d, 40 },
    { 0, 0 },
    { 0x8458527f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60785285, 61 },
    { 0xb7c85285, 90 },
    { 0x52385285, 43 },
    { 0xf7285288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e8528b, 76 },
    { 0xed48528b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e8528f, 90 },
    { 0xc158528f, 90 },
    { 0xc38528f, 61 },
    { 0x3eb85292, 77 },
    { 0, 0 },
    { 0x78b85294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77185299, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf068529e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34852aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a852ad, 55 },
    { 0, 0 },
    { 0x498852af, 76 },
    { 0x558852af, 98 },
    { 0x28852b1, 75 },
    { 0x19a852b2, 95 },
    { 0xefa852af, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c852b6, 90 },
    { 0xce6852b7, 78 },
    { 0xe61852b8, 23 },
    { 0, 0 },
    { 0xd4e852ba, 33 },
    { 0x4e8852ba, 91 },
    { 0xd57852bc, 90 },
    { 0xbae852bd, 23 },
    { 0, 0 },
    { 0x350852bf, 88 },
    { 0x593852bf, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef852c5, 55 },
    { 0xb95852c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e852c9, 90 },
    { 0, 0 },
    { 0x780852cb, 87 },
    { 0x9f2852cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a852d0, 33 },
    { 0x22e852d1, 96 },
    { 0xe4852d2, 45 },
    { 0x85e852d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91852d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x799852dc, 98 },
    { 0x895852dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2852e2, 43 },
    { 0, 0 },
    { 0x651852e4, 99 },
    { 0xfa0852e5, 98 },
    { 0xf7a852e6, 77 },
    { 0xefa852e6, 99 },
    { 0, 0 },
    { 0x84852e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29852ee, 76 },
    { 0xfc7852ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2852f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be852fc, 90 },
    { 0x8dc852fd, 23 },
    { 0x61f852fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c85305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4658530b, 61 },
    { 0xee8530b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c85310, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96985314, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97385318, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6668531e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c85323, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b85326, 69 },
    { 0x6eb85327, 59 },
    { 0xfed85326, 34 },
    { 0xcbf85329, 23 },
    { 0xf4585327, 61 },
    { 0, 0 },
    { 0xdea8532c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f685330, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12285338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb8533b, 81 },
    { 0x8ab8533b, 33 },
    { 0x98533d, 76 },
    { 0x7528533d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78485341, 91 },
    { 0x55e85342, 61 },
    { 0xde085341, 76 },
    { 0xb8185341, 33 },
    { 0xd3285344, 33 },
    { 0, 0 },
    { 0x34d85347, 81 },
    { 0xc7c85348, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5468534c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa208534f, 85 },
    { 0x32a85350, 23 },
    { 0xf2885351, 23 },
    { 0, 0 },
    { 0x15685353, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b8535b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a85361, 24 },
    { 0, 0 },
    { 0x72d85363, 99 },
    { 0, 0 },
    { 0x7ab85365, 66 },
    { 0, 0 },
    { 0x57c85367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae38536a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf98536d, 67 },
    { 0, 0 },
    { 0x1a78536f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc85372, 87 },
    { 0x37f85373, 78 },
    { 0x20085372, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d85378, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd8537b, 90 },
    { 0x2858537c, 59 },
    { 0x988537d, 95 },
    { 0xf2c8537d, 33 },
    { 0x1178537d, 55 },
    { 0, 0 },
    { 0xcfb85381, 85 },
    { 0x9a985382, 61 },
    { 0, 0 },
    { 0xc7485384, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed085388, 99 },
    { 0x55185388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff8538e, 95 },
    { 0xd3d8538f, 55 },
    { 0xcee85390, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18585393, 37 },
    { 0x63985394, 98 },
    { 0x25785395, 23 },
    { 0xb6485394, 95 },
    { 0, 0 },
    { 0xc5585398, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928539e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16853a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462853a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f853a8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d853ab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4853ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83853b6, 34 },
    { 0, 0 },
    { 0x6d6853b8, 61 },
    { 0x95e853b9, 46 },
    { 0x6cd853ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec853be, 65 },
    { 0x93b853bf, 34 },
    { 0xf4c853bf, 95 },
    { 0, 0 },
    { 0xea7853c2, 90 },
    { 0xc76853c3, 76 },
    { 0x89e853c4, 46 },
    { 0, 0 },
    { 0xfbd853c6, 33 },
    { 0xf7d853c6, 76 },
    { 0xa1d853c8, 23 },
    { 0x438853c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4853cf, 98 },
    { 0x4d1853cf, 61 },
    { 0x900853d1, 98 },
    { 0x422853d2, 61 },
    { 0x3d7853d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe853d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c853de, 91 },
    { 0xcb6853df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x523853e2, 88 },
    { 0x3eb853e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4853e8, 90 },
    { 0xd98853e9, 88 },
    { 0x5ef853e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b853ed, 93 },
    { 0, 0 },
    { 0x44d853ef, 34 },
    { 0xd23853f0, 93 },
    { 0, 0 },
    { 0xa7d853f2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973853f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a853fb, 98 },
    { 0x74853fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33685403, 66 },
    { 0x90285404, 83 },
    { 0, 0 },
    { 0x35d85406, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d8540a, 93 },
    { 0x2758540b, 81 },
    { 0x1878540b, 90 },
    { 0x8638540d, 76 },
    { 0x4468540d, 61 },
    { 0x32d8540f, 34 },
    { 0xbae85410, 68 },
    { 0, 0 },
    { 0x84e85412, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a685419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d8541e, 98 },
    { 0xef18541f, 98 },
    { 0x1f685420, 34 },
    { 0, 0 },
    { 0x38f85422, 61 },
    { 0, 0 },
    { 0x9f785424, 65 },
    { 0x8d785425, 87 },
    { 0xb7185424, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc8542f, 61 },
    { 0, 0 },
    { 0xbee85431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3585434, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2385437, 81 },
    { 0xade85438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80985442, 98 },
    { 0xf9185443, 98 },
    { 0xf2085443, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8544b, 23 },
    { 0xcf28544c, 78 },
    { 0xb608544b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf085450, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde685454, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8545d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89385460, 23 },
    { 0xe2085461, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b485465, 34 },
    { 0x78085465, 76 },
    { 0x5585467, 90 },
    { 0x9fd85467, 91 },
    { 0xa7285466, 66 },
    { 0xa268546a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe778546e, 95 },
    { 0, 0 },
    { 0xe5185470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a85474, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64785478, 23 },
    { 0, 0 },
    { 0x25f8547a, 95 },
    { 0x31e8547b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8547e, 33 },
    { 0x3f18547f, 52 },
    { 0x2a48547e, 23 },
    { 0xcc48547e, 55 },
    { 0xff88547f, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c85488, 90 },
    { 0xd3f85489, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb8548c, 76 },
    { 0, 0 },
    { 0x2ed8548e, 34 },
    { 0x67c8548f, 45 },
    { 0xa848548e, 66 },
    { 0xe7685491, 61 },
    { 0, 0 },
    { 0xbf985493, 76 },
    { 0x985494, 98 },
    { 0x4ed85494, 55 },
    { 0x4d085494, 61 },
    { 0x63885497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be8549d, 90 },
    { 0x7ff8549d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f854a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b854a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1854ad, 88 },
    { 0x521854ae, 23 },
    { 0x635854ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86854b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a854ba, 61 },
    { 0, 0 },
    { 0xe0b854bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2854c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3854cb, 90 },
    { 0, 0 },
    { 0x444854cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86854d1, 33 },
    { 0xcfc854d1, 61 },
    { 0x5af854d1, 76 },
    { 0, 0 },
    { 0xbfb854d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f854df, 33 },
    { 0x6e4854df, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874854e4, 33 },
    { 0, 0 },
    { 0x63f854e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1854ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f854f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1854f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d854fd, 78 },
    { 0x53854fe, 90 },
    { 0x49a854ff, 90 },
    { 0, 0 },
    { 0x24385501, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde85504, 98 },
    { 0x9ef85505, 93 },
    { 0, 0 },
    { 0xe7085507, 78 },
    { 0, 0 },
    { 0x87085509, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8550e, 84 },
    { 0, 0 },
    { 0xb7485510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be85514, 61 },
    { 0x6a485515, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c8551b, 33 },
    { 0xd168551c, 23 },
    { 0xcba8551d, 92 },
    { 0x2748551e, 43 },
    { 0, 0 },
    { 0x57085520, 66 },
    { 0xaff85521, 90 },
    { 0xc6785522, 76 },
    { 0x3af85521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe08552f, 43 },
    { 0, 0 },
    { 0xe9385531, 33 },
    { 0, 0 },
    { 0x4c685533, 90 },
    { 0, 0 },
    { 0x6eb85535, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f85539, 100 },
    { 0xd1285539, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62285540, 52 },
    { 0xbc485541, 61 },
    { 0x9e685540, 34 },
    { 0x55685542, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4185546, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86685549, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078554e, 23 },
    { 0x4c98554f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8685552, 98 },
    { 0x2da85553, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb585556, 33 },
    { 0x1d385556, 66 },
    { 0x81785557, 61 },
    { 0x81585558, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9938555c, 98 },
    { 0, 0 },
    { 0x4f68555e, 90 },
    { 0x7988555f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84285564, 46 },
    { 0xbe785565, 55 },
    { 0x3ce85566, 46 },
    { 0xba785567, 46 },
    { 0x70385565, 77 },
    { 0, 0 },
    { 0x4728556a, 53 },
    { 0x2468556b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158556f, 75 },
    { 0xac58556f, 76 },
    { 0x5c85570, 23 },
    { 0xbcf85572, 43 },
    { 0x33185572, 33 },
    { 0xd7d85574, 90 },
    { 0xdfa85572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa8557f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd185583, 98 },
    { 0xbff85583, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34685588, 95 },
    { 0xb6e85589, 98 },
    { 0xcb28558a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c8558e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d85592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb85596, 61 },
    { 0x9c485597, 93 },
    { 0x8e485598, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f8559f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832855a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2855a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481855ab, 76 },
    { 0, 0 },
    { 0x190855ad, 61 },
    { 0x682855ae, 93 },
    { 0xe7e855af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd855b2, 100 },
    { 0, 0 },
    { 0xe8b855b4, 55 },
    { 0, 0 },
    { 0x3c4855b6, 23 },
    { 0, 0 },
    { 0x1e9855b8, 33 },
    { 0xf70855b9, 76 },
    { 0x993855b8, 77 },
    { 0x48a855bb, 76 },
    { 0x87855bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c855c0, 23 },
    { 0x76e855c1, 34 },
    { 0x275855c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2855ca, 43 },
    { 0xb60855ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3855d0, 90 },
    { 0x45d855d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7855d8, 98 },
    { 0, 0 },
    { 0x5fc855da, 99 },
    { 0xc7d855db, 23 },
    { 0x999855dc, 33 },
    { 0x771855dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x571855e0, 61 },
    { 0xc49855e1, 33 },
    { 0x367855e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af855e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a855eb, 95 },
    { 0, 0 },
    { 0x7d0855ed, 99 },
    { 0x726855ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe855f4, 90 },
    { 0x58855f5, 76 },
    { 0, 0 },
    { 0x6e5855f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81855fd, 55 },
    { 0x6c6855fe, 80 },
    { 0x2d5855fe, 76 },
    { 0x2f985600, 33 },
    { 0xe8785601, 23 },
    { 0xf6a85602, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a8560e, 19 },
    { 0xf6e8560f, 90 },
    { 0xb8e85610, 34 },
    { 0x98f85610, 23 },
    { 0x1b785610, 98 },
    { 0x3cd85612, 90 },
    { 0xa6985611, 86 },
    { 0x50a85612, 75 },
    { 0x1ba85612, 55 },
    { 0x3be85617, 68 },
    { 0x59285618, 43 },
    { 0x78585619, 23 },
    { 0x22d85613, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98285622, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3285625, 61 },
    { 0, 0 },
    { 0x8a485627, 43 },
    { 0x9ff85628, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb428562c, 90 },
    { 0xe6e8562d, 33 },
    { 0xa3f8562e, 46 },
    { 0xe5c8562c, 33 },
    { 0xc3785630, 33 },
    { 0x18185631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed385635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd85639, 46 },
    { 0x5538563a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1848563d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31485648, 23 },
    { 0, 0 },
    { 0x30b8564a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b68564f, 90 },
    { 0xe2a85650, 23 },
    { 0, 0 },
    { 0x53d85652, 55 },
    { 0x85e85653, 23 },
    { 0xde185653, 61 },
    { 0x71e85652, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d8565b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e8565f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e85662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc85668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf368566b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db8566e, 64 },
    { 0xeb08566e, 91 },
    { 0xbe685670, 63 },
    { 0, 0 },
    { 0x85b85672, 90 },
    { 0xc6285673, 34 },
    { 0, 0 },
    { 0x50a85675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad85679, 90 },
    { 0, 0 },
    { 0x7d8567b, 52 },
    { 0, 0 },
    { 0xdae8567d, 46 },
    { 0x46a8567e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7685686, 43 },
    { 0x26e85687, 81 },
    { 0x82085687, 33 },
    { 0x4db85689, 43 },
    { 0x60c8568a, 76 },
    { 0x49f8568b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee8568e, 61 },
    { 0x4f28568f, 46 },
    { 0x1b885690, 61 },
    { 0x86f85690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec85694, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8385698, 68 },
    { 0xfe985698, 90 },
    { 0xc9485698, 33 },
    { 0xafe8569a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4918569e, 100 },
    { 0xc358569f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3856a3, 43 },
    { 0xbaf856a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd856a8, 95 },
    { 0x19a856a9, 90 },
    { 0x3c4856aa, 90 },
    { 0x35c856aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79856b0, 88 },
    { 0x610856b1, 98 },
    { 0, 0 },
    { 0x26f856b3, 88 },
    { 0xd0e856b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823856ba, 90 },
    { 0, 0 },
    { 0x108856bc, 95 },
    { 0x75c856bd, 61 },
    { 0, 0 },
    { 0xdfd856bf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325856c3, 76 },
    { 0, 0 },
    { 0x383856c5, 90 },
    { 0, 0 },
    { 0xd67856c7, 95 },
    { 0, 0 },
    { 0x484856c9, 55 },
    { 0x4ff856c9, 33 },
    { 0xb08856ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2856ce, 95 },
    { 0x983856cf, 43 },
    { 0xf76856cf, 98 },
    { 0x9a6856d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c856e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c856e6, 90 },
    { 0, 0 },
    { 0x4db856e8, 91 },
    { 0xee4856e8, 76 },
    { 0, 0 },
    { 0x88856eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x736856ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0856f2, 46 },
    { 0xdb5856f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8856f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed856fc, 76 },
    { 0x247856fc, 61 },
    { 0x507856fc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef85707, 52 },
    { 0xac185708, 33 },
    { 0x40485707, 33 },
    { 0x21c85709, 90 },
    { 0xa0d85707, 61 },
    { 0, 0 },
    { 0x1848570d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d85710, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b85716, 33 },
    { 0xb9485717, 61 },
    { 0xae385716, 98 },
    { 0x3a385719, 61 },
    { 0, 0 },
    { 0xccd8571b, 23 },
    { 0xf368571b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df8571f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e85726, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda985729, 61 },
    { 0x11c8572a, 90 },
    { 0xa3685729, 23 },
    { 0, 0 },
    { 0xbde8572d, 78 },
    { 0x2838572e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2785732, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b185737, 33 },
    { 0x56785738, 93 },
    { 0x72785738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e885745, 61 },
    { 0x72f85746, 76 },
    { 0xc1e85747, 91 },
    { 0x8c985746, 83 },
    { 0xf6d85747, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa85750, 46 },
    { 0x9a285751, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b85759, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e98575c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44685761, 76 },
    { 0xac785762, 80 },
    { 0xc9185763, 55 },
    { 0x9a785762, 33 },
    { 0xd7d85765, 72 },
    { 0x5bd85765, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98576a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c8577b, 61 },
    { 0x9438577b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d85786, 90 },
    { 0xbce85787, 34 },
    { 0, 0 },
    { 0xbb685789, 80 },
    { 0x6688578a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9068578d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc85791, 75 },
    { 0x7185792, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3538579d, 90 },
    { 0x3008579e, 76 },
    { 0xb608579f, 43 },
    { 0, 0 },
    { 0xc04857a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed857a6, 55 },
    { 0, 0 },
    { 0xf48857a8, 98 },
    { 0xd93857a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba857ac, 55 },
    { 0xf8d857ad, 90 },
    { 0xb53857ae, 76 },
    { 0x5a5857ae, 23 },
    { 0, 0 },
    { 0x2db857b1, 46 },
    { 0x54b857b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697857b6, 90 },
    { 0x164857b7, 80 },
    { 0xe41857b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4857bc, 95 },
    { 0, 0 },
    { 0xc41857be, 98 },
    { 0x6fd857bf, 61 },
    { 0, 0 },
    { 0x5d9857c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50857ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce857cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54857d0, 61 },
    { 0xa0c857d1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f857d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x320857d7, 90 },
    { 0x3e9857d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x423857db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4857de, 76 },
    { 0, 0 },
    { 0x23e857e0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6857e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1857e7, 65 },
    { 0xaf857e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93857eb, 60 },
    { 0, 0 },
    { 0x433857ed, 95 },
    { 0xf70857ee, 95 },
    { 0x144857ed, 61 },
    { 0, 0 },
    { 0xed8857f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd857f4, 98 },
    { 0xe97857f5, 34 },
    { 0, 0 },
    { 0xde6857f7, 23 },
    { 0x79d857f8, 90 },
    { 0x21a857f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x384857fc, 81 },
    { 0, 0 },
    { 0x202857fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xff785801, 75 },
    { 0x3f485802, 77 },
    { 0x7f685802, 23 },
    { 0x34285804, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c18580d, 98 },
    { 0xeb68580e, 33 },
    { 0x9268580d, 89 },
    { 0, 0 },
    { 0x70085811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc585814, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f85818, 23 },
    { 0, 0 },
    { 0x9458581a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc588581f, 61 },
    { 0, 0 },
    { 0xf9685821, 61 },
    { 0, 0 },
    { 0xcf885823, 85 },
    { 0x55d85824, 55 },
    { 0xb5985825, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae8582d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa685830, 61 },
    { 0xc6e85831, 98 },
    { 0x98885832, 33 },
    { 0xd3485832, 83 },
    { 0x71385833, 76 },
    { 0xe7a85833, 99 },
    { 0xa5985834, 61 },
    { 0xba085837, 61 },
    { 0xfb285837, 42 },
    { 0x2be85839, 95 },
    { 0x69c85839, 95 },
    { 0x7848583b, 91 },
    { 0x3978583c, 95 },
    { 0x6e38583c, 93 },
    { 0x54785837, 95 },
    { 0x6d68583f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d885842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a85845, 55 },
    { 0x14e85846, 61 },
    { 0x58e85847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c8584d, 46 },
    { 0, 0 },
    { 0xe028584f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd85852, 93 },
    { 0, 0 },
    { 0xcbf85854, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30185858, 34 },
    { 0, 0 },
    { 0xea58585a, 84 },
    { 0xb068585b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac85860, 90 },
    { 0x2c985861, 93 },
    { 0xa0085861, 33 },
    { 0xe5485861, 75 },
    { 0, 0 },
    { 0xebf85865, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78586c, 90 },
    { 0, 0 },
    { 0x2b48586e, 100 },
    { 0xa998586f, 44 },
    { 0xbc18586e, 23 },
    { 0x99f85870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80885877, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba8587a, 98 },
    { 0xf2d8587a, 98 },
    { 0, 0 },
    { 0x3bd8587d, 67 },
    { 0, 0 },
    { 0x3d18587f, 93 },
    { 0x15785880, 17 },
    { 0, 0 },
    { 0x32d85882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53785885, 98 },
    { 0x37385886, 85 },
    { 0x74585887, 61 },
    { 0xad585886, 61 },
    { 0xcd985888, 91 },
    { 0x95e8588a, 98 },
    { 0xe3885886, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb428588e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73585891, 90 },
    { 0xf8485891, 95 },
    { 0x83e85892, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc85896, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c85899, 91 },
    { 0xf5f8589a, 61 },
    { 0x3128589b, 83 },
    { 0x6c08589c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471858a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0858a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5858aa, 46 },
    { 0xd858ab, 77 },
    { 0x989858ab, 95 },
    { 0x6c2858ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86858b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8858bb, 33 },
    { 0x95b858bb, 100 },
    { 0x6c1858bb, 93 },
    { 0xc8b858be, 9 },
    { 0xd39858bb, 17 },
    { 0x82d858bb, 95 },
    { 0x5eb858be, 46 },
    { 0, 0 },
    { 0xa2858c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3858c6, 23 },
    { 0, 0 },
    { 0xf53858c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54858cc, 68 },
    { 0, 0 },
    { 0x75f858ce, 95 },
    { 0x55d858ce, 34 },
    { 0x149858cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4858d7, 90 },
    { 0, 0 },
    { 0x759858d9, 90 },
    { 0, 0 },
    { 0x885858db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb858e1, 43 },
    { 0xd39858e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1858e6, 61 },
    { 0x91c858e7, 93 },
    { 0xbeb858e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe858f9, 95 },
    { 0, 0 },
    { 0x71d858fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a858fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe85901, 80 },
    { 0xc6385902, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36585909, 91 },
    { 0, 0 },
    { 0x3888590b, 43 },
    { 0x49c8590c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67085910, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f85914, 91 },
    { 0x8a785915, 98 },
    { 0, 0 },
    { 0xdae85917, 34 },
    { 0x67485918, 23 },
    { 0x94785917, 69 },
    { 0, 0 },
    { 0x8da8591b, 46 },
    { 0, 0 },
    { 0xc68591d, 98 },
    { 0, 0 },
    { 0x6b58591f, 100 },
    { 0, 0 },
    { 0xe5a85921, 33 },
    { 0x37785921, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1185925, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb585928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7298592d, 76 },
    { 0x9bf8592e, 98 },
    { 0x1028592d, 95 },
    { 0x5bd85930, 61 },
    { 0x6fa85931, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418593a, 22 },
    { 0, 0 },
    { 0x2a98593c, 98 },
    { 0xa28593d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55085940, 88 },
    { 0xc5185941, 90 },
    { 0x85985940, 65 },
    { 0, 0 },
    { 0x87785944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb278594a, 23 },
    { 0x9288594b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48594f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24185955, 22 },
    { 0x5a385956, 98 },
    { 0xa0985956, 95 },
    { 0xc3885958, 95 },
    { 0, 0 },
    { 0xfa78595a, 61 },
    { 0xc578595b, 66 },
    { 0, 0 },
    { 0xd9c8595d, 55 },
    { 0xc8595e, 33 },
    { 0xf0c8595f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba85962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f85967, 33 },
    { 0x82c85967, 91 },
    { 0x7b685969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8588596c, 33 },
    { 0x8b78596c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68885978, 99 },
    { 0x80085979, 23 },
    { 0xb4b8597a, 87 },
    { 0x91f8597b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a85981, 66 },
    { 0x88f85981, 15 },
    { 0, 0 },
    { 0x56485984, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e785989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe85991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef885998, 43 },
    { 0x32285998, 99 },
    { 0xc4d8599a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2859a1, 76 },
    { 0xaaf859a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f859a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e859a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546859b1, 93 },
    { 0xe83859b2, 23 },
    { 0x662859b3, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713859bd, 76 },
    { 0x94f859be, 43 },
    { 0x8f859be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x711859c2, 61 },
    { 0, 0 },
    { 0x251859c4, 61 },
    { 0x340859c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x384859c8, 91 },
    { 0xb4a859c9, 46 },
    { 0xd61859c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x272859cd, 61 },
    { 0, 0 },
    { 0x435859cf, 33 },
    { 0x480859cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639859d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81859d7, 95 },
    { 0x169859d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f859de, 92 },
    { 0xcf0859df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0859e4, 91 },
    { 0x257859e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f859ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b859f0, 66 },
    { 0, 0 },
    { 0xb5e859f2, 55 },
    { 0x527859f3, 46 },
    { 0xbc8859f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3859fa, 33 },
    { 0xa6b859fb, 43 },
    { 0, 0 },
    { 0xa1859fd, 94 },
    { 0xef7859fe, 78 },
    { 0, 0 },
    { 0xf1a85a00, 34 },
    { 0xc2b85a00, 87 },
    { 0xfbf85a00, 85 },
    { 0x2d85a03, 95 },
    { 0x3df85a04, 95 },
    { 0xa2885a04, 43 },
    { 0xc6085a04, 40 },
    { 0, 0 },
    { 0x24885a08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76285a0e, 52 },
    { 0, 0 },
    { 0x86385a10, 76 },
    { 0, 0 },
    { 0xd0385a12, 98 },
    { 0xfea85a12, 33 },
    { 0xab685a14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf85a19, 84 },
    { 0, 0 },
    { 0x17985a1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38785a1e, 66 },
    { 0, 0 },
    { 0x13585a20, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc585a24, 76 },
    { 0, 0 },
    { 0xe0085a26, 33 },
    { 0, 0 },
    { 0x19585a28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c885a2b, 76 },
    { 0x5485a2c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b885a31, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff885a38, 66 },
    { 0x71d85a39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a085a41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f385a48, 98 },
    { 0, 0 },
    { 0x46385a4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe85a53, 55 },
    { 0x64885a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34685a58, 95 },
    { 0xd4985a59, 38 },
    { 0, 0 },
    { 0x67985a5b, 61 },
    { 0, 0 },
    { 0x38285a5d, 68 },
    { 0xc4185a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e85a66, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7985a6a, 90 },
    { 0xbad85a6b, 55 },
    { 0x7ee85a6b, 98 },
    { 0x9c285a6b, 23 },
    { 0xb0385a6b, 90 },
    { 0xefa85a6f, 33 },
    { 0x4a285a6d, 90 },
    { 0, 0 },
    { 0xa9785a72, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd85a75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b85a79, 33 },
    { 0xcd85a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f885a7d, 34 },
    { 0x4f685a7e, 93 },
    { 0x1c885a7f, 98 },
    { 0xae185a80, 90 },
    { 0x75f85a7f, 66 },
    { 0x3aa85a82, 90 },
    { 0x95e85a83, 56 },
    { 0xafc85a82, 69 },
    { 0, 0 },
    { 0xd0b85a86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e885a8e, 90 },
    { 0xa0b85a8f, 90 },
    { 0x3c485a90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee085a98, 37 },
    { 0xe885a99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64085a9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc85aa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe85aa4, 34 },
    { 0x15b85aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4085aa9, 34 },
    { 0xc1385aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53785ab4, 95 },
    { 0x32585ab4, 23 },
    { 0xda285ab6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39085abe, 24 },
    { 0x5f985abe, 43 },
    { 0, 0 },
    { 0x53c85ac1, 48 },
    { 0x2ac85ac1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd285ac7, 95 },
    { 0xc7d85ac8, 99 },
    { 0x77f85ac9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c85acf, 61 },
    { 0x48285ad0, 95 },
    { 0xae285ad1, 23 },
    { 0x5e085ad2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7185ad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e85ada, 90 },
    { 0x1a185adb, 90 },
    { 0xb7a85adc, 88 },
    { 0xff585adb, 76 },
    { 0xb0f85add, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c85ae7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f685aea, 95 },
    { 0x7e385aeb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c85af0, 33 },
    { 0x62685af1, 93 },
    { 0x1ae85af2, 23 },
    { 0xdc385af3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad85af6, 33 },
    { 0x64285af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2885afa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8285b01, 61 },
    { 0, 0 },
    { 0xbfd85b03, 33 },
    { 0x1d85b03, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed85b0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61785b15, 97 },
    { 0xfcb85b16, 99 },
    { 0, 0 },
    { 0xddd85b18, 33 },
    { 0x3da85b19, 75 },
    { 0xc2685b1a, 98 },
    { 0xddf85b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7885b1e, 98 },
    { 0, 0 },
    { 0x79e85b20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb85b23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e285b28, 33 },
    { 0x8aa85b29, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d85b2c, 34 },
    { 0x21285b2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda885b31, 46 },
    { 0, 0 },
    { 0x82e85b33, 77 },
    { 0xe0d85b34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd85b38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd85b41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0785b48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e385b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42385b50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f85b54, 90 },
    { 0, 0 },
    { 0x9af85b56, 90 },
    { 0xdda85b56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33885b5a, 33 },
    { 0, 0 },
    { 0x7b485b5c, 34 },
    { 0x5de85b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80785b63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f85b66, 99 },
    { 0, 0 },
    { 0x94385b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f585b6b, 98 },
    { 0, 0 },
    { 0x42f85b6d, 93 },
    { 0, 0 },
    { 0x1ca85b6f, 90 },
    { 0xef885b70, 38 },
    { 0x91785b70, 90 },
    { 0x3c185b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8785b77, 43 },
    { 0xa1285b78, 33 },
    { 0xfd185b79, 98 },
    { 0x37a85b78, 95 },
    { 0, 0 },
    { 0xf3185b7c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f85b7f, 55 },
    { 0xf8085b80, 93 },
    { 0x5aa85b81, 43 },
    { 0x87685b80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3485b85, 23 },
    { 0xaad85b86, 61 },
    { 0xfe285b86, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e985b8b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd85b8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b85b92, 99 },
    { 0x7ed85b93, 80 },
    { 0xc3a85b93, 92 },
    { 0xe7b85b95, 76 },
    { 0x13a85b95, 61 },
    { 0xbc385b92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9185ba0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f285ba4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19685ba8, 55 },
    { 0, 0 },
    { 0x60585baa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8885bb0, 61 },
    { 0xa6885bb0, 33 },
    { 0, 0 },
    { 0xcbc85bb3, 66 },
    { 0x56685bb4, 87 },
    { 0xd0e85bb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e285bbe, 98 },
    { 0x50185bbf, 90 },
    { 0xb1185bc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70385bc4, 98 },
    { 0, 0 },
    { 0x6a885bc6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e85bc9, 98 },
    { 0x12b85bca, 49 },
    { 0xa6585bc9, 95 },
    { 0xd1685bc9, 76 },
    { 0xa1585bcc, 33 },
    { 0xcb485bce, 99 },
    { 0, 0 },
    { 0xd6c85bd0, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa85bd4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96685bd8, 90 },
    { 0x79d85bd9, 33 },
    { 0xf3085bd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde85be1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf985be4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0485be7, 52 },
    { 0x4f085be8, 61 },
    { 0x51b85be9, 75 },
    { 0x3d485be9, 100 },
    { 0x90c85beb, 61 },
    { 0x68a85bec, 78 },
    { 0x21e85bed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5585bf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2585bf4, 76 },
    { 0xf7b85bf5, 61 },
    { 0xdba85bf5, 95 },
    { 0x1ac85bf7, 98 },
    { 0xaf785bf8, 95 },
    { 0xa0d85bf4, 90 },
    { 0xa3585bfa, 93 },
    { 0x96585bfb, 53 },
    { 0x5fc85bfc, 98 },
    { 0x1f285bfb, 61 },
    { 0xc5385bfc, 90 },
    { 0xe2185bff, 46 },
    { 0xb0185c00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7885c08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48885c0c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf785c12, 97 },
    { 0, 0 },
    { 0xc1f85c14, 95 },
    { 0x45185c15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59985c19, 52 },
    { 0xd7085c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10485c1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d85c27, 85 },
    { 0xe7585c28, 90 },
    { 0x5bb85c27, 84 },
    { 0xc9985c29, 61 },
    { 0, 0 },
    { 0x50485c2c, 52 },
    { 0x78385c2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e85c31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b85c36, 23 },
    { 0xcdc85c37, 93 },
    { 0xf8985c36, 100 },
    { 0x37685c39, 43 },
    { 0x68185c3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e685c3f, 90 },
    { 0xb5d85c40, 61 },
    { 0x37885c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b885c45, 78 },
    { 0xa285c45, 95 },
    { 0xda485c47, 23 },
    { 0x10385c47, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb085c4d, 61 },
    { 0, 0 },
    { 0x4ef85c4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8685c52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad85c59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22085c5c, 51 },
    { 0, 0 },
    { 0xad685c5e, 34 },
    { 0x21f85c5e, 33 },
    { 0x4b685c5f, 36 },
    { 0, 0 },
    { 0xa3485c62, 23 },
    { 0xef85c63, 17 },
    { 0x25a85c64, 55 },
    { 0xcb685c65, 90 },
    { 0xb8b85c65, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca85c6f, 43 },
    { 0xd7785c6f, 61 },
    { 0, 0 },
    { 0x62b85c72, 52 },
    { 0x6b685c73, 90 },
    { 0x34985c74, 76 },
    { 0xf3785c73, 98 },
    { 0xe1c85c76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe85c7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c85c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0885c8a, 33 },
    { 0xc1985c8a, 93 },
    { 0, 0 },
    { 0x98685c8d, 77 },
    { 0x4e485c8e, 95 },
    { 0x2b585c8d, 90 },
    { 0x15b85c90, 33 },
    { 0, 0 },
    { 0xf8c85c92, 43 },
    { 0xcb085c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a85c96, 95 },
    { 0x16085c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8685c9a, 98 },
    { 0xbf085c9a, 76 },
    { 0x9c985c9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a85ca7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41385cad, 95 },
    { 0x74f85cad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec285cb1, 39 },
    { 0x61b85cb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb785cb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b85cb9, 33 },
    { 0xeb685cba, 46 },
    { 0xda185cb9, 99 },
    { 0x91085cbc, 99 },
    { 0x9d385cbc, 23 },
    { 0, 0 },
    { 0x89d85cbf, 23 },
    { 0, 0 },
    { 0xb785cc1, 95 },
    { 0xd4e85cc2, 23 },
    { 0x36985cc2, 55 },
    { 0, 0 },
    { 0x5db85cc5, 90 },
    { 0x5c585cc6, 76 },
    { 0xc5b85cc5, 34 },
    { 0x11b85cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e385ccb, 93 },
    { 0x90685ccc, 55 },
    { 0x1bb85ccd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7785cd3, 76 },
    { 0x27f85cd4, 90 },
    { 0xc6185cd3, 55 },
    { 0xdb185cd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b85cda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac85cdf, 98 },
    { 0x7f85ce0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa385ce3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c785ce7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe785ceb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1985cee, 61 },
    { 0, 0 },
    { 0xb4885cf0, 81 },
    { 0xbfb85cf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b85cf9, 23 },
    { 0x50a85cfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84985cff, 61 },
    { 0, 0 },
    { 0x21085d01, 46 },
    { 0, 0 },
    { 0x14c85d03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c85d06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74085d09, 55 },
    { 0x1fe85d0a, 84 },
    { 0xee585d0b, 52 },
    { 0x41385d0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c85d12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac85d16, 34 },
    { 0, 0 },
    { 0x26b85d18, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf85d22, 93 },
    { 0, 0 },
    { 0xcd785d24, 98 },
    { 0xdc085d25, 23 },
    { 0xda585d26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a85d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a585d3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f85d41, 95 },
    { 0, 0 },
    { 0x1285d43, 61 },
    { 0, 0 },
    { 0xd1185d45, 43 },
    { 0, 0 },
    { 0xf5985d47, 61 },
    { 0xfe585d48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16385d4c, 61 },
    { 0x18085d4d, 83 },
    { 0x6c285d4e, 33 },
    { 0x8685d4f, 34 },
    { 0xac185d50, 23 },
    { 0x3dd85d51, 66 },
    { 0xa5885d51, 61 },
    { 0x7d185d53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4985d5e, 43 },
    { 0, 0 },
    { 0xf4385d60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b585d6c, 95 },
    { 0x5cf85d6d, 90 },
    { 0x73b85d6d, 23 },
    { 0x64c85d6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0685d75, 98 },
    { 0xbf085d76, 55 },
    { 0xefc85d77, 66 },
    { 0x8de85d76, 46 },
    { 0xd7785d78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6385d7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73785d83, 33 },
    { 0, 0 },
    { 0x6685d85, 90 },
    { 0xf8b85d86, 84 },
    { 0x4bc85d87, 33 },
    { 0x11785d88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff85d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8285d8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe85d94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb85d97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd385d9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb85d9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c85da3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49985db3, 68 },
    { 0, 0 },
    { 0x19c85db5, 90 },
    { 0, 0 },
    { 0x68c85db7, 67 },
    { 0, 0 },
    { 0x19f85db9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1085dbd, 85 },
    { 0xa1b85dbd, 61 },
    { 0x2d785dbf, 46 },
    { 0x8285dbf, 98 },
    { 0x11185dbe, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a85dc8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x64485dcb, 61 },
    { 0xafe85dcc, 93 },
    { 0x75785dcc, 61 },
    { 0xb3585dce, 90 },
    { 0x9485dce, 90 },
    { 0x10285dcb, 98 },
    { 0xce85dce, 99 },
    { 0, 0 },
    { 0xdd185dd3, 23 },
    { 0, 0 },
    { 0x78a85dd5, 76 },
    { 0x3f585dd5, 95 },
    { 0x96185dd7, 33 },
    { 0x66b85dd6, 98 },
    { 0x14285dd9, 34 },
    { 0xa0f85dd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd685de2, 55 },
    { 0xea385de2, 61 },
    { 0xc5685de3, 95 },
    { 0xb3185de4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x28585de8, 23 },
    { 0, 0 },
    { 0xf0c85dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51985ded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51985df3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x13585df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d785dfc, 93 },
    { 0x55485dfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45285e01, 98 },
    { 0x20c85e01, 95 },
    { 0xfc085e03, 93 },
    { 0xf9985e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b85e07, 85 },
    { 0, 0 },
    { 0x7ce85e09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd85e0f, 90 },
    { 0xef685e0f, 90 },
    { 0x1c985e11, 33 },
    { 0, 0 },
    { 0x4c685e13, 90 },
    { 0xc9285e14, 61 },
    { 0x94485e14, 91 },
    { 0x80a85e16, 98 },
    { 0x38a85e14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5785e1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7785e20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9785e23, 46 },
    { 0x41b85e24, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f85e2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe785e2f, 95 },
    { 0xed385e30, 90 },
    { 0xd0685e31, 61 },
    { 0xf5285e32, 90 },
    { 0xd5b85e32, 77 },
    { 0, 0 },
    { 0xe8a85e35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86885e38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2785e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a85e42, 68 },
    { 0xe0f85e43, 33 },
    { 0x6fc85e42, 55 },
    { 0x29485e42, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f485e4c, 77 },
    { 0, 0 },
    { 0xd9485e4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85885e51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce85e55, 33 },
    { 0x72185e55, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82785e5d, 98 },
    { 0xdb785e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac85e66, 88 },
    { 0x7aa85e67, 98 },
    { 0, 0 },
    { 0x43c85e69, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c585e6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea085e6f, 33 },
    { 0, 0 },
    { 0x86285e71, 33 },
    { 0x9685e72, 90 },
    { 0xf7f85e72, 34 },
    { 0xa585e74, 98 },
    { 0x31b85e75, 34 },
    { 0x5685e74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa85e7a, 61 },
    { 0xc6085e7b, 67 },
    { 0x9a185e7c, 23 },
    { 0x6ff85e7b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b985e80, 95 },
    { 0, 0 },
    { 0x5c485e82, 90 },
    { 0, 0 },
    { 0x1ef85e84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67085e92, 33 },
    { 0, 0 },
    { 0x55a85e94, 34 },
    { 0xcdb85e94, 95 },
    { 0xded85e95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf85e9a, 87 },
    { 0xfdd85e9b, 33 },
    { 0x30d85e9a, 33 },
    { 0xbaa85e9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a85ea4, 61 },
    { 0x96085ea5, 100 },
    { 0x60585ea6, 78 },
    { 0xab485ea5, 46 },
    { 0x14785ea6, 55 },
    { 0x2bd85ea6, 33 },
    { 0, 0 },
    { 0x7e585eab, 95 },
    { 0x9da85eac, 65 },
    { 0x24885eab, 52 },
    { 0xc5485eac, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b85eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b85eb6, 83 },
    { 0xb3085eb6, 33 },
    { 0, 0 },
    { 0x84a85eb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6285ebc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1685ebf, 23 },
    { 0, 0 },
    { 0x68185ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a085ec5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b385eca, 85 },
    { 0xd2b85eca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99585ece, 52 },
    { 0, 0 },
    { 0x6c685ed0, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33385ed6, 95 },
    { 0xd9885ed7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96085eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd85ede, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xae385ee1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f85ee5, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a85ee9, 33 },
    { 0x87485eea, 23 },
    { 0x99685eeb, 100 },
    { 0x7d985eec, 90 },
    { 0x76d85eec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa85efa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80085f03, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x13585f06, 87 },
    { 0xdac85f07, 34 },
    { 0, 0 },
    { 0xaa585f09, 93 },
    { 0x88485f0a, 33 },
    { 0, 0 },
    { 0xef585f0c, 33 },
    { 0, 0 },
    { 0x5ae85f0e, 23 },
    { 0, 0 },
    { 0xfb85f10, 43 },
    { 0xa5985f11, 23 },
    { 0x7d585f12, 100 },
    { 0x20485f12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0185f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6985f19, 61 },
    { 0x26785f1a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb485f23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5885f2a, 55 },
    { 0, 0 },
    { 0xc9585f2c, 55 },
    { 0x87d85f2c, 85 },
    { 0xfd085f2c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65485f33, 80 },
    { 0x53985f34, 46 },
    { 0xc2c85f34, 83 },
    { 0xd5185f36, 43 },
    { 0x18d85f37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c85f3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b085f3d, 47 },
    { 0, 0 },
    { 0xece85f3f, 100 },
    { 0x4d185f40, 90 },
    { 0, 0 },
    { 0x87685f42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d585f45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f85f48, 77 },
    { 0xc5385f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded85f4d, 98 },
    { 0x2d85f4e, 78 },
    { 0x59e85f4d, 17 },
    { 0x59985f4f, 75 },
    { 0x23a85f51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d785f59, 90 },
    { 0, 0 },
    { 0x8de85f5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9685f5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44485f62, 78 },
    { 0, 0 },
    { 0x84885f64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53785f68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18385f6b, 95 },
    { 0x1f885f6b, 76 },
    { 0xc9e85f6d, 88 },
    { 0xc7585f6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f85f71, 77 },
    { 0x55e85f71, 66 },
    { 0, 0 },
    { 0x53785f74, 98 },
    { 0, 0 },
    { 0xebe85f76, 93 },
    { 0, 0 },
    { 0xc1285f78, 77 },
    { 0x56985f78, 76 },
    { 0x51b85f7a, 55 },
    { 0x70a85f7b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8785f7e, 61 },
    { 0xd2385f7e, 52 },
    { 0, 0 },
    { 0x5aa85f81, 61 },
    { 0x4bd85f82, 46 },
    { 0, 0 },
    { 0x93585f84, 46 },
    { 0, 0 },
    { 0xa9f85f86, 40 },
    { 0x97c85f87, 23 },
    { 0, 0 },
    { 0x2285f89, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a185f8e, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d085f91, 46 },
    { 0x3a85f92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de85f96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x82085f99, 90 },
    { 0x30785f99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13785f9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c085fa5, 33 },
    { 0, 0 },
    { 0x9685fa7, 46 },
    { 0x4c785fa8, 33 },
    { 0x22e85fa7, 33 },
    { 0x41885fa7, 56 },
    { 0x97b85fa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf85fae, 95 },
    { 0x64d85faf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82685fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d185fc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f85fc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c85fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71785fd3, 95 },
    { 0, 0 },
    { 0xe0785fd5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb85fd8, 43 },
    { 0x36085fd9, 65 },
    { 0x5a385fda, 33 },
    { 0x2cd85fda, 55 },
    { 0x82485fdc, 23 },
    { 0x1d485fda, 90 },
    { 0xd1385fde, 33 },
    { 0x38c85fde, 90 },
    { 0xecf85fe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b985fe3, 90 },
    { 0, 0 },
    { 0xcf085fe5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d85feb, 80 },
    { 0, 0 },
    { 0x57785fed, 87 },
    { 0x13185fed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e85ff2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a85ff9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x76585ffc, 91 },
    { 0, 0 },
    { 0x29485ffe, 93 },
    { 0xe3085fff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a86009, 99 },
    { 0xe6e86009, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c8600e, 23 },
    { 0xc798600f, 34 },
    { 0, 0 },
    { 0x22186011, 98 },
    { 0, 0 },
    { 0xb3486013, 33 },
    { 0, 0 },
    { 0xa7f86015, 19 },
    { 0xfd986016, 90 },
    { 0xc1886017, 95 },
    { 0x40f86018, 98 },
    { 0x9ad86018, 61 },
    { 0x7fe8601a, 90 },
    { 0x7f686018, 77 },
    { 0x1a986019, 46 },
    { 0xe5c8601d, 43 },
    { 0x9b18601a, 95 },
    { 0x64e8601f, 55 },
    { 0x9658601f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c86023, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f86026, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9358602a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe86030, 46 },
    { 0x9a786030, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9986034, 90 },
    { 0x25886035, 61 },
    { 0xbf186036, 78 },
    { 0xf3686035, 83 },
    { 0, 0 },
    { 0xba286039, 90 },
    { 0x95086039, 95 },
    { 0, 0 },
    { 0x76e8603c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb086047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf8604c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45186050, 90 },
    { 0, 0 },
    { 0x91286052, 66 },
    { 0, 0 },
    { 0xe2486054, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf586057, 76 },
    { 0xecd86057, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8605f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84286064, 55 },
    { 0x5cb86065, 33 },
    { 0xfdc86066, 23 },
    { 0x8cf86067, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a28606b, 43 },
    { 0xafe8606c, 33 },
    { 0x7a88606c, 61 },
    { 0xce08606e, 66 },
    { 0x40e8606d, 66 },
    { 0x7f186070, 55 },
    { 0x1d886071, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f86079, 100 },
    { 0xbe88607a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9786081, 52 },
    { 0xd1c86082, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52986086, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xad886089, 76 },
    { 0x8258608a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac8608e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c86091, 75 },
    { 0x29986091, 23 },
    { 0, 0 },
    { 0x97b86094, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b98609a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6868609d, 95 },
    { 0x4158609d, 61 },
    { 0xc748609f, 98 },
    { 0xb518609d, 90 },
    { 0x83860a1, 81 },
    { 0x9a860a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a860a5, 52 },
    { 0xbc0860a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x378860a9, 84 },
    { 0x7a860aa, 36 },
    { 0xb0c860ab, 81 },
    { 0x49f860ab, 95 },
    { 0x9c3860aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7860b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3860b7, 55 },
    { 0x192860b8, 68 },
    { 0, 0 },
    { 0x5ac860ba, 82 },
    { 0xdbd860bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91860c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f860c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc860ca, 100 },
    { 0xfb4860cb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3860cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4860d4, 76 },
    { 0x379860d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e860d9, 67 },
    { 0xf67860da, 76 },
    { 0x2c4860db, 97 },
    { 0xfb5860dc, 34 },
    { 0, 0 },
    { 0x47a860de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3860e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62860e7, 93 },
    { 0x669860e8, 100 },
    { 0, 0 },
    { 0x6ec860ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df860ed, 55 },
    { 0x891860ee, 90 },
    { 0x152860ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00860f7, 67 },
    { 0, 0 },
    { 0x346860f9, 43 },
    { 0, 0 },
    { 0x58860fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b28610c, 99 },
    { 0, 0 },
    { 0x9228610e, 99 },
    { 0, 0 },
    { 0x75f86110, 34 },
    { 0xa9d86111, 95 },
    { 0xa7286112, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff86118, 77 },
    { 0, 0 },
    { 0xdbe8611a, 65 },
    { 0, 0 },
    { 0xc7e8611c, 95 },
    { 0, 0 },
    { 0x2798611e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb486125, 90 },
    { 0x9d386126, 90 },
    { 0, 0 },
    { 0x9a586128, 23 },
    { 0x67786129, 85 },
    { 0, 0 },
    { 0xbb98612b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d886134, 95 },
    { 0, 0 },
    { 0x53186136, 61 },
    { 0x88386136, 93 },
    { 0x8fc86138, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb486140, 23 },
    { 0xbb586141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a86144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb086147, 90 },
    { 0, 0 },
    { 0xe0086149, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9718614d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4286159, 66 },
    { 0xb3686159, 34 },
    { 0x7fa8615b, 43 },
    { 0xa68615a, 23 },
    { 0x4b28615c, 95 },
    { 0x3698615e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5886167, 81 },
    { 0xe6d86168, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2408616d, 55 },
    { 0x7698616e, 61 },
    { 0, 0 },
    { 0x2986170, 95 },
    { 0, 0 },
    { 0x1b186172, 90 },
    { 0x29486172, 94 },
    { 0x7c786174, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26386179, 90 },
    { 0x23686179, 33 },
    { 0xc248617b, 33 },
    { 0, 0 },
    { 0x5a28617d, 91 },
    { 0xba8617e, 33 },
    { 0x62f8617e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f86183, 98 },
    { 0x46886183, 46 },
    { 0x2a986184, 61 },
    { 0x87f86186, 76 },
    { 0, 0 },
    { 0x70086188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de8618b, 33 },
    { 0, 0 },
    { 0xaa18618d, 90 },
    { 0xef98618e, 84 },
    { 0x3348618e, 76 },
    { 0xab086190, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0086198, 33 },
    { 0x51586199, 23 },
    { 0xb7086199, 66 },
    { 0x4218619b, 46 },
    { 0xf818619a, 55 },
    { 0x3d78619a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1861a1, 88 },
    { 0xf56861a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63861a7, 98 },
    { 0x7dc861a8, 61 },
    { 0x1d4861a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da861ad, 95 },
    { 0x34c861ad, 33 },
    { 0x15861af, 90 },
    { 0x872861b0, 61 },
    { 0, 0 },
    { 0xcfc861b2, 95 },
    { 0, 0 },
    { 0x13c861b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331861b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512861be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8861c2, 93 },
    { 0, 0 },
    { 0x779861c4, 23 },
    { 0x76861c4, 98 },
    { 0, 0 },
    { 0xd4a861c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7861ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4861cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb861d3, 61 },
    { 0, 0 },
    { 0x4fc861d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x306861d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a861dc, 100 },
    { 0, 0 },
    { 0xc3b861de, 61 },
    { 0, 0 },
    { 0xfe1861e0, 61 },
    { 0xf8b861e1, 61 },
    { 0, 0 },
    { 0x8b6861e3, 61 },
    { 0xe7c861e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0861e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x75861ea, 95 },
    { 0, 0 },
    { 0x16b861ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a861f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267861f4, 76 },
    { 0, 0 },
    { 0x789861f6, 93 },
    { 0, 0 },
    { 0x505861f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48861fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27586200, 23 },
    { 0, 0 },
    { 0x6e86202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a86205, 36 },
    { 0x62186206, 61 },
    { 0, 0 },
    { 0x85386208, 23 },
    { 0x68286209, 98 },
    { 0xff58620a, 55 },
    { 0, 0 },
    { 0xfb18620c, 61 },
    { 0x858620d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f286211, 98 },
    { 0x4c386211, 90 },
    { 0x1686213, 95 },
    { 0xdf186213, 61 },
    { 0xce886215, 93 },
    { 0xa9786216, 33 },
    { 0x51a86216, 61 },
    { 0x35d86211, 95 },
    { 0xa486219, 33 },
    { 0xf9e8621a, 52 },
    { 0x20686219, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de86220, 33 },
    { 0xcc086221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda186224, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9886228, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1748622c, 33 },
    { 0x8c48622d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c86230, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x16386233, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386237, 33 },
    { 0x13e86238, 66 },
    { 0, 0 },
    { 0x61d8623a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf018623e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65886241, 90 },
    { 0xcb686242, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8624a, 23 },
    { 0x5fb8624b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c88624e, 66 },
    { 0, 0 },
    { 0xfd886250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb286254, 90 },
    { 0, 0 },
    { 0x93b86256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98625a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd398625d, 77 },
    { 0x6078625e, 87 },
    { 0x36b8625d, 90 },
    { 0xfdd86260, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c86263, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee786267, 95 },
    { 0, 0 },
    { 0x41586269, 34 },
    { 0x7a786269, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c8626d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf786277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8627b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb886280, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4486285, 61 },
    { 0, 0 },
    { 0x49d86287, 81 },
    { 0xe2786288, 90 },
    { 0xe2686289, 90 },
    { 0xdd686288, 61 },
    { 0x5a86289, 34 },
    { 0xaf38628c, 61 },
    { 0x17c86289, 23 },
    { 0x4eb8628e, 99 },
    { 0x75a8628e, 33 },
    { 0x5486290, 98 },
    { 0x74786288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5886296, 61 },
    { 0xf1f86297, 84 },
    { 0xfdc86298, 33 },
    { 0x99e86299, 76 },
    { 0xe1f86299, 55 },
    { 0x4e98629b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934862a1, 42 },
    { 0x371862a2, 34 },
    { 0, 0 },
    { 0x5b4862a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c862a7, 61 },
    { 0x371862a7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8862ac, 90 },
    { 0, 0 },
    { 0x6b7862ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0862b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f862b4, 34 },
    { 0x9fd862b5, 61 },
    { 0, 0 },
    { 0xc70862b7, 61 },
    { 0, 0 },
    { 0xbae862b9, 88 },
    { 0, 0 },
    { 0x44862bb, 61 },
    { 0xab6862bb, 76 },
    { 0xe862bd, 87 },
    { 0, 0 },
    { 0x149862bf, 77 },
    { 0x198862bf, 61 },
    { 0x83d862c1, 33 },
    { 0, 0 },
    { 0xe7862c3, 95 },
    { 0x13c862c3, 84 },
    { 0xab9862c5, 61 },
    { 0xf03862c6, 90 },
    { 0, 0 },
    { 0x2af862c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c862ce, 55 },
    { 0xfbf862cf, 61 },
    { 0x3f0862cf, 91 },
    { 0x793862d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9862d5, 46 },
    { 0xfc7862d5, 23 },
    { 0xb05862d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x927862da, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13862e0, 34 },
    { 0, 0 },
    { 0x9e9862e2, 48 },
    { 0x39c862e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785862ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66862f0, 61 },
    { 0x89862f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f862f4, 90 },
    { 0, 0 },
    { 0xe41862f6, 33 },
    { 0xa83862f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x712862fa, 52 },
    { 0, 0 },
    { 0x393862fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46486300, 95 },
    { 0x44f86300, 23 },
    { 0x5df86301, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa086306, 61 },
    { 0, 0 },
    { 0xc1886308, 87 },
    { 0xa7d86308, 95 },
    { 0xd2f8630a, 61 },
    { 0x4e386308, 93 },
    { 0xe7e8630a, 98 },
    { 0, 0 },
    { 0x2308630e, 43 },
    { 0, 0 },
    { 0x19f86310, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5586313, 33 },
    { 0, 0 },
    { 0xf5086315, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd186318, 77 },
    { 0xecf86319, 95 },
    { 0x3d38631a, 33 },
    { 0xbe48631b, 81 },
    { 0, 0 },
    { 0xb648631d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8586320, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1086323, 100 },
    { 0, 0 },
    { 0xc8286325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f86328, 90 },
    { 0x27586329, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4118632e, 60 },
    { 0xef08632f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63586338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf08633c, 46 },
    { 0x48e8633d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf986341, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56386344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb86347, 87 },
    { 0, 0 },
    { 0xa7f86349, 23 },
    { 0xc0186349, 52 },
    { 0xf0186349, 95 },
    { 0x20f8634c, 100 },
    { 0, 0 },
    { 0xd978634e, 74 },
    { 0xfa98634f, 93 },
    { 0x9b486350, 23 },
    { 0xb7886351, 90 },
    { 0xe7f86351, 95 },
    { 0x43d86352, 34 },
    { 0, 0 },
    { 0x4f986355, 55 },
    { 0x53d86356, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79186359, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8548635e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b86362, 90 },
    { 0xf3a86363, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd86367, 33 },
    { 0, 0 },
    { 0x59886369, 17 },
    { 0xfab8636a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1986370, 33 },
    { 0x19686371, 33 },
    { 0xb6686372, 95 },
    { 0, 0 },
    { 0xfe886374, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9068637a, 95 },
    { 0x8d68637b, 95 },
    { 0x2768637b, 76 },
    { 0, 0 },
    { 0x6ec8637e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x79186381, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4286385, 52 },
    { 0x92486385, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10486389, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf168638c, 23 },
    { 0, 0 },
    { 0xa298638e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34586392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae86395, 34 },
    { 0, 0 },
    { 0xf5586397, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d8639a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e88639f, 23 },
    { 0xc21863a0, 85 },
    { 0x6f7863a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2863a4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8863aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73863ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f863b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488863b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309863be, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09863c8, 98 },
    { 0x4db863c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0863cc, 61 },
    { 0, 0 },
    { 0x2cc863ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1863d8, 91 },
    { 0x304863d8, 91 },
    { 0xd34863da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08863e2, 95 },
    { 0xe6a863e3, 46 },
    { 0xdfe863e4, 61 },
    { 0xbfc863e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d863e8, 33 },
    { 0xb1f863e9, 81 },
    { 0x57e863e9, 33 },
    { 0, 0 },
    { 0x4d863ec, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60863ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1863f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6863fd, 43 },
    { 0, 0 },
    { 0xce6863ff, 61 },
    { 0xe3286400, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16086405, 90 },
    { 0, 0 },
    { 0xd8786407, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d86416, 90 },
    { 0xeb286417, 90 },
    { 0x16a86417, 76 },
    { 0xb0b86419, 98 },
    { 0x20c86417, 61 },
    { 0, 0 },
    { 0x8918641c, 95 },
    { 0x178641c, 98 },
    { 0xdd48641d, 76 },
    { 0x67d8641c, 95 },
    { 0x86f8641c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3078642e, 61 },
    { 0, 0 },
    { 0x37486430, 55 },
    { 0x95f86431, 40 },
    { 0xdb586432, 95 },
    { 0x1db86433, 33 },
    { 0, 0 },
    { 0xfbd86435, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b786439, 23 },
    { 0x1b88643a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c8643d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1086447, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd938644d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f586453, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb786456, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f8645b, 55 },
    { 0x28645c, 61 },
    { 0, 0 },
    { 0x8d28645e, 98 },
    { 0x1a88645f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35486462, 90 },
    { 0, 0 },
    { 0x30586464, 33 },
    { 0, 0 },
    { 0x59a86466, 95 },
    { 0xf8b86467, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf8646a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e8646d, 23 },
    { 0, 0 },
    { 0xe7c8646f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93086483, 33 },
    { 0x5ba86484, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11486487, 61 },
    { 0x8886487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b8648c, 50 },
    { 0, 0 },
    { 0xc188648e, 88 },
    { 0x958648f, 33 },
    { 0xfb086490, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9486495, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa328649b, 84 },
    { 0xfef8649b, 61 },
    { 0, 0 },
    { 0x6fb8649e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485864a9, 96 },
    { 0xe1864aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf864ad, 99 },
    { 0x2e6864ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b864b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9864bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d864c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef864c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a864d0, 61 },
    { 0x98a864d0, 90 },
    { 0, 0 },
    { 0x397864d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846864da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0864de, 88 },
    { 0, 0 },
    { 0xc27864e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699864e4, 34 },
    { 0x5d864e5, 91 },
    { 0x578864e5, 95 },
    { 0x72f864e7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c864ec, 46 },
    { 0x104864ed, 46 },
    { 0x62c864ee, 33 },
    { 0x840864ed, 38 },
    { 0x8d7864f0, 23 },
    { 0x1a3864f0, 61 },
    { 0xda9864f1, 33 },
    { 0x7c0864f2, 90 },
    { 0xb2b864f2, 33 },
    { 0xdb8864f5, 98 },
    { 0xe15864f5, 23 },
    { 0, 0 },
    { 0x5a5864f8, 84 },
    { 0x7e7864f8, 33 },
    { 0xfc4864f9, 75 },
    { 0xe99864fb, 33 },
    { 0x2c8864fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede86500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59886506, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa308650a, 87 },
    { 0x9508650b, 90 },
    { 0xd818650c, 61 },
    { 0x2c8650b, 65 },
    { 0xe4f8650e, 33 },
    { 0xd0d8650f, 34 },
    { 0xd3386510, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f8651a, 23 },
    { 0xeb08651b, 95 },
    { 0x7008651b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8651f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42586523, 98 },
    { 0xc6b86523, 90 },
    { 0x97886525, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d86529, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb86530, 87 },
    { 0xbdf86531, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e86536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8653f, 33 },
    { 0xb8f8653f, 46 },
    { 0x5ac86541, 33 },
    { 0x6d686542, 43 },
    { 0xfe686542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c86547, 66 },
    { 0, 0 },
    { 0xbc586549, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e8654d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab286554, 99 },
    { 0xdd486555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c38655b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8655e, 90 },
    { 0xb808655f, 76 },
    { 0xe4986560, 23 },
    { 0, 0 },
    { 0x2c286562, 59 },
    { 0x9dd86562, 76 },
    { 0xe3286564, 99 },
    { 0x20986565, 33 },
    { 0, 0 },
    { 0x96b86567, 95 },
    { 0, 0 },
    { 0xf9586569, 90 },
    { 0xd18656a, 88 },
    { 0x3908656a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed586570, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd486573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a686579, 33 },
    { 0xe508657a, 67 },
    { 0, 0 },
    { 0x5d78657c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d8657f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad886583, 33 },
    { 0xb6386583, 87 },
    { 0, 0 },
    { 0x18986586, 66 },
    { 0x2f786587, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e58658b, 95 },
    { 0, 0 },
    { 0xb98658d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f486592, 98 },
    { 0x7f586593, 38 },
    { 0, 0 },
    { 0x52486595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c86598, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f8659b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d865a0, 98 },
    { 0, 0 },
    { 0xebe865a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442865a7, 90 },
    { 0xbfe865a7, 99 },
    { 0x109865a9, 95 },
    { 0, 0 },
    { 0xd17865ab, 87 },
    { 0x9af865ac, 90 },
    { 0, 0 },
    { 0xc38865ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50865b2, 90 },
    { 0xd2e865b3, 90 },
    { 0x7865b4, 61 },
    { 0x516865b3, 95 },
    { 0x9ec865b3, 95 },
    { 0x89b865b7, 43 },
    { 0x98f865b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1865bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x576865c0, 61 },
    { 0x21a865c0, 67 },
    { 0, 0 },
    { 0x71e865c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905865c8, 55 },
    { 0, 0 },
    { 0x78c865ca, 90 },
    { 0x285865cb, 95 },
    { 0xe19865cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71865d0, 52 },
    { 0x92d865d0, 23 },
    { 0, 0 },
    { 0x685865d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e865d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53865d9, 85 },
    { 0x317865da, 33 },
    { 0x44a865db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71865de, 90 },
    { 0x17d865df, 61 },
    { 0, 0 },
    { 0xd7d865e1, 77 },
    { 0x886865e1, 95 },
    { 0, 0 },
    { 0x273865e4, 100 },
    { 0, 0 },
    { 0xbc7865e6, 68 },
    { 0xc05865e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8865ed, 90 },
    { 0x8e9865ee, 23 },
    { 0, 0 },
    { 0xbb4865f0, 90 },
    { 0xb9865f1, 61 },
    { 0x8e865f2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb865f5, 33 },
    { 0x11c865f6, 61 },
    { 0, 0 },
    { 0x7f6865f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x101865fb, 33 },
    { 0, 0 },
    { 0x2d5865fd, 61 },
    { 0x63f865fe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x41186601, 90 },
    { 0x7e986602, 33 },
    { 0x51786603, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x47786606, 90 },
    { 0xdd486607, 90 },
    { 0x96d86608, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e38660b, 87 },
    { 0x47a8660c, 84 },
    { 0x18e8660d, 43 },
    { 0xbee8660c, 12 },
    { 0xa078660f, 85 },
    { 0, 0 },
    { 0xd7986611, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d86614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2948661c, 61 },
    { 0x60f8661c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf086622, 95 },
    { 0x48f86623, 55 },
    { 0x88286623, 37 },
    { 0xcc686624, 34 },
    { 0x65d86626, 33 },
    { 0xe6d86627, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f38662b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xab88662e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc786633, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba38663c, 34 },
    { 0x408663d, 61 },
    { 0x2318663e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47586644, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24086647, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18664c, 76 },
    { 0x7318664c, 76 },
    { 0, 0 },
    { 0x8598664f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d86652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5886656, 61 },
    { 0, 0 },
    { 0x60686658, 33 },
    { 0, 0 },
    { 0xf188665a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8665e, 83 },
    { 0xef48665f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd886663, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f86667, 100 },
    { 0, 0 },
    { 0x7cf86669, 33 },
    { 0xc7086669, 33 },
    { 0x8208666b, 66 },
    { 0x17f8666b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e8666f, 90 },
    { 0xa3386670, 88 },
    { 0xc3f86670, 61 },
    { 0xdc686672, 33 },
    { 0xdf886670, 61 },
    { 0, 0 },
    { 0x4ca86675, 61 },
    { 0, 0 },
    { 0xa1586677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a18667c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a86688, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xf548668b, 65 },
    { 0x5bf8668c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78668f, 78 },
    { 0x73286690, 78 },
    { 0, 0 },
    { 0xd286692, 93 },
    { 0x1e86693, 23 },
    { 0x35186694, 98 },
    { 0xc186695, 33 },
    { 0xbe986693, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb8669a, 65 },
    { 0x8bd8669b, 88 },
    { 0xe4c8669c, 95 },
    { 0xaa88669a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd866a0, 95 },
    { 0xde8866a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14866a8, 95 },
    { 0xf84866a8, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8866ac, 91 },
    { 0, 0 },
    { 0xc1866ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68866b2, 90 },
    { 0, 0 },
    { 0x6b5866b4, 33 },
    { 0x519866b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8866b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x281866bb, 55 },
    { 0x3c0866bc, 68 },
    { 0x1ca866bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e866c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf866c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91866ca, 98 },
    { 0, 0 },
    { 0xdca866cc, 95 },
    { 0xe94866cc, 90 },
    { 0xc23866ce, 52 },
    { 0xa32866ce, 84 },
    { 0xaa2866d0, 61 },
    { 0xb26866cf, 93 },
    { 0x721866cf, 95 },
    { 0x3ce866d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa866d7, 33 },
    { 0x1ba866d8, 95 },
    { 0xa36866d8, 46 },
    { 0, 0 },
    { 0x421866db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d866df, 34 },
    { 0x59a866e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f866e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1866e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c866eb, 100 },
    { 0, 0 },
    { 0x8aa866ed, 94 },
    { 0, 0 },
    { 0xf5866ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f866f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e866f7, 90 },
    { 0x348866f7, 43 },
    { 0x380866f9, 77 },
    { 0x67c866fa, 23 },
    { 0xee8866fa, 33 },
    { 0x2ae866f9, 67 },
    { 0, 0 },
    { 0xe84866fe, 23 },
    { 0xf1d866fe, 95 },
    { 0, 0 },
    { 0xe4086701, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc486705, 95 },
    { 0x91386706, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac86709, 23 },
    { 0xc578670a, 90 },
    { 0x1b68670b, 90 },
    { 0, 0 },
    { 0x5d18670d, 67 },
    { 0x1828670e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x99286711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98286715, 46 },
    { 0, 0 },
    { 0x21c86717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d786722, 23 },
    { 0x38286722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31386727, 33 },
    { 0xe8f86728, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b78672c, 93 },
    { 0x8d48672c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe886735, 17 },
    { 0xa4786736, 76 },
    { 0x7c686737, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158673e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b786742, 76 },
    { 0, 0 },
    { 0x48b86744, 90 },
    { 0x97086745, 87 },
    { 0x43386744, 47 },
    { 0x1e386746, 53 },
    { 0xce886748, 55 },
    { 0xa3586749, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a8674c, 46 },
    { 0x44b8674c, 90 },
    { 0xa118674c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d386751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b986754, 90 },
    { 0, 0 },
    { 0xf9c86756, 75 },
    { 0x15886757, 90 },
    { 0xd886758, 56 },
    { 0x9e186759, 23 },
    { 0xdb18675a, 98 },
    { 0x5886757, 77 },
    { 0, 0 },
    { 0xabb8675d, 98 },
    { 0x2a8675d, 46 },
    { 0, 0 },
    { 0x4af86760, 84 },
    { 0xfa586760, 95 },
    { 0x72086762, 90 },
    { 0x80086761, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7086768, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd8676b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce86771, 85 },
    { 0xad086772, 66 },
    { 0xcd286772, 90 },
    { 0xb8086773, 90 },
    { 0, 0 },
    { 0xf3386776, 76 },
    { 0, 0 },
    { 0x6cb86778, 76 },
    { 0xa8c86778, 81 },
    { 0xa7c86778, 33 },
    { 0x71d8677b, 99 },
    { 0, 0 },
    { 0x17d8677d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac86780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd686787, 66 },
    { 0xf1986788, 91 },
    { 0xbea86787, 100 },
    { 0xa048678a, 90 },
    { 0x98c8678b, 33 },
    { 0, 0 },
    { 0x7d18678d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x10486790, 90 },
    { 0xb9186791, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d86798, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8398679b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d867a1, 34 },
    { 0, 0 },
    { 0x7d867a3, 76 },
    { 0x9a4867a4, 95 },
    { 0xbf8867a5, 33 },
    { 0x179867a6, 91 },
    { 0x24f867a6, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e867b1, 95 },
    { 0xa6a867b2, 76 },
    { 0xcdc867b1, 61 },
    { 0x706867b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809867b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c867be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b867c4, 66 },
    { 0x8ad867c5, 28 },
    { 0x3e3867c5, 61 },
    { 0x4b9867c7, 95 },
    { 0x3ba867c4, 48 },
    { 0, 0 },
    { 0x412867ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1867d6, 52 },
    { 0xf40867d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db867da, 43 },
    { 0x30b867da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c867de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a867eb, 98 },
    { 0, 0 },
    { 0x735867ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3867fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c86801, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1286804, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c086808, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48680f, 95 },
    { 0x23f8680f, 55 },
    { 0xc2786811, 23 },
    { 0xa5886812, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa786816, 87 },
    { 0xb6f86817, 78 },
    { 0, 0 },
    { 0xc086819, 61 },
    { 0, 0 },
    { 0x47b8681b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7268681e, 34 },
    { 0xc698681e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79886822, 33 },
    { 0, 0 },
    { 0xe2886824, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59786828, 61 },
    { 0, 0 },
    { 0x5e18682a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c686831, 61 },
    { 0xe286831, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20786838, 23 },
    { 0, 0 },
    { 0xf2b8683a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5648683d, 90 },
    { 0xd378683d, 23 },
    { 0x4e58683f, 61 },
    { 0xf2f86840, 98 },
    { 0x8b986841, 77 },
    { 0x12086842, 76 },
    { 0xb586842, 100 },
    { 0x50586840, 96 },
    { 0, 0 },
    { 0xd5f86846, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c86849, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46586850, 98 },
    { 0x19d86851, 90 },
    { 0xc8a86852, 61 },
    { 0xc5486852, 33 },
    { 0x65786851, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42386858, 80 },
    { 0xd4c86859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5038685c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9986864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e8686c, 67 },
    { 0, 0 },
    { 0x6ad8686e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5086872, 46 },
    { 0x6f286872, 46 },
    { 0xdc686874, 78 },
    { 0x10786874, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d86878, 95 },
    { 0x46f86879, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8687d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d86881, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe786886, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f86889, 93 },
    { 0x5798688a, 98 },
    { 0x6a08688b, 61 },
    { 0xbaf8688a, 61 },
    { 0x81b86889, 90 },
    { 0, 0 },
    { 0x938688f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa486892, 75 },
    { 0x81986893, 33 },
    { 0x75886894, 61 },
    { 0x15986894, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae86899, 53 },
    { 0, 0 },
    { 0x2b28689b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329868a1, 33 },
    { 0, 0 },
    { 0x437868a3, 68 },
    { 0x30b868a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc868a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87868ad, 98 },
    { 0xbd4868ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a868b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab868b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44868bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c868c4, 98 },
    { 0xe77868c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b868d1, 33 },
    { 0x63c868d2, 95 },
    { 0xfd6868d1, 46 },
    { 0, 0 },
    { 0x438868d5, 90 },
    { 0xc30868d5, 33 },
    { 0, 0 },
    { 0xbb9868d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc868db, 95 },
    { 0x522868db, 23 },
    { 0x663868dd, 78 },
    { 0xefe868dd, 90 },
    { 0xb868dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x372868e2, 96 },
    { 0xfbb868e3, 33 },
    { 0, 0 },
    { 0xca4868e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b868e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04868ec, 90 },
    { 0x9de868ec, 61 },
    { 0x45b868ee, 61 },
    { 0, 0 },
    { 0xbff868f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3868f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e868f6, 76 },
    { 0x167868f7, 36 },
    { 0x24c868f8, 100 },
    { 0x523868f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x871868fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51686900, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab986905, 76 },
    { 0xac486905, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc086909, 95 },
    { 0x1dd8690a, 17 },
    { 0xd348690a, 90 },
    { 0xd5f86909, 78 },
    { 0x5e8690d, 90 },
    { 0xd288690e, 95 },
    { 0x56e8690e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed086913, 88 },
    { 0, 0 },
    { 0xa4286915, 23 },
    { 0xa3b86916, 90 },
    { 0xca786917, 87 },
    { 0x45d86916, 77 },
    { 0x39086919, 61 },
    { 0x64286916, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x948691d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf386922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec86927, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c86931, 95 },
    { 0xb4986931, 76 },
    { 0xf5b86933, 36 },
    { 0, 0 },
    { 0x94686935, 33 },
    { 0x10e86936, 55 },
    { 0x25986935, 55 },
    { 0x74a86937, 93 },
    { 0x44686935, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa068693c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85786942, 99 },
    { 0, 0 },
    { 0x7d586944, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8286947, 33 },
    { 0xcef86947, 43 },
    { 0xb6c86948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8468694c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2686951, 98 },
    { 0x55486952, 84 },
    { 0xc9086951, 95 },
    { 0, 0 },
    { 0x70f86955, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c486959, 55 },
    { 0xef98695a, 84 },
    { 0xef08695b, 67 },
    { 0, 0 },
    { 0xbc08695d, 33 },
    { 0, 0 },
    { 0xc038695f, 52 },
    { 0xa908695f, 17 },
    { 0, 0 },
    { 0x2d486962, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52086969, 34 },
    { 0, 0 },
    { 0x73c8696b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf548696e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e286976, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x61086979, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e8697c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8786980, 93 },
    { 0x70286981, 34 },
    { 0x99d86981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c86988, 38 },
    { 0xa3686989, 68 },
    { 0x8478698a, 36 },
    { 0x2a28698b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa886996, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4486999, 90 },
    { 0x70486999, 65 },
    { 0x6c78699b, 88 },
    { 0x1dd8699c, 23 },
    { 0x40c8699a, 33 },
    { 0x4478699e, 82 },
    { 0xb9d8699c, 33 },
    { 0x2fa869a0, 34 },
    { 0xada869a0, 61 },
    { 0x6bc869a2, 98 },
    { 0x7e1869a0, 61 },
    { 0x5ce869a4, 67 },
    { 0xec8869a4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55869a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5869b3, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2869bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c869c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348869c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459869c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854869d3, 84 },
    { 0xc9a869d4, 61 },
    { 0x464869d5, 90 },
    { 0x7fa869d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6869dd, 90 },
    { 0xf1e869de, 46 },
    { 0, 0 },
    { 0xb4f869e0, 67 },
    { 0, 0 },
    { 0x7b4869e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977869e6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920869eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7869f2, 61 },
    { 0xe0f869f3, 68 },
    { 0x8f0869f4, 95 },
    { 0xab1869f5, 52 },
    { 0xdef869f4, 61 },
    { 0xa55869f5, 90 },
    { 0x748869f8, 95 },
    { 0, 0 },
    { 0xbbf869fa, 43 },
    { 0xaa9869fb, 23 },
    { 0x882869fc, 34 },
    { 0x2ba869fb, 23 },
    { 0xccf869fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b86a04, 90 },
    { 0xbba86a05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b86a09, 91 },
    { 0x30586a0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b86a0e, 91 },
    { 0x7b986a0f, 23 },
    { 0, 0 },
    { 0xec886a11, 76 },
    { 0xf1386a11, 43 },
    { 0xac386a13, 61 },
    { 0x4f586a14, 67 },
    { 0x7f986a14, 87 },
    { 0, 0 },
    { 0x64f86a17, 33 },
    { 0x3fb86a18, 74 },
    { 0, 0 },
    { 0x8fa86a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a486a1f, 23 },
    { 0x92286a20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b886a27, 66 },
    { 0, 0 },
    { 0xcc986a29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66686a2c, 68 },
    { 0x7d86a2d, 61 },
    { 0xba586a2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x24586a31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda86a36, 90 },
    { 0xf1c86a37, 98 },
    { 0x47086a37, 34 },
    { 0x5f586a38, 95 },
    { 0x3fd86a37, 98 },
    { 0x35386a3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43286a40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b86a43, 34 },
    { 0xb86a44, 98 },
    { 0, 0 },
    { 0xcd286a46, 23 },
    { 0, 0 },
    { 0x43086a48, 19 },
    { 0, 0 },
    { 0xb0086a4a, 68 },
    { 0x33286a4a, 77 },
    { 0x8ac86a4a, 76 },
    { 0x93d86a4c, 46 },
    { 0, 0 },
    { 0x2ef86a4f, 78 },
    { 0xb9886a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec86a55, 87 },
    { 0x54586a56, 95 },
    { 0xa0186a55, 90 },
    { 0x65986a58, 23 },
    { 0x13086a59, 33 },
    { 0xa8d86a57, 66 },
    { 0x2b186a5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab86a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f86a65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93886a69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f86a6f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96886a73, 90 },
    { 0xf0786a73, 55 },
    { 0, 0 },
    { 0x95f86a76, 95 },
    { 0, 0 },
    { 0x6686a78, 90 },
    { 0xc6486a79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca086a80, 34 },
    { 0xa4286a81, 61 },
    { 0x5a286a82, 61 },
    { 0x7b786a80, 95 },
    { 0x3bb86a82, 61 },
    { 0xafa86a85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d586a8f, 55 },
    { 0xc0286a90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x62286a93, 23 },
    { 0xd6186a93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68686a9a, 85 },
    { 0, 0 },
    { 0x44786a9c, 61 },
    { 0xfca86a9d, 76 },
    { 0x83d86a9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c986aa2, 90 },
    { 0, 0 },
    { 0x2dd86aa4, 90 },
    { 0, 0 },
    { 0x8d386aa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22986ab0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37786ab3, 43 },
    { 0, 0 },
    { 0x74b86ab5, 88 },
    { 0xb2086ab6, 33 },
    { 0, 0 },
    { 0xc5586ab8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4986abd, 33 },
    { 0xd5686abe, 43 },
    { 0x61186abf, 76 },
    { 0x5ed86abe, 33 },
    { 0xf2c86ac1, 84 },
    { 0xe5986ac1, 81 },
    { 0xa2286ac3, 23 },
    { 0xe3986ac4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6286ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3586ad0, 91 },
    { 0, 0 },
    { 0x5c786ad2, 23 },
    { 0, 0 },
    { 0xbd886ad4, 85 },
    { 0xc7286ad5, 61 },
    { 0, 0 },
    { 0x28586ad7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a86adb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f86ae0, 93 },
    { 0xfd86ae0, 98 },
    { 0xb7886ae2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd86ae6, 55 },
    { 0, 0 },
    { 0x9c586ae8, 46 },
    { 0x53c86ae9, 78 },
    { 0x29086aea, 34 },
    { 0x57886aeb, 61 },
    { 0xb0086aec, 61 },
    { 0x46f86aea, 99 },
    { 0x7c386aea, 61 },
    { 0x4d886aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a86af2, 66 },
    { 0, 0 },
    { 0xa1286af4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd086af7, 90 },
    { 0x7a186af8, 98 },
    { 0x8d986af9, 66 },
    { 0xc1e86afa, 98 },
    { 0x64886af9, 34 },
    { 0x9c886afa, 74 },
    { 0xa4686afd, 55 },
    { 0x63a86afd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4886b06, 90 },
    { 0xa9c86b06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b86b0f, 23 },
    { 0x74386b0f, 98 },
    { 0x45286b11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2986b14, 81 },
    { 0x49986b15, 95 },
    { 0xb6386b14, 33 },
    { 0xadc86b16, 34 },
    { 0x9bc86b18, 76 },
    { 0x2ca86b16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56786b1f, 61 },
    { 0, 0 },
    { 0x4786b21, 76 },
    { 0x9b886b22, 100 },
    { 0x55b86b23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b86b29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e786b30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4386b33, 34 },
    { 0, 0 },
    { 0xb6786b35, 76 },
    { 0x96e86b36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe286b3a, 61 },
    { 0x86986b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f786b43, 77 },
    { 0xdad86b44, 98 },
    { 0, 0 },
    { 0xd0586b46, 23 },
    { 0x1b586b47, 98 },
    { 0x90a86b48, 90 },
    { 0x49486b49, 76 },
    { 0xcd186b4a, 90 },
    { 0xcf386b4b, 33 },
    { 0x14886b4c, 98 },
    { 0xf8286b4d, 76 },
    { 0x7c686b4a, 76 },
    { 0x87486b4f, 90 },
    { 0, 0 },
    { 0x7aa86b51, 33 },
    { 0x56a86b52, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42186b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a86b5b, 98 },
    { 0xd5186b5c, 61 },
    { 0xf3786b5d, 90 },
    { 0x3b886b5b, 23 },
    { 0x35186b5e, 67 },
    { 0xa1186b60, 23 },
    { 0xbb786b60, 61 },
    { 0, 0 },
    { 0x9d886b63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b486b67, 40 },
    { 0xb8686b68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15986b6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38586b6f, 43 },
    { 0xa2686b6f, 61 },
    { 0x17386b71, 95 },
    { 0xd086b6f, 46 },
    { 0xbb386b70, 23 },
    { 0x35d86b74, 78 },
    { 0xceb86b75, 98 },
    { 0x44986b76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e586b7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4986b82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74386b87, 61 },
    { 0x77386b88, 52 },
    { 0, 0 },
    { 0xbf286b8a, 90 },
    { 0, 0 },
    { 0x6e286b8c, 88 },
    { 0x11386b8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f86b97, 90 },
    { 0, 0 },
    { 0xf9e86b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f86b9f, 93 },
    { 0x23b86b9f, 61 },
    { 0x6ab86ba0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66086ba4, 23 },
    { 0xb8486ba5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84186bad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75586bb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1386bbd, 90 },
    { 0x17a86bbe, 91 },
    { 0, 0 },
    { 0x3e286bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41086bc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd86bc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d886bcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a286bcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b886bd2, 69 },
    { 0xdd586bd3, 98 },
    { 0xe8d86bd3, 23 },
    { 0xb5a86bd3, 65 },
    { 0x5286bd3, 90 },
    { 0x6286bd6, 61 },
    { 0xe9f86bd8, 61 },
    { 0x2ed86bd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42886be4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80886bea, 67 },
    { 0x1b686beb, 67 },
    { 0xa5486bec, 76 },
    { 0x8286bed, 43 },
    { 0, 0 },
    { 0x54d86bef, 90 },
    { 0x75086bf0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8786bf4, 61 },
    { 0xf4286bf5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e186bf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72186bfd, 91 },
    { 0, 0 },
    { 0x4d286bff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf86c07, 85 },
    { 0, 0 },
    { 0x9b486c09, 81 },
    { 0xe3186c0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a86c12, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d86c17, 76 },
    { 0, 0 },
    { 0xfaa86c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa86c20, 46 },
    { 0x7e886c21, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f86c28, 90 },
    { 0, 0 },
    { 0xbbd86c2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd786c31, 95 },
    { 0, 0 },
    { 0x5b786c33, 99 },
    { 0xacc86c34, 52 },
    { 0x50486c35, 23 },
    { 0x38e86c36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed86c39, 76 },
    { 0x1c786c39, 34 },
    { 0x19d86c3b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41986c40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30986c44, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae886c4c, 98 },
    { 0x6f086c4d, 98 },
    { 0, 0 },
    { 0x3e886c4f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f86c54, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79386c58, 52 },
    { 0xc6c86c59, 76 },
    { 0xbf686c59, 96 },
    { 0xd0d86c5b, 98 },
    { 0, 0 },
    { 0x65186c5d, 90 },
    { 0, 0 },
    { 0xfa486c5f, 77 },
    { 0x3ff86c60, 76 },
    { 0, 0 },
    { 0x77f86c62, 98 },
    { 0x78586c62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd86c66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e86c6a, 90 },
    { 0x64186c6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d86c6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35586c78, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a286c7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f86c81, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a86c88, 23 },
    { 0xbba86c89, 95 },
    { 0, 0 },
    { 0xa5a86c8b, 77 },
    { 0x2e086c8c, 95 },
    { 0xfe886c8c, 61 },
    { 0xc6986c8d, 23 },
    { 0xb5686c8f, 33 },
    { 0, 0 },
    { 0xd2286c91, 90 },
    { 0xe1f86c92, 33 },
    { 0x41b86c93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x23986c96, 84 },
    { 0, 0 },
    { 0xafc86c98, 85 },
    { 0xcca86c99, 40 },
    { 0, 0 },
    { 0x3d786c9b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57686c9f, 61 },
    { 0xb1686c9f, 61 },
    { 0x2b686c9f, 84 },
    { 0xc2086ca2, 33 },
    { 0, 0 },
    { 0x71686ca4, 98 },
    { 0x9f86ca4, 95 },
    { 0x40986ca6, 61 },
    { 0xb1e86ca7, 33 },
    { 0x4e686ca5, 46 },
    { 0, 0 },
    { 0x91b86caa, 23 },
    { 0, 0 },
    { 0xb0786cac, 90 },
    { 0x81c86cad, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f86cb0, 98 },
    { 0, 0 },
    { 0x4986cb2, 91 },
    { 0xfd686cb3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0986cb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5086cba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x19386cbd, 61 },
    { 0xabb86cbe, 90 },
    { 0xbd86cbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f86cc3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b086cc7, 61 },
    { 0xa0f86cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42186ccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f86ccf, 90 },
    { 0x64286cd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62486cd3, 99 },
    { 0xfd986cd4, 90 },
    { 0, 0 },
    { 0xc4886cd6, 61 },
    { 0x6af86cd7, 46 },
    { 0xf386cd7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b86cdc, 90 },
    { 0, 0 },
    { 0xe8f86cde, 34 },
    { 0x58d86cdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0186ce4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c86cea, 90 },
    { 0xe8786ceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42586cef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c86cf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a86cf6, 90 },
    { 0xe9086cf7, 43 },
    { 0x1f686cf7, 98 },
    { 0xbed86cf9, 33 },
    { 0x6bd86cfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba86cfd, 68 },
    { 0xc2886cfd, 95 },
    { 0xc4086cfe, 85 },
    { 0x28886cfe, 98 },
    { 0xfee86cfd, 34 },
    { 0xc3986d02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f86d05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d86d0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef86d0f, 90 },
    { 0x92086d10, 98 },
    { 0xbb086d0f, 46 },
    { 0x3be86d11, 91 },
    { 0, 0 },
    { 0x30b86d14, 34 },
    { 0x81186d15, 61 },
    { 0xb6086d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a686d19, 95 },
    { 0, 0 },
    { 0x37486d1b, 95 },
    { 0x3f086d1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b86d1f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a386d23, 85 },
    { 0xf2586d24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40386d2b, 23 },
    { 0, 0 },
    { 0x5e586d2d, 98 },
    { 0xc6286d2e, 100 },
    { 0x3ec86d2f, 23 },
    { 0xb5d86d2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5386d36, 76 },
    { 0xddf86d37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95086d3b, 101 },
    { 0xfe086d3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73086d40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67686d43, 98 },
    { 0x41986d43, 67 },
    { 0xb4f86d44, 23 },
    { 0x28f86d44, 63 },
    { 0xa8086d47, 90 },
    { 0xc6e86d45, 33 },
    { 0x7a386d49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc586d4d, 92 },
    { 0x71886d4e, 68 },
    { 0x29886d4e, 98 },
    { 0, 0 },
    { 0x77086d51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20186d55, 69 },
    { 0, 0 },
    { 0xfbd86d57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f386d5b, 33 },
    { 0xca86d5b, 90 },
    { 0, 0 },
    { 0xa2f86d5e, 95 },
    { 0, 0 },
    { 0x55586d60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49686d64, 61 },
    { 0x21586d65, 78 },
    { 0xe1986d66, 95 },
    { 0x1a586d67, 87 },
    { 0, 0 },
    { 0x86d69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30386d6d, 90 },
    { 0xdf086d6e, 23 },
    { 0, 0 },
    { 0x67386d70, 95 },
    { 0xf3586d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1486d74, 68 },
    { 0, 0 },
    { 0x3c486d76, 43 },
    { 0, 0 },
    { 0xcef86d78, 33 },
    { 0, 0 },
    { 0xd8e86d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a786d82, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e86d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60786d8a, 78 },
    { 0x34e86d8b, 95 },
    { 0x19f86d8c, 33 },
    { 0x6fb86d8d, 61 },
    { 0x32286d8e, 68 },
    { 0x68186d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b86d93, 61 },
    { 0x52d86d93, 61 },
    { 0x83e86d95, 98 },
    { 0xe8d86d95, 90 },
    { 0xaa186d97, 67 },
    { 0xa0186d98, 33 },
    { 0x68886d95, 55 },
    { 0x8c86d9a, 90 },
    { 0, 0 },
    { 0x31a86d9c, 33 },
    { 0, 0 },
    { 0xa2f86d9e, 23 },
    { 0x15f86d9f, 66 },
    { 0x52286d9f, 98 },
    { 0x87486da1, 66 },
    { 0xdb186d9e, 91 },
    { 0, 0 },
    { 0x17b86da4, 34 },
    { 0xff386da5, 34 },
    { 0, 0 },
    { 0xe4f86da7, 76 },
    { 0x28f86da8, 95 },
    { 0, 0 },
    { 0x2c486daa, 52 },
    { 0x5c386dab, 55 },
    { 0, 0 },
    { 0x83a86dad, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd086db0, 43 },
    { 0x1986db0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50786db4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4186dbb, 95 },
    { 0x50b86dbc, 101 },
    { 0, 0 },
    { 0xc9686dbe, 66 },
    { 0xc4a86dbf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75186dc4, 61 },
    { 0, 0 },
    { 0xc7786dc6, 90 },
    { 0x62486dc7, 90 },
    { 0xd9a86dc8, 90 },
    { 0xa9c86dc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb86dcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae86dd3, 98 },
    { 0, 0 },
    { 0x9ad86dd5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd86dd8, 95 },
    { 0x8f786dd9, 76 },
    { 0x11b86dda, 85 },
    { 0xc586ddb, 75 },
    { 0x74e86ddb, 90 },
    { 0xdb886ddd, 82 },
    { 0x9a786dda, 87 },
    { 0, 0 },
    { 0x9fd86de0, 88 },
    { 0x3cf86de1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20286de7, 98 },
    { 0x47e86de8, 95 },
    { 0x24d86de8, 98 },
    { 0xe1686dea, 90 },
    { 0, 0 },
    { 0x91086dec, 61 },
    { 0, 0 },
    { 0xb5586dee, 90 },
    { 0x6ce86dee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7486df2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e86df7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d686dfa, 33 },
    { 0xea486dfa, 95 },
    { 0x56d86dfa, 61 },
    { 0xb1986dfd, 33 },
    { 0xaf286dfe, 90 },
    { 0x64786dff, 90 },
    { 0xf8286dfd, 43 },
    { 0xcda86e01, 90 },
    { 0x6aa86e02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe786e05, 23 },
    { 0, 0 },
    { 0xf3186e07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d86e0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0886e11, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb786e18, 66 },
    { 0, 0 },
    { 0x5a286e1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8386e28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df86e2b, 98 },
    { 0, 0 },
    { 0x1ca86e2d, 90 },
    { 0xb1786e2e, 98 },
    { 0xd3986e2f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x19686e32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a386e37, 33 },
    { 0x20886e37, 61 },
    { 0x486e39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48186e40, 90 },
    { 0x94686e41, 61 },
    { 0, 0 },
    { 0x6c486e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50886e4c, 34 },
    { 0, 0 },
    { 0x3c86e4e, 98 },
    { 0xd7886e4e, 46 },
    { 0x2e586e4f, 90 },
    { 0xdd786e4f, 51 },
    { 0xe6686e52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e86e58, 95 },
    { 0x80e86e58, 61 },
    { 0x65386e5a, 78 },
    { 0xd386e5b, 99 },
    { 0xcb86e5c, 91 },
    { 0x22886e5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc86e60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc86e65, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57686e6a, 61 },
    { 0xcf86e6b, 23 },
    { 0x21d86e6c, 90 },
    { 0, 0 },
    { 0xa5c86e6e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55486e71, 88 },
    { 0, 0 },
    { 0xfa086e73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79086e7d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98586e81, 55 },
    { 0, 0 },
    { 0xd0e86e83, 90 },
    { 0, 0 },
    { 0x4b186e85, 87 },
    { 0xfcd86e86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0586e8b, 33 },
    { 0xff86e8b, 90 },
    { 0x51786e8c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db86e90, 46 },
    { 0xf5e86e90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe586e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a86e9a, 78 },
    { 0x63d86e9b, 23 },
    { 0x74e86e9c, 55 },
    { 0x96f86e9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94986eb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b86eb7, 33 },
    { 0xe5886eb8, 97 },
    { 0x36686eb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53786ebd, 95 },
    { 0, 0 },
    { 0x77d86ebf, 55 },
    { 0x98186ec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a86ec5, 100 },
    { 0xa6b86ec5, 98 },
    { 0xaf86ec5, 66 },
    { 0x8fb86ec8, 72 },
    { 0x38b86ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e86ecd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11486ed2, 90 },
    { 0x98e86ed2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2586ed8, 43 },
    { 0x14886ed9, 61 },
    { 0xcf386eda, 81 },
    { 0x67386edb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d86ee1, 61 },
    { 0xaf586ee2, 33 },
    { 0x8d786ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb86ee6, 61 },
    { 0x1e086ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed86eec, 90 },
    { 0xf1786eed, 93 },
    { 0, 0 },
    { 0xdfd86eef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf86ef2, 100 },
    { 0, 0 },
    { 0x10586ef4, 99 },
    { 0xc4986ef5, 98 },
    { 0x1dc86ef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36286efb, 76 },
    { 0xb286efc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f86eff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d586f03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f86f06, 77 },
    { 0x9ca86f07, 46 },
    { 0xfc386f06, 70 },
    { 0x3af86f09, 41 },
    { 0x26386f0a, 77 },
    { 0x56286f0a, 85 },
    { 0x6e86f0c, 81 },
    { 0x63f86f0c, 84 },
    { 0x2e886f0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff86f11, 76 },
    { 0, 0 },
    { 0xfd786f13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf586f16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f86f1c, 61 },
    { 0xd5b86f1d, 90 },
    { 0, 0 },
    { 0xfbd86f1f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96186f22, 99 },
    { 0x38386f23, 66 },
    { 0x99a86f22, 84 },
    { 0x3bf86f23, 4 },
    { 0xf3c86f26, 98 },
    { 0x8ca86f27, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d086f32, 90 },
    { 0, 0 },
    { 0x75786f34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d686f37, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b286f45, 61 },
    { 0, 0 },
    { 0x46086f47, 98 },
    { 0xa2586f48, 61 },
    { 0xd5386f49, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d86f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d786f54, 76 },
    { 0, 0 },
    { 0xae286f56, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xff286f59, 61 },
    { 0, 0 },
    { 0x15786f5b, 98 },
    { 0x3d786f5c, 99 },
    { 0x5cc86f5d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9286f61, 91 },
    { 0, 0 },
    { 0xb9186f63, 95 },
    { 0, 0 },
    { 0x90186f65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6486f6b, 33 },
    { 0x65086f6c, 34 },
    { 0xae586f6d, 61 },
    { 0, 0 },
    { 0x27086f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6186f72, 90 },
    { 0x3ee86f73, 66 },
    { 0, 0 },
    { 0x7be86f75, 87 },
    { 0x7a986f75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9386f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31886f7d, 61 },
    { 0x99186f7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c86f8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x10386f8e, 94 },
    { 0, 0 },
    { 0x84686f90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1186f96, 95 },
    { 0xa2686f97, 95 },
    { 0x8286f96, 36 },
    { 0x83486f99, 61 },
    { 0x1bb86f97, 90 },
    { 0x9f786f9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a86f9e, 65 },
    { 0x5a286f9f, 90 },
    { 0xfc486f9f, 43 },
    { 0xffd86f9f, 38 },
    { 0x6b986fa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8186fa7, 95 },
    { 0x45986fa7, 95 },
    { 0xe2f86fa8, 46 },
    { 0, 0 },
    { 0xa686fab, 88 },
    { 0, 0 },
    { 0x3bd86fad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b486fb2, 19 },
    { 0xf286fb3, 67 },
    { 0, 0 },
    { 0x96486fb5, 84 },
    { 0xb5c86fb6, 50 },
    { 0xa9986fb7, 95 },
    { 0, 0 },
    { 0x68e86fb9, 43 },
    { 0, 0 },
    { 0x22286fbb, 55 },
    { 0xe0486fbc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10986fbf, 46 },
    { 0x8ec86fc0, 46 },
    { 0xcd286fbf, 98 },
    { 0x42e86fc2, 91 },
    { 0, 0 },
    { 0x9e786fc4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x19186fc7, 66 },
    { 0x14186fc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5086fcc, 91 },
    { 0x60486fcd, 33 },
    { 0x9e486fcd, 93 },
    { 0x22886fce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d86fd3, 33 },
    { 0, 0 },
    { 0xc5d86fd5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc86fd9, 99 },
    { 0x49d86fda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2786fe2, 90 },
    { 0, 0 },
    { 0xc3486fe4, 17 },
    { 0xd4f86fe4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4186ff5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a86ff8, 23 },
    { 0xd3b86ff9, 61 },
    { 0x29b86ffa, 61 },
    { 0x4a186ffb, 61 },
    { 0x7a786ffb, 100 },
    { 0x22986ffc, 90 },
    { 0xaba86ffc, 34 },
    { 0x8ba86fff, 98 },
    { 0x9ed86fff, 90 },
    { 0x41f87000, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c987005, 34 },
    { 0xef687006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e87009, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f8700c, 55 },
    { 0x2d48700d, 88 },
    { 0, 0 },
    { 0x4db8700f, 98 },
    { 0, 0 },
    { 0x68b87011, 95 },
    { 0xf8e87012, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x75287015, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f87018, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8701c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc888701f, 90 },
    { 0x6f88701f, 23 },
    { 0x3787020, 61 },
    { 0xf2187022, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c87028, 93 },
    { 0x4d887028, 95 },
    { 0x6218702a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38702d, 63 },
    { 0x6c68702e, 93 },
    { 0xc428702f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83387036, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3178703b, 33 },
    { 0x1408703c, 61 },
    { 0x4e68703c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b87040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd87045, 46 },
    { 0x30687045, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d687049, 61 },
    { 0, 0 },
    { 0xd7a8704b, 98 },
    { 0xa6d8704c, 93 },
    { 0x74e8704b, 76 },
    { 0, 0 },
    { 0x5868704f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec87056, 98 },
    { 0, 0 },
    { 0x33587058, 92 },
    { 0, 0 },
    { 0x81e8705a, 67 },
    { 0, 0 },
    { 0x10a8705c, 95 },
    { 0x66b8705d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61887060, 33 },
    { 0xbe287060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a387066, 34 },
    { 0x8e987067, 33 },
    { 0x71a87068, 98 },
    { 0xf5387069, 52 },
    { 0x8df87069, 90 },
    { 0, 0 },
    { 0xce98706c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487070, 33 },
    { 0xac687071, 90 },
    { 0xe5f87072, 23 },
    { 0, 0 },
    { 0x7787074, 82 },
    { 0, 0 },
    { 0xa5887076, 98 },
    { 0, 0 },
    { 0xcbb87078, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc8707b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf987080, 23 },
    { 0, 0 },
    { 0x57887082, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a187086, 52 },
    { 0x8b987087, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe8708a, 90 },
    { 0x6a28708b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c8708e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa87092, 55 },
    { 0, 0 },
    { 0xa3c87094, 84 },
    { 0x8bf87095, 33 },
    { 0x6c087095, 93 },
    { 0x29587097, 76 },
    { 0x55487096, 81 },
    { 0xf0987099, 99 },
    { 0, 0 },
    { 0xf1d8709b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa870a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb870a8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15870ad, 34 },
    { 0x1d1870ae, 95 },
    { 0xc49870ae, 76 },
    { 0, 0 },
    { 0x4c4870b1, 33 },
    { 0xb9870b2, 98 },
    { 0x284870b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743870b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa870be, 90 },
    { 0xd28870be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b870ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42870cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x890870d2, 85 },
    { 0x2c4870d3, 83 },
    { 0x708870d3, 100 },
    { 0x2c5870d5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f870dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556870e6, 95 },
    { 0xa56870e7, 90 },
    { 0xbc5870e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6870eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0870f3, 36 },
    { 0, 0 },
    { 0xbc3870f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e870fd, 91 },
    { 0x985870fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48487101, 34 },
    { 0, 0 },
    { 0x14c87103, 23 },
    { 0x68487104, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a8710e, 77 },
    { 0, 0 },
    { 0x63387110, 66 },
    { 0xad587111, 78 },
    { 0x4f087111, 33 },
    { 0xf7a87113, 23 },
    { 0x92687111, 99 },
    { 0x64787114, 31 },
    { 0, 0 },
    { 0x68087117, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc08711d, 100 },
    { 0xd2e8711d, 95 },
    { 0xec38711f, 61 },
    { 0xf58711d, 43 },
    { 0, 0 },
    { 0x8bb87122, 76 },
    { 0xdb687123, 33 },
    { 0x8a087124, 61 },
    { 0, 0 },
    { 0x3f87126, 33 },
    { 0, 0 },
    { 0xf2a87128, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b18712f, 17 },
    { 0x40f8712f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8713c, 98 },
    { 0xcd88713d, 101 },
    { 0xf308713e, 34 },
    { 0x3a28713f, 52 },
    { 0x9438713f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc87143, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x91787146, 91 },
    { 0x26d87147, 90 },
    { 0x15f87146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd8714c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db87154, 96 },
    { 0x60387154, 90 },
    { 0xcc087154, 40 },
    { 0x69487154, 98 },
    { 0x33987158, 61 },
    { 0x37387159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c38715e, 98 },
    { 0x6f78715e, 77 },
    { 0xdfd87160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54587163, 93 },
    { 0x4f287164, 76 },
    { 0, 0 },
    { 0xbd487166, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a8716f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54387173, 78 },
    { 0xb2c87173, 95 },
    { 0xdbd87175, 90 },
    { 0, 0 },
    { 0x8187177, 55 },
    { 0, 0 },
    { 0x43c87179, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc098717d, 23 },
    { 0x2c38717e, 78 },
    { 0x8c8717f, 66 },
    { 0xc778717f, 33 },
    { 0xf778717e, 48 },
    { 0xf0487180, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe687187, 90 },
    { 0x9b187188, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e8718f, 55 },
    { 0, 0 },
    { 0xd7187191, 76 },
    { 0xe687192, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x48587195, 44 },
    { 0x7eb87196, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec871a2, 90 },
    { 0, 0 },
    { 0x2d871a4, 90 },
    { 0, 0 },
    { 0x732871a6, 55 },
    { 0, 0 },
    { 0x85f871a8, 87 },
    { 0, 0 },
    { 0xcfc871aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102871ae, 52 },
    { 0xfe3871af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378871b3, 61 },
    { 0x193871b3, 95 },
    { 0x459871b5, 99 },
    { 0x5cd871b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de871b9, 93 },
    { 0, 0 },
    { 0x4d3871bb, 43 },
    { 0xb9d871bc, 90 },
    { 0, 0 },
    { 0x70e871be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98871c3, 90 },
    { 0, 0 },
    { 0x2ef871c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd871c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29871ce, 61 },
    { 0, 0 },
    { 0x876871d0, 95 },
    { 0, 0 },
    { 0x912871d2, 76 },
    { 0x67a871d2, 98 },
    { 0xd4f871d2, 90 },
    { 0x375871d3, 23 },
    { 0, 0 },
    { 0x7a4871d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b871db, 61 },
    { 0x996871db, 90 },
    { 0, 0 },
    { 0x84f871de, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14871e1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85871e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a871eb, 66 },
    { 0x7f8871eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9871f5, 98 },
    { 0xbd871f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b871fa, 95 },
    { 0, 0 },
    { 0x1f3871fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce87200, 76 },
    { 0x1a787201, 90 },
    { 0, 0 },
    { 0x6b687203, 55 },
    { 0x76a87204, 22 },
    { 0xbf487203, 98 },
    { 0x4e987205, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa87209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9138720d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b87211, 78 },
    { 0x4187211, 90 },
    { 0xe3287211, 43 },
    { 0x53487214, 91 },
    { 0, 0 },
    { 0xbc387216, 43 },
    { 0x2fc87216, 55 },
    { 0xa687216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f28721c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef28721f, 93 },
    { 0x42c87220, 61 },
    { 0x61e87220, 90 },
    { 0x8b387220, 93 },
    { 0x66787223, 90 },
    { 0xab687222, 33 },
    { 0x22e87225, 95 },
    { 0x80b87225, 90 },
    { 0, 0 },
    { 0xa0f87228, 95 },
    { 0xa6487229, 61 },
    { 0x17887228, 90 },
    { 0, 0 },
    { 0x6ef8722c, 90 },
    { 0xb38722d, 33 },
    { 0x7d58722e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c87236, 61 },
    { 0, 0 },
    { 0x87587238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd168723b, 95 },
    { 0x7d68723c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a8723f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19387242, 55 },
    { 0, 0 },
    { 0xe4d87244, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d87247, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe418724c, 17 },
    { 0xe508724c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb987257, 61 },
    { 0, 0 },
    { 0x73c87259, 90 },
    { 0xa1987259, 95 },
    { 0x10587259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd8725e, 33 },
    { 0x7378725e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82287264, 90 },
    { 0xab987265, 33 },
    { 0xf1b87265, 55 },
    { 0xd8787264, 61 },
    { 0x93a87268, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb8726b, 87 },
    { 0x4818726c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce8726f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25087272, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b87276, 23 },
    { 0x91487276, 61 },
    { 0, 0 },
    { 0x4aa87279, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2358727c, 90 },
    { 0xd008727d, 33 },
    { 0xd28727d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb587281, 75 },
    { 0, 0 },
    { 0x47e87283, 43 },
    { 0xf1687284, 61 },
    { 0xd1e87284, 90 },
    { 0x6f487283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a18728f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f687292, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b87298, 61 },
    { 0, 0 },
    { 0x3528729a, 95 },
    { 0, 0 },
    { 0x7b08729c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd872a2, 32 },
    { 0, 0 },
    { 0x32e872a4, 68 },
    { 0x803872a5, 61 },
    { 0x95a872a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x952872a9, 61 },
    { 0x91c872a9, 99 },
    { 0x67f872a9, 55 },
    { 0x2ff872ab, 61 },
    { 0xcf6872ab, 33 },
    { 0, 0 },
    { 0x144872af, 81 },
    { 0, 0 },
    { 0x293872b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f872b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d872b7, 84 },
    { 0xe8a872b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee872bc, 95 },
    { 0, 0 },
    { 0xfd2872be, 78 },
    { 0xeb9872be, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f872c3, 90 },
    { 0, 0 },
    { 0xf91872c5, 93 },
    { 0, 0 },
    { 0x17872c7, 34 },
    { 0, 0 },
    { 0xe8c872c9, 90 },
    { 0, 0 },
    { 0xb9e872cb, 34 },
    { 0xd29872cb, 61 },
    { 0, 0 },
    { 0xd99872ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc872d3, 61 },
    { 0x940872d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6872d9, 43 },
    { 0x84a872d9, 66 },
    { 0x4bb872db, 98 },
    { 0x915872db, 33 },
    { 0, 0 },
    { 0x174872de, 66 },
    { 0x2da872df, 33 },
    { 0, 0 },
    { 0xaf2872e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59872e8, 43 },
    { 0x119872e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9872ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c872f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9872fa, 90 },
    { 0xe5d872fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812872ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d87306, 93 },
    { 0xfcb87306, 78 },
    { 0x12687307, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8278730c, 99 },
    { 0xa668730d, 33 },
    { 0xd9d8730d, 61 },
    { 0x5398730f, 91 },
    { 0xa6387310, 90 },
    { 0x21c8730f, 23 },
    { 0xfa8730d, 90 },
    { 0x65d8730d, 33 },
    { 0xe8a87314, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb87317, 99 },
    { 0, 0 },
    { 0x13387319, 23 },
    { 0xc998731a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a087320, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x10887323, 33 },
    { 0, 0 },
    { 0xf6487325, 55 },
    { 0, 0 },
    { 0xf5a87327, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f8732c, 61 },
    { 0, 0 },
    { 0xee98732e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f87333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce87337, 61 },
    { 0, 0 },
    { 0x5b87339, 76 },
    { 0, 0 },
    { 0x9b48733b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf898733e, 55 },
    { 0xda88733f, 33 },
    { 0x4ce87340, 75 },
    { 0x61587341, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xff887344, 76 },
    { 0xbe987345, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8087349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x688734c, 91 },
    { 0, 0 },
    { 0xa718734e, 98 },
    { 0xa098734f, 66 },
    { 0xf5587350, 81 },
    { 0x99e8734e, 33 },
    { 0x27387352, 90 },
    { 0xcf587352, 32 },
    { 0xa7987352, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53187359, 90 },
    { 0x9a98735a, 61 },
    { 0, 0 },
    { 0xfd8735c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96487362, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b87365, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x86687368, 33 },
    { 0, 0 },
    { 0x3be8736a, 90 },
    { 0, 0 },
    { 0x75c8736c, 61 },
    { 0, 0 },
    { 0xc668736e, 34 },
    { 0, 0 },
    { 0x6d87370, 96 },
    { 0xac187371, 77 },
    { 0x9bb87372, 76 },
    { 0x33987371, 52 },
    { 0x43887372, 95 },
    { 0xd3787374, 43 },
    { 0x65987374, 93 },
    { 0x87287377, 61 },
    { 0xfa187373, 95 },
    { 0x28487379, 90 },
    { 0x287378, 61 },
    { 0x6478737b, 95 },
    { 0x4e087373, 53 },
    { 0xd418737d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab38738b, 93 },
    { 0x5f68738b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf278738f, 99 },
    { 0xcdb87390, 23 },
    { 0x8b087390, 64 },
    { 0x92787391, 95 },
    { 0xc368738f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c8739d, 61 },
    { 0xc4f8739e, 90 },
    { 0xaf68739f, 33 },
    { 0xe2a8739f, 33 },
    { 0x248739d, 85 },
    { 0xc5873a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f873a6, 76 },
    { 0x12c873a6, 34 },
    { 0, 0 },
    { 0xe76873a9, 93 },
    { 0, 0 },
    { 0x4c2873ab, 52 },
    { 0, 0 },
    { 0x587873ad, 95 },
    { 0x293873ae, 90 },
    { 0x880873af, 66 },
    { 0, 0 },
    { 0x1d0873b1, 100 },
    { 0xd0e873b2, 23 },
    { 0x777873b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599873b9, 33 },
    { 0xc56873b9, 55 },
    { 0xf60873bb, 67 },
    { 0xced873bb, 90 },
    { 0x7a6873bd, 95 },
    { 0xefb873bd, 75 },
    { 0x66c873bf, 83 },
    { 0x108873ba, 95 },
    { 0xda5873c1, 95 },
    { 0x23c873ba, 55 },
    { 0xc5b873c2, 65 },
    { 0xc50873c4, 96 },
    { 0x192873c4, 76 },
    { 0xf7873c6, 52 },
    { 0x694873c6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3873cb, 33 },
    { 0, 0 },
    { 0x49b873cd, 33 },
    { 0, 0 },
    { 0x263873cf, 17 },
    { 0, 0 },
    { 0x98f873d1, 88 },
    { 0x95c873d1, 61 },
    { 0xaf6873d1, 98 },
    { 0xea9873d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7873d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec873de, 23 },
    { 0, 0 },
    { 0xadd873e0, 43 },
    { 0x229873e0, 77 },
    { 0x524873e2, 61 },
    { 0x4a7873e0, 64 },
    { 0xd99873e4, 91 },
    { 0, 0 },
    { 0xe9c873e6, 90 },
    { 0xc8e873e7, 98 },
    { 0x2ac873e6, 99 },
    { 0xb17873e8, 55 },
    { 0xc04873ea, 97 },
    { 0x1e1873eb, 61 },
    { 0x7e7873ec, 43 },
    { 0xb49873ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173873f1, 33 },
    { 0xb99873f2, 43 },
    { 0x25a873f2, 76 },
    { 0x852873f2, 76 },
    { 0, 0 },
    { 0x143873f6, 90 },
    { 0xeae873f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72873fb, 93 },
    { 0x19b873fb, 33 },
    { 0x225873fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0087407, 61 },
    { 0xa9b87408, 33 },
    { 0x50c87409, 43 },
    { 0x8988740a, 46 },
    { 0x8e87407, 98 },
    { 0xdba8740b, 95 },
    { 0, 0 },
    { 0x9418740e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8587413, 17 },
    { 0x7587414, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1187417, 98 },
    { 0xdbc87417, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c8741f, 61 },
    { 0x17587420, 99 },
    { 0x5e087421, 78 },
    { 0x55987422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3887425, 90 },
    { 0x19b87426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc08742e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787432, 33 },
    { 0x69387432, 85 },
    { 0x10387434, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec87438, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78743c, 95 },
    { 0xd1a8743c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xee587440, 95 },
    { 0x587441, 61 },
    { 0xa087442, 61 },
    { 0x1b787442, 33 },
    { 0x50387442, 23 },
    { 0x8c987445, 8 },
    { 0, 0 },
    { 0xa387447, 46 },
    { 0x8bf87448, 90 },
    { 0xf4687448, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf68744e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d87454, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa87458, 78 },
    { 0x85387458, 98 },
    { 0x73287459, 38 },
    { 0x2dc8745b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df8745e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76787463, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e87467, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f8746a, 98 },
    { 0x8fb8746a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c387470, 33 },
    { 0, 0 },
    { 0xf4a87472, 61 },
    { 0x6ec87472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d87477, 55 },
    { 0x7a87478, 61 },
    { 0x31187479, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98747c, 61 },
    { 0xfa38747d, 55 },
    { 0x268747e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d687481, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a887488, 34 },
    { 0x6ad87489, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8748c, 90 },
    { 0xe788748c, 99 },
    { 0xb178748d, 33 },
    { 0xd4e8748f, 34 },
    { 0x7f787490, 76 },
    { 0x9458748f, 77 },
    { 0x60887492, 68 },
    { 0x93d87491, 90 },
    { 0x30d87494, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f287497, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68749d, 61 },
    { 0xa368749e, 33 },
    { 0x7fe8749f, 98 },
    { 0xd29874a0, 95 },
    { 0x337874a1, 98 },
    { 0x1c88749d, 23 },
    { 0xf958749d, 90 },
    { 0x6fa874a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399874a8, 90 },
    { 0, 0 },
    { 0x367874aa, 78 },
    { 0, 0 },
    { 0xf01874ac, 85 },
    { 0x728874ac, 90 },
    { 0, 0 },
    { 0x6e3874af, 52 },
    { 0x2e5874b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7874b5, 34 },
    { 0x7a874b6, 87 },
    { 0, 0 },
    { 0xf52874b8, 67 },
    { 0xfb1874b9, 97 },
    { 0x674874b8, 84 },
    { 0xe38874bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8874c4, 76 },
    { 0x950874c5, 52 },
    { 0xb6c874c4, 33 },
    { 0x246874c6, 42 },
    { 0x6d4874c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d874cc, 61 },
    { 0xc98874cd, 90 },
    { 0xb85874cd, 90 },
    { 0x837874cf, 98 },
    { 0x81874d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa874d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec874d6, 98 },
    { 0xd0874d7, 23 },
    { 0xf31874d8, 61 },
    { 0x3b1874d7, 43 },
    { 0, 0 },
    { 0x7d5874db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50874de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3874e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37874e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa874ea, 90 },
    { 0xd29874ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d874f4, 46 },
    { 0, 0 },
    { 0xb24874f6, 76 },
    { 0xf77874f7, 90 },
    { 0xbf2874f6, 98 },
    { 0xd87874f9, 55 },
    { 0xe48874fa, 61 },
    { 0x3f3874fa, 66 },
    { 0xe44874fa, 33 },
    { 0, 0 },
    { 0xc88874fe, 33 },
    { 0xf84874ff, 33 },
    { 0, 0 },
    { 0xacc87501, 90 },
    { 0, 0 },
    { 0x12587503, 90 },
    { 0, 0 },
    { 0xafc87505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158750c, 61 },
    { 0xd198750d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a287513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e287516, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef58751a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee78751f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e87523, 87 },
    { 0, 0 },
    { 0x1b987525, 98 },
    { 0x2a087526, 23 },
    { 0x3dc87526, 61 },
    { 0xfcc87528, 33 },
    { 0xe9587527, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d8752c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a87531, 98 },
    { 0xaa87532, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x15687535, 81 },
    { 0x19d87536, 76 },
    { 0x86b87537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8118753c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb387541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7998754a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7068754d, 95 },
    { 0x40b8754e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b587551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e987555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d87558, 33 },
    { 0x89087559, 93 },
    { 0xf198755a, 46 },
    { 0x1788755a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d8755f, 43 },
    { 0x8e58755f, 33 },
    { 0xc1f87561, 33 },
    { 0x68f87562, 34 },
    { 0x4a487562, 33 },
    { 0x46a87562, 61 },
    { 0xd3e87564, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a187568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2508756e, 99 },
    { 0x3298756e, 98 },
    { 0xdd87570, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d987573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f87579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1a8757c, 23 },
    { 0xeb28757c, 78 },
    { 0x8e28757e, 95 },
    { 0xaf88757d, 87 },
    { 0, 0 },
    { 0x1987581, 90 },
    { 0x47787581, 61 },
    { 0x3ee87583, 78 },
    { 0xac987583, 100 },
    { 0x25987584, 93 },
    { 0, 0 },
    { 0xa1887587, 61 },
    { 0, 0 },
    { 0xce187589, 95 },
    { 0x72b8758a, 90 },
    { 0x4d98758b, 23 },
    { 0x66d8758c, 90 },
    { 0, 0 },
    { 0xa908758e, 55 },
    { 0xaaf8758e, 85 },
    { 0, 0 },
    { 0x69387591, 33 },
    { 0, 0 },
    { 0xafd87593, 90 },
    { 0xc5187594, 95 },
    { 0x4fc87594, 95 },
    { 0x7dd87596, 61 },
    { 0x6ff87597, 61 },
    { 0x5a387596, 85 },
    { 0x76c87599, 46 },
    { 0, 0 },
    { 0xd578759b, 91 },
    { 0, 0 },
    { 0xe678759d, 88 },
    { 0xbfb8759e, 23 },
    { 0xef18759f, 17 },
    { 0xf85875a0, 90 },
    { 0xf3c875a1, 98 },
    { 0x38c875a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5875a7, 66 },
    { 0, 0 },
    { 0xd6e875a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82875ad, 34 },
    { 0xeea875ae, 46 },
    { 0, 0 },
    { 0x2cb875b0, 98 },
    { 0x3b6875b1, 85 },
    { 0xc74875b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47875b8, 99 },
    { 0xf7d875b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75875bf, 91 },
    { 0, 0 },
    { 0x905875c1, 84 },
    { 0, 0 },
    { 0xaeb875c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51875c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5875cc, 98 },
    { 0x42b875cd, 98 },
    { 0x16e875cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36875d6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x602875d9, 90 },
    { 0, 0 },
    { 0x4dc875db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d875df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef875e2, 90 },
    { 0x921875e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93875e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289875ea, 66 },
    { 0xff4875eb, 83 },
    { 0xce5875ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab875f0, 95 },
    { 0, 0 },
    { 0xfbd875f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5875f5, 33 },
    { 0x65c875f6, 61 },
    { 0x5d5875f7, 23 },
    { 0x78b875f8, 61 },
    { 0x2c3875f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911875ff, 85 },
    { 0xe5887600, 61 },
    { 0xfff87601, 66 },
    { 0, 0 },
    { 0xef187603, 61 },
    { 0, 0 },
    { 0x97a87605, 66 },
    { 0, 0 },
    { 0xdc887607, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec38760b, 90 },
    { 0xd1c8760c, 61 },
    { 0x3258760d, 61 },
    { 0, 0 },
    { 0xbe38760f, 87 },
    { 0xf2c87610, 93 },
    { 0x4b98760f, 33 },
    { 0x19687611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b87615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9298761b, 90 },
    { 0, 0 },
    { 0x4878761d, 80 },
    { 0x76a8761d, 36 },
    { 0xb318761f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a8762a, 61 },
    { 0, 0 },
    { 0x9808762c, 99 },
    { 0x9df8762d, 100 },
    { 0x1328762e, 99 },
    { 0x97f8762f, 95 },
    { 0x2418762c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f687633, 46 },
    { 0xf2687634, 99 },
    { 0x89487633, 98 },
    { 0x12387634, 33 },
    { 0x91487635, 61 },
    { 0x287638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff48763e, 90 },
    { 0xaae8763f, 61 },
    { 0xb5d8763f, 99 },
    { 0x76687641, 61 },
    { 0x5cb87642, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b87645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9588764a, 23 },
    { 0, 0 },
    { 0x1978764c, 61 },
    { 0x6c8764d, 33 },
    { 0, 0 },
    { 0xbfd8764f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f687654, 95 },
    { 0, 0 },
    { 0x2aa87656, 23 },
    { 0x38787657, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb928765a, 43 },
    { 0xb958765b, 61 },
    { 0x2428765b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f887661, 85 },
    { 0, 0 },
    { 0x6b287663, 66 },
    { 0x2c187664, 61 },
    { 0xe6d87665, 43 },
    { 0x70087665, 80 },
    { 0x84887665, 99 },
    { 0, 0 },
    { 0x1087669, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3487675, 17 },
    { 0, 0 },
    { 0xda87677, 99 },
    { 0, 0 },
    { 0xca587679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x368767c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2987682, 91 },
    { 0x89e87683, 43 },
    { 0x49387684, 23 },
    { 0x89987683, 33 },
    { 0xaa887684, 33 },
    { 0x19c87684, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf818768f, 34 },
    { 0, 0 },
    { 0xeb287691, 90 },
    { 0x75d87692, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a87695, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a98769a, 98 },
    { 0, 0 },
    { 0x6c8769c, 23 },
    { 0, 0 },
    { 0xa4c8769e, 42 },
    { 0, 0 },
    { 0x82876a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f876a3, 91 },
    { 0x2e2876a3, 90 },
    { 0x65d876a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687876ac, 43 },
    { 0x66d876ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35876b4, 33 },
    { 0x26876b5, 77 },
    { 0xd05876b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x767876b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a876bc, 95 },
    { 0x4bd876bd, 76 },
    { 0x34876be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a876c2, 46 },
    { 0x78a876c3, 55 },
    { 0x826876c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85876ca, 33 },
    { 0x1c5876ca, 91 },
    { 0, 0 },
    { 0xc72876cd, 23 },
    { 0, 0 },
    { 0xc1b876cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58876d2, 61 },
    { 0, 0 },
    { 0xab876d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85876da, 61 },
    { 0x5ad876db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db876df, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563876e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539876ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x920876f0, 98 },
    { 0xe9b876f1, 33 },
    { 0x27c876f2, 23 },
    { 0xcf6876f3, 33 },
    { 0, 0 },
    { 0x1a2876f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845876f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d876fc, 93 },
    { 0x93876fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5487703, 78 },
    { 0, 0 },
    { 0x59387705, 78 },
    { 0, 0 },
    { 0x63c87707, 23 },
    { 0xd4c87708, 76 },
    { 0, 0 },
    { 0x568770a, 78 },
    { 0, 0 },
    { 0x1948770c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc8770f, 61 },
    { 0xb3e8770f, 61 },
    { 0x32587711, 98 },
    { 0xc1f8770f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e87717, 91 },
    { 0x5a087717, 67 },
    { 0, 0 },
    { 0x9a28771a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8771d, 46 },
    { 0xeb48771e, 23 },
    { 0x4f8771f, 90 },
    { 0xcce87720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe887725, 66 },
    { 0x4c087726, 46 },
    { 0, 0 },
    { 0xd1a87728, 90 },
    { 0x6b87729, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838772c, 55 },
    { 0x84e8772d, 98 },
    { 0, 0 },
    { 0xafb8772f, 100 },
    { 0xc038772f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd187736, 96 },
    { 0x68487737, 96 },
    { 0x99687738, 95 },
    { 0x94087736, 87 },
    { 0x4cf87737, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1288773f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b87743, 23 },
    { 0x9a887744, 8 },
    { 0x75187745, 5 },
    { 0x28f87746, 61 },
    { 0xab887747, 43 },
    { 0x77e87747, 61 },
    { 0xd1d87747, 43 },
    { 0xe838774a, 98 },
    { 0x63887746, 23 },
    { 0xdd38774c, 46 },
    { 0xada87743, 34 },
    { 0, 0 },
    { 0x7df8774f, 23 },
    { 0xf3087750, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe87753, 55 },
    { 0, 0 },
    { 0x64787755, 90 },
    { 0x58b87756, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3088775d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b8776d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f87770, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58887778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f8777d, 34 },
    { 0x7f18777d, 87 },
    { 0, 0 },
    { 0xfc787780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a287784, 78 },
    { 0x9087785, 90 },
    { 0xceb87785, 33 },
    { 0x12d87787, 93 },
    { 0xb6f87787, 23 },
    { 0xc1d87789, 98 },
    { 0x95487789, 33 },
    { 0x7fc8778b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5118778e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd87791, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88887799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b8779d, 90 },
    { 0xf9e8779e, 33 },
    { 0xe3c8779f, 90 },
    { 0x998779f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e877a5, 75 },
    { 0xdea877a6, 98 },
    { 0x238877a7, 70 },
    { 0xd56877a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1877ac, 85 },
    { 0xadb877ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f877b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c877b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc877c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270877c9, 98 },
    { 0, 0 },
    { 0x242877cb, 95 },
    { 0x981877cc, 61 },
    { 0x4d6877cb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b877d3, 90 },
    { 0xc58877d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71877da, 98 },
    { 0x13f877db, 98 },
    { 0x73b877dc, 80 },
    { 0xc80877db, 90 },
    { 0x87d877de, 81 },
    { 0x267877de, 90 },
    { 0xc1b877e0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c877e3, 43 },
    { 0xbc7877e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf877e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0877ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e877f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606877f7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee387801, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d87804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1487807, 80 },
    { 0, 0 },
    { 0x50987809, 33 },
    { 0x3e28780a, 98 },
    { 0xfbb87809, 90 },
    { 0xe538780c, 98 },
    { 0x22187809, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33787810, 43 },
    { 0xbb87811, 95 },
    { 0x99687812, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3287816, 80 },
    { 0, 0 },
    { 0xe1187818, 98 },
    { 0x2dc87819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a8781c, 98 },
    { 0xa1b8781d, 88 },
    { 0x6d78781d, 98 },
    { 0xaa28781d, 23 },
    { 0x5c987820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6787827, 34 },
    { 0, 0 },
    { 0x81f87829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db87834, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28783b, 33 },
    { 0x6038783b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e8783f, 23 },
    { 0, 0 },
    { 0x4d987841, 33 },
    { 0, 0 },
    { 0x3dd87843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c87847, 33 },
    { 0x10c87848, 76 },
    { 0, 0 },
    { 0x4078784a, 40 },
    { 0, 0 },
    { 0x8508784c, 81 },
    { 0, 0 },
    { 0xb5c8784e, 76 },
    { 0xb2a8784f, 48 },
    { 0xe5d87850, 93 },
    { 0x13d87851, 76 },
    { 0xa387851, 69 },
    { 0x90b87851, 76 },
    { 0x24a87854, 61 },
    { 0x1db87852, 95 },
    { 0x43587856, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf668785f, 95 },
    { 0, 0 },
    { 0xb8087861, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9587864, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de87867, 23 },
    { 0xaf887868, 61 },
    { 0, 0 },
    { 0x5548786a, 23 },
    { 0xfe18786b, 83 },
    { 0xc578786c, 95 },
    { 0xfbc8786d, 76 },
    { 0xa78786e, 43 },
    { 0xb6e8786b, 33 },
    { 0, 0 },
    { 0x4f87871, 100 },
    { 0, 0 },
    { 0xfa487873, 98 },
    { 0, 0 },
    { 0xe6d87875, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8787d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3887886, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958788a, 91 },
    { 0, 0 },
    { 0x6ad8788c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda887890, 68 },
    { 0x47287891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a87894, 91 },
    { 0, 0 },
    { 0x37687896, 33 },
    { 0xb6287896, 34 },
    { 0xee187898, 23 },
    { 0x6cc87898, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8789d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3878a0, 90 },
    { 0x309878a1, 55 },
    { 0xcb0878a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5878ae, 61 },
    { 0x2ab878ae, 95 },
    { 0x120878af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x391878b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22878b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb878ba, 96 },
    { 0, 0 },
    { 0xb6f878bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb878c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344878c9, 61 },
    { 0, 0 },
    { 0x889878cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x187878ce, 61 },
    { 0xfce878cf, 90 },
    { 0x2c2878d0, 95 },
    { 0, 0 },
    { 0x748878d2, 23 },
    { 0x289878d3, 33 },
    { 0, 0 },
    { 0xed2878d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f878d8, 46 },
    { 0x2fd878d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e878e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a878e5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3878e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59878eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x476878ee, 61 },
    { 0, 0 },
    { 0xd1f878f0, 17 },
    { 0x9c6878f0, 90 },
    { 0x248878f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a878f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee878f9, 55 },
    { 0x19a878f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19287904, 76 },
    { 0x99887905, 76 },
    { 0xf7187906, 33 },
    { 0xb4f87907, 52 },
    { 0xf3c87904, 33 },
    { 0, 0 },
    { 0x5d38790a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268790f, 77 },
    { 0xc87910, 23 },
    { 0, 0 },
    { 0x5287912, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7587919, 55 },
    { 0x36d8791a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3e8791f, 61 },
    { 0x48787920, 61 },
    { 0x16c8791f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18792e, 84 },
    { 0, 0 },
    { 0xea887930, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f87933, 90 },
    { 0, 0 },
    { 0xd2487935, 61 },
    { 0x50787936, 55 },
    { 0, 0 },
    { 0x26687938, 33 },
    { 0, 0 },
    { 0xcd8793a, 98 },
    { 0x8ab8793b, 93 },
    { 0, 0 },
    { 0x3388793d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51787942, 46 },
    { 0xfee87943, 84 },
    { 0x6a87944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8794c, 34 },
    { 0xc88794d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25787955, 81 },
    { 0, 0 },
    { 0x32a87957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58795a, 83 },
    { 0x2808795b, 61 },
    { 0x378795c, 92 },
    { 0xb298795c, 33 },
    { 0xa658795c, 61 },
    { 0x26c8795f, 98 },
    { 0x3198795f, 23 },
    { 0, 0 },
    { 0x46587962, 78 },
    { 0x1c587962, 52 },
    { 0x97987964, 46 },
    { 0x2f387965, 95 },
    { 0x1387966, 95 },
    { 0xe4187967, 87 },
    { 0xc4887966, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x8648796b, 90 },
    { 0x3bd8796c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b87972, 76 },
    { 0xa5087973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c87978, 61 },
    { 0x3eb87979, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe98797c, 99 },
    { 0xc438797d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a87980, 76 },
    { 0x91487981, 61 },
    { 0xdb787982, 93 },
    { 0xbc187983, 98 },
    { 0, 0 },
    { 0x57e87985, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc68798b, 98 },
    { 0x87f8798c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52887990, 76 },
    { 0, 0 },
    { 0x61e87992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97887995, 95 },
    { 0x92e87995, 61 },
    { 0xa1887997, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad8799a, 90 },
    { 0x35d8799b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409879a1, 64 },
    { 0xeab879a2, 90 },
    { 0x492879a3, 98 },
    { 0x907879a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b879a8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198879b4, 76 },
    { 0xad1879b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x333879b8, 98 },
    { 0xc61879b9, 33 },
    { 0, 0 },
    { 0x2bf879bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e879be, 90 },
    { 0x939879bf, 84 },
    { 0x31f879bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b879c4, 76 },
    { 0x863879c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3879ca, 95 },
    { 0x681879ca, 98 },
    { 0xf23879cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x912879cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f879d3, 76 },
    { 0x4ef879d3, 83 },
    { 0, 0 },
    { 0x129879d6, 17 },
    { 0, 0 },
    { 0xab8879d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1879db, 95 },
    { 0xeb879dc, 61 },
    { 0xeda879dd, 33 },
    { 0, 0 },
    { 0xaf4879df, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55879e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d879f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e879f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d879f7, 23 },
    { 0, 0 },
    { 0xecc879f9, 98 },
    { 0x81d879fa, 23 },
    { 0x269879fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f87a04, 85 },
    { 0xa4987a05, 84 },
    { 0, 0 },
    { 0xeed87a07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a287a0a, 91 },
    { 0x3b887a0a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c287a10, 95 },
    { 0, 0 },
    { 0xff587a12, 23 },
    { 0xb4387a12, 99 },
    { 0xdde87a13, 23 },
    { 0x36887a15, 34 },
    { 0xd6987a13, 90 },
    { 0xe6e87a17, 90 },
    { 0x8d387a18, 52 },
    { 0xcbf87a18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c87a1d, 19 },
    { 0, 0 },
    { 0xd9b87a1f, 46 },
    { 0x72887a20, 68 },
    { 0x25c87a21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51987a27, 61 },
    { 0x59487a27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf887a2b, 90 },
    { 0, 0 },
    { 0xda187a2d, 88 },
    { 0xcda87a2e, 61 },
    { 0x97b87a2e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e387a32, 23 },
    { 0xcfd87a32, 99 },
    { 0xc6187a34, 43 },
    { 0x3fc87a34, 33 },
    { 0x21d87a36, 23 },
    { 0x76787a37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc87a3a, 61 },
    { 0x4b787a3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b87a3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2687a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61787a46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a687a4a, 90 },
    { 0x72187a4a, 90 },
    { 0xe5c87a4b, 61 },
    { 0, 0 },
    { 0x4be87a4e, 33 },
    { 0x94d87a4f, 99 },
    { 0x9e587a4e, 35 },
    { 0, 0 },
    { 0x2a987a52, 52 },
    { 0, 0 },
    { 0xd7f87a54, 23 },
    { 0x69587a55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0387a58, 95 },
    { 0, 0 },
    { 0xe3987a5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c287a5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6387a67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d887a6a, 95 },
    { 0x2f387a6a, 33 },
    { 0, 0 },
    { 0x80387a6d, 61 },
    { 0xb9d87a6e, 33 },
    { 0, 0 },
    { 0xc87a70, 33 },
    { 0xdcd87a70, 90 },
    { 0xced87a72, 87 },
    { 0x45387a72, 33 },
    { 0xa8887a74, 33 },
    { 0x9787a75, 90 },
    { 0x5d587a74, 91 },
    { 0x44787a76, 59 },
    { 0x7f787a78, 43 },
    { 0x16a87a79, 98 },
    { 0x11687a7a, 90 },
    { 0, 0 },
    { 0xac87a7c, 33 },
    { 0x3c987a7c, 90 },
    { 0x63e87a7e, 67 },
    { 0xc9687a7f, 23 },
    { 0x3e87a7e, 90 },
    { 0xca287a81, 78 },
    { 0x8e487a7e, 53 },
    { 0xcd287a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf87a87, 98 },
    { 0, 0 },
    { 0x78287a89, 55 },
    { 0x6e887a8a, 90 },
    { 0x19e87a8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef87a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2087a99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c87aa3, 23 },
    { 0, 0 },
    { 0x20987aa5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff87aa8, 81 },
    { 0xa4987aa9, 98 },
    { 0x1fd87aaa, 90 },
    { 0xd2387aab, 33 },
    { 0x33787aa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd87abe, 33 },
    { 0, 0 },
    { 0xac087ac0, 67 },
    { 0, 0 },
    { 0xa4b87ac2, 90 },
    { 0xa0687ac2, 34 },
    { 0, 0 },
    { 0xfdd87ac5, 98 },
    { 0x20387ac6, 34 },
    { 0x2a87ac5, 52 },
    { 0, 0 },
    { 0xb9c87ac9, 78 },
    { 0, 0 },
    { 0x39887acb, 61 },
    { 0x6d787acc, 17 },
    { 0x2e487acd, 90 },
    { 0x26a87acd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf87ad4, 85 },
    { 0, 0 },
    { 0xe8987ad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77287ada, 76 },
    { 0, 0 },
    { 0x53d87adc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87adf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e87ae3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a87ae6, 98 },
    { 0, 0 },
    { 0xb3087ae8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa187aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c787af2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c87aff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde787b03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a287b0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f87b0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b87b12, 77 },
    { 0, 0 },
    { 0x6da87b14, 76 },
    { 0xb5b87b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0287b1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb87b1f, 77 },
    { 0, 0 },
    { 0x65387b21, 43 },
    { 0xe2b87b22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38987b29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb787b2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25387b33, 93 },
    { 0xdad87b34, 46 },
    { 0, 0 },
    { 0xff787b36, 95 },
    { 0, 0 },
    { 0x5fc87b38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d87b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d87b47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f87b4b, 23 },
    { 0, 0 },
    { 0x38187b4d, 33 },
    { 0, 0 },
    { 0x5ff87b4f, 93 },
    { 0, 0 },
    { 0xade87b51, 61 },
    { 0x27f87b52, 98 },
    { 0x26a87b52, 88 },
    { 0xd3987b54, 46 },
    { 0x37487b52, 93 },
    { 0x90c87b56, 61 },
    { 0x56187b57, 90 },
    { 0xd7087b53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4887b5c, 33 },
    { 0xf787b5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf987b63, 64 },
    { 0, 0 },
    { 0xc9d87b65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea87b6b, 33 },
    { 0, 0 },
    { 0x40287b6d, 33 },
    { 0x76e87b6e, 90 },
    { 0x61487b6f, 37 },
    { 0x45e87b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb687b74, 55 },
    { 0, 0 },
    { 0xbd287b76, 98 },
    { 0xa2687b77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c87b7a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25587b83, 61 },
    { 0xed187b84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91087b89, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1487b8d, 98 },
    { 0xdf487b8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a187b91, 43 },
    { 0x31287b92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1087b95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55187b98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c87b9d, 33 },
    { 0xfa087b9e, 65 },
    { 0, 0 },
    { 0xd1d87ba0, 85 },
    { 0xf7d87ba1, 87 },
    { 0x6a087ba1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e87ba8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d87bab, 87 },
    { 0xa6f87bac, 98 },
    { 0xd6b87bac, 88 },
    { 0xdea87bae, 67 },
    { 0, 0 },
    { 0xd3887bb0, 33 },
    { 0xe7687bb1, 98 },
    { 0, 0 },
    { 0xca587bb3, 36 },
    { 0x7d387bb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc87bb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b87bbb, 43 },
    { 0xefd87bbb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x19587bbf, 23 },
    { 0x7787bbf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b687bc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c87bcc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d287bcf, 88 },
    { 0x63f87bcf, 87 },
    { 0x12887bd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec087bd8, 48 },
    { 0x59987bd8, 76 },
    { 0x99787bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56787bdd, 52 },
    { 0x6b087bde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be87be5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c87be9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2987bec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x28887bef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8987bf2, 46 },
    { 0, 0 },
    { 0x81987bf4, 81 },
    { 0x5ac87bf4, 95 },
    { 0, 0 },
    { 0xc2d87bf7, 72 },
    { 0xa2e87bf8, 23 },
    { 0, 0 },
    { 0x1ba87bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b87c00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58787c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc87c08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0187c0b, 90 },
    { 0, 0 },
    { 0xcd987c0d, 76 },
    { 0xa9987c0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x85787c11, 98 },
    { 0xf4c87c12, 61 },
    { 0xdee87c13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d287c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e87c1c, 95 },
    { 0x1e287c1d, 23 },
    { 0x7f587c1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d87c22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f87c2c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1987c31, 88 },
    { 0x1de87c32, 43 },
    { 0x90d87c33, 40 },
    { 0xe7b87c32, 23 },
    { 0x1b887c35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce87c39, 23 },
    { 0xc1787c3a, 55 },
    { 0x52b87c3b, 61 },
    { 0x7f87c3c, 23 },
    { 0, 0 },
    { 0x2b187c3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x73387c41, 46 },
    { 0xc2887c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f87c4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0987c4e, 95 },
    { 0x84087c4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf87c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce887c55, 77 },
    { 0x3bf87c56, 98 },
    { 0xc4c87c57, 90 },
    { 0x9fb87c58, 23 },
    { 0xb1c87c59, 34 },
    { 0x8eb87c5a, 84 },
    { 0x6db87c5b, 90 },
    { 0x16587c5b, 95 },
    { 0xa7e87c5d, 87 },
    { 0x9d587c5c, 52 },
    { 0xa0887c5b, 23 },
    { 0x38887c60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c87c64, 90 },
    { 0x3e487c65, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x74587c68, 55 },
    { 0xff987c69, 90 },
    { 0xc6387c6a, 85 },
    { 0, 0 },
    { 0x22287c6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec87c72, 90 },
    { 0x35187c73, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc87c77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35487c7d, 65 },
    { 0xa1b87c7d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b587c83, 33 },
    { 0, 0 },
    { 0x9fa87c85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x38987c88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea87c90, 65 },
    { 0, 0 },
    { 0x34f87c92, 33 },
    { 0, 0 },
    { 0x3c087c94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99687c98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad787c9e, 96 },
    { 0x29187c9f, 93 },
    { 0x9087ca0, 76 },
    { 0x49887ca1, 61 },
    { 0xeb487ca2, 90 },
    { 0x15587c9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da87ca9, 23 },
    { 0xdc287caa, 55 },
    { 0xb2d87caa, 33 },
    { 0x75387cac, 95 },
    { 0, 0 },
    { 0x3f787cae, 23 },
    { 0xb8387caf, 61 },
    { 0x63d87cb0, 66 },
    { 0xe6087cb0, 77 },
    { 0x93987cb2, 33 },
    { 0x74587cb3, 76 },
    { 0xae687cb4, 40 },
    { 0x98787cb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e587cb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba87cbc, 99 },
    { 0, 0 },
    { 0x5a187cbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e87cc7, 33 },
    { 0xff87cc8, 67 },
    { 0x5f487cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c87ccc, 78 },
    { 0x52987ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27287cd0, 74 },
    { 0xa6e87cd1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x10487cd4, 88 },
    { 0x18d87cd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f287cda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf587cde, 47 },
    { 0, 0 },
    { 0x56187ce0, 90 },
    { 0, 0 },
    { 0xafc87ce2, 90 },
    { 0xbe587ce3, 48 },
    { 0xab487ce4, 33 },
    { 0xe1787ce5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a87cea, 85 },
    { 0x36c87cea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4987cee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6587cf3, 76 },
    { 0, 0 },
    { 0xd9987cf5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf187cf9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8787cfc, 33 },
    { 0xe6a87cfd, 84 },
    { 0xd0e87cfc, 33 },
    { 0x98787cfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x61187d02, 46 },
    { 0, 0 },
    { 0x4b187d04, 23 },
    { 0x5d287d05, 76 },
    { 0x12787d06, 81 },
    { 0x61687d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad87d12, 76 },
    { 0x8b87d13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd87d21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af87d25, 52 },
    { 0, 0 },
    { 0x5be87d27, 100 },
    { 0, 0 },
    { 0x1e587d29, 46 },
    { 0, 0 },
    { 0x21a87d2b, 66 },
    { 0x52f87d2c, 23 },
    { 0x4f387d2b, 95 },
    { 0x3a587d2e, 93 },
    { 0x6a887d2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x51187d32, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7187d36, 90 },
    { 0, 0 },
    { 0x9a587d38, 90 },
    { 0x78f87d39, 95 },
    { 0, 0 },
    { 0x2af87d3b, 67 },
    { 0, 0 },
    { 0xa0287d3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb787d42, 23 },
    { 0xd8c87d43, 87 },
    { 0, 0 },
    { 0xcc87d45, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a87d4e, 46 },
    { 0, 0 },
    { 0xe5387d50, 52 },
    { 0x73687d50, 67 },
    { 0xdf687d52, 76 },
    { 0x5c787d50, 33 },
    { 0xdb487d54, 84 },
    { 0x7e387d54, 90 },
    { 0x55087d56, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71987d5d, 91 },
    { 0x8b087d5d, 33 },
    { 0x52287d5f, 34 },
    { 0xefb87d5f, 23 },
    { 0x89287d61, 95 },
    { 0xc8e87d60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d687d68, 98 },
    { 0xc3a87d69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c887d6d, 43 },
    { 0, 0 },
    { 0x4b987d6f, 90 },
    { 0x80987d70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76987d78, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe787d84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e87d87, 98 },
    { 0, 0 },
    { 0xb1f87d89, 33 },
    { 0x27087d8a, 77 },
    { 0, 0 },
    { 0x8fe87d8c, 96 },
    { 0x4e387d8d, 61 },
    { 0x73c87d8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f87d93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90287d98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed487d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36887da0, 90 },
    { 0xdc687da1, 67 },
    { 0, 0 },
    { 0x52f87da3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb887da7, 90 },
    { 0x6287da8, 23 },
    { 0x5a087da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb687dae, 34 },
    { 0x60087daf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea87db4, 69 },
    { 0, 0 },
    { 0x82287db6, 95 },
    { 0xbc687db7, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x46787dba, 95 },
    { 0xc3887dbb, 90 },
    { 0, 0 },
    { 0xd5a87dbd, 61 },
    { 0x5087dbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb187dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90787dc5, 83 },
    { 0xb6087dc6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39187dc9, 33 },
    { 0xf4087dca, 53 },
    { 0x76287dcb, 93 },
    { 0xb2087dcb, 98 },
    { 0x19b87dc9, 95 },
    { 0xcc087dc9, 90 },
    { 0xe3187dc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b987dd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b87dd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b87dd8, 33 },
    { 0, 0 },
    { 0xf4c87dda, 61 },
    { 0xb887dda, 100 },
    { 0x2bc87dda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc687ddf, 67 },
    { 0xb287ddf, 76 },
    { 0xcbc87de1, 34 },
    { 0x73587de2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d87de7, 98 },
    { 0xb5a87de8, 98 },
    { 0x9d487de9, 38 },
    { 0xb6187de9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f87ded, 34 },
    { 0x9b687dee, 33 },
    { 0xfe87def, 61 },
    { 0x8d387def, 34 },
    { 0x8b487dee, 34 },
    { 0x37787ded, 93 },
    { 0x62887df2, 90 },
    { 0x45b87dee, 93 },
    { 0x9e187df4, 65 },
    { 0xbe587df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a87e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7887e0b, 61 },
    { 0x49c87e0c, 61 },
    { 0xf4887e0d, 61 },
    { 0xb7187e0b, 55 },
    { 0x1be87e0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69887e12, 34 },
    { 0x7d87e13, 55 },
    { 0x4ad87e13, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb287e17, 86 },
    { 0x1e987e18, 46 },
    { 0xc7787e18, 87 },
    { 0x3dd87e1a, 23 },
    { 0x8fb87e1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22587e21, 34 },
    { 0x74d87e21, 61 },
    { 0, 0 },
    { 0xf8d87e24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4287e29, 23 },
    { 0x1a287e2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84187e30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb987e36, 34 },
    { 0x81787e37, 98 },
    { 0x61b87e38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f87e3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d487e43, 23 },
    { 0x95187e44, 78 },
    { 0, 0 },
    { 0xe7887e46, 34 },
    { 0, 0 },
    { 0x5ca87e48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3587e4f, 34 },
    { 0xab87e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb587e54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc987e5c, 55 },
    { 0, 0 },
    { 0x89187e5e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c887e61, 98 },
    { 0x83687e62, 61 },
    { 0x86c87e63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc487e66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c87e6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c87e6e, 76 },
    { 0x9da87e6f, 33 },
    { 0x75487e70, 33 },
    { 0xc1287e6e, 90 },
    { 0xfd887e72, 23 },
    { 0xd4b87e73, 33 },
    { 0, 0 },
    { 0x79987e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a987e78, 90 },
    { 0x12b87e79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c87e7e, 46 },
    { 0x91f87e7f, 90 },
    { 0x52a87e7f, 43 },
    { 0x58d87e80, 55 },
    { 0, 0 },
    { 0xd1387e83, 99 },
    { 0x6087e84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6987e87, 75 },
    { 0xea987e88, 80 },
    { 0xd4787e89, 33 },
    { 0x80387e87, 46 },
    { 0x51a87e8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df87e92, 93 },
    { 0x88a87e93, 95 },
    { 0x29987e94, 55 },
    { 0x90d87e95, 89 },
    { 0x4ce87e95, 90 },
    { 0x38f87e97, 95 },
    { 0, 0 },
    { 0x85e87e99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f87ea0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a87ea3, 34 },
    { 0xbee87ea4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f387ea7, 33 },
    { 0xf8687ea8, 61 },
    { 0x4b687ea9, 23 },
    { 0xcc887eaa, 90 },
    { 0x88987eaa, 61 },
    { 0xf887eac, 23 },
    { 0xf9887ead, 90 },
    { 0x47a87ead, 33 },
    { 0x86287eaa, 33 },
    { 0x3f387eb0, 76 },
    { 0xa1a87eb0, 76 },
    { 0xf4887eb1, 77 },
    { 0x46287eb3, 52 },
    { 0xd1787eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a87eb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d587ebd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a87ec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2287ec5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7087ecb, 46 },
    { 0x79787ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2987ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c87ed5, 61 },
    { 0xbdd87ed5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d87edb, 66 },
    { 0xdda87edc, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b87edf, 66 },
    { 0x1e487ee0, 98 },
    { 0xcf087ee1, 43 },
    { 0x7d587ee1, 23 },
    { 0x39687ee1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba87ee6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c87ee9, 23 },
    { 0xc5987ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b87eed, 61 },
    { 0, 0 },
    { 0x61f87eef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5387ef2, 93 },
    { 0, 0 },
    { 0x91c87ef4, 90 },
    { 0xef087ef4, 98 },
    { 0xf8c87ef4, 33 },
    { 0xd4187ef5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c987efb, 33 },
    { 0, 0 },
    { 0x4e187efd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1487f00, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6587f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7587f08, 76 },
    { 0xb3787f09, 61 },
    { 0x14887f08, 95 },
    { 0x31087f09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3687f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb087f12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78187f1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e887f23, 100 },
    { 0, 0 },
    { 0xe5687f25, 46 },
    { 0x27687f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2087f2d, 98 },
    { 0x1c187f2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9587f32, 90 },
    { 0xda887f33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37887f3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7387f3f, 33 },
    { 0x20687f40, 90 },
    { 0, 0 },
    { 0x5f787f42, 61 },
    { 0xc8a87f43, 61 },
    { 0x59287f44, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2887f49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78087f4d, 34 },
    { 0, 0 },
    { 0x77487f4f, 46 },
    { 0x5c787f4f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97387f59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb787f5d, 95 },
    { 0, 0 },
    { 0x21787f5f, 43 },
    { 0xb5487f60, 23 },
    { 0x2e387f61, 34 },
    { 0, 0 },
    { 0x33587f63, 34 },
    { 0xdd787f63, 55 },
    { 0xe2787f65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9787f6b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60987f71, 23 },
    { 0xccc87f71, 90 },
    { 0x47487f73, 76 },
    { 0, 0 },
    { 0x4ca87f75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f87f7a, 90 },
    { 0x94e87f7a, 95 },
    { 0xfdf87f7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc287f81, 61 },
    { 0xcbf87f81, 34 },
    { 0x43987f83, 66 },
    { 0x3a987f82, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98687f8a, 90 },
    { 0x38587f8b, 99 },
    { 0x82f87f8c, 61 },
    { 0xf987f8c, 23 },
    { 0x84987f8e, 95 },
    { 0x587f8f, 61 },
    { 0xd7b87f8f, 46 },
    { 0xbc087f91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0787f94, 90 },
    { 0xd3f87f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59987f99, 93 },
    { 0xbf087f99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b87f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d87fa1, 95 },
    { 0x57587fa1, 95 },
    { 0x47387fa3, 33 },
    { 0xa2987fa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd087fa8, 43 },
    { 0x33b87fa9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38887fad, 90 },
    { 0xdfe87fad, 87 },
    { 0xecf87faf, 95 },
    { 0x93687fb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a87fb3, 66 },
    { 0, 0 },
    { 0xfce87fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf87fb8, 33 },
    { 0, 0 },
    { 0x5ac87fba, 90 },
    { 0x2b887fbb, 90 },
    { 0x8dd87fbc, 81 },
    { 0xc9987fba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df87fc1, 23 },
    { 0xcfc87fc1, 52 },
    { 0xb1d87fc3, 98 },
    { 0, 0 },
    { 0x1e987fc5, 34 },
    { 0, 0 },
    { 0x48087fc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f87fcb, 98 },
    { 0, 0 },
    { 0x3c887fcd, 98 },
    { 0xfa687fce, 87 },
    { 0xd8087fce, 90 },
    { 0x65887fd0, 46 },
    { 0, 0 },
    { 0x86f87fd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41487fd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb87fd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a587fdc, 33 },
    { 0xf3b87fdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc87fe1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4187fea, 43 },
    { 0xd7287fea, 90 },
    { 0x76787fec, 23 },
    { 0x74187fed, 61 },
    { 0, 0 },
    { 0x72b87fef, 90 },
    { 0xf5787fef, 33 },
    { 0x49587ff1, 98 },
    { 0x1de87ff0, 85 },
    { 0x79d87ff3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86787ff6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3287ff9, 93 },
    { 0xa3f87ffa, 52 },
    { 0x29e87ffa, 91 },
    { 0x22887ffb, 88 },
    { 0x1de87ffb, 98 },
    { 0, 0 },
    { 0x2bb87fff, 76 },
    { 0xf5e87fff, 34 },
    { 0x8687fff, 100 },
    { 0, 0 },
    { 0xe7288003, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a088006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91988009, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb108800e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be88011, 61 },
    { 0xf4e88012, 77 },
    { 0xd4788012, 61 },
    { 0x4c388013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13388017, 55 },
    { 0x54e88018, 85 },
    { 0xeac88018, 90 },
    { 0x15a88018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2598801d, 67 },
    { 0x9b78801e, 33 },
    { 0, 0 },
    { 0x64088020, 33 },
    { 0, 0 },
    { 0xb0a88022, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b988027, 58 },
    { 0x5c388028, 43 },
    { 0xb5188028, 90 },
    { 0x14088029, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc88033, 33 },
    { 0xab88034, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a88042, 87 },
    { 0xced88043, 66 },
    { 0, 0 },
    { 0xa4188045, 33 },
    { 0x34988045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b68804b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f58804e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26788055, 98 },
    { 0x23088056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1888059, 66 },
    { 0xf5a88059, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a58805e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3988061, 61 },
    { 0xc3788062, 90 },
    { 0xcff88063, 43 },
    { 0xa888062, 99 },
    { 0x3f88065, 85 },
    { 0xb0988066, 33 },
    { 0x11a88067, 100 },
    { 0, 0 },
    { 0xd1488069, 67 },
    { 0, 0 },
    { 0x3508806b, 76 },
    { 0, 0 },
    { 0x9f18806d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f88072, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28807a, 23 },
    { 0x4e58807b, 95 },
    { 0xd6e8807c, 55 },
    { 0xa1e8807c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f88081, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f88085, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e8808d, 76 },
    { 0, 0 },
    { 0x50e8808f, 65 },
    { 0, 0 },
    { 0x31c88091, 90 },
    { 0x9b788092, 98 },
    { 0, 0 },
    { 0x19588094, 68 },
    { 0, 0 },
    { 0x1188096, 34 },
    { 0, 0 },
    { 0xc4288098, 90 },
    { 0xa4488099, 91 },
    { 0xcaa88098, 95 },
    { 0, 0 },
    { 0xaad8809c, 98 },
    { 0x2798809d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d880a1, 98 },
    { 0x978880a2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8880a5, 76 },
    { 0xed7880a6, 98 },
    { 0, 0 },
    { 0xb59880a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84880ad, 95 },
    { 0x1c1880ae, 61 },
    { 0xe97880ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5880b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x850880b8, 78 },
    { 0x1fb880b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23880bc, 82 },
    { 0x55a880bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c880c1, 33 },
    { 0x699880c2, 55 },
    { 0xd9880c3, 100 },
    { 0, 0 },
    { 0xd41880c5, 33 },
    { 0x226880c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71880c9, 90 },
    { 0xd02880ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129880cf, 98 },
    { 0x1bc880cf, 33 },
    { 0, 0 },
    { 0xce6880d2, 23 },
    { 0, 0 },
    { 0x325880d4, 43 },
    { 0xdc8880d5, 55 },
    { 0xe63880d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc880d9, 90 },
    { 0, 0 },
    { 0x60c880db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1880e4, 90 },
    { 0x6f880e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c880eb, 43 },
    { 0x48d880ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe880f0, 91 },
    { 0x226880f1, 95 },
    { 0xcaa880f2, 93 },
    { 0xe09880f1, 67 },
    { 0, 0 },
    { 0x238880f5, 61 },
    { 0xd83880f5, 33 },
    { 0x2f4880f5, 87 },
    { 0xb85880f5, 95 },
    { 0, 0 },
    { 0x8df880fa, 85 },
    { 0x688880fa, 84 },
    { 0x5dd880fc, 34 },
    { 0xff6880fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb88100, 68 },
    { 0x7888100, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e88105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78810b, 95 },
    { 0x2a08810c, 61 },
    { 0, 0 },
    { 0x8f28810e, 95 },
    { 0, 0 },
    { 0x79a88110, 99 },
    { 0x2f588110, 90 },
    { 0x57c88112, 52 },
    { 0xf2988113, 98 },
    { 0xe188112, 33 },
    { 0x2b88111, 33 },
    { 0x1e988112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b88126, 97 },
    { 0x2688127, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd968812a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65088132, 61 },
    { 0xecb88132, 100 },
    { 0x60788133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa198813a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4998813f, 33 },
    { 0xcb788140, 90 },
    { 0xd7588141, 33 },
    { 0, 0 },
    { 0xb6d88143, 95 },
    { 0, 0 },
    { 0x31888145, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf08814b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe148814e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1188151, 55 },
    { 0x60a88151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f88155, 65 },
    { 0x1e388156, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c8815c, 87 },
    { 0, 0 },
    { 0xade8815e, 99 },
    { 0, 0 },
    { 0x5d188160, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61788166, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e28816a, 55 },
    { 0xe378816a, 34 },
    { 0xd1b8816c, 90 },
    { 0x6938816d, 98 },
    { 0xb548816e, 61 },
    { 0, 0 },
    { 0xc1b88170, 93 },
    { 0x43e88171, 95 },
    { 0x6f388172, 69 },
    { 0x1d088173, 33 },
    { 0xc0388171, 61 },
    { 0x9bf88175, 33 },
    { 0x5e588176, 66 },
    { 0xdd188175, 34 },
    { 0xc488178, 23 },
    { 0x87a88175, 90 },
    { 0x51f88179, 98 },
    { 0xdb788172, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de8817f, 47 },
    { 0x53188180, 55 },
    { 0xd9c88181, 87 },
    { 0x5a88181, 33 },
    { 0xd6a88182, 76 },
    { 0x62a88181, 33 },
    { 0x65b88185, 99 },
    { 0x72988185, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33188193, 90 },
    { 0, 0 },
    { 0xb8188195, 76 },
    { 0x32c88196, 55 },
    { 0, 0 },
    { 0xca88198, 90 },
    { 0x5b788198, 98 },
    { 0xaa88819a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f881a1, 74 },
    { 0x1f3881a2, 98 },
    { 0x80e881a3, 33 },
    { 0x1a881a3, 90 },
    { 0x6a5881a4, 34 },
    { 0x8b6881a6, 98 },
    { 0x71e881a7, 90 },
    { 0xf41881a7, 78 },
    { 0x2cb881a9, 34 },
    { 0x858881a6, 46 },
    { 0x834881a7, 90 },
    { 0x24c881aa, 23 },
    { 0x4fa881ad, 45 },
    { 0xea6881ad, 77 },
    { 0x25d881ad, 23 },
    { 0x5dc881ae, 90 },
    { 0x7c1881aa, 95 },
    { 0x4e7881b0, 76 },
    { 0, 0 },
    { 0xf79881b4, 98 },
    { 0, 0 },
    { 0x863881b6, 98 },
    { 0xd71881b7, 91 },
    { 0xf49881b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759881c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884881c6, 33 },
    { 0xbee881c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c881ca, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b881d0, 95 },
    { 0xa56881d1, 98 },
    { 0x31c881d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a881d6, 87 },
    { 0x2fc881d7, 33 },
    { 0xe6b881d8, 99 },
    { 0x950881d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9881dd, 76 },
    { 0, 0 },
    { 0xd76881df, 61 },
    { 0xdc6881e0, 34 },
    { 0xaec881e1, 33 },
    { 0x45881e1, 61 },
    { 0x892881e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x75881e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e881e9, 93 },
    { 0xa89881e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0881ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef881f1, 33 },
    { 0xbfe881f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954881f9, 68 },
    { 0xe17881f9, 33 },
    { 0xc15881f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x642881fe, 43 },
    { 0, 0 },
    { 0x9e788200, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84188204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14388208, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3488213, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde988218, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe488220, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48388226, 98 },
    { 0xb6388227, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98822d, 95 },
    { 0x1b38822e, 90 },
    { 0x60c8822f, 91 },
    { 0x2bc8822d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a88235, 23 },
    { 0, 0 },
    { 0xf9e88237, 90 },
    { 0xa0088237, 76 },
    { 0xfe488239, 98 },
    { 0, 0 },
    { 0x42f8823b, 34 },
    { 0, 0 },
    { 0xce68823d, 95 },
    { 0x7f18823e, 61 },
    { 0xb818823d, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4188242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d88246, 33 },
    { 0, 0 },
    { 0xbdc88248, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e58824b, 90 },
    { 0x6178824b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d88252, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa88256, 95 },
    { 0xb7788256, 48 },
    { 0xdf688258, 77 },
    { 0x5f388257, 38 },
    { 0xf838825a, 91 },
    { 0, 0 },
    { 0x9608825c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a8825f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55388262, 66 },
    { 0, 0 },
    { 0x45588264, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f18826d, 55 },
    { 0x2628826e, 33 },
    { 0x5518826e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f88277, 61 },
    { 0, 0 },
    { 0xedb88279, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3358827d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f88282, 52 },
    { 0, 0 },
    { 0x35988284, 91 },
    { 0x51788285, 64 },
    { 0x13288284, 23 },
    { 0x67b88284, 98 },
    { 0x7a788288, 23 },
    { 0x1b688288, 59 },
    { 0x3c88289, 23 },
    { 0xe2d88286, 90 },
    { 0x19988284, 96 },
    { 0xabc88289, 99 },
    { 0xf8b88287, 46 },
    { 0xebc8828f, 36 },
    { 0xc7c88290, 91 },
    { 0x4118828f, 99 },
    { 0x95a8828d, 76 },
    { 0x76f8828c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d288299, 61 },
    { 0x8d08829a, 98 },
    { 0x7528829b, 43 },
    { 0x74f8829b, 23 },
    { 0, 0 },
    { 0xb0d8829e, 75 },
    { 0x5428829f, 34 },
    { 0x4ab8829e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2882a5, 61 },
    { 0xfbe882a5, 98 },
    { 0x168882a7, 100 },
    { 0x11a882a7, 90 },
    { 0x6b0882a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3882ac, 97 },
    { 0x568882ac, 95 },
    { 0x335882ae, 95 },
    { 0x5c2882af, 65 },
    { 0x628882ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x531882b3, 90 },
    { 0x438882b3, 91 },
    { 0xfeb882b3, 90 },
    { 0xae1882b4, 82 },
    { 0x475882b4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc882bb, 93 },
    { 0x7f7882bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539882c2, 90 },
    { 0, 0 },
    { 0x85882c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac882c7, 43 },
    { 0x61d882c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2882cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55882d3, 93 },
    { 0xdd882d4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230882da, 90 },
    { 0xffc882db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a882e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x338882e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685882e8, 75 },
    { 0x81c882e9, 33 },
    { 0xb54882e9, 23 },
    { 0x2d4882eb, 33 },
    { 0x7fb882eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420882f1, 17 },
    { 0xbcf882f1, 61 },
    { 0xbcc882f1, 90 },
    { 0x3da882f4, 55 },
    { 0xe1f882f5, 76 },
    { 0xa7b882f1, 33 },
    { 0xb99882f3, 34 },
    { 0xaf3882f8, 33 },
    { 0, 0 },
    { 0xcae882fa, 56 },
    { 0, 0 },
    { 0xdce882fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b688300, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72988309, 76 },
    { 0x47988309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a8830e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80888311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08831b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23188323, 23 },
    { 0, 0 },
    { 0xc3188325, 76 },
    { 0, 0 },
    { 0xb2588327, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab8832b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f8832e, 84 },
    { 0xea48832f, 95 },
    { 0xa058832e, 76 },
    { 0xbb688330, 95 },
    { 0xb08832e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e388337, 95 },
    { 0x88488338, 96 },
    { 0x39488339, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea8833e, 76 },
    { 0x66f8833f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45988343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53388347, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa38834b, 52 },
    { 0, 0 },
    { 0x2e8834d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca88350, 95 },
    { 0, 0 },
    { 0x43288352, 90 },
    { 0x24888353, 88 },
    { 0x24388352, 34 },
    { 0x94888355, 61 },
    { 0x90688356, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a78835e, 95 },
    { 0xa508835e, 61 },
    { 0x6e588360, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5488363, 61 },
    { 0x38f88363, 76 },
    { 0x45988365, 68 },
    { 0xac488366, 99 },
    { 0xab188366, 34 },
    { 0x8b288367, 81 },
    { 0, 0 },
    { 0xfa38836a, 67 },
    { 0x6c58836b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a88836e, 34 },
    { 0x4628836f, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97788376, 43 },
    { 0x26688377, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1298837a, 64 },
    { 0x88c8837b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1388381, 76 },
    { 0x62088382, 34 },
    { 0x85e88383, 95 },
    { 0x98788382, 61 },
    { 0, 0 },
    { 0xb0088386, 95 },
    { 0x25988387, 52 },
    { 0, 0 },
    { 0x6d988389, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb88394, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f288399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf8839f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0883a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70883a6, 43 },
    { 0, 0 },
    { 0x8f9883a8, 23 },
    { 0, 0 },
    { 0x4ac883aa, 95 },
    { 0, 0 },
    { 0x599883ac, 43 },
    { 0x702883ad, 76 },
    { 0x8b6883ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468883b5, 78 },
    { 0xf1a883b5, 23 },
    { 0xbdf883b7, 46 },
    { 0xc51883b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887883bd, 43 },
    { 0xe3a883bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796883c3, 23 },
    { 0, 0 },
    { 0xe9f883c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30883cc, 77 },
    { 0x315883cd, 65 },
    { 0xc7d883cd, 43 },
    { 0xc98883cd, 43 },
    { 0x8b3883cd, 76 },
    { 0x163883d1, 68 },
    { 0xbfb883d1, 87 },
    { 0x6d6883d2, 61 },
    { 0x8da883ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9883da, 33 },
    { 0, 0 },
    { 0xa97883dc, 61 },
    { 0, 0 },
    { 0x469883de, 91 },
    { 0x3d3883df, 98 },
    { 0x98a883e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x976883e3, 61 },
    { 0x5c9883e4, 66 },
    { 0xb07883e5, 67 },
    { 0xcf0883e6, 23 },
    { 0x15c883e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6883ed, 23 },
    { 0xfa4883ee, 77 },
    { 0x290883ef, 91 },
    { 0x4b883ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3883f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x745883fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11088405, 93 },
    { 0x9b88405, 33 },
    { 0, 0 },
    { 0x5d088408, 23 },
    { 0, 0 },
    { 0x8ab8840a, 95 },
    { 0, 0 },
    { 0x6f08840c, 23 },
    { 0x70f8840c, 100 },
    { 0x66b8840e, 52 },
    { 0x5368840f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c088414, 90 },
    { 0x52288415, 61 },
    { 0x20288416, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b88419, 69 },
    { 0xd178841a, 99 },
    { 0xc788419, 43 },
    { 0x7938841c, 95 },
    { 0x2288841d, 93 },
    { 0, 0 },
    { 0xd1a8841f, 98 },
    { 0xe4a8841f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb948842b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28842e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb988432, 23 },
    { 0x47188432, 61 },
    { 0x13188433, 61 },
    { 0xf1a88432, 46 },
    { 0, 0 },
    { 0x47e88437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c38843c, 78 },
    { 0, 0 },
    { 0x328843e, 33 },
    { 0x8278843f, 43 },
    { 0, 0 },
    { 0x59488441, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa988446, 23 },
    { 0xe988447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb8844b, 23 },
    { 0x6078844b, 61 },
    { 0x7788844d, 17 },
    { 0x5878844d, 93 },
    { 0x9be8844f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb588456, 90 },
    { 0, 0 },
    { 0xdbd88458, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd58845c, 40 },
    { 0xdaa8845d, 34 },
    { 0xa288845e, 43 },
    { 0x3158845f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e88467, 67 },
    { 0xfca88467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d8846b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f18846e, 61 },
    { 0, 0 },
    { 0x4f988470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd088473, 61 },
    { 0x84c88474, 90 },
    { 0x57888474, 98 },
    { 0xe0388474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb8847d, 43 },
    { 0, 0 },
    { 0xd538847f, 93 },
    { 0xda88847f, 98 },
    { 0x23f88481, 61 },
    { 0, 0 },
    { 0x42e88483, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2188486, 98 },
    { 0xf5188487, 90 },
    { 0x9c088486, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f8848d, 61 },
    { 0, 0 },
    { 0x61b8848f, 23 },
    { 0, 0 },
    { 0xe5288491, 95 },
    { 0, 0 },
    { 0xfc488493, 98 },
    { 0x5f288493, 87 },
    { 0x80a88495, 33 },
    { 0x8de88493, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8849b, 91 },
    { 0xbca8849c, 91 },
    { 0xc508849c, 90 },
    { 0xc438849e, 59 },
    { 0x1df8849f, 78 },
    { 0x1ea8849d, 66 },
    { 0xb218849d, 33 },
    { 0, 0 },
    { 0xfea884a3, 61 },
    { 0x81e884a4, 23 },
    { 0xb25884a5, 55 },
    { 0xd4d884a5, 90 },
    { 0xb2a884a7, 45 },
    { 0x96d884a6, 90 },
    { 0, 0 },
    { 0x8f7884aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98884ad, 98 },
    { 0xa05884ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a884b5, 33 },
    { 0x831884b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8884ba, 76 },
    { 0xe11884ba, 34 },
    { 0xff5884bc, 76 },
    { 0x8f3884bd, 68 },
    { 0xe00884bb, 93 },
    { 0x2fb884bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8884c3, 23 },
    { 0x80e884c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf884cb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8884d1, 76 },
    { 0xe02884d2, 55 },
    { 0xb80884d3, 17 },
    { 0x2b9884d2, 46 },
    { 0x2cc884d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81884d9, 44 },
    { 0, 0 },
    { 0x5ea884db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2884de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x744884e1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a884ec, 95 },
    { 0x78f884ec, 17 },
    { 0xb2a884ed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d884f3, 95 },
    { 0, 0 },
    { 0xeb9884f5, 23 },
    { 0xb4e884f6, 98 },
    { 0x8b1884f7, 95 },
    { 0x5dd884f6, 55 },
    { 0x334884f9, 95 },
    { 0xaed884f9, 17 },
    { 0x529884fb, 23 },
    { 0xc7a884f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d88500, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc88509, 33 },
    { 0xc918850a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3988850e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5488515, 43 },
    { 0x93588515, 33 },
    { 0xefe88517, 23 },
    { 0x95d88518, 43 },
    { 0, 0 },
    { 0x29d8851a, 78 },
    { 0xd1f8851a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b688521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad388527, 90 },
    { 0x90488528, 61 },
    { 0x18e88529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb238852f, 84 },
    { 0xa4588530, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x14988533, 88 },
    { 0x8e88534, 84 },
    { 0, 0 },
    { 0x7a588536, 66 },
    { 0xe0988536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb088853b, 95 },
    { 0, 0 },
    { 0x4c8853d, 34 },
    { 0x8a78853e, 90 },
    { 0x1e8853f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b88543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf988547, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df8854a, 76 },
    { 0, 0 },
    { 0xe308854c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb88552, 33 },
    { 0x99088553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1988556, 23 },
    { 0x19d88557, 46 },
    { 0xef888558, 33 },
    { 0x40188559, 90 },
    { 0, 0 },
    { 0xfd08855b, 61 },
    { 0x9dd8855c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1588560, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd88564, 55 },
    { 0x4488565, 46 },
    { 0, 0 },
    { 0xb4c88567, 33 },
    { 0x5fd88568, 43 },
    { 0xe088569, 61 },
    { 0xf1e88568, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b8856d, 33 },
    { 0xa798856e, 95 },
    { 0x10a8856e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca288573, 90 },
    { 0x1c488574, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84188578, 90 },
    { 0, 0 },
    { 0x48d8857a, 33 },
    { 0, 0 },
    { 0xb658857c, 85 },
    { 0, 0 },
    { 0x388857e, 98 },
    { 0, 0 },
    { 0x4da88580, 90 },
    { 0x4e388581, 100 },
    { 0x99788582, 33 },
    { 0x14d88580, 33 },
    { 0, 0 },
    { 0x40088585, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x54088588, 78 },
    { 0xf1488588, 23 },
    { 0x5f488589, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78858d, 33 },
    { 0xfa38858d, 23 },
    { 0x2648858d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff588592, 95 },
    { 0, 0 },
    { 0x80d88594, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a88598, 97 },
    { 0, 0 },
    { 0xb4b8859a, 55 },
    { 0xfea8859b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31885a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37885a8, 77 },
    { 0x95d885a9, 94 },
    { 0xb8c885aa, 34 },
    { 0xf4a885a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa885ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8885b2, 98 },
    { 0xd90885b2, 61 },
    { 0x6ce885b4, 85 },
    { 0, 0 },
    { 0x1d2885b6, 90 },
    { 0xfdc885b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0885bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a885be, 50 },
    { 0x820885be, 88 },
    { 0xee8885bf, 76 },
    { 0xc5b885be, 61 },
    { 0x775885c2, 76 },
    { 0x8ea885c3, 56 },
    { 0, 0 },
    { 0xf73885c5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f885c8, 82 },
    { 0, 0 },
    { 0x2ef885ca, 80 },
    { 0, 0 },
    { 0x8fd885cc, 76 },
    { 0, 0 },
    { 0xd19885ce, 93 },
    { 0xd40885cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a885d2, 33 },
    { 0, 0 },
    { 0x76d885d4, 98 },
    { 0x984885d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38885d8, 90 },
    { 0xcb4885d8, 76 },
    { 0xe99885d9, 33 },
    { 0x2a0885db, 98 },
    { 0x1bd885dc, 98 },
    { 0xc4c885d8, 90 },
    { 0x738885dd, 66 },
    { 0x3b8885de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264885e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece885ec, 33 },
    { 0, 0 },
    { 0x61f885ee, 90 },
    { 0x176885ef, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b885f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef885f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a885fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce88601, 76 },
    { 0xaaf88602, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4688605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f88608, 90 },
    { 0x73c88609, 61 },
    { 0x99e8860a, 95 },
    { 0x6dd8860a, 61 },
    { 0x7b58860b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96588611, 40 },
    { 0x3ff88612, 46 },
    { 0xfa788613, 90 },
    { 0x95d88612, 23 },
    { 0x71f88612, 91 },
    { 0x3ba88613, 46 },
    { 0x76f88617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2408861d, 90 },
    { 0x5458861d, 61 },
    { 0x8d68861f, 84 },
    { 0, 0 },
    { 0x21188621, 93 },
    { 0xbb388622, 40 },
    { 0x80d88623, 33 },
    { 0x83f88624, 38 },
    { 0, 0 },
    { 0xb4488626, 66 },
    { 0xfa188627, 33 },
    { 0xc5088628, 95 },
    { 0xb9888629, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e8862c, 80 },
    { 0x398862c, 95 },
    { 0, 0 },
    { 0x5a78862f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d88632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d88637, 23 },
    { 0x4388638, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f088644, 23 },
    { 0, 0 },
    { 0x1af88646, 33 },
    { 0x2a288647, 75 },
    { 0x6a88648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f8864d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b8865a, 33 },
    { 0xaac8865a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d88662, 33 },
    { 0x8f788663, 93 },
    { 0xe6388664, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf788667, 33 },
    { 0, 0 },
    { 0x9bf88669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d8866f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd88679, 91 },
    { 0, 0 },
    { 0xf148867b, 99 },
    { 0, 0 },
    { 0xb468867d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3188682, 76 },
    { 0xd0e88683, 33 },
    { 0xf3488684, 23 },
    { 0xc8588685, 90 },
    { 0, 0 },
    { 0x49488687, 76 },
    { 0x93388687, 55 },
    { 0xc88689, 98 },
    { 0x79e88687, 99 },
    { 0, 0 },
    { 0xac78868c, 77 },
    { 0xf7f8868c, 90 },
    { 0, 0 },
    { 0x73a8868f, 88 },
    { 0, 0 },
    { 0x9388691, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f88694, 23 },
    { 0x7bf88695, 93 },
    { 0x81188694, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2398869b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54886a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47886a5, 33 },
    { 0x996886a6, 98 },
    { 0x6af886a6, 90 },
    { 0x214886a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc886ad, 61 },
    { 0xa08886ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2886b3, 76 },
    { 0xe31886b3, 33 },
    { 0x9d0886b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b886ba, 61 },
    { 0, 0 },
    { 0x452886bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c886cb, 34 },
    { 0x225886cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101886d2, 93 },
    { 0x662886d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10886db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd886e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860886e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf886ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a886f1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc886f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3886f8, 90 },
    { 0x929886f8, 87 },
    { 0x22e886fa, 91 },
    { 0x81d886fb, 61 },
    { 0, 0 },
    { 0x3b6886fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13088700, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c088704, 34 },
    { 0xb4288705, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65488708, 17 },
    { 0x7c188709, 33 },
    { 0x71a88708, 95 },
    { 0x6168870b, 77 },
    { 0x53d8870b, 23 },
    { 0x4e48870d, 90 },
    { 0x51b8870b, 98 },
    { 0, 0 },
    { 0x43688710, 34 },
    { 0x1dc88710, 23 },
    { 0x84a88712, 33 },
    { 0x6a988713, 23 },
    { 0xbd788712, 69 },
    { 0x73f88712, 33 },
    { 0xe288712, 76 },
    { 0x2ed88713, 90 },
    { 0xaf488718, 46 },
    { 0xac988713, 33 },
    { 0xa408871a, 61 },
    { 0xd2c8871b, 85 },
    { 0xe78871c, 95 },
    { 0xf568871a, 98 },
    { 0x96588714, 66 },
    { 0x50d88717, 91 },
    { 0xee588720, 67 },
    { 0xc088720, 34 },
    { 0x33f88722, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d8872a, 90 },
    { 0x8bb8872a, 33 },
    { 0, 0 },
    { 0x7298872d, 88 },
    { 0xb348872e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b88732, 33 },
    { 0xa1d88733, 67 },
    { 0x5a188734, 23 },
    { 0, 0 },
    { 0xc4088736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac08873a, 90 },
    { 0x1928873b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc88748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf48874f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea588757, 33 },
    { 0x84588758, 61 },
    { 0x6f588759, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78875d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69788763, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7818876c, 61 },
    { 0x4518876c, 95 },
    { 0x7be8876e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82988778, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a8877b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b8877f, 34 },
    { 0x1b988780, 90 },
    { 0xb2f88781, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b88789, 90 },
    { 0xc2d88789, 95 },
    { 0xfb48878b, 23 },
    { 0x8eb8878c, 67 },
    { 0x40f8878d, 58 },
    { 0x87b8878e, 56 },
    { 0, 0 },
    { 0x97c88790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f88794, 61 },
    { 0x6d588795, 34 },
    { 0x5ee88796, 100 },
    { 0, 0 },
    { 0x5df88798, 90 },
    { 0, 0 },
    { 0x7f48879a, 65 },
    { 0xa5d8879b, 66 },
    { 0xfb38879b, 65 },
    { 0x2808879b, 91 },
    { 0xe888879b, 95 },
    { 0x59d8879c, 52 },
    { 0xb3b8879a, 55 },
    { 0, 0 },
    { 0x733887a2, 98 },
    { 0x501887a3, 33 },
    { 0x866887a2, 100 },
    { 0, 0 },
    { 0x41e887a6, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30887a9, 92 },
    { 0x729887a9, 66 },
    { 0xc7c887ab, 46 },
    { 0x271887ac, 84 },
    { 0xf0d887ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x171887b0, 98 },
    { 0x4a6887b1, 33 },
    { 0, 0 },
    { 0x973887b3, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9887b6, 90 },
    { 0xc73887b7, 90 },
    { 0xf7e887b8, 98 },
    { 0x46c887b9, 98 },
    { 0, 0 },
    { 0x3b2887bb, 76 },
    { 0x471887bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93887bf, 87 },
    { 0, 0 },
    { 0x9aa887c1, 33 },
    { 0xf60887c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d887c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da887ca, 34 },
    { 0x5d3887cb, 43 },
    { 0xb92887cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106887d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d887da, 95 },
    { 0xe3f887db, 61 },
    { 0x2cd887dc, 98 },
    { 0xd6a887dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d887e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11887e4, 61 },
    { 0, 0 },
    { 0x39c887e6, 61 },
    { 0xb6e887e7, 100 },
    { 0xace887e8, 90 },
    { 0x45d887e9, 23 },
    { 0xcd3887ea, 33 },
    { 0, 0 },
    { 0x7b3887ec, 95 },
    { 0x1a0887ed, 90 },
    { 0, 0 },
    { 0x92887ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca887f9, 87 },
    { 0, 0 },
    { 0x624887fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd88804, 84 },
    { 0, 0 },
    { 0x67688806, 84 },
    { 0x33e88807, 61 },
    { 0x2f488807, 91 },
    { 0xc7188806, 37 },
    { 0x8918880a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c8880d, 33 },
    { 0xb2c8880d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b88811, 65 },
    { 0x37988812, 76 },
    { 0xf9e88813, 23 },
    { 0x58488814, 90 },
    { 0xffd88812, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d288818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca98881f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e788822, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79288828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa888832, 98 },
    { 0xd2988833, 33 },
    { 0xaf088834, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8883f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd8884c, 77 },
    { 0xfa28884d, 43 },
    { 0, 0 },
    { 0xf898884f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e88855, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc788858, 33 },
    { 0x60b88859, 33 },
    { 0xdbf8885a, 76 },
    { 0, 0 },
    { 0xa4a8885c, 98 },
    { 0x5778885d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef88860, 91 },
    { 0, 0 },
    { 0x40288862, 64 },
    { 0xfa288863, 33 },
    { 0xbbc88863, 61 },
    { 0xf4788865, 34 },
    { 0xd1388866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458886b, 93 },
    { 0x6548886c, 77 },
    { 0xbce8886d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54488871, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef88875, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff88878, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6388881, 95 },
    { 0, 0 },
    { 0xb5b88883, 99 },
    { 0x4c588884, 33 },
    { 0, 0 },
    { 0x19c88886, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb8888c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b88891, 95 },
    { 0x2ea88892, 95 },
    { 0xc9588893, 95 },
    { 0, 0 },
    { 0x37788895, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff88898, 34 },
    { 0xe5188898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb58889d, 90 },
    { 0x8cd8889d, 55 },
    { 0x6b98889f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588888a4, 23 },
    { 0xff888a4, 95 },
    { 0x5c7888a4, 61 },
    { 0x63d888a6, 90 },
    { 0xf65888a4, 90 },
    { 0x6c888a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba888b3, 76 },
    { 0, 0 },
    { 0x5fd888b5, 90 },
    { 0x6e9888b5, 95 },
    { 0x4a3888b7, 95 },
    { 0xe14888b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd888bc, 90 },
    { 0x503888bc, 67 },
    { 0x83e888be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06888c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12888c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967888cb, 91 },
    { 0xb83888cc, 99 },
    { 0, 0 },
    { 0xca1888ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e888d4, 51 },
    { 0x30b888d5, 33 },
    { 0xa888d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f888da, 43 },
    { 0xdf1888db, 33 },
    { 0x120888dc, 88 },
    { 0x674888dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696888e3, 68 },
    { 0x885888e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54888ea, 89 },
    { 0, 0 },
    { 0xf78888ec, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b888f4, 61 },
    { 0, 0 },
    { 0x879888f6, 91 },
    { 0, 0 },
    { 0x920888f8, 76 },
    { 0, 0 },
    { 0xce7888fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f88903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1288906, 33 },
    { 0, 0 },
    { 0x8c588908, 90 },
    { 0, 0 },
    { 0x39e8890a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5228890d, 38 },
    { 0x70f8890d, 67 },
    { 0, 0 },
    { 0x2ce88910, 95 },
    { 0, 0 },
    { 0x18f88912, 85 },
    { 0xe5588912, 33 },
    { 0x4088912, 96 },
    { 0x98588915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43588922, 33 },
    { 0, 0 },
    { 0x43a88924, 98 },
    { 0x17888925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f88929, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9018892c, 93 },
    { 0, 0 },
    { 0xbab8892e, 55 },
    { 0, 0 },
    { 0x7f88930, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc88936, 90 },
    { 0, 0 },
    { 0x70e88938, 90 },
    { 0xd5b88939, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf568893d, 84 },
    { 0, 0 },
    { 0x9448893f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e88942, 91 },
    { 0xe7a88943, 23 },
    { 0xb7388944, 34 },
    { 0, 0 },
    { 0xee688946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f88949, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2058894f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d388953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51088959, 95 },
    { 0xf198895a, 87 },
    { 0xea788959, 61 },
    { 0, 0 },
    { 0xb5e8895d, 61 },
    { 0x90b8895e, 43 },
    { 0, 0 },
    { 0x5d088960, 90 },
    { 0x7cb88961, 93 },
    { 0xad988962, 95 },
    { 0x60a88960, 95 },
    { 0xa5688964, 98 },
    { 0xb4f88965, 87 },
    { 0x99388966, 93 },
    { 0x28188967, 34 },
    { 0xded88962, 36 },
    { 0xb8a88969, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b788976, 46 },
    { 0, 0 },
    { 0xc2988978, 91 },
    { 0x69788979, 90 },
    { 0x79888979, 34 },
    { 0x4618897b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f8897f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce88983, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9488988, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148898c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd488991, 61 },
    { 0x40688992, 78 },
    { 0x96188993, 66 },
    { 0x86588994, 23 },
    { 0x52888995, 17 },
    { 0x14088996, 90 },
    { 0x87c88997, 99 },
    { 0x9bd88998, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc28899b, 94 },
    { 0xe948899b, 61 },
    { 0, 0 },
    { 0xfec8899e, 85 },
    { 0, 0 },
    { 0xd53889a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96889b9, 61 },
    { 0xf24889b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99889bd, 61 },
    { 0xb37889be, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565889c4, 34 },
    { 0x5b8889c5, 75 },
    { 0, 0 },
    { 0x162889c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8889cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c889d7, 95 },
    { 0xd7a889d8, 77 },
    { 0x145889d7, 56 },
    { 0, 0 },
    { 0xaea889db, 95 },
    { 0, 0 },
    { 0x130889dd, 34 },
    { 0xe19889de, 98 },
    { 0, 0 },
    { 0xeca889e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88889e4, 83 },
    { 0, 0 },
    { 0xf31889e6, 55 },
    { 0x87b889e6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d889ed, 66 },
    { 0xc89889ee, 33 },
    { 0xeb3889ee, 34 },
    { 0, 0 },
    { 0xe28889f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501889f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29889fa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x154889fd, 98 },
    { 0, 0 },
    { 0x48889ff, 23 },
    { 0x5cd88a00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d88a03, 90 },
    { 0x58388a04, 81 },
    { 0xbef88a04, 61 },
    { 0xe9b88a06, 99 },
    { 0xf2588a05, 91 },
    { 0x6ed88a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf88a0b, 55 },
    { 0x16688a0c, 61 },
    { 0x63f88a0d, 23 },
    { 0x34f88a0e, 91 },
    { 0x64a88a0f, 8 },
    { 0x1088a10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b988a15, 55 },
    { 0x9de88a15, 93 },
    { 0, 0 },
    { 0x5e688a18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73088a1d, 88 },
    { 0x51088a1e, 33 },
    { 0xf8e88a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e88a26, 52 },
    { 0x16988a26, 34 },
    { 0, 0 },
    { 0x70888a29, 95 },
    { 0xb8a88a29, 90 },
    { 0, 0 },
    { 0xc2988a2c, 61 },
    { 0x7b88a2c, 98 },
    { 0xb1288a2e, 76 },
    { 0x61d88a2e, 61 },
    { 0x9a688a30, 100 },
    { 0x86588a31, 69 },
    { 0, 0 },
    { 0xc2c88a33, 90 },
    { 0x7de88a34, 33 },
    { 0x7a988a35, 61 },
    { 0x5a188a36, 95 },
    { 0, 0 },
    { 0xb2c88a38, 95 },
    { 0, 0 },
    { 0xe7688a3a, 91 },
    { 0x63988a3a, 23 },
    { 0x45388a3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d88a41, 90 },
    { 0x44688a41, 46 },
    { 0x17e88a43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a988a47, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b88a4b, 95 },
    { 0, 0 },
    { 0x82588a4d, 23 },
    { 0, 0 },
    { 0xae088a4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e88a52, 87 },
    { 0x5e588a52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3688a59, 33 },
    { 0, 0 },
    { 0xfa988a5b, 90 },
    { 0, 0 },
    { 0x56e88a5d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f88a64, 33 },
    { 0xb7288a65, 90 },
    { 0, 0 },
    { 0x55288a67, 61 },
    { 0x67988a67, 23 },
    { 0x49a88a68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d788a6d, 55 },
    { 0, 0 },
    { 0x9288a6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25688a72, 83 },
    { 0, 0 },
    { 0x71288a74, 91 },
    { 0x7ee88a74, 52 },
    { 0xc6d88a75, 52 },
    { 0, 0 },
    { 0xa8288a78, 100 },
    { 0x32888a79, 76 },
    { 0x5e188a78, 76 },
    { 0x25788a79, 61 },
    { 0x2c488a7c, 55 },
    { 0x72288a7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3088a82, 43 },
    { 0, 0 },
    { 0xe0588a84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c88a89, 66 },
    { 0x90f88a8a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e88a8d, 61 },
    { 0x89c88a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af88a96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a088a9b, 98 },
    { 0, 0 },
    { 0x59d88a9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c988aa2, 90 },
    { 0x3c088aa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49288aa8, 97 },
    { 0, 0 },
    { 0xb7288aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3088aaf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f88ab2, 23 },
    { 0x2b488ab3, 61 },
    { 0x10388ab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e88ab9, 95 },
    { 0x39988ab9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b88abd, 23 },
    { 0xe8888abe, 33 },
    { 0x5dc88abd, 61 },
    { 0xaae88ac0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b788ac4, 93 },
    { 0x93788ac5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71888aca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df88acd, 92 },
    { 0x26f88acd, 23 },
    { 0x41588acf, 98 },
    { 0xe3988ad0, 55 },
    { 0xfaa88ad1, 66 },
    { 0xd5a88ad2, 87 },
    { 0x36388ad2, 40 },
    { 0x3be88ad4, 66 },
    { 0x87288acd, 23 },
    { 0, 0 },
    { 0xb7e88ad7, 33 },
    { 0xa6288ad8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88adb, 90 },
    { 0, 0 },
    { 0x17b88add, 55 },
    { 0x41488ade, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2788ae3, 34 },
    { 0x66488ae4, 33 },
    { 0xe3488ae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14288aec, 87 },
    { 0x53388aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3688af0, 33 },
    { 0xc5b88af1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x53888af4, 23 },
    { 0x14588af5, 61 },
    { 0x82e88af6, 84 },
    { 0xe4888af4, 90 },
    { 0xf188af5, 95 },
    { 0x6e388af4, 90 },
    { 0, 0 },
    { 0x56b88afb, 61 },
    { 0, 0 },
    { 0x31b88afd, 92 },
    { 0x22588afe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52188b01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b88b04, 76 },
    { 0x15e88b05, 93 },
    { 0x62888b05, 61 },
    { 0x8fd88b04, 90 },
    { 0x16788b08, 95 },
    { 0xfab88b07, 75 },
    { 0xb5888b09, 33 },
    { 0x82188b0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa188b0e, 61 },
    { 0x69688b0f, 99 },
    { 0, 0 },
    { 0x56b88b11, 33 },
    { 0x59788b11, 90 },
    { 0xd5b88b13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa588b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70888b1f, 90 },
    { 0xb3d88b1f, 46 },
    { 0x89d88b21, 91 },
    { 0x82388b21, 90 },
    { 0xdf188b1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd88b29, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x50988b2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd588b33, 61 },
    { 0x15688b34, 90 },
    { 0xcc988b35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52488b3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc088b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c588b43, 95 },
    { 0xa6b88b44, 95 },
    { 0, 0 },
    { 0xfae88b46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12588b4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda488b4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81188b51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5888b57, 90 },
    { 0x43d88b57, 99 },
    { 0x6b088b59, 95 },
    { 0xffd88b58, 90 },
    { 0x5bd88b5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1888b5e, 61 },
    { 0xd9288b5f, 95 },
    { 0x9b788b60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd88b63, 99 },
    { 0, 0 },
    { 0xf388b65, 76 },
    { 0, 0 },
    { 0xa5588b67, 34 },
    { 0x7f88b67, 98 },
    { 0x18188b69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee088b6d, 76 },
    { 0xf8e88b6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff288b72, 90 },
    { 0x8e988b72, 61 },
    { 0xbcc88b74, 90 },
    { 0x2a988b75, 95 },
    { 0x96188b76, 78 },
    { 0xe0b88b77, 78 },
    { 0x67a88b74, 34 },
    { 0xc4188b79, 23 },
    { 0x45988b79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe388b7d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c88b84, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee88b8a, 61 },
    { 0, 0 },
    { 0x5288b8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8788b8f, 90 },
    { 0x93788b90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f688b95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25988b98, 61 },
    { 0x13388b99, 90 },
    { 0, 0 },
    { 0x4688b9b, 23 },
    { 0x9e388b9c, 33 },
    { 0xbba88b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b88ba0, 33 },
    { 0x65e88ba0, 90 },
    { 0x10e88ba2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf588ba5, 98 },
    { 0, 0 },
    { 0x18688ba7, 33 },
    { 0x8ca88ba7, 23 },
    { 0xb4688ba9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c288bac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3688baf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4688bb4, 23 },
    { 0, 0 },
    { 0x15788bb6, 90 },
    { 0x4c988bb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f88bba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f88bbe, 95 },
    { 0, 0 },
    { 0xf0388bc0, 91 },
    { 0x35a88bc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc88bc4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf88bc7, 34 },
    { 0x91888bc7, 33 },
    { 0xc2988bc8, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a988bcc, 90 },
    { 0x93988bcc, 33 },
    { 0xf7f88bce, 90 },
    { 0, 0 },
    { 0x2688bd0, 38 },
    { 0xe9488bd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a88bd4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b88bd7, 99 },
    { 0xe7b88bd8, 33 },
    { 0x6ff88bd9, 34 },
    { 0, 0 },
    { 0x1d388bdb, 77 },
    { 0, 0 },
    { 0x37988bdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a688be0, 52 },
    { 0xf4388be0, 22 },
    { 0x60288be2, 33 },
    { 0xc3688be3, 81 },
    { 0x6a588be3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb88be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4288bec, 23 },
    { 0x4e988bed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4388bf8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba388bfc, 95 },
    { 0xe3d88bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97788c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86788c03, 43 },
    { 0xfdd88c03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a88c07, 95 },
    { 0, 0 },
    { 0x43388c09, 61 },
    { 0x62e88c0a, 51 },
    { 0x3a988c0b, 99 },
    { 0x29d88c0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de88c0f, 55 },
    { 0x99e88c0f, 90 },
    { 0x64988c11, 98 },
    { 0x9a288c12, 93 },
    { 0x78688c10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa88c16, 98 },
    { 0x2fe88c16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d88c1b, 95 },
    { 0xc3e88c1c, 68 },
    { 0, 0 },
    { 0xca488c1e, 90 },
    { 0xd9988c1e, 33 },
    { 0x9588c1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51388c26, 93 },
    { 0x57688c26, 33 },
    { 0, 0 },
    { 0x52e88c29, 17 },
    { 0xccb88c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1888c33, 34 },
    { 0x9df88c34, 55 },
    { 0x54e88c35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e988c39, 76 },
    { 0x3f088c3a, 91 },
    { 0x87688c3b, 98 },
    { 0xbb788c3b, 95 },
    { 0xd9e88c3d, 98 },
    { 0x2f888c3d, 98 },
    { 0x3d888c3d, 84 },
    { 0xdf288c40, 55 },
    { 0xbca88c41, 33 },
    { 0, 0 },
    { 0xae488c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94088c46, 23 },
    { 0x8d988c46, 34 },
    { 0xf0588c48, 23 },
    { 0x58788c48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c88c4c, 100 },
    { 0xc4888c4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d88c50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee88c54, 76 },
    { 0xaa688c55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7488c59, 95 },
    { 0xd1e88c59, 90 },
    { 0x87188c5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5188c5f, 52 },
    { 0x71788c60, 33 },
    { 0, 0 },
    { 0xb0888c62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56488c65, 95 },
    { 0x52c88c66, 98 },
    { 0x55d88c66, 90 },
    { 0x94a88c68, 90 },
    { 0, 0 },
    { 0x6d588c6a, 87 },
    { 0xd3588c6a, 99 },
    { 0xe9f88c6a, 98 },
    { 0x6d588c6c, 55 },
    { 0x40688c6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a88c72, 98 },
    { 0x3b88c73, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac88c78, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2488c7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1988c7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60288c82, 90 },
    { 0x1a188c83, 90 },
    { 0xcfe88c83, 55 },
    { 0xe9f88c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46988c8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87288c92, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5288c96, 52 },
    { 0xd4088c97, 61 },
    { 0xb0788c96, 61 },
    { 0x16188c98, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5188c9c, 33 },
    { 0x91e88c9d, 61 },
    { 0x32088c9d, 98 },
    { 0x79488c9e, 70 },
    { 0x49588ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b88caa, 95 },
    { 0, 0 },
    { 0x15c88cac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a88caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c388cb6, 46 },
    { 0x6a488cb6, 52 },
    { 0x90088cb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14488cc1, 64 },
    { 0, 0 },
    { 0xeef88cc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65288cc6, 90 },
    { 0x67188cc7, 85 },
    { 0x29b88cc7, 98 },
    { 0x57d88cc7, 33 },
    { 0xdb288cc8, 95 },
    { 0x61088ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba888cd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea88cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6388cdd, 98 },
    { 0xc3688cde, 23 },
    { 0x21988cdf, 76 },
    { 0x82d88ce0, 93 },
    { 0x20e88cdd, 61 },
    { 0x2f988ce0, 90 },
    { 0xe0888ce3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75088cf1, 69 },
    { 0, 0 },
    { 0xace88cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f88cf6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44688cfb, 33 },
    { 0x7c188cfc, 23 },
    { 0x59c88cfd, 95 },
    { 0x95c88cfe, 90 },
    { 0, 0 },
    { 0x59588d00, 81 },
    { 0x17488d01, 23 },
    { 0xd4e88d01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb488d06, 61 },
    { 0x80f88d06, 61 },
    { 0xe2988d07, 68 },
    { 0x7af88d06, 34 },
    { 0xd0688d0a, 90 },
    { 0x98688d0a, 61 },
    { 0x7dd88d0c, 99 },
    { 0x2d88d0d, 76 },
    { 0x21088d0d, 22 },
    { 0x11588d0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c788d12, 33 },
    { 0, 0 },
    { 0xee688d14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f388d1a, 33 },
    { 0x3cc88d1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf88d1f, 98 },
    { 0, 0 },
    { 0x9de88d21, 33 },
    { 0xd4388d22, 90 },
    { 0x7c788d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83188d27, 100 },
    { 0x2cd88d27, 33 },
    { 0x2a088d28, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b88d32, 43 },
    { 0, 0 },
    { 0x5788d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29288d3a, 61 },
    { 0x26a88d3b, 33 },
    { 0xb0c88d3c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae88d3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b88d42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb688d4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d88d51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89288d57, 91 },
    { 0x38688d58, 61 },
    { 0x7dc88d58, 61 },
    { 0x6f788d59, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50788d62, 38 },
    { 0x28588d63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc588d66, 88 },
    { 0x81088d67, 90 },
    { 0, 0 },
    { 0xc9888d69, 61 },
    { 0, 0 },
    { 0xaf888d6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21288d71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11188d76, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x90688d79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc88d7c, 90 },
    { 0x9da88d7c, 78 },
    { 0xa0988d7d, 23 },
    { 0, 0 },
    { 0x89488d80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x86588d83, 46 },
    { 0xfed88d84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab88d88, 90 },
    { 0xcf488d89, 90 },
    { 0, 0 },
    { 0x1a88d8b, 85 },
    { 0xde788d8c, 99 },
    { 0x43888d8c, 55 },
    { 0x6d988d8d, 90 },
    { 0, 0 },
    { 0x8da88d90, 33 },
    { 0x77d88d90, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f88d97, 92 },
    { 0, 0 },
    { 0xc188d99, 91 },
    { 0xf3188d99, 4 },
    { 0xdd388d99, 99 },
    { 0, 0 },
    { 0x83888d9d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe688da0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49388da7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4388dab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee88dae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92288db2, 85 },
    { 0, 0 },
    { 0x54b88db4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7788db8, 66 },
    { 0xbf188db9, 77 },
    { 0x1e188dba, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x64888dbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80788dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed88dc8, 90 },
    { 0xb3288dc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17288dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74688dd7, 33 },
    { 0x3f288dd7, 33 },
    { 0xcd488dd9, 90 },
    { 0x32f88dd9, 95 },
    { 0, 0 },
    { 0x5088ddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e88ddf, 33 },
    { 0, 0 },
    { 0x84188de1, 55 },
    { 0xba788de2, 90 },
    { 0x57f88de3, 55 },
    { 0xd4b88de4, 90 },
    { 0x97e88de5, 33 },
    { 0, 0 },
    { 0x54888de7, 52 },
    { 0, 0 },
    { 0x5ce88de9, 90 },
    { 0x46288de9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b88ded, 95 },
    { 0xa7a88dee, 46 },
    { 0xb0988dee, 99 },
    { 0, 0 },
    { 0xfc588df1, 52 },
    { 0, 0 },
    { 0xbb888df3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81288df8, 98 },
    { 0xc9488df8, 90 },
    { 0x60388dfa, 99 },
    { 0xa7b88dfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a88dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d288e01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f88e07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12988e0a, 90 },
    { 0x6c988e0a, 75 },
    { 0, 0 },
    { 0x68388e0d, 33 },
    { 0x49188e0e, 90 },
    { 0xb6b88e0e, 66 },
    { 0xab788e10, 61 },
    { 0x54688e11, 90 },
    { 0x12388e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e788e17, 33 },
    { 0x40f88e18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b88e1f, 26 },
    { 0, 0 },
    { 0x68288e21, 43 },
    { 0x72a88e22, 34 },
    { 0x44388e23, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x50388e26, 33 },
    { 0x8ee88e27, 24 },
    { 0, 0 },
    { 0xf4688e29, 34 },
    { 0, 0 },
    { 0xa2488e2b, 90 },
    { 0, 0 },
    { 0xd6a88e2d, 66 },
    { 0x46a88e2d, 76 },
    { 0xe7088e2e, 46 },
    { 0x95d88e30, 43 },
    { 0, 0 },
    { 0x16088e32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e88e35, 75 },
    { 0xcb188e36, 91 },
    { 0, 0 },
    { 0x85f88e38, 95 },
    { 0x5f388e38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38388e3d, 78 },
    { 0x67288e3e, 33 },
    { 0x9dc88e3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a88e45, 76 },
    { 0x68288e46, 43 },
    { 0xac88e47, 33 },
    { 0xb9d88e48, 84 },
    { 0, 0 },
    { 0x74288e4a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce688e4f, 100 },
    { 0x24d88e50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0388e54, 95 },
    { 0, 0 },
    { 0x54788e56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa88e5a, 76 },
    { 0, 0 },
    { 0xe2288e5c, 98 },
    { 0x4c488e5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec88e60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc88e64, 33 },
    { 0, 0 },
    { 0x20688e66, 61 },
    { 0, 0 },
    { 0x32388e68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x35588e6b, 33 },
    { 0, 0 },
    { 0xde688e6d, 33 },
    { 0x5a988e6e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb388e73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91988e76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1588e7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a188e87, 95 },
    { 0xb0e88e88, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97988e90, 100 },
    { 0x30388e91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c088e97, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf88e9a, 77 },
    { 0x48688e9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44288ea1, 78 },
    { 0xc9f88ea2, 93 },
    { 0, 0 },
    { 0x45188ea4, 90 },
    { 0xdf688ea5, 17 },
    { 0, 0 },
    { 0xb1788ea7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x55088eaa, 85 },
    { 0, 0 },
    { 0x46088eac, 23 },
    { 0xe688ead, 44 },
    { 0, 0 },
    { 0xd0288eaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d88eb5, 90 },
    { 0x32f88eb6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a888ebb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc188ec0, 90 },
    { 0, 0 },
    { 0xf0688ec2, 55 },
    { 0x4988ec2, 95 },
    { 0x59688ec2, 53 },
    { 0xac388ec5, 55 },
    { 0x46c88ec2, 67 },
    { 0x7c888ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16988eca, 55 },
    { 0xcd88ecb, 52 },
    { 0x9ad88ecc, 61 },
    { 0, 0 },
    { 0xa8b88ece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e88ed1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33388ee1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4788ee8, 61 },
    { 0x28e88ee9, 90 },
    { 0x21288ee9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f88eed, 55 },
    { 0xd1788eee, 98 },
    { 0x6c388eef, 66 },
    { 0x3f288ef0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3488ef7, 90 },
    { 0x86388ef8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31788f00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c88f05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb88f08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0788f0c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99388f10, 40 },
    { 0, 0 },
    { 0x5188f12, 91 },
    { 0, 0 },
    { 0xcb488f14, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e188f18, 90 },
    { 0, 0 },
    { 0x13788f1a, 95 },
    { 0, 0 },
    { 0xbe088f1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f088f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d88f24, 87 },
    { 0x33388f25, 98 },
    { 0x9d688f26, 23 },
    { 0x28e88f27, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c588f2b, 90 },
    { 0xbdc88f2c, 61 },
    { 0x9d988f2c, 90 },
    { 0x51d88f2d, 98 },
    { 0x4a88f2f, 46 },
    { 0xe2b88f30, 91 },
    { 0xc7c88f30, 43 },
    { 0x65f88f32, 70 },
    { 0x48a88f30, 61 },
    { 0, 0 },
    { 0xf588f35, 23 },
    { 0xd2688f36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9488f3a, 43 },
    { 0x14288f3b, 33 },
    { 0, 0 },
    { 0x9cf88f3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40988f41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b88f44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae188f49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa88f51, 95 },
    { 0x89788f52, 55 },
    { 0, 0 },
    { 0x12388f54, 81 },
    { 0x94088f55, 76 },
    { 0x76f88f56, 55 },
    { 0x64b88f57, 33 },
    { 0, 0 },
    { 0x1aa88f59, 21 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83488f5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad88f6a, 61 },
    { 0, 0 },
    { 0x9cd88f6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5788f72, 61 },
    { 0xcf688f72, 90 },
    { 0xb4388f72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb188f7f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c788f83, 43 },
    { 0xb1088f83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1388f87, 98 },
    { 0x76a88f88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91688f8c, 33 },
    { 0x7e588f8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84688f94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a88f97, 61 },
    { 0x4e288f98, 55 },
    { 0, 0 },
    { 0x1ef88f9a, 43 },
    { 0xf788f9a, 95 },
    { 0x46488f9c, 76 },
    { 0, 0 },
    { 0xdeb88f9e, 84 },
    { 0xf8888f9e, 60 },
    { 0x46d88fa0, 90 },
    { 0, 0 },
    { 0x20688fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16988fb1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e88fb4, 98 },
    { 0x43988fb5, 17 },
    { 0xdc588fb6, 40 },
    { 0x73a88fb7, 90 },
    { 0xa5988fb7, 61 },
    { 0x29a88fb9, 66 },
    { 0x1e688fba, 43 },
    { 0x18e88fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a88fc0, 88 },
    { 0, 0 },
    { 0x47188fc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5888fcc, 90 },
    { 0xbd288fcc, 33 },
    { 0x3c088fcc, 23 },
    { 0xab488fcc, 90 },
    { 0x9a588fce, 76 },
    { 0xd8f88fd1, 52 },
    { 0x34988fd2, 93 },
    { 0xa4888fce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d788fd6, 23 },
    { 0xbf288fd7, 55 },
    { 0x3da88fd6, 34 },
    { 0x81588fd9, 98 },
    { 0x23688fd9, 80 },
    { 0xa4f88fdb, 23 },
    { 0xa6b88fdc, 23 },
    { 0xce888fdd, 23 },
    { 0x4cd88fde, 55 },
    { 0x38988fdd, 91 },
    { 0xe1d88fd6, 33 },
    { 0xb0b88fd6, 91 },
    { 0, 0 },
    { 0x3d688fe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d488fe7, 34 },
    { 0x83a88fe8, 23 },
    { 0xc0888fe9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae88fec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39288ff0, 98 },
    { 0x56088ff1, 33 },
    { 0x6588ff2, 46 },
    { 0xfff88ff2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d88ff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42488ffa, 100 },
    { 0x55488ffb, 61 },
    { 0, 0 },
    { 0xaf388ffd, 61 },
    { 0x91388ffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7389002, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f89007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b8900a, 52 },
    { 0, 0 },
    { 0xd78900c, 78 },
    { 0, 0 },
    { 0xe2e8900e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b89018, 46 },
    { 0xabc89019, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8901d, 61 },
    { 0, 0 },
    { 0x9e28901f, 69 },
    { 0xe208901f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f89026, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff98902b, 98 },
    { 0, 0 },
    { 0xf0c8902d, 43 },
    { 0x7af8902d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb789038, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8598903b, 61 },
    { 0x5528903c, 88 },
    { 0xb5f8903d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf489040, 90 },
    { 0xe3389041, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96189049, 33 },
    { 0, 0 },
    { 0x32a8904b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe248904f, 87 },
    { 0x48b8904f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd89054, 90 },
    { 0xc1a89055, 23 },
    { 0xfb589055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a89059, 33 },
    { 0x2508905a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1128905f, 61 },
    { 0, 0 },
    { 0x6a289061, 90 },
    { 0x8ea89061, 64 },
    { 0x61089063, 98 },
    { 0x9a89064, 99 },
    { 0xdc189065, 23 },
    { 0xd3189063, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd89073, 88 },
    { 0x58789074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf448907a, 60 },
    { 0x7f98907b, 43 },
    { 0, 0 },
    { 0xa2a8907d, 61 },
    { 0xb268907d, 65 },
    { 0x2a18907e, 17 },
    { 0, 0 },
    { 0xfab89081, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x58889084, 90 },
    { 0xff089084, 85 },
    { 0x73c89086, 69 },
    { 0x5d189084, 98 },
    { 0, 0 },
    { 0x5c289089, 98 },
    { 0x5d489089, 24 },
    { 0, 0 },
    { 0xc248908c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c989095, 100 },
    { 0x92a89096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e38909b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f8909e, 78 },
    { 0xe3b8909f, 95 },
    { 0xfc68909e, 91 },
    { 0x803890a1, 68 },
    { 0x4218909e, 83 },
    { 0x2b890a0, 33 },
    { 0x899890a4, 61 },
    { 0x9b6890a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658890aa, 99 },
    { 0x3a7890ab, 84 },
    { 0xc21890ab, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c890b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72890bd, 66 },
    { 0xff2890be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569890c3, 75 },
    { 0x78890c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2890cc, 52 },
    { 0, 0 },
    { 0x6c9890ce, 95 },
    { 0x3d9890cf, 87 },
    { 0x72f890ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e890d3, 78 },
    { 0, 0 },
    { 0xf5d890d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f890d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b890dc, 75 },
    { 0x74890dd, 98 },
    { 0x1a8890de, 61 },
    { 0x158890de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d890e5, 43 },
    { 0xbce890e5, 33 },
    { 0x58b890e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87890ed, 85 },
    { 0xcd5890ee, 88 },
    { 0xf83890ee, 90 },
    { 0xfff890f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41890f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920890fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15089100, 90 },
    { 0x6489100, 95 },
    { 0, 0 },
    { 0x4f589103, 84 },
    { 0xb2c89103, 88 },
    { 0xd6189104, 95 },
    { 0xc489106, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc078910c, 55 },
    { 0xdb98910d, 33 },
    { 0xba8910e, 30 },
    { 0x65e8910e, 95 },
    { 0, 0 },
    { 0x58089111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93889115, 55 },
    { 0xac889116, 95 },
    { 0, 0 },
    { 0xaad89118, 95 },
    { 0, 0 },
    { 0xde8911a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe889122, 66 },
    { 0xa0389122, 33 },
    { 0x29e89124, 85 },
    { 0, 0 },
    { 0xdd589126, 90 },
    { 0xf589127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c28912c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf08912f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b89133, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc89136, 52 },
    { 0xbbe89137, 98 },
    { 0x24889138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c8913c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e89140, 61 },
    { 0, 0 },
    { 0x9ff89142, 90 },
    { 0x48c89143, 34 },
    { 0xf1e89144, 33 },
    { 0xc789145, 23 },
    { 0xac689146, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb789149, 34 },
    { 0x4958914a, 98 },
    { 0x9689149, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3938914f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb789152, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8189156, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b8915a, 95 },
    { 0x48915b, 23 },
    { 0xb8a8915c, 52 },
    { 0, 0 },
    { 0x92d8915e, 78 },
    { 0xbf58915f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c89163, 95 },
    { 0x20f89163, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac8916a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8916d, 33 },
    { 0x498916e, 98 },
    { 0xbb58916f, 90 },
    { 0x61589170, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d89173, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a89178, 43 },
    { 0xfcb89178, 98 },
    { 0xfd38917a, 61 },
    { 0xbff89178, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa8917e, 42 },
    { 0x2c98917f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a89182, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41489186, 61 },
    { 0, 0 },
    { 0x15a89188, 76 },
    { 0xdab89189, 90 },
    { 0, 0 },
    { 0xf1c8918b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db8918f, 61 },
    { 0x9bf8918f, 100 },
    { 0, 0 },
    { 0x91489192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb89196, 77 },
    { 0, 0 },
    { 0x1f289198, 81 },
    { 0x10989199, 98 },
    { 0, 0 },
    { 0x1218919b, 98 },
    { 0xc368919c, 78 },
    { 0x28919d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26891a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62891af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11891b2, 81 },
    { 0, 0 },
    { 0x63a891b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f891bd, 78 },
    { 0xb42891bd, 98 },
    { 0x426891bf, 23 },
    { 0x5cd891c0, 87 },
    { 0xf43891c0, 33 },
    { 0x52e891c0, 95 },
    { 0, 0 },
    { 0xb2e891c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779891ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8891d3, 95 },
    { 0x92891d3, 33 },
    { 0xe2b891d4, 95 },
    { 0x245891d6, 93 },
    { 0x239891d7, 99 },
    { 0xc1b891d8, 52 },
    { 0xa39891d8, 98 },
    { 0xa9e891d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5891df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c891e5, 46 },
    { 0xef3891e6, 33 },
    { 0x56891e6, 95 },
    { 0xe32891e8, 40 },
    { 0xd0c891e8, 34 },
    { 0x9e5891e9, 43 },
    { 0x58a891eb, 55 },
    { 0xb9f891ec, 55 },
    { 0xcd5891ed, 67 },
    { 0x432891e6, 23 },
    { 0xc91891ef, 89 },
    { 0, 0 },
    { 0x82b891f1, 90 },
    { 0x45f891f1, 95 },
    { 0, 0 },
    { 0x893891f4, 55 },
    { 0x46c891f4, 23 },
    { 0x3ba891f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e891f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d89200, 33 },
    { 0xb7389200, 43 },
    { 0x25689202, 95 },
    { 0x78689203, 98 },
    { 0x55c89201, 86 },
    { 0x8af89205, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce58920b, 78 },
    { 0, 0 },
    { 0xe958920d, 55 },
    { 0, 0 },
    { 0xf5b8920f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8289212, 95 },
    { 0xb1d89213, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89217, 61 },
    { 0x90c89218, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a8921b, 17 },
    { 0x3f48921c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1878921f, 88 },
    { 0, 0 },
    { 0x64c89221, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59689226, 76 },
    { 0x83889227, 90 },
    { 0, 0 },
    { 0x18289229, 46 },
    { 0x6b58922a, 52 },
    { 0, 0 },
    { 0x7a48922c, 91 },
    { 0x70d8922d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a89236, 43 },
    { 0xdd989236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3848923b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6689240, 33 },
    { 0, 0 },
    { 0xf8789242, 23 },
    { 0x84089243, 33 },
    { 0xbcb89244, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16389252, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48589259, 95 },
    { 0x82a8925a, 95 },
    { 0xf028925b, 61 },
    { 0x6b98925c, 91 },
    { 0x7a18925b, 61 },
    { 0xcdd8925e, 34 },
    { 0x3e58925c, 95 },
    { 0, 0 },
    { 0x5a789261, 99 },
    { 0xcce89262, 87 },
    { 0x6a089262, 23 },
    { 0xc3f89264, 76 },
    { 0xf0389265, 100 },
    { 0xb4489264, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d88926a, 34 },
    { 0, 0 },
    { 0x798926c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa89273, 61 },
    { 0xfec89273, 33 },
    { 0xf1a89273, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f89279, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d78927c, 34 },
    { 0x8bb8927d, 84 },
    { 0, 0 },
    { 0xe448927f, 95 },
    { 0x18489280, 76 },
    { 0x4af89280, 98 },
    { 0xed489280, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b389290, 76 },
    { 0x32289291, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff289295, 55 },
    { 0xc9a89296, 33 },
    { 0, 0 },
    { 0xc4d89298, 95 },
    { 0x1c089299, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4892a1, 33 },
    { 0xb32892a2, 33 },
    { 0x5f2892a3, 90 },
    { 0xb00892a3, 34 },
    { 0, 0 },
    { 0xb42892a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed892a9, 76 },
    { 0, 0 },
    { 0xac8892ab, 40 },
    { 0x2ba892ab, 67 },
    { 0, 0 },
    { 0xfde892ae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc892b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e892b8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0892bb, 43 },
    { 0xe78892bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b892bf, 38 },
    { 0xfaa892c0, 33 },
    { 0, 0 },
    { 0xd06892c2, 56 },
    { 0x56c892c2, 33 },
    { 0xe6d892c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c892ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67892d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a892d5, 43 },
    { 0xf85892d6, 23 },
    { 0x492892d5, 84 },
    { 0xe9a892d6, 95 },
    { 0xace892d9, 76 },
    { 0x9dd892da, 75 },
    { 0xbde892db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37892de, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1892e4, 98 },
    { 0x590892e5, 93 },
    { 0xbb1892e6, 61 },
    { 0xa98892e5, 98 },
    { 0x547892e8, 67 },
    { 0, 0 },
    { 0x85b892ea, 90 },
    { 0, 0 },
    { 0x4bc892ec, 55 },
    { 0x205892ed, 75 },
    { 0x510892ee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213892f2, 23 },
    { 0x435892f2, 76 },
    { 0x882892f4, 98 },
    { 0, 0 },
    { 0x500892f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f892fb, 61 },
    { 0x1fe892fb, 43 },
    { 0x213892fd, 95 },
    { 0x870892fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f089301, 34 },
    { 0, 0 },
    { 0x4d389303, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x26489306, 33 },
    { 0, 0 },
    { 0x37389308, 64 },
    { 0x4ab89309, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c88930c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68930f, 90 },
    { 0x31789310, 77 },
    { 0x4e08930f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6289314, 95 },
    { 0xe6389314, 91 },
    { 0x6b489316, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e689319, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa678931d, 93 },
    { 0xb948931e, 61 },
    { 0xbf18931f, 33 },
    { 0xf98931f, 78 },
    { 0xb8389320, 99 },
    { 0xb3e8931d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39989326, 93 },
    { 0, 0 },
    { 0x3c689328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d89330, 23 },
    { 0, 0 },
    { 0xf1589332, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f89335, 93 },
    { 0x1589335, 34 },
    { 0x30489337, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d8933a, 90 },
    { 0x8898933b, 23 },
    { 0xf8933a, 33 },
    { 0x6728933d, 95 },
    { 0xbab8933c, 33 },
    { 0x5308933f, 53 },
    { 0x32889340, 90 },
    { 0xf9289341, 23 },
    { 0x82e8933e, 99 },
    { 0, 0 },
    { 0xa1c89344, 90 },
    { 0xc2289344, 61 },
    { 0, 0 },
    { 0x3b389347, 67 },
    { 0x71189348, 33 },
    { 0, 0 },
    { 0x6648934a, 95 },
    { 0x44f8934b, 68 },
    { 0xdb58934b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d89350, 95 },
    { 0xf2389351, 88 },
    { 0xfda89351, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1289355, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e89358, 88 },
    { 0xa1589358, 95 },
    { 0xce18935a, 43 },
    { 0, 0 },
    { 0x11e8935c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98689362, 33 },
    { 0, 0 },
    { 0x5f389364, 33 },
    { 0x4c289364, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf28936b, 34 },
    { 0x2ac8936c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b89373, 61 },
    { 0x1d989373, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7889379, 55 },
    { 0xc489379, 61 },
    { 0xfd28937b, 61 },
    { 0x67b8937c, 61 },
    { 0x3ec8937c, 76 },
    { 0xe48937a, 91 },
    { 0xf48937f, 95 },
    { 0, 0 },
    { 0x8a389381, 61 },
    { 0x1f389382, 91 },
    { 0x2e089382, 33 },
    { 0x7c789382, 43 },
    { 0, 0 },
    { 0x1e789386, 84 },
    { 0, 0 },
    { 0x28689388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408939d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0893a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x322893a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e893a8, 98 },
    { 0, 0 },
    { 0x99893aa, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7893ad, 34 },
    { 0xa5d893ae, 76 },
    { 0x382893ad, 91 },
    { 0, 0 },
    { 0x5cc893b1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x179893b4, 90 },
    { 0, 0 },
    { 0x9e9893b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b893b9, 98 },
    { 0x839893b9, 55 },
    { 0, 0 },
    { 0x1de893bc, 88 },
    { 0xefa893bd, 46 },
    { 0xcf1893be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777893c4, 87 },
    { 0, 0 },
    { 0xa47893c6, 100 },
    { 0x164893c7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad893cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0893ce, 55 },
    { 0xe05893ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d893d5, 33 },
    { 0xbef893d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0893dc, 90 },
    { 0xe37893dd, 85 },
    { 0xae8893dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x377893e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858893e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2893ea, 34 },
    { 0xfb5893ea, 95 },
    { 0, 0 },
    { 0xe0893ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba893f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b893f5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb893fb, 33 },
    { 0x596893fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac893ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab989405, 98 },
    { 0, 0 },
    { 0x3bc89407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b8940f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d89413, 23 },
    { 0x29589413, 90 },
    { 0x4c189415, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x63589418, 34 },
    { 0, 0 },
    { 0xdd98941a, 33 },
    { 0x9308941b, 66 },
    { 0x1498941c, 33 },
    { 0x9988941c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d389420, 76 },
    { 0, 0 },
    { 0x42789422, 55 },
    { 0, 0 },
    { 0x52089424, 33 },
    { 0x43489425, 76 },
    { 0x64889426, 93 },
    { 0xba689427, 23 },
    { 0xce489425, 90 },
    { 0, 0 },
    { 0x4668942a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a8942d, 87 },
    { 0, 0 },
    { 0x4758942f, 76 },
    { 0, 0 },
    { 0x41a89431, 33 },
    { 0x51389432, 23 },
    { 0x40589433, 90 },
    { 0x3b489434, 23 },
    { 0x36189431, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4189439, 33 },
    { 0xf4489439, 95 },
    { 0, 0 },
    { 0x7828943c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e89442, 33 },
    { 0xf889443, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd89446, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9488944a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa488944d, 61 },
    { 0xb1d8944e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25989451, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae89456, 33 },
    { 0x90c89457, 43 },
    { 0x44c89457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc098945b, 96 },
    { 0x6998945b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3968945f, 90 },
    { 0, 0 },
    { 0xc7789461, 33 },
    { 0x2b389462, 90 },
    { 0, 0 },
    { 0x99289464, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1589468, 5 },
    { 0xb0489469, 98 },
    { 0xf8e8946a, 93 },
    { 0xe328946b, 23 },
    { 0xd108946a, 98 },
    { 0xc088946d, 76 },
    { 0x22b8946e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b89475, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789479, 95 },
    { 0x9798947a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b289480, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc189487, 36 },
    { 0x5c789487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd618948b, 100 },
    { 0, 0 },
    { 0xb948948d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de89493, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf008949a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e8949f, 23 },
    { 0xb18949f, 90 },
    { 0x96d894a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e894ae, 61 },
    { 0x20894af, 33 },
    { 0xbdf894b0, 53 },
    { 0, 0 },
    { 0x70e894b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343894b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e894bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa894bf, 23 },
    { 0x1e3894c0, 90 },
    { 0x9d894bf, 95 },
    { 0xe26894c2, 84 },
    { 0x22f894c2, 100 },
    { 0x6de894c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f894c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c894cd, 34 },
    { 0x678894ce, 46 },
    { 0, 0 },
    { 0xb1e894d0, 88 },
    { 0xea7894d0, 95 },
    { 0x40c894d0, 90 },
    { 0, 0 },
    { 0x633894d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154894d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5894e3, 33 },
    { 0x15c894e3, 90 },
    { 0xb13894e5, 95 },
    { 0xfad894e6, 98 },
    { 0, 0 },
    { 0x5894e8, 90 },
    { 0xadb894e9, 33 },
    { 0xd74894ea, 98 },
    { 0xf84894eb, 85 },
    { 0x842894ec, 98 },
    { 0xe3a894ed, 78 },
    { 0x41e894ed, 80 },
    { 0xdee894ee, 91 },
    { 0x6b3894f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0894f3, 98 },
    { 0x6de894f4, 61 },
    { 0, 0 },
    { 0x558894f6, 55 },
    { 0, 0 },
    { 0xb7e894f8, 90 },
    { 0xcdc894f9, 100 },
    { 0xd19894fa, 90 },
    { 0xa2b894fb, 33 },
    { 0x468894f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a089501, 61 },
    { 0, 0 },
    { 0x18889503, 99 },
    { 0x40489504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6089507, 95 },
    { 0x6ac89508, 76 },
    { 0, 0 },
    { 0x7dc8950a, 86 },
    { 0xb0c8950a, 33 },
    { 0xe098950c, 55 },
    { 0, 0 },
    { 0x6f88950e, 77 },
    { 0, 0 },
    { 0xf8289510, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc189515, 90 },
    { 0xffe89515, 61 },
    { 0xd2a89516, 33 },
    { 0xaa189518, 23 },
    { 0xfc489518, 90 },
    { 0xb108951a, 95 },
    { 0xde08951b, 76 },
    { 0xc338951b, 23 },
    { 0x3f28951c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a89521, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a89524, 95 },
    { 0, 0 },
    { 0x55989526, 77 },
    { 0xff389527, 33 },
    { 0, 0 },
    { 0x74c89529, 34 },
    { 0, 0 },
    { 0xf528952b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e89531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e189536, 78 },
    { 0, 0 },
    { 0x54489538, 98 },
    { 0, 0 },
    { 0xbd8953a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82989544, 93 },
    { 0xbb289545, 99 },
    { 0, 0 },
    { 0x31889547, 84 },
    { 0, 0 },
    { 0x35089549, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b8954d, 76 },
    { 0, 0 },
    { 0xfa18954f, 23 },
    { 0x10a8954f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64389555, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3018955b, 33 },
    { 0x36a8955c, 88 },
    { 0x7398955c, 23 },
    { 0x5c8955e, 33 },
    { 0, 0 },
    { 0x9dc89560, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b89563, 84 },
    { 0, 0 },
    { 0x81f89565, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc89570, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac389578, 34 },
    { 0x6cb89578, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d8957d, 93 },
    { 0x5278957e, 95 },
    { 0, 0 },
    { 0xd1789580, 76 },
    { 0x99889581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec8958a, 46 },
    { 0, 0 },
    { 0x68c8958c, 52 },
    { 0xdf88958c, 34 },
    { 0xac8958e, 90 },
    { 0xda48958e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a89594, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa398959a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268959e, 23 },
    { 0xb3a8959f, 76 },
    { 0x6028959f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3895a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f895a6, 33 },
    { 0x4f2895a7, 85 },
    { 0xf3e895a8, 33 },
    { 0, 0 },
    { 0xe8c895aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb895ad, 61 },
    { 0, 0 },
    { 0xd14895af, 78 },
    { 0x28c895af, 23 },
    { 0xf46895b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b895b4, 90 },
    { 0xc48895b5, 98 },
    { 0x927895b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b895bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3895c5, 59 },
    { 0, 0 },
    { 0x332895c7, 33 },
    { 0x891895c7, 33 },
    { 0x979895c7, 83 },
    { 0, 0 },
    { 0xd33895cb, 43 },
    { 0x5f6895cc, 33 },
    { 0xf82895cc, 55 },
    { 0xa74895ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76895d2, 34 },
    { 0, 0 },
    { 0x675895d4, 61 },
    { 0xc2f895d4, 77 },
    { 0, 0 },
    { 0x380895d7, 88 },
    { 0xc3c895d8, 95 },
    { 0x5cd895d7, 100 },
    { 0, 0 },
    { 0x81e895db, 90 },
    { 0x53f895dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x488895df, 76 },
    { 0x58c895df, 34 },
    { 0x4cd895e0, 61 },
    { 0x328895e2, 90 },
    { 0x541895e3, 46 },
    { 0x670895e3, 33 },
    { 0xa4c895e3, 33 },
    { 0, 0 },
    { 0x74895e7, 90 },
    { 0xb7b895e8, 99 },
    { 0x34c895e7, 100 },
    { 0, 0 },
    { 0xefc895eb, 56 },
    { 0x500895ec, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813895f6, 34 },
    { 0x515895f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x459895fa, 52 },
    { 0x4a6895fb, 23 },
    { 0x82c895fc, 61 },
    { 0x6c1895fb, 55 },
    { 0xda9895fb, 98 },
    { 0x49f895ff, 98 },
    { 0x749895ff, 33 },
    { 0x88a895ff, 95 },
    { 0xc5b895fc, 33 },
    { 0xa15895fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83389606, 98 },
    { 0x50589607, 90 },
    { 0x1b189608, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8960c, 61 },
    { 0, 0 },
    { 0xdb28960e, 90 },
    { 0x4088960f, 95 },
    { 0x8718960f, 98 },
    { 0xd2789611, 33 },
    { 0x6db89611, 100 },
    { 0xa0e89613, 85 },
    { 0xaee89614, 23 },
    { 0x8ed89615, 76 },
    { 0x35a8960f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff8961d, 90 },
    { 0x34c8961e, 34 },
    { 0xb238961e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a389625, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea89628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7628962e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97489631, 43 },
    { 0x39a89632, 100 },
    { 0xfb989632, 99 },
    { 0x2689633, 61 },
    { 0, 0 },
    { 0xbdb89636, 85 },
    { 0xfda89637, 90 },
    { 0, 0 },
    { 0xf5d89639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e18963c, 43 },
    { 0, 0 },
    { 0xf488963e, 46 },
    { 0x37a8963f, 33 },
    { 0xb698963f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37989646, 61 },
    { 0xd5e89647, 55 },
    { 0x9b789646, 55 },
    { 0x4ae89649, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe368964c, 76 },
    { 0, 0 },
    { 0xf9b8964e, 46 },
    { 0, 0 },
    { 0x52c89650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d8965a, 98 },
    { 0x52a8965b, 46 },
    { 0xae48965c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a68965f, 90 },
    { 0xbe589660, 55 },
    { 0, 0 },
    { 0x64089662, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36589667, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6348966d, 90 },
    { 0xf558966e, 99 },
    { 0xb8a8966f, 55 },
    { 0xf2e89670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b8967b, 80 },
    { 0, 0 },
    { 0x3b88967d, 61 },
    { 0x9288967d, 61 },
    { 0x6f18967f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f389682, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868968a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b78968d, 33 },
    { 0x1f78968e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f89692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e89697, 90 },
    { 0x32289698, 98 },
    { 0x94c89699, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3896a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755896b1, 98 },
    { 0, 0 },
    { 0x210896b3, 66 },
    { 0xd75896b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5896b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e896ba, 76 },
    { 0x3fc896bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78896be, 68 },
    { 0x6a896be, 33 },
    { 0, 0 },
    { 0x973896c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8896c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7896c9, 87 },
    { 0, 0 },
    { 0x863896cb, 33 },
    { 0xdd1896cc, 61 },
    { 0, 0 },
    { 0xdd5896ce, 81 },
    { 0xa3e896cf, 46 },
    { 0x9e7896ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3896d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70896dd, 23 },
    { 0, 0 },
    { 0x5c4896df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x981896e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3896e6, 61 },
    { 0x3f1896e6, 90 },
    { 0x2c1896e7, 66 },
    { 0, 0 },
    { 0xbe7896ea, 55 },
    { 0x892896eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25896f1, 93 },
    { 0, 0 },
    { 0x14a896f3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4896f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda589704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81089708, 33 },
    { 0xba989709, 95 },
    { 0x6748970a, 23 },
    { 0x9558970b, 55 },
    { 0xa9c8970b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51189710, 61 },
    { 0x70689710, 67 },
    { 0, 0 },
    { 0x75b89713, 33 },
    { 0x42089714, 52 },
    { 0xcb889714, 23 },
    { 0, 0 },
    { 0xc5a89717, 77 },
    { 0, 0 },
    { 0x71c89719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd18971e, 43 },
    { 0x1b18971f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f89724, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f089729, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf788972c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9589730, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a89735, 52 },
    { 0, 0 },
    { 0xb8e89737, 55 },
    { 0x5fb89738, 90 },
    { 0x3d589737, 76 },
    { 0, 0 },
    { 0xf358973b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef89740, 52 },
    { 0x90189741, 98 },
    { 0x42489742, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3589749, 61 },
    { 0x948974a, 61 },
    { 0x9178974b, 98 },
    { 0x96d8974a, 23 },
    { 0xcea8974d, 61 },
    { 0x2b78974e, 61 },
    { 0x8e8974b, 43 },
    { 0xeee89750, 85 },
    { 0xa189751, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x90489754, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f89758, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe598975c, 75 },
    { 0x5ef8975c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b89760, 23 },
    { 0, 0 },
    { 0x62489762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec48976b, 66 },
    { 0x1b38976c, 23 },
    { 0x27d8976c, 90 },
    { 0x8b28976e, 87 },
    { 0x8f28976f, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d89773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d89777, 46 },
    { 0xfb489777, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1278977c, 78 },
    { 0, 0 },
    { 0xcb68977e, 98 },
    { 0x24e8977f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c89783, 61 },
    { 0, 0 },
    { 0xa2789785, 64 },
    { 0, 0 },
    { 0x42389787, 95 },
    { 0xf1289788, 95 },
    { 0, 0 },
    { 0x5e08978a, 34 },
    { 0, 0 },
    { 0x7228978c, 33 },
    { 0x7b28978d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64189793, 95 },
    { 0, 0 },
    { 0x2ed89795, 66 },
    { 0xc0789796, 33 },
    { 0x8f189797, 86 },
    { 0xdd189798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7788979c, 33 },
    { 0, 0 },
    { 0x828979e, 23 },
    { 0, 0 },
    { 0x78f897a0, 77 },
    { 0x24c897a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61897a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76897ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd897b0, 40 },
    { 0x3f2897b0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf897b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2897b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b897bd, 90 },
    { 0x92b897bd, 76 },
    { 0x9f7897bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93897c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72897c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce897d0, 65 },
    { 0xf1a897d1, 33 },
    { 0xcf9897d2, 23 },
    { 0x818897d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0897d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29897e0, 75 },
    { 0x8f2897e1, 23 },
    { 0x12e897e1, 91 },
    { 0x4a1897e3, 34 },
    { 0x6ca897e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792897e9, 85 },
    { 0, 0 },
    { 0x615897eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0897f3, 43 },
    { 0x4c0897f4, 95 },
    { 0x999897f5, 88 },
    { 0xcb2897f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec897fe, 95 },
    { 0x6b9897ff, 55 },
    { 0, 0 },
    { 0xfff89801, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf489806, 84 },
    { 0, 0 },
    { 0xccb89808, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f8980b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba89811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b889818, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c8981b, 34 },
    { 0x29a8981c, 99 },
    { 0xaf28981d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d89821, 95 },
    { 0, 0 },
    { 0x92789823, 90 },
    { 0x4c589823, 95 },
    { 0xa0d89825, 33 },
    { 0x2289824, 90 },
    { 0xd3789827, 23 },
    { 0xbe089828, 95 },
    { 0xe7889828, 95 },
    { 0, 0 },
    { 0x8a38982b, 84 },
    { 0x6b08982b, 33 },
    { 0x6b88982b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad89837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9118983b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a88983f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4589843, 61 },
    { 0x38389844, 43 },
    { 0x65889845, 78 },
    { 0xac289845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9789849, 66 },
    { 0xba68984a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448984f, 46 },
    { 0xa2189850, 65 },
    { 0xb5689851, 78 },
    { 0x63a8984f, 90 },
    { 0x83689853, 78 },
    { 0x63889853, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6689859, 98 },
    { 0, 0 },
    { 0xbfe8985b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0889862, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c89865, 90 },
    { 0x3a089866, 98 },
    { 0xf7789867, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb528986b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc8986f, 68 },
    { 0xc7789870, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c88987b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d389880, 98 },
    { 0x5989881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3668988d, 43 },
    { 0xcb58988d, 34 },
    { 0, 0 },
    { 0x46e89890, 85 },
    { 0xc3689891, 33 },
    { 0xf8489891, 56 },
    { 0, 0 },
    { 0xd2089894, 90 },
    { 0xe4889894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9488989b, 34 },
    { 0xb8c8989c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d898a0, 100 },
    { 0, 0 },
    { 0x322898a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710898a7, 91 },
    { 0x40f898a8, 78 },
    { 0xe77898a9, 95 },
    { 0x5b8898aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c898ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae898b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5898b9, 43 },
    { 0x4c9898ba, 98 },
    { 0x7b7898ba, 23 },
    { 0, 0 },
    { 0xe2d898bd, 98 },
    { 0xdf4898be, 98 },
    { 0x7ed898be, 88 },
    { 0x5f6898c0, 85 },
    { 0x53898c1, 61 },
    { 0x48b898be, 76 },
    { 0x98b898c3, 90 },
    { 0x5cd898c4, 55 },
    { 0x81e898c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977898ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab898d3, 98 },
    { 0x67e898d4, 95 },
    { 0x7ac898d5, 91 },
    { 0xb0c898d6, 34 },
    { 0, 0 },
    { 0xf6898d8, 93 },
    { 0x90a898d9, 95 },
    { 0, 0 },
    { 0x87f898db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b898e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c898e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8898ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996898ee, 66 },
    { 0x2d898ef, 95 },
    { 0x718898f0, 93 },
    { 0, 0 },
    { 0x70a898f2, 46 },
    { 0x141898f3, 65 },
    { 0xd81898f4, 96 },
    { 0x2e3898f4, 93 },
    { 0x23898f2, 24 },
    { 0xef6898f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c898fb, 90 },
    { 0x23c898fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8089904, 80 },
    { 0, 0 },
    { 0x2f589906, 90 },
    { 0, 0 },
    { 0x80a89908, 90 },
    { 0xaa389908, 98 },
    { 0, 0 },
    { 0xc998990b, 23 },
    { 0xe508990c, 23 },
    { 0, 0 },
    { 0x3958990e, 61 },
    { 0xaf88990e, 33 },
    { 0xea8990f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd89915, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1389919, 23 },
    { 0xca48991a, 99 },
    { 0x7e58991b, 17 },
    { 0x5cb8991c, 95 },
    { 0x6118991d, 98 },
    { 0x3cd8991d, 61 },
    { 0x5468991a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a89922, 78 },
    { 0xff989922, 23 },
    { 0x97c89924, 56 },
    { 0xd5389925, 43 },
    { 0x6a389926, 95 },
    { 0x57589926, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d8992a, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48992e, 98 },
    { 0xd798992f, 55 },
    { 0x6c989930, 61 },
    { 0x13f8992f, 100 },
    { 0, 0 },
    { 0xfd89933, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7189936, 23 },
    { 0xd6789937, 33 },
    { 0xc3b89938, 33 },
    { 0, 0 },
    { 0xd6c8993a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8993d, 61 },
    { 0xad98993d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb938994b, 66 },
    { 0xa788994c, 68 },
    { 0xdac8994d, 90 },
    { 0, 0 },
    { 0x5418994f, 36 },
    { 0x3d58994f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6689955, 23 },
    { 0x1a289955, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd8995a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46689964, 98 },
    { 0x33a89964, 61 },
    { 0, 0 },
    { 0x32f89967, 92 },
    { 0, 0 },
    { 0xa0089969, 40 },
    { 0, 0 },
    { 0x67e8996b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f489971, 23 },
    { 0xb9889972, 47 },
    { 0x93989972, 23 },
    { 0x7df89974, 90 },
    { 0x3e989975, 23 },
    { 0xb4b89975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4868997d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d89981, 38 },
    { 0xb2d89982, 93 },
    { 0xa0289982, 23 },
    { 0xf4689981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d089988, 90 },
    { 0x94789988, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf888998c, 99 },
    { 0x6af8998d, 98 },
    { 0x4ee8998e, 55 },
    { 0x8358998e, 34 },
    { 0, 0 },
    { 0x9b989991, 33 },
    { 0x27f89991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a89996, 93 },
    { 0x29d89997, 98 },
    { 0x7b689998, 61 },
    { 0x9e489999, 61 },
    { 0xb1589997, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f899a1, 33 },
    { 0, 0 },
    { 0x94c899a3, 78 },
    { 0x3ef899a4, 23 },
    { 0xa5c899a3, 34 },
    { 0xe28899a6, 78 },
    { 0xd92899a7, 78 },
    { 0xc6a899a8, 61 },
    { 0x150899a9, 90 },
    { 0x237899aa, 92 },
    { 0x75b899ab, 55 },
    { 0x246899a9, 33 },
    { 0x9ec899a5, 65 },
    { 0xe7f899aa, 95 },
    { 0, 0 },
    { 0xd14899b0, 99 },
    { 0x6dc899b1, 98 },
    { 0x1e5899b2, 93 },
    { 0x307899b3, 58 },
    { 0x8c4899b3, 85 },
    { 0, 0 },
    { 0x4b1899b6, 23 },
    { 0x5f4899b7, 76 },
    { 0x14b899b8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f899bc, 98 },
    { 0x3ab899bd, 66 },
    { 0, 0 },
    { 0xc6c899bf, 23 },
    { 0x7c0899c0, 78 },
    { 0xa0e899c1, 33 },
    { 0x36c899c0, 69 },
    { 0x43e899c3, 23 },
    { 0, 0 },
    { 0x150899c5, 95 },
    { 0, 0 },
    { 0x7de899c7, 23 },
    { 0x298899c7, 33 },
    { 0x7b0899c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d899cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc899d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47899dc, 46 },
    { 0x1b6899dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d899e0, 66 },
    { 0x873899e0, 95 },
    { 0x704899e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9899e8, 33 },
    { 0x6b5899e9, 33 },
    { 0xee6899ea, 33 },
    { 0, 0 },
    { 0x96899ec, 95 },
    { 0xdcf899ed, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1899f0, 95 },
    { 0x606899f1, 61 },
    { 0xa28899f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f899f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf899fa, 55 },
    { 0x55e899fb, 61 },
    { 0xa05899fc, 76 },
    { 0xaea899fd, 55 },
    { 0x702899fe, 91 },
    { 0xf65899ff, 46 },
    { 0x77a89a00, 90 },
    { 0x18899fd, 75 },
    { 0xfb5899fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27389a06, 90 },
    { 0x16489a06, 49 },
    { 0xc2789a08, 76 },
    { 0x6ca89a08, 38 },
    { 0x67189a0a, 53 },
    { 0xbcd89a0a, 90 },
    { 0xe4389a0c, 61 },
    { 0, 0 },
    { 0x39f89a0e, 61 },
    { 0x6f389a0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c989a12, 23 },
    { 0x21f89a13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b89a1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb89a1d, 46 },
    { 0, 0 },
    { 0xef689a1f, 84 },
    { 0x76389a20, 95 },
    { 0, 0 },
    { 0xc9589a22, 61 },
    { 0, 0 },
    { 0x1c489a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a89a2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4889a32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17789a35, 78 },
    { 0x49089a35, 90 },
    { 0xba089a37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d489a3b, 33 },
    { 0x29989a3c, 35 },
    { 0xd3389a3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d389a40, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3089a43, 51 },
    { 0x69c89a43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e89a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a389a4e, 52 },
    { 0x91e89a4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c289a53, 23 },
    { 0x6fa89a53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff489a59, 95 },
    { 0xd2c89a59, 90 },
    { 0x40789a5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc89a60, 95 },
    { 0xec689a61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e89a66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f89a6c, 97 },
    { 0, 0 },
    { 0x61d89a6e, 78 },
    { 0xfa989a6e, 78 },
    { 0, 0 },
    { 0xc189a71, 23 },
    { 0x6c489a72, 91 },
    { 0x91f89a72, 61 },
    { 0x8e89a74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32289a7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a89a7e, 91 },
    { 0x15d89a7e, 95 },
    { 0x3d489a7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64789a87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14d89a8a, 90 },
    { 0, 0 },
    { 0x83489a8c, 33 },
    { 0xeaf89a8c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d389a90, 93 },
    { 0x9ee89a91, 93 },
    { 0, 0 },
    { 0x74d89a93, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x40089a96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33489a9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28d89a9f, 98 },
    { 0xbd889aa0, 95 },
    { 0x8ef89aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52089aa6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d89aae, 92 },
    { 0xc7d89aaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd89ab2, 61 },
    { 0x30389ab2, 90 },
    { 0x55e89ab2, 23 },
    { 0, 0 },
    { 0x36189ab6, 46 },
    { 0, 0 },
    { 0xc8b89ab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b089ac3, 33 },
    { 0xb6c89ac4, 90 },
    { 0, 0 },
    { 0x43489ac6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c89aca, 23 },
    { 0, 0 },
    { 0xa3b89acc, 90 },
    { 0x91889acd, 95 },
    { 0, 0 },
    { 0xbfb89acf, 33 },
    { 0x76489ad0, 33 },
    { 0x91089ad0, 90 },
    { 0x46e89ad2, 85 },
    { 0xfea89ad0, 33 },
    { 0x99289ad3, 90 },
    { 0x47789ad1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce689ad9, 84 },
    { 0, 0 },
    { 0x43e89adb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e289ade, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67189ae4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e489ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc89aed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec689af4, 91 },
    { 0x70889af5, 100 },
    { 0xef289af5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2289afa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a89afe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d589b04, 95 },
    { 0, 0 },
    { 0xe6089b06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef289b09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7589b0d, 23 },
    { 0x28089b0d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6289b13, 46 },
    { 0xab489b14, 90 },
    { 0, 0 },
    { 0x5189b16, 90 },
    { 0x2d689b17, 88 },
    { 0x8c589b16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57389b1e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d889b25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae89b29, 98 },
    { 0x9a689b2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e589b2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81489b32, 61 },
    { 0xd2c89b32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b89b36, 98 },
    { 0xa9889b36, 87 },
    { 0xe5189b37, 99 },
    { 0, 0 },
    { 0xbe089b3a, 33 },
    { 0x13089b3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8589b4d, 98 },
    { 0x10c89b4d, 43 },
    { 0xd6a89b4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f89b54, 95 },
    { 0xff789b55, 98 },
    { 0x9489b56, 33 },
    { 0xf0689b56, 90 },
    { 0xf2b89b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd89b5d, 17 },
    { 0x66789b5e, 61 },
    { 0x2e489b5e, 90 },
    { 0xfbc89b60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce89b66, 75 },
    { 0x26089b67, 69 },
    { 0xd8a89b68, 61 },
    { 0xc8189b69, 43 },
    { 0, 0 },
    { 0x18089b6b, 90 },
    { 0, 0 },
    { 0x3c289b6d, 34 },
    { 0xb4789b6e, 90 },
    { 0x2ea89b6e, 90 },
    { 0xc7789b70, 61 },
    { 0x57e89b71, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6689b75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a689b81, 66 },
    { 0x9c289b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc89b8c, 61 },
    { 0x3bd89b8d, 95 },
    { 0, 0 },
    { 0x78689b8f, 55 },
    { 0, 0 },
    { 0x2b889b91, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8889b99, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c89b9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96389ba2, 34 },
    { 0xfe789ba2, 33 },
    { 0xe2389ba4, 93 },
    { 0xb2989ba5, 95 },
    { 0x25989ba6, 33 },
    { 0x5eb89ba4, 34 },
    { 0x62589ba7, 61 },
    { 0x6ac89ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf89baf, 97 },
    { 0xbb589bb0, 85 },
    { 0x33089bb0, 90 },
    { 0xd5289baf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d89bb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e89bb9, 55 },
    { 0x38e89bb9, 90 },
    { 0, 0 },
    { 0x36b89bbc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b89bbf, 93 },
    { 0, 0 },
    { 0x56189bc1, 76 },
    { 0xb2989bc2, 98 },
    { 0, 0 },
    { 0x9f489bc4, 68 },
    { 0xb3e89bc4, 23 },
    { 0, 0 },
    { 0xbf689bc7, 76 },
    { 0, 0 },
    { 0x5f289bc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51989bcc, 91 },
    { 0x82b89bcc, 84 },
    { 0x26589bcc, 87 },
    { 0x7b89bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa89bd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f189bd7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e89bdd, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea89be2, 33 },
    { 0xeca89be3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6889bec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7089bf3, 23 },
    { 0, 0 },
    { 0x18189bf5, 23 },
    { 0x5df89bf6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d89bfa, 98 },
    { 0, 0 },
    { 0x96589bfc, 33 },
    { 0xbb389bfd, 23 },
    { 0x4a789bfc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7089c03, 23 },
    { 0xcbe89c04, 98 },
    { 0x54f89c03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a89c09, 23 },
    { 0x53789c0a, 23 },
    { 0x39089c0a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c89c0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89789c11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4989c16, 99 },
    { 0, 0 },
    { 0xfcb89c18, 90 },
    { 0, 0 },
    { 0x1c89c1a, 17 },
    { 0, 0 },
    { 0xe0c89c1c, 34 },
    { 0x9b389c1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a889c20, 87 },
    { 0x2b589c21, 85 },
    { 0xc9a89c22, 95 },
    { 0, 0 },
    { 0x31b89c24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a789c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f89c33, 33 },
    { 0xb2889c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb89c3b, 90 },
    { 0xdf889c3c, 98 },
    { 0x14089c3d, 23 },
    { 0x26c89c3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a989c41, 23 },
    { 0xc9389c42, 95 },
    { 0xb0189c42, 95 },
    { 0, 0 },
    { 0x5d689c45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e89c51, 90 },
    { 0xcce89c52, 91 },
    { 0xee289c51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb189c57, 61 },
    { 0, 0 },
    { 0x1bd89c59, 95 },
    { 0x8fa89c5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab89c67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b789c6d, 48 },
    { 0x74189c6d, 55 },
    { 0, 0 },
    { 0x82189c70, 90 },
    { 0xe8689c70, 99 },
    { 0, 0 },
    { 0x8489c73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23989c76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb689c7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed89c7f, 98 },
    { 0xe1189c80, 65 },
    { 0xad489c81, 38 },
    { 0, 0 },
    { 0xc7489c83, 66 },
    { 0, 0 },
    { 0x82e89c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6189c8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b89c94, 78 },
    { 0x96489c95, 80 },
    { 0x33189c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3189c9b, 90 },
    { 0, 0 },
    { 0x2c989c9d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x25689ca0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48789ca5, 91 },
    { 0x9689ca6, 61 },
    { 0x79a89ca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57889cab, 77 },
    { 0, 0 },
    { 0xbe289cad, 90 },
    { 0xd9989cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf89cb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae489cb5, 76 },
    { 0x1f989cb6, 34 },
    { 0xd089cb6, 23 },
    { 0, 0 },
    { 0x7689cb9, 61 },
    { 0x82389cba, 72 },
    { 0, 0 },
    { 0x76989cbc, 80 },
    { 0xd3a89cbd, 95 },
    { 0x42d89cbe, 43 },
    { 0xeb589cbe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d89cc3, 78 },
    { 0, 0 },
    { 0x35789cc5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2989cc8, 90 },
    { 0, 0 },
    { 0x15f89cca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe389cce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9089cd9, 23 },
    { 0x1889cd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a389ce0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3789ceb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d89cef, 74 },
    { 0xe5c89cef, 90 },
    { 0x8e989cef, 55 },
    { 0x30689cf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba89cf6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9489d03, 95 },
    { 0x3e689d04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc489d11, 85 },
    { 0, 0 },
    { 0x8f489d13, 33 },
    { 0x38289d14, 33 },
    { 0x6c689d15, 61 },
    { 0x51189d16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x84589d19, 33 },
    { 0x5c89d1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73589d1e, 61 },
    { 0x86d89d1f, 78 },
    { 0xc6589d20, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0589d23, 40 },
    { 0xe9c89d24, 80 },
    { 0x70f89d23, 46 },
    { 0x48589d24, 33 },
    { 0x58889d23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda89d2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c489d34, 75 },
    { 0, 0 },
    { 0xf1089d36, 76 },
    { 0, 0 },
    { 0x47889d38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e89d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc389d48, 90 },
    { 0xda089d49, 99 },
    { 0, 0 },
    { 0x4ce89d4b, 61 },
    { 0, 0 },
    { 0xdff89d4d, 33 },
    { 0xd6e89d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e289d52, 69 },
    { 0x7e389d53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34689d57, 33 },
    { 0, 0 },
    { 0x16c89d59, 77 },
    { 0x4ae89d5a, 61 },
    { 0, 0 },
    { 0x18289d5c, 95 },
    { 0x15189d5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9989d64, 23 },
    { 0xd5589d65, 55 },
    { 0xe3089d66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc89d6b, 76 },
    { 0x50489d6b, 95 },
    { 0x22d89d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea89d70, 95 },
    { 0, 0 },
    { 0x46b89d72, 33 },
    { 0x2c689d73, 33 },
    { 0x3689d73, 95 },
    { 0x30189d75, 61 },
    { 0x71e89d76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1189d7a, 23 },
    { 0, 0 },
    { 0x25a89d7c, 46 },
    { 0xb2c89d7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46489d89, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b89d8d, 23 },
    { 0x21d89d8e, 76 },
    { 0, 0 },
    { 0x55b89d90, 33 },
    { 0, 0 },
    { 0x89489d92, 34 },
    { 0x5c189d93, 76 },
    { 0xf2189d92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3389d9b, 46 },
    { 0, 0 },
    { 0xff589d9d, 90 },
    { 0xa0589d9e, 43 },
    { 0, 0 },
    { 0xcd089da0, 90 },
    { 0x16e89da0, 23 },
    { 0x68a89da2, 33 },
    { 0x1ab89da3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88789da6, 76 },
    { 0x82889da6, 67 },
    { 0xdc889da8, 90 },
    { 0, 0 },
    { 0xa6d89daa, 23 },
    { 0, 0 },
    { 0xfa989dac, 90 },
    { 0xd9689dad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88089db4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de89db8, 100 },
    { 0xe0189db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd89dbd, 52 },
    { 0, 0 },
    { 0x8aa89dbf, 66 },
    { 0xa3089dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d589dc9, 33 },
    { 0, 0 },
    { 0x36689dcb, 34 },
    { 0, 0 },
    { 0xefa89dcd, 90 },
    { 0x1a089dcd, 95 },
    { 0x7d689dcf, 99 },
    { 0xe2789dcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc489dd5, 33 },
    { 0x19189dd5, 46 },
    { 0, 0 },
    { 0x68489dd8, 33 },
    { 0x70889dd8, 43 },
    { 0x5689dda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9289ddd, 33 },
    { 0x9c089dde, 33 },
    { 0x64f89ddf, 34 },
    { 0x9ce89de0, 34 },
    { 0xc7589de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f89de6, 93 },
    { 0x59489de6, 61 },
    { 0x69289de7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f89dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8889df2, 34 },
    { 0xba789df3, 91 },
    { 0xb1589df3, 23 },
    { 0x9289df5, 41 },
    { 0xf8e89df6, 90 },
    { 0x9c489df7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15389dfb, 40 },
    { 0xdf189dfc, 76 },
    { 0xb6b89dfd, 76 },
    { 0, 0 },
    { 0x6f589dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59689e03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84389e07, 34 },
    { 0x49f89e08, 43 },
    { 0xd6589e09, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x87889e0c, 96 },
    { 0x689e0d, 55 },
    { 0, 0 },
    { 0xcda89e0f, 93 },
    { 0x68989e10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f89e16, 93 },
    { 0x31489e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10489e1d, 33 },
    { 0, 0 },
    { 0x86089e1f, 76 },
    { 0xaf789e1f, 61 },
    { 0x69989e21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb89e24, 66 },
    { 0, 0 },
    { 0x90689e26, 76 },
    { 0x82d89e27, 33 },
    { 0x4b289e28, 78 },
    { 0x1d689e28, 84 },
    { 0x3e689e2a, 100 },
    { 0x8e689e29, 86 },
    { 0xd0c89e29, 61 },
    { 0x8ed89e2d, 61 },
    { 0x34689e2d, 78 },
    { 0xd8a89e2f, 90 },
    { 0xd9c89e2c, 33 },
    { 0x9fe89e31, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7389e36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3489e3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b89e45, 55 },
    { 0, 0 },
    { 0xf9289e47, 93 },
    { 0, 0 },
    { 0x15d89e49, 84 },
    { 0xd689e49, 95 },
    { 0xbc489e4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e89e4e, 17 },
    { 0, 0 },
    { 0x79889e50, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d889e55, 55 },
    { 0xc2589e56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb89e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c89e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789e61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e89e64, 33 },
    { 0, 0 },
    { 0xba589e66, 76 },
    { 0xb5289e67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc889e6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c889e70, 90 },
    { 0, 0 },
    { 0x35c89e72, 90 },
    { 0x46389e72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2189e76, 80 },
    { 0x6dc89e77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22789e7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91789e80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a89e85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68089e88, 34 },
    { 0, 0 },
    { 0xeb89e8a, 93 },
    { 0x4ed89e8b, 33 },
    { 0, 0 },
    { 0xc3289e8d, 99 },
    { 0xb6389e8e, 33 },
    { 0, 0 },
    { 0x7a89e90, 33 },
    { 0x67989e90, 91 },
    { 0, 0 },
    { 0x3ab89e93, 78 },
    { 0xaa89e94, 99 },
    { 0xaa489e95, 38 },
    { 0x1bd89e96, 93 },
    { 0xd5589e97, 83 },
    { 0x7ee89e94, 67 },
    { 0x49189e99, 61 },
    { 0x1dd89e99, 78 },
    { 0xdc689e9a, 34 },
    { 0, 0 },
    { 0x81f89e9d, 55 },
    { 0, 0 },
    { 0x78989e9f, 61 },
    { 0xf7b89e9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92189ea3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1989ea6, 90 },
    { 0x33389ea7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a89eac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb89eb4, 98 },
    { 0, 0 },
    { 0xbe89eb6, 46 },
    { 0, 0 },
    { 0x3bb89eb8, 81 },
    { 0xb9989eb9, 33 },
    { 0xe689eb8, 95 },
    { 0xa3689eb8, 90 },
    { 0xcb389ebc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb89ec0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca689ec3, 95 },
    { 0x9cd89ec4, 43 },
    { 0x1c89ec4, 33 },
    { 0x7e989ec6, 99 },
    { 0, 0 },
    { 0x52889ec8, 98 },
    { 0x23189ec9, 55 },
    { 0xb0689eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99089ecf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c189ed2, 90 },
    { 0, 0 },
    { 0xf0989ed4, 100 },
    { 0xe5c89ed5, 95 },
    { 0, 0 },
    { 0x4689ed7, 46 },
    { 0x76889ed7, 95 },
    { 0xb9489ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x34889edc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac889ee1, 93 },
    { 0x3f889ee1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34789ee8, 93 },
    { 0xe1689ee9, 23 },
    { 0x31989eea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57389eef, 87 },
    { 0x87089ef0, 76 },
    { 0, 0 },
    { 0x65589ef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f89ef5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea89efb, 33 },
    { 0x63789efb, 68 },
    { 0x19789efd, 61 },
    { 0xbd089efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc689f02, 33 },
    { 0x7cc89f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f89f0a, 33 },
    { 0xff989f0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74289f0f, 34 },
    { 0x48689f10, 55 },
    { 0xb5d89f10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78889f16, 61 },
    { 0x16289f16, 33 },
    { 0x7c89f16, 98 },
    { 0x8ce89f19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1189f1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e89f26, 76 },
    { 0xb7c89f27, 33 },
    { 0xa5089f26, 67 },
    { 0x3ff89f26, 90 },
    { 0x1ea89f2a, 78 },
    { 0xba489f26, 34 },
    { 0xf7689f2c, 61 },
    { 0x79f89f28, 61 },
    { 0x47e89f2e, 90 },
    { 0x90589f2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e89f33, 61 },
    { 0x3b89f34, 38 },
    { 0xfa089f35, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe89f3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99789f47, 76 },
    { 0, 0 },
    { 0xbcd89f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d789f4e, 76 },
    { 0x64789f4f, 34 },
    { 0x8e689f50, 90 },
    { 0, 0 },
    { 0xece89f52, 91 },
    { 0xd2889f53, 98 },
    { 0x35989f54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa089f59, 43 },
    { 0x1a89f5a, 43 },
    { 0x76089f5b, 23 },
    { 0xcca89f59, 95 },
    { 0, 0 },
    { 0xe4089f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9489f63, 80 },
    { 0x33889f64, 76 },
    { 0, 0 },
    { 0x73a89f66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb489f6b, 91 },
    { 0xbee89f6b, 33 },
    { 0xce489f6c, 98 },
    { 0xfb389f6d, 33 },
    { 0, 0 },
    { 0x86989f70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d89f78, 85 },
    { 0x60589f79, 90 },
    { 0x82e89f78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d89f80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3789f87, 90 },
    { 0, 0 },
    { 0x46a89f89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a889f8f, 61 },
    { 0x18f89f90, 85 },
    { 0xf889f91, 95 },
    { 0xa5789f91, 66 },
    { 0xb0489f93, 46 },
    { 0xb0c89f93, 99 },
    { 0xbf889f8f, 34 },
    { 0x889f96, 76 },
    { 0xd0989f97, 76 },
    { 0xf0289f97, 34 },
    { 0x1e489f8f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e89f9e, 91 },
    { 0x7ca89f9f, 98 },
    { 0x16e89fa0, 61 },
    { 0xd6889fa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a89fac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87589fb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b89fb4, 61 },
    { 0x55e89fb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f89fb8, 33 },
    { 0x3ba89fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6889fbc, 23 },
    { 0x33b89fbd, 91 },
    { 0xee889fbc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac489fc1, 90 },
    { 0x1ae89fc1, 96 },
    { 0, 0 },
    { 0xa2389fc4, 84 },
    { 0x3a89fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea89fcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9489fd0, 95 },
    { 0x72b89fd0, 33 },
    { 0x56789fd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xea589fd5, 61 },
    { 0xfb089fd6, 90 },
    { 0, 0 },
    { 0x6e889fd8, 38 },
    { 0xffb89fd9, 90 },
    { 0xdc789fda, 43 },
    { 0xab989fdb, 33 },
    { 0x82c89fdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xac389fdf, 98 },
    { 0, 0 },
    { 0x3f89fe1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26889fe8, 23 },
    { 0xd1889fe9, 81 },
    { 0x8e889fea, 52 },
    { 0xaa889fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c289fef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3989ff2, 90 },
    { 0x6f89ff2, 61 },
    { 0xf5b89ff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c389ff8, 93 },
    { 0x19489ff8, 99 },
    { 0x9de89ffa, 98 },
    { 0x38889ffb, 90 },
    { 0x93e89ffc, 61 },
    { 0x24189ffb, 33 },
    { 0x5ee89ffd, 23 },
    { 0x1b489fff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8a003, 90 },
    { 0xed98a003, 33 },
    { 0xf7d8a003, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc348a00b, 90 },
    { 0x1548a00c, 33 },
    { 0x3678a00b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca8a010, 76 },
    { 0x198a010, 43 },
    { 0xa618a012, 94 },
    { 0x7248a012, 33 },
    { 0, 0 },
    { 0x3a48a015, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf438a020, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf398a027, 95 },
    { 0xd2e8a028, 38 },
    { 0x94a8a027, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc748a02e, 55 },
    { 0, 0 },
    { 0x9b98a030, 98 },
    { 0x3938a031, 34 },
    { 0xd8a032, 95 },
    { 0x7e08a033, 90 },
    { 0x1788a032, 33 },
    { 0, 0 },
    { 0xf088a036, 90 },
    { 0x5bc8a037, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc018a03a, 55 },
    { 0, 0 },
    { 0x4c48a03c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a98a040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9618a043, 55 },
    { 0, 0 },
    { 0xe9b8a045, 100 },
    { 0, 0 },
    { 0x218a047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1488a04d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb8a051, 99 },
    { 0x7e98a051, 43 },
    { 0, 0 },
    { 0x8de8a054, 95 },
    { 0, 0 },
    { 0xeb78a056, 46 },
    { 0xd518a056, 78 },
    { 0xb8e8a058, 88 },
    { 0xb298a059, 61 },
    { 0x7c08a058, 90 },
    { 0xfdf8a056, 90 },
    { 0x2c58a05c, 43 },
    { 0x97f8a05d, 98 },
    { 0, 0 },
    { 0x3418a05f, 34 },
    { 0, 0 },
    { 0x6e88a061, 98 },
    { 0xca8a062, 33 },
    { 0x4428a063, 61 },
    { 0, 0 },
    { 0x1f18a065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3648a06b, 98 },
    { 0xdde8a06c, 90 },
    { 0, 0 },
    { 0xd9a8a06e, 46 },
    { 0xefe8a06f, 99 },
    { 0xdf78a06f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868a074, 98 },
    { 0x7538a075, 23 },
    { 0xd098a075, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da8a07a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f8a07d, 61 },
    { 0xbb08a07e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb698a089, 68 },
    { 0x948a089, 61 },
    { 0x4608a08b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2918a08e, 90 },
    { 0, 0 },
    { 0xd538a090, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d8a097, 61 },
    { 0, 0 },
    { 0x4328a099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a8a0a1, 93 },
    { 0, 0 },
    { 0x8268a0a3, 55 },
    { 0x9b8a0a3, 67 },
    { 0, 0 },
    { 0xdbf8a0a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde08a0aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c18a0ae, 34 },
    { 0xe2d8a0af, 87 },
    { 0xf4e8a0b0, 33 },
    { 0, 0 },
    { 0x56a8a0b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9218a0b5, 98 },
    { 0x2c88a0b6, 90 },
    { 0, 0 },
    { 0x5178a0b8, 33 },
    { 0x4d08a0b9, 23 },
    { 0x5118a0b9, 46 },
    { 0x2228a0bb, 76 },
    { 0xcc38a0bc, 33 },
    { 0xeb98a0b8, 34 },
    { 0x9ed8a0bc, 90 },
    { 0, 0 },
    { 0x8868a0c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d8a0c6, 43 },
    { 0, 0 },
    { 0xc4b8a0c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7338a0cc, 80 },
    { 0xeb68a0cd, 46 },
    { 0xfc8a0ce, 76 },
    { 0x78b8a0cc, 76 },
    { 0xc478a0cd, 90 },
    { 0, 0 },
    { 0x4098a0d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a58a0d5, 33 },
    { 0xc788a0d6, 52 },
    { 0, 0 },
    { 0xb5d8a0d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe8a0e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4248a0e9, 95 },
    { 0, 0 },
    { 0xf058a0eb, 98 },
    { 0x2988a0eb, 81 },
    { 0x40f8a0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d28a0f0, 76 },
    { 0x5a18a0f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df8a0f4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2338a100, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e8a103, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48a106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d8a10b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8a10f, 98 },
    { 0, 0 },
    { 0xb0a8a111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f8a115, 55 },
    { 0x5898a116, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638a120, 34 },
    { 0x3518a121, 90 },
    { 0xcd08a122, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf198a129, 46 },
    { 0, 0 },
    { 0x4b88a12b, 93 },
    { 0, 0 },
    { 0xaa48a12d, 23 },
    { 0xa888a12e, 91 },
    { 0x4908a12d, 33 },
    { 0xddb8a130, 55 },
    { 0x7fd8a131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa298a134, 81 },
    { 0x6338a135, 61 },
    { 0x348a136, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e98a13a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe668a13d, 34 },
    { 0, 0 },
    { 0x3e78a13f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5748a144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4448a148, 95 },
    { 0x5938a149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa038a14e, 23 },
    { 0x5708a14f, 65 },
    { 0x5f08a14e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca8a154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b08a15c, 91 },
    { 0x6298a15d, 90 },
    { 0, 0 },
    { 0xf178a15f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e48a165, 87 },
    { 0xb398a166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c8a16c, 61 },
    { 0, 0 },
    { 0xd3e8a16e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd928a178, 68 },
    { 0xde18a178, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc528a17d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c8a181, 76 },
    { 0xfa58a182, 91 },
    { 0xcb28a183, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8428a186, 61 },
    { 0, 0 },
    { 0xc838a188, 88 },
    { 0xd308a189, 33 },
    { 0xeb8a18a, 61 },
    { 0x34f8a18b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc8a18f, 33 },
    { 0xdeb8a190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f8a19c, 90 },
    { 0x1008a19c, 76 },
    { 0x4868a19c, 76 },
    { 0x4658a19f, 66 },
    { 0x7d08a1a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c98a1a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a8a1a7, 90 },
    { 0xc0e8a1a8, 46 },
    { 0x3408a1a8, 83 },
    { 0, 0 },
    { 0x46b8a1ab, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe588a1af, 46 },
    { 0, 0 },
    { 0x7798a1b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c8a1ba, 90 },
    { 0x1138a1bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1998a1be, 90 },
    { 0xd18a1bf, 43 },
    { 0x2138a1c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7118a1c5, 87 },
    { 0xb0b8a1c6, 43 },
    { 0x6548a1c7, 23 },
    { 0xa768a1c5, 84 },
    { 0x1568a1c9, 40 },
    { 0x9278a1ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b8a1d0, 42 },
    { 0x2cf8a1d0, 61 },
    { 0xe298a1d2, 55 },
    { 0, 0 },
    { 0xfda8a1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b8a1d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8a1da, 33 },
    { 0x9978a1db, 87 },
    { 0x6218a1db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a08a1df, 84 },
    { 0x8b8a1e0, 66 },
    { 0, 0 },
    { 0xfbc8a1e2, 87 },
    { 0x6a8a1e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc98a1e6, 34 },
    { 0x8778a1e7, 76 },
    { 0x1df8a1e8, 98 },
    { 0xe428a1e8, 98 },
    { 0, 0 },
    { 0xb928a1eb, 40 },
    { 0x978a1eb, 23 },
    { 0xbd38a1eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9658a1f0, 17 },
    { 0, 0 },
    { 0x95f8a1f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e8a1f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc668a1fe, 95 },
    { 0, 0 },
    { 0x4188a200, 33 },
    { 0, 0 },
    { 0xc6d8a202, 61 },
    { 0xf8e8a203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba38a207, 34 },
    { 0x8dd8a208, 76 },
    { 0xb5d8a209, 33 },
    { 0xdbf8a208, 84 },
    { 0, 0 },
    { 0xec68a20c, 90 },
    { 0x5758a20d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6898a210, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee18a219, 33 },
    { 0xdf8a21a, 46 },
    { 0xc718a21a, 65 },
    { 0x6508a21c, 66 },
    { 0x92c8a21b, 66 },
    { 0x458a21e, 76 },
    { 0x79f8a21d, 61 },
    { 0xd518a21c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc638a223, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5278a227, 98 },
    { 0, 0 },
    { 0xbb38a229, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb438a22f, 93 },
    { 0x67b8a230, 61 },
    { 0x7c48a22f, 43 },
    { 0x3628a230, 40 },
    { 0x5b98a232, 33 },
    { 0, 0 },
    { 0x5848a235, 43 },
    { 0x85e8a236, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48a239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a8a23d, 76 },
    { 0x3418a23d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f18a246, 98 },
    { 0xb458a246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4678a24b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d8a24e, 95 },
    { 0xa5c8a24e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38a252, 83 },
    { 0x2088a253, 93 },
    { 0x7f18a254, 69 },
    { 0x5098a255, 87 },
    { 0x66b8a256, 68 },
    { 0x5928a257, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c58a25c, 46 },
    { 0x3c18a25d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f18a261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb8a264, 90 },
    { 0xc8c8a265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd398a26e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8a271, 68 },
    { 0xb778a271, 90 },
    { 0x5528a273, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc8a27b, 23 },
    { 0xc648a27b, 85 },
    { 0xd268a27d, 85 },
    { 0xa708a27d, 46 },
    { 0xf188a27f, 95 },
    { 0x57d8a27f, 90 },
    { 0x2f08a281, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4608a287, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38a28a, 90 },
    { 0x1168a28b, 23 },
    { 0x8a18a28a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf018a294, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa58a29c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd8a2a2, 91 },
    { 0, 0 },
    { 0x5458a2a4, 95 },
    { 0xe788a2a5, 66 },
    { 0x7d08a2a6, 87 },
    { 0, 0 },
    { 0x9018a2a8, 76 },
    { 0xd5a8a2a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a8a2ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d8a2b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a8a2b5, 61 },
    { 0x5488a2b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c8a2bb, 33 },
    { 0xea98a2bb, 23 },
    { 0xfa98a2bd, 33 },
    { 0xa2c8a2be, 23 },
    { 0x2238a2bf, 95 },
    { 0, 0 },
    { 0x38f8a2c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4088a2c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d8a2c7, 23 },
    { 0, 0 },
    { 0x99d8a2c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf228a2cf, 91 },
    { 0x21b8a2d0, 65 },
    { 0x2868a2d0, 95 },
    { 0xa48a2d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda78a2d8, 99 },
    { 0xbb08a2d9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5628a2e0, 87 },
    { 0x5568a2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea68a2e7, 61 },
    { 0x18e8a2e8, 61 },
    { 0x878a2e7, 23 },
    { 0, 0 },
    { 0x5ce8a2eb, 90 },
    { 0xeb18a2ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d48a2f2, 95 },
    { 0x8ef8a2f3, 95 },
    { 0xee88a2f3, 76 },
    { 0x77f8a2f5, 33 },
    { 0xaa78a2f4, 55 },
    { 0, 0 },
    { 0xf588a2f8, 99 },
    { 0xd048a2f8, 95 },
    { 0x5228a2f9, 33 },
    { 0x90f8a2fb, 33 },
    { 0x3c08a2fa, 98 },
    { 0x79f8a2f9, 100 },
    { 0x3228a2fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed8a305, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2868a309, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa988a30f, 95 },
    { 0x7e28a310, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a88a314, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8128a31a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b8a321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e8a32a, 34 },
    { 0x7f08a32a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38a32e, 95 },
    { 0x2b38a32f, 76 },
    { 0x71f8a32f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf188a334, 61 },
    { 0xecf8a334, 83 },
    { 0x4f98a336, 33 },
    { 0xee88a335, 43 },
    { 0xcdb8a338, 93 },
    { 0x2f08a339, 23 },
    { 0x6538a334, 90 },
    { 0xade8a33b, 33 },
    { 0xa848a33c, 98 },
    { 0x45a8a338, 23 },
    { 0x95e8a339, 67 },
    { 0x1818a33f, 66 },
    { 0x2508a340, 52 },
    { 0x9ff8a341, 38 },
    { 0x2b58a341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b8a348, 83 },
    { 0, 0 },
    { 0x1e58a34a, 93 },
    { 0x4b98a34a, 25 },
    { 0xadc8a34c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c88a355, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8a360, 33 },
    { 0x6f68a361, 98 },
    { 0x5638a362, 61 },
    { 0x65e8a363, 33 },
    { 0xd848a363, 90 },
    { 0, 0 },
    { 0x3b48a366, 33 },
    { 0, 0 },
    { 0x2768a368, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8a36d, 98 },
    { 0xc318a36e, 91 },
    { 0, 0 },
    { 0x64f8a370, 98 },
    { 0, 0 },
    { 0xe968a372, 90 },
    { 0xb0b8a372, 98 },
    { 0x81f8a372, 87 },
    { 0, 0 },
    { 0x8408a376, 61 },
    { 0x7868a376, 90 },
    { 0, 0 },
    { 0x4608a379, 90 },
    { 0, 0 },
    { 0xd6a8a37b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198a37f, 85 },
    { 0, 0 },
    { 0xc5f8a381, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f8a386, 34 },
    { 0x8198a387, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6788a38d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6038a390, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528a393, 90 },
    { 0xa908a393, 90 },
    { 0, 0 },
    { 0x1d68a396, 99 },
    { 0, 0 },
    { 0x7168a398, 76 },
    { 0, 0 },
    { 0x4a18a39a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2098a39e, 95 },
    { 0, 0 },
    { 0xadb8a3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a8a3a7, 33 },
    { 0x3a28a3a7, 43 },
    { 0x8808a3a8, 65 },
    { 0x9f68a3aa, 99 },
    { 0xd668a3aa, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd38a3af, 23 },
    { 0, 0 },
    { 0x19b8a3b1, 99 },
    { 0x2538a3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc338a3b6, 67 },
    { 0x5cd8a3b6, 95 },
    { 0xc878a3b6, 97 },
    { 0x6e98a3b7, 95 },
    { 0x3f18a3ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1808a3c4, 90 },
    { 0, 0 },
    { 0xcb78a3c6, 95 },
    { 0x7c98a3c7, 23 },
    { 0, 0 },
    { 0x88d8a3c9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc8a3cf, 17 },
    { 0x2858a3d0, 33 },
    { 0xd9a8a3d0, 46 },
    { 0, 0 },
    { 0xacf8a3d3, 66 },
    { 0, 0 },
    { 0x6d18a3d5, 77 },
    { 0, 0 },
    { 0x2378a3d7, 36 },
    { 0x5de8a3d8, 78 },
    { 0x1128a3d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a8a3dc, 66 },
    { 0xdc88a3dd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b8a3e2, 90 },
    { 0x51a8a3e2, 34 },
    { 0xaf88a3e4, 96 },
    { 0x3698a3e3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe8a3ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c28a3ef, 90 },
    { 0x2e88a3ef, 90 },
    { 0xc0f8a3f1, 61 },
    { 0, 0 },
    { 0xb638a3f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba8a3fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c8a3fe, 34 },
    { 0x5e68a3ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3878a403, 61 },
    { 0, 0 },
    { 0xc388a405, 23 },
    { 0, 0 },
    { 0x6b58a407, 67 },
    { 0x56d8a408, 61 },
    { 0xf0a8a409, 61 },
    { 0, 0 },
    { 0x3638a40b, 33 },
    { 0xd708a40c, 33 },
    { 0, 0 },
    { 0x56f8a40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b8a411, 23 },
    { 0x958a412, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e98a415, 90 },
    { 0, 0 },
    { 0xee58a417, 61 },
    { 0x7d68a418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8a41b, 61 },
    { 0x6848a41c, 78 },
    { 0, 0 },
    { 0xf4d8a41e, 98 },
    { 0x1128a41f, 17 },
    { 0, 0 },
    { 0x708a421, 69 },
    { 0, 0 },
    { 0x4d98a423, 55 },
    { 0, 0 },
    { 0xdd58a425, 23 },
    { 0, 0 },
    { 0x49c8a427, 84 },
    { 0x5b78a427, 90 },
    { 0x4fa8a429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b8a42c, 33 },
    { 0x1dd8a42d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee8a432, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e78a436, 98 },
    { 0xb088a436, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb38a43c, 34 },
    { 0, 0 },
    { 0xd028a43e, 23 },
    { 0xa8f8a43f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd948a442, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e38a446, 84 },
    { 0, 0 },
    { 0x83b8a448, 93 },
    { 0x29c8a449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8308a44d, 66 },
    { 0x69a8a44e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2368a453, 95 },
    { 0x8f38a453, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff8a45a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6068a45e, 34 },
    { 0x9c18a45f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf58a463, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078a46b, 95 },
    { 0, 0 },
    { 0x7858a46d, 78 },
    { 0, 0 },
    { 0xef88a46f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6798a473, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c8a47c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a8a47f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5798a489, 33 },
    { 0x8c38a48a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908a48f, 90 },
    { 0x6de8a48f, 90 },
    { 0x4538a491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7368a494, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2478a497, 76 },
    { 0xf948a498, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa8a49c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468a4a1, 23 },
    { 0x3b78a4a2, 55 },
    { 0xe68a4a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8a4a7, 95 },
    { 0x2618a4a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5868a4b3, 66 },
    { 0x9a88a4b4, 75 },
    { 0, 0 },
    { 0xff18a4b6, 87 },
    { 0x1ba8a4b7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xae38a4ba, 61 },
    { 0x3fb8a4ba, 33 },
    { 0, 0 },
    { 0x8d28a4bd, 61 },
    { 0xc758a4be, 23 },
    { 0x6ae8a4bf, 95 },
    { 0xe7c8a4c0, 66 },
    { 0x5e8a4bd, 95 },
    { 0x3d8a4c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc8a4c5, 23 },
    { 0x7958a4c6, 23 },
    { 0x3818a4c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e88a4ca, 33 },
    { 0x43c8a4cb, 93 },
    { 0x7508a4cc, 93 },
    { 0x6978a4ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded8a4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8608a4d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8a4db, 33 },
    { 0x6ee8a4dc, 66 },
    { 0x4308a4dd, 90 },
    { 0xd8b8a4dd, 50 },
    { 0x30f8a4df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd8a4e2, 98 },
    { 0x95f8a4e3, 33 },
    { 0xb248a4e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a8a4ed, 85 },
    { 0xe188a4ee, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c8a4f5, 33 },
    { 0x6328a4f6, 100 },
    { 0xf738a4f7, 33 },
    { 0xa368a4f8, 98 },
    { 0xf9c8a4f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8a504, 90 },
    { 0, 0 },
    { 0x1888a506, 95 },
    { 0x7ef8a507, 95 },
    { 0x6ab8a508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b28a50b, 34 },
    { 0, 0 },
    { 0x3578a50d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc698a511, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d8a515, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b8a518, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7948a521, 33 },
    { 0xd398a521, 51 },
    { 0xe2b8a521, 61 },
    { 0xcae8a524, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4638a527, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f8a530, 95 },
    { 0x9798a530, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b58a538, 90 },
    { 0, 0 },
    { 0x60c8a53a, 78 },
    { 0x7258a53b, 2 },
    { 0xa588a53c, 41 },
    { 0, 0 },
    { 0x23f8a53e, 34 },
    { 0, 0 },
    { 0x76c8a540, 76 },
    { 0xdca8a540, 55 },
    { 0x98a8a542, 95 },
    { 0xb568a543, 90 },
    { 0, 0 },
    { 0x1128a545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f8a54a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8a550, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8018a556, 17 },
    { 0x3be8a556, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88a55f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f8a564, 90 },
    { 0, 0 },
    { 0x8058a566, 61 },
    { 0x8bf8a567, 68 },
    { 0xe368a567, 90 },
    { 0, 0 },
    { 0x6c38a56a, 87 },
    { 0xa248a56b, 75 },
    { 0x5638a56a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b8a572, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8a575, 81 },
    { 0x1af8a576, 90 },
    { 0x5448a576, 90 },
    { 0x5ad8a576, 33 },
    { 0, 0 },
    { 0x2268a57a, 98 },
    { 0xa9c8a57b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf38a57f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8098a583, 55 },
    { 0, 0 },
    { 0xddb8a585, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2668a58b, 55 },
    { 0, 0 },
    { 0xbd98a58d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde8a590, 96 },
    { 0x1f28a591, 91 },
    { 0x9438a592, 33 },
    { 0x8148a593, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d8a596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb8a599, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2388a59e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9378a5a6, 75 },
    { 0xd238a5a7, 90 },
    { 0, 0 },
    { 0x5c48a5a9, 78 },
    { 0xe668a5a9, 61 },
    { 0x32e8a5a9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1288a5af, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5428a5b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba68a5b8, 78 },
    { 0xf748a5b9, 98 },
    { 0x5808a5b8, 93 },
    { 0x7628a5b8, 98 },
    { 0x5f78a5bc, 34 },
    { 0xb7c8a5bc, 55 },
    { 0, 0 },
    { 0x828a5bf, 90 },
    { 0x9448a5c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc8a5c3, 99 },
    { 0x4308a5c3, 65 },
    { 0, 0 },
    { 0x54d8a5c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd308a5c9, 33 },
    { 0x1d58a5ca, 100 },
    { 0, 0 },
    { 0xf748a5cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7488a5d0, 33 },
    { 0x4138a5d1, 33 },
    { 0xb188a5d2, 77 },
    { 0xe298a5d3, 61 },
    { 0x678a5d2, 76 },
    { 0x4e68a5d5, 84 },
    { 0x4268a5d4, 23 },
    { 0xe558a5d7, 33 },
    { 0, 0 },
    { 0x7078a5d9, 23 },
    { 0, 0 },
    { 0xa628a5db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4208a5de, 49 },
    { 0, 0 },
    { 0xfc8a5e0, 98 },
    { 0x8278a5e1, 76 },
    { 0x1c98a5e0, 34 },
    { 0xd8a8a5e3, 55 },
    { 0, 0 },
    { 0xcc8a5e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe798a5ed, 46 },
    { 0x5a08a5ed, 78 },
    { 0x46e8a5ef, 34 },
    { 0xfd08a5f0, 90 },
    { 0x41b8a5f0, 96 },
    { 0x7ee8a5ed, 98 },
    { 0, 0 },
    { 0x3b58a5f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8a5f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf8a5fb, 99 },
    { 0x8528a5fc, 95 },
    { 0x1c28a5fc, 98 },
    { 0x6938a5fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6118a601, 99 },
    { 0x7138a602, 61 },
    { 0x8d58a603, 52 },
    { 0xad08a603, 61 },
    { 0xb428a605, 23 },
    { 0x5cb8a606, 33 },
    { 0xe5e8a604, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f58a60a, 98 },
    { 0, 0 },
    { 0xb0a8a60c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4988a60f, 46 },
    { 0xf1d8a60f, 23 },
    { 0xa68a60f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f48a615, 95 },
    { 0x7838a616, 55 },
    { 0, 0 },
    { 0x4678a618, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef18a61f, 17 },
    { 0x9e38a61f, 61 },
    { 0, 0 },
    { 0xd588a622, 33 },
    { 0, 0 },
    { 0x4af8a624, 61 },
    { 0, 0 },
    { 0xc458a626, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1138a629, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce68a62e, 46 },
    { 0, 0 },
    { 0x2b18a630, 98 },
    { 0, 0 },
    { 0x61d8a632, 23 },
    { 0x8658a632, 33 },
    { 0xd5e8a634, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b78a63b, 33 },
    { 0x3818a63c, 61 },
    { 0x5ea8a63d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e48a640, 23 },
    { 0x4d8a641, 99 },
    { 0xe878a642, 91 },
    { 0x3988a643, 55 },
    { 0xcbd8a644, 63 },
    { 0, 0 },
    { 0xa938a646, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd8a64c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98a657, 87 },
    { 0, 0 },
    { 0x90d8a659, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd8a65c, 78 },
    { 0, 0 },
    { 0x6958a65e, 46 },
    { 0, 0 },
    { 0x618a660, 76 },
    { 0x688a661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e8a665, 46 },
    { 0x1a48a666, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec8a669, 55 },
    { 0x3088a66a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8a672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7228a677, 76 },
    { 0xe618a678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd578a67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8a67e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f8a681, 93 },
    { 0, 0 },
    { 0x4ea8a683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d58a687, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd78a68f, 61 },
    { 0x6cf8a690, 61 },
    { 0x5088a691, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb8a694, 34 },
    { 0xa288a695, 98 },
    { 0x888a696, 45 },
    { 0x5ba8a697, 46 },
    { 0, 0 },
    { 0x1a48a699, 87 },
    { 0, 0 },
    { 0x9db8a69b, 90 },
    { 0xdb48a69b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc8a6a1, 90 },
    { 0x8a28a6a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba8a6ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4248a6af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c68a6b3, 33 },
    { 0x6748a6b3, 23 },
    { 0xf518a6b5, 66 },
    { 0x2ff8a6b3, 55 },
    { 0x7f8a6b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb038a6ba, 94 },
    { 0x6a48a6bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3988a6be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e8a6c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a08a6c7, 100 },
    { 0x4678a6c8, 23 },
    { 0x4f18a6c8, 46 },
    { 0, 0 },
    { 0x408a6cb, 34 },
    { 0x71a8a6cc, 98 },
    { 0x9d8a6cd, 90 },
    { 0x4998a6ce, 66 },
    { 0x8bd8a6cf, 66 },
    { 0x628a6cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5158a6db, 76 },
    { 0, 0 },
    { 0xf978a6dd, 90 },
    { 0x3da8a6dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8a6e5, 61 },
    { 0x5f58a6e6, 23 },
    { 0x8f58a6e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa668a6f0, 90 },
    { 0x4708a6f1, 80 },
    { 0xff68a6f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb48a6f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88a6fa, 90 },
    { 0xe348a6fb, 55 },
    { 0, 0 },
    { 0x81d8a6fd, 61 },
    { 0x3d78a6fd, 98 },
    { 0xf088a6fd, 23 },
    { 0x2e48a6ff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8a703, 77 },
    { 0, 0 },
    { 0xc308a705, 46 },
    { 0x898a705, 100 },
    { 0, 0 },
    { 0x93e8a708, 90 },
    { 0x50f8a709, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e58a70c, 46 },
    { 0, 0 },
    { 0xdaa8a70e, 61 },
    { 0, 0 },
    { 0xb848a710, 90 },
    { 0, 0 },
    { 0xddc8a712, 88 },
    { 0xc308a713, 17 },
    { 0xcf88a714, 90 },
    { 0x6bd8a715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f08a718, 90 },
    { 0x5848a719, 78 },
    { 0xae88a718, 90 },
    { 0xf2b8a71b, 33 },
    { 0, 0 },
    { 0xc018a71d, 93 },
    { 0xc068a71e, 66 },
    { 0xb928a71f, 98 },
    { 0x7cf8a720, 34 },
    { 0x3fd8a71f, 33 },
    { 0xf328a722, 78 },
    { 0xba88a722, 91 },
    { 0xd118a71e, 98 },
    { 0x5658a71e, 90 },
    { 0xa348a71e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb8a729, 66 },
    { 0, 0 },
    { 0x1008a72b, 93 },
    { 0, 0 },
    { 0x25f8a72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648a731, 61 },
    { 0xa488a732, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d8a735, 90 },
    { 0x48f8a736, 52 },
    { 0x6008a737, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad8a73c, 88 },
    { 0, 0 },
    { 0x2548a73e, 95 },
    { 0, 0 },
    { 0xca58a740, 33 },
    { 0, 0 },
    { 0x7a58a742, 52 },
    { 0, 0 },
    { 0x7ed8a744, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd108a74c, 55 },
    { 0xd908a74d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538a751, 33 },
    { 0, 0 },
    { 0x37d8a753, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9238a758, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e88a75c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c8a764, 95 },
    { 0x2f88a765, 78 },
    { 0x9f48a764, 61 },
    { 0xaf58a766, 95 },
    { 0xd608a764, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb48a76e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf898a776, 61 },
    { 0xefd8a776, 33 },
    { 0x21d8a776, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a782, 98 },
    { 0, 0 },
    { 0xacb8a784, 51 },
    { 0, 0 },
    { 0xf178a786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5908a78d, 61 },
    { 0, 0 },
    { 0xff78a78f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a8a794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe618a797, 90 },
    { 0xa908a797, 85 },
    { 0, 0 },
    { 0xd7a8a79a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa228a7a2, 90 },
    { 0x2ae8a7a3, 58 },
    { 0x3e18a7a4, 33 },
    { 0, 0 },
    { 0x7e98a7a6, 23 },
    { 0x36a8a7a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc388a7aa, 52 },
    { 0, 0 },
    { 0xebc8a7ac, 61 },
    { 0xef28a7ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8a7b8, 84 },
    { 0x69e8a7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7378a7be, 98 },
    { 0x218a7bf, 55 },
    { 0xf638a7bf, 61 },
    { 0x7eb8a7bf, 98 },
    { 0xc488a7c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78a7c8, 76 },
    { 0x798a7c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7238a7cd, 34 },
    { 0, 0 },
    { 0xc8e8a7cf, 95 },
    { 0x5e78a7d0, 93 },
    { 0x128a7d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a7d6, 91 },
    { 0x6bc8a7d7, 68 },
    { 0x9c98a7d8, 98 },
    { 0x21a8a7d9, 55 },
    { 0, 0 },
    { 0x8e98a7db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b8a7e2, 66 },
    { 0xaa98a7e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee68a7e7, 90 },
    { 0x748a7e7, 90 },
    { 0xef38a7e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f8a7ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf98a7f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd478a7f9, 61 },
    { 0xe688a7fa, 90 },
    { 0xd838a7fa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a8a7fe, 46 },
    { 0xd378a7fe, 80 },
    { 0xb298a7ff, 76 },
    { 0xee58a801, 98 },
    { 0xfbe8a801, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8a807, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e08a80d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd98a817, 52 },
    { 0x3418a817, 95 },
    { 0xe1f8a819, 66 },
    { 0, 0 },
    { 0xf5a8a81b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5498a81e, 61 },
    { 0, 0 },
    { 0x1298a820, 64 },
    { 0, 0 },
    { 0xccb8a822, 55 },
    { 0x8d38a823, 66 },
    { 0x8e68a824, 90 },
    { 0x58a823, 23 },
    { 0x23b8a826, 56 },
    { 0x4f8a827, 61 },
    { 0xe898a828, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f8a82b, 90 },
    { 0x3a48a82b, 83 },
    { 0x9d38a82d, 66 },
    { 0x1098a82e, 90 },
    { 0, 0 },
    { 0x1818a830, 33 },
    { 0, 0 },
    { 0x4bd8a832, 77 },
    { 0x5b48a832, 98 },
    { 0xe5c8a834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2258a83b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2048a83e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7008a843, 44 },
    { 0, 0 },
    { 0x1858a845, 93 },
    { 0xafa8a846, 90 },
    { 0x1bd8a847, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608a84a, 23 },
    { 0x6d08a84b, 98 },
    { 0, 0 },
    { 0x6cf8a84d, 76 },
    { 0xbe8a84d, 95 },
    { 0x6388a84f, 33 },
    { 0, 0 },
    { 0x6e28a851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a8a859, 46 },
    { 0x68e8a859, 85 },
    { 0x76a8a859, 23 },
    { 0xc0f8a85b, 90 },
    { 0xcf58a85a, 43 },
    { 0xe2b8a85e, 55 },
    { 0xda8a85a, 98 },
    { 0x3a48a859, 76 },
    { 0xa658a85c, 90 },
    { 0, 0 },
    { 0x1a98a863, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e58a866, 55 },
    { 0, 0 },
    { 0x3558a868, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9188a86b, 61 },
    { 0xdc88a86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5188a86f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238a873, 23 },
    { 0xc168a874, 85 },
    { 0x6188a875, 33 },
    { 0x5d58a876, 34 },
    { 0x73d8a876, 93 },
    { 0xc198a878, 61 },
    { 0x44b8a879, 91 },
    { 0xc6b8a874, 90 },
    { 0, 0 },
    { 0x3c68a87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd78a883, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398a886, 67 },
    { 0, 0 },
    { 0x3d18a888, 64 },
    { 0xb288a889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc068a88e, 23 },
    { 0xba08a88f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc8a898, 90 },
    { 0xe048a899, 98 },
    { 0x2c78a89a, 61 },
    { 0x18c8a89b, 90 },
    { 0x8368a89c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8688a8a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae8a8a4, 95 },
    { 0, 0 },
    { 0x2d28a8a6, 76 },
    { 0xc7e8a8a7, 98 },
    { 0x5018a8a8, 34 },
    { 0x53f8a8a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7018a8b1, 23 },
    { 0x49f8a8b1, 98 },
    { 0x2cf8a8b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4288a8bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc068a8c1, 98 },
    { 0x9b68a8c1, 46 },
    { 0x9678a8c3, 43 },
    { 0xbb58a8c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f8a8d1, 68 },
    { 0x3a68a8d1, 55 },
    { 0x4ee8a8d2, 33 },
    { 0xbf68a8d4, 90 },
    { 0x44e8a8d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a48a8d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa008a8dc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6968a8e3, 44 },
    { 0, 0 },
    { 0x2be8a8e5, 95 },
    { 0x7198a8e6, 23 },
    { 0x5358a8e5, 85 },
    { 0xf9e8a8e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c58a8eb, 34 },
    { 0xee58a8ec, 90 },
    { 0xe918a8ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a58a8f1, 76 },
    { 0x3468a8f2, 84 },
    { 0x2b78a8f3, 34 },
    { 0, 0 },
    { 0xaca8a8f5, 66 },
    { 0x47b8a8f5, 33 },
    { 0, 0 },
    { 0xc848a8f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1668a8fb, 23 },
    { 0, 0 },
    { 0xf228a8fd, 90 },
    { 0x6d8a8fe, 90 },
    { 0x75f8a8ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e8a905, 90 },
    { 0, 0 },
    { 0xc2b8a907, 91 },
    { 0xb698a907, 43 },
    { 0xc3a8a909, 91 },
    { 0x7938a90a, 76 },
    { 0x4fa8a90b, 40 },
    { 0xf6a8a90c, 33 },
    { 0xec88a90d, 78 },
    { 0x1b08a90c, 98 },
    { 0x64d8a90c, 95 },
    { 0xd978a90d, 98 },
    { 0xab8a911, 61 },
    { 0xb048a90a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b8a917, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba8a91e, 52 },
    { 0xafd8a91e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd898a924, 23 },
    { 0x74b8a925, 23 },
    { 0x77e8a926, 43 },
    { 0x7ae8a927, 99 },
    { 0x9768a927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc508a92b, 95 },
    { 0x21b8a92c, 23 },
    { 0, 0 },
    { 0xefe8a92e, 55 },
    { 0, 0 },
    { 0x48f8a930, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe858a934, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f8a937, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb8a93a, 61 },
    { 0xcba8a93b, 82 },
    { 0xb068a93b, 100 },
    { 0xad18a93d, 23 },
    { 0, 0 },
    { 0x48a93f, 98 },
    { 0x3a48a940, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08a94a, 93 },
    { 0xa718a94b, 99 },
    { 0xcb78a94b, 61 },
    { 0, 0 },
    { 0xa0b8a94e, 55 },
    { 0xf18a94f, 90 },
    { 0, 0 },
    { 0x2388a951, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8a954, 46 },
    { 0xd838a955, 78 },
    { 0x98a955, 98 },
    { 0, 0 },
    { 0x4078a958, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb8a95c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a8a964, 76 },
    { 0xf818a965, 63 },
    { 0, 0 },
    { 0xca88a967, 87 },
    { 0x67c8a968, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef58a96d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b8a971, 43 },
    { 0xab48a972, 23 },
    { 0x6118a973, 90 },
    { 0x57b8a974, 61 },
    { 0x72d8a975, 76 },
    { 0xf3e8a976, 36 },
    { 0x75b8a976, 83 },
    { 0xf508a976, 100 },
    { 0xf378a979, 87 },
    { 0xd458a97a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe098a97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd38a987, 90 },
    { 0x39f8a988, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c8a98b, 90 },
    { 0xb2b8a98c, 87 },
    { 0x3d68a98d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee8a991, 75 },
    { 0x77b8a992, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa08a995, 33 },
    { 0x9a28a996, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb258a99f, 61 },
    { 0xe7c8a99f, 95 },
    { 0x9768a9a1, 95 },
    { 0x2d38a99f, 23 },
    { 0, 0 },
    { 0x4628a9a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb378a9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf428a9ab, 78 },
    { 0xa468a9ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c48a9b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08a9b7, 95 },
    { 0xe958a9b8, 83 },
    { 0xe3a8a9b8, 77 },
    { 0x25c8a9b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78a9c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9778a9c5, 61 },
    { 0x4638a9c5, 23 },
    { 0, 0 },
    { 0xf468a9c8, 91 },
    { 0x9ce8a9c9, 33 },
    { 0x95d8a9c9, 85 },
    { 0xb6a8a9cb, 66 },
    { 0xc8a9ca, 61 },
    { 0, 0 },
    { 0x8868a9ce, 23 },
    { 0, 0 },
    { 0xfb18a9d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff8a9d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b48a9d8, 33 },
    { 0x31b8a9d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd858a9dc, 81 },
    { 0, 0 },
    { 0x2678a9de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad88a9e6, 95 },
    { 0x8668a9e7, 61 },
    { 0x14c8a9e8, 23 },
    { 0x3878a9e9, 85 },
    { 0xad58a9ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca8a9fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8918aa01, 33 },
    { 0x5898aa01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff8aa08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f8aa13, 43 },
    { 0x7a68aa14, 33 },
    { 0, 0 },
    { 0x37c8aa16, 98 },
    { 0, 0 },
    { 0x5348aa18, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8aa1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c8aa1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c8aa21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa018aa25, 61 },
    { 0xc7e8aa26, 90 },
    { 0, 0 },
    { 0x8c88aa28, 23 },
    { 0x9e58aa28, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf8aa31, 65 },
    { 0x9fc8aa31, 34 },
    { 0, 0 },
    { 0xa908aa34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1518aa38, 33 },
    { 0, 0 },
    { 0x41f8aa3a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e8aa3d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xde98aa40, 84 },
    { 0xc688aa41, 64 },
    { 0, 0 },
    { 0x5e18aa43, 66 },
    { 0x15c8aa44, 24 },
    { 0, 0 },
    { 0x2698aa46, 23 },
    { 0, 0 },
    { 0xc5a8aa48, 23 },
    { 0, 0 },
    { 0xdd68aa4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8aa4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8668aa51, 99 },
    { 0x2278aa51, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2028aa56, 88 },
    { 0x6c18aa57, 81 },
    { 0xfb58aa58, 90 },
    { 0x8328aa57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58aa5c, 46 },
    { 0xffc8aa5d, 33 },
    { 0xfcd8aa5e, 76 },
    { 0x5fb8aa5f, 90 },
    { 0xf288aa5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc8aa63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec78aa67, 98 },
    { 0xc778aa67, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d98aa6b, 34 },
    { 0, 0 },
    { 0x2a88aa6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde58aa70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6858aa73, 90 },
    { 0xa5d8aa74, 90 },
    { 0x1f98aa74, 56 },
    { 0x7158aa76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe28aa7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8878aa7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e98aa80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce68aa83, 34 },
    { 0xc678aa84, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd138aa89, 98 },
    { 0, 0 },
    { 0x5fd8aa8b, 95 },
    { 0, 0 },
    { 0xcdf8aa8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7718aa92, 90 },
    { 0x9f98aa92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8aa97, 34 },
    { 0xb8f8aa97, 76 },
    { 0, 0 },
    { 0xd3f8aa9a, 61 },
    { 0, 0 },
    { 0x4648aa9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28aaa5, 100 },
    { 0x818aaa6, 91 },
    { 0x22f8aaa5, 95 },
    { 0xda8aaa8, 67 },
    { 0x9958aaa9, 100 },
    { 0x6868aaa7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2188aaad, 66 },
    { 0xe7c8aaad, 61 },
    { 0x11b8aaad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b98aab4, 34 },
    { 0xd228aab5, 90 },
    { 0x3828aab6, 38 },
    { 0x2bc8aab7, 90 },
    { 0x9458aab5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88aabb, 33 },
    { 0x65e8aabc, 76 },
    { 0, 0 },
    { 0x96a8aabe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7798aac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca78aad4, 90 },
    { 0xaca8aad5, 43 },
    { 0x5a38aad6, 17 },
    { 0xfa38aad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a8aadd, 55 },
    { 0, 0 },
    { 0xd508aadf, 93 },
    { 0xe558aae0, 85 },
    { 0xc0a8aae1, 90 },
    { 0xa48aae2, 61 },
    { 0xb3d8aae3, 76 },
    { 0x8398aae3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8aaf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf228aafa, 80 },
    { 0xe028aafb, 38 },
    { 0xa3c8aafc, 49 },
    { 0, 0 },
    { 0x7c58aafe, 23 },
    { 0xb528aaff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9468ab04, 95 },
    { 0x79d8ab05, 75 },
    { 0xcd98ab06, 76 },
    { 0x8878ab07, 67 },
    { 0xa3f8ab08, 98 },
    { 0xb508ab07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158ab0d, 46 },
    { 0, 0 },
    { 0x8878ab0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8ab12, 90 },
    { 0x1358ab12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b8ab16, 90 },
    { 0, 0 },
    { 0x5668ab18, 66 },
    { 0xdd58ab18, 85 },
    { 0x3508ab1a, 87 },
    { 0xd408ab1b, 61 },
    { 0x8d68ab1c, 33 },
    { 0, 0 },
    { 0xdad8ab1e, 43 },
    { 0, 0 },
    { 0x8f38ab20, 93 },
    { 0xc398ab21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade8ab25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf98ab2b, 55 },
    { 0xb448ab2c, 100 },
    { 0, 0 },
    { 0xe2d8ab2e, 90 },
    { 0, 0 },
    { 0x9588ab30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1438ab34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8678ab3b, 33 },
    { 0, 0 },
    { 0xb358ab3d, 93 },
    { 0xc9a8ab3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848ab42, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc658ab48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7508ab4f, 98 },
    { 0x32a8ab50, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb58ab53, 33 },
    { 0, 0 },
    { 0xc8ab55, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4268ab59, 33 },
    { 0x5e58ab59, 61 },
    { 0x9828ab5b, 61 },
    { 0x2c58ab5c, 61 },
    { 0x6a38ab5d, 43 },
    { 0x9f88ab5e, 90 },
    { 0x4128ab5e, 76 },
    { 0x6a28ab60, 100 },
    { 0x4438ab5c, 43 },
    { 0xa448ab62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e8ab65, 90 },
    { 0x8af8ab66, 90 },
    { 0xd3b8ab66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a8ab6a, 67 },
    { 0, 0 },
    { 0x5eb8ab6c, 98 },
    { 0xda78ab6d, 95 },
    { 0x808ab6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a88ab74, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe88ab78, 67 },
    { 0xcf08ab78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8ab7d, 33 },
    { 0, 0 },
    { 0x5b48ab7f, 33 },
    { 0x4048ab7f, 34 },
    { 0x4508ab80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc048ab87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd458ab8a, 90 },
    { 0, 0 },
    { 0x5558ab8c, 90 },
    { 0, 0 },
    { 0x5848ab8e, 61 },
    { 0xd58ab8f, 53 },
    { 0xbea8ab90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef8ab95, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d8ab98, 33 },
    { 0, 0 },
    { 0x56a8ab9a, 61 },
    { 0x7ea8ab9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc8aba3, 93 },
    { 0x8618aba4, 61 },
    { 0x8798aba5, 98 },
    { 0xa598aba6, 96 },
    { 0, 0 },
    { 0x4c78aba8, 98 },
    { 0x9228aba8, 84 },
    { 0xa908abaa, 23 },
    { 0, 0 },
    { 0x2ff8abac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5738abb0, 93 },
    { 0, 0 },
    { 0x988abb2, 87 },
    { 0, 0 },
    { 0xea8abb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9158abb9, 33 },
    { 0x4378abba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f28abbf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a88abc3, 98 },
    { 0xd718abc3, 95 },
    { 0x8048abc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c8abc9, 61 },
    { 0xaed8abca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x868abcd, 76 },
    { 0x1108abcd, 33 },
    { 0x1718abcf, 76 },
    { 0x9b38abd0, 33 },
    { 0xc4b8abd0, 61 },
    { 0xbc8abd2, 69 },
    { 0x5048abd2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c8abd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x378abda, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c8abde, 52 },
    { 0x9308abde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc8abe4, 61 },
    { 0x24b8abe5, 61 },
    { 0x5b88abe6, 23 },
    { 0xd98abe5, 61 },
    { 0x3718abe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea8abeb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3998abee, 61 },
    { 0, 0 },
    { 0xf788abf0, 61 },
    { 0x5018abf1, 61 },
    { 0x2318abf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2828abf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b28abf8, 61 },
    { 0x5ca8abf8, 78 },
    { 0x6068abfa, 33 },
    { 0xefd8abfa, 90 },
    { 0xe208abfb, 95 },
    { 0x14e8abfd, 90 },
    { 0x4558abfe, 43 },
    { 0xf488abff, 33 },
    { 0, 0 },
    { 0x4ff8ac01, 23 },
    { 0, 0 },
    { 0x1a48ac03, 95 },
    { 0, 0 },
    { 0xa78ac05, 34 },
    { 0, 0 },
    { 0xd288ac07, 98 },
    { 0x5b68ac08, 90 },
    { 0xdeb8ac09, 23 },
    { 0xc338ac0a, 67 },
    { 0x6e48ac0a, 52 },
    { 0x9ef8ac0c, 81 },
    { 0x6038ac0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7268ac14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac19, 81 },
    { 0x9488ac19, 98 },
    { 0x42b8ac1a, 33 },
    { 0x71a8ac1c, 66 },
    { 0xc818ac1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64b8ac23, 68 },
    { 0x5a18ac23, 33 },
    { 0xb658ac24, 76 },
    { 0xcef8ac26, 34 },
    { 0, 0 },
    { 0x30c8ac28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e8ac2d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc498ac30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce48ac33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8ac38, 61 },
    { 0xb4b8ac38, 75 },
    { 0xf8a8ac3a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xef98ac3d, 36 },
    { 0xa68ac3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf388ac41, 95 },
    { 0, 0 },
    { 0x97e8ac43, 98 },
    { 0x2148ac44, 33 },
    { 0xd8a8ac45, 88 },
    { 0, 0 },
    { 0xec78ac47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde98ac4f, 66 },
    { 0x2e18ac50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d08ac53, 90 },
    { 0, 0 },
    { 0xad08ac55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd58ac60, 46 },
    { 0xa0d8ac61, 98 },
    { 0x7708ac62, 76 },
    { 0, 0 },
    { 0x3248ac64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a8ac67, 95 },
    { 0x6368ac67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88ac6c, 93 },
    { 0x32d8ac6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1148ac71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe678ac75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f58ac78, 91 },
    { 0x92c8ac79, 90 },
    { 0, 0 },
    { 0xeaf8ac7b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a68ac7e, 90 },
    { 0xa58ac7f, 91 },
    { 0x45f8ac7e, 84 },
    { 0xf218ac81, 55 },
    { 0, 0 },
    { 0x41c8ac83, 33 },
    { 0x74f8ac83, 94 },
    { 0, 0 },
    { 0x6648ac86, 46 },
    { 0x5bc8ac86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5018ac92, 90 },
    { 0, 0 },
    { 0x3418ac94, 78 },
    { 0, 0 },
    { 0xe118ac96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538ac9a, 46 },
    { 0, 0 },
    { 0x128ac9c, 95 },
    { 0x2358ac9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d8aca0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd8aca3, 23 },
    { 0x5df8aca3, 95 },
    { 0, 0 },
    { 0x9718aca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a38acac, 90 },
    { 0xa068acad, 90 },
    { 0, 0 },
    { 0xac08acaf, 90 },
    { 0, 0 },
    { 0xfa08acb1, 23 },
    { 0xe5f8acb1, 87 },
    { 0x9578acb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef58acbe, 23 },
    { 0x2ef8acbf, 23 },
    { 0x9f98acc0, 55 },
    { 0xa328acc1, 61 },
    { 0x4088acc1, 76 },
    { 0xa308acbf, 95 },
    { 0x8c18acc4, 99 },
    { 0x1808acc5, 66 },
    { 0x1ec8acbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c8accb, 90 },
    { 0xca28accb, 46 },
    { 0xffe8accd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718acd2, 98 },
    { 0x82b8acd3, 33 },
    { 0x41b8acd4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d88ace1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5468ace5, 52 },
    { 0x6f78ace5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb8aceb, 23 },
    { 0x9c48aceb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf898acef, 76 },
    { 0, 0 },
    { 0xac98acf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f68acf8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee28acfd, 90 },
    { 0x3558acfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d88ad02, 85 },
    { 0xdd38ad02, 66 },
    { 0xaeb8ad04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb138ad09, 95 },
    { 0, 0 },
    { 0x1c98ad0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938ad15, 43 },
    { 0, 0 },
    { 0xea98ad17, 80 },
    { 0x1048ad18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7908ad1f, 76 },
    { 0x92b8ad20, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908ad23, 61 },
    { 0x2388ad24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08ad2d, 95 },
    { 0xc2a8ad2e, 90 },
    { 0xb748ad2d, 98 },
    { 0xbe48ad30, 23 },
    { 0x1128ad30, 93 },
    { 0xe0e8ad32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb128ad36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd818ad39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd198ad3e, 100 },
    { 0x558ad3f, 90 },
    { 0x20d8ad3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398ad43, 61 },
    { 0, 0 },
    { 0xcbf8ad45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb8ad48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x448ad4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa298ad4f, 46 },
    { 0xce8ad50, 33 },
    { 0xefd8ad51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa378ad54, 99 },
    { 0x10f8ad55, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e8ad58, 100 },
    { 0, 0 },
    { 0xae98ad5a, 55 },
    { 0xe3b8ad5a, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118ad63, 55 },
    { 0x6728ad64, 90 },
    { 0xfe78ad65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c8ad6a, 33 },
    { 0x6a18ad6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8898ad6e, 95 },
    { 0, 0 },
    { 0x29c8ad70, 76 },
    { 0xd68ad71, 88 },
    { 0xd648ad70, 64 },
    { 0x8c78ad73, 52 },
    { 0xa6f8ad73, 61 },
    { 0x6fa8ad75, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf8ad78, 88 },
    { 0xd038ad78, 95 },
    { 0x6368ad7a, 43 },
    { 0x8d48ad7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8ad7f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad8ad84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8ad88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3528ad8e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xa828ad91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e8ad9c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae8ad9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a8ada5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9648ada8, 61 },
    { 0x7008ada9, 23 },
    { 0xa538ada9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278adaf, 55 },
    { 0, 0 },
    { 0x47d8adb1, 99 },
    { 0x4928adb2, 87 },
    { 0, 0 },
    { 0xf9c8adb4, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1218adb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88adc1, 55 },
    { 0, 0 },
    { 0x4488adc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b8add6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd418adde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ade6, 85 },
    { 0xf358ade7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe38aded, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1968adf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e98adf8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88adff, 46 },
    { 0xc3f8ae00, 23 },
    { 0xb568ae01, 76 },
    { 0x9988ae01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa8ae06, 55 },
    { 0, 0 },
    { 0x8eb8ae08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688ae0b, 23 },
    { 0xee18ae0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c8ae0f, 90 },
    { 0x9448ae10, 23 },
    { 0, 0 },
    { 0x3618ae12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f8ae15, 33 },
    { 0xafb8ae16, 98 },
    { 0x5788ae17, 61 },
    { 0x9128ae18, 98 },
    { 0x94f8ae19, 90 },
    { 0xd6d8ae18, 66 },
    { 0xad78ae1a, 36 },
    { 0x92e8ae1c, 23 },
    { 0xdbb8ae1c, 93 },
    { 0x7f8ae18, 95 },
    { 0xcac8ae16, 95 },
    { 0xd808ae1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf928ae23, 99 },
    { 0xdcd8ae24, 83 },
    { 0x4588ae24, 98 },
    { 0xe768ae24, 34 },
    { 0xe48ae27, 46 },
    { 0xd358ae28, 66 },
    { 0xb328ae28, 61 },
    { 0x4e08ae29, 23 },
    { 0xa6e8ae2a, 53 },
    { 0, 0 },
    { 0x4688ae2d, 99 },
    { 0, 0 },
    { 0x3438ae2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1038ae33, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe818ae3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2868ae40, 90 },
    { 0x7068ae41, 66 },
    { 0xa9a8ae41, 23 },
    { 0x13a8ae42, 33 },
    { 0x308ae40, 67 },
    { 0xa488ae41, 76 },
    { 0xf8d8ae46, 84 },
    { 0x5a18ae46, 33 },
    { 0x6c98ae48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa768ae4c, 36 },
    { 0xc6f8ae4d, 90 },
    { 0x3d08ae4d, 61 },
    { 0xcf98ae4f, 61 },
    { 0x4ea8ae4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d38ae5c, 98 },
    { 0x8058ae5d, 39 },
    { 0, 0 },
    { 0x1388ae5f, 61 },
    { 0x4538ae60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9428ae65, 98 },
    { 0xdd88ae65, 98 },
    { 0xccb8ae65, 98 },
    { 0x7958ae68, 61 },
    { 0x9cd8ae69, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ae6c, 75 },
    { 0xe3c8ae6d, 43 },
    { 0x5e48ae6e, 95 },
    { 0x50e8ae6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce8ae75, 55 },
    { 0, 0 },
    { 0xd838ae77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd8ae7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc8ae81, 84 },
    { 0, 0 },
    { 0xa3f8ae83, 43 },
    { 0, 0 },
    { 0x8a28ae85, 17 },
    { 0x18a8ae86, 90 },
    { 0xc008ae85, 61 },
    { 0, 0 },
    { 0xd3e8ae89, 33 },
    { 0x7078ae8a, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f58ae8d, 75 },
    { 0x2a18ae8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e8ae91, 55 },
    { 0xc9a8ae92, 33 },
    { 0xcae8ae91, 90 },
    { 0, 0 },
    { 0x7268ae95, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x4308ae98, 24 },
    { 0, 0 },
    { 0xb528ae9a, 95 },
    { 0x3c98ae9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa718ae9f, 90 },
    { 0xe528aea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd358aead, 65 },
    { 0, 0 },
    { 0x608aeaf, 85 },
    { 0xbc28aeaf, 33 },
    { 0x1208aeb1, 33 },
    { 0, 0 },
    { 0x65d8aeb3, 33 },
    { 0xb128aeb4, 85 },
    { 0x4078aeb4, 52 },
    { 0x48b8aeb6, 91 },
    { 0xdaa8aeb5, 81 },
    { 0x70a8aeb3, 33 },
    { 0x8598aeb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a78aebd, 46 },
    { 0x4598aebe, 33 },
    { 0x2de8aebf, 23 },
    { 0x53e8aebf, 90 },
    { 0x42c8aebe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb8aec4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb578aec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d8aecb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5438aed1, 61 },
    { 0, 0 },
    { 0x7d58aed3, 33 },
    { 0xd948aed4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2648aed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1108aedb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d38aee3, 90 },
    { 0, 0 },
    { 0x8bf8aee5, 43 },
    { 0x1008aee6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a88aeea, 65 },
    { 0x42f8aeeb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a08aeef, 23 },
    { 0xf28aef0, 23 },
    { 0x4198aeef, 17 },
    { 0, 0 },
    { 0x7068aef3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a8aefc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e8af01, 98 },
    { 0, 0 },
    { 0xfe8af03, 90 },
    { 0, 0 },
    { 0x1638af05, 98 },
    { 0x1f98af06, 33 },
    { 0xd578af06, 23 },
    { 0x1278af07, 93 },
    { 0, 0 },
    { 0xba8af0a, 33 },
    { 0, 0 },
    { 0xb0d8af0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd168af0f, 76 },
    { 0, 0 },
    { 0xf9c8af11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd78af14, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c8af17, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d8af1a, 23 },
    { 0, 0 },
    { 0xc3e8af1c, 33 },
    { 0x93c8af1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e08af20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28af27, 34 },
    { 0x438af28, 61 },
    { 0xbf48af27, 91 },
    { 0x6ea8af28, 17 },
    { 0x7a38af2b, 84 },
    { 0, 0 },
    { 0x1f88af2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc78af31, 93 },
    { 0, 0 },
    { 0xe598af33, 61 },
    { 0, 0 },
    { 0xb5b8af35, 23 },
    { 0x9738af35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2558af3a, 34 },
    { 0xd38af3b, 96 },
    { 0x2078af3c, 100 },
    { 0xe6c8af3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7838af41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e8af45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f28af48, 33 },
    { 0, 0 },
    { 0x7448af4a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd278af55, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8af5a, 91 },
    { 0x2a48af5b, 87 },
    { 0, 0 },
    { 0x4b68af5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c8af67, 55 },
    { 0, 0 },
    { 0xe2f8af69, 55 },
    { 0, 0 },
    { 0xd038af6b, 95 },
    { 0x7118af6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd8af6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa8af74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8af77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9668af7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf808af81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c8af84, 88 },
    { 0, 0 },
    { 0x3208af86, 77 },
    { 0x648af86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5748af95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc78af99, 55 },
    { 0, 0 },
    { 0x4898af9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a8af9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a68afab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf98afb3, 61 },
    { 0x7a78afb4, 23 },
    { 0, 0 },
    { 0x258afb6, 78 },
    { 0, 0 },
    { 0x30d8afb8, 52 },
    { 0xc7b8afb9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f8afbe, 61 },
    { 0, 0 },
    { 0xd068afc0, 61 },
    { 0xa358afc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8afc8, 90 },
    { 0x8408afc9, 78 },
    { 0xd928afc9, 100 },
    { 0, 0 },
    { 0x6d38afcc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d8afcf, 95 },
    { 0xe398afcf, 90 },
    { 0x1b28afd0, 23 },
    { 0xe748afd1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc398afd6, 76 },
    { 0xb7f8afd6, 34 },
    { 0xb7e8afd8, 61 },
    { 0x5d78afd9, 36 },
    { 0xe9e8afd7, 76 },
    { 0x6d28afdb, 33 },
    { 0xfb68afdb, 69 },
    { 0xcfb8afdb, 95 },
    { 0x6258afde, 95 },
    { 0xf598afd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408afe7, 92 },
    { 0xe1c8afe7, 23 },
    { 0xa7a8afe8, 77 },
    { 0x75b8afea, 78 },
    { 0x75f8afea, 90 },
    { 0x9538afe7, 34 },
    { 0xf948afed, 78 },
    { 0xdf28afe9, 98 },
    { 0xd2c8afea, 61 },
    { 0x2b8afea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e8aff6, 76 },
    { 0, 0 },
    { 0xada8aff8, 34 },
    { 0x5fb8aff9, 84 },
    { 0x83c8affa, 43 },
    { 0x6a78affa, 40 },
    { 0x39d8affa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe448b000, 90 },
    { 0xded8b001, 85 },
    { 0xaca8b001, 95 },
    { 0, 0 },
    { 0x2148b004, 90 },
    { 0x2948b004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa208b008, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f8b00b, 61 },
    { 0x2568b00b, 56 },
    { 0xec28b00c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418b011, 68 },
    { 0x86f8b012, 33 },
    { 0, 0 },
    { 0xc9d8b014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5278b01a, 33 },
    { 0, 0 },
    { 0xe2c8b01c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa318b021, 86 },
    { 0, 0 },
    { 0x2fc8b023, 55 },
    { 0xdc48b024, 100 },
    { 0x88f8b023, 90 },
    { 0x4a38b026, 78 },
    { 0x7ba8b026, 93 },
    { 0x9018b028, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b02f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d8b032, 95 },
    { 0, 0 },
    { 0x21d8b034, 81 },
    { 0xae88b034, 23 },
    { 0x50c8b036, 90 },
    { 0x7f18b037, 83 },
    { 0, 0 },
    { 0x85e8b039, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6138b03d, 23 },
    { 0, 0 },
    { 0x7308b03f, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838b042, 61 },
    { 0x8b043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268b046, 33 },
    { 0x7248b047, 66 },
    { 0, 0 },
    { 0xf9c8b049, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a8b04f, 95 },
    { 0x4128b050, 90 },
    { 0x4728b050, 33 },
    { 0, 0 },
    { 0x7b78b053, 90 },
    { 0, 0 },
    { 0x8aa8b055, 95 },
    { 0, 0 },
    { 0x4c8b057, 61 },
    { 0xe5e8b058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8b05d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a8b060, 90 },
    { 0, 0 },
    { 0x38b062, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c8b065, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d8b072, 95 },
    { 0xc818b073, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde8b077, 87 },
    { 0x8df8b077, 95 },
    { 0x968b079, 78 },
    { 0x52b8b07a, 33 },
    { 0xc198b079, 95 },
    { 0x3578b07a, 46 },
    { 0xd028b07d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc8b082, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x848b085, 61 },
    { 0xc888b086, 76 },
    { 0xee58b087, 98 },
    { 0x41b8b088, 43 },
    { 0, 0 },
    { 0x1ac8b08a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c8b08e, 61 },
    { 0, 0 },
    { 0xe658b090, 40 },
    { 0x8688b090, 61 },
    { 0x7f88b092, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d8b095, 61 },
    { 0xfaa8b096, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2678b099, 43 },
    { 0, 0 },
    { 0xa248b09b, 81 },
    { 0, 0 },
    { 0xb468b09d, 98 },
    { 0xf658b09e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea8b0a3, 43 },
    { 0xebd8b0a4, 98 },
    { 0x5158b0a5, 90 },
    { 0xb008b0a3, 33 },
    { 0x2e8b0a7, 43 },
    { 0xb038b0a3, 63 },
    { 0xee08b0a4, 33 },
    { 0x9d68b0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd8b0b1, 83 },
    { 0x24c8b0b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e8b0be, 33 },
    { 0x1778b0bf, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c8b0c5, 66 },
    { 0x45b8b0c6, 33 },
    { 0x13b8b0c7, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b0d1, 88 },
    { 0xde58b0d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4908b0d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf8b0dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df8b0e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd118b0e6, 95 },
    { 0xc048b0e7, 55 },
    { 0xc5a8b0e8, 43 },
    { 0x18b8b0e9, 91 },
    { 0x6c98b0e6, 33 },
    { 0, 0 },
    { 0xeb78b0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868b0f2, 76 },
    { 0xc4d8b0f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2908b0fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1298b101, 95 },
    { 0x7208b101, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c88b105, 61 },
    { 0, 0 },
    { 0xc3d8b107, 98 },
    { 0x1c98b108, 33 },
    { 0x99f8b107, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c8b10e, 23 },
    { 0x1238b10e, 23 },
    { 0x1688b110, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8b115, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb8b11c, 95 },
    { 0x6098b11d, 93 },
    { 0, 0 },
    { 0x8c8b11f, 90 },
    { 0x2e78b120, 88 },
    { 0x4e88b121, 55 },
    { 0xdf48b11f, 23 },
    { 0x9168b123, 98 },
    { 0x2918b124, 61 },
    { 0x2af8b11f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5518b128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e58b12d, 34 },
    { 0xec08b12e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf8b132, 33 },
    { 0, 0 },
    { 0xf008b134, 50 },
    { 0x48b135, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba8b13a, 81 },
    { 0x7e58b13a, 68 },
    { 0x7f8b13a, 34 },
    { 0x78e8b13d, 61 },
    { 0xb6f8b13e, 98 },
    { 0xf0f8b13f, 78 },
    { 0xc878b13b, 100 },
    { 0x4408b13f, 98 },
    { 0x72a8b13e, 93 },
    { 0, 0 },
    { 0xa1e8b144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe8b148, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1408b14e, 84 },
    { 0xcc98b14f, 98 },
    { 0, 0 },
    { 0xd748b151, 33 },
    { 0x8718b152, 23 },
    { 0xa948b153, 61 },
    { 0xc2f8b154, 61 },
    { 0x3878b155, 52 },
    { 0x1a88b153, 76 },
    { 0, 0 },
    { 0xf758b158, 55 },
    { 0x58d8b159, 61 },
    { 0x9b48b15a, 85 },
    { 0xf1d8b15a, 55 },
    { 0xeea8b159, 33 },
    { 0x16a8b15c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d8b162, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2898b165, 78 },
    { 0, 0 },
    { 0x72e8b167, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408b16c, 98 },
    { 0x99c8b16c, 90 },
    { 0xed58b16e, 17 },
    { 0, 0 },
    { 0x53c8b170, 61 },
    { 0x2528b170, 23 },
    { 0x61e8b172, 43 },
    { 0x9268b171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288b176, 55 },
    { 0x33d8b176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba8b183, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a08b189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a8b18d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b8b194, 33 },
    { 0x2598b195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa208b198, 96 },
    { 0xb888b199, 68 },
    { 0xfac8b199, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa808b19d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef8b1a2, 34 },
    { 0, 0 },
    { 0xfc78b1a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c88b1aa, 33 },
    { 0x5848b1ab, 33 },
    { 0x6eb8b1ac, 34 },
    { 0x5018b1aa, 98 },
    { 0, 0 },
    { 0xf018b1af, 75 },
    { 0xc878b1af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d8b1b3, 91 },
    { 0x47e8b1b3, 92 },
    { 0, 0 },
    { 0x98c8b1b6, 76 },
    { 0x62f8b1b7, 95 },
    { 0, 0 },
    { 0x78a8b1b9, 90 },
    { 0x9c68b1ba, 78 },
    { 0, 0 },
    { 0x3578b1bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868b1c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc8b1c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f8b1d0, 84 },
    { 0xe888b1d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a78b1d6, 23 },
    { 0, 0 },
    { 0xf478b1d8, 97 },
    { 0, 0 },
    { 0x20f8b1da, 55 },
    { 0, 0 },
    { 0xc498b1dc, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4588b1e1, 78 },
    { 0xfec8b1e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x998b1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d98b1ee, 33 },
    { 0x4888b1ef, 23 },
    { 0x5688b1f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd838b1f7, 90 },
    { 0xbd48b1f8, 33 },
    { 0, 0 },
    { 0x7958b1fa, 88 },
    { 0x6c68b1fb, 99 },
    { 0xca38b1fa, 99 },
    { 0x65a8b1fd, 33 },
    { 0x3a98b1fd, 33 },
    { 0x5028b1fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc518b202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd8b208, 33 },
    { 0x7b58b209, 98 },
    { 0x77b8b20a, 84 },
    { 0xda18b20a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628b212, 90 },
    { 0xee18b213, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9438b216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c68b21c, 23 },
    { 0x3c98b21c, 40 },
    { 0x3e68b21d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde8b227, 76 },
    { 0xc988b227, 33 },
    { 0, 0 },
    { 0xe7f8b22a, 55 },
    { 0, 0 },
    { 0xf68b22c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf118b22f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a8b234, 90 },
    { 0, 0 },
    { 0x20b8b236, 87 },
    { 0x3ce8b237, 80 },
    { 0xf688b238, 77 },
    { 0x8a98b238, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d8b23c, 81 },
    { 0, 0 },
    { 0xc98b23e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e8b241, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd88b245, 98 },
    { 0x1af8b246, 34 },
    { 0xf318b247, 43 },
    { 0xebb8b248, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d88b24c, 23 },
    { 0x7ac8b24d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5958b251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8b254, 43 },
    { 0x9388b255, 33 },
    { 0x4238b254, 90 },
    { 0x5ad8b256, 100 },
    { 0x8198b254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe978b25c, 90 },
    { 0, 0 },
    { 0x1b58b25e, 61 },
    { 0xcc78b25f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec8b268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f8b274, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac8b279, 61 },
    { 0xc8b8b279, 46 },
    { 0, 0 },
    { 0x7958b27c, 90 },
    { 0x6db8b27d, 46 },
    { 0x9398b27e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98b281, 61 },
    { 0x9208b281, 75 },
    { 0xa168b281, 23 },
    { 0xb168b281, 93 },
    { 0x7268b285, 56 },
    { 0, 0 },
    { 0xafd8b287, 23 },
    { 0x93e8b288, 33 },
    { 0xe748b289, 61 },
    { 0x7548b288, 61 },
    { 0xd728b287, 93 },
    { 0xd9f8b28c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8448b290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d8b294, 33 },
    { 0, 0 },
    { 0xb668b296, 98 },
    { 0x26d8b297, 90 },
    { 0x70c8b296, 95 },
    { 0, 0 },
    { 0xc238b29a, 33 },
    { 0x7368b29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe668b29f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe328b2a3, 76 },
    { 0x6628b2a3, 99 },
    { 0xd38b2a5, 97 },
    { 0, 0 },
    { 0xe638b2a7, 23 },
    { 0, 0 },
    { 0x19c8b2a9, 52 },
    { 0x1ea8b2a9, 34 },
    { 0xaf38b2ab, 87 },
    { 0xd38b2ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d38b2af, 33 },
    { 0xff18b2b0, 76 },
    { 0x4838b2b1, 90 },
    { 0x19e8b2b1, 98 },
    { 0x4c98b2b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18b2c0, 98 },
    { 0, 0 },
    { 0x7328b2c2, 90 },
    { 0, 0 },
    { 0xdcb8b2c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac8b2c7, 90 },
    { 0x8278b2c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568b2cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f8b2d0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb328b2d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5988b2d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc708b2dc, 90 },
    { 0, 0 },
    { 0x6448b2de, 69 },
    { 0, 0 },
    { 0xab8b2e0, 65 },
    { 0xa838b2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8b2e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c8b2e9, 55 },
    { 0x5d8b2ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef8b2ed, 76 },
    { 0, 0 },
    { 0xb1f8b2ef, 90 },
    { 0, 0 },
    { 0x5db8b2f1, 95 },
    { 0x8d48b2f1, 78 },
    { 0x8a08b2f1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec8b2f6, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88b2fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a8b2ff, 34 },
    { 0xe108b300, 90 },
    { 0x9508b300, 34 },
    { 0xe358b300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe178b307, 52 },
    { 0xa98b308, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7148b30d, 33 },
    { 0xa3c8b30d, 95 },
    { 0x8c58b30d, 90 },
    { 0, 0 },
    { 0x5858b311, 90 },
    { 0xe4d8b312, 61 },
    { 0, 0 },
    { 0x44e8b314, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e08b317, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf648b31a, 46 },
    { 0x4468b31b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3128b31e, 52 },
    { 0x8e58b31f, 87 },
    { 0x248b31f, 88 },
    { 0x4988b320, 90 },
    { 0x58b8b322, 52 },
    { 0xe2b8b31e, 55 },
    { 0, 0 },
    { 0xe838b325, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9978b32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee8b32d, 90 },
    { 0, 0 },
    { 0x68a8b32f, 17 },
    { 0, 0 },
    { 0xb0a8b331, 90 },
    { 0xb608b331, 95 },
    { 0, 0 },
    { 0xe348b334, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9398b33b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b33e, 90 },
    { 0x188b33f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f8b348, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8838b34d, 33 },
    { 0xa8b8b34e, 52 },
    { 0xb3f8b34d, 100 },
    { 0xe008b350, 90 },
    { 0x78b8b351, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c8b357, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8828b35c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8b361, 95 },
    { 0xf678b362, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c28b36c, 90 },
    { 0xe168b36d, 95 },
    { 0, 0 },
    { 0x5a38b36f, 34 },
    { 0xae98b370, 55 },
    { 0x778b370, 34 },
    { 0xe0d8b372, 95 },
    { 0, 0 },
    { 0x83c8b374, 98 },
    { 0x2fe8b375, 77 },
    { 0xbc48b375, 33 },
    { 0, 0 },
    { 0xac18b378, 98 },
    { 0xcd78b379, 43 },
    { 0x2d08b378, 61 },
    { 0, 0 },
    { 0x478b37c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5508b37f, 33 },
    { 0xa038b380, 90 },
    { 0x6d38b380, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4088b386, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de8b38a, 55 },
    { 0x44d8b38a, 33 },
    { 0x52a8b38c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5058b391, 95 },
    { 0x91d8b391, 23 },
    { 0xd778b393, 61 },
    { 0, 0 },
    { 0x178b395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35d8b398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c8b39b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c8b39f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x498b3a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf948b3a9, 98 },
    { 0x4e88b3a9, 95 },
    { 0xe48b3ab, 95 },
    { 0, 0 },
    { 0x1f88b3ad, 61 },
    { 0x9958b3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3628b3b1, 23 },
    { 0x7d78b3b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba8b3b6, 90 },
    { 0x2da8b3b7, 33 },
    { 0x6198b3b7, 23 },
    { 0, 0 },
    { 0x4878b3ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa608b3be, 93 },
    { 0x9e08b3bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e8b3c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc608b3c5, 85 },
    { 0, 0 },
    { 0xdd88b3c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3608b3cc, 61 },
    { 0xf828b3cd, 95 },
    { 0, 0 },
    { 0xd8e8b3cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf688b3d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb398b3d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b8b3da, 34 },
    { 0xff18b3db, 52 },
    { 0xd5c8b3dc, 90 },
    { 0xde08b3dc, 98 },
    { 0x64f8b3de, 76 },
    { 0xb658b3dc, 61 },
    { 0x95f8b3e0, 98 },
    { 0, 0 },
    { 0x74d8b3e2, 90 },
    { 0xc358b3e3, 34 },
    { 0xdf28b3e4, 90 },
    { 0x8ee8b3e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3058b3e8, 76 },
    { 0x6be8b3e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8b3f3, 99 },
    { 0xf0a8b3f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38b3f8, 87 },
    { 0xfb98b3f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d8b405, 43 },
    { 0, 0 },
    { 0xa898b407, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f68b40f, 98 },
    { 0xa598b40f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7818b413, 81 },
    { 0x4508b414, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf398b41d, 33 },
    { 0, 0 },
    { 0xb298b41f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b8b422, 84 },
    { 0x7a68b422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac8b428, 78 },
    { 0, 0 },
    { 0xcf98b42a, 52 },
    { 0xb7b8b42b, 77 },
    { 0, 0 },
    { 0x3d38b42d, 95 },
    { 0x8f38b42e, 33 },
    { 0x9918b42f, 61 },
    { 0, 0 },
    { 0xf9c8b431, 98 },
    { 0, 0 },
    { 0x2318b433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b8b439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c78b43f, 87 },
    { 0x5888b440, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6528b443, 34 },
    { 0xe1a8b444, 90 },
    { 0xdc28b443, 90 },
    { 0xd618b444, 90 },
    { 0x1c28b447, 85 },
    { 0x5bc8b443, 61 },
    { 0xede8b449, 90 },
    { 0x3478b44a, 65 },
    { 0x2e68b449, 55 },
    { 0xd2d8b44c, 40 },
    { 0, 0 },
    { 0x95b8b44e, 33 },
    { 0, 0 },
    { 0x4378b450, 90 },
    { 0x2c18b451, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a68b454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad8b45a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918b45f, 95 },
    { 0x9c18b45f, 90 },
    { 0x2c08b461, 55 },
    { 0xca38b45f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe478b465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168b46a, 43 },
    { 0, 0 },
    { 0xc2d8b46c, 61 },
    { 0xd578b46d, 61 },
    { 0x32d8b46c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7548b477, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d8b47a, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb8b480, 61 },
    { 0x7568b481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c8b489, 52 },
    { 0, 0 },
    { 0x5fe8b48b, 95 },
    { 0xc3c8b48c, 75 },
    { 0, 0 },
    { 0x4e58b48e, 23 },
    { 0, 0 },
    { 0x9d8b490, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a18b495, 46 },
    { 0, 0 },
    { 0xc068b497, 77 },
    { 0x8db8b498, 95 },
    { 0x86f8b499, 33 },
    { 0x8008b49a, 98 },
    { 0x2e38b49b, 90 },
    { 0x6898b49a, 23 },
    { 0x8408b49d, 76 },
    { 0xf5e8b49d, 90 },
    { 0x8b78b49f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e88b4a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca68b4a8, 84 },
    { 0x9788b4a9, 52 },
    { 0xf8c8b4aa, 48 },
    { 0, 0 },
    { 0x7358b4ac, 34 },
    { 0xca68b4ac, 23 },
    { 0, 0 },
    { 0x8038b4af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e8b4b4, 55 },
    { 0x24e8b4b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb818b4bb, 43 },
    { 0x7568b4bc, 67 },
    { 0, 0 },
    { 0x7648b4be, 61 },
    { 0x3078b4be, 75 },
    { 0x718b4bf, 36 },
    { 0x45c8b4bf, 33 },
    { 0, 0 },
    { 0x37a8b4c3, 61 },
    { 0x9b48b4c4, 89 },
    { 0x86a8b4c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee18b4c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf8b4ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3948b4d1, 77 },
    { 0xd358b4d2, 93 },
    { 0x6318b4d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2798b4d7, 95 },
    { 0x6ca8b4d8, 76 },
    { 0x9d88b4d9, 93 },
    { 0x27a8b4da, 43 },
    { 0, 0 },
    { 0x8d58b4dc, 84 },
    { 0x7188b4dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c8b4e0, 61 },
    { 0x84e8b4e1, 87 },
    { 0xf458b4e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c8b4e7, 100 },
    { 0, 0 },
    { 0x20c8b4e9, 91 },
    { 0, 0 },
    { 0xbaf8b4eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b4ee, 66 },
    { 0x7d78b4ee, 61 },
    { 0x7f8b4f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe8b4f3, 34 },
    { 0, 0 },
    { 0x6518b4f5, 76 },
    { 0x3358b4f6, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x4728b4f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e8b4fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d8b506, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4298b50c, 87 },
    { 0x8138b50c, 61 },
    { 0, 0 },
    { 0x33d8b50f, 78 },
    { 0, 0 },
    { 0x4d68b511, 33 },
    { 0x4a68b512, 34 },
    { 0xb648b513, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e8b51a, 81 },
    { 0, 0 },
    { 0x26a8b51c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd478b523, 43 },
    { 0xe528b524, 90 },
    { 0x4fc8b524, 76 },
    { 0x1d48b523, 33 },
    { 0x1ea8b524, 80 },
    { 0, 0 },
    { 0xcd08b529, 23 },
    { 0xa698b529, 33 },
    { 0xdf28b52b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab28b52f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3028b535, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef8b53a, 23 },
    { 0x8cc8b53b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8b53e, 90 },
    { 0x28c8b53f, 34 },
    { 0xd718b540, 23 },
    { 0xbf38b540, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c8b546, 84 },
    { 0, 0 },
    { 0xc778b548, 95 },
    { 0xab48b549, 33 },
    { 0x3b58b54a, 90 },
    { 0x83d8b549, 84 },
    { 0xaab8b54c, 61 },
    { 0xae38b54d, 61 },
    { 0x598b54b, 34 },
    { 0x6c28b54f, 65 },
    { 0xb508b54e, 68 },
    { 0x5798b551, 61 },
    { 0x42e8b54d, 99 },
    { 0xdb58b54f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad28b559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a8b560, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d8b564, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b98b568, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3348b56e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528b571, 33 },
    { 0xcf58b571, 34 },
    { 0x1aa8b573, 90 },
    { 0, 0 },
    { 0xdcc8b575, 55 },
    { 0xa318b576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc8b57e, 90 },
    { 0, 0 },
    { 0xbfd8b580, 61 },
    { 0, 0 },
    { 0xc298b582, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a98b586, 78 },
    { 0xe648b587, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1338b58a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2818b599, 76 },
    { 0, 0 },
    { 0xf08b59b, 61 },
    { 0xbcc8b59b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e8b5a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e8b5aa, 77 },
    { 0, 0 },
    { 0x90b8b5ac, 99 },
    { 0xfbd8b5ad, 98 },
    { 0x11d8b5ac, 99 },
    { 0, 0 },
    { 0xfa08b5b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf28b5b4, 52 },
    { 0x5008b5b4, 90 },
    { 0xf818b5b6, 67 },
    { 0xef78b5b7, 61 },
    { 0xf6a8b5b8, 66 },
    { 0x73d8b5b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af8b5be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd68b5c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df8b5c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f98b5cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7738b5d0, 90 },
    { 0x4e88b5d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e48b5d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd8b5d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408b5dc, 99 },
    { 0x5f28b5dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b5e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc538b5e7, 90 },
    { 0, 0 },
    { 0x7018b5e9, 98 },
    { 0, 0 },
    { 0x5648b5eb, 98 },
    { 0xa978b5ec, 95 },
    { 0xfa78b5ed, 23 },
    { 0xdb58b5ee, 90 },
    { 0, 0 },
    { 0x9bf8b5f0, 23 },
    { 0xdba8b5f0, 90 },
    { 0xe038b5f0, 91 },
    { 0, 0 },
    { 0xe388b5f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5598b5f7, 95 },
    { 0xac08b5f8, 23 },
    { 0, 0 },
    { 0x1418b5fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58b5fe, 23 },
    { 0x798b5ff, 90 },
    { 0xa898b600, 61 },
    { 0xedf8b600, 93 },
    { 0xbcc8b602, 46 },
    { 0x5748b602, 98 },
    { 0xc2d8b604, 90 },
    { 0x9818b604, 93 },
    { 0x6fb8b605, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8b609, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5768b60d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd38b611, 90 },
    { 0xb898b612, 90 },
    { 0xf78b613, 33 },
    { 0, 0 },
    { 0xc1f8b615, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b61e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe8b621, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad8b624, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f8b62c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7498b630, 76 },
    { 0x1e88b631, 43 },
    { 0xfdf8b632, 46 },
    { 0xeda8b630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad78b637, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d28b644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5298b64b, 65 },
    { 0x7d18b64c, 90 },
    { 0, 0 },
    { 0x5058b64e, 23 },
    { 0, 0 },
    { 0x4a98b650, 34 },
    { 0, 0 },
    { 0x2c58b652, 55 },
    { 0, 0 },
    { 0x67c8b654, 98 },
    { 0xe908b654, 23 },
    { 0, 0 },
    { 0x4488b657, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3878b65c, 33 },
    { 0x2058b65d, 78 },
    { 0, 0 },
    { 0xca68b65f, 33 },
    { 0x87f8b660, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a8b664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8248b668, 93 },
    { 0x6ac8b668, 36 },
    { 0x5718b66a, 34 },
    { 0x3958b66b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68b670, 55 },
    { 0x518b671, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a58b676, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568b67a, 33 },
    { 0x118b67b, 95 },
    { 0xa1a8b67c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c78b680, 90 },
    { 0xf038b680, 33 },
    { 0xd588b681, 95 },
    { 0x81f8b683, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4308b68a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3928b694, 99 },
    { 0, 0 },
    { 0xc118b696, 76 },
    { 0, 0 },
    { 0x47b8b698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x158b69b, 98 },
    { 0x6948b69c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebe8b6a0, 90 },
    { 0x1958b6a1, 61 },
    { 0x6408b6a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe208b6a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe8b6ae, 61 },
    { 0x9398b6af, 44 },
    { 0x20c8b6b0, 34 },
    { 0x2898b6ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec8b6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd598b6ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a8b6bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b68b6c2, 90 },
    { 0, 0 },
    { 0xff58b6c4, 95 },
    { 0, 0 },
    { 0xc3c8b6c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e8b6ca, 98 },
    { 0x15a8b6ca, 33 },
    { 0xad38b6cc, 35 },
    { 0xc2b8b6cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6098b6d2, 61 },
    { 0xbc58b6d3, 43 },
    { 0x7e48b6d4, 33 },
    { 0, 0 },
    { 0x5188b6d6, 66 },
    { 0xfbc8b6d6, 98 },
    { 0x7be8b6d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8478b6de, 85 },
    { 0xb028b6df, 33 },
    { 0x3408b6e0, 76 },
    { 0xe008b6df, 33 },
    { 0x2e58b6e2, 33 },
    { 0x93e8b6e3, 40 },
    { 0x13e8b6e4, 55 },
    { 0xb5c8b6e0, 55 },
    { 0x8dc8b6e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8b6ea, 93 },
    { 0x32c8b6ea, 95 },
    { 0x8d98b6ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a8b6ef, 100 },
    { 0, 0 },
    { 0x2a8b6f1, 43 },
    { 0x6c38b6f2, 98 },
    { 0, 0 },
    { 0xbdd8b6f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c8b6fd, 33 },
    { 0, 0 },
    { 0x50c8b6ff, 96 },
    { 0xaa88b700, 90 },
    { 0x3278b701, 99 },
    { 0xa858b700, 43 },
    { 0x31f8b703, 38 },
    { 0, 0 },
    { 0xb098b705, 90 },
    { 0xd028b705, 75 },
    { 0x8968b707, 98 },
    { 0xf768b707, 43 },
    { 0xb188b707, 100 },
    { 0x5178b707, 95 },
    { 0x7278b70b, 98 },
    { 0xccf8b70c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb798b716, 43 },
    { 0, 0 },
    { 0xa5c8b718, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b8b71b, 100 },
    { 0xcdb8b71c, 76 },
    { 0x9568b71d, 98 },
    { 0x2078b71e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d78b723, 98 },
    { 0xdf8b723, 55 },
    { 0, 0 },
    { 0x17c8b726, 98 },
    { 0, 0 },
    { 0xd3a8b728, 61 },
    { 0x1048b729, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a8b72d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa8b737, 84 },
    { 0x6248b737, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d88b73e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8b741, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb8b744, 61 },
    { 0, 0 },
    { 0x5678b746, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5978b74b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8b750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb088b753, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c8b75c, 61 },
    { 0x4378b75d, 23 },
    { 0xf458b75e, 98 },
    { 0xbef8b75f, 61 },
    { 0, 0 },
    { 0x3b8b761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58b768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8338b76d, 90 },
    { 0x8f28b76d, 75 },
    { 0xe2d8b76e, 85 },
    { 0x2eb8b770, 34 },
    { 0, 0 },
    { 0x39c8b772, 33 },
    { 0xba68b773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea8b776, 95 },
    { 0xbe98b777, 66 },
    { 0, 0 },
    { 0xd068b779, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7188b77f, 76 },
    { 0, 0 },
    { 0xbaf8b781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d8b785, 52 },
    { 0x1718b786, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b8b789, 67 },
    { 0, 0 },
    { 0xece8b78b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb8b791, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb8b795, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc8b798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5b8b79d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d8b7a5, 61 },
    { 0xbfa8b7a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea18b7a9, 61 },
    { 0x4cb8b7aa, 84 },
    { 0x44f8b7a9, 95 },
    { 0x4408b7ac, 23 },
    { 0, 0 },
    { 0x5f18b7ae, 95 },
    { 0x27b8b7af, 55 },
    { 0x8508b7b0, 99 },
    { 0x5138b7af, 85 },
    { 0x9808b7b2, 23 },
    { 0x6ab8b7b2, 23 },
    { 0xf068b7b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08b7b7, 52 },
    { 0xb9f8b7b7, 23 },
    { 0, 0 },
    { 0xb4f8b7ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd8b7be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa8b7c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb738b7c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba88b7cd, 55 },
    { 0x4cc8b7ce, 61 },
    { 0x3608b7cf, 66 },
    { 0xe918b7d0, 90 },
    { 0, 0 },
    { 0x22b8b7d2, 69 },
    { 0x2e98b7d3, 41 },
    { 0x7e28b7d4, 95 },
    { 0xe388b7d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68b7da, 61 },
    { 0, 0 },
    { 0x418b7dc, 90 },
    { 0xe978b7dc, 99 },
    { 0x2658b7de, 98 },
    { 0xfb18b7de, 55 },
    { 0xc1b8b7de, 90 },
    { 0, 0 },
    { 0x7598b7e2, 34 },
    { 0, 0 },
    { 0xacd8b7e4, 55 },
    { 0xaa28b7e5, 95 },
    { 0x4158b7e5, 46 },
    { 0x4218b7e7, 33 },
    { 0x8848b7e8, 61 },
    { 0x86a8b7e9, 95 },
    { 0x7708b7ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b8b7ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258b7f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c8b7f5, 90 },
    { 0x5e48b7f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec8b7fa, 55 },
    { 0x3118b7fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb48b802, 23 },
    { 0xc978b803, 95 },
    { 0x91c8b804, 99 },
    { 0, 0 },
    { 0x3278b806, 35 },
    { 0x5058b807, 33 },
    { 0x8908b808, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a8b80d, 93 },
    { 0x6928b80d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c08b814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38b819, 61 },
    { 0x23e8b819, 34 },
    { 0, 0 },
    { 0x4b08b81c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8b81f, 90 },
    { 0, 0 },
    { 0x68b821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e38b827, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1218b82c, 98 },
    { 0x96b8b82d, 33 },
    { 0x9558b82d, 88 },
    { 0x5b98b82f, 90 },
    { 0xfb38b82e, 23 },
    { 0xdae8b82c, 91 },
    { 0, 0 },
    { 0x2a98b833, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb8b836, 17 },
    { 0x7f68b836, 55 },
    { 0x3068b837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6638b842, 91 },
    { 0xe2e8b843, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398b84b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8b850, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a18b854, 67 },
    { 0x9738b855, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e88b859, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3578b85d, 48 },
    { 0, 0 },
    { 0xf768b85f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a8b862, 61 },
    { 0xdd28b863, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd558b866, 90 },
    { 0x78c8b867, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c8b86a, 33 },
    { 0xd568b86a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8b86f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c8b872, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c8b875, 90 },
    { 0x42d8b876, 95 },
    { 0xbde8b875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d38b87d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae8b882, 91 },
    { 0, 0 },
    { 0x8678b884, 98 },
    { 0, 0 },
    { 0xead8b886, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9518b88b, 95 },
    { 0, 0 },
    { 0xe608b88d, 68 },
    { 0xf4c8b88e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab8b893, 76 },
    { 0x3ea8b894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de8b897, 90 },
    { 0xdb38b898, 23 },
    { 0x4788b898, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f8b89f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8b8a2, 23 },
    { 0xdfe8b8a3, 32 },
    { 0xc948b8a3, 23 },
    { 0x9c8b8a4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb338b8a8, 90 },
    { 0xba38b8a9, 98 },
    { 0x2e38b8a8, 76 },
    { 0xdfb8b8ab, 100 },
    { 0xc78b8ac, 34 },
    { 0, 0 },
    { 0x2c98b8ae, 98 },
    { 0, 0 },
    { 0x3fe8b8b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8b8b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5788b8c0, 49 },
    { 0, 0 },
    { 0x8668b8c2, 81 },
    { 0x9b28b8c3, 33 },
    { 0x3d78b8c4, 90 },
    { 0xa988b8c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538b8cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc8b8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8b8d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f8b8db, 23 },
    { 0, 0 },
    { 0xda28b8dd, 23 },
    { 0x89b8b8de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb808b8e8, 88 },
    { 0x1168b8e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28b8ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18b8f1, 65 },
    { 0xca88b8f2, 66 },
    { 0xdfa8b8f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e8b8f6, 98 },
    { 0xfb68b8f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a8b8fa, 61 },
    { 0xb078b8fb, 90 },
    { 0xdc98b8fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa8b900, 76 },
    { 0xc648b901, 34 },
    { 0x5128b901, 46 },
    { 0xcfc8b903, 93 },
    { 0xbfb8b900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe688b907, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2498b90d, 61 },
    { 0xa5f8b90d, 77 },
    { 0x8d8b90d, 23 },
    { 0x9158b90e, 61 },
    { 0xb7a8b911, 33 },
    { 0, 0 },
    { 0x3588b913, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538b918, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8938b91b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc8b91f, 98 },
    { 0, 0 },
    { 0xea98b921, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf8b925, 78 },
    { 0, 0 },
    { 0xe068b927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec8b939, 90 },
    { 0, 0 },
    { 0xa858b93b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec08b943, 34 },
    { 0xc118b943, 99 },
    { 0xc2a8b945, 23 },
    { 0x4e18b945, 23 },
    { 0, 0 },
    { 0xf028b948, 61 },
    { 0, 0 },
    { 0x5cd8b94a, 61 },
    { 0xc038b94a, 90 },
    { 0x19a8b94c, 90 },
    { 0x3688b94c, 90 },
    { 0x23f8b94c, 98 },
    { 0, 0 },
    { 0x33e8b950, 23 },
    { 0, 0 },
    { 0x5248b952, 55 },
    { 0xa978b952, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c8b958, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2488b95f, 93 },
    { 0, 0 },
    { 0x7228b961, 68 },
    { 0xc778b961, 23 },
    { 0, 0 },
    { 0x43c8b964, 77 },
    { 0, 0 },
    { 0xb5b8b966, 23 },
    { 0x84f8b966, 88 },
    { 0xca78b968, 78 },
    { 0x57b8b966, 90 },
    { 0xd1f8b967, 91 },
    { 0xe8b96b, 76 },
    { 0x258b96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e8b975, 95 },
    { 0x49b8b975, 99 },
    { 0x9778b977, 61 },
    { 0, 0 },
    { 0xc768b979, 61 },
    { 0, 0 },
    { 0xb468b97b, 78 },
    { 0, 0 },
    { 0xb2b8b97d, 68 },
    { 0, 0 },
    { 0x878b97f, 61 },
    { 0x1808b980, 61 },
    { 0x9f18b981, 61 },
    { 0xebe8b981, 76 },
    { 0xc5d8b981, 78 },
    { 0xa98b983, 43 },
    { 0x318b981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a8b98a, 46 },
    { 0x6118b98b, 98 },
    { 0x74c8b98c, 61 },
    { 0xef88b98c, 95 },
    { 0xcb48b98c, 33 },
    { 0xeab8b98f, 23 },
    { 0xb5f8b98f, 40 },
    { 0xffd8b991, 33 },
    { 0xd9e8b98d, 34 },
    { 0, 0 },
    { 0x1038b994, 95 },
    { 0xf7f8b994, 98 },
    { 0, 0 },
    { 0x91f8b997, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd08b99f, 98 },
    { 0, 0 },
    { 0x69f8b9a1, 89 },
    { 0xe288b9a2, 33 },
    { 0xa628b9a3, 68 },
    { 0x7ca8b9a3, 33 },
    { 0x2798b9a5, 76 },
    { 0x1c88b9a6, 90 },
    { 0x6b78b9a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e8b9ab, 33 },
    { 0, 0 },
    { 0x8778b9ad, 99 },
    { 0xe808b9ae, 33 },
    { 0x51c8b9af, 34 },
    { 0x8ea8b9b0, 61 },
    { 0x4a48b9b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8b9b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc8b9bd, 90 },
    { 0x7e8b9be, 78 },
    { 0x3908b9be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08b9c4, 33 },
    { 0xcf88b9c5, 90 },
    { 0xcd8b9c6, 33 },
    { 0xacd8b9c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b8b9cb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6608b9cf, 66 },
    { 0, 0 },
    { 0xc438b9d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238b9d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8b9db, 46 },
    { 0x7f28b9db, 95 },
    { 0xd2f8b9dd, 88 },
    { 0x4048b9de, 43 },
    { 0, 0 },
    { 0x6bc8b9e0, 33 },
    { 0x9ca8b9e0, 61 },
    { 0x6908b9e1, 98 },
    { 0x2d98b9e3, 90 },
    { 0x6f48b9e0, 55 },
    { 0x2048b9e5, 43 },
    { 0xfd28b9e6, 84 },
    { 0, 0 },
    { 0x50e8b9e8, 90 },
    { 0, 0 },
    { 0x13e8b9ea, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf8b9ee, 23 },
    { 0x2588b9ef, 23 },
    { 0xbf78b9ee, 75 },
    { 0x6b88b9f1, 100 },
    { 0xdff8b9ef, 61 },
    { 0x7308b9f3, 95 },
    { 0xb878b9f0, 90 },
    { 0, 0 },
    { 0x26c8b9f6, 55 },
    { 0, 0 },
    { 0xd7e8b9f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6588b9fc, 91 },
    { 0x8c68b9fc, 23 },
    { 0xac18b9fe, 90 },
    { 0xa138b9fe, 98 },
    { 0xeb78b9ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf058ba08, 90 },
    { 0xc2d8ba08, 98 },
    { 0, 0 },
    { 0x8db8ba0b, 76 },
    { 0xd108ba0b, 90 },
    { 0x448ba0b, 23 },
    { 0xa178ba0b, 55 },
    { 0x24b8ba0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7678ba12, 33 },
    { 0x8c78ba12, 98 },
    { 0xb0e8ba14, 66 },
    { 0, 0 },
    { 0x4588ba16, 91 },
    { 0x1148ba17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f8ba1b, 61 },
    { 0, 0 },
    { 0x96d8ba1d, 90 },
    { 0, 0 },
    { 0x6498ba1f, 90 },
    { 0x3638ba20, 33 },
    { 0, 0 },
    { 0x8e8ba22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b8ba29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3778ba2c, 90 },
    { 0x1938ba2c, 34 },
    { 0, 0 },
    { 0xde08ba2f, 78 },
    { 0, 0 },
    { 0x3a58ba31, 84 },
    { 0, 0 },
    { 0x2d8ba33, 93 },
    { 0xad68ba34, 61 },
    { 0xfda8ba35, 98 },
    { 0xa048ba36, 90 },
    { 0xac78ba37, 34 },
    { 0x8768ba38, 65 },
    { 0, 0 },
    { 0x4958ba3a, 81 },
    { 0x4028ba3a, 33 },
    { 0x67d8ba3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e8ba40, 88 },
    { 0, 0 },
    { 0x9898ba42, 43 },
    { 0, 0 },
    { 0x5f8ba44, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08ba48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d8ba4c, 91 },
    { 0x8b8ba4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8ba50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d8ba53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e28ba59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58ba5d, 59 },
    { 0x398ba5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e8ba64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e8ba67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf58ba6f, 76 },
    { 0x7b98ba6f, 95 },
    { 0x5188ba70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef8ba78, 90 },
    { 0x1b38ba79, 99 },
    { 0x4f68ba7a, 33 },
    { 0x5018ba7b, 95 },
    { 0x8838ba7c, 43 },
    { 0x8468ba7a, 100 },
    { 0, 0 },
    { 0xffa8ba7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4738ba82, 95 },
    { 0xd48ba82, 95 },
    { 0x21a8ba84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b8ba88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558ba8c, 41 },
    { 0xdda8ba8c, 98 },
    { 0x8698ba8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6418ba91, 87 },
    { 0x8a78ba91, 90 },
    { 0x1048ba93, 98 },
    { 0, 0 },
    { 0xeec8ba95, 43 },
    { 0x33e8ba96, 61 },
    { 0, 0 },
    { 0x13f8ba98, 93 },
    { 0, 0 },
    { 0xd588ba9a, 61 },
    { 0, 0 },
    { 0xd168ba9c, 33 },
    { 0x54e8ba9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288baa0, 78 },
    { 0xe458baa1, 33 },
    { 0xecf8baa1, 61 },
    { 0x9bd8baa3, 43 },
    { 0x8888baa3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b8baa7, 85 },
    { 0x77d8baa8, 55 },
    { 0, 0 },
    { 0x2f58baaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf958baad, 33 },
    { 0xbf8baae, 17 },
    { 0, 0 },
    { 0x1ee8bab0, 33 },
    { 0x3238bab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5448bab7, 46 },
    { 0xc3c8bab8, 76 },
    { 0xea98bab7, 90 },
    { 0x838bab7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b8babe, 23 },
    { 0xe528babe, 58 },
    { 0, 0 },
    { 0xe248bac1, 90 },
    { 0, 0 },
    { 0x17e8bac3, 81 },
    { 0xb878bac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe378bac9, 23 },
    { 0, 0 },
    { 0x83b8bacb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c8bacf, 99 },
    { 0x2948bad0, 34 },
    { 0x34a8bad0, 33 },
    { 0x4ea8bad2, 75 },
    { 0xdfb8bad3, 33 },
    { 0, 0 },
    { 0x5188bad5, 90 },
    { 0xab58bad5, 90 },
    { 0, 0 },
    { 0xfbb8bad8, 99 },
    { 0xb978bad9, 43 },
    { 0, 0 },
    { 0x6498badb, 90 },
    { 0x2ef8badc, 83 },
    { 0x448badc, 31 },
    { 0x3fb8badd, 76 },
    { 0x67d8bade, 23 },
    { 0xa6b8bae0, 43 },
    { 0xed68bae0, 95 },
    { 0xc828bae2, 33 },
    { 0x8388bae3, 90 },
    { 0xd968bae2, 17 },
    { 0x6f88badc, 90 },
    { 0x4b8bae1, 61 },
    { 0x2538bae7, 76 },
    { 0x3798bae8, 93 },
    { 0xd3b8bae9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef88baee, 85 },
    { 0x7798baef, 88 },
    { 0x7598baef, 90 },
    { 0, 0 },
    { 0x1b58baf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd8baf5, 55 },
    { 0x2dd8baf6, 33 },
    { 0x16f8baf7, 97 },
    { 0xab98baf7, 43 },
    { 0xd968baf9, 90 },
    { 0xe618baf9, 33 },
    { 0xcc18baf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe8bb01, 23 },
    { 0, 0 },
    { 0x5668bb03, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb38bb0d, 55 },
    { 0xf818bb0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398bb1b, 95 },
    { 0x3cf8bb1c, 33 },
    { 0xc488bb1d, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7448bb22, 46 },
    { 0xfb68bb23, 99 },
    { 0x84f8bb24, 99 },
    { 0, 0 },
    { 0x5308bb26, 33 },
    { 0, 0 },
    { 0x8188bb28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf738bb2d, 33 },
    { 0xd248bb2e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d58bb31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd428bb35, 53 },
    { 0, 0 },
    { 0xef78bb37, 89 },
    { 0, 0 },
    { 0xfd88bb39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd8bb3d, 98 },
    { 0x8d98bb3d, 33 },
    { 0, 0 },
    { 0xeb48bb40, 23 },
    { 0x6968bb41, 43 },
    { 0xfd8bb42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf298bb45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad08bb55, 95 },
    { 0, 0 },
    { 0xc218bb57, 98 },
    { 0xd1e8bb58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa598bb5b, 40 },
    { 0x9808bb5b, 23 },
    { 0xfbf8bb5d, 33 },
    { 0xa2c8bb5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4338bb65, 23 },
    { 0, 0 },
    { 0x778bb67, 23 },
    { 0x34b8bb67, 55 },
    { 0xd5e8bb69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8bb6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3768bb70, 95 },
    { 0xe6e8bb71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c88bb83, 67 },
    { 0, 0 },
    { 0xbbc8bb85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd698bb88, 61 },
    { 0x988bb88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468bb8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a8bb93, 81 },
    { 0xdf18bb94, 23 },
    { 0xdb68bb94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8bb99, 53 },
    { 0, 0 },
    { 0x15e8bb9b, 33 },
    { 0, 0 },
    { 0x2dc8bb9d, 33 },
    { 0xace8bb9e, 98 },
    { 0x3ff8bb9e, 34 },
    { 0, 0 },
    { 0x4af8bba1, 61 },
    { 0x6748bba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8358bba8, 33 },
    { 0, 0 },
    { 0xf0b8bbaa, 100 },
    { 0xb928bbab, 95 },
    { 0xdf78bbab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8bbaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c98bbb3, 33 },
    { 0x6908bbb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e8bbb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8bbbf, 66 },
    { 0, 0 },
    { 0x7688bbc1, 66 },
    { 0xdfe8bbc2, 99 },
    { 0, 0 },
    { 0x3af8bbc4, 61 },
    { 0, 0 },
    { 0xa548bbc6, 95 },
    { 0xbd8bbc6, 33 },
    { 0xc638bbc8, 24 },
    { 0, 0 },
    { 0x5b98bbca, 33 },
    { 0xc828bbcb, 98 },
    { 0xdce8bbcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a8bbd0, 33 },
    { 0x1268bbd0, 40 },
    { 0, 0 },
    { 0x638bbd3, 98 },
    { 0, 0 },
    { 0xcbf8bbd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb458bbd9, 95 },
    { 0xff38bbd9, 61 },
    { 0, 0 },
    { 0x6f18bbdc, 93 },
    { 0x4d08bbdc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa628bbe0, 93 },
    { 0x62a8bbe1, 90 },
    { 0, 0 },
    { 0x8998bbe3, 77 },
    { 0x85d8bbe3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2838bbe7, 94 },
    { 0xe0d8bbe7, 61 },
    { 0xc368bbe9, 56 },
    { 0xe278bbea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7518bbf1, 76 },
    { 0, 0 },
    { 0xd208bbf3, 93 },
    { 0x3738bbf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e8bbf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8bbfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3588bc03, 95 },
    { 0xd518bc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2788bc0b, 77 },
    { 0x5728bc0b, 90 },
    { 0, 0 },
    { 0xba18bc0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef68bc18, 61 },
    { 0x1b58bc18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c28bc1e, 91 },
    { 0, 0 },
    { 0xaf8bc20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf8bc23, 66 },
    { 0x1378bc24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78bc27, 98 },
    { 0, 0 },
    { 0x3038bc29, 90 },
    { 0, 0 },
    { 0x6878bc2b, 33 },
    { 0x4de8bc2c, 76 },
    { 0, 0 },
    { 0x5ce8bc2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb498bc31, 93 },
    { 0, 0 },
    { 0xdf28bc33, 76 },
    { 0x7cf8bc34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c8bc39, 67 },
    { 0x7ff8bc3a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f8bc3d, 33 },
    { 0, 0 },
    { 0xaa28bc3f, 61 },
    { 0, 0 },
    { 0xd438bc41, 78 },
    { 0xf328bc41, 69 },
    { 0x5318bc43, 34 },
    { 0x75f8bc43, 95 },
    { 0xedf8bc45, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e58bc49, 43 },
    { 0xca88bc4a, 90 },
    { 0xcba8bc4b, 90 },
    { 0, 0 },
    { 0x9fa8bc4d, 98 },
    { 0xa628bc4d, 95 },
    { 0x4938bc4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5858bc52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e98bc55, 23 },
    { 0x1d18bc56, 61 },
    { 0xf3f8bc57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca8bc5a, 33 },
    { 0xb8b8bc5a, 46 },
    { 0, 0 },
    { 0x3ec8bc5d, 85 },
    { 0, 0 },
    { 0x9608bc5f, 85 },
    { 0x8248bc60, 34 },
    { 0xf408bc61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08bc64, 61 },
    { 0, 0 },
    { 0x44d8bc66, 90 },
    { 0x54c8bc67, 56 },
    { 0x668bc68, 23 },
    { 0x268bc69, 43 },
    { 0, 0 },
    { 0x23f8bc6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e8bc6f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe8bc72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9348bc76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f48bc79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c8bc7f, 95 },
    { 0x6688bc80, 34 },
    { 0, 0 },
    { 0xb408bc82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8388bc85, 66 },
    { 0, 0 },
    { 0x6fa8bc87, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc8bc8c, 66 },
    { 0xf538bc8c, 43 },
    { 0, 0 },
    { 0x4938bc8f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd78bc92, 95 },
    { 0x8228bc93, 46 },
    { 0, 0 },
    { 0x4458bc95, 98 },
    { 0x17d8bc95, 52 },
    { 0x4738bc97, 99 },
    { 0x16d8bc95, 61 },
    { 0xe5a8bc99, 52 },
    { 0x9e38bc97, 33 },
    { 0xcb58bc98, 90 },
    { 0xabd8bc9c, 95 },
    { 0x8d18bc9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5418bca4, 90 },
    { 0x2738bca4, 53 },
    { 0, 0 },
    { 0xda48bca7, 90 },
    { 0x24f8bca7, 77 },
    { 0x42d8bca8, 33 },
    { 0x6248bca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98bcad, 90 },
    { 0xdd98bcae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b58bcb2, 90 },
    { 0x17b8bcb3, 55 },
    { 0, 0 },
    { 0x1278bcb5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68bcba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c8bcbd, 93 },
    { 0x8038bcbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a98bcc5, 77 },
    { 0x1fb8bcc6, 68 },
    { 0x35a8bcc7, 23 },
    { 0x5eb8bcc7, 81 },
    { 0x2248bcc9, 93 },
    { 0x4668bcca, 55 },
    { 0xb048bcca, 23 },
    { 0xf728bccc, 90 },
    { 0x43b8bccd, 76 },
    { 0, 0 },
    { 0x6b8bccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d38bcd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8bcd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a28bcdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d98bce2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8438bce8, 78 },
    { 0x78f8bce8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248bcef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d8bcf3, 55 },
    { 0x33f8bcf4, 90 },
    { 0xc178bcf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea8bcfa, 52 },
    { 0, 0 },
    { 0xd668bcfc, 33 },
    { 0, 0 },
    { 0x9ae8bcfe, 61 },
    { 0xd5e8bcfe, 91 },
    { 0, 0 },
    { 0x578bd01, 90 },
    { 0xe408bd01, 33 },
    { 0x3638bd03, 98 },
    { 0x28a8bd04, 98 },
    { 0x90a8bd05, 78 },
    { 0x6598bd03, 90 },
    { 0xf1e8bd01, 34 },
    { 0, 0 },
    { 0x5f68bd09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188bd10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b8bd17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb8bd21, 65 },
    { 0, 0 },
    { 0x9058bd23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3428bd26, 76 },
    { 0x4d68bd26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e8bd2e, 55 },
    { 0xe6d8bd2f, 61 },
    { 0x1608bd30, 95 },
    { 0, 0 },
    { 0x2b48bd32, 76 },
    { 0x62b8bd32, 66 },
    { 0, 0 },
    { 0xd688bd35, 76 },
    { 0, 0 },
    { 0xfa18bd37, 99 },
    { 0xda18bd38, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa458bd3b, 98 },
    { 0x60b8bd3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f38bd3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb78bd42, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1498bd49, 77 },
    { 0, 0 },
    { 0x8648bd4b, 98 },
    { 0x43f8bd4c, 23 },
    { 0x6168bd4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad8bd51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c8bd55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c48bd59, 77 },
    { 0x5188bd5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8bd61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf08bd66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d58bd69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8bd6d, 61 },
    { 0xba78bd6e, 91 },
    { 0x2188bd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee08bd74, 43 },
    { 0x6a88bd74, 61 },
    { 0, 0 },
    { 0x3978bd77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4978bd7a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d8bd81, 95 },
    { 0x6de8bd82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4138bd85, 98 },
    { 0x4578bd85, 23 },
    { 0x2668bd85, 100 },
    { 0x4d68bd88, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148bd8e, 99 },
    { 0x5728bd8f, 43 },
    { 0xe78bd90, 61 },
    { 0x54b8bd8e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c8bd98, 61 },
    { 0xed98bd98, 61 },
    { 0x39b8bd99, 55 },
    { 0xbf88bd99, 33 },
    { 0, 0 },
    { 0x82f8bd9d, 33 },
    { 0xb718bd9d, 98 },
    { 0, 0 },
    { 0xda98bda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a8bda8, 23 },
    { 0, 0 },
    { 0x4838bdaa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db8bdae, 90 },
    { 0xb288bdae, 41 },
    { 0, 0 },
    { 0x758bdb1, 85 },
    { 0x9a38bdb1, 34 },
    { 0, 0 },
    { 0x74c8bdb4, 90 },
    { 0xdbd8bdb5, 99 },
    { 0xff98bdb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2458bdb9, 90 },
    { 0x1058bdba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc8bdbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2348bdc2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848bdc5, 90 },
    { 0xe188bdc5, 85 },
    { 0, 0 },
    { 0xee88bdc8, 91 },
    { 0xc3b8bdc8, 87 },
    { 0, 0 },
    { 0xd9d8bdcb, 77 },
    { 0x5318bdcc, 90 },
    { 0x4f78bdcb, 17 },
    { 0x8088bdcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca8bdd7, 99 },
    { 0x5ab8bdd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab98bddf, 23 },
    { 0xd6a8bddf, 61 },
    { 0, 0 },
    { 0x2c58bde2, 98 },
    { 0x9a68bde2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e8bdf2, 33 },
    { 0x1e78bdf3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8bdf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9498bdfc, 52 },
    { 0x7b28bdfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3628be04, 91 },
    { 0x9c88be05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba28be12, 85 },
    { 0xeba8be13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d8be19, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8be1c, 66 },
    { 0, 0 },
    { 0x1678be1e, 90 },
    { 0x3478be1f, 90 },
    { 0, 0 },
    { 0x3768be21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3258be26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1968be2c, 33 },
    { 0, 0 },
    { 0xd398be2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc8be31, 33 },
    { 0x7948be32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e78be35, 99 },
    { 0xafd8be36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc68be3f, 100 },
    { 0x8ac8be40, 61 },
    { 0x58a8be41, 61 },
    { 0xdf08be40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef8be46, 90 },
    { 0x3348be47, 96 },
    { 0, 0 },
    { 0x9d38be49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3428be50, 90 },
    { 0xf888be50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8be55, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408be59, 23 },
    { 0xcc18be59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c8be5d, 61 },
    { 0xd7c8be5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1018be63, 46 },
    { 0, 0 },
    { 0x5128be65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd8be6b, 51 },
    { 0xc48be6c, 64 },
    { 0x46c8be6d, 95 },
    { 0x49e8be6d, 84 },
    { 0x5c8be6b, 61 },
    { 0xd528be6f, 76 },
    { 0x3848be6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a8be75, 66 },
    { 0x25a8be76, 55 },
    { 0x2788be75, 33 },
    { 0xa4d8be78, 90 },
    { 0x28f8be79, 87 },
    { 0x9448be7a, 23 },
    { 0xdf38be79, 52 },
    { 0, 0 },
    { 0x52c8be7d, 75 },
    { 0x6178be7e, 100 },
    { 0xdab8be7f, 33 },
    { 0x7f88be80, 23 },
    { 0x4e8be80, 90 },
    { 0x1d98be81, 43 },
    { 0x54b8be83, 90 },
    { 0x9d98be84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd8be89, 61 },
    { 0, 0 },
    { 0x7a58be8b, 65 },
    { 0xbf38be8c, 76 },
    { 0xadb8be8d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f8be94, 93 },
    { 0x408be95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a98be9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8be9d, 78 },
    { 0x3848be9d, 43 },
    { 0xde88be9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb478bea4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8bead, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb8beb0, 90 },
    { 0x19f8beb1, 76 },
    { 0xf038beb1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8beba, 38 },
    { 0x41d8bebb, 91 },
    { 0xd118bebc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3248bec1, 98 },
    { 0xfac8bec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608becd, 23 },
    { 0x9df8bece, 52 },
    { 0xff48becd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5778bed4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1448bedb, 91 },
    { 0x9d98bedc, 81 },
    { 0xd758bedd, 98 },
    { 0xe508bedc, 34 },
    { 0x9d68bedb, 43 },
    { 0, 0 },
    { 0x5708bee1, 78 },
    { 0x76a8bee2, 98 },
    { 0x78bee3, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9278bee7, 43 },
    { 0, 0 },
    { 0x2b98bee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb28beed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb478bef3, 55 },
    { 0, 0 },
    { 0x1748bef5, 88 },
    { 0x7d28bef6, 55 },
    { 0x1f28bef7, 61 },
    { 0xc3b8bef8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7728befc, 77 },
    { 0x16d8befd, 34 },
    { 0, 0 },
    { 0x6088beff, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f8bf04, 95 },
    { 0, 0 },
    { 0x24d8bf06, 23 },
    { 0xc268bf07, 95 },
    { 0xc308bf08, 98 },
    { 0xc508bf09, 66 },
    { 0xfe88bf06, 99 },
    { 0xa308bf06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae08bf11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac8bf14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed18bf17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b08bf1b, 87 },
    { 0, 0 },
    { 0x2478bf1d, 65 },
    { 0x1bd8bf1e, 90 },
    { 0, 0 },
    { 0x3838bf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e8bf23, 66 },
    { 0x55b8bf24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x618bf27, 33 },
    { 0x5f68bf28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b28bf2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8bf32, 78 },
    { 0xace8bf32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9748bf3e, 23 },
    { 0, 0 },
    { 0xfc08bf40, 76 },
    { 0x3ab8bf41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e8bf44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a8bf47, 33 },
    { 0xda08bf48, 23 },
    { 0x6348bf48, 90 },
    { 0x9078bf49, 46 },
    { 0, 0 },
    { 0xfb08bf4c, 91 },
    { 0xed88bf4d, 33 },
    { 0x4b88bf4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d8bf52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb598bf5c, 61 },
    { 0, 0 },
    { 0xa408bf5e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7368bf61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e08bf65, 23 },
    { 0xe438bf66, 93 },
    { 0x10d8bf67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e8bf6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb908bf6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7898bf73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9648bf76, 88 },
    { 0, 0 },
    { 0xbf48bf78, 33 },
    { 0xf38bf79, 95 },
    { 0xca68bf7a, 90 },
    { 0x4428bf79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc8bf80, 23 },
    { 0x1078bf81, 61 },
    { 0x9588bf82, 76 },
    { 0xde28bf81, 76 },
    { 0, 0 },
    { 0x9f68bf85, 55 },
    { 0, 0 },
    { 0x30d8bf87, 90 },
    { 0xb328bf88, 23 },
    { 0, 0 },
    { 0x7858bf8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b8bf90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc18bf95, 67 },
    { 0x3038bf96, 85 },
    { 0xdd88bf95, 23 },
    { 0x51d8bf96, 23 },
    { 0x50a8bf99, 98 },
    { 0x6338bf98, 43 },
    { 0x7098bf9a, 95 },
    { 0xb998bf9c, 42 },
    { 0x22a8bf9c, 90 },
    { 0xf68bf9e, 33 },
    { 0xb158bf9f, 95 },
    { 0x34b8bf9d, 76 },
    { 0xd328bfa0, 80 },
    { 0x398bfa2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b48bfa6, 99 },
    { 0x75e8bfa7, 85 },
    { 0, 0 },
    { 0x348bfa9, 91 },
    { 0x2618bfaa, 61 },
    { 0x7e98bfaa, 17 },
    { 0x3fc8bfac, 61 },
    { 0x1828bfa9, 33 },
    { 0xc288bfa9, 94 },
    { 0x6188bfaf, 90 },
    { 0xe708bfaf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f8bfb3, 75 },
    { 0xbb68bfb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa8bfb8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa38bfbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed8bfbf, 67 },
    { 0x1b78bfc0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3198bfc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c8bfca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d88bfcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c28bfd0, 69 },
    { 0, 0 },
    { 0xd8f8bfd2, 61 },
    { 0, 0 },
    { 0x768bfd4, 99 },
    { 0, 0 },
    { 0x1c08bfd6, 34 },
    { 0, 0 },
    { 0xb368bfd8, 68 },
    { 0xc1d8bfd9, 61 },
    { 0xc2e8bfd9, 98 },
    { 0xbca8bfd8, 81 },
    { 0xde98bfdc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8578bfe2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2138bfe6, 99 },
    { 0, 0 },
    { 0x1388bfe8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d8bfef, 23 },
    { 0xa6c8bfef, 95 },
    { 0xc5f8bff1, 46 },
    { 0xa3c8bff1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc128bff5, 61 },
    { 0xe8c8bff6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c8bff9, 95 },
    { 0x2d28bffa, 48 },
    { 0, 0 },
    { 0x5698bffc, 90 },
    { 0x81b8bffc, 67 },
    { 0, 0 },
    { 0xc9a8bfff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b28c004, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea8c007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68c00b, 90 },
    { 0xe88c00c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x728c00f, 95 },
    { 0x62d8c00f, 36 },
    { 0, 0 },
    { 0xa228c012, 46 },
    { 0x3d78c013, 90 },
    { 0xa4d8c014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b18c01b, 83 },
    { 0x3108c01c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a8c029, 87 },
    { 0, 0 },
    { 0xfa8c02b, 99 },
    { 0xb5c8c02c, 98 },
    { 0xbde8c02c, 78 },
    { 0xc2d8c02c, 33 },
    { 0xb318c02f, 95 },
    { 0xd508c030, 93 },
    { 0x9958c031, 98 },
    { 0x1d18c032, 98 },
    { 0, 0 },
    { 0x35a8c034, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb938c037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c28c03d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa498c040, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc318c044, 55 },
    { 0x3f48c045, 99 },
    { 0, 0 },
    { 0x9c68c047, 93 },
    { 0x85f8c047, 33 },
    { 0, 0 },
    { 0x3758c04a, 90 },
    { 0x1698c04b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7088c051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3658c057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec98c05a, 66 },
    { 0xfdc8c05a, 43 },
    { 0xed08c05c, 91 },
    { 0xdb08c05d, 61 },
    { 0, 0 },
    { 0x16c8c05f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b8c069, 61 },
    { 0xf188c06a, 90 },
    { 0x258c06b, 61 },
    { 0x9308c06c, 23 },
    { 0x4fa8c06d, 34 },
    { 0xbf08c06d, 33 },
    { 0xf0c8c06e, 55 },
    { 0x7c58c070, 33 },
    { 0, 0 },
    { 0xbbc8c072, 98 },
    { 0x9028c072, 98 },
    { 0x60b8c074, 90 },
    { 0xb258c074, 83 },
    { 0x9798c072, 76 },
    { 0x70d8c077, 33 },
    { 0xae8c076, 61 },
    { 0, 0 },
    { 0x72c8c07a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc38c07e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf278c084, 77 },
    { 0, 0 },
    { 0xb628c086, 33 },
    { 0x2028c087, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c8c08a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8168c08e, 67 },
    { 0x4bf8c08e, 46 },
    { 0xfe98c08f, 95 },
    { 0x3d88c091, 52 },
    { 0x438c08e, 91 },
    { 0x26e8c090, 33 },
    { 0x95a8c094, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb008c099, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5098c0a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc958c0a8, 61 },
    { 0x8498c0a8, 83 },
    { 0x9ae8c0a8, 33 },
    { 0xf0f8c0ab, 84 },
    { 0xc918c0ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8c0af, 23 },
    { 0x12d8c0b0, 33 },
    { 0, 0 },
    { 0x4bb8c0b2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6798c0b5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd8c0ba, 23 },
    { 0xb3a8c0ba, 99 },
    { 0x1c98c0ba, 90 },
    { 0xa728c0ba, 61 },
    { 0x6078c0bc, 95 },
    { 0xfd8c0bf, 61 },
    { 0xb2a8c0bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c28c0c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c8c0ce, 46 },
    { 0x2808c0cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c78c0d3, 23 },
    { 0xfad8c0d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa598c0da, 33 },
    { 0x3d08c0db, 97 },
    { 0, 0 },
    { 0x3168c0dd, 98 },
    { 0x6148c0de, 91 },
    { 0xdd38c0dd, 33 },
    { 0x638c0e0, 90 },
    { 0x7e08c0e1, 84 },
    { 0xfd58c0e2, 98 },
    { 0x23e8c0e1, 95 },
    { 0xabf8c0e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5938c0e7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc8c0ef, 76 },
    { 0xd298c0f0, 90 },
    { 0, 0 },
    { 0x8f68c0f2, 93 },
    { 0xeaa8c0f3, 90 },
    { 0xbb58c0f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd8c0f9, 80 },
    { 0, 0 },
    { 0x5d98c0fb, 61 },
    { 0x5a38c0fc, 99 },
    { 0x4068c0fb, 90 },
    { 0, 0 },
    { 0x7968c0ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98c103, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2368c109, 91 },
    { 0x7228c10a, 90 },
    { 0x3418c10b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138c10f, 84 },
    { 0x1b58c110, 84 },
    { 0x64e8c111, 61 },
    { 0xa768c110, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f38c116, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc8c11a, 94 },
    { 0, 0 },
    { 0x3678c11c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78c122, 90 },
    { 0x9688c123, 23 },
    { 0xfb88c123, 95 },
    { 0x7808c125, 46 },
    { 0xd018c125, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c8c129, 52 },
    { 0, 0 },
    { 0x6718c12b, 95 },
    { 0x2bc8c12b, 98 },
    { 0, 0 },
    { 0x9678c12e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6778c138, 33 },
    { 0, 0 },
    { 0x76a8c13a, 76 },
    { 0x2468c13b, 95 },
    { 0x3538c13c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638c13f, 43 },
    { 0, 0 },
    { 0x398c141, 60 },
    { 0, 0 },
    { 0x9f08c143, 90 },
    { 0xff58c144, 76 },
    { 0, 0 },
    { 0x7f18c146, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d48c149, 88 },
    { 0x8f18c149, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b8c153, 33 },
    { 0xbb58c154, 69 },
    { 0x5208c154, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478c15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68c164, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b8c168, 95 },
    { 0, 0 },
    { 0xb228c16a, 88 },
    { 0, 0 },
    { 0xaa18c16c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da8c171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6018c174, 95 },
    { 0x5ca8c174, 90 },
    { 0xf6d8c176, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb8c17a, 56 },
    { 0x70f8c17b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff8c17e, 61 },
    { 0, 0 },
    { 0xdd08c180, 23 },
    { 0xb98c181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd368c185, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa8c188, 66 },
    { 0xb898c189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d78c18c, 90 },
    { 0, 0 },
    { 0x9598c18e, 52 },
    { 0x7598c18e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad98c198, 98 },
    { 0xea28c198, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee8c19c, 90 },
    { 0x17c8c19c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d8c1a1, 46 },
    { 0x3458c1a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88c1a6, 95 },
    { 0x5288c1a7, 93 },
    { 0x6318c1a7, 88 },
    { 0x7cd8c1a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2818c1b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa078c1bb, 95 },
    { 0xbb08c1bc, 90 },
    { 0x908c1bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9388c1c4, 95 },
    { 0x7a38c1c5, 90 },
    { 0x32f8c1c5, 19 },
    { 0xd6f8c1c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f8c1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc18c1d0, 33 },
    { 0x42b8c1d1, 95 },
    { 0x7258c1d1, 85 },
    { 0xe9a8c1d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9568c1d8, 66 },
    { 0x4bf8c1d8, 33 },
    { 0x97d8c1da, 23 },
    { 0xcdf8c1db, 61 },
    { 0x3e08c1dc, 17 },
    { 0x8058c1dc, 95 },
    { 0xa9a8c1dc, 90 },
    { 0x8318c1dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb618c1e2, 61 },
    { 0x3d68c1e3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe378c1e9, 65 },
    { 0x2878c1ea, 61 },
    { 0xd278c1ea, 61 },
    { 0, 0 },
    { 0x3b18c1ed, 34 },
    { 0x6558c1ee, 43 },
    { 0x3de8c1ef, 85 },
    { 0xdcb8c1ef, 76 },
    { 0, 0 },
    { 0xdde8c1f2, 95 },
    { 0x1368c1f3, 76 },
    { 0xa6c8c1f4, 95 },
    { 0, 0 },
    { 0xd148c1f6, 90 },
    { 0, 0 },
    { 0x74e8c1f8, 23 },
    { 0xdab8c1f9, 90 },
    { 0x7148c1f8, 90 },
    { 0, 0 },
    { 0x33d8c1fc, 61 },
    { 0x32a8c1fd, 55 },
    { 0, 0 },
    { 0xb8e8c1ff, 23 },
    { 0, 0 },
    { 0xcbe8c201, 17 },
    { 0x1a38c202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac8c205, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5338c20c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc8c211, 98 },
    { 0x8b38c212, 76 },
    { 0x1b08c212, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c58c217, 15 },
    { 0x1378c218, 98 },
    { 0x10b8c218, 95 },
    { 0, 0 },
    { 0xf3a8c21b, 76 },
    { 0, 0 },
    { 0x368c21d, 95 },
    { 0x128c21d, 76 },
    { 0, 0 },
    { 0x248c220, 98 },
    { 0xd4d8c221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f8c224, 100 },
    { 0x9d48c225, 100 },
    { 0x4e38c226, 2 },
    { 0x5438c227, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed8c22a, 95 },
    { 0x8a8c22a, 95 },
    { 0x7c08c22a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc878c230, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9528c234, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d8c239, 33 },
    { 0x2488c23a, 43 },
    { 0x1a28c23a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f8c241, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd58c247, 91 },
    { 0xaaf8c248, 95 },
    { 0, 0 },
    { 0x498c24a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9658c254, 61 },
    { 0xd998c254, 90 },
    { 0x5408c255, 99 },
    { 0xed38c257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb538c25c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7878c25f, 78 },
    { 0x9c58c260, 17 },
    { 0x9448c261, 100 },
    { 0x8158c262, 93 },
    { 0x1788c260, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1258c266, 90 },
    { 0xda38c267, 34 },
    { 0, 0 },
    { 0x1f08c269, 61 },
    { 0x5288c269, 90 },
    { 0x3ca8c26b, 34 },
    { 0x6178c26c, 95 },
    { 0x1618c26d, 67 },
    { 0x9948c26e, 90 },
    { 0x5758c26f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d48c275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5438c27a, 23 },
    { 0, 0 },
    { 0xaf88c27c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd088c282, 38 },
    { 0x2a48c283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc78c286, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8948c28a, 90 },
    { 0x6b28c28b, 61 },
    { 0, 0 },
    { 0xfe38c28d, 100 },
    { 0, 0 },
    { 0x278c28f, 33 },
    { 0x82b8c28f, 95 },
    { 0, 0 },
    { 0x9298c292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8c29e, 66 },
    { 0, 0 },
    { 0x87c8c2a0, 66 },
    { 0x53a8c2a0, 98 },
    { 0xf488c2a2, 76 },
    { 0, 0 },
    { 0xef38c2a4, 99 },
    { 0xfbc8c2a5, 65 },
    { 0x418c2a5, 87 },
    { 0, 0 },
    { 0xdc28c2a8, 40 },
    { 0, 0 },
    { 0xfa38c2aa, 98 },
    { 0x1568c2aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2548c2ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718c2b6, 90 },
    { 0xdd88c2b7, 76 },
    { 0xa148c2b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f48c2bd, 23 },
    { 0xc8c8c2bd, 99 },
    { 0, 0 },
    { 0xb428c2c0, 55 },
    { 0, 0 },
    { 0x6e88c2c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd788c2c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878c2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e28c2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed78c2d0, 93 },
    { 0, 0 },
    { 0xe5e8c2d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8c2d6, 77 },
    { 0xadc8c2d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d28c2db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4738c2de, 61 },
    { 0, 0 },
    { 0xdde8c2e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4718c2e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958c2ec, 76 },
    { 0xfe28c2ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a98c2f1, 52 },
    { 0xb868c2f2, 95 },
    { 0x3b58c2f2, 90 },
    { 0x1d78c2f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c2f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d8c300, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f8c305, 23 },
    { 0xd7c8c306, 23 },
    { 0xee58c307, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b8c30b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9928c30e, 90 },
    { 0, 0 },
    { 0x6a78c310, 91 },
    { 0x9d28c311, 95 },
    { 0x91c8c311, 43 },
    { 0xfe78c311, 55 },
    { 0x2918c314, 90 },
    { 0x4dc8c315, 98 },
    { 0x3a88c316, 95 },
    { 0xe888c315, 98 },
    { 0xf08c318, 33 },
    { 0, 0 },
    { 0x70a8c31a, 85 },
    { 0, 0 },
    { 0xdcb8c31c, 95 },
    { 0, 0 },
    { 0x85e8c31e, 90 },
    { 0x88c31f, 43 },
    { 0x6118c320, 55 },
    { 0, 0 },
    { 0xc4d8c322, 34 },
    { 0xff78c323, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c327, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7238c32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7698c331, 23 },
    { 0x8f18c331, 43 },
    { 0, 0 },
    { 0x3348c334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8c338, 90 },
    { 0x5848c339, 33 },
    { 0, 0 },
    { 0x4168c33b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4138c341, 23 },
    { 0xb458c341, 76 },
    { 0x88c8c341, 66 },
    { 0x85a8c344, 23 },
    { 0, 0 },
    { 0x7bf8c346, 88 },
    { 0x19f8c347, 90 },
    { 0xa7c8c348, 33 },
    { 0, 0 },
    { 0xb8c8c34a, 77 },
    { 0xda18c34b, 69 },
    { 0xf6f8c34c, 98 },
    { 0x21a8c34d, 33 },
    { 0x5708c34e, 90 },
    { 0xe968c34f, 33 },
    { 0x4e08c350, 76 },
    { 0xe2b8c351, 98 },
    { 0x59e8c351, 61 },
    { 0xfd8c350, 80 },
    { 0x5f68c351, 46 },
    { 0x3ed8c355, 61 },
    { 0xe8f8c356, 76 },
    { 0xa808c34b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe98c35a, 84 },
    { 0xb68c35b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c8c361, 61 },
    { 0, 0 },
    { 0x3b08c363, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928c369, 90 },
    { 0xae78c36a, 23 },
    { 0x1d58c36a, 65 },
    { 0x3778c36c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8378c374, 93 },
    { 0, 0 },
    { 0xe498c376, 95 },
    { 0, 0 },
    { 0x6ec8c378, 90 },
    { 0, 0 },
    { 0xd1f8c37a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba8c37e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf018c381, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1788c385, 75 },
    { 0x94d8c385, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b18c38b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd98c38f, 99 },
    { 0x4728c390, 53 },
    { 0xcb98c391, 98 },
    { 0x80e8c390, 33 },
    { 0, 0 },
    { 0x8108c394, 9 },
    { 0x7f58c395, 8 },
    { 0x8ff8c396, 90 },
    { 0x27c8c397, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148c39b, 55 },
    { 0, 0 },
    { 0x8bc8c39d, 91 },
    { 0x588c39d, 33 },
    { 0xfdc8c39d, 91 },
    { 0xeeb8c3a0, 65 },
    { 0xb388c3a1, 61 },
    { 0xd368c39f, 84 },
    { 0xc478c3a3, 33 },
    { 0x2758c3a3, 98 },
    { 0x6978c3a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd38c3ae, 76 },
    { 0, 0 },
    { 0x8ea8c3b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8c3b6, 61 },
    { 0x6828c3b6, 55 },
    { 0xa668c3b8, 87 },
    { 0xd128c3b7, 23 },
    { 0xfeb8c3ba, 76 },
    { 0, 0 },
    { 0x1478c3bc, 98 },
    { 0xc78c3bd, 90 },
    { 0x3728c3bc, 85 },
    { 0xf0d8c3be, 33 },
    { 0xfd28c3c0, 90 },
    { 0x1eb8c3c1, 33 },
    { 0xc58c3c2, 53 },
    { 0x3a38c3c3, 95 },
    { 0x85a8c3c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e8c3c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8c3cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad68c3cf, 76 },
    { 0, 0 },
    { 0x4218c3d1, 98 },
    { 0xb268c3d2, 23 },
    { 0x70c8c3d3, 33 },
    { 0, 0 },
    { 0xd768c3d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a68c3d9, 78 },
    { 0, 0 },
    { 0x6948c3db, 90 },
    { 0x7fc8c3dc, 23 },
    { 0xd8c3db, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec8c3e0, 33 },
    { 0x9d18c3e0, 61 },
    { 0xa948c3e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a8c3e5, 90 },
    { 0xf118c3e6, 61 },
    { 0x9f08c3e7, 75 },
    { 0x6c38c3e8, 100 },
    { 0, 0 },
    { 0xe138c3ea, 61 },
    { 0xb578c3eb, 33 },
    { 0, 0 },
    { 0x1358c3ed, 61 },
    { 0, 0 },
    { 0xe468c3ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8c3f2, 90 },
    { 0, 0 },
    { 0x3fa8c3f4, 76 },
    { 0, 0 },
    { 0xd018c3f6, 43 },
    { 0xc208c3f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd88c3fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd648c3ff, 61 },
    { 0xb7c8c400, 81 },
    { 0xd08c401, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d8c404, 99 },
    { 0xe428c405, 98 },
    { 0xc098c404, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f8c40e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08c414, 98 },
    { 0xbdf8c414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78c418, 78 },
    { 0x23b8c418, 33 },
    { 0x9338c41a, 76 },
    { 0x4388c418, 95 },
    { 0, 0 },
    { 0x9718c41d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a8c425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9578c428, 76 },
    { 0xe728c429, 81 },
    { 0x4738c429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc8c430, 98 },
    { 0x54b8c431, 95 },
    { 0, 0 },
    { 0xdf78c433, 40 },
    { 0x2408c434, 98 },
    { 0xb238c435, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dd8c43a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d8c440, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98c447, 34 },
    { 0x7f68c448, 61 },
    { 0xc058c449, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0x848c44c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc308c44f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a8c454, 78 },
    { 0x8368c454, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b8c45b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3158c45f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798c469, 23 },
    { 0x518c469, 90 },
    { 0xfed8c46b, 53 },
    { 0xb168c469, 75 },
    { 0x7f58c46d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9228c471, 77 },
    { 0x1f38c472, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc8c475, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7338c479, 90 },
    { 0xc428c47a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7178c47d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca8c481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe648c485, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3388c488, 78 },
    { 0, 0 },
    { 0x61c8c48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9d8c48d, 76 },
    { 0x578c48d, 23 },
    { 0x4848c48e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce8c493, 98 },
    { 0, 0 },
    { 0x6758c495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb8c499, 93 },
    { 0, 0 },
    { 0x9998c49b, 95 },
    { 0, 0 },
    { 0x2748c49d, 55 },
    { 0, 0 },
    { 0x3b78c49f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc678c4a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc8c4ad, 90 },
    { 0xd1d8c4ae, 23 },
    { 0xb118c4ae, 23 },
    { 0xb2b8c4ad, 55 },
    { 0x5508c4b1, 76 },
    { 0, 0 },
    { 0xe598c4b3, 76 },
    { 0x3848c4b4, 23 },
    { 0xcce8c4b5, 33 },
    { 0xbba8c4b5, 91 },
    { 0, 0 },
    { 0xaa08c4b8, 55 },
    { 0x7508c4b9, 76 },
    { 0x4ac8c4ba, 101 },
    { 0xb3c8c4bb, 98 },
    { 0, 0 },
    { 0xbef8c4bd, 67 },
    { 0xd448c4bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae8c4c3, 61 },
    { 0x72d8c4c3, 90 },
    { 0x6068c4c5, 61 },
    { 0x7e8c4c6, 55 },
    { 0xb0e8c4c7, 90 },
    { 0x6c88c4c8, 61 },
    { 0x6cc8c4c4, 93 },
    { 0xf788c4c3, 99 },
    { 0x7a08c4cb, 84 },
    { 0x1438c4cb, 90 },
    { 0x5d68c4cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa78c4d3, 99 },
    { 0, 0 },
    { 0x4ec8c4d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a38c4d8, 90 },
    { 0xc808c4d9, 33 },
    { 0xcfe8c4da, 46 },
    { 0x9408c4db, 90 },
    { 0xc3f8c4dc, 87 },
    { 0xda08c4dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c8c4e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8c4e3, 84 },
    { 0, 0 },
    { 0xeea8c4e5, 95 },
    { 0xbe98c4e6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6798c4eb, 33 },
    { 0x33a8c4ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xea48c4ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae8c4f3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c38c4f8, 75 },
    { 0x8138c4f9, 76 },
    { 0xb228c4f9, 95 },
    { 0x7ed8c4fb, 95 },
    { 0x78b8c4fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab8c500, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d8c505, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538c50b, 76 },
    { 0x7558c50b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3758c512, 100 },
    { 0xc638c513, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3388c516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb8c51f, 80 },
    { 0x8ab8c520, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be8c523, 91 },
    { 0, 0 },
    { 0xa368c525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc548c52f, 90 },
    { 0x4898c52f, 46 },
    { 0, 0 },
    { 0xe638c532, 91 },
    { 0x31c8c532, 84 },
    { 0x1e8c534, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd8c538, 95 },
    { 0, 0 },
    { 0xf6d8c53a, 34 },
    { 0xae88c53b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a8c53e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f78c542, 98 },
    { 0x4028c543, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc478c546, 61 },
    { 0x3f8c547, 95 },
    { 0x7168c547, 33 },
    { 0xbcf8c549, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa438c54c, 100 },
    { 0xb2e8c54d, 77 },
    { 0, 0 },
    { 0xc08c54f, 95 },
    { 0x7be8c54f, 95 },
    { 0, 0 },
    { 0x94a8c552, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x628c555, 90 },
    { 0, 0 },
    { 0xe498c557, 23 },
    { 0x388c558, 98 },
    { 0xdc38c559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb868c55e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3368c561, 78 },
    { 0x738c562, 33 },
    { 0x3ce8c562, 55 },
    { 0, 0 },
    { 0x9488c565, 95 },
    { 0, 0 },
    { 0xf5c8c567, 23 },
    { 0xf3a8c568, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e78c56d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4118c570, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98c573, 98 },
    { 0x7d48c574, 90 },
    { 0xcc68c575, 76 },
    { 0xe048c576, 56 },
    { 0xd048c577, 95 },
    { 0, 0 },
    { 0x68e8c579, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d18c580, 34 },
    { 0xe428c581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf78c584, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec8c58c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc568c591, 34 },
    { 0xfaf8c591, 33 },
    { 0x3bb8c593, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece8c597, 85 },
    { 0x15a8c597, 76 },
    { 0, 0 },
    { 0x6388c59a, 93 },
    { 0x9058c59b, 43 },
    { 0, 0 },
    { 0xc078c59d, 61 },
    { 0, 0 },
    { 0x7418c59f, 55 },
    { 0xb0b8c59f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd8c5a3, 90 },
    { 0xcc68c5a4, 61 },
    { 0x83e8c5a5, 76 },
    { 0x9568c5a5, 61 },
    { 0xf9b8c5a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df8c5b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2388c5b7, 90 },
    { 0x6ae8c5b8, 61 },
    { 0, 0 },
    { 0x5208c5ba, 17 },
    { 0xb98c5bb, 33 },
    { 0x7be8c5ba, 66 },
    { 0x1ab8c5ba, 36 },
    { 0xc88c5be, 83 },
    { 0x4278c5bd, 90 },
    { 0x6c18c5bc, 93 },
    { 0x5cc8c5c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb838c5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb908c5cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd8c5d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828c5d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8c5da, 98 },
    { 0x6158c5db, 91 },
    { 0x1ee8c5dc, 95 },
    { 0, 0 },
    { 0xed38c5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a8c5e4, 99 },
    { 0xd38c5e5, 98 },
    { 0, 0 },
    { 0x5f68c5e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c8c5ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8c5f3, 93 },
    { 0, 0 },
    { 0x69b8c5f5, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b8c5f9, 55 },
    { 0, 0 },
    { 0xc938c5fb, 95 },
    { 0x7d48c5fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce98c600, 95 },
    { 0x2368c601, 95 },
    { 0x7738c602, 85 },
    { 0, 0 },
    { 0xbc8c604, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608c608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58c613, 55 },
    { 0x2968c613, 66 },
    { 0x9198c613, 78 },
    { 0xf878c616, 34 },
    { 0, 0 },
    { 0x5b68c618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528c61d, 23 },
    { 0xf198c61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88c622, 33 },
    { 0, 0 },
    { 0xb1c8c624, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd8c630, 43 },
    { 0, 0 },
    { 0x10e8c632, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a08c636, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf68c63e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8c644, 76 },
    { 0x57e8c644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac78c648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6238c64c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc58c652, 69 },
    { 0, 0 },
    { 0x72e8c654, 99 },
    { 0xdc88c655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18c659, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb8c65c, 61 },
    { 0xce58c65c, 91 },
    { 0x6878c65e, 33 },
    { 0xf688c65d, 67 },
    { 0x9af8c660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc378c666, 33 },
    { 0x3e88c667, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b8c66a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2568c673, 96 },
    { 0x56d8c674, 78 },
    { 0, 0 },
    { 0xbb08c676, 61 },
    { 0xc4c8c676, 98 },
    { 0xb88c678, 33 },
    { 0xf3b8c677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9598c680, 85 },
    { 0xf708c681, 38 },
    { 0, 0 },
    { 0xae18c683, 23 },
    { 0, 0 },
    { 0x50d8c685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c98c68b, 76 },
    { 0x8b8c68c, 33 },
    { 0x6e08c68c, 33 },
    { 0xc978c68e, 95 },
    { 0x82e8c68e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2058c699, 33 },
    { 0x2178c69a, 55 },
    { 0x8228c69a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb8c69f, 66 },
    { 0x6358c6a0, 78 },
    { 0, 0 },
    { 0xa48c6a2, 43 },
    { 0, 0 },
    { 0xf408c6a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e78c6ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe138c6b1, 23 },
    { 0xffa8c6b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6598c6b8, 61 },
    { 0x9a28c6b9, 23 },
    { 0xa5d8c6ba, 42 },
    { 0xd088c6b8, 90 },
    { 0xbcf8c6bc, 61 },
    { 0xd968c6bd, 17 },
    { 0xfda8c6be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf448c6c2, 43 },
    { 0xc1e8c6c2, 90 },
    { 0xa178c6c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf78c6cb, 44 },
    { 0x51b8c6cc, 66 },
    { 0xcf38c6cd, 55 },
    { 0xd178c6cd, 55 },
    { 0x40d8c6cb, 43 },
    { 0x3878c6cd, 34 },
    { 0x2078c6ce, 55 },
    { 0, 0 },
    { 0x8b48c6d3, 76 },
    { 0x8898c6d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9748c6dc, 61 },
    { 0, 0 },
    { 0x17b8c6de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7568c6e5, 85 },
    { 0, 0 },
    { 0xcdb8c6e7, 95 },
    { 0x1a48c6e7, 91 },
    { 0x9e8c6e9, 33 },
    { 0, 0 },
    { 0x38a8c6eb, 76 },
    { 0xdce8c6ec, 23 },
    { 0x4648c6ed, 85 },
    { 0, 0 },
    { 0x7aa8c6ef, 90 },
    { 0x52a8c6f0, 90 },
    { 0xa668c6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a28c6f8, 86 },
    { 0xb9a8c6f8, 33 },
    { 0x28d8c6f9, 43 },
    { 0, 0 },
    { 0xc2b8c6fc, 95 },
    { 0x1d8c6fd, 33 },
    { 0x6c28c6fe, 55 },
    { 0x2518c6ff, 61 },
    { 0xfbd8c6ff, 76 },
    { 0, 0 },
    { 0xa728c702, 61 },
    { 0, 0 },
    { 0x1408c704, 55 },
    { 0xfb08c705, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe78c709, 66 },
    { 0xee48c70a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa8c70d, 61 },
    { 0, 0 },
    { 0x5a78c70f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e18c717, 87 },
    { 0x4658c718, 34 },
    { 0x64d8c719, 61 },
    { 0x1188c71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2118c71d, 87 },
    { 0x8108c71e, 95 },
    { 0xb488c71f, 88 },
    { 0xcc38c720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf18c726, 59 },
    { 0x7628c726, 91 },
    { 0, 0 },
    { 0x6c38c729, 90 },
    { 0xc358c72a, 61 },
    { 0x1608c72b, 99 },
    { 0, 0 },
    { 0xbad8c72d, 66 },
    { 0, 0 },
    { 0x9768c72f, 77 },
    { 0x8828c730, 61 },
    { 0xb3b8c731, 34 },
    { 0, 0 },
    { 0x71b8c733, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e8c73b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1968c73e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf158c747, 90 },
    { 0x8438c747, 95 },
    { 0x3638c749, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f88c74f, 77 },
    { 0, 0 },
    { 0xb8b8c751, 34 },
    { 0x9328c751, 46 },
    { 0xc4f8c753, 95 },
    { 0x9d58c754, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a58c759, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa828c75d, 95 },
    { 0xa718c75e, 34 },
    { 0xcfd8c75e, 87 },
    { 0, 0 },
    { 0xc278c761, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238c76a, 61 },
    { 0x1778c76b, 90 },
    { 0, 0 },
    { 0xa338c76d, 90 },
    { 0x7a48c76e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f8c772, 90 },
    { 0, 0 },
    { 0x3eb8c774, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad8c778, 78 },
    { 0, 0 },
    { 0x3fe8c77a, 87 },
    { 0x5a58c77b, 33 },
    { 0xb288c77c, 76 },
    { 0xb4d8c77d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe118c781, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e8c78a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b8c790, 23 },
    { 0x8a48c790, 97 },
    { 0, 0 },
    { 0xe2d8c793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6728c796, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3008c79c, 33 },
    { 0, 0 },
    { 0x90a8c79e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa448c7a2, 90 },
    { 0, 0 },
    { 0x4aa8c7a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd758c7a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e8c7ab, 98 },
    { 0x1838c7ac, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2538c7b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b08c7b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa348c7b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e98c7c6, 85 },
    { 0, 0 },
    { 0x6508c7c8, 91 },
    { 0x9478c7c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f8c7cc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb78c7d6, 81 },
    { 0, 0 },
    { 0xfe88c7d8, 93 },
    { 0xff28c7d9, 98 },
    { 0xdf58c7d8, 81 },
    { 0, 0 },
    { 0xa3c8c7dc, 23 },
    { 0, 0 },
    { 0x8738c7de, 90 },
    { 0x8558c7df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xced8c7e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f28c7e6, 33 },
    { 0x8008c7e7, 81 },
    { 0, 0 },
    { 0x6c58c7e9, 33 },
    { 0x47f8c7e9, 76 },
    { 0, 0 },
    { 0xd8c7ec, 23 },
    { 0x5c68c7ec, 76 },
    { 0xcd28c7ee, 80 },
    { 0x3688c7ec, 80 },
    { 0xd918c7f0, 43 },
    { 0, 0 },
    { 0xfb8c7f2, 61 },
    { 0x9fd8c7f3, 34 },
    { 0x8d18c7f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc458c7f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e8c7fa, 46 },
    { 0, 0 },
    { 0x6268c7fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f48c802, 98 },
    { 0xfb38c803, 33 },
    { 0xc168c802, 95 },
    { 0x6de8c803, 23 },
    { 0x3de8c804, 55 },
    { 0xbe88c802, 23 },
    { 0x49d8c808, 95 },
    { 0x4b88c806, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7218c80e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac8c811, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7338c815, 46 },
    { 0xeaf8c816, 33 },
    { 0xd9e8c817, 46 },
    { 0x51a8c817, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1558c81b, 98 },
    { 0x3e78c81c, 68 },
    { 0x4e8c81c, 46 },
    { 0xdcd8c81c, 100 },
    { 0x8418c81b, 90 },
    { 0, 0 },
    { 0x7618c821, 23 },
    { 0x7ff8c822, 84 },
    { 0xc708c821, 80 },
    { 0x1c28c824, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99f8c828, 68 },
    { 0xae68c828, 23 },
    { 0x9cf8c82a, 90 },
    { 0xef48c82b, 95 },
    { 0xa188c82c, 84 },
    { 0x1818c82b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8c834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e8c83b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2218c847, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b18c84f, 23 },
    { 0x5e8c850, 93 },
    { 0, 0 },
    { 0xc3b8c852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c8c856, 61 },
    { 0xbd88c856, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a18c85d, 95 },
    { 0x1dc8c85e, 33 },
    { 0x77c8c85e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6648c868, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed98c86c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f8c874, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad88c879, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee38c87d, 100 },
    { 0, 0 },
    { 0x7c8c87f, 4 },
    { 0, 0 },
    { 0xd528c881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a18c885, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9578c88a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c88d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f8c890, 90 },
    { 0, 0 },
    { 0xc518c892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e38c896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8718c89b, 88 },
    { 0xa5b8c89b, 76 },
    { 0x4a68c89c, 95 },
    { 0x7208c89e, 95 },
    { 0xd4f8c89e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3728c8a3, 67 },
    { 0, 0 },
    { 0x7f28c8a5, 78 },
    { 0x4f88c8a5, 61 },
    { 0x7578c8a5, 90 },
    { 0xca58c8a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6548c8ac, 61 },
    { 0, 0 },
    { 0x7aa8c8ae, 90 },
    { 0x9bd8c8ae, 23 },
    { 0xa318c8af, 33 },
    { 0x24e8c8b1, 43 },
    { 0x8d58c8b2, 55 },
    { 0x39f8c8b3, 64 },
    { 0xbb48c8b4, 93 },
    { 0xcd08c8b4, 46 },
    { 0x168c8b2, 84 },
    { 0x3e38c8b7, 61 },
    { 0x6528c8b5, 33 },
    { 0x8e78c8b9, 100 },
    { 0x6608c8ba, 33 },
    { 0xdeb8c8bb, 90 },
    { 0x1718c8bc, 100 },
    { 0x5088c8bd, 95 },
    { 0x1838c8bb, 23 },
    { 0x7258c8bd, 76 },
    { 0x1428c8b5, 98 },
    { 0x6e08c8c1, 93 },
    { 0xc4e8c8c2, 96 },
    { 0, 0 },
    { 0x97d8c8c4, 91 },
    { 0xb468c8c5, 90 },
    { 0x48b8c8c4, 61 },
    { 0x86f8c8c6, 90 },
    { 0x8408c8c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f8c8d0, 33 },
    { 0, 0 },
    { 0xa4d8c8d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe368c8d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb8c8dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada8c8e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c8c8ea, 88 },
    { 0, 0 },
    { 0x26f8c8ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46e8c8ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66b8c8fb, 87 },
    { 0x2878c8fc, 61 },
    { 0, 0 },
    { 0x8108c8fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e68c903, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8c907, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b78c90a, 67 },
    { 0x1788c90a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8258c910, 33 },
    { 0xcb48c910, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b8c915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3718c918, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab8c91b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf18c91e, 81 },
    { 0xe838c91f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2518c926, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb8c929, 23 },
    { 0x7998c92a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9828c92f, 43 },
    { 0x3538c930, 46 },
    { 0x81f8c931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6948c934, 90 },
    { 0, 0 },
    { 0x8748c936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8738c93b, 100 },
    { 0x4a68c93c, 76 },
    { 0, 0 },
    { 0x568c93e, 98 },
    { 0x9178c93f, 33 },
    { 0, 0 },
    { 0xe098c941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba8c949, 46 },
    { 0x2688c949, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3698c951, 33 },
    { 0xe618c951, 95 },
    { 0xc7b8c951, 95 },
    { 0, 0 },
    { 0xb8d8c955, 93 },
    { 0x22c8c956, 43 },
    { 0x5df8c957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2758c95c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a8c962, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf8c965, 33 },
    { 0x9ba8c965, 23 },
    { 0, 0 },
    { 0xd668c968, 68 },
    { 0xe1d8c968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f8c96d, 90 },
    { 0, 0 },
    { 0x2198c96f, 33 },
    { 0xa768c96f, 61 },
    { 0, 0 },
    { 0x6648c972, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde8c976, 98 },
    { 0x8448c977, 61 },
    { 0xd1d8c978, 61 },
    { 0xa228c977, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc818c982, 61 },
    { 0x4f78c983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c28c98e, 61 },
    { 0x3768c98f, 51 },
    { 0x89c8c98f, 61 },
    { 0x4dc8c991, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38c996, 45 },
    { 0xca18c996, 33 },
    { 0x8bb8c996, 99 },
    { 0, 0 },
    { 0xda68c99a, 76 },
    { 0, 0 },
    { 0x82a8c99c, 77 },
    { 0x5228c99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9208c9a1, 55 },
    { 0x2d28c9a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1428c9a8, 76 },
    { 0xe888c9a9, 78 },
    { 0x90e8c9a9, 61 },
    { 0x4d58c9a8, 61 },
    { 0, 0 },
    { 0xe938c9ad, 55 },
    { 0, 0 },
    { 0x5e38c9af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca38c9b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b8c9ba, 61 },
    { 0, 0 },
    { 0xc968c9bc, 76 },
    { 0xea78c9bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1948c9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d8c9cb, 33 },
    { 0x9638c9cc, 85 },
    { 0xf308c9cb, 90 },
    { 0xd508c9cd, 61 },
    { 0, 0 },
    { 0xe538c9d0, 77 },
    { 0, 0 },
    { 0x4fa8c9d2, 90 },
    { 0xda38c9d3, 80 },
    { 0, 0 },
    { 0xd898c9d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f8c9db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d8c9de, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a28c9e1, 69 },
    { 0xa88c9e1, 95 },
    { 0xfc8c9e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed58c9ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a8c9f8, 61 },
    { 0x1978c9f9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xae48c9fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x418c9ff, 61 },
    { 0, 0 },
    { 0xcf88ca01, 64 },
    { 0, 0 },
    { 0xc888ca03, 33 },
    { 0x6f28ca03, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5708ca07, 100 },
    { 0xa108ca07, 95 },
    { 0x8ab8ca09, 55 },
    { 0xb1b8ca07, 90 },
    { 0xaec8ca0a, 55 },
    { 0x9fb8ca08, 23 },
    { 0xea18ca0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b8ca10, 90 },
    { 0, 0 },
    { 0xcc48ca12, 90 },
    { 0x9168ca12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a8ca16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f8ca27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8508ca2f, 66 },
    { 0xd528ca2f, 33 },
    { 0x8ef8ca31, 23 },
    { 0xcf08ca32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3678ca39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8438ca41, 55 },
    { 0x7e68ca42, 90 },
    { 0xc9a8ca43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2678ca48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa8ca4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a68ca53, 23 },
    { 0, 0 },
    { 0xf468ca55, 90 },
    { 0xab88ca56, 66 },
    { 0xbdb8ca57, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb8ca5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbed8ca61, 90 },
    { 0x92e8ca62, 33 },
    { 0xeb88ca63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6768ca66, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d38ca6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc858ca6d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4538ca7b, 91 },
    { 0x8648ca7c, 46 },
    { 0, 0 },
    { 0xf448ca7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3038ca81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6598ca84, 98 },
    { 0, 0 },
    { 0x88f8ca86, 93 },
    { 0x6e8ca87, 98 },
    { 0xbd08ca86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a8ca8c, 55 },
    { 0, 0 },
    { 0x8328ca8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e8ca93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8ca97, 61 },
    { 0x1ed8ca98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b38ca9c, 67 },
    { 0, 0 },
    { 0xfc28ca9e, 34 },
    { 0x72b8ca9f, 23 },
    { 0xae58caa0, 34 },
    { 0xe0b8ca9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc618caa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc488caa9, 61 },
    { 0xf508caaa, 81 },
    { 0xd838caab, 90 },
    { 0xc938caac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3308caaf, 66 },
    { 0xfc78cab0, 91 },
    { 0xf3f8cab0, 95 },
    { 0xdc38cab1, 98 },
    { 0, 0 },
    { 0x1318cab4, 61 },
    { 0x7228cab5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc878caba, 61 },
    { 0, 0 },
    { 0xd1a8cabc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc58cabf, 90 },
    { 0, 0 },
    { 0xeda8cac1, 87 },
    { 0, 0 },
    { 0xf308cac3, 83 },
    { 0x5598cac4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08cac9, 90 },
    { 0, 0 },
    { 0x30e8cacb, 61 },
    { 0x30b8cacb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4978cacf, 91 },
    { 0, 0 },
    { 0x20d8cad1, 90 },
    { 0xa788cad2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c8cade, 90 },
    { 0xa7a8cade, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff78cae3, 52 },
    { 0x8b38cae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2838cae8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b8caf1, 90 },
    { 0xc168caf2, 34 },
    { 0xd1f8caf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1358caf6, 95 },
    { 0xf3c8caf6, 61 },
    { 0x2128caf8, 76 },
    { 0, 0 },
    { 0x5108cafa, 77 },
    { 0xef88cafb, 23 },
    { 0xf9c8cafc, 55 },
    { 0x5368cafb, 33 },
    { 0xcf28cafe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d8cb04, 61 },
    { 0xb718cb05, 100 },
    { 0x5988cb04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f8cb0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57d8cb1b, 61 },
    { 0xa648cb1c, 50 },
    { 0xbb08cb1c, 95 },
    { 0x3bd8cb1e, 90 },
    { 0x8d88cb1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d28cb23, 84 },
    { 0x25b8cb24, 93 },
    { 0, 0 },
    { 0x3138cb26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8cb29, 93 },
    { 0, 0 },
    { 0xb178cb2b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xae48cb2e, 52 },
    { 0x6cf8cb2f, 23 },
    { 0xd598cb30, 76 },
    { 0xc7b8cb31, 90 },
    { 0x7068cb32, 100 },
    { 0x1b8cb32, 34 },
    { 0, 0 },
    { 0xda08cb35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1428cb38, 98 },
    { 0xafc8cb38, 17 },
    { 0xf578cb39, 90 },
    { 0xa648cb3b, 33 },
    { 0x4538cb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a8cb40, 23 },
    { 0x8998cb41, 99 },
    { 0xfc88cb42, 43 },
    { 0xf838cb43, 34 },
    { 0x5418cb44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8cb48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e28cb4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe018cb4e, 90 },
    { 0x6048cb4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab88cb55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748cb5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28cb5f, 90 },
    { 0xf9c8cb60, 75 },
    { 0, 0 },
    { 0x5ac8cb62, 33 },
    { 0x8d08cb62, 33 },
    { 0xc798cb62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3028cb69, 95 },
    { 0, 0 },
    { 0x4138cb6b, 90 },
    { 0x15f8cb6c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7358cb6f, 76 },
    { 0xb4d8cb6f, 34 },
    { 0xdfc8cb71, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d58cb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9198cb7e, 76 },
    { 0x4328cb7f, 91 },
    { 0x83e8cb7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b8cb83, 85 },
    { 0xe428cb83, 33 },
    { 0xbc8cb84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf038cb8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d8cb92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df8cb98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd278cb9d, 85 },
    { 0x1268cb9e, 90 },
    { 0, 0 },
    { 0xe68cba0, 95 },
    { 0xf548cba1, 91 },
    { 0xf858cba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec8cba7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9608cbaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7788cbae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448cbb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2848cbb5, 95 },
    { 0x6088cbb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e8cbba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8cbbd, 23 },
    { 0xbc58cbbd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2878cbc1, 98 },
    { 0, 0 },
    { 0xab18cbc3, 33 },
    { 0x6cb8cbc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d18cbc8, 75 },
    { 0x9ec8cbc9, 99 },
    { 0, 0 },
    { 0x6eb8cbcb, 90 },
    { 0xc508cbcc, 98 },
    { 0x4df8cbcd, 61 },
    { 0x93a8cbcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2268cbd1, 46 },
    { 0xa918cbd1, 76 },
    { 0x6c28cbd2, 66 },
    { 0xfac8cbd3, 90 },
    { 0x33a8cbd5, 95 },
    { 0xd2e8cbd5, 99 },
    { 0x8c78cbd7, 23 },
    { 0x5bb8cbd8, 33 },
    { 0x8d68cbd1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd848cbdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4768cbe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c48cbe5, 95 },
    { 0xf748cbe6, 93 },
    { 0, 0 },
    { 0x9d68cbe8, 76 },
    { 0xc048cbe9, 23 },
    { 0, 0 },
    { 0x8d08cbeb, 23 },
    { 0xef88cbec, 55 },
    { 0xfa08cbed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db8cbf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1928cbfb, 61 },
    { 0x8138cbfc, 75 },
    { 0, 0 },
    { 0xc598cbfe, 61 },
    { 0, 0 },
    { 0xf518cc00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c8cc03, 76 },
    { 0xb498cc03, 61 },
    { 0xa88cc03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace8cc12, 76 },
    { 0, 0 },
    { 0xc9a8cc14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1918cc21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6308cc24, 23 },
    { 0, 0 },
    { 0x4278cc26, 95 },
    { 0, 0 },
    { 0x99b8cc28, 61 },
    { 0x6388cc29, 70 },
    { 0x6f68cc2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc8cc30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98cc33, 33 },
    { 0xc998cc34, 91 },
    { 0xa398cc35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf108cc3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd178cc40, 66 },
    { 0x5328cc40, 95 },
    { 0xb2d8cc42, 40 },
    { 0x2ee8cc42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8cc47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8cc4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3348cc50, 46 },
    { 0, 0 },
    { 0xe578cc52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d08cc56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c48cc5f, 33 },
    { 0x3488cc60, 95 },
    { 0, 0 },
    { 0xefe8cc62, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd8cc66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac08cc71, 34 },
    { 0, 0 },
    { 0xae58cc73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca08cc7d, 33 },
    { 0x3e28cc7e, 23 },
    { 0xfc38cc7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28cc82, 90 },
    { 0, 0 },
    { 0x65a8cc84, 93 },
    { 0xae8cc84, 33 },
    { 0, 0 },
    { 0x1af8cc87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd28cc8c, 61 },
    { 0xe408cc8d, 91 },
    { 0x7ec8cc8e, 61 },
    { 0x7068cc8e, 98 },
    { 0, 0 },
    { 0x3bc8cc91, 98 },
    { 0, 0 },
    { 0x6cb8cc93, 43 },
    { 0, 0 },
    { 0x58b8cc95, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x3698cc98, 100 },
    { 0x848cc98, 98 },
    { 0xbcd8cc9a, 98 },
    { 0x9258cc9b, 61 },
    { 0x1418cc9c, 93 },
    { 0x8538cc9d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe308cca0, 5 },
    { 0xa328cca1, 100 },
    { 0x8a68cca2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b8cca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c8cca9, 43 },
    { 0xf6a8ccaa, 69 },
    { 0, 0 },
    { 0xfb78ccac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6448ccaf, 38 },
    { 0x5c18ccaf, 95 },
    { 0xf6e8ccb1, 93 },
    { 0xd138ccb0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9828ccb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe578ccbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad48ccc0, 90 },
    { 0, 0 },
    { 0xa238ccc2, 33 },
    { 0, 0 },
    { 0x12e8ccc4, 87 },
    { 0x38b8ccc5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d68ccc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5138ccce, 72 },
    { 0, 0 },
    { 0x9b18ccd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c8ccd4, 66 },
    { 0xf5a8ccd5, 23 },
    { 0, 0 },
    { 0xbf48ccd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b98ccdb, 100 },
    { 0x2e38ccdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa68ccdf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8608cce4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf8cce7, 91 },
    { 0x7e18cce7, 76 },
    { 0x8f18cce9, 76 },
    { 0, 0 },
    { 0x5df8cceb, 98 },
    { 0x8af8ccec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a88ccf2, 84 },
    { 0x1a48ccf3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b8ccf7, 75 },
    { 0xdfd8ccf7, 98 },
    { 0, 0 },
    { 0x7b68ccfa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f8ccff, 55 },
    { 0x20c8cd00, 76 },
    { 0xed8cd00, 98 },
    { 0xf758cd01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8cd05, 66 },
    { 0x2a08cd05, 90 },
    { 0xb808cd07, 46 },
    { 0x2bd8cd08, 66 },
    { 0, 0 },
    { 0x7c88cd0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf118cd0d, 97 },
    { 0, 0 },
    { 0x6498cd0f, 91 },
    { 0x6258cd10, 90 },
    { 0x5538cd0f, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea38cd15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa888cd19, 78 },
    { 0xeeb8cd1a, 33 },
    { 0, 0 },
    { 0x21d8cd1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8718cd27, 23 },
    { 0, 0 },
    { 0x3158cd29, 23 },
    { 0xf708cd29, 55 },
    { 0xeed8cd2b, 75 },
    { 0x2098cd2c, 93 },
    { 0xc5c8cd2c, 61 },
    { 0xa928cd2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f8cd32, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc418cd36, 98 },
    { 0x7ad8cd36, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa098cd3a, 23 },
    { 0xc058cd3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a98cd44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82a8cd49, 87 },
    { 0x44a8cd4a, 34 },
    { 0, 0 },
    { 0x9218cd4c, 33 },
    { 0, 0 },
    { 0x3518cd4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a8cd53, 85 },
    { 0xa368cd53, 36 },
    { 0x8208cd54, 99 },
    { 0, 0 },
    { 0x7788cd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f8cd5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c8cd5e, 90 },
    { 0, 0 },
    { 0x7188cd60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab8cd65, 23 },
    { 0x5998cd66, 76 },
    { 0xe658cd67, 100 },
    { 0xe6d8cd65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf8cd6e, 61 },
    { 0xfe28cd6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc88cd7d, 55 },
    { 0x8578cd7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3138cd89, 90 },
    { 0, 0 },
    { 0x66d8cd8b, 90 },
    { 0x8348cd8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8548cd8f, 23 },
    { 0x4328cd90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe058cd94, 76 },
    { 0, 0 },
    { 0xd638cd96, 33 },
    { 0xc08cd97, 52 },
    { 0, 0 },
    { 0xeda8cd99, 46 },
    { 0xa7c8cd9a, 90 },
    { 0x1a28cd9b, 95 },
    { 0x688cd9b, 33 },
    { 0x70a8cd9c, 69 },
    { 0x1f18cd9e, 46 },
    { 0x67d8cd9a, 46 },
    { 0x6458cd9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9278cda3, 93 },
    { 0, 0 },
    { 0x2698cda5, 90 },
    { 0x6c88cda6, 98 },
    { 0x5848cda7, 98 },
    { 0xde88cda8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8cdac, 43 },
    { 0x5988cdad, 98 },
    { 0xdea8cdad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a8cdb5, 93 },
    { 0x5248cdb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c8cdba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a8cdbd, 95 },
    { 0x8be8cdbd, 65 },
    { 0xadb8cdbe, 77 },
    { 0xbd88cdc0, 33 },
    { 0x6698cdbe, 33 },
    { 0, 0 },
    { 0x1418cdc3, 90 },
    { 0x7cc8cdc4, 100 },
    { 0xd998cdc5, 95 },
    { 0, 0 },
    { 0x1558cdc7, 23 },
    { 0, 0 },
    { 0xbc18cdc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f88cdcd, 23 },
    { 0x82c8cdce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb8cdd2, 23 },
    { 0xe758cdd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1908cdd6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa858cdd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa8cddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6268cde5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b8cde9, 23 },
    { 0x2fc8cdea, 91 },
    { 0, 0 },
    { 0xf1a8cdec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f8cdef, 66 },
    { 0xc228cdef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d8cdf4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958cdf7, 66 },
    { 0xac88cdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6518ce00, 43 },
    { 0x5c18ce00, 33 },
    { 0x2e98ce01, 88 },
    { 0x2658ce03, 91 },
    { 0xa268ce03, 90 },
    { 0x8ff8ce05, 76 },
    { 0xf0f8ce03, 52 },
    { 0xb8ce07, 93 },
    { 0xe08ce03, 87 },
    { 0xd368ce05, 90 },
    { 0x6d38ce0a, 76 },
    { 0xd648ce0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1438ce0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe8ce11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528ce14, 66 },
    { 0, 0 },
    { 0xb848ce16, 90 },
    { 0, 0 },
    { 0x3e8ce18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf408ce1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe878ce20, 43 },
    { 0xde18ce21, 33 },
    { 0x1a78ce22, 76 },
    { 0xe0e8ce23, 90 },
    { 0x60b8ce24, 90 },
    { 0xed68ce23, 95 },
    { 0x6d98ce25, 61 },
    { 0x65d8ce24, 76 },
    { 0x2f28ce25, 76 },
    { 0x3f28ce24, 90 },
    { 0xddc8ce28, 23 },
    { 0x7cf8ce2b, 46 },
    { 0xbb78ce2c, 99 },
    { 0x4d88ce2b, 33 },
    { 0x4e88ce2e, 91 },
    { 0x3458ce2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa488ce32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb8ce36, 90 },
    { 0x3268ce36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc908ce3a, 23 },
    { 0, 0 },
    { 0x8448ce3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab88ce45, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd898ce48, 90 },
    { 0, 0 },
    { 0xdca8ce4a, 55 },
    { 0, 0 },
    { 0x94e8ce4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f08ce4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568ce53, 77 },
    { 0xf858ce54, 33 },
    { 0xf28ce55, 88 },
    { 0xc378ce56, 61 },
    { 0xf7c8ce56, 90 },
    { 0x7d18ce57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da8ce5e, 90 },
    { 0xae88ce5e, 90 },
    { 0xc878ce5f, 61 },
    { 0xb8e8ce5e, 23 },
    { 0, 0 },
    { 0x8a38ce63, 33 },
    { 0xb428ce63, 34 },
    { 0xc2b8ce65, 85 },
    { 0x8d8ce65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb8ce6a, 90 },
    { 0x7298ce6b, 46 },
    { 0x8668ce6c, 65 },
    { 0xe088ce6d, 95 },
    { 0x2898ce6e, 33 },
    { 0x4b98ce6f, 88 },
    { 0x6008ce70, 90 },
    { 0xf878ce6f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa838ce79, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7248ce83, 55 },
    { 0xe08ce84, 91 },
    { 0x89d8ce85, 75 },
    { 0x1d68ce86, 61 },
    { 0x16f8ce87, 90 },
    { 0x1f58ce88, 95 },
    { 0xbfe8ce83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4488ce93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa128ce97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2728ce9a, 36 },
    { 0xa088ce9b, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc8cea4, 55 },
    { 0, 0 },
    { 0x6c28cea6, 75 },
    { 0xddf8cea7, 78 },
    { 0, 0 },
    { 0xd08cea9, 90 },
    { 0xed8ceaa, 75 },
    { 0, 0 },
    { 0x94a8ceac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f8ceaf, 53 },
    { 0xa6e8ceb0, 99 },
    { 0x3ab8ceb1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe858cebb, 26 },
    { 0xd528cebb, 33 },
    { 0x37e8cebd, 99 },
    { 0xcf78cebe, 59 },
    { 0xceb8cebc, 33 },
    { 0x4678cec0, 55 },
    { 0x4838cebd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1448cec4, 52 },
    { 0x2448cec5, 76 },
    { 0, 0 },
    { 0x1d58cec7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb8cecb, 98 },
    { 0xbb78cecc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b8ced2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9878ced5, 90 },
    { 0xef28ced6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a8cedb, 85 },
    { 0x4498cedc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8cedf, 95 },
    { 0x5b28cee0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b8cee3, 34 },
    { 0x4a58cee4, 98 },
    { 0xea78cee4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd8ceea, 77 },
    { 0, 0 },
    { 0x4028ceec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8ceef, 85 },
    { 0x4f98cef0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8cef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1978cef9, 93 },
    { 0, 0 },
    { 0x8188cefb, 98 },
    { 0xe598cefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58cf01, 76 },
    { 0, 0 },
    { 0xbef8cf03, 90 },
    { 0x3dd8cf04, 48 },
    { 0x62d8cf04, 98 },
    { 0x178cf06, 61 },
    { 0x8988cf07, 98 },
    { 0x848cf06, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd58cf0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a8cf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d8cf16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a8cf1f, 66 },
    { 0xcb18cf1f, 61 },
    { 0x22b8cf21, 95 },
    { 0, 0 },
    { 0x8b38cf23, 61 },
    { 0xf948cf24, 34 },
    { 0, 0 },
    { 0xde98cf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a8cf29, 65 },
    { 0xf5d8cf29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f28cf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a8cf32, 61 },
    { 0, 0 },
    { 0x69c8cf34, 33 },
    { 0xe198cf35, 87 },
    { 0x4748cf36, 23 },
    { 0, 0 },
    { 0x7908cf38, 61 },
    { 0xd1c8cf39, 23 },
    { 0, 0 },
    { 0x1798cf3b, 98 },
    { 0x4038cf3c, 33 },
    { 0x1bb8cf3d, 90 },
    { 0x1fd8cf3e, 90 },
    { 0xc258cf3f, 61 },
    { 0x9908cf3f, 90 },
    { 0x9ba8cf41, 93 },
    { 0x3be8cf3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb8cf47, 66 },
    { 0, 0 },
    { 0x1128cf49, 33 },
    { 0xe4c8cf4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8268cf4f, 33 },
    { 0x26c8cf4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4888cf54, 17 },
    { 0, 0 },
    { 0xf0e8cf56, 17 },
    { 0x8fe8cf57, 55 },
    { 0xcb18cf58, 84 },
    { 0xaa98cf58, 33 },
    { 0x5b78cf57, 98 },
    { 0x8d28cf57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5888cf5e, 55 },
    { 0, 0 },
    { 0x7228cf60, 23 },
    { 0x6c38cf61, 33 },
    { 0x4688cf60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d8cf65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e08cf6a, 98 },
    { 0, 0 },
    { 0x4778cf6c, 90 },
    { 0x3578cf6d, 98 },
    { 0x4328cf6d, 52 },
    { 0x478cf6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb728cf79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f48cf7d, 55 },
    { 0xa158cf7e, 67 },
    { 0x9448cf7f, 98 },
    { 0xa2a8cf7f, 99 },
    { 0x7a88cf7f, 93 },
    { 0, 0 },
    { 0xa918cf83, 87 },
    { 0, 0 },
    { 0xe9b8cf85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f78cf88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe38cf8b, 34 },
    { 0x34b8cf8c, 98 },
    { 0xb0a8cf8d, 76 },
    { 0xba28cf8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a8cf91, 33 },
    { 0, 0 },
    { 0xca98cf93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf638cf96, 61 },
    { 0xab18cf97, 43 },
    { 0, 0 },
    { 0xfc78cf99, 99 },
    { 0x4028cf99, 40 },
    { 0x35e8cf99, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f8cfa0, 85 },
    { 0x5988cfa0, 23 },
    { 0x7708cfa1, 38 },
    { 0xbf38cfa3, 98 },
    { 0x79d8cfa4, 61 },
    { 0xd8c8cfa4, 33 },
    { 0x888cfa4, 90 },
    { 0xfc48cfa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b8cfac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed08cfb2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4018cfb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d78cfbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e8cfc1, 33 },
    { 0x4a58cfc1, 98 },
    { 0xdda8cfc3, 23 },
    { 0x7018cfc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff18cfcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88cfd0, 23 },
    { 0xe108cfd0, 34 },
    { 0x8598cfd2, 98 },
    { 0xe818cfd3, 33 },
    { 0, 0 },
    { 0xfb8cfd5, 34 },
    { 0x5f58cfd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6428cfd9, 33 },
    { 0x1ff8cfda, 90 },
    { 0x9bf8cfdb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef8cfdf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a8cfe2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e8cfe5, 64 },
    { 0x5b38cfe6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa8cfea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e78cff1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8cff4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38cff7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b8d000, 34 },
    { 0x9f28d000, 33 },
    { 0, 0 },
    { 0xd4b8d003, 61 },
    { 0xf278d004, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4258d00b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48d00e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398d012, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d019, 55 },
    { 0x8118d01a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f98d01d, 78 },
    { 0xd08d01e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2308d021, 61 },
    { 0, 0 },
    { 0x5718d023, 97 },
    { 0x7118d023, 90 },
    { 0xc568d023, 61 },
    { 0x2e88d023, 66 },
    { 0x13a8d027, 43 },
    { 0xacd8d027, 33 },
    { 0xb208d028, 95 },
    { 0x72a8d027, 33 },
    { 0, 0 },
    { 0x4f88d02c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358d032, 23 },
    { 0xea18d033, 66 },
    { 0, 0 },
    { 0x47f8d035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e48d039, 87 },
    { 0, 0 },
    { 0xb728d03b, 17 },
    { 0x4e68d03b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c8d046, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a88d04d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8678d054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda48d058, 33 },
    { 0x6728d058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8d05c, 90 },
    { 0xa4b8d05c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d8d063, 95 },
    { 0, 0 },
    { 0x34b8d065, 90 },
    { 0xbbb8d066, 93 },
    { 0x9008d067, 84 },
    { 0x2f38d068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b8d06d, 33 },
    { 0xa728d06e, 84 },
    { 0, 0 },
    { 0xa158d070, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398d076, 95 },
    { 0x5368d077, 61 },
    { 0x26f8d078, 34 },
    { 0xa1a8d077, 37 },
    { 0, 0 },
    { 0x84c8d07b, 55 },
    { 0, 0 },
    { 0x8718d07d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc328d083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a8d086, 33 },
    { 0, 0 },
    { 0x2c58d088, 23 },
    { 0, 0 },
    { 0xd638d08a, 88 },
    { 0xb8b8d08a, 99 },
    { 0x1428d08a, 61 },
    { 0xd4d8d08d, 55 },
    { 0xa6b8d08e, 55 },
    { 0x34f8d08f, 46 },
    { 0xcc68d08f, 76 },
    { 0x5dd8d08a, 90 },
    { 0xd618d08d, 36 },
    { 0x5ce8d08a, 95 },
    { 0xa538d092, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c8d098, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48d09f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad8d0a7, 90 },
    { 0, 0 },
    { 0xb6f8d0a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae8d0b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398d0c0, 76 },
    { 0x20f8d0c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e8d0c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf868d0c8, 33 },
    { 0xb328d0c9, 61 },
    { 0x4198d0ca, 75 },
    { 0, 0 },
    { 0xa138d0cc, 61 },
    { 0xaa28d0cc, 90 },
    { 0, 0 },
    { 0xbb68d0cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda78d0d2, 95 },
    { 0, 0 },
    { 0xc608d0d4, 98 },
    { 0x558d0d5, 55 },
    { 0xe898d0d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8d0db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb818d0df, 43 },
    { 0x7428d0e0, 93 },
    { 0xab38d0e1, 98 },
    { 0, 0 },
    { 0xf928d0e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d8d0eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b8d0f0, 33 },
    { 0, 0 },
    { 0x1138d0f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb228d0fa, 55 },
    { 0, 0 },
    { 0xe098d0fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c78d102, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3958d105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88d108, 90 },
    { 0xd508d109, 78 },
    { 0, 0 },
    { 0x23b8d10b, 92 },
    { 0x1288d10c, 33 },
    { 0, 0 },
    { 0x82e8d10e, 98 },
    { 0, 0 },
    { 0x63b8d110, 23 },
    { 0x2338d111, 77 },
    { 0x8338d112, 33 },
    { 0x5578d113, 17 },
    { 0xd978d112, 95 },
    { 0, 0 },
    { 0x4df8d116, 85 },
    { 0x6e58d117, 33 },
    { 0xb078d118, 100 },
    { 0xa18d119, 90 },
    { 0, 0 },
    { 0xd868d11b, 87 },
    { 0, 0 },
    { 0xe248d11d, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6758d121, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718d128, 65 },
    { 0xdd88d129, 90 },
    { 0x3358d129, 95 },
    { 0x51f8d12a, 90 },
    { 0x1f18d12c, 95 },
    { 0x1408d129, 51 },
    { 0x47e8d12e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a58d133, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa638d143, 90 },
    { 0x1ef8d143, 98 },
    { 0xa5c8d145, 95 },
    { 0xaf48d146, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb8d14c, 80 },
    { 0xf098d14d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a8d152, 99 },
    { 0x4fa8d153, 78 },
    { 0x878d153, 90 },
    { 0x5528d155, 76 },
    { 0, 0 },
    { 0x6e18d157, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9318d15c, 90 },
    { 0xe038d15d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf8d164, 90 },
    { 0, 0 },
    { 0x40e8d166, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc358d16b, 61 },
    { 0x8808d16b, 55 },
    { 0x1838d16b, 90 },
    { 0x6fd8d16b, 43 },
    { 0x1788d16c, 53 },
    { 0, 0 },
    { 0xac18d171, 33 },
    { 0xccd8d172, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc168d177, 76 },
    { 0xc2a8d178, 99 },
    { 0x4c98d179, 46 },
    { 0, 0 },
    { 0x2bb8d17b, 90 },
    { 0x9328d17b, 40 },
    { 0x5188d17d, 90 },
    { 0xcfe8d17e, 90 },
    { 0x428d17b, 90 },
    { 0x57f8d180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8d18b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928d18e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f18d193, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8d19a, 93 },
    { 0, 0 },
    { 0xf388d19c, 46 },
    { 0x63a8d19d, 43 },
    { 0x9228d19d, 84 },
    { 0, 0 },
    { 0x45e8d1a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9228d1a7, 91 },
    { 0x4e88d1a7, 23 },
    { 0, 0 },
    { 0xfb18d1aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab38d1ad, 33 },
    { 0, 0 },
    { 0xa588d1af, 64 },
    { 0x5a38d1b0, 34 },
    { 0, 0 },
    { 0xb768d1b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe808d1b6, 95 },
    { 0x7818d1b6, 95 },
    { 0xe4d8d1b8, 40 },
    { 0x2778d1b9, 43 },
    { 0xbd88d1b9, 23 },
    { 0x3398d1ba, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6418d1be, 76 },
    { 0, 0 },
    { 0xd518d1c0, 34 },
    { 0xfb68d1c1, 61 },
    { 0x7ba8d1c2, 91 },
    { 0x7708d1c3, 67 },
    { 0xa498d1c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9708d1c7, 95 },
    { 0x7ad8d1c8, 95 },
    { 0x6298d1c9, 98 },
    { 0x7b48d1ca, 33 },
    { 0x9688d1c9, 66 },
    { 0xd688d1cc, 67 },
    { 0x8248d1c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8238d1d0, 23 },
    { 0, 0 },
    { 0xe5b8d1d2, 61 },
    { 0x2358d1d2, 90 },
    { 0x1098d1d4, 94 },
    { 0x73c8d1d5, 94 },
    { 0x62c8d1d6, 33 },
    { 0xbd28d1d2, 17 },
    { 0, 0 },
    { 0xdab8d1d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf598d1dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b8d1e3, 95 },
    { 0x99a8d1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1788d1ea, 91 },
    { 0x6e08d1ea, 98 },
    { 0xce8d1ec, 87 },
    { 0x10c8d1ec, 95 },
    { 0xea38d1ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c48d1f2, 100 },
    { 0x8858d1f2, 90 },
    { 0x7b78d1f4, 33 },
    { 0x9cd8d1f4, 80 },
    { 0xf338d1f5, 68 },
    { 0, 0 },
    { 0xd178d1f8, 55 },
    { 0, 0 },
    { 0xe3a8d1fa, 33 },
    { 0x9918d1fb, 17 },
    { 0, 0 },
    { 0xccf8d1fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda68d204, 46 },
    { 0x23b8d204, 82 },
    { 0x7e48d206, 23 },
    { 0xc4d8d204, 95 },
    { 0, 0 },
    { 0xf8f8d209, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc688d20c, 23 },
    { 0xd468d20d, 33 },
    { 0, 0 },
    { 0xfa78d20f, 66 },
    { 0x4c48d210, 43 },
    { 0, 0 },
    { 0xb2e8d212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4208d215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e8d218, 88 },
    { 0x1988d218, 91 },
    { 0x8d28d21a, 33 },
    { 0x28d8d21b, 43 },
    { 0x5708d21b, 98 },
    { 0xc268d219, 34 },
    { 0x188d21e, 17 },
    { 0, 0 },
    { 0x3ab8d220, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e8d229, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88d233, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d8d236, 46 },
    { 0, 0 },
    { 0x3a8d238, 76 },
    { 0x7558d239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b8d23c, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a8d242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea8d24f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3598d257, 61 },
    { 0, 0 },
    { 0xdba8d259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f88d25d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a8d266, 72 },
    { 0, 0 },
    { 0x8c48d268, 61 },
    { 0x38c8d269, 76 },
    { 0xcf98d268, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefb8d26e, 55 },
    { 0x48e8d26e, 90 },
    { 0xf458d270, 95 },
    { 0, 0 },
    { 0x59a8d272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e88d278, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3948d27d, 88 },
    { 0, 0 },
    { 0xe58d27f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b08d286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac68d289, 90 },
    { 0x7198d289, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb68d28d, 34 },
    { 0xa6c8d28e, 97 },
    { 0x98a8d28e, 23 },
    { 0xf7a8d290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7378d294, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d48d29f, 76 },
    { 0x7a38d2a0, 51 },
    { 0x5c8d2a1, 78 },
    { 0, 0 },
    { 0x8ed8d2a3, 55 },
    { 0xb398d2a4, 76 },
    { 0x298d2a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6368d2a8, 88 },
    { 0, 0 },
    { 0x7f78d2aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d8d2ae, 43 },
    { 0xdd48d2af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1398d2b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a8d2bd, 70 },
    { 0x4f28d2be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d8d2c3, 23 },
    { 0, 0 },
    { 0xcad8d2c5, 52 },
    { 0x7fc8d2c6, 55 },
    { 0, 0 },
    { 0x20c8d2c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8438d2cf, 76 },
    { 0, 0 },
    { 0x4b88d2d1, 43 },
    { 0x23d8d2d2, 90 },
    { 0x5608d2d1, 43 },
    { 0x4118d2d3, 23 },
    { 0, 0 },
    { 0x2e98d2d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d8d2d9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3808d2dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e78d2df, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4408d2e3, 90 },
    { 0xd588d2e3, 67 },
    { 0x9f48d2e5, 90 },
    { 0x3d28d2e6, 90 },
    { 0x9528d2e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a08d2ed, 95 },
    { 0x9c88d2ee, 33 },
    { 0xeff8d2ef, 98 },
    { 0x2718d2f0, 33 },
    { 0xa58d2ee, 61 },
    { 0, 0 },
    { 0x3258d2f3, 34 },
    { 0x1d28d2f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c18d2f8, 90 },
    { 0x298d2f9, 65 },
    { 0x57d8d2fa, 87 },
    { 0x7248d2fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc8d302, 23 },
    { 0xd198d303, 90 },
    { 0xfac8d303, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8d309, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5818d30d, 99 },
    { 0xa6d8d30d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f78d312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe338d317, 43 },
    { 0x9998d318, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af8d31d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8d321, 33 },
    { 0xaa68d321, 23 },
    { 0x1758d323, 90 },
    { 0xdf78d324, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf338d330, 97 },
    { 0x2898d330, 17 },
    { 0x2418d332, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c98d335, 87 },
    { 0x2a48d336, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8788d33b, 65 },
    { 0, 0 },
    { 0x2e8d33d, 68 },
    { 0, 0 },
    { 0x7c38d33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd88d342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6478d345, 55 },
    { 0, 0 },
    { 0xcf18d347, 95 },
    { 0x8178d347, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c8d34b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x2658d34e, 95 },
    { 0x5158d34e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a8d353, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf768d357, 33 },
    { 0x6c58d358, 95 },
    { 0, 0 },
    { 0x8078d35a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca8d36c, 23 },
    { 0xb138d36d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf8d370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf28d373, 95 },
    { 0x5478d374, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2898d37a, 100 },
    { 0xa438d37b, 78 },
    { 0x2868d37c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2628d37f, 61 },
    { 0x3818d380, 95 },
    { 0xfd18d37f, 61 },
    { 0xf3c8d382, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3648d38a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc428d38f, 33 },
    { 0, 0 },
    { 0xfc08d391, 78 },
    { 0x7f88d391, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608d398, 90 },
    { 0x5638d399, 87 },
    { 0xbe98d398, 78 },
    { 0, 0 },
    { 0x44c8d39c, 95 },
    { 0, 0 },
    { 0x37f8d39e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8018d3a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e8d3a8, 93 },
    { 0, 0 },
    { 0xdc08d3aa, 33 },
    { 0xbb68d3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a8d3af, 90 },
    { 0, 0 },
    { 0x1398d3b1, 23 },
    { 0xde48d3b1, 95 },
    { 0xa798d3b3, 77 },
    { 0xc08d3b3, 90 },
    { 0x8f38d3b5, 99 },
    { 0xdfa8d3b6, 90 },
    { 0xcc98d3b7, 34 },
    { 0x4bf8d3b5, 90 },
    { 0xd178d3b9, 90 },
    { 0x5d38d3b3, 61 },
    { 0x53c8d3bb, 100 },
    { 0xa438d3bb, 90 },
    { 0x14a8d3bb, 33 },
    { 0x4c18d3bd, 90 },
    { 0x1bd8d3bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2788d3c4, 81 },
    { 0x61e8d3c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe868d3c8, 68 },
    { 0, 0 },
    { 0x7618d3ca, 84 },
    { 0x3338d3cb, 66 },
    { 0xe928d3cc, 76 },
    { 0xcc78d3cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb148d3d3, 99 },
    { 0, 0 },
    { 0x5958d3d5, 43 },
    { 0, 0 },
    { 0xdda8d3d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7458d3e3, 90 },
    { 0xd558d3e4, 90 },
    { 0x77e8d3e4, 87 },
    { 0xec58d3e4, 55 },
    { 0x2ef8d3e7, 83 },
    { 0x62d8d3e6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f8d3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c8d3f6, 33 },
    { 0xa178d3f7, 99 },
    { 0xf718d3f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa808d401, 33 },
    { 0x7c28d402, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608d406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b8d40d, 33 },
    { 0, 0 },
    { 0xa3d8d40f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7468d415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa948d418, 78 },
    { 0x36c8d418, 55 },
    { 0x4878d419, 33 },
    { 0xbfd8d418, 100 },
    { 0, 0 },
    { 0xbb8d41d, 33 },
    { 0x3b98d41e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8d427, 76 },
    { 0xe7c8d428, 23 },
    { 0x1878d429, 90 },
    { 0x32c8d429, 61 },
    { 0x6668d42b, 33 },
    { 0x1ff8d429, 95 },
    { 0x82e8d427, 95 },
    { 0xfe68d42a, 34 },
    { 0xaa78d42f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d432, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf8d435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78d438, 52 },
    { 0, 0 },
    { 0x1978d43a, 76 },
    { 0xff88d43b, 82 },
    { 0xecc8d43c, 90 },
    { 0x27c8d43c, 90 },
    { 0x3f8d43a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a8d444, 98 },
    { 0x5168d445, 55 },
    { 0, 0 },
    { 0x2388d447, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a8d44b, 98 },
    { 0x91f8d44c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3948d451, 90 },
    { 0xb98d452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb108d455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2968d459, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8d45f, 90 },
    { 0xef08d460, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6358d463, 90 },
    { 0x50a8d463, 90 },
    { 0xda38d465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5418d469, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec38d46d, 33 },
    { 0x1098d46d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8588d471, 33 },
    { 0x7878d471, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68d47d, 33 },
    { 0xd5b8d47d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd8d487, 90 },
    { 0xf6b8d488, 76 },
    { 0x388d489, 53 },
    { 0, 0 },
    { 0xe168d48b, 46 },
    { 0, 0 },
    { 0x1828d48d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1758d491, 75 },
    { 0, 0 },
    { 0x9948d493, 55 },
    { 0x4798d494, 90 },
    { 0x5048d495, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b28d499, 61 },
    { 0xf088d499, 90 },
    { 0xe458d49b, 78 },
    { 0, 0 },
    { 0x96a8d49d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a8d4a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4768d4a7, 23 },
    { 0xb838d4a8, 53 },
    { 0, 0 },
    { 0xe2f8d4aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98d4ad, 68 },
    { 0xd1d8d4ae, 23 },
    { 0x398d4ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e8d4b2, 81 },
    { 0xa6a8d4b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8528d4b7, 55 },
    { 0x9fd8d4b8, 90 },
    { 0xcc98d4b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf888d4c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade8d4c7, 87 },
    { 0xb08d4c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3768d4cc, 76 },
    { 0xb508d4cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e8d4d3, 61 },
    { 0x7198d4d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca8d4d8, 33 },
    { 0, 0 },
    { 0xb398d4da, 90 },
    { 0x94e8d4db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5358d4e6, 49 },
    { 0xa3f8d4e7, 61 },
    { 0xd8a8d4e6, 76 },
    { 0x7488d4e9, 33 },
    { 0xe738d4e6, 93 },
    { 0x5b58d4e6, 55 },
    { 0xdf88d4ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd288d4f0, 33 },
    { 0x59c8d4f1, 97 },
    { 0xf908d4f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588d4f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8758d4f8, 93 },
    { 0xa928d4f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6468d4fc, 43 },
    { 0x5208d4fd, 52 },
    { 0, 0 },
    { 0x5368d4ff, 93 },
    { 0, 0 },
    { 0x73d8d501, 98 },
    { 0xa528d502, 100 },
    { 0x41c8d502, 43 },
    { 0x5a58d503, 95 },
    { 0, 0 },
    { 0x1898d506, 66 },
    { 0, 0 },
    { 0x3918d508, 47 },
    { 0x5948d509, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f8d50c, 95 },
    { 0xd778d50d, 85 },
    { 0, 0 },
    { 0xcfb8d50f, 52 },
    { 0x31b8d510, 84 },
    { 0x7148d511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa8d515, 91 },
    { 0x1798d515, 90 },
    { 0xa8d515, 98 },
    { 0, 0 },
    { 0x9cf8d519, 77 },
    { 0, 0 },
    { 0x3a18d51b, 17 },
    { 0x1e28d51c, 98 },
    { 0x14d8d51b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb98d521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a8d527, 81 },
    { 0, 0 },
    { 0xbb68d529, 98 },
    { 0x7968d529, 33 },
    { 0xe738d52a, 90 },
    { 0, 0 },
    { 0xf718d52d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f08d532, 95 },
    { 0xca88d533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb658d53b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48d540, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe408d543, 76 },
    { 0x1888d543, 87 },
    { 0x7338d544, 34 },
    { 0x5c68d543, 61 },
    { 0xab58d547, 76 },
    { 0x8848d548, 98 },
    { 0x95b8d545, 33 },
    { 0xb338d54a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8908d551, 75 },
    { 0x9f58d552, 95 },
    { 0xc2a8d553, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d558, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf988d55c, 87 },
    { 0x4818d55d, 95 },
    { 0x5708d55e, 93 },
    { 0x5368d55f, 70 },
    { 0x9168d560, 61 },
    { 0x5e08d561, 23 },
    { 0xb008d562, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe8d567, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a8d56b, 66 },
    { 0xc3a8d56c, 100 },
    { 0xca98d56d, 61 },
    { 0, 0 },
    { 0x5fd8d56f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d577, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c8d57a, 34 },
    { 0x7b8d57b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b88d580, 61 },
    { 0, 0 },
    { 0x5ad8d582, 66 },
    { 0xd3c8d583, 95 },
    { 0x4e08d584, 48 },
    { 0xb778d585, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf48d58b, 33 },
    { 0xcce8d58c, 61 },
    { 0xefc8d58b, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b18d591, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba88d596, 33 },
    { 0, 0 },
    { 0x9c8d598, 33 },
    { 0x6188d599, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc28d59c, 23 },
    { 0xb538d59d, 90 },
    { 0x9e08d59e, 61 },
    { 0, 0 },
    { 0xf908d5a0, 55 },
    { 0x80b8d5a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d5a6, 90 },
    { 0x91b8d5a7, 61 },
    { 0, 0 },
    { 0x99a8d5a9, 90 },
    { 0, 0 },
    { 0x8c88d5ab, 55 },
    { 0, 0 },
    { 0x6be8d5ad, 78 },
    { 0x7cc8d5ad, 68 },
    { 0, 0 },
    { 0xdb8d5b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b18d5b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6458d5b8, 100 },
    { 0x4848d5b9, 77 },
    { 0x4328d5ba, 23 },
    { 0, 0 },
    { 0xbd18d5bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa328d5c0, 95 },
    { 0, 0 },
    { 0xca08d5c2, 100 },
    { 0, 0 },
    { 0x99e8d5c4, 34 },
    { 0x208d5c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa698d5c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb908d5cd, 43 },
    { 0xaae8d5ce, 76 },
    { 0xacd8d5cf, 90 },
    { 0x3e28d5cd, 76 },
    { 0xb858d5d1, 61 },
    { 0x3698d5d2, 66 },
    { 0x7af8d5d3, 40 },
    { 0xad78d5d1, 99 },
    { 0xfdb8d5d5, 61 },
    { 0x33f8d5d6, 80 },
    { 0x2ab8d5d7, 40 },
    { 0x6938d5d7, 90 },
    { 0xfc8d5d9, 52 },
    { 0xb978d5d9, 95 },
    { 0xc9a8d5db, 33 },
    { 0x4078d5ce, 90 },
    { 0x9b68d5d2, 90 },
    { 0x37e8d5de, 33 },
    { 0x26c8d5de, 98 },
    { 0x38a8d5de, 98 },
    { 0xf768d5e0, 90 },
    { 0xaf08d5e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe8d5e6, 78 },
    { 0xa1d8d5e7, 33 },
    { 0xa9c8d5e7, 55 },
    { 0xb1f8d5e9, 90 },
    { 0xa198d5ea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c28d5f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd8d5f7, 20 },
    { 0x5a68d5f7, 76 },
    { 0xccc8d5f9, 23 },
    { 0x3678d5f8, 93 },
    { 0x84e8d5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a18d605, 34 },
    { 0, 0 },
    { 0xb6d8d607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa8d60a, 55 },
    { 0x9f18d60a, 34 },
    { 0x65f8d60a, 93 },
    { 0xba58d60b, 93 },
    { 0, 0 },
    { 0x1f78d60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f88d616, 33 },
    { 0, 0 },
    { 0xdc18d618, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc8d61b, 46 },
    { 0x11e8d61b, 97 },
    { 0x4988d61d, 78 },
    { 0x70a8d61e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd708d623, 90 },
    { 0x31d8d624, 95 },
    { 0x3b88d625, 95 },
    { 0x78d624, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28d62a, 33 },
    { 0x95d8d62a, 66 },
    { 0x3fd8d62c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e8d62f, 90 },
    { 0x6418d630, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b78d635, 43 },
    { 0x4e98d636, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd8d639, 76 },
    { 0xaa8d639, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158d63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb98d641, 100 },
    { 0x7dd8d641, 87 },
    { 0xf218d642, 76 },
    { 0xa948d643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4468d647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde38d64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3938d64e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58d657, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba8d65a, 34 },
    { 0x4a58d65b, 55 },
    { 0x8ff8d65b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8d65f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b8d662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b8d665, 66 },
    { 0, 0 },
    { 0xb498d667, 91 },
    { 0x5308d667, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3058d66c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd728d679, 61 },
    { 0x3f88d67a, 90 },
    { 0, 0 },
    { 0xb188d67c, 33 },
    { 0, 0 },
    { 0x78d8d67e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa008d684, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c8d690, 23 },
    { 0x9888d691, 76 },
    { 0, 0 },
    { 0x7c88d693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8d696, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf078d69b, 97 },
    { 0xe5e8d69c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf18d69f, 61 },
    { 0x2188d6a0, 61 },
    { 0x95d8d6a1, 23 },
    { 0, 0 },
    { 0x1dd8d6a3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b8d6a6, 55 },
    { 0xb618d6a7, 93 },
    { 0, 0 },
    { 0xbb28d6a9, 98 },
    { 0xe908d6a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d28d6ad, 90 },
    { 0, 0 },
    { 0x3588d6af, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b8d6b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48d6c0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd708d6c6, 90 },
    { 0x8d08d6c7, 90 },
    { 0xb878d6c6, 90 },
    { 0, 0 },
    { 0xbe58d6ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb998d6cf, 78 },
    { 0x5488d6d0, 76 },
    { 0x48b8d6d0, 100 },
    { 0x1a58d6d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ae8d6d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d48d6d9, 34 },
    { 0xcbb8d6d9, 33 },
    { 0, 0 },
    { 0x118d6dc, 85 },
    { 0x8608d6dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4398d6e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c8d6e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb558d6e6, 76 },
    { 0, 0 },
    { 0x7128d6e8, 90 },
    { 0x6548d6e9, 77 },
    { 0xa308d6e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa8d6ef, 90 },
    { 0, 0 },
    { 0xcd78d6f1, 23 },
    { 0xd248d6f2, 23 },
    { 0, 0 },
    { 0xa1a8d6f4, 93 },
    { 0xfca8d6f5, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab8d6f8, 33 },
    { 0x3bb8d6f8, 81 },
    { 0xefc8d6fa, 81 },
    { 0x6c08d6f8, 23 },
    { 0x3508d6fc, 95 },
    { 0xcbb8d6fd, 98 },
    { 0x6e88d6fc, 40 },
    { 0xf7a8d6ff, 95 },
    { 0x8368d700, 23 },
    { 0x9928d700, 61 },
    { 0, 0 },
    { 0x3988d703, 76 },
    { 0x7668d703, 90 },
    { 0, 0 },
    { 0xa3d8d706, 95 },
    { 0xa348d707, 98 },
    { 0x4d48d708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa808d710, 98 },
    { 0xa6b8d710, 34 },
    { 0x94d8d712, 34 },
    { 0x4578d713, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb58d71c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb768d71f, 33 },
    { 0, 0 },
    { 0x18f8d721, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d727, 61 },
    { 0xadc8d728, 40 },
    { 0xc708d729, 66 },
    { 0x7b38d72a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf98d72d, 84 },
    { 0x2578d72e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa268d731, 23 },
    { 0, 0 },
    { 0xedb8d733, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc728d737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc8d73e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8d742, 55 },
    { 0xfcf8d743, 93 },
    { 0, 0 },
    { 0x5458d745, 33 },
    { 0xc908d746, 23 },
    { 0xc1f8d746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf428d755, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3388d75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc18d761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6498d765, 43 },
    { 0x87a8d765, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe8d76a, 43 },
    { 0x46b8d76b, 99 },
    { 0x5e38d76a, 95 },
    { 0, 0 },
    { 0x12d8d76e, 55 },
    { 0, 0 },
    { 0xdfa8d770, 61 },
    { 0xb9d8d771, 76 },
    { 0x7a8d772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e8d776, 52 },
    { 0x18c8d777, 91 },
    { 0x50e8d778, 98 },
    { 0, 0 },
    { 0xe6e8d77a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b8d77f, 99 },
    { 0x5408d780, 76 },
    { 0x8d68d780, 90 },
    { 0xe508d782, 93 },
    { 0x56e8d783, 95 },
    { 0xd6b8d781, 23 },
    { 0x4108d785, 98 },
    { 0x4c98d786, 95 },
    { 0xfea8d783, 94 },
    { 0xe558d788, 61 },
    { 0x6b18d787, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb18d799, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638d7a4, 85 },
    { 0, 0 },
    { 0x1668d7a6, 96 },
    { 0xc6e8d7a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b8d7ad, 55 },
    { 0xa618d7ad, 66 },
    { 0xd838d7af, 34 },
    { 0xb278d7b0, 66 },
    { 0x6ba8d7b1, 66 },
    { 0xa5d8d7b2, 90 },
    { 0, 0 },
    { 0xf408d7b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c28d7b9, 33 },
    { 0, 0 },
    { 0xa2d8d7bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe048d7bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba8d7c5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac98d7c9, 33 },
    { 0x7748d7ca, 52 },
    { 0, 0 },
    { 0xaab8d7cc, 90 },
    { 0x7a58d7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f28d7d1, 91 },
    { 0xccb8d7d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec8d7d5, 90 },
    { 0x72a8d7d6, 33 },
    { 0xc2a8d7d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa28d7dc, 52 },
    { 0, 0 },
    { 0x45f8d7de, 76 },
    { 0xc858d7df, 43 },
    { 0x2ca8d7df, 23 },
    { 0, 0 },
    { 0x1068d7e2, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf58d7e6, 95 },
    { 0x9088d7e6, 43 },
    { 0, 0 },
    { 0xbc28d7e9, 98 },
    { 0x7428d7e9, 98 },
    { 0xe7b8d7e9, 95 },
    { 0xd5c8d7ec, 23 },
    { 0x8a58d7ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8248d7f1, 90 },
    { 0xef68d7f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e8d7f7, 100 },
    { 0x7b28d7f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef8d7fb, 23 },
    { 0x5168d7fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe318d802, 90 },
    { 0xbfc8d803, 61 },
    { 0, 0 },
    { 0x3ec8d805, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c8d809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8d80d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d8d813, 52 },
    { 0x9f68d814, 84 },
    { 0x2fe8d814, 46 },
    { 0x5768d815, 95 },
    { 0x12a8d817, 98 },
    { 0xabb8d813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1118d81e, 99 },
    { 0xe628d81e, 55 },
    { 0xca68d820, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f8d824, 93 },
    { 0x9d18d824, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e8d830, 46 },
    { 0xc8b8d831, 98 },
    { 0xa8f8d832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df8d83c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6808d83f, 95 },
    { 0x9d68d840, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8998d843, 99 },
    { 0x1a18d843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a8d847, 98 },
    { 0, 0 },
    { 0xf518d849, 95 },
    { 0x1058d84a, 66 },
    { 0x1318d849, 61 },
    { 0, 0 },
    { 0x6158d84d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5108d850, 85 },
    { 0xd78d851, 78 },
    { 0, 0 },
    { 0xb8f8d853, 67 },
    { 0x3f08d853, 95 },
    { 0, 0 },
    { 0xe7e8d856, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48d85b, 92 },
    { 0xf538d85b, 43 },
    { 0, 0 },
    { 0xf378d85e, 88 },
    { 0, 0 },
    { 0x6898d860, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda18d863, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f8d866, 46 },
    { 0xf848d867, 85 },
    { 0, 0 },
    { 0x6bb8d869, 90 },
    { 0, 0 },
    { 0xc068d86b, 85 },
    { 0x8368d86c, 61 },
    { 0x4248d86d, 90 },
    { 0, 0 },
    { 0xe828d86f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8d874, 90 },
    { 0x8c18d875, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b8d878, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5018d87c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d18d880, 33 },
    { 0x348d881, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf8d884, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c08d889, 95 },
    { 0xc08d889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e38d88d, 43 },
    { 0xe8c8d88e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf918d893, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2348d897, 84 },
    { 0, 0 },
    { 0xd1b8d899, 76 },
    { 0x6148d89a, 33 },
    { 0xd1a8d89b, 90 },
    { 0xa628d89c, 100 },
    { 0x1d88d89b, 33 },
    { 0x19c8d89d, 34 },
    { 0x6858d89e, 98 },
    { 0x5208d8a0, 100 },
    { 0x5ad8d89e, 100 },
    { 0x5d58d89b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe398d8a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1808d8ac, 87 },
    { 0x37b8d8ad, 78 },
    { 0x7938d8ad, 33 },
    { 0, 0 },
    { 0x4038d8b0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd858d8b5, 90 },
    { 0xfaa8d8b5, 90 },
    { 0, 0 },
    { 0xbb98d8b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da8d8bc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd8d8bf, 76 },
    { 0x8638d8c0, 98 },
    { 0x4ed8d8c1, 43 },
    { 0, 0 },
    { 0xe3c8d8c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968d8c7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x868d8ca, 78 },
    { 0, 0 },
    { 0x4928d8cc, 33 },
    { 0x1ea8d8cd, 91 },
    { 0xf608d8cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed68d8d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb868d8db, 90 },
    { 0x4548d8dc, 75 },
    { 0, 0 },
    { 0xa738d8de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9808d8e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9238d8ec, 43 },
    { 0, 0 },
    { 0x39f8d8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb008d8f1, 34 },
    { 0xa8a8d8f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2288d8f6, 87 },
    { 0x5958d8f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88d8fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a8d8fe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d38d907, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d58d90e, 99 },
    { 0xc928d90f, 33 },
    { 0x44f8d910, 90 },
    { 0, 0 },
    { 0x7a58d912, 95 },
    { 0x5b58d913, 95 },
    { 0, 0 },
    { 0x8da8d915, 76 },
    { 0x7988d916, 91 },
    { 0x6388d917, 55 },
    { 0, 0 },
    { 0x2d98d919, 98 },
    { 0x9098d91a, 55 },
    { 0x8ab8d91b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528d923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b78d926, 33 },
    { 0x7538d927, 76 },
    { 0xa958d928, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d92e, 94 },
    { 0x2098d92f, 33 },
    { 0x218d92e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa678d933, 23 },
    { 0xb518d934, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b8d939, 98 },
    { 0xc878d93a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9338d93e, 43 },
    { 0xe538d93f, 35 },
    { 0x72a8d93f, 76 },
    { 0x9768d941, 34 },
    { 0x33f8d93e, 23 },
    { 0, 0 },
    { 0x7b68d944, 23 },
    { 0xec08d945, 77 },
    { 0, 0 },
    { 0x1e08d947, 87 },
    { 0, 0 },
    { 0x8db8d949, 61 },
    { 0x76b8d94a, 98 },
    { 0, 0 },
    { 0xdb38d94c, 43 },
    { 0x3b38d94c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b8d950, 61 },
    { 0xcc38d951, 98 },
    { 0x6028d952, 43 },
    { 0x6a28d950, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8948d957, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f8d95b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed8d95e, 98 },
    { 0xd858d95f, 95 },
    { 0x5528d95f, 95 },
    { 0xf268d961, 98 },
    { 0xcc88d961, 23 },
    { 0x9c38d962, 34 },
    { 0x4828d963, 90 },
    { 0x3918d962, 75 },
    { 0xf2b8d966, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6408d96b, 90 },
    { 0x40f8d96c, 61 },
    { 0x2178d96d, 90 },
    { 0x3ec8d96e, 76 },
    { 0x6f68d96d, 87 },
    { 0, 0 },
    { 0x9e28d971, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8818d974, 95 },
    { 0xbb48d975, 85 },
    { 0x7978d976, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a8d97b, 98 },
    { 0x4bc8d97c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c8d97f, 33 },
    { 0xee18d97f, 84 },
    { 0xde28d981, 93 },
    { 0xfff8d982, 68 },
    { 0x67b8d97f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd908d986, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8998d98c, 55 },
    { 0x2918d98d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a8d991, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6708d994, 61 },
    { 0x4158d995, 85 },
    { 0x2658d995, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7148d99e, 26 },
    { 0x68a8d99f, 99 },
    { 0x1368d9a0, 42 },
    { 0xac48d9a0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e8d9a5, 23 },
    { 0, 0 },
    { 0x1878d9a7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d9b0, 23 },
    { 0xbf18d9b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa88d9c0, 23 },
    { 0x62d8d9c0, 40 },
    { 0, 0 },
    { 0x60b8d9c3, 77 },
    { 0xad08d9c4, 53 },
    { 0xd978d9c4, 23 },
    { 0x5cb8d9c6, 78 },
    { 0x6518d9c5, 95 },
    { 0x3698d9c6, 76 },
    { 0x35d8d9c9, 76 },
    { 0x7768d9ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c58d9cd, 97 },
    { 0x6768d9cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4828d9d2, 95 },
    { 0x7508d9d3, 34 },
    { 0x20c8d9d3, 95 },
    { 0x2168d9d5, 78 },
    { 0, 0 },
    { 0xe768d9d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e8d9da, 95 },
    { 0xd648d9db, 43 },
    { 0, 0 },
    { 0xef98d9dd, 78 },
    { 0x3358d9de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc28d9e1, 76 },
    { 0xf478d9e2, 88 },
    { 0xf9b8d9e2, 90 },
    { 0xc7b8d9e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d8d9ea, 87 },
    { 0, 0 },
    { 0xb0f8d9ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4278d9f1, 90 },
    { 0x4d68d9f2, 90 },
    { 0, 0 },
    { 0xa398d9f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6318d9f7, 46 },
    { 0x7ac8d9f8, 23 },
    { 0x92b8d9f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb868d9fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef8da02, 68 },
    { 0, 0 },
    { 0x8fd8da04, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee8da09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff88da0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xec68da12, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b8da16, 78 },
    { 0x38c8da17, 85 },
    { 0x8608da17, 52 },
    { 0, 0 },
    { 0xd718da1a, 23 },
    { 0x898da1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf8da1e, 17 },
    { 0xd418da1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8da22, 53 },
    { 0xe168da23, 61 },
    { 0x3918da24, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f98da2a, 90 },
    { 0x17e8da2b, 52 },
    { 0x638da2b, 55 },
    { 0x1dd8da2b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a8da30, 98 },
    { 0xca38da30, 61 },
    { 0x52d8da32, 66 },
    { 0, 0 },
    { 0x4e88da34, 33 },
    { 0xb298da34, 61 },
    { 0, 0 },
    { 0x168da37, 43 },
    { 0xdd8da37, 46 },
    { 0xc098da39, 55 },
    { 0xe8da38, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd118da40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b8da43, 33 },
    { 0x3a78da43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c88da47, 77 },
    { 0xe4a8da48, 84 },
    { 0, 0 },
    { 0xdf88da4a, 66 },
    { 0x7918da4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848da4e, 95 },
    { 0, 0 },
    { 0x49d8da50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe888da54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d8da57, 98 },
    { 0x5e28da57, 43 },
    { 0, 0 },
    { 0x7868da5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5088da63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f8da68, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda98da73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb88da76, 56 },
    { 0x96b8da77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd8da85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f58da8f, 98 },
    { 0xc228da90, 100 },
    { 0x48b8da90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b98da94, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6298da98, 55 },
    { 0xa4e8da99, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7458da9c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b8daa1, 90 },
    { 0, 0 },
    { 0x91d8daa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e68daa6, 23 },
    { 0, 0 },
    { 0x2a38daa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8daab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe068dab3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b58dab6, 23 },
    { 0, 0 },
    { 0xa58dab8, 56 },
    { 0x3e18dab9, 98 },
    { 0x81a8dab8, 78 },
    { 0, 0 },
    { 0x2b08dabc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98dac0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea8dac3, 66 },
    { 0xfaf8dac4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3178dac7, 91 },
    { 0x83b8dac8, 43 },
    { 0xb648dac8, 95 },
    { 0xd138daca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b38dace, 95 },
    { 0x50f8dace, 98 },
    { 0x2ca8dace, 93 },
    { 0x8a8dad1, 84 },
    { 0, 0 },
    { 0xb58dad3, 90 },
    { 0x8e98dad4, 77 },
    { 0, 0 },
    { 0x1f78dad6, 85 },
    { 0x3428dad6, 90 },
    { 0, 0 },
    { 0xd018dad9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b88dade, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3808dae3, 98 },
    { 0xfb78dae4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8dae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc448daec, 51 },
    { 0x5e68daed, 52 },
    { 0x4028daee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc88daf1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8daf7, 75 },
    { 0xd3f8daf8, 87 },
    { 0, 0 },
    { 0x98e8dafa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x708dafd, 34 },
    { 0x9168dafe, 33 },
    { 0xaf78daff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1518db02, 55 },
    { 0x12b8db03, 17 },
    { 0xea28db03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac48db08, 95 },
    { 0xff78db09, 76 },
    { 0x1bd8db0a, 33 },
    { 0x60b8db0a, 27 },
    { 0, 0 },
    { 0x5758db0d, 34 },
    { 0xe248db0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb8db14, 76 },
    { 0x4a58db15, 85 },
    { 0x34a8db15, 98 },
    { 0x9648db16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b68db1a, 33 },
    { 0, 0 },
    { 0x7b28db1c, 33 },
    { 0xe0c8db1c, 23 },
    { 0xd3b8db1e, 61 },
    { 0xb8e8db1f, 92 },
    { 0x8cc8db20, 80 },
    { 0x2838db21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6768db26, 98 },
    { 0x9288db27, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398db2b, 78 },
    { 0, 0 },
    { 0xb178db2d, 87 },
    { 0x4698db2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8db32, 90 },
    { 0x8fc8db33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa028db37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818db3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f38db40, 90 },
    { 0, 0 },
    { 0xd108db42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb078db45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6388db49, 61 },
    { 0, 0 },
    { 0x8858db4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df8db4e, 61 },
    { 0x9788db4f, 69 },
    { 0x6738db4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df8db55, 90 },
    { 0, 0 },
    { 0xed38db57, 100 },
    { 0x38e8db58, 95 },
    { 0x4e98db57, 34 },
    { 0xed78db5a, 90 },
    { 0x66d8db5a, 95 },
    { 0x9e98db5c, 23 },
    { 0xef78db5d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d28db61, 61 },
    { 0x8b08db62, 33 },
    { 0x24d8db61, 95 },
    { 0x3ec8db61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a8db67, 17 },
    { 0x258db68, 76 },
    { 0, 0 },
    { 0xd318db6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158db6f, 34 },
    { 0x5f48db6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da8db79, 52 },
    { 0, 0 },
    { 0xf248db7b, 43 },
    { 0, 0 },
    { 0xbd08db7d, 84 },
    { 0x198db7e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3838db81, 91 },
    { 0x2ed8db82, 61 },
    { 0, 0 },
    { 0xe448db84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db87, 85 },
    { 0, 0 },
    { 0xa598db89, 90 },
    { 0x3408db8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a38db94, 98 },
    { 0x92c8db94, 90 },
    { 0xcd88db95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8db9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2608db9d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd18dba1, 76 },
    { 0, 0 },
    { 0x87d8dba3, 33 },
    { 0xe908dba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c78dba7, 76 },
    { 0x29c8dba7, 93 },
    { 0xb5a8dba7, 34 },
    { 0, 0 },
    { 0xa268dbab, 101 },
    { 0, 0 },
    { 0x8fc8dbad, 90 },
    { 0x5ad8dbae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4318dbba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a28dbc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd88dbc3, 33 },
    { 0x3ff8dbc3, 83 },
    { 0xa0e8dbc5, 58 },
    { 0, 0 },
    { 0x2c8dbc7, 93 },
    { 0, 0 },
    { 0xaaf8dbc9, 84 },
    { 0xc8b8dbca, 61 },
    { 0x9b98dbcb, 46 },
    { 0xbd18dbcc, 90 },
    { 0x48d8dbc9, 95 },
    { 0xa778dbce, 93 },
    { 0xedc8dbce, 77 },
    { 0x9e58dbd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f08dbd4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa8dbd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b8dbdc, 33 },
    { 0, 0 },
    { 0xad68dbde, 23 },
    { 0xedb8dbdf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b8dbe5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f8dbe9, 90 },
    { 0, 0 },
    { 0x6b68dbeb, 46 },
    { 0, 0 },
    { 0x8d8dbed, 91 },
    { 0xb738dbee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be8dbf1, 81 },
    { 0x77f8dbf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2188dbf8, 90 },
    { 0xc9f8dbf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1988dbfe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1148dc08, 95 },
    { 0, 0 },
    { 0x76e8dc0a, 90 },
    { 0, 0 },
    { 0x8ab8dc0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e8dc11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd8dc15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff68dc1a, 43 },
    { 0xf338dc1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8dc1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad58dc26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d08dc2d, 46 },
    { 0xd208dc2d, 76 },
    { 0x65b8dc2e, 90 },
    { 0x1ef8dc2f, 91 },
    { 0, 0 },
    { 0x55a8dc32, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c68dc3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f8dc3f, 43 },
    { 0xbb98dc3f, 34 },
    { 0x8e78dc41, 81 },
    { 0xa958dc42, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7378dc45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe8dc50, 76 },
    { 0xa98dc50, 55 },
    { 0, 0 },
    { 0xb468dc53, 76 },
    { 0xe918dc54, 90 },
    { 0x7cc8dc55, 90 },
    { 0xa798dc56, 43 },
    { 0x8238dc53, 44 },
    { 0x398dc53, 23 },
    { 0x5098dc54, 23 },
    { 0x7448dc5a, 23 },
    { 0xf0c8dc5b, 90 },
    { 0x47c8dc5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6148dc66, 98 },
    { 0, 0 },
    { 0x6618dc68, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7448dc6b, 34 },
    { 0, 0 },
    { 0xee58dc6d, 33 },
    { 0xcb78dc6e, 75 },
    { 0x4368dc6f, 61 },
    { 0x3638dc6e, 68 },
    { 0x1f98dc6f, 76 },
    { 0x7ce8dc6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5548dc77, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1808dc7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c08dc82, 90 },
    { 0, 0 },
    { 0xf868dc84, 34 },
    { 0x63b8dc85, 46 },
    { 0, 0 },
    { 0x23d8dc87, 90 },
    { 0xf758dc87, 95 },
    { 0xe058dc89, 83 },
    { 0, 0 },
    { 0x4f58dc8b, 95 },
    { 0x46a8dc8c, 33 },
    { 0x3a8dc8d, 33 },
    { 0x9398dc8e, 95 },
    { 0, 0 },
    { 0x95e8dc90, 95 },
    { 0xe6d8dc91, 61 },
    { 0x6f68dc92, 65 },
    { 0x1858dc92, 61 },
    { 0, 0 },
    { 0x7c78dc95, 81 },
    { 0x9438dc96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea8dc9b, 98 },
    { 0xaeb8dc9c, 66 },
    { 0, 0 },
    { 0xfc58dc9e, 90 },
    { 0, 0 },
    { 0xb638dca0, 61 },
    { 0xec98dca0, 53 },
    { 0x2ef8dca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a8dca7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc8dcaa, 61 },
    { 0, 0 },
    { 0xbb68dcac, 90 },
    { 0x1fd8dcad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a8dcb4, 46 },
    { 0xdd98dcb4, 23 },
    { 0x7b18dcb6, 90 },
    { 0xae68dcb7, 90 },
    { 0xf1c8dcb8, 55 },
    { 0x4da8dcb9, 95 },
    { 0x2498dcba, 61 },
    { 0, 0 },
    { 0x4328dcbc, 33 },
    { 0x1288dcbc, 33 },
    { 0xc6a8dcbc, 78 },
    { 0x8928dcbf, 66 },
    { 0x63a8dcc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98dcc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e28dcce, 76 },
    { 0x8428dccf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a8dcd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7278dcd5, 90 },
    { 0, 0 },
    { 0xfd18dcd7, 55 },
    { 0x5cb8dcd8, 93 },
    { 0xbe68dcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd658dcde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d18dce4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688dceb, 88 },
    { 0, 0 },
    { 0xcf38dced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d8dcf1, 61 },
    { 0x6758dcf2, 53 },
    { 0x458dcf1, 98 },
    { 0x1518dcf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb458dcfa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f8dcff, 34 },
    { 0x1948dd00, 90 },
    { 0x3128dd01, 26 },
    { 0x5578dd01, 90 },
    { 0x4d18dd02, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a78dd08, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd8dd10, 95 },
    { 0x1c98dd11, 23 },
    { 0xa98dd11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2708dd18, 76 },
    { 0, 0 },
    { 0x8088dd1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8dd1f, 33 },
    { 0x9918dd20, 90 },
    { 0xea58dd21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538dd2d, 23 },
    { 0xb998dd2d, 66 },
    { 0x6868dd2f, 91 },
    { 0x7748dd30, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf248dd34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18dd38, 61 },
    { 0, 0 },
    { 0x3348dd3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d98dd40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38dd43, 85 },
    { 0x5fc8dd44, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e08dd47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b8dd4a, 65 },
    { 0, 0 },
    { 0x1938dd4c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f8dd50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8dd53, 55 },
    { 0x2138dd54, 68 },
    { 0xc5d8dd54, 77 },
    { 0xe838dd55, 61 },
    { 0x4cf8dd57, 33 },
    { 0xe5b8dd54, 90 },
    { 0xeae8dd59, 61 },
    { 0x9878dd5a, 78 },
    { 0xab88dd5b, 90 },
    { 0x6188dd5b, 95 },
    { 0x1ed8dd58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b8dd62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3798dd66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4318dd6e, 93 },
    { 0x7328dd6e, 76 },
    { 0, 0 },
    { 0xd718dd71, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b8dd79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6718dd7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf138dd81, 78 },
    { 0xa48dd81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3908dd85, 33 },
    { 0x698dd85, 33 },
    { 0xeb68dd85, 77 },
    { 0, 0 },
    { 0x2d8dd89, 90 },
    { 0, 0 },
    { 0xa0d8dd8b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7198dd91, 34 },
    { 0, 0 },
    { 0xdaa8dd93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3958dd9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6838dda7, 90 },
    { 0x2048dda7, 76 },
    { 0, 0 },
    { 0xeda8ddaa, 34 },
    { 0, 0 },
    { 0xa598ddac, 33 },
    { 0xb0e8ddac, 66 },
    { 0, 0 },
    { 0x53f8ddaf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a8ddb3, 66 },
    { 0, 0 },
    { 0xf7e8ddb5, 95 },
    { 0x5428ddb6, 98 },
    { 0xd838ddb7, 61 },
    { 0x94b8ddb8, 98 },
    { 0x8458ddb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f88ddbc, 93 },
    { 0x4a8ddbd, 76 },
    { 0x55e8ddbe, 61 },
    { 0x3768ddbe, 34 },
    { 0x4638ddbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb618ddc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b8ddc6, 67 },
    { 0xe458ddc6, 91 },
    { 0x29f8ddc8, 76 },
    { 0, 0 },
    { 0x5508ddca, 99 },
    { 0x74a8ddca, 43 },
    { 0xea18ddcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf28ddcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4378ddd2, 84 },
    { 0x4cc8ddd3, 93 },
    { 0x9958ddd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe728dddd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5238dde0, 98 },
    { 0, 0 },
    { 0x4e08dde2, 33 },
    { 0x2768dde3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c8dde6, 83 },
    { 0xb118dde7, 23 },
    { 0x28a8dde8, 33 },
    { 0, 0 },
    { 0xd078ddea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1848ddee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d8ddf1, 33 },
    { 0x4f88ddf2, 40 },
    { 0, 0 },
    { 0x7278ddf4, 23 },
    { 0x2618ddf5, 90 },
    { 0x4398ddf6, 76 },
    { 0, 0 },
    { 0x4e98ddf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ddfc, 69 },
    { 0x4738ddfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8278de00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c68de03, 78 },
    { 0, 0 },
    { 0xa5e8de05, 23 },
    { 0, 0 },
    { 0xd1e8de07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf8de0b, 33 },
    { 0x74c8de0c, 61 },
    { 0xda58de0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f08de12, 76 },
    { 0xc388de13, 61 },
    { 0xf3d8de12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8de19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa188de1d, 66 },
    { 0, 0 },
    { 0xff8de1f, 61 },
    { 0, 0 },
    { 0x3dd8de21, 43 },
    { 0x2fe8de22, 87 },
    { 0x6338de23, 91 },
    { 0x87c8de24, 90 },
    { 0xff58de25, 61 },
    { 0xe718de25, 95 },
    { 0, 0 },
    { 0xa2f8de28, 98 },
    { 0, 0 },
    { 0x77a8de2a, 61 },
    { 0xa378de2b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd8de2e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d68de39, 90 },
    { 0x8af8de3a, 95 },
    { 0xc388de3b, 33 },
    { 0, 0 },
    { 0xa6d8de3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1948de45, 34 },
    { 0x3b28de45, 33 },
    { 0xa3b8de45, 68 },
    { 0x8768de48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d8de4b, 33 },
    { 0x5ba8de4c, 33 },
    { 0, 0 },
    { 0xb428de4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4938de61, 91 },
    { 0xbfa8de62, 99 },
    { 0x3908de62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb18de66, 46 },
    { 0x77e8de67, 78 },
    { 0, 0 },
    { 0x6e08de69, 84 },
    { 0x96a8de69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4888de6d, 98 },
    { 0xb408de6e, 17 },
    { 0x5918de6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58de72, 34 },
    { 0xf058de73, 76 },
    { 0xaa48de72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f8de7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd078de81, 55 },
    { 0x58de82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa8de8b, 61 },
    { 0x8998de8c, 33 },
    { 0xcbb8de8b, 17 },
    { 0x7708de8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e8de91, 76 },
    { 0, 0 },
    { 0x7988de93, 33 },
    { 0x47a8de93, 61 },
    { 0xbab8de95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda08de98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d8de9b, 78 },
    { 0xfcd8de9c, 23 },
    { 0x8048de9d, 81 },
    { 0x6468de9e, 43 },
    { 0xf4d8de9f, 46 },
    { 0x5038de9f, 95 },
    { 0, 0 },
    { 0x9398dea2, 93 },
    { 0x3218dea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4828dea7, 66 },
    { 0x8ba8dea8, 46 },
    { 0xf2e8dea7, 55 },
    { 0x9c48dea9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb8deaf, 23 },
    { 0xb658deb0, 55 },
    { 0x268deaf, 90 },
    { 0x4a68deb2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d18deba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee48debe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9598dec2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c48dec6, 61 },
    { 0xa208dec7, 95 },
    { 0x43e8dec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1718decb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec28dece, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c8ded1, 90 },
    { 0, 0 },
    { 0x1d68ded3, 90 },
    { 0xe1d8ded3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a8ded8, 90 },
    { 0x1048ded8, 96 },
    { 0, 0 },
    { 0x9ee8dedb, 66 },
    { 0x4ce8dedb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5638dee0, 82 },
    { 0x2b98dee1, 95 },
    { 0x6028dee2, 90 },
    { 0xd598dee3, 95 },
    { 0xfa38dee4, 98 },
    { 0x9d68dee5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c8deec, 95 },
    { 0xb778deed, 23 },
    { 0, 0 },
    { 0x6cd8deef, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe28def6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4178def9, 61 },
    { 0, 0 },
    { 0x7948defb, 100 },
    { 0xdf88defc, 97 },
    { 0x5778defc, 90 },
    { 0xe0a8defe, 33 },
    { 0x7d58deff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8df02, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8058df06, 90 },
    { 0xdd78df06, 90 },
    { 0, 0 },
    { 0x4f8df09, 76 },
    { 0x79f8df09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3958df0f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488df14, 95 },
    { 0x2048df15, 78 },
    { 0, 0 },
    { 0xf198df17, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d48df1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd28df1f, 90 },
    { 0xbbd8df20, 23 },
    { 0xb8e8df21, 99 },
    { 0x6628df1f, 33 },
    { 0, 0 },
    { 0x5a28df24, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c68df29, 34 },
    { 0xb338df2a, 33 },
    { 0x16d8df29, 99 },
    { 0x6408df2a, 90 },
    { 0x2bc8df2d, 77 },
    { 0x5378df2e, 55 },
    { 0x2e48df2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8df33, 23 },
    { 0x8e48df34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff8df39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d8df3c, 43 },
    { 0, 0 },
    { 0x5698df3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa418df42, 98 },
    { 0x4f68df43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8738df4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a8df4e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c8df56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf8df5e, 61 },
    { 0xdcb8df5e, 90 },
    { 0, 0 },
    { 0xa578df61, 61 },
    { 0xa18df62, 61 },
    { 0x69c8df63, 81 },
    { 0xf408df61, 99 },
    { 0, 0 },
    { 0xa078df66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c28df6a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8df74, 98 },
    { 0x84e8df75, 61 },
    { 0x4168df75, 33 },
    { 0xffa8df77, 98 },
    { 0, 0 },
    { 0x4a8df79, 76 },
    { 0, 0 },
    { 0x9d58df7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee98df7e, 23 },
    { 0xac38df7f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da8df82, 90 },
    { 0, 0 },
    { 0x25e8df84, 61 },
    { 0, 0 },
    { 0x968df86, 33 },
    { 0, 0 },
    { 0xdb78df88, 88 },
    { 0, 0 },
    { 0x4ae8df8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b8df8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae68df91, 98 },
    { 0, 0 },
    { 0xfce8df93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba78df96, 90 },
    { 0x5268df97, 87 },
    { 0xd8a8df96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe68df9b, 55 },
    { 0xb7c8df9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f8df9f, 95 },
    { 0x83c8df9f, 52 },
    { 0x5048df9f, 95 },
    { 0x98a8dfa2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd978dfa7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e8dfab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2748dfb4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xda18dfb7, 93 },
    { 0, 0 },
    { 0x6098dfb9, 55 },
    { 0xa588dfb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9148dfca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1028dfcd, 23 },
    { 0, 0 },
    { 0x7208dfcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8dfd4, 90 },
    { 0, 0 },
    { 0x6b8dfd6, 90 },
    { 0x9748dfd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c08dfda, 98 },
    { 0x1b08dfdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d8dfe0, 77 },
    { 0, 0 },
    { 0x5488dfe2, 23 },
    { 0xb678dfe3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf238dfe6, 61 },
    { 0, 0 },
    { 0x84b8dfe8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b88dfec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dfef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb498dffa, 23 },
    { 0x2fe8dffb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8e000, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e8e006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd8e009, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9798e00d, 55 },
    { 0x9a18e00e, 88 },
    { 0, 0 },
    { 0xcb28e010, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb158e014, 61 },
    { 0x9238e015, 76 },
    { 0xa7f8e015, 98 },
    { 0x398e017, 33 },
    { 0x4758e018, 33 },
    { 0x2df8e014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f8e01c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc238e020, 33 },
    { 0, 0 },
    { 0xcaf8e022, 52 },
    { 0x76e8e022, 95 },
    { 0xaca8e024, 33 },
    { 0xba98e024, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288e02b, 90 },
    { 0x79f8e02c, 34 },
    { 0, 0 },
    { 0x8768e02e, 64 },
    { 0x4078e02f, 98 },
    { 0, 0 },
    { 0x7408e031, 75 },
    { 0x25a8e032, 23 },
    { 0x2458e033, 91 },
    { 0xead8e032, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac8e038, 95 },
    { 0x1018e039, 90 },
    { 0, 0 },
    { 0xf5f8e03b, 95 },
    { 0, 0 },
    { 0xbbd8e03d, 90 },
    { 0xd558e03e, 18 },
    { 0xd1f8e03f, 93 },
    { 0, 0 },
    { 0x4508e041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff8e045, 23 },
    { 0xe478e045, 95 },
    { 0xfe8e045, 8 },
    { 0, 0 },
    { 0x4488e049, 81 },
    { 0x15a8e04a, 90 },
    { 0x9548e04a, 98 },
    { 0xc0f8e04c, 78 },
    { 0x4ab8e049, 33 },
    { 0, 0 },
    { 0x9e38e04f, 95 },
    { 0x3eb8e050, 83 },
    { 0xea78e050, 34 },
    { 0x77d8e052, 61 },
    { 0x7d98e053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c8e056, 61 },
    { 0xfe78e057, 23 },
    { 0xc388e057, 76 },
    { 0x5fd8e059, 77 },
    { 0xb278e058, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51c8e060, 67 },
    { 0x12c8e061, 76 },
    { 0x7238e062, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e78e066, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c8e06b, 98 },
    { 0xc218e06c, 34 },
    { 0xbf98e06d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9448e073, 61 },
    { 0xa88e073, 23 },
    { 0x9818e073, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde8e079, 95 },
    { 0x62f8e07a, 61 },
    { 0x3668e079, 33 },
    { 0xa838e07c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce98e085, 33 },
    { 0x9948e085, 55 },
    { 0xff8e086, 55 },
    { 0x2398e085, 95 },
    { 0x5c68e088, 43 },
    { 0xfec8e08a, 77 },
    { 0xff38e08b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5628e08f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5818e092, 98 },
    { 0x2648e093, 52 },
    { 0x1c48e094, 98 },
    { 0, 0 },
    { 0xa5f8e096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f8e099, 98 },
    { 0xa4a8e09a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb938e09d, 78 },
    { 0x9bf8e09e, 61 },
    { 0xf538e09f, 33 },
    { 0x9458e0a0, 84 },
    { 0x6f08e09e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9028e0a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef8e0ab, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc688e0ae, 23 },
    { 0x5bb8e0af, 23 },
    { 0, 0 },
    { 0xc8b8e0b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd18e0b4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d68e0b7, 33 },
    { 0x1608e0b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad58e0bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e8e0c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d18e0c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4968e0cd, 90 },
    { 0, 0 },
    { 0xd098e0cf, 95 },
    { 0x2fc8e0cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d8e0d7, 61 },
    { 0, 0 },
    { 0x2e68e0d9, 33 },
    { 0x6f38e0da, 33 },
    { 0, 0 },
    { 0x9a88e0dc, 66 },
    { 0x3e68e0dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4008e0e0, 34 },
    { 0, 0 },
    { 0x6bf8e0e2, 61 },
    { 0x718e0e3, 98 },
    { 0x7ac8e0e4, 43 },
    { 0x7668e0e5, 23 },
    { 0x6388e0e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b8e0e9, 76 },
    { 0, 0 },
    { 0xc4b8e0eb, 55 },
    { 0xdf8e0ec, 93 },
    { 0, 0 },
    { 0x60d8e0ee, 12 },
    { 0xe7a8e0ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1118e0f2, 76 },
    { 0, 0 },
    { 0x1048e0f4, 85 },
    { 0xf528e0f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca88e0fd, 33 },
    { 0, 0 },
    { 0x9bf8e0ff, 23 },
    { 0, 0 },
    { 0x3818e101, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1338e106, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa8e10b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c8e111, 66 },
    { 0xe828e111, 98 },
    { 0x5dc8e113, 61 },
    { 0x6c28e114, 68 },
    { 0x8f78e114, 98 },
    { 0xfe98e115, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d18e119, 90 },
    { 0x89b8e119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1078e11e, 93 },
    { 0x7e08e11f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c8e127, 87 },
    { 0x86e8e127, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c98e12c, 96 },
    { 0x6e98e12d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf8e130, 55 },
    { 0x66d8e131, 78 },
    { 0xfa8e131, 90 },
    { 0x5528e133, 66 },
    { 0x3a08e133, 61 },
    { 0x7d78e133, 61 },
    { 0x9538e136, 34 },
    { 0, 0 },
    { 0x2be8e138, 61 },
    { 0xd468e139, 61 },
    { 0xe848e139, 34 },
    { 0, 0 },
    { 0xf6a8e13c, 33 },
    { 0x95e8e13c, 55 },
    { 0, 0 },
    { 0x34c8e13f, 23 },
    { 0, 0 },
    { 0x6ee8e141, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9758e148, 23 },
    { 0xf578e149, 98 },
    { 0x6ed8e148, 90 },
    { 0, 0 },
    { 0xe298e14c, 33 },
    { 0x3588e14d, 95 },
    { 0x4bd8e14c, 46 },
    { 0x6608e14f, 99 },
    { 0x9068e14c, 61 },
    { 0x5a08e14c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xca48e154, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb648e159, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a8e15f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd68e163, 91 },
    { 0x1c18e164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a18e16a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b8e16f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d8e176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a58e179, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f8e17c, 52 },
    { 0x4808e17d, 90 },
    { 0x4ce8e17e, 90 },
    { 0x2a78e17f, 76 },
    { 0x6348e180, 38 },
    { 0, 0 },
    { 0x1a28e182, 90 },
    { 0x65d8e182, 98 },
    { 0x9908e184, 87 },
    { 0x92a8e185, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e8e18d, 43 },
    { 0x2bc8e18e, 61 },
    { 0x20f8e18f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a8e192, 75 },
    { 0, 0 },
    { 0x4b38e194, 98 },
    { 0xea28e195, 90 },
    { 0x3568e195, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f48e19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b58e1a2, 84 },
    { 0x7fd8e1a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2388e1a6, 40 },
    { 0x5f88e1a6, 46 },
    { 0x9398e1a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd738e1ac, 98 },
    { 0, 0 },
    { 0x8188e1ae, 98 },
    { 0, 0 },
    { 0x61d8e1b0, 90 },
    { 0x29c8e1b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c8e1b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4688e1bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518e1c0, 23 },
    { 0x7718e1c1, 61 },
    { 0x1228e1c1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa758e1cf, 23 },
    { 0xb9f8e1d0, 90 },
    { 0x6f8e1cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d78e1d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8e1da, 23 },
    { 0, 0 },
    { 0x6de8e1dc, 66 },
    { 0x7e18e1dd, 23 },
    { 0x8c88e1dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638e1e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb378e1e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee8e1e8, 84 },
    { 0x2ca8e1e9, 61 },
    { 0xc668e1e8, 90 },
    { 0xdc48e1eb, 33 },
    { 0, 0 },
    { 0x7cf8e1ed, 33 },
    { 0x6288e1ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8868e1f1, 61 },
    { 0, 0 },
    { 0xbe98e1f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea8e1f7, 76 },
    { 0, 0 },
    { 0xb868e1f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7628e1fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9488e200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288e206, 17 },
    { 0, 0 },
    { 0x1b98e208, 98 },
    { 0xad28e209, 8 },
    { 0x6878e208, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8e211, 88 },
    { 0xd6e8e211, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a78e215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f8e21a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff8e21f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda8e222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8e227, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8e22f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6668e232, 98 },
    { 0x63b8e232, 99 },
    { 0, 0 },
    { 0xe408e235, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8e23b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea8e241, 98 },
    { 0x3b88e242, 90 },
    { 0x7198e243, 33 },
    { 0x368e244, 76 },
    { 0x54d8e245, 98 },
    { 0, 0 },
    { 0x9048e247, 90 },
    { 0, 0 },
    { 0xdc08e249, 61 },
    { 0x73c8e24a, 61 },
    { 0x9918e24b, 76 },
    { 0xfe88e24c, 33 },
    { 0, 0 },
    { 0xf428e24e, 46 },
    { 0xe488e24e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda48e258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd8e25b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c18e25f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa528e265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b8e26c, 95 },
    { 0, 0 },
    { 0xcb78e26e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa8e271, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5988e286, 93 },
    { 0xf108e287, 55 },
    { 0, 0 },
    { 0x8b18e289, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd588e28c, 24 },
    { 0, 0 },
    { 0xa008e28e, 99 },
    { 0x8398e28f, 53 },
    { 0xde78e290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a28e296, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8e29d, 95 },
    { 0, 0 },
    { 0x8428e29f, 68 },
    { 0x5908e29f, 98 },
    { 0, 0 },
    { 0x73b8e2a2, 68 },
    { 0x9648e2a3, 95 },
    { 0x6458e2a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5448e2ac, 96 },
    { 0xa9e8e2ac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1838e2b0, 61 },
    { 0x7548e2b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b8e2b5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d8e2c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde18e2c8, 46 },
    { 0xa3c8e2c9, 66 },
    { 0xd6f8e2ca, 23 },
    { 0x3228e2ca, 95 },
    { 0x6528e2cc, 87 },
    { 0, 0 },
    { 0xad78e2ce, 43 },
    { 0x2968e2cf, 34 },
    { 0, 0 },
    { 0x5898e2d1, 100 },
    { 0xf618e2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b8e2d5, 98 },
    { 0xdb18e2d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e2da, 33 },
    { 0, 0 },
    { 0x1f08e2dc, 98 },
    { 0, 0 },
    { 0xe7e8e2de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4078e2e2, 34 },
    { 0xf58e2e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf8e2e8, 93 },
    { 0x8478e2e9, 33 },
    { 0, 0 },
    { 0x6308e2eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa698e2f3, 99 },
    { 0, 0 },
    { 0xd1a8e2f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6538e2f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc498e2fb, 98 },
    { 0, 0 },
    { 0x52a8e2fd, 98 },
    { 0x3ce8e2fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6748e303, 101 },
    { 0x87a8e303, 98 },
    { 0x1a58e305, 87 },
    { 0, 0 },
    { 0xfc48e307, 98 },
    { 0x8f28e307, 98 },
    { 0x1d98e308, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d68e30c, 76 },
    { 0xf8e8e30d, 90 },
    { 0x7848e30c, 33 },
    { 0, 0 },
    { 0x2ae8e310, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af8e315, 98 },
    { 0x2ef8e316, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f8e319, 69 },
    { 0xa8a8e31a, 55 },
    { 0xd328e31a, 95 },
    { 0x4318e31c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd8e31f, 56 },
    { 0xcb48e320, 33 },
    { 0x1ea8e320, 61 },
    { 0xb278e322, 87 },
    { 0xa688e323, 33 },
    { 0xb258e323, 23 },
    { 0xea28e320, 99 },
    { 0xa0c8e323, 95 },
    { 0, 0 },
    { 0x5eb8e328, 77 },
    { 0xd878e328, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58e32c, 33 },
    { 0, 0 },
    { 0x1578e32e, 90 },
    { 0xef78e32e, 33 },
    { 0x1cf8e32e, 61 },
    { 0x29d8e331, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf928e341, 95 },
    { 0xd8e8e342, 33 },
    { 0, 0 },
    { 0xf218e344, 90 },
    { 0, 0 },
    { 0x2648e346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d8e34c, 95 },
    { 0x51e8e34d, 91 },
    { 0x9ea8e34e, 52 },
    { 0, 0 },
    { 0x5a8e350, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1568e353, 91 },
    { 0xdca8e353, 95 },
    { 0xd278e355, 61 },
    { 0x7ab8e353, 46 },
    { 0, 0 },
    { 0x42b8e358, 23 },
    { 0xdef8e359, 43 },
    { 0x48e8e35a, 33 },
    { 0x4558e35a, 95 },
    { 0x9ca8e35c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa628e35f, 33 },
    { 0x6808e35f, 95 },
    { 0x9b68e360, 99 },
    { 0x2988e35f, 55 },
    { 0, 0 },
    { 0x1c68e364, 95 },
    { 0, 0 },
    { 0x1168e366, 90 },
    { 0xebf8e367, 46 },
    { 0, 0 },
    { 0xd138e369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf688e36c, 33 },
    { 0x66f8e36d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc198e374, 52 },
    { 0x7028e374, 97 },
    { 0x4938e376, 55 },
    { 0x2d08e375, 90 },
    { 0xbf88e378, 95 },
    { 0, 0 },
    { 0xda58e37a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7578e381, 95 },
    { 0xea78e382, 55 },
    { 0x7b28e383, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa788e387, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e8e38b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d38e38e, 90 },
    { 0x6498e38e, 95 },
    { 0xa518e390, 77 },
    { 0x7458e390, 76 },
    { 0x268e392, 87 },
    { 0xc4f8e393, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb588e39b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2248e3a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f8e3a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d88e3ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2668e3b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e3b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce08e3bb, 78 },
    { 0xc738e3bc, 98 },
    { 0xfbc8e3bc, 33 },
    { 0xe3a8e3be, 66 },
    { 0x61b8e3bf, 23 },
    { 0, 0 },
    { 0xad98e3c1, 61 },
    { 0xb3c8e3c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd78e3c5, 61 },
    { 0x1e88e3c5, 91 },
    { 0, 0 },
    { 0xb488e3c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd398e3cb, 52 },
    { 0x5908e3cc, 90 },
    { 0xd7b8e3cb, 34 },
    { 0x5818e3ce, 23 },
    { 0x6438e3ce, 23 },
    { 0, 0 },
    { 0xc548e3d1, 95 },
    { 0x1708e3d1, 23 },
    { 0xc338e3d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8328e3d7, 90 },
    { 0x3c88e3d8, 23 },
    { 0xe278e3d9, 95 },
    { 0xf3b8e3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde58e3e2, 61 },
    { 0x4228e3e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc788e3e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa068e3ed, 100 },
    { 0xc018e3ee, 66 },
    { 0xc008e3ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1798e3f6, 55 },
    { 0x3388e3f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d8e400, 23 },
    { 0x828e401, 83 },
    { 0x998e402, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab8e405, 23 },
    { 0x93f8e406, 87 },
    { 0xc7c8e405, 23 },
    { 0x9db8e405, 55 },
    { 0x8e98e406, 95 },
    { 0x22d8e408, 44 },
    { 0xbae8e40b, 33 },
    { 0x3198e40b, 98 },
    { 0x7a08e40d, 90 },
    { 0, 0 },
    { 0x6d78e40f, 95 },
    { 0x4bd8e410, 88 },
    { 0xcfc8e411, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3378e414, 90 },
    { 0xaf58e414, 34 },
    { 0x38c8e414, 34 },
    { 0, 0 },
    { 0x7118e418, 17 },
    { 0, 0 },
    { 0x8018e41a, 90 },
    { 0x3758e41b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e8e41f, 36 },
    { 0xa4f8e41f, 90 },
    { 0x32c8e421, 40 },
    { 0xc978e422, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e426, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9538e42e, 36 },
    { 0xb188e42f, 33 },
    { 0, 0 },
    { 0x6378e431, 77 },
    { 0, 0 },
    { 0x61d8e433, 93 },
    { 0, 0 },
    { 0xa978e435, 77 },
    { 0x1ac8e436, 34 },
    { 0, 0 },
    { 0xa238e438, 90 },
    { 0xa28e438, 95 },
    { 0, 0 },
    { 0xa7d8e43b, 90 },
    { 0x8218e43b, 93 },
    { 0x1088e43d, 61 },
    { 0xcf68e43e, 75 },
    { 0xc548e43f, 76 },
    { 0xf0a8e440, 33 },
    { 0xac08e43d, 23 },
    { 0, 0 },
    { 0xd198e443, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x858e446, 98 },
    { 0xdbe8e447, 95 },
    { 0x5b78e447, 43 },
    { 0xb5c8e447, 84 },
    { 0xba78e44a, 90 },
    { 0x1f38e44b, 98 },
    { 0x2cd8e44c, 76 },
    { 0x3698e44d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd988e453, 91 },
    { 0, 0 },
    { 0xc7f8e455, 33 },
    { 0x4038e456, 98 },
    { 0xa2d8e457, 88 },
    { 0x96b8e457, 95 },
    { 0xf438e459, 33 },
    { 0x2008e45a, 90 },
    { 0x21a8e45b, 33 },
    { 0, 0 },
    { 0x2998e45d, 33 },
    { 0x3e08e45e, 76 },
    { 0xca78e45d, 95 },
    { 0xf4d8e45f, 33 },
    { 0xfa88e461, 90 },
    { 0, 0 },
    { 0xe658e463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9188e466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4368e469, 23 },
    { 0xed58e469, 95 },
    { 0x6828e46b, 95 },
    { 0xeb58e46c, 88 },
    { 0, 0 },
    { 0xec98e46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5158e473, 98 },
    { 0, 0 },
    { 0x7b48e475, 61 },
    { 0, 0 },
    { 0xc578e477, 98 },
    { 0x2fc8e478, 95 },
    { 0x5c88e478, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb8e47d, 66 },
    { 0, 0 },
    { 0xe7e8e47f, 98 },
    { 0, 0 },
    { 0x7958e481, 55 },
    { 0xeb58e482, 95 },
    { 0x9e98e483, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea8e48a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf708e490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58e493, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7198e497, 33 },
    { 0x1618e498, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d8e49e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6958e4a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b8e4ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4348e4af, 76 },
    { 0x79a8e4af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab58e4b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc318e4bb, 97 },
    { 0x4668e4bc, 99 },
    { 0x6298e4bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88e4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf098e4c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8e4c9, 55 },
    { 0x2d28e4ca, 55 },
    { 0, 0 },
    { 0xd2a8e4cc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e98e4cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c8e4d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d8e4dc, 33 },
    { 0x3d8e4dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8598e4e0, 98 },
    { 0, 0 },
    { 0x60f8e4e2, 90 },
    { 0x4748e4e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb538e4e6, 61 },
    { 0x1ee8e4e7, 61 },
    { 0x9cf8e4e7, 90 },
    { 0xbc48e4e7, 33 },
    { 0, 0 },
    { 0x4a28e4eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388e4f3, 40 },
    { 0xc18e4f3, 43 },
    { 0xe408e4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b8e4f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb8e4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e8e502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f8e505, 23 },
    { 0, 0 },
    { 0x2508e507, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa8e50a, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a98e50e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe428e514, 95 },
    { 0, 0 },
    { 0xd6c8e516, 95 },
    { 0, 0 },
    { 0xbf98e518, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d8e51b, 33 },
    { 0xda18e51c, 85 },
    { 0, 0 },
    { 0x7568e51e, 100 },
    { 0x53a8e51f, 91 },
    { 0xdfb8e51f, 66 },
    { 0xf0a8e521, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc8e524, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee8e529, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf18e52d, 23 },
    { 0, 0 },
    { 0xe888e52f, 61 },
    { 0x8fe8e530, 87 },
    { 0xfee8e531, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7638e534, 100 },
    { 0, 0 },
    { 0x9e58e536, 33 },
    { 0x20a8e537, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d38e53a, 33 },
    { 0, 0 },
    { 0xa3c8e53c, 46 },
    { 0x8448e53c, 94 },
    { 0, 0 },
    { 0xca28e53f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x258e542, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc8e548, 66 },
    { 0x2d48e548, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc8e550, 17 },
    { 0xcc18e551, 61 },
    { 0, 0 },
    { 0x8b78e553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8628e557, 55 },
    { 0, 0 },
    { 0xbea8e559, 33 },
    { 0x6a88e55a, 85 },
    { 0, 0 },
    { 0xa708e55c, 87 },
    { 0x37c8e55d, 90 },
    { 0xa468e55e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b28e561, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8048e567, 61 },
    { 0xfc78e568, 95 },
    { 0, 0 },
    { 0x7ed8e56a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6608e578, 34 },
    { 0x8bd8e579, 98 },
    { 0, 0 },
    { 0x8928e57b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4588e584, 90 },
    { 0x8378e585, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e588, 76 },
    { 0x72b8e589, 68 },
    { 0xa898e588, 66 },
    { 0xacc8e58a, 55 },
    { 0x5678e58c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce8e591, 91 },
    { 0xe308e592, 55 },
    { 0x5d8e591, 95 },
    { 0, 0 },
    { 0x1598e595, 95 },
    { 0x3e8e595, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f8e599, 33 },
    { 0xe508e59a, 61 },
    { 0, 0 },
    { 0x6b68e59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b8e5a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c68e5a3, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f8e5aa, 61 },
    { 0xcaf8e5ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc548e5ae, 77 },
    { 0x5058e5af, 98 },
    { 0x1d88e5b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc98e5b3, 95 },
    { 0x6bd8e5b4, 33 },
    { 0x6858e5b5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc78e5bb, 55 },
    { 0, 0 },
    { 0xfde8e5bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e18e5c1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98e5ca, 34 },
    { 0, 0 },
    { 0x4d38e5cc, 90 },
    { 0, 0 },
    { 0x5b8e5ce, 91 },
    { 0x15d8e5cf, 76 },
    { 0, 0 },
    { 0x1338e5d1, 61 },
    { 0x7b38e5d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9558e5d5, 80 },
    { 0x2f18e5d5, 76 },
    { 0, 0 },
    { 0x5d38e5d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7068e5db, 66 },
    { 0, 0 },
    { 0x2728e5dd, 3 },
    { 0, 0 },
    { 0x94f8e5df, 66 },
    { 0xaab8e5df, 34 },
    { 0x8888e5e1, 33 },
    { 0x3198e5e2, 93 },
    { 0x6098e5e2, 76 },
    { 0x9b48e5e4, 43 },
    { 0x6d68e5e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6378e5f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e8e604, 100 },
    { 0xcdd8e605, 98 },
    { 0xc938e604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b98e60c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2028e60f, 90 },
    { 0, 0 },
    { 0xd738e611, 61 },
    { 0, 0 },
    { 0x4ff8e613, 66 },
    { 0xc048e614, 61 },
    { 0xdd28e615, 90 },
    { 0x46f8e616, 55 },
    { 0xa9c8e616, 95 },
    { 0xe3e8e616, 55 },
    { 0x2f68e619, 91 },
    { 0x61a8e61a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a98e620, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea8e623, 90 },
    { 0x3158e624, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad8e62f, 33 },
    { 0xfd28e630, 95 },
    { 0xad48e631, 90 },
    { 0x5d8e62f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e636, 33 },
    { 0x5438e636, 43 },
    { 0x7688e636, 43 },
    { 0x2128e639, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3928e63c, 98 },
    { 0x8f48e63d, 17 },
    { 0xa678e63e, 23 },
    { 0xf1a8e63f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb08e642, 95 },
    { 0x7108e643, 33 },
    { 0xc7c8e643, 43 },
    { 0, 0 },
    { 0xcce8e646, 23 },
    { 0, 0 },
    { 0x7ac8e648, 55 },
    { 0xd428e649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d8e64f, 23 },
    { 0x55d8e64f, 76 },
    { 0x65b8e651, 99 },
    { 0, 0 },
    { 0x1b58e653, 33 },
    { 0x2408e654, 90 },
    { 0, 0 },
    { 0x6a58e656, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f28e659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda38e65d, 34 },
    { 0xf358e65e, 90 },
    { 0x5648e65e, 43 },
    { 0xb9c8e660, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e98e669, 48 },
    { 0x7ff8e669, 23 },
    { 0x79b8e669, 55 },
    { 0, 0 },
    { 0x43b8e66d, 61 },
    { 0x1ff8e66e, 93 },
    { 0, 0 },
    { 0x7988e670, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c8e675, 95 },
    { 0x7768e676, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a8e67a, 98 },
    { 0x7438e67a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb928e67f, 95 },
    { 0, 0 },
    { 0xb8e8e681, 93 },
    { 0x348e682, 90 },
    { 0x2e78e683, 61 },
    { 0xd278e684, 33 },
    { 0x52d8e684, 33 },
    { 0xca38e685, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb8e68a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9548e68d, 95 },
    { 0, 0 },
    { 0x468e68f, 46 },
    { 0xdd28e690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7958e693, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2888e696, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2928e699, 49 },
    { 0, 0 },
    { 0x6f08e69b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad38e6a0, 43 },
    { 0xa278e6a1, 33 },
    { 0x2148e6a2, 55 },
    { 0, 0 },
    { 0xcb78e6a4, 99 },
    { 0x6b88e6a4, 93 },
    { 0x25d8e6a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d88e6a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2328e6ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c38e6b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf048e6b3, 90 },
    { 0x63c8e6b4, 23 },
    { 0x7788e6b5, 74 },
    { 0, 0 },
    { 0x52d8e6b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade8e6bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd908e6c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa358e6c3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8e6c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f8e6cb, 23 },
    { 0, 0 },
    { 0x8bc8e6cd, 40 },
    { 0, 0 },
    { 0xcc08e6cf, 23 },
    { 0x23c8e6cf, 33 },
    { 0, 0 },
    { 0xf718e6d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5988e6dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc938e6e1, 85 },
    { 0x9248e6e1, 46 },
    { 0x9ea8e6e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2888e6e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1928e6f0, 33 },
    { 0xbc8e6f1, 55 },
    { 0x4868e6f0, 34 },
    { 0x8e78e6f3, 84 },
    { 0x5618e6f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4988e6fb, 61 },
    { 0x5ca8e6fc, 93 },
    { 0, 0 },
    { 0xb8e6fe, 91 },
    { 0x8e08e6ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e18e702, 98 },
    { 0xa248e703, 34 },
    { 0x34c8e704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8768e708, 76 },
    { 0x7ac8e709, 69 },
    { 0, 0 },
    { 0x6f48e70b, 91 },
    { 0x1308e70b, 76 },
    { 0xf958e70d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2728e713, 95 },
    { 0x8b98e713, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee8e717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8e71b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a8e720, 95 },
    { 0xb508e721, 90 },
    { 0x3608e720, 84 },
    { 0, 0 },
    { 0xe4d8e724, 61 },
    { 0x74d8e724, 61 },
    { 0x9858e726, 43 },
    { 0x5cd8e727, 65 },
    { 0, 0 },
    { 0x6b38e729, 98 },
    { 0x25a8e72a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e88e72d, 99 },
    { 0xa328e72e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b8e73b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe638e73f, 90 },
    { 0x88e740, 23 },
    { 0, 0 },
    { 0xe5f8e742, 76 },
    { 0x75d8e743, 17 },
    { 0x3b58e743, 93 },
    { 0, 0 },
    { 0x7258e746, 61 },
    { 0, 0 },
    { 0x4818e748, 66 },
    { 0, 0 },
    { 0x90a8e74a, 90 },
    { 0xea28e74b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa758e74f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1778e752, 55 },
    { 0x9118e753, 61 },
    { 0x61d8e754, 98 },
    { 0xdb8e754, 87 },
    { 0xa358e754, 41 },
    { 0x7658e756, 93 },
    { 0, 0 },
    { 0xb588e759, 98 },
    { 0xb28e75a, 33 },
    { 0x4d78e75a, 84 },
    { 0x5e8e75c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568e762, 98 },
    { 0x91c8e762, 23 },
    { 0, 0 },
    { 0xc238e765, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8e76c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb8e76f, 11 },
    { 0, 0 },
    { 0x72d8e771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb8e777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e8e77a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7348e780, 95 },
    { 0xb3a8e781, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e785, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d38e789, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc828e78d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf298e791, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc78e794, 33 },
    { 0, 0 },
    { 0xed08e796, 87 },
    { 0x30a8e796, 55 },
    { 0, 0 },
    { 0xa3a8e799, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d38e79c, 90 },
    { 0, 0 },
    { 0xc438e79e, 83 },
    { 0, 0 },
    { 0xbc8e7a0, 38 },
    { 0, 0 },
    { 0x77b8e7a2, 87 },
    { 0x6398e7a3, 81 },
    { 0xd368e7a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1308e7a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a8e7ac, 90 },
    { 0xe4d8e7ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5908e7b2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c88e7b5, 23 },
    { 0xb5c8e7b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b8e7b9, 90 },
    { 0xb948e7ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b8e7bf, 90 },
    { 0, 0 },
    { 0x4628e7c1, 43 },
    { 0, 0 },
    { 0x1f8e7c3, 90 },
    { 0, 0 },
    { 0xd1c8e7c5, 64 },
    { 0x5298e7c6, 90 },
    { 0, 0 },
    { 0x19c8e7c8, 93 },
    { 0xb258e7c9, 76 },
    { 0xa5a8e7c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb8e7cd, 100 },
    { 0, 0 },
    { 0xe958e7cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a8e7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c8e7e1, 33 },
    { 0x4978e7e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be8e7e6, 98 },
    { 0xf298e7e7, 61 },
    { 0xd588e7e8, 100 },
    { 0x4668e7e9, 87 },
    { 0x2d08e7e7, 95 },
    { 0x1078e7eb, 95 },
    { 0x9d48e7ec, 33 },
    { 0, 0 },
    { 0xeb08e7ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb728e7f2, 61 },
    { 0x5f78e7f2, 88 },
    { 0xec08e7f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa28e7ff, 33 },
    { 0, 0 },
    { 0x24f8e801, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd58e804, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d8e80a, 33 },
    { 0xdc58e80b, 46 },
    { 0xd398e80b, 95 },
    { 0x9a78e80d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d48e811, 69 },
    { 0x70c8e812, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8188e819, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e28e81f, 33 },
    { 0x678e81f, 52 },
    { 0x8e98e821, 90 },
    { 0xc088e822, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c8e825, 61 },
    { 0xb738e825, 67 },
    { 0x12e8e827, 91 },
    { 0x5d28e828, 23 },
    { 0x2858e829, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa078e830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af8e834, 90 },
    { 0, 0 },
    { 0xf0c8e836, 61 },
    { 0x7468e837, 34 },
    { 0xff58e837, 85 },
    { 0x16a8e838, 33 },
    { 0x61d8e838, 43 },
    { 0x6808e83b, 33 },
    { 0x3d78e83c, 98 },
    { 0, 0 },
    { 0xda88e83e, 55 },
    { 0, 0 },
    { 0x3a78e840, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d8e843, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe358e84b, 33 },
    { 0, 0 },
    { 0xc1a8e84d, 67 },
    { 0x5f48e84e, 55 },
    { 0xb188e84d, 85 },
    { 0, 0 },
    { 0x5388e851, 33 },
    { 0xe718e852, 33 },
    { 0x7948e853, 75 },
    { 0xc9b8e854, 80 },
    { 0, 0 },
    { 0x5cf8e856, 95 },
    { 0xb668e857, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa508e85b, 95 },
    { 0x25d8e85c, 78 },
    { 0, 0 },
    { 0x7708e85e, 34 },
    { 0x2cd8e85f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd808e863, 99 },
    { 0xe038e864, 46 },
    { 0x8908e864, 97 },
    { 0, 0 },
    { 0x9fa8e867, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c8e86e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f8e872, 99 },
    { 0x5658e873, 99 },
    { 0x1e88e873, 87 },
    { 0x3698e875, 53 },
    { 0xd6d8e872, 75 },
    { 0xf98e877, 95 },
    { 0, 0 },
    { 0x4f28e879, 53 },
    { 0xec38e87a, 90 },
    { 0x3b58e87b, 98 },
    { 0x7b68e87c, 90 },
    { 0, 0 },
    { 0xef38e87e, 33 },
    { 0xa858e87f, 90 },
    { 0x8808e87e, 98 },
    { 0, 0 },
    { 0xe898e882, 76 },
    { 0, 0 },
    { 0xe6f8e884, 98 },
    { 0xff08e885, 61 },
    { 0, 0 },
    { 0x3828e887, 95 },
    { 0, 0 },
    { 0x5288e889, 68 },
    { 0, 0 },
    { 0x4dc8e88b, 95 },
    { 0x4b58e88c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe698e890, 61 },
    { 0xf9c8e891, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8e895, 76 },
    { 0, 0 },
    { 0x65b8e897, 61 },
    { 0x6b18e897, 90 },
    { 0x4178e899, 78 },
    { 0xda28e89a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b18e8a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf988e8af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f88e8b7, 76 },
    { 0xaa68e8b8, 92 },
    { 0, 0 },
    { 0x1b68e8ba, 98 },
    { 0x3798e8ba, 90 },
    { 0x62b8e8bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a8e8c1, 68 },
    { 0x98f8e8c2, 17 },
    { 0xd938e8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f8e8cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828e8d0, 74 },
    { 0x4f78e8d0, 90 },
    { 0, 0 },
    { 0x5b38e8d3, 55 },
    { 0, 0 },
    { 0xb378e8d5, 95 },
    { 0xeef8e8d6, 98 },
    { 0, 0 },
    { 0xc158e8d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c28e8db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8068e8e2, 100 },
    { 0x2e08e8e3, 90 },
    { 0x5748e8e3, 98 },
    { 0xeaa8e8e5, 34 },
    { 0x4da8e8e6, 98 },
    { 0x9c48e8e6, 61 },
    { 0x8cf8e8e8, 83 },
    { 0xec48e8e4, 34 },
    { 0x4368e8ea, 23 },
    { 0xd028e8ea, 23 },
    { 0xdf98e8e3, 87 },
    { 0x38e8e6, 34 },
    { 0x9fd8e8e9, 88 },
    { 0xf838e8ef, 88 },
    { 0xa418e8ef, 76 },
    { 0, 0 },
    { 0x18a8e8f2, 36 },
    { 0xfae8e8f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c8e8f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa78e8f9, 61 },
    { 0xdfb8e8fa, 23 },
    { 0xc428e8f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc628e8ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d98e907, 95 },
    { 0, 0 },
    { 0x26b8e909, 84 },
    { 0x1408e90a, 69 },
    { 0, 0 },
    { 0x8318e90c, 61 },
    { 0x2258e90c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3968e917, 66 },
    { 0, 0 },
    { 0xa858e919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6408e91d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb848e921, 90 },
    { 0x35f8e922, 33 },
    { 0x1e68e922, 55 },
    { 0x5808e922, 75 },
    { 0, 0 },
    { 0x28e8e926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea98e929, 61 },
    { 0, 0 },
    { 0x22a8e92b, 61 },
    { 0x6438e92c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf8e92f, 43 },
    { 0xb6c8e930, 90 },
    { 0x10c8e930, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c98e938, 98 },
    { 0x8ce8e939, 90 },
    { 0xa838e93a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8998e940, 33 },
    { 0, 0 },
    { 0x9268e942, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8898e945, 93 },
    { 0xf008e946, 66 },
    { 0, 0 },
    { 0x2a38e948, 93 },
    { 0xc208e949, 25 },
    { 0xb5a8e949, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c8e94d, 80 },
    { 0x738e94d, 23 },
    { 0, 0 },
    { 0x5448e950, 46 },
    { 0x7848e950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1068e955, 33 },
    { 0x80f8e955, 33 },
    { 0x2ec8e955, 76 },
    { 0xb528e958, 33 },
    { 0x8128e959, 61 },
    { 0x34d8e95a, 33 },
    { 0xb208e95a, 61 },
    { 0xdbb8e956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6048e95f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf098e962, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b8e966, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98e96a, 33 },
    { 0, 0 },
    { 0xadd8e96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f8e972, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b08e978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3898e97d, 95 },
    { 0x64f8e97e, 33 },
    { 0xec18e97d, 98 },
    { 0xc938e97d, 90 },
    { 0x6118e981, 97 },
    { 0, 0 },
    { 0xf648e983, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4818e989, 23 },
    { 0xbe68e98a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae58e996, 90 },
    { 0xd318e997, 77 },
    { 0, 0 },
    { 0x1988e999, 23 },
    { 0x2af8e999, 66 },
    { 0x84a8e999, 43 },
    { 0xf578e99b, 98 },
    { 0xe208e99d, 98 },
    { 0x9af8e99e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d68e9a1, 90 },
    { 0x8018e9a2, 88 },
    { 0x4598e9a3, 33 },
    { 0x75b8e9a2, 78 },
    { 0x1f8e9a3, 65 },
    { 0xc5d8e9a2, 33 },
    { 0x87f8e9a7, 61 },
    { 0x1848e9a8, 98 },
    { 0x2a8e9a6, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a8e9ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6168e9b1, 84 },
    { 0x3a58e9b2, 69 },
    { 0xef08e9b2, 55 },
    { 0, 0 },
    { 0x5778e9b5, 33 },
    { 0, 0 },
    { 0x1de8e9b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c38e9bc, 78 },
    { 0x1588e9bd, 95 },
    { 0x2ab8e9bc, 57 },
    { 0xa268e9bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb218e9c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a8e9cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6868e9d1, 52 },
    { 0x3238e9d1, 61 },
    { 0xf3b8e9d2, 43 },
    { 0, 0 },
    { 0xb9b8e9d5, 99 },
    { 0xbd28e9d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab8e9d9, 33 },
    { 0x9878e9da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8e9dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d58e9e0, 95 },
    { 0x7738e9e0, 90 },
    { 0xa2d8e9e2, 93 },
    { 0xe808e9e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a18e9e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc528e9ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb358e9f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8938e9f7, 88 },
    { 0, 0 },
    { 0x7158e9f9, 60 },
    { 0xd438e9f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2058e9fe, 76 },
    { 0x7d08e9fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e8ea04, 88 },
    { 0x7af8ea05, 68 },
    { 0, 0 },
    { 0x38c8ea07, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc58ea0d, 33 },
    { 0xd3a8ea0d, 33 },
    { 0x4848ea0d, 53 },
    { 0x5dc8ea0e, 90 },
    { 0x2588ea11, 30 },
    { 0xf4e8ea11, 91 },
    { 0xf468ea12, 23 },
    { 0xbdb8ea11, 68 },
    { 0xad58ea11, 99 },
    { 0xff98ea11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8ea22, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4728ea26, 4 },
    { 0, 0 },
    { 0xeb18ea28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a78ea2c, 33 },
    { 0xe7d8ea2d, 85 },
    { 0x3d98ea2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa8ea32, 87 },
    { 0x90d8ea32, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8ea3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c78ea3f, 23 },
    { 0x55d8ea3f, 51 },
    { 0xab18ea3f, 90 },
    { 0x1998ea42, 61 },
    { 0x24d8ea43, 55 },
    { 0xe7b8ea44, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa628ea48, 66 },
    { 0xe7e8ea49, 61 },
    { 0xdff8ea48, 66 },
    { 0x4fe8ea49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9938ea4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c18ea51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4738ea58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c8ea5c, 95 },
    { 0xd128ea5d, 93 },
    { 0, 0 },
    { 0xb688ea5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518ea65, 95 },
    { 0, 0 },
    { 0x2d98ea67, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8ea72, 87 },
    { 0x55e8ea73, 95 },
    { 0x628ea74, 61 },
    { 0x3918ea75, 90 },
    { 0x5e58ea76, 33 },
    { 0xe568ea76, 23 },
    { 0xee18ea76, 33 },
    { 0xc168ea79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc628ea81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78ea86, 81 },
    { 0x8d48ea87, 98 },
    { 0xde78ea88, 56 },
    { 0x9128ea89, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab78ea8d, 90 },
    { 0x7258ea8e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb158ea99, 98 },
    { 0xdfb8ea99, 23 },
    { 0, 0 },
    { 0xf718ea9c, 98 },
    { 0, 0 },
    { 0x1648ea9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888eaa2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a28eaa7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f98eaab, 90 },
    { 0, 0 },
    { 0x2a68eaad, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88eab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c18eab3, 75 },
    { 0, 0 },
    { 0x9818eab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf828eaba, 55 },
    { 0x5378eabb, 98 },
    { 0x1578eabc, 23 },
    { 0x2d38eaba, 87 },
    { 0xedd8eaba, 36 },
    { 0x73c8eabd, 66 },
    { 0x4588eabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8eac5, 90 },
    { 0x9198eac6, 52 },
    { 0x1228eac5, 95 },
    { 0xfe88eac8, 61 },
    { 0x3b78eac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e78eacf, 100 },
    { 0, 0 },
    { 0xdcd8ead1, 23 },
    { 0x74d8ead2, 87 },
    { 0x29a8ead2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a18ead7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d88eadc, 90 },
    { 0, 0 },
    { 0xdfd8eade, 91 },
    { 0xd158eadf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2208eae2, 61 },
    { 0, 0 },
    { 0x73b8eae4, 65 },
    { 0x7b68eae5, 90 },
    { 0xe248eae6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc468eaee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1938eaf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee08eaf7, 23 },
    { 0xc4a8eaf8, 76 },
    { 0x2028eaf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d68eb01, 43 },
    { 0, 0 },
    { 0x36b8eb03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9748eb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe088eb09, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd8eb0d, 34 },
    { 0xd308eb0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe618eb13, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe158eb19, 98 },
    { 0x3698eb19, 76 },
    { 0x3368eb1b, 46 },
    { 0x8788eb1b, 78 },
    { 0x6b78eb1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb8eb25, 33 },
    { 0, 0 },
    { 0xdc48eb27, 34 },
    { 0xeef8eb27, 19 },
    { 0x27f8eb29, 90 },
    { 0xad18eb2a, 61 },
    { 0, 0 },
    { 0x6978eb2c, 55 },
    { 0, 0 },
    { 0x3988eb2e, 98 },
    { 0x6a38eb2f, 78 },
    { 0x968eb2f, 61 },
    { 0x6c58eb2f, 52 },
    { 0, 0 },
    { 0xd9a8eb33, 33 },
    { 0xd38eb34, 89 },
    { 0xd438eb35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c88eb39, 68 },
    { 0, 0 },
    { 0x5c78eb3b, 98 },
    { 0xb868eb3c, 78 },
    { 0xcd68eb3c, 43 },
    { 0, 0 },
    { 0x2518eb3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a8eb43, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb288eb48, 81 },
    { 0xc668eb49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3308eb4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc8eb52, 92 },
    { 0xffa8eb53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca98eb57, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3558eb5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xce38eb5e, 43 },
    { 0x8fc8eb5f, 76 },
    { 0xe5e8eb60, 81 },
    { 0xb018eb60, 98 },
    { 0x5758eb62, 34 },
    { 0xc1b8eb63, 91 },
    { 0x2a58eb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a8eb69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8eb70, 76 },
    { 0, 0 },
    { 0xa2e8eb72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac8eb7b, 61 },
    { 0x4f48eb7b, 98 },
    { 0x68d8eb7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8028eb83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08eb8a, 84 },
    { 0x23d8eb8a, 76 },
    { 0x7a88eb8c, 81 },
    { 0x768eb8d, 76 },
    { 0xd098eb8d, 93 },
    { 0xfa98eb8f, 95 },
    { 0xda28eb8c, 33 },
    { 0, 0 },
    { 0x29e8eb92, 90 },
    { 0, 0 },
    { 0xaf58eb94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7308eb97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9608eb9e, 95 },
    { 0x68c8eb9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a68eba2, 91 },
    { 0x3ab8eba3, 34 },
    { 0, 0 },
    { 0xa9b8eba5, 90 },
    { 0, 0 },
    { 0xfe48eba7, 77 },
    { 0x2c28eba7, 98 },
    { 0x4c68eba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f38ebaf, 23 },
    { 0x8ee8ebb0, 34 },
    { 0xe028ebb1, 67 },
    { 0xe608ebb2, 87 },
    { 0x4f68ebb3, 78 },
    { 0xa08ebb1, 98 },
    { 0x5098ebb0, 91 },
    { 0x9478ebb6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b68ebb9, 61 },
    { 0, 0 },
    { 0x37a8ebbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e8ebbf, 90 },
    { 0x9ac8ebc0, 90 },
    { 0xc2b8ebc0, 95 },
    { 0x24b8ebc2, 93 },
    { 0x8908ebc3, 61 },
    { 0x4f8ebc2, 90 },
    { 0x7998ebc5, 90 },
    { 0xd978ebc6, 56 },
    { 0xc238ebc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb448ebca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb298ebcd, 98 },
    { 0, 0 },
    { 0xc68ebcf, 98 },
    { 0, 0 },
    { 0x5138ebd1, 17 },
    { 0x7108ebd2, 46 },
    { 0x2ca8ebd3, 23 },
    { 0xb5e8ebd4, 33 },
    { 0x2de8ebd4, 33 },
    { 0x6808ebd5, 99 },
    { 0, 0 },
    { 0x1ee8ebd8, 93 },
    { 0xd8d8ebd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1348ebdc, 90 },
    { 0xfb18ebdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ebe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a88ebe8, 46 },
    { 0, 0 },
    { 0x8968ebea, 33 },
    { 0x5758ebeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadb8ebee, 93 },
    { 0xdf98ebee, 98 },
    { 0, 0 },
    { 0x5988ebf1, 98 },
    { 0xb5c8ebf1, 95 },
    { 0, 0 },
    { 0x6888ebf4, 98 },
    { 0xb2f8ebf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c8ebf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c8ebfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe608ebff, 46 },
    { 0, 0 },
    { 0x7f18ec01, 23 },
    { 0xbfb8ec01, 59 },
    { 0x3748ec03, 61 },
    { 0, 0 },
    { 0x78a8ec05, 33 },
    { 0x22e8ec06, 68 },
    { 0x6b78ec07, 95 },
    { 0, 0 },
    { 0x2ba8ec09, 46 },
    { 0x23e8ec0a, 90 },
    { 0, 0 },
    { 0x37f8ec0c, 76 },
    { 0, 0 },
    { 0x9f78ec0e, 76 },
    { 0x92f8ec0f, 87 },
    { 0xdd68ec0e, 61 },
    { 0x7938ec0f, 95 },
    { 0x4128ec12, 90 },
    { 0x2528ec0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbff8ec16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b98ec19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5148ec1d, 78 },
    { 0x2ca8ec1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa798ec21, 90 },
    { 0x48f8ec22, 81 },
    { 0xbe38ec23, 98 },
    { 0xb0f8ec24, 90 },
    { 0x26f8ec25, 75 },
    { 0x4778ec26, 90 },
    { 0xc558ec27, 33 },
    { 0x6f18ec27, 84 },
    { 0x6a8ec29, 90 },
    { 0xfe18ec25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f78ec2d, 33 },
    { 0x22d8ec2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff8ec31, 68 },
    { 0x4198ec32, 61 },
    { 0x2d08ec32, 33 },
    { 0, 0 },
    { 0x5578ec35, 98 },
    { 0, 0 },
    { 0x7798ec37, 100 },
    { 0xdfc8ec37, 43 },
    { 0xd3d8ec39, 91 },
    { 0xd1b8ec3a, 83 },
    { 0x6078ec3a, 90 },
    { 0, 0 },
    { 0x7c58ec3d, 53 },
    { 0xed8ec3e, 76 },
    { 0, 0 },
    { 0x32b8ec40, 98 },
    { 0, 0 },
    { 0xc5e8ec42, 55 },
    { 0x3268ec42, 96 },
    { 0xee58ec44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9908ec4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c68ec55, 23 },
    { 0, 0 },
    { 0x7af8ec57, 61 },
    { 0, 0 },
    { 0x2928ec59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae48ec5c, 91 },
    { 0x3a98ec5d, 61 },
    { 0x14c8ec5e, 95 },
    { 0x18f8ec5f, 93 },
    { 0xd78ec60, 61 },
    { 0, 0 },
    { 0xbe98ec62, 98 },
    { 0xd1e8ec63, 77 },
    { 0xabc8ec64, 61 },
    { 0xb118ec65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca8ec76, 33 },
    { 0, 0 },
    { 0x8938ec78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae8ec7d, 90 },
    { 0x1fd8ec7e, 52 },
    { 0x3f8ec7f, 33 },
    { 0xbb28ec7e, 90 },
    { 0, 0 },
    { 0x48e8ec82, 34 },
    { 0xc828ec83, 90 },
    { 0xf928ec83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d38ec88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b8ec8b, 95 },
    { 0xed98ec8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c8ec92, 61 },
    { 0x1cc8ec93, 55 },
    { 0x5418ec94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f8ec9c, 52 },
    { 0x1ae8ec9d, 90 },
    { 0x8308ec9e, 64 },
    { 0x1998ec9e, 75 },
    { 0xe158ec9c, 95 },
    { 0x7348eca1, 78 },
    { 0xd678eca1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18eca7, 33 },
    { 0, 0 },
    { 0xed88eca9, 93 },
    { 0xec08ecaa, 34 },
    { 0xccb8ecab, 23 },
    { 0x44b8ecaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf48ecb0, 61 },
    { 0xfb8ecb1, 23 },
    { 0xa6d8ecb1, 93 },
    { 0, 0 },
    { 0x7068ecb4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ecb7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7128ecbb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9828ecc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed8ecc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9568eccd, 90 },
    { 0xad58ecce, 55 },
    { 0xaa88ecce, 95 },
    { 0, 0 },
    { 0x3148ecd1, 33 },
    { 0xd4a8ecd1, 61 },
    { 0x248ecd1, 91 },
    { 0xd8d8ecd3, 55 },
    { 0xd988ecd1, 95 },
    { 0x9f58ecd6, 33 },
    { 0, 0 },
    { 0xd778ecd8, 98 },
    { 0xb468ecd9, 97 },
    { 0x6b68ecda, 77 },
    { 0x8078ecd9, 88 },
    { 0, 0 },
    { 0x2958ecdd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab98ece1, 33 },
    { 0xa708ece2, 90 },
    { 0xdda8ece3, 33 },
    { 0x5788ece4, 90 },
    { 0x4958ece5, 43 },
    { 0, 0 },
    { 0x2078ece7, 99 },
    { 0x56c8ece8, 76 },
    { 0xd218ece9, 66 },
    { 0x4ac8ece8, 52 },
    { 0x7808ece8, 84 },
    { 0x18a8ecec, 98 },
    { 0x7e78ecec, 55 },
    { 0xc2d8ecec, 98 },
    { 0xfc98ecef, 95 },
    { 0x4a98ecef, 33 },
    { 0x458ecef, 95 },
    { 0xaad8ecec, 87 },
    { 0, 0 },
    { 0x778ecf4, 90 },
    { 0x1cc8ecf5, 91 },
    { 0x948ecf4, 61 },
    { 0xd3c8ecf7, 98 },
    { 0x4368ecf7, 43 },
    { 0, 0 },
    { 0xb118ecfa, 98 },
    { 0x90f8ecfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8528ecfe, 65 },
    { 0x2db8ecff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba8ed07, 33 },
    { 0x1718ed08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd8ed0c, 55 },
    { 0xcbb8ed0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b8ed10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f88ed15, 77 },
    { 0, 0 },
    { 0xc018ed17, 98 },
    { 0, 0 },
    { 0x36b8ed19, 95 },
    { 0x1fb8ed1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8ed20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b8ed23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff28ed2c, 33 },
    { 0, 0 },
    { 0x6c28ed2e, 90 },
    { 0, 0 },
    { 0x7618ed30, 90 },
    { 0, 0 },
    { 0xbc98ed32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f8ed38, 43 },
    { 0, 0 },
    { 0xa098ed3a, 43 },
    { 0, 0 },
    { 0xf0c8ed3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b38ed40, 34 },
    { 0x6c88ed41, 17 },
    { 0xd468ed41, 33 },
    { 0x1de8ed40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7688ed52, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d8ed5b, 100 },
    { 0x2d18ed5c, 91 },
    { 0x9b48ed5d, 85 },
    { 0x89f8ed5e, 93 },
    { 0, 0 },
    { 0xc568ed60, 88 },
    { 0, 0 },
    { 0xc1b8ed62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6318ed65, 33 },
    { 0x9b08ed65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2568ed69, 90 },
    { 0, 0 },
    { 0x3f18ed6b, 95 },
    { 0xffc8ed6c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58ed73, 78 },
    { 0x29c8ed74, 46 },
    { 0xe1c8ed74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc18ed7a, 34 },
    { 0xbd68ed7a, 76 },
    { 0, 0 },
    { 0x9b28ed7d, 91 },
    { 0, 0 },
    { 0xa068ed7f, 23 },
    { 0, 0 },
    { 0x2458ed81, 98 },
    { 0, 0 },
    { 0xc1a8ed83, 75 },
    { 0, 0 },
    { 0x5668ed85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c8ed8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b68ed90, 61 },
    { 0, 0 },
    { 0xacc8ed92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c8ed97, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4328ed9c, 87 },
    { 0xce08ed9d, 34 },
    { 0x9d48ed9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e68eda4, 46 },
    { 0x3bd8eda5, 100 },
    { 0x53b8eda6, 98 },
    { 0xba08eda7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e28edaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8edae, 66 },
    { 0x3fd8edae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1038edb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa058edbf, 76 },
    { 0x9f08edbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f8edc3, 43 },
    { 0x3338edc4, 77 },
    { 0x46c8edc4, 61 },
    { 0xfb18edc6, 81 },
    { 0x4bf8edc7, 33 },
    { 0, 0 },
    { 0x9668edc9, 61 },
    { 0, 0 },
    { 0xe68edcb, 61 },
    { 0x5588edcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad8edd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d58edd4, 61 },
    { 0x3e98edd4, 46 },
    { 0x15c8edd4, 55 },
    { 0x6368edd7, 33 },
    { 0xf7e8edd6, 33 },
    { 0x20c8edd6, 43 },
    { 0x5758edda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4348edde, 61 },
    { 0xd928eddf, 90 },
    { 0x2a48eddf, 76 },
    { 0x7008ede1, 34 },
    { 0xc338ede2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b8ede9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4588eded, 87 },
    { 0x40f8edee, 71 },
    { 0x4868edef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d78edf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db8edf9, 98 },
    { 0xb6f8edfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e08edfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc8ee01, 91 },
    { 0, 0 },
    { 0x3fc8ee03, 33 },
    { 0, 0 },
    { 0xd638ee05, 76 },
    { 0x5e78ee06, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1858ee0d, 91 },
    { 0x5d48ee0d, 52 },
    { 0, 0 },
    { 0x6e08ee10, 90 },
    { 0xd878ee11, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be8ee1a, 98 },
    { 0x18d8ee1b, 61 },
    { 0xe6c8ee1c, 95 },
    { 0x7978ee1d, 52 },
    { 0x8798ee1a, 90 },
    { 0, 0 },
    { 0xb778ee20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa468ee23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698ee26, 98 },
    { 0xcdb8ee26, 99 },
    { 0x9618ee28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54c8ee31, 93 },
    { 0xbe08ee32, 90 },
    { 0x60b8ee31, 43 },
    { 0x478ee34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb628ee3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a8ee3f, 61 },
    { 0x2be8ee40, 52 },
    { 0x6a8ee40, 23 },
    { 0x7718ee42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3688ee45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc8ee4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e8ee51, 84 },
    { 0xbdd8ee51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b8ee55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4478ee58, 98 },
    { 0x94d8ee59, 95 },
    { 0x2578ee5a, 78 },
    { 0, 0 },
    { 0x6378ee5c, 61 },
    { 0xa908ee5c, 61 },
    { 0x6fe8ee5d, 87 },
    { 0x7b8ee5c, 55 },
    { 0x5f48ee60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e98ee64, 43 },
    { 0x9ab8ee65, 95 },
    { 0x8828ee64, 100 },
    { 0xde38ee67, 61 },
    { 0, 0 },
    { 0x31c8ee69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce8ee6c, 33 },
    { 0x2a8ee6d, 76 },
    { 0x7168ee6e, 23 },
    { 0xdf98ee6d, 95 },
    { 0x6a18ee6c, 85 },
    { 0x248ee71, 61 },
    { 0x8f48ee72, 90 },
    { 0x17e8ee73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2158ee79, 65 },
    { 0xe808ee7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a8ee7d, 99 },
    { 0x6fb8ee7d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc8ee84, 61 },
    { 0, 0 },
    { 0xbee8ee86, 23 },
    { 0xd7e8ee87, 95 },
    { 0xe7d8ee88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e8ee8c, 38 },
    { 0xaaa8ee8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1908ee90, 90 },
    { 0xd548ee90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9798ee97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa8ee9e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb8eea2, 33 },
    { 0, 0 },
    { 0xe278eea4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf48eea7, 33 },
    { 0xeaf8eea8, 61 },
    { 0x5738eea9, 77 },
    { 0xad38eea8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448eeb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e98eeb7, 61 },
    { 0x9be8eeb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b8eebb, 69 },
    { 0xb238eebc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc378eec0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d78eec4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb528eed7, 55 },
    { 0xec88eed7, 90 },
    { 0x3298eed9, 90 },
    { 0xf7e8eeda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6818eedd, 81 },
    { 0, 0 },
    { 0xc118eedf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e8eee7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f8eeec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6788eeef, 96 },
    { 0xf6c8eeef, 84 },
    { 0xd3c8eeef, 43 },
    { 0, 0 },
    { 0xf838eef3, 23 },
    { 0xf378eef4, 95 },
    { 0x3d38eef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f38eef8, 91 },
    { 0x41a8eef9, 91 },
    { 0x4a38eef8, 77 },
    { 0, 0 },
    { 0x10f8eefc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b8ef01, 90 },
    { 0xfff8ef02, 98 },
    { 0, 0 },
    { 0x86a8ef04, 99 },
    { 0xfc58ef05, 98 },
    { 0x4938ef05, 23 },
    { 0xd2a8ef07, 61 },
    { 0xa5b8ef07, 91 },
    { 0, 0 },
    { 0xe898ef0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd518ef0f, 90 },
    { 0x2da8ef10, 55 },
    { 0, 0 },
    { 0x1908ef12, 98 },
    { 0x57d8ef12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad28ef18, 55 },
    { 0, 0 },
    { 0x3318ef1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9268ef1d, 98 },
    { 0, 0 },
    { 0xf708ef1f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b58ef22, 61 },
    { 0x3038ef22, 84 },
    { 0, 0 },
    { 0x7828ef25, 90 },
    { 0, 0 },
    { 0x8eb8ef27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa8ef2a, 95 },
    { 0x60a8ef2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe548ef30, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c8ef3b, 75 },
    { 0, 0 },
    { 0x468ef3d, 98 },
    { 0xe978ef3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1888ef43, 78 },
    { 0x4398ef44, 23 },
    { 0x9b18ef45, 61 },
    { 0x5a38ef46, 23 },
    { 0x6868ef47, 84 },
    { 0x788ef48, 23 },
    { 0x1998ef46, 98 },
    { 0x2a88ef4a, 61 },
    { 0xa708ef4a, 90 },
    { 0x64c8ef49, 68 },
    { 0x41c8ef4c, 34 },
    { 0xbff8ef4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb8ef52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe318ef57, 90 },
    { 0xf448ef58, 100 },
    { 0xa078ef59, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e98ef5c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb18ef60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb48ef6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8ef6e, 90 },
    { 0x1898ef6f, 91 },
    { 0, 0 },
    { 0x9fb8ef71, 95 },
    { 0x2408ef72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c28ef76, 33 },
    { 0x7ac8ef77, 68 },
    { 0xb648ef77, 61 },
    { 0x32c8ef76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ef7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b8ef84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f38ef89, 99 },
    { 0x6b18ef8a, 100 },
    { 0x1ee8ef8b, 65 },
    { 0xa608ef89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b98ef95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa78ef9c, 98 },
    { 0, 0 },
    { 0x8898ef9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a48efa2, 93 },
    { 0xe678efa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da8efa6, 76 },
    { 0, 0 },
    { 0x5a38efa8, 76 },
    { 0, 0 },
    { 0xccd8efaa, 34 },
    { 0xea8efab, 76 },
    { 0x7878efab, 78 },
    { 0xf5b8efab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f18efb2, 77 },
    { 0xa6b8efb2, 87 },
    { 0xdd28efb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f98efba, 78 },
    { 0x76f8efba, 98 },
    { 0xc088efbc, 76 },
    { 0xc1b8efbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e28efc3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce8efc6, 98 },
    { 0xf8f8efc7, 99 },
    { 0, 0 },
    { 0x1f88efc9, 100 },
    { 0xfd58efca, 33 },
    { 0x5e58efca, 61 },
    { 0xf6a8efcc, 46 },
    { 0xf68efcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7188efd7, 91 },
    { 0, 0 },
    { 0xdbf8efd9, 33 },
    { 0xf928efda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9858efde, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb318efe2, 33 },
    { 0x5d58efe3, 43 },
    { 0x1fb8efe4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6818efe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a8efee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e58eff1, 46 },
    { 0xe848eff2, 66 },
    { 0x1038eff2, 76 },
    { 0x6f48eff3, 93 },
    { 0x1798eff5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48effb, 61 },
    { 0xf1d8effb, 95 },
    { 0x8558effb, 93 },
    { 0x3158effe, 55 },
    { 0, 0 },
    { 0xaf18f000, 33 },
    { 0x1f98f000, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57f8f004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa78f009, 33 },
    { 0, 0 },
    { 0x6c48f00b, 33 },
    { 0, 0 },
    { 0x7c28f00d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8f011, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc148f015, 33 },
    { 0, 0 },
    { 0x19d8f017, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3998f01a, 23 },
    { 0x7a68f01b, 52 },
    { 0, 0 },
    { 0xca08f01d, 33 },
    { 0x62d8f01d, 61 },
    { 0x27b8f01f, 95 },
    { 0xe828f020, 76 },
    { 0x4dc8f020, 98 },
    { 0xa598f020, 46 },
    { 0xaa18f020, 98 },
    { 0xa228f024, 84 },
    { 0xf7f8f01e, 95 },
    { 0xbb08f01d, 23 },
    { 0xe558f025, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd588f02c, 65 },
    { 0xd7a8f02d, 81 },
    { 0xff68f02e, 90 },
    { 0x6ac8f02e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b8f033, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef68f037, 98 },
    { 0xcb88f038, 98 },
    { 0, 0 },
    { 0x2268f03a, 91 },
    { 0x36b8f03b, 73 },
    { 0, 0 },
    { 0x1208f03d, 76 },
    { 0, 0 },
    { 0x61d8f03f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f04c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc8f050, 68 },
    { 0xcbd8f050, 98 },
    { 0x93c8f052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e8f055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9838f058, 88 },
    { 0xe0f8f059, 91 },
    { 0, 0 },
    { 0x7e78f05b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef08f061, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8f068, 61 },
    { 0xde38f068, 34 },
    { 0xb78f06a, 61 },
    { 0, 0 },
    { 0x9f88f06c, 76 },
    { 0xf68f06c, 34 },
    { 0x228f06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a38f072, 33 },
    { 0x538f073, 90 },
    { 0x5578f072, 93 },
    { 0xf7a8f075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c8f078, 84 },
    { 0, 0 },
    { 0xf7c8f07a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8f083, 90 },
    { 0xbfd8f083, 53 },
    { 0x1d88f085, 61 },
    { 0, 0 },
    { 0x2918f087, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7058f08d, 33 },
    { 0, 0 },
    { 0xf868f08f, 90 },
    { 0xed88f08f, 76 },
    { 0, 0 },
    { 0x5958f092, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7128f097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac8f09c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a8f0a1, 34 },
    { 0x9938f0a1, 23 },
    { 0, 0 },
    { 0x1e18f0a4, 90 },
    { 0xae38f0a4, 4 },
    { 0xca98f0a6, 98 },
    { 0, 0 },
    { 0xef58f0a8, 95 },
    { 0, 0 },
    { 0xc6e8f0aa, 34 },
    { 0xef88f0aa, 61 },
    { 0x6618f0ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c78f0b1, 55 },
    { 0x8218f0b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a8f0b8, 76 },
    { 0, 0 },
    { 0x488f0ba, 23 },
    { 0x4ea8f0bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd98f0c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf8f0c4, 33 },
    { 0, 0 },
    { 0x2668f0c6, 95 },
    { 0x2e88f0c7, 23 },
    { 0x2ac8f0c8, 28 },
    { 0x6198f0c9, 46 },
    { 0x248f0c8, 77 },
    { 0xede8f0c7, 33 },
    { 0xdfe8f0cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8f0d0, 61 },
    { 0x1c8f0d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9088f0d6, 52 },
    { 0x9258f0d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc28f0dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3878f0e5, 23 },
    { 0xa348f0e6, 33 },
    { 0xf668f0e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f0eb, 98 },
    { 0x6428f0ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee68f0f7, 98 },
    { 0x1548f0f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f8f0fd, 33 },
    { 0xb0f8f0fe, 76 },
    { 0x6bd8f0ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb688f109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f8f10e, 61 },
    { 0x28d8f10f, 61 },
    { 0xee18f110, 76 },
    { 0x27c8f111, 98 },
    { 0x98d8f110, 33 },
    { 0xaaa8f110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc168f11c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f8f121, 33 },
    { 0x92a8f121, 23 },
    { 0x2698f123, 78 },
    { 0x3e8f124, 33 },
    { 0, 0 },
    { 0xdb98f126, 76 },
    { 0, 0 },
    { 0x4398f128, 95 },
    { 0xba48f129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6118f130, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18f134, 61 },
    { 0xb858f135, 95 },
    { 0, 0 },
    { 0x1f78f137, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f38f13d, 90 },
    { 0x9f38f13e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d8f144, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da8f148, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea8f14f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f8f156, 33 },
    { 0xc978f157, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe68f15a, 46 },
    { 0, 0 },
    { 0x4ba8f15c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a48f163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d8f167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd8f170, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1638f173, 33 },
    { 0x1b58f173, 61 },
    { 0xc458f174, 95 },
    { 0xc4a8f173, 95 },
    { 0x13f8f177, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f8f17a, 66 },
    { 0x12a8f17a, 90 },
    { 0x5588f17a, 34 },
    { 0x8168f17a, 98 },
    { 0xcd8f17e, 90 },
    { 0xcdf8f17e, 91 },
    { 0x1e08f180, 61 },
    { 0x24b8f181, 78 },
    { 0, 0 },
    { 0x5ae8f183, 52 },
    { 0, 0 },
    { 0x18b8f185, 98 },
    { 0x9158f185, 23 },
    { 0x3e58f187, 90 },
    { 0x2798f188, 23 },
    { 0x3238f185, 90 },
    { 0x5c58f187, 98 },
    { 0xa7a8f187, 98 },
    { 0, 0 },
    { 0x18f18d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c8f190, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e8f19a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18f19d, 75 },
    { 0x9718f19e, 90 },
    { 0x2ae8f19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd8f1a4, 52 },
    { 0x3138f1a5, 33 },
    { 0xb998f1a6, 78 },
    { 0, 0 },
    { 0x79e8f1a8, 90 },
    { 0x3b38f1a9, 61 },
    { 0x4fd8f1a8, 95 },
    { 0, 0 },
    { 0x9a48f1ac, 87 },
    { 0, 0 },
    { 0xb9f8f1ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e8f1b2, 52 },
    { 0x83d8f1b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7318f1b6, 56 },
    { 0x28d8f1b6, 95 },
    { 0xe5a8f1b6, 55 },
    { 0x2198f1b6, 95 },
    { 0, 0 },
    { 0x62e8f1bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2938f1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca8f1c2, 52 },
    { 0, 0 },
    { 0x1008f1c4, 33 },
    { 0xfa98f1c4, 23 },
    { 0x11e8f1c6, 100 },
    { 0x25e8f1c7, 34 },
    { 0x7a8f1c8, 61 },
    { 0xff98f1c9, 95 },
    { 0, 0 },
    { 0xbab8f1cb, 93 },
    { 0, 0 },
    { 0x4f78f1cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe538f1d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9378f1d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa8f1d7, 81 },
    { 0xd3e8f1d8, 77 },
    { 0xb588f1d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe218f1dd, 46 },
    { 0x5248f1de, 33 },
    { 0x2e78f1df, 34 },
    { 0x3678f1e0, 90 },
    { 0xf818f1e1, 46 },
    { 0xfa38f1de, 90 },
    { 0, 0 },
    { 0xcdb8f1e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8f1e8, 33 },
    { 0xb998f1e9, 65 },
    { 0x9288f1e9, 61 },
    { 0x30b8f1ea, 95 },
    { 0x4d38f1e8, 95 },
    { 0x2438f1ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3748f1f3, 98 },
    { 0xfbb8f1f3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd348f1fd, 23 },
    { 0, 0 },
    { 0x2318f1ff, 76 },
    { 0, 0 },
    { 0x58c8f201, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce8f205, 78 },
    { 0xe78f206, 61 },
    { 0xce08f205, 98 },
    { 0xde28f208, 23 },
    { 0, 0 },
    { 0x1e88f20a, 76 },
    { 0, 0 },
    { 0x8108f20c, 99 },
    { 0xa0f8f20d, 95 },
    { 0x5aa8f20e, 99 },
    { 0xbc38f20f, 90 },
    { 0x4c68f20e, 78 },
    { 0x23d8f210, 90 },
    { 0x9798f212, 68 },
    { 0xb428f213, 33 },
    { 0x84f8f214, 61 },
    { 0xd58f213, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28f21d, 100 },
    { 0x2898f21e, 100 },
    { 0x6f08f21f, 33 },
    { 0xb708f21e, 91 },
    { 0x9c38f221, 33 },
    { 0x5ef8f222, 95 },
    { 0xfff8f223, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa788f227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e18f22c, 17 },
    { 0x5618f22d, 98 },
    { 0x9c58f22d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6488f235, 33 },
    { 0x60f8f236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b58f242, 40 },
    { 0xf378f243, 98 },
    { 0x85c8f243, 34 },
    { 0x4688f245, 90 },
    { 0x5248f246, 90 },
    { 0xa018f245, 33 },
    { 0x7ad8f247, 90 },
    { 0x9a58f246, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd678f24d, 53 },
    { 0x3de8f24e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe838f251, 98 },
    { 0, 0 },
    { 0x77c8f253, 98 },
    { 0, 0 },
    { 0x89e8f255, 95 },
    { 0xb8c8f256, 95 },
    { 0, 0 },
    { 0xa938f258, 98 },
    { 0xb7e8f258, 95 },
    { 0, 0 },
    { 0xb88f25b, 34 },
    { 0x9ad8f25c, 95 },
    { 0x55c8f25d, 90 },
    { 0x9e88f25e, 33 },
    { 0x1a08f25b, 99 },
    { 0x3a88f260, 90 },
    { 0xf788f261, 98 },
    { 0xe738f261, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb28f272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7348f275, 78 },
    { 0x43c8f275, 33 },
    { 0x6768f276, 87 },
    { 0x90e8f278, 84 },
    { 0xe138f279, 36 },
    { 0x8448f27a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a8f27f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d8f287, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1508f295, 90 },
    { 0xb1e8f295, 55 },
    { 0x8eb8f297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9078f29a, 84 },
    { 0, 0 },
    { 0xb2d8f29c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe308f2a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e8f2a7, 23 },
    { 0x178f2a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c8f2ac, 33 },
    { 0x27c8f2ad, 28 },
    { 0, 0 },
    { 0xf908f2af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18f2b3, 33 },
    { 0, 0 },
    { 0x3cd8f2b5, 46 },
    { 0xe058f2b6, 97 },
    { 0xf758f2b7, 88 },
    { 0, 0 },
    { 0x9568f2b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6078f2be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e88f2c2, 61 },
    { 0xf8f8f2c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1308f2c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8f2cb, 90 },
    { 0xed18f2cc, 33 },
    { 0, 0 },
    { 0xfe18f2ce, 23 },
    { 0x4e38f2cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8918f2d3, 99 },
    { 0x1c8f2d4, 75 },
    { 0, 0 },
    { 0x82d8f2d6, 61 },
    { 0x1c18f2d6, 90 },
    { 0x7f68f2d8, 98 },
    { 0x6fb8f2d8, 76 },
    { 0xeee8f2da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb738f2de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4068f2e2, 55 },
    { 0x9958f2e3, 90 },
    { 0x4098f2e4, 90 },
    { 0xe0c8f2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868f2e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2718f2ed, 78 },
    { 0x7d08f2ee, 98 },
    { 0x1018f2ef, 33 },
    { 0x9898f2ef, 66 },
    { 0x7b38f2f1, 23 },
    { 0x8978f2f2, 34 },
    { 0x3258f2f3, 61 },
    { 0x8c38f2f4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb778f2f7, 33 },
    { 0, 0 },
    { 0x598f2f9, 17 },
    { 0x4a88f2f9, 95 },
    { 0x2ed8f2fb, 85 },
    { 0xb908f2fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d8f301, 23 },
    { 0xc938f302, 90 },
    { 0, 0 },
    { 0x5078f304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef98f30a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5528f315, 76 },
    { 0x3608f316, 43 },
    { 0x70c8f317, 91 },
    { 0x85a8f318, 23 },
    { 0xdea8f319, 61 },
    { 0x66d8f31a, 23 },
    { 0x2718f31b, 34 },
    { 0xfba8f31a, 76 },
    { 0xb0b8f316, 52 },
    { 0x5fb8f31e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8488f326, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f8f32b, 33 },
    { 0xf478f32c, 100 },
    { 0x3c98f32d, 76 },
    { 0x3028f32c, 98 },
    { 0xe498f32f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba08f334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7938f337, 61 },
    { 0xb2b8f337, 61 },
    { 0, 0 },
    { 0x85e8f33a, 17 },
    { 0, 0 },
    { 0xe648f33c, 61 },
    { 0, 0 },
    { 0x46c8f33e, 91 },
    { 0, 0 },
    { 0x3078f340, 34 },
    { 0, 0 },
    { 0x3ce8f342, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4948f345, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf8f349, 61 },
    { 0x33b8f34a, 78 },
    { 0x4ec8f349, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b8f34f, 33 },
    { 0xb678f34f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2518f353, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed88f357, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b8f35a, 98 },
    { 0x3f08f35b, 76 },
    { 0xaac8f35b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de8f35f, 33 },
    { 0x5ee8f35f, 76 },
    { 0, 0 },
    { 0x67a8f362, 90 },
    { 0x4f48f362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e18f368, 23 },
    { 0, 0 },
    { 0x6138f36a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf138f370, 100 },
    { 0x8c28f371, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff78f377, 90 },
    { 0xe3a8f378, 93 },
    { 0, 0 },
    { 0x54a8f37a, 34 },
    { 0, 0 },
    { 0x4cb8f37c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e8f37f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8f386, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8008f38f, 98 },
    { 0xc1f8f38f, 97 },
    { 0x9058f391, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb8f394, 76 },
    { 0, 0 },
    { 0xcde8f396, 65 },
    { 0, 0 },
    { 0x9428f398, 88 },
    { 0x7cc8f399, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f58f39e, 98 },
    { 0xf3c8f39f, 23 },
    { 0x1648f3a0, 33 },
    { 0x4888f3a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9328f3a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe498f3ae, 61 },
    { 0x9478f3af, 97 },
    { 0x8f08f3af, 33 },
    { 0x4268f3b0, 33 },
    { 0x70e8f3af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e8f3b7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4848f3bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d8f3c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6548f3c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9848f3cb, 90 },
    { 0x2218f3cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe908f3d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8398f3db, 23 },
    { 0x7758f3db, 95 },
    { 0x6388f3dd, 98 },
    { 0x9bf8f3de, 43 },
    { 0xea08f3db, 33 },
    { 0xe488f3de, 40 },
    { 0xd6b8f3e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8568f3e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a8f3f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a8f3fd, 90 },
    { 0x1ff8f3fd, 98 },
    { 0, 0 },
    { 0x7208f400, 64 },
    { 0, 0 },
    { 0xacb8f402, 23 },
    { 0x4278f403, 98 },
    { 0, 0 },
    { 0x18d8f405, 90 },
    { 0x1588f405, 95 },
    { 0xb9b8f407, 43 },
    { 0x4638f408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8f40c, 53 },
    { 0x8828f40c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8f410, 33 },
    { 0, 0 },
    { 0x4b28f412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f418, 33 },
    { 0xcb88f418, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde8f41c, 90 },
    { 0x87b8f41d, 76 },
    { 0x1408f41d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698f422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab8f425, 93 },
    { 0x87a8f426, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c98f42a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb108f42f, 98 },
    { 0x77d8f430, 33 },
    { 0x6e88f431, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb38f435, 90 },
    { 0xb0f8f436, 33 },
    { 0x4838f437, 90 },
    { 0xc728f436, 88 },
    { 0xb808f439, 90 },
    { 0x3198f43a, 23 },
    { 0x6758f43a, 75 },
    { 0x24c8f43c, 95 },
    { 0xb628f43c, 90 },
    { 0xdf38f43e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3368f441, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef8f445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5508f44b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa8f44e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f8f458, 85 },
    { 0x70a8f459, 91 },
    { 0, 0 },
    { 0x5c58f45b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc8f45e, 80 },
    { 0x2fe8f45f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a8f466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c8f46c, 17 },
    { 0x808f46c, 61 },
    { 0x48d8f46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf8f472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff8f475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d28f478, 90 },
    { 0x5e38f479, 98 },
    { 0x2a8f47a, 76 },
    { 0xe5b8f47b, 77 },
    { 0, 0 },
    { 0x30b8f47d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28f485, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c8f48b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98f48e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f38f493, 98 },
    { 0xde18f494, 90 },
    { 0, 0 },
    { 0xb028f496, 95 },
    { 0xd1d8f496, 23 },
    { 0x6c78f496, 33 },
    { 0, 0 },
    { 0xf478f49a, 34 },
    { 0xf6b8f49b, 61 },
    { 0xc88f49c, 98 },
    { 0x1b38f49d, 33 },
    { 0xe108f49c, 85 },
    { 0xe168f49f, 55 },
    { 0x678f4a0, 99 },
    { 0x8518f49f, 66 },
    { 0xd468f49c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4318f4aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8f4ae, 55 },
    { 0x7d78f4af, 23 },
    { 0x2a78f4b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6008f4b5, 61 },
    { 0, 0 },
    { 0xd818f4b7, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a8f4ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b8f4bd, 34 },
    { 0, 0 },
    { 0x398f4bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a8f4c2, 90 },
    { 0xdc08f4c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f8f4ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5438f4cd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe468f4d0, 67 },
    { 0x4a48f4d1, 34 },
    { 0x1cd8f4d2, 100 },
    { 0x3db8f4d3, 55 },
    { 0, 0 },
    { 0x4ca8f4d5, 78 },
    { 0xf168f4d6, 90 },
    { 0x3a98f4d7, 38 },
    { 0, 0 },
    { 0x2a28f4d9, 98 },
    { 0x58f8f4da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd8f4e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b8f4e6, 98 },
    { 0, 0 },
    { 0x4698f4e8, 98 },
    { 0xd268f4e9, 43 },
    { 0xf6d8f4e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8f4f1, 85 },
    { 0x1088f4f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b08f4f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4938f4fa, 33 },
    { 0, 0 },
    { 0xe098f4fc, 23 },
    { 0x9678f4fd, 90 },
    { 0xaad8f4fd, 90 },
    { 0xff58f4ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb8f50b, 85 },
    { 0, 0 },
    { 0xed38f50d, 95 },
    { 0xb4a8f50d, 90 },
    { 0x9588f50e, 61 },
    { 0xec48f50f, 61 },
    { 0x22b8f50e, 93 },
    { 0xc9f8f512, 84 },
    { 0xb328f513, 23 },
    { 0xcef8f512, 95 },
    { 0xe858f50e, 33 },
    { 0x448f510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c8f51b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde18f524, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b8f52a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d8f52d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1308f531, 85 },
    { 0, 0 },
    { 0xf0a8f533, 23 },
    { 0xf998f533, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3498f539, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f545, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc8f549, 65 },
    { 0x5628f54a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8f54e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de8f555, 60 },
    { 0xa8b8f556, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da8f55c, 33 },
    { 0, 0 },
    { 0xf488f55e, 75 },
    { 0x4f28f55f, 33 },
    { 0xe578f560, 23 },
    { 0x8208f55f, 98 },
    { 0x2048f562, 23 },
    { 0x3fb8f562, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1318f570, 87 },
    { 0, 0 },
    { 0xac38f572, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8468f576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6228f57d, 17 },
    { 0xc5b8f57d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5838f585, 78 },
    { 0x68c8f585, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3098f58c, 90 },
    { 0xad78f58c, 95 },
    { 0xe8f8f58e, 98 },
    { 0x2d58f58e, 87 },
    { 0, 0 },
    { 0x66a8f591, 98 },
    { 0, 0 },
    { 0x95d8f593, 93 },
    { 0, 0 },
    { 0x4428f595, 76 },
    { 0x1518f596, 33 },
    { 0x9168f597, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8f59b, 33 },
    { 0x818f59c, 98 },
    { 0xaa48f59d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f8f5a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa8f5a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d8f5aa, 33 },
    { 0, 0 },
    { 0x6678f5ac, 46 },
    { 0, 0 },
    { 0xeee8f5ae, 61 },
    { 0xe1e8f5af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd8f5b3, 99 },
    { 0xc6f8f5b4, 55 },
    { 0xd918f5b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d38f5b8, 33 },
    { 0, 0 },
    { 0x1f28f5ba, 98 },
    { 0x7a38f5ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa568f5be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5508f5c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a8f5d0, 84 },
    { 0xb718f5d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b8f5d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a8f5dd, 85 },
    { 0x6418f5dd, 46 },
    { 0xbde8f5df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88f5e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f8f5e5, 76 },
    { 0xc018f5e6, 76 },
    { 0xfe38f5e6, 34 },
    { 0x9178f5e8, 90 },
    { 0x3288f5e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1588f5f5, 23 },
    { 0, 0 },
    { 0xae78f5f7, 98 },
    { 0xcd48f5f8, 90 },
    { 0, 0 },
    { 0x5d38f5fa, 95 },
    { 0, 0 },
    { 0xd4a8f5fc, 61 },
    { 0, 0 },
    { 0xa838f5fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea98f601, 85 },
    { 0x7798f601, 98 },
    { 0xbf68f601, 93 },
    { 0, 0 },
    { 0xe878f605, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd568f609, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c88f60f, 33 },
    { 0x9ff8f610, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f614, 36 },
    { 0x38f8f615, 95 },
    { 0xa988f615, 33 },
    { 0, 0 },
    { 0xa0b8f618, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8f627, 98 },
    { 0x8468f628, 76 },
    { 0x7598f629, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b8f62c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f635, 33 },
    { 0xb498f636, 98 },
    { 0, 0 },
    { 0x4ab8f638, 23 },
    { 0x89d8f639, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b18f642, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b8f648, 40 },
    { 0, 0 },
    { 0xe228f64a, 95 },
    { 0, 0 },
    { 0x53e8f64c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f08f652, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde78f659, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f8f660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb418f663, 95 },
    { 0, 0 },
    { 0x5248f665, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f18f668, 78 },
    { 0xfa38f669, 91 },
    { 0xaba8f669, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4228f66d, 90 },
    { 0xec68f66d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5538f673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea8f67b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a8f67f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe618f683, 61 },
    { 0, 0 },
    { 0xb1e8f685, 69 },
    { 0x51a8f686, 90 },
    { 0xfa48f686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8428f690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb308f696, 61 },
    { 0, 0 },
    { 0x35b8f698, 33 },
    { 0x98c8f699, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8848f6a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf178f6a7, 77 },
    { 0, 0 },
    { 0xefc8f6a9, 55 },
    { 0, 0 },
    { 0xa758f6ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd888f6af, 52 },
    { 0, 0 },
    { 0xd1b8f6b1, 98 },
    { 0x18b8f6b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4178f6b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e8f6b9, 98 },
    { 0xf188f6ba, 33 },
    { 0xf338f6b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec8f6bf, 90 },
    { 0x5e18f6bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8f6c4, 42 },
    { 0, 0 },
    { 0x2748f6c6, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9848f6cb, 23 },
    { 0, 0 },
    { 0x628f6cd, 76 },
    { 0, 0 },
    { 0xd828f6cf, 33 },
    { 0x2fc8f6cf, 75 },
    { 0xb708f6cf, 61 },
    { 0, 0 },
    { 0x7cb8f6d3, 66 },
    { 0x5e78f6d3, 60 },
    { 0, 0 },
    { 0x30e8f6d6, 76 },
    { 0x6d88f6d7, 23 },
    { 0x87e8f6d8, 90 },
    { 0x5478f6d9, 33 },
    { 0, 0 },
    { 0xcea8f6db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3058f6df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb968f6e3, 55 },
    { 0x4638f6e4, 85 },
    { 0xf1a8f6e5, 34 },
    { 0x8888f6e5, 33 },
    { 0xe818f6e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc8f6f0, 95 },
    { 0xbf78f6f0, 99 },
    { 0xb2b8f6f0, 43 },
    { 0x7da8f6f1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc798f6f7, 34 },
    { 0x87d8f6f8, 33 },
    { 0xece8f6f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed8f700, 33 },
    { 0, 0 },
    { 0x5198f702, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d98f707, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b8f70c, 61 },
    { 0x47f8f70c, 95 },
    { 0x2388f70e, 90 },
    { 0xc918f70c, 38 },
    { 0xdfe8f710, 55 },
    { 0, 0 },
    { 0x49f8f712, 52 },
    { 0x9c8f713, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a8f716, 55 },
    { 0x23c8f717, 61 },
    { 0xfce8f716, 95 },
    { 0x7e48f718, 95 },
    { 0x3268f71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4678f71e, 98 },
    { 0xede8f71f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b28f722, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc98f725, 90 },
    { 0x9198f725, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b68f72a, 76 },
    { 0xd38f72b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c8f72f, 34 },
    { 0x81b8f730, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c8f739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5168f73f, 33 },
    { 0xc578f73f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5888f745, 98 },
    { 0x8dc8f746, 61 },
    { 0, 0 },
    { 0xfa58f748, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a8f74b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e8f74e, 34 },
    { 0, 0 },
    { 0x2558f750, 46 },
    { 0x6248f750, 90 },
    { 0xcc18f750, 90 },
    { 0x9ac8f750, 95 },
    { 0xc348f754, 23 },
    { 0x2398f750, 76 },
    { 0x1de8f752, 61 },
    { 0x2248f753, 78 },
    { 0xa9a8f758, 85 },
    { 0, 0 },
    { 0x2e18f75a, 66 },
    { 0x5ec8f75a, 43 },
    { 0x61c8f75c, 87 },
    { 0, 0 },
    { 0xd018f75e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08f766, 33 },
    { 0x6d78f767, 78 },
    { 0x3e08f766, 61 },
    { 0x5b48f769, 90 },
    { 0x18a8f768, 90 },
    { 0x2b8f768, 95 },
    { 0x8f76b, 61 },
    { 0xdab8f76d, 43 },
    { 0xafa8f769, 23 },
    { 0x4c08f768, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d8f773, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c78f776, 87 },
    { 0, 0 },
    { 0x44c8f778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd48f77e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b8f785, 43 },
    { 0x9a98f786, 33 },
    { 0x8ed8f786, 80 },
    { 0, 0 },
    { 0x8278f789, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8f78e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b98f791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e88f797, 17 },
    { 0x7578f798, 33 },
    { 0x2708f799, 90 },
    { 0x48e8f797, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4928f79e, 90 },
    { 0xb2a8f79f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c88f7a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98f7a7, 66 },
    { 0, 0 },
    { 0x3f48f7a9, 91 },
    { 0x1328f7aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcc8f7b7, 95 },
    { 0xb9f8f7b7, 98 },
    { 0, 0 },
    { 0xb9d8f7ba, 90 },
    { 0, 0 },
    { 0x1c18f7bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f38f7c1, 23 },
    { 0, 0 },
    { 0xea08f7c3, 61 },
    { 0, 0 },
    { 0x1db8f7c5, 52 },
    { 0, 0 },
    { 0x1f58f7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b8f7cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9188f7d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2628f7d8, 92 },
    { 0, 0 },
    { 0xa268f7da, 85 },
    { 0x2be8f7db, 66 },
    { 0x4b48f7dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab28f7e1, 85 },
    { 0x7988f7e2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd928f7e7, 84 },
    { 0, 0 },
    { 0x9cd8f7e9, 23 },
    { 0x8b38f7e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9848f7ef, 95 },
    { 0, 0 },
    { 0x3af8f7f1, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd78f7f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a8f7f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2438f7fb, 46 },
    { 0, 0 },
    { 0x26b8f7fd, 76 },
    { 0, 0 },
    { 0x6608f7ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea8f803, 99 },
    { 0, 0 },
    { 0x9b38f805, 76 },
    { 0xfed8f806, 33 },
    { 0x8ad8f807, 76 },
    { 0, 0 },
    { 0xe9a8f809, 66 },
    { 0xc2a8f80a, 33 },
    { 0xbb48f80b, 68 },
    { 0, 0 },
    { 0x9338f80d, 55 },
    { 0x9108f80e, 52 },
    { 0xdb88f80e, 61 },
    { 0x1f78f80f, 55 },
    { 0x8378f810, 84 },
    { 0x6e88f812, 76 },
    { 0xb78f813, 33 },
    { 0x1dd8f80d, 90 },
    { 0xa188f812, 100 },
    { 0, 0 },
    { 0xf9f8f817, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e8f81a, 77 },
    { 0xb448f81b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5938f820, 43 },
    { 0, 0 },
    { 0xc608f822, 65 },
    { 0, 0 },
    { 0x2378f824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf718f82e, 95 },
    { 0, 0 },
    { 0x3be8f830, 84 },
    { 0x96a8f831, 77 },
    { 0xaf98f831, 33 },
    { 0x3c18f833, 30 },
    { 0, 0 },
    { 0xce78f835, 33 },
    { 0x9268f836, 84 },
    { 0xed88f836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b8f83d, 33 },
    { 0xbef8f83e, 43 },
    { 0x7258f83f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d88f844, 87 },
    { 0x1e48f844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1468f84a, 34 },
    { 0, 0 },
    { 0x2738f84c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e48f851, 93 },
    { 0x3538f852, 23 },
    { 0x9b28f852, 95 },
    { 0x7b08f854, 65 },
    { 0x9ad8f854, 95 },
    { 0x83f8f852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1478f85a, 61 },
    { 0, 0 },
    { 0x39a8f85c, 23 },
    { 0x1418f85c, 33 },
    { 0xa3a8f85c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5968f863, 98 },
    { 0, 0 },
    { 0xfcd8f865, 98 },
    { 0x6798f866, 40 },
    { 0x4aa8f866, 95 },
    { 0xe2f8f868, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648f86c, 33 },
    { 0, 0 },
    { 0x3808f86e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc38f871, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b08f876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc8f87a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd058f87f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8f887, 88 },
    { 0x3e68f888, 99 },
    { 0, 0 },
    { 0x98f8f88a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa588f890, 43 },
    { 0x47f8f890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b88f897, 100 },
    { 0, 0 },
    { 0xc1b8f899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28f89f, 41 },
    { 0x40b8f8a0, 43 },
    { 0xc428f8a0, 46 },
    { 0, 0 },
    { 0x228f8a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2438f8a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb8f8b1, 61 },
    { 0x538f8b2, 67 },
    { 0xba08f8b1, 76 },
    { 0xd4c8f8b4, 23 },
    { 0, 0 },
    { 0xec58f8b6, 69 },
    { 0xdf8f8b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2368f8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7898f8c0, 61 },
    { 0x8dc8f8c0, 98 },
    { 0x22e8f8c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac8f8c5, 93 },
    { 0xe988f8c6, 90 },
    { 0xed88f8c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c8f8cd, 90 },
    { 0xad98f8ce, 55 },
    { 0xe88f8cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e18f8d7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5538f8db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa648f8de, 55 },
    { 0xa058f8de, 98 },
    { 0x92f8f8de, 84 },
    { 0xa338f8e1, 23 },
    { 0x1008f8df, 33 },
    { 0x1fd8f8e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb28f8e7, 26 },
    { 0x4d38f8e8, 61 },
    { 0xe1b8f8e7, 61 },
    { 0x8fe8f8e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8988f8f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a8f8fd, 100 },
    { 0x35f8f8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de8f902, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3868f907, 98 },
    { 0x5918f907, 67 },
    { 0xbb88f909, 33 },
    { 0x41a8f909, 61 },
    { 0x2d18f909, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8418f910, 90 },
    { 0, 0 },
    { 0x4c18f912, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe68f915, 95 },
    { 0x3e8f916, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba8f919, 100 },
    { 0x82e8f91a, 36 },
    { 0x5c68f91b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce28f921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6868f926, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc818f92e, 90 },
    { 0x5dc8f92e, 61 },
    { 0x4698f92f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f8f934, 46 },
    { 0x9d68f934, 61 },
    { 0x1ad8f936, 87 },
    { 0x10f8f937, 81 },
    { 0x25f8f936, 93 },
    { 0x1c28f939, 90 },
    { 0x54a8f93a, 19 },
    { 0x83c8f936, 66 },
    { 0xb618f93c, 76 },
    { 0x8988f93d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf288f943, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f68f948, 88 },
    { 0, 0 },
    { 0xc238f94a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a8f94f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xea28f952, 61 },
    { 0, 0 },
    { 0x64b8f954, 47 },
    { 0, 0 },
    { 0x778f956, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa648f959, 90 },
    { 0xded8f95a, 46 },
    { 0xeb38f95b, 98 },
    { 0x85f8f95b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa08f961, 90 },
    { 0, 0 },
    { 0xd0b8f963, 23 },
    { 0xee48f964, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa708f967, 95 },
    { 0, 0 },
    { 0xff68f969, 76 },
    { 0, 0 },
    { 0x2298f96b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f58f96e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8f973, 23 },
    { 0xcf8f974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2418f983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d28f986, 77 },
    { 0xba58f987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1868f98a, 95 },
    { 0x73f8f98b, 33 },
    { 0x7118f98c, 38 },
    { 0x7d08f98a, 99 },
    { 0xcf08f98a, 98 },
    { 0x1728f98d, 34 },
    { 0xd6c8f990, 23 },
    { 0xc068f991, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc638f994, 55 },
    { 0, 0 },
    { 0x87f8f996, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3128f999, 87 },
    { 0x7088f99a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8748f99d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1508f9a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a48f9a4, 34 },
    { 0, 0 },
    { 0xcaf8f9a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c8f9a9, 61 },
    { 0x5758f9aa, 98 },
    { 0x12c8f9aa, 76 },
    { 0, 0 },
    { 0xf4c8f9ad, 60 },
    { 0xdbb8f9ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c8f9b2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d8f9bf, 33 },
    { 0x3168f9bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff8f9c7, 44 },
    { 0, 0 },
    { 0x8fe8f9c9, 61 },
    { 0x18e8f9ca, 98 },
    { 0x2b88f9ca, 95 },
    { 0, 0 },
    { 0x7298f9cd, 98 },
    { 0x6688f9ce, 76 },
    { 0x7c08f9cd, 93 },
    { 0, 0 },
    { 0x9ee8f9d1, 43 },
    { 0x2db8f9d1, 100 },
    { 0x40c8f9d3, 98 },
    { 0x18b8f9d3, 34 },
    { 0, 0 },
    { 0x4868f9d6, 90 },
    { 0xd468f9d7, 40 },
    { 0x9398f9d8, 23 },
    { 0x5178f9d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d28f9dd, 33 },
    { 0x3428f9de, 55 },
    { 0x7d98f9df, 55 },
    { 0x4218f9dd, 43 },
    { 0xe9d8f9e1, 66 },
    { 0x3868f9e2, 98 },
    { 0x3ac8f9e3, 55 },
    { 0, 0 },
    { 0x4458f9e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3648f9e8, 23 },
    { 0, 0 },
    { 0xb998f9ea, 68 },
    { 0, 0 },
    { 0xf268f9ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b8f9f4, 90 },
    { 0x508f9f5, 77 },
    { 0xc958f9f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8f9fa, 95 },
    { 0xc048f9fb, 33 },
    { 0xb308f9fc, 46 },
    { 0xe488f9fc, 34 },
    { 0, 0 },
    { 0x51d8f9ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b8fa03, 95 },
    { 0x3d98fa03, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e8fa08, 90 },
    { 0xcee8fa09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa38fa0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fa11, 81 },
    { 0x11d8fa11, 90 },
    { 0x5be8fa12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1888fa1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f8fa1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f8fa22, 93 },
    { 0x6da8fa23, 90 },
    { 0, 0 },
    { 0x3b88fa25, 78 },
    { 0x4b08fa25, 23 },
    { 0x3ce8fa25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff8fa2b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa438fa2e, 23 },
    { 0, 0 },
    { 0x8e88fa30, 91 },
    { 0, 0 },
    { 0x5ac8fa32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef8fa36, 53 },
    { 0x9fe8fa36, 31 },
    { 0xab98fa37, 61 },
    { 0x58a8fa39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8428fa3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5308fa42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e8fa47, 85 },
    { 0xeb18fa48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdc8fa4d, 90 },
    { 0, 0 },
    { 0x46a8fa4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8fa52, 23 },
    { 0x58a8fa53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e8fa5a, 91 },
    { 0x45a8fa5a, 76 },
    { 0, 0 },
    { 0xf4c8fa5d, 23 },
    { 0x4ea8fa5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac88fa61, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf98fa67, 90 },
    { 0x23a8fa68, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7418fa6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e38fa74, 99 },
    { 0xace8fa75, 52 },
    { 0x7d8fa76, 78 },
    { 0, 0 },
    { 0x4898fa78, 33 },
    { 0, 0 },
    { 0x9dd8fa7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f58fa7e, 95 },
    { 0xf998fa7e, 23 },
    { 0x9c88fa7f, 90 },
    { 0xe948fa7e, 66 },
    { 0x6b8fa82, 34 },
    { 0xfee8fa81, 33 },
    { 0xf3d8fa84, 33 },
    { 0xbf38fa85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef8fa8b, 33 },
    { 0xa198fa8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b58fa92, 76 },
    { 0xef18fa93, 95 },
    { 0xabf8fa93, 61 },
    { 0x7bf8fa94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd18fa99, 87 },
    { 0, 0 },
    { 0x6c98fa9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3538fa9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5858faa2, 17 },
    { 0x6ad8faa3, 33 },
    { 0xdb18faa4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb58faa8, 43 },
    { 0xb558faa9, 94 },
    { 0, 0 },
    { 0xf398faab, 34 },
    { 0x5ea8faac, 61 },
    { 0x1178faad, 95 },
    { 0, 0 },
    { 0xf008faaf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f8fab2, 98 },
    { 0x6638fab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a48fab6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb058fabd, 66 },
    { 0xef68fabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b68fac4, 64 },
    { 0, 0 },
    { 0xd0a8fac6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d8faca, 61 },
    { 0x6878facb, 61 },
    { 0x4c08facc, 56 },
    { 0x1d98facc, 80 },
    { 0xf908face, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8fad1, 101 },
    { 0x2988fad2, 55 },
    { 0xa8f8fad2, 23 },
    { 0x5ba8fad3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d48fad8, 53 },
    { 0xefa8fad9, 90 },
    { 0x9708fad9, 46 },
    { 0xfde8fadb, 90 },
    { 0x7838fadb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a8fae1, 61 },
    { 0x86b8fae1, 33 },
    { 0x58fae3, 95 },
    { 0xcd28fae4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b8fae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4af8faed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3098faf2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x2678faf5, 90 },
    { 0x2668faf6, 33 },
    { 0x7e68faf5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8318fafb, 90 },
    { 0x8208fafc, 87 },
    { 0x7f08fafd, 100 },
    { 0, 0 },
    { 0x6d48faff, 90 },
    { 0, 0 },
    { 0x1df8fb01, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d8fb04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe468fb0b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe358fb0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9708fb13, 75 },
    { 0, 0 },
    { 0xa8c8fb15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d08fb1a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a8fb1e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d48fb22, 43 },
    { 0x4e98fb23, 85 },
    { 0, 0 },
    { 0x4c38fb25, 33 },
    { 0x96e8fb25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce8fb2e, 4 },
    { 0x1168fb2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168fb35, 23 },
    { 0xe418fb36, 33 },
    { 0x5dc8fb35, 90 },
    { 0xc768fb38, 90 },
    { 0, 0 },
    { 0x2278fb3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc8fb40, 46 },
    { 0x5278fb41, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc38fb53, 55 },
    { 0x19d8fb53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf88fb57, 67 },
    { 0x95f8fb58, 33 },
    { 0xab08fb59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fb61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xde08fb64, 90 },
    { 0xebd8fb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e8fb71, 90 },
    { 0xdb08fb72, 34 },
    { 0x74b8fb73, 98 },
    { 0xca18fb71, 61 },
    { 0x1f28fb75, 55 },
    { 0xa9e8fb75, 90 },
    { 0xdd78fb77, 90 },
    { 0x4118fb76, 33 },
    { 0x3a08fb79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc078fb7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad08fb80, 96 },
    { 0, 0 },
    { 0x5308fb82, 91 },
    { 0xaad8fb83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a78fb87, 90 },
    { 0x3558fb87, 55 },
    { 0xde58fb89, 90 },
    { 0, 0 },
    { 0xa978fb8b, 69 },
    { 0x9938fb8c, 76 },
    { 0x6dc8fb8b, 90 },
    { 0xf458fb8e, 46 },
    { 0xac98fb8d, 69 },
    { 0x2588fb90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e48fb9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3838fba3, 61 },
    { 0x8f58fba3, 33 },
    { 0xc438fba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d8fbad, 67 },
    { 0, 0 },
    { 0xe508fbaf, 61 },
    { 0x7f98fbaf, 90 },
    { 0xdf58fbb1, 90 },
    { 0xcdf8fbb2, 77 },
    { 0xa978fbb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de8fbbc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a8fbc0, 33 },
    { 0xf588fbc1, 93 },
    { 0x7f38fbc2, 43 },
    { 0x4418fbc1, 95 },
    { 0xe108fbc0, 95 },
    { 0x2278fbc0, 61 },
    { 0x2798fbc6, 87 },
    { 0xe998fbc7, 33 },
    { 0, 0 },
    { 0xfb28fbc9, 55 },
    { 0x9fb8fbca, 85 },
    { 0x59b8fbcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae18fbce, 52 },
    { 0xe18fbce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b38fbd2, 88 },
    { 0x6d18fbd3, 93 },
    { 0x3158fbd3, 90 },
    { 0xd218fbd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6678fbd8, 61 },
    { 0x518fbd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb348fbdf, 33 },
    { 0xf008fbe0, 61 },
    { 0xa078fbe1, 98 },
    { 0x4528fbdf, 93 },
    { 0xf748fbe3, 36 },
    { 0xd488fbe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e8fbec, 61 },
    { 0x5708fbed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4288fbf0, 85 },
    { 0x9cd8fbf0, 98 },
    { 0x1b8fbf0, 33 },
    { 0, 0 },
    { 0x4cf8fbf4, 61 },
    { 0x1ef8fbf5, 100 },
    { 0x7458fbf6, 33 },
    { 0x55d8fbf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e8fbfb, 76 },
    { 0xdc98fbfb, 33 },
    { 0x178fbfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf298fc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd928fc09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f08fc11, 33 },
    { 0xf568fc12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f8fc15, 98 },
    { 0xda58fc16, 98 },
    { 0x99b8fc17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed8fc1a, 98 },
    { 0, 0 },
    { 0xce18fc1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c8fc24, 99 },
    { 0x2c78fc25, 77 },
    { 0, 0 },
    { 0x2e48fc27, 98 },
    { 0xc408fc28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7328fc2f, 90 },
    { 0x73a8fc30, 55 },
    { 0xe768fc31, 61 },
    { 0xef48fc32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf528fc37, 61 },
    { 0x6a18fc38, 100 },
    { 0xd098fc37, 98 },
    { 0xaff8fc38, 95 },
    { 0, 0 },
    { 0x83e8fc3c, 76 },
    { 0, 0 },
    { 0x5c48fc3e, 76 },
    { 0xfae8fc3f, 98 },
    { 0x4988fc3f, 90 },
    { 0xb658fc41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a8fc44, 61 },
    { 0, 0 },
    { 0xf98fc46, 43 },
    { 0xb2f8fc46, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8fc4f, 90 },
    { 0x8e48fc50, 85 },
    { 0xd698fc50, 38 },
    { 0, 0 },
    { 0xdbe8fc53, 85 },
    { 0x6938fc54, 55 },
    { 0x9b88fc54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18fc5b, 67 },
    { 0xbe08fc5c, 69 },
    { 0x4b98fc5d, 23 },
    { 0x2488fc5d, 98 },
    { 0xe998fc5d, 61 },
    { 0x3588fc5b, 93 },
    { 0x43c8fc5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c8fc66, 55 },
    { 0xa368fc67, 33 },
    { 0, 0 },
    { 0x6528fc69, 90 },
    { 0xc5f8fc69, 52 },
    { 0x4228fc6b, 100 },
    { 0x98e8fc6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f8fc72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698fc76, 99 },
    { 0x89b8fc77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fc7a, 33 },
    { 0x6da8fc7b, 98 },
    { 0, 0 },
    { 0xf078fc7d, 87 },
    { 0xead8fc7e, 43 },
    { 0x23d8fc7e, 95 },
    { 0xf2f8fc80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58fc84, 33 },
    { 0xc988fc85, 61 },
    { 0x13c8fc85, 95 },
    { 0xc9d8fc86, 55 },
    { 0x6c48fc84, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8fc8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b8fc8f, 90 },
    { 0xd7d8fc8f, 55 },
    { 0, 0 },
    { 0xfac8fc92, 95 },
    { 0, 0 },
    { 0xf5e8fc94, 83 },
    { 0x6728fc95, 90 },
    { 0xbbf8fc95, 61 },
    { 0, 0 },
    { 0x61a8fc98, 98 },
    { 0x4248fc98, 78 },
    { 0xba78fc9a, 43 },
    { 0, 0 },
    { 0x9348fc9c, 90 },
    { 0x2048fc9d, 23 },
    { 0xb8a8fc9e, 85 },
    { 0x49f8fc9f, 52 },
    { 0xfc68fca0, 76 },
    { 0x4c28fca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb478fcad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee8fcb1, 78 },
    { 0xfbc8fcb1, 90 },
    { 0x82d8fcb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5208fcb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f98fcba, 86 },
    { 0xd508fcba, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae8fcc1, 23 },
    { 0, 0 },
    { 0x4d18fcc3, 61 },
    { 0, 0 },
    { 0xb58fcc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff58fcca, 78 },
    { 0x66f8fccb, 33 },
    { 0xe758fcca, 85 },
    { 0xb958fccb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf8fcd2, 90 },
    { 0x33d8fcd2, 90 },
    { 0xaa18fcd4, 33 },
    { 0xa148fcd4, 23 },
    { 0xf378fcd3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a8fcda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb828fce1, 52 },
    { 0, 0 },
    { 0xe718fce3, 43 },
    { 0, 0 },
    { 0x3198fce5, 93 },
    { 0xbb68fce6, 98 },
    { 0xcef8fce7, 33 },
    { 0xf1a8fce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe8fceb, 61 },
    { 0xe6d8fcec, 34 },
    { 0x4c48fcec, 61 },
    { 0xafa8fcee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3348fcf1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf8fcf8, 33 },
    { 0, 0 },
    { 0xdf28fcfa, 98 },
    { 0xdb58fcfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d8fcfe, 69 },
    { 0, 0 },
    { 0x4e08fd00, 76 },
    { 0xd08fd01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4648fd0b, 77 },
    { 0, 0 },
    { 0xb3c8fd0d, 76 },
    { 0xcbc8fd0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc158fd11, 90 },
    { 0xe1d8fd12, 33 },
    { 0xf648fd11, 98 },
    { 0, 0 },
    { 0xc0f8fd15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe388fd19, 100 },
    { 0x84b8fd1a, 95 },
    { 0x9a8fd1b, 67 },
    { 0xe2a8fd1c, 92 },
    { 0, 0 },
    { 0x69f8fd1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b8fd23, 88 },
    { 0, 0 },
    { 0xeb18fd25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a38fd2a, 55 },
    { 0, 0 },
    { 0xf6d8fd2c, 66 },
    { 0x5458fd2d, 90 },
    { 0, 0 },
    { 0x3438fd2f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf348fd3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998fd40, 34 },
    { 0x5e38fd41, 98 },
    { 0, 0 },
    { 0xad68fd43, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e08fd4e, 23 },
    { 0, 0 },
    { 0x59b8fd50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca58fd57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6528fd5e, 52 },
    { 0x23e8fd5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab8fd6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d8fd6d, 33 },
    { 0xb848fd6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b8fd71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38fd76, 61 },
    { 0x6108fd77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa8fd7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb28fd80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4558fd8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x858fd8d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd898fd91, 95 },
    { 0, 0 },
    { 0x18e8fd93, 46 },
    { 0x6818fd94, 65 },
    { 0x9a78fd95, 23 },
    { 0xed8fd95, 23 },
    { 0xbf88fd96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd28fd9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe8fd9d, 90 },
    { 0x3f38fd9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3198fda2, 33 },
    { 0xf98fda3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c68fda7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf928fdae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb8fdb2, 95 },
    { 0x6ca8fdb3, 76 },
    { 0x12c8fdb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2308fdb8, 90 },
    { 0, 0 },
    { 0xfb88fdba, 90 },
    { 0x1ac8fdba, 76 },
    { 0x37b8fdbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e08fdc0, 61 },
    { 0xb2b8fdc1, 61 },
    { 0xc988fdc0, 95 },
    { 0x9a08fdc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc8fdc9, 61 },
    { 0xc7a8fdca, 33 },
    { 0xf968fdc9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe368fdce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a8fdd2, 93 },
    { 0x6528fdd3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4628fdd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a8fde0, 87 },
    { 0x558fde0, 23 },
    { 0x6cc8fde2, 98 },
    { 0x7638fde3, 33 },
    { 0x73a8fde2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc218fde8, 33 },
    { 0x9a78fde9, 76 },
    { 0x14c8fde8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3788fded, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a8fdf0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c8fdf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7558fe02, 55 },
    { 0x8f8fe03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7608fe0a, 55 },
    { 0xa058fe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f88fe0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a38fe13, 36 },
    { 0x18a8fe14, 33 },
    { 0x2088fe15, 78 },
    { 0x5818fe13, 65 },
    { 0xbb18fe15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf128fe1a, 33 },
    { 0x2398fe1b, 95 },
    { 0x7dc8fe1c, 33 },
    { 0x4ba8fe1a, 95 },
    { 0x5e38fe1e, 23 },
    { 0x17b8fe1e, 90 },
    { 0x5de8fe20, 61 },
    { 0x75b8fe21, 66 },
    { 0x6b08fe21, 55 },
    { 0x4568fe22, 43 },
    { 0x4488fe24, 55 },
    { 0, 0 },
    { 0x1d98fe26, 98 },
    { 0, 0 },
    { 0x5d58fe28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f88fe2d, 90 },
    { 0x4a18fe2d, 84 },
    { 0xacb8fe2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8fe34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1828fe3d, 99 },
    { 0, 0 },
    { 0xf208fe3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf968fe42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa958fe47, 76 },
    { 0xab28fe48, 88 },
    { 0, 0 },
    { 0xdd78fe4a, 84 },
    { 0x3d88fe4b, 23 },
    { 0xe8a8fe4c, 95 },
    { 0x248fe4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac18fe50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08fe55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c8fe59, 61 },
    { 0, 0 },
    { 0x7388fe5b, 28 },
    { 0x2258fe5b, 46 },
    { 0x40e8fe5d, 98 },
    { 0xeb78fe5d, 98 },
    { 0, 0 },
    { 0xf928fe60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc8fe67, 34 },
    { 0x8028fe67, 17 },
    { 0xc18fe69, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3328fe79, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe888fe7f, 46 },
    { 0xe8c8fe7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fe85, 76 },
    { 0x8f98fe86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c8fe8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa638fe92, 90 },
    { 0x1358fe93, 55 },
    { 0, 0 },
    { 0x7428fe95, 61 },
    { 0x51b8fe95, 61 },
    { 0xdf08fe97, 52 },
    { 0x15b8fe95, 95 },
    { 0xe4d8fe99, 96 },
    { 0x8bb8fe96, 90 },
    { 0, 0 },
    { 0x5648fe9c, 81 },
    { 0xe408fe9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c28fea1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2798fea6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c8fea9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe008fead, 33 },
    { 0, 0 },
    { 0x8368feaf, 95 },
    { 0, 0 },
    { 0x6ae8feb1, 61 },
    { 0xfff8feb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb508feb6, 78 },
    { 0, 0 },
    { 0x9078feb8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb438febc, 66 },
    { 0x95b8febd, 23 },
    { 0xbb78febe, 76 },
    { 0, 0 },
    { 0x3778fec0, 33 },
    { 0, 0 },
    { 0x4618fec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e98fec7, 98 },
    { 0xee38fec8, 33 },
    { 0, 0 },
    { 0x12a8feca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b8fecd, 61 },
    { 0x9378fece, 55 },
    { 0x2e28fecf, 90 },
    { 0xcf58fecf, 46 },
    { 0x5f58fed0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc028fed6, 17 },
    { 0xa48fed6, 90 },
    { 0x1b78fed8, 76 },
    { 0x8408fed9, 17 },
    { 0x3fe8feda, 64 },
    { 0x3638fed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b78fee0, 43 },
    { 0xd208fee1, 85 },
    { 0, 0 },
    { 0xb218fee3, 76 },
    { 0x8d18fee4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf858feea, 23 },
    { 0xca48feea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf618fef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x968fef6, 90 },
    { 0x3298fef7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e8fefa, 98 },
    { 0, 0 },
    { 0x3738fefc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa668ff03, 90 },
    { 0xfd8ff03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa8ff07, 23 },
    { 0, 0 },
    { 0xb428ff09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c98ff0c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c8ff12, 98 },
    { 0, 0 },
    { 0x7248ff14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f8ff1a, 55 },
    { 0x8e8ff1b, 23 },
    { 0x37f8ff1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be8ff26, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb248ff29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af8ff2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe948ff32, 98 },
    { 0x5028ff33, 76 },
    { 0x7ae8ff32, 55 },
    { 0x15b8ff35, 69 },
    { 0xdfc8ff35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb8ff39, 99 },
    { 0xd488ff3a, 99 },
    { 0x85a8ff3b, 100 },
    { 0xf128ff3c, 95 },
    { 0xaae8ff3d, 33 },
    { 0, 0 },
    { 0x7688ff3f, 95 },
    { 0, 0 },
    { 0x4a48ff41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4798ff45, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a8ff4c, 98 },
    { 0x9a8ff4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e28ff50, 23 },
    { 0x1638ff51, 61 },
    { 0xa9b8ff50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2598ff5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f8ff62, 55 },
    { 0x65b8ff63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a8ff67, 98 },
    { 0x9ea8ff68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8ff70, 76 },
    { 0x14d8ff70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f18ff76, 43 },
    { 0x7118ff77, 99 },
    { 0x4bd8ff78, 61 },
    { 0, 0 },
    { 0x8018ff7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a8ff7f, 17 },
    { 0, 0 },
    { 0x16e8ff81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee8ff88, 23 },
    { 0x90b8ff88, 59 },
    { 0x3fc8ff8a, 52 },
    { 0x19d8ff8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6188ff90, 95 },
    { 0x2448ff91, 98 },
    { 0x7598ff92, 77 },
    { 0x7c28ff93, 34 },
    { 0x9c88ff91, 76 },
    { 0xe2e8ff91, 94 },
    { 0xa9c8ff96, 95 },
    { 0xd8f8ff90, 85 },
    { 0, 0 },
    { 0xb4d8ff99, 90 },
    { 0, 0 },
    { 0xc88ff9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b88ffa9, 95 },
    { 0xe8f8ffaa, 92 },
    { 0x93e8ffab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7248ffaf, 90 },
    { 0, 0 },
    { 0x4f78ffb1, 85 },
    { 0x7b28ffb1, 33 },
    { 0xa128ffb2, 23 },
    { 0, 0 },
    { 0xd248ffb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc158ffb8, 55 },
    { 0xe4f8ffb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac98ffbc, 95 },
    { 0, 0 },
    { 0x6e28ffbe, 34 },
    { 0xf308ffbf, 61 },
    { 0x3108ffc0, 23 },
    { 0x2338ffc0, 36 },
    { 0x7948ffc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a8ffc6, 66 },
    { 0x1b88ffc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc08ffcb, 61 },
    { 0x5898ffcc, 83 },
    { 0, 0 },
    { 0xa6c8ffce, 78 },
    { 0x1d88ffce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8ffd2, 76 },
    { 0xf778ffd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a8ffd6, 88 },
    { 0x4b78ffd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5408ffde, 100 },
    { 0x5238ffdf, 67 },
    { 0xf448ffe0, 77 },
    { 0x3838ffe1, 98 },
    { 0, 0 },
    { 0xb488ffe3, 87 },
    { 0x7a8ffe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a8ffe9, 76 },
    { 0xe98ffea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca8ffed, 61 },
    { 0x42f8ffee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a58fff5, 23 },
    { 0, 0 },
    { 0x59d8fff7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xad78fffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc8fffe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x70490001, 23 },
    { 0, 0 },
    { 0x4c90003, 76 },
    { 0, 0 },
    { 0xfac90005, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f9000c, 80 },
    { 0xe999000c, 91 },
    { 0x79c9000e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f190011, 61 },
    { 0x68c90012, 93 },
    { 0xe0e90013, 43 },
    { 0x40990013, 55 },
    { 0x7ad90014, 23 },
    { 0x5190016, 95 },
    { 0x5ae90017, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89001a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a9001e, 61 },
    { 0xba79001e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4990022, 81 },
    { 0x2cb90023, 87 },
    { 0x2c590023, 23 },
    { 0xca690023, 33 },
    { 0, 0 },
    { 0x64890027, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x299002a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5659002f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20090032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea190035, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0690039, 90 },
    { 0xa1a9003a, 98 },
    { 0, 0 },
    { 0x2409003c, 76 },
    { 0x1819003d, 91 },
    { 0xa269003c, 90 },
    { 0x46d9003f, 98 },
    { 0xab690040, 52 },
    { 0x24c9003f, 85 },
    { 0xe5a9003f, 90 },
    { 0x48e90043, 33 },
    { 0x46190041, 90 },
    { 0x1b990040, 95 },
    { 0x31d9003e, 55 },
    { 0x18690047, 90 },
    { 0, 0 },
    { 0x40390049, 34 },
    { 0xfa9004a, 90 },
    { 0x9aa90049, 55 },
    { 0, 0 },
    { 0x549004d, 33 },
    { 0, 0 },
    { 0x2ac9004f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc9005b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d39005f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e290063, 95 },
    { 0xe3590064, 90 },
    { 0x33890064, 90 },
    { 0xe9c90066, 55 },
    { 0x12d90064, 99 },
    { 0xe4390068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a9006d, 33 },
    { 0x13b9006e, 93 },
    { 0xd969006f, 34 },
    { 0, 0 },
    { 0x3bf90071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b90074, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b090078, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f9007e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad190082, 33 },
    { 0xaed90083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9990089, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a9008c, 52 },
    { 0x7219008d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7790094, 76 },
    { 0xdff90095, 87 },
    { 0x17890095, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f9009b, 34 },
    { 0x4339009c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e900a0, 96 },
    { 0xc8d900a1, 90 },
    { 0xd7a900a1, 87 },
    { 0xb22900a0, 99 },
    { 0x2f0900a1, 92 },
    { 0xde8900a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd900ab, 90 },
    { 0x83c900ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb900b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a900b5, 61 },
    { 0xc50900b6, 36 },
    { 0, 0 },
    { 0x90e900b8, 81 },
    { 0x7ee900b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x345900bc, 33 },
    { 0x2b2900bc, 67 },
    { 0x8bb900be, 90 },
    { 0x7df900bf, 33 },
    { 0, 0 },
    { 0xbf4900c1, 43 },
    { 0x49c900c1, 40 },
    { 0x7a900c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x289900c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6900ca, 99 },
    { 0x432900cb, 34 },
    { 0x88d900cc, 98 },
    { 0x9f1900cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284900d2, 66 },
    { 0, 0 },
    { 0x381900d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16900d8, 55 },
    { 0xad7900d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d900dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x62900df, 76 },
    { 0x4e8900df, 95 },
    { 0x58a900e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19900e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac900ea, 93 },
    { 0xb9e900eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39900ee, 76 },
    { 0xf2f900ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e900f2, 55 },
    { 0xdf2900f3, 46 },
    { 0, 0 },
    { 0xbb8900f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37900fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc90106, 61 },
    { 0x6d90107, 23 },
    { 0x5d490108, 83 },
    { 0xa4b90109, 98 },
    { 0xe3f9010a, 83 },
    { 0x1aa9010a, 95 },
    { 0xd059010c, 46 },
    { 0x9c49010a, 34 },
    { 0x2629010e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22390112, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd290117, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1569011d, 98 },
    { 0xafd9011e, 30 },
    { 0, 0 },
    { 0x4c290120, 93 },
    { 0x19390121, 90 },
    { 0x3ca90121, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c90125, 61 },
    { 0, 0 },
    { 0x85890127, 33 },
    { 0x8af90127, 34 },
    { 0xba890128, 95 },
    { 0xe009012a, 33 },
    { 0x60a9012b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e490134, 81 },
    { 0x4c290134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40790138, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4749013d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc390141, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef690149, 90 },
    { 0x8ba90149, 90 },
    { 0xe09014b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba99014f, 90 },
    { 0xb8d90150, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16790159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a9015c, 23 },
    { 0xa6b9015c, 23 },
    { 0xe629015c, 81 },
    { 0, 0 },
    { 0x66a90160, 23 },
    { 0x2ad90160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5890166, 33 },
    { 0, 0 },
    { 0x43290168, 95 },
    { 0xbc190168, 91 },
    { 0xd649016a, 55 },
    { 0, 0 },
    { 0x989016c, 95 },
    { 0x4e19016c, 101 },
    { 0xfb39016e, 55 },
    { 0x5929016e, 55 },
    { 0x7a190170, 61 },
    { 0x71490170, 64 },
    { 0x31690170, 61 },
    { 0xd389016e, 93 },
    { 0x31a90174, 33 },
    { 0x30890175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6090178, 33 },
    { 0, 0 },
    { 0x34c9017a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e79017d, 95 },
    { 0, 0 },
    { 0x60f9017f, 61 },
    { 0, 0 },
    { 0xad090181, 76 },
    { 0x86c90182, 87 },
    { 0x2e190183, 23 },
    { 0x32590184, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe689018b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7790191, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d90198, 90 },
    { 0, 0 },
    { 0xc989019a, 61 },
    { 0xe2d9019a, 23 },
    { 0xdd29019a, 85 },
    { 0x2069019b, 95 },
    { 0x7919019e, 43 },
    { 0x6b49019c, 23 },
    { 0xdb7901a0, 55 },
    { 0, 0 },
    { 0x59f901a2, 33 },
    { 0xcaa901a2, 17 },
    { 0x344901a2, 95 },
    { 0x2de901a5, 61 },
    { 0xa2b901a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37901ab, 81 },
    { 0xa41901ac, 45 },
    { 0x809901ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7901b0, 99 },
    { 0xe31901b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4901b5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710901b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae901bd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6901c3, 69 },
    { 0x49d901c4, 98 },
    { 0x886901c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69901c8, 34 },
    { 0xf7e901c9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f901cd, 85 },
    { 0xc6b901ce, 75 },
    { 0x6be901cd, 23 },
    { 0x39c901d0, 95 },
    { 0xe39901d1, 61 },
    { 0, 0 },
    { 0xafc901d3, 67 },
    { 0, 0 },
    { 0xc4e901d5, 33 },
    { 0x9c7901d5, 98 },
    { 0xf66901d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39901da, 84 },
    { 0, 0 },
    { 0xafa901dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3901e1, 33 },
    { 0x892901e2, 88 },
    { 0xde901e2, 43 },
    { 0x43e901e4, 67 },
    { 0xaad901e3, 61 },
    { 0, 0 },
    { 0x512901e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62901eb, 98 },
    { 0xc1f901ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x995901ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc901ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90202, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x46990205, 78 },
    { 0x95590205, 34 },
    { 0x96390207, 91 },
    { 0x3390207, 33 },
    { 0x76590209, 90 },
    { 0x35a9020a, 98 },
    { 0x4509020b, 77 },
    { 0, 0 },
    { 0x3e49020d, 93 },
    { 0x48f9020e, 76 },
    { 0x1099020d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x50590212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9021a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6249021d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e290223, 98 },
    { 0x78390223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae590229, 55 },
    { 0, 0 },
    { 0x8ac9022b, 77 },
    { 0xb659022c, 76 },
    { 0x8669022c, 75 },
    { 0xd619022e, 95 },
    { 0xddc9022e, 23 },
    { 0, 0 },
    { 0xe9990231, 61 },
    { 0x4f590231, 67 },
    { 0x91a90231, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f90236, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b49023e, 46 },
    { 0xde39023e, 87 },
    { 0xb3c9023e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8090243, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa90248, 46 },
    { 0, 0 },
    { 0x3d99024a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68690250, 55 },
    { 0x9c790251, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27190255, 43 },
    { 0xf5990256, 98 },
    { 0x2f990256, 93 },
    { 0, 0 },
    { 0x75690259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa9025d, 34 },
    { 0x8ea9025e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb90261, 66 },
    { 0, 0 },
    { 0x41490263, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f190268, 98 },
    { 0x2b790269, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b9026c, 90 },
    { 0x4ca9026d, 33 },
    { 0x809026e, 95 },
    { 0x9249026f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67690275, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a90279, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6019027c, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9027f, 90 },
    { 0xb8990280, 61 },
    { 0, 0 },
    { 0xcfb90282, 98 },
    { 0xcd390283, 94 },
    { 0x86890284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae90287, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de9028b, 33 },
    { 0x9dd9028b, 55 },
    { 0xee39028d, 34 },
    { 0x26f9028e, 78 },
    { 0xb2c9028c, 36 },
    { 0xc1090290, 34 },
    { 0, 0 },
    { 0xf3b90292, 76 },
    { 0, 0 },
    { 0xe5a90294, 99 },
    { 0, 0 },
    { 0xd890296, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc9029a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e902a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949902a4, 33 },
    { 0, 0 },
    { 0x42f902a6, 66 },
    { 0, 0 },
    { 0x974902a8, 61 },
    { 0x1cc902a9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557902b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99902bd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303902c1, 91 },
    { 0x7b3902c2, 78 },
    { 0, 0 },
    { 0x59a902c4, 33 },
    { 0, 0 },
    { 0xf02902c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x128902c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7902d3, 33 },
    { 0, 0 },
    { 0x683902d5, 65 },
    { 0xa9c902d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7902d9, 33 },
    { 0, 0 },
    { 0x368902db, 77 },
    { 0x845902dc, 76 },
    { 0, 0 },
    { 0x76c902de, 99 },
    { 0, 0 },
    { 0xe81902e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2902f4, 55 },
    { 0, 0 },
    { 0x6eb902f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc902fa, 67 },
    { 0xb69902fb, 46 },
    { 0x90c902fc, 84 },
    { 0x895902fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a90300, 95 },
    { 0x15d90301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89190304, 61 },
    { 0, 0 },
    { 0x5df90306, 33 },
    { 0xca90306, 33 },
    { 0x37190307, 99 },
    { 0xa1890308, 98 },
    { 0, 0 },
    { 0x169030b, 78 },
    { 0x41c9030c, 67 },
    { 0x5fd9030d, 90 },
    { 0x1fe9030c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69790311, 100 },
    { 0x34b90312, 76 },
    { 0x58690313, 90 },
    { 0xccb90314, 55 },
    { 0xe9890314, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa559031a, 81 },
    { 0x31e9031b, 55 },
    { 0x2449031a, 98 },
    { 0x3fc9031c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e90323, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2190328, 23 },
    { 0x79b90329, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e9032c, 55 },
    { 0x9609032d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7690330, 85 },
    { 0, 0 },
    { 0xc8c90332, 55 },
    { 0x76f90333, 33 },
    { 0x5ee90334, 93 },
    { 0x8c090335, 98 },
    { 0, 0 },
    { 0x8ce90337, 95 },
    { 0xbbd90338, 95 },
    { 0x21390339, 23 },
    { 0, 0 },
    { 0x75c9033b, 100 },
    { 0x1f9033c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d09033f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd590344, 23 },
    { 0, 0 },
    { 0x7fb90346, 96 },
    { 0xa0290346, 33 },
    { 0x7fc90348, 55 },
    { 0, 0 },
    { 0xdac9034a, 83 },
    { 0x10f9034b, 43 },
    { 0xc7b9034c, 55 },
    { 0x80f9034b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf990350, 23 },
    { 0, 0 },
    { 0x35b90352, 90 },
    { 0, 0 },
    { 0xa8e90354, 87 },
    { 0xda490355, 90 },
    { 0x71090356, 43 },
    { 0xf6490357, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789035d, 76 },
    { 0, 0 },
    { 0xfd19035f, 90 },
    { 0xcb390360, 90 },
    { 0x27490361, 23 },
    { 0x71e90362, 98 },
    { 0xb2f9035f, 33 },
    { 0x27c90361, 98 },
    { 0, 0 },
    { 0xc6490366, 76 },
    { 0x84490367, 75 },
    { 0x94a90367, 43 },
    { 0xb6890369, 34 },
    { 0xb1790369, 91 },
    { 0x6289036b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72090373, 43 },
    { 0x2c090374, 94 },
    { 0, 0 },
    { 0x89a90376, 88 },
    { 0x5de90377, 33 },
    { 0, 0 },
    { 0x50390379, 81 },
    { 0x9ab9037a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab9037d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a990383, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629038b, 55 },
    { 0, 0 },
    { 0x77c9038d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb90390, 43 },
    { 0xe2890391, 43 },
    { 0x9f890391, 70 },
    { 0, 0 },
    { 0xf4b90394, 92 },
    { 0x9b790395, 93 },
    { 0xd1690394, 95 },
    { 0, 0 },
    { 0xd9790398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8669039c, 34 },
    { 0, 0 },
    { 0xfce9039e, 99 },
    { 0xf779039f, 66 },
    { 0x95a9039f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975903a4, 23 },
    { 0xb1e903a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d903a9, 76 },
    { 0x34b903aa, 23 },
    { 0x8a6903ab, 46 },
    { 0xb4e903ac, 87 },
    { 0, 0 },
    { 0x41b903ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746903b4, 95 },
    { 0, 0 },
    { 0xc3d903b6, 88 },
    { 0x18c903b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b903bb, 84 },
    { 0xfd1903bc, 90 },
    { 0xac5903bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa903c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7903c3, 98 },
    { 0, 0 },
    { 0xd60903c5, 90 },
    { 0xe92903c5, 95 },
    { 0x3ce903c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a903cc, 23 },
    { 0xdec903cd, 91 },
    { 0, 0 },
    { 0xed6903cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800903d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af903da, 90 },
    { 0, 0 },
    { 0x4ac903dc, 61 },
    { 0, 0 },
    { 0xfbb903de, 61 },
    { 0x8b4903de, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c903e2, 55 },
    { 0, 0 },
    { 0xf5e903e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6903e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24903eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815903ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x155903f2, 55 },
    { 0xbda903f3, 90 },
    { 0xb11903f4, 90 },
    { 0xcf5903f4, 90 },
    { 0x970903f6, 93 },
    { 0x37a903f7, 87 },
    { 0xbe0903f2, 94 },
    { 0x77e903f9, 90 },
    { 0, 0 },
    { 0x5d2903fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a903fe, 91 },
    { 0x131903ff, 95 },
    { 0x2a8903ff, 76 },
    { 0, 0 },
    { 0xdfd90402, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9040b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9590410, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72890413, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc169041c, 61 },
    { 0x9a29041d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0590421, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b90425, 65 },
    { 0x5c290426, 52 },
    { 0, 0 },
    { 0xd3190428, 90 },
    { 0x6290429, 85 },
    { 0xc2890429, 66 },
    { 0x2b19042b, 17 },
    { 0x3e29042c, 65 },
    { 0, 0 },
    { 0x6369042e, 76 },
    { 0x9c9042f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43390433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa90439, 77 },
    { 0x12190439, 46 },
    { 0x5c59043a, 61 },
    { 0x6199043c, 67 },
    { 0x55a90439, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc90440, 17 },
    { 0x37b90441, 98 },
    { 0x54390441, 61 },
    { 0, 0 },
    { 0xd9590444, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe129044f, 90 },
    { 0x14290450, 30 },
    { 0x43590451, 65 },
    { 0, 0 },
    { 0xe0e90453, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91190459, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc919045e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a790462, 23 },
    { 0x38690462, 90 },
    { 0x8e290464, 81 },
    { 0x93990465, 90 },
    { 0x2dc90462, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86190469, 76 },
    { 0x7a19046a, 85 },
    { 0xab19046b, 84 },
    { 0x3f89046c, 95 },
    { 0x289046c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c90470, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7790474, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd190478, 33 },
    { 0xe0e90478, 33 },
    { 0xf8d90478, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a9047d, 55 },
    { 0x5369047e, 33 },
    { 0, 0 },
    { 0xb5e90480, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1790484, 87 },
    { 0x49990484, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9390488, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa39048f, 90 },
    { 0, 0 },
    { 0xd0b90491, 98 },
    { 0x2e090492, 98 },
    { 0xc8390491, 61 },
    { 0xd8690491, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad190499, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f89049e, 95 },
    { 0x9f19049e, 91 },
    { 0, 0 },
    { 0xe2e904a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15904ab, 77 },
    { 0xf1d904ab, 76 },
    { 0xf83904ac, 77 },
    { 0xaba904ae, 100 },
    { 0x234904ae, 95 },
    { 0x22c904b0, 98 },
    { 0x887904ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e904b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb904c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b904c8, 61 },
    { 0xe71904c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36904cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06904d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35904db, 61 },
    { 0, 0 },
    { 0x3fc904dd, 90 },
    { 0x313904de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28904e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e904e5, 34 },
    { 0x33b904e6, 98 },
    { 0, 0 },
    { 0x4d3904e8, 95 },
    { 0xba7904e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c904ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8904f2, 65 },
    { 0x39c904f3, 33 },
    { 0, 0 },
    { 0x8b7904f5, 98 },
    { 0xfc6904f5, 61 },
    { 0x8c904f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43904fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c90503, 55 },
    { 0x4af90503, 90 },
    { 0, 0 },
    { 0x54c90506, 46 },
    { 0x92f90507, 81 },
    { 0, 0 },
    { 0xff990509, 99 },
    { 0x6dd90509, 23 },
    { 0, 0 },
    { 0x2d79050c, 90 },
    { 0xdbc9050d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b90516, 98 },
    { 0, 0 },
    { 0xc6d90518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6059051f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d90523, 33 },
    { 0x26a90524, 33 },
    { 0x1e590525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a290528, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4269052e, 77 },
    { 0x8df9052e, 90 },
    { 0, 0 },
    { 0x56490531, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e90534, 23 },
    { 0x5fd90534, 95 },
    { 0x74d90536, 43 },
    { 0xe1990537, 90 },
    { 0, 0 },
    { 0x90290539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839053f, 95 },
    { 0x7db9053f, 61 },
    { 0xf589053f, 100 },
    { 0x89490542, 88 },
    { 0x42790541, 61 },
    { 0xc3a90544, 61 },
    { 0xc5290545, 90 },
    { 0xdbe90546, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19054b, 90 },
    { 0x819054c, 46 },
    { 0xbee9054d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78990550, 66 },
    { 0x21890551, 90 },
    { 0x8bd90551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a690556, 61 },
    { 0x6ce90556, 64 },
    { 0x38490558, 98 },
    { 0x9e390559, 98 },
    { 0, 0 },
    { 0x6439055b, 43 },
    { 0, 0 },
    { 0x189055d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c90560, 95 },
    { 0x25c90561, 66 },
    { 0, 0 },
    { 0x78c90563, 73 },
    { 0, 0 },
    { 0xfc390565, 95 },
    { 0, 0 },
    { 0x32790567, 95 },
    { 0xbf390567, 95 },
    { 0x8b890569, 33 },
    { 0x9a290569, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc9056f, 61 },
    { 0, 0 },
    { 0xb7090571, 94 },
    { 0xf4c90571, 95 },
    { 0x8f090571, 95 },
    { 0xc1a90574, 75 },
    { 0, 0 },
    { 0x1fd90576, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x14690579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d09057c, 34 },
    { 0, 0 },
    { 0x6ff9057e, 69 },
    { 0x8519057e, 95 },
    { 0x41890580, 98 },
    { 0x3ae9057e, 55 },
    { 0x8dd9057f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4190587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed99058a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb9058d, 77 },
    { 0x5199058e, 99 },
    { 0x7709058f, 55 },
    { 0, 0 },
    { 0x9a190591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d90594, 76 },
    { 0xd0b90595, 61 },
    { 0x9af90596, 85 },
    { 0x78090596, 46 },
    { 0x1890597, 99 },
    { 0x11990599, 33 },
    { 0x1b490599, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea905a4, 52 },
    { 0x9ef905a5, 90 },
    { 0xacd905a6, 76 },
    { 0xfe9905a6, 96 },
    { 0xc1905a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4905ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e905b2, 85 },
    { 0xfcc905b2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c905b6, 33 },
    { 0xe0b905b7, 33 },
    { 0x35e905b8, 46 },
    { 0x2ca905b7, 55 },
    { 0x37c905ba, 61 },
    { 0x7d905bb, 85 },
    { 0x58a905bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83905c1, 61 },
    { 0x4ae905c1, 98 },
    { 0, 0 },
    { 0xf88905c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c905c8, 85 },
    { 0x391905c9, 34 },
    { 0x864905c9, 98 },
    { 0, 0 },
    { 0xd90905cc, 91 },
    { 0xbf3905cc, 95 },
    { 0, 0 },
    { 0xb8c905cf, 95 },
    { 0, 0 },
    { 0xaf2905d1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2905e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x810905e3, 61 },
    { 0xa68905e4, 90 },
    { 0x2f1905e5, 87 },
    { 0xcbb905e6, 55 },
    { 0x171905e7, 66 },
    { 0xc8b905e3, 85 },
    { 0x360905e9, 34 },
    { 0x879905e9, 34 },
    { 0x6d1905ea, 40 },
    { 0xd05905e9, 46 },
    { 0x9cd905ec, 34 },
    { 0, 0 },
    { 0x5cd905ef, 34 },
    { 0, 0 },
    { 0x202905f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d905f9, 100 },
    { 0xa95905fa, 33 },
    { 0x372905f9, 61 },
    { 0x6d4905fc, 33 },
    { 0x40a905fc, 33 },
    { 0xee5905fe, 68 },
    { 0x603905fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49790602, 98 },
    { 0xb2290603, 85 },
    { 0x49b90602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c9060e, 98 },
    { 0xd949060f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19290612, 61 },
    { 0, 0 },
    { 0xbb190614, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9061d, 46 },
    { 0xb639061d, 99 },
    { 0x8ae9061f, 77 },
    { 0x9189061d, 38 },
    { 0x41f90621, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de90624, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae9062a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3099062e, 95 },
    { 0x7009062f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc190632, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc90635, 55 },
    { 0xb0290636, 76 },
    { 0xf0a90636, 55 },
    { 0, 0 },
    { 0x7290639, 34 },
    { 0x6229063a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d90642, 35 },
    { 0, 0 },
    { 0x37690644, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e9064a, 93 },
    { 0x269064b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c99064e, 61 },
    { 0x2999064f, 23 },
    { 0, 0 },
    { 0xb2090651, 95 },
    { 0xf9290652, 23 },
    { 0xd9b90653, 43 },
    { 0xa3590653, 98 },
    { 0xbc390655, 56 },
    { 0x6da90656, 33 },
    { 0x7ca90657, 61 },
    { 0x92190657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e90662, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82790668, 98 },
    { 0, 0 },
    { 0x6519066a, 95 },
    { 0, 0 },
    { 0x5a09066c, 66 },
    { 0x7689066d, 61 },
    { 0x3aa9066d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b090672, 90 },
    { 0xbcd90673, 23 },
    { 0x2f590674, 34 },
    { 0, 0 },
    { 0xda390676, 98 },
    { 0x4f290676, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d19067c, 61 },
    { 0xd459067d, 95 },
    { 0xce49067e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b90682, 66 },
    { 0, 0 },
    { 0xf3590684, 99 },
    { 0x71c90685, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae9068c, 92 },
    { 0x4b59068d, 33 },
    { 0x1c29068e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28190694, 23 },
    { 0x94790695, 61 },
    { 0x1f490696, 61 },
    { 0xb7f90697, 90 },
    { 0x94f90698, 90 },
    { 0x6c590695, 90 },
    { 0x7ed9069a, 90 },
    { 0xb769069b, 46 },
    { 0x1e09069c, 43 },
    { 0xb799069c, 90 },
    { 0xef09069e, 95 },
    { 0, 0 },
    { 0x2f1906a0, 99 },
    { 0x795906a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b906a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410906aa, 61 },
    { 0, 0 },
    { 0x476906ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981906b0, 23 },
    { 0xd21906b1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c906b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x102906b7, 34 },
    { 0xaec906b8, 98 },
    { 0x1a3906b9, 90 },
    { 0x6b6906b9, 76 },
    { 0, 0 },
    { 0x449906bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x671906bf, 99 },
    { 0x6bc906bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61906c4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a906c9, 81 },
    { 0x828906c9, 33 },
    { 0xa02906cb, 98 },
    { 0xa8c906ca, 99 },
    { 0xd45906cd, 76 },
    { 0x176906ce, 55 },
    { 0xe58906cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c906d7, 90 },
    { 0xf54906d8, 98 },
    { 0xb27906d9, 33 },
    { 0xbca906da, 69 },
    { 0xaf3906da, 92 },
    { 0xd52906dc, 95 },
    { 0x347906dd, 34 },
    { 0x85d906d8, 90 },
    { 0x523906df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64906e2, 87 },
    { 0, 0 },
    { 0x25b906e4, 34 },
    { 0x8ba906e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x339906e8, 95 },
    { 0xfb3906e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5906f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26906fa, 88 },
    { 0xf51906fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149906ff, 66 },
    { 0xb690700, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e59070a, 23 },
    { 0x8e89070a, 90 },
    { 0, 0 },
    { 0xb569070d, 93 },
    { 0, 0 },
    { 0xb09070f, 46 },
    { 0x5fa90710, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f90714, 55 },
    { 0x8590715, 61 },
    { 0x8ae90715, 43 },
    { 0x64a90717, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5859071a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2490721, 55 },
    { 0x6fb90722, 33 },
    { 0xd0a90723, 88 },
    { 0, 0 },
    { 0xb1690725, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd079072a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a690730, 33 },
    { 0, 0 },
    { 0xc6390732, 95 },
    { 0x7a590733, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85790737, 95 },
    { 0x4f90737, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe9073f, 90 },
    { 0x12090740, 33 },
    { 0xf4e90740, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16890745, 33 },
    { 0xe1890746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f90749, 55 },
    { 0, 0 },
    { 0x41e9074b, 90 },
    { 0x2be9074c, 98 },
    { 0xee69074c, 76 },
    { 0x2c59074d, 95 },
    { 0xda09074f, 90 },
    { 0x18b90750, 95 },
    { 0x50d90751, 23 },
    { 0x5b690752, 90 },
    { 0x7a79074b, 55 },
    { 0x56890754, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab590758, 100 },
    { 0, 0 },
    { 0xb099075a, 76 },
    { 0x579075b, 98 },
    { 0, 0 },
    { 0x76d9075d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c90760, 33 },
    { 0, 0 },
    { 0x22b90762, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9076b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb809076e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28990774, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc90778, 36 },
    { 0, 0 },
    { 0x6749077a, 33 },
    { 0, 0 },
    { 0x42d9077c, 98 },
    { 0xac49077d, 85 },
    { 0xe689077e, 51 },
    { 0xaa59077f, 46 },
    { 0, 0 },
    { 0x42e90781, 76 },
    { 0x96e90782, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c590786, 23 },
    { 0xadd90787, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9490791, 43 },
    { 0x27f90792, 43 },
    { 0x66390792, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0990797, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a9079a, 34 },
    { 0x73c9079a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209079e, 76 },
    { 0, 0 },
    { 0xd88907a0, 98 },
    { 0xf6f907a1, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb907a4, 98 },
    { 0, 0 },
    { 0xe42907a6, 33 },
    { 0xbbe907a7, 98 },
    { 0, 0 },
    { 0x8bd907a9, 64 },
    { 0, 0 },
    { 0x754907ab, 90 },
    { 0xddf907ac, 46 },
    { 0x500907ad, 85 },
    { 0x48d907ac, 46 },
    { 0x301907af, 93 },
    { 0x79c907ac, 98 },
    { 0xc32907b0, 23 },
    { 0x7ad907b2, 76 },
    { 0x5b4907b3, 81 },
    { 0xcd907b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6907b7, 33 },
    { 0x4d7907b8, 46 },
    { 0, 0 },
    { 0x959907ba, 95 },
    { 0x97907bb, 43 },
    { 0xa39907bb, 76 },
    { 0xe6d907bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68907c0, 33 },
    { 0, 0 },
    { 0x218907c2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x991907c5, 84 },
    { 0xec9907c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9907c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3907d1, 61 },
    { 0x81907d2, 43 },
    { 0x8e0907d3, 33 },
    { 0x475907d3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e907d7, 95 },
    { 0, 0 },
    { 0x5da907d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542907e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd907ea, 90 },
    { 0x667907ea, 34 },
    { 0x151907ea, 95 },
    { 0xa24907ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d907f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c907f4, 83 },
    { 0, 0 },
    { 0x18907f6, 33 },
    { 0x9be907f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181907fb, 99 },
    { 0, 0 },
    { 0x8d0907fd, 39 },
    { 0xd20907fe, 61 },
    { 0xe6b907ff, 52 },
    { 0xe5190800, 55 },
    { 0xef390800, 46 },
    { 0xd9c90800, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a90805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7990808, 17 },
    { 0xfdb90808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd89080d, 23 },
    { 0x5259080d, 33 },
    { 0x7029080f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e90815, 23 },
    { 0x6be90815, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b9081b, 23 },
    { 0, 0 },
    { 0x7ae9081d, 61 },
    { 0x9649081d, 23 },
    { 0, 0 },
    { 0x56490820, 90 },
    { 0, 0 },
    { 0xf1490822, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85990827, 76 },
    { 0, 0 },
    { 0xa4a90829, 77 },
    { 0xd1b9082a, 61 },
    { 0xd8d9082b, 46 },
    { 0x1a39082b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b9082f, 93 },
    { 0x8290830, 33 },
    { 0x36090831, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e090837, 67 },
    { 0xbc090838, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf9083b, 90 },
    { 0x83c9083b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42090843, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da90846, 90 },
    { 0, 0 },
    { 0xe6f90848, 81 },
    { 0, 0 },
    { 0x7bc9084a, 75 },
    { 0xdd59084a, 34 },
    { 0xf369084c, 80 },
    { 0, 0 },
    { 0x1cf9084e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91890851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71990855, 23 },
    { 0x1e190856, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd279085c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f390860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde90863, 61 },
    { 0, 0 },
    { 0xbd990865, 61 },
    { 0xe5790866, 23 },
    { 0xf7a90867, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2189086d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x70690870, 98 },
    { 0xd3790870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa690875, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5790878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2269087b, 91 },
    { 0xc3c9087b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17090880, 81 },
    { 0x60e90880, 85 },
    { 0xf2b90882, 99 },
    { 0x1b490883, 98 },
    { 0x82490883, 90 },
    { 0x23f90880, 34 },
    { 0, 0 },
    { 0xff690887, 95 },
    { 0xf090887, 61 },
    { 0xdfd90889, 33 },
    { 0x25490888, 17 },
    { 0x8879088b, 55 },
    { 0xa2f9088c, 98 },
    { 0, 0 },
    { 0x7459088e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b90899, 95 },
    { 0x36d9089a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9509089e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2908a2, 67 },
    { 0, 0 },
    { 0xc27908a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b908ab, 90 },
    { 0x90908ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e908b2, 76 },
    { 0xc15908b3, 100 },
    { 0xbc5908b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec908b8, 34 },
    { 0xe52908b9, 95 },
    { 0, 0 },
    { 0xc2b908bb, 68 },
    { 0, 0 },
    { 0xd85908bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8908c6, 95 },
    { 0x6b0908c7, 61 },
    { 0x3b908c8, 33 },
    { 0x279908c9, 61 },
    { 0x997908ca, 76 },
    { 0xd06908c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775908cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e908d3, 95 },
    { 0x3ab908d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f908dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e908e0, 65 },
    { 0x532908e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0908e4, 90 },
    { 0x23e908e5, 81 },
    { 0x9ee908e6, 61 },
    { 0x8a7908e7, 90 },
    { 0xf6908e8, 91 },
    { 0xe13908e9, 33 },
    { 0xc66908ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e908f0, 98 },
    { 0x56c908f1, 88 },
    { 0x6a0908f2, 90 },
    { 0x7af908f3, 90 },
    { 0x80e908f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7908ff, 56 },
    { 0x11890900, 66 },
    { 0x35690901, 98 },
    { 0x15a90902, 100 },
    { 0x2e790901, 90 },
    { 0xba190904, 88 },
    { 0xef908ff, 61 },
    { 0x5ef90903, 99 },
    { 0, 0 },
    { 0x48890908, 76 },
    { 0x81790908, 87 },
    { 0x2ed9090a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09090f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11390912, 52 },
    { 0x3d890912, 76 },
    { 0xf3190912, 23 },
    { 0x55e90915, 33 },
    { 0, 0 },
    { 0x81790917, 95 },
    { 0xe1e90918, 76 },
    { 0xe8890919, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90990920, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e890927, 95 },
    { 0xd6290927, 43 },
    { 0, 0 },
    { 0x9d09092a, 76 },
    { 0xc839092b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b90930, 68 },
    { 0x5c690930, 33 },
    { 0, 0 },
    { 0x4a090933, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b990937, 43 },
    { 0xe0090938, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca9093d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xff190940, 23 },
    { 0xfe890941, 98 },
    { 0x40890942, 33 },
    { 0x63390943, 33 },
    { 0, 0 },
    { 0x25990945, 23 },
    { 0xca190946, 95 },
    { 0, 0 },
    { 0x5090948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce9094c, 33 },
    { 0xd119094d, 34 },
    { 0xc1d9094e, 33 },
    { 0xf5f9094f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa490954, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e790959, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c9095d, 23 },
    { 0x8519095d, 95 },
    { 0x2289095f, 23 },
    { 0xb179095f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9290963, 34 },
    { 0xae190964, 100 },
    { 0x20290965, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9096b, 23 },
    { 0xa609096b, 52 },
    { 0x9be9096d, 88 },
    { 0, 0 },
    { 0x8f19096f, 51 },
    { 0xd909096f, 90 },
    { 0xa3390971, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce90976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b39097b, 68 },
    { 0xcf29097c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd9097f, 85 },
    { 0x80490980, 98 },
    { 0x39b90981, 98 },
    { 0xcb490982, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca90985, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85090989, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669098c, 93 },
    { 0xaf79098d, 23 },
    { 0x8cc9098e, 90 },
    { 0x1a99098f, 33 },
    { 0x57a9098e, 85 },
    { 0x6829098f, 64 },
    { 0x9ae9098f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa249099d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86909a4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2909a7, 33 },
    { 0x1d2909a7, 46 },
    { 0x4a5909a9, 23 },
    { 0x5a9909aa, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d909ad, 100 },
    { 0x4a4909ae, 67 },
    { 0x241909af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6909b4, 33 },
    { 0, 0 },
    { 0xd8f909b6, 61 },
    { 0xb7b909b6, 76 },
    { 0x5e8909b8, 23 },
    { 0x9c6909b9, 61 },
    { 0xd1909b7, 90 },
    { 0x117909bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62909c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2909c6, 93 },
    { 0x701909c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x229909ca, 77 },
    { 0xbaa909cb, 75 },
    { 0xfe4909ca, 90 },
    { 0x6df909cd, 90 },
    { 0x47f909cd, 52 },
    { 0xe6a909cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df909d3, 90 },
    { 0, 0 },
    { 0xa59909d5, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396909d9, 98 },
    { 0xfb909da, 93 },
    { 0xb28909da, 61 },
    { 0xfb0909dc, 23 },
    { 0x786909dc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac909e1, 66 },
    { 0x835909e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228909e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4909ec, 90 },
    { 0xf5909ed, 55 },
    { 0x95a909ee, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd909f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x41909f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d909fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90090a02, 23 },
    { 0xb8d90a03, 52 },
    { 0, 0 },
    { 0xe8890a05, 23 },
    { 0x36190a06, 98 },
    { 0x50990a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41590a0b, 85 },
    { 0xb2f90a0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f590a0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b890a13, 91 },
    { 0x19890a14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2190a17, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f690a1b, 87 },
    { 0xe2890a1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e90a22, 34 },
    { 0x75a90a23, 55 },
    { 0x44b90a24, 95 },
    { 0xbcf90a25, 100 },
    { 0, 0 },
    { 0xf2390a27, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd90a2d, 23 },
    { 0, 0 },
    { 0x61f90a2f, 99 },
    { 0xb6190a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d90a37, 77 },
    { 0xc8090a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbda90a42, 78 },
    { 0x10190a43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c790a47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38390a4d, 61 },
    { 0x43a90a4e, 90 },
    { 0xf5790a4f, 61 },
    { 0x79d90a50, 90 },
    { 0x68990a50, 76 },
    { 0x6ac90a52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d490a59, 90 },
    { 0xfe290a5a, 76 },
    { 0, 0 },
    { 0x93990a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f90a5f, 61 },
    { 0x3090a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c90a64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76590a68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf90a6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab290a71, 93 },
    { 0x6ed90a72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b90a76, 61 },
    { 0, 0 },
    { 0x8f590a78, 23 },
    { 0, 0 },
    { 0xe5490a7a, 61 },
    { 0xc6f90a7b, 33 },
    { 0, 0 },
    { 0x86c90a7d, 49 },
    { 0xe7290a7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba90a81, 88 },
    { 0x7f90a81, 61 },
    { 0xf6990a83, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69490a87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70390a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd90a91, 90 },
    { 0xd5190a92, 33 },
    { 0x82b90a93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c90a96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd90a99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x74690a9c, 23 },
    { 0xda890a9d, 100 },
    { 0x6dc90a9d, 90 },
    { 0xf7b90a9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d690aa2, 34 },
    { 0xeb690aa2, 90 },
    { 0x71d90aa4, 56 },
    { 0xc7a90aa2, 95 },
    { 0, 0 },
    { 0x4ac90aa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a90aab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d790aaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf90ab9, 90 },
    { 0x66290aba, 77 },
    { 0, 0 },
    { 0x89f90abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e90ac0, 43 },
    { 0x18c90ac1, 46 },
    { 0, 0 },
    { 0x5b190ac3, 90 },
    { 0xb1690ac4, 78 },
    { 0x80a90ac5, 98 },
    { 0xb9390ac5, 33 },
    { 0, 0 },
    { 0xd2290ac8, 19 },
    { 0xe3590ac9, 23 },
    { 0x82990ac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da90ad0, 33 },
    { 0x57e90ad1, 76 },
    { 0xb0390ad1, 95 },
    { 0xf090ad3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d390ad6, 61 },
    { 0x6990ad7, 100 },
    { 0x8d890ad6, 23 },
    { 0x90290ad8, 33 },
    { 0xf0c90ada, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab90ade, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a490ae7, 51 },
    { 0x90a90ae8, 61 },
    { 0x8ba90ae9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb690af1, 90 },
    { 0xc8390af1, 76 },
    { 0xf2090af1, 90 },
    { 0x1fb90af4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10790af7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec90afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43490b01, 46 },
    { 0x29990b02, 34 },
    { 0x9090b02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb490b07, 33 },
    { 0xcf890b08, 23 },
    { 0x4d190b09, 95 },
    { 0xc3a90b0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec90b0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90390b14, 43 },
    { 0, 0 },
    { 0x43c90b16, 61 },
    { 0, 0 },
    { 0xee290b18, 68 },
    { 0x47390b19, 91 },
    { 0x5290b19, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a990b20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d790b2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf090b33, 90 },
    { 0, 0 },
    { 0x8df90b35, 43 },
    { 0x59090b36, 93 },
    { 0xf6b90b37, 90 },
    { 0xe1590b38, 34 },
    { 0x15790b38, 68 },
    { 0xc6c90b38, 67 },
    { 0x56490b39, 95 },
    { 0x31790b37, 61 },
    { 0, 0 },
    { 0xf5590b3e, 23 },
    { 0xb2190b3f, 82 },
    { 0, 0 },
    { 0x4b990b41, 23 },
    { 0x77590b42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b90b47, 34 },
    { 0x25c90b48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb90b4b, 33 },
    { 0, 0 },
    { 0xa4b90b4d, 90 },
    { 0x8de90b4e, 52 },
    { 0x78f90b4e, 23 },
    { 0x1bb90b50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d90b54, 98 },
    { 0, 0 },
    { 0x14290b56, 61 },
    { 0x44f90b56, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b90b60, 33 },
    { 0xd1a90b61, 34 },
    { 0x73590b61, 33 },
    { 0, 0 },
    { 0xd4590b64, 90 },
    { 0, 0 },
    { 0xa1590b66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d90b6b, 87 },
    { 0x27090b6c, 17 },
    { 0x61590b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25590b72, 90 },
    { 0xca990b73, 33 },
    { 0xddb90b72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb390b7f, 90 },
    { 0xcec90b80, 55 },
    { 0, 0 },
    { 0x85790b82, 90 },
    { 0, 0 },
    { 0x28790b84, 90 },
    { 0, 0 },
    { 0x36f90b86, 23 },
    { 0x32990b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa90b93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b90b96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6890b9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf890ba3, 84 },
    { 0, 0 },
    { 0xbd790ba5, 55 },
    { 0, 0 },
    { 0xe1e90ba7, 23 },
    { 0xdbe90ba7, 61 },
    { 0xccd90ba9, 46 },
    { 0x50b90baa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15990bb0, 34 },
    { 0x6fd90bb1, 61 },
    { 0, 0 },
    { 0x7e190bb3, 95 },
    { 0x44890bb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d390bb7, 33 },
    { 0x63090bb8, 33 },
    { 0, 0 },
    { 0x8b90bba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa90bbe, 76 },
    { 0xf9490bbf, 95 },
    { 0, 0 },
    { 0x59090bc1, 90 },
    { 0x6f790bc2, 55 },
    { 0x82890bc3, 61 },
    { 0x94090bc3, 55 },
    { 0x45a90bc4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f90bc8, 23 },
    { 0xef890bc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3190bcc, 76 },
    { 0x590bcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad90bd0, 23 },
    { 0xbb690bd1, 23 },
    { 0, 0 },
    { 0x10090bd3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51990be0, 55 },
    { 0xfa790be1, 61 },
    { 0xc6a90be2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e90be8, 95 },
    { 0xc1090be9, 33 },
    { 0x44c90bea, 100 },
    { 0x23790beb, 90 },
    { 0x7aa90bec, 76 },
    { 0x36090bea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee190bf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b90bf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c90bff, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d90c02, 46 },
    { 0, 0 },
    { 0x43c90c04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e90c08, 76 },
    { 0, 0 },
    { 0xa2490c0a, 91 },
    { 0xc7e90c0b, 17 },
    { 0x3a90c0a, 55 },
    { 0x75390c0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a090c11, 61 },
    { 0xde190c12, 101 },
    { 0xe390c13, 43 },
    { 0x1d090c14, 88 },
    { 0x4f90c15, 23 },
    { 0xb5090c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5090c19, 80 },
    { 0x60b90c1a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e90c1d, 96 },
    { 0, 0 },
    { 0x7a190c1f, 17 },
    { 0, 0 },
    { 0x9490c21, 76 },
    { 0x58190c22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a090c26, 90 },
    { 0x5eb90c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f90c2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc90c2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa190c34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9590c39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f90c3d, 55 },
    { 0x33e90c3e, 33 },
    { 0xb6d90c3f, 46 },
    { 0x54190c3f, 23 },
    { 0x45190c3e, 95 },
    { 0x5bc90c3e, 81 },
    { 0x4a190c41, 61 },
    { 0x2c590c44, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59990c48, 52 },
    { 0x3d790c48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14590c4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97490c50, 88 },
    { 0, 0 },
    { 0x3bb90c52, 90 },
    { 0xd1b90c52, 67 },
    { 0x61b90c54, 77 },
    { 0xfd390c52, 76 },
    { 0x95790c56, 43 },
    { 0xea190c57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14890c5d, 17 },
    { 0, 0 },
    { 0xc6090c5f, 88 },
    { 0xb7990c60, 33 },
    { 0x4a690c60, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae790c65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26190c69, 87 },
    { 0, 0 },
    { 0xa8390c6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce690c71, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b90c74, 95 },
    { 0x9f90c75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3990c78, 33 },
    { 0xb4290c79, 90 },
    { 0x6bd90c7a, 61 },
    { 0x66b90c7b, 95 },
    { 0xfd790c7c, 95 },
    { 0x4a90c7d, 90 },
    { 0x21e90c7c, 99 },
    { 0x7d590c7f, 90 },
    { 0, 0 },
    { 0x5890c81, 61 },
    { 0xc0690c82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x92790c85, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14690c89, 61 },
    { 0x15790c8a, 76 },
    { 0xd190c89, 23 },
    { 0xda490c8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4490c90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6790c97, 90 },
    { 0, 0 },
    { 0x84190c99, 33 },
    { 0, 0 },
    { 0xd9390c9b, 81 },
    { 0x92190c9c, 85 },
    { 0x57a90c9d, 67 },
    { 0x2b990c9d, 43 },
    { 0x6d790c9e, 61 },
    { 0x89490ca0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba090ca5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5690cab, 61 },
    { 0xb1f90cac, 61 },
    { 0, 0 },
    { 0x32f90cae, 33 },
    { 0x98690caf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5690cb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8890cba, 85 },
    { 0, 0 },
    { 0xe9790cbc, 61 },
    { 0x19c90cbd, 90 },
    { 0, 0 },
    { 0xb2290cbf, 90 },
    { 0xf3390cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b990cc3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c90cc9, 90 },
    { 0, 0 },
    { 0x5cf90ccb, 100 },
    { 0xd2090ccb, 23 },
    { 0x3d290ccb, 93 },
    { 0x41f90ccc, 96 },
    { 0x11c90ccd, 93 },
    { 0x55190cd0, 55 },
    { 0x790cd1, 33 },
    { 0xa090cd2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb590cd5, 33 },
    { 0x55f90cd6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa290cdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5490cde, 88 },
    { 0x26990cdf, 95 },
    { 0xfa590ce0, 91 },
    { 0x80f90ce1, 55 },
    { 0xe0390ce2, 53 },
    { 0xe6590ce2, 98 },
    { 0x81b90ce2, 33 },
    { 0x76990ce5, 33 },
    { 0xce190ce6, 88 },
    { 0x1ef90ce7, 46 },
    { 0x9f590ce8, 76 },
    { 0x18690ce9, 77 },
    { 0xa1d90cdf, 9 },
    { 0xb6790ce4, 33 },
    { 0x3ac90cec, 61 },
    { 0x65a90ce5, 95 },
    { 0x7cb90cea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9990cf1, 95 },
    { 0, 0 },
    { 0xf4690cf3, 90 },
    { 0, 0 },
    { 0x19390cf5, 77 },
    { 0x56b90cf6, 96 },
    { 0, 0 },
    { 0x5090cf8, 75 },
    { 0, 0 },
    { 0xb0b90cfa, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x61990cfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4590d02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d90d07, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c90d0b, 98 },
    { 0x3ff90d0b, 17 },
    { 0x2a790d0d, 78 },
    { 0x50490d0c, 95 },
    { 0x2ed90d0f, 34 },
    { 0x73490d10, 61 },
    { 0x2b190d0f, 95 },
    { 0x34f90d0e, 33 },
    { 0xe0890d13, 33 },
    { 0xf6b90d13, 61 },
    { 0x4fe90d15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7790d19, 98 },
    { 0, 0 },
    { 0xdde90d1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e590d1e, 23 },
    { 0x50090d1f, 61 },
    { 0, 0 },
    { 0xbf190d21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af90d27, 76 },
    { 0xe5f90d28, 95 },
    { 0x15a90d29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c90d2c, 61 },
    { 0xd4590d2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d290d32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80890d38, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xee090d3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d90d41, 33 },
    { 0x3d990d42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb690d46, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b490d4b, 23 },
    { 0xa4690d4b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1690d50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c890d53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a90d58, 34 },
    { 0x6790d59, 23 },
    { 0x47790d58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1590d62, 76 },
    { 0xbba90d63, 100 },
    { 0, 0 },
    { 0x9c490d65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb490d70, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58490d74, 90 },
    { 0x67490d75, 90 },
    { 0x7c390d76, 90 },
    { 0xf1690d77, 99 },
    { 0, 0 },
    { 0x2e390d79, 49 },
    { 0, 0 },
    { 0xf1690d7b, 98 },
    { 0x2cc90d7c, 23 },
    { 0xdbc90d7d, 95 },
    { 0xee490d7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e90d81, 23 },
    { 0xa5690d82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b990d85, 34 },
    { 0x64190d85, 98 },
    { 0, 0 },
    { 0x77390d88, 46 },
    { 0xec390d88, 95 },
    { 0, 0 },
    { 0xaf290d8b, 88 },
    { 0xa5290d8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb90d90, 97 },
    { 0xa7b90d91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29490d94, 90 },
    { 0, 0 },
    { 0xcf890d96, 61 },
    { 0x8fa90d97, 85 },
    { 0, 0 },
    { 0xd2390d99, 90 },
    { 0xd5990d9a, 76 },
    { 0, 0 },
    { 0xddb90d9c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57990da2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a90da5, 85 },
    { 0x98590da6, 23 },
    { 0x77290da7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5490dac, 34 },
    { 0xfaf90dad, 23 },
    { 0x72390dae, 52 },
    { 0xbd090daf, 61 },
    { 0x2f890daf, 93 },
    { 0xc8390db1, 40 },
    { 0x6bf90db1, 34 },
    { 0x5fb90db2, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee90db6, 23 },
    { 0xb190db6, 98 },
    { 0xf6190db8, 84 },
    { 0x10a90db7, 33 },
    { 0xa4490db8, 33 },
    { 0xd2c90dbb, 100 },
    { 0xf0f90dbc, 23 },
    { 0x5f490dbd, 76 },
    { 0xbeb90db7, 95 },
    { 0x69190dbf, 98 },
    { 0x5890dc0, 87 },
    { 0x5c090dc0, 98 },
    { 0xcbb90dc2, 33 },
    { 0xd8b90dc2, 68 },
    { 0x3f790dc0, 66 },
    { 0xe5e90dc2, 55 },
    { 0xe8390dc6, 84 },
    { 0x5890dbc, 59 },
    { 0x28b90dc8, 34 },
    { 0x8a790dc9, 23 },
    { 0xdac90dc8, 75 },
    { 0xa2b90db8, 55 },
    { 0x87090dc6, 95 },
    { 0xcc090dc8, 90 },
    { 0x30c90dce, 34 },
    { 0xf5690dcf, 46 },
    { 0x88590dc0, 55 },
    { 0xd3190dd1, 23 },
    { 0, 0 },
    { 0xe1590dd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d90dd8, 76 },
    { 0xd9f90dd9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd90dde, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f90de1, 23 },
    { 0xc6790de2, 90 },
    { 0x78c90de3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2290de6, 87 },
    { 0x4be90de7, 43 },
    { 0x22390de8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a90deb, 81 },
    { 0x23990dec, 66 },
    { 0xdc090dec, 99 },
    { 0x17d90dee, 90 },
    { 0x91e90def, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a490df6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79990dfc, 76 },
    { 0x23290dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb90e07, 93 },
    { 0x44090e08, 98 },
    { 0, 0 },
    { 0x8ea90e0a, 81 },
    { 0x67790e0b, 46 },
    { 0xca690e0c, 48 },
    { 0, 0 },
    { 0x7ed90e0e, 85 },
    { 0xbd290e0f, 90 },
    { 0, 0 },
    { 0xa8e90e11, 76 },
    { 0x36f90e12, 23 },
    { 0xba790e12, 76 },
    { 0x55b90e14, 98 },
    { 0x4ab90e13, 84 },
    { 0x21490e12, 53 },
    { 0, 0 },
    { 0x78190e18, 67 },
    { 0x3e90e19, 95 },
    { 0x92290e1a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8890e1e, 64 },
    { 0xc7e90e1e, 61 },
    { 0x89290e20, 34 },
    { 0x45d90e1e, 33 },
    { 0, 0 },
    { 0x37b90e23, 95 },
    { 0x14a90e24, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b990e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7590e2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f90e30, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c90e37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d90e3a, 61 },
    { 0x13390e3b, 96 },
    { 0x3c990e3c, 95 },
    { 0x90e3c, 98 },
    { 0xa3890e3d, 84 },
    { 0x74790e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a90e45, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5090e4d, 76 },
    { 0xc2290e4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81590e52, 68 },
    { 0, 0 },
    { 0x4dd90e54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51990e59, 98 },
    { 0, 0 },
    { 0x92190e5b, 33 },
    { 0, 0 },
    { 0x35d90e5d, 61 },
    { 0x59790e5e, 88 },
    { 0x78690e5d, 77 },
    { 0x20090e5e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd390e63, 77 },
    { 0xa2790e63, 33 },
    { 0x91990e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b990e69, 98 },
    { 0xb4590e6a, 23 },
    { 0xe3b90e6b, 33 },
    { 0x6b190e6a, 40 },
    { 0x3d090e6c, 24 },
    { 0, 0 },
    { 0xc2c90e6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a090e72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa90e75, 90 },
    { 0xec790e76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f490e7a, 98 },
    { 0x87a90e7b, 93 },
    { 0, 0 },
    { 0x64290e7d, 53 },
    { 0x80c90e7e, 90 },
    { 0x60290e7f, 100 },
    { 0xa90e7d, 61 },
    { 0x1ce90e81, 23 },
    { 0x3da90e82, 76 },
    { 0x75890e7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50490e87, 75 },
    { 0x9e090e88, 43 },
    { 0xcec90e88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92790e8d, 34 },
    { 0, 0 },
    { 0x2f690e8f, 93 },
    { 0xad890e90, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2290e93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85a90e97, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea090e9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5290ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a90ea4, 43 },
    { 0x76890ea4, 95 },
    { 0, 0 },
    { 0x8090ea7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e790eb0, 66 },
    { 0x4f690eb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde90eba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4790ec0, 90 },
    { 0, 0 },
    { 0x14d90ec2, 90 },
    { 0, 0 },
    { 0xbd290ec4, 55 },
    { 0xac590ec5, 98 },
    { 0x25590ec6, 98 },
    { 0xe0d90ec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d90eca, 76 },
    { 0x4f990eca, 84 },
    { 0xae190ecc, 91 },
    { 0x12990ecb, 33 },
    { 0, 0 },
    { 0x1b890ecf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76490ed8, 61 },
    { 0xb0290ed9, 77 },
    { 0, 0 },
    { 0xf9d90edb, 43 },
    { 0, 0 },
    { 0x95190edd, 95 },
    { 0xa0b90ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf290ee5, 98 },
    { 0x7fa90ee6, 78 },
    { 0, 0 },
    { 0x34390ee8, 87 },
    { 0x7f690ee9, 61 },
    { 0xfad90ee8, 43 },
    { 0, 0 },
    { 0x5c290eec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7690eef, 34 },
    { 0x38c90ef0, 90 },
    { 0x3b690ef1, 95 },
    { 0x86e90ef2, 95 },
    { 0xd6590ef2, 64 },
    { 0x41990ef4, 91 },
    { 0x77290ef4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49090efa, 85 },
    { 0xc1290efb, 76 },
    { 0, 0 },
    { 0x32190efd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9190f01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd90f04, 33 },
    { 0x5a090f05, 98 },
    { 0x4bf90f06, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x48290f09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d90f14, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe90f19, 75 },
    { 0xd3a90f19, 33 },
    { 0x45790f1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8390f1f, 90 },
    { 0x62490f1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf490f24, 33 },
    { 0x40d90f25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad90f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f90f37, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c90f3b, 90 },
    { 0x57290f3c, 46 },
    { 0xb3190f3d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa890f44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a090f48, 43 },
    { 0x29c90f49, 33 },
    { 0xd2190f4a, 67 },
    { 0xe3b90f4a, 34 },
    { 0x3ac90f48, 90 },
    { 0x9f790f4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde790f50, 33 },
    { 0x4f090f50, 90 },
    { 0xb9290f52, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c90f55, 23 },
    { 0x8a090f56, 76 },
    { 0xea790f57, 98 },
    { 0xd7690f58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1590f5c, 76 },
    { 0x2de90f5d, 98 },
    { 0, 0 },
    { 0x49f90f5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d90f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23690f66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c390f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd90f6f, 38 },
    { 0x65090f6f, 61 },
    { 0x96f90f6f, 98 },
    { 0xb9790f72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda90f76, 34 },
    { 0xe6d90f76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df90f7c, 61 },
    { 0, 0 },
    { 0x8390f7e, 23 },
    { 0x94590f7f, 90 },
    { 0x2e290f80, 95 },
    { 0, 0 },
    { 0x60390f82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53290f89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f90f8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51890f91, 76 },
    { 0, 0 },
    { 0xeb990f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46390f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd90f9a, 43 },
    { 0, 0 },
    { 0xffb90f9c, 91 },
    { 0xa1390f9c, 91 },
    { 0, 0 },
    { 0x60c90f9f, 93 },
    { 0, 0 },
    { 0xa6c90fa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2190fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76490fb0, 55 },
    { 0xc4c90fb1, 98 },
    { 0x5ad90fb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e90fb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4190fbe, 61 },
    { 0x11590fbe, 55 },
    { 0xcfc90fc0, 98 },
    { 0x65990fc0, 97 },
    { 0x6c190fc2, 40 },
    { 0, 0 },
    { 0x70690fc4, 90 },
    { 0, 0 },
    { 0xbdc90fc6, 23 },
    { 0, 0 },
    { 0x67b90fc8, 78 },
    { 0x9f590fc9, 34 },
    { 0x9c990fca, 90 },
    { 0x14190fcb, 33 },
    { 0x93d90fcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b990fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f690fd3, 76 },
    { 0xedb90fd4, 68 },
    { 0xc0190fd4, 90 },
    { 0x3ec90fd5, 61 },
    { 0xb5090fd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e990fda, 52 },
    { 0, 0 },
    { 0xaf390fdc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a90fdf, 95 },
    { 0x9f090fe0, 95 },
    { 0xd4c90fe0, 90 },
    { 0x6a90fe0, 34 },
    { 0x1a890fe3, 90 },
    { 0x32190fe3, 33 },
    { 0x2bc90fe5, 61 },
    { 0xb8a90fe6, 76 },
    { 0xfce90fe4, 61 },
    { 0x8a90fe8, 95 },
    { 0xd5a90fe8, 90 },
    { 0xb7590fea, 87 },
    { 0xbd490fdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b90fee, 78 },
    { 0x20990fef, 23 },
    { 0x86d90fee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16790ff3, 90 },
    { 0x6690ff4, 87 },
    { 0, 0 },
    { 0xa6790ff6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41990ffc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f190fff, 61 },
    { 0, 0 },
    { 0x66091001, 95 },
    { 0, 0 },
    { 0xbf591003, 61 },
    { 0xbd891003, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b91008, 23 },
    { 0, 0 },
    { 0xedb9100a, 68 },
    { 0x7f49100a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f991014, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5189101b, 55 },
    { 0x5279101c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb99101f, 87 },
    { 0x2d491020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2891024, 93 },
    { 0xae891025, 23 },
    { 0x3d491024, 34 },
    { 0xccc91027, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a9102e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5991033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb91037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe769103c, 96 },
    { 0xd209103d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7191040, 88 },
    { 0x20191041, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94191045, 98 },
    { 0x57a91046, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1519104b, 99 },
    { 0, 0 },
    { 0xf039104d, 33 },
    { 0, 0 },
    { 0x74b9104f, 33 },
    { 0x2491050, 34 },
    { 0x35691051, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1991058, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4191062, 33 },
    { 0, 0 },
    { 0x21891064, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74191068, 61 },
    { 0x84291069, 55 },
    { 0x4539106a, 81 },
    { 0, 0 },
    { 0x48e9106c, 95 },
    { 0xca9106d, 98 },
    { 0x8aa9106c, 90 },
    { 0x3039106f, 95 },
    { 0x2e9106d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde91076, 99 },
    { 0x8fb91076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e69107d, 77 },
    { 0xd249107e, 23 },
    { 0xd99107e, 90 },
    { 0x72f91080, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd91083, 23 },
    { 0xd5c91083, 23 },
    { 0, 0 },
    { 0xe4f91086, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e9108a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe91090, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf191096, 55 },
    { 0, 0 },
    { 0xdc891098, 33 },
    { 0x99591099, 61 },
    { 0, 0 },
    { 0xa779109b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9109e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x682910a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee910a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60910ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2910af, 91 },
    { 0xca5910af, 76 },
    { 0x85d910b1, 90 },
    { 0xc67910b2, 98 },
    { 0x9f2910b1, 98 },
    { 0, 0 },
    { 0x96c910b5, 76 },
    { 0x60e910b6, 40 },
    { 0, 0 },
    { 0xf59910b8, 61 },
    { 0, 0 },
    { 0xff2910ba, 98 },
    { 0xd00910bb, 33 },
    { 0, 0 },
    { 0x90910bd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4910c0, 85 },
    { 0, 0 },
    { 0x6c7910c2, 90 },
    { 0, 0 },
    { 0xcd5910c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71910c9, 75 },
    { 0, 0 },
    { 0x3e910cb, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x618910ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9910d3, 76 },
    { 0, 0 },
    { 0x6a4910d5, 33 },
    { 0, 0 },
    { 0x996910d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48910da, 46 },
    { 0x5cb910db, 90 },
    { 0x3bd910da, 93 },
    { 0x834910dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9910e1, 55 },
    { 0xb0e910e2, 61 },
    { 0x980910e3, 78 },
    { 0x715910e2, 34 },
    { 0x2a8910e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f910ee, 55 },
    { 0x884910ef, 98 },
    { 0x762910f0, 95 },
    { 0x49f910f0, 90 },
    { 0x660910f2, 23 },
    { 0x6a7910f3, 46 },
    { 0, 0 },
    { 0x9c1910f5, 90 },
    { 0, 0 },
    { 0xe9a910f7, 95 },
    { 0x519910f8, 90 },
    { 0x1e2910f9, 95 },
    { 0x211910fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6910fd, 95 },
    { 0x1b5910fe, 61 },
    { 0, 0 },
    { 0x9c391100, 77 },
    { 0x2d891100, 84 },
    { 0, 0 },
    { 0x38591103, 34 },
    { 0x26691104, 78 },
    { 0x26991105, 80 },
    { 0x6ab91106, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9110b, 98 },
    { 0, 0 },
    { 0xb7e9110d, 90 },
    { 0, 0 },
    { 0xb819110f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e91116, 36 },
    { 0xaea91116, 95 },
    { 0x8b891116, 61 },
    { 0x22891118, 61 },
    { 0xfee9111a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9111d, 33 },
    { 0x54b9111e, 46 },
    { 0xb2e9111f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb91123, 92 },
    { 0x1a291123, 90 },
    { 0xc0c91123, 91 },
    { 0x58591126, 98 },
    { 0, 0 },
    { 0x3a991128, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c9112b, 61 },
    { 0xf7d9112c, 61 },
    { 0x5019112d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26991133, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b29113c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee91142, 90 },
    { 0, 0 },
    { 0x58291144, 76 },
    { 0xe4a91145, 75 },
    { 0xfd691146, 95 },
    { 0xd4f91147, 36 },
    { 0x15b91148, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2319114b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad91150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f791154, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9791157, 33 },
    { 0, 0 },
    { 0x43e91159, 23 },
    { 0, 0 },
    { 0xc59115b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f69115f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac91163, 87 },
    { 0, 0 },
    { 0x8fa91165, 87 },
    { 0, 0 },
    { 0xf0591167, 93 },
    { 0, 0 },
    { 0xe4691169, 23 },
    { 0, 0 },
    { 0x9359116b, 95 },
    { 0xe929116c, 33 },
    { 0, 0 },
    { 0x9b69116e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8191171, 90 },
    { 0x20691171, 95 },
    { 0, 0 },
    { 0xdf991174, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c9117c, 33 },
    { 0x9029117d, 90 },
    { 0xf279117e, 65 },
    { 0x2fb9117f, 55 },
    { 0x1ce91180, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13091184, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e591188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec19118e, 23 },
    { 0, 0 },
    { 0x58191190, 95 },
    { 0, 0 },
    { 0xb7291192, 94 },
    { 0xff291193, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b91199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b9119c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae911a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc911a5, 90 },
    { 0x98f911a5, 56 },
    { 0, 0 },
    { 0x3ae911a8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7911ab, 90 },
    { 0xec6911ac, 33 },
    { 0, 0 },
    { 0xab8911ae, 55 },
    { 0x963911af, 61 },
    { 0xb72911b0, 99 },
    { 0xed1911b1, 61 },
    { 0xfa8911b1, 95 },
    { 0, 0 },
    { 0x59d911b4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6911b7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89911bc, 98 },
    { 0, 0 },
    { 0x1ae911be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec911c8, 23 },
    { 0xe0b911c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f911ce, 23 },
    { 0, 0 },
    { 0xa87911d0, 98 },
    { 0xb63911d0, 23 },
    { 0x723911d1, 98 },
    { 0, 0 },
    { 0x1b5911d4, 90 },
    { 0xbc9911d5, 93 },
    { 0x445911d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed911d9, 90 },
    { 0, 0 },
    { 0x52a911db, 43 },
    { 0, 0 },
    { 0x969911dd, 90 },
    { 0xc1a911de, 90 },
    { 0xaad911df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7911e2, 23 },
    { 0x374911e3, 94 },
    { 0, 0 },
    { 0xa4f911e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5911e8, 98 },
    { 0xff3911e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0911ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3911ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c911f3, 61 },
    { 0x952911f4, 90 },
    { 0x84911f3, 26 },
    { 0xbed911f5, 55 },
    { 0x385911f7, 95 },
    { 0xecc911f8, 55 },
    { 0xb70911f9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b91201, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ba91207, 90 },
    { 0x8c191208, 87 },
    { 0, 0 },
    { 0xf19120a, 90 },
    { 0x45c9120b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a91211, 90 },
    { 0x5e091212, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9191216, 43 },
    { 0x5f991217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9121a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19121f, 46 },
    { 0x62a9121f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac91226, 32 },
    { 0x70b91227, 65 },
    { 0xc9991227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62891231, 90 },
    { 0x79591231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f591235, 84 },
    { 0x96391236, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea79123d, 84 },
    { 0x6909123e, 23 },
    { 0, 0 },
    { 0x34b91240, 91 },
    { 0, 0 },
    { 0x80291242, 78 },
    { 0, 0 },
    { 0x69591244, 96 },
    { 0x9aa91244, 99 },
    { 0xdc291246, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb199124a, 61 },
    { 0xbb29124a, 76 },
    { 0, 0 },
    { 0x9799124d, 84 },
    { 0x33b9124d, 95 },
    { 0x85d9124f, 81 },
    { 0xe9691250, 23 },
    { 0xb129124f, 23 },
    { 0x8a591252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a91259, 90 },
    { 0x2af9125a, 95 },
    { 0x7b09125a, 52 },
    { 0x3629125c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19191262, 98 },
    { 0, 0 },
    { 0xf8791264, 90 },
    { 0, 0 },
    { 0x15f91266, 33 },
    { 0x91d91267, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf579126c, 90 },
    { 0xb039126c, 34 },
    { 0xe3f9126c, 61 },
    { 0, 0 },
    { 0xa4391270, 33 },
    { 0xa5991271, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8891275, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca59127d, 85 },
    { 0x6569127e, 34 },
    { 0x7359127e, 90 },
    { 0, 0 },
    { 0x38891281, 95 },
    { 0x46891282, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70491289, 98 },
    { 0xa3e9128a, 46 },
    { 0x9fb9128b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8969128f, 61 },
    { 0x2fe91290, 98 },
    { 0xe4391291, 90 },
    { 0x72991292, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b391297, 77 },
    { 0xff891297, 98 },
    { 0x80591299, 76 },
    { 0x89191299, 95 },
    { 0x3529129a, 100 },
    { 0xdac9129c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568912a3, 98 },
    { 0, 0 },
    { 0xd6b912a5, 33 },
    { 0, 0 },
    { 0xea8912a7, 23 },
    { 0xb52912a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab912ab, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772912b0, 100 },
    { 0, 0 },
    { 0xcb0912b2, 34 },
    { 0x2d5912b2, 46 },
    { 0xc6b912b4, 98 },
    { 0x37d912b4, 65 },
    { 0x7a9912b3, 36 },
    { 0xbed912b3, 65 },
    { 0x11d912b3, 95 },
    { 0xc63912b6, 61 },
    { 0xca4912ba, 61 },
    { 0x977912bb, 98 },
    { 0, 0 },
    { 0x32b912bd, 76 },
    { 0, 0 },
    { 0x951912bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa912c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd912c6, 90 },
    { 0, 0 },
    { 0x9b8912c8, 65 },
    { 0x7e4912c9, 95 },
    { 0x20b912c9, 33 },
    { 0, 0 },
    { 0x92a912cc, 95 },
    { 0, 0 },
    { 0x939912ce, 90 },
    { 0xd3f912cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2912d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50912d6, 75 },
    { 0x58b912d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34f912da, 61 },
    { 0, 0 },
    { 0xa40912dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb912e1, 23 },
    { 0xc42912e1, 61 },
    { 0x288912e3, 98 },
    { 0x5e6912e3, 85 },
    { 0x7e1912e2, 55 },
    { 0, 0 },
    { 0x74e912e7, 99 },
    { 0, 0 },
    { 0x370912e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc912f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148912fa, 23 },
    { 0, 0 },
    { 0x69c912fc, 95 },
    { 0x643912fd, 61 },
    { 0xe21912fe, 87 },
    { 0x68e912ff, 34 },
    { 0x66e912fe, 90 },
    { 0x740912fe, 40 },
    { 0xbb191302, 66 },
    { 0x5dc91302, 98 },
    { 0xe4391304, 33 },
    { 0, 0 },
    { 0x12c91306, 90 },
    { 0x9ef91306, 23 },
    { 0, 0 },
    { 0x9c591309, 23 },
    { 0xf79130a, 98 },
    { 0x4259130b, 97 },
    { 0xe219130b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb791315, 100 },
    { 0, 0 },
    { 0x84391317, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60591322, 73 },
    { 0xa5391323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc491327, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb029132d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e191332, 90 },
    { 0x43291333, 99 },
    { 0x19e91334, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f91339, 95 },
    { 0x7291339, 95 },
    { 0, 0 },
    { 0x4e89133c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a491340, 87 },
    { 0x43391341, 61 },
    { 0xf3391341, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa339134b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec91350, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe39135b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e91364, 98 },
    { 0xe0591364, 98 },
    { 0xa1191366, 38 },
    { 0xe8591367, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dc91370, 53 },
    { 0x9891371, 33 },
    { 0x24991372, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xce091375, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979137a, 88 },
    { 0x12b9137b, 84 },
    { 0xcfb9137c, 98 },
    { 0xe859137c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4a91383, 52 },
    { 0x23791384, 55 },
    { 0x80591385, 61 },
    { 0x5f891386, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c91389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e9138c, 33 },
    { 0x2ed9138c, 55 },
    { 0, 0 },
    { 0x5f99138f, 61 },
    { 0xe759138f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a191393, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc91397, 67 },
    { 0xa9e91398, 90 },
    { 0, 0 },
    { 0xbc9139a, 98 },
    { 0x2ac9139b, 87 },
    { 0xb4a9139b, 98 },
    { 0x2839139d, 95 },
    { 0xc639139a, 95 },
    { 0, 0 },
    { 0xfa5913a0, 84 },
    { 0x9a7913a1, 81 },
    { 0xc7a913a1, 46 },
    { 0xf48913a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278913a7, 90 },
    { 0x92d913a8, 87 },
    { 0x27e913a7, 87 },
    { 0xfb3913a8, 23 },
    { 0xbc8913ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167913b0, 67 },
    { 0, 0 },
    { 0x8ae913b2, 33 },
    { 0xbb1913b3, 23 },
    { 0x891913b4, 61 },
    { 0xdcb913b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191913b9, 77 },
    { 0xf02913ba, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x873913bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3913c0, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b913c4, 55 },
    { 0xbec913c5, 33 },
    { 0xdec913c6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9913ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9913d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6913d5, 55 },
    { 0, 0 },
    { 0xc56913d7, 98 },
    { 0, 0 },
    { 0xd97913d9, 98 },
    { 0xfd7913da, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb913de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5913e3, 55 },
    { 0, 0 },
    { 0x94913e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c913e9, 23 },
    { 0x100913ea, 66 },
    { 0xb71913eb, 66 },
    { 0xa34913ec, 77 },
    { 0x150913ec, 94 },
    { 0, 0 },
    { 0x944913ef, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14913f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac913f7, 98 },
    { 0x24d913f7, 99 },
    { 0x3e0913f9, 23 },
    { 0x237913f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4913ff, 61 },
    { 0xc93913ff, 98 },
    { 0x67091400, 95 },
    { 0x457913ff, 52 },
    { 0x1f91403, 43 },
    { 0x9f191400, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x10191407, 95 },
    { 0x49e91408, 98 },
    { 0xa1b91408, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3209140e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e91411, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee91414, 90 },
    { 0x57091415, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b9141b, 90 },
    { 0x7449141c, 43 },
    { 0xdae9141d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x20491420, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d91423, 88 },
    { 0x5fc91424, 99 },
    { 0x8c91424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae91428, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d9142e, 61 },
    { 0xd369142e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d91436, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4291439, 98 },
    { 0, 0 },
    { 0x21e9143b, 98 },
    { 0xf219143b, 93 },
    { 0, 0 },
    { 0x3389143e, 90 },
    { 0x78f9143e, 66 },
    { 0xe639143f, 43 },
    { 0x28c91440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d991445, 46 },
    { 0xb7a91446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4899144d, 90 },
    { 0x7519144e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb891457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8949145a, 33 },
    { 0xe9b9145b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb91462, 46 },
    { 0xac491462, 52 },
    { 0x15e91463, 91 },
    { 0x35e91465, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a29146a, 98 },
    { 0, 0 },
    { 0x1159146c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x599146f, 78 },
    { 0x7eb91470, 43 },
    { 0xbf89146f, 64 },
    { 0, 0 },
    { 0x1a691473, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e591476, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a9147b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b291480, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e91485, 91 },
    { 0xc5791485, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b91491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7091498, 76 },
    { 0x4a691499, 33 },
    { 0x37191499, 98 },
    { 0xd1a9149b, 90 },
    { 0x1a79149c, 100 },
    { 0, 0 },
    { 0xa9a9149e, 81 },
    { 0x2ec9149f, 98 },
    { 0, 0 },
    { 0x392914a1, 23 },
    { 0x713914a2, 91 },
    { 0x80b914a3, 34 },
    { 0, 0 },
    { 0xf7b914a5, 95 },
    { 0x97e914a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57914ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec914ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220914b4, 23 },
    { 0x9c1914b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7914b8, 34 },
    { 0x87f914b8, 100 },
    { 0, 0 },
    { 0xf30914bb, 61 },
    { 0x56c914bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b914bf, 78 },
    { 0x319914c0, 85 },
    { 0xb49914bf, 61 },
    { 0x627914c2, 65 },
    { 0xb99914c3, 33 },
    { 0x9e6914bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92914c8, 68 },
    { 0xaba914c8, 61 },
    { 0, 0 },
    { 0xb94914cb, 88 },
    { 0, 0 },
    { 0xaa7914cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e914d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9914d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af914dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7914ec, 98 },
    { 0x14e914ec, 33 },
    { 0, 0 },
    { 0xc91914ef, 61 },
    { 0xd1e914f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1914f5, 33 },
    { 0, 0 },
    { 0xe9d914f7, 34 },
    { 0, 0 },
    { 0xeeb914f9, 95 },
    { 0x721914f9, 99 },
    { 0, 0 },
    { 0x48d914fc, 55 },
    { 0xc65914fc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc91501, 98 },
    { 0x23491502, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c091506, 66 },
    { 0x98491507, 23 },
    { 0x72d91508, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15491510, 101 },
    { 0x65891511, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4591514, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd91518, 91 },
    { 0xdba91519, 34 },
    { 0xa8591519, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2891523, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9091526, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b791529, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef9152e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa91535, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe391539, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f9153c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf859153f, 76 },
    { 0, 0 },
    { 0xbec91541, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7591545, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b91549, 93 },
    { 0x4fd9154a, 90 },
    { 0xf329154a, 36 },
    { 0x33d91549, 90 },
    { 0x9079154b, 98 },
    { 0xe9f9154d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf91556, 55 },
    { 0x49991557, 75 },
    { 0xe991558, 33 },
    { 0x4c191559, 78 },
    { 0xf259155a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c91560, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0291566, 19 },
    { 0x96691567, 76 },
    { 0, 0 },
    { 0x2f91569, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x299156c, 52 },
    { 0x5f39156d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb91575, 100 },
    { 0x20f91575, 96 },
    { 0x8b391577, 66 },
    { 0x5ae91577, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e9157d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18191585, 33 },
    { 0x33b91586, 87 },
    { 0x1e091587, 33 },
    { 0x82a91588, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9158b, 98 },
    { 0xe449158b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb9158f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9391593, 33 },
    { 0x4f091594, 33 },
    { 0x99091594, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5449159b, 85 },
    { 0xab19159b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae915a1, 90 },
    { 0x8e915a2, 46 },
    { 0xf85915a3, 100 },
    { 0, 0 },
    { 0xc86915a5, 98 },
    { 0x9da915a6, 93 },
    { 0x149915a6, 83 },
    { 0x3be915a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332915ac, 68 },
    { 0, 0 },
    { 0x544915ae, 91 },
    { 0, 0 },
    { 0xfbc915b0, 61 },
    { 0x550915b1, 40 },
    { 0x866915b1, 98 },
    { 0x6ab915b3, 95 },
    { 0x5ab915b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9915b9, 88 },
    { 0x4c9915ba, 55 },
    { 0x998915bb, 98 },
    { 0x13a915bb, 23 },
    { 0xbf6915bb, 90 },
    { 0xeb0915bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e915c1, 90 },
    { 0xd90915c2, 45 },
    { 0xb26915c2, 33 },
    { 0xeee915c2, 23 },
    { 0x1fc915c5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17915c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41915cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x536915ce, 61 },
    { 0xd92915ce, 98 },
    { 0x986915d0, 34 },
    { 0x57a915d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9915d5, 68 },
    { 0xa81915d5, 33 },
    { 0xd5a915d7, 43 },
    { 0xf5915d8, 90 },
    { 0, 0 },
    { 0x3c9915da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e915e5, 52 },
    { 0xf2a915e5, 87 },
    { 0x765915e5, 90 },
    { 0x6ed915e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662915f2, 67 },
    { 0xfb1915f2, 85 },
    { 0, 0 },
    { 0xf0c915f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08915f8, 61 },
    { 0, 0 },
    { 0xf58915fa, 61 },
    { 0x2c3915fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4491602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61591606, 17 },
    { 0x75691607, 43 },
    { 0xff591608, 84 },
    { 0, 0 },
    { 0x10f9160a, 65 },
    { 0xeee9160b, 53 },
    { 0xa249160c, 65 },
    { 0x6009160d, 33 },
    { 0x59f9160d, 24 },
    { 0x82b9160b, 76 },
    { 0x88891610, 90 },
    { 0x1f791610, 33 },
    { 0, 0 },
    { 0x75e91613, 66 },
    { 0x49b91614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc591618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee9161d, 23 },
    { 0x62f9161e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f91624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9809162a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19162e, 66 },
    { 0, 0 },
    { 0x42491630, 98 },
    { 0xd5391631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff91637, 55 },
    { 0x94491638, 69 },
    { 0xf491638, 33 },
    { 0, 0 },
    { 0x3b19163b, 78 },
    { 0xd329163c, 61 },
    { 0x1e29163d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81791642, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1191647, 84 },
    { 0x88791648, 61 },
    { 0x77d91648, 90 },
    { 0xf639164a, 61 },
    { 0x2a19164b, 98 },
    { 0x1c39164c, 90 },
    { 0xda49164d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf91653, 95 },
    { 0xea991654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e91657, 99 },
    { 0xfe891658, 99 },
    { 0xdd91659, 55 },
    { 0x3c29165a, 78 },
    { 0, 0 },
    { 0x279165c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74991660, 76 },
    { 0x8ef91660, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a49166a, 35 },
    { 0, 0 },
    { 0x9b99166c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3509166f, 76 },
    { 0x77a91670, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1891673, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc191676, 98 },
    { 0, 0 },
    { 0xaab91678, 23 },
    { 0, 0 },
    { 0xbca9167a, 98 },
    { 0, 0 },
    { 0x8cd9167c, 23 },
    { 0x44e9167d, 66 },
    { 0xdab9167d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e91685, 98 },
    { 0xd4c91685, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d79168a, 78 },
    { 0, 0 },
    { 0x40b9168c, 98 },
    { 0xdbc9168c, 98 },
    { 0, 0 },
    { 0xb799168f, 33 },
    { 0x1b891690, 46 },
    { 0x74f91691, 76 },
    { 0xed291692, 33 },
    { 0, 0 },
    { 0xfa691694, 33 },
    { 0x6e291694, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa91698, 43 },
    { 0, 0 },
    { 0x3a19169a, 55 },
    { 0xd989169b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5916a0, 33 },
    { 0x871916a1, 43 },
    { 0xa18916a1, 75 },
    { 0x7da916a3, 33 },
    { 0x506916a4, 98 },
    { 0x251916a5, 97 },
    { 0, 0 },
    { 0x452916a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc916aa, 98 },
    { 0xadf916ab, 42 },
    { 0x5f5916ac, 65 },
    { 0x95c916ad, 98 },
    { 0xc95916ad, 23 },
    { 0xef4916ac, 66 },
    { 0, 0 },
    { 0xe17916b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab916b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a916be, 33 },
    { 0, 0 },
    { 0x1ce916c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568916c6, 76 },
    { 0x8c5916c6, 95 },
    { 0x8ea916c8, 90 },
    { 0xc19916c6, 95 },
    { 0xbef916ca, 85 },
    { 0xe3f916c7, 61 },
    { 0, 0 },
    { 0x6b2916cd, 23 },
    { 0, 0 },
    { 0x8a916cf, 61 },
    { 0x693916d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7916d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e916d6, 55 },
    { 0xf01916d7, 95 },
    { 0, 0 },
    { 0xa8a916d9, 76 },
    { 0, 0 },
    { 0xf67916db, 95 },
    { 0x772916dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7916e2, 34 },
    { 0xcf9916e3, 90 },
    { 0x29916e4, 99 },
    { 0, 0 },
    { 0x181916e6, 33 },
    { 0x65b916e7, 23 },
    { 0x4b8916e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1916eb, 95 },
    { 0x52b916ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34916ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f916f9, 98 },
    { 0xa5f916fa, 33 },
    { 0x735916fb, 23 },
    { 0, 0 },
    { 0xb4e916fd, 84 },
    { 0, 0 },
    { 0xf50916ff, 90 },
    { 0x7ea91700, 61 },
    { 0xb9591700, 95 },
    { 0x70c91702, 87 },
    { 0x9f491703, 46 },
    { 0xbf91704, 55 },
    { 0x7e391704, 90 },
    { 0, 0 },
    { 0xa1591707, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e291717, 33 },
    { 0xbe991718, 40 },
    { 0x73491718, 61 },
    { 0x6a291718, 61 },
    { 0, 0 },
    { 0x11b9171c, 33 },
    { 0x6fc9171d, 98 },
    { 0x58f9171e, 34 },
    { 0xab69171e, 23 },
    { 0x4de9171d, 98 },
    { 0xf7691720, 98 },
    { 0x8e991722, 33 },
    { 0, 0 },
    { 0xa4191724, 33 },
    { 0, 0 },
    { 0x40e91726, 23 },
    { 0x2d391726, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed9172a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a39172e, 84 },
    { 0x98d9172f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc991732, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52491735, 90 },
    { 0x7f391735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d9173e, 46 },
    { 0xe019173f, 33 },
    { 0x2af91740, 43 },
    { 0, 0 },
    { 0xb1791742, 93 },
    { 0, 0 },
    { 0xe3291744, 33 },
    { 0x88291744, 52 },
    { 0xefc91746, 93 },
    { 0, 0 },
    { 0x73491748, 33 },
    { 0, 0 },
    { 0x1e19174a, 98 },
    { 0, 0 },
    { 0x39f9174c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40491756, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47691760, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2891763, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce591769, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e09176e, 91 },
    { 0, 0 },
    { 0x56791770, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2491775, 88 },
    { 0x4c391775, 95 },
    { 0xa1791777, 90 },
    { 0, 0 },
    { 0xb8d91779, 90 },
    { 0x8289177a, 98 },
    { 0x42d9177b, 90 },
    { 0x4ef9177b, 93 },
    { 0x9339177d, 33 },
    { 0x9cc9177e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c91782, 33 },
    { 0x9e391782, 76 },
    { 0x18291784, 90 },
    { 0x5bd91782, 90 },
    { 0xdff91786, 33 },
    { 0x4a191786, 65 },
    { 0x41591788, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9178c, 33 },
    { 0x8b9178d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb91794, 23 },
    { 0xea091795, 95 },
    { 0xa4791796, 84 },
    { 0x5c391797, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5229179a, 55 },
    { 0, 0 },
    { 0x37c9179c, 34 },
    { 0, 0 },
    { 0x81f9179e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9917a7, 95 },
    { 0x8e3917a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301917ae, 77 },
    { 0, 0 },
    { 0x287917b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc917b6, 66 },
    { 0x207917b7, 98 },
    { 0x410917b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700917bc, 69 },
    { 0, 0 },
    { 0x36a917be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7917c2, 23 },
    { 0xee1917c3, 66 },
    { 0, 0 },
    { 0xbea917c5, 95 },
    { 0xf99917c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36917cc, 61 },
    { 0x252917cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204917d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5917d8, 33 },
    { 0xfc917d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368917e3, 98 },
    { 0, 0 },
    { 0xa08917e5, 42 },
    { 0, 0 },
    { 0xc75917e7, 90 },
    { 0x29f917e7, 99 },
    { 0, 0 },
    { 0xe5b917ea, 61 },
    { 0x2bb917eb, 78 },
    { 0x8f9917ec, 33 },
    { 0x1d2917ea, 33 },
    { 0xf34917ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558917f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91917f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x666917fa, 22 },
    { 0xed3917fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7917fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x22291801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a91805, 95 },
    { 0, 0 },
    { 0xfac91807, 100 },
    { 0x62f91808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb629180b, 55 },
    { 0x7349180c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb91812, 33 },
    { 0xdf291813, 90 },
    { 0, 0 },
    { 0xff191815, 98 },
    { 0x48b91816, 19 },
    { 0, 0 },
    { 0x18b91818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca9181d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5091820, 75 },
    { 0, 0 },
    { 0xa9391822, 61 },
    { 0x14091823, 61 },
    { 0x72291822, 76 },
    { 0x8ff91825, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a691829, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba9182d, 61 },
    { 0x44a9182e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce91831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca191838, 98 },
    { 0xced91838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789183d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca91843, 95 },
    { 0x76991843, 69 },
    { 0, 0 },
    { 0x14691846, 85 },
    { 0x1fd91847, 61 },
    { 0x8e691848, 19 },
    { 0, 0 },
    { 0x9f09184a, 33 },
    { 0x5be9184b, 61 },
    { 0x52f9184c, 93 },
    { 0x2bb9184c, 90 },
    { 0, 0 },
    { 0xfef9184f, 90 },
    { 0xcdf91850, 61 },
    { 0xf7491850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd91858, 55 },
    { 0x7191859, 78 },
    { 0xba391859, 61 },
    { 0xabf91858, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff9185e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f91861, 33 },
    { 0x5a391862, 23 },
    { 0, 0 },
    { 0xd0691864, 61 },
    { 0x26091865, 98 },
    { 0x54291866, 91 },
    { 0x23b91864, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e9186a, 87 },
    { 0xc5e9186b, 55 },
    { 0, 0 },
    { 0xe159186d, 98 },
    { 0, 0 },
    { 0x7459186f, 23 },
    { 0x3e891870, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca91873, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5191877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4769187a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c891880, 23 },
    { 0x23891881, 65 },
    { 0xe2491880, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f91885, 52 },
    { 0x74f91886, 17 },
    { 0x57491887, 31 },
    { 0xf6191888, 95 },
    { 0x6d91886, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5e91890, 76 },
    { 0, 0 },
    { 0x44891892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64491897, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc9189c, 99 },
    { 0x6fd9189c, 95 },
    { 0, 0 },
    { 0xfd79189f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1918a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c918aa, 33 },
    { 0x4bf918ab, 61 },
    { 0xcb9918ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x702918af, 23 },
    { 0x9f0918b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f918b4, 87 },
    { 0x106918b5, 95 },
    { 0xca8918b5, 95 },
    { 0, 0 },
    { 0x414918b8, 98 },
    { 0, 0 },
    { 0x4ae918ba, 77 },
    { 0x7f3918bb, 91 },
    { 0x1c2918ba, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0918c0, 33 },
    { 0xf61918c0, 33 },
    { 0x92e918c2, 43 },
    { 0, 0 },
    { 0x5eb918c4, 90 },
    { 0, 0 },
    { 0x503918c6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9918cd, 76 },
    { 0x49918ce, 98 },
    { 0x37a918ce, 76 },
    { 0x12e918d0, 90 },
    { 0, 0 },
    { 0xc9c918d2, 90 },
    { 0x1a4918d3, 95 },
    { 0x743918d4, 78 },
    { 0x56f918d4, 33 },
    { 0, 0 },
    { 0x9ee918d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9918db, 99 },
    { 0, 0 },
    { 0xde918dd, 90 },
    { 0x3b9918de, 33 },
    { 0x88d918df, 93 },
    { 0x7e8918e0, 61 },
    { 0xb86918dd, 90 },
    { 0x170918e2, 90 },
    { 0x710918e2, 98 },
    { 0x8ad918e4, 61 },
    { 0x855918e3, 78 },
    { 0xb63918e6, 46 },
    { 0x1ac918e7, 95 },
    { 0x48b918e8, 100 },
    { 0xdaa918e8, 34 },
    { 0xcfc918e7, 82 },
    { 0x5dd918ea, 78 },
    { 0, 0 },
    { 0x869918ed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x84918f0, 85 },
    { 0xf90918f1, 66 },
    { 0xbf1918f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5918f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4918fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb191900, 34 },
    { 0, 0 },
    { 0x49891902, 61 },
    { 0, 0 },
    { 0x9ac91904, 23 },
    { 0x16c91904, 93 },
    { 0, 0 },
    { 0x42a91907, 61 },
    { 0xe7391908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7469190d, 78 },
    { 0x36e9190e, 55 },
    { 0x27a9190f, 90 },
    { 0, 0 },
    { 0x55291911, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91916, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09191a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8591920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f791925, 77 },
    { 0xa4d91925, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e9192c, 61 },
    { 0x31f9192d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff491931, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf091936, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9629193a, 90 },
    { 0xd1f9193a, 34 },
    { 0x6a49193b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb9193f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf91942, 17 },
    { 0xb0391942, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d91946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa809194b, 87 },
    { 0x319194b, 91 },
    { 0, 0 },
    { 0x2789194e, 87 },
    { 0xf739194e, 98 },
    { 0, 0 },
    { 0xc6791951, 90 },
    { 0x92c91952, 23 },
    { 0, 0 },
    { 0xe6691954, 61 },
    { 0xbcf91955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb79195c, 95 },
    { 0xae99195d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58391961, 76 },
    { 0x5b091962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc69196b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87791971, 85 },
    { 0x40391972, 85 },
    { 0xaef91973, 90 },
    { 0, 0 },
    { 0x20d91975, 93 },
    { 0xb7c91976, 95 },
    { 0, 0 },
    { 0x23e91978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e9197b, 23 },
    { 0xfc39197c, 33 },
    { 0xeef9197d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc91983, 61 },
    { 0x38d91983, 43 },
    { 0x8991985, 23 },
    { 0x51e91986, 95 },
    { 0xee791987, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe839198b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51f91990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9391993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7391997, 61 },
    { 0xe9a91997, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b9199c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309919a1, 34 },
    { 0xc01919a2, 61 },
    { 0x5ea919a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69919ac, 23 },
    { 0x3db919ad, 95 },
    { 0, 0 },
    { 0xb69919af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54919b9, 83 },
    { 0xf7c919b9, 52 },
    { 0x146919ba, 98 },
    { 0x29f919ba, 61 },
    { 0xf29919ba, 90 },
    { 0x1a4919be, 76 },
    { 0xb8f919bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70919c2, 90 },
    { 0, 0 },
    { 0x51a919c4, 90 },
    { 0xf3a919c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161919c9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9919cd, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5919d1, 33 },
    { 0x3d9919d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9919d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47919d8, 84 },
    { 0x90d919d9, 61 },
    { 0, 0 },
    { 0x1ce919db, 85 },
    { 0, 0 },
    { 0xcaf919dd, 98 },
    { 0x2f9919dd, 95 },
    { 0xaee919df, 90 },
    { 0xd02919e0, 75 },
    { 0x5d6919e0, 95 },
    { 0x86b919e2, 46 },
    { 0xd20919e1, 17 },
    { 0xc83919e3, 55 },
    { 0x81c919e5, 61 },
    { 0x2d2919e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48919f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2919f3, 90 },
    { 0x472919f4, 55 },
    { 0x97f919f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b919f8, 43 },
    { 0xfbe919f8, 95 },
    { 0, 0 },
    { 0x7c9919fb, 98 },
    { 0x442919fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd919ff, 93 },
    { 0x732919ff, 90 },
    { 0xea691a01, 84 },
    { 0xe3f91a02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa91a09, 77 },
    { 0x37991a09, 61 },
    { 0xa8891a0a, 98 },
    { 0xf7c91a0a, 33 },
    { 0xc2f91a0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x52891a10, 33 },
    { 0, 0 },
    { 0x32b91a12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a91a1e, 33 },
    { 0x1ef91a1f, 93 },
    { 0, 0 },
    { 0x63691a21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8991a24, 90 },
    { 0x3bb91a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e591a30, 61 },
    { 0xa5191a30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37891a35, 68 },
    { 0x8e491a36, 46 },
    { 0xe2191a36, 85 },
    { 0x4c191a38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c491a40, 87 },
    { 0, 0 },
    { 0x54c91a42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2091a45, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c91a49, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf791a4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3591a4f, 98 },
    { 0xf3191a50, 23 },
    { 0xa4f91a51, 76 },
    { 0x92191a4f, 98 },
    { 0x86791a52, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc91a56, 76 },
    { 0xad491a57, 95 },
    { 0xda291a58, 61 },
    { 0xdd791a58, 43 },
    { 0x33e91a58, 98 },
    { 0x44391a5b, 76 },
    { 0, 0 },
    { 0x20191a5d, 93 },
    { 0x42b91a5e, 43 },
    { 0x51691a5e, 90 },
    { 0x31691a60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5391a63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9291a66, 23 },
    { 0xa1491a67, 52 },
    { 0, 0 },
    { 0x72491a69, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df91a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b91a72, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2391a77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x75091a7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d91a83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f191a86, 96 },
    { 0, 0 },
    { 0x57591a88, 98 },
    { 0x19291a88, 81 },
    { 0xc3291a8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e91a8f, 93 },
    { 0, 0 },
    { 0xd7491a91, 23 },
    { 0, 0 },
    { 0x71a91a93, 78 },
    { 0x4ec91a94, 33 },
    { 0, 0 },
    { 0xe2191a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa291a99, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d91a9c, 89 },
    { 0, 0 },
    { 0x69391a9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7991aa4, 61 },
    { 0, 0 },
    { 0x23791aa6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e191aa9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2891aac, 87 },
    { 0x53391aad, 96 },
    { 0, 0 },
    { 0xfeb91aaf, 99 },
    { 0x58091ab0, 61 },
    { 0x62591ab1, 76 },
    { 0x63d91aaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c91ab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac91abc, 33 },
    { 0xa8b91abc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0991ac0, 84 },
    { 0, 0 },
    { 0xa4891ac2, 98 },
    { 0x41291ac3, 91 },
    { 0xb2b91ac4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65391ad0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63691ad4, 87 },
    { 0x59b91ad4, 23 },
    { 0x20991ad6, 90 },
    { 0xd8691ad6, 98 },
    { 0xa0591ad8, 52 },
    { 0x10091ad8, 87 },
    { 0, 0 },
    { 0x9fc91adb, 78 },
    { 0, 0 },
    { 0x50c91add, 95 },
    { 0xcee91ade, 95 },
    { 0x5191add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab91aeb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d091aee, 87 },
    { 0x90f91aef, 23 },
    { 0xe0791aef, 90 },
    { 0x8b991aef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49591af8, 77 },
    { 0x16791af9, 23 },
    { 0xa6a91af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed891afd, 66 },
    { 0x86091afd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d91b03, 43 },
    { 0, 0 },
    { 0x39091b05, 76 },
    { 0x2991b06, 85 },
    { 0x8c591b07, 90 },
    { 0x38c91b07, 61 },
    { 0, 0 },
    { 0x18091b0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c91b0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf391b11, 91 },
    { 0, 0 },
    { 0xb0891b13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2391b1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b91b23, 43 },
    { 0, 0 },
    { 0x39d91b25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4591b32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1791b36, 88 },
    { 0, 0 },
    { 0x60291b38, 23 },
    { 0xd0191b39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc91b3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a91b47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae91b52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2091b5d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2591b61, 76 },
    { 0xfac91b62, 34 },
    { 0xcb791b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54391b6a, 23 },
    { 0x28391b6b, 90 },
    { 0x61891b6c, 99 },
    { 0, 0 },
    { 0xe2491b6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12891b71, 61 },
    { 0x12291b72, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b91b75, 95 },
    { 0, 0 },
    { 0xa0891b77, 78 },
    { 0, 0 },
    { 0x1fa91b79, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d91b81, 68 },
    { 0xd791b82, 61 },
    { 0xc7491b83, 95 },
    { 0x29e91b84, 78 },
    { 0xaf291b85, 33 },
    { 0xcbb91b86, 61 },
    { 0x45b91b81, 98 },
    { 0x7ab91b83, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5191b8c, 91 },
    { 0x85091b8d, 23 },
    { 0xda591b8e, 14 },
    { 0x34a91b8d, 52 },
    { 0xb3691b8d, 87 },
    { 0x89791b91, 23 },
    { 0x8d191b8d, 98 },
    { 0x64491b91, 84 },
    { 0xd8391b8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5391b9c, 33 },
    { 0xb5491b9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x89091ba0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ba3, 33 },
    { 0x40791ba4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f691bab, 93 },
    { 0xa6891bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d91bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20691bb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6791bb6, 99 },
    { 0x8ba91bb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f91bba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e91bbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96491bc2, 61 },
    { 0, 0 },
    { 0xa3d91bc4, 97 },
    { 0xb3391bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7891bca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9091bce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17791bd2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa91bd7, 61 },
    { 0xa5c91bd8, 23 },
    { 0xe3591bd9, 46 },
    { 0x11a91bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f91bdd, 65 },
    { 0xe3191bde, 85 },
    { 0, 0 },
    { 0xd5191be0, 90 },
    { 0xfd091be0, 78 },
    { 0, 0 },
    { 0x83991be3, 23 },
    { 0xd2991be4, 95 },
    { 0xfb391be3, 95 },
    { 0x63f91be6, 61 },
    { 0, 0 },
    { 0xf2591be8, 100 },
    { 0xf5191be9, 46 },
    { 0xd7f91be8, 23 },
    { 0x2b091beb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf391bee, 23 },
    { 0xaa391bef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93291bf5, 95 },
    { 0x26491bf6, 90 },
    { 0xed391bf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe191bfc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10a91c00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a91c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9091c08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f91c0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53391c0f, 33 },
    { 0x10591c10, 33 },
    { 0xc7e91c11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a991c16, 90 },
    { 0x8a91c16, 99 },
    { 0x68a91c18, 90 },
    { 0xb7091c19, 90 },
    { 0x41891c1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c791c1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e91c29, 91 },
    { 0x7d091c2a, 34 },
    { 0, 0 },
    { 0x8591c2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5291c30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db91c37, 78 },
    { 0xd0b91c38, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55091c3c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d291c3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f91c42, 33 },
    { 0xd7091c42, 46 },
    { 0xbe091c44, 23 },
    { 0xa791c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0091c52, 34 },
    { 0xced91c53, 33 },
    { 0x7ff91c54, 46 },
    { 0x24a91c55, 100 },
    { 0xae291c56, 95 },
    { 0x3ac91c53, 66 },
    { 0xee291c56, 33 },
    { 0x28791c59, 98 },
    { 0, 0 },
    { 0x9cc91c5b, 90 },
    { 0x5ab91c5b, 34 },
    { 0xeeb91c5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e891c61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3691c67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea91c6a, 35 },
    { 0, 0 },
    { 0x23291c6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39491c70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d491c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77891c7e, 33 },
    { 0x3a91c7f, 76 },
    { 0x68f91c7f, 98 },
    { 0x64691c81, 95 },
    { 0x9491c82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53191c85, 90 },
    { 0x3d691c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d91c8c, 23 },
    { 0xd8c91c8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d991c90, 97 },
    { 0, 0 },
    { 0x24991c92, 88 },
    { 0, 0 },
    { 0xa5d91c94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f91c97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83891c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e491c9e, 46 },
    { 0, 0 },
    { 0xc4891ca0, 90 },
    { 0xd5091ca1, 90 },
    { 0xe3f91ca1, 95 },
    { 0xccf91ca3, 90 },
    { 0x52891ca1, 98 },
    { 0xc6091ca0, 98 },
    { 0, 0 },
    { 0x44e91ca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf791caa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41091cae, 55 },
    { 0x1e591caf, 91 },
    { 0, 0 },
    { 0xa6191cb1, 66 },
    { 0x70c91cb2, 98 },
    { 0, 0 },
    { 0x9e891cb4, 90 },
    { 0, 0 },
    { 0xb1591cb6, 76 },
    { 0xe4391cb7, 66 },
    { 0x9291cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab891cbd, 95 },
    { 0, 0 },
    { 0x67791cbf, 95 },
    { 0x4e791cbf, 100 },
    { 0x2db91cbf, 101 },
    { 0xb1691cc2, 87 },
    { 0x91791cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0191cc8, 33 },
    { 0xc6891cc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12991ccd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b91cd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d91cd7, 90 },
    { 0, 0 },
    { 0x28d91cd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72b91cdc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e491ce3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37691ce8, 76 },
    { 0, 0 },
    { 0x93491cea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0291cf1, 90 },
    { 0x19891cf2, 55 },
    { 0xdcc91cf1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3891cf7, 90 },
    { 0x91b91cf8, 90 },
    { 0xb1a91cf7, 34 },
    { 0xa0c91cfa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d91cff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb091d04, 33 },
    { 0xf8391d05, 95 },
    { 0x55691d06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f91d09, 84 },
    { 0, 0 },
    { 0xf0391d0b, 87 },
    { 0x4f391d0c, 23 },
    { 0xac91d0c, 76 },
    { 0x24b91d0c, 23 },
    { 0x4d891d0f, 90 },
    { 0x93b91d0f, 23 },
    { 0xb7591d10, 90 },
    { 0x39291d12, 88 },
    { 0x5f091d12, 76 },
    { 0xac191d0f, 33 },
    { 0x17191d14, 46 },
    { 0x36691d16, 33 },
    { 0xbb691d17, 46 },
    { 0xc6091d17, 76 },
    { 0, 0 },
    { 0xf6a91d1a, 43 },
    { 0xf8191d1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83291d22, 98 },
    { 0, 0 },
    { 0x36a91d24, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b91d28, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f91d2f, 90 },
    { 0x67c91d30, 67 },
    { 0xb2d91d31, 98 },
    { 0x36191d31, 34 },
    { 0x18291d33, 35 },
    { 0xd6791d33, 33 },
    { 0x7b191d35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a91d39, 46 },
    { 0x55091d3a, 90 },
    { 0xa4291d3b, 91 },
    { 0xffb91d3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a91d3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f391d42, 87 },
    { 0x8891d43, 55 },
    { 0x9de91d44, 95 },
    { 0x77591d42, 95 },
    { 0x7191d44, 84 },
    { 0x16991d47, 76 },
    { 0x2e191d44, 55 },
    { 0x96991d49, 75 },
    { 0xe7191d4a, 23 },
    { 0x3a891d4b, 61 },
    { 0xdcc91d4b, 52 },
    { 0x8cb91d4a, 76 },
    { 0xc6491d4e, 90 },
    { 0x7a291d4e, 95 },
    { 0x8e891d4c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84391d54, 88 },
    { 0x40391d55, 88 },
    { 0x8f791d56, 98 },
    { 0xba391d55, 61 },
    { 0x6c791d58, 67 },
    { 0x5ac91d58, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c91d5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43791d61, 61 },
    { 0xda091d61, 93 },
    { 0x97391d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d91d67, 95 },
    { 0, 0 },
    { 0x81191d69, 61 },
    { 0x38391d69, 33 },
    { 0x3a291d6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81391d6e, 23 },
    { 0, 0 },
    { 0xc5491d70, 44 },
    { 0, 0 },
    { 0xe4e91d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d91d78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x691d7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6891d7f, 90 },
    { 0x37791d80, 90 },
    { 0xb8591d81, 66 },
    { 0x9ee91d82, 46 },
    { 0xe3e91d80, 91 },
    { 0, 0 },
    { 0xdf591d85, 90 },
    { 0x6dc91d86, 76 },
    { 0xb6791d86, 98 },
    { 0xcf191d88, 33 },
    { 0x48491d89, 87 },
    { 0x8df91d8a, 76 },
    { 0xc7491d86, 99 },
    { 0xc8491d88, 61 },
    { 0xa1891d89, 98 },
    { 0x6ef91d8e, 90 },
    { 0xfaf91d8e, 76 },
    { 0xa1991d8e, 90 },
    { 0x8ae91d91, 81 },
    { 0x29b91d87, 95 },
    { 0x73e91d93, 95 },
    { 0xb4791d94, 95 },
    { 0x11291d93, 98 },
    { 0x70791d96, 61 },
    { 0x5cc91d97, 95 },
    { 0xe6291d95, 23 },
    { 0x77e91d8a, 61 },
    { 0x21091d9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce91d9d, 66 },
    { 0, 0 },
    { 0x60e91d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c691da4, 78 },
    { 0, 0 },
    { 0x9de91da6, 34 },
    { 0x6ec91da7, 33 },
    { 0xeaa91da7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b91dab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3891dae, 43 },
    { 0, 0 },
    { 0xa4991db0, 38 },
    { 0, 0 },
    { 0xd7a91db2, 90 },
    { 0, 0 },
    { 0x1be91db4, 99 },
    { 0x60191db5, 52 },
    { 0x8ae91db5, 23 },
    { 0, 0 },
    { 0x81891db8, 91 },
    { 0xcd591db9, 90 },
    { 0x4da91dba, 33 },
    { 0xa8391dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f691dbf, 61 },
    { 0x59c91dbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e991dc4, 76 },
    { 0x76791dc5, 99 },
    { 0x6d491dc6, 90 },
    { 0x5b891dc5, 91 },
    { 0x16591dc4, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e91dcb, 76 },
    { 0xad991dcc, 34 },
    { 0xbcd91dcb, 33 },
    { 0x6ff91dcd, 34 },
    { 0, 0 },
    { 0xba291dd0, 76 },
    { 0xab691dd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10691dd5, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c091dd9, 61 },
    { 0, 0 },
    { 0xe5f91ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81691dde, 23 },
    { 0, 0 },
    { 0xdab91de0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c91de3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f791de6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c291dea, 90 },
    { 0, 0 },
    { 0x97a91dec, 55 },
    { 0x7c791ded, 90 },
    { 0x8b791dee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e91df4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e91df8, 33 },
    { 0x92791df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c91dfc, 90 },
    { 0xddb91dfd, 76 },
    { 0x27191dfe, 90 },
    { 0x10591dff, 23 },
    { 0xa4191dfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f91e03, 52 },
    { 0x5f591e03, 34 },
    { 0, 0 },
    { 0x37f91e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e491e09, 78 },
    { 0x2ba91e0a, 55 },
    { 0xb0191e0b, 23 },
    { 0, 0 },
    { 0xcae91e0d, 66 },
    { 0xfba91e0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2391e15, 33 },
    { 0x15591e16, 23 },
    { 0x2a991e17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b591e1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e91e1f, 90 },
    { 0x15391e1f, 90 },
    { 0x22691e1f, 33 },
    { 0x25291e22, 95 },
    { 0xa1391e20, 95 },
    { 0x6991e24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c91e29, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8891e2c, 61 },
    { 0x69991e2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95991e31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77691e35, 98 },
    { 0x5e391e35, 34 },
    { 0xe3591e37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15891e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84891e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc691e4d, 93 },
    { 0xc4991e4d, 95 },
    { 0x3591e4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50991e52, 90 },
    { 0x7a291e53, 56 },
    { 0, 0 },
    { 0xddc91e55, 40 },
    { 0, 0 },
    { 0x7df91e57, 90 },
    { 0, 0 },
    { 0x7c91e59, 95 },
    { 0xb5891e5a, 33 },
    { 0xb3e91e5b, 43 },
    { 0xd8f91e5b, 23 },
    { 0x2f691e5a, 69 },
    { 0xdc391e5b, 93 },
    { 0x57291e5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29391e63, 90 },
    { 0xfa491e64, 43 },
    { 0, 0 },
    { 0xb4991e66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a91e69, 98 },
    { 0xd6b91e69, 33 },
    { 0x8f491e6b, 98 },
    { 0x79391e6b, 43 },
    { 0x2ce91e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe091e70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa91e73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd91e76, 99 },
    { 0, 0 },
    { 0x2f491e78, 66 },
    { 0, 0 },
    { 0x98b91e7a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd91e82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e91e85, 90 },
    { 0xe0991e85, 90 },
    { 0x9b491e85, 67 },
    { 0, 0 },
    { 0xcfa91e89, 95 },
    { 0xae591e8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37191e8e, 76 },
    { 0x86791e8e, 55 },
    { 0xafa91e90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab91e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58391e97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75791e9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e91e9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39791ea1, 85 },
    { 0xb7d91ea1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82091ea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a691eac, 90 },
    { 0x2ef91ead, 95 },
    { 0x1ee91eae, 90 },
    { 0xfba91eaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4191eb2, 80 },
    { 0, 0 },
    { 0xb9e91eb4, 46 },
    { 0x4f91eb4, 90 },
    { 0x8ff91eb4, 95 },
    { 0x33391eb7, 90 },
    { 0x7f291eb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59691ec0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8391ec7, 61 },
    { 0x70c91ec8, 76 },
    { 0xa0f91ec9, 46 },
    { 0xe5891ec9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41991ed0, 95 },
    { 0x80391ed1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e91ed4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff991ed7, 98 },
    { 0x73991ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8591edb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc691ee0, 61 },
    { 0, 0 },
    { 0x2cb91ee2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac691ee7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac91eef, 98 },
    { 0x16c91ef0, 43 },
    { 0, 0 },
    { 0x2d591ef2, 85 },
    { 0x38491ef3, 55 },
    { 0, 0 },
    { 0x66791ef5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12891ef9, 61 },
    { 0, 0 },
    { 0x5b691efb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29891eff, 52 },
    { 0xb4191f00, 61 },
    { 0x83b91eff, 23 },
    { 0xf6391f02, 87 },
    { 0x26791f03, 87 },
    { 0xeae91eff, 55 },
    { 0x42e91f05, 76 },
    { 0x86a91f06, 99 },
    { 0xec491f07, 33 },
    { 0, 0 },
    { 0x57b91f09, 100 },
    { 0, 0 },
    { 0xf8c91f0b, 78 },
    { 0x74b91f0b, 46 },
    { 0xac991f0c, 61 },
    { 0x3a491f0e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6791f11, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6691f15, 46 },
    { 0x61a91f16, 61 },
    { 0, 0 },
    { 0x4ff91f18, 61 },
    { 0xff391f19, 90 },
    { 0xcbe91f1a, 61 },
    { 0x64991f1b, 90 },
    { 0xdf991f18, 90 },
    { 0x1e891f1d, 75 },
    { 0x86f91f18, 90 },
    { 0x96591f1b, 38 },
    { 0x67291f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1291f24, 33 },
    { 0, 0 },
    { 0x9e191f26, 61 },
    { 0x26191f26, 61 },
    { 0x2df91f28, 95 },
    { 0x28191f26, 33 },
    { 0xe1991f2a, 61 },
    { 0, 0 },
    { 0xa7091f2c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b91f31, 46 },
    { 0x1bd91f32, 61 },
    { 0xdd091f32, 34 },
    { 0xef691f31, 90 },
    { 0x54d91f35, 90 },
    { 0, 0 },
    { 0x2c191f37, 95 },
    { 0, 0 },
    { 0xb2e91f39, 43 },
    { 0xbb791f3a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xab391f3d, 43 },
    { 0, 0 },
    { 0xb3091f3f, 90 },
    { 0x79291f40, 91 },
    { 0xc391f3f, 93 },
    { 0x26391f41, 76 },
    { 0x2d691f43, 91 },
    { 0xc7191f40, 55 },
    { 0x16991f45, 78 },
    { 0xf8091f46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf491f49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb491f4d, 61 },
    { 0, 0 },
    { 0x57891f4f, 61 },
    { 0, 0 },
    { 0xae791f51, 91 },
    { 0, 0 },
    { 0xda991f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3691f5a, 88 },
    { 0xa3191f5a, 33 },
    { 0xbed91f5a, 33 },
    { 0x2a091f5b, 90 },
    { 0x37191f5e, 90 },
    { 0xaa791f5f, 90 },
    { 0x76091f5e, 90 },
    { 0, 0 },
    { 0xfb191f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3891f68, 90 },
    { 0x2f91f69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7891f6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee991f70, 43 },
    { 0x2f91f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a91f75, 61 },
    { 0x62191f75, 65 },
    { 0, 0 },
    { 0x54f91f78, 34 },
    { 0, 0 },
    { 0xd8891f7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e91f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb91f84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c91f87, 98 },
    { 0x43b91f87, 99 },
    { 0, 0 },
    { 0xf0391f8a, 55 },
    { 0, 0 },
    { 0xa1391f8c, 61 },
    { 0xc4091f8c, 23 },
    { 0, 0 },
    { 0xff091f8f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62391f93, 76 },
    { 0x71f91f94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3091f9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e91fa2, 90 },
    { 0x61f91fa2, 53 },
    { 0x3a191fa4, 67 },
    { 0x71091fa5, 84 },
    { 0xdf191fa6, 90 },
    { 0xd2791fa7, 61 },
    { 0xe5091fa4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e391fad, 61 },
    { 0xf9391fad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3591fb4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b91fbe, 17 },
    { 0, 0 },
    { 0x51091fc0, 90 },
    { 0x35591fc1, 90 },
    { 0x88c91fc2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36291fc6, 61 },
    { 0, 0 },
    { 0x5991fc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46891fce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee491fd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2391fd8, 33 },
    { 0x19391fd8, 91 },
    { 0xae891fd9, 95 },
    { 0x89a91fda, 46 },
    { 0xea291fdc, 61 },
    { 0x82d91fdc, 33 },
    { 0x86991fdd, 66 },
    { 0x56291fde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f691fe2, 61 },
    { 0x76591fe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d791fe6, 61 },
    { 0, 0 },
    { 0x30791fe8, 97 },
    { 0xc9b91fe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed91fec, 46 },
    { 0x21391fed, 55 },
    { 0x3d791fee, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb91ff4, 61 },
    { 0, 0 },
    { 0xf2291ff6, 81 },
    { 0xf5791ff7, 95 },
    { 0xf8091ff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc91ffe, 33 },
    { 0x37391fff, 64 },
    { 0x47192000, 90 },
    { 0x1da92000, 43 },
    { 0, 0 },
    { 0x24b92003, 33 },
    { 0x45892004, 87 },
    { 0xdb692004, 61 },
    { 0xb0992006, 90 },
    { 0x5f692005, 94 },
    { 0x23d92008, 33 },
    { 0x29b92009, 76 },
    { 0, 0 },
    { 0xbc49200b, 52 },
    { 0x2089200c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc92012, 91 },
    { 0xa8992013, 100 },
    { 0x8c192012, 83 },
    { 0xfda92014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1092019, 90 },
    { 0x5239201a, 61 },
    { 0x70e9201a, 96 },
    { 0, 0 },
    { 0x1a79201d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9892020, 91 },
    { 0x35792021, 61 },
    { 0x31e92022, 90 },
    { 0x6da92023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde992026, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc079202d, 75 },
    { 0, 0 },
    { 0x1d39202f, 33 },
    { 0x5c292030, 34 },
    { 0x2bb92031, 61 },
    { 0xd0992032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d39203d, 61 },
    { 0xde89203e, 17 },
    { 0x20c9203f, 33 },
    { 0, 0 },
    { 0x65992041, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c292044, 61 },
    { 0x27b92044, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4392049, 52 },
    { 0x5f092049, 77 },
    { 0x90f9204b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33292051, 61 },
    { 0x34892051, 76 },
    { 0xda992053, 76 },
    { 0xbd592053, 98 },
    { 0x8f892053, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5192058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe089205b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7489206e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e392073, 61 },
    { 0xe8992074, 87 },
    { 0x3ee92075, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78492078, 90 },
    { 0x65f92079, 61 },
    { 0xbdc92078, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4592082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c92089, 33 },
    { 0xa3d92089, 77 },
    { 0xb0f9208b, 84 },
    { 0xee69208c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5892091, 33 },
    { 0, 0 },
    { 0xd0e92093, 66 },
    { 0x56c92093, 61 },
    { 0x3b892095, 33 },
    { 0, 0 },
    { 0xdfb92097, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a9209f, 33 },
    { 0, 0 },
    { 0x92920a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2920a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967920ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187920be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d920c3, 33 },
    { 0, 0 },
    { 0xf4a920c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12920ca, 90 },
    { 0, 0 },
    { 0xc2d920cc, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5920cf, 61 },
    { 0x15920d0, 40 },
    { 0x27c920cf, 46 },
    { 0x9b5920d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78920d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad920d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946920dd, 98 },
    { 0x653920dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x281920e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7920e8, 93 },
    { 0, 0 },
    { 0x88c920ea, 77 },
    { 0x29b920ea, 90 },
    { 0x663920ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36920ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8920f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x514920f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432920ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79492104, 90 },
    { 0xa1592105, 95 },
    { 0, 0 },
    { 0x6d992107, 76 },
    { 0xa7792108, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9210b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb692113, 95 },
    { 0x3b792114, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24792119, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9299211e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d92121, 64 },
    { 0xde392122, 61 },
    { 0xe0d92123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d79212a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a9212f, 90 },
    { 0xd8c92130, 23 },
    { 0x86892131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9992134, 95 },
    { 0, 0 },
    { 0x37b92136, 91 },
    { 0x6bd92137, 33 },
    { 0xe3092136, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e9213b, 33 },
    { 0x75b9213c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c92142, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55992148, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd279214b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1392157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2989215b, 40 },
    { 0xc1b9215c, 33 },
    { 0x6129215d, 46 },
    { 0x2b59215e, 98 },
    { 0x5289215d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca92162, 76 },
    { 0, 0 },
    { 0x592164, 95 },
    { 0xa5892165, 55 },
    { 0xe4592165, 90 },
    { 0xeb892167, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569216a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f9216e, 90 },
    { 0, 0 },
    { 0x4ff92170, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c792173, 46 },
    { 0, 0 },
    { 0x1d492175, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f92178, 23 },
    { 0, 0 },
    { 0xa499217a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad09217f, 90 },
    { 0x6af92180, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7992192, 90 },
    { 0xe6692193, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb92198, 93 },
    { 0x6fe92199, 96 },
    { 0x2c89219a, 76 },
    { 0xda29219a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a79219f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712921a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1921a7, 98 },
    { 0x4cd921a8, 34 },
    { 0x5aa921a8, 67 },
    { 0, 0 },
    { 0xe5d921ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54921af, 90 },
    { 0, 0 },
    { 0xaa1921b1, 46 },
    { 0x7fa921b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c921b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865921c3, 95 },
    { 0x970921c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c921c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0921cd, 85 },
    { 0xa2d921ce, 17 },
    { 0xb38921cf, 90 },
    { 0, 0 },
    { 0x7d6921d1, 90 },
    { 0x897921d1, 61 },
    { 0xa08921d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a921d8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478921dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83921e1, 76 },
    { 0x871921e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f921e5, 55 },
    { 0, 0 },
    { 0x440921e7, 95 },
    { 0, 0 },
    { 0x8c921e9, 95 },
    { 0, 0 },
    { 0x323921eb, 43 },
    { 0xb97921ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf921ef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f921f2, 95 },
    { 0x1a7921f3, 98 },
    { 0x330921f3, 90 },
    { 0xba3921f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a921f9, 34 },
    { 0x9d3921f9, 90 },
    { 0x518921f9, 78 },
    { 0, 0 },
    { 0x4cd921fd, 52 },
    { 0x526921fe, 85 },
    { 0x8b9921fe, 66 },
    { 0xb4192200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e92203, 46 },
    { 0x21b92203, 85 },
    { 0xed092203, 65 },
    { 0x92892206, 23 },
    { 0, 0 },
    { 0x14292208, 76 },
    { 0x38392208, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde59220c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa92213, 76 },
    { 0x40992214, 87 },
    { 0x1ca92215, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4092219, 76 },
    { 0x6e792219, 23 },
    { 0x3d39221b, 91 },
    { 0x9a39221c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54492220, 23 },
    { 0xd8592221, 98 },
    { 0, 0 },
    { 0x30592223, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a92226, 75 },
    { 0xc9f92227, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f9222a, 33 },
    { 0x1ae9222b, 55 },
    { 0x6299222c, 33 },
    { 0x18a9222c, 33 },
    { 0x4a79222c, 61 },
    { 0xf159222f, 33 },
    { 0x73c9222f, 23 },
    { 0xbcf9222f, 97 },
    { 0x74c92232, 67 },
    { 0xae592231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb9223b, 33 },
    { 0x4ae9223b, 98 },
    { 0x6d59223c, 33 },
    { 0xd559223c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b792241, 55 },
    { 0xc6992242, 23 },
    { 0xfe892243, 61 },
    { 0xff892243, 34 },
    { 0x87692245, 90 },
    { 0x5b692246, 52 },
    { 0x9ad92246, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94692252, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x42192255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b92259, 90 },
    { 0, 0 },
    { 0x1689225b, 98 },
    { 0, 0 },
    { 0xd669225d, 90 },
    { 0x7399225e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42292266, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb9226d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff92271, 23 },
    { 0xfdc92272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d92276, 91 },
    { 0, 0 },
    { 0xfde92278, 99 },
    { 0, 0 },
    { 0x3819227a, 90 },
    { 0xfac9227a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7892280, 95 },
    { 0x33392281, 68 },
    { 0x13592282, 77 },
    { 0x27192282, 85 },
    { 0xc7892280, 61 },
    { 0, 0 },
    { 0xece92286, 98 },
    { 0, 0 },
    { 0xf92288, 95 },
    { 0x21b92288, 93 },
    { 0xb289228a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8392290, 65 },
    { 0x53992291, 90 },
    { 0x33d92292, 55 },
    { 0x53892293, 98 },
    { 0x7a492294, 33 },
    { 0xd4192292, 90 },
    { 0xcda92291, 76 },
    { 0x5c692294, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5419229c, 66 },
    { 0, 0 },
    { 0x54f9229e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6922a1, 90 },
    { 0, 0 },
    { 0x723922a3, 90 },
    { 0, 0 },
    { 0x91b922a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc922ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf922af, 99 },
    { 0, 0 },
    { 0x1cd922b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7922b5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95922b9, 23 },
    { 0xc2922ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x527922bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9922c1, 95 },
    { 0xd28922c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce922c7, 43 },
    { 0xd922c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72922cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885922cf, 90 },
    { 0x58e922cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5922d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e922d8, 34 },
    { 0xa0b922d8, 68 },
    { 0, 0 },
    { 0xb71922db, 95 },
    { 0x4a9922dc, 91 },
    { 0, 0 },
    { 0xbd4922de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656922e2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920922e7, 81 },
    { 0xfd7922e8, 87 },
    { 0x6ff922e9, 23 },
    { 0x218922ea, 85 },
    { 0x222922eb, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x794922ee, 23 },
    { 0x3b4922ef, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb922f2, 36 },
    { 0x469922f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47922f8, 23 },
    { 0x8b4922f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1922fc, 91 },
    { 0x3c9922fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95292309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea9230c, 85 },
    { 0x8b39230c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ee92315, 84 },
    { 0, 0 },
    { 0x2892317, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1459231a, 90 },
    { 0, 0 },
    { 0x31a9231c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xee89231f, 46 },
    { 0, 0 },
    { 0x19092321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda592324, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50792327, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f9232b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069232e, 43 },
    { 0x8989232f, 76 },
    { 0x75a9232f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a92333, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5692336, 87 },
    { 0x19b92337, 95 },
    { 0xa7192337, 76 },
    { 0, 0 },
    { 0xf009233a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c79233e, 67 },
    { 0x9379233f, 40 },
    { 0x94b92340, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24892344, 83 },
    { 0x53692345, 33 },
    { 0xb1692346, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f92349, 70 },
    { 0xd7a9234a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a09234e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f492351, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b92356, 61 },
    { 0x33892357, 61 },
    { 0x85d92358, 61 },
    { 0x97792358, 61 },
    { 0xa3892357, 43 },
    { 0, 0 },
    { 0xf899235c, 61 },
    { 0x5879235d, 95 },
    { 0x3139235e, 98 },
    { 0xa1a9235f, 23 },
    { 0x5ff9235f, 98 },
    { 0xf009235e, 98 },
    { 0xda79235f, 66 },
    { 0x2ea92363, 98 },
    { 0x74592363, 76 },
    { 0xe1992364, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf9236d, 33 },
    { 0xc929236e, 33 },
    { 0, 0 },
    { 0x5a292370, 98 },
    { 0xdb192371, 61 },
    { 0, 0 },
    { 0xd7792373, 33 },
    { 0x65092373, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7692377, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a9237a, 95 },
    { 0, 0 },
    { 0xb799237c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b92381, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34892384, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8992387, 66 },
    { 0, 0 },
    { 0x71d92389, 33 },
    { 0x6db9238a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1099238f, 92 },
    { 0xb759238f, 90 },
    { 0xcda9238f, 90 },
    { 0xe3f9238f, 61 },
    { 0x24f9238f, 61 },
    { 0x61692392, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba192399, 23 },
    { 0x5a692399, 77 },
    { 0xd369239b, 76 },
    { 0x7759239a, 66 },
    { 0x4609239d, 90 },
    { 0, 0 },
    { 0xd2c9239f, 53 },
    { 0, 0 },
    { 0x2b6923a1, 99 },
    { 0x9a6923a2, 90 },
    { 0x80c923a3, 23 },
    { 0x55c923a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af923aa, 95 },
    { 0x4dd923ab, 95 },
    { 0xd51923ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74923af, 92 },
    { 0x2fb923af, 61 },
    { 0xe6f923af, 75 },
    { 0xd6c923b1, 95 },
    { 0x98b923b3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e923b6, 95 },
    { 0xbb2923b7, 98 },
    { 0x913923b8, 87 },
    { 0x5f4923b6, 90 },
    { 0xcb3923ba, 43 },
    { 0xfb2923bb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c923bf, 55 },
    { 0xab923c0, 99 },
    { 0x4d923c1, 55 },
    { 0, 0 },
    { 0x174923c3, 23 },
    { 0x5ef923c3, 75 },
    { 0xe5c923c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf923cc, 95 },
    { 0x7b0923cc, 96 },
    { 0, 0 },
    { 0x54923cf, 96 },
    { 0, 0 },
    { 0x105923d1, 33 },
    { 0x301923d1, 66 },
    { 0x781923d3, 90 },
    { 0x9a7923d4, 55 },
    { 0, 0 },
    { 0x306923d6, 95 },
    { 0x726923d7, 34 },
    { 0x94e923d7, 93 },
    { 0x6e923d9, 33 },
    { 0x659923da, 43 },
    { 0, 0 },
    { 0xd26923dc, 90 },
    { 0x2d0923dd, 55 },
    { 0, 0 },
    { 0x279923df, 81 },
    { 0xa96923e0, 33 },
    { 0x90b923df, 93 },
    { 0, 0 },
    { 0xef8923e3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9923ec, 33 },
    { 0, 0 },
    { 0x8e4923ee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c923fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b92403, 66 },
    { 0xfb592404, 33 },
    { 0x9b492403, 23 },
    { 0, 0 },
    { 0x97b92407, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad9240a, 98 },
    { 0x6b29240b, 95 },
    { 0, 0 },
    { 0xa049240d, 98 },
    { 0xb699240e, 33 },
    { 0xd5e9240d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34492413, 95 },
    { 0x95e92413, 95 },
    { 0xdcb92415, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7109241b, 90 },
    { 0x659241b, 67 },
    { 0x47b9241d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76392423, 84 },
    { 0, 0 },
    { 0xf9192425, 33 },
    { 0xd0892426, 76 },
    { 0xdc992427, 43 },
    { 0x1be92427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a9242d, 33 },
    { 0x8409242e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c92431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae992443, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70192447, 95 },
    { 0xa6392448, 90 },
    { 0xa0c92449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf679244c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a392450, 61 },
    { 0xf492450, 90 },
    { 0xaba92452, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d292455, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff92458, 23 },
    { 0x16492458, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f69245d, 78 },
    { 0x1c99245e, 98 },
    { 0x419245e, 95 },
    { 0x5219245f, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2992466, 65 },
    { 0, 0 },
    { 0x1fd92468, 33 },
    { 0, 0 },
    { 0x8709246a, 61 },
    { 0xefe9246a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b9246e, 33 },
    { 0, 0 },
    { 0xd2e92470, 23 },
    { 0xbfa92471, 33 },
    { 0xc2392472, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62592475, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe592479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc92485, 61 },
    { 0, 0 },
    { 0x75d92487, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a9248a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9248d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a92491, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4592496, 66 },
    { 0x9bd92496, 33 },
    { 0x64192498, 61 },
    { 0xd6292497, 99 },
    { 0x9ae9249a, 33 },
    { 0x9019249b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5779249f, 33 },
    { 0, 0 },
    { 0xd15924a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae924ac, 65 },
    { 0, 0 },
    { 0x3e8924ae, 23 },
    { 0xa8a924af, 90 },
    { 0, 0 },
    { 0x3a9924b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911924ba, 44 },
    { 0xc2b924bb, 55 },
    { 0xcbe924bc, 99 },
    { 0x199924bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd924c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53924cb, 33 },
    { 0x614924cc, 43 },
    { 0x670924cd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed924d2, 98 },
    { 0, 0 },
    { 0xf19924d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315924d9, 33 },
    { 0x117924da, 46 },
    { 0x1bb924da, 95 },
    { 0x377924dc, 78 },
    { 0xda0924d9, 95 },
    { 0x9d9924dd, 58 },
    { 0x460924df, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6924e4, 33 },
    { 0, 0 },
    { 0x495924e6, 23 },
    { 0x38d924e6, 33 },
    { 0xc4a924e8, 61 },
    { 0x7a7924e9, 23 },
    { 0x9d8924e9, 100 },
    { 0xfbc924eb, 61 },
    { 0x4ed924ec, 61 },
    { 0xfaa924ec, 84 },
    { 0, 0 },
    { 0x725924ef, 90 },
    { 0xff5924ef, 95 },
    { 0, 0 },
    { 0x7a3924f2, 76 },
    { 0x189924f3, 61 },
    { 0x186924f4, 93 },
    { 0xbbb924f5, 23 },
    { 0xcfb924f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177924fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5924ff, 91 },
    { 0, 0 },
    { 0x78d92501, 61 },
    { 0xdc792501, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859250a, 34 },
    { 0, 0 },
    { 0xc559250c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe349250f, 55 },
    { 0x4af92510, 99 },
    { 0x6bc92511, 76 },
    { 0xf3292511, 23 },
    { 0x81092513, 90 },
    { 0x2ca92514, 98 },
    { 0x8899250f, 95 },
    { 0xad692510, 81 },
    { 0xa4b92517, 95 },
    { 0xf6b92518, 98 },
    { 0xbda92519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d9251d, 33 },
    { 0xa4e9251e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad92521, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a492525, 23 },
    { 0x61092526, 55 },
    { 0x35b92527, 95 },
    { 0x4fa92528, 90 },
    { 0, 0 },
    { 0xf7f9252a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d9252d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7492531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed9253b, 33 },
    { 0xe879253c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47192542, 95 },
    { 0, 0 },
    { 0x22592544, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8892547, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d39254d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd92550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeac92553, 90 },
    { 0xe5a92553, 95 },
    { 0xb0a92555, 77 },
    { 0x6be92556, 95 },
    { 0, 0 },
    { 0xefd92558, 61 },
    { 0xea992559, 95 },
    { 0, 0 },
    { 0x2ba9255b, 98 },
    { 0x6859255c, 61 },
    { 0xda39255d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d92561, 95 },
    { 0, 0 },
    { 0x9ca92563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee592566, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8492569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4599256e, 33 },
    { 0x10b9256e, 98 },
    { 0xd7592570, 33 },
    { 0xa2292571, 90 },
    { 0x46c92570, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c79257b, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9592582, 90 },
    { 0xac92583, 55 },
    { 0, 0 },
    { 0x44292585, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d9258a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a9258d, 23 },
    { 0x4ac9258d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70192595, 98 },
    { 0x28e92595, 65 },
    { 0, 0 },
    { 0xfb992598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf099259b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd719259e, 99 },
    { 0xae9259e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee925a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e925aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895925b0, 90 },
    { 0x95e925b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9925b7, 44 },
    { 0x8f7925b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135925bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb925c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95925c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d925c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121925cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x972925d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61925d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d925da, 95 },
    { 0xd16925da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87b925e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff925e8, 98 },
    { 0x27b925e9, 96 },
    { 0xd51925ea, 46 },
    { 0, 0 },
    { 0x544925ec, 34 },
    { 0, 0 },
    { 0x279925ee, 23 },
    { 0x360925ef, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0x318925f2, 44 },
    { 0, 0 },
    { 0x95c925f4, 76 },
    { 0xeba925f4, 93 },
    { 0xcb9925f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6925f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4925fe, 46 },
    { 0, 0 },
    { 0x42892600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d992608, 33 },
    { 0x75492608, 61 },
    { 0x669260a, 67 },
    { 0xe309260a, 90 },
    { 0x88b9260c, 76 },
    { 0x61c92609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd792611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef92614, 76 },
    { 0xbac92615, 78 },
    { 0xa0d92616, 67 },
    { 0x96992616, 90 },
    { 0, 0 },
    { 0x8e992619, 76 },
    { 0x8499261a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09261e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae92625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d92629, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a892633, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef92636, 72 },
    { 0, 0 },
    { 0x1b392638, 95 },
    { 0, 0 },
    { 0xc09263a, 52 },
    { 0x9ec9263b, 93 },
    { 0, 0 },
    { 0x2119263d, 98 },
    { 0x6c89263e, 85 },
    { 0xe189263f, 67 },
    { 0x7239263f, 81 },
    { 0xb9a9263e, 34 },
    { 0xcdc92641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d192645, 90 },
    { 0x6b92645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe339264c, 90 },
    { 0x6f29264d, 95 },
    { 0xf669264c, 93 },
    { 0, 0 },
    { 0x9d092650, 61 },
    { 0xd692650, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69592659, 90 },
    { 0x59d9265a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c99265f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45892665, 76 },
    { 0xcae92666, 90 },
    { 0, 0 },
    { 0x7f192668, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2692670, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd819267b, 34 },
    { 0x6f99267c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12592687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa39268d, 76 },
    { 0xffc9268e, 43 },
    { 0x6129268e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6592692, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f92695, 90 },
    { 0, 0 },
    { 0xde092697, 99 },
    { 0, 0 },
    { 0xbba92699, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3999269d, 98 },
    { 0xf859269e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec926a1, 76 },
    { 0, 0 },
    { 0x44a926a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d926a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b926a9, 93 },
    { 0xfb1926aa, 67 },
    { 0xcf9926aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d926b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79926b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82926b8, 66 },
    { 0xc42926b8, 77 },
    { 0xc5b926ba, 99 },
    { 0x39f926ba, 55 },
    { 0x980926bc, 55 },
    { 0, 0 },
    { 0xbb0926be, 17 },
    { 0, 0 },
    { 0x7cf926c0, 33 },
    { 0x292926c0, 84 },
    { 0x637926c1, 61 },
    { 0, 0 },
    { 0x9ce926c4, 61 },
    { 0, 0 },
    { 0xe85926c6, 93 },
    { 0, 0 },
    { 0x62f926c8, 90 },
    { 0x4b926c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3926cc, 43 },
    { 0x813926cc, 64 },
    { 0, 0 },
    { 0x88f926cf, 33 },
    { 0, 0 },
    { 0x41926d1, 98 },
    { 0x5c4926d2, 33 },
    { 0x28f926d2, 85 },
    { 0, 0 },
    { 0xe92926d5, 98 },
    { 0, 0 },
    { 0xa46926d7, 61 },
    { 0xbe4926d7, 61 },
    { 0x985926d9, 76 },
    { 0, 0 },
    { 0x79d926db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d926de, 88 },
    { 0xd97926de, 90 },
    { 0xd08926de, 95 },
    { 0xe55926e1, 90 },
    { 0x20f926e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642926e6, 90 },
    { 0x131926e7, 43 },
    { 0xb7a926e8, 55 },
    { 0, 0 },
    { 0x3c8926ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442926f0, 46 },
    { 0xe0d926f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565926f7, 75 },
    { 0xc52926f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6926fd, 40 },
    { 0xbdf926fd, 33 },
    { 0x820926ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1592708, 77 },
    { 0xf7d92709, 90 },
    { 0x4669270a, 98 },
    { 0xdcf92709, 65 },
    { 0x12d9270c, 90 },
    { 0xc029270d, 43 },
    { 0xdf29270e, 98 },
    { 0x8bb9270f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0592715, 23 },
    { 0, 0 },
    { 0xd1392717, 17 },
    { 0xa0d92717, 34 },
    { 0x55e92719, 46 },
    { 0x20692717, 23 },
    { 0, 0 },
    { 0xde29271c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf39271f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b92727, 75 },
    { 0xfa092727, 90 },
    { 0x67292729, 46 },
    { 0x8192728, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79392734, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4092738, 76 },
    { 0, 0 },
    { 0xee89273a, 33 },
    { 0x3859273b, 43 },
    { 0x8089273c, 55 },
    { 0x2419273d, 23 },
    { 0xa609273b, 98 },
    { 0, 0 },
    { 0x51792740, 33 },
    { 0xc4a92740, 82 },
    { 0xbd292741, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3292746, 34 },
    { 0, 0 },
    { 0x7a092748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f9274e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5292752, 65 },
    { 0x7a292753, 61 },
    { 0x16692754, 99 },
    { 0xd3092755, 46 },
    { 0, 0 },
    { 0x25f92757, 88 },
    { 0x55292758, 43 },
    { 0x5cf92757, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a92760, 87 },
    { 0, 0 },
    { 0x63192762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc539276a, 93 },
    { 0xc179276b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b792774, 61 },
    { 0x7aa92774, 95 },
    { 0x7ec92776, 88 },
    { 0, 0 },
    { 0xfad92778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b09277d, 98 },
    { 0x1689277e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x39892781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48592786, 87 },
    { 0x33c92786, 90 },
    { 0x59e92787, 90 },
    { 0x4a92789, 43 },
    { 0x27f92788, 33 },
    { 0xbef9278b, 33 },
    { 0xf8d92786, 90 },
    { 0xaf49278d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12392797, 66 },
    { 0x39492798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb519279b, 66 },
    { 0xb349279c, 90 },
    { 0x7b79279d, 95 },
    { 0x5559279e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3927a1, 56 },
    { 0x5d5927a2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04927a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475927aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e927ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a927b3, 98 },
    { 0x901927b4, 67 },
    { 0, 0 },
    { 0xc17927b6, 66 },
    { 0xcbd927b7, 90 },
    { 0x326927b7, 46 },
    { 0xbd927b9, 33 },
    { 0, 0 },
    { 0x84f927bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x348927be, 34 },
    { 0, 0 },
    { 0xc77927c0, 99 },
    { 0xb9927c1, 98 },
    { 0xed8927c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a927c5, 61 },
    { 0, 0 },
    { 0x9e5927c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x151927ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816927d1, 3 },
    { 0, 0 },
    { 0x23a927d3, 23 },
    { 0x201927d4, 90 },
    { 0xd29927d4, 55 },
    { 0xcf927d5, 95 },
    { 0x979927d6, 61 },
    { 0x30a927d8, 66 },
    { 0xd1e927d9, 69 },
    { 0xe21927da, 33 },
    { 0xfb7927d5, 70 },
    { 0x463927da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3927df, 90 },
    { 0x494927e0, 90 },
    { 0xe2f927e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x695927e4, 81 },
    { 0xd50927e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9927e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x719927eb, 95 },
    { 0x125927eb, 55 },
    { 0xebf927ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x799927f0, 65 },
    { 0, 0 },
    { 0x9f6927f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x493927f5, 98 },
    { 0x66b927f6, 34 },
    { 0, 0 },
    { 0x29e927f8, 76 },
    { 0xf927f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60592803, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6392808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2899280c, 90 },
    { 0, 0 },
    { 0xa419280e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8092811, 76 },
    { 0x88c92811, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b792816, 46 },
    { 0, 0 },
    { 0x9b692818, 34 },
    { 0x1dc92818, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c9281c, 95 },
    { 0x2809281d, 69 },
    { 0xfc69281d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b92822, 93 },
    { 0x1a892822, 98 },
    { 0x7d92824, 77 },
    { 0x6d892825, 61 },
    { 0x792826, 43 },
    { 0x98692823, 33 },
    { 0x3b392828, 43 },
    { 0, 0 },
    { 0xaec9282a, 93 },
    { 0, 0 },
    { 0x5309282c, 68 },
    { 0x1fb9282c, 96 },
    { 0x8959282c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15292833, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb9283c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44092840, 61 },
    { 0xfa492840, 23 },
    { 0xcf792841, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd992846, 98 },
    { 0, 0 },
    { 0x94992848, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5869284c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84992854, 61 },
    { 0xf8592855, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x50092858, 33 },
    { 0x76892858, 43 },
    { 0x5739285a, 85 },
    { 0x1c49285a, 98 },
    { 0xa109285c, 33 },
    { 0x7ed9285c, 90 },
    { 0xbcc9285e, 99 },
    { 0x7b69285f, 61 },
    { 0x3d992860, 33 },
    { 0xab292861, 85 },
    { 0, 0 },
    { 0xe6592863, 34 },
    { 0xb292864, 98 },
    { 0xbc592864, 33 },
    { 0xc2192866, 33 },
    { 0x44a92866, 95 },
    { 0xb4492868, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc59286c, 98 },
    { 0xa889286c, 55 },
    { 0x4cb9286d, 34 },
    { 0x64e9286c, 90 },
    { 0x9a292870, 23 },
    { 0x62492871, 43 },
    { 0x39d92871, 90 },
    { 0, 0 },
    { 0x8e392874, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3292877, 80 },
    { 0x27d92878, 98 },
    { 0xd9892879, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2092881, 23 },
    { 0, 0 },
    { 0x1d592883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1192886, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc9288d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7792891, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe192899, 93 },
    { 0xd589289a, 98 },
    { 0x8be9289a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce928a0, 100 },
    { 0, 0 },
    { 0x3d9928a2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282928a9, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72928ad, 67 },
    { 0x59a928ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774928b2, 91 },
    { 0xf2e928b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x683928b6, 85 },
    { 0, 0 },
    { 0xa2d928b8, 69 },
    { 0xeaa928b9, 98 },
    { 0, 0 },
    { 0x447928bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6928bf, 23 },
    { 0xdea928c0, 95 },
    { 0x2c7928c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12928c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c928c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876928cf, 95 },
    { 0x1cc928d0, 61 },
    { 0, 0 },
    { 0x553928d2, 90 },
    { 0, 0 },
    { 0xaf0928d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e928d7, 33 },
    { 0xa32928d8, 23 },
    { 0xdad928d7, 61 },
    { 0x78d928da, 55 },
    { 0x2d928db, 95 },
    { 0, 0 },
    { 0x3d0928dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae928e2, 76 },
    { 0x841928e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a928e7, 84 },
    { 0xbc0928e8, 23 },
    { 0x477928e9, 90 },
    { 0x6a4928ea, 46 },
    { 0xe4928e9, 46 },
    { 0x2af928ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de928ef, 55 },
    { 0xd6d928f0, 61 },
    { 0xfd4928f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6928f6, 85 },
    { 0, 0 },
    { 0x593928f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b928fe, 33 },
    { 0x9d3928ff, 67 },
    { 0x2d592900, 61 },
    { 0x752928fe, 95 },
    { 0x46d92900, 18 },
    { 0x6f392903, 55 },
    { 0x8b492900, 98 },
    { 0, 0 },
    { 0xb3992906, 61 },
    { 0x54b92906, 95 },
    { 0, 0 },
    { 0x71192909, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3839290c, 98 },
    { 0x8b9290d, 93 },
    { 0x2139290d, 90 },
    { 0, 0 },
    { 0xe1992910, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac92915, 33 },
    { 0xb6992916, 91 },
    { 0x26792916, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde9291b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4492921, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe592924, 23 },
    { 0x2f992925, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c392928, 98 },
    { 0, 0 },
    { 0x7d99292a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2239293d, 85 },
    { 0xd0e9293e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3992943, 67 },
    { 0x43a92943, 90 },
    { 0, 0 },
    { 0xf0792946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d69294d, 23 },
    { 0x9299294d, 81 },
    { 0, 0 },
    { 0x9d992950, 62 },
    { 0x74c92951, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6069295a, 38 },
    { 0x5059295b, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc19295e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f92962, 61 },
    { 0xc6592962, 95 },
    { 0xbb92964, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x39592967, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa9296a, 98 },
    { 0, 0 },
    { 0x119296c, 95 },
    { 0xe989296c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c092973, 61 },
    { 0x8ef92973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b92977, 76 },
    { 0, 0 },
    { 0x76892979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b92988, 61 },
    { 0xd9992989, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8639298d, 95 },
    { 0x4ed9298d, 90 },
    { 0x6ef9298f, 34 },
    { 0x1219298e, 33 },
    { 0x3b69298d, 98 },
    { 0xa492992, 99 },
    { 0, 0 },
    { 0xa3992994, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae492998, 99 },
    { 0xcbc92999, 34 },
    { 0x74f9299a, 61 },
    { 0, 0 },
    { 0x3589299c, 91 },
    { 0, 0 },
    { 0x1fe9299e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe929a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83929a7, 95 },
    { 0, 0 },
    { 0x6d7929a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d929ac, 95 },
    { 0xd16929ac, 23 },
    { 0x979929ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10929b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d929bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469929c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0929c9, 76 },
    { 0x5c929ca, 101 },
    { 0x2a929ca, 90 },
    { 0, 0 },
    { 0xd29929cd, 46 },
    { 0x93929ce, 98 },
    { 0, 0 },
    { 0xdbf929d0, 90 },
    { 0xc89929d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5929d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958929de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972929e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e929ea, 61 },
    { 0, 0 },
    { 0x9d0929ec, 95 },
    { 0x6b6929ed, 87 },
    { 0x35929ed, 98 },
    { 0, 0 },
    { 0x9d929f0, 90 },
    { 0x55f929f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138929f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2929fb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc992a05, 53 },
    { 0x1d692a06, 55 },
    { 0xbd992a07, 33 },
    { 0x5c292a08, 98 },
    { 0, 0 },
    { 0x3dc92a0a, 100 },
    { 0, 0 },
    { 0xf0192a0c, 85 },
    { 0, 0 },
    { 0x71792a0e, 76 },
    { 0, 0 },
    { 0x3bb92a10, 98 },
    { 0x66492a11, 100 },
    { 0x24992a12, 90 },
    { 0x27392a13, 33 },
    { 0x1f92a13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95792a1b, 33 },
    { 0x7a092a1c, 38 },
    { 0x25492a1d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc892a29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae792a2c, 100 },
    { 0x1b492a2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e92a36, 78 },
    { 0, 0 },
    { 0x96b92a38, 61 },
    { 0xa1a92a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb592a3d, 23 },
    { 0x1d192a3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea92a42, 95 },
    { 0x2b092a43, 77 },
    { 0x24992a42, 23 },
    { 0xea92a45, 66 },
    { 0x70692a43, 99 },
    { 0x15592a42, 99 },
    { 0, 0 },
    { 0x6d592a49, 76 },
    { 0x54192a49, 69 },
    { 0xba492a4b, 95 },
    { 0, 0 },
    { 0x97392a4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c592a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22992a58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c92a5d, 33 },
    { 0xc6492a5e, 95 },
    { 0x99b92a5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c92a63, 78 },
    { 0, 0 },
    { 0x59a92a65, 46 },
    { 0x16192a66, 61 },
    { 0, 0 },
    { 0xcf192a68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a92a6b, 43 },
    { 0x26592a6c, 76 },
    { 0xfa592a6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8092a71, 90 },
    { 0x17b92a72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7892a79, 84 },
    { 0, 0 },
    { 0x5fd92a7b, 23 },
    { 0x27992a7b, 43 },
    { 0xb1c92a7d, 33 },
    { 0xe7392a7e, 90 },
    { 0x7e792a7f, 46 },
    { 0, 0 },
    { 0x70692a81, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b492a85, 78 },
    { 0xd5992a85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc92a8d, 12 },
    { 0xad292a8e, 90 },
    { 0xf2992a8e, 100 },
    { 0xbc92a90, 76 },
    { 0, 0 },
    { 0x5ad92a92, 23 },
    { 0, 0 },
    { 0xa2b92a94, 33 },
    { 0x42d92a95, 55 },
    { 0xa4f92a96, 33 },
    { 0x3f492a96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72592a9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9792a9e, 91 },
    { 0, 0 },
    { 0x51592aa0, 17 },
    { 0xcf192aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d92aa4, 95 },
    { 0x24392aa5, 91 },
    { 0xcc492aa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa92aaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x58892aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24192ab2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42992ab9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef92ac0, 90 },
    { 0, 0 },
    { 0xbb692ac2, 90 },
    { 0x87f92ac3, 93 },
    { 0x73d92ac3, 100 },
    { 0xe8592ac5, 61 },
    { 0x87a92ac2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d92ac9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2392ad2, 68 },
    { 0, 0 },
    { 0x83f92ad4, 17 },
    { 0, 0 },
    { 0x47192ad6, 90 },
    { 0x70f92ad7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73792ae1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d692ae4, 55 },
    { 0x5b192ae5, 68 },
    { 0, 0 },
    { 0x72292ae7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x63692aea, 99 },
    { 0x3e092aeb, 60 },
    { 0x6192aeb, 23 },
    { 0x28d92aeb, 95 },
    { 0, 0 },
    { 0xca692aef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e092af2, 52 },
    { 0x92492af2, 61 },
    { 0x7db92af2, 93 },
    { 0x45792af5, 23 },
    { 0, 0 },
    { 0x82392af7, 95 },
    { 0x2c492af8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3192afb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe292aff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2192b02, 65 },
    { 0, 0 },
    { 0x66b92b04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b92b07, 88 },
    { 0x5b792b08, 52 },
    { 0x58e92b07, 81 },
    { 0xfee92b0a, 95 },
    { 0xfca92b0b, 55 },
    { 0, 0 },
    { 0x15392b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e92b10, 95 },
    { 0x7992b11, 100 },
    { 0x5e92b10, 55 },
    { 0, 0 },
    { 0xda092b14, 98 },
    { 0x2e792b15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e92b1c, 23 },
    { 0x60492b1d, 98 },
    { 0x85f92b1d, 95 },
    { 0xca392b1f, 95 },
    { 0x3e992b20, 78 },
    { 0xac492b21, 23 },
    { 0xfd192b1e, 69 },
    { 0x61892b23, 61 },
    { 0x84492b1c, 66 },
    { 0, 0 },
    { 0xea92b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f92b29, 76 },
    { 0, 0 },
    { 0x84392b2b, 67 },
    { 0x35092b2b, 33 },
    { 0x7c392b2d, 66 },
    { 0x9f292b2d, 77 },
    { 0xaf892b2e, 61 },
    { 0x7b392b2d, 90 },
    { 0x31092b31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x52792b34, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe92b38, 34 },
    { 0x1e992b39, 33 },
    { 0x1092b39, 33 },
    { 0x81292b39, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d92b3e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb92b46, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x54792b49, 90 },
    { 0x9bc92b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e192b4d, 23 },
    { 0xee492b4d, 66 },
    { 0x68c92b4f, 23 },
    { 0xca492b50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a92b53, 66 },
    { 0, 0 },
    { 0x84592b55, 33 },
    { 0x69b92b55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0c92b5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e92b64, 90 },
    { 0x93892b65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c92b69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa492b6f, 81 },
    { 0x55292b70, 85 },
    { 0x5d192b71, 90 },
    { 0, 0 },
    { 0x60b92b73, 90 },
    { 0x31592b73, 69 },
    { 0, 0 },
    { 0x83792b76, 76 },
    { 0xe7b92b76, 43 },
    { 0, 0 },
    { 0xad692b79, 100 },
    { 0x8f592b7a, 93 },
    { 0x9eb92b7b, 95 },
    { 0, 0 },
    { 0x4ca92b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6692b85, 84 },
    { 0xc1292b86, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf892b8a, 78 },
    { 0x5a692b8b, 93 },
    { 0, 0 },
    { 0x64592b8d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7892b94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61d92b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13892b9d, 23 },
    { 0x97f92b9d, 98 },
    { 0x49f92b9f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b92ba5, 66 },
    { 0x77092ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a792baa, 90 },
    { 0x18892bab, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61592bb0, 61 },
    { 0x92492bb1, 95 },
    { 0x15e92bb2, 34 },
    { 0, 0 },
    { 0xbed92bb4, 61 },
    { 0x26892bb4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9292bb9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1892bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8292bbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7292bc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9292bcc, 87 },
    { 0, 0 },
    { 0x7a592bce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b92bdb, 33 },
    { 0xe6c92bdb, 61 },
    { 0x1f92bdd, 98 },
    { 0, 0 },
    { 0x5aa92bdf, 61 },
    { 0xd1692bdf, 98 },
    { 0, 0 },
    { 0xe2f92be2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb92be5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x90492be8, 33 },
    { 0x67c92be9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa092bed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f92bf0, 95 },
    { 0x84c92bf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2692bf4, 61 },
    { 0xb392bf5, 61 },
    { 0xcae92bf5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e92bfd, 61 },
    { 0xf0e92bfe, 52 },
    { 0x47392bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec92c04, 87 },
    { 0x4c292c04, 95 },
    { 0x5ae92c06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c92c09, 58 },
    { 0x1a292c0a, 95 },
    { 0xa1e92c09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e92c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e092c13, 95 },
    { 0, 0 },
    { 0xb1392c15, 84 },
    { 0x4cb92c16, 90 },
    { 0x93c92c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36792c1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61792c26, 90 },
    { 0xef592c27, 90 },
    { 0x80192c28, 99 },
    { 0, 0 },
    { 0x34492c2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2192c2e, 40 },
    { 0xf7592c2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd92c37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17092c3c, 90 },
    { 0xe6292c3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5292c45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c92c4a, 83 },
    { 0xd6f92c4b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33392c50, 98 },
    { 0x3c292c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b92c58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7192c6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd392c71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d592c75, 61 },
    { 0, 0 },
    { 0xb92c77, 88 },
    { 0xe9692c78, 66 },
    { 0xb3792c79, 83 },
    { 0x21392c79, 76 },
    { 0, 0 },
    { 0x65192c7c, 90 },
    { 0xe4492c7d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b92c80, 78 },
    { 0xdaf92c81, 67 },
    { 0x38792c82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec092c89, 66 },
    { 0, 0 },
    { 0xfb992c8b, 22 },
    { 0x7e092c8c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e92c90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b92c93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10892c97, 93 },
    { 0x7bf92c98, 90 },
    { 0, 0 },
    { 0xcb392c9a, 76 },
    { 0, 0 },
    { 0x17a92c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f92ca3, 23 },
    { 0, 0 },
    { 0x46392ca5, 53 },
    { 0x69f92ca6, 95 },
    { 0x3f492ca7, 81 },
    { 0xd8992ca8, 98 },
    { 0xadb92ca9, 95 },
    { 0xa0292ca7, 98 },
    { 0x11b92ca7, 23 },
    { 0xd4592ca6, 96 },
    { 0x77f92ca6, 85 },
    { 0x5fb92cae, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f92cb2, 17 },
    { 0x69a92cb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5792cb7, 77 },
    { 0xc6c92cb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e92cbb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88692ccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78492cd0, 34 },
    { 0x64b92cd1, 87 },
    { 0xd2992cd0, 75 },
    { 0xcae92cd3, 33 },
    { 0xe9192cd4, 33 },
    { 0x12092cd2, 46 },
    { 0x35292cd6, 61 },
    { 0xa4992cd7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3192ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b992ce8, 90 },
    { 0, 0 },
    { 0x61f92cea, 76 },
    { 0, 0 },
    { 0x19492cec, 84 },
    { 0xc0792cec, 33 },
    { 0, 0 },
    { 0xdad92cef, 95 },
    { 0x23592cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36892cf4, 90 },
    { 0, 0 },
    { 0x72b92cf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f292cfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0392d09, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc492d0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57992d12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34092d1b, 95 },
    { 0x11192d1c, 90 },
    { 0xd2992d1d, 90 },
    { 0x92c92d1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b92d22, 90 },
    { 0x9d692d23, 69 },
    { 0, 0 },
    { 0xede92d25, 23 },
    { 0xacf92d26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde92d29, 90 },
    { 0x5a292d2a, 76 },
    { 0, 0 },
    { 0x84292d2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3792d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4692d32, 33 },
    { 0, 0 },
    { 0x69a92d34, 61 },
    { 0x29d92d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63292d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b92d3c, 98 },
    { 0x5fa92d3d, 90 },
    { 0, 0 },
    { 0x78492d3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2692d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42892d47, 23 },
    { 0x5ab92d47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f892d4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce892d4f, 88 },
    { 0x41292d50, 85 },
    { 0x40192d51, 98 },
    { 0x9f292d52, 95 },
    { 0x3aa92d53, 98 },
    { 0x60e92d53, 90 },
    { 0x10e92d51, 33 },
    { 0xc2c92d56, 76 },
    { 0xad092d57, 53 },
    { 0x7a692d57, 66 },
    { 0x1b092d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb92d5c, 90 },
    { 0x18c92d5d, 90 },
    { 0, 0 },
    { 0x95992d5f, 55 },
    { 0, 0 },
    { 0x65f92d61, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da92d64, 61 },
    { 0x85192d65, 90 },
    { 0x1092d65, 34 },
    { 0x2ff92d66, 98 },
    { 0xb1a92d68, 87 },
    { 0, 0 },
    { 0x8d92d6a, 76 },
    { 0x73392d6a, 33 },
    { 0x78a92d6a, 33 },
    { 0x1692d6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd92d70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d92d76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff892d79, 98 },
    { 0xbb92d7a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb392d81, 61 },
    { 0x8b892d82, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x26892d85, 93 },
    { 0xbd492d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda992d8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e092d93, 52 },
    { 0xe592d94, 95 },
    { 0xcd092d95, 23 },
    { 0xd2c92d94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f392d9b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4492d9e, 33 },
    { 0xd1992d9f, 49 },
    { 0xc9492d9e, 90 },
    { 0x55a92da0, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78192da8, 90 },
    { 0x23492da8, 33 },
    { 0xe8292daa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef092dae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6492db1, 83 },
    { 0x62692db1, 93 },
    { 0x4d392db3, 87 },
    { 0xd1892db4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c92db7, 55 },
    { 0x34992db8, 90 },
    { 0xf8f92db9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac92dbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2592dc6, 33 },
    { 0x17792dc7, 46 },
    { 0x59692dc8, 55 },
    { 0x4ad92dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17692dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb192dd1, 55 },
    { 0xd0b92dd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e92dd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74492ddc, 95 },
    { 0xea192ddd, 98 },
    { 0xf8d92dde, 55 },
    { 0xb2692ddf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb92de3, 98 },
    { 0xdbb92de4, 61 },
    { 0x7ca92de4, 78 },
    { 0, 0 },
    { 0x73792de7, 33 },
    { 0xb292de8, 93 },
    { 0xbc692de8, 87 },
    { 0x47592de9, 87 },
    { 0, 0 },
    { 0xf6392dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0692def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a92df3, 61 },
    { 0xb9f92df3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x95192df7, 61 },
    { 0, 0 },
    { 0x67292df9, 95 },
    { 0x63d92dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f92dfe, 90 },
    { 0, 0 },
    { 0x56992e00, 90 },
    { 0, 0 },
    { 0xdee92e02, 90 },
    { 0, 0 },
    { 0xe5192e04, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc992e09, 34 },
    { 0x2b892e0a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec92e0e, 76 },
    { 0xf7492e0f, 46 },
    { 0xf7692e10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77992e13, 61 },
    { 0xa9b92e14, 90 },
    { 0x29492e14, 90 },
    { 0x17492e15, 91 },
    { 0x35e92e17, 33 },
    { 0xe9492e16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f92e1b, 66 },
    { 0, 0 },
    { 0xddd92e1d, 95 },
    { 0xd992e1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab292e22, 61 },
    { 0x9bb92e22, 85 },
    { 0x3d992e24, 85 },
    { 0xeb692e23, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xed892e28, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85292e2c, 52 },
    { 0, 0 },
    { 0x4a992e2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a092e37, 61 },
    { 0x8b792e37, 62 },
    { 0x86d92e39, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b92e3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84792e42, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13292e45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c92e48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0792e50, 90 },
    { 0xa6b92e50, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e92e54, 98 },
    { 0xb5692e54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be92e5f, 90 },
    { 0x2ea92e60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf692e63, 69 },
    { 0x5d692e63, 76 },
    { 0, 0 },
    { 0xbec92e66, 57 },
    { 0, 0 },
    { 0x41292e68, 76 },
    { 0xe1592e69, 87 },
    { 0, 0 },
    { 0x4ad92e6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8992e74, 100 },
    { 0, 0 },
    { 0x62492e76, 95 },
    { 0, 0 },
    { 0xd5892e78, 98 },
    { 0x8a192e79, 98 },
    { 0x8e792e79, 76 },
    { 0x20d92e79, 95 },
    { 0x68c92e7c, 23 },
    { 0x5292e7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e92e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5992e89, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66892e8e, 33 },
    { 0, 0 },
    { 0x62692e90, 23 },
    { 0x77892e91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d692e96, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd92e99, 61 },
    { 0x90792e99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67992e9e, 98 },
    { 0x61892e9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e92ea3, 61 },
    { 0xf1c92ea4, 55 },
    { 0x75092ea5, 61 },
    { 0x2e792ea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6592eaa, 77 },
    { 0xcf892eaa, 75 },
    { 0xf392eaa, 69 },
    { 0x21292ead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc92eb3, 33 },
    { 0x3b692eb4, 75 },
    { 0, 0 },
    { 0x30692eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd992ebe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51892ec7, 77 },
    { 0x49592ec8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e92ecf, 90 },
    { 0xbb92ecf, 91 },
    { 0x31192ed1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3892ed6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41692eda, 55 },
    { 0x19a92edb, 98 },
    { 0x56392edc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b92ee1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c792ee5, 98 },
    { 0x7cf92ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f92eea, 98 },
    { 0x4c092eeb, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73692ef0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f92ef4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef92ef7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56792efa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d92eff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed92f04, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1892f08, 95 },
    { 0, 0 },
    { 0x82092f0a, 61 },
    { 0, 0 },
    { 0x7b192f0c, 46 },
    { 0x38a92f0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee692f11, 90 },
    { 0xcd492f11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a092f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53392f20, 67 },
    { 0, 0 },
    { 0xf6c92f22, 76 },
    { 0, 0 },
    { 0x79492f24, 40 },
    { 0x7f792f25, 90 },
    { 0, 0 },
    { 0x9dd92f27, 33 },
    { 0x4cc92f27, 33 },
    { 0, 0 },
    { 0x9d992f2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d92f2d, 23 },
    { 0, 0 },
    { 0x93792f2f, 90 },
    { 0, 0 },
    { 0x4f892f31, 23 },
    { 0xc0092f32, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea492f38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae92f46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b992f49, 23 },
    { 0, 0 },
    { 0x192f4b, 23 },
    { 0x6292f4c, 33 },
    { 0, 0 },
    { 0xb5c92f4e, 66 },
    { 0, 0 },
    { 0xd7e92f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21792f55, 33 },
    { 0x30192f56, 33 },
    { 0, 0 },
    { 0x8b692f58, 98 },
    { 0xfde92f58, 101 },
    { 0x69692f58, 76 },
    { 0, 0 },
    { 0xbeb92f5c, 23 },
    { 0x27492f5c, 77 },
    { 0x60f92f5d, 90 },
    { 0x93392f5c, 34 },
    { 0xaf792f60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e292f66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93492f6e, 52 },
    { 0xc5892f6f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd692f73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f692f76, 98 },
    { 0x7ac92f77, 75 },
    { 0xd5292f78, 98 },
    { 0x86892f76, 93 },
    { 0xafd92f7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9692f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0192f86, 65 },
    { 0, 0 },
    { 0xeb492f88, 90 },
    { 0xfed92f88, 81 },
    { 0x78f92f88, 82 },
    { 0xdb992f8a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf992f95, 52 },
    { 0, 0 },
    { 0x49392f97, 76 },
    { 0x9de92f98, 34 },
    { 0x16392f99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0792f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1992f9f, 91 },
    { 0xca92fa0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d92fa6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a92fad, 77 },
    { 0xd8592fae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd92fb1, 33 },
    { 0x70a92fb1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf92fb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1192fc0, 100 },
    { 0, 0 },
    { 0x4fe92fc2, 61 },
    { 0x67a92fc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f92fd3, 78 },
    { 0xf2492fd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a92fd7, 52 },
    { 0x6e992fd8, 23 },
    { 0, 0 },
    { 0x73492fda, 34 },
    { 0x34892fdb, 65 },
    { 0, 0 },
    { 0xf0b92fdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78892fe2, 55 },
    { 0x8f392fe3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b092fe6, 95 },
    { 0xc8592fe7, 33 },
    { 0x7092fe6, 61 },
    { 0x9bd92fe9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce92ff6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x26392ff9, 76 },
    { 0xe9392ffa, 61 },
    { 0xd1192ffb, 95 },
    { 0xa3292ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c92fff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea93002, 90 },
    { 0xccf93003, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf679300a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d9300d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1093010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf593017, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4579301c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6789301f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f693023, 61 },
    { 0x4a593024, 52 },
    { 0x74693025, 98 },
    { 0x25493026, 33 },
    { 0x2d493027, 34 },
    { 0x20a93028, 95 },
    { 0xfd593029, 23 },
    { 0xc5093025, 33 },
    { 0xf979302b, 95 },
    { 0x31393026, 77 },
    { 0xc719302a, 61 },
    { 0xeba9302e, 90 },
    { 0x6de93024, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97993033, 33 },
    { 0, 0 },
    { 0x55493035, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89593038, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939303b, 80 },
    { 0, 0 },
    { 0x6d69303d, 33 },
    { 0xe7c9303d, 66 },
    { 0, 0 },
    { 0x4b993040, 84 },
    { 0x23293041, 76 },
    { 0x9cc93042, 90 },
    { 0xa0993041, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4993048, 33 },
    { 0, 0 },
    { 0xda9304a, 99 },
    { 0, 0 },
    { 0x4bf9304c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f93050, 90 },
    { 0xdc93051, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e93056, 33 },
    { 0xa7293057, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb9305b, 90 },
    { 0xe999305c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc893061, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8893066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789306b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d9307d, 46 },
    { 0, 0 },
    { 0xa089307f, 98 },
    { 0x74a93080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c93083, 34 },
    { 0x34593084, 33 },
    { 0x1b593083, 34 },
    { 0x63793086, 98 },
    { 0xf6893087, 66 },
    { 0x4dd93087, 99 },
    { 0x1bf93088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a293095, 98 },
    { 0xfb793096, 55 },
    { 0, 0 },
    { 0xd5c93098, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6229309b, 23 },
    { 0xa59309c, 33 },
    { 0, 0 },
    { 0xed19309e, 98 },
    { 0x9799309f, 90 },
    { 0x489930a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21930ac, 90 },
    { 0, 0 },
    { 0xdec930ae, 95 },
    { 0x3ab930af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c930b3, 95 },
    { 0xd99930b3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa930b8, 34 },
    { 0xe00930b9, 33 },
    { 0x778930b9, 95 },
    { 0, 0 },
    { 0xd7b930bc, 90 },
    { 0x7b1930bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d930c0, 75 },
    { 0xa86930c1, 90 },
    { 0, 0 },
    { 0x97f930c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3930c6, 75 },
    { 0xc97930c7, 98 },
    { 0xae0930c7, 95 },
    { 0x38d930c7, 95 },
    { 0xb0930ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0930d3, 70 },
    { 0xc9f930d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d930d9, 93 },
    { 0, 0 },
    { 0x7ad930db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b930de, 61 },
    { 0, 0 },
    { 0x2df930e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71930e5, 55 },
    { 0x8c8930e6, 61 },
    { 0xb63930e6, 33 },
    { 0xda6930e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7930eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x255930ee, 67 },
    { 0x311930ef, 90 },
    { 0, 0 },
    { 0x36f930f1, 99 },
    { 0xeca930f2, 84 },
    { 0, 0 },
    { 0xdeb930f4, 61 },
    { 0, 0 },
    { 0x6a2930f6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac930fa, 90 },
    { 0xe60930fb, 91 },
    { 0x4f2930fb, 43 },
    { 0x31d930fb, 66 },
    { 0xc930fe, 33 },
    { 0, 0 },
    { 0x83093100, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f493105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f293109, 98 },
    { 0xd279310a, 67 },
    { 0x2d19310a, 17 },
    { 0x60e9310b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e93110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe993120, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17293127, 33 },
    { 0, 0 },
    { 0x90c93129, 15 },
    { 0xdc19312a, 98 },
    { 0xae69312b, 23 },
    { 0, 0 },
    { 0x5599312d, 23 },
    { 0xe549312e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f93131, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2493137, 55 },
    { 0x5a093138, 66 },
    { 0x95093137, 90 },
    { 0x5079313a, 33 },
    { 0, 0 },
    { 0x9a89313c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd169313f, 99 },
    { 0x4029313f, 61 },
    { 0x4c193140, 61 },
    { 0x57f93142, 81 },
    { 0x8de93141, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab93147, 55 },
    { 0x16893148, 61 },
    { 0x73593148, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c9314c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d9315b, 66 },
    { 0x2bc9315c, 67 },
    { 0, 0 },
    { 0xb3e9315e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9293164, 61 },
    { 0x40493164, 43 },
    { 0xce593165, 93 },
    { 0, 0 },
    { 0xc4d93168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c79316b, 61 },
    { 0x2449316b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a593171, 95 },
    { 0x79d93172, 55 },
    { 0x19093173, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f693177, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f89317f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65693182, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5293187, 95 },
    { 0x27a93188, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819318c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1e9318f, 98 },
    { 0x8db9318f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27393194, 23 },
    { 0, 0 },
    { 0x71293196, 85 },
    { 0x3f893197, 34 },
    { 0, 0 },
    { 0x8da93199, 85 },
    { 0xf2693199, 95 },
    { 0x82b9319b, 61 },
    { 0xaa69319b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5931a0, 98 },
    { 0xd06931a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff931a5, 90 },
    { 0, 0 },
    { 0xd2b931a7, 90 },
    { 0x28d931a8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab931ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75931ae, 55 },
    { 0, 0 },
    { 0x2c9931b0, 95 },
    { 0xbc9931b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c931b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24931b7, 43 },
    { 0x590931b8, 95 },
    { 0, 0 },
    { 0xd86931ba, 33 },
    { 0x6e931bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49931bf, 34 },
    { 0, 0 },
    { 0x30e931c1, 90 },
    { 0, 0 },
    { 0xd45931c3, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9931c6, 43 },
    { 0xee931c7, 33 },
    { 0x443931c7, 55 },
    { 0x278931c6, 90 },
    { 0x78f931ca, 33 },
    { 0x8a2931cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x819931ce, 80 },
    { 0, 0 },
    { 0xbbe931d0, 85 },
    { 0, 0 },
    { 0xd1c931d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee931df, 33 },
    { 0x8c8931df, 90 },
    { 0x753931e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x644931e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c931e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3931ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a931f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c931f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb931f9, 23 },
    { 0x49d931fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3931ff, 88 },
    { 0x2a593200, 90 },
    { 0x5a693201, 88 },
    { 0, 0 },
    { 0x8b593203, 77 },
    { 0xb3993203, 95 },
    { 0x16593204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb9320b, 33 },
    { 0, 0 },
    { 0x1099320d, 87 },
    { 0x7e19320d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2929321a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3993223, 91 },
    { 0, 0 },
    { 0xa6593225, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74493229, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a19322c, 90 },
    { 0, 0 },
    { 0x6219322e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b93233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a9323d, 90 },
    { 0x1e39323e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4393241, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0293245, 66 },
    { 0, 0 },
    { 0x9c193247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b19324c, 33 },
    { 0x4679324d, 55 },
    { 0x2699324e, 95 },
    { 0xe169324f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef93254, 90 },
    { 0xf1093254, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x12993258, 96 },
    { 0x14793259, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd9325c, 98 },
    { 0x2829325c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3393261, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2093264, 98 },
    { 0x4ab93265, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4df93268, 61 },
    { 0xd2d93269, 90 },
    { 0xd4993269, 61 },
    { 0xd649326b, 61 },
    { 0, 0 },
    { 0x84d9326d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a93278, 78 },
    { 0x4d293279, 61 },
    { 0x69c93278, 95 },
    { 0x4ae9327b, 98 },
    { 0, 0 },
    { 0xed49327d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b93281, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1169328a, 67 },
    { 0x6c19328b, 66 },
    { 0, 0 },
    { 0x4a9328d, 55 },
    { 0xbf69328e, 34 },
    { 0xabf9328d, 35 },
    { 0, 0 },
    { 0xbe293291, 17 },
    { 0, 0 },
    { 0x73793293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea993299, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91932a1, 90 },
    { 0xad7932a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a932a8, 95 },
    { 0x1a1932a9, 34 },
    { 0, 0 },
    { 0x64d932ab, 76 },
    { 0, 0 },
    { 0xa84932ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b932b0, 95 },
    { 0, 0 },
    { 0x7c0932b2, 98 },
    { 0xd1f932b3, 77 },
    { 0x11d932b3, 23 },
    { 0x1e0932b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13932b8, 34 },
    { 0, 0 },
    { 0xa7c932ba, 76 },
    { 0xf39932bb, 98 },
    { 0x172932bc, 46 },
    { 0xbf932bd, 98 },
    { 0x14b932bc, 98 },
    { 0, 0 },
    { 0xecf932c0, 23 },
    { 0xe6d932c1, 76 },
    { 0xbe6932c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3932c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9932cb, 95 },
    { 0xee9932cb, 77 },
    { 0xed0932cd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb932d0, 61 },
    { 0x397932d0, 56 },
    { 0, 0 },
    { 0x37a932d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61932d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x769932d9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f932dc, 23 },
    { 0x15d932dd, 33 },
    { 0, 0 },
    { 0xfe1932df, 34 },
    { 0, 0 },
    { 0x787932e1, 80 },
    { 0x5f7932e2, 66 },
    { 0x9e2932e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179932ea, 76 },
    { 0, 0 },
    { 0x38e932ec, 23 },
    { 0, 0 },
    { 0x696932ee, 90 },
    { 0x567932ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1932f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3932f6, 34 },
    { 0xde1932f7, 61 },
    { 0xe75932f8, 33 },
    { 0x415932f9, 100 },
    { 0x1d932fa, 52 },
    { 0, 0 },
    { 0x993932fc, 99 },
    { 0xaaf932fd, 100 },
    { 0x637932fe, 33 },
    { 0x4a7932fc, 34 },
    { 0x322932ff, 66 },
    { 0, 0 },
    { 0x50193302, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f93306, 52 },
    { 0, 0 },
    { 0xbc593308, 76 },
    { 0xaf493309, 95 },
    { 0, 0 },
    { 0xc9f9330b, 98 },
    { 0xf3e9330b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a93310, 90 },
    { 0, 0 },
    { 0x73793312, 43 },
    { 0xeca93313, 43 },
    { 0xf1293314, 90 },
    { 0x59293315, 34 },
    { 0, 0 },
    { 0x37793317, 91 },
    { 0xf8293318, 33 },
    { 0xead93319, 70 },
    { 0xe9b9331a, 90 },
    { 0, 0 },
    { 0xf849331c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24393324, 98 },
    { 0x72393324, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769332d, 43 },
    { 0xba59332e, 55 },
    { 0x3c79332f, 55 },
    { 0, 0 },
    { 0x42193331, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7093335, 23 },
    { 0, 0 },
    { 0xd5593337, 98 },
    { 0x4ba93337, 90 },
    { 0xd293337, 90 },
    { 0x53f9333a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77293340, 99 },
    { 0, 0 },
    { 0x73893342, 61 },
    { 0, 0 },
    { 0x90f93344, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x94593347, 90 },
    { 0, 0 },
    { 0x90493349, 90 },
    { 0xefc9334a, 90 },
    { 0xd369334b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e9334e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a993351, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a793357, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb29335b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf339335e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc293366, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80d93369, 95 },
    { 0, 0 },
    { 0xc009336b, 88 },
    { 0x5589336c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d93372, 9 },
    { 0xb8e93373, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8493379, 40 },
    { 0x85093379, 95 },
    { 0, 0 },
    { 0x8879337c, 43 },
    { 0, 0 },
    { 0xa1e9337e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a93383, 82 },
    { 0x5f293384, 43 },
    { 0x1e693384, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa439338f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b93394, 95 },
    { 0xff893394, 89 },
    { 0x9293396, 33 },
    { 0x1c793396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a9339d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744933a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa933a8, 61 },
    { 0x752933a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e933b0, 78 },
    { 0xcf933b0, 90 },
    { 0xde3933b2, 95 },
    { 0x944933b1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c933bd, 90 },
    { 0, 0 },
    { 0x6ea933bf, 75 },
    { 0, 0 },
    { 0x79c933c1, 95 },
    { 0, 0 },
    { 0x384933c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8933c6, 38 },
    { 0x9ee933c7, 55 },
    { 0x8c0933c8, 76 },
    { 0xe933c7, 33 },
    { 0xf78933ca, 61 },
    { 0x17e933c8, 18 },
    { 0x929933cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4933d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693933dd, 46 },
    { 0xa4b933de, 88 },
    { 0, 0 },
    { 0x39f933e0, 90 },
    { 0, 0 },
    { 0x84e933e2, 69 },
    { 0xeb3933e3, 76 },
    { 0, 0 },
    { 0x545933e5, 33 },
    { 0, 0 },
    { 0x1a7933e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c933ef, 90 },
    { 0xc53933f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4933f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2933f9, 55 },
    { 0xc6d933f9, 76 },
    { 0xdd6933fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf993403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93407, 85 },
    { 0x8da93408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4529340c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb9340f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e093415, 33 },
    { 0x72d93416, 67 },
    { 0, 0 },
    { 0x71593418, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe779341f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff93423, 90 },
    { 0x47193424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd969342d, 85 },
    { 0xda09342e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e93432, 66 },
    { 0x4fb93432, 40 },
    { 0x2f293434, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb393437, 52 },
    { 0x4ff93438, 76 },
    { 0x34d93439, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e9343c, 98 },
    { 0x3389343c, 55 },
    { 0xfbf9343c, 98 },
    { 0xd479343f, 33 },
    { 0xf2e9343f, 76 },
    { 0x49493440, 33 },
    { 0xff493442, 61 },
    { 0x86e93442, 100 },
    { 0xf1a93443, 23 },
    { 0x9c493443, 38 },
    { 0xe293446, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb9344e, 93 },
    { 0x9c9344f, 93 },
    { 0x2349344f, 61 },
    { 0x3f693451, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c89345d, 95 },
    { 0x9ec9345d, 33 },
    { 0xd3e9345f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0293465, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7709346a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4779346d, 43 },
    { 0xf4f9346e, 90 },
    { 0xc579346f, 66 },
    { 0xc4193470, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1569347b, 42 },
    { 0xc9d9347b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6299347f, 98 },
    { 0x81993480, 95 },
    { 0x54b93481, 91 },
    { 0x30e93480, 61 },
    { 0x96093482, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc93487, 95 },
    { 0x35593488, 34 },
    { 0xa8a93488, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad9348e, 55 },
    { 0x1869348f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b293495, 90 },
    { 0, 0 },
    { 0x81693497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2539349c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16f934a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461934ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6934b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa934b7, 34 },
    { 0, 0 },
    { 0xc38934b9, 76 },
    { 0x2e3934b9, 33 },
    { 0x922934b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5934c3, 61 },
    { 0, 0 },
    { 0xc93934c5, 43 },
    { 0x3ef934c6, 61 },
    { 0, 0 },
    { 0x4a934c8, 93 },
    { 0x536934c9, 90 },
    { 0x4a0934ca, 34 },
    { 0, 0 },
    { 0x4ea934cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c934cf, 75 },
    { 0, 0 },
    { 0xb1e934d1, 91 },
    { 0, 0 },
    { 0x34d934d3, 95 },
    { 0, 0 },
    { 0x6d934d5, 90 },
    { 0x46d934d6, 34 },
    { 0xf98934d7, 98 },
    { 0xe2b934d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea934db, 61 },
    { 0xf33934db, 61 },
    { 0, 0 },
    { 0x785934de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c934e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4934e5, 38 },
    { 0, 0 },
    { 0x8e3934e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c934ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa934ef, 98 },
    { 0x7b0934f0, 90 },
    { 0xc8934f1, 36 },
    { 0, 0 },
    { 0xdb5934f3, 52 },
    { 0, 0 },
    { 0x90934f5, 64 },
    { 0x5e934f6, 46 },
    { 0, 0 },
    { 0xe04934f8, 52 },
    { 0, 0 },
    { 0xff8934fa, 61 },
    { 0, 0 },
    { 0xe0e934fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6293509, 99 },
    { 0xd39350a, 33 },
    { 0x2979350b, 98 },
    { 0x8ae9350c, 91 },
    { 0xcd39350c, 46 },
    { 0x7af9350d, 95 },
    { 0x9869350a, 23 },
    { 0x8df93509, 76 },
    { 0xfd79350c, 95 },
    { 0x2dd93509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42793515, 95 },
    { 0x63b93516, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b93519, 52 },
    { 0, 0 },
    { 0x9299351b, 55 },
    { 0x3709351c, 76 },
    { 0x7e69351c, 90 },
    { 0, 0 },
    { 0x6b69351f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8293522, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4293527, 36 },
    { 0, 0 },
    { 0xb8e93529, 90 },
    { 0x7ee93529, 64 },
    { 0xfbb9352b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54793531, 95 },
    { 0xa1e93532, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f193535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd949353b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb329353e, 90 },
    { 0x5659353f, 93 },
    { 0xa79353e, 61 },
    { 0, 0 },
    { 0xa0393542, 81 },
    { 0xe8e93542, 33 },
    { 0xf5893544, 76 },
    { 0x86393544, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b193548, 46 },
    { 0x5f193548, 43 },
    { 0x6fd93548, 43 },
    { 0, 0 },
    { 0x2f49354c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13893551, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50393559, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6999355f, 61 },
    { 0xb9393560, 98 },
    { 0xcb193561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27793567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e9356a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4593578, 33 },
    { 0xc3d93578, 34 },
    { 0xddf93579, 55 },
    { 0x8ad9357b, 95 },
    { 0x53b9357b, 34 },
    { 0x6c99357b, 85 },
    { 0x2539357c, 33 },
    { 0xb409357f, 78 },
    { 0x79a9357f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec93586, 95 },
    { 0xbe993587, 90 },
    { 0xac93588, 98 },
    { 0xd893589, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd9358e, 84 },
    { 0, 0 },
    { 0x53393590, 90 },
    { 0, 0 },
    { 0x5b193592, 55 },
    { 0, 0 },
    { 0x1a293594, 92 },
    { 0, 0 },
    { 0xc3f93596, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d19359a, 55 },
    { 0xb489359b, 90 },
    { 0xa739359b, 33 },
    { 0x5109359d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xede935a0, 98 },
    { 0x6e7935a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce935a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a935ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89935b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee935be, 76 },
    { 0x703935bf, 76 },
    { 0, 0 },
    { 0xa70935c1, 95 },
    { 0xb5b935c1, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8935c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1935ca, 90 },
    { 0x663935ca, 33 },
    { 0xfb7935ca, 33 },
    { 0x21e935cd, 61 },
    { 0xd72935cd, 100 },
    { 0x92935cf, 91 },
    { 0xbae935cd, 76 },
    { 0x4b3935d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x327935d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd935dd, 61 },
    { 0, 0 },
    { 0xa48935df, 66 },
    { 0x459935e0, 61 },
    { 0x9d8935e0, 65 },
    { 0, 0 },
    { 0xb55935e3, 59 },
    { 0, 0 },
    { 0x52f935e5, 95 },
    { 0x59d935e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa935ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x493935f2, 43 },
    { 0, 0 },
    { 0x735935f4, 90 },
    { 0xdf9935f5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4935f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b935fd, 90 },
    { 0, 0 },
    { 0xb20935ff, 90 },
    { 0x54c93600, 90 },
    { 0xc2493601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b93607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299360a, 55 },
    { 0, 0 },
    { 0x6699360c, 93 },
    { 0x2209360d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a93613, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3049361e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5693622, 100 },
    { 0, 0 },
    { 0xca193624, 90 },
    { 0, 0 },
    { 0xfb093626, 43 },
    { 0xe5a93627, 23 },
    { 0xcc693628, 84 },
    { 0xe93627, 95 },
    { 0x4189362a, 98 },
    { 0x5929362b, 98 },
    { 0x28a9362a, 33 },
    { 0x1a69362d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e193630, 76 },
    { 0xe1893631, 87 },
    { 0xa2593632, 98 },
    { 0x98893630, 23 },
    { 0x5e393634, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d593637, 23 },
    { 0x23393638, 66 },
    { 0xb2193638, 43 },
    { 0x97e93637, 33 },
    { 0xef93639, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd59363e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20293642, 93 },
    { 0xeb993642, 33 },
    { 0x71793644, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a93647, 17 },
    { 0xddc93648, 61 },
    { 0xb2f93647, 95 },
    { 0x2889364a, 99 },
    { 0, 0 },
    { 0x4439364c, 77 },
    { 0x6159364d, 90 },
    { 0x5169364e, 90 },
    { 0x2de9364e, 43 },
    { 0xa1e9364e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c293656, 55 },
    { 0, 0 },
    { 0x22493658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9c9365b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d93662, 61 },
    { 0xd2093662, 19 },
    { 0, 0 },
    { 0x12593665, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5193668, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59366e, 95 },
    { 0x2f9366e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89993674, 93 },
    { 0, 0 },
    { 0x57593676, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f9367a, 90 },
    { 0xba9367b, 47 },
    { 0x1519367b, 98 },
    { 0, 0 },
    { 0x25f9367e, 55 },
    { 0xa789367f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1593683, 55 },
    { 0xc3f93683, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec93687, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a39368a, 95 },
    { 0x58c9368b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b9368e, 67 },
    { 0x5a79368f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a193694, 52 },
    { 0x85e93694, 23 },
    { 0xfa93695, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319936ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd936ae, 90 },
    { 0x7a5936af, 91 },
    { 0, 0 },
    { 0xec0936b1, 90 },
    { 0xca4936b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806936ba, 93 },
    { 0, 0 },
    { 0xb19936bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a936c3, 93 },
    { 0x7c2936c4, 33 },
    { 0x548936c5, 76 },
    { 0x728936c4, 23 },
    { 0x42936c7, 95 },
    { 0xb28936c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e936cc, 84 },
    { 0x378936cd, 76 },
    { 0xa0b936ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0936d8, 55 },
    { 0x7e2936d9, 95 },
    { 0x1a1936da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc936dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb936e1, 55 },
    { 0x8a9936e1, 90 },
    { 0, 0 },
    { 0xf8d936e4, 66 },
    { 0x1b7936e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d936e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a936ee, 90 },
    { 0x55b936ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35936f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x192936f5, 76 },
    { 0x360936f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73936fa, 33 },
    { 0x888936fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39493709, 33 },
    { 0xd079370a, 76 },
    { 0xb729370b, 76 },
    { 0xc2c9370c, 99 },
    { 0x819370d, 52 },
    { 0xed89370e, 61 },
    { 0, 0 },
    { 0x52b93710, 77 },
    { 0x7e093710, 33 },
    { 0x7d793712, 55 },
    { 0x1893712, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b93716, 33 },
    { 0xfbd93717, 68 },
    { 0xd5c93717, 91 },
    { 0xf4e93719, 90 },
    { 0, 0 },
    { 0x2b49371b, 46 },
    { 0, 0 },
    { 0x3d99371d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d93720, 34 },
    { 0x9fb93720, 90 },
    { 0, 0 },
    { 0xde493723, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc093727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599372e, 66 },
    { 0x7519372f, 90 },
    { 0x7f793730, 23 },
    { 0xf9593731, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x39193734, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22993737, 90 },
    { 0xd9693738, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e9373c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af93743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e93746, 91 },
    { 0, 0 },
    { 0xc6593748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129374b, 90 },
    { 0, 0 },
    { 0x8779374d, 98 },
    { 0, 0 },
    { 0xdaa9374f, 61 },
    { 0x20293750, 100 },
    { 0, 0 },
    { 0x8ef93752, 95 },
    { 0, 0 },
    { 0xc5793754, 56 },
    { 0xfa693754, 98 },
    { 0x13893756, 90 },
    { 0x2fc93757, 98 },
    { 0x48393757, 90 },
    { 0xc4993757, 43 },
    { 0x54e9375a, 33 },
    { 0x2989375b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7493766, 76 },
    { 0xd4993767, 95 },
    { 0x3cf93767, 61 },
    { 0xe393769, 46 },
    { 0x59d93766, 98 },
    { 0x3039376b, 75 },
    { 0x46793769, 76 },
    { 0x94e9376d, 23 },
    { 0xb579376e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43193774, 33 },
    { 0xc793775, 85 },
    { 0x7b593775, 95 },
    { 0xc8d93775, 85 },
    { 0, 0 },
    { 0xb9893779, 55 },
    { 0xede9377a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8519377d, 23 },
    { 0, 0 },
    { 0x9609377f, 33 },
    { 0x73f9377f, 95 },
    { 0, 0 },
    { 0xadc93782, 52 },
    { 0xb993783, 75 },
    { 0x2b193784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc893788, 23 },
    { 0x20393788, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb493792, 90 },
    { 0x82393793, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15e9379b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5349379e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff937a1, 23 },
    { 0, 0 },
    { 0xb48937a3, 78 },
    { 0xb8a937a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2937a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55937ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4937af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d937b2, 33 },
    { 0, 0 },
    { 0xcb0937b4, 33 },
    { 0, 0 },
    { 0x4aa937b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09937bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f937c2, 78 },
    { 0x2fc937c2, 95 },
    { 0, 0 },
    { 0xb59937c5, 90 },
    { 0x737937c5, 90 },
    { 0xd5c937c7, 87 },
    { 0x472937c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd937cb, 87 },
    { 0x55c937cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39937d0, 61 },
    { 0, 0 },
    { 0xa07937d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b937d6, 98 },
    { 0x4b9937d6, 95 },
    { 0, 0 },
    { 0xc85937d9, 90 },
    { 0x63a937d9, 33 },
    { 0xdea937db, 90 },
    { 0x1d4937dc, 61 },
    { 0, 0 },
    { 0xeb0937de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe937e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856937e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f937ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f937ed, 43 },
    { 0xc3e937ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639937f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x684937fb, 43 },
    { 0x3dd937fb, 76 },
    { 0xb91937fd, 95 },
    { 0xf91937fe, 90 },
    { 0, 0 },
    { 0x82393800, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1493804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f93808, 61 },
    { 0xf4a93809, 95 },
    { 0x6b693809, 61 },
    { 0x6ab9380b, 23 },
    { 0xd9b9380c, 61 },
    { 0xf9b9380c, 33 },
    { 0xa0b9380e, 55 },
    { 0x4a29380e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab993812, 46 },
    { 0x34893813, 93 },
    { 0x48293813, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f793817, 55 },
    { 0x7fb93817, 61 },
    { 0, 0 },
    { 0x3f39381a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5079381e, 61 },
    { 0, 0 },
    { 0x98b93820, 33 },
    { 0x6a693820, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61293832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb93837, 23 },
    { 0, 0 },
    { 0x32793839, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d9383c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb93842, 91 },
    { 0, 0 },
    { 0x1393844, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c893847, 33 },
    { 0x8a593847, 61 },
    { 0x18d93849, 95 },
    { 0xf549384a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce9384d, 23 },
    { 0xe839384e, 76 },
    { 0xce49384d, 81 },
    { 0x2a693850, 76 },
    { 0xcf393851, 55 },
    { 0xa9193851, 90 },
    { 0xe6193851, 75 },
    { 0x17893854, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1793858, 90 },
    { 0x38993858, 61 },
    { 0x56c93859, 87 },
    { 0x7a69385b, 84 },
    { 0x8119385b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1693861, 61 },
    { 0xd2993862, 55 },
    { 0, 0 },
    { 0x63a93864, 23 },
    { 0x40e93865, 88 },
    { 0x32993865, 100 },
    { 0x22293867, 61 },
    { 0xa5093868, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab9386f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd93873, 33 },
    { 0x6a893874, 100 },
    { 0xcf593874, 67 },
    { 0, 0 },
    { 0xb1993877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719387a, 23 },
    { 0x4b9387b, 38 },
    { 0x9309387c, 98 },
    { 0xad69387b, 76 },
    { 0x6799387e, 77 },
    { 0x4769387f, 76 },
    { 0xde89387e, 95 },
    { 0xb6e93880, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcda9388a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd679388f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f93896, 98 },
    { 0x1a793896, 61 },
    { 0xb8493898, 67 },
    { 0x29293899, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae9389d, 98 },
    { 0x29b9389e, 23 },
    { 0x7289389f, 68 },
    { 0x1fd9389f, 81 },
    { 0, 0 },
    { 0x297938a2, 61 },
    { 0, 0 },
    { 0x5cc938a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa938a7, 34 },
    { 0x557938a7, 23 },
    { 0xb8a938a9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80938ad, 90 },
    { 0x21a938ae, 90 },
    { 0xb15938ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x340938b2, 96 },
    { 0x900938b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5938b6, 78 },
    { 0x194938b7, 95 },
    { 0, 0 },
    { 0x9b1938b9, 85 },
    { 0x5eb938ba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a938c2, 51 },
    { 0x244938c3, 90 },
    { 0x46b938c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d938c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x765938cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe938d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861938d7, 46 },
    { 0x1f5938d8, 46 },
    { 0xb90938d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0938e0, 90 },
    { 0, 0 },
    { 0x53a938e2, 23 },
    { 0, 0 },
    { 0x842938e4, 33 },
    { 0, 0 },
    { 0xbe4938e6, 90 },
    { 0, 0 },
    { 0xf03938e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54938ee, 33 },
    { 0x7f6938ef, 33 },
    { 0x2fa938f0, 87 },
    { 0x212938f1, 33 },
    { 0xd2d938f0, 34 },
    { 0xb01938f3, 43 },
    { 0xb17938ef, 23 },
    { 0, 0 },
    { 0x3d938f6, 33 },
    { 0, 0 },
    { 0xec3938f8, 46 },
    { 0, 0 },
    { 0xba5938fa, 23 },
    { 0x8c938fb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x26938fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26193902, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7899390b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a79390e, 98 },
    { 0, 0 },
    { 0x11393910, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e93918, 66 },
    { 0, 0 },
    { 0x6d89391a, 91 },
    { 0x9bb9391b, 90 },
    { 0x1ae9391b, 34 },
    { 0xd1d9391d, 91 },
    { 0xb369391c, 90 },
    { 0x2f99391b, 87 },
    { 0x4529391f, 76 },
    { 0xab79391f, 78 },
    { 0xf0293922, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e93925, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7493928, 76 },
    { 0x7f693929, 61 },
    { 0xb8393929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca93932, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c9393a, 33 },
    { 0, 0 },
    { 0x6399393c, 98 },
    { 0, 0 },
    { 0x1999393e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e93943, 33 },
    { 0, 0 },
    { 0xec093945, 66 },
    { 0x4d393945, 33 },
    { 0x11d93947, 61 },
    { 0x67793947, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc449394f, 84 },
    { 0x6593950, 77 },
    { 0xf5393950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a893954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd779395b, 100 },
    { 0x5089395b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c793960, 93 },
    { 0, 0 },
    { 0x2f93962, 90 },
    { 0x96493963, 61 },
    { 0x22c93964, 100 },
    { 0xe7493965, 66 },
    { 0x73993963, 88 },
    { 0x3593966, 33 },
    { 0xf3193965, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb49396b, 90 },
    { 0x18e9396c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1293971, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf93974, 99 },
    { 0, 0 },
    { 0x47993976, 33 },
    { 0, 0 },
    { 0x8e793978, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9a9397e, 88 },
    { 0xdad9397e, 99 },
    { 0x7da93980, 90 },
    { 0x3009397f, 43 },
    { 0x4d693981, 87 },
    { 0, 0 },
    { 0x81093984, 23 },
    { 0, 0 },
    { 0x31d93986, 33 },
    { 0x5e693987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8759398a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b9398d, 52 },
    { 0xa419398d, 65 },
    { 0x3639398e, 43 },
    { 0xafe9398e, 46 },
    { 0x7a293991, 90 },
    { 0, 0 },
    { 0x1c193993, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa793996, 98 },
    { 0x64693997, 23 },
    { 0xea793998, 43 },
    { 0, 0 },
    { 0xd49399a, 33 },
    { 0x4109399b, 56 },
    { 0x6729399a, 61 },
    { 0xaa39399d, 97 },
    { 0x2059399d, 23 },
    { 0x1a69399f, 98 },
    { 0x20d9399f, 55 },
    { 0x4ee9399c, 61 },
    { 0x9cd9399d, 83 },
    { 0x8ff939a3, 99 },
    { 0x327939a3, 95 },
    { 0, 0 },
    { 0x6ff939a6, 33 },
    { 0x36d939a7, 98 },
    { 0x528939a6, 95 },
    { 0xdb3939a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e939ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a939b1, 66 },
    { 0, 0 },
    { 0xeca939b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d939b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372939bd, 61 },
    { 0xee7939bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e939c1, 61 },
    { 0x7d0939c2, 68 },
    { 0x147939c3, 17 },
    { 0, 0 },
    { 0x605939c5, 91 },
    { 0x3f2939c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4939cd, 61 },
    { 0, 0 },
    { 0xeb8939cf, 81 },
    { 0, 0 },
    { 0xf45939d1, 61 },
    { 0x9de939d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8939d6, 61 },
    { 0x863939d6, 98 },
    { 0, 0 },
    { 0x796939d9, 90 },
    { 0x73e939da, 84 },
    { 0, 0 },
    { 0xb23939dc, 90 },
    { 0xee1939dc, 75 },
    { 0, 0 },
    { 0x289939df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0939e3, 33 },
    { 0xa0b939e3, 91 },
    { 0xb4f939e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0939e9, 33 },
    { 0xd3f939e9, 43 },
    { 0xf02939ea, 33 },
    { 0xda1939ec, 61 },
    { 0xca8939ec, 33 },
    { 0x24e939ee, 76 },
    { 0, 0 },
    { 0x1be939f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b939f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6939f6, 98 },
    { 0, 0 },
    { 0x9dd939f8, 61 },
    { 0x127939f8, 33 },
    { 0x691939fa, 23 },
    { 0x634939fb, 61 },
    { 0xc8e939fc, 46 },
    { 0xe30939fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f993a00, 78 },
    { 0x54193a01, 23 },
    { 0, 0 },
    { 0x7493a03, 55 },
    { 0x7a693a04, 43 },
    { 0x1d93a04, 98 },
    { 0xa8193a06, 34 },
    { 0x1c593a07, 90 },
    { 0x38f93a07, 65 },
    { 0xfb793a07, 36 },
    { 0x69993a06, 33 },
    { 0, 0 },
    { 0x35393a0c, 52 },
    { 0x1f893a0c, 98 },
    { 0x99b93a0c, 33 },
    { 0, 0 },
    { 0x8f293a10, 33 },
    { 0xb5993a11, 83 },
    { 0, 0 },
    { 0x18493a13, 90 },
    { 0xcf93a13, 90 },
    { 0x56d93a15, 98 },
    { 0xdb693a16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a93a1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b93a21, 94 },
    { 0xfa293a22, 43 },
    { 0xe2b93a23, 34 },
    { 0x6db93a22, 100 },
    { 0x96b93a21, 90 },
    { 0x92c93a22, 22 },
    { 0xa9593a27, 90 },
    { 0x3e93a27, 91 },
    { 0, 0 },
    { 0x8a93a2a, 23 },
    { 0xeec93a2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f593a3c, 23 },
    { 0xc9393a3c, 33 },
    { 0, 0 },
    { 0xc7c93a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75993a42, 90 },
    { 0xd1293a42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf693a47, 33 },
    { 0x57f93a47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2593a4b, 69 },
    { 0, 0 },
    { 0x40993a4d, 37 },
    { 0, 0 },
    { 0x30693a4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75093a53, 90 },
    { 0xcae93a53, 55 },
    { 0x8a493a55, 98 },
    { 0x93793a55, 33 },
    { 0x2f193a53, 23 },
    { 0xef393a54, 36 },
    { 0x37893a59, 81 },
    { 0x74b93a5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87e93a61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85493a64, 93 },
    { 0, 0 },
    { 0x6e93a66, 90 },
    { 0, 0 },
    { 0xdf793a68, 22 },
    { 0xf2093a69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f93a73, 91 },
    { 0xe5593a73, 43 },
    { 0, 0 },
    { 0x58c93a76, 98 },
    { 0xdf693a76, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d93a7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa693a81, 43 },
    { 0, 0 },
    { 0x9e493a83, 33 },
    { 0, 0 },
    { 0xf493a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f393a89, 98 },
    { 0x63f93a89, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b993a8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e593a90, 61 },
    { 0xa1493a91, 61 },
    { 0, 0 },
    { 0x6e493a93, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a93a98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c93a9f, 33 },
    { 0, 0 },
    { 0xbdf93aa1, 78 },
    { 0, 0 },
    { 0x95293aa3, 95 },
    { 0xdc093aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8193aa7, 61 },
    { 0xacf93aa8, 34 },
    { 0xc0693aa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe193aad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51693ab0, 55 },
    { 0xebe93ab1, 52 },
    { 0, 0 },
    { 0xb2d93ab3, 61 },
    { 0xa7193ab4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe893ab9, 46 },
    { 0, 0 },
    { 0x1dc93abb, 34 },
    { 0xcc93abb, 12 },
    { 0, 0 },
    { 0x6cf93abe, 40 },
    { 0x59a93abf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c93ac2, 77 },
    { 0xb8293ac3, 76 },
    { 0, 0 },
    { 0x68493ac5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43293acc, 33 },
    { 0xdd193acd, 23 },
    { 0xe1a93acc, 52 },
    { 0, 0 },
    { 0xa9193ad0, 69 },
    { 0x58e93ad1, 78 },
    { 0xdd93ad2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f893ad5, 23 },
    { 0xcd293ad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4293adf, 66 },
    { 0, 0 },
    { 0x98093ae1, 67 },
    { 0xf2d93ae2, 95 },
    { 0xa4593ae1, 33 },
    { 0xba993ae4, 33 },
    { 0xd0f93ae5, 84 },
    { 0xc0893ae4, 90 },
    { 0x72f93ae7, 99 },
    { 0x38f93ae8, 66 },
    { 0x6b093ae9, 75 },
    { 0x25193aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da93aed, 33 },
    { 0x2ec93aee, 98 },
    { 0x98393aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b93af8, 80 },
    { 0, 0 },
    { 0xf2493afa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de93afd, 98 },
    { 0xba793afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba93b01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e93b04, 33 },
    { 0, 0 },
    { 0xebb93b06, 33 },
    { 0, 0 },
    { 0xde893b08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b393b0b, 77 },
    { 0xa0c93b0b, 43 },
    { 0x97d93b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d093b10, 95 },
    { 0, 0 },
    { 0xccf93b12, 98 },
    { 0x66793b13, 90 },
    { 0, 0 },
    { 0x73693b15, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2493b18, 34 },
    { 0, 0 },
    { 0xa9893b1a, 52 },
    { 0x69993b1b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de93b20, 78 },
    { 0xe393b20, 76 },
    { 0x32893b22, 55 },
    { 0xf5b93b23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72993b28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3893b2c, 23 },
    { 0x9d993b2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5993b31, 61 },
    { 0x6c693b31, 61 },
    { 0x2e93b33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb93b36, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a993b3d, 90 },
    { 0x5fe93b3e, 85 },
    { 0x6a293b3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49393b44, 78 },
    { 0x84293b45, 68 },
    { 0x98593b45, 93 },
    { 0x61893b46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42093b4a, 34 },
    { 0x48d93b4a, 33 },
    { 0xd0993b4c, 52 },
    { 0x6a993b4d, 77 },
    { 0x36d93b4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a93b51, 17 },
    { 0xe2b93b52, 76 },
    { 0, 0 },
    { 0x53493b54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e93b57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45893b5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7593b61, 23 },
    { 0x7c593b61, 23 },
    { 0xd293b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41893b6b, 90 },
    { 0x92493b6c, 95 },
    { 0x26c93b6d, 33 },
    { 0x60793b6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc593b77, 65 },
    { 0x30493b77, 93 },
    { 0xb7893b79, 33 },
    { 0, 0 },
    { 0xf0e93b7b, 55 },
    { 0x80f93b7c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46893b80, 98 },
    { 0, 0 },
    { 0x3e393b82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3093b85, 44 },
    { 0xcdf93b86, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f93b89, 90 },
    { 0x7d893b8a, 91 },
    { 0x59993b89, 33 },
    { 0xb4a93b8c, 34 },
    { 0x84093b89, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c93b90, 99 },
    { 0, 0 },
    { 0x38a93b92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e93b98, 78 },
    { 0xaef93b99, 100 },
    { 0x37d93b9a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc93ba0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf93ba8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b93bac, 76 },
    { 0x4f093bad, 23 },
    { 0xfb293bae, 100 },
    { 0x81393baf, 77 },
    { 0x7b93baf, 96 },
    { 0xd8093bb1, 81 },
    { 0x45793bb1, 76 },
    { 0x77b93bb3, 43 },
    { 0x69193bb0, 33 },
    { 0xa3293bb5, 55 },
    { 0, 0 },
    { 0xd7e93bb7, 76 },
    { 0, 0 },
    { 0xa1f93bb9, 83 },
    { 0xe5793bba, 81 },
    { 0x18193bba, 78 },
    { 0x53d93bba, 55 },
    { 0, 0 },
    { 0x3f193bbe, 90 },
    { 0x96b93bbf, 61 },
    { 0xa5d93bc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71293bc3, 95 },
    { 0, 0 },
    { 0x8ec93bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33593bc8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d093bcb, 78 },
    { 0xeb793bcb, 33 },
    { 0x12093bcc, 87 },
    { 0xb3193bcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad993bd3, 95 },
    { 0, 0 },
    { 0x4193bd5, 66 },
    { 0xa2393bd5, 90 },
    { 0xf9693bd7, 17 },
    { 0x13593bd8, 76 },
    { 0x87c93bd9, 93 },
    { 0, 0 },
    { 0xd6393bdb, 61 },
    { 0x58993bdb, 98 },
    { 0, 0 },
    { 0x90093bde, 95 },
    { 0, 0 },
    { 0x54893be0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e093be4, 76 },
    { 0x70293be5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe93bea, 91 },
    { 0xa393beb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1493bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b393bf9, 87 },
    { 0, 0 },
    { 0xc0493bfb, 90 },
    { 0x39a93bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e193c03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1293c06, 34 },
    { 0x3b593c06, 90 },
    { 0x5ef93c08, 55 },
    { 0xb6293c08, 95 },
    { 0xbd293c0a, 77 },
    { 0x21693c09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c93c13, 33 },
    { 0x3a293c13, 46 },
    { 0xf7693c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d93c1a, 90 },
    { 0x13f93c1a, 90 },
    { 0x2dd93c1c, 61 },
    { 0x4c793c1d, 23 },
    { 0, 0 },
    { 0x93c93c1f, 61 },
    { 0x4c593c20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d93c25, 33 },
    { 0x1d793c25, 61 },
    { 0x6d493c27, 90 },
    { 0xf0293c28, 61 },
    { 0x43f93c27, 98 },
    { 0x4ff93c29, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9193c34, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40893c39, 98 },
    { 0, 0 },
    { 0xa393c3b, 68 },
    { 0x67c93c3c, 33 },
    { 0xb0093c3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x30393c40, 61 },
    { 0xb1893c41, 90 },
    { 0x5a93c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa93c45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21793c49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0593c4f, 61 },
    { 0xfb993c50, 40 },
    { 0x46693c50, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c93c5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36793c5d, 61 },
    { 0x34093c5e, 68 },
    { 0xeb693c5d, 93 },
    { 0xbe593c5e, 61 },
    { 0x94793c61, 55 },
    { 0x4cd93c61, 66 },
    { 0x4293c5f, 90 },
    { 0xf0093c64, 28 },
    { 0x82193c64, 40 },
    { 0x1d693c66, 33 },
    { 0x3a793c5d, 90 },
    { 0x4e593c68, 78 },
    { 0, 0 },
    { 0x6993c6a, 23 },
    { 0, 0 },
    { 0x7793c6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c293c75, 46 },
    { 0, 0 },
    { 0xfb193c77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49693c7d, 33 },
    { 0x6fe93c7d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe93c82, 61 },
    { 0xe7193c82, 90 },
    { 0, 0 },
    { 0x83f93c85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed993c8e, 23 },
    { 0x6b093c8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f93c97, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73193c9f, 33 },
    { 0x51c93ca0, 90 },
    { 0xa293ca0, 84 },
    { 0x35d93ca2, 90 },
    { 0x4093ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b93ca6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d093cb2, 43 },
    { 0x71393cb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf893cb8, 56 },
    { 0x85693cb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3693cbd, 61 },
    { 0, 0 },
    { 0x4293cbf, 33 },
    { 0xb993cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b93cc9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9193ccd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b93cd5, 66 },
    { 0x36093cd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a93ce0, 76 },
    { 0x91f93ce1, 33 },
    { 0xc4d93ce1, 90 },
    { 0xe3293ce1, 61 },
    { 0xc9393ce4, 93 },
    { 0x80393ce5, 90 },
    { 0x52293ce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51693ce9, 85 },
    { 0x59c93cea, 76 },
    { 0x31093ceb, 90 },
    { 0x8ff93ce9, 98 },
    { 0x53f93ced, 33 },
    { 0x50493ced, 61 },
    { 0, 0 },
    { 0xe3c93cf0, 61 },
    { 0x39093cf1, 23 },
    { 0xc2693cf0, 84 },
    { 0xe693cf3, 61 },
    { 0xbc493cf4, 98 },
    { 0x7b493cf3, 95 },
    { 0x58b93cf4, 55 },
    { 0, 0 },
    { 0x87893cf8, 59 },
    { 0, 0 },
    { 0xff893cfa, 45 },
    { 0x9fd93cfb, 90 },
    { 0xff993cfc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3393cff, 90 },
    { 0, 0 },
    { 0x5993d01, 55 },
    { 0xa1893d02, 90 },
    { 0x52493d01, 23 },
    { 0, 0 },
    { 0x45993d05, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e393d08, 55 },
    { 0, 0 },
    { 0x54f93d0a, 33 },
    { 0, 0 },
    { 0x11593d0c, 84 },
    { 0x77893d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa393d10, 33 },
    { 0x5ed93d10, 61 },
    { 0xf3f93d12, 55 },
    { 0x79893d13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x57593d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8093d1c, 85 },
    { 0x64093d1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81793d21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42593d25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b93d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa693d2b, 23 },
    { 0xa8993d2c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25493d31, 33 },
    { 0, 0 },
    { 0x50c93d33, 76 },
    { 0x2b793d34, 67 },
    { 0x10a93d34, 99 },
    { 0x8b993d34, 61 },
    { 0xf9993d35, 87 },
    { 0xded93d38, 90 },
    { 0, 0 },
    { 0xc093d3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d93d41, 23 },
    { 0x26093d41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93693d45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea93d49, 98 },
    { 0x56c93d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c393d50, 55 },
    { 0x73f93d51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60393d55, 98 },
    { 0x55a93d56, 90 },
    { 0xb2993d56, 66 },
    { 0, 0 },
    { 0x81393d59, 55 },
    { 0x6b93d5a, 66 },
    { 0x91f93d5b, 90 },
    { 0x9c693d5c, 95 },
    { 0xadb93d5d, 76 },
    { 0x5b293d5e, 98 },
    { 0x9c693d5f, 98 },
    { 0xb3593d5f, 90 },
    { 0xf8193d5f, 55 },
    { 0xce693d62, 90 },
    { 0xffb93d62, 89 },
    { 0x7b393d62, 55 },
    { 0x11193d65, 100 },
    { 0x73e93d66, 81 },
    { 0x29493d66, 93 },
    { 0xeec93d63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f893d6e, 33 },
    { 0x9c793d6f, 93 },
    { 0x19c93d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b93d7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a93d80, 95 },
    { 0x9eb93d81, 33 },
    { 0x5ac93d82, 51 },
    { 0x43b93d83, 78 },
    { 0x8f393d84, 98 },
    { 0x4d993d84, 76 },
    { 0x7fa93d82, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b93d89, 61 },
    { 0, 0 },
    { 0x3c093d8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf93d8f, 96 },
    { 0x98093d90, 23 },
    { 0x6a793d90, 95 },
    { 0x2b793d92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce93d96, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26393d9c, 40 },
    { 0x60e93d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90393da2, 30 },
    { 0xf1c93da3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1393da7, 75 },
    { 0, 0 },
    { 0x24593da9, 90 },
    { 0x8d893daa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd93db1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e93db4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6993db9, 55 },
    { 0x1293dba, 90 },
    { 0x18593dbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad393dbe, 55 },
    { 0xf4393dbe, 61 },
    { 0, 0 },
    { 0x4ea93dc1, 34 },
    { 0xe4a93dc2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a93dc6, 64 },
    { 0, 0 },
    { 0x68993dc8, 90 },
    { 0x6aa93dc9, 90 },
    { 0x40893dca, 61 },
    { 0xb4193dcb, 46 },
    { 0x4d093dcb, 33 },
    { 0x7a893dc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c093dd0, 23 },
    { 0x53393dd1, 46 },
    { 0xf2093dd2, 23 },
    { 0xe6693dd2, 33 },
    { 0x75793dd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f93dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5793ddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce493de2, 95 },
    { 0xe4d93de3, 90 },
    { 0, 0 },
    { 0xa7193de5, 46 },
    { 0xddb93de5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5193dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab393df6, 34 },
    { 0, 0 },
    { 0x85a93df8, 75 },
    { 0x8f393df9, 44 },
    { 0xb2e93df8, 55 },
    { 0xe3a93dfa, 100 },
    { 0x49093dfc, 90 },
    { 0xee293dfc, 93 },
    { 0x3e293df8, 33 },
    { 0xf1393df9, 23 },
    { 0x45893e00, 95 },
    { 0x4c493e00, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2093e04, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33893e0d, 76 },
    { 0xb8493e0e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c893e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6093e15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52893e1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee93e1f, 38 },
    { 0x48b93e20, 76 },
    { 0x9b193e20, 55 },
    { 0x8b893e22, 98 },
    { 0x21793e22, 90 },
    { 0, 0 },
    { 0xfb693e25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e93e2c, 99 },
    { 0x88e93e2d, 45 },
    { 0, 0 },
    { 0x48293e2f, 49 },
    { 0xbb493e2f, 23 },
    { 0x94793e30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc93e35, 61 },
    { 0x29d93e35, 82 },
    { 0, 0 },
    { 0xd2d93e38, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41093e3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66093e44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a93e47, 33 },
    { 0x49a93e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a993e4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c93e53, 60 },
    { 0x7d93e53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a93e58, 33 },
    { 0x92b93e59, 93 },
    { 0, 0 },
    { 0x25093e5b, 87 },
    { 0, 0 },
    { 0xaef93e5d, 98 },
    { 0x8de93e5d, 90 },
    { 0x39e93e5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c593e63, 90 },
    { 0xa4593e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd993e68, 90 },
    { 0x3cb93e69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0093e6e, 95 },
    { 0, 0 },
    { 0xf9893e70, 76 },
    { 0x54793e71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d93e74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a193e7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20493e86, 61 },
    { 0x1d493e87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd93e94, 95 },
    { 0x4fc93e94, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3293e98, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d93e9e, 46 },
    { 0x9de93e9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7293ea2, 43 },
    { 0x7d693ea3, 76 },
    { 0, 0 },
    { 0xb9f93ea5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56e93eaa, 55 },
    { 0xcca93eaa, 67 },
    { 0, 0 },
    { 0x92793ead, 81 },
    { 0x95893eae, 90 },
    { 0x6cb93eaf, 66 },
    { 0xe6093eb0, 52 },
    { 0x53f93eb0, 24 },
    { 0x38693eb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d93eb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d193eba, 90 },
    { 0x2ef93ebb, 23 },
    { 0x86093ebc, 98 },
    { 0, 0 },
    { 0x44d93ebe, 95 },
    { 0xfcd93ebf, 90 },
    { 0x64193ec0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e693ec3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f93ec7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca593ece, 34 },
    { 0x63d93ecf, 65 },
    { 0, 0 },
    { 0x68c93ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d93eda, 90 },
    { 0, 0 },
    { 0x29393edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f93eea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee93eef, 53 },
    { 0xed993ef0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c93ef3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d93ef6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a293ef9, 87 },
    { 0xda693ef9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48293eff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e93f04, 90 },
    { 0xe7293f05, 93 },
    { 0x62893f06, 33 },
    { 0, 0 },
    { 0x12093f08, 34 },
    { 0x74693f08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3893f14, 33 },
    { 0x25393f14, 78 },
    { 0x49793f14, 84 },
    { 0x82893f16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b93f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb393f20, 76 },
    { 0x16893f21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3793f24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d693f2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x493f2e, 98 },
    { 0xfd393f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e393f33, 23 },
    { 0x9c93f34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91893f39, 61 },
    { 0x6f693f3a, 43 },
    { 0x9d393f3b, 78 },
    { 0xb4c93f3c, 33 },
    { 0x11a93f39, 96 },
    { 0x31093f3b, 33 },
    { 0xb3d93f3f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab393f43, 55 },
    { 0x63893f44, 34 },
    { 0xd1793f45, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0493f48, 95 },
    { 0x7aa93f49, 98 },
    { 0x79093f4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca693f4e, 33 },
    { 0, 0 },
    { 0x28493f50, 61 },
    { 0xf1493f50, 33 },
    { 0xda93f51, 78 },
    { 0x23a93f51, 90 },
    { 0, 0 },
    { 0xfd393f55, 17 },
    { 0, 0 },
    { 0x2b393f57, 90 },
    { 0xffc93f58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef93f60, 69 },
    { 0, 0 },
    { 0x9e893f62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5693f67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c093f6a, 61 },
    { 0xc2593f6b, 61 },
    { 0x6d793f6c, 90 },
    { 0xa5293f6d, 55 },
    { 0x50593f6e, 77 },
    { 0x80193f6a, 33 },
    { 0x25693f6c, 17 },
    { 0x6d293f71, 95 },
    { 0, 0 },
    { 0x67a93f73, 85 },
    { 0x8da93f73, 95 },
    { 0x59993f75, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e93f7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f493f7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d993f82, 33 },
    { 0xc2193f83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa93f8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f93f90, 96 },
    { 0xd9393f91, 85 },
    { 0x65993f90, 90 },
    { 0xf8093f90, 61 },
    { 0xbd593f94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d393f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77693f9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f93fa4, 78 },
    { 0x6f493fa5, 90 },
    { 0x17d93fa5, 34 },
    { 0x44193fa7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0893fac, 92 },
    { 0, 0 },
    { 0x9c193fae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b93fb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff593fb5, 77 },
    { 0, 0 },
    { 0xdee93fb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf193fbd, 34 },
    { 0xd5993fbd, 43 },
    { 0, 0 },
    { 0xbec93fc0, 55 },
    { 0x9193fc1, 93 },
    { 0x99493fc2, 23 },
    { 0xce293fc2, 95 },
    { 0, 0 },
    { 0x42293fc5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e93fc8, 88 },
    { 0, 0 },
    { 0xce493fca, 23 },
    { 0x25c93fcb, 90 },
    { 0, 0 },
    { 0xf0493fcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb93fd1, 98 },
    { 0x56e93fd2, 91 },
    { 0x96493fd2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42293fde, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44793fe3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1593fe7, 76 },
    { 0x4d093fe8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a93ff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56193ff7, 43 },
    { 0xa7593ff7, 93 },
    { 0xe3093ff8, 34 },
    { 0xd8893ffa, 90 },
    { 0x3eb93ff8, 34 },
    { 0xb2f93ffc, 34 },
    { 0x29793ffd, 28 },
    { 0xb3393ffd, 33 },
    { 0xca093ffe, 33 },
    { 0x6da93ffe, 98 },
    { 0xf1393ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1094007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5959400c, 98 },
    { 0xff59400d, 95 },
    { 0x6959400c, 43 },
    { 0xddb9400f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68894012, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac94015, 34 },
    { 0xce694015, 34 },
    { 0, 0 },
    { 0x6cb94018, 97 },
    { 0x84294018, 85 },
    { 0, 0 },
    { 0xef09401b, 76 },
    { 0x77a9401b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc9401f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d94023, 35 },
    { 0x18094024, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b94027, 61 },
    { 0x45d94027, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc49402b, 55 },
    { 0x28e9402c, 43 },
    { 0x4ed9402d, 94 },
    { 0, 0 },
    { 0x3259402f, 84 },
    { 0x65a94030, 61 },
    { 0xc5394031, 61 },
    { 0x88a94030, 95 },
    { 0x99994032, 33 },
    { 0x91a94032, 95 },
    { 0xa994035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5249403d, 55 },
    { 0xd7a9403e, 43 },
    { 0xed59403f, 91 },
    { 0x4f194040, 61 },
    { 0x42294041, 34 },
    { 0, 0 },
    { 0x59094043, 78 },
    { 0, 0 },
    { 0xd6294045, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf9404c, 46 },
    { 0x4c59404d, 93 },
    { 0xd2d9404d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5794051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e94054, 76 },
    { 0x98694055, 76 },
    { 0x21e94055, 99 },
    { 0, 0 },
    { 0xbc194058, 95 },
    { 0x33894058, 66 },
    { 0x54a9405a, 33 },
    { 0xae39405b, 61 },
    { 0x4b9405c, 90 },
    { 0xf429405d, 33 },
    { 0xeeb9405d, 33 },
    { 0x2109405f, 23 },
    { 0xcbf94060, 33 },
    { 0, 0 },
    { 0x77594062, 61 },
    { 0xb2994063, 33 },
    { 0, 0 },
    { 0xdc894065, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9406b, 76 },
    { 0xf4e9406b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf94074, 33 },
    { 0xca894075, 95 },
    { 0x2094074, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c9407a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da9408c, 55 },
    { 0, 0 },
    { 0x6539408e, 87 },
    { 0x2859408f, 90 },
    { 0x7569408e, 98 },
    { 0x3b194091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f94094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af94098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc329409b, 43 },
    { 0xdb29409c, 69 },
    { 0x5059409d, 34 },
    { 0x9529409d, 99 },
    { 0xf619409f, 61 },
    { 0x6519409f, 33 },
    { 0x70c940a1, 90 },
    { 0xa8a9409e, 33 },
    { 0x141940a3, 56 },
    { 0xb129409e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46940a7, 90 },
    { 0, 0 },
    { 0xcbc940a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8940ac, 90 },
    { 0xe82940ad, 33 },
    { 0, 0 },
    { 0x8e5940af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f940b9, 40 },
    { 0, 0 },
    { 0x578940bb, 90 },
    { 0x3ba940bc, 55 },
    { 0x271940bd, 33 },
    { 0x9ea940be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39940c1, 23 },
    { 0x327940c2, 90 },
    { 0xdfd940c2, 40 },
    { 0x5a0940c4, 23 },
    { 0, 0 },
    { 0x67e940c6, 55 },
    { 0xceb940c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83c940cb, 98 },
    { 0xe30940cc, 33 },
    { 0x50940cd, 80 },
    { 0xa4d940cd, 33 },
    { 0x3c7940cc, 52 },
    { 0xb99940cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xec940d3, 61 },
    { 0x189940d4, 61 },
    { 0xe79940d5, 33 },
    { 0x92a940d5, 95 },
    { 0xa6940d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e940db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1940e1, 61 },
    { 0x2f9940e2, 23 },
    { 0xb62940e1, 24 },
    { 0xafe940e1, 23 },
    { 0xe90940e5, 85 },
    { 0x697940e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac940e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e940ee, 99 },
    { 0, 0 },
    { 0xd1e940f0, 61 },
    { 0x68940f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a940fc, 88 },
    { 0xf2b940fc, 23 },
    { 0, 0 },
    { 0x7b6940ff, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x86594102, 43 },
    { 0, 0 },
    { 0x1b294104, 90 },
    { 0xb294104, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x45494108, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31994110, 98 },
    { 0xa194110, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4894114, 61 },
    { 0xcff94114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b9411c, 33 },
    { 0x36e9411d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6794122, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c494129, 61 },
    { 0x91894129, 90 },
    { 0xbee9412a, 93 },
    { 0xbc19412c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe449412f, 95 },
    { 0x31294130, 91 },
    { 0x7dd94131, 23 },
    { 0, 0 },
    { 0x6d794133, 90 },
    { 0xe3494134, 23 },
    { 0xf394135, 33 },
    { 0x55094135, 98 },
    { 0xcfc94137, 95 },
    { 0xa4594137, 33 },
    { 0, 0 },
    { 0x5309413a, 90 },
    { 0x4189413b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed9413e, 90 },
    { 0xab49413f, 17 },
    { 0x7d9413f, 96 },
    { 0x9a69413f, 33 },
    { 0x59a94141, 81 },
    { 0x4b394143, 33 },
    { 0x67994143, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e89414c, 33 },
    { 0, 0 },
    { 0x8029414e, 46 },
    { 0xd239414f, 46 },
    { 0x5a94150, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1994153, 83 },
    { 0xebb94154, 98 },
    { 0x79794153, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9394159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1394162, 33 },
    { 0xaff94163, 61 },
    { 0x7994164, 93 },
    { 0x97694165, 61 },
    { 0, 0 },
    { 0xe7894167, 84 },
    { 0xf8494167, 33 },
    { 0x76194169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd9416d, 23 },
    { 0, 0 },
    { 0x9949416f, 43 },
    { 0xdd19416f, 91 },
    { 0, 0 },
    { 0x37294172, 95 },
    { 0x83d94173, 60 },
    { 0x7f394174, 90 },
    { 0x50d94173, 55 },
    { 0x23c94175, 34 },
    { 0, 0 },
    { 0xae994178, 23 },
    { 0, 0 },
    { 0x9af9417a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd049417d, 98 },
    { 0, 0 },
    { 0x2209417f, 87 },
    { 0, 0 },
    { 0x17d94181, 40 },
    { 0, 0 },
    { 0xc3394183, 61 },
    { 0xcab94183, 93 },
    { 0x93f94185, 23 },
    { 0x22a94186, 90 },
    { 0x48594183, 91 },
    { 0x97594188, 61 },
    { 0, 0 },
    { 0xd1b9418a, 90 },
    { 0x9549418b, 23 },
    { 0xd279418c, 33 },
    { 0, 0 },
    { 0x6219418e, 96 },
    { 0xe7a9418f, 95 },
    { 0xfcf94190, 61 },
    { 0xe289418e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0194195, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55594198, 95 },
    { 0, 0 },
    { 0xfe29419a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17941a4, 90 },
    { 0x85941a5, 90 },
    { 0x8ab941a6, 84 },
    { 0x719941a6, 33 },
    { 0x946941a8, 95 },
    { 0x683941a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69941b0, 90 },
    { 0x75f941b1, 23 },
    { 0x842941b0, 75 },
    { 0x515941b3, 87 },
    { 0x5e6941b4, 56 },
    { 0x3e941b4, 90 },
    { 0x55b941b5, 39 },
    { 0xa95941b4, 75 },
    { 0x903941b4, 95 },
    { 0, 0 },
    { 0x22f941ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52941be, 61 },
    { 0, 0 },
    { 0x7c8941c0, 23 },
    { 0x169941c1, 90 },
    { 0x693941c1, 90 },
    { 0x3a6941c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380941c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268941d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3941de, 68 },
    { 0xbb1941de, 90 },
    { 0, 0 },
    { 0xdd0941e1, 98 },
    { 0, 0 },
    { 0x8c4941e3, 87 },
    { 0x4c7941e4, 76 },
    { 0x55c941e4, 23 },
    { 0xdfa941e6, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2941ea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157941f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c941f4, 87 },
    { 0xad2941f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1941f9, 76 },
    { 0xae4941fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a894204, 90 },
    { 0x33894204, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a94208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc249420b, 76 },
    { 0x26f9420c, 95 },
    { 0xeb79420c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3894212, 98 },
    { 0xd9d94213, 23 },
    { 0xccd94213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f794218, 33 },
    { 0x53e94219, 77 },
    { 0, 0 },
    { 0xabf9421b, 90 },
    { 0, 0 },
    { 0x5ed9421d, 90 },
    { 0, 0 },
    { 0x70c9421f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd94222, 91 },
    { 0x6d594222, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f94226, 52 },
    { 0x3be94227, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e9422b, 33 },
    { 0x5b99422c, 46 },
    { 0xd359422c, 76 },
    { 0x2a79422b, 94 },
    { 0x22f9422f, 84 },
    { 0x8b594230, 98 },
    { 0xad59422f, 90 },
    { 0xd9b9422c, 95 },
    { 0x1ab94233, 61 },
    { 0xebd9422b, 98 },
    { 0, 0 },
    { 0x9e494236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd94245, 23 },
    { 0xd6b94246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7209424d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a994250, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee994255, 61 },
    { 0xdb394256, 100 },
    { 0x77394257, 33 },
    { 0x54594258, 95 },
    { 0x2594259, 23 },
    { 0x2959425a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6419425f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8294266, 33 },
    { 0xe4f94267, 90 },
    { 0, 0 },
    { 0x18394269, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf129426d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe94271, 90 },
    { 0x6b394272, 99 },
    { 0, 0 },
    { 0x9e494274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4594278, 44 },
    { 0, 0 },
    { 0xf1c9427a, 61 },
    { 0x5d69427b, 59 },
    { 0x24e9427c, 90 },
    { 0, 0 },
    { 0x58d9427e, 93 },
    { 0xdda9427f, 95 },
    { 0x19694280, 34 },
    { 0x14b9427f, 76 },
    { 0xb6f94282, 77 },
    { 0, 0 },
    { 0x7d794284, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36294288, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb439428b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e94290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e94296, 67 },
    { 0x7a894296, 98 },
    { 0x11a94298, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af9429c, 33 },
    { 0, 0 },
    { 0xa2c9429e, 90 },
    { 0xf89429f, 98 },
    { 0xa90942a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225942a6, 90 },
    { 0x9b1942a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2942ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2942b2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6942b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4942c0, 95 },
    { 0x489942c1, 33 },
    { 0x641942c1, 98 },
    { 0xb07942c3, 95 },
    { 0, 0 },
    { 0xf57942c5, 95 },
    { 0x80c942c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00942c9, 33 },
    { 0, 0 },
    { 0x30942cb, 33 },
    { 0, 0 },
    { 0xebe942cd, 43 },
    { 0, 0 },
    { 0xbea942cf, 100 },
    { 0x574942d0, 78 },
    { 0x3c3942d1, 33 },
    { 0x2c5942d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a942d7, 34 },
    { 0xc40942d8, 33 },
    { 0x868942d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x457942dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149942e1, 34 },
    { 0x37942e2, 80 },
    { 0xbe7942e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x305942e6, 34 },
    { 0, 0 },
    { 0x160942e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56942eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4942ef, 91 },
    { 0x3bd942ef, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8942f3, 38 },
    { 0x4f3942f3, 66 },
    { 0xa4942f3, 23 },
    { 0x187942f6, 84 },
    { 0xa80942f6, 61 },
    { 0x9b0942f8, 33 },
    { 0xccd942f5, 98 },
    { 0x5ae942fa, 23 },
    { 0x193942f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f94304, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x18394307, 85 },
    { 0x5fe94308, 34 },
    { 0, 0 },
    { 0x4fa9430a, 23 },
    { 0x9c9430a, 43 },
    { 0x1689430c, 34 },
    { 0xf949430c, 55 },
    { 0xbaf9430e, 23 },
    { 0, 0 },
    { 0x1e794310, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70394314, 55 },
    { 0xbd094315, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb494318, 78 },
    { 0x62194318, 33 },
    { 0xad894318, 33 },
    { 0x7279431b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa069431f, 98 },
    { 0, 0 },
    { 0xc9494321, 90 },
    { 0x4f94322, 33 },
    { 0x2494323, 61 },
    { 0x2e594323, 91 },
    { 0x74b94325, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62894329, 52 },
    { 0x7f94329, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1979432e, 75 },
    { 0xbf69432f, 90 },
    { 0, 0 },
    { 0xd0394331, 33 },
    { 0x13094332, 34 },
    { 0x2094333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b794339, 33 },
    { 0x1bf9433a, 77 },
    { 0, 0 },
    { 0x7159433c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac94340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5394343, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82894349, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b94354, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9294358, 23 },
    { 0, 0 },
    { 0x4ac9435a, 76 },
    { 0x9f49435a, 46 },
    { 0, 0 },
    { 0x3759435d, 65 },
    { 0x5909435e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e94364, 90 },
    { 0, 0 },
    { 0x67494366, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129436b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x669436e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea94371, 61 },
    { 0xbb894372, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25794377, 71 },
    { 0x9f794378, 34 },
    { 0x7a94377, 61 },
    { 0x40f94377, 46 },
    { 0, 0 },
    { 0xbe99437c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34394382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f194386, 64 },
    { 0xf0194387, 84 },
    { 0, 0 },
    { 0x1c94389, 84 },
    { 0x6009438a, 78 },
    { 0x5d19438b, 33 },
    { 0x26694389, 98 },
    { 0xb219438d, 61 },
    { 0xce29438d, 61 },
    { 0xa189438e, 61 },
    { 0x73f9438c, 92 },
    { 0x4059438c, 33 },
    { 0xdb694392, 76 },
    { 0x37a94393, 23 },
    { 0x14f94392, 88 },
    { 0x5df94395, 76 },
    { 0xfd194393, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8294399, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c19439c, 90 },
    { 0x6079439d, 90 },
    { 0, 0 },
    { 0x51f9439f, 55 },
    { 0x80b943a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f943a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5943a8, 95 },
    { 0, 0 },
    { 0xbe5943aa, 78 },
    { 0xa42943ab, 80 },
    { 0, 0 },
    { 0x363943ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a943b2, 61 },
    { 0xcbf943b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83943be, 66 },
    { 0x9c9943bf, 33 },
    { 0, 0 },
    { 0xc1d943c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8943c7, 90 },
    { 0xa2f943c8, 87 },
    { 0xe1a943c8, 43 },
    { 0xc10943ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4943d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08943d3, 77 },
    { 0xd2f943d4, 55 },
    { 0, 0 },
    { 0x5f943d6, 61 },
    { 0xabb943d7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c943de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x278943e1, 98 },
    { 0, 0 },
    { 0xd0e943e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6943e6, 80 },
    { 0x45d943e7, 81 },
    { 0x8ac943e8, 95 },
    { 0x4ee943e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513943ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370943f3, 90 },
    { 0xba5943f3, 33 },
    { 0xeb3943f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602943fa, 76 },
    { 0x7be943fb, 67 },
    { 0xe99943fc, 23 },
    { 0x9f8943fd, 26 },
    { 0, 0 },
    { 0xb94943ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe479440a, 76 },
    { 0xd399440b, 52 },
    { 0xf29440a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e94414, 33 },
    { 0, 0 },
    { 0x1f694416, 33 },
    { 0xfcb94417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce69441f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f94423, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80994428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6594430, 61 },
    { 0, 0 },
    { 0xd6c94432, 87 },
    { 0, 0 },
    { 0x21f94434, 61 },
    { 0x5d094434, 99 },
    { 0xa6f94436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae9443a, 90 },
    { 0x4a39443b, 33 },
    { 0x3989443c, 78 },
    { 0x5b99443d, 61 },
    { 0, 0 },
    { 0xb709443f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6794449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed19444d, 43 },
    { 0, 0 },
    { 0x1e89444f, 95 },
    { 0x4ae94450, 23 },
    { 0x7df94451, 76 },
    { 0x1fd94452, 61 },
    { 0xa9094453, 33 },
    { 0xa8194453, 93 },
    { 0x28694455, 98 },
    { 0xdda94456, 90 },
    { 0x50d94455, 23 },
    { 0, 0 },
    { 0xb1894459, 61 },
    { 0, 0 },
    { 0x5f09445b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d94460, 34 },
    { 0x8c294461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e94464, 95 },
    { 0xc7794465, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da94468, 90 },
    { 0x8994469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe429446e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe194472, 33 },
    { 0, 0 },
    { 0xea494474, 38 },
    { 0, 0 },
    { 0xd3c94476, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5394479, 95 },
    { 0x23a94479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3689447d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b194482, 90 },
    { 0, 0 },
    { 0x3d494484, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4594487, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c394492, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa49449a, 46 },
    { 0, 0 },
    { 0xaff9449c, 78 },
    { 0x28f9449c, 90 },
    { 0xb689449e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f944a2, 84 },
    { 0x6fa944a3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711944ac, 46 },
    { 0, 0 },
    { 0xc8c944ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee944b4, 90 },
    { 0x419944b4, 53 },
    { 0xc47944b4, 52 },
    { 0x5c2944b7, 65 },
    { 0, 0 },
    { 0xd2d944b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52944c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a944c5, 90 },
    { 0x8b5944c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf944ce, 98 },
    { 0x659944cf, 76 },
    { 0xaf8944d0, 33 },
    { 0, 0 },
    { 0xde7944d2, 90 },
    { 0x6d944d2, 95 },
    { 0x21b944d4, 33 },
    { 0, 0 },
    { 0x776944d6, 93 },
    { 0x452944d7, 78 },
    { 0xa1a944d7, 43 },
    { 0x751944d9, 23 },
    { 0xbd944d9, 98 },
    { 0, 0 },
    { 0x9ff944dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd944eb, 90 },
    { 0xefd944ec, 33 },
    { 0xc47944ed, 95 },
    { 0xb7a944ec, 88 },
    { 0x3c4944ef, 61 },
    { 0xc19944f0, 78 },
    { 0xd16944f1, 90 },
    { 0xfbc944f1, 99 },
    { 0, 0 },
    { 0x3f7944f4, 52 },
    { 0, 0 },
    { 0xfe7944f6, 43 },
    { 0, 0 },
    { 0x887944f8, 87 },
    { 0xdcf944f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x539944fc, 55 },
    { 0x5f5944fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd994503, 88 },
    { 0, 0 },
    { 0x5ca94505, 82 },
    { 0xa3c94506, 61 },
    { 0x2fe94507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b39450e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2394512, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5789451a, 33 },
    { 0, 0 },
    { 0xc909451c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3094520, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e94526, 40 },
    { 0xf1594526, 60 },
    { 0, 0 },
    { 0x4d394529, 90 },
    { 0xb9b9452a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc9452f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b094532, 61 },
    { 0x6894533, 90 },
    { 0x7f094534, 90 },
    { 0xe8894532, 100 },
    { 0x72694536, 33 },
    { 0x50994537, 43 },
    { 0, 0 },
    { 0x18c94539, 66 },
    { 0xd969453a, 55 },
    { 0x859453b, 68 },
    { 0xceb94539, 34 },
    { 0x7749453d, 99 },
    { 0, 0 },
    { 0xbe19453f, 33 },
    { 0, 0 },
    { 0xb2094541, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a94545, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c94548, 90 },
    { 0, 0 },
    { 0x4469454a, 90 },
    { 0, 0 },
    { 0xdc19454c, 90 },
    { 0, 0 },
    { 0xa5a9454e, 75 },
    { 0x1f19454f, 90 },
    { 0xa19454f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b94555, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b94559, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1569455c, 100 },
    { 0, 0 },
    { 0x1419455e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c94563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18094567, 78 },
    { 0x86a94568, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4849456f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d94572, 90 },
    { 0, 0 },
    { 0xa4d94574, 43 },
    { 0x2cf94575, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6894578, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429457d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d994582, 46 },
    { 0x86394583, 66 },
    { 0x90894582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a9458a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc409458d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a94591, 23 },
    { 0x9b694591, 34 },
    { 0x24294591, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb94596, 76 },
    { 0x23694596, 61 },
    { 0x58a94596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd869459c, 98 },
    { 0, 0 },
    { 0x7249459e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0945ae, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309945b2, 90 },
    { 0xe90945b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c945b8, 98 },
    { 0, 0 },
    { 0xb2945ba, 85 },
    { 0, 0 },
    { 0x563945bc, 66 },
    { 0, 0 },
    { 0xa71945be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc945c1, 100 },
    { 0, 0 },
    { 0xac7945c3, 46 },
    { 0, 0 },
    { 0xeb6945c5, 90 },
    { 0xc48945c5, 61 },
    { 0x177945c7, 66 },
    { 0x23e945c6, 23 },
    { 0x85d945c6, 100 },
    { 0xaa1945c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a945cd, 99 },
    { 0x5b3945ce, 61 },
    { 0xd90945cf, 33 },
    { 0x660945ce, 95 },
    { 0, 0 },
    { 0x1c0945d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a945d5, 90 },
    { 0, 0 },
    { 0x2bf945d7, 98 },
    { 0x939945d8, 77 },
    { 0x44945d8, 90 },
    { 0, 0 },
    { 0x9e945db, 43 },
    { 0x21945dc, 23 },
    { 0, 0 },
    { 0x91f945de, 61 },
    { 0xd14945de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32945e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b945e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d945ec, 61 },
    { 0xfe5945ec, 61 },
    { 0x7d8945ee, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b945f4, 67 },
    { 0xaa945f4, 85 },
    { 0x1b945f5, 90 },
    { 0x12b945f7, 78 },
    { 0x1ae945f8, 76 },
    { 0, 0 },
    { 0x683945fa, 34 },
    { 0x15b945fa, 33 },
    { 0xc27945fc, 90 },
    { 0x23e945fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57494604, 98 },
    { 0x39894604, 90 },
    { 0x13c94606, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce494611, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6e94614, 81 },
    { 0xe8794615, 98 },
    { 0xa7a94614, 67 },
    { 0xe9e94616, 43 },
    { 0x3a594618, 76 },
    { 0x5d294619, 33 },
    { 0, 0 },
    { 0x93b9461b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16894624, 40 },
    { 0xca294624, 97 },
    { 0, 0 },
    { 0x8f994627, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xac89462a, 78 },
    { 0xa9b9462a, 76 },
    { 0x15d9462c, 55 },
    { 0xf809462c, 93 },
    { 0, 0 },
    { 0xe179462f, 27 },
    { 0, 0 },
    { 0xbe494631, 33 },
    { 0x32e94631, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f94636, 97 },
    { 0xa5194637, 34 },
    { 0, 0 },
    { 0x7c394639, 90 },
    { 0xb859463a, 90 },
    { 0x63e94639, 33 },
    { 0x34e9463c, 69 },
    { 0xd7b9463a, 83 },
    { 0x689463b, 90 },
    { 0, 0 },
    { 0xab794640, 61 },
    { 0x53d94641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae894647, 76 },
    { 0x1b994647, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0294650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94694656, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c94659, 90 },
    { 0, 0 },
    { 0x55a9465b, 23 },
    { 0, 0 },
    { 0xfbd9465d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x32394660, 76 },
    { 0, 0 },
    { 0x9b994662, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a294666, 66 },
    { 0x85c94667, 33 },
    { 0x3f294668, 90 },
    { 0x94394669, 33 },
    { 0, 0 },
    { 0x5779466b, 75 },
    { 0, 0 },
    { 0x3a29466d, 61 },
    { 0, 0 },
    { 0xcb89466f, 43 },
    { 0xa2194670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d9467a, 98 },
    { 0x19d9467a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf299467e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30094681, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1994689, 98 },
    { 0x93494689, 90 },
    { 0, 0 },
    { 0xe169468c, 95 },
    { 0xb8d9468d, 55 },
    { 0x5ca9468c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe94697, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109469b, 99 },
    { 0x84d9469c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41946a0, 33 },
    { 0xd71946a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee946a6, 76 },
    { 0xf37946a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed946aa, 23 },
    { 0, 0 },
    { 0x4946ac, 90 },
    { 0, 0 },
    { 0xaf946ae, 33 },
    { 0x84d946af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864946b4, 33 },
    { 0x6e0946b5, 95 },
    { 0, 0 },
    { 0xa0b946b7, 95 },
    { 0x373946b8, 34 },
    { 0xb38946b9, 34 },
    { 0xb83946ba, 90 },
    { 0x937946bb, 98 },
    { 0x352946ba, 90 },
    { 0, 0 },
    { 0x396946be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e946c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84946cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779946d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f946d4, 88 },
    { 0x605946d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454946d9, 33 },
    { 0xab0946d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3946e2, 98 },
    { 0x52946e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b946e6, 95 },
    { 0, 0 },
    { 0x22a946e8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7946eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257946f0, 90 },
    { 0xdeb946f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a946f4, 91 },
    { 0x65e946f4, 61 },
    { 0x265946f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd946fb, 76 },
    { 0xfdd946fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa94701, 64 },
    { 0, 0 },
    { 0x97994703, 77 },
    { 0x30a94703, 36 },
    { 0xdc294705, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89470a, 85 },
    { 0x709470b, 33 },
    { 0xfdd9470c, 91 },
    { 0x3809470d, 56 },
    { 0xdaf9470e, 95 },
    { 0xd8c9470d, 90 },
    { 0x2d59470d, 97 },
    { 0xd269470a, 95 },
    { 0x3319470c, 90 },
    { 0xfbb94711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40594716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a9471a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d9471d, 93 },
    { 0, 0 },
    { 0x3849471f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a94728, 90 },
    { 0xd2394729, 36 },
    { 0xe99472a, 55 },
    { 0x5a19472b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb19472f, 76 },
    { 0, 0 },
    { 0xdf994731, 90 },
    { 0xc5394732, 61 },
    { 0, 0 },
    { 0xae794734, 67 },
    { 0xf8b94734, 40 },
    { 0, 0 },
    { 0x6094737, 95 },
    { 0xfbe94738, 76 },
    { 0x55b94739, 61 },
    { 0x8e79473a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf89473f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff194745, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x65194748, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab99474c, 33 },
    { 0, 0 },
    { 0x62c9474e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5494752, 33 },
    { 0xf5194753, 34 },
    { 0, 0 },
    { 0xfe094755, 93 },
    { 0, 0 },
    { 0x7294757, 66 },
    { 0xf7e94758, 78 },
    { 0, 0 },
    { 0xe829475a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38794765, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d9476a, 43 },
    { 0, 0 },
    { 0xa859476c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b29476f, 61 },
    { 0x26b94770, 98 },
    { 0, 0 },
    { 0x60494772, 34 },
    { 0x81e94773, 98 },
    { 0, 0 },
    { 0xb3b94775, 34 },
    { 0x6b294776, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b29477a, 85 },
    { 0, 0 },
    { 0xa309477c, 98 },
    { 0, 0 },
    { 0x89477e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b94781, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb49478c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d94791, 99 },
    { 0xe8b94792, 34 },
    { 0, 0 },
    { 0xa0894794, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c494798, 34 },
    { 0x2cf94798, 61 },
    { 0, 0 },
    { 0x1a9479b, 57 },
    { 0xac49479c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8947a4, 23 },
    { 0xc15947a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44947a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8947ae, 66 },
    { 0x766947af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66947b8, 43 },
    { 0, 0 },
    { 0xc67947ba, 23 },
    { 0xd1c947bb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c947c0, 93 },
    { 0xf02947c1, 90 },
    { 0x6af947c1, 43 },
    { 0, 0 },
    { 0xf04947c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a947c7, 66 },
    { 0x68b947c8, 99 },
    { 0x454947c9, 61 },
    { 0xeb9947ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f947d0, 23 },
    { 0xda2947d1, 46 },
    { 0xd9c947d1, 84 },
    { 0xb8c947d1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda947d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2947d9, 76 },
    { 0, 0 },
    { 0xb98947db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84e947e9, 76 },
    { 0xfba947ea, 98 },
    { 0, 0 },
    { 0x62947ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258947f5, 90 },
    { 0x1a2947f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5947fb, 95 },
    { 0xae0947fc, 61 },
    { 0x9f2947fd, 98 },
    { 0xaa0947fe, 99 },
    { 0, 0 },
    { 0x73894800, 61 },
    { 0, 0 },
    { 0x4394802, 33 },
    { 0xfe594802, 8 },
    { 0x10494803, 29 },
    { 0, 0 },
    { 0xa5694806, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77094809, 84 },
    { 0xf229480a, 78 },
    { 0x8f29480b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2394812, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca794816, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef9481d, 33 },
    { 0x71e9481e, 98 },
    { 0, 0 },
    { 0xbf194820, 90 },
    { 0, 0 },
    { 0xf4194822, 67 },
    { 0xa9294823, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87294827, 98 },
    { 0x7d094828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e9482b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a94830, 76 },
    { 0xc2c94831, 88 },
    { 0x43994832, 76 },
    { 0x19394831, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53e94837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a94842, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f59484a, 33 },
    { 0xf079484b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b9484e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac794851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f194855, 76 },
    { 0, 0 },
    { 0xb2794857, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58794860, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb94863, 52 },
    { 0, 0 },
    { 0x99f94865, 76 },
    { 0xf5f94866, 95 },
    { 0xef894865, 98 },
    { 0xdb294865, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf799486b, 33 },
    { 0x6f59486c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db9486f, 72 },
    { 0, 0 },
    { 0x38f94871, 92 },
    { 0, 0 },
    { 0x56694873, 90 },
    { 0x8b394874, 100 },
    { 0xcd594874, 98 },
    { 0xb0d94876, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f9487d, 80 },
    { 0, 0 },
    { 0xcdb9487f, 100 },
    { 0x20094880, 90 },
    { 0, 0 },
    { 0x76c94882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c94886, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c79488a, 23 },
    { 0x94f9488a, 95 },
    { 0xec59488c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58994890, 92 },
    { 0x4ce94890, 17 },
    { 0, 0 },
    { 0x6a894893, 95 },
    { 0x5594893, 90 },
    { 0, 0 },
    { 0x79494896, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1529489d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd948a0, 90 },
    { 0xcce948a0, 93 },
    { 0x20e948a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a948a5, 93 },
    { 0x86f948a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52948aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6948ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x280948b1, 76 },
    { 0x787948b2, 98 },
    { 0x2a5948b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87948b7, 75 },
    { 0xcfb948b8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb948bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9948be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd948c4, 67 },
    { 0x8c8948c5, 61 },
    { 0, 0 },
    { 0x872948c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200948cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98948d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4948d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x754948d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x646948d9, 33 },
    { 0x195948da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c948df, 64 },
    { 0, 0 },
    { 0xce9948e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132948e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17948ea, 90 },
    { 0x291948eb, 81 },
    { 0, 0 },
    { 0x12c948ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82948f0, 61 },
    { 0, 0 },
    { 0x264948f2, 23 },
    { 0xa12948f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7948f7, 61 },
    { 0x8c3948f7, 23 },
    { 0xd85948f9, 98 },
    { 0x61c948f9, 99 },
    { 0, 0 },
    { 0xaa4948fc, 101 },
    { 0x87c948fd, 23 },
    { 0xd36948fc, 43 },
    { 0xe52948ff, 17 },
    { 0xa7a948ff, 55 },
    { 0x7ae94901, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x75594904, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b394907, 43 },
    { 0xf8e94908, 81 },
    { 0x3eb94909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9490d, 77 },
    { 0xec39490d, 83 },
    { 0xb339490d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc94912, 68 },
    { 0, 0 },
    { 0x4594914, 99 },
    { 0, 0 },
    { 0x2d694916, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa459491a, 33 },
    { 0x1c39491b, 40 },
    { 0xd99491b, 23 },
    { 0x999491d, 87 },
    { 0, 0 },
    { 0xa089491f, 95 },
    { 0x7c194920, 81 },
    { 0, 0 },
    { 0xd9e94922, 75 },
    { 0x4e394922, 33 },
    { 0, 0 },
    { 0xac094925, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf19492d, 98 },
    { 0, 0 },
    { 0xc4d9492f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74394932, 23 },
    { 0xf194933, 61 },
    { 0x8194934, 65 },
    { 0xb2194933, 55 },
    { 0x1bb94935, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84394939, 96 },
    { 0x8ef9493a, 40 },
    { 0x4a49493b, 93 },
    { 0x3299493c, 55 },
    { 0x88c9493c, 99 },
    { 0x70d9493b, 93 },
    { 0xbef9493c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe199494d, 78 },
    { 0, 0 },
    { 0xa0a9494f, 64 },
    { 0x70194950, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c894953, 76 },
    { 0x3e694953, 55 },
    { 0xcc794954, 61 },
    { 0, 0 },
    { 0xc1294957, 76 },
    { 0xc7194958, 90 },
    { 0, 0 },
    { 0xb269495a, 95 },
    { 0xb1a9495a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9994964, 33 },
    { 0, 0 },
    { 0x41894966, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5339496a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d9496e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9694971, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8094975, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82b9497a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc519497f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c94989, 33 },
    { 0xed79498a, 33 },
    { 0xc8994989, 61 },
    { 0x6b69498b, 98 },
    { 0xc99498c, 33 },
    { 0x5929498e, 55 },
    { 0xeb09498e, 82 },
    { 0xddb9498a, 34 },
    { 0xcdf94991, 43 },
    { 0x80594991, 98 },
    { 0x3cd94993, 33 },
    { 0xe9f94994, 90 },
    { 0xa8394994, 55 },
    { 0x45a94994, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d79499c, 90 },
    { 0, 0 },
    { 0xd419499e, 52 },
    { 0x5fa9499f, 33 },
    { 0x831949a0, 33 },
    { 0x497949a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d949a9, 99 },
    { 0x3c949aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28949b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4949b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0949ba, 33 },
    { 0xe23949bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a949be, 93 },
    { 0xe2e949bf, 46 },
    { 0x99a949c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x916949c3, 66 },
    { 0, 0 },
    { 0xb27949c5, 23 },
    { 0xcef949c6, 61 },
    { 0xcb9949c6, 98 },
    { 0x2fb949c8, 85 },
    { 0x7ee949c9, 87 },
    { 0, 0 },
    { 0x5d0949cb, 90 },
    { 0, 0 },
    { 0xcb9949cd, 90 },
    { 0x5c7949ce, 67 },
    { 0, 0 },
    { 0x722949d0, 76 },
    { 0x7a7949d1, 23 },
    { 0, 0 },
    { 0x6f4949d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb949d6, 33 },
    { 0xa45949d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x750949da, 76 },
    { 0x150949da, 46 },
    { 0x50e949da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147949e0, 95 },
    { 0x8fc949e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3949e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x283949ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f949ed, 43 },
    { 0x509949ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b949f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79949fe, 33 },
    { 0xa12949ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9f94a04, 76 },
    { 0xe2794a05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62b94a09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21894a0d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39994a13, 87 },
    { 0xbd194a14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5394a19, 61 },
    { 0x87e94a19, 90 },
    { 0xd3694a1b, 98 },
    { 0xd6894a1c, 90 },
    { 0xf4694a1c, 98 },
    { 0xcd794a1e, 23 },
    { 0xd9f94a1f, 95 },
    { 0x73494a1e, 95 },
    { 0x44d94a1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f94a25, 95 },
    { 0xaa294a25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xef794a29, 33 },
    { 0x1b94a2a, 90 },
    { 0, 0 },
    { 0x3ed94a2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2294a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a94a36, 90 },
    { 0, 0 },
    { 0x8e994a38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae394a3e, 84 },
    { 0xf3694a3f, 95 },
    { 0x8c294a3e, 43 },
    { 0xa7194a3e, 67 },
    { 0x37194a42, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7394a46, 61 },
    { 0x63f94a47, 65 },
    { 0xeb494a48, 95 },
    { 0, 0 },
    { 0x4694a4a, 34 },
    { 0x69594a4b, 78 },
    { 0xeaa94a4c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e94a4f, 90 },
    { 0xd8c94a50, 46 },
    { 0, 0 },
    { 0x1c194a52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e994a57, 90 },
    { 0, 0 },
    { 0x15094a59, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c94a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b794a60, 33 },
    { 0xcc94a61, 91 },
    { 0, 0 },
    { 0x2ab94a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26294a69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c94a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77794a72, 87 },
    { 0, 0 },
    { 0x9794a74, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13694a7b, 55 },
    { 0xbb694a7c, 33 },
    { 0, 0 },
    { 0x11d94a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad94a83, 93 },
    { 0xcf94a84, 33 },
    { 0, 0 },
    { 0xff194a86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63994a8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f94a94, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b494a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5094a9f, 33 },
    { 0, 0 },
    { 0xb9c94aa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65894aa6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6294aa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b94aac, 43 },
    { 0xfb894aad, 76 },
    { 0x27e94aad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b94ab2, 66 },
    { 0x69794ab3, 46 },
    { 0xd4d94ab4, 33 },
    { 0, 0 },
    { 0xe5294ab6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92594aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c94aca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a94acf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f494ad2, 76 },
    { 0x25594ad2, 61 },
    { 0x30094ad4, 67 },
    { 0xed994ad3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7394adb, 33 },
    { 0xb3694adc, 95 },
    { 0x78d94add, 69 },
    { 0xded94ade, 95 },
    { 0x16494ade, 34 },
    { 0x71b94ae0, 54 },
    { 0xdb494ae0, 33 },
    { 0x62894ae2, 46 },
    { 0x20794ae0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41894ae8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c94af1, 34 },
    { 0x37694af2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58594af9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa594b00, 76 },
    { 0xf8594b01, 23 },
    { 0xb1d94b00, 91 },
    { 0x7894b02, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b94b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4594b0f, 33 },
    { 0, 0 },
    { 0xed094b11, 98 },
    { 0, 0 },
    { 0xc6c94b13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af94b16, 90 },
    { 0x23394b17, 23 },
    { 0x86294b18, 93 },
    { 0x31794b19, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df94b1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3594b1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b94b26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f694b2b, 76 },
    { 0xfc294b2b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf594b30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a94b34, 65 },
    { 0x67394b35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9894b41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24894b48, 78 },
    { 0, 0 },
    { 0x76994b4a, 95 },
    { 0x35094b4b, 65 },
    { 0, 0 },
    { 0x38094b4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd94b50, 34 },
    { 0xac494b51, 88 },
    { 0x2a194b50, 34 },
    { 0x4fe94b50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84994b58, 61 },
    { 0, 0 },
    { 0x6c94b5a, 61 },
    { 0xdeb94b5b, 55 },
    { 0xfb694b5b, 90 },
    { 0, 0 },
    { 0xcfc94b5e, 95 },
    { 0xdc694b5f, 76 },
    { 0xf7094b60, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f494b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28994b69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47394b6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41394b75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9594b7a, 33 },
    { 0x14894b7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6894b7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b94b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c794b8c, 90 },
    { 0, 0 },
    { 0xcd394b8e, 40 },
    { 0xb2c94b8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f794b93, 33 },
    { 0, 0 },
    { 0x58794b95, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e094b99, 34 },
    { 0x3e294b9a, 98 },
    { 0xbdf94b9a, 99 },
    { 0x62a94b9c, 23 },
    { 0xad094b99, 90 },
    { 0, 0 },
    { 0x3d594b9f, 91 },
    { 0xbe394ba0, 90 },
    { 0xa4a94ba1, 90 },
    { 0x68394ba1, 55 },
    { 0x8a194ba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f94ba7, 55 },
    { 0x3ac94ba7, 33 },
    { 0x1ae94ba9, 23 },
    { 0xbb994baa, 23 },
    { 0x15b94baa, 87 },
    { 0, 0 },
    { 0x39294bad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x81394bb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e94bb5, 55 },
    { 0xe5394bb6, 61 },
    { 0, 0 },
    { 0xe4494bb8, 34 },
    { 0x3cd94bb9, 23 },
    { 0x7c694bb8, 93 },
    { 0, 0 },
    { 0x53e94bbc, 61 },
    { 0x88794bbd, 90 },
    { 0xfcf94bbe, 23 },
    { 0, 0 },
    { 0xa6594bc0, 90 },
    { 0xd8394bc0, 87 },
    { 0x6394bc0, 61 },
    { 0xb9c94bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e494bc6, 85 },
    { 0, 0 },
    { 0xc8e94bc8, 46 },
    { 0xfe394bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa994bcc, 75 },
    { 0x31394bcc, 90 },
    { 0xcc94bce, 33 },
    { 0x9f394bcf, 81 },
    { 0x88194bd0, 55 },
    { 0xf5394bce, 98 },
    { 0x54b94bcc, 55 },
    { 0x2c994bd3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba94bd7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf94bda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d794bde, 78 },
    { 0xd3794bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c94be5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0294be8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c494bf1, 33 },
    { 0xa4c94bf2, 83 },
    { 0xcca94bf1, 34 },
    { 0, 0 },
    { 0xd0694bf5, 98 },
    { 0, 0 },
    { 0x1b294bf7, 33 },
    { 0x85c94bf7, 95 },
    { 0x61b94bf9, 90 },
    { 0x7c94bfa, 61 },
    { 0xc9294bfb, 90 },
    { 0x31c94bfc, 99 },
    { 0x80194bfc, 95 },
    { 0xa6494bfe, 23 },
    { 0xd7994bfd, 61 },
    { 0xe9a94bff, 52 },
    { 0xd0c94bfe, 67 },
    { 0xd8294c02, 91 },
    { 0x7cc94c02, 61 },
    { 0xa1294c03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0294c07, 17 },
    { 0, 0 },
    { 0xbd094c09, 98 },
    { 0, 0 },
    { 0x3f094c0b, 90 },
    { 0x1cc94c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f894c10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dd94c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf94c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b894c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99894c21, 76 },
    { 0xc0994c22, 95 },
    { 0x75f94c22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60794c27, 98 },
    { 0, 0 },
    { 0x6b194c29, 84 },
    { 0xd7594c2a, 85 },
    { 0x1c894c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa94c30, 98 },
    { 0xfd694c31, 33 },
    { 0x6de94c32, 93 },
    { 0xa6394c31, 90 },
    { 0, 0 },
    { 0xd9494c35, 33 },
    { 0xcd194c36, 90 },
    { 0x92a94c36, 90 },
    { 0, 0 },
    { 0x5b94c39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b94c3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57194c45, 61 },
    { 0xd5e94c45, 61 },
    { 0x1e094c47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5694c55, 90 },
    { 0xe5194c55, 95 },
    { 0, 0 },
    { 0xc1794c58, 61 },
    { 0x83d94c58, 55 },
    { 0, 0 },
    { 0x98994c5b, 34 },
    { 0xa1094c5c, 33 },
    { 0x7b494c5d, 33 },
    { 0xd194c5e, 84 },
    { 0, 0 },
    { 0x84194c60, 55 },
    { 0x25f94c61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63194c64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd94c6b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de94c6e, 65 },
    { 0x36494c6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33794c73, 99 },
    { 0x8be94c73, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb394c77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c194c7c, 96 },
    { 0x4e694c7c, 85 },
    { 0xd8294c7e, 99 },
    { 0xea994c7e, 93 },
    { 0x8c694c7d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff594c8a, 61 },
    { 0xaca94c8b, 91 },
    { 0, 0 },
    { 0xf7694c8d, 33 },
    { 0x7e894c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f94c93, 90 },
    { 0x73394c93, 34 },
    { 0, 0 },
    { 0xf8994c96, 95 },
    { 0, 0 },
    { 0x2a294c98, 90 },
    { 0x61294c99, 34 },
    { 0x3794c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead94ca8, 23 },
    { 0x57b94ca9, 43 },
    { 0x8e494caa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd194cb0, 67 },
    { 0, 0 },
    { 0x7e394cb2, 90 },
    { 0xfda94cb3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a94cb6, 98 },
    { 0, 0 },
    { 0x2b94cb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13494cbf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c94cc3, 34 },
    { 0x8e94cc4, 90 },
    { 0, 0 },
    { 0x56b94cc6, 98 },
    { 0xe0194cc7, 55 },
    { 0, 0 },
    { 0xc1e94cc9, 100 },
    { 0x52094cca, 91 },
    { 0x62294ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47994cd0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e94cd6, 61 },
    { 0x6cb94cd6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x13394cda, 90 },
    { 0x89c94cdb, 90 },
    { 0, 0 },
    { 0x36194cdd, 97 },
    { 0x19194cde, 91 },
    { 0x43e94cde, 34 },
    { 0x96b94ce0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb94ce3, 98 },
    { 0x5db94ce3, 97 },
    { 0x47694ce5, 23 },
    { 0xcff94ce6, 61 },
    { 0, 0 },
    { 0xbc894ce8, 61 },
    { 0, 0 },
    { 0xf3e94cea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa94cf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2594cf7, 40 },
    { 0x6a194cf8, 33 },
    { 0x8d694cf7, 61 },
    { 0, 0 },
    { 0x2d294cfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f94cfe, 34 },
    { 0, 0 },
    { 0x39794d00, 95 },
    { 0x53994d00, 55 },
    { 0, 0 },
    { 0xa1e94d03, 95 },
    { 0x1b694d04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d94d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb94d10, 23 },
    { 0x63894d10, 83 },
    { 0x88394d11, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f94d16, 90 },
    { 0xf594d17, 61 },
    { 0, 0 },
    { 0xa1194d19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1294d1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a094d23, 61 },
    { 0, 0 },
    { 0x50294d25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a094d28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7394d30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb994d33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f94d36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24494d3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2794d3e, 33 },
    { 0, 0 },
    { 0xdb294d40, 77 },
    { 0xbf194d41, 68 },
    { 0x8d094d42, 90 },
    { 0x8bf94d42, 23 },
    { 0xaac94d44, 84 },
    { 0xbcd94d45, 90 },
    { 0x10194d45, 87 },
    { 0x1ea94d47, 98 },
    { 0x84b94d45, 23 },
    { 0, 0 },
    { 0x32f94d4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b94d56, 95 },
    { 0x27b94d57, 90 },
    { 0x18a94d58, 23 },
    { 0x77794d59, 61 },
    { 0, 0 },
    { 0xa8694d5b, 61 },
    { 0xd6f94d5c, 100 },
    { 0x50294d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf94d64, 78 },
    { 0x8dc94d65, 46 },
    { 0x48294d66, 33 },
    { 0x9a594d64, 46 },
    { 0x4e94d68, 76 },
    { 0xf794d67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb594d6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf294d73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3594d77, 61 },
    { 0x57794d77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf94d7b, 50 },
    { 0x9c94d7c, 98 },
    { 0, 0 },
    { 0xbc94d7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f94d83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89794d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad94d8d, 90 },
    { 0x4e694d8e, 90 },
    { 0x60e94d8f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6894d92, 55 },
    { 0xd4394d92, 98 },
    { 0x84394d94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56894d99, 87 },
    { 0, 0 },
    { 0x10494d9b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f294d9e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f694da1, 23 },
    { 0x46f94da2, 81 },
    { 0, 0 },
    { 0xb7094da4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91694da8, 23 },
    { 0x6b894da9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b094db1, 90 },
    { 0x17294db2, 23 },
    { 0, 0 },
    { 0x26794db4, 52 },
    { 0x6c094db5, 46 },
    { 0x97c94db5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0794dbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b94dc1, 90 },
    { 0xefe94dc2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef94dc6, 77 },
    { 0x56c94dc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34794dcc, 91 },
    { 0, 0 },
    { 0xe5294dce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27494dd5, 23 },
    { 0xff094dd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf994ddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94294de3, 33 },
    { 0x66994de4, 67 },
    { 0x1d894de3, 23 },
    { 0xc6594de6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d94de9, 34 },
    { 0, 0 },
    { 0xbfa94deb, 46 },
    { 0, 0 },
    { 0xe6c94ded, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92594df0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b94df3, 90 },
    { 0, 0 },
    { 0x5d194df5, 90 },
    { 0xfe594df6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad94dfa, 76 },
    { 0xd1f94dfb, 88 },
    { 0, 0 },
    { 0x5b094dfd, 76 },
    { 0x6ae94dfe, 76 },
    { 0x65094dff, 87 },
    { 0, 0 },
    { 0xc7b94e01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2494e06, 98 },
    { 0xd7494e06, 98 },
    { 0, 0 },
    { 0x1ed94e09, 55 },
    { 0, 0 },
    { 0xfc694e0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c794e0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad94e12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2894e15, 23 },
    { 0x93694e15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa94e1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa994e22, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a94e25, 33 },
    { 0, 0 },
    { 0x2d994e27, 99 },
    { 0, 0 },
    { 0x2a94e29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb394e2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x60694e2f, 43 },
    { 0x8c194e30, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b394e34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f94e38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8094e3b, 78 },
    { 0xf9994e3b, 55 },
    { 0x29b94e3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51394e4a, 90 },
    { 0, 0 },
    { 0x32e94e4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe94e4f, 90 },
    { 0, 0 },
    { 0xc4294e51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18794e55, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c094e5a, 84 },
    { 0xbde94e5a, 23 },
    { 0, 0 },
    { 0x9cf94e5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4694e66, 90 },
    { 0, 0 },
    { 0x2ab94e68, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba94e6b, 33 },
    { 0x65694e6c, 98 },
    { 0, 0 },
    { 0x6a294e6e, 81 },
    { 0, 0 },
    { 0x89894e70, 95 },
    { 0x3ca94e71, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90694e7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94894e83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa94e8a, 33 },
    { 0, 0 },
    { 0xd7c94e8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c94e90, 93 },
    { 0xb8494e91, 34 },
    { 0, 0 },
    { 0xcef94e93, 77 },
    { 0x4ea94e93, 61 },
    { 0x27594e94, 80 },
    { 0xa9e94e93, 66 },
    { 0, 0 },
    { 0xff394e98, 33 },
    { 0, 0 },
    { 0x7bc94e9a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8194e9e, 61 },
    { 0xa1b94e9f, 76 },
    { 0x62294ea0, 75 },
    { 0xe4194ea0, 34 },
    { 0x8f994ea0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7194ea7, 65 },
    { 0, 0 },
    { 0xc8394ea9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b94eaf, 33 },
    { 0x4494eb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff894eb7, 55 },
    { 0xf0794eb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c494ebc, 76 },
    { 0x8a394ebd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb94ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1294ec5, 40 },
    { 0x67994ec6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb794ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4994ecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5094ed4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0394ed7, 51 },
    { 0xd5594ed7, 56 },
    { 0x69a94ed9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f194edc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55c94ee2, 34 },
    { 0x22c94ee3, 88 },
    { 0xda94ee2, 46 },
    { 0x25494ee3, 36 },
    { 0x8d094ee6, 61 },
    { 0x73694ee7, 76 },
    { 0x48d94ee8, 88 },
    { 0x7be94ee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e94eec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e94eef, 90 },
    { 0, 0 },
    { 0x2c094ef1, 76 },
    { 0x87794ef2, 84 },
    { 0, 0 },
    { 0x82c94ef4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e694efe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f94f06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x23194f09, 97 },
    { 0xe3094f09, 80 },
    { 0xa2694f0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc094f0e, 52 },
    { 0, 0 },
    { 0xf9694f10, 23 },
    { 0xe5d94f10, 34 },
    { 0x74294f12, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af94f17, 76 },
    { 0x74694f18, 66 },
    { 0x97494f19, 61 },
    { 0x45e94f1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a94f1e, 34 },
    { 0, 0 },
    { 0xbbc94f20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19094f29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3894f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a94f3b, 84 },
    { 0, 0 },
    { 0x35f94f3d, 95 },
    { 0x17b94f3d, 93 },
    { 0xff594f3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb894f44, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6894f48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34394f56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3094f5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0594f5f, 87 },
    { 0xe9894f5f, 98 },
    { 0, 0 },
    { 0x97894f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7594f6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6494f6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde94f73, 33 },
    { 0xe9194f73, 76 },
    { 0xba394f75, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f94f78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1594f7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6694f7e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f94f82, 98 },
    { 0x54394f82, 61 },
    { 0x62b94f84, 23 },
    { 0xf1394f85, 34 },
    { 0xe6a94f84, 90 },
    { 0x89194f85, 76 },
    { 0, 0 },
    { 0x9a694f89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d94f8f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x85194f92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf94f9a, 61 },
    { 0xc9694f9b, 46 },
    { 0xb9b94f9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77894f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c94fa2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82994fad, 90 },
    { 0xd6b94fae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae94fb9, 34 },
    { 0, 0 },
    { 0xe3794fbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f94fc0, 98 },
    { 0x4cb94fc1, 95 },
    { 0, 0 },
    { 0x88a94fc3, 100 },
    { 0, 0 },
    { 0xb9794fc5, 90 },
    { 0x7b594fc6, 73 },
    { 0x9d794fc7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde94fca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e194fcf, 61 },
    { 0x89994fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f294fd3, 93 },
    { 0xd8e94fd3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e94fd7, 61 },
    { 0, 0 },
    { 0xebc94fd9, 87 },
    { 0, 0 },
    { 0x52b94fdb, 76 },
    { 0xdc594fdc, 33 },
    { 0x8c494fdd, 33 },
    { 0xf7a94fde, 52 },
    { 0xaa794fdd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2394fe2, 98 },
    { 0xead94fe3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f994fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde494fef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f494ff7, 46 },
    { 0x2f194ff8, 59 },
    { 0xacc94ff7, 33 },
    { 0xc8394ffa, 90 },
    { 0x19e94ffb, 99 },
    { 0, 0 },
    { 0x73a94ffd, 61 },
    { 0x52294ffe, 61 },
    { 0xe5194fff, 33 },
    { 0x74995000, 90 },
    { 0x6ac95001, 90 },
    { 0x32694ffd, 95 },
    { 0x11b94ffd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d95007, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc95010, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd9501c, 61 },
    { 0xdbb9501d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7395021, 55 },
    { 0x56195022, 55 },
    { 0xad795023, 66 },
    { 0xa2795021, 34 },
    { 0x68395025, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c095028, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb849502d, 61 },
    { 0x89502e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa95032, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7995036, 91 },
    { 0xa1595037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5395042, 23 },
    { 0x58395043, 33 },
    { 0xb4895043, 98 },
    { 0x82795043, 61 },
    { 0, 0 },
    { 0x95195047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec9504a, 23 },
    { 0, 0 },
    { 0x3709504c, 33 },
    { 0xee69504c, 95 },
    { 0, 0 },
    { 0x5609504f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80e95055, 23 },
    { 0x32595056, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f9505c, 34 },
    { 0x8cb9505c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52895067, 66 },
    { 0x85695068, 76 },
    { 0, 0 },
    { 0xf909506a, 23 },
    { 0xb7b9506a, 65 },
    { 0x2139506c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d95072, 95 },
    { 0, 0 },
    { 0xc0b95074, 98 },
    { 0x8195075, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d95079, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5679507c, 90 },
    { 0x86a9507d, 33 },
    { 0xb159507e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xae695081, 95 },
    { 0xa4d95082, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3a95085, 95 },
    { 0x7b895085, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09508a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde9508f, 76 },
    { 0x5389508f, 95 },
    { 0xa2f95091, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df95098, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9509b, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161950a0, 61 },
    { 0x68f950a1, 85 },
    { 0x616950a1, 98 },
    { 0x4e950a2, 33 },
    { 0x40f950a4, 98 },
    { 0x41c950a1, 34 },
    { 0x95e950a6, 33 },
    { 0x32950a4, 98 },
    { 0x28950a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7950ab, 33 },
    { 0xdd950ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b950b1, 98 },
    { 0xafe950b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82950b5, 33 },
    { 0xf84950b5, 34 },
    { 0, 0 },
    { 0x9f950b8, 43 },
    { 0xc30950b8, 48 },
    { 0, 0 },
    { 0x84c950bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7950c3, 80 },
    { 0, 0 },
    { 0x22e950c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8950cd, 95 },
    { 0x11950ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1950d1, 43 },
    { 0x75950d2, 33 },
    { 0xb50950d3, 95 },
    { 0x201950d1, 75 },
    { 0x3b8950d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa950da, 98 },
    { 0xab5950db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c950df, 99 },
    { 0, 0 },
    { 0x15b950e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea950e5, 34 },
    { 0xeed950e5, 61 },
    { 0x22a950e7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b950ea, 61 },
    { 0x9f950ea, 66 },
    { 0x822950ec, 46 },
    { 0x20d950ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a950f3, 65 },
    { 0x37d950f3, 90 },
    { 0, 0 },
    { 0xd15950f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c950fb, 4 },
    { 0x951950fc, 8 },
    { 0xa5e950fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f95101, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7295104, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4195107, 90 },
    { 0x92595108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4495113, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda395117, 55 },
    { 0xdab95118, 95 },
    { 0, 0 },
    { 0x1d69511a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd95120, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c95123, 90 },
    { 0x5f795124, 78 },
    { 0x7ef95123, 61 },
    { 0xe4595124, 33 },
    { 0x9d395125, 43 },
    { 0xa9495125, 90 },
    { 0xed895129, 98 },
    { 0x5169512a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc9512d, 46 },
    { 0xac29512d, 55 },
    { 0x4569512e, 33 },
    { 0xf3d95130, 81 },
    { 0x5e69512d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e795134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d595138, 76 },
    { 0x99995139, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b9513d, 76 },
    { 0x9139513e, 61 },
    { 0xabe9513f, 90 },
    { 0x1a695140, 17 },
    { 0x73f9513f, 87 },
    { 0x2869513e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b995148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a95151, 61 },
    { 0xa5d95151, 61 },
    { 0x7ec95153, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef95156, 61 },
    { 0xff395156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809515c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34495163, 61 },
    { 0x42e95163, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1395167, 33 },
    { 0, 0 },
    { 0x66095169, 77 },
    { 0, 0 },
    { 0x42a9516b, 90 },
    { 0, 0 },
    { 0x4379516d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a95172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf695177, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb9517c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1995180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x13495183, 77 },
    { 0, 0 },
    { 0x65c95185, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2789518e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d295191, 55 },
    { 0x2c495192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99495195, 98 },
    { 0, 0 },
    { 0xd3195197, 46 },
    { 0xf5995198, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299519d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7951a7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x300951aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3951ad, 33 },
    { 0x234951ae, 90 },
    { 0xb9d951af, 95 },
    { 0xb5f951b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29951b3, 23 },
    { 0, 0 },
    { 0xc7951b5, 93 },
    { 0xb46951b6, 33 },
    { 0, 0 },
    { 0xb69951b8, 84 },
    { 0x1c4951b8, 33 },
    { 0xd88951b8, 90 },
    { 0x141951bb, 33 },
    { 0x565951bc, 33 },
    { 0x480951bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7951c6, 91 },
    { 0x1d9951c7, 33 },
    { 0x7d1951c7, 55 },
    { 0, 0 },
    { 0x9d5951ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d951ce, 90 },
    { 0xf4c951cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f951d2, 61 },
    { 0xbb7951d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1951d6, 98 },
    { 0x309951d6, 43 },
    { 0x84e951d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387951dd, 55 },
    { 0x355951dd, 99 },
    { 0x4c951df, 84 },
    { 0xdb3951dd, 43 },
    { 0x2ed951de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d951e6, 90 },
    { 0xfdc951e7, 76 },
    { 0x291951e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736951f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d951f9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635951fd, 95 },
    { 0xe16951fd, 95 },
    { 0x3d8951ff, 23 },
    { 0x34795200, 61 },
    { 0x18b95201, 46 },
    { 0x9c595201, 55 },
    { 0xd2a95201, 43 },
    { 0xdf95202, 95 },
    { 0, 0 },
    { 0xbb795206, 55 },
    { 0x43b95207, 93 },
    { 0x9e495207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d9520b, 98 },
    { 0xdbc9520c, 33 },
    { 0, 0 },
    { 0xbaa9520e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85495216, 98 },
    { 0, 0 },
    { 0x58795218, 19 },
    { 0, 0 },
    { 0x2b59521a, 87 },
    { 0x4cb9521a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9521e, 43 },
    { 0, 0 },
    { 0xc7495220, 34 },
    { 0x55b95221, 90 },
    { 0x2395220, 61 },
    { 0x4895221, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xae495226, 78 },
    { 0, 0 },
    { 0x8c995228, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8429522b, 93 },
    { 0xc969522c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39522f, 33 },
    { 0, 0 },
    { 0x4f295231, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72795236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0095239, 65 },
    { 0xe5e9523a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa509523e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60195244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9524d, 33 },
    { 0xee29524e, 90 },
    { 0x9939524d, 95 },
    { 0, 0 },
    { 0x99995251, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa95254, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf429525e, 61 },
    { 0xe599525f, 88 },
    { 0xeca95260, 43 },
    { 0xf495261, 61 },
    { 0, 0 },
    { 0x7ca95263, 61 },
    { 0xf6295264, 93 },
    { 0xd5995263, 52 },
    { 0, 0 },
    { 0x1c95267, 33 },
    { 0x5fe95268, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c9526c, 34 },
    { 0xcac9526d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2695273, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e95276, 23 },
    { 0x13e95277, 65 },
    { 0x42d95277, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9029527b, 90 },
    { 0, 0 },
    { 0x9869527d, 52 },
    { 0xb769527d, 87 },
    { 0x5b9527d, 87 },
    { 0xcaa95280, 46 },
    { 0x4d195280, 55 },
    { 0x59d95282, 81 },
    { 0xd0195283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3195286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf9528b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9528f, 43 },
    { 0xecf9528f, 90 },
    { 0, 0 },
    { 0x45c95292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21995299, 66 },
    { 0x5f99529a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c99529d, 61 },
    { 0, 0 },
    { 0x7d59529f, 61 },
    { 0, 0 },
    { 0xba6952a1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa952a7, 43 },
    { 0x1b7952a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1e952ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628952b5, 55 },
    { 0, 0 },
    { 0xd6952b7, 23 },
    { 0xd51952b8, 23 },
    { 0x8d0952b9, 98 },
    { 0, 0 },
    { 0x4c952bb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0952be, 98 },
    { 0x17e952bf, 65 },
    { 0x8be952bf, 99 },
    { 0x82f952c0, 84 },
    { 0xf22952c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a952ca, 76 },
    { 0, 0 },
    { 0x268952cc, 90 },
    { 0x97d952cc, 61 },
    { 0xba1952cc, 55 },
    { 0xae952cf, 34 },
    { 0x294952d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc952d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524952dc, 23 },
    { 0x4b952dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196952e1, 90 },
    { 0x3f2952e2, 76 },
    { 0x754952e2, 55 },
    { 0x83c952e4, 43 },
    { 0x74a952e5, 87 },
    { 0x11b952e4, 23 },
    { 0x158952e7, 91 },
    { 0x4b9952e8, 43 },
    { 0x23f952e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223952f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb952f4, 52 },
    { 0xee6952f5, 40 },
    { 0x564952f6, 33 },
    { 0x78c952f7, 88 },
    { 0xcbe952f4, 90 },
    { 0x66f952f9, 34 },
    { 0x5c5952f7, 100 },
    { 0x89b952fb, 84 },
    { 0x973952fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b95301, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4095305, 95 },
    { 0, 0 },
    { 0xc9095307, 98 },
    { 0x55795308, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d95313, 61 },
    { 0xee695313, 61 },
    { 0, 0 },
    { 0x50595316, 17 },
    { 0xc0895317, 90 },
    { 0, 0 },
    { 0xb7c95319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b9531d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c9532e, 53 },
    { 0, 0 },
    { 0x86995330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb95335, 23 },
    { 0x66495336, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df9533a, 33 },
    { 0, 0 },
    { 0x5839533c, 23 },
    { 0, 0 },
    { 0x8449533e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4695344, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64095348, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d89534b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e29534e, 34 },
    { 0x33d9534f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe95354, 33 },
    { 0x49995355, 76 },
    { 0x41195356, 33 },
    { 0x52a95357, 91 },
    { 0xd3c95356, 19 },
    { 0x82b95354, 84 },
    { 0xd219535a, 100 },
    { 0xea99535b, 61 },
    { 0xc1d9535c, 75 },
    { 0xded9535a, 61 },
    { 0x80a95354, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b95363, 55 },
    { 0x61395364, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc795368, 93 },
    { 0x45d95369, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f29536e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4495375, 76 },
    { 0, 0 },
    { 0x64295377, 95 },
    { 0x4eb95377, 90 },
    { 0xec995379, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e95385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d9538a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c9538f, 87 },
    { 0xfda95390, 33 },
    { 0x4a895390, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23595394, 23 },
    { 0x1c695395, 61 },
    { 0xcee95396, 61 },
    { 0x86c95396, 61 },
    { 0, 0 },
    { 0x15195399, 76 },
    { 0x4359539a, 85 },
    { 0x8f09539b, 98 },
    { 0, 0 },
    { 0x6479539d, 84 },
    { 0x20f9539e, 33 },
    { 0x8469539f, 34 },
    { 0x1a39539d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64953a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa953a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405953ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5953af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31953b3, 98 },
    { 0, 0 },
    { 0xb4a953b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53953b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f953bd, 61 },
    { 0, 0 },
    { 0x92953bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935953c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381953cb, 61 },
    { 0x5b953cb, 90 },
    { 0xe21953cd, 33 },
    { 0x42b953ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286953d4, 95 },
    { 0, 0 },
    { 0x941953d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397953dd, 90 },
    { 0x72e953de, 84 },
    { 0, 0 },
    { 0x7b6953e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15953e6, 33 },
    { 0x5d4953e7, 33 },
    { 0, 0 },
    { 0x881953e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f953ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x406953f0, 98 },
    { 0, 0 },
    { 0xe16953f2, 98 },
    { 0xfd8953f3, 22 },
    { 0x1f1953f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0953fc, 23 },
    { 0x53a953fd, 56 },
    { 0, 0 },
    { 0xdc3953ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc595403, 61 },
    { 0, 0 },
    { 0x15395405, 98 },
    { 0x8e295406, 43 },
    { 0xa6295406, 76 },
    { 0x43295407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0895412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a195415, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86595418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7609541c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94495421, 33 },
    { 0xc7f95422, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3095425, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x97195428, 61 },
    { 0x2d895429, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829542e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b95433, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef95437, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad09543c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd095447, 90 },
    { 0, 0 },
    { 0x92c95449, 61 },
    { 0x76c9544a, 17 },
    { 0x5c49544b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3969544f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17695454, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0595458, 33 },
    { 0, 0 },
    { 0x2189545a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3895464, 61 },
    { 0x20695465, 34 },
    { 0xf2795464, 81 },
    { 0, 0 },
    { 0xbfe95468, 98 },
    { 0xb6a95468, 61 },
    { 0, 0 },
    { 0x469546b, 98 },
    { 0, 0 },
    { 0x89a9546d, 43 },
    { 0x5259546e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d695471, 61 },
    { 0x1dc95472, 76 },
    { 0xb6f95473, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e695477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb9547b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a29547f, 84 },
    { 0, 0 },
    { 0xff595481, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba95488, 23 },
    { 0, 0 },
    { 0xdda9548a, 65 },
    { 0xdef9548b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9548e, 33 },
    { 0, 0 },
    { 0xb8795490, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b9549a, 81 },
    { 0xd539549a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6349549e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1954a2, 33 },
    { 0, 0 },
    { 0x146954a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e954a7, 81 },
    { 0x277954a8, 84 },
    { 0x988954a9, 46 },
    { 0xf83954aa, 61 },
    { 0x821954a8, 93 },
    { 0x623954a9, 61 },
    { 0, 0 },
    { 0x5bb954ae, 55 },
    { 0xee8954af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x711954b2, 55 },
    { 0x2bc954b2, 98 },
    { 0x25b954b3, 75 },
    { 0, 0 },
    { 0x5a954b6, 100 },
    { 0xf88954b6, 55 },
    { 0, 0 },
    { 0xbe5954b9, 34 },
    { 0x7b5954b9, 98 },
    { 0xbca954bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b954be, 61 },
    { 0x7ee954be, 34 },
    { 0x33b954c0, 75 },
    { 0x7e1954c1, 23 },
    { 0xd8f954c2, 55 },
    { 0x5c9954c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf954c9, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0954cc, 33 },
    { 0x714954cc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b954d0, 100 },
    { 0x83954d1, 97 },
    { 0xe8e954d2, 65 },
    { 0, 0 },
    { 0x2ce954d4, 43 },
    { 0x9f8954d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e954da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc954de, 23 },
    { 0xdc5954de, 75 },
    { 0x730954de, 55 },
    { 0xabd954e1, 88 },
    { 0x14b954de, 66 },
    { 0, 0 },
    { 0x93b954e4, 67 },
    { 0, 0 },
    { 0x827954e6, 43 },
    { 0xe02954e7, 23 },
    { 0xacf954e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36954eb, 61 },
    { 0xd0d954ec, 98 },
    { 0, 0 },
    { 0x4e4954ee, 61 },
    { 0x280954ee, 93 },
    { 0x757954f0, 66 },
    { 0x2b7954f0, 34 },
    { 0, 0 },
    { 0x508954f3, 52 },
    { 0x23b954f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5954f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9954ff, 95 },
    { 0, 0 },
    { 0xc2b95501, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b95506, 33 },
    { 0xaf995507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb209550a, 46 },
    { 0x24f9550b, 98 },
    { 0x6509550c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9550f, 33 },
    { 0x7919550f, 78 },
    { 0x73895511, 61 },
    { 0xc4195512, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63795516, 34 },
    { 0, 0 },
    { 0xb0c95518, 95 },
    { 0x52695519, 78 },
    { 0, 0 },
    { 0x7759551b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b9551f, 48 },
    { 0x16395520, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed95523, 76 },
    { 0, 0 },
    { 0xa3995525, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed95528, 55 },
    { 0, 0 },
    { 0xb389552a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3895532, 55 },
    { 0xc8295532, 65 },
    { 0, 0 },
    { 0x99395535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda59553a, 33 },
    { 0xa6b9553a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d95540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90495544, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2395548, 71 },
    { 0x67895549, 33 },
    { 0, 0 },
    { 0x87d9554b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33b95552, 95 },
    { 0x5f395552, 34 },
    { 0x1cb95554, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2495559, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26995560, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d95564, 34 },
    { 0, 0 },
    { 0x53995566, 23 },
    { 0, 0 },
    { 0x5e295568, 33 },
    { 0x96695569, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8f9556d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52995570, 23 },
    { 0x8c095571, 95 },
    { 0xfaf95572, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16195577, 17 },
    { 0x91995578, 98 },
    { 0, 0 },
    { 0x73d9557a, 81 },
    { 0x3da9557a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe9557f, 66 },
    { 0, 0 },
    { 0x50d95581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88095584, 33 },
    { 0, 0 },
    { 0x2f395586, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be9558e, 78 },
    { 0x91b9558e, 91 },
    { 0x6859558e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf59559b, 81 },
    { 0x61b9559b, 78 },
    { 0x8139559c, 90 },
    { 0, 0 },
    { 0x4b99559f, 34 },
    { 0, 0 },
    { 0xafe955a1, 61 },
    { 0x8e6955a2, 90 },
    { 0x3fb955a2, 55 },
    { 0x44f955a1, 98 },
    { 0, 0 },
    { 0x8a1955a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b955ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a955af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6955b3, 33 },
    { 0, 0 },
    { 0xed3955b5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5955b8, 90 },
    { 0x496955b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898955bf, 91 },
    { 0, 0 },
    { 0xf31955c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd955c5, 33 },
    { 0xaaf955c6, 23 },
    { 0xf86955c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa955d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead955de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7955e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06955e4, 52 },
    { 0xb4f955e4, 93 },
    { 0, 0 },
    { 0x80d955e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c955ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7955ed, 87 },
    { 0x132955ed, 61 },
    { 0xfa7955ed, 23 },
    { 0x847955f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37955f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39955f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39955fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a39560d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a95614, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f995617, 95 },
    { 0, 0 },
    { 0x69795619, 33 },
    { 0x4679561a, 91 },
    { 0x41f9561a, 23 },
    { 0xd0295619, 61 },
    { 0x7319561d, 23 },
    { 0x4a49561e, 33 },
    { 0xc389561f, 55 },
    { 0, 0 },
    { 0x5c95621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c95629, 90 },
    { 0xd679562a, 95 },
    { 0x30d9562b, 85 },
    { 0x56c95629, 66 },
    { 0, 0 },
    { 0x5029562e, 100 },
    { 0x2109562f, 23 },
    { 0xff89562f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56195638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc819563d, 87 },
    { 0, 0 },
    { 0x18c9563f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81695643, 91 },
    { 0, 0 },
    { 0x51a95645, 55 },
    { 0, 0 },
    { 0xf095647, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc9564b, 99 },
    { 0xa2d9564c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4295651, 85 },
    { 0xec295652, 76 },
    { 0xd9995653, 90 },
    { 0, 0 },
    { 0x7a595655, 61 },
    { 0, 0 },
    { 0xcb195657, 93 },
    { 0x12495657, 61 },
    { 0xa5795659, 90 },
    { 0, 0 },
    { 0x3779565b, 33 },
    { 0x2149565c, 38 },
    { 0x1269565b, 61 },
    { 0x4619565e, 61 },
    { 0, 0 },
    { 0x81395660, 40 },
    { 0xd795660, 33 },
    { 0x5fe95662, 67 },
    { 0x5f695661, 90 },
    { 0x52e95664, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd095667, 52 },
    { 0, 0 },
    { 0x88995669, 23 },
    { 0x1289566a, 96 },
    { 0xe1e9566a, 33 },
    { 0xbac9566c, 93 },
    { 0, 0 },
    { 0xafb9566e, 55 },
    { 0xa3e9566e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84095675, 61 },
    { 0, 0 },
    { 0x23c95677, 86 },
    { 0x91395677, 33 },
    { 0, 0 },
    { 0xbfc9567a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe679567f, 84 },
    { 0, 0 },
    { 0x53995681, 34 },
    { 0, 0 },
    { 0x52b95683, 90 },
    { 0x92395683, 98 },
    { 0x2c295684, 55 },
    { 0, 0 },
    { 0xc6495687, 67 },
    { 0x5d595688, 46 },
    { 0, 0 },
    { 0x2069568a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f9568d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac95692, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2095695, 55 },
    { 0x1895695, 66 },
    { 0xa2795695, 66 },
    { 0, 0 },
    { 0xbc595699, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5956a0, 76 },
    { 0x7c956a1, 84 },
    { 0, 0 },
    { 0x89e956a3, 98 },
    { 0x64956a4, 43 },
    { 0xd9d956a4, 34 },
    { 0x8b3956a4, 80 },
    { 0, 0 },
    { 0xfd5956a8, 76 },
    { 0, 0 },
    { 0x36e956aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9956ae, 23 },
    { 0xf31956ae, 33 },
    { 0x298956af, 55 },
    { 0x86b956ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883956ca, 90 },
    { 0xbc4956cb, 61 },
    { 0x211956cb, 80 },
    { 0xe9956cd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e956d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235956d9, 33 },
    { 0, 0 },
    { 0x753956db, 43 },
    { 0xccb956dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53956df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0956e4, 77 },
    { 0xb68956e5, 33 },
    { 0xcee956e6, 95 },
    { 0x602956e6, 98 },
    { 0, 0 },
    { 0xde8956e9, 94 },
    { 0, 0 },
    { 0xb6a956eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe956ef, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82956f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9956f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51956ff, 46 },
    { 0xae9956ff, 80 },
    { 0, 0 },
    { 0xe5195702, 33 },
    { 0, 0 },
    { 0x3ec95704, 90 },
    { 0x56295705, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d9570f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d095714, 98 },
    { 0x5a095714, 33 },
    { 0x5bb95716, 88 },
    { 0xf5e95714, 61 },
    { 0xc5895718, 55 },
    { 0x30395714, 46 },
    { 0x9139571a, 23 },
    { 0x47a9571a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48695721, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53795731, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90695736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb95739, 56 },
    { 0x63795739, 61 },
    { 0, 0 },
    { 0x9249573c, 90 },
    { 0xc9f9573c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e95746, 100 },
    { 0, 0 },
    { 0x88995748, 61 },
    { 0, 0 },
    { 0x2189574a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d9574d, 98 },
    { 0xebd9574e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e95752, 95 },
    { 0, 0 },
    { 0x8bf95754, 96 },
    { 0x6d095755, 61 },
    { 0xc4f95755, 33 },
    { 0x94695755, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee19575a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xba09575d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a95761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5695765, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c95771, 55 },
    { 0x13b95772, 23 },
    { 0xdc295772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23795776, 33 },
    { 0, 0 },
    { 0xf0095778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5019577b, 67 },
    { 0, 0 },
    { 0x9d89577d, 77 },
    { 0xfa99577d, 61 },
    { 0, 0 },
    { 0x2c95780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9695784, 61 },
    { 0, 0 },
    { 0xed295786, 90 },
    { 0x3e695787, 99 },
    { 0xed95788, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9578d, 90 },
    { 0, 0 },
    { 0xb7b9578f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10195796, 33 },
    { 0, 0 },
    { 0x75395798, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29579c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900957a5, 61 },
    { 0x14b957a6, 60 },
    { 0x357957a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf957aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b957af, 55 },
    { 0xa04957b0, 61 },
    { 0x7db957b1, 90 },
    { 0x3d3957b2, 76 },
    { 0x3ee957b0, 98 },
    { 0x49957b3, 83 },
    { 0, 0 },
    { 0xe3957b6, 33 },
    { 0x4b957b7, 61 },
    { 0xb36957b6, 43 },
    { 0x487957b9, 90 },
    { 0x7ee957ba, 90 },
    { 0x1cf957ba, 17 },
    { 0x445957ba, 98 },
    { 0, 0 },
    { 0x9af957be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60957d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995957d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3957d9, 98 },
    { 0, 0 },
    { 0xd72957db, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x560957de, 100 },
    { 0x9f957df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b957e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62957e7, 78 },
    { 0x3cf957e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135957ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297957f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81957f8, 91 },
    { 0, 0 },
    { 0x8f0957fa, 33 },
    { 0, 0 },
    { 0x8f9957fc, 78 },
    { 0xeeb957fc, 61 },
    { 0, 0 },
    { 0x8c2957ff, 66 },
    { 0, 0 },
    { 0xaa295801, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21095807, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3929580d, 33 },
    { 0x3869580d, 99 },
    { 0x619580d, 23 },
    { 0x68f95810, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2495816, 98 },
    { 0x40295816, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff95821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf649582a, 81 },
    { 0, 0 },
    { 0x4689582c, 33 },
    { 0x1879582d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b95830, 34 },
    { 0, 0 },
    { 0x33595832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b95841, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee95848, 95 },
    { 0x77895849, 61 },
    { 0x71795848, 55 },
    { 0xfcf9584b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c95850, 23 },
    { 0xe1395851, 23 },
    { 0xb0c95852, 34 },
    { 0x29295852, 90 },
    { 0, 0 },
    { 0x5ee95855, 55 },
    { 0x75a95855, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf979585e, 75 },
    { 0x3fd9585f, 66 },
    { 0x6d49585f, 34 },
    { 0x64e95861, 33 },
    { 0x1b795860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df95865, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42295869, 93 },
    { 0xfe39586a, 33 },
    { 0x3839586a, 76 },
    { 0xce69586a, 90 },
    { 0xbae9586d, 78 },
    { 0, 0 },
    { 0x7739586f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe95874, 93 },
    { 0x24195875, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa9587c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f95886, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf49588c, 33 },
    { 0x4ef9588d, 98 },
    { 0, 0 },
    { 0xf619588f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85095893, 98 },
    { 0xcb595894, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b95897, 5 },
    { 0x59095897, 61 },
    { 0, 0 },
    { 0x7729589a, 81 },
    { 0x509589b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d958a5, 84 },
    { 0x3fb958a5, 76 },
    { 0xc17958a7, 90 },
    { 0x304958a7, 90 },
    { 0x41b958a7, 23 },
    { 0xd5c958a7, 61 },
    { 0x2b8958ab, 93 },
    { 0x884958ab, 33 },
    { 0x6dd958a5, 78 },
    { 0xc41958ae, 99 },
    { 0x8d958a8, 75 },
    { 0xfb958a7, 8 },
    { 0, 0 },
    { 0x8c5958b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2958b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x860958ba, 33 },
    { 0x3e7958bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a958c0, 23 },
    { 0x896958c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2958c4, 76 },
    { 0xca5958c4, 34 },
    { 0x3fd958c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31958cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7958d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc958d5, 46 },
    { 0xcf8958d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5958da, 93 },
    { 0x316958db, 93 },
    { 0x63a958dc, 40 },
    { 0x932958db, 78 },
    { 0x96b958de, 61 },
    { 0, 0 },
    { 0x25e958e0, 52 },
    { 0x24a958e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6958e5, 67 },
    { 0, 0 },
    { 0xa19958e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4958eb, 36 },
    { 0x5b958ec, 33 },
    { 0xc83958ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29958f2, 33 },
    { 0xdf2958f3, 93 },
    { 0x4ff958f2, 61 },
    { 0x302958f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5958f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06958fd, 33 },
    { 0xd6f958fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee495902, 61 },
    { 0x41095903, 90 },
    { 0xce995903, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32195907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29590a, 91 },
    { 0x4919590b, 66 },
    { 0xaee9590b, 33 },
    { 0xd5e9590b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db95910, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xde395913, 98 },
    { 0, 0 },
    { 0x42f95915, 90 },
    { 0, 0 },
    { 0x9ec95917, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3559591b, 81 },
    { 0x7a19591c, 23 },
    { 0, 0 },
    { 0x2ad9591e, 17 },
    { 0x9fc9591f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2995923, 98 },
    { 0x6bb95924, 61 },
    { 0, 0 },
    { 0xbc795926, 65 },
    { 0xb1e95927, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd489592b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c95933, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e95937, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d9593a, 65 },
    { 0x779593b, 87 },
    { 0x27d9593a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee9593f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55795942, 33 },
    { 0, 0 },
    { 0x4c995944, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0295949, 34 },
    { 0x3ab95949, 61 },
    { 0x2b195949, 61 },
    { 0xc6e9594c, 61 },
    { 0x7399594d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c795958, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5839595f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795967, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c69596d, 98 },
    { 0, 0 },
    { 0x72e9596f, 34 },
    { 0, 0 },
    { 0xbe395971, 100 },
    { 0x3ef95972, 90 },
    { 0xaab95973, 34 },
    { 0x7a795974, 53 },
    { 0x7c895973, 90 },
    { 0x90995975, 55 },
    { 0, 0 },
    { 0x91795978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2949597b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f9597e, 33 },
    { 0x7b79597e, 91 },
    { 0, 0 },
    { 0xe9b95981, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df95984, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3695989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe629598e, 33 },
    { 0xd369598e, 33 },
    { 0x2839598f, 61 },
    { 0, 0 },
    { 0x7cb95992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c95996, 61 },
    { 0, 0 },
    { 0x1bd95998, 90 },
    { 0x63595999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3799599f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859959a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838959a8, 93 },
    { 0xa4a959a8, 61 },
    { 0xbd1959a8, 34 },
    { 0x8a0959aa, 61 },
    { 0x717959a9, 55 },
    { 0x90959ad, 27 },
    { 0xb23959ac, 95 },
    { 0xd4e959af, 33 },
    { 0xfa9959b0, 76 },
    { 0xc66959af, 33 },
    { 0xf28959b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b959b5, 23 },
    { 0x453959b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x72959b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4959bc, 33 },
    { 0, 0 },
    { 0x82e959be, 77 },
    { 0, 0 },
    { 0x391959c0, 23 },
    { 0x58959c0, 43 },
    { 0xc68959c1, 23 },
    { 0xfea959c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf959c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38e959c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29959d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487959d8, 91 },
    { 0x468959d9, 76 },
    { 0x16c959d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f959dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f959e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587959e5, 95 },
    { 0, 0 },
    { 0xf86959e7, 98 },
    { 0xbb3959e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94959eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f959f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xced959f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x588959fa, 33 },
    { 0, 0 },
    { 0xc87959fc, 93 },
    { 0x1fc959fd, 90 },
    { 0xc24959fe, 91 },
    { 0x46d959fe, 52 },
    { 0, 0 },
    { 0x73495a01, 84 },
    { 0x7e295a02, 90 },
    { 0, 0 },
    { 0xc4f95a04, 98 },
    { 0x9c995a05, 34 },
    { 0x45595a06, 91 },
    { 0, 0 },
    { 0xf1995a08, 66 },
    { 0, 0 },
    { 0x38195a0a, 61 },
    { 0x82c95a0b, 90 },
    { 0x8d395a0c, 90 },
    { 0, 0 },
    { 0xcb295a0e, 61 },
    { 0x3e95a0e, 90 },
    { 0x83b95a10, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e195a16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90895a1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25795a21, 56 },
    { 0x46395a22, 95 },
    { 0xfe195a23, 78 },
    { 0, 0 },
    { 0xd0195a25, 33 },
    { 0x55695a26, 33 },
    { 0xdb95a26, 23 },
    { 0x94295a28, 98 },
    { 0xf8895a29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c95a2c, 52 },
    { 0, 0 },
    { 0x89495a2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af95a33, 99 },
    { 0x8a995a33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0295a3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61495a43, 34 },
    { 0, 0 },
    { 0x39a95a45, 23 },
    { 0, 0 },
    { 0x5b95a47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e695a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a95a55, 61 },
    { 0x69e95a56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5795a5c, 98 },
    { 0xff895a5c, 55 },
    { 0x4fd95a5e, 64 },
    { 0xf3395a5e, 90 },
    { 0x6e195a60, 43 },
    { 0x13595a61, 61 },
    { 0x5ac95a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5e95a66, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf95a6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4595a75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4395a78, 23 },
    { 0, 0 },
    { 0x72c95a7a, 43 },
    { 0xe795a7a, 95 },
    { 0x8f295a7b, 34 },
    { 0xd5495a7b, 55 },
    { 0x3a795a7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73895a81, 33 },
    { 0x5d695a82, 34 },
    { 0xa295a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a95a86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67295a8a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d95a8f, 94 },
    { 0, 0 },
    { 0x5fb95a91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36995a95, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76595a9e, 33 },
    { 0, 0 },
    { 0xede95aa0, 99 },
    { 0xe0e95aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b95aa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34195ab0, 47 },
    { 0, 0 },
    { 0x46a95ab2, 52 },
    { 0x4a895ab2, 90 },
    { 0x92a95ab4, 90 },
    { 0xd0895ab2, 52 },
    { 0, 0 },
    { 0x6b695ab7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3595ac0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22195ac5, 61 },
    { 0, 0 },
    { 0x5ea95ac7, 91 },
    { 0xf6295ac7, 98 },
    { 0x1d695ac7, 34 },
    { 0xa7795ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a95acd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91695ad0, 76 },
    { 0xa4c95ad1, 61 },
    { 0x14495ad2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac195ad5, 76 },
    { 0x17995ad6, 90 },
    { 0x5f695ad6, 98 },
    { 0x44495ad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc595ae1, 74 },
    { 0, 0 },
    { 0xaed95ae3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27295af2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xff695af5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50695afd, 23 },
    { 0x34195afd, 43 },
    { 0, 0 },
    { 0xa695b00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9295b04, 98 },
    { 0, 0 },
    { 0x1c595b06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d695b09, 83 },
    { 0x7b695b09, 98 },
    { 0xcd195b09, 55 },
    { 0x8f995b0c, 90 },
    { 0x8b295b0a, 88 },
    { 0xd1695b0e, 43 },
    { 0, 0 },
    { 0x8fc95b10, 76 },
    { 0xc3d95b10, 90 },
    { 0x29395b11, 93 },
    { 0x8e395b13, 61 },
    { 0x88f95b14, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89995b19, 91 },
    { 0xe0d95b1a, 43 },
    { 0, 0 },
    { 0x10995b1c, 91 },
    { 0x4db95b1c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba995b23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6395b27, 95 },
    { 0xdaa95b27, 33 },
    { 0x15b95b29, 33 },
    { 0x1d195b2a, 98 },
    { 0x66195b29, 90 },
    { 0x3b395b2c, 76 },
    { 0x9c095b2d, 23 },
    { 0x87b95b28, 100 },
    { 0xe695b2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f95b34, 77 },
    { 0, 0 },
    { 0xe9595b36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd195b3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c95b3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c95b41, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9195b49, 90 },
    { 0x80595b4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83795b52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4495b55, 90 },
    { 0x4b095b56, 17 },
    { 0xe6b95b56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91c95b5b, 90 },
    { 0, 0 },
    { 0x8cf95b5d, 34 },
    { 0, 0 },
    { 0xdcc95b5f, 100 },
    { 0x99795b60, 65 },
    { 0x9dc95b5f, 33 },
    { 0x59695b5f, 33 },
    { 0xe1695b61, 95 },
    { 0, 0 },
    { 0x73e95b65, 98 },
    { 0xcf195b66, 23 },
    { 0xacc95b67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4295b6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a95b77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65f95b7d, 95 },
    { 0x28195b7e, 88 },
    { 0x1b595b7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57095b82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3a95b86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f95b8a, 9 },
    { 0x20595b8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba95b8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a395b91, 66 },
    { 0, 0 },
    { 0x26b95b93, 95 },
    { 0xe0995b93, 23 },
    { 0, 0 },
    { 0x47995b96, 75 },
    { 0xe4a95b96, 43 },
    { 0, 0 },
    { 0xdb295b99, 33 },
    { 0x47295b9a, 61 },
    { 0, 0 },
    { 0xbda95b9c, 98 },
    { 0xbb795b9c, 90 },
    { 0x595b9e, 47 },
    { 0, 0 },
    { 0x94995ba0, 98 },
    { 0x30f95ba1, 98 },
    { 0xf8d95ba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0295ba8, 91 },
    { 0xb3e95ba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5295bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6995bb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a95bb6, 95 },
    { 0, 0 },
    { 0x4a995bb8, 33 },
    { 0, 0 },
    { 0x30e95bba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38495bc1, 98 },
    { 0x6e095bc1, 95 },
    { 0x65395bc3, 23 },
    { 0xa3f95bc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b795bc9, 55 },
    { 0xf9695bca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f95bd0, 85 },
    { 0xe4995bd1, 80 },
    { 0x65995bd0, 98 },
    { 0, 0 },
    { 0xc6a95bd4, 33 },
    { 0xb2795bd4, 87 },
    { 0, 0 },
    { 0x4ed95bd7, 76 },
    { 0x1ae95bd8, 85 },
    { 0x5c195bd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f195be1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe395be4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e95be7, 90 },
    { 0, 0 },
    { 0xbc395be9, 67 },
    { 0x4195bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa95bef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca95bf3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b695bfe, 61 },
    { 0x68295bfe, 90 },
    { 0, 0 },
    { 0x79895c01, 95 },
    { 0x16295c01, 39 },
    { 0xa1195c03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc095c08, 23 },
    { 0, 0 },
    { 0x91b95c0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b095c0d, 67 },
    { 0xd6695c0e, 48 },
    { 0x9e095c0e, 100 },
    { 0xefd95c0f, 77 },
    { 0x41e95c11, 90 },
    { 0x64d95c12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8195c16, 5 },
    { 0x1b795c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6a95c1c, 77 },
    { 0, 0 },
    { 0xb3995c1e, 55 },
    { 0xab795c1e, 77 },
    { 0xc1495c20, 67 },
    { 0x5ff95c21, 90 },
    { 0xcd095c22, 67 },
    { 0, 0 },
    { 0x18795c24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce95c2a, 77 },
    { 0x2a295c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7495c32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53095c3b, 61 },
    { 0x4da95c3b, 95 },
    { 0xf4a95c3c, 90 },
    { 0x89295c3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x27995c41, 23 },
    { 0x94b95c41, 98 },
    { 0xecb95c43, 90 },
    { 0x4a295c42, 23 },
    { 0xbe595c45, 61 },
    { 0xd5b95c46, 67 },
    { 0xdf95c45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10695c4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0795c52, 33 },
    { 0xb2895c53, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a995c56, 99 },
    { 0, 0 },
    { 0xebb95c58, 95 },
    { 0x39f95c59, 90 },
    { 0x78095c5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6195c5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdb95c65, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8395c6e, 33 },
    { 0xedb95c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc95c74, 90 },
    { 0, 0 },
    { 0x79e95c76, 17 },
    { 0xc8095c76, 66 },
    { 0x29995c77, 23 },
    { 0, 0 },
    { 0x92095c7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9195c7d, 90 },
    { 0xc795c7e, 33 },
    { 0, 0 },
    { 0xdb695c80, 34 },
    { 0, 0 },
    { 0xf7395c82, 78 },
    { 0x8cf95c82, 90 },
    { 0xdbb95c84, 34 },
    { 0xeae95c84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0095c88, 95 },
    { 0x3195c89, 98 },
    { 0xd6d95c8a, 94 },
    { 0x61895c8a, 33 },
    { 0x5a995c89, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68795c91, 80 },
    { 0x54295c91, 98 },
    { 0x64195c93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f295c96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x80795c99, 33 },
    { 0x6ba95c99, 76 },
    { 0x32b95c9b, 98 },
    { 0xf6995c9c, 98 },
    { 0x9a695c9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6995ca0, 77 },
    { 0xc2b95ca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4795ca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c95ca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc595cab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b395cae, 76 },
    { 0, 0 },
    { 0x4a895cb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69795cb7, 23 },
    { 0x1c695cb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12195cbc, 91 },
    { 0x41c95cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e95cc0, 98 },
    { 0x78495cc0, 90 },
    { 0xc5095cc1, 33 },
    { 0x85295cc3, 90 },
    { 0xc6495cc0, 85 },
    { 0x7be95cc4, 90 },
    { 0xf2895cc6, 43 },
    { 0xf9595cc6, 90 },
    { 0x1c595cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed95ccd, 76 },
    { 0, 0 },
    { 0xd95ccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d95cd5, 33 },
    { 0x6895cd5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e95cdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5795cdf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c95ce7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6895cf1, 85 },
    { 0, 0 },
    { 0x12a95cf3, 33 },
    { 0, 0 },
    { 0xddb95cf5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x20195cf8, 61 },
    { 0x9e595cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a895cfd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23695d0a, 98 },
    { 0, 0 },
    { 0x8e795d0c, 46 },
    { 0xe9d95d0d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4995d13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34995d1d, 76 },
    { 0xf8895d1e, 90 },
    { 0x5d295d1f, 66 },
    { 0xdd995d20, 53 },
    { 0x75d95d20, 66 },
    { 0x78095d22, 81 },
    { 0, 0 },
    { 0x49f95d24, 17 },
    { 0xf0e95d24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98c95d29, 43 },
    { 0x76595d2a, 64 },
    { 0xeee95d2b, 46 },
    { 0xcb395d2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd95d30, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3295d36, 68 },
    { 0x99495d37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f95d3a, 55 },
    { 0, 0 },
    { 0xc9595d3c, 90 },
    { 0x38c95d3c, 23 },
    { 0xa1795d3e, 23 },
    { 0xe8e95d3e, 33 },
    { 0, 0 },
    { 0x96695d41, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2f95d44, 39 },
    { 0xca995d45, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a95d4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94195d54, 23 },
    { 0, 0 },
    { 0x13c95d56, 46 },
    { 0x14c95d57, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe95d5d, 61 },
    { 0, 0 },
    { 0x74695d5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b95d67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0895d77, 17 },
    { 0x5a195d78, 84 },
    { 0xf3495d78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe95d7d, 68 },
    { 0x4c395d7e, 23 },
    { 0x89895d7e, 81 },
    { 0xa6595d80, 46 },
    { 0x36695d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf895d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f95d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5695d8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68595d95, 76 },
    { 0, 0 },
    { 0x91695d97, 78 },
    { 0xf1795d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84895d9d, 76 },
    { 0, 0 },
    { 0xcec95d9f, 61 },
    { 0x6ea95da0, 55 },
    { 0, 0 },
    { 0xe995da2, 46 },
    { 0xc9795da3, 67 },
    { 0xe3695da4, 76 },
    { 0x1f495da5, 90 },
    { 0xebc95da4, 66 },
    { 0x1c95da5, 95 },
    { 0x4c695da8, 46 },
    { 0, 0 },
    { 0xbf495daa, 61 },
    { 0xcf795dab, 90 },
    { 0x67c95dac, 52 },
    { 0, 0 },
    { 0x2ac95dae, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a695dba, 55 },
    { 0xdc995dbb, 76 },
    { 0x2c95dbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70095dc0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6395dc4, 34 },
    { 0, 0 },
    { 0x5a195dc6, 33 },
    { 0x6495dc7, 61 },
    { 0xe6695dc7, 76 },
    { 0x15995dc6, 52 },
    { 0x10495dc6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6095dcf, 61 },
    { 0x98595dcf, 75 },
    { 0x695dd1, 98 },
    { 0xa6e95dd2, 90 },
    { 0xd4395dcf, 67 },
    { 0, 0 },
    { 0xf7795dd5, 85 },
    { 0xe4395dd6, 46 },
    { 0x60295dd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6995ddc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9795de0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5395de7, 98 },
    { 0x4c95de7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc795ded, 52 },
    { 0x8e095dee, 23 },
    { 0xbfb95ded, 75 },
    { 0xfca95ded, 98 },
    { 0, 0 },
    { 0xada95df2, 61 },
    { 0x97195df2, 90 },
    { 0xc5d95df3, 38 },
    { 0, 0 },
    { 0xfeb95df6, 67 },
    { 0, 0 },
    { 0xabe95df8, 95 },
    { 0, 0 },
    { 0x1af95dfa, 61 },
    { 0x77f95dfb, 91 },
    { 0x83195dfb, 95 },
    { 0xec295dfa, 55 },
    { 0xae495dfd, 90 },
    { 0xff795dff, 61 },
    { 0xd6a95e00, 33 },
    { 0, 0 },
    { 0x495e02, 96 },
    { 0x30f95e03, 95 },
    { 0xdbe95e03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e695e09, 55 },
    { 0x6e795e0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45495e0d, 84 },
    { 0x14d95e0e, 90 },
    { 0, 0 },
    { 0xc695e10, 61 },
    { 0x21b95e10, 93 },
    { 0, 0 },
    { 0x55695e13, 90 },
    { 0x62495e13, 95 },
    { 0xa8a95e13, 55 },
    { 0xdb895e16, 34 },
    { 0x4a195e17, 90 },
    { 0x29395e18, 90 },
    { 0x88695e18, 90 },
    { 0xd5795e19, 95 },
    { 0x9e195e1b, 61 },
    { 0x5ed95e1b, 95 },
    { 0xd1295e1d, 23 },
    { 0xed995e1e, 38 },
    { 0x12995e1d, 94 },
    { 0x5cd95e20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd95e24, 55 },
    { 0xe1495e24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a195e2b, 23 },
    { 0, 0 },
    { 0xb9f95e2d, 23 },
    { 0xa3e95e2e, 76 },
    { 0x6195e2f, 100 },
    { 0x4e095e30, 23 },
    { 0xc5495e30, 95 },
    { 0x95e32, 98 },
    { 0xbb495e33, 84 },
    { 0x8ea95e34, 33 },
    { 0x8ac95e2d, 80 },
    { 0x46495e2d, 95 },
    { 0x90095e37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3295e3a, 46 },
    { 0x61e95e3b, 66 },
    { 0, 0 },
    { 0x33a95e3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b95e45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28795e48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12195e4e, 68 },
    { 0, 0 },
    { 0x12995e50, 23 },
    { 0x5a495e51, 35 },
    { 0x17d95e52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55195e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc95e5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb95e60, 34 },
    { 0x70995e61, 77 },
    { 0, 0 },
    { 0x44b95e63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d95e6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1295e6e, 23 },
    { 0x9d395e6f, 33 },
    { 0xb3295e70, 43 },
    { 0xa6395e70, 84 },
    { 0xf5e95e70, 34 },
    { 0xa3895e72, 95 },
    { 0xe1195e73, 62 },
    { 0x3da95e6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37495e7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98995e80, 61 },
    { 0, 0 },
    { 0x38c95e82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8995e85, 90 },
    { 0x65a95e85, 33 },
    { 0, 0 },
    { 0x8e995e88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec95e8b, 33 },
    { 0, 0 },
    { 0x9aa95e8d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a595e92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a195e96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d95e99, 99 },
    { 0x38b95e99, 55 },
    { 0xcdc95e9a, 66 },
    { 0xd4195e9a, 81 },
    { 0x1095e9d, 87 },
    { 0x58795e9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a95ea1, 61 },
    { 0x94995ea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15595ea5, 65 },
    { 0x24e95ea6, 78 },
    { 0, 0 },
    { 0x5a395ea8, 53 },
    { 0, 0 },
    { 0x4895eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd95eae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92295eb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9595ebb, 33 },
    { 0x99295ebb, 90 },
    { 0xf1b95ebb, 91 },
    { 0x23595ebe, 87 },
    { 0x9c895ebf, 52 },
    { 0xa5e95ec0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c95ec4, 61 },
    { 0x3c395ec5, 33 },
    { 0xc6295ec5, 84 },
    { 0x11595ec7, 76 },
    { 0xc2895ec8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa95ecc, 98 },
    { 0xcce95ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e795ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea95ed3, 87 },
    { 0xa7495ed4, 55 },
    { 0x98395ed4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae95eda, 97 },
    { 0x4295eda, 33 },
    { 0x89995edc, 67 },
    { 0xe4595edd, 78 },
    { 0x6d595edb, 98 },
    { 0xe0695edf, 90 },
    { 0, 0 },
    { 0xed695ee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1895ee7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b495eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f95eef, 93 },
    { 0xea495eef, 90 },
    { 0x19095ef1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d95ef6, 95 },
    { 0xdb695ef6, 98 },
    { 0x9d895ef6, 61 },
    { 0x8fb95ef9, 95 },
    { 0x95a95efa, 98 },
    { 0x5fe95efb, 84 },
    { 0xf9f95efb, 87 },
    { 0xd8995efb, 61 },
    { 0xe2695ef9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce95f05, 61 },
    { 0x53495f06, 90 },
    { 0x8c495f06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57995f0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea95f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed95f14, 91 },
    { 0xa3c95f14, 61 },
    { 0xd6c95f14, 23 },
    { 0x6d795f17, 81 },
    { 0xc4395f14, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68895f1c, 76 },
    { 0x77695f1d, 61 },
    { 0x49195f1d, 61 },
    { 0x3cb95f1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe095f23, 68 },
    { 0x7195f24, 34 },
    { 0xf0895f25, 76 },
    { 0xf6195f26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d95f2f, 98 },
    { 0xb9595f2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25695f34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c95f38, 95 },
    { 0x6f495f39, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b795f3d, 55 },
    { 0x2795f3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22595f42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12795f45, 33 },
    { 0x73395f46, 78 },
    { 0x17295f46, 76 },
    { 0x6da95f48, 76 },
    { 0xd9e95f49, 46 },
    { 0x97095f4a, 33 },
    { 0, 0 },
    { 0xb5c95f4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e395f4f, 61 },
    { 0xe4795f50, 84 },
    { 0x5ca95f4f, 33 },
    { 0xca295f50, 90 },
    { 0, 0 },
    { 0xec595f54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee195f5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc95f61, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c695f65, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f95f68, 90 },
    { 0, 0 },
    { 0x89295f6a, 66 },
    { 0xab795f6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2595f71, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc895f74, 90 },
    { 0xfb195f75, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec295f7c, 33 },
    { 0, 0 },
    { 0xbce95f7e, 95 },
    { 0, 0 },
    { 0x34e95f80, 93 },
    { 0x3595f81, 61 },
    { 0x33f95f81, 95 },
    { 0xb2c95f83, 95 },
    { 0x14c95f84, 96 },
    { 0, 0 },
    { 0x89e95f86, 90 },
    { 0xfed95f87, 66 },
    { 0, 0 },
    { 0x79995f89, 61 },
    { 0x34595f89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf95f93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e895f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40995f9a, 36 },
    { 0x8e895f9b, 98 },
    { 0x74295f9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3595fa1, 95 },
    { 0x7095fa2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e995fa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3895fad, 23 },
    { 0xa0095fae, 88 },
    { 0xa2695faf, 34 },
    { 0, 0 },
    { 0x81895fb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc95fb4, 84 },
    { 0xcf895fb5, 61 },
    { 0xa8195fb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69495fb9, 66 },
    { 0, 0 },
    { 0x4a95fbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41e95fbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c95fc2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68595fc9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a395fcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1195fd0, 55 },
    { 0, 0 },
    { 0xf0395fd2, 43 },
    { 0x54795fd3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4795fdf, 61 },
    { 0x6a295fe0, 43 },
    { 0, 0 },
    { 0x4cb95fe2, 85 },
    { 0x96595fe3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3295fea, 93 },
    { 0x32895feb, 19 },
    { 0, 0 },
    { 0xbe495fed, 46 },
    { 0x7b895fed, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e95ff1, 85 },
    { 0x71295ff1, 95 },
    { 0x95995ff1, 95 },
    { 0x85195ff2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e95ff8, 55 },
    { 0x17595ff8, 78 },
    { 0x37b95ffa, 61 },
    { 0, 0 },
    { 0xb895ffc, 85 },
    { 0xd3395ffd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3696001, 67 },
    { 0, 0 },
    { 0x21896003, 55 },
    { 0xc6496003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf296009, 66 },
    { 0xaf49600a, 90 },
    { 0x2e19600a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4496010, 61 },
    { 0x4b96010, 100 },
    { 0x36196012, 76 },
    { 0x33196013, 61 },
    { 0x5af96014, 61 },
    { 0x72b96010, 34 },
    { 0x6a96015, 47 },
    { 0x89496017, 61 },
    { 0, 0 },
    { 0xcb96019, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9829601d, 61 },
    { 0xa049601e, 61 },
    { 0x2209601f, 76 },
    { 0x71996020, 43 },
    { 0, 0 },
    { 0x19b96022, 98 },
    { 0xdf596022, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a99602d, 93 },
    { 0x4269602e, 43 },
    { 0x9789602f, 33 },
    { 0xcbe9602d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c96038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c96047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd9604b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe279604f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x56b96052, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e96055, 90 },
    { 0xa8c96056, 90 },
    { 0x12096057, 75 },
    { 0x9a996058, 95 },
    { 0x71d96057, 34 },
    { 0xba09605a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd829605d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3796064, 33 },
    { 0x28196065, 95 },
    { 0xba396065, 52 },
    { 0xd8e96067, 23 },
    { 0xf0296067, 46 },
    { 0xc3396067, 90 },
    { 0x35a9606a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b9606e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce96072, 43 },
    { 0, 0 },
    { 0x8eb96074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67196079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f396081, 40 },
    { 0, 0 },
    { 0xee296083, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d496088, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b796090, 61 },
    { 0x9bf96090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34696094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e596098, 99 },
    { 0x96696099, 98 },
    { 0x8859609a, 43 },
    { 0, 0 },
    { 0xaec9609c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178960a1, 98 },
    { 0x38c960a2, 87 },
    { 0x2b6960a3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e960a6, 65 },
    { 0xc3e960a7, 66 },
    { 0, 0 },
    { 0x27c960a9, 77 },
    { 0xacb960aa, 55 },
    { 0x767960ab, 34 },
    { 0, 0 },
    { 0xa03960ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7960b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7f960b7, 91 },
    { 0x12f960b8, 88 },
    { 0xfbd960b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306960be, 95 },
    { 0x926960bf, 77 },
    { 0x66d960bf, 90 },
    { 0x4e4960be, 98 },
    { 0, 0 },
    { 0x954960c3, 61 },
    { 0x4de960c4, 82 },
    { 0x968960c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97960c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40960ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636960dc, 52 },
    { 0x768960dd, 65 },
    { 0xf3b960dc, 23 },
    { 0xe3960dd, 33 },
    { 0xae8960df, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975960e4, 33 },
    { 0, 0 },
    { 0x777960e6, 95 },
    { 0xac2960e7, 76 },
    { 0xb9c960e7, 76 },
    { 0x6f5960e9, 61 },
    { 0xe99960e8, 100 },
    { 0x1fe960eb, 52 },
    { 0x507960ec, 90 },
    { 0x7c4960ed, 43 },
    { 0x8b7960ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170960f2, 80 },
    { 0x6e4960f3, 83 },
    { 0x3a960f2, 23 },
    { 0x93960f5, 81 },
    { 0xd38960f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74960f9, 90 },
    { 0x3d9960f9, 90 },
    { 0xfe2960fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68960fe, 76 },
    { 0x986960fe, 84 },
    { 0x49596100, 34 },
    { 0x8ed96100, 33 },
    { 0xd5296100, 23 },
    { 0xf2296101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe296107, 84 },
    { 0x51996108, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49610d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df96110, 46 },
    { 0x45296111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c96114, 76 },
    { 0xe5596115, 46 },
    { 0x7b696115, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa859611a, 43 },
    { 0x9389611b, 33 },
    { 0x83a9611b, 46 },
    { 0xba79611c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b96120, 82 },
    { 0, 0 },
    { 0x15496122, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee96126, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d96129, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe289612c, 33 },
    { 0xb7f9612d, 95 },
    { 0x2a69612e, 55 },
    { 0, 0 },
    { 0x62a96130, 23 },
    { 0x5e496130, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5996134, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c296139, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2919613d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3696141, 78 },
    { 0x5f396142, 95 },
    { 0xb6e96143, 23 },
    { 0x41896143, 85 },
    { 0, 0 },
    { 0xfe596146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f69614a, 33 },
    { 0, 0 },
    { 0x97e9614c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb96153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439615a, 33 },
    { 0x4679615a, 92 },
    { 0x6389615b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32596167, 34 },
    { 0xc5196168, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469616f, 61 },
    { 0xcb896170, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac96173, 46 },
    { 0xcda96173, 91 },
    { 0xcfc96175, 33 },
    { 0, 0 },
    { 0x5cd96177, 75 },
    { 0, 0 },
    { 0x93b96179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a9617c, 78 },
    { 0x1309617c, 91 },
    { 0xce89617e, 95 },
    { 0x6649617f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x95996182, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7496185, 90 },
    { 0, 0 },
    { 0x4a696187, 76 },
    { 0xb1596188, 34 },
    { 0xfb896188, 61 },
    { 0xeac9618a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef996190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a96193, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5496197, 95 },
    { 0x93096198, 98 },
    { 0x72c96199, 95 },
    { 0, 0 },
    { 0x30c9619b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7df9619e, 52 },
    { 0xcdf9619e, 36 },
    { 0x7ab9619e, 66 },
    { 0x998961a1, 99 },
    { 0x10961a2, 61 },
    { 0x78961a2, 98 },
    { 0x73d961a3, 23 },
    { 0xcb7961a0, 33 },
    { 0xa3b961a2, 61 },
    { 0xe72961a0, 98 },
    { 0, 0 },
    { 0x349961a9, 99 },
    { 0, 0 },
    { 0x4ad961ab, 100 },
    { 0x2cd961ab, 76 },
    { 0x4a4961ad, 97 },
    { 0x567961ae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ff961b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112961b6, 90 },
    { 0, 0 },
    { 0x775961b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6961bc, 66 },
    { 0, 0 },
    { 0xf8b961be, 64 },
    { 0x797961be, 61 },
    { 0, 0 },
    { 0xdc7961c1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d961c6, 90 },
    { 0, 0 },
    { 0x4b4961c8, 98 },
    { 0xdfc961c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984961cd, 55 },
    { 0x61f961ce, 55 },
    { 0xd16961cf, 87 },
    { 0xd48961cf, 33 },
    { 0xe89961d1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5961da, 95 },
    { 0xcd6961db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61f961df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2961e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923961ea, 95 },
    { 0xddc961eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836961ef, 61 },
    { 0, 0 },
    { 0xe52961f1, 33 },
    { 0x88e961f2, 69 },
    { 0, 0 },
    { 0xc9c961f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d961fa, 76 },
    { 0x926961fb, 34 },
    { 0x865961fc, 93 },
    { 0x89f961fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65596201, 23 },
    { 0xeb496202, 55 },
    { 0xe3d96203, 95 },
    { 0xae496204, 87 },
    { 0x73f96205, 87 },
    { 0xeaf96206, 98 },
    { 0xc3396207, 63 },
    { 0x65196208, 46 },
    { 0x7d896208, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed796210, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796217, 82 },
    { 0xd0196218, 90 },
    { 0x15996218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d89621c, 61 },
    { 0, 0 },
    { 0xe769621e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea596221, 61 },
    { 0xe4896221, 23 },
    { 0, 0 },
    { 0xf5a96224, 76 },
    { 0xb7a96225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae696228, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596235, 23 },
    { 0x82a96235, 61 },
    { 0x7ce96237, 90 },
    { 0x49396237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f49623b, 76 },
    { 0xa399623c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1029623f, 23 },
    { 0x45696240, 90 },
    { 0, 0 },
    { 0x46d96242, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d96246, 98 },
    { 0, 0 },
    { 0x2e796248, 98 },
    { 0, 0 },
    { 0x64a9624a, 91 },
    { 0, 0 },
    { 0x1b49624c, 98 },
    { 0x1359624c, 61 },
    { 0, 0 },
    { 0x6f59624f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef96253, 98 },
    { 0x45896254, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f196257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff9625c, 99 },
    { 0x2839625c, 90 },
    { 0x4759625c, 90 },
    { 0, 0 },
    { 0x8fa96260, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4996265, 78 },
    { 0x66b96265, 46 },
    { 0xf3996265, 78 },
    { 0, 0 },
    { 0xf8b96269, 90 },
    { 0xf2996269, 78 },
    { 0x4f196269, 90 },
    { 0x13796269, 61 },
    { 0x6279626b, 23 },
    { 0, 0 },
    { 0xd279626f, 68 },
    { 0x1379626f, 40 },
    { 0, 0 },
    { 0x6f596272, 55 },
    { 0, 0 },
    { 0x11896274, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af9627c, 95 },
    { 0xdd69627d, 33 },
    { 0x98a9627e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3396282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed96285, 28 },
    { 0x1dc96286, 91 },
    { 0, 0 },
    { 0x22396288, 88 },
    { 0x61d96289, 90 },
    { 0xe0696289, 95 },
    { 0x7df9628b, 90 },
    { 0x4a496288, 100 },
    { 0x9e79628d, 98 },
    { 0, 0 },
    { 0xf999628f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55796292, 90 },
    { 0x8b096292, 76 },
    { 0xd2796294, 76 },
    { 0xae696295, 90 },
    { 0x52296295, 90 },
    { 0x2cb96296, 33 },
    { 0x66796294, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc689629b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea09629f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x136962a2, 49 },
    { 0xf85962a3, 33 },
    { 0, 0 },
    { 0x5c0962a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5962b1, 46 },
    { 0, 0 },
    { 0xb4b962b3, 61 },
    { 0x9ec962b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae962b7, 78 },
    { 0, 0 },
    { 0xcd3962b9, 63 },
    { 0, 0 },
    { 0x69b962bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c962c1, 90 },
    { 0xde962c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589962cb, 66 },
    { 0, 0 },
    { 0x8f6962cd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4962d1, 61 },
    { 0x49e962d2, 76 },
    { 0x80962d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972962d8, 90 },
    { 0, 0 },
    { 0xe1c962da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07962dd, 88 },
    { 0x236962de, 17 },
    { 0xdb7962df, 90 },
    { 0x83b962df, 33 },
    { 0x83962e1, 55 },
    { 0xc22962e2, 98 },
    { 0xf19962e2, 56 },
    { 0xf0a962e4, 43 },
    { 0xc38962e2, 93 },
    { 0, 0 },
    { 0xfaa962e7, 90 },
    { 0xfb6962e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9962eb, 90 },
    { 0xa35962ec, 61 },
    { 0x4c9962ec, 99 },
    { 0, 0 },
    { 0xa1b962ef, 55 },
    { 0xc7d962f0, 99 },
    { 0, 0 },
    { 0x314962f2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48962fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07962ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4796302, 11 },
    { 0xcd196303, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7896307, 46 },
    { 0xdd796308, 90 },
    { 0xfab96307, 33 },
    { 0xe0396307, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb9630e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa96311, 33 },
    { 0xd3696312, 98 },
    { 0xc2996313, 61 },
    { 0, 0 },
    { 0xefd96315, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac9631a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3349631f, 67 },
    { 0x49896320, 33 },
    { 0x4579631f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a896324, 52 },
    { 0x2a996325, 23 },
    { 0xc6b96326, 33 },
    { 0, 0 },
    { 0xc5396328, 46 },
    { 0, 0 },
    { 0x3059632a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b9632f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xcae96332, 90 },
    { 0x1c796332, 81 },
    { 0x8fa96332, 55 },
    { 0xf1396335, 3 },
    { 0xdb96334, 90 },
    { 0xea096337, 85 },
    { 0x52e96337, 46 },
    { 0, 0 },
    { 0xcb99633a, 52 },
    { 0x9ba9633a, 85 },
    { 0x1119633c, 43 },
    { 0, 0 },
    { 0x35a9633e, 99 },
    { 0xad79633e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c96345, 81 },
    { 0, 0 },
    { 0xe7496347, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9634d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5296352, 23 },
    { 0x82f96353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68896358, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c9635e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b796362, 55 },
    { 0xb9e96363, 46 },
    { 0xebb96364, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e96368, 90 },
    { 0, 0 },
    { 0xb9e9636a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30396373, 34 },
    { 0, 0 },
    { 0x1f796375, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef29637e, 78 },
    { 0x8a89637f, 43 },
    { 0x43e96380, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf496385, 33 },
    { 0, 0 },
    { 0x8896387, 61 },
    { 0x51b96388, 81 },
    { 0x7fe96387, 23 },
    { 0x7d396388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98796391, 90 },
    { 0xe3696392, 34 },
    { 0, 0 },
    { 0x83196394, 90 },
    { 0x54d96395, 33 },
    { 0xeb96396, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x81396399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8709639e, 34 },
    { 0xb639639f, 90 },
    { 0xf79639e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a963a3, 33 },
    { 0, 0 },
    { 0xe41963a5, 23 },
    { 0xa47963a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457963ad, 76 },
    { 0x53f963ae, 100 },
    { 0x656963ae, 91 },
    { 0, 0 },
    { 0x40c963b1, 61 },
    { 0x62f963b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33963ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da963be, 90 },
    { 0, 0 },
    { 0x4b8963c0, 46 },
    { 0, 0 },
    { 0x357963c2, 46 },
    { 0x49d963c2, 61 },
    { 0xe6b963c4, 61 },
    { 0, 0 },
    { 0x2e0963c6, 43 },
    { 0x569963c6, 95 },
    { 0x417963c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5963cc, 55 },
    { 0, 0 },
    { 0xdaf963ce, 76 },
    { 0, 0 },
    { 0xb67963d0, 90 },
    { 0x234963d0, 91 },
    { 0x1c3963d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d963d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994963d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369963dd, 61 },
    { 0, 0 },
    { 0x2a1963df, 38 },
    { 0, 0 },
    { 0x41f963e1, 76 },
    { 0x3b3963e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896963e6, 75 },
    { 0xd09963e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141963eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f963ee, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3963f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x321963fc, 34 },
    { 0xedc963fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12896400, 67 },
    { 0x7b096401, 90 },
    { 0, 0 },
    { 0xc4c96403, 43 },
    { 0x6f896404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44196409, 95 },
    { 0x7439640a, 67 },
    { 0, 0 },
    { 0x7fe9640c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e96411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6596414, 65 },
    { 0x4f96415, 98 },
    { 0x34e96416, 68 },
    { 0x51d96417, 68 },
    { 0xdf696416, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f9641c, 34 },
    { 0xfeb9641c, 23 },
    { 0xd9b9641c, 100 },
    { 0, 0 },
    { 0x75c96420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee396425, 61 },
    { 0, 0 },
    { 0x90796427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8789642d, 33 },
    { 0xb229642e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa96434, 87 },
    { 0x27396435, 90 },
    { 0x32296435, 91 },
    { 0xa3896437, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb319643b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9643e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b96441, 76 },
    { 0x12c96442, 95 },
    { 0xc2396443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99496449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b96450, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc96453, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa269645a, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef09645f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55896462, 34 },
    { 0xf4696462, 87 },
    { 0xd6b96462, 46 },
    { 0xb6496462, 84 },
    { 0x1a596464, 61 },
    { 0, 0 },
    { 0xbf496468, 34 },
    { 0xadf96469, 33 },
    { 0x3089646a, 90 },
    { 0xbee9646a, 90 },
    { 0x3669646b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b496470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9996476, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d896479, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6359647d, 43 },
    { 0x64f9647e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49696481, 33 },
    { 0x9c696481, 33 },
    { 0xb9196482, 90 },
    { 0x8af96481, 76 },
    { 0, 0 },
    { 0x46396486, 33 },
    { 0x63096486, 90 },
    { 0, 0 },
    { 0x74496489, 84 },
    { 0, 0 },
    { 0xe9f9648b, 46 },
    { 0x7ac9648c, 98 },
    { 0, 0 },
    { 0x4039648e, 91 },
    { 0, 0 },
    { 0x94996490, 80 },
    { 0xce696491, 55 },
    { 0, 0 },
    { 0xcc696493, 40 },
    { 0x50696494, 90 },
    { 0xdc596494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7796498, 23 },
    { 0x5f196499, 61 },
    { 0xd1d9649a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5259649e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9964a2, 33 },
    { 0x586964a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f964a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a964ab, 23 },
    { 0xe96964ac, 90 },
    { 0, 0 },
    { 0x35f964ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a964b3, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee964b6, 95 },
    { 0x86964b7, 98 },
    { 0x91c964b8, 34 },
    { 0, 0 },
    { 0x90f964ba, 25 },
    { 0, 0 },
    { 0x5964bc, 78 },
    { 0x333964bd, 76 },
    { 0x995964be, 23 },
    { 0x11964bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6964c7, 90 },
    { 0xd8d964c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8964cb, 33 },
    { 0x69c964cc, 90 },
    { 0x558964cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06964d0, 95 },
    { 0, 0 },
    { 0x3df964d2, 90 },
    { 0xfc5964d3, 87 },
    { 0x836964d4, 90 },
    { 0xc69964d5, 98 },
    { 0x506964d5, 34 },
    { 0, 0 },
    { 0xa09964d8, 40 },
    { 0x48c964d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x634964dc, 46 },
    { 0xa5e964dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d964e3, 17 },
    { 0, 0 },
    { 0x324964e5, 55 },
    { 0, 0 },
    { 0x1bf964e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7964eb, 95 },
    { 0, 0 },
    { 0x142964ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9964f0, 55 },
    { 0x7a4964f0, 100 },
    { 0, 0 },
    { 0xe1d964f3, 33 },
    { 0x92964f4, 83 },
    { 0x28964f5, 61 },
    { 0xe9a964f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18964f9, 61 },
    { 0, 0 },
    { 0x8ff964fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f964ff, 34 },
    { 0xc1896500, 61 },
    { 0xe2e96500, 43 },
    { 0x63496502, 40 },
    { 0xc1b96503, 43 },
    { 0x46a96504, 98 },
    { 0x1f296504, 76 },
    { 0xbec964ff, 90 },
    { 0xef096507, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7759650a, 61 },
    { 0x11f9650a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93996511, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3696517, 33 },
    { 0, 0 },
    { 0xde696519, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9651d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc996520, 90 },
    { 0x3b196521, 46 },
    { 0, 0 },
    { 0x31496523, 98 },
    { 0xf1796524, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d96527, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6689652a, 33 },
    { 0x2219652b, 43 },
    { 0x41c9652b, 90 },
    { 0x2329652c, 17 },
    { 0x9d39652e, 95 },
    { 0xf299652c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc696533, 85 },
    { 0xb4996534, 93 },
    { 0x36696535, 23 },
    { 0x88996536, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4459653e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce696544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d99654a, 99 },
    { 0, 0 },
    { 0xdd79654c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60496550, 95 },
    { 0, 0 },
    { 0x33396552, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x70996555, 98 },
    { 0x4396556, 61 },
    { 0, 0 },
    { 0xea996558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa529655d, 61 },
    { 0xd119655e, 43 },
    { 0xeff9655e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53096563, 99 },
    { 0xd4996563, 52 },
    { 0, 0 },
    { 0xbd196566, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9656d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea96575, 87 },
    { 0, 0 },
    { 0x29596577, 90 },
    { 0, 0 },
    { 0xe7796579, 93 },
    { 0, 0 },
    { 0xe439657b, 90 },
    { 0x6f49657c, 90 },
    { 0xff39657d, 90 },
    { 0, 0 },
    { 0x2439657f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x52096582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33896589, 24 },
    { 0xd5496589, 33 },
    { 0xc319658b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0296592, 95 },
    { 0, 0 },
    { 0x9b996594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6379659b, 90 },
    { 0, 0 },
    { 0x3a69659d, 55 },
    { 0xeeb9659e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4965a4, 33 },
    { 0x28e965a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b965a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816965af, 93 },
    { 0x629965b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43965b4, 61 },
    { 0xd5d965b4, 85 },
    { 0xd9a965b5, 95 },
    { 0x64965b7, 23 },
    { 0, 0 },
    { 0x124965b9, 85 },
    { 0x486965ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a965bf, 87 },
    { 0x4ac965bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5965c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754965ca, 88 },
    { 0x8e1965ca, 23 },
    { 0xf67965cb, 66 },
    { 0xa00965cd, 90 },
    { 0x665965ce, 90 },
    { 0x306965cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39965d5, 90 },
    { 0, 0 },
    { 0x612965d7, 90 },
    { 0, 0 },
    { 0x9c965d9, 43 },
    { 0x530965da, 33 },
    { 0xdce965db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4965df, 90 },
    { 0x139965df, 78 },
    { 0, 0 },
    { 0x4c7965e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca965e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d965e9, 34 },
    { 0x30c965e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf965ed, 90 },
    { 0xad5965ee, 81 },
    { 0, 0 },
    { 0xb7f965f0, 95 },
    { 0xcb9965f0, 77 },
    { 0x46965f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef965f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76965fb, 78 },
    { 0x2b9965fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68796604, 95 },
    { 0, 0 },
    { 0x81396606, 33 },
    { 0xa1f96607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99660d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc96614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b896617, 52 },
    { 0x8c496618, 95 },
    { 0x83896619, 85 },
    { 0x26b9661a, 84 },
    { 0xb8b96619, 61 },
    { 0, 0 },
    { 0x6139661d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b96622, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5996627, 90 },
    { 0x6e496627, 33 },
    { 0x8496629, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e396632, 95 },
    { 0x82196633, 33 },
    { 0xcf596634, 55 },
    { 0xefb96633, 61 },
    { 0x20896634, 55 },
    { 0xb8f96637, 23 },
    { 0x15b96638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe919663d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe496642, 90 },
    { 0, 0 },
    { 0x56396644, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa809664a, 90 },
    { 0x26b9664b, 23 },
    { 0xb2a9664a, 98 },
    { 0x9ad9664d, 33 },
    { 0, 0 },
    { 0x6519664f, 76 },
    { 0x51996650, 87 },
    { 0, 0 },
    { 0x4d196652, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5396656, 68 },
    { 0x41e96657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e896663, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2196669, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84496671, 55 },
    { 0x76b96671, 80 },
    { 0xd7996673, 87 },
    { 0, 0 },
    { 0x18f96675, 43 },
    { 0x2dd96676, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd269667a, 33 },
    { 0xb29667b, 95 },
    { 0x6ae9667c, 95 },
    { 0x18d9667c, 84 },
    { 0x2649667b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a96684, 61 },
    { 0x8db96685, 98 },
    { 0xf1d96686, 95 },
    { 0x28996687, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a9668a, 23 },
    { 0xbe89668b, 76 },
    { 0xd989668b, 76 },
    { 0x7579668d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38396692, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d96695, 43 },
    { 0x8b896696, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f9669b, 76 },
    { 0, 0 },
    { 0x3c49669d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04966a3, 23 },
    { 0x431966a3, 100 },
    { 0x646966a3, 23 },
    { 0x80d966a6, 85 },
    { 0x740966a7, 23 },
    { 0x2e5966a8, 76 },
    { 0xc52966a8, 17 },
    { 0x35d966a8, 95 },
    { 0xee1966ab, 34 },
    { 0x3e8966a6, 43 },
    { 0x6b4966ab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2966b1, 90 },
    { 0xa4a966b1, 95 },
    { 0x39e966b3, 33 },
    { 0x4ed966b3, 98 },
    { 0, 0 },
    { 0x980966b6, 96 },
    { 0xe5e966b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33966ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66966be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5966c2, 55 },
    { 0xe51966c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866966c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac966ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a966d3, 88 },
    { 0x96b966d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28966da, 91 },
    { 0x404966db, 98 },
    { 0xdcb966da, 34 },
    { 0x18b966dd, 95 },
    { 0xe90966db, 90 },
    { 0x8d7966db, 98 },
    { 0xaf4966dc, 33 },
    { 0xb27966e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88966e6, 85 },
    { 0x752966e7, 95 },
    { 0x60966e8, 98 },
    { 0x468966e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e966ed, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec966f2, 90 },
    { 0xf31966f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711966f9, 90 },
    { 0xb24966f9, 91 },
    { 0, 0 },
    { 0x31966fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30996701, 33 },
    { 0x24896701, 33 },
    { 0, 0 },
    { 0xc2a96704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b796707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd409670e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd996713, 95 },
    { 0x42c96714, 40 },
    { 0, 0 },
    { 0xb2596716, 99 },
    { 0xaa796716, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b96720, 93 },
    { 0x64496721, 90 },
    { 0xf8e96722, 23 },
    { 0, 0 },
    { 0x9b796724, 95 },
    { 0, 0 },
    { 0x80896726, 33 },
    { 0x59996727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea9672a, 33 },
    { 0xbe79672b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729672e, 90 },
    { 0x69b9672f, 91 },
    { 0x2e89672f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17896734, 38 },
    { 0, 0 },
    { 0xa6996736, 85 },
    { 0xdad96736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c9673b, 90 },
    { 0xb829673c, 34 },
    { 0x8a39673d, 99 },
    { 0, 0 },
    { 0xbb19673f, 90 },
    { 0x36696740, 90 },
    { 0xdb096741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c496744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309674b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7996750, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8896754, 90 },
    { 0x26396755, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e96758, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5819675f, 55 },
    { 0xe7e96760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a96766, 43 },
    { 0xb7496766, 33 },
    { 0x65a96768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439676c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf619676f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7796772, 93 },
    { 0x89d96773, 66 },
    { 0x85596774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819677c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26496780, 90 },
    { 0xa9b96781, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56096788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9678c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f96790, 23 },
    { 0x18496791, 23 },
    { 0x29796790, 95 },
    { 0, 0 },
    { 0x51496794, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb649679a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8589679d, 33 },
    { 0, 0 },
    { 0x6869679f, 87 },
    { 0x8589679f, 61 },
    { 0xd40967a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9967a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4967a7, 76 },
    { 0, 0 },
    { 0x456967a9, 66 },
    { 0xb6f967a9, 90 },
    { 0, 0 },
    { 0x534967ac, 34 },
    { 0, 0 },
    { 0xe9f967ae, 76 },
    { 0, 0 },
    { 0x5f8967b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44967bb, 98 },
    { 0xb69967bb, 33 },
    { 0xf07967bb, 33 },
    { 0xd37967be, 33 },
    { 0xd72967bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e967c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51967ca, 66 },
    { 0x90967cb, 33 },
    { 0, 0 },
    { 0x5db967cd, 61 },
    { 0, 0 },
    { 0xe99967cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67967d6, 100 },
    { 0, 0 },
    { 0xc58967d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b967db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206967df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112967e6, 77 },
    { 0xc1f967e7, 98 },
    { 0x5e3967e8, 90 },
    { 0xdb7967e8, 100 },
    { 0x9b0967e7, 90 },
    { 0x732967eb, 66 },
    { 0, 0 },
    { 0x15e967ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834967f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10967f6, 34 },
    { 0xc7e967f7, 23 },
    { 0x5d8967f8, 95 },
    { 0x158967f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c967fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c96800, 95 },
    { 0x99d96801, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa96807, 99 },
    { 0xffc96807, 95 },
    { 0x3c796809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c9680d, 23 },
    { 0xdc39680d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b96813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4096818, 61 },
    { 0xd9796819, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b39681e, 84 },
    { 0, 0 },
    { 0x9aa96820, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83996825, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe96831, 43 },
    { 0x3a696831, 55 },
    { 0, 0 },
    { 0xa3996834, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b96838, 93 },
    { 0x90096839, 61 },
    { 0xdae96839, 61 },
    { 0x71c9683b, 91 },
    { 0x71b9683b, 40 },
    { 0x5c9683c, 90 },
    { 0x2959683e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21496843, 36 },
    { 0xf1596844, 33 },
    { 0xdc696845, 88 },
    { 0, 0 },
    { 0x5ba96847, 52 },
    { 0x41296847, 98 },
    { 0, 0 },
    { 0x9559684a, 91 },
    { 0x3059684b, 61 },
    { 0x9c89684a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26a96850, 77 },
    { 0x6e796851, 33 },
    { 0, 0 },
    { 0x6f396853, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e96862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96596866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4319686d, 80 },
    { 0x2189686e, 66 },
    { 0, 0 },
    { 0xa3696870, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56796877, 90 },
    { 0xaa196878, 17 },
    { 0, 0 },
    { 0x4569687a, 33 },
    { 0x7ef9687b, 88 },
    { 0x69c9687c, 33 },
    { 0xa389687c, 70 },
    { 0x3c19687e, 90 },
    { 0, 0 },
    { 0x48096880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d96885, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a96889, 55 },
    { 0x3569688a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a96892, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x86096895, 98 },
    { 0xfb796895, 23 },
    { 0xe7796897, 33 },
    { 0xd0a96895, 66 },
    { 0, 0 },
    { 0xfa69689a, 95 },
    { 0x9f09689a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda59689e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29968a2, 91 },
    { 0, 0 },
    { 0x562968a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6968a8, 77 },
    { 0, 0 },
    { 0x865968aa, 78 },
    { 0x973968ab, 23 },
    { 0, 0 },
    { 0x8e4968ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x197968b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833968b5, 61 },
    { 0x917968b6, 61 },
    { 0xe1a968b7, 23 },
    { 0xd2d968b8, 84 },
    { 0x162968b9, 78 },
    { 0x9d3968ba, 95 },
    { 0, 0 },
    { 0x7c3968bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf968c0, 34 },
    { 0x493968c1, 43 },
    { 0, 0 },
    { 0xe1f968c3, 61 },
    { 0x7aa968c4, 34 },
    { 0, 0 },
    { 0x362968c6, 90 },
    { 0x255968c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536968cc, 91 },
    { 0xce3968cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1968d0, 33 },
    { 0, 0 },
    { 0x86b968d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4968d6, 66 },
    { 0xd98968d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e968df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810968e4, 98 },
    { 0xb65968e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12968ec, 61 },
    { 0xf1f968ec, 17 },
    { 0x7c1968ee, 78 },
    { 0x659968ee, 78 },
    { 0xf77968f0, 90 },
    { 0xbd6968f1, 46 },
    { 0xd5c968f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f968f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x881968f8, 98 },
    { 0x955968f9, 61 },
    { 0, 0 },
    { 0x71e968fb, 69 },
    { 0x9c0968fb, 90 },
    { 0x436968fb, 33 },
    { 0x8e9968fe, 23 },
    { 0, 0 },
    { 0x83796900, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21896908, 83 },
    { 0xb9196909, 23 },
    { 0xe589690a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6596911, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x35996914, 78 },
    { 0, 0 },
    { 0xbc796916, 98 },
    { 0, 0 },
    { 0x89f96918, 84 },
    { 0, 0 },
    { 0xbc29691a, 55 },
    { 0xe639691b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5496921, 61 },
    { 0xd9596922, 33 },
    { 0xb1296923, 23 },
    { 0x68e96922, 87 },
    { 0x66796925, 34 },
    { 0x12996926, 23 },
    { 0xdb996925, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199692b, 46 },
    { 0x2a29692c, 90 },
    { 0xb569692b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29196932, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4196936, 66 },
    { 0xd8e96937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369693a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8539693d, 43 },
    { 0x9a49693e, 98 },
    { 0, 0 },
    { 0xc0296940, 33 },
    { 0, 0 },
    { 0xb0f96942, 55 },
    { 0, 0 },
    { 0x6ea96944, 40 },
    { 0xae896945, 78 },
    { 0x84396945, 65 },
    { 0x9d96947, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409694c, 78 },
    { 0, 0 },
    { 0xb249694e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb96954, 23 },
    { 0x85796955, 66 },
    { 0x49496955, 90 },
    { 0, 0 },
    { 0x45c96958, 33 },
    { 0, 0 },
    { 0x96b9695a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70496960, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd96969, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d29696f, 66 },
    { 0xa0c96970, 65 },
    { 0x8596971, 40 },
    { 0x8e39696f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4696976, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d49697a, 55 },
    { 0x939697a, 55 },
    { 0x54f9697c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d96980, 81 },
    { 0, 0 },
    { 0xd6c96982, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c696989, 93 },
    { 0, 0 },
    { 0x3519698b, 33 },
    { 0, 0 },
    { 0x6a09698d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96994, 61 },
    { 0xa1396995, 61 },
    { 0x8a696995, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8379699b, 33 },
    { 0x2e49699c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9969a5, 61 },
    { 0x665969a6, 55 },
    { 0xb85969a7, 61 },
    { 0x842969a5, 43 },
    { 0xd2969a9, 76 },
    { 0, 0 },
    { 0x8b8969ab, 33 },
    { 0xce969ac, 23 },
    { 0xfef969ab, 46 },
    { 0, 0 },
    { 0xfc7969af, 34 },
    { 0x6ad969b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a969b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918969bc, 33 },
    { 0x8b6969bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x544969c0, 33 },
    { 0x8f6969c0, 43 },
    { 0x1e3969c1, 33 },
    { 0x6c7969c3, 33 },
    { 0xcf5969c4, 90 },
    { 0xfc5969c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1969c8, 55 },
    { 0xe2969c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99969cd, 33 },
    { 0x7a6969ce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb969d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870969d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0969d8, 33 },
    { 0x12c969d8, 61 },
    { 0, 0 },
    { 0x4969db, 43 },
    { 0, 0 },
    { 0x7d1969dd, 76 },
    { 0x191969dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1969e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1969ec, 43 },
    { 0x7c4969ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b969f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18969f4, 61 },
    { 0, 0 },
    { 0xead969f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274969fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c996a07, 90 },
    { 0, 0 },
    { 0xfd296a09, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c96a16, 90 },
    { 0x9d596a17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e496a1c, 98 },
    { 0xec96a1d, 90 },
    { 0x28996a1d, 76 },
    { 0xc3896a1f, 23 },
    { 0x61596a20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd96a23, 33 },
    { 0x55f96a24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac96a29, 33 },
    { 0x5a96a2a, 76 },
    { 0x2e796a2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5796a30, 2 },
    { 0x88e96a31, 91 },
    { 0x16496a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8896a36, 23 },
    { 0x77296a37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca896a3c, 87 },
    { 0, 0 },
    { 0xc5696a3e, 55 },
    { 0x22296a3f, 90 },
    { 0x99b96a3e, 95 },
    { 0xf6296a41, 61 },
    { 0xbc296a42, 52 },
    { 0x92a96a3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f96a4e, 83 },
    { 0, 0 },
    { 0x10d96a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf896a59, 4 },
    { 0xf096a5a, 33 },
    { 0x2b296a5a, 33 },
    { 0, 0 },
    { 0xc3396a5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0896a61, 90 },
    { 0, 0 },
    { 0x4d896a63, 84 },
    { 0xd7f96a64, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x52896a67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e196a6b, 66 },
    { 0x16796a6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b96a6f, 55 },
    { 0x35096a70, 55 },
    { 0, 0 },
    { 0x21396a72, 90 },
    { 0, 0 },
    { 0xb9996a74, 46 },
    { 0xda996a74, 90 },
    { 0x47496a76, 52 },
    { 0xbb096a75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d96a84, 99 },
    { 0, 0 },
    { 0x58a96a86, 67 },
    { 0x5696a87, 64 },
    { 0, 0 },
    { 0x74f96a89, 76 },
    { 0x88096a89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce96a8e, 95 },
    { 0x78f96a8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7996a92, 90 },
    { 0x7396a93, 46 },
    { 0x24d96a94, 90 },
    { 0, 0 },
    { 0xba496a96, 61 },
    { 0, 0 },
    { 0x59196a98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xba696a9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd96a9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc96aa4, 90 },
    { 0xe7a96aa5, 90 },
    { 0x5396aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6796aae, 17 },
    { 0, 0 },
    { 0x7d396ab0, 23 },
    { 0xcb096ab1, 90 },
    { 0xb1196ab2, 43 },
    { 0xb3196ab2, 100 },
    { 0x6f196ab4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17796ab8, 34 },
    { 0x22b96ab9, 40 },
    { 0x17996aba, 98 },
    { 0xabb96abb, 90 },
    { 0x2fa96ab9, 61 },
    { 0, 0 },
    { 0x27b96abe, 66 },
    { 0x1da96abf, 98 },
    { 0, 0 },
    { 0x60396ac1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78996ac6, 33 },
    { 0, 0 },
    { 0x23496ac8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab296acc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd96ad1, 23 },
    { 0, 0 },
    { 0xbc496ad3, 43 },
    { 0x71c96ad4, 90 },
    { 0, 0 },
    { 0x75896ad6, 87 },
    { 0x5fa96ad7, 46 },
    { 0x83b96ad8, 55 },
    { 0x8ab96ad8, 46 },
    { 0, 0 },
    { 0xd3496adb, 93 },
    { 0x2c496adc, 55 },
    { 0xeec96adc, 61 },
    { 0x13396ade, 33 },
    { 0, 0 },
    { 0x33996ae0, 61 },
    { 0x89d96ae1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd696ae4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e196ae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f296aeb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7896af1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55496af4, 46 },
    { 0, 0 },
    { 0x14d96af6, 23 },
    { 0xf8596af7, 95 },
    { 0xb5996af8, 90 },
    { 0x16b96af9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9596afd, 90 },
    { 0, 0 },
    { 0xb896aff, 67 },
    { 0, 0 },
    { 0x78596b01, 75 },
    { 0xd3f96b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c96b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f696b09, 90 },
    { 0, 0 },
    { 0x79596b0b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32196b0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8996b13, 33 },
    { 0xc6896b14, 81 },
    { 0x7cf96b14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30796b1d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c96b22, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c96b27, 76 },
    { 0x3bd96b28, 23 },
    { 0x48496b29, 87 },
    { 0xe1e96b28, 34 },
    { 0xef396b27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74496b36, 90 },
    { 0x8f696b37, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e96b3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c96b40, 61 },
    { 0x1a396b41, 46 },
    { 0x13496b41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1996b45, 34 },
    { 0xd2a96b46, 34 },
    { 0x3a596b47, 90 },
    { 0xb4796b48, 61 },
    { 0, 0 },
    { 0xe3596b4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d496b4d, 93 },
    { 0, 0 },
    { 0x2d796b4f, 55 },
    { 0x8d696b50, 95 },
    { 0x81396b51, 90 },
    { 0x15d96b50, 90 },
    { 0x39496b53, 98 },
    { 0x6df96b54, 98 },
    { 0xc096b55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee96b59, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb96b5e, 55 },
    { 0x80d96b5f, 34 },
    { 0x24996b60, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd96b64, 91 },
    { 0x21996b65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda096b75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23596b7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a96b7e, 97 },
    { 0xbbd96b7f, 90 },
    { 0x8e996b80, 61 },
    { 0, 0 },
    { 0x35796b82, 76 },
    { 0, 0 },
    { 0xc1b96b84, 36 },
    { 0x88496b84, 33 },
    { 0x2e496b86, 95 },
    { 0, 0 },
    { 0x8db96b88, 61 },
    { 0, 0 },
    { 0x92b96b8a, 90 },
    { 0x90496b8b, 33 },
    { 0, 0 },
    { 0x28e96b8d, 85 },
    { 0xb8c96b8d, 90 },
    { 0x4f296b8f, 61 },
    { 0xb4896b90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2296b94, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b796b99, 66 },
    { 0xde96b99, 76 },
    { 0xd9a96b9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b96b9f, 95 },
    { 0xec896ba0, 61 },
    { 0, 0 },
    { 0x7e596ba2, 46 },
    { 0xc1f96ba3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f396ba9, 61 },
    { 0x3fa96baa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c96bad, 77 },
    { 0x27996bae, 98 },
    { 0xbe96baf, 43 },
    { 0xda996baf, 33 },
    { 0xaf796bb1, 76 },
    { 0, 0 },
    { 0xe9f96bb3, 23 },
    { 0x19296bb4, 77 },
    { 0x61796bb4, 61 },
    { 0xefb96bb3, 43 },
    { 0x1bd96bb7, 84 },
    { 0, 0 },
    { 0x3ab96bb9, 90 },
    { 0, 0 },
    { 0xf3b96bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe696bbe, 66 },
    { 0xf5896bbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64996bc2, 90 },
    { 0, 0 },
    { 0x10d96bc4, 53 },
    { 0, 0 },
    { 0x11596bc6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d796bcd, 93 },
    { 0x8e896bcd, 66 },
    { 0x6a296bcf, 61 },
    { 0x45896bcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c96bd5, 88 },
    { 0xb6a96bd6, 90 },
    { 0x2e996bd5, 91 },
    { 0, 0 },
    { 0xa896bd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82096be1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xde396be4, 17 },
    { 0, 0 },
    { 0xd4396be6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4396beb, 33 },
    { 0xf7996bec, 90 },
    { 0, 0 },
    { 0x48e96bee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d196bf2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x50996bf5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a396bf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69096c03, 89 },
    { 0, 0 },
    { 0x4a696c05, 23 },
    { 0xab796c06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1696c09, 90 },
    { 0, 0 },
    { 0x94c96c0b, 34 },
    { 0xd4996c0c, 94 },
    { 0x93496c0d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f096c18, 89 },
    { 0x7e496c19, 23 },
    { 0x57996c18, 95 },
    { 0, 0 },
    { 0xe7096c1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76c96c20, 95 },
    { 0x8596c21, 67 },
    { 0, 0 },
    { 0x29496c23, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b096c28, 33 },
    { 0x15e96c28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xae296c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17596c35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f96c3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3396c3e, 93 },
    { 0, 0 },
    { 0x9f996c40, 61 },
    { 0, 0 },
    { 0xf2496c42, 90 },
    { 0xed96c43, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb096c47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50496c4f, 55 },
    { 0xd8b96c50, 61 },
    { 0xf8a96c51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c96c59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd196c5c, 43 },
    { 0, 0 },
    { 0x5bb96c5e, 33 },
    { 0xcbc96c5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf696c64, 90 },
    { 0xcce96c65, 90 },
    { 0x2af96c66, 95 },
    { 0x5d496c66, 43 },
    { 0x72b96c65, 43 },
    { 0xe8596c69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c196c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b696c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db96c73, 95 },
    { 0x1ea96c74, 90 },
    { 0x9d796c75, 64 },
    { 0, 0 },
    { 0x5af96c77, 84 },
    { 0x5a896c78, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f996c7d, 55 },
    { 0x38a96c7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab296c86, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d96c8b, 68 },
    { 0xb5096c8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4996c93, 98 },
    { 0x7f596c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e896c98, 95 },
    { 0, 0 },
    { 0x396c9a, 98 },
    { 0x67c96c9a, 93 },
    { 0x71b96c9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e996c9f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x72396ca2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37396ca6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2896caf, 33 },
    { 0, 0 },
    { 0x98496cb1, 99 },
    { 0xdc296cb1, 87 },
    { 0xebf96cb2, 87 },
    { 0, 0 },
    { 0xef596cb5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf296cbb, 33 },
    { 0xac196cbc, 23 },
    { 0xb7f96cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f296cc1, 93 },
    { 0x3e296cc2, 77 },
    { 0xd8196cc3, 95 },
    { 0, 0 },
    { 0xd6b96cc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc096cc9, 90 },
    { 0xf5d96cca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93296cd1, 76 },
    { 0x67496cd2, 94 },
    { 0x5ab96cd3, 98 },
    { 0xfea96cd4, 52 },
    { 0xce896cd3, 95 },
    { 0xbb796cd6, 75 },
    { 0x68496cd4, 90 },
    { 0x21596cd8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b96cdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9696ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe96ce5, 33 },
    { 0x53c96ce6, 47 },
    { 0xb1c96ce7, 78 },
    { 0xba396ce8, 97 },
    { 0xde596ce7, 98 },
    { 0, 0 },
    { 0xecb96ceb, 23 },
    { 0, 0 },
    { 0x4ce96ced, 55 },
    { 0x8ce96cee, 87 },
    { 0xb8396cef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf96cf2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea96cf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa96cf9, 55 },
    { 0xb3e96cfa, 34 },
    { 0, 0 },
    { 0x4f496cfc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a96d02, 84 },
    { 0xe8796d03, 23 },
    { 0, 0 },
    { 0x1d896d05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38996d08, 61 },
    { 0xd0496d09, 61 },
    { 0xa0196d09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff96d0e, 98 },
    { 0xffa96d0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a396d20, 90 },
    { 0, 0 },
    { 0x73496d22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13396d25, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e96d2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80296d2d, 75 },
    { 0x58c96d2d, 100 },
    { 0x77f96d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee296d32, 58 },
    { 0, 0 },
    { 0xfee96d34, 100 },
    { 0x8f596d35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e96d3b, 90 },
    { 0, 0 },
    { 0x2f96d3d, 76 },
    { 0x22c96d3e, 78 },
    { 0xcf796d3f, 98 },
    { 0x96d96d40, 90 },
    { 0, 0 },
    { 0xb2696d42, 76 },
    { 0, 0 },
    { 0x8ff96d44, 90 },
    { 0xc0596d44, 95 },
    { 0x12496d44, 88 },
    { 0, 0 },
    { 0x27f96d48, 65 },
    { 0, 0 },
    { 0xf7696d4a, 98 },
    { 0xcd96d4b, 34 },
    { 0xa2f96d4c, 33 },
    { 0, 0 },
    { 0xd5296d4e, 98 },
    { 0x78c96d4e, 83 },
    { 0, 0 },
    { 0x49796d51, 34 },
    { 0xd3396d51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f96d55, 33 },
    { 0x72f96d55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c96d59, 95 },
    { 0x296d59, 23 },
    { 0xbbf96d5b, 33 },
    { 0xe096d59, 90 },
    { 0x93796d5a, 95 },
    { 0, 0 },
    { 0xe6d96d5f, 61 },
    { 0x13496d60, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c96d66, 33 },
    { 0, 0 },
    { 0x8f896d68, 95 },
    { 0x28496d68, 68 },
    { 0x77896d6a, 90 },
    { 0x58596d6a, 80 },
    { 0x89f96d6c, 90 },
    { 0x89d96d6c, 61 },
    { 0x9c396d6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b96d75, 78 },
    { 0, 0 },
    { 0x92596d77, 90 },
    { 0xd3496d77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf396d7c, 61 },
    { 0xb0b96d7d, 33 },
    { 0xaa196d7e, 90 },
    { 0x69996d7e, 52 },
    { 0x95796d80, 67 },
    { 0x16d96d81, 99 },
    { 0x24f96d81, 90 },
    { 0x3f596d82, 90 },
    { 0x5dc96d84, 95 },
    { 0x1e196d84, 61 },
    { 0x58896d86, 64 },
    { 0, 0 },
    { 0x86d96d88, 61 },
    { 0xe7596d88, 33 },
    { 0xdf196d8a, 93 },
    { 0xb2296d8a, 61 },
    { 0xd0196d8b, 95 },
    { 0x4a296d8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x18496d90, 93 },
    { 0x7fa96d91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9496d95, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b696d9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94096da1, 61 },
    { 0x4fb96da1, 90 },
    { 0xa1e96da3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f496da9, 98 },
    { 0x20596da9, 61 },
    { 0, 0 },
    { 0x20c96dac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6996db0, 90 },
    { 0x13296db1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d296dbd, 90 },
    { 0x1b496dbd, 85 },
    { 0, 0 },
    { 0xe1a96dc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86596dc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d696dca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x59396dcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf96dd7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61296ddb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f496de5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2996deb, 23 },
    { 0, 0 },
    { 0xa0a96ded, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7396df1, 68 },
    { 0xa0196df1, 90 },
    { 0x5c696df2, 98 },
    { 0x41296df4, 17 },
    { 0xb0996df4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff96dfa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a96e04, 46 },
    { 0x45196e04, 100 },
    { 0x77696e06, 55 },
    { 0xb7c96e07, 90 },
    { 0x87296e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d96e0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c96e10, 93 },
    { 0, 0 },
    { 0xc3196e12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf996e15, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f96e18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a996e1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c496e1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b96e23, 85 },
    { 0x5fb96e24, 33 },
    { 0, 0 },
    { 0x79996e26, 78 },
    { 0x99b96e27, 23 },
    { 0, 0 },
    { 0x54096e29, 76 },
    { 0x8a796e2a, 90 },
    { 0xbe296e29, 55 },
    { 0xa0196e2a, 98 },
    { 0x60796e2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6296e30, 85 },
    { 0, 0 },
    { 0xbc396e32, 55 },
    { 0xd4796e33, 33 },
    { 0, 0 },
    { 0x2cd96e35, 66 },
    { 0xfd796e35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21796e3a, 34 },
    { 0x9b896e3b, 67 },
    { 0, 0 },
    { 0x14196e3d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b696e43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e696e50, 68 },
    { 0xcd196e50, 81 },
    { 0, 0 },
    { 0xb8b96e53, 61 },
    { 0xfb396e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95896e5b, 91 },
    { 0x3a996e5c, 61 },
    { 0x1e196e5c, 90 },
    { 0xc4e96e5e, 52 },
    { 0x1e296e5e, 33 },
    { 0xe0296e60, 66 },
    { 0xad496e60, 67 },
    { 0xc9596e5e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b596e6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53596e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d596e79, 46 },
    { 0xd7e96e7a, 33 },
    { 0x54a96e7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e396e81, 67 },
    { 0x1d496e82, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb696e85, 33 },
    { 0x7a896e86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b396e8e, 98 },
    { 0xbec96e8e, 90 },
    { 0, 0 },
    { 0x97a96e91, 33 },
    { 0x60896e91, 93 },
    { 0x61796e93, 46 },
    { 0xbd096e92, 98 },
    { 0, 0 },
    { 0xda396e96, 90 },
    { 0xcf996e96, 99 },
    { 0xda996e98, 90 },
    { 0x3e196e99, 95 },
    { 0x97896e9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c896ea0, 52 },
    { 0x3e596ea0, 46 },
    { 0x3b896ea1, 46 },
    { 0xc3d96ea3, 87 },
    { 0, 0 },
    { 0xe7d96ea5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc96eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d96eb0, 33 },
    { 0x6a596eb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3296eb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a896eb7, 55 },
    { 0x58296eb8, 99 },
    { 0, 0 },
    { 0xf2996eba, 90 },
    { 0xb8896ebb, 95 },
    { 0, 0 },
    { 0x66c96ebd, 33 },
    { 0x23896ebe, 52 },
    { 0x50c96ebf, 55 },
    { 0xa9996ec0, 33 },
    { 0, 0 },
    { 0x12896ec2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6896ec5, 97 },
    { 0, 0 },
    { 0x6a96ec7, 66 },
    { 0, 0 },
    { 0x61196ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e96ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3196ed0, 95 },
    { 0, 0 },
    { 0xfe096ed2, 90 },
    { 0, 0 },
    { 0x2a196ed4, 61 },
    { 0x16a96ed5, 98 },
    { 0, 0 },
    { 0xac096ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6996eda, 33 },
    { 0, 0 },
    { 0x29f96edc, 84 },
    { 0x20796edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1296ee3, 23 },
    { 0x6996ee4, 61 },
    { 0xcd096ee5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d96eea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8496ef4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2196ef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa96f06, 43 },
    { 0x6cc96f07, 76 },
    { 0xe3f96f08, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a96f0b, 66 },
    { 0x39996f0c, 93 },
    { 0xe9896f0c, 61 },
    { 0xb8396f0b, 43 },
    { 0xf8a96f0f, 69 },
    { 0x23396f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f96f19, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a796f1f, 90 },
    { 0, 0 },
    { 0xee96f21, 76 },
    { 0, 0 },
    { 0xda696f23, 95 },
    { 0xec596f24, 33 },
    { 0x28596f25, 61 },
    { 0xc1796f25, 77 },
    { 0x12396f27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a96f2d, 90 },
    { 0x46c96f2e, 34 },
    { 0xf6f96f2e, 100 },
    { 0x60f96f2e, 93 },
    { 0x50996f31, 33 },
    { 0xf4e96f31, 90 },
    { 0x20396f33, 46 },
    { 0x19d96f34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19496f3a, 90 },
    { 0x95396f3b, 33 },
    { 0xa3b96f3c, 68 },
    { 0, 0 },
    { 0x51696f3e, 90 },
    { 0xf4e96f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1596f44, 95 },
    { 0x4296f45, 61 },
    { 0xe96f45, 33 },
    { 0x61896f44, 93 },
    { 0x2d96f48, 95 },
    { 0x39996f49, 95 },
    { 0xe3a96f4a, 52 },
    { 0x58296f4a, 33 },
    { 0x2c96f4a, 23 },
    { 0xe1496f49, 91 },
    { 0x24496f4d, 98 },
    { 0x4be96f4f, 33 },
    { 0, 0 },
    { 0x32796f51, 55 },
    { 0, 0 },
    { 0x52b96f53, 34 },
    { 0, 0 },
    { 0x1a996f55, 43 },
    { 0x28e96f55, 84 },
    { 0, 0 },
    { 0xeed96f58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24696f5b, 77 },
    { 0xf0396f5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d996f61, 95 },
    { 0x29596f62, 76 },
    { 0xf5a96f63, 76 },
    { 0x5c696f64, 76 },
    { 0x52596f62, 44 },
    { 0x6b196f66, 33 },
    { 0xd7e96f66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61996f6e, 33 },
    { 0xd4196f6f, 76 },
    { 0x42796f70, 61 },
    { 0x6d796f6f, 34 },
    { 0xbf296f72, 55 },
    { 0x28796f73, 77 },
    { 0xa7296f73, 84 },
    { 0x7196f75, 87 },
    { 0xd3f96f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ac96f79, 93 },
    { 0x8ec96f7a, 93 },
    { 0, 0 },
    { 0xe9196f7c, 90 },
    { 0x59396f7d, 76 },
    { 0xff496f7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80f96f82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a96f8c, 98 },
    { 0x34d96f8d, 90 },
    { 0x3c696f8d, 87 },
    { 0xda796f8e, 77 },
    { 0xcfe96f8e, 90 },
    { 0, 0 },
    { 0x50a96f92, 34 },
    { 0xc9f96f93, 93 },
    { 0xc6196f92, 33 },
    { 0x6aa96f95, 52 },
    { 0x16096f92, 95 },
    { 0xc796f97, 33 },
    { 0xa3f96f98, 33 },
    { 0xa9b96f99, 61 },
    { 0xde996f9a, 34 },
    { 0x91496f93, 33 },
    { 0xbbc96f98, 33 },
    { 0, 0 },
    { 0x6696f9e, 46 },
    { 0x5ab96f9e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e96fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb96faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce96fad, 95 },
    { 0xcec96fae, 95 },
    { 0xca96faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcf96fb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc396fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20896fbb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f96fc4, 90 },
    { 0x17396fc4, 33 },
    { 0, 0 },
    { 0x6d196fc7, 87 },
    { 0, 0 },
    { 0x64096fc9, 61 },
    { 0, 0 },
    { 0xc796fcb, 100 },
    { 0xf7396fcc, 78 },
    { 0x7d96fcc, 55 },
    { 0, 0 },
    { 0xc8496fcf, 98 },
    { 0, 0 },
    { 0x7d396fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e96fd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d96fdc, 98 },
    { 0xc8f96fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b96fe1, 17 },
    { 0x60d96fe1, 95 },
    { 0x85a96fe2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8296fe8, 43 },
    { 0x82a96fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46f96fee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x22496ff1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94a96ff6, 68 },
    { 0, 0 },
    { 0xfd596ff8, 52 },
    { 0x98d96ff9, 98 },
    { 0x1a996ff8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f696ffd, 43 },
    { 0, 0 },
    { 0xe5d96fff, 34 },
    { 0, 0 },
    { 0x27e97001, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d97006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89700b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3897016, 76 },
    { 0x33a97017, 99 },
    { 0x6c297018, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa499701b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc39701f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73797028, 90 },
    { 0, 0 },
    { 0xa099702a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f097037, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b9703c, 80 },
    { 0x7ab9703d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55897040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e197044, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25897048, 38 },
    { 0, 0 },
    { 0xfe79704a, 84 },
    { 0, 0 },
    { 0x2649704c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb97052, 100 },
    { 0x3597053, 92 },
    { 0, 0 },
    { 0x39197055, 33 },
    { 0xd6897056, 90 },
    { 0x6f697057, 55 },
    { 0xb5397058, 78 },
    { 0x5b297058, 95 },
    { 0x8d897057, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e9705e, 98 },
    { 0, 0 },
    { 0xa9597060, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc97068, 76 },
    { 0, 0 },
    { 0x9279706a, 98 },
    { 0, 0 },
    { 0x84e9706c, 90 },
    { 0x68f9706c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6497071, 90 },
    { 0, 0 },
    { 0x6397073, 23 },
    { 0xf1f97074, 43 },
    { 0xaa597073, 90 },
    { 0, 0 },
    { 0x9a797077, 40 },
    { 0, 0 },
    { 0x7bc97079, 87 },
    { 0x3619707a, 43 },
    { 0x21697079, 33 },
    { 0xd7d9707c, 78 },
    { 0x18b9707c, 23 },
    { 0x34f97079, 33 },
    { 0x29b9707c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b697082, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f697086, 69 },
    { 0x19097087, 95 },
    { 0xa7597087, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf399708d, 84 },
    { 0, 0 },
    { 0xc569708f, 61 },
    { 0xff9708f, 90 },
    { 0x8399708f, 95 },
    { 0x379708f, 23 },
    { 0x4d897093, 46 },
    { 0xb6d97094, 90 },
    { 0xb0697095, 95 },
    { 0xe4497096, 90 },
    { 0xf9497096, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b970a9, 36 },
    { 0x8d0970aa, 91 },
    { 0, 0 },
    { 0xedf970ac, 98 },
    { 0, 0 },
    { 0x7af970ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x347970b1, 61 },
    { 0, 0 },
    { 0xf46970b3, 98 },
    { 0xef6970b4, 90 },
    { 0xba6970b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7970b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87970be, 33 },
    { 0, 0 },
    { 0x35b970c0, 61 },
    { 0, 0 },
    { 0x796970c2, 76 },
    { 0x9a0970c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c970c8, 23 },
    { 0, 0 },
    { 0x807970ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101970d0, 90 },
    { 0x2c5970d0, 34 },
    { 0xe18970d2, 33 },
    { 0xd09970d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1970db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5970e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeee970e9, 95 },
    { 0, 0 },
    { 0x488970eb, 95 },
    { 0x814970ec, 90 },
    { 0x3e4970ed, 33 },
    { 0x71b970ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a970f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4970f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da970f7, 39 },
    { 0, 0 },
    { 0x45a970f9, 33 },
    { 0x7ed970fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6897100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5397104, 33 },
    { 0xe8197105, 32 },
    { 0, 0 },
    { 0x1b797107, 23 },
    { 0x43697108, 61 },
    { 0, 0 },
    { 0x3079710a, 55 },
    { 0, 0 },
    { 0xe1b9710c, 90 },
    { 0, 0 },
    { 0x8349710e, 33 },
    { 0xbac9710e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f97114, 55 },
    { 0x29b97115, 98 },
    { 0xf3197114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9711c, 40 },
    { 0x6e99711c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d697121, 34 },
    { 0x20797122, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149712a, 33 },
    { 0, 0 },
    { 0x35f9712c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b897131, 34 },
    { 0xac997131, 76 },
    { 0, 0 },
    { 0xd5197134, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b97137, 90 },
    { 0x3c397138, 95 },
    { 0xef897138, 77 },
    { 0x4549713a, 90 },
    { 0xf4d97137, 61 },
    { 0xd249713b, 61 },
    { 0xf589713d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5197141, 93 },
    { 0x1a797142, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43897148, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa559714b, 66 },
    { 0xde59714c, 98 },
    { 0, 0 },
    { 0x9039714e, 33 },
    { 0x6d99714e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb97153, 78 },
    { 0, 0 },
    { 0xba897155, 87 },
    { 0x77697156, 23 },
    { 0x2dd97157, 100 },
    { 0x53497158, 90 },
    { 0, 0 },
    { 0x4c79715a, 55 },
    { 0xdd9715b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a97161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b97166, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569716c, 33 },
    { 0, 0 },
    { 0x8ad9716e, 76 },
    { 0, 0 },
    { 0x6a597170, 76 },
    { 0xea497171, 95 },
    { 0x7f697172, 61 },
    { 0x6c497170, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d097177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2499717a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82497180, 90 },
    { 0x3c197181, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xef597184, 98 },
    { 0, 0 },
    { 0xe3b97186, 61 },
    { 0xc0297187, 76 },
    { 0xfd797188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519718d, 95 },
    { 0xa829718d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e97191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c9719d, 23 },
    { 0x28d9719e, 98 },
    { 0x8a39719e, 43 },
    { 0, 0 },
    { 0x943971a1, 78 },
    { 0x24c971a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62971a5, 95 },
    { 0x1eb971a6, 46 },
    { 0, 0 },
    { 0x91e971a8, 66 },
    { 0xda4971a9, 17 },
    { 0x906971a9, 95 },
    { 0xef9971ab, 95 },
    { 0x9ad971ab, 90 },
    { 0x923971ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x606971b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e971b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf971b8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cc971bb, 61 },
    { 0xf50971bc, 98 },
    { 0x2c8971bd, 34 },
    { 0xcc9971be, 33 },
    { 0x669971bf, 98 },
    { 0xa39971bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7971c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90971c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd971d0, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167971d4, 33 },
    { 0x36d971d4, 43 },
    { 0x254971d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c971dd, 90 },
    { 0x372971dd, 34 },
    { 0x5e0971dd, 99 },
    { 0, 0 },
    { 0xb21971e1, 68 },
    { 0, 0 },
    { 0xda7971e3, 90 },
    { 0, 0 },
    { 0x214971e5, 95 },
    { 0, 0 },
    { 0x570971e7, 66 },
    { 0x7fb971e8, 75 },
    { 0, 0 },
    { 0x8ac971ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5971f1, 61 },
    { 0x5d5971f2, 65 },
    { 0xf36971f1, 76 },
    { 0, 0 },
    { 0xf4e971f5, 61 },
    { 0xbf5971f6, 98 },
    { 0x5b4971f7, 34 },
    { 0xab2971f6, 23 },
    { 0xa76971f5, 90 },
    { 0x427971f9, 37 },
    { 0xb53971fb, 33 },
    { 0x503971f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee9720b, 61 },
    { 0x68d9720c, 81 },
    { 0x5839720d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23797210, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17697215, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7639721a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e97224, 33 },
    { 0, 0 },
    { 0x29c97226, 78 },
    { 0xb4297226, 33 },
    { 0xc7e97227, 23 },
    { 0xb1297229, 33 },
    { 0xd269722a, 75 },
    { 0xfb69722b, 43 },
    { 0xc4e9722b, 46 },
    { 0, 0 },
    { 0xd659722e, 95 },
    { 0, 0 },
    { 0xc197230, 61 },
    { 0x13d97231, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31297236, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8199723d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea97240, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d97244, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f497247, 93 },
    { 0, 0 },
    { 0xaa97249, 95 },
    { 0x18a9724a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c9724e, 98 },
    { 0x3439724f, 88 },
    { 0xccf9724e, 98 },
    { 0xbc597251, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb97254, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe9725b, 43 },
    { 0xb659725c, 33 },
    { 0x2249725d, 33 },
    { 0xa5e9725e, 23 },
    { 0xd3d9725f, 43 },
    { 0, 0 },
    { 0x83697261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a9726b, 98 },
    { 0, 0 },
    { 0xdee9726d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b597270, 56 },
    { 0, 0 },
    { 0xa2097272, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x54397275, 91 },
    { 0xbea97276, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x38097279, 33 },
    { 0xcb69727a, 52 },
    { 0x6f79727b, 23 },
    { 0x7069727b, 95 },
    { 0x7d79727c, 61 },
    { 0xe929727a, 95 },
    { 0xb809727f, 98 },
    { 0x75097280, 46 },
    { 0x75797281, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e97286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8229728a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b9728f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6599729a, 99 },
    { 0, 0 },
    { 0x9c49729c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a39729f, 93 },
    { 0, 0 },
    { 0x467972a1, 34 },
    { 0xf37972a2, 90 },
    { 0xc08972a2, 23 },
    { 0, 0 },
    { 0x801972a5, 95 },
    { 0x30e972a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f972a9, 46 },
    { 0, 0 },
    { 0x2a3972ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea972af, 93 },
    { 0xcc8972af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816972b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68972b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e972bd, 55 },
    { 0xe09972be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c972c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81972ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f972d1, 78 },
    { 0x92f972d1, 65 },
    { 0, 0 },
    { 0xc12972d4, 90 },
    { 0xf70972d5, 88 },
    { 0x786972d4, 23 },
    { 0, 0 },
    { 0x54c972d8, 34 },
    { 0x851972d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c972dc, 58 },
    { 0xe19972dc, 65 },
    { 0x16f972de, 67 },
    { 0, 0 },
    { 0x921972e0, 76 },
    { 0x4ae972e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x807972e4, 84 },
    { 0x8f1972e5, 76 },
    { 0x8b3972e4, 87 },
    { 0xefc972e7, 55 },
    { 0xb1c972e4, 88 },
    { 0x847972e9, 95 },
    { 0xcce972e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52972ef, 95 },
    { 0x277972f0, 95 },
    { 0x67b972f1, 87 },
    { 0x49c972f2, 98 },
    { 0xcc8972f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34972f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a972fc, 55 },
    { 0xd81972fd, 90 },
    { 0x9b5972fd, 55 },
    { 0x152972ff, 46 },
    { 0x782972ff, 91 },
    { 0x2ad972ff, 23 },
    { 0, 0 },
    { 0x24e97303, 98 },
    { 0x6a097303, 90 },
    { 0xda397305, 53 },
    { 0x22497306, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5239730a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x289730d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b97314, 61 },
    { 0xef397314, 33 },
    { 0xd3397314, 43 },
    { 0x8e497314, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x659731a, 33 },
    { 0x4d49731b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb697322, 90 },
    { 0x12997323, 77 },
    { 0x83597323, 46 },
    { 0xf0b97325, 90 },
    { 0xf9797326, 95 },
    { 0x34597327, 90 },
    { 0xa7997328, 34 },
    { 0x17d97327, 34 },
    { 0x14397329, 33 },
    { 0x7b79732b, 100 },
    { 0xf869732b, 33 },
    { 0, 0 },
    { 0x9b89732e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a97332, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48397338, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed69733f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9734a, 90 },
    { 0x13c9734a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f39734e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd97353, 23 },
    { 0x78697354, 33 },
    { 0xc1e97355, 33 },
    { 0x65997355, 52 },
    { 0x47797356, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6319735c, 90 },
    { 0x719735d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34397360, 95 },
    { 0x13197360, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a09736a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8319736e, 98 },
    { 0x9439736f, 77 },
    { 0x56597370, 98 },
    { 0xa0d97370, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c9737a, 65 },
    { 0, 0 },
    { 0x6089737c, 55 },
    { 0x65c9737c, 83 },
    { 0x8ee9737c, 17 },
    { 0xfe89737c, 95 },
    { 0x7229737d, 33 },
    { 0x4697381, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f397385, 61 },
    { 0xb7997385, 90 },
    { 0x3b597387, 78 },
    { 0x82997386, 90 },
    { 0x10397389, 61 },
    { 0x7297385, 90 },
    { 0, 0 },
    { 0x889738c, 76 },
    { 0x8ae9738d, 33 },
    { 0, 0 },
    { 0x65f9738f, 55 },
    { 0, 0 },
    { 0xd5897391, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64197397, 95 },
    { 0x22a97398, 90 },
    { 0xa0d97398, 23 },
    { 0, 0 },
    { 0xbb79739b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4d973a9, 95 },
    { 0x445973a9, 90 },
    { 0xc33973ab, 33 },
    { 0x555973aa, 95 },
    { 0xc10973ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468973b2, 23 },
    { 0xba7973b3, 46 },
    { 0, 0 },
    { 0xca2973b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0973ba, 90 },
    { 0, 0 },
    { 0xbfc973bc, 46 },
    { 0x418973bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d973c4, 90 },
    { 0x513973c5, 23 },
    { 0x291973c4, 98 },
    { 0x395973c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f973cc, 75 },
    { 0x7ed973cd, 93 },
    { 0, 0 },
    { 0x740973cf, 9 },
    { 0xba5973d0, 90 },
    { 0xf32973d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990973d5, 87 },
    { 0, 0 },
    { 0xc08973d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429973df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3973e2, 61 },
    { 0xa45973e3, 90 },
    { 0xda2973e3, 98 },
    { 0x90a973e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a973e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e973ed, 90 },
    { 0xd1f973ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1973f1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358973f9, 90 },
    { 0x169973fa, 23 },
    { 0xb02973fb, 98 },
    { 0xce0973fa, 56 },
    { 0, 0 },
    { 0x80f973fe, 95 },
    { 0xa40973ff, 98 },
    { 0x4f397400, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65a97404, 95 },
    { 0, 0 },
    { 0xa0097406, 95 },
    { 0x94a97406, 100 },
    { 0xca597406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab9740b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf219740f, 23 },
    { 0xa3197410, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x88597413, 33 },
    { 0x7c397414, 45 },
    { 0x71e97415, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d9741f, 90 },
    { 0xa97420, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb97427, 61 },
    { 0xe1a97428, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c49742b, 90 },
    { 0xdd69742c, 78 },
    { 0xaf79742d, 81 },
    { 0x58e9742d, 23 },
    { 0xbdf9742b, 55 },
    { 0x40497430, 75 },
    { 0x8a097431, 99 },
    { 0x85197431, 76 },
    { 0xd797433, 94 },
    { 0xc149742d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37997438, 93 },
    { 0xb7797439, 76 },
    { 0x13897439, 90 },
    { 0x1e097438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b97444, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6397447, 34 },
    { 0xa8697448, 90 },
    { 0, 0 },
    { 0xf1e9744a, 76 },
    { 0x9759744a, 61 },
    { 0, 0 },
    { 0x81a9744d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d97450, 55 },
    { 0x8e697451, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1297454, 23 },
    { 0, 0 },
    { 0xadc97456, 90 },
    { 0x3f397457, 75 },
    { 0x6fc97458, 91 },
    { 0, 0 },
    { 0xbfc9745a, 65 },
    { 0x73b9745b, 61 },
    { 0xec99745c, 43 },
    { 0, 0 },
    { 0xfc39745e, 33 },
    { 0x7d9745f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c97463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e197467, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c09746c, 34 },
    { 0x98b9746c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3097471, 33 },
    { 0, 0 },
    { 0x71197473, 23 },
    { 0xcc097474, 87 },
    { 0xc7797475, 95 },
    { 0xa9197475, 76 },
    { 0x96b97477, 91 },
    { 0x63e97475, 61 },
    { 0x16997475, 43 },
    { 0x11597476, 43 },
    { 0x5eb9747b, 98 },
    { 0x7ad9747c, 90 },
    { 0xdaa9747d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f97480, 95 },
    { 0xea997480, 69 },
    { 0xfba97482, 90 },
    { 0, 0 },
    { 0x76a97484, 90 },
    { 0xa3e97484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d9748d, 90 },
    { 0x7409748e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0097494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8b97497, 93 },
    { 0x42a97497, 100 },
    { 0x64197499, 34 },
    { 0, 0 },
    { 0x1149749b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9749f, 23 },
    { 0xd819749f, 90 },
    { 0x4aa9749f, 87 },
    { 0x649974a2, 64 },
    { 0x72b9749f, 61 },
    { 0x4d69749f, 43 },
    { 0xfdc974a5, 84 },
    { 0xa57974a6, 66 },
    { 0x35f974a4, 95 },
    { 0x539974a8, 98 },
    { 0x3e9974a9, 34 },
    { 0x829974aa, 52 },
    { 0xdef974a8, 53 },
    { 0xcab974ac, 90 },
    { 0xc2e974ad, 61 },
    { 0xaba974a9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93974bb, 34 },
    { 0xce8974bc, 43 },
    { 0, 0 },
    { 0xe62974be, 52 },
    { 0xf50974bf, 61 },
    { 0x4b1974bf, 76 },
    { 0x815974be, 95 },
    { 0xc84974c2, 77 },
    { 0x7cd974c0, 33 },
    { 0, 0 },
    { 0x3f8974c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b974cc, 61 },
    { 0x696974cd, 77 },
    { 0, 0 },
    { 0x7cb974cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173974d3, 77 },
    { 0x455974d3, 64 },
    { 0x43974d5, 61 },
    { 0x1cc974d5, 88 },
    { 0x75f974d7, 61 },
    { 0x2b6974d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2974e5, 23 },
    { 0xb35974e5, 90 },
    { 0, 0 },
    { 0x3c974e8, 23 },
    { 0x816974e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b974f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d974f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56974fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee97502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf97507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c9750a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb097513, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e97516, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79751f, 33 },
    { 0x53f9751f, 95 },
    { 0x8bc97521, 78 },
    { 0x3c497522, 61 },
    { 0, 0 },
    { 0x32797524, 33 },
    { 0x3e697525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3999752c, 97 },
    { 0xaca9752d, 90 },
    { 0xc369752e, 100 },
    { 0x4d9752f, 33 },
    { 0xcba97530, 61 },
    { 0x2d19752d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65197535, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb9753a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb97541, 95 },
    { 0xc8197542, 33 },
    { 0x42e97542, 61 },
    { 0xc7c97542, 90 },
    { 0x18e97542, 33 },
    { 0, 0 },
    { 0x8ee97547, 95 },
    { 0xd6497548, 55 },
    { 0xca297548, 76 },
    { 0, 0 },
    { 0x139754b, 33 },
    { 0, 0 },
    { 0x34b9754d, 76 },
    { 0, 0 },
    { 0xb6a9754f, 34 },
    { 0xd4597550, 90 },
    { 0x7ac97551, 98 },
    { 0xb797550, 33 },
    { 0x83a97553, 100 },
    { 0, 0 },
    { 0xbbd97555, 68 },
    { 0x1ec97556, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d97559, 76 },
    { 0x3a79755a, 34 },
    { 0x30897559, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89755f, 34 },
    { 0xe9f97560, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e797567, 90 },
    { 0x56197568, 68 },
    { 0xc1197569, 98 },
    { 0xa6597568, 100 },
    { 0xb097568, 23 },
    { 0x33c9756c, 61 },
    { 0xa6b9756c, 76 },
    { 0, 0 },
    { 0x54e9756f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60997574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87197579, 33 },
    { 0x1d89757a, 56 },
    { 0, 0 },
    { 0x3bd9757c, 23 },
    { 0, 0 },
    { 0xb599757e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd197581, 76 },
    { 0, 0 },
    { 0xfe297583, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c997587, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e89758a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d9758d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e97592, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62097596, 33 },
    { 0x5ee97597, 33 },
    { 0x20697596, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd975a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f975aa, 98 },
    { 0, 0 },
    { 0xaec975ac, 95 },
    { 0x345975ad, 95 },
    { 0xae4975ae, 63 },
    { 0x650975af, 33 },
    { 0xe6d975b0, 61 },
    { 0x560975af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d975b4, 66 },
    { 0, 0 },
    { 0xe1b975b6, 33 },
    { 0xe75975b7, 33 },
    { 0xbbd975b8, 91 },
    { 0xeef975b7, 33 },
    { 0x33a975ba, 61 },
    { 0x5a8975bb, 61 },
    { 0xfe9975bc, 33 },
    { 0x215975bd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8975c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef975ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29975d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea975d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd975d9, 65 },
    { 0x26975da, 33 },
    { 0, 0 },
    { 0x187975dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x150975df, 99 },
    { 0xa76975e0, 95 },
    { 0xb3b975e0, 96 },
    { 0, 0 },
    { 0xf3a975e3, 90 },
    { 0xe66975e4, 23 },
    { 0x394975e5, 23 },
    { 0x9ea975e6, 98 },
    { 0xb5f975e7, 33 },
    { 0, 0 },
    { 0x7cc975e9, 33 },
    { 0xe01975ea, 84 },
    { 0xa7975ea, 23 },
    { 0, 0 },
    { 0x46f975ed, 90 },
    { 0, 0 },
    { 0x2fd975ef, 84 },
    { 0x7d3975f0, 55 },
    { 0xceb975ef, 33 },
    { 0x5e2975f1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf975f8, 23 },
    { 0xb49975f8, 77 },
    { 0, 0 },
    { 0x7e5975fb, 90 },
    { 0x5ca975fc, 90 },
    { 0x18e975fd, 90 },
    { 0x6f9975fe, 55 },
    { 0x695975ff, 90 },
    { 0x518975fd, 23 },
    { 0xba1975fd, 90 },
    { 0x5a4975ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5797606, 98 },
    { 0, 0 },
    { 0xf8697608, 93 },
    { 0xae197609, 34 },
    { 0x4c9760a, 98 },
    { 0x899760b, 66 },
    { 0, 0 },
    { 0x1ee9760d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88697614, 75 },
    { 0, 0 },
    { 0xeb497616, 98 },
    { 0, 0 },
    { 0x16c97618, 61 },
    { 0xa3097618, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739761d, 68 },
    { 0xc3c9761d, 95 },
    { 0xeeb9761f, 90 },
    { 0x3499761f, 98 },
    { 0x14b97621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9979762b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a97634, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x49897637, 81 },
    { 0xb2597638, 98 },
    { 0x55797639, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff9763c, 90 },
    { 0x88c9763d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a97644, 66 },
    { 0x5497644, 90 },
    { 0, 0 },
    { 0x96097647, 47 },
    { 0, 0 },
    { 0x7c097649, 93 },
    { 0x9d69764a, 76 },
    { 0x62d9764b, 90 },
    { 0x5f99764c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb219764f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb597652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea97656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1889765a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1239765d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e97660, 55 },
    { 0x6a597661, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f97668, 90 },
    { 0xc4e97669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e9766d, 33 },
    { 0x62f9766d, 64 },
    { 0xa979766f, 61 },
    { 0xf1f9766f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa97674, 33 },
    { 0, 0 },
    { 0x5cf97676, 95 },
    { 0xc1a97677, 23 },
    { 0, 0 },
    { 0xf8497679, 55 },
    { 0xed29767a, 95 },
    { 0x1fc9767b, 90 },
    { 0, 0 },
    { 0xe5c9767d, 23 },
    { 0xc869767e, 61 },
    { 0x58a9767f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a9768d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76597694, 90 },
    { 0, 0 },
    { 0x4f897696, 34 },
    { 0xe4997697, 90 },
    { 0xce997696, 99 },
    { 0, 0 },
    { 0x3339769a, 85 },
    { 0, 0 },
    { 0xf619769c, 90 },
    { 0xf1c9769c, 61 },
    { 0xfd29769d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f976a3, 39 },
    { 0xe27976a4, 90 },
    { 0xa03976a4, 98 },
    { 0x7d5976a6, 52 },
    { 0x4a4976a3, 98 },
    { 0x877976a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308976ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785976b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d976b6, 46 },
    { 0, 0 },
    { 0x782976b8, 95 },
    { 0xeb0976b9, 98 },
    { 0, 0 },
    { 0x1c5976bb, 23 },
    { 0, 0 },
    { 0xea976bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314976c3, 23 },
    { 0xee6976c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed976cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9976da, 90 },
    { 0x9d976da, 46 },
    { 0x49d976dc, 61 },
    { 0x215976da, 55 },
    { 0, 0 },
    { 0x352976df, 33 },
    { 0x1d4976e0, 33 },
    { 0xbdb976e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4e976e5, 90 },
    { 0x5df976e6, 100 },
    { 0xabc976e7, 61 },
    { 0, 0 },
    { 0x1ac976e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd976ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66f976f1, 91 },
    { 0x722976f2, 90 },
    { 0x2c9976f3, 90 },
    { 0x381976f4, 84 },
    { 0x1db976f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98976f8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15976ff, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98497703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf889770d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84397712, 23 },
    { 0, 0 },
    { 0x1f997714, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d97717, 95 },
    { 0xf7097717, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7839771b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf259771e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfe97722, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2597725, 23 },
    { 0x13e97726, 90 },
    { 0xbb997726, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa609772a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae397730, 76 },
    { 0, 0 },
    { 0xc4f97732, 61 },
    { 0, 0 },
    { 0xa6497734, 23 },
    { 0x56997735, 24 },
    { 0x74997735, 95 },
    { 0, 0 },
    { 0x10597738, 61 },
    { 0xf5a97739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19773c, 90 },
    { 0xa879773d, 74 },
    { 0xda09773e, 46 },
    { 0xf129773d, 43 },
    { 0xbb197740, 23 },
    { 0x41b9773e, 61 },
    { 0xaaf97742, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd697748, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a9774b, 76 },
    { 0xef79774c, 91 },
    { 0x7e29774b, 98 },
    { 0x7ed9774b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4197752, 33 },
    { 0x4e997753, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x47997756, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba697760, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55097766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2589776b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9776f, 90 },
    { 0xbc997770, 48 },
    { 0x62d97770, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0697777, 23 },
    { 0xd4e97777, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d9777d, 23 },
    { 0xc0c9777d, 93 },
    { 0xb3c9777d, 61 },
    { 0, 0 },
    { 0xece97781, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb097786, 95 },
    { 0x9c197787, 33 },
    { 0xbf897788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1459778b, 90 },
    { 0x3ae9778c, 81 },
    { 0, 0 },
    { 0x7dd9778e, 43 },
    { 0, 0 },
    { 0x60997790, 46 },
    { 0xaaf97790, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b97798, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a977a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08977ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420977b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c977b5, 61 },
    { 0, 0 },
    { 0x67c977b7, 90 },
    { 0x355977b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b977bc, 61 },
    { 0x912977bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793977c1, 23 },
    { 0xd67977c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426977c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92977ca, 43 },
    { 0x89b977ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745977d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa977da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46977e5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0977ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c977ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42977f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f977fb, 90 },
    { 0xef3977fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c97800, 75 },
    { 0x5ca97800, 61 },
    { 0, 0 },
    { 0xacc97803, 23 },
    { 0x1ef97804, 93 },
    { 0x5c97803, 55 },
    { 0, 0 },
    { 0x2ef97807, 93 },
    { 0, 0 },
    { 0x4b697809, 87 },
    { 0x9e29780a, 33 },
    { 0xfe597809, 90 },
    { 0x9139780a, 61 },
    { 0xcc797809, 33 },
    { 0x39e9780d, 23 },
    { 0, 0 },
    { 0x1cc97810, 43 },
    { 0, 0 },
    { 0xb4f97812, 23 },
    { 0, 0 },
    { 0x68397814, 61 },
    { 0x3f097815, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb497818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1409781b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df97823, 95 },
    { 0xbdb97824, 98 },
    { 0, 0 },
    { 0x2c797826, 61 },
    { 0xbb197826, 34 },
    { 0, 0 },
    { 0x48797829, 33 },
    { 0x88d97829, 61 },
    { 0x9e9782b, 53 },
    { 0x2fe97829, 98 },
    { 0, 0 },
    { 0x7019782e, 75 },
    { 0xbbe9782e, 33 },
    { 0x61897830, 95 },
    { 0xc7097831, 90 },
    { 0xfa897832, 100 },
    { 0x9bb97833, 98 },
    { 0x3a097834, 91 },
    { 0x8329782e, 33 },
    { 0x20a9782e, 61 },
    { 0, 0 },
    { 0x3db97838, 84 },
    { 0x8a997839, 95 },
    { 0x93297838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7797842, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e297847, 98 },
    { 0x2f597848, 61 },
    { 0xeee97849, 91 },
    { 0, 0 },
    { 0xdc49784b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d97850, 87 },
    { 0x2e797851, 61 },
    { 0xefe97851, 95 },
    { 0x21097852, 98 },
    { 0x17897854, 88 },
    { 0xcb297855, 61 },
    { 0xd3897856, 19 },
    { 0x73f97856, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa9785b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6769785e, 52 },
    { 0, 0 },
    { 0x44897860, 76 },
    { 0xcb197861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac97866, 88 },
    { 0, 0 },
    { 0x35597868, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9459786b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15597871, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea97874, 100 },
    { 0, 0 },
    { 0x7db97876, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e97879, 25 },
    { 0xc669787a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b49787d, 67 },
    { 0x9e99787e, 90 },
    { 0xbde9787f, 99 },
    { 0xeb29787e, 33 },
    { 0xdbf97881, 32 },
    { 0x32897882, 99 },
    { 0xfb97883, 90 },
    { 0, 0 },
    { 0x13c97885, 87 },
    { 0x14197886, 98 },
    { 0x46b97885, 23 },
    { 0, 0 },
    { 0x4f097889, 55 },
    { 0, 0 },
    { 0x4ed9788b, 33 },
    { 0, 0 },
    { 0xfd9788d, 33 },
    { 0xe7a9788d, 95 },
    { 0x5e49788f, 23 },
    { 0xd5097890, 83 },
    { 0, 0 },
    { 0x9e197892, 23 },
    { 0x61d97893, 33 },
    { 0, 0 },
    { 0x90797895, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58897899, 69 },
    { 0x52b9789a, 78 },
    { 0xea49789a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce978a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c978ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91978b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4978b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9978ba, 55 },
    { 0x444978ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bc978be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11978c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb978cc, 55 },
    { 0x82f978cd, 61 },
    { 0x4c4978ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9978d1, 43 },
    { 0, 0 },
    { 0x4de978d3, 90 },
    { 0x604978d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02978d8, 23 },
    { 0x5ef978d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf978de, 34 },
    { 0xb6f978df, 66 },
    { 0x735978e0, 43 },
    { 0x26e978de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb978e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d978eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41978ef, 95 },
    { 0, 0 },
    { 0xdb4978f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519978f8, 33 },
    { 0x63a978f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53978fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a97902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24997910, 100 },
    { 0xc397910, 90 },
    { 0x10197912, 95 },
    { 0x83997911, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75997916, 55 },
    { 0xe8e97916, 33 },
    { 0, 0 },
    { 0xbd597919, 40 },
    { 0x6109791a, 67 },
    { 0, 0 },
    { 0xc29791c, 34 },
    { 0x37b9791d, 66 },
    { 0x8999791d, 61 },
    { 0, 0 },
    { 0xa1097920, 90 },
    { 0x3ca97920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f397924, 33 },
    { 0, 0 },
    { 0xb8897926, 61 },
    { 0xe8e97926, 95 },
    { 0x6fe97927, 98 },
    { 0x58f97926, 98 },
    { 0x4859792a, 83 },
    { 0x40b9792a, 34 },
    { 0, 0 },
    { 0xc729792d, 98 },
    { 0xe8f9792e, 61 },
    { 0, 0 },
    { 0x11b97930, 23 },
    { 0x6bc97931, 61 },
    { 0, 0 },
    { 0xccc97933, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af97936, 76 },
    { 0x5e397936, 98 },
    { 0x59797938, 90 },
    { 0xe4397937, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf859793f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f97946, 83 },
    { 0x62497947, 33 },
    { 0xd2a97948, 90 },
    { 0, 0 },
    { 0x7249794a, 55 },
    { 0xa5b9794b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b297951, 23 },
    { 0xfe897952, 76 },
    { 0x7c497952, 76 },
    { 0x5a297954, 61 },
    { 0x79b97955, 76 },
    { 0, 0 },
    { 0x5e497957, 23 },
    { 0x8c097958, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c39795d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0997962, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e197965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f97969, 23 },
    { 0x7a49796a, 61 },
    { 0, 0 },
    { 0xfde9796c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99796f, 61 },
    { 0x53497970, 43 },
    { 0x7a397970, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4197975, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8699797b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8197983, 33 },
    { 0, 0 },
    { 0x7fa97985, 90 },
    { 0x71b97986, 98 },
    { 0xbf397987, 100 },
    { 0x17397988, 61 },
    { 0x3ab97988, 66 },
    { 0xd2397988, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60997990, 61 },
    { 0xb8d97991, 61 },
    { 0, 0 },
    { 0xf9297993, 23 },
    { 0x5e997994, 61 },
    { 0xb9897995, 23 },
    { 0xa5997995, 98 },
    { 0xe9697997, 61 },
    { 0xffa97998, 33 },
    { 0x9a797999, 90 },
    { 0xd0397999, 23 },
    { 0x9029799b, 68 },
    { 0x5189799b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f9799f, 95 },
    { 0, 0 },
    { 0xd0979a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1979a5, 90 },
    { 0x87b979a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcf979ab, 46 },
    { 0x7cc979ac, 43 },
    { 0x268979ad, 93 },
    { 0x5d0979ad, 66 },
    { 0xc5f979af, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30979b5, 95 },
    { 0x995979b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e979b9, 88 },
    { 0x6bb979ba, 78 },
    { 0xdc5979ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b979c0, 94 },
    { 0x191979c1, 84 },
    { 0x1dd979c2, 95 },
    { 0xd92979c2, 23 },
    { 0xcc7979c4, 76 },
    { 0, 0 },
    { 0x1e3979c6, 90 },
    { 0x305979c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57979ca, 33 },
    { 0xa98979cb, 98 },
    { 0x9cc979cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25979d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2979d6, 91 },
    { 0xf06979d7, 90 },
    { 0x353979d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1979e0, 34 },
    { 0x1c5979e1, 69 },
    { 0xf3c979e2, 90 },
    { 0, 0 },
    { 0xd79979e4, 90 },
    { 0xa53979e5, 23 },
    { 0x798979e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e979e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7979ec, 52 },
    { 0, 0 },
    { 0x564979ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x850979f1, 98 },
    { 0xf87979f2, 55 },
    { 0x17f979f3, 43 },
    { 0xebb979f4, 74 },
    { 0x464979f2, 90 },
    { 0, 0 },
    { 0xa92979f7, 70 },
    { 0xbe8979f7, 66 },
    { 0x5ed979f7, 90 },
    { 0, 0 },
    { 0x316979fb, 46 },
    { 0xeb7979fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e997a00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d697a03, 55 },
    { 0, 0 },
    { 0x22097a05, 61 },
    { 0x3d997a06, 23 },
    { 0xe8997a06, 76 },
    { 0xc8197a08, 38 },
    { 0x2bd97a09, 76 },
    { 0, 0 },
    { 0xa7097a0b, 98 },
    { 0x51f97a0b, 100 },
    { 0x3b997a0b, 98 },
    { 0xb2197a0e, 90 },
    { 0xa397a0e, 90 },
    { 0x3f297a0f, 98 },
    { 0xddd97a11, 95 },
    { 0, 0 },
    { 0x45997a13, 88 },
    { 0x79097a13, 33 },
    { 0, 0 },
    { 0xd3a97a16, 40 },
    { 0, 0 },
    { 0xc8997a18, 93 },
    { 0, 0 },
    { 0xf6997a1a, 33 },
    { 0, 0 },
    { 0xc8d97a1c, 78 },
    { 0xdbc97a1d, 34 },
    { 0xeea97a1e, 23 },
    { 0xe0097a1f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31497a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6397a27, 55 },
    { 0x39497a28, 23 },
    { 0x35497a29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b497a2c, 61 },
    { 0, 0 },
    { 0x38597a2e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86797a37, 34 },
    { 0x60897a37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76597a3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d97a47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f697a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58e97a52, 95 },
    { 0xae97a53, 17 },
    { 0x5c597a53, 23 },
    { 0, 0 },
    { 0xc3197a56, 61 },
    { 0x42f97a57, 55 },
    { 0x8b097a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7397a5b, 95 },
    { 0, 0 },
    { 0xe1097a5d, 95 },
    { 0, 0 },
    { 0xe1697a5f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f97a62, 34 },
    { 0x80497a63, 100 },
    { 0x9ef97a63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85797a67, 84 },
    { 0xd797a68, 77 },
    { 0, 0 },
    { 0x75897a6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b97a6d, 76 },
    { 0x11b97a6e, 98 },
    { 0x4fa97a6f, 90 },
    { 0xae897a70, 78 },
    { 0xb8997a70, 61 },
    { 0x7b797a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e297a77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1997a7b, 66 },
    { 0, 0 },
    { 0x98e97a7d, 61 },
    { 0, 0 },
    { 0xd6b97a7f, 76 },
    { 0, 0 },
    { 0xb6d97a81, 55 },
    { 0, 0 },
    { 0xec197a83, 55 },
    { 0xdfe97a84, 90 },
    { 0, 0 },
    { 0x82297a86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b97a8b, 61 },
    { 0xfbd97a8c, 55 },
    { 0x30397a8c, 95 },
    { 0x2d197a8b, 61 },
    { 0x78c97a8d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a97a92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f97a98, 34 },
    { 0x9e797a98, 33 },
    { 0, 0 },
    { 0x89897a9b, 67 },
    { 0, 0 },
    { 0x50797a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57097aa3, 23 },
    { 0xe0f97aa4, 61 },
    { 0, 0 },
    { 0x46c97aa6, 95 },
    { 0x58f97aa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70497aae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd297ab5, 61 },
    { 0, 0 },
    { 0xccd97ab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b97abb, 43 },
    { 0, 0 },
    { 0xf4397abd, 55 },
    { 0x1fe97abd, 61 },
    { 0x21597abd, 96 },
    { 0x13697abf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a97ac4, 33 },
    { 0xe1f97ac5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7697ac9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c197ace, 23 },
    { 0x8dd97ace, 23 },
    { 0, 0 },
    { 0xc0797ad1, 69 },
    { 0, 0 },
    { 0xa4a97ad3, 68 },
    { 0xe0797ad4, 91 },
    { 0x46897ad5, 95 },
    { 0x39997ad6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f97ad9, 93 },
    { 0xdc297ada, 61 },
    { 0x5f597adb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d97ade, 33 },
    { 0x44f97ade, 23 },
    { 0xcfd97ade, 23 },
    { 0, 0 },
    { 0xce197ae2, 23 },
    { 0xc3497ae3, 98 },
    { 0xe9f97ae3, 17 },
    { 0x9f397ae5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc97ae9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f97af1, 98 },
    { 0x48e97af2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73097af8, 93 },
    { 0xe2797af9, 95 },
    { 0x70297af9, 90 },
    { 0x24797afb, 65 },
    { 0xb9b97afc, 19 },
    { 0xd0d97afc, 85 },
    { 0x4fe97afe, 33 },
    { 0xb6597afc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed397b03, 34 },
    { 0x6f97b04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce197b07, 95 },
    { 0x5b97b08, 76 },
    { 0x6af97b08, 34 },
    { 0x5e997b07, 95 },
    { 0, 0 },
    { 0x15397b0c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3597b11, 87 },
    { 0x12897b12, 68 },
    { 0, 0 },
    { 0x24297b14, 33 },
    { 0x10997b14, 53 },
    { 0xad97b16, 82 },
    { 0, 0 },
    { 0x4af97b18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8597b1c, 61 },
    { 0x84a97b1d, 80 },
    { 0, 0 },
    { 0xc9397b1f, 61 },
    { 0, 0 },
    { 0x1c597b21, 95 },
    { 0, 0 },
    { 0x4be97b23, 66 },
    { 0, 0 },
    { 0xfeb97b25, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8597b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea97b2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a697b35, 90 },
    { 0, 0 },
    { 0xd2297b37, 76 },
    { 0xb0897b38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c697b3b, 23 },
    { 0x42a97b3c, 90 },
    { 0, 0 },
    { 0x99497b3e, 52 },
    { 0x9a197b3f, 88 },
    { 0xa4897b3f, 98 },
    { 0x33497b41, 34 },
    { 0x2c097b42, 90 },
    { 0xd2797b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a297b46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e97b4a, 90 },
    { 0x6b297b4a, 90 },
    { 0, 0 },
    { 0xf597b4d, 33 },
    { 0x12597b4e, 99 },
    { 0x64f97b4f, 33 },
    { 0, 0 },
    { 0x6b897b51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a97b56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28a97b59, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9097b5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e97b60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7797b64, 67 },
    { 0x2cc97b65, 77 },
    { 0x16a97b65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3197b6a, 98 },
    { 0, 0 },
    { 0x7cf97b6c, 98 },
    { 0x87697b6c, 66 },
    { 0x12697b6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7497b71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a97b75, 91 },
    { 0xa2497b76, 33 },
    { 0, 0 },
    { 0x4c097b78, 33 },
    { 0x4a497b79, 98 },
    { 0, 0 },
    { 0x5fd97b7b, 61 },
    { 0xae897b7c, 33 },
    { 0, 0 },
    { 0x8c597b7e, 33 },
    { 0x5f297b7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb97b82, 76 },
    { 0x69797b83, 91 },
    { 0xabd97b83, 53 },
    { 0, 0 },
    { 0xddd97b86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29597b89, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc97b8d, 34 },
    { 0x13197b8d, 90 },
    { 0xc1f97b8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f97b92, 90 },
    { 0x7b297b93, 76 },
    { 0xf4a97b93, 90 },
    { 0x22f97b92, 51 },
    { 0x58597b93, 33 },
    { 0, 0 },
    { 0x4f97b98, 96 },
    { 0xe297b99, 95 },
    { 0xa0d97b9a, 85 },
    { 0xc1c97b9a, 23 },
    { 0x38f97b9c, 88 },
    { 0, 0 },
    { 0x7b597b9e, 93 },
    { 0x95297b9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a97ba4, 99 },
    { 0, 0 },
    { 0xdcb97ba6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5497bac, 76 },
    { 0x57c97bac, 95 },
    { 0, 0 },
    { 0xce997baf, 76 },
    { 0, 0 },
    { 0x95d97bb1, 66 },
    { 0x59e97bb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b597bb6, 90 },
    { 0, 0 },
    { 0xe5b97bb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f97bc0, 23 },
    { 0, 0 },
    { 0x5a997bc2, 90 },
    { 0xefc97bc3, 85 },
    { 0x62c97bc3, 98 },
    { 0, 0 },
    { 0x48097bc6, 90 },
    { 0x7ca97bc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b97bcb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36097bd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45897bd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76797bd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f997bdd, 87 },
    { 0xb3897bdd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c697be1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e97be5, 84 },
    { 0, 0 },
    { 0xae597be7, 95 },
    { 0x16d97be7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21297bf0, 61 },
    { 0, 0 },
    { 0x80097bf2, 23 },
    { 0x31097bf3, 40 },
    { 0xf7c97bf3, 61 },
    { 0x38797bf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f97bf9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0297bfc, 90 },
    { 0x19b97bfc, 33 },
    { 0, 0 },
    { 0x5be97bff, 80 },
    { 0, 0 },
    { 0x53097c01, 33 },
    { 0, 0 },
    { 0xd6297c03, 90 },
    { 0x15a97c03, 33 },
    { 0, 0 },
    { 0xc0197c06, 88 },
    { 0xb3497c07, 98 },
    { 0, 0 },
    { 0xd2e97c09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c97c0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9297c0f, 52 },
    { 0x11e97c10, 90 },
    { 0xc7397c11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d397c18, 23 },
    { 0xb2897c18, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f997c1f, 76 },
    { 0, 0 },
    { 0x89397c21, 61 },
    { 0, 0 },
    { 0x2d097c23, 43 },
    { 0, 0 },
    { 0xc4c97c25, 33 },
    { 0x88997c25, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xac897c29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c97c2f, 61 },
    { 0, 0 },
    { 0xed997c31, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de97c3d, 100 },
    { 0, 0 },
    { 0xf0597c3f, 33 },
    { 0, 0 },
    { 0xe3597c41, 33 },
    { 0xd5497c42, 98 },
    { 0x6b797c43, 51 },
    { 0x43797c44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f97c4a, 90 },
    { 0xb6b97c4a, 90 },
    { 0x28b97c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f97c50, 90 },
    { 0xd1697c51, 23 },
    { 0x1fb97c52, 90 },
    { 0x36c97c53, 90 },
    { 0x56a97c53, 94 },
    { 0x87b97c55, 78 },
    { 0x44497c54, 90 },
    { 0x89f97c55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x35497c5a, 33 },
    { 0x8b297c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0897c61, 90 },
    { 0x6f197c61, 98 },
    { 0xb6297c63, 23 },
    { 0xbb197c64, 90 },
    { 0x14b97c62, 95 },
    { 0xa3c97c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4497c6b, 46 },
    { 0x22f97c6c, 87 },
    { 0x1dc97c6d, 93 },
    { 0x7be97c6d, 100 },
    { 0xa1a97c6f, 32 },
    { 0x26c97c70, 33 },
    { 0xe0e97c71, 33 },
    { 0x9a497c6d, 95 },
    { 0xcfd97c6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde197c7b, 46 },
    { 0, 0 },
    { 0xbc397c7d, 98 },
    { 0x68497c7e, 61 },
    { 0xfbf97c7f, 33 },
    { 0x43e97c7f, 23 },
    { 0x8db97c81, 23 },
    { 0xf097c81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb97c86, 67 },
    { 0, 0 },
    { 0x82b97c88, 55 },
    { 0x4fb97c88, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x47897c8c, 98 },
    { 0, 0 },
    { 0x7d097c8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64397c91, 98 },
    { 0, 0 },
    { 0x24897c93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18597c96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a997c99, 61 },
    { 0x52197c9a, 33 },
    { 0x55697c9b, 91 },
    { 0xf7897c9c, 61 },
    { 0x69397c9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1197ca0, 99 },
    { 0, 0 },
    { 0x71797ca2, 39 },
    { 0, 0 },
    { 0x12297ca4, 98 },
    { 0, 0 },
    { 0x75297ca6, 90 },
    { 0xed697ca7, 98 },
    { 0x3c097ca6, 46 },
    { 0, 0 },
    { 0xfb297caa, 90 },
    { 0xa4297caa, 90 },
    { 0x2c97cab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37397cb0, 61 },
    { 0xe8d97cb0, 93 },
    { 0x59797cb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67297cbc, 34 },
    { 0xbf597cbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e97cc2, 90 },
    { 0, 0 },
    { 0x58997cc4, 53 },
    { 0x83a97cc5, 100 },
    { 0xacc97cc6, 64 },
    { 0x8797cc7, 91 },
    { 0x4e97cc4, 23 },
    { 0xd2697cc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a597ccc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc97ccf, 61 },
    { 0x3d497ccf, 61 },
    { 0xf5c97cd1, 85 },
    { 0xdb497cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d97cd7, 90 },
    { 0xcfb97cd7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3997cdb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a697ce5, 75 },
    { 0x8f697ce6, 33 },
    { 0x2b497ce6, 33 },
    { 0xbe397ce7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a97ceb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c97cf2, 43 },
    { 0xdfc97cf3, 33 },
    { 0xfb797cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7397cfd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef97d02, 33 },
    { 0, 0 },
    { 0xfad97d04, 38 },
    { 0x29197d05, 66 },
    { 0x6197d06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf97d0c, 34 },
    { 0, 0 },
    { 0x14397d0e, 61 },
    { 0x7b497d0e, 61 },
    { 0x7ef97d10, 93 },
    { 0x2d697d11, 90 },
    { 0x7be97d12, 91 },
    { 0x57797d13, 33 },
    { 0, 0 },
    { 0x72097d15, 95 },
    { 0x96397d16, 77 },
    { 0x62597d17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6297d1b, 76 },
    { 0x3d797d1b, 23 },
    { 0x8bf97d1d, 90 },
    { 0x7497d1e, 66 },
    { 0x76a97d1e, 90 },
    { 0xdde97d20, 33 },
    { 0xdea97d20, 55 },
    { 0x3997d20, 46 },
    { 0xd2c97d1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f497d27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56297d2b, 23 },
    { 0, 0 },
    { 0x85f97d2d, 61 },
    { 0x43e97d2e, 76 },
    { 0, 0 },
    { 0x85597d30, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5997d35, 26 },
    { 0x97697d36, 33 },
    { 0x11e97d35, 76 },
    { 0x5a697d36, 55 },
    { 0, 0 },
    { 0xb8d97d3a, 36 },
    { 0x6d697d3b, 90 },
    { 0xd5997d3a, 98 },
    { 0xf8c97d3b, 43 },
    { 0x39497d3c, 23 },
    { 0xf7c97d3f, 61 },
    { 0x59897d3d, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d97d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea097d47, 61 },
    { 0, 0 },
    { 0x55c97d49, 61 },
    { 0xac397d49, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0097d51, 75 },
    { 0xb0897d51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c797d55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69997d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32b97d62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c697d65, 100 },
    { 0, 0 },
    { 0xfc097d67, 95 },
    { 0, 0 },
    { 0xdb997d69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a97d6f, 17 },
    { 0, 0 },
    { 0x1d297d71, 98 },
    { 0x98097d72, 81 },
    { 0, 0 },
    { 0xfe997d74, 91 },
    { 0xae597d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b97d7a, 98 },
    { 0x2b697d7b, 61 },
    { 0x44697d7c, 76 },
    { 0x4ab97d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab97d81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18897d84, 61 },
    { 0x87c97d85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe97d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc97d8b, 84 },
    { 0x45a97d8c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1797d94, 43 },
    { 0x62b97d95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e897d99, 61 },
    { 0x20697d9a, 76 },
    { 0, 0 },
    { 0xdde97d9c, 90 },
    { 0, 0 },
    { 0xf7497d9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4097da3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13997da8, 90 },
    { 0xfde97da8, 43 },
    { 0x7b797daa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b397dae, 23 },
    { 0xe0e97daf, 61 },
    { 0x5ac97db0, 99 },
    { 0, 0 },
    { 0x97497db2, 67 },
    { 0xee697db2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe197db8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60797dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc997dcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d97dcf, 43 },
    { 0, 0 },
    { 0x7297dd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b97dd7, 42 },
    { 0x1f097dd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e97ddd, 23 },
    { 0xadc97ddd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e97de6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xee97de9, 66 },
    { 0xcae97dea, 61 },
    { 0x34597deb, 66 },
    { 0x73c97dec, 98 },
    { 0x25a97dea, 76 },
    { 0xa4b97dec, 23 },
    { 0x1ea97def, 95 },
    { 0xd7297df0, 33 },
    { 0xe2097df0, 95 },
    { 0x36f97df2, 33 },
    { 0x2ad97df0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa697df6, 98 },
    { 0x38397df7, 90 },
    { 0x23b97df8, 87 },
    { 0xe4197df9, 64 },
    { 0xdfd97df8, 34 },
    { 0x7ce97df7, 100 },
    { 0, 0 },
    { 0x5f97dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed597e04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b997e07, 95 },
    { 0, 0 },
    { 0x80997e09, 61 },
    { 0, 0 },
    { 0xcee97e0b, 90 },
    { 0x49297e0c, 34 },
    { 0xfe797e0b, 82 },
    { 0xe9997e0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b197e17, 33 },
    { 0xd9e97e18, 23 },
    { 0xf8c97e17, 98 },
    { 0, 0 },
    { 0x70697e1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb97e1f, 76 },
    { 0xfdf97e20, 33 },
    { 0xbb497e21, 76 },
    { 0x3c997e22, 85 },
    { 0, 0 },
    { 0xd7097e24, 33 },
    { 0x45097e25, 98 },
    { 0x61597e25, 61 },
    { 0, 0 },
    { 0x2b497e28, 78 },
    { 0x8c97e29, 90 },
    { 0, 0 },
    { 0xf9f97e2b, 61 },
    { 0, 0 },
    { 0x85497e2d, 93 },
    { 0xc897e2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3997e32, 17 },
    { 0, 0 },
    { 0xd9b97e34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6c97e37, 90 },
    { 0, 0 },
    { 0xf8d97e39, 55 },
    { 0x77c97e3a, 23 },
    { 0, 0 },
    { 0x90b97e3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80797e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89597e42, 34 },
    { 0xe3a97e43, 33 },
    { 0x34297e44, 61 },
    { 0, 0 },
    { 0x17397e46, 98 },
    { 0x9797e46, 33 },
    { 0xa3f97e46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9597e4b, 43 },
    { 0x7e197e4b, 43 },
    { 0x61497e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf497e52, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x87297e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef97e58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4697e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd997e5f, 40 },
    { 0x8697e60, 46 },
    { 0, 0 },
    { 0xccc97e62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f297e6d, 51 },
    { 0xa1997e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d897e71, 55 },
    { 0x86b97e71, 90 },
    { 0, 0 },
    { 0xc2e97e74, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f97e7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c097e7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f97e87, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc997e8a, 45 },
    { 0, 0 },
    { 0x63397e8c, 84 },
    { 0x4fb97e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66397e92, 34 },
    { 0, 0 },
    { 0x36997e94, 48 },
    { 0, 0 },
    { 0x57f97e96, 95 },
    { 0x3da97e97, 52 },
    { 0, 0 },
    { 0xee397e99, 46 },
    { 0x2997e9a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc97e9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xef297ea1, 33 },
    { 0, 0 },
    { 0x22d97ea3, 23 },
    { 0xba97ea4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f97ea7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4497eac, 66 },
    { 0x3cf97ead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55797eb0, 90 },
    { 0, 0 },
    { 0xca397eb2, 23 },
    { 0, 0 },
    { 0x7d397eb4, 100 },
    { 0xad797eb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe697ebd, 78 },
    { 0x86497ebe, 76 },
    { 0xb4197ebf, 98 },
    { 0x14c97ec0, 100 },
    { 0x46d97ec1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d97ec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa97ec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43897ecb, 17 },
    { 0xbb097ecb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7797ed4, 33 },
    { 0, 0 },
    { 0x1ce97ed6, 78 },
    { 0xe6f97ed6, 23 },
    { 0x36f97ed7, 33 },
    { 0x5b297ed6, 98 },
    { 0xe5097eda, 46 },
    { 0x17897ed7, 95 },
    { 0x3bd97eda, 46 },
    { 0x25e97edd, 34 },
    { 0xaf397ede, 23 },
    { 0x4c897ed7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2197ee3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f97ee6, 46 },
    { 0x1eb97ee7, 43 },
    { 0x1ea97ee8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40997eee, 67 },
    { 0x6a597eee, 93 },
    { 0x4a97ef0, 61 },
    { 0xab197ef1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2697ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18397ef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d97f0f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c397f1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6197f20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23497f23, 33 },
    { 0xdee97f23, 98 },
    { 0, 0 },
    { 0x10297f26, 61 },
    { 0x4c097f27, 100 },
    { 0x38b97f28, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b497f2b, 90 },
    { 0x90797f2b, 46 },
    { 0, 0 },
    { 0x8a997f2e, 46 },
    { 0, 0 },
    { 0x9c497f30, 23 },
    { 0x9a397f31, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93397f36, 45 },
    { 0x44f97f36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd97f3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98397f44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e97f47, 23 },
    { 0xac597f48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47697f4c, 85 },
    { 0x49797f4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb597f55, 43 },
    { 0xe3697f56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee097f5c, 93 },
    { 0x22c97f5d, 75 },
    { 0xee297f5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf97f62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a997f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a397f6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d97f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49097f74, 23 },
    { 0x7e597f74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb197f79, 95 },
    { 0x20d97f7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb97f80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9197f87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f197f8c, 90 },
    { 0, 0 },
    { 0xc3797f8e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d97f9b, 95 },
    { 0x46297f9c, 34 },
    { 0x76b97f9b, 98 },
    { 0, 0 },
    { 0x15b97f9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48097fa2, 55 },
    { 0, 0 },
    { 0x83897fa4, 33 },
    { 0x8f497fa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e97fa8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5797fac, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4597fb1, 88 },
    { 0x4f697fb1, 61 },
    { 0x77097fb1, 43 },
    { 0, 0 },
    { 0x20697fb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14397fba, 76 },
    { 0, 0 },
    { 0xb0997fbc, 100 },
    { 0xfbe97fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b97fc2, 33 },
    { 0, 0 },
    { 0x94f97fc4, 69 },
    { 0, 0 },
    { 0xe0897fc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97fc9, 61 },
    { 0x38597fca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e97fce, 95 },
    { 0x8af97fce, 68 },
    { 0x2897fd0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x12397fd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1897fde, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb797fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84497fe8, 75 },
    { 0, 0 },
    { 0xe8397fea, 33 },
    { 0x79e97fea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d97ff4, 33 },
    { 0x8af97ff5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x99797ff8, 95 },
    { 0, 0 },
    { 0xfa797ffa, 55 },
    { 0xfe897ffb, 98 },
    { 0xd7497ffb, 100 },
    { 0xa597ffd, 23 },
    { 0xf8397ffd, 33 },
    { 0x1cb97fff, 55 },
    { 0, 0 },
    { 0x29498001, 33 },
    { 0, 0 },
    { 0x3d098003, 61 },
    { 0x4b398004, 76 },
    { 0x48c98005, 85 },
    { 0x18298005, 59 },
    { 0x4598007, 61 },
    { 0xe4f98008, 23 },
    { 0xb2d98008, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41198010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee98013, 88 },
    { 0x72998014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d298018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd39801e, 33 },
    { 0xccc9801f, 62 },
    { 0, 0 },
    { 0x72498021, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd98029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b9802e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1998031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd249803c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa19803f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45998044, 90 },
    { 0x5c98045, 90 },
    { 0xdd698044, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29804d, 66 },
    { 0, 0 },
    { 0xa439804f, 77 },
    { 0, 0 },
    { 0xae798051, 33 },
    { 0, 0 },
    { 0x77f98053, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab798056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9805a, 61 },
    { 0, 0 },
    { 0x8359805c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a39805f, 61 },
    { 0xdde9805f, 68 },
    { 0xb7098061, 34 },
    { 0xc1498060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5798065, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9806b, 23 },
    { 0, 0 },
    { 0xa99806d, 98 },
    { 0, 0 },
    { 0xb3b9806f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73298074, 78 },
    { 0xe0298074, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d9807b, 36 },
    { 0, 0 },
    { 0xe9c9807d, 95 },
    { 0xd9e9807e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66898086, 93 },
    { 0x2c98087, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24998091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8798094, 33 },
    { 0, 0 },
    { 0xd6398096, 98 },
    { 0x8ed98096, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb99809a, 61 },
    { 0x64e9809a, 98 },
    { 0x3af9809c, 100 },
    { 0x6bb9809b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2980a0, 61 },
    { 0x909980a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49980a4, 95 },
    { 0xdd7980a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13980a8, 90 },
    { 0xac0980a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d980ad, 93 },
    { 0, 0 },
    { 0xf8f980af, 55 },
    { 0, 0 },
    { 0x555980b1, 93 },
    { 0x7cc980b1, 93 },
    { 0xfb9980b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7980ba, 83 },
    { 0, 0 },
    { 0x24f980bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83980c5, 91 },
    { 0, 0 },
    { 0xe6d980c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea980d1, 61 },
    { 0xef8980d2, 87 },
    { 0x1d4980d1, 76 },
    { 0, 0 },
    { 0x2ba980d5, 90 },
    { 0x23f980d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8980db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24980e0, 34 },
    { 0xe63980e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26e980e4, 95 },
    { 0, 0 },
    { 0x31d980e6, 33 },
    { 0, 0 },
    { 0x4a1980e8, 33 },
    { 0xb46980e9, 66 },
    { 0xe21980ea, 23 },
    { 0xad7980eb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4980ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0980fa, 98 },
    { 0, 0 },
    { 0xdce980fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe98100, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef398103, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a698109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19810c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c98110, 61 },
    { 0, 0 },
    { 0x35f98112, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0098118, 43 },
    { 0xb8c98118, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22e9811f, 100 },
    { 0, 0 },
    { 0xe6198121, 34 },
    { 0x7f698122, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d698126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e59812a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079812d, 76 },
    { 0, 0 },
    { 0xe979812f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a98135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4498139, 98 },
    { 0x9c89813a, 84 },
    { 0x79498139, 61 },
    { 0x35c98139, 34 },
    { 0x87c9813d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498143, 55 },
    { 0, 0 },
    { 0x5f98145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b9814d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9098152, 33 },
    { 0xed098153, 46 },
    { 0x28e98153, 87 },
    { 0x1cf98155, 98 },
    { 0, 0 },
    { 0xf6298157, 17 },
    { 0, 0 },
    { 0xd3f98159, 90 },
    { 0, 0 },
    { 0x2549815b, 84 },
    { 0x8a29815c, 98 },
    { 0xc959815c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4798161, 38 },
    { 0x2fc98162, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74198166, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46498170, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c98173, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e298177, 61 },
    { 0, 0 },
    { 0x71898179, 90 },
    { 0x39c9817a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb9817f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8698185, 33 },
    { 0x68b98185, 58 },
    { 0x16098187, 75 },
    { 0xc9f98186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae59818b, 90 },
    { 0x6bc9818b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2498190, 98 },
    { 0x18a98191, 91 },
    { 0x4b498191, 90 },
    { 0x90b98190, 84 },
    { 0x19898194, 61 },
    { 0x88098193, 43 },
    { 0x82198196, 98 },
    { 0, 0 },
    { 0xa7798198, 90 },
    { 0xde398198, 88 },
    { 0x5c29819a, 33 },
    { 0xdd39819b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3981a2, 67 },
    { 0xbbf981a3, 95 },
    { 0, 0 },
    { 0x43a981a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5981a8, 66 },
    { 0x4b2981a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac981ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b981b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574981bb, 55 },
    { 0x8a6981bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2981bf, 87 },
    { 0xd28981c0, 78 },
    { 0xdde981c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c981c5, 34 },
    { 0, 0 },
    { 0xf27981c7, 93 },
    { 0, 0 },
    { 0x913981c9, 84 },
    { 0xfb6981ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9981d1, 98 },
    { 0x431981d2, 33 },
    { 0x43d981d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f981d6, 61 },
    { 0x932981d7, 90 },
    { 0x175981d7, 90 },
    { 0x8ed981d8, 95 },
    { 0, 0 },
    { 0xb0c981db, 98 },
    { 0x4c981db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e981e1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62981e4, 17 },
    { 0x12c981e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f981e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d981ec, 55 },
    { 0, 0 },
    { 0x92c981ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3981f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19981f6, 23 },
    { 0xfba981f7, 66 },
    { 0xa61981f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x215981fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a98201, 90 },
    { 0x4d98202, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d298205, 75 },
    { 0, 0 },
    { 0xca898207, 55 },
    { 0, 0 },
    { 0xa0698209, 23 },
    { 0, 0 },
    { 0xa909820b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1919820f, 61 },
    { 0, 0 },
    { 0x27798211, 66 },
    { 0xd6998212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27598216, 33 },
    { 0xea698216, 55 },
    { 0, 0 },
    { 0xe5a98219, 90 },
    { 0x6589821a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a9821e, 90 },
    { 0, 0 },
    { 0xe1a98220, 55 },
    { 0x7aa98221, 61 },
    { 0xd9d98222, 76 },
    { 0x55198222, 95 },
    { 0x3698224, 76 },
    { 0x7f998221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe98229, 61 },
    { 0x4a99822a, 17 },
    { 0x3059822b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15298232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3398236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6259823a, 99 },
    { 0x9389823a, 90 },
    { 0, 0 },
    { 0x1689823d, 93 },
    { 0x1009823d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84498246, 98 },
    { 0x4aa98247, 33 },
    { 0x68098246, 33 },
    { 0x7ca98249, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5169824c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c98251, 51 },
    { 0, 0 },
    { 0x95798253, 76 },
    { 0, 0 },
    { 0x7e198255, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd998258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa349825e, 34 },
    { 0, 0 },
    { 0x73f98260, 90 },
    { 0xb2898261, 99 },
    { 0x83298262, 91 },
    { 0, 0 },
    { 0x89c98264, 76 },
    { 0x14c98264, 34 },
    { 0xd2398266, 10 },
    { 0x1bf98265, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c59826c, 95 },
    { 0x6999826d, 80 },
    { 0x9449826e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab98274, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab9827a, 17 },
    { 0xf1b9827b, 88 },
    { 0x54b9827b, 46 },
    { 0x2fa9827d, 33 },
    { 0xc799827e, 40 },
    { 0xbe49827e, 55 },
    { 0x56698280, 100 },
    { 0x8b9827f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd98288, 34 },
    { 0xb2698288, 90 },
    { 0x1669828a, 76 },
    { 0x5e798288, 66 },
    { 0x7b49828b, 33 },
    { 0, 0 },
    { 0xfb49828e, 55 },
    { 0x7af9828f, 80 },
    { 0xa5598290, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb598296, 99 },
    { 0x98b98297, 91 },
    { 0x34298298, 78 },
    { 0xc0d98299, 90 },
    { 0x9bf98296, 90 },
    { 0, 0 },
    { 0x8dd9829c, 98 },
    { 0xa49829d, 90 },
    { 0xa19829e, 78 },
    { 0xb479829e, 55 },
    { 0, 0 },
    { 0x402982a1, 23 },
    { 0x2a2982a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2982ab, 90 },
    { 0, 0 },
    { 0xec2982ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659982b2, 90 },
    { 0xbaa982b3, 61 },
    { 0xa87982b3, 61 },
    { 0x248982b3, 23 },
    { 0xc12982b2, 93 },
    { 0x7b8982b2, 98 },
    { 0x7a982b8, 95 },
    { 0, 0 },
    { 0xbc7982ba, 33 },
    { 0xa66982ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd982be, 90 },
    { 0xa46982bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce982c7, 90 },
    { 0x27a982c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x925982cb, 23 },
    { 0, 0 },
    { 0xaf8982cd, 23 },
    { 0xd60982ce, 81 },
    { 0xf9c982cd, 34 },
    { 0xcb6982ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267982d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100982da, 23 },
    { 0, 0 },
    { 0x934982dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87982df, 98 },
    { 0x2f7982e0, 78 },
    { 0, 0 },
    { 0xadd982e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681982e6, 95 },
    { 0xab8982e7, 23 },
    { 0x87f982e7, 33 },
    { 0x4d3982e9, 34 },
    { 0, 0 },
    { 0x2b0982eb, 84 },
    { 0, 0 },
    { 0xe17982ed, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4982f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2982f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a982fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd982ff, 61 },
    { 0xa5298300, 55 },
    { 0, 0 },
    { 0x3e198302, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca98305, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5939830a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe429830f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b898312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e98315, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f9831d, 23 },
    { 0, 0 },
    { 0x4609831f, 98 },
    { 0x89b9831f, 33 },
    { 0x8f29831f, 95 },
    { 0xa4198322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19698326, 23 },
    { 0x8bd98327, 90 },
    { 0x38f98326, 91 },
    { 0xe8a98328, 90 },
    { 0, 0 },
    { 0x87b9832b, 61 },
    { 0xbf19832c, 33 },
    { 0xa859832b, 33 },
    { 0x4499832e, 19 },
    { 0, 0 },
    { 0x4d098330, 23 },
    { 0, 0 },
    { 0x2de98332, 55 },
    { 0, 0 },
    { 0x5a498334, 98 },
    { 0, 0 },
    { 0xe7198336, 65 },
    { 0x2bd98337, 23 },
    { 0x91d98337, 46 },
    { 0xf7798339, 90 },
    { 0xd7a9833a, 90 },
    { 0xb799833a, 43 },
    { 0xe5c9833c, 98 },
    { 0xd569833d, 77 },
    { 0xcf79833e, 69 },
    { 0, 0 },
    { 0xf5b98340, 61 },
    { 0x68c98341, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e98344, 40 },
    { 0xa3e98345, 98 },
    { 0, 0 },
    { 0x10e98347, 33 },
    { 0x3398347, 95 },
    { 0x9698349, 85 },
    { 0x9c398347, 98 },
    { 0x99834b, 52 },
    { 0xe69834c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x109834f, 95 },
    { 0x1709834f, 55 },
    { 0xcaa98350, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77498354, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b98358, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc349835d, 55 },
    { 0xe7d9835e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc798361, 55 },
    { 0xaaa98361, 67 },
    { 0x5e798363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6279836a, 76 },
    { 0x3f89836b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49836e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88198374, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f9837e, 90 },
    { 0x6f09837e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf498382, 33 },
    { 0x4ec98383, 23 },
    { 0, 0 },
    { 0x1f098385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49598388, 95 },
    { 0xc98389, 61 },
    { 0, 0 },
    { 0x3019838b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b9838e, 90 },
    { 0x3559838e, 95 },
    { 0x67098390, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb198394, 77 },
    { 0, 0 },
    { 0xc0198396, 33 },
    { 0x5e498397, 52 },
    { 0xd4a98397, 85 },
    { 0xd5898399, 95 },
    { 0xbfe98398, 33 },
    { 0x91998397, 98 },
    { 0, 0 },
    { 0x7389839d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e983a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f983a4, 76 },
    { 0, 0 },
    { 0x3a1983a6, 98 },
    { 0x825983a7, 85 },
    { 0xb9a983a8, 61 },
    { 0x47f983a9, 55 },
    { 0xf3b983aa, 33 },
    { 0x811983a6, 61 },
    { 0xcad983aa, 90 },
    { 0x585983ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x545983b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a983b3, 76 },
    { 0x4ab983b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea983be, 98 },
    { 0, 0 },
    { 0xd58983c0, 95 },
    { 0x440983c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53983c4, 61 },
    { 0, 0 },
    { 0xc57983c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688983ca, 23 },
    { 0xe1a983ca, 75 },
    { 0xddc983cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e983cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0983d2, 43 },
    { 0, 0 },
    { 0x2e7983d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2983db, 90 },
    { 0, 0 },
    { 0x66c983dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82983e2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8983e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0983ee, 33 },
    { 0, 0 },
    { 0x1a2983f0, 95 },
    { 0x7ab983f1, 60 },
    { 0xbb1983f2, 46 },
    { 0xe56983f1, 61 },
    { 0, 0 },
    { 0x862983f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a983fe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x13198401, 90 },
    { 0x28298401, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e98407, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d9840a, 87 },
    { 0, 0 },
    { 0xe2e9840c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fb98410, 67 },
    { 0x44698411, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe598415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30598418, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f898420, 90 },
    { 0x4c698421, 55 },
    { 0, 0 },
    { 0xa4e98423, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26298427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde79842c, 95 },
    { 0, 0 },
    { 0x4d39842e, 90 },
    { 0x6a69842f, 87 },
    { 0xa429842f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e89843c, 61 },
    { 0xa879843d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbca98448, 34 },
    { 0xf7098448, 52 },
    { 0x9c9844a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81898460, 95 },
    { 0, 0 },
    { 0x49f98462, 84 },
    { 0x84a98463, 95 },
    { 0x9d898464, 33 },
    { 0x34198463, 90 },
    { 0x1de98466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39846a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d98471, 17 },
    { 0xc9398472, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9698479, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8829847e, 98 },
    { 0xe219847e, 61 },
    { 0x698480, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99098484, 90 },
    { 0, 0 },
    { 0x8dc98486, 61 },
    { 0xe4398486, 23 },
    { 0xc8998486, 34 },
    { 0x41298488, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9209848d, 93 },
    { 0xb859848e, 33 },
    { 0, 0 },
    { 0x9ff98490, 33 },
    { 0xd9c98491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14398498, 33 },
    { 0xa4d98498, 90 },
    { 0xac898498, 43 },
    { 0xce39849b, 90 },
    { 0xc4898499, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8984a0, 61 },
    { 0, 0 },
    { 0x8d3984a2, 98 },
    { 0x764984a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1984a7, 49 },
    { 0x845984a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb984b3, 98 },
    { 0x17a984b4, 65 },
    { 0x36f984b5, 33 },
    { 0x7a2984b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6984b9, 65 },
    { 0xf96984ba, 46 },
    { 0xfbc984bb, 61 },
    { 0x92b984bc, 46 },
    { 0x7d7984bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7984c1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d984c5, 55 },
    { 0, 0 },
    { 0x8c9984c7, 43 },
    { 0, 0 },
    { 0xac6984c9, 33 },
    { 0x9c1984c9, 61 },
    { 0, 0 },
    { 0x45984cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f984d0, 90 },
    { 0xec984d1, 61 },
    { 0, 0 },
    { 0x23a984d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x581984d6, 36 },
    { 0x279984d7, 61 },
    { 0xeba984d6, 66 },
    { 0x3b1984d9, 33 },
    { 0x3a5984da, 76 },
    { 0x692984db, 33 },
    { 0x50c984dc, 61 },
    { 0x48b984dc, 98 },
    { 0, 0 },
    { 0xaea984df, 98 },
    { 0x457984e0, 91 },
    { 0x2a6984e1, 61 },
    { 0x6dd984e2, 98 },
    { 0xf45984e3, 23 },
    { 0x9ac984df, 55 },
    { 0x72e984e5, 43 },
    { 0xd62984e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f984e9, 66 },
    { 0, 0 },
    { 0x818984eb, 55 },
    { 0xbea984ec, 90 },
    { 0, 0 },
    { 0x51a984ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53984f1, 90 },
    { 0, 0 },
    { 0x62b984f3, 95 },
    { 0x477984f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x777984f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d984fa, 33 },
    { 0xeab984fb, 90 },
    { 0x364984fc, 61 },
    { 0xe39984fd, 61 },
    { 0, 0 },
    { 0x588984ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b98508, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe569850c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd079850f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf98512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d798516, 95 },
    { 0, 0 },
    { 0x73598518, 90 },
    { 0, 0 },
    { 0xd319851a, 23 },
    { 0x2cb9851a, 87 },
    { 0x6a79851c, 46 },
    { 0, 0 },
    { 0x6299851e, 61 },
    { 0, 0 },
    { 0xd2398520, 33 },
    { 0x98c98521, 23 },
    { 0xb2498520, 68 },
    { 0, 0 },
    { 0x99698524, 76 },
    { 0xc6a98525, 61 },
    { 0, 0 },
    { 0x5ea98527, 46 },
    { 0xec598528, 43 },
    { 0xbc698529, 93 },
    { 0xe5398529, 98 },
    { 0xca09852b, 34 },
    { 0xfbe98529, 55 },
    { 0xec39852d, 34 },
    { 0x49852d, 61 },
    { 0x7ea9852b, 98 },
    { 0x8ae98530, 33 },
    { 0, 0 },
    { 0xb5398532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4998535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf98539, 76 },
    { 0xcbd98539, 61 },
    { 0xa6e9853b, 34 },
    { 0x13a9853a, 100 },
    { 0, 0 },
    { 0x61a9853e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c98543, 23 },
    { 0x1b698543, 95 },
    { 0x79898545, 84 },
    { 0x32e98546, 83 },
    { 0, 0 },
    { 0x85398548, 84 },
    { 0, 0 },
    { 0xcb99854a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1298552, 55 },
    { 0xad098553, 24 },
    { 0, 0 },
    { 0x3ac98555, 61 },
    { 0x62398556, 33 },
    { 0x81d98556, 95 },
    { 0xe5c98558, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c698568, 33 },
    { 0x81598569, 98 },
    { 0x5649856a, 78 },
    { 0x1159856b, 90 },
    { 0x14698568, 66 },
    { 0xf2b9856d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0698570, 61 },
    { 0, 0 },
    { 0xf4f98572, 67 },
    { 0x97c98573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e98579, 90 },
    { 0x7fe9857a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a9857d, 33 },
    { 0xa949857d, 55 },
    { 0x8d99857e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2098582, 90 },
    { 0, 0 },
    { 0x3698584, 76 },
    { 0x44798585, 95 },
    { 0xf7798585, 23 },
    { 0x3d998587, 78 },
    { 0x9ba98588, 55 },
    { 0x93898589, 95 },
    { 0x26198587, 93 },
    { 0, 0 },
    { 0x2049858c, 99 },
    { 0x4c19858d, 98 },
    { 0x9f29858c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c398591, 100 },
    { 0x2cf98592, 97 },
    { 0x62f98593, 61 },
    { 0, 0 },
    { 0xebf98595, 98 },
    { 0x77e98596, 100 },
    { 0, 0 },
    { 0x5f198598, 61 },
    { 0x2c698598, 94 },
    { 0, 0 },
    { 0xd769859b, 78 },
    { 0x1459859c, 90 },
    { 0xfb59859d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0985a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e985a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e985ab, 68 },
    { 0xcc0985ac, 90 },
    { 0xf89985ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5985b0, 55 },
    { 0x975985b1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0985b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687985bb, 91 },
    { 0xba985bc, 95 },
    { 0xbc8985bd, 66 },
    { 0xddd985bd, 76 },
    { 0x62d985bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248985c3, 23 },
    { 0xfaf985c3, 23 },
    { 0xbcc985c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b985c9, 91 },
    { 0x322985ca, 76 },
    { 0x457985ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7985d2, 100 },
    { 0x1ea985d3, 98 },
    { 0, 0 },
    { 0xb26985d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78985d8, 23 },
    { 0, 0 },
    { 0x6c9985da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd985de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f985e3, 37 },
    { 0, 0 },
    { 0x4df985e5, 90 },
    { 0, 0 },
    { 0xb08985e7, 46 },
    { 0xeb985e7, 77 },
    { 0xf5a985e8, 23 },
    { 0x6cc985ea, 33 },
    { 0xe19985eb, 76 },
    { 0x4f2985ec, 33 },
    { 0xc0a985e7, 95 },
    { 0x5d4985ee, 84 },
    { 0x268985ec, 34 },
    { 0x64e985f0, 76 },
    { 0xbf4985f1, 90 },
    { 0x90b985f2, 33 },
    { 0x883985e7, 23 },
    { 0xa51985f4, 81 },
    { 0x665985ec, 98 },
    { 0xfe0985ed, 98 },
    { 0x64c985f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95985fa, 23 },
    { 0x24e985fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5598600, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb98606, 93 },
    { 0x44d98607, 43 },
    { 0xeb498608, 90 },
    { 0x77698608, 90 },
    { 0xe1a9860a, 100 },
    { 0x46e98606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3199860e, 76 },
    { 0xbaa9860e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99498612, 93 },
    { 0, 0 },
    { 0xf8d98614, 33 },
    { 0x35798614, 33 },
    { 0x30398616, 28 },
    { 0xbdf98617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2759861b, 91 },
    { 0x6b89861b, 91 },
    { 0x60a9861c, 46 },
    { 0, 0 },
    { 0xe479861f, 98 },
    { 0x85f9861f, 98 },
    { 0x6049861f, 90 },
    { 0, 0 },
    { 0x58798623, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d9862a, 98 },
    { 0x4449862b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe179862f, 33 },
    { 0, 0 },
    { 0xd7598631, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c9863f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9298645, 76 },
    { 0x5e998646, 26 },
    { 0xf4298647, 55 },
    { 0x4d98648, 95 },
    { 0x17798649, 33 },
    { 0x45498645, 66 },
    { 0x3c09864b, 76 },
    { 0x1a89864c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e598652, 55 },
    { 0x65698653, 91 },
    { 0, 0 },
    { 0x4ee98655, 90 },
    { 0xcb998656, 77 },
    { 0, 0 },
    { 0x1c698658, 33 },
    { 0xedd98658, 95 },
    { 0xdc49865a, 98 },
    { 0xea398658, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba9865e, 43 },
    { 0xe809865f, 33 },
    { 0x83b98660, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab98667, 61 },
    { 0xf2298667, 76 },
    { 0xd8698667, 31 },
    { 0x1359866a, 100 },
    { 0x43b98667, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c29866e, 33 },
    { 0, 0 },
    { 0xe2598670, 23 },
    { 0x46698671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c98674, 90 },
    { 0x7be98675, 95 },
    { 0x26a98676, 23 },
    { 0, 0 },
    { 0xf9698678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7889867b, 33 },
    { 0, 0 },
    { 0x7d79867d, 23 },
    { 0x3f29867e, 76 },
    { 0x6e69867d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b398684, 93 },
    { 0x34c98685, 64 },
    { 0, 0 },
    { 0xb3a98687, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d9868b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1198692, 95 },
    { 0xb2398692, 61 },
    { 0xc3f98692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b98697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba9869a, 33 },
    { 0x9489869b, 23 },
    { 0x8289869a, 90 },
    { 0xe89869a, 61 },
    { 0x9289869a, 93 },
    { 0x929869f, 69 },
    { 0, 0 },
    { 0x4c9986a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5986a5, 90 },
    { 0x3ce986a6, 76 },
    { 0xc0d986a5, 66 },
    { 0x5df986a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e986ab, 23 },
    { 0xbaf986ab, 61 },
    { 0, 0 },
    { 0x783986ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7986b1, 90 },
    { 0x320986b2, 90 },
    { 0xcc5986b3, 76 },
    { 0x5d7986b3, 91 },
    { 0x3f1986b5, 33 },
    { 0xf94986b6, 78 },
    { 0x230986b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37986bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7986c2, 78 },
    { 0xc1d986c3, 78 },
    { 0xd97986c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942986cb, 81 },
    { 0xed7986cc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd986d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf986d4, 90 },
    { 0xb61986d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc986d9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc986df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20986e3, 90 },
    { 0xc2f986e3, 55 },
    { 0x471986e3, 66 },
    { 0x577986e6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e986ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432986ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6986f6, 43 },
    { 0xd34986f7, 23 },
    { 0x516986f7, 75 },
    { 0x1c1986f8, 90 },
    { 0xc45986fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4498701, 33 },
    { 0, 0 },
    { 0x58f98703, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e9870b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d9870e, 96 },
    { 0x6ed9870f, 23 },
    { 0, 0 },
    { 0x3eb98711, 80 },
    { 0x9098712, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b98717, 66 },
    { 0xb9798717, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9871b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf9871e, 33 },
    { 0xd009871f, 17 },
    { 0x20098720, 90 },
    { 0xe6798720, 95 },
    { 0x9a198720, 61 },
    { 0xb9898722, 95 },
    { 0x23898724, 93 },
    { 0xd0098725, 23 },
    { 0xf6c98726, 90 },
    { 0, 0 },
    { 0x96898728, 34 },
    { 0, 0 },
    { 0x86a9872a, 46 },
    { 0xab19872b, 100 },
    { 0x9629872c, 23 },
    { 0xa89872d, 76 },
    { 0xcff9872e, 77 },
    { 0, 0 },
    { 0x1fc98730, 33 },
    { 0x4a198731, 90 },
    { 0x2f298730, 33 },
    { 0x9a198731, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87498739, 95 },
    { 0xeb098739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9873d, 91 },
    { 0x7f79873e, 61 },
    { 0xabd9873f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c98743, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b998748, 84 },
    { 0, 0 },
    { 0xb429874a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb449874e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45098752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1798755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82998758, 23 },
    { 0, 0 },
    { 0x2b49875a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9875e, 87 },
    { 0xdd9875f, 90 },
    { 0x65d98760, 34 },
    { 0xc0298761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9876a, 76 },
    { 0, 0 },
    { 0xb4a9876c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30498771, 52 },
    { 0x64198772, 23 },
    { 0x60a98772, 90 },
    { 0xcb698774, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c798778, 100 },
    { 0xc898778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa39877c, 52 },
    { 0, 0 },
    { 0xec79877e, 33 },
    { 0xd79877f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15498784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec798788, 90 },
    { 0x4e498788, 95 },
    { 0, 0 },
    { 0xad89878b, 65 },
    { 0x67c9878b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49698790, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc598796, 81 },
    { 0x51e98797, 91 },
    { 0x9a298798, 55 },
    { 0xe9d98797, 33 },
    { 0xdfd98799, 46 },
    { 0, 0 },
    { 0xe89879c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a9879f, 69 },
    { 0xb3b9879f, 43 },
    { 0, 0 },
    { 0x751987a2, 33 },
    { 0x7ef987a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x182987a6, 75 },
    { 0x873987a7, 78 },
    { 0, 0 },
    { 0x5bc987a9, 33 },
    { 0x65e987aa, 76 },
    { 0x2de987ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b987b1, 23 },
    { 0x560987b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279987be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28987c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a987c6, 61 },
    { 0x8e8987c7, 90 },
    { 0, 0 },
    { 0x8cc987c9, 33 },
    { 0, 0 },
    { 0x253987cb, 61 },
    { 0x7aa987cb, 61 },
    { 0x904987cd, 98 },
    { 0, 0 },
    { 0x260987cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41987d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f987dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0d987df, 17 },
    { 0x211987e0, 61 },
    { 0x7ed987e1, 91 },
    { 0xf36987df, 23 },
    { 0xb4b987e3, 82 },
    { 0, 0 },
    { 0x8c8987e5, 68 },
    { 0x770987e6, 90 },
    { 0xb74987e5, 76 },
    { 0x621987e5, 24 },
    { 0, 0 },
    { 0xcf3987ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2d987ef, 23 },
    { 0x842987ef, 90 },
    { 0, 0 },
    { 0xffe987f2, 90 },
    { 0x4b4987f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f987ff, 33 },
    { 0x62898800, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7098809, 61 },
    { 0x2ef9880a, 98 },
    { 0xf709880b, 64 },
    { 0x33c9880c, 66 },
    { 0x9889880c, 34 },
    { 0x2319880b, 55 },
    { 0x7659880f, 76 },
    { 0xc0398810, 98 },
    { 0x2f498811, 46 },
    { 0x38698810, 55 },
    { 0xa9b9880f, 95 },
    { 0xc1d98814, 75 },
    { 0xe9b98815, 90 },
    { 0, 0 },
    { 0x20298817, 90 },
    { 0x5aa98817, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad9881c, 90 },
    { 0x779881d, 61 },
    { 0xd89881d, 66 },
    { 0xa929881f, 64 },
    { 0, 0 },
    { 0xdff98821, 81 },
    { 0x58e98822, 23 },
    { 0, 0 },
    { 0xd3a98824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5499882d, 61 },
    { 0x3509882d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c198831, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81598838, 100 },
    { 0x43298839, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19883d, 33 },
    { 0x3629883e, 33 },
    { 0x7d69883f, 33 },
    { 0x7659883d, 61 },
    { 0x94198841, 98 },
    { 0xd1c98840, 23 },
    { 0, 0 },
    { 0x44c98844, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35598847, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc9884b, 33 },
    { 0, 0 },
    { 0x5ea9884d, 98 },
    { 0xf389884d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce98853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e98857, 91 },
    { 0xa5998858, 91 },
    { 0x20c98859, 77 },
    { 0x3d498859, 33 },
    { 0xbc29885b, 33 },
    { 0x22198858, 60 },
    { 0x6889885d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24798863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0598870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2998873, 95 },
    { 0x4c598874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f498877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2089887f, 33 },
    { 0xb4d98880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa98883, 76 },
    { 0, 0 },
    { 0x25f98885, 33 },
    { 0x1c098886, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c49888d, 78 },
    { 0x1889888e, 61 },
    { 0xbd69888f, 78 },
    { 0x86b9888f, 33 },
    { 0xaa898890, 98 },
    { 0x9f798890, 23 },
    { 0x53e98893, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b9889b, 46 },
    { 0x3be9889b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c988ae, 82 },
    { 0x17c988af, 38 },
    { 0x5e1988b0, 98 },
    { 0x79c988ae, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7988b6, 52 },
    { 0x3a9988b7, 83 },
    { 0xb9e988b7, 95 },
    { 0xf6c988b6, 55 },
    { 0, 0 },
    { 0x833988bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x582988be, 61 },
    { 0x31e988bf, 67 },
    { 0xee6988c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc988c4, 90 },
    { 0xc55988c5, 90 },
    { 0x7d4988c6, 95 },
    { 0x1ee988c7, 56 },
    { 0, 0 },
    { 0x60988c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33988cc, 46 },
    { 0x10c988cc, 99 },
    { 0x471988cd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392988d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8988d7, 77 },
    { 0x891988d8, 55 },
    { 0, 0 },
    { 0xe66988da, 33 },
    { 0x8c6988db, 46 },
    { 0x18c988db, 98 },
    { 0x96988dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba988e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34988ed, 78 },
    { 0x992988ee, 46 },
    { 0x536988ed, 99 },
    { 0x82a988ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5988f4, 33 },
    { 0xd87988f5, 98 },
    { 0x427988f4, 33 },
    { 0, 0 },
    { 0xadf988f8, 77 },
    { 0xcc2988f9, 66 },
    { 0xb3a988fa, 93 },
    { 0x9bc988fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x497988fe, 34 },
    { 0x792988ff, 61 },
    { 0x10798900, 61 },
    { 0xc5c98900, 17 },
    { 0, 0 },
    { 0x13598903, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf098906, 84 },
    { 0x71e98907, 90 },
    { 0xac298908, 23 },
    { 0x66e98907, 55 },
    { 0x9169890a, 91 },
    { 0, 0 },
    { 0xb559890c, 90 },
    { 0xa799890c, 61 },
    { 0xeda9890e, 33 },
    { 0xa249890f, 98 },
    { 0x3149890e, 33 },
    { 0, 0 },
    { 0xe8e98912, 76 },
    { 0, 0 },
    { 0xd1998914, 48 },
    { 0x58d98914, 66 },
    { 0x8ea98916, 90 },
    { 0x8c998917, 39 },
    { 0xc8898918, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc39891d, 99 },
    { 0, 0 },
    { 0xb069891f, 61 },
    { 0, 0 },
    { 0xaf698921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5098924, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94198927, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7829892b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1209892e, 61 },
    { 0xd639892f, 53 },
    { 0xbda98930, 33 },
    { 0x4eb9892e, 55 },
    { 0xc5698932, 61 },
    { 0xdab98932, 23 },
    { 0x6898934, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf698937, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e79893a, 33 },
    { 0xcd79893b, 93 },
    { 0, 0 },
    { 0x15c9893d, 65 },
    { 0xe0c9893d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4098942, 23 },
    { 0, 0 },
    { 0x11198944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a498947, 98 },
    { 0, 0 },
    { 0x98998949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2179894d, 98 },
    { 0x7669894e, 77 },
    { 0x3d29894f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9298952, 95 },
    { 0xcd398953, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x24898956, 23 },
    { 0xc3b98956, 61 },
    { 0xfa98956, 55 },
    { 0x6da98956, 34 },
    { 0x53498958, 76 },
    { 0x49a98958, 28 },
    { 0, 0 },
    { 0xa7f9895d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21498962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f598968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19896d, 52 },
    { 0xe7d9896e, 90 },
    { 0, 0 },
    { 0xc4798970, 23 },
    { 0xf8098971, 98 },
    { 0xc0d98971, 61 },
    { 0x84398971, 43 },
    { 0x87f98974, 76 },
    { 0x11e98974, 90 },
    { 0, 0 },
    { 0x1aa98977, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1859897a, 76 },
    { 0x899897b, 23 },
    { 0xefa9897b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3498980, 75 },
    { 0, 0 },
    { 0x13898982, 98 },
    { 0xe6698983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9998989, 93 },
    { 0, 0 },
    { 0x5809898b, 34 },
    { 0x979898c, 33 },
    { 0xe309898d, 33 },
    { 0x37f9898e, 66 },
    { 0, 0 },
    { 0x78298990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac98998, 95 },
    { 0x8b898998, 93 },
    { 0xca698999, 75 },
    { 0x1ff9899b, 34 },
    { 0, 0 },
    { 0xf179899d, 77 },
    { 0x6df9899e, 90 },
    { 0x3709899f, 61 },
    { 0, 0 },
    { 0x3c5989a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4989ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338989b8, 81 },
    { 0x4af989b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a989bc, 77 },
    { 0xdaf989bd, 33 },
    { 0, 0 },
    { 0x9fe989bf, 33 },
    { 0, 0 },
    { 0x2de989c1, 98 },
    { 0, 0 },
    { 0x5bd989c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61e989c6, 90 },
    { 0x16d989c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0989cc, 61 },
    { 0, 0 },
    { 0xe43989ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2989d1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de989dd, 46 },
    { 0x7ea989dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x486989e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c989e9, 46 },
    { 0x655989ea, 43 },
    { 0xdeb989eb, 98 },
    { 0x413989ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a989f1, 56 },
    { 0xd24989f2, 23 },
    { 0, 0 },
    { 0x2dd989f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930989f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db989fc, 23 },
    { 0x497989fd, 98 },
    { 0x619989fe, 75 },
    { 0x3a1989ff, 95 },
    { 0xeb698a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5398a08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca98a0b, 98 },
    { 0x2bd98a0c, 61 },
    { 0, 0 },
    { 0xbf998a0e, 33 },
    { 0x1ee98a0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39098a13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44598a1a, 23 },
    { 0, 0 },
    { 0x23e98a1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9298a21, 55 },
    { 0xfb598a22, 87 },
    { 0, 0 },
    { 0x33a98a24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef98a2b, 90 },
    { 0xea298a2b, 34 },
    { 0x90098a2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8698a30, 23 },
    { 0xd0e98a31, 23 },
    { 0x29598a32, 95 },
    { 0x18798a30, 43 },
    { 0x22f98a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f098a37, 76 },
    { 0x41598a37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0198a41, 93 },
    { 0x2c798a42, 98 },
    { 0x65898a42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3298a46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b198a4b, 77 },
    { 0xd298a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2798a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d498a56, 23 },
    { 0x7d198a56, 90 },
    { 0, 0 },
    { 0x7eb98a59, 95 },
    { 0x5d498a5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d998a63, 61 },
    { 0xe6398a63, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bb98a68, 98 },
    { 0, 0 },
    { 0x87698a6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d998a73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1098a76, 77 },
    { 0x15098a77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df98a7c, 33 },
    { 0x96798a7c, 76 },
    { 0x17c98a7c, 95 },
    { 0x87898a7f, 55 },
    { 0x64f98a80, 23 },
    { 0x3d498a7e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b798a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca98a8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd98a8e, 94 },
    { 0x28b98a8f, 93 },
    { 0xe2e98a8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d298a93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d798a99, 98 },
    { 0, 0 },
    { 0x14c98a9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33298a9f, 95 },
    { 0x8f798a9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c98aa3, 66 },
    { 0xf6498aa4, 90 },
    { 0x54598aa4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe98aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce98aaf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x43698ab2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7798ab5, 98 },
    { 0x6198ab6, 85 },
    { 0x88c98ab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a898aba, 33 },
    { 0x73698abb, 43 },
    { 0x4a998abc, 66 },
    { 0x81498abc, 61 },
    { 0x65998abe, 17 },
    { 0, 0 },
    { 0xc7e98ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf198ac4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2098ac9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x68998acc, 55 },
    { 0x3b898acc, 61 },
    { 0xa1298acc, 95 },
    { 0x77698ace, 55 },
    { 0xdc398ad0, 75 },
    { 0x71c98ad0, 95 },
    { 0, 0 },
    { 0xda798ad3, 53 },
    { 0x56798ad3, 44 },
    { 0xd5398ad3, 95 },
    { 0x75598ad4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x50998ad9, 34 },
    { 0xebe98ada, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d98ade, 33 },
    { 0xd98ade, 23 },
    { 0x4e798ae0, 66 },
    { 0x10498adf, 46 },
    { 0x9cd98ae1, 19 },
    { 0xef998ae2, 95 },
    { 0, 0 },
    { 0x90298ae5, 66 },
    { 0x2a898ae5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2598aea, 40 },
    { 0x17598aea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3598af2, 90 },
    { 0, 0 },
    { 0x7a998af4, 55 },
    { 0, 0 },
    { 0x30398af6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf98afa, 23 },
    { 0x89d98afa, 23 },
    { 0x7cf98afb, 61 },
    { 0, 0 },
    { 0x1d498afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76498b01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf98b04, 33 },
    { 0xab298b05, 95 },
    { 0x57b98b06, 90 },
    { 0, 0 },
    { 0x23498b08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17398b0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f98b11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4998b17, 23 },
    { 0xd6f98b18, 69 },
    { 0x30798b19, 98 },
    { 0x9b498b1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a498b1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4198b2a, 95 },
    { 0xdfe98b2b, 90 },
    { 0x20d98b2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb198b30, 88 },
    { 0xc3a98b31, 98 },
    { 0, 0 },
    { 0xe1b98b33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b298b37, 90 },
    { 0, 0 },
    { 0xe4e98b39, 90 },
    { 0x18e98b3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d298b3d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac98b42, 43 },
    { 0xb9e98b43, 98 },
    { 0x6bf98b44, 84 },
    { 0x50b98b45, 98 },
    { 0, 0 },
    { 0x8c398b47, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8098b4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e98b51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d098b54, 98 },
    { 0x11e98b55, 33 },
    { 0xef198b56, 61 },
    { 0x4f298b57, 90 },
    { 0, 0 },
    { 0xc8e98b59, 81 },
    { 0x3c798b5a, 61 },
    { 0, 0 },
    { 0x1a598b5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c98b5f, 95 },
    { 0xe98b60, 90 },
    { 0x3998b61, 91 },
    { 0xdf798b62, 17 },
    { 0x4b598b62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b98b66, 98 },
    { 0x6f898b67, 95 },
    { 0x49a98b68, 38 },
    { 0x2c598b66, 95 },
    { 0x1cc98b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80898b72, 76 },
    { 0, 0 },
    { 0xe7998b74, 23 },
    { 0x76798b75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a198b79, 61 },
    { 0, 0 },
    { 0x1ab98b7b, 95 },
    { 0x35d98b7c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a98b7f, 43 },
    { 0, 0 },
    { 0x91198b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85498b84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba98b87, 61 },
    { 0x9da98b87, 88 },
    { 0x38498b88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f598b90, 23 },
    { 0xf0b98b90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e498b96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64798b9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb98b9f, 56 },
    { 0xfec98b9f, 84 },
    { 0x9de98ba1, 64 },
    { 0, 0 },
    { 0xb1198ba3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae698ba7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75098bad, 76 },
    { 0xbe698bad, 93 },
    { 0, 0 },
    { 0xbdd98bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53598bb4, 56 },
    { 0x1ce98bb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b98bb9, 55 },
    { 0, 0 },
    { 0x70b98bbb, 99 },
    { 0xf3998bbc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4398bc3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0098bc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b98bcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb198bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe98be1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75298be4, 33 },
    { 0, 0 },
    { 0x30098be6, 23 },
    { 0, 0 },
    { 0xe7f98be8, 98 },
    { 0, 0 },
    { 0xcb198bea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41198bed, 52 },
    { 0x3fb98bee, 91 },
    { 0xeda98bee, 76 },
    { 0x85598bf0, 99 },
    { 0x1d198bf0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb098bf8, 90 },
    { 0xb5298bf9, 95 },
    { 0x20498bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e398c01, 61 },
    { 0, 0 },
    { 0x68498c03, 33 },
    { 0xd5798c04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f98c08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f98c11, 61 },
    { 0x99698c12, 61 },
    { 0x14098c13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe98c1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1998c1f, 33 },
    { 0, 0 },
    { 0xf2d98c21, 90 },
    { 0xef098c21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae98c26, 90 },
    { 0x3dc98c27, 61 },
    { 0, 0 },
    { 0xefc98c29, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d98c2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cf98c30, 23 },
    { 0x25498c30, 33 },
    { 0x5dd98c32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e98c3e, 61 },
    { 0x51d98c3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7198c43, 98 },
    { 0, 0 },
    { 0x5cb98c45, 75 },
    { 0xe0c98c46, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c98c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef498c4e, 87 },
    { 0x4cd98c4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58498c52, 33 },
    { 0, 0 },
    { 0xd3f98c54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec898c57, 34 },
    { 0x52698c57, 23 },
    { 0x53d98c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66898c5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc98c66, 87 },
    { 0x77898c67, 90 },
    { 0x91c98c67, 95 },
    { 0x51798c69, 95 },
    { 0x62198c69, 66 },
    { 0x9698c6a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xec998c6e, 77 },
    { 0, 0 },
    { 0xac598c70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78e98c77, 98 },
    { 0x74d98c78, 43 },
    { 0, 0 },
    { 0x34798c7a, 90 },
    { 0xa898c7b, 46 },
    { 0, 0 },
    { 0x53598c7d, 61 },
    { 0xcc798c7e, 76 },
    { 0x51998c7f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5798c85, 95 },
    { 0x4eb98c86, 80 },
    { 0x46398c87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1498c8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a698c93, 100 },
    { 0xb2698c94, 93 },
    { 0x4cf98c95, 77 },
    { 0x5f898c93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d98c9c, 61 },
    { 0x7f698c9d, 90 },
    { 0xb0e98c9d, 61 },
    { 0, 0 },
    { 0xad098ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5598ca3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1498ca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90498caa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea98caf, 33 },
    { 0x49498cb0, 61 },
    { 0x63a98cb0, 40 },
    { 0x1df98caf, 55 },
    { 0x30b98cb3, 98 },
    { 0xd5d98cb4, 87 },
    { 0x9e998cb5, 33 },
    { 0, 0 },
    { 0x10d98cb7, 75 },
    { 0xfb198cb8, 88 },
    { 0xbee98cb9, 76 },
    { 0x5598cba, 90 },
    { 0x22e98cb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc98cbf, 23 },
    { 0x6a398cc0, 43 },
    { 0xb9a98cc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39898cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60198cc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc98cce, 43 },
    { 0, 0 },
    { 0xe0d98cd0, 23 },
    { 0, 0 },
    { 0xd0d98cd2, 33 },
    { 0x8b798cd3, 76 },
    { 0xf3c98cd4, 23 },
    { 0x27b98cd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe598cda, 95 },
    { 0, 0 },
    { 0xa6a98cdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29498cdf, 67 },
    { 0xf7e98ce0, 98 },
    { 0x2cf98ce1, 67 },
    { 0x5c298ce2, 62 },
    { 0x83498ce3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x898ce6, 78 },
    { 0x57598ce7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f98cec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb98cf3, 61 },
    { 0x88c98cf4, 81 },
    { 0x69f98cf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8598cfb, 85 },
    { 0x6ac98cfc, 98 },
    { 0x8a098cfb, 90 },
    { 0xce498cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d98d01, 66 },
    { 0, 0 },
    { 0xae498d03, 46 },
    { 0x65498d04, 33 },
    { 0x4e098d04, 76 },
    { 0xcad98d03, 61 },
    { 0xd8298d07, 99 },
    { 0x30c98d07, 77 },
    { 0xd9b98d06, 33 },
    { 0x5d898d0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3998d0e, 98 },
    { 0x7ac98d0f, 55 },
    { 0, 0 },
    { 0xdc398d11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5298d15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x32098d18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e98d1f, 75 },
    { 0x15798d20, 23 },
    { 0xd8398d21, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a98d27, 93 },
    { 0x3d398d28, 46 },
    { 0, 0 },
    { 0xfc398d2a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b98d2d, 34 },
    { 0x40598d2e, 77 },
    { 0xf7798d2f, 95 },
    { 0x3c298d30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3598d33, 33 },
    { 0xe4c98d33, 98 },
    { 0xee298d35, 98 },
    { 0x64d98d36, 23 },
    { 0, 0 },
    { 0xba798d38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8498d3b, 69 },
    { 0x92398d3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d98d3f, 95 },
    { 0x1e898d40, 99 },
    { 0x22098d41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc498d45, 61 },
    { 0xfed98d45, 90 },
    { 0x98a98d47, 91 },
    { 0xa0e98d47, 33 },
    { 0x1df98d47, 93 },
    { 0x91198d49, 85 },
    { 0x50298d48, 95 },
    { 0x99898d46, 75 },
    { 0xaea98d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6898d55, 90 },
    { 0xee798d55, 81 },
    { 0xafd98d55, 33 },
    { 0xedb98d58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa998d5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6998d67, 90 },
    { 0x37798d68, 55 },
    { 0xeec98d69, 23 },
    { 0xcd298d68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb898d71, 46 },
    { 0xa8e98d72, 33 },
    { 0x13798d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78398d76, 33 },
    { 0x29b98d77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7898d80, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2698d84, 77 },
    { 0, 0 },
    { 0x2ca98d86, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4898d8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e98d8e, 33 },
    { 0x69998d8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd198d92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b198d95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeca98d99, 100 },
    { 0x4b998d9a, 84 },
    { 0xc5098d99, 23 },
    { 0x72198d9c, 98 },
    { 0xe8798d9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e98da3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38698da9, 90 },
    { 0, 0 },
    { 0xbcc98dab, 23 },
    { 0x61198dac, 55 },
    { 0x5be98dac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e98db5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac98dbf, 76 },
    { 0, 0 },
    { 0x91998dc1, 90 },
    { 0xc9798dc1, 90 },
    { 0xc5798dc3, 77 },
    { 0x61598dc4, 33 },
    { 0xc5a98dc4, 55 },
    { 0xcd998dc3, 90 },
    { 0x8398dc2, 95 },
    { 0x16798dc5, 66 },
    { 0xf9498dc2, 55 },
    { 0, 0 },
    { 0xd9498dcb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5398dce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88398dd8, 40 },
    { 0x6ca98dd8, 99 },
    { 0x31098dda, 61 },
    { 0x69d98ddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6498de0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x82198de3, 69 },
    { 0, 0 },
    { 0xc8798de5, 76 },
    { 0x77198de6, 90 },
    { 0x7ea98de6, 90 },
    { 0x23d98de5, 98 },
    { 0, 0 },
    { 0x14298dea, 95 },
    { 0, 0 },
    { 0xf1e98dec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a698df5, 61 },
    { 0xeb798df6, 87 },
    { 0xc6e98df6, 77 },
    { 0x95498df5, 55 },
    { 0, 0 },
    { 0xd4398dfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd798dfd, 23 },
    { 0xb6698dfe, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed98e09, 95 },
    { 0xd8898e0a, 90 },
    { 0x15598e0b, 65 },
    { 0x2a698e0a, 98 },
    { 0x78b98e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e698e10, 61 },
    { 0xcfe98e10, 69 },
    { 0x18f98e12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8398e17, 90 },
    { 0, 0 },
    { 0x3ad98e19, 98 },
    { 0, 0 },
    { 0x91f98e1b, 61 },
    { 0x1b598e1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc98e24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84698e27, 55 },
    { 0x29e98e28, 99 },
    { 0xea798e28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f98e2e, 55 },
    { 0xc1d98e2f, 75 },
    { 0xe0e98e30, 75 },
    { 0x2e998e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34598e35, 33 },
    { 0xf3998e35, 98 },
    { 0, 0 },
    { 0x29998e38, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa98e3e, 34 },
    { 0x14f98e3e, 33 },
    { 0x36298e3e, 87 },
    { 0xdb998e41, 68 },
    { 0xb5698e42, 67 },
    { 0x9ba98e3e, 90 },
    { 0, 0 },
    { 0xcff98e45, 90 },
    { 0, 0 },
    { 0x8f598e47, 78 },
    { 0x51498e48, 55 },
    { 0x93598e49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41698e4c, 23 },
    { 0x898e4c, 33 },
    { 0, 0 },
    { 0x28998e4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f98e55, 91 },
    { 0, 0 },
    { 0x2af98e57, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae198e5b, 95 },
    { 0x68398e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf098e61, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e98e65, 69 },
    { 0, 0 },
    { 0xbbd98e67, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3998e6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda598e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d698e73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e98e78, 34 },
    { 0, 0 },
    { 0x27098e7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd398e7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf298e8a, 78 },
    { 0x11098e8a, 55 },
    { 0xf8098e8c, 90 },
    { 0x8bb98e8b, 33 },
    { 0x48998e8a, 90 },
    { 0x21898e8c, 98 },
    { 0x41f98e8b, 98 },
    { 0, 0 },
    { 0x32a98e92, 61 },
    { 0, 0 },
    { 0x82b98e94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de98e99, 76 },
    { 0xeea98e9a, 90 },
    { 0, 0 },
    { 0xa0998e9c, 61 },
    { 0, 0 },
    { 0xdea98e9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43798ea2, 98 },
    { 0xb8198ea3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c98ea6, 33 },
    { 0x9a98ea7, 90 },
    { 0x5fe98ea8, 99 },
    { 0, 0 },
    { 0x10398eaa, 33 },
    { 0x95c98eab, 98 },
    { 0xaa998eab, 98 },
    { 0x42b98ead, 33 },
    { 0, 0 },
    { 0x1ee98eaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39398eb5, 61 },
    { 0x2e598eb5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd098ebd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53198ecb, 90 },
    { 0, 0 },
    { 0x70a98ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1998ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a398ed5, 90 },
    { 0xb398ed5, 98 },
    { 0xdde98ed7, 55 },
    { 0xef698ed8, 43 },
    { 0, 0 },
    { 0xa9698eda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf398ee2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47898ee6, 99 },
    { 0, 0 },
    { 0x61698ee8, 88 },
    { 0x6d898ee9, 55 },
    { 0xab198eea, 91 },
    { 0x29898eea, 33 },
    { 0x5f098ee9, 98 },
    { 0xc4098eed, 33 },
    { 0, 0 },
    { 0x4e598eef, 95 },
    { 0xc9e98ef0, 90 },
    { 0x6ad98ef1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f98ef4, 90 },
    { 0xed098ef4, 33 },
    { 0x28c98ef6, 61 },
    { 0x24f98ef7, 93 },
    { 0xba398ef8, 33 },
    { 0xb3a98ef9, 55 },
    { 0x4ef98efa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b098f04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af98f0b, 95 },
    { 0xa4498f0c, 61 },
    { 0, 0 },
    { 0x85598f0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e98f11, 90 },
    { 0, 0 },
    { 0x8ff98f13, 23 },
    { 0, 0 },
    { 0xd2298f15, 69 },
    { 0, 0 },
    { 0x4b498f17, 76 },
    { 0xda198f18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c598f1c, 95 },
    { 0, 0 },
    { 0x23b98f1e, 96 },
    { 0, 0 },
    { 0x72298f20, 95 },
    { 0, 0 },
    { 0x1ed98f22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f098f25, 90 },
    { 0, 0 },
    { 0xb5698f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc198f2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8698f2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa98f31, 33 },
    { 0, 0 },
    { 0x81498f33, 76 },
    { 0x43d98f33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d98f38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e98f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba698f45, 100 },
    { 0, 0 },
    { 0x3c698f47, 87 },
    { 0x2e098f48, 61 },
    { 0x6cc98f48, 90 },
    { 0xa6098f4a, 95 },
    { 0xbeb98f48, 98 },
    { 0xa8498f4c, 90 },
    { 0, 0 },
    { 0x20f98f4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c98f59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5698f63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d998f6b, 61 },
    { 0x35998f6b, 52 },
    { 0xb0898f6b, 98 },
    { 0, 0 },
    { 0xeee98f6f, 69 },
    { 0, 0 },
    { 0x9dc98f71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e98f75, 90 },
    { 0, 0 },
    { 0xeae98f77, 66 },
    { 0xb7498f78, 46 },
    { 0x12698f77, 95 },
    { 0, 0 },
    { 0x46698f7b, 23 },
    { 0x46498f7b, 97 },
    { 0x56a98f7c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9798f80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe498f84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6b98f87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af98f8b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b98f8f, 33 },
    { 0x50898f8f, 43 },
    { 0xe8c98f90, 85 },
    { 0xe7498f92, 43 },
    { 0x94898f90, 98 },
    { 0, 0 },
    { 0xec398f95, 34 },
    { 0x50f98f96, 95 },
    { 0, 0 },
    { 0x85498f98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea98f9b, 88 },
    { 0, 0 },
    { 0xc3198f9d, 43 },
    { 0, 0 },
    { 0xbda98f9f, 99 },
    { 0, 0 },
    { 0x8cf98fa1, 98 },
    { 0x1e698fa2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0598fa6, 98 },
    { 0, 0 },
    { 0x7bd98fa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d98fac, 95 },
    { 0x3ed98fad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad98fb3, 90 },
    { 0, 0 },
    { 0xedc98fb5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca398fba, 23 },
    { 0xd3f98fbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad98fc2, 90 },
    { 0, 0 },
    { 0x41998fc4, 23 },
    { 0x10f98fc5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56898fc8, 61 },
    { 0, 0 },
    { 0x8c598fca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32598fcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c98fd1, 90 },
    { 0xf2798fd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40398fd5, 90 },
    { 0xc1998fd5, 90 },
    { 0, 0 },
    { 0x55498fd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x13198fdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f398fe2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e98fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe098feb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df98fef, 98 },
    { 0x97598ff0, 95 },
    { 0x9d398ff1, 90 },
    { 0x99a98ff1, 95 },
    { 0x53e98ff3, 34 },
    { 0x6598ff4, 61 },
    { 0, 0 },
    { 0x1c98ff6, 88 },
    { 0x998ff7, 90 },
    { 0x90e98ff8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d298ffb, 90 },
    { 0x15a98ffc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d199002, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20199006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac99009, 43 },
    { 0x3f79900a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d999011, 81 },
    { 0, 0 },
    { 0x98599013, 90 },
    { 0x62a99014, 78 },
    { 0x7b799014, 95 },
    { 0x22b99015, 95 },
    { 0x5a299013, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49901b, 95 },
    { 0, 0 },
    { 0x2a79901d, 66 },
    { 0xaed9901e, 61 },
    { 0x41f9901f, 61 },
    { 0x95b99020, 67 },
    { 0x80499021, 97 },
    { 0x6f899020, 84 },
    { 0x27b99023, 98 },
    { 0x3c29901e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93799028, 66 },
    { 0xb8d99029, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df9902c, 98 },
    { 0, 0 },
    { 0xeff9902e, 23 },
    { 0xc1e9902f, 36 },
    { 0x64499030, 52 },
    { 0x2269902e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1099038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e79903c, 85 },
    { 0x2539903d, 100 },
    { 0, 0 },
    { 0x9479903f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a99043, 98 },
    { 0, 0 },
    { 0xdcc99045, 68 },
    { 0x56799046, 33 },
    { 0x4d999047, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x129904a, 98 },
    { 0xe549904b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a99056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a599064, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e99067, 99 },
    { 0, 0 },
    { 0x66e99069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa9906e, 64 },
    { 0, 0 },
    { 0x8999070, 23 },
    { 0x11699070, 61 },
    { 0x7cf99071, 61 },
    { 0x2f199073, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a899076, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x92199079, 98 },
    { 0xe189907a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66999080, 76 },
    { 0xdee99081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d99084, 90 },
    { 0xa5399085, 95 },
    { 0x72d99086, 90 },
    { 0, 0 },
    { 0x21299088, 33 },
    { 0x7899089, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4379908f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6699095, 85 },
    { 0xd9499096, 90 },
    { 0x24799097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109909b, 98 },
    { 0x2519909c, 43 },
    { 0, 0 },
    { 0x79d9909e, 19 },
    { 0xe7b9909f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8990a5, 33 },
    { 0xc63990a6, 95 },
    { 0x808990a7, 98 },
    { 0, 0 },
    { 0x5b3990a9, 90 },
    { 0, 0 },
    { 0x864990ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870990b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0990b5, 74 },
    { 0, 0 },
    { 0xa03990b7, 40 },
    { 0xf8a990b7, 34 },
    { 0x4fe990b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509990c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x324990c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9990c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1990cd, 67 },
    { 0x63d990ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6990d5, 95 },
    { 0x60f990d6, 23 },
    { 0x53b990d6, 99 },
    { 0, 0 },
    { 0xd3c990d9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0990dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812990e2, 34 },
    { 0x4bf990e3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a990e8, 34 },
    { 0x746990e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede990ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f990f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc990f5, 90 },
    { 0x70990f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa990fe, 33 },
    { 0xa4b990ff, 98 },
    { 0x26599100, 33 },
    { 0xe1099101, 90 },
    { 0, 0 },
    { 0xf3e99103, 90 },
    { 0, 0 },
    { 0xd3699105, 95 },
    { 0x80999106, 61 },
    { 0xb6699107, 34 },
    { 0x86a99108, 55 },
    { 0xe6c99108, 33 },
    { 0x729910a, 33 },
    { 0xc109910b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5459910f, 33 },
    { 0xa8e99110, 33 },
    { 0x88299110, 43 },
    { 0x1899111, 61 },
    { 0x24f99113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f99117, 90 },
    { 0xd7f99117, 33 },
    { 0, 0 },
    { 0x56f9911a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd349911d, 33 },
    { 0, 0 },
    { 0x56e9911f, 98 },
    { 0, 0 },
    { 0xd4699121, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d099125, 98 },
    { 0x9599126, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad9912c, 55 },
    { 0, 0 },
    { 0x26d9912e, 95 },
    { 0, 0 },
    { 0x9f499130, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a999133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a399138, 61 },
    { 0x66199138, 61 },
    { 0x6b79913a, 55 },
    { 0x9eb9913b, 23 },
    { 0x94499138, 98 },
    { 0xffc9913d, 76 },
    { 0xd799913d, 33 },
    { 0x7839913d, 46 },
    { 0x15099140, 90 },
    { 0xe0599140, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c799149, 90 },
    { 0xd829914a, 26 },
    { 0x7589914a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f99150, 33 },
    { 0x1a899151, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c99156, 61 },
    { 0xb3c99156, 23 },
    { 0x2c299158, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa629915c, 66 },
    { 0, 0 },
    { 0xa979915e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7899163, 61 },
    { 0, 0 },
    { 0xa1d99165, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f099169, 85 },
    { 0x51999169, 76 },
    { 0, 0 },
    { 0x91f9916c, 61 },
    { 0x49a9916d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8199170, 33 },
    { 0x4de99171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42299174, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f99177, 33 },
    { 0x8f99177, 84 },
    { 0x55699179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da9917e, 46 },
    { 0x5139917f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c99182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec599185, 61 },
    { 0x57599186, 76 },
    { 0x7ce99187, 55 },
    { 0, 0 },
    { 0x6a099189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2899190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a99197, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1991a3, 41 },
    { 0, 0 },
    { 0x9ac991a5, 67 },
    { 0, 0 },
    { 0x92a991a7, 95 },
    { 0, 0 },
    { 0xaba991a9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74991af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e991b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef991ba, 59 },
    { 0x5cb991ba, 91 },
    { 0x24d991bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2991c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67991c3, 98 },
    { 0x7ec991c4, 98 },
    { 0xd86991c5, 34 },
    { 0x974991c5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c991ca, 90 },
    { 0x7e991cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6991ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615991d2, 76 },
    { 0x42b991d3, 100 },
    { 0x2dd991d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4991d8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x46991db, 90 },
    { 0, 0 },
    { 0x603991dd, 76 },
    { 0x4f2991de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x222991e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20991e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e991ec, 61 },
    { 0x75991ed, 75 },
    { 0x86f991ee, 55 },
    { 0, 0 },
    { 0x2ff991f0, 90 },
    { 0x8e5991f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a991f7, 68 },
    { 0xa7d991f8, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c99200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27e99203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed999206, 23 },
    { 0x12b99207, 33 },
    { 0xc7299208, 33 },
    { 0xa499207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5299212, 23 },
    { 0x9ae99213, 15 },
    { 0x5b99212, 98 },
    { 0xd7699215, 43 },
    { 0x49399216, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b899219, 87 },
    { 0x2909921a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26199220, 76 },
    { 0, 0 },
    { 0xe2599222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b9922b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7299922f, 61 },
    { 0x7ce99230, 98 },
    { 0xeb99231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2499234, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6559923d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e999242, 90 },
    { 0x96599243, 56 },
    { 0x45399243, 61 },
    { 0x74299245, 23 },
    { 0x55a99244, 90 },
    { 0, 0 },
    { 0xeff99248, 94 },
    { 0xc3c99249, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b9924c, 38 },
    { 0x2a09924d, 61 },
    { 0x8289924d, 95 },
    { 0xb269924c, 75 },
    { 0x9e199250, 95 },
    { 0x69499250, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b999254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98399258, 65 },
    { 0x68b99258, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7599260, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f499263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c99266, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c9926a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b9926e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbba99277, 46 },
    { 0xe6399278, 61 },
    { 0x6d899279, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1179927e, 87 },
    { 0xe039927e, 98 },
    { 0x9f299280, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0299283, 61 },
    { 0x59e99284, 95 },
    { 0x13799283, 98 },
    { 0xf0b99286, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429928b, 95 },
    { 0, 0 },
    { 0xf4b9928d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0399291, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0299297, 90 },
    { 0xa3e99297, 23 },
    { 0, 0 },
    { 0xbf39929a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7319929f, 33 },
    { 0x9929929f, 33 },
    { 0xa49992a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7992af, 40 },
    { 0xd0f992b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2992b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918992ba, 95 },
    { 0, 0 },
    { 0x11e992bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8992c0, 33 },
    { 0x386992c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80992c7, 98 },
    { 0, 0 },
    { 0xa01992c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a992cd, 33 },
    { 0xe40992ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7992d2, 88 },
    { 0x494992d3, 66 },
    { 0x2fd992d4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d992da, 95 },
    { 0x741992da, 55 },
    { 0x4f6992da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0992df, 95 },
    { 0x727992e0, 23 },
    { 0xdd8992e1, 61 },
    { 0xd0992e1, 33 },
    { 0x15f992e3, 43 },
    { 0, 0 },
    { 0xab0992e5, 34 },
    { 0xe9992e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214992ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x497992ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873992fe, 61 },
    { 0x587992ff, 46 },
    { 0x585992fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40499305, 46 },
    { 0x7dd99305, 95 },
    { 0x9099307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1779930d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa699313, 78 },
    { 0x37999313, 33 },
    { 0xdb499313, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8189931a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7649931d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5699322, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc99325, 90 },
    { 0x52299326, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f79932b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b19932f, 40 },
    { 0, 0 },
    { 0x31a99331, 61 },
    { 0x23199332, 93 },
    { 0x2a799333, 85 },
    { 0, 0 },
    { 0x8499335, 98 },
    { 0xc9a99335, 100 },
    { 0xccd99335, 33 },
    { 0xcc799338, 61 },
    { 0x89f99339, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b99344, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f99347, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759934a, 84 },
    { 0x919934b, 98 },
    { 0, 0 },
    { 0xa2c9934d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7899350, 61 },
    { 0xdba99350, 95 },
    { 0, 0 },
    { 0xfc399353, 33 },
    { 0, 0 },
    { 0xecb99355, 90 },
    { 0xa9799355, 76 },
    { 0xef199357, 76 },
    { 0xba999355, 76 },
    { 0xc9e99356, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b9935c, 52 },
    { 0x42c9935d, 61 },
    { 0x5469935d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b99366, 93 },
    { 0xa3c99367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489936c, 61 },
    { 0, 0 },
    { 0xf29936e, 98 },
    { 0, 0 },
    { 0xabf99370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae99374, 46 },
    { 0x6a299375, 55 },
    { 0, 0 },
    { 0x54799377, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca49937c, 83 },
    { 0, 0 },
    { 0xa089937e, 43 },
    { 0x3e29937f, 95 },
    { 0xcf999380, 99 },
    { 0, 0 },
    { 0xc5b99382, 90 },
    { 0x43699383, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d099386, 34 },
    { 0x20399387, 90 },
    { 0x68299388, 98 },
    { 0x25699389, 61 },
    { 0x5ce9938a, 98 },
    { 0, 0 },
    { 0xb6c9938c, 34 },
    { 0xf3d9938c, 61 },
    { 0, 0 },
    { 0x6979938f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f99393, 93 },
    { 0xbbc99394, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca99399, 46 },
    { 0x9e79939a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d9939e, 85 },
    { 0x2859939f, 61 },
    { 0xfea9939f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266993a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406993ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18993af, 23 },
    { 0xee6993b0, 66 },
    { 0, 0 },
    { 0xa84993b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e993b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71c993c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa993c6, 91 },
    { 0x3ef993c6, 61 },
    { 0, 0 },
    { 0xf8a993c9, 90 },
    { 0x207993c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4993d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d993d8, 99 },
    { 0x75c993d9, 93 },
    { 0x189993da, 43 },
    { 0, 0 },
    { 0xe91993dc, 33 },
    { 0x4dc993dd, 98 },
    { 0x38f993de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f993e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3993eb, 91 },
    { 0xf32993ec, 61 },
    { 0x19993ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9993f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9f993f4, 34 },
    { 0x76e993f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332993f9, 91 },
    { 0, 0 },
    { 0xd14993fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b993ff, 90 },
    { 0x41499400, 78 },
    { 0x49f99401, 33 },
    { 0x28999402, 98 },
    { 0xe5499402, 91 },
    { 0, 0 },
    { 0x71b99405, 99 },
    { 0x80a99406, 66 },
    { 0xdb599405, 33 },
    { 0xf9099408, 23 },
    { 0x89499407, 93 },
    { 0xde89940a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77999413, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1899416, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0399419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d09941d, 52 },
    { 0x6ca9941e, 43 },
    { 0x2a09941e, 34 },
    { 0, 0 },
    { 0x91799421, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09942e, 33 },
    { 0xa6c9942f, 46 },
    { 0xa149942e, 43 },
    { 0xa4699431, 84 },
    { 0xea399431, 43 },
    { 0, 0 },
    { 0xb0399434, 99 },
    { 0xc7f99434, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29943b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9943e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a99441, 61 },
    { 0x8f099442, 90 },
    { 0, 0 },
    { 0x3ac99444, 95 },
    { 0, 0 },
    { 0x51899446, 93 },
    { 0xc9f99447, 33 },
    { 0x21499448, 95 },
    { 0x59599449, 33 },
    { 0x20199448, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe589944e, 95 },
    { 0, 0 },
    { 0xc4b99450, 98 },
    { 0xd899451, 53 },
    { 0, 0 },
    { 0x37b99453, 33 },
    { 0x89999453, 90 },
    { 0x74699455, 98 },
    { 0, 0 },
    { 0xc3a99457, 90 },
    { 0, 0 },
    { 0x50e99459, 33 },
    { 0xf229945a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9945d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa99462, 55 },
    { 0x30099463, 17 },
    { 0, 0 },
    { 0x63a99465, 78 },
    { 0x38b99465, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829946a, 43 },
    { 0, 0 },
    { 0x9fc9946c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d99474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31999477, 95 },
    { 0, 0 },
    { 0x3b999479, 23 },
    { 0, 0 },
    { 0xfc09947b, 23 },
    { 0, 0 },
    { 0x45e9947d, 95 },
    { 0x8319947e, 61 },
    { 0xc369947e, 93 },
    { 0xc789947f, 55 },
    { 0xaf899481, 61 },
    { 0x56399482, 61 },
    { 0x83899483, 84 },
    { 0xa3c99482, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d199487, 52 },
    { 0, 0 },
    { 0xf6099489, 33 },
    { 0xa2f9948a, 42 },
    { 0xe419948b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a99490, 98 },
    { 0, 0 },
    { 0xda699492, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c199495, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb59949c, 33 },
    { 0x97c9949d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2994a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d994ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb994b0, 65 },
    { 0xcca994b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3994b4, 61 },
    { 0xa9e994b5, 76 },
    { 0x843994b4, 55 },
    { 0, 0 },
    { 0xef994b8, 90 },
    { 0x7c2994b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x388994bc, 33 },
    { 0x9c1994bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4994c2, 55 },
    { 0, 0 },
    { 0xd6f994c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e994c9, 90 },
    { 0, 0 },
    { 0xf78994cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f994d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d994d6, 23 },
    { 0x9df994d6, 23 },
    { 0, 0 },
    { 0x3c2994d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb994dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b994df, 36 },
    { 0x94b994e0, 90 },
    { 0x561994e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb994e4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x897994e7, 93 },
    { 0xf3a994e7, 90 },
    { 0x1994e9, 53 },
    { 0xad1994e8, 43 },
    { 0x6ee994eb, 91 },
    { 0xe8f994ec, 67 },
    { 0x17c994eb, 61 },
    { 0x89e994ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb994f1, 52 },
    { 0x226994f2, 36 },
    { 0xa9d994f3, 91 },
    { 0xcd9994f3, 93 },
    { 0x6d7994f1, 52 },
    { 0xc52994f6, 43 },
    { 0xd7f994f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f994ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad99502, 61 },
    { 0xed999502, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9399507, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af99510, 77 },
    { 0xf7d99510, 90 },
    { 0xe8e99512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e199516, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x39999519, 98 },
    { 0, 0 },
    { 0x2719951b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d99523, 46 },
    { 0x5de99524, 34 },
    { 0x57799523, 33 },
    { 0, 0 },
    { 0x1a699527, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d99530, 33 },
    { 0x8af99531, 34 },
    { 0, 0 },
    { 0x99d99533, 84 },
    { 0x35799533, 90 },
    { 0x1b099535, 55 },
    { 0xc6e99535, 23 },
    { 0x37b99537, 44 },
    { 0x7a599534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b99953d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d99540, 33 },
    { 0xdfc99541, 33 },
    { 0x77199542, 87 },
    { 0x75499540, 33 },
    { 0, 0 },
    { 0xd5d99545, 92 },
    { 0xe8999546, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3779954d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c499555, 93 },
    { 0x5b099556, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9955a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4179955d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc099564, 88 },
    { 0xc8599565, 33 },
    { 0x94599565, 90 },
    { 0, 0 },
    { 0x399568, 43 },
    { 0x91a99569, 90 },
    { 0x93299569, 76 },
    { 0xbf399568, 33 },
    { 0x5d39956c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f49956f, 46 },
    { 0, 0 },
    { 0xee099571, 76 },
    { 0xa9899571, 61 },
    { 0x19599572, 34 },
    { 0x79499573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75299577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a79957a, 78 },
    { 0, 0 },
    { 0x819957c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f09958a, 95 },
    { 0xcac9958b, 90 },
    { 0x94b9958b, 23 },
    { 0, 0 },
    { 0x56c9958e, 95 },
    { 0, 0 },
    { 0x2fc99590, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb299593, 98 },
    { 0xabc99593, 83 },
    { 0x20699595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8699598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc9959b, 98 },
    { 0x7259959b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608995a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55995a6, 99 },
    { 0x1b9995a7, 55 },
    { 0x655995a7, 98 },
    { 0x1ec995a8, 95 },
    { 0xcc1995a8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d995ae, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84995b3, 84 },
    { 0xa40995b4, 78 },
    { 0x230995b5, 33 },
    { 0x388995b6, 61 },
    { 0x55995b7, 61 },
    { 0xaab995b8, 93 },
    { 0xc5e995b8, 55 },
    { 0xefb995ba, 46 },
    { 0x5ef995bb, 95 },
    { 0xa8b995b9, 85 },
    { 0xcd995bc, 66 },
    { 0x728995be, 61 },
    { 0xbea995b5, 95 },
    { 0xf5c995b6, 98 },
    { 0, 0 },
    { 0x750995c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d995c7, 76 },
    { 0x212995c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861995ce, 66 },
    { 0xd49995cf, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29995d3, 23 },
    { 0xd29995d4, 87 },
    { 0x339995d3, 98 },
    { 0x188995d6, 96 },
    { 0x545995d4, 75 },
    { 0xec5995d8, 81 },
    { 0xdc5995d8, 93 },
    { 0x239995d9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04995e9, 98 },
    { 0x566995e9, 61 },
    { 0x92f995eb, 90 },
    { 0xd2a995eb, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13995ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592995f3, 88 },
    { 0x71e995f3, 95 },
    { 0xdd0995f5, 33 },
    { 0xa4e995f5, 33 },
    { 0, 0 },
    { 0xd28995f8, 95 },
    { 0xc4d995f8, 98 },
    { 0xb12995fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe399600, 75 },
    { 0x83a99601, 55 },
    { 0x5399602, 34 },
    { 0x7cc99601, 38 },
    { 0x41f99604, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd99607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f9960b, 95 },
    { 0x5f89960c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4789960f, 90 },
    { 0x91699610, 76 },
    { 0x82e99611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f199614, 95 },
    { 0xca899615, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9961e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e9962c, 98 },
    { 0xe8f9962d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da99633, 43 },
    { 0x2c699634, 23 },
    { 0x59599633, 75 },
    { 0x5dd99636, 93 },
    { 0x32699636, 78 },
    { 0x84599638, 99 },
    { 0xdec99639, 98 },
    { 0xa7d99633, 100 },
    { 0x6919963b, 61 },
    { 0, 0 },
    { 0xa0e9963d, 95 },
    { 0x56d9963e, 76 },
    { 0xad9963f, 93 },
    { 0, 0 },
    { 0x35799641, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9399644, 23 },
    { 0, 0 },
    { 0xb4e99646, 52 },
    { 0x94899646, 33 },
    { 0xbae99647, 61 },
    { 0x6499649, 53 },
    { 0x4849964a, 17 },
    { 0x2329964b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe999652, 98 },
    { 0x2d199653, 23 },
    { 0xdcd99653, 90 },
    { 0x6e899653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0699659, 98 },
    { 0x4ec9965a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f299661, 90 },
    { 0xa1599661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd99665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d9966a, 93 },
    { 0, 0 },
    { 0xfbc9966c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb539966f, 90 },
    { 0xac799670, 61 },
    { 0, 0 },
    { 0x6e399672, 34 },
    { 0x49e99673, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9659967b, 55 },
    { 0, 0 },
    { 0xb979967d, 85 },
    { 0xa5f9967e, 98 },
    { 0x8609967e, 43 },
    { 0xa5299680, 97 },
    { 0xc689967f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d99685, 61 },
    { 0, 0 },
    { 0xadc99687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5859968a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf199691, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b99695, 91 },
    { 0x7c599696, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd99699, 34 },
    { 0x9a699699, 33 },
    { 0x81399699, 95 },
    { 0x4a49969a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68996a9, 98 },
    { 0, 0 },
    { 0x8d5996ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae996b1, 55 },
    { 0, 0 },
    { 0xfad996b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf996b8, 61 },
    { 0xfcf996b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x288996bc, 61 },
    { 0, 0 },
    { 0xa4f996be, 95 },
    { 0x99c996bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81996c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb996c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b996ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea996d1, 90 },
    { 0, 0 },
    { 0x4f6996d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c996d7, 76 },
    { 0x8de996d7, 33 },
    { 0xec7996d9, 33 },
    { 0x25d996da, 33 },
    { 0x379996db, 55 },
    { 0x9f7996dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e996e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922996e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e996e8, 78 },
    { 0xc1a996e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76996ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15996f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c996f4, 64 },
    { 0x447996f5, 52 },
    { 0x262996f6, 46 },
    { 0xe51996f7, 33 },
    { 0x3a3996f8, 95 },
    { 0xc68996f5, 23 },
    { 0, 0 },
    { 0x7d7996fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce99701, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x81099704, 88 },
    { 0, 0 },
    { 0x6e799706, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4539970a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb9970f, 87 },
    { 0x61499710, 52 },
    { 0x2bf9970f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79d9971c, 90 },
    { 0xb599971d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5299721, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc899724, 33 },
    { 0xd6c99725, 33 },
    { 0xb8099726, 91 },
    { 0, 0 },
    { 0x9e999728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1079972b, 95 },
    { 0x1ff9972c, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b99730, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22099734, 23 },
    { 0xddf99735, 77 },
    { 0x97d99736, 100 },
    { 0xd7299734, 88 },
    { 0, 0 },
    { 0xdc99739, 61 },
    { 0x13e9973a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919973e, 90 },
    { 0x4c99973f, 33 },
    { 0xef29973f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc99748, 61 },
    { 0xc7999749, 40 },
    { 0x7819974a, 95 },
    { 0, 0 },
    { 0x56f9974c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeb9975a, 98 },
    { 0xf629975a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb9975f, 95 },
    { 0, 0 },
    { 0xc1a99761, 100 },
    { 0x52699761, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb899765, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479976b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x51a9976e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46899775, 76 },
    { 0x7a299776, 99 },
    { 0, 0 },
    { 0x73799778, 65 },
    { 0x67c99779, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe739977c, 90 },
    { 0xe889977c, 90 },
    { 0, 0 },
    { 0x8a29977f, 98 },
    { 0xce999780, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2399783, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b9978e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999791, 100 },
    { 0xb099792, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53699797, 61 },
    { 0x8ca99798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec9979d, 95 },
    { 0x9289979d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71997a2, 91 },
    { 0x88b997a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5997a6, 14 },
    { 0x65997a7, 93 },
    { 0x8d7997a7, 33 },
    { 0x962997a8, 90 },
    { 0x984997aa, 84 },
    { 0, 0 },
    { 0x41b997ac, 87 },
    { 0xb6e997ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375997b4, 88 },
    { 0x4c5997b4, 80 },
    { 0xff4997b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5997b9, 53 },
    { 0x8ee997ba, 34 },
    { 0xb87997bb, 33 },
    { 0, 0 },
    { 0x3a8997bd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbe997c0, 61 },
    { 0xd7f997c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27f997c9, 88 },
    { 0x3bd997ca, 98 },
    { 0x949997cb, 34 },
    { 0x56b997cc, 87 },
    { 0x3b7997cd, 85 },
    { 0x420997ce, 23 },
    { 0xc18997cf, 61 },
    { 0x6e1997cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205997d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4d997da, 95 },
    { 0x2e997db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e997e3, 23 },
    { 0x43e997e4, 85 },
    { 0xf0a997e5, 34 },
    { 0xf0997e5, 80 },
    { 0, 0 },
    { 0x775997e8, 34 },
    { 0, 0 },
    { 0xe3a997ea, 90 },
    { 0xb2d997ea, 93 },
    { 0, 0 },
    { 0x362997ed, 33 },
    { 0x7f9997ed, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691997f5, 95 },
    { 0x45c997f6, 23 },
    { 0x459997f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd997ff, 43 },
    { 0x3ee99800, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2899807, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3269980a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64699811, 33 },
    { 0x78799812, 90 },
    { 0x34c99811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4699818, 76 },
    { 0x7e899819, 55 },
    { 0x51799819, 55 },
    { 0x3589981b, 88 },
    { 0xef9981c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46599820, 93 },
    { 0x7ea99820, 85 },
    { 0, 0 },
    { 0x6a899823, 33 },
    { 0, 0 },
    { 0xae799825, 61 },
    { 0x8e999825, 38 },
    { 0xc3399825, 61 },
    { 0xe0299826, 23 },
    { 0xa2599827, 84 },
    { 0, 0 },
    { 0x7d09982b, 43 },
    { 0xfad9982c, 23 },
    { 0, 0 },
    { 0xe1f9982e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb99838, 33 },
    { 0xc7d99839, 46 },
    { 0xa049983a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d99840, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3b99844, 33 },
    { 0x54b99845, 90 },
    { 0, 0 },
    { 0x4aa99847, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd309984d, 67 },
    { 0, 0 },
    { 0xde59984f, 99 },
    { 0x2819984f, 55 },
    { 0x1ef9984f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeb99856, 61 },
    { 0x82b99856, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c99985c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58399861, 97 },
    { 0x25a99862, 61 },
    { 0x22999862, 90 },
    { 0x79499863, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23899868, 90 },
    { 0xf7099869, 90 },
    { 0, 0 },
    { 0x7759986b, 66 },
    { 0x9d9986c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d99870, 23 },
    { 0x41299871, 75 },
    { 0, 0 },
    { 0xc5999873, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4699876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a99879, 61 },
    { 0xe0899879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979987d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9299882, 95 },
    { 0xba799882, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a399886, 33 },
    { 0, 0 },
    { 0xe6199888, 61 },
    { 0x38b99889, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e59988d, 33 },
    { 0xe3d9988d, 98 },
    { 0x2979988f, 87 },
    { 0, 0 },
    { 0x50b99891, 55 },
    { 0, 0 },
    { 0x93e99893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95099896, 98 },
    { 0xdfb99897, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c9989d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0998a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x177998a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x875998a6, 61 },
    { 0xe00998a7, 66 },
    { 0xeac998a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8998ab, 67 },
    { 0, 0 },
    { 0xc79998ad, 90 },
    { 0, 0 },
    { 0x71998af, 95 },
    { 0x3ab998b0, 55 },
    { 0xf08998b0, 98 },
    { 0xc19998b2, 91 },
    { 0x8c8998b3, 58 },
    { 0, 0 },
    { 0x3b998b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a998bc, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8998bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6998c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cc998c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52998c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463998d3, 90 },
    { 0x518998d4, 33 },
    { 0xc51998d5, 61 },
    { 0xdee998d6, 66 },
    { 0xc74998d7, 46 },
    { 0x17f998d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71998df, 66 },
    { 0xc5b998e0, 17 },
    { 0xd0998e1, 67 },
    { 0xbb8998e2, 61 },
    { 0xb7a998e3, 52 },
    { 0x2e998df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05998eb, 23 },
    { 0, 0 },
    { 0x859998ed, 95 },
    { 0x648998ed, 61 },
    { 0xa34998ef, 52 },
    { 0, 0 },
    { 0x606998f1, 95 },
    { 0xf00998f2, 61 },
    { 0, 0 },
    { 0x3c9998f4, 90 },
    { 0, 0 },
    { 0x500998f6, 23 },
    { 0x1e5998f6, 17 },
    { 0xfa0998f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x989998fb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d998fe, 68 },
    { 0xd41998ff, 95 },
    { 0x9c299900, 23 },
    { 0xc9199901, 98 },
    { 0xf2599902, 98 },
    { 0xfd899903, 76 },
    { 0xf0099901, 34 },
    { 0, 0 },
    { 0x95499906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2999909, 61 },
    { 0x8359990a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd719990d, 68 },
    { 0xeb89990d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7599917, 43 },
    { 0x8999918, 78 },
    { 0, 0 },
    { 0x8949991a, 76 },
    { 0x7489991a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe419991e, 52 },
    { 0x6009991f, 95 },
    { 0, 0 },
    { 0x80399921, 46 },
    { 0x28b99921, 61 },
    { 0x59899921, 90 },
    { 0, 0 },
    { 0xb2f99925, 33 },
    { 0xc4399925, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd9992c, 95 },
    { 0x7d29992d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5899930, 95 },
    { 0x1a99931, 90 },
    { 0x77199932, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19499936, 52 },
    { 0x20199936, 43 },
    { 0x10899938, 90 },
    { 0, 0 },
    { 0xe919993a, 90 },
    { 0, 0 },
    { 0xaa9993c, 90 },
    { 0x57c9993d, 80 },
    { 0x3919993e, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64699942, 87 },
    { 0x7799942, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e99949, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd99953, 23 },
    { 0xfa799954, 23 },
    { 0, 0 },
    { 0xf6099956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6809995b, 61 },
    { 0x58a9995c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c699961, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d99966, 95 },
    { 0x22c99966, 66 },
    { 0, 0 },
    { 0xc2c99969, 61 },
    { 0x9bc9996a, 66 },
    { 0x5cd9996a, 33 },
    { 0x159996c, 90 },
    { 0xf0599969, 78 },
    { 0x22b99969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b99974, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0499977, 99 },
    { 0, 0 },
    { 0x5e699979, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe9997c, 67 },
    { 0x96a9997d, 90 },
    { 0x2379997e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c99986, 33 },
    { 0x34399986, 64 },
    { 0, 0 },
    { 0x4b99989, 55 },
    { 0x3d99998a, 90 },
    { 0x1d399989, 33 },
    { 0xbd19998c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9079998f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1899992, 33 },
    { 0, 0 },
    { 0xb2d99994, 40 },
    { 0, 0 },
    { 0x84a99996, 33 },
    { 0x4ef99996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee9999a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf69999d, 33 },
    { 0x1399999e, 55 },
    { 0x4729999f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1999a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19999a8, 99 },
    { 0xcd4999a8, 61 },
    { 0, 0 },
    { 0xe69999ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8999b0, 95 },
    { 0x2c6999b1, 93 },
    { 0, 0 },
    { 0xc16999b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3999b7, 53 },
    { 0xbf2999b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef999bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372999c2, 61 },
    { 0xb70999c3, 76 },
    { 0x43a999c4, 99 },
    { 0x172999c2, 75 },
    { 0xfab999c6, 98 },
    { 0xf72999c7, 23 },
    { 0, 0 },
    { 0x688999c9, 76 },
    { 0xf0999ca, 33 },
    { 0xba5999cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19999cf, 33 },
    { 0xe2b999d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3999d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228999d7, 90 },
    { 0x2f3999d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74999dd, 61 },
    { 0x7af999de, 88 },
    { 0, 0 },
    { 0xaff999e0, 87 },
    { 0, 0 },
    { 0x574999e2, 95 },
    { 0x3c1999e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e999e7, 33 },
    { 0, 0 },
    { 0x632999e9, 84 },
    { 0, 0 },
    { 0xab6999eb, 76 },
    { 0xdee999ec, 99 },
    { 0, 0 },
    { 0x21b999ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e999f4, 76 },
    { 0, 0 },
    { 0xdea999f6, 34 },
    { 0xed9999f6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a999ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c99a0a, 34 },
    { 0, 0 },
    { 0x57b99a0c, 34 },
    { 0xaf799a0d, 34 },
    { 0xc0499a0e, 55 },
    { 0, 0 },
    { 0x82e99a10, 99 },
    { 0x5f199a10, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d399a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40699a18, 33 },
    { 0x54999a19, 33 },
    { 0x3ae99a1a, 95 },
    { 0, 0 },
    { 0x18999a1c, 78 },
    { 0x9c099a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c799a22, 66 },
    { 0, 0 },
    { 0x38899a24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba599a29, 34 },
    { 0, 0 },
    { 0x20599a2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0799a32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b899a35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d99a3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2899a42, 61 },
    { 0, 0 },
    { 0x35e99a44, 61 },
    { 0x1f899a44, 91 },
    { 0x3dd99a46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d399a49, 90 },
    { 0xbff99a49, 33 },
    { 0xce399a4a, 33 },
    { 0x1a99a4c, 98 },
    { 0xfcd99a4b, 90 },
    { 0x72b99a4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40799a53, 46 },
    { 0x9d199a54, 90 },
    { 0x83499a55, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b99a58, 67 },
    { 0, 0 },
    { 0xb4c99a5a, 17 },
    { 0, 0 },
    { 0x7899a5c, 61 },
    { 0, 0 },
    { 0xd0399a5e, 87 },
    { 0x37999a5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a99a67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20199a6c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42399a70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cc99a73, 33 },
    { 0, 0 },
    { 0xa3299a75, 98 },
    { 0x7c099a76, 90 },
    { 0, 0 },
    { 0x63599a78, 33 },
    { 0, 0 },
    { 0x6c199a7a, 76 },
    { 0x52599a7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef799a80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x59699a83, 52 },
    { 0xcee99a84, 17 },
    { 0x6cc99a83, 52 },
    { 0, 0 },
    { 0x98b99a87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7799a8a, 91 },
    { 0, 0 },
    { 0xf9e99a8c, 34 },
    { 0, 0 },
    { 0x82599a8e, 33 },
    { 0, 0 },
    { 0x4a699a90, 52 },
    { 0xc1899a91, 90 },
    { 0x85999a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3699a96, 23 },
    { 0x1d99a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed299a9d, 87 },
    { 0xcfa99a9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f699aa1, 90 },
    { 0x8fd99aa2, 43 },
    { 0, 0 },
    { 0x8b999aa4, 90 },
    { 0xe9199aa4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c399aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef99aab, 78 },
    { 0xbcc99aab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd899ab0, 87 },
    { 0x61199ab1, 95 },
    { 0xb7a99ab1, 76 },
    { 0, 0 },
    { 0x1e99ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a99ab8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3699abb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c499ac0, 33 },
    { 0xd5c99ac1, 85 },
    { 0x2bf99ac1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f199ac6, 95 },
    { 0x30999ac7, 84 },
    { 0xd0599ac8, 61 },
    { 0x69b99ac9, 33 },
    { 0, 0 },
    { 0x15d99acb, 95 },
    { 0x64999acb, 43 },
    { 0, 0 },
    { 0x6f299ace, 52 },
    { 0, 0 },
    { 0x18699ad0, 90 },
    { 0x77199ad0, 33 },
    { 0xdfd99ad2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11699ad6, 95 },
    { 0xc4099ad6, 98 },
    { 0x7ee99ad6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d99adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb99ade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d99ae1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30099ae9, 33 },
    { 0xdb99ae9, 33 },
    { 0x75899aea, 76 },
    { 0, 0 },
    { 0x43599aed, 23 },
    { 0xa0499aed, 33 },
    { 0xb499aef, 52 },
    { 0xbd099aed, 40 },
    { 0x51d99af1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93099af4, 68 },
    { 0x49099af4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25899afe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8499b04, 55 },
    { 0xd3099b05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e599b09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0f99b0c, 65 },
    { 0x8c999b0c, 55 },
    { 0, 0 },
    { 0x54399b0f, 76 },
    { 0x2a299b0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be99b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9899b17, 61 },
    { 0, 0 },
    { 0x7b299b19, 84 },
    { 0x5ce99b19, 61 },
    { 0x97d99b1b, 84 },
    { 0x7cb99b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7999b22, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec399b26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b99b2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f99b31, 23 },
    { 0, 0 },
    { 0x1d799b33, 67 },
    { 0x22b99b34, 33 },
    { 0, 0 },
    { 0xf999b36, 55 },
    { 0x76c99b37, 65 },
    { 0, 0 },
    { 0x36c99b39, 33 },
    { 0, 0 },
    { 0xa99b3b, 55 },
    { 0, 0 },
    { 0xf2b99b3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b99b40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0099b45, 75 },
    { 0xcb799b46, 90 },
    { 0x21999b47, 61 },
    { 0x87e99b48, 90 },
    { 0x92b99b49, 98 },
    { 0x2e099b45, 61 },
    { 0, 0 },
    { 0xf9599b4c, 96 },
    { 0x82a99b4d, 61 },
    { 0x5c599b4d, 23 },
    { 0x22999b4f, 98 },
    { 0xc6099b50, 90 },
    { 0x6a299b51, 99 },
    { 0, 0 },
    { 0x9a299b53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e899b5c, 78 },
    { 0x41b99b5c, 33 },
    { 0x7b699b5d, 78 },
    { 0, 0 },
    { 0x12e99b60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f399b65, 99 },
    { 0x23199b65, 43 },
    { 0x11a99b65, 47 },
    { 0x39999b67, 90 },
    { 0, 0 },
    { 0xb0399b6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c99b6d, 90 },
    { 0x33d99b6e, 76 },
    { 0xab99b6f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x86999b72, 84 },
    { 0x91499b72, 95 },
    { 0xa8099b74, 90 },
    { 0xdf699b73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a99b7c, 34 },
    { 0, 0 },
    { 0xf3b99b7e, 95 },
    { 0x60299b7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c99b82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0899b89, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb99b90, 23 },
    { 0xc1c99b91, 46 },
    { 0x35d99b91, 100 },
    { 0, 0 },
    { 0xd1b99b94, 97 },
    { 0, 0 },
    { 0xa6699b96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f699b99, 33 },
    { 0x42599b99, 98 },
    { 0, 0 },
    { 0xcea99b9c, 95 },
    { 0x74a99b9d, 33 },
    { 0x12d99b9d, 77 },
    { 0x4b99b9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3799ba2, 95 },
    { 0, 0 },
    { 0x62e99ba4, 95 },
    { 0x14999ba4, 91 },
    { 0x7df99ba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac99ba9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11299bae, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3499bb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x10499bb4, 61 },
    { 0x3d099bb5, 76 },
    { 0x1599bb6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x35c99bb9, 46 },
    { 0xe7899bba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82099bbe, 40 },
    { 0, 0 },
    { 0x62d99bc0, 55 },
    { 0xa4399bc0, 95 },
    { 0xfa799bc0, 23 },
    { 0x82f99bc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a99bc7, 91 },
    { 0x1f099bc7, 95 },
    { 0xe7899bc9, 90 },
    { 0x77199bc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee199bcf, 95 },
    { 0, 0 },
    { 0x94a99bd1, 43 },
    { 0xfff99bd1, 23 },
    { 0x6ce99bd3, 66 },
    { 0x8f99bd1, 40 },
    { 0, 0 },
    { 0x32a99bd6, 55 },
    { 0x50899bd7, 93 },
    { 0x3c999bd7, 64 },
    { 0x68899bd7, 98 },
    { 0, 0 },
    { 0x1ff99bdb, 23 },
    { 0, 0 },
    { 0x2b99bdd, 61 },
    { 0x31a99bde, 90 },
    { 0x32199bdf, 98 },
    { 0x71099bde, 87 },
    { 0xdce99be1, 78 },
    { 0xdb699be2, 95 },
    { 0x22499be3, 88 },
    { 0x87699be3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14599bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24299bed, 55 },
    { 0x2f799bee, 55 },
    { 0x4ce99bef, 90 },
    { 0x35e99bf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db99bf3, 56 },
    { 0xe9999bf4, 69 },
    { 0x14b99bf4, 98 },
    { 0x599bf4, 36 },
    { 0x65999bf3, 98 },
    { 0, 0 },
    { 0x1c299bf9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59399bfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36199c00, 90 },
    { 0xabd99c00, 90 },
    { 0x5b099c02, 98 },
    { 0x61099c03, 90 },
    { 0x81f99c04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf99c09, 23 },
    { 0x94899c0a, 93 },
    { 0x35b99c0b, 33 },
    { 0x7b499c0c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9299c0f, 76 },
    { 0xfb799c0f, 61 },
    { 0, 0 },
    { 0x9fa99c12, 67 },
    { 0x33999c13, 90 },
    { 0xeee99c14, 90 },
    { 0xf6f99c14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf199c19, 23 },
    { 0, 0 },
    { 0x64b99c1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c99c23, 23 },
    { 0xab899c24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0999c2d, 33 },
    { 0, 0 },
    { 0xb9399c2f, 46 },
    { 0x87c99c2f, 51 },
    { 0x53399c31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d699c35, 23 },
    { 0xe2d99c36, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a99c45, 69 },
    { 0, 0 },
    { 0x6f799c47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa99c4a, 33 },
    { 0, 0 },
    { 0x1bf99c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0399c50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf399c56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28e99c59, 90 },
    { 0x74999c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b99c5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6099c62, 55 },
    { 0xb8999c63, 95 },
    { 0x8c699c64, 61 },
    { 0x51499c64, 23 },
    { 0x68399c66, 90 },
    { 0x22599c67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e99c6a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e399c6d, 33 },
    { 0x56499c6e, 43 },
    { 0x8cb99c6f, 43 },
    { 0x2c199c70, 43 },
    { 0x2b199c70, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f599c75, 87 },
    { 0xf5d99c76, 52 },
    { 0x5dd99c76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d99c7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73899c7f, 81 },
    { 0x48899c80, 46 },
    { 0x3899c81, 98 },
    { 0, 0 },
    { 0xa9399c83, 61 },
    { 0, 0 },
    { 0x22499c85, 34 },
    { 0, 0 },
    { 0x38899c87, 90 },
    { 0x1c699c88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61a99c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48499c8f, 61 },
    { 0, 0 },
    { 0xec899c91, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x63f99c94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb299ca0, 61 },
    { 0x98699ca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30199ca6, 95 },
    { 0xc7099ca7, 46 },
    { 0x4d699ca7, 98 },
    { 0x71b99ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1599cb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98699cba, 76 },
    { 0, 0 },
    { 0x11399cbc, 84 },
    { 0xdc299cbc, 33 },
    { 0x43b99cbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e99cc3, 95 },
    { 0, 0 },
    { 0x31099cc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e99cc8, 98 },
    { 0x83599cc9, 46 },
    { 0x1999cc9, 52 },
    { 0x35b99ccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b99cd0, 87 },
    { 0x35499cd1, 46 },
    { 0x64b99cd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78099cd7, 44 },
    { 0, 0 },
    { 0x45c99cd9, 37 },
    { 0xb0e99cda, 90 },
    { 0x10099cdb, 90 },
    { 0, 0 },
    { 0x2a99cdd, 90 },
    { 0xcb499cdd, 61 },
    { 0xe7199cdf, 90 },
    { 0xd8599ce0, 34 },
    { 0x8b699ce1, 87 },
    { 0x899cde, 95 },
    { 0x91199ce3, 33 },
    { 0x5ca99ce4, 76 },
    { 0, 0 },
    { 0x6ca99ce6, 43 },
    { 0x7fd99ce7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a499ceb, 66 },
    { 0, 0 },
    { 0x46099ced, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31799cf0, 90 },
    { 0xa5199cf0, 61 },
    { 0x38299cf2, 61 },
    { 0, 0 },
    { 0x48099cf4, 90 },
    { 0, 0 },
    { 0x71d99cf6, 55 },
    { 0xb499cf7, 76 },
    { 0, 0 },
    { 0x48f99cf9, 92 },
    { 0x1e399cfa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c99cfe, 43 },
    { 0x5399cff, 52 },
    { 0xae99d00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b99d03, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x79599d06, 90 },
    { 0x20099d06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88799d12, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af99d1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a99d20, 23 },
    { 0x58999d20, 95 },
    { 0x4aa99d22, 22 },
    { 0xce999d22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1999d28, 61 },
    { 0x30699d28, 90 },
    { 0, 0 },
    { 0xdf599d2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b099d2e, 74 },
    { 0x21a99d2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93a99d33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a99d3a, 33 },
    { 0xe3d99d3a, 90 },
    { 0xf0299d3b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec99d41, 33 },
    { 0xa599d42, 46 },
    { 0xbf699d43, 90 },
    { 0xf7b99d44, 90 },
    { 0xf9499d43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b99d4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2999d4d, 66 },
    { 0xb7299d4e, 61 },
    { 0xd2d99d4f, 95 },
    { 0xce199d50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b99d54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdc99d57, 67 },
    { 0, 0 },
    { 0xa1f99d59, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x95499d5c, 98 },
    { 0, 0 },
    { 0x13199d5e, 99 },
    { 0x9dd99d5f, 40 },
    { 0x41199d60, 95 },
    { 0xc1599d61, 90 },
    { 0xbdc99d62, 90 },
    { 0, 0 },
    { 0x40199d64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47199d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb899d6a, 52 },
    { 0x19d99d6a, 75 },
    { 0, 0 },
    { 0x2b599d6d, 36 },
    { 0x39c99d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f99d72, 23 },
    { 0x36799d73, 61 },
    { 0, 0 },
    { 0x3c599d75, 76 },
    { 0xed699d76, 90 },
    { 0x32799d77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec99d7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30599d7f, 87 },
    { 0x3ab99d7f, 93 },
    { 0xbd099d81, 66 },
    { 0, 0 },
    { 0xc5a99d83, 61 },
    { 0xb0699d83, 33 },
    { 0, 0 },
    { 0x5df99d86, 55 },
    { 0xf3d99d87, 90 },
    { 0, 0 },
    { 0x67399d89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37b99d8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9599d95, 98 },
    { 0, 0 },
    { 0x11299d97, 81 },
    { 0x5c699d98, 55 },
    { 0, 0 },
    { 0x67a99d9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d999d9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9299da2, 99 },
    { 0xfba99da3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a999da6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d99da9, 100 },
    { 0x8d399daa, 61 },
    { 0xb8099dab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca99daf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a99db2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1799db6, 61 },
    { 0x55299db7, 99 },
    { 0, 0 },
    { 0xfe099db9, 23 },
    { 0x4fa99dba, 51 },
    { 0x31599dbb, 66 },
    { 0xd9599dbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d499dc6, 95 },
    { 0xd8f99dc7, 43 },
    { 0, 0 },
    { 0xa4599dc9, 33 },
    { 0xfd299dca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae99dcd, 88 },
    { 0, 0 },
    { 0xeb799dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11399dd2, 61 },
    { 0x15c99dd2, 88 },
    { 0x39999dd4, 61 },
    { 0, 0 },
    { 0x4ff99dd6, 90 },
    { 0x88899dd7, 55 },
    { 0x1c99dd8, 61 },
    { 0, 0 },
    { 0x21399dda, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae99dde, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b899de2, 33 },
    { 0x30a99de2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23599de8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf99dec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c99df6, 23 },
    { 0x1b999df7, 61 },
    { 0xe9199df6, 90 },
    { 0, 0 },
    { 0x51199dfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13499dfd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e399e05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db99e09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1999e0c, 34 },
    { 0xd6899e0d, 34 },
    { 0x9d499e0d, 91 },
    { 0, 0 },
    { 0x9ba99e10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78599e13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47799e1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94399e26, 33 },
    { 0x6a199e27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd599e2a, 88 },
    { 0xfbe99e2b, 78 },
    { 0xf0499e2b, 76 },
    { 0, 0 },
    { 0xa8d99e2e, 46 },
    { 0x25999e2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60299e32, 90 },
    { 0x1599e33, 76 },
    { 0xea999e32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d899e37, 90 },
    { 0x4c899e37, 98 },
    { 0xf0299e39, 23 },
    { 0, 0 },
    { 0x6ef99e3b, 90 },
    { 0x16999e3c, 61 },
    { 0xead99e3d, 66 },
    { 0x3ce99e3e, 95 },
    { 0x7af99e3f, 88 },
    { 0x12499e3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b199e45, 33 },
    { 0xda499e45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33099e49, 88 },
    { 0x2e399e49, 85 },
    { 0xf0e99e4a, 98 },
    { 0xa7f99e4c, 23 },
    { 0x199e4c, 91 },
    { 0xc0399e4e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25399e52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd599e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399e5c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1499e5f, 95 },
    { 0, 0 },
    { 0xee999e61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32099e66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c99e6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60399e72, 88 },
    { 0x8a999e73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69299e7a, 95 },
    { 0x38499e7b, 90 },
    { 0x4d899e7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde599e7f, 84 },
    { 0x34d99e80, 66 },
    { 0x36799e80, 61 },
    { 0x51e99e82, 90 },
    { 0x79799e83, 76 },
    { 0xca199e84, 23 },
    { 0xb8d99e7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b999e88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66299e91, 98 },
    { 0x9d499e92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1499e95, 23 },
    { 0x3399e96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0599e9b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc99ea3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f299ea7, 76 },
    { 0, 0 },
    { 0xaf99ea9, 78 },
    { 0, 0 },
    { 0xada99eab, 93 },
    { 0x4a799eab, 90 },
    { 0x52899eac, 95 },
    { 0x15099eab, 98 },
    { 0xf0399eab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d99eb5, 34 },
    { 0x7c999eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f99eba, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd099ec1, 90 },
    { 0xef199ec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b99ec5, 87 },
    { 0xf1899ec6, 84 },
    { 0xda599ec7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf99ed2, 87 },
    { 0xea799ed2, 46 },
    { 0x82299ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a199eda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1f99ee7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18099eea, 66 },
    { 0, 0 },
    { 0xb1499eec, 23 },
    { 0x95c99eed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22899ef0, 17 },
    { 0, 0 },
    { 0x7e899ef2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bf99ef9, 64 },
    { 0xf8c99efa, 23 },
    { 0x15399ef9, 98 },
    { 0x51a99efb, 76 },
    { 0xaaa99efd, 87 },
    { 0x92199efc, 67 },
    { 0xd7a99efa, 75 },
    { 0xaa399f00, 23 },
    { 0xb1399efc, 61 },
    { 0x55699efc, 61 },
    { 0xb8d99f03, 33 },
    { 0xdbb99f03, 85 },
    { 0xdd299f05, 95 },
    { 0xb9299f03, 90 },
    { 0xc8c99f04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a299f0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb599f0f, 23 },
    { 0xa2299f0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb799f15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f599f18, 33 },
    { 0, 0 },
    { 0x21799f1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74799f21, 55 },
    { 0x11499f22, 52 },
    { 0xfaa99f23, 34 },
    { 0x9aa99f24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f99f29, 61 },
    { 0, 0 },
    { 0xb5599f2b, 98 },
    { 0x80999f2b, 33 },
    { 0, 0 },
    { 0x8b399f2e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75099f37, 61 },
    { 0xbf499f38, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e99f3b, 92 },
    { 0xe1d99f3c, 61 },
    { 0xb5e99f3d, 46 },
    { 0, 0 },
    { 0x34399f3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc799f42, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e499f4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf999f4e, 61 },
    { 0, 0 },
    { 0x2fb99f50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97499f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8999f58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb399f5b, 99 },
    { 0x6a799f5c, 33 },
    { 0x71a99f5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78299f60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10099f64, 33 },
    { 0, 0 },
    { 0xfce99f66, 23 },
    { 0x3f999f67, 95 },
    { 0x1ce99f68, 64 },
    { 0xf3699f69, 90 },
    { 0xb3499f67, 23 },
    { 0x6fa99f6b, 85 },
    { 0xd9799f6b, 83 },
    { 0x4cd99f69, 84 },
    { 0x81099f69, 98 },
    { 0x68199f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f699f75, 55 },
    { 0x8a899f75, 76 },
    { 0x199f77, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2d99f7a, 38 },
    { 0x6db99f7a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d99f7e, 85 },
    { 0, 0 },
    { 0x65899f80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc99f88, 84 },
    { 0, 0 },
    { 0xd8c99f8a, 34 },
    { 0x5a799f8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a99f91, 87 },
    { 0, 0 },
    { 0xe5199f93, 90 },
    { 0, 0 },
    { 0xe0899f95, 81 },
    { 0x83f99f96, 95 },
    { 0xe1299f97, 78 },
    { 0x1f199f97, 61 },
    { 0x55b99f96, 66 },
    { 0xaff99f9a, 87 },
    { 0xbc899f9b, 76 },
    { 0xd5e99f9c, 76 },
    { 0x64899f9d, 87 },
    { 0xcbe99f9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed99fad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c099fb2, 61 },
    { 0xc7399fb3, 65 },
    { 0x85899fb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37499fb7, 34 },
    { 0x6b99fb7, 93 },
    { 0xa3d99fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e799fbd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96799fc1, 33 },
    { 0xc7999fc2, 84 },
    { 0x83999fc3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab99fc6, 90 },
    { 0xd399fc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a99fcb, 88 },
    { 0xad999fcc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a799fd0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa99fd5, 77 },
    { 0, 0 },
    { 0xa5e99fd7, 52 },
    { 0, 0 },
    { 0xace99fd9, 61 },
    { 0x8d699fda, 61 },
    { 0, 0 },
    { 0xdc899fdc, 88 },
    { 0x90299fdd, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa099fe4, 43 },
    { 0, 0 },
    { 0x5d899fe6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1699fe9, 90 },
    { 0xea099fe9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599fef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12999ff5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5399ffb, 95 },
    { 0xa3f99ffb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd549a004, 23 },
    { 0x7939a005, 90 },
    { 0x1489a006, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a9a00e, 90 },
    { 0x9d19a00f, 36 },
    { 0xa0f9a010, 55 },
    { 0xb849a011, 75 },
    { 0x3a39a012, 33 },
    { 0x25f9a013, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd89a016, 77 },
    { 0x7a9a016, 23 },
    { 0xda69a018, 61 },
    { 0x5609a016, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d69a024, 23 },
    { 0xc0a9a025, 33 },
    { 0x3bb9a025, 99 },
    { 0, 0 },
    { 0xfbd9a028, 46 },
    { 0x7389a029, 61 },
    { 0x3469a02a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c09a031, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5609a034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d89a039, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd129a03c, 100 },
    { 0xb89a03d, 90 },
    { 0, 0 },
    { 0x5689a03f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e9a044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b9a050, 90 },
    { 0xdcc9a051, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe499a054, 34 },
    { 0, 0 },
    { 0x3339a056, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1629a059, 87 },
    { 0xe8e9a05a, 88 },
    { 0x1bc9a05b, 93 },
    { 0x7f09a05b, 36 },
    { 0, 0 },
    { 0x2039a05e, 81 },
    { 0x9669a05e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b9a064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce9a069, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1589a072, 76 },
    { 0xf689a073, 66 },
    { 0x7299a073, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a9a078, 94 },
    { 0, 0 },
    { 0xf7e9a07a, 55 },
    { 0, 0 },
    { 0xe469a07c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19a07f, 23 },
    { 0, 0 },
    { 0xe0f9a081, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf39a089, 88 },
    { 0, 0 },
    { 0x8fa9a08b, 61 },
    { 0, 0 },
    { 0xdd09a08d, 97 },
    { 0xf8c9a08d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f39a092, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc779a095, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7899a09b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d9a0a1, 98 },
    { 0x16b9a0a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4319a0a5, 55 },
    { 0, 0 },
    { 0x1449a0a7, 53 },
    { 0, 0 },
    { 0x8cc9a0a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc899a0ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4829a0b0, 87 },
    { 0xcae9a0b1, 23 },
    { 0xf9a9a0b2, 76 },
    { 0, 0 },
    { 0xc629a0b4, 98 },
    { 0x4fa9a0b4, 91 },
    { 0x7e49a0b4, 90 },
    { 0x50f9a0b7, 33 },
    { 0xd5b9a0b5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x85c9a0bb, 61 },
    { 0xcfa9a0bb, 98 },
    { 0x8a99a0bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa489a0c5, 85 },
    { 0x8fb9a0c6, 90 },
    { 0x6309a0c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df9a0ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e9a0d1, 61 },
    { 0, 0 },
    { 0xbe9a0d3, 98 },
    { 0, 0 },
    { 0xf0e9a0d5, 90 },
    { 0, 0 },
    { 0x7c29a0d7, 81 },
    { 0, 0 },
    { 0x7629a0d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce19a0de, 9 },
    { 0, 0 },
    { 0xcfe9a0e0, 83 },
    { 0x1a69a0e1, 95 },
    { 0x67f9a0e1, 33 },
    { 0x3f89a0e3, 68 },
    { 0xea19a0e0, 55 },
    { 0x4e69a0e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d9a0e9, 36 },
    { 0xd39a0e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc699a0ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4e9a0f2, 98 },
    { 0, 0 },
    { 0x5499a0f4, 95 },
    { 0x26a9a0f4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb849a0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b9a101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef99a106, 43 },
    { 0, 0 },
    { 0x3399a108, 66 },
    { 0x9219a109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a9a110, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3649a113, 98 },
    { 0x8099a114, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19a11a, 77 },
    { 0x9629a11a, 61 },
    { 0x9359a11b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49a120, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0x7439a123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee9a128, 90 },
    { 0xd829a129, 88 },
    { 0x14b9a129, 99 },
    { 0x7b59a12b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf9a12f, 95 },
    { 0xf2a9a130, 34 },
    { 0xb509a130, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769a134, 77 },
    { 0x4d99a135, 87 },
    { 0x259a135, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa829a139, 99 },
    { 0, 0 },
    { 0x8bc9a13b, 99 },
    { 0xc809a13c, 90 },
    { 0x43e9a13c, 90 },
    { 0x8ad9a13c, 55 },
    { 0x42a9a13c, 99 },
    { 0, 0 },
    { 0xb049a141, 76 },
    { 0x2479a142, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7229a145, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd299a14a, 90 },
    { 0, 0 },
    { 0x8f59a14c, 76 },
    { 0x6cd9a14d, 94 },
    { 0x46d9a14c, 76 },
    { 0x9329a14f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce9a153, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9a156, 78 },
    { 0x3559a157, 87 },
    { 0xd809a158, 43 },
    { 0x9dc9a157, 61 },
    { 0x9059a158, 99 },
    { 0x8cd9a157, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa659a160, 23 },
    { 0, 0 },
    { 0xe5d9a162, 77 },
    { 0, 0 },
    { 0xb559a164, 93 },
    { 0, 0 },
    { 0x5629a166, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee69a16d, 76 },
    { 0x11f9a16e, 87 },
    { 0x2da9a16f, 90 },
    { 0, 0 },
    { 0x2c39a171, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4199a178, 91 },
    { 0x7259a179, 33 },
    { 0xe3a9a179, 66 },
    { 0x709a17b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9339a183, 61 },
    { 0x1539a184, 55 },
    { 0x6199a184, 61 },
    { 0x9e39a186, 43 },
    { 0xe249a186, 91 },
    { 0x2709a188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429a18d, 65 },
    { 0, 0 },
    { 0xece9a18f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7519a192, 76 },
    { 0, 0 },
    { 0xa299a194, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc19a197, 23 },
    { 0, 0 },
    { 0x88c9a199, 85 },
    { 0x91c9a19a, 33 },
    { 0, 0 },
    { 0x389a19c, 90 },
    { 0, 0 },
    { 0x6419a19e, 66 },
    { 0x40c9a19e, 17 },
    { 0x6509a1a0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa389a1a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429a1ad, 23 },
    { 0, 0 },
    { 0xfa49a1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb69a1b2, 90 },
    { 0, 0 },
    { 0x2a49a1b4, 33 },
    { 0xa049a1b5, 23 },
    { 0, 0 },
    { 0x9c09a1b7, 93 },
    { 0x67b9a1b7, 95 },
    { 0xf249a1b7, 76 },
    { 0xa599a1ba, 55 },
    { 0x7679a1bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d39a1be, 61 },
    { 0xfd99a1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949a1c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe119a1c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7069a1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769a1d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a1d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc279a1dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da9a1e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1789a1e4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1069a1ec, 25 },
    { 0, 0 },
    { 0xc999a1ee, 90 },
    { 0x39f9a1ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb79a1f5, 33 },
    { 0, 0 },
    { 0x63d9a1f7, 66 },
    { 0xa509a1f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3769a1fd, 95 },
    { 0x10b9a1fe, 76 },
    { 0xfa69a1ff, 95 },
    { 0xa0c9a200, 52 },
    { 0, 0 },
    { 0x9e39a202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3199a205, 90 },
    { 0x9219a206, 95 },
    { 0xb8f9a207, 46 },
    { 0xbce9a208, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989a20e, 23 },
    { 0x4409a20f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5609a212, 61 },
    { 0, 0 },
    { 0x4cb9a214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9219a217, 43 },
    { 0, 0 },
    { 0x5679a219, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9a21c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe79a223, 76 },
    { 0xa609a224, 43 },
    { 0, 0 },
    { 0xbc9a226, 84 },
    { 0x2dd9a226, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc59a22f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde99a232, 61 },
    { 0x7879a232, 23 },
    { 0x9919a234, 49 },
    { 0x8479a233, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819a23b, 66 },
    { 0x6729a23c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa879a244, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d9a247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9a24a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44d9a24d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d9a250, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e9a253, 90 },
    { 0, 0 },
    { 0xc039a255, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b19a25a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4509a25e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9a266, 90 },
    { 0x62e9a267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2069a26c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb089a26f, 95 },
    { 0x71e9a270, 55 },
    { 0xfbb9a271, 95 },
    { 0, 0 },
    { 0xecb9a273, 23 },
    { 0x53d9a273, 98 },
    { 0xaca9a274, 100 },
    { 0x6f09a276, 100 },
    { 0x7449a275, 34 },
    { 0, 0 },
    { 0x9919a279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d9a27c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f9a27f, 98 },
    { 0, 0 },
    { 0xd019a281, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d9a286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd9a289, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d9a28c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a290, 33 },
    { 0xd639a290, 33 },
    { 0, 0 },
    { 0xba69a293, 34 },
    { 0x7a69a293, 55 },
    { 0x28e9a293, 84 },
    { 0xacf9a296, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x75e9a299, 91 },
    { 0, 0 },
    { 0x2159a29b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e9a29f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e9a2a6, 98 },
    { 0, 0 },
    { 0x38a9a2a8, 75 },
    { 0, 0 },
    { 0x27f9a2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2539a2b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9a2b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x179a2b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d9a2c2, 43 },
    { 0, 0 },
    { 0x5a69a2c4, 24 },
    { 0xf189a2c5, 23 },
    { 0x30b9a2c6, 99 },
    { 0xadc9a2c7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3469a2cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f09a2d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d9a2d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b9a2d7, 95 },
    { 0, 0 },
    { 0x8549a2d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f89a2dd, 84 },
    { 0x5409a2de, 34 },
    { 0, 0 },
    { 0x4529a2e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9389a2ed, 23 },
    { 0xbd99a2ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f9a2f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc879a2f6, 23 },
    { 0, 0 },
    { 0x1aa9a2f8, 33 },
    { 0x859a2f8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4959a301, 38 },
    { 0, 0 },
    { 0x8a59a303, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c9a309, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9a30e, 43 },
    { 0xc079a30f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd79a314, 36 },
    { 0xbbe9a315, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a319, 85 },
    { 0, 0 },
    { 0x6539a31b, 23 },
    { 0x4459a31c, 43 },
    { 0, 0 },
    { 0x39e9a31e, 33 },
    { 0x9479a31e, 55 },
    { 0xd059a320, 100 },
    { 0xb829a320, 43 },
    { 0, 0 },
    { 0x2c79a323, 33 },
    { 0x5fb9a324, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e79a329, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1979a32d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5979a330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979a334, 81 },
    { 0x7d09a334, 17 },
    { 0x12d9a336, 46 },
    { 0xf6f9a337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca9a33a, 23 },
    { 0xae29a33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d9a33e, 23 },
    { 0xb1d9a33f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f69a342, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa89a348, 91 },
    { 0x8339a349, 90 },
    { 0x60f9a34a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc09a350, 98 },
    { 0, 0 },
    { 0x4079a352, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f69a35b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc199a35e, 33 },
    { 0xbc39a35e, 34 },
    { 0x4a29a360, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e99a363, 93 },
    { 0xa899a364, 100 },
    { 0x65d9a365, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d9a368, 95 },
    { 0xab99a368, 95 },
    { 0xe89a368, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc09a36f, 66 },
    { 0, 0 },
    { 0xf299a371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b9a377, 95 },
    { 0x7cf9a378, 87 },
    { 0xa969a379, 90 },
    { 0xea99a37a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e9a37e, 33 },
    { 0xd159a37f, 61 },
    { 0, 0 },
    { 0xffa9a381, 33 },
    { 0xc0e9a382, 61 },
    { 0, 0 },
    { 0x3ee9a384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf639a389, 61 },
    { 0, 0 },
    { 0xc279a38b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b9a38e, 98 },
    { 0x6aa9a38f, 95 },
    { 0x5e09a38e, 90 },
    { 0x5b89a391, 69 },
    { 0x2109a392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f99a395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec9a39a, 46 },
    { 0x2d99a39a, 61 },
    { 0x6d79a39c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa9a39f, 61 },
    { 0x5b59a3a0, 33 },
    { 0x42b9a3a1, 65 },
    { 0x96d9a3a1, 98 },
    { 0x8e19a3a2, 55 },
    { 0, 0 },
    { 0x5969a3a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9a3ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9439a3af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a49a3b4, 61 },
    { 0, 0 },
    { 0xce79a3b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e9a3b9, 73 },
    { 0, 0 },
    { 0xea39a3bb, 66 },
    { 0x6d19a3bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b9a3c0, 33 },
    { 0x5e39a3c1, 95 },
    { 0x26f9a3c1, 55 },
    { 0x4f59a3c2, 61 },
    { 0x2419a3c2, 90 },
    { 0x4479a3c3, 33 },
    { 0x5829a3c1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa419a3ce, 98 },
    { 0xf409a3cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e9a3d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839a3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eb9a3dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a09a3e1, 55 },
    { 0, 0 },
    { 0xa2f9a3e3, 55 },
    { 0xd009a3e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8969a3e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f79a3eb, 23 },
    { 0xf249a3ec, 46 },
    { 0, 0 },
    { 0xe269a3ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb69a3f1, 23 },
    { 0xf299a3f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5079a3f5, 95 },
    { 0, 0 },
    { 0x20d9a3f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2849a405, 53 },
    { 0, 0 },
    { 0xb189a407, 55 },
    { 0x48e9a408, 95 },
    { 0x1189a409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c29a417, 76 },
    { 0, 0 },
    { 0x58d9a419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a39a41c, 90 },
    { 0, 0 },
    { 0x4a09a41e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa79a425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9a428, 33 },
    { 0, 0 },
    { 0x1439a42a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e9a42e, 55 },
    { 0, 0 },
    { 0x21f9a430, 90 },
    { 0, 0 },
    { 0x189a432, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef9a43a, 33 },
    { 0xfc29a43a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3a9a43f, 95 },
    { 0x3349a43f, 69 },
    { 0x8d19a441, 85 },
    { 0xc149a442, 98 },
    { 0x6e39a442, 36 },
    { 0x5859a444, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab9a44a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d9a44d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa309a450, 90 },
    { 0xead9a451, 52 },
    { 0xa229a452, 90 },
    { 0x1199a453, 99 },
    { 0x25d9a454, 33 },
    { 0, 0 },
    { 0x5a19a456, 19 },
    { 0x34e9a457, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb319a45b, 55 },
    { 0x97f9a45c, 33 },
    { 0xc389a45d, 78 },
    { 0xeee9a45c, 34 },
    { 0x5e09a45f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539a466, 90 },
    { 0x2249a466, 90 },
    { 0x1299a468, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea9a46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b89a472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9a478, 23 },
    { 0, 0 },
    { 0xdcc9a47a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d9a485, 61 },
    { 0x4ca9a486, 33 },
    { 0, 0 },
    { 0xf9e9a488, 42 },
    { 0xeb79a489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789a48c, 93 },
    { 0x4a69a48d, 67 },
    { 0xb8b9a48e, 90 },
    { 0x88f9a48f, 61 },
    { 0xd409a48e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c89a494, 33 },
    { 0, 0 },
    { 0x9ce9a496, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa979a49d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac9a4a0, 23 },
    { 0x129a4a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d9a4a5, 23 },
    { 0, 0 },
    { 0x8a49a4a7, 87 },
    { 0xb629a4a7, 90 },
    { 0, 0 },
    { 0xba19a4aa, 33 },
    { 0, 0 },
    { 0x8fa9a4ac, 33 },
    { 0x15e9a4ad, 84 },
    { 0x9cd9a4ae, 100 },
    { 0x5e09a4ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9a4ba, 89 },
    { 0xce99a4ba, 76 },
    { 0x3ed9a4ba, 90 },
    { 0x49a4ba, 52 },
    { 0x7289a4be, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c9a4c1, 68 },
    { 0x5589a4c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc59a4c5, 33 },
    { 0x35f9a4c6, 67 },
    { 0, 0 },
    { 0x7ba9a4c8, 43 },
    { 0x6679a4c8, 67 },
    { 0, 0 },
    { 0xb419a4cb, 76 },
    { 0x7f79a4cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa419a4d3, 61 },
    { 0xab59a4d3, 74 },
    { 0xa3c9a4d5, 88 },
    { 0xcce9a4d3, 91 },
    { 0, 0 },
    { 0x3599a4d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f9a4de, 76 },
    { 0xd409a4df, 61 },
    { 0xf49a4df, 95 },
    { 0x45e9a4e1, 33 },
    { 0xdbc9a4de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9789a4eb, 99 },
    { 0xc779a4ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x1409a4ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e99a4f2, 33 },
    { 0x5e99a4f3, 87 },
    { 0xf8f9a4f3, 66 },
    { 0xf439a4f3, 69 },
    { 0x2a99a4f6, 90 },
    { 0, 0 },
    { 0xcb99a4f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b9a4fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c9a500, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e59a505, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3669a50b, 33 },
    { 0, 0 },
    { 0x92c9a50d, 17 },
    { 0x63a9a50d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f49a511, 84 },
    { 0, 0 },
    { 0x7d19a513, 98 },
    { 0x3f89a514, 92 },
    { 0x6089a515, 90 },
    { 0, 0 },
    { 0xf899a517, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd9a51d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd9a521, 23 },
    { 0xb2d9a522, 33 },
    { 0, 0 },
    { 0x1a99a524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd9a527, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e9a52a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa9a52d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f29a532, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb349a535, 23 },
    { 0x4979a535, 90 },
    { 0x56c9a537, 23 },
    { 0x4b99a536, 24 },
    { 0xd1a9a539, 64 },
    { 0x9769a536, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef9a53e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c39a541, 90 },
    { 0xa799a542, 61 },
    { 0x67b9a543, 88 },
    { 0x12b9a544, 95 },
    { 0x2009a542, 90 },
    { 0x98d9a546, 34 },
    { 0x39c9a543, 33 },
    { 0x5879a548, 95 },
    { 0x9929a549, 97 },
    { 0x3129a548, 65 },
    { 0x8f69a54b, 90 },
    { 0x8879a54c, 61 },
    { 0, 0 },
    { 0xf799a54e, 85 },
    { 0xce29a54f, 80 },
    { 0, 0 },
    { 0xcfb9a551, 83 },
    { 0, 0 },
    { 0xa909a553, 85 },
    { 0x1399a553, 95 },
    { 0, 0 },
    { 0xa959a556, 33 },
    { 0xcd69a556, 33 },
    { 0, 0 },
    { 0x1d29a559, 23 },
    { 0xc3a9a55a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6429a55e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa049a561, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb489a564, 96 },
    { 0xd7b9a564, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa9a56b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe9a56e, 85 },
    { 0, 0 },
    { 0xae39a570, 33 },
    { 0x4499a571, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c9a574, 61 },
    { 0, 0 },
    { 0xeea9a576, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9a57a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b9a582, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe849a585, 98 },
    { 0x2759a586, 33 },
    { 0xc5b9a586, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebc9a58a, 90 },
    { 0xaca9a58b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xba89a58e, 52 },
    { 0, 0 },
    { 0x4cf9a590, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f99a593, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b9a597, 91 },
    { 0x7259a598, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc789a59c, 61 },
    { 0xa479a59c, 52 },
    { 0x7ae9a59c, 46 },
    { 0x9be9a59c, 23 },
    { 0xfb19a5a0, 43 },
    { 0, 0 },
    { 0x3e09a5a2, 66 },
    { 0xe219a5a3, 90 },
    { 0x4779a5a2, 33 },
    { 0, 0 },
    { 0xeef9a5a6, 44 },
    { 0xd429a5a7, 81 },
    { 0, 0 },
    { 0x5eb9a5a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899a5ad, 90 },
    { 0x7099a5ae, 33 },
    { 0x9029a5af, 61 },
    { 0x1979a5af, 43 },
    { 0, 0 },
    { 0x9a99a5b2, 95 },
    { 0xf959a5b2, 23 },
    { 0x6839a5b4, 95 },
    { 0xfe79a5b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4329a5b9, 90 },
    { 0xbd89a5b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3129a5c2, 61 },
    { 0, 0 },
    { 0x6dd9a5c4, 98 },
    { 0x7c89a5c5, 77 },
    { 0x3409a5c5, 95 },
    { 0, 0 },
    { 0xca19a5c8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe589a5cb, 61 },
    { 0x5af9a5cc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e39a5d0, 76 },
    { 0, 0 },
    { 0xf6d9a5d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe879a5d9, 90 },
    { 0xbde9a5da, 90 },
    { 0x979a5d9, 55 },
    { 0, 0 },
    { 0xea29a5dd, 63 },
    { 0xa69a5dd, 90 },
    { 0xf2a9a5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a29a5e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b09a5e5, 43 },
    { 0x87c9a5e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19a5ea, 90 },
    { 0x99d9a5eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb589a5ef, 68 },
    { 0x4fc9a5f0, 33 },
    { 0xe149a5f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf29a5f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6959a5f9, 61 },
    { 0, 0 },
    { 0x8929a5fb, 95 },
    { 0xfb89a5fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a9a604, 61 },
    { 0, 0 },
    { 0x9a59a606, 33 },
    { 0x7299a607, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1379a60a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b9a60d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4219a610, 84 },
    { 0x2f19a611, 98 },
    { 0xdea9a611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3859a616, 33 },
    { 0, 0 },
    { 0xcd59a618, 52 },
    { 0xe7c9a619, 66 },
    { 0x6a89a61a, 98 },
    { 0xc4c9a61b, 23 },
    { 0xf089a619, 90 },
    { 0x7d9a61d, 33 },
    { 0xaed9a61c, 46 },
    { 0x80e9a61a, 98 },
    { 0, 0 },
    { 0x8eb9a621, 43 },
    { 0x9699a621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5619a625, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f9a62a, 55 },
    { 0xd29a62b, 33 },
    { 0xef29a62c, 33 },
    { 0, 0 },
    { 0xb7a9a62e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a09a631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a637, 23 },
    { 0xf509a638, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1529a63d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb9a640, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a69a643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99a647, 90 },
    { 0x2509a648, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b89a650, 61 },
    { 0xafd9a650, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819a658, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9569a664, 90 },
    { 0, 0 },
    { 0xc5e9a666, 98 },
    { 0xeaf9a667, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad9a66f, 74 },
    { 0, 0 },
    { 0x1969a671, 99 },
    { 0xc559a672, 90 },
    { 0x26a9a671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e69a677, 52 },
    { 0x1a09a677, 33 },
    { 0xb169a679, 77 },
    { 0xcb09a677, 23 },
    { 0x4d59a67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a69a67f, 55 },
    { 0x80f9a680, 90 },
    { 0, 0 },
    { 0xcfd9a682, 99 },
    { 0x7569a683, 53 },
    { 0, 0 },
    { 0x6979a685, 91 },
    { 0x9799a685, 99 },
    { 0xb229a687, 90 },
    { 0xc8b9a688, 99 },
    { 0xc939a689, 84 },
    { 0x51e9a68a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a9a68d, 61 },
    { 0, 0 },
    { 0x6319a68f, 98 },
    { 0x1db9a68f, 55 },
    { 0x6ae9a691, 33 },
    { 0, 0 },
    { 0xbbd9a693, 61 },
    { 0x3269a694, 94 },
    { 0xc359a694, 61 },
    { 0x5fd9a694, 100 },
    { 0, 0 },
    { 0x8139a698, 76 },
    { 0xf6c9a699, 46 },
    { 0xab19a69a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98b9a6a5, 100 },
    { 0x8d79a6a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29a6ab, 61 },
    { 0x6fe9a6ab, 77 },
    { 0, 0 },
    { 0xbcf9a6ae, 98 },
    { 0xb939a6af, 52 },
    { 0, 0 },
    { 0xcc9a6b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9a6b4, 33 },
    { 0, 0 },
    { 0xeeb9a6b6, 33 },
    { 0xa979a6b6, 33 },
    { 0, 0 },
    { 0xfbb9a6b9, 33 },
    { 0, 0 },
    { 0x9399a6bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629a6be, 33 },
    { 0x1259a6be, 66 },
    { 0, 0 },
    { 0x1589a6c1, 98 },
    { 0x3739a6c1, 98 },
    { 0, 0 },
    { 0xcd49a6c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf99a6c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e9a6d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329a6d5, 90 },
    { 0, 0 },
    { 0x27d9a6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9a6da, 52 },
    { 0, 0 },
    { 0xd1b9a6dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a9a6e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a9a6e6, 95 },
    { 0xbdd9a6e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999a6eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf249a6f0, 80 },
    { 0, 0 },
    { 0x3389a6f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd109a6f7, 34 },
    { 0xc249a6f7, 95 },
    { 0x9f19a6f7, 33 },
    { 0, 0 },
    { 0xb989a6fb, 76 },
    { 0, 0 },
    { 0xa3d9a6fd, 61 },
    { 0, 0 },
    { 0x16c9a6ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae9a702, 68 },
    { 0, 0 },
    { 0x2009a704, 34 },
    { 0xb989a705, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe059a70a, 55 },
    { 0, 0 },
    { 0xa5e9a70c, 33 },
    { 0, 0 },
    { 0xf309a70e, 34 },
    { 0xa6f9a70f, 85 },
    { 0, 0 },
    { 0x1b99a711, 90 },
    { 0, 0 },
    { 0x6819a713, 55 },
    { 0xcae9a714, 67 },
    { 0x2bd9a715, 95 },
    { 0x55b9a716, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1859a71a, 34 },
    { 0x3cb9a71b, 52 },
    { 0xf579a71b, 90 },
    { 0x8259a71d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf889a721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab9a726, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba89a729, 98 },
    { 0, 0 },
    { 0x9709a72b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd9a730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799a735, 67 },
    { 0xdbc9a735, 33 },
    { 0, 0 },
    { 0xf739a738, 23 },
    { 0x1489a738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809a73c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8249a74a, 95 },
    { 0, 0 },
    { 0xab39a74c, 92 },
    { 0x40c9a74c, 23 },
    { 0x8169a74e, 76 },
    { 0xd509a74e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8e9a753, 34 },
    { 0x67e9a753, 80 },
    { 0x1629a754, 93 },
    { 0x5139a755, 23 },
    { 0xfc9a755, 56 },
    { 0xb439a756, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c9a75c, 23 },
    { 0x7749a75c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9339a760, 61 },
    { 0x3ce9a760, 36 },
    { 0x5339a761, 33 },
    { 0, 0 },
    { 0xda49a764, 55 },
    { 0x8f39a765, 90 },
    { 0x839a764, 23 },
    { 0, 0 },
    { 0x7769a768, 61 },
    { 0xb659a769, 61 },
    { 0x1629a769, 98 },
    { 0x28e9a769, 17 },
    { 0x4669a76c, 61 },
    { 0, 0 },
    { 0x3749a76e, 98 },
    { 0x3d9a76f, 84 },
    { 0x27a9a770, 75 },
    { 0, 0 },
    { 0x3fb9a772, 85 },
    { 0x5509a773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a9a776, 90 },
    { 0x6e59a777, 69 },
    { 0, 0 },
    { 0x4869a779, 95 },
    { 0xa699a77a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc129a77e, 95 },
    { 0x68f9a77f, 100 },
    { 0xb199a780, 36 },
    { 0xe249a780, 33 },
    { 0xdb99a782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e69a785, 61 },
    { 0xd0c9a785, 90 },
    { 0, 0 },
    { 0x9da9a788, 52 },
    { 0, 0 },
    { 0x8e9a78a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c39a78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf9a791, 78 },
    { 0, 0 },
    { 0xdc49a793, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e49a797, 34 },
    { 0x6ca9a798, 98 },
    { 0, 0 },
    { 0x3779a79a, 99 },
    { 0xe369a79b, 90 },
    { 0, 0 },
    { 0x78c9a79d, 90 },
    { 0, 0 },
    { 0x8c79a79f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c9a7a7, 76 },
    { 0, 0 },
    { 0xa039a7a9, 91 },
    { 0x7dc9a7aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb9a7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729a7b1, 87 },
    { 0x6c39a7b2, 75 },
    { 0xa789a7b2, 55 },
    { 0xac09a7b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd09a7b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50f9a7bb, 90 },
    { 0x3789a7bc, 61 },
    { 0xf219a7bc, 81 },
    { 0xfb19a7bd, 84 },
    { 0xef19a7bc, 90 },
    { 0x75e9a7c0, 95 },
    { 0xa4a9a7c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9629a7c4, 33 },
    { 0x6d9a7c4, 67 },
    { 0x2f9a7c6, 33 },
    { 0xee59a7c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf109a7ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b89a7d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89a7d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a9a7dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6869a7e6, 95 },
    { 0, 0 },
    { 0x75d9a7e8, 98 },
    { 0x36b9a7e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe139a7f2, 33 },
    { 0x709a7f2, 68 },
    { 0x5aa9a7f4, 34 },
    { 0x8219a7f5, 76 },
    { 0, 0 },
    { 0xcaf9a7f7, 61 },
    { 0xf439a7f8, 23 },
    { 0x5379a7f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee9a7fe, 34 },
    { 0, 0 },
    { 0xde09a800, 76 },
    { 0xfad9a801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbe9a804, 98 },
    { 0x6119a805, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd9a808, 91 },
    { 0, 0 },
    { 0xde59a80a, 67 },
    { 0x4539a80b, 33 },
    { 0x1709a80b, 34 },
    { 0xfb49a80c, 98 },
    { 0x4209a80d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa9a814, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819a819, 100 },
    { 0, 0 },
    { 0x3ef9a81b, 43 },
    { 0xd0d9a81c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb59a823, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9a826, 90 },
    { 0x1259a826, 61 },
    { 0, 0 },
    { 0xf099a829, 90 },
    { 0x4b59a82a, 61 },
    { 0x3df9a82a, 77 },
    { 0, 0 },
    { 0x4b69a82d, 97 },
    { 0, 0 },
    { 0xf389a82f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9a835, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9a838, 55 },
    { 0x5189a839, 75 },
    { 0x4d49a83a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1629a83d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa259a846, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e89a84e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9079a853, 80 },
    { 0, 0 },
    { 0xfdf9a855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e39a85b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5299a861, 43 },
    { 0x9219a862, 46 },
    { 0x8949a861, 33 },
    { 0x4639a864, 67 },
    { 0xbd09a862, 95 },
    { 0, 0 },
    { 0xea69a867, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8709a86b, 92 },
    { 0x6dd9a86c, 85 },
    { 0xc9c9a86d, 93 },
    { 0xb079a86e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa529a871, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f69a874, 23 },
    { 0, 0 },
    { 0x1079a876, 33 },
    { 0, 0 },
    { 0x8509a878, 93 },
    { 0, 0 },
    { 0xde19a87a, 33 },
    { 0xf579a87b, 15 },
    { 0xc639a87a, 95 },
    { 0x7699a87d, 33 },
    { 0xffd9a87c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef29a881, 52 },
    { 0x3379a882, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9a885, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb139a889, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f9a88e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad39a894, 43 },
    { 0x6ce9a894, 61 },
    { 0xd9a9a894, 93 },
    { 0x5639a897, 23 },
    { 0xd7e9a897, 76 },
    { 0x8649a898, 56 },
    { 0xdb19a89a, 61 },
    { 0, 0 },
    { 0xae49a89c, 99 },
    { 0, 0 },
    { 0x8319a89e, 100 },
    { 0, 0 },
    { 0x8929a8a0, 46 },
    { 0, 0 },
    { 0x46f9a8a2, 34 },
    { 0xad89a8a2, 38 },
    { 0x7749a8a4, 61 },
    { 0xa399a8a5, 61 },
    { 0x52a9a8a6, 55 },
    { 0xe049a8a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa319a8b1, 90 },
    { 0x1129a8b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8639a8b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2289a8b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb9a8bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf39a8c0, 90 },
    { 0x929a8c1, 67 },
    { 0xa799a8c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8659a8c5, 78 },
    { 0xa7e9a8c6, 55 },
    { 0, 0 },
    { 0x74e9a8c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9a8cb, 90 },
    { 0x5679a8cc, 61 },
    { 0, 0 },
    { 0x50e9a8ce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf9a8d7, 98 },
    { 0x7069a8d8, 61 },
    { 0xbbd9a8d7, 33 },
    { 0x439a8da, 95 },
    { 0x69b9a8da, 33 },
    { 0x6129a8dc, 33 },
    { 0xfed9a8d9, 52 },
    { 0x92b9a8dd, 33 },
    { 0, 0 },
    { 0x8459a8e0, 61 },
    { 0xa739a8e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8489a8e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a9a8e7, 34 },
    { 0x5de9a8e8, 88 },
    { 0x11f9a8e7, 76 },
    { 0x1959a8ea, 76 },
    { 0x2f49a8eb, 55 },
    { 0x9d69a8eb, 66 },
    { 0xb89a8ed, 90 },
    { 0x5889a8ee, 95 },
    { 0x139a8ef, 56 },
    { 0x3679a8ef, 88 },
    { 0xcf89a8f1, 79 },
    { 0xcb49a8ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f79a8fc, 33 },
    { 0xb3e9a8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf979a900, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc449a903, 99 },
    { 0, 0 },
    { 0x7e99a905, 95 },
    { 0, 0 },
    { 0xa9f9a907, 23 },
    { 0xd879a908, 95 },
    { 0x7629a908, 23 },
    { 0x2699a908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c29a911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d99a91f, 43 },
    { 0xd129a920, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba59a924, 90 },
    { 0, 0 },
    { 0x5079a926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5909a92c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69a933, 67 },
    { 0x4159a934, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c9a93a, 33 },
    { 0xdcf9a93b, 43 },
    { 0, 0 },
    { 0xf1b9a93d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c9a941, 61 },
    { 0x5009a941, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1699a948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb9a94b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91d9a950, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a9a954, 91 },
    { 0xcda9a955, 77 },
    { 0x5d89a956, 59 },
    { 0xadb9a957, 46 },
    { 0x6c69a956, 33 },
    { 0xf419a959, 65 },
    { 0, 0 },
    { 0x2f09a95b, 76 },
    { 0xb1b9a95b, 55 },
    { 0x37b9a95d, 90 },
    { 0, 0 },
    { 0xf9c9a95f, 93 },
    { 0x8599a960, 46 },
    { 0xda9a95f, 98 },
    { 0xaf09a962, 61 },
    { 0, 0 },
    { 0x69a9a964, 95 },
    { 0x8929a965, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c19a968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6569a96b, 76 },
    { 0, 0 },
    { 0xc229a96d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a89a971, 90 },
    { 0xd749a972, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d99a976, 90 },
    { 0x1d99a976, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd9a97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a9a97e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c9a983, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd469a988, 43 },
    { 0x8c09a989, 90 },
    { 0x3589a98a, 23 },
    { 0xd379a98b, 55 },
    { 0xc4d9a98c, 43 },
    { 0x15d9a98b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa189a991, 98 },
    { 0, 0 },
    { 0x8a19a993, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c9a998, 78 },
    { 0x7db9a999, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb9a99d, 88 },
    { 0, 0 },
    { 0x7de9a99f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1909a9a3, 85 },
    { 0x3b29a9a3, 93 },
    { 0x5ed9a9a5, 34 },
    { 0x4109a9a6, 33 },
    { 0x7ab9a9a7, 95 },
    { 0x2b19a9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf409a9ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe679a9af, 90 },
    { 0, 0 },
    { 0xd929a9b1, 23 },
    { 0, 0 },
    { 0x8719a9b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1389a9b6, 61 },
    { 0, 0 },
    { 0xd779a9b8, 33 },
    { 0x7fa9a9b8, 84 },
    { 0xb899a9ba, 61 },
    { 0xe7f9a9bb, 90 },
    { 0xf6d9a9bc, 33 },
    { 0x3829a9bc, 33 },
    { 0x10c9a9bd, 78 },
    { 0x8fb9a9bd, 61 },
    { 0x4829a9bd, 66 },
    { 0xb569a9bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x379a9c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa719a9cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb499a9d4, 95 },
    { 0xcb29a9d5, 95 },
    { 0x9a89a9d6, 66 },
    { 0xef99a9d6, 87 },
    { 0xe109a9d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf559a9de, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8209a9e4, 33 },
    { 0x9d59a9e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b9a9f3, 81 },
    { 0, 0 },
    { 0x7d09a9f5, 61 },
    { 0x3099a9f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc249a9fc, 75 },
    { 0xe649a9fd, 76 },
    { 0xad19a9fe, 83 },
    { 0x5269a9ff, 76 },
    { 0x11a9a9ff, 33 },
    { 0x5ce9aa01, 91 },
    { 0x88f9a9ff, 90 },
    { 0, 0 },
    { 0xc949aa04, 55 },
    { 0xf4a9aa05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9aa08, 61 },
    { 0, 0 },
    { 0xa539aa0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9aa0e, 43 },
    { 0, 0 },
    { 0xfc99aa10, 46 },
    { 0, 0 },
    { 0xebb9aa12, 90 },
    { 0x2bb9aa13, 84 },
    { 0x7f59aa13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569aa19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c9aa1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a9aa20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1239aa23, 90 },
    { 0x7429aa23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda59aa27, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa69aa2b, 33 },
    { 0x5db9aa2b, 60 },
    { 0, 0 },
    { 0x9c89aa2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e49aa32, 90 },
    { 0xde39aa33, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359aa36, 61 },
    { 0x6c09aa37, 55 },
    { 0x8799aa36, 33 },
    { 0x8b19aa39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c29aa42, 43 },
    { 0, 0 },
    { 0x3e69aa44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd69aa47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c39aa4e, 95 },
    { 0, 0 },
    { 0xf109aa50, 53 },
    { 0x2539aa51, 81 },
    { 0xd589aa51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3919aa56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5409aa5c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69aa65, 93 },
    { 0x3069aa66, 95 },
    { 0x3f79aa66, 98 },
    { 0, 0 },
    { 0x7c9aa69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109aa6e, 55 },
    { 0x8789aa6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae9aa78, 61 },
    { 0, 0 },
    { 0xbe59aa7a, 100 },
    { 0xa49aa7b, 61 },
    { 0xd9a9aa7b, 90 },
    { 0x199aa7b, 34 },
    { 0xdac9aa7d, 98 },
    { 0xa0a9aa7a, 90 },
    { 0, 0 },
    { 0xb789aa81, 33 },
    { 0xc529aa82, 23 },
    { 0x71e9aa83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3429aa87, 66 },
    { 0, 0 },
    { 0x3679aa89, 88 },
    { 0, 0 },
    { 0x9789aa8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x899aa8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9aa91, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c9aa95, 55 },
    { 0, 0 },
    { 0x9389aa97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb49aa9b, 33 },
    { 0, 0 },
    { 0x1099aa9d, 90 },
    { 0x4bf9aa9e, 61 },
    { 0x2d9aa9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf039aaa3, 98 },
    { 0xbf49aaa4, 46 },
    { 0xa469aaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb79aaa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c9aaae, 98 },
    { 0x87f9aaae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399aab6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b9aaba, 66 },
    { 0, 0 },
    { 0xdd59aabc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d79aac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe119aac5, 33 },
    { 0xe989aac5, 61 },
    { 0xf7c9aac7, 52 },
    { 0, 0 },
    { 0x1029aac9, 90 },
    { 0xe729aaca, 95 },
    { 0x55a9aacb, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb49aacf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379aada, 23 },
    { 0x4d19aadb, 61 },
    { 0x8279aadc, 90 },
    { 0, 0 },
    { 0x4c79aade, 66 },
    { 0, 0 },
    { 0xa2d9aae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb09aae3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x35f9aae6, 23 },
    { 0, 0 },
    { 0xc559aae8, 33 },
    { 0xa3b9aae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2109aaf2, 75 },
    { 0, 0 },
    { 0x8dc9aaf4, 33 },
    { 0, 0 },
    { 0x7599aaf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3779aaf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec59aafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa39ab07, 90 },
    { 0x94f9ab08, 100 },
    { 0, 0 },
    { 0x469ab0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf539ab0e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f89ab14, 52 },
    { 0x4049ab15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d19ab19, 33 },
    { 0, 0 },
    { 0x7779ab1b, 90 },
    { 0, 0 },
    { 0x7eb9ab1d, 43 },
    { 0x79e9ab1e, 34 },
    { 0x8769ab1f, 33 },
    { 0xf219ab20, 22 },
    { 0, 0 },
    { 0x1ba9ab22, 78 },
    { 0, 0 },
    { 0xec49ab24, 61 },
    { 0, 0 },
    { 0x2fd9ab26, 34 },
    { 0xe399ab26, 98 },
    { 0, 0 },
    { 0x3d49ab29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b9ab2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3409ab32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9ab39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1119ab43, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a79ab46, 61 },
    { 0xf469ab47, 78 },
    { 0, 0 },
    { 0xbac9ab49, 61 },
    { 0x66b9ab4a, 61 },
    { 0x9d49ab4a, 55 },
    { 0x7569ab4c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd179ab55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c9ab5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca29ab66, 69 },
    { 0x6b89ab67, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ab6a, 43 },
    { 0, 0 },
    { 0xac49ab6c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa49ab6f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a9ab7b, 94 },
    { 0, 0 },
    { 0x8149ab7d, 90 },
    { 0x9349ab7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c9ab83, 91 },
    { 0, 0 },
    { 0xcda9ab85, 17 },
    { 0x12e9ab85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5449ab8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c09ab90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a9ab96, 90 },
    { 0xee19ab97, 91 },
    { 0x1fa9ab98, 78 },
    { 0x99a9ab98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe999ab9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299aba5, 85 },
    { 0xd5a9aba5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb289aba9, 34 },
    { 0x2d59abaa, 76 },
    { 0x2509abab, 90 },
    { 0x32a9abaa, 98 },
    { 0xc509abab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe29abb2, 17 },
    { 0, 0 },
    { 0xe859abb4, 67 },
    { 0xe519abb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd9abbc, 52 },
    { 0x8fd9abbd, 98 },
    { 0xb879abbe, 85 },
    { 0x3c49abbe, 33 },
    { 0, 0 },
    { 0x20b9abc1, 46 },
    { 0xb289abc2, 98 },
    { 0x1749abc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa849abc7, 33 },
    { 0x41d9abc7, 98 },
    { 0x7e69abc9, 90 },
    { 0, 0 },
    { 0x9829abcb, 76 },
    { 0, 0 },
    { 0x3599abcd, 76 },
    { 0xf079abce, 82 },
    { 0, 0 },
    { 0x5bc9abd0, 33 },
    { 0, 0 },
    { 0x8959abd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2579abd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9abdc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a9abdf, 33 },
    { 0xb19abdf, 23 },
    { 0xcab9abdf, 90 },
    { 0x1009abe2, 95 },
    { 0x45a9abe2, 90 },
    { 0xe809abe4, 90 },
    { 0, 0 },
    { 0xf5a9abe6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1169abe9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee49abec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de9abf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa9abf3, 55 },
    { 0, 0 },
    { 0x6969abf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb369abfc, 52 },
    { 0x8859abfc, 90 },
    { 0, 0 },
    { 0x9559abff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3039ac02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11c9ac06, 61 },
    { 0, 0 },
    { 0xd6c9ac08, 55 },
    { 0, 0 },
    { 0xcc29ac0a, 61 },
    { 0x2b09ac0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf19ac0e, 46 },
    { 0x39a9ac0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d9ac16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4509ac1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a9ac20, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x5749ac23, 33 },
    { 0, 0 },
    { 0x46b9ac25, 93 },
    { 0x8c89ac26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7b9ac2d, 90 },
    { 0x2469ac2e, 84 },
    { 0, 0 },
    { 0x7359ac30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439ac34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d9ac3d, 61 },
    { 0, 0 },
    { 0xb319ac3f, 90 },
    { 0x1f69ac40, 33 },
    { 0, 0 },
    { 0xd009ac42, 78 },
    { 0x1079ac43, 90 },
    { 0, 0 },
    { 0x95e9ac45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9ac48, 90 },
    { 0xc599ac48, 95 },
    { 0xd4e9ac49, 95 },
    { 0xb0e9ac4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf229ac4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd269ac51, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4149ac5c, 98 },
    { 0x1629ac5d, 61 },
    { 0, 0 },
    { 0x62e9ac5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1299ac62, 61 },
    { 0x3079ac63, 85 },
    { 0x1c89ac62, 98 },
    { 0, 0 },
    { 0xfe49ac66, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d79ac6a, 46 },
    { 0x9a69ac6b, 52 },
    { 0x13d9ac6b, 87 },
    { 0xa329ac6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa29ac71, 33 },
    { 0x82c9ac72, 61 },
    { 0x2fb9ac72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c89ac7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69ac82, 23 },
    { 0, 0 },
    { 0x93a9ac84, 95 },
    { 0xe789ac85, 23 },
    { 0x7a99ac86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c9ac8c, 76 },
    { 0xa919ac8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7469ac91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c9ac97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99ac9a, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xef09ac9d, 87 },
    { 0x5f39ac9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f9aca2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c9aca6, 84 },
    { 0x42a9aca6, 43 },
    { 0x5c89aca8, 61 },
    { 0, 0 },
    { 0x3199acaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b9acae, 95 },
    { 0x2349acaf, 90 },
    { 0x2fb9acb0, 55 },
    { 0x9f89acb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52a9acb7, 46 },
    { 0, 0 },
    { 0xd3a9acb9, 90 },
    { 0xfd9acb9, 23 },
    { 0xf5a9acba, 98 },
    { 0x4299acbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99acbf, 90 },
    { 0x6e49acc0, 23 },
    { 0xa909acc1, 23 },
    { 0, 0 },
    { 0xc809acc3, 67 },
    { 0, 0 },
    { 0xfa09acc5, 23 },
    { 0, 0 },
    { 0xc29acc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e9accc, 33 },
    { 0x12c9accd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa959acd2, 78 },
    { 0, 0 },
    { 0xa0b9acd4, 91 },
    { 0x3eb9acd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719acd8, 29 },
    { 0x4339acd8, 77 },
    { 0, 0 },
    { 0x1709acdb, 61 },
    { 0x2b59acdc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4729ace4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8109ace7, 76 },
    { 0xd119ace8, 45 },
    { 0xa4f9ace9, 81 },
    { 0, 0 },
    { 0x7959aceb, 88 },
    { 0x4549aceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa9acf2, 61 },
    { 0, 0 },
    { 0xc759acf4, 23 },
    { 0, 0 },
    { 0x1a49acf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9689acfb, 90 },
    { 0x41e9acfb, 93 },
    { 0, 0 },
    { 0xb439acfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc129ad02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ad08, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7119ad0b, 33 },
    { 0, 0 },
    { 0x8eb9ad0d, 33 },
    { 0, 0 },
    { 0x3e89ad0f, 90 },
    { 0x4cf9ad10, 61 },
    { 0, 0 },
    { 0x1349ad12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8f9ad17, 61 },
    { 0x5fa9ad18, 90 },
    { 0, 0 },
    { 0x4969ad1a, 90 },
    { 0x7f39ad1b, 61 },
    { 0x1a39ad1c, 95 },
    { 0xca19ad1a, 58 },
    { 0x7009ad1e, 83 },
    { 0, 0 },
    { 0xaa09ad20, 43 },
    { 0xa809ad21, 90 },
    { 0xd299ad20, 98 },
    { 0xe859ad23, 34 },
    { 0, 0 },
    { 0xf089ad25, 23 },
    { 0, 0 },
    { 0xcbb9ad27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939ad2c, 86 },
    { 0x2e59ad2c, 90 },
    { 0x26a9ad2d, 61 },
    { 0, 0 },
    { 0x29d9ad30, 95 },
    { 0x3249ad30, 61 },
    { 0xcf59ad30, 23 },
    { 0xaad9ad33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9359ad36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf39ad3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e49ad3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7769ad43, 61 },
    { 0xe4c9ad44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a9ad49, 34 },
    { 0x94e9ad49, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba9ad4e, 33 },
    { 0x799ad4e, 76 },
    { 0, 0 },
    { 0x4c89ad51, 99 },
    { 0, 0 },
    { 0x4af9ad53, 90 },
    { 0x43a9ad54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2349ad59, 33 },
    { 0xa709ad5a, 90 },
    { 0x6ef9ad5b, 98 },
    { 0x4749ad5b, 33 },
    { 0x5069ad5d, 77 },
    { 0xe69ad5d, 95 },
    { 0xe859ad5c, 66 },
    { 0x8b69ad60, 23 },
    { 0x3339ad61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee39ad64, 76 },
    { 0xda69ad65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf59ad69, 46 },
    { 0x9569ad6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf9ad6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859ad73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc79ad7b, 95 },
    { 0x9b19ad7b, 90 },
    { 0, 0 },
    { 0x5c39ad7e, 95 },
    { 0x9329ad7e, 93 },
    { 0x949ad80, 76 },
    { 0x8c29ad7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42b9ad86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5619ad92, 90 },
    { 0x38a9ad93, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9089ad99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7489ad9f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9ada3, 99 },
    { 0x3239ada4, 43 },
    { 0xa369ada4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe759ada8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d9adba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb409adbd, 93 },
    { 0xc579adbe, 40 },
    { 0x7b79adbf, 90 },
    { 0x8989adbd, 98 },
    { 0x9fc9adc1, 34 },
    { 0x5b99adc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e19adc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc9adce, 95 },
    { 0xe969adcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7229add9, 22 },
    { 0x3519adda, 84 },
    { 0, 0 },
    { 0x5649addc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf199addf, 23 },
    { 0xc39ade0, 76 },
    { 0x6fd9ade0, 95 },
    { 0x7619ade2, 84 },
    { 0x90f9ade2, 90 },
    { 0, 0 },
    { 0xd099ade5, 78 },
    { 0x8e99ade5, 34 },
    { 0x7029ade7, 95 },
    { 0x3be9ade8, 98 },
    { 0x7a19ade9, 90 },
    { 0x8389ade9, 34 },
    { 0x87c9ade7, 61 },
    { 0xaae9ade6, 90 },
    { 0, 0 },
    { 0x61f9adee, 34 },
    { 0x6ca9adee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe699adf3, 90 },
    { 0xb179adf4, 95 },
    { 0x4db9adf4, 33 },
    { 0xeb89adf6, 55 },
    { 0x59adf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c69adfc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf249ae00, 76 },
    { 0, 0 },
    { 0xe7d9ae02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb789ae05, 84 },
    { 0, 0 },
    { 0xb7e9ae07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f29ae0a, 61 },
    { 0, 0 },
    { 0xaab9ae0c, 34 },
    { 0xa729ae0d, 81 },
    { 0x5539ae0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd9ae12, 98 },
    { 0x2b19ae13, 90 },
    { 0x5e29ae12, 95 },
    { 0, 0 },
    { 0xff79ae16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5729ae1e, 52 },
    { 0x1029ae1f, 95 },
    { 0xb49ae1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5459ae25, 98 },
    { 0, 0 },
    { 0x2279ae27, 33 },
    { 0, 0 },
    { 0x4ae9ae29, 95 },
    { 0, 0 },
    { 0xd369ae2b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2589ae33, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9ae36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f9ae3b, 34 },
    { 0, 0 },
    { 0xff19ae3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec9ae40, 81 },
    { 0x5f29ae41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b79ae46, 90 },
    { 0, 0 },
    { 0x3969ae48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd9ae4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979ae53, 67 },
    { 0, 0 },
    { 0x6439ae55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3989ae66, 85 },
    { 0x2ac9ae67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb9ae6d, 67 },
    { 0x6219ae6e, 76 },
    { 0xe179ae6f, 33 },
    { 0x56c9ae70, 76 },
    { 0x4cb9ae71, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b9ae75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8609ae7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf69ae83, 33 },
    { 0x49b9ae83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd9ae8d, 90 },
    { 0, 0 },
    { 0x1919ae8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a9ae92, 93 },
    { 0, 0 },
    { 0x7779ae94, 98 },
    { 0x23a9ae94, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e9ae9c, 23 },
    { 0xc819ae9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa209aea0, 90 },
    { 0x8699aea1, 90 },
    { 0x58e9aea1, 23 },
    { 0, 0 },
    { 0xab59aea4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc719aea9, 90 },
    { 0x509aeaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd49aeae, 52 },
    { 0xd2b9aeaf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9aeb2, 65 },
    { 0xf3e9aeb2, 66 },
    { 0x3059aeb4, 53 },
    { 0x7a59aeb5, 23 },
    { 0x9f29aeb6, 23 },
    { 0xfbe9aeb7, 23 },
    { 0xb769aeb7, 46 },
    { 0xd659aeb8, 55 },
    { 0x5639aeb8, 90 },
    { 0, 0 },
    { 0x2969aebc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8229aec0, 61 },
    { 0x3a59aec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b89aec6, 76 },
    { 0x9409aec7, 87 },
    { 0x7e09aec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9aed4, 98 },
    { 0, 0 },
    { 0x4cb9aed6, 31 },
    { 0, 0 },
    { 0xa449aed8, 76 },
    { 0, 0 },
    { 0xec59aeda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa49aedd, 80 },
    { 0, 0 },
    { 0xf599aedf, 90 },
    { 0xc3f9aedf, 17 },
    { 0xf3c9aee1, 52 },
    { 0, 0 },
    { 0x49d9aee3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9799aee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde69aeeb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d19aeee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7489aef2, 66 },
    { 0, 0 },
    { 0x9329aef4, 55 },
    { 0x5699aef4, 95 },
    { 0xacf9aef6, 76 },
    { 0xd6c9aef6, 90 },
    { 0x6179aef6, 23 },
    { 0xa939aef9, 46 },
    { 0, 0 },
    { 0x5a49aefb, 34 },
    { 0, 0 },
    { 0xe9d9aefd, 55 },
    { 0x1989aefe, 33 },
    { 0x5ae9aeff, 61 },
    { 0x4989af00, 23 },
    { 0x9de9af00, 46 },
    { 0xece9af01, 78 },
    { 0x8dd9af03, 64 },
    { 0x4b59af00, 95 },
    { 0x4929af05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed99af08, 87 },
    { 0xaeb9af09, 33 },
    { 0x5e79af0a, 61 },
    { 0, 0 },
    { 0x3509af0c, 98 },
    { 0xbf59af0d, 90 },
    { 0xbd39af0d, 69 },
    { 0xb949af0c, 33 },
    { 0, 0 },
    { 0xf719af11, 84 },
    { 0, 0 },
    { 0x6419af13, 34 },
    { 0, 0 },
    { 0xb139af15, 61 },
    { 0xddc9af16, 90 },
    { 0xf6e9af16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa009af1e, 34 },
    { 0xa889af1f, 33 },
    { 0x8c89af20, 78 },
    { 0xe4d9af20, 98 },
    { 0xa8c9af1f, 90 },
    { 0xe869af20, 100 },
    { 0x43a9af1f, 34 },
    { 0x82b9af25, 34 },
    { 0xac19af26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed9af2a, 40 },
    { 0, 0 },
    { 0xbc79af2c, 76 },
    { 0x2609af2d, 33 },
    { 0, 0 },
    { 0xf0b9af2f, 91 },
    { 0xb759af2f, 96 },
    { 0x14c9af31, 97 },
    { 0, 0 },
    { 0xd6d9af33, 90 },
    { 0x7a29af33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439af3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c9af3e, 78 },
    { 0xf7e9af3f, 34 },
    { 0x9ba9af3e, 66 },
    { 0xe409af41, 43 },
    { 0xb599af42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3769af49, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a49af4d, 87 },
    { 0x3989af4d, 40 },
    { 0x2649af4f, 85 },
    { 0x7bb9af50, 95 },
    { 0, 0 },
    { 0xbed9af52, 90 },
    { 0, 0 },
    { 0x4309af54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe909af5a, 77 },
    { 0x8f9af5b, 77 },
    { 0x39a9af5c, 77 },
    { 0x3679af5d, 61 },
    { 0x7da9af5e, 93 },
    { 0xded9af5e, 78 },
    { 0xbf39af60, 23 },
    { 0x4919af5d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4269af67, 33 },
    { 0xa4c9af68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b9af6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2969af6f, 90 },
    { 0, 0 },
    { 0x3409af71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e9af75, 99 },
    { 0xe8e9af76, 23 },
    { 0xb709af76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d9af7a, 90 },
    { 0, 0 },
    { 0xf639af7c, 33 },
    { 0, 0 },
    { 0x1de9af7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd39af82, 90 },
    { 0, 0 },
    { 0x5389af84, 23 },
    { 0xe579af85, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb9af88, 98 },
    { 0, 0 },
    { 0x2ca9af8a, 76 },
    { 0xbc89af8a, 34 },
    { 0x4d99af8c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9af91, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729af94, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf729af97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd869afa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a9afa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3499afaa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c59afae, 98 },
    { 0x7e59afae, 84 },
    { 0x6719afb0, 95 },
    { 0, 0 },
    { 0x5459afb2, 67 },
    { 0x9df9afb3, 33 },
    { 0xb269afb4, 36 },
    { 0x1e29afb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9afb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f9afbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xece9afc4, 95 },
    { 0, 0 },
    { 0xf949afc6, 98 },
    { 0xa529afc7, 98 },
    { 0x3259afc8, 52 },
    { 0x37c9afc9, 23 },
    { 0x23c9afca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9849afcd, 90 },
    { 0x9a89afcd, 93 },
    { 0, 0 },
    { 0xcb99afd0, 34 },
    { 0xf379afd1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89afd4, 75 },
    { 0, 0 },
    { 0xfca9afd6, 76 },
    { 0x2f9afd7, 34 },
    { 0xa1b9afd8, 95 },
    { 0xd849afd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2479afe0, 99 },
    { 0x4ae9afe0, 95 },
    { 0xb4b9afe2, 33 },
    { 0x29d9afe3, 84 },
    { 0x9c89afe1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b9afe9, 76 },
    { 0x5cb9afea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a09aff0, 98 },
    { 0, 0 },
    { 0x6b09aff2, 87 },
    { 0, 0 },
    { 0x54f9aff4, 90 },
    { 0, 0 },
    { 0x2d59aff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a09affd, 33 },
    { 0xc549affe, 23 },
    { 0xb459afff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa69b003, 44 },
    { 0, 0 },
    { 0xc759b005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf79b008, 90 },
    { 0xa209b009, 33 },
    { 0, 0 },
    { 0x53a9b00b, 39 },
    { 0xda19b00c, 34 },
    { 0xb3a9b00c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b49b014, 40 },
    { 0x98f9b014, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b9b01a, 23 },
    { 0x7339b01b, 93 },
    { 0x7769b01a, 90 },
    { 0x97d9b01c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa099b021, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b49b026, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e79b02d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939b031, 98 },
    { 0, 0 },
    { 0xf1c9b033, 33 },
    { 0, 0 },
    { 0xf429b035, 87 },
    { 0x3219b036, 90 },
    { 0xd029b037, 98 },
    { 0xa619b038, 76 },
    { 0x94e9b038, 52 },
    { 0xc1f9b039, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eb9b03e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119b048, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d9b04b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6289b050, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f59b053, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf99b057, 78 },
    { 0x20f9b058, 93 },
    { 0x5969b057, 61 },
    { 0xd6a9b05a, 90 },
    { 0x1eb9b05b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f39b062, 61 },
    { 0x60e9b063, 61 },
    { 0x40a9b064, 98 },
    { 0, 0 },
    { 0x68d9b066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa9b069, 88 },
    { 0x2e29b06a, 95 },
    { 0xcd09b06b, 52 },
    { 0x2e19b06b, 84 },
    { 0, 0 },
    { 0x1b39b06e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e9b071, 24 },
    { 0x9809b071, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb9b075, 98 },
    { 0, 0 },
    { 0x3fb9b077, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1899b07d, 23 },
    { 0xd439b07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129b082, 90 },
    { 0x18c9b082, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf759b087, 85 },
    { 0x88f9b087, 78 },
    { 0x6859b088, 23 },
    { 0xf419b08a, 33 },
    { 0x7fe9b08a, 61 },
    { 0x3459b08c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c59b090, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819b094, 33 },
    { 0xb9e9b095, 61 },
    { 0, 0 },
    { 0x7d39b097, 61 },
    { 0xe669b098, 90 },
    { 0x69c9b099, 61 },
    { 0xb369b09a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb699b09d, 98 },
    { 0x2c29b09e, 98 },
    { 0xa369b09e, 61 },
    { 0x18a9b09d, 33 },
    { 0, 0 },
    { 0x3e49b0a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ba9b0a5, 61 },
    { 0x46c9b0a6, 46 },
    { 0xb899b0a7, 95 },
    { 0, 0 },
    { 0xedd9b0a9, 90 },
    { 0, 0 },
    { 0x6ee9b0ab, 100 },
    { 0xd309b0ac, 43 },
    { 0xdb39b0ad, 55 },
    { 0, 0 },
    { 0x7739b0af, 61 },
    { 0x6259b0b0, 98 },
    { 0x73a9b0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd999b0b5, 23 },
    { 0x939b0b5, 33 },
    { 0x7439b0b7, 98 },
    { 0xced9b0b6, 43 },
    { 0, 0 },
    { 0x57d9b0ba, 81 },
    { 0xd709b0bb, 90 },
    { 0xb799b0bc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1169b0c0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4179b0c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd449b0c8, 33 },
    { 0x7f69b0c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489b0cc, 77 },
    { 0xf6d9b0cd, 95 },
    { 0x3a69b0ce, 19 },
    { 0x72a9b0cc, 23 },
    { 0x6309b0d0, 23 },
    { 0xf5b9b0d1, 61 },
    { 0xe3f9b0d2, 23 },
    { 0x4eb9b0d3, 46 },
    { 0x8db9b0d4, 61 },
    { 0x78d9b0d3, 90 },
    { 0x3589b0d6, 52 },
    { 0xb339b0d5, 55 },
    { 0x81b9b0d5, 95 },
    { 0x2269b0d9, 61 },
    { 0xeb39b0da, 61 },
    { 0x9569b0d9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf9b0de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b39b0e5, 100 },
    { 0x44f9b0e6, 46 },
    { 0x9b19b0e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf69b0ec, 91 },
    { 0, 0 },
    { 0x56b9b0ee, 61 },
    { 0x8279b0ef, 76 },
    { 0x4679b0ee, 66 },
    { 0x6f09b0f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7b9b0fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd9b102, 61 },
    { 0, 0 },
    { 0x90b9b104, 61 },
    { 0, 0 },
    { 0xabd9b106, 85 },
    { 0xe7a9b107, 98 },
    { 0xa0b9b108, 46 },
    { 0, 0 },
    { 0xa479b10a, 33 },
    { 0x5aa9b10a, 81 },
    { 0xb7a9b10a, 90 },
    { 0, 0 },
    { 0x2c49b10e, 76 },
    { 0x4e59b10f, 100 },
    { 0xc679b10f, 61 },
    { 0, 0 },
    { 0x489b112, 52 },
    { 0xdcb9b113, 87 },
    { 0xfa09b114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce39b118, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc9b11d, 76 },
    { 0x4319b11d, 33 },
    { 0xa739b11d, 55 },
    { 0xaa79b120, 91 },
    { 0xc569b11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89b124, 61 },
    { 0x4029b125, 34 },
    { 0, 0 },
    { 0x6d99b127, 95 },
    { 0x8989b128, 81 },
    { 0, 0 },
    { 0x3509b12a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb9b12f, 68 },
    { 0xc7a9b130, 33 },
    { 0x959b130, 23 },
    { 0x6d89b12f, 23 },
    { 0xac59b133, 33 },
    { 0x6529b130, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29b138, 33 },
    { 0x8c19b138, 23 },
    { 0x72b9b13a, 34 },
    { 0x97b9b13b, 55 },
    { 0xeda9b13c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529b140, 89 },
    { 0x2079b141, 33 },
    { 0x1a29b141, 34 },
    { 0xdd39b143, 90 },
    { 0xe2f9b143, 52 },
    { 0xdb59b145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6089b148, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c69b151, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429b155, 52 },
    { 0xfe9b156, 90 },
    { 0x1359b155, 61 },
    { 0x8c89b158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x159b15b, 33 },
    { 0, 0 },
    { 0xa2c9b15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649b162, 90 },
    { 0x42b9b163, 84 },
    { 0x4259b164, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df9b16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba9b16f, 91 },
    { 0xfea9b170, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d09b173, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9159b17b, 75 },
    { 0x84e9b17b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f9b183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a9b186, 23 },
    { 0x75c9b187, 76 },
    { 0, 0 },
    { 0xac29b189, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9b18c, 17 },
    { 0x30a9b18c, 61 },
    { 0, 0 },
    { 0xa3c9b18f, 95 },
    { 0x1239b190, 90 },
    { 0xa6b9b191, 46 },
    { 0xb3f9b191, 95 },
    { 0, 0 },
    { 0xebf9b194, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe209b197, 52 },
    { 0x4e29b198, 25 },
    { 0xe949b199, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a69b19d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8089b1a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc909b1a6, 90 },
    { 0xd609b1a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b9b1b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e19b1b6, 99 },
    { 0xc7c9b1b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2489b1ba, 91 },
    { 0xf7e9b1ba, 55 },
    { 0x7539b1bc, 61 },
    { 0x6239b1ba, 95 },
    { 0, 0 },
    { 0x48f9b1bf, 65 },
    { 0x17a9b1c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c9b1c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fe9b1c9, 84 },
    { 0x5329b1ca, 93 },
    { 0, 0 },
    { 0x8419b1cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9889b1d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8449b1d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d9b1db, 99 },
    { 0xd019b1dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf199b1df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c39b1e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a9b1e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569b1ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe29b1f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de9b1fa, 61 },
    { 0, 0 },
    { 0xb599b1fc, 98 },
    { 0x5bc9b1fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9639b203, 34 },
    { 0xc199b203, 55 },
    { 0xa39b204, 100 },
    { 0xb949b206, 90 },
    { 0xd159b205, 75 },
    { 0x5da9b204, 95 },
    { 0x36e9b207, 46 },
    { 0x7559b204, 61 },
    { 0x10c9b20b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3389b210, 55 },
    { 0, 0 },
    { 0x3aa9b212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a9b215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc579b218, 98 },
    { 0xce09b219, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88b9b21c, 76 },
    { 0, 0 },
    { 0xf559b21e, 33 },
    { 0, 0 },
    { 0x4739b220, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5359b225, 55 },
    { 0xfb49b226, 96 },
    { 0x7239b227, 55 },
    { 0x5679b228, 33 },
    { 0xd239b226, 98 },
    { 0x5999b22a, 90 },
    { 0xcf09b22a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519b22e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5039b233, 33 },
    { 0x289b233, 93 },
    { 0, 0 },
    { 0x3ee9b236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4609b239, 95 },
    { 0x4309b23a, 66 },
    { 0x409b23b, 33 },
    { 0, 0 },
    { 0x979b23d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b39b245, 33 },
    { 0, 0 },
    { 0xff79b247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1179b24a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa199b24e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b9b251, 68 },
    { 0x9699b252, 76 },
    { 0x27d9b253, 33 },
    { 0xb419b254, 61 },
    { 0, 0 },
    { 0xe239b256, 61 },
    { 0x51e9b256, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc9b25a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e9b261, 98 },
    { 0, 0 },
    { 0x2dd9b263, 65 },
    { 0xead9b263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb569b269, 90 },
    { 0, 0 },
    { 0xe149b26b, 90 },
    { 0x7419b26b, 33 },
    { 0xc599b26d, 23 },
    { 0, 0 },
    { 0xb7f9b26f, 93 },
    { 0xd629b270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9759b275, 61 },
    { 0xcbb9b275, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5649b282, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9b285, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9b288, 95 },
    { 0x8d59b289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9b28e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd799b291, 23 },
    { 0x60c9b292, 98 },
    { 0, 0 },
    { 0xa349b294, 33 },
    { 0xfc89b295, 43 },
    { 0x26b9b296, 33 },
    { 0, 0 },
    { 0x3ef9b298, 78 },
    { 0, 0 },
    { 0x35d9b29a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139b2a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8179b2a7, 78 },
    { 0x75e9b2a8, 90 },
    { 0xdf29b2a7, 95 },
    { 0x87b9b2aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e9b2af, 55 },
    { 0x7f99b2b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a49b2b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc39b2b9, 23 },
    { 0x9119b2ba, 61 },
    { 0x3199b2b9, 100 },
    { 0, 0 },
    { 0x53d9b2bd, 90 },
    { 0xc7c9b2bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9b2c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc959b2c6, 36 },
    { 0x9dd9b2c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa369b2cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139b2d4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c9b2dd, 76 },
    { 0, 0 },
    { 0xe1a9b2df, 43 },
    { 0, 0 },
    { 0x6d19b2e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c99b2e5, 93 },
    { 0, 0 },
    { 0xaa99b2e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e09b2ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae09b2f4, 46 },
    { 0xb149b2f5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb249b2f8, 81 },
    { 0x9bc9b2f8, 33 },
    { 0x6409b2fa, 43 },
    { 0x94f9b2fb, 90 },
    { 0x8719b2f9, 61 },
    { 0x729b2fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc439b306, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb9b30b, 43 },
    { 0, 0 },
    { 0xdfa9b30d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5969b312, 52 },
    { 0xd539b313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a9b31d, 78 },
    { 0, 0 },
    { 0xab39b31f, 87 },
    { 0, 0 },
    { 0xa689b321, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5789b325, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8829b329, 22 },
    { 0xb849b32a, 55 },
    { 0x4b49b32b, 64 },
    { 0, 0 },
    { 0x96a9b32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a39b331, 90 },
    { 0x5a69b332, 100 },
    { 0x9419b333, 98 },
    { 0x6689b334, 55 },
    { 0x44e9b333, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac9b344, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbde9b348, 46 },
    { 0, 0 },
    { 0xc589b34a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99b34d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc29b353, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd9b357, 78 },
    { 0, 0 },
    { 0xea99b359, 33 },
    { 0, 0 },
    { 0xb929b35b, 93 },
    { 0, 0 },
    { 0xcaf9b35d, 81 },
    { 0x5309b35e, 61 },
    { 0, 0 },
    { 0x1bd9b360, 23 },
    { 0xbe9b361, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7519b364, 58 },
    { 0x73e9b365, 33 },
    { 0, 0 },
    { 0x7499b367, 61 },
    { 0xdc59b367, 95 },
    { 0xcc49b369, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119b36d, 97 },
    { 0x6d19b36e, 55 },
    { 0x7519b36e, 34 },
    { 0x2d9b370, 23 },
    { 0, 0 },
    { 0x4d09b372, 33 },
    { 0xd9c9b372, 34 },
    { 0, 0 },
    { 0x4ea9b375, 55 },
    { 0, 0 },
    { 0x1b99b377, 81 },
    { 0, 0 },
    { 0x1b49b379, 98 },
    { 0x82f9b379, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb9b37d, 33 },
    { 0x83c9b37d, 98 },
    { 0x2a49b37f, 90 },
    { 0xeb69b380, 23 },
    { 0, 0 },
    { 0xcf49b382, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c99b386, 77 },
    { 0xc0d9b387, 90 },
    { 0, 0 },
    { 0xc5d9b389, 76 },
    { 0x7909b38a, 95 },
    { 0x49b38b, 90 },
    { 0x9549b38c, 95 },
    { 0xe39b38a, 34 },
    { 0x3a79b38e, 61 },
    { 0x4369b38f, 95 },
    { 0x9939b38c, 98 },
    { 0x5bf9b38c, 55 },
    { 0x1eb9b392, 95 },
    { 0x4939b392, 95 },
    { 0xdf59b392, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde89b397, 43 },
    { 0, 0 },
    { 0x8089b399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9b3a0, 95 },
    { 0x7049b3a1, 53 },
    { 0, 0 },
    { 0x96f9b3a3, 33 },
    { 0xdb59b3a3, 76 },
    { 0, 0 },
    { 0x55a9b3a6, 95 },
    { 0, 0 },
    { 0x8779b3a8, 65 },
    { 0xcbb9b3a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db9b3af, 17 },
    { 0x5239b3b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9659b3b3, 33 },
    { 0xeaa9b3b4, 95 },
    { 0x8919b3b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb549b3b8, 33 },
    { 0, 0 },
    { 0xb6f9b3ba, 33 },
    { 0, 0 },
    { 0x62a9b3bc, 95 },
    { 0xc799b3bd, 90 },
    { 0xf989b3be, 64 },
    { 0, 0 },
    { 0xeae9b3c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59b3c5, 43 },
    { 0, 0 },
    { 0xb639b3c7, 98 },
    { 0x2249b3c7, 23 },
    { 0xb8f9b3c9, 95 },
    { 0x7e89b3ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949b3cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1479b3d3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb9b3d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9b3dc, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee9b3e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69c9b3e4, 66 },
    { 0x5b99b3e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe489b3ee, 99 },
    { 0, 0 },
    { 0x1ad9b3f0, 46 },
    { 0x4d69b3f1, 67 },
    { 0x4189b3f2, 43 },
    { 0x3589b3f3, 98 },
    { 0, 0 },
    { 0xcfc9b3f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9b3f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f9b401, 33 },
    { 0xe9c9b402, 90 },
    { 0x7489b402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd9b409, 76 },
    { 0xe3c9b40a, 98 },
    { 0xa6d9b40b, 98 },
    { 0, 0 },
    { 0x1579b40d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce9b410, 88 },
    { 0xf419b411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf159b415, 33 },
    { 0xe509b416, 33 },
    { 0, 0 },
    { 0xea99b418, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc519b41c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf739b41f, 66 },
    { 0, 0 },
    { 0xf889b421, 69 },
    { 0, 0 },
    { 0x2e9b423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8689b42a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f39b42e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369b43d, 95 },
    { 0x6319b43d, 61 },
    { 0x1d39b43e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b9b442, 95 },
    { 0, 0 },
    { 0x1a29b444, 61 },
    { 0xfcb9b445, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e9b449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf9b44d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca9b453, 98 },
    { 0x5a9b454, 53 },
    { 0x4029b455, 84 },
    { 0xf919b456, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea59b45d, 23 },
    { 0xd899b45e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf29b461, 95 },
    { 0x1ef9b462, 33 },
    { 0xd6a9b463, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc69b467, 33 },
    { 0, 0 },
    { 0x97c9b469, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb209b46d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7449b473, 90 },
    { 0x2829b474, 87 },
    { 0x1259b473, 90 },
    { 0x6a49b476, 95 },
    { 0xe959b476, 90 },
    { 0, 0 },
    { 0x18a9b479, 90 },
    { 0x2a69b479, 78 },
    { 0x7579b47b, 33 },
    { 0, 0 },
    { 0xad69b47d, 98 },
    { 0x3d99b47e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7599b482, 33 },
    { 0xb929b483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7509b488, 33 },
    { 0, 0 },
    { 0x9499b48a, 100 },
    { 0xc909b48a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce9b48e, 55 },
    { 0x15d9b48f, 23 },
    { 0xc7d9b48e, 66 },
    { 0, 0 },
    { 0x5829b492, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6689b497, 81 },
    { 0x43e9b497, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd279b49d, 55 },
    { 0x2339b49d, 61 },
    { 0x96a9b49f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca9b4a3, 85 },
    { 0x93d9b4a4, 78 },
    { 0xade9b4a5, 61 },
    { 0xef29b4a3, 76 },
    { 0x9e69b4a7, 99 },
    { 0x839b4a8, 23 },
    { 0x2649b4a5, 93 },
    { 0, 0 },
    { 0x8c99b4ab, 33 },
    { 0, 0 },
    { 0x9e39b4ad, 76 },
    { 0, 0 },
    { 0x7c79b4af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xba49b4b2, 76 },
    { 0x3299b4b3, 90 },
    { 0xb159b4b3, 87 },
    { 0x7259b4b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9b4ba, 61 },
    { 0x39c9b4ba, 90 },
    { 0x8f59b4ba, 95 },
    { 0, 0 },
    { 0xdbe9b4be, 46 },
    { 0x1e9b4be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c9b4c2, 33 },
    { 0x62c9b4c2, 33 },
    { 0xf479b4c3, 90 },
    { 0xf3d9b4c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa509b4c9, 34 },
    { 0, 0 },
    { 0x2989b4cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b49b4ce, 76 },
    { 0, 0 },
    { 0x68d9b4d0, 90 },
    { 0xad49b4d1, 95 },
    { 0xf229b4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769b4d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa469b4dd, 76 },
    { 0xa199b4dd, 90 },
    { 0x7e29b4df, 61 },
    { 0xe9b9b4e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9339b4e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb519b4e9, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c9b4ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f9b4f2, 98 },
    { 0xdc79b4f3, 33 },
    { 0xc679b4f2, 61 },
    { 0x56b9b4f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d9b4f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb929b4fb, 89 },
    { 0x3949b4fc, 90 },
    { 0, 0 },
    { 0x6359b4fe, 95 },
    { 0x6b19b4ff, 83 },
    { 0x3369b500, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf579b503, 90 },
    { 0, 0 },
    { 0x1519b505, 34 },
    { 0, 0 },
    { 0xaca9b507, 85 },
    { 0xac89b507, 98 },
    { 0x7ed9b509, 95 },
    { 0xbd59b509, 52 },
    { 0x5eb9b50a, 33 },
    { 0, 0 },
    { 0x1619b50d, 90 },
    { 0, 0 },
    { 0x659b50f, 61 },
    { 0x719b510, 40 },
    { 0xa3b9b511, 86 },
    { 0xb979b512, 23 },
    { 0x8179b510, 33 },
    { 0x6999b514, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xed29b517, 61 },
    { 0x4aa9b518, 95 },
    { 0x4309b519, 100 },
    { 0x4559b51a, 90 },
    { 0xdb59b51b, 90 },
    { 0xa939b51c, 61 },
    { 0x8049b517, 90 },
    { 0xeef9b51e, 90 },
    { 0xa109b51e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x99b522, 90 },
    { 0, 0 },
    { 0x5159b524, 64 },
    { 0x559b524, 69 },
    { 0xf629b526, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf089b529, 90 },
    { 0, 0 },
    { 0xbec9b52b, 66 },
    { 0x1c89b52c, 90 },
    { 0x81d9b52c, 76 },
    { 0x1ae9b52b, 23 },
    { 0x6519b52f, 90 },
    { 0x59f9b530, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7409b538, 90 },
    { 0, 0 },
    { 0xf999b53a, 84 },
    { 0, 0 },
    { 0x1269b53c, 61 },
    { 0xf909b53d, 23 },
    { 0x41f9b53e, 90 },
    { 0x54d9b53f, 95 },
    { 0xcb9b540, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea9b544, 76 },
    { 0x71c9b545, 93 },
    { 0x1279b544, 99 },
    { 0x7e29b545, 97 },
    { 0x6649b547, 33 },
    { 0, 0 },
    { 0xbd69b54a, 40 },
    { 0x9129b54b, 61 },
    { 0x319b54c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979b54f, 77 },
    { 0x6d09b54f, 52 },
    { 0, 0 },
    { 0x2b79b552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a9b556, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a9b559, 90 },
    { 0, 0 },
    { 0xc6e9b55b, 76 },
    { 0x3d69b55c, 67 },
    { 0, 0 },
    { 0x1c99b55e, 44 },
    { 0, 0 },
    { 0x8029b560, 95 },
    { 0xec79b561, 78 },
    { 0, 0 },
    { 0xb449b563, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a9b566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d9b56b, 90 },
    { 0x6249b56b, 65 },
    { 0, 0 },
    { 0xbac9b56e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9df9b572, 94 },
    { 0, 0 },
    { 0x5009b574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5729b579, 85 },
    { 0x9b99b57a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb9b57f, 78 },
    { 0x3e29b580, 61 },
    { 0x8f39b581, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4919b584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd819b58a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c9b58d, 98 },
    { 0, 0 },
    { 0xbb09b58f, 34 },
    { 0xc769b590, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b595, 33 },
    { 0, 0 },
    { 0xe879b597, 77 },
    { 0x409b598, 34 },
    { 0x809b598, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e9b59d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf739b5a4, 23 },
    { 0xde9b5a5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a9b5ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a69b5af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c39b5b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd69b5b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99b5b8, 75 },
    { 0, 0 },
    { 0xdbc9b5ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7969b5bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc539b5c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29b9b5c5, 90 },
    { 0x3409b5c6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd79b5ca, 98 },
    { 0, 0 },
    { 0x86f9b5cc, 66 },
    { 0, 0 },
    { 0x5ef9b5ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf639b5d1, 43 },
    { 0x81a9b5d1, 88 },
    { 0x1909b5d1, 101 },
    { 0x4239b5d4, 99 },
    { 0x49a9b5d2, 95 },
    { 0x4ce9b5d3, 33 },
    { 0, 0 },
    { 0x569b5d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29b5dd, 61 },
    { 0x53b9b5de, 90 },
    { 0x3e29b5dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a9b5e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9b5eb, 98 },
    { 0xb179b5ec, 33 },
    { 0xd499b5ed, 68 },
    { 0x6cb9b5eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac9b5f1, 76 },
    { 0, 0 },
    { 0x26e9b5f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc459b5f8, 66 },
    { 0x2d59b5f9, 90 },
    { 0x39d9b5fa, 95 },
    { 0xdfb9b5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b9b600, 91 },
    { 0, 0 },
    { 0x6d89b602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2479b605, 75 },
    { 0x9959b606, 33 },
    { 0, 0 },
    { 0x5079b608, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6529b60d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a9b610, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d9b614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839b61f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9b625, 23 },
    { 0xa909b626, 61 },
    { 0x4c49b625, 52 },
    { 0x50b9b628, 98 },
    { 0xf49b628, 75 },
    { 0x4389b629, 23 },
    { 0x9749b628, 75 },
    { 0, 0 },
    { 0xf759b62d, 33 },
    { 0x55e9b62d, 84 },
    { 0xafb9b62f, 77 },
    { 0x1369b630, 87 },
    { 0x2689b631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c9b638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a19b63e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc199b642, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4119b649, 100 },
    { 0x9249b649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc369b64d, 81 },
    { 0, 0 },
    { 0x1c9b64f, 90 },
    { 0x529b650, 55 },
    { 0, 0 },
    { 0xdf29b652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc919b655, 34 },
    { 0xb149b656, 90 },
    { 0xe019b657, 98 },
    { 0x7209b658, 98 },
    { 0x54e9b656, 95 },
    { 0xacd9b65a, 90 },
    { 0x1549b656, 76 },
    { 0xc599b65b, 36 },
    { 0xc5b9b655, 98 },
    { 0xac49b65e, 61 },
    { 0, 0 },
    { 0x8c09b660, 90 },
    { 0, 0 },
    { 0x7fb9b662, 23 },
    { 0x4359b663, 90 },
    { 0, 0 },
    { 0x92f9b665, 38 },
    { 0xcc29b665, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd969b66a, 98 },
    { 0xab09b66a, 90 },
    { 0x71b9b66a, 95 },
    { 0x25f9b66d, 26 },
    { 0x4ae9b66d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d9b673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae29b677, 19 },
    { 0, 0 },
    { 0x76e9b679, 80 },
    { 0, 0 },
    { 0x5ba9b67b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd9b686, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc349b68a, 100 },
    { 0, 0 },
    { 0xb6d9b68c, 95 },
    { 0, 0 },
    { 0x3129b68e, 95 },
    { 0x7ed9b68e, 90 },
    { 0x4499b68e, 38 },
    { 0x1aa9b68e, 23 },
    { 0x6cd9b692, 33 },
    { 0x2859b692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec9b696, 93 },
    { 0xab69b697, 97 },
    { 0, 0 },
    { 0xbd99b699, 34 },
    { 0xd2f9b69a, 36 },
    { 0x7489b69a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1729b6a0, 33 },
    { 0x3c99b6a1, 81 },
    { 0xac29b6a2, 33 },
    { 0x7ef9b6a3, 95 },
    { 0x3609b6a4, 23 },
    { 0xbba9b6a5, 95 },
    { 0x2e29b6a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce9b6ad, 51 },
    { 0xfc19b6ad, 93 },
    { 0, 0 },
    { 0x639b6b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29b6b3, 98 },
    { 0x9b19b6b3, 90 },
    { 0, 0 },
    { 0x70b9b6b6, 76 },
    { 0xbaa9b6b7, 87 },
    { 0xbd09b6b8, 95 },
    { 0, 0 },
    { 0x6d59b6ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf649b6bd, 93 },
    { 0x6429b6be, 76 },
    { 0x4929b6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89b6c4, 23 },
    { 0, 0 },
    { 0x96e9b6c6, 75 },
    { 0xf259b6c7, 23 },
    { 0xffe9b6c6, 66 },
    { 0xb539b6c7, 55 },
    { 0, 0 },
    { 0x6c29b6cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1c9b6d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2039b6dc, 34 },
    { 0xe5e9b6dc, 55 },
    { 0x7a79b6de, 43 },
    { 0x3a69b6dd, 66 },
    { 0xbd29b6e0, 66 },
    { 0x3149b6e0, 95 },
    { 0x1309b6e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce69b6e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e39b6ea, 90 },
    { 0, 0 },
    { 0x7479b6ec, 76 },
    { 0x9a29b6ed, 78 },
    { 0, 0 },
    { 0x9799b6ef, 95 },
    { 0, 0 },
    { 0x1189b6f1, 61 },
    { 0xee89b6f2, 100 },
    { 0x26e9b6f3, 93 },
    { 0, 0 },
    { 0x27d9b6f5, 95 },
    { 0x6589b6f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d9b6fb, 33 },
    { 0, 0 },
    { 0x7589b6fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa739b703, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b9b70a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf879b70e, 98 },
    { 0xd579b70e, 100 },
    { 0x2f99b710, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9669b717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79b71a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79b71e, 55 },
    { 0xd479b71f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20a9b724, 76 },
    { 0xb039b725, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6349b729, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd9b72d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd839b731, 34 },
    { 0x1cb9b732, 90 },
    { 0, 0 },
    { 0x809b734, 99 },
    { 0x3fe9b735, 61 },
    { 0xff69b736, 33 },
    { 0xe49b737, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4159b73b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b59b73f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe599b744, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed9b747, 55 },
    { 0xf499b748, 90 },
    { 0x8ae9b749, 90 },
    { 0x7dd9b74a, 90 },
    { 0, 0 },
    { 0x59e9b74c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b74f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b9b756, 94 },
    { 0, 0 },
    { 0x9209b758, 56 },
    { 0x32f9b758, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac9b75d, 23 },
    { 0x3389b75e, 67 },
    { 0, 0 },
    { 0x6969b760, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9b763, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979b76a, 77 },
    { 0, 0 },
    { 0xa5f9b76c, 90 },
    { 0xe389b76d, 90 },
    { 0x8d59b76e, 61 },
    { 0xb6d9b76f, 81 },
    { 0, 0 },
    { 0xac59b771, 76 },
    { 0, 0 },
    { 0xd409b773, 61 },
    { 0x9169b774, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b9b777, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c9b77d, 90 },
    { 0x66a9b77e, 84 },
    { 0x4829b77f, 61 },
    { 0xd9b9b780, 76 },
    { 0, 0 },
    { 0x6089b782, 55 },
    { 0, 0 },
    { 0xa349b784, 43 },
    { 0xc1d9b785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36c9b788, 90 },
    { 0x81a9b788, 90 },
    { 0x7c99b78a, 98 },
    { 0, 0 },
    { 0xf789b78c, 46 },
    { 0x4f9b78d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9b792, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1369b795, 33 },
    { 0, 0 },
    { 0xb759b797, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1299b79b, 33 },
    { 0, 0 },
    { 0x5a39b79d, 43 },
    { 0x1f9b79e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a9b7a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76e9b7a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7179b7aa, 98 },
    { 0x19f9b7aa, 33 },
    { 0, 0 },
    { 0x97c9b7ad, 90 },
    { 0x9b19b7ad, 90 },
    { 0x3a79b7af, 33 },
    { 0x6bd9b7b0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f29b7b4, 43 },
    { 0, 0 },
    { 0xf7a9b7b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf9b7b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b49b7bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe729b7c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e19b7cb, 90 },
    { 0xb069b7cc, 95 },
    { 0xfe09b7cd, 90 },
    { 0x3b79b7cb, 43 },
    { 0xbec9b7ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f9b7d6, 93 },
    { 0x9ed9b7d6, 84 },
    { 0xc699b7d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4859b7de, 90 },
    { 0x90b9b7de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c9b7e5, 61 },
    { 0xd099b7e6, 23 },
    { 0xc999b7e5, 61 },
    { 0x88d9b7e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe09b7ec, 90 },
    { 0, 0 },
    { 0xaf79b7ee, 61 },
    { 0x9f69b7ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9709b7f2, 33 },
    { 0x54e9b7f2, 93 },
    { 0xb2f9b7f4, 76 },
    { 0x64a9b7f4, 33 },
    { 0x50b9b7f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d09b7f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f9b800, 90 },
    { 0x7149b801, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b49b806, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5309b80f, 77 },
    { 0xb329b80f, 76 },
    { 0xac69b810, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae9b816, 90 },
    { 0xa2f9b817, 98 },
    { 0x5f79b818, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2089b81c, 43 },
    { 0x8949b81c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6209b822, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4319b825, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be9b829, 33 },
    { 0xb499b82a, 33 },
    { 0x7cf9b82b, 46 },
    { 0xb379b82c, 33 },
    { 0xe879b82d, 76 },
    { 0, 0 },
    { 0x57e9b82f, 78 },
    { 0x1b39b830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1689b833, 90 },
    { 0x8519b833, 84 },
    { 0xce79b835, 99 },
    { 0x56b9b835, 61 },
    { 0, 0 },
    { 0xfe99b838, 61 },
    { 0x4259b838, 90 },
    { 0x7b69b83a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2779b83d, 55 },
    { 0, 0 },
    { 0xf3f9b83f, 90 },
    { 0x6259b83f, 23 },
    { 0xf889b841, 61 },
    { 0x4f89b841, 78 },
    { 0xc199b842, 36 },
    { 0x2669b843, 90 },
    { 0xb999b845, 78 },
    { 0xa899b846, 80 },
    { 0xff09b847, 64 },
    { 0x70f9b847, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c9b84d, 90 },
    { 0, 0 },
    { 0x7d79b84f, 95 },
    { 0x83e9b850, 61 },
    { 0x6369b851, 95 },
    { 0x3129b852, 33 },
    { 0xbbe9b853, 95 },
    { 0x3069b850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9b858, 93 },
    { 0x8329b859, 90 },
    { 0, 0 },
    { 0x6859b85b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29b860, 98 },
    { 0x9ae9b861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b39b865, 98 },
    { 0, 0 },
    { 0x9fd9b867, 90 },
    { 0x6f39b868, 46 },
    { 0xbd9b868, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a9b86c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9b870, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c9b875, 55 },
    { 0xc9f9b876, 23 },
    { 0xac59b875, 33 },
    { 0xfec9b877, 90 },
    { 0x6989b877, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4609b87d, 61 },
    { 0, 0 },
    { 0x6f89b87f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd9b885, 67 },
    { 0x8649b885, 95 },
    { 0, 0 },
    { 0xf529b888, 98 },
    { 0xf689b888, 98 },
    { 0xc519b88a, 61 },
    { 0xcf89b88b, 67 },
    { 0xa2a9b88b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d9b890, 71 },
    { 0x8c19b891, 77 },
    { 0, 0 },
    { 0x1b49b893, 90 },
    { 0xfe39b894, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9b897, 90 },
    { 0, 0 },
    { 0xb2b9b899, 33 },
    { 0x5109b89a, 33 },
    { 0, 0 },
    { 0x2089b89c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9b89f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9b8a2, 89 },
    { 0x7479b8a3, 33 },
    { 0x21e9b8a4, 99 },
    { 0xb369b8a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9b8ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9b8b0, 90 },
    { 0xf929b8b0, 61 },
    { 0xaac9b8b2, 77 },
    { 0xedb9b8b3, 87 },
    { 0x1b69b8b4, 85 },
    { 0x4a19b8b5, 61 },
    { 0x4fd9b8b6, 33 },
    { 0x1a49b8b7, 98 },
    { 0, 0 },
    { 0xd259b8b9, 95 },
    { 0, 0 },
    { 0x8839b8bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3929b8be, 23 },
    { 0, 0 },
    { 0x6729b8c0, 33 },
    { 0x9da9b8c1, 23 },
    { 0xd299b8c2, 95 },
    { 0, 0 },
    { 0x79a9b8c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd9b8c8, 87 },
    { 0x96b9b8c9, 55 },
    { 0, 0 },
    { 0x7069b8cb, 33 },
    { 0xb7f9b8cc, 77 },
    { 0x7169b8cd, 34 },
    { 0x1589b8cb, 95 },
    { 0xb1e9b8cf, 61 },
    { 0x79a9b8d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a49b8d5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be9b8db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09b8df, 23 },
    { 0x2eb9b8e0, 98 },
    { 0, 0 },
    { 0x1db9b8e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xace9b8e5, 78 },
    { 0x1c99b8e6, 66 },
    { 0xd579b8e7, 67 },
    { 0x5199b8e8, 74 },
    { 0x1c89b8e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb39b8ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda19b8ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd9b8f2, 55 },
    { 0x5a19b8f3, 61 },
    { 0x8f29b8f4, 76 },
    { 0x8e9b8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2619b8fa, 33 },
    { 0x2f89b8fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c9b8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7029b902, 95 },
    { 0xe0e9b903, 95 },
    { 0, 0 },
    { 0x9de9b905, 23 },
    { 0x54a9b906, 33 },
    { 0x2709b906, 78 },
    { 0x7869b908, 77 },
    { 0xa0a9b908, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859b90f, 33 },
    { 0x48c9b90f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4f9b914, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6839b91a, 95 },
    { 0xda79b91a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99b91e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b39b922, 93 },
    { 0x9fb9b923, 83 },
    { 0x2679b923, 98 },
    { 0, 0 },
    { 0x2c89b926, 90 },
    { 0, 0 },
    { 0xf4c9b928, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9b92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9b931, 33 },
    { 0x1259b932, 90 },
    { 0xe119b932, 100 },
    { 0x5529b934, 40 },
    { 0xb909b934, 95 },
    { 0x65c9b933, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b93d, 88 },
    { 0xcf09b93e, 43 },
    { 0xc699b93e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb269b942, 43 },
    { 0x1fc9b943, 95 },
    { 0x1999b943, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb269b94a, 23 },
    { 0x1c59b94b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd339b94f, 33 },
    { 0, 0 },
    { 0xe289b951, 87 },
    { 0, 0 },
    { 0xe529b953, 33 },
    { 0x1309b954, 76 },
    { 0xd649b955, 90 },
    { 0xb199b956, 90 },
    { 0xe359b956, 95 },
    { 0xdd29b954, 33 },
    { 0, 0 },
    { 0x6849b95a, 55 },
    { 0x8039b95b, 66 },
    { 0xadc9b95c, 23 },
    { 0, 0 },
    { 0xbf89b95e, 95 },
    { 0x2299b95f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92f9b962, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1619b966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab9b96b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed19b96e, 98 },
    { 0, 0 },
    { 0x17f9b970, 66 },
    { 0xa7c9b971, 61 },
    { 0, 0 },
    { 0xcdd9b973, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x3609b976, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9b979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819b97e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2079b98a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1639b98d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269b991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479b996, 85 },
    { 0x3309b997, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a9b99a, 76 },
    { 0x12b9b99b, 33 },
    { 0xb8e9b99b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc19b9a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6069b9a7, 95 },
    { 0x87d9b9a7, 98 },
    { 0x3859b9a9, 33 },
    { 0x93c9b9a8, 40 },
    { 0x4dd9b9ab, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a9b9b0, 95 },
    { 0x1a09b9b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c89b9b7, 77 },
    { 0x7929b9b8, 90 },
    { 0x4ab9b9b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f9b9bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289b9c0, 98 },
    { 0xa0c9b9c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa589b9c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359b9c8, 90 },
    { 0x8759b9c8, 55 },
    { 0x609b9ca, 52 },
    { 0xdb19b9cb, 55 },
    { 0x3f89b9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae19b9d0, 93 },
    { 0, 0 },
    { 0x49d9b9d2, 98 },
    { 0, 0 },
    { 0x28e9b9d4, 17 },
    { 0xdbe9b9d5, 52 },
    { 0x5249b9d6, 95 },
    { 0xccd9b9d7, 61 },
    { 0x4f29b9d4, 33 },
    { 0x3ac9b9d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc219b9dc, 76 },
    { 0x25e9b9dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a09b9e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109b9eb, 84 },
    { 0x33d9b9eb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc9b9ef, 61 },
    { 0xa139b9f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2539b9f4, 87 },
    { 0xb239b9f4, 90 },
    { 0x7639b9f4, 61 },
    { 0, 0 },
    { 0x6f59b9f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d89b9fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9b9ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759ba03, 90 },
    { 0xe009ba03, 90 },
    { 0xcb79ba05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139ba08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe389ba0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e79ba12, 76 },
    { 0xba69ba12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c89ba16, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec99ba1e, 61 },
    { 0x979ba1f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1459ba27, 98 },
    { 0, 0 },
    { 0x4a79ba29, 95 },
    { 0x4369ba2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5409ba2f, 80 },
    { 0, 0 },
    { 0xe799ba31, 33 },
    { 0, 0 },
    { 0xe959ba33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a9ba37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd559ba3c, 76 },
    { 0xb789ba3d, 61 },
    { 0, 0 },
    { 0x9fb9ba3f, 99 },
    { 0x18a9ba40, 33 },
    { 0xd3b9ba41, 68 },
    { 0, 0 },
    { 0xc7d9ba43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae9ba4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949ba51, 91 },
    { 0x5f39ba51, 95 },
    { 0, 0 },
    { 0xb1f9ba54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9ba67, 58 },
    { 0x3979ba68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc69ba6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadd9ba6f, 33 },
    { 0x72b9ba70, 33 },
    { 0x5049ba6f, 33 },
    { 0x1189ba72, 43 },
    { 0, 0 },
    { 0xa749ba74, 33 },
    { 0xbbb9ba75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19ba7a, 33 },
    { 0xe2b9ba7b, 81 },
    { 0x8729ba7c, 61 },
    { 0x7bb9ba7c, 98 },
    { 0x6b39ba7a, 90 },
    { 0x47c9ba7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a59ba83, 40 },
    { 0xcf59ba84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b9ba89, 90 },
    { 0x1489ba8a, 61 },
    { 0xfb59ba89, 33 },
    { 0x2409ba8a, 17 },
    { 0, 0 },
    { 0x9179ba8e, 90 },
    { 0x1479ba8f, 55 },
    { 0, 0 },
    { 0xbd69ba91, 99 },
    { 0, 0 },
    { 0x2149ba93, 55 },
    { 0x5d09ba94, 23 },
    { 0, 0 },
    { 0xe09ba96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b9baa0, 98 },
    { 0x6d9baa1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c39baa7, 93 },
    { 0xc159baa8, 95 },
    { 0xc949baa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa119baaf, 61 },
    { 0, 0 },
    { 0x3e89bab1, 23 },
    { 0x6259bab2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee69bab5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf79bab9, 76 },
    { 0xbde9baba, 33 },
    { 0x5a39baba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9babe, 96 },
    { 0, 0 },
    { 0x8049bac0, 88 },
    { 0, 0 },
    { 0x7a99bac2, 87 },
    { 0x13f9bac3, 95 },
    { 0x88f9bac4, 33 },
    { 0x68a9bac4, 95 },
    { 0x3e89bac4, 55 },
    { 0x50e9bac4, 48 },
    { 0xb549bac8, 93 },
    { 0xc8b9bac9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569bace, 43 },
    { 0xd199bace, 76 },
    { 0x3139bad0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9bad4, 43 },
    { 0xed9bad5, 95 },
    { 0x34e9bad6, 78 },
    { 0, 0 },
    { 0x4669bad8, 23 },
    { 0x2889bad8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a69badd, 52 },
    { 0xba69badd, 91 },
    { 0x7fa9bade, 90 },
    { 0xf019badf, 33 },
    { 0xdb29bae1, 23 },
    { 0xaf29bade, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4549bae8, 90 },
    { 0xf6b9bae9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a9baf3, 43 },
    { 0x63f9baf4, 46 },
    { 0, 0 },
    { 0x2419baf6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb9bafb, 55 },
    { 0, 0 },
    { 0x519bafd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59bb04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf49bb0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd9bb10, 46 },
    { 0x47e9bb10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d39bb17, 52 },
    { 0xc8d9bb17, 90 },
    { 0xde79bb19, 55 },
    { 0, 0 },
    { 0xdef9bb1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d19bb1f, 85 },
    { 0, 0 },
    { 0x7639bb21, 67 },
    { 0x1179bb21, 61 },
    { 0x929bb23, 90 },
    { 0xd8b9bb23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd9bb27, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f9bb2b, 23 },
    { 0x9459bb2c, 33 },
    { 0x82d9bb2c, 90 },
    { 0xb2c9bb2d, 55 },
    { 0, 0 },
    { 0x36d9bb30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaf9bb3a, 33 },
    { 0x4759bb3b, 95 },
    { 0x6d19bb3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86b9bb45, 61 },
    { 0, 0 },
    { 0x3559bb47, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f9bb4a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf089bb4d, 95 },
    { 0, 0 },
    { 0xc9d9bb4f, 98 },
    { 0, 0 },
    { 0xdea9bb51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9909bb54, 90 },
    { 0x6649bb54, 99 },
    { 0, 0 },
    { 0xab59bb57, 76 },
    { 0x1ef9bb57, 98 },
    { 0x4b19bb59, 76 },
    { 0xdb99bb5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf379bb5f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xba89bb62, 76 },
    { 0x23c9bb63, 76 },
    { 0x9c89bb62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b9bb6a, 61 },
    { 0, 0 },
    { 0x7399bb6c, 87 },
    { 0xb539bb6c, 91 },
    { 0, 0 },
    { 0x9d39bb6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a29bb73, 88 },
    { 0x7919bb73, 43 },
    { 0x8249bb75, 23 },
    { 0, 0 },
    { 0xfe99bb77, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb429bb7a, 76 },
    { 0xd349bb7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799bb80, 61 },
    { 0x9559bb81, 23 },
    { 0x6c39bb82, 43 },
    { 0x6709bb83, 93 },
    { 0xcce9bb82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6119bb89, 67 },
    { 0x8369bb89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6609bb93, 33 },
    { 0xb7a9bb94, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9bb9a, 23 },
    { 0, 0 },
    { 0x5299bb9c, 34 },
    { 0, 0 },
    { 0x4519bb9e, 61 },
    { 0x9a99bb9e, 33 },
    { 0x7289bb9f, 34 },
    { 0x2c49bba1, 38 },
    { 0xd889bba1, 23 },
    { 0xd4f9bba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c9bba6, 90 },
    { 0x9f69bba6, 87 },
    { 0x1559bba8, 33 },
    { 0x1599bba7, 90 },
    { 0xbbf9bba8, 75 },
    { 0x36d9bbab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7359bbb0, 91 },
    { 0x5279bbb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc69bbb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6699bbb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2989bbba, 33 },
    { 0xbf9bbbb, 99 },
    { 0x2f69bbbc, 84 },
    { 0x53f9bbba, 61 },
    { 0x39a9bbbc, 33 },
    { 0xf159bbbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9bbc4, 85 },
    { 0x30c9bbc5, 55 },
    { 0x61a9bbc6, 95 },
    { 0x5c99bbc7, 55 },
    { 0xc4b9bbc6, 33 },
    { 0x7e39bbc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3d9bbd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad29bbd7, 55 },
    { 0x63d9bbd8, 61 },
    { 0x639bbd9, 90 },
    { 0, 0 },
    { 0xe0e9bbdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79bbde, 98 },
    { 0x51b9bbdf, 61 },
    { 0x5b99bbe0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa259bbe3, 33 },
    { 0, 0 },
    { 0xe7a9bbe5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3c9bbea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49bbf0, 88 },
    { 0x2479bbf1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x149bbf4, 33 },
    { 0x6619bbf4, 90 },
    { 0, 0 },
    { 0x7c59bbf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e19bbfa, 34 },
    { 0, 0 },
    { 0xce59bbfc, 61 },
    { 0xf889bbfd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e89bc00, 33 },
    { 0, 0 },
    { 0x5bf9bc02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f79bc05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe9bc0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x859bc0d, 33 },
    { 0x2179bc0d, 76 },
    { 0x7e49bc0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89bc13, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa299bc1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9bc23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d9bc29, 23 },
    { 0x40e9bc29, 77 },
    { 0x2489bc2b, 33 },
    { 0xe489bc2c, 23 },
    { 0x9169bc29, 61 },
    { 0x29e9bc2e, 64 },
    { 0, 0 },
    { 0xe889bc30, 95 },
    { 0x31f9bc30, 98 },
    { 0xd959bc31, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe809bc3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a9bc43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c99bc48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7939bc4b, 64 },
    { 0x5759bc4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a49bc51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcac9bc55, 98 },
    { 0xb1e9bc55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe609bc5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3449bc63, 61 },
    { 0, 0 },
    { 0x1b49bc65, 33 },
    { 0x4459bc66, 86 },
    { 0xbe99bc66, 98 },
    { 0x3599bc65, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9bc6c, 55 },
    { 0x6b39bc6d, 98 },
    { 0, 0 },
    { 0x8cd9bc6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed9bc74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f9bc77, 85 },
    { 0x8e89bc78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d49bc81, 23 },
    { 0xc19bc82, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4419bc86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eb9bc89, 61 },
    { 0, 0 },
    { 0x3879bc8b, 93 },
    { 0x4ba9bc8c, 78 },
    { 0xc609bc8d, 56 },
    { 0x8299bc8d, 34 },
    { 0x1fc9bc8f, 46 },
    { 0xd389bc90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d9bc93, 85 },
    { 0x94e9bc93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb339bc97, 46 },
    { 0xf399bc98, 34 },
    { 0x5d99bc97, 90 },
    { 0x9e49bc9a, 61 },
    { 0x9309bc9b, 95 },
    { 0x29d9bc9b, 40 },
    { 0, 0 },
    { 0xa749bc9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c9bca1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52b9bca5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f9bca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9249bcb8, 81 },
    { 0x5bf9bcb9, 34 },
    { 0, 0 },
    { 0xe869bcbb, 88 },
    { 0xd589bcbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a9bcbf, 90 },
    { 0, 0 },
    { 0x82f9bcc1, 43 },
    { 0x66a9bcc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1429bcc7, 81 },
    { 0xacc9bcc7, 55 },
    { 0, 0 },
    { 0xf599bcca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe59bccd, 98 },
    { 0xad59bccd, 95 },
    { 0xea19bccf, 98 },
    { 0x4019bcd0, 88 },
    { 0x9599bcd1, 78 },
    { 0x7bc9bcd2, 98 },
    { 0xde99bcd2, 61 },
    { 0x7e59bcd2, 90 },
    { 0x9e49bcd5, 43 },
    { 0x7a79bcd5, 95 },
    { 0xc239bccd, 61 },
    { 0x8ce9bccd, 88 },
    { 0x98f9bcd9, 88 },
    { 0x31b9bcd8, 98 },
    { 0x5c49bcdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39bcde, 81 },
    { 0xf869bcdf, 68 },
    { 0x4d9bcde, 23 },
    { 0xd049bce0, 84 },
    { 0x7c29bce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f19bce6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7429bcea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf9bcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f9bcf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee29bcfb, 33 },
    { 0xd559bcfb, 93 },
    { 0x86c9bcfc, 90 },
    { 0x3e69bcfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f39bd01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc739bd09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c9bd0f, 33 },
    { 0, 0 },
    { 0x85e9bd11, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb9bd16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bd1b, 67 },
    { 0x2d9bd1b, 33 },
    { 0, 0 },
    { 0x9b9bd1e, 98 },
    { 0xefe9bd1f, 93 },
    { 0, 0 },
    { 0x4599bd21, 61 },
    { 0xa199bd21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc559bd25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa389bd28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9bd2b, 98 },
    { 0xd959bd2b, 98 },
    { 0, 0 },
    { 0xdf29bd2e, 66 },
    { 0xddb9bd2f, 23 },
    { 0xb9c9bd30, 95 },
    { 0x55f9bd31, 17 },
    { 0x27b9bd32, 55 },
    { 0, 0 },
    { 0x2dd9bd34, 75 },
    { 0x9ac9bd35, 95 },
    { 0x9249bd35, 93 },
    { 0, 0 },
    { 0x6889bd38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6349bd3b, 33 },
    { 0xe189bd3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7379bd41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad89bd45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda99bd48, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6649bd50, 34 },
    { 0x7519bd51, 33 },
    { 0xb999bd51, 87 },
    { 0xfab9bd53, 33 },
    { 0x4909bd54, 65 },
    { 0x6439bd54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec9bd58, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf989bd67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf569bd6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb479bd6f, 33 },
    { 0xbc19bd70, 98 },
    { 0x9499bd70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d9bd75, 98 },
    { 0, 0 },
    { 0x8749bd77, 90 },
    { 0, 0 },
    { 0x7de9bd79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc029bd7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9bd85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f9bd8b, 66 },
    { 0x48c9bd8b, 76 },
    { 0xc959bd8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa199bd90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a99bd93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf9bd98, 90 },
    { 0x40d9bd99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae9bd9c, 33 },
    { 0x3fb9bd9d, 76 },
    { 0x3d69bd9d, 98 },
    { 0x87e9bd9f, 99 },
    { 0, 0 },
    { 0x3cf9bda1, 55 },
    { 0xc1f9bda2, 90 },
    { 0x4369bda3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9bda9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669bdaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9bdb2, 96 },
    { 0, 0 },
    { 0x75e9bdb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bdb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf9bdbf, 90 },
    { 0xea09bdc0, 95 },
    { 0x10d9bdc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3279bdc9, 95 },
    { 0xe699bdca, 98 },
    { 0x479bdcb, 76 },
    { 0, 0 },
    { 0xc849bdcd, 33 },
    { 0, 0 },
    { 0x57c9bdcf, 90 },
    { 0, 0 },
    { 0x9a69bdd1, 23 },
    { 0, 0 },
    { 0x94d9bdd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7be9bdd9, 53 },
    { 0x40d9bdda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9bde1, 78 },
    { 0x5679bde2, 90 },
    { 0xc6d9bde1, 55 },
    { 0x4009bde4, 33 },
    { 0x93b9bde5, 76 },
    { 0, 0 },
    { 0x2889bde7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9bdea, 17 },
    { 0xb0d9bdeb, 90 },
    { 0, 0 },
    { 0x5419bded, 33 },
    { 0, 0 },
    { 0x999bdef, 61 },
    { 0x7399bdf0, 76 },
    { 0, 0 },
    { 0xba19bdf2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a9bdf5, 61 },
    { 0x3699bdf6, 95 },
    { 0x3869bdf7, 97 },
    { 0x9509bdf7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca49be04, 90 },
    { 0xe69be05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b69be0f, 90 },
    { 0, 0 },
    { 0xed89be11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2309be15, 40 },
    { 0x6d59be16, 78 },
    { 0, 0 },
    { 0xdc69be18, 33 },
    { 0x4b79be18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe289be1c, 76 },
    { 0x5419be1d, 33 },
    { 0x6119be1e, 99 },
    { 0x9699be1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4029be27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2099be2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a9be31, 95 },
    { 0x3de9be32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf9be35, 61 },
    { 0x71b9be36, 33 },
    { 0xb59be36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d29be3e, 81 },
    { 0, 0 },
    { 0x68c9be40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf229be45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e9be48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a9be4b, 95 },
    { 0, 0 },
    { 0x24a9be4d, 98 },
    { 0, 0 },
    { 0x71e9be4f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9be52, 88 },
    { 0xb7b9be52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec39be58, 34 },
    { 0, 0 },
    { 0x6789be5a, 90 },
    { 0x9959be5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5569be5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a9be65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e9be68, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc9be6b, 88 },
    { 0x4799be6c, 76 },
    { 0, 0 },
    { 0x4fb9be6e, 23 },
    { 0xe209be6f, 90 },
    { 0x8459be6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c9be73, 17 },
    { 0x5959be74, 95 },
    { 0, 0 },
    { 0x4dc9be76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3079be7a, 98 },
    { 0x2209be7b, 55 },
    { 0xab69be7c, 33 },
    { 0x7db9be7c, 81 },
    { 0x33d9be7a, 33 },
    { 0xac09be7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4879be83, 34 },
    { 0xe309be84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c9be87, 23 },
    { 0, 0 },
    { 0x61e9be89, 55 },
    { 0, 0 },
    { 0x2739be8b, 55 },
    { 0xea19be8c, 90 },
    { 0x7f09be8d, 23 },
    { 0, 0 },
    { 0x9659be8f, 55 },
    { 0xab9be90, 46 },
    { 0x5099be90, 34 },
    { 0xebe9be92, 62 },
    { 0x1b29be93, 43 },
    { 0x1cc9be92, 61 },
    { 0xf039be8f, 99 },
    { 0xe689be96, 100 },
    { 0, 0 },
    { 0xa449be98, 43 },
    { 0, 0 },
    { 0xd4c9be9a, 100 },
    { 0x169be9a, 55 },
    { 0, 0 },
    { 0x3d79be9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac09bea4, 23 },
    { 0x5a29bea5, 76 },
    { 0xd1e9bea6, 23 },
    { 0x6359bea7, 33 },
    { 0xf4f9bea8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f9beac, 90 },
    { 0xb329bead, 61 },
    { 0xb9c9beae, 98 },
    { 0x4d19beae, 90 },
    { 0xe5d9beae, 23 },
    { 0, 0 },
    { 0x4559beb2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b99beb5, 65 },
    { 0, 0 },
    { 0x6dd9beb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b69bebe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8669bec3, 91 },
    { 0xc809bec4, 52 },
    { 0x5b49bec5, 76 },
    { 0xe639bec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f9becd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d9bed1, 61 },
    { 0, 0 },
    { 0xd439bed3, 93 },
    { 0xa8c9bed3, 84 },
    { 0x2c79bed4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9bed8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa319bee1, 43 },
    { 0x5959bee2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb669bee5, 61 },
    { 0, 0 },
    { 0xad99bee7, 33 },
    { 0x9eb9bee7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa549beec, 90 },
    { 0, 0 },
    { 0x86c9beee, 61 },
    { 0, 0 },
    { 0xadc9bef0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca9bef3, 43 },
    { 0xda69bef4, 100 },
    { 0xa919bef5, 46 },
    { 0x8179bef6, 90 },
    { 0xed39bef5, 67 },
    { 0x4a79bef7, 87 },
    { 0x9b29bef9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89bf07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7229bf13, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54d9bf1b, 23 },
    { 0x4609bf1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd249bf20, 90 },
    { 0x30f9bf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb9bf28, 46 },
    { 0, 0 },
    { 0x8979bf2a, 46 },
    { 0, 0 },
    { 0x3149bf2c, 61 },
    { 0xa4f9bf2c, 67 },
    { 0, 0 },
    { 0x5549bf2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab9bf38, 90 },
    { 0xc779bf39, 90 },
    { 0xa079bf3a, 46 },
    { 0xe869bf39, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa139bf40, 90 },
    { 0x319bf41, 73 },
    { 0x4249bf41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9809bf48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9bf56, 61 },
    { 0, 0 },
    { 0xc709bf58, 93 },
    { 0, 0 },
    { 0xc939bf5a, 90 },
    { 0x8fb9bf5a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e9bf5e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6619bf64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809bf68, 98 },
    { 0xc179bf69, 43 },
    { 0x3079bf6a, 95 },
    { 0, 0 },
    { 0xf309bf6c, 95 },
    { 0, 0 },
    { 0x8599bf6e, 43 },
    { 0x3dc9bf6f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa9bf76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd949bf7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab9bf7e, 76 },
    { 0, 0 },
    { 0xfaf9bf80, 61 },
    { 0, 0 },
    { 0x11d9bf82, 34 },
    { 0, 0 },
    { 0x639bf84, 81 },
    { 0, 0 },
    { 0xf0b9bf86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f79bf94, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc59bf98, 95 },
    { 0x8379bf99, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74a9bf9d, 87 },
    { 0x9529bf9e, 95 },
    { 0x41f9bf9f, 75 },
    { 0x5979bf9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21f9bfa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f9bfaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c9bfad, 36 },
    { 0x4d49bfae, 33 },
    { 0x689bfad, 91 },
    { 0x4b59bfb0, 67 },
    { 0xfca9bfb1, 46 },
    { 0x8cd9bfb2, 84 },
    { 0x1609bfb1, 55 },
    { 0, 0 },
    { 0xe9d9bfb5, 95 },
    { 0, 0 },
    { 0xf0f9bfb7, 77 },
    { 0x5e89bfb8, 90 },
    { 0, 0 },
    { 0xfd99bfba, 100 },
    { 0xef39bfbb, 90 },
    { 0x7939bfba, 61 },
    { 0x8d39bfbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x679bfc0, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xd129bfc3, 61 },
    { 0, 0 },
    { 0xafa9bfc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf389bfc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8039bfce, 61 },
    { 0xb669bfcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad9bfd4, 90 },
    { 0xdd79bfd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb29bfdb, 32 },
    { 0, 0 },
    { 0x4a39bfdd, 90 },
    { 0x8619bfde, 76 },
    { 0x6849bfdd, 46 },
    { 0xb149bfe0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa089bfe6, 98 },
    { 0x8469bfe6, 61 },
    { 0, 0 },
    { 0xaec9bfe9, 33 },
    { 0xfc49bfea, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e79bfed, 90 },
    { 0x5fd9bfee, 46 },
    { 0x2af9bfef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6199bff5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba9bffc, 78 },
    { 0x5f29bffd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3969c002, 95 },
    { 0, 0 },
    { 0xdd9c004, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad9c00a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149c010, 67 },
    { 0x9b89c011, 93 },
    { 0, 0 },
    { 0x2b69c013, 91 },
    { 0x97f9c014, 61 },
    { 0, 0 },
    { 0x54d9c016, 95 },
    { 0, 0 },
    { 0xb179c018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c89c022, 61 },
    { 0x3b49c023, 17 },
    { 0x1c39c022, 33 },
    { 0, 0 },
    { 0x8df9c026, 55 },
    { 0x6e59c027, 90 },
    { 0, 0 },
    { 0x4069c029, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b59c02c, 23 },
    { 0, 0 },
    { 0x14c9c02e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a9c033, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee9c03a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1629c040, 61 },
    { 0x5d29c041, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329c04a, 34 },
    { 0x2f89c04b, 90 },
    { 0x2509c04c, 95 },
    { 0, 0 },
    { 0x2b39c04e, 90 },
    { 0, 0 },
    { 0x8ed9c050, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf59c053, 68 },
    { 0x8a09c054, 98 },
    { 0, 0 },
    { 0xac09c056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09c05d, 46 },
    { 0x6ba9c05d, 93 },
    { 0x4299c05f, 61 },
    { 0xea49c060, 98 },
    { 0xbcc9c05f, 98 },
    { 0, 0 },
    { 0x15e9c063, 93 },
    { 0x3d59c064, 78 },
    { 0xa2d9c065, 33 },
    { 0, 0 },
    { 0x8bc9c067, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c9c06a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6279c06d, 81 },
    { 0xb549c06e, 33 },
    { 0xdd99c06e, 90 },
    { 0xb6e9c06f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f89c075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8539c079, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c9c07d, 82 },
    { 0xab19c07e, 90 },
    { 0, 0 },
    { 0xbc79c080, 33 },
    { 0x7dd9c080, 80 },
    { 0x8939c082, 46 },
    { 0xb719c083, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbec9c08d, 55 },
    { 0x6609c08e, 85 },
    { 0x88f9c08e, 88 },
    { 0x99d9c090, 61 },
    { 0x8e89c090, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df9c098, 23 },
    { 0x23e9c098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f9c09d, 61 },
    { 0, 0 },
    { 0x2029c09f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439c0a2, 55 },
    { 0x1c09c0a2, 33 },
    { 0x95c9c0a3, 101 },
    { 0, 0 },
    { 0xa249c0a6, 61 },
    { 0xe089c0a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed29c0ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b19c0b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2029c0b5, 34 },
    { 0, 0 },
    { 0x46d9c0b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff59c0be, 33 },
    { 0, 0 },
    { 0xbd19c0c0, 95 },
    { 0xcaf9c0c0, 33 },
    { 0xb0e9c0c2, 61 },
    { 0x29d9c0c3, 23 },
    { 0xab19c0c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a9c0c7, 61 },
    { 0, 0 },
    { 0x9b89c0c9, 78 },
    { 0, 0 },
    { 0x4569c0cb, 67 },
    { 0xc0b9c0cc, 33 },
    { 0xf479c0cd, 61 },
    { 0x7319c0cd, 61 },
    { 0x3039c0cf, 76 },
    { 0x9c49c0cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d9c0d3, 33 },
    { 0xda79c0d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ee9c0d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb9c0dc, 55 },
    { 0x6489c0dc, 85 },
    { 0x70a9c0de, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc29c0e3, 100 },
    { 0x96c9c0e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c09c0ef, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459c0f2, 98 },
    { 0xbf99c0f3, 90 },
    { 0xe629c0f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee9c0f9, 55 },
    { 0xd939c0fa, 46 },
    { 0xb8c9c0fb, 90 },
    { 0x7459c0fc, 55 },
    { 0x6dd9c0fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc189c105, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce9c10a, 55 },
    { 0, 0 },
    { 0x4e89c10c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7009c10f, 98 },
    { 0, 0 },
    { 0xdc89c111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf169c114, 76 },
    { 0, 0 },
    { 0x4779c116, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a9c11b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc9c122, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9959c125, 93 },
    { 0x5799c125, 76 },
    { 0x4139c127, 52 },
    { 0x55c9c128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c49c12d, 61 },
    { 0xca39c12e, 95 },
    { 0xdd9c12f, 66 },
    { 0x4b29c12e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b9c138, 64 },
    { 0, 0 },
    { 0xbd59c13a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479c13e, 46 },
    { 0x25b9c13f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd9c145, 34 },
    { 0xbd59c146, 90 },
    { 0x8069c146, 43 },
    { 0xce79c145, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6b9c14c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9c153, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd139c15a, 67 },
    { 0xe799c15b, 90 },
    { 0x789c15c, 90 },
    { 0x93d9c15d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea39c161, 85 },
    { 0x3599c161, 55 },
    { 0xd709c163, 55 },
    { 0x9d89c162, 99 },
    { 0xfb99c165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b9c169, 43 },
    { 0x2c19c16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429c16e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd89c171, 61 },
    { 0xb019c171, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x549c175, 61 },
    { 0x6149c175, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c179, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f39c180, 85 },
    { 0xce39c180, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c9c189, 33 },
    { 0x5049c189, 100 },
    { 0x3809c189, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7649c18e, 17 },
    { 0xcf99c18f, 95 },
    { 0xc799c190, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19c19c, 95 },
    { 0xe679c19c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f99c1a6, 76 },
    { 0, 0 },
    { 0x7739c1a8, 23 },
    { 0, 0 },
    { 0xb2b9c1aa, 90 },
    { 0x44e9c1ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b9c1af, 61 },
    { 0x8bd9c1b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3769c1be, 61 },
    { 0xc539c1bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb69c1c7, 93 },
    { 0, 0 },
    { 0xa319c1c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8339c1cc, 98 },
    { 0x40b9c1cd, 85 },
    { 0xc929c1cd, 95 },
    { 0x79f9c1ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa039c1d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd739c1da, 100 },
    { 0xae79c1db, 33 },
    { 0, 0 },
    { 0x2b29c1dd, 90 },
    { 0x6669c1de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa999c1e3, 33 },
    { 0x9a79c1e3, 98 },
    { 0xffd9c1e4, 61 },
    { 0x9099c1e3, 76 },
    { 0xb739c1e7, 90 },
    { 0xe39c1e4, 76 },
    { 0xb479c1e9, 17 },
    { 0xdc39c1e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe519c1f1, 23 },
    { 0x10f9c1f2, 85 },
    { 0xf979c1f1, 46 },
    { 0x3789c1f3, 98 },
    { 0xa6e9c1f5, 23 },
    { 0xc89c1f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca9c1fb, 78 },
    { 0x1559c1fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb9c200, 61 },
    { 0x6889c201, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf069c206, 61 },
    { 0x9309c206, 95 },
    { 0x2719c208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad9c20b, 33 },
    { 0, 0 },
    { 0x4319c20d, 78 },
    { 0xcac9c20d, 46 },
    { 0xafe9c20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db9c212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e9c215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf749c219, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9949c21d, 23 },
    { 0x42c9c21e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f9c222, 55 },
    { 0x1239c222, 61 },
    { 0x7579c222, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9c229, 98 },
    { 0, 0 },
    { 0xada9c22b, 98 },
    { 0xf009c22c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ca9c22f, 90 },
    { 0xf4d9c230, 90 },
    { 0xc089c22f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f09c236, 23 },
    { 0xe969c237, 36 },
    { 0, 0 },
    { 0xbdf9c239, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8529c23e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9569c241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd79c244, 33 },
    { 0xcc59c244, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9c248, 95 },
    { 0x689c249, 85 },
    { 0, 0 },
    { 0x2819c24b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2689c24e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b9c251, 23 },
    { 0x6209c251, 61 },
    { 0xa749c253, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c9c257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c59c25f, 76 },
    { 0, 0 },
    { 0xcca9c261, 23 },
    { 0xc4a9c262, 95 },
    { 0x9669c263, 33 },
    { 0, 0 },
    { 0x5fd9c265, 98 },
    { 0xa239c265, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fd9c269, 98 },
    { 0x3a59c26a, 84 },
    { 0xbf39c26b, 61 },
    { 0xf9b9c26b, 61 },
    { 0xa0e9c26c, 95 },
    { 0, 0 },
    { 0xfef9c26f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3b9c273, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e9c276, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7909c27a, 77 },
    { 0, 0 },
    { 0x9a39c27c, 55 },
    { 0x3b9c27d, 91 },
    { 0, 0 },
    { 0xc0b9c27f, 68 },
    { 0xb889c27f, 23 },
    { 0xbdc9c281, 55 },
    { 0x3809c282, 95 },
    { 0, 0 },
    { 0x5809c284, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda19c28a, 90 },
    { 0x4389c28b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3b9c290, 55 },
    { 0x1829c291, 90 },
    { 0xca9c292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8179c298, 98 },
    { 0xf479c299, 67 },
    { 0xf969c29a, 43 },
    { 0xebb9c29b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca99c2a0, 90 },
    { 0x9b69c2a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d9c2a5, 55 },
    { 0x9059c2a6, 65 },
    { 0, 0 },
    { 0x1b79c2a8, 93 },
    { 0, 0 },
    { 0x1719c2aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d49c2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa269c2b8, 81 },
    { 0x5e59c2b9, 34 },
    { 0xfcd9c2b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3369c2bf, 88 },
    { 0xc4c9c2bf, 84 },
    { 0, 0 },
    { 0x4129c2c2, 55 },
    { 0x8a49c2c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4589c2cb, 98 },
    { 0, 0 },
    { 0xb239c2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa89c2d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a39c2d6, 52 },
    { 0x68d9c2d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b9c2dc, 33 },
    { 0x7849c2dd, 76 },
    { 0xfa09c2dc, 24 },
    { 0, 0 },
    { 0xfdb9c2e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9c2e3, 23 },
    { 0x4449c2e4, 28 },
    { 0x15d9c2e3, 90 },
    { 0, 0 },
    { 0xc639c2e7, 84 },
    { 0x8809c2e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa299c2eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339c2ef, 98 },
    { 0xb239c2f0, 43 },
    { 0x5789c2f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba9c2f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f09c2f8, 66 },
    { 0x3729c2f8, 33 },
    { 0x2179c2f9, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x8039c2fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3889c301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a79c305, 95 },
    { 0, 0 },
    { 0x1629c307, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d9c30a, 90 },
    { 0x649c30a, 43 },
    { 0xec89c30c, 77 },
    { 0x41a9c30d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b9c315, 43 },
    { 0x61c9c316, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4a9c31e, 100 },
    { 0x9119c31f, 98 },
    { 0x1c99c31f, 90 },
    { 0, 0 },
    { 0xd769c322, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x759c325, 87 },
    { 0x7d89c326, 77 },
    { 0xfc19c327, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b9c330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c9c333, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e39c338, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc019c33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde79c33f, 98 },
    { 0xc859c340, 91 },
    { 0xc6a9c340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4699c344, 65 },
    { 0, 0 },
    { 0xc1f9c346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe769c34e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539c356, 90 },
    { 0xefd9c357, 75 },
    { 0xe2b9c358, 61 },
    { 0xcea9c359, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf709c35d, 33 },
    { 0, 0 },
    { 0x5029c35f, 90 },
    { 0x26b9c360, 33 },
    { 0x48a9c361, 66 },
    { 0x9409c362, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9c366, 87 },
    { 0xc9b9c367, 46 },
    { 0x9709c367, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa349c36b, 85 },
    { 0x31f9c36c, 87 },
    { 0x59b9c36c, 91 },
    { 0xe0c9c36e, 34 },
    { 0x6759c36f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff59c375, 91 },
    { 0, 0 },
    { 0x54f9c377, 90 },
    { 0x3c39c378, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9309c380, 88 },
    { 0x3bd9c381, 33 },
    { 0xb59c382, 55 },
    { 0x7709c383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc299c387, 95 },
    { 0xbca9c387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd669c38b, 34 },
    { 0x5169c38c, 85 },
    { 0xa689c38b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1739c3a0, 98 },
    { 0, 0 },
    { 0xfb29c3a2, 90 },
    { 0xa109c3a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c49c3a6, 43 },
    { 0xda59c3a7, 33 },
    { 0x8529c3a6, 98 },
    { 0, 0 },
    { 0xb259c3aa, 98 },
    { 0, 0 },
    { 0x9899c3ac, 46 },
    { 0xd389c3ad, 100 },
    { 0x9d59c3ae, 33 },
    { 0xbbe9c3ae, 23 },
    { 0xa919c3af, 98 },
    { 0xd9c3ae, 23 },
    { 0xcad9c3ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81f9c3b6, 90 },
    { 0, 0 },
    { 0xc469c3b8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f9c3bb, 76 },
    { 0x6199c3bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc29c3bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e9c3c2, 99 },
    { 0x2d49c3c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89c3c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee9c3cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6409c3cf, 61 },
    { 0, 0 },
    { 0xebb9c3d1, 98 },
    { 0xdc79c3d1, 90 },
    { 0, 0 },
    { 0x9179c3d4, 61 },
    { 0, 0 },
    { 0x9089c3d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c69c3dc, 33 },
    { 0x7979c3dd, 23 },
    { 0x629c3de, 95 },
    { 0x90c9c3df, 99 },
    { 0, 0 },
    { 0x5e39c3e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e9c3e4, 43 },
    { 0x2899c3e5, 23 },
    { 0, 0 },
    { 0xbcc9c3e7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9119c3ea, 34 },
    { 0xaa09c3ea, 55 },
    { 0x2029c3ec, 33 },
    { 0xa3b9c3ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f9c3ff, 98 },
    { 0x8c59c400, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5419c403, 23 },
    { 0, 0 },
    { 0x9459c405, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc9c40c, 33 },
    { 0, 0 },
    { 0x7389c40e, 87 },
    { 0xf119c40e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc609c414, 90 },
    { 0, 0 },
    { 0xbea9c416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49c9c41b, 33 },
    { 0xf159c41c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639c420, 91 },
    { 0xa329c421, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4089c424, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319c42b, 36 },
    { 0xb559c42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c79c431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bc9c434, 95 },
    { 0, 0 },
    { 0x33b9c436, 93 },
    { 0x9b69c437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa9c43b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9c43e, 76 },
    { 0x92a9c43e, 100 },
    { 0x5c39c440, 33 },
    { 0x9799c440, 66 },
    { 0x33d9c442, 55 },
    { 0x5129c443, 23 },
    { 0xc089c443, 34 },
    { 0xf899c442, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e9c448, 76 },
    { 0, 0 },
    { 0x23d9c44a, 52 },
    { 0, 0 },
    { 0x1a99c44c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f9c44f, 23 },
    { 0x4ec9c450, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359c454, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec9c459, 33 },
    { 0x8019c45a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e9c460, 98 },
    { 0, 0 },
    { 0xe39c462, 52 },
    { 0, 0 },
    { 0x5f39c464, 43 },
    { 0xd739c465, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c9c46b, 76 },
    { 0x3869c46c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8909c46f, 33 },
    { 0xb5b9c470, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1059c473, 67 },
    { 0x9039c474, 77 },
    { 0x76c9c475, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d9c478, 95 },
    { 0x5419c479, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b9c486, 90 },
    { 0, 0 },
    { 0xdff9c488, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb419c48e, 90 },
    { 0x2d79c48f, 66 },
    { 0, 0 },
    { 0x77c9c491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf639c499, 90 },
    { 0xcae9c499, 93 },
    { 0x4bd9c49b, 90 },
    { 0x35f9c49c, 36 },
    { 0, 0 },
    { 0x5d49c49e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8029c4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29c4a5, 34 },
    { 0xe1e9c4a6, 95 },
    { 0x3ed9c4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669c4aa, 78 },
    { 0, 0 },
    { 0xbcb9c4ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x9019c4af, 95 },
    { 0x4559c4b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef9c4b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c9c4b7, 34 },
    { 0, 0 },
    { 0x5189c4b9, 5 },
    { 0x5ad9c4ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd209c4c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f9c4c4, 34 },
    { 0, 0 },
    { 0x16b9c4c6, 43 },
    { 0xa769c4c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a29c4ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc9c4cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc89c4d3, 90 },
    { 0x1159c4d3, 67 },
    { 0xfb89c4d5, 90 },
    { 0x4da9c4d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8949c4dc, 43 },
    { 0, 0 },
    { 0xa4b9c4de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8249c4e3, 34 },
    { 0, 0 },
    { 0x669c4e5, 61 },
    { 0x6fc9c4e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdf9c4e9, 95 },
    { 0xc179c4ea, 98 },
    { 0x25b9c4eb, 76 },
    { 0x2d69c4ec, 76 },
    { 0x4c59c4ed, 81 },
    { 0xf549c4ee, 61 },
    { 0, 0 },
    { 0x6959c4f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8279c4f6, 33 },
    { 0x20e9c4f7, 61 },
    { 0x41d9c4f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a89c502, 99 },
    { 0xc199c503, 67 },
    { 0, 0 },
    { 0x4639c505, 23 },
    { 0xc879c506, 23 },
    { 0xbe89c507, 90 },
    { 0xb8f9c505, 43 },
    { 0, 0 },
    { 0x1ec9c50a, 61 },
    { 0xa929c50a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb729c50e, 23 },
    { 0, 0 },
    { 0xc8d9c510, 61 },
    { 0, 0 },
    { 0x8099c512, 33 },
    { 0x2e9c512, 100 },
    { 0, 0 },
    { 0x36f9c515, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9409c519, 91 },
    { 0x2be9c51a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139c51f, 61 },
    { 0xb6e9c520, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9c523, 61 },
    { 0x4da9c524, 65 },
    { 0xd579c524, 90 },
    { 0x6dc9c526, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd29c52c, 23 },
    { 0xa359c52d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f9c530, 61 },
    { 0xaa69c531, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd09c536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a39c53a, 61 },
    { 0x23d9c53b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad89c53f, 90 },
    { 0x35c9c540, 76 },
    { 0x31b9c541, 90 },
    { 0xad29c540, 95 },
    { 0x4299c543, 33 },
    { 0x8449c544, 68 },
    { 0xb259c545, 95 },
    { 0xfff9c545, 95 },
    { 0, 0 },
    { 0x589c548, 23 },
    { 0, 0 },
    { 0xcf99c54a, 55 },
    { 0, 0 },
    { 0x989c54c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d19c555, 93 },
    { 0, 0 },
    { 0x7709c557, 77 },
    { 0x2be9c558, 61 },
    { 0, 0 },
    { 0x8059c55a, 61 },
    { 0x3499c55a, 90 },
    { 0x93b9c55c, 90 },
    { 0x5329c55a, 43 },
    { 0, 0 },
    { 0xe3c9c55f, 95 },
    { 0x3339c560, 47 },
    { 0xb269c561, 91 },
    { 0x5a99c561, 46 },
    { 0x31d9c560, 23 },
    { 0x439c564, 97 },
    { 0, 0 },
    { 0x42b9c566, 93 },
    { 0xee39c567, 68 },
    { 0xd6e9c568, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5749c56b, 23 },
    { 0xaab9c56c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4f9c575, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e29c578, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd589c57b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b9c585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd29c58c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfb9c590, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb89c596, 98 },
    { 0xd659c596, 65 },
    { 0, 0 },
    { 0xc769c599, 34 },
    { 0x3b69c59a, 46 },
    { 0xbdb9c59a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb529c5a1, 34 },
    { 0xe7d9c5a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9c5a6, 95 },
    { 0, 0 },
    { 0xbec9c5a8, 61 },
    { 0x8499c5a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa149c5ac, 23 },
    { 0x32a9c5ad, 61 },
    { 0xd429c5ae, 46 },
    { 0xd5e9c5ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1289c5b4, 95 },
    { 0xf319c5b5, 33 },
    { 0x7789c5b6, 33 },
    { 0, 0 },
    { 0xc209c5b8, 90 },
    { 0x5f59c5b9, 90 },
    { 0xe419c5b9, 61 },
    { 0xcaa9c5bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c59c5be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e9c5c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4499c5c7, 43 },
    { 0x6e39c5c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d89c5ce, 33 },
    { 0x1d59c5cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae9c5d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9139c5da, 90 },
    { 0, 0 },
    { 0x79b9c5dc, 95 },
    { 0x1339c5dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d9c5e1, 76 },
    { 0x2bc9c5e1, 95 },
    { 0xc279c5e3, 95 },
    { 0xa429c5e4, 43 },
    { 0x6a69c5e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8109c5ea, 43 },
    { 0x6ab9c5ea, 55 },
    { 0, 0 },
    { 0xed19c5ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c5f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x759c5f3, 78 },
    { 0xdbf9c5f4, 99 },
    { 0x9c9c5f5, 88 },
    { 0x8179c5f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9c9c5fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339c5fe, 95 },
    { 0, 0 },
    { 0x8299c600, 55 },
    { 0, 0 },
    { 0xbcb9c602, 34 },
    { 0x7c19c603, 43 },
    { 0xa2c9c604, 66 },
    { 0, 0 },
    { 0xe4d9c606, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd9c60b, 40 },
    { 0xe7f9c60c, 33 },
    { 0xbf79c60d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4629c612, 55 },
    { 0x67c9c613, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9c617, 76 },
    { 0x4be9c618, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9c61b, 94 },
    { 0x59e9c61c, 52 },
    { 0xe109c61d, 90 },
    { 0xbfe9c61b, 61 },
    { 0xa9f9c61d, 67 },
    { 0, 0 },
    { 0xdb69c621, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c59c627, 46 },
    { 0x2409c627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac69c62d, 43 },
    { 0x10a9c62d, 98 },
    { 0xf4c9c62e, 90 },
    { 0, 0 },
    { 0x9919c631, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0e9c636, 76 },
    { 0, 0 },
    { 0x9849c638, 90 },
    { 0, 0 },
    { 0x7f99c63a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe539c63d, 61 },
    { 0, 0 },
    { 0x85e9c63f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb879c653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba99c65c, 87 },
    { 0xd599c65c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38f9c663, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb9c666, 95 },
    { 0x6f19c667, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb779c673, 90 },
    { 0x5c19c673, 65 },
    { 0xab79c675, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e9c67b, 90 },
    { 0, 0 },
    { 0x34e9c67d, 43 },
    { 0xe419c67d, 34 },
    { 0x1059c67d, 44 },
    { 0xd3d9c680, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa639c685, 95 },
    { 0, 0 },
    { 0xd659c687, 34 },
    { 0xa9d9c687, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d9c68c, 90 },
    { 0x3019c68c, 98 },
    { 0xcaf9c68c, 34 },
    { 0, 0 },
    { 0x7b39c690, 61 },
    { 0xb719c691, 66 },
    { 0xee99c692, 23 },
    { 0xb459c693, 81 },
    { 0xf579c694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ba9c69a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe499c69e, 90 },
    { 0x7d69c69e, 98 },
    { 0x5bb9c6a0, 90 },
    { 0x5d79c69f, 43 },
    { 0x3509c6a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329c6a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9c6ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b9c6ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2779c6b1, 84 },
    { 0xa189c6b1, 90 },
    { 0xac39c6b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5569c6b8, 93 },
    { 0x1ed9c6b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7339c6bd, 78 },
    { 0x33a9c6bd, 95 },
    { 0xc929c6bf, 33 },
    { 0x7c49c6be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669c6c7, 98 },
    { 0xe0a9c6c8, 23 },
    { 0xe9e9c6c7, 90 },
    { 0xe139c6ca, 76 },
    { 0xbc9c6c7, 61 },
    { 0xe369c6cb, 61 },
    { 0x3c39c6ca, 55 },
    { 0x83c9c6cb, 55 },
    { 0x979c6ce, 95 },
    { 0, 0 },
    { 0xc89c6d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9c9c6d7, 95 },
    { 0x7f29c6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949c6dd, 67 },
    { 0xd2c9c6de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5479c6e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639c6e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c79c6ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b9c6f0, 100 },
    { 0xe609c6f1, 90 },
    { 0, 0 },
    { 0xd6a9c6f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c9c6f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b9c6fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd879c6fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c09c701, 90 },
    { 0xa899c702, 33 },
    { 0, 0 },
    { 0x2ce9c704, 99 },
    { 0x32c9c705, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa9c70d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf619c710, 90 },
    { 0, 0 },
    { 0x7f69c712, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe029c718, 61 },
    { 0, 0 },
    { 0x1af9c71a, 90 },
    { 0x25e9c71b, 55 },
    { 0x9c59c71b, 95 },
    { 0x6989c71a, 43 },
    { 0xb859c71e, 98 },
    { 0x5869c71e, 90 },
    { 0x7549c71f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d19c726, 76 },
    { 0, 0 },
    { 0xcb49c728, 98 },
    { 0, 0 },
    { 0x2d19c72a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d9c72e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d9c733, 55 },
    { 0x6dd9c733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd09c737, 61 },
    { 0xc0f9c738, 46 },
    { 0, 0 },
    { 0xa399c73a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f9c73d, 93 },
    { 0xc669c73d, 95 },
    { 0xf009c73f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff9c743, 95 },
    { 0xa79c743, 98 },
    { 0xfc29c745, 61 },
    { 0xe349c745, 61 },
    { 0xf799c745, 90 },
    { 0xb1f9c746, 34 },
    { 0, 0 },
    { 0xbb99c74a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29c74e, 90 },
    { 0xee09c74f, 90 },
    { 0x8089c750, 90 },
    { 0xca9c751, 82 },
    { 0xabd9c751, 90 },
    { 0x2d9c752, 90 },
    { 0x5449c752, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209c757, 95 },
    { 0, 0 },
    { 0x9799c759, 75 },
    { 0x9a09c75a, 34 },
    { 0xc3b9c75a, 99 },
    { 0xe729c75c, 90 },
    { 0x2019c759, 33 },
    { 0xce79c75e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c89c763, 61 },
    { 0x28f9c764, 23 },
    { 0x8159c764, 93 },
    { 0, 0 },
    { 0x8c19c767, 95 },
    { 0, 0 },
    { 0x9d69c769, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e9c771, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289c778, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2789c77c, 90 },
    { 0, 0 },
    { 0x2f9c77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c9c784, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949c787, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb9c78a, 83 },
    { 0, 0 },
    { 0xa8a9c78c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc09c78f, 90 },
    { 0, 0 },
    { 0x5649c791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e9c794, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc89c798, 85 },
    { 0x4109c799, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef9c79c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd19c7a0, 98 },
    { 0x8ec9c7a1, 55 },
    { 0, 0 },
    { 0x1bb9c7a3, 33 },
    { 0xc989c7a3, 98 },
    { 0x1c19c7a4, 40 },
    { 0xc689c7a4, 99 },
    { 0xe79c7a6, 69 },
    { 0, 0 },
    { 0xce19c7a9, 67 },
    { 0, 0 },
    { 0x4a69c7ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef9c7ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f99c7b4, 99 },
    { 0, 0 },
    { 0xf59c7b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c9c7bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd029c7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c9c7cb, 52 },
    { 0, 0 },
    { 0xef79c7cd, 98 },
    { 0xe979c7ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4599c7d4, 33 },
    { 0x8669c7d5, 33 },
    { 0, 0 },
    { 0x389c7d7, 95 },
    { 0xa0f9c7d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed99c7df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab9c7e2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb9c7e5, 33 },
    { 0xabf9c7e6, 93 },
    { 0x46f9c7e6, 90 },
    { 0xfe39c7e7, 33 },
    { 0xa419c7e9, 23 },
    { 0x4349c7e8, 61 },
    { 0xed9c7eb, 66 },
    { 0, 0 },
    { 0xacf9c7ed, 91 },
    { 0x2b49c7ed, 95 },
    { 0xb0a9c7ef, 99 },
    { 0xd739c7ef, 91 },
    { 0, 0 },
    { 0x2b09c7f2, 95 },
    { 0, 0 },
    { 0xe1f9c7f4, 91 },
    { 0, 0 },
    { 0x29e9c7f6, 33 },
    { 0xaa99c7f6, 93 },
    { 0x27e9c7f7, 93 },
    { 0x7269c7f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb29c7fe, 93 },
    { 0xd769c7fe, 33 },
    { 0x3d99c7fe, 61 },
    { 0, 0 },
    { 0x8e99c802, 61 },
    { 0x5499c802, 65 },
    { 0x6789c804, 76 },
    { 0x1809c805, 68 },
    { 0x5f79c805, 90 },
    { 0x51d9c804, 33 },
    { 0xbc09c806, 98 },
    { 0x6199c809, 98 },
    { 0x7609c809, 90 },
    { 0xd049c807, 33 },
    { 0x1649c80c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46d9c80f, 91 },
    { 0x8859c810, 76 },
    { 0, 0 },
    { 0x4819c812, 76 },
    { 0, 0 },
    { 0xc089c814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xade9c817, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4299c81b, 90 },
    { 0xd2d9c81c, 95 },
    { 0xea69c81d, 66 },
    { 0, 0 },
    { 0xd2c9c81f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e19c822, 96 },
    { 0xb089c822, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd29c82a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5269c82f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a79c832, 23 },
    { 0x34d9c833, 46 },
    { 0x2119c834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b9c83a, 98 },
    { 0, 0 },
    { 0xbfa9c83c, 98 },
    { 0x299c83d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf799c842, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b9c84b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c9c84f, 23 },
    { 0x91b9c84f, 76 },
    { 0, 0 },
    { 0x9529c852, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429c85b, 34 },
    { 0, 0 },
    { 0xb09c85d, 23 },
    { 0xf5c9c85e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c9c862, 46 },
    { 0x6169c863, 98 },
    { 0xb1e9c864, 66 },
    { 0xe729c865, 61 },
    { 0x95d9c866, 85 },
    { 0, 0 },
    { 0x77e9c868, 46 },
    { 0xfbf9c868, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf529c86c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd9c86f, 33 },
    { 0xa289c86f, 65 },
    { 0x1859c871, 55 },
    { 0x17b9c871, 90 },
    { 0xbc89c873, 84 },
    { 0xcc69c873, 78 },
    { 0xe959c875, 98 },
    { 0x7db9c871, 90 },
    { 0x39c9c871, 88 },
    { 0x9d59c873, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3939c87c, 98 },
    { 0, 0 },
    { 0x91f9c87e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1129c882, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2889c885, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09c889, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9c88d, 77 },
    { 0xd779c88e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe999c893, 34 },
    { 0x5ac9c893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a9c89b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc39c89f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf159c8a7, 90 },
    { 0x2299c8a7, 75 },
    { 0, 0 },
    { 0x149c8aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e9c8af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff9c8b3, 98 },
    { 0x1e79c8b4, 98 },
    { 0x5839c8b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3299c8c0, 34 },
    { 0x6499c8c0, 61 },
    { 0x35c9c8c2, 93 },
    { 0x55a9c8c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda99c8c7, 99 },
    { 0xa39c8c8, 46 },
    { 0x589c8c9, 66 },
    { 0x2029c8c9, 55 },
    { 0x9a9c8c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f09c8d0, 90 },
    { 0x1bd9c8d1, 77 },
    { 0xe509c8d1, 23 },
    { 0x809c8d3, 98 },
    { 0x9889c8d1, 95 },
    { 0x6f59c8d3, 90 },
    { 0x8769c8d1, 95 },
    { 0x8f99c8d3, 77 },
    { 0, 0 },
    { 0x4d39c8d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf009c8df, 95 },
    { 0x32c9c8df, 61 },
    { 0x97e9c8e1, 67 },
    { 0x7079c8e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3d9c8e7, 99 },
    { 0x48f9c8e8, 78 },
    { 0xea59c8e7, 76 },
    { 0xedc9c8ea, 43 },
    { 0xead9c8ea, 68 },
    { 0xe659c8ec, 33 },
    { 0, 0 },
    { 0xf269c8ee, 93 },
    { 0x56c9c8ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a9c8f6, 43 },
    { 0x7cc9c8f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f9c8fe, 90 },
    { 0xc879c8ff, 23 },
    { 0x3af9c900, 95 },
    { 0, 0 },
    { 0x7169c902, 90 },
    { 0x9c39c902, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8049c90a, 90 },
    { 0x7009c90b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c49c90e, 38 },
    { 0x5859c90f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e9c91e, 65 },
    { 0xc3c9c91f, 61 },
    { 0, 0 },
    { 0x6e49c921, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf9c924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f9c927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d69c92a, 91 },
    { 0xc679c92b, 33 },
    { 0x4189c92c, 95 },
    { 0, 0 },
    { 0xfe69c92e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fc9c936, 94 },
    { 0xae59c937, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d09c93d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc109c940, 90 },
    { 0, 0 },
    { 0x3f39c942, 75 },
    { 0, 0 },
    { 0x1c99c944, 76 },
    { 0x4b9c944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a9c949, 76 },
    { 0x1639c94a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc639c950, 33 },
    { 0x2a39c951, 33 },
    { 0x2159c951, 99 },
    { 0xfac9c952, 90 },
    { 0, 0 },
    { 0xece9c955, 81 },
    { 0x3e09c955, 87 },
    { 0x4fb9c957, 33 },
    { 0xccb9c957, 43 },
    { 0x9da9c959, 61 },
    { 0x6179c958, 33 },
    { 0xa799c959, 61 },
    { 0, 0 },
    { 0x9a09c95d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55d9c961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b9c964, 33 },
    { 0x3e09c964, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7879c969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab9c96d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7329c973, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9c97c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1789c983, 61 },
    { 0x1629c984, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6939c989, 23 },
    { 0x6d49c98a, 90 },
    { 0, 0 },
    { 0x419c98c, 90 },
    { 0x83b9c98d, 98 },
    { 0x5049c98e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3269c992, 80 },
    { 0xe609c993, 61 },
    { 0x539c994, 33 },
    { 0x6049c995, 90 },
    { 0x4df9c996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa9c999, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b59c9a3, 98 },
    { 0xf999c9a3, 66 },
    { 0xd909c9a5, 98 },
    { 0x8439c9a6, 33 },
    { 0x9809c9a7, 85 },
    { 0, 0 },
    { 0xfe09c9a9, 90 },
    { 0, 0 },
    { 0x3cb9c9ab, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc9c9ae, 61 },
    { 0x5199c9af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dc9c9b6, 87 },
    { 0xd4d9c9b7, 23 },
    { 0, 0 },
    { 0x5289c9b9, 55 },
    { 0x9ac9c9ba, 17 },
    { 0x4169c9bb, 76 },
    { 0x6569c9bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f49c9bf, 65 },
    { 0x2cf9c9c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec9c9c7, 76 },
    { 0x7629c9c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77b9c9cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa459c9d0, 99 },
    { 0xca29c9d0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5279c9d5, 99 },
    { 0, 0 },
    { 0x4199c9d7, 95 },
    { 0, 0 },
    { 0xfb89c9d9, 34 },
    { 0x3259c9da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9c9dd, 90 },
    { 0x799c9de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe939c9e2, 46 },
    { 0xc6e9c9e3, 43 },
    { 0, 0 },
    { 0xcda9c9e5, 68 },
    { 0xce89c9e6, 75 },
    { 0x3559c9e5, 33 },
    { 0xa959c9e6, 17 },
    { 0xb159c9e8, 33 },
    { 0xfe59c9ea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7529c9ed, 23 },
    { 0x1dc9c9ed, 85 },
    { 0, 0 },
    { 0xc1e9c9f0, 90 },
    { 0xcbe9c9f1, 95 },
    { 0xba9c9f0, 61 },
    { 0x9e59c9f0, 23 },
    { 0xc59c9f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd89c9f7, 46 },
    { 0xf879c9f7, 78 },
    { 0, 0 },
    { 0x8449c9fa, 80 },
    { 0, 0 },
    { 0x8d59c9fc, 76 },
    { 0x4ad9c9fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a89ca05, 95 },
    { 0, 0 },
    { 0x3b49ca07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3789ca0d, 23 },
    { 0, 0 },
    { 0xafb9ca0f, 61 },
    { 0xe059ca0f, 77 },
    { 0x7539ca11, 23 },
    { 0x3af9ca11, 95 },
    { 0xe199ca12, 98 },
    { 0, 0 },
    { 0xc6c9ca15, 93 },
    { 0, 0 },
    { 0x7af9ca17, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9ca1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2609ca24, 91 },
    { 0x6129ca25, 59 },
    { 0, 0 },
    { 0x5f59ca27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99ca2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e9ca33, 98 },
    { 0x6d49ca34, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb619ca3a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569ca40, 66 },
    { 0x2319ca41, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb569ca48, 61 },
    { 0, 0 },
    { 0x7b49ca4a, 98 },
    { 0x9e19ca4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96f9ca4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b09ca52, 43 },
    { 0x8809ca52, 43 },
    { 0, 0 },
    { 0x7889ca55, 8 },
    { 0, 0 },
    { 0x3cb9ca57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b29ca5a, 95 },
    { 0x9029ca5b, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e89ca5f, 43 },
    { 0, 0 },
    { 0x98d9ca61, 90 },
    { 0xe419ca61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d9ca67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7209ca6b, 67 },
    { 0xce29ca6c, 33 },
    { 0xb349ca6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7649ca74, 78 },
    { 0x18f9ca75, 56 },
    { 0x8a09ca75, 95 },
    { 0, 0 },
    { 0x1a59ca78, 95 },
    { 0x80a9ca78, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d59ca7d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d9ca81, 76 },
    { 0xfbf9ca81, 33 },
    { 0x3419ca83, 33 },
    { 0x1609ca82, 95 },
    { 0xa4e9ca85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5029ca88, 84 },
    { 0x33e9ca88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d9ca93, 33 },
    { 0x17f9ca94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe229ca98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59ca9f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9caa4, 33 },
    { 0x5b59caa5, 65 },
    { 0xa129caa4, 61 },
    { 0xc679caa7, 98 },
    { 0xd629caa5, 61 },
    { 0x76e9caa8, 95 },
    { 0xe19caaa, 23 },
    { 0x73a9caa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c9cab0, 90 },
    { 0x4fa9cab1, 33 },
    { 0x4359cab1, 98 },
    { 0, 0 },
    { 0x50d9cab4, 61 },
    { 0, 0 },
    { 0x349cab6, 23 },
    { 0x11d9cab7, 61 },
    { 0xcbb9cab8, 46 },
    { 0x3469cab9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8629cac3, 65 },
    { 0xe399cac4, 94 },
    { 0x9ad9cac5, 23 },
    { 0x33b9cac4, 76 },
    { 0x8c9cac4, 33 },
    { 0, 0 },
    { 0x5af9cac9, 66 },
    { 0x3ef9caca, 17 },
    { 0x1c19caca, 91 },
    { 0, 0 },
    { 0xc2c9cacd, 23 },
    { 0x3209cace, 100 },
    { 0xab19cacf, 95 },
    { 0x4c99cad0, 76 },
    { 0x36c9cace, 90 },
    { 0xcd79cace, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3389cad5, 100 },
    { 0xd6b9cad6, 90 },
    { 0, 0 },
    { 0x8699cad8, 81 },
    { 0xe4a9cad8, 66 },
    { 0, 0 },
    { 0xe459cadb, 76 },
    { 0xbc79cadc, 95 },
    { 0x24a9cadd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb9cae1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb489cae4, 33 },
    { 0x2629cae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba99caeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4679caee, 44 },
    { 0x4639caee, 90 },
    { 0, 0 },
    { 0x8839caf1, 23 },
    { 0, 0 },
    { 0x4b09caf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14f9caf8, 33 },
    { 0, 0 },
    { 0xc39cafa, 66 },
    { 0x70f9cafa, 91 },
    { 0xec69cafb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6319caff, 17 },
    { 0, 0 },
    { 0x1f29cb01, 33 },
    { 0x7cb9cb01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e9cb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4569cb09, 46 },
    { 0xf899cb0a, 66 },
    { 0, 0 },
    { 0xc369cb0c, 90 },
    { 0x4149cb0c, 84 },
    { 0x50a9cb0e, 95 },
    { 0xb839cb0f, 96 },
    { 0x5a59cb10, 95 },
    { 0x2fc9cb10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb769cb1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf09cb2e, 61 },
    { 0x2289cb2f, 98 },
    { 0xa879cb2e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae9cb38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb649cb3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c9cb41, 55 },
    { 0, 0 },
    { 0x8029cb43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9cb46, 98 },
    { 0, 0 },
    { 0xac69cb48, 61 },
    { 0xd0b9cb49, 98 },
    { 0x1989cb4a, 55 },
    { 0xaef9cb48, 33 },
    { 0x35f9cb4c, 93 },
    { 0x899cb4d, 95 },
    { 0xb1f9cb4b, 78 },
    { 0xa3f9cb4a, 23 },
    { 0xf7f9cb4a, 99 },
    { 0xc509cb4a, 93 },
    { 0x9fc9cb52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb9cb55, 61 },
    { 0x58e9cb56, 23 },
    { 0xe0c9cb57, 90 },
    { 0xfb19cb58, 95 },
    { 0xc4b9cb59, 85 },
    { 0xd4c9cb5a, 98 },
    { 0, 0 },
    { 0xc2e9cb5c, 61 },
    { 0, 0 },
    { 0x3dd9cb5e, 90 },
    { 0, 0 },
    { 0x91c9cb60, 46 },
    { 0xa139cb61, 98 },
    { 0x309cb60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29cb65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99cb6a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b29cb6e, 93 },
    { 0x739cb6e, 46 },
    { 0x77b9cb70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7479cb75, 64 },
    { 0x8819cb76, 90 },
    { 0, 0 },
    { 0xfae9cb78, 100 },
    { 0xb529cb78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc749cb7d, 81 },
    { 0x8c9cb7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe649cb86, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9a9cb8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7959cb8d, 90 },
    { 0xf439cb8e, 90 },
    { 0x9ab9cb8f, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbc9cb93, 98 },
    { 0x1409cb93, 88 },
    { 0x209cb95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa559cb9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429cb9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a19cba2, 52 },
    { 0, 0 },
    { 0xb819cba4, 90 },
    { 0, 0 },
    { 0x76d9cba6, 46 },
    { 0xcdf9cba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe599cbaa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3839cbad, 33 },
    { 0, 0 },
    { 0x4829cbaf, 85 },
    { 0, 0 },
    { 0xf899cbb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c9cbb8, 34 },
    { 0xfb19cbb9, 98 },
    { 0x79f9cbba, 84 },
    { 0xa279cbb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b99cbbe, 87 },
    { 0x7729cbbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b29cbc2, 95 },
    { 0xea99cbc3, 96 },
    { 0x9a59cbc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea9cbce, 84 },
    { 0, 0 },
    { 0x5c79cbd0, 33 },
    { 0x8b79cbd0, 55 },
    { 0, 0 },
    { 0x9e59cbd3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9cbd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639cbdc, 75 },
    { 0x7089cbdd, 34 },
    { 0xc009cbdd, 46 },
    { 0x6eb9cbdf, 34 },
    { 0xc809cbe0, 33 },
    { 0x1bb9cbdf, 90 },
    { 0x67e9cbe2, 61 },
    { 0x6769cbdc, 90 },
    { 0, 0 },
    { 0x1429cbe5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5009cbe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa979cbed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe409cbf0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xff39cbf3, 95 },
    { 0xfed9cbf4, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639cbfe, 75 },
    { 0x8229cbff, 66 },
    { 0xdd09cbff, 17 },
    { 0xf4c9cc01, 46 },
    { 0x3039cc02, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd989cc07, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7559cc10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9cc16, 23 },
    { 0, 0 },
    { 0xfe19cc18, 95 },
    { 0x2139cc18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa579cc1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a19cc26, 33 },
    { 0x6dc9cc26, 33 },
    { 0xb889cc27, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bd9cc2c, 68 },
    { 0x1289cc2d, 61 },
    { 0x2b39cc2c, 98 },
    { 0x10a9cc2f, 61 },
    { 0x9a49cc2f, 98 },
    { 0xe409cc30, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4399cc34, 98 },
    { 0x2009cc35, 61 },
    { 0x8309cc35, 99 },
    { 0xa799cc37, 76 },
    { 0, 0 },
    { 0xa4f9cc39, 52 },
    { 0xd229cc3a, 33 },
    { 0, 0 },
    { 0x8809cc3c, 33 },
    { 0x269cc3d, 76 },
    { 0xe159cc3c, 61 },
    { 0xa8d9cc3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e19cc43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca9cc47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab9cc4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf779cc51, 34 },
    { 0, 0 },
    { 0xfd59cc53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc69cc56, 95 },
    { 0xf9c9cc57, 99 },
    { 0x4a39cc58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9589cc5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb439cc62, 33 },
    { 0xe9c9cc63, 43 },
    { 0xa449cc63, 61 },
    { 0x12e9cc65, 61 },
    { 0xc849cc66, 90 },
    { 0x7bd9cc67, 91 },
    { 0x3609cc68, 33 },
    { 0x3d29cc67, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca9cc6c, 93 },
    { 0x8c59cc6d, 33 },
    { 0, 0 },
    { 0xbea9cc6f, 61 },
    { 0x2f69cc6f, 23 },
    { 0xb359cc70, 23 },
    { 0xdbe9cc72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd899cc79, 76 },
    { 0x1999cc79, 61 },
    { 0, 0 },
    { 0x4ab9cc7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a9cc80, 98 },
    { 0, 0 },
    { 0xda89cc82, 90 },
    { 0xd7d9cc83, 65 },
    { 0xb029cc84, 98 },
    { 0xe089cc83, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3c9cc8b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b19cc8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c49cc91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9cc96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5b9cc99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4579cc9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6299cc9f, 52 },
    { 0x53d9cc9f, 95 },
    { 0x2ef9cca1, 33 },
    { 0xa4a9cca2, 33 },
    { 0x3639cca3, 76 },
    { 0xe989cca3, 61 },
    { 0x4fd9cca1, 98 },
    { 0x9e49cca1, 55 },
    { 0x3929cca7, 65 },
    { 0x14c9cca8, 33 },
    { 0x3fc9cca8, 90 },
    { 0xea89cca9, 99 },
    { 0, 0 },
    { 0xf8f9ccac, 98 },
    { 0x1859ccad, 90 },
    { 0x2e69ccae, 90 },
    { 0, 0 },
    { 0xcc69ccb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5649ccb3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e19ccb6, 33 },
    { 0x1779ccb6, 61 },
    { 0x1c39ccb8, 67 },
    { 0xc899ccb7, 91 },
    { 0xc829ccb9, 87 },
    { 0xa9d9ccba, 59 },
    { 0x53b9ccb9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6579ccbf, 33 },
    { 0, 0 },
    { 0x1ef9ccc1, 91 },
    { 0xf019ccc1, 61 },
    { 0x5f79ccc3, 90 },
    { 0x6539ccc2, 25 },
    { 0x769ccc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b59ccca, 34 },
    { 0xcf79cccb, 61 },
    { 0xf8e9cccc, 61 },
    { 0x6469cccb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9ccd4, 95 },
    { 0x7d69ccd4, 90 },
    { 0, 0 },
    { 0x19ccd7, 23 },
    { 0xd2d9ccd8, 61 },
    { 0x1439ccd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859ccdf, 90 },
    { 0x4da9cce0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8329cce3, 78 },
    { 0x9169cce4, 55 },
    { 0, 0 },
    { 0xd679cce6, 100 },
    { 0xfc99cce6, 61 },
    { 0xa3c9cce8, 61 },
    { 0x4589cce8, 23 },
    { 0, 0 },
    { 0x8da9cceb, 95 },
    { 0x3829ccec, 34 },
    { 0x1ad9cced, 55 },
    { 0xf359ccee, 95 },
    { 0x6f69ccef, 33 },
    { 0, 0 },
    { 0x7479ccf1, 98 },
    { 0x2d89ccf2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec99ccf5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae9cd05, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7619cd08, 99 },
    { 0xab39cd09, 83 },
    { 0x44f9cd09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b99cd0f, 90 },
    { 0x5629cd10, 98 },
    { 0x7b59cd11, 76 },
    { 0x2fb9cd12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb879cd19, 100 },
    { 0xba09cd1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a9cd1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba9cd21, 98 },
    { 0x21f9cd22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4279cd26, 83 },
    { 0, 0 },
    { 0xc9e9cd28, 90 },
    { 0xe199cd28, 61 },
    { 0x86f9cd2a, 84 },
    { 0xe3e9cd2b, 23 },
    { 0x1759cd2b, 87 },
    { 0x3669cd2d, 23 },
    { 0xf779cd2d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b9cd32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f79cd36, 69 },
    { 0x3229cd36, 95 },
    { 0x5879cd38, 34 },
    { 0x3579cd38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba59cd3c, 43 },
    { 0x2b39cd3d, 23 },
    { 0x2d09cd3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9cd43, 33 },
    { 0x51e9cd43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb9cd47, 95 },
    { 0, 0 },
    { 0x4419cd49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859cd4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xece9cd50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d9cd54, 23 },
    { 0, 0 },
    { 0x15c9cd56, 40 },
    { 0x9109cd56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac9cd5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd9cd5d, 33 },
    { 0x7189cd5d, 76 },
    { 0x5fe9cd5f, 90 },
    { 0x3ac9cd60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3329cd64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9989cd67, 90 },
    { 0x11f9cd68, 61 },
    { 0x6f09cd68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3169cd6c, 93 },
    { 0x2839cd6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d9cd78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab09cd7d, 37 },
    { 0x2539cd7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b9cd82, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4899cd85, 98 },
    { 0x6c89cd86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b9cd8a, 91 },
    { 0, 0 },
    { 0x26a9cd8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439cd93, 46 },
    { 0, 0 },
    { 0xa639cd95, 95 },
    { 0x17c9cd96, 87 },
    { 0x34b9cd97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b9cd9b, 93 },
    { 0x1f49cd9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a9cd9f, 81 },
    { 0, 0 },
    { 0x7cd9cda1, 34 },
    { 0, 0 },
    { 0x5049cda3, 98 },
    { 0xba29cda4, 34 },
    { 0xe0f9cda5, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df9cda9, 90 },
    { 0xa599cda9, 90 },
    { 0x1499cdab, 33 },
    { 0x3a79cdac, 90 },
    { 0x4889cdad, 23 },
    { 0xc479cdab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a9cdb4, 67 },
    { 0x7339cdb4, 66 },
    { 0, 0 },
    { 0x6d99cdb7, 90 },
    { 0xdec9cdb8, 98 },
    { 0xf009cdb9, 33 },
    { 0, 0 },
    { 0xeae9cdbb, 95 },
    { 0, 0 },
    { 0xb7b9cdbd, 33 },
    { 0x509cdbd, 98 },
    { 0, 0 },
    { 0x3889cdc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe799cdc4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b9cdc7, 95 },
    { 0x7799cdc8, 95 },
    { 0xdb59cdc9, 85 },
    { 0x5949cdca, 67 },
    { 0x67b9cdc7, 95 },
    { 0xc8a9cdc9, 85 },
    { 0x3e09cdcd, 90 },
    { 0x3ef9cdce, 23 },
    { 0x4849cdce, 77 },
    { 0x9139cdd0, 95 },
    { 0, 0 },
    { 0x9fc9cdd2, 95 },
    { 0xe6b9cdd3, 98 },
    { 0xe009cdd4, 46 },
    { 0xd849cdd2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xea99cdd8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8239cde4, 99 },
    { 0, 0 },
    { 0x89f9cde6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c29cdeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa9cdf0, 90 },
    { 0x83d9cdf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b69cdf4, 87 },
    { 0xf5b9cdf5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d79cdfa, 22 },
    { 0x99e9cdfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe969ce00, 95 },
    { 0x1d89ce00, 43 },
    { 0, 0 },
    { 0x1fd9ce03, 95 },
    { 0x9329ce04, 90 },
    { 0, 0 },
    { 0xf6d9ce06, 84 },
    { 0xfee9ce07, 84 },
    { 0x2bc9ce07, 76 },
    { 0x509ce09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9ce10, 33 },
    { 0x81c9ce10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299ce16, 76 },
    { 0xc919ce17, 95 },
    { 0x47e9ce17, 36 },
    { 0x8e39ce16, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8159ce23, 33 },
    { 0x9c29ce24, 33 },
    { 0, 0 },
    { 0xb409ce26, 33 },
    { 0x2839ce27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a69ce2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d39ce2e, 98 },
    { 0x1c39ce2f, 61 },
    { 0xba89ce2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce9ce35, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e49ce38, 98 },
    { 0x4429ce39, 98 },
    { 0, 0 },
    { 0x98a9ce3b, 98 },
    { 0x9c59ce3b, 61 },
    { 0x1089ce3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc189ce40, 76 },
    { 0x68a9ce41, 55 },
    { 0x33a9ce41, 36 },
    { 0x5689ce43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc79ce46, 67 },
    { 0, 0 },
    { 0xd159ce48, 76 },
    { 0xdfa9ce48, 93 },
    { 0xd809ce4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d29ce4e, 33 },
    { 0xb59ce4e, 76 },
    { 0x80b9ce4e, 68 },
    { 0xace9ce4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed49ce54, 78 },
    { 0x1629ce55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a9ce59, 95 },
    { 0x67a9ce5a, 23 },
    { 0xc459ce5a, 93 },
    { 0xb209ce5c, 80 },
    { 0xbeb9ce5d, 76 },
    { 0xcfa9ce5e, 61 },
    { 0x2779ce5f, 23 },
    { 0x74d9ce60, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b89ce64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd9ce67, 61 },
    { 0x6649ce68, 78 },
    { 0x2859ce69, 23 },
    { 0x7b39ce69, 33 },
    { 0x7859ce68, 55 },
    { 0xdc29ce6c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1189ce6f, 95 },
    { 0, 0 },
    { 0xdc9ce71, 98 },
    { 0, 0 },
    { 0xa89ce73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7899ce76, 55 },
    { 0xfa79ce76, 61 },
    { 0x1a29ce77, 90 },
    { 0xcae9ce79, 75 },
    { 0, 0 },
    { 0x5d19ce7b, 98 },
    { 0x3a99ce7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f19ce86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a9ce89, 98 },
    { 0xeb9ce8a, 87 },
    { 0x239ce8b, 23 },
    { 0x27a9ce8c, 95 },
    { 0x8019ce8d, 98 },
    { 0, 0 },
    { 0x7679ce8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2859ce9d, 98 },
    { 0x41b9ce9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf99cea2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d9cea5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee59ceaa, 33 },
    { 0x48c9ceab, 34 },
    { 0xa8e9ceac, 77 },
    { 0x6419ceaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb9ceb0, 85 },
    { 0x9649ceb1, 33 },
    { 0xa3a9ceb0, 33 },
    { 0, 0 },
    { 0xa139ceb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d9ceb7, 33 },
    { 0x3359ceb7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8089cebc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc49cec0, 65 },
    { 0x3399cec1, 100 },
    { 0xc4a9cec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe9cec5, 98 },
    { 0x7f9cec5, 100 },
    { 0xe429cec7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d9ceca, 90 },
    { 0x9d89ceca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe9ced4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ad9ced7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519ceda, 93 },
    { 0, 0 },
    { 0x3809cedc, 68 },
    { 0x9bb9cedc, 76 },
    { 0x94e9cede, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f29cee3, 64 },
    { 0xe3d9cee4, 43 },
    { 0x86d9cee5, 80 },
    { 0xb759cee5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5299ceea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a9ceee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9cef2, 95 },
    { 0xfea9cef3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b59cefa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb9cefe, 93 },
    { 0x4009ceff, 100 },
    { 0x3619cf00, 90 },
    { 0x3459cf01, 98 },
    { 0xe8d9cf00, 33 },
    { 0xbf89cf01, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e79cf0b, 19 },
    { 0x52d9cf0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5589cf0f, 20 },
    { 0xf59cf0f, 87 },
    { 0, 0 },
    { 0x7389cf12, 99 },
    { 0xe2b9cf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5a9cf16, 52 },
    { 0x23f9cf16, 90 },
    { 0x2fc9cf16, 76 },
    { 0x9cc9cf19, 23 },
    { 0, 0 },
    { 0x39a9cf1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1409cf1e, 99 },
    { 0x899cf1f, 84 },
    { 0x2bc9cf20, 90 },
    { 0, 0 },
    { 0x96f9cf22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6529cf27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69a9cf33, 46 },
    { 0x4019cf33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbe9cf3a, 61 },
    { 0xa629cf3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf99cf3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b99cf48, 77 },
    { 0x2429cf49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd9cf52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5519cf5c, 100 },
    { 0x6709cf5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c9cf62, 52 },
    { 0xaec9cf62, 90 },
    { 0x1259cf64, 95 },
    { 0, 0 },
    { 0x41f9cf66, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec9cf6e, 77 },
    { 0xba9cf6f, 43 },
    { 0x8089cf70, 23 },
    { 0xb059cf71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb229cf75, 98 },
    { 0x48d9cf76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xead9cf79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69cf7c, 87 },
    { 0, 0 },
    { 0x3029cf7e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9cf83, 76 },
    { 0x68d9cf83, 100 },
    { 0x4559cf85, 95 },
    { 0, 0 },
    { 0xb309cf87, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe19cf8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9cf93, 95 },
    { 0xc279cf94, 46 },
    { 0x949cf95, 88 },
    { 0x55b9cf94, 69 },
    { 0x3f49cf97, 90 },
    { 0, 0 },
    { 0xe4a9cf99, 90 },
    { 0, 0 },
    { 0xc979cf9b, 95 },
    { 0x19d9cf9c, 55 },
    { 0x36a9cf9c, 33 },
    { 0, 0 },
    { 0x7119cf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bb9cfa3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef9cfa6, 61 },
    { 0x779cfa7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc869cfaa, 53 },
    { 0xda99cfaa, 61 },
    { 0, 0 },
    { 0x2bb9cfad, 87 },
    { 0, 0 },
    { 0x5c9cfaf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c9cfb2, 90 },
    { 0, 0 },
    { 0xece9cfb4, 61 },
    { 0, 0 },
    { 0x4499cfb6, 98 },
    { 0xda69cfb7, 61 },
    { 0x2399cfb8, 91 },
    { 0x8499cfb9, 95 },
    { 0, 0 },
    { 0x4069cfbb, 98 },
    { 0, 0 },
    { 0x9a99cfbd, 98 },
    { 0, 0 },
    { 0x8639cfbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f9cfc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9cfc9, 81 },
    { 0, 0 },
    { 0x5349cfcb, 23 },
    { 0xa2c9cfcc, 95 },
    { 0xdba9cfcd, 52 },
    { 0xc89cfce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b79cfd4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2119cfde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8089cfe6, 33 },
    { 0x9319cfe7, 98 },
    { 0, 0 },
    { 0xc9c9cfe9, 90 },
    { 0xb1e9cfea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aa9cfed, 76 },
    { 0x9c69cfee, 66 },
    { 0xc4a9cfee, 55 },
    { 0xdb99cff0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe349cff3, 100 },
    { 0, 0 },
    { 0xaa69cff5, 90 },
    { 0xc5a9cff6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7c9cffc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959d000, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf209d005, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7259d009, 98 },
    { 0xab9d00a, 80 },
    { 0, 0 },
    { 0x8499d00c, 90 },
    { 0x419d00c, 90 },
    { 0x7a69d00d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af9d011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad29d019, 98 },
    { 0xb9b9d01a, 90 },
    { 0x45f9d01b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c39d024, 93 },
    { 0xdcb9d025, 75 },
    { 0x8ce9d026, 77 },
    { 0x93c9d027, 95 },
    { 0x9c89d025, 90 },
    { 0, 0 },
    { 0x1249d02a, 90 },
    { 0, 0 },
    { 0xc59d02c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f9d030, 66 },
    { 0xbdc9d030, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2749d03b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc109d043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa149d047, 95 },
    { 0, 0 },
    { 0xaa59d049, 34 },
    { 0x53d9d049, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d9d04d, 78 },
    { 0xda49d04d, 55 },
    { 0, 0 },
    { 0x49e9d050, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd729d058, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d05b, 91 },
    { 0xce79d05b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6769d060, 85 },
    { 0xa0c9d061, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe59d065, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff9d06a, 23 },
    { 0x1869d06b, 33 },
    { 0, 0 },
    { 0x5c99d06d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc679d071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea9d077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99d07c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99d081, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be9d085, 98 },
    { 0x1ef9d086, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c9d089, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9d08c, 38 },
    { 0x6699d08d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a9d090, 36 },
    { 0, 0 },
    { 0x659d092, 33 },
    { 0x2ef9d093, 55 },
    { 0x18d9d093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7a9d09c, 88 },
    { 0xebd9d09d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b9d0a1, 78 },
    { 0x52d9d0a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd649d0a5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc439d0a9, 90 },
    { 0, 0 },
    { 0x3b09d0ab, 46 },
    { 0x91d9d0ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e79d0af, 23 },
    { 0xe819d0b0, 33 },
    { 0, 0 },
    { 0x8879d0b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc9d0b6, 61 },
    { 0xff49d0b7, 98 },
    { 0xca09d0b7, 91 },
    { 0x88a9d0b8, 95 },
    { 0, 0 },
    { 0x56b9d0bb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd89d0bf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d0c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9679d0c7, 33 },
    { 0x1fd9d0c8, 61 },
    { 0x4f89d0c9, 55 },
    { 0, 0 },
    { 0x8089d0cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb99d0d1, 95 },
    { 0xef89d0d1, 46 },
    { 0, 0 },
    { 0x59b9d0d4, 23 },
    { 0, 0 },
    { 0x14c9d0d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57b9d0db, 34 },
    { 0, 0 },
    { 0x38c9d0dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc509d0e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6389d0e3, 90 },
    { 0xf7b9d0e3, 23 },
    { 0xa259d0e3, 95 },
    { 0x359d0e5, 99 },
    { 0xda39d0e7, 90 },
    { 0x949d0e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99d0f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa089d0f5, 33 },
    { 0xc649d0f6, 93 },
    { 0, 0 },
    { 0x63c9d0f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999d100, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af9d103, 69 },
    { 0x3c39d103, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789d108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4329d10d, 90 },
    { 0, 0 },
    { 0xd0e9d10f, 46 },
    { 0, 0 },
    { 0xb0d9d111, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d29d115, 90 },
    { 0x6139d116, 46 },
    { 0x9dc9d115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6989d11f, 76 },
    { 0, 0 },
    { 0xdf49d121, 61 },
    { 0x9b99d122, 97 },
    { 0x50c9d123, 61 },
    { 0, 0 },
    { 0x51c9d125, 90 },
    { 0x46f9d125, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9229d12c, 88 },
    { 0, 0 },
    { 0xfd9d12e, 33 },
    { 0x1679d12f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f9d132, 78 },
    { 0x6f19d132, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe79d136, 33 },
    { 0, 0 },
    { 0xe549d138, 34 },
    { 0xa079d139, 90 },
    { 0xda09d13a, 90 },
    { 0xbd59d13a, 68 },
    { 0xb299d13b, 90 },
    { 0x6929d13d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1419d140, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f9d144, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9d147, 97 },
    { 0xc879d147, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c69d150, 76 },
    { 0xb109d151, 100 },
    { 0, 0 },
    { 0x2299d153, 90 },
    { 0x47b9d154, 40 },
    { 0, 0 },
    { 0x7389d156, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4609d15b, 76 },
    { 0x3fa9d15c, 43 },
    { 0x23f9d15c, 61 },
    { 0x5d89d15e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd089d16b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c59d16f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4b9d174, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa629d183, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca49d187, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd9d18c, 87 },
    { 0xed19d18d, 23 },
    { 0x80a9d18e, 65 },
    { 0, 0 },
    { 0xac59d190, 100 },
    { 0, 0 },
    { 0xc7f9d192, 77 },
    { 0xa669d192, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e9d196, 33 },
    { 0xe369d197, 76 },
    { 0x25f9d197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab09d1a6, 99 },
    { 0x7539d1a6, 76 },
    { 0xc619d1a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49d1ae, 23 },
    { 0, 0 },
    { 0x3d39d1b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefa9d1b4, 85 },
    { 0xf3a9d1b5, 65 },
    { 0xa329d1b6, 90 },
    { 0, 0 },
    { 0x5899d1b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede9d1bf, 23 },
    { 0x48a9d1c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d9d1c5, 51 },
    { 0x1419d1c6, 76 },
    { 0xdb99d1c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2319d1ca, 95 },
    { 0x7039d1cb, 78 },
    { 0, 0 },
    { 0x9199d1cd, 55 },
    { 0x1119d1ce, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa849d1d2, 76 },
    { 0, 0 },
    { 0x2679d1d4, 98 },
    { 0, 0 },
    { 0x25a9d1d6, 76 },
    { 0, 0 },
    { 0x27f9d1d8, 46 },
    { 0, 0 },
    { 0xa339d1da, 100 },
    { 0x30c9d1da, 84 },
    { 0xebf9d1dc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6189d1e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f9d1e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e9d1ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f99d1ed, 69 },
    { 0x2c89d1ee, 48 },
    { 0x10c9d1ef, 95 },
    { 0x8d19d1f0, 33 },
    { 0x1d29d1f1, 23 },
    { 0xa29d1f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6199d1f6, 55 },
    { 0x1ef9d1f7, 95 },
    { 0, 0 },
    { 0xad59d1f9, 23 },
    { 0x51e9d1fa, 90 },
    { 0xb2e9d1fb, 90 },
    { 0x6d09d1f9, 95 },
    { 0xd279d1fc, 98 },
    { 0x7b69d1fe, 90 },
    { 0x9b09d1fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4659d203, 68 },
    { 0xfb49d204, 34 },
    { 0x7c59d204, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d99d208, 98 },
    { 0x3859d209, 88 },
    { 0xc059d208, 92 },
    { 0xc099d20b, 90 },
    { 0xe699d20c, 55 },
    { 0xdf9d20d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7039d213, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ff9d216, 90 },
    { 0xbbf9d217, 90 },
    { 0x7599d218, 77 },
    { 0x7219d219, 91 },
    { 0, 0 },
    { 0x4229d21b, 66 },
    { 0x2d09d21b, 95 },
    { 0xa939d21d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d49d221, 55 },
    { 0x1759d222, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8579d229, 99 },
    { 0xf009d229, 90 },
    { 0x429d22a, 95 },
    { 0x279d22c, 98 },
    { 0, 0 },
    { 0x4919d22e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd699d231, 93 },
    { 0x6339d231, 90 },
    { 0x1cc9d232, 90 },
    { 0xa519d234, 65 },
    { 0x33c9d235, 87 },
    { 0xadc9d236, 76 },
    { 0xf1d9d237, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d39d242, 23 },
    { 0x9b29d242, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5449d248, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799d24b, 33 },
    { 0xc1b9d24c, 76 },
    { 0x5c89d24d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23f9d250, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea9d255, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29d259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9d265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf269d268, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b9d275, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9d27a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be9d27d, 98 },
    { 0x9679d27d, 46 },
    { 0x8819d27f, 92 },
    { 0, 0 },
    { 0x8d9d281, 75 },
    { 0xf8f9d282, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x589d285, 23 },
    { 0x1a9d286, 90 },
    { 0x6d99d286, 61 },
    { 0xd6f9d288, 33 },
    { 0xfa69d288, 95 },
    { 0xe989d289, 34 },
    { 0xbbb9d288, 33 },
    { 0xea99d285, 23 },
    { 0xdc39d28d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9d293, 90 },
    { 0xd4e9d294, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a9d299, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe69d29e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2559d2a5, 90 },
    { 0x3289d2a6, 33 },
    { 0x7da9d2a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2399d2ae, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2909d2b3, 55 },
    { 0x30c9d2b4, 61 },
    { 0, 0 },
    { 0x2199d2b6, 78 },
    { 0xe2c9d2b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe89d2bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a9d2be, 34 },
    { 0, 0 },
    { 0x4da9d2c0, 33 },
    { 0x9f59d2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f9d2ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb419d2cd, 33 },
    { 0xcdf9d2ce, 90 },
    { 0xdb9d2cf, 78 },
    { 0xb349d2cd, 23 },
    { 0xe959d2d1, 100 },
    { 0xac59d2d1, 34 },
    { 0, 0 },
    { 0xb99d2d4, 23 },
    { 0x1d69d2d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc299d2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd429d2df, 33 },
    { 0x8659d2df, 83 },
    { 0, 0 },
    { 0xeab9d2e2, 35 },
    { 0x179d2e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49d2e7, 23 },
    { 0xcdb9d2e8, 95 },
    { 0xfbb9d2e8, 80 },
    { 0x63f9d2ea, 78 },
    { 0x1599d2eb, 90 },
    { 0x9469d2ea, 64 },
    { 0x9fc9d2ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe339d2f0, 23 },
    { 0x5199d2f0, 76 },
    { 0, 0 },
    { 0x85a9d2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6329d2f7, 33 },
    { 0x2d49d2f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3289d2fd, 55 },
    { 0x39d9d2fe, 90 },
    { 0x7d59d2ff, 81 },
    { 0xb949d300, 65 },
    { 0x70a9d301, 61 },
    { 0x62b9d300, 98 },
    { 0x7919d2ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d9d306, 95 },
    { 0, 0 },
    { 0x4f49d308, 76 },
    { 0, 0 },
    { 0xc489d30a, 98 },
    { 0, 0 },
    { 0xb529d30c, 66 },
    { 0xe059d30c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd9d312, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69d315, 90 },
    { 0x90d9d315, 43 },
    { 0x7679d317, 90 },
    { 0xf459d318, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea79d31c, 61 },
    { 0xeea9d31c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6569d327, 61 },
    { 0x3ab9d327, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc29d32b, 69 },
    { 0, 0 },
    { 0x8599d32d, 85 },
    { 0xc9d9d32e, 76 },
    { 0xc579d32e, 34 },
    { 0x3419d330, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11a9d333, 68 },
    { 0, 0 },
    { 0xbfa9d335, 93 },
    { 0x1cb9d335, 90 },
    { 0xd6a9d337, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcb9d33d, 55 },
    { 0x4ba9d33e, 55 },
    { 0x7329d33f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2609d346, 95 },
    { 0, 0 },
    { 0xcb19d348, 55 },
    { 0x3f89d348, 33 },
    { 0xf9d9d348, 33 },
    { 0x90a9d349, 33 },
    { 0, 0 },
    { 0xe5a9d34d, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4739d352, 76 },
    { 0, 0 },
    { 0xd8d9d354, 66 },
    { 0xda59d355, 90 },
    { 0x35d9d356, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec9d35b, 43 },
    { 0x9d35b, 90 },
    { 0x10b9d35b, 46 },
    { 0, 0 },
    { 0x7c59d35f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f9d363, 23 },
    { 0, 0 },
    { 0x26b9d365, 33 },
    { 0xbe29d366, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9809d36c, 95 },
    { 0, 0 },
    { 0x2229d36e, 52 },
    { 0x339d36f, 90 },
    { 0x2719d36e, 61 },
    { 0x4be9d371, 23 },
    { 0x1749d36e, 61 },
    { 0x4a89d373, 55 },
    { 0x1409d374, 33 },
    { 0x4b69d374, 46 },
    { 0x68d9d374, 61 },
    { 0x26e9d377, 17 },
    { 0xa4b9d378, 98 },
    { 0x9149d376, 90 },
    { 0, 0 },
    { 0x1649d37b, 95 },
    { 0xe7a9d37b, 95 },
    { 0x2d59d37d, 17 },
    { 0x7889d37d, 85 },
    { 0x4ef9d37f, 55 },
    { 0, 0 },
    { 0xb079d381, 55 },
    { 0xf699d382, 69 },
    { 0xc319d382, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea19d386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeb9d389, 98 },
    { 0xf199d38a, 33 },
    { 0, 0 },
    { 0x1e19d38c, 33 },
    { 0xcbd9d38d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19d392, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb129d39f, 65 },
    { 0x7f79d39f, 77 },
    { 0x97c9d3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc419d3a9, 90 },
    { 0xc519d3aa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c9d3b2, 77 },
    { 0x6b49d3b3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e39d3b8, 20 },
    { 0xe749d3b8, 98 },
    { 0, 0 },
    { 0x3c49d3bb, 76 },
    { 0x6999d3bb, 55 },
    { 0x3c49d3bc, 98 },
    { 0xdc69d3be, 33 },
    { 0xb6b9d3bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a19d3c7, 90 },
    { 0x57a9d3c8, 98 },
    { 0x1a49d3c8, 61 },
    { 0x50f9d3ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc29d3ce, 84 },
    { 0x4509d3cf, 33 },
    { 0x4729d3cf, 40 },
    { 0xa9f9d3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3309d3d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6939d3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ff9d3e1, 98 },
    { 0x9639d3e2, 76 },
    { 0xf659d3e1, 99 },
    { 0xbb19d3e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7579d3e7, 98 },
    { 0x5a9d3e7, 95 },
    { 0, 0 },
    { 0xc409d3ea, 46 },
    { 0, 0 },
    { 0x7919d3ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd959d3f0, 95 },
    { 0xe4d9d3f1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d19d3f4, 90 },
    { 0xb89d3f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99d3fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c9d3ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c9d405, 47 },
    { 0xe879d406, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb19d40f, 61 },
    { 0x1fd9d410, 93 },
    { 0x2339d410, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819d418, 98 },
    { 0x7c79d418, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd299d41c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3889d41f, 46 },
    { 0x1469d41f, 55 },
    { 0x4a09d421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a9d424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a89d427, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe99d42b, 33 },
    { 0x1a49d42b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9d430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e9d435, 33 },
    { 0x1e59d436, 65 },
    { 0x50f9d435, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2229d43d, 98 },
    { 0xbd49d43e, 90 },
    { 0xd59d43f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a9d446, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59d44a, 81 },
    { 0, 0 },
    { 0x8829d44c, 23 },
    { 0x9909d44d, 77 },
    { 0x77a9d44c, 43 },
    { 0x7019d44f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb369d457, 76 },
    { 0, 0 },
    { 0xefe9d459, 90 },
    { 0xf6d9d45a, 78 },
    { 0x81c9d45b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad59d460, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f49d472, 55 },
    { 0x6739d473, 61 },
    { 0, 0 },
    { 0x9949d475, 76 },
    { 0, 0 },
    { 0xa219d477, 22 },
    { 0x3ea9d477, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e49d47c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac9d484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc799d487, 55 },
    { 0, 0 },
    { 0xaf89d489, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b09d48d, 23 },
    { 0xce39d48e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679d492, 95 },
    { 0, 0 },
    { 0x1db9d494, 98 },
    { 0x3159d495, 46 },
    { 0x5e79d494, 75 },
    { 0x72e9d497, 90 },
    { 0x35e9d498, 43 },
    { 0, 0 },
    { 0x6d59d49a, 98 },
    { 0, 0 },
    { 0x2319d49c, 95 },
    { 0, 0 },
    { 0xe369d49e, 98 },
    { 0, 0 },
    { 0xecd9d4a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e99d4a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc89d4a8, 33 },
    { 0x1ca9d4a9, 23 },
    { 0xcf59d4a9, 95 },
    { 0, 0 },
    { 0x7919d4ac, 61 },
    { 0xe509d4ad, 90 },
    { 0x5139d4ae, 68 },
    { 0xe009d4ac, 60 },
    { 0xba89d4ac, 61 },
    { 0x47d9d4af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d59d4b7, 81 },
    { 0xde19d4b7, 66 },
    { 0, 0 },
    { 0x98c9d4ba, 90 },
    { 0xca89d4ba, 98 },
    { 0x159d4bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf969d4bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd129d4c2, 98 },
    { 0x60e9d4c3, 76 },
    { 0x6c69d4c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5349d4c8, 33 },
    { 0, 0 },
    { 0xb9f9d4ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb999d4ce, 85 },
    { 0xbd39d4cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7639d4d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb9d4d5, 23 },
    { 0x2b49d4d6, 33 },
    { 0xd429d4d6, 55 },
    { 0x5e69d4d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8749d4de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e99d4e4, 23 },
    { 0x7a49d4e5, 84 },
    { 0xb319d4e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb39d4e9, 90 },
    { 0, 0 },
    { 0x9a9d4eb, 98 },
    { 0x3e19d4ec, 90 },
    { 0x1a79d4ec, 33 },
    { 0x9329d4ee, 46 },
    { 0xe579d4ef, 100 },
    { 0xeee9d4ec, 23 },
    { 0x5ba9d4f1, 78 },
    { 0xa279d4f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb9d4f5, 90 },
    { 0, 0 },
    { 0xc059d4f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc99d4fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3059d501, 33 },
    { 0xbd9d502, 61 },
    { 0, 0 },
    { 0xd739d504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda9d507, 33 },
    { 0x8839d508, 91 },
    { 0x39d9d508, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d50e, 23 },
    { 0, 0 },
    { 0x57c9d510, 55 },
    { 0x2739d511, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f79d519, 33 },
    { 0, 0 },
    { 0xa789d51b, 90 },
    { 0x6409d51b, 98 },
    { 0xf009d51d, 90 },
    { 0x81d9d51e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd089d523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef9d529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf519d52d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d9d534, 66 },
    { 0x4f79d535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe09d53b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9d541, 91 },
    { 0, 0 },
    { 0xce59d543, 84 },
    { 0, 0 },
    { 0x3319d545, 55 },
    { 0xd349d545, 33 },
    { 0x7d09d547, 76 },
    { 0xb3a9d548, 95 },
    { 0x8609d547, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9d54e, 76 },
    { 0x19b9d54f, 90 },
    { 0xbf59d54f, 98 },
    { 0x6549d551, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec59d557, 90 },
    { 0x1bb9d557, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bc9d55c, 77 },
    { 0, 0 },
    { 0x33b9d55e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9d561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a9d564, 61 },
    { 0, 0 },
    { 0x5949d566, 76 },
    { 0x5489d567, 90 },
    { 0x2079d568, 90 },
    { 0xbac9d569, 23 },
    { 0xe979d56a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c49d575, 56 },
    { 0x3c89d576, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe429d579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeb9d57e, 33 },
    { 0, 0 },
    { 0x78f9d580, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c9d586, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a9d589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb79d58d, 33 },
    { 0x1b49d58e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e49d593, 52 },
    { 0x51c9d594, 75 },
    { 0xbc29d595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf9d599, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5239d59e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81a9d5a1, 61 },
    { 0, 0 },
    { 0x8b89d5a3, 76 },
    { 0x7779d5a4, 33 },
    { 0x31e9d5a5, 76 },
    { 0xc839d5a5, 95 },
    { 0x2759d5a7, 77 },
    { 0x9ff9d5a7, 98 },
    { 0x199d5a8, 61 },
    { 0x9559d5aa, 90 },
    { 0xe2c9d5aa, 95 },
    { 0x4c69d5ac, 93 },
    { 0xc179d5a6, 66 },
    { 0x47e9d5a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9d5b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x939d5b6, 100 },
    { 0xec89d5b6, 55 },
    { 0xce89d5b8, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0x3849d5bb, 98 },
    { 0x5a29d5bc, 61 },
    { 0, 0 },
    { 0x6d29d5be, 55 },
    { 0, 0 },
    { 0xdd39d5c0, 34 },
    { 0, 0 },
    { 0x6349d5c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17e9d5d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9d5d6, 43 },
    { 0x3ff9d5d7, 84 },
    { 0x5559d5d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d9d5db, 33 },
    { 0x2f9d5db, 91 },
    { 0x6eb9d5db, 40 },
    { 0xbae9d5dd, 44 },
    { 0, 0 },
    { 0x4059d5e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d9d5e4, 66 },
    { 0, 0 },
    { 0x69d9d5e6, 95 },
    { 0xfd99d5e7, 99 },
    { 0x2679d5e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a9d5eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d5f1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb9d5f4, 90 },
    { 0x43c9d5f5, 98 },
    { 0, 0 },
    { 0x31f9d5f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe39d5fb, 96 },
    { 0xa39d5fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d9d600, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5759d603, 61 },
    { 0x2639d604, 88 },
    { 0xe6f9d604, 61 },
    { 0xd4a9d603, 90 },
    { 0, 0 },
    { 0x5719d608, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d09d60c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa59d610, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b9d614, 23 },
    { 0xcda9d615, 34 },
    { 0xb039d616, 61 },
    { 0x6489d617, 33 },
    { 0x53d9d618, 64 },
    { 0x99d9d619, 61 },
    { 0xaa79d61a, 90 },
    { 0x46e9d61b, 98 },
    { 0x34b9d616, 76 },
    { 0, 0 },
    { 0x4ab9d61e, 76 },
    { 0xc179d61f, 33 },
    { 0, 0 },
    { 0xed29d621, 88 },
    { 0x8889d622, 99 },
    { 0xd239d623, 78 },
    { 0x7ea9d624, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd619d628, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd449d62c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5269d62f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e99d635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda69d63c, 84 },
    { 0xb3b9d63c, 95 },
    { 0xa69d63e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079d641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce9d649, 98 },
    { 0x43e9d64a, 90 },
    { 0xd719d64a, 77 },
    { 0x2c49d64c, 61 },
    { 0x60c9d64c, 90 },
    { 0xc309d64e, 33 },
    { 0x50e9d649, 43 },
    { 0xa129d64b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe399d653, 46 },
    { 0, 0 },
    { 0xe729d655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7b9d658, 33 },
    { 0xce19d659, 95 },
    { 0xcda9d65a, 33 },
    { 0xe799d659, 67 },
    { 0xee59d65c, 95 },
    { 0x2c29d65d, 90 },
    { 0, 0 },
    { 0xcbf9d65f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d89d666, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4029d669, 100 },
    { 0xa799d66a, 33 },
    { 0xba39d66b, 33 },
    { 0xeca9d66c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f9d677, 61 },
    { 0, 0 },
    { 0xbd79d679, 23 },
    { 0xa8d9d67a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd069d67d, 55 },
    { 0, 0 },
    { 0x9c09d67f, 87 },
    { 0x1d79d680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c9d685, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b9d68a, 99 },
    { 0, 0 },
    { 0xd289d68c, 90 },
    { 0xf649d68d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c89d693, 77 },
    { 0, 0 },
    { 0xc769d695, 34 },
    { 0, 0 },
    { 0x4139d697, 23 },
    { 0x9e99d697, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc9d69e, 43 },
    { 0, 0 },
    { 0xec9d6a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9129d6a4, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd339d6a8, 90 },
    { 0x4459d6a9, 88 },
    { 0xe219d6aa, 76 },
    { 0x36d9d6aa, 34 },
    { 0x5939d6aa, 90 },
    { 0x8229d6ad, 100 },
    { 0x24f9d6ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe49d6b7, 33 },
    { 0x5b49d6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab99d6bc, 61 },
    { 0xc6a9d6bc, 95 },
    { 0x1a49d6be, 43 },
    { 0x9189d6bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5509d6c2, 23 },
    { 0, 0 },
    { 0x9cc9d6c4, 61 },
    { 0x6f59d6c5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d59d6c9, 84 },
    { 0x6269d6ca, 55 },
    { 0, 0 },
    { 0x1e29d6cc, 98 },
    { 0x3379d6cd, 91 },
    { 0x1eb9d6cc, 95 },
    { 0x3fc9d6ce, 95 },
    { 0xfe49d6ce, 76 },
    { 0x4739d6d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4e9d6d5, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a9d6d8, 23 },
    { 0xafd9d6d9, 33 },
    { 0xb209d6d8, 75 },
    { 0xecf9d6db, 98 },
    { 0, 0 },
    { 0xa849d6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c9d6e7, 34 },
    { 0x3d19d6e7, 90 },
    { 0, 0 },
    { 0x67a9d6ea, 61 },
    { 0x8009d6eb, 34 },
    { 0x2b19d6ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa619d6f3, 61 },
    { 0x52b9d6f3, 98 },
    { 0, 0 },
    { 0xc549d6f6, 55 },
    { 0, 0 },
    { 0x44b9d6f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2009d701, 81 },
    { 0x9149d701, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c9d70c, 48 },
    { 0xabc9d70d, 76 },
    { 0, 0 },
    { 0x9c9d70f, 78 },
    { 0, 0 },
    { 0x2d19d711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0c9d719, 61 },
    { 0xd8b9d71a, 52 },
    { 0x3389d71b, 93 },
    { 0x72e9d71c, 76 },
    { 0x31f9d71d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a9d722, 55 },
    { 0x60f9d723, 98 },
    { 0xdc29d724, 34 },
    { 0x3c19d725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca79d728, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec9d72b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa79d72e, 98 },
    { 0, 0 },
    { 0xfb89d730, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9019d734, 99 },
    { 0, 0 },
    { 0x2849d736, 78 },
    { 0, 0 },
    { 0x7009d738, 23 },
    { 0xedb9d739, 85 },
    { 0xaf89d738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f99d73d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d49d741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab9d746, 61 },
    { 0x57e9d747, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa789d74d, 43 },
    { 0xf9a9d74d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e9d752, 90 },
    { 0x6119d753, 34 },
    { 0xea19d753, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa189d758, 99 },
    { 0x34d9d759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5169d75c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629d763, 88 },
    { 0x1ea9d764, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa419d768, 55 },
    { 0xb639d768, 33 },
    { 0xe329d76a, 61 },
    { 0xdc99d76a, 87 },
    { 0x44d9d76c, 55 },
    { 0x9809d76d, 43 },
    { 0xdb49d76e, 61 },
    { 0xa109d76f, 76 },
    { 0xb029d76a, 43 },
    { 0xeba9d768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca9d777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a9d77c, 76 },
    { 0, 0 },
    { 0xcb69d77e, 76 },
    { 0, 0 },
    { 0xf0a9d780, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d9d783, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae9d787, 43 },
    { 0xce59d788, 33 },
    { 0xb689d788, 23 },
    { 0xed99d788, 23 },
    { 0x5419d78b, 95 },
    { 0, 0 },
    { 0xbee9d78d, 23 },
    { 0, 0 },
    { 0xac59d78f, 76 },
    { 0xe6b9d78f, 61 },
    { 0xdb29d791, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9d796, 100 },
    { 0, 0 },
    { 0xef49d798, 85 },
    { 0, 0 },
    { 0x9559d79a, 33 },
    { 0x9409d79a, 93 },
    { 0, 0 },
    { 0x2559d79d, 43 },
    { 0x2c99d79e, 61 },
    { 0x7379d79f, 93 },
    { 0xd6f9d79f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6069d7a3, 95 },
    { 0x1b59d7a4, 33 },
    { 0, 0 },
    { 0x1859d7a6, 91 },
    { 0xb329d7a7, 76 },
    { 0x5539d7a8, 23 },
    { 0, 0 },
    { 0x3279d7aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9c9d7b2, 91 },
    { 0x2529d7b2, 91 },
    { 0, 0 },
    { 0x28b9d7b5, 33 },
    { 0, 0 },
    { 0x7169d7b7, 90 },
    { 0, 0 },
    { 0x6819d7b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb979d7be, 95 },
    { 0xed29d7bf, 33 },
    { 0, 0 },
    { 0xd909d7c1, 90 },
    { 0x4e19d7c1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa379d7c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf39d7c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e9d7cd, 61 },
    { 0x72f9d7ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4709d7d1, 95 },
    { 0xd799d7d2, 23 },
    { 0x5d59d7d2, 90 },
    { 0xe789d7d1, 93 },
    { 0xd959d7d5, 55 },
    { 0x7a59d7d5, 77 },
    { 0, 0 },
    { 0xc079d7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e09d7db, 53 },
    { 0x2299d7dc, 24 },
    { 0, 0 },
    { 0x8649d7de, 87 },
    { 0xbfc9d7df, 95 },
    { 0x9b9d7e0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4999d7e5, 76 },
    { 0, 0 },
    { 0xb6a9d7e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659d7ed, 65 },
    { 0x6b89d7ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f59d7f4, 23 },
    { 0xd29d7f5, 23 },
    { 0x7d09d7f6, 90 },
    { 0xc779d7f7, 55 },
    { 0x8c59d7f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd599d7fe, 65 },
    { 0, 0 },
    { 0xa6d9d800, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf449d807, 52 },
    { 0x5749d808, 33 },
    { 0x41c9d808, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb9d80d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f69d814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e9d817, 90 },
    { 0, 0 },
    { 0xe349d819, 33 },
    { 0x4e39d81a, 46 },
    { 0, 0 },
    { 0xce9d81c, 77 },
    { 0x5419d81c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd9d820, 66 },
    { 0x55e9d820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e89d827, 85 },
    { 0x20d9d828, 90 },
    { 0xfcc9d829, 51 },
    { 0, 0 },
    { 0xe0b9d82b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c29d830, 83 },
    { 0, 0 },
    { 0x8469d832, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d9d835, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9589d83b, 33 },
    { 0, 0 },
    { 0xbc59d83d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4659d842, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc469d845, 90 },
    { 0x2e59d846, 84 },
    { 0x2559d847, 80 },
    { 0xfb79d848, 78 },
    { 0x2cd9d845, 61 },
    { 0x1939d848, 95 },
    { 0xa339d847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda49d852, 90 },
    { 0xcef9d853, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329d859, 98 },
    { 0x5b09d85a, 85 },
    { 0xb699d85b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8429d85e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549d865, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd379d86d, 90 },
    { 0xcc59d86e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b09d871, 98 },
    { 0, 0 },
    { 0x2ce9d873, 33 },
    { 0x7d09d874, 61 },
    { 0xda49d873, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a9d878, 90 },
    { 0x54f9d879, 68 },
    { 0x8c79d878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe279d87d, 68 },
    { 0xfef9d87d, 84 },
    { 0, 0 },
    { 0x5a99d880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4439d889, 33 },
    { 0, 0 },
    { 0xe99d88b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa439d88f, 90 },
    { 0x4159d890, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d59d895, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7539d89b, 61 },
    { 0x1a19d89c, 90 },
    { 0x2079d89c, 90 },
    { 0x4859d89b, 95 },
    { 0, 0 },
    { 0xe0d9d8a0, 90 },
    { 0, 0 },
    { 0xa239d8a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d89d8a8, 88 },
    { 0xe959d8a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf379d8b0, 46 },
    { 0x6e49d8b1, 30 },
    { 0x5139d8b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd99d8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc489d8be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a9d8c3, 76 },
    { 0, 0 },
    { 0x6239d8c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb49d8ca, 23 },
    { 0xcb9d8ca, 90 },
    { 0, 0 },
    { 0x3049d8cd, 43 },
    { 0x4309d8ce, 78 },
    { 0, 0 },
    { 0x66e9d8d0, 61 },
    { 0xebf9d8d0, 99 },
    { 0x5399d8d2, 33 },
    { 0, 0 },
    { 0xa609d8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d39d8d8, 61 },
    { 0, 0 },
    { 0xd1f9d8da, 90 },
    { 0xcbf9d8db, 98 },
    { 0xf129d8dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129d8e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9d8e5, 90 },
    { 0xfb99d8e6, 15 },
    { 0x5339d8e7, 65 },
    { 0x9df9d8e7, 55 },
    { 0xcd29d8e9, 68 },
    { 0x3849d8ea, 90 },
    { 0, 0 },
    { 0x4389d8ec, 99 },
    { 0, 0 },
    { 0xc759d8ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc9d8f3, 33 },
    { 0x23d9d8f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a59d8ff, 78 },
    { 0, 0 },
    { 0xe059d901, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae9d905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd9d909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf629d911, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d9d915, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd459d918, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc59d91e, 33 },
    { 0xc29d91f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f39d924, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689d928, 98 },
    { 0x7f39d929, 67 },
    { 0, 0 },
    { 0x9d29d92b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeb9d932, 23 },
    { 0xd2a9d933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129d936, 95 },
    { 0, 0 },
    { 0x8e19d938, 75 },
    { 0, 0 },
    { 0xbb49d93a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a9d93e, 98 },
    { 0, 0 },
    { 0x5fd9d940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9d943, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9d946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b89d94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3029d952, 90 },
    { 0, 0 },
    { 0xc4b9d954, 90 },
    { 0xab29d955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069d95a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x33d9d95d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6949d960, 46 },
    { 0xc319d961, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb949d965, 33 },
    { 0x2909d966, 61 },
    { 0, 0 },
    { 0xc359d968, 55 },
    { 0xf89d969, 55 },
    { 0x9ef9d96a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f59d973, 66 },
    { 0x91f9d974, 23 },
    { 0x7699d974, 95 },
    { 0, 0 },
    { 0xad79d977, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9009d97a, 23 },
    { 0xd689d97b, 98 },
    { 0, 0 },
    { 0x67c9d97d, 34 },
    { 0, 0 },
    { 0x43d9d97f, 77 },
    { 0xc0c9d980, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac49d984, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa829d987, 61 },
    { 0, 0 },
    { 0xc19d989, 90 },
    { 0x4769d98a, 66 },
    { 0x52e9d98b, 55 },
    { 0, 0 },
    { 0xbf09d98d, 23 },
    { 0x2169d98e, 84 },
    { 0, 0 },
    { 0xff49d990, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe719d993, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a9d99b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefd9d9a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de9d9aa, 76 },
    { 0, 0 },
    { 0x1119d9ac, 26 },
    { 0x7179d9ad, 33 },
    { 0xdfa9d9ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8399d9b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9d9b5, 76 },
    { 0x19c9d9b6, 76 },
    { 0, 0 },
    { 0x89d9d9b8, 90 },
    { 0x5bc9d9b9, 98 },
    { 0xd669d9ba, 61 },
    { 0x89a9d9ba, 17 },
    { 0x839d9bc, 90 },
    { 0x41b9d9bc, 33 },
    { 0xc3d9d9bc, 90 },
    { 0x5069d9bf, 43 },
    { 0x6849d9bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199d9c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a19d9cd, 59 },
    { 0x1359d9cd, 90 },
    { 0x8de9d9cf, 33 },
    { 0xebc9d9ce, 33 },
    { 0x54a9d9d1, 93 },
    { 0xa599d9d2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa399d9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb59d9da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249d9e2, 90 },
    { 0xbbc9d9e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd949d9e8, 67 },
    { 0xb839d9e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa199d9f0, 93 },
    { 0, 0 },
    { 0x53f9d9f2, 43 },
    { 0, 0 },
    { 0x5be9d9f4, 76 },
    { 0x5e79d9f4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d9f9, 98 },
    { 0x8459d9fa, 34 },
    { 0xcf49d9f9, 43 },
    { 0xbd39d9fc, 95 },
    { 0xf7b9d9fd, 84 },
    { 0, 0 },
    { 0x5e49d9ff, 100 },
    { 0, 0 },
    { 0x5509da01, 93 },
    { 0x3759da02, 98 },
    { 0x8219da03, 55 },
    { 0x21d9da03, 77 },
    { 0, 0 },
    { 0xacf9da06, 90 },
    { 0, 0 },
    { 0x2499da08, 46 },
    { 0x7a79da09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5189da0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d79da10, 95 },
    { 0xc529da11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fc9da14, 55 },
    { 0xb999da14, 23 },
    { 0x8e39da15, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f9da1a, 34 },
    { 0, 0 },
    { 0x9cb9da1c, 98 },
    { 0x4b69da1d, 61 },
    { 0xd899da1e, 61 },
    { 0xeb29da1c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a79da24, 98 },
    { 0x6ae9da25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41b9da2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef09da33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa489da37, 95 },
    { 0, 0 },
    { 0x31d9da39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd9da3d, 76 },
    { 0x4679da3e, 98 },
    { 0x2bd9da3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe799da43, 93 },
    { 0, 0 },
    { 0xf459da45, 27 },
    { 0x8559da46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd69da4a, 96 },
    { 0x5639da4b, 23 },
    { 0xba89da4b, 17 },
    { 0x4929da4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c09da55, 95 },
    { 0x6189da56, 23 },
    { 0x1b49da57, 75 },
    { 0xddb9da57, 90 },
    { 0xaf39da56, 90 },
    { 0xd759da55, 33 },
    { 0x8ea9da5b, 61 },
    { 0x6199da5c, 91 },
    { 0, 0 },
    { 0x3869da5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e9da61, 78 },
    { 0, 0 },
    { 0x4f09da63, 76 },
    { 0xacb9da64, 61 },
    { 0x96a9da64, 23 },
    { 0xa319da63, 99 },
    { 0, 0 },
    { 0x79a9da68, 46 },
    { 0xb0e9da69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9da6d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1159da73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc759da78, 95 },
    { 0x7bc9da79, 98 },
    { 0, 0 },
    { 0xa6f9da7b, 55 },
    { 0, 0 },
    { 0xfea9da7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc49da81, 66 },
    { 0x7049da82, 90 },
    { 0xea89da82, 23 },
    { 0, 0 },
    { 0xb349da85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9da88, 99 },
    { 0xa2a9da89, 23 },
    { 0x5329da89, 33 },
    { 0x8999da8b, 23 },
    { 0x2cc9da8c, 84 },
    { 0, 0 },
    { 0xe069da8e, 23 },
    { 0x10d9da8e, 61 },
    { 0x5a89da90, 55 },
    { 0xfdd9da8f, 68 },
    { 0xd7f9da8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7939da99, 52 },
    { 0xffc9da9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de9da9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe789daa3, 33 },
    { 0x7d89daa3, 77 },
    { 0, 0 },
    { 0x2459daa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd09daac, 76 },
    { 0x1b59daac, 55 },
    { 0x1b89daad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99dab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb969dab5, 23 },
    { 0x2439dab6, 46 },
    { 0xa2e9dab6, 95 },
    { 0x6129dab8, 61 },
    { 0x8679dab8, 61 },
    { 0xd8a9daba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c9dabf, 53 },
    { 0xaed9dac0, 93 },
    { 0xe749dac1, 95 },
    { 0xd8e9dabf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc9dac7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9daca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e9dace, 90 },
    { 0, 0 },
    { 0x5819dad0, 55 },
    { 0x38f9dad1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7139dad4, 43 },
    { 0x2239dad4, 43 },
    { 0xb09dad6, 33 },
    { 0, 0 },
    { 0xf2b9dad8, 34 },
    { 0x2d49dad9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3149dae0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd59daeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf9daf2, 87 },
    { 0x2b29daf3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf909daf9, 78 },
    { 0, 0 },
    { 0x8f39dafb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9db02, 76 },
    { 0, 0 },
    { 0x8cb9db04, 61 },
    { 0, 0 },
    { 0x66a9db06, 98 },
    { 0x57f9db07, 90 },
    { 0xf9d9db08, 87 },
    { 0x9969db09, 58 },
    { 0x47d9db07, 66 },
    { 0x2889db0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a69db0e, 98 },
    { 0xaa79db0e, 43 },
    { 0xf8e9db10, 76 },
    { 0, 0 },
    { 0x5ec9db12, 90 },
    { 0, 0 },
    { 0x7a89db14, 91 },
    { 0xd709db15, 90 },
    { 0x3b09db16, 95 },
    { 0x7659db17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab9db1f, 61 },
    { 0x86b9db20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629db27, 93 },
    { 0, 0 },
    { 0xac29db29, 61 },
    { 0, 0 },
    { 0x7539db2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb89db2f, 90 },
    { 0x45e9db30, 61 },
    { 0xf8f9db30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec19db34, 61 },
    { 0, 0 },
    { 0x8d79db36, 61 },
    { 0, 0 },
    { 0x7d99db38, 55 },
    { 0xe369db39, 23 },
    { 0xc8b9db3a, 43 },
    { 0, 0 },
    { 0xe0a9db3c, 43 },
    { 0x70a9db3d, 52 },
    { 0x5da9db3e, 91 },
    { 0xeb79db3e, 98 },
    { 0xcba9db40, 77 },
    { 0x1989db3e, 95 },
    { 0x2529db42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd99db47, 76 },
    { 0x5449db48, 93 },
    { 0x1059db49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e59db4d, 46 },
    { 0, 0 },
    { 0x6869db4f, 68 },
    { 0x1ca9db4f, 93 },
    { 0xd659db4f, 90 },
    { 0x9c79db52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e9db55, 34 },
    { 0xafc9db56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf09db60, 76 },
    { 0xac9db61, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9979db64, 33 },
    { 0, 0 },
    { 0xd059db66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2869db6a, 83 },
    { 0, 0 },
    { 0x5909db6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a9db72, 90 },
    { 0x3f59db72, 61 },
    { 0x1ab9db74, 46 },
    { 0, 0 },
    { 0xe799db76, 90 },
    { 0x59f9db77, 23 },
    { 0x5a89db77, 98 },
    { 0xc69db79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b9db7d, 90 },
    { 0x9189db7e, 76 },
    { 0x48a9db7e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7189db82, 90 },
    { 0xef99db83, 98 },
    { 0x3d29db83, 68 },
    { 0xf329db83, 33 },
    { 0xc5a9db82, 33 },
    { 0x80a9db82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb419db95, 90 },
    { 0xf539db95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2359db9a, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29db9d, 90 },
    { 0x2b09db9e, 90 },
    { 0x3c79db9d, 61 },
    { 0xb269dba0, 46 },
    { 0xe149dba1, 76 },
    { 0x5a09dba0, 98 },
    { 0, 0 },
    { 0xa3e9dba4, 90 },
    { 0, 0 },
    { 0x66a9dba6, 95 },
    { 0xe3f9dba7, 61 },
    { 0xa069dba7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d9dbac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6459dbb2, 84 },
    { 0x2d99dbb3, 33 },
    { 0, 0 },
    { 0x86e9dbb5, 33 },
    { 0x1379dbb5, 43 },
    { 0xda9dbb7, 34 },
    { 0x4b39dbb8, 90 },
    { 0x69e9dbb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf789dbbe, 90 },
    { 0x1c99dbbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8349dbc6, 90 },
    { 0x3069dbc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5739dbcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc369dbce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9dbd1, 90 },
    { 0, 0 },
    { 0x7139dbd3, 65 },
    { 0x9b49dbd3, 33 },
    { 0, 0 },
    { 0xf049dbd6, 90 },
    { 0x2b29dbd7, 47 },
    { 0x4ca9dbd8, 98 },
    { 0x9c39dbd8, 33 },
    { 0xaf09dbda, 61 },
    { 0x93b9dbdb, 43 },
    { 0, 0 },
    { 0xf6c9dbdd, 91 },
    { 0x5019dbde, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xddf9dbe1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe439dbe9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd889dbf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b79dbf6, 55 },
    { 0x419dbf7, 61 },
    { 0x96d9dbf8, 33 },
    { 0x94c9dbf9, 91 },
    { 0x6b9dbf8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0f9dc04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2259dc07, 94 },
    { 0x76d9dc07, 55 },
    { 0xb569dc07, 98 },
    { 0xb709dc0a, 67 },
    { 0xe029dc0b, 61 },
    { 0x7529dc0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ac9dc0f, 66 },
    { 0xcfd9dc0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd9dc13, 98 },
    { 0x6019dc13, 64 },
    { 0x1c59dc15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e9dc19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb79dc1d, 33 },
    { 0x5ae9dc1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1359dc21, 98 },
    { 0x55c9dc22, 61 },
    { 0xc8b9dc23, 95 },
    { 0, 0 },
    { 0xf7a9dc25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc639dc2d, 23 },
    { 0x2db9dc2d, 45 },
    { 0x20c9dc2e, 61 },
    { 0, 0 },
    { 0x6d19dc31, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8859dc36, 77 },
    { 0, 0 },
    { 0xe9e9dc38, 46 },
    { 0xc869dc39, 67 },
    { 0x7009dc3a, 93 },
    { 0x5ab9dc38, 98 },
    { 0, 0 },
    { 0xa659dc3d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5859dc43, 46 },
    { 0xb5f9dc43, 23 },
    { 0, 0 },
    { 0x5909dc46, 23 },
    { 0x4479dc47, 17 },
    { 0, 0 },
    { 0x4b89dc49, 76 },
    { 0x1fe9dc49, 68 },
    { 0xcb9dc49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6c9dc4e, 75 },
    { 0x5af9dc4f, 33 },
    { 0x7409dc50, 94 },
    { 0x38f9dc4e, 61 },
    { 0, 0 },
    { 0xd729dc53, 90 },
    { 0x6949dc54, 26 },
    { 0, 0 },
    { 0x3ea9dc56, 61 },
    { 0x6699dc57, 61 },
    { 0x12d9dc58, 61 },
    { 0x5699dc59, 55 },
    { 0xd919dc5a, 33 },
    { 0xfe69dc56, 95 },
    { 0, 0 },
    { 0xd39dc5d, 83 },
    { 0x9239dc5e, 23 },
    { 0x3a69dc5e, 43 },
    { 0x9f49dc60, 61 },
    { 0xf249dc5e, 90 },
    { 0x4919dc5f, 99 },
    { 0x53b9dc63, 66 },
    { 0xea89dc64, 33 },
    { 0xcc89dc65, 55 },
    { 0xc3b9dc5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1659dc6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b49dc74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f09dc7d, 91 },
    { 0x1ec9dc7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7379dc82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e9dc85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d9dc8a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee9dc8e, 96 },
    { 0xa0d9dc8f, 61 },
    { 0x57c9dc90, 99 },
    { 0x36f9dc91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f9dc98, 61 },
    { 0x7b39dc99, 33 },
    { 0x3789dc9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb9dc9e, 90 },
    { 0xdf99dc9f, 33 },
    { 0x5da9dca0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b59dca3, 23 },
    { 0x3a09dca3, 87 },
    { 0x1609dca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df9dcab, 61 },
    { 0x73c9dcab, 90 },
    { 0xf7c9dcab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea9dcb0, 98 },
    { 0x4059dcb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9dcb5, 33 },
    { 0x9969dcb6, 99 },
    { 0x1709dcb6, 55 },
    { 0x7b89dcb7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb9dcbe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcc2, 55 },
    { 0xfcd9dcc2, 98 },
    { 0x6b9dcc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef9dcc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9dccd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb9dcd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78b9dcd8, 33 },
    { 0xc29dcd9, 98 },
    { 0x9049dcda, 96 },
    { 0, 0 },
    { 0x8de9dcdc, 90 },
    { 0x99a9dcdd, 61 },
    { 0xe529dcdd, 100 },
    { 0x4929dcdd, 95 },
    { 0, 0 },
    { 0xfb39dce1, 33 },
    { 0x3389dce2, 34 },
    { 0xfc69dce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5409dce7, 95 },
    { 0, 0 },
    { 0x7e79dce9, 43 },
    { 0x5229dce9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc39dced, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dcf5, 42 },
    { 0, 0 },
    { 0x4419dcf7, 52 },
    { 0, 0 },
    { 0xede9dcf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc449dcfd, 61 },
    { 0x18b9dcfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de9dd02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc49dd05, 23 },
    { 0x9719dd06, 90 },
    { 0x46c9dd05, 33 },
    { 0xb719dd06, 23 },
    { 0xa6e9dd09, 66 },
    { 0x9919dd06, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe9dd0d, 98 },
    { 0, 0 },
    { 0x9b39dd0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae9dd14, 93 },
    { 0xdba9dd15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d39dd18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d79dd22, 90 },
    { 0x5159dd23, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9dd29, 78 },
    { 0x83a9dd29, 91 },
    { 0x55a9dd2b, 33 },
    { 0xaff9dd2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82f9dd2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2309dd33, 33 },
    { 0xd759dd34, 93 },
    { 0x6509dd35, 93 },
    { 0x5da9dd36, 91 },
    { 0x2049dd34, 90 },
    { 0x4619dd38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f29dd3b, 95 },
    { 0, 0 },
    { 0x8189dd3d, 98 },
    { 0, 0 },
    { 0x80a9dd3f, 61 },
    { 0xb859dd3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd69dd43, 90 },
    { 0xf779dd44, 33 },
    { 0xb069dd45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09dd4e, 33 },
    { 0x8789dd4f, 76 },
    { 0xd859dd50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9839dd5a, 100 },
    { 0xef39dd5a, 34 },
    { 0, 0 },
    { 0x2199dd5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d19dd61, 61 },
    { 0, 0 },
    { 0x4139dd63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e69dd6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb759dd6e, 91 },
    { 0x3a09dd6f, 52 },
    { 0x7a9dd70, 90 },
    { 0x4679dd71, 23 },
    { 0x30f9dd6e, 33 },
    { 0xa609dd73, 101 },
    { 0xed39dd70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2889dd79, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce9dd7c, 33 },
    { 0x8b9dd7c, 33 },
    { 0x3199dd7c, 43 },
    { 0x7a9dd7f, 33 },
    { 0xbc19dd80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf9dd85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9dd88, 91 },
    { 0x179dd88, 90 },
    { 0x4169dd8a, 95 },
    { 0, 0 },
    { 0x21b9dd8c, 43 },
    { 0, 0 },
    { 0xa6b9dd8e, 90 },
    { 0, 0 },
    { 0x4df9dd90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b79dd93, 85 },
    { 0x6119dd93, 67 },
    { 0, 0 },
    { 0x2689dd96, 84 },
    { 0, 0 },
    { 0x179dd98, 95 },
    { 0, 0 },
    { 0x11e9dd9a, 78 },
    { 0x27c9dd9b, 76 },
    { 0xafd9dd9c, 95 },
    { 0xcaa9dd9c, 98 },
    { 0, 0 },
    { 0xbb29dd9f, 23 },
    { 0xcab9dda0, 78 },
    { 0, 0 },
    { 0x8869dda2, 93 },
    { 0, 0 },
    { 0x68c9dda4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf49dda8, 95 },
    { 0, 0 },
    { 0xe749ddaa, 78 },
    { 0x8619ddab, 65 },
    { 0x2899ddac, 98 },
    { 0x16c9ddac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2509ddb0, 90 },
    { 0, 0 },
    { 0x4409ddb2, 52 },
    { 0x54c9ddb3, 99 },
    { 0x20a9ddb2, 99 },
    { 0xdb99ddb5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e19ddba, 90 },
    { 0x7e99ddba, 95 },
    { 0x76b9ddbb, 67 },
    { 0x9899ddbd, 98 },
    { 0, 0 },
    { 0xc0b9ddbf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7379ddc4, 23 },
    { 0x81c9ddc4, 93 },
    { 0xf6f9ddc5, 95 },
    { 0xe3f9ddc6, 17 },
    { 0, 0 },
    { 0x54f9ddc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e89ddce, 77 },
    { 0x8019ddce, 52 },
    { 0x8a79ddce, 77 },
    { 0x5e19ddd1, 95 },
    { 0xb829ddd2, 23 },
    { 0x49d9ddd2, 90 },
    { 0xeeb9ddd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99dddc, 76 },
    { 0, 0 },
    { 0x64a9ddde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9ddee, 85 },
    { 0xe8e9ddee, 33 },
    { 0xfbf9ddf0, 100 },
    { 0, 0 },
    { 0xf599ddf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd49ddf8, 92 },
    { 0x1129ddf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9899ddfc, 90 },
    { 0xfe19ddfc, 23 },
    { 0x5079ddfc, 90 },
    { 0x4f89ddfd, 33 },
    { 0, 0 },
    { 0xe599de01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb79de04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc099de07, 52 },
    { 0x7339de08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9de0b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49de12, 84 },
    { 0x39d9de12, 33 },
    { 0x2659de12, 64 },
    { 0x1029de15, 61 },
    { 0x409de16, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2399de1a, 61 },
    { 0x2569de1b, 55 },
    { 0x47b9de1c, 99 },
    { 0x96f9de1b, 38 },
    { 0x5ec9de1b, 34 },
    { 0xc549de1f, 95 },
    { 0, 0 },
    { 0x8839de21, 85 },
    { 0x7b49de22, 33 },
    { 0x4239de21, 61 },
    { 0, 0 },
    { 0xabf9de25, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc9de29, 90 },
    { 0xa869de2a, 55 },
    { 0, 0 },
    { 0x5bd9de2c, 46 },
    { 0x7479de2d, 87 },
    { 0, 0 },
    { 0x6829de2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf449de35, 95 },
    { 0x9d39de36, 23 },
    { 0xaa39de37, 46 },
    { 0x88d9de37, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd99de3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce89de3f, 93 },
    { 0xbc79de40, 43 },
    { 0x43b9de41, 23 },
    { 0x3239de42, 93 },
    { 0xba29de43, 23 },
    { 0x83c9de43, 99 },
    { 0x1759de43, 77 },
    { 0x1699de42, 33 },
    { 0xf949de47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb969de4b, 92 },
    { 0x1719de4c, 48 },
    { 0x3999de4d, 98 },
    { 0xca09de4e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe889de51, 67 },
    { 0, 0 },
    { 0xdeb9de53, 23 },
    { 0, 0 },
    { 0x7ce9de55, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679de5a, 61 },
    { 0x1069de5b, 33 },
    { 0, 0 },
    { 0xe869de5d, 33 },
    { 0, 0 },
    { 0x2b69de5f, 87 },
    { 0xb319de60, 95 },
    { 0, 0 },
    { 0x9759de62, 55 },
    { 0x6c09de63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9de68, 23 },
    { 0, 0 },
    { 0xdf69de6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c9de6d, 99 },
    { 0x9dc9de6e, 34 },
    { 0, 0 },
    { 0xdc89de70, 61 },
    { 0, 0 },
    { 0x1699de72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719de76, 34 },
    { 0x6dd9de77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c9de7b, 90 },
    { 0x3479de7c, 88 },
    { 0, 0 },
    { 0x3c99de7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f79de81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d9de84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb049de8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc09de98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c69de9f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c39dea4, 95 },
    { 0, 0 },
    { 0x3019dea6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69deab, 34 },
    { 0xe799deac, 90 },
    { 0x7789deab, 98 },
    { 0, 0 },
    { 0x2629deaf, 87 },
    { 0x9c19deb0, 23 },
    { 0x10b9deb0, 84 },
    { 0, 0 },
    { 0x6a79deb3, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c9deb6, 61 },
    { 0, 0 },
    { 0x5689deb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca9debd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c9dec2, 61 },
    { 0x4c49dec3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c69deca, 100 },
    { 0x3be9decb, 61 },
    { 0, 0 },
    { 0xf969decd, 37 },
    { 0x5899dece, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f59ded3, 33 },
    { 0, 0 },
    { 0x6209ded5, 75 },
    { 0x7079ded6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c9deda, 33 },
    { 0xef19dedb, 98 },
    { 0, 0 },
    { 0x3469dedd, 23 },
    { 0xa719dede, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4079dee2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5509dee5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d39dee8, 99 },
    { 0, 0 },
    { 0x6309deea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf9deee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf659def2, 23 },
    { 0, 0 },
    { 0xaf59def4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e9defa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9079deff, 78 },
    { 0xcae9df00, 83 },
    { 0, 0 },
    { 0x82c9df02, 95 },
    { 0, 0 },
    { 0x759df04, 100 },
    { 0xd149df04, 33 },
    { 0x6709df06, 98 },
    { 0xd339df07, 90 },
    { 0xa829df08, 61 },
    { 0x3159df04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd069df0c, 99 },
    { 0x18d9df0c, 87 },
    { 0x46c9df0d, 98 },
    { 0xd1f9df0d, 61 },
    { 0, 0 },
    { 0xd0b9df11, 36 },
    { 0x8f79df11, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c9df19, 33 },
    { 0xeb9df19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a39df1f, 90 },
    { 0x77a9df1f, 90 },
    { 0xa6b9df21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d9df25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa449df2d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f9df30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f9df33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129df38, 55 },
    { 0, 0 },
    { 0x7b59df3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa819df3d, 88 },
    { 0x919df3d, 61 },
    { 0xcd49df3e, 98 },
    { 0x5e69df40, 85 },
    { 0, 0 },
    { 0xf739df42, 17 },
    { 0, 0 },
    { 0xa869df44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea9df4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1699df50, 61 },
    { 0, 0 },
    { 0x6e39df52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4579df58, 34 },
    { 0, 0 },
    { 0xf939df5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a9df5e, 61 },
    { 0, 0 },
    { 0x1c49df60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1149df64, 88 },
    { 0x339df65, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x8069df68, 33 },
    { 0x20d9df69, 34 },
    { 0, 0 },
    { 0xd1a9df6b, 91 },
    { 0xec9df6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59df6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad39df72, 90 },
    { 0x5e79df72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f9df77, 90 },
    { 0x2cc9df77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2509df7b, 46 },
    { 0x95e9df7b, 90 },
    { 0xb169df7b, 33 },
    { 0, 0 },
    { 0x87d9df7f, 90 },
    { 0xc3e9df80, 90 },
    { 0xd1f9df80, 90 },
    { 0xed79df81, 34 },
    { 0, 0 },
    { 0x2f9df84, 66 },
    { 0, 0 },
    { 0xf6b9df86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc9df8c, 76 },
    { 0, 0 },
    { 0x13e9df8e, 33 },
    { 0xaaa9df8f, 23 },
    { 0, 0 },
    { 0x22a9df91, 61 },
    { 0x3989df92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2e9df95, 85 },
    { 0, 0 },
    { 0xf899df97, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d49df9a, 101 },
    { 0xacc9df9a, 95 },
    { 0x2f09df9c, 33 },
    { 0xbb39df9c, 55 },
    { 0x1d9df9e, 55 },
    { 0xccb9df9e, 84 },
    { 0xa299dfa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9dfa4, 91 },
    { 0, 0 },
    { 0x3dd9dfa6, 95 },
    { 0, 0 },
    { 0xd6b9dfa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef89dfac, 53 },
    { 0, 0 },
    { 0x8ed9dfae, 99 },
    { 0x1fd9dfaf, 33 },
    { 0x54d9dfb0, 83 },
    { 0xb939dfb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5649dfb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9dfbc, 34 },
    { 0xa6b9dfbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f9dfc3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a9dfcc, 43 },
    { 0, 0 },
    { 0xadc9dfce, 76 },
    { 0, 0 },
    { 0xcbd9dfd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5129dfd4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9e9dfd7, 98 },
    { 0, 0 },
    { 0x819dfd9, 93 },
    { 0x8bf9dfd9, 34 },
    { 0xf609dfda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa489dfde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a9dfe8, 61 },
    { 0x6d99dfe9, 95 },
    { 0xad69dfe8, 33 },
    { 0, 0 },
    { 0xa099dfec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e9dfef, 90 },
    { 0x8e79dff0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b9dff6, 90 },
    { 0x4d19dff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b69dffa, 33 },
    { 0, 0 },
    { 0x16e9dffc, 34 },
    { 0, 0 },
    { 0x9ae9dffe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8589e001, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a29e005, 90 },
    { 0xd9d9e006, 93 },
    { 0x489e007, 100 },
    { 0x6e79e005, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b99e00b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf709e00e, 98 },
    { 0xc719e00f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec79e014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb9e017, 90 },
    { 0, 0 },
    { 0x54f9e019, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf99e01c, 55 },
    { 0x8f69e01d, 78 },
    { 0xbfa9e01e, 61 },
    { 0x1579e01f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9e022, 61 },
    { 0xe5c9e023, 78 },
    { 0x4de9e023, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079e02a, 33 },
    { 0, 0 },
    { 0x4549e02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a9e034, 52 },
    { 0, 0 },
    { 0x1b99e036, 55 },
    { 0x1ea9e037, 23 },
    { 0x1af9e038, 61 },
    { 0, 0 },
    { 0x7fc9e03a, 34 },
    { 0x19c9e03b, 33 },
    { 0xa569e03c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8169e040, 90 },
    { 0x3339e041, 34 },
    { 0xb2e9e042, 61 },
    { 0x4409e043, 90 },
    { 0, 0 },
    { 0x6a29e045, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aa9e049, 46 },
    { 0x63f9e049, 76 },
    { 0, 0 },
    { 0xa9d9e04c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf149e051, 95 },
    { 0x7f69e052, 95 },
    { 0, 0 },
    { 0xfb79e054, 34 },
    { 0x2789e054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa889e059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4159e05e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf199e062, 95 },
    { 0x1aa9e062, 84 },
    { 0x74a9e064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b9e067, 87 },
    { 0x6199e068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc659e06b, 95 },
    { 0, 0 },
    { 0x8049e06d, 52 },
    { 0x7869e06e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a89e073, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf439e076, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c9e07c, 90 },
    { 0x9ef9e07d, 43 },
    { 0x5a09e07e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9289e082, 55 },
    { 0xa4f9e083, 77 },
    { 0xfcd9e083, 61 },
    { 0xe2e9e085, 90 },
    { 0xb179e084, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb59e08c, 95 },
    { 0, 0 },
    { 0xb529e08e, 67 },
    { 0x94d9e08f, 56 },
    { 0xa39e08f, 76 },
    { 0x1a9e091, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35a9e096, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb39e099, 66 },
    { 0x7f29e09a, 38 },
    { 0x1229e09b, 23 },
    { 0xb779e09c, 95 },
    { 0xcfe9e09c, 23 },
    { 0, 0 },
    { 0x16c9e09f, 99 },
    { 0x20a9e09f, 23 },
    { 0xef19e0a1, 61 },
    { 0xf0b9e0a2, 33 },
    { 0xe179e0a3, 95 },
    { 0x4969e0a4, 100 },
    { 0xfcd9e0a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd09e0ae, 78 },
    { 0xa029e0af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf59e0b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb9e0b6, 23 },
    { 0xa379e0b6, 98 },
    { 0x8179e0b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb779e0bd, 85 },
    { 0, 0 },
    { 0x2ab9e0bf, 23 },
    { 0x9309e0c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fc9e0c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc339e0c7, 23 },
    { 0x7239e0c8, 34 },
    { 0x8869e0c9, 90 },
    { 0x81e9e0c9, 95 },
    { 0x7979e0c9, 93 },
    { 0x5949e0cc, 93 },
    { 0xfcb9e0cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba59e0d5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56f9e0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e89e0de, 61 },
    { 0xea09e0df, 33 },
    { 0, 0 },
    { 0x9eb9e0e1, 95 },
    { 0xf3c9e0e1, 98 },
    { 0x6579e0e3, 61 },
    { 0, 0 },
    { 0xe199e0e5, 76 },
    { 0x6819e0e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3169e0eb, 32 },
    { 0, 0 },
    { 0xdf49e0ed, 66 },
    { 0xa829e0ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea9e0f1, 23 },
    { 0xbce9e0f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9949e0f6, 90 },
    { 0xce59e0f6, 76 },
    { 0xd3a9e0f7, 33 },
    { 0x4079e0f9, 33 },
    { 0x1039e0fa, 68 },
    { 0, 0 },
    { 0xb0f9e0fc, 33 },
    { 0xa979e0fd, 85 },
    { 0xb109e0fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2649e102, 93 },
    { 0xa739e102, 90 },
    { 0x6a09e103, 98 },
    { 0x4c39e105, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7729e109, 99 },
    { 0x1e49e10a, 52 },
    { 0xbe99e10b, 66 },
    { 0xcff9e10c, 46 },
    { 0xeca9e10d, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x3999e110, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1269e115, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca39e11a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6749e11f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff09e123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8179e126, 90 },
    { 0, 0 },
    { 0xb159e128, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da9e12b, 33 },
    { 0x1e9e12c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d9e130, 66 },
    { 0x1729e131, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf309e135, 33 },
    { 0x6f59e135, 61 },
    { 0x3cf9e137, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x269e13a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1499e13d, 23 },
    { 0, 0 },
    { 0xd7d9e13f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6689e146, 61 },
    { 0xbc69e147, 23 },
    { 0, 0 },
    { 0x5069e149, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3929e14d, 23 },
    { 0xdcd9e14e, 90 },
    { 0xe0e9e14f, 34 },
    { 0x6e19e150, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2019e15d, 85 },
    { 0x28d9e15e, 34 },
    { 0xcd49e15f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1349e164, 90 },
    { 0xf0f9e164, 97 },
    { 0, 0 },
    { 0x9ad9e167, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7339e16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6759e16e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d9e174, 84 },
    { 0, 0 },
    { 0x7699e176, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf699e179, 93 },
    { 0x1469e17a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7b9e17f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c9e183, 98 },
    { 0xf4e9e184, 17 },
    { 0xe969e185, 55 },
    { 0x7fc9e185, 76 },
    { 0x6a09e187, 23 },
    { 0, 0 },
    { 0xa0d9e189, 61 },
    { 0x4089e18a, 99 },
    { 0x6299e18b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4259e18e, 34 },
    { 0x9de9e18f, 76 },
    { 0x89e190, 33 },
    { 0, 0 },
    { 0xd289e192, 33 },
    { 0, 0 },
    { 0x2af9e194, 33 },
    { 0x65f9e195, 87 },
    { 0x90e9e196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e49e19b, 33 },
    { 0x7369e19b, 33 },
    { 0, 0 },
    { 0x2829e19e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8749e1a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f9e1ab, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc539e1af, 84 },
    { 0x3ca9e1b0, 33 },
    { 0x7799e1b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f09e1b4, 68 },
    { 0xf329e1b5, 43 },
    { 0x5739e1b6, 61 },
    { 0x8829e1b7, 55 },
    { 0xad09e1b8, 90 },
    { 0x1979e1b9, 98 },
    { 0xcac9e1b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f9e1c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb269e1c4, 95 },
    { 0x7a9e1c5, 98 },
    { 0xccd9e1c6, 55 },
    { 0, 0 },
    { 0x2c9e1c8, 33 },
    { 0x9e69e1c9, 17 },
    { 0x40f9e1c9, 33 },
    { 0, 0 },
    { 0x64b9e1cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd519e1d0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe329e1d7, 33 },
    { 0x57a9e1d8, 95 },
    { 0xde29e1d9, 95 },
    { 0xa9d9e1da, 95 },
    { 0, 0 },
    { 0x74a9e1dc, 98 },
    { 0xcb39e1dd, 90 },
    { 0x7fb9e1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd149e1e2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe369e1ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8009e1f2, 98 },
    { 0x8a59e1f3, 23 },
    { 0x4f29e1f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629e1fc, 34 },
    { 0x57b9e1fd, 55 },
    { 0x14c9e1fe, 66 },
    { 0xa1d9e1ff, 84 },
    { 0xfe09e200, 26 },
    { 0x94a9e200, 61 },
    { 0x5f79e201, 34 },
    { 0x3f69e202, 90 },
    { 0x6b79e203, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d79e208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8329e20b, 99 },
    { 0, 0 },
    { 0x56a9e20d, 43 },
    { 0, 0 },
    { 0xa599e20f, 23 },
    { 0x5a09e210, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c9e214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d9e219, 95 },
    { 0xd329e21a, 33 },
    { 0x1c9e21a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b9e21e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd329e226, 61 },
    { 0xf5c9e227, 33 },
    { 0, 0 },
    { 0x66c9e229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd9e22e, 55 },
    { 0x83a9e22f, 90 },
    { 0xc0d9e22f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4359e235, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f9e238, 90 },
    { 0, 0 },
    { 0x6829e23a, 76 },
    { 0xefb9e23a, 61 },
    { 0, 0 },
    { 0xac39e23d, 61 },
    { 0, 0 },
    { 0xa199e23f, 66 },
    { 0xa0e9e240, 78 },
    { 0x5a39e240, 66 },
    { 0x47a9e242, 76 },
    { 0x58b9e243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd029e248, 76 },
    { 0xe9e249, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc69e24f, 23 },
    { 0xf19e24f, 33 },
    { 0x60b9e251, 17 },
    { 0x3c59e252, 76 },
    { 0, 0 },
    { 0xf3b9e254, 34 },
    { 0xa749e255, 61 },
    { 0x4439e255, 90 },
    { 0xa8a9e257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729e25c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc429e25f, 100 },
    { 0xcaa9e260, 61 },
    { 0x4749e260, 66 },
    { 0xae69e261, 61 },
    { 0xe2e9e263, 34 },
    { 0xe299e25f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259e268, 90 },
    { 0x77d9e269, 90 },
    { 0xbad9e269, 78 },
    { 0x7ab9e26b, 87 },
    { 0x3a79e26c, 95 },
    { 0x3ea9e26c, 61 },
    { 0xe899e26e, 43 },
    { 0xdd59e26d, 55 },
    { 0x6df9e26d, 43 },
    { 0, 0 },
    { 0xf759e272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49e275, 46 },
    { 0, 0 },
    { 0xee39e277, 33 },
    { 0x6ce9e278, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f9e27c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd579e283, 46 },
    { 0x5809e284, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa259e289, 52 },
    { 0xb429e289, 95 },
    { 0xebe9e289, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xebf9e28e, 46 },
    { 0x7099e28f, 52 },
    { 0xf119e28f, 31 },
    { 0xe79e28f, 43 },
    { 0, 0 },
    { 0x9d39e293, 34 },
    { 0, 0 },
    { 0x24a9e295, 61 },
    { 0x8179e296, 90 },
    { 0x41e9e297, 90 },
    { 0x4199e296, 46 },
    { 0x4459e297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae99e2a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc029e2a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c09e2aa, 23 },
    { 0xf319e2ab, 76 },
    { 0, 0 },
    { 0xe659e2ad, 33 },
    { 0xdd29e2ae, 98 },
    { 0x1239e2af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d9e2b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8759e2b8, 90 },
    { 0xc9b9e2b8, 98 },
    { 0, 0 },
    { 0x7ca9e2bb, 78 },
    { 0xc479e2bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5189e2c0, 23 },
    { 0xcb99e2c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b9e2c8, 90 },
    { 0xd1b9e2c8, 98 },
    { 0x3eb9e2ca, 88 },
    { 0xf439e2ca, 91 },
    { 0xb389e2ca, 66 },
    { 0x9ad9e2cb, 33 },
    { 0x1bd9e2ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7789e2d2, 98 },
    { 0xa599e2d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8869e2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9e2dc, 99 },
    { 0xb9c9e2dd, 35 },
    { 0x47b9e2de, 23 },
    { 0x999e2df, 95 },
    { 0x60d9e2de, 34 },
    { 0x4989e2e1, 84 },
    { 0xdd09e2e2, 91 },
    { 0x6c99e2e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9e2e6, 93 },
    { 0, 0 },
    { 0x719e2e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40a9e2ef, 34 },
    { 0, 0 },
    { 0x1539e2f1, 61 },
    { 0xa9f9e2f2, 55 },
    { 0x6c79e2f3, 33 },
    { 0xb949e2f4, 91 },
    { 0, 0 },
    { 0xd599e2f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7329e2ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa479e303, 33 },
    { 0x5849e304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79e30a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf359e310, 90 },
    { 0xc6c9e310, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa459e314, 90 },
    { 0x9d49e315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e9e318, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xced9e31b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d39e31e, 76 },
    { 0, 0 },
    { 0x6ad9e320, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e9e324, 90 },
    { 0x8d49e325, 67 },
    { 0, 0 },
    { 0xdb39e327, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc9e32b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99e333, 55 },
    { 0, 0 },
    { 0x2f89e335, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b9e338, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f49e341, 33 },
    { 0x6d19e342, 90 },
    { 0, 0 },
    { 0x4e09e344, 98 },
    { 0, 0 },
    { 0xeaf9e346, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9019e34e, 61 },
    { 0x4e99e34f, 23 },
    { 0xa3c9e350, 46 },
    { 0xcc99e350, 55 },
    { 0x5e79e352, 40 },
    { 0x5e09e353, 90 },
    { 0xdfc9e354, 90 },
    { 0, 0 },
    { 0xa49e356, 76 },
    { 0xd599e356, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9e35b, 78 },
    { 0x43c9e35c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9689e365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3609e375, 23 },
    { 0, 0 },
    { 0xa189e377, 91 },
    { 0, 0 },
    { 0xd5e9e379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfca9e37c, 85 },
    { 0x2909e37c, 90 },
    { 0x129e37e, 61 },
    { 0x98c9e37f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d59e38c, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a9e38f, 34 },
    { 0, 0 },
    { 0xcc79e391, 90 },
    { 0x4219e392, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab9e396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9879e39f, 36 },
    { 0xcd9e3a0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd419e3a4, 85 },
    { 0xc1b9e3a5, 98 },
    { 0x3199e3a6, 90 },
    { 0xbc09e3a6, 61 },
    { 0, 0 },
    { 0x6f39e3a9, 90 },
    { 0xf0c9e3aa, 94 },
    { 0xc819e3a9, 77 },
    { 0x2899e3ac, 100 },
    { 0xae89e3ac, 90 },
    { 0xca09e3ad, 95 },
    { 0x8f79e3ad, 34 },
    { 0x7389e3b0, 33 },
    { 0, 0 },
    { 0xe639e3b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc909e3b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfc9e3b9, 87 },
    { 0, 0 },
    { 0xc179e3bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e09e3c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc779e3c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf019e3d2, 46 },
    { 0, 0 },
    { 0x58a9e3d4, 34 },
    { 0x6a99e3d5, 61 },
    { 0x7fd9e3d6, 49 },
    { 0, 0 },
    { 0x2ba9e3d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b9e3df, 40 },
    { 0, 0 },
    { 0x65e9e3e1, 87 },
    { 0xda79e3e2, 93 },
    { 0xecf9e3e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae9e3eb, 81 },
    { 0xe3d9e3ec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b69e3ef, 61 },
    { 0, 0 },
    { 0x8299e3f1, 98 },
    { 0x759e3f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3939e3f9, 90 },
    { 0x5719e3fa, 53 },
    { 0x12f9e3fb, 85 },
    { 0x54f9e3fb, 23 },
    { 0x6be9e3fd, 98 },
    { 0xe49e3fc, 53 },
    { 0, 0 },
    { 0x15d9e400, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a19e403, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e9e407, 33 },
    { 0xaec9e408, 23 },
    { 0x4a09e409, 99 },
    { 0, 0 },
    { 0x3f59e40b, 55 },
    { 0xf6e9e40b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x779e40f, 75 },
    { 0x5469e40f, 98 },
    { 0, 0 },
    { 0x10c9e412, 94 },
    { 0xee19e412, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b49e41a, 55 },
    { 0xa69e41b, 93 },
    { 0xeee9e41c, 98 },
    { 0x6a49e41d, 38 },
    { 0x1569e41e, 33 },
    { 0xc999e41f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e89e424, 90 },
    { 0x4699e425, 98 },
    { 0xfdd9e426, 52 },
    { 0x73e9e427, 95 },
    { 0xf69e425, 23 },
    { 0xb2e9e429, 23 },
    { 0x2d29e429, 17 },
    { 0x23b9e42b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce9e430, 76 },
    { 0xd769e430, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec09e434, 63 },
    { 0x40f9e435, 90 },
    { 0, 0 },
    { 0x3ec9e437, 75 },
    { 0xbfc9e438, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f9e445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3899e448, 66 },
    { 0x3f69e449, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8609e450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e455, 91 },
    { 0xc59e455, 23 },
    { 0xd359e457, 95 },
    { 0, 0 },
    { 0x2da9e459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec9e45c, 90 },
    { 0x3c09e45c, 88 },
    { 0, 0 },
    { 0xe099e45f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7629e464, 95 },
    { 0x4fc9e465, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd559e46a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9729e46d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d9e472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8859e475, 76 },
    { 0, 0 },
    { 0x10d9e477, 33 },
    { 0x6f39e478, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe59e47f, 95 },
    { 0, 0 },
    { 0x6b29e481, 43 },
    { 0x3ec9e482, 52 },
    { 0xd0a9e483, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24a9e486, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e29e48b, 52 },
    { 0x11f9e48c, 97 },
    { 0xef99e48c, 61 },
    { 0x32c9e48e, 61 },
    { 0x8e09e48f, 90 },
    { 0xdf99e48d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd119e495, 76 },
    { 0, 0 },
    { 0xb299e497, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb899e49d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c49e4a0, 61 },
    { 0x5c59e4a1, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5109e4a5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b9e4a9, 90 },
    { 0x4449e4a9, 99 },
    { 0, 0 },
    { 0xfbd9e4ac, 55 },
    { 0, 0 },
    { 0xbe9e4ae, 40 },
    { 0x79b9e4af, 98 },
    { 0x5199e4b0, 43 },
    { 0xe949e4ae, 76 },
    { 0xa6b9e4b2, 23 },
    { 0x6949e4b3, 33 },
    { 0, 0 },
    { 0x52e9e4b5, 78 },
    { 0, 0 },
    { 0x9209e4b7, 43 },
    { 0xb569e4b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4299e4be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6709e4c1, 69 },
    { 0, 0 },
    { 0x4d79e4c3, 90 },
    { 0, 0 },
    { 0x2d69e4c5, 91 },
    { 0, 0 },
    { 0x5c59e4c7, 23 },
    { 0xa7a9e4c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d49e4cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f9e4d1, 76 },
    { 0x109e4d1, 90 },
    { 0xf249e4d1, 99 },
    { 0x4059e4d3, 66 },
    { 0x27d9e4d5, 98 },
    { 0x9759e4d5, 23 },
    { 0x6bd9e4d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa669e4df, 100 },
    { 0x2489e4e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e4e6, 34 },
    { 0x63d9e4e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a9e4eb, 90 },
    { 0xb779e4eb, 52 },
    { 0xb479e4ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6099e4f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9e4f4, 87 },
    { 0, 0 },
    { 0x5549e4f6, 76 },
    { 0x5da9e4f7, 90 },
    { 0, 0 },
    { 0x5af9e4f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab89e501, 83 },
    { 0x1469e501, 61 },
    { 0x39c9e502, 80 },
    { 0x2ef9e502, 17 },
    { 0xdc79e505, 90 },
    { 0x5aa9e506, 90 },
    { 0xb789e507, 61 },
    { 0xc159e508, 100 },
    { 0xa549e502, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d9e50c, 95 },
    { 0, 0 },
    { 0x61c9e50e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed9e513, 61 },
    { 0, 0 },
    { 0xe519e515, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c19e51a, 95 },
    { 0x8a39e51b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e9e51f, 90 },
    { 0, 0 },
    { 0x8279e521, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64e9e527, 88 },
    { 0, 0 },
    { 0x219e529, 76 },
    { 0x8549e529, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e530, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b09e533, 99 },
    { 0xdf39e533, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9559e537, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379e53a, 33 },
    { 0xe089e53a, 43 },
    { 0x2409e53b, 86 },
    { 0x6229e53a, 66 },
    { 0x8f69e53d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39e542, 80 },
    { 0x9db9e543, 77 },
    { 0xaf99e544, 100 },
    { 0, 0 },
    { 0x4db9e546, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc569e549, 90 },
    { 0xda09e54a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc9e54e, 95 },
    { 0, 0 },
    { 0x3ca9e550, 84 },
    { 0x20d9e551, 23 },
    { 0xf839e550, 23 },
    { 0x9019e550, 33 },
    { 0x6f89e554, 76 },
    { 0x96c9e555, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb09e55b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa9e561, 98 },
    { 0x6619e562, 55 },
    { 0, 0 },
    { 0x6769e564, 40 },
    { 0x7c49e565, 95 },
    { 0x9129e566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1299e56d, 23 },
    { 0x30c9e56e, 90 },
    { 0x4429e56d, 76 },
    { 0xd049e570, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb9e577, 33 },
    { 0x7059e578, 34 },
    { 0x4c9e579, 34 },
    { 0, 0 },
    { 0xb0c9e57b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b9e583, 56 },
    { 0, 0 },
    { 0x1cf9e585, 91 },
    { 0x7a49e586, 61 },
    { 0, 0 },
    { 0x5469e588, 91 },
    { 0x4299e589, 95 },
    { 0, 0 },
    { 0x1ee9e58b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6809e593, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x19e9e596, 76 },
    { 0xeb49e597, 17 },
    { 0x77c9e598, 98 },
    { 0x2679e599, 52 },
    { 0xfba9e599, 95 },
    { 0x849e598, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f59e59e, 90 },
    { 0x89c9e59f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac9e5a4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49e5a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64a9e5ab, 23 },
    { 0x9bf9e5ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac79e5b6, 69 },
    { 0, 0 },
    { 0x8e9e5b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5989e5bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f9e5be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc9e5c1, 98 },
    { 0x6749e5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9e5c6, 33 },
    { 0x2759e5c7, 61 },
    { 0xdd99e5c7, 61 },
    { 0xd859e5c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe449e5cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f89e5d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c89e5d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9e5db, 90 },
    { 0x5ee9e5dc, 68 },
    { 0x2149e5dc, 33 },
    { 0, 0 },
    { 0x9bb9e5df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x479e5e2, 81 },
    { 0xeda9e5e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe9e5e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa9e5ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a9e5f0, 23 },
    { 0, 0 },
    { 0x1149e5f2, 76 },
    { 0xcca9e5f3, 90 },
    { 0, 0 },
    { 0x7bf9e5f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb79e601, 93 },
    { 0x7719e602, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3a9e607, 90 },
    { 0, 0 },
    { 0xc0b9e609, 33 },
    { 0x1b9e60a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d29e60e, 76 },
    { 0, 0 },
    { 0x2bd9e610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9e614, 23 },
    { 0x3fc9e615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4879e619, 33 },
    { 0x3aa9e61a, 33 },
    { 0x3cd9e61b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa339e61e, 90 },
    { 0xd59e61f, 61 },
    { 0x8009e61f, 53 },
    { 0x7cd9e621, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2039e624, 43 },
    { 0xec89e625, 91 },
    { 0x8bb9e624, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb069e62a, 90 },
    { 0, 0 },
    { 0x1329e62c, 99 },
    { 0x7f39e62d, 90 },
    { 0, 0 },
    { 0xb269e62f, 95 },
    { 0xe669e630, 98 },
    { 0xce49e62f, 40 },
    { 0xdad9e631, 34 },
    { 0x5299e633, 43 },
    { 0xe39e634, 77 },
    { 0xfaa9e633, 67 },
    { 0xd829e630, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4569e63e, 23 },
    { 0xd769e63f, 61 },
    { 0x259e63e, 61 },
    { 0xbc69e63e, 23 },
    { 0x19d9e63e, 61 },
    { 0xc5d9e643, 23 },
    { 0x409e644, 55 },
    { 0x2a39e645, 46 },
    { 0xd7a9e646, 91 },
    { 0xe749e647, 43 },
    { 0, 0 },
    { 0xe059e649, 61 },
    { 0x3279e64a, 95 },
    { 0x55b9e64a, 95 },
    { 0x4499e64a, 90 },
    { 0x1479e64a, 91 },
    { 0x819e649, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1479e652, 61 },
    { 0x38f9e653, 33 },
    { 0x4329e654, 33 },
    { 0x2ab9e652, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9639e65b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a9e660, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf589e664, 33 },
    { 0x64b9e665, 23 },
    { 0x69e9e665, 87 },
    { 0x7de9e665, 98 },
    { 0xebe9e668, 76 },
    { 0xe4a9e668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc659e675, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67e9e67a, 91 },
    { 0x70f9e67b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a9e67e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b9e681, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9e684, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3619e68a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9489e68d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a29e690, 17 },
    { 0x1ab9e690, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6369e696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc9e69e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf49e6a4, 3 },
    { 0x29e9e6a5, 95 },
    { 0xf7c9e6a5, 100 },
    { 0xbe9e6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8719e6af, 90 },
    { 0x2989e6b0, 33 },
    { 0xe589e6b0, 93 },
    { 0x1a49e6b2, 77 },
    { 0x4209e6b3, 76 },
    { 0x2b69e6b3, 95 },
    { 0x3fa9e6b4, 90 },
    { 0x42a9e6b6, 91 },
    { 0xe009e6b6, 85 },
    { 0, 0 },
    { 0x9219e6b9, 61 },
    { 0x8479e6b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f9e6be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb9e6c3, 61 },
    { 0xcec9e6c4, 98 },
    { 0x2399e6c5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e19e6c9, 98 },
    { 0xd469e6c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5849e6cd, 82 },
    { 0x6ee9e6ce, 90 },
    { 0x79e9e6cf, 61 },
    { 0, 0 },
    { 0xa99e6d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8889e6d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c29e6d7, 33 },
    { 0x23f9e6d8, 99 },
    { 0x32b9e6d9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f9e6dd, 61 },
    { 0x21c9e6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac29e6e7, 98 },
    { 0xbf29e6e8, 33 },
    { 0, 0 },
    { 0xe4c9e6ea, 61 },
    { 0x88e9e6eb, 90 },
    { 0x1989e6ec, 95 },
    { 0x6869e6ed, 33 },
    { 0x4529e6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d9e6f5, 23 },
    { 0x2a9e6f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa109e6fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc9e6ff, 90 },
    { 0xa909e700, 61 },
    { 0x3e39e700, 55 },
    { 0xa4d9e702, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c49e70a, 61 },
    { 0x4039e70b, 90 },
    { 0, 0 },
    { 0xec29e70d, 46 },
    { 0x38e9e70d, 55 },
    { 0xb709e70e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b89e712, 52 },
    { 0x8919e713, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d9e716, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5549e71b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e9e71e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f89e722, 33 },
    { 0xb959e723, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a9e72a, 100 },
    { 0x3bf9e72b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44c9e72f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8809e734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1919e737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b09e73b, 40 },
    { 0, 0 },
    { 0x6c9e73d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8169e740, 98 },
    { 0, 0 },
    { 0x70d9e742, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e9e747, 95 },
    { 0xd989e747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9e74b, 81 },
    { 0x1769e74c, 95 },
    { 0xbe59e74d, 55 },
    { 0x8cc9e74e, 95 },
    { 0x3329e74f, 95 },
    { 0x2c59e750, 23 },
    { 0xa379e74e, 95 },
    { 0x1b09e752, 98 },
    { 0xc9b9e753, 90 },
    { 0xf029e754, 66 },
    { 0xd299e74c, 33 },
    { 0xc209e74d, 33 },
    { 0xf649e757, 81 },
    { 0xb179e758, 34 },
    { 0, 0 },
    { 0x7599e75a, 69 },
    { 0, 0 },
    { 0x7d89e75c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3df9e765, 69 },
    { 0x4419e765, 98 },
    { 0xe7e9e767, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c59e76e, 55 },
    { 0x5df9e76f, 76 },
    { 0, 0 },
    { 0x71a9e771, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b9e776, 90 },
    { 0, 0 },
    { 0x3a89e778, 23 },
    { 0x67c9e779, 43 },
    { 0, 0 },
    { 0x18a9e77b, 23 },
    { 0x8fe9e77c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc359e785, 66 },
    { 0, 0 },
    { 0x2f39e787, 88 },
    { 0x7a9e787, 98 },
    { 0x4cc9e789, 65 },
    { 0xa529e788, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc799e790, 80 },
    { 0xb3e9e791, 43 },
    { 0x2b69e791, 23 },
    { 0x98a9e790, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ef9e7a4, 88 },
    { 0, 0 },
    { 0xb939e7a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9e7ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b49e7b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779e7b6, 55 },
    { 0x319e7b7, 95 },
    { 0x5679e7b7, 76 },
    { 0, 0 },
    { 0x5699e7ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e9e7bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f09e7c2, 61 },
    { 0x1ca9e7c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe979e7c9, 78 },
    { 0x5739e7c9, 99 },
    { 0xe7f9e7cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab39e7cf, 33 },
    { 0x41b9e7d0, 33 },
    { 0xca89e7d1, 33 },
    { 0, 0 },
    { 0x6419e7d3, 23 },
    { 0x9c69e7d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d09e7dd, 95 },
    { 0xc749e7dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f79e7e2, 33 },
    { 0, 0 },
    { 0x4e09e7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa209e7e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe9e7eb, 23 },
    { 0x1ca9e7ec, 85 },
    { 0x47b9e7eb, 90 },
    { 0x4e79e7ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9e7f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d9e7f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e9e7fe, 95 },
    { 0x4519e7fe, 69 },
    { 0x93b9e800, 98 },
    { 0x7319e7ff, 90 },
    { 0x83c9e802, 76 },
    { 0xbcf9e803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79e809, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa9e80e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9e811, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1239e814, 33 },
    { 0x9349e814, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6649e81a, 33 },
    { 0xc0e9e81b, 33 },
    { 0xcdf9e81c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8769e823, 23 },
    { 0, 0 },
    { 0x9699e825, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf69e828, 33 },
    { 0x849e829, 98 },
    { 0, 0 },
    { 0x7dd9e82b, 46 },
    { 0x3db9e82c, 95 },
    { 0, 0 },
    { 0x389e82e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e69e831, 23 },
    { 0x1219e831, 98 },
    { 0x2749e831, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd719e837, 93 },
    { 0x7789e837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e09e842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe959e847, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8199e84f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c9e852, 23 },
    { 0xcb9e852, 98 },
    { 0x9e854, 67 },
    { 0x6179e855, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e59e85b, 33 },
    { 0x2309e85c, 61 },
    { 0x6c9e85d, 52 },
    { 0, 0 },
    { 0x2ea9e85f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6d9e863, 98 },
    { 0x7ba9e864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3299e86c, 99 },
    { 0x5cb9e86d, 81 },
    { 0xfb29e86d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d9e873, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1249e879, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d29e87e, 33 },
    { 0, 0 },
    { 0x3cd9e880, 90 },
    { 0xa019e881, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3749e886, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa029e88c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7749e88f, 76 },
    { 0xc469e890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a69e893, 61 },
    { 0, 0 },
    { 0xdec9e895, 61 },
    { 0xfdc9e896, 99 },
    { 0x4b69e897, 61 },
    { 0xb939e897, 93 },
    { 0x8959e898, 23 },
    { 0x66b9e89a, 95 },
    { 0x5709e898, 61 },
    { 0xafe9e899, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaf9e8a6, 40 },
    { 0, 0 },
    { 0xc199e8a8, 23 },
    { 0xaef9e8a9, 88 },
    { 0x3f09e8a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5879e8ae, 95 },
    { 0xba9e8af, 90 },
    { 0, 0 },
    { 0xf999e8b1, 95 },
    { 0x5049e8b2, 23 },
    { 0xaf69e8b2, 61 },
    { 0xebf9e8b4, 75 },
    { 0xb289e8b4, 95 },
    { 0x78c9e8b6, 81 },
    { 0, 0 },
    { 0x56a9e8b8, 78 },
    { 0xa0f9e8b9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf929e8bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729e8c3, 33 },
    { 0xd859e8c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e9e8ca, 23 },
    { 0, 0 },
    { 0xc89e8cc, 90 },
    { 0xee89e8cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a69e8d0, 88 },
    { 0xaf19e8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a49e8d5, 95 },
    { 0x3a89e8d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a49e8da, 61 },
    { 0xf549e8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f69e8de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc709e8e1, 52 },
    { 0x5c69e8e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fb9e8e5, 33 },
    { 0x7b9e8e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38a9e8eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e29e8f5, 55 },
    { 0xd6c9e8f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce9e8f9, 99 },
    { 0x9c89e8f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf249e8fd, 91 },
    { 0x4f89e8fe, 95 },
    { 0xfce9e8ff, 17 },
    { 0x65a9e8ff, 90 },
    { 0xc99e901, 76 },
    { 0xd769e901, 33 },
    { 0, 0 },
    { 0xc789e904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb199e907, 61 },
    { 0xe829e907, 100 },
    { 0x8d29e909, 98 },
    { 0, 0 },
    { 0xa919e90b, 61 },
    { 0xdd69e90b, 55 },
    { 0x1929e90c, 61 },
    { 0x98a9e90b, 95 },
    { 0x2d79e90f, 91 },
    { 0xe4a9e90c, 55 },
    { 0x8bb9e911, 76 },
    { 0xe419e912, 100 },
    { 0xbf99e912, 55 },
    { 0xc9a9e912, 90 },
    { 0, 0 },
    { 0xf2d9e916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1249e91d, 95 },
    { 0, 0 },
    { 0xdfc9e91f, 85 },
    { 0, 0 },
    { 0xe789e921, 98 },
    { 0, 0 },
    { 0x7669e923, 76 },
    { 0, 0 },
    { 0x1149e925, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47b9e928, 43 },
    { 0, 0 },
    { 0x6959e92a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe949e92e, 23 },
    { 0xe699e92f, 33 },
    { 0x37e9e930, 33 },
    { 0x6e59e931, 34 },
    { 0xc479e931, 90 },
    { 0, 0 },
    { 0x33a9e934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9e937, 51 },
    { 0x1339e938, 76 },
    { 0x6ca9e937, 61 },
    { 0xd599e939, 98 },
    { 0x5eb9e938, 55 },
    { 0xb1d9e93c, 33 },
    { 0xc9e9e93d, 76 },
    { 0x5919e938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f09e94b, 23 },
    { 0xfd19e94b, 98 },
    { 0x5799e94b, 53 },
    { 0x4029e94e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd29e951, 98 },
    { 0x9a69e951, 33 },
    { 0x4c79e951, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7d9e957, 76 },
    { 0x5249e958, 90 },
    { 0x3c59e958, 90 },
    { 0xce99e95a, 90 },
    { 0x80a9e95b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9359e964, 93 },
    { 0xc0d9e964, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd9e96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd189e96f, 77 },
    { 0, 0 },
    { 0x4979e971, 90 },
    { 0x9999e972, 90 },
    { 0x2379e973, 91 },
    { 0x8d89e973, 83 },
    { 0x3679e975, 23 },
    { 0xcdb9e975, 33 },
    { 0, 0 },
    { 0x4359e978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2319e97f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7999e983, 68 },
    { 0xf7e9e983, 33 },
    { 0xbf59e985, 61 },
    { 0x7c29e986, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdd9e98e, 77 },
    { 0x1f29e98f, 95 },
    { 0xab19e98e, 90 },
    { 0xc3e9e98e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab19e99d, 93 },
    { 0x5729e99e, 68 },
    { 0x94b9e99e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa9e9a2, 55 },
    { 0, 0 },
    { 0x18c9e9a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2059e9aa, 61 },
    { 0x2849e9ab, 61 },
    { 0xf029e9ac, 84 },
    { 0, 0 },
    { 0x4289e9ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b9e9b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4499e9b9, 33 },
    { 0xcac9e9ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a9e9c2, 43 },
    { 0, 0 },
    { 0xd139e9c4, 76 },
    { 0, 0 },
    { 0x2f39e9c6, 23 },
    { 0x3439e9c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9e9cb, 91 },
    { 0x35a9e9cc, 55 },
    { 0x5e09e9cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2169e9d3, 33 },
    { 0x2849e9d3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f89e9da, 91 },
    { 0x6329e9da, 66 },
    { 0xe349e9dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e59e9e5, 98 },
    { 0x6839e9e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9e9ef, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7199e9f3, 61 },
    { 0x1799e9f4, 77 },
    { 0x2159e9f3, 95 },
    { 0x9769e9f3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc209e9fd, 98 },
    { 0x99b9e9fe, 66 },
    { 0x1ac9e9ff, 75 },
    { 0x1bf9e9fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2459ea04, 65 },
    { 0x9d79ea05, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bc9ea08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa829ea0b, 61 },
    { 0xaa49ea0c, 33 },
    { 0, 0 },
    { 0x9a89ea0e, 46 },
    { 0xc579ea0f, 36 },
    { 0xc749ea0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e39ea1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b9ea1e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c9ea22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf09ea25, 61 },
    { 0, 0 },
    { 0xe3c9ea27, 43 },
    { 0x1509ea28, 33 },
    { 0, 0 },
    { 0xd619ea2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf259ea2e, 85 },
    { 0x1c79ea2f, 46 },
    { 0xdc39ea30, 55 },
    { 0xf7f9ea2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c79ea36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce09ea3a, 61 },
    { 0xd9e9ea3a, 23 },
    { 0x18a9ea3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459ea3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729ea46, 81 },
    { 0x8149ea46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d59ea4b, 91 },
    { 0x39a9ea4c, 83 },
    { 0xe5f9ea4d, 95 },
    { 0x9019ea4e, 61 },
    { 0xe929ea4e, 90 },
    { 0xd459ea50, 87 },
    { 0xa9a9ea51, 23 },
    { 0x56f9ea52, 33 },
    { 0x2e79ea53, 91 },
    { 0x8d49ea53, 46 },
    { 0x6f89ea55, 61 },
    { 0x79d9ea56, 34 },
    { 0xb729ea52, 55 },
    { 0, 0 },
    { 0xa1f9ea59, 98 },
    { 0xd689ea5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14b9ea61, 61 },
    { 0x5229ea61, 95 },
    { 0, 0 },
    { 0xd279ea64, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xccf9ea67, 90 },
    { 0x25b9ea68, 95 },
    { 0, 0 },
    { 0x64e9ea6a, 95 },
    { 0x44e9ea6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b9ea6f, 76 },
    { 0x7d79ea6f, 98 },
    { 0xa039ea71, 23 },
    { 0x5c69ea6f, 78 },
    { 0, 0 },
    { 0xb3e9ea74, 33 },
    { 0x4659ea75, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f79ea81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e9ea87, 100 },
    { 0xb789ea88, 90 },
    { 0, 0 },
    { 0x46a9ea8a, 55 },
    { 0x9829ea8b, 100 },
    { 0xd0b9ea8c, 76 },
    { 0xe669ea8c, 33 },
    { 0x7099ea8a, 23 },
    { 0x2689ea8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b9ea93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1159ea96, 90 },
    { 0xa159ea96, 76 },
    { 0x7a29ea98, 66 },
    { 0, 0 },
    { 0x72e9ea9a, 76 },
    { 0x7259ea9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa739eaa5, 93 },
    { 0, 0 },
    { 0xa9eaa7, 69 },
    { 0, 0 },
    { 0x4a9eaa9, 76 },
    { 0, 0 },
    { 0x8a99eaab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc769eab3, 99 },
    { 0, 0 },
    { 0x66e9eab5, 95 },
    { 0xd879eab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9eab9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c9eabf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe039eac7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8629eacb, 95 },
    { 0x93f9eacc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd409ead3, 90 },
    { 0x35d9ead3, 98 },
    { 0x50a9ead5, 61 },
    { 0, 0 },
    { 0x7a89ead7, 90 },
    { 0, 0 },
    { 0xd229ead9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8709eadd, 67 },
    { 0xe19eade, 40 },
    { 0x979eadf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe869eae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8af9eae7, 49 },
    { 0x78a9eae8, 23 },
    { 0x2d59eae9, 98 },
    { 0, 0 },
    { 0xa569eaeb, 33 },
    { 0xe3d9eaeb, 33 },
    { 0, 0 },
    { 0xb859eaee, 23 },
    { 0x1a9eaee, 98 },
    { 0xd759eaf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5479eaf3, 61 },
    { 0x4869eaf3, 61 },
    { 0xbf9eaf4, 23 },
    { 0xdb49eaf6, 88 },
    { 0x9cd9eaf4, 33 },
    { 0, 0 },
    { 0x1d89eaf9, 52 },
    { 0x52c9eafa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76a9eb00, 89 },
    { 0, 0 },
    { 0x6d89eb02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bd9eb05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa699eb08, 90 },
    { 0x2659eb09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4089eb0d, 91 },
    { 0x7a69eb0d, 90 },
    { 0xb7b9eb0f, 98 },
    { 0x6469eb0e, 90 },
    { 0x4cd9eb11, 98 },
    { 0xca59eb12, 43 },
    { 0x21b9eb12, 91 },
    { 0x66f9eb14, 91 },
    { 0x3f79eb15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3919eb1b, 98 },
    { 0xd6b9eb1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c59eb1f, 77 },
    { 0x9a49eb20, 93 },
    { 0x9519eb20, 23 },
    { 0xb7b9eb22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109eb28, 90 },
    { 0xdbe9eb29, 90 },
    { 0xe519eb29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d9eb30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc909eb35, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b59eb38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e9eb3c, 62 },
    { 0xaa49eb3c, 66 },
    { 0x1b29eb3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed89eb42, 86 },
    { 0x2d29eb43, 43 },
    { 0x29e9eb43, 95 },
    { 0xad9eb45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d69eb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6559eb4f, 61 },
    { 0xaeb9eb4f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9eb57, 85 },
    { 0xab89eb58, 34 },
    { 0, 0 },
    { 0x38a9eb5a, 23 },
    { 0, 0 },
    { 0x75e9eb5c, 90 },
    { 0x84a9eb5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90f9eb62, 17 },
    { 0x1039eb63, 95 },
    { 0, 0 },
    { 0xcdd9eb65, 90 },
    { 0xd349eb65, 98 },
    { 0x9329eb66, 61 },
    { 0xad59eb67, 61 },
    { 0, 0 },
    { 0x9e49eb6a, 33 },
    { 0xd089eb6b, 63 },
    { 0x5a09eb6c, 88 },
    { 0x8c29eb6d, 88 },
    { 0x3249eb6a, 34 },
    { 0x7bd9eb6f, 99 },
    { 0x4129eb70, 52 },
    { 0, 0 },
    { 0x4b69eb72, 98 },
    { 0x4da9eb72, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6559eb77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6e9eb7d, 43 },
    { 0xe3b9eb7e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf19eb81, 90 },
    { 0xbcb9eb82, 98 },
    { 0x73f9eb82, 90 },
    { 0x59a9eb84, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7189eb87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc9eb8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a9eb91, 99 },
    { 0xee9eb91, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc679eb98, 78 },
    { 0, 0 },
    { 0x35a9eb9a, 36 },
    { 0xc979eb9b, 34 },
    { 0x61c9eb9c, 98 },
    { 0x7be9eb9d, 61 },
    { 0, 0 },
    { 0x169eb9f, 87 },
    { 0xa099eb9f, 99 },
    { 0xe189eba0, 76 },
    { 0xdbc9eba2, 34 },
    { 0xd139eba3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae9ebb0, 95 },
    { 0, 0 },
    { 0x5829ebb2, 33 },
    { 0xd989ebb2, 98 },
    { 0xc419ebb2, 95 },
    { 0, 0 },
    { 0x6219ebb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da9ebb9, 85 },
    { 0, 0 },
    { 0x7d9ebbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb689ebbf, 90 },
    { 0xfb09ebc0, 98 },
    { 0xfcb9ebc1, 55 },
    { 0x3c79ebc0, 32 },
    { 0xb5b9ebc3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19ebc8, 90 },
    { 0x8a19ebc9, 52 },
    { 0xc329ebc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe59ebcd, 33 },
    { 0xfad9ebce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf989ebd3, 66 },
    { 0x14f9ebd4, 90 },
    { 0xb4c9ebd4, 61 },
    { 0xe879ebd6, 87 },
    { 0xb389ebd5, 85 },
    { 0x1d49ebd4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269ebdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb449ebe1, 33 },
    { 0, 0 },
    { 0x21c9ebe3, 78 },
    { 0, 0 },
    { 0x8e79ebe5, 87 },
    { 0xe579ebe5, 66 },
    { 0x30e9ebe7, 85 },
    { 0xf239ebe7, 95 },
    { 0x1bb9ebe8, 98 },
    { 0xc529ebea, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9ebee, 33 },
    { 0x3669ebee, 61 },
    { 0xfbd9ebee, 98 },
    { 0x3709ebf1, 61 },
    { 0x6f69ebf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9559ebf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7459ebfa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be9ebfd, 61 },
    { 0x4109ebfe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdde9ec05, 33 },
    { 0x9ca9ec06, 55 },
    { 0, 0 },
    { 0x2449ec08, 40 },
    { 0xd889ec09, 23 },
    { 0, 0 },
    { 0xda49ec0b, 33 },
    { 0x7679ec0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf439ec11, 43 },
    { 0x57d9ec12, 33 },
    { 0, 0 },
    { 0xf669ec14, 90 },
    { 0x72d9ec14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb379ec19, 23 },
    { 0, 0 },
    { 0x7899ec1b, 77 },
    { 0x72a9ec1c, 99 },
    { 0xb9e9ec1d, 95 },
    { 0x16b9ec1e, 23 },
    { 0xf879ec1f, 100 },
    { 0x3a79ec20, 66 },
    { 0x2919ec21, 66 },
    { 0xf819ec22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdb9ec29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d9ec2d, 33 },
    { 0x8e89ec2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ec31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9249ec36, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc49ec3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad69ec41, 90 },
    { 0, 0 },
    { 0xc3c9ec43, 23 },
    { 0x5999ec43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5989ec48, 98 },
    { 0xff09ec49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ec4d, 91 },
    { 0x7f89ec4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b9ec51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53f9ec59, 100 },
    { 0, 0 },
    { 0xa289ec5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a9ec60, 34 },
    { 0xe749ec61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209ec66, 99 },
    { 0x8ed9ec66, 23 },
    { 0xe699ec66, 33 },
    { 0xea19ec68, 23 },
    { 0, 0 },
    { 0x9c39ec6b, 69 },
    { 0xa49ec6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc19ec6f, 33 },
    { 0, 0 },
    { 0x9dd9ec71, 33 },
    { 0, 0 },
    { 0x14e9ec73, 91 },
    { 0xbfe9ec74, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d69ec77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c29ec7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf49ec7e, 80 },
    { 0xa119ec7e, 99 },
    { 0x9309ec7e, 95 },
    { 0xcab9ec7f, 61 },
    { 0xf389ec80, 61 },
    { 0x9f79ec82, 67 },
    { 0x4b49ec84, 33 },
    { 0xca49ec85, 33 },
    { 0, 0 },
    { 0x339ec87, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9959ec8c, 55 },
    { 0x269ec8d, 90 },
    { 0xc2d9ec8d, 17 },
    { 0x3659ec8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe149ec92, 43 },
    { 0xbce9ec93, 100 },
    { 0xd299ec94, 100 },
    { 0, 0 },
    { 0x62c9ec96, 33 },
    { 0xb9d9ec96, 34 },
    { 0, 0 },
    { 0xcf99ec99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb119ec9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7259ec9f, 46 },
    { 0, 0 },
    { 0x6939eca1, 100 },
    { 0, 0 },
    { 0xe189eca3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b9eca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1929eca9, 80 },
    { 0x4099eca9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b29ecae, 61 },
    { 0, 0 },
    { 0x1c09ecb0, 46 },
    { 0xebf9ecb1, 76 },
    { 0, 0 },
    { 0x2369ecb3, 93 },
    { 0x1159ecb4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d69ecbb, 23 },
    { 0xd629ecbc, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x9669ecbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b09ecc2, 90 },
    { 0xf589ecc3, 81 },
    { 0xc499ecc3, 46 },
    { 0xc539ecc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5099ecca, 95 },
    { 0, 0 },
    { 0xb269eccc, 98 },
    { 0x4319eccd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e9ecd5, 75 },
    { 0xbda9ecd6, 95 },
    { 0, 0 },
    { 0x6b29ecd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b99ece0, 65 },
    { 0, 0 },
    { 0x8609ece2, 61 },
    { 0, 0 },
    { 0x2c09ece4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e9eceb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6929ecee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9369ecf4, 76 },
    { 0x2fc9ecf5, 59 },
    { 0x3c59ecf6, 76 },
    { 0x4d89ecf6, 61 },
    { 0xc309ecf8, 64 },
    { 0x45f9ecf4, 95 },
    { 0x50e9ecf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd49ed0d, 98 },
    { 0xa239ed0e, 33 },
    { 0xb2c9ed0f, 68 },
    { 0x2c29ed0f, 91 },
    { 0x4a9ed0e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6519ed1a, 75 },
    { 0x2639ed1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db9ed1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc09ed22, 95 },
    { 0xf809ed23, 84 },
    { 0, 0 },
    { 0xca49ed25, 98 },
    { 0xdd9ed25, 90 },
    { 0x3089ed27, 66 },
    { 0x27e9ed25, 55 },
    { 0x34e9ed29, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6099ed2c, 95 },
    { 0xeaf9ed2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f9ed31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe059ed34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef9ed37, 66 },
    { 0x3d09ed37, 34 },
    { 0, 0 },
    { 0xe3f9ed3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc9ed40, 90 },
    { 0, 0 },
    { 0xf429ed42, 99 },
    { 0xf59ed43, 46 },
    { 0x349ed44, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d9ed4d, 52 },
    { 0x9569ed4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ed53, 61 },
    { 0x6ef9ed54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec09ed57, 34 },
    { 0x2b69ed58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9929ed5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7349ed5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2139ed63, 96 },
    { 0, 0 },
    { 0xf5b9ed65, 23 },
    { 0x8ba9ed66, 55 },
    { 0x7929ed67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c9ed6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d9ed6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa719ed73, 91 },
    { 0xd1d9ed74, 55 },
    { 0x9f59ed75, 33 },
    { 0, 0 },
    { 0xa489ed77, 90 },
    { 0, 0 },
    { 0x7ab9ed79, 55 },
    { 0x7d29ed7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5439ed7e, 91 },
    { 0, 0 },
    { 0xb679ed80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb9ed85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5709ed88, 90 },
    { 0x6999ed89, 81 },
    { 0xf799ed8a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4479ed90, 87 },
    { 0x5699ed91, 93 },
    { 0xd229ed92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c89ed99, 90 },
    { 0xe119ed9a, 91 },
    { 0, 0 },
    { 0x3579ed9c, 52 },
    { 0x2d9ed9d, 61 },
    { 0x1a39ed9e, 23 },
    { 0xb759ed9f, 34 },
    { 0xb719eda0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a29eda9, 78 },
    { 0, 0 },
    { 0xcd39edab, 33 },
    { 0x89a9edac, 90 },
    { 0xee29edad, 34 },
    { 0xdfe9edad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc309edb1, 61 },
    { 0, 0 },
    { 0x5a99edb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9edc6, 33 },
    { 0xc499edc6, 24 },
    { 0x6969edc8, 78 },
    { 0xeb59edc8, 90 },
    { 0x5ce9edc9, 34 },
    { 0xf1c9edcb, 98 },
    { 0xe7e9edcb, 98 },
    { 0xaae9edcb, 76 },
    { 0x3d79edc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf9edd2, 81 },
    { 0, 0 },
    { 0xbe59edd4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8529edd9, 73 },
    { 0x3b59edd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2b9ede0, 34 },
    { 0x2219ede0, 51 },
    { 0, 0 },
    { 0x5e49ede3, 76 },
    { 0xd769ede4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1509eded, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe29edf4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3489edf8, 61 },
    { 0x35e9edf9, 61 },
    { 0x5179edfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb779ee05, 13 },
    { 0xf689ee06, 33 },
    { 0x3599ee05, 80 },
    { 0x10f9ee08, 34 },
    { 0xef09ee09, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7389ee0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c9ee11, 95 },
    { 0x8ca9ee11, 23 },
    { 0x9ac9ee13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c59ee16, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda69ee1a, 98 },
    { 0xd139ee1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ea9ee1e, 23 },
    { 0x2af9ee1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb9ee25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939ee2b, 34 },
    { 0x9f09ee2c, 98 },
    { 0x3d59ee2d, 33 },
    { 0xc689ee2b, 34 },
    { 0x6e29ee2d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ee32, 33 },
    { 0, 0 },
    { 0xa699ee34, 90 },
    { 0xb5d9ee35, 23 },
    { 0xea69ee34, 98 },
    { 0, 0 },
    { 0x6119ee38, 98 },
    { 0xe6e9ee39, 55 },
    { 0x4f89ee39, 61 },
    { 0x1f49ee3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4489ee3e, 55 },
    { 0xc389ee3f, 81 },
    { 0x17b9ee40, 81 },
    { 0x76e9ee40, 43 },
    { 0x8b29ee3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf19ee4a, 23 },
    { 0x1509ee4b, 75 },
    { 0xd059ee4c, 61 },
    { 0xe8b9ee4d, 90 },
    { 0x2f9ee4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee89ee55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd59ee58, 23 },
    { 0xda79ee59, 34 },
    { 0xd6e9ee5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f9ee63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3779ee6c, 88 },
    { 0, 0 },
    { 0x7c9ee6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf999ee73, 98 },
    { 0, 0 },
    { 0xca69ee75, 17 },
    { 0, 0 },
    { 0xae49ee77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6539ee7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7909ee80, 83 },
    { 0xb429ee80, 98 },
    { 0xb79ee81, 76 },
    { 0x1e49ee80, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59ee8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c9ee8f, 76 },
    { 0xc279ee8f, 65 },
    { 0x2df9ee8f, 33 },
    { 0xea29ee92, 33 },
    { 0xbf79ee8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa9ee96, 93 },
    { 0xc509ee97, 52 },
    { 0, 0 },
    { 0x7d19ee99, 85 },
    { 0x6679ee99, 90 },
    { 0xc989ee9b, 33 },
    { 0xe29ee9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ea9ee9f, 43 },
    { 0xe19eea0, 98 },
    { 0x8129ee9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c59eea4, 95 },
    { 0, 0 },
    { 0x36c9eea6, 61 },
    { 0, 0 },
    { 0x2149eea8, 46 },
    { 0x2729eea8, 34 },
    { 0, 0 },
    { 0xd8e9eeab, 100 },
    { 0xa479eeab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c49eeb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe329eeb3, 91 },
    { 0, 0 },
    { 0xf979eeb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f9eeb8, 90 },
    { 0x2dd9eeb9, 36 },
    { 0x50c9eeb9, 33 },
    { 0xae9eebb, 98 },
    { 0x6ed9eebb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b19eebf, 23 },
    { 0xd2c9eec0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda9eec4, 90 },
    { 0, 0 },
    { 0x3329eec6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c19eec9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb69eecc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f9eecf, 90 },
    { 0x2cd9eed0, 22 },
    { 0xf5c9eed1, 91 },
    { 0x66f9eed2, 81 },
    { 0, 0 },
    { 0xdaf9eed4, 33 },
    { 0xdc39eed5, 33 },
    { 0x749eed6, 35 },
    { 0x3059eed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979eee6, 76 },
    { 0, 0 },
    { 0xf529eee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac9eeed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc9eef1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74c9eef6, 83 },
    { 0xb059eef7, 33 },
    { 0x50d9eef7, 23 },
    { 0x87e9eef8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369eefe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1939ef01, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ef09, 46 },
    { 0, 0 },
    { 0xa3c9ef0b, 78 },
    { 0, 0 },
    { 0xb1e9ef0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4c9ef10, 23 },
    { 0x2f19ef11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6709ef14, 33 },
    { 0xa0a9ef15, 33 },
    { 0xc389ef16, 67 },
    { 0xdbc9ef16, 95 },
    { 0xc289ef14, 33 },
    { 0x60f9ef18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f39ef1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3529ef22, 55 },
    { 0, 0 },
    { 0xe3f9ef24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa539ef27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ed9ef2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9829ef2e, 81 },
    { 0x6219ef2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe59ef37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd299ef3e, 87 },
    { 0, 0 },
    { 0x3989ef40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd629ef44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6659ef47, 34 },
    { 0x15c9ef48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd419ef51, 90 },
    { 0x7ac9ef52, 61 },
    { 0, 0 },
    { 0x81c9ef54, 99 },
    { 0xffb9ef55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9319ef61, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc9ef64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf89ef68, 23 },
    { 0xfc69ef69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99a9ef6c, 23 },
    { 0xe7d9ef6d, 46 },
    { 0, 0 },
    { 0xfe89ef6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d9ef72, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafb9ef76, 34 },
    { 0x49ef77, 95 },
    { 0xe569ef78, 90 },
    { 0, 0 },
    { 0x3219ef7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6199ef82, 84 },
    { 0, 0 },
    { 0xdde9ef84, 95 },
    { 0, 0 },
    { 0x14e9ef86, 99 },
    { 0, 0 },
    { 0x5049ef88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8889ef8f, 55 },
    { 0x9839ef8f, 43 },
    { 0x7909ef91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7589ef94, 76 },
    { 0x65e9ef95, 55 },
    { 0xd949ef95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe669efa0, 95 },
    { 0x2409efa1, 40 },
    { 0x489efa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e9efae, 33 },
    { 0xeb09efae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7669efb2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe029efb5, 87 },
    { 0, 0 },
    { 0x2499efb7, 91 },
    { 0x5379efb8, 85 },
    { 0x8119efb9, 77 },
    { 0xa1d9efb9, 33 },
    { 0xe849efba, 95 },
    { 0xd0b9efbb, 33 },
    { 0, 0 },
    { 0x27e9efbe, 76 },
    { 0xb069efbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaf9efc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b9efc6, 55 },
    { 0x1689efc6, 23 },
    { 0xb779efc6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa9efd0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a9efd4, 33 },
    { 0x65d9efd5, 78 },
    { 0x9b89efd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a49efdc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c9efe7, 98 },
    { 0, 0 },
    { 0x9b89efe9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1149efec, 87 },
    { 0x8449efed, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1469eff4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a9effb, 98 },
    { 0x4f39effc, 61 },
    { 0xc769effc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8369f000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2769f006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf459f00a, 33 },
    { 0xd899f00a, 23 },
    { 0, 0 },
    { 0x93a9f00d, 55 },
    { 0, 0 },
    { 0x3bb9f00f, 98 },
    { 0x5f49f010, 98 },
    { 0x1ed9f011, 33 },
    { 0, 0 },
    { 0x8fc9f013, 43 },
    { 0, 0 },
    { 0x98e9f015, 98 },
    { 0x46d9f015, 76 },
    { 0xddf9f015, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9939f023, 46 },
    { 0, 0 },
    { 0x5f9f025, 43 },
    { 0, 0 },
    { 0x85a9f027, 33 },
    { 0x55e9f028, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c59f02c, 38 },
    { 0x3bb9f02c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5349f030, 99 },
    { 0, 0 },
    { 0x8c09f032, 23 },
    { 0, 0 },
    { 0xaa39f034, 23 },
    { 0, 0 },
    { 0x2599f036, 76 },
    { 0x6799f037, 90 },
    { 0x55c9f038, 98 },
    { 0x8a9f039, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc479f041, 61 },
    { 0x5bf9f042, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x2139f045, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc9f04e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e9f051, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c79f054, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83f9f058, 99 },
    { 0, 0 },
    { 0x3ad9f05a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f9f05d, 61 },
    { 0xa7e9f05e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3119f065, 93 },
    { 0x5ac9f066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac9f06d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339f070, 61 },
    { 0, 0 },
    { 0xdb79f072, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e29f078, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f09f07b, 87 },
    { 0x5499f07c, 23 },
    { 0xa989f07d, 15 },
    { 0, 0 },
    { 0x4f69f07f, 61 },
    { 0xf119f080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf519f084, 90 },
    { 0xfc09f084, 55 },
    { 0x61c9f086, 61 },
    { 0x76a9f087, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9929f08e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf099f091, 90 },
    { 0x17a9f092, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd159f096, 23 },
    { 0xf389f097, 61 },
    { 0x9a29f098, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6889f09b, 90 },
    { 0x84e9f09c, 95 },
    { 0x269f09b, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9b9f0a2, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf89f0a5, 90 },
    { 0xe5d9f0a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f9f0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a9f0ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d99f0b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc039f0b7, 55 },
    { 0, 0 },
    { 0x5b9f0b9, 80 },
    { 0xb249f0ba, 43 },
    { 0x43a9f0b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de9f0c4, 81 },
    { 0x2819f0c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f09f0c9, 99 },
    { 0x7029f0c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b9f0d1, 90 },
    { 0x8e19f0d1, 43 },
    { 0x50a9f0d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd9f0d8, 55 },
    { 0x4729f0d8, 93 },
    { 0xefe9f0da, 66 },
    { 0xf219f0db, 33 },
    { 0xb829f0db, 95 },
    { 0xdaa9f0db, 33 },
    { 0x4ab9f0de, 55 },
    { 0xbe19f0de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd869f0e3, 93 },
    { 0x4319f0e4, 34 },
    { 0, 0 },
    { 0x8ba9f0e6, 33 },
    { 0x20c9f0e7, 33 },
    { 0x1a99f0e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba49f0f0, 95 },
    { 0, 0 },
    { 0x9329f0f2, 35 },
    { 0, 0 },
    { 0xceb9f0f4, 98 },
    { 0x8199f0f5, 34 },
    { 0xe39f0f5, 23 },
    { 0xacd9f0f7, 23 },
    { 0x4e9f0f4, 23 },
    { 0x1d89f0f9, 87 },
    { 0x7119f0fa, 98 },
    { 0x88b9f0f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b9f0ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6259f104, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb89f107, 67 },
    { 0x85b9f107, 76 },
    { 0x96a9f109, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4b9f10d, 95 },
    { 0x6da9f10e, 33 },
    { 0x829f10f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc319f113, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce9f118, 51 },
    { 0x42d9f119, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd19f11d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f99f122, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f127, 87 },
    { 0xf8e9f128, 23 },
    { 0x80b9f128, 90 },
    { 0xf6d9f129, 98 },
    { 0xf39f12a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1999f12e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b9f134, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd419f13a, 91 },
    { 0xa1e9f13b, 76 },
    { 0, 0 },
    { 0x8229f13d, 100 },
    { 0x59a9f13d, 87 },
    { 0xdf19f13e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9f143, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db9f14a, 95 },
    { 0xe009f14b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2819f14e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6219f153, 55 },
    { 0x2909f153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe759f159, 46 },
    { 0x1459f15a, 90 },
    { 0xfa9f15a, 23 },
    { 0x5819f15c, 55 },
    { 0x4569f15b, 77 },
    { 0, 0 },
    { 0xe879f15f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ea9f163, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca9f168, 90 },
    { 0x8019f169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4f9f16d, 34 },
    { 0, 0 },
    { 0xf49f16f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa379f172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1a9f176, 95 },
    { 0, 0 },
    { 0x58e9f178, 33 },
    { 0x8759f179, 98 },
    { 0, 0 },
    { 0x7019f17b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa229f17e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6089f183, 53 },
    { 0xc629f183, 85 },
    { 0x8589f184, 42 },
    { 0xfad9f185, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c9f18a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb029f18e, 55 },
    { 0x90c9f18f, 88 },
    { 0xa099f18f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff49f196, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c9f199, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3859f19c, 91 },
    { 0xf729f19c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe759f1a4, 33 },
    { 0x74e9f1a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe149f1a8, 43 },
    { 0xcca9f1a9, 91 },
    { 0x82f9f1a8, 34 },
    { 0, 0 },
    { 0x5d19f1ac, 55 },
    { 0xfd29f1ad, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1319f1b7, 95 },
    { 0xa889f1b8, 90 },
    { 0x2c69f1b9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469f1bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e79f1c5, 98 },
    { 0x2439f1c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa099f1cd, 17 },
    { 0x6069f1ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc9f1d4, 43 },
    { 0xbe29f1d4, 90 },
    { 0, 0 },
    { 0x41b9f1d7, 98 },
    { 0xb609f1d8, 66 },
    { 0x4ea9f1d8, 76 },
    { 0xd2f9f1d7, 46 },
    { 0, 0 },
    { 0xdc69f1dc, 99 },
    { 0, 0 },
    { 0xd669f1de, 95 },
    { 0, 0 },
    { 0xa19f1e0, 33 },
    { 0x1439f1e1, 90 },
    { 0x86b9f1e2, 33 },
    { 0, 0 },
    { 0xb529f1e4, 55 },
    { 0x4be9f1e5, 58 },
    { 0, 0 },
    { 0xe059f1e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b9f1ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c39f1f2, 46 },
    { 0xf399f1f3, 43 },
    { 0x6639f1f4, 90 },
    { 0xa509f1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f9f1f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9f1fc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe199f200, 98 },
    { 0x3359f201, 55 },
    { 0xfcd9f202, 98 },
    { 0x70e9f203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaad9f206, 85 },
    { 0xa839f207, 98 },
    { 0, 0 },
    { 0x1749f209, 95 },
    { 0, 0 },
    { 0xc829f20b, 99 },
    { 0xfb19f20c, 81 },
    { 0x9899f20d, 98 },
    { 0xc379f20e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b9f212, 43 },
    { 0x1009f213, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f39f21d, 61 },
    { 0, 0 },
    { 0xd0b9f21f, 69 },
    { 0xcb19f21f, 61 },
    { 0x9419f21f, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c69f224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea39f237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f23a, 81 },
    { 0, 0 },
    { 0x67d9f23c, 33 },
    { 0xce29f23d, 33 },
    { 0, 0 },
    { 0x51a9f23f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9f244, 65 },
    { 0, 0 },
    { 0x629f246, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f9f24d, 61 },
    { 0, 0 },
    { 0x12a9f24f, 33 },
    { 0, 0 },
    { 0x9ac9f251, 95 },
    { 0x4ad9f252, 85 },
    { 0, 0 },
    { 0xffa9f254, 81 },
    { 0, 0 },
    { 0xc129f256, 23 },
    { 0x9df9f257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb869f25c, 34 },
    { 0x87c9f25c, 76 },
    { 0x34a9f25d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9f261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23a9f268, 93 },
    { 0xe89f269, 33 },
    { 0xc739f26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc469f270, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x85b9f273, 46 },
    { 0, 0 },
    { 0x17a9f275, 93 },
    { 0, 0 },
    { 0x3a49f277, 33 },
    { 0xc159f278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6709f27b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809f27e, 76 },
    { 0xf2f9f27f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9999f288, 90 },
    { 0x26b9f289, 46 },
    { 0, 0 },
    { 0xdb29f28b, 17 },
    { 0x2969f28c, 90 },
    { 0x9119f28b, 76 },
    { 0x4c99f28d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf39f295, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a09f298, 90 },
    { 0x56b9f299, 23 },
    { 0x44a9f298, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac9f29d, 76 },
    { 0, 0 },
    { 0x84f9f29f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c9f2a6, 23 },
    { 0, 0 },
    { 0xac59f2a8, 85 },
    { 0, 0 },
    { 0xbd49f2aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a69f2ae, 61 },
    { 0, 0 },
    { 0x1e49f2b0, 90 },
    { 0, 0 },
    { 0x4819f2b2, 98 },
    { 0xcf09f2b3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b9f2bd, 90 },
    { 0xfa59f2bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9f2c3, 95 },
    { 0x3699f2c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee79f2ca, 90 },
    { 0x8f69f2ca, 43 },
    { 0xb8c9f2cc, 33 },
    { 0xee19f2cd, 17 },
    { 0x8989f2cd, 64 },
    { 0x3e89f2cd, 43 },
    { 0, 0 },
    { 0x6df9f2d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xedc9f2d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1259f2e3, 61 },
    { 0xf729f2e3, 98 },
    { 0x6819f2e4, 33 },
    { 0x69e9f2e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa469f2e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8499f2ec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179f2f1, 52 },
    { 0x1399f2f1, 98 },
    { 0x8399f2f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c9f2f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ef9f2fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d9f2fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f9f301, 43 },
    { 0x8399f302, 95 },
    { 0xfb69f302, 76 },
    { 0, 0 },
    { 0x4029f305, 33 },
    { 0x6809f306, 98 },
    { 0xc159f306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e89f30c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe419f310, 88 },
    { 0x6a59f311, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7659f316, 23 },
    { 0x9fc9f317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4789f324, 55 },
    { 0x29a9f325, 23 },
    { 0xc5a9f326, 33 },
    { 0x4459f326, 34 },
    { 0x6d49f328, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3b9f32f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f9f335, 43 },
    { 0, 0 },
    { 0x4309f337, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x249f33a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe109f33e, 52 },
    { 0x31b9f33e, 61 },
    { 0xcc79f33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe229f344, 46 },
    { 0x8659f345, 94 },
    { 0xa379f346, 80 },
    { 0x8ae9f346, 69 },
    { 0x289f347, 48 },
    { 0x6ae9f349, 96 },
    { 0xa969f348, 61 },
    { 0x7b29f344, 31 },
    { 0xc759f34c, 61 },
    { 0x7069f34d, 90 },
    { 0, 0 },
    { 0x11a9f34f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf789f359, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e59f35c, 33 },
    { 0x1219f35c, 33 },
    { 0x9439f35d, 95 },
    { 0x8aa9f35d, 81 },
    { 0x10a9f360, 61 },
    { 0, 0 },
    { 0xfad9f362, 90 },
    { 0x3179f362, 34 },
    { 0x2499f364, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x349f367, 98 },
    { 0x19c9f367, 33 },
    { 0x599f368, 33 },
    { 0x91d9f36a, 34 },
    { 0xb8a9f36a, 98 },
    { 0xe409f36c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b9f371, 55 },
    { 0x8959f372, 43 },
    { 0xd4d9f372, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc999f378, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89f37e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ae9f381, 94 },
    { 0, 0 },
    { 0x9e69f383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb359f386, 90 },
    { 0xbbf9f387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe949f38d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5469f392, 33 },
    { 0, 0 },
    { 0x5f59f394, 67 },
    { 0xc729f394, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5779f398, 91 },
    { 0x3ad9f399, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf079f39c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73f9f3a2, 64 },
    { 0, 0 },
    { 0x1d59f3a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e89f3a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1869f3b0, 80 },
    { 0xf269f3b1, 33 },
    { 0xed49f3b1, 61 },
    { 0x5489f3b1, 38 },
    { 0, 0 },
    { 0x6549f3b5, 90 },
    { 0, 0 },
    { 0x7799f3b7, 96 },
    { 0x6719f3b8, 98 },
    { 0x62f9f3b9, 98 },
    { 0x2509f3ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9f3bf, 46 },
    { 0xac9f3c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2009f3c3, 61 },
    { 0x9e89f3c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb109f3c9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59f3d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5559f3d3, 33 },
    { 0x4dd9f3d4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9f3d7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc9f3db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb179f3de, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e9f3e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6189f3e5, 98 },
    { 0xfd09f3e5, 78 },
    { 0xe779f3e7, 95 },
    { 0xf8a9f3e7, 98 },
    { 0x9c79f3e9, 84 },
    { 0x5249f3e8, 46 },
    { 0x29d9f3eb, 55 },
    { 0x55f9f3e7, 23 },
    { 0, 0 },
    { 0x2bc9f3ee, 100 },
    { 0, 0 },
    { 0x4f29f3f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e09f3f3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d9f3f8, 34 },
    { 0x7029f3f9, 43 },
    { 0xc499f3fa, 95 },
    { 0x8ba9f3fb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf439f405, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb159f40a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b69f40f, 95 },
    { 0, 0 },
    { 0x139f411, 33 },
    { 0x2749f411, 34 },
    { 0xc0c9f413, 55 },
    { 0, 0 },
    { 0x3669f415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f418, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8569f41b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2169f420, 91 },
    { 0x2cc9f420, 76 },
    { 0xc7d9f422, 98 },
    { 0x7aa9f423, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4829f432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab9f435, 61 },
    { 0, 0 },
    { 0x7329f437, 85 },
    { 0xe2a9f438, 90 },
    { 0x7fb9f439, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80c9f441, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9f44a, 76 },
    { 0, 0 },
    { 0x9bf9f44c, 90 },
    { 0, 0 },
    { 0x7c69f44e, 81 },
    { 0xdca9f44f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93f9f453, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129f45a, 98 },
    { 0x3f99f45b, 46 },
    { 0x82b9f45c, 65 },
    { 0xae19f45a, 91 },
    { 0, 0 },
    { 0x26d9f45f, 85 },
    { 0, 0 },
    { 0xc0b9f461, 78 },
    { 0, 0 },
    { 0x9979f463, 67 },
    { 0x7059f464, 33 },
    { 0x8709f464, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e09f468, 91 },
    { 0x48b9f469, 88 },
    { 0xef79f46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c9f46e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59f479, 88 },
    { 0, 0 },
    { 0x1a39f47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59b9f47e, 23 },
    { 0xf19f47f, 23 },
    { 0xcfa9f480, 23 },
    { 0, 0 },
    { 0x40c9f482, 90 },
    { 0x9e29f482, 64 },
    { 0xa469f483, 17 },
    { 0xa6e9f485, 67 },
    { 0xd749f486, 33 },
    { 0xfb59f487, 55 },
    { 0x88a9f488, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71d9f490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7499f493, 61 },
    { 0xa1c9f494, 23 },
    { 0x8c09f495, 98 },
    { 0x169f496, 61 },
    { 0x9309f497, 69 },
    { 0xe0d9f497, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce9f49b, 68 },
    { 0xaa59f49b, 61 },
    { 0x7c09f49b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb759f4a4, 93 },
    { 0x1bf9f4a4, 98 },
    { 0, 0 },
    { 0xe1b9f4a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b9f4b1, 81 },
    { 0x1d89f4b2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af9f4b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9f4b8, 90 },
    { 0xc8a9f4b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5f9f4bd, 91 },
    { 0x9579f4bd, 55 },
    { 0, 0 },
    { 0x749f4c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4789f4c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a89f4c7, 78 },
    { 0, 0 },
    { 0x2409f4c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd9f4cc, 33 },
    { 0x2d79f4cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf89f4d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea9f4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa939f4d8, 91 },
    { 0xbae9f4d9, 90 },
    { 0x5d69f4da, 78 },
    { 0, 0 },
    { 0x7ee9f4dc, 46 },
    { 0x2a09f4dc, 46 },
    { 0xfcd9f4dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c9f4e8, 17 },
    { 0xa8c9f4e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe389f4ec, 76 },
    { 0x3a29f4ed, 95 },
    { 0xb569f4ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda79f4f1, 33 },
    { 0xe59f4f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa139f4f9, 52 },
    { 0x3f9f4f9, 76 },
    { 0xe7f9f4fb, 90 },
    { 0x93d9f4fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c9f502, 95 },
    { 0x209f503, 60 },
    { 0x77b9f504, 100 },
    { 0, 0 },
    { 0xf689f506, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a59f50d, 93 },
    { 0, 0 },
    { 0x58a9f50f, 98 },
    { 0xf3b9f50f, 100 },
    { 0x8539f50f, 43 },
    { 0x94c9f510, 95 },
    { 0xdc89f510, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd559f517, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c9f51d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c9f522, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea9f530, 76 },
    { 0xa049f531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe709f538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de9f53b, 66 },
    { 0x4609f53c, 90 },
    { 0x8339f53d, 23 },
    { 0, 0 },
    { 0xf1c9f53f, 66 },
    { 0xd9b9f540, 68 },
    { 0xf459f53f, 55 },
    { 0, 0 },
    { 0xd19f543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9739f54a, 90 },
    { 0, 0 },
    { 0x6b29f54c, 100 },
    { 0x19a9f54c, 43 },
    { 0xafb9f54e, 78 },
    { 0xf1b9f54c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c9f556, 40 },
    { 0xe9f557, 23 },
    { 0x6de9f557, 98 },
    { 0, 0 },
    { 0xe4f9f55a, 33 },
    { 0, 0 },
    { 0x9609f55c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a9f560, 33 },
    { 0xe849f561, 61 },
    { 0, 0 },
    { 0x3019f563, 95 },
    { 0x1be9f563, 17 },
    { 0x90c9f565, 61 },
    { 0xdec9f566, 23 },
    { 0x7b19f566, 61 },
    { 0x20c9f566, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed9f56d, 91 },
    { 0, 0 },
    { 0xc749f56f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa89f573, 95 },
    { 0x7ac9f573, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299f577, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a9f57c, 98 },
    { 0x2759f57d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eb9f582, 66 },
    { 0xf999f583, 76 },
    { 0x6589f583, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d99f58b, 95 },
    { 0xc5e9f58b, 90 },
    { 0x4e89f58d, 34 },
    { 0xc5d9f58e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9f594, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdda9f59d, 23 },
    { 0x1809f59d, 61 },
    { 0xc3c9f59f, 33 },
    { 0x1b59f5a0, 98 },
    { 0xdad9f5a0, 25 },
    { 0x9059f59d, 33 },
    { 0, 0 },
    { 0xfba9f5a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ea9f5af, 23 },
    { 0xbaa9f5af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8159f5b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ce9f5ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d29f5bd, 99 },
    { 0, 0 },
    { 0x8a79f5bf, 75 },
    { 0xc7f9f5c0, 76 },
    { 0, 0 },
    { 0xcac9f5c2, 90 },
    { 0x2e59f5c2, 98 },
    { 0x12d9f5c4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7549f5c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9699f5d6, 55 },
    { 0x4399f5d7, 34 },
    { 0x2da9f5d6, 43 },
    { 0x83c9f5d9, 23 },
    { 0x2249f5d9, 33 },
    { 0x5259f5db, 85 },
    { 0xa359f5dc, 34 },
    { 0x1849f5d8, 56 },
    { 0x1ea9f5de, 87 },
    { 0xed9f5db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129f5e3, 99 },
    { 0x2b9f5e4, 95 },
    { 0, 0 },
    { 0xee99f5e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed49f5ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e9f5ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e29f5f1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c39f5f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9209f5f7, 91 },
    { 0x5c99f5f7, 90 },
    { 0xa679f5f8, 33 },
    { 0xf0b9f5f8, 100 },
    { 0x4029f5f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea29f600, 43 },
    { 0x1d29f600, 77 },
    { 0, 0 },
    { 0xbe49f603, 52 },
    { 0xc3e9f604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde9f607, 61 },
    { 0, 0 },
    { 0x6a49f609, 92 },
    { 0xca69f609, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x359f60d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d69f615, 75 },
    { 0x149f615, 67 },
    { 0x29d9f616, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ee9f625, 78 },
    { 0xd4b9f625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d79f629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1b9f632, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3459f63b, 55 },
    { 0x7909f63c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf629f640, 91 },
    { 0x44e9f640, 23 },
    { 0xc069f640, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e9f646, 33 },
    { 0xa889f647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc339f64a, 90 },
    { 0x8169f64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee79f653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f65a, 99 },
    { 0, 0 },
    { 0x6b19f65c, 90 },
    { 0x47a9f65c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b9f666, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc19f669, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac9f66d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb049f670, 43 },
    { 0x2279f671, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a9f678, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f9f67b, 88 },
    { 0x7e99f67c, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1109f680, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x129f683, 17 },
    { 0xea39f684, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb809f687, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7449f68c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be9f690, 23 },
    { 0x9209f691, 85 },
    { 0x9ee9f692, 33 },
    { 0, 0 },
    { 0x7ab9f694, 98 },
    { 0, 0 },
    { 0x4169f696, 66 },
    { 0x1389f696, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a9f69b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5229f69f, 55 },
    { 0x3d29f6a0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1069f6a5, 55 },
    { 0xffe9f6a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf549f6ab, 52 },
    { 0xf4f9f6ac, 65 },
    { 0, 0 },
    { 0x4999f6ae, 33 },
    { 0x9ae9f6af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de9f6b4, 33 },
    { 0xe89f6b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4399f6c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9519f6c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b9f6c8, 23 },
    { 0xd079f6c8, 23 },
    { 0x9f6ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3af9f6cd, 99 },
    { 0xb5e9f6cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4499f6d5, 92 },
    { 0x9139f6d6, 90 },
    { 0x63a9f6d6, 76 },
    { 0x3269f6d6, 98 },
    { 0x8509f6d8, 98 },
    { 0x8de9f6da, 90 },
    { 0x4f79f6d6, 33 },
    { 0x7959f6d6, 51 },
    { 0x5439f6db, 78 },
    { 0x8979f6d6, 76 },
    { 0xe7c9f6d5, 91 },
    { 0x13c9f6e0, 55 },
    { 0x10d9f6e1, 98 },
    { 0x8d99f6e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe689f6e8, 55 },
    { 0x6bb9f6e9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669f6ee, 33 },
    { 0, 0 },
    { 0xbf9f6f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa9f6f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9f6fb, 23 },
    { 0xf149f6fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacf9f6ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2329f704, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb9f70f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8129f715, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a9f718, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d9f71b, 87 },
    { 0xfe39f71c, 34 },
    { 0x2de9f71d, 98 },
    { 0xf4f9f71e, 96 },
    { 0x8089f71e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae9f726, 38 },
    { 0xeb09f727, 98 },
    { 0x4b9f727, 90 },
    { 0xd859f729, 87 },
    { 0, 0 },
    { 0xf659f72b, 98 },
    { 0xb8e9f72c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa889f732, 23 },
    { 0x4339f733, 61 },
    { 0x5089f732, 90 },
    { 0x1d49f735, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f09f738, 98 },
    { 0xc149f738, 90 },
    { 0x5429f73a, 55 },
    { 0x3299f73b, 46 },
    { 0, 0 },
    { 0xb0a9f73d, 98 },
    { 0x8079f73d, 61 },
    { 0xd519f73f, 76 },
    { 0x13d9f740, 98 },
    { 0x8ac9f73d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be9f746, 61 },
    { 0xc629f747, 34 },
    { 0x3259f748, 23 },
    { 0x79e9f749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb719f74e, 65 },
    { 0xc879f74f, 90 },
    { 0x4de9f750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c99f753, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2259f757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1d9f75b, 76 },
    { 0, 0 },
    { 0x69f75d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b49f762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9f76a, 46 },
    { 0x37f9f76b, 90 },
    { 0x9c49f76c, 98 },
    { 0, 0 },
    { 0xffc9f76e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef9f772, 61 },
    { 0xc669f773, 33 },
    { 0x6659f773, 43 },
    { 0, 0 },
    { 0x8239f776, 90 },
    { 0x2d59f776, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e9f77c, 90 },
    { 0, 0 },
    { 0xf999f77e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db9f783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5369f789, 61 },
    { 0xf4b9f78a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6339f78f, 61 },
    { 0, 0 },
    { 0xd4f9f791, 81 },
    { 0x63a9f792, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8859f798, 95 },
    { 0xaa29f799, 91 },
    { 0xd4b9f799, 55 },
    { 0xfd49f799, 95 },
    { 0xe5f9f799, 76 },
    { 0xbd39f79b, 68 },
    { 0xbfb9f799, 66 },
    { 0xc2d9f79f, 99 },
    { 0x7219f7a0, 90 },
    { 0x78c9f7a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1229f7a4, 66 },
    { 0x1b19f7a5, 34 },
    { 0xc799f7a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f9f7aa, 46 },
    { 0xc019f7ab, 98 },
    { 0xd579f7ac, 98 },
    { 0x38a9f7ab, 95 },
    { 0x3d99f7ae, 48 },
    { 0x7219f7af, 95 },
    { 0x2739f7af, 33 },
    { 0xd979f7ac, 100 },
    { 0xd069f7aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xea19f7b5, 76 },
    { 0x3cb9f7b5, 34 },
    { 0, 0 },
    { 0x6469f7b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49d9f7bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c79f7c0, 61 },
    { 0x2ac9f7c0, 81 },
    { 0x4ec9f7c2, 90 },
    { 0x2de9f7c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769f7c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe379f7ca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3469f7cd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa729f7d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9f7d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e9f7db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3579f7e0, 91 },
    { 0x7999f7e1, 76 },
    { 0xe09f7e2, 52 },
    { 0x6ab9f7e3, 91 },
    { 0xe1b9f7e2, 61 },
    { 0x16c9f7e5, 95 },
    { 0x7e39f7e3, 33 },
    { 0xc89f7e7, 61 },
    { 0x5569f7e3, 95 },
    { 0x6549f7e9, 98 },
    { 0xa619f7e7, 61 },
    { 0xc209f7eb, 78 },
    { 0x3d99f7eb, 95 },
    { 0x5e09f7ed, 67 },
    { 0xee19f7ee, 84 },
    { 0xc209f7ed, 98 },
    { 0x2a29f7ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7209f7f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809f7fd, 23 },
    { 0xdf29f7fe, 76 },
    { 0, 0 },
    { 0x1e9f800, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa519f805, 33 },
    { 0xb359f805, 94 },
    { 0x2949f807, 84 },
    { 0x22b9f805, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c69f810, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa19f813, 90 },
    { 0xed09f814, 76 },
    { 0x2619f815, 55 },
    { 0x95b9f814, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c9f81c, 33 },
    { 0, 0 },
    { 0xb819f81e, 53 },
    { 0xede9f81f, 23 },
    { 0x5e39f81e, 95 },
    { 0xea19f821, 23 },
    { 0, 0 },
    { 0xac69f823, 33 },
    { 0, 0 },
    { 0xe579f825, 93 },
    { 0x51e9f826, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9069f829, 90 },
    { 0x3b29f829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d79f82e, 43 },
    { 0, 0 },
    { 0x1a09f830, 95 },
    { 0xab89f831, 93 },
    { 0x2be9f830, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd979f835, 66 },
    { 0, 0 },
    { 0x2689f837, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e9f842, 98 },
    { 0, 0 },
    { 0x5809f844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1729f84b, 98 },
    { 0x7819f84b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f84f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9f853, 61 },
    { 0x9909f854, 34 },
    { 0, 0 },
    { 0xad59f856, 98 },
    { 0xe9a9f856, 78 },
    { 0x1de9f858, 56 },
    { 0x9689f859, 98 },
    { 0xff69f858, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1f9f86a, 55 },
    { 0, 0 },
    { 0x8589f86c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d09f872, 95 },
    { 0, 0 },
    { 0xfea9f874, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f877, 61 },
    { 0, 0 },
    { 0xe2b9f879, 61 },
    { 0, 0 },
    { 0x42e9f87b, 86 },
    { 0x639f87b, 81 },
    { 0x1969f87b, 23 },
    { 0, 0 },
    { 0xc179f87f, 84 },
    { 0x1fc9f87f, 69 },
    { 0x5e59f881, 65 },
    { 0, 0 },
    { 0xe8a9f883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1429f888, 33 },
    { 0xdd89f888, 90 },
    { 0x729f88a, 33 },
    { 0, 0 },
    { 0x5869f88c, 95 },
    { 0x2919f88c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9f892, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa569f898, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab9f89b, 90 },
    { 0, 0 },
    { 0xa569f89d, 55 },
    { 0xa039f89e, 98 },
    { 0x339f89f, 33 },
    { 0xc1a9f8a0, 91 },
    { 0x149f8a0, 66 },
    { 0xb3a9f89d, 90 },
    { 0xd229f8a3, 90 },
    { 0xf729f8a0, 100 },
    { 0x2549f8a5, 61 },
    { 0, 0 },
    { 0xcd19f8a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb089f8aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a9f8ae, 34 },
    { 0xc379f8ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c9f8b4, 61 },
    { 0x7949f8b5, 46 },
    { 0, 0 },
    { 0x8b99f8b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48d9f8bf, 23 },
    { 0x7799f8c0, 78 },
    { 0x5d09f8c1, 55 },
    { 0xac79f8c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c9f8c6, 76 },
    { 0, 0 },
    { 0x7d99f8c8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd829f8cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1819f8d7, 65 },
    { 0xaa89f8d8, 91 },
    { 0x2769f8d9, 90 },
    { 0xefb9f8d8, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4829f8de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e29f8e1, 90 },
    { 0x58e9f8e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d19f8e5, 81 },
    { 0x2ce9f8e6, 23 },
    { 0, 0 },
    { 0x349f8e8, 98 },
    { 0x2c09f8e9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2c9f8ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6139f8f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92a9f8f8, 76 },
    { 0x3bf9f8f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5289f900, 98 },
    { 0, 0 },
    { 0xd469f902, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e99f90d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9f911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4099f91b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f89f921, 90 },
    { 0x2299f922, 81 },
    { 0x2e69f923, 81 },
    { 0xccf9f924, 95 },
    { 0xd089f925, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3759f931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc179f934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3429f939, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2079f93f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c9f944, 90 },
    { 0x7f99f945, 43 },
    { 0, 0 },
    { 0xb459f947, 23 },
    { 0, 0 },
    { 0xeda9f949, 34 },
    { 0, 0 },
    { 0xfff9f94b, 69 },
    { 0x8a79f94c, 87 },
    { 0x12f9f94c, 61 },
    { 0x8bf9f94c, 43 },
    { 0xea19f94f, 33 },
    { 0xa29f94f, 91 },
    { 0x7ae9f94d, 35 },
    { 0, 0 },
    { 0x6039f953, 33 },
    { 0, 0 },
    { 0x9b29f955, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe899f95b, 40 },
    { 0x6e99f95c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3439f960, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd9f964, 87 },
    { 0x1619f965, 61 },
    { 0x59b9f964, 98 },
    { 0, 0 },
    { 0x8109f968, 55 },
    { 0xd6c9f968, 23 },
    { 0x2389f96a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c9f96e, 61 },
    { 0x22f9f96f, 61 },
    { 0x7049f96f, 33 },
    { 0x8869f96e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c29f976, 76 },
    { 0, 0 },
    { 0x5ec9f978, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a9f97b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c9f985, 55 },
    { 0xdbe9f986, 90 },
    { 0xf419f987, 33 },
    { 0, 0 },
    { 0x6c69f989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc299f98c, 93 },
    { 0x6819f98d, 61 },
    { 0, 0 },
    { 0x1749f98f, 61 },
    { 0x1b89f990, 81 },
    { 0xeb9f990, 98 },
    { 0, 0 },
    { 0x91b9f993, 98 },
    { 0x6709f994, 95 },
    { 0, 0 },
    { 0x57d9f996, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d9f99b, 46 },
    { 0x6a39f99c, 61 },
    { 0x4329f99c, 61 },
    { 0x9309f99c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb59f9a9, 61 },
    { 0x7129f9a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1519f9ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e09f9b2, 61 },
    { 0, 0 },
    { 0x7a79f9b4, 76 },
    { 0xda19f9b5, 76 },
    { 0xe819f9b5, 61 },
    { 0x4a69f9b6, 61 },
    { 0x58f9f9b8, 33 },
    { 0x71b9f9b9, 90 },
    { 0x7dc9f9b9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c19f9be, 87 },
    { 0xdde9f9be, 84 },
    { 0, 0 },
    { 0x449f9c1, 33 },
    { 0, 0 },
    { 0x2419f9c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6859f9ce, 80 },
    { 0x2369f9ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc519f9d2, 98 },
    { 0, 0 },
    { 0x5419f9d4, 90 },
    { 0, 0 },
    { 0xea39f9d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4339f9da, 90 },
    { 0, 0 },
    { 0xadf9f9dc, 93 },
    { 0, 0 },
    { 0x9889f9de, 52 },
    { 0x5b69f9df, 99 },
    { 0x39d9f9df, 33 },
    { 0x3119f9e1, 34 },
    { 0, 0 },
    { 0x15b9f9e3, 91 },
    { 0x89f9f9e4, 33 },
    { 0, 0 },
    { 0x9469f9e6, 33 },
    { 0, 0 },
    { 0xa849f9e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee69f9ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b9f9f3, 90 },
    { 0, 0 },
    { 0xb1d9f9f5, 77 },
    { 0x9479f9f6, 46 },
    { 0, 0 },
    { 0x649f9f8, 90 },
    { 0x1329f9f8, 95 },
    { 0, 0 },
    { 0xce39f9fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7439f9ff, 33 },
    { 0x2469fa00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2109fa05, 98 },
    { 0x70c9fa06, 23 },
    { 0x3569fa05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef9fa0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c89fa11, 84 },
    { 0x3f89fa11, 99 },
    { 0, 0 },
    { 0x7e99fa14, 95 },
    { 0, 0 },
    { 0xfef9fa16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d9fa1c, 23 },
    { 0xfa09fa1c, 99 },
    { 0, 0 },
    { 0x6ee9fa1f, 90 },
    { 0x3759fa20, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79fa25, 90 },
    { 0x8399fa25, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb009fa2b, 90 },
    { 0, 0 },
    { 0x6709fa2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7c9fa31, 33 },
    { 0, 0 },
    { 0x7c59fa33, 98 },
    { 0x1f69fa34, 90 },
    { 0, 0 },
    { 0x1829fa36, 23 },
    { 0x1419fa37, 66 },
    { 0x1e79fa37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4019fa3d, 91 },
    { 0x61a9fa3d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f9fa41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0a9fa4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54f9fa52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd9fa57, 33 },
    { 0x93a9fa58, 90 },
    { 0x2a19fa59, 90 },
    { 0x51c9fa5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda39fa61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b79fa64, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4e9fa68, 81 },
    { 0x939fa69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4059fa6d, 46 },
    { 0, 0 },
    { 0x9179fa6f, 34 },
    { 0xad99fa6f, 23 },
    { 0x3c79fa71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe299fa76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3639fa79, 98 },
    { 0, 0 },
    { 0x669fa7b, 33 },
    { 0x9789fa7c, 81 },
    { 0, 0 },
    { 0xd3e9fa7e, 61 },
    { 0, 0 },
    { 0xa999fa80, 23 },
    { 0, 0 },
    { 0xd509fa82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c9fa85, 98 },
    { 0xca9fa86, 87 },
    { 0, 0 },
    { 0x7469fa88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf799fa8b, 61 },
    { 0xd989fa8c, 100 },
    { 0xdba9fa8b, 35 },
    { 0x4709fa8e, 28 },
    { 0x3389fa8f, 46 },
    { 0, 0 },
    { 0x69e9fa91, 43 },
    { 0xb8f9fa92, 93 },
    { 0x3d79fa93, 90 },
    { 0xd759fa92, 90 },
    { 0x56a9fa91, 33 },
    { 0, 0 },
    { 0x6ef9fa97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa239fa9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa659faa5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4429faae, 33 },
    { 0x9249faaf, 93 },
    { 0xb149fab0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e9fab3, 90 },
    { 0x8959fab4, 88 },
    { 0, 0 },
    { 0x3ef9fab6, 23 },
    { 0xb4b9fab7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf79fabd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99fac6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5269fac9, 61 },
    { 0x9f59faca, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b9facf, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e9fad6, 91 },
    { 0x4019fad7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d9fadb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9399fadf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ba9fae4, 87 },
    { 0xddf9fae5, 61 },
    { 0x3b69fae4, 52 },
    { 0x57a9fae5, 76 },
    { 0x9b09fae8, 61 },
    { 0xac99fae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9669faed, 49 },
    { 0x7bb9faee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd9faf9, 33 },
    { 0x8a79faf9, 56 },
    { 0, 0 },
    { 0xcd39fafc, 90 },
    { 0xc0b9fafd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb799fb00, 34 },
    { 0, 0 },
    { 0x87c9fb02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c9fb05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf9fb0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa609fb1a, 34 },
    { 0, 0 },
    { 0x3459fb1c, 33 },
    { 0xcd99fb1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a59fb21, 61 },
    { 0x8919fb22, 84 },
    { 0x1089fb23, 98 },
    { 0xaeb9fb22, 83 },
    { 0x5659fb21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a9fb28, 95 },
    { 0xf279fb28, 97 },
    { 0xa259fb28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b9fb2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc599fb34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c9fb39, 85 },
    { 0xeee9fb3a, 98 },
    { 0xd169fb39, 95 },
    { 0xada9fb3c, 23 },
    { 0x2b19fb3c, 33 },
    { 0x3349fb3e, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1d9fb42, 23 },
    { 0, 0 },
    { 0x7a69fb44, 75 },
    { 0x4149fb44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdad9fb4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe69fb55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd859fb5f, 84 },
    { 0xfd29fb60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9fb6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4729fb70, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1619fb74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49fb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9939fb7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6959fb7e, 33 },
    { 0x6b9fb7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a29fb82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2509fb87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1c9fb8e, 55 },
    { 0x3f59fb8f, 90 },
    { 0, 0 },
    { 0x7a39fb91, 90 },
    { 0xe89fb92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd439fb95, 88 },
    { 0x2a79fb96, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd569fb9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7949fb9f, 93 },
    { 0xc979fba0, 53 },
    { 0xc99fba0, 90 },
    { 0, 0 },
    { 0x679fba3, 90 },
    { 0x1089fba4, 61 },
    { 0x32d9fba5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba9fbb0, 23 },
    { 0xa169fbb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9b9fbb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f59fbb7, 61 },
    { 0x9fc9fbb8, 89 },
    { 0x4189fbb8, 91 },
    { 0x55c9fbba, 61 },
    { 0x569fbb8, 76 },
    { 0xf309fbb7, 33 },
    { 0xbe69fbbd, 78 },
    { 0xe7b9fbbd, 75 },
    { 0x8e29fbbf, 78 },
    { 0, 0 },
    { 0x65d9fbc1, 34 },
    { 0, 0 },
    { 0x69d9fbc3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b9fbca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fbd0, 33 },
    { 0x9d9fbd1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf89fbd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fa9fbda, 95 },
    { 0x869fbdb, 55 },
    { 0x5029fbda, 61 },
    { 0x7719fbda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d19fbe3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c9fbe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9fbef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a99fbf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6649fbf5, 43 },
    { 0xc579fbf6, 93 },
    { 0x79d9fbf7, 23 },
    { 0xefd9fbf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9fc00, 61 },
    { 0xb489fc00, 90 },
    { 0, 0 },
    { 0xb79fc03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c9fc06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a49fc09, 23 },
    { 0xe149fc09, 23 },
    { 0, 0 },
    { 0xd489fc0c, 46 },
    { 0, 0 },
    { 0x1839fc0e, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a9fc12, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20e9fc18, 100 },
    { 0xc829fc19, 93 },
    { 0x9469fc1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9fc1d, 88 },
    { 0xe689fc1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e89fc24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b9fc27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb79fc2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e9fc30, 76 },
    { 0x449fc31, 51 },
    { 0x7529fc31, 90 },
    { 0xda79fc33, 90 },
    { 0, 0 },
    { 0xf749fc35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7e9fc38, 23 },
    { 0x6d89fc38, 40 },
    { 0xa599fc3a, 90 },
    { 0x8629fc3b, 55 },
    { 0, 0 },
    { 0x53f9fc3d, 55 },
    { 0xb49fc3e, 61 },
    { 0x1659fc3e, 51 },
    { 0x1f79fc40, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7529fc44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea79fc4f, 34 },
    { 0, 0 },
    { 0x1d09fc51, 52 },
    { 0x8ac9fc51, 38 },
    { 0, 0 },
    { 0xf279fc54, 90 },
    { 0x5fe9fc55, 33 },
    { 0xfc49fc55, 33 },
    { 0, 0 },
    { 0x4919fc58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e9fc5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42e9fc62, 87 },
    { 0xfee9fc63, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1959fc66, 90 },
    { 0x62d9fc67, 95 },
    { 0xd5f9fc66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2849fc6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3179fc70, 95 },
    { 0x199fc71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3019fc75, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff9fc78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f79fc7f, 98 },
    { 0x7f69fc80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e9fc88, 94 },
    { 0, 0 },
    { 0x8b39fc8a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f9fc95, 76 },
    { 0x1ee9fc95, 95 },
    { 0xbb09fc95, 98 },
    { 0, 0 },
    { 0x2559fc99, 95 },
    { 0, 0 },
    { 0x7929fc9b, 76 },
    { 0x3289fc9c, 34 },
    { 0x35b9fc9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9009fca1, 90 },
    { 0, 0 },
    { 0xc6a9fca3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce99fcad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaac9fcb5, 33 },
    { 0, 0 },
    { 0x649fcb7, 90 },
    { 0x6479fcb8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1129fcbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf19fcbf, 33 },
    { 0xcfd9fcbf, 55 },
    { 0xf729fcc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc129fcc4, 95 },
    { 0xf159fcc4, 23 },
    { 0x92d9fcc6, 99 },
    { 0x5f59fcc4, 55 },
    { 0x3c79fcc8, 55 },
    { 0x8dc9fcc8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9639fcce, 43 },
    { 0xfb79fccf, 33 },
    { 0xcaf9fcd0, 52 },
    { 0xb249fcce, 90 },
    { 0xd289fcd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa9fcd5, 76 },
    { 0x3579fcd6, 61 },
    { 0x7029fcd7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb29fcdb, 98 },
    { 0x1ed9fcdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb179fcdf, 33 },
    { 0x6e19fcdf, 33 },
    { 0x5c59fcdf, 76 },
    { 0, 0 },
    { 0xfa79fce3, 61 },
    { 0x97f9fce4, 61 },
    { 0x5a49fce4, 46 },
    { 0x7bd9fce5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5419fcec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f9fcf1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4999fcf5, 34 },
    { 0, 0 },
    { 0xe649fcf7, 90 },
    { 0x6fc9fcf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dd9fcfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc619fd01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9459fd08, 66 },
    { 0x2769fd09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29d9fd0c, 55 },
    { 0x44b9fd0d, 95 },
    { 0xb439fd0e, 76 },
    { 0xc819fd0c, 95 },
    { 0x2e49fd10, 84 },
    { 0x8349fd11, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a9fd15, 55 },
    { 0x279fd16, 46 },
    { 0x40c9fd17, 78 },
    { 0x73e9fd18, 90 },
    { 0x68c9fd15, 33 },
    { 0xe6a9fd1a, 80 },
    { 0xd9c9fd1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5019fd1e, 61 },
    { 0xa1a9fd1e, 98 },
    { 0x2e19fd20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b19fd24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9db9fd2a, 77 },
    { 0, 0 },
    { 0x6269fd2c, 55 },
    { 0, 0 },
    { 0xc69fd2e, 17 },
    { 0x5f79fd2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4269fd34, 84 },
    { 0x46c9fd35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb029fd3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3e9fd3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd069fd43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb29fd47, 33 },
    { 0x8ee9fd47, 43 },
    { 0x87f9fd49, 23 },
    { 0, 0 },
    { 0x7879fd4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1949fd53, 61 },
    { 0, 0 },
    { 0xcc09fd55, 52 },
    { 0x4cd9fd56, 90 },
    { 0xcb09fd57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce9fd5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfef9fd61, 98 },
    { 0, 0 },
    { 0x5e19fd63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa899fd68, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x889fd6b, 98 },
    { 0, 0 },
    { 0xd339fd6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa749fd70, 33 },
    { 0xb299fd71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf809fd79, 90 },
    { 0, 0 },
    { 0x8149fd7b, 61 },
    { 0x6b09fd7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f59fd80, 61 },
    { 0xea19fd81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf69fd87, 55 },
    { 0x93a9fd88, 87 },
    { 0x8e19fd89, 90 },
    { 0x4b09fd88, 65 },
    { 0, 0 },
    { 0x1fe9fd8c, 33 },
    { 0xf379fd8c, 95 },
    { 0x5409fd8e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4129fd91, 95 },
    { 0x3f99fd91, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x2da9fd95, 95 },
    { 0, 0 },
    { 0xaf19fd97, 34 },
    { 0, 0 },
    { 0x91c9fd99, 38 },
    { 0x829fd9a, 93 },
    { 0x589fd9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d99fd9f, 34 },
    { 0x3f9fd9f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8969fda5, 76 },
    { 0, 0 },
    { 0x7fc9fda7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa859fdaa, 34 },
    { 0xc3c9fdab, 75 },
    { 0, 0 },
    { 0x1ac9fdad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba9fdb5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd9fdb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb009fdbc, 78 },
    { 0, 0 },
    { 0x5c69fdbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x889fdc1, 61 },
    { 0x80f9fdc2, 61 },
    { 0, 0 },
    { 0xa679fdc4, 33 },
    { 0xfb89fdc5, 83 },
    { 0x4c09fdc4, 90 },
    { 0, 0 },
    { 0xd4c9fdc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc379fdcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f9fdd1, 33 },
    { 0x2a99fdd2, 66 },
    { 0, 0 },
    { 0x85c9fdd4, 34 },
    { 0xfe49fdd5, 93 },
    { 0xf609fdd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b9fdd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef9fdde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65b9fde6, 98 },
    { 0, 0 },
    { 0xfc89fde8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18c9fdec, 43 },
    { 0xc6d9fdec, 27 },
    { 0x3259fdee, 55 },
    { 0xd889fdef, 90 },
    { 0x3b79fdf0, 90 },
    { 0xcaf9fdef, 90 },
    { 0xf4b9fdf0, 98 },
    { 0xf649fdf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb859fdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1189fdfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c99fe00, 69 },
    { 0xa39fe01, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x9259fe04, 76 },
    { 0, 0 },
    { 0x7ac9fe06, 98 },
    { 0x96b9fe06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2799fe0a, 98 },
    { 0x4069fe0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f29fe10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad9fe1b, 78 },
    { 0xf09fe1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6979fe1f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c69fe23, 33 },
    { 0x1b9fe24, 95 },
    { 0x599fe25, 61 },
    { 0x15d9fe23, 95 },
    { 0x1389fe27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe619fe2a, 98 },
    { 0x56e9fe2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b69fe34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d9fe3d, 98 },
    { 0, 0 },
    { 0xb2e9fe3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b29fe42, 90 },
    { 0x679fe43, 98 },
    { 0xab79fe44, 93 },
    { 0, 0 },
    { 0x6429fe46, 90 },
    { 0xb1d9fe46, 90 },
    { 0xb449fe48, 78 },
    { 0x8c49fe49, 23 },
    { 0x12c9fe49, 76 },
    { 0, 0 },
    { 0x2ea9fe4c, 76 },
    { 0x86f9fe4c, 55 },
    { 0xf9c9fe4d, 66 },
    { 0xb69fe4c, 75 },
    { 0, 0 },
    { 0x5059fe51, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e9fe56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25e9fe5a, 67 },
    { 0, 0 },
    { 0x1f29fe5c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f49fe5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6be9fe65, 87 },
    { 0xd59fe66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9fe69, 34 },
    { 0x2b19fe6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6f9fe6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9fe77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae89fe7a, 96 },
    { 0, 0 },
    { 0xa4f9fe7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8689fe84, 91 },
    { 0, 0 },
    { 0x9f9fe86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4059fe8b, 17 },
    { 0x6009fe8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ae9fe8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9fe95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a9fe98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa359fe9b, 33 },
    { 0xc779fe9c, 76 },
    { 0, 0 },
    { 0x7e79fe9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc509fea3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda89fea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c59feab, 33 },
    { 0, 0 },
    { 0x3a9fead, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c9feb5, 46 },
    { 0xe1c9feb6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a9feb9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcc9febc, 46 },
    { 0, 0 },
    { 0x98f9febe, 90 },
    { 0xcc9febf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d9fec5, 46 },
    { 0, 0 },
    { 0x779fec7, 90 },
    { 0, 0 },
    { 0x2b09fec9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf209fecc, 46 },
    { 0x9129fecd, 90 },
    { 0x6c09fecd, 33 },
    { 0xba59fecf, 33 },
    { 0xc9b9fed0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5669fed8, 55 },
    { 0, 0 },
    { 0xcd29feda, 100 },
    { 0xf7a9feda, 66 },
    { 0x21f9fedc, 66 },
    { 0xd5d9feda, 66 },
    { 0, 0 },
    { 0x13c9fedf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd539fee2, 23 },
    { 0, 0 },
    { 0x3169fee4, 61 },
    { 0xc9d9fee4, 23 },
    { 0xfa9fee6, 33 },
    { 0xe259fee7, 61 },
    { 0, 0 },
    { 0xfda9fee9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8079fef4, 34 },
    { 0xa229fef4, 76 },
    { 0x4a89fef6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x769fef9, 66 },
    { 0x6e39fef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ff04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e9ff07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f39ff0a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4909ff0d, 43 },
    { 0, 0 },
    { 0x41d9ff0f, 87 },
    { 0x13f9ff10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dc9ff13, 43 },
    { 0, 0 },
    { 0x4269ff15, 90 },
    { 0x5aa9ff16, 98 },
    { 0x1dd9ff16, 76 },
    { 0, 0 },
    { 0x27e9ff19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a9ff1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce9ff22, 52 },
    { 0, 0 },
    { 0x4819ff24, 55 },
    { 0xbdb9ff25, 33 },
    { 0x5a99ff26, 34 },
    { 0x6779ff26, 93 },
    { 0xded9ff27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8979ff2c, 61 },
    { 0xc2f9ff2c, 33 },
    { 0x7939ff2c, 87 },
    { 0x5da9ff2f, 90 },
    { 0, 0 },
    { 0xcea9ff31, 52 },
    { 0xae59ff32, 76 },
    { 0x1a19ff33, 61 },
    { 0x5779ff31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f9ff39, 67 },
    { 0x44c9ff3a, 61 },
    { 0xea39ff3b, 90 },
    { 0xe989ff3c, 65 },
    { 0x4b79ff3c, 91 },
    { 0xb599ff3a, 66 },
    { 0x9d29ff39, 34 },
    { 0xfd59ff40, 61 },
    { 0xccc9ff41, 95 },
    { 0xd859ff42, 84 },
    { 0x9299ff43, 100 },
    { 0x95f9ff44, 43 },
    { 0x8b09ff43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c9ff48, 55 },
    { 0x7ea9ff49, 85 },
    { 0xfe99ff4a, 96 },
    { 0x8dd9ff49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf769ff4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ff51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc419ff57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b9ff5a, 90 },
    { 0x9ed9ff5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da9ff61, 23 },
    { 0x3c09ff62, 23 },
    { 0xd489ff63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e9ff6f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c19ff72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed9ff75, 87 },
    { 0xaf79ff76, 33 },
    { 0x8fb9ff76, 77 },
    { 0xa7c9ff76, 33 },
    { 0x3a9ff77, 95 },
    { 0x5929ff7a, 33 },
    { 0xd639ff7b, 23 },
    { 0xc129ff7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9ff84, 55 },
    { 0xf79ff85, 55 },
    { 0x7069ff86, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7e9ff8a, 33 },
    { 0xfd49ff8b, 46 },
    { 0, 0 },
    { 0x9929ff8d, 76 },
    { 0xa779ff8e, 90 },
    { 0x2e49ff8f, 91 },
    { 0x7129ff8f, 90 },
    { 0x24c9ff8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf99ff96, 78 },
    { 0xa3a9ff96, 23 },
    { 0xfd89ff98, 33 },
    { 0xf309ff99, 34 },
    { 0xc109ff9a, 67 },
    { 0xd1e9ff9a, 90 },
    { 0xbb9ff9c, 61 },
    { 0, 0 },
    { 0xac69ff9e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x519ffa1, 91 },
    { 0xb639ffa1, 33 },
    { 0xbf9ffa3, 17 },
    { 0xf6a9ffa2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc29ffa8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb069ffab, 33 },
    { 0, 0 },
    { 0x2229ffad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99ffb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce9ffb4, 66 },
    { 0x26f9ffb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83e9ffb8, 90 },
    { 0xc959ffb9, 65 },
    { 0xc4a9ffb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4469ffbe, 61 },
    { 0, 0 },
    { 0x7819ffc0, 76 },
    { 0x7d59ffc1, 90 },
    { 0x8b9ffc2, 66 },
    { 0x6089ffc3, 93 },
    { 0xa8e9ffc3, 34 },
    { 0, 0 },
    { 0x5ec9ffc6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d69ffc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc9ffd1, 55 },
    { 0xd3f9ffd2, 87 },
    { 0x1d49ffd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2c9ffda, 61 },
    { 0xb69ffdb, 61 },
    { 0xecd9ffdc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4189ffe2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1069ffe8, 33 },
    { 0xfc9ffe9, 66 },
    { 0x1789ffe8, 67 },
    { 0x2ef9ffe9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2759ffef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef9fff7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf19fffb, 95 },
    { 0xb7d9fffb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfba0000, 60 },
    { 0x49da0000, 43 },
    { 0x595a0001, 6 },
    { 0xd36a0002, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7a0007, 61 },
    { 0x7eaa0008, 33 },
    { 0, 0 },
    { 0x4bda000a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34a000d, 23 },
    { 0xd51a000d, 95 },
    { 0x9e1a000f, 34 },
    { 0x10aa0010, 23 },
    { 0xa7a0010, 61 },
    { 0, 0 },
    { 0x957a0013, 98 },
    { 0xe70a0013, 95 },
    { 0x874a0015, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2a0019, 95 },
    { 0x3c6a0019, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x207a001d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6a0023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x988a0026, 90 },
    { 0x26aa0026, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81a002b, 95 },
    { 0, 0 },
    { 0x83ca002d, 85 },
    { 0x846a002d, 98 },
    { 0, 0 },
    { 0x4cda0030, 33 },
    { 0x40aa0031, 84 },
    { 0, 0 },
    { 0xd5fa0033, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a0038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72aa003d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0a0042, 95 },
    { 0, 0 },
    { 0x836a0044, 61 },
    { 0x43ea0044, 23 },
    { 0xf78a0046, 33 },
    { 0xac8a0047, 46 },
    { 0, 0 },
    { 0x16fa0049, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ca004d, 55 },
    { 0x71fa004d, 33 },
    { 0x923a004e, 75 },
    { 0xe64a0050, 52 },
    { 0xca7a004f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fa0057, 51 },
    { 0xf13a0058, 96 },
    { 0xf61a0059, 55 },
    { 0xa0058, 76 },
    { 0x214a0058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedca005f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a0065, 87 },
    { 0xf03a0066, 90 },
    { 0x5c2a0067, 29 },
    { 0, 0 },
    { 0x256a0069, 91 },
    { 0xbcda006a, 90 },
    { 0x750a006a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a0070, 52 },
    { 0x9c0a0071, 46 },
    { 0x873a0070, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a0075, 66 },
    { 0, 0 },
    { 0xd5ca0077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67a007a, 91 },
    { 0x1f9a007a, 55 },
    { 0x8baa007c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a0085, 33 },
    { 0x1cda0085, 23 },
    { 0x363a0087, 98 },
    { 0xb6aa0088, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05a008c, 78 },
    { 0, 0 },
    { 0x364a008e, 76 },
    { 0xf8ca008f, 43 },
    { 0x40a008f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a0096, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230a00a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea00a3, 33 },
    { 0x53ba00a4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x703a00a7, 85 },
    { 0xa00a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3a00ac, 43 },
    { 0, 0 },
    { 0x839a00ae, 95 },
    { 0xe25a00af, 49 },
    { 0x44ba00b0, 95 },
    { 0xaea00ae, 90 },
    { 0x8dba00b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa00b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba00b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ca00bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x186a00c0, 23 },
    { 0x419a00c0, 90 },
    { 0, 0 },
    { 0xc4ea00c3, 46 },
    { 0xe1a00c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa00c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ea00ce, 61 },
    { 0x1b3a00cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939a00d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7a00d8, 91 },
    { 0x14ba00d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649a00de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3a00e5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x324a00e8, 34 },
    { 0xfb8a00e8, 34 },
    { 0, 0 },
    { 0xdd9a00eb, 90 },
    { 0x749a00ec, 98 },
    { 0xc63a00eb, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x694a00f0, 90 },
    { 0xffaa00f0, 98 },
    { 0xe6da00f1, 40 },
    { 0xd34a00f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61da00fc, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa0101, 90 },
    { 0xc8ba0101, 95 },
    { 0x734a0103, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ba0107, 87 },
    { 0x2b3a0108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a010b, 23 },
    { 0xd91a010c, 90 },
    { 0, 0 },
    { 0xc16a010e, 33 },
    { 0x915a010f, 90 },
    { 0xde9a010e, 61 },
    { 0, 0 },
    { 0x26ea0112, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a0115, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba011a, 61 },
    { 0, 0 },
    { 0x6bea011c, 98 },
    { 0xc06a011c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848a0121, 33 },
    { 0xdfca0122, 33 },
    { 0x244a0123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85a0126, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9a0129, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51a012f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854a0133, 90 },
    { 0, 0 },
    { 0xda2a0135, 61 },
    { 0x94ea0136, 61 },
    { 0x23aa0136, 95 },
    { 0, 0 },
    { 0xff9a0139, 75 },
    { 0, 0 },
    { 0xd31a013b, 43 },
    { 0, 0 },
    { 0x503a013d, 43 },
    { 0x15ba013d, 66 },
    { 0x3f2a013f, 95 },
    { 0xf7ea013f, 33 },
    { 0xfa1a0140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fca0144, 75 },
    { 0, 0 },
    { 0xd83a0146, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8da014c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64aa0150, 67 },
    { 0, 0 },
    { 0xa0ca0152, 93 },
    { 0, 0 },
    { 0x20da0154, 68 },
    { 0x781a0155, 33 },
    { 0xff1a0156, 78 },
    { 0x75fa0154, 98 },
    { 0, 0 },
    { 0x12ca0159, 65 },
    { 0x272a0159, 93 },
    { 0x11da015a, 70 },
    { 0, 0 },
    { 0x1efa015d, 55 },
    { 0x7f0a015e, 95 },
    { 0x89da015f, 61 },
    { 0xaf5a015d, 95 },
    { 0x4b0a015f, 23 },
    { 0xf0ea0162, 61 },
    { 0xb94a0163, 33 },
    { 0x385a0164, 90 },
    { 0, 0 },
    { 0x3a1a0166, 46 },
    { 0xacda0166, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ca016d, 76 },
    { 0xab9a016e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0a0176, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ca017b, 78 },
    { 0xbb7a017b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ea0180, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5a0183, 78 },
    { 0x6d6a0183, 23 },
    { 0xa39a0183, 67 },
    { 0x85aa0184, 90 },
    { 0x49da0184, 100 },
    { 0x5fca0188, 61 },
    { 0x458a0184, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ca0193, 78 },
    { 0xf03a0194, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400a0198, 95 },
    { 0, 0 },
    { 0x655a019a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802a019e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23a01ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7a01b2, 46 },
    { 0xdd6a01b2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a01b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x132a01b9, 76 },
    { 0x267a01ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a01bf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ea01c4, 66 },
    { 0x9b7a01c4, 75 },
    { 0, 0 },
    { 0xd5fa01c7, 84 },
    { 0, 0 },
    { 0xf7ba01c9, 43 },
    { 0xcd3a01ca, 23 },
    { 0, 0 },
    { 0x34aa01cc, 23 },
    { 0x1cca01cc, 95 },
    { 0x998a01cd, 61 },
    { 0x9bda01cf, 87 },
    { 0x28ba01d0, 90 },
    { 0x812a01d1, 23 },
    { 0xfa3a01d2, 23 },
    { 0, 0 },
    { 0xebfa01d4, 84 },
    { 0x1bea01d5, 90 },
    { 0x4c3a01d6, 46 },
    { 0xf00a01d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fa01db, 95 },
    { 0xe24a01dc, 40 },
    { 0, 0 },
    { 0x967a01de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12a01e1, 43 },
    { 0x1f8a01e2, 61 },
    { 0xea0a01e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca01e6, 43 },
    { 0, 0 },
    { 0x265a01e8, 76 },
    { 0x74ea01e8, 100 },
    { 0x4fea01e8, 90 },
    { 0x157a01e9, 61 },
    { 0, 0 },
    { 0xa32a01ed, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a01f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a01f4, 33 },
    { 0, 0 },
    { 0xe45a01f6, 100 },
    { 0x747a01f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0aa01fa, 33 },
    { 0, 0 },
    { 0x7daa01fc, 61 },
    { 0, 0 },
    { 0x21ea01fe, 95 },
    { 0, 0 },
    { 0x927a0200, 23 },
    { 0x26fa0201, 61 },
    { 0, 0 },
    { 0x982a0203, 95 },
    { 0x82ca0204, 23 },
    { 0xd7ea0205, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9a0209, 98 },
    { 0, 0 },
    { 0x8f6a020b, 46 },
    { 0x22da020b, 98 },
    { 0x694a020d, 33 },
    { 0, 0 },
    { 0xf26a020f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998a0213, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174a0218, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba021c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fa0226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bba022d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a0230, 95 },
    { 0x92ba0231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca0235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da023f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6a0243, 46 },
    { 0, 0 },
    { 0x303a0245, 78 },
    { 0x9bda0246, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a0249, 33 },
    { 0xc8aa024a, 56 },
    { 0xd9ea0249, 23 },
    { 0x40ca024c, 34 },
    { 0x412a024d, 61 },
    { 0xdefa024e, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a0252, 98 },
    { 0x313a0253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9a0256, 99 },
    { 0x373a0257, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78a025b, 51 },
    { 0x191a025c, 46 },
    { 0, 0 },
    { 0x7baa025e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05a0262, 91 },
    { 0x8fea0263, 33 },
    { 0x42fa0264, 61 },
    { 0, 0 },
    { 0x3c4a0266, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76a026a, 98 },
    { 0xb38a026b, 61 },
    { 0xf49a026c, 98 },
    { 0x812a026d, 33 },
    { 0x916a026a, 33 },
    { 0, 0 },
    { 0x58fa0270, 75 },
    { 0xea4a0270, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea0277, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fa027a, 90 },
    { 0, 0 },
    { 0x5ffa027c, 61 },
    { 0x6dea027c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91a0283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33a028c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a028f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca0292, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ba0298, 77 },
    { 0x20a0299, 61 },
    { 0xfc9a0299, 23 },
    { 0x8ea029b, 61 },
    { 0x35ca029c, 33 },
    { 0, 0 },
    { 0x669a029e, 90 },
    { 0, 0 },
    { 0x39da02a0, 78 },
    { 0, 0 },
    { 0xa4ca02a2, 61 },
    { 0xf2ea02a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a02a7, 69 },
    { 0, 0 },
    { 0x724a02a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904a02b2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa02b9, 76 },
    { 0xac2a02ba, 34 },
    { 0, 0 },
    { 0x24aa02bc, 90 },
    { 0, 0 },
    { 0xca0a02be, 33 },
    { 0xb7fa02bf, 23 },
    { 0x8e0a02c0, 62 },
    { 0x5cba02c0, 98 },
    { 0x3c3a02c0, 23 },
    { 0x313a02c3, 76 },
    { 0, 0 },
    { 0x676a02c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeba02cb, 23 },
    { 0x947a02cc, 33 },
    { 0xb74a02cb, 55 },
    { 0xfa7a02cc, 69 },
    { 0x9e0a02cf, 34 },
    { 0x88aa02cf, 43 },
    { 0x813a02cf, 98 },
    { 0x451a02d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ba02d5, 33 },
    { 0, 0 },
    { 0x5a8a02d7, 46 },
    { 0x4d8a02d7, 61 },
    { 0, 0 },
    { 0x617a02da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caa02dd, 95 },
    { 0, 0 },
    { 0xd43a02df, 34 },
    { 0x679a02e0, 33 },
    { 0x4e2a02e1, 48 },
    { 0xa2ba02e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7a02e5, 43 },
    { 0, 0 },
    { 0xb9a02e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2a02ed, 75 },
    { 0xbeea02ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba02f4, 77 },
    { 0xbfda02f5, 66 },
    { 0x76aa02f4, 43 },
    { 0x18aa02f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a02fc, 95 },
    { 0x198a02fd, 98 },
    { 0xb0da02fd, 23 },
    { 0x992a02ff, 61 },
    { 0x18ea0300, 33 },
    { 0x81ca0300, 38 },
    { 0xcf0a0302, 95 },
    { 0xbdca0303, 34 },
    { 0x395a0303, 93 },
    { 0x154a0305, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a030c, 33 },
    { 0x58da030c, 91 },
    { 0xbc1a030e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fa0316, 46 },
    { 0x7ffa0316, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a031b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a0322, 76 },
    { 0, 0 },
    { 0xbd9a0324, 99 },
    { 0, 0 },
    { 0x3b4a0326, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a0329, 76 },
    { 0, 0 },
    { 0x827a032b, 33 },
    { 0xeb9a032c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86a0333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ca033d, 90 },
    { 0xd40a033e, 43 },
    { 0, 0 },
    { 0x757a0340, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4a0345, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ba034a, 33 },
    { 0xf90a034b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64a034e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5a0352, 59 },
    { 0x3f0a0353, 98 },
    { 0, 0 },
    { 0xe1a0355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2a0358, 61 },
    { 0x8f7a0358, 23 },
    { 0xedba035a, 33 },
    { 0x3a5a035b, 66 },
    { 0xbfea035c, 33 },
    { 0xdcda035d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea0363, 87 },
    { 0xad9a0364, 99 },
    { 0x120a0365, 90 },
    { 0x255a0364, 91 },
    { 0x1d5a0367, 61 },
    { 0x32da0367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fa036e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61a0371, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ea0375, 88 },
    { 0, 0 },
    { 0x78ba0377, 33 },
    { 0x39da0378, 33 },
    { 0x693a0379, 23 },
    { 0x186a0379, 95 },
    { 0xe41a037a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a037f, 66 },
    { 0x181a0380, 88 },
    { 0, 0 },
    { 0x12da0382, 61 },
    { 0, 0 },
    { 0x6fca0384, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06a0387, 61 },
    { 0x913a0388, 77 },
    { 0xd05a0389, 90 },
    { 0, 0 },
    { 0x5dca038b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a0391, 23 },
    { 0x266a0392, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a0396, 90 },
    { 0x265a0396, 84 },
    { 0, 0 },
    { 0x94ba0399, 76 },
    { 0, 0 },
    { 0x313a039b, 85 },
    { 0, 0 },
    { 0x403a039d, 23 },
    { 0xcdfa039e, 17 },
    { 0xf27a039d, 82 },
    { 0, 0 },
    { 0xa7aa03a1, 43 },
    { 0x8dea03a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaa03a5, 90 },
    { 0, 0 },
    { 0x1c8a03a7, 33 },
    { 0x52ba03a8, 33 },
    { 0x592a03a9, 43 },
    { 0xf57a03a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a03ad, 95 },
    { 0x352a03ae, 95 },
    { 0x20ca03ae, 46 },
    { 0x978a03b0, 90 },
    { 0x209a03b1, 76 },
    { 0, 0 },
    { 0x2dba03b3, 77 },
    { 0xbe7a03b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15a03b9, 98 },
    { 0x9cca03ba, 76 },
    { 0, 0 },
    { 0x9cfa03bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a03bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302a03c5, 33 },
    { 0, 0 },
    { 0xcafa03c7, 43 },
    { 0x8e8a03c8, 98 },
    { 0x72da03c7, 78 },
    { 0x9e6a03ca, 91 },
    { 0x293a03cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34aa03d2, 23 },
    { 0x6d3a03d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27a03d6, 98 },
    { 0x5e7a03d7, 84 },
    { 0x5a5a03d8, 33 },
    { 0x905a03d7, 87 },
    { 0xca03d8, 90 },
    { 0x2e1a03d7, 90 },
    { 0x724a03dc, 90 },
    { 0xb0ba03d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29a03e2, 34 },
    { 0x72ca03e2, 33 },
    { 0xaf4a03e3, 98 },
    { 0x583a03e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53aa03e8, 33 },
    { 0x458a03e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621a03f0, 90 },
    { 0x250a03f1, 98 },
    { 0xebca03f2, 33 },
    { 0xf9da03f1, 80 },
    { 0xb89a03f4, 76 },
    { 0, 0 },
    { 0x264a03f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea03fb, 17 },
    { 0x28a03fc, 34 },
    { 0, 0 },
    { 0xd7ca03fe, 98 },
    { 0xf25a03ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a0403, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a0411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93a0415, 61 },
    { 0x190a0416, 66 },
    { 0, 0 },
    { 0x5a4a0418, 96 },
    { 0, 0 },
    { 0xd03a041a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90a041d, 61 },
    { 0xd02a041e, 76 },
    { 0xf3fa041e, 67 },
    { 0x476a0420, 95 },
    { 0x141a041f, 56 },
    { 0x9e6a0422, 61 },
    { 0x50fa0423, 93 },
    { 0xfcba0424, 90 },
    { 0x4ca0424, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758a0429, 23 },
    { 0, 0 },
    { 0xf7a042b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea042f, 23 },
    { 0xfe9a042f, 78 },
    { 0, 0 },
    { 0x45fa0432, 95 },
    { 0, 0 },
    { 0x33ca0434, 95 },
    { 0x94aa0435, 95 },
    { 0, 0 },
    { 0x517a0437, 46 },
    { 0, 0 },
    { 0xde9a0439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bba0445, 90 },
    { 0xdf0a0445, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a044c, 87 },
    { 0xd21a044d, 76 },
    { 0x3ea044d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6a0452, 81 },
    { 0xb59a0453, 55 },
    { 0x6d7a0452, 98 },
    { 0xa4ca0454, 74 },
    { 0x6a7a0456, 91 },
    { 0x1cfa0452, 46 },
    { 0x56aa0458, 90 },
    { 0xcca0459, 23 },
    { 0xe7ca0453, 33 },
    { 0x8d9a045b, 33 },
    { 0, 0 },
    { 0x1d2a045d, 23 },
    { 0xeaba045e, 78 },
    { 0x99fa045f, 76 },
    { 0x9fea0460, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8a046e, 93 },
    { 0x1eca046e, 77 },
    { 0x5fa046e, 90 },
    { 0xfbba046f, 90 },
    { 0xf10a046e, 23 },
    { 0x5a4a0472, 33 },
    { 0x592a046e, 43 },
    { 0xaaa0475, 78 },
    { 0x636a0476, 33 },
    { 0x7eba0477, 95 },
    { 0x229a0477, 34 },
    { 0x4f7a0479, 76 },
    { 0x2e1a047a, 33 },
    { 0x652a047b, 61 },
    { 0x565a047a, 90 },
    { 0xceba0479, 98 },
    { 0, 0 },
    { 0x45ba047f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a0483, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ea0486, 98 },
    { 0xbcca0487, 91 },
    { 0, 0 },
    { 0x4a5a0489, 66 },
    { 0x93a048a, 61 },
    { 0x492a0489, 87 },
    { 0x304a048c, 33 },
    { 0xec5a048d, 76 },
    { 0xe49a048d, 99 },
    { 0x504a048a, 33 },
    { 0x48da0490, 90 },
    { 0x155a0491, 17 },
    { 0x42ea0490, 90 },
    { 0x907a0493, 84 },
    { 0, 0 },
    { 0xb8da0495, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabda049b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aea049e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4a04a5, 98 },
    { 0, 0 },
    { 0x4e1a04a7, 95 },
    { 0x55ca04a8, 33 },
    { 0xa31a04a7, 84 },
    { 0x4ca04aa, 23 },
    { 0x588a04ab, 55 },
    { 0, 0 },
    { 0x66ca04ad, 90 },
    { 0, 0 },
    { 0xb6ca04af, 61 },
    { 0xf4fa04b0, 100 },
    { 0x371a04b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba04b4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284a04b9, 100 },
    { 0x789a04b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a04be, 52 },
    { 0, 0 },
    { 0x6d4a04c0, 23 },
    { 0, 0 },
    { 0xd47a04c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7a04c7, 76 },
    { 0xdbea04c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a04cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7a04d0, 95 },
    { 0, 0 },
    { 0xef0a04d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaa04d5, 55 },
    { 0, 0 },
    { 0xf1fa04d7, 23 },
    { 0x330a04d8, 23 },
    { 0, 0 },
    { 0x9b8a04da, 60 },
    { 0x7e0a04db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ea04e1, 33 },
    { 0x294a04e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03a04eb, 34 },
    { 0x727a04ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93a04ef, 90 },
    { 0, 0 },
    { 0x199a04f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a04f7, 55 },
    { 0xaf0a04f8, 68 },
    { 0xad1a04f8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694a0500, 90 },
    { 0x9eca0501, 98 },
    { 0xef4a0502, 61 },
    { 0xd30a0503, 76 },
    { 0x86aa0502, 90 },
    { 0x67da0505, 52 },
    { 0xb25a0506, 78 },
    { 0, 0 },
    { 0x416a0508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1a050d, 76 },
    { 0xdc2a050e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7a0514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a0518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaca051b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ea0523, 75 },
    { 0, 0 },
    { 0xe0ba0525, 95 },
    { 0x70fa0526, 95 },
    { 0x935a0526, 66 },
    { 0x202a0525, 76 },
    { 0x591a0527, 98 },
    { 0x38ba0528, 85 },
    { 0xbf6a0525, 76 },
    { 0xc66a0528, 84 },
    { 0x7e3a052d, 90 },
    { 0xf8ba052e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa0533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeea0536, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa0540, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fa0548, 99 },
    { 0xaeba0548, 90 },
    { 0x5f4a054a, 93 },
    { 0xdea054b, 84 },
    { 0, 0 },
    { 0x7c2a054d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a0550, 90 },
    { 0, 0 },
    { 0xd98a0552, 34 },
    { 0, 0 },
    { 0x7b2a0554, 23 },
    { 0xaa4a0555, 23 },
    { 0x468a0556, 55 },
    { 0x88fa0557, 98 },
    { 0x4a3a0556, 61 },
    { 0xb4ea0555, 55 },
    { 0, 0 },
    { 0xb36a055b, 96 },
    { 0xd68a055b, 67 },
    { 0x1a3a055d, 46 },
    { 0x9f9a055d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cda0564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3a0567, 67 },
    { 0, 0 },
    { 0xddfa0569, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309a056e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfa0575, 95 },
    { 0, 0 },
    { 0xfe7a0577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441a057d, 51 },
    { 0x154a057e, 87 },
    { 0, 0 },
    { 0x121a0580, 95 },
    { 0x3c9a0581, 95 },
    { 0, 0 },
    { 0xb28a0583, 76 },
    { 0xa9a0584, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea058c, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8da0592, 23 },
    { 0, 0 },
    { 0x4e2a0594, 90 },
    { 0, 0 },
    { 0xfbca0596, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x708a0599, 61 },
    { 0, 0 },
    { 0x819a059b, 93 },
    { 0x853a059c, 36 },
    { 0x2b3a059d, 87 },
    { 0xfc1a059e, 93 },
    { 0x186a059e, 34 },
    { 0x969a059f, 55 },
    { 0x730a05a1, 43 },
    { 0, 0 },
    { 0x175a05a3, 90 },
    { 0xbeca05a4, 33 },
    { 0, 0 },
    { 0x890a05a6, 90 },
    { 0, 0 },
    { 0xb2ca05a8, 52 },
    { 0, 0 },
    { 0x34fa05aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ea05ae, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a05b1, 33 },
    { 0x2b5a05b2, 76 },
    { 0xcb5a05b3, 98 },
    { 0x441a05b4, 84 },
    { 0xef7a05b5, 52 },
    { 0x43a05b4, 100 },
    { 0x296a05b7, 83 },
    { 0x521a05b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a05bb, 99 },
    { 0xb66a05bc, 33 },
    { 0x642a05bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6a05c0, 90 },
    { 0, 0 },
    { 0xa27a05c2, 76 },
    { 0x8a0a05c2, 43 },
    { 0x476a05c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114a05c8, 34 },
    { 0x311a05c9, 64 },
    { 0x367a05ca, 55 },
    { 0xa71a05c8, 98 },
    { 0xaeda05c9, 83 },
    { 0xfe0a05cd, 43 },
    { 0x651a05ce, 95 },
    { 0xebfa05ce, 33 },
    { 0, 0 },
    { 0xfbea05d1, 36 },
    { 0, 0 },
    { 0xd93a05d3, 23 },
    { 0, 0 },
    { 0x968a05d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870a05df, 90 },
    { 0, 0 },
    { 0x61fa05e1, 23 },
    { 0x63ea05e1, 90 },
    { 0x7b1a05e2, 23 },
    { 0xd40a05e3, 55 },
    { 0x461a05e3, 33 },
    { 0x1dfa05e2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20a05ef, 90 },
    { 0xb26a05f0, 52 },
    { 0xb8ba05f1, 90 },
    { 0x75da05f1, 23 },
    { 0x4b8a05f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a05f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x455a05f9, 95 },
    { 0x5e0a05fa, 43 },
    { 0xb37a05fb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98a0604, 23 },
    { 0xbcca0605, 61 },
    { 0, 0 },
    { 0xe5da0607, 23 },
    { 0xbc5a0608, 98 },
    { 0xf0da0609, 93 },
    { 0x40fa0607, 98 },
    { 0xae6a060b, 56 },
    { 0, 0 },
    { 0xf9ba060d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32a0610, 23 },
    { 0x39ea0610, 90 },
    { 0x9d7a0610, 95 },
    { 0x38a0613, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9a0617, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3a061a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea061d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a062a, 46 },
    { 0x677a062b, 87 },
    { 0, 0 },
    { 0xc44a062d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea0630, 100 },
    { 0, 0 },
    { 0x56ba0632, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faa0637, 91 },
    { 0, 0 },
    { 0xe4aa0639, 90 },
    { 0x3c8a063a, 43 },
    { 0x87fa063a, 76 },
    { 0x72ea063a, 87 },
    { 0xbe2a063d, 90 },
    { 0, 0 },
    { 0x46ea063f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9a0642, 85 },
    { 0x73aa0642, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ea064e, 69 },
    { 0, 0 },
    { 0x92a0650, 52 },
    { 0xefca0651, 55 },
    { 0x54ba0650, 95 },
    { 0x12ca0650, 46 },
    { 0xdb3a0653, 99 },
    { 0, 0 },
    { 0xd95a0656, 46 },
    { 0, 0 },
    { 0xfe5a0658, 95 },
    { 0xacda0659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1aa065c, 61 },
    { 0xeb8a065d, 46 },
    { 0xdc2a065e, 23 },
    { 0x42da065f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ca0669, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6a0670, 88 },
    { 0xbb4a0670, 87 },
    { 0x444a0671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bba0675, 61 },
    { 0x587a0676, 8 },
    { 0xbe7a0676, 55 },
    { 0x13ea0678, 23 },
    { 0x2d2a0679, 93 },
    { 0x2a5a0676, 87 },
    { 0xefba067b, 61 },
    { 0x28da067c, 95 },
    { 0, 0 },
    { 0xb48a067e, 85 },
    { 0xe93a067e, 99 },
    { 0xd4a0680, 9 },
    { 0x81ba0681, 34 },
    { 0, 0 },
    { 0x8d9a0683, 93 },
    { 0xa0683, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5a068b, 61 },
    { 0x4c5a068b, 34 },
    { 0x33ea068d, 33 },
    { 0, 0 },
    { 0x487a068f, 33 },
    { 0, 0 },
    { 0x6a9a0691, 34 },
    { 0xa3da0692, 61 },
    { 0, 0 },
    { 0xc22a0694, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2a0697, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403a069b, 90 },
    { 0, 0 },
    { 0x37ea069d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffa06a1, 100 },
    { 0x90ca06a2, 68 },
    { 0xf4a06a3, 23 },
    { 0xa7fa06a2, 90 },
    { 0x40ca06a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266a06ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a06b1, 61 },
    { 0x8ffa06b2, 66 },
    { 0, 0 },
    { 0x3a3a06b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efa06bc, 81 },
    { 0x247a06bc, 90 },
    { 0x77ba06bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0a06c2, 95 },
    { 0x349a06c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3a06c7, 33 },
    { 0xe54a06c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ba06cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843a06d1, 98 },
    { 0x91aa06d2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574a06df, 33 },
    { 0x760a06e0, 76 },
    { 0, 0 },
    { 0xdf7a06e2, 76 },
    { 0, 0 },
    { 0xe63a06e4, 87 },
    { 0, 0 },
    { 0xa81a06e6, 68 },
    { 0xb24a06e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61da06eb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a06ef, 48 },
    { 0xa94a06f0, 61 },
    { 0xcf0a06ef, 99 },
    { 0xf74a06f1, 61 },
    { 0xeb7a06ef, 69 },
    { 0, 0 },
    { 0xefa06f5, 33 },
    { 0x112a06f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fa06fb, 23 },
    { 0, 0 },
    { 0xfb8a06fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50a0700, 36 },
    { 0, 0 },
    { 0xc5da0702, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fa0706, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95a070b, 43 },
    { 0, 0 },
    { 0xb5fa070d, 90 },
    { 0x9d1a070d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a0712, 52 },
    { 0x6e9a0713, 77 },
    { 0, 0 },
    { 0x6f9a0715, 23 },
    { 0xc24a0715, 81 },
    { 0xd6ea0716, 23 },
    { 0xa3ea0717, 33 },
    { 0x29aa0719, 98 },
    { 0, 0 },
    { 0xa56a071b, 43 },
    { 0, 0 },
    { 0xe0ca071d, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0x319a0720, 95 },
    { 0x11fa0721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x521a0724, 88 },
    { 0x753a0724, 33 },
    { 0, 0 },
    { 0x5eba0727, 33 },
    { 0x3c5a0727, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa072d, 76 },
    { 0, 0 },
    { 0xb83a072f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baa0734, 76 },
    { 0, 0 },
    { 0x1d1a0736, 96 },
    { 0xc2da0737, 76 },
    { 0x251a0738, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa073c, 78 },
    { 0x1aba073c, 98 },
    { 0x213a073e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aa0744, 19 },
    { 0x7bfa0745, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fea0749, 61 },
    { 0x8ea0749, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eda074d, 52 },
    { 0x1dca074e, 90 },
    { 0x22aa074d, 33 },
    { 0xd60a074f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30a0754, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6da0757, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6da075a, 33 },
    { 0xed7a075a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a0763, 91 },
    { 0x652a0764, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a0768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xada076b, 43 },
    { 0xac4a076b, 90 },
    { 0x587a076d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79a0772, 93 },
    { 0x47a0773, 95 },
    { 0, 0 },
    { 0x8c9a0775, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa077e, 23 },
    { 0x9cda077f, 33 },
    { 0, 0 },
    { 0xfd6a0781, 64 },
    { 0x18da0782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3a0788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8a078b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7a078e, 90 },
    { 0x3eba078f, 98 },
    { 0x1ea078e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a0795, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ca0799, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7a07a1, 90 },
    { 0, 0 },
    { 0x13ba07a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a07a7, 75 },
    { 0x4a7a07a8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20a07ad, 33 },
    { 0xf69a07ae, 90 },
    { 0xc51a07ae, 95 },
    { 0xf3fa07b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6a07b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7a07bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ba07bf, 95 },
    { 0x13a07c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3a07c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0a07ce, 76 },
    { 0, 0 },
    { 0xb3a07d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45a07d9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93a07df, 90 },
    { 0x86a07e0, 66 },
    { 0x3e0a07e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a07e7, 90 },
    { 0x713a07e8, 95 },
    { 0, 0 },
    { 0x373a07ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fa07ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea07f8, 91 },
    { 0x287a07f9, 61 },
    { 0x1b7a07f9, 33 },
    { 0, 0 },
    { 0x8f2a07fc, 95 },
    { 0, 0 },
    { 0x9aa07fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45da0802, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743a0806, 23 },
    { 0xc2fa0807, 77 },
    { 0xa93a0808, 95 },
    { 0x11ca0809, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9a0810, 67 },
    { 0, 0 },
    { 0xfaea0812, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ba0818, 23 },
    { 0x631a0818, 84 },
    { 0, 0 },
    { 0xda9a081b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637a081f, 78 },
    { 0xce2a0820, 76 },
    { 0x6e1a0820, 33 },
    { 0xd8ca0822, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeda0829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76a082f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4a0834, 99 },
    { 0x1f4a0835, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782a083c, 64 },
    { 0x200a083c, 33 },
    { 0x518a083e, 90 },
    { 0x364a083f, 95 },
    { 0x1dca083f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2a084a, 95 },
    { 0xcaba084a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fa084e, 99 },
    { 0, 0 },
    { 0xdd6a0850, 76 },
    { 0x2b1a0851, 90 },
    { 0x39da0852, 23 },
    { 0x650a0851, 43 },
    { 0xacca0854, 19 },
    { 0x290a0854, 90 },
    { 0x2dba0856, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ea0859, 53 },
    { 0x5d1a0859, 23 },
    { 0x30ca0859, 100 },
    { 0x39ea085b, 55 },
    { 0xb4ca085b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a0864, 98 },
    { 0, 0 },
    { 0x595a0866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a086b, 90 },
    { 0x4d0a086b, 55 },
    { 0xb54a086d, 95 },
    { 0, 0 },
    { 0xcc9a086f, 90 },
    { 0x825a0870, 95 },
    { 0x303a0871, 52 },
    { 0x4ca0870, 90 },
    { 0xf10a0873, 34 },
    { 0xf9ba0874, 33 },
    { 0xf03a0875, 98 },
    { 0x40aa0875, 33 },
    { 0x292a0877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6a0881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6a0885, 34 },
    { 0x755a0886, 93 },
    { 0xeaba0885, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2a088a, 91 },
    { 0, 0 },
    { 0x928a088c, 43 },
    { 0xd89a088d, 95 },
    { 0, 0 },
    { 0xb5da088f, 98 },
    { 0xcc8a0890, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaa0893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0897, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaa089a, 91 },
    { 0, 0 },
    { 0x467a089c, 90 },
    { 0, 0 },
    { 0x1c0a089e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92da08a2, 90 },
    { 0xf16a08a2, 61 },
    { 0xd3a08a4, 34 },
    { 0x375a08a2, 90 },
    { 0, 0 },
    { 0xe96a08a7, 67 },
    { 0, 0 },
    { 0x72ca08a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2a08b3, 61 },
    { 0x2bca08b3, 52 },
    { 0x14ea08b5, 76 },
    { 0, 0 },
    { 0x4ba08b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75a08bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8a08c4, 95 },
    { 0, 0 },
    { 0x859a08c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0a08c9, 33 },
    { 0xae7a08ca, 95 },
    { 0x127a08cb, 61 },
    { 0xabda08cc, 87 },
    { 0x276a08ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aca08d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aa08da, 95 },
    { 0x8a0a08db, 55 },
    { 0xff5a08db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65aa08e3, 98 },
    { 0, 0 },
    { 0x56da08e5, 95 },
    { 0x87ea08e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31da08ec, 95 },
    { 0, 0 },
    { 0xcd8a08ee, 43 },
    { 0x110a08ef, 61 },
    { 0, 0 },
    { 0xd15a08f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844a08f6, 43 },
    { 0xfc6a08f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946a08fb, 33 },
    { 0x7e2a08fc, 100 },
    { 0, 0 },
    { 0xebfa08fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa0903, 95 },
    { 0, 0 },
    { 0x4a1a0905, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13a0909, 75 },
    { 0xe1ba090a, 46 },
    { 0x384a0909, 40 },
    { 0x873a090c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7a0914, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dca0918, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15a091b, 98 },
    { 0x95ea091c, 23 },
    { 0, 0 },
    { 0x4a6a091e, 33 },
    { 0xfe9a091f, 95 },
    { 0, 0 },
    { 0x1b4a0921, 95 },
    { 0x624a0922, 33 },
    { 0xee6a0923, 61 },
    { 0x344a0924, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48a092a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63da092d, 88 },
    { 0x6ca092e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754a0932, 55 },
    { 0, 0 },
    { 0xd1da0934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211a0938, 95 },
    { 0, 0 },
    { 0x49aa093a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a0943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ea0946, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48a0949, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fa094d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98a0951, 61 },
    { 0, 0 },
    { 0xad1a0953, 91 },
    { 0xb72a0953, 46 },
    { 0xa32a0953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a0959, 61 },
    { 0x438a095a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04a095d, 33 },
    { 0xe4da095e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a0961, 61 },
    { 0x236a0961, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x622a0965, 55 },
    { 0x22ba0965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429a096b, 90 },
    { 0x6bea096c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0a0974, 55 },
    { 0xceca0975, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4a0979, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ba097e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11a0983, 33 },
    { 0x49ca0984, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a0988, 90 },
    { 0x4e9a0988, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2a098c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566a0991, 61 },
    { 0, 0 },
    { 0x994a0993, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x601a0996, 100 },
    { 0x770a0997, 55 },
    { 0xd86a0998, 61 },
    { 0x44aa0996, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579a09a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a09a3, 95 },
    { 0x1c5a09a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08a09a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a09aa, 43 },
    { 0, 0 },
    { 0x3a5a09ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x390a09af, 98 },
    { 0xaafa09b0, 98 },
    { 0, 0 },
    { 0x7fea09b2, 67 },
    { 0, 0 },
    { 0xbf2a09b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3a09b7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fa09bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ba09c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ea09c7, 23 },
    { 0xa02a09c7, 33 },
    { 0x34aa09c9, 76 },
    { 0xa00a09ca, 77 },
    { 0x4e9a09cb, 23 },
    { 0x22ba09cc, 66 },
    { 0xab6a09c9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412a09d9, 33 },
    { 0x6dfa09da, 99 },
    { 0, 0 },
    { 0x351a09dc, 91 },
    { 0xbe5a09dd, 61 },
    { 0x4f8a09dd, 34 },
    { 0x7a4a09dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x213a09e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9a09ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13a09ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ea09f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37a09f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560a0a00, 94 },
    { 0, 0 },
    { 0x513a0a02, 66 },
    { 0x8e4a0a02, 98 },
    { 0x8fda0a04, 67 },
    { 0xc68a0a05, 95 },
    { 0x227a0a06, 98 },
    { 0x349a0a04, 43 },
    { 0, 0 },
    { 0xa09a0a09, 99 },
    { 0x33fa0a0a, 23 },
    { 0, 0 },
    { 0x9ca0a0c, 33 },
    { 0xae5a0a0d, 78 },
    { 0xf9da0a0d, 90 },
    { 0, 0 },
    { 0xbd5a0a10, 76 },
    { 0x75da0a10, 33 },
    { 0xd11a0a12, 33 },
    { 0xe57a0a13, 55 },
    { 0xaa2a0a11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afa0a19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65da0a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca0a21, 61 },
    { 0, 0 },
    { 0xf56a0a23, 90 },
    { 0, 0 },
    { 0x85ca0a25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405a0a2c, 90 },
    { 0, 0 },
    { 0x412a0a2e, 95 },
    { 0, 0 },
    { 0x60ea0a30, 40 },
    { 0, 0 },
    { 0x160a0a32, 52 },
    { 0, 0 },
    { 0x1b4a0a34, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1a0a37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea0a3a, 78 },
    { 0xdb6a0a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fa0a3e, 95 },
    { 0x3bfa0a3f, 17 },
    { 0, 0 },
    { 0x43aa0a41, 93 },
    { 0x3b9a0a41, 52 },
    { 0x7d7a0a43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75da0a49, 76 },
    { 0x9f7a0a4a, 95 },
    { 0, 0 },
    { 0x744a0a4c, 93 },
    { 0xaeea0a4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x408a0a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ca0a54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dba0a57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a0a5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a0a61, 99 },
    { 0x711a0a62, 90 },
    { 0xc34a0a61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fda0a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x956a0a6a, 55 },
    { 0, 0 },
    { 0x45aa0a6c, 61 },
    { 0xb04a0a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa0a77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3a0a7c, 93 },
    { 0, 0 },
    { 0x9ea0a7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x752a0a81, 95 },
    { 0x79ca0a82, 99 },
    { 0xabaa0a81, 90 },
    { 0xa68a0a84, 91 },
    { 0x726a0a83, 99 },
    { 0, 0 },
    { 0xb63a0a87, 90 },
    { 0, 0 },
    { 0x5d6a0a89, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7a0a8c, 98 },
    { 0x4a4a0a8c, 61 },
    { 0, 0 },
    { 0x949a0a8f, 76 },
    { 0, 0 },
    { 0x6b1a0a91, 77 },
    { 0xbe6a0a92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x306a0a95, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975a0a99, 46 },
    { 0xf90a0a99, 100 },
    { 0, 0 },
    { 0xb3fa0a9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7a0aa0, 90 },
    { 0x9e1a0aa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a0aa6, 91 },
    { 0xde7a0aa7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09a0aab, 61 },
    { 0xac2a0aac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0a0aaf, 90 },
    { 0x62da0aaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aa0ab4, 52 },
    { 0xcc8a0ab5, 78 },
    { 0x1cda0ab5, 90 },
    { 0x308a0ab7, 90 },
    { 0, 0 },
    { 0x26aa0ab9, 90 },
    { 0x348a0aba, 34 },
    { 0x365a0aba, 98 },
    { 0xa02a0aba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a0ac9, 33 },
    { 0x18a0aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834a0ad2, 98 },
    { 0x4ca0ad3, 61 },
    { 0xa8ca0ad2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fa0adc, 56 },
    { 0xb7fa0add, 90 },
    { 0, 0 },
    { 0xf6a0adf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x335a0ae2, 98 },
    { 0, 0 },
    { 0xf6da0ae4, 90 },
    { 0x3ada0ae5, 98 },
    { 0, 0 },
    { 0x7c8a0ae7, 99 },
    { 0xca3a0ae8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ba0aeb, 90 },
    { 0xa7da0aec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a0af0, 34 },
    { 0, 0 },
    { 0xd2fa0af2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97a0af5, 95 },
    { 0x85aa0af6, 55 },
    { 0x61ea0af7, 23 },
    { 0xcea0af7, 33 },
    { 0x61fa0af9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93a0afd, 76 },
    { 0, 0 },
    { 0x2e1a0aff, 66 },
    { 0x960a0aff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963a0b06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca0b0a, 98 },
    { 0xb29a0b0b, 33 },
    { 0, 0 },
    { 0x3e1a0b0d, 99 },
    { 0, 0 },
    { 0x85a0b0f, 33 },
    { 0, 0 },
    { 0xca9a0b11, 99 },
    { 0x74ca0b12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870a0b16, 46 },
    { 0xec1a0b17, 33 },
    { 0x6cfa0b16, 90 },
    { 0x41aa0b19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eba0b1f, 100 },
    { 0, 0 },
    { 0xeb5a0b21, 84 },
    { 0x9d8a0b22, 66 },
    { 0, 0 },
    { 0xdc1a0b24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700a0b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8a0b31, 61 },
    { 0, 0 },
    { 0xcea0b33, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8a0b37, 90 },
    { 0xd1ba0b38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4a0b3e, 33 },
    { 0, 0 },
    { 0xe22a0b40, 98 },
    { 0xb43a0b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca0b45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1a0b48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a0b4d, 61 },
    { 0xf60a0b4e, 61 },
    { 0, 0 },
    { 0x93ca0b50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12a0b56, 34 },
    { 0x2b1a0b57, 85 },
    { 0xf5fa0b56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63a0b5c, 95 },
    { 0xf82a0b5c, 95 },
    { 0x155a0b5d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x456a0b61, 33 },
    { 0, 0 },
    { 0x3bca0b63, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a0b77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19a0b81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9aa0b86, 53 },
    { 0x151a0b87, 78 },
    { 0xd4a0b88, 61 },
    { 0x83aa0b87, 23 },
    { 0x152a0b86, 75 },
    { 0, 0 },
    { 0x342a0b8c, 23 },
    { 0, 0 },
    { 0x3d2a0b8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a0b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bca0b98, 90 },
    { 0x9c7a0b99, 98 },
    { 0x239a0b9a, 61 },
    { 0, 0 },
    { 0x393a0b9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a0ba1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3a0bab, 61 },
    { 0xda0a0bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a0bb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6a0bba, 55 },
    { 0xe43a0bbb, 90 },
    { 0x1c2a0bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a0bc0, 90 },
    { 0x2aa0bc1, 61 },
    { 0, 0 },
    { 0x7a3a0bc3, 76 },
    { 0, 0 },
    { 0x78da0bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1a0bcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ea0bd5, 34 },
    { 0, 0 },
    { 0xe41a0bd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5a0bda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dba0be0, 61 },
    { 0, 0 },
    { 0x7fba0be2, 76 },
    { 0, 0 },
    { 0x5c9a0be4, 33 },
    { 0xbf0a0be5, 75 },
    { 0x131a0be6, 76 },
    { 0xc3aa0be7, 98 },
    { 0x687a0be7, 17 },
    { 0xb0aa0be7, 98 },
    { 0, 0 },
    { 0x918a0beb, 84 },
    { 0, 0 },
    { 0xd66a0bed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1a0bf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3a0bf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ca0bfc, 23 },
    { 0x9b2a0bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a0c00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41a0c0a, 46 },
    { 0xc57a0c0a, 90 },
    { 0xd6aa0c0c, 90 },
    { 0xc4aa0c0d, 61 },
    { 0, 0 },
    { 0x1d3a0c0f, 95 },
    { 0xc28a0c10, 90 },
    { 0, 0 },
    { 0x1bfa0c12, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a0c15, 90 },
    { 0x6aa0c15, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1a0c1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ca0c23, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ba0c26, 23 },
    { 0x3ffa0c27, 52 },
    { 0x614a0c26, 100 },
    { 0x3f6a0c29, 90 },
    { 0x270a0c2a, 23 },
    { 0x26fa0c28, 98 },
    { 0xb03a0c26, 75 },
    { 0xe75a0c2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eea0c35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313a0c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaa0c4e, 17 },
    { 0, 0 },
    { 0xbb1a0c50, 55 },
    { 0xe7fa0c51, 95 },
    { 0x204a0c51, 91 },
    { 0x2cfa0c53, 91 },
    { 0xd92a0c53, 61 },
    { 0x8eca0c51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba0c5a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaa0c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359a0c66, 94 },
    { 0x99aa0c66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea0c6a, 12 },
    { 0, 0 },
    { 0x70ca0c6c, 95 },
    { 0xaada0c6d, 90 },
    { 0x8b5a0c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942a0c72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5a0c76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8a0c7b, 65 },
    { 0, 0 },
    { 0xd47a0c7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0da0c81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda0c84, 81 },
    { 0xceea0c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeba0c8c, 75 },
    { 0x717a0c8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520a0c91, 66 },
    { 0x7b3a0c92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x468a0c95, 61 },
    { 0, 0 },
    { 0x183a0c97, 90 },
    { 0xaf1a0c98, 93 },
    { 0x681a0c98, 87 },
    { 0x926a0c98, 95 },
    { 0xdada0c9b, 98 },
    { 0xbc8a0c9c, 90 },
    { 0xd1aa0c9c, 98 },
    { 0, 0 },
    { 0x4a9a0c9f, 87 },
    { 0xc15a0ca0, 23 },
    { 0, 0 },
    { 0xb47a0ca2, 74 },
    { 0x7e8a0ca2, 84 },
    { 0x6eca0ca3, 84 },
    { 0xf95a0ca5, 76 },
    { 0, 0 },
    { 0xfc8a0ca7, 90 },
    { 0x788a0ca8, 61 },
    { 0x5d6a0ca9, 76 },
    { 0, 0 },
    { 0x2dea0cab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2a0cb5, 90 },
    { 0, 0 },
    { 0x2bea0cb7, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x569a0cba, 76 },
    { 0x46ea0cbb, 90 },
    { 0x574a0cbc, 90 },
    { 0xfe9a0cbd, 93 },
    { 0x2a0cbc, 66 },
    { 0, 0 },
    { 0xde5a0cc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6a0cc3, 61 },
    { 0x394a0cc3, 61 },
    { 0, 0 },
    { 0x97ea0cc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8a0cc9, 83 },
    { 0x513a0cca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15a0cd2, 33 },
    { 0x3cea0cd3, 78 },
    { 0xd64a0cd2, 33 },
    { 0x5b5a0cd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0a0cd9, 61 },
    { 0x778a0cd9, 43 },
    { 0, 0 },
    { 0x2efa0cdc, 90 },
    { 0, 0 },
    { 0xaafa0cde, 76 },
    { 0x8f9a0cdf, 95 },
    { 0, 0 },
    { 0x4b1a0ce1, 65 },
    { 0x46da0ce2, 33 },
    { 0, 0 },
    { 0x4b0a0ce4, 93 },
    { 0x62fa0ce5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x302a0ce8, 90 },
    { 0xb93a0ce8, 65 },
    { 0x625a0cea, 61 },
    { 0xc25a0ceb, 100 },
    { 0xee6a0cea, 23 },
    { 0xf74a0ced, 90 },
    { 0xaf5a0cea, 55 },
    { 0, 0 },
    { 0xdb5a0cf0, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3a0cf7, 98 },
    { 0xb98a0cf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ba0cfb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a0d01, 48 },
    { 0, 0 },
    { 0x649a0d03, 34 },
    { 0x7e3a0d04, 23 },
    { 0xe72a0d04, 98 },
    { 0, 0 },
    { 0x14da0d07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dea0d0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea0d0d, 23 },
    { 0x1a8a0d0d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6a0d17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624a0d1e, 90 },
    { 0xac0a0d1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617a0d32, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737a0d39, 76 },
    { 0, 0 },
    { 0x9f4a0d3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a0d3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a0d41, 80 },
    { 0xb4ea0d41, 87 },
    { 0xa1aa0d43, 77 },
    { 0x47ea0d43, 95 },
    { 0x530a0d45, 90 },
    { 0x936a0d45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a0d4c, 90 },
    { 0x17da0d4d, 95 },
    { 0xa47a0d4d, 43 },
    { 0x94ba0d4f, 43 },
    { 0x78da0d4f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aa0d53, 36 },
    { 0, 0 },
    { 0x593a0d55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fea0d58, 98 },
    { 0x915a0d58, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582a0d5d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x764a0d60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x951a0d63, 100 },
    { 0x73a0d64, 92 },
    { 0x96a0d65, 93 },
    { 0xbe7a0d66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a0d6a, 33 },
    { 0x178a0d6b, 61 },
    { 0xcdea0d6a, 43 },
    { 0, 0 },
    { 0x9a0a0d6e, 76 },
    { 0xe91a0d6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cca0d74, 90 },
    { 0x747a0d75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10a0d79, 95 },
    { 0x22fa0d7a, 33 },
    { 0x69ba0d79, 67 },
    { 0x4cda0d7c, 34 },
    { 0x1efa0d7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585a0d81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a0d84, 61 },
    { 0xcbba0d85, 23 },
    { 0x8e7a0d86, 33 },
    { 0x33ea0d86, 90 },
    { 0xc66a0d87, 23 },
    { 0x670a0d89, 76 },
    { 0, 0 },
    { 0xf4a0d8b, 61 },
    { 0x8a0a0d8c, 100 },
    { 0x474a0d8b, 61 },
    { 0x463a0d8c, 33 },
    { 0xf4fa0d8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91a0d96, 87 },
    { 0, 0 },
    { 0x3fa0d98, 77 },
    { 0xa4ca0d99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235a0d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ca0da1, 96 },
    { 0, 0 },
    { 0x801a0da3, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13a0da8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeea0dab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42a0dae, 43 },
    { 0, 0 },
    { 0x564a0db0, 95 },
    { 0xcb8a0db1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8a0db7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bea0dc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7a0dc6, 38 },
    { 0xaa0a0dc7, 33 },
    { 0x735a0dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a0dcb, 43 },
    { 0xe3fa0dcb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedea0dd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637a0dda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62da0dde, 61 },
    { 0x16aa0ddf, 80 },
    { 0x5e7a0de0, 90 },
    { 0x41fa0de1, 90 },
    { 0xab8a0dde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120a0de8, 76 },
    { 0, 0 },
    { 0xfcba0dea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1a0df0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499a0df9, 77 },
    { 0x65ba0df9, 88 },
    { 0xae7a0df9, 33 },
    { 0x26da0dfc, 34 },
    { 0xf9ba0dfc, 33 },
    { 0x32fa0dfe, 61 },
    { 0x84da0dff, 46 },
    { 0x39ea0e00, 66 },
    { 0xf3fa0e01, 61 },
    { 0x86aa0e02, 43 },
    { 0x360a0e03, 90 },
    { 0xf99a0dff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a0e09, 40 },
    { 0, 0 },
    { 0xfc2a0e0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfa0e11, 95 },
    { 0x859a0e12, 76 },
    { 0x441a0e13, 78 },
    { 0x348a0e12, 90 },
    { 0x472a0e15, 23 },
    { 0xf08a0e16, 34 },
    { 0xf7a0e12, 34 },
    { 0, 0 },
    { 0x9fca0e19, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319a0e1f, 77 },
    { 0x870a0e20, 61 },
    { 0x1c9a0e20, 55 },
    { 0x3e1a0e22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6a0e26, 34 },
    { 0xe3ea0e26, 46 },
    { 0x1efa0e28, 87 },
    { 0, 0 },
    { 0x16a0e2a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaca0e31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60a0e34, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefa0e3b, 40 },
    { 0xb8fa0e3c, 90 },
    { 0x2dda0e3d, 33 },
    { 0x216a0e3d, 23 },
    { 0x10ca0e3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x982a0e42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1aa0e4d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x227a0e50, 91 },
    { 0xb9ea0e51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26a0e57, 66 },
    { 0x45ca0e58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7a0e5b, 33 },
    { 0xaefa0e5b, 55 },
    { 0xba0e5d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57a0e62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18a0e67, 23 },
    { 0x2e5a0e68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a0e6e, 34 },
    { 0x788a0e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x526a0e72, 46 },
    { 0xee8a0e72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffa0e7d, 98 },
    { 0xb04a0e7e, 68 },
    { 0xa95a0e7f, 95 },
    { 0x74a0e80, 23 },
    { 0x451a0e7d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63a0e84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179a0e8a, 90 },
    { 0x8cea0e8b, 66 },
    { 0x23ea0e8a, 90 },
    { 0xd9aa0e8a, 98 },
    { 0, 0 },
    { 0xfb7a0e8f, 34 },
    { 0x6dda0e90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ea0e93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1a0e96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ea0e9a, 34 },
    { 0x906a0e9b, 45 },
    { 0x2a8a0e9b, 43 },
    { 0xe4a0e9d, 90 },
    { 0xa4a0e9b, 80 },
    { 0, 0 },
    { 0xe45a0ea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a0ea3, 55 },
    { 0xdeaa0ea4, 8 },
    { 0, 0 },
    { 0xfa7a0ea6, 93 },
    { 0, 0 },
    { 0xc8ba0ea8, 93 },
    { 0, 0 },
    { 0x490a0eaa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efa0eb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6a0ebc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x225a0ebf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1a0ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73a0ec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa0ec8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346a0ece, 94 },
    { 0x673a0ece, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a0ed3, 34 },
    { 0, 0 },
    { 0xff3a0ed5, 33 },
    { 0x360a0ed6, 90 },
    { 0x117a0ed6, 90 },
    { 0x101a0ed7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5a0edb, 55 },
    { 0, 0 },
    { 0x34ea0edd, 61 },
    { 0, 0 },
    { 0x3b5a0edf, 98 },
    { 0xca5a0ee0, 61 },
    { 0x3bba0ee0, 23 },
    { 0x4b6a0ee2, 55 },
    { 0x2d4a0ee3, 33 },
    { 0xf2ba0ee0, 19 },
    { 0, 0 },
    { 0x1c1a0ee6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fa0ee9, 90 },
    { 0x1fa0eea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x422a0eed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5a0ef3, 93 },
    { 0x488a0ef3, 78 },
    { 0, 0 },
    { 0xb7fa0ef6, 95 },
    { 0, 0 },
    { 0x51ea0ef8, 52 },
    { 0x45ea0ef8, 43 },
    { 0x93fa0efa, 40 },
    { 0x219a0ef8, 98 },
    { 0xda4a0efc, 67 },
    { 0xf97a0efd, 95 },
    { 0x256a0efe, 88 },
    { 0x3cfa0efc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33a0f04, 90 },
    { 0x4d9a0f05, 61 },
    { 0x9c9a0f04, 68 },
    { 0, 0 },
    { 0x87fa0f08, 93 },
    { 0xa24a0f09, 67 },
    { 0x6e0a0f09, 43 },
    { 0xf19a0f0b, 78 },
    { 0x46ba0f0b, 33 },
    { 0xf80a0f0d, 95 },
    { 0x228a0f0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2a0f14, 33 },
    { 0xa50a0f15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89da0f1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a0f1e, 91 },
    { 0x622a0f1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ea0f23, 69 },
    { 0x975a0f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaa0f28, 61 },
    { 0x321a0f28, 85 },
    { 0xae7a0f29, 33 },
    { 0, 0 },
    { 0xcc5a0f2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1a0f2f, 43 },
    { 0x45ba0f30, 64 },
    { 0x1bba0f31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385a0f35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfa0f38, 90 },
    { 0xa4ea0f39, 23 },
    { 0, 0 },
    { 0xb3aa0f3b, 23 },
    { 0x75ba0f3c, 76 },
    { 0x28fa0f3d, 93 },
    { 0xc18a0f3e, 52 },
    { 0x3afa0f3e, 33 },
    { 0, 0 },
    { 0xb87a0f41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cea0f44, 95 },
    { 0x45ea0f44, 43 },
    { 0x20aa0f46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4a0f4a, 23 },
    { 0, 0 },
    { 0xaf5a0f4c, 90 },
    { 0x9e9a0f4c, 61 },
    { 0xbda0f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ca0f53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270a0f57, 93 },
    { 0, 0 },
    { 0x13ea0f59, 23 },
    { 0, 0 },
    { 0xabda0f5b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fa0f5f, 53 },
    { 0xbfea0f5f, 66 },
    { 0x353a0f61, 23 },
    { 0xdd9a0f61, 46 },
    { 0x1b8a0f5f, 98 },
    { 0xca0a0f64, 17 },
    { 0xae1a0f65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04a0f6a, 84 },
    { 0x6e4a0f6b, 90 },
    { 0, 0 },
    { 0x4d2a0f6d, 90 },
    { 0xe44a0f6e, 23 },
    { 0xaa5a0f6d, 55 },
    { 0x6b8a0f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dba0f73, 61 },
    { 0x8bea0f73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ea0f77, 95 },
    { 0x5a3a0f77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ea0f7c, 61 },
    { 0x195a0f7d, 95 },
    { 0x331a0f7e, 95 },
    { 0x98ea0f7f, 33 },
    { 0x494a0f7d, 55 },
    { 0x55ea0f7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a0f85, 98 },
    { 0, 0 },
    { 0x1e1a0f87, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749a0f8b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x596a0f8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63a0f96, 33 },
    { 0, 0 },
    { 0xc40a0f98, 90 },
    { 0x7ea0f99, 90 },
    { 0x794a0f9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x176a0f9d, 90 },
    { 0x87ca0f9e, 76 },
    { 0xce4a0f9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a0fa9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150a0fae, 61 },
    { 0xdcba0faf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ea0fb5, 90 },
    { 0xfdda0fb6, 98 },
    { 0x59aa0fb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a0fba, 90 },
    { 0x49ca0fbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94a0fbf, 33 },
    { 0xdbea0fc0, 55 },
    { 0x3a0fc1, 98 },
    { 0x278a0fc2, 98 },
    { 0x584a0fc3, 90 },
    { 0x7aaa0fc0, 88 },
    { 0xd24a0fc5, 78 },
    { 0, 0 },
    { 0xa90a0fc7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a0fcc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bba0fcf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2da0fd5, 90 },
    { 0x86ba0fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2a0fd9, 95 },
    { 0x492a0fda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2a0fe0, 87 },
    { 0xd97a0fe1, 33 },
    { 0xd92a0fe2, 77 },
    { 0x55ba0fe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a0fe8, 90 },
    { 0xf77a0fe9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a0ff1, 95 },
    { 0x2c3a0ff1, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ea0ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862a0ff9, 15 },
    { 0, 0 },
    { 0xe9aa0ffb, 33 },
    { 0x279a0ffc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3a1001, 23 },
    { 0x229a1002, 33 },
    { 0, 0 },
    { 0x86da1004, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x355a1007, 46 },
    { 0xbd0a1007, 38 },
    { 0, 0 },
    { 0xa10a100a, 67 },
    { 0x306a100b, 91 },
    { 0xea5a100b, 43 },
    { 0xa59a100d, 91 },
    { 0, 0 },
    { 0x39ea100f, 61 },
    { 0x7f3a1010, 33 },
    { 0, 0 },
    { 0xce9a1012, 95 },
    { 0x9aea1013, 61 },
    { 0x5c7a1014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada101a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a101d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa1021, 34 },
    { 0xce3a1021, 98 },
    { 0, 0 },
    { 0x9d0a1024, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9a102c, 98 },
    { 0xfc3a102d, 90 },
    { 0x4c5a102c, 34 },
    { 0x222a102f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37a1032, 61 },
    { 0x7e5a1032, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5a1036, 85 },
    { 0xcfea1037, 98 },
    { 0x68da1038, 98 },
    { 0x3ea1038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a1040, 56 },
    { 0xf84a1040, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ea1045, 23 },
    { 0xcb0a1046, 98 },
    { 0x753a1047, 33 },
    { 0x654a1048, 98 },
    { 0x6fda1047, 33 },
    { 0xf85a1045, 95 },
    { 0xd52a104b, 34 },
    { 0xfb4a1046, 33 },
    { 0x36a1045, 100 },
    { 0x1a104e, 95 },
    { 0xd3da104e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea1053, 23 },
    { 0x82a1054, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23a1057, 95 },
    { 0xd82a1058, 56 },
    { 0, 0 },
    { 0x87da105a, 43 },
    { 0xb22a105a, 33 },
    { 0x396a105b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20a1060, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146a1065, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a1068, 52 },
    { 0, 0 },
    { 0x897a106a, 90 },
    { 0x29fa106a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849a106f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ba1072, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8a1078, 61 },
    { 0, 0 },
    { 0x820a107a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9a107d, 95 },
    { 0, 0 },
    { 0xda5a107f, 34 },
    { 0, 0 },
    { 0xc3a1081, 46 },
    { 0xdd9a1082, 90 },
    { 0, 0 },
    { 0x583a1084, 78 },
    { 0xc21a1084, 90 },
    { 0, 0 },
    { 0x17fa1087, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42a108f, 85 },
    { 0xca5a1090, 90 },
    { 0x25fa1090, 69 },
    { 0x1fba108f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77a109d, 46 },
    { 0, 0 },
    { 0xef7a109f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a10a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a10a9, 98 },
    { 0x731a10a9, 67 },
    { 0x824a10ab, 81 },
    { 0, 0 },
    { 0xf9a10ad, 33 },
    { 0xa4a10ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7a10b1, 95 },
    { 0, 0 },
    { 0xef3a10b3, 23 },
    { 0xc7ba10b4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65a10b8, 91 },
    { 0, 0 },
    { 0x297a10ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ca10bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8a10c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51aa10d1, 33 },
    { 0xf33a10d2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fa10d7, 61 },
    { 0, 0 },
    { 0xad3a10d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca10de, 34 },
    { 0x285a10df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ba10e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3a10e7, 33 },
    { 0xa93a10e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a10ec, 49 },
    { 0xe1fa10ed, 90 },
    { 0xf61a10ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x979a10f1, 61 },
    { 0x1fa10f2, 90 },
    { 0xed7a10f3, 81 },
    { 0xa38a10f4, 78 },
    { 0x62ea10f1, 98 },
    { 0x2dca10f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739a10fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8a1100, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17a1103, 43 },
    { 0x119a1104, 56 },
    { 0xc98a1105, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a1113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a1117, 46 },
    { 0x81ca1118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fa111c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca1121, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fa1124, 61 },
    { 0x77ea1124, 67 },
    { 0xbbba1126, 43 },
    { 0, 0 },
    { 0x5eaa1128, 78 },
    { 0, 0 },
    { 0x260a112a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98a112e, 55 },
    { 0xab0a112f, 88 },
    { 0, 0 },
    { 0xe05a1131, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56a1135, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a1139, 77 },
    { 0xd4a113a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38a1141, 23 },
    { 0x260a1142, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaa1148, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26a114b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba1151, 87 },
    { 0, 0 },
    { 0x979a1153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1a1158, 98 },
    { 0xa56a1158, 100 },
    { 0x233a1158, 33 },
    { 0, 0 },
    { 0xdc9a115c, 90 },
    { 0xe19a115d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60a1163, 84 },
    { 0, 0 },
    { 0x415a1165, 65 },
    { 0x13fa1166, 98 },
    { 0, 0 },
    { 0x11ca1168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a116d, 95 },
    { 0x901a116e, 90 },
    { 0xe1ba116f, 90 },
    { 0x8fa1170, 98 },
    { 0x843a1170, 34 },
    { 0x26ba1171, 64 },
    { 0xc8ba1172, 23 },
    { 0x66ea1174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bca1177, 75 },
    { 0x95ea1178, 67 },
    { 0x6f2a1177, 23 },
    { 0xff4a1178, 85 },
    { 0, 0 },
    { 0x3f4a117c, 33 },
    { 0x4f2a117d, 34 },
    { 0x3d8a117e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6a1181, 75 },
    { 0, 0 },
    { 0xf06a1183, 98 },
    { 0xd22a1184, 33 },
    { 0, 0 },
    { 0xc06a1186, 23 },
    { 0x3fea1187, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ba118d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512a1194, 61 },
    { 0, 0 },
    { 0x8aa1196, 90 },
    { 0, 0 },
    { 0x4bda1198, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18a119b, 23 },
    { 0x88ba119b, 95 },
    { 0, 0 },
    { 0x27a119e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7a11aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa11ae, 61 },
    { 0x78ba11af, 55 },
    { 0x3b1a11b0, 90 },
    { 0x39ca11b1, 61 },
    { 0, 0 },
    { 0x113a11b3, 61 },
    { 0x61da11b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa11bd, 84 },
    { 0, 0 },
    { 0x803a11bf, 23 },
    { 0x152a11bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883a11c6, 95 },
    { 0, 0 },
    { 0xbc3a11c8, 61 },
    { 0x31aa11c9, 98 },
    { 0xe0ca11c9, 90 },
    { 0x141a11c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80a11d2, 100 },
    { 0x6c4a11d3, 95 },
    { 0, 0 },
    { 0x7b1a11d5, 43 },
    { 0, 0 },
    { 0x15ba11d7, 95 },
    { 0xe96a11d7, 87 },
    { 0x780a11d9, 69 },
    { 0x4f1a11da, 81 },
    { 0, 0 },
    { 0x93a11dc, 23 },
    { 0xb02a11dd, 98 },
    { 0x5f5a11dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x481a11e1, 84 },
    { 0x125a11e2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453a11e6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a11e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aa11ec, 95 },
    { 0, 0 },
    { 0x67ba11ee, 90 },
    { 0x58fa11ee, 33 },
    { 0xd31a11ef, 95 },
    { 0xa7aa11f1, 90 },
    { 0xc49a11f2, 90 },
    { 0x452a11f3, 68 },
    { 0x5c6a11f4, 93 },
    { 0, 0 },
    { 0xe6da11f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea11fc, 98 },
    { 0x724a11fd, 78 },
    { 0x12ba11fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7a1209, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68a120c, 75 },
    { 0, 0 },
    { 0x66da120e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x539a1211, 68 },
    { 0x1d6a1212, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8da1215, 93 },
    { 0, 0 },
    { 0x7ba1217, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa121b, 98 },
    { 0x36da121c, 91 },
    { 0xc2ba121d, 55 },
    { 0, 0 },
    { 0x8a8a121f, 52 },
    { 0xc36a121f, 61 },
    { 0, 0 },
    { 0xd03a1222, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13a122b, 19 },
    { 0x9e1a122c, 77 },
    { 0x697a122d, 90 },
    { 0x547a122c, 23 },
    { 0xc87a122f, 87 },
    { 0x46a1230, 23 },
    { 0xa0ca1231, 33 },
    { 0, 0 },
    { 0xc49a1233, 90 },
    { 0, 0 },
    { 0x3eda1235, 95 },
    { 0x852a1236, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea1239, 85 },
    { 0xaf5a1239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fa123f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46a1244, 34 },
    { 0x376a1245, 90 },
    { 0xffca1246, 95 },
    { 0x68aa1246, 33 },
    { 0xbd2a1247, 17 },
    { 0, 0 },
    { 0xaf0a124a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bea124e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a1251, 55 },
    { 0xed5a1251, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a1256, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fa125d, 95 },
    { 0x3eda125e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa1262, 84 },
    { 0x67da1263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a1268, 90 },
    { 0xb17a1269, 66 },
    { 0x93a126a, 36 },
    { 0xe70a126a, 57 },
    { 0xca8a1268, 95 },
    { 0, 0 },
    { 0x4f4a126e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310a1273, 23 },
    { 0, 0 },
    { 0x9faa1275, 95 },
    { 0, 0 },
    { 0x6c9a1277, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96a127b, 33 },
    { 0x9c1a127b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fea127f, 76 },
    { 0xd4ca1280, 55 },
    { 0x78da127f, 98 },
    { 0x6b6a1282, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480a128b, 34 },
    { 0xdd8a128c, 80 },
    { 0, 0 },
    { 0x350a128e, 84 },
    { 0x694a128f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3a1296, 61 },
    { 0, 0 },
    { 0x6c0a1298, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7a129d, 98 },
    { 0x9a1a129e, 76 },
    { 0x12fa129f, 98 },
    { 0xcf6a129e, 90 },
    { 0xd5ca12a1, 87 },
    { 0, 0 },
    { 0x2b5a12a3, 33 },
    { 0, 0 },
    { 0x82ea12a5, 43 },
    { 0xe2a12a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x697a12a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56a12ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea12bb, 61 },
    { 0xc5ca12bc, 17 },
    { 0xe96a12bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ca12c0, 23 },
    { 0x77ba12c1, 61 },
    { 0x35ba12c2, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9a12c7, 33 },
    { 0xcbfa12c8, 34 },
    { 0xbd1a12c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a12cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a12d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aa12d7, 78 },
    { 0, 0 },
    { 0x65da12d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a12e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79a12ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4a12f3, 33 },
    { 0, 0 },
    { 0x84ca12f5, 66 },
    { 0x269a12f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaba12f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a1301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeda1306, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aca1317, 90 },
    { 0xd35a1318, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0a131e, 78 },
    { 0x27aa131f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3a1322, 33 },
    { 0x989a1323, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fa1327, 17 },
    { 0, 0 },
    { 0xd73a1329, 55 },
    { 0xb73a1329, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545a1331, 33 },
    { 0, 0 },
    { 0xff7a1333, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2a1336, 23 },
    { 0xda6a1336, 36 },
    { 0x5cfa1336, 61 },
    { 0xcfba1338, 61 },
    { 0, 0 },
    { 0x41ba133b, 90 },
    { 0xfb4a133b, 61 },
    { 0, 0 },
    { 0x74ea133e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda1346, 81 },
    { 0xd2ca1347, 34 },
    { 0xb7aa1346, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ba1352, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a135c, 43 },
    { 0, 0 },
    { 0x526a135e, 34 },
    { 0, 0 },
    { 0x523a1360, 95 },
    { 0x1bea1360, 30 },
    { 0, 0 },
    { 0xab7a1363, 95 },
    { 0xed2a1363, 99 },
    { 0x890a1365, 55 },
    { 0x8c5a1365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34a136c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7a1371, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ba1375, 91 },
    { 0, 0 },
    { 0x473a1377, 66 },
    { 0xc7ea1378, 52 },
    { 0xc8ba1378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea137f, 61 },
    { 0xb96a1380, 68 },
    { 0xf5ca1381, 90 },
    { 0x638a1380, 23 },
    { 0xeeda1383, 97 },
    { 0xfeaa1384, 98 },
    { 0xd8ba1384, 55 },
    { 0x1a7a1386, 66 },
    { 0xd1aa1387, 55 },
    { 0x6b8a1386, 100 },
    { 0x4e0a1389, 98 },
    { 0x8eba1386, 34 },
    { 0xfdba1389, 84 },
    { 0, 0 },
    { 0x732a138d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x873a1390, 61 },
    { 0x71ba1390, 75 },
    { 0xcb1a1391, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2a1395, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89a1399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623a139d, 78 },
    { 0x397a139d, 46 },
    { 0xee0a139d, 90 },
    { 0x283a13a0, 80 },
    { 0xae4a13a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da13a4, 75 },
    { 0x76ea13a4, 81 },
    { 0, 0 },
    { 0x55aa13a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94a13aa, 46 },
    { 0x40fa13aa, 90 },
    { 0xbdea13ac, 61 },
    { 0x30da13ac, 76 },
    { 0xb71a13ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12a13b1, 33 },
    { 0x2b1a13b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515a13bb, 34 },
    { 0, 0 },
    { 0xdb1a13bd, 61 },
    { 0xdd0a13bd, 90 },
    { 0xe0ea13bf, 98 },
    { 0, 0 },
    { 0x75ca13c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6a13c4, 33 },
    { 0x5bfa13c4, 93 },
    { 0, 0 },
    { 0xc7fa13c7, 93 },
    { 0x667a13c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7a13da, 98 },
    { 0xf99a13db, 67 },
    { 0x95ba13dc, 76 },
    { 0xccda13dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6a13e1, 43 },
    { 0xe40a13e2, 33 },
    { 0, 0 },
    { 0xd5aa13e4, 95 },
    { 0xd29a13e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13a13ea, 52 },
    { 0xb8fa13eb, 66 },
    { 0, 0 },
    { 0xd22a13ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ba13f5, 98 },
    { 0x2e7a13f6, 33 },
    { 0x830a13f6, 78 },
    { 0x482a13f8, 23 },
    { 0, 0 },
    { 0x1dba13fa, 98 },
    { 0, 0 },
    { 0x2b7a13fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefea1404, 95 },
    { 0, 0 },
    { 0x859a1406, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577a140b, 23 },
    { 0x81ea140c, 98 },
    { 0xd96a140b, 52 },
    { 0, 0 },
    { 0xe58a140f, 46 },
    { 0x71ea140f, 33 },
    { 0x81ba1411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afa1419, 23 },
    { 0, 0 },
    { 0x5c6a141b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a141e, 43 },
    { 0, 0 },
    { 0xeca1420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5a1423, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ca1427, 61 },
    { 0xa8aa1428, 99 },
    { 0, 0 },
    { 0x311a142a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0a1430, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8a1433, 78 },
    { 0, 0 },
    { 0x5b9a1435, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98a1438, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da143c, 34 },
    { 0xd93a143c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5a1441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324a144c, 87 },
    { 0x7e2a144d, 76 },
    { 0x2fa144e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3a1454, 38 },
    { 0, 0 },
    { 0x178a1456, 98 },
    { 0, 0 },
    { 0xd1a1458, 33 },
    { 0x3e8a1459, 55 },
    { 0x54da145a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba145d, 55 },
    { 0xd79a145e, 33 },
    { 0x107a145d, 67 },
    { 0x5dea1460, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30a1465, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfca146a, 46 },
    { 0, 0 },
    { 0xcfea146c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7a146f, 98 },
    { 0xb65a1470, 81 },
    { 0, 0 },
    { 0x22aa1472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ba1475, 52 },
    { 0xd0ea1475, 100 },
    { 0x11ca1476, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61a147e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa1481, 33 },
    { 0xfc4a1482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13a1486, 52 },
    { 0x198a1487, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x501a148a, 90 },
    { 0, 0 },
    { 0x3d9a148c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792a1496, 23 },
    { 0xba1497, 93 },
    { 0, 0 },
    { 0x24da1499, 52 },
    { 0xecaa149a, 98 },
    { 0xdfba149b, 55 },
    { 0x163a149c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257a14a0, 23 },
    { 0x482a14a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9a14a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x342a14ab, 19 },
    { 0, 0 },
    { 0x9cea14ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2a14b1, 33 },
    { 0x69ba14b2, 55 },
    { 0xeb2a14b3, 90 },
    { 0xd3da14b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0aa14b7, 33 },
    { 0xbc7a14b7, 76 },
    { 0x872a14b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fa14c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fa14c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177a14d4, 23 },
    { 0x96ea14d4, 100 },
    { 0x2e7a14d6, 23 },
    { 0x405a14d6, 23 },
    { 0xf63a14d8, 75 },
    { 0x6c2a14d9, 34 },
    { 0x475a14d8, 67 },
    { 0x6dca14da, 90 },
    { 0x50da14dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aa14e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44da14e3, 52 },
    { 0x266a14e4, 61 },
    { 0xadca14e3, 43 },
    { 0, 0 },
    { 0x91ba14e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaa14eb, 95 },
    { 0xdb4a14ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62a14f0, 43 },
    { 0x218a14f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x611a14f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faa14f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51a14ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319a1503, 91 },
    { 0, 0 },
    { 0xf5aa1505, 78 },
    { 0x3aba1505, 90 },
    { 0, 0 },
    { 0xd75a1508, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bba150e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23a1511, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aca1514, 95 },
    { 0x1c4a1515, 23 },
    { 0xb1ca1516, 99 },
    { 0x2c2a1517, 55 },
    { 0x43ca1518, 98 },
    { 0, 0 },
    { 0x44ca151a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2a151e, 95 },
    { 0xcaca151f, 99 },
    { 0xbaca1520, 87 },
    { 0x9aaa1521, 91 },
    { 0x9f3a1521, 33 },
    { 0x458a1520, 34 },
    { 0, 0 },
    { 0x13ba1525, 95 },
    { 0x3e3a1526, 55 },
    { 0, 0 },
    { 0x874a1528, 98 },
    { 0x523a1529, 23 },
    { 0x407a1529, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfa1538, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ca153b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9a153e, 55 },
    { 0xfb0a153e, 90 },
    { 0x32ba1540, 61 },
    { 0x64ba1540, 98 },
    { 0x824a1540, 84 },
    { 0xd5ba1543, 100 },
    { 0x884a1544, 98 },
    { 0xd67a1544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3a1548, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ea154b, 90 },
    { 0x5a4a154c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ea154f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422a1553, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8a1558, 55 },
    { 0xff8a1559, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aa155d, 99 },
    { 0xd62a155e, 61 },
    { 0, 0 },
    { 0x819a1560, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859a1564, 61 },
    { 0xf29a1565, 90 },
    { 0xa85a1566, 33 },
    { 0xb2ea1564, 92 },
    { 0xa75a1568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fca156b, 87 },
    { 0xbd4a156c, 17 },
    { 0xfa2a156c, 33 },
    { 0xd00a156b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34a1572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x578a1575, 33 },
    { 0x66fa1576, 95 },
    { 0, 0 },
    { 0x1efa1578, 90 },
    { 0xc73a1579, 90 },
    { 0x7b7a157a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ca157e, 66 },
    { 0x220a157f, 33 },
    { 0x972a157f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fa1584, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ea1588, 90 },
    { 0xf48a1589, 61 },
    { 0x57a158a, 90 },
    { 0xeaa1588, 91 },
    { 0, 0 },
    { 0x2baa158d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45a1592, 52 },
    { 0x312a1592, 33 },
    { 0x5bba1594, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4a1598, 61 },
    { 0xd2da1599, 66 },
    { 0, 0 },
    { 0xa82a159b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269a15a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca15b3, 61 },
    { 0x195a15b4, 23 },
    { 0x854a15b5, 61 },
    { 0xd41a15b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x805a15b9, 61 },
    { 0xaf9a15ba, 66 },
    { 0x2e3a15bb, 61 },
    { 0xd74a15bb, 61 },
    { 0x434a15bd, 99 },
    { 0xfb4a15bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x594a15c1, 76 },
    { 0x64fa15c2, 43 },
    { 0, 0 },
    { 0xe8ea15c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18a15cc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a15d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158a15db, 68 },
    { 0x2e5a15dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6da15e0, 23 },
    { 0xdfba15e1, 33 },
    { 0x8d8a15e2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9a15e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3a15e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eca15ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2a15ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33a15f5, 67 },
    { 0xe8ba15f5, 55 },
    { 0x384a15f7, 66 },
    { 0xcb7a15f8, 55 },
    { 0, 0 },
    { 0x81aa15fa, 90 },
    { 0, 0 },
    { 0x4c3a15fc, 87 },
    { 0x700a15fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49a1600, 61 },
    { 0xb37a1601, 55 },
    { 0, 0 },
    { 0xceca1603, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfa1606, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a160e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa1612, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a1616, 96 },
    { 0xd1ba1617, 61 },
    { 0, 0 },
    { 0x1bca1619, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47a161e, 33 },
    { 0x51da161e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x132a1622, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26a1625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282a162e, 95 },
    { 0, 0 },
    { 0x400a1630, 61 },
    { 0x4f2a1630, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddea1636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a163b, 23 },
    { 0x5cea163c, 33 },
    { 0, 0 },
    { 0xbcaa163e, 61 },
    { 0xcaea163f, 23 },
    { 0x400a1640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefa164c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1a164f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2a165a, 67 },
    { 0, 0 },
    { 0xd1da165c, 98 },
    { 0x3d6a165c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x256a1660, 61 },
    { 0, 0 },
    { 0x5fea1662, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a1665, 93 },
    { 0, 0 },
    { 0x1c4a1667, 68 },
    { 0xe5fa1667, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba166b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca166f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542a1673, 43 },
    { 0xb04a1674, 30 },
    { 0xeb5a1673, 95 },
    { 0, 0 },
    { 0x81ba1677, 44 },
    { 0x318a1677, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x681a167b, 23 },
    { 0, 0 },
    { 0xd15a167d, 23 },
    { 0x85ca167e, 23 },
    { 0xd25a167f, 46 },
    { 0x9ba1680, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989a1685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa168a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea168d, 61 },
    { 0xc30a168d, 93 },
    { 0xb5ea168d, 33 },
    { 0, 0 },
    { 0xe4ba1691, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a1694, 61 },
    { 0xec1a1695, 55 },
    { 0xaba1695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58a169a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145a16a0, 76 },
    { 0xfc3a16a1, 83 },
    { 0xfe7a16a2, 98 },
    { 0, 0 },
    { 0xf4da16a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafa16a7, 90 },
    { 0xde7a16a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaea16ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578a16b7, 90 },
    { 0xf83a16b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea16bc, 76 },
    { 0x2d3a16bd, 95 },
    { 0x7e2a16be, 76 },
    { 0x531a16be, 98 },
    { 0xaaa16be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1a16c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1a16cc, 77 },
    { 0xa0fa16cd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1a16d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a16db, 61 },
    { 0, 0 },
    { 0xf38a16dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6a16e5, 98 },
    { 0xb2ba16e6, 61 },
    { 0xddba16e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8a16eb, 76 },
    { 0, 0 },
    { 0x262a16ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9a16f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a16fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92a16ff, 23 },
    { 0xbfca1700, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a1704, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910a170a, 90 },
    { 0x73ba170b, 90 },
    { 0x134a170c, 98 },
    { 0x49ca170d, 43 },
    { 0x9f5a170b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ba1714, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849a1719, 95 },
    { 0xaeca171a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea171d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54a1724, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6a1729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ea172f, 98 },
    { 0xe23a1730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ca1734, 61 },
    { 0x431a1734, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a173c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5a1740, 88 },
    { 0xbfa1741, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30aa1745, 55 },
    { 0xadda1745, 68 },
    { 0xaafa1747, 34 },
    { 0x17ea1748, 92 },
    { 0, 0 },
    { 0xe73a174a, 98 },
    { 0x145a174b, 23 },
    { 0x98ba174b, 80 },
    { 0xa8fa174d, 56 },
    { 0x209a174c, 90 },
    { 0x664a174f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa1752, 99 },
    { 0xeaa1753, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926a1757, 77 },
    { 0x6d9a1758, 61 },
    { 0, 0 },
    { 0xf9ea175a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3a175f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x439a1762, 90 },
    { 0, 0 },
    { 0xd6a1764, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186a176d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea1770, 61 },
    { 0, 0 },
    { 0x432a1772, 88 },
    { 0x4c3a1773, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a177b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83a1787, 90 },
    { 0x8f9a1788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524a178d, 90 },
    { 0, 0 },
    { 0x82ca178f, 34 },
    { 0x61fa1790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9a1793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ba1796, 76 },
    { 0, 0 },
    { 0xc82a1798, 93 },
    { 0xf66a1798, 43 },
    { 0xd42a179a, 77 },
    { 0, 0 },
    { 0xc93a179c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdda17a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ea17a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40a17aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a17ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a17b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ca17b6, 81 },
    { 0x97a17b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8a17bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5a17c0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a17c5, 33 },
    { 0x33da17c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a17cf, 90 },
    { 0, 0 },
    { 0xfd6a17d1, 99 },
    { 0, 0 },
    { 0x4b3a17d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ba17d6, 95 },
    { 0x43a17d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9a17dc, 55 },
    { 0xf5fa17dd, 61 },
    { 0x9b1a17de, 23 },
    { 0xe32a17df, 91 },
    { 0x5bea17e0, 98 },
    { 0x11ca17e1, 90 },
    { 0xfe1a17e2, 90 },
    { 0x67da17e1, 90 },
    { 0xda5a17df, 77 },
    { 0, 0 },
    { 0x9f5a17e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x395a17e9, 61 },
    { 0x27ea17ea, 66 },
    { 0x1f3a17ea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9a17ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ba17f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66a17f5, 81 },
    { 0xe6ca17f6, 75 },
    { 0x72a17f7, 98 },
    { 0x9dda17f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1a17fb, 95 },
    { 0, 0 },
    { 0xd1a17fd, 55 },
    { 0xd1aa17fd, 55 },
    { 0x306a17fe, 90 },
    { 0xe12a17fe, 93 },
    { 0x395a1801, 33 },
    { 0x777a1802, 33 },
    { 0xa5fa1801, 84 },
    { 0x773a1803, 98 },
    { 0x2bca1805, 78 },
    { 0x22da1802, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a1809, 90 },
    { 0, 0 },
    { 0xf48a180b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aba180f, 34 },
    { 0xfa2a1810, 65 },
    { 0x946a1810, 91 },
    { 0xf13a1811, 34 },
    { 0x1d2a1813, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa1816, 43 },
    { 0xffba1817, 61 },
    { 0, 0 },
    { 0xa9ea1819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aa181f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcda1829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ca1831, 61 },
    { 0, 0 },
    { 0x9e0a1833, 87 },
    { 0x61fa1834, 36 },
    { 0x286a1833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2a1839, 90 },
    { 0, 0 },
    { 0x935a183b, 90 },
    { 0xd9ba183c, 43 },
    { 0x901a183b, 68 },
    { 0x9afa183c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240a1844, 90 },
    { 0xb91a1844, 95 },
    { 0x910a1846, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91a1849, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbca184d, 43 },
    { 0x65ca184e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596a1852, 95 },
    { 0x9b1a1853, 33 },
    { 0xdb0a1854, 23 },
    { 0xba4a1855, 61 },
    { 0x641a1853, 33 },
    { 0xbaaa1857, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6a185b, 85 },
    { 0xd8ea185c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7a185f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24aa1866, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaa1872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984a1876, 46 },
    { 0, 0 },
    { 0x302a1878, 93 },
    { 0xe47a1878, 38 },
    { 0x975a187a, 98 },
    { 0xeeba187b, 95 },
    { 0, 0 },
    { 0x73da187d, 61 },
    { 0, 0 },
    { 0xeb6a187f, 33 },
    { 0x699a187f, 34 },
    { 0xe16a1881, 52 },
    { 0x63da187f, 17 },
    { 0x671a1880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aa1886, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca188a, 76 },
    { 0x825a188b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0a1893, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bca189b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa189e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664a18a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aa18ad, 34 },
    { 0, 0 },
    { 0xabfa18af, 95 },
    { 0xd09a18b0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714a18b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955a18c0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeba18c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967a18cd, 95 },
    { 0, 0 },
    { 0x44ba18cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7a18d5, 52 },
    { 0, 0 },
    { 0x505a18d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ea18dc, 90 },
    { 0xe4da18dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3a18e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704a18e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71da18e9, 98 },
    { 0xe2a18e9, 61 },
    { 0xf8ea18eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11a18f1, 90 },
    { 0x2f9a18f2, 22 },
    { 0, 0 },
    { 0xd27a18f4, 90 },
    { 0x996a18f5, 43 },
    { 0x8b7a18f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x743a18f9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfa18fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a1906, 66 },
    { 0, 0 },
    { 0xfa3a1908, 23 },
    { 0x8c2a1908, 76 },
    { 0xe63a190a, 23 },
    { 0x366a190b, 61 },
    { 0xef5a190c, 90 },
    { 0x926a190d, 90 },
    { 0xf00a190e, 43 },
    { 0xd5ea190e, 90 },
    { 0xd12a1910, 61 },
    { 0x1f5a190f, 99 },
    { 0x8d3a190d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70a1916, 91 },
    { 0x951a1916, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x985a191a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9a1920, 70 },
    { 0x21ea1921, 61 },
    { 0, 0 },
    { 0x540a1923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a1928, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a192b, 75 },
    { 0, 0 },
    { 0x70fa192d, 61 },
    { 0x7a0a192e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0a1931, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ea1935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaa193d, 33 },
    { 0xf98a193e, 33 },
    { 0, 0 },
    { 0xd85a1940, 90 },
    { 0xd95a1940, 34 },
    { 0, 0 },
    { 0x2fa1943, 95 },
    { 0x391a1943, 33 },
    { 0xcc2a1943, 33 },
    { 0x824a1946, 98 },
    { 0x460a1946, 93 },
    { 0x8e6a1944, 99 },
    { 0x3b7a1949, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x27da194c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ada1952, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba1957, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa195b, 76 },
    { 0xa8ea195c, 85 },
    { 0x5d8a195b, 58 },
    { 0x470a195e, 78 },
    { 0xfe5a195f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a1962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1a1969, 97 },
    { 0, 0 },
    { 0x6dfa196b, 98 },
    { 0x3bea196c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fa1970, 98 },
    { 0x4bfa1971, 55 },
    { 0, 0 },
    { 0x7a7a1973, 23 },
    { 0, 0 },
    { 0x794a1975, 76 },
    { 0x499a1976, 61 },
    { 0x484a1977, 76 },
    { 0xf3aa1977, 98 },
    { 0, 0 },
    { 0xb95a197a, 33 },
    { 0x9f6a197b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164a197f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fa1982, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9a1988, 90 },
    { 0x352a1989, 85 },
    { 0x377a198a, 34 },
    { 0, 0 },
    { 0x6cea198c, 76 },
    { 0x911a198c, 33 },
    { 0x3d6a198d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37aa1996, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836a199b, 23 },
    { 0x6cba199b, 98 },
    { 0, 0 },
    { 0xf7ca199e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7a19a2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aba19a7, 33 },
    { 0xaf1a19a8, 75 },
    { 0, 0 },
    { 0x9cea19aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2a19ad, 55 },
    { 0, 0 },
    { 0x331a19af, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aa19b3, 33 },
    { 0, 0 },
    { 0x3c8a19b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda19b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a19c0, 61 },
    { 0, 0 },
    { 0xefca19c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527a19c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a19cd, 95 },
    { 0x899a19ce, 40 },
    { 0x982a19ce, 76 },
    { 0x810a19d0, 23 },
    { 0x30ba19cd, 95 },
    { 0x412a19d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x653a19d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda19dd, 85 },
    { 0x5a2a19de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73da19e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x258a19e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dea19e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a19ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6da19ef, 61 },
    { 0xddba19f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a19f4, 66 },
    { 0x3d2a19f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326a19f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea19fc, 90 },
    { 0, 0 },
    { 0x742a19fe, 61 },
    { 0xc89a19ff, 76 },
    { 0, 0 },
    { 0xdaca1a01, 95 },
    { 0x9f2a1a02, 95 },
    { 0x352a1a01, 55 },
    { 0xb58a1a04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9a1a07, 55 },
    { 0x66ea1a08, 46 },
    { 0, 0 },
    { 0xfdaa1a0a, 76 },
    { 0, 0 },
    { 0xbf4a1a0c, 98 },
    { 0xd66a1a0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da1a22, 55 },
    { 0x547a1a23, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ca1a26, 61 },
    { 0x6da1a26, 95 },
    { 0x82a1a27, 61 },
    { 0, 0 },
    { 0xf54a1a2a, 55 },
    { 0xe7ca1a2b, 98 },
    { 0xf3a1a2a, 61 },
    { 0x73ba1a2d, 33 },
    { 0x855a1a2e, 61 },
    { 0, 0 },
    { 0x6f7a1a30, 98 },
    { 0xe83a1a31, 95 },
    { 0x355a1a32, 88 },
    { 0x175a1a32, 77 },
    { 0, 0 },
    { 0xd81a1a35, 99 },
    { 0x3caa1a36, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21a1a3a, 96 },
    { 0xecfa1a3b, 98 },
    { 0x756a1a3c, 68 },
    { 0x9fba1a3d, 61 },
    { 0x492a1a3c, 33 },
    { 0, 0 },
    { 0x706a1a40, 99 },
    { 0x3f8a1a40, 23 },
    { 0xcb0a1a40, 90 },
    { 0xd6fa1a43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x103a1a46, 95 },
    { 0, 0 },
    { 0x274a1a48, 78 },
    { 0x95ca1a49, 93 },
    { 0x490a1a4a, 90 },
    { 0x7aea1a4a, 46 },
    { 0xa86a1a4c, 34 },
    { 0xecfa1a4d, 23 },
    { 0x187a1a4e, 66 },
    { 0xa74a1a4c, 55 },
    { 0, 0 },
    { 0x952a1a51, 43 },
    { 0x84ea1a52, 83 },
    { 0xdada1a52, 70 },
    { 0, 0 },
    { 0x82aa1a55, 98 },
    { 0xf1ca1a56, 33 },
    { 0xad2a1a56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2a1a62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811a1a67, 28 },
    { 0x32a1a68, 40 },
    { 0, 0 },
    { 0xb67a1a6a, 85 },
    { 0x2a5a1a6a, 90 },
    { 0x68fa1a6c, 34 },
    { 0, 0 },
    { 0x30ea1a6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21a1a71, 88 },
    { 0x3d6a1a72, 52 },
    { 0x146a1a73, 88 },
    { 0xb20a1a74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94a1a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4a1a80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x211a1a83, 91 },
    { 0xc68a1a83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ea1a92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a1a97, 90 },
    { 0x24fa1a98, 66 },
    { 0xd01a1a98, 90 },
    { 0x4a3a1a97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x348a1a9d, 33 },
    { 0x829a1a9e, 80 },
    { 0, 0 },
    { 0x34ea1aa0, 61 },
    { 0, 0 },
    { 0x208a1aa2, 98 },
    { 0, 0 },
    { 0xc7ea1aa4, 99 },
    { 0x7c0a1aa5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a1ab9, 17 },
    { 0, 0 },
    { 0x221a1abb, 99 },
    { 0xb9aa1abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a1ac0, 98 },
    { 0xce2a1ac1, 98 },
    { 0xc32a1ac1, 46 },
    { 0, 0 },
    { 0xe00a1ac4, 76 },
    { 0x625a1ac4, 90 },
    { 0xed8a1ac6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbea1acc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa1ad6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x879a1ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1adc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ca1adf, 90 },
    { 0xdada1ae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279a1ae5, 33 },
    { 0xf03a1ae6, 55 },
    { 0xcd8a1ae5, 33 },
    { 0x31ba1ae5, 90 },
    { 0x71aa1ae9, 17 },
    { 0x135a1aea, 98 },
    { 0xdb3a1aea, 80 },
    { 0x156a1aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbba1af0, 99 },
    { 0x60ea1af0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eca1af6, 33 },
    { 0, 0 },
    { 0x1f3a1af8, 23 },
    { 0x6dda1af9, 95 },
    { 0, 0 },
    { 0x273a1afb, 76 },
    { 0x6e6a1afb, 33 },
    { 0xea4a1afd, 84 },
    { 0xbd4a1afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533a1b04, 84 },
    { 0x6f2a1b05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a1b08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a1b0b, 90 },
    { 0xccaa1b0c, 100 },
    { 0, 0 },
    { 0xf95a1b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19a1b13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33a1b16, 61 },
    { 0xcc2a1b17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03a1b1c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cca1b24, 93 },
    { 0, 0 },
    { 0x5d5a1b26, 95 },
    { 0xa66a1b27, 52 },
    { 0x128a1b28, 23 },
    { 0, 0 },
    { 0xea5a1b2a, 34 },
    { 0x924a1b2b, 34 },
    { 0x6ea1b2c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770a1b30, 19 },
    { 0x50da1b30, 34 },
    { 0, 0 },
    { 0x68aa1b33, 95 },
    { 0, 0 },
    { 0x4aa1b35, 98 },
    { 0, 0 },
    { 0xd3aa1b37, 34 },
    { 0, 0 },
    { 0x78fa1b39, 85 },
    { 0, 0 },
    { 0xf70a1b3b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ea1b3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30a1b41, 90 },
    { 0x109a1b42, 67 },
    { 0xd6a1b42, 52 },
    { 0xcd7a1b42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50a1b47, 98 },
    { 0x343a1b48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ea1b4c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddca1b50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda1b58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a1b5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a1b66, 23 },
    { 0x375a1b67, 55 },
    { 0x403a1b67, 58 },
    { 0, 0 },
    { 0x4d1a1b6a, 61 },
    { 0, 0 },
    { 0x1da1b6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a1b72, 67 },
    { 0x2a5a1b73, 36 },
    { 0x81ca1b72, 95 },
    { 0x570a1b73, 82 },
    { 0x261a1b76, 98 },
    { 0x47fa1b77, 100 },
    { 0x2e7a1b78, 99 },
    { 0x294a1b78, 90 },
    { 0, 0 },
    { 0x80fa1b7b, 61 },
    { 0, 0 },
    { 0x9dda1b7d, 23 },
    { 0xec9a1b7e, 46 },
    { 0x451a1b7e, 61 },
    { 0xe79a1b7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dba1b87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72a1b8c, 90 },
    { 0x6eda1b8d, 98 },
    { 0xe22a1b8c, 33 },
    { 0x8e9a1b8e, 90 },
    { 0xdd6a1b90, 46 },
    { 0x9eaa1b91, 95 },
    { 0x186a1b90, 33 },
    { 0x1b9a1b93, 88 },
    { 0x5c3a1b90, 98 },
    { 0x7c8a1b95, 33 },
    { 0x9aea1b95, 90 },
    { 0xc47a1b8e, 98 },
    { 0xb10a1b8d, 66 },
    { 0, 0 },
    { 0x37a1b9a, 91 },
    { 0xbf1a1b9b, 51 },
    { 0x145a1b9b, 17 },
    { 0x4daa1b9d, 95 },
    { 0, 0 },
    { 0x8a9a1b9f, 8 },
    { 0xe2fa1ba0, 61 },
    { 0xed4a1ba1, 61 },
    { 0, 0 },
    { 0x620a1ba3, 99 },
    { 0xec7a1ba3, 52 },
    { 0x7eba1ba5, 90 },
    { 0xac2a1ba4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ca1ba9, 33 },
    { 0x23a1baa, 90 },
    { 0, 0 },
    { 0xbfba1bac, 61 },
    { 0x46ca1bad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cba1bb0, 60 },
    { 0, 0 },
    { 0xc3fa1bb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a1bb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca1bbc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a1bbf, 33 },
    { 0, 0 },
    { 0x31ba1bc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa1bcd, 34 },
    { 0xcb1a1bce, 43 },
    { 0xb72a1bcf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151a1bd3, 76 },
    { 0, 0 },
    { 0xe0ba1bd5, 80 },
    { 0, 0 },
    { 0x10a1bd7, 19 },
    { 0xb7a1bd8, 23 },
    { 0x641a1bd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6a1bdc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88da1be1, 90 },
    { 0x242a1be2, 66 },
    { 0xd73a1be3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877a1bea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca1bed, 33 },
    { 0, 0 },
    { 0xdfaa1bef, 52 },
    { 0, 0 },
    { 0x79ea1bf1, 46 },
    { 0, 0 },
    { 0xb24a1bf3, 22 },
    { 0xb09a1bf3, 61 },
    { 0x332a1bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4a1bfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137a1bfe, 46 },
    { 0x52fa1bff, 66 },
    { 0x3c4a1c00, 95 },
    { 0, 0 },
    { 0xa51a1c02, 46 },
    { 0x8a0a1c03, 33 },
    { 0, 0 },
    { 0xf1ea1c05, 17 },
    { 0x658a1c05, 90 },
    { 0x1e8a1c06, 98 },
    { 0xc3fa1c08, 34 },
    { 0xde5a1c08, 90 },
    { 0x228a1c09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x143a1c0d, 75 },
    { 0, 0 },
    { 0x1a6a1c0f, 100 },
    { 0xb82a1c10, 91 },
    { 0xc69a1c10, 78 },
    { 0x6aaa1c12, 23 },
    { 0xfa3a1c0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91da1c16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfa1c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36a1c21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3a1c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874a1c2a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dca1c2e, 23 },
    { 0, 0 },
    { 0xa2fa1c30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cba1c3a, 90 },
    { 0x33fa1c3b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ea1c3e, 33 },
    { 0x39ba1c3e, 61 },
    { 0xe8da1c40, 90 },
    { 0xf95a1c41, 61 },
    { 0x6f7a1c42, 33 },
    { 0x15ca1c42, 66 },
    { 0x798a1c43, 33 },
    { 0xfa0a1c41, 98 },
    { 0xc3da1c46, 98 },
    { 0x754a1c42, 61 },
    { 0x393a1c48, 55 },
    { 0x82a1c49, 98 },
    { 0, 0 },
    { 0x810a1c4b, 90 },
    { 0x76fa1c4c, 34 },
    { 0, 0 },
    { 0xad0a1c4e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fa1c52, 61 },
    { 0x430a1c52, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x213a1c56, 90 },
    { 0x7e2a1c57, 34 },
    { 0xb5a1c57, 87 },
    { 0x643a1c59, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afa1c5f, 76 },
    { 0xa2ba1c5f, 43 },
    { 0x509a1c5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8a1c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x871a1c67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1a1c6c, 97 },
    { 0x91ea1c6d, 23 },
    { 0xe21a1c6e, 87 },
    { 0x4d9a1c6e, 61 },
    { 0x792a1c6c, 90 },
    { 0xf5fa1c71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aa1c75, 43 },
    { 0x304a1c76, 77 },
    { 0x3d9a1c76, 90 },
    { 0xea1a1c77, 17 },
    { 0x8a1c79, 99 },
    { 0xc65a1c7a, 85 },
    { 0xa7fa1c75, 33 },
    { 0xc10a1c7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74a1c80, 98 },
    { 0xb04a1c81, 46 },
    { 0xfe5a1c81, 43 },
    { 0x689a1c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba1c86, 90 },
    { 0xf6ea1c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80a1c8b, 33 },
    { 0, 0 },
    { 0xfb5a1c8d, 33 },
    { 0x2fca1c8e, 85 },
    { 0x9d6a1c8f, 91 },
    { 0x758a1c8d, 95 },
    { 0, 0 },
    { 0x45da1c92, 99 },
    { 0x8aa1c93, 78 },
    { 0, 0 },
    { 0xfeda1c95, 52 },
    { 0xfd9a1c95, 90 },
    { 0x41ca1c97, 33 },
    { 0, 0 },
    { 0x61ea1c99, 33 },
    { 0xa15a1c9a, 34 },
    { 0, 0 },
    { 0x364a1c9c, 90 },
    { 0, 0 },
    { 0x500a1c9e, 91 },
    { 0x6aea1c9f, 46 },
    { 0x304a1c9f, 93 },
    { 0xe5ba1ca1, 76 },
    { 0xfc0a1c9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890a1ca9, 55 },
    { 0x7efa1ca9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0a1cae, 23 },
    { 0x402a1caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ca1cb2, 23 },
    { 0xe5ea1cb3, 91 },
    { 0x506a1cb4, 23 },
    { 0xc5ca1cb4, 61 },
    { 0x341a1cb4, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca1cbc, 76 },
    { 0, 0 },
    { 0x888a1cbe, 61 },
    { 0, 0 },
    { 0x32ea1cc0, 61 },
    { 0x2aba1cc1, 61 },
    { 0xd39a1cc2, 23 },
    { 0x4e6a1cc3, 61 },
    { 0x1dba1cc0, 76 },
    { 0x6b8a1cc5, 87 },
    { 0x36aa1cc5, 75 },
    { 0xff9a1cc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7a1cca, 33 },
    { 0x210a1ccb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742a1cd4, 90 },
    { 0x6b5a1cd5, 90 },
    { 0, 0 },
    { 0x977a1cd7, 33 },
    { 0xb0a1cd8, 33 },
    { 0, 0 },
    { 0x86a1cda, 23 },
    { 0, 0 },
    { 0x7bba1cdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a1ce0, 87 },
    { 0, 0 },
    { 0x29ca1ce2, 61 },
    { 0x7b4a1ce3, 95 },
    { 0x4b2a1ce4, 81 },
    { 0xfe1a1ce2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x204a1ce8, 23 },
    { 0, 0 },
    { 0x357a1cea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7a1cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2a1cf4, 43 },
    { 0, 0 },
    { 0x67ca1cf6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ea1cfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821a1cff, 100 },
    { 0x6f1a1d00, 34 },
    { 0xf46a1d01, 93 },
    { 0x3cfa1d01, 90 },
    { 0x25ba1d03, 90 },
    { 0x36da1d03, 93 },
    { 0, 0 },
    { 0x5e9a1d06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2a1d0b, 46 },
    { 0x999a1d0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a1d0f, 46 },
    { 0, 0 },
    { 0xdd6a1d11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06a1d15, 23 },
    { 0xfd5a1d15, 61 },
    { 0x3bca1d15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43a1d1a, 90 },
    { 0x829a1d1b, 43 },
    { 0x219a1d1c, 46 },
    { 0x110a1d1c, 90 },
    { 0x468a1d1e, 100 },
    { 0x5c2a1d1f, 88 },
    { 0x6eda1d1d, 61 },
    { 0xd31a1d1d, 95 },
    { 0x53ca1d21, 95 },
    { 0xbaaa1d1a, 98 },
    { 0x75ca1d24, 61 },
    { 0, 0 },
    { 0xec5a1d26, 33 },
    { 0, 0 },
    { 0xa39a1d28, 43 },
    { 0x6d3a1d29, 93 },
    { 0xdf0a1d28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a1d2e, 55 },
    { 0x271a1d2f, 95 },
    { 0xbc4a1d2f, 34 },
    { 0xf5aa1d30, 95 },
    { 0, 0 },
    { 0x5dba1d33, 100 },
    { 0, 0 },
    { 0x10ca1d35, 61 },
    { 0, 0 },
    { 0x2e4a1d37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294a1d3f, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bea1d44, 93 },
    { 0x387a1d44, 76 },
    { 0, 0 },
    { 0xb63a1d47, 95 },
    { 0xf89a1d48, 98 },
    { 0, 0 },
    { 0x45ba1d4a, 61 },
    { 0xe19a1d4b, 91 },
    { 0x952a1d4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ba1d51, 76 },
    { 0xe51a1d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a1d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x902a1d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ea1d5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580a1d62, 43 },
    { 0x13da1d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1a1d69, 33 },
    { 0xe3aa1d6a, 90 },
    { 0, 0 },
    { 0x69fa1d6c, 61 },
    { 0, 0 },
    { 0xb3fa1d6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x801a1d71, 76 },
    { 0xae5a1d72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea1d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36a1d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a1d88, 93 },
    { 0x87ba1d89, 61 },
    { 0, 0 },
    { 0xf0ba1d8b, 43 },
    { 0xaaea1d8c, 90 },
    { 0x20ca1d8c, 61 },
    { 0x5aea1d8c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da1d91, 53 },
    { 0x839a1d91, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78a1d95, 43 },
    { 0x56da1d96, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cba1d99, 95 },
    { 0xbbfa1d9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22a1d9d, 85 },
    { 0x3f2a1d9e, 90 },
    { 0xda3a1d9e, 98 },
    { 0xe87a1da0, 17 },
    { 0xdd1a1da1, 85 },
    { 0xd89a1da2, 98 },
    { 0x172a1da1, 75 },
    { 0, 0 },
    { 0x2f4a1da5, 33 },
    { 0x61aa1da5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x352a1da9, 88 },
    { 0x7a2a1daa, 43 },
    { 0x82aa1dab, 77 },
    { 0x377a1dab, 46 },
    { 0xfc1a1dad, 98 },
    { 0, 0 },
    { 0xfc8a1daf, 46 },
    { 0xccea1db0, 23 },
    { 0xf72a1db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa1db8, 95 },
    { 0x2f1a1db9, 95 },
    { 0xd31a1db8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0a1dbe, 95 },
    { 0xca2a1dbf, 67 },
    { 0x221a1dc0, 61 },
    { 0, 0 },
    { 0x455a1dc2, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aa1dc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a1dcc, 98 },
    { 0x744a1dcd, 98 },
    { 0x882a1dce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54a1dd1, 61 },
    { 0x39da1dd2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ba1dd6, 23 },
    { 0x105a1dd7, 61 },
    { 0x22ca1dd6, 93 },
    { 0, 0 },
    { 0x946a1dda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ea1dde, 23 },
    { 0x1a8a1dde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ea1de2, 23 },
    { 0xb54a1de2, 91 },
    { 0x414a1de2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fa1dec, 55 },
    { 0x3c0a1dec, 75 },
    { 0, 0 },
    { 0x9cea1def, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a1df2, 90 },
    { 0x963a1df2, 98 },
    { 0, 0 },
    { 0xb1a1df5, 87 },
    { 0x3d1a1df6, 23 },
    { 0x8e7a1df5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14aa1dff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x596a1e02, 98 },
    { 0xddea1e03, 67 },
    { 0x1d2a1e03, 95 },
    { 0, 0 },
    { 0xaeca1e06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0a1e0a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69a1e12, 90 },
    { 0x403a1e12, 67 },
    { 0xa00a1e14, 23 },
    { 0x4cba1e12, 91 },
    { 0x95da1e16, 33 },
    { 0x822a1e16, 85 },
    { 0xf2fa1e17, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a1e1b, 33 },
    { 0xc0aa1e1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba1e20, 76 },
    { 0x9e5a1e21, 23 },
    { 0x95ba1e21, 93 },
    { 0x8f5a1e22, 90 },
    { 0x7c8a1e20, 33 },
    { 0x21ca1e22, 94 },
    { 0x9bba1e26, 61 },
    { 0x41aa1e23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7a1e2d, 61 },
    { 0, 0 },
    { 0x610a1e2f, 61 },
    { 0x2dca1e30, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ea1e33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a1e39, 95 },
    { 0, 0 },
    { 0x4b4a1e3b, 94 },
    { 0xcf0a1e3c, 66 },
    { 0x40ea1e3c, 61 },
    { 0xcfda1e3c, 94 },
    { 0x672a1e3f, 78 },
    { 0xd6aa1e3f, 90 },
    { 0x45a1e3f, 94 },
    { 0x8e8a1e42, 93 },
    { 0x96ca1e42, 66 },
    { 0x337a1e44, 98 },
    { 0x119a1e3f, 85 },
    { 0x5f9a1e3e, 51 },
    { 0, 0 },
    { 0x372a1e48, 33 },
    { 0, 0 },
    { 0xf84a1e4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4a1e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x731a1e50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ea1e53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3a1e58, 61 },
    { 0x85aa1e59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2a1e64, 61 },
    { 0xaa4a1e65, 90 },
    { 0x2d6a1e66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41a1e69, 61 },
    { 0xb58a1e6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ea1e6f, 98 },
    { 0xac9a1e6f, 23 },
    { 0x70fa1e6f, 23 },
    { 0xd2ba1e72, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14a1e76, 90 },
    { 0xe10a1e76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a1e7a, 52 },
    { 0, 0 },
    { 0x36ba1e7c, 93 },
    { 0xd70a1e7c, 61 },
    { 0x82a1e7e, 90 },
    { 0x1d9a1e7f, 98 },
    { 0xca2a1e7f, 84 },
    { 0x721a1e81, 55 },
    { 0xe2ea1e82, 93 },
    { 0x8dfa1e7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19a1e87, 90 },
    { 0xfcca1e87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2a1e8c, 61 },
    { 0x923a1e8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfa1e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x146a1e94, 46 },
    { 0x639a1e94, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ba1e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ea1e9e, 55 },
    { 0xd6ca1e9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ea1ea4, 84 },
    { 0xfcaa1ea4, 87 },
    { 0xa55a1ea6, 99 },
    { 0x9faa1ea7, 36 },
    { 0x6c1a1ea7, 76 },
    { 0xa6fa1ea7, 75 },
    { 0x1aaa1eaa, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02a1eae, 66 },
    { 0x68a1eae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x836a1eb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba1eb5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a1eb9, 61 },
    { 0, 0 },
    { 0xb3ea1ebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613a1ebf, 90 },
    { 0x924a1ebf, 44 },
    { 0x90a1ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a1ec8, 78 },
    { 0, 0 },
    { 0x5dba1eca, 23 },
    { 0xf97a1eca, 95 },
    { 0x61fa1ecc, 98 },
    { 0x1d5a1ecd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eca1ed4, 84 },
    { 0, 0 },
    { 0x6b4a1ed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x569a1ed9, 78 },
    { 0, 0 },
    { 0xe07a1edb, 90 },
    { 0x3b1a1edb, 33 },
    { 0xd8a1edc, 84 },
    { 0x696a1edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaa1ee2, 85 },
    { 0x41ca1ee3, 95 },
    { 0xb46a1ee4, 61 },
    { 0xab1a1ee4, 98 },
    { 0x997a1ee4, 90 },
    { 0xed5a1ee7, 76 },
    { 0xa0fa1ee7, 84 },
    { 0xae5a1ee7, 61 },
    { 0xafa1ee4, 61 },
    { 0xb32a1eeb, 33 },
    { 0x496a1eec, 91 },
    { 0xbe7a1eec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a1ef4, 98 },
    { 0x30a1ef5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eba1ef8, 87 },
    { 0x2d2a1ef9, 23 },
    { 0x511a1efa, 61 },
    { 0x938a1ef9, 43 },
    { 0xee3a1efc, 98 },
    { 0x6a8a1efc, 67 },
    { 0, 0 },
    { 0xeeda1eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a1f05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89a1f09, 90 },
    { 0xb4ba1f09, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x531a1f0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a1f12, 69 },
    { 0, 0 },
    { 0xe23a1f14, 95 },
    { 0x44da1f14, 84 },
    { 0, 0 },
    { 0xbd7a1f17, 78 },
    { 0x83fa1f17, 61 },
    { 0, 0 },
    { 0x16ca1f1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda1f1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a1f25, 61 },
    { 0x560a1f26, 52 },
    { 0x2afa1f27, 85 },
    { 0xc2aa1f28, 34 },
    { 0xaeba1f29, 93 },
    { 0x45a1f28, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84a1f2e, 100 },
    { 0x35ba1f2f, 95 },
    { 0x8d1a1f30, 55 },
    { 0xf04a1f2e, 17 },
    { 0x70ea1f30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20a1f40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fda1f44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x308a1f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fea1f4b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a1f51, 80 },
    { 0x6c7a1f52, 67 },
    { 0x44da1f53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425a1f57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a1f5b, 33 },
    { 0x4f3a1f5c, 61 },
    { 0, 0 },
    { 0x389a1f5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3a1f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0a1f66, 90 },
    { 0xe0aa1f66, 76 },
    { 0, 0 },
    { 0x509a1f69, 23 },
    { 0x6eba1f6a, 98 },
    { 0x82ca1f6a, 43 },
    { 0xd6aa1f6c, 87 },
    { 0xf4da1f6b, 61 },
    { 0xe98a1f6e, 98 },
    { 0xbb9a1f6e, 99 },
    { 0, 0 },
    { 0x3e0a1f71, 90 },
    { 0, 0 },
    { 0x377a1f73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ada1f7b, 76 },
    { 0x64da1f7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4a1f80, 100 },
    { 0x11aa1f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485a1f88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5a1f8d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x158a1f90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eba1f99, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5daa1f9c, 33 },
    { 0x9f7a1f9d, 83 },
    { 0, 0 },
    { 0x20fa1f9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7a1fa4, 99 },
    { 0, 0 },
    { 0xd2ba1fa6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5a1fb6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30a1fb9, 33 },
    { 0, 0 },
    { 0xfcfa1fbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a1fbe, 95 },
    { 0, 0 },
    { 0x99ba1fc0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53a1fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80a1fc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9a1fcd, 23 },
    { 0x349a1fce, 33 },
    { 0x622a1fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6a1fd2, 76 },
    { 0, 0 },
    { 0x816a1fd4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afa1fd8, 46 },
    { 0xd7aa1fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822a1fde, 33 },
    { 0x5f9a1fde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48a1fe3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1a1fe6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76a1fec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ba1fef, 33 },
    { 0xf00a1ff0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fa1ff4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a1ff9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25a1ffe, 77 },
    { 0x7bea1ffe, 61 },
    { 0x9efa1fff, 90 },
    { 0x2bba2001, 95 },
    { 0x2a2002, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384a2006, 23 },
    { 0xde3a2007, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20a200c, 52 },
    { 0x692a200d, 76 },
    { 0x5dca200e, 98 },
    { 0xc47a200f, 94 },
    { 0xac3a2010, 90 },
    { 0xcc8a2011, 90 },
    { 0xc1ca2012, 90 },
    { 0xe79a2012, 93 },
    { 0x7b2a2014, 33 },
    { 0xf10a2015, 98 },
    { 0x214a2016, 95 },
    { 0xefaa200f, 90 },
    { 0x513a2018, 95 },
    { 0x5e7a2019, 65 },
    { 0x32ca201a, 46 },
    { 0xa2fa2018, 55 },
    { 0x1e5a201a, 46 },
    { 0xd3aa201d, 84 },
    { 0xf13a201d, 87 },
    { 0xaa8a200f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492a2024, 91 },
    { 0x8a6a2025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fca2031, 77 },
    { 0xd49a2031, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fca2035, 99 },
    { 0x312a2036, 23 },
    { 0x166a2037, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fa203b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9da203f, 85 },
    { 0, 0 },
    { 0xf62a2041, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaca204d, 43 },
    { 0x6a204e, 33 },
    { 0, 0 },
    { 0x5ba2050, 98 },
    { 0xda1a2051, 95 },
    { 0x494a2052, 33 },
    { 0, 0 },
    { 0xb1fa2054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746a2058, 95 },
    { 0xd6a2059, 33 },
    { 0, 0 },
    { 0x670a205b, 90 },
    { 0x611a205c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782a2062, 87 },
    { 0x3d4a2063, 55 },
    { 0x780a2064, 33 },
    { 0x1f4a2065, 61 },
    { 0xd08a2066, 76 },
    { 0x1cfa2065, 98 },
    { 0x492a2065, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fa206c, 52 },
    { 0x705a206d, 98 },
    { 0x525a206e, 34 },
    { 0x1daa206d, 76 },
    { 0xa97a206d, 100 },
    { 0x1f2a2071, 23 },
    { 0, 0 },
    { 0x93fa2073, 90 },
    { 0x2dea2074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52a2077, 90 },
    { 0xccea2078, 84 },
    { 0, 0 },
    { 0xc9fa207a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4a207d, 88 },
    { 0, 0 },
    { 0xa5ea207f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a2082, 55 },
    { 0x444a2082, 34 },
    { 0xff2a2084, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ba2088, 95 },
    { 0x86aa2088, 24 },
    { 0x53ea208a, 98 },
    { 0x4a9a208b, 95 },
    { 0xe1ea208c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715a2091, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9a2097, 13 },
    { 0, 0 },
    { 0x400a2099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fa209e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ca20a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07a20ad, 90 },
    { 0x618a20ae, 33 },
    { 0x6e0a20ad, 87 },
    { 0x8b8a20b0, 91 },
    { 0x733a20b1, 66 },
    { 0, 0 },
    { 0xfa3a20b3, 34 },
    { 0xa53a20b3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291a20b9, 78 },
    { 0xf76a20b9, 43 },
    { 0x3a4a20bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909a20c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24a20c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a20ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a20ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x196a20d1, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2a20d4, 23 },
    { 0xd78a20d5, 90 },
    { 0x5e3a20d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a20dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5a20e0, 78 },
    { 0, 0 },
    { 0xb18a20e2, 53 },
    { 0x2ba20e3, 46 },
    { 0xa71a20e3, 61 },
    { 0x94ea20e5, 90 },
    { 0xa61a20e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a20e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686a20ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdba20ff, 87 },
    { 0xa90a2100, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7a2105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a210b, 98 },
    { 0, 0 },
    { 0x937a210d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a2110, 98 },
    { 0x478a2110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39a2115, 90 },
    { 0xf31a2116, 69 },
    { 0x71ca2117, 33 },
    { 0, 0 },
    { 0xbc5a2119, 76 },
    { 0x66ba211a, 98 },
    { 0x5b7a211a, 23 },
    { 0, 0 },
    { 0x6dda211d, 91 },
    { 0x31a211e, 61 },
    { 0x72ea211d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907a2123, 90 },
    { 0x384a2124, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x530a2127, 90 },
    { 0x4eea2127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1a212d, 33 },
    { 0xf45a212e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1a2131, 17 },
    { 0xb54a2132, 95 },
    { 0x2daa2133, 67 },
    { 0x87aa2134, 83 },
    { 0xfd3a2134, 98 },
    { 0x9f5a2136, 33 },
    { 0x47a2137, 65 },
    { 0x465a2134, 81 },
    { 0x8e1a2134, 75 },
    { 0xb8a213a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0a2142, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20a214b, 23 },
    { 0xda6a214b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca2150, 55 },
    { 0x145a2151, 55 },
    { 0xedfa2151, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911a2158, 84 },
    { 0x634a2159, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93da215c, 76 },
    { 0, 0 },
    { 0x8a9a215e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a2168, 76 },
    { 0x53da2168, 90 },
    { 0, 0 },
    { 0x6faa216b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ca2171, 91 },
    { 0x5aca2172, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6a2175, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290a217a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a217d, 33 },
    { 0x696a217e, 61 },
    { 0xd04a217f, 34 },
    { 0x54a217d, 34 },
    { 0x8cfa2181, 90 },
    { 0x4bea2182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19a2186, 23 },
    { 0xea2a2186, 76 },
    { 0x928a2188, 61 },
    { 0x80ea2188, 95 },
    { 0x7b7a2187, 95 },
    { 0, 0 },
    { 0xe14a218c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907a2190, 84 },
    { 0, 0 },
    { 0x865a2192, 90 },
    { 0, 0 },
    { 0xacca2194, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x684a2197, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fa219c, 90 },
    { 0, 0 },
    { 0xc89a219e, 95 },
    { 0xe21a219e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x648a21a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aa21ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2aa21b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x801a21b5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7a21bf, 43 },
    { 0xce6a21bf, 98 },
    { 0x4c0a21c1, 98 },
    { 0x223a21c2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfa21c7, 98 },
    { 0xfdfa21c8, 76 },
    { 0x65fa21c9, 91 },
    { 0x48da21ca, 33 },
    { 0xfcfa21cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a21d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9a21d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9a21d8, 52 },
    { 0x314a21d9, 98 },
    { 0xb1fa21d8, 85 },
    { 0x8aaa21d9, 98 },
    { 0xf63a21dc, 43 },
    { 0x7d4a21d9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79a21e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ca21e5, 61 },
    { 0, 0 },
    { 0x170a21e7, 95 },
    { 0x1fca21e8, 76 },
    { 0x85a21e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a21f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877a21f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09a21fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5a2202, 100 },
    { 0x8afa2203, 33 },
    { 0x37ba2204, 90 },
    { 0x179a2202, 90 },
    { 0xf8ba2204, 83 },
    { 0, 0 },
    { 0x7afa2208, 95 },
    { 0, 0 },
    { 0x225a220a, 76 },
    { 0x83ca220b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64a2213, 90 },
    { 0xc3ba2214, 76 },
    { 0xd1da2215, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789a221a, 98 },
    { 0, 0 },
    { 0x1a5a221c, 46 },
    { 0xa76a221d, 61 },
    { 0xcfca221d, 90 },
    { 0x4fda221f, 78 },
    { 0xb1a2220, 95 },
    { 0xd38a221e, 99 },
    { 0x9d4a2222, 36 },
    { 0x69da221f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ea222a, 34 },
    { 0, 0 },
    { 0xad8a222c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8a2233, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7a2239, 23 },
    { 0x2f3a223a, 33 },
    { 0x70ea2239, 61 },
    { 0xadea223c, 76 },
    { 0x5ea2239, 90 },
    { 0xcd2a223e, 33 },
    { 0xbaea223f, 33 },
    { 0xf7ba223f, 33 },
    { 0, 0 },
    { 0x73a2242, 91 },
    { 0x193a2242, 34 },
    { 0x36ba2242, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13a224b, 88 },
    { 0x8b9a224b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7da2251, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aa2256, 95 },
    { 0, 0 },
    { 0x12ba2258, 67 },
    { 0x1d5a2259, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca225c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dda225f, 52 },
    { 0xc08a2260, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89a2264, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59a2267, 23 },
    { 0xbaba2267, 55 },
    { 0x16ba2269, 77 },
    { 0x9ca226a, 61 },
    { 0x5dba226a, 93 },
    { 0xa00a226b, 61 },
    { 0, 0 },
    { 0x896a226e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x835a2271, 91 },
    { 0, 0 },
    { 0x914a2273, 65 },
    { 0x740a2274, 99 },
    { 0x688a2275, 33 },
    { 0xf0aa2273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6a2279, 23 },
    { 0x4b6a227a, 33 },
    { 0x3c5a227b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ca227e, 55 },
    { 0, 0 },
    { 0x577a2280, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331a2285, 17 },
    { 0x7cba2285, 66 },
    { 0xf3fa2287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eda228a, 34 },
    { 0, 0 },
    { 0x102a228c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cda2292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a2295, 23 },
    { 0, 0 },
    { 0xfe1a2297, 67 },
    { 0x437a2298, 61 },
    { 0, 0 },
    { 0xbd5a229a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751a22a3, 76 },
    { 0x8a5a22a4, 43 },
    { 0x7b9a22a4, 61 },
    { 0x857a22a6, 90 },
    { 0x10fa22a5, 87 },
    { 0x82a22a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924a22ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56a22af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa22b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaa22b6, 87 },
    { 0x40da22b7, 53 },
    { 0xb01a22b8, 98 },
    { 0, 0 },
    { 0xe02a22ba, 95 },
    { 0xc8a22bb, 90 },
    { 0xb06a22bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xecea22bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4a22c5, 88 },
    { 0x764a22c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9a22ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eda22cf, 56 },
    { 0, 0 },
    { 0xa22d1, 67 },
    { 0, 0 },
    { 0x7b8a22d3, 69 },
    { 0xbd9a22d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372a22de, 61 },
    { 0x83a22df, 95 },
    { 0x55fa22e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426a22e6, 23 },
    { 0, 0 },
    { 0x8a0a22e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x614a22eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x843a22ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ea22f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea22f5, 95 },
    { 0x57ba22f5, 61 },
    { 0, 0 },
    { 0x3a3a22f8, 76 },
    { 0x52fa22f8, 61 },
    { 0xfbda22f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196a2301, 61 },
    { 0x93da2301, 55 },
    { 0, 0 },
    { 0x2ffa2304, 61 },
    { 0, 0 },
    { 0xf93a2306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3a2309, 90 },
    { 0x665a230a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2a230e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba2313, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x87da2316, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afa231a, 55 },
    { 0x5dea231b, 61 },
    { 0x109a231c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a2320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3a2324, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46a2334, 55 },
    { 0, 0 },
    { 0xf7aa2336, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa233a, 98 },
    { 0, 0 },
    { 0x217a233c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffa2340, 78 },
    { 0x20fa2341, 23 },
    { 0xcbfa2342, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9a2345, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ba2348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fa2358, 61 },
    { 0, 0 },
    { 0x485a235a, 98 },
    { 0xf9fa235b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2a235e, 55 },
    { 0xefea235f, 52 },
    { 0x402a2360, 52 },
    { 0x9f9a2361, 90 },
    { 0xb97a2362, 88 },
    { 0xdb4a2363, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a2369, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372a236d, 98 },
    { 0xc1da236e, 91 },
    { 0xd92a236f, 23 },
    { 0xba236d, 34 },
    { 0x2b9a236d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aba2376, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x439a2379, 52 },
    { 0x685a237a, 65 },
    { 0xc27a237b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6a2384, 23 },
    { 0, 0 },
    { 0x372a2386, 74 },
    { 0, 0 },
    { 0x939a2388, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x828a238b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a238e, 34 },
    { 0x8d1a238f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51a2392, 23 },
    { 0x995a2393, 90 },
    { 0x8dba2393, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea2397, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a239a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36da239e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7a23a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x739a23a4, 90 },
    { 0x168a23a5, 75 },
    { 0x7f5a23a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48a23ab, 90 },
    { 0, 0 },
    { 0x7ada23ad, 33 },
    { 0xd2aa23ae, 33 },
    { 0, 0 },
    { 0x1b5a23b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa23b8, 67 },
    { 0xd48a23b9, 61 },
    { 0xc3fa23ba, 76 },
    { 0x2b4a23bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687a23c3, 87 },
    { 0x416a23c4, 33 },
    { 0, 0 },
    { 0xce0a23c6, 76 },
    { 0x5f6a23c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310a23ce, 90 },
    { 0, 0 },
    { 0x6c9a23d0, 61 },
    { 0, 0 },
    { 0x91ea23d2, 23 },
    { 0xd22a23d3, 51 },
    { 0x17ea23d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a23d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fba23db, 55 },
    { 0, 0 },
    { 0xbe3a23dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x875a23e0, 78 },
    { 0x41ba23e0, 34 },
    { 0x713a23e2, 90 },
    { 0x335a23e3, 23 },
    { 0, 0 },
    { 0x869a23e5, 93 },
    { 0xf46a23e6, 85 },
    { 0x5f2a23e7, 75 },
    { 0x338a23e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71a23eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60a23f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6da23f4, 90 },
    { 0x247a23f4, 90 },
    { 0x1eda23f6, 81 },
    { 0x388a23f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794a23fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8a2405, 33 },
    { 0xeaea2406, 55 },
    { 0x1c4a2406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x680a240a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84da2415, 33 },
    { 0x4e2a2415, 99 },
    { 0xbbfa2417, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ca241b, 52 },
    { 0xd2ea241c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23a241f, 33 },
    { 0x7f4a241f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7a2425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5a2429, 98 },
    { 0, 0 },
    { 0xe59a242b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x119a242e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55a2433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a2436, 61 },
    { 0x438a2436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a243a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82a2440, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfca2446, 96 },
    { 0x29da2447, 33 },
    { 0xd7fa2446, 85 },
    { 0x74da2449, 61 },
    { 0x51aa2447, 43 },
    { 0xbeda244b, 93 },
    { 0, 0 },
    { 0x7e3a244d, 95 },
    { 0xb62a244e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x593a2451, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x75da2454, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03a2457, 88 },
    { 0xb8da2457, 46 },
    { 0xa33a2459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38aa245f, 98 },
    { 0x748a245f, 95 },
    { 0, 0 },
    { 0x58ca2462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275a2466, 90 },
    { 0x64ca2466, 95 },
    { 0x7a7a2466, 66 },
    { 0x2b8a2469, 77 },
    { 0x931a2469, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3a246d, 55 },
    { 0xa77a246e, 91 },
    { 0, 0 },
    { 0x234a2470, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa2477, 33 },
    { 0, 0 },
    { 0x46ea2479, 95 },
    { 0xabaa2479, 80 },
    { 0x760a2479, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11da2481, 55 },
    { 0x517a2482, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5a2485, 87 },
    { 0x5c0a2485, 40 },
    { 0, 0 },
    { 0xe75a2488, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ca248d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487a2494, 46 },
    { 0xdca2494, 33 },
    { 0xa71a2496, 43 },
    { 0xe9ea2496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8daa249b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ea249f, 98 },
    { 0x713a249f, 90 },
    { 0, 0 },
    { 0x40a24a2, 90 },
    { 0x81fa24a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43aa24a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13a24ab, 93 },
    { 0x607a24ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495a24b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9da24b4, 85 },
    { 0xadca24b5, 78 },
    { 0x47ca24b6, 43 },
    { 0xa95a24b7, 61 },
    { 0xdffa24b8, 61 },
    { 0, 0 },
    { 0x27da24ba, 67 },
    { 0x704a24bb, 34 },
    { 0x6d5a24bc, 100 },
    { 0xfa8a24bd, 94 },
    { 0x8aca24bd, 34 },
    { 0xbf7a24bb, 95 },
    { 0xcc0a24bf, 61 },
    { 0, 0 },
    { 0x2cea24c2, 93 },
    { 0x84aa24c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ca24c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80a24cc, 46 },
    { 0xa8fa24cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba24d1, 55 },
    { 0, 0 },
    { 0x27ba24d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x858a24d6, 61 },
    { 0x4e8a24d7, 38 },
    { 0x764a24d7, 95 },
    { 0xe09a24d6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65a24dc, 90 },
    { 0x6d2a24dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795a24e1, 93 },
    { 0x1b6a24e2, 95 },
    { 0, 0 },
    { 0x3cda24e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a24e7, 99 },
    { 0xa3ba24e7, 78 },
    { 0xe8da24e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01a24ec, 34 },
    { 0x483a24ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a24f0, 76 },
    { 0xdd2a24f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88a24f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x177a24f7, 61 },
    { 0, 0 },
    { 0x6a8a24f9, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5a24fc, 90 },
    { 0x765a24fd, 93 },
    { 0xb50a24fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33a2501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xada2504, 90 },
    { 0xb45a2504, 90 },
    { 0x454a2504, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779a250a, 61 },
    { 0x47fa250b, 76 },
    { 0x194a250c, 90 },
    { 0, 0 },
    { 0x714a250e, 95 },
    { 0x1dfa250f, 95 },
    { 0x627a2510, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eca2513, 97 },
    { 0x175a2514, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721a251a, 33 },
    { 0xa28a251b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a2523, 43 },
    { 0x392a2523, 61 },
    { 0xe06a2523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aa2528, 90 },
    { 0, 0 },
    { 0xb0ca252a, 95 },
    { 0, 0 },
    { 0x25da252c, 98 },
    { 0xc42a252d, 99 },
    { 0, 0 },
    { 0x47fa252f, 55 },
    { 0, 0 },
    { 0x9eea2531, 55 },
    { 0xe3ba2532, 33 },
    { 0, 0 },
    { 0xc19a2534, 33 },
    { 0x675a2535, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7a2539, 93 },
    { 0xc1ea253a, 99 },
    { 0x44da253b, 55 },
    { 0, 0 },
    { 0x410a253d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6a2543, 90 },
    { 0xe06a2544, 55 },
    { 0, 0 },
    { 0xbe1a2546, 55 },
    { 0xb81a2546, 87 },
    { 0, 0 },
    { 0x892a2549, 90 },
    { 0, 0 },
    { 0xbc2a254b, 35 },
    { 0x594a254c, 98 },
    { 0xc4ca254d, 95 },
    { 0x338a254c, 43 },
    { 0x8f6a254f, 90 },
    { 0xa4fa2550, 98 },
    { 0x40aa2551, 79 },
    { 0, 0 },
    { 0xbb6a2553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219a255e, 90 },
    { 0, 0 },
    { 0x551a2560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fa2565, 60 },
    { 0, 0 },
    { 0xd6da2567, 33 },
    { 0, 0 },
    { 0xc2ca2569, 87 },
    { 0xac8a256a, 90 },
    { 0x73ca256b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a256e, 78 },
    { 0x82ea256e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822a2576, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca257e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a2584, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57a2587, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca258c, 98 },
    { 0, 0 },
    { 0xa1ca258e, 23 },
    { 0xc3aa258e, 34 },
    { 0, 0 },
    { 0x98ea2591, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3a2594, 66 },
    { 0, 0 },
    { 0xc6fa2596, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea2599, 90 },
    { 0x7a6a259a, 95 },
    { 0xbe5a259a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a259e, 90 },
    { 0xad1a259f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a25a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a25a7, 33 },
    { 0x738a25a8, 90 },
    { 0x4eca25a9, 66 },
    { 0x5d2a25a8, 95 },
    { 0xb0ba25a7, 98 },
    { 0, 0 },
    { 0xb77a25ad, 61 },
    { 0, 0 },
    { 0x52ea25af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a25b3, 95 },
    { 0xd7a25b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaa25be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3a25c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5a25c4, 76 },
    { 0, 0 },
    { 0x981a25c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x377a25c9, 55 },
    { 0, 0 },
    { 0x411a25cb, 77 },
    { 0x706a25cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ba25d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118a25d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ea25db, 23 },
    { 0x7a7a25dc, 98 },
    { 0xf0ca25dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea25eb, 90 },
    { 0xca6a25eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea25f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a25f3, 17 },
    { 0x343a25f3, 76 },
    { 0, 0 },
    { 0x74da25f6, 95 },
    { 0x8dfa25f6, 91 },
    { 0, 0 },
    { 0x5d1a25f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a25fc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a2605, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x799a2608, 77 },
    { 0x611a2609, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a260e, 55 },
    { 0, 0 },
    { 0x9cda2610, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37a2613, 23 },
    { 0xbc5a2613, 61 },
    { 0x442a2614, 61 },
    { 0x715a2616, 90 },
    { 0xebfa2615, 61 },
    { 0x77ba2618, 81 },
    { 0, 0 },
    { 0x918a261a, 61 },
    { 0, 0 },
    { 0x892a261c, 33 },
    { 0, 0 },
    { 0xddaa261e, 46 },
    { 0x1c8a261f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ba2624, 33 },
    { 0xcc9a2625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435a2629, 95 },
    { 0xdb5a262a, 88 },
    { 0x311a2629, 90 },
    { 0x17ba262c, 53 },
    { 0x1daa262d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa2632, 87 },
    { 0x5e5a2633, 84 },
    { 0x2dca2634, 90 },
    { 0, 0 },
    { 0xaa5a2636, 69 },
    { 0, 0 },
    { 0x5e0a2638, 33 },
    { 0xd7fa2638, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdba2641, 46 },
    { 0, 0 },
    { 0xec6a2643, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x685a2646, 95 },
    { 0xffca2647, 66 },
    { 0x3eca2648, 61 },
    { 0x855a2647, 76 },
    { 0xe66a2646, 90 },
    { 0, 0 },
    { 0x357a264c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9a264f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ca2652, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2da2655, 33 },
    { 0x491a2655, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ba265d, 61 },
    { 0xacca265d, 95 },
    { 0x7d2a265d, 98 },
    { 0x143a265e, 76 },
    { 0xa265f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fa2664, 95 },
    { 0x5e8a2665, 33 },
    { 0xb9aa2666, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaa2669, 84 },
    { 0xacba266a, 87 },
    { 0x18aa2669, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfa2671, 90 },
    { 0xa06a2671, 61 },
    { 0x703a2673, 43 },
    { 0x37ca2673, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x507a2677, 43 },
    { 0, 0 },
    { 0x758a2679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x473a267c, 33 },
    { 0x3b3a267d, 61 },
    { 0xc17a267e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04a2684, 78 },
    { 0xc8aa2685, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a268a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a268f, 76 },
    { 0x5e4a2690, 69 },
    { 0xe84a2690, 76 },
    { 0xdb4a2692, 61 },
    { 0x914a2692, 17 },
    { 0, 0 },
    { 0x21da2695, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a2698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a26a3, 78 },
    { 0x32aa26a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95a26b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a26bc, 34 },
    { 0xfaa26bd, 66 },
    { 0, 0 },
    { 0xfe8a26bf, 91 },
    { 0, 0 },
    { 0x168a26c1, 70 },
    { 0x3fea26c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a26c9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1a26cc, 61 },
    { 0x5bba26cd, 52 },
    { 0xdb8a26ce, 33 },
    { 0x2b9a26cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4aa26d4, 95 },
    { 0, 0 },
    { 0x45ea26d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13da26dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4a26e0, 98 },
    { 0, 0 },
    { 0x452a26e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85a26e5, 76 },
    { 0xfe9a26e6, 93 },
    { 0, 0 },
    { 0x5b2a26e8, 78 },
    { 0x83aa26e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x67da26ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x292a26ef, 55 },
    { 0x331a26f0, 98 },
    { 0x5f2a26ef, 23 },
    { 0x672a26f2, 25 },
    { 0x717a26f1, 43 },
    { 0, 0 },
    { 0xc60a26f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x957a26f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730a26fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a2701, 90 },
    { 0x5da2701, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fa2707, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57a270c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4a2712, 43 },
    { 0x4aba2713, 98 },
    { 0, 0 },
    { 0xc2ea2715, 90 },
    { 0x79da2715, 23 },
    { 0x242a2715, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fa271b, 68 },
    { 0, 0 },
    { 0x1d5a271d, 78 },
    { 0x6b1a271e, 61 },
    { 0, 0 },
    { 0x25da2720, 55 },
    { 0xafaa2721, 61 },
    { 0xca2722, 98 },
    { 0, 0 },
    { 0x797a2724, 76 },
    { 0, 0 },
    { 0xa9ea2726, 90 },
    { 0xf00a2727, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dda272b, 90 },
    { 0, 0 },
    { 0xc53a272d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ea2730, 61 },
    { 0x53aa2730, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58aa2735, 87 },
    { 0x112a2736, 99 },
    { 0x836a2737, 66 },
    { 0x11fa2737, 84 },
    { 0x5b0a2739, 90 },
    { 0xf94a273a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284a273f, 55 },
    { 0x7f5a273f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28a2745, 88 },
    { 0xaf8a2745, 92 },
    { 0x238a2745, 61 },
    { 0xc60a2748, 34 },
    { 0xa7aa2749, 100 },
    { 0xd63a274a, 100 },
    { 0xe51a2746, 77 },
    { 0xe42a274c, 76 },
    { 0x9f4a274c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x454a2750, 34 },
    { 0x6f2a2751, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a2754, 76 },
    { 0, 0 },
    { 0xdcaa2756, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5a2762, 85 },
    { 0xfa4a2762, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaa2766, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a2769, 95 },
    { 0x9da2769, 88 },
    { 0xe43a276b, 76 },
    { 0x27ca276c, 90 },
    { 0xbfa276c, 33 },
    { 0, 0 },
    { 0xb54a276f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39da2773, 40 },
    { 0xe6ba2773, 61 },
    { 0xd7ba2775, 98 },
    { 0x800a2773, 90 },
    { 0xdd6a2775, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aa277c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42a277f, 34 },
    { 0xc62a277f, 61 },
    { 0x29fa2781, 95 },
    { 0x6ea2782, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4a2785, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619a278b, 77 },
    { 0x37ba278b, 93 },
    { 0, 0 },
    { 0xb2ea278e, 52 },
    { 0, 0 },
    { 0x5dda2790, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cea2795, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5a2798, 90 },
    { 0xf92a2798, 61 },
    { 0x364a279a, 61 },
    { 0x948a279b, 91 },
    { 0x9c2a279c, 87 },
    { 0xc92a279d, 90 },
    { 0x16ba279c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x77da27a1, 95 },
    { 0, 0 },
    { 0x5d8a27a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ba27a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ea27ad, 33 },
    { 0x680a27ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ca27b1, 84 },
    { 0x57a27b2, 55 },
    { 0x133a27b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cea27bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626a27c0, 90 },
    { 0x607a27c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacda27c6, 80 },
    { 0x807a27c7, 34 },
    { 0, 0 },
    { 0x90ea27c9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ba27d1, 43 },
    { 0x9aea27d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16da27d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x933a27d9, 55 },
    { 0x510a27d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1a27dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x526a27e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da27e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22a27ec, 84 },
    { 0xfaea27ec, 61 },
    { 0xea6a27ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10a27f2, 52 },
    { 0, 0 },
    { 0x58da27f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a27f8, 76 },
    { 0x6fca27f9, 43 },
    { 0x2b5a27fa, 61 },
    { 0, 0 },
    { 0xaaba27fc, 90 },
    { 0, 0 },
    { 0xcd8a27fe, 43 },
    { 0xd43a27ff, 23 },
    { 0x2f1a27ff, 23 },
    { 0xca2a2801, 84 },
    { 0, 0 },
    { 0x358a2803, 100 },
    { 0x4dda2803, 98 },
    { 0x3eda2805, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942a2809, 76 },
    { 0x6eaa280a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16a280d, 98 },
    { 0xa10a280d, 33 },
    { 0, 0 },
    { 0xc5ca2810, 90 },
    { 0, 0 },
    { 0x6d6a2812, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1a2815, 98 },
    { 0, 0 },
    { 0x1c4a2817, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570a281e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12a2821, 76 },
    { 0x864a2822, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x642a2825, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a2829, 55 },
    { 0, 0 },
    { 0x9e7a282b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74a2832, 91 },
    { 0x637a2832, 23 },
    { 0xf8da2834, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1a2838, 76 },
    { 0x7d5a2839, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x92da283c, 90 },
    { 0, 0 },
    { 0x94fa283e, 33 },
    { 0xa2da283f, 55 },
    { 0xf44a2840, 61 },
    { 0x409a283f, 34 },
    { 0x6f8a2842, 90 },
    { 0x222a283f, 90 },
    { 0, 0 },
    { 0xf94a2845, 75 },
    { 0x674a2846, 46 },
    { 0x59ba2847, 55 },
    { 0xc64a2848, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8a2853, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaea2856, 76 },
    { 0xba2a2856, 90 },
    { 0, 0 },
    { 0x5aaa2859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x453a285c, 76 },
    { 0x7c2a285c, 76 },
    { 0x8dfa285d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50a2864, 78 },
    { 0, 0 },
    { 0x9dba2866, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983a286c, 61 },
    { 0xf79a286d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51a287b, 33 },
    { 0xffa287b, 33 },
    { 0x3cba287d, 33 },
    { 0, 0 },
    { 0x666a287f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78a2887, 61 },
    { 0x692a2888, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaa288e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ba2894, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23a289a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4da289f, 61 },
    { 0, 0 },
    { 0x6a28a1, 93 },
    { 0x711a28a1, 61 },
    { 0, 0 },
    { 0xb0a28a4, 46 },
    { 0, 0 },
    { 0x2ea28a6, 23 },
    { 0x5e2a28a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a28ac, 23 },
    { 0xabea28ad, 34 },
    { 0xb28a28ae, 66 },
    { 0xe5ca28ae, 33 },
    { 0x104a28b0, 61 },
    { 0xe2ca28ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a28b6, 67 },
    { 0, 0 },
    { 0x494a28b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x346a28bb, 65 },
    { 0xb2ca28bc, 40 },
    { 0, 0 },
    { 0xa1fa28be, 95 },
    { 0, 0 },
    { 0xcfa28c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca28c5, 66 },
    { 0x3c0a28c6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9a28ca, 100 },
    { 0, 0 },
    { 0x398a28cc, 52 },
    { 0x6dba28cd, 90 },
    { 0, 0 },
    { 0x4f9a28cf, 90 },
    { 0x300a28d0, 90 },
    { 0xd35a28d1, 87 },
    { 0x999a28d2, 23 },
    { 0xde8a28d3, 90 },
    { 0xa54a28d3, 80 },
    { 0x9f8a28d5, 61 },
    { 0x6a5a28d5, 61 },
    { 0x416a28d3, 66 },
    { 0xaa2a28d8, 90 },
    { 0xc7fa28d4, 93 },
    { 0x4a4a28d1, 76 },
    { 0, 0 },
    { 0x8cca28dc, 98 },
    { 0, 0 },
    { 0x4eea28de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373a28e3, 46 },
    { 0xd0ca28e3, 76 },
    { 0x791a28e5, 61 },
    { 0x836a28e6, 33 },
    { 0x349a28e7, 88 },
    { 0x4bda28e7, 98 },
    { 0xbeba28e7, 55 },
    { 0x20ba28e3, 98 },
    { 0x9e7a28e5, 95 },
    { 0, 0 },
    { 0x75ba28ed, 87 },
    { 0x197a28ee, 55 },
    { 0, 0 },
    { 0xee1a28f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137a28f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747a28fb, 17 },
    { 0xb54a28fc, 90 },
    { 0x575a28fd, 95 },
    { 0xa79a28fb, 91 },
    { 0x557a28ff, 90 },
    { 0x275a28fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ea2908, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ba2919, 5 },
    { 0x4eca291a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8a291d, 95 },
    { 0xf82a291e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a2923, 61 },
    { 0x497a2923, 95 },
    { 0, 0 },
    { 0x910a2926, 33 },
    { 0x784a2926, 100 },
    { 0x611a2928, 37 },
    { 0xfb8a2928, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7a292c, 84 },
    { 0xf34a292d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2a2932, 99 },
    { 0x7fca2933, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3a2937, 63 },
    { 0x7c2a2938, 90 },
    { 0, 0 },
    { 0xb8aa293a, 23 },
    { 0, 0 },
    { 0xc41a293c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa2944, 55 },
    { 0x4aea2945, 85 },
    { 0x5f5a2946, 33 },
    { 0xa1ea2947, 33 },
    { 0xfd4a2948, 76 },
    { 0x232a2944, 55 },
    { 0, 0 },
    { 0x836a294b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2a294f, 76 },
    { 0xd3fa294f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518a2959, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da2968, 90 },
    { 0, 0 },
    { 0x301a296a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77a296e, 98 },
    { 0x950a296f, 100 },
    { 0x70ea2970, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71a2976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5da297c, 90 },
    { 0, 0 },
    { 0x465a297e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88a2986, 33 },
    { 0, 0 },
    { 0x6a1a2988, 23 },
    { 0x7e5a2989, 100 },
    { 0x8d8a298a, 84 },
    { 0x1f7a2989, 34 },
    { 0x302a298a, 23 },
    { 0, 0 },
    { 0xa62a298e, 90 },
    { 0x5e3a298e, 33 },
    { 0, 0 },
    { 0xf16a2991, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36a2996, 93 },
    { 0, 0 },
    { 0xaf4a2998, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447a299c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3a29a2, 61 },
    { 0xce4a29a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947a29a8, 100 },
    { 0, 0 },
    { 0x96aa29aa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aa29b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x292a29b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295a29bc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97a29c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3da29c3, 43 },
    { 0, 0 },
    { 0x946a29c5, 95 },
    { 0x894a29c6, 55 },
    { 0, 0 },
    { 0x2f8a29c8, 93 },
    { 0x77ba29c9, 89 },
    { 0xcada29c9, 78 },
    { 0xcf8a29cb, 43 },
    { 0x2d5a29cb, 34 },
    { 0x383a29c8, 78 },
    { 0, 0 },
    { 0x6e0a29cf, 88 },
    { 0, 0 },
    { 0x116a29d1, 34 },
    { 0xe1da29d2, 43 },
    { 0x360a29d3, 98 },
    { 0, 0 },
    { 0x9a1a29d5, 33 },
    { 0xc3aa29d6, 84 },
    { 0x15ca29d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2a29dc, 76 },
    { 0xb02a29dd, 95 },
    { 0x996a29de, 61 },
    { 0xd59a29de, 95 },
    { 0x8dda29df, 34 },
    { 0x10ca29e1, 43 },
    { 0xbbfa29e0, 8 },
    { 0x5e6a29dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144a29e8, 93 },
    { 0x3dba29e9, 85 },
    { 0x2bda29ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8a29f3, 95 },
    { 0, 0 },
    { 0x615a29f5, 23 },
    { 0xccca29f6, 78 },
    { 0x793a29f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x564a29fa, 33 },
    { 0, 0 },
    { 0xb91a29fc, 23 },
    { 0xe1ba29fc, 98 },
    { 0x7d5a29fd, 100 },
    { 0xd7aa29fd, 98 },
    { 0x790a29fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20a2a03, 96 },
    { 0xed7a2a03, 34 },
    { 0x5e4a2a04, 91 },
    { 0, 0 },
    { 0x653a2a07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ea2a0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3a2a0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fa2a11, 90 },
    { 0xdf7a2a11, 98 },
    { 0, 0 },
    { 0xa96a2a14, 33 },
    { 0, 0 },
    { 0x375a2a16, 90 },
    { 0, 0 },
    { 0xdc7a2a18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a2a1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fa2a1f, 61 },
    { 0x32a2a20, 23 },
    { 0x7a6a2a21, 23 },
    { 0x144a2a22, 90 },
    { 0x93ba2a23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3a2a2c, 66 },
    { 0x12ea2a2d, 91 },
    { 0xc0ba2a2e, 61 },
    { 0xba4a2a2d, 81 },
    { 0xfaea2a30, 33 },
    { 0, 0 },
    { 0x4eca2a32, 95 },
    { 0, 0 },
    { 0x74fa2a34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82a2a38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a2a3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ba2a42, 52 },
    { 0xa50a2a43, 17 },
    { 0x438a2a43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a2a47, 34 },
    { 0x328a2a48, 61 },
    { 0x11ba2a49, 43 },
    { 0x706a2a47, 61 },
    { 0x210a2a49, 93 },
    { 0xf14a2a4c, 84 },
    { 0, 0 },
    { 0xc70a2a4e, 98 },
    { 0x587a2a4f, 35 },
    { 0xe08a2a50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba2a54, 23 },
    { 0, 0 },
    { 0x29ba2a56, 95 },
    { 0, 0 },
    { 0x5e5a2a58, 66 },
    { 0x127a2a58, 81 },
    { 0x7dfa2a59, 80 },
    { 0x610a2a5b, 34 },
    { 0, 0 },
    { 0xd92a2a5d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535a2a61, 23 },
    { 0xe67a2a61, 98 },
    { 0x38ea2a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1a2a67, 55 },
    { 0xb01a2a68, 43 },
    { 0, 0 },
    { 0x585a2a6a, 66 },
    { 0, 0 },
    { 0x273a2a6c, 90 },
    { 0x59a2a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aa2a75, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a2a79, 33 },
    { 0, 0 },
    { 0x898a2a7b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x700a2a7e, 76 },
    { 0, 0 },
    { 0x9c3a2a80, 91 },
    { 0xadfa2a81, 98 },
    { 0x797a2a82, 56 },
    { 0x214a2a81, 34 },
    { 0x21aa2a84, 99 },
    { 0x5b0a2a85, 90 },
    { 0x4c8a2a84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3a2a89, 90 },
    { 0, 0 },
    { 0xe63a2a8b, 78 },
    { 0, 0 },
    { 0xaea2a8d, 90 },
    { 0x381a2a8e, 98 },
    { 0, 0 },
    { 0xc5ea2a90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaa2a96, 76 },
    { 0x199a2a96, 99 },
    { 0x866a2a98, 90 },
    { 0, 0 },
    { 0xe88a2a9a, 90 },
    { 0, 0 },
    { 0xf10a2a9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x810a2a9f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8da2aab, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33da2aaf, 85 },
    { 0, 0 },
    { 0x25ba2ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689a2ab5, 95 },
    { 0x4cda2ab6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xadda2ab9, 23 },
    { 0xf23a2aba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cca2abe, 90 },
    { 0xd35a2abe, 90 },
    { 0x8b2a2ac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6a2aca, 33 },
    { 0x8fca2acb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fa2ace, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x156a2ad1, 37 },
    { 0, 0 },
    { 0xe23a2ad3, 87 },
    { 0, 0 },
    { 0xf16a2ad5, 33 },
    { 0, 0 },
    { 0x5d6a2ad7, 98 },
    { 0xd57a2ad8, 77 },
    { 0, 0 },
    { 0x200a2ada, 88 },
    { 0x9d7a2adb, 93 },
    { 0x8c3a2adc, 34 },
    { 0x746a2adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82a2ae0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeca2ae3, 98 },
    { 0xe56a2ae3, 33 },
    { 0x22da2ae5, 66 },
    { 0x7cea2ae6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ca2aeb, 98 },
    { 0xa58a2aeb, 23 },
    { 0xf5da2aed, 76 },
    { 0x2faa2aec, 98 },
    { 0x513a2aef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1a2af5, 61 },
    { 0, 0 },
    { 0x90aa2af7, 90 },
    { 0, 0 },
    { 0x551a2af9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33a2b00, 23 },
    { 0x3fda2b01, 34 },
    { 0, 0 },
    { 0x1f7a2b03, 65 },
    { 0, 0 },
    { 0x9dba2b05, 46 },
    { 0, 0 },
    { 0x4b8a2b07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dca2b0b, 87 },
    { 0xe55a2b0b, 61 },
    { 0x60ea2b0c, 98 },
    { 0xcc2a2b0e, 87 },
    { 0x92ca2b0e, 98 },
    { 0xf6ba2b10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894a2b15, 99 },
    { 0x67aa2b16, 84 },
    { 0xe90a2b17, 76 },
    { 0xf63a2b18, 23 },
    { 0xae8a2b19, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4a2b20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba2b23, 64 },
    { 0, 0 },
    { 0xddca2b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbada2b29, 76 },
    { 0, 0 },
    { 0x455a2b2b, 47 },
    { 0xa15a2b2c, 23 },
    { 0, 0 },
    { 0x76da2b2e, 33 },
    { 0x7faa2b2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a2b38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x90a2b3b, 90 },
    { 0x739a2b3c, 55 },
    { 0xb14a2b3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23a2b43, 43 },
    { 0xedba2b44, 34 },
    { 0xfd2a2b43, 33 },
    { 0x808a2b46, 97 },
    { 0xa1da2b47, 61 },
    { 0, 0 },
    { 0x81ea2b49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x795a2b4c, 90 },
    { 0x7b7a2b4c, 61 },
    { 0x715a2b4c, 98 },
    { 0xef7a2b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726a2b53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9a2b58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824a2b5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109a2b62, 23 },
    { 0xb1ea2b63, 98 },
    { 0xb1ba2b64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aca2b69, 23 },
    { 0x2e2a2b6a, 65 },
    { 0xbfa2b6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664a2b72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a2b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fa2b79, 65 },
    { 0x299a2b7a, 90 },
    { 0, 0 },
    { 0xb65a2b7c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a2b85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fca2b8e, 34 },
    { 0x46aa2b8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2a2b98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea2ba0, 23 },
    { 0xbbfa2ba1, 85 },
    { 0, 0 },
    { 0xfd2a2ba3, 95 },
    { 0xd44a2ba3, 17 },
    { 0, 0 },
    { 0x20a2ba6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ba2ba9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17a2bae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94a2bb1, 98 },
    { 0xfc5a2bb1, 90 },
    { 0xf8da2bb2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a2bb6, 85 },
    { 0xcfea2bb7, 93 },
    { 0xa30a2bb8, 93 },
    { 0x456a2bb9, 76 },
    { 0xf39a2bba, 61 },
    { 0x3dda2bb9, 96 },
    { 0xac2a2bb7, 90 },
    { 0, 0 },
    { 0x944a2bbe, 66 },
    { 0x61aa2bbf, 99 },
    { 0, 0 },
    { 0xad7a2bc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a2bc6, 61 },
    { 0, 0 },
    { 0x42ea2bc8, 61 },
    { 0, 0 },
    { 0x26fa2bca, 50 },
    { 0xe83a2bca, 23 },
    { 0x5aea2bca, 61 },
    { 0x7bea2bcd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811a2bd2, 95 },
    { 0, 0 },
    { 0x6d4a2bd4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427a2bd8, 90 },
    { 0, 0 },
    { 0x279a2bda, 23 },
    { 0x1fea2bdb, 61 },
    { 0, 0 },
    { 0xfe9a2bdd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ba2be1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322a2be5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa2be9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29a2bf0, 78 },
    { 0xbf3a2bf1, 23 },
    { 0, 0 },
    { 0x86ba2bf3, 33 },
    { 0x856a2bf3, 90 },
    { 0x14a2bf5, 46 },
    { 0x5e3a2bf5, 76 },
    { 0x65ea2bf6, 84 },
    { 0x4d4a2bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa2bfc, 76 },
    { 0xa00a2bfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9a2c03, 33 },
    { 0xddda2c03, 95 },
    { 0x21aa2c05, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfa2c08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbada2c11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccba2c18, 34 },
    { 0, 0 },
    { 0x71fa2c1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2a2c1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14a2c22, 90 },
    { 0x698a2c22, 33 },
    { 0x34ba2c24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aca2c27, 88 },
    { 0, 0 },
    { 0xa59a2c29, 90 },
    { 0, 0 },
    { 0x9a6a2c2b, 90 },
    { 0, 0 },
    { 0xdaaa2c2d, 34 },
    { 0xeb8a2c2d, 23 },
    { 0xebca2c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fba2c33, 34 },
    { 0xd16a2c34, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221a2c38, 98 },
    { 0xb49a2c38, 75 },
    { 0x50ea2c3a, 90 },
    { 0x9bfa2c3a, 98 },
    { 0x860a2c3c, 66 },
    { 0xa16a2c3c, 90 },
    { 0x21a2c3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da2c42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0a2c46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca2c49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a2c4c, 83 },
    { 0x891a2c4c, 67 },
    { 0, 0 },
    { 0xe4a2c4f, 84 },
    { 0x934a2c50, 98 },
    { 0x3eba2c51, 95 },
    { 0x3cca2c52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da2c56, 33 },
    { 0x1f5a2c56, 61 },
    { 0x646a2c57, 46 },
    { 0x82aa2c58, 95 },
    { 0x4cca2c5a, 95 },
    { 0, 0 },
    { 0xd32a2c5c, 95 },
    { 0x4e8a2c5d, 87 },
    { 0x4a2c5e, 61 },
    { 0x7fa2c5f, 23 },
    { 0, 0 },
    { 0xa79a2c61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6a2c69, 55 },
    { 0x295a2c6a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a2c71, 78 },
    { 0x342a2c72, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ba2c76, 33 },
    { 0x1cda2c76, 76 },
    { 0x2b0a2c76, 61 },
    { 0x2e8a2c78, 55 },
    { 0xf37a2c7a, 23 },
    { 0xac2a2c7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a2c7e, 34 },
    { 0, 0 },
    { 0xb75a2c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a2c83, 34 },
    { 0xe08a2c83, 100 },
    { 0x2f4a2c85, 61 },
    { 0x12ba2c86, 77 },
    { 0xa55a2c87, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116a2c8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a2c91, 98 },
    { 0x735a2c92, 43 },
    { 0x210a2c91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ba2c9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafa2c9d, 61 },
    { 0xe11a2c9e, 51 },
    { 0x65ea2c9f, 23 },
    { 0xe0ca2ca0, 100 },
    { 0xddea2ca1, 95 },
    { 0, 0 },
    { 0x61aa2ca3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fa2ca7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da2caf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a2cb6, 33 },
    { 0x7a8a2cb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcba2cbb, 33 },
    { 0xbc5a2cbb, 33 },
    { 0x7c3a2cbb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2a2cc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca2cca, 85 },
    { 0, 0 },
    { 0xe60a2ccc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6a2cda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aea2cdd, 90 },
    { 0x56fa2cdd, 33 },
    { 0xe70a2cdf, 98 },
    { 0x36ea2ce0, 98 },
    { 0, 0 },
    { 0xaafa2ce2, 85 },
    { 0x7f9a2ce3, 98 },
    { 0xb5aa2ce4, 90 },
    { 0x44da2ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa2cea, 33 },
    { 0x27da2ceb, 78 },
    { 0xf20a2cea, 98 },
    { 0xa5ca2ced, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ada2cf1, 66 },
    { 0x50da2cf2, 61 },
    { 0x5aea2cf3, 61 },
    { 0x3f3a2cf4, 95 },
    { 0x6a8a2cf2, 33 },
    { 0x47a2cf6, 95 },
    { 0, 0 },
    { 0x613a2cf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ea2cfb, 100 },
    { 0xd17a2cfc, 66 },
    { 0xa5aa2cfc, 61 },
    { 0xfb2a2cfd, 95 },
    { 0x6e4a2cfc, 33 },
    { 0x577a2cfe, 90 },
    { 0x6faa2d01, 61 },
    { 0xb67a2cfe, 55 },
    { 0xec0a2d03, 55 },
    { 0x20fa2d03, 95 },
    { 0xeeda2d03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ca2d0f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4a2d12, 67 },
    { 0x682a2d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6a2d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a2d21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8a2d24, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7da2d2a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652a2d31, 55 },
    { 0x372a2d32, 24 },
    { 0x38fa2d33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x922a2d36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a2d3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7a2d41, 90 },
    { 0x637a2d42, 98 },
    { 0x5a5a2d41, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44a2d4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6a2d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5a2d56, 98 },
    { 0x609a2d57, 99 },
    { 0x568a2d58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba2d5c, 90 },
    { 0xdc1a2d5d, 80 },
    { 0xe34a2d5e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9a2d63, 98 },
    { 0x4eba2d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243a2d6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccea2d72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251a2d77, 33 },
    { 0, 0 },
    { 0xf42a2d79, 95 },
    { 0, 0 },
    { 0xb88a2d7b, 77 },
    { 0x3bfa2d7c, 46 },
    { 0x72da2d7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ca2d91, 98 },
    { 0xd45a2d92, 84 },
    { 0x2efa2d92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9a2d9f, 61 },
    { 0, 0 },
    { 0x20aa2da1, 95 },
    { 0x134a2da1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8a2da5, 99 },
    { 0x46ea2da6, 33 },
    { 0x6d1a2da7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413a2dab, 17 },
    { 0xa06a2dab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fa2db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ea2db3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68a2db7, 90 },
    { 0xa6ba2db8, 90 },
    { 0x3c0a2db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17a2dbf, 90 },
    { 0x98fa2dc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84a2dc7, 52 },
    { 0, 0 },
    { 0xa3da2dc9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ca2dcc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaa2dcf, 33 },
    { 0, 0 },
    { 0x3c0a2dd1, 98 },
    { 0xc06a2dd2, 95 },
    { 0x52ea2dd1, 33 },
    { 0x333a2dd4, 78 },
    { 0, 0 },
    { 0xbf8a2dd6, 17 },
    { 0x556a2dd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677a2ddf, 76 },
    { 0x8cda2de0, 61 },
    { 0x2eca2ddf, 96 },
    { 0, 0 },
    { 0xb65a2de3, 80 },
    { 0xc10a2de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0a2dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba2df1, 61 },
    { 0x9a3a2df1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3a2df5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea2df8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a2dff, 33 },
    { 0x10ca2e00, 23 },
    { 0x899a2e01, 84 },
    { 0x8e5a2e02, 43 },
    { 0x990a2e01, 23 },
    { 0x41a2e04, 90 },
    { 0, 0 },
    { 0xe17a2e06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a2e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48da2e14, 90 },
    { 0, 0 },
    { 0xde9a2e16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705a2e1b, 65 },
    { 0xb6ea2e1b, 33 },
    { 0xd0ba2e1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a2e22, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1a2e25, 90 },
    { 0xfc8a2e26, 90 },
    { 0, 0 },
    { 0x824a2e28, 76 },
    { 0xacea2e29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0a2e2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fa2e31, 66 },
    { 0xe97a2e32, 78 },
    { 0, 0 },
    { 0xd2fa2e34, 43 },
    { 0, 0 },
    { 0x3cda2e36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4a2e3a, 88 },
    { 0x58aa2e3a, 64 },
    { 0x36fa2e3c, 100 },
    { 0xa76a2e3d, 76 },
    { 0x5b8a2e3b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8a2e42, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782a2e48, 95 },
    { 0xd36a2e49, 33 },
    { 0xe91a2e48, 61 },
    { 0x41aa2e48, 95 },
    { 0xb83a2e4b, 93 },
    { 0x2a7a2e4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0a2e52, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220a2e5d, 55 },
    { 0x5aca2e5e, 52 },
    { 0x960a2e5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03a2e67, 95 },
    { 0x98a2e68, 23 },
    { 0xf19a2e68, 33 },
    { 0x6f5a2e6a, 33 },
    { 0, 0 },
    { 0xbfaa2e6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75a2e6f, 84 },
    { 0, 0 },
    { 0x751a2e71, 23 },
    { 0xa78a2e71, 61 },
    { 0, 0 },
    { 0x5dea2e74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95a2e79, 98 },
    { 0xe66a2e7a, 81 },
    { 0xe81a2e7a, 33 },
    { 0x50fa2e7a, 46 },
    { 0xd3fa2e7d, 98 },
    { 0x333a2e7e, 33 },
    { 0xfaba2e7f, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a2e83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a2e8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fa2e8d, 90 },
    { 0x155a2e8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9a2e92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1a2e99, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fa2e9e, 84 },
    { 0xebaa2e9e, 33 },
    { 0, 0 },
    { 0x506a2ea1, 95 },
    { 0, 0 },
    { 0xea0a2ea3, 55 },
    { 0, 0 },
    { 0x827a2ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a2ea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66a2eaf, 43 },
    { 0, 0 },
    { 0x537a2eb1, 61 },
    { 0, 0 },
    { 0x1fca2eb3, 100 },
    { 0x5d9a2eb4, 55 },
    { 0x926a2eb5, 78 },
    { 0x80ca2eb6, 33 },
    { 0x475a2eb6, 90 },
    { 0xb9ea2eb8, 23 },
    { 0, 0 },
    { 0x82ea2eba, 90 },
    { 0x81fa2eba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a2ec0, 69 },
    { 0x25ba2ec1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133a2ec7, 52 },
    { 0xf7aa2ec8, 76 },
    { 0, 0 },
    { 0x62fa2eca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2a2ed3, 23 },
    { 0, 0 },
    { 0xa7fa2ed5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6a2eda, 52 },
    { 0x750a2edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666a2ee0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ba2ee3, 95 },
    { 0, 0 },
    { 0xdf0a2ee5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ba2ee9, 23 },
    { 0x497a2eea, 91 },
    { 0xe8ba2eea, 95 },
    { 0xcdea2eec, 55 },
    { 0xbd3a2ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a2ef4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x176a2ef7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78a2efa, 67 },
    { 0, 0 },
    { 0x94ca2efc, 85 },
    { 0xa22a2efd, 34 },
    { 0xeeda2efd, 61 },
    { 0x520a2eff, 33 },
    { 0xb98a2f00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18a2f04, 43 },
    { 0xf95a2f05, 95 },
    { 0x774a2f06, 33 },
    { 0x6c3a2f07, 66 },
    { 0xf87a2f08, 82 },
    { 0xbbaa2f09, 90 },
    { 0x173a2f0a, 66 },
    { 0xf3ba2f08, 33 },
    { 0, 0 },
    { 0x102a2f0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a2f13, 33 },
    { 0x72ca2f14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139a2f1a, 99 },
    { 0, 0 },
    { 0xe2ba2f1c, 33 },
    { 0xb7aa2f1c, 61 },
    { 0xeea2f1c, 23 },
    { 0xa4a2f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa2f24, 45 },
    { 0xce6a2f25, 52 },
    { 0xa4da2f24, 23 },
    { 0x3c8a2f27, 87 },
    { 0xfd0a2f28, 90 },
    { 0x989a2f28, 33 },
    { 0xdba2f28, 90 },
    { 0, 0 },
    { 0xe4da2f2c, 67 },
    { 0x685a2f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54a2f30, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72a2f38, 77 },
    { 0x977a2f39, 90 },
    { 0, 0 },
    { 0xa41a2f3b, 76 },
    { 0xe19a2f3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271a2f43, 46 },
    { 0x43ca2f43, 98 },
    { 0xfe6a2f45, 98 },
    { 0, 0 },
    { 0x928a2f47, 61 },
    { 0xbb6a2f48, 98 },
    { 0, 0 },
    { 0x929a2f4a, 90 },
    { 0, 0 },
    { 0x34fa2f4c, 93 },
    { 0xf39a2f4d, 98 },
    { 0xe34a2f4e, 33 },
    { 0xb3fa2f4f, 95 },
    { 0xd8ba2f4e, 34 },
    { 0xd9ea2f51, 55 },
    { 0, 0 },
    { 0xa9a2f53, 61 },
    { 0xf6aa2f54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960a2f59, 49 },
    { 0xd8ca2f59, 61 },
    { 0x40fa2f5b, 95 },
    { 0x2c9a2f5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3a2f63, 33 },
    { 0x4a8a2f64, 61 },
    { 0x538a2f65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8a2f6a, 43 },
    { 0, 0 },
    { 0x273a2f6c, 23 },
    { 0, 0 },
    { 0x41ba2f6e, 65 },
    { 0, 0 },
    { 0x8c4a2f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ba2f77, 17 },
    { 0xaa3a2f77, 66 },
    { 0, 0 },
    { 0x899a2f7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8a2f7d, 95 },
    { 0, 0 },
    { 0x264a2f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a2f89, 87 },
    { 0xbcba2f8a, 61 },
    { 0, 0 },
    { 0xbb3a2f8c, 23 },
    { 0, 0 },
    { 0x761a2f8e, 61 },
    { 0xb09a2f8f, 90 },
    { 0x841a2f8f, 52 },
    { 0xe06a2f91, 76 },
    { 0xf0ea2f8f, 98 },
    { 0x9dea2f8e, 78 },
    { 0x9c4a2f94, 98 },
    { 0x12ba2f95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a2f9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127a2f9f, 17 },
    { 0, 0 },
    { 0xbeba2fa1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761a2fa8, 44 },
    { 0, 0 },
    { 0x63a2faa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6a2fae, 34 },
    { 0x7a4a2faf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76da2fb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3a2fb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9a2fc3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81a2fc7, 40 },
    { 0x95ca2fc8, 34 },
    { 0x88aa2fc9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a2fcd, 90 },
    { 0x8dfa2fce, 90 },
    { 0x533a2fcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fa2fd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670a2fdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9daa2fe1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca2fe4, 61 },
    { 0xd20a2fe5, 90 },
    { 0xa8ea2fe6, 90 },
    { 0x5e3a2fe5, 90 },
    { 0xbd0a2fe6, 84 },
    { 0, 0 },
    { 0xa21a2fea, 33 },
    { 0xd89a2fea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa2ff0, 61 },
    { 0x9c1a2ff0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a2ff4, 55 },
    { 0x3aa2ff4, 23 },
    { 0xc7aa2ff5, 98 },
    { 0, 0 },
    { 0x9e0a2ff8, 34 },
    { 0, 0 },
    { 0xda6a2ffa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30a2ffe, 91 },
    { 0xd53a2fff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ea3003, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ca300a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97a300d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6a3012, 55 },
    { 0xb9ea3013, 33 },
    { 0x8fa3013, 98 },
    { 0, 0 },
    { 0xe79a3016, 90 },
    { 0x5f4a3017, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfa301d, 23 },
    { 0xaba301e, 46 },
    { 0xc0a301f, 90 },
    { 0xaca3020, 34 },
    { 0x2eba3021, 90 },
    { 0, 0 },
    { 0xaa6a3023, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28a3030, 98 },
    { 0, 0 },
    { 0x59da3032, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da303b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15a3044, 92 },
    { 0x6b9a3045, 98 },
    { 0x927a3044, 74 },
    { 0, 0 },
    { 0x3b2a3048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09a304d, 90 },
    { 0x3b7a304d, 98 },
    { 0xca2a304e, 85 },
    { 0x9a2a304f, 90 },
    { 0x191a3051, 23 },
    { 0x50ea3052, 34 },
    { 0, 0 },
    { 0x876a3054, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75a3059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26a305f, 33 },
    { 0, 0 },
    { 0xb2ea3061, 90 },
    { 0x20ba3062, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799a306d, 34 },
    { 0x721a306e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afa3071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6a307c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5a3081, 93 },
    { 0x71da3082, 33 },
    { 0x5a1a3083, 46 },
    { 0xca9a3082, 84 },
    { 0x32a3085, 33 },
    { 0x51ea3083, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7a3089, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bea308f, 77 },
    { 0x9a1a308f, 87 },
    { 0xf40a3090, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba3095, 33 },
    { 0x822a3095, 49 },
    { 0x125a3097, 61 },
    { 0x106a3097, 90 },
    { 0xeda3096, 90 },
    { 0xc39a309a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527a309e, 90 },
    { 0x81ea309f, 90 },
    { 0x770a30a0, 55 },
    { 0xe1fa30a1, 55 },
    { 0x19ba30a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafa30a8, 43 },
    { 0, 0 },
    { 0xf46a30aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746a30ae, 95 },
    { 0xd83a30af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aa30b2, 33 },
    { 0, 0 },
    { 0xc50a30b4, 90 },
    { 0x4b4a30b4, 85 },
    { 0x6fba30b6, 55 },
    { 0x954a30b5, 77 },
    { 0x8da30b4, 43 },
    { 0x823a30b8, 61 },
    { 0x5ba30b5, 93 },
    { 0x4cea30bb, 34 },
    { 0, 0 },
    { 0x124a30bd, 76 },
    { 0x1a6a30bd, 95 },
    { 0x33da30bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x501a30c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa30c5, 34 },
    { 0, 0 },
    { 0x90ea30c7, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625a30cd, 76 },
    { 0x688a30cd, 78 },
    { 0x690a30cf, 91 },
    { 0x9d5a30ce, 77 },
    { 0xa20a30d1, 19 },
    { 0x2d3a30d2, 23 },
    { 0x3e5a30cd, 91 },
    { 0xd38a30d4, 67 },
    { 0xcb1a30d4, 66 },
    { 0xdfaa30cd, 98 },
    { 0xd43a30d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482a30db, 34 },
    { 0x39aa30db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5a30e1, 91 },
    { 0, 0 },
    { 0xbdda30e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3a30ea, 76 },
    { 0xbfa30ea, 23 },
    { 0xfe0a30eb, 23 },
    { 0, 0 },
    { 0xc12a30ee, 90 },
    { 0x591a30ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da30f2, 61 },
    { 0x5e8a30f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78a30f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aa30fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169a3104, 52 },
    { 0, 0 },
    { 0x10a3106, 93 },
    { 0xa7a3107, 90 },
    { 0x4e1a3108, 84 },
    { 0x13fa3108, 95 },
    { 0, 0 },
    { 0xb70a310b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ca310e, 95 },
    { 0, 0 },
    { 0x16aa3110, 98 },
    { 0, 0 },
    { 0xfdda3112, 95 },
    { 0x99ea3112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690a3117, 99 },
    { 0xb40a3118, 61 },
    { 0x6d1a3119, 90 },
    { 0x7bfa311a, 90 },
    { 0x2b4a3118, 76 },
    { 0x6a2a311c, 33 },
    { 0x91aa3118, 33 },
    { 0, 0 },
    { 0x711a311f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72a312a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeca312e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba3132, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11a3137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76a313a, 2 },
    { 0xf5aa313a, 23 },
    { 0, 0 },
    { 0xe42a313d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515a3141, 67 },
    { 0xaaa3141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455a3146, 43 },
    { 0x4dea3147, 98 },
    { 0x7d0a3148, 68 },
    { 0, 0 },
    { 0x2e5a314a, 66 },
    { 0, 0 },
    { 0x387a314c, 34 },
    { 0, 0 },
    { 0xd95a314e, 91 },
    { 0x3fca314f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ca3158, 76 },
    { 0x62ca3159, 100 },
    { 0x62ea315a, 52 },
    { 0, 0 },
    { 0x3cfa315c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ba315f, 90 },
    { 0x638a315f, 90 },
    { 0, 0 },
    { 0xe49a3162, 76 },
    { 0x278a3163, 23 },
    { 0xefaa3163, 61 },
    { 0, 0 },
    { 0xe49a3166, 55 },
    { 0x18fa3167, 81 },
    { 0x8dea3168, 78 },
    { 0, 0 },
    { 0x58ba316a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ba316d, 23 },
    { 0, 0 },
    { 0x321a316f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291a3177, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645a317d, 90 },
    { 0, 0 },
    { 0x9eaa317f, 33 },
    { 0x6aa3180, 95 },
    { 0x33ba3181, 33 },
    { 0x196a3182, 90 },
    { 0xda4a3182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eea3188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1a318b, 46 },
    { 0x7e7a318c, 61 },
    { 0xf6fa318c, 44 },
    { 0, 0 },
    { 0x874a318f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a3192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716a3199, 95 },
    { 0x3c3a3199, 61 },
    { 0, 0 },
    { 0x59da319c, 76 },
    { 0x74ea319c, 33 },
    { 0, 0 },
    { 0x25ba319f, 84 },
    { 0, 0 },
    { 0xfe2a31a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaca31aa, 98 },
    { 0xc7a31ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca31af, 91 },
    { 0xf61a31b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x848a31b3, 95 },
    { 0xb0da31b4, 55 },
    { 0, 0 },
    { 0xbaba31b6, 95 },
    { 0x1f4a31b7, 98 },
    { 0x1aa31b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cea31bc, 90 },
    { 0x682a31bd, 98 },
    { 0, 0 },
    { 0xa60a31bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0a31c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a31c6, 46 },
    { 0xa50a31c7, 75 },
    { 0xf8fa31c8, 85 },
    { 0, 0 },
    { 0x9b8a31ca, 33 },
    { 0x7e3a31cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738a31d4, 23 },
    { 0x6da31d5, 61 },
    { 0xe79a31d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddda31d9, 43 },
    { 0xbb5a31da, 91 },
    { 0, 0 },
    { 0x923a31dc, 33 },
    { 0, 0 },
    { 0xf5ba31de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa31e1, 33 },
    { 0xf34a31e2, 55 },
    { 0xf6ea31e2, 43 },
    { 0, 0 },
    { 0x2b7a31e5, 99 },
    { 0xacda31e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562a31eb, 90 },
    { 0x218a31eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a31f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada31fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9a31ff, 98 },
    { 0xe60a3200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aa3207, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6a320c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ca320f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36a3215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0a321d, 66 },
    { 0xf5ca321e, 55 },
    { 0x125a321f, 90 },
    { 0xfb5a321f, 100 },
    { 0x8c4a3221, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fa322a, 90 },
    { 0, 0 },
    { 0x236a322c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59a3231, 93 },
    { 0x4ba3232, 55 },
    { 0xdf6a3231, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aca3237, 88 },
    { 0xea7a3237, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90a323c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ba3240, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4a3243, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89a3246, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a324b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4a3250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fa3257, 90 },
    { 0xbaca3257, 93 },
    { 0xa6ca3258, 95 },
    { 0xe1fa325a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfba3261, 75 },
    { 0x3cfa3261, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4a326a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6a326f, 34 },
    { 0x304a3270, 61 },
    { 0x71aa3270, 95 },
    { 0x2e3a3272, 61 },
    { 0xd19a3272, 95 },
    { 0x382a3274, 90 },
    { 0xadca3274, 55 },
    { 0x30ba3276, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeea3279, 99 },
    { 0, 0 },
    { 0x886a327b, 76 },
    { 0x446a327b, 98 },
    { 0xd7ca327b, 78 },
    { 0xf86a327d, 93 },
    { 0x244a327d, 95 },
    { 0xad2a327c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99a3285, 76 },
    { 0xd9ea3286, 95 },
    { 0, 0 },
    { 0xdcfa3288, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ea328b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803a328f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xadca3292, 34 },
    { 0x8e9a3292, 66 },
    { 0xd66a3294, 61 },
    { 0, 0 },
    { 0xd4da3296, 95 },
    { 0, 0 },
    { 0x103a3298, 90 },
    { 0x5b1a3299, 78 },
    { 0xeda329a, 66 },
    { 0x277a3299, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314a329f, 34 },
    { 0x923a32a0, 43 },
    { 0x4bfa32a1, 85 },
    { 0xb18a32a0, 33 },
    { 0xf9ea32a0, 34 },
    { 0x5e8a32a4, 76 },
    { 0x9ea32a5, 8 },
    { 0x4a4a32a3, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdca32ac, 55 },
    { 0, 0 },
    { 0xccea32ae, 23 },
    { 0xbbba32af, 23 },
    { 0x9ea32ae, 95 },
    { 0, 0 },
    { 0x9b7a32b2, 90 },
    { 0xff6a32b3, 90 },
    { 0xe94a32b4, 66 },
    { 0xfbda32b2, 23 },
    { 0, 0 },
    { 0xb5aa32b7, 67 },
    { 0x842a32b7, 98 },
    { 0x291a32b9, 90 },
    { 0xf64a32b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6a32bd, 46 },
    { 0x56ca32be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aa32c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea32c7, 98 },
    { 0xdeda32c8, 90 },
    { 0xaada32c9, 95 },
    { 0, 0 },
    { 0x361a32cb, 68 },
    { 0, 0 },
    { 0x71ca32cd, 87 },
    { 0, 0 },
    { 0x208a32cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977a32d6, 33 },
    { 0xa7fa32d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633a32de, 46 },
    { 0x9d9a32de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaa32e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4a32e5, 24 },
    { 0xa1ca32e6, 61 },
    { 0, 0 },
    { 0x418a32e8, 75 },
    { 0, 0 },
    { 0x2eca32ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74a32f3, 95 },
    { 0, 0 },
    { 0x32ba32f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328a32f9, 36 },
    { 0xd7ba32fa, 76 },
    { 0xa82a32fb, 55 },
    { 0x1a3a32fc, 61 },
    { 0, 0 },
    { 0x827a32fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a3301, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ca3306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ca3309, 23 },
    { 0, 0 },
    { 0xbdfa330b, 90 },
    { 0x4f1a330c, 98 },
    { 0xe1ba330c, 43 },
    { 0xb0ea330e, 84 },
    { 0x512a330f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cea3313, 17 },
    { 0, 0 },
    { 0x8b0a3315, 98 },
    { 0, 0 },
    { 0xb18a3317, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a331d, 61 },
    { 0x8ba331e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ca3324, 87 },
    { 0x2faa3324, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10a3328, 43 },
    { 0, 0 },
    { 0x343a332a, 84 },
    { 0x59ca332b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4a3330, 95 },
    { 0x550a3331, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faa3343, 61 },
    { 0, 0 },
    { 0x8c9a3345, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9a3349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825a334d, 61 },
    { 0xd7fa334e, 69 },
    { 0x798a334e, 87 },
    { 0xcf1a3350, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ea3353, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987a3358, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634a335c, 52 },
    { 0xb39a335c, 85 },
    { 0xfb6a335c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcea3361, 23 },
    { 0xe24a3361, 90 },
    { 0x394a3362, 17 },
    { 0xb37a3364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a3369, 85 },
    { 0, 0 },
    { 0xb58a336b, 33 },
    { 0x172a336b, 33 },
    { 0x922a336c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2a3374, 98 },
    { 0x371a3375, 52 },
    { 0xc0da3376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521a3380, 90 },
    { 0, 0 },
    { 0x9d6a3382, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3a3385, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dba338b, 95 },
    { 0x1bda338c, 98 },
    { 0x296a338d, 55 },
    { 0, 0 },
    { 0x3a9a338f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267a3393, 76 },
    { 0x74ca3394, 61 },
    { 0, 0 },
    { 0x77ea3396, 65 },
    { 0x9c8a3397, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecba33a3, 86 },
    { 0, 0 },
    { 0x93ca33a5, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7a33ab, 66 },
    { 0xbca33ab, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4a33af, 95 },
    { 0xc98a33af, 53 },
    { 0xc1a33b1, 33 },
    { 0, 0 },
    { 0xf2aa33b3, 33 },
    { 0x742a33b4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ba33b7, 78 },
    { 0x506a33b8, 66 },
    { 0, 0 },
    { 0x47a33ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20a33c4, 90 },
    { 0x1bba33c5, 23 },
    { 0x89ba33c6, 90 },
    { 0xeb7a33c5, 88 },
    { 0xb49a33c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2a33cd, 76 },
    { 0xc95a33cd, 90 },
    { 0xc07a33cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ada33d5, 90 },
    { 0x413a33d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0a33d9, 90 },
    { 0, 0 },
    { 0xd54a33db, 90 },
    { 0x405a33dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ba33e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32a33e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ea33e7, 91 },
    { 0x6cca33e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34da33eb, 98 },
    { 0xe2aa33eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0a33f2, 23 },
    { 0x150a33f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da33f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fba33fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ba3402, 59 },
    { 0xc69a3403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bda3406, 52 },
    { 0x3faa3406, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x91a340a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x999a340d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ba3411, 90 },
    { 0x55ea3412, 90 },
    { 0, 0 },
    { 0xe6ba3414, 23 },
    { 0x32da3414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a3418, 91 },
    { 0, 0 },
    { 0x638a341a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8a3423, 93 },
    { 0x687a3423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ba3429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05a3431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846a3436, 48 },
    { 0x47ba3436, 90 },
    { 0, 0 },
    { 0x605a3439, 90 },
    { 0, 0 },
    { 0x306a343b, 61 },
    { 0xa36a343b, 76 },
    { 0x97ea343d, 98 },
    { 0x5dca343e, 90 },
    { 0xb43a343f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x278a3442, 90 },
    { 0x605a3443, 52 },
    { 0, 0 },
    { 0xacea3445, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221a344f, 34 },
    { 0, 0 },
    { 0x98ea3451, 78 },
    { 0x7c0a3452, 33 },
    { 0, 0 },
    { 0xf23a3454, 43 },
    { 0, 0 },
    { 0x5aba3456, 93 },
    { 0, 0 },
    { 0x18aa3458, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122a345c, 98 },
    { 0, 0 },
    { 0x237a345e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ea3462, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627a3468, 33 },
    { 0x63ea3468, 98 },
    { 0xbf3a346a, 95 },
    { 0xd13a3469, 43 },
    { 0xc68a346c, 22 },
    { 0x3f0a346d, 98 },
    { 0x3a2a346e, 66 },
    { 0x5f3a346f, 87 },
    { 0, 0 },
    { 0x4c7a3471, 90 },
    { 0, 0 },
    { 0xc62a3473, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa3479, 55 },
    { 0, 0 },
    { 0xefea347b, 43 },
    { 0, 0 },
    { 0x84da347d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614a3482, 61 },
    { 0x1fea3483, 81 },
    { 0xb58a3483, 87 },
    { 0xbaea3484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42a348c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaba348f, 33 },
    { 0, 0 },
    { 0x803a3491, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8a3494, 61 },
    { 0, 0 },
    { 0x2fea3496, 64 },
    { 0, 0 },
    { 0x520a3498, 55 },
    { 0, 0 },
    { 0xb2ca349a, 95 },
    { 0x2e5a349b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592a34a1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ca34a9, 76 },
    { 0xe2fa34aa, 61 },
    { 0x6a4a34aa, 90 },
    { 0x691a34aa, 90 },
    { 0, 0 },
    { 0x630a34ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa34b1, 95 },
    { 0, 0 },
    { 0xf18a34b3, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a34bb, 55 },
    { 0x905a34bb, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81a34bf, 90 },
    { 0, 0 },
    { 0x51ea34c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba34c4, 76 },
    { 0xabfa34c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46a34cf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53a34d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ea34d9, 33 },
    { 0, 0 },
    { 0xe9ca34db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98a34de, 61 },
    { 0, 0 },
    { 0x951a34e0, 58 },
    { 0x828a34e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a34e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6a34e9, 55 },
    { 0, 0 },
    { 0xfcba34eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfa34f5, 33 },
    { 0, 0 },
    { 0xf4ba34f7, 33 },
    { 0x23a34f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5a34fc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16a3502, 33 },
    { 0x809a3502, 95 },
    { 0, 0 },
    { 0x1aa3505, 61 },
    { 0x790a3506, 75 },
    { 0xc2aa3507, 76 },
    { 0xc6ba3508, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ea350b, 34 },
    { 0x636a350b, 34 },
    { 0xea5a350d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3a3511, 33 },
    { 0xc65a3512, 23 },
    { 0, 0 },
    { 0x682a3514, 78 },
    { 0, 0 },
    { 0xc13a3516, 52 },
    { 0xf92a3517, 93 },
    { 0, 0 },
    { 0xde6a3519, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63aa3520, 95 },
    { 0xf76a3521, 97 },
    { 0x29ea3522, 23 },
    { 0, 0 },
    { 0x9aba3524, 98 },
    { 0x80ba3525, 34 },
    { 0, 0 },
    { 0x125a3527, 98 },
    { 0, 0 },
    { 0xd08a3529, 33 },
    { 0x726a3529, 55 },
    { 0x38da3529, 99 },
    { 0x350a352c, 81 },
    { 0, 0 },
    { 0xdc0a352e, 46 },
    { 0, 0 },
    { 0xfffa3530, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0a3535, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eda3542, 90 },
    { 0x2f1a3543, 76 },
    { 0x451a3544, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcca3547, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da354b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16a354e, 61 },
    { 0x9e1a354e, 90 },
    { 0x602a3550, 43 },
    { 0x263a3551, 98 },
    { 0xc3a3551, 85 },
    { 0x164a3552, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353a3557, 90 },
    { 0x35fa3557, 46 },
    { 0x131a3559, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6a355c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509a3562, 33 },
    { 0, 0 },
    { 0xfbfa3564, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92a356c, 91 },
    { 0xc14a356d, 23 },
    { 0xceda356e, 61 },
    { 0, 0 },
    { 0x24da3570, 34 },
    { 0x92da3571, 46 },
    { 0x683a3572, 33 },
    { 0x321a3573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6a3578, 43 },
    { 0x7eea3579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47a357c, 98 },
    { 0, 0 },
    { 0xdd5a357e, 90 },
    { 0, 0 },
    { 0x35ea3580, 66 },
    { 0x90aa3581, 23 },
    { 0x3fa3580, 99 },
    { 0x8f8a3581, 23 },
    { 0x82a3581, 99 },
    { 0x60ea3581, 46 },
    { 0x359a3586, 23 },
    { 0x428a3587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43a358e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x57da3591, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x71aa3594, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a3597, 34 },
    { 0x766a3598, 90 },
    { 0, 0 },
    { 0x880a359a, 93 },
    { 0xf00a359b, 33 },
    { 0xc55a359c, 61 },
    { 0x92ba359d, 76 },
    { 0x225a359d, 61 },
    { 0x8b7a359f, 83 },
    { 0, 0 },
    { 0x669a35a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x33da35a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4a35ab, 93 },
    { 0x23ba35ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1a35b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462a35b8, 91 },
    { 0xa99a35b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ba35bd, 93 },
    { 0xd71a35be, 88 },
    { 0x332a35be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aca35c6, 23 },
    { 0, 0 },
    { 0x9ea35c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5a35cc, 34 },
    { 0x53fa35cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2a35d2, 95 },
    { 0xbaa35d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9a35da, 95 },
    { 0, 0 },
    { 0x7bfa35dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3aa35e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca35e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x174a35eb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69a35f0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaba35f8, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x75aa35fb, 76 },
    { 0x754a35fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a35ff, 61 },
    { 0x4f0a3600, 55 },
    { 0x6c4a3601, 36 },
    { 0, 0 },
    { 0xf45a3603, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a3608, 90 },
    { 0x4eea3609, 95 },
    { 0xeaea360a, 33 },
    { 0xa58a360a, 91 },
    { 0xcf9a360c, 85 },
    { 0x958a360a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddea3610, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62a3614, 23 },
    { 0x89fa3615, 34 },
    { 0xd03a3616, 43 },
    { 0xc77a3616, 85 },
    { 0x321a3614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4a361e, 95 },
    { 0, 0 },
    { 0xbbba3620, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fa3623, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8a3626, 46 },
    { 0x625a3627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85da362d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59a3630, 33 },
    { 0x225a3631, 43 },
    { 0xd75a3631, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ea3638, 23 },
    { 0, 0 },
    { 0xb0a363a, 76 },
    { 0x367a363b, 90 },
    { 0xb52a363c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16a3642, 68 },
    { 0xfc6a3643, 76 },
    { 0xd85a3642, 90 },
    { 0x425a3644, 99 },
    { 0x499a3646, 61 },
    { 0x83ca3643, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a364c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1a3650, 55 },
    { 0x3b5a3650, 43 },
    { 0xef5a3652, 76 },
    { 0, 0 },
    { 0xa7aa3654, 55 },
    { 0, 0 },
    { 0x99aa3656, 78 },
    { 0x7c0a3657, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8a365a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6a3661, 75 },
    { 0xfb8a3662, 34 },
    { 0x83a3663, 46 },
    { 0xafa3662, 100 },
    { 0, 0 },
    { 0x360a3666, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a3669, 94 },
    { 0, 0 },
    { 0x945a366b, 98 },
    { 0x688a366c, 98 },
    { 0xa9ea366c, 61 },
    { 0, 0 },
    { 0xb23a366f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ea3679, 23 },
    { 0xca0a3679, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06a367e, 94 },
    { 0, 0 },
    { 0x240a3680, 55 },
    { 0, 0 },
    { 0x616a3682, 33 },
    { 0xd45a3683, 76 },
    { 0xc92a3684, 59 },
    { 0x835a3682, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43a368d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fea3691, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aba3696, 33 },
    { 0xb9ba3696, 61 },
    { 0x833a3697, 77 },
    { 0x991a3699, 23 },
    { 0x3dfa369a, 61 },
    { 0x103a369a, 76 },
    { 0, 0 },
    { 0xebba369d, 95 },
    { 0x6b0a369d, 61 },
    { 0x99fa369f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ca36a3, 90 },
    { 0xccba36a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a36a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a36aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248a36ae, 64 },
    { 0, 0 },
    { 0x4cea36b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fa36b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fa36bb, 41 },
    { 0, 0 },
    { 0xa2ca36bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0a36c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7a36c3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efa36c6, 33 },
    { 0x3c5a36c7, 61 },
    { 0x5b7a36c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaa36cc, 34 },
    { 0x708a36cd, 93 },
    { 0x67da36cd, 90 },
    { 0x5a4a36cf, 90 },
    { 0x2ffa36d0, 66 },
    { 0x43a36d1, 95 },
    { 0xce5a36d0, 34 },
    { 0xd92a36d3, 90 },
    { 0x22ba36d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ea36d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aa36dc, 23 },
    { 0xb6ca36dc, 43 },
    { 0xda9a36de, 43 },
    { 0x6a3a36df, 61 },
    { 0xb14a36e0, 95 },
    { 0xcf1a36dd, 90 },
    { 0x2aea36de, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ba36e8, 98 },
    { 0x7daa36e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a36ec, 55 },
    { 0x5a0a36ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497a36f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aa36fb, 90 },
    { 0xde5a36fc, 52 },
    { 0x3d0a36fd, 61 },
    { 0x6a2a36fd, 90 },
    { 0x993a36ff, 68 },
    { 0xee9a36fd, 98 },
    { 0xe33a3701, 52 },
    { 0, 0 },
    { 0xe21a3703, 55 },
    { 0x9f3a3704, 52 },
    { 0x1cda3703, 95 },
    { 0x7b8a3706, 36 },
    { 0x9f9a3706, 90 },
    { 0x17da3708, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca370b, 95 },
    { 0x42aa370b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6a3711, 61 },
    { 0x320a3711, 33 },
    { 0, 0 },
    { 0x31da3714, 49 },
    { 0x40da3715, 99 },
    { 0xba5a3716, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a371b, 33 },
    { 0, 0 },
    { 0x3b3a371d, 76 },
    { 0xff8a371e, 49 },
    { 0xdfaa371f, 43 },
    { 0x5a4a371d, 55 },
    { 0xc44a371f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8a3727, 95 },
    { 0x8a9a3728, 90 },
    { 0xdcba3729, 55 },
    { 0x48a3727, 81 },
    { 0xbd3a372b, 95 },
    { 0xe20a3729, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ba3732, 17 },
    { 0x5eca3732, 85 },
    { 0, 0 },
    { 0x96a3735, 40 },
    { 0xff6a3736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x355a3739, 33 },
    { 0x364a373a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ea373f, 98 },
    { 0x970a3740, 95 },
    { 0x2b3a3741, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ba3748, 34 },
    { 0x843a3749, 34 },
    { 0x934a3748, 55 },
    { 0, 0 },
    { 0x9cda374c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25a374f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06a3752, 90 },
    { 0x685a3753, 33 },
    { 0x483a3752, 95 },
    { 0x257a3755, 23 },
    { 0, 0 },
    { 0x85ba3757, 91 },
    { 0xc9aa3757, 88 },
    { 0, 0 },
    { 0xe6aa375a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996a375f, 96 },
    { 0x397a375f, 91 },
    { 0xf8ca3760, 43 },
    { 0x7faa3760, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da3765, 61 },
    { 0x9aba3766, 98 },
    { 0x6a9a3765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196a376b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x480a376e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdba3771, 61 },
    { 0x8a2a3772, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01a3777, 95 },
    { 0, 0 },
    { 0xce4a3779, 95 },
    { 0xbc4a3779, 66 },
    { 0xfbaa3779, 61 },
    { 0x2e4a377c, 95 },
    { 0x86ea377d, 91 },
    { 0x553a377e, 66 },
    { 0x841a377d, 68 },
    { 0x8fda3780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aa3783, 43 },
    { 0xfcaa3784, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52a3787, 91 },
    { 0, 0 },
    { 0x56da3789, 23 },
    { 0xb6ba378a, 46 },
    { 0, 0 },
    { 0xbf3a378c, 90 },
    { 0x2d8a378c, 33 },
    { 0, 0 },
    { 0x180a378f, 90 },
    { 0xeda3790, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30a3793, 33 },
    { 0xd25a3793, 89 },
    { 0xe52a3795, 90 },
    { 0xb28a3796, 36 },
    { 0x7fba3797, 23 },
    { 0xaaba3798, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27a379b, 55 },
    { 0xd7ea379c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79a379f, 61 },
    { 0x4b2a37a0, 43 },
    { 0x8a8a379f, 95 },
    { 0x56ba37a2, 95 },
    { 0, 0 },
    { 0x260a37a4, 75 },
    { 0xbe7a37a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2a37a8, 17 },
    { 0x751a37a9, 90 },
    { 0, 0 },
    { 0x151a37ab, 55 },
    { 0, 0 },
    { 0x3a1a37ad, 34 },
    { 0xec8a37ae, 90 },
    { 0x8e3a37af, 61 },
    { 0xb79a37b0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x352a37b3, 98 },
    { 0x21fa37b4, 61 },
    { 0xed0a37b5, 78 },
    { 0x72ea37b6, 90 },
    { 0x6d4a37b7, 61 },
    { 0x355a37b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650a37bf, 23 },
    { 0x1ada37c0, 76 },
    { 0xc85a37bf, 43 },
    { 0x792a37c2, 94 },
    { 0xa3aa37c3, 34 },
    { 0, 0 },
    { 0xe1a37c5, 46 },
    { 0x1cda37c5, 65 },
    { 0, 0 },
    { 0x67aa37c8, 98 },
    { 0, 0 },
    { 0x845a37ca, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09a37d4, 61 },
    { 0x982a37d5, 34 },
    { 0xb8da37d6, 90 },
    { 0xecfa37d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ba37db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753a37df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6a37e2, 76 },
    { 0x9bda37e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ca37e6, 100 },
    { 0xbfca37e7, 90 },
    { 0xf33a37e8, 34 },
    { 0xf61a37e9, 17 },
    { 0x6b9a37ea, 98 },
    { 0xa4a37e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252a37ef, 55 },
    { 0x935a37ef, 61 },
    { 0xb0aa37f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fa37f6, 91 },
    { 0x3dba37f7, 87 },
    { 0xcb2a37f8, 38 },
    { 0xf63a37f9, 90 },
    { 0x844a37fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363a37fe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39a3801, 76 },
    { 0xdfaa3802, 91 },
    { 0x80ba3801, 33 },
    { 0x851a3804, 90 },
    { 0x8f4a3805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eca3809, 100 },
    { 0, 0 },
    { 0x345a380b, 90 },
    { 0x502a380b, 93 },
    { 0xe6aa380c, 98 },
    { 0x9b0a380e, 66 },
    { 0xa94a380f, 95 },
    { 0x1caa3810, 88 },
    { 0x903a3810, 90 },
    { 0, 0 },
    { 0xb2ba3813, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84a3827, 33 },
    { 0x1eca3828, 94 },
    { 0x351a3828, 90 },
    { 0x51da382a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3da382d, 33 },
    { 0, 0 },
    { 0xd2ea382f, 90 },
    { 0x4c1a3830, 61 },
    { 0, 0 },
    { 0x99ba3832, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aa3838, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x10da383b, 83 },
    { 0, 0 },
    { 0xdd2a383d, 95 },
    { 0, 0 },
    { 0x7cda383f, 99 },
    { 0x652a383f, 92 },
    { 0xb2a3840, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x741a3844, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a384a, 66 },
    { 0xbbaa384b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ba385b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9a3864, 33 },
    { 0xedea3864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229a386d, 55 },
    { 0x76fa386d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa3872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49da3875, 90 },
    { 0xd3ba3876, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6da3883, 23 },
    { 0, 0 },
    { 0x39ea3885, 76 },
    { 0x992a3886, 95 },
    { 0, 0 },
    { 0xb96a3888, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefca388b, 93 },
    { 0, 0 },
    { 0x7f8a388d, 99 },
    { 0x200a388e, 64 },
    { 0xa0aa388f, 87 },
    { 0xeea388d, 78 },
    { 0x5e4a3891, 34 },
    { 0, 0 },
    { 0x444a3893, 98 },
    { 0xeada3893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaa389a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba389d, 76 },
    { 0x6e3a389e, 33 },
    { 0x8b6a389f, 34 },
    { 0x963a38a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ea38a7, 91 },
    { 0x820a38a7, 36 },
    { 0x8f2a38a9, 95 },
    { 0xa0da38a8, 38 },
    { 0x730a38a7, 77 },
    { 0, 0 },
    { 0x473a38ad, 33 },
    { 0x4fea38ae, 23 },
    { 0x560a38ae, 99 },
    { 0, 0 },
    { 0xbe2a38b1, 90 },
    { 0, 0 },
    { 0x78a38b3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59a38b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ca38bc, 98 },
    { 0xf19a38bc, 87 },
    { 0xddfa38be, 33 },
    { 0xc2da38bf, 61 },
    { 0xad9a38c0, 23 },
    { 0xe5ea38bd, 95 },
    { 0, 0 },
    { 0x775a38c3, 78 },
    { 0, 0 },
    { 0x7b1a38c5, 34 },
    { 0x1fca38c6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcaa38ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x924a38cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x487a38d0, 76 },
    { 0x557a38d1, 61 },
    { 0x84aa38d1, 33 },
    { 0x32ea38d2, 76 },
    { 0x2c6a38d3, 98 },
    { 0x4a1a38d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8a38dd, 90 },
    { 0, 0 },
    { 0xe48a38df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39a38e2, 65 },
    { 0, 0 },
    { 0x397a38e4, 98 },
    { 0, 0 },
    { 0x649a38e6, 34 },
    { 0, 0 },
    { 0x11a38e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9da38eb, 93 },
    { 0, 0 },
    { 0xfdfa38ed, 98 },
    { 0x3e3a38ed, 81 },
    { 0, 0 },
    { 0xf1aa38f0, 90 },
    { 0x8c3a38f1, 23 },
    { 0xf41a38f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9a38f5, 33 },
    { 0x7eca38f6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x278a38f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x142a38fc, 55 },
    { 0xc0ba38fd, 33 },
    { 0, 0 },
    { 0x59aa38ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x777a3902, 93 },
    { 0x2afa3903, 33 },
    { 0xcbca3902, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2a3909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdea390e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafa3915, 61 },
    { 0xc53a3915, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982a391b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a3923, 90 },
    { 0xe6ea3924, 61 },
    { 0x275a3925, 98 },
    { 0, 0 },
    { 0x653a3927, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a392b, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128a3933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2a3938, 90 },
    { 0, 0 },
    { 0x706a393a, 33 },
    { 0x272a393a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0a3946, 23 },
    { 0x16da3947, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2a394d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ba395a, 59 },
    { 0x299a395b, 34 },
    { 0, 0 },
    { 0xeb1a395d, 33 },
    { 0, 0 },
    { 0x8b5a395f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x602a3962, 33 },
    { 0, 0 },
    { 0x5eca3964, 23 },
    { 0xc46a3965, 46 },
    { 0, 0 },
    { 0x633a3967, 65 },
    { 0x10aa3968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa396f, 87 },
    { 0, 0 },
    { 0x1fea3971, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8a3976, 91 },
    { 0xa7ca3977, 34 },
    { 0x9c5a3977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ca397d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ca3983, 95 },
    { 0x3b1a3983, 90 },
    { 0, 0 },
    { 0xc01a3986, 93 },
    { 0x5a6a3986, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x522a398a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6da398f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81aa3992, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x430a3995, 94 },
    { 0xbffa3995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7a3999, 46 },
    { 0xa28a3999, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a399f, 33 },
    { 0xc23a399f, 23 },
    { 0x9aea399f, 99 },
    { 0xd72a399f, 76 },
    { 0x494a39a0, 12 },
    { 0x465a39a4, 61 },
    { 0xb81a39a1, 78 },
    { 0x6c9a39a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48a39aa, 90 },
    { 0x61a39ab, 95 },
    { 0, 0 },
    { 0x278a39ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8a39b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2da39b8, 61 },
    { 0, 0 },
    { 0xb3a39ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9a39c0, 33 },
    { 0, 0 },
    { 0x80fa39c2, 33 },
    { 0x72ba39c2, 34 },
    { 0x2e4a39c4, 55 },
    { 0xa69a39c5, 33 },
    { 0x7d2a39c6, 93 },
    { 0x29da39c6, 3 },
    { 0x725a39c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770a39cd, 55 },
    { 0, 0 },
    { 0xdc1a39cf, 34 },
    { 0x3e3a39cf, 88 },
    { 0, 0 },
    { 0x9aba39d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeba39d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x44da39db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a39de, 98 },
    { 0x52ca39df, 69 },
    { 0x2caa39e0, 33 },
    { 0, 0 },
    { 0x558a39e2, 23 },
    { 0x3eea39e3, 23 },
    { 0, 0 },
    { 0xca0a39e5, 90 },
    { 0xf8a39e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07a39ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ba39f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5a39fb, 55 },
    { 0, 0 },
    { 0x622a39fd, 99 },
    { 0xcc6a39fd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3a3a04, 61 },
    { 0xbcaa3a05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256a3a09, 36 },
    { 0, 0 },
    { 0xc95a3a0b, 91 },
    { 0, 0 },
    { 0x39ea3a0d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cca3a10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830a3a15, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aa3a19, 76 },
    { 0x7d4a3a19, 76 },
    { 0, 0 },
    { 0x9b4a3a1c, 76 },
    { 0xd29a3a1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bda3a23, 33 },
    { 0xacaa3a23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69a3a28, 91 },
    { 0x424a3a29, 85 },
    { 0xac6a3a28, 55 },
    { 0, 0 },
    { 0x95da3a2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289a3a33, 98 },
    { 0xfdea3a33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2a3a3b, 90 },
    { 0xbe0a3a3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368a3a42, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fa3a46, 23 },
    { 0, 0 },
    { 0x1b5a3a48, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbea3a4b, 33 },
    { 0xd36a3a4b, 23 },
    { 0xb8ba3a4c, 66 },
    { 0xeb4a3a4e, 98 },
    { 0xbd2a3a4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bea3a54, 33 },
    { 0x582a3a55, 33 },
    { 0xf7ba3a56, 90 },
    { 0xd23a3a55, 93 },
    { 0x311a3a55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ca3a5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63a3a5f, 90 },
    { 0x798a3a60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a3a64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7a3a67, 34 },
    { 0xb5da3a68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3a3a6e, 46 },
    { 0x52a3a6e, 95 },
    { 0, 0 },
    { 0x42ca3a71, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6da3a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aa3a7c, 61 },
    { 0, 0 },
    { 0xc41a3a7e, 90 },
    { 0x7c2a3a7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32a3a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eea3a8f, 90 },
    { 0xb04a3a90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735a3a94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01a3a9a, 93 },
    { 0xfdaa3a9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6da3aa0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ea3aa6, 95 },
    { 0x339a3aa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35a3aac, 61 },
    { 0, 0 },
    { 0xaf5a3aae, 43 },
    { 0, 0 },
    { 0x3a7a3ab0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3a3ab7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71a3abc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891a3ac6, 43 },
    { 0x312a3ac7, 87 },
    { 0x765a3ac8, 61 },
    { 0x8f6a3ac8, 95 },
    { 0xcd1a3ac6, 98 },
    { 0, 0 },
    { 0x5a2a3acc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07a3acf, 61 },
    { 0x597a3ad0, 61 },
    { 0x84aa3ad0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133a3ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83a3ad8, 88 },
    { 0x7c4a3ad9, 34 },
    { 0x107a3ada, 40 },
    { 0x49da3ada, 55 },
    { 0, 0 },
    { 0x289a3add, 98 },
    { 0xf15a3add, 97 },
    { 0x2aaa3add, 33 },
    { 0x7b3a3ae0, 33 },
    { 0x688a3ae1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ba3ae4, 91 },
    { 0xfdba3ae4, 90 },
    { 0xad2a3ae5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eca3ae9, 90 },
    { 0xe80a3aea, 98 },
    { 0xad4a3ae9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143a3af2, 95 },
    { 0, 0 },
    { 0x942a3af4, 92 },
    { 0xbdca3af4, 61 },
    { 0x809a3af6, 66 },
    { 0, 0 },
    { 0x663a3af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440a3afd, 95 },
    { 0xf9a3afd, 90 },
    { 0xd31a3aff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6a3b09, 34 },
    { 0x339a3b0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4a3b10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0a3b13, 23 },
    { 0xff2a3b13, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ba3b18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4a3b1b, 43 },
    { 0, 0 },
    { 0x8aca3b1d, 23 },
    { 0xcaaa3b1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0a3b21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a3b26, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01a3b2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5a3b2d, 90 },
    { 0, 0 },
    { 0x811a3b2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20a3b32, 61 },
    { 0x1e0a3b32, 23 },
    { 0x6dda3b34, 90 },
    { 0x546a3b35, 23 },
    { 0x96ba3b36, 85 },
    { 0xddfa3b37, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaba3b3c, 91 },
    { 0, 0 },
    { 0x53da3b3e, 61 },
    { 0, 0 },
    { 0x745a3b40, 95 },
    { 0, 0 },
    { 0xebaa3b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29a3b54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4a3b57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5a3b5b, 95 },
    { 0xd03a3b5c, 100 },
    { 0x8d6a3b5d, 90 },
    { 0, 0 },
    { 0x698a3b5f, 23 },
    { 0x1fca3b60, 95 },
    { 0, 0 },
    { 0x780a3b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78aa3b6a, 34 },
    { 0xd3ea3b6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36a3b74, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dea3b79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1da3b7d, 98 },
    { 0x53ea3b7e, 78 },
    { 0x68fa3b7f, 98 },
    { 0, 0 },
    { 0xfdca3b81, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05a3b87, 33 },
    { 0, 0 },
    { 0x168a3b89, 81 },
    { 0xf76a3b8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4aa3b8d, 33 },
    { 0x944a3b8e, 33 },
    { 0x33ca3b8d, 61 },
    { 0x7a4a3b8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5a3b93, 90 },
    { 0, 0 },
    { 0xb9aa3b95, 40 },
    { 0, 0 },
    { 0xca7a3b97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3a3b9a, 76 },
    { 0xc85a3b9b, 99 },
    { 0x464a3b9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fa3ba2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dda3baa, 43 },
    { 0x270a3bab, 99 },
    { 0xc39a3bac, 23 },
    { 0, 0 },
    { 0xe06a3bae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78da3bb5, 33 },
    { 0x210a3bb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ca3bbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ca3bc1, 43 },
    { 0x33da3bc1, 90 },
    { 0x115a3bc3, 90 },
    { 0x99ba3bc4, 80 },
    { 0x394a3bc5, 61 },
    { 0xce9a3bc6, 65 },
    { 0x906a3bc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1a3bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x450a3bcd, 35 },
    { 0, 0 },
    { 0xa96a3bcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203a3bd9, 77 },
    { 0xd93a3bda, 33 },
    { 0xbd1a3bda, 91 },
    { 0xa96a3bdc, 33 },
    { 0, 0 },
    { 0x96fa3bde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4a3be2, 98 },
    { 0, 0 },
    { 0x8b9a3be4, 95 },
    { 0x2b8a3be4, 95 },
    { 0xfb9a3be5, 90 },
    { 0x651a3be5, 61 },
    { 0xc0ba3be8, 95 },
    { 0xfa3a3be9, 67 },
    { 0x6e6a3be8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9baa3bed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaa3bf0, 53 },
    { 0, 0 },
    { 0xdc4a3bf2, 33 },
    { 0x98a3bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9da3bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ea3c04, 98 },
    { 0x976a3c05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6a3c09, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27a3c0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2a3c10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ea3c14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6a3c19, 97 },
    { 0x711a3c19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37a3c1d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0a3c27, 99 },
    { 0x8bea3c28, 95 },
    { 0xd7da3c28, 75 },
    { 0xeb5a3c2a, 33 },
    { 0, 0 },
    { 0xe08a3c2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a3c2f, 95 },
    { 0, 0 },
    { 0x5e7a3c31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3a3c34, 61 },
    { 0x32ba3c35, 43 },
    { 0x6e8a3c34, 90 },
    { 0x9baa3c37, 90 },
    { 0x1c9a3c38, 98 },
    { 0x50a3c39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9a3c3e, 61 },
    { 0x9f5a3c3f, 61 },
    { 0x964a3c40, 23 },
    { 0, 0 },
    { 0x525a3c42, 23 },
    { 0x41a3c42, 98 },
    { 0x55a3c44, 23 },
    { 0xb6fa3c45, 96 },
    { 0x412a3c44, 23 },
    { 0, 0 },
    { 0x1b1a3c48, 36 },
    { 0xd18a3c49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a3c4d, 81 },
    { 0xfbea3c4e, 90 },
    { 0xe72a3c4f, 95 },
    { 0x10ca3c4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea3c53, 66 },
    { 0, 0 },
    { 0x5fba3c55, 95 },
    { 0x5c8a3c56, 90 },
    { 0x5eca3c57, 17 },
    { 0xca1a3c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fa3c5b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230a3c5f, 33 },
    { 0xbc6a3c5f, 90 },
    { 0xf15a3c61, 67 },
    { 0x3a2a3c62, 95 },
    { 0, 0 },
    { 0xcaea3c64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90a3c67, 33 },
    { 0x4e0a3c68, 55 },
    { 0xd1aa3c69, 33 },
    { 0xa94a3c69, 34 },
    { 0xad4a3c6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675a3c6f, 67 },
    { 0xd62a3c70, 95 },
    { 0, 0 },
    { 0x161a3c72, 33 },
    { 0xfdba3c73, 61 },
    { 0x7c2a3c74, 34 },
    { 0x631a3c75, 33 },
    { 0xee2a3c76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a3c7c, 33 },
    { 0x2e0a3c7c, 61 },
    { 0xf3da3c7e, 77 },
    { 0, 0 },
    { 0x303a3c80, 61 },
    { 0, 0 },
    { 0x34da3c82, 33 },
    { 0, 0 },
    { 0x170a3c84, 84 },
    { 0, 0 },
    { 0x6faa3c86, 33 },
    { 0xbefa3c87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ea3c93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957a3c98, 95 },
    { 0x91fa3c99, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a3c9e, 66 },
    { 0x817a3c9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759a3ca5, 23 },
    { 0x9c1a3ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fca3caa, 84 },
    { 0, 0 },
    { 0xfb1a3cac, 61 },
    { 0x5c8a3cad, 90 },
    { 0x209a3cad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356a3cb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9a3cbc, 55 },
    { 0x4eaa3cbd, 98 },
    { 0x49a3cbd, 41 },
    { 0x347a3cbd, 40 },
    { 0x337a3cc0, 75 },
    { 0x85fa3cc1, 66 },
    { 0x2ca3cbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fa3cc9, 52 },
    { 0xd85a3cc9, 90 },
    { 0x3efa3cca, 23 },
    { 0x978a3cca, 95 },
    { 0x1ca3ccd, 61 },
    { 0x5f2a3cce, 83 },
    { 0, 0 },
    { 0xae4a3cd0, 95 },
    { 0, 0 },
    { 0x8bba3cd2, 23 },
    { 0, 0 },
    { 0x40da3cd4, 33 },
    { 0xc3a3cd4, 66 },
    { 0xb7ea3cd6, 76 },
    { 0xb06a3cd7, 98 },
    { 0xe84a3cd6, 98 },
    { 0x945a3cd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a3cdd, 61 },
    { 0, 0 },
    { 0xcaaa3cdf, 76 },
    { 0, 0 },
    { 0x2c2a3ce1, 95 },
    { 0, 0 },
    { 0xc31a3ce3, 52 },
    { 0, 0 },
    { 0x27ca3ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fa3cfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fca3cfd, 95 },
    { 0, 0 },
    { 0x443a3cff, 33 },
    { 0x34da3d00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31a3d0a, 46 },
    { 0x834a3d0b, 90 },
    { 0xa22a3d0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23a3d13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x954a3d16, 33 },
    { 0x125a3d17, 93 },
    { 0xcfea3d18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a3d1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92a3d1e, 90 },
    { 0xb00a3d1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5a3d22, 93 },
    { 0x6c7a3d22, 90 },
    { 0xa5fa3d24, 33 },
    { 0x67ea3d24, 61 },
    { 0, 0 },
    { 0xb64a3d27, 33 },
    { 0x9e8a3d27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebda3d2b, 35 },
    { 0x931a3d2b, 98 },
    { 0xa9a3d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450a3d31, 93 },
    { 0x7e7a3d32, 76 },
    { 0x5ada3d32, 75 },
    { 0, 0 },
    { 0x8eca3d35, 98 },
    { 0, 0 },
    { 0xd28a3d37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x777a3d3a, 91 },
    { 0xbb2a3d3a, 33 },
    { 0, 0 },
    { 0x5b5a3d3d, 76 },
    { 0x1cba3d3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0a3d4a, 78 },
    { 0xf27a3d4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccba3d4e, 88 },
    { 0x8f3a3d4e, 95 },
    { 0, 0 },
    { 0xb2ea3d51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda3d56, 99 },
    { 0x85ca3d57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a3d5a, 98 },
    { 0xaeca3d5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366a3d63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xafda3d66, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ca3d69, 95 },
    { 0, 0 },
    { 0x7aea3d6b, 91 },
    { 0, 0 },
    { 0x8a9a3d6d, 23 },
    { 0x92ca3d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7a3d73, 55 },
    { 0, 0 },
    { 0xbb4a3d75, 61 },
    { 0, 0 },
    { 0xaf4a3d77, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x641a3d7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x191a3d7d, 23 },
    { 0x506a3d7e, 55 },
    { 0x6b2a3d7f, 87 },
    { 0x78ca3d7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a3d83, 90 },
    { 0x7e9a3d84, 33 },
    { 0xddaa3d85, 33 },
    { 0xd26a3d83, 95 },
    { 0x76ca3d87, 90 },
    { 0xccba3d87, 33 },
    { 0x71a3d85, 76 },
    { 0x84ca3d8a, 90 },
    { 0xab9a3d88, 90 },
    { 0, 0 },
    { 0x3eda3d8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1a3d90, 61 },
    { 0x274a3d91, 90 },
    { 0x498a3d92, 91 },
    { 0xec2a3d93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a3d96, 43 },
    { 0, 0 },
    { 0xc2ca3d98, 33 },
    { 0x27ba3d99, 87 },
    { 0x6b9a3d9a, 95 },
    { 0xb99a3d9b, 61 },
    { 0, 0 },
    { 0x247a3d9d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195a3da2, 46 },
    { 0, 0 },
    { 0xd48a3da4, 91 },
    { 0x349a3da5, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14a3dac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56a3db1, 98 },
    { 0, 0 },
    { 0x61ca3db3, 23 },
    { 0, 0 },
    { 0xad7a3db5, 95 },
    { 0x112a3db6, 90 },
    { 0, 0 },
    { 0x3aba3db8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ea3dbd, 23 },
    { 0x6a3a3dbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da3dc6, 34 },
    { 0x8a0a3dc6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a3dcd, 23 },
    { 0, 0 },
    { 0x995a3dcf, 40 },
    { 0x21a3dcf, 98 },
    { 0xbffa3dd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aa3dd8, 85 },
    { 0x13ca3dd9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dca3ddd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba3de2, 95 },
    { 0xf4fa3de3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9a3def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca3df2, 90 },
    { 0xab7a3df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3a3df7, 88 },
    { 0x654a3df7, 23 },
    { 0xd35a3df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ba3e00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810a3e05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13a3e09, 93 },
    { 0xcd5a3e0a, 95 },
    { 0xdf8a3e0b, 91 },
    { 0x13ea3e0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54aa3e12, 84 },
    { 0x892a3e13, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524a3e17, 83 },
    { 0xaa7a3e18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa3e21, 76 },
    { 0, 0 },
    { 0xd4fa3e23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaa3e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa3e2b, 23 },
    { 0, 0 },
    { 0x190a3e2d, 98 },
    { 0x7dea3e2d, 98 },
    { 0x552a3e2f, 23 },
    { 0, 0 },
    { 0xf3ba3e31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26a3e38, 95 },
    { 0xc4ba3e39, 61 },
    { 0x89aa3e3a, 52 },
    { 0xfcda3e3a, 66 },
    { 0x39fa3e3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5da3e40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7a3e44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x993a3e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfa3e4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1a3e55, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44a3e58, 99 },
    { 0x5bca3e58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a3e5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ba3e62, 61 },
    { 0xcb5a3e63, 98 },
    { 0xda5a3e63, 69 },
    { 0x4d5a3e63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93a3e6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da3e6f, 98 },
    { 0x16ea3e70, 90 },
    { 0x6bba3e6f, 95 },
    { 0x51ea3e72, 91 },
    { 0, 0 },
    { 0x49ba3e74, 34 },
    { 0xa4ba3e75, 90 },
    { 0xf71a3e74, 87 },
    { 0x270a3e77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0a3e7d, 76 },
    { 0x1b4a3e7d, 52 },
    { 0x6baa3e7f, 52 },
    { 0x1fca3e80, 100 },
    { 0x376a3e81, 46 },
    { 0, 0 },
    { 0x640a3e83, 24 },
    { 0x7daa3e83, 55 },
    { 0xb97a3e83, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5aa3e8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a3e8e, 61 },
    { 0x2b9a3e8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8a3e95, 61 },
    { 0x211a3e95, 95 },
    { 0, 0 },
    { 0x231a3e98, 93 },
    { 0xdeda3e99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fa3e9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973a3ea1, 87 },
    { 0xa33a3ea2, 90 },
    { 0xa9ca3ea3, 33 },
    { 0xcffa3ea3, 66 },
    { 0x6d7a3ea5, 37 },
    { 0x987a3ea5, 33 },
    { 0, 0 },
    { 0xb9fa3ea8, 66 },
    { 0, 0 },
    { 0xaba3eaa, 66 },
    { 0, 0 },
    { 0x98a3eac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222a3eaf, 90 },
    { 0xa71a3eb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fda3eb4, 61 },
    { 0x4d5a3eb5, 90 },
    { 0, 0 },
    { 0xb7aa3eb7, 78 },
    { 0x975a3eb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da3ebb, 66 },
    { 0xbcba3ebc, 61 },
    { 0x8b9a3ebc, 95 },
    { 0x4afa3ebe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eba3ec1, 90 },
    { 0, 0 },
    { 0x76a3ec3, 81 },
    { 0, 0 },
    { 0x6eea3ec5, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53a3ecb, 95 },
    { 0, 0 },
    { 0xca2a3ecd, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ea3ed1, 33 },
    { 0, 0 },
    { 0xe33a3ed3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a3ee0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eda3ee3, 95 },
    { 0xa9ca3ee4, 90 },
    { 0xd22a3ee4, 43 },
    { 0, 0 },
    { 0xb51a3ee7, 90 },
    { 0x5ada3ee7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ea3eed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a3ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fa3ef8, 33 },
    { 0x68ba3ef8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefaa3efc, 76 },
    { 0xab5a3efd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76a3f00, 66 },
    { 0, 0 },
    { 0xf56a3f02, 33 },
    { 0, 0 },
    { 0xdf8a3f04, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522a3f0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x914a3f0d, 33 },
    { 0, 0 },
    { 0x968a3f0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4a3f15, 100 },
    { 0, 0 },
    { 0xb15a3f17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710a3f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69a3f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaada3f25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a3f28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ba3f2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x269a3f2e, 66 },
    { 0xa5ca3f2f, 33 },
    { 0xcd9a3f2e, 33 },
    { 0xecba3f31, 90 },
    { 0x6c8a3f31, 76 },
    { 0x889a3f33, 17 },
    { 0x8d7a3f34, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4a3f3a, 33 },
    { 0x19fa3f3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4a3f3f, 43 },
    { 0xdcca3f3f, 91 },
    { 0xd9a3f40, 90 },
    { 0xf16a3f42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57a3f46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbea3f4a, 94 },
    { 0x9d6a3f4a, 33 },
    { 0x7b1a3f4a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffa3f4f, 90 },
    { 0, 0 },
    { 0x88da3f51, 98 },
    { 0x7daa3f51, 55 },
    { 0x432a3f51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fa3f57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407a3f64, 91 },
    { 0x9e5a3f64, 90 },
    { 0x558a3f66, 76 },
    { 0x891a3f67, 43 },
    { 0, 0 },
    { 0x5e0a3f69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47a3f6d, 61 },
    { 0x1a7a3f6e, 90 },
    { 0x1f5a3f6d, 81 },
    { 0, 0 },
    { 0x638a3f71, 90 },
    { 0, 0 },
    { 0x554a3f73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bda3f78, 95 },
    { 0x68aa3f79, 40 },
    { 0, 0 },
    { 0x891a3f7b, 66 },
    { 0x580a3f7b, 84 },
    { 0, 0 },
    { 0x9f7a3f7e, 33 },
    { 0, 0 },
    { 0x2afa3f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3da3f84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3a3f88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97da3f97, 90 },
    { 0x868a3f97, 33 },
    { 0x518a3f97, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7a3fa1, 81 },
    { 0x8d4a3fa2, 23 },
    { 0xfc7a3fa3, 84 },
    { 0xceaa3fa1, 87 },
    { 0, 0 },
    { 0x338a3fa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x382a3fa9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6a3fb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67a3fb3, 34 },
    { 0x415a3fb3, 33 },
    { 0xc0ea3fb4, 55 },
    { 0xee0a3fb6, 43 },
    { 0xdc8a3fb5, 98 },
    { 0xecaa3fb8, 76 },
    { 0x798a3fb9, 88 },
    { 0x793a3fb3, 90 },
    { 0x5c4a3fbb, 23 },
    { 0x2baa3fbb, 23 },
    { 0, 0 },
    { 0x8f2a3fbe, 33 },
    { 0xcb3a3fbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1a3fc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a3fcc, 90 },
    { 0x18fa3fcd, 33 },
    { 0x644a3fcd, 90 },
    { 0xcbda3fcf, 90 },
    { 0xd20a3fd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cba3fd3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92a3fd7, 98 },
    { 0x271a3fd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164a3fdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ca3fe0, 68 },
    { 0xd8fa3fe1, 33 },
    { 0, 0 },
    { 0x47ba3fe3, 90 },
    { 0xa7ca3fe3, 76 },
    { 0, 0 },
    { 0xafda3fe6, 84 },
    { 0x7fda3fe7, 85 },
    { 0x8bba3fe8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eca3feb, 33 },
    { 0x54ba3feb, 33 },
    { 0xd47a3fed, 97 },
    { 0x53ca3fed, 84 },
    { 0, 0 },
    { 0xf7ba3ff0, 34 },
    { 0xcd2a3ff1, 61 },
    { 0x3eaa3ff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55a3ff5, 33 },
    { 0, 0 },
    { 0xf76a3ff7, 88 },
    { 0x8f0a3ff7, 95 },
    { 0, 0 },
    { 0x3cca3ffa, 67 },
    { 0, 0 },
    { 0x29aa3ffc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9a3fff, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x774a4002, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90a4009, 95 },
    { 0x9aba400a, 90 },
    { 0x581a400a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aa4010, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ba4013, 23 },
    { 0x13da4013, 34 },
    { 0x774a4013, 46 },
    { 0x1a3a4016, 33 },
    { 0x5da4015, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x623a401a, 61 },
    { 0xf77a401b, 23 },
    { 0xb3ca401c, 61 },
    { 0x89aa401d, 99 },
    { 0x6f3a401a, 81 },
    { 0x779a401f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80a4024, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7a4028, 76 },
    { 0, 0 },
    { 0x1baa402a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da402e, 65 },
    { 0x69a402e, 62 },
    { 0xb6ba402f, 90 },
    { 0x9ada402e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14a4036, 90 },
    { 0xdbda4037, 95 },
    { 0xd2aa4038, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fa403b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ba4042, 95 },
    { 0, 0 },
    { 0x77da4044, 95 },
    { 0xb0fa4044, 98 },
    { 0x504a4046, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6a404a, 85 },
    { 0x5ada404a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83da4062, 43 },
    { 0x677a4063, 23 },
    { 0x296a4063, 68 },
    { 0x502a4062, 33 },
    { 0x1c4a4062, 95 },
    { 0xd31a4064, 90 },
    { 0xf00a4062, 100 },
    { 0xfd7a4069, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eba406f, 77 },
    { 0xa6a4070, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a4075, 59 },
    { 0xf8fa4076, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41a407b, 33 },
    { 0xe92a407b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11a407f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ca4082, 69 },
    { 0, 0 },
    { 0x99fa4084, 61 },
    { 0x4cfa4085, 98 },
    { 0x18ca4084, 23 },
    { 0, 0 },
    { 0x3aea4088, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50a408d, 55 },
    { 0x5d8a408d, 58 },
    { 0xf23a408f, 43 },
    { 0xad3a408f, 95 },
    { 0x16ca4090, 75 },
    { 0x3a1a408f, 55 },
    { 0x105a4093, 90 },
    { 0, 0 },
    { 0x282a4095, 95 },
    { 0xbda4096, 23 },
    { 0, 0 },
    { 0xe23a4098, 98 },
    { 0xa50a4099, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a409e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0a40a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a40a8, 61 },
    { 0x9fa40a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563a40b1, 95 },
    { 0xd0a40b1, 98 },
    { 0, 0 },
    { 0xe42a40b4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x883a40b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2a40bb, 76 },
    { 0, 0 },
    { 0xbf2a40bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32a40c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fa40d0, 61 },
    { 0xc15a40d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3a40d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca40d8, 34 },
    { 0, 0 },
    { 0x862a40da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46a40e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56a40e4, 90 },
    { 0xa57a40e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0a40e9, 23 },
    { 0x405a40ea, 55 },
    { 0xef4a40ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a40f1, 68 },
    { 0x71aa40f2, 98 },
    { 0, 0 },
    { 0x5f7a40f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763a40ff, 95 },
    { 0, 0 },
    { 0x8d9a4101, 61 },
    { 0x4dda4101, 81 },
    { 0xaca4101, 95 },
    { 0x328a4104, 34 },
    { 0x44ba4105, 66 },
    { 0xb12a4106, 66 },
    { 0xd74a4107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69a410a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x771a410d, 87 },
    { 0xc3fa410e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51a4111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ea4114, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6a411a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32a411d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ca4124, 46 },
    { 0x227a4125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ba4129, 55 },
    { 0x6cba412a, 99 },
    { 0x7a0a412b, 81 },
    { 0x6aea412c, 90 },
    { 0x380a412a, 90 },
    { 0xdb5a412e, 98 },
    { 0, 0 },
    { 0x242a4130, 77 },
    { 0xc3aa4131, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aa4134, 52 },
    { 0x114a4135, 34 },
    { 0x8d9a4136, 99 },
    { 0xf4a4137, 55 },
    { 0xd82a4138, 61 },
    { 0x6ea4139, 84 },
    { 0xa8ea413a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78a413e, 98 },
    { 0x9e0a413e, 91 },
    { 0xd0aa413e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a4144, 81 },
    { 0x80ba4145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efa4149, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x129a414c, 90 },
    { 0x282a414d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85a4151, 99 },
    { 0x19ea4152, 87 },
    { 0xe1a4153, 52 },
    { 0, 0 },
    { 0x2d9a4155, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16a415b, 100 },
    { 0, 0 },
    { 0x959a415d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ba4165, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20a416c, 93 },
    { 0x8fca416d, 75 },
    { 0xfa7a416e, 23 },
    { 0x127a416d, 95 },
    { 0, 0 },
    { 0x4e0a4171, 61 },
    { 0, 0 },
    { 0xc46a4173, 34 },
    { 0xb57a4174, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x221a4177, 55 },
    { 0x7c2a4178, 23 },
    { 0, 0 },
    { 0x694a417a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9a4187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ea418a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896a418e, 23 },
    { 0x8fda418f, 95 },
    { 0xb1aa4190, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4a4193, 80 },
    { 0, 0 },
    { 0x137a4195, 98 },
    { 0x7d7a4196, 95 },
    { 0x87ca4197, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8a41a0, 46 },
    { 0x4a4a41a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a41a6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fa41a9, 75 },
    { 0, 0 },
    { 0x57ca41ab, 23 },
    { 0, 0 },
    { 0x43ba41ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda41b0, 34 },
    { 0, 0 },
    { 0x852a41b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fa41b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eda41be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a41c3, 23 },
    { 0xc42a41c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a41c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a41ca, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a41d3, 38 },
    { 0, 0 },
    { 0x4dca41d5, 61 },
    { 0x721a41d5, 98 },
    { 0x5c4a41d7, 76 },
    { 0x82ea41d7, 55 },
    { 0x541a41d7, 23 },
    { 0xc0ea41d8, 84 },
    { 0xa59a41da, 98 },
    { 0x805a41d8, 46 },
    { 0x9f7a41d9, 76 },
    { 0x960a41d5, 98 },
    { 0x5a2a41df, 55 },
    { 0x283a41e0, 90 },
    { 0x268a41e1, 76 },
    { 0xaf6a41db, 98 },
    { 0xc5ca41e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaea41e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0a41ec, 66 },
    { 0x203a41ed, 61 },
    { 0, 0 },
    { 0xd98a41ef, 95 },
    { 0x6c2a41f0, 23 },
    { 0x5aa41ef, 80 },
    { 0x505a41f2, 98 },
    { 0x7b4a41f2, 81 },
    { 0xafca41ef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33a41f9, 81 },
    { 0xb4fa41fa, 40 },
    { 0x93fa41fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39aa4201, 81 },
    { 0x918a4201, 95 },
    { 0, 0 },
    { 0xc5da4204, 34 },
    { 0x800a4204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74a4208, 70 },
    { 0, 0 },
    { 0xdb3a420a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x393a420d, 95 },
    { 0x7f8a420e, 91 },
    { 0x951a420d, 34 },
    { 0xee3a4210, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31a4214, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a4219, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6a421c, 17 },
    { 0x5c7a421c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccca4225, 61 },
    { 0, 0 },
    { 0xd74a4227, 75 },
    { 0x60da4228, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fa422c, 19 },
    { 0x6b6a422c, 33 },
    { 0x9f9a422d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x451a4231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfa4234, 55 },
    { 0x137a4235, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfca423c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ca4242, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1aa4245, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81a4249, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ca4252, 23 },
    { 0x8e2a4253, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0a4256, 61 },
    { 0x70ea4257, 90 },
    { 0x60aa4258, 23 },
    { 0x4e7a4259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fa4263, 23 },
    { 0xfe6a4264, 97 },
    { 0x648a4265, 90 },
    { 0x9cda4263, 43 },
    { 0x2f5a4267, 61 },
    { 0xd57a4268, 61 },
    { 0x2dda4268, 33 },
    { 0x42da4269, 95 },
    { 0, 0 },
    { 0x14ca426c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aa4270, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a4278, 61 },
    { 0, 0 },
    { 0x41fa427a, 98 },
    { 0x5aba427b, 66 },
    { 0, 0 },
    { 0x72ca427d, 61 },
    { 0x6a7a427e, 75 },
    { 0xad6a427f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68a4284, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6a4287, 87 },
    { 0x3a0a4288, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffa428d, 100 },
    { 0, 0 },
    { 0x54aa428f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ba4293, 61 },
    { 0xd54a4294, 55 },
    { 0x75ea4294, 33 },
    { 0x5d4a4296, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a429c, 23 },
    { 0x978a429d, 100 },
    { 0x33a429d, 43 },
    { 0, 0 },
    { 0x6d8a42a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacda42a3, 100 },
    { 0x5e8a42a4, 33 },
    { 0, 0 },
    { 0x18a42a6, 33 },
    { 0, 0 },
    { 0x590a42a8, 61 },
    { 0, 0 },
    { 0x24ca42aa, 66 },
    { 0x930a42aa, 33 },
    { 0x776a42ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889a42b3, 55 },
    { 0x156a42b4, 55 },
    { 0x378a42b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aa42ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92a42c0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x23da42c3, 43 },
    { 0, 0 },
    { 0x490a42c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefda42ca, 75 },
    { 0x2eba42cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ea42cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa42d5, 61 },
    { 0x848a42d6, 61 },
    { 0x2dca42d5, 34 },
    { 0, 0 },
    { 0x2d7a42d9, 78 },
    { 0x796a42da, 33 },
    { 0, 0 },
    { 0xd47a42dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7a42e2, 98 },
    { 0xf9a42e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998a42e7, 90 },
    { 0, 0 },
    { 0xcbea42e9, 33 },
    { 0, 0 },
    { 0xacba42eb, 40 },
    { 0x629a42eb, 43 },
    { 0x307a42ed, 33 },
    { 0x62fa42ed, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaa42f3, 90 },
    { 0x77fa42f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334a4300, 23 },
    { 0x90aa4300, 66 },
    { 0xaaa4301, 93 },
    { 0, 0 },
    { 0xf12a4304, 33 },
    { 0x80ba4305, 90 },
    { 0x5c6a4305, 23 },
    { 0x804a4304, 61 },
    { 0x259a4308, 56 },
    { 0xa9aa4305, 95 },
    { 0x317a4305, 90 },
    { 0x3da430b, 81 },
    { 0x428a430b, 33 },
    { 0xb64a430d, 55 },
    { 0x32a4304, 23 },
    { 0, 0 },
    { 0xf04a4310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a4315, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a4319, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a4324, 52 },
    { 0, 0 },
    { 0xc69a4326, 90 },
    { 0x790a4327, 95 },
    { 0x625a4328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ca432c, 93 },
    { 0xca0a432d, 43 },
    { 0, 0 },
    { 0x93ba432f, 90 },
    { 0xb4ba432f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ca4333, 55 },
    { 0x68da4334, 91 },
    { 0xd8da4334, 23 },
    { 0x229a4336, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa4347, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ba4356, 52 },
    { 0, 0 },
    { 0xa5aa4358, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa435f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fa4362, 55 },
    { 0x53aa4363, 61 },
    { 0, 0 },
    { 0x49ea4365, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ea4368, 95 },
    { 0x79ea4368, 98 },
    { 0xe25a436a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7a4371, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637a437a, 61 },
    { 0x658a437b, 95 },
    { 0x21ba437c, 61 },
    { 0x804a437d, 17 },
    { 0x4cfa437d, 67 },
    { 0, 0 },
    { 0x4b7a4380, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4383, 98 },
    { 0, 0 },
    { 0xc14a4385, 61 },
    { 0x6b1a4386, 23 },
    { 0xafaa4387, 61 },
    { 0x15aa4387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a438c, 61 },
    { 0x324a438d, 23 },
    { 0x4b9a438e, 23 },
    { 0x28aa438f, 95 },
    { 0x2d2a438c, 55 },
    { 0x806a438e, 33 },
    { 0xdd1a438e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x260a4395, 33 },
    { 0xc15a4395, 90 },
    { 0x42fa4395, 43 },
    { 0x8b1a4396, 98 },
    { 0, 0 },
    { 0x2a7a439a, 75 },
    { 0x56da439b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40a439f, 90 },
    { 0, 0 },
    { 0x97da43a1, 34 },
    { 0x540a43a2, 85 },
    { 0x6d1a43a1, 98 },
    { 0, 0 },
    { 0x927a43a5, 88 },
    { 0xb08a43a6, 76 },
    { 0x59aa43a6, 46 },
    { 0xa2ca43a8, 33 },
    { 0, 0 },
    { 0xb7ba43aa, 99 },
    { 0x954a43ab, 52 },
    { 0x34a43ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8a43af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a43b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22a43b8, 43 },
    { 0x8a3a43b9, 98 },
    { 0, 0 },
    { 0x188a43bb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ba43c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7a43c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fba43c7, 34 },
    { 0x109a43c8, 61 },
    { 0x3d0a43c9, 31 },
    { 0x16a43ca, 91 },
    { 0xf62a43c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17a43ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846a43d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda43da, 93 },
    { 0, 0 },
    { 0xefa43dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07a43e1, 55 },
    { 0x54ba43e2, 55 },
    { 0, 0 },
    { 0x70fa43e4, 23 },
    { 0x18ba43e5, 33 },
    { 0x117a43e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1a43e9, 33 },
    { 0, 0 },
    { 0x417a43eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95a43ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232a43f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592a43f8, 95 },
    { 0x351a43f9, 87 },
    { 0x717a43fa, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a43fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba4401, 66 },
    { 0xea7a4401, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579a4407, 90 },
    { 0x2a4a4407, 78 },
    { 0x3f6a4409, 95 },
    { 0x41da440a, 46 },
    { 0, 0 },
    { 0x4b9a440c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaa4410, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812a4418, 17 },
    { 0xaf9a4419, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234a4420, 87 },
    { 0xd71a4421, 34 },
    { 0x963a4422, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823a4426, 87 },
    { 0, 0 },
    { 0xd6fa4428, 61 },
    { 0, 0 },
    { 0x9aea442a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8a442d, 33 },
    { 0, 0 },
    { 0x37ca442f, 40 },
    { 0xa9ca442f, 34 },
    { 0xd00a4431, 34 },
    { 0, 0 },
    { 0x3f7a4433, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208a4438, 33 },
    { 0xaa7a4439, 23 },
    { 0, 0 },
    { 0xe5ba443b, 84 },
    { 0x2fca443c, 95 },
    { 0x6d0a443d, 87 },
    { 0xf0ea443e, 95 },
    { 0xe7aa443e, 99 },
    { 0xab3a4440, 61 },
    { 0x6cba4440, 51 },
    { 0x655a4441, 43 },
    { 0x57da4443, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ea4448, 95 },
    { 0, 0 },
    { 0x197a444a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785a444e, 43 },
    { 0, 0 },
    { 0x484a4450, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3a4454, 68 },
    { 0x105a4454, 33 },
    { 0, 0 },
    { 0x404a4457, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28a445e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2a4464, 61 },
    { 0xa1da4465, 33 },
    { 0x240a4466, 90 },
    { 0x28ba4467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaa446b, 90 },
    { 0x3e2a446c, 91 },
    { 0x442a446c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285a4476, 95 },
    { 0x5d4a4477, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfba447a, 49 },
    { 0x2e3a447b, 66 },
    { 0x1e9a447c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90a4484, 99 },
    { 0, 0 },
    { 0xbeda4486, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1a448e, 84 },
    { 0x4dea448e, 95 },
    { 0x75a448f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4a4493, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fa4497, 95 },
    { 0, 0 },
    { 0xe8ba4499, 76 },
    { 0, 0 },
    { 0x435a449b, 98 },
    { 0x1a1a449c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bea449f, 23 },
    { 0x30aa44a0, 90 },
    { 0, 0 },
    { 0x2daa44a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a44a8, 100 },
    { 0x8c1a44a9, 95 },
    { 0xd55a44a9, 51 },
    { 0x318a44a9, 23 },
    { 0x195a44ac, 76 },
    { 0, 0 },
    { 0xafca44ae, 23 },
    { 0x1a6a44af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607a44b3, 23 },
    { 0, 0 },
    { 0x2caa44b5, 90 },
    { 0x401a44b6, 90 },
    { 0xfb9a44b7, 34 },
    { 0x7faa44b6, 36 },
    { 0x6d1a44b9, 61 },
    { 0, 0 },
    { 0x1bda44bb, 69 },
    { 0x361a44bb, 23 },
    { 0x64fa44bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a44c3, 33 },
    { 0x2d4a44c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ca44d1, 33 },
    { 0, 0 },
    { 0xcefa44d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eda44db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a44e3, 78 },
    { 0xc55a44e4, 61 },
    { 0xd62a44e5, 84 },
    { 0x886a44e6, 61 },
    { 0x51da44e5, 98 },
    { 0x4d4a44e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806a44ec, 55 },
    { 0xcf3a44ed, 78 },
    { 0x2aaa44ed, 34 },
    { 0x616a44ee, 33 },
    { 0, 0 },
    { 0x1e6a44f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffba44f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca44f9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfa44fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fa44ff, 90 },
    { 0, 0 },
    { 0x636a4501, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca4505, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea450a, 69 },
    { 0x669a450a, 93 },
    { 0x189a450c, 78 },
    { 0xb5a450a, 61 },
    { 0x3f5a450e, 90 },
    { 0x19ca450f, 90 },
    { 0x543a4510, 87 },
    { 0x4e9a450f, 76 },
    { 0xbeca4512, 33 },
    { 0xc6a4512, 23 },
    { 0xc16a4514, 23 },
    { 0x56ba450c, 90 },
    { 0, 0 },
    { 0x7f4a4517, 90 },
    { 0xf17a4518, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9a451b, 90 },
    { 0, 0 },
    { 0xb4ca451d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a4529, 90 },
    { 0, 0 },
    { 0x4eea452b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ca452f, 33 },
    { 0x727a4530, 90 },
    { 0, 0 },
    { 0x6e1a4532, 75 },
    { 0x23da4533, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9a4536, 93 },
    { 0x1d0a4536, 90 },
    { 0, 0 },
    { 0x8e9a4539, 76 },
    { 0xc32a453a, 46 },
    { 0x9f4a453a, 55 },
    { 0xe19a453a, 87 },
    { 0x85ca453c, 34 },
    { 0x5fea453c, 33 },
    { 0xaada453f, 87 },
    { 0x3c6a453d, 23 },
    { 0xd71a4541, 17 },
    { 0x6ada4541, 96 },
    { 0x158a4543, 95 },
    { 0x2f0a4543, 95 },
    { 0x7a5a4541, 31 },
    { 0xdbda4546, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3a454f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28a4555, 61 },
    { 0xf52a4556, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ba4559, 91 },
    { 0xfb8a455a, 33 },
    { 0x144a455b, 75 },
    { 0, 0 },
    { 0x1f3a455d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2a4560, 61 },
    { 0x40fa4561, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a4566, 61 },
    { 0xa1ea4567, 81 },
    { 0, 0 },
    { 0xc1da4569, 93 },
    { 0, 0 },
    { 0x16aa456b, 95 },
    { 0, 0 },
    { 0xd57a456d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xecca4570, 91 },
    { 0x17a4571, 98 },
    { 0x546a4570, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xecda4575, 33 },
    { 0x38ca4576, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da4579, 61 },
    { 0x623a4579, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516a457f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea4583, 19 },
    { 0xc46a4584, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a4588, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09a458c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76a458f, 93 },
    { 0x6b4a4590, 98 },
    { 0, 0 },
    { 0x2bfa4592, 61 },
    { 0xfd8a4593, 33 },
    { 0x869a4594, 78 },
    { 0xbf2a4594, 55 },
    { 0x73aa4595, 90 },
    { 0x632a4597, 61 },
    { 0xcb6a4598, 61 },
    { 0x9bda4599, 90 },
    { 0x37ca4598, 98 },
    { 0xfaaa4596, 99 },
    { 0xf3fa459c, 90 },
    { 0x894a4594, 98 },
    { 0x7ba4595, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ea45a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a45aa, 91 },
    { 0x1e1a45ab, 95 },
    { 0, 0 },
    { 0xf13a45ad, 90 },
    { 0x8fba45ad, 33 },
    { 0x8cba45af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fa45b8, 98 },
    { 0x324a45b8, 33 },
    { 0, 0 },
    { 0x606a45bb, 61 },
    { 0x171a45bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda45bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa45c4, 23 },
    { 0, 0 },
    { 0x6a4a45c6, 23 },
    { 0x305a45c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448a45cb, 95 },
    { 0x378a45cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81a45d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971a45e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dca45e4, 76 },
    { 0xf15a45e5, 33 },
    { 0x614a45e6, 23 },
    { 0x604a45e7, 90 },
    { 0, 0 },
    { 0x7eea45e9, 34 },
    { 0x4fba45ea, 66 },
    { 0x965a45eb, 98 },
    { 0xae1a45ea, 76 },
    { 0xc72a45e9, 87 },
    { 0, 0 },
    { 0x8cea45ef, 91 },
    { 0x5aca45ef, 61 },
    { 0x239a45f1, 43 },
    { 0xcaba45f0, 34 },
    { 0xc62a45f3, 55 },
    { 0, 0 },
    { 0x15da45f5, 66 },
    { 0x9eda45f6, 78 },
    { 0x632a45f7, 98 },
    { 0xdfba45f8, 98 },
    { 0, 0 },
    { 0x55a45fa, 90 },
    { 0, 0 },
    { 0xdbba45fc, 90 },
    { 0, 0 },
    { 0xd81a45fe, 23 },
    { 0xe55a45ff, 98 },
    { 0x852a4600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a4603, 90 },
    { 0x256a4604, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ca4607, 98 },
    { 0x508a4608, 93 },
    { 0xa95a4609, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa460c, 90 },
    { 0xf42a460d, 90 },
    { 0x479a460e, 90 },
    { 0xa7fa460f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7a4613, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x886a4616, 90 },
    { 0xfe9a4617, 91 },
    { 0x4e5a4617, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xadda461b, 80 },
    { 0, 0 },
    { 0x7eaa461d, 97 },
    { 0x27ea461d, 23 },
    { 0xbb4a461d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x569a4622, 66 },
    { 0x236a4622, 98 },
    { 0x52ca4623, 90 },
    { 0x13a4625, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea462c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da462f, 95 },
    { 0x87ea4630, 95 },
    { 0x54ea4631, 19 },
    { 0x76ea4632, 61 },
    { 0x849a4633, 90 },
    { 0xad4a4631, 23 },
    { 0x4aea462f, 99 },
    { 0, 0 },
    { 0x3d2a4637, 95 },
    { 0xd15a4638, 78 },
    { 0xf46a4637, 55 },
    { 0xafda463a, 23 },
    { 0xd72a4638, 95 },
    { 0x55a463c, 33 },
    { 0x58ca463d, 61 },
    { 0x845a463c, 87 },
    { 0xa75a463f, 99 },
    { 0x7b8a4640, 98 },
    { 0xc3ca4641, 88 },
    { 0x73ea4642, 34 },
    { 0x746a4643, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253a4647, 90 },
    { 0xb17a4647, 23 },
    { 0xd84a4649, 55 },
    { 0x247a464a, 55 },
    { 0xef6a4649, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa4650, 23 },
    { 0, 0 },
    { 0x5d6a4652, 61 },
    { 0xdc3a4653, 55 },
    { 0xff0a4654, 33 },
    { 0xcc1a4652, 99 },
    { 0x221a4656, 33 },
    { 0, 0 },
    { 0x2aca4658, 90 },
    { 0xc5aa4659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba465e, 90 },
    { 0xe4ea465f, 33 },
    { 0x402a465e, 82 },
    { 0, 0 },
    { 0x81ca4662, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1a4665, 34 },
    { 0x12ea4666, 55 },
    { 0x760a4667, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3a466a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69a4671, 92 },
    { 0x280a4672, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a4675, 61 },
    { 0, 0 },
    { 0xacfa4677, 65 },
    { 0x7a1a4677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7a467b, 87 },
    { 0x7e0a467c, 93 },
    { 0x6d3a467b, 90 },
    { 0x2d5a467b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370a4684, 76 },
    { 0x18da4685, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ba4688, 33 },
    { 0x909a4689, 61 },
    { 0, 0 },
    { 0x8cfa468b, 88 },
    { 0, 0 },
    { 0x177a468d, 99 },
    { 0, 0 },
    { 0xb8da468f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daa4693, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afa469b, 100 },
    { 0, 0 },
    { 0x76ca469d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876a46a9, 23 },
    { 0x810a46aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95a46b0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a46b3, 98 },
    { 0, 0 },
    { 0x4a3a46b5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a46b8, 76 },
    { 0x9b2a46b8, 40 },
    { 0xbc2a46b9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865a46be, 95 },
    { 0, 0 },
    { 0x9eea46c0, 34 },
    { 0, 0 },
    { 0x7a1a46c2, 90 },
    { 0, 0 },
    { 0x7c5a46c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59a46cc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfea46d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02a46d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a46d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a46eb, 87 },
    { 0xeeda46ec, 91 },
    { 0xe41a46ed, 98 },
    { 0x2c6a46eb, 61 },
    { 0, 0 },
    { 0x2efa46f0, 84 },
    { 0x406a46f0, 23 },
    { 0x41fa46f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eba46f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fca46fc, 95 },
    { 0x132a46fd, 33 },
    { 0x8faa46fd, 34 },
    { 0x4cca46fe, 94 },
    { 0, 0 },
    { 0xf58a4701, 95 },
    { 0xe88a4702, 69 },
    { 0xd32a4703, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a4709, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90a470e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a471a, 46 },
    { 0x182a471b, 76 },
    { 0, 0 },
    { 0xe8ca471d, 91 },
    { 0, 0 },
    { 0x75aa471f, 98 },
    { 0x765a4720, 43 },
    { 0x648a4720, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83a4724, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88da4728, 34 },
    { 0xcbca4729, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x207a472c, 84 },
    { 0x48aa472d, 23 },
    { 0, 0 },
    { 0xf22a472f, 33 },
    { 0, 0 },
    { 0xf43a4731, 61 },
    { 0, 0 },
    { 0xac1a4733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49da4737, 76 },
    { 0x62a4737, 33 },
    { 0x5d8a4739, 61 },
    { 0, 0 },
    { 0xd47a473b, 90 },
    { 0x560a473c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94a4740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ba4745, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9a4748, 90 },
    { 0xc8fa4749, 61 },
    { 0xcafa474a, 36 },
    { 0, 0 },
    { 0xc00a474c, 99 },
    { 0, 0 },
    { 0xf5a474e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a4752, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a475c, 61 },
    { 0x361a475d, 61 },
    { 0, 0 },
    { 0xf6da475f, 61 },
    { 0xea7a4760, 67 },
    { 0xbd9a4761, 85 },
    { 0x345a4760, 55 },
    { 0, 0 },
    { 0x229a4764, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796a4769, 23 },
    { 0xfeca476a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6a4773, 85 },
    { 0x596a4773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba4777, 81 },
    { 0xacca4778, 88 },
    { 0xb79a4779, 61 },
    { 0x451a4777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2a477e, 95 },
    { 0x7ba477f, 33 },
    { 0xcb8a4780, 20 },
    { 0, 0 },
    { 0xa4782, 55 },
    { 0, 0 },
    { 0x71aa4784, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85a4787, 17 },
    { 0x79da4788, 93 },
    { 0x42ba4789, 55 },
    { 0x75da4789, 55 },
    { 0x344a4787, 100 },
    { 0, 0 },
    { 0x812a478d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2a4790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea4795, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2a479b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2a479e, 48 },
    { 0x8faa479f, 43 },
    { 0xb40a47a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a47a4, 55 },
    { 0, 0 },
    { 0x30da47a6, 55 },
    { 0, 0 },
    { 0x9eba47a8, 33 },
    { 0x1a8a47a9, 34 },
    { 0xc75a47aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89a47ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08a47b3, 98 },
    { 0x8c9a47b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ba47b9, 100 },
    { 0, 0 },
    { 0xc21a47bb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69a47bf, 61 },
    { 0x198a47c0, 23 },
    { 0x684a47c1, 33 },
    { 0x630a47c2, 40 },
    { 0x309a47c1, 23 },
    { 0x52da47bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x983a47c7, 90 },
    { 0x47da47c8, 75 },
    { 0x906a47c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ba47ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7a47d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52a47d5, 100 },
    { 0x5fba47d5, 33 },
    { 0xc89a47d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702a47dc, 68 },
    { 0x98a47dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da47e3, 93 },
    { 0x1b7a47e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa47e7, 66 },
    { 0x5fba47e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a47ec, 33 },
    { 0x6cda47ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247a47f3, 65 },
    { 0xc7aa47f4, 43 },
    { 0x76aa47f5, 68 },
    { 0x229a47f6, 95 },
    { 0xecca47f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398a47fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927a4800, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca4803, 95 },
    { 0, 0 },
    { 0xb08a4805, 98 },
    { 0x5cfa4805, 23 },
    { 0, 0 },
    { 0xf77a4808, 67 },
    { 0, 0 },
    { 0x74da480a, 98 },
    { 0, 0 },
    { 0xb23a480c, 98 },
    { 0, 0 },
    { 0xcb9a480e, 91 },
    { 0x360a480e, 76 },
    { 0xe37a4810, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fa4815, 95 },
    { 0x460a4816, 99 },
    { 0xd21a4815, 65 },
    { 0xa28a4818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16a481b, 98 },
    { 0x8aaa481b, 33 },
    { 0x902a481d, 100 },
    { 0, 0 },
    { 0xb30a481f, 34 },
    { 0x35da4820, 90 },
    { 0x3d9a4821, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53a4826, 33 },
    { 0, 0 },
    { 0x762a4828, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a482b, 61 },
    { 0xa31a482c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a482f, 55 },
    { 0x498a4830, 98 },
    { 0xfeba482f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33a483d, 81 },
    { 0, 0 },
    { 0x490a483f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x175a4842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27a4849, 90 },
    { 0, 0 },
    { 0x184a484b, 77 },
    { 0x3cfa484b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dba484f, 68 },
    { 0x1c6a484f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796a485d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aa4861, 61 },
    { 0xb0a4861, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafa4866, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ea4869, 23 },
    { 0xc5ca486a, 33 },
    { 0x33da486b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a486e, 100 },
    { 0xce6a486e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x239a4872, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a487d, 66 },
    { 0x8bfa487e, 23 },
    { 0x3eba487f, 76 },
    { 0xba6a4880, 87 },
    { 0x4cea4881, 33 },
    { 0xd78a4882, 23 },
    { 0xcf5a487f, 91 },
    { 0xbdda4884, 33 },
    { 0x510a4885, 23 },
    { 0x26aa4886, 33 },
    { 0xfd5a487e, 23 },
    { 0, 0 },
    { 0xcb3a4889, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56da4890, 76 },
    { 0x99ca4891, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06a4894, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dba489d, 61 },
    { 0xf2ca489e, 90 },
    { 0, 0 },
    { 0x286a48a0, 55 },
    { 0x996a48a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2a48a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ca48aa, 66 },
    { 0, 0 },
    { 0xa6ea48ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a48b0, 33 },
    { 0xb08a48b0, 61 },
    { 0x779a48b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a48b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa48b9, 23 },
    { 0xbe3a48ba, 61 },
    { 0xa19a48bb, 98 },
    { 0x76aa48bb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496a48c3, 61 },
    { 0, 0 },
    { 0x51a48c5, 95 },
    { 0xdb3a48c6, 98 },
    { 0xf8aa48c5, 81 },
    { 0x5faa48c8, 46 },
    { 0x9dea48c7, 61 },
    { 0xfcaa48c8, 98 },
    { 0, 0 },
    { 0xa89a48cc, 34 },
    { 0xfdea48cd, 23 },
    { 0, 0 },
    { 0x5d9a48cf, 33 },
    { 0x460a48d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150a48d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29a48d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19a48dd, 61 },
    { 0, 0 },
    { 0x8f1a48df, 85 },
    { 0x7eea48df, 90 },
    { 0xbe8a48e1, 43 },
    { 0x9f1a48e0, 85 },
    { 0x299a48e3, 61 },
    { 0, 0 },
    { 0x814a48e5, 52 },
    { 0xff7a48e6, 67 },
    { 0x70da48e6, 61 },
    { 0x199a48e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca48ec, 81 },
    { 0x8e9a48ed, 33 },
    { 0x833a48ee, 98 },
    { 0x889a48ef, 34 },
    { 0x5e8a48ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbba48f7, 61 },
    { 0x8cda48f8, 23 },
    { 0xe2a48f9, 90 },
    { 0, 0 },
    { 0x86ba48fb, 61 },
    { 0x2daa48fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39a4901, 95 },
    { 0, 0 },
    { 0xa28a4903, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632a490a, 87 },
    { 0x545a490b, 33 },
    { 0, 0 },
    { 0x9cea490d, 66 },
    { 0xc01a490d, 23 },
    { 0x32ca490f, 67 },
    { 0x37ea4910, 90 },
    { 0x2d0a4911, 90 },
    { 0x274a4911, 95 },
    { 0x51ea4913, 76 },
    { 0x172a490f, 90 },
    { 0x38fa490e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5a4919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793a491d, 33 },
    { 0x878a491e, 90 },
    { 0xf19a491f, 95 },
    { 0x86fa491f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5a4925, 90 },
    { 0x13ca4925, 87 },
    { 0x28fa4925, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafa4930, 61 },
    { 0x250a4931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a4935, 46 },
    { 0xcf3a4935, 33 },
    { 0xd6a4936, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da493b, 67 },
    { 0, 0 },
    { 0xbe2a493d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eaa4941, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ea4945, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a4949, 75 },
    { 0x2a2a4949, 81 },
    { 0xd2da494b, 40 },
    { 0x595a494c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a4950, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7a4954, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aa4957, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda495a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2a495e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45a4962, 55 },
    { 0xafea4962, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37a496c, 77 },
    { 0, 0 },
    { 0x128a496e, 46 },
    { 0x227a496e, 98 },
    { 0x9da496f, 95 },
    { 0x72ea4971, 43 },
    { 0, 0 },
    { 0xe0da4973, 33 },
    { 0, 0 },
    { 0xb77a4975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fa4979, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a4983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1a4989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfa4992, 87 },
    { 0x669a4993, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5a4999, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faa499c, 34 },
    { 0x4f7a499d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5da49a3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72a49ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea49b2, 38 },
    { 0, 0 },
    { 0xeeda49b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a49ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67da49be, 69 },
    { 0x216a49bf, 66 },
    { 0, 0 },
    { 0x9ea49c1, 33 },
    { 0x2aaa49c2, 55 },
    { 0x2d5a49c3, 66 },
    { 0xbdfa49c4, 52 },
    { 0, 0 },
    { 0xdd8a49c6, 98 },
    { 0x6a9a49c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa49cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899a49d0, 33 },
    { 0x260a49d1, 95 },
    { 0x957a49d1, 84 },
    { 0, 0 },
    { 0x9dfa49d4, 61 },
    { 0xe48a49d5, 33 },
    { 0xf7da49d5, 23 },
    { 0x56ba49d7, 67 },
    { 0, 0 },
    { 0x160a49d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba49dc, 21 },
    { 0, 0 },
    { 0x93ba49de, 98 },
    { 0xbcaa49df, 81 },
    { 0, 0 },
    { 0x80da49e1, 77 },
    { 0x73da49e1, 65 },
    { 0x195a49e3, 43 },
    { 0x8cea49e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705a49e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26a49f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xffea49f3, 85 },
    { 0, 0 },
    { 0x70fa49f5, 23 },
    { 0x100a49f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5a49f9, 66 },
    { 0, 0 },
    { 0x55fa49fb, 67 },
    { 0, 0 },
    { 0x56ca49fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622a4a04, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fa4a07, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9a4a11, 61 },
    { 0xd00a4a12, 91 },
    { 0xeaaa4a13, 46 },
    { 0xb85a4a11, 40 },
    { 0xce2a4a14, 98 },
    { 0x545a4a16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a4a1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aa4a21, 23 },
    { 0, 0 },
    { 0x512a4a23, 33 },
    { 0, 0 },
    { 0x49a4a25, 93 },
    { 0xcd1a4a26, 78 },
    { 0, 0 },
    { 0x41ea4a28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73a4a2d, 90 },
    { 0x792a4a2e, 23 },
    { 0x488a4a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a4a32, 23 },
    { 0xe89a4a33, 23 },
    { 0xf4da4a33, 99 },
    { 0x33a4a35, 95 },
    { 0, 0 },
    { 0x775a4a37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdca4a3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aea4a3f, 95 },
    { 0x2f7a4a40, 90 },
    { 0xef3a4a41, 81 },
    { 0x7b0a4a42, 90 },
    { 0x38ea4a43, 66 },
    { 0x3d1a4a44, 77 },
    { 0xd4ea4a44, 95 },
    { 0x95da4a40, 91 },
    { 0x17fa4a47, 23 },
    { 0x93ea4a44, 46 },
    { 0, 0 },
    { 0xd08a4a4a, 99 },
    { 0x70fa4a4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x185a4a4e, 65 },
    { 0x1f7a4a4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a4a54, 98 },
    { 0xc50a4a55, 61 },
    { 0xfeba4a55, 95 },
    { 0x450a4a57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3a4a5a, 3 },
    { 0, 0 },
    { 0xb20a4a5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9a4a5f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204a4a63, 33 },
    { 0x21fa4a63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01a4a68, 90 },
    { 0x9b8a4a69, 99 },
    { 0xf9a4a6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2a4a6d, 98 },
    { 0, 0 },
    { 0xd9ea4a6f, 91 },
    { 0, 0 },
    { 0xa2ca4a71, 23 },
    { 0, 0 },
    { 0x82ba4a73, 46 },
    { 0xabfa4a74, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380a4a79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645a4a7f, 93 },
    { 0x698a4a80, 90 },
    { 0x9d4a4a80, 46 },
    { 0x781a4a81, 61 },
    { 0x330a4a82, 90 },
    { 0xc8aa4a84, 99 },
    { 0xe70a4a82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83a4a91, 46 },
    { 0x2b1a4a92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ea4a96, 90 },
    { 0x404a4a97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9a4a9a, 52 },
    { 0x89da4a9a, 33 },
    { 0xee4a4a9c, 95 },
    { 0x90ba4a9c, 81 },
    { 0x675a4a9c, 90 },
    { 0x436a4a9a, 95 },
    { 0xa46a4aa0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91a4aa3, 68 },
    { 0xf03a4aa3, 90 },
    { 0xf84a4aa5, 97 },
    { 0x827a4aa5, 97 },
    { 0x6a4a4aa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca4aab, 98 },
    { 0x504a4aac, 93 },
    { 0x3f2a4aab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5a4ab1, 97 },
    { 0, 0 },
    { 0x4c7a4ab3, 23 },
    { 0x757a4ab4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00a4ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8a4aba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73a4ac1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ca4ac4, 95 },
    { 0xe5a4ac5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836a4ac9, 99 },
    { 0xac4a4aca, 100 },
    { 0x627a4aca, 95 },
    { 0xfa4a4acb, 33 },
    { 0xb8ba4acb, 34 },
    { 0xfb0a4ace, 95 },
    { 0x84aa4aca, 98 },
    { 0x930a4ad0, 90 },
    { 0x603a4ad1, 33 },
    { 0, 0 },
    { 0x7b1a4ad3, 98 },
    { 0x6b6a4ad4, 98 },
    { 0, 0 },
    { 0x9fa4ad6, 33 },
    { 0xa75a4ad7, 93 },
    { 0, 0 },
    { 0x9b3a4ad9, 23 },
    { 0xf87a4ada, 98 },
    { 0x508a4ad9, 43 },
    { 0xb07a4adb, 40 },
    { 0x62ca4adc, 93 },
    { 0xa17a4ade, 61 },
    { 0x91da4adb, 33 },
    { 0xc25a4ada, 95 },
    { 0x85ba4ade, 61 },
    { 0x943a4ae1, 61 },
    { 0x376a4ad9, 23 },
    { 0xb1ba4ad9, 93 },
    { 0, 0 },
    { 0x82ca4ae6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ca4aea, 65 },
    { 0, 0 },
    { 0xfc4a4aec, 33 },
    { 0x668a4aec, 61 },
    { 0x7d8a4aee, 43 },
    { 0xb9fa4aef, 95 },
    { 0xf3ca4aee, 61 },
    { 0x276a4aee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a4af5, 33 },
    { 0, 0 },
    { 0xf5ea4af7, 95 },
    { 0x7e9a4af8, 33 },
    { 0xa3ca4af9, 76 },
    { 0xdc1a4afa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eba4afd, 98 },
    { 0xcaaa4afe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1a4b03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaa4b0b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eba4b15, 61 },
    { 0, 0 },
    { 0xc3ca4b17, 33 },
    { 0x533a4b18, 97 },
    { 0xe6da4b18, 53 },
    { 0xb52a4b1a, 76 },
    { 0xa4ca4b1a, 81 },
    { 0xdfea4b1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8a4b21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94a4b25, 90 },
    { 0x64aa4b25, 43 },
    { 0x10fa4b27, 76 },
    { 0, 0 },
    { 0xad8a4b29, 23 },
    { 0, 0 },
    { 0xf45a4b2b, 61 },
    { 0, 0 },
    { 0x209a4b2d, 17 },
    { 0x69aa4b2e, 55 },
    { 0x318a4b2f, 85 },
    { 0xd40a4b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6a4b36, 23 },
    { 0, 0 },
    { 0xa0fa4b38, 61 },
    { 0x6e0a4b39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6a4b3d, 76 },
    { 0, 0 },
    { 0x7f0a4b3f, 22 },
    { 0xf8ba4b40, 90 },
    { 0, 0 },
    { 0x2b3a4b42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743a4b46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaa4b4d, 40 },
    { 0x4aaa4b4d, 98 },
    { 0, 0 },
    { 0x70fa4b50, 100 },
    { 0xc26a4b51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a4b57, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a4b5e, 23 },
    { 0, 0 },
    { 0x6f7a4b60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a4b68, 93 },
    { 0xb7da4b69, 90 },
    { 0x370a4b69, 33 },
    { 0x127a4b6b, 98 },
    { 0x515a4b6c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a4b71, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ea4b74, 84 },
    { 0x27a4b74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ba4b7a, 17 },
    { 0, 0 },
    { 0xb76a4b7c, 95 },
    { 0xcb4a4b7d, 90 },
    { 0, 0 },
    { 0xcfaa4b7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4a4b83, 56 },
    { 0x8d6a4b84, 33 },
    { 0x39ba4b85, 90 },
    { 0, 0 },
    { 0x9ca4b87, 68 },
    { 0, 0 },
    { 0x9a9a4b89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26a4b8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85a4b94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36a4b98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a4ba1, 33 },
    { 0x33ba4ba2, 34 },
    { 0x921a4ba3, 39 },
    { 0, 0 },
    { 0xcd2a4ba5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46da4ba9, 68 },
    { 0xf1ea4baa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9a4bad, 98 },
    { 0xa22a4bae, 43 },
    { 0x9fa4baf, 90 },
    { 0x59fa4bb0, 98 },
    { 0xa78a4bb1, 84 },
    { 0x731a4bb2, 66 },
    { 0xbaaa4bad, 46 },
    { 0xeb0a4bb4, 90 },
    { 0x9f5a4bb4, 76 },
    { 0x8bca4bb5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6a4bbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21a4bbe, 33 },
    { 0x5bea4bbf, 95 },
    { 0xb49a4bc0, 23 },
    { 0xb72a4bc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5a4bc9, 93 },
    { 0x68ca4bca, 98 },
    { 0xbd1a4bca, 100 },
    { 0xc79a4bcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a4bd0, 55 },
    { 0xdd4a4bd0, 90 },
    { 0, 0 },
    { 0x78ca4bd3, 90 },
    { 0xee4a4bd4, 88 },
    { 0xde4a4bd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aa4bd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50a4bde, 95 },
    { 0x31ea4bdf, 98 },
    { 0, 0 },
    { 0xb33a4be1, 91 },
    { 0, 0 },
    { 0x30ca4be3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda4be7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a4bee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11a4bf1, 90 },
    { 0x24fa4bf2, 33 },
    { 0xf7ba4bf2, 90 },
    { 0, 0 },
    { 0x7aaa4bf5, 61 },
    { 0, 0 },
    { 0xfeea4bf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa4bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ca4bfd, 61 },
    { 0x3b2a4bfd, 43 },
    { 0, 0 },
    { 0xaf6a4c00, 43 },
    { 0, 0 },
    { 0xfbca4c02, 34 },
    { 0, 0 },
    { 0xc6da4c04, 90 },
    { 0, 0 },
    { 0xa3ea4c06, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2a4c0c, 76 },
    { 0x992a4c0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea4c10, 98 },
    { 0, 0 },
    { 0x854a4c12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a4c18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5a4c1c, 93 },
    { 0, 0 },
    { 0x707a4c1e, 88 },
    { 0x2bba4c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfba4c24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a4c2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ba4c2f, 33 },
    { 0, 0 },
    { 0xf46a4c31, 76 },
    { 0, 0 },
    { 0xd97a4c33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91a4c38, 76 },
    { 0, 0 },
    { 0x377a4c3a, 61 },
    { 0xe87a4c3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0a4c3e, 90 },
    { 0, 0 },
    { 0x15da4c40, 33 },
    { 0x135a4c41, 66 },
    { 0x47ca4c42, 33 },
    { 0x232a4c40, 46 },
    { 0, 0 },
    { 0x71aa4c45, 75 },
    { 0, 0 },
    { 0x16ba4c47, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4a4c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a4c50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5a4c55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a4c59, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5a4c5c, 33 },
    { 0xcbfa4c5d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x847a4c60, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fa4c63, 69 },
    { 0xb12a4c64, 84 },
    { 0x31ea4c64, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba4c69, 23 },
    { 0x760a4c6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23a4c73, 98 },
    { 0x8f9a4c74, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa4c79, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeca4c7f, 90 },
    { 0xdaaa4c80, 75 },
    { 0x3e4a4c81, 93 },
    { 0xbd4a4c82, 61 },
    { 0xf04a4c80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfda4c88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x277a4c8b, 76 },
    { 0, 0 },
    { 0x176a4c8d, 90 },
    { 0, 0 },
    { 0xfaa4c8f, 66 },
    { 0, 0 },
    { 0x477a4c91, 91 },
    { 0xad6a4c92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fa4c95, 78 },
    { 0x7ffa4c95, 34 },
    { 0, 0 },
    { 0xc94a4c98, 68 },
    { 0, 0 },
    { 0x810a4c9a, 33 },
    { 0xb1ea4c9b, 38 },
    { 0xdf9a4c9c, 91 },
    { 0xfe5a4c9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a4ca4, 43 },
    { 0xe40a4ca5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a4ca8, 85 },
    { 0x40da4ca8, 95 },
    { 0xf12a4ca8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a4cb1, 98 },
    { 0x1a4cb2, 98 },
    { 0x872a4cb1, 33 },
    { 0, 0 },
    { 0x580a4cb5, 33 },
    { 0xb32a4cb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3a4cba, 23 },
    { 0xcdba4cba, 97 },
    { 0x383a4cbc, 87 },
    { 0xdcea4cbc, 33 },
    { 0, 0 },
    { 0xe39a4cbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4a4cc6, 33 },
    { 0x590a4cc7, 85 },
    { 0x11da4cc6, 34 },
    { 0, 0 },
    { 0x60fa4cca, 90 },
    { 0, 0 },
    { 0x36ba4ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45a4cd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a4cd5, 67 },
    { 0xb91a4cd6, 43 },
    { 0x153a4cd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518a4ce4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8a4ce8, 33 },
    { 0x375a4ce8, 95 },
    { 0x12ca4ce8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55a4cee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5a4cf2, 34 },
    { 0, 0 },
    { 0x9e7a4cf4, 46 },
    { 0x58da4cf5, 93 },
    { 0x753a4cf6, 87 },
    { 0x30da4cf7, 61 },
    { 0xb9fa4cf8, 55 },
    { 0, 0 },
    { 0x23ca4cfa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549a4cfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x168a4d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0a4d08, 61 },
    { 0, 0 },
    { 0x754a4d0a, 45 },
    { 0, 0 },
    { 0xed9a4d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca4d10, 33 },
    { 0x55fa4d11, 90 },
    { 0x78ca4d12, 82 },
    { 0x4fba4d12, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4da4d17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a4d1d, 61 },
    { 0, 0 },
    { 0x390a4d1f, 93 },
    { 0, 0 },
    { 0x803a4d21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02a4d26, 90 },
    { 0xd0fa4d27, 34 },
    { 0x626a4d28, 23 },
    { 0x28ea4d27, 34 },
    { 0, 0 },
    { 0x48da4d2b, 61 },
    { 0x157a4d2c, 90 },
    { 0xe60a4d2d, 95 },
    { 0x38ba4d2e, 90 },
    { 0x7efa4d2f, 33 },
    { 0x5bea4d2f, 33 },
    { 0x8eda4d2f, 76 },
    { 0xd5fa4d32, 98 },
    { 0x809a4d30, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45a4d38, 33 },
    { 0x5c8a4d39, 95 },
    { 0xa54a4d3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39a4d3d, 38 },
    { 0, 0 },
    { 0x503a4d3f, 67 },
    { 0x723a4d3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffa4d4a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ea4d4e, 33 },
    { 0xe9a4d4f, 43 },
    { 0xf3ca4d4f, 23 },
    { 0x350a4d50, 95 },
    { 0x961a4d52, 95 },
    { 0xc5ba4d53, 90 },
    { 0xe0ba4d52, 90 },
    { 0xf7fa4d55, 76 },
    { 0xfd6a4d53, 66 },
    { 0x9cda4d56, 34 },
    { 0x502a4d55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734a4d5c, 84 },
    { 0x8aba4d5c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x251a4d60, 68 },
    { 0, 0 },
    { 0xb32a4d62, 33 },
    { 0, 0 },
    { 0xc72a4d64, 98 },
    { 0x988a4d64, 33 },
    { 0x9e6a4d65, 93 },
    { 0, 0 },
    { 0xec4a4d68, 95 },
    { 0x9a6a4d68, 23 },
    { 0, 0 },
    { 0x9d0a4d6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda4d6f, 68 },
    { 0xf3da4d6f, 23 },
    { 0x891a4d71, 81 },
    { 0xe72a4d72, 76 },
    { 0x30aa4d71, 90 },
    { 0x403a4d73, 23 },
    { 0x6cea4d6f, 90 },
    { 0x9aba4d71, 98 },
    { 0xb8aa4d76, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x97a4d7a, 99 },
    { 0, 0 },
    { 0x1c7a4d7c, 60 },
    { 0xa33a4d7c, 61 },
    { 0x8a7a4d7d, 43 },
    { 0xc91a4d7d, 90 },
    { 0x4c6a4d80, 55 },
    { 0, 0 },
    { 0xff2a4d82, 90 },
    { 0, 0 },
    { 0xe39a4d84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2a4d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba4d8c, 55 },
    { 0xff0a4d8d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x71a4d90, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5a4d93, 78 },
    { 0, 0 },
    { 0x175a4d95, 90 },
    { 0x6e7a4d96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556a4d9e, 68 },
    { 0xf43a4d9e, 33 },
    { 0x66ca4d9e, 55 },
    { 0x467a4da1, 84 },
    { 0x5d6a4da2, 100 },
    { 0xf04a4da2, 95 },
    { 0xd23a4da1, 95 },
    { 0xd6fa4da5, 77 },
    { 0x1daa4d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffa4da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aea4daf, 33 },
    { 0, 0 },
    { 0x6fea4db1, 95 },
    { 0xaeaa4db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88aa4db8, 76 },
    { 0xa67a4db8, 90 },
    { 0, 0 },
    { 0xc21a4dbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82a4dbf, 67 },
    { 0x80ea4dc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cca4dc9, 76 },
    { 0x3dda4dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda4dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5a4dd2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386a4dd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaa4ddd, 93 },
    { 0x745a4ddd, 61 },
    { 0xa0ba4ddf, 61 },
    { 0, 0 },
    { 0x537a4de1, 33 },
    { 0xcf9a4de2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45a4de5, 77 },
    { 0xf32a4de5, 88 },
    { 0x534a4de7, 68 },
    { 0x5dea4de8, 90 },
    { 0, 0 },
    { 0xae8a4dea, 85 },
    { 0x954a4dea, 55 },
    { 0x8a4dec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9a4df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x323a4df7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ca4dfc, 61 },
    { 0x767a4dfd, 61 },
    { 0xfd5a4dfc, 61 },
    { 0xecfa4dfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x377a4e02, 90 },
    { 0x8d3a4e03, 77 },
    { 0, 0 },
    { 0x3ffa4e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4e0a, 90 },
    { 0x5fda4e0a, 87 },
    { 0xc1ba4e0a, 33 },
    { 0xa97a4e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x910a4e10, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403a4e14, 98 },
    { 0x31ba4e15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53a4e1e, 23 },
    { 0xb08a4e1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55a4e23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ea4e28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347a4e2d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aca4e35, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccda4e3a, 100 },
    { 0x598a4e3b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7da4e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82a4e45, 95 },
    { 0xccca4e46, 61 },
    { 0x2d0a4e47, 78 },
    { 0, 0 },
    { 0xd03a4e49, 95 },
    { 0x14ea4e4a, 61 },
    { 0, 0 },
    { 0xaf6a4e4c, 90 },
    { 0x15aa4e4d, 95 },
    { 0xa6ea4e4e, 100 },
    { 0x50da4e4f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6a4e62, 33 },
    { 0, 0 },
    { 0xad6a4e64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ba4e6c, 33 },
    { 0, 0 },
    { 0x8e6a4e6e, 23 },
    { 0x3dba4e6f, 55 },
    { 0, 0 },
    { 0x75aa4e71, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771a4e75, 33 },
    { 0, 0 },
    { 0xa86a4e77, 34 },
    { 0, 0 },
    { 0x546a4e79, 78 },
    { 0, 0 },
    { 0xaeca4e7b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203a4e7f, 61 },
    { 0, 0 },
    { 0x8dfa4e81, 76 },
    { 0x2bca4e82, 61 },
    { 0xcf2a4e83, 33 },
    { 0x528a4e83, 33 },
    { 0x606a4e85, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a4e8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99a4e93, 46 },
    { 0, 0 },
    { 0xfe1a4e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa4e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcba4e9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa4ea0, 98 },
    { 0, 0 },
    { 0x86ea4ea2, 87 },
    { 0, 0 },
    { 0x91da4ea4, 90 },
    { 0x677a4ea4, 33 },
    { 0x5c2a4ea6, 100 },
    { 0xadda4ea6, 90 },
    { 0xe1da4ea8, 86 },
    { 0x4d7a4ea9, 61 },
    { 0xb8fa4eaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a4eb1, 76 },
    { 0, 0 },
    { 0x86ba4eb3, 98 },
    { 0xc9a4eb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62a4eba, 98 },
    { 0xcaa4ebb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733a4ebf, 95 },
    { 0, 0 },
    { 0xd41a4ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6a4ec5, 61 },
    { 0x42fa4ec6, 93 },
    { 0x9a8a4ec7, 98 },
    { 0x685a4ec8, 98 },
    { 0xf8a4ec9, 55 },
    { 0xd74a4eca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecca4ecf, 11 },
    { 0x5c2a4ed0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475a4ed4, 33 },
    { 0xb04a4ed5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfa4ed8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3a4edb, 23 },
    { 0x99fa4edc, 33 },
    { 0x796a4edd, 90 },
    { 0xc06a4ede, 23 },
    { 0xb10a4edf, 61 },
    { 0x5afa4edf, 76 },
    { 0x7c8a4ee0, 75 },
    { 0xd26a4ee2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80a4ee6, 48 },
    { 0x487a4ee7, 55 },
    { 0x461a4ee8, 55 },
    { 0x92ea4ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bba4eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354a4ef1, 90 },
    { 0xf73a4ef2, 90 },
    { 0, 0 },
    { 0xbfba4ef4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x994a4ef7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2a4efa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ca4f05, 23 },
    { 0x490a4f05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ba4f09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fa4f0c, 33 },
    { 0xc71a4f0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79da4f12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ba4f16, 98 },
    { 0x46a4f17, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbea4f1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfa4f1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0a4f21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fa4f24, 67 },
    { 0x8eaa4f25, 76 },
    { 0x4b0a4f26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80a4f2b, 90 },
    { 0xc32a4f2b, 61 },
    { 0x4c9a4f2c, 90 },
    { 0x58fa4f2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244a4f32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a4f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ba4f3e, 95 },
    { 0, 0 },
    { 0x68ca4f40, 91 },
    { 0, 0 },
    { 0xe96a4f42, 98 },
    { 0, 0 },
    { 0xae2a4f44, 85 },
    { 0x90ba4f44, 53 },
    { 0xd71a4f46, 43 },
    { 0x887a4f44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90a4f4b, 80 },
    { 0xddea4f4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6a4f50, 98 },
    { 0x54fa4f50, 23 },
    { 0, 0 },
    { 0xc1aa4f53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1a4f58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9a4f5c, 61 },
    { 0xe74a4f5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7a4f63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x923a4f66, 90 },
    { 0xad6a4f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdda4f6b, 90 },
    { 0, 0 },
    { 0x478a4f6d, 90 },
    { 0xa5aa4f6e, 33 },
    { 0xc29a4f6e, 98 },
    { 0x80ba4f70, 61 },
    { 0x66fa4f70, 43 },
    { 0x1e3a4f71, 43 },
    { 0x558a4f73, 69 },
    { 0x2d0a4f74, 23 },
    { 0x66ba4f74, 33 },
    { 0x15ba4f72, 98 },
    { 0x371a4f6e, 77 },
    { 0x82ba4f78, 61 },
    { 0, 0 },
    { 0x727a4f7a, 43 },
    { 0x3e7a4f7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a4f89, 34 },
    { 0x436a4f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa4f8d, 43 },
    { 0x967a4f8e, 98 },
    { 0x7c9a4f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba4f95, 95 },
    { 0, 0 },
    { 0xcc4a4f97, 100 },
    { 0xd7fa4f98, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81a4fa2, 98 },
    { 0xdcda4fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7da4faa, 76 },
    { 0x117a4faa, 95 },
    { 0, 0 },
    { 0x62ca4fad, 55 },
    { 0x1c6a4fae, 61 },
    { 0x20ba4faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467a4fb4, 98 },
    { 0, 0 },
    { 0xdd6a4fb6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca4fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca4fd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1a4fd3, 87 },
    { 0, 0 },
    { 0xa53a4fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a4fda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959a4fe0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xada4fe5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911a4fea, 98 },
    { 0xb0da4feb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdda4fee, 33 },
    { 0x7f3a4fef, 90 },
    { 0xe3fa4fee, 95 },
    { 0xdeea4ff1, 96 },
    { 0x7ca4ff2, 61 },
    { 0, 0 },
    { 0x24ba4ff4, 90 },
    { 0xca2a4ff5, 55 },
    { 0xf4ba4ff6, 90 },
    { 0xd31a4ff4, 67 },
    { 0x7eca4ff6, 98 },
    { 0x1e8a4ff4, 52 },
    { 0, 0 },
    { 0xe92a4ffb, 76 },
    { 0x176a4ffc, 55 },
    { 0, 0 },
    { 0x747a4ffe, 76 },
    { 0, 0 },
    { 0x9eda5000, 55 },
    { 0xdeba5000, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aea5004, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2a5007, 100 },
    { 0x945a5008, 33 },
    { 0x507a5009, 90 },
    { 0xbeba500a, 95 },
    { 0, 0 },
    { 0x5e6a500c, 33 },
    { 0x240a500d, 61 },
    { 0x594a500d, 90 },
    { 0, 0 },
    { 0x57a5010, 61 },
    { 0xa91a5011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafa5014, 61 },
    { 0, 0 },
    { 0x6f9a5016, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fa5019, 46 },
    { 0xae7a501a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07a5020, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a5023, 23 },
    { 0, 0 },
    { 0x4bda5025, 33 },
    { 0, 0 },
    { 0xc4da5027, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21da502c, 33 },
    { 0x10aa502c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ba5030, 33 },
    { 0xec3a5030, 67 },
    { 0xef8a5031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fa5035, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719a5039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7a5043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x816a5046, 93 },
    { 0xd44a5046, 61 },
    { 0xdeda5048, 33 },
    { 0xc0fa5046, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8a504d, 61 },
    { 0x275a504d, 85 },
    { 0x46ca504e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63a5053, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aa505b, 34 },
    { 0x611a505c, 36 },
    { 0xb1ba505d, 23 },
    { 0x60da505d, 61 },
    { 0x1a7a505d, 76 },
    { 0x73ca5060, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baa5067, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41da506a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffa506e, 61 },
    { 0, 0 },
    { 0x7a5070, 37 },
    { 0x694a5070, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8a5077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ba507e, 84 },
    { 0x724a507e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915a5085, 100 },
    { 0x194a5085, 61 },
    { 0xdada5087, 98 },
    { 0x6e2a5088, 33 },
    { 0xfc1a5089, 93 },
    { 0x810a5089, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x560a508d, 87 },
    { 0x730a508e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317a5092, 76 },
    { 0xc13a5092, 61 },
    { 0x92da5094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0a5098, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497a509e, 95 },
    { 0x793a509e, 95 },
    { 0xa12a50a0, 91 },
    { 0x10ba50a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bea50a4, 95 },
    { 0xb41a50a4, 61 },
    { 0, 0 },
    { 0xf5fa50a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a50ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4a50b1, 61 },
    { 0xe21a50b2, 78 },
    { 0x9f0a50b2, 52 },
    { 0x71da50b2, 43 },
    { 0x52ba50b5, 61 },
    { 0x228a50b4, 98 },
    { 0xcb6a50b2, 90 },
    { 0x1d7a50b8, 99 },
    { 0x6d3a50b4, 90 },
    { 0, 0 },
    { 0x46ca50bb, 76 },
    { 0xea6a50bb, 34 },
    { 0, 0 },
    { 0x10ca50be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6a50c2, 95 },
    { 0xd77a50c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a50c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaa50cb, 66 },
    { 0, 0 },
    { 0x512a50cd, 61 },
    { 0, 0 },
    { 0xa53a50cf, 55 },
    { 0x82ea50d0, 95 },
    { 0xd3a50d1, 67 },
    { 0x964a50d1, 34 },
    { 0, 0 },
    { 0x3afa50d4, 98 },
    { 0, 0 },
    { 0x473a50d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a50d9, 29 },
    { 0x124a50da, 46 },
    { 0xb9ea50da, 95 },
    { 0, 0 },
    { 0x380a50dd, 95 },
    { 0, 0 },
    { 0x260a50df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59a50e2, 93 },
    { 0x42aa50e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920a50ea, 78 },
    { 0xca2a50ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66da50ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ba50f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18a50f5, 95 },
    { 0x2b6a50f6, 100 },
    { 0xf0ba50f7, 23 },
    { 0x69fa50f5, 90 },
    { 0, 0 },
    { 0xcd7a50fa, 76 },
    { 0x4dba50fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8a50fe, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca5101, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1a5104, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301a5108, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da510c, 99 },
    { 0x443a510d, 33 },
    { 0, 0 },
    { 0x480a510f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9aa511c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x36aa511f, 46 },
    { 0x3c0a5120, 98 },
    { 0xe5ea5121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211a5127, 98 },
    { 0xafaa5128, 23 },
    { 0, 0 },
    { 0x52da512a, 90 },
    { 0xb43a512b, 64 },
    { 0x1b3a512c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121a5131, 61 },
    { 0xfbca5131, 61 },
    { 0, 0 },
    { 0xbb7a5134, 34 },
    { 0xdf1a5135, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87a513a, 98 },
    { 0, 0 },
    { 0x649a513c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfa5141, 43 },
    { 0x281a5141, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x171a5145, 90 },
    { 0xfe2a5146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ea5153, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefba5157, 76 },
    { 0, 0 },
    { 0x696a5159, 90 },
    { 0, 0 },
    { 0x39ea515b, 85 },
    { 0xc8a515b, 23 },
    { 0, 0 },
    { 0xf96a515e, 90 },
    { 0, 0 },
    { 0x4aca5160, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a5169, 98 },
    { 0x47ea516a, 43 },
    { 0xe5ea516b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2a516f, 95 },
    { 0xc61a5170, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafda5173, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ca5176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46a517b, 55 },
    { 0xfdea517b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17da5180, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48a5183, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6a5187, 33 },
    { 0x41ba5188, 84 },
    { 0xfeca5188, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa5190, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6a5195, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54a519b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14a519f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17a51a2, 90 },
    { 0, 0 },
    { 0x981a51a4, 93 },
    { 0xf34a51a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca51ac, 95 },
    { 0x5c8a51ad, 90 },
    { 0, 0 },
    { 0x61ca51af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a51b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201a51b6, 34 },
    { 0, 0 },
    { 0xccfa51b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ca51bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13a51c0, 43 },
    { 0xdfba51c0, 33 },
    { 0x303a51c2, 69 },
    { 0, 0 },
    { 0xe67a51c4, 90 },
    { 0, 0 },
    { 0x24a51c6, 88 },
    { 0xa83a51c6, 88 },
    { 0xc5ca51c6, 34 },
    { 0x4faa51c9, 23 },
    { 0x551a51c9, 23 },
    { 0x5f8a51cb, 61 },
    { 0x9f5a51ca, 61 },
    { 0x1eba51cd, 98 },
    { 0xa2fa51cd, 98 },
    { 0x1dba51cf, 91 },
    { 0x5daa51cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0da51d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4aa51da, 23 },
    { 0xc50a51db, 76 },
    { 0, 0 },
    { 0x7da51dd, 61 },
    { 0x5bda51dd, 33 },
    { 0, 0 },
    { 0xde6a51e0, 95 },
    { 0xe5aa51e0, 33 },
    { 0x21a51e2, 101 },
    { 0, 0 },
    { 0x9f3a51e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea51e8, 98 },
    { 0, 0 },
    { 0xf36a51ea, 66 },
    { 0xef2a51eb, 99 },
    { 0xc97a51ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bba51f2, 66 },
    { 0xffaa51f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5a51fd, 61 },
    { 0xa32a51fe, 93 },
    { 0x386a51ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffa5202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505a5206, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ea520b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca520e, 23 },
    { 0x175a520f, 90 },
    { 0, 0 },
    { 0x3cda5211, 23 },
    { 0xb0ea5211, 98 },
    { 0x57ca5213, 61 },
    { 0xddda5212, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bda5217, 99 },
    { 0, 0 },
    { 0xa41a5219, 23 },
    { 0x281a5219, 33 },
    { 0, 0 },
    { 0xc86a521c, 95 },
    { 0, 0 },
    { 0x813a521e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea5223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x729a5226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ca5229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a522e, 46 },
    { 0x13fa522f, 61 },
    { 0x11ea522e, 33 },
    { 0x224a5231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74a5238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcba523b, 46 },
    { 0xe66a523c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da523f, 76 },
    { 0, 0 },
    { 0x9e0a5241, 61 },
    { 0x848a5242, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72aa5247, 90 },
    { 0xf65a5248, 98 },
    { 0xa20a5248, 98 },
    { 0x3b5a524a, 93 },
    { 0, 0 },
    { 0xafba524c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72a5250, 90 },
    { 0x2f5a5251, 65 },
    { 0, 0 },
    { 0x82ca5253, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaa5256, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ea525d, 65 },
    { 0xab5a525e, 97 },
    { 0xf48a525f, 90 },
    { 0xcb4a5260, 91 },
    { 0x8dba5260, 70 },
    { 0xf43a525e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599a5266, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a526b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7a5271, 90 },
    { 0x690a5271, 78 },
    { 0x86ea5273, 95 },
    { 0x8e1a5274, 90 },
    { 0xe2a5275, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51a527c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2a5287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400a528b, 61 },
    { 0xe6ca528c, 84 },
    { 0x261a528d, 33 },
    { 0, 0 },
    { 0x8c6a528f, 76 },
    { 0, 0 },
    { 0x486a5291, 98 },
    { 0x7fa5291, 90 },
    { 0, 0 },
    { 0x14ba5294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a5297, 88 },
    { 0xe07a5298, 61 },
    { 0xc83a5298, 33 },
    { 0x47ea5297, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2a529e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfa52a3, 78 },
    { 0x3c2a52a4, 52 },
    { 0x5d5a52a5, 55 },
    { 0x569a52a5, 93 },
    { 0x7dda52a7, 65 },
    { 0x954a52a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33a52ac, 81 },
    { 0x2bea52ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947a52b2, 33 },
    { 0x921a52b3, 33 },
    { 0x6cba52b3, 91 },
    { 0x560a52b3, 23 },
    { 0xa98a52b6, 98 },
    { 0x3b2a52b7, 91 },
    { 0x49aa52b8, 90 },
    { 0, 0 },
    { 0x222a52ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74a52c1, 23 },
    { 0x816a52c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24a52c7, 19 },
    { 0x974a52c8, 91 },
    { 0x8afa52c8, 90 },
    { 0x7d0a52ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba52cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44a52d1, 33 },
    { 0xec1a52d2, 33 },
    { 0x9a2a52d3, 33 },
    { 0x236a52d3, 65 },
    { 0x791a52d2, 53 },
    { 0x7aca52d1, 23 },
    { 0, 0 },
    { 0xaa3a52d8, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ea52e2, 90 },
    { 0x183a52e3, 35 },
    { 0x6a52e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185a52e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fa52f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefa52f4, 23 },
    { 0, 0 },
    { 0xfdfa52f6, 98 },
    { 0x3fda52f6, 55 },
    { 0, 0 },
    { 0xecaa52f9, 23 },
    { 0xc55a52f9, 95 },
    { 0x649a52fb, 61 },
    { 0, 0 },
    { 0xb85a52fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aa5300, 76 },
    { 0, 0 },
    { 0xaa1a5302, 98 },
    { 0, 0 },
    { 0x9cea5304, 53 },
    { 0x8fca5304, 95 },
    { 0x41fa5305, 95 },
    { 0, 0 },
    { 0x54aa5308, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2a530e, 61 },
    { 0xd6ba530f, 55 },
    { 0x5ffa530f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fa5314, 78 },
    { 0x57fa5315, 77 },
    { 0xe7ca5316, 90 },
    { 0x26a5317, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5a531b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6a5321, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8a5326, 46 },
    { 0x515a5326, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dba532b, 95 },
    { 0x23ca532c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1a532f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a5332, 67 },
    { 0xb13a5333, 61 },
    { 0x127a5332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6a5338, 76 },
    { 0, 0 },
    { 0xdcda533a, 85 },
    { 0xd69a533a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fa5341, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ca534c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ba534f, 90 },
    { 0, 0 },
    { 0x687a5351, 52 },
    { 0, 0 },
    { 0x237a5353, 66 },
    { 0x2c2a5354, 85 },
    { 0x7b9a5354, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578a535d, 84 },
    { 0xebba535e, 91 },
    { 0x898a535f, 34 },
    { 0x33ca535f, 23 },
    { 0, 0 },
    { 0x665a5362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ea5366, 33 },
    { 0x8e1a5366, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a536c, 43 },
    { 0xc51a536d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12a5370, 69 },
    { 0xc29a5371, 100 },
    { 0, 0 },
    { 0xc6ca5373, 90 },
    { 0x2bda5374, 87 },
    { 0, 0 },
    { 0xc0fa5376, 100 },
    { 0x69fa5377, 76 },
    { 0xa6fa5378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52a537b, 61 },
    { 0, 0 },
    { 0xb3ba537d, 33 },
    { 0, 0 },
    { 0x6ba537f, 93 },
    { 0, 0 },
    { 0xd0da5381, 95 },
    { 0x7a7a5381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5a5385, 61 },
    { 0x2c5a5386, 98 },
    { 0xdb6a5386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2da538d, 53 },
    { 0xad3a538d, 91 },
    { 0x556a538e, 52 },
    { 0x313a5390, 33 },
    { 0x666a538e, 23 },
    { 0xe4a5392, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611a5398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3a539b, 90 },
    { 0x793a539b, 43 },
    { 0, 0 },
    { 0x442a539e, 98 },
    { 0, 0 },
    { 0x8aea53a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417a53a4, 90 },
    { 0xafba53a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244a53a9, 76 },
    { 0x9d7a53aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90a53b2, 90 },
    { 0xb3ea53b3, 34 },
    { 0x359a53b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4a53b9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa53c3, 65 },
    { 0x827a53c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a53c8, 93 },
    { 0xe5aa53c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a53ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a53d3, 46 },
    { 0xc30a53d4, 55 },
    { 0xc6da53d3, 78 },
    { 0x8dba53d5, 33 },
    { 0x556a53d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503a53dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeda53e0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x402a53e3, 23 },
    { 0, 0 },
    { 0x47ca53e5, 90 },
    { 0, 0 },
    { 0xd06a53e7, 33 },
    { 0xc5fa53e8, 23 },
    { 0, 0 },
    { 0xfc9a53ea, 98 },
    { 0xfd9a53ea, 76 },
    { 0, 0 },
    { 0x54ba53ed, 23 },
    { 0xe92a53ed, 33 },
    { 0xd39a53ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716a53f5, 77 },
    { 0xb34a53f6, 76 },
    { 0xf5ca53f7, 52 },
    { 0xc53a53f8, 90 },
    { 0, 0 },
    { 0xdcca53fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0da53fe, 65 },
    { 0xf64a53ff, 98 },
    { 0xa21a5400, 33 },
    { 0, 0 },
    { 0x270a5402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bea540d, 34 },
    { 0, 0 },
    { 0x307a540f, 95 },
    { 0x506a5410, 93 },
    { 0, 0 },
    { 0x729a5412, 93 },
    { 0, 0 },
    { 0xb14a5414, 87 },
    { 0x9b7a5414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5a5418, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ca541c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556a5421, 34 },
    { 0xac4a5421, 98 },
    { 0, 0 },
    { 0xdea5424, 66 },
    { 0x407a5425, 98 },
    { 0x138a5426, 34 },
    { 0x5d9a5424, 77 },
    { 0, 0 },
    { 0x7b0a5429, 90 },
    { 0x251a542a, 95 },
    { 0x31ea542b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448a5430, 55 },
    { 0x475a5431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4a5435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1a5438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba543d, 91 },
    { 0, 0 },
    { 0x999a543f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ba5444, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692a5449, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfa544d, 76 },
    { 0x400a544d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ca5451, 95 },
    { 0x625a5451, 69 },
    { 0x4c7a5453, 40 },
    { 0x2e9a5452, 55 },
    { 0x35a5455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929a545a, 90 },
    { 0x210a545b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56a545e, 61 },
    { 0x29aa545f, 64 },
    { 0, 0 },
    { 0xbf5a5461, 61 },
    { 0x56ba5461, 78 },
    { 0xaba5463, 100 },
    { 0xfaea5463, 98 },
    { 0x5d1a5465, 23 },
    { 0xd1a5463, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ba546d, 75 },
    { 0xbeca546d, 93 },
    { 0x3f6a546f, 65 },
    { 0xc99a546d, 76 },
    { 0x135a546d, 76 },
    { 0xa4aa5472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebca547a, 61 },
    { 0, 0 },
    { 0xb09a547c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4da5483, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6da5486, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187a548a, 43 },
    { 0xe87a548b, 33 },
    { 0x3aa548a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ca548f, 33 },
    { 0, 0 },
    { 0x837a5491, 61 },
    { 0x50ba5492, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5a5496, 61 },
    { 0xce5a5497, 95 },
    { 0, 0 },
    { 0xef5a5499, 66 },
    { 0xa37a5499, 84 },
    { 0x952a549b, 95 },
    { 0xfbaa549a, 82 },
    { 0, 0 },
    { 0x3b3a549e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12a54a1, 61 },
    { 0x7b2a54a2, 90 },
    { 0xd86a54a2, 95 },
    { 0x878a54a3, 100 },
    { 0xee5a54a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89a54aa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baa54ae, 46 },
    { 0x97da54af, 61 },
    { 0xc74a54b0, 67 },
    { 0, 0 },
    { 0x2ba54b2, 90 },
    { 0, 0 },
    { 0xa0da54b4, 95 },
    { 0xcc5a54b5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca54bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112a54c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3a54ca, 46 },
    { 0x70aa54cb, 51 },
    { 0x120a54cc, 90 },
    { 0x882a54cc, 98 },
    { 0x926a54cd, 46 },
    { 0xadda54cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1a54d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a54d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3a54d9, 66 },
    { 0x317a54d9, 95 },
    { 0x69ea54da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa54df, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ca54e8, 90 },
    { 0x598a54e9, 76 },
    { 0, 0 },
    { 0xaf9a54eb, 90 },
    { 0x13fa54ec, 34 },
    { 0x828a54ed, 33 },
    { 0, 0 },
    { 0xae7a54ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7a54f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853a54fa, 23 },
    { 0x234a54fa, 65 },
    { 0xc2a54fc, 68 },
    { 0x660a54fc, 90 },
    { 0x91ea54fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x472a5501, 66 },
    { 0, 0 },
    { 0x9ffa5503, 95 },
    { 0xd72a5503, 90 },
    { 0x7d1a5505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cea5509, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3a550e, 55 },
    { 0x483a550f, 61 },
    { 0, 0 },
    { 0xe5da5511, 33 },
    { 0x318a5512, 33 },
    { 0xe4a5513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52aa5518, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x186a551b, 33 },
    { 0x43da551c, 90 },
    { 0x906a551c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31da5524, 90 },
    { 0x1e1a5525, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aba5528, 43 },
    { 0x3bba5528, 90 },
    { 0x18da552a, 99 },
    { 0xc83a5528, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0a552e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x665a5531, 76 },
    { 0, 0 },
    { 0x8a9a5533, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a553d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959a554a, 61 },
    { 0xa7ca554b, 65 },
    { 0xfe9a554c, 98 },
    { 0x633a554c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda5550, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2aa5553, 88 },
    { 0x404a5554, 61 },
    { 0x376a5555, 99 },
    { 0x878a5555, 97 },
    { 0x88ea5556, 23 },
    { 0x99ba5558, 77 },
    { 0x115a5556, 23 },
    { 0xf63a555a, 10 },
    { 0xee5a555a, 91 },
    { 0, 0 },
    { 0x7ba555d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4aa5560, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40a5566, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561a556a, 52 },
    { 0xd8fa556a, 48 },
    { 0x129a556c, 33 },
    { 0x909a556d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a5570, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6a5573, 98 },
    { 0xf73a5573, 23 },
    { 0x52a5575, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a5578, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaca557c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968a5580, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bba5583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eba5586, 81 },
    { 0x35aa5587, 90 },
    { 0, 0 },
    { 0xbeaa5589, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aa5590, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a559e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fa55a2, 66 },
    { 0, 0 },
    { 0x7dfa55a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1a55ab, 49 },
    { 0xbbca55ac, 71 },
    { 0xa97a55ad, 36 },
    { 0xdc4a55ae, 43 },
    { 0, 0 },
    { 0x9baa55b0, 78 },
    { 0x619a55b1, 97 },
    { 0x73a55b2, 76 },
    { 0x1d7a55b0, 90 },
    { 0x9e0a55b4, 33 },
    { 0x19aa55b5, 78 },
    { 0x8fda55b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86a55ba, 23 },
    { 0xbaa55ba, 90 },
    { 0, 0 },
    { 0x833a55bd, 52 },
    { 0x14ba55be, 98 },
    { 0x928a55bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947a55c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffa55c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36a55cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a55d0, 69 },
    { 0, 0 },
    { 0x3ffa55d2, 90 },
    { 0x944a55d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa55d9, 61 },
    { 0x285a55da, 90 },
    { 0xeda55da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x281a55de, 98 },
    { 0, 0 },
    { 0xb8a55e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bda55e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a55ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677a55f7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74da55fc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a5600, 33 },
    { 0xd09a5601, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa5606, 23 },
    { 0xd99a5607, 91 },
    { 0xe84a5608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19a560e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172a5616, 61 },
    { 0x427a5617, 77 },
    { 0xe4ca5616, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ba561d, 90 },
    { 0x385a561e, 78 },
    { 0xc6ba561e, 98 },
    { 0x3f5a5620, 90 },
    { 0x6bfa561e, 91 },
    { 0x2b7a5622, 46 },
    { 0xd55a5623, 100 },
    { 0, 0 },
    { 0x366a5625, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a562a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371a562f, 52 },
    { 0xa37a5630, 95 },
    { 0x3c2a5630, 93 },
    { 0x15ba5632, 33 },
    { 0xd5ba5633, 98 },
    { 0xe75a5631, 33 },
    { 0, 0 },
    { 0xb4fa5636, 69 },
    { 0xb76a5637, 33 },
    { 0x159a5638, 55 },
    { 0xc8ba5639, 95 },
    { 0xb07a563a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6a563f, 95 },
    { 0, 0 },
    { 0x137a5641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ca5645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea5648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa564b, 52 },
    { 0x39fa564c, 48 },
    { 0, 0 },
    { 0x184a564e, 76 },
    { 0, 0 },
    { 0x444a5650, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ea5654, 90 },
    { 0x9aa5655, 61 },
    { 0x650a5656, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a565b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a565e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a5661, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fa5664, 90 },
    { 0x9b3a5664, 95 },
    { 0x1dba5666, 98 },
    { 0x684a5665, 33 },
    { 0x1caa5668, 33 },
    { 0x569a5665, 33 },
    { 0x719a5667, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaa566d, 84 },
    { 0x5d7a566d, 98 },
    { 0x725a566e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a5673, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fa5679, 43 },
    { 0x20da567a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a5683, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382a5688, 90 },
    { 0xea5688, 66 },
    { 0, 0 },
    { 0x1a5a568b, 43 },
    { 0, 0 },
    { 0xbd1a568d, 61 },
    { 0x93ca568e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2a5691, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a5695, 101 },
    { 0x98a5696, 34 },
    { 0x2aa5697, 66 },
    { 0x975a5697, 91 },
    { 0, 0 },
    { 0xd92a569a, 90 },
    { 0, 0 },
    { 0x3c3a569c, 91 },
    { 0, 0 },
    { 0x4eca569e, 55 },
    { 0x569a569f, 61 },
    { 0, 0 },
    { 0xeeca56a1, 55 },
    { 0x135a56a2, 42 },
    { 0xa11a56a2, 23 },
    { 0, 0 },
    { 0xa34a56a5, 90 },
    { 0x167a56a6, 90 },
    { 0xd99a56a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ca56aa, 87 },
    { 0x6dda56ab, 61 },
    { 0x7c0a56aa, 90 },
    { 0x3a1a56aa, 43 },
    { 0xd83a56ae, 91 },
    { 0xd4fa56af, 60 },
    { 0x9bca56ae, 99 },
    { 0xb19a56b1, 61 },
    { 0xd39a56b1, 33 },
    { 0xcf0a56b1, 95 },
    { 0xe44a56b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429a56b8, 99 },
    { 0, 0 },
    { 0x91da56ba, 34 },
    { 0, 0 },
    { 0x271a56bc, 43 },
    { 0, 0 },
    { 0x146a56be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398a56c2, 76 },
    { 0, 0 },
    { 0x6b6a56c4, 96 },
    { 0, 0 },
    { 0xfaaa56c6, 61 },
    { 0xb7fa56c7, 90 },
    { 0, 0 },
    { 0x924a56c9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9a56cd, 90 },
    { 0x4e2a56cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca56d3, 15 },
    { 0, 0 },
    { 0xe13a56d5, 55 },
    { 0xdd8a56d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a56d9, 23 },
    { 0x2d4a56da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239a56df, 46 },
    { 0x50aa56df, 33 },
    { 0x6e4a56e1, 33 },
    { 0xc18a56e2, 98 },
    { 0x32ca56e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fa56e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x272a56eb, 93 },
    { 0xda0a56ec, 46 },
    { 0x514a56ed, 76 },
    { 0xcefa56ee, 33 },
    { 0, 0 },
    { 0xfcea56f0, 61 },
    { 0x8fda56f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390a56f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aa56fb, 55 },
    { 0x1aa56fc, 90 },
    { 0x625a56fd, 61 },
    { 0xb23a56fe, 61 },
    { 0xef2a56fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ba5706, 34 },
    { 0, 0 },
    { 0x931a5708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a570b, 95 },
    { 0x8bba570c, 95 },
    { 0, 0 },
    { 0x966a570e, 33 },
    { 0, 0 },
    { 0x49fa5710, 55 },
    { 0, 0 },
    { 0x89fa5712, 33 },
    { 0x278a5713, 95 },
    { 0xfcca5713, 61 },
    { 0x833a5712, 55 },
    { 0x9fea5716, 66 },
    { 0x818a5717, 55 },
    { 0xc4aa5718, 43 },
    { 0x968a5719, 55 },
    { 0x323a5718, 68 },
    { 0x17ba571b, 23 },
    { 0x6efa5718, 78 },
    { 0, 0 },
    { 0x19aa571e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fba5722, 61 },
    { 0xe54a5723, 23 },
    { 0, 0 },
    { 0x39da5725, 33 },
    { 0x799a5726, 95 },
    { 0x806a5727, 61 },
    { 0xf20a5727, 90 },
    { 0, 0 },
    { 0x652a572a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea572f, 95 },
    { 0x5f6a5730, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa5734, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a5739, 91 },
    { 0x1f3a573a, 90 },
    { 0x30ea573a, 61 },
    { 0x6a1a573c, 76 },
    { 0, 0 },
    { 0xcc2a573e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afa5742, 93 },
    { 0x73aa5743, 93 },
    { 0x430a5744, 95 },
    { 0x99da5744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0da5749, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8a574c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba574f, 98 },
    { 0, 0 },
    { 0xe7a5751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39a5754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5aa575d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23da5764, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a5767, 95 },
    { 0, 0 },
    { 0x6afa5769, 88 },
    { 0x122a576a, 23 },
    { 0x38ba576a, 95 },
    { 0x21aa576c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7a5772, 61 },
    { 0x3ada5772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460a577a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5a577d, 61 },
    { 0x75aa577e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5a5784, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea5788, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95a5791, 23 },
    { 0, 0 },
    { 0x3bfa5793, 33 },
    { 0x9da5794, 46 },
    { 0x3eda5795, 67 },
    { 0x22da5793, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97a579b, 33 },
    { 0x6e2a579b, 61 },
    { 0x8c3a579d, 95 },
    { 0xe8ba579e, 33 },
    { 0xff4a579d, 61 },
    { 0xeaaa57a0, 69 },
    { 0x3f7a57a0, 78 },
    { 0xee1a57a2, 98 },
    { 0x314a57a3, 33 },
    { 0x53da57a2, 33 },
    { 0x85ca57a5, 33 },
    { 0x739a579e, 93 },
    { 0x740a57a7, 61 },
    { 0x788a57a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8a57af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ea57b5, 69 },
    { 0x95fa57b6, 55 },
    { 0x37da57b7, 91 },
    { 0x99ca57b8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a57bc, 61 },
    { 0, 0 },
    { 0x9da57be, 81 },
    { 0xad7a57bf, 81 },
    { 0xfd9a57c0, 95 },
    { 0xd09a57c1, 90 },
    { 0xfaea57c2, 49 },
    { 0x1fba57bf, 76 },
    { 0xe9ba57c3, 65 },
    { 0x448a57c5, 100 },
    { 0xd31a57bf, 84 },
    { 0x88ca57c7, 66 },
    { 0x69fa57c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ea57cc, 14 },
    { 0, 0 },
    { 0xf03a57ce, 98 },
    { 0x749a57ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ba57d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92a57d7, 98 },
    { 0xfc0a57d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafa57de, 23 },
    { 0, 0 },
    { 0x4daa57e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8a57e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca57e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5a57ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426a57f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87aa57ff, 43 },
    { 0xa87a5800, 33 },
    { 0x4e7a57ff, 90 },
    { 0x776a5802, 95 },
    { 0x9a0a5802, 23 },
    { 0xc82a5803, 90 },
    { 0, 0 },
    { 0xacaa5806, 34 },
    { 0, 0 },
    { 0x1f6a5808, 61 },
    { 0x7aea5809, 90 },
    { 0x463a580a, 90 },
    { 0x1a9a5809, 67 },
    { 0x1d5a580c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a5813, 43 },
    { 0xb57a5814, 77 },
    { 0x724a5814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bca581a, 78 },
    { 0, 0 },
    { 0x6c7a581c, 88 },
    { 0x863a581c, 90 },
    { 0, 0 },
    { 0xdba581f, 55 },
    { 0x688a5820, 84 },
    { 0x8b8a5821, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aba582a, 23 },
    { 0xb77a582a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaa582f, 95 },
    { 0, 0 },
    { 0x516a5831, 55 },
    { 0xa56a5831, 91 },
    { 0x5afa5833, 23 },
    { 0x9bea5834, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a5838, 46 },
    { 0x232a5839, 61 },
    { 0xbf7a583a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92a5840, 76 },
    { 0x42ea5841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566a5845, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ba584f, 90 },
    { 0, 0 },
    { 0x2e2a5851, 90 },
    { 0x926a5852, 46 },
    { 0x7b3a5852, 43 },
    { 0x1caa5853, 23 },
    { 0x304a5851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2a5858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82a585e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61a5869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8a586d, 95 },
    { 0xac0a586e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246a5873, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x603a5876, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765a587a, 91 },
    { 0xe76a587b, 23 },
    { 0x1cea587c, 90 },
    { 0xdf6a587c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da5881, 91 },
    { 0x2f9a5881, 93 },
    { 0x13fa5883, 23 },
    { 0x63ba5881, 98 },
    { 0, 0 },
    { 0x263a5886, 34 },
    { 0x1daa5886, 61 },
    { 0xec7a5887, 95 },
    { 0x946a5889, 76 },
    { 0xc9fa5889, 43 },
    { 0x11da588b, 23 },
    { 0xe3fa588a, 52 },
    { 0xef3a588d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x259a5890, 95 },
    { 0, 0 },
    { 0x411a5892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8a5898, 23 },
    { 0, 0 },
    { 0xa1fa589a, 23 },
    { 0, 0 },
    { 0x8c6a589c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ca589f, 92 },
    { 0x29ba58a0, 95 },
    { 0xfca58a1, 34 },
    { 0xec2a58a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79a58a7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a58af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x668a58b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fa58b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fa58b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a58bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1a58c1, 95 },
    { 0xdada58c2, 85 },
    { 0xfa8a58c2, 23 },
    { 0x82fa58c2, 95 },
    { 0x882a58c5, 43 },
    { 0, 0 },
    { 0x951a58c7, 43 },
    { 0x6f4a58c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1a58cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52a58d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ba58d6, 88 },
    { 0, 0 },
    { 0x6bca58d8, 69 },
    { 0xf4da58d8, 90 },
    { 0xca2a58da, 90 },
    { 0, 0 },
    { 0x548a58dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9a58e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9a58e9, 33 },
    { 0xdcfa58ea, 99 },
    { 0x7a8a58eb, 76 },
    { 0xa3fa58ec, 98 },
    { 0x8bea58ed, 98 },
    { 0xc37a58ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72a58f2, 90 },
    { 0x1b3a58f2, 87 },
    { 0x29ba58f2, 55 },
    { 0xa4a58f2, 96 },
    { 0x75a58f6, 90 },
    { 0, 0 },
    { 0xb95a58f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a58fb, 78 },
    { 0, 0 },
    { 0x95fa58fd, 23 },
    { 0, 0 },
    { 0x38fa58ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1a5903, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bda5906, 100 },
    { 0x3c6a5907, 61 },
    { 0x3d0a5907, 33 },
    { 0x3b4a5909, 53 },
    { 0x634a590a, 90 },
    { 0x50ba590b, 66 },
    { 0x32ca590a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ea590f, 72 },
    { 0, 0 },
    { 0x76fa5911, 91 },
    { 0x66ea5911, 78 },
    { 0x8d8a5913, 90 },
    { 0, 0 },
    { 0x30fa5915, 95 },
    { 0xa78a5916, 76 },
    { 0xaeea5915, 90 },
    { 0xa13a5918, 55 },
    { 0xadfa5919, 33 },
    { 0, 0 },
    { 0x72ca591b, 98 },
    { 0, 0 },
    { 0xc86a591d, 90 },
    { 0xaeca591d, 55 },
    { 0xc4a591f, 61 },
    { 0, 0 },
    { 0x980a5921, 55 },
    { 0xe64a5922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x484a5925, 90 },
    { 0xb35a5925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9a5929, 61 },
    { 0, 0 },
    { 0x693a592b, 55 },
    { 0xccba592b, 40 },
    { 0, 0 },
    { 0x307a592e, 99 },
    { 0xa3ea592f, 95 },
    { 0x95da5930, 55 },
    { 0xd7ea5931, 23 },
    { 0x67fa5932, 76 },
    { 0xe39a5930, 90 },
    { 0xccda5934, 95 },
    { 0x618a5935, 61 },
    { 0xf52a5936, 61 },
    { 0x57ea5934, 84 },
    { 0xfb0a5934, 83 },
    { 0x7eda5935, 95 },
    { 0, 0 },
    { 0xa9da593b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788a5940, 33 },
    { 0x16fa5941, 95 },
    { 0xf3aa5942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3a5945, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0a594b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x564a594e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934a5952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x786a5955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fba595b, 33 },
    { 0, 0 },
    { 0x619a595d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41a5962, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4a5966, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8a596b, 43 },
    { 0x81fa596c, 98 },
    { 0xb65a596c, 93 },
    { 0, 0 },
    { 0x148a596f, 33 },
    { 0x1e2a5970, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a5977, 81 },
    { 0xba7a5977, 34 },
    { 0x882a5979, 66 },
    { 0x49ca597a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea597d, 23 },
    { 0x9f5a597e, 91 },
    { 0x43ea597f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x388a5982, 48 },
    { 0, 0 },
    { 0xf4ea5984, 90 },
    { 0xf89a5985, 43 },
    { 0x521a5985, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9a598a, 90 },
    { 0x982a598a, 81 },
    { 0x253a598b, 33 },
    { 0x9b4a598a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9a5991, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ca5994, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49a5997, 85 },
    { 0x103a5998, 33 },
    { 0x5c3a5999, 33 },
    { 0x8c5a5999, 95 },
    { 0, 0 },
    { 0x985a599c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcada599f, 98 },
    { 0xe6ca599f, 65 },
    { 0x66ba59a0, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9a59a4, 76 },
    { 0x259a59a4, 100 },
    { 0xb0da59a4, 95 },
    { 0x1fda59a7, 92 },
    { 0, 0 },
    { 0xa13a59a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x390a59ac, 90 },
    { 0xbc5a59ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99a59b0, 98 },
    { 0, 0 },
    { 0x2e3a59b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca59b8, 52 },
    { 0, 0 },
    { 0x821a59ba, 55 },
    { 0x37da59ba, 98 },
    { 0xb47a59bc, 90 },
    { 0x8b1a59bd, 34 },
    { 0xfc9a59be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cea59c3, 85 },
    { 0xb21a59c4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9a59cb, 23 },
    { 0x95ba59cb, 23 },
    { 0x2b6a59cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x585a59d0, 61 },
    { 0xdb7a59d1, 98 },
    { 0x284a59d2, 55 },
    { 0xb25a59d0, 90 },
    { 0x681a59d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fa59d7, 95 },
    { 0xc3ca59d8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a59db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x198a59de, 98 },
    { 0, 0 },
    { 0x9d1a59e0, 98 },
    { 0xa64a59e1, 90 },
    { 0x52ea59e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fa59e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fa59e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aa59ef, 99 },
    { 0, 0 },
    { 0x510a59f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853a59fe, 87 },
    { 0x2cda59ff, 33 },
    { 0, 0 },
    { 0xf80a5a01, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ca5a05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2da5a0a, 47 },
    { 0x8cea5a0b, 33 },
    { 0xd19a5a0b, 65 },
    { 0, 0 },
    { 0x378a5a0e, 66 },
    { 0, 0 },
    { 0xeaaa5a10, 61 },
    { 0x82ea5a11, 98 },
    { 0x245a5a11, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35a5a15, 55 },
    { 0xc6ba5a15, 90 },
    { 0, 0 },
    { 0xbfda5a18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50a5a1c, 84 },
    { 0, 0 },
    { 0xad5a5a1e, 98 },
    { 0xc35a5a1f, 66 },
    { 0, 0 },
    { 0x89ba5a21, 33 },
    { 0xd46a5a22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80a5a29, 33 },
    { 0x780a5a2a, 90 },
    { 0xbfba5a2b, 90 },
    { 0, 0 },
    { 0x77da5a2d, 33 },
    { 0x594a5a2e, 61 },
    { 0xdca5a2e, 90 },
    { 0x14a5a30, 98 },
    { 0, 0 },
    { 0x92ea5a32, 69 },
    { 0, 0 },
    { 0x782a5a34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaca5a38, 61 },
    { 0, 0 },
    { 0x726a5a3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xafda5a3d, 23 },
    { 0xca2a5a3e, 95 },
    { 0xd6ba5a3d, 90 },
    { 0xbf3a5a40, 90 },
    { 0, 0 },
    { 0x925a5a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3a5a46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fa5a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79a5a54, 36 },
    { 0x620a5a55, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20da5a5b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aa5a5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869a5a64, 90 },
    { 0xb3ca5a65, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a5a68, 43 },
    { 0x93ca5a68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5a5a6d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x151a5a70, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a5a77, 34 },
    { 0, 0 },
    { 0xe34a5a79, 55 },
    { 0, 0 },
    { 0x6caa5a7b, 90 },
    { 0, 0 },
    { 0x9d1a5a7d, 84 },
    { 0, 0 },
    { 0xfd6a5a7f, 55 },
    { 0x146a5a80, 55 },
    { 0, 0 },
    { 0xb1ba5a82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152a5a89, 65 },
    { 0, 0 },
    { 0x8fca5a8b, 61 },
    { 0x694a5a8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69da5a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea5aa0, 23 },
    { 0x14a5aa0, 98 },
    { 0x1caa5aa2, 76 },
    { 0x97ca5aa2, 34 },
    { 0xebea5aa2, 90 },
    { 0, 0 },
    { 0x8da5aa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bca5aaa, 98 },
    { 0x16fa5aab, 90 },
    { 0x8ada5aab, 61 },
    { 0x769a5aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42a5ab0, 95 },
    { 0x1dca5ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33da5ab5, 33 },
    { 0x3b8a5ab6, 40 },
    { 0, 0 },
    { 0x58ea5ab8, 90 },
    { 0x6bfa5ab8, 87 },
    { 0xe1fa5aba, 95 },
    { 0x825a5ab8, 95 },
    { 0, 0 },
    { 0xa90a5abd, 98 },
    { 0xf2fa5abe, 55 },
    { 0, 0 },
    { 0x23aa5ac0, 99 },
    { 0x868a5ac1, 23 },
    { 0x4b3a5ac1, 33 },
    { 0xac9a5ac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356a5acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567a5ad1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4a5ad4, 61 },
    { 0x999a5ad4, 33 },
    { 0x4ea5ad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa5ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25a5add, 76 },
    { 0x315a5ade, 95 },
    { 0x172a5adf, 46 },
    { 0xc35a5ade, 95 },
    { 0xa75a5ae1, 76 },
    { 0x837a5ae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x193a5ae5, 4 },
    { 0x20a5ae6, 98 },
    { 0x866a5ae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dda5aeb, 55 },
    { 0xa66a5aec, 85 },
    { 0x2d5a5aec, 75 },
    { 0xfb0a5aeb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a5af5, 34 },
    { 0x931a5af6, 67 },
    { 0x14da5af7, 99 },
    { 0, 0 },
    { 0xf9aa5af9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ca5afe, 55 },
    { 0x1caa5aff, 96 },
    { 0x193a5b00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bba5b03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a5b07, 99 },
    { 0xdd8a5b08, 76 },
    { 0x429a5b09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbea5b15, 28 },
    { 0x811a5b15, 55 },
    { 0, 0 },
    { 0x54a5b18, 85 },
    { 0xd61a5b18, 51 },
    { 0x490a5b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2a5b1f, 99 },
    { 0x2d3a5b20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110a5b25, 98 },
    { 0xfda5b25, 76 },
    { 0, 0 },
    { 0x24a5b28, 23 },
    { 0xe51a5b29, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8a5b2c, 52 },
    { 0xd1da5b2d, 95 },
    { 0x265a5b2d, 90 },
    { 0xc62a5b2e, 34 },
    { 0, 0 },
    { 0x7d5a5b31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa5b34, 91 },
    { 0x340a5b34, 90 },
    { 0x50aa5b36, 93 },
    { 0x9d8a5b37, 90 },
    { 0x6cda5b38, 68 },
    { 0xc8ba5b35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aa5b3c, 100 },
    { 0x35fa5b3c, 87 },
    { 0xa54a5b3e, 76 },
    { 0x6aba5b3f, 19 },
    { 0xceaa5b3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfda5b45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410a5b49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70a5b4d, 90 },
    { 0x20a5b4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42a5b54, 66 },
    { 0x861a5b55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ba5b5d, 76 },
    { 0x996a5b5e, 40 },
    { 0xa79a5b5f, 61 },
    { 0xbc5a5b60, 73 },
    { 0xf60a5b5d, 43 },
    { 0x4e1a5b5e, 91 },
    { 0x880a5b63, 96 },
    { 0x9aa5b63, 93 },
    { 0xbe7a5b65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ea5b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5a5b6b, 55 },
    { 0x794a5b6c, 61 },
    { 0x78ca5b6b, 98 },
    { 0x6c0a5b6e, 23 },
    { 0x595a5b6e, 67 },
    { 0x484a5b70, 33 },
    { 0x55ea5b71, 23 },
    { 0x9bea5b72, 40 },
    { 0x302a5b73, 55 },
    { 0xd2da5b72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8a5b78, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbea5b84, 90 },
    { 0, 0 },
    { 0xc35a5b86, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cda5b89, 78 },
    { 0, 0 },
    { 0x3c8a5b8b, 93 },
    { 0, 0 },
    { 0x79ba5b8d, 93 },
    { 0x5b8a5b8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fa5b96, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ada5b99, 95 },
    { 0xdd7a5b9a, 61 },
    { 0x900a5b9b, 33 },
    { 0xa0ea5b9b, 33 },
    { 0xacda5b9a, 98 },
    { 0xa22a5b9e, 90 },
    { 0x8f1a5b9f, 98 },
    { 0xbeaa5ba0, 43 },
    { 0x283a5b9f, 34 },
    { 0, 0 },
    { 0x837a5ba3, 90 },
    { 0x17da5ba4, 99 },
    { 0, 0 },
    { 0xe6aa5ba6, 78 },
    { 0x7bba5ba7, 78 },
    { 0x70ca5ba8, 91 },
    { 0, 0 },
    { 0xf1a5baa, 33 },
    { 0x10ea5bab, 95 },
    { 0xcbba5baa, 84 },
    { 0x443a5bad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bb1, 90 },
    { 0xd05a5bb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea5bb5, 33 },
    { 0x3f8a5bb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91a5bba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aa5bc0, 95 },
    { 0x5d7a5bc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a5bc4, 99 },
    { 0xcf2a5bc5, 95 },
    { 0xfbba5bc6, 75 },
    { 0xda7a5bc7, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49a5bce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5bd2, 61 },
    { 0, 0 },
    { 0xd5ba5bd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1a5bd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ea5bdd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a5be0, 85 },
    { 0xd39a5be1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295a5be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4a5be9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67a5bee, 43 },
    { 0xc94a5bef, 76 },
    { 0, 0 },
    { 0xca5a5bf1, 90 },
    { 0xf4a5bf2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fa5bf5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa5bfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3a5bfd, 57 },
    { 0xc56a5bfd, 93 },
    { 0xa67a5bff, 90 },
    { 0xdbaa5c00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daa5c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ba5c07, 23 },
    { 0xc1ea5c08, 68 },
    { 0xe96a5c09, 90 },
    { 0xc15a5c07, 23 },
    { 0, 0 },
    { 0xc46a5c0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16da5c0f, 52 },
    { 0xd21a5c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aea5c14, 23 },
    { 0x6aca5c15, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ca5c18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a5c1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea5c21, 90 },
    { 0xbf1a5c21, 69 },
    { 0x320a5c23, 77 },
    { 0x3aca5c23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26a5c27, 33 },
    { 0x814a5c28, 98 },
    { 0x1b8a5c29, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1da5c2d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27a5c31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454a5c39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ca5c3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a5c40, 23 },
    { 0, 0 },
    { 0x4baa5c42, 93 },
    { 0xfbaa5c43, 90 },
    { 0x80ba5c44, 23 },
    { 0x739a5c45, 34 },
    { 0, 0 },
    { 0xbba5c47, 23 },
    { 0xd44a5c48, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300a5c4c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270a5c54, 98 },
    { 0, 0 },
    { 0x701a5c56, 46 },
    { 0xaada5c57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ea5c5b, 98 },
    { 0xa2ea5c5c, 23 },
    { 0xf35a5c5d, 39 },
    { 0, 0 },
    { 0x897a5c5f, 72 },
    { 0x67ea5c5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2aa5c6e, 98 },
    { 0, 0 },
    { 0xd9aa5c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5a5c75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda5c7f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ba5c8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dda5c97, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeba5c9a, 55 },
    { 0xa51a5c9b, 23 },
    { 0xe19a5c9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffa5ca0, 55 },
    { 0x7b6a5ca1, 61 },
    { 0xeb6a5ca1, 95 },
    { 0x42fa5ca3, 33 },
    { 0x4cfa5ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ea5caa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a5cad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3a5cb3, 49 },
    { 0, 0 },
    { 0x8c5a5cb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cca5cbb, 33 },
    { 0x176a5cbb, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834a5cc1, 33 },
    { 0x593a5cc1, 87 },
    { 0, 0 },
    { 0x86a5cc4, 95 },
    { 0x6bea5cc5, 90 },
    { 0x467a5cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddda5cc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x674a5ccc, 90 },
    { 0xd71a5ccd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29a5cd5, 98 },
    { 0x46ea5cd5, 33 },
    { 0, 0 },
    { 0x33ea5cd8, 98 },
    { 0, 0 },
    { 0x294a5cda, 88 },
    { 0xc6aa5cdb, 67 },
    { 0, 0 },
    { 0x81ca5cdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73aa5ce0, 77 },
    { 0x95da5ce1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34a5ce6, 100 },
    { 0xe3ca5ce7, 24 },
    { 0, 0 },
    { 0x449a5ce9, 23 },
    { 0, 0 },
    { 0xf44a5ceb, 33 },
    { 0x174a5cec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143a5cf0, 98 },
    { 0, 0 },
    { 0x22fa5cf2, 96 },
    { 0xa06a5cf2, 84 },
    { 0x5c6a5cf3, 84 },
    { 0xcfaa5cf5, 55 },
    { 0xf43a5cf6, 29 },
    { 0xf68a5cf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3a5cfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31a5d03, 23 },
    { 0xf72a5d04, 90 },
    { 0xc5da5d05, 61 },
    { 0x3bea5d06, 95 },
    { 0xa38a5d07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558a5d0c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22a5d0f, 44 },
    { 0x632a5d0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x750a5d13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x509a5d16, 23 },
    { 0x433a5d17, 76 },
    { 0, 0 },
    { 0xe9da5d19, 23 },
    { 0x8a2a5d19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69da5d1e, 75 },
    { 0, 0 },
    { 0xd1da5d20, 90 },
    { 0, 0 },
    { 0xda1a5d22, 33 },
    { 0x87da5d23, 52 },
    { 0x162a5d24, 99 },
    { 0x978a5d25, 61 },
    { 0xda3a5d24, 90 },
    { 0xaaa5d27, 88 },
    { 0x182a5d26, 95 },
    { 0xe99a5d29, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612a5d2d, 85 },
    { 0, 0 },
    { 0x462a5d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ca5d34, 93 },
    { 0x7afa5d34, 33 },
    { 0, 0 },
    { 0x4fa5d37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ca5d3b, 93 },
    { 0xf39a5d3b, 93 },
    { 0xeeda5d3d, 76 },
    { 0x26ca5d3e, 66 },
    { 0, 0 },
    { 0xb7ca5d40, 46 },
    { 0xe69a5d40, 55 },
    { 0x372a5d40, 90 },
    { 0x35ea5d43, 95 },
    { 0x5f9a5d44, 23 },
    { 0xcf3a5d45, 61 },
    { 0xbd3a5d46, 95 },
    { 0xda2a5d47, 61 },
    { 0xc58a5d48, 90 },
    { 0x1f7a5d44, 34 },
    { 0x38ea5d4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3a5d4d, 95 },
    { 0xec4a5d4d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aa5d58, 31 },
    { 0x5f3a5d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a5d5f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x282a5d62, 85 },
    { 0xfbaa5d62, 23 },
    { 0x884a5d63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0a5d6a, 84 },
    { 0x8ba5d6b, 76 },
    { 0x692a5d6c, 95 },
    { 0x46a5d6c, 67 },
    { 0x2efa5d6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda5d71, 90 },
    { 0xeb9a5d72, 78 },
    { 0xef3a5d72, 66 },
    { 0, 0 },
    { 0xec8a5d75, 61 },
    { 0, 0 },
    { 0x97ea5d77, 23 },
    { 0x66fa5d78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12a5d7c, 33 },
    { 0x34da5d7d, 91 },
    { 0xc4aa5d7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41a5d84, 87 },
    { 0, 0 },
    { 0x39aa5d86, 90 },
    { 0x7a3a5d87, 90 },
    { 0x2f6a5d86, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57a5d8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56da5d94, 77 },
    { 0xa47a5d95, 61 },
    { 0, 0 },
    { 0x7b3a5d97, 33 },
    { 0, 0 },
    { 0x561a5d99, 34 },
    { 0, 0 },
    { 0xbc2a5d9b, 90 },
    { 0x155a5d9b, 99 },
    { 0x9bda5d9b, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964a5da4, 23 },
    { 0xad1a5da4, 43 },
    { 0xafea5da6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6a5dad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aa5db0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73a5db3, 100 },
    { 0, 0 },
    { 0x85a5db5, 77 },
    { 0xf4ca5db5, 76 },
    { 0xd1fa5db7, 95 },
    { 0x2aca5db7, 55 },
    { 0, 0 },
    { 0xb56a5dba, 52 },
    { 0x8a1a5dbb, 61 },
    { 0x26ba5dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x855a5dbf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2a5dc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7a5dc9, 61 },
    { 0xea5dca, 88 },
    { 0xe5da5dc9, 23 },
    { 0, 0 },
    { 0xfeaa5dcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa5dd2, 95 },
    { 0x3d0a5dd3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738a5dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57a5de1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ca5de7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3a5dec, 93 },
    { 0xeefa5dec, 88 },
    { 0, 0 },
    { 0x25fa5def, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaa5df6, 23 },
    { 0x783a5df7, 23 },
    { 0x227a5df8, 98 },
    { 0, 0 },
    { 0x4eca5dfa, 61 },
    { 0xecca5dfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1a5dfe, 84 },
    { 0xa72a5dff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ca5e03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696a5e09, 67 },
    { 0, 0 },
    { 0xb70a5e0b, 95 },
    { 0, 0 },
    { 0xf0ba5e0d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a5e13, 84 },
    { 0, 0 },
    { 0xc6fa5e15, 95 },
    { 0xce8a5e16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783a5e1b, 76 },
    { 0xab9a5e1b, 81 },
    { 0, 0 },
    { 0xcfba5e1e, 90 },
    { 0x74ca5e1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293a5e24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9a5e2d, 98 },
    { 0x291a5e2e, 17 },
    { 0x9a5e2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564a5e35, 90 },
    { 0xc56a5e36, 61 },
    { 0x389a5e35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532a5e3b, 52 },
    { 0, 0 },
    { 0x1d1a5e3d, 87 },
    { 0x414a5e3e, 33 },
    { 0, 0 },
    { 0xaf9a5e40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca5e46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9a5e49, 23 },
    { 0, 0 },
    { 0x6c7a5e4b, 90 },
    { 0, 0 },
    { 0x70da5e4d, 90 },
    { 0, 0 },
    { 0x17ca5e4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ba5e52, 99 },
    { 0xf2aa5e53, 46 },
    { 0, 0 },
    { 0xd89a5e55, 55 },
    { 0xebaa5e55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeea5e5a, 95 },
    { 0x3c3a5e5b, 33 },
    { 0xe1ba5e5a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fa5e61, 95 },
    { 0xd53a5e62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ba5e68, 90 },
    { 0x8f4a5e68, 61 },
    { 0xe1ca5e6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31a5e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x676a5e71, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dca5e74, 76 },
    { 0x86ca5e75, 87 },
    { 0xb94a5e76, 95 },
    { 0x27ca5e77, 61 },
    { 0xe6ba5e78, 90 },
    { 0x236a5e74, 34 },
    { 0, 0 },
    { 0x6d2a5e7b, 23 },
    { 0x3b4a5e7c, 90 },
    { 0xeb4a5e7d, 100 },
    { 0xddaa5e7e, 70 },
    { 0x6aca5e7f, 43 },
    { 0x5b0a5e7c, 76 },
    { 0, 0 },
    { 0xc96a5e82, 76 },
    { 0, 0 },
    { 0xdbfa5e84, 98 },
    { 0xe29a5e85, 33 },
    { 0xf23a5e86, 38 },
    { 0x6c8a5e84, 77 },
    { 0, 0 },
    { 0x8ffa5e89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x741a5e8c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486a5e9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0a5ea3, 46 },
    { 0x118a5ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22a5ead, 91 },
    { 0x2e8a5eae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ba5eb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37da5eb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ea5ebd, 97 },
    { 0xa63a5ebe, 23 },
    { 0xbb9a5ebf, 90 },
    { 0, 0 },
    { 0x801a5ec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809a5ec5, 85 },
    { 0x6bfa5ec6, 61 },
    { 0, 0 },
    { 0x782a5ec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca5ece, 61 },
    { 0, 0 },
    { 0x28ba5ed0, 61 },
    { 0x8ba5ed1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadda5ed7, 76 },
    { 0, 0 },
    { 0x4c9a5ed9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fa5edd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeba5ee2, 93 },
    { 0x2da5ee3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ca5ee7, 76 },
    { 0xf54a5ee7, 61 },
    { 0, 0 },
    { 0x2f0a5eea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340a5ef2, 33 },
    { 0xf65a5ef3, 34 },
    { 0xe8ea5ef3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fa5efc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eba5eff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12a5f05, 66 },
    { 0xde4a5f06, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a5f0a, 90 },
    { 0x118a5f0b, 46 },
    { 0xf12a5f0a, 61 },
    { 0xab4a5f0d, 23 },
    { 0, 0 },
    { 0x5eea5f0f, 66 },
    { 0x87a5f10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dea5f17, 61 },
    { 0xd44a5f18, 67 },
    { 0, 0 },
    { 0x590a5f1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8a5f1e, 84 },
    { 0x423a5f1f, 90 },
    { 0xa83a5f1e, 95 },
    { 0xd37a5f1e, 81 },
    { 0xaa4a5f22, 68 },
    { 0x2fca5f22, 85 },
    { 0xa4ba5f22, 90 },
    { 0x1b4a5f1f, 90 },
    { 0x86aa5f26, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x524a5f29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eea5f2f, 90 },
    { 0xef1a5f30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6a5f3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157a5f3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a5f45, 90 },
    { 0x85da5f45, 55 },
    { 0, 0 },
    { 0x52ba5f48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475a5f51, 90 },
    { 0x31a5f51, 33 },
    { 0, 0 },
    { 0x3bfa5f54, 61 },
    { 0xfc1a5f55, 46 },
    { 0, 0 },
    { 0xa35a5f57, 99 },
    { 0x725a5f58, 98 },
    { 0x322a5f59, 23 },
    { 0xb89a5f58, 95 },
    { 0xb56a5f59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a5f5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28a5f64, 90 },
    { 0x21ea5f65, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9a5f71, 95 },
    { 0xb12a5f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aa5f79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aa5f7f, 23 },
    { 0, 0 },
    { 0x678a5f81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16a5f87, 61 },
    { 0x92aa5f87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fa5f92, 93 },
    { 0, 0 },
    { 0xa49a5f94, 98 },
    { 0, 0 },
    { 0x286a5f96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4a5f9a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfa5f9d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4a5fa0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8a5fa5, 61 },
    { 0x74ca5fa6, 90 },
    { 0x90ea5fa7, 98 },
    { 0x5fea5fa6, 34 },
    { 0, 0 },
    { 0xd47a5faa, 33 },
    { 0xc05a5faa, 78 },
    { 0x2bfa5fac, 84 },
    { 0xd3a5fab, 67 },
    { 0, 0 },
    { 0x34fa5faf, 98 },
    { 0, 0 },
    { 0xdc3a5fb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789a5fb8, 52 },
    { 0xa83a5fb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ca5fbf, 33 },
    { 0x4e8a5fc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11a5fc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x760a5fc8, 90 },
    { 0xe3ea5fc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547a5fcd, 46 },
    { 0x2f9a5fce, 98 },
    { 0, 0 },
    { 0xff8a5fd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda5fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7a5fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88aa5fdf, 98 },
    { 0x2a0a5fdf, 98 },
    { 0xb55a5fdf, 77 },
    { 0, 0 },
    { 0xfc6a5fe3, 23 },
    { 0xc2fa5fe4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ba5feb, 95 },
    { 0x146a5fec, 99 },
    { 0x955a5fec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a5ff6, 93 },
    { 0, 0 },
    { 0x6f3a5ff8, 90 },
    { 0x8dfa5ff9, 76 },
    { 0xc18a5ffa, 93 },
    { 0x8f2a5ffa, 90 },
    { 0x27fa5ffc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11a6000, 95 },
    { 0, 0 },
    { 0xdd8a6002, 34 },
    { 0xf95a6002, 90 },
    { 0, 0 },
    { 0xb9da6005, 87 },
    { 0xd85a6005, 33 },
    { 0, 0 },
    { 0x9a6008, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03a600e, 87 },
    { 0xf2fa600f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114a6013, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aa6017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26a601b, 33 },
    { 0x7ada601c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08a6020, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x131a6023, 98 },
    { 0xfcca6023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2a602a, 98 },
    { 0x89ca602b, 61 },
    { 0x762a602a, 33 },
    { 0, 0 },
    { 0x424a602e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434a6035, 84 },
    { 0x53da6036, 61 },
    { 0x6e0a6035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5a6044, 33 },
    { 0x451a6045, 96 },
    { 0x9aba6045, 85 },
    { 0, 0 },
    { 0xecfa6048, 33 },
    { 0, 0 },
    { 0xa6aa604a, 88 },
    { 0x489a604b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8a604e, 61 },
    { 0, 0 },
    { 0x52fa6050, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330a6054, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eda605a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aa605d, 91 },
    { 0x430a605e, 65 },
    { 0, 0 },
    { 0xf6a6060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75a6065, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80a6068, 93 },
    { 0xdcfa6069, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5a606d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bba6071, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaa6077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aca607f, 90 },
    { 0x46fa6080, 95 },
    { 0x692a607f, 90 },
    { 0x524a6082, 76 },
    { 0xa23a6083, 33 },
    { 0x576a6083, 66 },
    { 0x605a6085, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97a6088, 61 },
    { 0x231a6089, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3a608f, 88 },
    { 0x28fa6090, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ca60a4, 99 },
    { 0, 0 },
    { 0x3b9a60a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6a60a9, 81 },
    { 0x93fa60a9, 76 },
    { 0, 0 },
    { 0xcd4a60ac, 90 },
    { 0xdb3a60ac, 34 },
    { 0, 0 },
    { 0x3aa60af, 98 },
    { 0x683a60b0, 98 },
    { 0x527a60af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0aa60b5, 61 },
    { 0, 0 },
    { 0x450a60b7, 90 },
    { 0x9d0a60b8, 90 },
    { 0x3afa60b7, 90 },
    { 0, 0 },
    { 0xb1da60bb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a60c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ba60c3, 90 },
    { 0x9dfa60c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24a60c8, 85 },
    { 0xc63a60c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4a60cc, 61 },
    { 0, 0 },
    { 0xa7fa60ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5a60d1, 52 },
    { 0, 0 },
    { 0x41aa60d3, 99 },
    { 0, 0 },
    { 0x18aa60d5, 34 },
    { 0x433a60d6, 23 },
    { 0xa3ba60d7, 36 },
    { 0xd3aa60d8, 95 },
    { 0xa9a60d7, 95 },
    { 0xe5fa60da, 33 },
    { 0x93ea60d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6a60df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79aa60e3, 94 },
    { 0xd41a60e3, 61 },
    { 0, 0 },
    { 0x8a0a60e6, 95 },
    { 0x811a60e7, 61 },
    { 0, 0 },
    { 0x1b8a60e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76da60ef, 23 },
    { 0x6f4a60ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x715a60f3, 95 },
    { 0x8fca60f4, 85 },
    { 0x445a60f4, 93 },
    { 0xbeda60f6, 98 },
    { 0, 0 },
    { 0xd4ea60f8, 88 },
    { 0, 0 },
    { 0x5eea60fa, 95 },
    { 0x6ea60fb, 90 },
    { 0x89da60fc, 85 },
    { 0, 0 },
    { 0x331a60fe, 23 },
    { 0x9b7a60ff, 90 },
    { 0, 0 },
    { 0x8daa6101, 99 },
    { 0xb01a6102, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70a610b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a6110, 95 },
    { 0xfe0a6110, 3 },
    { 0xcbba6112, 90 },
    { 0x4fba6113, 23 },
    { 0x3e1a6114, 46 },
    { 0x30a6115, 55 },
    { 0xbe9a6116, 98 },
    { 0x655a6115, 69 },
    { 0x148a6114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fa611d, 66 },
    { 0xea8a611e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x753a6121, 23 },
    { 0x827a6122, 33 },
    { 0x1f0a6121, 99 },
    { 0xceaa6124, 61 },
    { 0, 0 },
    { 0xecda6126, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a612f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673a6133, 91 },
    { 0, 0 },
    { 0xe19a6135, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dca6138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a613b, 61 },
    { 0x26a613c, 12 },
    { 0, 0 },
    { 0x72ca613e, 66 },
    { 0x1e1a613f, 97 },
    { 0, 0 },
    { 0x720a6141, 90 },
    { 0x5ba6142, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a6146, 23 },
    { 0x65da6147, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01a614c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaa615a, 85 },
    { 0x689a615b, 95 },
    { 0, 0 },
    { 0x759a615d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a6160, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aa6166, 91 },
    { 0xceba6167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x884a616a, 34 },
    { 0, 0 },
    { 0x235a616c, 33 },
    { 0x594a616d, 91 },
    { 0xeba616e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a6173, 34 },
    { 0, 0 },
    { 0xd5ca6175, 55 },
    { 0xe35a6176, 55 },
    { 0x43fa6176, 98 },
    { 0x892a6177, 98 },
    { 0, 0 },
    { 0xa8a617a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4da617e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fba6181, 98 },
    { 0xf35a6181, 46 },
    { 0xf34a6181, 90 },
    { 0x44a6184, 33 },
    { 0, 0 },
    { 0xc23a6186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdea6189, 61 },
    { 0x8f3a6189, 55 },
    { 0, 0 },
    { 0x2f8a618c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06a618f, 23 },
    { 0xf54a6190, 90 },
    { 0x94fa6191, 80 },
    { 0x13ca6190, 17 },
    { 0, 0 },
    { 0x466a6194, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166a619a, 40 },
    { 0, 0 },
    { 0x493a619c, 93 },
    { 0x80fa619d, 98 },
    { 0, 0 },
    { 0x938a619f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a61a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fa61a7, 66 },
    { 0, 0 },
    { 0x17da61a9, 46 },
    { 0xc55a61a9, 33 },
    { 0x86fa61ab, 91 },
    { 0x6a6a61ac, 90 },
    { 0x3daa61a9, 76 },
    { 0x136a61ae, 90 },
    { 0, 0 },
    { 0xcfca61b0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938a61ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770a61c5, 66 },
    { 0x7b4a61c6, 76 },
    { 0x262a61c7, 43 },
    { 0xcc4a61c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x955a61cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cea61d0, 75 },
    { 0x668a61d0, 90 },
    { 0x9dea61d2, 93 },
    { 0x294a61d2, 61 },
    { 0x377a61d4, 87 },
    { 0xd08a61d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512a61db, 98 },
    { 0, 0 },
    { 0x5d7a61dd, 76 },
    { 0xf93a61de, 23 },
    { 0x5a9a61df, 90 },
    { 0x989a61df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902a61e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa61eb, 40 },
    { 0xd05a61ec, 33 },
    { 0, 0 },
    { 0x631a61ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a61f6, 98 },
    { 0, 0 },
    { 0xeb6a61f8, 61 },
    { 0x418a61f9, 87 },
    { 0x63aa61f9, 61 },
    { 0x2baa61fa, 91 },
    { 0x220a61fb, 40 },
    { 0xeb1a61fc, 100 },
    { 0, 0 },
    { 0x4b8a61ff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fa6202, 90 },
    { 0xf34a6202, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981a6207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59a620b, 85 },
    { 0x6e7a620b, 61 },
    { 0x6d5a620c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda6216, 80 },
    { 0x354a6216, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6a621c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7a621f, 52 },
    { 0x53da6220, 95 },
    { 0x5a8a6221, 90 },
    { 0x773a6222, 61 },
    { 0x4aaa6222, 33 },
    { 0xbc1a6221, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bea6228, 80 },
    { 0xa44a6229, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ba622e, 66 },
    { 0, 0 },
    { 0xfa5a6230, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ea6235, 33 },
    { 0x1eda6236, 87 },
    { 0, 0 },
    { 0xed3a6238, 88 },
    { 0, 0 },
    { 0xb41a623a, 68 },
    { 0xbffa623a, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fa623e, 95 },
    { 0xbf9a623e, 43 },
    { 0xd94a623e, 61 },
    { 0x13aa6241, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ca6246, 76 },
    { 0xafba6246, 90 },
    { 0x8cca6246, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aa624b, 23 },
    { 0x60aa624b, 61 },
    { 0, 0 },
    { 0x66a624e, 34 },
    { 0x5b8a624f, 61 },
    { 0x809a624e, 23 },
    { 0x23aa6251, 90 },
    { 0x899a6252, 66 },
    { 0, 0 },
    { 0xe6ca6254, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9a6257, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77a625c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x738a625f, 90 },
    { 0x50ca625f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7da6263, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952a6268, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6a626b, 99 },
    { 0, 0 },
    { 0xb9da626d, 33 },
    { 0xf94a626e, 23 },
    { 0xf19a626d, 95 },
    { 0x6b6a6270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898a6274, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a6278, 33 },
    { 0, 0 },
    { 0x269a627a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88a627e, 98 },
    { 0x346a627f, 67 },
    { 0x16ba6280, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a628a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaa6293, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654a629a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457a629e, 43 },
    { 0x967a629f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3a62b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32a62b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdea62b9, 23 },
    { 0x9e9a62b9, 33 },
    { 0x7b1a62b9, 23 },
    { 0, 0 },
    { 0xee3a62bd, 98 },
    { 0, 0 },
    { 0x1d1a62bf, 76 },
    { 0x4d9a62bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4a62c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0a62c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5a62cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fa62cf, 46 },
    { 0, 0 },
    { 0xe1ca62d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1a62d4, 61 },
    { 0, 0 },
    { 0x21ba62d6, 61 },
    { 0x74ea62d7, 33 },
    { 0xa0ea62d7, 98 },
    { 0x75fa62d6, 55 },
    { 0xa3da62da, 76 },
    { 0, 0 },
    { 0xabea62dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0a62df, 98 },
    { 0x170a62e0, 68 },
    { 0xa9da62e1, 87 },
    { 0x2e3a62e0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65a62ea, 61 },
    { 0xe52a62ea, 23 },
    { 0, 0 },
    { 0x3f3a62ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68a62f7, 95 },
    { 0x35ea62f8, 98 },
    { 0x4e3a62f8, 61 },
    { 0x1f2a62f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12a6303, 90 },
    { 0x89fa6304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaaa6307, 87 },
    { 0xcf6a6307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a6312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6a631a, 95 },
    { 0xb69a631b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338a6320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959a632b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x133a632e, 90 },
    { 0xb7ca632f, 23 },
    { 0x2da632e, 43 },
    { 0xb02a6331, 33 },
    { 0x4c5a6331, 34 },
    { 0x8a9a6333, 33 },
    { 0, 0 },
    { 0xdf3a6335, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a633e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4a6342, 90 },
    { 0, 0 },
    { 0x8d6a6344, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eba6348, 61 },
    { 0, 0 },
    { 0xb04a634a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aa6352, 99 },
    { 0x857a6353, 78 },
    { 0x6a1a6353, 90 },
    { 0, 0 },
    { 0x494a6356, 53 },
    { 0xb0da6357, 61 },
    { 0x628a6357, 46 },
    { 0x8ba6359, 85 },
    { 0, 0 },
    { 0xcfea635b, 23 },
    { 0xbcaa635c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdba6360, 84 },
    { 0xfb8a6361, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105a6365, 23 },
    { 0xa50a6366, 61 },
    { 0x74da6367, 61 },
    { 0xce5a6365, 43 },
    { 0x258a6369, 90 },
    { 0x56ea636a, 52 },
    { 0xe2ca636b, 61 },
    { 0x5f4a636c, 98 },
    { 0x7f3a636d, 90 },
    { 0xf85a6366, 55 },
    { 0x155a636c, 23 },
    { 0xff1a6370, 88 },
    { 0x310a6371, 90 },
    { 0x747a6369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ba6375, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa6379, 90 },
    { 0x73ca637a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ba637d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aa6382, 67 },
    { 0xba0a6382, 61 },
    { 0x757a6384, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ba638a, 17 },
    { 0xf10a638b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa638f, 61 },
    { 0, 0 },
    { 0xda6a6391, 90 },
    { 0, 0 },
    { 0x1f4a6393, 91 },
    { 0x83fa6394, 56 },
    { 0xfaea6395, 95 },
    { 0x9a6a6396, 34 },
    { 0x7bfa6397, 76 },
    { 0x76ca6398, 95 },
    { 0x81fa6398, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2da639e, 23 },
    { 0xa04a639f, 65 },
    { 0xe9ca639f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a63a3, 81 },
    { 0, 0 },
    { 0xa2fa63a5, 76 },
    { 0x2aa63a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fa63ac, 66 },
    { 0xb0fa63ac, 55 },
    { 0xe40a63ac, 43 },
    { 0xf3ba63ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cba63b3, 90 },
    { 0x4ffa63b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477a63c2, 87 },
    { 0x4e9a63c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ca63c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x541a63ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ca63ce, 95 },
    { 0, 0 },
    { 0x1f2a63d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a63d4, 64 },
    { 0, 0 },
    { 0xd1da63d6, 95 },
    { 0x4efa63d7, 43 },
    { 0, 0 },
    { 0xe35a63d9, 98 },
    { 0, 0 },
    { 0xe1da63db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83a63df, 98 },
    { 0x21da63df, 100 },
    { 0x6a2a63e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a63eb, 17 },
    { 0x8ba63ec, 61 },
    { 0x3d1a63ed, 33 },
    { 0xbd0a63ee, 90 },
    { 0xc79a63ed, 61 },
    { 0xe56a63eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a63f3, 85 },
    { 0xee9a63f3, 90 },
    { 0x5dba63f5, 90 },
    { 0, 0 },
    { 0x7bca63f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a63fd, 61 },
    { 0xd1aa63fd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6a6402, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709a6407, 95 },
    { 0xdd7a6407, 61 },
    { 0x6c4a6409, 90 },
    { 0x8a3a6409, 90 },
    { 0, 0 },
    { 0x750a640c, 91 },
    { 0x82fa640c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437a6413, 61 },
    { 0xb1a6413, 67 },
    { 0, 0 },
    { 0x2bea6416, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cda6419, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aa641c, 23 },
    { 0xddba641c, 65 },
    { 0x908a641e, 76 },
    { 0x693a641d, 85 },
    { 0x569a641f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeca6423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca6426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07a642a, 90 },
    { 0x145a642b, 61 },
    { 0x955a642b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x986a642f, 99 },
    { 0x324a6430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aba6435, 33 },
    { 0, 0 },
    { 0x7b4a6437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x921a643a, 90 },
    { 0, 0 },
    { 0x323a643c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70a644a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7a6450, 55 },
    { 0, 0 },
    { 0xf52a6452, 98 },
    { 0x94a6453, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fa645a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5a645d, 95 },
    { 0, 0 },
    { 0xdada645f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938a646c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aa6472, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938a6477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2a647b, 33 },
    { 0xd98a647b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a6480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ca6484, 61 },
    { 0, 0 },
    { 0x757a6486, 76 },
    { 0x466a6487, 90 },
    { 0x239a6487, 40 },
    { 0x65a6486, 34 },
    { 0, 0 },
    { 0xfe4a648b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318a648f, 98 },
    { 0xc45a6490, 88 },
    { 0xf57a6491, 33 },
    { 0x4fba648f, 98 },
    { 0xffba6493, 33 },
    { 0x2aa6493, 33 },
    { 0x22ea6494, 64 },
    { 0x6fba6496, 23 },
    { 0xfdda6492, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75a649a, 94 },
    { 0, 0 },
    { 0x5c7a649c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72a649f, 90 },
    { 0, 0 },
    { 0xd4ca64a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bea64a9, 93 },
    { 0x151a64a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680a64ae, 55 },
    { 0xaa64af, 98 },
    { 0x3a1a64af, 90 },
    { 0x90ba64b1, 33 },
    { 0x2fba64af, 66 },
    { 0xe0ca64b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbda64b6, 23 },
    { 0x69a64b7, 85 },
    { 0x505a64b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a64bd, 76 },
    { 0, 0 },
    { 0x2e0a64bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52a64c6, 95 },
    { 0x12aa64c6, 33 },
    { 0, 0 },
    { 0xb9ba64c9, 61 },
    { 0, 0 },
    { 0x7eca64cb, 95 },
    { 0, 0 },
    { 0xe21a64cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efa64d0, 61 },
    { 0x38ba64d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44a64d5, 91 },
    { 0, 0 },
    { 0x8bfa64d7, 33 },
    { 0, 0 },
    { 0x9a9a64d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca64dc, 98 },
    { 0x190a64dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9baa64e2, 90 },
    { 0x63ca64e3, 90 },
    { 0xea7a64e4, 34 },
    { 0x13a64e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a64e8, 90 },
    { 0x5bea64e9, 76 },
    { 0x4caa64ea, 23 },
    { 0x177a64eb, 77 },
    { 0x875a64ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5a64f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba64f9, 98 },
    { 0, 0 },
    { 0x58ca64fb, 23 },
    { 0xbe6a64fc, 100 },
    { 0x409a64fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa6502, 61 },
    { 0xa68a6502, 98 },
    { 0x166a6503, 61 },
    { 0x41a6502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ba6508, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baa650c, 95 },
    { 0x304a650d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bda6511, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fea651e, 23 },
    { 0xb6a651f, 34 },
    { 0, 0 },
    { 0x4c9a6521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77a6529, 33 },
    { 0x9cca652a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a652d, 61 },
    { 0, 0 },
    { 0x6efa652f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9a6534, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342a653a, 100 },
    { 0, 0 },
    { 0x708a653c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a6544, 95 },
    { 0xb76a6545, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ea6548, 90 },
    { 0, 0 },
    { 0xe8fa654a, 78 },
    { 0x396a654a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a654e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bda6552, 98 },
    { 0, 0 },
    { 0xf35a6554, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda655e, 23 },
    { 0x2b8a655f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3da6562, 95 },
    { 0xfca6562, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5a6566, 90 },
    { 0x1a9a6566, 23 },
    { 0x219a6568, 76 },
    { 0x48ea6569, 76 },
    { 0x2a0a6566, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bca6571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20a6574, 76 },
    { 0x2e0a6575, 43 },
    { 0xcfda6576, 55 },
    { 0xa45a6575, 84 },
    { 0, 0 },
    { 0xce4a6579, 95 },
    { 0, 0 },
    { 0x9a0a657b, 90 },
    { 0, 0 },
    { 0xbb9a657d, 90 },
    { 0x581a657e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fa6582, 77 },
    { 0x98a6583, 90 },
    { 0xac7a6584, 90 },
    { 0x305a6582, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9a658d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36a6594, 91 },
    { 0xee5a6595, 90 },
    { 0x644a6595, 34 },
    { 0x62a6597, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2a659c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98a65a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a65a4, 75 },
    { 0, 0 },
    { 0x68da65a6, 95 },
    { 0, 0 },
    { 0x955a65a8, 33 },
    { 0, 0 },
    { 0xd58a65aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751a65c1, 33 },
    { 0, 0 },
    { 0xe88a65c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ca65c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84a65ca, 90 },
    { 0, 0 },
    { 0x354a65cc, 61 },
    { 0xd4ea65cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dda65d0, 98 },
    { 0x382a65d1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10a65d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a65d7, 33 },
    { 0x3d5a65d8, 61 },
    { 0x8c7a65d8, 90 },
    { 0xe34a65da, 76 },
    { 0xa77a65da, 96 },
    { 0x29da65db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfa65e0, 43 },
    { 0, 0 },
    { 0x255a65e2, 98 },
    { 0x6d3a65e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452a65e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823a65ed, 23 },
    { 0x7fea65ed, 61 },
    { 0x515a65ed, 55 },
    { 0xab1a65ee, 76 },
    { 0x4d4a65ed, 34 },
    { 0x56ba65f2, 98 },
    { 0xed9a65f3, 23 },
    { 0xc0ea65f2, 76 },
    { 0x85ba65ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaea65f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfa65fc, 53 },
    { 0, 0 },
    { 0xb51a65fe, 85 },
    { 0xf4ca65ff, 33 },
    { 0, 0 },
    { 0xc31a6601, 43 },
    { 0, 0 },
    { 0x9ffa6603, 33 },
    { 0xac6a6604, 43 },
    { 0, 0 },
    { 0x615a6606, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40a6609, 98 },
    { 0, 0 },
    { 0x121a660b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08a6612, 85 },
    { 0, 0 },
    { 0xf71a6614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6a6618, 61 },
    { 0, 0 },
    { 0xb68a661a, 95 },
    { 0, 0 },
    { 0x3a5a661c, 52 },
    { 0xf81a661d, 97 },
    { 0xc3fa661c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69a6624, 90 },
    { 0xba6625, 96 },
    { 0x599a6625, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840a662a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89da662e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ea6634, 61 },
    { 0, 0 },
    { 0x934a6636, 88 },
    { 0x7daa6637, 69 },
    { 0x60ca6637, 76 },
    { 0, 0 },
    { 0xba663a, 23 },
    { 0, 0 },
    { 0xa5da663c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19a6640, 76 },
    { 0, 0 },
    { 0x20aa6642, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x841a6645, 33 },
    { 0x2d1a6646, 90 },
    { 0xfeca6645, 83 },
    { 0x680a6648, 33 },
    { 0x9d5a6649, 98 },
    { 0x958a664a, 33 },
    { 0, 0 },
    { 0x7aaa664c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98a6651, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42a6655, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a665c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95a6661, 61 },
    { 0, 0 },
    { 0x548a6663, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeda6669, 90 },
    { 0x39da6669, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a666f, 68 },
    { 0x97fa6670, 84 },
    { 0xf2ea6671, 61 },
    { 0, 0 },
    { 0xb89a6673, 76 },
    { 0xf30a6674, 33 },
    { 0xb2fa6675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194a6679, 17 },
    { 0xdd3a667a, 98 },
    { 0x7e8a667b, 33 },
    { 0, 0 },
    { 0xe3aa667d, 61 },
    { 0, 0 },
    { 0xb4ba667f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6a6683, 84 },
    { 0, 0 },
    { 0x4cea6685, 61 },
    { 0xd38a6685, 61 },
    { 0, 0 },
    { 0x57a6688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7a668e, 76 },
    { 0x107a668e, 84 },
    { 0xe6da668e, 61 },
    { 0xe7ba6691, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97a6694, 55 },
    { 0x31a6695, 98 },
    { 0x6e3a6696, 95 },
    { 0x286a6697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894a669c, 78 },
    { 0x12ca669c, 78 },
    { 0xef9a669e, 93 },
    { 0xa64a669f, 55 },
    { 0xe70a66a0, 85 },
    { 0xde3a669e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410a66a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x542a66ab, 97 },
    { 0x37aa66ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfa66b0, 99 },
    { 0x905a66b0, 90 },
    { 0xe95a66b1, 87 },
    { 0, 0 },
    { 0xd28a66b4, 90 },
    { 0xdc1a66b4, 95 },
    { 0, 0 },
    { 0xe91a66b7, 95 },
    { 0xee0a66b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264a66be, 61 },
    { 0x1d5a66bf, 66 },
    { 0, 0 },
    { 0xcd8a66c1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8a66c5, 66 },
    { 0xdcda66c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50a66c9, 17 },
    { 0x2a2a66c9, 93 },
    { 0x47da66cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ca66ce, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea66d2, 61 },
    { 0x7f9a66d2, 61 },
    { 0xe4ca66d4, 90 },
    { 0x182a66d5, 99 },
    { 0, 0 },
    { 0x48fa66d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcca66dd, 43 },
    { 0xa42a66dd, 95 },
    { 0x5eda66dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x925a66e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecea66e9, 61 },
    { 0x9aa66ea, 61 },
    { 0xcf2a66e9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a66f1, 95 },
    { 0, 0 },
    { 0x42ca66f3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a66fa, 23 },
    { 0, 0 },
    { 0xae8a66fc, 23 },
    { 0xffaa66fc, 98 },
    { 0xefba66fe, 84 },
    { 0, 0 },
    { 0x30aa6700, 95 },
    { 0x8cfa6701, 61 },
    { 0x4f2a6702, 95 },
    { 0x502a6703, 87 },
    { 0x31a6701, 34 },
    { 0xf43a6701, 90 },
    { 0x2f9a6706, 81 },
    { 0x60fa6706, 90 },
    { 0xe2fa6708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21a670b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea670f, 98 },
    { 0, 0 },
    { 0x463a6711, 68 },
    { 0xe29a6711, 17 },
    { 0x412a6713, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6a6716, 61 },
    { 0x29ca6717, 17 },
    { 0, 0 },
    { 0x827a6719, 52 },
    { 0, 0 },
    { 0x131a671b, 33 },
    { 0xbaaa671c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965a6721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea6727, 90 },
    { 0xd60a6728, 23 },
    { 0x100a6728, 91 },
    { 0x496a672a, 23 },
    { 0xcdea672b, 76 },
    { 0, 0 },
    { 0x46ca672d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3a6732, 61 },
    { 0, 0 },
    { 0xfa2a6734, 66 },
    { 0x343a6735, 95 },
    { 0xb20a6736, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1a673b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92a673f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a6743, 55 },
    { 0, 0 },
    { 0xb82a6745, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5a6748, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fa674b, 23 },
    { 0xbaa674c, 95 },
    { 0, 0 },
    { 0x121a674e, 93 },
    { 0, 0 },
    { 0xbdda6750, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcba6758, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4a675f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6a6762, 85 },
    { 0x63da6763, 66 },
    { 0xb5ea6764, 61 },
    { 0xaefa6763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0a6769, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fa676d, 90 },
    { 0x2da676e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47da6772, 88 },
    { 0, 0 },
    { 0x981a6774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a6777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32da677c, 23 },
    { 0x8dfa677d, 93 },
    { 0x338a677e, 33 },
    { 0xf7fa677e, 95 },
    { 0xc62a677e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0a6783, 61 },
    { 0xac6a6783, 67 },
    { 0xc6a6785, 61 },
    { 0, 0 },
    { 0x649a6787, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9a678b, 76 },
    { 0, 0 },
    { 0x5daa678d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcea6790, 93 },
    { 0xf12a6791, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21a6794, 93 },
    { 0xfc9a6795, 33 },
    { 0xc4aa6795, 60 },
    { 0x6bea6795, 90 },
    { 0x993a6797, 46 },
    { 0xa94a6799, 97 },
    { 0x5fca679a, 90 },
    { 0, 0 },
    { 0xe8aa679c, 61 },
    { 0, 0 },
    { 0xb78a679e, 52 },
    { 0, 0 },
    { 0x4f7a67a0, 43 },
    { 0x49a67a1, 33 },
    { 0xb60a67a2, 76 },
    { 0x76a67a3, 61 },
    { 0x2b6a67a4, 92 },
    { 0, 0 },
    { 0x863a67a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fa67ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a67b1, 61 },
    { 0x715a67b2, 76 },
    { 0x899a67b2, 33 },
    { 0x7bba67b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaa67b7, 95 },
    { 0x803a67b8, 33 },
    { 0x110a67b8, 33 },
    { 0, 0 },
    { 0x57aa67bb, 76 },
    { 0xd05a67bb, 52 },
    { 0x3f9a67bd, 33 },
    { 0xc72a67be, 95 },
    { 0x6da67bd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58a67c2, 33 },
    { 0x4d6a67c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8a67cb, 90 },
    { 0xaa1a67cc, 95 },
    { 0, 0 },
    { 0x28a67ce, 91 },
    { 0x141a67cf, 95 },
    { 0x8a6a67d0, 95 },
    { 0xb5aa67d0, 34 },
    { 0x869a67d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7a67d7, 46 },
    { 0xac8a67d8, 90 },
    { 0xca0a67d9, 23 },
    { 0x6eaa67da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219a67df, 77 },
    { 0x97ba67e0, 98 },
    { 0xfc2a67e1, 43 },
    { 0xef6a67df, 97 },
    { 0x421a67e0, 61 },
    { 0xf29a67e1, 82 },
    { 0x71ea67e5, 33 },
    { 0x1bfa67e3, 90 },
    { 0, 0 },
    { 0xbb0a67e8, 61 },
    { 0x51aa67e9, 33 },
    { 0x91da67e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7a67ef, 66 },
    { 0x37a67ef, 87 },
    { 0xfe2a67f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efa67f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea6802, 99 },
    { 0, 0 },
    { 0x858a6804, 61 },
    { 0x137a6805, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a680b, 61 },
    { 0, 0 },
    { 0xb22a680d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x747a6810, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa6816, 46 },
    { 0x67ca6816, 99 },
    { 0xc9ca6818, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05a681d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82a6829, 95 },
    { 0, 0 },
    { 0x766a682b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9da682e, 61 },
    { 0xa6aa682f, 33 },
    { 0xc6da6830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9a6833, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7a6838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceea683e, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ea6844, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fa6849, 76 },
    { 0x946a684a, 34 },
    { 0x906a684a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa684f, 43 },
    { 0xd9aa684f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a6858, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4a685b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cea6860, 90 },
    { 0, 0 },
    { 0x70ba6862, 76 },
    { 0x2f2a6863, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cba6869, 95 },
    { 0xa4ba6869, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ba686e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5a6872, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4a6876, 76 },
    { 0, 0 },
    { 0x674a6878, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdda687b, 84 },
    { 0x302a687c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x704a687f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ca6886, 55 },
    { 0, 0 },
    { 0xa1ea6888, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeca688c, 90 },
    { 0x5eba688c, 46 },
    { 0x808a688e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722a6893, 90 },
    { 0, 0 },
    { 0x8f6a6895, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ba689f, 23 },
    { 0, 0 },
    { 0xc92a68a1, 77 },
    { 0x76da68a2, 90 },
    { 0xe87a68a3, 61 },
    { 0x2fca68a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601a68af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a68b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392a68b6, 61 },
    { 0xfea68b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41a68ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663a68c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1a68c7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a68cc, 33 },
    { 0, 0 },
    { 0x970a68ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0a68d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba68d6, 98 },
    { 0x31ca68d7, 76 },
    { 0x5d9a68d7, 90 },
    { 0, 0 },
    { 0x375a68da, 61 },
    { 0xd9da68db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806a68df, 98 },
    { 0, 0 },
    { 0xde9a68e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea68e4, 55 },
    { 0xdeba68e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13a68eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aa68fc, 95 },
    { 0xa41a68fd, 99 },
    { 0, 0 },
    { 0x6fba68ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ea6902, 80 },
    { 0x2b0a6903, 68 },
    { 0xe7fa6903, 43 },
    { 0xb83a6904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57a6908, 67 },
    { 0x1c1a6909, 95 },
    { 0xfb6a690a, 90 },
    { 0xaf4a690b, 98 },
    { 0xbaba690a, 99 },
    { 0xa1a6909, 88 },
    { 0x9aaa690e, 23 },
    { 0x4bfa690f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba6912, 38 },
    { 0xddea6913, 100 },
    { 0, 0 },
    { 0x3eba6915, 67 },
    { 0xad1a6916, 33 },
    { 0x7d3a6915, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0da6922, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53a6929, 43 },
    { 0xdd6a692a, 90 },
    { 0xa8ba692b, 95 },
    { 0x123a692c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5a6930, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a6933, 33 },
    { 0, 0 },
    { 0x2a3a6935, 66 },
    { 0x1fda6936, 23 },
    { 0, 0 },
    { 0xc1ea6938, 98 },
    { 0x8bba6938, 34 },
    { 0xacda693a, 44 },
    { 0x2c1a693b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45a6945, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa694a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1a694f, 61 },
    { 0xc5da6950, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaa6958, 33 },
    { 0x7cda6959, 95 },
    { 0xf80a695a, 61 },
    { 0xe5ba6958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ba695f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56aa6964, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x41da6967, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ca696b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8a696e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2a6971, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaba6979, 55 },
    { 0xff3a697a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeba697e, 90 },
    { 0xeb4a697f, 52 },
    { 0xb59a6980, 76 },
    { 0xa11a6981, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5a6984, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70a6989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9a698d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aaa6993, 90 },
    { 0x41ea6993, 33 },
    { 0xacea6995, 33 },
    { 0xc8fa6995, 76 },
    { 0x752a6997, 23 },
    { 0xe45a6998, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2a699f, 88 },
    { 0x67ea69a0, 91 },
    { 0, 0 },
    { 0x4d7a69a2, 75 },
    { 0, 0 },
    { 0x484a69a4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ea69a7, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a69ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391a69b4, 46 },
    { 0x55fa69b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31a69b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2a69bb, 33 },
    { 0x751a69bc, 43 },
    { 0xa69a69bd, 34 },
    { 0xee2a69bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aa69c5, 95 },
    { 0xe2a69c6, 90 },
    { 0x344a69c5, 80 },
    { 0x625a69c8, 43 },
    { 0x120a69c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06a69d0, 90 },
    { 0x193a69d1, 23 },
    { 0xb99a69d0, 61 },
    { 0xb7da69d3, 91 },
    { 0xd1ca69d4, 33 },
    { 0x14ba69d5, 67 },
    { 0xfc2a69d2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a69da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75a69df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6a69e7, 34 },
    { 0, 0 },
    { 0xec4a69e9, 95 },
    { 0, 0 },
    { 0x9e6a69eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2a69f0, 100 },
    { 0x313a69f1, 55 },
    { 0xc7da69f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fa69f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daa69fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76a69ff, 61 },
    { 0x5f5a6a00, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504a6a06, 67 },
    { 0xadfa6a07, 89 },
    { 0x174a6a08, 77 },
    { 0xd98a6a07, 33 },
    { 0xaa2a6a0a, 98 },
    { 0x788a6a09, 33 },
    { 0xf7a6a0c, 98 },
    { 0xef9a6a0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fba6a10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x388a6a13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caa6a19, 33 },
    { 0, 0 },
    { 0x2c4a6a1b, 78 },
    { 0xde4a6a1b, 61 },
    { 0x92aa6a1b, 78 },
    { 0x4fca6a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3a6a23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35a6a2c, 34 },
    { 0xda2a6a2d, 61 },
    { 0xd7a6a2e, 81 },
    { 0x216a6a2f, 34 },
    { 0x1faa6a2f, 98 },
    { 0x176a6a31, 77 },
    { 0xfe5a6a2c, 93 },
    { 0xb96a6a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620a6a37, 23 },
    { 0x940a6a38, 46 },
    { 0xef5a6a38, 98 },
    { 0x9a0a6a3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ba6a40, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5a6a43, 68 },
    { 0x1f7a6a44, 23 },
    { 0xd1fa6a45, 34 },
    { 0xb71a6a44, 61 },
    { 0x682a6a46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fa6a4b, 61 },
    { 0x805a6a4c, 52 },
    { 0x941a6a4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30a6a52, 72 },
    { 0, 0 },
    { 0xc74a6a54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81a6a5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3da6a65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a6a69, 91 },
    { 0x405a6a6a, 55 },
    { 0xd7a6a6b, 98 },
    { 0x8bca6a69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a6a70, 43 },
    { 0x3caa6a71, 81 },
    { 0xebca6a72, 43 },
    { 0xd6ea6a73, 61 },
    { 0x1d5a6a73, 33 },
    { 0x94ca6a75, 90 },
    { 0, 0 },
    { 0xb7fa6a77, 33 },
    { 0xe26a6a78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a6a7b, 38 },
    { 0xcdba6a7c, 61 },
    { 0x8daa6a7c, 33 },
    { 0x645a6a7e, 61 },
    { 0x81ea6a7f, 101 },
    { 0xb7ea6a7f, 75 },
    { 0xe20a6a7e, 34 },
    { 0xd1fa6a7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232a6a88, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1da6a8e, 90 },
    { 0x6a4a6a8e, 23 },
    { 0, 0 },
    { 0xdcca6a91, 23 },
    { 0x667a6a92, 76 },
    { 0x51ba6a93, 58 },
    { 0x2a3a6a92, 43 },
    { 0, 0 },
    { 0xd5a6a96, 38 },
    { 0xa02a6a97, 95 },
    { 0, 0 },
    { 0xccaa6a99, 78 },
    { 0, 0 },
    { 0x25ea6a9b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246a6aa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba6aa7, 43 },
    { 0, 0 },
    { 0x1d9a6aa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada6aad, 84 },
    { 0, 0 },
    { 0x5eaa6aaf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5a6ab6, 76 },
    { 0, 0 },
    { 0x369a6ab8, 76 },
    { 0, 0 },
    { 0x4dca6aba, 33 },
    { 0x36ca6abb, 91 },
    { 0x658a6abc, 76 },
    { 0xb4ca6abd, 66 },
    { 0x3aca6abc, 85 },
    { 0xd9ba6abd, 78 },
    { 0x4b2a6abd, 78 },
    { 0x710a6abd, 33 },
    { 0x46ca6abc, 64 },
    { 0, 0 },
    { 0x8f2a6ac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x895a6ac7, 55 },
    { 0, 0 },
    { 0xb20a6ac9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3a6ad2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa6ad5, 23 },
    { 0x578a6ad5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12a6add, 91 },
    { 0, 0 },
    { 0x9d5a6adf, 61 },
    { 0xa40a6ae0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7a6ae3, 99 },
    { 0x36ca6ae4, 55 },
    { 0x961a6ae4, 95 },
    { 0, 0 },
    { 0x366a6ae7, 90 },
    { 0xc60a6ae8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5a6aed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5a6af1, 99 },
    { 0, 0 },
    { 0xf16a6af3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56a6af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4a6afb, 90 },
    { 0, 0 },
    { 0x247a6afd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a6b01, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fa6b07, 52 },
    { 0xb4fa6b07, 95 },
    { 0, 0 },
    { 0x42da6b0a, 46 },
    { 0, 0 },
    { 0x1a6a6b0c, 90 },
    { 0, 0 },
    { 0xa4da6b0e, 61 },
    { 0, 0 },
    { 0x9da6b10, 66 },
    { 0x519a6b11, 97 },
    { 0xae7a6b10, 67 },
    { 0x680a6b13, 65 },
    { 0xec2a6b10, 33 },
    { 0, 0 },
    { 0x44ca6b16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7a6b19, 95 },
    { 0, 0 },
    { 0x17ba6b1b, 95 },
    { 0xe5ca6b1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775a6b2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18a6b2f, 90 },
    { 0x320a6b30, 23 },
    { 0xa1aa6b30, 98 },
    { 0x1a1a6b32, 66 },
    { 0x588a6b30, 23 },
    { 0xd8fa6b33, 98 },
    { 0x2d7a6b35, 23 },
    { 0x33fa6b30, 43 },
    { 0xfe5a6b37, 66 },
    { 0x416a6b35, 93 },
    { 0x5c6a6b39, 84 },
    { 0, 0 },
    { 0x63a6b3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffa6b41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7a6b44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74a6b49, 61 },
    { 0x7e7a6b4a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7a6b4d, 55 },
    { 0, 0 },
    { 0x77fa6b4f, 98 },
    { 0xd3a6b50, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92a6b53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8a6b57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fa6b5d, 90 },
    { 0x332a6b5e, 98 },
    { 0xc0aa6b5f, 33 },
    { 0x411a6b5d, 66 },
    { 0xe86a6b5d, 43 },
    { 0x8d9a6b62, 61 },
    { 0xe4fa6b5e, 80 },
    { 0x1e7a6b64, 90 },
    { 0xbb1a6b64, 23 },
    { 0xcf1a6b64, 98 },
    { 0x338a6b67, 66 },
    { 0xfca6b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ea6b6b, 61 },
    { 0, 0 },
    { 0xeea6b6d, 93 },
    { 0x17ba6b6e, 90 },
    { 0, 0 },
    { 0x208a6b70, 61 },
    { 0x102a6b71, 90 },
    { 0, 0 },
    { 0xc3ca6b73, 76 },
    { 0x39ba6b73, 33 },
    { 0, 0 },
    { 0x756a6b76, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a6b7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4a6b82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547a6b8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ea6b8f, 95 },
    { 0xbe8a6b90, 78 },
    { 0xf00a6b91, 55 },
    { 0xeaca6b92, 52 },
    { 0xca3a6b93, 19 },
    { 0x277a6b94, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea6b99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402a6b9d, 90 },
    { 0xf32a6b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5a6ba2, 43 },
    { 0, 0 },
    { 0xe64a6ba4, 95 },
    { 0x6efa6ba5, 98 },
    { 0xfaca6ba5, 33 },
    { 0x197a6ba7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6a6bab, 98 },
    { 0x65a6bac, 100 },
    { 0x5c2a6bad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x677a6bb0, 84 },
    { 0xc7aa6bb0, 69 },
    { 0x9baa6bb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1a6bb8, 87 },
    { 0, 0 },
    { 0x7e9a6bba, 77 },
    { 0xaefa6bba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a6bbf, 98 },
    { 0x309a6bbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240a6bc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba6bd0, 61 },
    { 0x664a6bd0, 76 },
    { 0xd8da6bd0, 90 },
    { 0x248a6bd1, 90 },
    { 0xe42a6bd4, 80 },
    { 0x23da6bd5, 61 },
    { 0x695a6bd6, 43 },
    { 0xdfba6bd7, 23 },
    { 0xf41a6bd7, 95 },
    { 0x7dba6bd7, 33 },
    { 0xb11a6bd8, 100 },
    { 0xd28a6bdb, 66 },
    { 0x666a6bdc, 49 },
    { 0, 0 },
    { 0xbeba6bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca6be1, 90 },
    { 0, 0 },
    { 0x719a6be3, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fa6bea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ea6bed, 36 },
    { 0, 0 },
    { 0x4cea6bef, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea6bf3, 90 },
    { 0x5eaa6bf3, 67 },
    { 0xff5a6bf4, 33 },
    { 0, 0 },
    { 0x1a2a6bf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84a6bfd, 33 },
    { 0x67da6bfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ea6c01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bda6c0a, 33 },
    { 0x4cba6c0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4a6c0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ca6c19, 76 },
    { 0xa71a6c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ca6c1d, 33 },
    { 0, 0 },
    { 0xfe4a6c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffa6c22, 61 },
    { 0x973a6c23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a6c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9a6c2b, 90 },
    { 0x9d9a6c2c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995a6c31, 95 },
    { 0, 0 },
    { 0x62ba6c33, 95 },
    { 0xc6aa6c34, 76 },
    { 0x6b1a6c35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477a6c3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca6c3e, 76 },
    { 0x7caa6c3f, 91 },
    { 0x5ada6c40, 90 },
    { 0x9c3a6c41, 61 },
    { 0xbeca6c3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ea6c47, 76 },
    { 0, 0 },
    { 0x684a6c49, 33 },
    { 0, 0 },
    { 0x854a6c4b, 95 },
    { 0x237a6c4c, 77 },
    { 0x3c7a6c4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77a6c54, 99 },
    { 0, 0 },
    { 0x64ea6c56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a6c5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffa6c61, 55 },
    { 0x58a6c62, 77 },
    { 0xde0a6c63, 33 },
    { 0x801a6c64, 78 },
    { 0x3a7a6c65, 55 },
    { 0x2b0a6c66, 85 },
    { 0xa90a6c67, 87 },
    { 0x19a6c67, 90 },
    { 0, 0 },
    { 0xc10a6c6a, 95 },
    { 0xb1a6c6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ea6c70, 46 },
    { 0x7a5a6c70, 90 },
    { 0, 0 },
    { 0x2dfa6c73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x334a6c76, 55 },
    { 0, 0 },
    { 0xb2aa6c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900a6c7d, 32 },
    { 0xbbda6c7e, 33 },
    { 0x355a6c7f, 84 },
    { 0xb9ba6c80, 93 },
    { 0xa7ea6c80, 76 },
    { 0x8eea6c81, 67 },
    { 0xac5a6c83, 55 },
    { 0xfefa6c81, 76 },
    { 0x9a6c80, 46 },
    { 0xbe4a6c82, 95 },
    { 0, 0 },
    { 0xca2a6c88, 33 },
    { 0, 0 },
    { 0xf24a6c8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3a6c8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3a6c97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dca6c9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56a6c9f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365a6ca9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a6caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x182a6cb2, 33 },
    { 0x2ffa6cb2, 98 },
    { 0xda6cb4, 76 },
    { 0xea5a6cb5, 33 },
    { 0xf8ea6cb3, 55 },
    { 0xd6a6cb3, 23 },
    { 0xf48a6cb8, 17 },
    { 0, 0 },
    { 0x759a6cba, 65 },
    { 0xc0ba6cbb, 98 },
    { 0, 0 },
    { 0xa50a6cbd, 33 },
    { 0xc11a6cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0a6cc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43a6cc4, 68 },
    { 0x36ea6cc4, 98 },
    { 0xa9aa6cc6, 98 },
    { 0x610a6cc7, 23 },
    { 0, 0 },
    { 0x47da6cc9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31a6ccc, 33 },
    { 0x7e3a6ccd, 98 },
    { 0x86aa6ccd, 55 },
    { 0x3aaa6ccc, 77 },
    { 0x811a6ccd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ea6cd7, 95 },
    { 0xa60a6cd8, 95 },
    { 0x776a6cd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835a6cde, 61 },
    { 0, 0 },
    { 0xb57a6ce0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0a6ce3, 100 },
    { 0x8eea6ce4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681a6ced, 46 },
    { 0, 0 },
    { 0xc81a6cef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2a6cf2, 61 },
    { 0x422a6cf3, 98 },
    { 0xbb0a6cf4, 90 },
    { 0xd04a6cf4, 33 },
    { 0, 0 },
    { 0x719a6cf7, 56 },
    { 0, 0 },
    { 0x74ea6cf9, 76 },
    { 0, 0 },
    { 0xffea6cfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688a6cff, 99 },
    { 0xbc5a6d00, 67 },
    { 0x87ca6d01, 43 },
    { 0xb31a6d00, 55 },
    { 0x611a6d03, 52 },
    { 0x281a6d01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a6d07, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36a6d14, 98 },
    { 0x5e9a6d15, 90 },
    { 0x2b7a6d16, 23 },
    { 0x93ea6d16, 33 },
    { 0x393a6d18, 81 },
    { 0x357a6d16, 98 },
    { 0x6dba6d1a, 33 },
    { 0x5e1a6d19, 46 },
    { 0xf55a6d1c, 80 },
    { 0xf4ea6d1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1a6d20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a6d24, 34 },
    { 0xbfaa6d25, 33 },
    { 0x3cba6d26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebca6d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4a6d2f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca6d34, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82a6d37, 100 },
    { 0xa80a6d37, 100 },
    { 0xb10a6d38, 90 },
    { 0, 0 },
    { 0x8c6a6d3b, 91 },
    { 0, 0 },
    { 0x4c1a6d3d, 87 },
    { 0x3aaa6d3d, 33 },
    { 0x639a6d3f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a6d45, 33 },
    { 0xe5a6d46, 90 },
    { 0x6f2a6d47, 90 },
    { 0, 0 },
    { 0x5da6d49, 66 },
    { 0x99da6d49, 23 },
    { 0xdf8a6d4b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5a6d50, 98 },
    { 0, 0 },
    { 0x9ba6d52, 75 },
    { 0xae5a6d53, 33 },
    { 0x333a6d54, 33 },
    { 0xd7a6d54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca6d5b, 100 },
    { 0x8da6d5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7a6d61, 46 },
    { 0xc14a6d61, 87 },
    { 0, 0 },
    { 0x4eda6d64, 34 },
    { 0x3b8a6d65, 90 },
    { 0x608a6d66, 98 },
    { 0x4d6a6d67, 81 },
    { 0x800a6d68, 93 },
    { 0x36ea6d66, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684a6d6e, 100 },
    { 0x527a6d6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6a6d75, 23 },
    { 0x60ba6d76, 61 },
    { 0x2fea6d76, 77 },
    { 0xec6a6d75, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4a6d7c, 33 },
    { 0x6c8a6d7d, 52 },
    { 0xc4ca6d7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7a6d81, 78 },
    { 0xa7da6d81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5a6d8a, 75 },
    { 0, 0 },
    { 0xe0aa6d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda6d90, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x541a6d93, 90 },
    { 0x575a6d94, 76 },
    { 0x696a6d94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295a6d99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904a6d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7a6da1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ca6dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43a6db4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45a6db8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1a6dbc, 98 },
    { 0xadfa6dbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84a6dc0, 23 },
    { 0x7cda6dc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6a6dc8, 23 },
    { 0x3b0a6dc9, 61 },
    { 0x60fa6dc8, 98 },
    { 0xe4a6dc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38a6dcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a6dd4, 33 },
    { 0, 0 },
    { 0x801a6dd6, 76 },
    { 0xf6a6dd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4a6dda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194a6ddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0a6de5, 95 },
    { 0xbdca6de5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03a6deb, 95 },
    { 0x7fba6dec, 90 },
    { 0x19aa6ded, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834a6df1, 49 },
    { 0x41a6df1, 61 },
    { 0x219a6df1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1a6df6, 55 },
    { 0xce1a6df7, 17 },
    { 0, 0 },
    { 0x205a6df9, 52 },
    { 0xc04a6dfa, 99 },
    { 0x285a6df9, 76 },
    { 0x5b5a6dfc, 90 },
    { 0, 0 },
    { 0xae8a6dfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755a6e02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eba6e0c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4a6e0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ea6e15, 90 },
    { 0, 0 },
    { 0x7ada6e17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5a6e1a, 33 },
    { 0xba1a6e1a, 76 },
    { 0xca2a6e1c, 61 },
    { 0xc3fa6e1a, 78 },
    { 0xb56a6e1e, 33 },
    { 0x22ba6e1e, 95 },
    { 0x2b0a6e1a, 95 },
    { 0, 0 },
    { 0x3aa6e22, 95 },
    { 0x99aa6e23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcba6e2b, 23 },
    { 0x20ba6e2c, 61 },
    { 0xcd8a6e2d, 17 },
    { 0x6eaa6e2e, 90 },
    { 0xd87a6e2c, 34 },
    { 0xc4ca6e30, 76 },
    { 0, 0 },
    { 0xb37a6e32, 90 },
    { 0, 0 },
    { 0x194a6e34, 34 },
    { 0x267a6e35, 61 },
    { 0, 0 },
    { 0xadfa6e37, 43 },
    { 0, 0 },
    { 0x9bda6e39, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea6e3c, 67 },
    { 0x9ada6e3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12a6e40, 33 },
    { 0, 0 },
    { 0x3f8a6e42, 17 },
    { 0x392a6e42, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91a6e48, 33 },
    { 0, 0 },
    { 0xce0a6e4a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67da6e52, 34 },
    { 0x195a6e53, 93 },
    { 0xc7ea6e52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aa6e58, 91 },
    { 0xf75a6e59, 77 },
    { 0, 0 },
    { 0xea9a6e5b, 99 },
    { 0x6d3a6e5b, 40 },
    { 0, 0 },
    { 0x413a6e5e, 46 },
    { 0, 0 },
    { 0xe16a6e60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06a6e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8da6e69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aa6e78, 76 },
    { 0x76fa6e79, 33 },
    { 0xdb2a6e7a, 90 },
    { 0x4bea6e7b, 98 },
    { 0, 0 },
    { 0xffba6e7d, 90 },
    { 0x9c4a6e7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fa6e81, 76 },
    { 0xd87a6e82, 61 },
    { 0xd60a6e83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba6e87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188a6e8c, 34 },
    { 0, 0 },
    { 0xc42a6e8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a6e93, 33 },
    { 0x41ca6e94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522a6e99, 100 },
    { 0x5fa6e9a, 65 },
    { 0xb89a6e99, 76 },
    { 0, 0 },
    { 0xd24a6e9d, 90 },
    { 0, 0 },
    { 0x890a6e9f, 68 },
    { 0x114a6ea0, 33 },
    { 0x403a6ea1, 98 },
    { 0x379a6ea2, 88 },
    { 0xb35a6ea3, 98 },
    { 0, 0 },
    { 0x9e7a6ea5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ea6ea8, 23 },
    { 0, 0 },
    { 0xf27a6eaa, 61 },
    { 0x6cca6eab, 61 },
    { 0, 0 },
    { 0x29ba6ead, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x144a6eb0, 90 },
    { 0, 0 },
    { 0x48aa6eb2, 34 },
    { 0, 0 },
    { 0x448a6eb4, 91 },
    { 0, 0 },
    { 0x6e9a6eb6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba6ebb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145a6ec2, 90 },
    { 0xe96a6ec3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40a6ec6, 84 },
    { 0, 0 },
    { 0x8cfa6ec8, 98 },
    { 0xe32a6ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148a6ecf, 90 },
    { 0x8ea6ecf, 61 },
    { 0x2f4a6ed0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45a6edb, 43 },
    { 0, 0 },
    { 0x40a6edd, 33 },
    { 0xcfea6ede, 55 },
    { 0, 0 },
    { 0x15ea6ee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ea6ef0, 95 },
    { 0xb1da6ef1, 55 },
    { 0, 0 },
    { 0xe1fa6ef3, 95 },
    { 0x104a6ef3, 55 },
    { 0xb7aa6ef4, 90 },
    { 0x817a6ef6, 78 },
    { 0x488a6ef5, 97 },
    { 0, 0 },
    { 0x1e0a6ef9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caa6efd, 88 },
    { 0xfb3a6efe, 46 },
    { 0, 0 },
    { 0xbe1a6f00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06a6f05, 55 },
    { 0xff1a6f06, 87 },
    { 0x6c2a6f07, 52 },
    { 0xb07a6f07, 90 },
    { 0x763a6f09, 93 },
    { 0x15ea6f0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a6f0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaea6f10, 65 },
    { 0x951a6f11, 95 },
    { 0x756a6f12, 87 },
    { 0, 0 },
    { 0x88a6f14, 98 },
    { 0, 0 },
    { 0xc84a6f16, 77 },
    { 0x70ca6f17, 33 },
    { 0, 0 },
    { 0x5bda6f19, 98 },
    { 0x107a6f1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfa6f1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aa6f21, 95 },
    { 0xb91a6f22, 34 },
    { 0xa44a6f22, 66 },
    { 0x94da6f24, 33 },
    { 0, 0 },
    { 0x1aea6f26, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53a6f2f, 95 },
    { 0x93ca6f30, 93 },
    { 0xfa0a6f2f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1a6f34, 98 },
    { 0x76da6f35, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9a6f39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cda6f40, 33 },
    { 0xb38a6f41, 90 },
    { 0x588a6f41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6f46, 33 },
    { 0xe62a6f47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea6f4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5a6f52, 23 },
    { 0x569a6f52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9da6f5c, 43 },
    { 0, 0 },
    { 0xc15a6f5e, 61 },
    { 0xa25a6f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85da6f62, 46 },
    { 0x383a6f63, 76 },
    { 0xbeea6f64, 90 },
    { 0xde0a6f63, 90 },
    { 0, 0 },
    { 0x375a6f67, 43 },
    { 0x784a6f67, 90 },
    { 0x2aba6f69, 53 },
    { 0xa7ca6f68, 61 },
    { 0x51ca6f6b, 90 },
    { 0x128a6f69, 90 },
    { 0x144a6f6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca6f70, 33 },
    { 0xcb0a6f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa6f77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65a6f7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa6f7d, 33 },
    { 0x12ea6f7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81a6f81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30a6f85, 66 },
    { 0x5afa6f86, 19 },
    { 0x122a6f86, 34 },
    { 0x22ea6f88, 93 },
    { 0xf89a6f88, 23 },
    { 0xf1da6f89, 76 },
    { 0xab5a6f8b, 87 },
    { 0x48aa6f8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135a6f91, 61 },
    { 0x586a6f92, 95 },
    { 0x6f4a6f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8a6f9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaa6fa1, 85 },
    { 0xd0a6fa2, 90 },
    { 0x143a6fa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aa6fa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ca6fab, 90 },
    { 0, 0 },
    { 0x530a6fad, 90 },
    { 0, 0 },
    { 0x478a6faf, 90 },
    { 0, 0 },
    { 0xec2a6fb1, 90 },
    { 0x6b5a6fb2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456a6fb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5a6fb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2a6fc1, 55 },
    { 0, 0 },
    { 0x575a6fc3, 65 },
    { 0x9eca6fc4, 90 },
    { 0, 0 },
    { 0x4c3a6fc6, 85 },
    { 0, 0 },
    { 0x385a6fc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fea6fcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30aa6fcf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a6fd2, 100 },
    { 0xd42a6fd3, 36 },
    { 0xa0ba6fd2, 75 },
    { 0, 0 },
    { 0x5d4a6fd6, 90 },
    { 0xb08a6fd7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a6fda, 75 },
    { 0xf7aa6fda, 33 },
    { 0x50ea6fdc, 34 },
    { 0xaca6fdd, 88 },
    { 0xd03a6fde, 43 },
    { 0xf4ba6fdd, 84 },
    { 0x199a6fdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x866a6fe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfba6fe9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a6ff2, 96 },
    { 0xf80a6ff2, 46 },
    { 0x955a6ff4, 55 },
    { 0x1eba6ff2, 90 },
    { 0x450a6ff5, 67 },
    { 0xa3ea6ff4, 90 },
    { 0x122a6ff8, 98 },
    { 0xf57a6ff9, 52 },
    { 0, 0 },
    { 0xd8a6ffb, 23 },
    { 0, 0 },
    { 0x672a6ffd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405a7001, 90 },
    { 0, 0 },
    { 0x8c5a7003, 61 },
    { 0, 0 },
    { 0xf55a7005, 81 },
    { 0x2aca7006, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1a700d, 68 },
    { 0x2f7a700d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73a7015, 90 },
    { 0xc57a7016, 99 },
    { 0xe0a7017, 61 },
    { 0xb91a7018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1a701d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980a7024, 100 },
    { 0xd93a7024, 90 },
    { 0xdd0a7026, 64 },
    { 0xe8aa7027, 66 },
    { 0x4b1a7028, 55 },
    { 0xf9da7027, 91 },
    { 0, 0 },
    { 0x919a702b, 90 },
    { 0, 0 },
    { 0x49da702d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x919a7030, 90 },
    { 0x5b4a7030, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3a7036, 98 },
    { 0xd43a7036, 23 },
    { 0xab2a7038, 23 },
    { 0x313a7038, 61 },
    { 0x1a5a703a, 90 },
    { 0xdda703a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0a703e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ba704f, 33 },
    { 0x15ca7050, 90 },
    { 0, 0 },
    { 0xd97a7052, 33 },
    { 0x4e6a7052, 90 },
    { 0x99aa7052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcea705b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaa7060, 98 },
    { 0x7c4a7061, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceea7067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ea706e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5a7071, 69 },
    { 0x86ca7072, 61 },
    { 0, 0 },
    { 0x5cfa7074, 33 },
    { 0x9eda7075, 55 },
    { 0x7e8a7076, 61 },
    { 0x8b6a7077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275a707c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ca707f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ea7084, 99 },
    { 0x8d1a7085, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a7088, 33 },
    { 0x9c9a7088, 61 },
    { 0, 0 },
    { 0x2da708b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2a708f, 42 },
    { 0x9d5a708f, 23 },
    { 0xdaaa7091, 56 },
    { 0xc1fa7092, 95 },
    { 0, 0 },
    { 0x2e4a7094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3a7098, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09a709e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0a70a3, 33 },
    { 0, 0 },
    { 0x66ea70a5, 98 },
    { 0xd84a70a5, 53 },
    { 0xa02a70a7, 33 },
    { 0, 0 },
    { 0x47da70a9, 46 },
    { 0, 0 },
    { 0x1ffa70ab, 43 },
    { 0xb21a70ac, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491a70b4, 55 },
    { 0xf46a70b5, 65 },
    { 0x97ca70b6, 61 },
    { 0x406a70b7, 76 },
    { 0xf91a70b8, 76 },
    { 0x2f3a70b9, 46 },
    { 0x1bea70ba, 66 },
    { 0xa2da70bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222a70c0, 90 },
    { 0xfeca70c1, 93 },
    { 0xfbea70c2, 43 },
    { 0, 0 },
    { 0xfcba70c4, 90 },
    { 0, 0 },
    { 0x458a70c6, 91 },
    { 0xf35a70c6, 34 },
    { 0x9aaa70c6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60a70cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2a70cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3a70d8, 40 },
    { 0xdeea70d8, 55 },
    { 0xb10a70da, 33 },
    { 0, 0 },
    { 0xad9a70dc, 76 },
    { 0, 0 },
    { 0x52da70de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8a70e4, 55 },
    { 0xaba70e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7a70ea, 95 },
    { 0xe96a70eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8a70ee, 98 },
    { 0x331a70ef, 98 },
    { 0x60fa70ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbca70f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75a70f9, 95 },
    { 0xa99a70fa, 98 },
    { 0x2d4a70fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x899a70fe, 84 },
    { 0x53a70ff, 90 },
    { 0, 0 },
    { 0x990a7101, 90 },
    { 0x8e6a7102, 33 },
    { 0xac6a7103, 23 },
    { 0xcc1a7104, 77 },
    { 0x363a7103, 93 },
    { 0x6c9a7106, 61 },
    { 0, 0 },
    { 0x90aa7108, 33 },
    { 0, 0 },
    { 0x7ada710a, 68 },
    { 0x79da710b, 67 },
    { 0, 0 },
    { 0xca0a710d, 99 },
    { 0x5a1a710d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25a7111, 78 },
    { 0xa70a7112, 46 },
    { 0xce5a7113, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa7116, 93 },
    { 0x1f3a7117, 61 },
    { 0xda7116, 90 },
    { 0xc15a7119, 100 },
    { 0x758a7117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea711f, 76 },
    { 0x93da7120, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0da7123, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8a7126, 98 },
    { 0x392a7126, 61 },
    { 0x2cea7126, 76 },
    { 0, 0 },
    { 0xe61a712a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aa712d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ca7131, 61 },
    { 0x735a7132, 83 },
    { 0xa57a7133, 46 },
    { 0, 0 },
    { 0xd7aa7135, 76 },
    { 0x9cea7136, 55 },
    { 0, 0 },
    { 0x9bfa7138, 90 },
    { 0xa0da7139, 75 },
    { 0x87aa7139, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17a713f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65a7148, 34 },
    { 0x1b4a7149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08a714d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbda7152, 23 },
    { 0x22a7153, 33 },
    { 0xff7a7154, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ca7158, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2a715e, 33 },
    { 0, 0 },
    { 0x182a7160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a7163, 98 },
    { 0, 0 },
    { 0x778a7165, 66 },
    { 0, 0 },
    { 0x437a7167, 61 },
    { 0xbf7a7168, 33 },
    { 0, 0 },
    { 0xf02a716a, 94 },
    { 0x2dca716b, 96 },
    { 0xd66a716c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fa7171, 95 },
    { 0, 0 },
    { 0xb5ea7173, 17 },
    { 0x62aa7174, 95 },
    { 0x10da7175, 23 },
    { 0xd53a7176, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x742a7179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x160a717c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a7181, 76 },
    { 0xe26a7182, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba7186, 33 },
    { 0xdb2a7187, 90 },
    { 0, 0 },
    { 0xae7a7189, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425a718d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca7195, 34 },
    { 0x5caa7196, 90 },
    { 0xceba7197, 83 },
    { 0, 0 },
    { 0xa56a7199, 61 },
    { 0xccfa719a, 34 },
    { 0, 0 },
    { 0xad9a719c, 90 },
    { 0x712a719d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690a71a1, 96 },
    { 0xcf0a71a2, 23 },
    { 0, 0 },
    { 0x22ca71a4, 100 },
    { 0xd55a71a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66a71a8, 61 },
    { 0x79ea71a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363a71ad, 53 },
    { 0x6b7a71ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa71b1, 75 },
    { 0, 0 },
    { 0x671a71b3, 83 },
    { 0x32ba71b4, 56 },
    { 0, 0 },
    { 0x23da71b6, 84 },
    { 0, 0 },
    { 0xc6ba71b8, 33 },
    { 0x672a71b9, 95 },
    { 0xe58a71b8, 66 },
    { 0x4e0a71bb, 81 },
    { 0xf3ca71b8, 76 },
    { 0xcb4a71bd, 78 },
    { 0x497a71b8, 23 },
    { 0xd3aa71bf, 88 },
    { 0xe95a71bf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1a71c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdca71c6, 77 },
    { 0xa96a71c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69a71ca, 90 },
    { 0x4b2a71ca, 68 },
    { 0x8bba71cb, 95 },
    { 0, 0 },
    { 0xd44a71ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ca71d1, 33 },
    { 0x50fa71d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fa71d6, 34 },
    { 0, 0 },
    { 0xd85a71d8, 43 },
    { 0x60fa71d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a71dd, 87 },
    { 0x2dfa71dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160a71e4, 33 },
    { 0x9fca71e5, 55 },
    { 0, 0 },
    { 0xed1a71e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933a71ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1a71f2, 90 },
    { 0x874a71f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfa71f7, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa71fc, 34 },
    { 0x4e0a71fc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228a7202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fa7205, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daa720b, 61 },
    { 0xb44a720b, 76 },
    { 0, 0 },
    { 0x913a720e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ca7213, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ba721a, 68 },
    { 0x80ea721b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aa7222, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efa7227, 23 },
    { 0x47a7228, 66 },
    { 0xa97a7227, 33 },
    { 0, 0 },
    { 0x4d6a722b, 95 },
    { 0x2c3a722c, 55 },
    { 0, 0 },
    { 0x4a5a722e, 42 },
    { 0, 0 },
    { 0xf6ba7230, 78 },
    { 0xd2ea7231, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76a7235, 91 },
    { 0, 0 },
    { 0xff7a7237, 93 },
    { 0, 0 },
    { 0xb8ba7239, 61 },
    { 0, 0 },
    { 0x399a723b, 53 },
    { 0x203a723c, 95 },
    { 0x6efa723d, 90 },
    { 0x318a723e, 55 },
    { 0xe06a723b, 61 },
    { 0xd21a7240, 87 },
    { 0x258a7241, 40 },
    { 0x123a7240, 84 },
    { 0xfcfa7243, 55 },
    { 0xa84a7241, 100 },
    { 0x755a7245, 90 },
    { 0x57fa7245, 23 },
    { 0, 0 },
    { 0x9b3a7248, 95 },
    { 0xf83a7249, 36 },
    { 0xac4a724a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77aa7252, 91 },
    { 0, 0 },
    { 0xa14a7254, 55 },
    { 0, 0 },
    { 0x500a7256, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a725a, 37 },
    { 0xa73a725a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9a7260, 61 },
    { 0, 0 },
    { 0xb71a7262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24a7265, 34 },
    { 0, 0 },
    { 0xf05a7267, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bba7274, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da7278, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cda727d, 65 },
    { 0, 0 },
    { 0x2e1a727f, 97 },
    { 0xe1fa7280, 34 },
    { 0xa14a7281, 98 },
    { 0, 0 },
    { 0xc18a7283, 66 },
    { 0x291a7284, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dea728b, 34 },
    { 0x8b1a728b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fa7290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21da7295, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ba7298, 53 },
    { 0x4a3a7298, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110a729e, 84 },
    { 0x8eda729e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeea72aa, 34 },
    { 0x51ba72aa, 61 },
    { 0x82ca72ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbba72b3, 91 },
    { 0, 0 },
    { 0x807a72b5, 76 },
    { 0, 0 },
    { 0x511a72b7, 91 },
    { 0, 0 },
    { 0x7e7a72b9, 23 },
    { 0xd83a72b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a72bf, 47 },
    { 0, 0 },
    { 0xa99a72c1, 33 },
    { 0, 0 },
    { 0x35ca72c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4a72c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x361a72c9, 33 },
    { 0x253a72ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386a72ce, 90 },
    { 0xea9a72cf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5a72d7, 61 },
    { 0xe03a72d7, 36 },
    { 0x5a3a72d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ba72dc, 90 },
    { 0x124a72dc, 78 },
    { 0x6aba72dc, 61 },
    { 0x6c4a72dc, 61 },
    { 0x53fa72e0, 33 },
    { 0xf60a72df, 76 },
    { 0x738a72dd, 17 },
    { 0x94ca72e3, 95 },
    { 0, 0 },
    { 0x27da72e5, 90 },
    { 0xe13a72e5, 38 },
    { 0xe99a72e7, 78 },
    { 0, 0 },
    { 0x572a72e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439a72ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85da72f4, 99 },
    { 0x355a72f4, 90 },
    { 0xf44a72f6, 100 },
    { 0x5a9a72f4, 33 },
    { 0xd6ea72f4, 95 },
    { 0xa7ca72f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ea72fc, 3 },
    { 0x94ca72fc, 95 },
    { 0, 0 },
    { 0x2c3a72ff, 67 },
    { 0xe71a7300, 61 },
    { 0x563a7301, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88a7304, 33 },
    { 0, 0 },
    { 0x6dba7306, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36da730e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a7311, 23 },
    { 0xb7fa7312, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaca7317, 90 },
    { 0xe6a7318, 88 },
    { 0x6a4a7319, 33 },
    { 0x32da731a, 90 },
    { 0, 0 },
    { 0x96ca731c, 87 },
    { 0, 0 },
    { 0x6ffa731e, 33 },
    { 0x657a731f, 77 },
    { 0x555a731e, 98 },
    { 0xeb6a7321, 23 },
    { 0xb38a7321, 84 },
    { 0, 0 },
    { 0x5faa7324, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a732a, 23 },
    { 0x828a732b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679a7330, 65 },
    { 0xbd7a7331, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cba733b, 43 },
    { 0x316a733c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fa7347, 61 },
    { 0x800a7348, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aa734e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fca7353, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95a7357, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2aa735c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6a735f, 90 },
    { 0x94ca735f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aa7364, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77a7367, 98 },
    { 0x8c4a7368, 23 },
    { 0, 0 },
    { 0x76ea736a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80a7373, 95 },
    { 0xfcca7373, 98 },
    { 0x638a7375, 85 },
    { 0, 0 },
    { 0xd04a7377, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470a737b, 23 },
    { 0x9a5a737c, 65 },
    { 0x9b1a737d, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea7381, 76 },
    { 0x9e2a7382, 61 },
    { 0, 0 },
    { 0xb73a7384, 33 },
    { 0, 0 },
    { 0xa68a7386, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397a738b, 99 },
    { 0x424a738c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x574a738f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2a7393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3a7397, 95 },
    { 0, 0 },
    { 0xa4ca7399, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5a739d, 46 },
    { 0xda4a739e, 77 },
    { 0xcca739f, 98 },
    { 0, 0 },
    { 0xd00a73a1, 67 },
    { 0x41fa73a2, 23 },
    { 0xd58a73a3, 90 },
    { 0, 0 },
    { 0xc42a73a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ea73aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45a73b5, 61 },
    { 0, 0 },
    { 0x64aa73b7, 95 },
    { 0x1c2a73b8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x685a73bb, 23 },
    { 0xf1aa73bc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aba73c3, 23 },
    { 0x8baa73c4, 90 },
    { 0x782a73c4, 98 },
    { 0x3c8a73c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a73c9, 93 },
    { 0xf6ca73c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678a73cf, 23 },
    { 0x8f7a73cf, 33 },
    { 0x44ca73d0, 100 },
    { 0, 0 },
    { 0x15da73d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca73da, 33 },
    { 0x88fa73da, 76 },
    { 0x724a73dc, 66 },
    { 0x94a73dd, 98 },
    { 0x70aa73da, 34 },
    { 0, 0 },
    { 0x430a73e0, 78 },
    { 0x6ada73e1, 61 },
    { 0xc75a73e2, 76 },
    { 0x147a73e2, 23 },
    { 0, 0 },
    { 0x973a73e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297a73ed, 61 },
    { 0x1bba73ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5a73f2, 90 },
    { 0, 0 },
    { 0xd0a73f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0a73fa, 61 },
    { 0, 0 },
    { 0x850a73fc, 61 },
    { 0xa21a73fd, 36 },
    { 0x407a73fe, 55 },
    { 0x768a73ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdca7405, 99 },
    { 0x184a7405, 78 },
    { 0x371a7407, 98 },
    { 0x1eda7408, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7aa740f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7a7412, 33 },
    { 0, 0 },
    { 0x7d6a7414, 55 },
    { 0, 0 },
    { 0x2d0a7416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35aa7420, 99 },
    { 0, 0 },
    { 0x55ca7422, 61 },
    { 0x33aa7422, 23 },
    { 0x538a7424, 61 },
    { 0xf4ca7425, 98 },
    { 0xa78a7426, 95 },
    { 0xceca7425, 98 },
    { 0x293a7425, 61 },
    { 0x9d3a7429, 43 },
    { 0xfa9a742a, 77 },
    { 0x184a742a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaea7431, 84 },
    { 0, 0 },
    { 0xceaa7433, 95 },
    { 0x690a7434, 61 },
    { 0x15ea7435, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76a7438, 90 },
    { 0xbe8a7439, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633a7440, 61 },
    { 0x9c0a7441, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773a7448, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223a744f, 98 },
    { 0, 0 },
    { 0xcaaa7451, 33 },
    { 0x5b7a7452, 33 },
    { 0xd67a7451, 98 },
    { 0x797a7454, 56 },
    { 0x3f4a7455, 64 },
    { 0, 0 },
    { 0x22aa7457, 75 },
    { 0x139a7458, 98 },
    { 0x96ea7458, 78 },
    { 0xec6a745a, 90 },
    { 0x211a745b, 33 },
    { 0x2bda7459, 93 },
    { 0, 0 },
    { 0xaeba745e, 98 },
    { 0x318a745f, 93 },
    { 0xd79a745f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbea7463, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638a7468, 23 },
    { 0xdf0a7469, 93 },
    { 0x4eaa7469, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8a7475, 68 },
    { 0x258a7475, 66 },
    { 0x4b9a7475, 98 },
    { 0x3afa7476, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a747b, 40 },
    { 0x7f1a747b, 43 },
    { 0x17a747b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30a7480, 98 },
    { 0x1f1a7480, 43 },
    { 0, 0 },
    { 0xbe8a7483, 95 },
    { 0x7fea7483, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefa748e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3a7496, 33 },
    { 0x6cda7497, 23 },
    { 0x587a7497, 76 },
    { 0xdcca7499, 33 },
    { 0, 0 },
    { 0xa39a749b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a74a3, 98 },
    { 0x1a0a74a4, 90 },
    { 0xc78a74a3, 33 },
    { 0x716a74a6, 17 },
    { 0xf30a74a4, 85 },
    { 0x17aa74a8, 55 },
    { 0xc26a74a9, 76 },
    { 0x6e3a74aa, 23 },
    { 0x1eaa74aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39a74b0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ca74b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981a74b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6a74c0, 91 },
    { 0, 0 },
    { 0x44ba74c2, 23 },
    { 0, 0 },
    { 0x74da74c4, 52 },
    { 0, 0 },
    { 0xe95a74c6, 97 },
    { 0x323a74c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1a74ca, 78 },
    { 0, 0 },
    { 0x9aca74cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ca74cf, 61 },
    { 0, 0 },
    { 0x972a74d1, 98 },
    { 0xbcda74d1, 34 },
    { 0xe05a74d3, 94 },
    { 0xb43a74d4, 12 },
    { 0xcbfa74d5, 95 },
    { 0x17a74d5, 23 },
    { 0x6aba74d7, 61 },
    { 0, 0 },
    { 0x3aaa74d9, 76 },
    { 0, 0 },
    { 0xdaba74db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975a74df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199a74e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7a74e6, 67 },
    { 0xd9ea74e6, 43 },
    { 0x4a3a74e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x973a74eb, 33 },
    { 0xfba74eb, 99 },
    { 0, 0 },
    { 0x837a74ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4a74f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250a7502, 43 },
    { 0x703a7503, 84 },
    { 0xe6a7503, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76a7507, 90 },
    { 0xbbaa7508, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x202a750b, 76 },
    { 0x487a750b, 24 },
    { 0, 0 },
    { 0x47da750e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660a751c, 90 },
    { 0x1c8a751d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a7520, 98 },
    { 0x1f0a7521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48a7524, 61 },
    { 0, 0 },
    { 0x654a7526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x344a7529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea752e, 95 },
    { 0x998a752f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dba7532, 61 },
    { 0x9b3a7533, 52 },
    { 0x2aa7533, 90 },
    { 0, 0 },
    { 0xe32a7536, 98 },
    { 0xa87a7536, 33 },
    { 0x94ca7537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ca753b, 33 },
    { 0x8c0a753c, 100 },
    { 0xb69a753b, 98 },
    { 0xc14a753c, 23 },
    { 0x7a2a753f, 90 },
    { 0xb6fa7540, 87 },
    { 0x3a1a7541, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa7544, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a754c, 95 },
    { 0x15ba754d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa7551, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4a7554, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x449a7557, 33 },
    { 0x154a7558, 81 },
    { 0xae5a7559, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3a755e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x477a7561, 33 },
    { 0, 0 },
    { 0x110a7563, 93 },
    { 0, 0 },
    { 0x535a7565, 78 },
    { 0x719a7566, 95 },
    { 0x4c4a7566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73da756d, 95 },
    { 0, 0 },
    { 0x8f5a756f, 61 },
    { 0x505a7570, 90 },
    { 0x21aa7571, 46 },
    { 0xffea7570, 34 },
    { 0xf11a7572, 100 },
    { 0x971a7574, 40 },
    { 0x4b8a7574, 23 },
    { 0xc20a756f, 91 },
    { 0xeb6a7577, 77 },
    { 0x599a7578, 77 },
    { 0x265a7572, 33 },
    { 0xfd2a7576, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135a757f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59a7585, 33 },
    { 0xf27a7585, 34 },
    { 0xe05a7587, 77 },
    { 0x6b0a7588, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2a758b, 90 },
    { 0x7b7a758c, 93 },
    { 0x2e8a758d, 84 },
    { 0, 0 },
    { 0x77da758f, 95 },
    { 0xc95a7590, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fa759a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fa75a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fa75a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a75a8, 95 },
    { 0xd69a75a9, 55 },
    { 0xc7a75a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103a75b4, 90 },
    { 0, 0 },
    { 0x779a75b6, 98 },
    { 0xd5ba75b7, 65 },
    { 0, 0 },
    { 0x874a75b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40a75bf, 98 },
    { 0, 0 },
    { 0x736a75c1, 95 },
    { 0xe48a75c2, 33 },
    { 0x5d1a75c3, 43 },
    { 0x66ba75c4, 34 },
    { 0x57a75c2, 33 },
    { 0xbb3a75c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3a75ca, 99 },
    { 0x369a75cb, 61 },
    { 0x216a75cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca75cf, 34 },
    { 0, 0 },
    { 0xafca75d1, 23 },
    { 0xb7ba75d2, 23 },
    { 0x769a75d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ca75d7, 98 },
    { 0xbc3a75d7, 87 },
    { 0, 0 },
    { 0xda2a75da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6a75de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5a75e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ba75e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02a75ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1a75ed, 101 },
    { 0xa90a75ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2aa75f3, 33 },
    { 0x584a75f4, 95 },
    { 0, 0 },
    { 0x5b2a75f6, 61 },
    { 0x78a75f6, 43 },
    { 0xe77a75f6, 91 },
    { 0xf12a75f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67a75fe, 91 },
    { 0, 0 },
    { 0x3d5a7600, 90 },
    { 0x906a7601, 34 },
    { 0x7fea7602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7605, 61 },
    { 0xbdda7606, 95 },
    { 0xafda7607, 65 },
    { 0x9eca7606, 23 },
    { 0, 0 },
    { 0xe29a760a, 55 },
    { 0x83da760b, 34 },
    { 0x9b5a760c, 36 },
    { 0xfb3a760d, 76 },
    { 0x462a760e, 90 },
    { 0xd71a760f, 61 },
    { 0xb94a7610, 61 },
    { 0xe24a7611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61a7615, 98 },
    { 0, 0 },
    { 0xc76a7617, 90 },
    { 0xaada7618, 23 },
    { 0x61a7618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba761c, 98 },
    { 0xd59a761d, 98 },
    { 0x830a761e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1a7624, 61 },
    { 0x6e1a7625, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1a762d, 55 },
    { 0, 0 },
    { 0xc7aa762f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x749a7632, 46 },
    { 0x650a7633, 90 },
    { 0x14ea7634, 84 },
    { 0x5dda7632, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951a7639, 33 },
    { 0x9fda763a, 87 },
    { 0x478a763b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2a763e, 61 },
    { 0xd49a763f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30a764c, 90 },
    { 0, 0 },
    { 0xd3aa764e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ba7654, 85 },
    { 0, 0 },
    { 0xb6ea7656, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37a765a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284a7661, 65 },
    { 0, 0 },
    { 0xf8ba7663, 23 },
    { 0xe7da7664, 33 },
    { 0xdaaa7665, 67 },
    { 0x502a7666, 90 },
    { 0, 0 },
    { 0x920a7668, 90 },
    { 0xf98a7669, 17 },
    { 0, 0 },
    { 0xa2da766b, 61 },
    { 0xb31a766b, 33 },
    { 0x995a766c, 23 },
    { 0x4e2a766c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa7671, 23 },
    { 0, 0 },
    { 0x58da7673, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46a7679, 98 },
    { 0x399a767a, 98 },
    { 0, 0 },
    { 0x1f2a767c, 90 },
    { 0xb87a767c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x170a7680, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ea7684, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ea768a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ca7691, 23 },
    { 0, 0 },
    { 0x9dea7693, 90 },
    { 0xecda7694, 23 },
    { 0xa5fa7695, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ea769b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7a76a2, 93 },
    { 0xdaca76a2, 33 },
    { 0, 0 },
    { 0x718a76a5, 55 },
    { 0x90da76a6, 68 },
    { 0xaf6a76a6, 75 },
    { 0, 0 },
    { 0x71da76a9, 91 },
    { 0xd06a76aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7da76b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x268a76ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772a76d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97a76d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a76dd, 85 },
    { 0, 0 },
    { 0xb21a76df, 23 },
    { 0, 0 },
    { 0xbc9a76e1, 87 },
    { 0, 0 },
    { 0x587a76e3, 23 },
    { 0, 0 },
    { 0xa2aa76e5, 33 },
    { 0xa5da76e5, 33 },
    { 0x1a3a76e6, 33 },
    { 0xe52a76e8, 33 },
    { 0xa08a76e9, 61 },
    { 0xf6ba76ea, 95 },
    { 0x95ba76ea, 93 },
    { 0, 0 },
    { 0x9b7a76ed, 38 },
    { 0x46fa76ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faa76f5, 33 },
    { 0, 0 },
    { 0x43a76f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4a76fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807a7701, 100 },
    { 0xad7a7702, 95 },
    { 0, 0 },
    { 0x996a7704, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9a7708, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55a770b, 95 },
    { 0x88a770c, 33 },
    { 0, 0 },
    { 0xc64a770e, 61 },
    { 0x94aa770f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ca7714, 90 },
    { 0, 0 },
    { 0x29ba7716, 33 },
    { 0, 0 },
    { 0x4f6a7718, 95 },
    { 0x710a7719, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13a771d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0a7720, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3a7725, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafba7728, 90 },
    { 0x810a7729, 95 },
    { 0xa2ba772a, 97 },
    { 0x81a7728, 52 },
    { 0x691a772c, 78 },
    { 0x6b0a772d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ca7732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x871a7735, 93 },
    { 0, 0 },
    { 0x431a7737, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfa7741, 76 },
    { 0xdc1a7742, 33 },
    { 0x3b2a7741, 60 },
    { 0, 0 },
    { 0x872a7745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4a774d, 34 },
    { 0, 0 },
    { 0xd04a774f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43a7754, 95 },
    { 0, 0 },
    { 0xcf5a7756, 33 },
    { 0x5f0a7756, 61 },
    { 0xb96a7758, 34 },
    { 0x2f3a7758, 46 },
    { 0xc37a775a, 88 },
    { 0x800a775a, 43 },
    { 0x2c3a7759, 47 },
    { 0x45a775a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baa7760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eca7765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7a776b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aa7770, 33 },
    { 0x24aa7771, 96 },
    { 0, 0 },
    { 0x221a7773, 91 },
    { 0x488a7773, 98 },
    { 0, 0 },
    { 0x1a7a7776, 78 },
    { 0, 0 },
    { 0x248a7778, 76 },
    { 0, 0 },
    { 0x97ea777a, 33 },
    { 0xc45a777a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cca777e, 60 },
    { 0x777a777f, 23 },
    { 0x421a7780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x543a7783, 99 },
    { 0xb8ba7784, 33 },
    { 0xeba7785, 33 },
    { 0x3f7a7786, 76 },
    { 0x6e0a7787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9a778a, 43 },
    { 0xbe1a778b, 61 },
    { 0, 0 },
    { 0x240a778d, 98 },
    { 0xa49a778e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232a7792, 61 },
    { 0xacaa7792, 33 },
    { 0x4a1a7792, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949a7798, 40 },
    { 0, 0 },
    { 0x7eaa779a, 90 },
    { 0, 0 },
    { 0xc85a779c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x533a779f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea77a6, 76 },
    { 0x421a77a7, 76 },
    { 0x948a77a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x362a77ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba77ae, 58 },
    { 0xd64a77ae, 43 },
    { 0xd8aa77b0, 95 },
    { 0x542a77af, 33 },
    { 0x55ba77b2, 66 },
    { 0, 0 },
    { 0xe78a77b4, 33 },
    { 0, 0 },
    { 0x7aaa77b6, 95 },
    { 0x13ba77b7, 98 },
    { 0xca4a77b7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629a77bc, 84 },
    { 0, 0 },
    { 0xf86a77be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eba77c5, 84 },
    { 0xf87a77c6, 52 },
    { 0x456a77c7, 90 },
    { 0xbcfa77c8, 95 },
    { 0x9c6a77c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x893a77cc, 90 },
    { 0, 0 },
    { 0x22aa77ce, 34 },
    { 0x52ea77cf, 100 },
    { 0xbc0a77d0, 66 },
    { 0, 0 },
    { 0x649a77d2, 88 },
    { 0, 0 },
    { 0x55ba77d4, 95 },
    { 0xa40a77d4, 33 },
    { 0x88a77d6, 61 },
    { 0, 0 },
    { 0x3a7a77d8, 33 },
    { 0xfe3a77d9, 55 },
    { 0, 0 },
    { 0xad5a77db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44a77e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fa77e9, 23 },
    { 0x407a77ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca77ed, 61 },
    { 0x681a77ee, 95 },
    { 0xe69a77ef, 55 },
    { 0x81ea77f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6a77f4, 87 },
    { 0x376a77f5, 76 },
    { 0, 0 },
    { 0x462a77f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a77fe, 90 },
    { 0xad4a77ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4a7802, 80 },
    { 0xa07a7802, 84 },
    { 0xf2a7803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65a7808, 55 },
    { 0, 0 },
    { 0x7a1a780a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556a7810, 98 },
    { 0, 0 },
    { 0xe00a7812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ba7815, 88 },
    { 0xf8fa7816, 90 },
    { 0xcb6a7817, 78 },
    { 0x8a6a7817, 76 },
    { 0x184a7816, 95 },
    { 0xb5a781a, 90 },
    { 0x263a7816, 95 },
    { 0xefba7816, 81 },
    { 0xd3fa7817, 23 },
    { 0x57a781e, 78 },
    { 0x495a781e, 55 },
    { 0xefa7818, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ca7826, 33 },
    { 0, 0 },
    { 0xaf0a7828, 66 },
    { 0xdada7829, 33 },
    { 0xcf5a782a, 66 },
    { 0xcc4a7829, 34 },
    { 0xe7ba782c, 33 },
    { 0, 0 },
    { 0x4d5a782e, 40 },
    { 0x980a782e, 78 },
    { 0xcc7a782e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ca7834, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780a783e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fda7848, 34 },
    { 0, 0 },
    { 0x6d0a784a, 90 },
    { 0, 0 },
    { 0x466a784c, 33 },
    { 0x4eaa784c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63a7854, 61 },
    { 0xd59a7855, 76 },
    { 0, 0 },
    { 0xec2a7857, 43 },
    { 0x58ea7857, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6a785f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fda7863, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x262a7866, 98 },
    { 0x6c2a7867, 34 },
    { 0xc54a7867, 55 },
    { 0x8dca7866, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x958a786c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ba7873, 98 },
    { 0x3bda7874, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101a7880, 66 },
    { 0x795a7881, 68 },
    { 0x2da7882, 34 },
    { 0, 0 },
    { 0xa8da7884, 77 },
    { 0xbf3a7885, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x388a7888, 83 },
    { 0xc49a7889, 91 },
    { 0x4e1a7889, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fa788d, 95 },
    { 0, 0 },
    { 0xd97a788f, 75 },
    { 0, 0 },
    { 0x868a7891, 55 },
    { 0x15fa7892, 98 },
    { 0xc66a7893, 90 },
    { 0x196a7894, 100 },
    { 0xca9a7894, 23 },
    { 0x51da7891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a789a, 23 },
    { 0xacfa789a, 23 },
    { 0x998a789a, 100 },
    { 0, 0 },
    { 0xf89a789e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ca78a1, 33 },
    { 0xb5ba78a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0a78a7, 43 },
    { 0xf3ba78a7, 33 },
    { 0, 0 },
    { 0xedea78aa, 34 },
    { 0xabea78ab, 23 },
    { 0xea8a78ac, 61 },
    { 0x984a78ab, 100 },
    { 0, 0 },
    { 0x988a78af, 23 },
    { 0x6a0a78b0, 34 },
    { 0, 0 },
    { 0x8b5a78b2, 98 },
    { 0x2f3a78b3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8a78b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da78bc, 85 },
    { 0x66a78bc, 98 },
    { 0, 0 },
    { 0x7e1a78bf, 55 },
    { 0xeb6a78c0, 98 },
    { 0x403a78c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41a78c4, 46 },
    { 0x3efa78c4, 95 },
    { 0x72fa78c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fa78ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85a78cd, 61 },
    { 0, 0 },
    { 0xc30a78cf, 91 },
    { 0, 0 },
    { 0x3e9a78d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7a78d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ca78dc, 34 },
    { 0, 0 },
    { 0x4c9a78de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58a78e1, 80 },
    { 0, 0 },
    { 0xc5ca78e3, 55 },
    { 0x27fa78e4, 98 },
    { 0xde4a78e5, 87 },
    { 0x581a78e6, 78 },
    { 0x2bfa78e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4a78ea, 95 },
    { 0x62ba78eb, 98 },
    { 0xb67a78ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faa78f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a78f7, 52 },
    { 0x70ca78f8, 95 },
    { 0xdb9a78f9, 91 },
    { 0x4dea78f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbea78fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39da7900, 90 },
    { 0x1cda7901, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02a7907, 76 },
    { 0xaa1a7908, 98 },
    { 0xe9ea7909, 78 },
    { 0xfdea7909, 55 },
    { 0, 0 },
    { 0xd2ca790c, 23 },
    { 0x981a790d, 101 },
    { 0, 0 },
    { 0x8fea790f, 61 },
    { 0, 0 },
    { 0xf8ba7911, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389a7918, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa791c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ca7920, 61 },
    { 0xceda7920, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ca7928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fa792d, 99 },
    { 0, 0 },
    { 0x476a792f, 95 },
    { 0xe67a792f, 52 },
    { 0xc2ea7931, 95 },
    { 0x993a7931, 95 },
    { 0x134a7930, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a7936, 90 },
    { 0x3d5a7937, 23 },
    { 0, 0 },
    { 0x5cfa7939, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602a7942, 33 },
    { 0xd2ea7943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251a7947, 88 },
    { 0x4c2a7948, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823a794d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366a7952, 46 },
    { 0, 0 },
    { 0x155a7954, 33 },
    { 0x7bba7954, 90 },
    { 0xdc8a7955, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcba795b, 90 },
    { 0xb5ea795c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01a7961, 83 },
    { 0x7c6a7961, 33 },
    { 0x379a7962, 55 },
    { 0, 0 },
    { 0x391a7965, 38 },
    { 0, 0 },
    { 0x1f1a7967, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdea796a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ba796e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469a7978, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a797e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fa7981, 33 },
    { 0, 0 },
    { 0x168a7983, 95 },
    { 0, 0 },
    { 0x6f8a7985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31a798b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1a798e, 77 },
    { 0, 0 },
    { 0x988a7990, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666a799d, 61 },
    { 0xf65a799e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a79a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506a79aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caa79b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396a79b7, 46 },
    { 0xe01a79b8, 98 },
    { 0x7d4a79b7, 33 },
    { 0, 0 },
    { 0x99ea79bb, 91 },
    { 0x782a79bc, 76 },
    { 0x254a79bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3a79c1, 95 },
    { 0, 0 },
    { 0x705a79c3, 87 },
    { 0x3f1a79c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aea79c9, 100 },
    { 0, 0 },
    { 0x39aa79cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa79cf, 33 },
    { 0, 0 },
    { 0x4eba79d1, 85 },
    { 0x2b7a79d1, 33 },
    { 0x5baa79d3, 66 },
    { 0, 0 },
    { 0x224a79d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7a79ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19a79f0, 46 },
    { 0x3ca79f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa79f5, 34 },
    { 0x9f3a79f6, 68 },
    { 0x1d0a79f7, 33 },
    { 0xbbca79f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399a79fc, 93 },
    { 0x391a79fc, 33 },
    { 0xdb7a79fe, 23 },
    { 0x5b7a79fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59a7a05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faa7a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0a7a16, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687a7a1b, 34 },
    { 0x721a7a1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8a7a22, 17 },
    { 0xfada7a22, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afa7a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674a7a2c, 99 },
    { 0x640a7a2c, 87 },
    { 0, 0 },
    { 0x50fa7a2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15a7a3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5a7a41, 76 },
    { 0xd05a7a42, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81a7a46, 95 },
    { 0x34fa7a47, 33 },
    { 0, 0 },
    { 0x67a7a49, 90 },
    { 0x23fa7a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207a7a4f, 61 },
    { 0, 0 },
    { 0x18ea7a51, 33 },
    { 0xb46a7a52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51a7a59, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ba7a5c, 66 },
    { 0xfdca7a5c, 33 },
    { 0x449a7a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70a7a66, 77 },
    { 0, 0 },
    { 0xcdaa7a68, 46 },
    { 0x5cba7a69, 88 },
    { 0xaada7a68, 93 },
    { 0, 0 },
    { 0xb09a7a6c, 33 },
    { 0xf6a7a6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ba7a70, 100 },
    { 0x959a7a71, 90 },
    { 0x73aa7a72, 95 },
    { 0xd57a7a73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ea7a76, 55 },
    { 0x52ba7a77, 33 },
    { 0, 0 },
    { 0xf2ba7a79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550a7a8b, 84 },
    { 0x6d7a7a8c, 61 },
    { 0x2fda7a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fa7a90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340a7a95, 90 },
    { 0x907a7a96, 69 },
    { 0x9b7a7a97, 96 },
    { 0x17ca7a95, 46 },
    { 0xcd3a7a96, 95 },
    { 0x901a7a9a, 98 },
    { 0x95aa7a95, 76 },
    { 0xbbfa7a96, 95 },
    { 0, 0 },
    { 0xc6a7a9e, 56 },
    { 0x17fa7a9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3a7aa2, 98 },
    { 0, 0 },
    { 0xa14a7aa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66da7aa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cda7aac, 90 },
    { 0x1c3a7aac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa7ab9, 98 },
    { 0, 0 },
    { 0x368a7abb, 77 },
    { 0x99da7abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ca7ac5, 55 },
    { 0x7d8a7ac5, 77 },
    { 0xc6aa7ac6, 76 },
    { 0xed3a7ac8, 23 },
    { 0xae0a7ac5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1a7acc, 52 },
    { 0x502a7acc, 90 },
    { 0, 0 },
    { 0x957a7acf, 95 },
    { 0x600a7acf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfea7ad4, 61 },
    { 0, 0 },
    { 0x8e7a7ad6, 90 },
    { 0x1caa7ad7, 33 },
    { 0x9ffa7ad8, 98 },
    { 0x3d6a7ad8, 95 },
    { 0x8bba7ad9, 61 },
    { 0, 0 },
    { 0x286a7adc, 17 },
    { 0x58fa7add, 90 },
    { 0, 0 },
    { 0x727a7adf, 55 },
    { 0x74ca7ae0, 90 },
    { 0x1dda7ae1, 33 },
    { 0x528a7ae2, 38 },
    { 0x5c2a7ae3, 76 },
    { 0x61aa7ae4, 95 },
    { 0x19aa7ae2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49a7aea, 95 },
    { 0xcd6a7aeb, 98 },
    { 0x3c3a7aec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3a7af0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9da7afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9a7b01, 68 },
    { 0, 0 },
    { 0x319a7b03, 98 },
    { 0, 0 },
    { 0x321a7b05, 23 },
    { 0xa89a7b06, 95 },
    { 0x812a7b07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aa7b0b, 90 },
    { 0x912a7b0b, 99 },
    { 0x729a7b0d, 80 },
    { 0xc42a7b0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85a7b12, 76 },
    { 0x742a7b13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a7b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986a7b1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x663a7b1f, 91 },
    { 0, 0 },
    { 0x74aa7b21, 33 },
    { 0x652a7b22, 75 },
    { 0x8b7a7b23, 61 },
    { 0x33ca7b21, 90 },
    { 0x588a7b25, 93 },
    { 0x462a7b26, 93 },
    { 0x3c7a7b22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ba7b2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9a7b32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3a7b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86a7b40, 92 },
    { 0xf7a7b41, 90 },
    { 0x5bda7b42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867a7b46, 76 },
    { 0x198a7b46, 46 },
    { 0x4a9a7b48, 96 },
    { 0, 0 },
    { 0xdbea7b4a, 33 },
    { 0, 0 },
    { 0xfa4a7b4c, 33 },
    { 0x34aa7b4d, 88 },
    { 0x2dea7b4d, 100 },
    { 0, 0 },
    { 0x92a7b50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974a7b61, 98 },
    { 0x7eda7b62, 33 },
    { 0x847a7b63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ca7b66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9a7b69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa7b6e, 77 },
    { 0xcb0a7b6f, 34 },
    { 0, 0 },
    { 0x643a7b71, 38 },
    { 0x3a5a7b72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ea7b7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99da7b7e, 98 },
    { 0xd51a7b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95da7b82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca7b85, 61 },
    { 0x871a7b86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339a7b8a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7a7b8d, 33 },
    { 0, 0 },
    { 0xcf1a7b8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962a7b97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64a7b9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a7ba1, 76 },
    { 0x602a7ba2, 95 },
    { 0x299a7ba3, 55 },
    { 0, 0 },
    { 0x321a7ba5, 91 },
    { 0x45ea7ba6, 43 },
    { 0x313a7ba6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a7baa, 87 },
    { 0, 0 },
    { 0xda7bac, 33 },
    { 0x908a7bad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9a7bb1, 55 },
    { 0x1bfa7bb2, 75 },
    { 0x1a7a7bb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4a7bb6, 66 },
    { 0, 0 },
    { 0x666a7bb8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ea7bbc, 84 },
    { 0x44aa7bbc, 55 },
    { 0x273a7bbc, 65 },
    { 0, 0 },
    { 0x984a7bc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1aa7bc3, 17 },
    { 0x725a7bc4, 93 },
    { 0xa33a7bc4, 23 },
    { 0xc9ca7bc6, 90 },
    { 0, 0 },
    { 0x4a7bc8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceda7bcf, 61 },
    { 0x711a7bd0, 46 },
    { 0x454a7bcf, 87 },
    { 0xbbaa7bd2, 23 },
    { 0x31aa7bd0, 34 },
    { 0xbbfa7bd4, 17 },
    { 0x87ca7bd4, 77 },
    { 0xea7a7bd6, 33 },
    { 0, 0 },
    { 0x456a7bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70a7bdb, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0a7be4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3a7be7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4a7bea, 90 },
    { 0xdc4a7bea, 43 },
    { 0x3d8a7bec, 23 },
    { 0xf78a7bec, 90 },
    { 0x65a7bee, 74 },
    { 0xc2ea7bed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a7bf2, 77 },
    { 0x504a7bf2, 98 },
    { 0, 0 },
    { 0x337a7bf5, 38 },
    { 0x1fba7bf5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880a7bff, 33 },
    { 0, 0 },
    { 0x2c0a7c01, 98 },
    { 0, 0 },
    { 0xb87a7c03, 38 },
    { 0x721a7c03, 42 },
    { 0, 0 },
    { 0x96ca7c06, 33 },
    { 0, 0 },
    { 0x62fa7c08, 55 },
    { 0xfb2a7c08, 99 },
    { 0, 0 },
    { 0x482a7c0b, 87 },
    { 0, 0 },
    { 0xbb0a7c0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dea7c13, 61 },
    { 0x288a7c14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4a7c19, 55 },
    { 0x4cca7c19, 61 },
    { 0x4afa7c1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ea7c1e, 81 },
    { 0xee4a7c1e, 95 },
    { 0x739a7c20, 33 },
    { 0xe91a7c20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6a7c27, 76 },
    { 0x5b7a7c28, 91 },
    { 0xb53a7c29, 90 },
    { 0x951a7c28, 95 },
    { 0x765a7c2b, 55 },
    { 0x707a7c2c, 52 },
    { 0x298a7c2b, 68 },
    { 0x9ba7c2e, 40 },
    { 0x2b0a7c27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fca7c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aa7c35, 87 },
    { 0x4d5a7c36, 23 },
    { 0xc2aa7c35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ca7c3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871a7c44, 95 },
    { 0x268a7c45, 87 },
    { 0, 0 },
    { 0x512a7c47, 23 },
    { 0xc2ca7c47, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688a7c4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43a7c57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a7c5d, 76 },
    { 0xa40a7c5e, 33 },
    { 0, 0 },
    { 0xb53a7c60, 87 },
    { 0x351a7c60, 23 },
    { 0x53a7c62, 76 },
    { 0, 0 },
    { 0x5c3a7c64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7a7c67, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17a7c6b, 98 },
    { 0xeaca7c6b, 46 },
    { 0x655a7c6d, 33 },
    { 0x694a7c6d, 90 },
    { 0xb0a7c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baa7c72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcca7c76, 98 },
    { 0x423a7c77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4a7c7d, 84 },
    { 0, 0 },
    { 0xdd5a7c7f, 33 },
    { 0, 0 },
    { 0xf96a7c81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x216a7c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48a7c88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792a7c8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1a7c8f, 95 },
    { 0x51a7c90, 98 },
    { 0xfdca7c91, 90 },
    { 0x4b4a7c92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ca7c97, 23 },
    { 0, 0 },
    { 0x27a7c99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1a7c9c, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ba7c9f, 98 },
    { 0, 0 },
    { 0x39fa7ca1, 33 },
    { 0x9f4a7ca2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ea7ca6, 90 },
    { 0x545a7ca7, 95 },
    { 0, 0 },
    { 0x757a7ca9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfa7cac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579a7cb3, 91 },
    { 0x483a7cb3, 90 },
    { 0, 0 },
    { 0x4e3a7cb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86a7cb9, 99 },
    { 0x26da7cba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18a7ccc, 95 },
    { 0x534a7ccd, 18 },
    { 0x738a7ccc, 33 },
    { 0x38ba7cce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3a7cd7, 99 },
    { 0xabaa7cd7, 34 },
    { 0, 0 },
    { 0x24ca7cda, 87 },
    { 0, 0 },
    { 0x116a7cdc, 87 },
    { 0x1ada7cdd, 94 },
    { 0, 0 },
    { 0x9d6a7cdf, 33 },
    { 0, 0 },
    { 0x70aa7ce1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7a7ce5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fa7cec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24a7cef, 77 },
    { 0x742a7cef, 61 },
    { 0xca6a7cf1, 33 },
    { 0x482a7cf1, 91 },
    { 0xdb8a7cf3, 23 },
    { 0x529a7cef, 93 },
    { 0, 0 },
    { 0x5e0a7cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8a7cf9, 95 },
    { 0, 0 },
    { 0xca5a7cfb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87a7d05, 33 },
    { 0, 0 },
    { 0xbaea7d07, 95 },
    { 0xf8ca7d07, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ea7d0b, 98 },
    { 0x18aa7d0b, 17 },
    { 0xb9ea7d0d, 61 },
    { 0, 0 },
    { 0xa7ea7d0f, 95 },
    { 0xb2ba7d10, 33 },
    { 0, 0 },
    { 0x373a7d12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5a7d15, 23 },
    { 0xc5aa7d16, 33 },
    { 0x20da7d15, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ea7d1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2a7d1f, 61 },
    { 0, 0 },
    { 0x336a7d21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52a7d24, 33 },
    { 0xa8ca7d25, 17 },
    { 0, 0 },
    { 0xc1da7d27, 91 },
    { 0xab5a7d28, 95 },
    { 0x8e5a7d29, 61 },
    { 0, 0 },
    { 0xaf9a7d2b, 61 },
    { 0x5b0a7d2c, 87 },
    { 0xddaa7d2d, 93 },
    { 0, 0 },
    { 0x889a7d2f, 46 },
    { 0x838a7d2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88a7d37, 68 },
    { 0, 0 },
    { 0xf51a7d39, 95 },
    { 0x617a7d3a, 67 },
    { 0, 0 },
    { 0xb73a7d3c, 76 },
    { 0x495a7d3d, 43 },
    { 0x25fa7d3e, 96 },
    { 0xb4aa7d3f, 23 },
    { 0, 0 },
    { 0xfcaa7d41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcca7d47, 85 },
    { 0, 0 },
    { 0x2bda7d49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50a7d50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9a7d54, 87 },
    { 0x196a7d55, 55 },
    { 0x251a7d56, 99 },
    { 0xc41a7d57, 52 },
    { 0, 0 },
    { 0xd14a7d59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46a7d5c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x664a7d5f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5a7d63, 98 },
    { 0, 0 },
    { 0xcc9a7d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5a7d6b, 61 },
    { 0, 0 },
    { 0xc8aa7d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fa7d72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90da7d76, 43 },
    { 0x778a7d76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa7d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9a7d81, 69 },
    { 0, 0 },
    { 0xf75a7d83, 88 },
    { 0, 0 },
    { 0xef1a7d85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330a7d8b, 33 },
    { 0xaaaa7d8c, 98 },
    { 0x133a7d8d, 98 },
    { 0x630a7d8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741a7d92, 33 },
    { 0x859a7d92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bba7d9b, 23 },
    { 0xc7aa7d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x819a7d9f, 43 },
    { 0x11ea7da0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba7da3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54da7da7, 98 },
    { 0xa86a7da8, 93 },
    { 0x11ba7da9, 33 },
    { 0, 0 },
    { 0xca6a7dab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87a7db2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x813a7db5, 96 },
    { 0, 0 },
    { 0xfe1a7db7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3a7dba, 23 },
    { 0x615a7dbb, 95 },
    { 0xb1ba7dbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08a7dbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ca7dc2, 90 },
    { 0, 0 },
    { 0xdfba7dc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2a7dc7, 55 },
    { 0, 0 },
    { 0x14ca7dc9, 43 },
    { 0x128a7dc9, 90 },
    { 0x7b1a7dcb, 46 },
    { 0xf4aa7dcc, 90 },
    { 0x130a7dc9, 76 },
    { 0x511a7dce, 95 },
    { 0xa96a7dcf, 95 },
    { 0x2aa7dce, 91 },
    { 0xf4ca7dd0, 84 },
    { 0x192a7dc9, 61 },
    { 0, 0 },
    { 0x49ea7dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba7dd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aa7ddd, 33 },
    { 0, 0 },
    { 0xa1da7ddf, 23 },
    { 0, 0 },
    { 0x3b9a7de1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a7de4, 55 },
    { 0x939a7de4, 99 },
    { 0xce2a7de6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259a7deb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ea7df7, 33 },
    { 0x91ca7df8, 33 },
    { 0xc89a7df9, 99 },
    { 0, 0 },
    { 0xa00a7dfb, 95 },
    { 0, 0 },
    { 0x43ba7dfd, 90 },
    { 0, 0 },
    { 0xd93a7dff, 90 },
    { 0xaa8a7e00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6a7e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357a7e08, 33 },
    { 0xeb9a7e08, 101 },
    { 0x5c0a7e0a, 95 },
    { 0, 0 },
    { 0xcdea7e0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9a7e0f, 100 },
    { 0, 0 },
    { 0x178a7e11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdda7e1b, 97 },
    { 0xb8ca7e1b, 76 },
    { 0xdcfa7e1d, 76 },
    { 0x270a7e1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cba7e2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa7e34, 90 },
    { 0, 0 },
    { 0xc42a7e36, 97 },
    { 0xeafa7e37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a7e3c, 61 },
    { 0xd97a7e3c, 46 },
    { 0, 0 },
    { 0x6dea7e3f, 43 },
    { 0, 0 },
    { 0x8a1a7e41, 76 },
    { 0xc94a7e42, 36 },
    { 0x222a7e43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ca7e49, 61 },
    { 0xf26a7e4a, 76 },
    { 0, 0 },
    { 0x114a7e4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fa7e50, 33 },
    { 0, 0 },
    { 0x399a7e52, 55 },
    { 0, 0 },
    { 0xf94a7e54, 33 },
    { 0xf4aa7e55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33a7e5b, 84 },
    { 0, 0 },
    { 0x155a7e5d, 61 },
    { 0x9eda7e5e, 33 },
    { 0, 0 },
    { 0xabea7e60, 90 },
    { 0, 0 },
    { 0xe8da7e62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dda7e6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30a7e70, 46 },
    { 0xf2ba7e70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90a7e79, 33 },
    { 0x169a7e79, 23 },
    { 0x50da7e7b, 100 },
    { 0x4aaa7e79, 23 },
    { 0xedaa7e7d, 61 },
    { 0xa1ea7e7e, 76 },
    { 0x59da7e7e, 61 },
    { 0x8fba7e7e, 95 },
    { 0xe18a7e80, 61 },
    { 0xd6fa7e7c, 98 },
    { 0xe89a7e83, 100 },
    { 0xa85a7e84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795a7e8a, 95 },
    { 0x961a7e8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80a7e90, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4a7e94, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7a7e98, 23 },
    { 0x6c3a7e99, 90 },
    { 0xc08a7e9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8a7e9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592a7ea4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656a7ea9, 34 },
    { 0, 0 },
    { 0xdbda7eab, 46 },
    { 0x8e2a7eac, 33 },
    { 0, 0 },
    { 0xd69a7eae, 95 },
    { 0, 0 },
    { 0xe45a7eb0, 96 },
    { 0x1a1a7eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0aa7eb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a7eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea7ec0, 90 },
    { 0, 0 },
    { 0x693a7ec2, 23 },
    { 0x5cba7ec3, 33 },
    { 0, 0 },
    { 0x52aa7ec5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968a7eca, 23 },
    { 0, 0 },
    { 0x109a7ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x549a7ecf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0a7ed3, 40 },
    { 0xa6ca7ed4, 23 },
    { 0, 0 },
    { 0x95ba7ed6, 23 },
    { 0x8d3a7ed6, 33 },
    { 0x437a7ed8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6a7edb, 66 },
    { 0xdbea7edc, 98 },
    { 0, 0 },
    { 0xdc7a7ede, 85 },
    { 0, 0 },
    { 0xae6a7ee0, 33 },
    { 0x94ba7ee1, 61 },
    { 0, 0 },
    { 0x9c0a7ee3, 90 },
    { 0, 0 },
    { 0x3fca7ee5, 78 },
    { 0xe63a7ee6, 90 },
    { 0x667a7ee5, 23 },
    { 0x66fa7ee8, 23 },
    { 0x33ea7ee8, 33 },
    { 0, 0 },
    { 0x32ca7eeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8a7ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfca7efc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a7f00, 98 },
    { 0x567a7f00, 90 },
    { 0, 0 },
    { 0x216a7f03, 88 },
    { 0x516a7f03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31a7f0b, 91 },
    { 0x4d2a7f0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ba7f10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15a7f16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ea7f19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43aa7f1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ea7f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x992a7f2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38a7f2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29a7f35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771a7f39, 93 },
    { 0x8b1a7f3a, 91 },
    { 0xf8ca7f3b, 66 },
    { 0x98ca7f3a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1a7f3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da7f45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x761a7f48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aba7f4b, 52 },
    { 0xe33a7f4c, 33 },
    { 0x595a7f4d, 90 },
    { 0x75fa7f4e, 55 },
    { 0x6c3a7f4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a7f55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84a7f5c, 95 },
    { 0x342a7f5c, 75 },
    { 0, 0 },
    { 0xddba7f5f, 77 },
    { 0xee1a7f60, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952a7f64, 95 },
    { 0x4c9a7f65, 9 },
    { 0xf81a7f66, 61 },
    { 0x897a7f64, 66 },
    { 0xce6a7f64, 76 },
    { 0x9f5a7f66, 61 },
    { 0x215a7f6a, 95 },
    { 0x77aa7f6b, 95 },
    { 0x4ffa7f6b, 55 },
    { 0xbbda7f64, 43 },
    { 0xc5aa7f6e, 33 },
    { 0x6fca7f6f, 90 },
    { 0x1faa7f70, 90 },
    { 0x601a7f71, 34 },
    { 0xb17a7f6c, 33 },
    { 0x67fa7f73, 95 },
    { 0x616a7f74, 33 },
    { 0x939a7f73, 90 },
    { 0x777a7f67, 61 },
    { 0xdf8a7f68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aa7f7d, 100 },
    { 0x197a7f7e, 23 },
    { 0x476a7f7d, 67 },
    { 0xbfba7f7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x707a7f83, 76 },
    { 0x3eba7f84, 61 },
    { 0, 0 },
    { 0xd2ca7f86, 55 },
    { 0, 0 },
    { 0xbf7a7f88, 55 },
    { 0, 0 },
    { 0xbc2a7f8a, 91 },
    { 0, 0 },
    { 0x126a7f8c, 76 },
    { 0xc49a7f8d, 84 },
    { 0x343a7f8e, 90 },
    { 0xacca7f8f, 66 },
    { 0x9f8a7f8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8a7f9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356a7fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fca7fa3, 52 },
    { 0x4d6a7fa4, 23 },
    { 0, 0 },
    { 0x6f4a7fa6, 33 },
    { 0x66ca7fa7, 55 },
    { 0xc1da7fa7, 76 },
    { 0x855a7fa7, 80 },
    { 0x5caa7fa9, 95 },
    { 0, 0 },
    { 0xb7ca7fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eea7faf, 61 },
    { 0x351a7faf, 55 },
    { 0, 0 },
    { 0x946a7fb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1a7fb8, 90 },
    { 0x67da7fb9, 55 },
    { 0x544a7fb9, 61 },
    { 0x624a7fb8, 61 },
    { 0x245a7fb9, 55 },
    { 0, 0 },
    { 0xc51a7fbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ea7fc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ba7fce, 33 },
    { 0xfc7a7fce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5a7fd5, 61 },
    { 0x166a7fd6, 55 },
    { 0, 0 },
    { 0xcc8a7fd8, 17 },
    { 0, 0 },
    { 0x7c1a7fda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dda7fde, 69 },
    { 0xc2ba7fdf, 90 },
    { 0, 0 },
    { 0x43fa7fe1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1a7fe6, 33 },
    { 0, 0 },
    { 0x2c1a7fe8, 90 },
    { 0x1b2a7fe9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3a7ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a7ff9, 23 },
    { 0, 0 },
    { 0x81aa7ffb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fba8003, 61 },
    { 0x5d9a8004, 34 },
    { 0x1cea8005, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928a800b, 53 },
    { 0, 0 },
    { 0x68fa800d, 93 },
    { 0xa6ca800e, 17 },
    { 0xfea800f, 55 },
    { 0x9ca800f, 52 },
    { 0, 0 },
    { 0x354a8012, 23 },
    { 0x60ca8013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daa8016, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ca8020, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aba8023, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea8026, 96 },
    { 0, 0 },
    { 0x42ea8028, 90 },
    { 0x4b1a8029, 90 },
    { 0xe2fa802a, 55 },
    { 0x5dca8028, 90 },
    { 0x739a802c, 98 },
    { 0x415a802c, 76 },
    { 0x575a802a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123a8032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369a8037, 33 },
    { 0x231a8037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x972a803b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaba8045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba8048, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fa804b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fa804e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aa8051, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffa8056, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44a805a, 98 },
    { 0x42ca805b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00a8062, 33 },
    { 0x240a8062, 87 },
    { 0, 0 },
    { 0xb61a8065, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aa806b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaa806e, 90 },
    { 0xac1a806f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1a8074, 76 },
    { 0x5efa8074, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2a8079, 88 },
    { 0xdbfa807a, 90 },
    { 0xe8da807b, 90 },
    { 0, 0 },
    { 0x131a807d, 90 },
    { 0xf6ca807e, 55 },
    { 0xafca807d, 17 },
    { 0xe2ea8080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x255a8083, 23 },
    { 0, 0 },
    { 0x65fa8085, 61 },
    { 0x798a8086, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8a8089, 78 },
    { 0x7e5a808a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2a8090, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cda8093, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x613a8096, 34 },
    { 0x316a8096, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19da809c, 61 },
    { 0xe5fa809d, 98 },
    { 0x6baa809e, 95 },
    { 0x8f8a809d, 90 },
    { 0xcc2a80a0, 87 },
    { 0x4a6a80a1, 33 },
    { 0x839a80a1, 48 },
    { 0x3f9a80a3, 33 },
    { 0x635a80a4, 33 },
    { 0xb9fa80a5, 61 },
    { 0x19fa80a6, 34 },
    { 0x73da80a7, 55 },
    { 0x7fca809f, 67 },
    { 0x28ca80a9, 90 },
    { 0x1a6a80aa, 33 },
    { 0x52ba80ab, 95 },
    { 0x4f9a80ab, 23 },
    { 0x3cca80ad, 61 },
    { 0xac9a80a9, 90 },
    { 0, 0 },
    { 0xc49a80b0, 66 },
    { 0xfa2a80b1, 85 },
    { 0x3b6a80b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5a80b6, 98 },
    { 0xa2da80b7, 61 },
    { 0xd76a80b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628a80c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ca80c4, 23 },
    { 0x7e2a80c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfa80c8, 43 },
    { 0xef1a80c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a80ce, 85 },
    { 0, 0 },
    { 0x92fa80d0, 100 },
    { 0xecaa80d0, 61 },
    { 0x41da80d2, 33 },
    { 0xcf0a80d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33a80d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230a80db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30a80df, 95 },
    { 0xdd2a80e0, 98 },
    { 0x118a80e0, 98 },
    { 0xbd3a80e2, 85 },
    { 0x2a5a80e3, 90 },
    { 0, 0 },
    { 0xb53a80e5, 64 },
    { 0, 0 },
    { 0x3c6a80e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58a80ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600a80f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9a80f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a80f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627a80fe, 93 },
    { 0x81ba80ff, 90 },
    { 0x35ba8100, 90 },
    { 0x15ea8101, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ea8104, 67 },
    { 0x7c0a8104, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5a8108, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975a810e, 61 },
    { 0x189a810f, 93 },
    { 0, 0 },
    { 0x79a8111, 33 },
    { 0x12da8111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca811b, 66 },
    { 0x196a811b, 52 },
    { 0xb7da811d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ba8121, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa8126, 61 },
    { 0xccaa8127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999a812c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ca812f, 90 },
    { 0xef5a812f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6a8139, 90 },
    { 0x66aa8139, 77 },
    { 0x308a813a, 61 },
    { 0x998a813b, 98 },
    { 0, 0 },
    { 0xa72a813e, 93 },
    { 0x2a813e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589a8145, 61 },
    { 0xd21a8145, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cba814a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79a8158, 67 },
    { 0, 0 },
    { 0x272a815a, 90 },
    { 0xfa6a815b, 98 },
    { 0, 0 },
    { 0x603a815d, 23 },
    { 0xd06a815e, 90 },
    { 0, 0 },
    { 0x749a8160, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ba816d, 98 },
    { 0, 0 },
    { 0x9fca816f, 34 },
    { 0, 0 },
    { 0xba1a8171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4a8176, 55 },
    { 0x797a8177, 23 },
    { 0xc08a8178, 43 },
    { 0, 0 },
    { 0x1c2a817a, 33 },
    { 0xb05a817b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bda8182, 33 },
    { 0x16aa8182, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca8186, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427a818a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459a8190, 61 },
    { 0, 0 },
    { 0x970a8192, 55 },
    { 0, 0 },
    { 0xa1fa8194, 61 },
    { 0xa0aa8195, 36 },
    { 0xd80a8196, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79a819d, 78 },
    { 0, 0 },
    { 0x4bba819f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7a81a2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a81a6, 33 },
    { 0xc58a81a7, 98 },
    { 0x5dda81a8, 95 },
    { 0x20ba81a9, 95 },
    { 0xb63a81aa, 61 },
    { 0xf03a81a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbaa81ae, 68 },
    { 0xfbba81af, 95 },
    { 0xa67a81b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4a81b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ba81bd, 33 },
    { 0, 0 },
    { 0xa07a81bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x665a81c2, 93 },
    { 0x628a81c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ca81c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41a81cc, 85 },
    { 0x45ba81cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a81d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794a81d6, 61 },
    { 0, 0 },
    { 0x185a81d8, 87 },
    { 0, 0 },
    { 0xd74a81da, 55 },
    { 0xf74a81da, 90 },
    { 0x2dca81dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x992a81df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fa81e4, 98 },
    { 0x297a81e4, 23 },
    { 0x327a81e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6a81eb, 61 },
    { 0x48ea81eb, 90 },
    { 0, 0 },
    { 0xc89a81ee, 23 },
    { 0x7d4a81ef, 78 },
    { 0x5fca81ef, 99 },
    { 0xd9da81f1, 61 },
    { 0x661a81f2, 38 },
    { 0xecba81f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30a81f6, 61 },
    { 0x2ffa81f7, 90 },
    { 0xbe2a81f7, 76 },
    { 0, 0 },
    { 0x6b4a81fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23da8200, 23 },
    { 0x7c6a8201, 40 },
    { 0, 0 },
    { 0x9fca8203, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1a820e, 98 },
    { 0, 0 },
    { 0x422a8210, 98 },
    { 0x301a8210, 75 },
    { 0x2ba8212, 68 },
    { 0xd3a8212, 59 },
    { 0x615a8214, 33 },
    { 0x566a8215, 52 },
    { 0xfe7a8215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7a821a, 90 },
    { 0x879a821b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471a8220, 23 },
    { 0, 0 },
    { 0xdf4a8222, 90 },
    { 0x8bfa8222, 99 },
    { 0xfcaa8222, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466a822c, 17 },
    { 0, 0 },
    { 0x30a822e, 34 },
    { 0x1e9a822e, 75 },
    { 0xfe6a8230, 61 },
    { 0xfb3a8230, 61 },
    { 0x3a0a8232, 87 },
    { 0, 0 },
    { 0xea2a8234, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaa8237, 98 },
    { 0xf27a8238, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ba8240, 65 },
    { 0x82ca8241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x218a8244, 66 },
    { 0x3d9a8244, 36 },
    { 0, 0 },
    { 0x4b6a8247, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4a824d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a8252, 96 },
    { 0x397a8253, 33 },
    { 0x1a0a8254, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4da825e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a8262, 61 },
    { 0xf1ba8262, 76 },
    { 0, 0 },
    { 0xd94a8265, 46 },
    { 0xf4a8266, 33 },
    { 0xf55a8266, 61 },
    { 0x584a8268, 87 },
    { 0x616a8266, 76 },
    { 0x404a826a, 52 },
    { 0xf22a826a, 93 },
    { 0xc3a826b, 88 },
    { 0, 0 },
    { 0x317a826e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfa8272, 75 },
    { 0xec3a8273, 85 },
    { 0xbc3a8272, 66 },
    { 0x4f5a8275, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fca8279, 90 },
    { 0, 0 },
    { 0xfb0a827b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99aa827f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a8284, 33 },
    { 0x68fa8285, 100 },
    { 0x21aa8284, 23 },
    { 0x98a8286, 95 },
    { 0x114a8287, 55 },
    { 0x8e6a8289, 80 },
    { 0xb54a828a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaa8290, 43 },
    { 0x424a8291, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52a8298, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55a829d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cba82a5, 90 },
    { 0, 0 },
    { 0x426a82a7, 34 },
    { 0, 0 },
    { 0x7a4a82a9, 33 },
    { 0x54ca82a9, 95 },
    { 0x1aa82aa, 61 },
    { 0x728a82a9, 55 },
    { 0x572a82ad, 66 },
    { 0, 0 },
    { 0xfada82af, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda82b2, 90 },
    { 0x789a82b3, 100 },
    { 0x8fea82b3, 90 },
    { 0x698a82b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1a82b8, 95 },
    { 0, 0 },
    { 0xea9a82ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55a82c0, 76 },
    { 0x1dda82c1, 43 },
    { 0x76ea82c2, 61 },
    { 0x6cca82c3, 90 },
    { 0, 0 },
    { 0x7da82c5, 95 },
    { 0xba9a82c6, 95 },
    { 0xe04a82c6, 33 },
    { 0x6b4a82c8, 38 },
    { 0x2d2a82c8, 98 },
    { 0xeb2a82ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ca82cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fa82d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2a82db, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x197a82de, 34 },
    { 0xdc8a82df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ba82e8, 78 },
    { 0x619a82e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x945a82ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752a82f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x153a82f5, 33 },
    { 0xffa82f6, 95 },
    { 0, 0 },
    { 0x10ea82f8, 90 },
    { 0xac1a82f9, 90 },
    { 0, 0 },
    { 0xe93a82fb, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcea8301, 23 },
    { 0, 0 },
    { 0xdf8a8303, 61 },
    { 0x779a8303, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24a830b, 87 },
    { 0, 0 },
    { 0xe89a830d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ea8316, 98 },
    { 0xff2a8317, 43 },
    { 0x402a8318, 93 },
    { 0, 0 },
    { 0x4f9a831a, 61 },
    { 0x9a5a831b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659a8324, 98 },
    { 0, 0 },
    { 0xb86a8326, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1a832c, 90 },
    { 0x68ea832d, 61 },
    { 0x151a832e, 61 },
    { 0xfb0a832c, 34 },
    { 0xcfaa832f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cea8338, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97a833c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x670a833f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0a8344, 33 },
    { 0, 0 },
    { 0x7c0a8346, 95 },
    { 0x71aa8347, 75 },
    { 0, 0 },
    { 0xb41a8349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3a834c, 33 },
    { 0x9daa834d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba8355, 91 },
    { 0, 0 },
    { 0xbea8357, 46 },
    { 0, 0 },
    { 0xad2a8359, 78 },
    { 0, 0 },
    { 0x8b1a835b, 23 },
    { 0x98aa835b, 17 },
    { 0xa9ca835d, 98 },
    { 0xa04a835e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efa8363, 87 },
    { 0xc6a8363, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64a836a, 90 },
    { 0, 0 },
    { 0x13aa836c, 23 },
    { 0, 0 },
    { 0xa35a836e, 61 },
    { 0xd97a836f, 33 },
    { 0xcdaa8370, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cea8376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x157a8379, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57a837d, 95 },
    { 0x705a837e, 52 },
    { 0x6dfa837d, 99 },
    { 0xd08a8380, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a8383, 76 },
    { 0x192a8384, 51 },
    { 0x92ca8385, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ca8388, 33 },
    { 0, 0 },
    { 0x466a838a, 95 },
    { 0, 0 },
    { 0xf57a838c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aa8391, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa8394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x450a8397, 61 },
    { 0x804a8398, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8da839b, 98 },
    { 0x76aa839c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706a83a3, 90 },
    { 0x6b2a83a3, 95 },
    { 0x177a83a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8a83b1, 23 },
    { 0x886a83b2, 90 },
    { 0xb9fa83b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6a83bd, 98 },
    { 0xa31a83be, 35 },
    { 0xec5a83bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4da83c2, 23 },
    { 0x570a83c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8a83c6, 33 },
    { 0xc77a83c6, 67 },
    { 0, 0 },
    { 0x28aa83c9, 61 },
    { 0x59ba83ca, 90 },
    { 0x5a8a83ca, 98 },
    { 0, 0 },
    { 0x4bca83cd, 43 },
    { 0x526a83ce, 33 },
    { 0xc22a83cd, 83 },
    { 0x5eea83d0, 23 },
    { 0, 0 },
    { 0xb8a83d2, 95 },
    { 0x517a83d2, 95 },
    { 0x304a83d3, 90 },
    { 0xafda83d5, 95 },
    { 0xfa3a83d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a83de, 90 },
    { 0x147a83df, 80 },
    { 0xc78a83e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1a83e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ea83e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfa83ed, 61 },
    { 0x6a3a83ed, 76 },
    { 0x911a83ef, 23 },
    { 0xb7ea83ee, 33 },
    { 0x1bfa83f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2a83f9, 90 },
    { 0x7bea83fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x702a83fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45a8404, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a840a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca8412, 88 },
    { 0, 0 },
    { 0x9da8414, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8a841a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaca841d, 95 },
    { 0x490a841e, 66 },
    { 0x693a841e, 46 },
    { 0xf12a841e, 99 },
    { 0xee8a841f, 95 },
    { 0xbeca8420, 17 },
    { 0x27ea8423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532a8426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111a842c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ba8430, 95 },
    { 0x731a8431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82a8434, 33 },
    { 0, 0 },
    { 0x590a8436, 76 },
    { 0xcf1a8437, 33 },
    { 0, 0 },
    { 0x87a8439, 43 },
    { 0, 0 },
    { 0xe37a843b, 61 },
    { 0x740a843b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99a8440, 98 },
    { 0xa4a8440, 100 },
    { 0xae9a8440, 23 },
    { 0x1aca8442, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95a8448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714a844c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58a8450, 90 },
    { 0xc43a8451, 95 },
    { 0x370a8452, 95 },
    { 0x606a8453, 67 },
    { 0x962a8453, 95 },
    { 0x18fa8452, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cba845d, 98 },
    { 0x4bea845e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7a8461, 23 },
    { 0, 0 },
    { 0x58da8463, 77 },
    { 0xafba8463, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8468, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2a846b, 52 },
    { 0xbeea846c, 69 },
    { 0xf36a846d, 61 },
    { 0x386a846d, 61 },
    { 0xf51a846f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192a8473, 61 },
    { 0x621a8473, 55 },
    { 0x42a8474, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaba8478, 77 },
    { 0x640a8478, 98 },
    { 0x396a847a, 95 },
    { 0x1d8a847a, 90 },
    { 0xd54a847c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93a8481, 90 },
    { 0, 0 },
    { 0xb8ca8483, 99 },
    { 0xda5a8483, 93 },
    { 0x967a8484, 93 },
    { 0x869a8486, 2 },
    { 0, 0 },
    { 0xfa7a8488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12da848d, 91 },
    { 0x4fca848e, 23 },
    { 0xdeaa848e, 46 },
    { 0x11ea8490, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cca8493, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1a8497, 98 },
    { 0x41ea8497, 33 },
    { 0xc50a8498, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93a849c, 87 },
    { 0x82fa849d, 33 },
    { 0xcc1a849c, 84 },
    { 0x78fa849e, 38 },
    { 0xae3a84a0, 66 },
    { 0x95a84a0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aa84a5, 61 },
    { 0x8b4a84a6, 83 },
    { 0xe72a84a6, 23 },
    { 0x552a84a8, 23 },
    { 0xbd4a84a6, 61 },
    { 0x3f8a84a6, 95 },
    { 0xd22a84a5, 93 },
    { 0x995a84ac, 90 },
    { 0xda0a84ad, 88 },
    { 0x37ba84ae, 50 },
    { 0x171a84ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3a84b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460a84b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51a84bd, 33 },
    { 0, 0 },
    { 0xcb6a84bf, 34 },
    { 0, 0 },
    { 0xa8ea84c1, 55 },
    { 0x827a84c2, 17 },
    { 0, 0 },
    { 0x22ca84c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49a84c9, 61 },
    { 0, 0 },
    { 0x274a84cb, 55 },
    { 0x9e5a84cc, 33 },
    { 0xcc0a84cd, 23 },
    { 0, 0 },
    { 0xc94a84cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4a84d3, 34 },
    { 0xfbfa84d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4a84da, 88 },
    { 0x5bca84db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaea84e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64a84e5, 98 },
    { 0, 0 },
    { 0x7b4a84e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ea84ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x323a84ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca84f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736a84f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737a84f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8a84fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ea8502, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aa8505, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ca850f, 95 },
    { 0xa6a850f, 90 },
    { 0, 0 },
    { 0x7c3a8512, 98 },
    { 0x6cca8512, 43 },
    { 0x731a8514, 40 },
    { 0x818a8514, 34 },
    { 0x790a8515, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06a8520, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1a8526, 55 },
    { 0x16ea8527, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0a8530, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fca8535, 67 },
    { 0, 0 },
    { 0xb5ba8537, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4a853c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x817a853f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00a8542, 61 },
    { 0, 0 },
    { 0xc5a8544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaa854d, 33 },
    { 0x205a854e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67a8553, 33 },
    { 0xd3ba8554, 98 },
    { 0xf29a8553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136a855a, 90 },
    { 0x51a855b, 84 },
    { 0x57a855c, 90 },
    { 0x463a855d, 43 },
    { 0x2dda855d, 90 },
    { 0x3bba855f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ba8569, 55 },
    { 0x4a2a856a, 98 },
    { 0, 0 },
    { 0x9a856c, 61 },
    { 0xf2ca856d, 17 },
    { 0x3a3a856e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a8571, 95 },
    { 0, 0 },
    { 0x88ba8573, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1a857a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72a857e, 68 },
    { 0x3fda857e, 98 },
    { 0x756a8580, 95 },
    { 0, 0 },
    { 0x1d0a8582, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0a8585, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ba8589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfa858c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1a8590, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12a8596, 90 },
    { 0xcaea8597, 61 },
    { 0x9e2a8598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dea859b, 78 },
    { 0, 0 },
    { 0x4eda859d, 77 },
    { 0xba3a859e, 33 },
    { 0xd38a859f, 90 },
    { 0x5d1a859d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccea85a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ea85b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fa85b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1da85be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baa85c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ea85c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fa85cc, 90 },
    { 0x533a85cc, 33 },
    { 0x33ea85ce, 23 },
    { 0x604a85cf, 33 },
    { 0xe76a85cc, 33 },
    { 0x16ba85d1, 85 },
    { 0xe36a85d2, 61 },
    { 0x3bea85d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482a85dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6a85e0, 61 },
    { 0x5f5a85e1, 34 },
    { 0, 0 },
    { 0x915a85e3, 33 },
    { 0, 0 },
    { 0x5b5a85e5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19a85ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ba85f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x869a85f7, 33 },
    { 0x661a85f8, 69 },
    { 0x560a85f9, 84 },
    { 0, 0 },
    { 0xf43a85fb, 67 },
    { 0xf13a85fb, 95 },
    { 0xa15a85fc, 93 },
    { 0, 0 },
    { 0x1a9a85ff, 55 },
    { 0x410a8600, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbba8603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ada8606, 33 },
    { 0x503a8606, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cba860a, 90 },
    { 0x4e0a860b, 34 },
    { 0xf6ba860c, 66 },
    { 0xe80a860d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcca8611, 95 },
    { 0xc3a8611, 55 },
    { 0, 0 },
    { 0x135a8614, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fa8617, 84 },
    { 0x32fa8618, 90 },
    { 0xde8a8618, 55 },
    { 0x23fa861a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaea861f, 61 },
    { 0xe5aa861f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8a8626, 52 },
    { 0xfea8627, 90 },
    { 0x7eca8628, 66 },
    { 0x13aa8629, 90 },
    { 0x470a8629, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1a862d, 76 },
    { 0xb3ba862e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4a8633, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463a863a, 88 },
    { 0, 0 },
    { 0x7aa863c, 43 },
    { 0x1a4a863c, 90 },
    { 0x34ea863c, 95 },
    { 0x90a863f, 55 },
    { 0x817a863e, 84 },
    { 0x329a8641, 84 },
    { 0x258a8642, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3a8646, 61 },
    { 0x80aa8646, 46 },
    { 0xc8a8647, 90 },
    { 0xb39a8649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42a864d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a8658, 61 },
    { 0, 0 },
    { 0x60a865a, 84 },
    { 0xc74a865a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7a8661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccea8666, 43 },
    { 0x5baa8667, 98 },
    { 0, 0 },
    { 0x34fa8669, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cca866c, 52 },
    { 0x90fa866c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfba8671, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6a867a, 61 },
    { 0, 0 },
    { 0xc9da867c, 43 },
    { 0x3cba867d, 33 },
    { 0xabba867e, 41 },
    { 0x131a867c, 90 },
    { 0xbaca867f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa8685, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ca8689, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4a868c, 23 },
    { 0, 0 },
    { 0x763a868e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ba8693, 23 },
    { 0x5d4a8694, 84 },
    { 0, 0 },
    { 0x33ca8696, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceca869b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39a86a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5a86a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31a86af, 33 },
    { 0xb04a86b0, 90 },
    { 0x36a86b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500a86b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0a86bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9a86bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98da86c2, 33 },
    { 0, 0 },
    { 0x152a86c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a86c7, 90 },
    { 0, 0 },
    { 0x471a86c9, 77 },
    { 0xe6ca86ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75a86cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9a86d7, 88 },
    { 0x5c5a86d7, 81 },
    { 0xfd5a86d9, 87 },
    { 0xa30a86d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4a86e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66a86e6, 12 },
    { 0, 0 },
    { 0xe8da86e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4a86f4, 76 },
    { 0x567a86f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fa86f8, 33 },
    { 0xeb4a86f9, 94 },
    { 0x7eda86f9, 23 },
    { 0xd5ea86fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da8702, 61 },
    { 0, 0 },
    { 0xd9ca8704, 75 },
    { 0xcd3a8704, 90 },
    { 0, 0 },
    { 0x8baa8707, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0da870a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264a870e, 64 },
    { 0x73ea870f, 33 },
    { 0x313a870f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618a8717, 61 },
    { 0x993a8717, 34 },
    { 0x9e2a8719, 46 },
    { 0x74aa871a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100a871e, 95 },
    { 0, 0 },
    { 0x4c3a8720, 90 },
    { 0x330a8721, 43 },
    { 0x926a8722, 95 },
    { 0x848a8723, 55 },
    { 0xbaa8724, 98 },
    { 0x424a8725, 98 },
    { 0x2fa8726, 95 },
    { 0x510a8726, 95 },
    { 0, 0 },
    { 0x879a8729, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4a872d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26a8732, 90 },
    { 0xe52a8732, 100 },
    { 0x46aa8734, 87 },
    { 0xf57a8734, 23 },
    { 0, 0 },
    { 0xb24a8737, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a873a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3a873d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5a8741, 33 },
    { 0xdb9a8741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faa8747, 23 },
    { 0, 0 },
    { 0x202a8749, 43 },
    { 0x93ea874a, 55 },
    { 0x539a8749, 95 },
    { 0x9aea8749, 98 },
    { 0x68ea874d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1a8752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ea8757, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210a875b, 33 },
    { 0x939a875b, 88 },
    { 0x8a0a875b, 34 },
    { 0xa23a875e, 69 },
    { 0x84a875c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0a8763, 55 },
    { 0xad1a8764, 90 },
    { 0xa74a8765, 61 },
    { 0xd2ba8766, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812a876d, 90 },
    { 0, 0 },
    { 0x78aa876f, 98 },
    { 0xbf2a8770, 98 },
    { 0, 0 },
    { 0xcb3a8772, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaa8778, 90 },
    { 0xc36a8779, 55 },
    { 0, 0 },
    { 0x153a877b, 61 },
    { 0xb6fa877c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faa8781, 85 },
    { 0x776a8781, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a878b, 91 },
    { 0xa82a878c, 33 },
    { 0x9ea878d, 99 },
    { 0x6c6a878b, 90 },
    { 0xd24a878c, 76 },
    { 0x206a878e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4da8795, 56 },
    { 0, 0 },
    { 0xaaa8797, 85 },
    { 0x745a8798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fa879e, 99 },
    { 0, 0 },
    { 0x137a87a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a87a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ba87aa, 78 },
    { 0x6a4a87aa, 95 },
    { 0x578a87ab, 46 },
    { 0x702a87ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a87b1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bea87b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a87b7, 69 },
    { 0, 0 },
    { 0x2bea87b9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2a87bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a87c0, 93 },
    { 0xe55a87c0, 23 },
    { 0, 0 },
    { 0xbeea87c3, 88 },
    { 0x251a87c4, 33 },
    { 0x888a87c4, 33 },
    { 0, 0 },
    { 0x5cca87c7, 88 },
    { 0, 0 },
    { 0xb1fa87c9, 33 },
    { 0x27ba87ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daa87d0, 90 },
    { 0x2dca87d1, 61 },
    { 0, 0 },
    { 0x28aa87d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34a87d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4a87df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dca87e2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0a87e8, 95 },
    { 0, 0 },
    { 0xd38a87ea, 69 },
    { 0xc67a87eb, 55 },
    { 0xb8a87ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aa87f2, 98 },
    { 0xb29a87f3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfa87f6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fa87fb, 61 },
    { 0xdb1a87fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0a8805, 90 },
    { 0x376a8806, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68da8813, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47a8819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aa881d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1a8820, 52 },
    { 0x442a8820, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5a8824, 61 },
    { 0, 0 },
    { 0x795a8826, 98 },
    { 0x9c4a8827, 61 },
    { 0, 0 },
    { 0x53a8829, 90 },
    { 0xce0a8829, 76 },
    { 0xde9a882b, 95 },
    { 0x64fa882c, 46 },
    { 0xd09a882c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3a8843, 43 },
    { 0x301a8844, 99 },
    { 0xe16a8844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43a8848, 77 },
    { 0, 0 },
    { 0xa83a884a, 23 },
    { 0, 0 },
    { 0x9f9a884c, 90 },
    { 0, 0 },
    { 0x14aa884e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dda8851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2a8855, 55 },
    { 0x637a8856, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ba8859, 33 },
    { 0x959a885a, 98 },
    { 0x516a8859, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aa8864, 23 },
    { 0x5e0a8864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aa8868, 33 },
    { 0x7cda8868, 100 },
    { 0xb78a886a, 90 },
    { 0x483a8868, 93 },
    { 0xae7a886c, 55 },
    { 0x675a886d, 90 },
    { 0x360a886e, 33 },
    { 0x7e9a8868, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaa8874, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a887b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ea887e, 33 },
    { 0x96a887f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299a8885, 95 },
    { 0x53da8886, 90 },
    { 0x1b5a8885, 33 },
    { 0x31aa8887, 55 },
    { 0x678a8889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872a888d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722a8892, 61 },
    { 0xba6a8893, 90 },
    { 0x856a8894, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69a8897, 34 },
    { 0, 0 },
    { 0x818a8899, 61 },
    { 0x28a8899, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515a88a1, 95 },
    { 0xf34a88a2, 90 },
    { 0xbb8a88a3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfca88a6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778a88ad, 91 },
    { 0xdd5a88ae, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dea88b7, 90 },
    { 0, 0 },
    { 0x894a88b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75a88bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95da88c2, 55 },
    { 0xf79a88c3, 99 },
    { 0, 0 },
    { 0xb25a88c5, 98 },
    { 0x8fba88c6, 23 },
    { 0x8f2a88c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda88ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585a88d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3a88d5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3a88d8, 52 },
    { 0x53ea88d8, 68 },
    { 0, 0 },
    { 0x4baa88db, 95 },
    { 0xd3ba88dc, 88 },
    { 0x1d5a88dd, 33 },
    { 0x92da88de, 90 },
    { 0, 0 },
    { 0x510a88e0, 91 },
    { 0x2d1a88e1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fa88e5, 23 },
    { 0, 0 },
    { 0xe22a88e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x571a88ea, 93 },
    { 0x9a4a88eb, 100 },
    { 0x734a88ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefa88ef, 78 },
    { 0x78ca88f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4a88f6, 83 },
    { 0, 0 },
    { 0xdf1a88f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7a88fb, 55 },
    { 0x335a88fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7a8901, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0a8904, 90 },
    { 0xaa8a8904, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68aa8908, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9da890b, 95 },
    { 0, 0 },
    { 0xb54a890d, 55 },
    { 0, 0 },
    { 0x76da890f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbda8914, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81da8919, 77 },
    { 0xba1a891a, 76 },
    { 0, 0 },
    { 0x665a891c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88a8921, 98 },
    { 0x82ea8922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1a8926, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ca892e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x905a8931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefa8938, 33 },
    { 0, 0 },
    { 0xecaa893a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x192a893d, 61 },
    { 0x86aa893e, 95 },
    { 0x799a893f, 33 },
    { 0x17da8940, 95 },
    { 0, 0 },
    { 0x963a8942, 34 },
    { 0, 0 },
    { 0xb08a8944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88da894b, 66 },
    { 0xda0a894c, 23 },
    { 0xc48a894d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242a8951, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1a8955, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81a8958, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfa895c, 30 },
    { 0x89ea895d, 61 },
    { 0xdcca895e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839a8962, 87 },
    { 0x79da8963, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69a8969, 61 },
    { 0x940a896a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a896f, 68 },
    { 0xafea8970, 93 },
    { 0xfe1a8971, 46 },
    { 0x8fa8972, 76 },
    { 0x334a8973, 23 },
    { 0x118a8971, 55 },
    { 0x8e1a8971, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0a897b, 33 },
    { 0, 0 },
    { 0x6b4a897d, 90 },
    { 0, 0 },
    { 0x4e1a897f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89a8988, 23 },
    { 0, 0 },
    { 0x2eba898a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ba8990, 61 },
    { 0xaf5a8991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afa8996, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aa8999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3a899d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3a89a6, 61 },
    { 0xe06a89a7, 98 },
    { 0x268a89a7, 90 },
    { 0x4d2a89a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a89ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7a89b1, 76 },
    { 0xd86a89b2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624a89b9, 33 },
    { 0x5aba89b9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfa89bf, 77 },
    { 0xc10a89c0, 100 },
    { 0, 0 },
    { 0x3d4a89c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85a89c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ea89ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ba89cd, 91 },
    { 0x656a89ce, 85 },
    { 0x510a89cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2a89d5, 33 },
    { 0, 0 },
    { 0x5e2a89d7, 33 },
    { 0x561a89d7, 90 },
    { 0x743a89d9, 76 },
    { 0x329a89d7, 95 },
    { 0x709a89db, 93 },
    { 0xd9aa89dc, 43 },
    { 0xf23a89dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ca89e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6a89e5, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffa89e9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca89f0, 61 },
    { 0x41ea89f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fa89f4, 90 },
    { 0xb35a89f4, 95 },
    { 0xb27a89f6, 69 },
    { 0x410a89f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa8a00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fba8a03, 90 },
    { 0, 0 },
    { 0xd9ea8a05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5da8a12, 61 },
    { 0xadba8a13, 99 },
    { 0x779a8a14, 95 },
    { 0x48da8a14, 68 },
    { 0x9aca8a16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aba8a19, 61 },
    { 0, 0 },
    { 0x537a8a1b, 33 },
    { 0x246a8a1c, 46 },
    { 0xb72a8a1d, 46 },
    { 0x342a8a1e, 68 },
    { 0x16ca8a1e, 95 },
    { 0xfa5a8a20, 98 },
    { 0xba1a8a1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a8a26, 61 },
    { 0x8fda8a27, 53 },
    { 0x3faa8a28, 23 },
    { 0x511a8a28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4a8a2c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aa8a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79a8a33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420a8a3a, 76 },
    { 0x17ca8a3b, 36 },
    { 0x585a8a3c, 61 },
    { 0x471a8a3d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350a8a41, 61 },
    { 0, 0 },
    { 0xe04a8a43, 53 },
    { 0x85ea8a44, 34 },
    { 0x536a8a43, 95 },
    { 0xaa9a8a46, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8a8a4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a8a51, 46 },
    { 0, 0 },
    { 0xc27a8a53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778a8a57, 93 },
    { 0, 0 },
    { 0xe60a8a59, 97 },
    { 0, 0 },
    { 0x52fa8a5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x248a8a5e, 78 },
    { 0, 0 },
    { 0xceda8a60, 81 },
    { 0x749a8a60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeda8a6a, 90 },
    { 0x4c4a8a6a, 33 },
    { 0x4dea8a6c, 55 },
    { 0x2a3a8a6d, 66 },
    { 0x208a8a6e, 90 },
    { 0x975a8a6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30a8a76, 76 },
    { 0x809a8a77, 61 },
    { 0, 0 },
    { 0xebca8a79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fa8a7d, 38 },
    { 0, 0 },
    { 0x15da8a7f, 34 },
    { 0, 0 },
    { 0xe36a8a81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a8a89, 99 },
    { 0, 0 },
    { 0x3b9a8a8b, 90 },
    { 0x214a8a8b, 93 },
    { 0xab7a8a8d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ca8a91, 61 },
    { 0x641a8a91, 61 },
    { 0x860a8a93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140a8a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04a8a9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ea8a9d, 95 },
    { 0x1eca8a9e, 91 },
    { 0xb3ba8a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cca8aa3, 75 },
    { 0x418a8aa4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915a8aa8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445a8aac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfa8aaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fa8ab2, 90 },
    { 0x884a8ab3, 98 },
    { 0xa65a8ab4, 55 },
    { 0x144a8ab5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaa8abb, 61 },
    { 0, 0 },
    { 0xc84a8abd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9a8ac1, 41 },
    { 0x2a6a8ac2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a8ac9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ca8acc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a8ad0, 99 },
    { 0, 0 },
    { 0x8caa8ad2, 33 },
    { 0xedfa8ad2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7a8ad6, 43 },
    { 0, 0 },
    { 0x159a8ad8, 95 },
    { 0x523a8ad9, 34 },
    { 0x6d5a8ada, 93 },
    { 0xb17a8adb, 33 },
    { 0xdc2a8adb, 33 },
    { 0x775a8adc, 90 },
    { 0xff7a8ad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ba8ae1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892a8ae5, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dda8aec, 34 },
    { 0x52ca8aed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x845a8af0, 55 },
    { 0, 0 },
    { 0x930a8af2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea8af7, 61 },
    { 0xf8ca8af8, 76 },
    { 0x947a8af9, 61 },
    { 0xd92a8af7, 90 },
    { 0, 0 },
    { 0x9aa8afc, 95 },
    { 0xa9fa8afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca8b00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47a8b03, 93 },
    { 0, 0 },
    { 0xba6a8b05, 66 },
    { 0, 0 },
    { 0x1d5a8b07, 46 },
    { 0x4c1a8b08, 95 },
    { 0xd9fa8b08, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2a8b12, 23 },
    { 0x174a8b13, 43 },
    { 0, 0 },
    { 0x634a8b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35da8b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x896a8b1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ea8b23, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeda8b26, 33 },
    { 0xc09a8b27, 33 },
    { 0, 0 },
    { 0x6fca8b29, 99 },
    { 0, 0 },
    { 0x518a8b2b, 55 },
    { 0, 0 },
    { 0x20ea8b2d, 66 },
    { 0xb0da8b2e, 77 },
    { 0x29da8b2e, 33 },
    { 0xaefa8b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2a8b35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8a8b3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56a8b42, 46 },
    { 0x8f2a8b43, 34 },
    { 0x78da8b42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315a8b48, 93 },
    { 0xb95a8b48, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176a8b4d, 95 },
    { 0x79ba8b4e, 23 },
    { 0x1d1a8b4e, 95 },
    { 0x198a8b50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aaa8b53, 46 },
    { 0x290a8b54, 33 },
    { 0x4aaa8b54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3a8b5c, 85 },
    { 0xb13a8b5c, 61 },
    { 0, 0 },
    { 0xb33a8b5f, 61 },
    { 0, 0 },
    { 0x231a8b61, 84 },
    { 0x8aca8b62, 33 },
    { 0x64a8b63, 100 },
    { 0x3da8b63, 33 },
    { 0, 0 },
    { 0x2cda8b66, 61 },
    { 0xb88a8b67, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xffba8b6a, 33 },
    { 0xc3a8b6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eea8b6e, 98 },
    { 0x863a8b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfa8b73, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ea8b79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3a8b7e, 23 },
    { 0, 0 },
    { 0xce0a8b80, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8a8b84, 52 },
    { 0x3eba8b85, 76 },
    { 0x31ea8b86, 98 },
    { 0x87fa8b87, 90 },
    { 0, 0 },
    { 0x9ca8b89, 38 },
    { 0xe2a8b8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bea8b8d, 66 },
    { 0x978a8b8e, 95 },
    { 0x3a1a8b8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baa8b99, 33 },
    { 0x405a8b9a, 23 },
    { 0, 0 },
    { 0x82da8b9c, 98 },
    { 0xa4da8b9d, 90 },
    { 0xb0da8b9e, 84 },
    { 0xb6ca8b9c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ea8ba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ba8bad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdca8bb2, 34 },
    { 0, 0 },
    { 0xf9ea8bb4, 96 },
    { 0x98a8bb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0a8bbb, 77 },
    { 0x552a8bbc, 84 },
    { 0xfdaa8bbd, 69 },
    { 0xd2ea8bbb, 95 },
    { 0x907a8bbe, 76 },
    { 0xdb8a8bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eba8bc5, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545a8bcb, 90 },
    { 0xa6da8bcc, 23 },
    { 0x5fca8bcb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888a8bd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5a8bd8, 34 },
    { 0, 0 },
    { 0xa04a8bda, 95 },
    { 0xb5fa8bda, 33 },
    { 0, 0 },
    { 0x5fa8bdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60da8be1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85a8be5, 49 },
    { 0x894a8be6, 78 },
    { 0x6fca8be6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59a8bec, 91 },
    { 0, 0 },
    { 0x6cba8bee, 33 },
    { 0x9eda8bef, 98 },
    { 0x9c9a8bf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8a8bf6, 98 },
    { 0, 0 },
    { 0x719a8bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10a8bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ba8bfe, 52 },
    { 0xd44a8bff, 66 },
    { 0xc2fa8bfe, 33 },
    { 0x416a8c01, 33 },
    { 0x272a8c02, 95 },
    { 0x11a8c03, 66 },
    { 0x545a8c03, 34 },
    { 0xc5ca8c04, 43 },
    { 0x29aa8c06, 65 },
    { 0x29ca8c06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9da8c0a, 34 },
    { 0x11ea8c0b, 95 },
    { 0x7a4a8c0c, 90 },
    { 0xbd7a8c0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583a8c11, 91 },
    { 0xc6ca8c12, 33 },
    { 0, 0 },
    { 0x6eea8c14, 76 },
    { 0, 0 },
    { 0x6eda8c16, 81 },
    { 0xa17a8c17, 33 },
    { 0, 0 },
    { 0xf5ea8c19, 76 },
    { 0x964a8c19, 90 },
    { 0x502a8c1b, 65 },
    { 0, 0 },
    { 0xe97a8c1d, 90 },
    { 0, 0 },
    { 0x3b4a8c1f, 55 },
    { 0x21a8c20, 23 },
    { 0, 0 },
    { 0x908a8c22, 85 },
    { 0x81ba8c23, 85 },
    { 0x2baa8c23, 61 },
    { 0x47fa8c23, 43 },
    { 0x992a8c26, 61 },
    { 0x5da8c27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1a8c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7a8c35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4a8c39, 33 },
    { 0x769a8c3a, 43 },
    { 0x2d6a8c3a, 61 },
    { 0xd0ba8c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7a8c3f, 66 },
    { 0, 0 },
    { 0x306a8c41, 23 },
    { 0, 0 },
    { 0x636a8c43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354a8c47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21a8c4e, 90 },
    { 0xfcba8c4e, 95 },
    { 0x99a8c4f, 95 },
    { 0, 0 },
    { 0xa51a8c52, 98 },
    { 0xa12a8c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476a8c59, 66 },
    { 0, 0 },
    { 0xd05a8c5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efa8c5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a8c64, 76 },
    { 0, 0 },
    { 0x4a4a8c66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93a8c74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1a8c79, 95 },
    { 0x74a8c7a, 61 },
    { 0xda8c7a, 23 },
    { 0x76ca8c79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183a8c82, 43 },
    { 0, 0 },
    { 0xc69a8c84, 46 },
    { 0xaa7a8c85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697a8c8a, 61 },
    { 0xf63a8c8b, 95 },
    { 0x5a7a8c8c, 82 },
    { 0xc38a8c8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45da8c90, 99 },
    { 0x7a3a8c91, 23 },
    { 0xa69a8c92, 90 },
    { 0x2f6a8c90, 93 },
    { 0x3aca8c90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caa8c98, 87 },
    { 0xf91a8c98, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5a8ca0, 90 },
    { 0xa43a8ca1, 23 },
    { 0xeea8ca2, 66 },
    { 0xf90a8ca1, 61 },
    { 0xf05a8ca4, 85 },
    { 0xfffa8ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ca8ca8, 34 },
    { 0, 0 },
    { 0x680a8caa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ca8cb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeda8cb7, 33 },
    { 0x4c5a8cb8, 52 },
    { 0x844a8cb9, 34 },
    { 0xbc9a8cba, 23 },
    { 0x5f4a8cb8, 88 },
    { 0xb49a8cbb, 93 },
    { 0x723a8cb9, 95 },
    { 0x84ba8cbb, 23 },
    { 0, 0 },
    { 0x9cba8cc0, 90 },
    { 0xcaaa8cc1, 95 },
    { 0, 0 },
    { 0xfc1a8cc3, 23 },
    { 0, 0 },
    { 0x6f5a8cc5, 95 },
    { 0x4f3a8cc6, 77 },
    { 0x83fa8cc7, 98 },
    { 0x559a8cc8, 91 },
    { 0x352a8cc5, 66 },
    { 0xc8ea8cca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26a8cd1, 61 },
    { 0xaa6a8cd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4da8cd6, 61 },
    { 0xddca8cd7, 23 },
    { 0xbbfa8cd8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79a8cdc, 100 },
    { 0x425a8cdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61a8ce1, 33 },
    { 0, 0 },
    { 0x784a8ce3, 91 },
    { 0, 0 },
    { 0xfcba8ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60a8ce8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fa8cee, 61 },
    { 0xcd2a8cef, 61 },
    { 0x672a8cee, 34 },
    { 0x875a8cf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8a8cf5, 98 },
    { 0, 0 },
    { 0x86fa8cf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a8cfb, 33 },
    { 0, 0 },
    { 0x2c9a8cfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2a8d01, 76 },
    { 0xe0fa8d02, 23 },
    { 0xed8a8d03, 23 },
    { 0xf67a8d04, 90 },
    { 0x877a8d04, 33 },
    { 0x15a8d05, 98 },
    { 0x1b4a8d07, 55 },
    { 0xe77a8d08, 90 },
    { 0xe10a8d06, 98 },
    { 0xfe8a8d02, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfa8d13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8a8d16, 90 },
    { 0x28ca8d17, 61 },
    { 0xb14a8d18, 33 },
    { 0x90ba8d18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9a8d1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4a8d21, 90 },
    { 0, 0 },
    { 0x6f7a8d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ca8d29, 55 },
    { 0, 0 },
    { 0xc4aa8d2b, 84 },
    { 0, 0 },
    { 0x463a8d2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33a8d31, 33 },
    { 0xe83a8d32, 76 },
    { 0x445a8d32, 40 },
    { 0xc9aa8d33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ba8d37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36a8d3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aa8d45, 43 },
    { 0, 0 },
    { 0x56ca8d47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51a8d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1a8d4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ca8d52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaa8d55, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aa8d58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6a8d5d, 67 },
    { 0, 0 },
    { 0x5f4a8d5f, 91 },
    { 0, 0 },
    { 0x33aa8d61, 100 },
    { 0, 0 },
    { 0xd93a8d63, 98 },
    { 0, 0 },
    { 0x4b1a8d65, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8d68, 61 },
    { 0x7a4a8d69, 67 },
    { 0x876a8d6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fa8d6e, 95 },
    { 0, 0 },
    { 0x17ca8d70, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7aa8d77, 91 },
    { 0xf64a8d78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358a8d7d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9a8d84, 23 },
    { 0x59ba8d84, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0a8d88, 17 },
    { 0xa7ea8d88, 98 },
    { 0xb1da8d89, 46 },
    { 0x68ba8d8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478a8d90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffa8d94, 66 },
    { 0, 0 },
    { 0x36ca8d96, 61 },
    { 0x40ba8d96, 90 },
    { 0, 0 },
    { 0xed4a8d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222a8d9e, 98 },
    { 0xa18a8d9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba8da3, 81 },
    { 0x376a8da3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351a8da8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7a8dac, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca8db2, 61 },
    { 0xb1fa8db3, 33 },
    { 0xd1a8db4, 76 },
    { 0, 0 },
    { 0x143a8db6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbea8dbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a8dbf, 90 },
    { 0xb8da8dbf, 33 },
    { 0x934a8dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x260a8dc4, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ca8dd1, 90 },
    { 0x33ba8dd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a8dd6, 90 },
    { 0x8f1a8dd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87aa8dda, 33 },
    { 0, 0 },
    { 0x9a8ddc, 95 },
    { 0x6ca8ddd, 61 },
    { 0x1e4a8dde, 33 },
    { 0, 0 },
    { 0x5cea8de0, 93 },
    { 0x8cda8de1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32a8de4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897a8de9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ca8dee, 46 },
    { 0x436a8def, 23 },
    { 0, 0 },
    { 0xf0ca8df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247a8df5, 23 },
    { 0x1e2a8df5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88a8dfc, 90 },
    { 0xbeda8dfd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fea8e01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8a8e08, 90 },
    { 0x91ca8e09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfa8e14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06a8e1a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1a8e1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45a8e24, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x50da8e27, 56 },
    { 0, 0 },
    { 0xbd5a8e29, 23 },
    { 0, 0 },
    { 0xe57a8e2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4a8e2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccba8e33, 43 },
    { 0xff1a8e34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x703a8e37, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428a8e41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7a8e45, 95 },
    { 0x5da8e45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a8e4b, 99 },
    { 0xe86a8e4b, 65 },
    { 0xa3ca8e4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ba8e53, 80 },
    { 0, 0 },
    { 0x7bfa8e55, 46 },
    { 0x380a8e56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21a8e5a, 40 },
    { 0, 0 },
    { 0x62ea8e5c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceda8e61, 19 },
    { 0xb66a8e62, 91 },
    { 0xc53a8e63, 33 },
    { 0, 0 },
    { 0xaf9a8e65, 55 },
    { 0, 0 },
    { 0x9caa8e67, 67 },
    { 0x20ca8e68, 33 },
    { 0xdffa8e68, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9a8e6f, 98 },
    { 0xc0fa8e70, 61 },
    { 0x6f0a8e70, 61 },
    { 0, 0 },
    { 0xb09a8e73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84a8e80, 61 },
    { 0x7d9a8e81, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697a8e85, 93 },
    { 0xaaca8e86, 33 },
    { 0, 0 },
    { 0x706a8e88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762a8e8d, 55 },
    { 0, 0 },
    { 0x3b5a8e8f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dea8e93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6a8e96, 95 },
    { 0x888a8e96, 76 },
    { 0xf2aa8e98, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a8e9e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fa8ea1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36a8ea4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca8ead, 93 },
    { 0xd54a8eae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fa8eb2, 46 },
    { 0x1e1a8eb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2a8eb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825a8ebb, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a8ebe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a8ec2, 81 },
    { 0xc09a8ec3, 46 },
    { 0xf8ea8ec4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ca8ec9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x403a8ecc, 98 },
    { 0xb6da8ecc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x574a8ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x353a8ed3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cea8ed9, 81 },
    { 0xdc2a8eda, 98 },
    { 0xfd6a8ed9, 46 },
    { 0x1a4a8edc, 98 },
    { 0x75ba8edd, 95 },
    { 0x4bea8edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581a8ee6, 78 },
    { 0xc0a8ee7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8a8eed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355a8ef1, 33 },
    { 0xf56a8ef2, 23 },
    { 0x3faa8ef3, 66 },
    { 0, 0 },
    { 0xafda8ef5, 95 },
    { 0x18fa8ef6, 95 },
    { 0x383a8ef6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34da8f00, 23 },
    { 0x2f0a8f00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2a8f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6a8f0a, 67 },
    { 0xd74a8f0a, 75 },
    { 0, 0 },
    { 0x1ffa8f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeca8f12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a8f17, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaa8f1a, 93 },
    { 0x87a8f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fa8f26, 95 },
    { 0x27da8f26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ba8f2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x152a8f2d, 23 },
    { 0, 0 },
    { 0xcaaa8f2f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885a8f34, 95 },
    { 0x873a8f35, 90 },
    { 0xc1a8f36, 23 },
    { 0x7b0a8f35, 95 },
    { 0x942a8f38, 90 },
    { 0xb2da8f36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x511a8f3c, 93 },
    { 0, 0 },
    { 0x4fa8f3e, 33 },
    { 0x27ca8f3f, 61 },
    { 0x134a8f3e, 98 },
    { 0x6dba8f41, 98 },
    { 0xccfa8f42, 33 },
    { 0x21ea8f41, 23 },
    { 0xa7da8f41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1a8f47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x180a8f4a, 33 },
    { 0x551a8f4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x240a8f4e, 34 },
    { 0, 0 },
    { 0x2e1a8f50, 55 },
    { 0xb49a8f51, 33 },
    { 0xd4a8f52, 23 },
    { 0x753a8f53, 34 },
    { 0xfe6a8f54, 81 },
    { 0x620a8f55, 33 },
    { 0x630a8f56, 34 },
    { 0, 0 },
    { 0x6bca8f58, 90 },
    { 0x36ca8f58, 61 },
    { 0, 0 },
    { 0x6fea8f5b, 72 },
    { 0x9daa8f5c, 46 },
    { 0x9aea8f5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a8f60, 33 },
    { 0x2dda8f61, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fa8f67, 91 },
    { 0x335a8f67, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37a8f6c, 95 },
    { 0x44ba8f6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64a8f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ba8f76, 98 },
    { 0xe19a8f76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aa8f7f, 88 },
    { 0, 0 },
    { 0x2a0a8f81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fa8f87, 23 },
    { 0x381a8f88, 90 },
    { 0x222a8f89, 76 },
    { 0x689a8f8a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9a8f90, 84 },
    { 0, 0 },
    { 0xe3aa8f92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ea8f96, 23 },
    { 0xde0a8f97, 23 },
    { 0x218a8f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbca8f9b, 23 },
    { 0, 0 },
    { 0xf83a8f9d, 90 },
    { 0, 0 },
    { 0xe56a8f9f, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50a8fa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570a8fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x797a8fab, 77 },
    { 0x312a8fac, 33 },
    { 0x8bda8fad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8a8fb4, 91 },
    { 0x4dfa8fb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8a8fb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fa8fc0, 61 },
    { 0xa3da8fc1, 77 },
    { 0x385a8fc0, 90 },
    { 0xea2a8fc3, 94 },
    { 0x4eba8fc3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139a8fcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfda8fd0, 33 },
    { 0x392a8fd1, 98 },
    { 0x123a8fd2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49a8fdf, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a8fe7, 34 },
    { 0, 0 },
    { 0x3cba8fe9, 76 },
    { 0xe3fa8fe9, 34 },
    { 0x74ba8feb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fa8fee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40a8ff3, 38 },
    { 0, 0 },
    { 0xaaea8ff5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dea8ffa, 42 },
    { 0xe6fa8ffb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7a8fff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ba9003, 81 },
    { 0, 0 },
    { 0xbd4a9005, 34 },
    { 0xa9ea9005, 81 },
    { 0, 0 },
    { 0x7bba9008, 33 },
    { 0x865a9009, 99 },
    { 0x7f5a900a, 75 },
    { 0x239a900a, 76 },
    { 0x4cda9009, 99 },
    { 0x4f1a9008, 90 },
    { 0x847a900e, 52 },
    { 0, 0 },
    { 0x3b4a9010, 81 },
    { 0xfc4a9011, 65 },
    { 0xf11a9012, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3a9018, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a901c, 95 },
    { 0xf86a901c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52a9020, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dda9024, 86 },
    { 0xda2a9024, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baa9028, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18a902f, 33 },
    { 0, 0 },
    { 0xd00a9031, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474a9036, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5a9039, 90 },
    { 0x1ada903a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cea9042, 65 },
    { 0xe32a9043, 23 },
    { 0x6a5a9043, 95 },
    { 0, 0 },
    { 0xa0fa9046, 95 },
    { 0x1cda9047, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0a904c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258a9055, 52 },
    { 0xd77a9055, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x929a9059, 43 },
    { 0x477a9059, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ca905e, 90 },
    { 0, 0 },
    { 0xa37a9060, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1a9066, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568a906a, 77 },
    { 0, 0 },
    { 0xb3a906c, 34 },
    { 0xb60a906d, 95 },
    { 0x1e6a906e, 98 },
    { 0x6c5a906e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63a9072, 55 },
    { 0x76fa9073, 61 },
    { 0xfb2a9073, 98 },
    { 0, 0 },
    { 0x5a4a9076, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7a907f, 98 },
    { 0x853a907f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aa9086, 33 },
    { 0xe8ea9087, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2a908a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66a908d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856a9092, 17 },
    { 0x31fa9093, 90 },
    { 0x99da9092, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137a9099, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404a90a0, 87 },
    { 0x759a90a1, 23 },
    { 0xf9ea90a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223a90a6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ea90aa, 61 },
    { 0x94ba90ab, 33 },
    { 0, 0 },
    { 0xca3a90ad, 90 },
    { 0x971a90ad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a90b1, 88 },
    { 0xccca90b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aa90b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14a90bc, 90 },
    { 0x6e1a90bd, 101 },
    { 0xb6ba90be, 40 },
    { 0xc2ba90be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bca90cc, 61 },
    { 0x450a90cd, 24 },
    { 0x72ba90ce, 90 },
    { 0x434a90cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5a90d2, 76 },
    { 0x872a90d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x560a90d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ca90dc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3a90e0, 43 },
    { 0x577a90e1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ba90e4, 34 },
    { 0x242a90e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x849a90e8, 64 },
    { 0x567a90e9, 61 },
    { 0x214a90ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda90ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fa90f1, 81 },
    { 0x327a90f2, 76 },
    { 0x693a90f2, 98 },
    { 0xe2fa90f4, 84 },
    { 0xb50a90f3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812a90f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8daa90fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfa9101, 95 },
    { 0xb69a9102, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0a9106, 88 },
    { 0x171a9107, 43 },
    { 0, 0 },
    { 0xbe3a9109, 33 },
    { 0xb5fa910a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ca910d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8a9117, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09a911b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ba911f, 23 },
    { 0x3e1a911f, 43 },
    { 0x30fa9120, 88 },
    { 0x400a9122, 52 },
    { 0xc00a911f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4a9127, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ba912c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58a912f, 61 },
    { 0x782a912f, 90 },
    { 0, 0 },
    { 0xc03a9132, 33 },
    { 0x3b8a9133, 93 },
    { 0x2f9a9132, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x228a9137, 84 },
    { 0x754a9138, 65 },
    { 0x9f6a9137, 87 },
    { 0xa81a913a, 95 },
    { 0x1b5a913a, 23 },
    { 0, 0 },
    { 0xbc7a913d, 61 },
    { 0xa3da913e, 88 },
    { 0x7a9a913d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4a9142, 95 },
    { 0x364a9142, 61 },
    { 0, 0 },
    { 0xa79a9145, 61 },
    { 0, 0 },
    { 0xd7da9147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aa914d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x243a9150, 85 },
    { 0, 0 },
    { 0x6e8a9152, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a915b, 78 },
    { 0xc41a915c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7a9162, 43 },
    { 0, 0 },
    { 0xf9ca9164, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aa9167, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffa916b, 46 },
    { 0, 0 },
    { 0xaa0a916d, 98 },
    { 0x9daa916e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaca9171, 53 },
    { 0x535a9172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aa9175, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ca9178, 95 },
    { 0x76ea9179, 34 },
    { 0x57a917a, 78 },
    { 0x179a917b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203a917f, 91 },
    { 0, 0 },
    { 0x374a9181, 61 },
    { 0xb9ca9182, 95 },
    { 0xd20a9183, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7a918c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4a918f, 23 },
    { 0x1d7a918f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ba9195, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753a919e, 90 },
    { 0x704a919e, 67 },
    { 0, 0 },
    { 0xd3ea91a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fea91a6, 99 },
    { 0xacaa91a6, 95 },
    { 0, 0 },
    { 0x258a91a9, 33 },
    { 0, 0 },
    { 0x8e5a91ab, 23 },
    { 0xb2fa91ac, 33 },
    { 0, 0 },
    { 0x567a91ae, 34 },
    { 0xdd1a91ae, 93 },
    { 0, 0 },
    { 0x9dba91b1, 52 },
    { 0, 0 },
    { 0xeeda91b3, 23 },
    { 0x8e3a91b4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0a91b8, 98 },
    { 0, 0 },
    { 0x14fa91ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffca91bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7a91c1, 38 },
    { 0xb55a91c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744a91c8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332a91cd, 61 },
    { 0, 0 },
    { 0x7bda91cf, 65 },
    { 0xafca91cf, 61 },
    { 0xa43a91d1, 61 },
    { 0x9d3a91d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ba91d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855a91db, 33 },
    { 0, 0 },
    { 0xf7fa91dd, 61 },
    { 0x5cfa91de, 33 },
    { 0, 0 },
    { 0xc84a91e0, 90 },
    { 0, 0 },
    { 0xc2ca91e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfa91e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ba91ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaa91f8, 98 },
    { 0x46ba91f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488a9200, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fa9209, 43 },
    { 0x1dfa920a, 90 },
    { 0x8efa920a, 99 },
    { 0x417a920c, 51 },
    { 0x7bea920c, 23 },
    { 0x5a9209, 33 },
    { 0xe09a920f, 84 },
    { 0x39aa920f, 61 },
    { 0xa3da920d, 23 },
    { 0xceea920f, 52 },
    { 0, 0 },
    { 0x6a9a9214, 69 },
    { 0, 0 },
    { 0x1ca9216, 34 },
    { 0x4d8a9217, 91 },
    { 0xedea9217, 34 },
    { 0x9eaa9219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a9221, 23 },
    { 0xca9a9221, 90 },
    { 0, 0 },
    { 0x445a9224, 52 },
    { 0xcd3a9224, 67 },
    { 0xa22a9224, 40 },
    { 0x37ba9227, 23 },
    { 0xdfba9228, 33 },
    { 0, 0 },
    { 0x78fa922a, 85 },
    { 0x427a922b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21da9233, 95 },
    { 0x658a9233, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ba9238, 33 },
    { 0xbdea9239, 33 },
    { 0, 0 },
    { 0x80ba923b, 78 },
    { 0x35ca923b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2a9245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7a9249, 98 },
    { 0xf45a9249, 23 },
    { 0xcea924b, 52 },
    { 0x75da924c, 77 },
    { 0x580a924c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ea9250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15a9259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571a925e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2a9262, 53 },
    { 0x808a9262, 61 },
    { 0, 0 },
    { 0xc7fa9265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4a926a, 76 },
    { 0, 0 },
    { 0x62da926c, 83 },
    { 0xd47a926d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dea9270, 52 },
    { 0xbe7a9271, 95 },
    { 0x1c8a9272, 90 },
    { 0, 0 },
    { 0xf7ca9274, 43 },
    { 0x959a9275, 31 },
    { 0xa99a9276, 100 },
    { 0x954a9275, 55 },
    { 0xec3a9275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cda9280, 23 },
    { 0x977a9281, 100 },
    { 0x883a9281, 93 },
    { 0x53a9283, 76 },
    { 0x73ea9281, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x184a9287, 98 },
    { 0x81ba9288, 59 },
    { 0xa88a9289, 64 },
    { 0, 0 },
    { 0x548a928b, 23 },
    { 0xbe8a928c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aa9290, 84 },
    { 0xc8ba9291, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fa9296, 90 },
    { 0x561a9297, 33 },
    { 0x299a9297, 96 },
    { 0x9e1a9299, 91 },
    { 0xf76a929a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdba929e, 55 },
    { 0x76aa929f, 67 },
    { 0xd0aa929e, 95 },
    { 0x376a92a1, 66 },
    { 0, 0 },
    { 0x4e0a92a3, 90 },
    { 0x9eea92a4, 61 },
    { 0x2e9a92a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fa92a9, 93 },
    { 0, 0 },
    { 0x2b4a92ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43a92b0, 23 },
    { 0xf1ca92b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7a92b7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8a92bb, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeea92be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeea92c4, 17 },
    { 0x836a92c5, 23 },
    { 0x544a92c5, 90 },
    { 0x109a92c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92a92ca, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2a92d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8a92d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9a92dc, 99 },
    { 0xc02a92dd, 93 },
    { 0, 0 },
    { 0xb13a92df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcca92e3, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfa92e7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718a92eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eea92ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1a92f7, 98 },
    { 0x3d9a92f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423a92fc, 33 },
    { 0xcf5a92fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fa9301, 23 },
    { 0, 0 },
    { 0x559a9303, 90 },
    { 0, 0 },
    { 0x41a9305, 100 },
    { 0x922a9305, 64 },
    { 0, 0 },
    { 0x431a9308, 77 },
    { 0, 0 },
    { 0x527a930a, 46 },
    { 0x7f7a930b, 33 },
    { 0xf07a930c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7a930f, 61 },
    { 0, 0 },
    { 0x108a9311, 84 },
    { 0, 0 },
    { 0x412a9313, 23 },
    { 0xeeaa9314, 90 },
    { 0, 0 },
    { 0xa36a9316, 55 },
    { 0x2c6a9317, 75 },
    { 0, 0 },
    { 0xd06a9319, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840a931d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6a9320, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540a9325, 67 },
    { 0x795a9326, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84a9329, 93 },
    { 0, 0 },
    { 0x979a932b, 61 },
    { 0x297a932b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1a932f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6a9334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67a933b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74a933e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea9342, 100 },
    { 0x245a9343, 55 },
    { 0x6bba9343, 90 },
    { 0xe72a9345, 23 },
    { 0x705a9346, 91 },
    { 0, 0 },
    { 0x5faa9348, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06a9350, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14a9354, 98 },
    { 0, 0 },
    { 0x6b6a9356, 95 },
    { 0x5c5a9356, 90 },
    { 0xf8ba9358, 61 },
    { 0, 0 },
    { 0x933a935a, 40 },
    { 0xc38a935a, 23 },
    { 0xa9a935a, 33 },
    { 0xfb8a935c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ba9360, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56a9368, 33 },
    { 0xd2fa9369, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46a9375, 33 },
    { 0, 0 },
    { 0x3d0a9377, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebba937b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ea937f, 61 },
    { 0x75aa937f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32a938c, 23 },
    { 0, 0 },
    { 0x639a938e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ca9391, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaa9399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77da939c, 46 },
    { 0xb3ba939d, 33 },
    { 0xfa2a939e, 61 },
    { 0, 0 },
    { 0xcf7a93a0, 77 },
    { 0x478a93a0, 87 },
    { 0xbea93a2, 33 },
    { 0x7f2a93a3, 93 },
    { 0x3dca93a1, 90 },
    { 0x5c5a93a2, 61 },
    { 0x584a93a6, 95 },
    { 0xb09a93a7, 67 },
    { 0xecfa93a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0a93ac, 98 },
    { 0, 0 },
    { 0x587a93ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628a93b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca93b9, 76 },
    { 0x97a93ba, 23 },
    { 0x2f2a93ba, 90 },
    { 0xcdfa93bc, 90 },
    { 0x4c7a93bd, 66 },
    { 0xc2fa93be, 97 },
    { 0x430a93be, 23 },
    { 0x4b8a93be, 90 },
    { 0xcf9a93bb, 17 },
    { 0, 0 },
    { 0xe80a93c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594a93cc, 34 },
    { 0xd0aa93cd, 90 },
    { 0x5a2a93cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23a93d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86a93d4, 23 },
    { 0xddfa93d4, 80 },
    { 0, 0 },
    { 0x135a93d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ea93da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x948a93dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ca93e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516a93e6, 100 },
    { 0xda0a93e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6a93ed, 97 },
    { 0, 0 },
    { 0x9cea93ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2a93f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19a93f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37a93f9, 32 },
    { 0xac8a93f9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85a93fd, 78 },
    { 0xad8a93fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71aa9407, 52 },
    { 0xf54a9408, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734a940f, 23 },
    { 0, 0 },
    { 0x3eba9411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94a9414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x991a9417, 61 },
    { 0, 0 },
    { 0x51ba9419, 76 },
    { 0x7eca9419, 85 },
    { 0, 0 },
    { 0x13ca941c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628a9425, 61 },
    { 0, 0 },
    { 0x579a9427, 23 },
    { 0, 0 },
    { 0x36ea9429, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2a9432, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1a9436, 81 },
    { 0xf29a9436, 98 },
    { 0xdc5a9436, 95 },
    { 0xceda9438, 95 },
    { 0xd7ea943a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1a943d, 90 },
    { 0x746a943d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139a9443, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5a944e, 23 },
    { 0, 0 },
    { 0x48aa9450, 84 },
    { 0x7b5a9451, 87 },
    { 0xb2a9450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73a9455, 23 },
    { 0x623a9455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38a9459, 95 },
    { 0x478a945a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x904a945d, 61 },
    { 0x70da945e, 68 },
    { 0, 0 },
    { 0x214a9460, 90 },
    { 0, 0 },
    { 0x9cea9462, 75 },
    { 0x1b8a9463, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26a9467, 84 },
    { 0xde3a9468, 95 },
    { 0xc34a9467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365a946d, 33 },
    { 0xbc0a946e, 76 },
    { 0, 0 },
    { 0x455a9470, 33 },
    { 0x810a9470, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649a9480, 98 },
    { 0x248a9481, 61 },
    { 0x75da9482, 23 },
    { 0, 0 },
    { 0x58ca9484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5a948a, 61 },
    { 0xf44a948b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ca9490, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3a9496, 95 },
    { 0, 0 },
    { 0xd92a9498, 61 },
    { 0x553a9499, 33 },
    { 0xd9da9499, 61 },
    { 0xddba9498, 98 },
    { 0xd77a949c, 77 },
    { 0x29ba949d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x734a94a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0a94a8, 85 },
    { 0x26fa94a9, 65 },
    { 0x95a94a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87a94ae, 91 },
    { 0x8a0a94af, 76 },
    { 0xf8da94b0, 77 },
    { 0x8e5a94b0, 76 },
    { 0x645a94b2, 90 },
    { 0xe3ca94b3, 52 },
    { 0xc71a94b4, 66 },
    { 0x9e1a94b3, 76 },
    { 0xefda94b6, 76 },
    { 0x10aa94b7, 46 },
    { 0x521a94af, 84 },
    { 0x31ba94ae, 61 },
    { 0, 0 },
    { 0xcf8a94bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3a94c0, 55 },
    { 0x922a94c1, 34 },
    { 0x546a94c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5a94c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591a94ca, 76 },
    { 0x56ba94cb, 55 },
    { 0xc77a94cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56a94d5, 90 },
    { 0, 0 },
    { 0x63fa94d7, 78 },
    { 0, 0 },
    { 0xb69a94d9, 95 },
    { 0xcc9a94d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971a94e2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x21aa94e5, 93 },
    { 0, 0 },
    { 0x168a94e7, 95 },
    { 0x60ca94e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994a94ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bca94f0, 61 },
    { 0, 0 },
    { 0xe46a94f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845a94f6, 33 },
    { 0xb51a94f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ba94fa, 33 },
    { 0, 0 },
    { 0xccea94fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2a9500, 93 },
    { 0x4eea9500, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfa9504, 76 },
    { 0x98ba9505, 33 },
    { 0x1c5a9505, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46da950a, 98 },
    { 0xf69a950b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94a950e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eca9512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3a9515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950a951a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68a9520, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436a9525, 55 },
    { 0, 0 },
    { 0xc8ca9527, 95 },
    { 0xd10a9528, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfa952b, 23 },
    { 0xb44a952c, 33 },
    { 0xd12a952c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0a9530, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6a9533, 96 },
    { 0x8d3a9533, 61 },
    { 0x51da9535, 17 },
    { 0x421a9536, 67 },
    { 0x39a9535, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2a953e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89a9541, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03a9544, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34a9547, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x175a954a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300a955b, 98 },
    { 0xc1ca955c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x584a955f, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788a9564, 66 },
    { 0x205a9565, 68 },
    { 0x34a9566, 23 },
    { 0xc3da9565, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fa956e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fa9574, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5a9577, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ea957f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8a9582, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41a9585, 68 },
    { 0xb98a9585, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672a958b, 34 },
    { 0x3ea958c, 61 },
    { 0x658a958d, 90 },
    { 0xe90a958d, 23 },
    { 0x6aaa958e, 34 },
    { 0x28fa958d, 55 },
    { 0x658a9591, 90 },
    { 0x44a9592, 34 },
    { 0xd39a958d, 98 },
    { 0xbe5a9594, 90 },
    { 0, 0 },
    { 0x1a3a9596, 93 },
    { 0x60da9597, 23 },
    { 0x9cca9598, 98 },
    { 0x599a9598, 77 },
    { 0x10da959a, 95 },
    { 0x51fa959b, 88 },
    { 0x979a959b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6a959f, 90 },
    { 0xa64a959f, 93 },
    { 0xda4a95a1, 78 },
    { 0x72aa95a2, 90 },
    { 0xb6aa959f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5a95a8, 55 },
    { 0x981a95a8, 98 },
    { 0xab0a95a8, 34 },
    { 0, 0 },
    { 0x9aa95ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a95b4, 91 },
    { 0, 0 },
    { 0xb40a95b6, 33 },
    { 0xbf3a95b7, 46 },
    { 0x1a2a95b8, 98 },
    { 0x77a95b9, 76 },
    { 0x1da95b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98a95bd, 23 },
    { 0x80aa95bd, 34 },
    { 0xf50a95be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffa95c4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x368a95c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25a95cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36a95d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ca95de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7a95e2, 90 },
    { 0x18da95e3, 51 },
    { 0x28ba95e4, 76 },
    { 0x6a4a95e5, 33 },
    { 0xdaba95e5, 76 },
    { 0x193a95e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7a95ea, 52 },
    { 0x959a95eb, 52 },
    { 0x841a95eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34a95fa, 17 },
    { 0, 0 },
    { 0x9a6a95fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3a9609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7a960f, 61 },
    { 0xa0aa9610, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6a9615, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x385a9618, 88 },
    { 0x779a9618, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98da961e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x188a9621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7a9624, 77 },
    { 0, 0 },
    { 0x902a9626, 93 },
    { 0, 0 },
    { 0x782a9628, 98 },
    { 0xe5ca9628, 95 },
    { 0xed5a962a, 46 },
    { 0xecfa962a, 95 },
    { 0, 0 },
    { 0x3d6a962d, 93 },
    { 0xa91a962e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x866a9631, 43 },
    { 0x1eea9632, 8 },
    { 0x727a9633, 33 },
    { 0, 0 },
    { 0xcc9a9635, 61 },
    { 0x979a9636, 61 },
    { 0xe10a9637, 40 },
    { 0x6aea9638, 90 },
    { 0x42ea9636, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eba9641, 34 },
    { 0xd95a9641, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912a964a, 93 },
    { 0x3c5a964b, 33 },
    { 0, 0 },
    { 0x917a964d, 93 },
    { 0xc8ba964d, 61 },
    { 0x596a964f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636a9653, 98 },
    { 0x5b3a9654, 33 },
    { 0, 0 },
    { 0x190a9656, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14a965a, 52 },
    { 0, 0 },
    { 0xfb7a965c, 33 },
    { 0, 0 },
    { 0x2e2a965e, 80 },
    { 0x25fa965f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54a9663, 46 },
    { 0, 0 },
    { 0xfd8a9665, 33 },
    { 0xe56a9666, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fa966a, 76 },
    { 0x1eca966b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9a966e, 98 },
    { 0xab8a966f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408a9673, 95 },
    { 0x2e9a9674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a9678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33a967b, 23 },
    { 0, 0 },
    { 0xaaca967d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1a9687, 80 },
    { 0xf1ea9688, 61 },
    { 0xf07a9688, 61 },
    { 0x625a9689, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bda968d, 77 },
    { 0x9f7a968e, 90 },
    { 0, 0 },
    { 0x736a9690, 34 },
    { 0x66ba9691, 52 },
    { 0xee5a9691, 33 },
    { 0xd61a9692, 26 },
    { 0x58ca9693, 100 },
    { 0, 0 },
    { 0xf49a9696, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0a9699, 68 },
    { 0, 0 },
    { 0xc21a969b, 61 },
    { 0, 0 },
    { 0x537a969d, 98 },
    { 0x926a969d, 76 },
    { 0xd04a969f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57a96a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8a96a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ea96bb, 90 },
    { 0xd6ba96bc, 77 },
    { 0x820a96bd, 81 },
    { 0x1c1a96bb, 95 },
    { 0x365a96bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ea96c3, 66 },
    { 0, 0 },
    { 0xa80a96c5, 76 },
    { 0x611a96c6, 98 },
    { 0xe45a96c7, 90 },
    { 0, 0 },
    { 0xe6ca96c9, 90 },
    { 0, 0 },
    { 0xea2a96cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6a96ce, 61 },
    { 0, 0 },
    { 0x601a96d0, 66 },
    { 0xf42a96d0, 61 },
    { 0xd92a96d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4a96d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x274a96da, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65a96dd, 98 },
    { 0, 0 },
    { 0x9dca96df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3a96e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1a96e7, 61 },
    { 0xf16a96e7, 90 },
    { 0x50aa96e9, 90 },
    { 0, 0 },
    { 0xf6fa96eb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6a96f0, 33 },
    { 0x44ba96f1, 95 },
    { 0x571a96f1, 98 },
    { 0xe08a96f3, 90 },
    { 0xc73a96f3, 95 },
    { 0x612a96f3, 33 },
    { 0xb9a96f6, 78 },
    { 0x14a96f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x918a96fa, 95 },
    { 0x80da96fa, 95 },
    { 0, 0 },
    { 0xb89a96fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ca9700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543a9704, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7a9708, 76 },
    { 0xd72a9709, 76 },
    { 0x4bca9709, 95 },
    { 0xd92a970a, 61 },
    { 0x9a9709, 90 },
    { 0xc9ca970d, 93 },
    { 0, 0 },
    { 0xc18a970f, 43 },
    { 0xa49a970f, 33 },
    { 0xf25a9711, 33 },
    { 0x8e0a9712, 33 },
    { 0x622a9713, 38 },
    { 0xe26a9714, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ca971a, 52 },
    { 0xc72a971a, 87 },
    { 0xa87a971c, 100 },
    { 0x76ea971d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ca9720, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeca9724, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47a9727, 87 },
    { 0xb67a9727, 95 },
    { 0x9ea9729, 76 },
    { 0xbe7a972a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05a972d, 61 },
    { 0, 0 },
    { 0xb2aa972f, 91 },
    { 0x6fda9730, 33 },
    { 0, 0 },
    { 0x31aa9732, 40 },
    { 0x964a9732, 33 },
    { 0x69fa9733, 23 },
    { 0xcfaa9735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a973a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x465a973d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa9741, 95 },
    { 0xc7ba9742, 91 },
    { 0, 0 },
    { 0x3b1a9744, 34 },
    { 0, 0 },
    { 0xe1ca9746, 90 },
    { 0xdeba9747, 93 },
    { 0x553a9746, 95 },
    { 0x2e1a9747, 52 },
    { 0xfdda974a, 76 },
    { 0xc8fa9747, 43 },
    { 0x86ea974c, 55 },
    { 0x7d0a974d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1a9752, 90 },
    { 0x699a9752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7aa9759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ba9767, 53 },
    { 0xdb4a9768, 93 },
    { 0x10ba9769, 90 },
    { 0xb76a976a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aa976d, 90 },
    { 0x60da976e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04a9771, 61 },
    { 0x6da9771, 17 },
    { 0xd9a9771, 61 },
    { 0, 0 },
    { 0x5c4a9775, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7a977a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacea977f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x205a9782, 91 },
    { 0x82aa9782, 61 },
    { 0x406a9784, 43 },
    { 0x19ca9785, 52 },
    { 0x4c3a9786, 61 },
    { 0xf28a9784, 61 },
    { 0x3aa9785, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca978c, 90 },
    { 0, 0 },
    { 0xb6ea978e, 81 },
    { 0x8a7a978e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17a979b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5a97a0, 61 },
    { 0, 0 },
    { 0xdbea97a2, 76 },
    { 0x684a97a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaa97a6, 91 },
    { 0, 0 },
    { 0xee3a97a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ca97ac, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263a97b0, 98 },
    { 0x2d9a97b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9a97b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106a97c3, 98 },
    { 0, 0 },
    { 0x54ea97c5, 84 },
    { 0x981a97c6, 78 },
    { 0x27da97c7, 87 },
    { 0xf06a97c7, 23 },
    { 0x8fa97c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616a97cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79a97d1, 84 },
    { 0x2f9a97d2, 61 },
    { 0x5e7a97d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69a97d7, 23 },
    { 0xd93a97d8, 98 },
    { 0xd58a97d7, 90 },
    { 0x1e0a97d7, 90 },
    { 0x192a97db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x508a97de, 75 },
    { 0x183a97df, 23 },
    { 0x89fa97e0, 90 },
    { 0xaeca97e1, 33 },
    { 0x330a97e1, 33 },
    { 0xcc2a97e3, 91 },
    { 0x3a9a97e4, 81 },
    { 0x758a97e4, 76 },
    { 0, 0 },
    { 0x57ca97e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895a97ed, 84 },
    { 0x4fea97ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18a97f1, 23 },
    { 0x4ea97f2, 90 },
    { 0xf56a97f3, 95 },
    { 0x25ba97f4, 23 },
    { 0xb8ca97f5, 33 },
    { 0, 0 },
    { 0x845a97f7, 23 },
    { 0x26aa97f8, 98 },
    { 0x97ca97f9, 43 },
    { 0x639a97fa, 46 },
    { 0xf36a97fb, 100 },
    { 0xc6ea97fc, 76 },
    { 0x8eca97fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66a9804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ba9808, 84 },
    { 0x69ca9809, 34 },
    { 0, 0 },
    { 0xd5fa980b, 84 },
    { 0x272a980c, 55 },
    { 0x3c5a980b, 95 },
    { 0, 0 },
    { 0x15ea980f, 90 },
    { 0x2eca980f, 98 },
    { 0x360a9811, 90 },
    { 0x409a9811, 99 },
    { 0, 0 },
    { 0xf04a9814, 95 },
    { 0xe24a9814, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aba981d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838a9822, 96 },
    { 0x17aa9823, 61 },
    { 0x880a9823, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aba982b, 100 },
    { 0x15ba982c, 61 },
    { 0x7f3a982d, 33 },
    { 0xe9fa982e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8a9831, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cda9834, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x861a9837, 69 },
    { 0x615a9837, 95 },
    { 0xa26a9839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254a983e, 85 },
    { 0x5f8a983f, 66 },
    { 0xc82a9840, 33 },
    { 0x138a9840, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ba9846, 23 },
    { 0xfaaa9847, 90 },
    { 0, 0 },
    { 0x769a9849, 23 },
    { 0x3cfa984a, 34 },
    { 0, 0 },
    { 0xa5fa984c, 90 },
    { 0xb5fa984d, 98 },
    { 0xac5a984d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ea9851, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfda985b, 43 },
    { 0x7c0a985c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151a9862, 55 },
    { 0x746a9862, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8a9867, 76 },
    { 0x74da9867, 90 },
    { 0x304a9869, 33 },
    { 0x9aba9867, 90 },
    { 0x4b9a986b, 55 },
    { 0xd8aa986c, 34 },
    { 0x10aa986d, 93 },
    { 0x2aa986e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fa9871, 61 },
    { 0, 0 },
    { 0x130a9873, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaa9880, 93 },
    { 0, 0 },
    { 0xc0fa9882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923a9886, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14a988e, 91 },
    { 0x8d9a988e, 51 },
    { 0, 0 },
    { 0xca0a9891, 36 },
    { 0xc0a9892, 88 },
    { 0, 0 },
    { 0xaeda9894, 46 },
    { 0x653a9895, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa9898, 23 },
    { 0x3a4a9898, 96 },
    { 0, 0 },
    { 0x15a989b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58a989f, 90 },
    { 0x93ea98a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aa98a4, 76 },
    { 0x14da98a4, 43 },
    { 0xe6fa98a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ca98ad, 23 },
    { 0xd13a98ae, 82 },
    { 0xbb2a98af, 43 },
    { 0x5c8a98ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ca98b3, 98 },
    { 0, 0 },
    { 0x165a98b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7a98bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555a98c3, 23 },
    { 0xd5a98c3, 90 },
    { 0x743a98c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ba98c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87a98cb, 76 },
    { 0xc04a98cc, 55 },
    { 0x49ca98cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x480a98d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ca98d4, 90 },
    { 0xcefa98d4, 98 },
    { 0x9e2a98d6, 33 },
    { 0, 0 },
    { 0xbeba98d8, 90 },
    { 0x1cba98d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ca98dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85a98df, 76 },
    { 0x6bba98df, 55 },
    { 0xe04a98e1, 17 },
    { 0, 0 },
    { 0x5d6a98e3, 46 },
    { 0x314a98e4, 23 },
    { 0x500a98e4, 68 },
    { 0xba0a98e4, 61 },
    { 0, 0 },
    { 0x47ea98e8, 33 },
    { 0, 0 },
    { 0xa2aa98ea, 61 },
    { 0xe7ba98eb, 33 },
    { 0, 0 },
    { 0xf6aa98ed, 90 },
    { 0, 0 },
    { 0x13ca98ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8a98f2, 90 },
    { 0x885a98f3, 77 },
    { 0, 0 },
    { 0xbc0a98f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0a98fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eea9903, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0a990a, 43 },
    { 0, 0 },
    { 0xbfba990c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2a9911, 95 },
    { 0x347a9912, 90 },
    { 0xd6ba9913, 61 },
    { 0x9d1a9914, 43 },
    { 0x4a8a9913, 33 },
    { 0, 0 },
    { 0x290a9917, 33 },
    { 0xc78a9918, 34 },
    { 0x301a9918, 98 },
    { 0x965a991a, 34 },
    { 0x974a991b, 88 },
    { 0xafca991c, 78 },
    { 0x154a9918, 23 },
    { 0x4aa991e, 23 },
    { 0x1cea9919, 100 },
    { 0x150a9920, 34 },
    { 0, 0 },
    { 0xa86a9922, 23 },
    { 0x9d8a9923, 61 },
    { 0x89fa9923, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebea9928, 98 },
    { 0x9d4a9929, 40 },
    { 0x3c8a9929, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77a992d, 98 },
    { 0xe8a992d, 98 },
    { 0xf02a992f, 99 },
    { 0xddca992e, 85 },
    { 0, 0 },
    { 0xc04a9932, 90 },
    { 0xe1aa9933, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffa993a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ea993d, 90 },
    { 0, 0 },
    { 0xf7da993f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653a9943, 98 },
    { 0xa78a9944, 68 },
    { 0, 0 },
    { 0x73da9946, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fa994d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2a9951, 61 },
    { 0xe64a9951, 33 },
    { 0, 0 },
    { 0x3a9a9954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ba9960, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608a9969, 8 },
    { 0xc15a9969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x286a996d, 90 },
    { 0x38ea996e, 95 },
    { 0x64ea996f, 23 },
    { 0xc66a996e, 90 },
    { 0x154a9971, 95 },
    { 0x6e8a9971, 77 },
    { 0, 0 },
    { 0xb32a9974, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34a997a, 98 },
    { 0x7fea997b, 62 },
    { 0x1a6a997b, 43 },
    { 0xac5a997c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dea9987, 61 },
    { 0x508a9987, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ada9990, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7a9993, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55a9997, 67 },
    { 0, 0 },
    { 0x8a4a9999, 55 },
    { 0xfe1a999a, 76 },
    { 0x47ea999b, 33 },
    { 0x8bda999b, 23 },
    { 0x8a4a999d, 87 },
    { 0xd7ea999d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fba99a2, 68 },
    { 0xceea99a2, 55 },
    { 0, 0 },
    { 0x9c8a99a5, 55 },
    { 0xf7aa99a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bca99a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691a99b0, 23 },
    { 0x6aea99b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451a99b6, 61 },
    { 0, 0 },
    { 0xa6da99b8, 90 },
    { 0xfa1a99b9, 99 },
    { 0, 0 },
    { 0x7a6a99bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8a99be, 90 },
    { 0xbbea99bf, 90 },
    { 0, 0 },
    { 0xbbba99c1, 61 },
    { 0, 0 },
    { 0x2f9a99c3, 90 },
    { 0, 0 },
    { 0x723a99c5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968a99c9, 76 },
    { 0x6bba99ca, 93 },
    { 0xcfa99cb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206a99cf, 34 },
    { 0, 0 },
    { 0x7d6a99d1, 46 },
    { 0xf05a99d1, 23 },
    { 0x373a99d3, 95 },
    { 0xf6a99d4, 61 },
    { 0xa8ca99d5, 87 },
    { 0x5da99d4, 46 },
    { 0x1d4a99d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ca99dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dca99e2, 88 },
    { 0xd06a99e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedba99ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5a99ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13a99f2, 61 },
    { 0xae5a99f3, 93 },
    { 0xd7da99f2, 93 },
    { 0x524a99f5, 98 },
    { 0x541a99f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bda99fa, 67 },
    { 0, 0 },
    { 0x2e3a99fc, 17 },
    { 0x802a99fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2a9a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aa9a08, 87 },
    { 0, 0 },
    { 0x798a9a0a, 61 },
    { 0, 0 },
    { 0x68ca9a0c, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfa9a0f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0a9a13, 61 },
    { 0, 0 },
    { 0x1e2a9a15, 33 },
    { 0, 0 },
    { 0x49a9a17, 33 },
    { 0xaf4a9a17, 98 },
    { 0, 0 },
    { 0x5cda9a1a, 55 },
    { 0xa60a9a1b, 33 },
    { 0, 0 },
    { 0xa4aa9a1d, 95 },
    { 0x48ba9a1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05a9a21, 90 },
    { 0, 0 },
    { 0x778a9a23, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241a9a27, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aa9a2e, 34 },
    { 0, 0 },
    { 0xbc0a9a30, 43 },
    { 0x198a9a31, 66 },
    { 0x7f9a9a30, 93 },
    { 0, 0 },
    { 0xe7ea9a34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341a9a39, 76 },
    { 0xd6ea9a3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304a9a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17a9a47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2a9a4a, 100 },
    { 0xbf0a9a4b, 61 },
    { 0xe54a9a4c, 76 },
    { 0x65da9a4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65a9a51, 61 },
    { 0, 0 },
    { 0xd0da9a53, 76 },
    { 0, 0 },
    { 0xda5a9a55, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6a9a5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x165a9a5f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62a9a62, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54a9a65, 90 },
    { 0xacaa9a65, 90 },
    { 0x27a9a67, 66 },
    { 0x840a9a65, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462a9a6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21a9a70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fa9a73, 98 },
    { 0xda7a9a74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8a9a77, 97 },
    { 0x853a9a78, 98 },
    { 0x7fea9a77, 61 },
    { 0, 0 },
    { 0xf00a9a7b, 23 },
    { 0xeb3a9a7c, 61 },
    { 0x8a2a9a7d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29a9a81, 101 },
    { 0xfd8a9a82, 61 },
    { 0x182a9a82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eea9a87, 98 },
    { 0x2f8a9a87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ca9a8c, 76 },
    { 0, 0 },
    { 0xd5da9a8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75a9a91, 98 },
    { 0xaeba9a92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddba9a98, 34 },
    { 0x6cea9a99, 40 },
    { 0, 0 },
    { 0xf4ba9a9b, 93 },
    { 0x345a9a9c, 99 },
    { 0x20fa9a9d, 55 },
    { 0xa09a9a9c, 76 },
    { 0, 0 },
    { 0x8fba9aa0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4a9aa3, 9 },
    { 0x9d7a9aa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefa9aa7, 100 },
    { 0x13da9aa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fa9aac, 33 },
    { 0x31ca9aad, 77 },
    { 0x65fa9aae, 55 },
    { 0, 0 },
    { 0xe0ea9ab0, 68 },
    { 0xd3a9ab1, 23 },
    { 0, 0 },
    { 0x922a9ab3, 23 },
    { 0, 0 },
    { 0x9f5a9ab5, 66 },
    { 0x7b2a9ab6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7a9ac0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0a9ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fda9acc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ea9acf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda9ad7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833a9ade, 68 },
    { 0, 0 },
    { 0x8eca9ae0, 33 },
    { 0xb60a9ae0, 76 },
    { 0x8d5a9ae1, 61 },
    { 0xdcda9ae1, 37 },
    { 0, 0 },
    { 0x6c1a9ae5, 90 },
    { 0x778a9ae6, 95 },
    { 0, 0 },
    { 0x5f9a9ae8, 34 },
    { 0x78ba9ae8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7a9aee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06a9af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490a9af9, 55 },
    { 0, 0 },
    { 0xdd9a9afb, 61 },
    { 0xd0da9afc, 33 },
    { 0x973a9afd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63aa9b02, 95 },
    { 0xd51a9b03, 43 },
    { 0x614a9b04, 98 },
    { 0xf8da9b05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ea9b09, 81 },
    { 0xbcda9b09, 78 },
    { 0xd43a9b09, 100 },
    { 0x313a9b09, 40 },
    { 0x46aa9b0c, 95 },
    { 0, 0 },
    { 0xfa1a9b0f, 95 },
    { 0xfa4a9b10, 100 },
    { 0x220a9b0f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea9b14, 98 },
    { 0xca1a9b15, 85 },
    { 0xc03a9b15, 46 },
    { 0x96a9b15, 28 },
    { 0, 0 },
    { 0x7da9b19, 61 },
    { 0, 0 },
    { 0xc71a9b1b, 23 },
    { 0, 0 },
    { 0xd9da9b1d, 95 },
    { 0xcf9a9b1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5a9b23, 69 },
    { 0xc82a9b24, 61 },
    { 0x42ea9b24, 24 },
    { 0x4c1a9b26, 90 },
    { 0x7cda9b27, 43 },
    { 0xa5a9b28, 61 },
    { 0x106a9b29, 76 },
    { 0xa33a9b25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aca9b2e, 98 },
    { 0xb67a9b2e, 23 },
    { 0xe5ea9b30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bea9b33, 78 },
    { 0x673a9b33, 90 },
    { 0, 0 },
    { 0xba7a9b36, 76 },
    { 0xb58a9b36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1a9b3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea9b42, 91 },
    { 0xc47a9b42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303a9b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3da9b4a, 4 },
    { 0, 0 },
    { 0x72ea9b4c, 95 },
    { 0xdcca9b4d, 90 },
    { 0, 0 },
    { 0x4efa9b4f, 95 },
    { 0xfcfa9b50, 33 },
    { 0, 0 },
    { 0xa90a9b52, 43 },
    { 0x3d3a9b53, 23 },
    { 0xb57a9b54, 61 },
    { 0xbb5a9b54, 33 },
    { 0x4bea9b56, 95 },
    { 0, 0 },
    { 0x9b9a9b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546a9b5e, 76 },
    { 0x977a9b5f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91a9b64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fa9b69, 33 },
    { 0x261a9b6a, 46 },
    { 0xac5a9b6b, 55 },
    { 0x921a9b6b, 76 },
    { 0x66a9b6d, 46 },
    { 0xbfea9b6b, 76 },
    { 0x102a9b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4a9b74, 90 },
    { 0, 0 },
    { 0xa7ca9b76, 61 },
    { 0, 0 },
    { 0xdbaa9b78, 93 },
    { 0x2c9a9b79, 93 },
    { 0, 0 },
    { 0x9b4a9b7b, 33 },
    { 0x395a9b7c, 34 },
    { 0xc26a9b7d, 90 },
    { 0x6a7a9b7e, 55 },
    { 0xf01a9b7e, 98 },
    { 0x7e9a9b80, 39 },
    { 0x792a9b7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54a9b8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733a9b90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931a9b95, 85 },
    { 0x34ba9b96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfa9b9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ba9b9e, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x370a9ba1, 95 },
    { 0xb6a9ba2, 23 },
    { 0, 0 },
    { 0xf9ca9ba4, 95 },
    { 0x7c7a9ba4, 95 },
    { 0xe05a9ba6, 78 },
    { 0x790a9ba5, 98 },
    { 0xd31a9ba4, 84 },
    { 0x9b3a9ba9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfca9baf, 98 },
    { 0xd12a9bb0, 88 },
    { 0xadfa9bb0, 55 },
    { 0x63ea9bb2, 99 },
    { 0xc4da9bb3, 61 },
    { 0x885a9bb4, 87 },
    { 0xfbca9bb5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ba9bb9, 43 },
    { 0x541a9bba, 43 },
    { 0xb3da9bbb, 90 },
    { 0xb23a9bb9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaa9bbf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03a9bcd, 23 },
    { 0x186a9bce, 61 },
    { 0xf7a9bce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6a9bd2, 43 },
    { 0xce2a9bd3, 52 },
    { 0x1bfa9bd3, 76 },
    { 0xd2fa9bd4, 85 },
    { 0, 0 },
    { 0xc6aa9bd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8a9bdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x134a9be2, 90 },
    { 0x7ffa9be3, 61 },
    { 0x880a9be4, 90 },
    { 0xa70a9be5, 85 },
    { 0x9afa9be2, 33 },
    { 0x314a9be7, 95 },
    { 0, 0 },
    { 0x4dca9be9, 95 },
    { 0xe69a9bea, 90 },
    { 0x82ea9bea, 33 },
    { 0x2d3a9bec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cda9bf1, 98 },
    { 0, 0 },
    { 0x32da9bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afa9bf6, 95 },
    { 0, 0 },
    { 0xdc3a9bf8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9da9bfd, 33 },
    { 0, 0 },
    { 0x36ea9bff, 55 },
    { 0, 0 },
    { 0x203a9c01, 76 },
    { 0, 0 },
    { 0x4cfa9c03, 46 },
    { 0x867a9c04, 33 },
    { 0xbcaa9c05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfa9c09, 52 },
    { 0, 0 },
    { 0x2f8a9c0b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99a9c11, 46 },
    { 0x2b8a9c12, 83 },
    { 0, 0 },
    { 0x106a9c14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536a9c19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7a9c1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ba9c1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483a9c23, 33 },
    { 0xebda9c23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaa9c2c, 33 },
    { 0x682a9c2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7a9c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25a9c37, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca9c3a, 87 },
    { 0x9ea9c3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2a9c3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x604a9c41, 76 },
    { 0, 0 },
    { 0xa93a9c43, 91 },
    { 0x123a9c44, 90 },
    { 0x248a9c44, 61 },
    { 0x581a9c46, 93 },
    { 0xb9da9c47, 93 },
    { 0x4b5a9c46, 36 },
    { 0x2c0a9c46, 55 },
    { 0x281a9c47, 69 },
    { 0x2a4a9c4b, 100 },
    { 0xa5a9c48, 60 },
    { 0xf70a9c45, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35a9c52, 90 },
    { 0xfcda9c52, 90 },
    { 0, 0 },
    { 0x5d9a9c55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576a9c5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2a9c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ca9c69, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aba9c6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632a9c73, 55 },
    { 0xb69a9c74, 46 },
    { 0x810a9c74, 40 },
    { 0x485a9c76, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x162a9c79, 61 },
    { 0xa6a9c7a, 93 },
    { 0xb04a9c7b, 61 },
    { 0x21aa9c7b, 33 },
    { 0xa5fa9c7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ea9c82, 85 },
    { 0x8daa9c83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1a9c8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2a9c90, 38 },
    { 0xfa0a9c90, 61 },
    { 0, 0 },
    { 0x13fa9c93, 40 },
    { 0, 0 },
    { 0xa31a9c95, 98 },
    { 0, 0 },
    { 0x4fca9c97, 88 },
    { 0x79ea9c97, 90 },
    { 0, 0 },
    { 0xcbda9c9a, 93 },
    { 0x6dda9c9a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa9c9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05a9ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3a9ca8, 90 },
    { 0xecea9ca8, 23 },
    { 0, 0 },
    { 0x780a9cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x47aa9cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x107a9cb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ea9cb4, 98 },
    { 0xd19a9cb4, 95 },
    { 0xfb3a9cb6, 98 },
    { 0x632a9cb7, 33 },
    { 0xb75a9cb6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fa9cbc, 52 },
    { 0, 0 },
    { 0xcfca9cbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2a9cc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fa9ccc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbba9cd1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x854a9cd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ba9cdb, 100 },
    { 0x633a9cdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ca9ce3, 61 },
    { 0, 0 },
    { 0x6ca9ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1a9ce9, 61 },
    { 0xa61a9ce9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531a9cef, 33 },
    { 0xa64a9cef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccca9cf3, 98 },
    { 0, 0 },
    { 0xc56a9cf5, 76 },
    { 0, 0 },
    { 0xeeea9cf7, 90 },
    { 0xc43a9cf8, 100 },
    { 0x636a9cf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9a9cfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641a9d05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfa9d09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7a9d0c, 64 },
    { 0, 0 },
    { 0x50fa9d0e, 90 },
    { 0, 0 },
    { 0x10ba9d10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aa9d18, 33 },
    { 0xe6ca9d18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565a9d1d, 76 },
    { 0x9aa9d1e, 96 },
    { 0xbbda9d1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x922a9d22, 91 },
    { 0x7e4a9d22, 23 },
    { 0x2eba9d23, 33 },
    { 0x503a9d25, 88 },
    { 0xacda9d25, 61 },
    { 0x68ca9d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ea9d2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7a9d2e, 52 },
    { 0x570a9d2f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38a9d32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22da9d35, 61 },
    { 0x602a9d36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728a9d3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584a9d3f, 90 },
    { 0x781a9d3f, 76 },
    { 0, 0 },
    { 0xba4a9d42, 33 },
    { 0xe36a9d42, 90 },
    { 0, 0 },
    { 0x501a9d45, 95 },
    { 0x714a9d46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2da9d4c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36a9d56, 66 },
    { 0, 0 },
    { 0xf29a9d58, 61 },
    { 0xae4a9d59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46a9d5f, 95 },
    { 0xdf6a9d60, 43 },
    { 0x422a9d61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0a9d64, 90 },
    { 0, 0 },
    { 0x264a9d66, 33 },
    { 0xa1da9d67, 66 },
    { 0x51fa9d68, 33 },
    { 0x26ba9d66, 90 },
    { 0x9e8a9d69, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986a9d71, 43 },
    { 0xc9ba9d72, 98 },
    { 0x33aa9d73, 87 },
    { 0x94ea9d72, 5 },
    { 0x38ea9d75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbea9d7c, 67 },
    { 0x27da9d7d, 95 },
    { 0, 0 },
    { 0xa60a9d7f, 23 },
    { 0xf76a9d80, 67 },
    { 0, 0 },
    { 0xc52a9d82, 66 },
    { 0x5c3a9d83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563a9d89, 95 },
    { 0xe9fa9d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ca9d8e, 66 },
    { 0x968a9d8f, 90 },
    { 0xe7ba9d8e, 93 },
    { 0x61ca9d8f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1a9d95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65a9d9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7a9da0, 33 },
    { 0xd1fa9da1, 23 },
    { 0xd7fa9da2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93aa9da8, 55 },
    { 0x359a9da9, 52 },
    { 0x7ea9daa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaca9dad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1a9db2, 76 },
    { 0xef4a9db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ea9db7, 87 },
    { 0xa11a9db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daa9dbc, 23 },
    { 0, 0 },
    { 0xca6a9dbe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x714a9dc1, 96 },
    { 0x9a6a9dc2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09a9dc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0a9dd2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729a9dd6, 90 },
    { 0x405a9dd7, 87 },
    { 0, 0 },
    { 0x87ca9dd9, 56 },
    { 0xe47a9dd9, 33 },
    { 0xb02a9dda, 95 },
    { 0, 0 },
    { 0xa20a9ddd, 98 },
    { 0x5c2a9dde, 33 },
    { 0x11ba9ddf, 95 },
    { 0, 0 },
    { 0x12ea9de1, 33 },
    { 0xf85a9de1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187a9de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda9dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2a9def, 48 },
    { 0xe36a9def, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40a9df4, 90 },
    { 0x88ca9df5, 23 },
    { 0x27ba9df6, 90 },
    { 0x7a9a9df7, 87 },
    { 0, 0 },
    { 0x522a9df9, 95 },
    { 0, 0 },
    { 0x36ca9dfb, 55 },
    { 0x29a9dfc, 91 },
    { 0x242a9dfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48a9e01, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217a9e06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aa9e0d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373a9e12, 98 },
    { 0xa19a9e12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910a9e17, 36 },
    { 0x51a9e17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1a9e1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bba9e25, 61 },
    { 0x17fa9e26, 98 },
    { 0xaf5a9e25, 43 },
    { 0, 0 },
    { 0x699a9e29, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249a9e2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ca9e30, 52 },
    { 0xf2aa9e31, 23 },
    { 0xaeba9e31, 76 },
    { 0, 0 },
    { 0xe9fa9e34, 33 },
    { 0, 0 },
    { 0x52ca9e36, 98 },
    { 0x9eba9e36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x800a9e3a, 23 },
    { 0xf10a9e3b, 90 },
    { 0xee2a9e3c, 93 },
    { 0x122a9e3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aca9e40, 33 },
    { 0, 0 },
    { 0x313a9e42, 95 },
    { 0x6f3a9e43, 34 },
    { 0xdcfa9e43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eca9e48, 87 },
    { 0x5c0a9e48, 66 },
    { 0, 0 },
    { 0xeefa9e4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224a9e52, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859a9e57, 23 },
    { 0x5ada9e58, 75 },
    { 0, 0 },
    { 0xbf0a9e5a, 49 },
    { 0x48ba9e5b, 43 },
    { 0x8eca9e5b, 90 },
    { 0xc2ba9e5c, 36 },
    { 0xf33a9e5a, 75 },
    { 0xad3a9e5e, 23 },
    { 0x432a9e5d, 95 },
    { 0, 0 },
    { 0xa2fa9e62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bda9e65, 76 },
    { 0x4c7a9e66, 78 },
    { 0, 0 },
    { 0x87da9e68, 85 },
    { 0xa08a9e68, 90 },
    { 0, 0 },
    { 0xec6a9e6b, 34 },
    { 0, 0 },
    { 0x187a9e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29a9e70, 77 },
    { 0x45da9e71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ba9e74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8a9e78, 90 },
    { 0x7d5a9e79, 83 },
    { 0x639a9e78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26a9e7f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740a9e84, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949a9e88, 80 },
    { 0x218a9e89, 23 },
    { 0xd05a9e88, 17 },
    { 0xd1fa9e8b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22a9e8e, 61 },
    { 0x8d9a9e8f, 97 },
    { 0x53fa9e90, 93 },
    { 0x3aca9e91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x909a9e94, 88 },
    { 0x36aa9e95, 33 },
    { 0xeb0a9e96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ea9e9a, 43 },
    { 0, 0 },
    { 0xf22a9e9c, 34 },
    { 0, 0 },
    { 0x7ffa9e9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ba9ea3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458a9eb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282a9eb4, 98 },
    { 0, 0 },
    { 0xab8a9eb6, 61 },
    { 0xb80a9eb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804a9ebd, 34 },
    { 0xd45a9ebd, 33 },
    { 0x963a9ebf, 61 },
    { 0x694a9ec0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95a9ec3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8a9ec6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0a9ecc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87da9ecf, 61 },
    { 0x9e8a9ed0, 98 },
    { 0xb82a9ed1, 33 },
    { 0, 0 },
    { 0xdf6a9ed3, 34 },
    { 0x2fa9ed3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5a9ed7, 98 },
    { 0xa8ba9ed8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3a9edd, 33 },
    { 0xa2da9edd, 67 },
    { 0, 0 },
    { 0x27da9ee0, 43 },
    { 0xa57a9ee1, 33 },
    { 0xc5aa9ee1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x593a9ee5, 99 },
    { 0xf75a9ee6, 90 },
    { 0x369a9ee7, 43 },
    { 0, 0 },
    { 0xfba9ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6a9ef0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cca9ef9, 17 },
    { 0xeb4a9ef9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6a9eff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122a9f04, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ea9f0c, 67 },
    { 0x509a9f0d, 90 },
    { 0, 0 },
    { 0x495a9f0f, 52 },
    { 0x20ca9f10, 61 },
    { 0, 0 },
    { 0xcb6a9f12, 61 },
    { 0x42a9f13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ba9f16, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x854a9f19, 23 },
    { 0, 0 },
    { 0xb76a9f1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49a9f1f, 46 },
    { 0, 0 },
    { 0x2bea9f21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35a9f2c, 66 },
    { 0x7c3a9f2d, 90 },
    { 0x5f9a9f2e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8a9f3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cea9f41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ea9f44, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1a9f47, 78 },
    { 0xa36a9f48, 90 },
    { 0, 0 },
    { 0xdbea9f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15a9f4f, 61 },
    { 0, 0 },
    { 0x859a9f51, 90 },
    { 0, 0 },
    { 0x255a9f53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fa9f5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784a9f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36a9f62, 85 },
    { 0x376a9f63, 55 },
    { 0xda9a9f64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x320a9f67, 90 },
    { 0, 0 },
    { 0xa67a9f69, 87 },
    { 0, 0 },
    { 0x1e6a9f6b, 75 },
    { 0xc99a9f6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36a9f70, 23 },
    { 0x78aa9f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaa9f74, 43 },
    { 0xd87a9f75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x634a9f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x764a9f7b, 52 },
    { 0x811a9f7c, 91 },
    { 0x447a9f7d, 98 },
    { 0x29ba9f7d, 95 },
    { 0, 0 },
    { 0xc96a9f80, 23 },
    { 0, 0 },
    { 0x806a9f82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x316a9f85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195a9f8f, 78 },
    { 0x3d7a9f8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96a9f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3a9fa1, 95 },
    { 0xe5a9fa2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ea9fa5, 90 },
    { 0x32fa9fa6, 76 },
    { 0, 0 },
    { 0x7aaa9fa8, 33 },
    { 0xfcfa9fa9, 95 },
    { 0x804a9fa9, 66 },
    { 0x693a9fab, 87 },
    { 0xc20a9fab, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394a9fb1, 33 },
    { 0, 0 },
    { 0x6bca9fb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6a9fb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x326a9fba, 61 },
    { 0x241a9fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2a9fc4, 65 },
    { 0, 0 },
    { 0xa3a9fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aba9fc9, 23 },
    { 0, 0 },
    { 0x3a8a9fcb, 95 },
    { 0, 0 },
    { 0x8dca9fcd, 33 },
    { 0x754a9fcd, 76 },
    { 0, 0 },
    { 0x7b6a9fd0, 68 },
    { 0xb49a9fd0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22a9fd4, 88 },
    { 0x147a9fd4, 34 },
    { 0xeb3a9fd6, 33 },
    { 0x950a9fd7, 43 },
    { 0x196a9fd8, 43 },
    { 0x37aa9fd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69a9fdd, 23 },
    { 0, 0 },
    { 0x3bea9fdf, 55 },
    { 0xbf9a9fe0, 90 },
    { 0x2d9a9fdf, 90 },
    { 0x245a9fe2, 33 },
    { 0x57ea9fe2, 23 },
    { 0x691a9fe2, 23 },
    { 0x5a3a9fe5, 61 },
    { 0x9f9a9fe6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8a9fec, 90 },
    { 0x9d3a9fed, 91 },
    { 0, 0 },
    { 0x6baa9fef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728a9ff8, 61 },
    { 0, 0 },
    { 0x5c3a9ffa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8a9fff, 66 },
    { 0x974aa000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0aa009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6faa00c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6aa010, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33aa013, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79aa016, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3aa022, 23 },
    { 0, 0 },
    { 0xb3aaa024, 99 },
    { 0x736aa025, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8aa029, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62faa02e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572aa032, 61 },
    { 0xe19aa032, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfaa036, 23 },
    { 0, 0 },
    { 0xc45aa038, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecaa03b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aaa040, 87 },
    { 0x6bbaa041, 17 },
    { 0, 0 },
    { 0x335aa043, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x344aa046, 68 },
    { 0x74eaa047, 98 },
    { 0xcbaaa047, 76 },
    { 0xbfbaa046, 95 },
    { 0xd49aa047, 95 },
    { 0x52eaa04b, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaa04e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935aa053, 34 },
    { 0xdb3aa054, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3aa059, 95 },
    { 0xaefaa05a, 33 },
    { 0xa4baa059, 100 },
    { 0, 0 },
    { 0xfefaa05d, 61 },
    { 0xa1aa05e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deaa065, 66 },
    { 0x813aa065, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2aa06a, 61 },
    { 0, 0 },
    { 0xeeeaa06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeaa07e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0aa081, 95 },
    { 0, 0 },
    { 0x7eaaa083, 46 },
    { 0x703aa084, 61 },
    { 0x1fdaa085, 61 },
    { 0x6dfaa086, 46 },
    { 0, 0 },
    { 0xd42aa088, 81 },
    { 0, 0 },
    { 0x3bfaa08a, 95 },
    { 0xabbaa08a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538aa08f, 99 },
    { 0, 0 },
    { 0x44faa091, 61 },
    { 0xccaa091, 67 },
    { 0x3eaa091, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3aa096, 61 },
    { 0xfe7aa096, 90 },
    { 0xd35aa097, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4aa09b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471aa0a4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffaa0a8, 66 },
    { 0, 0 },
    { 0x90aaa0aa, 95 },
    { 0, 0 },
    { 0xa3eaa0ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdaa0b1, 97 },
    { 0, 0 },
    { 0xa11aa0b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aaa0b6, 76 },
    { 0, 0 },
    { 0x136aa0b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaaa0bb, 91 },
    { 0, 0 },
    { 0xd2eaa0bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4aa0c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40aa0c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04aa0cd, 76 },
    { 0, 0 },
    { 0x340aa0cf, 90 },
    { 0x10eaa0d0, 23 },
    { 0xe2eaa0d1, 61 },
    { 0xe27aa0d0, 76 },
    { 0x8a2aa0d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5aa0d6, 95 },
    { 0x7c0aa0d7, 33 },
    { 0x9e2aa0d6, 78 },
    { 0, 0 },
    { 0xd0aa0da, 61 },
    { 0x7e4aa0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x453aa0de, 93 },
    { 0, 0 },
    { 0x68eaa0e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660aa0e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8aa0ee, 85 },
    { 0x488aa0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49daa0f4, 90 },
    { 0xaa2aa0f5, 95 },
    { 0xee4aa0f5, 87 },
    { 0xdeaa0f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4feaa0fb, 90 },
    { 0, 0 },
    { 0x820aa0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85aa105, 23 },
    { 0x5c5aa106, 77 },
    { 0x9fbaa107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16eaa10c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2aa114, 43 },
    { 0xca7aa115, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfaa11a, 17 },
    { 0, 0 },
    { 0x8f1aa11c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25aa122, 76 },
    { 0xba0aa122, 95 },
    { 0x3d1aa124, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefaa12e, 90 },
    { 0x390aa12f, 65 },
    { 0xd6baa130, 33 },
    { 0x5f2aa12e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40faa139, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x981aa13c, 95 },
    { 0x23daa13d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296aa144, 52 },
    { 0x2d9aa145, 90 },
    { 0xcefaa145, 91 },
    { 0, 0 },
    { 0x2b2aa148, 100 },
    { 0x9e4aa149, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1aa14e, 95 },
    { 0, 0 },
    { 0x4b9aa150, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41aa154, 81 },
    { 0x267aa155, 33 },
    { 0xe55aa156, 99 },
    { 0x520aa157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311aa15b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2aa15e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06aa162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97aa165, 33 },
    { 0, 0 },
    { 0xe4aa167, 68 },
    { 0x83aa168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64faa16f, 90 },
    { 0x5ebaa170, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551aa17a, 93 },
    { 0xc08aa17a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85daa180, 90 },
    { 0x62faa180, 98 },
    { 0x11eaa182, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0aa189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x786aa18c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6aa190, 23 },
    { 0, 0 },
    { 0xef7aa192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399aa198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaaa19d, 76 },
    { 0x23aaa19e, 99 },
    { 0xc57aa19e, 100 },
    { 0xef8aa1a0, 90 },
    { 0, 0 },
    { 0x74baa1a2, 90 },
    { 0xcebaa1a3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfaa1a9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aaa1ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x815aa1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eaa1b5, 86 },
    { 0x72aaa1b6, 33 },
    { 0x9aaa1b6, 99 },
    { 0, 0 },
    { 0x282aa1b9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374aa1bd, 55 },
    { 0, 0 },
    { 0x188aa1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792aa1c3, 33 },
    { 0xbb1aa1c4, 95 },
    { 0x743aa1c5, 52 },
    { 0xf85aa1c6, 61 },
    { 0, 0 },
    { 0xf00aa1c8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189aa1cd, 55 },
    { 0x2a3aa1ce, 84 },
    { 0x1c9aa1cf, 91 },
    { 0xf03aa1d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08aa1d4, 23 },
    { 0, 0 },
    { 0x35eaa1d6, 46 },
    { 0x52daa1d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4aa1dc, 93 },
    { 0xc7daa1dc, 61 },
    { 0, 0 },
    { 0x793aa1df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196aa1e3, 33 },
    { 0x2deaa1e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2aa1ec, 100 },
    { 0, 0 },
    { 0x917aa1ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14eaa1f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19aa1fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35aa200, 76 },
    { 0x85aaa201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1aa204, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9aa208, 95 },
    { 0xb46aa209, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70eaa20d, 33 },
    { 0x8abaa20d, 99 },
    { 0, 0 },
    { 0x7e1aa210, 55 },
    { 0x9a2aa211, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26daa219, 78 },
    { 0x294aa219, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfaa21f, 65 },
    { 0xb27aa220, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceaa224, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14baa22b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64aa241, 61 },
    { 0x667aa241, 93 },
    { 0x532aa241, 23 },
    { 0xe63aa243, 33 },
    { 0xe4faa243, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffaa248, 91 },
    { 0xa23aa248, 33 },
    { 0x3f3aa248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9aa24d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baaa250, 33 },
    { 0, 0 },
    { 0xfd9aa252, 33 },
    { 0x9adaa253, 55 },
    { 0, 0 },
    { 0x44caa255, 90 },
    { 0x76aa256, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46aa25e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15daa264, 90 },
    { 0xf5aaa265, 33 },
    { 0x65daa266, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94baa269, 34 },
    { 0xffdaa26a, 46 },
    { 0, 0 },
    { 0x69caa26c, 36 },
    { 0, 0 },
    { 0xd2faa26e, 98 },
    { 0xecaa26f, 90 },
    { 0xd72aa270, 76 },
    { 0x83caa271, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x951aa274, 61 },
    { 0x847aa275, 17 },
    { 0x73aa275, 61 },
    { 0, 0 },
    { 0x83eaa278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7aa27e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504aa285, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x838aa288, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270aa28f, 23 },
    { 0, 0 },
    { 0xd37aa291, 33 },
    { 0x29aa292, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afaa297, 83 },
    { 0x3f6aa297, 69 },
    { 0xbacaa299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67aa29c, 90 },
    { 0xe14aa29c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151aa2a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37aa2a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12daa2aa, 43 },
    { 0x793aa2ab, 95 },
    { 0x979aa2ac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x33daa2af, 33 },
    { 0xca8aa2af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdaa2b3, 4 },
    { 0x339aa2b3, 91 },
    { 0xe78aa2b5, 84 },
    { 0xeb0aa2b6, 98 },
    { 0xf00aa2b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4aa2ba, 90 },
    { 0x58daa2ba, 61 },
    { 0x85caa2bc, 68 },
    { 0xe00aa2bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61faa2c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6aa2c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29faa2cc, 66 },
    { 0, 0 },
    { 0x5a3aa2ce, 33 },
    { 0x950aa2ce, 100 },
    { 0x443aa2d0, 33 },
    { 0, 0 },
    { 0xebdaa2d2, 95 },
    { 0, 0 },
    { 0xacaaa2d4, 33 },
    { 0xb38aa2d4, 34 },
    { 0x87aa2d6, 95 },
    { 0x74daa2d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaaa2de, 46 },
    { 0x281aa2df, 33 },
    { 0, 0 },
    { 0xf0daa2e1, 67 },
    { 0, 0 },
    { 0x9a6aa2e3, 23 },
    { 0x25aa2e3, 61 },
    { 0, 0 },
    { 0x13aaa2e6, 98 },
    { 0xa3eaa2e7, 69 },
    { 0x31caa2e8, 95 },
    { 0, 0 },
    { 0x6f8aa2ea, 87 },
    { 0xe2aa2eb, 23 },
    { 0x28daa2eb, 43 },
    { 0, 0 },
    { 0x5e7aa2ee, 93 },
    { 0x7a7aa2ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3aa2f4, 55 },
    { 0, 0 },
    { 0x968aa2f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27aa2ff, 98 },
    { 0x43eaa300, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x726aa303, 43 },
    { 0, 0 },
    { 0x51faa305, 23 },
    { 0, 0 },
    { 0x9d3aa307, 61 },
    { 0, 0 },
    { 0xce9aa309, 34 },
    { 0, 0 },
    { 0x6a8aa30b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8aa310, 61 },
    { 0x6fcaa311, 67 },
    { 0x2d5aa312, 76 },
    { 0x909aa313, 97 },
    { 0x4ecaa313, 23 },
    { 0x43baa315, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83aa318, 93 },
    { 0xd1eaa319, 95 },
    { 0xebcaa319, 34 },
    { 0x842aa31b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacaa321, 61 },
    { 0, 0 },
    { 0x111aa323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eaa328, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eaa32d, 90 },
    { 0, 0 },
    { 0x663aa32f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adaa334, 33 },
    { 0x858aa335, 78 },
    { 0x4d9aa335, 61 },
    { 0, 0 },
    { 0xcd1aa338, 98 },
    { 0x855aa339, 23 },
    { 0xed8aa339, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20baa341, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67faa345, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1aa349, 46 },
    { 0xe65aa34a, 90 },
    { 0x907aa34b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4baa34e, 34 },
    { 0x142aa34f, 98 },
    { 0x516aa34f, 83 },
    { 0x473aa351, 90 },
    { 0xcdbaa352, 23 },
    { 0, 0 },
    { 0x204aa354, 61 },
    { 0x596aa355, 61 },
    { 0, 0 },
    { 0x88eaa357, 75 },
    { 0, 0 },
    { 0x483aa359, 98 },
    { 0, 0 },
    { 0x719aa35b, 90 },
    { 0x29caa35c, 33 },
    { 0xf70aa35d, 93 },
    { 0x7d2aa35c, 95 },
    { 0xec6aa35b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5aa364, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16caa36c, 93 },
    { 0x936aa36d, 90 },
    { 0x7bcaa36e, 66 },
    { 0, 0 },
    { 0x306aa370, 90 },
    { 0, 0 },
    { 0xd81aa372, 33 },
    { 0x903aa373, 98 },
    { 0xbe1aa373, 61 },
    { 0xd20aa372, 90 },
    { 0x6a7aa376, 33 },
    { 0x9daaa376, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7aa37a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6aa37e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1aa381, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180aa385, 43 },
    { 0x9ceaa385, 33 },
    { 0x770aa387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07aa38b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2aa38f, 78 },
    { 0x37daa390, 69 },
    { 0xf18aa391, 33 },
    { 0x724aa392, 23 },
    { 0, 0 },
    { 0x1aaa394, 85 },
    { 0, 0 },
    { 0xaccaa396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51caa3a0, 90 },
    { 0xd8aaa3a1, 23 },
    { 0xdc4aa3a2, 76 },
    { 0x606aa3a0, 75 },
    { 0x217aa3a4, 90 },
    { 0, 0 },
    { 0x168aa3a6, 33 },
    { 0x65caa3a6, 95 },
    { 0, 0 },
    { 0xb5aa3a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948aa3b0, 93 },
    { 0, 0 },
    { 0x991aa3b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x864aa3b5, 90 },
    { 0, 0 },
    { 0x596aa3b7, 100 },
    { 0x11aa3b8, 75 },
    { 0, 0 },
    { 0x276aa3ba, 76 },
    { 0xcb9aa3bb, 23 },
    { 0xfd5aa3bc, 23 },
    { 0x60daa3bd, 77 },
    { 0x6bbaa3be, 90 },
    { 0, 0 },
    { 0x2b3aa3c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x105aa3c3, 43 },
    { 0xe11aa3c3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddaa3c8, 23 },
    { 0xd1daa3c8, 17 },
    { 0xd7daa3ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0aa3ce, 33 },
    { 0x174aa3ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888aa3d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6aa3d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752aa3e2, 95 },
    { 0, 0 },
    { 0xc60aa3e4, 33 },
    { 0x74daa3e5, 55 },
    { 0, 0 },
    { 0x437aa3e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2aa3ea, 76 },
    { 0xf49aa3eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbaa3ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955aa3f7, 23 },
    { 0x963aa3f7, 34 },
    { 0xc4baa3f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621aa400, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eaa403, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195aa40b, 77 },
    { 0x943aa40c, 23 },
    { 0xef7aa40c, 23 },
    { 0, 0 },
    { 0x309aa40f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200aa413, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0aa41b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3aa41f, 55 },
    { 0, 0 },
    { 0xd73aa421, 95 },
    { 0, 0 },
    { 0x4c0aa423, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954aa429, 76 },
    { 0x2faa429, 90 },
    { 0, 0 },
    { 0x7e8aa42c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9aa433, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339aa437, 61 },
    { 0xddeaa438, 80 },
    { 0x9a9aa438, 46 },
    { 0xe8faa43a, 46 },
    { 0xc18aa437, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0aa43f, 46 },
    { 0xcd9aa43f, 33 },
    { 0xb1daa441, 38 },
    { 0, 0 },
    { 0x538aa443, 93 },
    { 0x7beaa443, 95 },
    { 0xd2baa443, 55 },
    { 0, 0 },
    { 0x1c0aa447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0aa44e, 90 },
    { 0x8cfaa44f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x818aa452, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x423aa455, 43 },
    { 0xe7faa456, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602aa45a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9aa45f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdaa463, 33 },
    { 0x111aa464, 95 },
    { 0x462aa465, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96aa469, 98 },
    { 0, 0 },
    { 0x663aa46b, 46 },
    { 0x5d5aa46c, 95 },
    { 0x6fcaa46c, 80 },
    { 0xf15aa46b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7aa471, 65 },
    { 0x35faa472, 95 },
    { 0, 0 },
    { 0x4c8aa474, 84 },
    { 0, 0 },
    { 0x771aa476, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6daa486, 66 },
    { 0xe20aa487, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faa48f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832aa498, 55 },
    { 0x375aa498, 89 },
    { 0, 0 },
    { 0x2f7aa49b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaaa4a4, 98 },
    { 0xbb6aa4a5, 90 },
    { 0x7a9aa4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250aa4aa, 76 },
    { 0xea7aa4ab, 95 },
    { 0xa9baa4ab, 33 },
    { 0xff4aa4ad, 23 },
    { 0xef4aa4ae, 90 },
    { 0xc69aa4af, 90 },
    { 0xef2aa4ae, 55 },
    { 0, 0 },
    { 0x360aa4b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x77baa4b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34aa4bc, 90 },
    { 0xf06aa4bc, 84 },
    { 0x6acaa4be, 90 },
    { 0x23daa4bf, 61 },
    { 0, 0 },
    { 0xc3eaa4c1, 61 },
    { 0x2a5aa4c2, 76 },
    { 0, 0 },
    { 0xc51aa4c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21aa4c9, 55 },
    { 0x6d9aa4ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69aaa4d0, 85 },
    { 0, 0 },
    { 0x969aa4d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5aa4d6, 90 },
    { 0x5ddaa4d7, 34 },
    { 0x77faa4d6, 33 },
    { 0x429aa4d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22aa4dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4baa4df, 23 },
    { 0x5f0aa4df, 90 },
    { 0, 0 },
    { 0x2d3aa4e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5aa4e9, 23 },
    { 0xc2faa4ea, 99 },
    { 0, 0 },
    { 0x4f0aa4ec, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42aa4f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdaa4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860aa4f9, 33 },
    { 0xffaa4fa, 33 },
    { 0x55aaa4fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26baa4ff, 43 },
    { 0, 0 },
    { 0x3f2aa501, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23aaa505, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eaa508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9aa50d, 90 },
    { 0, 0 },
    { 0xcabaa50f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeceaa51b, 78 },
    { 0x62aa51b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daaa520, 55 },
    { 0x283aa521, 52 },
    { 0, 0 },
    { 0x3c1aa523, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdaa52c, 76 },
    { 0, 0 },
    { 0x63daa52e, 61 },
    { 0x479aa52f, 33 },
    { 0x81aaa52f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aa534, 98 },
    { 0x64aaa535, 36 },
    { 0x257aa535, 78 },
    { 0xbf1aa537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x575aa53a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4aa53e, 90 },
    { 0x55caa53f, 66 },
    { 0xba7aa53e, 76 },
    { 0x169aa541, 98 },
    { 0xab0aa541, 23 },
    { 0xbd1aa543, 46 },
    { 0x378aa541, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3aa54c, 93 },
    { 0xfe2aa54c, 66 },
    { 0xa08aa54e, 33 },
    { 0x28aa54f, 98 },
    { 0xe8eaa54c, 55 },
    { 0, 0 },
    { 0xb46aa552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aaaa557, 17 },
    { 0x52caa557, 87 },
    { 0x5caaa559, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aa55c, 61 },
    { 0x644aa55d, 52 },
    { 0xb84aa55c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55baa563, 100 },
    { 0xddaaa563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7baa567, 68 },
    { 0xf8eaa568, 9 },
    { 0x4f4aa569, 46 },
    { 0x25aaa568, 95 },
    { 0x3e7aa56b, 52 },
    { 0x717aa56c, 76 },
    { 0, 0 },
    { 0xe76aa56e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06aa571, 90 },
    { 0, 0 },
    { 0x5afaa573, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25aa577, 90 },
    { 0, 0 },
    { 0xd47aa579, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x24caa57c, 55 },
    { 0x374aa57d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2aa580, 90 },
    { 0x2bfaa581, 33 },
    { 0xe99aa582, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebaa58a, 33 },
    { 0x470aa58b, 78 },
    { 0x193aa58c, 33 },
    { 0xccfaa58a, 61 },
    { 0xcd8aa58e, 98 },
    { 0x7b0aa58f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2daa594, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21aa598, 95 },
    { 0x311aa599, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x697aa59c, 95 },
    { 0x270aa59c, 98 },
    { 0xd41aa59d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32daa5a1, 33 },
    { 0, 0 },
    { 0x9eaa5a3, 78 },
    { 0x441aa5a4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x24aaa5a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x841aa5aa, 55 },
    { 0, 0 },
    { 0xe09aa5ac, 61 },
    { 0, 0 },
    { 0x984aa5ae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9aa5b1, 90 },
    { 0x990aa5b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x611aa5b5, 91 },
    { 0xf70aa5b6, 23 },
    { 0x186aa5b5, 34 },
    { 0x30eaa5b8, 55 },
    { 0x170aa5b7, 84 },
    { 0xdd8aa5b8, 23 },
    { 0xfbbaa5ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aaa5be, 95 },
    { 0x643aa5bf, 61 },
    { 0xdc0aa5c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6aa5c4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edaa5c9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93aa5cc, 33 },
    { 0, 0 },
    { 0x6eeaa5ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deaa5d2, 93 },
    { 0x3bcaa5d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efaa5dc, 90 },
    { 0x90caa5dd, 90 },
    { 0x412aa5dd, 90 },
    { 0x9dcaa5df, 87 },
    { 0x48aaa5dd, 33 },
    { 0, 0 },
    { 0xf7eaa5e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3aa5e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4aa5eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05aa5f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4aa5f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3aa602, 95 },
    { 0xb27aa603, 55 },
    { 0xdb8aa604, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aa607, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73aa60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574aa610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aa614, 55 },
    { 0xbc5aa615, 90 },
    { 0xf34aa616, 33 },
    { 0x35aa617, 90 },
    { 0x567aa618, 33 },
    { 0, 0 },
    { 0x77aa61a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aa61e, 33 },
    { 0, 0 },
    { 0x84aa620, 55 },
    { 0xde5aa621, 43 },
    { 0x2e4aa620, 95 },
    { 0x870aa623, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aaa626, 33 },
    { 0xd41aa627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8baa62c, 98 },
    { 0x7d3aa62d, 34 },
    { 0x3c4aa62d, 51 },
    { 0xa45aa62f, 33 },
    { 0xae9aa630, 33 },
    { 0x2deaa631, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8aa639, 93 },
    { 0x6aeaa639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6aa63d, 61 },
    { 0x277aa63e, 90 },
    { 0xa81aa63f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96caa643, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182aa64a, 68 },
    { 0x421aa64b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673aa654, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43faa657, 23 },
    { 0, 0 },
    { 0xe08aa659, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x204aa65c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832aa663, 80 },
    { 0x62eaa664, 33 },
    { 0, 0 },
    { 0xaf4aa666, 95 },
    { 0x9c8aa666, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfaa66b, 98 },
    { 0x70eaa66c, 91 },
    { 0xc1baa66b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90aa670, 95 },
    { 0, 0 },
    { 0x53eaa672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaaa675, 90 },
    { 0xffbaa676, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14aa67a, 52 },
    { 0x39eaa67b, 33 },
    { 0x6d0aa67c, 98 },
    { 0, 0 },
    { 0xe68aa67e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x502aa681, 52 },
    { 0, 0 },
    { 0xf6daa683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859aa687, 33 },
    { 0x317aa687, 56 },
    { 0, 0 },
    { 0x18aaa68a, 95 },
    { 0, 0 },
    { 0x91aaa68c, 68 },
    { 0x34aaa68c, 76 },
    { 0x212aa68e, 68 },
    { 0xfb6aa68f, 90 },
    { 0x7f4aa690, 90 },
    { 0x92aa691, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121aa695, 95 },
    { 0xdcaa695, 90 },
    { 0x3b2aa697, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5aa69a, 87 },
    { 0x93aa69a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0aa6ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6aa6b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daaa6b6, 23 },
    { 0xd67aa6b7, 66 },
    { 0xb58aa6b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beaa6bc, 33 },
    { 0, 0 },
    { 0xd97aa6be, 55 },
    { 0xa6aa6be, 11 },
    { 0x23aaa6c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x741aa6c3, 61 },
    { 0x249aa6c4, 33 },
    { 0, 0 },
    { 0xeaeaa6c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0aa6cf, 77 },
    { 0x9e0aa6cf, 55 },
    { 0x46aaa6d1, 46 },
    { 0xcf9aa6cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7baa6d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17baa6e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535aa6e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aaa6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73aa6f4, 78 },
    { 0x778aa6f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13faa6f8, 23 },
    { 0x81faa6f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9aa6fc, 99 },
    { 0xd4caa6fc, 38 },
    { 0, 0 },
    { 0x9b1aa6ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12aa704, 23 },
    { 0x667aa705, 34 },
    { 0, 0 },
    { 0x2deaa707, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579aa70b, 34 },
    { 0x8daaa70b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0aa70f, 91 },
    { 0, 0 },
    { 0x136aa711, 98 },
    { 0, 0 },
    { 0x7c6aa713, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50faa719, 56 },
    { 0xbfaa71a, 80 },
    { 0x4e8aa71b, 95 },
    { 0xd9caa719, 34 },
    { 0x274aa71d, 38 },
    { 0x8daa71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13aa721, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x472aa724, 40 },
    { 0x4c6aa725, 90 },
    { 0x9ccaa726, 90 },
    { 0x8cbaa727, 100 },
    { 0x97daa726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7aa72c, 96 },
    { 0xf30aa72d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1aa730, 34 },
    { 0x456aa730, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99aa735, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5aa739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307aa73d, 33 },
    { 0xbbcaa73e, 98 },
    { 0x1c5aa73f, 65 },
    { 0x896aa740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73aa743, 52 },
    { 0x95baa743, 95 },
    { 0, 0 },
    { 0x5abaa746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x331aa749, 90 },
    { 0x79eaa74a, 66 },
    { 0x16baa74b, 90 },
    { 0xcffaa74c, 90 },
    { 0, 0 },
    { 0xedfaa74e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03aa756, 33 },
    { 0xe6faa757, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25caa75c, 23 },
    { 0xadaaa75d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1aa760, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71aa763, 23 },
    { 0x8eaa764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9aa769, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaa76d, 61 },
    { 0x60aa76d, 95 },
    { 0x37aaa76f, 33 },
    { 0x75aaa770, 95 },
    { 0x6b0aa771, 43 },
    { 0x93faa770, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2aa77a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceaa77d, 98 },
    { 0x35eaa77e, 55 },
    { 0, 0 },
    { 0xfb9aa780, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0daa786, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2aa789, 33 },
    { 0xf20aa789, 95 },
    { 0, 0 },
    { 0x42eaa78c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x340aa78f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deaa793, 33 },
    { 0xae6aa793, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9aa799, 99 },
    { 0x215aa79a, 88 },
    { 0x4dcaa79a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adaa79f, 90 },
    { 0, 0 },
    { 0x8d2aa7a1, 52 },
    { 0, 0 },
    { 0x448aa7a3, 33 },
    { 0xff4aa7a4, 78 },
    { 0x1e6aa7a4, 33 },
    { 0, 0 },
    { 0xc88aa7a7, 90 },
    { 0x849aa7a8, 78 },
    { 0x4a1aa7a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfaa7ac, 98 },
    { 0xe62aa7ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65daa7b5, 96 },
    { 0, 0 },
    { 0x573aa7b7, 33 },
    { 0xd3aaa7b8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aa7be, 23 },
    { 0, 0 },
    { 0x824aa7c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x200aa7c3, 33 },
    { 0x348aa7c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4daa7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82aa7cc, 17 },
    { 0x601aa7cc, 90 },
    { 0, 0 },
    { 0x7b0aa7cf, 76 },
    { 0, 0 },
    { 0xa4caa7d1, 76 },
    { 0x1a7aa7d2, 23 },
    { 0x9acaa7d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29aa7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104aa7dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8aa7df, 95 },
    { 0, 0 },
    { 0xcc3aa7e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0aa7e4, 90 },
    { 0x761aa7e4, 43 },
    { 0x5aaaa7e5, 23 },
    { 0x1c3aa7e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2aa7eb, 55 },
    { 0, 0 },
    { 0xde9aa7ed, 55 },
    { 0x2fcaa7ee, 98 },
    { 0, 0 },
    { 0x8f5aa7f0, 66 },
    { 0xe0aa7f0, 98 },
    { 0, 0 },
    { 0x597aa7f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaa7f6, 90 },
    { 0x2f7aa7f7, 84 },
    { 0x17faa7f8, 93 },
    { 0xe3faa7f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41aa801, 76 },
    { 0xa1caa802, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x130aa805, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8aa80b, 82 },
    { 0x734aa80c, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefaa811, 91 },
    { 0, 0 },
    { 0x248aa813, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12aa816, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eaa81a, 78 },
    { 0x285aa81a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50aa81f, 95 },
    { 0x6f9aa820, 61 },
    { 0xacdaa821, 90 },
    { 0xd33aa822, 81 },
    { 0xa5baa821, 17 },
    { 0, 0 },
    { 0xfc3aa825, 17 },
    { 0xc0aaa826, 33 },
    { 0x32aaa827, 90 },
    { 0, 0 },
    { 0x487aa829, 76 },
    { 0, 0 },
    { 0x40caa82b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2aa832, 33 },
    { 0, 0 },
    { 0x65daa834, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4aa83c, 85 },
    { 0xac5aa83d, 68 },
    { 0x2b5aa83e, 61 },
    { 0, 0 },
    { 0xc8aa840, 84 },
    { 0x957aa840, 43 },
    { 0xdaeaa840, 76 },
    { 0x9d7aa843, 100 },
    { 0xe8faa841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x62daa847, 98 },
    { 0, 0 },
    { 0x7a8aa849, 90 },
    { 0x8f6aa84a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09aa84d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629aa856, 95 },
    { 0xe48aa857, 68 },
    { 0x63aaa857, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x348aa85b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95aa85e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54aa863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x307aa866, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x708aa869, 90 },
    { 0xc61aa86a, 83 },
    { 0xddbaa86b, 95 },
    { 0x756aa86b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecaa86f, 23 },
    { 0xcfdaa870, 90 },
    { 0, 0 },
    { 0xf6aa872, 55 },
    { 0xd4daa872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaaa878, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5aa87e, 61 },
    { 0x6ffaa87f, 99 },
    { 0x47aaa87f, 55 },
    { 0x900aa881, 23 },
    { 0, 0 },
    { 0xc0baa883, 85 },
    { 0x897aa884, 100 },
    { 0x59baa885, 33 },
    { 0, 0 },
    { 0xc02aa887, 67 },
    { 0, 0 },
    { 0xc18aa889, 23 },
    { 0xf64aa889, 95 },
    { 0xce5aa88a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aeaa88e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7aa892, 55 },
    { 0x754aa893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5aa89d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96aa8a1, 23 },
    { 0x8f1aa8a2, 61 },
    { 0xae3aa8a3, 65 },
    { 0xb30aa8a1, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2aa8a7, 61 },
    { 0x6b1aa8a8, 17 },
    { 0x70aaa8a7, 90 },
    { 0xbcaa8aa, 61 },
    { 0x827aa8aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5aa8b0, 91 },
    { 0, 0 },
    { 0x327aa8b2, 33 },
    { 0, 0 },
    { 0xa9eaa8b4, 46 },
    { 0xdacaa8b5, 93 },
    { 0xcb8aa8b6, 85 },
    { 0x53aaa8b7, 95 },
    { 0, 0 },
    { 0x7caa8b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x583aa8bc, 98 },
    { 0, 0 },
    { 0xc97aa8be, 99 },
    { 0x729aa8be, 100 },
    { 0, 0 },
    { 0x6e7aa8c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39eaa8c5, 33 },
    { 0x342aa8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18aa8cb, 46 },
    { 0xd5baa8cc, 61 },
    { 0x93daa8cc, 90 },
    { 0x861aa8cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aa8d1, 22 },
    { 0x22aaa8d2, 65 },
    { 0xf3baa8d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4aa8d9, 51 },
    { 0x396aa8da, 77 },
    { 0xc20aa8d9, 76 },
    { 0xef7aa8da, 23 },
    { 0xf76aa8dd, 98 },
    { 0xaa6aa8de, 43 },
    { 0, 0 },
    { 0xff7aa8e0, 90 },
    { 0x510aa8e1, 43 },
    { 0x656aa8e0, 46 },
    { 0x392aa8e3, 78 },
    { 0, 0 },
    { 0x130aa8e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377aa8e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfaa8ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbaa8f0, 61 },
    { 0, 0 },
    { 0xe51aa8f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbaa8fc, 67 },
    { 0x6faa8fd, 98 },
    { 0xc1aa8fd, 84 },
    { 0x648aa8ff, 95 },
    { 0xa28aa8fe, 61 },
    { 0, 0 },
    { 0xb75aa902, 95 },
    { 0, 0 },
    { 0x7dbaa904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x955aa907, 46 },
    { 0x718aa908, 75 },
    { 0xc89aa909, 78 },
    { 0, 0 },
    { 0xfc4aa90b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97baa913, 76 },
    { 0, 0 },
    { 0xe03aa915, 33 },
    { 0xe71aa916, 55 },
    { 0, 0 },
    { 0x301aa918, 67 },
    { 0xc9caa918, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x40caa91c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6aa91f, 98 },
    { 0xb9aaa91f, 90 },
    { 0, 0 },
    { 0xdccaa922, 76 },
    { 0x961aa923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9aa926, 67 },
    { 0xa1aa927, 61 },
    { 0xec0aa926, 33 },
    { 0xf64aa929, 91 },
    { 0x508aa92a, 90 },
    { 0x7aaa927, 90 },
    { 0x76daa92c, 99 },
    { 0xa1daa92d, 84 },
    { 0xc74aa92e, 34 },
    { 0x55aaa92f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829aa938, 90 },
    { 0, 0 },
    { 0x9e8aa93a, 88 },
    { 0xf04aa93b, 46 },
    { 0xfd0aa93a, 33 },
    { 0x5c6aa93c, 98 },
    { 0, 0 },
    { 0x96eaa93f, 87 },
    { 0x472aa940, 98 },
    { 0xbf5aa941, 23 },
    { 0, 0 },
    { 0xa20aa943, 98 },
    { 0x95daa944, 53 },
    { 0xd8faa945, 61 },
    { 0xe89aa946, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357aa952, 95 },
    { 0, 0 },
    { 0x62aaa954, 68 },
    { 0x605aa954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6aa960, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318aa965, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91baa96a, 77 },
    { 0xba1aa96b, 88 },
    { 0xef4aa96c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eaa973, 76 },
    { 0x666aa974, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2aa97d, 98 },
    { 0, 0 },
    { 0x13eaa97f, 61 },
    { 0xe82aa97f, 46 },
    { 0, 0 },
    { 0xfd0aa982, 33 },
    { 0, 0 },
    { 0xbf7aa984, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacaa987, 61 },
    { 0x55faa988, 81 },
    { 0x23aaa988, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11aa98d, 23 },
    { 0x3c5aa98d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21aa993, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2aa999, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6aa99c, 61 },
    { 0, 0 },
    { 0xdd1aa99e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8aa9a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717aa9af, 93 },
    { 0x6eaa9af, 90 },
    { 0x854aa9af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42aa9b7, 66 },
    { 0x438aa9b8, 77 },
    { 0x1cbaa9b8, 67 },
    { 0xb9daa9ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96aa9c0, 77 },
    { 0x2eaa9c0, 52 },
    { 0x46daa9c2, 46 },
    { 0x483aa9c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18aa9c6, 95 },
    { 0x25aa9c7, 61 },
    { 0x97faa9c7, 90 },
    { 0, 0 },
    { 0x488aa9ca, 55 },
    { 0, 0 },
    { 0xe43aa9cc, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1aa9d3, 99 },
    { 0x6e6aa9d3, 98 },
    { 0xb90aa9d5, 100 },
    { 0xc41aa9d5, 66 },
    { 0xbeaa9d7, 34 },
    { 0, 0 },
    { 0x619aa9d9, 23 },
    { 0, 0 },
    { 0x6daaa9db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723aa9e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966aa9ea, 33 },
    { 0xa83aa9eb, 23 },
    { 0xb23aa9ec, 61 },
    { 0xb83aa9ed, 93 },
    { 0xdbfaa9ee, 34 },
    { 0xeb1aa9ea, 23 },
    { 0, 0 },
    { 0x32faa9f1, 76 },
    { 0x71faa9f1, 67 },
    { 0x2faaa9f1, 61 },
    { 0x9afaa9f4, 90 },
    { 0, 0 },
    { 0x18faa9f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccaa9f9, 23 },
    { 0x21baa9fa, 77 },
    { 0x391aa9fa, 33 },
    { 0x769aa9fc, 90 },
    { 0x9b4aa9fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbaaa01, 46 },
    { 0, 0 },
    { 0xa79aaa03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974aaa07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfaaa0d, 61 },
    { 0x34faaa0e, 93 },
    { 0x680aaa0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5aaaa14, 23 },
    { 0xec2aaa15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadaaa1a, 61 },
    { 0xcb6aaa1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50aaa1e, 76 },
    { 0xf51aaa1e, 40 },
    { 0xebbaaa1f, 67 },
    { 0x829aaa20, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76aaa26, 40 },
    { 0x39caaa27, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72baaa2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x125aaa31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0aaa35, 61 },
    { 0x74eaaa35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275aaa3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739aaa41, 33 },
    { 0xeaeaaa42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0aaa46, 90 },
    { 0x1fcaaa47, 84 },
    { 0x1dbaaa48, 33 },
    { 0x2a6aaa49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738aaa4d, 33 },
    { 0x737aaa4e, 52 },
    { 0x241aaa4e, 98 },
    { 0, 0 },
    { 0x54baaa51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbaaa59, 55 },
    { 0x156aaa5a, 61 },
    { 0, 0 },
    { 0x469aaa5c, 77 },
    { 0x812aaa5c, 84 },
    { 0, 0 },
    { 0x80aaa5f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x63baaa62, 43 },
    { 0, 0 },
    { 0x452aaa64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77caaa6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbaaa71, 33 },
    { 0xbb7aaa72, 76 },
    { 0xb98aaa73, 90 },
    { 0x51caaa74, 90 },
    { 0x45eaaa75, 34 },
    { 0xfd1aaa73, 38 },
    { 0xedaaaa77, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x69baaa7a, 34 },
    { 0x336aaa7b, 78 },
    { 0, 0 },
    { 0xf14aaa7d, 71 },
    { 0x448aaa7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabaaa81, 95 },
    { 0x516aaa82, 36 },
    { 0x60baaa81, 93 },
    { 0, 0 },
    { 0xbadaaa85, 52 },
    { 0x953aaa86, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0aaa89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1aaa8c, 98 },
    { 0, 0 },
    { 0x8d4aaa8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86caaa93, 98 },
    { 0x894aaa94, 90 },
    { 0x97baaa93, 98 },
    { 0x331aaa93, 33 },
    { 0, 0 },
    { 0x192aaa98, 61 },
    { 0xaabaaa99, 84 },
    { 0x587aaa9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1aaa9d, 61 },
    { 0, 0 },
    { 0xddbaaa9f, 55 },
    { 0x29caaa9f, 55 },
    { 0, 0 },
    { 0xca6aaaa2, 47 },
    { 0x2b5aaaa2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13aaaa7, 90 },
    { 0xa4daaaa8, 77 },
    { 0x36daaaa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aaaaad, 34 },
    { 0x4c3aaaad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8aaab6, 95 },
    { 0xf16aaab7, 33 },
    { 0x7a1aaab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9daaabc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10faaac2, 52 },
    { 0xe20aaac3, 90 },
    { 0xd1faaac3, 33 },
    { 0x4aaaaac5, 33 },
    { 0x37caaac6, 90 },
    { 0x1afaaac7, 99 },
    { 0, 0 },
    { 0xff6aaac9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1aaace, 90 },
    { 0xbeaaace, 96 },
    { 0x4e0aaad0, 52 },
    { 0, 0 },
    { 0x927aaad2, 95 },
    { 0xeaaaaad3, 90 },
    { 0xa1aaad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbaaadb, 55 },
    { 0xbdaaadc, 34 },
    { 0, 0 },
    { 0xb7eaaade, 90 },
    { 0x67aaaadf, 61 },
    { 0x3beaaae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97aaae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743aaae8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5aaaec, 95 },
    { 0x9ddaaaed, 23 },
    { 0x528aaaed, 75 },
    { 0xf98aaaef, 34 },
    { 0, 0 },
    { 0x92baaaf1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9aaaf5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1aaaff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7aab05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53aaab09, 98 },
    { 0, 0 },
    { 0x21aaab0b, 46 },
    { 0, 0 },
    { 0x7b7aab0d, 90 },
    { 0x8cfaab0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01aab12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79faab17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6aab1f, 40 },
    { 0, 0 },
    { 0xf25aab21, 76 },
    { 0x662aab21, 95 },
    { 0, 0 },
    { 0xddaab24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x150aab27, 76 },
    { 0xe41aab27, 52 },
    { 0xc10aab28, 64 },
    { 0x641aab27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x522aab2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20aab34, 90 },
    { 0xa5caab35, 90 },
    { 0xe8daab35, 93 },
    { 0xa91aab36, 93 },
    { 0x882aab38, 95 },
    { 0x3f0aab39, 33 },
    { 0x546aab35, 95 },
    { 0xa69aab3b, 85 },
    { 0, 0 },
    { 0x96aab3d, 55 },
    { 0xd3eaab3d, 61 },
    { 0xc0aaab3f, 98 },
    { 0xcacaab3d, 61 },
    { 0x777aab41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16aab45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eaab49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9faab4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191aab51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658aab57, 51 },
    { 0x24baab58, 65 },
    { 0, 0 },
    { 0xc15aab5a, 34 },
    { 0, 0 },
    { 0x7ccaab5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5aab63, 33 },
    { 0xdf3aab64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4caab67, 61 },
    { 0x593aab68, 95 },
    { 0, 0 },
    { 0x89aaab6a, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6baab6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706aab72, 98 },
    { 0x2daaab72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecaab76, 38 },
    { 0, 0 },
    { 0xaedaab78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29aab7b, 23 },
    { 0xd8faab7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x318aab7f, 93 },
    { 0x705aab80, 76 },
    { 0x649aab7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77caab86, 78 },
    { 0xd8aab87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7aab8c, 55 },
    { 0xd33aab8d, 100 },
    { 0x401aab8e, 23 },
    { 0xf95aab8e, 90 },
    { 0x7deaab90, 90 },
    { 0x5baaab90, 93 },
    { 0xb4aaab8d, 61 },
    { 0xc6eaab93, 61 },
    { 0xe01aab92, 75 },
    { 0xcf1aab95, 23 },
    { 0x1d5aab95, 91 },
    { 0, 0 },
    { 0x43eaab98, 95 },
    { 0, 0 },
    { 0x1a7aab9a, 76 },
    { 0, 0 },
    { 0x66aab9c, 33 },
    { 0, 0 },
    { 0xb11aab9e, 61 },
    { 0x858aab9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232aaba5, 33 },
    { 0, 0 },
    { 0xacbaaba7, 55 },
    { 0x16aaaba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68aabad, 61 },
    { 0xb15aabad, 52 },
    { 0x4d9aabad, 61 },
    { 0xda7aabb0, 98 },
    { 0x476aabad, 52 },
    { 0x3b9aabb2, 36 },
    { 0x89baabb2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4aabb6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25aabbd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81aaabc3, 96 },
    { 0, 0 },
    { 0xd08aabc5, 98 },
    { 0x1f3aabc5, 99 },
    { 0x26aabc7, 90 },
    { 0x3a4aabc8, 43 },
    { 0x679aabc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433aabcd, 100 },
    { 0, 0 },
    { 0x849aabcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49faabd2, 90 },
    { 0xe47aabd3, 34 },
    { 0xe9aabd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09aabd7, 95 },
    { 0x2a7aabd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276aabdd, 61 },
    { 0xe07aabde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3aabe1, 88 },
    { 0xf8aabe1, 33 },
    { 0xcc3aabe3, 34 },
    { 0x2c7aabe3, 61 },
    { 0x559aabe5, 98 },
    { 0x9f3aabe5, 55 },
    { 0, 0 },
    { 0xe3eaabe8, 61 },
    { 0xffdaabe9, 43 },
    { 0x6a2aabea, 98 },
    { 0x974aabeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128aabf4, 95 },
    { 0xb08aabf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9aabf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5aabfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82aabff, 90 },
    { 0, 0 },
    { 0x2e2aac01, 90 },
    { 0, 0 },
    { 0x61aaac03, 85 },
    { 0x824aac04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415aac0b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x125aac0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2aac14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aaac17, 98 },
    { 0xea5aac18, 94 },
    { 0xec9aac19, 46 },
    { 0xcaaaac1a, 100 },
    { 0xf0faac1b, 90 },
    { 0x460aac18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeaac1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7baac25, 90 },
    { 0x55faac26, 90 },
    { 0xf57aac25, 94 },
    { 0x8f7aac28, 66 },
    { 0x588aac25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752aac2f, 33 },
    { 0x7bcaac30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aac33, 33 },
    { 0x5deaac34, 98 },
    { 0, 0 },
    { 0x8e3aac36, 95 },
    { 0x4daac37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4aac3a, 61 },
    { 0xae9aac3b, 23 },
    { 0x5a5aac3b, 99 },
    { 0xaf6aac3c, 90 },
    { 0x18aaac3e, 66 },
    { 0, 0 },
    { 0xa4eaac40, 55 },
    { 0x692aac40, 33 },
    { 0x66faac42, 95 },
    { 0x4eeaac43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21aaac4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86baac4d, 61 },
    { 0x7bfaac4d, 95 },
    { 0x5f1aac4f, 98 },
    { 0x71faac4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184aac56, 95 },
    { 0x587aac57, 98 },
    { 0, 0 },
    { 0x5a7aac59, 90 },
    { 0xd5daac5a, 85 },
    { 0x195aac59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12aac5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaaaac61, 98 },
    { 0xd6faac62, 77 },
    { 0x83eaac63, 40 },
    { 0xbd2aac63, 76 },
    { 0x9bfaac63, 95 },
    { 0xe5faac66, 90 },
    { 0x7f7aac67, 91 },
    { 0x700aac68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53aac6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528aac78, 89 },
    { 0xe50aac79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653aac7e, 33 },
    { 0x232aac7f, 76 },
    { 0, 0 },
    { 0x89baac81, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163aac85, 33 },
    { 0x51baac86, 52 },
    { 0xd45aac87, 93 },
    { 0x59faac88, 40 },
    { 0xde1aac89, 43 },
    { 0x88daac86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56caac8d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855aac91, 23 },
    { 0, 0 },
    { 0x4fdaac93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6aac96, 90 },
    { 0x1b6aac96, 84 },
    { 0x261aac98, 98 },
    { 0x485aac99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aac9d, 95 },
    { 0x5baaac9e, 98 },
    { 0, 0 },
    { 0xdddaaca0, 81 },
    { 0, 0 },
    { 0x9ddaaca2, 33 },
    { 0xb72aaca3, 43 },
    { 0, 0 },
    { 0x55aaaca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73daaca8, 34 },
    { 0x568aaca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68aacaf, 93 },
    { 0xc7aaacb0, 61 },
    { 0x8a4aacb0, 33 },
    { 0x14caacaf, 55 },
    { 0, 0 },
    { 0x701aacb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81aacb7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961aacbc, 76 },
    { 0, 0 },
    { 0x393aacbe, 98 },
    { 0x72eaacbe, 98 },
    { 0xaedaacc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaacc4, 95 },
    { 0x987aacc5, 91 },
    { 0xfdeaacc6, 33 },
    { 0x966aacc6, 95 },
    { 0x55aaacc8, 40 },
    { 0xd44aacc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4caaccd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x711aacd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1aacd3, 90 },
    { 0, 0 },
    { 0x4ecaacd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1daacd8, 61 },
    { 0x58faacd9, 33 },
    { 0x5c2aacda, 90 },
    { 0x7b6aacdb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x906aacde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacaace4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbaacea, 34 },
    { 0x756aaceb, 17 },
    { 0, 0 },
    { 0x134aaced, 85 },
    { 0xf19aacee, 93 },
    { 0xa64aacee, 43 },
    { 0xa43aacef, 61 },
    { 0xb1baacef, 66 },
    { 0, 0 },
    { 0xaabaacf3, 95 },
    { 0x589aacf4, 66 },
    { 0x713aacf5, 55 },
    { 0xe1baacf3, 91 },
    { 0xf82aacf3, 33 },
    { 0x785aacf8, 56 },
    { 0x362aacf8, 90 },
    { 0xc2aaacf8, 90 },
    { 0xc0aaacfa, 84 },
    { 0x3b2aacf8, 55 },
    { 0x4d6aacfd, 91 },
    { 0, 0 },
    { 0xf96aacff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635aad05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868aad0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aaad0d, 90 },
    { 0x827aad0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993aad14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfaad18, 66 },
    { 0xea4aad18, 90 },
    { 0xa54aad1a, 76 },
    { 0x469aad1b, 33 },
    { 0x3f9aad1c, 93 },
    { 0xec1aad1c, 66 },
    { 0xee4aad18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aaad21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3aad24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75aaad27, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81faad2b, 66 },
    { 0x5d0aad2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7aad31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aad36, 43 },
    { 0x504aad37, 23 },
    { 0xf8aad38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43faad3c, 95 },
    { 0xd50aad3d, 61 },
    { 0xab2aad3e, 33 },
    { 0xd15aad3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2aad43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9aad4a, 90 },
    { 0, 0 },
    { 0x7b1aad4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdaad51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9aad58, 67 },
    { 0x9a8aad59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2aad5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754aad63, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308aad6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7aad6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3aad74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aad79, 23 },
    { 0xa78aad79, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10faad7e, 95 },
    { 0xaadaad7f, 90 },
    { 0x788aad80, 95 },
    { 0xae8aad7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8aad86, 33 },
    { 0x635aad87, 55 },
    { 0xaeeaad87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x41daad8b, 33 },
    { 0, 0 },
    { 0x1aeaad8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83aad90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629aada9, 40 },
    { 0xd4aaadaa, 33 },
    { 0x1dcaadab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655aadb2, 96 },
    { 0x722aadb2, 76 },
    { 0x87faadb2, 95 },
    { 0x2bdaadb5, 93 },
    { 0, 0 },
    { 0x5c1aadb7, 98 },
    { 0x1d6aadb8, 55 },
    { 0, 0 },
    { 0xeceaadba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefaadbe, 33 },
    { 0, 0 },
    { 0x94faadc0, 98 },
    { 0xa78aadc1, 61 },
    { 0, 0 },
    { 0xb41aadc3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59aadc6, 93 },
    { 0xcf7aadc6, 55 },
    { 0x45baadc8, 23 },
    { 0x5a9aadc7, 34 },
    { 0, 0 },
    { 0x5aeaadcb, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edaadce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0aadd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eaaadd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935aaddb, 55 },
    { 0xb5baaddb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87aade2, 23 },
    { 0, 0 },
    { 0x1d5aade4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60aade8, 77 },
    { 0xecbaade8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10aadec, 23 },
    { 0x8a0aadec, 81 },
    { 0x24aaadec, 33 },
    { 0xe66aadef, 33 },
    { 0xfa5aadf0, 90 },
    { 0x352aadf1, 85 },
    { 0x97caadef, 95 },
    { 0xb62aadf1, 98 },
    { 0x863aadf0, 23 },
    { 0xc1daadf5, 33 },
    { 0x34daadf5, 98 },
    { 0x12eaadf7, 55 },
    { 0xc0aadee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4aadfc, 61 },
    { 0xbeaadfc, 36 },
    { 0x6d0aadfe, 46 },
    { 0x40aadff, 90 },
    { 0x6dbaae00, 90 },
    { 0x74aadff, 85 },
    { 0x804aae01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782aae06, 33 },
    { 0xbdeaae07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704aae0c, 100 },
    { 0, 0 },
    { 0x9f4aae0e, 33 },
    { 0x3afaae0f, 81 },
    { 0xca4aae0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x924aae13, 53 },
    { 0x351aae14, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787aae21, 90 },
    { 0xa5eaae22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39aae2c, 61 },
    { 0xc43aae2c, 61 },
    { 0x4a2aae2e, 33 },
    { 0x4d2aae2f, 34 },
    { 0xe86aae2e, 66 },
    { 0, 0 },
    { 0x4cfaae32, 100 },
    { 0x298aae33, 95 },
    { 0, 0 },
    { 0x9fdaae35, 91 },
    { 0x545aae36, 33 },
    { 0x1aeaae36, 84 },
    { 0, 0 },
    { 0x88aaae39, 90 },
    { 0x7eaae3a, 33 },
    { 0x512aae3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6aae3e, 87 },
    { 0x684aae3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6aae44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23faae47, 97 },
    { 0x376aae47, 90 },
    { 0x459aae47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927aae57, 100 },
    { 0x479aae58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x343aae5b, 61 },
    { 0, 0 },
    { 0x3b0aae5d, 55 },
    { 0x5cbaae5e, 23 },
    { 0, 0 },
    { 0x3f0aae60, 76 },
    { 0x37caae60, 98 },
    { 0x38caae60, 93 },
    { 0x3eaaae63, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcaaae69, 33 },
    { 0x9a3aae6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4aae6d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6aae71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4aae7c, 90 },
    { 0xb31aae7d, 95 },
    { 0x61faae7c, 66 },
    { 0xb80aae7f, 55 },
    { 0, 0 },
    { 0x3f5aae81, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8aae84, 95 },
    { 0xe10aae85, 33 },
    { 0xeeaae85, 85 },
    { 0xcbcaae87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2daae8b, 66 },
    { 0xcaeaae8b, 38 },
    { 0x339aae8c, 95 },
    { 0xfe8aae8e, 98 },
    { 0x3e6aae8f, 43 },
    { 0x5feaae8e, 95 },
    { 0, 0 },
    { 0xfe0aae92, 46 },
    { 0, 0 },
    { 0x9efaae94, 91 },
    { 0, 0 },
    { 0x70daae96, 98 },
    { 0xd08aae97, 52 },
    { 0x131aae98, 87 },
    { 0x5a3aae98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x321aae9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2aaea0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefaaea5, 95 },
    { 0xf1faaea6, 46 },
    { 0x305aaea7, 46 },
    { 0x85daaea8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9aaeaf, 42 },
    { 0xe46aaeaf, 78 },
    { 0, 0 },
    { 0x264aaeb2, 95 },
    { 0, 0 },
    { 0x51baaeb4, 33 },
    { 0x4e9aaeb5, 90 },
    { 0x3d7aaeb6, 52 },
    { 0x6f7aaeb6, 95 },
    { 0xe6aaaeb7, 33 },
    { 0x740aaeb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784aaec2, 98 },
    { 0, 0 },
    { 0xb2aaec4, 81 },
    { 0x599aaec4, 61 },
    { 0x92baaec4, 95 },
    { 0x533aaec7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63aaecf, 95 },
    { 0x7e4aaed0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93eaaedb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964aaee7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x280aaeea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0aaeed, 76 },
    { 0x851aaeee, 99 },
    { 0x6d1aaeef, 33 },
    { 0xb2daaef0, 65 },
    { 0, 0 },
    { 0xacbaaef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16aaef5, 61 },
    { 0x3feaaef6, 77 },
    { 0xdeaaaef7, 66 },
    { 0xef8aaef8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adaaefe, 61 },
    { 0, 0 },
    { 0x37caaf00, 98 },
    { 0x8e3aaf00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2aaf04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x376aaf07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3aaf0a, 87 },
    { 0xc28aaf0b, 97 },
    { 0x41daaf0c, 76 },
    { 0xe9eaaf0c, 76 },
    { 0x192aaf0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1aaf11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x639aaf14, 85 },
    { 0x953aaf15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29baaf1b, 91 },
    { 0x49faaf1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x336aaf1f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x215aaf22, 23 },
    { 0x784aaf23, 98 },
    { 0, 0 },
    { 0x53eaaf25, 100 },
    { 0xdf0aaf25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72eaaf2b, 33 },
    { 0, 0 },
    { 0xd5eaaf2d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x128aaf30, 98 },
    { 0, 0 },
    { 0x92daaf32, 66 },
    { 0x909aaf33, 95 },
    { 0, 0 },
    { 0x830aaf35, 95 },
    { 0x463aaf36, 81 },
    { 0x231aaf37, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781aaf3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceaaf49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14baaf4d, 40 },
    { 0xa96aaf4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afaaf52, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x13caaf55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5caaaf58, 95 },
    { 0, 0 },
    { 0x52aaaf5a, 90 },
    { 0, 0 },
    { 0x749aaf5c, 91 },
    { 0x914aaf5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53aaf68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8aaf6c, 96 },
    { 0x741aaf6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8baaf72, 98 },
    { 0, 0 },
    { 0xcc8aaf74, 61 },
    { 0x440aaf74, 95 },
    { 0, 0 },
    { 0xaadaaf77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65aaf7a, 33 },
    { 0, 0 },
    { 0x399aaf7c, 95 },
    { 0xd8faaf7d, 61 },
    { 0xf72aaf7d, 66 },
    { 0x316aaf7d, 23 },
    { 0, 0 },
    { 0x9d4aaf81, 93 },
    { 0x411aaf82, 53 },
    { 0x9e2aaf83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437aaf89, 88 },
    { 0xa13aaf89, 93 },
    { 0x7a9aaf8b, 88 },
    { 0, 0 },
    { 0x43faaf8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eaaf94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbaaf97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x729aaf9a, 61 },
    { 0x84daaf9b, 40 },
    { 0, 0 },
    { 0x786aaf9d, 90 },
    { 0x91aaaf9e, 98 },
    { 0, 0 },
    { 0x624aafa0, 33 },
    { 0x5d3aafa1, 95 },
    { 0xdb7aafa2, 46 },
    { 0xf5baafa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbaafaa, 90 },
    { 0x49baafaa, 61 },
    { 0x630aafac, 61 },
    { 0x4b0aafac, 99 },
    { 0, 0 },
    { 0x3ddaafaf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628aafb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadaafbf, 23 },
    { 0x84eaafc0, 84 },
    { 0xfe7aafc1, 55 },
    { 0x33aafc2, 33 },
    { 0xabdaafc1, 84 },
    { 0xcffaafc4, 55 },
    { 0x62daafc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2aaafc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4aafd1, 61 },
    { 0x20faafd2, 85 },
    { 0x296aafd3, 90 },
    { 0xddaafd4, 61 },
    { 0xc9faafd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3aafdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x186aafe0, 61 },
    { 0x884aafe1, 95 },
    { 0x7ffaafe2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaaafe5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304aafe9, 33 },
    { 0x13faafea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61caafed, 95 },
    { 0x764aafee, 61 },
    { 0x2f5aafee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84aaff4, 55 },
    { 0x777aaff5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3aab000, 87 },
    { 0x47cab001, 34 },
    { 0, 0 },
    { 0x14aab003, 78 },
    { 0, 0 },
    { 0x1f8ab005, 84 },
    { 0x1b8ab006, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ab00a, 96 },
    { 0x99bab00b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x291ab00e, 33 },
    { 0x16aab00e, 98 },
    { 0xf57ab010, 77 },
    { 0xa6eab010, 55 },
    { 0xb7bab00e, 33 },
    { 0x292ab00e, 23 },
    { 0x51bab012, 23 },
    { 0x833ab015, 90 },
    { 0x937ab015, 78 },
    { 0xb48ab013, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89ab020, 61 },
    { 0xaedab021, 98 },
    { 0xcd5ab022, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eab029, 76 },
    { 0xe82ab02a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5ab02e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25ab032, 98 },
    { 0x485ab033, 95 },
    { 0x51fab033, 95 },
    { 0xd53ab033, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853ab03a, 76 },
    { 0, 0 },
    { 0x285ab03c, 90 },
    { 0x85bab03d, 76 },
    { 0x2f0ab03e, 27 },
    { 0x383ab03f, 65 },
    { 0x2ceab040, 98 },
    { 0xaefab040, 23 },
    { 0x3eab042, 88 },
    { 0x1eeab043, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cab04a, 76 },
    { 0xb62ab04b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4ab050, 33 },
    { 0xeddab051, 90 },
    { 0xb5cab052, 90 },
    { 0xbd0ab052, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ab056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x516ab059, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x338ab05c, 93 },
    { 0, 0 },
    { 0xa6ab05e, 80 },
    { 0xdf7ab05e, 23 },
    { 0x8e0ab060, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x189ab063, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5ab066, 98 },
    { 0x481ab067, 61 },
    { 0xad9ab067, 98 },
    { 0x3a7ab068, 67 },
    { 0x21ab06a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdab06e, 95 },
    { 0, 0 },
    { 0x206ab070, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54ab078, 17 },
    { 0xfe9ab079, 95 },
    { 0xa86ab07a, 33 },
    { 0, 0 },
    { 0xe06ab07c, 95 },
    { 0x52fab07d, 85 },
    { 0xd23ab07d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932ab085, 93 },
    { 0x2bcab086, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24ab08a, 46 },
    { 0, 0 },
    { 0x5fcab08c, 90 },
    { 0, 0 },
    { 0x3fdab08e, 76 },
    { 0, 0 },
    { 0xf7ab090, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aab097, 90 },
    { 0x2e6ab098, 90 },
    { 0xa84ab099, 96 },
    { 0xc54ab098, 33 },
    { 0xd15ab099, 95 },
    { 0xb7aab097, 90 },
    { 0xe7aab09d, 43 },
    { 0, 0 },
    { 0xc8bab09f, 68 },
    { 0x4c3ab0a0, 98 },
    { 0x3daab0a0, 98 },
    { 0, 0 },
    { 0xe6bab0a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ab0a6, 46 },
    { 0x578ab0a7, 33 },
    { 0x97fab0a6, 55 },
    { 0x7c0ab0a9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410ab0ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126ab0bc, 34 },
    { 0x943ab0bd, 78 },
    { 0, 0 },
    { 0xec5ab0bf, 55 },
    { 0xe7eab0c0, 90 },
    { 0x862ab0c1, 95 },
    { 0x842ab0c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32ab0c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bab0d4, 46 },
    { 0xc56ab0d5, 76 },
    { 0, 0 },
    { 0x69eab0d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x917ab0da, 95 },
    { 0x20aab0db, 98 },
    { 0, 0 },
    { 0xb7ab0dd, 98 },
    { 0x4e1ab0de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdab0e2, 33 },
    { 0, 0 },
    { 0xe01ab0e4, 33 },
    { 0xe11ab0e4, 95 },
    { 0, 0 },
    { 0x9f1ab0e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6ab0ea, 33 },
    { 0x2edab0eb, 36 },
    { 0xd22ab0ec, 61 },
    { 0xf59ab0ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cab0f0, 80 },
    { 0xf03ab0f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243ab0f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eab0fe, 98 },
    { 0, 0 },
    { 0xe9eab100, 95 },
    { 0, 0 },
    { 0xc06ab102, 55 },
    { 0, 0 },
    { 0x6eab104, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bab108, 61 },
    { 0, 0 },
    { 0xc74ab10a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29ab10d, 90 },
    { 0, 0 },
    { 0xc85ab10f, 22 },
    { 0, 0 },
    { 0x5c4ab111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8ab118, 99 },
    { 0x81fab119, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650ab11e, 76 },
    { 0, 0 },
    { 0xebcab120, 61 },
    { 0x36dab121, 61 },
    { 0x698ab121, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122ab128, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ab12b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65ab12e, 61 },
    { 0x8fcab12f, 33 },
    { 0x55aab130, 3 },
    { 0x60eab131, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccab134, 91 },
    { 0, 0 },
    { 0xeeaab136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8ab139, 99 },
    { 0xa69ab13a, 23 },
    { 0x7e3ab13b, 90 },
    { 0, 0 },
    { 0xedcab13d, 91 },
    { 0xf4ab13d, 77 },
    { 0xa57ab13d, 55 },
    { 0xbffab13e, 55 },
    { 0xff5ab141, 78 },
    { 0xd34ab13f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dab147, 87 },
    { 0x176ab147, 33 },
    { 0x91eab149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ab14f, 61 },
    { 0xa66ab14f, 93 },
    { 0x636ab14f, 55 },
    { 0x96cab152, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ab157, 88 },
    { 0x892ab157, 78 },
    { 0x134ab159, 61 },
    { 0xeb5ab157, 100 },
    { 0x3c6ab157, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bab15e, 33 },
    { 0x9e4ab15e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927ab16a, 98 },
    { 0, 0 },
    { 0x18dab16c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bab173, 23 },
    { 0xeccab174, 61 },
    { 0, 0 },
    { 0x79fab176, 23 },
    { 0, 0 },
    { 0xbfdab178, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbab17b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ab17e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ab188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bab190, 66 },
    { 0x51ab191, 23 },
    { 0xb53ab190, 61 },
    { 0x2c5ab193, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fab19e, 55 },
    { 0xdfcab19e, 98 },
    { 0, 0 },
    { 0xa19ab1a1, 23 },
    { 0x8aab1a2, 90 },
    { 0xa1eab1a2, 76 },
    { 0xdf0ab1a3, 55 },
    { 0x876ab1a1, 95 },
    { 0x4d8ab1a4, 55 },
    { 0xf43ab1a1, 83 },
    { 0x9b1ab1a8, 68 },
    { 0, 0 },
    { 0xc3cab1aa, 34 },
    { 0x7edab1ab, 34 },
    { 0x9bcab1ab, 55 },
    { 0xb2bab1ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379ab1b3, 55 },
    { 0x953ab1b3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ab1ba, 61 },
    { 0, 0 },
    { 0x19dab1bc, 43 },
    { 0xcd0ab1bc, 17 },
    { 0xbd6ab1be, 88 },
    { 0, 0 },
    { 0xf03ab1c0, 52 },
    { 0x6cfab1c0, 76 },
    { 0, 0 },
    { 0xbd6ab1c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9ab1ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecab1d3, 23 },
    { 0, 0 },
    { 0xa87ab1d5, 98 },
    { 0, 0 },
    { 0xb7fab1d7, 61 },
    { 0xeeab1d8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5ab1db, 76 },
    { 0x340ab1db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecab1e0, 33 },
    { 0xdccab1e0, 40 },
    { 0x26bab1e2, 55 },
    { 0x558ab1e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dab1e6, 76 },
    { 0x879ab1e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2ab1ec, 34 },
    { 0x6c7ab1ed, 95 },
    { 0xe36ab1ed, 95 },
    { 0x688ab1ec, 95 },
    { 0x6a7ab1f0, 76 },
    { 0x1b9ab1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79ab1f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fab1f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fab1fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x287ab1ff, 66 },
    { 0, 0 },
    { 0xe6eab201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1ab204, 55 },
    { 0x778ab205, 61 },
    { 0xb5fab205, 90 },
    { 0x2e3ab207, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3ab20a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecab211, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eab215, 95 },
    { 0xbe2ab215, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634ab21d, 66 },
    { 0x20aab21d, 85 },
    { 0xc8bab21f, 66 },
    { 0x4aeab220, 84 },
    { 0x781ab220, 87 },
    { 0x96cab222, 34 },
    { 0x8f9ab222, 76 },
    { 0, 0 },
    { 0x5e2ab225, 67 },
    { 0x603ab226, 75 },
    { 0x676ab227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdab22a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ab22f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25ab233, 23 },
    { 0x7daab234, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcab23a, 33 },
    { 0x618ab23b, 23 },
    { 0, 0 },
    { 0xb32ab23d, 100 },
    { 0x79ab23e, 33 },
    { 0x2e4ab23d, 90 },
    { 0x3aab240, 91 },
    { 0xe17ab23d, 98 },
    { 0x70fab23d, 90 },
    { 0, 0 },
    { 0x42bab244, 61 },
    { 0, 0 },
    { 0x8c6ab246, 61 },
    { 0, 0 },
    { 0xdaaab248, 77 },
    { 0x63aab249, 98 },
    { 0x81bab24a, 90 },
    { 0x870ab24b, 33 },
    { 0x14eab24c, 33 },
    { 0x5c2ab24c, 90 },
    { 0xd28ab24e, 45 },
    { 0x3f8ab24c, 55 },
    { 0x948ab249, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329ab258, 90 },
    { 0, 0 },
    { 0x8e8ab25a, 52 },
    { 0, 0 },
    { 0x757ab25c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bab262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ab266, 90 },
    { 0x3d5ab267, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdab26a, 98 },
    { 0xe69ab26b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbab26e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ab272, 26 },
    { 0xd90ab273, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0ab27a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3ab27e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ab281, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848ab285, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9ab289, 46 },
    { 0xc0fab28a, 23 },
    { 0x714ab28b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ab28f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aab294, 90 },
    { 0, 0 },
    { 0xf28ab296, 33 },
    { 0x93ab297, 43 },
    { 0x764ab296, 95 },
    { 0xde4ab299, 61 },
    { 0, 0 },
    { 0xa02ab29b, 23 },
    { 0xdb9ab29c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526ab2a0, 95 },
    { 0xdc7ab2a1, 90 },
    { 0x377ab2a2, 84 },
    { 0xa71ab2a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77ab2a6, 23 },
    { 0xca6ab2a6, 61 },
    { 0x581ab2a7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ab2ab, 98 },
    { 0xe23ab2ac, 61 },
    { 0x70dab2ad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eab2b5, 55 },
    { 0xe61ab2b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6ab2b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaab2be, 23 },
    { 0xb07ab2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42ab2c3, 34 },
    { 0xddab2c4, 66 },
    { 0, 0 },
    { 0xea6ab2c6, 90 },
    { 0x842ab2c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16eab2ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x577ab2cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fab2d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2ab2d6, 88 },
    { 0xad6ab2d7, 65 },
    { 0xaa2ab2d6, 98 },
    { 0xd4eab2d6, 43 },
    { 0x11dab2d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x531ab2dd, 100 },
    { 0, 0 },
    { 0x7cbab2df, 90 },
    { 0xc9fab2e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dab2e3, 100 },
    { 0x2dbab2e4, 90 },
    { 0x35bab2e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbab2e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477ab2f1, 76 },
    { 0x842ab2f2, 90 },
    { 0x73dab2f3, 90 },
    { 0x728ab2f4, 67 },
    { 0xc59ab2f4, 23 },
    { 0x4bcab2f6, 87 },
    { 0, 0 },
    { 0x959ab2f8, 33 },
    { 0xcf5ab2f9, 68 },
    { 0, 0 },
    { 0xadbab2fb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aeab2fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0ab303, 95 },
    { 0x2d4ab304, 87 },
    { 0, 0 },
    { 0xa59ab306, 87 },
    { 0xf81ab307, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x838ab30a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934ab312, 55 },
    { 0xc6ab313, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaaab317, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0ab31a, 98 },
    { 0, 0 },
    { 0xd14ab31c, 95 },
    { 0x29dab31c, 34 },
    { 0x43dab31e, 61 },
    { 0, 0 },
    { 0xa95ab320, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cab324, 61 },
    { 0, 0 },
    { 0x273ab326, 75 },
    { 0x342ab327, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07ab32a, 33 },
    { 0xc86ab32b, 93 },
    { 0xff7ab32c, 23 },
    { 0xfab32d, 98 },
    { 0, 0 },
    { 0xdd2ab32f, 33 },
    { 0x6dbab330, 100 },
    { 0, 0 },
    { 0xf0dab332, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7ab335, 61 },
    { 0xb7cab335, 99 },
    { 0x618ab337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefab33c, 61 },
    { 0xf13ab33d, 90 },
    { 0xe39ab33e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05ab341, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ab348, 61 },
    { 0, 0 },
    { 0xe4fab34a, 23 },
    { 0xdbbab34b, 68 },
    { 0xd64ab34c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80ab34f, 95 },
    { 0xa60ab350, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bab354, 95 },
    { 0, 0 },
    { 0xd56ab356, 61 },
    { 0, 0 },
    { 0x27bab358, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921ab35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deab360, 23 },
    { 0x4fab360, 33 },
    { 0x1c7ab360, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x519ab365, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687ab36b, 90 },
    { 0xf00ab36b, 33 },
    { 0, 0 },
    { 0xa78ab36e, 90 },
    { 0x50bab36e, 76 },
    { 0, 0 },
    { 0x992ab371, 34 },
    { 0x912ab372, 33 },
    { 0xa58ab373, 43 },
    { 0xf8dab373, 23 },
    { 0xc9dab371, 97 },
    { 0x40eab376, 66 },
    { 0, 0 },
    { 0x6afab378, 23 },
    { 0xeab379, 33 },
    { 0, 0 },
    { 0x8e2ab37b, 23 },
    { 0xafdab37c, 61 },
    { 0, 0 },
    { 0x5c5ab37e, 61 },
    { 0, 0 },
    { 0xa77ab380, 61 },
    { 0x8e7ab381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2ab389, 90 },
    { 0, 0 },
    { 0x70dab38b, 55 },
    { 0, 0 },
    { 0x49bab38d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cab391, 76 },
    { 0x3ecab391, 61 },
    { 0xaa7ab391, 76 },
    { 0xbab394, 88 },
    { 0x759ab395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bab39c, 91 },
    { 0x1afab39d, 55 },
    { 0x811ab39e, 46 },
    { 0x71aab39f, 100 },
    { 0, 0 },
    { 0xce9ab3a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bab3a4, 77 },
    { 0, 0 },
    { 0x69fab3a6, 98 },
    { 0x73aab3a7, 90 },
    { 0, 0 },
    { 0x6f2ab3a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9ab3b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9ab3b5, 98 },
    { 0, 0 },
    { 0x707ab3b7, 23 },
    { 0x4c0ab3b8, 92 },
    { 0, 0 },
    { 0x88cab3ba, 23 },
    { 0x7aab3bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ab3be, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccab3c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaab3c7, 77 },
    { 0xd02ab3c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422ab3ce, 52 },
    { 0x71bab3cf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edab3da, 77 },
    { 0x559ab3db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dab3e4, 93 },
    { 0, 0 },
    { 0xeefab3e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cab3e9, 36 },
    { 0x636ab3e9, 24 },
    { 0x545ab3ea, 55 },
    { 0, 0 },
    { 0xceab3ed, 55 },
    { 0, 0 },
    { 0xbb5ab3ef, 98 },
    { 0x663ab3f0, 46 },
    { 0x813ab3f0, 61 },
    { 0, 0 },
    { 0xe0ab3f3, 61 },
    { 0x94fab3f4, 55 },
    { 0x441ab3f5, 33 },
    { 0xa64ab3f6, 65 },
    { 0xd4eab3f5, 61 },
    { 0x27aab3f8, 23 },
    { 0xac9ab3f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822ab400, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faab404, 90 },
    { 0xb7aab405, 33 },
    { 0x4c3ab404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bab409, 23 },
    { 0xaf7ab40a, 33 },
    { 0x25bab40b, 46 },
    { 0x307ab40c, 98 },
    { 0, 0 },
    { 0x68cab40e, 65 },
    { 0xdbfab40f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9ab412, 90 },
    { 0xbbcab413, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47ab417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x667ab41a, 61 },
    { 0, 0 },
    { 0x88dab41c, 83 },
    { 0x826ab41d, 46 },
    { 0xc01ab41d, 98 },
    { 0, 0 },
    { 0x351ab420, 61 },
    { 0xc71ab421, 61 },
    { 0x29dab422, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fab425, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdab429, 98 },
    { 0, 0 },
    { 0x5c8ab42b, 99 },
    { 0, 0 },
    { 0x539ab42d, 90 },
    { 0x37bab42e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1ab437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x482ab43a, 33 },
    { 0, 0 },
    { 0x1fbab43c, 66 },
    { 0xe49ab43c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291ab443, 33 },
    { 0, 0 },
    { 0x649ab445, 95 },
    { 0xc8fab446, 33 },
    { 0xee1ab447, 90 },
    { 0x673ab448, 95 },
    { 0x6dab449, 52 },
    { 0x86aab44a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdab451, 23 },
    { 0x900ab452, 33 },
    { 0x361ab451, 100 },
    { 0, 0 },
    { 0xf32ab455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15ab458, 61 },
    { 0, 0 },
    { 0xf0fab45a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x607ab45d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdab463, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ab466, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26ab470, 61 },
    { 0, 0 },
    { 0x172ab472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dab476, 98 },
    { 0x598ab476, 90 },
    { 0x801ab477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63ab47d, 61 },
    { 0x250ab47e, 100 },
    { 0xe5bab47f, 96 },
    { 0xc66ab480, 81 },
    { 0xc5ab481, 34 },
    { 0xf39ab481, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2ab488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490ab48d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfab494, 85 },
    { 0xd27ab495, 95 },
    { 0xd3fab496, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aab499, 88 },
    { 0xd79ab49a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x755ab49d, 34 },
    { 0xeeab49e, 23 },
    { 0xdb1ab49f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8aab4a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ab4a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510ab4b0, 95 },
    { 0x57dab4b1, 98 },
    { 0, 0 },
    { 0x236ab4b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ab4b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aab4ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771ab4c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180ab4c8, 61 },
    { 0xeaaab4c9, 90 },
    { 0xc20ab4c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2ab4cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ab4d4, 61 },
    { 0x6f7ab4d5, 76 },
    { 0x286ab4d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759ab4da, 61 },
    { 0x231ab4da, 61 },
    { 0xa0ab4db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5ab4e0, 61 },
    { 0x4e6ab4e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979ab4e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ab4ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7ab4f0, 23 },
    { 0x610ab4f0, 88 },
    { 0x6e1ab4f2, 23 },
    { 0x7ecab4f3, 33 },
    { 0xdd9ab4f4, 53 },
    { 0x87eab4f4, 95 },
    { 0, 0 },
    { 0x98fab4f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eab4fa, 76 },
    { 0, 0 },
    { 0x2efab4fc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2ab4ff, 98 },
    { 0xe21ab4ff, 46 },
    { 0x17fab501, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ab504, 23 },
    { 0xc68ab505, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4ab50c, 23 },
    { 0x576ab50d, 23 },
    { 0x3beab50d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0ab511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bab515, 33 },
    { 0xbdab516, 23 },
    { 0, 0 },
    { 0xa39ab518, 85 },
    { 0, 0 },
    { 0x57ab51a, 33 },
    { 0, 0 },
    { 0x175ab51c, 33 },
    { 0x8adab51c, 33 },
    { 0x80bab51e, 17 },
    { 0x9e1ab51f, 61 },
    { 0, 0 },
    { 0x9d1ab521, 77 },
    { 0x222ab522, 33 },
    { 0xe24ab523, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42ab526, 90 },
    { 0x5f7ab527, 33 },
    { 0x2cab526, 33 },
    { 0, 0 },
    { 0x825ab52a, 100 },
    { 0, 0 },
    { 0x67bab52c, 90 },
    { 0xa10ab52d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3ab530, 46 },
    { 0x8fbab530, 34 },
    { 0x12fab530, 23 },
    { 0x2f5ab533, 61 },
    { 0xca3ab531, 61 },
    { 0x555ab533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9ab53a, 67 },
    { 0, 0 },
    { 0xa75ab53c, 95 },
    { 0xfcaab53c, 61 },
    { 0xe9bab53e, 76 },
    { 0xbb8ab53f, 90 },
    { 0xd4dab53f, 34 },
    { 0x6d7ab53c, 23 },
    { 0x844ab53d, 91 },
    { 0xd9eab543, 90 },
    { 0xd37ab544, 34 },
    { 0xe43ab53d, 83 },
    { 0x37bab546, 90 },
    { 0, 0 },
    { 0x20dab548, 33 },
    { 0x2e5ab549, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36ab54c, 55 },
    { 0xd4dab54c, 75 },
    { 0x612ab54e, 98 },
    { 0xfceab54f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aab553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab558, 55 },
    { 0x88aab559, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dab565, 80 },
    { 0, 0 },
    { 0xf26ab567, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858ab56e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fab572, 33 },
    { 0, 0 },
    { 0x901ab574, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bab579, 93 },
    { 0x734ab579, 23 },
    { 0, 0 },
    { 0xbcab57c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fab580, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ab583, 46 },
    { 0x567ab584, 34 },
    { 0x2b3ab584, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cab588, 90 },
    { 0x4b6ab589, 98 },
    { 0, 0 },
    { 0x6f2ab58b, 43 },
    { 0x10bab58b, 70 },
    { 0x8b2ab58d, 33 },
    { 0x190ab58e, 33 },
    { 0x27dab58f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cab597, 34 },
    { 0, 0 },
    { 0x33bab599, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbab5a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5ab5a5, 95 },
    { 0x10aab5a6, 90 },
    { 0, 0 },
    { 0xb91ab5a8, 76 },
    { 0, 0 },
    { 0x2cbab5aa, 61 },
    { 0xbb9ab5aa, 33 },
    { 0x592ab5aa, 90 },
    { 0xa68ab5aa, 51 },
    { 0x6c5ab5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241ab5b7, 91 },
    { 0xde7ab5b7, 78 },
    { 0, 0 },
    { 0x9b2ab5ba, 88 },
    { 0xa4eab5ba, 90 },
    { 0xf9fab5bc, 76 },
    { 0, 0 },
    { 0xb8dab5be, 90 },
    { 0xd60ab5be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bab5c2, 90 },
    { 0x61cab5c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dab5c6, 90 },
    { 0x540ab5c6, 90 },
    { 0x857ab5c8, 90 },
    { 0xd4fab5c8, 34 },
    { 0x1eab5ca, 52 },
    { 0x314ab5ca, 66 },
    { 0x891ab5ca, 95 },
    { 0xd8cab5cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2ab5d2, 33 },
    { 0, 0 },
    { 0xdf1ab5d4, 100 },
    { 0xdedab5d5, 23 },
    { 0, 0 },
    { 0x8aaab5d7, 33 },
    { 0, 0 },
    { 0xe63ab5d9, 66 },
    { 0, 0 },
    { 0x4fdab5db, 84 },
    { 0x690ab5dc, 33 },
    { 0x4b3ab5dd, 76 },
    { 0, 0 },
    { 0xa69ab5df, 33 },
    { 0, 0 },
    { 0x356ab5e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ab5e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faab5eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89ab5ef, 95 },
    { 0xbcaab5f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daab5f5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x776ab5f8, 78 },
    { 0xa22ab5f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4ab5fc, 84 },
    { 0, 0 },
    { 0x88bab5fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69ab601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ab607, 98 },
    { 0x4afab607, 76 },
    { 0xfc9ab608, 55 },
    { 0x65cab60a, 33 },
    { 0, 0 },
    { 0x542ab60c, 34 },
    { 0, 0 },
    { 0xf52ab60e, 87 },
    { 0x23dab60f, 62 },
    { 0x6e0ab60f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3ab613, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acab616, 55 },
    { 0, 0 },
    { 0x6f9ab618, 98 },
    { 0xdf3ab618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ab61e, 36 },
    { 0xda2ab61f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2ab622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dab629, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17ab62f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938ab637, 98 },
    { 0xc4dab638, 61 },
    { 0x452ab639, 61 },
    { 0, 0 },
    { 0xcfdab63b, 33 },
    { 0x95aab63b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8ab63f, 90 },
    { 0x78aab640, 98 },
    { 0x734ab640, 17 },
    { 0x6e1ab641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x400ab645, 17 },
    { 0x472ab645, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633ab64a, 23 },
    { 0, 0 },
    { 0xdfdab64c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dab650, 98 },
    { 0, 0 },
    { 0xcab652, 43 },
    { 0x6e3ab653, 100 },
    { 0x9a1ab654, 33 },
    { 0x5f2ab652, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250ab65a, 90 },
    { 0, 0 },
    { 0x3eeab65c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1ab65f, 95 },
    { 0xfffab660, 98 },
    { 0, 0 },
    { 0x49aab662, 46 },
    { 0xd40ab663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70eab667, 33 },
    { 0x6f1ab668, 61 },
    { 0xa77ab668, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ab66e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71ab672, 90 },
    { 0x532ab672, 90 },
    { 0xb03ab674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dab677, 98 },
    { 0x2ccab678, 55 },
    { 0x8a5ab678, 34 },
    { 0x9edab679, 95 },
    { 0xdd0ab679, 17 },
    { 0xfaab67a, 90 },
    { 0x580ab678, 76 },
    { 0xa35ab67e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412ab684, 76 },
    { 0, 0 },
    { 0x9cfab686, 23 },
    { 0xf41ab686, 55 },
    { 0x1c6ab688, 98 },
    { 0, 0 },
    { 0x22dab68a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaab68f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5ab695, 93 },
    { 0xe41ab696, 33 },
    { 0x94cab697, 95 },
    { 0x21fab696, 23 },
    { 0x824ab699, 23 },
    { 0x4d1ab699, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaab69e, 98 },
    { 0x26aab69f, 76 },
    { 0x5aab69e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03ab6a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7ab6ad, 61 },
    { 0xae1ab6ad, 23 },
    { 0xeceab6af, 52 },
    { 0x47bab6b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2ab6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfab6c5, 33 },
    { 0, 0 },
    { 0xebbab6c7, 95 },
    { 0x44cab6c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fab6cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ab6d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fab6d5, 23 },
    { 0, 0 },
    { 0x1c6ab6d7, 33 },
    { 0xd17ab6d8, 23 },
    { 0, 0 },
    { 0xcc6ab6da, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdab6e3, 46 },
    { 0, 0 },
    { 0x913ab6e5, 76 },
    { 0xd36ab6e6, 55 },
    { 0x3d6ab6e6, 76 },
    { 0, 0 },
    { 0x12cab6e9, 34 },
    { 0x510ab6e9, 98 },
    { 0x318ab6eb, 61 },
    { 0, 0 },
    { 0x823ab6ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x396ab6f0, 33 },
    { 0x71eab6f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4ab6f4, 67 },
    { 0x10eab6f4, 61 },
    { 0x497ab6f6, 77 },
    { 0x682ab6f7, 61 },
    { 0xaa2ab6f6, 90 },
    { 0x56bab6f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4ab6fc, 46 },
    { 0xcedab6fc, 76 },
    { 0, 0 },
    { 0xcfab6ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658ab704, 87 },
    { 0x9e2ab704, 61 },
    { 0x5b0ab704, 65 },
    { 0x6d3ab707, 90 },
    { 0, 0 },
    { 0x510ab709, 23 },
    { 0x32aab70a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68ab70e, 69 },
    { 0x15eab70f, 90 },
    { 0x61cab710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56ab71b, 17 },
    { 0x5b9ab71b, 93 },
    { 0x236ab71d, 33 },
    { 0, 0 },
    { 0x76fab71f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdab723, 23 },
    { 0xd78ab723, 52 },
    { 0x158ab725, 17 },
    { 0xa01ab725, 33 },
    { 0xb6eab723, 95 },
    { 0xf7dab728, 43 },
    { 0, 0 },
    { 0xe8aab72a, 95 },
    { 0, 0 },
    { 0xdddab72c, 98 },
    { 0xe75ab72d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42aab731, 91 },
    { 0xc3eab732, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ab735, 55 },
    { 0, 0 },
    { 0x350ab737, 33 },
    { 0x636ab738, 91 },
    { 0, 0 },
    { 0xd66ab73a, 33 },
    { 0xe62ab73b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3ab73e, 90 },
    { 0x68dab73f, 33 },
    { 0xbe5ab740, 33 },
    { 0, 0 },
    { 0xea1ab742, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x802ab745, 98 },
    { 0x528ab746, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fab74a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feab750, 8 },
    { 0x701ab751, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x919ab754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadab757, 84 },
    { 0, 0 },
    { 0xbfab759, 46 },
    { 0, 0 },
    { 0xcf6ab75b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546ab770, 34 },
    { 0xd3fab770, 81 },
    { 0xeb1ab772, 78 },
    { 0, 0 },
    { 0xb2eab774, 67 },
    { 0, 0 },
    { 0xdd6ab776, 93 },
    { 0, 0 },
    { 0xd11ab778, 90 },
    { 0x7e0ab778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676ab77d, 33 },
    { 0x28bab77e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cab786, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71ab789, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42ab78d, 85 },
    { 0, 0 },
    { 0x50cab78f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x716ab792, 91 },
    { 0xd3ab793, 98 },
    { 0, 0 },
    { 0xb5bab795, 33 },
    { 0x669ab796, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6ab79a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab79d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cab7a1, 33 },
    { 0, 0 },
    { 0xa5cab7a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18ab7a8, 98 },
    { 0x412ab7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x326ab7ac, 85 },
    { 0xbc3ab7ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74ab7b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649ab7b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x962ab7b9, 78 },
    { 0, 0 },
    { 0x900ab7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dab7be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77ab7c4, 43 },
    { 0x810ab7c5, 75 },
    { 0x73eab7c6, 23 },
    { 0, 0 },
    { 0x83bab7c8, 33 },
    { 0x249ab7c8, 98 },
    { 0x786ab7ca, 55 },
    { 0xc93ab7cb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bab7d0, 95 },
    { 0xa01ab7d0, 90 },
    { 0, 0 },
    { 0x80cab7d3, 84 },
    { 0x234ab7d4, 17 },
    { 0xb72ab7d5, 33 },
    { 0x9e6ab7d3, 23 },
    { 0x862ab7d7, 36 },
    { 0xb8ab7d7, 90 },
    { 0xa4aab7d9, 95 },
    { 0, 0 },
    { 0x225ab7db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fab7df, 61 },
    { 0xbc7ab7e0, 91 },
    { 0xfd7ab7e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab7e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abab7ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41ab7f3, 43 },
    { 0x8f8ab7f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbab7f8, 23 },
    { 0x1d8ab7f9, 34 },
    { 0, 0 },
    { 0xe44ab7fb, 61 },
    { 0, 0 },
    { 0xa55ab7fd, 61 },
    { 0xe26ab7fe, 61 },
    { 0xf5fab7fe, 61 },
    { 0x526ab800, 90 },
    { 0x7b8ab801, 36 },
    { 0x880ab800, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ab805, 33 },
    { 0, 0 },
    { 0x8bfab807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64ab80c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aab80f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ab816, 98 },
    { 0xf05ab817, 61 },
    { 0x3a1ab818, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dab81f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8ab825, 49 },
    { 0x360ab825, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x988ab829, 34 },
    { 0, 0 },
    { 0x81cab82b, 33 },
    { 0, 0 },
    { 0x2e2ab82d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bab832, 65 },
    { 0, 0 },
    { 0x7eaab834, 87 },
    { 0x2c2ab835, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fab83a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ab83e, 78 },
    { 0, 0 },
    { 0x565ab840, 61 },
    { 0, 0 },
    { 0xe21ab842, 34 },
    { 0xa16ab843, 55 },
    { 0x3bdab844, 33 },
    { 0x699ab845, 33 },
    { 0x746ab846, 95 },
    { 0x95dab843, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdab84a, 90 },
    { 0x5bbab84b, 76 },
    { 0xa64ab84c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33ab84f, 91 },
    { 0x531ab850, 23 },
    { 0x60dab851, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfab854, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x484ab857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2ab85c, 66 },
    { 0xa4eab85d, 52 },
    { 0xe3cab85e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ab861, 43 },
    { 0x9c1ab862, 33 },
    { 0, 0 },
    { 0xb62ab864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adab86c, 33 },
    { 0xd8fab86d, 93 },
    { 0x423ab86d, 67 },
    { 0x8c3ab86d, 23 },
    { 0xa66ab86e, 91 },
    { 0x2ceab870, 23 },
    { 0xa61ab871, 61 },
    { 0, 0 },
    { 0x7bab874, 46 },
    { 0x282ab875, 33 },
    { 0xc9bab875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01ab87d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fab882, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276ab889, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaab88e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x290ab891, 76 },
    { 0, 0 },
    { 0xe40ab893, 65 },
    { 0x609ab894, 98 },
    { 0x3c6ab893, 90 },
    { 0x1bdab895, 90 },
    { 0xa24ab893, 33 },
    { 0x2c8ab895, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42ab89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4ab89e, 33 },
    { 0xe36ab89e, 46 },
    { 0xd50ab8a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fab8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5aab8a9, 43 },
    { 0x7c0ab8aa, 34 },
    { 0x4ceab8ab, 90 },
    { 0, 0 },
    { 0x8d7ab8ad, 61 },
    { 0xdfeab8ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beab8b4, 23 },
    { 0xb1dab8b5, 66 },
    { 0, 0 },
    { 0xd3aab8b7, 66 },
    { 0xb2eab8b8, 65 },
    { 0x125ab8b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bab8bf, 95 },
    { 0xaaaab8bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x389ab8c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698ab8c8, 23 },
    { 0xe85ab8c9, 90 },
    { 0x747ab8ca, 61 },
    { 0x7bdab8cb, 46 },
    { 0x487ab8ca, 90 },
    { 0x7bfab8cd, 61 },
    { 0, 0 },
    { 0x279ab8cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791ab8d3, 76 },
    { 0xe78ab8d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986ab8d9, 92 },
    { 0x20bab8da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454ab8df, 23 },
    { 0x2a7ab8e0, 76 },
    { 0, 0 },
    { 0xb8cab8e2, 90 },
    { 0, 0 },
    { 0x566ab8e4, 98 },
    { 0x79aab8e5, 52 },
    { 0xcb3ab8e5, 55 },
    { 0x953ab8e5, 46 },
    { 0, 0 },
    { 0x5c0ab8e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ab8ee, 33 },
    { 0x104ab8ee, 23 },
    { 0xab8ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1ab8f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfab8fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2ab8fd, 76 },
    { 0xef4ab8fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4ab903, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cab907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cab90b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ab90f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837ab918, 23 },
    { 0, 0 },
    { 0xd1ab91a, 33 },
    { 0x593ab91b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aab91e, 90 },
    { 0x17dab91f, 66 },
    { 0x322ab920, 76 },
    { 0, 0 },
    { 0xdb1ab922, 61 },
    { 0, 0 },
    { 0x3b8ab924, 95 },
    { 0xa0dab924, 91 },
    { 0x862ab926, 85 },
    { 0xda3ab926, 55 },
    { 0xca4ab928, 99 },
    { 0xc77ab929, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27ab930, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ab933, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeeab938, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bab940, 91 },
    { 0, 0 },
    { 0x6ab942, 61 },
    { 0, 0 },
    { 0x480ab944, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1ab94a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cab94f, 90 },
    { 0xf47ab950, 95 },
    { 0, 0 },
    { 0x43bab952, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686ab957, 90 },
    { 0x9bdab957, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cab95b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeab963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ab966, 93 },
    { 0xe62ab967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777ab96e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adab971, 90 },
    { 0x1aaab971, 90 },
    { 0x8f4ab971, 17 },
    { 0xf73ab974, 93 },
    { 0x82fab973, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaab97a, 76 },
    { 0, 0 },
    { 0x950ab97c, 61 },
    { 0xfb5ab97d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43ab982, 90 },
    { 0, 0 },
    { 0x4c3ab984, 76 },
    { 0xafab985, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7ab98b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fab98e, 61 },
    { 0, 0 },
    { 0xe5cab990, 33 },
    { 0xaeaab990, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ab997, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ab99c, 23 },
    { 0, 0 },
    { 0x4b7ab99e, 100 },
    { 0x795ab99f, 90 },
    { 0x190ab9a0, 99 },
    { 0, 0 },
    { 0x715ab9a2, 84 },
    { 0x4faab9a3, 88 },
    { 0, 0 },
    { 0x86bab9a5, 3 },
    { 0, 0 },
    { 0x4e3ab9a7, 97 },
    { 0, 0 },
    { 0xc6aab9a9, 88 },
    { 0xdcdab9a9, 69 },
    { 0xb47ab9ab, 33 },
    { 0xb84ab9ac, 85 },
    { 0x6fab9ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4ab9b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afab9bd, 98 },
    { 0xb1eab9be, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x249ab9c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcab9ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5ab9cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547ab9d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252ab9d5, 67 },
    { 0, 0 },
    { 0x76bab9d7, 90 },
    { 0, 0 },
    { 0xbc6ab9d9, 34 },
    { 0, 0 },
    { 0x971ab9db, 52 },
    { 0xc64ab9dc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cab9e3, 68 },
    { 0, 0 },
    { 0xc1eab9e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ab9e9, 90 },
    { 0xfb7ab9e9, 98 },
    { 0x993ab9eb, 76 },
    { 0, 0 },
    { 0xf41ab9ed, 64 },
    { 0xe43ab9ee, 61 },
    { 0xb24ab9ee, 98 },
    { 0x5d0ab9ee, 76 },
    { 0, 0 },
    { 0xaf0ab9f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6ab9f7, 90 },
    { 0xbd4ab9f8, 33 },
    { 0x38eab9f8, 61 },
    { 0, 0 },
    { 0x29cab9fb, 66 },
    { 0xbc3ab9fc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28aba02, 90 },
    { 0x8c2aba03, 90 },
    { 0, 0 },
    { 0x225aba05, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0aba0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aaba0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaaba12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390aba16, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9aba1a, 98 },
    { 0, 0 },
    { 0x623aba1c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643aba20, 81 },
    { 0xcbfaba21, 65 },
    { 0x758aba22, 90 },
    { 0xb15aba21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19aba28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57aba2b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65aba30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529aba36, 90 },
    { 0, 0 },
    { 0x14baba38, 95 },
    { 0xf7aba39, 76 },
    { 0x8c0aba3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5caba44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642aba48, 34 },
    { 0, 0 },
    { 0xe02aba4a, 95 },
    { 0xaf7aba4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283aba4f, 95 },
    { 0x200aba50, 61 },
    { 0xe46aba51, 90 },
    { 0x436aba51, 94 },
    { 0xc99aba51, 61 },
    { 0xea8aba54, 33 },
    { 0, 0 },
    { 0x307aba56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0aba5c, 90 },
    { 0xf91aba5d, 61 },
    { 0x407aba5d, 36 },
    { 0xcbdaba5f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47caba6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1faba73, 94 },
    { 0, 0 },
    { 0xee2aba75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9aba7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0aba83, 33 },
    { 0xb8daba84, 33 },
    { 0x5adaba83, 95 },
    { 0xf74aba84, 34 },
    { 0xa96aba87, 77 },
    { 0xf36aba88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90aba8b, 23 },
    { 0x31eaba8c, 33 },
    { 0x86baba8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbaba91, 90 },
    { 0, 0 },
    { 0x732aba93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76aba9f, 23 },
    { 0x2b2abaa0, 90 },
    { 0x283abaa1, 98 },
    { 0x358abaa1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86abaa7, 100 },
    { 0x7e2abaa7, 83 },
    { 0x98dabaa9, 78 },
    { 0x350abaa9, 55 },
    { 0xbb8abaab, 61 },
    { 0xaababaac, 83 },
    { 0x105abaab, 61 },
    { 0xa1fabaa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765abab5, 67 },
    { 0xaf7abab5, 98 },
    { 0x4aaabab7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feababb, 98 },
    { 0xbfeababc, 98 },
    { 0x24cababc, 95 },
    { 0xb40ababe, 98 },
    { 0x4b5ababe, 90 },
    { 0, 0 },
    { 0xf7babac1, 90 },
    { 0x1a7abac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5abac9, 98 },
    { 0x5f4abaca, 56 },
    { 0xc06abaca, 93 },
    { 0, 0 },
    { 0x9faabacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910abad1, 92 },
    { 0x629abad2, 98 },
    { 0xab7abad3, 33 },
    { 0x74cabad4, 77 },
    { 0x79eabad5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4abada, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30eabadd, 90 },
    { 0x5f6abade, 98 },
    { 0x42fabadf, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2abae4, 76 },
    { 0xdb0abae5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5abae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eabaf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41aabafd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4abb09, 33 },
    { 0xb70abb0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fabb0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6abb10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26abb16, 93 },
    { 0, 0 },
    { 0x584abb18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2abb1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cabb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fabb32, 34 },
    { 0x720abb33, 23 },
    { 0x643abb34, 95 },
    { 0xf30abb35, 88 },
    { 0xf3cabb35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595abb41, 55 },
    { 0, 0 },
    { 0x524abb43, 50 },
    { 0xc21abb44, 61 },
    { 0x315abb45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fabb49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8abb4e, 23 },
    { 0x790abb4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aabb53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2abb59, 90 },
    { 0xa0eabb5a, 34 },
    { 0x5e5abb5b, 91 },
    { 0xf59abb5c, 90 },
    { 0xa8fabb5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68abb61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41abb65, 93 },
    { 0, 0 },
    { 0xb14abb67, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86abb6b, 23 },
    { 0x856abb6c, 61 },
    { 0x90eabb6d, 61 },
    { 0x9d4abb6e, 61 },
    { 0x14dabb6f, 98 },
    { 0x613abb6e, 34 },
    { 0x476abb71, 81 },
    { 0x1e9abb6f, 98 },
    { 0xf70abb6f, 95 },
    { 0x1ebabb74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83abb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9abb7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4abb85, 85 },
    { 0xbcdabb85, 23 },
    { 0x12cabb86, 76 },
    { 0x699abb88, 96 },
    { 0x187abb89, 92 },
    { 0, 0 },
    { 0xdbdabb8b, 90 },
    { 0x598abb8b, 84 },
    { 0, 0 },
    { 0x9b5abb8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46abb96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902abb9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3abb9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afabba9, 52 },
    { 0, 0 },
    { 0xcc8abbab, 90 },
    { 0xceaabbab, 22 },
    { 0, 0 },
    { 0xb2abbae, 76 },
    { 0, 0 },
    { 0x4c1abbb0, 44 },
    { 0x808abbb1, 46 },
    { 0, 0 },
    { 0x36cabbb3, 33 },
    { 0xea6abbb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2abbba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efabbbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4abbc0, 23 },
    { 0xee0abbc0, 61 },
    { 0x784abbc0, 61 },
    { 0x389abbc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dabbcb, 33 },
    { 0x4eeabbcc, 95 },
    { 0x366abbcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cabbd1, 90 },
    { 0x37abbd1, 81 },
    { 0, 0 },
    { 0x99fabbd4, 33 },
    { 0, 0 },
    { 0x92fabbd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25abbda, 61 },
    { 0x97cabbdb, 90 },
    { 0xe28abbdb, 43 },
    { 0x926abbdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dabbe0, 93 },
    { 0, 0 },
    { 0xed7abbe2, 64 },
    { 0, 0 },
    { 0x841abbe4, 90 },
    { 0x643abbe5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5abbe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9abbee, 33 },
    { 0xc7dabbee, 43 },
    { 0x51cabbf0, 61 },
    { 0x25dabbf0, 99 },
    { 0x92abbf2, 33 },
    { 0xdf4abbf2, 98 },
    { 0x1a0abbf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47abbfa, 40 },
    { 0, 0 },
    { 0x88babbfc, 90 },
    { 0xa69abbfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeabc01, 95 },
    { 0, 0 },
    { 0xdd1abc03, 76 },
    { 0, 0 },
    { 0xe2fabc05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbabc08, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x723abc0b, 95 },
    { 0, 0 },
    { 0x2d6abc0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745abc16, 61 },
    { 0, 0 },
    { 0xc0abc18, 90 },
    { 0x31babc18, 90 },
    { 0, 0 },
    { 0x505abc1b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x41eabc1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adabc24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279abc29, 61 },
    { 0x10cabc2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87babc2e, 55 },
    { 0, 0 },
    { 0x771abc30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559abc34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefabc37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54abc3b, 34 },
    { 0xd3abc3c, 33 },
    { 0, 0 },
    { 0x7daabc3e, 90 },
    { 0xb25abc3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3abc48, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fabc4c, 95 },
    { 0x2c6abc4d, 76 },
    { 0xb8abc4e, 23 },
    { 0x87aabc4f, 46 },
    { 0, 0 },
    { 0x91dabc51, 90 },
    { 0xf04abc51, 95 },
    { 0x856abc52, 33 },
    { 0, 0 },
    { 0x9f8abc55, 43 },
    { 0xa40abc56, 90 },
    { 0x96dabc57, 61 },
    { 0xc0babc57, 95 },
    { 0xa50abc58, 46 },
    { 0x1fdabc5a, 78 },
    { 0x540abc5b, 91 },
    { 0x243abc5b, 88 },
    { 0x13aabc5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63eabc64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdabc67, 66 },
    { 0xb9fabc67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2abc6b, 98 },
    { 0xebfabc6c, 90 },
    { 0xabfabc6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fabc73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427abc79, 95 },
    { 0x14babc7a, 90 },
    { 0xaebabc7b, 77 },
    { 0xc83abc7c, 66 },
    { 0x66cabc7a, 61 },
    { 0xedeabc7e, 43 },
    { 0xb32abc7f, 84 },
    { 0x461abc80, 23 },
    { 0x84eabc81, 55 },
    { 0x26fabc82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6aabc85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0abc89, 33 },
    { 0, 0 },
    { 0xcbbabc8b, 33 },
    { 0xb32abc8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6abc8f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x984abc92, 84 },
    { 0, 0 },
    { 0x1efabc94, 12 },
    { 0x3d7abc94, 98 },
    { 0xf21abc96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6abc9b, 87 },
    { 0xa70abc9b, 36 },
    { 0x5fdabc9c, 90 },
    { 0x42dabc9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cabca2, 61 },
    { 0x1c6abca3, 93 },
    { 0, 0 },
    { 0xdf2abca5, 92 },
    { 0xf41abca6, 33 },
    { 0xdbabca7, 23 },
    { 0xa67abca7, 98 },
    { 0x74cabca7, 23 },
    { 0xeb6abca7, 95 },
    { 0, 0 },
    { 0xb46abcac, 33 },
    { 0xbb4abcad, 34 },
    { 0x60aabcae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6abcb1, 33 },
    { 0xed4abcb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38abcb5, 66 },
    { 0, 0 },
    { 0xb97abcb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeabcba, 43 },
    { 0, 0 },
    { 0xc35abcbc, 33 },
    { 0x628abcbd, 55 },
    { 0xa26abcbe, 87 },
    { 0x43cabcbc, 43 },
    { 0xeffabcc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecabcc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x767abcc7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefabccf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcabcd4, 33 },
    { 0x35babcd4, 98 },
    { 0xc55abcd6, 55 },
    { 0xd3babcd4, 78 },
    { 0xa1abcd6, 55 },
    { 0xe4cabcd9, 76 },
    { 0xb9fabcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368abcde, 52 },
    { 0, 0 },
    { 0x3b3abce0, 23 },
    { 0x3c0abce1, 43 },
    { 0, 0 },
    { 0xed8abce3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fabce6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7abced, 76 },
    { 0, 0 },
    { 0x3deabcef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7abcf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0abcfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aabd01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91abd05, 99 },
    { 0xcb1abd06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56abd0f, 61 },
    { 0, 0 },
    { 0x4aeabd11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cabd18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76abd21, 91 },
    { 0x71dabd22, 33 },
    { 0x579abd23, 90 },
    { 0xe48abd22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eabd27, 61 },
    { 0x728abd27, 34 },
    { 0, 0 },
    { 0x4abd2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdabd30, 76 },
    { 0, 0 },
    { 0x8c1abd32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x999abd35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303abd39, 90 },
    { 0xf90abd39, 23 },
    { 0xa41abd3b, 95 },
    { 0xefabd39, 76 },
    { 0xa72abd3c, 76 },
    { 0xbfabd3c, 77 },
    { 0, 0 },
    { 0xf6eabd40, 83 },
    { 0xc6eabd41, 85 },
    { 0xb2abd42, 95 },
    { 0x9d5abd43, 90 },
    { 0x716abd43, 55 },
    { 0x90abd43, 76 },
    { 0x5e6abd43, 23 },
    { 0, 0 },
    { 0x647abd48, 33 },
    { 0, 0 },
    { 0xddabd4a, 95 },
    { 0, 0 },
    { 0x325abd4c, 34 },
    { 0xd89abd4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3abd52, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x77abd55, 98 },
    { 0x80dabd56, 46 },
    { 0xa26abd56, 90 },
    { 0x2bbabd58, 55 },
    { 0xa06abd59, 85 },
    { 0xeceabd5a, 95 },
    { 0xfdabd5a, 43 },
    { 0xdceabd57, 33 },
    { 0x4cfabd5d, 23 },
    { 0x379abd5d, 56 },
    { 0x60eabd58, 61 },
    { 0x483abd5c, 90 },
    { 0x603abd61, 90 },
    { 0x97dabd61, 90 },
    { 0x123abd63, 33 },
    { 0x55abd64, 43 },
    { 0x677abd64, 61 },
    { 0x3aeabd64, 61 },
    { 0xa7cabd67, 69 },
    { 0x6c4abd68, 76 },
    { 0x84fabd63, 90 },
    { 0xef3abd6a, 23 },
    { 0, 0 },
    { 0xd9fabd6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3abd71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89abd74, 34 },
    { 0xfecabd75, 94 },
    { 0x401abd76, 100 },
    { 0x58fabd76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cabd7a, 66 },
    { 0, 0 },
    { 0x1c9abd7c, 91 },
    { 0x1c0abd7d, 46 },
    { 0, 0 },
    { 0x134abd7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818abd83, 98 },
    { 0x171abd84, 98 },
    { 0x560abd83, 96 },
    { 0x3b7abd83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dabd8e, 98 },
    { 0, 0 },
    { 0x95babd90, 33 },
    { 0xf7cabd90, 66 },
    { 0, 0 },
    { 0xd3eabd93, 61 },
    { 0xb7cabd94, 90 },
    { 0, 0 },
    { 0x456abd96, 33 },
    { 0x737abd97, 88 },
    { 0x648abd98, 34 },
    { 0, 0 },
    { 0xb96abd9a, 93 },
    { 0x989abd9a, 46 },
    { 0x363abd9b, 55 },
    { 0xb42abd9d, 95 },
    { 0, 0 },
    { 0x42eabd9f, 88 },
    { 0x1adabd9f, 95 },
    { 0xb2babda1, 98 },
    { 0x4caabda2, 98 },
    { 0xacdabda3, 85 },
    { 0, 0 },
    { 0x275abda5, 99 },
    { 0x696abda6, 98 },
    { 0xc6eabda7, 33 },
    { 0, 0 },
    { 0xd04abda9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111abdad, 100 },
    { 0x84babdad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854abdb2, 65 },
    { 0, 0 },
    { 0xe98abdb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x142abdb7, 61 },
    { 0x7d8abdb7, 61 },
    { 0xd5abdb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609abdbe, 95 },
    { 0x44eabdbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55abdc2, 55 },
    { 0, 0 },
    { 0x604abdc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbabdc8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3abdcf, 34 },
    { 0, 0 },
    { 0xf0dabdd1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fabdd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2abdd9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fabddf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aabde8, 69 },
    { 0, 0 },
    { 0x515abdea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dabdf0, 34 },
    { 0, 0 },
    { 0x460abdf2, 72 },
    { 0x82fabdf3, 87 },
    { 0xcb5abdf3, 43 },
    { 0x38cabdf5, 99 },
    { 0x5a9abdf3, 55 },
    { 0xf64abdf2, 23 },
    { 0, 0 },
    { 0xe60abdf9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x312abdfc, 66 },
    { 0xdaeabdfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cabe05, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabe08, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85abe0d, 61 },
    { 0xc95abe0e, 33 },
    { 0xc25abe0f, 84 },
    { 0x709abe10, 61 },
    { 0x2b7abe11, 90 },
    { 0x1c7abe12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cabe15, 43 },
    { 0x309abe16, 66 },
    { 0xd6cabe16, 76 },
    { 0xf23abe17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5abe1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880abe23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edabe27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dabe2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844abe34, 17 },
    { 0, 0 },
    { 0x21babe36, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaabe39, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eabe3c, 76 },
    { 0x115abe3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x88babe40, 61 },
    { 0, 0 },
    { 0x49dabe42, 100 },
    { 0x44eabe42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6abe47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3abe4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1abe50, 90 },
    { 0xac9abe51, 33 },
    { 0x2eabe52, 55 },
    { 0x3ffabe53, 61 },
    { 0x82dabe50, 95 },
    { 0x96eabe50, 98 },
    { 0, 0 },
    { 0x2adabe57, 90 },
    { 0x27cabe58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8abe5b, 90 },
    { 0x5caabe5b, 95 },
    { 0, 0 },
    { 0xa2cabe5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x498abe61, 33 },
    { 0x62fabe62, 61 },
    { 0xb87abe61, 90 },
    { 0xb63abe64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563abe6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23abe6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eabe73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4abe7e, 90 },
    { 0x57cabe7f, 61 },
    { 0x1d5abe7e, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639abe84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dabe89, 76 },
    { 0, 0 },
    { 0xab6abe8b, 52 },
    { 0, 0 },
    { 0xae2abe8d, 98 },
    { 0, 0 },
    { 0x2e6abe8f, 90 },
    { 0xd5babe90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dabe94, 66 },
    { 0x7d5abe94, 91 },
    { 0x95abe96, 95 },
    { 0, 0 },
    { 0xc79abe98, 93 },
    { 0xf06abe98, 78 },
    { 0xf4abe99, 61 },
    { 0x353abe98, 23 },
    { 0, 0 },
    { 0x9fbabe9d, 85 },
    { 0, 0 },
    { 0x520abe9f, 61 },
    { 0x299abe9f, 43 },
    { 0x446abea1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88abea5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97abeaa, 95 },
    { 0x188abeab, 93 },
    { 0x62abeac, 33 },
    { 0xd90abeab, 33 },
    { 0x58cabeae, 95 },
    { 0xcdaabeaa, 61 },
    { 0x30cabeb0, 98 },
    { 0xeaeabeb0, 55 },
    { 0xe95abeb2, 33 },
    { 0, 0 },
    { 0xd06abeb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91babeb8, 52 },
    { 0xe0fabeb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53abebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffabec2, 23 },
    { 0, 0 },
    { 0x9d6abec4, 61 },
    { 0x3b8abec5, 33 },
    { 0xfe7abec6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fabecd, 61 },
    { 0x2d5abecd, 55 },
    { 0, 0 },
    { 0x682abed0, 77 },
    { 0, 0 },
    { 0x8b4abed2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abed6, 61 },
    { 0xd49abed7, 95 },
    { 0x83eabed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfabedd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05abee1, 43 },
    { 0xae0abee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463abeec, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927abef3, 95 },
    { 0x1d3abef4, 93 },
    { 0, 0 },
    { 0xf32abef6, 55 },
    { 0, 0 },
    { 0xbbbabef8, 95 },
    { 0x506abef8, 90 },
    { 0x837abefa, 33 },
    { 0x9ffabefa, 90 },
    { 0x993abefc, 43 },
    { 0x494abefb, 98 },
    { 0xe1abefe, 90 },
    { 0xc44abeff, 23 },
    { 0xce0abf00, 23 },
    { 0xb38abf01, 46 },
    { 0xf6fabf02, 93 },
    { 0x61babf03, 33 },
    { 0xa1eabf03, 99 },
    { 0x336abf05, 99 },
    { 0xaa7abf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957abf0b, 84 },
    { 0xe56abf0c, 96 },
    { 0xa98abf0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5abf10, 61 },
    { 0, 0 },
    { 0xeddabf12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3abf18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07abf1e, 81 },
    { 0x5a6abf1e, 90 },
    { 0xffbabf1e, 76 },
    { 0x9d6abf21, 34 },
    { 0, 0 },
    { 0x19abf23, 55 },
    { 0xfdaabf23, 52 },
    { 0, 0 },
    { 0x4edabf26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5abf2b, 55 },
    { 0, 0 },
    { 0x140abf2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccabf30, 33 },
    { 0xdbeabf31, 90 },
    { 0x785abf31, 95 },
    { 0x26cabf33, 90 },
    { 0x76babf34, 90 },
    { 0x726abf32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574abf40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420abf4c, 80 },
    { 0xf2abf4d, 33 },
    { 0, 0 },
    { 0x18babf4f, 90 },
    { 0x6c7abf50, 98 },
    { 0, 0 },
    { 0x4ecabf52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeabf57, 64 },
    { 0x15cabf58, 76 },
    { 0, 0 },
    { 0xda1abf5a, 75 },
    { 0x29dabf5b, 9 },
    { 0, 0 },
    { 0x5b7abf5d, 33 },
    { 0x610abf5d, 98 },
    { 0x40babf5f, 76 },
    { 0, 0 },
    { 0x772abf61, 46 },
    { 0x464abf62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77abf67, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cabf6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6abf73, 76 },
    { 0xb8babf74, 91 },
    { 0, 0 },
    { 0xb3babf76, 76 },
    { 0xf98abf76, 95 },
    { 0x99fabf78, 93 },
    { 0x678abf76, 88 },
    { 0x887abf78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0babf7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x962abf81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1abf85, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52abf8b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fabf91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x947abf94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0abf9a, 90 },
    { 0, 0 },
    { 0x178abf9c, 90 },
    { 0x312abf9d, 91 },
    { 0x8cbabf9e, 33 },
    { 0x24eabf9f, 90 },
    { 0x541abf9e, 75 },
    { 0xd0aabfa1, 98 },
    { 0xcf0abfa1, 61 },
    { 0xcfaabfa2, 90 },
    { 0x502abfa2, 33 },
    { 0x15cabf9f, 93 },
    { 0x6bbabfa5, 98 },
    { 0xdbabfa7, 95 },
    { 0x94eabfa7, 12 },
    { 0, 0 },
    { 0x385abfaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124abfb2, 87 },
    { 0x5e6abfb3, 23 },
    { 0xb1dabfb4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4aabfb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2abfbc, 43 },
    { 0xaf7abfbd, 85 },
    { 0x2d6abfbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eabfc9, 23 },
    { 0x65babfc9, 23 },
    { 0, 0 },
    { 0xb8eabfcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddabfd0, 98 },
    { 0, 0 },
    { 0xa7fabfd2, 98 },
    { 0, 0 },
    { 0x60abfd4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2abfd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01abfdc, 93 },
    { 0x9f0abfdc, 100 },
    { 0, 0 },
    { 0x85fabfdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cabfe5, 85 },
    { 0x94cabfe5, 90 },
    { 0xfb9abfe7, 23 },
    { 0x99abfe7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6abfee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35abff4, 23 },
    { 0x87dabff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eabffb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aabffe, 33 },
    { 0x8d2abfff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbac003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2ac007, 47 },
    { 0x721ac008, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55ac00b, 66 },
    { 0x4b4ac00c, 90 },
    { 0x71ac00d, 23 },
    { 0x398ac00e, 66 },
    { 0x2f2ac00e, 98 },
    { 0x92dac00f, 33 },
    { 0, 0 },
    { 0xad0ac012, 99 },
    { 0x58aac013, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34ac016, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac01a, 61 },
    { 0xdebac01b, 91 },
    { 0x228ac01a, 91 },
    { 0x3f6ac01d, 98 },
    { 0x28fac01d, 90 },
    { 0x3f4ac01f, 34 },
    { 0xbdbac01c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cac023, 23 },
    { 0, 0 },
    { 0x918ac025, 76 },
    { 0x926ac026, 93 },
    { 0, 0 },
    { 0x725ac028, 98 },
    { 0xbb1ac029, 81 },
    { 0x363ac02a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9ac032, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x855ac035, 52 },
    { 0xd6aac035, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554ac03d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afac042, 23 },
    { 0xbedac043, 61 },
    { 0xa17ac044, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889ac04d, 40 },
    { 0xe51ac04e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46ac051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816ac055, 87 },
    { 0xcb0ac056, 56 },
    { 0xbb7ac057, 40 },
    { 0xe6dac058, 98 },
    { 0x154ac057, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ac05d, 33 },
    { 0, 0 },
    { 0x968ac05f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bac063, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0ac066, 98 },
    { 0xc2aac066, 23 },
    { 0xed8ac066, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35aac06c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ac06f, 23 },
    { 0x5c2ac070, 98 },
    { 0xd91ac070, 99 },
    { 0x348ac072, 33 },
    { 0xb2eac073, 90 },
    { 0xa2aac074, 55 },
    { 0xb9dac075, 90 },
    { 0xac1ac076, 36 },
    { 0, 0 },
    { 0x291ac078, 33 },
    { 0, 0 },
    { 0xc05ac07a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bac07d, 55 },
    { 0xbcaac07e, 87 },
    { 0, 0 },
    { 0xb9cac080, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dac083, 23 },
    { 0x41bac084, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17aac089, 78 },
    { 0xfd2ac08a, 93 },
    { 0x3b6ac08a, 98 },
    { 0xc73ac08b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fac08f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33ac094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679ac098, 34 },
    { 0, 0 },
    { 0xd44ac09a, 98 },
    { 0x43bac09b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15ac0a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6ac0a4, 46 },
    { 0x499ac0a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93ac0a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aac0ac, 46 },
    { 0x814ac0ac, 84 },
    { 0, 0 },
    { 0x9d8ac0af, 76 },
    { 0x283ac0b0, 65 },
    { 0x6bdac0b1, 61 },
    { 0xab5ac0b2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x538ac0b5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bac0ba, 23 },
    { 0x11cac0bb, 34 },
    { 0xb9fac0bb, 23 },
    { 0x3d8ac0bc, 76 },
    { 0x885ac0be, 52 },
    { 0x7d7ac0bf, 33 },
    { 0x479ac0bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bac0c3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeac0cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1ac0d0, 34 },
    { 0xb5eac0d1, 90 },
    { 0x572ac0d1, 95 },
    { 0x9d7ac0d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40ac0d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ac0d9, 43 },
    { 0x625ac0da, 98 },
    { 0xfe8ac0db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15ac0e4, 98 },
    { 0, 0 },
    { 0x820ac0e6, 40 },
    { 0, 0 },
    { 0x8aac0e8, 84 },
    { 0xb78ac0e9, 52 },
    { 0xd07ac0ea, 98 },
    { 0x265ac0ea, 93 },
    { 0xdebac0e9, 33 },
    { 0x8cac0ec, 96 },
    { 0x7baac0ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdac0f1, 43 },
    { 0, 0 },
    { 0xf7eac0f3, 93 },
    { 0, 0 },
    { 0x6dcac0f5, 93 },
    { 0x606ac0f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eac0fd, 23 },
    { 0, 0 },
    { 0xcefac0ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ac102, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ac107, 76 },
    { 0x1f2ac108, 67 },
    { 0x993ac109, 76 },
    { 0xe08ac10a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fac111, 33 },
    { 0x684ac112, 76 },
    { 0x17aac113, 98 },
    { 0xd0aac114, 76 },
    { 0x91eac115, 23 },
    { 0, 0 },
    { 0xf28ac117, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6ac122, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69ac12a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffac12d, 33 },
    { 0xf60ac12e, 61 },
    { 0xf98ac12e, 33 },
    { 0x6d4ac130, 67 },
    { 0xa1ac131, 85 },
    { 0x65aac131, 78 },
    { 0x80aac12e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9ac137, 76 },
    { 0x3aeac138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bac13d, 76 },
    { 0x409ac13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981ac144, 90 },
    { 0x7eac145, 61 },
    { 0xaeac146, 43 },
    { 0xd1ac146, 91 },
    { 0x3f1ac148, 90 },
    { 0xb60ac149, 90 },
    { 0x676ac14a, 23 },
    { 0, 0 },
    { 0x281ac14c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48ac150, 43 },
    { 0, 0 },
    { 0x120ac152, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188ac156, 95 },
    { 0xe62ac157, 52 },
    { 0x297ac157, 90 },
    { 0xb48ac158, 76 },
    { 0x44cac156, 33 },
    { 0x1d5ac159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ac161, 76 },
    { 0xf87ac161, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbac165, 91 },
    { 0, 0 },
    { 0x95dac167, 61 },
    { 0, 0 },
    { 0xdd7ac169, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70ac16c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6ac171, 34 },
    { 0, 0 },
    { 0x5ddac173, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ac176, 33 },
    { 0xcdfac176, 95 },
    { 0, 0 },
    { 0x60fac179, 90 },
    { 0xeabac179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13ac17d, 23 },
    { 0, 0 },
    { 0x246ac17f, 56 },
    { 0x3b2ac180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcac188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceac191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93ac196, 90 },
    { 0xc54ac197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44ac19c, 93 },
    { 0x94aac19d, 77 },
    { 0, 0 },
    { 0xb34ac19f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dac1a2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1aac1ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ac1b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bac1b5, 91 },
    { 0x6afac1b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747ac1bd, 61 },
    { 0, 0 },
    { 0xab0ac1bf, 90 },
    { 0, 0 },
    { 0xaaac1c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefac1c4, 81 },
    { 0, 0 },
    { 0x95fac1c6, 55 },
    { 0x3d9ac1c7, 95 },
    { 0xf09ac1c8, 93 },
    { 0, 0 },
    { 0x569ac1ca, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dac1cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9ac1d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x71aac1db, 34 },
    { 0x1c9ac1dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x701ac1df, 90 },
    { 0, 0 },
    { 0x2abac1e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700ac1e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bac1f0, 43 },
    { 0, 0 },
    { 0x1dfac1f2, 91 },
    { 0x229ac1f3, 61 },
    { 0x45cac1f4, 87 },
    { 0, 0 },
    { 0xa3bac1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eac1fa, 60 },
    { 0xf00ac1fb, 90 },
    { 0, 0 },
    { 0x122ac1fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ac205, 90 },
    { 0xf88ac206, 52 },
    { 0x21bac207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x208ac20a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61ac20d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1ac210, 76 },
    { 0, 0 },
    { 0xc0bac212, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28ac21b, 61 },
    { 0x83bac21c, 93 },
    { 0x61fac21d, 95 },
    { 0, 0 },
    { 0x77ac21f, 98 },
    { 0x742ac220, 90 },
    { 0x964ac221, 76 },
    { 0, 0 },
    { 0x2aeac223, 76 },
    { 0x10bac224, 90 },
    { 0, 0 },
    { 0x9b2ac226, 55 },
    { 0xa2aac227, 61 },
    { 0x7adac228, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cac22e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539ac239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71eac23e, 33 },
    { 0x1c9ac23f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155ac244, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffac248, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bac24c, 61 },
    { 0xf0cac24d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510ac251, 99 },
    { 0x866ac251, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bac257, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ac25c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeac262, 23 },
    { 0x841ac263, 33 },
    { 0xa91ac264, 87 },
    { 0, 0 },
    { 0x468ac266, 33 },
    { 0xbaaac267, 76 },
    { 0, 0 },
    { 0xb53ac269, 55 },
    { 0xf47ac26a, 90 },
    { 0, 0 },
    { 0xa9eac26c, 23 },
    { 0, 0 },
    { 0xaf8ac26e, 90 },
    { 0x565ac26f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac274, 95 },
    { 0x2f1ac275, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24ac278, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21ac285, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ac28b, 33 },
    { 0, 0 },
    { 0x37eac28d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420ac291, 55 },
    { 0, 0 },
    { 0x99eac293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3ac297, 66 },
    { 0, 0 },
    { 0x6bbac299, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0ac29c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aac2a1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2eac2a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67ac2a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ac2ac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cac2af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64ac2b2, 84 },
    { 0, 0 },
    { 0xdc8ac2b4, 23 },
    { 0x86ac2b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54aac2b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x790ac2bb, 90 },
    { 0x8a9ac2bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ac2c4, 23 },
    { 0x3a7ac2c5, 95 },
    { 0xc1eac2c6, 90 },
    { 0x715ac2c7, 90 },
    { 0xbc6ac2c8, 52 },
    { 0xbeaac2c9, 33 },
    { 0xc1dac2ca, 81 },
    { 0, 0 },
    { 0xee4ac2cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345ac2d3, 95 },
    { 0, 0 },
    { 0xeb4ac2d5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4ac2d8, 46 },
    { 0, 0 },
    { 0x1f7ac2da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fac2df, 46 },
    { 0, 0 },
    { 0x58cac2e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bac2e4, 71 },
    { 0xb23ac2e5, 95 },
    { 0, 0 },
    { 0x15fac2e7, 52 },
    { 0x481ac2e8, 61 },
    { 0, 0 },
    { 0x716ac2ea, 78 },
    { 0x9abac2ea, 23 },
    { 0xcdac2ec, 81 },
    { 0x98ac2ed, 95 },
    { 0xef0ac2ec, 91 },
    { 0xd90ac2eb, 95 },
    { 0, 0 },
    { 0x76fac2f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490ac2f6, 90 },
    { 0x133ac2f7, 78 },
    { 0xef9ac2f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413ac2fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ac300, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518ac304, 66 },
    { 0xe92ac305, 84 },
    { 0, 0 },
    { 0x13fac307, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eac30b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6ac313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ac319, 23 },
    { 0xc13ac31a, 93 },
    { 0, 0 },
    { 0x925ac31c, 93 },
    { 0xaabac31d, 98 },
    { 0x643ac31e, 95 },
    { 0, 0 },
    { 0xea0ac320, 98 },
    { 0x6f1ac321, 33 },
    { 0xa7fac321, 33 },
    { 0, 0 },
    { 0xd4ac324, 23 },
    { 0xe2fac325, 90 },
    { 0xd8cac325, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ac32b, 76 },
    { 0x8a5ac32b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedac32f, 90 },
    { 0xe53ac330, 98 },
    { 0xf9cac331, 98 },
    { 0xa5dac331, 90 },
    { 0, 0 },
    { 0x7fac334, 43 },
    { 0xa06ac335, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffac344, 33 },
    { 0x4abac345, 95 },
    { 0x69dac344, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eac34a, 93 },
    { 0, 0 },
    { 0xd42ac34c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dac350, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac359, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35ac35d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efac365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1ac368, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0ac373, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5ac376, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x839ac379, 33 },
    { 0x20ac37a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eac382, 76 },
    { 0, 0 },
    { 0x29cac384, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9ac389, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35ac38c, 98 },
    { 0x3f0ac38c, 61 },
    { 0, 0 },
    { 0x5a9ac38f, 88 },
    { 0xcccac38f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ac395, 61 },
    { 0xef3ac395, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82ac39b, 98 },
    { 0xb39ac39c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95aac39f, 33 },
    { 0, 0 },
    { 0xebeac3a1, 40 },
    { 0, 0 },
    { 0x778ac3a3, 23 },
    { 0xed0ac3a4, 55 },
    { 0x8a0ac3a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4ac3bb, 98 },
    { 0xeb5ac3bc, 61 },
    { 0, 0 },
    { 0x781ac3be, 68 },
    { 0xb00ac3bf, 98 },
    { 0xea5ac3c0, 61 },
    { 0, 0 },
    { 0x6e5ac3c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cac3cb, 97 },
    { 0xde8ac3cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5ac3cf, 100 },
    { 0xefac3d0, 33 },
    { 0xfd5ac3cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91ac3d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6ac3dd, 87 },
    { 0, 0 },
    { 0x457ac3df, 23 },
    { 0xc36ac3df, 33 },
    { 0x60cac3e1, 90 },
    { 0x926ac3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fac3e7, 33 },
    { 0x857ac3e8, 23 },
    { 0x8ceac3e8, 90 },
    { 0x3e1ac3ea, 61 },
    { 0x913ac3ea, 87 },
    { 0x919ac3ec, 91 },
    { 0x359ac3ed, 23 },
    { 0x162ac3ed, 19 },
    { 0x52cac3ee, 61 },
    { 0, 0 },
    { 0xbf6ac3f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584ac3f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478ac3fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7ac400, 61 },
    { 0, 0 },
    { 0x2cac402, 46 },
    { 0xb6aac403, 33 },
    { 0x8aac403, 88 },
    { 0x4a9ac403, 55 },
    { 0x144ac406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fac40c, 98 },
    { 0x52eac40c, 61 },
    { 0xb39ac40e, 61 },
    { 0xef2ac40e, 75 },
    { 0x39cac40c, 90 },
    { 0xf79ac411, 61 },
    { 0xc41ac412, 58 },
    { 0, 0 },
    { 0xaeac414, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3ac418, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653ac41d, 61 },
    { 0x5c7ac41d, 61 },
    { 0x9a5ac41f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bac422, 55 },
    { 0, 0 },
    { 0x96aac424, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dac427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eac42c, 61 },
    { 0, 0 },
    { 0x954ac42e, 99 },
    { 0, 0 },
    { 0x70fac430, 34 },
    { 0xc86ac431, 78 },
    { 0x8dcac432, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cac435, 83 },
    { 0, 0 },
    { 0x8e6ac437, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aeac43b, 95 },
    { 0, 0 },
    { 0x8e1ac43d, 76 },
    { 0, 0 },
    { 0xcb9ac43f, 61 },
    { 0x250ac440, 55 },
    { 0x223ac43f, 100 },
    { 0xd24ac43f, 98 },
    { 0x103ac441, 33 },
    { 0xfd6ac444, 93 },
    { 0x1e4ac43f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1ac450, 90 },
    { 0x5b2ac451, 17 },
    { 0, 0 },
    { 0xe96ac453, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x968ac456, 76 },
    { 0xe91ac457, 43 },
    { 0x31eac458, 55 },
    { 0xce1ac456, 96 },
    { 0x55fac45a, 91 },
    { 0xfceac456, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225ac461, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25ac464, 77 },
    { 0x1eaac465, 33 },
    { 0, 0 },
    { 0x698ac467, 33 },
    { 0, 0 },
    { 0x44eac469, 61 },
    { 0, 0 },
    { 0x309ac46b, 55 },
    { 0xf9dac46c, 33 },
    { 0x172ac46d, 95 },
    { 0xe94ac46e, 55 },
    { 0xdbac46f, 33 },
    { 0x16cac46f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fac476, 23 },
    { 0xb9ac477, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63ac47b, 61 },
    { 0xf47ac47c, 17 },
    { 0, 0 },
    { 0x320ac47e, 91 },
    { 0x72cac47f, 53 },
    { 0x34dac47f, 95 },
    { 0x730ac481, 23 },
    { 0x50eac481, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5ac485, 33 },
    { 0, 0 },
    { 0xc77ac487, 55 },
    { 0xfe2ac487, 33 },
    { 0xe59ac488, 91 },
    { 0xb11ac48a, 33 },
    { 0x942ac488, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eac48e, 90 },
    { 0xc31ac48f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feac493, 61 },
    { 0x19dac494, 87 },
    { 0, 0 },
    { 0x9f4ac496, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35ac49b, 78 },
    { 0x124ac49b, 55 },
    { 0x354ac49c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9ac4a2, 76 },
    { 0x9b7ac4a3, 33 },
    { 0x333ac4a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0ac4aa, 33 },
    { 0, 0 },
    { 0x28dac4ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8aac4b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600ac4b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7ac4be, 76 },
    { 0xcb9ac4be, 95 },
    { 0xabdac4be, 90 },
    { 0x9b9ac4c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebac4c6, 61 },
    { 0x6d9ac4c7, 46 },
    { 0x810ac4c6, 17 },
    { 0x3c0ac4c9, 33 },
    { 0, 0 },
    { 0x88dac4cb, 81 },
    { 0x57aac4cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163ac4d4, 98 },
    { 0x1cfac4d5, 90 },
    { 0, 0 },
    { 0x647ac4d7, 95 },
    { 0xd14ac4d8, 46 },
    { 0x30bac4d8, 61 },
    { 0xa31ac4da, 90 },
    { 0, 0 },
    { 0x1c1ac4dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359ac4e0, 34 },
    { 0x267ac4e1, 95 },
    { 0x82fac4e0, 72 },
    { 0x6a2ac4e3, 55 },
    { 0x37eac4e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5ac4f0, 76 },
    { 0x547ac4f1, 90 },
    { 0, 0 },
    { 0xfddac4f3, 55 },
    { 0x4e5ac4f4, 85 },
    { 0xf64ac4f5, 87 },
    { 0, 0 },
    { 0xadcac4f7, 91 },
    { 0, 0 },
    { 0xfafac4f9, 98 },
    { 0, 0 },
    { 0x890ac4fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170ac4ff, 61 },
    { 0xe78ac500, 90 },
    { 0, 0 },
    { 0x259ac502, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baac507, 23 },
    { 0x54eac508, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1ac50d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fac512, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bac516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcac51b, 95 },
    { 0, 0 },
    { 0x68bac51d, 55 },
    { 0xbe7ac51e, 12 },
    { 0, 0 },
    { 0x695ac520, 55 },
    { 0x6b3ac521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x879ac524, 90 },
    { 0x53fac525, 66 },
    { 0, 0 },
    { 0xe63ac527, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95ac52e, 55 },
    { 0xbfaac52f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ac533, 90 },
    { 0x69fac533, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bac539, 52 },
    { 0xfdbac539, 101 },
    { 0x469ac539, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4aac53f, 61 },
    { 0, 0 },
    { 0x93bac541, 100 },
    { 0x6b6ac541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacac54b, 98 },
    { 0x123ac54c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x368ac54f, 99 },
    { 0, 0 },
    { 0xcf5ac551, 95 },
    { 0x2d2ac552, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeac556, 46 },
    { 0xee6ac557, 76 },
    { 0x156ac556, 85 },
    { 0, 0 },
    { 0xa55ac55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693ac55f, 61 },
    { 0x7a2ac55f, 68 },
    { 0x4f2ac561, 43 },
    { 0, 0 },
    { 0x4dfac563, 85 },
    { 0, 0 },
    { 0xd50ac565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7eac568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2ac572, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709ac577, 68 },
    { 0xbf7ac578, 95 },
    { 0x43cac577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dac57d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ac583, 23 },
    { 0xbaac584, 46 },
    { 0x200ac585, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912ac58b, 90 },
    { 0, 0 },
    { 0x87cac58d, 33 },
    { 0x270ac58e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644ac592, 93 },
    { 0x7d7ac593, 23 },
    { 0x9adac594, 34 },
    { 0x1edac595, 91 },
    { 0xcc1ac594, 61 },
    { 0xd98ac593, 46 },
    { 0x203ac598, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704ac59f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abac5a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ac5a6, 95 },
    { 0xfa3ac5a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddac5aa, 34 },
    { 0, 0 },
    { 0x492ac5ac, 23 },
    { 0x8f7ac5ad, 33 },
    { 0xc75ac5ae, 66 },
    { 0xe4bac5af, 85 },
    { 0x6cfac5ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140ac5b5, 61 },
    { 0x65ac5b6, 23 },
    { 0xf8fac5b6, 33 },
    { 0x1a2ac5b6, 91 },
    { 0, 0 },
    { 0xc67ac5ba, 80 },
    { 0x9b3ac5bb, 33 },
    { 0x88bac5bc, 23 },
    { 0x83cac5bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eac5c1, 43 },
    { 0, 0 },
    { 0xc77ac5c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ac5d3, 91 },
    { 0, 0 },
    { 0x2c7ac5d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dac5d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417ac5dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137ac5e8, 33 },
    { 0x198ac5e8, 61 },
    { 0x991ac5ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcac5ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eac5f1, 23 },
    { 0x828ac5f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871ac5f7, 98 },
    { 0x750ac5f7, 75 },
    { 0, 0 },
    { 0x5cac5fa, 95 },
    { 0, 0 },
    { 0x230ac5fc, 23 },
    { 0x16aac5fd, 33 },
    { 0, 0 },
    { 0xf8cac5ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96ac604, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3ac608, 98 },
    { 0xbcdac608, 23 },
    { 0x4f0ac60a, 55 },
    { 0x70fac60a, 90 },
    { 0x9c3ac60a, 95 },
    { 0xceeac60d, 34 },
    { 0x1faac60e, 95 },
    { 0x192ac60d, 33 },
    { 0x164ac610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac614, 76 },
    { 0x7a9ac614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edac618, 61 },
    { 0xdfcac618, 93 },
    { 0x271ac618, 87 },
    { 0, 0 },
    { 0x7d0ac61c, 23 },
    { 0xde8ac61d, 99 },
    { 0xfbcac61e, 49 },
    { 0, 0 },
    { 0x6f7ac620, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dac624, 33 },
    { 0x3dcac625, 62 },
    { 0x913ac626, 80 },
    { 0x468ac626, 90 },
    { 0x768ac628, 98 },
    { 0, 0 },
    { 0x4abac62a, 33 },
    { 0x375ac62b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ac633, 61 },
    { 0x8a6ac633, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9ac638, 38 },
    { 0, 0 },
    { 0xe9ac63a, 55 },
    { 0, 0 },
    { 0xdf2ac63c, 78 },
    { 0, 0 },
    { 0x78fac63e, 40 },
    { 0x9d1ac63e, 90 },
    { 0x947ac63e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30ac643, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ac646, 90 },
    { 0, 0 },
    { 0x1bdac648, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ac64b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ac64e, 61 },
    { 0x202ac64e, 53 },
    { 0x258ac64e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3ac654, 78 },
    { 0, 0 },
    { 0xa6aac656, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cac65c, 46 },
    { 0, 0 },
    { 0xe15ac65e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eac668, 98 },
    { 0x429ac669, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcac66c, 43 },
    { 0, 0 },
    { 0x60eac66e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x407ac671, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558ac675, 46 },
    { 0, 0 },
    { 0xa1dac677, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136ac67d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bac685, 66 },
    { 0x57dac685, 61 },
    { 0xee0ac687, 91 },
    { 0x68eac687, 26 },
    { 0xb99ac689, 55 },
    { 0x26dac689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438ac68e, 52 },
    { 0xf00ac68f, 76 },
    { 0x526ac68f, 90 },
    { 0x9a2ac690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39ac694, 23 },
    { 0xaf1ac695, 23 },
    { 0x8f5ac696, 95 },
    { 0x48ac696, 98 },
    { 0, 0 },
    { 0x71ac699, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ac69e, 91 },
    { 0xdf8ac69e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dac6a6, 87 },
    { 0xec7ac6a7, 90 },
    { 0xb93ac6a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6ac6b1, 40 },
    { 0x3b2ac6b1, 95 },
    { 0xda0ac6b3, 33 },
    { 0x62eac6b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcac6b8, 78 },
    { 0, 0 },
    { 0xa4ac6ba, 38 },
    { 0x923ac6ba, 66 },
    { 0x363ac6bb, 98 },
    { 0xa6dac6ba, 98 },
    { 0x2e6ac6ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964ac6c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abac6c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdac6cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cac6cf, 34 },
    { 0x5f6ac6d0, 90 },
    { 0x5cac6d1, 35 },
    { 0x732ac6d1, 67 },
    { 0x174ac6d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fac6d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04ac6db, 90 },
    { 0x4d5ac6dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5ac6e0, 55 },
    { 0xad0ac6e1, 95 },
    { 0x750ac6e2, 76 },
    { 0x437ac6e3, 51 },
    { 0, 0 },
    { 0xffac6e5, 99 },
    { 0xc19ac6e6, 61 },
    { 0x3baac6e7, 93 },
    { 0x668ac6e6, 90 },
    { 0, 0 },
    { 0xcb4ac6ea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2ac6ed, 61 },
    { 0xe41ac6ed, 43 },
    { 0x38eac6ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ac6f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7ac6f7, 33 },
    { 0xb93ac6f8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bac6fb, 90 },
    { 0x8f8ac6fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57ac700, 61 },
    { 0xc3eac701, 95 },
    { 0x105ac702, 90 },
    { 0x15fac700, 33 },
    { 0x713ac704, 81 },
    { 0x9d7ac702, 98 },
    { 0x974ac703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fac70a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cac715, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ac718, 34 },
    { 0xfacac718, 53 },
    { 0xfeaac71a, 84 },
    { 0x86dac71b, 33 },
    { 0x8bbac71b, 95 },
    { 0xc86ac71a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199ac723, 55 },
    { 0x43cac723, 90 },
    { 0xcc1ac723, 23 },
    { 0xfb4ac723, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbac72a, 87 },
    { 0, 0 },
    { 0x74bac72c, 95 },
    { 0x1ebac72c, 90 },
    { 0x2c1ac72c, 95 },
    { 0x38bac72f, 23 },
    { 0x58aac72f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51ac733, 61 },
    { 0x6e8ac734, 87 },
    { 0x78ac735, 33 },
    { 0x4caac734, 95 },
    { 0xc0bac737, 95 },
    { 0x85aac738, 95 },
    { 0x8a0ac738, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08ac73c, 90 },
    { 0x83bac73c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1ac741, 61 },
    { 0, 0 },
    { 0xaacac743, 87 },
    { 0, 0 },
    { 0xc8eac745, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9ac748, 93 },
    { 0x233ac748, 33 },
    { 0x1edac749, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2ac750, 33 },
    { 0xc15ac751, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ac754, 46 },
    { 0xea3ac755, 33 },
    { 0x4b4ac756, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ac759, 23 },
    { 0, 0 },
    { 0xf5ac75b, 67 },
    { 0, 0 },
    { 0x104ac75d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cac762, 34 },
    { 0x3d3ac762, 43 },
    { 0, 0 },
    { 0x463ac765, 61 },
    { 0x366ac766, 100 },
    { 0xba8ac767, 43 },
    { 0, 0 },
    { 0xdceac769, 90 },
    { 0x734ac769, 95 },
    { 0x1ceac76b, 97 },
    { 0x562ac76c, 98 },
    { 0x24eac76b, 76 },
    { 0xcf2ac769, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dac771, 61 },
    { 0, 0 },
    { 0xb93ac773, 77 },
    { 0x98fac774, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ac780, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9ac784, 46 },
    { 0, 0 },
    { 0x297ac786, 25 },
    { 0xdb3ac787, 80 },
    { 0x540ac788, 76 },
    { 0x3dcac789, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acac78e, 46 },
    { 0, 0 },
    { 0x662ac790, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44ac794, 93 },
    { 0, 0 },
    { 0x732ac796, 46 },
    { 0x1c6ac797, 98 },
    { 0x67fac796, 95 },
    { 0, 0 },
    { 0xd16ac79a, 67 },
    { 0xa56ac79b, 95 },
    { 0x9c6ac79a, 67 },
    { 0, 0 },
    { 0x6ffac79e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281ac7a2, 95 },
    { 0xf07ac7a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603ac7a7, 33 },
    { 0, 0 },
    { 0xeb0ac7a9, 76 },
    { 0x53fac7aa, 33 },
    { 0xb8ac7ab, 76 },
    { 0xa18ac7ac, 23 },
    { 0x873ac7aa, 84 },
    { 0x1f6ac7ae, 23 },
    { 0xe05ac7af, 91 },
    { 0xd2eac7b0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88ac7bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dac7c0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4ac7c5, 95 },
    { 0x269ac7c6, 90 },
    { 0x7d7ac7c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5ac7ca, 76 },
    { 0, 0 },
    { 0xd29ac7cc, 95 },
    { 0x3bcac7cd, 61 },
    { 0xdaaac7ce, 33 },
    { 0x28cac7cf, 93 },
    { 0xaa9ac7cf, 76 },
    { 0x76aac7cd, 33 },
    { 0x4beac7d2, 84 },
    { 0x31ac7d3, 33 },
    { 0xc04ac7d2, 91 },
    { 0xeb2ac7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecac7d8, 90 },
    { 0, 0 },
    { 0x6daac7da, 95 },
    { 0x255ac7db, 90 },
    { 0x34fac7dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ac7e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cac7e5, 100 },
    { 0x134ac7e6, 17 },
    { 0x4e7ac7e7, 100 },
    { 0xe2aac7e7, 98 },
    { 0xf7bac7e7, 95 },
    { 0x230ac7ea, 55 },
    { 0, 0 },
    { 0x499ac7ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2ac7f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ac7f7, 43 },
    { 0xe50ac7f8, 80 },
    { 0xb61ac7f8, 98 },
    { 0, 0 },
    { 0x89dac7fb, 91 },
    { 0, 0 },
    { 0xff6ac7fd, 23 },
    { 0x505ac7fd, 86 },
    { 0xe64ac7ff, 98 },
    { 0x77dac800, 23 },
    { 0x1c2ac801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddac804, 90 },
    { 0x57fac804, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54ac808, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89ac80b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefac811, 23 },
    { 0xfa1ac812, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x752ac815, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfac818, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55ac81c, 95 },
    { 0xd36ac81d, 98 },
    { 0x76bac81d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bac821, 98 },
    { 0xcfac821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ac825, 76 },
    { 0x26dac825, 76 },
    { 0x64fac826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650ac82d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dac830, 46 },
    { 0xff1ac830, 66 },
    { 0x409ac832, 66 },
    { 0x5a2ac833, 43 },
    { 0xaa6ac832, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dac838, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452ac83e, 33 },
    { 0, 0 },
    { 0x80dac840, 33 },
    { 0, 0 },
    { 0x1b1ac842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97ac845, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcac84c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ac850, 34 },
    { 0xab7ac851, 52 },
    { 0x72eac852, 55 },
    { 0x145ac851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590ac859, 46 },
    { 0x58fac85a, 61 },
    { 0, 0 },
    { 0xc74ac85c, 33 },
    { 0xce9ac85d, 17 },
    { 0xa80ac85c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0ac863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10ac866, 98 },
    { 0x685ac866, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82ac86d, 61 },
    { 0xcf1ac86e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bac871, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5ac875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fac878, 76 },
    { 0xe15ac879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167ac87e, 23 },
    { 0, 0 },
    { 0x410ac880, 81 },
    { 0x576ac881, 38 },
    { 0x640ac880, 84 },
    { 0x35cac883, 17 },
    { 0xb14ac882, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2ac887, 93 },
    { 0xb15ac887, 43 },
    { 0xb8cac889, 33 },
    { 0, 0 },
    { 0xb44ac88b, 68 },
    { 0xdcbac88b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79ac891, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x407ac894, 90 },
    { 0x8c8ac894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43ac898, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4ac89c, 98 },
    { 0, 0 },
    { 0xdb9ac89e, 43 },
    { 0xca1ac89e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbac8a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ac8aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ac8b2, 93 },
    { 0x48ac8b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eac8b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33ac8bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bac8bf, 98 },
    { 0xc81ac8bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cac8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0ac8c8, 34 },
    { 0x2ebac8c8, 99 },
    { 0xb5bac8ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ac8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532ac8d5, 52 },
    { 0xd77ac8d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedac8dc, 100 },
    { 0, 0 },
    { 0x7eeac8de, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ac8e1, 93 },
    { 0, 0 },
    { 0x18ac8e3, 43 },
    { 0xdb2ac8e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379ac8e9, 55 },
    { 0x8a9ac8ea, 77 },
    { 0x8ccac8ea, 46 },
    { 0x8d3ac8eb, 23 },
    { 0xeb9ac8ed, 61 },
    { 0xaedac8ec, 90 },
    { 0x2e1ac8ee, 98 },
    { 0x68bac8f0, 66 },
    { 0x7f8ac8f1, 33 },
    { 0x6c1ac8f2, 33 },
    { 0xafeac8f1, 76 },
    { 0x31dac8ea, 34 },
    { 0xb85ac8f5, 95 },
    { 0xd0aac8f5, 98 },
    { 0x7b8ac8ef, 90 },
    { 0xfd1ac8e9, 61 },
    { 0x23ac8f9, 76 },
    { 0, 0 },
    { 0x218ac8fb, 82 },
    { 0x2fdac8fc, 90 },
    { 0x72ac8fc, 90 },
    { 0xd94ac8fd, 66 },
    { 0, 0 },
    { 0xfe1ac900, 61 },
    { 0, 0 },
    { 0xf8bac902, 98 },
    { 0x7a3ac903, 34 },
    { 0, 0 },
    { 0xbdeac905, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ac908, 43 },
    { 0, 0 },
    { 0xfd0ac90a, 95 },
    { 0, 0 },
    { 0x6f0ac90c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7ac910, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495ac915, 33 },
    { 0, 0 },
    { 0xecbac917, 23 },
    { 0xe0aac918, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4ac91b, 90 },
    { 0, 0 },
    { 0x8a1ac91d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376ac923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cac92a, 33 },
    { 0, 0 },
    { 0xa48ac92c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebac936, 91 },
    { 0x289ac937, 23 },
    { 0xfe0ac938, 95 },
    { 0xaaeac939, 61 },
    { 0, 0 },
    { 0x9f9ac93b, 90 },
    { 0x761ac93c, 90 },
    { 0x8c1ac93d, 23 },
    { 0x226ac93d, 33 },
    { 0xd85ac93d, 25 },
    { 0xc82ac940, 68 },
    { 0, 0 },
    { 0xeaaac942, 34 },
    { 0x120ac942, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ac947, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600ac94b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2ac950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dac953, 73 },
    { 0x25aac954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cac95a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bac95d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42ac960, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94ac964, 66 },
    { 0x72ac964, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ac96b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eac972, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92ac976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13ac979, 23 },
    { 0, 0 },
    { 0x14cac97b, 98 },
    { 0x5dbac97b, 84 },
    { 0x7fbac97c, 61 },
    { 0xea0ac97e, 34 },
    { 0, 0 },
    { 0x77cac980, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94ac983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdac987, 95 },
    { 0xaefac987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3ac98d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4ac990, 33 },
    { 0x926ac991, 34 },
    { 0xd27ac990, 66 },
    { 0x8c0ac993, 23 },
    { 0, 0 },
    { 0xef7ac995, 23 },
    { 0x9ddac996, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bac9a0, 90 },
    { 0x21dac9a0, 23 },
    { 0, 0 },
    { 0x4ac9a3, 67 },
    { 0x20ac9a4, 90 },
    { 0xf80ac9a5, 61 },
    { 0xa9fac9a6, 81 },
    { 0x2a6ac9a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cac9aa, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbac9b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9ac9ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ac9c2, 90 },
    { 0x2bfac9c3, 76 },
    { 0, 0 },
    { 0x3bbac9c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac9c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5ac9cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ac9d9, 61 },
    { 0x79aac9da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaac9dd, 61 },
    { 0xd55ac9dd, 76 },
    { 0xe12ac9dd, 19 },
    { 0x714ac9e0, 33 },
    { 0xdb0ac9e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeac9e7, 66 },
    { 0x586ac9e8, 23 },
    { 0x73bac9e7, 84 },
    { 0x39cac9e8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x845ac9ed, 90 },
    { 0x596ac9ed, 52 },
    { 0, 0 },
    { 0x416ac9f0, 91 },
    { 0xbefac9f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0ac9f4, 99 },
    { 0x2ceac9f4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6ac9fc, 55 },
    { 0xd02ac9fd, 55 },
    { 0x4a4ac9fc, 100 },
    { 0x223ac9fd, 23 },
    { 0xfc6aca00, 43 },
    { 0, 0 },
    { 0xbaaca02, 33 },
    { 0, 0 },
    { 0x69faca04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29aca08, 46 },
    { 0, 0 },
    { 0x8dbaca0a, 61 },
    { 0x7f6aca0a, 90 },
    { 0, 0 },
    { 0x3fbaca0d, 95 },
    { 0x65baca0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549aca16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56aca1b, 91 },
    { 0xe01aca1c, 61 },
    { 0x1faaca1d, 76 },
    { 0x6c2aca1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x281aca21, 34 },
    { 0x202aca21, 55 },
    { 0, 0 },
    { 0xea5aca24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefaca27, 76 },
    { 0xdd0aca28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacaca2b, 23 },
    { 0, 0 },
    { 0x252aca2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcaca30, 68 },
    { 0x35faca30, 76 },
    { 0, 0 },
    { 0x12eaca33, 61 },
    { 0, 0 },
    { 0xcc7aca35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x682aca38, 61 },
    { 0x940aca39, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646aca3e, 33 },
    { 0x6c7aca3f, 34 },
    { 0xd3faca3f, 33 },
    { 0, 0 },
    { 0x322aca42, 90 },
    { 0xba4aca43, 90 },
    { 0x11daca44, 87 },
    { 0xfd0aca44, 65 },
    { 0xc02aca46, 76 },
    { 0xab8aca46, 93 },
    { 0xa51aca48, 61 },
    { 0x48eaca49, 98 },
    { 0x4daca43, 61 },
    { 0x6e6aca48, 76 },
    { 0x1bfaca49, 46 },
    { 0, 0 },
    { 0x9c7aca4e, 55 },
    { 0xcb4aca4e, 52 },
    { 0xd89aca50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3aaca53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2aca5b, 84 },
    { 0, 0 },
    { 0x205aca5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715aca62, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30aca65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920aca69, 33 },
    { 0x4fdaca6a, 76 },
    { 0xb27aca6b, 98 },
    { 0xc59aca6b, 90 },
    { 0, 0 },
    { 0xd52aca6e, 65 },
    { 0x2d1aca6f, 33 },
    { 0xb7baca70, 91 },
    { 0x192aca71, 33 },
    { 0x2ffaca71, 52 },
    { 0, 0 },
    { 0x43eaca74, 74 },
    { 0xb39aca75, 98 },
    { 0, 0 },
    { 0x3aaca77, 43 },
    { 0x532aca78, 96 },
    { 0x6cdaca77, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3aca7c, 98 },
    { 0, 0 },
    { 0x29aca7e, 43 },
    { 0x85baca7f, 33 },
    { 0x211aca7e, 34 },
    { 0xdabaca81, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1aca88, 34 },
    { 0, 0 },
    { 0xb91aca8a, 33 },
    { 0x558aca8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x13daca8e, 90 },
    { 0, 0 },
    { 0x3aca90, 55 },
    { 0xe5aca90, 95 },
    { 0x70eaca92, 98 },
    { 0xfa3aca93, 34 },
    { 0x1aaaca93, 55 },
    { 0x5f8aca94, 90 },
    { 0x10aaca96, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5aca9d, 34 },
    { 0xb31aca9d, 84 },
    { 0x726aca9f, 76 },
    { 0, 0 },
    { 0xb3bacaa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29acaa6, 61 },
    { 0x521acaa6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93acab1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0acab5, 77 },
    { 0x1c7acab6, 52 },
    { 0xcbfacab7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcacac0, 52 },
    { 0x897acac0, 98 },
    { 0xd59acac2, 55 },
    { 0xf5cacac3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2acac6, 76 },
    { 0x1ceacac7, 23 },
    { 0x151acac8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0acad3, 55 },
    { 0xc10acad4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffacad8, 95 },
    { 0, 0 },
    { 0xf45acada, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135acae4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dacaeb, 33 },
    { 0x134acaec, 76 },
    { 0, 0 },
    { 0xa5facaee, 100 },
    { 0, 0 },
    { 0x56cacaf0, 84 },
    { 0xa91acaf0, 17 },
    { 0xb0bacaf2, 78 },
    { 0x5fbacaf0, 93 },
    { 0xe77acaf4, 88 },
    { 0x243acaf5, 90 },
    { 0xdb7acaf6, 98 },
    { 0, 0 },
    { 0x711acaf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9acafd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bacb0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2acb10, 91 },
    { 0, 0 },
    { 0x3f9acb12, 23 },
    { 0xc86acb13, 33 },
    { 0x2b5acb13, 95 },
    { 0xb36acb13, 76 },
    { 0x960acb13, 23 },
    { 0xbfdacb17, 90 },
    { 0, 0 },
    { 0x10bacb19, 23 },
    { 0x3c5acb19, 81 },
    { 0, 0 },
    { 0xb68acb1c, 90 },
    { 0, 0 },
    { 0x906acb1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88acb24, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477acb29, 95 },
    { 0x964acb2a, 98 },
    { 0, 0 },
    { 0x2aaacb2c, 33 },
    { 0x7ceacb2d, 98 },
    { 0, 0 },
    { 0x380acb2f, 91 },
    { 0, 0 },
    { 0x380acb31, 61 },
    { 0x984acb31, 61 },
    { 0x29facb31, 34 },
    { 0x765acb33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aacb40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaacb4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5acb4d, 43 },
    { 0x82acb4e, 98 },
    { 0x67aacb4f, 33 },
    { 0xabacb50, 61 },
    { 0xe77acb50, 33 },
    { 0x61dacb51, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646acb58, 61 },
    { 0, 0 },
    { 0xd31acb5a, 55 },
    { 0x383acb5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65facb60, 34 },
    { 0, 0 },
    { 0xc98acb62, 23 },
    { 0, 0 },
    { 0xa6bacb64, 76 },
    { 0, 0 },
    { 0xa31acb66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0acb70, 95 },
    { 0x401acb70, 100 },
    { 0x93cacb70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaacb76, 46 },
    { 0x91cacb77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9acb7b, 90 },
    { 0xf1facb7b, 93 },
    { 0xb37acb7d, 98 },
    { 0, 0 },
    { 0xa1acb7f, 100 },
    { 0xf57acb7f, 99 },
    { 0xb8cacb81, 33 },
    { 0xd94acb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68eacb88, 78 },
    { 0xa23acb88, 76 },
    { 0x1c2acb8a, 36 },
    { 0, 0 },
    { 0x920acb8c, 46 },
    { 0x55facb8d, 90 },
    { 0xe77acb8e, 90 },
    { 0x5f4acb8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bacb95, 98 },
    { 0x294acb96, 93 },
    { 0x7abacb97, 75 },
    { 0x95dacb98, 95 },
    { 0x35acb99, 61 },
    { 0xc81acb9a, 61 },
    { 0x1faacb98, 100 },
    { 0x92cacb96, 84 },
    { 0xb9facb9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6acba0, 76 },
    { 0, 0 },
    { 0x3fdacba2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4acbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593acbae, 38 },
    { 0x4adacbaf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbacbb2, 98 },
    { 0x881acbb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10acbb6, 90 },
    { 0xbd0acbb6, 55 },
    { 0xa25acbb8, 32 },
    { 0x7bbacbb8, 90 },
    { 0x5aacbba, 99 },
    { 0, 0 },
    { 0x64bacbbc, 23 },
    { 0x18cacbbd, 33 },
    { 0x353acbbd, 34 },
    { 0x6d6acbbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380acbc3, 90 },
    { 0x478acbc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0acbc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2acbd2, 46 },
    { 0, 0 },
    { 0xaadacbd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eacbd7, 90 },
    { 0x1eaacbd7, 93 },
    { 0xaaaacbd9, 95 },
    { 0x24facbda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56facbdd, 52 },
    { 0x799acbdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9acbe2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9acbe5, 99 },
    { 0x15acbe6, 43 },
    { 0, 0 },
    { 0x3d4acbe8, 77 },
    { 0x507acbe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3acbed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0acbf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9acbf5, 23 },
    { 0x6a8acbf5, 98 },
    { 0xbf6acbf7, 38 },
    { 0x977acbf6, 34 },
    { 0x304acbf9, 33 },
    { 0x5d9acbf8, 93 },
    { 0xa53acbfb, 23 },
    { 0xdf9acbfb, 66 },
    { 0, 0 },
    { 0x9eaacbfe, 34 },
    { 0x1f3acbff, 97 },
    { 0, 0 },
    { 0x1f5acc01, 95 },
    { 0x5a1acc02, 90 },
    { 0x1f7acc01, 33 },
    { 0xf24acc04, 66 },
    { 0x47acc04, 43 },
    { 0, 0 },
    { 0xdc8acc07, 76 },
    { 0xa2eacc08, 95 },
    { 0x9beacc08, 95 },
    { 0x36cacc08, 87 },
    { 0xf7facc0b, 52 },
    { 0xd18acc09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68acc0f, 34 },
    { 0xa22acc0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4facc13, 61 },
    { 0x25cacc13, 33 },
    { 0x16aacc14, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadacc1b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3acc1e, 55 },
    { 0xc2facc1f, 61 },
    { 0x8deacc20, 90 },
    { 0x272acc21, 17 },
    { 0, 0 },
    { 0x498acc23, 75 },
    { 0xe9acc24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23acc29, 90 },
    { 0x399acc29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x940acc2d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc30, 91 },
    { 0x6d8acc31, 90 },
    { 0x5f9acc32, 81 },
    { 0x453acc30, 33 },
    { 0xbcaacc34, 88 },
    { 0x27cacc34, 61 },
    { 0, 0 },
    { 0x25bacc37, 33 },
    { 0x165acc38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dacc3c, 98 },
    { 0xb01acc3d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x60aacc40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30aacc46, 33 },
    { 0x860acc47, 61 },
    { 0x905acc46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dacc52, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66acc55, 61 },
    { 0xeeeacc56, 95 },
    { 0x62cacc56, 34 },
    { 0xb0cacc57, 95 },
    { 0x87cacc59, 90 },
    { 0, 0 },
    { 0x6facc5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84acc5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeacc63, 61 },
    { 0, 0 },
    { 0x175acc65, 33 },
    { 0x24aacc65, 23 },
    { 0xd48acc66, 90 },
    { 0x4f3acc68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01acc6b, 95 },
    { 0xb9cacc6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x69acc6f, 33 },
    { 0x6d1acc70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efacc73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcacc77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x685acc7a, 76 },
    { 0xed6acc7a, 61 },
    { 0, 0 },
    { 0x2b4acc7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2acc81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91acc87, 76 },
    { 0x2fbacc87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cacc90, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8facc94, 76 },
    { 0xc3facc94, 23 },
    { 0xf54acc96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebacc9c, 90 },
    { 0x9cbacc9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967acca4, 90 },
    { 0, 0 },
    { 0x26acca6, 33 },
    { 0x188acca6, 23 },
    { 0xaf7acca6, 33 },
    { 0x60acca7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaccac, 90 },
    { 0x160accac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27baccb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1accb3, 98 },
    { 0, 0 },
    { 0x9efaccb5, 34 },
    { 0xf8daccb5, 23 },
    { 0x6d9accb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaccba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x725accbd, 91 },
    { 0x76aaccbe, 85 },
    { 0xe54accbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4accc7, 90 },
    { 0xb62accc8, 99 },
    { 0x8daaccc8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x75facccc, 90 },
    { 0, 0 },
    { 0x819accce, 98 },
    { 0, 0 },
    { 0x6d9accd0, 76 },
    { 0x13daccd1, 33 },
    { 0x1f5accd2, 33 },
    { 0, 0 },
    { 0x573accd4, 61 },
    { 0x204accd5, 43 },
    { 0x918accd6, 93 },
    { 0x46caccd7, 61 },
    { 0xba3accd7, 98 },
    { 0xbabaccd5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55accdc, 52 },
    { 0xc27accdd, 55 },
    { 0x68aaccde, 95 },
    { 0x51faccdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x954acce2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94baccf0, 46 },
    { 0x7eeaccf0, 98 },
    { 0xaa3accf2, 45 },
    { 0xac3accf3, 55 },
    { 0x83aaccf4, 43 },
    { 0x40faccf4, 23 },
    { 0xcc4accf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9accfa, 78 },
    { 0x8a1accfb, 61 },
    { 0x2f2accfc, 17 },
    { 0, 0 },
    { 0x5d1accfe, 91 },
    { 0, 0 },
    { 0x54cacd00, 55 },
    { 0, 0 },
    { 0x37cacd02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcacd0a, 88 },
    { 0x7aaacd0a, 90 },
    { 0x97eacd0c, 90 },
    { 0, 0 },
    { 0x8bcacd0e, 95 },
    { 0, 0 },
    { 0x6cbacd10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783acd14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaacd1a, 98 },
    { 0, 0 },
    { 0x53facd1c, 61 },
    { 0xe9facd1d, 61 },
    { 0xdd9acd1e, 53 },
    { 0x1d1acd1d, 98 },
    { 0, 0 },
    { 0xb98acd21, 46 },
    { 0xb89acd21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eacd26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1acd29, 55 },
    { 0x92cacd2a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7acd35, 100 },
    { 0x33facd36, 78 },
    { 0xd33acd37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edacd3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3acd3e, 90 },
    { 0xbf3acd3e, 67 },
    { 0xbc3acd40, 75 },
    { 0x1d6acd41, 95 },
    { 0x8feacd42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487acd46, 33 },
    { 0x6e3acd46, 34 },
    { 0x21acd48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65acd4b, 34 },
    { 0xf7bacd4c, 90 },
    { 0xdf1acd4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bacd52, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14facd56, 55 },
    { 0, 0 },
    { 0x180acd58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecacd5b, 52 },
    { 0, 0 },
    { 0xdabacd5d, 99 },
    { 0xfcacd5e, 98 },
    { 0x2dbacd5f, 76 },
    { 0x9c9acd5f, 95 },
    { 0x2c7acd61, 77 },
    { 0xc50acd62, 68 },
    { 0x22acd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15acd68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30acd6b, 33 },
    { 0xb05acd6b, 95 },
    { 0x9e3acd6d, 46 },
    { 0x9aaacd6e, 99 },
    { 0xbdfacd6f, 81 },
    { 0x69acd70, 55 },
    { 0xc9cacd71, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669acd76, 23 },
    { 0x27cacd77, 78 },
    { 0, 0 },
    { 0xdc6acd79, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761acd80, 55 },
    { 0xc7acd80, 46 },
    { 0x81bacd82, 55 },
    { 0x950acd83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616acd87, 17 },
    { 0, 0 },
    { 0xe20acd89, 33 },
    { 0x1ecacd8a, 76 },
    { 0xbcdacd8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79aacd8f, 91 },
    { 0xc9facd90, 87 },
    { 0xb33acd90, 33 },
    { 0x309acd92, 83 },
    { 0x24bacd93, 52 },
    { 0x424acd90, 46 },
    { 0xb0aacd90, 90 },
    { 0x698acd96, 93 },
    { 0xfc9acd97, 100 },
    { 0x281acd98, 23 },
    { 0, 0 },
    { 0xed3acd9a, 52 },
    { 0x30eacd9a, 67 },
    { 0xaa3acd9c, 99 },
    { 0x520acd9c, 34 },
    { 0x665acd9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1eacda3, 76 },
    { 0x345acda3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8acda7, 61 },
    { 0, 0 },
    { 0x58eacda9, 33 },
    { 0, 0 },
    { 0xd82acdab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cacdb1, 98 },
    { 0xbc2acdb2, 95 },
    { 0xea7acdb2, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eacdb7, 101 },
    { 0xee8acdb7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86facdbf, 33 },
    { 0, 0 },
    { 0x1fcacdc1, 91 },
    { 0, 0 },
    { 0xe34acdc3, 61 },
    { 0x4b0acdc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94acdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadacdd2, 90 },
    { 0x70aacdd3, 95 },
    { 0, 0 },
    { 0xb9aacdd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaacdd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356acde0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92acde3, 90 },
    { 0, 0 },
    { 0x1e2acde5, 76 },
    { 0x160acde6, 33 },
    { 0x4daacde5, 69 },
    { 0x3b9acde8, 61 },
    { 0xbd0acde8, 76 },
    { 0x900acdea, 90 },
    { 0x484acdeb, 33 },
    { 0x5e6acdeb, 90 },
    { 0x7b7acded, 90 },
    { 0xb23acdeb, 43 },
    { 0xb1facdef, 61 },
    { 0xb4eacdef, 33 },
    { 0x436acdf1, 90 },
    { 0x73bacdf2, 76 },
    { 0x640acdf2, 46 },
    { 0x5f6acdf3, 95 },
    { 0xb18acdf0, 98 },
    { 0xaf4acdf6, 99 },
    { 0x842acdf7, 95 },
    { 0x783acde8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cacdfb, 55 },
    { 0xc35acdfc, 61 },
    { 0x66acdfc, 23 },
    { 0, 0 },
    { 0xba3acdff, 33 },
    { 0x9faace00, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xbface03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baace06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378ace10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ace15, 98 },
    { 0x764ace16, 75 },
    { 0xa8cace17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946ace1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bace22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939ace28, 46 },
    { 0xa4aace28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x852ace2c, 23 },
    { 0x693ace2c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60ace31, 23 },
    { 0xbd5ace31, 78 },
    { 0, 0 },
    { 0xe77ace34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6ace38, 95 },
    { 0x4cdace38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccace3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bace44, 42 },
    { 0x465ace45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0ace4a, 38 },
    { 0, 0 },
    { 0xb27ace4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dace52, 23 },
    { 0, 0 },
    { 0xff6ace54, 90 },
    { 0x8c1ace54, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cace58, 23 },
    { 0x937ace58, 34 },
    { 0, 0 },
    { 0x9baace5b, 95 },
    { 0xda3ace5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91aace60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaace65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23ace68, 23 },
    { 0x168ace68, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153ace6e, 91 },
    { 0, 0 },
    { 0x2dcace70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeface73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaace77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844ace7b, 83 },
    { 0xe73ace7b, 34 },
    { 0x6edace7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59face80, 55 },
    { 0x4f7ace81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aace8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80ace8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cace91, 61 },
    { 0x520ace92, 75 },
    { 0x67face93, 100 },
    { 0x13ace93, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576ace9a, 33 },
    { 0xa99ace9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5ace9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bacea6, 34 },
    { 0, 0 },
    { 0xa55acea8, 98 },
    { 0, 0 },
    { 0xdf2aceaa, 55 },
    { 0xa56aceab, 61 },
    { 0, 0 },
    { 0xbb4acead, 90 },
    { 0x483aceae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434aceb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116aceba, 23 },
    { 0x3ccacebb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0acebf, 78 },
    { 0x219acec0, 17 },
    { 0x6eeacec0, 90 },
    { 0x973acec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bacec6, 34 },
    { 0, 0 },
    { 0x7d0acec8, 67 },
    { 0xaecacec9, 23 },
    { 0x23cacec8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315aced2, 68 },
    { 0xd44aced3, 46 },
    { 0, 0 },
    { 0xfaaced5, 85 },
    { 0xe11aced6, 94 },
    { 0, 0 },
    { 0x665aced8, 87 },
    { 0x3c1aced8, 23 },
    { 0xc89aced8, 98 },
    { 0x86eacedb, 88 },
    { 0x191acedc, 55 },
    { 0xb28acedd, 90 },
    { 0x647aceda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x546acee1, 75 },
    { 0, 0 },
    { 0x6f6acee3, 87 },
    { 0xa80acee4, 33 },
    { 0x2d8acee5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cacee9, 33 },
    { 0x384aceea, 95 },
    { 0x9cfaceea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670acef1, 95 },
    { 0xea1acef2, 76 },
    { 0xe81acef3, 95 },
    { 0, 0 },
    { 0x303acef5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9acef8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92acf0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5acf14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bacf19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44acf1c, 90 },
    { 0x844acf1c, 43 },
    { 0, 0 },
    { 0x9d8acf1f, 93 },
    { 0, 0 },
    { 0x528acf21, 84 },
    { 0x574acf21, 100 },
    { 0, 0 },
    { 0x46bacf24, 93 },
    { 0x946acf25, 23 },
    { 0xbdcacf26, 90 },
    { 0x773acf27, 17 },
    { 0x2a2acf27, 52 },
    { 0xbabacf29, 75 },
    { 0xa89acf2a, 61 },
    { 0, 0 },
    { 0x212acf2c, 66 },
    { 0xab1acf2c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576acf31, 100 },
    { 0x441acf32, 93 },
    { 0xd0bacf33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29facf36, 34 },
    { 0x71eacf36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cacf3b, 66 },
    { 0x465acf3c, 98 },
    { 0xa72acf3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2acf45, 46 },
    { 0x452acf45, 87 },
    { 0xec4acf47, 33 },
    { 0, 0 },
    { 0xd96acf49, 33 },
    { 0xe17acf49, 23 },
    { 0xcb6acf4b, 100 },
    { 0, 0 },
    { 0xe0facf4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4acf52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5acf56, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35acf5a, 76 },
    { 0, 0 },
    { 0xc95acf5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7acf5f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379acf67, 43 },
    { 0, 0 },
    { 0x480acf69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52acf6c, 88 },
    { 0x440acf6d, 34 },
    { 0xdb0acf6e, 87 },
    { 0x491acf6e, 34 },
    { 0xe1dacf6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984acf79, 23 },
    { 0x2a9acf7a, 99 },
    { 0, 0 },
    { 0xd3facf7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eacf80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bacf83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51acf88, 33 },
    { 0, 0 },
    { 0xa44acf8a, 61 },
    { 0, 0 },
    { 0x6e2acf8c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595acf92, 33 },
    { 0x73cacf93, 68 },
    { 0xac0acf94, 33 },
    { 0, 0 },
    { 0x81eacf96, 90 },
    { 0x30acf97, 93 },
    { 0x126acf97, 52 },
    { 0x2d7acf96, 34 },
    { 0, 0 },
    { 0xf33acf9b, 33 },
    { 0, 0 },
    { 0x21aacf9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5acfa1, 98 },
    { 0xd5cacfa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17acfa5, 61 },
    { 0x476acfa6, 90 },
    { 0xde4acfa5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9facfaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292acfb1, 90 },
    { 0, 0 },
    { 0xdd0acfb3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68acfba, 46 },
    { 0x6d0acfbb, 77 },
    { 0xe4acfbb, 61 },
    { 0xe2eacfbd, 43 },
    { 0x72facfbc, 100 },
    { 0x77bacfba, 34 },
    { 0xdfacfc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3acfc3, 91 },
    { 0xf5aacfc3, 76 },
    { 0x965acfc5, 23 },
    { 0xc6bacfc5, 33 },
    { 0x7cacfc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beacfca, 23 },
    { 0, 0 },
    { 0x3aacfcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9acfd4, 33 },
    { 0xe02acfd4, 43 },
    { 0x53bacfd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cacfda, 90 },
    { 0x543acfda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5acfe0, 95 },
    { 0x7c8acfe1, 61 },
    { 0x65aacfe1, 23 },
    { 0, 0 },
    { 0xd2dacfe4, 95 },
    { 0x3d6acfe5, 61 },
    { 0x7acacfe6, 95 },
    { 0x1a7acfe7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x589acfea, 94 },
    { 0, 0 },
    { 0x1abacfec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbacfef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bacff3, 23 },
    { 0, 0 },
    { 0xce0acff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacacffb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ad005, 34 },
    { 0x9f7ad006, 98 },
    { 0x8e9ad006, 33 },
    { 0, 0 },
    { 0xf7cad009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9ad012, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ead017, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ad01d, 23 },
    { 0xeeead01d, 75 },
    { 0x16fad01e, 85 },
    { 0, 0 },
    { 0xbc7ad021, 85 },
    { 0xe7fad022, 55 },
    { 0x41aad021, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ead026, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0ad02a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30ad02d, 66 },
    { 0x1ecad02d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56aad031, 34 },
    { 0xb4ead032, 80 },
    { 0xe44ad033, 90 },
    { 0xad1ad031, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ad039, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ad043, 34 },
    { 0, 0 },
    { 0xc85ad045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450ad049, 76 },
    { 0xbaad049, 98 },
    { 0xa2fad04b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7ad04e, 66 },
    { 0xfbaad04e, 93 },
    { 0, 0 },
    { 0xe40ad051, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbad059, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedad05c, 76 },
    { 0, 0 },
    { 0x5cfad05e, 61 },
    { 0, 0 },
    { 0x5b2ad060, 87 },
    { 0, 0 },
    { 0x5b5ad062, 99 },
    { 0x702ad063, 95 },
    { 0x13fad064, 77 },
    { 0, 0 },
    { 0xaebad066, 43 },
    { 0, 0 },
    { 0x2ddad068, 33 },
    { 0x816ad069, 91 },
    { 0x57ead06a, 76 },
    { 0x8a5ad068, 100 },
    { 0, 0 },
    { 0x1b8ad06d, 33 },
    { 0x35cad06e, 61 },
    { 0x7eaad06f, 85 },
    { 0x846ad06f, 75 },
    { 0xadad071, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ad075, 33 },
    { 0x512ad076, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ead083, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ad089, 34 },
    { 0, 0 },
    { 0x9cead08b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ad08f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303ad094, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ead09b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x351ad09e, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cead0a5, 90 },
    { 0, 0 },
    { 0xabcad0a7, 81 },
    { 0xbe9ad0a8, 90 },
    { 0x66cad0a9, 33 },
    { 0x9eead0aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6ad0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad0b8, 23 },
    { 0x2e9ad0b8, 66 },
    { 0xcfad0ba, 61 },
    { 0xa91ad0b8, 80 },
    { 0, 0 },
    { 0xebfad0bd, 23 },
    { 0x7c1ad0be, 33 },
    { 0, 0 },
    { 0xfd8ad0c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3ad0c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ad0c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dad0cc, 80 },
    { 0xcc3ad0cd, 46 },
    { 0, 0 },
    { 0xa95ad0cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ad0d7, 100 },
    { 0x894ad0d8, 84 },
    { 0x75fad0d7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfad0de, 61 },
    { 0x5f6ad0de, 33 },
    { 0x86aad0de, 90 },
    { 0, 0 },
    { 0x292ad0e2, 33 },
    { 0x207ad0e3, 95 },
    { 0x2f9ad0e4, 93 },
    { 0x71cad0e5, 76 },
    { 0xba8ad0e6, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612ad0ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caad0ee, 78 },
    { 0, 0 },
    { 0xc15ad0f0, 99 },
    { 0x8baad0f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdad0f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37ad0f7, 76 },
    { 0, 0 },
    { 0x480ad0f9, 100 },
    { 0x878ad0fa, 76 },
    { 0x912ad0f9, 67 },
    { 0, 0 },
    { 0xa08ad0fd, 52 },
    { 0, 0 },
    { 0x801ad0ff, 33 },
    { 0x2ebad100, 66 },
    { 0, 0 },
    { 0xcbad102, 69 },
    { 0, 0 },
    { 0x72dad104, 61 },
    { 0x4baad104, 84 },
    { 0xd9ead106, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abad109, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339ad110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x224ad113, 61 },
    { 0xda7ad114, 76 },
    { 0xbebad114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x748ad118, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8ad11f, 98 },
    { 0x41dad120, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918ad127, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95ad12c, 95 },
    { 0, 0 },
    { 0xe0aad12e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ad133, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ead139, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fad13d, 90 },
    { 0x2b6ad13d, 98 },
    { 0x191ad13f, 33 },
    { 0xa62ad140, 65 },
    { 0xa6cad140, 61 },
    { 0xed0ad142, 78 },
    { 0x631ad13f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9ad148, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ad14c, 98 },
    { 0xeb5ad14d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ad153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21ad157, 95 },
    { 0x7dbad157, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0ad15c, 90 },
    { 0xa19ad15d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604ad161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198ad165, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ead168, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cad16e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x599ad171, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6ad179, 59 },
    { 0x794ad17a, 90 },
    { 0x499ad17a, 90 },
    { 0x653ad179, 98 },
    { 0x7b0ad17d, 23 },
    { 0xeacad17d, 75 },
    { 0xd63ad17f, 33 },
    { 0x2e0ad17e, 76 },
    { 0xa18ad17e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55ad184, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0ad187, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cad18a, 87 },
    { 0xabfad18a, 33 },
    { 0x9d8ad18c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92ad194, 95 },
    { 0xc32ad195, 23 },
    { 0, 0 },
    { 0x367ad197, 61 },
    { 0xe8cad198, 84 },
    { 0xb72ad197, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aad19e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982ad1a4, 90 },
    { 0x2a6ad1a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dad1aa, 33 },
    { 0xf99ad1aa, 90 },
    { 0, 0 },
    { 0x322ad1ad, 98 },
    { 0x843ad1ad, 65 },
    { 0x85ead1af, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aad1b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7ad1b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21ad1bf, 98 },
    { 0, 0 },
    { 0x8b6ad1c1, 33 },
    { 0, 0 },
    { 0xffad1c3, 90 },
    { 0, 0 },
    { 0xf10ad1c5, 96 },
    { 0x3d5ad1c6, 90 },
    { 0x22cad1c7, 43 },
    { 0xb4fad1c8, 61 },
    { 0x954ad1c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0ad1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x132ad1d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586ad1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efad1e1, 98 },
    { 0xadaad1e1, 61 },
    { 0xfa9ad1e3, 90 },
    { 0, 0 },
    { 0xda2ad1e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5ad1e8, 95 },
    { 0xe2cad1e8, 33 },
    { 0, 0 },
    { 0x3b0ad1eb, 76 },
    { 0xb2dad1eb, 55 },
    { 0x8f4ad1ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522ad1f2, 33 },
    { 0x258ad1f3, 98 },
    { 0, 0 },
    { 0x226ad1f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ead1f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194ad1fc, 95 },
    { 0x3bead1fd, 34 },
    { 0xd72ad1fd, 61 },
    { 0xe6fad1fe, 61 },
    { 0, 0 },
    { 0x4a8ad201, 91 },
    { 0x49cad202, 34 },
    { 0x20ead202, 98 },
    { 0, 0 },
    { 0x4adad205, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2ad20a, 34 },
    { 0x544ad20b, 33 },
    { 0, 0 },
    { 0xebcad20d, 33 },
    { 0xbc9ad20d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ad212, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eead215, 33 },
    { 0x7f3ad215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3ad21f, 85 },
    { 0x273ad220, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dead227, 90 },
    { 0x1afad227, 34 },
    { 0, 0 },
    { 0xca0ad22a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bad22d, 17 },
    { 0, 0 },
    { 0x5e7ad22f, 23 },
    { 0, 0 },
    { 0x434ad231, 76 },
    { 0, 0 },
    { 0xbf4ad233, 69 },
    { 0x97bad233, 46 },
    { 0, 0 },
    { 0xf44ad236, 55 },
    { 0, 0 },
    { 0x3a7ad238, 90 },
    { 0, 0 },
    { 0x46ead23a, 90 },
    { 0x395ad23b, 87 },
    { 0xb55ad23c, 98 },
    { 0xf86ad23c, 76 },
    { 0xf30ad23e, 90 },
    { 0x3bbad23c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6ad242, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5ad24c, 90 },
    { 0, 0 },
    { 0x7e1ad24e, 33 },
    { 0xd7dad24f, 95 },
    { 0xdb2ad250, 85 },
    { 0x42cad24f, 90 },
    { 0xbffad252, 95 },
    { 0x722ad252, 95 },
    { 0xec6ad254, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43ad257, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bead25f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ead266, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3ad26d, 76 },
    { 0, 0 },
    { 0x44ead26f, 61 },
    { 0x6a6ad270, 23 },
    { 0xb8cad271, 34 },
    { 0, 0 },
    { 0xb2ead273, 96 },
    { 0x696ad274, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2ad27a, 46 },
    { 0xbc9ad27b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bad27e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56ad286, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23ad289, 99 },
    { 0x20bad28a, 76 },
    { 0x54ead28b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8ad28e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x768ad291, 61 },
    { 0x93dad292, 66 },
    { 0, 0 },
    { 0xa86ad294, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14aad298, 17 },
    { 0xfa0ad299, 61 },
    { 0x82cad298, 33 },
    { 0x6f3ad29b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfad29e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbead2a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x833ad2a4, 95 },
    { 0, 0 },
    { 0x1cdad2a6, 33 },
    { 0x771ad2a6, 93 },
    { 0xd4ead2a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad2ae, 84 },
    { 0x8e0ad2ae, 100 },
    { 0x228ad2b0, 93 },
    { 0, 0 },
    { 0x9c0ad2b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ad2b5, 100 },
    { 0, 0 },
    { 0x141ad2b7, 90 },
    { 0xc24ad2b7, 46 },
    { 0, 0 },
    { 0x11ad2ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dad2c0, 23 },
    { 0, 0 },
    { 0x380ad2c2, 43 },
    { 0xe40ad2c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112ad2c7, 17 },
    { 0xf3bad2c7, 43 },
    { 0, 0 },
    { 0x760ad2ca, 95 },
    { 0xac4ad2cb, 90 },
    { 0x636ad2ca, 95 },
    { 0x33aad2cd, 99 },
    { 0x7ddad2ce, 98 },
    { 0, 0 },
    { 0x872ad2d0, 76 },
    { 0, 0 },
    { 0xea1ad2d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ad2d6, 17 },
    { 0x436ad2d7, 42 },
    { 0, 0 },
    { 0x7d0ad2d9, 95 },
    { 0, 0 },
    { 0x66fad2db, 81 },
    { 0x51bad2dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adad2e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ad2e3, 61 },
    { 0xb1ead2e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fad2e7, 90 },
    { 0xc93ad2e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6ad2eb, 23 },
    { 0xe5fad2ec, 33 },
    { 0x5c8ad2ed, 23 },
    { 0x129ad2ee, 43 },
    { 0x9c0ad2ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ad2f2, 34 },
    { 0x5f6ad2f3, 33 },
    { 0x7d3ad2f3, 56 },
    { 0xa32ad2f3, 90 },
    { 0x873ad2f3, 96 },
    { 0x9adad2f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719ad2fd, 90 },
    { 0xe29ad2fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447ad30a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ad30f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ad313, 81 },
    { 0x1d5ad314, 100 },
    { 0, 0 },
    { 0x753ad316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aad31b, 33 },
    { 0x37ead31c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453ad321, 88 },
    { 0x86cad322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9ad331, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32ad337, 33 },
    { 0, 0 },
    { 0xfe6ad339, 23 },
    { 0x56ead33a, 33 },
    { 0x2f2ad33b, 99 },
    { 0x3e9ad33a, 87 },
    { 0xe2dad33b, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bad340, 65 },
    { 0xe2ead340, 61 },
    { 0x9b7ad342, 33 },
    { 0xe72ad342, 34 },
    { 0x841ad344, 98 },
    { 0x9fcad345, 98 },
    { 0x15ead341, 38 },
    { 0, 0 },
    { 0xaf8ad348, 98 },
    { 0, 0 },
    { 0xcead34a, 76 },
    { 0x94dad34a, 23 },
    { 0, 0 },
    { 0x6cbad34d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932ad352, 61 },
    { 0, 0 },
    { 0x7d8ad354, 90 },
    { 0, 0 },
    { 0xccbad356, 90 },
    { 0x48ad357, 61 },
    { 0, 0 },
    { 0xcf4ad359, 61 },
    { 0x820ad35a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aad35d, 33 },
    { 0xd1dad35e, 46 },
    { 0x2d8ad35f, 90 },
    { 0x4dcad360, 52 },
    { 0, 0 },
    { 0x8f3ad362, 76 },
    { 0, 0 },
    { 0x3cbad364, 33 },
    { 0x3efad364, 61 },
    { 0, 0 },
    { 0x2ad367, 90 },
    { 0xafead367, 66 },
    { 0x6abad369, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x588ad36c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44ad370, 61 },
    { 0x71ad370, 23 },
    { 0xeffad372, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02ad375, 90 },
    { 0, 0 },
    { 0xc11ad377, 28 },
    { 0, 0 },
    { 0x413ad379, 33 },
    { 0x35fad37a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1ad37d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8ad380, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cad386, 23 },
    { 0xd12ad386, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ad38b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bad38f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862ad398, 61 },
    { 0xb8ad398, 99 },
    { 0, 0 },
    { 0x42aad39b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fad3a0, 33 },
    { 0, 0 },
    { 0x4e9ad3a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aad3a6, 93 },
    { 0x794ad3a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ad3ac, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212ad3b2, 99 },
    { 0xaccad3b3, 90 },
    { 0, 0 },
    { 0xc3fad3b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174ad3c0, 90 },
    { 0, 0 },
    { 0xb5dad3c2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ad3c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58ad3cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cad3d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad3db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9ad3e3, 46 },
    { 0, 0 },
    { 0xb45ad3e5, 99 },
    { 0xeb2ad3e5, 33 },
    { 0, 0 },
    { 0x900ad3e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ad3ef, 98 },
    { 0x2c9ad3ef, 98 },
    { 0x304ad3f1, 55 },
    { 0, 0 },
    { 0xd5cad3f3, 90 },
    { 0, 0 },
    { 0xa90ad3f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x632ad3f8, 17 },
    { 0x556ad3f9, 66 },
    { 0xf8dad3fa, 33 },
    { 0x53dad3f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69ad3fe, 66 },
    { 0xa3ead3fe, 61 },
    { 0, 0 },
    { 0xa81ad401, 23 },
    { 0, 0 },
    { 0x1d1ad403, 33 },
    { 0xc6cad404, 61 },
    { 0x5c4ad403, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721ad409, 95 },
    { 0xe21ad40a, 76 },
    { 0x749ad40a, 95 },
    { 0x946ad40c, 55 },
    { 0, 0 },
    { 0x798ad40e, 34 },
    { 0xc58ad40f, 90 },
    { 0x805ad40f, 34 },
    { 0xc04ad40f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556ad41a, 23 },
    { 0x1a1ad41b, 17 },
    { 0, 0 },
    { 0x315ad41d, 95 },
    { 0xc9aad41d, 95 },
    { 0xa5bad41d, 93 },
    { 0xb12ad41d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67ad423, 90 },
    { 0x374ad423, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383ad42f, 46 },
    { 0, 0 },
    { 0x8edad431, 90 },
    { 0xad3ad431, 23 },
    { 0x781ad433, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baad439, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19ad43d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efad440, 23 },
    { 0xca3ad440, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720ad445, 75 },
    { 0x7ffad446, 47 },
    { 0x1c4ad446, 95 },
    { 0, 0 },
    { 0x560ad449, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798ad450, 90 },
    { 0x112ad451, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2ad457, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4ad45a, 61 },
    { 0x844ad45b, 99 },
    { 0, 0 },
    { 0xfd2ad45d, 90 },
    { 0xb31ad45e, 34 },
    { 0x834ad45f, 67 },
    { 0x6e1ad460, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aead464, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcad46c, 23 },
    { 0x31ad46d, 95 },
    { 0x65cad46e, 46 },
    { 0x31aad46c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dad473, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ad486, 90 },
    { 0xa66ad487, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7ad48c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ad48f, 34 },
    { 0x68ad490, 98 },
    { 0x8b9ad491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136ad496, 38 },
    { 0xf89ad497, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x841ad49a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fad49e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267ad4a7, 90 },
    { 0xa29ad4a8, 93 },
    { 0x208ad4a8, 97 },
    { 0x49dad4a9, 33 },
    { 0xce2ad4ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aad4b3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388ad4b9, 85 },
    { 0x926ad4ba, 61 },
    { 0x442ad4ba, 61 },
    { 0x4aad4bc, 87 },
    { 0x946ad4bd, 85 },
    { 0xc98ad4be, 40 },
    { 0x1c2ad4bd, 43 },
    { 0xd1bad4c0, 98 },
    { 0, 0 },
    { 0x68cad4c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdad4c7, 95 },
    { 0, 0 },
    { 0x189ad4c9, 76 },
    { 0x216ad4c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ad4cd, 55 },
    { 0x9c0ad4ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378ad4d2, 76 },
    { 0xffcad4d3, 95 },
    { 0, 0 },
    { 0xf62ad4d5, 33 },
    { 0x561ad4d5, 33 },
    { 0x1abad4d7, 33 },
    { 0x9fead4d8, 78 },
    { 0xa0fad4d5, 33 },
    { 0x581ad4d9, 46 },
    { 0x237ad4db, 95 },
    { 0, 0 },
    { 0x370ad4dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fad4e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831ad4e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bad4ed, 100 },
    { 0xfd8ad4ee, 67 },
    { 0xecfad4ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efad4f2, 90 },
    { 0, 0 },
    { 0x50dad4f4, 65 },
    { 0, 0 },
    { 0x941ad4f6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ad4f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dad4fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ad4ff, 75 },
    { 0, 0 },
    { 0xef1ad501, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ad509, 33 },
    { 0x80ead50a, 77 },
    { 0x916ad50a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dad512, 90 },
    { 0xf79ad513, 55 },
    { 0x18fad513, 61 },
    { 0x344ad513, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40ad518, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6ad523, 90 },
    { 0xff8ad524, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984ad52a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89ad52e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39ad531, 88 },
    { 0x7b1ad532, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562ad539, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7ad540, 90 },
    { 0xc5bad540, 61 },
    { 0, 0 },
    { 0x32bad543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ead546, 91 },
    { 0xae5ad546, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aad54c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfad54f, 100 },
    { 0xc66ad550, 99 },
    { 0xf42ad54f, 95 },
    { 0x70fad552, 90 },
    { 0x8c5ad553, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafad55b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cad560, 33 },
    { 0x4a0ad561, 38 },
    { 0x21bad561, 33 },
    { 0, 0 },
    { 0x530ad564, 55 },
    { 0, 0 },
    { 0x43aad566, 46 },
    { 0, 0 },
    { 0x4b6ad568, 33 },
    { 0x5adad568, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x385ad56c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4ad570, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2ad578, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cad580, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36ad585, 33 },
    { 0x5b3ad585, 90 },
    { 0, 0 },
    { 0xc0fad588, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88ad58e, 90 },
    { 0x914ad58f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad593, 90 },
    { 0xd3ead594, 43 },
    { 0x337ad595, 56 },
    { 0x2f4ad593, 61 },
    { 0, 0 },
    { 0x402ad598, 52 },
    { 0x94ad598, 90 },
    { 0, 0 },
    { 0xe39ad59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ad5a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fad5ad, 33 },
    { 0, 0 },
    { 0xca0ad5af, 61 },
    { 0xb43ad5b0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694ad5b6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x813ad5b9, 23 },
    { 0, 0 },
    { 0x312ad5bb, 55 },
    { 0x104ad5bc, 95 },
    { 0x9ad5bd, 90 },
    { 0x72dad5be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600ad5c7, 100 },
    { 0x4bfad5c7, 90 },
    { 0, 0 },
    { 0x115ad5ca, 19 },
    { 0xcd6ad5ca, 69 },
    { 0, 0 },
    { 0x9b5ad5cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ad5d0, 95 },
    { 0x2c8ad5d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dad5d4, 55 },
    { 0, 0 },
    { 0x1baad5d6, 90 },
    { 0xaaaad5d7, 33 },
    { 0x4caad5d6, 95 },
    { 0xeb6ad5d7, 12 },
    { 0, 0 },
    { 0x4f5ad5db, 88 },
    { 0x4c9ad5dc, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80ad5e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9faad5ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x711ad5ef, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ad5f5, 90 },
    { 0x777ad5f6, 76 },
    { 0xb22ad5f6, 61 },
    { 0xc7dad5f8, 34 },
    { 0xfb4ad5f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x544ad5fc, 78 },
    { 0x58ead5fc, 52 },
    { 0x8a5ad5fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbad605, 34 },
    { 0, 0 },
    { 0xec8ad607, 78 },
    { 0, 0 },
    { 0x247ad609, 33 },
    { 0xf4bad60a, 61 },
    { 0x8d3ad60b, 61 },
    { 0x803ad60b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19ad612, 61 },
    { 0xd47ad612, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baad616, 23 },
    { 0, 0 },
    { 0xc5ad618, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ad61b, 59 },
    { 0, 0 },
    { 0x4e4ad61d, 43 },
    { 0, 0 },
    { 0x72aad61f, 100 },
    { 0xacdad620, 64 },
    { 0xb9cad621, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddad628, 100 },
    { 0xeadad629, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aad62d, 23 },
    { 0x9f6ad62d, 95 },
    { 0x686ad62f, 94 },
    { 0xffcad62f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8ad636, 90 },
    { 0xa05ad637, 90 },
    { 0x4f5ad638, 98 },
    { 0xe90ad639, 77 },
    { 0, 0 },
    { 0xa75ad63b, 98 },
    { 0x8f7ad63c, 90 },
    { 0, 0 },
    { 0x687ad63e, 66 },
    { 0x667ad63e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7ad647, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ad64b, 76 },
    { 0xd7aad64b, 55 },
    { 0, 0 },
    { 0xbfbad64e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682ad658, 23 },
    { 0xd9bad658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8ad65f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ad663, 55 },
    { 0x7fad664, 85 },
    { 0x54fad663, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ad66b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ad66e, 90 },
    { 0, 0 },
    { 0x4afad670, 90 },
    { 0, 0 },
    { 0x63cad672, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfad675, 52 },
    { 0, 0 },
    { 0xe20ad677, 55 },
    { 0, 0 },
    { 0x2f9ad679, 43 },
    { 0x38ead67a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaad67d, 90 },
    { 0, 0 },
    { 0xbf7ad67f, 85 },
    { 0xa11ad680, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1ad685, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5ad68b, 98 },
    { 0x7e5ad68c, 90 },
    { 0, 0 },
    { 0x1fdad68e, 76 },
    { 0xe60ad68f, 61 },
    { 0x1f6ad68e, 61 },
    { 0x68cad690, 88 },
    { 0, 0 },
    { 0x2f9ad693, 34 },
    { 0xa9dad694, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bad698, 90 },
    { 0xfb1ad698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6ad69f, 98 },
    { 0, 0 },
    { 0x43ead6a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ad6a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5ad6a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95ad6af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dad6b2, 95 },
    { 0x4d6ad6b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ad6b6, 33 },
    { 0, 0 },
    { 0x843ad6b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35ad6c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddad6cd, 90 },
    { 0, 0 },
    { 0x70cad6cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbad6d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66ad6d9, 95 },
    { 0x319ad6d9, 95 },
    { 0, 0 },
    { 0xdedad6dc, 93 },
    { 0x296ad6dd, 90 },
    { 0xceaad6dd, 33 },
    { 0x6b2ad6dc, 33 },
    { 0xb37ad6e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adad6e4, 78 },
    { 0, 0 },
    { 0xa48ad6e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237ad6ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ad6ef, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dad6f4, 76 },
    { 0, 0 },
    { 0x704ad6f6, 90 },
    { 0xeb5ad6f7, 87 },
    { 0xaddad6f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ad6fc, 98 },
    { 0x45aad6fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6ad702, 33 },
    { 0, 0 },
    { 0x7b8ad704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ad709, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dad70c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad710, 95 },
    { 0xf9ad711, 33 },
    { 0xfeead710, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x64aad715, 33 },
    { 0, 0 },
    { 0xaddad717, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fead71a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ad71e, 76 },
    { 0, 0 },
    { 0x8aaad720, 23 },
    { 0, 0 },
    { 0x22cad722, 93 },
    { 0x7d9ad722, 33 },
    { 0x88dad724, 87 },
    { 0, 0 },
    { 0xacad726, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148ad72b, 84 },
    { 0x8c2ad72c, 81 },
    { 0xfe7ad72d, 49 },
    { 0, 0 },
    { 0x507ad72f, 23 },
    { 0xf83ad730, 78 },
    { 0xdc0ad731, 99 },
    { 0x5c5ad730, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57ad736, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02ad73a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ead73f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6ad743, 93 },
    { 0, 0 },
    { 0xb1aad745, 85 },
    { 0x132ad745, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aad749, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91ad74c, 23 },
    { 0x260ad74c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbad750, 98 },
    { 0x1d7ad750, 55 },
    { 0x73cad750, 87 },
    { 0xfebad751, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90ad757, 84 },
    { 0x71cad758, 77 },
    { 0xc0ad759, 23 },
    { 0, 0 },
    { 0xa3ead75b, 81 },
    { 0x8baad75b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dad75f, 61 },
    { 0x7b9ad760, 76 },
    { 0xcbad760, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cad765, 23 },
    { 0, 0 },
    { 0x25ead767, 98 },
    { 0xe17ad768, 33 },
    { 0, 0 },
    { 0x45ead76a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227ad76e, 81 },
    { 0, 0 },
    { 0x298ad770, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331ad774, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dad778, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7ad77c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ad781, 90 },
    { 0x143ad782, 61 },
    { 0xe21ad783, 86 },
    { 0x921ad783, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afad788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fad791, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x772ad794, 91 },
    { 0x7f5ad794, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acad798, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad79c, 33 },
    { 0, 0 },
    { 0xef3ad79e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1ad7a2, 90 },
    { 0x741ad7a3, 61 },
    { 0x30fad7a4, 88 },
    { 0xd4ad7a5, 90 },
    { 0xacbad7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ad7ae, 95 },
    { 0, 0 },
    { 0xbaead7b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ad7b6, 61 },
    { 0x1f3ad7b6, 34 },
    { 0, 0 },
    { 0x8e7ad7b9, 90 },
    { 0, 0 },
    { 0x139ad7bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ad7be, 95 },
    { 0x7d7ad7bf, 95 },
    { 0x865ad7be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0ad7c9, 76 },
    { 0x16cad7c9, 66 },
    { 0, 0 },
    { 0x985ad7cc, 98 },
    { 0xc9fad7cc, 33 },
    { 0xc55ad7cc, 33 },
    { 0, 0 },
    { 0xd27ad7d0, 84 },
    { 0xb6bad7d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x929ad7d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cad7db, 90 },
    { 0x8bfad7dc, 23 },
    { 0x4ebad7dd, 23 },
    { 0x513ad7dd, 95 },
    { 0x51fad7dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02ad7e2, 76 },
    { 0x733ad7e3, 67 },
    { 0x6e5ad7e3, 17 },
    { 0x6d8ad7e5, 98 },
    { 0, 0 },
    { 0x344ad7e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bad7eb, 23 },
    { 0x3c6ad7ec, 90 },
    { 0x1faad7ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dad7f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ad7f4, 46 },
    { 0x247ad7f4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9ad7f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56ad7fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1ad801, 98 },
    { 0xb18ad801, 81 },
    { 0, 0 },
    { 0x632ad804, 46 },
    { 0, 0 },
    { 0xbbdad806, 98 },
    { 0xa7cad807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aad815, 98 },
    { 0, 0 },
    { 0xaf8ad817, 76 },
    { 0x4a1ad818, 33 },
    { 0, 0 },
    { 0xc7dad81a, 33 },
    { 0xce5ad81b, 80 },
    { 0x7efad81c, 46 },
    { 0x50ad81d, 98 },
    { 0x51cad81a, 55 },
    { 0x5e3ad81b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bad823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x145ad826, 95 },
    { 0x902ad827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ad82d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ad830, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ad833, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cad836, 33 },
    { 0x18ad837, 33 },
    { 0x8f2ad838, 95 },
    { 0, 0 },
    { 0x367ad83a, 98 },
    { 0x36dad83b, 76 },
    { 0x4b5ad83b, 34 },
    { 0xe8ead83d, 23 },
    { 0xa90ad83e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41ad841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92ad845, 61 },
    { 0xd2cad846, 95 },
    { 0xa07ad847, 33 },
    { 0xe22ad848, 98 },
    { 0x712ad848, 23 },
    { 0, 0 },
    { 0x220ad84b, 90 },
    { 0, 0 },
    { 0xdc0ad84d, 66 },
    { 0, 0 },
    { 0xc42ad84f, 23 },
    { 0, 0 },
    { 0xdd8ad851, 61 },
    { 0, 0 },
    { 0x450ad853, 61 },
    { 0xb5cad854, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fad857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5ad85b, 33 },
    { 0xde3ad85c, 23 },
    { 0xe1bad85c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128ad863, 55 },
    { 0x495ad864, 33 },
    { 0x179ad864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946ad869, 90 },
    { 0x7fad86a, 46 },
    { 0, 0 },
    { 0x41bad86c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ad86f, 23 },
    { 0xc50ad86f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x217ad873, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29ad876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffad87c, 33 },
    { 0x9ad87d, 9 },
    { 0x606ad87e, 90 },
    { 0x2cad87f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9ad882, 90 },
    { 0x367ad883, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6ad887, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ad88a, 33 },
    { 0x64dad88b, 66 },
    { 0xe23ad88c, 69 },
    { 0x7f4ad88d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57ad890, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edad895, 98 },
    { 0x21ad896, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ead89a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2ad89e, 98 },
    { 0x9f1ad89f, 61 },
    { 0, 0 },
    { 0x5fcad8a1, 61 },
    { 0xa08ad8a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235ad8ab, 90 },
    { 0xb9ead8ab, 23 },
    { 0xdd7ad8ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3ad8b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcad8b4, 98 },
    { 0, 0 },
    { 0xd33ad8b6, 61 },
    { 0x358ad8b7, 87 },
    { 0xb6bad8b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cad8bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7ad8bf, 61 },
    { 0xa45ad8c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38ad8c4, 95 },
    { 0xf88ad8c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5ad8c9, 98 },
    { 0xc06ad8ca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cad8cf, 84 },
    { 0x2f7ad8d0, 34 },
    { 0, 0 },
    { 0x32cad8d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bad8d5, 78 },
    { 0, 0 },
    { 0xaaaad8d7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0ad8e2, 100 },
    { 0x179ad8e3, 33 },
    { 0x399ad8e4, 23 },
    { 0x35ad8e4, 91 },
    { 0xfb6ad8e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8ad8f0, 81 },
    { 0xcad8f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbad8f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ad8fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ad8ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fad905, 90 },
    { 0x45dad906, 33 },
    { 0xbcead907, 46 },
    { 0xb6dad908, 99 },
    { 0x273ad905, 23 },
    { 0, 0 },
    { 0xa7ead90b, 76 },
    { 0xbd1ad90c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacad90f, 76 },
    { 0xfbad910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ad914, 61 },
    { 0xfcbad914, 98 },
    { 0xf67ad916, 90 },
    { 0x624ad917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ad91a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cad91e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804ad924, 91 },
    { 0xb28ad924, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7aad928, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ad92c, 55 },
    { 0x632ad92c, 91 },
    { 0x8ccad92e, 91 },
    { 0, 0 },
    { 0x9e8ad930, 76 },
    { 0x2bcad930, 61 },
    { 0, 0 },
    { 0xf0dad933, 34 },
    { 0, 0 },
    { 0x7bfad935, 23 },
    { 0, 0 },
    { 0x6acad937, 33 },
    { 0xfedad938, 43 },
    { 0x498ad938, 82 },
    { 0xe72ad93a, 95 },
    { 0x6a3ad937, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ad93e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcad942, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccead949, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71ad951, 61 },
    { 0x4d5ad951, 98 },
    { 0x615ad952, 23 },
    { 0x664ad954, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ad962, 33 },
    { 0x848ad962, 95 },
    { 0, 0 },
    { 0x273ad965, 17 },
    { 0, 0 },
    { 0xa1ad967, 33 },
    { 0xd03ad967, 90 },
    { 0xcfead968, 94 },
    { 0xd8dad96a, 98 },
    { 0, 0 },
    { 0x637ad96c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fad970, 23 },
    { 0xdb2ad970, 67 },
    { 0x597ad971, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146ad97a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302ad97f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ad982, 33 },
    { 0xc15ad983, 95 },
    { 0xe2ad984, 17 },
    { 0xd4ad984, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ad988, 90 },
    { 0xe38ad989, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ad98e, 100 },
    { 0xdc2ad98f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386ad993, 76 },
    { 0x363ad993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9ad999, 46 },
    { 0, 0 },
    { 0xb75ad99b, 90 },
    { 0, 0 },
    { 0x766ad99d, 23 },
    { 0xb8ad99e, 46 },
    { 0xccad99f, 75 },
    { 0xf94ad99e, 17 },
    { 0xc40ad99d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ad9ab, 38 },
    { 0xd6fad9ab, 55 },
    { 0x6d4ad9ad, 78 },
    { 0x2bead9ae, 23 },
    { 0, 0 },
    { 0x14fad9b0, 43 },
    { 0xbdead9b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdad9b7, 33 },
    { 0, 0 },
    { 0xeadad9b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ad9bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9daad9c4, 76 },
    { 0xa09ad9c4, 61 },
    { 0, 0 },
    { 0x22bad9c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7ad9cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6ad9ce, 23 },
    { 0x4e1ad9cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x403ad9d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dad9d5, 23 },
    { 0, 0 },
    { 0xe58ad9d7, 55 },
    { 0xb44ad9d8, 98 },
    { 0xb92ad9d9, 95 },
    { 0x8dfad9da, 61 },
    { 0x213ad9db, 85 },
    { 0, 0 },
    { 0x8bcad9dd, 34 },
    { 0xd52ad9de, 93 },
    { 0x9f6ad9de, 52 },
    { 0x55ad9de, 33 },
    { 0, 0 },
    { 0x5b1ad9e2, 90 },
    { 0, 0 },
    { 0x503ad9e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0ad9f0, 95 },
    { 0x290ad9f1, 23 },
    { 0, 0 },
    { 0xe81ad9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fad9f7, 95 },
    { 0, 0 },
    { 0x604ad9f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0ad9fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bada00, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ada04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efada0a, 23 },
    { 0, 0 },
    { 0x8c6ada0c, 98 },
    { 0, 0 },
    { 0xeaada0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18ada11, 90 },
    { 0xbc8ada11, 38 },
    { 0x700ada12, 43 },
    { 0, 0 },
    { 0xc27ada15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523ada1d, 90 },
    { 0xeb7ada1e, 55 },
    { 0xf28ada1d, 90 },
    { 0xdd7ada1e, 33 },
    { 0xf89ada20, 23 },
    { 0x1c8ada21, 95 },
    { 0x22bada1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6ada26, 88 },
    { 0x52cada27, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06ada2b, 33 },
    { 0, 0 },
    { 0x793ada2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9aada30, 68 },
    { 0x66ada31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581ada36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bada3a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x973ada3d, 90 },
    { 0xaa3ada3d, 75 },
    { 0xa23ada3f, 88 },
    { 0x46dada40, 90 },
    { 0, 0 },
    { 0x73dada42, 46 },
    { 0xfabada43, 23 },
    { 0x4d0ada44, 23 },
    { 0x4a6ada45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75ada4c, 84 },
    { 0, 0 },
    { 0xfd5ada4e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x28aada51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646ada58, 33 },
    { 0, 0 },
    { 0x273ada5a, 43 },
    { 0x424ada5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x65aada5e, 95 },
    { 0, 0 },
    { 0xba2ada60, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cada6a, 53 },
    { 0xaffada6b, 90 },
    { 0xdb1ada6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42aada6f, 23 },
    { 0, 0 },
    { 0x7ffada71, 23 },
    { 0, 0 },
    { 0xe1bada73, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fada76, 90 },
    { 0x1c4ada77, 33 },
    { 0xa9ada77, 33 },
    { 0, 0 },
    { 0xac0ada7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8ada7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bada81, 98 },
    { 0x37aada81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ada86, 43 },
    { 0, 0 },
    { 0xb1dada88, 23 },
    { 0, 0 },
    { 0xf3fada8a, 76 },
    { 0, 0 },
    { 0xb63ada8c, 43 },
    { 0xd98ada8d, 61 },
    { 0, 0 },
    { 0x406ada8f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381ada93, 43 },
    { 0, 0 },
    { 0xb45ada95, 61 },
    { 0x9c2ada96, 90 },
    { 0xbdfada96, 66 },
    { 0x2b1ada98, 23 },
    { 0x8cdada99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfada9c, 90 },
    { 0x14fada9c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00adaa2, 90 },
    { 0x375adaa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51adaa6, 61 },
    { 0xe5dadaa7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0adaae, 95 },
    { 0, 0 },
    { 0xdbbadab0, 33 },
    { 0, 0 },
    { 0xcb6adab2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eadab6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9adabb, 82 },
    { 0xcd2adabc, 95 },
    { 0xe79adabc, 33 },
    { 0x187adabd, 95 },
    { 0, 0 },
    { 0x617adac0, 95 },
    { 0, 0 },
    { 0x8d5adac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adadacb, 23 },
    { 0xe22adacc, 90 },
    { 0x2badacd, 61 },
    { 0x647adace, 91 },
    { 0x9ddadacf, 99 },
    { 0x56badace, 43 },
    { 0x8fadacd, 61 },
    { 0x3d9adad2, 61 },
    { 0, 0 },
    { 0x20cadad4, 61 },
    { 0, 0 },
    { 0x84fadad6, 98 },
    { 0x2aadad7, 98 },
    { 0xa94adad7, 56 },
    { 0, 0 },
    { 0x1d1adada, 95 },
    { 0x422adadb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109adae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66adae3, 81 },
    { 0xe88adae4, 90 },
    { 0xbdbadae5, 98 },
    { 0x3c3adae5, 98 },
    { 0x26adae6, 90 },
    { 0xac2adae8, 61 },
    { 0xd99adae9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dadaef, 95 },
    { 0, 0 },
    { 0xdcbadaf1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94adaf4, 88 },
    { 0xc73adaf5, 87 },
    { 0x130adaf5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2adafa, 43 },
    { 0xf09adafb, 23 },
    { 0xcacadafc, 67 },
    { 0x8ecadafd, 23 },
    { 0x307adafe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baadb02, 84 },
    { 0xc70adb03, 61 },
    { 0x212adb04, 66 },
    { 0, 0 },
    { 0x588adb06, 78 },
    { 0x29cadb07, 91 },
    { 0xb9cadb08, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aadb12, 78 },
    { 0, 0 },
    { 0xbc2adb14, 38 },
    { 0, 0 },
    { 0x43fadb16, 10 },
    { 0, 0 },
    { 0x26adb18, 90 },
    { 0x5eadb18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeadb20, 33 },
    { 0xd51adb21, 76 },
    { 0xa08adb22, 98 },
    { 0xb81adb23, 65 },
    { 0xcc3adb23, 66 },
    { 0x2f1adb23, 90 },
    { 0, 0 },
    { 0x17eadb27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x178adb2a, 61 },
    { 0xbe7adb2b, 90 },
    { 0xd38adb2b, 33 },
    { 0, 0 },
    { 0x5c1adb2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dadb32, 77 },
    { 0x24cadb33, 87 },
    { 0x4efadb33, 85 },
    { 0x1a7adb32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x679adb38, 34 },
    { 0x38badb39, 61 },
    { 0x587adb38, 61 },
    { 0x422adb3b, 98 },
    { 0, 0 },
    { 0xd25adb3d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaadb42, 55 },
    { 0x2c9adb43, 43 },
    { 0xd19adb44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x540adb47, 33 },
    { 0x2ffadb48, 98 },
    { 0xf49adb49, 61 },
    { 0xad3adb4a, 23 },
    { 0x484adb4a, 95 },
    { 0x34adb4c, 78 },
    { 0xb2adb49, 23 },
    { 0xffadb4e, 90 },
    { 0x796adb48, 61 },
    { 0x3c8adb50, 76 },
    { 0x1a5adb51, 90 },
    { 0xc5adb52, 23 },
    { 0x6dbadb51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8adb5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cadb63, 23 },
    { 0xe8aadb64, 33 },
    { 0, 0 },
    { 0xce4adb66, 33 },
    { 0x653adb66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22adb6d, 40 },
    { 0, 0 },
    { 0xa00adb6f, 33 },
    { 0, 0 },
    { 0x266adb71, 52 },
    { 0, 0 },
    { 0x830adb73, 33 },
    { 0, 0 },
    { 0xcfbadb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4adb7c, 34 },
    { 0x646adb7d, 61 },
    { 0xa4adb7e, 61 },
    { 0x757adb7f, 93 },
    { 0xc33adb7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65adb84, 95 },
    { 0, 0 },
    { 0x801adb86, 90 },
    { 0x3f3adb86, 91 },
    { 0x15badb87, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746adb91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cadba0, 88 },
    { 0x657adba0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92aadba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfadba8, 99 },
    { 0xe45adba8, 76 },
    { 0xf09adbaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adadbae, 43 },
    { 0, 0 },
    { 0xd3badbb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7adbb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406adbb9, 98 },
    { 0, 0 },
    { 0x2faadbbb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9adbc0, 23 },
    { 0x288adbc1, 43 },
    { 0xe06adbc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29adbc5, 61 },
    { 0x5d8adbc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55adbca, 90 },
    { 0xa25adbca, 55 },
    { 0x54cadbcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0adbd0, 46 },
    { 0x1f9adbd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x982adbd4, 88 },
    { 0xbdfadbd5, 98 },
    { 0x233adbd4, 98 },
    { 0xaaadbd5, 95 },
    { 0, 0 },
    { 0xb7dadbd9, 55 },
    { 0xa9adbda, 61 },
    { 0, 0 },
    { 0x4b4adbdc, 61 },
    { 0, 0 },
    { 0x624adbde, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12adbe3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3adbe7, 33 },
    { 0x4c5adbe8, 61 },
    { 0, 0 },
    { 0x2d5adbea, 91 },
    { 0x6d4adbeb, 33 },
    { 0xc14adbea, 61 },
    { 0, 0 },
    { 0x8c7adbee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebadbf1, 33 },
    { 0, 0 },
    { 0x2a6adbf3, 76 },
    { 0x9f7adbf4, 95 },
    { 0, 0 },
    { 0x8cbadbf6, 68 },
    { 0xfaaadbf7, 99 },
    { 0x22aadbf7, 93 },
    { 0xf05adbf7, 75 },
    { 0x25eadbfa, 91 },
    { 0x8ffadbfb, 98 },
    { 0xbb2adbf8, 61 },
    { 0x397adbf7, 23 },
    { 0xdcadbfe, 23 },
    { 0x2d5adbff, 81 },
    { 0x579adbf7, 95 },
    { 0, 0 },
    { 0x6dcadc02, 33 },
    { 0x4cdadc03, 81 },
    { 0x638adc03, 33 },
    { 0x497adc04, 90 },
    { 0xd64adc03, 99 },
    { 0, 0 },
    { 0xaefadc08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aadc0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37adc15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffadc1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574adc1f, 90 },
    { 0, 0 },
    { 0x646adc21, 93 },
    { 0x868adc22, 34 },
    { 0x934adc23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4adc2b, 77 },
    { 0x6b3adc2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84adc30, 90 },
    { 0xedadc30, 98 },
    { 0, 0 },
    { 0x2a6adc33, 90 },
    { 0x9d6adc34, 23 },
    { 0x98fadc34, 95 },
    { 0x6e5adc36, 76 },
    { 0xb0aadc37, 99 },
    { 0x2efadc38, 61 },
    { 0, 0 },
    { 0xf6badc3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6adc3e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dadc42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x461adc45, 43 },
    { 0x5a2adc46, 23 },
    { 0, 0 },
    { 0xf72adc48, 95 },
    { 0, 0 },
    { 0x99dadc4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdeadc4e, 33 },
    { 0xceadc4f, 98 },
    { 0x5b0adc50, 90 },
    { 0x9e6adc51, 100 },
    { 0x566adc52, 98 },
    { 0x35fadc50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9adc58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaadc5b, 66 },
    { 0xed6adc5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x572adc5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x470adc62, 98 },
    { 0x5eeadc63, 40 },
    { 0x6d0adc64, 61 },
    { 0xb4cadc64, 76 },
    { 0xf69adc62, 76 },
    { 0x9d9adc63, 90 },
    { 0xa5aadc64, 95 },
    { 0, 0 },
    { 0x172adc6a, 46 },
    { 0, 0 },
    { 0x597adc6c, 95 },
    { 0xc32adc6c, 93 },
    { 0, 0 },
    { 0xa4cadc6f, 99 },
    { 0x5c0adc70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50adc74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0adc79, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eadc7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efadc84, 43 },
    { 0x4e1adc84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebadc8a, 76 },
    { 0xfedadc8b, 46 },
    { 0, 0 },
    { 0x23badc8d, 23 },
    { 0, 0 },
    { 0x72badc8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eadc98, 33 },
    { 0, 0 },
    { 0x616adc9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34adc9d, 91 },
    { 0xd4eadc9e, 34 },
    { 0x20dadc9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7adca7, 90 },
    { 0xdedadca7, 75 },
    { 0xec0adca9, 52 },
    { 0, 0 },
    { 0x3a7adcab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726adcaf, 55 },
    { 0xa6fadcb0, 90 },
    { 0, 0 },
    { 0xd84adcb2, 55 },
    { 0x6cdadcb3, 91 },
    { 0xb74adcb4, 87 },
    { 0x35dadcb4, 95 },
    { 0x25cadcb3, 93 },
    { 0xdc2adcb7, 33 },
    { 0xda3adcb7, 95 },
    { 0xceadcb9, 90 },
    { 0x524adcb7, 95 },
    { 0, 0 },
    { 0x31fadcbc, 61 },
    { 0x392adcbc, 67 },
    { 0x938adcbc, 90 },
    { 0, 0 },
    { 0x939adcc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfadcc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5adcc8, 34 },
    { 0xeecadcc9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2adcd0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdadcd3, 95 },
    { 0x845adcd4, 33 },
    { 0x79eadcd5, 61 },
    { 0x174adcd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcadcde, 76 },
    { 0, 0 },
    { 0x5a4adce0, 61 },
    { 0x664adce1, 98 },
    { 0x12badce1, 90 },
    { 0, 0 },
    { 0xb5cadce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588adce8, 23 },
    { 0x55dadce9, 91 },
    { 0xedeadcea, 99 },
    { 0x19eadceb, 52 },
    { 0x649adceb, 23 },
    { 0x7c4adcea, 23 },
    { 0, 0 },
    { 0x560adcef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x813adcf2, 99 },
    { 0x4b4adcf3, 33 },
    { 0, 0 },
    { 0x4a8adcf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962adcfd, 99 },
    { 0x988adcfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279add05, 90 },
    { 0x6dbadd06, 90 },
    { 0, 0 },
    { 0xf71add08, 67 },
    { 0x30cadd09, 98 },
    { 0xabfadd09, 17 },
    { 0, 0 },
    { 0x225add0c, 23 },
    { 0xd47add0d, 90 },
    { 0, 0 },
    { 0xa4cadd0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecadd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dadd1d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x673add20, 55 },
    { 0xa7cadd21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aadd2c, 90 },
    { 0xfdcadd2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81add32, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3add37, 95 },
    { 0xcd8add38, 95 },
    { 0xc50add39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74add3c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aadd40, 66 },
    { 0x99fadd41, 61 },
    { 0x80dadd42, 17 },
    { 0, 0 },
    { 0x30eadd44, 65 },
    { 0xa13add45, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95add48, 64 },
    { 0, 0 },
    { 0xb37add4a, 95 },
    { 0x8e4add4b, 61 },
    { 0xec5add4c, 62 },
    { 0xb53add4a, 23 },
    { 0xb8eadd4d, 90 },
    { 0x9fbadd4f, 61 },
    { 0x195add4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6add53, 95 },
    { 0xa03add54, 95 },
    { 0xe32add55, 33 },
    { 0xca0add54, 33 },
    { 0xb86add55, 95 },
    { 0xa67add56, 33 },
    { 0x563add55, 95 },
    { 0xa28add53, 91 },
    { 0, 0 },
    { 0xa91add5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92add5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x274add62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31add65, 33 },
    { 0xce7add66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfadd69, 61 },
    { 0x731add6a, 76 },
    { 0x927add69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfadd6e, 61 },
    { 0, 0 },
    { 0xa45add70, 83 },
    { 0x1eeadd71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227add75, 61 },
    { 0xb8dadd76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dadd7c, 76 },
    { 0, 0 },
    { 0xa4cadd7e, 61 },
    { 0xb73add7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252add85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6add8e, 67 },
    { 0xa4aadd8f, 85 },
    { 0x274add8e, 55 },
    { 0x5b4add8f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x486add94, 33 },
    { 0xa34add95, 46 },
    { 0x2d3add95, 94 },
    { 0x184add97, 61 },
    { 0, 0 },
    { 0xde0add99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbadd9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6add9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaadda7, 61 },
    { 0, 0 },
    { 0x184adda9, 23 },
    { 0x1bcaddaa, 34 },
    { 0x3eaaddab, 23 },
    { 0x465addac, 88 },
    { 0x171adda9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x378addb0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdaddb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7addb8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9addbe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4addc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3addc4, 61 },
    { 0x7b3addc4, 33 },
    { 0x365addc6, 43 },
    { 0x6e4addc7, 61 },
    { 0xa96addc6, 34 },
    { 0xdfaaddc9, 61 },
    { 0, 0 },
    { 0x9d9addcb, 55 },
    { 0, 0 },
    { 0x912addcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfaddd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48addda, 97 },
    { 0x221adddb, 90 },
    { 0, 0 },
    { 0x99adddd, 76 },
    { 0, 0 },
    { 0xb59adddf, 64 },
    { 0, 0 },
    { 0xd36adde1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3addec, 69 },
    { 0, 0 },
    { 0xdbcaddee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x305addf1, 91 },
    { 0, 0 },
    { 0x44addf3, 61 },
    { 0x1b8addf4, 46 },
    { 0xbb7addf5, 55 },
    { 0x443addf5, 61 },
    { 0xaa8addf7, 84 },
    { 0xe05addf4, 98 },
    { 0, 0 },
    { 0x52baddfa, 33 },
    { 0, 0 },
    { 0xce8addfc, 99 },
    { 0xf6baddfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594ade09, 23 },
    { 0xd58ade0a, 43 },
    { 0x54eade0b, 33 },
    { 0x29dade0b, 98 },
    { 0xcc9ade0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ade11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x630ade14, 76 },
    { 0xa92ade15, 76 },
    { 0x17cade16, 23 },
    { 0x62cade17, 101 },
    { 0x5d2ade18, 33 },
    { 0xc3dade19, 67 },
    { 0x7bbade1a, 55 },
    { 0xfd8ade15, 75 },
    { 0xfdbade14, 43 },
    { 0x180ade18, 23 },
    { 0x349ade1e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdade21, 87 },
    { 0xe9ade22, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73ade2d, 61 },
    { 0x71bade2e, 93 },
    { 0x1aade2e, 23 },
    { 0x742ade30, 90 },
    { 0x58eade30, 61 },
    { 0xb7fade32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964ade36, 61 },
    { 0x4a6ade37, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bade3b, 33 },
    { 0x8d0ade3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ade46, 43 },
    { 0xd5dade47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x248ade4a, 80 },
    { 0, 0 },
    { 0x7c7ade4c, 61 },
    { 0x4a5ade4d, 78 },
    { 0x9fcade4c, 95 },
    { 0x7dfade4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x175ade52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cade55, 77 },
    { 0x958ade56, 33 },
    { 0x171ade57, 77 },
    { 0xb55ade55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbade5b, 90 },
    { 0x8bade5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ade61, 90 },
    { 0, 0 },
    { 0xfdcade63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3ade6b, 85 },
    { 0, 0 },
    { 0x9d1ade6d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aade70, 61 },
    { 0x722ade70, 95 },
    { 0x64ade70, 75 },
    { 0, 0 },
    { 0x98eade74, 91 },
    { 0x2dade75, 61 },
    { 0x82cade74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebade7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fade7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adade90, 85 },
    { 0xee5ade91, 61 },
    { 0x4d2ade92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432ade96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786ade9e, 90 },
    { 0xe7dade9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3adea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603adea6, 90 },
    { 0xf98adea6, 95 },
    { 0xeeeadea7, 100 },
    { 0xaf5adea9, 24 },
    { 0xc2fadeaa, 95 },
    { 0xa4adeab, 100 },
    { 0x9d5adeac, 55 },
    { 0xa09adeab, 90 },
    { 0x2b8adeae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906adeb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33adeb9, 43 },
    { 0, 0 },
    { 0x919adebb, 33 },
    { 0, 0 },
    { 0xaa5adebd, 33 },
    { 0xf41adebe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07adec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563adec7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x515adeca, 98 },
    { 0x5a1adeca, 100 },
    { 0x1ebadecc, 33 },
    { 0, 0 },
    { 0xc75adece, 61 },
    { 0, 0 },
    { 0x726aded0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x96daded3, 91 },
    { 0xb54aded4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cadeda, 23 },
    { 0xf21adedb, 75 },
    { 0x1c5adedc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7adee3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9adee6, 90 },
    { 0, 0 },
    { 0x4f4adee8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7adeec, 23 },
    { 0, 0 },
    { 0xffdadeee, 76 },
    { 0, 0 },
    { 0x474adef0, 55 },
    { 0, 0 },
    { 0xd4eadef2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2adef6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfadefa, 61 },
    { 0x72eadefb, 96 },
    { 0xf5adefc, 99 },
    { 0xc64adefd, 43 },
    { 0x9eadefe, 98 },
    { 0x4a9adefc, 66 },
    { 0, 0 },
    { 0x6b1adf01, 95 },
    { 0xdf6adf02, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0adf08, 90 },
    { 0xd8cadf08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecadf0c, 78 },
    { 0x38fadf0d, 33 },
    { 0x502adf0d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9adf11, 33 },
    { 0x572adf12, 75 },
    { 0x3b8adf12, 81 },
    { 0, 0 },
    { 0x836adf15, 76 },
    { 0, 0 },
    { 0xf39adf17, 22 },
    { 0, 0 },
    { 0xe07adf19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9adf1d, 23 },
    { 0xbe8adf1e, 87 },
    { 0xd6adf1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x576adf22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48adf28, 56 },
    { 0, 0 },
    { 0xa65adf2a, 98 },
    { 0x5dbadf2a, 43 },
    { 0x761adf2c, 61 },
    { 0xc09adf2c, 61 },
    { 0, 0 },
    { 0xfadf2f, 98 },
    { 0xc78adf2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953adf35, 87 },
    { 0xacaadf36, 95 },
    { 0xd07adf36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424adf3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x798adf41, 95 },
    { 0x56fadf41, 98 },
    { 0, 0 },
    { 0x80fadf44, 95 },
    { 0xf0badf45, 52 },
    { 0x90badf44, 61 },
    { 0xa60adf47, 23 },
    { 0x980adf48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9adf4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebadf50, 100 },
    { 0xdd9adf51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3adf54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2adf57, 68 },
    { 0x256adf57, 61 },
    { 0x132adf57, 23 },
    { 0x34badf57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9badf5d, 90 },
    { 0, 0 },
    { 0x96aadf5f, 34 },
    { 0x574adf60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604adf67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efadf6c, 76 },
    { 0xdc1adf6d, 97 },
    { 0x713adf6d, 90 },
    { 0, 0 },
    { 0x125adf70, 98 },
    { 0x5b7adf71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aadf7a, 90 },
    { 0xe50adf7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eadf82, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbadf87, 55 },
    { 0xa72adf88, 90 },
    { 0x80eadf88, 98 },
    { 0x3a7adf8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7adf8d, 75 },
    { 0x336adf8e, 47 },
    { 0x715adf8d, 61 },
    { 0x2faadf90, 12 },
    { 0x685adf91, 61 },
    { 0xf51adf92, 100 },
    { 0x52badf93, 34 },
    { 0x354adf94, 98 },
    { 0xf1cadf94, 90 },
    { 0xc5fadf96, 76 },
    { 0x339adf97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0adf9c, 53 },
    { 0x232adf9c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x733adfa0, 34 },
    { 0, 0 },
    { 0xad6adfa2, 61 },
    { 0xbaadfa3, 95 },
    { 0, 0 },
    { 0xa72adfa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2adfaf, 33 },
    { 0, 0 },
    { 0x440adfb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866adfb6, 61 },
    { 0, 0 },
    { 0x48cadfb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91adfbd, 36 },
    { 0, 0 },
    { 0x1f8adfbf, 61 },
    { 0x898adfc0, 98 },
    { 0, 0 },
    { 0xfdeadfc2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56adfc5, 46 },
    { 0x5a0adfc6, 55 },
    { 0x794adfc7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62adfcb, 84 },
    { 0, 0 },
    { 0xecbadfcd, 95 },
    { 0, 0 },
    { 0x18cadfcf, 66 },
    { 0x4a3adfd0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85adfd7, 46 },
    { 0xc0fadfd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6adfdb, 34 },
    { 0x19badfdc, 22 },
    { 0x6d7adfdc, 78 },
    { 0x657adfdc, 33 },
    { 0xce8adfdc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23adfe8, 90 },
    { 0, 0 },
    { 0x1a7adfea, 52 },
    { 0xaddadfeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0adff6, 23 },
    { 0xb99adff7, 100 },
    { 0x5e9adff8, 46 },
    { 0xe62adff8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19adffd, 78 },
    { 0x987adffd, 55 },
    { 0, 0 },
    { 0xfacae000, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445ae008, 33 },
    { 0xbfae008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x641ae00c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cae00f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0ae012, 81 },
    { 0x814ae013, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceae019, 23 },
    { 0xc21ae01a, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x637ae01d, 90 },
    { 0xa24ae01d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95ae021, 99 },
    { 0xb55ae021, 87 },
    { 0x89fae023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bae028, 77 },
    { 0xa68ae029, 43 },
    { 0x517ae02a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296ae02e, 100 },
    { 0x2b5ae02e, 91 },
    { 0x6bbae02e, 98 },
    { 0x496ae02e, 90 },
    { 0x5dae02e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dae036, 67 },
    { 0, 0 },
    { 0x2bdae038, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x866ae03b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aeae044, 33 },
    { 0x99ae045, 76 },
    { 0, 0 },
    { 0x79cae047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaae04b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x992ae04e, 80 },
    { 0, 0 },
    { 0x875ae050, 95 },
    { 0, 0 },
    { 0x378ae052, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666ae05a, 66 },
    { 0x5eeae05b, 66 },
    { 0, 0 },
    { 0x3f2ae05d, 37 },
    { 0, 0 },
    { 0xb42ae05f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bae062, 93 },
    { 0x52fae062, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0ae066, 90 },
    { 0, 0 },
    { 0x76aae068, 34 },
    { 0, 0 },
    { 0x35ae06a, 52 },
    { 0x50fae06b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86eae06f, 91 },
    { 0, 0 },
    { 0x9c7ae071, 88 },
    { 0x2c3ae072, 33 },
    { 0xb7cae071, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371ae078, 33 },
    { 0, 0 },
    { 0xf10ae07a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ae07f, 98 },
    { 0x23aae07f, 98 },
    { 0xeecae081, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9ae084, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77eae088, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7ae08c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693ae094, 23 },
    { 0, 0 },
    { 0x73dae096, 93 },
    { 0, 0 },
    { 0x203ae098, 33 },
    { 0xadcae099, 61 },
    { 0x6ebae09a, 99 },
    { 0, 0 },
    { 0x2d9ae09c, 90 },
    { 0xf5aae09c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae0a1, 34 },
    { 0x675ae0a2, 93 },
    { 0, 0 },
    { 0x60ae0a4, 34 },
    { 0x86aae0a5, 33 },
    { 0x8cbae0a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4ae0a9, 55 },
    { 0x98bae0aa, 88 },
    { 0x1a9ae0ab, 84 },
    { 0x596ae0ac, 98 },
    { 0xa6bae0ad, 88 },
    { 0x888ae0ac, 95 },
    { 0x5d8ae0ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483ae0b3, 61 },
    { 0, 0 },
    { 0x775ae0b5, 61 },
    { 0x680ae0b6, 87 },
    { 0x7b2ae0b7, 34 },
    { 0, 0 },
    { 0x60bae0b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58ae0bc, 66 },
    { 0x3e0ae0bd, 43 },
    { 0xec4ae0bc, 23 },
    { 0xe5dae0bf, 33 },
    { 0x7eeae0bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587ae0ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6ae0d8, 33 },
    { 0xcbbae0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbae0de, 90 },
    { 0xeecae0df, 95 },
    { 0x471ae0de, 67 },
    { 0x352ae0df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73ae0e4, 90 },
    { 0xf7ae0e4, 76 },
    { 0xa29ae0e6, 36 },
    { 0x278ae0e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ae0ea, 23 },
    { 0x6afae0eb, 61 },
    { 0x89bae0ec, 43 },
    { 0x304ae0ea, 55 },
    { 0x9bae0ee, 33 },
    { 0x8baae0ec, 33 },
    { 0, 0 },
    { 0x881ae0f1, 98 },
    { 0, 0 },
    { 0x685ae0f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54ae0f8, 90 },
    { 0x769ae0f8, 61 },
    { 0x1cfae0fa, 90 },
    { 0xfd8ae0f9, 61 },
    { 0x276ae0fc, 23 },
    { 0xd14ae0fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9ae101, 90 },
    { 0x85cae102, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ae107, 36 },
    { 0, 0 },
    { 0x4f5ae109, 66 },
    { 0xf09ae109, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfae112, 23 },
    { 0x11bae112, 66 },
    { 0x931ae114, 81 },
    { 0x660ae114, 33 },
    { 0xa2eae115, 95 },
    { 0, 0 },
    { 0x386ae118, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bae128, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3ae12b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680ae131, 90 },
    { 0, 0 },
    { 0x484ae133, 34 },
    { 0, 0 },
    { 0x421ae135, 34 },
    { 0xdf8ae136, 43 },
    { 0x193ae137, 78 },
    { 0x5e8ae137, 90 },
    { 0x325ae139, 95 },
    { 0x3a5ae137, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dae140, 75 },
    { 0xf94ae140, 66 },
    { 0x4beae142, 95 },
    { 0xcc5ae141, 100 },
    { 0x6d2ae144, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4ae147, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x961ae14a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3ae14d, 33 },
    { 0x404ae14e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ae156, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bae15b, 61 },
    { 0xf14ae15c, 98 },
    { 0, 0 },
    { 0xc0dae15e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fae161, 81 },
    { 0x73fae162, 98 },
    { 0x9d2ae162, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x683ae166, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ae16b, 76 },
    { 0, 0 },
    { 0x598ae16d, 33 },
    { 0x3b3ae16e, 93 },
    { 0xfbaae16e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cae174, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebae17f, 23 },
    { 0xc97ae180, 76 },
    { 0xdebae181, 67 },
    { 0x4c7ae181, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae185, 48 },
    { 0, 0 },
    { 0x6f9ae187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fae18a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96ae18e, 33 },
    { 0x391ae18e, 47 },
    { 0xd08ae190, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fae193, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eae196, 84 },
    { 0, 0 },
    { 0x1ffae198, 90 },
    { 0x3a4ae199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854ae19d, 56 },
    { 0x937ae19d, 78 },
    { 0, 0 },
    { 0x49dae1a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ae1a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8ae1a8, 90 },
    { 0x6c7ae1a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82ae1ac, 68 },
    { 0x4a6ae1ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbae1b0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90ae1b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecae1b7, 33 },
    { 0xc08ae1b7, 90 },
    { 0xbfdae1b9, 85 },
    { 0x950ae1ba, 61 },
    { 0, 0 },
    { 0xe5bae1bc, 77 },
    { 0x2f4ae1bd, 19 },
    { 0x1acae1be, 90 },
    { 0x67cae1bf, 23 },
    { 0x31fae1c0, 23 },
    { 0xcccae1c1, 90 },
    { 0, 0 },
    { 0x155ae1c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ae1c7, 55 },
    { 0x9dae1c7, 43 },
    { 0xfbdae1c8, 76 },
    { 0, 0 },
    { 0xef7ae1cb, 66 },
    { 0, 0 },
    { 0xb5cae1cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccae1d1, 33 },
    { 0x223ae1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859ae1d6, 95 },
    { 0, 0 },
    { 0x588ae1d8, 33 },
    { 0, 0 },
    { 0xb7ae1da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fae1dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82ae1e2, 85 },
    { 0x644ae1e3, 55 },
    { 0xa08ae1e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ae1ea, 33 },
    { 0, 0 },
    { 0xcaaae1ec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcae1f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ae1f4, 95 },
    { 0x401ae1f5, 95 },
    { 0x9edae1f6, 98 },
    { 0x665ae1f5, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeae1fa, 87 },
    { 0x567ae1fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae1fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bae202, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bae20a, 61 },
    { 0xd9ae20b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ae20e, 33 },
    { 0xd5eae20f, 33 },
    { 0xf59ae210, 90 },
    { 0x249ae20e, 95 },
    { 0, 0 },
    { 0xfdaae213, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ae216, 78 },
    { 0x9cae217, 23 },
    { 0xeceae216, 90 },
    { 0xa87ae219, 43 },
    { 0xe78ae21a, 90 },
    { 0, 0 },
    { 0xc0bae21c, 90 },
    { 0xbedae21c, 61 },
    { 0x7e4ae21e, 95 },
    { 0x1f9ae21f, 98 },
    { 0x81eae21e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ae223, 90 },
    { 0xc5ae224, 85 },
    { 0xe24ae223, 43 },
    { 0, 0 },
    { 0xe5fae227, 98 },
    { 0xd15ae227, 61 },
    { 0xd67ae227, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ae22d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cae230, 17 },
    { 0, 0 },
    { 0xeb4ae232, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aae236, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16ae239, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8ae23c, 67 },
    { 0x43eae23c, 33 },
    { 0, 0 },
    { 0xb4eae23f, 98 },
    { 0, 0 },
    { 0xbecae241, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62ae245, 23 },
    { 0xdfaae245, 61 },
    { 0, 0 },
    { 0xa58ae248, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aae24d, 100 },
    { 0x873ae24d, 93 },
    { 0xb1aae24f, 98 },
    { 0xc51ae250, 95 },
    { 0xfa3ae251, 23 },
    { 0, 0 },
    { 0x707ae253, 17 },
    { 0xc8cae254, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66ae257, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cae25a, 34 },
    { 0xa7ae25b, 100 },
    { 0x8baae25c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998ae262, 95 },
    { 0, 0 },
    { 0xfa3ae264, 43 },
    { 0x93dae265, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695ae26b, 90 },
    { 0, 0 },
    { 0x530ae26d, 90 },
    { 0xb9fae26e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcceae278, 85 },
    { 0x135ae279, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ae282, 76 },
    { 0xb51ae283, 93 },
    { 0xebcae282, 100 },
    { 0x7a1ae285, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ae28c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46aae28f, 61 },
    { 0, 0 },
    { 0x51cae291, 68 },
    { 0xef6ae292, 93 },
    { 0x701ae293, 88 },
    { 0x8c7ae293, 84 },
    { 0, 0 },
    { 0x895ae296, 61 },
    { 0, 0 },
    { 0x94aae298, 81 },
    { 0x796ae298, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9baae29c, 33 },
    { 0x3d3ae29c, 95 },
    { 0xb25ae29d, 95 },
    { 0, 0 },
    { 0x906ae2a0, 34 },
    { 0xdb4ae2a1, 34 },
    { 0x8a9ae2a1, 58 },
    { 0xfcbae2a3, 99 },
    { 0, 0 },
    { 0x2fbae2a5, 88 },
    { 0x9dae2a5, 55 },
    { 0x731ae2a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaae2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987ae2af, 90 },
    { 0xd95ae2b0, 43 },
    { 0x450ae2af, 43 },
    { 0xec6ae2b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1ae2b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512ae2bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68ae2c0, 65 },
    { 0x4dcae2c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcae2d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeae2d3, 90 },
    { 0x3b5ae2d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aae2d9, 67 },
    { 0xf94ae2da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29ae2ec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11ae2f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7ae2f4, 98 },
    { 0x3ecae2f5, 33 },
    { 0xfe6ae2f6, 80 },
    { 0, 0 },
    { 0x137ae2f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20ae2ff, 76 },
    { 0xf69ae300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x248ae303, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae306, 84 },
    { 0xdffae307, 61 },
    { 0x958ae307, 23 },
    { 0x453ae309, 90 },
    { 0, 0 },
    { 0x16ae30b, 52 },
    { 0x532ae30b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8ae311, 34 },
    { 0, 0 },
    { 0x3c7ae313, 61 },
    { 0, 0 },
    { 0xd96ae315, 68 },
    { 0, 0 },
    { 0x6e8ae317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bae31c, 77 },
    { 0xac5ae31d, 65 },
    { 0x904ae31e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fae321, 33 },
    { 0xe05ae321, 90 },
    { 0, 0 },
    { 0xc89ae324, 90 },
    { 0x677ae325, 52 },
    { 0, 0 },
    { 0x8b2ae327, 33 },
    { 0, 0 },
    { 0x902ae329, 34 },
    { 0x4aae32a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ae32d, 33 },
    { 0x555ae32d, 90 },
    { 0, 0 },
    { 0xe40ae330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6ae334, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x678ae337, 85 },
    { 0x962ae338, 23 },
    { 0x1beae338, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcae33f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acae342, 90 },
    { 0x68ae342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bae346, 23 },
    { 0x66cae347, 90 },
    { 0xb12ae348, 33 },
    { 0xc5aae349, 93 },
    { 0xfa4ae34a, 98 },
    { 0xbcfae34b, 33 },
    { 0x57cae348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeae34f, 93 },
    { 0, 0 },
    { 0x3d7ae351, 53 },
    { 0x3a8ae351, 76 },
    { 0, 0 },
    { 0x397ae354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5ae358, 95 },
    { 0x882ae359, 77 },
    { 0xb1ae35a, 98 },
    { 0xeadae35a, 66 },
    { 0xdeaae35a, 34 },
    { 0x7ebae35d, 100 },
    { 0x8b2ae35b, 33 },
    { 0x64aae35f, 91 },
    { 0x9cbae35f, 76 },
    { 0, 0 },
    { 0x1deae362, 46 },
    { 0x19eae363, 84 },
    { 0xe1bae364, 61 },
    { 0x285ae363, 94 },
    { 0x112ae366, 67 },
    { 0x691ae367, 85 },
    { 0x82fae367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eae36d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bae376, 100 },
    { 0, 0 },
    { 0x1f2ae378, 90 },
    { 0xf02ae379, 98 },
    { 0xe25ae37a, 95 },
    { 0, 0 },
    { 0xf9dae37c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2ae37f, 33 },
    { 0xae380, 95 },
    { 0x479ae381, 90 },
    { 0, 0 },
    { 0xa02ae383, 91 },
    { 0x37fae383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683ae389, 75 },
    { 0x22ae38a, 85 },
    { 0x881ae38a, 46 },
    { 0xadaae389, 95 },
    { 0x81eae38d, 33 },
    { 0x7eeae38b, 66 },
    { 0, 0 },
    { 0x370ae390, 66 },
    { 0x540ae391, 90 },
    { 0, 0 },
    { 0x7efae393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6ae399, 77 },
    { 0x889ae39a, 90 },
    { 0xf51ae399, 78 },
    { 0, 0 },
    { 0xa6bae39d, 76 },
    { 0x75dae39d, 23 },
    { 0x69bae39d, 33 },
    { 0xaf9ae3a0, 85 },
    { 0x6dcae3a0, 61 },
    { 0x4e1ae3a2, 46 },
    { 0x220ae3a3, 93 },
    { 0x68ae3a2, 92 },
    { 0x4e4ae3a5, 43 },
    { 0xa7fae3a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7ae3ab, 61 },
    { 0xd99ae3ac, 90 },
    { 0xd7aae3ac, 33 },
    { 0xd81ae3ad, 34 },
    { 0xba5ae3af, 46 },
    { 0, 0 },
    { 0x621ae3b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61ae3ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ae3c0, 43 },
    { 0, 0 },
    { 0xe0eae3c2, 33 },
    { 0x194ae3c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26ae3c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdae3cb, 90 },
    { 0x6c5ae3cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4ae3dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17ae3df, 61 },
    { 0, 0 },
    { 0xd15ae3e1, 61 },
    { 0, 0 },
    { 0x3faae3e3, 93 },
    { 0x9baae3e4, 93 },
    { 0xc9fae3e3, 36 },
    { 0x3e5ae3e6, 55 },
    { 0xeb1ae3e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ae3ea, 76 },
    { 0, 0 },
    { 0xf3dae3ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15ae3f1, 98 },
    { 0xdd5ae3f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822ae3f6, 76 },
    { 0x333ae3f7, 34 },
    { 0xc19ae3f8, 43 },
    { 0xda0ae3f9, 34 },
    { 0xbb9ae3fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79ae3fd, 90 },
    { 0xc36ae3fe, 78 },
    { 0, 0 },
    { 0x6f8ae400, 46 },
    { 0xe7aae401, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ae404, 33 },
    { 0xb52ae405, 33 },
    { 0x93cae406, 90 },
    { 0xd5eae406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896ae40c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554ae414, 17 },
    { 0, 0 },
    { 0xc92ae416, 78 },
    { 0xc61ae417, 33 },
    { 0xd32ae418, 66 },
    { 0xec5ae419, 23 },
    { 0x9c3ae41a, 99 },
    { 0x4c9ae41b, 95 },
    { 0x4daae41b, 90 },
    { 0xa8aae41c, 33 },
    { 0xa1eae417, 90 },
    { 0xe4ae41d, 23 },
    { 0x8d4ae416, 98 },
    { 0x4ccae417, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908ae428, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3ae42d, 23 },
    { 0, 0 },
    { 0x3d7ae42f, 61 },
    { 0, 0 },
    { 0x576ae431, 55 },
    { 0x8d9ae432, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29ae43a, 98 },
    { 0xb4dae43b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2ae442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341ae447, 91 },
    { 0x9e3ae448, 76 },
    { 0xab0ae449, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55ae44f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccae456, 17 },
    { 0x8f2ae457, 33 },
    { 0xbcae458, 33 },
    { 0x3c0ae459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742ae45f, 98 },
    { 0xe9ae460, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370ae464, 33 },
    { 0xaf4ae464, 33 },
    { 0x80aae466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cae46a, 90 },
    { 0xfd5ae46b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6ae472, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fae478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bae482, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fae487, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ae48c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969ae495, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367ae49a, 61 },
    { 0x817ae49b, 98 },
    { 0x394ae49b, 66 },
    { 0x6a4ae49d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae4a2, 95 },
    { 0x65fae4a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deae4a6, 46 },
    { 0x627ae4a6, 98 },
    { 0, 0 },
    { 0xd8eae4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcae4ac, 33 },
    { 0x279ae4ac, 84 },
    { 0, 0 },
    { 0xd85ae4af, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392ae4b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fae4b7, 46 },
    { 0xea3ae4b8, 42 },
    { 0, 0 },
    { 0x122ae4ba, 93 },
    { 0x1daae4bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779ae4c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedae4c8, 33 },
    { 0xad8ae4c8, 46 },
    { 0, 0 },
    { 0x10bae4cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3ae4cf, 34 },
    { 0xf5dae4d0, 98 },
    { 0xaaae4d1, 15 },
    { 0x7daae4d2, 61 },
    { 0, 0 },
    { 0xd73ae4d4, 23 },
    { 0xf93ae4d5, 90 },
    { 0xc1fae4d4, 87 },
    { 0x5ae4d7, 84 },
    { 0xc86ae4d8, 55 },
    { 0x7d2ae4d8, 91 },
    { 0xb9cae4d7, 75 },
    { 0xf1fae4da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eae4de, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5ae4e4, 43 },
    { 0xac4ae4e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cae4e8, 98 },
    { 0x2c4ae4e8, 95 },
    { 0, 0 },
    { 0x58aae4eb, 61 },
    { 0, 0 },
    { 0x74aae4ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceae4f1, 23 },
    { 0xc0dae4f1, 95 },
    { 0xd43ae4f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cae4f6, 99 },
    { 0x383ae4f6, 66 },
    { 0, 0 },
    { 0xc03ae4f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eaae4ff, 90 },
    { 0x3c5ae4ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5ae505, 61 },
    { 0, 0 },
    { 0xd9fae507, 52 },
    { 0, 0 },
    { 0x64cae509, 97 },
    { 0, 0 },
    { 0x4c3ae50b, 77 },
    { 0x6c8ae50c, 90 },
    { 0x77aae50b, 34 },
    { 0, 0 },
    { 0xfd6ae50f, 61 },
    { 0, 0 },
    { 0x8f7ae511, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fae516, 43 },
    { 0xa1eae517, 61 },
    { 0x4ceae516, 98 },
    { 0xcfae519, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ae51c, 34 },
    { 0x205ae51d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ae520, 43 },
    { 0, 0 },
    { 0x1a5ae522, 55 },
    { 0x987ae523, 95 },
    { 0, 0 },
    { 0x5dae525, 95 },
    { 0, 0 },
    { 0xe07ae527, 95 },
    { 0x85eae528, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dae52c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fae530, 52 },
    { 0x8fcae531, 93 },
    { 0xe42ae532, 99 },
    { 0, 0 },
    { 0xc5aae534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbae537, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5aae53a, 43 },
    { 0x366ae53b, 55 },
    { 0x8d7ae53a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x842ae53f, 90 },
    { 0, 0 },
    { 0x9dcae541, 78 },
    { 0x3deae541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fae547, 77 },
    { 0x312ae548, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06ae555, 61 },
    { 0xf7bae556, 98 },
    { 0, 0 },
    { 0x562ae558, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eae560, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacae563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4ae566, 99 },
    { 0xf2dae566, 80 },
    { 0, 0 },
    { 0xb75ae569, 59 },
    { 0xa8dae56a, 90 },
    { 0x1c4ae56b, 90 },
    { 0xc55ae56b, 33 },
    { 0x19aae56d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94ae571, 34 },
    { 0x3d9ae572, 33 },
    { 0xdf3ae573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7ae57f, 40 },
    { 0x411ae580, 52 },
    { 0xc38ae580, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ae58a, 55 },
    { 0x110ae58b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cae592, 90 },
    { 0x36eae592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deae596, 98 },
    { 0x4fbae597, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50ae59b, 61 },
    { 0x859ae59b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8ae59f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ae5a3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26ae5a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47aae5aa, 17 },
    { 0, 0 },
    { 0x7d7ae5ac, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803ae5b2, 23 },
    { 0, 0 },
    { 0x565ae5b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ae5b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cae5bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feae5c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0ae5cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebae5d0, 61 },
    { 0xc29ae5d1, 90 },
    { 0xfadae5d2, 88 },
    { 0xcfbae5d1, 98 },
    { 0x245ae5d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ae5da, 95 },
    { 0xbf4ae5db, 100 },
    { 0xcfeae5dc, 78 },
    { 0x252ae5dd, 76 },
    { 0x6aeae5dd, 61 },
    { 0x85eae5df, 61 },
    { 0, 0 },
    { 0xcd1ae5e1, 90 },
    { 0x822ae5e1, 95 },
    { 0, 0 },
    { 0xe58ae5e4, 46 },
    { 0xaa6ae5e5, 98 },
    { 0x231ae5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592ae5ea, 90 },
    { 0xb32ae5eb, 98 },
    { 0, 0 },
    { 0x3edae5ed, 98 },
    { 0xdf1ae5ed, 90 },
    { 0xa6eae5ed, 78 },
    { 0xd9bae5ef, 95 },
    { 0xbc9ae5f1, 23 },
    { 0xed0ae5f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143ae5ff, 61 },
    { 0xc76ae5ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbae603, 90 },
    { 0x55dae603, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fae608, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89ae60b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefae616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8ae61c, 84 },
    { 0x1fcae61d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ae622, 77 },
    { 0x9e0ae622, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bae626, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106ae62b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cae62e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bae634, 95 },
    { 0x458ae634, 43 },
    { 0x7c9ae636, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cae641, 93 },
    { 0x623ae642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9ae646, 93 },
    { 0, 0 },
    { 0x10cae648, 46 },
    { 0, 0 },
    { 0xc78ae64a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cae64d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ae65c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ae65f, 33 },
    { 0x460ae660, 93 },
    { 0x7cdae661, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35eae66c, 33 },
    { 0, 0 },
    { 0x559ae66e, 61 },
    { 0xae0ae66f, 34 },
    { 0, 0 },
    { 0x6c8ae671, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ae679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aeae67e, 30 },
    { 0, 0 },
    { 0x62fae680, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x715ae683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fae687, 56 },
    { 0x74bae688, 98 },
    { 0, 0 },
    { 0xf4dae68a, 90 },
    { 0, 0 },
    { 0x5b5ae68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23aae691, 23 },
    { 0x7fbae691, 90 },
    { 0x2bdae693, 93 },
    { 0x6e9ae693, 95 },
    { 0x270ae695, 87 },
    { 0x234ae696, 87 },
    { 0x972ae694, 75 },
    { 0x26cae698, 95 },
    { 0, 0 },
    { 0xe3ae69a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fae69f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430ae6a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3ae6a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779ae6ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73ae6b2, 55 },
    { 0x724ae6b2, 23 },
    { 0, 0 },
    { 0x713ae6b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662ae6bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x244ae6c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfae6c5, 61 },
    { 0x2c9ae6c6, 96 },
    { 0x40eae6c7, 34 },
    { 0x80aae6c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ae6ce, 33 },
    { 0xf7ae6ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaae6d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ae6d7, 90 },
    { 0, 0 },
    { 0x7d0ae6d9, 43 },
    { 0xc94ae6da, 76 },
    { 0x45cae6db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0ae6e4, 90 },
    { 0x339ae6e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952ae6ec, 91 },
    { 0xc65ae6ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ae6f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99aae6f7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ae6fa, 100 },
    { 0, 0 },
    { 0x3b8ae6fc, 68 },
    { 0xf3fae6fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eae703, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bae709, 76 },
    { 0, 0 },
    { 0x3ddae70b, 66 },
    { 0, 0 },
    { 0x7f8ae70d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dae711, 93 },
    { 0, 0 },
    { 0x5d2ae713, 99 },
    { 0, 0 },
    { 0x139ae715, 65 },
    { 0xe7eae716, 76 },
    { 0xec2ae717, 33 },
    { 0, 0 },
    { 0x815ae719, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47ae71c, 48 },
    { 0, 0 },
    { 0x70aae71e, 90 },
    { 0, 0 },
    { 0xe38ae720, 78 },
    { 0x861ae720, 36 },
    { 0x291ae722, 61 },
    { 0x2daae721, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2ae727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdae72d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdae731, 90 },
    { 0xb95ae732, 98 },
    { 0x25eae732, 98 },
    { 0, 0 },
    { 0x5c1ae735, 90 },
    { 0, 0 },
    { 0x9c8ae737, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfae73a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5ae73e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fae744, 46 },
    { 0x7edae745, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2ae748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ae758, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720ae75b, 100 },
    { 0, 0 },
    { 0x898ae75d, 93 },
    { 0xcc7ae75d, 34 },
    { 0x86fae75f, 90 },
    { 0xa0eae75e, 89 },
    { 0, 0 },
    { 0x1dbae762, 61 },
    { 0xa84ae762, 39 },
    { 0xcf2ae764, 33 },
    { 0x82dae765, 93 },
    { 0, 0 },
    { 0x932ae767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546ae76b, 65 },
    { 0, 0 },
    { 0xb7aae76d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06ae770, 66 },
    { 0x2e5ae770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dae774, 55 },
    { 0, 0 },
    { 0x2aeae776, 33 },
    { 0xc8cae777, 77 },
    { 0x3c0ae777, 87 },
    { 0x834ae779, 34 },
    { 0, 0 },
    { 0x934ae77b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaae781, 90 },
    { 0x6afae782, 84 },
    { 0x684ae783, 95 },
    { 0, 0 },
    { 0x4c4ae785, 76 },
    { 0x387ae786, 91 },
    { 0, 0 },
    { 0x3bdae788, 91 },
    { 0x316ae789, 91 },
    { 0xca4ae788, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ae792, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8ae796, 23 },
    { 0xcae797, 76 },
    { 0x845ae796, 93 },
    { 0x8a6ae799, 90 },
    { 0x8d3ae79a, 84 },
    { 0xa15ae79a, 90 },
    { 0xb44ae79c, 95 },
    { 0x41bae79d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ae7a1, 98 },
    { 0, 0 },
    { 0x5f4ae7a3, 76 },
    { 0, 0 },
    { 0xdc3ae7a5, 61 },
    { 0xe45ae7a6, 33 },
    { 0x539ae7a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fae7aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317ae7b0, 78 },
    { 0x59ae7b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654ae7b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcae7b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818ae7be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fae7c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ae7c7, 90 },
    { 0xba0ae7c7, 93 },
    { 0x408ae7c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecae7cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaeae7d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faae7d9, 23 },
    { 0x9abae7d9, 33 },
    { 0x371ae7db, 52 },
    { 0xd5fae7dc, 66 },
    { 0, 0 },
    { 0xb21ae7de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efae7e5, 93 },
    { 0x761ae7e6, 55 },
    { 0x89dae7e5, 91 },
    { 0xc0fae7e5, 76 },
    { 0x2dcae7e8, 101 },
    { 0x9c7ae7ea, 68 },
    { 0xc1aae7e5, 61 },
    { 0x556ae7e7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ae7ef, 67 },
    { 0x620ae7f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412ae7f4, 34 },
    { 0xde4ae7f5, 46 },
    { 0x80fae7f5, 90 },
    { 0, 0 },
    { 0xbcbae7f8, 78 },
    { 0, 0 },
    { 0x204ae7fa, 90 },
    { 0xb72ae7fb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ae7ff, 65 },
    { 0, 0 },
    { 0x33aae801, 95 },
    { 0x21aae802, 90 },
    { 0x4faae803, 90 },
    { 0xc6ae803, 43 },
    { 0xf1eae801, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253ae811, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2ae814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0ae817, 78 },
    { 0x55dae818, 78 },
    { 0xa0ae819, 34 },
    { 0, 0 },
    { 0x493ae81b, 98 },
    { 0, 0 },
    { 0x192ae81d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bae821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13ae826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548ae82a, 23 },
    { 0xa97ae82b, 90 },
    { 0x915ae82c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37ae830, 33 },
    { 0, 0 },
    { 0x6f8ae832, 78 },
    { 0, 0 },
    { 0x70aae834, 23 },
    { 0, 0 },
    { 0x104ae836, 40 },
    { 0, 0 },
    { 0xdbaae838, 90 },
    { 0x82cae838, 95 },
    { 0x2e9ae838, 23 },
    { 0xa99ae83b, 33 },
    { 0, 0 },
    { 0xed7ae83d, 33 },
    { 0xe2eae83e, 90 },
    { 0x9eeae83f, 33 },
    { 0x6bae83d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30ae848, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbae84b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92ae851, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cae856, 46 },
    { 0xbc8ae856, 33 },
    { 0, 0 },
    { 0xba5ae859, 98 },
    { 0xe35ae85a, 61 },
    { 0x99cae85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fae866, 33 },
    { 0, 0 },
    { 0x800ae868, 85 },
    { 0xd72ae869, 90 },
    { 0, 0 },
    { 0x9bae86b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67ae872, 33 },
    { 0, 0 },
    { 0x46cae874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98ae877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9ae87a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x515ae87d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eae881, 90 },
    { 0, 0 },
    { 0xb3fae883, 55 },
    { 0, 0 },
    { 0x22bae885, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7ae88a, 34 },
    { 0x50fae88b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae88e, 55 },
    { 0, 0 },
    { 0x616ae890, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbae894, 25 },
    { 0, 0 },
    { 0xcefae896, 23 },
    { 0, 0 },
    { 0x567ae898, 98 },
    { 0x94cae899, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791ae8a1, 43 },
    { 0, 0 },
    { 0x3b7ae8a3, 98 },
    { 0xbc9ae8a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bae8a8, 76 },
    { 0xd81ae8a9, 95 },
    { 0, 0 },
    { 0xc60ae8ab, 23 },
    { 0xe16ae8ab, 33 },
    { 0x5a3ae8ad, 61 },
    { 0x18fae8ac, 34 },
    { 0x277ae8ab, 76 },
    { 0, 0 },
    { 0x70eae8b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ae8b5, 90 },
    { 0xbefae8b6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612ae8be, 93 },
    { 0, 0 },
    { 0xd50ae8c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ae8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cae8cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6ae8ce, 23 },
    { 0x813ae8cf, 98 },
    { 0x930ae8d0, 33 },
    { 0xfb9ae8d1, 34 },
    { 0x2fdae8d2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261ae8dd, 23 },
    { 0x6a9ae8dd, 78 },
    { 0x2e4ae8dd, 33 },
    { 0xbcfae8e0, 84 },
    { 0x931ae8e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38ae8e4, 87 },
    { 0x56fae8e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757ae8e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ae8ec, 87 },
    { 0x732ae8ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5ae8f4, 33 },
    { 0x683ae8f5, 33 },
    { 0x453ae8f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ae8fa, 98 },
    { 0x42fae8fb, 52 },
    { 0xccbae8fb, 23 },
    { 0x38aae8fd, 90 },
    { 0x99bae8fe, 81 },
    { 0, 0 },
    { 0x949ae900, 52 },
    { 0x490ae901, 46 },
    { 0, 0 },
    { 0x607ae903, 33 },
    { 0, 0 },
    { 0x603ae905, 98 },
    { 0xf47ae906, 90 },
    { 0xf5dae907, 90 },
    { 0xf44ae908, 98 },
    { 0x516ae905, 98 },
    { 0xc0eae90a, 55 },
    { 0x96fae90b, 78 },
    { 0xfc6ae90c, 23 },
    { 0, 0 },
    { 0xf37ae90e, 56 },
    { 0xd3cae90f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980ae914, 23 },
    { 0x77fae915, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8ae918, 78 },
    { 0xb26ae918, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffae91f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cae923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91ae929, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742ae92d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ae930, 55 },
    { 0x1bfae931, 33 },
    { 0xab4ae931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7ae939, 90 },
    { 0x174ae93a, 90 },
    { 0x477ae939, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28ae93e, 61 },
    { 0x8aaae93f, 77 },
    { 0xcd4ae93e, 69 },
    { 0x18cae941, 76 },
    { 0x134ae942, 33 },
    { 0x948ae942, 61 },
    { 0x61dae944, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dae947, 90 },
    { 0x88bae948, 90 },
    { 0x277ae948, 23 },
    { 0xd1cae948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1aae953, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbeae958, 95 },
    { 0xd2bae958, 90 },
    { 0xd77ae95a, 61 },
    { 0, 0 },
    { 0x2a3ae95c, 90 },
    { 0, 0 },
    { 0xdf1ae95e, 61 },
    { 0, 0 },
    { 0xf81ae960, 33 },
    { 0, 0 },
    { 0x9feae962, 33 },
    { 0xadaae963, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9ae968, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aae96c, 23 },
    { 0, 0 },
    { 0x541ae96e, 90 },
    { 0x210ae96f, 29 },
    { 0xcfae96f, 23 },
    { 0x5daae971, 61 },
    { 0xd84ae972, 66 },
    { 0x2dcae973, 46 },
    { 0x553ae96f, 23 },
    { 0x9b5ae975, 98 },
    { 0x27aae971, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8ae97a, 17 },
    { 0xdb5ae97a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ae980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06ae985, 76 },
    { 0xf78ae986, 78 },
    { 0, 0 },
    { 0x349ae988, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dae98b, 91 },
    { 0, 0 },
    { 0x6edae98d, 84 },
    { 0xb25ae98d, 95 },
    { 0x524ae98f, 76 },
    { 0x125ae98f, 90 },
    { 0, 0 },
    { 0x8deae992, 23 },
    { 0x81fae993, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651ae998, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7ae99b, 52 },
    { 0x796ae99b, 34 },
    { 0x5f9ae99d, 43 },
    { 0xbb8ae99e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ae9a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6ae9a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2ae9af, 90 },
    { 0x56fae9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ae9b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x589ae9b6, 90 },
    { 0x2dfae9b6, 33 },
    { 0x72eae9b7, 55 },
    { 0xbe5ae9b6, 55 },
    { 0x765ae9ba, 55 },
    { 0x170ae9bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8ae9c1, 90 },
    { 0x235ae9c2, 33 },
    { 0xcd2ae9c2, 95 },
    { 0xd9cae9c4, 76 },
    { 0xce4ae9c5, 61 },
    { 0x6ccae9c4, 98 },
    { 0x81aae9c7, 34 },
    { 0x1ddae9c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cae9cc, 97 },
    { 0x6ceae9cd, 36 },
    { 0, 0 },
    { 0x8f3ae9cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807ae9d9, 56 },
    { 0, 0 },
    { 0x383ae9db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266ae9df, 85 },
    { 0x100ae9e0, 61 },
    { 0x910ae9e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eae9e4, 23 },
    { 0x166ae9e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19ae9eb, 98 },
    { 0xa9dae9eb, 90 },
    { 0x75dae9ed, 84 },
    { 0xe67ae9eb, 23 },
    { 0x31cae9ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99ae9f2, 90 },
    { 0, 0 },
    { 0x896ae9f4, 66 },
    { 0x739ae9f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2ae9f8, 91 },
    { 0x51cae9f9, 93 },
    { 0x777ae9f9, 95 },
    { 0, 0 },
    { 0x7baae9fc, 40 },
    { 0xeffae9fd, 34 },
    { 0, 0 },
    { 0xfe3ae9ff, 33 },
    { 0xe5eaea00, 98 },
    { 0x9cfae9ff, 61 },
    { 0, 0 },
    { 0x136aea03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccaea06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9aea0f, 43 },
    { 0xed3aea10, 63 },
    { 0x2c5aea11, 67 },
    { 0xc10aea10, 61 },
    { 0x807aea13, 55 },
    { 0x126aea10, 90 },
    { 0xa42aea14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976aea1c, 99 },
    { 0, 0 },
    { 0xa7eaea1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82daea22, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeaea2c, 93 },
    { 0x75faea2c, 80 },
    { 0x901aea2e, 98 },
    { 0x205aea2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60aea33, 98 },
    { 0xadbaea34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbaea37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36aea3a, 76 },
    { 0xd5aaea3a, 95 },
    { 0xfc3aea3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209aea40, 43 },
    { 0, 0 },
    { 0x6e1aea42, 90 },
    { 0, 0 },
    { 0xdecaea44, 55 },
    { 0xf0eaea45, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269aea4a, 52 },
    { 0x6b1aea4b, 23 },
    { 0xf22aea4a, 75 },
    { 0xb49aea4d, 78 },
    { 0xa15aea4d, 95 },
    { 0x21eaea4f, 95 },
    { 0xba8aea50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60caea55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921aea5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3aea5f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3aea64, 91 },
    { 0x9a5aea64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9daea71, 98 },
    { 0xfdcaea72, 90 },
    { 0, 0 },
    { 0x45aaea74, 33 },
    { 0x3d6aea74, 33 },
    { 0x8bbaea76, 95 },
    { 0xfbaea74, 61 },
    { 0x5feaea78, 33 },
    { 0x970aea79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x38caea7c, 66 },
    { 0x600aea7d, 81 },
    { 0x2afaea7d, 95 },
    { 0x8c4aea7f, 85 },
    { 0x667aea7f, 33 },
    { 0x782aea81, 83 },
    { 0x64daea81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aea88, 78 },
    { 0, 0 },
    { 0xfa2aea8a, 46 },
    { 0, 0 },
    { 0xa07aea8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2aea90, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73caea96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x944aea99, 87 },
    { 0x485aea99, 76 },
    { 0xea7aea9a, 90 },
    { 0xc2faea9c, 23 },
    { 0, 0 },
    { 0x285aea9e, 33 },
    { 0x3efaea9f, 90 },
    { 0x139aea9e, 33 },
    { 0, 0 },
    { 0x855aeaa2, 61 },
    { 0, 0 },
    { 0x567aeaa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9baeaab, 33 },
    { 0x8a0aeaab, 95 },
    { 0xa37aeaad, 88 },
    { 0x568aeaad, 34 },
    { 0x531aeaae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deaeab4, 33 },
    { 0x16daeab4, 90 },
    { 0, 0 },
    { 0xbdaaeab7, 33 },
    { 0, 0 },
    { 0x6d6aeab9, 76 },
    { 0x7c8aeaba, 90 },
    { 0xff5aeabb, 94 },
    { 0x65daeaba, 52 },
    { 0xd61aeabd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaaeac1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570aeac5, 33 },
    { 0x3ecaeac6, 99 },
    { 0xbc3aeac7, 98 },
    { 0xb3aeac8, 23 },
    { 0xf0daeac5, 98 },
    { 0xbbcaeaca, 33 },
    { 0, 0 },
    { 0x354aeacc, 33 },
    { 0, 0 },
    { 0x634aeace, 33 },
    { 0x212aeace, 99 },
    { 0x869aead0, 55 },
    { 0x75daead1, 90 },
    { 0x7f3aead2, 95 },
    { 0x361aeace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223aead8, 33 },
    { 0, 0 },
    { 0xbb9aeada, 90 },
    { 0xd69aeadb, 33 },
    { 0x7f8aeadc, 91 },
    { 0, 0 },
    { 0x874aeade, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1aaeae7, 76 },
    { 0, 0 },
    { 0xdcbaeae9, 95 },
    { 0x6e0aeaea, 38 },
    { 0xab1aeaeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756aeaf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdaeaf8, 55 },
    { 0, 0 },
    { 0x895aeafa, 90 },
    { 0x14eaeafb, 23 },
    { 0xfa1aeafa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6aeb00, 55 },
    { 0x98faeb01, 52 },
    { 0xe97aeb02, 34 },
    { 0xd7faeb03, 77 },
    { 0xc5baeb03, 22 },
    { 0x1e7aeb05, 90 },
    { 0x545aeb05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3aeb0e, 30 },
    { 0, 0 },
    { 0xe60aeb10, 55 },
    { 0x397aeb11, 98 },
    { 0x6b4aeb12, 61 },
    { 0, 0 },
    { 0xdf5aeb14, 46 },
    { 0x9fbaeb15, 65 },
    { 0xfb3aeb14, 61 },
    { 0x96aaeb17, 100 },
    { 0xe2eaeb18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8aeb1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9aeb1f, 98 },
    { 0, 0 },
    { 0xf26aeb21, 23 },
    { 0x7c4aeb21, 90 },
    { 0x4aeaeb23, 33 },
    { 0x382aeb24, 90 },
    { 0x2a2aeb24, 33 },
    { 0xfe0aeb23, 90 },
    { 0, 0 },
    { 0xba4aeb28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6baeb2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02aeb2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5aeb35, 98 },
    { 0x303aeb36, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8aeb39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7aeb3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6aeb3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x288aeb42, 90 },
    { 0x259aeb42, 85 },
    { 0x88aaeb44, 33 },
    { 0x89aaeb45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7aeb4b, 90 },
    { 0x3eeaeb4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6aeb50, 19 },
    { 0xa67aeb51, 55 },
    { 0x34aeb52, 61 },
    { 0x146aeb53, 90 },
    { 0x5f0aeb53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823aeb59, 81 },
    { 0x123aeb5a, 33 },
    { 0xd58aeb5b, 33 },
    { 0xe88aeb5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1aeb61, 90 },
    { 0x26aeb62, 61 },
    { 0xd4baeb62, 33 },
    { 0x67baeb64, 93 },
    { 0x668aeb64, 95 },
    { 0xf1aeb63, 33 },
    { 0xc46aeb61, 17 },
    { 0xcbaaeb63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770aeb6d, 90 },
    { 0, 0 },
    { 0x80daeb6f, 43 },
    { 0x5e4aeb6f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20faeb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0aeb79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6caeb81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91faeb85, 90 },
    { 0xdf7aeb86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdaeb8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5aeb90, 95 },
    { 0, 0 },
    { 0xd5aeb92, 90 },
    { 0x1b8aeb93, 46 },
    { 0xd4daeb94, 61 },
    { 0, 0 },
    { 0xbd7aeb96, 91 },
    { 0, 0 },
    { 0x1f9aeb98, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238aeb9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1aeb9f, 96 },
    { 0xdeaaeb9f, 95 },
    { 0x683aeba1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x352aeba4, 90 },
    { 0, 0 },
    { 0xe9daeba6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74aebaa, 90 },
    { 0xab0aebab, 98 },
    { 0x94aebac, 95 },
    { 0x2f5aebad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1aebb4, 17 },
    { 0x316aebb5, 98 },
    { 0xf7faebb6, 84 },
    { 0xf3eaebb7, 90 },
    { 0x55faebb8, 67 },
    { 0x247aebb9, 96 },
    { 0x2e7aebb8, 95 },
    { 0x9ebaebbb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaebc0, 90 },
    { 0, 0 },
    { 0xe33aebc2, 90 },
    { 0xe87aebc2, 34 },
    { 0, 0 },
    { 0x1d2aebc5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedaebc9, 93 },
    { 0x3f5aebc9, 75 },
    { 0xecdaebcb, 98 },
    { 0x18faebcc, 90 },
    { 0, 0 },
    { 0xd83aebce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39aebd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffaebd6, 98 },
    { 0xe9aebd7, 61 },
    { 0xe90aebd7, 33 },
    { 0x23aebd8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110aebde, 52 },
    { 0, 0 },
    { 0xb50aebe0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7aebe3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142aebe7, 98 },
    { 0xd10aebe8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcaebed, 61 },
    { 0xb96aebee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95aebf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x401aebf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44aec06, 74 },
    { 0x86aec07, 23 },
    { 0xa04aec08, 23 },
    { 0x509aec09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27daec0c, 95 },
    { 0x7feaec0c, 52 },
    { 0x24eaec0e, 90 },
    { 0x173aec0e, 55 },
    { 0, 0 },
    { 0x23aec11, 23 },
    { 0x6daaec11, 23 },
    { 0, 0 },
    { 0x81daec14, 34 },
    { 0x5eaec15, 90 },
    { 0x3f8aec14, 90 },
    { 0, 0 },
    { 0x817aec18, 95 },
    { 0, 0 },
    { 0x938aec1a, 95 },
    { 0xb5aaec1a, 12 },
    { 0xc2aec1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3aaec26, 99 },
    { 0, 0 },
    { 0xb7aaec28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116aec30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baec33, 65 },
    { 0x28faec34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecaec37, 36 },
    { 0x9c7aec38, 96 },
    { 0xa1aaec39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699aec3f, 99 },
    { 0xf64aec40, 55 },
    { 0x69aaec3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eaec44, 23 },
    { 0x937aec45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391aec4a, 87 },
    { 0, 0 },
    { 0x194aec4c, 33 },
    { 0xee8aec4d, 33 },
    { 0x98faec4d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282aec58, 23 },
    { 0x413aec59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2aec5c, 33 },
    { 0xb8aaec5c, 93 },
    { 0x1c1aec5c, 61 },
    { 0x744aec5f, 17 },
    { 0xc4daec5f, 75 },
    { 0x1faaec5e, 98 },
    { 0x61daec62, 33 },
    { 0x4d0aec60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128aec67, 66 },
    { 0xfadaec68, 46 },
    { 0, 0 },
    { 0x524aec6a, 93 },
    { 0x27baec6b, 34 },
    { 0, 0 },
    { 0x3b5aec6d, 76 },
    { 0x496aec6e, 77 },
    { 0xfa4aec6f, 9 },
    { 0xfa7aec6d, 96 },
    { 0x464aec71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859aec77, 33 },
    { 0xa63aec77, 61 },
    { 0xd17aec78, 61 },
    { 0, 0 },
    { 0x68eaec7b, 81 },
    { 0xfe1aec7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4aec81, 95 },
    { 0x523aec82, 33 },
    { 0xb0aaec82, 91 },
    { 0xf64aec84, 95 },
    { 0xdc9aec83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aaec88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30aec8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccaec8f, 90 },
    { 0x26baec90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aec93, 43 },
    { 0xf44aec93, 23 },
    { 0, 0 },
    { 0x2e6aec96, 61 },
    { 0x259aec96, 98 },
    { 0x226aec96, 61 },
    { 0xc60aec99, 76 },
    { 0xc92aec99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20aec9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5aeca2, 81 },
    { 0x77daeca2, 87 },
    { 0, 0 },
    { 0xe75aeca5, 55 },
    { 0, 0 },
    { 0x8edaeca7, 33 },
    { 0, 0 },
    { 0x8c0aeca9, 95 },
    { 0x7d6aeca9, 90 },
    { 0xe0daecab, 73 },
    { 0xaefaecab, 65 },
    { 0x8d0aecab, 55 },
    { 0x325aecab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36daecb2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81baecb8, 75 },
    { 0, 0 },
    { 0xffaaecba, 17 },
    { 0xc47aecbb, 99 },
    { 0x6f2aecbc, 23 },
    { 0x650aecbd, 69 },
    { 0x173aecbb, 95 },
    { 0x68daecbf, 52 },
    { 0xeeaaecbc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602aecc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943aecce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16aecd1, 85 },
    { 0x498aecd2, 23 },
    { 0x916aecd3, 90 },
    { 0x88caecd1, 91 },
    { 0, 0 },
    { 0x834aecd6, 36 },
    { 0xc91aecd7, 33 },
    { 0xc0daecd7, 61 },
    { 0xda0aecd9, 91 },
    { 0x1daecda, 53 },
    { 0x758aecda, 61 },
    { 0, 0 },
    { 0xbeaecdd, 66 },
    { 0xdbaaecde, 33 },
    { 0x83aecdf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4aece2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x25caece5, 68 },
    { 0x988aece6, 33 },
    { 0x957aece7, 61 },
    { 0xde3aece5, 93 },
    { 0xa3daece5, 34 },
    { 0x616aecea, 66 },
    { 0x1d4aece5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x993aecee, 93 },
    { 0x8d4aecee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825aecf5, 95 },
    { 0, 0 },
    { 0x3b2aecf7, 38 },
    { 0, 0 },
    { 0x351aecf9, 95 },
    { 0xa11aecfa, 90 },
    { 0x7dfaecfb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9daed05, 33 },
    { 0x703aed06, 90 },
    { 0xf4caed06, 17 },
    { 0x43faed07, 90 },
    { 0x81aed06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355aed0d, 33 },
    { 0xb18aed0e, 95 },
    { 0x144aed0f, 68 },
    { 0x672aed10, 69 },
    { 0xad1aed10, 33 },
    { 0, 0 },
    { 0xb0daed13, 98 },
    { 0, 0 },
    { 0xdf0aed15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2aed1c, 55 },
    { 0, 0 },
    { 0x1d5aed1e, 34 },
    { 0x8faed1e, 34 },
    { 0xf9daed20, 23 },
    { 0xd34aed20, 90 },
    { 0, 0 },
    { 0xd9caed23, 43 },
    { 0xa4aed24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x422aed27, 23 },
    { 0xa64aed28, 33 },
    { 0x9c7aed27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbaed2d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9baed30, 77 },
    { 0x51baed31, 90 },
    { 0x51daed30, 61 },
    { 0x41baed33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829aed37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66aed3a, 61 },
    { 0, 0 },
    { 0x615aed3c, 87 },
    { 0x5bcaed3d, 98 },
    { 0xeecaed3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x16aaed41, 90 },
    { 0x311aed42, 61 },
    { 0, 0 },
    { 0x9d3aed44, 98 },
    { 0xf3eaed44, 33 },
    { 0x82eaed45, 87 },
    { 0x35daed45, 90 },
    { 0xc31aed48, 77 },
    { 0x187aed49, 98 },
    { 0x7ddaed4a, 95 },
    { 0x382aed4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98aed4e, 98 },
    { 0, 0 },
    { 0x5f3aed50, 55 },
    { 0, 0 },
    { 0x53caed52, 81 },
    { 0x200aed52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0baed56, 61 },
    { 0x385aed57, 61 },
    { 0xb6baed56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41aed60, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46aed63, 23 },
    { 0, 0 },
    { 0x9d0aed65, 95 },
    { 0, 0 },
    { 0x457aed67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8daed6b, 90 },
    { 0xd8daed6c, 66 },
    { 0xf37aed6d, 78 },
    { 0xcaaaed6c, 33 },
    { 0x6c8aed6d, 43 },
    { 0, 0 },
    { 0x76baed71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1aed75, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4aed7c, 76 },
    { 0xcb1aed7d, 52 },
    { 0, 0 },
    { 0xd32aed7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ceaed84, 90 },
    { 0, 0 },
    { 0x3faaed86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdaed89, 66 },
    { 0xd03aed89, 67 },
    { 0, 0 },
    { 0xcf6aed8c, 90 },
    { 0, 0 },
    { 0xfcfaed8e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccaed91, 23 },
    { 0, 0 },
    { 0x62eaed93, 34 },
    { 0x1bdaed93, 95 },
    { 0x870aed94, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0baed9a, 90 },
    { 0xaf1aed9b, 55 },
    { 0x8c1aed9c, 98 },
    { 0xa10aed9c, 43 },
    { 0x7c5aed9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8aeda9, 55 },
    { 0xaf2aedaa, 68 },
    { 0x26eaedaa, 61 },
    { 0x5e7aedac, 95 },
    { 0x774aedac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x26daedb0, 61 },
    { 0, 0 },
    { 0x233aedb2, 90 },
    { 0, 0 },
    { 0xf6baedb4, 61 },
    { 0x8d6aedb5, 78 },
    { 0x9b3aedb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdaedc0, 43 },
    { 0xec0aedc0, 90 },
    { 0, 0 },
    { 0x700aedc3, 61 },
    { 0, 0 },
    { 0x428aedc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7baedc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x521aedcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2aedda, 66 },
    { 0xd3aaedda, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcaede0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caede8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eaeded, 98 },
    { 0xb04aeded, 100 },
    { 0, 0 },
    { 0x1b9aedf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645aedf4, 32 },
    { 0x2cdaedf4, 23 },
    { 0, 0 },
    { 0xd9daedf7, 95 },
    { 0xd99aedf7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x914aedfb, 68 },
    { 0xe71aedfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64aee00, 95 },
    { 0x884aee01, 90 },
    { 0xbd5aee02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1aee0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00aee0e, 34 },
    { 0x34aee0e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x72baee12, 46 },
    { 0x84aee12, 93 },
    { 0x355aee14, 56 },
    { 0, 0 },
    { 0x114aee16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eaee1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646aee24, 98 },
    { 0x1aee25, 23 },
    { 0x63caee26, 90 },
    { 0, 0 },
    { 0x245aee28, 23 },
    { 0xb06aee29, 43 },
    { 0x3bfaee29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824aee2f, 61 },
    { 0x43caee30, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x18faee33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5aee36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4aee3d, 95 },
    { 0, 0 },
    { 0xba7aee3f, 23 },
    { 0, 0 },
    { 0x971aee41, 76 },
    { 0xd28aee42, 98 },
    { 0x146aee43, 61 },
    { 0x9cfaee43, 77 },
    { 0xd5faee43, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5caee48, 95 },
    { 0, 0 },
    { 0x4eaaee4a, 64 },
    { 0, 0 },
    { 0xcc2aee4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdaee4f, 33 },
    { 0, 0 },
    { 0xeb5aee51, 77 },
    { 0, 0 },
    { 0x957aee53, 61 },
    { 0, 0 },
    { 0x63faee55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3aee59, 98 },
    { 0, 0 },
    { 0xb82aee5b, 78 },
    { 0, 0 },
    { 0xbdaaee5d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0aee64, 76 },
    { 0x3e6aee65, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70aee69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9faee6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aaee71, 90 },
    { 0x4f4aee72, 98 },
    { 0xe62aee72, 99 },
    { 0xfbaaee74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250aee7b, 90 },
    { 0, 0 },
    { 0x5aee7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x810aee80, 33 },
    { 0x324aee80, 61 },
    { 0xbbaee82, 91 },
    { 0x19eaee83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598aee88, 52 },
    { 0, 0 },
    { 0x59caee8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceaee90, 46 },
    { 0x66aee90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34baee9a, 93 },
    { 0x5f0aee9a, 90 },
    { 0xffaaee9c, 33 },
    { 0, 0 },
    { 0x643aee9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34aeea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceaeeaa, 23 },
    { 0x662aeeaa, 55 },
    { 0x71caeeaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53aeeb3, 55 },
    { 0, 0 },
    { 0x111aeeb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baaeeb9, 95 },
    { 0x5faaeeba, 76 },
    { 0, 0 },
    { 0xbe3aeebc, 33 },
    { 0x562aeebd, 51 },
    { 0x718aeebe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7daeec4, 33 },
    { 0x12eaeec5, 89 },
    { 0x49baeec4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceaeed0, 61 },
    { 0, 0 },
    { 0xedcaeed2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aaeed7, 93 },
    { 0, 0 },
    { 0x4fdaeed9, 76 },
    { 0x17aeed9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30faeedf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfaeee4, 23 },
    { 0x2cbaeee4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597aeee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652aeeee, 17 },
    { 0x8abaeeee, 43 },
    { 0x53daeeee, 76 },
    { 0, 0 },
    { 0xa7eaeef2, 95 },
    { 0xcf5aeef3, 90 },
    { 0, 0 },
    { 0xc1aeef5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718aeefb, 23 },
    { 0x2efaeefc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78faef00, 98 },
    { 0x190aef01, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8aef04, 95 },
    { 0, 0 },
    { 0x42faef06, 33 },
    { 0x3fbaef07, 81 },
    { 0x342aef08, 33 },
    { 0, 0 },
    { 0xa9aef0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaef0e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554aef15, 33 },
    { 0x89aef15, 33 },
    { 0, 0 },
    { 0xe76aef18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfaef1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9aef1f, 66 },
    { 0xfcdaef20, 46 },
    { 0x993aef1f, 84 },
    { 0x2cdaef22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0eaef26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50daef2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aaef35, 56 },
    { 0xaa8aef36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x891aef39, 95 },
    { 0, 0 },
    { 0x4ccaef3b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0aef40, 61 },
    { 0x51baef40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeaef48, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9aef4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbaef4e, 40 },
    { 0, 0 },
    { 0xa5aef50, 61 },
    { 0x591aef51, 69 },
    { 0, 0 },
    { 0x75daef53, 98 },
    { 0xdefaef53, 33 },
    { 0x2aef53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62aef58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9aef5b, 98 },
    { 0x94baef5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96aef5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199aef65, 43 },
    { 0, 0 },
    { 0x40aaef67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49daef6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcaef6f, 56 },
    { 0, 0 },
    { 0x91eaef71, 53 },
    { 0xd7aef72, 90 },
    { 0xa46aef73, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5aef7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55aef7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eaef8b, 95 },
    { 0, 0 },
    { 0xf2eaef8d, 78 },
    { 0, 0 },
    { 0x846aef8f, 23 },
    { 0xc77aef90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1aef96, 33 },
    { 0x14aef97, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aef9e, 93 },
    { 0x6e4aef9e, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54caefa5, 84 },
    { 0x16aefa6, 98 },
    { 0, 0 },
    { 0x48aefa8, 100 },
    { 0x88aaefa9, 43 },
    { 0x9cfaefa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29aefad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baefb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6aefb5, 100 },
    { 0xc68aefb6, 33 },
    { 0x77caefb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x860aefba, 90 },
    { 0, 0 },
    { 0x59aaefbc, 33 },
    { 0xe19aefbd, 98 },
    { 0x954aefbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeaefc1, 61 },
    { 0x21caefc1, 17 },
    { 0, 0 },
    { 0x729aefc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7aefca, 33 },
    { 0, 0 },
    { 0x9fcaefcc, 85 },
    { 0, 0 },
    { 0x71caefce, 98 },
    { 0, 0 },
    { 0x159aefd0, 90 },
    { 0x64daefd0, 93 },
    { 0x158aefd1, 90 },
    { 0x5bcaefd2, 83 },
    { 0xca3aefd4, 97 },
    { 0xce7aefd4, 33 },
    { 0xd31aefd6, 33 },
    { 0, 0 },
    { 0xb7daefd8, 52 },
    { 0x1b5aefd9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3aefea, 76 },
    { 0, 0 },
    { 0x862aefec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfaeff4, 43 },
    { 0x53caeff5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x576aeff8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990aeffc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eaefff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6af003, 99 },
    { 0, 0 },
    { 0x1f1af005, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5af009, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258af00e, 23 },
    { 0, 0 },
    { 0x9a4af010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65baf013, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478af017, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68baf01b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4af021, 78 },
    { 0xf13af022, 19 },
    { 0xabbaf022, 61 },
    { 0xd14af021, 52 },
    { 0x298af025, 52 },
    { 0xed4af025, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1af02b, 33 },
    { 0, 0 },
    { 0xf6aaf02d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4af033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfaf036, 66 },
    { 0, 0 },
    { 0x945af038, 33 },
    { 0, 0 },
    { 0xef0af03a, 98 },
    { 0xcf1af03b, 90 },
    { 0, 0 },
    { 0xcb7af03d, 33 },
    { 0, 0 },
    { 0x69caf03f, 90 },
    { 0xdf0af040, 84 },
    { 0xb9caf03f, 90 },
    { 0x295af041, 23 },
    { 0, 0 },
    { 0x36daf044, 43 },
    { 0x284af044, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34af050, 76 },
    { 0x813af050, 90 },
    { 0x582af050, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41af055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989af05e, 90 },
    { 0x7acaf05f, 55 },
    { 0xbfaaf060, 76 },
    { 0xe7aaf05e, 33 },
    { 0xc6af062, 33 },
    { 0, 0 },
    { 0x879af064, 17 },
    { 0x45faf064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x642af068, 98 },
    { 0xe19af069, 91 },
    { 0xaefaf069, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82baf06e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65daf072, 53 },
    { 0x197af073, 33 },
    { 0xc84af074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80baf079, 95 },
    { 0x773af07a, 100 },
    { 0xedaf079, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468af080, 66 },
    { 0x1f0af081, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36af084, 95 },
    { 0xfcaaf085, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abaf088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaaf08c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7af08f, 33 },
    { 0, 0 },
    { 0x90faf091, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af094, 61 },
    { 0x524af094, 90 },
    { 0x225af096, 33 },
    { 0x70af097, 66 },
    { 0, 0 },
    { 0x1bdaf099, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75af09c, 66 },
    { 0x273af09c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9af0a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eaf0a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16faf0ac, 33 },
    { 0, 0 },
    { 0xd77af0ae, 98 },
    { 0x2f4af0ae, 90 },
    { 0x956af0b0, 78 },
    { 0xd70af0b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eaf0b4, 61 },
    { 0xc88af0b5, 100 },
    { 0x6f7af0b6, 76 },
    { 0, 0 },
    { 0xe11af0b8, 61 },
    { 0xa26af0b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eaf0be, 33 },
    { 0xd90af0be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beaf0c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0faf0ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72af0cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aaf0d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x563af0d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43af0d8, 98 },
    { 0x6e7af0d9, 85 },
    { 0x588af0d8, 46 },
    { 0, 0 },
    { 0x512af0dc, 33 },
    { 0x369af0dc, 90 },
    { 0x514af0de, 95 },
    { 0xbaeaf0df, 95 },
    { 0x2b9af0e0, 34 },
    { 0, 0 },
    { 0xc96af0e2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdaf0e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90af0e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94af0ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89af0f2, 55 },
    { 0x3a5af0f3, 90 },
    { 0x6daf0f4, 98 },
    { 0xbedaf0f5, 98 },
    { 0x79baf0f6, 96 },
    { 0xde8af0f7, 33 },
    { 0x4d0af0f6, 98 },
    { 0xfa3af0f9, 76 },
    { 0xe0af0fa, 34 },
    { 0xd33af0fa, 87 },
    { 0xdd4af0f4, 66 },
    { 0x692af0f8, 66 },
    { 0x71faf0fe, 93 },
    { 0x36eaf0ff, 95 },
    { 0x790af100, 90 },
    { 0x433af101, 75 },
    { 0xeeeaf101, 82 },
    { 0xdd7af100, 98 },
    { 0x99baf0fd, 93 },
    { 0, 0 },
    { 0x753af106, 61 },
    { 0, 0 },
    { 0xe51af108, 84 },
    { 0xb31af109, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6af10e, 90 },
    { 0xa60af10e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266af113, 61 },
    { 0x746af114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444af118, 76 },
    { 0x177af119, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0af11c, 34 },
    { 0, 0 },
    { 0x5ecaf11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4af121, 38 },
    { 0x270af121, 24 },
    { 0xddbaf123, 81 },
    { 0, 0 },
    { 0x816af125, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413af12c, 33 },
    { 0, 0 },
    { 0x4caf12e, 78 },
    { 0xb8aaf12e, 99 },
    { 0x3e6af130, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x168af133, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdaf137, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5af13a, 64 },
    { 0x157af13b, 95 },
    { 0xca4af13c, 55 },
    { 0x156af13d, 98 },
    { 0x184af13e, 88 },
    { 0xb74af13f, 55 },
    { 0x4eaaf140, 33 },
    { 0x6ffaf141, 33 },
    { 0x121af142, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aaf148, 75 },
    { 0xa18af149, 61 },
    { 0xa41af148, 95 },
    { 0x43af14b, 52 },
    { 0xfc5af14c, 33 },
    { 0xc70af14d, 55 },
    { 0, 0 },
    { 0x58faf14f, 95 },
    { 0x563af150, 95 },
    { 0, 0 },
    { 0x13baf152, 84 },
    { 0xba1af153, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4af159, 65 },
    { 0x73baf15a, 23 },
    { 0xb57af159, 90 },
    { 0x785af15a, 61 },
    { 0x764af15a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853af165, 66 },
    { 0x575af166, 55 },
    { 0, 0 },
    { 0xfedaf168, 61 },
    { 0xee0af168, 55 },
    { 0, 0 },
    { 0x7ebaf16b, 77 },
    { 0x137af16c, 33 },
    { 0x2b9af16c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaf170, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82af178, 61 },
    { 0x4e3af179, 34 },
    { 0x89aaf17a, 33 },
    { 0xc77af17b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3eaf17e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eaf181, 98 },
    { 0x4d8af182, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42af185, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4af18b, 77 },
    { 0xf06af18c, 90 },
    { 0x2a3af18d, 95 },
    { 0x5d6af18e, 90 },
    { 0, 0 },
    { 0x6eaaf190, 53 },
    { 0xc2af191, 78 },
    { 0x320af191, 100 },
    { 0, 0 },
    { 0xdc8af194, 23 },
    { 0, 0 },
    { 0xed2af196, 33 },
    { 0, 0 },
    { 0x4a1af198, 90 },
    { 0xeb1af199, 76 },
    { 0x60faf19a, 90 },
    { 0, 0 },
    { 0x650af19c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6af1a1, 33 },
    { 0, 0 },
    { 0x4d2af1a3, 51 },
    { 0x82daf1a4, 81 },
    { 0xc2baf1a5, 88 },
    { 0xc76af1a6, 61 },
    { 0, 0 },
    { 0xafaf1a8, 95 },
    { 0xf48af1a8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48baf1ac, 90 },
    { 0x6a6af1ad, 90 },
    { 0, 0 },
    { 0xa23af1af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155af1b4, 55 },
    { 0, 0 },
    { 0x1b1af1b6, 87 },
    { 0x88daf1b7, 46 },
    { 0x3af1b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69af1bc, 101 },
    { 0x3c0af1bc, 90 },
    { 0, 0 },
    { 0xc6caf1bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddaf1c3, 98 },
    { 0x266af1c4, 34 },
    { 0, 0 },
    { 0xb09af1c6, 61 },
    { 0x610af1c6, 100 },
    { 0xd06af1c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aaf1cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x351af1d0, 61 },
    { 0x734af1d0, 33 },
    { 0, 0 },
    { 0xc9caf1d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1af1d7, 90 },
    { 0, 0 },
    { 0x795af1d9, 43 },
    { 0x4e0af1d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0af1dd, 90 },
    { 0x1f8af1dd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7af1e1, 46 },
    { 0x1c7af1e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90af1e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91af1e9, 52 },
    { 0, 0 },
    { 0x48faf1eb, 33 },
    { 0, 0 },
    { 0xd55af1ed, 19 },
    { 0x3d8af1ee, 93 },
    { 0x758af1ef, 91 },
    { 0, 0 },
    { 0x5daaf1f1, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703af1f5, 66 },
    { 0xd2daf1f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5af1fd, 56 },
    { 0xdecaf1fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcaf201, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2af204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdaf209, 34 },
    { 0x545af209, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baf20f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44af212, 78 },
    { 0xafeaf213, 23 },
    { 0xcc1af212, 90 },
    { 0xc11af214, 95 },
    { 0x4f9af215, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749af21a, 67 },
    { 0x8f3af21b, 34 },
    { 0xcf7af21b, 65 },
    { 0xc1faf21d, 34 },
    { 0, 0 },
    { 0x9d6af21f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abaf222, 67 },
    { 0x64faf223, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdaf227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24aaf232, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aaf241, 99 },
    { 0x854af241, 23 },
    { 0, 0 },
    { 0xa85af244, 33 },
    { 0xbd0af244, 99 },
    { 0x658af246, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1af249, 33 },
    { 0xb90af249, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703af252, 69 },
    { 0xf15af252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560af25f, 61 },
    { 0, 0 },
    { 0x3aeaf261, 55 },
    { 0, 0 },
    { 0x16daf263, 99 },
    { 0xfc2af264, 95 },
    { 0x40aaf265, 61 },
    { 0x3a1af266, 76 },
    { 0x1aaaf267, 46 },
    { 0x442af264, 52 },
    { 0xeffaf269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8af26d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7caf271, 93 },
    { 0, 0 },
    { 0x6abaf273, 95 },
    { 0xadfaf274, 76 },
    { 0x22caf275, 61 },
    { 0, 0 },
    { 0x3e1af277, 84 },
    { 0x7e9af277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7af280, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86af283, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70caf287, 84 },
    { 0x554af288, 95 },
    { 0xa65af289, 34 },
    { 0xc45af289, 33 },
    { 0x3a8af289, 98 },
    { 0xc37af28a, 77 },
    { 0, 0 },
    { 0x4fdaf28e, 78 },
    { 0, 0 },
    { 0xde0af290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x81caf293, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7af297, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaaf29e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8af2a2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2af2a6, 23 },
    { 0x98af2a6, 23 },
    { 0xab7af2a6, 67 },
    { 0, 0 },
    { 0xe5caf2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23baf2ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2af2b0, 43 },
    { 0xea8af2b0, 90 },
    { 0x89daf2b1, 33 },
    { 0x2fcaf2b2, 46 },
    { 0x709af2b3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0af2b7, 23 },
    { 0, 0 },
    { 0x44caf2b9, 23 },
    { 0xbdfaf2ba, 33 },
    { 0, 0 },
    { 0xad8af2bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00af2c0, 33 },
    { 0xa96af2c1, 34 },
    { 0, 0 },
    { 0xc03af2c3, 93 },
    { 0x5cdaf2c4, 90 },
    { 0x16eaf2c3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaaf2cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845af2d6, 61 },
    { 0xd57af2d7, 23 },
    { 0x178af2d8, 98 },
    { 0xb3faf2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6af2e1, 99 },
    { 0x29caf2e2, 87 },
    { 0x430af2e2, 93 },
    { 0x161af2e4, 66 },
    { 0x28faf2e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7caf2e8, 55 },
    { 0x1eaaf2e8, 78 },
    { 0, 0 },
    { 0xc48af2eb, 95 },
    { 0xce0af2eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbaf2ef, 61 },
    { 0x3a6af2ef, 95 },
    { 0, 0 },
    { 0x819af2f2, 95 },
    { 0xeefaf2f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x367af2f6, 33 },
    { 0xa5eaf2f6, 98 },
    { 0x556af2f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9af2ff, 76 },
    { 0, 0 },
    { 0x51baf301, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3af30b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0baf30e, 34 },
    { 0xe1eaf30f, 77 },
    { 0x8bdaf310, 55 },
    { 0x12caf311, 95 },
    { 0xaf1af312, 90 },
    { 0xed3af312, 98 },
    { 0xcebaf314, 46 },
    { 0x3faaf315, 67 },
    { 0, 0 },
    { 0x5ebaf317, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6faf31b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77baf327, 98 },
    { 0, 0 },
    { 0x93caf329, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26baf330, 23 },
    { 0xae5af331, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242af336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859af343, 99 },
    { 0x875af343, 23 },
    { 0xbaeaf345, 61 },
    { 0, 0 },
    { 0x202af347, 98 },
    { 0x2ecaf348, 95 },
    { 0x2a5af349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x917af34c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5faf351, 95 },
    { 0x49aaf352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65af35b, 12 },
    { 0xc9faf35b, 87 },
    { 0x643af35d, 95 },
    { 0, 0 },
    { 0x465af35f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6af367, 98 },
    { 0x83aaf368, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x958af36b, 33 },
    { 0x1ccaf36b, 61 },
    { 0, 0 },
    { 0x6dbaf36e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7af371, 34 },
    { 0xfb1af372, 23 },
    { 0x883af373, 55 },
    { 0x131af374, 90 },
    { 0x469af374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78af378, 33 },
    { 0x5e4af379, 98 },
    { 0x248af37a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16af37f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24baf384, 90 },
    { 0xd08af385, 23 },
    { 0xeccaf385, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7af38a, 95 },
    { 0x5c9af38b, 66 },
    { 0xb51af38c, 66 },
    { 0x238af38c, 91 },
    { 0xdebaf38e, 46 },
    { 0xad9af38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0faf392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180af397, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x175af39a, 61 },
    { 0, 0 },
    { 0x9c4af39c, 90 },
    { 0, 0 },
    { 0xbd3af39e, 66 },
    { 0x8f3af39f, 61 },
    { 0x2faf3a0, 95 },
    { 0, 0 },
    { 0x736af3a2, 66 },
    { 0, 0 },
    { 0x47daf3a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831af3aa, 61 },
    { 0, 0 },
    { 0xfa8af3ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758af3b3, 90 },
    { 0x81daf3b4, 98 },
    { 0x60aaf3b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9af3b8, 43 },
    { 0, 0 },
    { 0x7aaf3ba, 33 },
    { 0x73daf3bb, 23 },
    { 0x47caf3bc, 98 },
    { 0x29af3bc, 95 },
    { 0x940af3be, 33 },
    { 0, 0 },
    { 0x654af3c0, 55 },
    { 0x52caf3c1, 85 },
    { 0, 0 },
    { 0xc36af3c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0af3c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x701af3d1, 95 },
    { 0xf3baf3d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9af3d5, 33 },
    { 0, 0 },
    { 0x5dcaf3d7, 43 },
    { 0, 0 },
    { 0xffbaf3d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfaf3e4, 84 },
    { 0xa26af3e5, 40 },
    { 0x3b7af3e6, 93 },
    { 0x19caf3e7, 88 },
    { 0xcdbaf3e5, 95 },
    { 0x88aaf3e6, 33 },
    { 0x849af3ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583af3ee, 90 },
    { 0, 0 },
    { 0xf89af3f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3af3f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7aaf3f7, 33 },
    { 0xed6af3f8, 61 },
    { 0xba4af3f8, 91 },
    { 0, 0 },
    { 0xad5af3fb, 55 },
    { 0x1aaf3fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86faf3ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981af406, 60 },
    { 0xd02af407, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659af40b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787af410, 76 },
    { 0xe40af410, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54baf415, 46 },
    { 0x31baf416, 23 },
    { 0x2baf417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7faf41a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deaf41d, 88 },
    { 0, 0 },
    { 0xb1aaf41f, 23 },
    { 0x8c5af420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddaf423, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1af426, 23 },
    { 0x25aaf427, 84 },
    { 0, 0 },
    { 0xc31af429, 76 },
    { 0, 0 },
    { 0xddcaf42b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42aaf431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceaf436, 36 },
    { 0x689af437, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eaf43e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x442af441, 66 },
    { 0, 0 },
    { 0x177af443, 23 },
    { 0, 0 },
    { 0x710af445, 98 },
    { 0, 0 },
    { 0x3c5af447, 90 },
    { 0x53caf448, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92baf44f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x266af452, 23 },
    { 0xafaf453, 76 },
    { 0, 0 },
    { 0x42daf455, 90 },
    { 0x33af455, 95 },
    { 0, 0 },
    { 0x42daf458, 33 },
    { 0, 0 },
    { 0xd53af45a, 95 },
    { 0, 0 },
    { 0xa16af45c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x76daf45f, 65 },
    { 0x6b4af460, 95 },
    { 0, 0 },
    { 0xea8af462, 66 },
    { 0x1b9af463, 23 },
    { 0x1e2af462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840af468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5af46e, 23 },
    { 0, 0 },
    { 0x61caf470, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5caf476, 95 },
    { 0xf1caf477, 17 },
    { 0x9c2af478, 90 },
    { 0xe1af479, 95 },
    { 0xff2af47a, 43 },
    { 0x28eaf47b, 90 },
    { 0x3e8af478, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602af482, 66 },
    { 0x1a8af482, 84 },
    { 0x525af484, 98 },
    { 0x398af484, 61 },
    { 0, 0 },
    { 0x4abaf487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8af48a, 23 },
    { 0xa1baf48b, 34 },
    { 0xac0af48b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21af48f, 52 },
    { 0xb72af490, 61 },
    { 0x155af491, 23 },
    { 0xc0daf492, 90 },
    { 0x36aaf490, 43 },
    { 0xd79af494, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41caf49a, 95 },
    { 0, 0 },
    { 0x60aaf49c, 34 },
    { 0xf1eaf49d, 40 },
    { 0x5baaf49e, 23 },
    { 0x75af49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9af4a2, 90 },
    { 0xdb2af4a2, 66 },
    { 0xfcdaf4a4, 23 },
    { 0xdebaf4a5, 23 },
    { 0, 0 },
    { 0x810af4a7, 87 },
    { 0, 0 },
    { 0x84daf4a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21af4ac, 23 },
    { 0x5c7af4ad, 93 },
    { 0x4ccaf4ae, 33 },
    { 0x9bcaf4af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0af4b5, 90 },
    { 0, 0 },
    { 0x5d4af4b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35caf4ba, 85 },
    { 0, 0 },
    { 0x273af4bc, 33 },
    { 0, 0 },
    { 0xe9baf4be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x351af4c1, 95 },
    { 0, 0 },
    { 0x41caf4c3, 64 },
    { 0xaceaf4c4, 95 },
    { 0x55aaf4c5, 90 },
    { 0x8dfaf4c5, 98 },
    { 0x2e6af4c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66af4cd, 81 },
    { 0x480af4ce, 98 },
    { 0, 0 },
    { 0x1bdaf4d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aaf4d3, 91 },
    { 0, 0 },
    { 0x10caf4d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64af4d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7af4dd, 76 },
    { 0x23af4de, 90 },
    { 0x7fcaf4de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6af4e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x927af4e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aaf4ee, 78 },
    { 0xf28af4ef, 90 },
    { 0, 0 },
    { 0x723af4f1, 76 },
    { 0x4b7af4f2, 95 },
    { 0, 0 },
    { 0x692af4f4, 2 },
    { 0, 0 },
    { 0x850af4f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcaf4fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50faf4fe, 38 },
    { 0x94af4ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770af504, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaf513, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x318af516, 100 },
    { 0, 0 },
    { 0xa88af518, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x811af51b, 33 },
    { 0, 0 },
    { 0x58aaf51d, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3baf523, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feaf526, 76 },
    { 0x53aaf526, 33 },
    { 0xbc8af528, 23 },
    { 0xbf6af529, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551af52d, 34 },
    { 0, 0 },
    { 0xe76af52f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aaf532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86af538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03af53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3daf542, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feaf545, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acaf54a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5af54d, 61 },
    { 0, 0 },
    { 0xc6faf54f, 23 },
    { 0, 0 },
    { 0x9dbaf551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66af555, 84 },
    { 0xb5faf556, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10caf55a, 90 },
    { 0, 0 },
    { 0xf8faf55c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504af562, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529af567, 90 },
    { 0xa71af568, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972af56c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827af570, 61 },
    { 0x23aaf571, 100 },
    { 0xc6eaf571, 43 },
    { 0x3caf573, 23 },
    { 0xd0af573, 98 },
    { 0, 0 },
    { 0x708af576, 23 },
    { 0xe83af577, 46 },
    { 0xb98af578, 90 },
    { 0, 0 },
    { 0x50baf57a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x70caf57d, 95 },
    { 0x332af57e, 85 },
    { 0x125af57f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9af585, 52 },
    { 0xbc8af586, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eaf58d, 84 },
    { 0, 0 },
    { 0x744af58f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174af594, 90 },
    { 0x107af595, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9caf598, 23 },
    { 0, 0 },
    { 0x6e9af59a, 46 },
    { 0, 0 },
    { 0xcb9af59c, 96 },
    { 0xa30af59c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552af5a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x362af5a5, 85 },
    { 0, 0 },
    { 0xbd1af5a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1af5b0, 23 },
    { 0, 0 },
    { 0xd5caf5b2, 98 },
    { 0x78baf5b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0af5b6, 61 },
    { 0x845af5b6, 55 },
    { 0xbc2af5b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefaf5c2, 90 },
    { 0x3b4af5c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807af5c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1af5cb, 90 },
    { 0xc23af5cc, 95 },
    { 0x1edaf5cd, 55 },
    { 0, 0 },
    { 0x86caf5cf, 23 },
    { 0, 0 },
    { 0x3e4af5d1, 84 },
    { 0xb9daf5d1, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8af5d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36af5d9, 90 },
    { 0, 0 },
    { 0x804af5db, 34 },
    { 0x605af5dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4af5e1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594af5e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9af5ea, 76 },
    { 0, 0 },
    { 0x21caf5ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971af5f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358af5f8, 93 },
    { 0, 0 },
    { 0x9bcaf5fa, 95 },
    { 0x7d9af5fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47caf5ff, 98 },
    { 0x4f6af5ff, 43 },
    { 0xabdaf601, 33 },
    { 0x928af5ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcaf608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01af60d, 90 },
    { 0x39faf60e, 93 },
    { 0, 0 },
    { 0xbc4af610, 90 },
    { 0x532af610, 90 },
    { 0xaadaf610, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9af616, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474af61c, 33 },
    { 0x65faf61c, 98 },
    { 0xac4af61e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139af623, 61 },
    { 0x49aaf624, 95 },
    { 0x885af625, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93af628, 91 },
    { 0x63caf628, 34 },
    { 0x508af62a, 44 },
    { 0xaf2af628, 98 },
    { 0x332af62c, 90 },
    { 0x225af62d, 70 },
    { 0x3dbaf62e, 65 },
    { 0x401af62f, 33 },
    { 0xe84af62a, 61 },
    { 0x35caf630, 46 },
    { 0x5f2af628, 55 },
    { 0x1fbaf633, 85 },
    { 0x855af634, 95 },
    { 0xc7daf633, 61 },
    { 0x1a3af62b, 33 },
    { 0xf12af632, 43 },
    { 0, 0 },
    { 0xbdcaf639, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352af641, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcaf646, 33 },
    { 0x1dfaf647, 46 },
    { 0x937af648, 90 },
    { 0x347af647, 34 },
    { 0xebaf648, 90 },
    { 0xd7faf64a, 98 },
    { 0x5ceaf64b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaaf651, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5af654, 55 },
    { 0x161af655, 87 },
    { 0x4c3af655, 98 },
    { 0x150af657, 90 },
    { 0xf8daf658, 34 },
    { 0xc74af657, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcaf663, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eaf667, 65 },
    { 0x971af668, 75 },
    { 0, 0 },
    { 0x2acaf66a, 53 },
    { 0x4e2af66b, 87 },
    { 0x1efaf66c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0af66f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15af672, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eaf675, 33 },
    { 0, 0 },
    { 0x590af677, 33 },
    { 0x35daf678, 43 },
    { 0x90daf678, 75 },
    { 0, 0 },
    { 0x2b0af67b, 90 },
    { 0x17eaf67b, 78 },
    { 0, 0 },
    { 0x438af67e, 76 },
    { 0xd2baf67e, 40 },
    { 0, 0 },
    { 0x1eaaf681, 17 },
    { 0, 0 },
    { 0x856af683, 95 },
    { 0, 0 },
    { 0xa1af685, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86af692, 33 },
    { 0x786af692, 98 },
    { 0xe37af694, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75aaf69b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48af6a2, 33 },
    { 0, 0 },
    { 0x78aaf6a4, 23 },
    { 0xf86af6a5, 95 },
    { 0xfb7af6a6, 38 },
    { 0, 0 },
    { 0x513af6a8, 61 },
    { 0x543af6a9, 90 },
    { 0x626af6a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34af6af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfaf6b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aaf6b9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53af6c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381af6c4, 76 },
    { 0xab8af6c5, 55 },
    { 0x22daf6c6, 76 },
    { 0xe67af6c6, 77 },
    { 0x1bcaf6c8, 96 },
    { 0, 0 },
    { 0xd97af6ca, 87 },
    { 0xb00af6cb, 43 },
    { 0, 0 },
    { 0xd92af6cd, 34 },
    { 0x691af6ce, 24 },
    { 0x5d7af6ce, 66 },
    { 0, 0 },
    { 0x18caf6d1, 97 },
    { 0xeb6af6d2, 99 },
    { 0, 0 },
    { 0xf69af6d4, 90 },
    { 0x258af6d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52daf6d8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x326af6db, 85 },
    { 0x3daf6db, 23 },
    { 0x449af6dc, 77 },
    { 0x5e3af6de, 100 },
    { 0xcccaf6dc, 23 },
    { 0xf18af6dd, 34 },
    { 0xcd2af6e1, 90 },
    { 0x24eaf6e1, 77 },
    { 0x9b9af6e0, 33 },
    { 0, 0 },
    { 0x855af6e5, 55 },
    { 0x6b1af6e6, 33 },
    { 0xa36af6e7, 75 },
    { 0, 0 },
    { 0xf30af6e9, 90 },
    { 0, 0 },
    { 0x517af6eb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0af6f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0faf6f5, 90 },
    { 0xac0af6f6, 23 },
    { 0, 0 },
    { 0xc51af6f8, 67 },
    { 0x3d6af6f9, 46 },
    { 0, 0 },
    { 0xd34af6fb, 95 },
    { 0xa24af6fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17af6ff, 80 },
    { 0x834af6ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00af705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10af708, 98 },
    { 0, 0 },
    { 0x7a8af70a, 75 },
    { 0xdb1af70a, 99 },
    { 0xf12af70c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1af712, 33 },
    { 0, 0 },
    { 0x7e6af714, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7daf719, 23 },
    { 0xfd1af71a, 98 },
    { 0x2bbaf71b, 61 },
    { 0x849af71b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4af724, 99 },
    { 0x5faaf724, 33 },
    { 0xc1baf726, 100 },
    { 0x21daf727, 52 },
    { 0x2a3af724, 76 },
    { 0xb15af724, 33 },
    { 0x2eeaf725, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15faf733, 95 },
    { 0, 0 },
    { 0x864af735, 61 },
    { 0xc8faf735, 61 },
    { 0x3e1af737, 33 },
    { 0x1a5af736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60af73d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4af741, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x70baf744, 98 },
    { 0xde3af745, 77 },
    { 0xaffaf745, 61 },
    { 0x4cfaf745, 77 },
    { 0x10eaf748, 23 },
    { 0xf40af747, 98 },
    { 0xf01af746, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeaf74e, 88 },
    { 0x400af74f, 38 },
    { 0xe4eaf74f, 98 },
    { 0, 0 },
    { 0x9cfaf752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92caf759, 88 },
    { 0, 0 },
    { 0x91af75b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4faf760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4af764, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73af768, 98 },
    { 0, 0 },
    { 0xd40af76a, 98 },
    { 0x32baf76a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29af772, 66 },
    { 0x542af773, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32af779, 34 },
    { 0x61caf77a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913af77e, 52 },
    { 0x868af77f, 90 },
    { 0, 0 },
    { 0x2bfaf781, 61 },
    { 0x959af781, 33 },
    { 0xe06af782, 87 },
    { 0x4a9af783, 98 },
    { 0, 0 },
    { 0xeb5af786, 56 },
    { 0x6a1af787, 87 },
    { 0x329af787, 95 },
    { 0xe97af788, 76 },
    { 0xcbdaf789, 43 },
    { 0x417af788, 76 },
    { 0x462af78a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09af78f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6baf794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceaf798, 52 },
    { 0xd58af798, 93 },
    { 0x8f6af79a, 38 },
    { 0x80faf79a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1af7a3, 61 },
    { 0x2afaf7a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3af7a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af7b1, 61 },
    { 0, 0 },
    { 0xe75af7b3, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feaf7b6, 90 },
    { 0x4a5af7b6, 61 },
    { 0x7b1af7b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869af7be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4af7c2, 90 },
    { 0, 0 },
    { 0x7b0af7c4, 33 },
    { 0x7dbaf7c5, 23 },
    { 0x1adaf7c6, 61 },
    { 0x4f1af7c7, 90 },
    { 0xb16af7c7, 33 },
    { 0xf2baf7c9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276af7cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133af7d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79af7d7, 90 },
    { 0x84caf7d8, 95 },
    { 0x455af7d8, 87 },
    { 0x32af7da, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9aaf7de, 76 },
    { 0x9c0af7de, 55 },
    { 0, 0 },
    { 0xd12af7e1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520af7e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36af7ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x224af7f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9af7f3, 46 },
    { 0x934af7f4, 98 },
    { 0xe53af7f5, 52 },
    { 0, 0 },
    { 0xb53af7f7, 34 },
    { 0xdefaf7f8, 66 },
    { 0, 0 },
    { 0x198af7fa, 98 },
    { 0xc34af7fb, 93 },
    { 0, 0 },
    { 0xe04af7fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704af802, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aaf809, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6af80c, 100 },
    { 0xf35af80c, 90 },
    { 0xa61af80e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5af811, 33 },
    { 0x800af812, 61 },
    { 0x7daaf811, 100 },
    { 0x89caf814, 55 },
    { 0xf1baf815, 61 },
    { 0, 0 },
    { 0x4e2af817, 33 },
    { 0x4b5af817, 33 },
    { 0x53af819, 76 },
    { 0x8c1af81a, 61 },
    { 0xaadaf81b, 17 },
    { 0xfc3af81c, 76 },
    { 0x6afaf81c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x53baf820, 55 },
    { 0xfd1af820, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfaf824, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aaf82a, 95 },
    { 0xe65af82a, 100 },
    { 0xe54af82c, 55 },
    { 0xbffaf82d, 98 },
    { 0, 0 },
    { 0xe4daf82f, 100 },
    { 0, 0 },
    { 0x80af831, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af83b, 33 },
    { 0x8a9af83c, 34 },
    { 0xf3baf83b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86daf842, 76 },
    { 0, 0 },
    { 0x1a2af844, 98 },
    { 0x58eaf845, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaf84a, 98 },
    { 0x107af84b, 61 },
    { 0x40aaf84c, 98 },
    { 0xd5baf84b, 34 },
    { 0x21eaf84b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22af851, 76 },
    { 0xd22af851, 33 },
    { 0x92eaf853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2faf856, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750af85a, 23 },
    { 0xb4caf85b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7af85f, 91 },
    { 0, 0 },
    { 0x730af861, 84 },
    { 0x4f6af862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35daf866, 90 },
    { 0x5bcaf867, 33 },
    { 0x7a2af868, 67 },
    { 0, 0 },
    { 0xa92af86a, 87 },
    { 0, 0 },
    { 0xc81af86c, 98 },
    { 0, 0 },
    { 0x4c0af86e, 98 },
    { 0x85baf86e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadaf872, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdaf87a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36caf87e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefaf883, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6af886, 61 },
    { 0xe33af886, 90 },
    { 0xe2baf888, 33 },
    { 0x460af887, 76 },
    { 0x60af886, 90 },
    { 0, 0 },
    { 0x285af88c, 33 },
    { 0, 0 },
    { 0xaffaf88e, 93 },
    { 0, 0 },
    { 0xcf0af890, 34 },
    { 0x761af891, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8af895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8af89b, 33 },
    { 0x8b2af89c, 91 },
    { 0xeafaf89c, 93 },
    { 0, 0 },
    { 0x2f3af89f, 90 },
    { 0x15eaf8a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108af8a7, 26 },
    { 0x1a9af8a8, 90 },
    { 0x884af8a9, 43 },
    { 0xb8af8aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faaf8ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9faf8b4, 55 },
    { 0xbb4af8b4, 52 },
    { 0x2c3af8b4, 33 },
    { 0, 0 },
    { 0x4b1af8b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5af8bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16af8c1, 61 },
    { 0, 0 },
    { 0x7ddaf8c3, 98 },
    { 0x21daf8c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eaf8c7, 43 },
    { 0xcc8af8c7, 77 },
    { 0x512af8c9, 95 },
    { 0x4c7af8ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13af8cd, 34 },
    { 0xdebaf8ce, 99 },
    { 0x72aaf8cf, 43 },
    { 0xf5daf8d0, 95 },
    { 0xbb1af8d0, 61 },
    { 0x869af8d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29daf8d8, 33 },
    { 0xb43af8d9, 33 },
    { 0x438af8d9, 90 },
    { 0xd14af8db, 67 },
    { 0x7b1af8d9, 90 },
    { 0xcdcaf8dc, 61 },
    { 0x7eaaf8dc, 81 },
    { 0x7d1af8df, 93 },
    { 0, 0 },
    { 0xb6baf8e1, 43 },
    { 0, 0 },
    { 0x656af8e3, 61 },
    { 0xb72af8e4, 23 },
    { 0x7fdaf8e5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eaf8ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1caf8f3, 23 },
    { 0x9fdaf8f4, 95 },
    { 0x731af8f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbaaf8fa, 46 },
    { 0, 0 },
    { 0xcc2af8fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45af8ff, 61 },
    { 0xe88af900, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07af904, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6af907, 46 },
    { 0, 0 },
    { 0x17baf909, 55 },
    { 0x190af90a, 46 },
    { 0, 0 },
    { 0x276af90c, 23 },
    { 0xac8af90c, 55 },
    { 0xe46af90e, 90 },
    { 0x6dbaf90f, 91 },
    { 0xb48af910, 91 },
    { 0xc51af911, 52 },
    { 0xea7af910, 64 },
    { 0xabbaf913, 23 },
    { 0xd42af913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4af917, 90 },
    { 0, 0 },
    { 0xa32af919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396af920, 90 },
    { 0, 0 },
    { 0x1e3af922, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x90caf925, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0af929, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0af92d, 90 },
    { 0x22caf92e, 95 },
    { 0x98baf92e, 33 },
    { 0x23faf92f, 99 },
    { 0, 0 },
    { 0xf15af932, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eaf93a, 69 },
    { 0xbb0af93a, 23 },
    { 0xb31af93b, 76 },
    { 0, 0 },
    { 0x465af93e, 78 },
    { 0x6d5af93f, 84 },
    { 0x711af940, 90 },
    { 0x1bcaf940, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963af94e, 23 },
    { 0x8a4af94f, 61 },
    { 0x81baf950, 46 },
    { 0, 0 },
    { 0xcfaaf952, 98 },
    { 0x835af952, 99 },
    { 0x1a9af954, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580af959, 23 },
    { 0xf78af959, 90 },
    { 0x6c8af959, 33 },
    { 0x715af95c, 17 },
    { 0xf14af95a, 90 },
    { 0x734af95e, 46 },
    { 0, 0 },
    { 0x95faf960, 23 },
    { 0x700af960, 95 },
    { 0x9c3af960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbaf968, 52 },
    { 0x223af969, 87 },
    { 0xb73af968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1af96e, 90 },
    { 0xffeaf96e, 43 },
    { 0xed6af970, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20daf974, 98 },
    { 0xb48af975, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45af97a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7baf981, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287af986, 52 },
    { 0, 0 },
    { 0x631af988, 67 },
    { 0xbe8af988, 76 },
    { 0x284af98a, 76 },
    { 0, 0 },
    { 0xb33af98c, 98 },
    { 0, 0 },
    { 0x1bdaf98e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadaf993, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12baf996, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5baf9a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x40baf9a9, 61 },
    { 0x778af9a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77af9af, 90 },
    { 0xfacaf9af, 95 },
    { 0x56caf9b1, 99 },
    { 0x7fdaf9b1, 84 },
    { 0, 0 },
    { 0x166af9b4, 76 },
    { 0xa3baf9b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0daf9c5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742af9c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99af9ce, 55 },
    { 0xa54af9cf, 76 },
    { 0x606af9ce, 90 },
    { 0x8feaf9cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38af9d6, 34 },
    { 0x7c7af9d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45af9da, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x133af9dd, 75 },
    { 0, 0 },
    { 0x960af9df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x921af9e2, 77 },
    { 0xb09af9e2, 78 },
    { 0x3d2af9e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65caf9ea, 29 },
    { 0x5e7af9eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666af9f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85af9f5, 90 },
    { 0xe9af9f6, 52 },
    { 0x5b9af9f7, 23 },
    { 0xcc8af9f8, 33 },
    { 0xe10af9f6, 90 },
    { 0x40aaf9fa, 23 },
    { 0xd9aaf9f5, 85 },
    { 0, 0 },
    { 0x276af9fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdafa03, 98 },
    { 0, 0 },
    { 0xd42afa05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816afa10, 90 },
    { 0, 0 },
    { 0xdafa12, 90 },
    { 0, 0 },
    { 0x4a2afa14, 33 },
    { 0xb85afa15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5afa1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x173afa1e, 90 },
    { 0, 0 },
    { 0x427afa20, 33 },
    { 0x4d5afa21, 66 },
    { 0x78afa20, 98 },
    { 0xfb5afa23, 61 },
    { 0, 0 },
    { 0xbafa25, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dafa29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06afa2e, 55 },
    { 0x155afa2f, 23 },
    { 0, 0 },
    { 0xe3eafa31, 34 },
    { 0x96cafa32, 76 },
    { 0, 0 },
    { 0x6dbafa34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9afa3b, 61 },
    { 0, 0 },
    { 0xe4cafa3d, 34 },
    { 0x63bafa3e, 93 },
    { 0xa05afa3f, 87 },
    { 0x46cafa3f, 33 },
    { 0, 0 },
    { 0x950afa42, 98 },
    { 0, 0 },
    { 0xfaeafa44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x238afa47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffafa4a, 93 },
    { 0xa37afa4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafa52, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655afa59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfafa5f, 61 },
    { 0x6a9afa60, 55 },
    { 0, 0 },
    { 0x9d3afa62, 90 },
    { 0x4b2afa63, 90 },
    { 0, 0 },
    { 0x6fcafa65, 78 },
    { 0x538afa65, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x136afa69, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548afa6f, 87 },
    { 0xb3afa70, 76 },
    { 0x496afa6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190afa77, 55 },
    { 0xffcafa78, 90 },
    { 0xca5afa78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69afa7c, 91 },
    { 0, 0 },
    { 0xc9bafa7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00afa82, 52 },
    { 0xf88afa82, 90 },
    { 0, 0 },
    { 0x108afa85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14afa8b, 90 },
    { 0x46dafa8c, 66 },
    { 0x6fcafa8d, 93 },
    { 0xd6eafa8d, 76 },
    { 0x7afa8f, 43 },
    { 0xd32afa8d, 33 },
    { 0xd9fafa8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138afa95, 67 },
    { 0xc3bafa96, 98 },
    { 0x8a0afa97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709afa9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2afaa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37afaa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dafaa9, 78 },
    { 0x9e1afaa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdafab1, 52 },
    { 0, 0 },
    { 0xcb2afab3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaafab8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6afabb, 61 },
    { 0x9e8afabb, 61 },
    { 0x210afabc, 55 },
    { 0x749afabb, 61 },
    { 0xe38afabf, 90 },
    { 0x8c4afabf, 98 },
    { 0x652afac1, 90 },
    { 0xc11afac1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceafac5, 90 },
    { 0x1e0afac6, 33 },
    { 0xd62afac7, 55 },
    { 0, 0 },
    { 0xa2eafac9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44aface, 95 },
    { 0x18aaface, 33 },
    { 0, 0 },
    { 0x900afad1, 61 },
    { 0xeccafad1, 87 },
    { 0xcefafad3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bafad7, 34 },
    { 0, 0 },
    { 0x4bbafad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aafadd, 33 },
    { 0x31dafadd, 33 },
    { 0xddfafadf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5afae2, 46 },
    { 0x540afae3, 23 },
    { 0x8deafae4, 61 },
    { 0x75dafae4, 98 },
    { 0, 0 },
    { 0xaa2afae7, 93 },
    { 0x3d3afae7, 33 },
    { 0x80bafae8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443afaee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96afaf3, 23 },
    { 0, 0 },
    { 0xf34afaf5, 33 },
    { 0, 0 },
    { 0xb7cafaf7, 76 },
    { 0x86afaf8, 23 },
    { 0x4cfafaf9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1afaff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345afb03, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bafb06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3afb09, 61 },
    { 0, 0 },
    { 0xdfafb0b, 46 },
    { 0xcd0afb0b, 34 },
    { 0, 0 },
    { 0xfc9afb0e, 61 },
    { 0xd32afb0f, 90 },
    { 0x3fafb0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528afb14, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4afb17, 100 },
    { 0xddeafb18, 98 },
    { 0x1e8afb19, 23 },
    { 0x7deafb1a, 34 },
    { 0x65afb1b, 42 },
    { 0x36bafb1c, 33 },
    { 0xf00afb1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dafb20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70afb25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cafb2c, 76 },
    { 0x3ccafb2d, 76 },
    { 0, 0 },
    { 0x21cafb2f, 87 },
    { 0xd5aafb30, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202afb35, 61 },
    { 0, 0 },
    { 0x16dafb37, 33 },
    { 0x5a4afb38, 17 },
    { 0xf48afb39, 34 },
    { 0x6f8afb38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46afb4a, 85 },
    { 0, 0 },
    { 0x14eafb4c, 97 },
    { 0, 0 },
    { 0x147afb4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4afb51, 91 },
    { 0xd2dafb52, 33 },
    { 0x9f2afb52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6afb56, 78 },
    { 0xe02afb57, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bafb60, 46 },
    { 0, 0 },
    { 0x2d0afb62, 85 },
    { 0xbf4afb62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43afb6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6aafb6f, 78 },
    { 0xd1cafb6f, 98 },
    { 0, 0 },
    { 0x5bfafb72, 100 },
    { 0xdc4afb73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19afb78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528afb80, 17 },
    { 0xaceafb81, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750afb86, 61 },
    { 0, 0 },
    { 0xd7fafb88, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198afb8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7afb92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdafb95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995afb9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9afb9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09afba1, 84 },
    { 0x1cbafba2, 100 },
    { 0xf31afba2, 84 },
    { 0x551afba4, 76 },
    { 0x720afba4, 43 },
    { 0xa35afba6, 76 },
    { 0x8e5afba6, 95 },
    { 0x960afba8, 33 },
    { 0xbb1afba9, 98 },
    { 0x999afbaa, 65 },
    { 0x574afbab, 61 },
    { 0x4f9afbac, 90 },
    { 0x836afbac, 34 },
    { 0x4ceafbac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aafbb2, 61 },
    { 0, 0 },
    { 0x201afbb4, 95 },
    { 0x2f2afbb5, 75 },
    { 0xab9afbb6, 95 },
    { 0, 0 },
    { 0x4b8afbb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6afbbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aafbbe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x440afbc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332afbca, 95 },
    { 0x82afbcb, 98 },
    { 0xbc6afbcc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4afbd2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dafbd6, 75 },
    { 0xff7afbd6, 98 },
    { 0, 0 },
    { 0x30eafbd9, 83 },
    { 0x1a4afbda, 38 },
    { 0xc0aafbdb, 98 },
    { 0x9f5afbd9, 78 },
    { 0x1eafbdc, 90 },
    { 0xc02afbde, 23 },
    { 0xc9aafbdf, 99 },
    { 0, 0 },
    { 0xb84afbe1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78afbe5, 98 },
    { 0, 0 },
    { 0x26cafbe7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daafbee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687afbf4, 95 },
    { 0xceafbf5, 67 },
    { 0x10dafbf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65afbf9, 67 },
    { 0x53aafbfa, 97 },
    { 0x64fafbfb, 33 },
    { 0xda9afbfc, 81 },
    { 0x1eaafbfb, 90 },
    { 0xdbafbfe, 95 },
    { 0, 0 },
    { 0x283afc00, 90 },
    { 0x95cafc00, 85 },
    { 0x817afc02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84eafc0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50afc0f, 94 },
    { 0xf8dafc10, 55 },
    { 0xed5afc11, 91 },
    { 0x879afc11, 90 },
    { 0, 0 },
    { 0x75aafc14, 61 },
    { 0x85fafc15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aafc19, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6afc1c, 98 },
    { 0x1bdafc1d, 98 },
    { 0xe4dafc1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700afc29, 93 },
    { 0, 0 },
    { 0x230afc2b, 33 },
    { 0xeb1afc2c, 78 },
    { 0xc53afc2d, 43 },
    { 0x620afc2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8afc37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9afc3b, 61 },
    { 0, 0 },
    { 0xd7aafc3d, 98 },
    { 0xd8eafc3e, 91 },
    { 0xf93afc3e, 33 },
    { 0xa0afc40, 76 },
    { 0x457afc41, 46 },
    { 0xd26afc42, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fafc45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874afc51, 95 },
    { 0xd37afc52, 90 },
    { 0x2beafc52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dafc57, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1afc5b, 91 },
    { 0xb37afc5b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bafc61, 95 },
    { 0xaf7afc62, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962afc6e, 77 },
    { 0x3deafc6f, 96 },
    { 0, 0 },
    { 0x75aafc71, 98 },
    { 0, 0 },
    { 0x93dafc73, 33 },
    { 0x40aafc74, 33 },
    { 0, 0 },
    { 0x368afc76, 76 },
    { 0x4bbafc77, 23 },
    { 0x34dafc77, 23 },
    { 0, 0 },
    { 0x8bcafc7a, 76 },
    { 0x2a2afc7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7afc7f, 90 },
    { 0xad2afc7f, 43 },
    { 0x7a8afc81, 76 },
    { 0x20afc82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576afc8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3afc8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2afc91, 61 },
    { 0, 0 },
    { 0xda2afc93, 61 },
    { 0x99dafc94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385afc98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745afc9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7afc9f, 78 },
    { 0xe24afca0, 90 },
    { 0, 0 },
    { 0x4d3afca2, 90 },
    { 0x4e3afca3, 52 },
    { 0xf3afca3, 61 },
    { 0, 0 },
    { 0x6bdafca6, 46 },
    { 0x23bafca7, 98 },
    { 0xd09afca7, 75 },
    { 0, 0 },
    { 0x8d4afcaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63afcb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549afcba, 33 },
    { 0x418afcba, 98 },
    { 0xb4aafcbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714afcc2, 76 },
    { 0x8b7afcc3, 90 },
    { 0x13aafcc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bafccb, 95 },
    { 0xc35afccc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6deafccf, 78 },
    { 0, 0 },
    { 0xdd8afcd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddafcd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8afcdb, 94 },
    { 0, 0 },
    { 0xe86afcdd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08afce3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5afce7, 23 },
    { 0xb1dafce7, 33 },
    { 0x6e8afce9, 55 },
    { 0x5c7afce9, 90 },
    { 0x3f5afceb, 99 },
    { 0, 0 },
    { 0x44aafced, 93 },
    { 0x190afcee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92afcf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffafcf4, 61 },
    { 0, 0 },
    { 0x4b2afcf6, 93 },
    { 0, 0 },
    { 0xb0aafcf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x879afcfb, 81 },
    { 0, 0 },
    { 0xabeafcfd, 23 },
    { 0x790afcfe, 90 },
    { 0x644afcff, 61 },
    { 0x757afd00, 61 },
    { 0xb8eafd00, 66 },
    { 0xf83afd02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76afd08, 90 },
    { 0x334afd08, 61 },
    { 0x62afd09, 90 },
    { 0x422afd09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50afd0e, 55 },
    { 0, 0 },
    { 0x303afd10, 43 },
    { 0xbf3afd10, 76 },
    { 0x78afd11, 98 },
    { 0x152afd13, 78 },
    { 0x9f7afd13, 69 },
    { 0x6fdafd15, 98 },
    { 0x174afd13, 69 },
    { 0, 0 },
    { 0x5aeafd18, 33 },
    { 0, 0 },
    { 0xcb5afd1a, 95 },
    { 0, 0 },
    { 0xd88afd1c, 61 },
    { 0x9a2afd1d, 93 },
    { 0x6efafd1d, 76 },
    { 0, 0 },
    { 0x452afd20, 61 },
    { 0, 0 },
    { 0x29fafd22, 78 },
    { 0, 0 },
    { 0x6dbafd24, 90 },
    { 0xd2bafd25, 33 },
    { 0x90cafd26, 98 },
    { 0x217afd27, 90 },
    { 0xf2afd28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767afd2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deafd32, 90 },
    { 0x96eafd33, 76 },
    { 0xf73afd32, 23 },
    { 0xb94afd34, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fafd38, 33 },
    { 0, 0 },
    { 0x9f6afd3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21afd3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fafd42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bafd45, 90 },
    { 0, 0 },
    { 0x5a2afd47, 76 },
    { 0, 0 },
    { 0x83dafd49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x640afd4c, 95 },
    { 0, 0 },
    { 0x1fdafd4e, 95 },
    { 0x250afd4e, 43 },
    { 0, 0 },
    { 0x4aeafd51, 43 },
    { 0x54afd52, 64 },
    { 0, 0 },
    { 0x201afd54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49afd5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88afd67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bafd6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cafd6f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x848afd72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2afd78, 95 },
    { 0, 0 },
    { 0x938afd7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdafd81, 23 },
    { 0, 0 },
    { 0x803afd83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5afd86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5afd8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bafd8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beafd90, 61 },
    { 0, 0 },
    { 0xb24afd92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70afd96, 46 },
    { 0xa58afd97, 23 },
    { 0xa25afd98, 95 },
    { 0x122afd99, 46 },
    { 0x693afd9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcafd9d, 95 },
    { 0x75eafd9e, 61 },
    { 0x503afd9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96afda3, 87 },
    { 0xccafda3, 95 },
    { 0x701afda3, 61 },
    { 0x45afda6, 90 },
    { 0x2e7afda4, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10afdae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9afdb1, 100 },
    { 0x992afdb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9afdb5, 61 },
    { 0x5a9afdb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76afdba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeafdbd, 43 },
    { 0x563afdbd, 81 },
    { 0xeaaafdbf, 43 },
    { 0xc09afdc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821afdc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137afdcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afdd7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968afddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2afde0, 90 },
    { 0, 0 },
    { 0x286afde2, 66 },
    { 0xfdafde3, 66 },
    { 0, 0 },
    { 0x7c6afde5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dafde8, 61 },
    { 0x6e9afde9, 77 },
    { 0xc65afdea, 87 },
    { 0x18cafde8, 38 },
    { 0x1c9afdec, 38 },
    { 0x1a1afdec, 33 },
    { 0xd61afdee, 46 },
    { 0xb52afded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396afdf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687afdfb, 90 },
    { 0xd37afdfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4afe04, 33 },
    { 0, 0 },
    { 0x264afe06, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7afe0f, 23 },
    { 0xd4fafe10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03afe1a, 90 },
    { 0xd64afe1b, 38 },
    { 0, 0 },
    { 0x1e9afe1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aafe21, 90 },
    { 0x962afe21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957afe28, 61 },
    { 0xb93afe28, 90 },
    { 0x387afe28, 90 },
    { 0x86fafe29, 68 },
    { 0x83aafe2b, 101 },
    { 0xa41afe2b, 90 },
    { 0, 0 },
    { 0x943afe2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2afe33, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x388afe36, 95 },
    { 0, 0 },
    { 0xabaafe38, 17 },
    { 0x37cafe39, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeafe3c, 99 },
    { 0x54cafe3d, 93 },
    { 0xb33afe3e, 90 },
    { 0xe05afe3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40afe45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eafe48, 23 },
    { 0xa25afe48, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176afe4e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bafe53, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65afe58, 46 },
    { 0x113afe58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aafe5c, 95 },
    { 0xae3afe5d, 98 },
    { 0x3cbafe5e, 95 },
    { 0x1c7afe5c, 90 },
    { 0x183afe60, 76 },
    { 0xec2afe61, 61 },
    { 0x849afe62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66afe6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eafe73, 68 },
    { 0x5f2afe73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8afe78, 61 },
    { 0x3a8afe79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdafe7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cafe80, 100 },
    { 0, 0 },
    { 0x70fafe82, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128afe86, 66 },
    { 0x4bcafe86, 61 },
    { 0, 0 },
    { 0x832afe89, 43 },
    { 0x440afe8a, 90 },
    { 0, 0 },
    { 0x24afe8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fafe91, 77 },
    { 0x215afe91, 98 },
    { 0x2c2afe93, 98 },
    { 0x7c0afe93, 91 },
    { 0xec8afe93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dafe98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdafe9b, 46 },
    { 0x797afe9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dafe9f, 33 },
    { 0, 0 },
    { 0xa69afea1, 98 },
    { 0xc05afea2, 95 },
    { 0x7ffafea2, 95 },
    { 0, 0 },
    { 0x3feafea5, 43 },
    { 0x31bafea5, 55 },
    { 0xe0afea5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314afeb0, 98 },
    { 0, 0 },
    { 0xf07afeb2, 23 },
    { 0xa88afeb3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46afec0, 90 },
    { 0x603afec0, 90 },
    { 0xa8afec0, 95 },
    { 0x4c8afec3, 85 },
    { 0x93afec2, 90 },
    { 0x3fcafec3, 84 },
    { 0x93dafec6, 55 },
    { 0xadeafec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120afecd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadafed2, 23 },
    { 0x1faafed3, 38 },
    { 0xeecafed4, 34 },
    { 0xb49afed3, 76 },
    { 0x6c6afed6, 23 },
    { 0x474afed6, 23 },
    { 0xb9bafed8, 78 },
    { 0xe9eafed8, 61 },
    { 0x48bafed6, 76 },
    { 0xbbcafedb, 93 },
    { 0x651afedc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecafee4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cafee9, 61 },
    { 0xcb9afee9, 84 },
    { 0xeafeeb, 61 },
    { 0x3c3afeec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aafeef, 67 },
    { 0x7a8afef0, 61 },
    { 0x973afef1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13afef8, 87 },
    { 0xf3bafef8, 76 },
    { 0x8edafef8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543afefe, 98 },
    { 0xb76afeff, 98 },
    { 0x2faff00, 40 },
    { 0, 0 },
    { 0x83daff02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3caff07, 97 },
    { 0, 0 },
    { 0xe54aff09, 33 },
    { 0xb4eaff0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918aff0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aaff13, 90 },
    { 0x409aff14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291aff1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58baff1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10faff24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x371aff27, 90 },
    { 0, 0 },
    { 0x488aff29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7caff2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7aff31, 90 },
    { 0, 0 },
    { 0x91aff33, 99 },
    { 0xb6aff33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21aff3f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7aff4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbaff53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2faff56, 55 },
    { 0x1fdaff57, 65 },
    { 0xf1aaff58, 34 },
    { 0, 0 },
    { 0x2a8aff5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868aff5e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7aff62, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daaff66, 52 },
    { 0, 0 },
    { 0x912aff68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacaff6c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758aff78, 43 },
    { 0, 0 },
    { 0xceaff7a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedaff7e, 43 },
    { 0x9afaff7f, 33 },
    { 0x41faff80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723aff84, 95 },
    { 0x996aff85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14baff8a, 95 },
    { 0, 0 },
    { 0xf08aff8c, 98 },
    { 0xb4eaff8d, 55 },
    { 0, 0 },
    { 0xc9faff8f, 90 },
    { 0, 0 },
    { 0xc66aff91, 66 },
    { 0xa7eaff92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0aff97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33aff9a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5aff9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0affa0, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1affa5, 46 },
    { 0x308affa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x617affa9, 67 },
    { 0, 0 },
    { 0x932affab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaaffaf, 61 },
    { 0xee1affb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42affb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310affb9, 17 },
    { 0xfb8affba, 34 },
    { 0x45baffb9, 90 },
    { 0, 0 },
    { 0x213affbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314affc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14caffc6, 34 },
    { 0, 0 },
    { 0x756affc8, 33 },
    { 0, 0 },
    { 0xd95affca, 90 },
    { 0xcaaffca, 38 },
    { 0x9a6affcc, 34 },
    { 0x504affcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0baffd1, 69 },
    { 0, 0 },
    { 0x717affd3, 95 },
    { 0xb74affd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860affd8, 17 },
    { 0xbc7affd9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeaffe1, 90 },
    { 0x87daffe2, 98 },
    { 0x3e8affe3, 53 },
    { 0, 0 },
    { 0xf8caffe5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14affec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33affef, 33 },
    { 0x462affef, 93 },
    { 0x845afff1, 46 },
    { 0x605afff2, 67 },
    { 0xbd8afff3, 76 },
    { 0x9f1afff3, 90 },
    { 0, 0 },
    { 0x2caafff6, 90 },
    { 0x8c0afff6, 34 },
    { 0, 0 },
    { 0x329afff9, 95 },
    { 0, 0 },
    { 0xee7afffb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57affff, 40 },
    { 0x7e4b0000, 55 },
    { 0, 0 },
    { 0x1a9b0002, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb0007, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb000b, 46 },
    { 0x656b000c, 61 },
    { 0xe81b000b, 46 },
    { 0x40fb000d, 95 },
    { 0x4d5b000f, 76 },
    { 0, 0 },
    { 0x1dbb0011, 34 },
    { 0xb41b0012, 98 },
    { 0x284b0013, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcb0017, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cb001b, 33 },
    { 0x44cb001c, 90 },
    { 0xf22b001c, 90 },
    { 0xd1db001c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0b0023, 90 },
    { 0x265b0023, 67 },
    { 0x8eab0025, 98 },
    { 0, 0 },
    { 0x324b0027, 90 },
    { 0xccfb0027, 34 },
    { 0, 0 },
    { 0x506b002a, 93 },
    { 0x146b002b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edb0033, 90 },
    { 0, 0 },
    { 0xab3b0035, 33 },
    { 0x9f8b0036, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54b0039, 55 },
    { 0xa4bb003a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13b0040, 98 },
    { 0x393b0041, 61 },
    { 0, 0 },
    { 0x83cb0043, 95 },
    { 0, 0 },
    { 0xe6cb0045, 17 },
    { 0, 0 },
    { 0x2bdb0047, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26b004a, 90 },
    { 0xc61b004b, 61 },
    { 0x44ab004c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99eb004f, 33 },
    { 0x27eb004f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643b0056, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5b005b, 78 },
    { 0, 0 },
    { 0xaaab005d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfb0060, 90 },
    { 0x787b0060, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb0064, 81 },
    { 0x83bb0064, 93 },
    { 0x742b0066, 43 },
    { 0xc94b0065, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27b006f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09b0073, 61 },
    { 0x20eb0073, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb0077, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488b007d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fb0083, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571b0087, 98 },
    { 0x736b0088, 90 },
    { 0xafeb0087, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5b008d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfb0092, 81 },
    { 0x154b0092, 98 },
    { 0x589b0094, 61 },
    { 0x97db0095, 95 },
    { 0, 0 },
    { 0x464b0097, 87 },
    { 0x8f5b0097, 99 },
    { 0, 0 },
    { 0x7c3b009a, 76 },
    { 0x591b009b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bb009e, 98 },
    { 0, 0 },
    { 0xf5b00a0, 95 },
    { 0x5f6b00a1, 43 },
    { 0xc33b00a2, 91 },
    { 0xda8b00a0, 98 },
    { 0x7ccb00a1, 66 },
    { 0x473b00a3, 34 },
    { 0x923b00a1, 61 },
    { 0xd7cb00a1, 90 },
    { 0x9eb00a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x478b00ab, 46 },
    { 0xab0b00ab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187b00b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393b00b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440b00bc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b00c5, 33 },
    { 0xde3b00c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65b00cb, 90 },
    { 0x604b00cc, 90 },
    { 0, 0 },
    { 0xbd3b00ce, 90 },
    { 0, 0 },
    { 0xd49b00d0, 76 },
    { 0, 0 },
    { 0x504b00d2, 33 },
    { 0, 0 },
    { 0xd30b00d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cab00dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4b00e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3b00e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4b00eb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460b00f0, 76 },
    { 0, 0 },
    { 0x596b00f2, 61 },
    { 0x5c2b00f2, 90 },
    { 0xdc8b00f3, 91 },
    { 0x735b00f3, 67 },
    { 0x4b5b00f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576b00fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17b00ff, 43 },
    { 0, 0 },
    { 0xf28b0101, 23 },
    { 0, 0 },
    { 0xe6bb0103, 76 },
    { 0, 0 },
    { 0xab5b0105, 98 },
    { 0xd8db0105, 33 },
    { 0, 0 },
    { 0x195b0108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfeb010b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcb0115, 78 },
    { 0x304b0116, 23 },
    { 0, 0 },
    { 0x79eb0118, 33 },
    { 0, 0 },
    { 0x7db011a, 61 },
    { 0, 0 },
    { 0x110b011c, 23 },
    { 0x60db011d, 95 },
    { 0xa4cb011e, 61 },
    { 0xe7b011c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75b0128, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb012b, 33 },
    { 0, 0 },
    { 0xb9db012d, 33 },
    { 0, 0 },
    { 0xd1bb012f, 55 },
    { 0xbacb012f, 33 },
    { 0x10cb012f, 8 },
    { 0, 0 },
    { 0x9bb0133, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4b0137, 100 },
    { 0xbd4b0137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x831b013b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb013f, 61 },
    { 0x87ab0140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978b0147, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58eb0150, 33 },
    { 0xad1b0151, 98 },
    { 0x9b6b0151, 61 },
    { 0x772b0153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ab0158, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdb015c, 90 },
    { 0, 0 },
    { 0x8bfb015e, 98 },
    { 0x823b015f, 33 },
    { 0xa6cb0160, 98 },
    { 0x45eb0160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccb0165, 33 },
    { 0, 0 },
    { 0xb1cb0167, 55 },
    { 0, 0 },
    { 0xa36b0169, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcb016d, 98 },
    { 0, 0 },
    { 0xeeab016f, 23 },
    { 0x3e0b016f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fb0173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb0176, 33 },
    { 0, 0 },
    { 0xe82b0178, 55 },
    { 0xf3db0178, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16b017c, 78 },
    { 0x761b017d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1b0181, 100 },
    { 0xd54b0182, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25b0189, 76 },
    { 0x42b0189, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03b0191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9b0199, 85 },
    { 0x533b0199, 98 },
    { 0x434b019a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b019f, 100 },
    { 0x3cbb019f, 66 },
    { 0, 0 },
    { 0x12eb01a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773b01ab, 33 },
    { 0, 0 },
    { 0xbcb01ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bb01b0, 55 },
    { 0x174b01b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb01b7, 65 },
    { 0x352b01b7, 22 },
    { 0x9db01b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbb01c0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bb01c5, 76 },
    { 0x571b01c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17b01ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cab01d1, 61 },
    { 0x448b01d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1b01e4, 33 },
    { 0xba4b01e4, 61 },
    { 0, 0 },
    { 0x987b01e7, 90 },
    { 0x5cdb01e8, 52 },
    { 0x2c5b01e7, 100 },
    { 0xe03b01ea, 34 },
    { 0xbadb01ea, 23 },
    { 0xf29b01ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bb01ef, 65 },
    { 0xf3ab01f0, 90 },
    { 0x515b01ef, 69 },
    { 0, 0 },
    { 0x161b01f3, 67 },
    { 0x18bb01f3, 77 },
    { 0x6bdb01f4, 75 },
    { 0x3b3b01f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fb01f9, 68 },
    { 0, 0 },
    { 0x32ab01fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0b01ff, 100 },
    { 0xf2cb0200, 90 },
    { 0, 0 },
    { 0xac6b0202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205b0206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdb0209, 99 },
    { 0xd6ab0209, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9b020d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbb0211, 90 },
    { 0x72b0212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fb0216, 23 },
    { 0x5eab0217, 76 },
    { 0xeb1b0218, 55 },
    { 0, 0 },
    { 0x406b021a, 90 },
    { 0xa58b021b, 33 },
    { 0xa09b021b, 76 },
    { 0x60cb021b, 98 },
    { 0, 0 },
    { 0xb50b021f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x458b0222, 76 },
    { 0, 0 },
    { 0x7b9b0224, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b022c, 98 },
    { 0, 0 },
    { 0xbceb022e, 53 },
    { 0, 0 },
    { 0x4fcb0230, 76 },
    { 0x670b0231, 23 },
    { 0xd52b0230, 34 },
    { 0xdc7b0233, 96 },
    { 0xabbb0234, 81 },
    { 0xcd1b0233, 91 },
    { 0xf2eb0236, 23 },
    { 0, 0 },
    { 0x50cb0238, 87 },
    { 0x877b0239, 77 },
    { 0, 0 },
    { 0x741b023b, 55 },
    { 0, 0 },
    { 0x1c6b023d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bb0245, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eb024b, 33 },
    { 0x252b024b, 90 },
    { 0, 0 },
    { 0x61b024e, 98 },
    { 0, 0 },
    { 0xd9cb0250, 61 },
    { 0, 0 },
    { 0x87eb0252, 33 },
    { 0x4a9b0252, 33 },
    { 0x20eb0253, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcb025a, 61 },
    { 0, 0 },
    { 0xb0eb025c, 95 },
    { 0xd36b025d, 61 },
    { 0, 0 },
    { 0x1fcb025f, 90 },
    { 0, 0 },
    { 0xde1b0261, 34 },
    { 0, 0 },
    { 0x589b0263, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261b0269, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b026c, 98 },
    { 0x16fb026d, 98 },
    { 0, 0 },
    { 0x526b026f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457b0278, 23 },
    { 0, 0 },
    { 0x9b5b027a, 90 },
    { 0xd8cb027b, 33 },
    { 0x262b027a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x409b027f, 55 },
    { 0x61fb027f, 34 },
    { 0xdf1b027f, 33 },
    { 0xb71b027f, 90 },
    { 0, 0 },
    { 0x4c1b0284, 99 },
    { 0xa17b0285, 46 },
    { 0x6adb0284, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b028b, 46 },
    { 0, 0 },
    { 0x3b4b028d, 23 },
    { 0, 0 },
    { 0x607b028f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96b0292, 25 },
    { 0xacdb0293, 43 },
    { 0, 0 },
    { 0x2bb0295, 61 },
    { 0xd73b0295, 43 },
    { 0x291b0297, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692b029b, 33 },
    { 0xb80b029c, 90 },
    { 0, 0 },
    { 0x479b029e, 87 },
    { 0xc38b029e, 98 },
    { 0xeefb029e, 69 },
    { 0xac9b02a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ab02a6, 95 },
    { 0, 0 },
    { 0xcf7b02a8, 76 },
    { 0x3fcb02a9, 34 },
    { 0x2a3b02a9, 61 },
    { 0xedb02ab, 95 },
    { 0xffeb02a9, 67 },
    { 0x80db02ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffb02b4, 93 },
    { 0x8b5b02b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0b02b8, 61 },
    { 0x38db02b9, 90 },
    { 0xfedb02ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ab02be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b02c6, 98 },
    { 0x351b02c7, 90 },
    { 0x59eb02c7, 23 },
    { 0x717b02c7, 33 },
    { 0x961b02c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb02d1, 23 },
    { 0x586b02d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffb02d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab02db, 23 },
    { 0, 0 },
    { 0x23b02dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b02e9, 75 },
    { 0, 0 },
    { 0xd9db02eb, 99 },
    { 0xa91b02ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8b02f0, 84 },
    { 0xda3b02f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0b02f7, 66 },
    { 0, 0 },
    { 0x920b02f9, 61 },
    { 0, 0 },
    { 0x8dcb02fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225b02ff, 95 },
    { 0xadb0300, 90 },
    { 0x739b0301, 98 },
    { 0xf1db0301, 77 },
    { 0xde9b0303, 33 },
    { 0x855b0303, 33 },
    { 0x911b0305, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188b0309, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0b030f, 91 },
    { 0xf36b0310, 95 },
    { 0x88fb0311, 90 },
    { 0xce6b0311, 69 },
    { 0x365b0311, 90 },
    { 0x995b0313, 61 },
    { 0x73db030f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abb0319, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881b031d, 55 },
    { 0, 0 },
    { 0xdb9b031f, 66 },
    { 0xb2bb0320, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82b0323, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x61db0326, 81 },
    { 0xeccb0326, 98 },
    { 0xe51b0328, 66 },
    { 0xfbab0329, 91 },
    { 0x9e5b032a, 61 },
    { 0xb25b032a, 91 },
    { 0xf4b0328, 55 },
    { 0x2c9b032d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97b0330, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb0333, 90 },
    { 0x746b0333, 33 },
    { 0xf40b0333, 61 },
    { 0xd51b0336, 77 },
    { 0xfdab0336, 65 },
    { 0x3e2b0338, 85 },
    { 0, 0 },
    { 0x83b033a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6b033e, 99 },
    { 0x907b033f, 91 },
    { 0, 0 },
    { 0x8fb0341, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x198b0344, 98 },
    { 0, 0 },
    { 0xd87b0346, 23 },
    { 0x81cb0346, 90 },
    { 0x25bb0346, 61 },
    { 0xe4db0347, 34 },
    { 0xbb0b0346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b034e, 53 },
    { 0xe16b034f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68b0352, 43 },
    { 0x93fb0352, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b035a, 90 },
    { 0, 0 },
    { 0x1f7b035c, 33 },
    { 0, 0 },
    { 0x86fb035e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b0363, 88 },
    { 0, 0 },
    { 0x955b0365, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96db0369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fb036f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x970b0372, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799b037c, 93 },
    { 0xa83b037c, 98 },
    { 0x3f7b037e, 90 },
    { 0, 0 },
    { 0x7edb0380, 95 },
    { 0xf14b0381, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eb0384, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cb0388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4b038b, 99 },
    { 0x997b038c, 23 },
    { 0xa73b038d, 85 },
    { 0xc47b038e, 91 },
    { 0, 0 },
    { 0x806b0390, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ab0395, 78 },
    { 0x50db0395, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b0399, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367b03a5, 40 },
    { 0x638b03a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b03ab, 76 },
    { 0xf09b03ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18db03af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b03b7, 95 },
    { 0xc05b03b8, 61 },
    { 0, 0 },
    { 0xd99b03ba, 34 },
    { 0, 0 },
    { 0xbdb03bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551b03c5, 23 },
    { 0x735b03c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db03c9, 46 },
    { 0xfc6b03c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01b03cf, 33 },
    { 0, 0 },
    { 0xd10b03d1, 66 },
    { 0x687b03d1, 61 },
    { 0xc7eb03d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9b03d8, 23 },
    { 0x39fb03d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4b03dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21b03e2, 93 },
    { 0x9cab03e3, 76 },
    { 0x28fb03e4, 68 },
    { 0, 0 },
    { 0x6f4b03e6, 90 },
    { 0, 0 },
    { 0xa48b03e8, 33 },
    { 0x55b03e9, 95 },
    { 0, 0 },
    { 0x8acb03eb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7b03f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaab03f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db03fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cb03ff, 98 },
    { 0, 0 },
    { 0xf1fb0401, 90 },
    { 0xa1eb0402, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23b0407, 51 },
    { 0, 0 },
    { 0x885b0409, 33 },
    { 0x587b040a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00b040d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9b041c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6b0425, 95 },
    { 0xb3bb0426, 100 },
    { 0x36eb0426, 100 },
    { 0, 0 },
    { 0xda5b0429, 61 },
    { 0xb36b042a, 68 },
    { 0x454b042b, 34 },
    { 0xbaeb042b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacb0430, 17 },
    { 0x2a6b0430, 55 },
    { 0xf5fb0430, 95 },
    { 0x2ffb0433, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x445b0436, 43 },
    { 0, 0 },
    { 0x648b0438, 98 },
    { 0x16ab0438, 98 },
    { 0x67b0438, 87 },
    { 0xb08b043b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54b043f, 85 },
    { 0x2a2b043f, 95 },
    { 0xd1bb0440, 90 },
    { 0x248b0442, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7b0447, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300b044d, 66 },
    { 0xc11b044e, 23 },
    { 0x2e3b044f, 93 },
    { 0xf7cb044d, 90 },
    { 0x2f1b0451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eb0454, 23 },
    { 0xe2db0454, 90 },
    { 0, 0 },
    { 0xb30b0457, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8b045a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x209b045d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadb0460, 98 },
    { 0x92db0461, 88 },
    { 0x149b0461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbb0465, 66 },
    { 0x91db0466, 55 },
    { 0, 0 },
    { 0xd36b0468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb046b, 23 },
    { 0x53fb046b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bb0470, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4deb0473, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb0479, 61 },
    { 0, 0 },
    { 0xc9cb047b, 33 },
    { 0x9efb047b, 90 },
    { 0xa85b047d, 90 },
    { 0x9b4b047d, 76 },
    { 0x32ab047f, 40 },
    { 0, 0 },
    { 0x54eb0481, 23 },
    { 0, 0 },
    { 0x46db0483, 55 },
    { 0x9a6b0484, 67 },
    { 0, 0 },
    { 0x997b0486, 34 },
    { 0, 0 },
    { 0x27bb0488, 61 },
    { 0xbd6b0489, 90 },
    { 0xfd9b048a, 66 },
    { 0x29eb0488, 95 },
    { 0, 0 },
    { 0xbc2b048d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1deb049b, 46 },
    { 0xaadb049c, 23 },
    { 0x37bb049b, 90 },
    { 0x128b049c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebb04a1, 66 },
    { 0xc83b04a2, 77 },
    { 0, 0 },
    { 0xcefb04a4, 33 },
    { 0x878b04a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552b04a9, 17 },
    { 0x36bb04aa, 98 },
    { 0xd1b04ab, 76 },
    { 0x475b04aa, 78 },
    { 0xcc0b04ad, 46 },
    { 0, 0 },
    { 0x35db04af, 17 },
    { 0x7fdb04af, 95 },
    { 0, 0 },
    { 0x83ab04b2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfb04b5, 33 },
    { 0, 0 },
    { 0x528b04b7, 99 },
    { 0x6a4b04b8, 23 },
    { 0xe6ab04b8, 100 },
    { 0xb6eb04ba, 76 },
    { 0x62fb04bb, 23 },
    { 0xf5fb04b7, 90 },
    { 0x343b04bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6b04c3, 33 },
    { 0xf74b04c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb04c9, 33 },
    { 0xe5eb04c9, 23 },
    { 0xd8cb04cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5b04da, 99 },
    { 0x564b04da, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bb04e9, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6b04ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x468b04f0, 23 },
    { 0xa38b04f1, 17 },
    { 0x23eb04f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ab04f7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ab0503, 24 },
    { 0x5b7b0504, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x444b0507, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x959b050a, 87 },
    { 0, 0 },
    { 0x2abb050c, 23 },
    { 0, 0 },
    { 0x386b050e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcb0514, 90 },
    { 0xde5b0515, 95 },
    { 0x4b6b0516, 69 },
    { 0, 0 },
    { 0x72db0518, 61 },
    { 0x3a0b0519, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x597b051c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eb051f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bb0527, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x511b052a, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b0534, 98 },
    { 0, 0 },
    { 0x703b0536, 98 },
    { 0x15bb0536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abb053b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452b0540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3b0543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x780b0546, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590b054a, 33 },
    { 0x9b2b054b, 55 },
    { 0, 0 },
    { 0xcceb054d, 33 },
    { 0x3d9b054d, 61 },
    { 0x9c9b054e, 23 },
    { 0, 0 },
    { 0xf21b0551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb0559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b055d, 87 },
    { 0, 0 },
    { 0x29bb055f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb0564, 43 },
    { 0, 0 },
    { 0x930b0566, 61 },
    { 0x11ab0567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67b056b, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1b056e, 34 },
    { 0, 0 },
    { 0xc95b0570, 66 },
    { 0xc0eb0571, 55 },
    { 0x4b0b0571, 55 },
    { 0xd36b0570, 33 },
    { 0, 0 },
    { 0x32cb0575, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bb0579, 76 },
    { 0xa8ab057a, 43 },
    { 0x13eb057b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acb057f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb0582, 33 },
    { 0xc9db0583, 85 },
    { 0x54b0583, 66 },
    { 0xccbb0585, 90 },
    { 0x918b0583, 23 },
    { 0x918b0587, 33 },
    { 0xbdeb0587, 65 },
    { 0, 0 },
    { 0xa27b058a, 98 },
    { 0, 0 },
    { 0xbadb058c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321b0591, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0b0598, 61 },
    { 0, 0 },
    { 0xdd6b059a, 98 },
    { 0x6acb059b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2b059f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101b05a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b05a9, 84 },
    { 0xe7bb05aa, 90 },
    { 0, 0 },
    { 0xab5b05ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x264b05af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217b05ba, 33 },
    { 0x5bcb05bb, 23 },
    { 0, 0 },
    { 0x5bdb05bd, 99 },
    { 0x867b05bd, 90 },
    { 0x7d4b05bf, 23 },
    { 0xa7bb05c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9b05c3, 76 },
    { 0x60bb05c4, 33 },
    { 0, 0 },
    { 0xe4fb05c6, 34 },
    { 0x889b05c7, 90 },
    { 0xe01b05c6, 19 },
    { 0x8ebb05c9, 90 },
    { 0x7d5b05c8, 93 },
    { 0x1c0b05cb, 78 },
    { 0x8d4b05c9, 46 },
    { 0x34fb05cd, 95 },
    { 0xebab05ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197b05d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8b05d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x728b05dc, 95 },
    { 0x46eb05dd, 55 },
    { 0xb86b05dd, 67 },
    { 0x6dab05dd, 55 },
    { 0, 0 },
    { 0x323b05e1, 90 },
    { 0xf86b05e2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeb05e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37b05ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4b05f2, 23 },
    { 0, 0 },
    { 0xfe5b05f4, 61 },
    { 0xb3cb05f5, 33 },
    { 0x679b05f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b05fb, 98 },
    { 0x85b05fc, 52 },
    { 0x620b05fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b0601, 95 },
    { 0, 0 },
    { 0xe85b0603, 91 },
    { 0x74ab0604, 80 },
    { 0xfc7b0605, 33 },
    { 0xdabb0606, 33 },
    { 0, 0 },
    { 0x5fb0608, 96 },
    { 0x215b0609, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38b060f, 84 },
    { 0xe78b060f, 33 },
    { 0x1c0b0611, 90 },
    { 0xefdb0612, 61 },
    { 0, 0 },
    { 0xba6b0614, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210b061b, 75 },
    { 0, 0 },
    { 0xe24b061d, 95 },
    { 0, 0 },
    { 0xc88b061f, 76 },
    { 0, 0 },
    { 0xc93b0621, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dab0624, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b0627, 34 },
    { 0x5b7b0628, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49b062e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8b0631, 87 },
    { 0x100b0632, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b0637, 33 },
    { 0, 0 },
    { 0x5a3b0639, 88 },
    { 0x323b063a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5b063e, 33 },
    { 0x41db063e, 60 },
    { 0x979b0640, 23 },
    { 0xccbb0641, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b0645, 98 },
    { 0, 0 },
    { 0xefdb0647, 55 },
    { 0xc6fb0648, 87 },
    { 0xf0ab0649, 34 },
    { 0xb7bb0649, 80 },
    { 0x4c6b064a, 95 },
    { 0, 0 },
    { 0xe6ab064d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70b0651, 90 },
    { 0xae6b0652, 90 },
    { 0x6a9b0653, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32b0656, 61 },
    { 0x2ecb0657, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdb065f, 61 },
    { 0xc6ab0660, 34 },
    { 0, 0 },
    { 0x4f6b0662, 55 },
    { 0x350b0663, 67 },
    { 0xdfbb0663, 34 },
    { 0x9d0b0664, 45 },
    { 0xa4fb0666, 95 },
    { 0x8b4b0667, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fb066a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909b066f, 77 },
    { 0xdf6b0670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eb0674, 33 },
    { 0, 0 },
    { 0x5ddb0676, 33 },
    { 0, 0 },
    { 0x2c4b0678, 33 },
    { 0, 0 },
    { 0x993b067a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94b067d, 61 },
    { 0, 0 },
    { 0xda0b067f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b0686, 90 },
    { 0x751b0686, 90 },
    { 0x50b0687, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535b068e, 55 },
    { 0x841b068e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x606b0692, 84 },
    { 0, 0 },
    { 0xf23b0694, 90 },
    { 0, 0 },
    { 0xa9fb0696, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1b0699, 85 },
    { 0x64cb0699, 99 },
    { 0x15cb069b, 84 },
    { 0x93ab069c, 61 },
    { 0x7db069a, 40 },
    { 0x15eb069c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9b06a2, 43 },
    { 0x522b06a3, 55 },
    { 0x202b06a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75b06ac, 90 },
    { 0x446b06ad, 95 },
    { 0xb44b06ae, 33 },
    { 0x29b06af, 81 },
    { 0x651b06ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7b06b5, 98 },
    { 0x2abb06b5, 53 },
    { 0xe03b06b7, 34 },
    { 0xe33b06b8, 33 },
    { 0x809b06b9, 33 },
    { 0x5a4b06ba, 55 },
    { 0, 0 },
    { 0x27fb06bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b06c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78b06c5, 33 },
    { 0, 0 },
    { 0xba2b06c7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5b06cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cb06cf, 66 },
    { 0xe1b06cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181b06d4, 98 },
    { 0x7d8b06d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab06df, 33 },
    { 0x55db06df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cb06e3, 96 },
    { 0x98bb06e4, 61 },
    { 0x4ab06e4, 90 },
    { 0, 0 },
    { 0x9a1b06e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffb06eb, 61 },
    { 0x452b06eb, 46 },
    { 0xb81b06eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8b06f1, 61 },
    { 0, 0 },
    { 0x129b06f3, 23 },
    { 0, 0 },
    { 0xda4b06f5, 90 },
    { 0xa90b06f5, 84 },
    { 0xb7ab06f7, 76 },
    { 0xabfb06f8, 90 },
    { 0x970b06f9, 90 },
    { 0, 0 },
    { 0xfcdb06fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02b0700, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ab0705, 34 },
    { 0xa68b0706, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b0709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10b070f, 43 },
    { 0xbc0b070f, 38 },
    { 0xa7db0711, 95 },
    { 0, 0 },
    { 0xaa8b0713, 90 },
    { 0xefab0714, 23 },
    { 0x6f3b0715, 76 },
    { 0x679b0716, 76 },
    { 0x9a8b0717, 98 },
    { 0x35eb0717, 23 },
    { 0x429b0719, 38 },
    { 0xaacb0718, 61 },
    { 0xa75b0715, 82 },
    { 0x31eb071c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ab0725, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b0729, 61 },
    { 0x7cab072a, 90 },
    { 0xfe8b072b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58db0730, 23 },
    { 0x106b0730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b0735, 90 },
    { 0, 0 },
    { 0xfd4b0737, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ab073a, 43 },
    { 0, 0 },
    { 0xce8b073c, 90 },
    { 0, 0 },
    { 0x183b073e, 90 },
    { 0x9dbb073f, 61 },
    { 0x780b0740, 33 },
    { 0, 0 },
    { 0xfddb0742, 84 },
    { 0xedbb0743, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312b0749, 67 },
    { 0x808b0749, 55 },
    { 0xd5ab074b, 90 },
    { 0x557b074b, 84 },
    { 0xc6ab074d, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7b0750, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25b0753, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617b0757, 90 },
    { 0, 0 },
    { 0x2d6b0759, 61 },
    { 0xa7cb075a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b075d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982b0760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x284b0763, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20b0767, 34 },
    { 0xe32b0768, 33 },
    { 0x511b0769, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8b076d, 61 },
    { 0x3b5b076e, 90 },
    { 0xc11b076f, 33 },
    { 0xf5b076f, 33 },
    { 0xf52b0771, 90 },
    { 0x191b0772, 61 },
    { 0xcb1b0771, 55 },
    { 0xefab0773, 40 },
    { 0x98fb0774, 90 },
    { 0xb62b076f, 61 },
    { 0x75b0771, 86 },
    { 0xc64b0778, 91 },
    { 0x406b0774, 99 },
    { 0x9d9b077a, 84 },
    { 0x22ab0772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2b077e, 61 },
    { 0x884b077f, 65 },
    { 0xd69b0780, 100 },
    { 0, 0 },
    { 0x16ab0782, 23 },
    { 0x2b7b0782, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ab0787, 33 },
    { 0x9f9b0787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb078f, 23 },
    { 0xda8b0790, 61 },
    { 0x964b078f, 55 },
    { 0x66db0792, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41db0796, 90 },
    { 0x60b0797, 52 },
    { 0x83eb0798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb079c, 92 },
    { 0, 0 },
    { 0xb12b079e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83db07a2, 78 },
    { 0xd23b07a3, 55 },
    { 0x460b07a3, 23 },
    { 0x91fb07a2, 95 },
    { 0x4c3b07a3, 93 },
    { 0xb83b07a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ab07ac, 90 },
    { 0xb71b07ad, 76 },
    { 0x41b07ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26b07b1, 100 },
    { 0x88cb07b2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x325b07b5, 99 },
    { 0, 0 },
    { 0xf0fb07b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb07bd, 90 },
    { 0x8a9b07bd, 76 },
    { 0x4d0b07bf, 33 },
    { 0, 0 },
    { 0xc83b07c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fb07c4, 61 },
    { 0, 0 },
    { 0x686b07c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9db07cc, 23 },
    { 0, 0 },
    { 0x127b07ce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b07df, 55 },
    { 0, 0 },
    { 0x59ab07e1, 93 },
    { 0xa26b07e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cb07e8, 90 },
    { 0x2a4b07e8, 90 },
    { 0x555b07ea, 95 },
    { 0xe4bb07ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7b07f0, 77 },
    { 0xa09b07f0, 95 },
    { 0x97ab07f2, 95 },
    { 0xf13b07f3, 90 },
    { 0x503b07f2, 98 },
    { 0x25db07f3, 93 },
    { 0, 0 },
    { 0xf2eb07f7, 90 },
    { 0, 0 },
    { 0xfcdb07f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cb07fc, 55 },
    { 0, 0 },
    { 0xacab07fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6b0801, 33 },
    { 0x7c7b0801, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x522b0805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95b0808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb080b, 23 },
    { 0xbd9b080c, 5 },
    { 0x8fbb080c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508b0811, 33 },
    { 0, 0 },
    { 0x113b0813, 33 },
    { 0xf04b0813, 88 },
    { 0x61ab0814, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b081a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b081d, 55 },
    { 0x53fb081d, 93 },
    { 0xf21b081f, 95 },
    { 0x8db0820, 55 },
    { 0x89ab0821, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b0829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x187b082c, 90 },
    { 0, 0 },
    { 0x11ab082e, 78 },
    { 0, 0 },
    { 0x99eb0830, 17 },
    { 0x2db0831, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75b0837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ab083a, 34 },
    { 0x33bb083b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b083f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530b0843, 90 },
    { 0, 0 },
    { 0x66cb0845, 61 },
    { 0, 0 },
    { 0x5bb0847, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8b084d, 76 },
    { 0x209b084e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bb0859, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25b085c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feb0864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab0869, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962b086d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cb0871, 84 },
    { 0x221b0872, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b0878, 66 },
    { 0xad2b0879, 52 },
    { 0x543b0878, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b0887, 43 },
    { 0x76bb0888, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b088c, 61 },
    { 0x518b088c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ab0893, 61 },
    { 0xf30b0894, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00b0897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edb08a4, 90 },
    { 0x741b08a4, 23 },
    { 0, 0 },
    { 0x68cb08a7, 59 },
    { 0xe92b08a7, 61 },
    { 0x944b08a9, 45 },
    { 0, 0 },
    { 0x270b08ab, 90 },
    { 0xf91b08ab, 98 },
    { 0xf98b08ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95b08b2, 98 },
    { 0xb77b08b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50db08ba, 33 },
    { 0x871b08ba, 23 },
    { 0xd95b08bc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eb08c0, 91 },
    { 0xdb6b08c1, 81 },
    { 0x198b08c1, 98 },
    { 0x748b08c1, 88 },
    { 0x712b08c4, 90 },
    { 0xeccb08c0, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859b08c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91b08ce, 77 },
    { 0x45db08cf, 53 },
    { 0x697b08cf, 61 },
    { 0xfdfb08d0, 78 },
    { 0xaf5b08d2, 90 },
    { 0xd54b08d3, 34 },
    { 0x35fb08ce, 76 },
    { 0x774b08cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10b08dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb08e1, 23 },
    { 0, 0 },
    { 0x75eb08e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb08e9, 81 },
    { 0x5d9b08e9, 33 },
    { 0, 0 },
    { 0xd2b08ec, 80 },
    { 0xbbbb08ed, 55 },
    { 0x67bb08ee, 78 },
    { 0x12fb08ee, 33 },
    { 0x17eb08f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b08f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225b08f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89b0906, 23 },
    { 0x86fb0906, 93 },
    { 0, 0 },
    { 0xc82b0909, 33 },
    { 0, 0 },
    { 0x569b090b, 90 },
    { 0x4cbb090b, 67 },
    { 0x379b090d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b0910, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485b0915, 95 },
    { 0, 0 },
    { 0x68cb0917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b091a, 90 },
    { 0x7a6b091b, 95 },
    { 0, 0 },
    { 0x44ab091d, 90 },
    { 0, 0 },
    { 0xa48b091f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0b0922, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456b0926, 23 },
    { 0, 0 },
    { 0x83eb0928, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23db092b, 34 },
    { 0x173b092c, 78 },
    { 0x77eb092d, 87 },
    { 0xeb0b092c, 91 },
    { 0x287b092b, 91 },
    { 0x338b0930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5b093f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b0944, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeb094a, 98 },
    { 0x902b094b, 46 },
    { 0xfb3b094c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6db0959, 90 },
    { 0xe08b095a, 23 },
    { 0, 0 },
    { 0xcfab095c, 34 },
    { 0, 0 },
    { 0xdbcb095e, 33 },
    { 0x5f0b095e, 43 },
    { 0, 0 },
    { 0x25bb0961, 66 },
    { 0x6c4b0962, 23 },
    { 0x6adb0963, 100 },
    { 0x1c5b0964, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbb0967, 61 },
    { 0xa36b0968, 33 },
    { 0xf1db0967, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b096e, 43 },
    { 0x24cb096f, 98 },
    { 0x855b0970, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bb0979, 95 },
    { 0xbdfb097a, 95 },
    { 0x398b097b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b0987, 95 },
    { 0, 0 },
    { 0x75fb0989, 87 },
    { 0, 0 },
    { 0x6a0b098b, 77 },
    { 0x5eab098c, 90 },
    { 0x816b098d, 76 },
    { 0x8ab098e, 95 },
    { 0xc74b098f, 43 },
    { 0xb96b0990, 61 },
    { 0x368b0990, 81 },
    { 0xc6fb0992, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb0996, 61 },
    { 0x186b0997, 33 },
    { 0, 0 },
    { 0x7b2b0999, 33 },
    { 0, 0 },
    { 0x9e3b099b, 77 },
    { 0xc65b099c, 23 },
    { 0xf47b099d, 33 },
    { 0x74ab099e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ab09a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6b09a9, 98 },
    { 0x727b09aa, 55 },
    { 0, 0 },
    { 0x986b09ac, 52 },
    { 0xba0b09ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32b09b0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cb09b4, 43 },
    { 0, 0 },
    { 0xd91b09b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0db09b9, 43 },
    { 0xeedb09ba, 75 },
    { 0, 0 },
    { 0x122b09bc, 90 },
    { 0xeaab09bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b09c0, 99 },
    { 0, 0 },
    { 0xd7db09c2, 98 },
    { 0x2a3b09c3, 78 },
    { 0xf7bb09c4, 95 },
    { 0x8a5b09c5, 90 },
    { 0x335b09c3, 52 },
    { 0, 0 },
    { 0xe34b09c8, 95 },
    { 0x695b09c9, 61 },
    { 0x358b09c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adb09ce, 67 },
    { 0, 0 },
    { 0xa05b09d0, 84 },
    { 0xbf3b09d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b09d4, 56 },
    { 0, 0 },
    { 0x9b9b09d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9b09d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2b09df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12b09e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db09e8, 98 },
    { 0x717b09e9, 23 },
    { 0xf5b09e9, 93 },
    { 0xb94b09eb, 52 },
    { 0xfa0b09eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db09f1, 61 },
    { 0x9cbb09f2, 40 },
    { 0xea9b09f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbb09f7, 33 },
    { 0x8e5b09f8, 90 },
    { 0x4a8b09f8, 61 },
    { 0xf8eb09fa, 84 },
    { 0x375b09f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb09fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b0a01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68b0a05, 33 },
    { 0, 0 },
    { 0x3dfb0a07, 67 },
    { 0xe33b0a07, 46 },
    { 0xe79b0a09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b0a0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cb0a13, 98 },
    { 0xd17b0a14, 17 },
    { 0, 0 },
    { 0x5b3b0a16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbb0a19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ab0a1c, 55 },
    { 0, 0 },
    { 0xc38b0a1e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1b0a22, 61 },
    { 0xacab0a23, 99 },
    { 0xea4b0a23, 33 },
    { 0xdf0b0a23, 99 },
    { 0xa8fb0a25, 90 },
    { 0x9c7b0a27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2b0a2c, 90 },
    { 0xd1eb0a2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efb0a32, 22 },
    { 0x54ab0a33, 95 },
    { 0x96b0a34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6deb0a3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1b0a3f, 33 },
    { 0, 0 },
    { 0x3b2b0a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bb0a4a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28b0a51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb0a56, 98 },
    { 0x511b0a57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5b0a62, 23 },
    { 0, 0 },
    { 0xeafb0a64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2b0a69, 87 },
    { 0, 0 },
    { 0x396b0a6b, 95 },
    { 0x186b0a6c, 55 },
    { 0x281b0a6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db0a70, 23 },
    { 0x7d2b0a70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x629b0a74, 100 },
    { 0x6feb0a75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724b0a79, 23 },
    { 0xb4ab0a7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4b0a7e, 66 },
    { 0, 0 },
    { 0xc50b0a80, 65 },
    { 0, 0 },
    { 0xba6b0a82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5b0a85, 33 },
    { 0x328b0a86, 98 },
    { 0x1f0b0a87, 77 },
    { 0xa40b0a86, 43 },
    { 0x5e4b0a89, 33 },
    { 0, 0 },
    { 0xa19b0a8b, 56 },
    { 0, 0 },
    { 0xf63b0a8d, 55 },
    { 0x624b0a8e, 91 },
    { 0x304b0a8d, 67 },
    { 0xe6b0a8e, 23 },
    { 0x7d6b0a8d, 78 },
    { 0xbacb0a92, 90 },
    { 0, 0 },
    { 0x55eb0a94, 33 },
    { 0xdc1b0a95, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfb0a9a, 33 },
    { 0xaf5b0a9b, 81 },
    { 0x97b0a9a, 33 },
    { 0x165b0a9d, 98 },
    { 0, 0 },
    { 0xadbb0a9f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44b0aa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01b0aaa, 77 },
    { 0x392b0aaa, 33 },
    { 0, 0 },
    { 0xa87b0aad, 75 },
    { 0, 0 },
    { 0xb87b0aaf, 87 },
    { 0, 0 },
    { 0xe53b0ab1, 61 },
    { 0xcb6b0ab1, 34 },
    { 0xa8db0ab3, 95 },
    { 0, 0 },
    { 0x9f3b0ab5, 23 },
    { 0x72cb0ab6, 99 },
    { 0x75eb0ab6, 76 },
    { 0x620b0ab8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4b0abd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ab0ac1, 90 },
    { 0x88b0ac1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9b0acc, 87 },
    { 0x68cb0acd, 95 },
    { 0x3e4b0acc, 93 },
    { 0xbb9b0ace, 23 },
    { 0x7f2b0ad0, 93 },
    { 0, 0 },
    { 0xc8eb0ad2, 34 },
    { 0xe2bb0ad2, 85 },
    { 0x9deb0ad4, 46 },
    { 0xf5ab0ad2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x56db0ad8, 82 },
    { 0xc16b0ad9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198b0add, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659b0ae1, 61 },
    { 0x6f6b0ae2, 23 },
    { 0x18cb0ae3, 55 },
    { 0x9d6b0ae3, 17 },
    { 0, 0 },
    { 0x38fb0ae6, 76 },
    { 0xb1b0ae7, 98 },
    { 0, 0 },
    { 0x8ebb0ae9, 66 },
    { 0, 0 },
    { 0xb0eb0aeb, 78 },
    { 0x2a8b0aec, 33 },
    { 0x56db0aed, 98 },
    { 0, 0 },
    { 0x98b0aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3b0af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63b0af8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07b0afc, 98 },
    { 0, 0 },
    { 0xef2b0afe, 34 },
    { 0xa10b0aff, 33 },
    { 0x400b0b00, 40 },
    { 0x575b0aff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863b0b08, 77 },
    { 0xe73b0b08, 46 },
    { 0, 0 },
    { 0xa3bb0b0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafb0b0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808b0b13, 90 },
    { 0xd42b0b13, 95 },
    { 0xd9eb0b15, 90 },
    { 0xf35b0b16, 95 },
    { 0xadbb0b17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab0b1c, 95 },
    { 0xc6b0b1c, 48 },
    { 0x9d8b0b1d, 100 },
    { 0x8aeb0b1c, 19 },
    { 0x47fb0b1d, 61 },
    { 0xb48b0b1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9b0b25, 33 },
    { 0xc08b0b25, 98 },
    { 0, 0 },
    { 0x766b0b28, 43 },
    { 0, 0 },
    { 0xa4eb0b2a, 98 },
    { 0xabbb0b2b, 90 },
    { 0, 0 },
    { 0xe0fb0b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4b0b30, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66b0b3b, 23 },
    { 0x3f3b0b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4b0b41, 52 },
    { 0, 0 },
    { 0x8b6b0b43, 61 },
    { 0x578b0b43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387b0b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aab0b4f, 67 },
    { 0, 0 },
    { 0xdb1b0b51, 23 },
    { 0, 0 },
    { 0x262b0b53, 61 },
    { 0x183b0b54, 61 },
    { 0, 0 },
    { 0x756b0b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73db0b5b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31b0b60, 87 },
    { 0x20db0b61, 90 },
    { 0xdb2b0b62, 55 },
    { 0xc36b0b63, 76 },
    { 0x9c9b0b64, 34 },
    { 0, 0 },
    { 0x503b0b66, 23 },
    { 0x9c7b0b67, 78 },
    { 0x6f1b0b67, 23 },
    { 0xf83b0b66, 88 },
    { 0x494b0b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eb0b6d, 98 },
    { 0xe2b0b6d, 67 },
    { 0, 0 },
    { 0xadbb0b70, 98 },
    { 0x119b0b71, 78 },
    { 0xbc1b0b70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619b0b78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb0b7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eb0b80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4b0b85, 36 },
    { 0, 0 },
    { 0xc1eb0b87, 33 },
    { 0xba3b0b88, 40 },
    { 0x738b0b89, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ab0b8c, 98 },
    { 0x84cb0b8d, 23 },
    { 0, 0 },
    { 0x712b0b8f, 43 },
    { 0, 0 },
    { 0xa6db0b91, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46b0b98, 61 },
    { 0, 0 },
    { 0x1d6b0b9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb0b9d, 76 },
    { 0xab6b0b9e, 90 },
    { 0x867b0b9f, 23 },
    { 0, 0 },
    { 0x241b0ba1, 95 },
    { 0x9c6b0ba1, 76 },
    { 0, 0 },
    { 0x6aeb0ba4, 49 },
    { 0xd74b0ba4, 33 },
    { 0x7e8b0ba6, 98 },
    { 0x92cb0ba5, 61 },
    { 0, 0 },
    { 0x6c4b0ba9, 61 },
    { 0x423b0baa, 65 },
    { 0xbb7b0baa, 23 },
    { 0x9f1b0bac, 34 },
    { 0x9fdb0ba9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0b0bb1, 90 },
    { 0x271b0bb2, 76 },
    { 0x87b0bb3, 52 },
    { 0x16b0bb1, 43 },
    { 0, 0 },
    { 0xe77b0bb6, 34 },
    { 0x6bcb0bb6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedb0bba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50b0bc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745b0bc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b0bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b0bcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95b0bd0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaeb0bd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b0bdc, 95 },
    { 0x81fb0bdd, 75 },
    { 0x475b0bde, 17 },
    { 0, 0 },
    { 0xabab0be0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6b0be3, 43 },
    { 0x73db0be3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b0bea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223b0bf1, 23 },
    { 0x5d3b0bf2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x932b0bf5, 91 },
    { 0, 0 },
    { 0xd45b0bf7, 66 },
    { 0, 0 },
    { 0x976b0bf9, 46 },
    { 0xe65b0bfa, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bb0bfd, 17 },
    { 0, 0 },
    { 0x16cb0bff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb0c02, 95 },
    { 0, 0 },
    { 0x55fb0c04, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462b0c0b, 33 },
    { 0, 0 },
    { 0x4dcb0c0d, 93 },
    { 0x825b0c0e, 66 },
    { 0xff5b0c0f, 85 },
    { 0x529b0c0f, 55 },
    { 0, 0 },
    { 0x24fb0c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b0c18, 90 },
    { 0x3cfb0c18, 75 },
    { 0x804b0c1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eb0c21, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fb0c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbb0c35, 61 },
    { 0x79b0c35, 34 },
    { 0, 0 },
    { 0xe57b0c38, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb0c41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7b0c48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db0c4d, 78 },
    { 0xae7b0c4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46b0c51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526b0c56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b0c5c, 87 },
    { 0, 0 },
    { 0xe07b0c5e, 61 },
    { 0, 0 },
    { 0x55bb0c60, 33 },
    { 0, 0 },
    { 0xf68b0c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82db0c66, 23 },
    { 0, 0 },
    { 0xfa9b0c68, 61 },
    { 0, 0 },
    { 0x1e3b0c6a, 76 },
    { 0x63bb0c6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbb0c70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89b0c74, 95 },
    { 0, 0 },
    { 0x8a1b0c76, 55 },
    { 0xb3ab0c77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceb0c7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fb0c82, 67 },
    { 0, 0 },
    { 0xa79b0c84, 90 },
    { 0x60ab0c84, 64 },
    { 0x76bb0c86, 38 },
    { 0, 0 },
    { 0x141b0c88, 90 },
    { 0xf0b0c89, 61 },
    { 0xfd6b0c8a, 91 },
    { 0x2beb0c8b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5b0c90, 43 },
    { 0x53db0c91, 68 },
    { 0xd52b0c92, 84 },
    { 0, 0 },
    { 0xc0bb0c94, 52 },
    { 0x79fb0c95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553b0c99, 61 },
    { 0xfe1b0c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913b0ca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9b0ca4, 34 },
    { 0, 0 },
    { 0xd3bb0ca6, 66 },
    { 0, 0 },
    { 0x8db0ca8, 33 },
    { 0x51b0ca9, 36 },
    { 0x8db0caa, 61 },
    { 0xa54b0caa, 46 },
    { 0, 0 },
    { 0x4feb0cad, 61 },
    { 0xfc6b0cae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffb0cb4, 95 },
    { 0, 0 },
    { 0x9f6b0cb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ab0cbb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8b0cbf, 99 },
    { 0x70eb0cc0, 76 },
    { 0, 0 },
    { 0x857b0cc2, 33 },
    { 0x87b0cc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x579b0cc6, 33 },
    { 0xe2bb0cc7, 61 },
    { 0, 0 },
    { 0xbebb0cc9, 55 },
    { 0x1b9b0cca, 34 },
    { 0x50bb0ccb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ab0cd1, 23 },
    { 0x12eb0cd2, 61 },
    { 0x470b0cd3, 90 },
    { 0x922b0cd3, 99 },
    { 0xbebb0cd5, 76 },
    { 0, 0 },
    { 0x25eb0cd7, 23 },
    { 0x73cb0cd8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27b0cdc, 99 },
    { 0x8c9b0cdd, 98 },
    { 0xc99b0cde, 17 },
    { 0, 0 },
    { 0xcd5b0ce0, 55 },
    { 0x60ab0ce1, 33 },
    { 0xf5bb0ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72b0ce7, 23 },
    { 0x30bb0ce8, 33 },
    { 0x26db0ce8, 55 },
    { 0, 0 },
    { 0xfa8b0ceb, 33 },
    { 0x259b0ceb, 34 },
    { 0xd16b0cec, 77 },
    { 0xc1bb0cee, 95 },
    { 0, 0 },
    { 0x4c8b0cf0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1b0cf3, 90 },
    { 0xfc7b0cf3, 61 },
    { 0xbdb0cf5, 61 },
    { 0xb09b0cf5, 14 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fb0cf9, 33 },
    { 0xe46b0cf9, 90 },
    { 0xf1cb0cfa, 61 },
    { 0xc1bb0cfb, 76 },
    { 0x697b0cfd, 34 },
    { 0xef5b0cfe, 95 },
    { 0xa2bb0cfe, 99 },
    { 0, 0 },
    { 0x1b4b0d01, 90 },
    { 0xf66b0d02, 93 },
    { 0, 0 },
    { 0x86eb0d04, 67 },
    { 0x841b0d05, 98 },
    { 0xf21b0d06, 95 },
    { 0x750b0d07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5b0d0b, 61 },
    { 0, 0 },
    { 0x28db0d0d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30b0d10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eb0d13, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75b0d1a, 61 },
    { 0x7c8b0d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4db0d1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1b0d22, 84 },
    { 0, 0 },
    { 0x25db0d24, 43 },
    { 0, 0 },
    { 0xf2eb0d26, 90 },
    { 0x527b0d27, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2b0d2a, 76 },
    { 0x161b0d2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fb0d2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348b0d37, 98 },
    { 0xfa5b0d38, 90 },
    { 0xd38b0d39, 61 },
    { 0, 0 },
    { 0x6aeb0d3b, 95 },
    { 0, 0 },
    { 0x466b0d3d, 95 },
    { 0xdf3b0d3d, 98 },
    { 0xb2cb0d3f, 90 },
    { 0x93ab0d40, 38 },
    { 0xba3b0d41, 40 },
    { 0x40db0d3f, 75 },
    { 0x23b0d3d, 66 },
    { 0x1b0d44, 55 },
    { 0xf79b0d3d, 95 },
    { 0xeeeb0d3d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcb0d4a, 87 },
    { 0, 0 },
    { 0x685b0d4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51b0d52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43b0d57, 17 },
    { 0x780b0d58, 33 },
    { 0x712b0d58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab0d5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41b0d61, 98 },
    { 0, 0 },
    { 0x55fb0d63, 33 },
    { 0, 0 },
    { 0xffb0d65, 61 },
    { 0x3efb0d65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacb0d69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5b0d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839b0d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3b0d7d, 93 },
    { 0xf45b0d7d, 87 },
    { 0, 0 },
    { 0xa89b0d80, 90 },
    { 0x575b0d80, 91 },
    { 0, 0 },
    { 0xf56b0d83, 98 },
    { 0, 0 },
    { 0xc6cb0d85, 52 },
    { 0, 0 },
    { 0x58ab0d87, 90 },
    { 0x512b0d88, 55 },
    { 0x15ab0d89, 88 },
    { 0, 0 },
    { 0x7c1b0d8b, 91 },
    { 0x4d3b0d8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598b0d91, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19b0d94, 76 },
    { 0x377b0d95, 34 },
    { 0, 0 },
    { 0xe2db0d97, 23 },
    { 0, 0 },
    { 0x52ab0d99, 95 },
    { 0, 0 },
    { 0x864b0d9b, 90 },
    { 0, 0 },
    { 0xf66b0d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521b0da2, 33 },
    { 0xffcb0da3, 55 },
    { 0, 0 },
    { 0x3dfb0da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b0daa, 84 },
    { 0, 0 },
    { 0xbbcb0dac, 90 },
    { 0xb3cb0dad, 90 },
    { 0xd60b0dac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ab0db2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169b0db8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bb0dbd, 66 },
    { 0xe30b0dbe, 33 },
    { 0xd30b0dbf, 61 },
    { 0x1c3b0dc0, 95 },
    { 0, 0 },
    { 0x67eb0dc2, 33 },
    { 0x968b0dc3, 90 },
    { 0, 0 },
    { 0xddeb0dc5, 78 },
    { 0x5fcb0dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab0dc9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b0dd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddb0dda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b0dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x618b0de1, 33 },
    { 0, 0 },
    { 0x6feb0de3, 93 },
    { 0, 0 },
    { 0xed1b0de5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eb0de9, 98 },
    { 0x46eb0dea, 98 },
    { 0x26cb0dea, 33 },
    { 0x566b0dea, 55 },
    { 0xb7fb0deb, 95 },
    { 0x3c7b0dee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eb0df1, 33 },
    { 0, 0 },
    { 0x310b0df3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b0df6, 67 },
    { 0x985b0df7, 48 },
    { 0x7cdb0df8, 98 },
    { 0x691b0df9, 91 },
    { 0x2bb0df7, 91 },
    { 0xa8b0df7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b0e01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ab0e04, 78 },
    { 0xd8fb0e04, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4b0e0a, 78 },
    { 0xc15b0e0b, 90 },
    { 0x892b0e0b, 23 },
    { 0x22bb0e0a, 95 },
    { 0x327b0e0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4feb0e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb0e16, 34 },
    { 0x5a8b0e16, 61 },
    { 0xce3b0e17, 90 },
    { 0xa16b0e18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276b0e1e, 90 },
    { 0, 0 },
    { 0x9abb0e20, 33 },
    { 0x47db0e20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bb0e24, 61 },
    { 0, 0 },
    { 0xf8b0e26, 100 },
    { 0x507b0e27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3b0e2a, 91 },
    { 0xca1b0e2b, 93 },
    { 0x957b0e2c, 61 },
    { 0x4db0e2c, 23 },
    { 0x9e9b0e2e, 75 },
    { 0, 0 },
    { 0x2d4b0e30, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab0e34, 39 },
    { 0x9b1b0e34, 33 },
    { 0xbc6b0e36, 55 },
    { 0xf40b0e36, 98 },
    { 0x4abb0e38, 46 },
    { 0xf78b0e38, 100 },
    { 0xe1db0e36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4b0e3e, 46 },
    { 0x110b0e3f, 34 },
    { 0x609b0e40, 87 },
    { 0x28b0e41, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834b0e46, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434b0e51, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67b0e61, 46 },
    { 0, 0 },
    { 0x4f2b0e63, 66 },
    { 0x307b0e64, 81 },
    { 0x793b0e65, 76 },
    { 0x9feb0e63, 23 },
    { 0x8f3b0e65, 75 },
    { 0, 0 },
    { 0xfd1b0e69, 43 },
    { 0, 0 },
    { 0xd19b0e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5b0e6e, 48 },
    { 0, 0 },
    { 0xcfcb0e70, 99 },
    { 0xe7bb0e70, 95 },
    { 0x3abb0e72, 66 },
    { 0x783b0e70, 98 },
    { 0xd10b0e74, 33 },
    { 0xae5b0e75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb0e79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28b0e7d, 61 },
    { 0x584b0e7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00b0e81, 90 },
    { 0x3adb0e82, 42 },
    { 0xcdb0e82, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebb0e86, 43 },
    { 0, 0 },
    { 0x7eab0e88, 33 },
    { 0, 0 },
    { 0xf59b0e8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb0e92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x922b0e95, 81 },
    { 0xe35b0e95, 98 },
    { 0, 0 },
    { 0x2b0b0e98, 91 },
    { 0, 0 },
    { 0x87b0e9a, 46 },
    { 0x9c6b0e9b, 98 },
    { 0xd36b0e9c, 98 },
    { 0xff9b0e9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b0ea1, 90 },
    { 0x7a4b0ea2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cb0ea5, 33 },
    { 0x386b0ea5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6b0ea9, 33 },
    { 0x8d6b0eaa, 76 },
    { 0x793b0eab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbb0eaf, 33 },
    { 0x6b3b0eb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x303b0eb3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b0eb9, 33 },
    { 0, 0 },
    { 0x230b0ebb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97b0ec0, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66b0ec4, 99 },
    { 0x6ccb0ec5, 61 },
    { 0, 0 },
    { 0xe7db0ec7, 46 },
    { 0x764b0ec7, 46 },
    { 0x3e4b0ec9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x199b0ecc, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945b0ed5, 26 },
    { 0xbf5b0ed6, 98 },
    { 0, 0 },
    { 0x64db0ed8, 81 },
    { 0, 0 },
    { 0x980b0eda, 66 },
    { 0, 0 },
    { 0x7e1b0edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bb0ee2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0b0eea, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ab0eee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4b0ef4, 95 },
    { 0x34bb0ef5, 95 },
    { 0xf5b0ef6, 91 },
    { 0xc7db0ef5, 40 },
    { 0xcbab0ef8, 61 },
    { 0, 0 },
    { 0x16fb0efa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b0efe, 61 },
    { 0x484b0efe, 90 },
    { 0, 0 },
    { 0x347b0f01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234b0f05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23db0f0b, 23 },
    { 0xbe9b0f0b, 43 },
    { 0x603b0f0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b0f12, 76 },
    { 0x79bb0f12, 33 },
    { 0x93eb0f13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4b0f19, 90 },
    { 0x53ab0f1a, 61 },
    { 0x5ebb0f1b, 93 },
    { 0xa1bb0f1b, 98 },
    { 0x1eb0f1d, 81 },
    { 0x928b0f1e, 66 },
    { 0, 0 },
    { 0xe91b0f20, 90 },
    { 0, 0 },
    { 0x95fb0f22, 88 },
    { 0x54cb0f22, 33 },
    { 0xa2cb0f22, 46 },
    { 0xb8cb0f23, 95 },
    { 0xd1fb0f24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fb0f2d, 95 },
    { 0, 0 },
    { 0xdd3b0f2f, 17 },
    { 0x262b0f30, 50 },
    { 0x8e5b0f31, 87 },
    { 0x7a1b0f2f, 46 },
    { 0, 0 },
    { 0x344b0f34, 46 },
    { 0xb8bb0f35, 65 },
    { 0xa43b0f36, 95 },
    { 0x32eb0f35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab0f3a, 95 },
    { 0xd93b0f3b, 33 },
    { 0x954b0f3c, 33 },
    { 0x920b0f3c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4b0f40, 23 },
    { 0xa29b0f41, 100 },
    { 0, 0 },
    { 0x46db0f43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b0f4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cb0f4e, 33 },
    { 0xaa0b0f4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eb0f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4b0f56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x766b0f59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b0f5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b0f61, 76 },
    { 0xcd4b0f62, 98 },
    { 0x645b0f63, 95 },
    { 0, 0 },
    { 0xd0ab0f65, 95 },
    { 0xaa5b0f66, 77 },
    { 0x843b0f66, 61 },
    { 0xaeab0f68, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1b0f6b, 98 },
    { 0x86bb0f6c, 80 },
    { 0xe7bb0f6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0b0f72, 61 },
    { 0xe20b0f72, 90 },
    { 0xeafb0f72, 23 },
    { 0x2b0b0f75, 95 },
    { 0xd82b0f75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cb0f7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213b0f81, 91 },
    { 0x730b0f81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fb0f86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb0f8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9b0f91, 88 },
    { 0xb9cb0f92, 33 },
    { 0, 0 },
    { 0x3a1b0f94, 37 },
    { 0x985b0f95, 87 },
    { 0, 0 },
    { 0xe5b0f97, 65 },
    { 0xa45b0f98, 33 },
    { 0, 0 },
    { 0xf50b0f9a, 33 },
    { 0x2d2b0f9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfab0fa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cb0fa4, 90 },
    { 0xa0bb0fa4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533b0fb2, 55 },
    { 0, 0 },
    { 0xc41b0fb4, 55 },
    { 0, 0 },
    { 0xa6bb0fb6, 34 },
    { 0xe35b0fb7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcb0fba, 98 },
    { 0, 0 },
    { 0x573b0fbc, 90 },
    { 0xf02b0fbd, 53 },
    { 0xaa6b0fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b0fc1, 33 },
    { 0xc5cb0fc2, 33 },
    { 0xc11b0fc2, 68 },
    { 0, 0 },
    { 0x133b0fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x981b0fc8, 78 },
    { 0x88b0fc9, 52 },
    { 0xca8b0fca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x18b0fcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94b0fd0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefb0fd5, 90 },
    { 0, 0 },
    { 0x444b0fd7, 33 },
    { 0xbf8b0fd7, 95 },
    { 0xd3ab0fd9, 61 },
    { 0xcbbb0fd9, 90 },
    { 0xb62b0fdb, 90 },
    { 0, 0 },
    { 0xc1bb0fdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fb0fe0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdb0fec, 90 },
    { 0xdf8b0fed, 42 },
    { 0xda2b0fee, 91 },
    { 0x78eb0fef, 23 },
    { 0x22db0ff0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b0ff4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68db0ffc, 88 },
    { 0xbdbb0ffd, 95 },
    { 0, 0 },
    { 0x77eb0fff, 90 },
    { 0x19eb1000, 61 },
    { 0x28bb1000, 87 },
    { 0x9aab0fff, 95 },
    { 0xa77b1000, 34 },
    { 0, 0 },
    { 0x525b1005, 90 },
    { 0xcd3b1006, 95 },
    { 0, 0 },
    { 0x2e8b1008, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aab100b, 97 },
    { 0xc27b100c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbb1010, 65 },
    { 0, 0 },
    { 0xb0b1012, 61 },
    { 0, 0 },
    { 0xa48b1014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b1018, 69 },
    { 0x348b1019, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ab1022, 88 },
    { 0x238b1022, 98 },
    { 0xb4fb1024, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cb1027, 91 },
    { 0xebfb1028, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b102b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x883b102e, 98 },
    { 0x977b102f, 61 },
    { 0xd7b102f, 95 },
    { 0x6ecb1030, 52 },
    { 0x856b102f, 61 },
    { 0, 0 },
    { 0xb14b1034, 33 },
    { 0xbedb1035, 37 },
    { 0x69cb1036, 91 },
    { 0x57cb1036, 43 },
    { 0xd23b1038, 33 },
    { 0xccbb1038, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83b103d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4b1042, 61 },
    { 0x3afb1042, 34 },
    { 0, 0 },
    { 0x7b0b1045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455b1049, 90 },
    { 0x1e2b104a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x135b104d, 69 },
    { 0xa97b104e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb1056, 90 },
    { 0, 0 },
    { 0x37eb1058, 99 },
    { 0x72ab1059, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9b105c, 66 },
    { 0x959b105d, 88 },
    { 0, 0 },
    { 0xd05b105f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aab1064, 100 },
    { 0, 0 },
    { 0xca2b1066, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18b1069, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b106f, 46 },
    { 0x87bb106f, 90 },
    { 0, 0 },
    { 0x781b1072, 90 },
    { 0x38b1073, 90 },
    { 0x94fb1074, 33 },
    { 0x318b1075, 23 },
    { 0x5cdb1076, 100 },
    { 0x56bb1077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fb107d, 55 },
    { 0x51cb107e, 95 },
    { 0xbe5b107f, 61 },
    { 0xe50b1080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fb1085, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacb1089, 76 },
    { 0, 0 },
    { 0x17cb108b, 75 },
    { 0x343b108c, 98 },
    { 0x148b108b, 80 },
    { 0x632b108d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b1096, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b109d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90b10a2, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23b10a9, 95 },
    { 0x3bdb10a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bb10ae, 90 },
    { 0x896b10af, 98 },
    { 0, 0 },
    { 0x9c4b10b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3b10b4, 55 },
    { 0x301b10b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9b10be, 23 },
    { 0xdecb10bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eb10c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786b10c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b10c9, 61 },
    { 0, 0 },
    { 0x203b10cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b10ce, 90 },
    { 0xf1eb10cf, 95 },
    { 0xc3cb10ce, 98 },
    { 0x705b10d1, 43 },
    { 0xa7fb10d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ab10d6, 90 },
    { 0, 0 },
    { 0x15bb10d8, 55 },
    { 0xa60b10d9, 23 },
    { 0xce3b10da, 91 },
    { 0xa2eb10da, 33 },
    { 0x4edb10dc, 90 },
    { 0x439b10db, 93 },
    { 0x11ab10dc, 67 },
    { 0x2c8b10de, 33 },
    { 0xb68b10e0, 87 },
    { 0, 0 },
    { 0x68eb10e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145b10ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834b10f0, 34 },
    { 0x94ab10f1, 33 },
    { 0x62bb10f1, 61 },
    { 0xd50b10f2, 33 },
    { 0x1dfb10f4, 95 },
    { 0xba7b10f1, 33 },
    { 0x677b10f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fb10fd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b1101, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3eb1106, 98 },
    { 0x3ecb1106, 66 },
    { 0, 0 },
    { 0x96ab1109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x114b110c, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536b1114, 98 },
    { 0xa3cb1115, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b1118, 33 },
    { 0x4cbb1119, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x266b111c, 93 },
    { 0x772b111d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x868b1120, 33 },
    { 0x425b1120, 91 },
    { 0x5a9b1122, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b1125, 61 },
    { 0, 0 },
    { 0xbc7b1127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8b112e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb1132, 99 },
    { 0xbf5b1133, 61 },
    { 0xca6b1132, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382b113e, 100 },
    { 0x6d1b113f, 33 },
    { 0xcacb113e, 33 },
    { 0x609b113f, 55 },
    { 0xcf4b113e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9b1145, 61 },
    { 0x67eb1146, 55 },
    { 0xfa0b1146, 66 },
    { 0xa62b1148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afb114c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb1152, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70b1156, 87 },
    { 0, 0 },
    { 0x16ab1158, 95 },
    { 0xde9b1159, 61 },
    { 0x621b1158, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0b115f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7b1163, 90 },
    { 0xb1ab1163, 76 },
    { 0x442b1164, 34 },
    { 0, 0 },
    { 0x44cb1167, 61 },
    { 0, 0 },
    { 0xaf4b1169, 23 },
    { 0x652b1169, 95 },
    { 0x6fbb116b, 76 },
    { 0, 0 },
    { 0x740b116d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69b1174, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ab117a, 67 },
    { 0, 0 },
    { 0x332b117c, 90 },
    { 0, 0 },
    { 0xaacb117e, 34 },
    { 0, 0 },
    { 0x6deb1180, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b1183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857b118c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181b1191, 75 },
    { 0xdb7b1192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab1198, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfb119c, 80 },
    { 0x50db119d, 68 },
    { 0x834b119e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb11a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6b11a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ab11ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74eb11b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb11b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362b11b9, 84 },
    { 0xd38b11ba, 33 },
    { 0, 0 },
    { 0x10fb11bc, 33 },
    { 0x726b11bd, 98 },
    { 0x314b11bc, 93 },
    { 0xa16b11be, 66 },
    { 0xac7b11c0, 55 },
    { 0, 0 },
    { 0x7b8b11c2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x24db11c5, 93 },
    { 0, 0 },
    { 0x8bab11c7, 98 },
    { 0x677b11c7, 93 },
    { 0, 0 },
    { 0x3eb11ca, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352b11ce, 33 },
    { 0xc34b11ce, 76 },
    { 0x248b11d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32b11d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9b11da, 68 },
    { 0, 0 },
    { 0xec7b11dc, 33 },
    { 0xbd8b11dd, 61 },
    { 0, 0 },
    { 0xa89b11df, 33 },
    { 0x123b11e0, 95 },
    { 0xd4b11df, 61 },
    { 0x141b11e2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecb11e9, 90 },
    { 0x41db11e9, 33 },
    { 0, 0 },
    { 0x680b11ec, 90 },
    { 0xd8eb11ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x660b11f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0b11f5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73b11f9, 98 },
    { 0xf10b11fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401b1200, 76 },
    { 0x421b1201, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x660b1204, 46 },
    { 0x6f4b1205, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b120b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b1213, 61 },
    { 0x66db1213, 96 },
    { 0, 0 },
    { 0x21b1216, 23 },
    { 0, 0 },
    { 0xd3fb1218, 55 },
    { 0xd99b1219, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06b1223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231b1229, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672b1232, 33 },
    { 0x41cb1233, 76 },
    { 0, 0 },
    { 0x997b1235, 33 },
    { 0, 0 },
    { 0xdaab1237, 98 },
    { 0x25eb1238, 80 },
    { 0x7eab1238, 93 },
    { 0, 0 },
    { 0x61eb123b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb123e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdab1241, 17 },
    { 0x8a6b1242, 46 },
    { 0x142b1243, 78 },
    { 0xb58b1243, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x431b1247, 75 },
    { 0x37eb1248, 87 },
    { 0x43db1248, 33 },
    { 0xd36b1247, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x522b124d, 23 },
    { 0x36eb124e, 33 },
    { 0xc60b124f, 33 },
    { 0x539b1250, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34b1257, 61 },
    { 0x56fb1258, 98 },
    { 0, 0 },
    { 0x112b125a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb125d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4b1260, 55 },
    { 0x7fb1261, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38b1265, 55 },
    { 0xb01b1266, 95 },
    { 0x5fbb1267, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b1275, 23 },
    { 0x3bb1276, 61 },
    { 0x9b1275, 90 },
    { 0, 0 },
    { 0x4bdb1279, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb127c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x555b127f, 90 },
    { 0x13bb127f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784b1286, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcb128a, 98 },
    { 0x6deb128a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5b1296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35b129a, 67 },
    { 0xb97b129b, 99 },
    { 0x83cb129c, 80 },
    { 0x737b129c, 84 },
    { 0x304b129c, 85 },
    { 0xb5cb129f, 90 },
    { 0x517b12a0, 90 },
    { 0xf1fb12a1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadb12a8, 61 },
    { 0, 0 },
    { 0x860b12aa, 93 },
    { 0x4b12ab, 61 },
    { 0xdfeb12ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b12b2, 55 },
    { 0, 0 },
    { 0xd13b12b4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60b12b9, 97 },
    { 0xa70b12ba, 95 },
    { 0xf73b12bb, 99 },
    { 0, 0 },
    { 0x778b12bd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b12c1, 34 },
    { 0, 0 },
    { 0x392b12c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ab12c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7b12cc, 61 },
    { 0, 0 },
    { 0xc0b12ce, 55 },
    { 0xa24b12cf, 55 },
    { 0x807b12cf, 95 },
    { 0, 0 },
    { 0xd1bb12d2, 100 },
    { 0xc38b12d2, 24 },
    { 0xad1b12d4, 33 },
    { 0x675b12d5, 61 },
    { 0xedfb12d6, 33 },
    { 0xbb7b12d6, 99 },
    { 0xcdfb12d8, 43 },
    { 0xb52b12d9, 77 },
    { 0, 0 },
    { 0xe0cb12db, 95 },
    { 0x6ceb12dc, 99 },
    { 0x44fb12dd, 66 },
    { 0xa52b12de, 66 },
    { 0xf1ab12df, 81 },
    { 0x9afb12e0, 61 },
    { 0xb02b12de, 90 },
    { 0, 0 },
    { 0x1a2b12e3, 61 },
    { 0, 0 },
    { 0xe1bb12e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b12ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95b12ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceb12f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb12f7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3b12fb, 77 },
    { 0xc42b12fc, 51 },
    { 0x48b12fb, 33 },
    { 0, 0 },
    { 0x143b12ff, 76 },
    { 0, 0 },
    { 0x29bb1301, 90 },
    { 0x83cb1301, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23b1309, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb1312, 68 },
    { 0, 0 },
    { 0xe0ab1314, 33 },
    { 0xa55b1314, 98 },
    { 0, 0 },
    { 0xe13b1317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceb1322, 90 },
    { 0x5d7b1323, 23 },
    { 0x7b5b1324, 61 },
    { 0x839b1325, 98 },
    { 0x6ceb1326, 76 },
    { 0x4db1327, 88 },
    { 0x8c8b1327, 87 },
    { 0xc72b1327, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb132c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6b1332, 91 },
    { 0, 0 },
    { 0xf7bb1334, 90 },
    { 0x887b1335, 95 },
    { 0, 0 },
    { 0x9dfb1337, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb133c, 76 },
    { 0xae3b133d, 76 },
    { 0x8e4b133e, 33 },
    { 0xedfb133f, 88 },
    { 0, 0 },
    { 0x69bb1341, 84 },
    { 0x702b1341, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecb1347, 34 },
    { 0x353b1348, 43 },
    { 0x9cfb1349, 90 },
    { 0xcb0b134a, 90 },
    { 0xbccb134a, 90 },
    { 0xa47b134a, 90 },
    { 0x7eb134d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb1351, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9b1356, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cb135a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9eb1360, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb1365, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb1368, 90 },
    { 0, 0 },
    { 0xbfbb136a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acb136d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92b1370, 99 },
    { 0x237b1371, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04b137c, 33 },
    { 0x9dfb137c, 23 },
    { 0x549b137e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1b1382, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5b1387, 76 },
    { 0xa90b1388, 33 },
    { 0xffeb1388, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649b138e, 34 },
    { 0, 0 },
    { 0x6edb1390, 33 },
    { 0xda5b1391, 100 },
    { 0xd72b1392, 55 },
    { 0x402b1391, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4b1396, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9b139a, 33 },
    { 0, 0 },
    { 0x7d3b139c, 76 },
    { 0x23db139d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71b13a8, 67 },
    { 0, 0 },
    { 0xa27b13aa, 33 },
    { 0x10eb13aa, 52 },
    { 0x3ceb13ab, 93 },
    { 0xc16b13ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x535b13b0, 93 },
    { 0x6eab13b1, 99 },
    { 0x2b5b13b2, 75 },
    { 0, 0 },
    { 0x38b13b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94b13b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b13ba, 34 },
    { 0xc2cb13bb, 90 },
    { 0x9c6b13bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603b13c0, 76 },
    { 0x11cb13c1, 23 },
    { 0, 0 },
    { 0x36cb13c3, 55 },
    { 0xc0cb13c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab13c7, 19 },
    { 0x8deb13c8, 12 },
    { 0xce3b13c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820b13d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706b13d7, 99 },
    { 0, 0 },
    { 0xae0b13d9, 33 },
    { 0xce3b13da, 98 },
    { 0, 0 },
    { 0xe1cb13dc, 61 },
    { 0, 0 },
    { 0x418b13de, 99 },
    { 0xd3bb13df, 33 },
    { 0x117b13e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18b13e8, 33 },
    { 0xa4fb13e9, 61 },
    { 0xc1bb13ea, 61 },
    { 0x8afb13eb, 91 },
    { 0x7c9b13ea, 46 },
    { 0x4cdb13ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b13f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6b13fb, 34 },
    { 0x50bb13fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3b1402, 90 },
    { 0, 0 },
    { 0x187b1404, 55 },
    { 0, 0 },
    { 0x6b0b1406, 90 },
    { 0, 0 },
    { 0x54fb1408, 90 },
    { 0xb62b1408, 90 },
    { 0xea3b140a, 61 },
    { 0x68ab140a, 69 },
    { 0xe2fb140c, 46 },
    { 0xe82b140a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fb1413, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdab1419, 100 },
    { 0x7f9b141a, 84 },
    { 0x962b141b, 68 },
    { 0xb43b1419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b1422, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb1426, 66 },
    { 0xa16b1427, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02b142d, 95 },
    { 0x85db142d, 52 },
    { 0x7ab142e, 90 },
    { 0xe71b1430, 33 },
    { 0, 0 },
    { 0x846b1432, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87b1435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31b1438, 95 },
    { 0x6d0b1439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ab143e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b1447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15b144e, 95 },
    { 0xee0b144f, 93 },
    { 0, 0 },
    { 0xb5fb1451, 95 },
    { 0xfcbb1452, 52 },
    { 0, 0 },
    { 0xeb6b1454, 33 },
    { 0xc05b1455, 90 },
    { 0x6eb1456, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bb145d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb1460, 93 },
    { 0xb79b1460, 96 },
    { 0xd3db1462, 76 },
    { 0xb0bb1462, 95 },
    { 0x2f2b1462, 61 },
    { 0x5a7b1465, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb146e, 33 },
    { 0xd00b146f, 33 },
    { 0x2b0b1470, 90 },
    { 0x7c4b1470, 90 },
    { 0x9c6b1472, 90 },
    { 0x16eb1473, 95 },
    { 0, 0 },
    { 0xf89b1475, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7b1478, 88 },
    { 0x267b1478, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dab147d, 34 },
    { 0, 0 },
    { 0xee7b147f, 61 },
    { 0xb5eb1480, 61 },
    { 0x903b1480, 95 },
    { 0xb80b1480, 61 },
    { 0x9bdb1481, 90 },
    { 0x46db1484, 90 },
    { 0xc74b1484, 55 },
    { 0x47cb1486, 33 },
    { 0x37b1487, 85 },
    { 0x3f3b1487, 61 },
    { 0x46bb1487, 98 },
    { 0, 0 },
    { 0xc3fb148b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ab148e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5b1493, 95 },
    { 0, 0 },
    { 0x960b1495, 55 },
    { 0, 0 },
    { 0x56ab1497, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321b149e, 95 },
    { 0x23cb149f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb14a2, 98 },
    { 0xe42b14a2, 23 },
    { 0, 0 },
    { 0xb92b14a5, 95 },
    { 0x3a1b14a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176b14aa, 34 },
    { 0xda9b14ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b14b3, 15 },
    { 0xc29b14b3, 98 },
    { 0, 0 },
    { 0xcdeb14b6, 81 },
    { 0x4f6b14b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b14bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb14c3, 34 },
    { 0x542b14c4, 55 },
    { 0xc3bb14c4, 98 },
    { 0x9f3b14c6, 66 },
    { 0x655b14c7, 95 },
    { 0x92bb14c8, 95 },
    { 0x534b14c8, 94 },
    { 0x39fb14ca, 100 },
    { 0xc5b14cb, 43 },
    { 0xf91b14c5, 33 },
    { 0, 0 },
    { 0xecab14ce, 23 },
    { 0x8b2b14cf, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4b14d3, 33 },
    { 0xbfab14d4, 76 },
    { 0x707b14d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4b14d9, 98 },
    { 0, 0 },
    { 0xe09b14db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b14df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cb14e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82b14e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb14f8, 98 },
    { 0, 0 },
    { 0xdc6b14fa, 90 },
    { 0x177b14fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05b1502, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648b150b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fb150e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbb1515, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0b1518, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fb1521, 95 },
    { 0xe4fb1522, 61 },
    { 0x9d6b1523, 85 },
    { 0xd51b1521, 90 },
    { 0xde7b1525, 61 },
    { 0, 0 },
    { 0xcb9b1527, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13db152a, 78 },
    { 0, 0 },
    { 0xa49b152c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ab1533, 90 },
    { 0, 0 },
    { 0x78eb1535, 33 },
    { 0xbf9b1536, 46 },
    { 0, 0 },
    { 0x558b1538, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eeb153b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721b153f, 95 },
    { 0x332b1540, 98 },
    { 0, 0 },
    { 0x675b1542, 95 },
    { 0x475b1543, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29b1548, 83 },
    { 0x193b1549, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5db154e, 23 },
    { 0, 0 },
    { 0xed4b1550, 77 },
    { 0, 0 },
    { 0x1cdb1552, 52 },
    { 0xdd5b1552, 61 },
    { 0, 0 },
    { 0x912b1555, 96 },
    { 0xbefb1556, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fb1559, 78 },
    { 0xfe2b1559, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x356b155d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553b1561, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0b1564, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388b1568, 33 },
    { 0, 0 },
    { 0xbbeb156a, 95 },
    { 0x61db156b, 76 },
    { 0xf2bb156c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703b1570, 76 },
    { 0xb12b1570, 46 },
    { 0, 0 },
    { 0xa92b1573, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cab1576, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ab1579, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cb1580, 23 },
    { 0x868b1581, 33 },
    { 0xa38b1582, 95 },
    { 0x302b1583, 46 },
    { 0, 0 },
    { 0x199b1585, 87 },
    { 0x1c6b1585, 52 },
    { 0x200b1586, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7b158b, 24 },
    { 0x1bcb158b, 69 },
    { 0x9b9b158c, 40 },
    { 0xee0b158e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b1593, 23 },
    { 0, 0 },
    { 0x4cb1595, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4b159b, 95 },
    { 0x83b159c, 84 },
    { 0xfb1b159c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbb15a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cb15a8, 95 },
    { 0xbdcb15a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccb15ac, 23 },
    { 0x7c2b15ac, 98 },
    { 0, 0 },
    { 0xbd1b15af, 91 },
    { 0x8cb15b0, 43 },
    { 0xa1cb15b0, 23 },
    { 0xdecb15b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37b15b8, 95 },
    { 0xaa9b15b9, 99 },
    { 0, 0 },
    { 0x6fb15bb, 90 },
    { 0xf8b15bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7b15bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedb15c2, 98 },
    { 0x638b15c2, 43 },
    { 0, 0 },
    { 0xc69b15c5, 91 },
    { 0x33bb15c6, 66 },
    { 0xce0b15c6, 34 },
    { 0, 0 },
    { 0x1cb15c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3b15ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0b15d9, 98 },
    { 0xf10b15da, 90 },
    { 0xb8fb15db, 85 },
    { 0x731b15dc, 95 },
    { 0xbbab15dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bb15e5, 75 },
    { 0, 0 },
    { 0x8e3b15e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29b15ea, 48 },
    { 0, 0 },
    { 0xbcfb15ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecb15ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0b15f2, 33 },
    { 0x4c1b15f3, 90 },
    { 0xb98b15f4, 61 },
    { 0xeceb15f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfb15f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5b15fc, 90 },
    { 0xcf5b15fc, 95 },
    { 0x47eb15fe, 55 },
    { 0, 0 },
    { 0x531b1600, 34 },
    { 0x343b1601, 84 },
    { 0x9f3b1601, 93 },
    { 0xb42b1603, 90 },
    { 0x4fcb1602, 98 },
    { 0xd95b1605, 52 },
    { 0xa00b1605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77b160e, 61 },
    { 0xa39b160f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabb1612, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658b1616, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afb1619, 98 },
    { 0, 0 },
    { 0xd68b161b, 95 },
    { 0, 0 },
    { 0x47cb161d, 52 },
    { 0x28fb161e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2b1624, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0b1628, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b162c, 66 },
    { 0, 0 },
    { 0x1dbb162e, 98 },
    { 0, 0 },
    { 0x368b1630, 78 },
    { 0, 0 },
    { 0x344b1632, 95 },
    { 0xf68b1633, 78 },
    { 0x871b1634, 84 },
    { 0xc8fb1633, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x835b1638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7b163b, 61 },
    { 0, 0 },
    { 0x609b163d, 34 },
    { 0x9dcb163e, 99 },
    { 0xbefb163e, 93 },
    { 0xb24b1640, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b1648, 88 },
    { 0xfcfb1648, 34 },
    { 0xba7b1649, 84 },
    { 0x53ab164a, 66 },
    { 0x63db164c, 95 },
    { 0x60eb164d, 40 },
    { 0, 0 },
    { 0x4e1b164f, 78 },
    { 0x7cb1650, 23 },
    { 0, 0 },
    { 0xf52b1652, 33 },
    { 0xe7eb1653, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97db1657, 80 },
    { 0x697b1657, 43 },
    { 0xf24b1658, 34 },
    { 0x6a2b1658, 95 },
    { 0, 0 },
    { 0x924b165c, 23 },
    { 0x6a6b165d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb1660, 90 },
    { 0x3ab1661, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbb1666, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b166b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264b166f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754b1674, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4b167b, 61 },
    { 0, 0 },
    { 0xb34b167d, 97 },
    { 0, 0 },
    { 0xd5fb167f, 90 },
    { 0x153b1680, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4b1683, 95 },
    { 0x2f6b1684, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991b1689, 90 },
    { 0x911b168a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eb168d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fb1695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1b1698, 93 },
    { 0x458b1699, 76 },
    { 0xe0b1698, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355b169e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3b16a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eb16a8, 33 },
    { 0x735b16a9, 52 },
    { 0x8b4b16a8, 98 },
    { 0xe20b16aa, 52 },
    { 0xc0db16ac, 100 },
    { 0x47eb16ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb16b0, 53 },
    { 0xc8db16b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933b16b6, 43 },
    { 0xee8b16b6, 23 },
    { 0, 0 },
    { 0xd9eb16b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfeb16bd, 36 },
    { 0, 0 },
    { 0x7db16bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82b16c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241b16cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82b16d4, 55 },
    { 0, 0 },
    { 0x8db16d6, 95 },
    { 0xe4db16d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b16da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bb16dd, 33 },
    { 0xf12b16de, 90 },
    { 0, 0 },
    { 0x885b16e0, 95 },
    { 0xce2b16e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b16e9, 78 },
    { 0x62db16e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1eb16ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cab16f1, 85 },
    { 0x7b16f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b16f7, 55 },
    { 0x6f8b16f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7db16ff, 61 },
    { 0x42db16ff, 75 },
    { 0x90eb1700, 95 },
    { 0x1b8b1702, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ab1705, 90 },
    { 0, 0 },
    { 0x245b1707, 99 },
    { 0, 0 },
    { 0x7edb1709, 34 },
    { 0x17cb170a, 95 },
    { 0xcfab170a, 23 },
    { 0x422b1709, 85 },
    { 0, 0 },
    { 0xc14b170e, 98 },
    { 0xd55b170f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cb1713, 23 },
    { 0xb3b1714, 75 },
    { 0x21eb1714, 90 },
    { 0x4b6b1714, 75 },
    { 0, 0 },
    { 0x329b1718, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fb1722, 91 },
    { 0xb45b1723, 98 },
    { 0, 0 },
    { 0x6d7b1725, 61 },
    { 0x30bb1726, 46 },
    { 0x133b1727, 23 },
    { 0xc5eb1727, 65 },
    { 0, 0 },
    { 0xa75b172a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb172d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43b1733, 55 },
    { 0, 0 },
    { 0xa68b1735, 46 },
    { 0x9ddb1736, 61 },
    { 0x163b1735, 98 },
    { 0xea6b1738, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629b173c, 90 },
    { 0, 0 },
    { 0xb59b173e, 90 },
    { 0x1c6b173f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beb1742, 76 },
    { 0, 0 },
    { 0x50b1744, 23 },
    { 0, 0 },
    { 0x5a3b1746, 67 },
    { 0xa71b1747, 90 },
    { 0x1a3b1748, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b174d, 90 },
    { 0x217b174d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8db1753, 61 },
    { 0x200b1754, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb1759, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71b175c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45b1762, 90 },
    { 0, 0 },
    { 0x311b1764, 81 },
    { 0x5f6b1765, 99 },
    { 0x9b9b1766, 76 },
    { 0xde2b1767, 90 },
    { 0xc39b1766, 61 },
    { 0, 0 },
    { 0xaf9b176a, 68 },
    { 0, 0 },
    { 0x79eb176c, 33 },
    { 0x1cfb176d, 51 },
    { 0xaf2b176d, 98 },
    { 0xd80b176f, 76 },
    { 0x30b176d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdb1774, 61 },
    { 0xcdab1775, 23 },
    { 0, 0 },
    { 0xd47b1777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b177c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bb177f, 52 },
    { 0xdebb1780, 34 },
    { 0x15db177f, 90 },
    { 0x30cb1780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eb1785, 52 },
    { 0x7f6b1785, 61 },
    { 0x4ccb1787, 67 },
    { 0, 0 },
    { 0xf6fb1789, 95 },
    { 0x85db178a, 93 },
    { 0x83b178b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eb1791, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbb1795, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab1799, 61 },
    { 0, 0 },
    { 0xe83b179b, 46 },
    { 0, 0 },
    { 0x8ccb179d, 61 },
    { 0xebeb179e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b17a2, 98 },
    { 0x199b17a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751b17a9, 85 },
    { 0, 0 },
    { 0xfab17ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42b17b1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eb17b9, 81 },
    { 0xcf2b17ba, 23 },
    { 0x35db17b9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b17c3, 23 },
    { 0, 0 },
    { 0x8c0b17c5, 90 },
    { 0x43ab17c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x344b17c9, 90 },
    { 0x853b17ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fab17cf, 95 },
    { 0x2d8b17d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437b17d5, 91 },
    { 0, 0 },
    { 0x1cab17d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb17dc, 100 },
    { 0xaf0b17dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64b17e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x909b17e4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xcceb17e7, 76 },
    { 0, 0 },
    { 0x88fb17e9, 91 },
    { 0x6f7b17ea, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5b17f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92b17f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbb17f9, 90 },
    { 0xcb8b17fa, 52 },
    { 0xb2bb17fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b1802, 87 },
    { 0xe95b1803, 61 },
    { 0xf44b1804, 90 },
    { 0x33fb1802, 76 },
    { 0x748b1804, 33 },
    { 0xbfcb1807, 61 },
    { 0x409b1808, 23 },
    { 0xcc9b1809, 46 },
    { 0xeacb1808, 61 },
    { 0x838b1803, 100 },
    { 0x76fb1808, 90 },
    { 0xa45b180d, 55 },
    { 0xd68b180e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9b1811, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516b1815, 95 },
    { 0xe63b1815, 90 },
    { 0xc48b1817, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab181b, 77 },
    { 0x96db181c, 95 },
    { 0xb13b181d, 87 },
    { 0, 0 },
    { 0x86eb181f, 33 },
    { 0x3b0b181f, 85 },
    { 0, 0 },
    { 0x7e8b1822, 95 },
    { 0xfd0b1823, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x294b1826, 55 },
    { 0, 0 },
    { 0x2cb1828, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3b1830, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212b1836, 100 },
    { 0x8a5b1837, 90 },
    { 0x7c4b1836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02b183c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb1840, 52 },
    { 0xedb1840, 33 },
    { 0, 0 },
    { 0xdb7b1843, 98 },
    { 0xd80b1843, 80 },
    { 0, 0 },
    { 0x44bb1846, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb184a, 68 },
    { 0x6e6b184a, 75 },
    { 0x18fb184c, 90 },
    { 0xb3bb184c, 75 },
    { 0, 0 },
    { 0x8dfb184f, 95 },
    { 0xf9db1850, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x954b1853, 90 },
    { 0x134b1853, 98 },
    { 0xcadb1855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cb185a, 95 },
    { 0, 0 },
    { 0xf00b185c, 23 },
    { 0x5ccb185d, 40 },
    { 0x7fb185e, 98 },
    { 0xa64b185e, 95 },
    { 0xa89b185e, 33 },
    { 0, 0 },
    { 0x91ab1862, 99 },
    { 0, 0 },
    { 0x80bb1864, 90 },
    { 0, 0 },
    { 0xa12b1866, 61 },
    { 0xb75b1867, 61 },
    { 0x5a2b1866, 23 },
    { 0x78ab1866, 23 },
    { 0, 0 },
    { 0x600b186b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ab186e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69b1872, 95 },
    { 0x4aeb1873, 33 },
    { 0, 0 },
    { 0xd2eb1875, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x589b1878, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64b187b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb187e, 96 },
    { 0x22db187e, 90 },
    { 0x1a4b1880, 61 },
    { 0x933b1881, 90 },
    { 0x1bcb1882, 33 },
    { 0x727b1883, 99 },
    { 0x502b1882, 66 },
    { 0x8e6b1885, 95 },
    { 0xccb1886, 34 },
    { 0xcb8b1887, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14b188b, 61 },
    { 0x4c5b188c, 90 },
    { 0, 0 },
    { 0x82b188e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bb1892, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680b1898, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43b189e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1b18a1, 76 },
    { 0x138b18a2, 90 },
    { 0xaa5b18a1, 55 },
    { 0xeceb18a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0b18ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b18b1, 23 },
    { 0x459b18b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66b18b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568b18ba, 33 },
    { 0xbcfb18bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcb18bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98b18c4, 90 },
    { 0, 0 },
    { 0x726b18c6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8b18c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60db18cf, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745b18d3, 87 },
    { 0, 0 },
    { 0x2edb18d5, 55 },
    { 0x769b18d5, 93 },
    { 0, 0 },
    { 0x1b7b18d8, 98 },
    { 0x6bb18d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9b18de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b18e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2b18e6, 33 },
    { 0xa81b18e7, 93 },
    { 0x90bb18e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bb18f0, 23 },
    { 0x5e7b18f1, 90 },
    { 0xc41b18f0, 33 },
    { 0x2d5b18f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x478b18f6, 84 },
    { 0x71cb18f7, 90 },
    { 0x53fb18f7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829b1900, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2b1903, 90 },
    { 0x5eeb1904, 55 },
    { 0, 0 },
    { 0x7b1b1906, 84 },
    { 0x38fb1906, 33 },
    { 0x49ab1906, 95 },
    { 0x2d8b1909, 98 },
    { 0x5c1b190a, 23 },
    { 0x4dab190b, 33 },
    { 0x155b190b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984b1911, 90 },
    { 0xbdab1912, 98 },
    { 0, 0 },
    { 0xe40b1914, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ab191e, 33 },
    { 0x83eb191f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995b1923, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0b1928, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ab192c, 93 },
    { 0xac9b192c, 90 },
    { 0x1bbb192e, 95 },
    { 0x876b192d, 34 },
    { 0xc9ab1930, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eb1936, 69 },
    { 0, 0 },
    { 0x33fb1938, 100 },
    { 0x46cb1939, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70b1941, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1b1944, 33 },
    { 0xcf6b1945, 76 },
    { 0x680b1945, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3b1949, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6b194c, 99 },
    { 0xd28b194d, 43 },
    { 0xa6ab194e, 23 },
    { 0x8feb194f, 78 },
    { 0, 0 },
    { 0xd2bb1951, 90 },
    { 0x67b1951, 85 },
    { 0x136b1952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572b1957, 95 },
    { 0, 0 },
    { 0x796b1959, 55 },
    { 0x610b195a, 34 },
    { 0xa69b1959, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bb195f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb1962, 52 },
    { 0x94db1963, 55 },
    { 0x44cb1963, 98 },
    { 0xffcb1965, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32b196a, 81 },
    { 0xd38b196b, 67 },
    { 0xd2db196c, 88 },
    { 0xb40b196d, 90 },
    { 0xf95b196e, 61 },
    { 0x601b196e, 52 },
    { 0x891b196d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab1973, 100 },
    { 0x333b1974, 23 },
    { 0x186b1973, 95 },
    { 0x13ab1973, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62b1979, 61 },
    { 0x967b197a, 34 },
    { 0xd7db197a, 90 },
    { 0x253b197b, 33 },
    { 0, 0 },
    { 0x6cbb197e, 46 },
    { 0x429b197e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789b1984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cb1987, 64 },
    { 0xe35b1988, 67 },
    { 0x81eb1989, 95 },
    { 0x533b1987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71b198d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb1991, 23 },
    { 0, 0 },
    { 0x8a4b1993, 17 },
    { 0x2a8b1994, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fab1999, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9b19a4, 90 },
    { 0x8d7b19a5, 88 },
    { 0, 0 },
    { 0x7ab19a7, 95 },
    { 0x7e3b19a8, 46 },
    { 0x2db19a9, 61 },
    { 0, 0 },
    { 0xea9b19ab, 66 },
    { 0x63db19ac, 33 },
    { 0x7e0b19ad, 91 },
    { 0xbf1b19ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db19b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b19b6, 48 },
    { 0x4a2b19b7, 90 },
    { 0, 0 },
    { 0xcd2b19b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271b19c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753b19c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb19ca, 80 },
    { 0, 0 },
    { 0xec1b19cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661b19d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62b19d5, 98 },
    { 0x520b19d6, 90 },
    { 0xc57b19d7, 33 },
    { 0xf2eb19d7, 34 },
    { 0xaccb19d7, 81 },
    { 0x5f5b19da, 90 },
    { 0x65fb19d5, 55 },
    { 0x490b19dc, 23 },
    { 0xf9bb19dd, 76 },
    { 0x19db19de, 23 },
    { 0x481b19df, 98 },
    { 0, 0 },
    { 0xe7b19e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81b19e4, 43 },
    { 0, 0 },
    { 0xa91b19e6, 23 },
    { 0x7b1b19e7, 87 },
    { 0x7f9b19e8, 66 },
    { 0xb59b19e8, 98 },
    { 0x3d7b19ea, 78 },
    { 0, 0 },
    { 0x893b19ec, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fb19f2, 33 },
    { 0xc73b19f3, 90 },
    { 0xca2b19f2, 43 },
    { 0x4efb19f5, 33 },
    { 0x80db19f4, 43 },
    { 0x2c3b19f7, 76 },
    { 0x155b19f8, 78 },
    { 0x2dcb19f7, 61 },
    { 0xc40b19fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b1a05, 77 },
    { 0xfd9b1a06, 33 },
    { 0, 0 },
    { 0x251b1a08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ab1a0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860b1a13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8b1a17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3db1a1a, 90 },
    { 0xe0eb1a1a, 76 },
    { 0xa11b1a1c, 90 },
    { 0, 0 },
    { 0x32bb1a1e, 90 },
    { 0, 0 },
    { 0x62fb1a20, 91 },
    { 0xc52b1a21, 33 },
    { 0x151b1a22, 90 },
    { 0x48eb1a21, 66 },
    { 0, 0 },
    { 0xe3eb1a25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b1a28, 91 },
    { 0xdbdb1a29, 33 },
    { 0x9c1b1a2a, 98 },
    { 0, 0 },
    { 0x2fdb1a2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b1a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b1a36, 98 },
    { 0, 0 },
    { 0x1f5b1a38, 95 },
    { 0, 0 },
    { 0xa63b1a3a, 34 },
    { 0xbcab1a3a, 91 },
    { 0xaf5b1a3b, 84 },
    { 0x71fb1a3d, 33 },
    { 0, 0 },
    { 0x7eb1a3f, 99 },
    { 0xf89b1a40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x511b1a43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bb1a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x646b1a4a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb1a52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191b1a5a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b1a5f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bb1a65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb1a68, 61 },
    { 0xff7b1a68, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53b1a6c, 90 },
    { 0xd8b1a6d, 94 },
    { 0, 0 },
    { 0x62eb1a6f, 98 },
    { 0x6a5b1a6f, 66 },
    { 0xc71b1a71, 66 },
    { 0, 0 },
    { 0x428b1a73, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966b1a78, 61 },
    { 0x658b1a78, 61 },
    { 0x43fb1a7a, 43 },
    { 0x8b2b1a7a, 33 },
    { 0x85cb1a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab1a80, 98 },
    { 0, 0 },
    { 0x46b1a82, 61 },
    { 0, 0 },
    { 0x830b1a84, 98 },
    { 0xbe6b1a84, 33 },
    { 0xc1bb1a84, 61 },
    { 0, 0 },
    { 0xa1cb1a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4b1a8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb1a90, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acb1a95, 76 },
    { 0xf67b1a95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b1a99, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb1a9d, 61 },
    { 0, 0 },
    { 0x69fb1a9f, 90 },
    { 0, 0 },
    { 0x36eb1aa1, 65 },
    { 0x38fb1aa2, 33 },
    { 0, 0 },
    { 0xe5fb1aa4, 23 },
    { 0, 0 },
    { 0xb9b1aa6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bb1aa9, 93 },
    { 0xbbcb1aaa, 100 },
    { 0x13bb1aab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14b1aaf, 61 },
    { 0xbedb1aaf, 33 },
    { 0, 0 },
    { 0xe58b1ab2, 33 },
    { 0, 0 },
    { 0xc5eb1ab4, 55 },
    { 0x2c8b1ab4, 85 },
    { 0xbf9b1ab6, 46 },
    { 0xcc3b1ab7, 68 },
    { 0, 0 },
    { 0x6d1b1ab9, 84 },
    { 0x6f8b1ab9, 83 },
    { 0x30ab1abb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7b1abe, 91 },
    { 0xb66b1abe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518b1ac3, 95 },
    { 0, 0 },
    { 0x751b1ac5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82b1ac8, 76 },
    { 0xa48b1ac8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b1acd, 91 },
    { 0, 0 },
    { 0xf3cb1acf, 93 },
    { 0, 0 },
    { 0x426b1ad1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eb1ad5, 61 },
    { 0xacdb1ad6, 91 },
    { 0, 0 },
    { 0x4c7b1ad8, 93 },
    { 0xaa9b1ad9, 61 },
    { 0, 0 },
    { 0x710b1adb, 78 },
    { 0x351b1adc, 33 },
    { 0x134b1add, 76 },
    { 0x5bbb1add, 98 },
    { 0x4e7b1adf, 95 },
    { 0xf63b1ae0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbab1ae5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cb1ae8, 87 },
    { 0, 0 },
    { 0xef6b1aea, 23 },
    { 0x2e0b1aea, 98 },
    { 0x4a1b1aec, 90 },
    { 0x577b1aec, 55 },
    { 0xf34b1aee, 17 },
    { 0x2fb1aea, 98 },
    { 0xe89b1af0, 46 },
    { 0xde9b1af1, 68 },
    { 0, 0 },
    { 0x7f8b1af3, 55 },
    { 0x2a7b1af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab1af7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7b1afa, 23 },
    { 0x2c8b1afb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5b1b02, 66 },
    { 0x8d7b1b03, 23 },
    { 0, 0 },
    { 0x892b1b05, 61 },
    { 0, 0 },
    { 0xe94b1b07, 23 },
    { 0x4db1b08, 90 },
    { 0, 0 },
    { 0xd00b1b0a, 23 },
    { 0xf5db1b0b, 77 },
    { 0x11bb1b0c, 33 },
    { 0x29eb1b0d, 90 },
    { 0x7a6b1b0a, 75 },
    { 0xbbb1b0a, 40 },
    { 0x9c8b1b0f, 75 },
    { 0xa51b1b11, 43 },
    { 0x3bab1b12, 61 },
    { 0xa9b1b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cb1b1a, 61 },
    { 0xa99b1b1b, 66 },
    { 0xd4ab1b1a, 77 },
    { 0xcd4b1b1a, 40 },
    { 0xe98b1b1e, 98 },
    { 0x928b1b1e, 90 },
    { 0, 0 },
    { 0x78db1b21, 88 },
    { 0xfc8b1b21, 95 },
    { 0, 0 },
    { 0x8e5b1b24, 66 },
    { 0x927b1b24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x17db1b28, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cb1b2e, 46 },
    { 0, 0 },
    { 0x917b1b30, 99 },
    { 0, 0 },
    { 0x65b1b32, 91 },
    { 0, 0 },
    { 0x7b1b34, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfb1b3a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb1b3e, 76 },
    { 0, 0 },
    { 0x5cdb1b40, 43 },
    { 0, 0 },
    { 0x9f0b1b42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0b1b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeb1b4e, 78 },
    { 0x4ab1b4f, 85 },
    { 0x920b1b50, 87 },
    { 0x2b0b1b51, 61 },
    { 0x349b1b52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fb1b55, 87 },
    { 0x193b1b56, 52 },
    { 0, 0 },
    { 0x9fb1b58, 61 },
    { 0x949b1b59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4b1b5e, 78 },
    { 0, 0 },
    { 0x581b1b60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x581b1b63, 90 },
    { 0xe9fb1b63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2b1b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5b1b6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b1b71, 66 },
    { 0, 0 },
    { 0x446b1b73, 23 },
    { 0, 0 },
    { 0x45fb1b75, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadb1b79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5b1b7e, 52 },
    { 0x8acb1b7f, 33 },
    { 0xe7ab1b80, 23 },
    { 0, 0 },
    { 0xfd6b1b82, 93 },
    { 0x3d3b1b83, 77 },
    { 0xfd0b1b84, 61 },
    { 0x4e8b1b84, 76 },
    { 0, 0 },
    { 0x791b1b87, 88 },
    { 0, 0 },
    { 0x8b5b1b89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52db1b8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901b1b97, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01b1b9c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816b1ba0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615b1ba4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eb1ba8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccb1bae, 90 },
    { 0xdb0b1baf, 43 },
    { 0xc54b1bb0, 46 },
    { 0x4fcb1baf, 46 },
    { 0xaa5b1bb2, 76 },
    { 0x59db1bb3, 77 },
    { 0x710b1bb4, 55 },
    { 0x187b1bb3, 90 },
    { 0xcb5b1bb5, 17 },
    { 0xaefb1bb7, 33 },
    { 0xbe4b1bb8, 67 },
    { 0xd5ab1bb9, 76 },
    { 0x809b1bb1, 23 },
    { 0x140b1bae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddb1bbe, 87 },
    { 0, 0 },
    { 0x38fb1bc0, 90 },
    { 0, 0 },
    { 0x59ab1bc2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fb1bc5, 33 },
    { 0xa9db1bc6, 33 },
    { 0, 0 },
    { 0x6b1b1bc8, 23 },
    { 0x8db1bc9, 61 },
    { 0xe8fb1bc9, 43 },
    { 0, 0 },
    { 0x177b1bcc, 84 },
    { 0, 0 },
    { 0xe11b1bce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x655b1bd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcb1bdd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x86db1be0, 95 },
    { 0x61fb1be1, 61 },
    { 0, 0 },
    { 0x24eb1be3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2b1beb, 55 },
    { 0, 0 },
    { 0x9d8b1bed, 98 },
    { 0xec1b1bee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200b1bf6, 99 },
    { 0xe7fb1bf7, 61 },
    { 0xe59b1bf6, 95 },
    { 0xdeab1bf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0b1bfc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1c03, 95 },
    { 0, 0 },
    { 0x957b1c05, 83 },
    { 0x257b1c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22b1c09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23b1c0f, 95 },
    { 0x401b1c10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609b1c15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x550b1c18, 90 },
    { 0x4a2b1c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb1c1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781b1c21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b1c2d, 90 },
    { 0xb2db1c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7b1c31, 37 },
    { 0, 0 },
    { 0x71b1c33, 98 },
    { 0x9e5b1c34, 69 },
    { 0, 0 },
    { 0x75bb1c36, 33 },
    { 0x450b1c36, 23 },
    { 0, 0 },
    { 0x47ab1c39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b1c3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x622b1c40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feb1c45, 43 },
    { 0xbe0b1c46, 93 },
    { 0x17ab1c47, 33 },
    { 0x3dbb1c48, 90 },
    { 0x868b1c48, 46 },
    { 0, 0 },
    { 0x9f9b1c4b, 93 },
    { 0x3c5b1c4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bb1c4f, 46 },
    { 0x37fb1c50, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5b1c53, 91 },
    { 0x12eb1c54, 95 },
    { 0xdddb1c54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbb1c5c, 61 },
    { 0xdfbb1c5d, 97 },
    { 0x7e8b1c5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b1c62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cb1c65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543b1c69, 81 },
    { 0, 0 },
    { 0xbfb1c6b, 66 },
    { 0x378b1c6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52b1c70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12b1c75, 90 },
    { 0x6a3b1c76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b1c7b, 33 },
    { 0xf07b1c7c, 61 },
    { 0, 0 },
    { 0x463b1c7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296b1c83, 90 },
    { 0x89ab1c84, 93 },
    { 0, 0 },
    { 0x115b1c86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb1c8b, 68 },
    { 0xb1c8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b1c92, 33 },
    { 0, 0 },
    { 0x576b1c94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07b1c98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7b1ca0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79b1ca7, 93 },
    { 0xfa0b1ca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1db1cb0, 90 },
    { 0x1d5b1cb0, 90 },
    { 0, 0 },
    { 0x5c6b1cb3, 67 },
    { 0xdd3b1cb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb1cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b1cbc, 35 },
    { 0x970b1cbc, 81 },
    { 0xf46b1cbc, 98 },
    { 0xa33b1cbf, 98 },
    { 0x6b9b1cc0, 98 },
    { 0xf3bb1cc1, 33 },
    { 0x67ab1cc0, 78 },
    { 0xe32b1cc3, 95 },
    { 0xf5cb1cc3, 90 },
    { 0x193b1cbe, 76 },
    { 0, 0 },
    { 0x59fb1cc7, 80 },
    { 0, 0 },
    { 0x7f3b1cc9, 61 },
    { 0, 0 },
    { 0x35fb1ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x305b1cce, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fb1cd1, 76 },
    { 0, 0 },
    { 0x2a4b1cd3, 90 },
    { 0x1bbb1cd4, 98 },
    { 0x7fab1cd3, 52 },
    { 0, 0 },
    { 0xc39b1cd7, 87 },
    { 0, 0 },
    { 0x594b1cd9, 93 },
    { 0, 0 },
    { 0xd6b1cdb, 95 },
    { 0xc34b1cdc, 43 },
    { 0, 0 },
    { 0x529b1cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76db1ce6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62b1ce9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b1ced, 76 },
    { 0x728b1ced, 93 },
    { 0, 0 },
    { 0xe35b1cf0, 66 },
    { 0, 0 },
    { 0xa8b1cf2, 68 },
    { 0x477b1cf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b1cf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162b1cfd, 87 },
    { 0x4d8b1cfe, 90 },
    { 0x51eb1cff, 90 },
    { 0x626b1cff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32b1d04, 90 },
    { 0x9c6b1d04, 95 },
    { 0xcf4b1d06, 90 },
    { 0x5d7b1d07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x669b1d0f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6b1d15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b1d18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1b1d1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39b1d25, 23 },
    { 0xb27b1d26, 90 },
    { 0, 0 },
    { 0xd23b1d28, 98 },
    { 0, 0 },
    { 0xa9eb1d2a, 67 },
    { 0xe07b1d2a, 90 },
    { 0x84bb1d2c, 81 },
    { 0x577b1d2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b1d30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5b1d40, 23 },
    { 0, 0 },
    { 0x3e9b1d42, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1b1d46, 23 },
    { 0x998b1d47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77b1d4a, 95 },
    { 0, 0 },
    { 0x506b1d4c, 96 },
    { 0x6aab1d4d, 87 },
    { 0xce7b1d4e, 88 },
    { 0x78ab1d4e, 90 },
    { 0x79ab1d4e, 98 },
    { 0x904b1d4c, 33 },
    { 0x7f5b1d4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bb1d56, 33 },
    { 0x97b1d57, 33 },
    { 0xe59b1d56, 84 },
    { 0x16eb1d59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9db1d5d, 61 },
    { 0x10cb1d5d, 67 },
    { 0xf71b1d5f, 66 },
    { 0x8afb1d5f, 90 },
    { 0, 0 },
    { 0x326b1d62, 98 },
    { 0xd96b1d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930b1d68, 33 },
    { 0x8c4b1d69, 93 },
    { 0, 0 },
    { 0x9f2b1d6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644b1d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86b1d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445b1d7b, 81 },
    { 0x6b1b1d7c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7b1d80, 76 },
    { 0, 0 },
    { 0xbebb1d82, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2b1d89, 49 },
    { 0x733b1d89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948b1d90, 61 },
    { 0x9ecb1d91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5db1d98, 99 },
    { 0, 0 },
    { 0x173b1d9a, 90 },
    { 0x813b1d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829b1da0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6b1da8, 34 },
    { 0, 0 },
    { 0x8cdb1daa, 90 },
    { 0xf3bb1daa, 78 },
    { 0, 0 },
    { 0xd35b1dad, 99 },
    { 0x41db1dae, 98 },
    { 0x7d0b1daf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb1db4, 77 },
    { 0x17cb1db5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cb1dba, 61 },
    { 0xf6cb1dbb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x188b1dbe, 67 },
    { 0x629b1dbe, 34 },
    { 0x355b1dc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359b1dc9, 33 },
    { 0x62db1dca, 61 },
    { 0, 0 },
    { 0x47b1dcc, 46 },
    { 0xd08b1dcd, 91 },
    { 0x446b1dcd, 76 },
    { 0xcedb1dcf, 33 },
    { 0xbd9b1dcc, 90 },
    { 0x3abb1dd1, 47 },
    { 0x439b1dd2, 34 },
    { 0x355b1dcd, 23 },
    { 0x513b1dd4, 66 },
    { 0x98bb1dcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271b1dd9, 90 },
    { 0x802b1dd9, 95 },
    { 0x81b1ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x406b1dde, 61 },
    { 0x2fb1dde, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdb1de4, 95 },
    { 0x7e1b1de5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838b1dee, 75 },
    { 0, 0 },
    { 0xa60b1df0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bb1df6, 33 },
    { 0, 0 },
    { 0x242b1df8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7b1dfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0b1e00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cb1e04, 90 },
    { 0xe01b1e04, 66 },
    { 0x6f1b1e05, 76 },
    { 0xe50b1e07, 33 },
    { 0x496b1e08, 87 },
    { 0x1f3b1e08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x255b1e0c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfb1e10, 23 },
    { 0xd49b1e11, 33 },
    { 0x59eb1e11, 90 },
    { 0x371b1e11, 84 },
    { 0x452b1e14, 61 },
    { 0x8b3b1e15, 98 },
    { 0xdcdb1e13, 33 },
    { 0xd86b1e17, 33 },
    { 0x68b1e14, 91 },
    { 0xcc4b1e19, 66 },
    { 0xa0cb1e15, 52 },
    { 0xaf5b1e1b, 95 },
    { 0x195b1e15, 76 },
    { 0x6bdb1e1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdb1e26, 33 },
    { 0x1a0b1e27, 33 },
    { 0x8f9b1e27, 61 },
    { 0xabfb1e27, 98 },
    { 0xda1b1e2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33b1e38, 75 },
    { 0x356b1e38, 33 },
    { 0x308b1e38, 17 },
    { 0x7bdb1e3b, 95 },
    { 0xdceb1e3b, 56 },
    { 0xcc2b1e39, 90 },
    { 0, 0 },
    { 0x71fb1e3f, 99 },
    { 0x104b1e40, 61 },
    { 0x8f3b1e40, 76 },
    { 0x9ffb1e3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afb1e4a, 61 },
    { 0x8b4b1e4b, 61 },
    { 0x68ab1e4c, 93 },
    { 0x19ab1e4c, 46 },
    { 0, 0 },
    { 0xbe3b1e4f, 33 },
    { 0x7e8b1e4f, 95 },
    { 0x99b1e51, 100 },
    { 0x21cb1e51, 90 },
    { 0xfdab1e51, 66 },
    { 0x1b3b1e54, 33 },
    { 0x33b1e54, 76 },
    { 0x4fab1e56, 91 },
    { 0xabfb1e55, 23 },
    { 0, 0 },
    { 0x5d1b1e59, 33 },
    { 0x5aeb1e5a, 90 },
    { 0, 0 },
    { 0x315b1e5c, 98 },
    { 0x353b1e5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243b1e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddb1e68, 34 },
    { 0, 0 },
    { 0x9fb1e6a, 76 },
    { 0xb4fb1e6b, 52 },
    { 0, 0 },
    { 0x59bb1e6d, 23 },
    { 0xd20b1e6e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570b1e74, 90 },
    { 0, 0 },
    { 0xeb1b1e76, 76 },
    { 0, 0 },
    { 0x68fb1e78, 33 },
    { 0, 0 },
    { 0x55bb1e7a, 76 },
    { 0x23ab1e7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x806b1e7e, 90 },
    { 0, 0 },
    { 0xdcab1e80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52b1e84, 78 },
    { 0x73fb1e84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106b1e8e, 90 },
    { 0, 0 },
    { 0x6aab1e90, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ab1e93, 87 },
    { 0xcd7b1e94, 46 },
    { 0, 0 },
    { 0xab5b1e96, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb1e9c, 33 },
    { 0x948b1e9c, 90 },
    { 0xccdb1e9c, 90 },
    { 0x4b0b1e9f, 90 },
    { 0x6d9b1e9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7b1eb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bb1eb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb1eb9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93b1ebf, 52 },
    { 0xaab1ec0, 33 },
    { 0x5d9b1ebf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cb1ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ab1ec9, 61 },
    { 0xe20b1eca, 91 },
    { 0x9fb1ec9, 52 },
    { 0xc82b1ecc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecb1ecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42b1ed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571b1ed9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb1edc, 61 },
    { 0x5b5b1edc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bb1ee0, 55 },
    { 0x870b1ee0, 98 },
    { 0x47ab1ee2, 84 },
    { 0x705b1ee3, 90 },
    { 0xa7eb1ee0, 55 },
    { 0x1ab1ee5, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ab1ee9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3b1eee, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcb1ef3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cb1ef7, 90 },
    { 0xaceb1ef8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b1efc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b1f00, 78 },
    { 0x378b1f00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cb1f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b1f0b, 95 },
    { 0x1c6b1f0b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2b1f14, 90 },
    { 0, 0 },
    { 0x6c5b1f16, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ab1f1a, 97 },
    { 0x42bb1f1a, 33 },
    { 0x9c9b1f1c, 95 },
    { 0xaab1f1b, 43 },
    { 0, 0 },
    { 0x2a5b1f1f, 87 },
    { 0, 0 },
    { 0x20bb1f21, 95 },
    { 0x8c3b1f22, 98 },
    { 0xdfab1f23, 98 },
    { 0xa17b1f22, 70 },
    { 0x37cb1f25, 17 },
    { 0x233b1f26, 55 },
    { 0x10eb1f22, 93 },
    { 0x686b1f28, 23 },
    { 0xdd3b1f29, 90 },
    { 0x70fb1f2a, 33 },
    { 0xcdb1f2b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b1f2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbb1f35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76b1f38, 95 },
    { 0, 0 },
    { 0xc93b1f3a, 23 },
    { 0xfe8b1f3b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x173b1f3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x656b1f41, 99 },
    { 0x780b1f42, 96 },
    { 0xc38b1f41, 93 },
    { 0xe6cb1f44, 33 },
    { 0xdbeb1f45, 20 },
    { 0x91b1f46, 61 },
    { 0x2d0b1f44, 76 },
    { 0x62b1f48, 98 },
    { 0xba7b1f47, 23 },
    { 0x38fb1f42, 69 },
    { 0x615b1f47, 98 },
    { 0x5f6b1f45, 23 },
    { 0x19eb1f4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131b1f52, 98 },
    { 0x683b1f52, 98 },
    { 0xd98b1f54, 33 },
    { 0, 0 },
    { 0xb81b1f56, 52 },
    { 0x430b1f57, 33 },
    { 0x74cb1f58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892b1f5c, 90 },
    { 0x778b1f5c, 33 },
    { 0, 0 },
    { 0x258b1f5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2db1f62, 90 },
    { 0x4ab1f63, 95 },
    { 0x9aeb1f64, 90 },
    { 0x83cb1f65, 90 },
    { 0x9d4b1f65, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b1f6a, 52 },
    { 0x3a7b1f6b, 98 },
    { 0, 0 },
    { 0x9afb1f6d, 95 },
    { 0xab6b1f6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78b1f72, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bb1f76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403b1f7a, 76 },
    { 0x93b1f7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2b1f80, 90 },
    { 0x8ebb1f80, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x213b1f84, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4b1f8a, 67 },
    { 0xfc3b1f8a, 96 },
    { 0xab0b1f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4b1f90, 90 },
    { 0, 0 },
    { 0x4c4b1f92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1f95, 95 },
    { 0xaa4b1f96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x137b1f99, 33 },
    { 0xe08b1f99, 75 },
    { 0xef7b1f99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdab1fa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b1faa, 61 },
    { 0, 0 },
    { 0xc20b1fac, 90 },
    { 0x4beb1fac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4b1fb1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2b1fb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b1fbe, 90 },
    { 0x5d7b1fbe, 17 },
    { 0x297b1fc0, 95 },
    { 0x2d0b1fc0, 76 },
    { 0, 0 },
    { 0xf07b1fc3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5b1fc6, 90 },
    { 0xa7fb1fc7, 76 },
    { 0xb30b1fc8, 33 },
    { 0, 0 },
    { 0x97b1fca, 34 },
    { 0x34fb1fcb, 33 },
    { 0x644b1fcb, 61 },
    { 0, 0 },
    { 0xe9eb1fce, 23 },
    { 0, 0 },
    { 0xbf4b1fd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65db1fd5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b1fd9, 95 },
    { 0, 0 },
    { 0xb1eb1fdb, 95 },
    { 0x4d8b1fdc, 61 },
    { 0, 0 },
    { 0xd87b1fde, 77 },
    { 0, 0 },
    { 0x478b1fe0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb1fe3, 100 },
    { 0x826b1fe3, 75 },
    { 0xb4eb1fe5, 90 },
    { 0x2eab1fe6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eab1fe9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b1fee, 90 },
    { 0xeb5b1fee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b1ff3, 34 },
    { 0, 0 },
    { 0xc8fb1ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b1ff8, 90 },
    { 0x68eb1ff9, 78 },
    { 0xaf6b1ff9, 85 },
    { 0xc7b1ffb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfb1fff, 93 },
    { 0x294b2000, 55 },
    { 0xfbab2001, 100 },
    { 0xebbb2002, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eb2008, 93 },
    { 0xdedb2009, 61 },
    { 0, 0 },
    { 0x970b200b, 80 },
    { 0x74b200b, 98 },
    { 0, 0 },
    { 0x621b200e, 55 },
    { 0, 0 },
    { 0xf48b2010, 90 },
    { 0xa97b2011, 35 },
    { 0, 0 },
    { 0x9b2013, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b2017, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcab201a, 55 },
    { 0xb1eb201b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744b2027, 90 },
    { 0, 0 },
    { 0x5e9b2029, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefb202d, 90 },
    { 0x547b202e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b2032, 95 },
    { 0x12b2032, 95 },
    { 0x4ecb2033, 95 },
    { 0, 0 },
    { 0xa14b2036, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb203b, 17 },
    { 0xa4cb203c, 95 },
    { 0xf28b203d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0b2040, 99 },
    { 0xc44b2041, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab204a, 98 },
    { 0, 0 },
    { 0x46db204c, 61 },
    { 0x3bbb204d, 90 },
    { 0, 0 },
    { 0x59eb204f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b2054, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb205a, 46 },
    { 0xcc2b205b, 76 },
    { 0x8c9b205a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b205f, 55 },
    { 0, 0 },
    { 0x40cb2061, 53 },
    { 0, 0 },
    { 0x40eb2063, 23 },
    { 0x2ddb2064, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91b206b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3b2079, 33 },
    { 0xcbcb207a, 43 },
    { 0x766b207b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ab207f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b2087, 61 },
    { 0, 0 },
    { 0xca7b2089, 90 },
    { 0x656b2089, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b208d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b2093, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b2098, 61 },
    { 0, 0 },
    { 0x4ccb209a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeb209d, 23 },
    { 0, 0 },
    { 0xdf9b209f, 90 },
    { 0xa3eb20a0, 78 },
    { 0x39bb209f, 33 },
    { 0xa91b20a2, 91 },
    { 0x52fb209f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80eb20ad, 91 },
    { 0xd99b20ae, 90 },
    { 0x23bb20ae, 55 },
    { 0x2b7b20ad, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceb20b7, 98 },
    { 0, 0 },
    { 0x569b20b9, 90 },
    { 0, 0 },
    { 0x735b20bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950b20c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb20c5, 90 },
    { 0xe9ab20c6, 68 },
    { 0, 0 },
    { 0x669b20c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcb20ce, 91 },
    { 0xf7db20cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8b20d4, 40 },
    { 0x1d8b20d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fb20d9, 61 },
    { 0, 0 },
    { 0x61fb20db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacb20de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32b20e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bb20e9, 17 },
    { 0xb92b20ea, 33 },
    { 0xff3b20eb, 61 },
    { 0x172b20ec, 43 },
    { 0x124b20ed, 98 },
    { 0, 0 },
    { 0xd4fb20ef, 99 },
    { 0, 0 },
    { 0x1a5b20f1, 98 },
    { 0, 0 },
    { 0xed9b20f3, 90 },
    { 0x3afb20f3, 98 },
    { 0x8e5b20f3, 91 },
    { 0x6c4b20f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b2101, 87 },
    { 0x8dcb2101, 90 },
    { 0xaafb2103, 17 },
    { 0, 0 },
    { 0x973b2105, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b2108, 33 },
    { 0, 0 },
    { 0x3c3b210a, 67 },
    { 0x41cb210b, 61 },
    { 0x875b210c, 33 },
    { 0, 0 },
    { 0x223b210e, 93 },
    { 0xeecb210e, 90 },
    { 0x433b2110, 78 },
    { 0x6f1b2111, 68 },
    { 0xd0b2112, 46 },
    { 0, 0 },
    { 0xcc9b2114, 81 },
    { 0xd09b2115, 90 },
    { 0x2e4b2116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b211c, 33 },
    { 0x565b211d, 98 },
    { 0xa04b211e, 23 },
    { 0, 0 },
    { 0x39fb2120, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391b2127, 23 },
    { 0, 0 },
    { 0xdc4b2129, 66 },
    { 0x760b212a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88b2132, 98 },
    { 0x2e1b2133, 90 },
    { 0x9dcb2134, 33 },
    { 0xd1fb2135, 95 },
    { 0x957b2132, 98 },
    { 0x857b2137, 55 },
    { 0x304b2138, 95 },
    { 0x2c7b2134, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bb2146, 98 },
    { 0, 0 },
    { 0x7b8b2148, 61 },
    { 0, 0 },
    { 0x260b214a, 90 },
    { 0x70ab214b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8b2152, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcb2157, 85 },
    { 0xe02b2158, 43 },
    { 0, 0 },
    { 0x423b215a, 34 },
    { 0xc67b215a, 85 },
    { 0, 0 },
    { 0x7e3b215d, 66 },
    { 0, 0 },
    { 0xaf2b215f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484b2164, 33 },
    { 0x81db2164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b2169, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fb216c, 77 },
    { 0x2ecb216d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734b2172, 61 },
    { 0, 0 },
    { 0xaeab2174, 90 },
    { 0x967b2175, 43 },
    { 0x52eb2174, 100 },
    { 0x19bb2174, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebb217b, 90 },
    { 0, 0 },
    { 0xc6b217d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34b2181, 91 },
    { 0xb74b2181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db218b, 81 },
    { 0x832b218b, 33 },
    { 0x2bab218b, 77 },
    { 0, 0 },
    { 0x5f4b218f, 55 },
    { 0x53bb2190, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334b2196, 76 },
    { 0xeeab2197, 33 },
    { 0xc9fb2196, 95 },
    { 0xf96b2197, 23 },
    { 0xc08b219a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128b219e, 23 },
    { 0x979b219f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1b21a3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835b21a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38b21ac, 99 },
    { 0xd9bb21ad, 85 },
    { 0xf8ab21ae, 90 },
    { 0x89ab21ad, 61 },
    { 0x193b21af, 61 },
    { 0x9feb21b1, 61 },
    { 0xb2bb21ac, 61 },
    { 0xc03b21af, 33 },
    { 0x183b21b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9b21b8, 90 },
    { 0xf10b21b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cb21bc, 95 },
    { 0, 0 },
    { 0xaf2b21be, 52 },
    { 0, 0 },
    { 0x743b21c0, 98 },
    { 0, 0 },
    { 0xb28b21c2, 88 },
    { 0xc29b21c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4b21ce, 33 },
    { 0, 0 },
    { 0xc1db21d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843b21d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7b21d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeab21dc, 23 },
    { 0x53b21dd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9db21e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab21e4, 55 },
    { 0x4a9b21e5, 17 },
    { 0x859b21e6, 68 },
    { 0xdedb21e7, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69b21eb, 23 },
    { 0x6f5b21ec, 66 },
    { 0, 0 },
    { 0xfe6b21ee, 78 },
    { 0x1eeb21ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffb21f2, 90 },
    { 0xe61b21f3, 55 },
    { 0x229b21f4, 98 },
    { 0xf0cb21f5, 46 },
    { 0, 0 },
    { 0x16db21f7, 34 },
    { 0, 0 },
    { 0x680b21f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4b21fe, 95 },
    { 0, 0 },
    { 0x86fb2200, 90 },
    { 0xdadb2200, 56 },
    { 0xf39b2200, 95 },
    { 0xd42b2202, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b2206, 55 },
    { 0x747b2207, 33 },
    { 0, 0 },
    { 0xa9cb2209, 61 },
    { 0xa43b220a, 66 },
    { 0x8edb220a, 95 },
    { 0x40eb220c, 55 },
    { 0xbdbb220c, 61 },
    { 0x7a4b220c, 81 },
    { 0x889b220a, 76 },
    { 0x546b2210, 34 },
    { 0xd96b2211, 61 },
    { 0x8f2b2212, 33 },
    { 0xaf3b2210, 43 },
    { 0x56ab2212, 52 },
    { 0x54fb2210, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbb221a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28b221f, 90 },
    { 0x53bb2220, 81 },
    { 0x614b2221, 90 },
    { 0, 0 },
    { 0xf0bb2223, 33 },
    { 0x7a5b2224, 90 },
    { 0xf38b2225, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bb2229, 23 },
    { 0xd93b222a, 33 },
    { 0xc81b222b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37b2231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433b2235, 91 },
    { 0, 0 },
    { 0x968b2237, 34 },
    { 0x791b2237, 61 },
    { 0xaf8b2239, 77 },
    { 0x29fb2237, 33 },
    { 0xb22b2238, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94b223e, 87 },
    { 0, 0 },
    { 0x54fb2240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eb2244, 91 },
    { 0, 0 },
    { 0x86db2246, 90 },
    { 0x3c4b2247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4b224a, 98 },
    { 0xd00b224a, 23 },
    { 0x27b224c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0eb2255, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bb2260, 99 },
    { 0, 0 },
    { 0x86bb2262, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3b2265, 95 },
    { 0x24fb2266, 33 },
    { 0x5fb2265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bb226a, 33 },
    { 0, 0 },
    { 0xd30b226c, 61 },
    { 0xbcab226c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b2271, 100 },
    { 0xf6db2272, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eb2276, 98 },
    { 0xf4b2277, 52 },
    { 0x869b2277, 46 },
    { 0, 0 },
    { 0xc87b227a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89db227d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb2282, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91b2288, 95 },
    { 0xccab2289, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605b2292, 55 },
    { 0x63bb2293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb2299, 55 },
    { 0xf76b2299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eb22a2, 23 },
    { 0x2db22a3, 23 },
    { 0, 0 },
    { 0xbf2b22a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cb22a8, 88 },
    { 0xb51b22a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b22ac, 90 },
    { 0x801b22ac, 23 },
    { 0x7b6b22ae, 61 },
    { 0x1c4b22af, 55 },
    { 0, 0 },
    { 0x14db22b1, 61 },
    { 0xc46b22b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171b22b7, 93 },
    { 0, 0 },
    { 0xbcdb22b9, 90 },
    { 0, 0 },
    { 0x421b22bb, 78 },
    { 0xe26b22bc, 98 },
    { 0x728b22bd, 85 },
    { 0xbb7b22bc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ab22c3, 99 },
    { 0, 0 },
    { 0xfb7b22c5, 33 },
    { 0x2abb22c6, 76 },
    { 0xcf2b22c6, 61 },
    { 0, 0 },
    { 0xe70b22c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176b22cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770b22d1, 99 },
    { 0x7cbb22d2, 33 },
    { 0xbe6b22d3, 93 },
    { 0, 0 },
    { 0x7c3b22d5, 100 },
    { 0x4c9b22d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5b22db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510b22e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06b22e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b22eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b22ee, 66 },
    { 0x321b22ef, 99 },
    { 0, 0 },
    { 0x223b22f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ab22f9, 66 },
    { 0x751b22fa, 95 },
    { 0x411b22fa, 43 },
    { 0xfe1b22fc, 23 },
    { 0x2eeb22fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b2304, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eb2309, 33 },
    { 0x82b2309, 33 },
    { 0x6bfb230b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8db230f, 87 },
    { 0, 0 },
    { 0x5dcb2311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56b2314, 95 },
    { 0xe31b2315, 90 },
    { 0xc76b2315, 33 },
    { 0xac4b2317, 47 },
    { 0xfc3b2318, 33 },
    { 0x825b2316, 76 },
    { 0x2f8b231a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54b231f, 93 },
    { 0, 0 },
    { 0xbdeb2321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fb2325, 33 },
    { 0xa1bb2326, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ab232d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453b2333, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beb2336, 33 },
    { 0, 0 },
    { 0x631b2338, 88 },
    { 0, 0 },
    { 0x2aeb233a, 90 },
    { 0x174b233b, 23 },
    { 0x65fb233b, 33 },
    { 0, 0 },
    { 0x4a6b233e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bb2342, 34 },
    { 0x468b2343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28b2346, 67 },
    { 0x4cb2347, 23 },
    { 0x91db2348, 98 },
    { 0, 0 },
    { 0xea9b234a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b234e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b2351, 85 },
    { 0, 0 },
    { 0x7f6b2353, 43 },
    { 0xe35b2353, 23 },
    { 0x3c9b2355, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58b2359, 33 },
    { 0, 0 },
    { 0xd42b235b, 95 },
    { 0, 0 },
    { 0xb19b235d, 95 },
    { 0x5eab235e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5eb2363, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cb2366, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb2369, 43 },
    { 0, 0 },
    { 0x72ab236b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283b2372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbb2375, 98 },
    { 0, 0 },
    { 0xec2b2377, 95 },
    { 0x2c0b2378, 95 },
    { 0x1cb2378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcb237d, 78 },
    { 0x302b237d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3b2381, 84 },
    { 0, 0 },
    { 0xbc6b2383, 61 },
    { 0xd0eb2384, 46 },
    { 0x240b2385, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2b2388, 23 },
    { 0x45eb2389, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb238d, 88 },
    { 0xbfeb238d, 90 },
    { 0xb5bb238f, 51 },
    { 0x224b2390, 23 },
    { 0, 0 },
    { 0xe13b2392, 95 },
    { 0xa55b2393, 46 },
    { 0xe8bb2393, 76 },
    { 0x356b2395, 43 },
    { 0x92bb2396, 90 },
    { 0x2ddb2397, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddb239f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfb23a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9b23a7, 55 },
    { 0x768b23a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1db23af, 55 },
    { 0x5e9b23b0, 90 },
    { 0x737b23b1, 98 },
    { 0x7f0b23b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34b23b9, 98 },
    { 0, 0 },
    { 0xdfb23bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342b23bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39b23c2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5b23c5, 34 },
    { 0x8d8b23c5, 55 },
    { 0x413b23c5, 43 },
    { 0x72eb23c7, 33 },
    { 0x79bb23c9, 85 },
    { 0xa96b23ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9b23d2, 95 },
    { 0, 0 },
    { 0xd12b23d4, 98 },
    { 0xab6b23d5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196b23d9, 33 },
    { 0xaafb23da, 43 },
    { 0x4d8b23db, 61 },
    { 0, 0 },
    { 0x693b23dd, 33 },
    { 0x1ecb23de, 39 },
    { 0, 0 },
    { 0xbeab23e0, 4 },
    { 0, 0 },
    { 0x392b23e2, 78 },
    { 0xb0b23e3, 93 },
    { 0x499b23e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b23e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476b23ed, 85 },
    { 0xa7eb23ee, 52 },
    { 0, 0 },
    { 0xd3ab23f0, 33 },
    { 0x72bb23f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9b23fb, 64 },
    { 0xfffb23fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdb23ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b2402, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b2409, 98 },
    { 0x67ab240a, 33 },
    { 0x420b240b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46b240f, 98 },
    { 0, 0 },
    { 0xcd3b2411, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390b2416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb241a, 77 },
    { 0xe2b241b, 61 },
    { 0xcf7b241a, 99 },
    { 0, 0 },
    { 0x517b241e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bb2422, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794b2429, 90 },
    { 0x490b2429, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b242f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b2432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b2435, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8b243b, 55 },
    { 0xec1b243c, 88 },
    { 0x695b243d, 98 },
    { 0x3bfb243b, 61 },
    { 0xa4eb243e, 90 },
    { 0x59fb2440, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beb2443, 34 },
    { 0xd0b2444, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98db2449, 33 },
    { 0x836b244a, 61 },
    { 0, 0 },
    { 0x96fb244c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3b244f, 91 },
    { 0x17db2450, 76 },
    { 0xa9cb2451, 97 },
    { 0xb39b2452, 85 },
    { 0xe91b2453, 90 },
    { 0x70fb2454, 23 },
    { 0x767b2455, 95 },
    { 0xd8bb2453, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68b245b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb2460, 93 },
    { 0, 0 },
    { 0x541b2462, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0b2466, 76 },
    { 0x617b2466, 17 },
    { 0xbafb2468, 93 },
    { 0x846b2469, 34 },
    { 0, 0 },
    { 0xdfeb246b, 2 },
    { 0x836b246c, 23 },
    { 0xcf4b246b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b2471, 81 },
    { 0x907b2472, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5b2475, 95 },
    { 0x905b2476, 67 },
    { 0xbb9b2477, 80 },
    { 0x7eab2477, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3b247c, 87 },
    { 0, 0 },
    { 0x3c0b247e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x509b2481, 76 },
    { 0x733b2481, 69 },
    { 0x641b2483, 34 },
    { 0x2f3b2484, 23 },
    { 0x82cb2481, 90 },
    { 0x3a3b2486, 99 },
    { 0xdb3b2486, 78 },
    { 0xe66b2487, 43 },
    { 0xb7ab2486, 95 },
    { 0xdc4b248a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b248d, 33 },
    { 0x8eeb248e, 23 },
    { 0xe0ab248e, 100 },
    { 0xdc4b248e, 80 },
    { 0, 0 },
    { 0x693b2492, 81 },
    { 0x4bbb2493, 90 },
    { 0x128b2494, 90 },
    { 0, 0 },
    { 0xc83b2496, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdb249c, 52 },
    { 0, 0 },
    { 0x9e1b249e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b24a2, 33 },
    { 0x759b24a3, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9b24a7, 34 },
    { 0xe2fb24a8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60b24ac, 94 },
    { 0xa8fb24ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceb24b3, 90 },
    { 0, 0 },
    { 0xcc1b24b5, 98 },
    { 0x5e7b24b6, 71 },
    { 0x319b24b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x609b24ba, 95 },
    { 0xaebb24bb, 95 },
    { 0x37bb24bc, 76 },
    { 0xcbfb24bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x724b24c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6b24c5, 85 },
    { 0x9deb24c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8b24c9, 17 },
    { 0xa1db24ca, 78 },
    { 0xbc5b24cb, 65 },
    { 0xf7fb24cc, 43 },
    { 0x153b24ca, 33 },
    { 0xfeab24ce, 46 },
    { 0x54eb24cc, 90 },
    { 0x444b24ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eb24d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb24d7, 77 },
    { 0xd18b24d8, 95 },
    { 0xca9b24d7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x13b24dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ab24df, 99 },
    { 0, 0 },
    { 0x9e5b24e1, 46 },
    { 0x8eb24e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8b24e6, 23 },
    { 0, 0 },
    { 0x74b24e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x783b24eb, 23 },
    { 0xd27b24ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56b24f1, 64 },
    { 0x50fb24f1, 34 },
    { 0xb25b24f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb24f6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cab24f9, 66 },
    { 0, 0 },
    { 0x7a2b24fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14db2502, 66 },
    { 0x365b2502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fb2506, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345b250a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472b250e, 78 },
    { 0x827b250e, 95 },
    { 0xd5fb250f, 78 },
    { 0, 0 },
    { 0x917b2512, 76 },
    { 0xa55b2513, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375b2518, 98 },
    { 0xca8b2519, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55b251c, 92 },
    { 0, 0 },
    { 0xa97b251e, 87 },
    { 0xea6b251f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7b2523, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3b252a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06b252e, 61 },
    { 0x370b252f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccb2532, 55 },
    { 0, 0 },
    { 0x5e1b2534, 84 },
    { 0x314b2534, 34 },
    { 0, 0 },
    { 0xb4cb2537, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00b2541, 61 },
    { 0xd99b2541, 33 },
    { 0x16cb2543, 98 },
    { 0xe94b2544, 78 },
    { 0xbaeb2543, 76 },
    { 0x680b2543, 33 },
    { 0xc1eb2547, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b254a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cb254d, 33 },
    { 0x5b2b254e, 33 },
    { 0x580b254e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cb2552, 52 },
    { 0xccbb2553, 90 },
    { 0x72cb2553, 93 },
    { 0xcacb2555, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b2558, 68 },
    { 0x9b4b2559, 98 },
    { 0x59bb255a, 43 },
    { 0, 0 },
    { 0x129b255c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdb2561, 95 },
    { 0, 0 },
    { 0x773b2563, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ab2567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb256d, 33 },
    { 0xf69b256e, 91 },
    { 0x165b256f, 23 },
    { 0x2d6b2570, 87 },
    { 0x539b256e, 61 },
    { 0, 0 },
    { 0xb4db2573, 90 },
    { 0x4aeb2573, 52 },
    { 0x16db2574, 61 },
    { 0xc01b2575, 33 },
    { 0, 0 },
    { 0x839b2578, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b257b, 95 },
    { 0, 0 },
    { 0x917b257d, 77 },
    { 0x7b9b257e, 90 },
    { 0x98bb257d, 23 },
    { 0xcabb2580, 65 },
    { 0, 0 },
    { 0xa55b2582, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cb2585, 34 },
    { 0x141b2586, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2b258a, 61 },
    { 0xe97b258a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adb258f, 33 },
    { 0xb7ab2590, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x599b2593, 61 },
    { 0x477b2594, 55 },
    { 0x432b2595, 90 },
    { 0, 0 },
    { 0x146b2597, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cb259b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102b25a1, 90 },
    { 0xf07b25a2, 90 },
    { 0xb01b25a3, 61 },
    { 0x495b25a4, 96 },
    { 0xa0ab25a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cb25a8, 43 },
    { 0x2c5b25a9, 61 },
    { 0x723b25aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x319b25ad, 34 },
    { 0xb9fb25ad, 43 },
    { 0, 0 },
    { 0x7f5b25b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363b25b8, 55 },
    { 0x59b25b9, 85 },
    { 0x486b25b8, 52 },
    { 0x31eb25b8, 95 },
    { 0x61b25bc, 75 },
    { 0x1d0b25bb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169b25c2, 66 },
    { 0xae9b25c2, 33 },
    { 0x9b7b25c4, 90 },
    { 0, 0 },
    { 0xae2b25c6, 84 },
    { 0x4dfb25c7, 76 },
    { 0xf46b25c7, 33 },
    { 0, 0 },
    { 0xddb25ca, 61 },
    { 0x691b25ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eab25ce, 52 },
    { 0, 0 },
    { 0xc1cb25d0, 33 },
    { 0x985b25d0, 98 },
    { 0, 0 },
    { 0xbe4b25d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87b25d9, 55 },
    { 0xd58b25d9, 91 },
    { 0, 0 },
    { 0xdcfb25dc, 90 },
    { 0x901b25dd, 55 },
    { 0x17db25de, 94 },
    { 0x904b25df, 90 },
    { 0xeffb25e0, 90 },
    { 0xe26b25e1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14b25e4, 77 },
    { 0, 0 },
    { 0x2dcb25e6, 23 },
    { 0xf66b25e6, 98 },
    { 0xc3bb25e8, 47 },
    { 0, 0 },
    { 0xc13b25ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bab25ee, 85 },
    { 0, 0 },
    { 0xfa7b25f0, 95 },
    { 0x564b25f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x977b25f4, 95 },
    { 0xb42b25f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bb25f8, 43 },
    { 0xc5db25f8, 23 },
    { 0x85db25fa, 66 },
    { 0xdceb25fa, 43 },
    { 0, 0 },
    { 0x5e5b25fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577b2601, 95 },
    { 0xe0ab2602, 55 },
    { 0, 0 },
    { 0x967b2604, 61 },
    { 0, 0 },
    { 0xbf7b2606, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b2609, 43 },
    { 0, 0 },
    { 0xbeb260b, 55 },
    { 0x332b260c, 95 },
    { 0, 0 },
    { 0xe83b260e, 61 },
    { 0x748b260f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4b261b, 33 },
    { 0x6cbb261c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31b2620, 33 },
    { 0xe7db2621, 47 },
    { 0x2b5b2622, 17 },
    { 0xed3b2621, 90 },
    { 0x830b2621, 61 },
    { 0x569b2625, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fb2628, 55 },
    { 0x5c1b2629, 23 },
    { 0, 0 },
    { 0x1d9b262b, 80 },
    { 0xea9b262b, 90 },
    { 0x794b262d, 95 },
    { 0x3b4b262e, 61 },
    { 0x397b262f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461b2633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ab2636, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x426b2639, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b263c, 33 },
    { 0x4d7b263c, 55 },
    { 0xd42b263e, 23 },
    { 0xe1ab263d, 33 },
    { 0, 0 },
    { 0xfa8b2641, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920b2647, 61 },
    { 0xa41b2648, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44b264d, 76 },
    { 0xca8b264e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83b2657, 95 },
    { 0xb3ab2657, 95 },
    { 0xe40b2659, 90 },
    { 0, 0 },
    { 0xc2db265b, 61 },
    { 0, 0 },
    { 0x704b265d, 91 },
    { 0x5fbb265d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b2663, 34 },
    { 0, 0 },
    { 0x3a4b2665, 98 },
    { 0, 0 },
    { 0xa30b2667, 98 },
    { 0x62b2667, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcb266c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0b2673, 98 },
    { 0xe74b2673, 100 },
    { 0, 0 },
    { 0x1e5b2676, 98 },
    { 0xc70b2677, 90 },
    { 0, 0 },
    { 0x9cab2679, 23 },
    { 0xc29b2679, 33 },
    { 0x65eb267b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2b2680, 61 },
    { 0, 0 },
    { 0x691b2682, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b268c, 22 },
    { 0xa8cb268c, 99 },
    { 0x551b268e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2b2696, 90 },
    { 0, 0 },
    { 0x10fb2698, 61 },
    { 0, 0 },
    { 0x433b269a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72db269d, 98 },
    { 0x1c0b269e, 90 },
    { 0x698b269f, 90 },
    { 0xfddb269d, 52 },
    { 0x1d9b26a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7b26a8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8b26ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0b26af, 23 },
    { 0xfdab26b0, 61 },
    { 0x98fb26b1, 99 },
    { 0, 0 },
    { 0x115b26b3, 90 },
    { 0x7e4b26b4, 99 },
    { 0xd1ab26b4, 23 },
    { 0x98cb26b4, 98 },
    { 0xf62b26b7, 90 },
    { 0xdb6b26b6, 52 },
    { 0xb39b26b8, 53 },
    { 0xf68b26b6, 33 },
    { 0x288b26bb, 90 },
    { 0x68bb26ba, 23 },
    { 0x78cb26b3, 95 },
    { 0x8fdb26b7, 99 },
    { 0, 0 },
    { 0x30b26c0, 90 },
    { 0, 0 },
    { 0xb3ab26c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20db26c7, 56 },
    { 0x85ab26c7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cb26d0, 95 },
    { 0xb53b26d0, 61 },
    { 0, 0 },
    { 0x41db26d3, 78 },
    { 0x70ab26d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b26d7, 90 },
    { 0x9db26d8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b26de, 78 },
    { 0x53bb26df, 85 },
    { 0x384b26df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67b26e3, 76 },
    { 0xa6ab26e4, 90 },
    { 0x2a7b26e5, 46 },
    { 0x465b26e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248b26ec, 91 },
    { 0, 0 },
    { 0x9b8b26ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0b26f2, 91 },
    { 0xf5b26f3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9b26f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdb26fa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744b2700, 52 },
    { 0x6f2b2700, 61 },
    { 0x281b2702, 55 },
    { 0x9abb2703, 52 },
    { 0xc17b2704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7b2708, 90 },
    { 0, 0 },
    { 0xa60b270a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb270d, 84 },
    { 0x13db270e, 76 },
    { 0x58db270f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x889b2712, 87 },
    { 0x6cfb2713, 87 },
    { 0x63cb2712, 37 },
    { 0xad8b2715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292b2719, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x957b271c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2b271f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x384b2722, 90 },
    { 0xde3b2723, 84 },
    { 0, 0 },
    { 0xe82b2725, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb2729, 51 },
    { 0x15eb2729, 61 },
    { 0, 0 },
    { 0x8b2b272c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9b2732, 23 },
    { 0, 0 },
    { 0x25eb2734, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62b2738, 90 },
    { 0x461b2738, 100 },
    { 0xb7ab2738, 66 },
    { 0x287b273b, 23 },
    { 0xf5ab273c, 43 },
    { 0x5dcb273c, 76 },
    { 0x5e3b273e, 34 },
    { 0x60fb273d, 76 },
    { 0xaa0b2740, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab2747, 23 },
    { 0, 0 },
    { 0x6e5b2749, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb274c, 90 },
    { 0xe0bb274d, 90 },
    { 0x395b274e, 98 },
    { 0x5d3b274f, 95 },
    { 0xa88b2750, 78 },
    { 0, 0 },
    { 0xba7b2752, 87 },
    { 0, 0 },
    { 0x1b4b2754, 33 },
    { 0x53bb2755, 33 },
    { 0, 0 },
    { 0x3f8b2757, 90 },
    { 0xe3eb2757, 84 },
    { 0, 0 },
    { 0x7bbb275a, 61 },
    { 0x528b275b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b275e, 23 },
    { 0, 0 },
    { 0x6ecb2760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bb2763, 23 },
    { 0xf1cb2763, 95 },
    { 0, 0 },
    { 0xe94b2766, 91 },
    { 0x49b2767, 95 },
    { 0x6b8b2768, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40b276b, 98 },
    { 0, 0 },
    { 0x91eb276d, 61 },
    { 0xe50b276d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65b2774, 90 },
    { 0xf05b2775, 98 },
    { 0, 0 },
    { 0xb1bb2777, 76 },
    { 0xfbbb2778, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54db2780, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80b2783, 33 },
    { 0, 0 },
    { 0xc03b2785, 61 },
    { 0xb7eb2786, 33 },
    { 0x1f0b2787, 90 },
    { 0xc2fb2787, 61 },
    { 0, 0 },
    { 0x7ebb278a, 93 },
    { 0x6dbb278b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ab278e, 98 },
    { 0x7a0b278f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfb2792, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ab2796, 33 },
    { 0x9aab2797, 98 },
    { 0xd9b2796, 33 },
    { 0x353b2797, 90 },
    { 0, 0 },
    { 0x5ccb279b, 90 },
    { 0xdfab279b, 68 },
    { 0x470b279b, 95 },
    { 0xa38b279e, 34 },
    { 0x8b5b279d, 98 },
    { 0xc6eb279c, 90 },
    { 0xec7b279b, 61 },
    { 0x38bb279c, 43 },
    { 0x23fb27a3, 85 },
    { 0, 0 },
    { 0xf06b27a5, 33 },
    { 0x47cb27a5, 76 },
    { 0xed6b27a5, 33 },
    { 0x47b27a6, 97 },
    { 0x268b27a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321b27b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x985b27b6, 95 },
    { 0xaf2b27b7, 78 },
    { 0xb21b27b7, 90 },
    { 0x5f5b27b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eb27bd, 23 },
    { 0, 0 },
    { 0xb7bb27bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b27c2, 34 },
    { 0xa2db27c3, 76 },
    { 0xfc1b27c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fb27c9, 91 },
    { 0, 0 },
    { 0xbbeb27cb, 34 },
    { 0x7eeb27cb, 33 },
    { 0x127b27cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b27d0, 69 },
    { 0x517b27d1, 95 },
    { 0xf43b27d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30b27d5, 23 },
    { 0x1fab27d5, 61 },
    { 0x91b27d6, 33 },
    { 0xa33b27d6, 76 },
    { 0xe6cb27d7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1b27de, 55 },
    { 0x87eb27de, 90 },
    { 0xd21b27e0, 61 },
    { 0x3b3b27e0, 33 },
    { 0xe72b27df, 23 },
    { 0, 0 },
    { 0xb85b27e4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cb27e7, 90 },
    { 0, 0 },
    { 0x5f0b27e9, 33 },
    { 0x383b27ea, 90 },
    { 0x709b27eb, 33 },
    { 0, 0 },
    { 0xb20b27ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bb27f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b27f7, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205b27fc, 76 },
    { 0xde3b27fd, 18 },
    { 0x9edb27fe, 95 },
    { 0xeb0b27fc, 68 },
    { 0xaedb2800, 61 },
    { 0x9dcb2800, 76 },
    { 0x431b27fe, 33 },
    { 0x4bbb2803, 90 },
    { 0x22ab27fc, 95 },
    { 0, 0 },
    { 0x86db2806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fb280a, 99 },
    { 0xc76b280a, 98 },
    { 0xeeab280c, 99 },
    { 0xeafb280c, 76 },
    { 0xb2b280e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bb2814, 33 },
    { 0xc2bb2814, 98 },
    { 0, 0 },
    { 0x885b2817, 83 },
    { 0x370b2817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bb281b, 33 },
    { 0, 0 },
    { 0x5afb281d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eb2828, 23 },
    { 0xfbb2829, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x803b282c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ab282f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792b2835, 61 },
    { 0xb81b2836, 61 },
    { 0x1cfb2837, 90 },
    { 0, 0 },
    { 0x73cb2839, 23 },
    { 0, 0 },
    { 0x8c0b283b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb2842, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb2845, 61 },
    { 0, 0 },
    { 0x588b2847, 95 },
    { 0x6dcb2848, 76 },
    { 0x335b2849, 93 },
    { 0x185b284a, 100 },
    { 0x9c1b284b, 95 },
    { 0xab6b284c, 98 },
    { 0, 0 },
    { 0x278b284e, 95 },
    { 0, 0 },
    { 0xa14b2850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b2853, 83 },
    { 0xe09b2854, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22db2858, 52 },
    { 0x977b2859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6b2860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb2864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7b2867, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5beb286e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18b2872, 77 },
    { 0x9ebb2873, 63 },
    { 0x5a9b2874, 78 },
    { 0, 0 },
    { 0x7e5b2876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fb287e, 66 },
    { 0x839b287f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x365b2882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fb2885, 33 },
    { 0, 0 },
    { 0x696b2887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x926b288a, 93 },
    { 0, 0 },
    { 0x6b1b288c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40b288f, 95 },
    { 0x8a9b2890, 90 },
    { 0, 0 },
    { 0x483b2892, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882b289b, 52 },
    { 0x468b289b, 90 },
    { 0xd67b289b, 23 },
    { 0xcc7b289e, 68 },
    { 0xec2b289d, 90 },
    { 0, 0 },
    { 0x491b28a1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdb28a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfeb28ac, 87 },
    { 0x3f5b28ac, 61 },
    { 0x361b28ac, 77 },
    { 0xc4fb28af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcb28b3, 34 },
    { 0, 0 },
    { 0xb8ab28b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7b28ba, 61 },
    { 0xe09b28bb, 98 },
    { 0x736b28bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cb28c7, 55 },
    { 0, 0 },
    { 0xc23b28c9, 95 },
    { 0, 0 },
    { 0x3f9b28cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efb28d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8b28d5, 93 },
    { 0xa87b28d5, 61 },
    { 0x354b28d6, 90 },
    { 0, 0 },
    { 0x45b28d9, 61 },
    { 0x257b28da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab28dd, 33 },
    { 0x8db28dd, 76 },
    { 0x15db28de, 36 },
    { 0xab2b28de, 65 },
    { 0, 0 },
    { 0x63fb28e2, 95 },
    { 0, 0 },
    { 0xa7cb28e4, 90 },
    { 0x198b28e4, 43 },
    { 0xb3b28e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbb28ea, 99 },
    { 0x842b28eb, 23 },
    { 0xfc1b28ea, 76 },
    { 0x320b28ec, 4 },
    { 0x3c9b28ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b28f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9b2902, 68 },
    { 0x4ab2903, 90 },
    { 0x3c2b2902, 55 },
    { 0, 0 },
    { 0xc55b2906, 33 },
    { 0xa41b2907, 55 },
    { 0xc1bb2907, 85 },
    { 0x725b2908, 90 },
    { 0x518b2906, 33 },
    { 0x88ab290b, 85 },
    { 0x1e1b2908, 90 },
    { 0, 0 },
    { 0xdb0b290e, 90 },
    { 0x202b290f, 39 },
    { 0xb33b2910, 33 },
    { 0x3feb2911, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938b2917, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feb2922, 90 },
    { 0x428b2923, 75 },
    { 0x933b2924, 98 },
    { 0x2eab2923, 34 },
    { 0x577b2926, 43 },
    { 0xa20b2927, 69 },
    { 0, 0 },
    { 0x828b2929, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab2930, 90 },
    { 0xee6b2931, 90 },
    { 0x5f4b2932, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550b2936, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79eb293d, 23 },
    { 0, 0 },
    { 0xba8b293f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7b2944, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7b294e, 95 },
    { 0, 0 },
    { 0xea4b2950, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189b2954, 23 },
    { 0x6adb2954, 23 },
    { 0x845b2956, 98 },
    { 0xe3ab2957, 90 },
    { 0x65fb2956, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6b295b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261b2960, 90 },
    { 0xd60b2961, 98 },
    { 0, 0 },
    { 0xf66b2963, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22b2969, 61 },
    { 0x9eab2969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0b296f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99b2982, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bb298d, 33 },
    { 0, 0 },
    { 0x3c9b298f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3b2992, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3b2996, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8b299a, 98 },
    { 0, 0 },
    { 0x22fb299c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1b299f, 87 },
    { 0x453b29a0, 99 },
    { 0, 0 },
    { 0xd02b29a2, 91 },
    { 0x2e2b29a3, 61 },
    { 0x763b29a2, 75 },
    { 0xb4bb29a2, 85 },
    { 0xb84b29a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x761b29a9, 90 },
    { 0x5e4b29a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8b29b0, 84 },
    { 0x732b29b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1b29b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b29bc, 95 },
    { 0x108b29bc, 95 },
    { 0xcf6b29be, 46 },
    { 0x6ccb29be, 90 },
    { 0x2efb29c0, 91 },
    { 0xd03b29c1, 55 },
    { 0xaecb29c1, 23 },
    { 0xf78b29c3, 90 },
    { 0x4c2b29c3, 95 },
    { 0x870b29bd, 95 },
    { 0x2f4b29c3, 95 },
    { 0x57ab29c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb29cb, 23 },
    { 0, 0 },
    { 0x34fb29cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136b29d1, 99 },
    { 0x52eb29d2, 23 },
    { 0xe9db29d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9b29d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b29da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb29e4, 44 },
    { 0x1d5b29e5, 46 },
    { 0, 0 },
    { 0xa1b29e7, 90 },
    { 0xc18b29e8, 34 },
    { 0xb83b29e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fb29ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b29f8, 95 },
    { 0xda3b29f8, 75 },
    { 0xe72b29f9, 43 },
    { 0, 0 },
    { 0xc44b29fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86b29ff, 34 },
    { 0, 0 },
    { 0xd1bb2a01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2b2a05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eb2a0a, 95 },
    { 0x6fdb2a0b, 84 },
    { 0, 0 },
    { 0x462b2a0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3eb2a11, 98 },
    { 0xce2b2a12, 90 },
    { 0x8d7b2a13, 77 },
    { 0xc44b2a12, 76 },
    { 0x940b2a11, 19 },
    { 0xffbb2a16, 55 },
    { 0x818b2a16, 76 },
    { 0, 0 },
    { 0x6afb2a19, 17 },
    { 0, 0 },
    { 0xff3b2a1b, 99 },
    { 0xb2ab2a1c, 76 },
    { 0x6f0b2a1c, 99 },
    { 0xb53b2a1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x291b2a21, 33 },
    { 0x609b2a22, 98 },
    { 0x28cb2a22, 95 },
    { 0x85cb2a23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0b2a2a, 34 },
    { 0xd7cb2a2b, 90 },
    { 0x102b2a2c, 90 },
    { 0x86db2a2d, 88 },
    { 0x930b2a2c, 90 },
    { 0xff8b2a2a, 75 },
    { 0, 0 },
    { 0xeceb2a31, 95 },
    { 0, 0 },
    { 0x395b2a33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4db2a37, 61 },
    { 0, 0 },
    { 0x48cb2a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b2a46, 34 },
    { 0xe3db2a46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb2a4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db2a4d, 61 },
    { 0, 0 },
    { 0x9cb2a4f, 98 },
    { 0x448b2a4f, 46 },
    { 0x8db2a51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b2a56, 98 },
    { 0x59ab2a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cb2a5c, 90 },
    { 0x274b2a5d, 61 },
    { 0xf44b2a5d, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5db2a63, 84 },
    { 0xb85b2a63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25eb2a67, 98 },
    { 0x50fb2a68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764b2a6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168b2a73, 90 },
    { 0, 0 },
    { 0x10cb2a75, 52 },
    { 0, 0 },
    { 0x960b2a77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7b2a81, 33 },
    { 0xcd2b2a82, 33 },
    { 0, 0 },
    { 0x77eb2a84, 23 },
    { 0xf30b2a85, 76 },
    { 0x549b2a85, 33 },
    { 0xc51b2a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97b2a8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b2a92, 90 },
    { 0x51b2a92, 61 },
    { 0, 0 },
    { 0x4b8b2a95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bab2a9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7b2a9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157b2aa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1b2aa9, 76 },
    { 0xeeb2aaa, 93 },
    { 0xad2b2aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfb2ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b2ab4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3b2ab8, 66 },
    { 0xba4b2ab9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4b2abd, 67 },
    { 0x2b8b2abd, 61 },
    { 0xdddb2abf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1eb2ac5, 95 },
    { 0x626b2ac6, 61 },
    { 0x709b2ac7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6b2acb, 33 },
    { 0xc60b2acc, 34 },
    { 0x2cbb2acc, 46 },
    { 0x43ab2ace, 33 },
    { 0x845b2acd, 34 },
    { 0x17b2acf, 76 },
    { 0, 0 },
    { 0x8deb2ad2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafb2ad8, 93 },
    { 0, 0 },
    { 0x8e6b2ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ab2add, 61 },
    { 0, 0 },
    { 0x325b2adf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b2ae3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b2ae6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668b2aea, 98 },
    { 0xf9db2aeb, 76 },
    { 0xf1eb2aeb, 61 },
    { 0x1d0b2aeb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cb2af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649b2af9, 33 },
    { 0xf1fb2afa, 23 },
    { 0xf6db2af9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb2b00, 66 },
    { 0xd57b2b00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x519b2b04, 33 },
    { 0xbecb2b05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30b2b09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb2b0f, 98 },
    { 0xa72b2b10, 33 },
    { 0, 0 },
    { 0x2f2b2b12, 81 },
    { 0x9cdb2b13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78b2b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19db2b25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164b2b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43b2b30, 61 },
    { 0xe49b2b31, 90 },
    { 0xe8db2b31, 23 },
    { 0xd4bb2b31, 23 },
    { 0x921b2b32, 75 },
    { 0xf3ab2b32, 66 },
    { 0, 0 },
    { 0x2e8b2b37, 61 },
    { 0, 0 },
    { 0x43cb2b39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x664b2b3c, 43 },
    { 0x4a0b2b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fb2b40, 23 },
    { 0, 0 },
    { 0xc68b2b42, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2b2b45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb2b49, 98 },
    { 0x311b2b4a, 91 },
    { 0x938b2b4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80b2b51, 33 },
    { 0x28eb2b52, 76 },
    { 0, 0 },
    { 0xe0ab2b54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149b2b58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206b2b60, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8b2b64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8b2b68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64b2b6e, 76 },
    { 0, 0 },
    { 0x480b2b70, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ab2b73, 52 },
    { 0, 0 },
    { 0x4e6b2b75, 33 },
    { 0x62cb2b75, 52 },
    { 0x8c8b2b77, 67 },
    { 0x382b2b78, 100 },
    { 0x8b1b2b79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb2b7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x51db2b7f, 76 },
    { 0, 0 },
    { 0x86bb2b81, 43 },
    { 0x98ab2b82, 23 },
    { 0, 0 },
    { 0x8adb2b84, 61 },
    { 0, 0 },
    { 0xabbb2b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869b2b8b, 46 },
    { 0, 0 },
    { 0x15cb2b8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b2b92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277b2b96, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebb2b99, 98 },
    { 0xd95b2b9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db2b9e, 34 },
    { 0x866b2b9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb2ba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224b2bad, 93 },
    { 0x5e9b2bae, 61 },
    { 0xee7b2bae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b2bb4, 92 },
    { 0, 0 },
    { 0x76ab2bb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfb2bb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7b2bbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bb2bc4, 84 },
    { 0x6b1b2bc5, 33 },
    { 0x62ab2bc4, 98 },
    { 0, 0 },
    { 0xaefb2bc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x471b2bcb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cb2bd1, 17 },
    { 0xf9b2bd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2b2bd9, 61 },
    { 0xfb5b2bd9, 95 },
    { 0x437b2bdb, 78 },
    { 0xc16b2bdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fb2bdf, 100 },
    { 0, 0 },
    { 0xb1cb2be1, 93 },
    { 0x662b2be1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x942b2be5, 93 },
    { 0, 0 },
    { 0xd7ab2be7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b2bea, 77 },
    { 0, 0 },
    { 0xecb2bec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852b2bf7, 61 },
    { 0xccdb2bf7, 67 },
    { 0xa16b2bf7, 93 },
    { 0, 0 },
    { 0xd4db2bfb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b2c00, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bb2c05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0b2c09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2b2c0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841b2c12, 46 },
    { 0, 0 },
    { 0x1d2b2c14, 33 },
    { 0xcedb2c14, 95 },
    { 0, 0 },
    { 0x3e4b2c17, 33 },
    { 0xca7b2c18, 95 },
    { 0, 0 },
    { 0xeaeb2c1a, 40 },
    { 0x24cb2c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdb2c1e, 33 },
    { 0xd5cb2c1f, 95 },
    { 0x63ab2c20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110b2c24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab2c2a, 23 },
    { 0x36bb2c2a, 34 },
    { 0x3b6b2c2c, 33 },
    { 0xba7b2c2a, 61 },
    { 0, 0 },
    { 0xf65b2c2f, 33 },
    { 0x116b2c30, 61 },
    { 0x902b2c31, 61 },
    { 0x75bb2c2f, 98 },
    { 0x384b2c33, 48 },
    { 0x81fb2c34, 61 },
    { 0x3b2b2c35, 100 },
    { 0xcc9b2c36, 76 },
    { 0xfffb2c37, 43 },
    { 0x8c4b2c38, 55 },
    { 0x6d3b2c39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512b2c3e, 99 },
    { 0, 0 },
    { 0xfafb2c40, 81 },
    { 0x760b2c41, 100 },
    { 0x31eb2c42, 61 },
    { 0x2dcb2c40, 55 },
    { 0x3cb2c43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cb2c54, 80 },
    { 0, 0 },
    { 0x70eb2c56, 61 },
    { 0x6eab2c57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857b2c5b, 40 },
    { 0x330b2c5c, 33 },
    { 0x2f4b2c5c, 43 },
    { 0xe66b2c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb2c62, 90 },
    { 0xb4cb2c63, 68 },
    { 0x806b2c63, 55 },
    { 0, 0 },
    { 0x237b2c66, 93 },
    { 0xacdb2c66, 78 },
    { 0xf57b2c68, 23 },
    { 0x33eb2c68, 84 },
    { 0, 0 },
    { 0x8f8b2c6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8b2c6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16b2c72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb2c79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfb2c7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb2c81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfb2c86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bb2c8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b2c8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788b2c98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20b2ca1, 68 },
    { 0, 0 },
    { 0x5eab2ca3, 23 },
    { 0xe09b2ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ab2cac, 90 },
    { 0x913b2cac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379b2cb1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9b2cb4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495b2cb9, 84 },
    { 0xb2eb2cba, 98 },
    { 0x91b2cba, 34 },
    { 0xe4cb2cba, 95 },
    { 0xe06b2cbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4b2cc1, 85 },
    { 0, 0 },
    { 0xaafb2cc3, 90 },
    { 0, 0 },
    { 0xdc4b2cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11b2cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8b2cd0, 85 },
    { 0xdf8b2cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb2cd4, 93 },
    { 0, 0 },
    { 0xbbbb2cd6, 81 },
    { 0x9bab2cd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadb2cdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cb2cdf, 23 },
    { 0xf28b2ce0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2b2ce5, 60 },
    { 0x34b2ce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62eb2ceb, 90 },
    { 0, 0 },
    { 0x322b2ced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afb2cf5, 93 },
    { 0, 0 },
    { 0x9e3b2cf7, 55 },
    { 0x2f2b2cf8, 33 },
    { 0x8fcb2cf9, 61 },
    { 0xe0eb2cf9, 98 },
    { 0, 0 },
    { 0x277b2cfc, 90 },
    { 0, 0 },
    { 0x6b6b2cfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x793b2d01, 84 },
    { 0, 0 },
    { 0x36fb2d03, 61 },
    { 0, 0 },
    { 0x7feb2d05, 98 },
    { 0x44db2d05, 90 },
    { 0xfc6b2d05, 61 },
    { 0x7a0b2d08, 55 },
    { 0x985b2d07, 61 },
    { 0xf70b2d07, 61 },
    { 0xae9b2d06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8b2d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0b2d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933b2d1c, 94 },
    { 0xfc1b2d1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b2d21, 93 },
    { 0, 0 },
    { 0xa8eb2d23, 33 },
    { 0x3e4b2d24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7b2d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x802b2d2d, 61 },
    { 0, 0 },
    { 0x87fb2d2f, 99 },
    { 0, 0 },
    { 0xed6b2d31, 23 },
    { 0x404b2d31, 91 },
    { 0, 0 },
    { 0x36eb2d34, 90 },
    { 0x552b2d34, 95 },
    { 0, 0 },
    { 0xabb2d37, 77 },
    { 0x4bcb2d38, 46 },
    { 0xb4bb2d39, 76 },
    { 0x877b2d3a, 33 },
    { 0x6aeb2d3b, 61 },
    { 0x480b2d3c, 91 },
    { 0x90eb2d3a, 76 },
    { 0x730b2d3b, 98 },
    { 0xf70b2d38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffb2d42, 33 },
    { 0x280b2d43, 66 },
    { 0x57ab2d44, 61 },
    { 0x4ecb2d45, 85 },
    { 0x6ceb2d43, 76 },
    { 0x10b2d45, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b2d4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b2d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3b2d57, 100 },
    { 0, 0 },
    { 0x6d3b2d59, 23 },
    { 0x6beb2d5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fb2d5e, 33 },
    { 0xd4cb2d5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fb2d68, 91 },
    { 0, 0 },
    { 0x11eb2d6a, 61 },
    { 0xf1db2d6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0b2d78, 99 },
    { 0xd6eb2d78, 55 },
    { 0x21fb2d7a, 34 },
    { 0, 0 },
    { 0x57bb2d7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65b2d8a, 46 },
    { 0, 0 },
    { 0x9b7b2d8c, 61 },
    { 0x249b2d8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b2d92, 90 },
    { 0x624b2d93, 23 },
    { 0x443b2d94, 33 },
    { 0xbceb2d92, 90 },
    { 0xcc0b2d96, 76 },
    { 0x9c1b2d97, 100 },
    { 0xb7bb2d98, 76 },
    { 0, 0 },
    { 0x275b2d9a, 34 },
    { 0x3b0b2d9a, 96 },
    { 0xebeb2d9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20b2da0, 61 },
    { 0, 0 },
    { 0x1a7b2da2, 34 },
    { 0x6c5b2da3, 90 },
    { 0, 0 },
    { 0x17ab2da5, 61 },
    { 0x3dfb2da5, 95 },
    { 0x168b2da7, 85 },
    { 0, 0 },
    { 0x43ab2da9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x386b2dac, 61 },
    { 0x696b2dac, 90 },
    { 0x959b2dac, 93 },
    { 0x8bb2daf, 76 },
    { 0x272b2daf, 61 },
    { 0x151b2db1, 43 },
    { 0x113b2db2, 34 },
    { 0x33bb2dad, 90 },
    { 0, 0 },
    { 0xe7b2db5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503b2db9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466b2dc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bb2dc4, 33 },
    { 0x54b2dc5, 33 },
    { 0xd59b2dc6, 95 },
    { 0x855b2dc7, 65 },
    { 0xc30b2dc4, 95 },
    { 0xabbb2dc9, 23 },
    { 0xd15b2dca, 33 },
    { 0xd87b2dc9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b2dd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439b2dd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8b2dda, 61 },
    { 0x8adb2ddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b2dde, 91 },
    { 0xa72b2ddf, 61 },
    { 0xb4b2de0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb2de6, 52 },
    { 0x935b2de7, 99 },
    { 0xdc5b2de8, 78 },
    { 0x63cb2de9, 33 },
    { 0, 0 },
    { 0x84b2deb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16b2df0, 93 },
    { 0x197b2df1, 33 },
    { 0x1a3b2df2, 34 },
    { 0xb6fb2df3, 87 },
    { 0, 0 },
    { 0xba1b2df5, 61 },
    { 0, 0 },
    { 0xfddb2df7, 92 },
    { 0x89bb2df8, 99 },
    { 0x3adb2df9, 33 },
    { 0x80b2df8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0b2dfd, 90 },
    { 0, 0 },
    { 0x938b2dff, 61 },
    { 0xfa9b2dff, 76 },
    { 0x7b8b2e01, 85 },
    { 0xed9b2dff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x790b2e05, 33 },
    { 0, 0 },
    { 0xdb3b2e07, 55 },
    { 0x31ab2e07, 93 },
    { 0x6c8b2e09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b2e0c, 76 },
    { 0x37eb2e0d, 98 },
    { 0, 0 },
    { 0x52eb2e0f, 61 },
    { 0x654b2e0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdb2e15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b2e1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290b2e23, 93 },
    { 0, 0 },
    { 0xdacb2e25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x842b2e28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb2e2d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1beb2e34, 43 },
    { 0x35eb2e35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878b2e3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78b2e40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eb2e43, 23 },
    { 0, 0 },
    { 0xc59b2e45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b2e49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b2e4c, 90 },
    { 0x2c9b2e4d, 78 },
    { 0x997b2e4e, 33 },
    { 0xb0eb2e4f, 34 },
    { 0xba0b2e4d, 100 },
    { 0x1a8b2e51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41eb2e55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db2e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ab2e5d, 95 },
    { 0, 0 },
    { 0x3f3b2e5f, 61 },
    { 0, 0 },
    { 0xc34b2e61, 91 },
    { 0, 0 },
    { 0xebdb2e63, 90 },
    { 0xdb1b2e64, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x590b2e67, 98 },
    { 0x816b2e67, 98 },
    { 0x320b2e69, 90 },
    { 0x43fb2e6a, 90 },
    { 0x75b2e68, 33 },
    { 0x692b2e6a, 90 },
    { 0x4e0b2e6d, 78 },
    { 0xff3b2e6c, 61 },
    { 0, 0 },
    { 0x815b2e70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fb2e73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8b2e79, 90 },
    { 0xc2fb2e7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfb2e80, 43 },
    { 0x288b2e80, 84 },
    { 0xb96b2e82, 55 },
    { 0x879b2e82, 61 },
    { 0, 0 },
    { 0x190b2e85, 98 },
    { 0x314b2e86, 61 },
    { 0, 0 },
    { 0xf36b2e88, 23 },
    { 0xdc2b2e89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912b2e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0db2e90, 33 },
    { 0x176b2e91, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb2e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449b2e99, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980b2e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405b2ea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5b2ea9, 34 },
    { 0xedeb2eaa, 66 },
    { 0x56db2eaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42b2eae, 78 },
    { 0x90db2eaf, 61 },
    { 0xd4db2eaf, 69 },
    { 0x98eb2eb1, 33 },
    { 0xb8bb2eaf, 100 },
    { 0, 0 },
    { 0x7d9b2eb4, 78 },
    { 0, 0 },
    { 0x438b2eb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ab2ebc, 19 },
    { 0, 0 },
    { 0x948b2ebe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301b2ec2, 34 },
    { 0, 0 },
    { 0xd88b2ec4, 43 },
    { 0x141b2ec4, 95 },
    { 0x68bb2ec6, 39 },
    { 0xf24b2ec6, 76 },
    { 0x2a9b2ec8, 90 },
    { 0x551b2ec8, 95 },
    { 0, 0 },
    { 0x587b2ecb, 93 },
    { 0x29b2ecc, 23 },
    { 0, 0 },
    { 0xa9ab2ece, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1b2ed1, 68 },
    { 0x5eeb2ed2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27b2ed5, 90 },
    { 0x14b2ed5, 91 },
    { 0x3eab2ed7, 94 },
    { 0xa35b2ed8, 99 },
    { 0xcb4b2ed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfb2edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3b2edf, 55 },
    { 0xc1fb2edf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebb2eec, 33 },
    { 0x67db2eec, 67 },
    { 0xd30b2eec, 86 },
    { 0xb90b2eee, 90 },
    { 0, 0 },
    { 0x701b2ef1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2ef4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfb2ef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986b2efc, 61 },
    { 0xb7ab2efc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1b2f00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10b2f03, 98 },
    { 0, 0 },
    { 0xd25b2f05, 76 },
    { 0x447b2f06, 98 },
    { 0x59db2f07, 95 },
    { 0x377b2f08, 23 },
    { 0x737b2f08, 46 },
    { 0x848b2f09, 33 },
    { 0xf0db2f0b, 61 },
    { 0x8b4b2f0c, 87 },
    { 0, 0 },
    { 0x626b2f0e, 23 },
    { 0x1f5b2f0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bb2f14, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5b2f17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5b2f1c, 98 },
    { 0, 0 },
    { 0x3a2b2f1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39b2f21, 96 },
    { 0x142b2f22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88eb2f2b, 100 },
    { 0xab2b2f2c, 81 },
    { 0xca0b2f2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecb2f32, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcb2f3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9b2f40, 87 },
    { 0x3f8b2f41, 98 },
    { 0xc46b2f41, 23 },
    { 0, 0 },
    { 0xffb2f44, 76 },
    { 0, 0 },
    { 0x673b2f46, 66 },
    { 0xcdcb2f46, 99 },
    { 0xf9ab2f48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771b2f4c, 98 },
    { 0, 0 },
    { 0xf9db2f4e, 33 },
    { 0x190b2f4f, 38 },
    { 0, 0 },
    { 0x1b2f51, 40 },
    { 0, 0 },
    { 0x59db2f53, 15 },
    { 0x148b2f54, 95 },
    { 0xea6b2f54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9b2f59, 44 },
    { 0xb04b2f59, 93 },
    { 0xde1b2f5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbb2f5e, 95 },
    { 0, 0 },
    { 0x8c2b2f60, 98 },
    { 0, 0 },
    { 0x504b2f62, 93 },
    { 0xdb4b2f63, 61 },
    { 0x222b2f64, 98 },
    { 0, 0 },
    { 0x285b2f66, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4b2f6f, 33 },
    { 0x150b2f70, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ab2f7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297b2f82, 91 },
    { 0, 0 },
    { 0xa78b2f84, 95 },
    { 0, 0 },
    { 0x3e7b2f86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebb2f89, 34 },
    { 0x29cb2f89, 95 },
    { 0x610b2f8b, 98 },
    { 0xecdb2f89, 95 },
    { 0, 0 },
    { 0xec5b2f8e, 95 },
    { 0x3b1b2f8f, 91 },
    { 0xb6cb2f90, 46 },
    { 0xcdb2f91, 76 },
    { 0, 0 },
    { 0x175b2f93, 90 },
    { 0xc0fb2f94, 23 },
    { 0, 0 },
    { 0x432b2f96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb2f9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13b2fa1, 33 },
    { 0, 0 },
    { 0xa3fb2fa3, 61 },
    { 0x7acb2fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x710b2fa7, 61 },
    { 0x961b2fa8, 33 },
    { 0x381b2fa7, 93 },
    { 0, 0 },
    { 0x8d8b2fab, 33 },
    { 0, 0 },
    { 0xc70b2fad, 43 },
    { 0x923b2fae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b2fb1, 90 },
    { 0, 0 },
    { 0xc5eb2fb3, 81 },
    { 0x4f8b2fb4, 65 },
    { 0, 0 },
    { 0x1b5b2fb6, 55 },
    { 0x12cb2fb7, 43 },
    { 0, 0 },
    { 0xd61b2fb9, 23 },
    { 0, 0 },
    { 0xca3b2fbb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x534b2fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x697b2fc1, 95 },
    { 0, 0 },
    { 0x678b2fc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59db2fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb2fcb, 78 },
    { 0x61bb2fcb, 90 },
    { 0x36fb2fcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaab2fd0, 61 },
    { 0, 0 },
    { 0x574b2fd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66b2fd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4b2fe2, 34 },
    { 0x65cb2fe2, 90 },
    { 0, 0 },
    { 0xc05b2fe5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25db2fef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96eb2ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ab2ff6, 33 },
    { 0x9d6b2ff7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b2ffa, 98 },
    { 0x349b2ffa, 23 },
    { 0xe0ab2ffa, 33 },
    { 0x39cb2ffd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12b3005, 98 },
    { 0x97bb3006, 99 },
    { 0, 0 },
    { 0x692b3008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeb300c, 55 },
    { 0, 0 },
    { 0x594b300e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748b301b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627b301f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77b3024, 34 },
    { 0, 0 },
    { 0xaf5b3026, 90 },
    { 0xc75b3026, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5b302a, 61 },
    { 0x8ccb302a, 95 },
    { 0xb0cb302c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb302f, 90 },
    { 0xdcbb3030, 34 },
    { 0x577b302f, 95 },
    { 0, 0 },
    { 0x3e2b3033, 33 },
    { 0, 0 },
    { 0xb2bb3035, 99 },
    { 0xba5b3036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52b303d, 23 },
    { 0, 0 },
    { 0x781b303f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b3044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5b304d, 77 },
    { 0, 0 },
    { 0x133b304f, 23 },
    { 0xed6b3050, 87 },
    { 0x577b3051, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5b3054, 95 },
    { 0x179b3055, 33 },
    { 0xb81b3056, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0b3059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb3062, 68 },
    { 0xf40b3063, 93 },
    { 0xb2cb3064, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705b3068, 98 },
    { 0x7c8b3069, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b3083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8b3088, 61 },
    { 0x5e7b3089, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b308e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66b3092, 87 },
    { 0xe59b3093, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fb3096, 90 },
    { 0xa38b3096, 76 },
    { 0, 0 },
    { 0xe54b3099, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b309d, 78 },
    { 0x470b309e, 23 },
    { 0x3a2b309f, 61 },
    { 0x53cb30a0, 93 },
    { 0x6ab30a1, 33 },
    { 0xf1fb30a1, 61 },
    { 0x76ab30a0, 55 },
    { 0, 0 },
    { 0x565b30a5, 65 },
    { 0x5aab30a6, 43 },
    { 0x844b30a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39b30ab, 98 },
    { 0x2dfb30ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3b30b0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2b30bc, 61 },
    { 0x240b30bd, 36 },
    { 0xda6b30bd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cb30c2, 34 },
    { 0, 0 },
    { 0x124b30c4, 52 },
    { 0, 0 },
    { 0xca9b30c6, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6b30cb, 55 },
    { 0x679b30cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473b30d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cb30d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3b30d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5b30da, 33 },
    { 0x7bb30da, 90 },
    { 0xa4fb30dc, 61 },
    { 0x5d6b30db, 90 },
    { 0xeb2b30da, 55 },
    { 0xb77b30da, 61 },
    { 0xb01b30df, 61 },
    { 0, 0 },
    { 0x7b6b30e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cb30e6, 95 },
    { 0xb06b30e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70b30ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b30ed, 76 },
    { 0x67db30ee, 95 },
    { 0xf6db30ef, 90 },
    { 0x127b30f0, 76 },
    { 0xd5eb30f1, 90 },
    { 0x914b30f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61b30f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bb30f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b30fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63eb3102, 55 },
    { 0, 0 },
    { 0x4cdb3104, 90 },
    { 0xed4b3105, 23 },
    { 0, 0 },
    { 0xa47b3107, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b3111, 61 },
    { 0, 0 },
    { 0xfecb3113, 93 },
    { 0xb7b3114, 90 },
    { 0x8e3b3115, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891b311a, 95 },
    { 0xac1b311b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61b311f, 61 },
    { 0x29db3120, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3b3125, 95 },
    { 0, 0 },
    { 0xae6b3127, 84 },
    { 0, 0 },
    { 0x2b6b3129, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9b312f, 52 },
    { 0x669b312f, 33 },
    { 0, 0 },
    { 0xd02b3132, 90 },
    { 0xfd4b3133, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1b313b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cb3143, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bb3146, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebb314b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242b314f, 34 },
    { 0x5ddb314f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1b3153, 95 },
    { 0xd4ab3154, 61 },
    { 0, 0 },
    { 0xd3cb3156, 55 },
    { 0xe55b3157, 90 },
    { 0xdf6b3158, 33 },
    { 0x3f1b3159, 95 },
    { 0x478b315a, 93 },
    { 0xaefb315a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6b315e, 33 },
    { 0x898b315e, 95 },
    { 0x7c6b3160, 81 },
    { 0x8e1b315e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2db3164, 76 },
    { 0x7f9b3165, 94 },
    { 0x433b3166, 76 },
    { 0x2bb3167, 66 },
    { 0x4b7b3168, 33 },
    { 0, 0 },
    { 0x829b316a, 69 },
    { 0x6cdb316a, 99 },
    { 0xd8bb316a, 80 },
    { 0, 0 },
    { 0xbfdb316e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08b3178, 33 },
    { 0x38cb3178, 23 },
    { 0xed3b317a, 98 },
    { 0xb0ab317b, 61 },
    { 0x5ab317c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b3180, 90 },
    { 0x405b3181, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eb3184, 90 },
    { 0xbb4b3185, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x193b3188, 78 },
    { 0, 0 },
    { 0xc2db318a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74b318d, 95 },
    { 0, 0 },
    { 0x899b318f, 23 },
    { 0x626b3190, 33 },
    { 0x9cfb318f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b3194, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x512b3197, 61 },
    { 0xf0b3198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bb31a0, 90 },
    { 0, 0 },
    { 0x5f5b31a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ab31aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3b31b7, 100 },
    { 0xa87b31b8, 99 },
    { 0xbddb31b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0b31be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbb31c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x750b31c4, 90 },
    { 0x264b31c5, 61 },
    { 0x6f1b31c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9b31cb, 46 },
    { 0, 0 },
    { 0x443b31cd, 34 },
    { 0x66db31ce, 95 },
    { 0xe30b31cf, 33 },
    { 0xc53b31d0, 23 },
    { 0xaadb31d1, 61 },
    { 0, 0 },
    { 0xddfb31d3, 70 },
    { 0, 0 },
    { 0x662b31d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfb31d9, 84 },
    { 0x88eb31da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1deb31e0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fab31e3, 33 },
    { 0x8b7b31e4, 78 },
    { 0x2e7b31e5, 33 },
    { 0xf84b31e4, 24 },
    { 0xac0b31e7, 66 },
    { 0x3eab31e8, 23 },
    { 0xbeeb31e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bb31ed, 33 },
    { 0x9f4b31ee, 90 },
    { 0, 0 },
    { 0xc3db31f0, 90 },
    { 0x2b0b31f1, 83 },
    { 0xe24b31f2, 78 },
    { 0xce2b31f0, 23 },
    { 0x278b31f4, 43 },
    { 0x76db31f4, 77 },
    { 0xdb7b31f6, 68 },
    { 0xedbb31f7, 90 },
    { 0xbfcb31f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492b31fd, 77 },
    { 0xe67b31fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db321a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7b321f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92b3222, 43 },
    { 0, 0 },
    { 0x611b3224, 56 },
    { 0x662b3224, 23 },
    { 0, 0 },
    { 0x993b3227, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x49eb322a, 33 },
    { 0xc82b322b, 90 },
    { 0x9b6b322b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14b322f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbb3234, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92eb323d, 90 },
    { 0x1e2b323d, 55 },
    { 0xee3b323f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b3246, 55 },
    { 0, 0 },
    { 0x296b3248, 90 },
    { 0x607b3249, 40 },
    { 0xaeb324a, 76 },
    { 0xb9db324b, 95 },
    { 0x343b324c, 61 },
    { 0, 0 },
    { 0xd76b324e, 90 },
    { 0xdb2b324f, 33 },
    { 0, 0 },
    { 0xe15b3251, 95 },
    { 0x581b3252, 61 },
    { 0xa69b3252, 95 },
    { 0xd0ab3254, 98 },
    { 0xbcfb3255, 61 },
    { 0x756b3255, 33 },
    { 0xd4ab3257, 23 },
    { 0x23bb3257, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880b3264, 43 },
    { 0x9a9b3265, 66 },
    { 0x90bb3266, 33 },
    { 0x2abb3267, 98 },
    { 0x94fb3266, 90 },
    { 0, 0 },
    { 0xd06b326a, 76 },
    { 0x2fcb326b, 67 },
    { 0xad8b326a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b3270, 13 },
    { 0x1a4b3271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47b327b, 43 },
    { 0, 0 },
    { 0x492b327d, 88 },
    { 0xd80b327d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804b3282, 33 },
    { 0x10eb3283, 61 },
    { 0, 0 },
    { 0xea7b3285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9b3289, 86 },
    { 0x8f0b328a, 61 },
    { 0, 0 },
    { 0x9d3b328c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cb3291, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68b3295, 90 },
    { 0xf57b3296, 90 },
    { 0x908b3297, 95 },
    { 0x890b3295, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x458b329b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cb32a3, 90 },
    { 0x79b32a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x983b32a7, 90 },
    { 0, 0 },
    { 0x2a9b32a9, 90 },
    { 0x2afb32aa, 98 },
    { 0, 0 },
    { 0x994b32ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecb32af, 52 },
    { 0x159b32b0, 61 },
    { 0x486b32b1, 61 },
    { 0x528b32b0, 90 },
    { 0xc08b32b3, 34 },
    { 0, 0 },
    { 0x4d6b32b5, 23 },
    { 0x87fb32b6, 98 },
    { 0x12db32b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2db32ba, 52 },
    { 0x32bb32ba, 33 },
    { 0xc6eb32bb, 95 },
    { 0, 0 },
    { 0xc9ab32be, 69 },
    { 0x6abb32bf, 43 },
    { 0, 0 },
    { 0x7aeb32c1, 61 },
    { 0x2c2b32c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72b32c6, 46 },
    { 0x9e5b32c7, 90 },
    { 0, 0 },
    { 0x1dbb32c9, 90 },
    { 0, 0 },
    { 0xbf3b32cb, 76 },
    { 0x61fb32cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9b32cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4b32d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x80eb32d5, 99 },
    { 0x2f1b32d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x501b32d9, 95 },
    { 0x594b32da, 99 },
    { 0x693b32da, 43 },
    { 0x1d9b32dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796b32e8, 90 },
    { 0xef6b32e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3b32f2, 98 },
    { 0, 0 },
    { 0x915b32f4, 77 },
    { 0x638b32f4, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7b32f9, 43 },
    { 0, 0 },
    { 0x626b32fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eb32ff, 90 },
    { 0x3beb3300, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb3303, 52 },
    { 0x306b3304, 90 },
    { 0x77bb3303, 98 },
    { 0x39db3303, 17 },
    { 0xbf5b3307, 100 },
    { 0x971b3308, 62 },
    { 0, 0 },
    { 0x170b330a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1db330e, 34 },
    { 0xa68b330f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb3318, 90 },
    { 0, 0 },
    { 0x7ddb331a, 36 },
    { 0xb6b331a, 67 },
    { 0, 0 },
    { 0x2b0b331d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17b3321, 98 },
    { 0, 0 },
    { 0x243b3323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bb3326, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b332b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b332f, 93 },
    { 0, 0 },
    { 0xf0cb3331, 34 },
    { 0x36fb3332, 76 },
    { 0xf14b3332, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666b3337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ab333b, 23 },
    { 0xe6cb333c, 99 },
    { 0, 0 },
    { 0x248b333e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fb3345, 23 },
    { 0xbd2b3346, 78 },
    { 0x9e3b3347, 61 },
    { 0x5f0b3345, 98 },
    { 0x3c9b3347, 86 },
    { 0xeedb3345, 85 },
    { 0xec1b334b, 95 },
    { 0x98db334b, 33 },
    { 0x395b3346, 85 },
    { 0xb81b334d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb3352, 95 },
    { 0x9beb3353, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3b3356, 93 },
    { 0x97b3357, 88 },
    { 0xd47b3358, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdb335b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ab335f, 55 },
    { 0x456b3360, 23 },
    { 0, 0 },
    { 0xe7eb3362, 77 },
    { 0xf41b3362, 61 },
    { 0xb4ab3362, 90 },
    { 0x41ab3362, 33 },
    { 0xffcb3363, 33 },
    { 0x727b3367, 43 },
    { 0x63db3367, 90 },
    { 0xe0b3366, 98 },
    { 0xec4b3363, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5b3370, 98 },
    { 0x706b3370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33b3376, 77 },
    { 0x89db3376, 95 },
    { 0, 0 },
    { 0xc70b3379, 95 },
    { 0xa87b3379, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6b3380, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab3383, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90b3387, 68 },
    { 0xa23b3388, 95 },
    { 0, 0 },
    { 0xce3b338a, 88 },
    { 0xea7b338a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b338f, 95 },
    { 0x7b6b3390, 23 },
    { 0xd6fb3391, 88 },
    { 0x8b6b338f, 23 },
    { 0xdd9b3391, 61 },
    { 0x5c7b3393, 90 },
    { 0x441b3395, 93 },
    { 0x93ab3396, 81 },
    { 0xcb4b3395, 90 },
    { 0, 0 },
    { 0x755b3399, 61 },
    { 0xdabb3399, 33 },
    { 0x846b339a, 90 },
    { 0x3f2b339b, 46 },
    { 0x78db339d, 43 },
    { 0x2a7b339e, 33 },
    { 0x3d9b3399, 90 },
    { 0xa6cb33a0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb33a5, 78 },
    { 0x26fb33a6, 52 },
    { 0xe28b33a7, 90 },
    { 0xf62b33a6, 76 },
    { 0x2cdb33a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0b33ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40b33b5, 43 },
    { 0, 0 },
    { 0xb96b33b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4b33bd, 99 },
    { 0xc4bb33bd, 33 },
    { 0xc05b33be, 23 },
    { 0x2d2b33bf, 67 },
    { 0x140b33c1, 98 },
    { 0xbbdb33be, 61 },
    { 0, 0 },
    { 0xccdb33c4, 33 },
    { 0x44fb33c5, 53 },
    { 0xa46b33c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50b33c9, 43 },
    { 0x3dab33c9, 77 },
    { 0, 0 },
    { 0x95bb33cc, 46 },
    { 0, 0 },
    { 0xa65b33ce, 77 },
    { 0x182b33cf, 91 },
    { 0xabbb33cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b33d5, 53 },
    { 0xa2eb33d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb33d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aab33df, 33 },
    { 0x9cdb33df, 98 },
    { 0xabb33e1, 46 },
    { 0xb65b33e2, 90 },
    { 0xaa7b33df, 61 },
    { 0, 0 },
    { 0xe33b33e5, 61 },
    { 0x9f4b33e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfb33eb, 46 },
    { 0xed6b33eb, 23 },
    { 0x26db33ec, 98 },
    { 0xbc9b33ee, 33 },
    { 0, 0 },
    { 0x684b33f0, 55 },
    { 0xf65b33f1, 23 },
    { 0, 0 },
    { 0x212b33f3, 95 },
    { 0, 0 },
    { 0x632b33f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527b33fb, 90 },
    { 0x43cb33fc, 46 },
    { 0xb5eb33fb, 61 },
    { 0x5e2b33fd, 61 },
    { 0x8abb33ff, 66 },
    { 0xb7db33fd, 61 },
    { 0xaceb33fd, 61 },
    { 0x311b3402, 46 },
    { 0xbc7b33fb, 98 },
    { 0, 0 },
    { 0x1f0b3405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7b3409, 98 },
    { 0, 0 },
    { 0x27bb340b, 61 },
    { 0, 0 },
    { 0xfe8b340d, 95 },
    { 0xd28b340e, 33 },
    { 0x484b340e, 23 },
    { 0xbdfb340d, 61 },
    { 0, 0 },
    { 0xa8fb3412, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b3419, 67 },
    { 0, 0 },
    { 0x6ab341b, 77 },
    { 0, 0 },
    { 0x465b341d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bb3420, 43 },
    { 0xd7bb3421, 66 },
    { 0, 0 },
    { 0x1cb3423, 93 },
    { 0x574b3424, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb3427, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4db3430, 66 },
    { 0x581b3431, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x828b3434, 93 },
    { 0x209b3435, 43 },
    { 0, 0 },
    { 0x78b3437, 46 },
    { 0x212b3437, 23 },
    { 0x6cbb3437, 23 },
    { 0, 0 },
    { 0x428b343b, 75 },
    { 0x5e7b343b, 99 },
    { 0x9bfb343d, 61 },
    { 0xc01b343e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74b3441, 23 },
    { 0, 0 },
    { 0x9afb3443, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bb3447, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x34eb344a, 82 },
    { 0xd24b344b, 78 },
    { 0xd06b344c, 93 },
    { 0xfc0b344c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ab3451, 100 },
    { 0xed1b3452, 43 },
    { 0x9c4b3453, 98 },
    { 0xf37b3453, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b3458, 33 },
    { 0, 0 },
    { 0x1d7b345a, 84 },
    { 0, 0 },
    { 0x7bdb345c, 90 },
    { 0x841b345c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bb3462, 23 },
    { 0, 0 },
    { 0x15db3464, 90 },
    { 0x897b3464, 90 },
    { 0, 0 },
    { 0x3eab3467, 78 },
    { 0xbf8b3468, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98b346d, 100 },
    { 0, 0 },
    { 0xbcb346f, 53 },
    { 0x251b3470, 23 },
    { 0x8cb3471, 90 },
    { 0xba6b3472, 85 },
    { 0xebab3473, 40 },
    { 0xa5bb3474, 90 },
    { 0xc23b3474, 55 },
    { 0, 0 },
    { 0x5e3b3477, 34 },
    { 0x48db3477, 90 },
    { 0x34cb3479, 76 },
    { 0x2abb3478, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ceb3483, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b348b, 61 },
    { 0, 0 },
    { 0x742b348d, 76 },
    { 0x59db348e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96b3491, 81 },
    { 0x49db3492, 95 },
    { 0, 0 },
    { 0xbeab3494, 33 },
    { 0xfcab3495, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb3499, 66 },
    { 0xce1b3499, 55 },
    { 0xadbb349a, 46 },
    { 0x219b349b, 61 },
    { 0x2cdb349d, 77 },
    { 0xcdfb349e, 33 },
    { 0x6efb349f, 80 },
    { 0xf39b349f, 76 },
    { 0x539b34a1, 90 },
    { 0x168b34a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ab34a8, 23 },
    { 0x922b34a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96b34ac, 95 },
    { 0xd8b34ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845b34b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2b34bb, 43 },
    { 0, 0 },
    { 0x762b34bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1b34c1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4b34c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1b34cc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x457b34cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb34d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcb34d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58b34dc, 78 },
    { 0, 0 },
    { 0x494b34de, 33 },
    { 0x533b34df, 93 },
    { 0xa12b34de, 95 },
    { 0, 0 },
    { 0x6b7b34e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cb34e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7b34e8, 52 },
    { 0xc5eb34e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745b34ed, 55 },
    { 0, 0 },
    { 0x28cb34ef, 33 },
    { 0x2a8b34f0, 99 },
    { 0x6a9b34ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b34f5, 87 },
    { 0, 0 },
    { 0x611b34f7, 88 },
    { 0xd94b34f7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06b34fb, 90 },
    { 0, 0 },
    { 0x37b34fd, 33 },
    { 0x159b34fe, 61 },
    { 0xf1ab34fe, 87 },
    { 0, 0 },
    { 0xf57b3501, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48b3504, 34 },
    { 0x571b3504, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beb3508, 61 },
    { 0xc5ab3508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x433b350c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494b3513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ab3517, 90 },
    { 0x9fbb3518, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b3520, 99 },
    { 0, 0 },
    { 0x805b3522, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5b352a, 77 },
    { 0x119b352a, 43 },
    { 0x58eb352c, 33 },
    { 0x8bdb352d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705b3535, 68 },
    { 0xc47b3535, 90 },
    { 0, 0 },
    { 0x5eeb3538, 43 },
    { 0x80b3539, 61 },
    { 0, 0 },
    { 0x4f8b353b, 61 },
    { 0x44eb353c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7b3540, 93 },
    { 0x2a7b3541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1b3548, 46 },
    { 0xc84b3549, 55 },
    { 0x225b354a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3b354d, 90 },
    { 0x84db354e, 36 },
    { 0x276b354e, 95 },
    { 0xd60b354e, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6b3554, 33 },
    { 0, 0 },
    { 0xbf9b3556, 46 },
    { 0x67bb3556, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3b355a, 90 },
    { 0xc1fb355a, 46 },
    { 0x2d9b355a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7b3560, 33 },
    { 0x919b3561, 85 },
    { 0x4fbb3561, 75 },
    { 0x10fb3562, 33 },
    { 0x7d2b3564, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2b356c, 61 },
    { 0x7afb356d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bb3575, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3b3578, 90 },
    { 0, 0 },
    { 0x2a8b357a, 33 },
    { 0x2e2b357b, 90 },
    { 0x229b357b, 95 },
    { 0x329b357c, 100 },
    { 0x437b357d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4b3581, 75 },
    { 0, 0 },
    { 0x71cb3583, 23 },
    { 0x9cfb3584, 77 },
    { 0xbc5b3584, 90 },
    { 0x200b3584, 66 },
    { 0, 0 },
    { 0x96fb3588, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b358b, 46 },
    { 0x72eb358b, 91 },
    { 0x7dcb358c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569b3594, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fb3598, 34 },
    { 0, 0 },
    { 0xbddb359a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffb359e, 67 },
    { 0x21db359f, 52 },
    { 0x98db35a0, 90 },
    { 0x50cb35a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bb35a5, 23 },
    { 0x46fb35a6, 46 },
    { 0x97ab35a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b35aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b35ae, 88 },
    { 0x611b35af, 23 },
    { 0, 0 },
    { 0x351b35b1, 78 },
    { 0, 0 },
    { 0x25cb35b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eb35b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32b35bd, 90 },
    { 0x1b1b35be, 91 },
    { 0xd23b35bf, 93 },
    { 0xba0b35be, 69 },
    { 0, 0 },
    { 0xe44b35c2, 23 },
    { 0xd79b35c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934b35cb, 90 },
    { 0, 0 },
    { 0x311b35cd, 55 },
    { 0, 0 },
    { 0x3f1b35cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bb35d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61b35d5, 76 },
    { 0xd8b35d5, 33 },
    { 0, 0 },
    { 0x43cb35d8, 98 },
    { 0, 0 },
    { 0x61b35da, 93 },
    { 0xc97b35db, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b35de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8db35e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65b35e4, 98 },
    { 0x85ab35e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bb35ec, 17 },
    { 0, 0 },
    { 0x6fbb35ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236b35fd, 91 },
    { 0x5dcb35fd, 90 },
    { 0xdf8b35ff, 90 },
    { 0x7d6b35fe, 76 },
    { 0xe30b35ff, 98 },
    { 0x5ecb3601, 66 },
    { 0xdafb3603, 93 },
    { 0, 0 },
    { 0x56eb3605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21b360a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7b360e, 46 },
    { 0, 0 },
    { 0x9bb3610, 61 },
    { 0x5e1b3611, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fb3614, 100 },
    { 0xb1db3614, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738b3619, 33 },
    { 0, 0 },
    { 0x696b361b, 77 },
    { 0x41eb361b, 43 },
    { 0, 0 },
    { 0x2afb361e, 40 },
    { 0xd4bb361f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60b3622, 17 },
    { 0x3cbb3622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6b3627, 23 },
    { 0x401b3628, 52 },
    { 0x616b3627, 98 },
    { 0xf84b362a, 4 },
    { 0x257b3628, 34 },
    { 0xcc6b3627, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ab3631, 52 },
    { 0, 0 },
    { 0x245b3633, 61 },
    { 0x114b3634, 33 },
    { 0x442b3635, 90 },
    { 0, 0 },
    { 0x74eb3637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90b3640, 33 },
    { 0x3eeb3641, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2b3651, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3b365c, 76 },
    { 0xe2b365c, 22 },
    { 0x654b365e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b3661, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feb3665, 33 },
    { 0x3e1b3666, 76 },
    { 0x697b3665, 55 },
    { 0, 0 },
    { 0x135b3669, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400b366f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fb3679, 91 },
    { 0x340b367a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x322b367d, 55 },
    { 0x2f7b367d, 95 },
    { 0, 0 },
    { 0xcb5b3680, 61 },
    { 0x824b3681, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfb3684, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86b3688, 81 },
    { 0x7dab3688, 34 },
    { 0x6a4b3689, 98 },
    { 0xff2b368b, 55 },
    { 0x6c1b368c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edb3690, 77 },
    { 0xc0eb3691, 93 },
    { 0x5b3692, 61 },
    { 0x779b3691, 23 },
    { 0xd49b3694, 90 },
    { 0x710b3695, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x739b3698, 33 },
    { 0x7dab3699, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4b36a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fb36a4, 66 },
    { 0, 0 },
    { 0xb40b36a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b36a9, 90 },
    { 0x790b36aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eab36af, 90 },
    { 0xe2bb36b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fb36b8, 90 },
    { 0x606b36b9, 34 },
    { 0xe4eb36ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664b36c0, 81 },
    { 0x25bb36c1, 91 },
    { 0x8a5b36c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fb36ce, 84 },
    { 0x756b36cf, 90 },
    { 0, 0 },
    { 0xfe5b36d1, 90 },
    { 0x36db36d1, 95 },
    { 0x1d6b36d3, 95 },
    { 0xe69b36d4, 90 },
    { 0x9edb36d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b36de, 77 },
    { 0x2cdb36df, 55 },
    { 0x26eb36e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5b36e5, 33 },
    { 0, 0 },
    { 0x61eb36e7, 90 },
    { 0, 0 },
    { 0x166b36e9, 83 },
    { 0x3f1b36e9, 55 },
    { 0, 0 },
    { 0xbebb36ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cb36f0, 93 },
    { 0x83fb36f1, 61 },
    { 0x87ab36f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12b36fa, 93 },
    { 0, 0 },
    { 0x927b36fc, 34 },
    { 0x638b36fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bb3700, 39 },
    { 0x9a1b3701, 99 },
    { 0, 0 },
    { 0x2f1b3703, 55 },
    { 0, 0 },
    { 0x4db3705, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffb3708, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62b370f, 84 },
    { 0, 0 },
    { 0xb91b3711, 68 },
    { 0xf7db3712, 75 },
    { 0xb68b3711, 98 },
    { 0xbaab3713, 90 },
    { 0x751b3714, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25b3718, 85 },
    { 0xa0b3719, 93 },
    { 0, 0 },
    { 0x541b371b, 51 },
    { 0x3a3b371c, 55 },
    { 0xa88b371c, 76 },
    { 0xfcb371b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1b3723, 61 },
    { 0, 0 },
    { 0x1deb3725, 55 },
    { 0x6b3b3725, 33 },
    { 0xf2fb3727, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bb372d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8db3731, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b3737, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eb373a, 95 },
    { 0x778b373b, 23 },
    { 0x233b373c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623b3740, 19 },
    { 0xf39b3741, 95 },
    { 0x3d0b3742, 95 },
    { 0x411b3743, 77 },
    { 0x6afb3742, 52 },
    { 0xa68b3745, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b374c, 90 },
    { 0xdfcb374c, 23 },
    { 0, 0 },
    { 0x1f1b374f, 33 },
    { 0xbf4b3750, 61 },
    { 0, 0 },
    { 0x962b3752, 61 },
    { 0xaa3b3753, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3b3756, 34 },
    { 0xb7b3757, 23 },
    { 0, 0 },
    { 0x549b3759, 76 },
    { 0xccdb375a, 33 },
    { 0xddbb375b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1b3763, 87 },
    { 0xeafb3763, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb3768, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ab3773, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b377a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x816b377d, 81 },
    { 0x17cb377d, 98 },
    { 0, 0 },
    { 0xcc0b3780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfb3784, 67 },
    { 0x89db3785, 43 },
    { 0x295b3784, 61 },
    { 0x16fb3787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afb3790, 33 },
    { 0x452b3791, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163b3799, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8b37a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61eb37aa, 98 },
    { 0xc2bb37aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b37ae, 55 },
    { 0, 0 },
    { 0x97db37b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6b37b6, 98 },
    { 0, 0 },
    { 0xa4db37b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710b37bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3b37c2, 85 },
    { 0x70b37c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacab37c7, 90 },
    { 0xf20b37c8, 40 },
    { 0x70ab37c9, 95 },
    { 0, 0 },
    { 0x3ecb37cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bb37d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb37d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1b37df, 95 },
    { 0x7c3b37df, 83 },
    { 0x76eb37e1, 92 },
    { 0, 0 },
    { 0x656b37e3, 93 },
    { 0x30eb37e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb37e9, 90 },
    { 0xf6fb37e9, 81 },
    { 0, 0 },
    { 0xcd2b37ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4b37f0, 23 },
    { 0, 0 },
    { 0xdbab37f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beb37f7, 33 },
    { 0x20b37f7, 55 },
    { 0xa8bb37f9, 33 },
    { 0, 0 },
    { 0x95b37fb, 34 },
    { 0x46cb37fc, 76 },
    { 0, 0 },
    { 0x41eb37fe, 90 },
    { 0x12bb37fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebb3802, 65 },
    { 0x9cdb3803, 77 },
    { 0, 0 },
    { 0x43eb3805, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b3808, 91 },
    { 0, 0 },
    { 0x6b3b380a, 34 },
    { 0x7fbb380b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2b380e, 49 },
    { 0x2f8b380f, 91 },
    { 0, 0 },
    { 0x9b0b3811, 76 },
    { 0x9deb3812, 95 },
    { 0x94fb3813, 76 },
    { 0xa66b3814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fb3817, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0b381c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74b381f, 95 },
    { 0, 0 },
    { 0x840b3821, 67 },
    { 0, 0 },
    { 0x748b3823, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55b3826, 90 },
    { 0x513b3827, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x924b382a, 23 },
    { 0x47eb382b, 33 },
    { 0xe4fb382c, 90 },
    { 0xf22b382b, 43 },
    { 0, 0 },
    { 0x563b382f, 98 },
    { 0x81bb3830, 90 },
    { 0xbe4b3831, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b3836, 73 },
    { 0xbe0b3837, 33 },
    { 0, 0 },
    { 0x4e3b3839, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b383e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b3843, 100 },
    { 0x667b3844, 61 },
    { 0, 0 },
    { 0xf15b3846, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bb3849, 61 },
    { 0, 0 },
    { 0x9dcb384b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eb3851, 61 },
    { 0x479b3852, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83b3855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bb385a, 98 },
    { 0xe8eb385a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15b385e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2b3863, 90 },
    { 0xda2b3864, 23 },
    { 0x779b3864, 61 },
    { 0x5eab3865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cb386e, 52 },
    { 0xffbb386f, 61 },
    { 0, 0 },
    { 0x257b3871, 76 },
    { 0x6dfb3872, 55 },
    { 0x761b3873, 90 },
    { 0xfdb3873, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0b3877, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cb387a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545b387e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0b3883, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b388a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429b388e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39db3893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18db3896, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x714b3899, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944b38a2, 23 },
    { 0x13db38a3, 95 },
    { 0x423b38a4, 61 },
    { 0xcfdb38a5, 90 },
    { 0xd4cb38a6, 55 },
    { 0, 0 },
    { 0x3adb38a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ab38b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311b38b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b38b8, 55 },
    { 0xb77b38b8, 87 },
    { 0xfbab38ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cb38bf, 17 },
    { 0xe77b38c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7b38c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb38c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b38cb, 55 },
    { 0x994b38cc, 95 },
    { 0, 0 },
    { 0x2cb38ce, 33 },
    { 0, 0 },
    { 0xec0b38d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842b38d4, 96 },
    { 0x652b38d5, 95 },
    { 0xb99b38d6, 55 },
    { 0x4e6b38d7, 99 },
    { 0x9fb38d7, 90 },
    { 0x365b38d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96b38dc, 61 },
    { 0, 0 },
    { 0xf97b38de, 56 },
    { 0x695b38df, 43 },
    { 0, 0 },
    { 0x6efb38e1, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30b38e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x256b38e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159b38ec, 23 },
    { 0xf86b38ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698b38f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eb38f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x971b38fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2b38fe, 95 },
    { 0xb5eb38ff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723b3904, 93 },
    { 0x82ab3904, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2b3909, 78 },
    { 0, 0 },
    { 0x257b390b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b390e, 84 },
    { 0x48db390f, 61 },
    { 0x4c9b390e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b3913, 46 },
    { 0, 0 },
    { 0xe8b3915, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb391d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06b3926, 55 },
    { 0x88fb3927, 96 },
    { 0, 0 },
    { 0xe77b3929, 98 },
    { 0xe73b3929, 34 },
    { 0x4cb392b, 55 },
    { 0x12cb392c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ab392f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9b3932, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140b3937, 90 },
    { 0x9c9b3938, 93 },
    { 0x8d8b3939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x183b393c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951b3943, 95 },
    { 0x544b3943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10b3947, 33 },
    { 0, 0 },
    { 0xe29b3949, 76 },
    { 0, 0 },
    { 0x421b394b, 23 },
    { 0, 0 },
    { 0x2e6b394d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02b3950, 90 },
    { 0x3c1b3951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfb3959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55b395c, 93 },
    { 0xec8b395d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0b3961, 33 },
    { 0xbcb3961, 85 },
    { 0, 0 },
    { 0x7efb3964, 98 },
    { 0, 0 },
    { 0xf92b3966, 33 },
    { 0x28db3966, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefb396b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb396e, 61 },
    { 0x85eb396e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84b3977, 61 },
    { 0xb36b3977, 17 },
    { 0, 0 },
    { 0x1f4b397a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4b3981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9db3985, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab3988, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db398b, 95 },
    { 0, 0 },
    { 0x7f9b398d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb3991, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59b3997, 33 },
    { 0x706b3998, 76 },
    { 0xdd2b3998, 55 },
    { 0, 0 },
    { 0x269b399b, 23 },
    { 0, 0 },
    { 0x6feb399d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cb39a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb39a5, 68 },
    { 0, 0 },
    { 0x3a0b39a7, 100 },
    { 0x7c0b39a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ab39ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96b39b2, 33 },
    { 0, 0 },
    { 0x420b39b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4b39b7, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x285b39ba, 33 },
    { 0x52cb39ba, 55 },
    { 0xee3b39ba, 92 },
    { 0, 0 },
    { 0xa10b39be, 23 },
    { 0xbe1b39be, 90 },
    { 0x3ffb39bf, 23 },
    { 0xfd7b39c1, 23 },
    { 0x1f1b39c2, 43 },
    { 0xc14b39be, 55 },
    { 0xb90b39c2, 33 },
    { 0, 0 },
    { 0x916b39c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bb39ca, 98 },
    { 0, 0 },
    { 0x11db39cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x816b39cf, 95 },
    { 0xff8b39d0, 95 },
    { 0x192b39d1, 95 },
    { 0x101b39d2, 88 },
    { 0xbbfb39d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bab39d8, 98 },
    { 0, 0 },
    { 0x32db39da, 90 },
    { 0xd4cb39db, 23 },
    { 0x607b39dc, 90 },
    { 0x433b39dd, 91 },
    { 0x9e9b39dd, 52 },
    { 0x235b39df, 43 },
    { 0xd2eb39da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209b39ed, 38 },
    { 0, 0 },
    { 0x900b39ef, 90 },
    { 0x8e9b39ef, 90 },
    { 0xe6cb39f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cb39f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b3a03, 33 },
    { 0, 0 },
    { 0x885b3a05, 76 },
    { 0x8cfb3a05, 61 },
    { 0, 0 },
    { 0xea3b3a08, 33 },
    { 0, 0 },
    { 0x64fb3a0a, 90 },
    { 0xf4fb3a0b, 85 },
    { 0xa49b3a0a, 90 },
    { 0x87cb3a0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b3a10, 90 },
    { 0, 0 },
    { 0xd5eb3a12, 90 },
    { 0, 0 },
    { 0x6d0b3a14, 43 },
    { 0xb12b3a15, 69 },
    { 0x6a2b3a16, 46 },
    { 0x10b3a17, 90 },
    { 0xe34b3a14, 34 },
    { 0, 0 },
    { 0x53db3a1a, 98 },
    { 0xa57b3a1b, 52 },
    { 0xb72b3a1b, 90 },
    { 0x4d8b3a1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ab3a23, 90 },
    { 0x4bdb3a24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88b3a27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cb3a2c, 76 },
    { 0x84fb3a2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b3a3f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bb3a4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b3a50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53b3a53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6b3a59, 46 },
    { 0xd81b3a59, 33 },
    { 0, 0 },
    { 0xce0b3a5c, 99 },
    { 0x4aab3a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203b3a63, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70b3a6b, 90 },
    { 0x959b3a6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239b3a71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ab3a74, 76 },
    { 0x5f4b3a75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8b3a7d, 76 },
    { 0, 0 },
    { 0xa8ab3a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b3a85, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x92b3a88, 43 },
    { 0xe5db3a88, 61 },
    { 0, 0 },
    { 0x53db3a8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2db3a8e, 68 },
    { 0x766b3a8f, 23 },
    { 0x96ab3a90, 69 },
    { 0, 0 },
    { 0x978b3a92, 33 },
    { 0x9feb3a93, 90 },
    { 0x762b3a94, 33 },
    { 0x839b3a94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b3a98, 40 },
    { 0xe4eb3a99, 98 },
    { 0x263b3a9a, 90 },
    { 0xdfbb3a9a, 98 },
    { 0x5adb3a9c, 33 },
    { 0, 0 },
    { 0xe4fb3a9e, 83 },
    { 0xd05b3a9f, 52 },
    { 0xafab3aa0, 76 },
    { 0x2a9b3aa1, 33 },
    { 0x138b3aa2, 93 },
    { 0x746b3aa0, 61 },
    { 0x420b3aa4, 90 },
    { 0x485b3aa4, 33 },
    { 0xea5b3aa6, 33 },
    { 0x5a3b3aa4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cb3aae, 98 },
    { 0x266b3aaf, 76 },
    { 0, 0 },
    { 0xe6ab3ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aab3ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96b3abc, 90 },
    { 0x443b3abd, 61 },
    { 0x2b8b3abd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bb3ac2, 61 },
    { 0xc20b3ac3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b3ac8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80b3acb, 85 },
    { 0xa1fb3acc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cb3ad3, 34 },
    { 0xc7eb3ad4, 33 },
    { 0xc82b3ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eb3ad8, 23 },
    { 0x491b3ad9, 23 },
    { 0x486b3ad8, 90 },
    { 0, 0 },
    { 0x235b3adc, 90 },
    { 0xfb3adc, 36 },
    { 0, 0 },
    { 0xe25b3adf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeeb3ae5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb3ae8, 78 },
    { 0, 0 },
    { 0x5cab3aea, 90 },
    { 0xd4b3aeb, 55 },
    { 0x41db3aec, 33 },
    { 0xe34b3aec, 100 },
    { 0xaacb3aee, 95 },
    { 0x6a7b3aef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb3af3, 98 },
    { 0xee7b3af4, 95 },
    { 0x986b3af5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cb3afd, 46 },
    { 0x2b8b3afe, 98 },
    { 0x619b3afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337b3b03, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284b3b08, 76 },
    { 0x39bb3b08, 33 },
    { 0xb7bb3b0a, 91 },
    { 0xf2cb3b0a, 93 },
    { 0x7bdb3b0c, 76 },
    { 0x680b3b0a, 85 },
    { 0x8c0b3b0e, 61 },
    { 0x7e8b3b0e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8b3b14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24b3b17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b3b1c, 98 },
    { 0, 0 },
    { 0xa15b3b1e, 33 },
    { 0, 0 },
    { 0xb4ab3b20, 66 },
    { 0xf1ab3b21, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7b3b29, 76 },
    { 0, 0 },
    { 0x55bb3b2b, 66 },
    { 0, 0 },
    { 0xb0cb3b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242b3b31, 52 },
    { 0xa50b3b32, 90 },
    { 0, 0 },
    { 0x8edb3b34, 95 },
    { 0xb8db3b34, 91 },
    { 0x7beb3b35, 23 },
    { 0xce5b3b37, 17 },
    { 0xb0ab3b38, 98 },
    { 0x8f3b3b39, 40 },
    { 0xb8cb3b3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33eb3b3e, 78 },
    { 0, 0 },
    { 0x730b3b40, 61 },
    { 0x86db3b41, 23 },
    { 0xf35b3b40, 55 },
    { 0x50b3b43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467b3b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b3b4d, 33 },
    { 0, 0 },
    { 0x65ab3b4f, 95 },
    { 0, 0 },
    { 0x8dfb3b51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bb3b5a, 34 },
    { 0x483b3b5b, 90 },
    { 0xe3fb3b5c, 84 },
    { 0x53cb3b5d, 53 },
    { 0x4b0b3b5e, 33 },
    { 0, 0 },
    { 0xd17b3b60, 61 },
    { 0x2fbb3b61, 23 },
    { 0x274b3b62, 84 },
    { 0xef6b3b61, 43 },
    { 0xcaeb3b64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b3b68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb3b73, 90 },
    { 0xf35b3b74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85b3b78, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8b3b7b, 36 },
    { 0x564b3b7c, 95 },
    { 0xf61b3b7d, 46 },
    { 0, 0 },
    { 0xb71b3b7f, 88 },
    { 0, 0 },
    { 0x81bb3b81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db3b8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db3b8e, 76 },
    { 0x11fb3b8e, 33 },
    { 0x9b8b3b8e, 90 },
    { 0xffdb3b91, 85 },
    { 0x95ab3b92, 91 },
    { 0, 0 },
    { 0xac6b3b94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb3b9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5b3b9d, 101 },
    { 0x7d8b3b9e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bb3ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ab3ba8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ab3bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb3baf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03b3bb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6b3bbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465b3bc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07b3bca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeb3bd1, 95 },
    { 0xcaeb3bd2, 95 },
    { 0x23db3bd3, 33 },
    { 0x903b3bd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514b3bd8, 90 },
    { 0, 0 },
    { 0xb03b3bda, 33 },
    { 0x976b3bdb, 52 },
    { 0xeebb3bdc, 100 },
    { 0x576b3bdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb3be2, 23 },
    { 0x9e0b3be3, 68 },
    { 0x667b3be4, 98 },
    { 0x371b3be5, 100 },
    { 0xaa9b3be6, 55 },
    { 0xe4fb3be7, 77 },
    { 0x31cb3be6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ab3beb, 61 },
    { 0, 0 },
    { 0x23eb3bed, 43 },
    { 0, 0 },
    { 0x4e5b3bef, 95 },
    { 0x194b3bf0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ab3bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4b3bf9, 33 },
    { 0x774b3bfa, 90 },
    { 0, 0 },
    { 0x724b3bfc, 33 },
    { 0, 0 },
    { 0x164b3bfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcb3c03, 95 },
    { 0x74db3c04, 90 },
    { 0x5bcb3c04, 23 },
    { 0, 0 },
    { 0xc2cb3c07, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb3c0b, 90 },
    { 0x80db3c0c, 90 },
    { 0x48db3c0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x859b3c10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32b3c13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceb3c16, 33 },
    { 0x23db3c17, 67 },
    { 0x550b3c16, 95 },
    { 0x82fb3c17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadb3c1c, 95 },
    { 0x2bab3c1d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875b3c26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9eb3c2c, 85 },
    { 0, 0 },
    { 0xa26b3c2e, 78 },
    { 0xa68b3c2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb3c32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9b3c36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b3c3a, 52 },
    { 0, 0 },
    { 0xdeb3c3c, 90 },
    { 0, 0 },
    { 0x4eeb3c3e, 34 },
    { 0x9b9b3c3e, 33 },
    { 0x419b3c3f, 76 },
    { 0xb91b3c3f, 93 },
    { 0x67eb3c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db3c47, 98 },
    { 0, 0 },
    { 0xfddb3c49, 59 },
    { 0xbd7b3c4a, 65 },
    { 0, 0 },
    { 0xf8fb3c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9b3c53, 76 },
    { 0, 0 },
    { 0x543b3c55, 61 },
    { 0, 0 },
    { 0x2e4b3c57, 90 },
    { 0x666b3c58, 95 },
    { 0xd62b3c59, 33 },
    { 0x72fb3c5a, 52 },
    { 0x1dfb3c5a, 88 },
    { 0x6e4b3c5a, 61 },
    { 0x4b7b3c5d, 61 },
    { 0x541b3c5a, 98 },
    { 0x705b3c5f, 90 },
    { 0x66cb3c57, 76 },
    { 0x87bb3c61, 36 },
    { 0x391b3c62, 95 },
    { 0x34db3c61, 76 },
    { 0xe68b3c59, 98 },
    { 0x253b3c61, 23 },
    { 0x6deb3c66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39b3c6f, 46 },
    { 0x7a3b3c70, 33 },
    { 0x16fb3c71, 67 },
    { 0x7cfb3c71, 33 },
    { 0xb68b3c73, 55 },
    { 0x5c8b3c71, 37 },
    { 0x41ab3c75, 90 },
    { 0xc8fb3c76, 90 },
    { 0xba6b3c76, 43 },
    { 0, 0 },
    { 0xca5b3c79, 24 },
    { 0, 0 },
    { 0xdddb3c7b, 95 },
    { 0, 0 },
    { 0x8afb3c7d, 61 },
    { 0, 0 },
    { 0x5d4b3c7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bb3c82, 23 },
    { 0, 0 },
    { 0xaa2b3c84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedb3c8f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffb3c92, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x856b3c95, 67 },
    { 0x4bcb3c96, 40 },
    { 0, 0 },
    { 0xc23b3c98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73b3c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4b3ca6, 23 },
    { 0, 0 },
    { 0x5f0b3ca8, 66 },
    { 0x753b3ca9, 93 },
    { 0x8deb3caa, 95 },
    { 0x2ab3cab, 100 },
    { 0x8a4b3cab, 76 },
    { 0x8bab3cac, 34 },
    { 0x37fb3cae, 46 },
    { 0xee9b3caf, 99 },
    { 0xc62b3cb0, 68 },
    { 0x2f3b3ca9, 55 },
    { 0, 0 },
    { 0xcf9b3cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472b3cbf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4b3cc3, 76 },
    { 0xe0bb3cc3, 66 },
    { 0x727b3cc5, 90 },
    { 0, 0 },
    { 0xd88b3cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5b3cca, 76 },
    { 0x7f9b3ccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdb3cd8, 88 },
    { 0xf26b3cd8, 90 },
    { 0, 0 },
    { 0xd65b3cdb, 61 },
    { 0x972b3cdb, 77 },
    { 0, 0 },
    { 0xbb5b3cde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcb3ce2, 78 },
    { 0xb1eb3ce2, 34 },
    { 0, 0 },
    { 0xba4b3ce5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cb3ce8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87b3ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb3cf3, 52 },
    { 0x32eb3cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b3d00, 95 },
    { 0, 0 },
    { 0x89eb3d02, 23 },
    { 0x581b3d02, 61 },
    { 0x9e2b3d04, 55 },
    { 0x7deb3d05, 33 },
    { 0xfbb3d06, 33 },
    { 0xef9b3d06, 90 },
    { 0x79b3d06, 55 },
    { 0xb28b3d02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8db3d11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b3d1f, 33 },
    { 0x8a5b3d20, 78 },
    { 0, 0 },
    { 0x30fb3d22, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b3d25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb3d28, 52 },
    { 0x660b3d28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6b3d2d, 90 },
    { 0, 0 },
    { 0xb3bb3d2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8b3d34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2b3d37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fb3d3d, 33 },
    { 0, 0 },
    { 0x69eb3d3f, 87 },
    { 0xa60b3d40, 95 },
    { 0, 0 },
    { 0x1ebb3d42, 81 },
    { 0xba7b3d42, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6b3d46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b3d4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5b3d50, 90 },
    { 0x972b3d51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6eb3d57, 90 },
    { 0xadbb3d58, 90 },
    { 0xa44b3d59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceb3d5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x95b3d5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4b3d66, 91 },
    { 0, 0 },
    { 0x550b3d68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b3d6c, 98 },
    { 0xb87b3d6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6b3d70, 95 },
    { 0, 0 },
    { 0x864b3d72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5b3d77, 95 },
    { 0x7ddb3d78, 90 },
    { 0, 0 },
    { 0x978b3d7a, 33 },
    { 0xc4cb3d7b, 33 },
    { 0x915b3d7a, 90 },
    { 0, 0 },
    { 0x4b6b3d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419b3d87, 98 },
    { 0xe13b3d87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5b3d8f, 98 },
    { 0x64fb3d90, 78 },
    { 0, 0 },
    { 0x344b3d92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b3d98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ab3d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14b3d9e, 33 },
    { 0x6a3b3d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15b3da4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bb3dad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfb3db6, 90 },
    { 0xcbcb3db6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492b3dc0, 90 },
    { 0x43cb3dc1, 75 },
    { 0xe02b3dc2, 55 },
    { 0x760b3dc3, 33 },
    { 0, 0 },
    { 0xd00b3dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x655b3dc8, 78 },
    { 0x890b3dc9, 23 },
    { 0xa94b3dca, 23 },
    { 0x4bab3dcb, 90 },
    { 0x66db3dc8, 78 },
    { 0x623b3dcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b3dd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323b3dd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddb3dd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4b3ddc, 55 },
    { 0, 0 },
    { 0xa61b3dde, 65 },
    { 0xc1ab3ddf, 46 },
    { 0xc32b3de0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1b3de3, 90 },
    { 0x95cb3de3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ab3de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08b3def, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ab3df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cb3df8, 66 },
    { 0, 0 },
    { 0xc2eb3dfa, 98 },
    { 0xf53b3dfb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208b3e07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdb3e1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2b3e24, 65 },
    { 0x81b3e24, 55 },
    { 0xf67b3e26, 23 },
    { 0, 0 },
    { 0x61b3e28, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x545b3e2b, 23 },
    { 0x216b3e2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6b3e32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b3e3d, 90 },
    { 0x6ffb3e3d, 84 },
    { 0x686b3e3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x84db3e42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51b3e49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccb3e4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b3e52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0b3e5e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39b3e64, 100 },
    { 0x148b3e65, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfb3e6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb3e6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb3e75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ab3e78, 90 },
    { 0x2cbb3e79, 76 },
    { 0x7cab3e7a, 98 },
    { 0, 0 },
    { 0x9dfb3e7c, 66 },
    { 0, 0 },
    { 0x86ab3e7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b3e89, 23 },
    { 0xbfcb3e8a, 61 },
    { 0xec7b3e8b, 66 },
    { 0xab3e8a, 76 },
    { 0x1a1b3e8a, 33 },
    { 0xb84b3e8a, 33 },
    { 0xae7b3e8f, 23 },
    { 0xaabb3e8d, 23 },
    { 0xd63b3e91, 90 },
    { 0x84db3e91, 93 },
    { 0x429b3e93, 98 },
    { 0xa93b3e8f, 61 },
    { 0, 0 },
    { 0xb86b3e96, 95 },
    { 0x4b3b3e96, 61 },
    { 0x637b3e98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9b3e9b, 34 },
    { 0xc85b3e9b, 61 },
    { 0x7f3b3e9d, 61 },
    { 0x12bb3e9b, 55 },
    { 0, 0 },
    { 0x251b3ea0, 23 },
    { 0xb11b3ea1, 76 },
    { 0xc38b3ea1, 93 },
    { 0x6f5b3ea2, 33 },
    { 0xe98b3ea1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722b3eaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635b3eaf, 76 },
    { 0xf4fb3eb0, 61 },
    { 0xe69b3eb1, 33 },
    { 0xc94b3eb2, 90 },
    { 0xc59b3eb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab3ebb, 33 },
    { 0xc8eb3ebc, 81 },
    { 0xb34b3ebd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb3ec4, 93 },
    { 0x9a6b3ec5, 98 },
    { 0xe9ab3ec5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb3ecb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b3ed1, 61 },
    { 0xa6b3ed2, 61 },
    { 0x747b3ed2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6b3ed6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x963b3ed9, 76 },
    { 0xad4b3eda, 55 },
    { 0x490b3eda, 95 },
    { 0, 0 },
    { 0xab8b3edd, 95 },
    { 0x841b3ede, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afb3ee1, 90 },
    { 0x1f6b3ee1, 90 },
    { 0, 0 },
    { 0x21eb3ee4, 98 },
    { 0, 0 },
    { 0x981b3ee6, 84 },
    { 0x61eb3ee6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb3eef, 78 },
    { 0, 0 },
    { 0x314b3ef1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8b3ef5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508b3ef9, 34 },
    { 0, 0 },
    { 0x2ccb3efb, 33 },
    { 0x4c5b3efc, 22 },
    { 0, 0 },
    { 0x65ab3efe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7b3f06, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b3f0d, 98 },
    { 0x64db3f0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cb3f11, 33 },
    { 0x222b3f12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bb3f18, 98 },
    { 0x92b3f18, 91 },
    { 0xb79b3f1a, 66 },
    { 0x3db3f19, 23 },
    { 0xf8b3f1c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820b3f20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30b3f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7b3f2d, 43 },
    { 0x21cb3f2e, 33 },
    { 0x958b3f2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x499b3f32, 43 },
    { 0x66cb3f33, 43 },
    { 0, 0 },
    { 0xe5ab3f35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7b3f38, 23 },
    { 0, 0 },
    { 0x99cb3f3a, 90 },
    { 0xab5b3f3a, 84 },
    { 0, 0 },
    { 0xe3b3f3d, 91 },
    { 0x473b3f3e, 43 },
    { 0xf5cb3f3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41b3f44, 33 },
    { 0x899b3f45, 101 },
    { 0, 0 },
    { 0xc79b3f47, 33 },
    { 0x2f2b3f48, 90 },
    { 0x7a7b3f49, 33 },
    { 0, 0 },
    { 0x648b3f4b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114b3f51, 69 },
    { 0x674b3f51, 33 },
    { 0x914b3f53, 48 },
    { 0x3c7b3f53, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x338b3f57, 55 },
    { 0x232b3f58, 98 },
    { 0, 0 },
    { 0x757b3f5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabb3f5d, 33 },
    { 0, 0 },
    { 0x87bb3f5f, 67 },
    { 0x37bb3f60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eb3f63, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65b3f66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bb3f6a, 98 },
    { 0, 0 },
    { 0xe9eb3f6c, 98 },
    { 0x4e8b3f6d, 66 },
    { 0x113b3f6c, 33 },
    { 0x3db3f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3db3f76, 76 },
    { 0, 0 },
    { 0x70fb3f78, 84 },
    { 0x470b3f79, 43 },
    { 0, 0 },
    { 0x332b3f7b, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffb3f7f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26b3f82, 100 },
    { 0xcf5b3f82, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5b3f86, 76 },
    { 0x415b3f87, 76 },
    { 0x7abb3f88, 46 },
    { 0x826b3f89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4b3f8c, 61 },
    { 0, 0 },
    { 0x412b3f8e, 76 },
    { 0x2aab3f8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdceb3f92, 76 },
    { 0x2bfb3f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83b3f98, 95 },
    { 0, 0 },
    { 0xcceb3f9a, 90 },
    { 0xf96b3f9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93b3f9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52b3fa1, 23 },
    { 0, 0 },
    { 0x478b3fa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbb3fa7, 98 },
    { 0xec1b3fa8, 98 },
    { 0, 0 },
    { 0x335b3faa, 95 },
    { 0x3b3fab, 99 },
    { 0, 0 },
    { 0x2cb3fad, 46 },
    { 0x16eb3fad, 95 },
    { 0xe0fb3fae, 84 },
    { 0, 0 },
    { 0xbcdb3fb1, 17 },
    { 0, 0 },
    { 0xd3db3fb3, 33 },
    { 0, 0 },
    { 0xd98b3fb5, 69 },
    { 0x999b3fb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28b3fbb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fab3fc0, 95 },
    { 0, 0 },
    { 0xdb5b3fc2, 61 },
    { 0xc94b3fc3, 76 },
    { 0x2eb3fc2, 55 },
    { 0xbc8b3fc5, 43 },
    { 0xbcbb3fc6, 61 },
    { 0xfbcb3fc7, 23 },
    { 0xcd4b3fc7, 43 },
    { 0x926b3fc9, 33 },
    { 0x5f2b3fc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8b3fcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b3fd3, 91 },
    { 0x699b3fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50b3fd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fb3fdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dab3fe4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3fe7, 95 },
    { 0x73eb3fe7, 100 },
    { 0, 0 },
    { 0xd79b3fea, 23 },
    { 0xcd9b3feb, 76 },
    { 0xb93b3fea, 84 },
    { 0, 0 },
    { 0xdd1b3fee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b3ff3, 46 },
    { 0x7b4b3ff4, 34 },
    { 0x187b3ff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x730b3ff8, 88 },
    { 0x4c7b3ff8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fb3ffd, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1b4000, 85 },
    { 0, 0 },
    { 0x73ab4002, 95 },
    { 0xf73b4003, 34 },
    { 0x42db4004, 87 },
    { 0, 0 },
    { 0xf99b4006, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feb4009, 95 },
    { 0x95bb400a, 43 },
    { 0x5ddb400b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffb4012, 99 },
    { 0, 0 },
    { 0xa30b4014, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b4019, 95 },
    { 0xa03b401a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x160b401d, 34 },
    { 0x838b401d, 34 },
    { 0, 0 },
    { 0xf0b4020, 93 },
    { 0x2beb4021, 90 },
    { 0, 0 },
    { 0x1f1b4023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x151b4026, 95 },
    { 0x175b4027, 93 },
    { 0, 0 },
    { 0x903b4029, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x945b402c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9b402f, 23 },
    { 0x135b4030, 23 },
    { 0xbbb402f, 33 },
    { 0x487b4032, 23 },
    { 0xecab4033, 31 },
    { 0x7ddb4030, 90 },
    { 0xd8db4035, 90 },
    { 0x349b402f, 84 },
    { 0, 0 },
    { 0x233b4038, 33 },
    { 0xd31b4038, 29 },
    { 0xa1b403a, 23 },
    { 0, 0 },
    { 0x324b403c, 46 },
    { 0x877b403d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2b4041, 87 },
    { 0x4d6b4041, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2b4045, 91 },
    { 0x3c5b4046, 55 },
    { 0xe64b4047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58b404b, 90 },
    { 0x5d5b404b, 75 },
    { 0xff6b404d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b4056, 98 },
    { 0, 0 },
    { 0x414b4058, 33 },
    { 0, 0 },
    { 0x10eb405a, 95 },
    { 0, 0 },
    { 0x8e7b405c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58b4061, 90 },
    { 0xebbb4061, 61 },
    { 0xee9b4063, 61 },
    { 0x650b4061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ceb4068, 78 },
    { 0, 0 },
    { 0xa14b406a, 90 },
    { 0, 0 },
    { 0x142b406c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466b4070, 93 },
    { 0x75ab4071, 90 },
    { 0xd75b4072, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3b4078, 95 },
    { 0x386b4079, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69b407d, 68 },
    { 0, 0 },
    { 0x28fb407f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb4082, 85 },
    { 0x8a1b4083, 61 },
    { 0x216b4082, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43b408a, 55 },
    { 0x121b408b, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x978b408e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eb4092, 33 },
    { 0, 0 },
    { 0xfb9b4094, 77 },
    { 0xabdb4095, 78 },
    { 0x12db4096, 83 },
    { 0x187b4095, 33 },
    { 0x261b4098, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb409d, 23 },
    { 0, 0 },
    { 0xe1fb409f, 94 },
    { 0, 0 },
    { 0x29cb40a1, 23 },
    { 0, 0 },
    { 0xca1b40a3, 88 },
    { 0xa75b40a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b40ae, 61 },
    { 0xc35b40af, 33 },
    { 0xb50b40b0, 43 },
    { 0x94bb40b1, 90 },
    { 0x4bdb40b0, 98 },
    { 0xaddb40b1, 95 },
    { 0xe5cb40b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913b40b9, 76 },
    { 0x343b40ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0b40c2, 55 },
    { 0xe4cb40c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0b40c7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802b40cd, 61 },
    { 0x1a8b40ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b40d8, 95 },
    { 0, 0 },
    { 0xc97b40da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17b40dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db40e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6b40e3, 98 },
    { 0x52fb40e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0b40e7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ab40eb, 61 },
    { 0, 0 },
    { 0xe4b40ed, 95 },
    { 0, 0 },
    { 0x30fb40ef, 81 },
    { 0, 0 },
    { 0xddab40f1, 78 },
    { 0, 0 },
    { 0x724b40f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b40f9, 23 },
    { 0xf4ab40fa, 81 },
    { 0xb42b40fa, 33 },
    { 0xc2cb40fc, 88 },
    { 0x3f6b40fd, 33 },
    { 0x40ab40fd, 90 },
    { 0xbf1b40f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x347b4102, 33 },
    { 0xd03b4102, 90 },
    { 0xd7cb4102, 33 },
    { 0xefb4105, 99 },
    { 0, 0 },
    { 0x64b4107, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b410b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55b4111, 90 },
    { 0, 0 },
    { 0x690b4113, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fb4117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731b411c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17db4122, 76 },
    { 0x8b0b4123, 52 },
    { 0xde9b4124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b412b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ab4130, 90 },
    { 0xad2b4131, 69 },
    { 0xcd8b4132, 52 },
    { 0xcaab4133, 34 },
    { 0x6a1b4134, 88 },
    { 0xf5ab4134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feb413e, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x996b4141, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84b4148, 99 },
    { 0x15db4149, 90 },
    { 0xe22b4148, 93 },
    { 0, 0 },
    { 0x26db414c, 90 },
    { 0, 0 },
    { 0xe19b414e, 33 },
    { 0, 0 },
    { 0xe1fb4150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b4155, 87 },
    { 0xcccb4156, 76 },
    { 0, 0 },
    { 0x9d2b4158, 17 },
    { 0x6db4159, 88 },
    { 0x721b4159, 34 },
    { 0x74cb415b, 81 },
    { 0, 0 },
    { 0xae3b415d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cb4161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fb4164, 76 },
    { 0xb97b4165, 90 },
    { 0, 0 },
    { 0x8aab4167, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafb416d, 33 },
    { 0xbafb416d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90b4173, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934b4179, 33 },
    { 0, 0 },
    { 0xb8b417b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9b4182, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667b4188, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6b418b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfb418f, 87 },
    { 0, 0 },
    { 0xe77b4191, 34 },
    { 0xa5bb4192, 76 },
    { 0x404b4192, 33 },
    { 0xaabb4194, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455b419c, 98 },
    { 0xc2eb419d, 53 },
    { 0x2d5b419d, 84 },
    { 0xd2eb419e, 85 },
    { 0x24cb419f, 90 },
    { 0, 0 },
    { 0xb3ab41a2, 76 },
    { 0x5acb41a2, 90 },
    { 0x8f8b41a4, 76 },
    { 0x4f2b41a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89db41a8, 95 },
    { 0x612b41a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1b41ae, 77 },
    { 0, 0 },
    { 0x5ccb41b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b41b9, 98 },
    { 0, 0 },
    { 0xdd8b41bb, 100 },
    { 0, 0 },
    { 0xf09b41bd, 91 },
    { 0, 0 },
    { 0x58cb41bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b41c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776b41cb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7b41d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeb41d7, 90 },
    { 0x83db41d7, 98 },
    { 0x57cb41d9, 34 },
    { 0, 0 },
    { 0x7c4b41db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4b41df, 91 },
    { 0xab3b41df, 91 },
    { 0, 0 },
    { 0x77b41e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44b41f1, 33 },
    { 0, 0 },
    { 0x4d9b41f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56b41f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7b41fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b4204, 55 },
    { 0x8eab4205, 65 },
    { 0, 0 },
    { 0x612b4207, 66 },
    { 0xee8b4207, 23 },
    { 0xd8ab4208, 33 },
    { 0xad5b4209, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b420d, 90 },
    { 0xad1b420e, 9 },
    { 0x8a9b420d, 55 },
    { 0xb2bb4210, 98 },
    { 0, 0 },
    { 0xd0ab4212, 43 },
    { 0x2a5b4212, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1b421e, 78 },
    { 0xefcb421e, 76 },
    { 0x8bcb4220, 98 },
    { 0x242b4220, 98 },
    { 0, 0 },
    { 0x1afb4223, 33 },
    { 0x3a9b4224, 98 },
    { 0x46eb4223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02b4228, 23 },
    { 0x62cb4228, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9b4230, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966b4234, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69b4238, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420b423d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373b4242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14b4245, 98 },
    { 0x4b2b4246, 23 },
    { 0x20ab4246, 33 },
    { 0x259b4248, 98 },
    { 0xa42b4246, 23 },
    { 0x1d0b4246, 95 },
    { 0x66db424b, 55 },
    { 0xf45b424b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccb4255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x236b4258, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b425b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540b425f, 33 },
    { 0x3fdb425f, 23 },
    { 0xa64b4260, 34 },
    { 0, 0 },
    { 0xc10b4263, 90 },
    { 0x2bb4263, 34 },
    { 0, 0 },
    { 0xd6db4266, 19 },
    { 0xe76b4267, 33 },
    { 0x948b4267, 23 },
    { 0x39cb4267, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cb4271, 99 },
    { 0, 0 },
    { 0xae7b4273, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2b4278, 61 },
    { 0xf2b4279, 33 },
    { 0, 0 },
    { 0xcc7b427b, 81 },
    { 0xbc8b427c, 55 },
    { 0x69cb427d, 100 },
    { 0xdfab427e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4b4282, 61 },
    { 0, 0 },
    { 0x1e7b4284, 98 },
    { 0x397b4285, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1b428f, 23 },
    { 0xd87b4290, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ab4293, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x73db4296, 52 },
    { 0, 0 },
    { 0x977b4298, 77 },
    { 0x21db4298, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404b42a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddb42a4, 78 },
    { 0x554b42a5, 98 },
    { 0xe33b42a6, 23 },
    { 0x803b42a4, 38 },
    { 0, 0 },
    { 0x6c4b42a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb42b8, 33 },
    { 0, 0 },
    { 0xcb2b42ba, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edb42be, 93 },
    { 0x3edb42bf, 90 },
    { 0x95db42bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446b42c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3b42c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccb42cc, 93 },
    { 0, 0 },
    { 0x6d6b42ce, 43 },
    { 0, 0 },
    { 0x64cb42d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74b42d6, 81 },
    { 0xcaeb42d6, 76 },
    { 0x366b42d6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb42e1, 77 },
    { 0x55ab42e1, 77 },
    { 0x86db42e3, 95 },
    { 0x15bb42e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92b42e7, 80 },
    { 0, 0 },
    { 0x7fab42e9, 76 },
    { 0x504b42e9, 61 },
    { 0, 0 },
    { 0x565b42ec, 93 },
    { 0, 0 },
    { 0x89cb42ee, 95 },
    { 0x3cab42ef, 46 },
    { 0xe03b42f0, 90 },
    { 0x719b42f0, 80 },
    { 0x128b42f2, 87 },
    { 0xec0b42f2, 33 },
    { 0x656b42f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7b42fa, 75 },
    { 0x3efb42fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afb4307, 87 },
    { 0x452b4307, 23 },
    { 0x364b4309, 53 },
    { 0xdf3b430a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279b430e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26b4311, 90 },
    { 0x516b4312, 33 },
    { 0x52ab4313, 90 },
    { 0xbc1b4314, 33 },
    { 0xa28b4313, 90 },
    { 0xcc4b4316, 33 },
    { 0xebb4315, 95 },
    { 0x1aab4318, 98 },
    { 0, 0 },
    { 0xf5eb431a, 33 },
    { 0xea8b431b, 61 },
    { 0xf70b431b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4b4320, 23 },
    { 0, 0 },
    { 0xbe7b4322, 61 },
    { 0x19cb4323, 98 },
    { 0, 0 },
    { 0x9f3b4325, 33 },
    { 0xb7ab4326, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b432e, 64 },
    { 0x1cbb432f, 66 },
    { 0x4edb432e, 90 },
    { 0xaf7b4331, 76 },
    { 0xf6fb4332, 23 },
    { 0x115b4333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ab4336, 90 },
    { 0x403b4336, 95 },
    { 0x6d3b4338, 61 },
    { 0x6ceb4339, 98 },
    { 0x13cb4338, 90 },
    { 0x7bcb433b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b433e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x362b4341, 52 },
    { 0, 0 },
    { 0xd69b4343, 33 },
    { 0, 0 },
    { 0x971b4345, 98 },
    { 0xf04b4346, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235b434a, 23 },
    { 0xae9b434b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb434f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0b4353, 90 },
    { 0, 0 },
    { 0xfecb4355, 76 },
    { 0, 0 },
    { 0x19cb4357, 90 },
    { 0xc34b4357, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fb435d, 61 },
    { 0xe16b435d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eb4361, 33 },
    { 0xe14b4362, 61 },
    { 0x2a5b4361, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76b4366, 23 },
    { 0x18ab4367, 23 },
    { 0xb1bb4368, 99 },
    { 0x634b4369, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0b436c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eb436f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b4373, 84 },
    { 0, 0 },
    { 0x8e5b4375, 43 },
    { 0, 0 },
    { 0xdffb4377, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b437b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb4382, 99 },
    { 0, 0 },
    { 0xd72b4384, 91 },
    { 0xec5b4385, 91 },
    { 0xce2b4385, 88 },
    { 0x4bfb4385, 23 },
    { 0xd3bb4388, 90 },
    { 0, 0 },
    { 0xa1bb438a, 61 },
    { 0, 0 },
    { 0xaab438c, 90 },
    { 0x2bcb438d, 65 },
    { 0x8fdb438e, 68 },
    { 0x4bfb438d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47b4394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392b4399, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ab439c, 81 },
    { 0x5b7b439d, 48 },
    { 0xebeb439e, 33 },
    { 0x773b439e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bb43a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x42db43aa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4b43af, 61 },
    { 0, 0 },
    { 0x736b43b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb43b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b43bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53b43bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42b43c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549b43c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b43cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fb43d3, 95 },
    { 0x9c3b43d4, 90 },
    { 0xf51b43d5, 65 },
    { 0x2bab43d6, 33 },
    { 0x61ab43d6, 98 },
    { 0x2feb43d8, 23 },
    { 0xdecb43d8, 17 },
    { 0x40bb43d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bb43e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63b43e4, 43 },
    { 0, 0 },
    { 0xe07b43e6, 43 },
    { 0x1e1b43e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648b43ec, 88 },
    { 0xa1db43ed, 90 },
    { 0x885b43ec, 76 },
    { 0x1b4b43ef, 61 },
    { 0x12bb43ee, 90 },
    { 0xc41b43f1, 76 },
    { 0x1ffb43f2, 76 },
    { 0xe17b43f3, 98 },
    { 0xf1b43f3, 61 },
    { 0x4c0b43f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233b43fb, 91 },
    { 0, 0 },
    { 0x39cb43fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3b4404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab440d, 43 },
    { 0xc55b440e, 88 },
    { 0xfbcb440f, 33 },
    { 0xfb9b440e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52b4417, 66 },
    { 0x192b4418, 95 },
    { 0x490b4419, 78 },
    { 0x830b4419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1b441d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x246b4420, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afb4424, 77 },
    { 0, 0 },
    { 0x75fb4426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbab4429, 90 },
    { 0x9b0b442a, 88 },
    { 0x9f3b442a, 99 },
    { 0xa00b442a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92b442f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbb4432, 95 },
    { 0x146b4432, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefb4438, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62db443c, 35 },
    { 0x102b443d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120b4443, 90 },
    { 0x808b4444, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930b444e, 90 },
    { 0, 0 },
    { 0x525b4450, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5b4453, 84 },
    { 0x879b4454, 100 },
    { 0x847b4455, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682b445b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbab445e, 90 },
    { 0xaf1b445f, 90 },
    { 0x107b4460, 61 },
    { 0xbffb4461, 90 },
    { 0xb6b4462, 95 },
    { 0x7b3b4460, 33 },
    { 0x20b4464, 68 },
    { 0x960b4465, 23 },
    { 0xfd7b4464, 33 },
    { 0x44fb4465, 61 },
    { 0x840b4468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdb446c, 85 },
    { 0x3f4b446d, 90 },
    { 0xbe6b446e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcb4471, 34 },
    { 0xdbcb4471, 33 },
    { 0x187b4473, 33 },
    { 0xa87b4473, 23 },
    { 0x15bb4473, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89b4478, 95 },
    { 0x7d9b4479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4b447f, 61 },
    { 0x3b2b447f, 93 },
    { 0x51b4481, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2b4484, 61 },
    { 0, 0 },
    { 0x184b4486, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146b448e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x72db4491, 61 },
    { 0, 0 },
    { 0xfe3b4493, 98 },
    { 0x32db4493, 33 },
    { 0, 0 },
    { 0x7b2b4496, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529b449b, 61 },
    { 0, 0 },
    { 0xf86b449d, 55 },
    { 0x4ccb449e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2b44a6, 81 },
    { 0, 0 },
    { 0xac1b44a8, 46 },
    { 0, 0 },
    { 0x7e1b44aa, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddb44ae, 90 },
    { 0xc53b44ae, 91 },
    { 0, 0 },
    { 0x7c3b44b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b44b6, 24 },
    { 0x7db44b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88b44ba, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x606b44bd, 48 },
    { 0, 0 },
    { 0x497b44bf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cb44c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729b44c8, 46 },
    { 0, 0 },
    { 0xf2fb44ca, 33 },
    { 0xf9b44cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79eb44ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5b44d3, 34 },
    { 0xc90b44d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5b44dd, 90 },
    { 0xa7db44dd, 55 },
    { 0x5b8b44df, 61 },
    { 0x65b44e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b44e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8b44eb, 96 },
    { 0x128b44ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcb44f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413b44fd, 33 },
    { 0xe75b44fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x540b4501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cb450a, 34 },
    { 0xbe3b450a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b4510, 33 },
    { 0xb54b4511, 81 },
    { 0x73eb4512, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364b4517, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdb451f, 34 },
    { 0x292b4520, 34 },
    { 0x926b4521, 76 },
    { 0x2a9b4520, 33 },
    { 0x56eb4523, 23 },
    { 0xf07b4524, 90 },
    { 0xd79b4525, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7b452a, 90 },
    { 0xe19b452b, 55 },
    { 0, 0 },
    { 0xf05b452d, 33 },
    { 0xb46b452d, 90 },
    { 0x755b452f, 52 },
    { 0, 0 },
    { 0xc82b4531, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cb4536, 55 },
    { 0xe7cb4537, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aeb453b, 61 },
    { 0xfc6b453c, 46 },
    { 0xaf2b453d, 33 },
    { 0x15cb453d, 95 },
    { 0xad3b453f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb4542, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eb4547, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0eb454d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3b4550, 91 },
    { 0x6cfb4551, 98 },
    { 0x4c9b4552, 23 },
    { 0, 0 },
    { 0xe3cb4554, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2b4558, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968b455f, 87 },
    { 0, 0 },
    { 0x295b4561, 66 },
    { 0x9c2b4561, 90 },
    { 0, 0 },
    { 0x9cbb4564, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587b4568, 17 },
    { 0xfdeb4568, 83 },
    { 0x5dfb456a, 84 },
    { 0x790b456b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b456f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b4573, 98 },
    { 0xe64b4574, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24b4577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8b4583, 61 },
    { 0, 0 },
    { 0x568b4585, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b4588, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cb458f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53b4592, 23 },
    { 0, 0 },
    { 0x505b4594, 76 },
    { 0xcebb4595, 55 },
    { 0xb20b4596, 93 },
    { 0x421b4597, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cb459f, 55 },
    { 0x4d4b45a0, 46 },
    { 0xbcfb45a1, 23 },
    { 0xc8bb45a0, 67 },
    { 0x772b45a3, 88 },
    { 0x50b45a1, 95 },
    { 0x237b45a5, 33 },
    { 0xc3ab45a6, 99 },
    { 0xc64b45a7, 66 },
    { 0x67fb45a8, 98 },
    { 0x7b8b45a3, 95 },
    { 0xf77b45a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fb45af, 33 },
    { 0, 0 },
    { 0x639b45b1, 23 },
    { 0, 0 },
    { 0xc60b45b3, 12 },
    { 0, 0 },
    { 0xa2fb45b5, 52 },
    { 0xc95b45b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23b45b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956b45c0, 46 },
    { 0xf64b45c1, 90 },
    { 0x13db45c1, 96 },
    { 0, 0 },
    { 0x1e1b45c4, 61 },
    { 0x305b45c5, 66 },
    { 0x562b45c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b45d3, 46 },
    { 0, 0 },
    { 0xa9db45d5, 43 },
    { 0xb7eb45d5, 61 },
    { 0xb59b45d7, 84 },
    { 0xc59b45d6, 43 },
    { 0, 0 },
    { 0x3cb45da, 46 },
    { 0, 0 },
    { 0x86fb45dc, 90 },
    { 0x82cb45dd, 33 },
    { 0, 0 },
    { 0x612b45df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962b45e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b45e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982b45eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b45f0, 55 },
    { 0x807b45f1, 98 },
    { 0x150b45f1, 91 },
    { 0xaa9b45f3, 98 },
    { 0, 0 },
    { 0x91eb45f5, 90 },
    { 0, 0 },
    { 0xcafb45f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ab45fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eb4603, 93 },
    { 0xe52b4603, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13b4607, 93 },
    { 0xd11b4607, 93 },
    { 0xfa1b4607, 90 },
    { 0xbadb460a, 33 },
    { 0, 0 },
    { 0x4eab460c, 76 },
    { 0x79fb460d, 96 },
    { 0x973b460e, 81 },
    { 0, 0 },
    { 0xa3bb4610, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b4617, 56 },
    { 0x221b4618, 76 },
    { 0xcb6b4618, 77 },
    { 0x813b461a, 23 },
    { 0x652b461b, 61 },
    { 0x39fb461b, 61 },
    { 0xbc4b461b, 33 },
    { 0x681b461a, 34 },
    { 0, 0 },
    { 0x4d1b4620, 100 },
    { 0x6b5b4621, 78 },
    { 0x1d1b4621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcb4627, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828b462b, 80 },
    { 0, 0 },
    { 0x6afb462d, 98 },
    { 0x659b462e, 34 },
    { 0xc9b462d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554b4635, 61 },
    { 0, 0 },
    { 0xba8b4637, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73eb463f, 87 },
    { 0, 0 },
    { 0x1dfb4641, 90 },
    { 0x784b4642, 68 },
    { 0x92eb4642, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b464a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eb464d, 76 },
    { 0, 0 },
    { 0x585b464f, 90 },
    { 0x1e5b4650, 23 },
    { 0x51db4651, 43 },
    { 0xa70b4651, 23 },
    { 0x33cb4653, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adb4657, 61 },
    { 0x637b4658, 75 },
    { 0x7bdb4658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb465d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x861b4660, 43 },
    { 0, 0 },
    { 0xe10b4662, 46 },
    { 0, 0 },
    { 0xca2b4664, 76 },
    { 0, 0 },
    { 0xe28b4666, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x124b4669, 61 },
    { 0x83eb466a, 61 },
    { 0, 0 },
    { 0x763b466c, 33 },
    { 0x4e8b466d, 76 },
    { 0, 0 },
    { 0xb30b466f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150b4674, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb4677, 90 },
    { 0, 0 },
    { 0xd8eb4679, 98 },
    { 0xbc0b4679, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2b467f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12b468a, 98 },
    { 0x305b468a, 91 },
    { 0x123b468c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0b468f, 61 },
    { 0x4c4b4690, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cb4694, 23 },
    { 0x5eab4695, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x727b4698, 85 },
    { 0x421b4699, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5b46a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfb46a5, 98 },
    { 0x43db46a6, 99 },
    { 0, 0 },
    { 0x5ab46a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x582b46ab, 90 },
    { 0x2b4b46ac, 55 },
    { 0, 0 },
    { 0xb2db46ae, 48 },
    { 0x13bb46ae, 23 },
    { 0xc02b46ae, 93 },
    { 0xf59b46b1, 61 },
    { 0, 0 },
    { 0x84b46b3, 95 },
    { 0, 0 },
    { 0x99eb46b5, 88 },
    { 0xb42b46b6, 78 },
    { 0, 0 },
    { 0x904b46b8, 34 },
    { 0xe1b46b9, 98 },
    { 0, 0 },
    { 0x9e4b46bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02b46bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ab46c7, 33 },
    { 0x826b46c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x411b46cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b46ce, 95 },
    { 0x2b3b46ce, 23 },
    { 0x296b46d0, 33 },
    { 0x3f5b46d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489b46d5, 34 },
    { 0x153b46d5, 23 },
    { 0xa9eb46d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b46de, 33 },
    { 0, 0 },
    { 0x5b1b46e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bb46e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfab46e9, 90 },
    { 0x34eb46e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b46ed, 46 },
    { 0x124b46ed, 33 },
    { 0, 0 },
    { 0x91cb46f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb46f3, 90 },
    { 0xa8bb46f4, 52 },
    { 0xf79b46f5, 98 },
    { 0, 0 },
    { 0x2e0b46f7, 65 },
    { 0, 0 },
    { 0x9e4b46f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b46fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ab4704, 76 },
    { 0x64fb4704, 95 },
    { 0xe9fb4706, 85 },
    { 0x65fb4707, 51 },
    { 0xe5bb4708, 64 },
    { 0xd79b4709, 17 },
    { 0x56fb470a, 33 },
    { 0xe91b470b, 78 },
    { 0, 0 },
    { 0x4cdb470d, 98 },
    { 0x80db470d, 34 },
    { 0xef8b470e, 51 },
    { 0x4b3b4710, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80b4714, 43 },
    { 0, 0 },
    { 0xe93b4716, 61 },
    { 0xaf5b4717, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcb471b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacb471e, 77 },
    { 0x3b4b471f, 33 },
    { 0x71db4720, 98 },
    { 0, 0 },
    { 0xef0b4722, 76 },
    { 0x2bb4722, 87 },
    { 0x7a5b4724, 98 },
    { 0x603b4722, 67 },
    { 0, 0 },
    { 0xcd4b4727, 87 },
    { 0xa1db4728, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83b4733, 66 },
    { 0x723b4734, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ab4737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb473b, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bb4747, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8b474a, 87 },
    { 0, 0 },
    { 0xe07b474c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8b474f, 46 },
    { 0x5a7b4750, 77 },
    { 0xf59b4751, 76 },
    { 0x58bb4751, 95 },
    { 0x2ffb4753, 91 },
    { 0x5eeb4751, 19 },
    { 0xce8b4755, 95 },
    { 0xdb2b4756, 60 },
    { 0xb32b4757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1b475a, 87 },
    { 0x286b475b, 93 },
    { 0x4e5b475a, 52 },
    { 0x218b475c, 43 },
    { 0xf49b475e, 33 },
    { 0x580b475f, 93 },
    { 0xa9cb475e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b4770, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b477b, 33 },
    { 0xa0ab477c, 33 },
    { 0x910b477d, 23 },
    { 0xaeb477e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86b4781, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbab4786, 66 },
    { 0x8d1b4787, 61 },
    { 0xd67b4787, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528b4793, 76 },
    { 0x68eb4794, 90 },
    { 0xfd7b4795, 98 },
    { 0xacb4796, 76 },
    { 0xadab4797, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5db479b, 95 },
    { 0x7a1b479c, 90 },
    { 0x24cb479b, 98 },
    { 0x336b479b, 33 },
    { 0x528b479c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1b47a4, 55 },
    { 0, 0 },
    { 0xea4b47a6, 61 },
    { 0, 0 },
    { 0x8f2b47a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b47ac, 66 },
    { 0xf75b47ad, 95 },
    { 0, 0 },
    { 0x9d9b47af, 90 },
    { 0, 0 },
    { 0xa09b47b1, 81 },
    { 0xdb4b47b1, 61 },
    { 0x582b47b3, 95 },
    { 0x91bb47b3, 33 },
    { 0x1cb47b5, 61 },
    { 0x468b47b5, 98 },
    { 0x733b47b7, 76 },
    { 0x65fb47b7, 99 },
    { 0, 0 },
    { 0xda0b47ba, 101 },
    { 0, 0 },
    { 0x143b47bc, 95 },
    { 0xf17b47bd, 93 },
    { 0, 0 },
    { 0x7e3b47bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0db47c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57b47c6, 98 },
    { 0, 0 },
    { 0x1acb47c8, 90 },
    { 0xfd4b47c9, 33 },
    { 0, 0 },
    { 0x5bbb47cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14b47ce, 52 },
    { 0, 0 },
    { 0xdc6b47d0, 81 },
    { 0xd11b47d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cb47d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b47d9, 90 },
    { 0x178b47da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fb47e0, 33 },
    { 0xf3fb47e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bb47e6, 66 },
    { 0x6e0b47e6, 90 },
    { 0, 0 },
    { 0x9abb47e9, 34 },
    { 0x58eb47ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x762b47ed, 98 },
    { 0x23db47ee, 99 },
    { 0x852b47ed, 33 },
    { 0xc5b47ed, 98 },
    { 0x3f4b47f1, 93 },
    { 0xa8eb47ee, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb47f6, 61 },
    { 0, 0 },
    { 0x9fbb47f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57b47fc, 43 },
    { 0, 0 },
    { 0xa1b47fe, 90 },
    { 0xd79b47ff, 90 },
    { 0xa4db47ff, 55 },
    { 0x30ab47ff, 93 },
    { 0, 0 },
    { 0x8eb4803, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1b4806, 100 },
    { 0x324b4807, 90 },
    { 0x3e4b4808, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7b480b, 68 },
    { 0x9d1b480c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97b4810, 33 },
    { 0xd58b4811, 78 },
    { 0x92b4810, 61 },
    { 0x25b4812, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41b481a, 98 },
    { 0xecdb481a, 23 },
    { 0, 0 },
    { 0x4c2b481d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ab4829, 90 },
    { 0xde4b482a, 85 },
    { 0x201b4829, 38 },
    { 0x354b482c, 93 },
    { 0x5d7b482d, 34 },
    { 0, 0 },
    { 0x7fdb482f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8b4832, 76 },
    { 0x56db4833, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b4837, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615b4841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1b4845, 46 },
    { 0xdb9b4846, 90 },
    { 0xde0b4847, 93 },
    { 0, 0 },
    { 0x38eb4849, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98b484c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0b484f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fb4852, 34 },
    { 0x3bcb4853, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52b4857, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b485a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x738b485d, 90 },
    { 0, 0 },
    { 0xa20b485f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477b4868, 98 },
    { 0, 0 },
    { 0x979b486a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794b486f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cb4877, 77 },
    { 0x16cb4878, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ab487c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afb4881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fb4885, 34 },
    { 0xa04b4886, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b488c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab488f, 61 },
    { 0xe63b4890, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ab4897, 33 },
    { 0x399b4898, 90 },
    { 0x803b4897, 33 },
    { 0xc05b4898, 95 },
    { 0xf86b489a, 76 },
    { 0xb49b489c, 61 },
    { 0, 0 },
    { 0xc5b489e, 61 },
    { 0x7e9b489f, 23 },
    { 0xb04b489e, 33 },
    { 0xc40b48a1, 66 },
    { 0, 0 },
    { 0x2a2b48a3, 55 },
    { 0x4f7b48a4, 61 },
    { 0x24eb48a3, 61 },
    { 0x5f4b48a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fb48b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b48b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbb48ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25b48bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab48c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cb48c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4b48cb, 84 },
    { 0xcf7b48cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450b48d0, 33 },
    { 0xea7b48d1, 61 },
    { 0x15db48d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507b48d7, 98 },
    { 0x80cb48d8, 95 },
    { 0, 0 },
    { 0xcdab48da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b48df, 61 },
    { 0, 0 },
    { 0xc1cb48e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x383b48e4, 61 },
    { 0x293b48e5, 61 },
    { 0, 0 },
    { 0x122b48e7, 33 },
    { 0, 0 },
    { 0x24b48e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb48ed, 93 },
    { 0, 0 },
    { 0x92b48ef, 53 },
    { 0, 0 },
    { 0xdf9b48f1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82b48f4, 61 },
    { 0xfd9b48f5, 23 },
    { 0x5f3b48f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x518b48f9, 33 },
    { 0x6c3b48fa, 95 },
    { 0x23cb48fb, 93 },
    { 0xffeb48fc, 61 },
    { 0xc59b48fd, 68 },
    { 0x21b48fd, 33 },
    { 0x781b48fb, 61 },
    { 0x263b4900, 101 },
    { 0xddb4900, 95 },
    { 0x2a2b4900, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6b4907, 90 },
    { 0x1bcb4908, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2b490c, 33 },
    { 0x5d5b490d, 98 },
    { 0x6bb490e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bb4912, 33 },
    { 0x83bb4912, 61 },
    { 0xc3db4914, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeab491a, 78 },
    { 0xb34b491a, 33 },
    { 0xd08b491b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08b491f, 46 },
    { 0x254b4920, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb4924, 76 },
    { 0xee4b4924, 95 },
    { 0x5ddb4924, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eb4929, 91 },
    { 0x571b4929, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b492e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fb4931, 61 },
    { 0x495b4931, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb4935, 98 },
    { 0x61cb4935, 90 },
    { 0x7a7b4937, 43 },
    { 0xe5fb4938, 61 },
    { 0x73fb4939, 98 },
    { 0xb39b4939, 86 },
    { 0, 0 },
    { 0x77bb493c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9b4940, 33 },
    { 0x185b4940, 99 },
    { 0xb41b4941, 98 },
    { 0, 0 },
    { 0x295b4944, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb4947, 95 },
    { 0xb60b4948, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3b494d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb4953, 55 },
    { 0x963b4954, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5b4957, 76 },
    { 0, 0 },
    { 0x607b4959, 95 },
    { 0x402b495a, 43 },
    { 0, 0 },
    { 0x198b495c, 98 },
    { 0x7cb495c, 55 },
    { 0xd1cb495e, 48 },
    { 0, 0 },
    { 0x947b4960, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x706b4963, 90 },
    { 0xaa7b4964, 43 },
    { 0x6b8b4965, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ab496b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x989b496e, 55 },
    { 0xba9b496f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x532b4972, 23 },
    { 0, 0 },
    { 0xb86b4974, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42b4977, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feb4983, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0b4986, 94 },
    { 0xd64b4986, 78 },
    { 0x5d2b4988, 100 },
    { 0xc8eb4989, 23 },
    { 0xe0fb4988, 17 },
    { 0xc27b4987, 40 },
    { 0xc4b4989, 67 },
    { 0x24fb498d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adb4994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b499a, 31 },
    { 0x86cb499b, 43 },
    { 0xc7fb499b, 55 },
    { 0x2dab499d, 76 },
    { 0x97ab499a, 65 },
    { 0xd37b499d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b49a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b49a5, 98 },
    { 0xd50b49a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51b49ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5b49b6, 33 },
    { 0xbf4b49b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6b49ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b49c6, 33 },
    { 0, 0 },
    { 0x412b49c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48b49cd, 33 },
    { 0, 0 },
    { 0x9b2b49cf, 33 },
    { 0x354b49cf, 34 },
    { 0, 0 },
    { 0x72ab49d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9b49da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb49dd, 88 },
    { 0x48fb49dd, 76 },
    { 0x75eb49df, 95 },
    { 0x913b49e0, 100 },
    { 0x364b49de, 95 },
    { 0, 0 },
    { 0x5e5b49e3, 33 },
    { 0x5ccb49e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13b49e9, 53 },
    { 0, 0 },
    { 0x799b49eb, 43 },
    { 0xcbb49ec, 95 },
    { 0xd2cb49ec, 55 },
    { 0xd6db49ee, 90 },
    { 0, 0 },
    { 0xb69b49f0, 90 },
    { 0x455b49f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcb49f6, 98 },
    { 0x565b49f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389b49fc, 98 },
    { 0, 0 },
    { 0xbebb49fe, 23 },
    { 0, 0 },
    { 0xb2b4a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9b4a04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x907b4a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acb4a0e, 93 },
    { 0, 0 },
    { 0x76eb4a10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57b4a13, 90 },
    { 0, 0 },
    { 0x5eb4a15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cb4a18, 84 },
    { 0, 0 },
    { 0xc03b4a1a, 55 },
    { 0x37db4a1b, 90 },
    { 0, 0 },
    { 0xa07b4a1d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5b4a20, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbab4a24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b4a27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48b4a2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ab4a36, 23 },
    { 0, 0 },
    { 0xf6db4a38, 66 },
    { 0, 0 },
    { 0xf7cb4a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b4a40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x469b4a43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352b4a49, 23 },
    { 0xac9b4a49, 90 },
    { 0x6a9b4a4b, 98 },
    { 0xe6cb4a4a, 55 },
    { 0, 0 },
    { 0x8dcb4a4e, 61 },
    { 0, 0 },
    { 0x83bb4a50, 90 },
    { 0xd91b4a51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cb4a55, 95 },
    { 0x9bab4a56, 23 },
    { 0xb0fb4a57, 91 },
    { 0x15bb4a58, 23 },
    { 0x476b4a59, 76 },
    { 0, 0 },
    { 0x593b4a5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b4a60, 46 },
    { 0x99fb4a61, 88 },
    { 0x8c0b4a62, 93 },
    { 0x585b4a61, 55 },
    { 0xf33b4a64, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5b4a67, 23 },
    { 0xb6bb4a68, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5b4a6c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fb4a71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255b4a75, 46 },
    { 0, 0 },
    { 0xdfcb4a77, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb4a7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b4a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139b4a8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147b4a8f, 61 },
    { 0, 0 },
    { 0xdfcb4a91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5b4a94, 95 },
    { 0, 0 },
    { 0xc3db4a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ab4a9d, 33 },
    { 0, 0 },
    { 0xc6fb4a9f, 97 },
    { 0x3fcb4a9f, 87 },
    { 0xdbeb4aa1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1b4aaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09b4aad, 90 },
    { 0xb3fb4aae, 33 },
    { 0x313b4aaf, 98 },
    { 0x70bb4aaf, 76 },
    { 0xe00b4ab1, 17 },
    { 0xd35b4ab0, 98 },
    { 0xbbab4ab3, 61 },
    { 0x695b4aaf, 23 },
    { 0, 0 },
    { 0xde1b4ab6, 61 },
    { 0, 0 },
    { 0x1b2b4ab8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0b4abe, 91 },
    { 0x772b4abe, 33 },
    { 0xfabb4ac0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fb4ac6, 90 },
    { 0xb65b4ac7, 88 },
    { 0xaaab4ac7, 61 },
    { 0xfa7b4ac9, 52 },
    { 0x6b9b4aca, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04b4ad2, 33 },
    { 0x637b4ad3, 95 },
    { 0x263b4ad4, 33 },
    { 0xa76b4ad3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4b4adc, 90 },
    { 0x97cb4add, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919b4ae3, 31 },
    { 0, 0 },
    { 0x673b4ae5, 90 },
    { 0xac0b4ae6, 88 },
    { 0, 0 },
    { 0x794b4ae8, 23 },
    { 0xafb4ae8, 33 },
    { 0, 0 },
    { 0xb25b4aeb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746b4aef, 17 },
    { 0x7b0b4af0, 43 },
    { 0xbb1b4af1, 61 },
    { 0x7b8b4af1, 95 },
    { 0x1f6b4af3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb4afe, 98 },
    { 0, 0 },
    { 0x476b4b00, 90 },
    { 0x870b4b00, 23 },
    { 0x562b4b01, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5b4b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fb4b0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1b4b0f, 61 },
    { 0x297b4b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6b4b1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x703b4b1d, 95 },
    { 0, 0 },
    { 0x46ab4b1f, 33 },
    { 0x809b4b1f, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2b4b26, 88 },
    { 0, 0 },
    { 0xbe0b4b28, 95 },
    { 0x3db4b29, 90 },
    { 0x230b4b2a, 43 },
    { 0, 0 },
    { 0x4f6b4b2c, 33 },
    { 0x338b4b2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fb4b33, 34 },
    { 0x746b4b34, 98 },
    { 0x776b4b35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65b4b38, 61 },
    { 0x93ab4b39, 78 },
    { 0x18b4b39, 23 },
    { 0x482b4b3a, 52 },
    { 0, 0 },
    { 0xdcbb4b3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb4b42, 95 },
    { 0, 0 },
    { 0x42fb4b44, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9b4b48, 76 },
    { 0, 0 },
    { 0x940b4b4a, 33 },
    { 0, 0 },
    { 0x5c1b4b4c, 43 },
    { 0, 0 },
    { 0x953b4b4e, 99 },
    { 0x134b4b4f, 90 },
    { 0x2d4b4b4f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09b4b54, 61 },
    { 0x2f4b4b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbb4b5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x184b4b5e, 90 },
    { 0xdfb4b5f, 34 },
    { 0x234b4b60, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecb4b64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efb4b68, 76 },
    { 0, 0 },
    { 0x40cb4b6a, 55 },
    { 0xc86b4b6b, 84 },
    { 0, 0 },
    { 0xeab4b6d, 66 },
    { 0x2f0b4b6e, 43 },
    { 0x6eeb4b6f, 52 },
    { 0, 0 },
    { 0xb10b4b71, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b4b74, 88 },
    { 0xe70b4b74, 33 },
    { 0xcb6b4b74, 98 },
    { 0, 0 },
    { 0x96cb4b78, 80 },
    { 0x187b4b78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ab4b7d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445b4b84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbb4b8a, 93 },
    { 0x443b4b8a, 76 },
    { 0x7a0b4b8c, 95 },
    { 0x332b4b8d, 95 },
    { 0, 0 },
    { 0xaadb4b8f, 78 },
    { 0xb9cb4b90, 23 },
    { 0, 0 },
    { 0x7c6b4b92, 93 },
    { 0, 0 },
    { 0x9bb4b94, 23 },
    { 0x7acb4b95, 98 },
    { 0x596b4b96, 61 },
    { 0, 0 },
    { 0x10db4b98, 17 },
    { 0x3e9b4b99, 90 },
    { 0, 0 },
    { 0x817b4b9b, 61 },
    { 0x88cb4b9c, 52 },
    { 0, 0 },
    { 0x6d6b4b9e, 43 },
    { 0, 0 },
    { 0xcd4b4ba0, 90 },
    { 0, 0 },
    { 0x403b4ba2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5b4ba6, 90 },
    { 0xaefb4ba7, 100 },
    { 0x2e0b4ba7, 61 },
    { 0xcf9b4ba6, 90 },
    { 0x315b4baa, 99 },
    { 0x128b4bab, 61 },
    { 0xf2ab4ba7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x687b4baf, 40 },
    { 0x91bb4bb0, 61 },
    { 0x6ddb4bb1, 23 },
    { 0xcf4b4bb2, 33 },
    { 0x5d7b4bb0, 95 },
    { 0, 0 },
    { 0x3dfb4bb5, 98 },
    { 0xc9eb4bb6, 93 },
    { 0, 0 },
    { 0x12b4bb8, 93 },
    { 0, 0 },
    { 0xcebb4bba, 23 },
    { 0xc72b4bbb, 23 },
    { 0, 0 },
    { 0x8d0b4bbd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76b4bc0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822b4bc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1b4bcd, 33 },
    { 0, 0 },
    { 0x17cb4bcf, 90 },
    { 0x2b1b4bcf, 90 },
    { 0, 0 },
    { 0xb37b4bd2, 90 },
    { 0, 0 },
    { 0x21b4bd4, 66 },
    { 0xa1ab4bd5, 98 },
    { 0, 0 },
    { 0x37bb4bd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50b4bdb, 91 },
    { 0, 0 },
    { 0xc42b4bdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6b4be0, 67 },
    { 0, 0 },
    { 0x19b4be2, 66 },
    { 0, 0 },
    { 0xfc9b4be4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6b4be8, 98 },
    { 0xa09b4be9, 46 },
    { 0, 0 },
    { 0xc24b4beb, 33 },
    { 0xe3fb4beb, 66 },
    { 0, 0 },
    { 0x8e4b4bee, 93 },
    { 0x81fb4bef, 95 },
    { 0, 0 },
    { 0x4b2b4bf1, 67 },
    { 0x537b4bf2, 95 },
    { 0, 0 },
    { 0xd4b4bf4, 23 },
    { 0, 0 },
    { 0xff7b4bf6, 61 },
    { 0x4ecb4bf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2b4bfa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fb4c02, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b4c05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb4c08, 33 },
    { 0x56cb4c09, 43 },
    { 0x627b4c0a, 19 },
    { 0x5d9b4c0a, 33 },
    { 0xb7eb4c09, 23 },
    { 0xae0b4c0b, 33 },
    { 0, 0 },
    { 0xb12b4c0f, 23 },
    { 0x8f4b4c10, 61 },
    { 0xc2ab4c11, 23 },
    { 0, 0 },
    { 0x3d3b4c13, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fb4c16, 90 },
    { 0xd94b4c17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94b4c1e, 61 },
    { 0x8e4b4c1f, 76 },
    { 0, 0 },
    { 0xd81b4c21, 90 },
    { 0x229b4c21, 87 },
    { 0x377b4c23, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7b4c27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b4c2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dab4c33, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550b4c39, 87 },
    { 0x4fcb4c39, 90 },
    { 0, 0 },
    { 0xf6fb4c3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8b4c40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149b4c47, 90 },
    { 0xa39b4c48, 90 },
    { 0x57eb4c49, 85 },
    { 0x2abb4c4a, 43 },
    { 0, 0 },
    { 0x592b4c4c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5b4c4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9b4c53, 33 },
    { 0x295b4c54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcb4c58, 98 },
    { 0x5a8b4c59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204b4c63, 100 },
    { 0x274b4c64, 55 },
    { 0x7e4b4c64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260b4c69, 5 },
    { 0, 0 },
    { 0x845b4c6b, 90 },
    { 0x35cb4c6c, 95 },
    { 0x5fcb4c6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70b4c72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b4c7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b4c84, 68 },
    { 0xfbcb4c84, 23 },
    { 0, 0 },
    { 0x504b4c87, 33 },
    { 0xbb0b4c88, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb4c8c, 23 },
    { 0xe9eb4c8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cb4c98, 90 },
    { 0x1b0b4c99, 43 },
    { 0, 0 },
    { 0x49bb4c9b, 68 },
    { 0, 0 },
    { 0x240b4c9d, 90 },
    { 0x58fb4c9e, 33 },
    { 0x79ab4c9d, 81 },
    { 0x621b4c9f, 64 },
    { 0xe75b4c9e, 33 },
    { 0, 0 },
    { 0x599b4ca3, 23 },
    { 0, 0 },
    { 0xc1fb4ca5, 94 },
    { 0xcdeb4ca5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b4caa, 33 },
    { 0, 0 },
    { 0xbd3b4cac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552b4cbd, 52 },
    { 0xf53b4cbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15b4cc1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0b4cc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5b4cc8, 76 },
    { 0, 0 },
    { 0x334b4cca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0b4cd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89b4ce7, 68 },
    { 0x227b4ce8, 90 },
    { 0x2cdb4ce9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10b4cf2, 98 },
    { 0xc26b4cf2, 33 },
    { 0, 0 },
    { 0xae5b4cf5, 93 },
    { 0, 0 },
    { 0xe8fb4cf7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5b4cfe, 43 },
    { 0xfa3b4cff, 98 },
    { 0x94db4d00, 33 },
    { 0x345b4d01, 95 },
    { 0x6c0b4d02, 90 },
    { 0, 0 },
    { 0x25ab4d04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2b4d09, 13 },
    { 0, 0 },
    { 0xc38b4d0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbb4d11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab4d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cb4d1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b4d1f, 90 },
    { 0x54eb4d20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8b4d24, 91 },
    { 0xed5b4d24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b4d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ab4d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281b4d3f, 46 },
    { 0x89ab4d3f, 84 },
    { 0x496b4d3f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57b4d44, 33 },
    { 0x16eb4d45, 75 },
    { 0x4f0b4d46, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60b4d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bb4d50, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8b4d55, 55 },
    { 0xd3bb4d55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3b4d59, 62 },
    { 0x9e1b4d5a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91b4d5d, 90 },
    { 0, 0 },
    { 0x294b4d5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b4d66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1b4d69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffb4d6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b4d73, 76 },
    { 0xffbb4d74, 23 },
    { 0xc38b4d75, 44 },
    { 0x54b4d76, 98 },
    { 0x45eb4d76, 34 },
    { 0x907b4d76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49b4d7b, 23 },
    { 0xb7fb4d7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cb4d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b4d86, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b4d8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab4d94, 90 },
    { 0xffdb4d95, 43 },
    { 0xc06b4d95, 61 },
    { 0x11db4d97, 61 },
    { 0xe79b4d98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597b4d9f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92eb4da4, 91 },
    { 0xecbb4da4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fb4dab, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ab4db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612b4db7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bb4dbb, 85 },
    { 0xba4b4dbc, 90 },
    { 0x938b4dbd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcb4dc0, 33 },
    { 0, 0 },
    { 0xd69b4dc2, 90 },
    { 0, 0 },
    { 0xddbb4dc4, 46 },
    { 0x361b4dc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431b4dcf, 46 },
    { 0xbd9b4dd0, 52 },
    { 0xe5fb4dcf, 12 },
    { 0xe63b4dd0, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9b4dd5, 23 },
    { 0x2a1b4dd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x516b4dd9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670b4dde, 34 },
    { 0xaafb4dde, 64 },
    { 0, 0 },
    { 0x74ab4de1, 43 },
    { 0x36bb4de2, 55 },
    { 0x548b4de3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6b4dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58b4def, 78 },
    { 0, 0 },
    { 0x1e2b4df1, 25 },
    { 0, 0 },
    { 0x518b4df3, 90 },
    { 0, 0 },
    { 0x7d1b4df5, 95 },
    { 0x20cb4df5, 33 },
    { 0x23b4df7, 98 },
    { 0xdbab4df8, 90 },
    { 0x579b4df9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24b4dfd, 76 },
    { 0x166b4dfe, 34 },
    { 0x840b4dff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75b4e02, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520b4e08, 95 },
    { 0, 0 },
    { 0xea7b4e0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7b4e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151b4e12, 33 },
    { 0x544b4e13, 90 },
    { 0, 0 },
    { 0xc78b4e15, 46 },
    { 0x634b4e15, 55 },
    { 0x262b4e17, 97 },
    { 0x6eab4e17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8b4e20, 95 },
    { 0, 0 },
    { 0x278b4e22, 43 },
    { 0xd47b4e23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38b4e28, 90 },
    { 0, 0 },
    { 0x2ddb4e2a, 66 },
    { 0x8ab4e2a, 95 },
    { 0x57ab4e2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7b4e30, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feb4e33, 52 },
    { 0x4eb4e34, 76 },
    { 0, 0 },
    { 0x1ceb4e36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b4e40, 98 },
    { 0, 0 },
    { 0x4f6b4e42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab4e47, 85 },
    { 0x49bb4e47, 80 },
    { 0, 0 },
    { 0xcb4b4e4a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11db4e4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29eb4e50, 93 },
    { 0xe6bb4e51, 43 },
    { 0x46bb4e52, 90 },
    { 0x92fb4e53, 33 },
    { 0x104b4e54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfb4e64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cab4e68, 84 },
    { 0x4b3b4e68, 95 },
    { 0xb1ab4e68, 55 },
    { 0x941b4e69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x495b4e6e, 98 },
    { 0x698b4e6f, 65 },
    { 0x647b4e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x688b4e73, 99 },
    { 0, 0 },
    { 0x707b4e75, 90 },
    { 0x590b4e75, 23 },
    { 0x506b4e77, 90 },
    { 0, 0 },
    { 0xbd9b4e79, 23 },
    { 0x270b4e79, 33 },
    { 0x14fb4e7a, 43 },
    { 0x2ccb4e7a, 90 },
    { 0, 0 },
    { 0x63cb4e7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b4e84, 78 },
    { 0xc6db4e85, 76 },
    { 0, 0 },
    { 0x2b4e87, 98 },
    { 0x679b4e88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75b4e8c, 55 },
    { 0x952b4e8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bb4e90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb4e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3b4e9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcb4ea3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b4ea8, 90 },
    { 0x39cb4ea9, 40 },
    { 0x64bb4eaa, 98 },
    { 0, 0 },
    { 0x6bfb4eac, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b4eb2, 3 },
    { 0x478b4eb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619b4eb7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154b4ebb, 61 },
    { 0x9e5b4ebc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3b4ec0, 78 },
    { 0x56fb4ec1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987b4ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5b4ece, 52 },
    { 0xb41b4ece, 98 },
    { 0xa55b4ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86b4ed4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b4ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x718b4eda, 101 },
    { 0, 0 },
    { 0xc2ab4edc, 61 },
    { 0xaf4b4edd, 95 },
    { 0xb04b4ede, 33 },
    { 0x5e6b4edf, 61 },
    { 0xd97b4edf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2b4ee7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9b4eeb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fb4eef, 90 },
    { 0x7fbb4ef0, 85 },
    { 0xe62b4ef0, 91 },
    { 0x8b4ef0, 75 },
    { 0x621b4ef3, 33 },
    { 0x9f3b4ef4, 94 },
    { 0xc88b4ef5, 53 },
    { 0x14eb4ef6, 91 },
    { 0xbd2b4ef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508b4efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bb4f02, 98 },
    { 0x708b4f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eb4f06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9b4f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6b4f0c, 90 },
    { 0x65bb4f0d, 33 },
    { 0xf3b4f0c, 34 },
    { 0, 0 },
    { 0xe1db4f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5b4f13, 95 },
    { 0, 0 },
    { 0xfe2b4f15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eb4f19, 66 },
    { 0, 0 },
    { 0xd4eb4f1b, 61 },
    { 0x922b4f1c, 52 },
    { 0, 0 },
    { 0x531b4f1e, 34 },
    { 0xc4cb4f1f, 90 },
    { 0xe0bb4f1e, 81 },
    { 0xb0bb4f21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b4f24, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b4f27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beb4f36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0b4f3f, 99 },
    { 0x812b4f3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2b4f46, 23 },
    { 0xe7cb4f47, 93 },
    { 0, 0 },
    { 0xa8cb4f49, 97 },
    { 0x8c3b4f4a, 55 },
    { 0x688b4f4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774b4f56, 76 },
    { 0x699b4f56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bb4f60, 53 },
    { 0, 0 },
    { 0x73cb4f62, 61 },
    { 0xeeab4f63, 33 },
    { 0xf1bb4f62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xafeb4f67, 55 },
    { 0x99b4f67, 33 },
    { 0x9f6b4f69, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afb4f71, 95 },
    { 0x8e9b4f72, 52 },
    { 0, 0 },
    { 0x4b9b4f74, 88 },
    { 0x6cdb4f75, 51 },
    { 0xeccb4f76, 55 },
    { 0x458b4f74, 90 },
    { 0, 0 },
    { 0xe42b4f79, 34 },
    { 0xbc4b4f7a, 90 },
    { 0xceeb4f79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222b4f7e, 93 },
    { 0xdb6b4f7f, 90 },
    { 0x71ab4f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bb4f84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9db4f89, 52 },
    { 0, 0 },
    { 0x17eb4f8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beb4f8f, 66 },
    { 0xca6b4f8f, 95 },
    { 0xbe6b4f91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ab4f95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505b4fa0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fb4fab, 34 },
    { 0xa66b4fac, 34 },
    { 0x9f1b4fac, 67 },
    { 0xe7fb4fae, 93 },
    { 0, 0 },
    { 0x984b4fb0, 33 },
    { 0, 0 },
    { 0x6e7b4fb2, 46 },
    { 0x768b4fb2, 90 },
    { 0xb11b4fb4, 95 },
    { 0xefcb4fb3, 90 },
    { 0xb42b4fb6, 55 },
    { 0, 0 },
    { 0xbc7b4fb8, 46 },
    { 0x1fdb4fb8, 23 },
    { 0, 0 },
    { 0x591b4fbb, 98 },
    { 0xdb9b4fbb, 66 },
    { 0x2ab4fbd, 43 },
    { 0xa9cb4fbc, 90 },
    { 0, 0 },
    { 0xa5ab4fc0, 55 },
    { 0x550b4fc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b4fcb, 95 },
    { 0x3e1b4fcb, 90 },
    { 0x69fb4fcc, 33 },
    { 0x6a4b4fce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaab4fd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceb4fd4, 90 },
    { 0, 0 },
    { 0x576b4fd6, 55 },
    { 0, 0 },
    { 0x318b4fd8, 55 },
    { 0, 0 },
    { 0x7bb4fda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27b4fde, 91 },
    { 0, 0 },
    { 0xfabb4fe0, 100 },
    { 0, 0 },
    { 0xde3b4fe2, 61 },
    { 0xab5b4fe2, 66 },
    { 0xaa6b4fe4, 95 },
    { 0x19db4fe5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb4fe9, 67 },
    { 0x928b4fea, 99 },
    { 0x12cb4feb, 61 },
    { 0, 0 },
    { 0xe5cb4fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eab4ff0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518b4ff7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ab4ffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5b4ffd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47b5005, 74 },
    { 0xc2eb5005, 46 },
    { 0xfd6b5007, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4b500d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b5012, 77 },
    { 0x8e8b5013, 98 },
    { 0x4bab5013, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecb5019, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4b501c, 55 },
    { 0xa2eb501d, 69 },
    { 0, 0 },
    { 0x898b501f, 76 },
    { 0x4e1b501f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6b5028, 93 },
    { 0x36b5029, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8db502c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98b5032, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393b5039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9b503f, 33 },
    { 0x218b5040, 98 },
    { 0, 0 },
    { 0xa9b5042, 98 },
    { 0x203b5043, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bb5048, 34 },
    { 0x6a7b5049, 61 },
    { 0, 0 },
    { 0x101b504b, 61 },
    { 0xdcab504c, 33 },
    { 0xa44b504b, 33 },
    { 0x154b504e, 46 },
    { 0x3b4b504c, 98 },
    { 0x97fb5050, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cb5053, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x348b5056, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x676b5059, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05b5060, 77 },
    { 0xa19b5061, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116b5065, 23 },
    { 0xc93b5066, 40 },
    { 0x4a9b5067, 76 },
    { 0xb5ab5066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44b506b, 98 },
    { 0, 0 },
    { 0xff5b506d, 43 },
    { 0, 0 },
    { 0xb85b506f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334b5081, 91 },
    { 0x6aab5082, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7b5087, 65 },
    { 0, 0 },
    { 0x979b5089, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b508c, 90 },
    { 0xe2fb508d, 99 },
    { 0x54b508d, 100 },
    { 0, 0 },
    { 0x318b5090, 34 },
    { 0x571b5091, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723b5098, 77 },
    { 0x620b5099, 34 },
    { 0x7a1b509a, 76 },
    { 0x601b509a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460b509f, 97 },
    { 0, 0 },
    { 0x4c5b50a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b50ab, 61 },
    { 0xdd1b50ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963b50b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafb50b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb50bd, 98 },
    { 0x568b50bd, 90 },
    { 0x50cb50bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ab50c5, 98 },
    { 0x2abb50c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9b50cf, 33 },
    { 0x5c6b50cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb50d3, 85 },
    { 0, 0 },
    { 0xc96b50d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583b50e0, 91 },
    { 0, 0 },
    { 0xf31b50e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b50e5, 17 },
    { 0x2bab50e6, 80 },
    { 0x3edb50e7, 46 },
    { 0x61eb50e8, 77 },
    { 0x906b50e8, 61 },
    { 0x880b50ea, 61 },
    { 0x24cb50ea, 98 },
    { 0xa94b50ea, 90 },
    { 0xa15b50ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb50f0, 46 },
    { 0, 0 },
    { 0xf03b50f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493b50f6, 98 },
    { 0x900b50f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bb50fb, 61 },
    { 0x90b50fc, 43 },
    { 0, 0 },
    { 0x692b50fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b5107, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4b510a, 17 },
    { 0x7adb510b, 90 },
    { 0, 0 },
    { 0x70ab510d, 52 },
    { 0, 0 },
    { 0x935b510f, 91 },
    { 0x79b5110, 56 },
    { 0xec0b510f, 23 },
    { 0x717b5112, 90 },
    { 0xb80b5113, 91 },
    { 0x628b5113, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb5117, 66 },
    { 0, 0 },
    { 0xd00b5119, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9b511d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feb5121, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459b5126, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdb512a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20b512f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31b5134, 61 },
    { 0, 0 },
    { 0x26fb5136, 99 },
    { 0x752b5137, 33 },
    { 0xfebb5138, 76 },
    { 0xc1b5139, 53 },
    { 0x7afb513a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cb513e, 23 },
    { 0, 0 },
    { 0xbaab5140, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14b514a, 90 },
    { 0x422b514b, 90 },
    { 0x2f0b514c, 90 },
    { 0x820b514b, 78 },
    { 0, 0 },
    { 0xc31b514f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3b5153, 76 },
    { 0x162b5154, 99 },
    { 0x6e6b5153, 93 },
    { 0x604b5156, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8b515c, 76 },
    { 0x2a4b515d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb5160, 80 },
    { 0xf92b5161, 23 },
    { 0xc00b5160, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b5166, 67 },
    { 0xbb0b5167, 75 },
    { 0, 0 },
    { 0x85cb5169, 78 },
    { 0xed2b516a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8eb516f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6b5177, 55 },
    { 0x605b5178, 90 },
    { 0x945b5177, 95 },
    { 0xd2eb517a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdb5181, 52 },
    { 0x820b5182, 19 },
    { 0, 0 },
    { 0xc77b5184, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbb5188, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21db518b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x303b518e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cb5191, 90 },
    { 0, 0 },
    { 0x375b5193, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9b5196, 78 },
    { 0, 0 },
    { 0x225b5198, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304b519d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58b51a0, 98 },
    { 0x74ab51a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b51a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb51a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb51b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bb51ba, 94 },
    { 0, 0 },
    { 0xa37b51bc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb51c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cb51ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cb51cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847b51d8, 34 },
    { 0xbe2b51d9, 23 },
    { 0x6f0b51da, 90 },
    { 0xaa6b51db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eb51df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab51e6, 61 },
    { 0, 0 },
    { 0xa63b51e8, 98 },
    { 0x6b51e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957b51ed, 78 },
    { 0x58ab51ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6b51f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02b51f6, 34 },
    { 0, 0 },
    { 0xb3b51f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7b51fd, 98 },
    { 0x5aab51fe, 61 },
    { 0xa73b51fd, 53 },
    { 0, 0 },
    { 0xe4ab5201, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33b5207, 85 },
    { 0, 0 },
    { 0x531b5209, 23 },
    { 0xf30b5209, 76 },
    { 0, 0 },
    { 0x5f1b520c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b5211, 33 },
    { 0x5efb5212, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b5218, 90 },
    { 0xfd7b5218, 80 },
    { 0x48bb521a, 95 },
    { 0x11b521b, 88 },
    { 0x48cb521c, 76 },
    { 0x66cb521a, 98 },
    { 0xec2b5218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eb5225, 99 },
    { 0xc99b5226, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x747b5229, 61 },
    { 0, 0 },
    { 0x2a7b522b, 90 },
    { 0x213b522b, 61 },
    { 0xcc3b522d, 76 },
    { 0x57eb522d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33b5231, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ab5234, 14 },
    { 0, 0 },
    { 0xdbdb5236, 93 },
    { 0xfc5b5237, 52 },
    { 0x4e6b5238, 23 },
    { 0x878b5237, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fb523c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bb5240, 33 },
    { 0x74bb5241, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fb5244, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0b5247, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfb524a, 61 },
    { 0, 0 },
    { 0x158b524c, 88 },
    { 0, 0 },
    { 0x747b524e, 61 },
    { 0, 0 },
    { 0xc07b5250, 93 },
    { 0, 0 },
    { 0x56eb5252, 99 },
    { 0x686b5253, 55 },
    { 0x56db5253, 84 },
    { 0x9c5b5252, 95 },
    { 0x7ccb5252, 90 },
    { 0xdceb5257, 38 },
    { 0xfaab5256, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bb525d, 76 },
    { 0, 0 },
    { 0xa5ab525f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x17db5262, 98 },
    { 0x317b5262, 98 },
    { 0x8afb5264, 61 },
    { 0x761b5264, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6b526b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b526e, 33 },
    { 0x961b526f, 87 },
    { 0, 0 },
    { 0xf53b5271, 100 },
    { 0x3cb5271, 61 },
    { 0x6b0b5273, 70 },
    { 0x47fb5274, 23 },
    { 0xd3fb5275, 88 },
    { 0xef1b5274, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efb5280, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8b5289, 51 },
    { 0xdadb528a, 95 },
    { 0xeabb528b, 55 },
    { 0x552b528b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eb5290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x274b5293, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8db5296, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eb529b, 93 },
    { 0xb6b529b, 68 },
    { 0x55b529d, 76 },
    { 0x15eb529d, 82 },
    { 0x413b529e, 23 },
    { 0x2f5b52a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b52a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eab52b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318b52b7, 43 },
    { 0x655b52b8, 43 },
    { 0xdb3b52b7, 33 },
    { 0xd59b52b7, 76 },
    { 0x34ab52bb, 34 },
    { 0x360b52bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8b52c2, 23 },
    { 0, 0 },
    { 0xa4eb52c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3b52c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250b52cb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8b52ce, 61 },
    { 0xf8b52ce, 52 },
    { 0x74cb52ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fb52da, 43 },
    { 0xb14b52db, 82 },
    { 0x531b52dc, 85 },
    { 0, 0 },
    { 0xd6bb52de, 61 },
    { 0, 0 },
    { 0xed1b52e0, 65 },
    { 0x744b52e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5b52e8, 100 },
    { 0xf1bb52e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bb52ed, 95 },
    { 0xd57b52ee, 23 },
    { 0x106b52ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b52f2, 61 },
    { 0x126b52f2, 90 },
    { 0xe90b52f3, 33 },
    { 0, 0 },
    { 0x144b52f6, 23 },
    { 0xc99b52f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628b52fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b5304, 90 },
    { 0, 0 },
    { 0x60b5306, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb5309, 90 },
    { 0x2dfb530a, 98 },
    { 0x409b5309, 33 },
    { 0x115b530b, 90 },
    { 0, 0 },
    { 0x8cdb530e, 33 },
    { 0x4dfb530f, 33 },
    { 0xd27b530f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92b5313, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecb5316, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987b531a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b5323, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b5327, 61 },
    { 0x3d9b5327, 98 },
    { 0x581b5329, 61 },
    { 0x6f2b5328, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfb5333, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0b5336, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423b533a, 52 },
    { 0, 0 },
    { 0x7f1b533c, 78 },
    { 0, 0 },
    { 0x4f4b533e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x612b5341, 52 },
    { 0x2c3b5342, 61 },
    { 0, 0 },
    { 0x62ab5344, 98 },
    { 0, 0 },
    { 0x7bbb5346, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1b5349, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25b534c, 90 },
    { 0, 0 },
    { 0xb65b534e, 23 },
    { 0x255b534e, 23 },
    { 0x92db534e, 75 },
    { 0x2a8b5351, 93 },
    { 0x577b5352, 65 },
    { 0xa3b5353, 27 },
    { 0x4d1b5352, 76 },
    { 0x56fb5355, 90 },
    { 0x219b5354, 100 },
    { 0x6a6b5357, 95 },
    { 0xa12b5358, 78 },
    { 0x4f1b5359, 76 },
    { 0x4b3b5359, 46 },
    { 0xb2fb5354, 55 },
    { 0xa55b535c, 61 },
    { 0, 0 },
    { 0x460b535e, 90 },
    { 0xd43b535e, 76 },
    { 0, 0 },
    { 0x860b5361, 98 },
    { 0x712b5362, 34 },
    { 0x625b5362, 75 },
    { 0x87eb5364, 98 },
    { 0xb62b5365, 33 },
    { 0, 0 },
    { 0x86eb5367, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905b536b, 89 },
    { 0x681b536c, 91 },
    { 0xfb5b536b, 98 },
    { 0x47eb536e, 38 },
    { 0, 0 },
    { 0x9c8b5370, 34 },
    { 0x9acb5370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b5374, 61 },
    { 0x84fb5375, 95 },
    { 0x15cb5376, 55 },
    { 0xf43b5377, 78 },
    { 0x63ab5376, 95 },
    { 0xbb0b5379, 23 },
    { 0, 0 },
    { 0xcccb537b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35b5381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ab538e, 90 },
    { 0x6a5b538f, 34 },
    { 0x3a8b538e, 93 },
    { 0x66b5391, 61 },
    { 0, 0 },
    { 0xa96b5393, 90 },
    { 0x628b5394, 34 },
    { 0, 0 },
    { 0x549b5396, 90 },
    { 0, 0 },
    { 0x642b5398, 61 },
    { 0xefcb5399, 55 },
    { 0x5e7b5399, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b53a3, 33 },
    { 0x964b53a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x453b53a7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56b53ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8b53af, 33 },
    { 0xfb6b53b0, 90 },
    { 0x61fb53b1, 90 },
    { 0xc93b53b2, 98 },
    { 0x9e0b53b3, 61 },
    { 0xaab53b4, 90 },
    { 0x103b53b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fb53b9, 90 },
    { 0xab5b53b9, 91 },
    { 0x85b53bb, 95 },
    { 0x3f6b53bc, 66 },
    { 0, 0 },
    { 0x791b53be, 76 },
    { 0xfa5b53bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bb53c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27db53c6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48db53cc, 43 },
    { 0x17eb53cd, 91 },
    { 0x8ccb53cc, 90 },
    { 0x486b53ce, 98 },
    { 0x489b53cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687b53d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233b53db, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eb53e3, 90 },
    { 0x861b53e4, 90 },
    { 0x6ccb53e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb53ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b53f4, 85 },
    { 0xceb53f4, 23 },
    { 0xac8b53f5, 91 },
    { 0x3b7b53f7, 78 },
    { 0x1e3b53f7, 23 },
    { 0xaf4b53f9, 90 },
    { 0x135b53f8, 61 },
    { 0x9c1b53f5, 98 },
    { 0xb35b53fc, 33 },
    { 0xd78b53fc, 76 },
    { 0xdc0b53f8, 43 },
    { 0, 0 },
    { 0x61bb5400, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b5406, 90 },
    { 0, 0 },
    { 0xccbb5408, 33 },
    { 0, 0 },
    { 0xe09b540a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806b5412, 93 },
    { 0xb6cb5413, 95 },
    { 0x223b5414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224b541d, 90 },
    { 0, 0 },
    { 0x13bb541f, 98 },
    { 0x19cb5420, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeb5424, 55 },
    { 0xe48b5425, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eb542a, 90 },
    { 0xf56b542b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31b542e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb5432, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269b5438, 46 },
    { 0xb04b5439, 46 },
    { 0xdefb543a, 33 },
    { 0xc69b543a, 34 },
    { 0xe1cb543b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197b5440, 85 },
    { 0xe5eb5441, 33 },
    { 0xdbfb5442, 76 },
    { 0x861b5443, 23 },
    { 0, 0 },
    { 0x8a3b5445, 95 },
    { 0x146b5445, 98 },
    { 0, 0 },
    { 0xb4bb5448, 34 },
    { 0x4cfb5448, 94 },
    { 0xb5fb544a, 23 },
    { 0, 0 },
    { 0x52ab544c, 76 },
    { 0, 0 },
    { 0x1a8b544e, 43 },
    { 0x90bb544f, 98 },
    { 0x33eb544e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cb5453, 91 },
    { 0, 0 },
    { 0x4c1b5455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bb5458, 61 },
    { 0x7feb5459, 91 },
    { 0xd10b545a, 90 },
    { 0x74eb545a, 34 },
    { 0, 0 },
    { 0x61db545d, 23 },
    { 0xf59b545d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826b5463, 67 },
    { 0xac5b5464, 90 },
    { 0x28bb5465, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985b5469, 99 },
    { 0, 0 },
    { 0x5e7b546b, 23 },
    { 0xd2db546b, 61 },
    { 0xc4bb546c, 76 },
    { 0x77cb546e, 76 },
    { 0xbd1b546f, 90 },
    { 0x3bdb546e, 56 },
    { 0xa2ab5471, 33 },
    { 0xabab5471, 65 },
    { 0x888b5473, 33 },
    { 0x368b5474, 61 },
    { 0xb9bb5473, 95 },
    { 0xd40b5472, 52 },
    { 0x104b5477, 90 },
    { 0x2c8b5477, 33 },
    { 0, 0 },
    { 0x73b547a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442b5481, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x812b5484, 38 },
    { 0x58bb5485, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cb548b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb548e, 99 },
    { 0, 0 },
    { 0xa6bb5490, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821b5497, 87 },
    { 0, 0 },
    { 0xfc8b5499, 33 },
    { 0, 0 },
    { 0x6e8b549b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4b549e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb54a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdb54a7, 61 },
    { 0xd37b54a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9b54ac, 23 },
    { 0x4dbb54ad, 78 },
    { 0, 0 },
    { 0xb56b54af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfb54b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5b54bb, 43 },
    { 0x9e5b54bb, 55 },
    { 0, 0 },
    { 0x568b54be, 34 },
    { 0, 0 },
    { 0x1f3b54c0, 84 },
    { 0, 0 },
    { 0x5b1b54c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6b54c8, 87 },
    { 0x212b54c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab54cd, 61 },
    { 0xfe0b54ce, 90 },
    { 0xd1fb54cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1b54d3, 91 },
    { 0, 0 },
    { 0xdc2b54d5, 67 },
    { 0x927b54d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929b54da, 43 },
    { 0xd8eb54da, 100 },
    { 0x3acb54da, 93 },
    { 0, 0 },
    { 0x526b54de, 61 },
    { 0x821b54df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239b54e5, 33 },
    { 0, 0 },
    { 0x37b54e7, 34 },
    { 0x9b0b54e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3b54eb, 34 },
    { 0x368b54eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681b54f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6b54f7, 94 },
    { 0, 0 },
    { 0xc9b54f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6b54fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05b54ff, 95 },
    { 0x60ab5500, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50b5503, 75 },
    { 0x1d6b5504, 23 },
    { 0x4f2b5505, 90 },
    { 0x272b5504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fb550c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13b5512, 23 },
    { 0xb68b5513, 46 },
    { 0x322b5514, 95 },
    { 0, 0 },
    { 0xe5bb5516, 23 },
    { 0x490b5517, 33 },
    { 0x8c9b5518, 99 },
    { 0x654b5517, 90 },
    { 0xcc6b551a, 23 },
    { 0, 0 },
    { 0xbafb551c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55b5521, 66 },
    { 0xe0bb5522, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbb5525, 90 },
    { 0xeb1b5525, 78 },
    { 0, 0 },
    { 0x9a3b5528, 77 },
    { 0xaf7b5529, 69 },
    { 0, 0 },
    { 0xd20b552b, 55 },
    { 0x56cb552c, 55 },
    { 0xe85b552d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5b5532, 88 },
    { 0xf41b5533, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83b5536, 95 },
    { 0, 0 },
    { 0xddeb5538, 55 },
    { 0x401b5539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4b553c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x653b553f, 67 },
    { 0, 0 },
    { 0x9e1b5541, 98 },
    { 0xa32b5541, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cb554a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cb5556, 33 },
    { 0x9e4b5557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878b555b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59b5560, 61 },
    { 0, 0 },
    { 0x75fb5562, 40 },
    { 0, 0 },
    { 0xc94b5564, 33 },
    { 0x1b7b5565, 61 },
    { 0x1dfb5565, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4b5569, 100 },
    { 0x60b556a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bb556e, 61 },
    { 0x5f4b556e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9b5572, 88 },
    { 0, 0 },
    { 0x420b5574, 61 },
    { 0, 0 },
    { 0x732b5576, 78 },
    { 0x2c0b5577, 98 },
    { 0x649b5578, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9b557b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x475b557e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beb5584, 34 },
    { 0x632b5585, 55 },
    { 0x84eb5585, 55 },
    { 0x319b5587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b558a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750b5591, 95 },
    { 0x754b5592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ab5595, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298b559c, 94 },
    { 0xb1fb559d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9b55a3, 33 },
    { 0x1eb55a4, 92 },
    { 0x9a9b55a5, 69 },
    { 0, 0 },
    { 0xc5b55a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ab55ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x675b55ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fb55b6, 90 },
    { 0, 0 },
    { 0x428b55b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542b55bc, 43 },
    { 0, 0 },
    { 0x395b55be, 61 },
    { 0, 0 },
    { 0x1dfb55c0, 34 },
    { 0x382b55c0, 33 },
    { 0xf50b55c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x245b55c5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270b55c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848b55ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fb55d6, 90 },
    { 0, 0 },
    { 0xdc0b55d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfb55dc, 61 },
    { 0x3b0b55dd, 17 },
    { 0xd35b55dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72b55e4, 76 },
    { 0x2f7b55e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cb55e8, 95 },
    { 0x8cdb55e9, 33 },
    { 0x4e5b55ea, 23 },
    { 0, 0 },
    { 0x275b55ec, 52 },
    { 0xbf5b55ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b55f0, 98 },
    { 0xd28b55f1, 98 },
    { 0x78cb55f1, 76 },
    { 0, 0 },
    { 0x94bb55f4, 93 },
    { 0, 0 },
    { 0x595b55f6, 12 },
    { 0x7c0b55f7, 34 },
    { 0, 0 },
    { 0x293b55f9, 95 },
    { 0x351b55f9, 33 },
    { 0xb3fb55fb, 95 },
    { 0xefeb55fc, 61 },
    { 0xc14b55fd, 23 },
    { 0xbb9b55fd, 56 },
    { 0x61db55fd, 95 },
    { 0x715b55ff, 23 },
    { 0xa59b55fd, 90 },
    { 0, 0 },
    { 0x916b5603, 46 },
    { 0, 0 },
    { 0x362b5605, 67 },
    { 0xff0b5605, 46 },
    { 0, 0 },
    { 0x9c1b5608, 98 },
    { 0x508b5609, 23 },
    { 0x635b560a, 95 },
    { 0xb1fb560a, 61 },
    { 0, 0 },
    { 0x52eb560d, 33 },
    { 0, 0 },
    { 0x2d4b560f, 81 },
    { 0, 0 },
    { 0xabdb5611, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x760b5614, 61 },
    { 0xed5b5615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480b561a, 98 },
    { 0xa7b561a, 33 },
    { 0x303b561c, 95 },
    { 0x40fb561d, 91 },
    { 0xd47b561d, 95 },
    { 0, 0 },
    { 0xf8bb5620, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35b5623, 98 },
    { 0xc0db5624, 33 },
    { 0x618b5625, 55 },
    { 0, 0 },
    { 0xd4ab5627, 33 },
    { 0, 0 },
    { 0xd81b5629, 76 },
    { 0x220b562a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ceb562d, 33 },
    { 0, 0 },
    { 0x9f5b562f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x477b5632, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x817b5635, 55 },
    { 0x882b5636, 98 },
    { 0, 0 },
    { 0xf8fb5638, 81 },
    { 0, 0 },
    { 0x24bb563a, 98 },
    { 0xe18b563a, 23 },
    { 0x849b563c, 77 },
    { 0x959b563a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2b5640, 61 },
    { 0xb3eb5641, 81 },
    { 0x470b5640, 96 },
    { 0x728b5641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154b564e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab5653, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377b5658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eb5661, 84 },
    { 0x576b5662, 76 },
    { 0x36bb5661, 98 },
    { 0xe60b5664, 84 },
    { 0x7b5b5664, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bb566c, 93 },
    { 0xe24b566d, 61 },
    { 0xa70b566c, 32 },
    { 0x95bb566c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7db5675, 78 },
    { 0xa41b5675, 98 },
    { 0x558b5677, 90 },
    { 0x429b5677, 87 },
    { 0x76ab5678, 76 },
    { 0xe5cb5675, 61 },
    { 0x3b5b5675, 85 },
    { 0x523b567c, 95 },
    { 0x92fb5678, 90 },
    { 0x7f2b5679, 33 },
    { 0x539b567e, 100 },
    { 0xcfcb5680, 33 },
    { 0, 0 },
    { 0x358b5682, 100 },
    { 0xa52b5683, 90 },
    { 0, 0 },
    { 0xb1fb5685, 90 },
    { 0xe71b5686, 76 },
    { 0xae7b5687, 77 },
    { 0x49cb5686, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fb568b, 96 },
    { 0, 0 },
    { 0x14fb568d, 95 },
    { 0x44cb568e, 33 },
    { 0xb5b568f, 98 },
    { 0x2d2b5690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4b5694, 76 },
    { 0x7afb5694, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b569d, 98 },
    { 0x193b569e, 70 },
    { 0xd41b569f, 98 },
    { 0, 0 },
    { 0xa1cb56a1, 33 },
    { 0x11cb56a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacab56a6, 68 },
    { 0xf66b56a7, 55 },
    { 0x75db56a8, 33 },
    { 0xb2db56a7, 90 },
    { 0xbdab56a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833b56b0, 33 },
    { 0x985b56b0, 95 },
    { 0x11cb56b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25db56b8, 95 },
    { 0x72fb56b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bb56bd, 90 },
    { 0x22fb56bd, 34 },
    { 0x6b0b56be, 90 },
    { 0x497b56c0, 33 },
    { 0, 0 },
    { 0x23cb56c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db56c7, 46 },
    { 0, 0 },
    { 0xf6cb56c9, 93 },
    { 0x3c1b56ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cb56d4, 95 },
    { 0xe70b56d4, 33 },
    { 0, 0 },
    { 0x780b56d7, 61 },
    { 0, 0 },
    { 0x155b56d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb56de, 61 },
    { 0x53db56de, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19b56e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b56e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab56e9, 61 },
    { 0x292b56ea, 90 },
    { 0x859b56eb, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94b56f2, 90 },
    { 0xfa3b56f3, 34 },
    { 0xbd7b56f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38b56f7, 43 },
    { 0xa1b56f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772b56fc, 98 },
    { 0, 0 },
    { 0xd3bb56fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1b5702, 98 },
    { 0xa57b5703, 33 },
    { 0x332b5704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ab5707, 33 },
    { 0, 0 },
    { 0x2d1b5709, 33 },
    { 0, 0 },
    { 0xb4b570b, 90 },
    { 0x8cb570c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b5711, 76 },
    { 0, 0 },
    { 0x8ffb5713, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x855b5716, 23 },
    { 0x6dfb5716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226b571b, 33 },
    { 0x5ccb571c, 95 },
    { 0xe21b571d, 75 },
    { 0x9e0b571d, 46 },
    { 0xa1b571f, 98 },
    { 0x873b571c, 34 },
    { 0x82ab5721, 55 },
    { 0x792b5722, 33 },
    { 0x584b571b, 33 },
    { 0x6c9b5724, 55 },
    { 0xffdb5725, 23 },
    { 0, 0 },
    { 0x70fb5727, 34 },
    { 0x1e8b5728, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06b572b, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391b5731, 52 },
    { 0, 0 },
    { 0xc81b5733, 77 },
    { 0xdc6b5733, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb5744, 98 },
    { 0x390b5744, 98 },
    { 0, 0 },
    { 0xa5eb5747, 23 },
    { 0xcbdb5748, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3b574b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68db574f, 98 },
    { 0x9f2b5750, 66 },
    { 0x521b5751, 23 },
    { 0x22b5752, 34 },
    { 0x8ccb5753, 46 },
    { 0, 0 },
    { 0x2aab5755, 55 },
    { 0xa66b5755, 33 },
    { 0x4aab5757, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bb575b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fb575f, 100 },
    { 0xa38b575f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23b576a, 90 },
    { 0xc92b576b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ab5770, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ab5773, 95 },
    { 0x72bb5773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2b577c, 98 },
    { 0x99fb577c, 33 },
    { 0xdd7b577e, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4b5783, 76 },
    { 0xd63b5784, 46 },
    { 0, 0 },
    { 0x8fcb5786, 95 },
    { 0xb4b5786, 95 },
    { 0, 0 },
    { 0xa98b5789, 98 },
    { 0x5f3b578a, 64 },
    { 0x4bab578b, 61 },
    { 0, 0 },
    { 0x61b578d, 61 },
    { 0, 0 },
    { 0xbecb578f, 33 },
    { 0x69b5790, 33 },
    { 0x29eb578f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313b5798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x803b579b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54b579f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b57a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379b57ae, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fb57b5, 101 },
    { 0x926b57b5, 98 },
    { 0x17eb57b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb57ba, 76 },
    { 0, 0 },
    { 0x578b57bc, 98 },
    { 0x67ab57bd, 34 },
    { 0xf7ab57be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680b57c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fb57ca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83b57d1, 43 },
    { 0xbccb57d1, 34 },
    { 0, 0 },
    { 0xb11b57d4, 77 },
    { 0xd8db57d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edb57d9, 90 },
    { 0xaf9b57da, 90 },
    { 0x53db57db, 23 },
    { 0x80fb57dc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abb57e3, 95 },
    { 0, 0 },
    { 0x9fdb57e5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7b57e9, 34 },
    { 0, 0 },
    { 0xee7b57eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9b57ee, 34 },
    { 0, 0 },
    { 0x3fbb57f0, 90 },
    { 0x2bab57f0, 66 },
    { 0x9ab57f2, 87 },
    { 0x2a1b57f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb57f9, 90 },
    { 0x8cfb57fa, 93 },
    { 0x518b57fa, 76 },
    { 0xae8b57fc, 61 },
    { 0xc88b57fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46b5801, 34 },
    { 0, 0 },
    { 0xdbdb5803, 61 },
    { 0x70fb5804, 76 },
    { 0x704b5805, 23 },
    { 0xf90b5803, 98 },
    { 0x88bb5807, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bb580d, 88 },
    { 0x841b580e, 43 },
    { 0x709b580e, 90 },
    { 0x2b0b5810, 87 },
    { 0, 0 },
    { 0x1c8b5812, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341b5816, 46 },
    { 0xc98b5816, 92 },
    { 0x4a6b5816, 23 },
    { 0x675b5816, 65 },
    { 0x9dab581a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86b581f, 90 },
    { 0xf8ab5820, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995b582b, 93 },
    { 0, 0 },
    { 0xa81b582d, 61 },
    { 0x739b582e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9b5833, 43 },
    { 0x1deb5834, 75 },
    { 0, 0 },
    { 0x334b5836, 88 },
    { 0xe23b5837, 90 },
    { 0xc39b5836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b583c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721b5841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0b5845, 40 },
    { 0, 0 },
    { 0x26fb5847, 61 },
    { 0, 0 },
    { 0xba8b5849, 54 },
    { 0x383b5849, 90 },
    { 0x47b584b, 55 },
    { 0x6abb584c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ab5850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39b5857, 43 },
    { 0x31ab5857, 33 },
    { 0xf41b5858, 34 },
    { 0xdfab585a, 43 },
    { 0x2b8b585b, 95 },
    { 0x378b585a, 61 },
    { 0xd2eb5857, 98 },
    { 0, 0 },
    { 0xc2cb585f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x940b5862, 95 },
    { 0x2ccb5863, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bb5868, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dab5870, 33 },
    { 0xef7b5871, 39 },
    { 0xaf4b5870, 34 },
    { 0x3a7b5873, 33 },
    { 0xb33b5874, 58 },
    { 0x4c3b5875, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827b5880, 61 },
    { 0, 0 },
    { 0x49b5882, 33 },
    { 0x7c5b5883, 55 },
    { 0xb2cb5884, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cb5897, 78 },
    { 0xb4fb5898, 84 },
    { 0x852b5899, 33 },
    { 0x555b589a, 91 },
    { 0xe16b589a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fb58a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdab58a3, 77 },
    { 0x397b58a3, 100 },
    { 0xd0bb58a5, 61 },
    { 0x9eab58a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927b58ac, 61 },
    { 0xa8b58ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00b58b0, 81 },
    { 0x3fbb58b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b58bc, 90 },
    { 0, 0 },
    { 0x1f8b58be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbb58c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66b58c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b58cb, 34 },
    { 0x5e7b58cc, 93 },
    { 0x5a6b58cd, 66 },
    { 0, 0 },
    { 0x11ab58cf, 52 },
    { 0, 0 },
    { 0x148b58d1, 33 },
    { 0xcc0b58d2, 33 },
    { 0xcfcb58d3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17b58d8, 46 },
    { 0xa24b58d9, 95 },
    { 0xb99b58da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64b58dd, 90 },
    { 0, 0 },
    { 0x812b58df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939b58e9, 52 },
    { 0x611b58e9, 95 },
    { 0xaa3b58eb, 90 },
    { 0x81cb58ec, 61 },
    { 0, 0 },
    { 0x65eb58ee, 90 },
    { 0xf9ab58ef, 55 },
    { 0xb6ab58f0, 61 },
    { 0x3b7b58ef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32b58f4, 98 },
    { 0x86b58f5, 99 },
    { 0xaf2b58f5, 90 },
    { 0xda3b58f7, 53 },
    { 0xbb58f8, 53 },
    { 0x3eb58f6, 98 },
    { 0x22bb58f4, 46 },
    { 0x77eb58f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb58fe, 55 },
    { 0, 0 },
    { 0xe7cb5900, 33 },
    { 0x958b5901, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3deb5905, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b5909, 98 },
    { 0, 0 },
    { 0x52ab590b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bb590f, 90 },
    { 0x77eb590f, 40 },
    { 0x908b590f, 61 },
    { 0xa9eb5912, 23 },
    { 0xb81b5913, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b5916, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81b591a, 33 },
    { 0xdd2b591b, 43 },
    { 0, 0 },
    { 0xd60b591d, 23 },
    { 0xe6fb591e, 93 },
    { 0xf43b591e, 76 },
    { 0x192b591d, 33 },
    { 0x994b5920, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eb5925, 98 },
    { 0xf5ab5926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84b5929, 85 },
    { 0x682b592a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0b592d, 95 },
    { 0xc61b592e, 46 },
    { 0, 0 },
    { 0xe2ab5930, 23 },
    { 0x455b5931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4b5935, 43 },
    { 0x94b5935, 61 },
    { 0x1e6b5936, 61 },
    { 0x2db5938, 98 },
    { 0x83ab5939, 52 },
    { 0x7c2b593a, 77 },
    { 0, 0 },
    { 0xb12b593c, 61 },
    { 0, 0 },
    { 0xd70b593e, 85 },
    { 0x8b593f, 69 },
    { 0xa76b5940, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386b5944, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeb5950, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbb5953, 98 },
    { 0x647b5954, 40 },
    { 0x702b5955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34b5958, 69 },
    { 0xe00b5959, 90 },
    { 0x100b5959, 68 },
    { 0xa91b595a, 61 },
    { 0x5dbb595c, 90 },
    { 0xa6cb595d, 23 },
    { 0xc50b595b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8b5964, 33 },
    { 0, 0 },
    { 0xf6fb5966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eb596a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6b5970, 95 },
    { 0x22b5970, 52 },
    { 0, 0 },
    { 0x1c8b5973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0b5977, 61 },
    { 0x223b5978, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb597c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeb597f, 23 },
    { 0x34cb5980, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8b5989, 23 },
    { 0, 0 },
    { 0xd6db598b, 90 },
    { 0x752b598b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33b598f, 61 },
    { 0xc5db5990, 40 },
    { 0, 0 },
    { 0x709b5992, 61 },
    { 0, 0 },
    { 0x920b5994, 23 },
    { 0, 0 },
    { 0xa90b5996, 88 },
    { 0x909b5996, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6deb599a, 99 },
    { 0x5b2b599b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40b59a4, 91 },
    { 0x7ceb59a4, 33 },
    { 0, 0 },
    { 0x6e9b59a7, 91 },
    { 0x719b59a8, 87 },
    { 0, 0 },
    { 0x613b59aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aab59b2, 52 },
    { 0x596b59b2, 17 },
    { 0x463b59b4, 84 },
    { 0x2f0b59b5, 66 },
    { 0x589b59b4, 55 },
    { 0x3ecb59b7, 92 },
    { 0, 0 },
    { 0x4d2b59b9, 100 },
    { 0x9a9b59ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x970b59bd, 98 },
    { 0x5a0b59bd, 51 },
    { 0x56fb59bf, 90 },
    { 0x672b59c0, 23 },
    { 0x5c1b59c1, 33 },
    { 0x8b5b59c1, 95 },
    { 0xf6ab59c3, 48 },
    { 0xe5eb59c3, 46 },
    { 0x986b59c3, 23 },
    { 0x3c1b59c3, 90 },
    { 0x67eb59c7, 61 },
    { 0xd7b59c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5db59cc, 46 },
    { 0, 0 },
    { 0xb8ab59ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29b59d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eeb59d5, 95 },
    { 0, 0 },
    { 0x742b59d7, 33 },
    { 0xe5cb59d8, 23 },
    { 0x599b59d9, 61 },
    { 0, 0 },
    { 0x399b59db, 100 },
    { 0x7eb59db, 43 },
    { 0, 0 },
    { 0x4cbb59de, 76 },
    { 0, 0 },
    { 0xa6db59e0, 66 },
    { 0xef0b59e0, 33 },
    { 0x29ab59e1, 46 },
    { 0x4cfb59e3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9b59e6, 98 },
    { 0, 0 },
    { 0x865b59e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab59ed, 61 },
    { 0, 0 },
    { 0xee5b59ef, 98 },
    { 0x49bb59ef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ab59f5, 33 },
    { 0x1b2b59f6, 33 },
    { 0xe9eb59f7, 90 },
    { 0xe7eb59f8, 98 },
    { 0x43bb59f6, 95 },
    { 0xf44b59fa, 98 },
    { 0, 0 },
    { 0x5f7b59fc, 55 },
    { 0, 0 },
    { 0x607b59fe, 78 },
    { 0x43eb59ff, 9 },
    { 0x316b59ff, 53 },
    { 0x6d6b59fe, 43 },
    { 0x353b5a02, 23 },
    { 0xb1eb5a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9b5a06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167b5a0c, 33 },
    { 0x515b5a0d, 98 },
    { 0xcccb5a0c, 90 },
    { 0xad7b5a0f, 83 },
    { 0x68eb5a0f, 90 },
    { 0x996b5a10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64b5a14, 89 },
    { 0x248b5a15, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173b5a19, 78 },
    { 0x6d1b5a1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74eb5a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46b5a23, 83 },
    { 0x652b5a23, 43 },
    { 0x214b5a25, 76 },
    { 0xd03b5a26, 100 },
    { 0xba6b5a27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3b5a2b, 85 },
    { 0x8b5a2c, 90 },
    { 0x843b5a2d, 98 },
    { 0xfdab5a2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16b5a3a, 90 },
    { 0, 0 },
    { 0x697b5a3c, 67 },
    { 0, 0 },
    { 0x1a0b5a3e, 99 },
    { 0xa12b5a3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x344b5a42, 99 },
    { 0x27fb5a42, 61 },
    { 0x3c0b5a43, 87 },
    { 0x55bb5a45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22b5a49, 81 },
    { 0x370b5a4a, 90 },
    { 0x52fb5a4b, 90 },
    { 0, 0 },
    { 0x6d5b5a4d, 67 },
    { 0, 0 },
    { 0x8a0b5a4f, 87 },
    { 0x7dcb5a50, 98 },
    { 0x37b5a51, 84 },
    { 0, 0 },
    { 0x788b5a53, 55 },
    { 0, 0 },
    { 0xb4eb5a55, 96 },
    { 0x8d5b5a55, 61 },
    { 0xd63b5a57, 94 },
    { 0xe96b5a55, 34 },
    { 0xeb5a59, 61 },
    { 0x673b5a59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17b5a5d, 76 },
    { 0, 0 },
    { 0xfe8b5a5f, 98 },
    { 0, 0 },
    { 0x771b5a61, 14 },
    { 0x43ab5a62, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ab5a65, 90 },
    { 0x47db5a66, 95 },
    { 0x890b5a66, 91 },
    { 0x6dcb5a68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7b5a6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ab5a70, 61 },
    { 0x18fb5a71, 79 },
    { 0, 0 },
    { 0xf92b5a73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28b5a76, 90 },
    { 0, 0 },
    { 0x17db5a78, 61 },
    { 0x56fb5a78, 90 },
    { 0xeb8b5a7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfb5a7e, 43 },
    { 0x20eb5a7f, 92 },
    { 0, 0 },
    { 0xeb9b5a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0b5a85, 61 },
    { 0x1acb5a86, 84 },
    { 0x461b5a85, 61 },
    { 0, 0 },
    { 0xa14b5a89, 99 },
    { 0xd2db5a89, 95 },
    { 0, 0 },
    { 0x42b5a8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982b5a90, 95 },
    { 0, 0 },
    { 0x2fab5a92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6b5a95, 81 },
    { 0, 0 },
    { 0xdbcb5a97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2b5a9a, 33 },
    { 0, 0 },
    { 0x5f9b5a9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b5a9f, 94 },
    { 0, 0 },
    { 0x130b5aa1, 78 },
    { 0xef4b5aa1, 23 },
    { 0, 0 },
    { 0xffcb5aa4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afb5aa7, 76 },
    { 0xe03b5aa8, 93 },
    { 0x476b5aa9, 23 },
    { 0, 0 },
    { 0xf8fb5aab, 95 },
    { 0, 0 },
    { 0x200b5aad, 78 },
    { 0x465b5aad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4b5ab4, 95 },
    { 0x94fb5ab5, 76 },
    { 0x2d9b5ab6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ab5ab9, 61 },
    { 0x5beb5ab9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0b5ac1, 90 },
    { 0xe89b5ac2, 17 },
    { 0x13fb5ac1, 61 },
    { 0x387b5ac4, 78 },
    { 0x549b5ac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29b5acd, 98 },
    { 0xe7bb5ace, 40 },
    { 0x17bb5acf, 98 },
    { 0x1deb5ace, 61 },
    { 0xc4fb5ad1, 83 },
    { 0x4fab5ad2, 23 },
    { 0x614b5ace, 95 },
    { 0, 0 },
    { 0x100b5ad5, 88 },
    { 0xd58b5ad6, 46 },
    { 0x9b3b5ad7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33b5ada, 95 },
    { 0x650b5adb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8b5adf, 85 },
    { 0, 0 },
    { 0x623b5ae1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb5ae4, 52 },
    { 0, 0 },
    { 0x69eb5ae6, 95 },
    { 0, 0 },
    { 0x647b5ae8, 99 },
    { 0, 0 },
    { 0xa4fb5aea, 90 },
    { 0xe37b5aea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87b5aee, 55 },
    { 0x516b5aef, 61 },
    { 0xfd0b5af0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36b5af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656b5afd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafb5b00, 33 },
    { 0xc09b5b01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cb5b04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fb5b12, 23 },
    { 0xce9b5b13, 55 },
    { 0xa60b5b14, 40 },
    { 0xbfeb5b15, 55 },
    { 0x1b5b16, 88 },
    { 0x69b5b16, 61 },
    { 0x462b5b18, 43 },
    { 0xc50b5b16, 55 },
    { 0xfd6b5b18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bb5b22, 77 },
    { 0x6feb5b23, 98 },
    { 0x83b5b24, 90 },
    { 0xc18b5b24, 98 },
    { 0xca4b5b26, 98 },
    { 0x429b5b27, 98 },
    { 0xb7eb5b28, 61 },
    { 0x4d0b5b27, 66 },
    { 0x9c3b5b2a, 90 },
    { 0x6d4b5b29, 100 },
    { 0x6eeb5b2b, 90 },
    { 0x866b5b2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab5b30, 65 },
    { 0, 0 },
    { 0x11db5b32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8b5b35, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5b5b3a, 61 },
    { 0x6f6b5b3b, 43 },
    { 0x96b5b3c, 61 },
    { 0xe17b5b3d, 33 },
    { 0x424b5b3e, 91 },
    { 0x62fb5b3f, 23 },
    { 0x869b5b3c, 36 },
    { 0xd3bb5b41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cb5b44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b5b47, 55 },
    { 0xe0cb5b48, 55 },
    { 0, 0 },
    { 0xe16b5b4a, 93 },
    { 0x271b5b4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b5b4f, 61 },
    { 0xee3b5b50, 33 },
    { 0, 0 },
    { 0x197b5b52, 90 },
    { 0xf0bb5b53, 34 },
    { 0x915b5b53, 52 },
    { 0, 0 },
    { 0x257b5b56, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fb5b65, 90 },
    { 0, 0 },
    { 0x352b5b67, 87 },
    { 0x3c3b5b68, 33 },
    { 0, 0 },
    { 0x1aab5b6a, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feb5b6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deb5b73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47b5b76, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8b5b7a, 61 },
    { 0, 0 },
    { 0x717b5b7c, 23 },
    { 0x33db5b7d, 33 },
    { 0x73b5b7e, 33 },
    { 0xe97b5b7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8b5b83, 43 },
    { 0xf3bb5b83, 33 },
    { 0x947b5b85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5b5b88, 90 },
    { 0x14b5b89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7feb5b8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384b5b99, 95 },
    { 0, 0 },
    { 0x349b5b9b, 43 },
    { 0x490b5b9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab5ba4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cb5ba7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b5bac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53b5baf, 61 },
    { 0xe4ab5bb0, 34 },
    { 0x836b5bb1, 76 },
    { 0x8d5b5baf, 40 },
    { 0x489b5bb3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01b5bb7, 100 },
    { 0x2e3b5bb8, 46 },
    { 0, 0 },
    { 0x5b5b5bba, 90 },
    { 0, 0 },
    { 0xad6b5bbc, 90 },
    { 0, 0 },
    { 0x681b5bbe, 78 },
    { 0x7ccb5bbf, 61 },
    { 0, 0 },
    { 0x517b5bc1, 93 },
    { 0, 0 },
    { 0x1e7b5bc3, 33 },
    { 0xeccb5bc3, 23 },
    { 0, 0 },
    { 0xc7b5bc6, 66 },
    { 0xe96b5bc7, 46 },
    { 0x383b5bc8, 34 },
    { 0xd09b5bc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x752b5bcc, 66 },
    { 0xa96b5bcd, 61 },
    { 0x112b5bce, 90 },
    { 0x95fb5bcf, 40 },
    { 0xbf0b5bcf, 90 },
    { 0x4edb5bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcb5bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07b5bda, 90 },
    { 0xb54b5bda, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb5bde, 98 },
    { 0xa5db5bde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95b5be2, 95 },
    { 0, 0 },
    { 0x3e4b5be4, 90 },
    { 0xdb6b5be5, 76 },
    { 0xc68b5be6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fb5bf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8b5bfb, 34 },
    { 0x669b5bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1b5bff, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22b5c0b, 33 },
    { 0x38bb5c0c, 33 },
    { 0x614b5c0d, 76 },
    { 0xcc7b5c0e, 95 },
    { 0, 0 },
    { 0xcbfb5c10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7b5c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb5c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2b5c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732b5c25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7b5c28, 67 },
    { 0x55fb5c29, 87 },
    { 0, 0 },
    { 0x5ebb5c2b, 98 },
    { 0x22bb5c2c, 67 },
    { 0x8d5b5c2d, 53 },
    { 0x750b5c2b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffb5c33, 43 },
    { 0x3f6b5c34, 33 },
    { 0xc2bb5c35, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecb5c39, 83 },
    { 0, 0 },
    { 0xd45b5c3b, 33 },
    { 0x9d9b5c3c, 23 },
    { 0x737b5c3c, 93 },
    { 0xfecb5c3e, 23 },
    { 0xf34b5c3e, 23 },
    { 0, 0 },
    { 0x284b5c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffb5c44, 98 },
    { 0x382b5c45, 90 },
    { 0x10b5c44, 90 },
    { 0xd63b5c47, 34 },
    { 0xda9b5c44, 33 },
    { 0xefcb5c49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7b5c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab5c52, 42 },
    { 0x2edb5c53, 66 },
    { 0x68db5c52, 98 },
    { 0xea8b5c55, 33 },
    { 0xfa9b5c56, 77 },
    { 0x44ab5c56, 61 },
    { 0x598b5c56, 90 },
    { 0x916b5c59, 95 },
    { 0x386b5c5a, 52 },
    { 0, 0 },
    { 0x819b5c5c, 52 },
    { 0x16cb5c5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b5c60, 61 },
    { 0, 0 },
    { 0x117b5c62, 52 },
    { 0x3a8b5c63, 95 },
    { 0x9ab5c64, 93 },
    { 0x138b5c64, 77 },
    { 0xe65b5c66, 66 },
    { 0xc9bb5c67, 56 },
    { 0x85b5c68, 23 },
    { 0xa5db5c69, 93 },
    { 0xdfcb5c6a, 33 },
    { 0x427b5c67, 33 },
    { 0x386b5c67, 23 },
    { 0, 0 },
    { 0x55cb5c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0b5c71, 23 },
    { 0, 0 },
    { 0x3dcb5c73, 23 },
    { 0x4cb5c73, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26b5c77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791b5c7b, 96 },
    { 0, 0 },
    { 0xaf8b5c7d, 95 },
    { 0, 0 },
    { 0x2aeb5c7f, 23 },
    { 0, 0 },
    { 0x526b5c81, 23 },
    { 0x872b5c82, 61 },
    { 0x59fb5c83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862b5c8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3b5c8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1b5c92, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aab5c95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331b5c99, 88 },
    { 0, 0 },
    { 0x189b5c9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55b5c9f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0b5ca3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cb5ca9, 61 },
    { 0x5aab5caa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795b5cb3, 34 },
    { 0, 0 },
    { 0xf42b5cb5, 98 },
    { 0xeb9b5cb5, 61 },
    { 0x249b5cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485b5cbb, 76 },
    { 0, 0 },
    { 0xbcb5cbd, 61 },
    { 0xc2fb5cbd, 23 },
    { 0, 0 },
    { 0x567b5cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cb5cc5, 66 },
    { 0xf63b5cc5, 33 },
    { 0xf7bb5cc6, 90 },
    { 0, 0 },
    { 0x8fb5cc9, 100 },
    { 0x615b5cc9, 98 },
    { 0x8f2b5cc9, 87 },
    { 0x344b5cca, 43 },
    { 0x4e5b5ccb, 90 },
    { 0xbe1b5cce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecb5cd2, 76 },
    { 0xd59b5cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdb5cd6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3b5cdc, 95 },
    { 0x554b5cdd, 33 },
    { 0, 0 },
    { 0x713b5cdf, 88 },
    { 0, 0 },
    { 0x8cbb5ce1, 97 },
    { 0, 0 },
    { 0x8f7b5ce3, 34 },
    { 0xa8eb5ce4, 68 },
    { 0xce2b5ce5, 52 },
    { 0x1e3b5ce3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x316b5ce9, 23 },
    { 0xf95b5cea, 95 },
    { 0, 0 },
    { 0x250b5cec, 43 },
    { 0x5bdb5ced, 46 },
    { 0x4deb5cee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866b5cf9, 76 },
    { 0x33fb5cfa, 90 },
    { 0x6eb5cf9, 95 },
    { 0x1ceb5cfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2b5d00, 76 },
    { 0, 0 },
    { 0xd9ab5d02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46b5d08, 33 },
    { 0, 0 },
    { 0xddcb5d0a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ab5d13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787b5d1d, 85 },
    { 0, 0 },
    { 0x32b5d1f, 95 },
    { 0, 0 },
    { 0x713b5d21, 52 },
    { 0x964b5d21, 53 },
    { 0, 0 },
    { 0x45fb5d24, 83 },
    { 0x8b3b5d24, 76 },
    { 0, 0 },
    { 0xd1db5d27, 100 },
    { 0xf56b5d28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4b5d2e, 93 },
    { 0xd9ab5d2e, 33 },
    { 0x94b5d30, 23 },
    { 0xdbfb5d31, 87 },
    { 0x5f4b5d2e, 61 },
    { 0x986b5d33, 90 },
    { 0xf58b5d2e, 61 },
    { 0x54ab5d30, 90 },
    { 0xf9b5d36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b5d3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8b5d3d, 23 },
    { 0xafcb5d3e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b5d49, 46 },
    { 0, 0 },
    { 0xc83b5d4b, 33 },
    { 0x614b5d4b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8db5d50, 33 },
    { 0x70fb5d51, 61 },
    { 0, 0 },
    { 0x687b5d53, 90 },
    { 0x13cb5d54, 77 },
    { 0xe62b5d54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15b5d58, 88 },
    { 0x642b5d59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54b5d5c, 90 },
    { 0x376b5d5d, 55 },
    { 0xdaab5d5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db5d65, 64 },
    { 0xd21b5d65, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b5d6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74b5d70, 95 },
    { 0x949b5d70, 23 },
    { 0xf23b5d72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71b5d75, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133b5d79, 61 },
    { 0, 0 },
    { 0x810b5d7b, 78 },
    { 0, 0 },
    { 0x145b5d7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92b5d85, 90 },
    { 0x9db5d85, 46 },
    { 0xd49b5d86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bb5d8e, 98 },
    { 0x9abb5d8e, 98 },
    { 0, 0 },
    { 0x279b5d91, 55 },
    { 0, 0 },
    { 0xdb4b5d93, 23 },
    { 0, 0 },
    { 0x184b5d95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18b5d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619b5da2, 17 },
    { 0xeeb5da3, 81 },
    { 0, 0 },
    { 0xe4eb5da5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eb5dab, 84 },
    { 0x15bb5dab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57b5daf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86b5db2, 61 },
    { 0xfbb5db2, 92 },
    { 0x9d5b5db4, 95 },
    { 0x63ab5db2, 40 },
    { 0x85cb5db6, 23 },
    { 0, 0 },
    { 0xf3ab5db8, 90 },
    { 0x92ab5db8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x449b5dbc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484b5dc8, 90 },
    { 0, 0 },
    { 0xe1db5dca, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5b5dcd, 33 },
    { 0x2a7b5dcd, 98 },
    { 0, 0 },
    { 0x451b5dd0, 23 },
    { 0xf06b5dd1, 61 },
    { 0, 0 },
    { 0x287b5dd3, 77 },
    { 0xb4b5dd4, 98 },
    { 0x5f0b5dd3, 75 },
    { 0xd5db5dd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x641b5dd9, 82 },
    { 0, 0 },
    { 0xc62b5ddb, 99 },
    { 0x703b5ddb, 95 },
    { 0xb75b5ddd, 23 },
    { 0xcc0b5dde, 90 },
    { 0xeffb5dde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x729b5de2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x832b5de5, 66 },
    { 0xa83b5de6, 55 },
    { 0x198b5de7, 98 },
    { 0xca4b5de7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b5deb, 61 },
    { 0x2bcb5deb, 55 },
    { 0xf77b5dec, 95 },
    { 0xc91b5dec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xefab5df1, 99 },
    { 0, 0 },
    { 0xd04b5df3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bb5e02, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bb5e05, 95 },
    { 0x10b5e06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b5e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3b5e0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b5e13, 52 },
    { 0xe07b5e14, 87 },
    { 0x294b5e14, 33 },
    { 0x3db5e13, 61 },
    { 0xdbb5e17, 90 },
    { 0x200b5e14, 90 },
    { 0x107b5e19, 69 },
    { 0xe10b5e1a, 90 },
    { 0x904b5e1b, 38 },
    { 0x208b5e19, 52 },
    { 0xd50b5e1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cb5e20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb5e23, 84 },
    { 0, 0 },
    { 0x277b5e25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474b5e29, 66 },
    { 0x176b5e2a, 98 },
    { 0xbf3b5e29, 33 },
    { 0x208b5e2b, 23 },
    { 0xbf4b5e2b, 90 },
    { 0x4cfb5e2a, 95 },
    { 0x971b5e2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1b5e33, 97 },
    { 0x290b5e34, 68 },
    { 0xf72b5e34, 34 },
    { 0, 0 },
    { 0x7c5b5e37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eb5e3b, 33 },
    { 0xd78b5e3b, 98 },
    { 0x4d8b5e3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b5e42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1b5e46, 91 },
    { 0xbddb5e47, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764b5e4d, 90 },
    { 0xf94b5e4d, 76 },
    { 0, 0 },
    { 0x86bb5e50, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893b5e59, 77 },
    { 0xfd6b5e5a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484b5e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eeb5e6b, 95 },
    { 0x56b5e6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eb5e72, 33 },
    { 0x686b5e72, 55 },
    { 0xa16b5e74, 76 },
    { 0x9f6b5e75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6b5e78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8b5e84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19b5e88, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b5e94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45b5e99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb5e9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x733b5e9f, 90 },
    { 0x11fb5ea0, 55 },
    { 0xe2db5ea1, 22 },
    { 0x39ab5ea2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecb5ea5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b5ea9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecb5ead, 23 },
    { 0, 0 },
    { 0x2f2b5eaf, 33 },
    { 0, 0 },
    { 0x2abb5eb1, 96 },
    { 0, 0 },
    { 0x692b5eb3, 95 },
    { 0, 0 },
    { 0x933b5eb5, 36 },
    { 0x18ab5eb6, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbb5eb9, 90 },
    { 0x6f8b5eb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x392b5ebd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61b5ec9, 80 },
    { 0xec6b5eca, 46 },
    { 0x7abb5ecb, 88 },
    { 0x851b5ecb, 66 },
    { 0xa2bb5ecd, 23 },
    { 0xbbbb5eca, 99 },
    { 0, 0 },
    { 0x987b5ed0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222b5ed3, 98 },
    { 0, 0 },
    { 0x5a6b5ed5, 81 },
    { 0, 0 },
    { 0x95b5ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcb5eda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffb5ede, 76 },
    { 0xac3b5edf, 90 },
    { 0x6cfb5ee0, 61 },
    { 0, 0 },
    { 0xeefb5ee2, 98 },
    { 0, 0 },
    { 0x140b5ee4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4db5ee9, 76 },
    { 0x831b5eea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635b5ef1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ab5ef4, 98 },
    { 0xc1fb5ef4, 55 },
    { 0, 0 },
    { 0x69fb5ef7, 90 },
    { 0xb1ab5ef7, 43 },
    { 0xaa5b5ef9, 90 },
    { 0xbd6b5ef8, 95 },
    { 0, 0 },
    { 0x9cdb5efc, 67 },
    { 0xc1fb5efd, 43 },
    { 0, 0 },
    { 0xdc7b5eff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410b5f04, 52 },
    { 0x340b5f05, 52 },
    { 0, 0 },
    { 0x510b5f07, 85 },
    { 0xb65b5f08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafeb5f0b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x395b5f0e, 33 },
    { 0, 0 },
    { 0x589b5f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb5f16, 17 },
    { 0, 0 },
    { 0x400b5f18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5b5f1c, 23 },
    { 0x5e9b5f1d, 90 },
    { 0xbcab5f1c, 90 },
    { 0xc7eb5f1f, 33 },
    { 0, 0 },
    { 0xe3b5f21, 95 },
    { 0x7bdb5f22, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6b5f29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb5f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642b5f31, 67 },
    { 0, 0 },
    { 0x48bb5f33, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83b5f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x128b5f3c, 87 },
    { 0xc1fb5f3c, 98 },
    { 0, 0 },
    { 0x52bb5f3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bb5f42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb5f45, 90 },
    { 0xe59b5f46, 90 },
    { 0xa7cb5f47, 33 },
    { 0, 0 },
    { 0xc26b5f49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b5f51, 88 },
    { 0xce9b5f51, 76 },
    { 0, 0 },
    { 0xa2fb5f54, 23 },
    { 0, 0 },
    { 0xa5eb5f56, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55b5f59, 61 },
    { 0x78db5f5a, 23 },
    { 0x1c1b5f5b, 78 },
    { 0x319b5f5b, 61 },
    { 0x6d5b5f5a, 90 },
    { 0x5e8b5f5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ab5f61, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392b5f6e, 33 },
    { 0xc2bb5f6f, 78 },
    { 0xc3fb5f6f, 61 },
    { 0, 0 },
    { 0x59bb5f72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4b5f75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08b5f78, 33 },
    { 0xcbcb5f79, 100 },
    { 0xd79b5f7a, 76 },
    { 0, 0 },
    { 0x71db5f7c, 95 },
    { 0x914b5f7c, 90 },
    { 0x650b5f7c, 90 },
    { 0x239b5f7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb5f88, 61 },
    { 0xe23b5f88, 95 },
    { 0x63eb5f8a, 55 },
    { 0x652b5f8b, 90 },
    { 0x689b5f8b, 95 },
    { 0x398b5f8c, 98 },
    { 0x479b5f8c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92b5f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x688b5f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b5f99, 61 },
    { 0x51ab5f9a, 91 },
    { 0xbc6b5f9b, 61 },
    { 0x498b5f9c, 61 },
    { 0x5aab5f9c, 61 },
    { 0xf3db5f99, 33 },
    { 0x34fb5f9f, 87 },
    { 0x72eb5fa0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cb5fa7, 77 },
    { 0, 0 },
    { 0x298b5fa9, 33 },
    { 0xf2eb5fa9, 33 },
    { 0x9e3b5faa, 99 },
    { 0, 0 },
    { 0x840b5fad, 61 },
    { 0xe3cb5fae, 98 },
    { 0x5d1b5faf, 69 },
    { 0xe91b5faf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eb5fb4, 87 },
    { 0xe06b5fb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12b5fbb, 67 },
    { 0, 0 },
    { 0x714b5fbd, 98 },
    { 0x5e2b5fbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825b5fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b5fcc, 55 },
    { 0x4d6b5fcc, 23 },
    { 0, 0 },
    { 0xeb0b5fcf, 61 },
    { 0xb4b5fcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784b5fd5, 46 },
    { 0, 0 },
    { 0x4dfb5fd7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482b5fdc, 33 },
    { 0x1cfb5fdc, 33 },
    { 0x3cab5fde, 76 },
    { 0x44db5fde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74b5fe2, 66 },
    { 0x55b5fe3, 98 },
    { 0x21eb5fe2, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fb5fea, 34 },
    { 0x2dbb5feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b5ff0, 81 },
    { 0x2f2b5ff1, 34 },
    { 0x2f3b5ff1, 33 },
    { 0, 0 },
    { 0x8ddb5ff4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3b5ff9, 36 },
    { 0, 0 },
    { 0xaf6b5ffb, 33 },
    { 0, 0 },
    { 0x1ab5ffd, 55 },
    { 0, 0 },
    { 0xd68b5fff, 90 },
    { 0x20bb6000, 43 },
    { 0x65db6001, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6db6004, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6b600b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab6018, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6b601c, 61 },
    { 0, 0 },
    { 0x125b601e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49eb6023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ab6028, 95 },
    { 0, 0 },
    { 0x70bb602a, 61 },
    { 0xf5b602b, 98 },
    { 0x7d3b602b, 61 },
    { 0xecdb602a, 34 },
    { 0xd3fb602e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b6031, 93 },
    { 0, 0 },
    { 0x3c2b6033, 33 },
    { 0, 0 },
    { 0x8e5b6035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05b6038, 90 },
    { 0, 0 },
    { 0x958b603a, 98 },
    { 0x718b603b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab603e, 76 },
    { 0x564b603e, 23 },
    { 0, 0 },
    { 0xa3b6041, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136b6046, 33 },
    { 0x56db6046, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51b604d, 94 },
    { 0x36ab604e, 78 },
    { 0x822b604e, 33 },
    { 0xbd3b604f, 77 },
    { 0x4dbb6051, 33 },
    { 0, 0 },
    { 0x5d8b6053, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b6057, 98 },
    { 0x5a3b6058, 23 },
    { 0, 0 },
    { 0x104b605a, 61 },
    { 0xbe6b605b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b605e, 43 },
    { 0x99cb605f, 33 },
    { 0xf5db6060, 23 },
    { 0x8b4b605f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ab6064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb606b, 76 },
    { 0xe83b606c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95b6073, 76 },
    { 0x8e0b6074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334b607b, 78 },
    { 0xaa6b607b, 98 },
    { 0xe8b607c, 90 },
    { 0xdf0b607c, 61 },
    { 0x2dbb607f, 46 },
    { 0x7b607d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b6086, 98 },
    { 0xb2db6087, 93 },
    { 0x53eb6088, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140b608f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cb6096, 98 },
    { 0x959b6097, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675b609b, 85 },
    { 0xa61b609b, 81 },
    { 0xe14b609d, 91 },
    { 0xc7ab609e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceab60a2, 95 },
    { 0, 0 },
    { 0x25cb60a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cb60ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66b60ae, 33 },
    { 0, 0 },
    { 0x501b60b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517b60b4, 33 },
    { 0, 0 },
    { 0x7deb60b6, 90 },
    { 0x5c8b60b7, 33 },
    { 0xc34b60b7, 90 },
    { 0, 0 },
    { 0x23b60ba, 91 },
    { 0x699b60bb, 34 },
    { 0x72fb60bb, 55 },
    { 0, 0 },
    { 0x5e2b60be, 23 },
    { 0xf84b60bf, 98 },
    { 0x82cb60c0, 33 },
    { 0x557b60c1, 90 },
    { 0xf07b60c2, 61 },
    { 0x333b60c0, 100 },
    { 0x3c3b60bf, 87 },
    { 0, 0 },
    { 0xac1b60c6, 90 },
    { 0x2f8b60c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcb60ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21b60d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfb60d6, 95 },
    { 0, 0 },
    { 0x7deb60d8, 78 },
    { 0x27db60d8, 84 },
    { 0x56bb60da, 67 },
    { 0xc36b60db, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07b60df, 55 },
    { 0, 0 },
    { 0xd25b60e1, 68 },
    { 0x816b60e1, 61 },
    { 0x506b60e1, 98 },
    { 0x74cb60e4, 90 },
    { 0x630b60e5, 61 },
    { 0, 0 },
    { 0x35ab60e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fb60f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9b60f8, 23 },
    { 0xa3cb60f9, 98 },
    { 0x1fab60fa, 94 },
    { 0xef0b60fb, 76 },
    { 0xc39b60fb, 90 },
    { 0, 0 },
    { 0xb80b60fe, 90 },
    { 0x4b1b60ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913b6104, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b6108, 43 },
    { 0xf5fb6108, 77 },
    { 0, 0 },
    { 0x453b610b, 90 },
    { 0, 0 },
    { 0x704b610d, 40 },
    { 0x922b610d, 61 },
    { 0x82b610e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ab6112, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86b6118, 33 },
    { 0xe7cb6119, 98 },
    { 0, 0 },
    { 0xfecb611b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6b6120, 66 },
    { 0xb2cb6121, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edb6124, 90 },
    { 0, 0 },
    { 0x859b6126, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb612a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82db6132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdb6136, 95 },
    { 0x262b6137, 98 },
    { 0, 0 },
    { 0xb98b6139, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cb613c, 33 },
    { 0x318b613d, 33 },
    { 0xa66b613e, 95 },
    { 0xb09b613e, 87 },
    { 0x242b6140, 23 },
    { 0xbfab6141, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64b6144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27b6147, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbb6150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17b6157, 61 },
    { 0, 0 },
    { 0xd9db6159, 98 },
    { 0, 0 },
    { 0x925b615b, 90 },
    { 0xcdeb615b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b615f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bb6162, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60b6165, 98 },
    { 0x621b6165, 93 },
    { 0xbe4b6165, 77 },
    { 0x83cb6165, 61 },
    { 0, 0 },
    { 0x653b616a, 93 },
    { 0x5d6b616b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3b6172, 78 },
    { 0xc05b6173, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b6177, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdb617b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecb617f, 34 },
    { 0x700b6180, 99 },
    { 0x264b6181, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31b6186, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1b6189, 87 },
    { 0, 0 },
    { 0xb40b618b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eb618e, 38 },
    { 0x7b5b618e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724b619c, 43 },
    { 0, 0 },
    { 0x255b619e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5db61a1, 95 },
    { 0, 0 },
    { 0xf57b61a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240b61a7, 95 },
    { 0, 0 },
    { 0x40bb61a9, 90 },
    { 0x23b61aa, 33 },
    { 0xed2b61ab, 34 },
    { 0, 0 },
    { 0x46fb61ad, 55 },
    { 0, 0 },
    { 0xf92b61af, 61 },
    { 0xe02b61af, 76 },
    { 0xdc0b61b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x462b61b4, 34 },
    { 0xd22b61b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x398b61b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fb61bb, 90 },
    { 0xba1b61bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6b61c0, 23 },
    { 0x6f6b61c1, 95 },
    { 0x56bb61c2, 23 },
    { 0x972b61c0, 90 },
    { 0x7b61c1, 78 },
    { 0x380b61c3, 90 },
    { 0xfc9b61c4, 46 },
    { 0xfdb61c3, 88 },
    { 0x234b61c7, 67 },
    { 0x9abb61c9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5b61cc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb61d3, 95 },
    { 0x6aeb61d4, 64 },
    { 0xbb5b61d5, 23 },
    { 0xa2ab61d4, 90 },
    { 0x34db61d3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cb61db, 33 },
    { 0, 0 },
    { 0xe92b61dd, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdab61e3, 55 },
    { 0, 0 },
    { 0x41eb61e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bb61e8, 99 },
    { 0xd46b61e9, 90 },
    { 0x2abb61ea, 95 },
    { 0xecb61eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cb61f6, 99 },
    { 0xfd7b61f7, 85 },
    { 0x581b61f7, 66 },
    { 0x689b61f9, 17 },
    { 0xb6b61fa, 98 },
    { 0, 0 },
    { 0x208b61fc, 87 },
    { 0, 0 },
    { 0x626b61fe, 61 },
    { 0, 0 },
    { 0x597b6200, 92 },
    { 0, 0 },
    { 0x89b6202, 33 },
    { 0x99eb6202, 95 },
    { 0xf36b6204, 76 },
    { 0, 0 },
    { 0x7e0b6206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539b620c, 93 },
    { 0xe51b620d, 93 },
    { 0xb3fb620d, 90 },
    { 0x453b620e, 75 },
    { 0xe74b6210, 33 },
    { 0, 0 },
    { 0xe1bb6212, 90 },
    { 0xd8eb6213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473b6217, 52 },
    { 0x336b6218, 55 },
    { 0, 0 },
    { 0x649b621a, 91 },
    { 0x88fb621b, 90 },
    { 0xab4b621c, 23 },
    { 0x444b621a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873b6223, 95 },
    { 0, 0 },
    { 0xb0b6225, 34 },
    { 0, 0 },
    { 0xf04b6227, 98 },
    { 0xca2b6228, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5b622d, 61 },
    { 0, 0 },
    { 0x6d7b622f, 33 },
    { 0xb04b6230, 33 },
    { 0xa96b6231, 90 },
    { 0xdbbb622f, 90 },
    { 0x21fb6233, 90 },
    { 0, 0 },
    { 0x295b6235, 98 },
    { 0, 0 },
    { 0x3b5b6237, 24 },
    { 0xc63b6238, 61 },
    { 0x5eeb6239, 85 },
    { 0x157b623a, 55 },
    { 0x683b6239, 66 },
    { 0x8b4b6239, 68 },
    { 0, 0 },
    { 0x6b9b623e, 61 },
    { 0xe77b623f, 33 },
    { 0xdc6b623e, 97 },
    { 0x8cbb6241, 90 },
    { 0x33bb6241, 61 },
    { 0, 0 },
    { 0x5e8b6244, 76 },
    { 0x53cb6245, 93 },
    { 0xae7b6246, 90 },
    { 0x170b6246, 46 },
    { 0, 0 },
    { 0xe9fb6249, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1b624c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab6253, 98 },
    { 0xdb5b6254, 99 },
    { 0x406b6253, 61 },
    { 0x4a5b6256, 90 },
    { 0x904b6257, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x914b625a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x500b625d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdb6260, 90 },
    { 0x9f1b6261, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5b6264, 33 },
    { 0x46cb6265, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1b626f, 56 },
    { 0x139b6270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b6275, 90 },
    { 0x497b6276, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1b627a, 76 },
    { 0x481b627b, 98 },
    { 0x5c6b627c, 90 },
    { 0xc57b627c, 93 },
    { 0xf47b627c, 23 },
    { 0x180b627f, 61 },
    { 0x362b6280, 42 },
    { 0, 0 },
    { 0x739b6282, 95 },
    { 0x381b6283, 46 },
    { 0x290b6284, 98 },
    { 0x847b6283, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53b6288, 34 },
    { 0x712b6289, 61 },
    { 0xdfcb6289, 43 },
    { 0x3f8b6288, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9b628f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb6296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fab6299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45b629f, 23 },
    { 0x657b62a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b62a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fb62ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afb62b9, 33 },
    { 0xf31b62ba, 61 },
    { 0, 0 },
    { 0xbd9b62bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eb62bf, 65 },
    { 0xb3eb62c0, 99 },
    { 0x6bb62c0, 87 },
    { 0x853b62c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502b62c6, 78 },
    { 0x9a2b62c7, 61 },
    { 0x669b62c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b62cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0b62cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cb62d2, 33 },
    { 0x986b62d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2b62d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eb62d9, 99 },
    { 0, 0 },
    { 0x7aeb62db, 78 },
    { 0x479b62dc, 90 },
    { 0, 0 },
    { 0xa53b62de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7b62ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb62ef, 57 },
    { 0x8a6b62ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632b62f3, 90 },
    { 0xcbb62f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7b62f8, 90 },
    { 0, 0 },
    { 0xdb6b62fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268b62fe, 23 },
    { 0xe56b62ff, 95 },
    { 0xf87b62fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfb6306, 68 },
    { 0x108b6306, 33 },
    { 0x854b6308, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2b630f, 55 },
    { 0, 0 },
    { 0xcd6b6311, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fb6315, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bb6319, 93 },
    { 0x30eb631a, 23 },
    { 0xeab6319, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffb6323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x834b6326, 95 },
    { 0x5ab6327, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7b632a, 33 },
    { 0x6db632b, 90 },
    { 0xbdb632c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbb6332, 23 },
    { 0x65bb6332, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596b6337, 95 },
    { 0xd58b6338, 23 },
    { 0x62db6339, 33 },
    { 0xf95b633a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadb6340, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb6343, 65 },
    { 0x81b6343, 46 },
    { 0x854b6343, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0b6349, 61 },
    { 0xc27b634a, 90 },
    { 0xa5cb634b, 76 },
    { 0x865b634c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fb6351, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeb6355, 90 },
    { 0xa7ab6356, 95 },
    { 0xa42b6356, 33 },
    { 0x2bfb6357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cb635c, 76 },
    { 0x2db635c, 95 },
    { 0xcceb635e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2b6362, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beb6369, 55 },
    { 0xf98b636a, 55 },
    { 0, 0 },
    { 0x8e9b636c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75b6373, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abb637a, 88 },
    { 0, 0 },
    { 0xd09b637c, 76 },
    { 0xe3eb637d, 61 },
    { 0, 0 },
    { 0x58eb637f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x23db6382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3b6386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23b638d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24b6391, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49b6399, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cb639c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73db63a1, 76 },
    { 0xb8cb63a2, 33 },
    { 0x1d0b63a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x857b63a6, 76 },
    { 0, 0 },
    { 0x78eb63a8, 61 },
    { 0, 0 },
    { 0x900b63aa, 33 },
    { 0, 0 },
    { 0xaaab63ac, 23 },
    { 0x379b63ac, 33 },
    { 0x75bb63ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160b63ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x258b63bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deb63c0, 90 },
    { 0, 0 },
    { 0x587b63c2, 98 },
    { 0xcd6b63c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cb63c8, 23 },
    { 0xaf0b63c9, 94 },
    { 0x4ab63c8, 91 },
    { 0xcf5b63c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6b63ce, 100 },
    { 0x3ab63ce, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678b63d9, 34 },
    { 0xb46b63d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecb63dd, 33 },
    { 0, 0 },
    { 0x2deb63df, 61 },
    { 0xd27b63df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977b63eb, 61 },
    { 0x202b63ec, 90 },
    { 0x51b63ed, 61 },
    { 0x73cb63ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eb63f7, 90 },
    { 0xcf5b63f8, 90 },
    { 0x7b8b63f9, 90 },
    { 0, 0 },
    { 0xd54b63fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcb6407, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428b640f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487b6414, 95 },
    { 0x61bb6415, 90 },
    { 0xcfdb6416, 90 },
    { 0x26fb6414, 90 },
    { 0x995b6417, 46 },
    { 0xde3b6419, 95 },
    { 0xbc8b641a, 65 },
    { 0, 0 },
    { 0x76db641c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cb641f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cb6422, 22 },
    { 0x9ab6423, 61 },
    { 0x862b6424, 76 },
    { 0x28cb6425, 23 },
    { 0x93b6426, 95 },
    { 0, 0 },
    { 0xb22b6428, 87 },
    { 0x80db6429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813b6430, 95 },
    { 0x21db6430, 24 },
    { 0x13eb6432, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eb6436, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x852b6439, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab643e, 90 },
    { 0xe6fb643e, 95 },
    { 0x88cb6440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcb6444, 93 },
    { 0x172b6445, 61 },
    { 0x574b6446, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2b6449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8b644d, 98 },
    { 0xfcbb644e, 90 },
    { 0x389b644f, 23 },
    { 0x85db644f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718b6455, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fb645a, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44b645f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1b6463, 33 },
    { 0x80eb6463, 61 },
    { 0x773b6464, 98 },
    { 0x10eb6466, 90 },
    { 0, 0 },
    { 0x41bb6468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbb646b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b646e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fb6473, 87 },
    { 0, 0 },
    { 0xeb4b6475, 23 },
    { 0x34fb6476, 55 },
    { 0xfbbb6476, 33 },
    { 0xd9ab6477, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7b647e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffb6483, 23 },
    { 0, 0 },
    { 0x778b6485, 34 },
    { 0, 0 },
    { 0x357b6487, 95 },
    { 0x58bb6488, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2b648b, 23 },
    { 0, 0 },
    { 0x2f6b648d, 95 },
    { 0x8f6b648e, 34 },
    { 0x4f1b648f, 66 },
    { 0x93b6490, 96 },
    { 0x94cb6491, 33 },
    { 0x1c2b6492, 46 },
    { 0xf20b6491, 91 },
    { 0x68eb6491, 34 },
    { 0x50ab6492, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb649a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ab649e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5b64a1, 23 },
    { 0, 0 },
    { 0x436b64a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63b64a8, 90 },
    { 0xe64b64a9, 86 },
    { 0x43b64aa, 90 },
    { 0x107b64ab, 78 },
    { 0x27fb64ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2b64af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3b64b6, 49 },
    { 0x8dfb64b6, 76 },
    { 0x1cbb64b8, 90 },
    { 0, 0 },
    { 0x2a3b64ba, 90 },
    { 0x9dcb64bb, 56 },
    { 0x105b64bb, 78 },
    { 0x11db64bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb64c0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45b64c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63b64ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ab64cd, 90 },
    { 0, 0 },
    { 0xc1fb64cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eb64d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4b64db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38b64e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdb64e6, 81 },
    { 0, 0 },
    { 0xef3b64e8, 90 },
    { 0, 0 },
    { 0x83eb64ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edb64ed, 95 },
    { 0x9d7b64ee, 98 },
    { 0xde3b64ed, 61 },
    { 0, 0 },
    { 0x5bab64f1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbb64f4, 52 },
    { 0xf9fb64f5, 88 },
    { 0xd91b64f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51b64fb, 95 },
    { 0xba1b64fc, 90 },
    { 0xff5b64fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbb6503, 76 },
    { 0x25ab6504, 34 },
    { 0xfe3b6504, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb6508, 68 },
    { 0, 0 },
    { 0x88b650a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefb6513, 90 },
    { 0, 0 },
    { 0xa87b6515, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8b651d, 98 },
    { 0xa18b651e, 61 },
    { 0x4cbb651f, 99 },
    { 0x828b651d, 61 },
    { 0xcb0b6521, 43 },
    { 0x1a7b6522, 90 },
    { 0, 0 },
    { 0x485b6524, 23 },
    { 0x2e0b6525, 78 },
    { 0, 0 },
    { 0x73fb6527, 99 },
    { 0x3a9b6528, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504b652d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b6530, 33 },
    { 0, 0 },
    { 0x4dab6532, 90 },
    { 0xddb6532, 84 },
    { 0x43db6532, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ab6538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbb653b, 90 },
    { 0xa9bb653c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765b6540, 78 },
    { 0xdb8b6541, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb6544, 67 },
    { 0xeedb6545, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfb654b, 33 },
    { 0x558b654c, 76 },
    { 0x15db654d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1b6552, 68 },
    { 0x959b6552, 95 },
    { 0xca4b6554, 33 },
    { 0x7fcb6555, 90 },
    { 0, 0 },
    { 0x2f8b6557, 90 },
    { 0xd6b6558, 100 },
    { 0x47eb6557, 76 },
    { 0xd8b655a, 76 },
    { 0x44eb655b, 84 },
    { 0, 0 },
    { 0x5bcb655d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3b6560, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeb6563, 33 },
    { 0x47bb6564, 68 },
    { 0, 0 },
    { 0xafdb6566, 68 },
    { 0x697b6567, 66 },
    { 0x5e7b6568, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eb656b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb6575, 33 },
    { 0, 0 },
    { 0xb75b6577, 91 },
    { 0x7e1b6578, 23 },
    { 0, 0 },
    { 0xa5bb657a, 91 },
    { 0, 0 },
    { 0x650b657c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x896b657f, 90 },
    { 0xe63b657f, 33 },
    { 0x91db6581, 61 },
    { 0x990b6581, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36b6585, 25 },
    { 0, 0 },
    { 0xf5ab6587, 93 },
    { 0x13ab6587, 90 },
    { 0xdd4b6588, 76 },
    { 0, 0 },
    { 0x1cfb658b, 40 },
    { 0x4ceb658c, 90 },
    { 0, 0 },
    { 0xe07b658e, 95 },
    { 0x829b658f, 90 },
    { 0x78fb6590, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4db6594, 55 },
    { 0x308b6595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4db65a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31b65a9, 23 },
    { 0x6d5b65aa, 23 },
    { 0xae2b65aa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adb65ae, 98 },
    { 0x6e0b65af, 46 },
    { 0x3b4b65b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987b65b4, 100 },
    { 0, 0 },
    { 0x1f9b65b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x393b65b9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5b65bd, 46 },
    { 0x978b65be, 61 },
    { 0x403b65bf, 34 },
    { 0x83eb65c0, 90 },
    { 0xd16b65c1, 23 },
    { 0xdab65c0, 55 },
    { 0, 0 },
    { 0xdddb65c4, 99 },
    { 0x7ffb65c4, 98 },
    { 0x1c8b65c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb65cf, 90 },
    { 0xcaeb65cf, 40 },
    { 0x92cb65d1, 90 },
    { 0x13cb65d2, 90 },
    { 0x90db65d2, 76 },
    { 0x43eb65cf, 43 },
    { 0x9b3b65d2, 56 },
    { 0x4eb65d6, 90 },
    { 0, 0 },
    { 0xf9ab65d8, 33 },
    { 0xdb7b65d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b65dd, 90 },
    { 0, 0 },
    { 0x497b65df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab65e2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x842b65e5, 93 },
    { 0xedfb65e6, 33 },
    { 0x5f5b65e5, 95 },
    { 0, 0 },
    { 0xee6b65e9, 55 },
    { 0x148b65ea, 98 },
    { 0xf85b65eb, 61 },
    { 0x474b65ec, 85 },
    { 0x56bb65ed, 65 },
    { 0x4d2b65ea, 65 },
    { 0x7c3b65ef, 90 },
    { 0xc07b65f0, 95 },
    { 0x5db65f1, 90 },
    { 0x350b65f2, 34 },
    { 0x186b65f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7b65f7, 55 },
    { 0, 0 },
    { 0x4c4b65f9, 55 },
    { 0x813b65fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66b65ff, 93 },
    { 0x6e7b65ff, 75 },
    { 0x23cb6601, 33 },
    { 0x62ab6602, 61 },
    { 0xd42b6603, 40 },
    { 0xaeab6603, 90 },
    { 0x506b6605, 66 },
    { 0xabb6606, 61 },
    { 0xb58b6607, 95 },
    { 0x138b6606, 33 },
    { 0x65fb6609, 46 },
    { 0xed8b6604, 37 },
    { 0xb66b660b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9b6611, 90 },
    { 0, 0 },
    { 0xc1fb6613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eb6617, 95 },
    { 0x6a9b6618, 81 },
    { 0x286b6619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb6620, 95 },
    { 0xe22b6621, 61 },
    { 0x371b6622, 90 },
    { 0x1c7b6623, 90 },
    { 0x11ab6624, 4 },
    { 0x539b6622, 33 },
    { 0x445b6623, 67 },
    { 0xa1eb6622, 34 },
    { 0x8b8b6626, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35b662e, 91 },
    { 0, 0 },
    { 0xeaab6630, 31 },
    { 0xf1ab6631, 55 },
    { 0x705b6632, 33 },
    { 0x5f2b6631, 33 },
    { 0x8a0b6634, 55 },
    { 0xf67b6635, 98 },
    { 0xa8fb6636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcb663c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6b663f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ab6644, 67 },
    { 0, 0 },
    { 0xb66b6646, 33 },
    { 0xd49b6647, 61 },
    { 0, 0 },
    { 0xd2fb6649, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5b664c, 95 },
    { 0x3a7b664d, 61 },
    { 0, 0 },
    { 0x1e1b664f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b6652, 90 },
    { 0xc9db6653, 34 },
    { 0x4ffb6654, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02b6657, 78 },
    { 0x3cab6657, 69 },
    { 0x442b6659, 33 },
    { 0xcf5b665a, 90 },
    { 0xdc5b6659, 34 },
    { 0x61db665c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcab6663, 43 },
    { 0xdc9b6664, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x590b6667, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4b666a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eb6674, 61 },
    { 0xdb7b6674, 67 },
    { 0, 0 },
    { 0x955b6677, 85 },
    { 0xf4fb6678, 95 },
    { 0, 0 },
    { 0x62eb667a, 55 },
    { 0xb06b667b, 43 },
    { 0xb11b667b, 33 },
    { 0x3c2b667c, 23 },
    { 0xfeb667d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168b6693, 43 },
    { 0x9dab6694, 75 },
    { 0x8c9b6693, 33 },
    { 0xc90b6696, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5b669c, 61 },
    { 0, 0 },
    { 0xc52b669e, 52 },
    { 0, 0 },
    { 0x5abb66a0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5b66a5, 66 },
    { 0x66fb66a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30eb66b1, 93 },
    { 0x129b66b2, 61 },
    { 0xf80b66b3, 91 },
    { 0x4f0b66b1, 55 },
    { 0xff9b66b5, 100 },
    { 0xc6bb66b6, 81 },
    { 0x2c3b66b1, 99 },
    { 0, 0 },
    { 0x31ab66b9, 95 },
    { 0, 0 },
    { 0x79bb66bb, 76 },
    { 0x8a8b66bc, 55 },
    { 0xa6eb66bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357b66c1, 95 },
    { 0, 0 },
    { 0xe1fb66c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0b66cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcb66cf, 98 },
    { 0xd7eb66d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516b66d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49b66dc, 23 },
    { 0x7ddb66dd, 93 },
    { 0xf96b66dd, 23 },
    { 0, 0 },
    { 0x647b66e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81b66e4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377b66e8, 55 },
    { 0, 0 },
    { 0x8bdb66ea, 93 },
    { 0xac1b66eb, 68 },
    { 0, 0 },
    { 0xc4cb66ed, 78 },
    { 0x5b8b66ee, 34 },
    { 0, 0 },
    { 0xdc8b66f0, 76 },
    { 0x5b9b66f0, 95 },
    { 0x2fb66f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fb66f6, 76 },
    { 0, 0 },
    { 0x1fbb66f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3b66fe, 23 },
    { 0x724b66ff, 61 },
    { 0x3c9b6700, 90 },
    { 0xa94b66ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b6704, 88 },
    { 0x4c4b6705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700b670a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcab670d, 78 },
    { 0x175b670e, 67 },
    { 0xddfb670f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deb6716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x618b6719, 90 },
    { 0, 0 },
    { 0x670b671b, 99 },
    { 0xfc6b671c, 90 },
    { 0x69b671c, 71 },
    { 0x96ab671e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27b6722, 61 },
    { 0x2b2b6723, 34 },
    { 0, 0 },
    { 0xe0eb6725, 87 },
    { 0xb51b6725, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601b6732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13b673b, 84 },
    { 0x492b673b, 99 },
    { 0x5bfb673c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdb6742, 88 },
    { 0xcecb6743, 23 },
    { 0x2b6b6742, 98 },
    { 0xc38b6743, 23 },
    { 0xe1eb6746, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeeb674e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x275b6751, 80 },
    { 0x364b6751, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x201b6755, 46 },
    { 0xa3cb6755, 75 },
    { 0, 0 },
    { 0xcdbb6758, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127b675c, 34 },
    { 0, 0 },
    { 0x90b675e, 90 },
    { 0xa5bb675e, 61 },
    { 0xc57b6760, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ab6766, 98 },
    { 0, 0 },
    { 0x698b6768, 61 },
    { 0, 0 },
    { 0xcfab676a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdb676e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8b6775, 33 },
    { 0x86b6775, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07b677f, 95 },
    { 0, 0 },
    { 0x3d6b6781, 90 },
    { 0xd25b6782, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcb678a, 33 },
    { 0x6ecb678b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cb678f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306b6796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02b679c, 23 },
    { 0x530b679d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7b67a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2b67a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8b67a8, 55 },
    { 0x1a7b67a9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab67ad, 90 },
    { 0x6eb67ae, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97b67b9, 93 },
    { 0xcb2b67ba, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9b67c0, 95 },
    { 0, 0 },
    { 0xa88b67c2, 96 },
    { 0, 0 },
    { 0x56b67c4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x145b67c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53b67cb, 95 },
    { 0xd89b67cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960b67d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9eb67d6, 85 },
    { 0x6a3b67d7, 87 },
    { 0xebeb67d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b67df, 46 },
    { 0xcdbb67e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457b67e4, 85 },
    { 0, 0 },
    { 0x76ab67e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380b67eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07b67ee, 61 },
    { 0x9b0b67ef, 90 },
    { 0xe76b67f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cb67f3, 23 },
    { 0x78fb67f4, 25 },
    { 0x7b7b67f5, 33 },
    { 0x6eb67f6, 90 },
    { 0xaecb67f7, 95 },
    { 0xa44b67f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1b67fc, 90 },
    { 0xbacb67fd, 90 },
    { 0xbecb67fe, 33 },
    { 0x9aab67fe, 90 },
    { 0xa85b6800, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51b6805, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8b680a, 100 },
    { 0, 0 },
    { 0x29b680c, 33 },
    { 0x22bb680d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb6818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abb681b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fb6821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5b6826, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x921b6829, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4b682c, 67 },
    { 0x5e6b682d, 98 },
    { 0x69cb682e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2b6832, 95 },
    { 0x489b6832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb6836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747b683a, 33 },
    { 0, 0 },
    { 0xb68b683c, 99 },
    { 0, 0 },
    { 0x918b683e, 95 },
    { 0, 0 },
    { 0x1b8b6840, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0b6846, 77 },
    { 0, 0 },
    { 0x5dcb6848, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccb684c, 61 },
    { 0xb67b684c, 90 },
    { 0xe55b684e, 95 },
    { 0x824b684c, 34 },
    { 0x4a7b684d, 61 },
    { 0xf40b6851, 98 },
    { 0, 0 },
    { 0xa7ab6853, 82 },
    { 0, 0 },
    { 0x723b6855, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b6859, 23 },
    { 0x344b6859, 23 },
    { 0x621b6859, 33 },
    { 0xc0fb685c, 98 },
    { 0x6bab685d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fb6860, 34 },
    { 0, 0 },
    { 0x7f9b6862, 90 },
    { 0x5f9b6862, 33 },
    { 0x795b6864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74db686e, 66 },
    { 0x100b686e, 61 },
    { 0, 0 },
    { 0x7a1b6871, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7b6874, 67 },
    { 0x1ccb6875, 98 },
    { 0x6ab6874, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdb6879, 46 },
    { 0x3bbb687a, 61 },
    { 0x5dfb687b, 98 },
    { 0x646b687c, 33 },
    { 0xb96b687a, 98 },
    { 0x8e1b687b, 43 },
    { 0x2e2b687e, 33 },
    { 0, 0 },
    { 0x41db6881, 98 },
    { 0, 0 },
    { 0xc97b6883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7b6887, 17 },
    { 0x38eb6888, 98 },
    { 0x6fb6887, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6db688f, 85 },
    { 0xba5b6890, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fb6894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2b6899, 33 },
    { 0x2c5b689a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x698b689d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5b68a1, 93 },
    { 0x59cb68a2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3b68a9, 76 },
    { 0x661b68aa, 66 },
    { 0, 0 },
    { 0xe4ab68ac, 61 },
    { 0, 0 },
    { 0x15bb68ae, 90 },
    { 0x684b68af, 95 },
    { 0x866b68b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ab68b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904b68b7, 61 },
    { 0, 0 },
    { 0xe0ab68b9, 43 },
    { 0, 0 },
    { 0x8f5b68bb, 90 },
    { 0x2dfb68bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db68c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddb68c5, 87 },
    { 0x28eb68c5, 33 },
    { 0x9bcb68c7, 95 },
    { 0x72eb68c8, 15 },
    { 0xe8db68c9, 55 },
    { 0x5bbb68ca, 55 },
    { 0, 0 },
    { 0xc4b68cc, 90 },
    { 0x85bb68cd, 61 },
    { 0, 0 },
    { 0x185b68cf, 55 },
    { 0xe1b68d0, 34 },
    { 0xc59b68d1, 61 },
    { 0x7e9b68d2, 61 },
    { 0x950b68d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb68d7, 81 },
    { 0x1cb68d8, 55 },
    { 0x741b68d7, 98 },
    { 0, 0 },
    { 0xf49b68db, 61 },
    { 0xfe9b68dc, 90 },
    { 0xf5cb68dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0b68e1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ab68e5, 76 },
    { 0xf88b68e6, 23 },
    { 0xe48b68e7, 76 },
    { 0xf8bb68e8, 98 },
    { 0xf8eb68e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8b68ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fb68f6, 90 },
    { 0x419b68f7, 61 },
    { 0x91b68f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bb68fb, 88 },
    { 0x4e1b68fc, 85 },
    { 0x27db68fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bb6901, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fb6904, 87 },
    { 0x8aeb6905, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x441b6908, 33 },
    { 0xdd1b6909, 34 },
    { 0x57db6908, 91 },
    { 0x212b690a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138b6912, 98 },
    { 0, 0 },
    { 0x9c4b6914, 61 },
    { 0xd46b6915, 61 },
    { 0, 0 },
    { 0x2f8b6917, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4db691b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fb6921, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ab6926, 76 },
    { 0, 0 },
    { 0x13ab6928, 55 },
    { 0, 0 },
    { 0x3bb692a, 61 },
    { 0xdd6b692a, 99 },
    { 0xed5b692b, 90 },
    { 0x2b9b692d, 84 },
    { 0x26fb692c, 95 },
    { 0, 0 },
    { 0xfe0b6930, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x235b6933, 43 },
    { 0x812b6933, 61 },
    { 0xd6cb6935, 84 },
    { 0x38fb6933, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63b6939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b6940, 76 },
    { 0x448b6940, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab6944, 61 },
    { 0x87bb6944, 49 },
    { 0, 0 },
    { 0x5d6b6947, 90 },
    { 0x95eb6948, 76 },
    { 0x205b6948, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fb694f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbb6954, 90 },
    { 0xfeb6955, 78 },
    { 0x3ffb6956, 98 },
    { 0x74db6957, 76 },
    { 0x10b6958, 55 },
    { 0xc69b6955, 34 },
    { 0x9d6b695a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b695e, 33 },
    { 0x2b8b695f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1db6965, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeb696a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5db696d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b6971, 98 },
    { 0, 0 },
    { 0x21cb6973, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183b6977, 61 },
    { 0xfbeb6978, 61 },
    { 0xa22b6979, 75 },
    { 0xdf9b6979, 52 },
    { 0x95db697a, 93 },
    { 0x686b697a, 43 },
    { 0x1fcb6979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1b6981, 52 },
    { 0x6b7b6981, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x191b6985, 61 },
    { 0x45db6986, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62b698d, 61 },
    { 0xa8eb698e, 23 },
    { 0xf72b698f, 46 },
    { 0xa7ab6990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bb6994, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcb6998, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8b699c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x640b699f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454b69ae, 95 },
    { 0x5bb69ae, 46 },
    { 0, 0 },
    { 0xba4b69b1, 99 },
    { 0, 0 },
    { 0x7bcb69b3, 23 },
    { 0, 0 },
    { 0xd03b69b5, 100 },
    { 0x9f3b69b5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb69ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8b69bd, 93 },
    { 0xe3cb69be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49b69c1, 75 },
    { 0xc84b69c2, 95 },
    { 0x63b69c3, 61 },
    { 0xbfeb69c4, 98 },
    { 0x87ab69c4, 34 },
    { 0xe5bb69c5, 61 },
    { 0x4ebb69c7, 90 },
    { 0xf18b69c8, 52 },
    { 0x426b69c5, 93 },
    { 0xb1db69c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b69ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x632b69d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ab69d8, 64 },
    { 0x458b69d9, 23 },
    { 0x964b69da, 87 },
    { 0, 0 },
    { 0x94bb69dc, 34 },
    { 0, 0 },
    { 0xaacb69de, 81 },
    { 0x970b69de, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eb69e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacb69e6, 33 },
    { 0, 0 },
    { 0xba9b69e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbb69eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92b69f1, 61 },
    { 0, 0 },
    { 0x6d7b69f3, 76 },
    { 0, 0 },
    { 0xacfb69f5, 93 },
    { 0x6b5b69f5, 98 },
    { 0x7e3b69f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307b69fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79b6a00, 93 },
    { 0x483b6a01, 90 },
    { 0, 0 },
    { 0x31fb6a03, 61 },
    { 0xd88b6a04, 76 },
    { 0, 0 },
    { 0xc8bb6a06, 66 },
    { 0x2cdb6a07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41b6a0a, 93 },
    { 0xef7b6a0b, 67 },
    { 0, 0 },
    { 0x744b6a0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8b6a10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663b6a17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3b6a1b, 87 },
    { 0x61fb6a1b, 33 },
    { 0x82fb6a1d, 90 },
    { 0x30fb6a1d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedb6a22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dab6a26, 99 },
    { 0x6a4b6a27, 33 },
    { 0, 0 },
    { 0xb74b6a29, 33 },
    { 0x41cb6a2a, 61 },
    { 0x1f1b6a2b, 98 },
    { 0xc59b6a2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efb6a33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517b6a3a, 23 },
    { 0x997b6a3b, 38 },
    { 0x792b6a3b, 67 },
    { 0xd3cb6a3d, 90 },
    { 0x732b6a3e, 23 },
    { 0, 0 },
    { 0xcc0b6a40, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x79b6a43, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cb6a47, 95 },
    { 0x87fb6a47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfb6a4b, 34 },
    { 0, 0 },
    { 0x36eb6a4d, 84 },
    { 0, 0 },
    { 0xd6b6a4f, 93 },
    { 0xd16b6a50, 77 },
    { 0x392b6a51, 23 },
    { 0xbbbb6a52, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904b6a59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4b6a62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bb6a67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb6a6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2b6a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cb6a76, 90 },
    { 0x728b6a77, 61 },
    { 0x9dab6a78, 100 },
    { 0x698b6a78, 87 },
    { 0xfb2b6a7a, 90 },
    { 0x6bab6a7a, 23 },
    { 0xdfab6a79, 55 },
    { 0xa44b6a7d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb6a84, 75 },
    { 0, 0 },
    { 0xd03b6a86, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275b6a8b, 61 },
    { 0xa26b6a8c, 55 },
    { 0xff9b6a8d, 100 },
    { 0x255b6a8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb6a93, 90 },
    { 0, 0 },
    { 0x79fb6a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab6a98, 95 },
    { 0xf00b6a99, 61 },
    { 0x28ab6a9a, 90 },
    { 0x9bdb6a9b, 23 },
    { 0, 0 },
    { 0x81b6a9d, 90 },
    { 0x5b3b6a9e, 98 },
    { 0, 0 },
    { 0x874b6aa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271b6aa4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ab6aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x640b6aad, 76 },
    { 0x65fb6aad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75b6ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1b6ab4, 95 },
    { 0x5bbb6ab5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0eb6ab9, 78 },
    { 0xf74b6ab9, 34 },
    { 0, 0 },
    { 0xd7ab6abc, 90 },
    { 0xc0ab6abd, 33 },
    { 0x1a2b6abe, 66 },
    { 0x1ffb6abc, 95 },
    { 0, 0 },
    { 0x13cb6ac1, 89 },
    { 0xe7b6ac2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fab6ad2, 23 },
    { 0, 0 },
    { 0x407b6ad4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x70db6ad7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bb6ada, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10b6add, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bb6ae2, 68 },
    { 0, 0 },
    { 0x4bfb6ae4, 98 },
    { 0, 0 },
    { 0x4d6b6ae6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x364b6ae9, 95 },
    { 0, 0 },
    { 0x5c1b6aeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcceb6aef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636b6af4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bb6afd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151b6b03, 90 },
    { 0x38b6b04, 90 },
    { 0, 0 },
    { 0xa7fb6b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eb6b0c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fb6b14, 67 },
    { 0x7cbb6b14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663b6b1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35eb6b1e, 34 },
    { 0x92b6b1f, 95 },
    { 0xc12b6b20, 61 },
    { 0, 0 },
    { 0x87b6b22, 52 },
    { 0, 0 },
    { 0xfb1b6b24, 61 },
    { 0x21b6b24, 75 },
    { 0x670b6b26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68b6b2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88b6b31, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8b6b37, 98 },
    { 0, 0 },
    { 0x478b6b39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edb6b3d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bb6b41, 90 },
    { 0, 0 },
    { 0xc29b6b43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966b6b47, 33 },
    { 0x12ab6b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fb6b4c, 98 },
    { 0, 0 },
    { 0xc5eb6b4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bb6b51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8b6b54, 90 },
    { 0, 0 },
    { 0xed7b6b56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2b6b5b, 76 },
    { 0, 0 },
    { 0x653b6b5d, 23 },
    { 0x9d6b6b5e, 98 },
    { 0, 0 },
    { 0x649b6b60, 90 },
    { 0, 0 },
    { 0xb44b6b62, 95 },
    { 0x361b6b63, 95 },
    { 0xfcfb6b64, 85 },
    { 0xbabb6b64, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5b6b68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x354b6b6b, 66 },
    { 0, 0 },
    { 0x691b6b6d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x132b6b70, 61 },
    { 0xee0b6b71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bb6b75, 46 },
    { 0x523b6b76, 33 },
    { 0xeb5b6b77, 69 },
    { 0, 0 },
    { 0xae1b6b79, 95 },
    { 0x70fb6b7a, 90 },
    { 0x945b6b7b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ab6b7e, 90 },
    { 0x22fb6b7f, 33 },
    { 0xf8db6b80, 55 },
    { 0xeb2b6b81, 23 },
    { 0x9cdb6b81, 33 },
    { 0xf2fb6b81, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2b6b8e, 46 },
    { 0xc74b6b8f, 64 },
    { 0xf7cb6b8e, 98 },
    { 0, 0 },
    { 0xdfbb6b92, 76 },
    { 0x6e7b6b93, 23 },
    { 0, 0 },
    { 0x7a7b6b95, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cb6b99, 61 },
    { 0x7d7b6b9a, 52 },
    { 0, 0 },
    { 0xdcb6b9c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5b6ba1, 90 },
    { 0x455b6ba2, 91 },
    { 0x493b6ba2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1b6ba7, 90 },
    { 0x22ab6ba8, 33 },
    { 0x7b1b6ba8, 55 },
    { 0, 0 },
    { 0xc23b6bab, 34 },
    { 0x46ab6bac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b6bb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x97b6bb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbb6bb6, 33 },
    { 0, 0 },
    { 0x51cb6bb8, 33 },
    { 0x86ab6bb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeb6bbc, 55 },
    { 0, 0 },
    { 0x5f5b6bbe, 84 },
    { 0x277b6bbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882b6bc7, 66 },
    { 0, 0 },
    { 0x8bdb6bc9, 34 },
    { 0x642b6bc9, 34 },
    { 0xd06b6bcb, 90 },
    { 0x8cb6bcc, 34 },
    { 0xc7b6bcd, 76 },
    { 0xcb6b6bce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679b6bd2, 55 },
    { 0x444b6bd2, 90 },
    { 0, 0 },
    { 0xc77b6bd5, 98 },
    { 0x775b6bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b6bd9, 33 },
    { 0xd42b6bda, 90 },
    { 0x34b6bd9, 77 },
    { 0x41ab6bdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306b6be1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b6be5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930b6bee, 90 },
    { 0xc64b6bee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdb6bf6, 33 },
    { 0x997b6bf7, 98 },
    { 0xe56b6bf8, 43 },
    { 0x1a8b6bf9, 98 },
    { 0, 0 },
    { 0xe61b6bfb, 100 },
    { 0, 0 },
    { 0xb50b6bfd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eb6c00, 23 },
    { 0x5beb6c01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eab6c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccb6c0b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bb6c0e, 90 },
    { 0x2aab6c0f, 33 },
    { 0x885b6c0f, 55 },
    { 0xbccb6c11, 33 },
    { 0x6fbb6c11, 61 },
    { 0xf10b6c13, 61 },
    { 0x502b6c14, 34 },
    { 0, 0 },
    { 0x541b6c16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fab6c1f, 90 },
    { 0x965b6c20, 38 },
    { 0x40ab6c20, 55 },
    { 0x70bb6c22, 33 },
    { 0x2e7b6c22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9b6c26, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769b6c2b, 33 },
    { 0x6eab6c2c, 90 },
    { 0x1b6b6c2b, 91 },
    { 0x883b6c2e, 76 },
    { 0xcd8b6c2d, 95 },
    { 0x927b6c30, 61 },
    { 0xd42b6c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b6c34, 98 },
    { 0, 0 },
    { 0x6a2b6c36, 78 },
    { 0xf3b6c36, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecab6c3b, 61 },
    { 0x843b6c3c, 99 },
    { 0xe2bb6c3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eb6c43, 43 },
    { 0, 0 },
    { 0xa6fb6c45, 34 },
    { 0x49fb6c45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51b6c4a, 52 },
    { 0xa46b6c4b, 98 },
    { 0x762b6c4c, 61 },
    { 0x713b6c4d, 33 },
    { 0xf2cb6c4d, 95 },
    { 0xb2ab6c4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3b6c56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x595b6c59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1beb6c5e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3b6c64, 76 },
    { 0xe73b6c65, 61 },
    { 0x208b6c64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ab6c6a, 78 },
    { 0x722b6c6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1b6c75, 83 },
    { 0x6dbb6c75, 23 },
    { 0xe53b6c77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201b6c7c, 61 },
    { 0, 0 },
    { 0x2dbb6c7e, 23 },
    { 0x3fcb6c7f, 17 },
    { 0xd8bb6c80, 64 },
    { 0x503b6c81, 95 },
    { 0x67eb6c82, 66 },
    { 0xeb4b6c83, 95 },
    { 0, 0 },
    { 0x18ab6c85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12b6c8b, 95 },
    { 0, 0 },
    { 0x6a8b6c8d, 61 },
    { 0xbfeb6c8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8b6c91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeeb6c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2db6c9b, 77 },
    { 0x12cb6c9b, 53 },
    { 0xe39b6c9d, 46 },
    { 0xfccb6c9b, 55 },
    { 0, 0 },
    { 0x947b6ca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4b6ca3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25b6ca8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989b6cac, 90 },
    { 0xf6cb6cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5b6cba, 91 },
    { 0x7ab6cbb, 76 },
    { 0xb3cb6cbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fb6cbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab6cc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3b6cc6, 95 },
    { 0xe7db6cc7, 98 },
    { 0x903b6cc8, 76 },
    { 0x933b6cc8, 91 },
    { 0, 0 },
    { 0xf52b6ccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b6cd1, 61 },
    { 0, 0 },
    { 0xe29b6cd3, 95 },
    { 0, 0 },
    { 0x85eb6cd5, 100 },
    { 0xdb9b6cd6, 23 },
    { 0x540b6cd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fb6cda, 43 },
    { 0xd3bb6cdb, 61 },
    { 0, 0 },
    { 0x73cb6cdd, 98 },
    { 0x81b6cdd, 42 },
    { 0xbbab6cde, 90 },
    { 0x4f8b6ce0, 98 },
    { 0x715b6ce0, 66 },
    { 0, 0 },
    { 0x5a3b6ce3, 87 },
    { 0x8dbb6ce3, 52 },
    { 0, 0 },
    { 0x8a5b6ce6, 76 },
    { 0x8dfb6ce6, 91 },
    { 0xb40b6ce7, 91 },
    { 0, 0 },
    { 0xc44b6cea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x362b6ced, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafb6cf4, 61 },
    { 0xe68b6cf5, 90 },
    { 0, 0 },
    { 0x452b6cf7, 23 },
    { 0, 0 },
    { 0xcfdb6cf9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfb6d01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6b6d06, 90 },
    { 0xf51b6d06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0db6d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadb6d13, 95 },
    { 0xb6fb6d13, 76 },
    { 0, 0 },
    { 0x3e6b6d16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81b6d1b, 66 },
    { 0xba8b6d1b, 42 },
    { 0, 0 },
    { 0x1ebb6d1e, 93 },
    { 0x179b6d1f, 33 },
    { 0xa96b6d20, 76 },
    { 0x379b6d21, 61 },
    { 0x364b6d20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90eb6d26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67b6d29, 95 },
    { 0, 0 },
    { 0x36db6d2b, 95 },
    { 0xb07b6d2c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x986b6d2f, 23 },
    { 0xab4b6d30, 100 },
    { 0xe3eb6d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebb6d35, 33 },
    { 0, 0 },
    { 0xb30b6d37, 90 },
    { 0, 0 },
    { 0x7ab6d39, 100 },
    { 0x66cb6d3a, 91 },
    { 0x140b6d3a, 55 },
    { 0xd93b6d39, 90 },
    { 0, 0 },
    { 0x465b6d3e, 76 },
    { 0x10bb6d3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75db6d43, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1b6d4a, 23 },
    { 0xa46b6d4b, 23 },
    { 0x57b6d4c, 76 },
    { 0, 0 },
    { 0xd6fb6d4e, 76 },
    { 0x286b6d4e, 55 },
    { 0, 0 },
    { 0x52bb6d51, 95 },
    { 0x324b6d51, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70b6d56, 76 },
    { 0xc96b6d57, 91 },
    { 0x411b6d58, 43 },
    { 0x54cb6d56, 66 },
    { 0x102b6d5a, 61 },
    { 0x320b6d5a, 90 },
    { 0xbd5b6d5b, 76 },
    { 0x7e2b6d5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeb6d64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ab6d6a, 55 },
    { 0xf2cb6d6b, 34 },
    { 0x98b6d6c, 46 },
    { 0x7fab6d6c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325b6d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72b6d78, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceb6d7b, 84 },
    { 0x7deb6d7c, 87 },
    { 0, 0 },
    { 0x840b6d7e, 68 },
    { 0x40eb6d7f, 61 },
    { 0xa7eb6d7e, 100 },
    { 0xa94b6d80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0b6d87, 61 },
    { 0xf33b6d88, 34 },
    { 0x8fdb6d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332b6d8f, 77 },
    { 0x875b6d90, 98 },
    { 0, 0 },
    { 0x3b8b6d92, 23 },
    { 0xaefb6d93, 88 },
    { 0x1e1b6d93, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adb6d9b, 75 },
    { 0, 0 },
    { 0xf54b6d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93b6da5, 98 },
    { 0, 0 },
    { 0x687b6da7, 98 },
    { 0xd14b6da8, 33 },
    { 0x743b6da8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbb6dae, 61 },
    { 0x20cb6dae, 78 },
    { 0xb69b6db0, 98 },
    { 0x635b6db1, 61 },
    { 0x2e2b6db1, 77 },
    { 0x703b6db2, 66 },
    { 0x4afb6db4, 66 },
    { 0, 0 },
    { 0x16b6db6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdb6db9, 34 },
    { 0xe2eb6dba, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464b6dbe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6b6dc3, 90 },
    { 0x4cbb6dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98b6dc7, 23 },
    { 0x9e9b6dc8, 90 },
    { 0xedeb6dc9, 23 },
    { 0xfe8b6dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cb6dd0, 90 },
    { 0xc55b6dd1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb6dd5, 93 },
    { 0x679b6dd5, 90 },
    { 0xb14b6dd6, 95 },
    { 0x77bb6dd8, 33 },
    { 0x938b6dd9, 23 },
    { 0x726b6dda, 75 },
    { 0, 0 },
    { 0x130b6ddc, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322b6de2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cb6de5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb6de8, 34 },
    { 0x5d4b6de9, 95 },
    { 0, 0 },
    { 0xf8b6deb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ab6dee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1b6df1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb6df4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81db6dfa, 90 },
    { 0, 0 },
    { 0xdeab6dfc, 81 },
    { 0xcb6dfd, 53 },
    { 0x5f3b6dfd, 66 },
    { 0xc1b6dff, 76 },
    { 0x406b6e00, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681b6e08, 99 },
    { 0, 0 },
    { 0x5b0b6e0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47b6e0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b6e14, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8b6e1a, 34 },
    { 0xe76b6e1b, 66 },
    { 0x243b6e1c, 100 },
    { 0xc8ab6e1d, 69 },
    { 0xbb9b6e1e, 61 },
    { 0xfe2b6e1f, 84 },
    { 0, 0 },
    { 0x6b5b6e21, 90 },
    { 0x2e4b6e22, 75 },
    { 0xd2ab6e22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaeb6e29, 95 },
    { 0, 0 },
    { 0x658b6e2b, 66 },
    { 0x936b6e2c, 90 },
    { 0x73fb6e2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9b6e33, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83b6e38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcb6e43, 84 },
    { 0, 0 },
    { 0x3b8b6e45, 95 },
    { 0, 0 },
    { 0x27ab6e47, 69 },
    { 0x2d1b6e48, 66 },
    { 0, 0 },
    { 0x634b6e4a, 55 },
    { 0x28ab6e4b, 76 },
    { 0x151b6e4c, 85 },
    { 0x6a8b6e4d, 95 },
    { 0x9a8b6e4d, 34 },
    { 0, 0 },
    { 0xf93b6e50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73b6e54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927b6e58, 61 },
    { 0, 0 },
    { 0x43eb6e5a, 90 },
    { 0x3b9b6e5b, 88 },
    { 0xc7fb6e5c, 67 },
    { 0x8efb6e5b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b6e63, 76 },
    { 0x2dab6e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x763b6e67, 78 },
    { 0x6afb6e68, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1b6e6c, 23 },
    { 0x73eb6e6d, 33 },
    { 0x24cb6e6d, 66 },
    { 0xd2eb6e6f, 34 },
    { 0x966b6e70, 66 },
    { 0xcc3b6e70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b6e74, 33 },
    { 0xcedb6e75, 33 },
    { 0x700b6e76, 61 },
    { 0xbdbb6e74, 98 },
    { 0x2b8b6e78, 99 },
    { 0x8f3b6e79, 46 },
    { 0xef0b6e79, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104b6e7f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182b6e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ab6e8b, 95 },
    { 0, 0 },
    { 0x58eb6e8d, 90 },
    { 0x41fb6e8e, 52 },
    { 0xb12b6e8e, 52 },
    { 0x9f1b6e8d, 99 },
    { 0x506b6e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21db6e98, 90 },
    { 0x146b6e98, 76 },
    { 0x2d9b6e99, 90 },
    { 0x785b6e9b, 95 },
    { 0, 0 },
    { 0x999b6e9d, 34 },
    { 0xefb6e9e, 34 },
    { 0xcf6b6e9f, 90 },
    { 0x5b5b6ea0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70b6ea4, 34 },
    { 0, 0 },
    { 0x657b6ea6, 34 },
    { 0x403b6ea7, 77 },
    { 0xbffb6ea8, 23 },
    { 0xa27b6ea7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eb6eb0, 33 },
    { 0xc10b6eb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eb6eb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7b6eb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab6ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8b6ec6, 33 },
    { 0, 0 },
    { 0x575b6ec8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb6ecc, 34 },
    { 0x205b6ecc, 98 },
    { 0xe0fb6ece, 23 },
    { 0x8cbb6ecf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ab6ed5, 90 },
    { 0x15fb6ed6, 33 },
    { 0xd70b6ed7, 98 },
    { 0xfb3b6ed7, 33 },
    { 0xc9ab6ed9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9b6edc, 33 },
    { 0xc90b6edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x312b6ee0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb6ee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb6ee7, 90 },
    { 0, 0 },
    { 0xab0b6ee9, 90 },
    { 0xa3cb6eea, 95 },
    { 0x33ab6eeb, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeab6ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35b6ef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fb6efa, 46 },
    { 0x5feb6efb, 23 },
    { 0x5cb6efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5b6f01, 90 },
    { 0, 0 },
    { 0x8c3b6f03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b6f0c, 77 },
    { 0x87ab6f0c, 33 },
    { 0xe0db6f0e, 85 },
    { 0xcbab6f0d, 61 },
    { 0x43eb6f10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6b6f13, 61 },
    { 0, 0 },
    { 0x43db6f15, 66 },
    { 0x87b6f16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0b6f19, 98 },
    { 0, 0 },
    { 0xd36b6f1b, 43 },
    { 0, 0 },
    { 0xde9b6f1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6b6f20, 43 },
    { 0x46bb6f21, 33 },
    { 0, 0 },
    { 0x87cb6f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2b6f26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bb6f2b, 33 },
    { 0, 0 },
    { 0xce8b6f2d, 33 },
    { 0xe5cb6f2e, 95 },
    { 0, 0 },
    { 0x21ab6f30, 90 },
    { 0, 0 },
    { 0x932b6f32, 33 },
    { 0x36ab6f33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bb6f43, 33 },
    { 0xf70b6f44, 33 },
    { 0, 0 },
    { 0x5b6f46, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8b6f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31b6f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4eb6f50, 99 },
    { 0, 0 },
    { 0xb3db6f52, 61 },
    { 0, 0 },
    { 0xa60b6f54, 84 },
    { 0, 0 },
    { 0x1ddb6f56, 76 },
    { 0x7deb6f57, 90 },
    { 0x4b0b6f58, 61 },
    { 0x3f0b6f59, 61 },
    { 0xa9cb6f57, 91 },
    { 0x8aab6f5b, 33 },
    { 0xe78b6f59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x877b6f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eb6f63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568b6f69, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2b6f6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b6f6f, 98 },
    { 0x3b6b6f70, 90 },
    { 0xb64b6f70, 80 },
    { 0x176b6f72, 99 },
    { 0x154b6f70, 61 },
    { 0x5c3b6f74, 34 },
    { 0x167b6f6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b6f78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eeb6f7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2b6f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bb6f82, 98 },
    { 0x74eb6f83, 95 },
    { 0, 0 },
    { 0x843b6f85, 99 },
    { 0xefab6f86, 90 },
    { 0, 0 },
    { 0xff3b6f88, 90 },
    { 0x537b6f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8b6f8d, 34 },
    { 0, 0 },
    { 0x28cb6f8f, 46 },
    { 0xe12b6f8f, 98 },
    { 0xfa5b6f90, 76 },
    { 0, 0 },
    { 0x6c6b6f93, 98 },
    { 0, 0 },
    { 0xecdb6f95, 43 },
    { 0x64fb6f95, 66 },
    { 0, 0 },
    { 0x2f8b6f98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x820b6f9b, 23 },
    { 0, 0 },
    { 0x7f6b6f9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eb6fa0, 61 },
    { 0x7e6b6fa1, 99 },
    { 0xcb0b6fa2, 61 },
    { 0x8a9b6fa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fb6fa8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eb6fab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204b6fb1, 43 },
    { 0x54b6fb2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b6fb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98b6fbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7b6fc6, 67 },
    { 0x36db6fc7, 67 },
    { 0x763b6fc6, 91 },
    { 0x161b6fc7, 66 },
    { 0x7f8b6fca, 67 },
    { 0x418b6fca, 75 },
    { 0x9ebb6fcc, 98 },
    { 0x56bb6fca, 90 },
    { 0, 0 },
    { 0x285b6fcf, 90 },
    { 0, 0 },
    { 0xd82b6fd1, 23 },
    { 0x440b6fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2b6fdf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9b6fe7, 33 },
    { 0xcdfb6fe8, 90 },
    { 0x5bfb6fe7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6b6ff0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73b6ff5, 87 },
    { 0, 0 },
    { 0xc37b6ff7, 23 },
    { 0xa92b6ff8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db6ffc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fb7001, 88 },
    { 0xaf3b7002, 61 },
    { 0x220b7003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391b7007, 77 },
    { 0, 0 },
    { 0x3a7b7009, 68 },
    { 0x866b700a, 41 },
    { 0x206b700b, 61 },
    { 0, 0 },
    { 0x96ab700d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfb7014, 23 },
    { 0, 0 },
    { 0xd4ab7016, 23 },
    { 0, 0 },
    { 0x9d6b7018, 23 },
    { 0xd90b7019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cb7020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bb7025, 33 },
    { 0xd36b7025, 90 },
    { 0xddfb7025, 61 },
    { 0, 0 },
    { 0x754b7029, 33 },
    { 0x7f6b702a, 93 },
    { 0x4f0b702a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfb7031, 40 },
    { 0xd84b7031, 98 },
    { 0, 0 },
    { 0xb3b7034, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32eb7037, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4b703b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ab703f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab7042, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237b7046, 85 },
    { 0xb2ab7046, 90 },
    { 0xe7b7048, 43 },
    { 0x858b7048, 33 },
    { 0xb5eb704a, 34 },
    { 0xdf4b7049, 67 },
    { 0x317b704c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1b704f, 28 },
    { 0, 0 },
    { 0x800b7051, 98 },
    { 0x118b7052, 23 },
    { 0x312b7052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ab7058, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbb705d, 98 },
    { 0x718b705e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1db7062, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fb7065, 33 },
    { 0, 0 },
    { 0x6e3b7067, 46 },
    { 0xed4b7067, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eb7070, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x269b7073, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x376b7076, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccb7079, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cb707c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb7081, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcb708a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfb708f, 33 },
    { 0, 0 },
    { 0xa54b7091, 61 },
    { 0xc44b7092, 90 },
    { 0xaf4b7093, 95 },
    { 0x69eb7094, 87 },
    { 0xdb1b7094, 90 },
    { 0xd56b7096, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeb709e, 91 },
    { 0xe74b709f, 34 },
    { 0, 0 },
    { 0x632b70a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5b70a5, 23 },
    { 0, 0 },
    { 0xdeeb70a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28b70af, 23 },
    { 0x9e9b70b0, 75 },
    { 0xd6fb70b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eb70b4, 93 },
    { 0x201b70b4, 55 },
    { 0, 0 },
    { 0xd3b70b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbb70c7, 37 },
    { 0xf8fb70c8, 55 },
    { 0xed3b70c9, 91 },
    { 0x95db70c9, 99 },
    { 0x850b70c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecb70d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bb70e5, 90 },
    { 0x223b70e6, 95 },
    { 0xf14b70e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68b70eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bb70ee, 87 },
    { 0x72cb70ef, 90 },
    { 0, 0 },
    { 0xdcab70f1, 33 },
    { 0x1feb70f1, 87 },
    { 0x317b70f3, 33 },
    { 0xa1fb70f1, 17 },
    { 0, 0 },
    { 0x681b70f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x258b70f9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bab7102, 93 },
    { 0x5beb7102, 53 },
    { 0, 0 },
    { 0x10eb7105, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31b7109, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x557b710c, 61 },
    { 0xc81b710d, 46 },
    { 0xe34b710d, 98 },
    { 0xe67b710f, 61 },
    { 0, 0 },
    { 0xb04b7111, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deb7119, 87 },
    { 0x51bb7119, 40 },
    { 0xbb6b711a, 76 },
    { 0xe7bb711c, 98 },
    { 0x912b7119, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5b7120, 34 },
    { 0x878b7121, 95 },
    { 0xecab7122, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db7129, 89 },
    { 0x220b712a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08b712d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895b7131, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcb7138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0b713c, 95 },
    { 0x3d6b713d, 90 },
    { 0x692b713e, 90 },
    { 0xfabb713f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdb7148, 43 },
    { 0x651b7149, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfb714e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bab7154, 67 },
    { 0x2db7155, 95 },
    { 0, 0 },
    { 0xcabb7157, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490b715b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0b7163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eb7167, 61 },
    { 0x7deb7168, 87 },
    { 0xddeb7168, 90 },
    { 0x6f0b716a, 95 },
    { 0xa01b716b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdb716f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xceab7172, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1b7177, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1b7183, 95 },
    { 0xe8ab7184, 76 },
    { 0x43ab7185, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x805b7188, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcb718b, 95 },
    { 0xb79b718c, 98 },
    { 0x2c4b718c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172b719b, 98 },
    { 0x75ab719c, 43 },
    { 0xa0bb719d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebb71a0, 91 },
    { 0, 0 },
    { 0x4f1b71a2, 34 },
    { 0, 0 },
    { 0x684b71a4, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0b71a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2b71ab, 23 },
    { 0, 0 },
    { 0x126b71ad, 23 },
    { 0x7b3b71ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ab71b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75b71b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b71bd, 4 },
    { 0x495b71be, 61 },
    { 0x350b71bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9b71c2, 38 },
    { 0xbb1b71c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fb71c6, 93 },
    { 0x4a8b71c6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9b71ca, 61 },
    { 0xe96b71cb, 55 },
    { 0xcc3b71cc, 90 },
    { 0, 0 },
    { 0xe1bb71ce, 33 },
    { 0, 0 },
    { 0x128b71d0, 61 },
    { 0x514b71d1, 56 },
    { 0, 0 },
    { 0x8a3b71d3, 93 },
    { 0, 0 },
    { 0x1a5b71d5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0b71dc, 61 },
    { 0x490b71dd, 33 },
    { 0x4c4b71de, 95 },
    { 0xdebb71df, 98 },
    { 0, 0 },
    { 0x477b71e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6b71eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afb71ef, 55 },
    { 0xe4b71f0, 76 },
    { 0xe85b71f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eb71f6, 38 },
    { 0xd97b71f7, 65 },
    { 0x7f2b71f8, 100 },
    { 0x889b71f9, 98 },
    { 0x70ab71f8, 90 },
    { 0x4dab71f6, 33 },
    { 0, 0 },
    { 0x29ab71fd, 66 },
    { 0, 0 },
    { 0x26db71ff, 76 },
    { 0xa4cb7200, 34 },
    { 0, 0 },
    { 0x810b7202, 34 },
    { 0xf34b7202, 46 },
    { 0x156b7204, 66 },
    { 0xf14b7202, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79b720d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ab7210, 61 },
    { 0x95fb7210, 23 },
    { 0x10eb7211, 46 },
    { 0x63db7213, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x780b7216, 68 },
    { 0, 0 },
    { 0xefbb7218, 90 },
    { 0, 0 },
    { 0x850b721a, 34 },
    { 0, 0 },
    { 0xa49b721c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309b7220, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb7223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800b7230, 43 },
    { 0x322b7230, 95 },
    { 0x517b7230, 100 },
    { 0, 0 },
    { 0x8e3b7234, 67 },
    { 0xdedb7235, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4b723c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4b723f, 95 },
    { 0xad1b7240, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5b7243, 52 },
    { 0, 0 },
    { 0x83eb7245, 33 },
    { 0xbbcb7245, 61 },
    { 0xf62b7245, 66 },
    { 0xf23b7245, 23 },
    { 0x2d5b7249, 52 },
    { 0x948b724a, 33 },
    { 0xe4fb724a, 17 },
    { 0x6dab724c, 33 },
    { 0x142b724d, 90 },
    { 0, 0 },
    { 0x9efb724f, 95 },
    { 0xeabb7250, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb7257, 65 },
    { 0x49fb7258, 61 },
    { 0, 0 },
    { 0x3ffb725a, 90 },
    { 0x62db725b, 78 },
    { 0, 0 },
    { 0x877b725d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bb7260, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790b7264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514b7269, 61 },
    { 0x1f2b726a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fb726d, 67 },
    { 0x33eb726e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fb7271, 33 },
    { 0x255b7272, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbb7275, 76 },
    { 0xb57b7276, 98 },
    { 0xeb3b7277, 90 },
    { 0x5a7b7278, 93 },
    { 0xf2ab7278, 98 },
    { 0x36db727a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb727e, 69 },
    { 0, 0 },
    { 0xd8ab7280, 75 },
    { 0, 0 },
    { 0xbfab7282, 49 },
    { 0xcbdb7282, 67 },
    { 0x6b2b7283, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08b7288, 78 },
    { 0x154b7289, 93 },
    { 0xf69b7288, 33 },
    { 0x1e2b728b, 90 },
    { 0x5dbb728b, 61 },
    { 0xb0b728b, 61 },
    { 0xbf6b728c, 98 },
    { 0, 0 },
    { 0xdb8b7290, 95 },
    { 0x55eb7290, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eb7299, 33 },
    { 0xd88b7299, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb729e, 84 },
    { 0xf5fb729f, 61 },
    { 0x192b72a0, 90 },
    { 0x540b729e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbab72a4, 43 },
    { 0, 0 },
    { 0x3b2b72a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b72ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fb72ae, 61 },
    { 0, 0 },
    { 0x4b2b72b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935b72b5, 91 },
    { 0x197b72b6, 85 },
    { 0x153b72b7, 33 },
    { 0xed2b72b8, 55 },
    { 0, 0 },
    { 0x48eb72ba, 61 },
    { 0xf3ab72ba, 34 },
    { 0x48cb72ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1b72c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb72ca, 46 },
    { 0xafdb72cb, 61 },
    { 0x68cb72cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb72d2, 61 },
    { 0, 0 },
    { 0x9abb72d4, 98 },
    { 0, 0 },
    { 0x3dfb72d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x22eb72d9, 23 },
    { 0, 0 },
    { 0xbd3b72db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592b72e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780b72e9, 34 },
    { 0xbc7b72ea, 90 },
    { 0, 0 },
    { 0xb17b72ec, 46 },
    { 0x3e6b72ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bb72f0, 95 },
    { 0xd06b72f0, 90 },
    { 0x485b72f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349b72f8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x295b72fb, 23 },
    { 0, 0 },
    { 0xb7ab72fd, 90 },
    { 0x62fb72fe, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1b7308, 23 },
    { 0xf5b7309, 77 },
    { 0, 0 },
    { 0x6f5b730b, 93 },
    { 0x323b730c, 95 },
    { 0x761b730b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00b7315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbceb7318, 33 },
    { 0, 0 },
    { 0x4cab731a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddab731f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710b7324, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08b7328, 23 },
    { 0, 0 },
    { 0x1ffb732a, 33 },
    { 0, 0 },
    { 0xaf3b732c, 76 },
    { 0xf10b732d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b7330, 90 },
    { 0x2ebb7331, 46 },
    { 0xf89b7332, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb7339, 87 },
    { 0x5a2b733a, 100 },
    { 0x61bb733b, 46 },
    { 0x881b733b, 83 },
    { 0x366b7339, 61 },
    { 0xbffb733c, 34 },
    { 0xcc0b7339, 61 },
    { 0x3db7340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54db7349, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28b734d, 69 },
    { 0xdd0b734e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bb7352, 36 },
    { 0x25cb7353, 90 },
    { 0x2fcb7354, 33 },
    { 0, 0 },
    { 0xee2b7356, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b735a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eb7362, 90 },
    { 0x4db7363, 95 },
    { 0xc60b7363, 84 },
    { 0, 0 },
    { 0xe51b7366, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbb7369, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb736e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdb7372, 23 },
    { 0x74eb7372, 99 },
    { 0x5e9b7374, 90 },
    { 0x4c6b7373, 91 },
    { 0x2e8b7376, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdb737a, 98 },
    { 0x96fb737b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782b737f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349b7383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3b7388, 99 },
    { 0, 0 },
    { 0x57cb738a, 61 },
    { 0xf6ab738b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841b7393, 52 },
    { 0xca3b7393, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773b739e, 5 },
    { 0, 0 },
    { 0xfb1b73a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b73a5, 66 },
    { 0xbf4b73a5, 34 },
    { 0xb1ab73a7, 33 },
    { 0xd12b73a8, 43 },
    { 0x27db73a7, 100 },
    { 0, 0 },
    { 0xa38b73ab, 61 },
    { 0, 0 },
    { 0x8c4b73ad, 33 },
    { 0x2db73ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00b73b5, 98 },
    { 0x52bb73b6, 90 },
    { 0x511b73b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b73bb, 66 },
    { 0xba4b73bc, 23 },
    { 0xeedb73bd, 23 },
    { 0xc67b73bc, 61 },
    { 0xed2b73bf, 84 },
    { 0x971b73be, 61 },
    { 0xaa1b73c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccb73c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb73ca, 61 },
    { 0x99ab73cb, 33 },
    { 0, 0 },
    { 0xcc9b73cd, 90 },
    { 0xd61b73ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0b73d2, 98 },
    { 0, 0 },
    { 0xf9db73d4, 95 },
    { 0xf9ab73d5, 17 },
    { 0xba5b73d4, 55 },
    { 0x9cab73d7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b73db, 85 },
    { 0x201b73dc, 61 },
    { 0x7ab73dd, 98 },
    { 0x1fdb73de, 91 },
    { 0xcbbb73dd, 33 },
    { 0xa6cb73e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91eb73ef, 55 },
    { 0, 0 },
    { 0xbd2b73f1, 61 },
    { 0x1d0b73f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6b73f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53b73fd, 23 },
    { 0x2dcb73fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ab7406, 78 },
    { 0xbb0b7407, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fb740d, 61 },
    { 0x944b740e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eb7411, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3b7414, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb7417, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99b741b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2db7420, 90 },
    { 0x3a2b7421, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ab7427, 43 },
    { 0xee9b7428, 23 },
    { 0xb2b7429, 90 },
    { 0, 0 },
    { 0xcacb742b, 43 },
    { 0x9f1b742c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500b7431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686b7438, 90 },
    { 0x2f6b7438, 23 },
    { 0, 0 },
    { 0xafeb743b, 95 },
    { 0x966b743c, 95 },
    { 0, 0 },
    { 0xc18b743e, 95 },
    { 0xbe9b743e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6db7442, 78 },
    { 0xe5ab7443, 98 },
    { 0xe46b7442, 90 },
    { 0x300b7443, 43 },
    { 0, 0 },
    { 0x5e0b7447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x316b744a, 23 },
    { 0, 0 },
    { 0x50cb744c, 23 },
    { 0, 0 },
    { 0x1eab744e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b7453, 78 },
    { 0x77cb7453, 95 },
    { 0x74fb7455, 98 },
    { 0x72cb7454, 90 },
    { 0xdb5b7457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447b745b, 23 },
    { 0, 0 },
    { 0x8ccb745d, 77 },
    { 0xc4b745e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fb7461, 76 },
    { 0, 0 },
    { 0xe7bb7463, 76 },
    { 0x439b7464, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ab7468, 98 },
    { 0x92bb7469, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b746c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x361b746f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03b7478, 23 },
    { 0, 0 },
    { 0x3d4b747a, 33 },
    { 0xef1b747a, 90 },
    { 0x5c0b747c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b7480, 95 },
    { 0xeb7480, 23 },
    { 0x253b7481, 33 },
    { 0x47eb7483, 76 },
    { 0xd7db7484, 56 },
    { 0x391b7484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdb7488, 90 },
    { 0, 0 },
    { 0xc14b748a, 101 },
    { 0xbabb748b, 61 },
    { 0xde2b748c, 69 },
    { 0xa4db748d, 90 },
    { 0x93cb748d, 61 },
    { 0x1a3b748e, 95 },
    { 0xdd5b748d, 98 },
    { 0xd80b748d, 34 },
    { 0xf03b7492, 33 },
    { 0xa92b7493, 23 },
    { 0, 0 },
    { 0xac6b7495, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76b749b, 100 },
    { 0x228b749b, 33 },
    { 0xc52b749d, 23 },
    { 0x6f4b749d, 61 },
    { 0xf0bb749f, 23 },
    { 0, 0 },
    { 0x21cb74a1, 84 },
    { 0, 0 },
    { 0xb2ab74a3, 23 },
    { 0xaaeb74a3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1b74a8, 87 },
    { 0xbb3b74a9, 23 },
    { 0x56fb74aa, 23 },
    { 0x14b74ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6b74b0, 95 },
    { 0xaf6b74b1, 61 },
    { 0, 0 },
    { 0xdd0b74b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efb74b6, 98 },
    { 0x777b74b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878b74bb, 90 },
    { 0, 0 },
    { 0xe02b74bd, 17 },
    { 0xb47b74bd, 90 },
    { 0x58ab74bd, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6b74c8, 61 },
    { 0x61ab74c8, 24 },
    { 0, 0 },
    { 0xdcdb74cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb74d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6b74d8, 45 },
    { 0xbc1b74d8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40b74de, 90 },
    { 0x843b74df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb74eb, 90 },
    { 0x2e8b74ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb74f2, 90 },
    { 0x36db74f2, 23 },
    { 0xf88b74f2, 23 },
    { 0x1d9b74f3, 33 },
    { 0x530b74f6, 76 },
    { 0, 0 },
    { 0x2c2b74f8, 43 },
    { 0x52cb74f9, 90 },
    { 0x9dbb74f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4b74fe, 33 },
    { 0, 0 },
    { 0x148b7500, 33 },
    { 0x181b7500, 23 },
    { 0xed4b7502, 77 },
    { 0, 0 },
    { 0x699b7504, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481b7511, 100 },
    { 0xdafb7511, 90 },
    { 0xfb1b7511, 33 },
    { 0x620b7514, 61 },
    { 0xab7512, 95 },
    { 0xf48b7515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feb751a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24b751f, 95 },
    { 0x67b751f, 23 },
    { 0xdcab751f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edb7528, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8b753a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eb753e, 95 },
    { 0x60b753f, 53 },
    { 0x820b753e, 95 },
    { 0x631b7541, 95 },
    { 0, 0 },
    { 0x190b7543, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b754d, 23 },
    { 0x748b754e, 33 },
    { 0x53ab754d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11db7552, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989b7557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b755c, 23 },
    { 0x15cb755d, 38 },
    { 0x77b755d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82b7561, 34 },
    { 0, 0 },
    { 0x769b7563, 95 },
    { 0x3fab7564, 23 },
    { 0, 0 },
    { 0x465b7566, 56 },
    { 0x86b7567, 93 },
    { 0x127b7566, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bb756b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebb7572, 100 },
    { 0xa68b7572, 34 },
    { 0x85bb7574, 23 },
    { 0, 0 },
    { 0xec0b7576, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24b7579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ab757d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19b7583, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb758e, 67 },
    { 0x989b758f, 98 },
    { 0x3a3b758f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfb7596, 33 },
    { 0xa0cb7597, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ab759b, 33 },
    { 0x326b759b, 98 },
    { 0xfb9b759d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ab75a2, 81 },
    { 0, 0 },
    { 0x982b75a4, 33 },
    { 0x222b75a5, 33 },
    { 0x8bdb75a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacb75ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5b75b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebb75b5, 76 },
    { 0x18db75b5, 33 },
    { 0xdeb75b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11b75be, 77 },
    { 0xc89b75bf, 68 },
    { 0xc54b75c0, 88 },
    { 0x41fb75bf, 23 },
    { 0xa9db75c2, 87 },
    { 0, 0 },
    { 0x5b9b75c4, 38 },
    { 0x78ab75c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceb75c9, 90 },
    { 0x1db75c9, 99 },
    { 0x619b75c9, 61 },
    { 0, 0 },
    { 0x58bb75cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9b75d2, 93 },
    { 0, 0 },
    { 0x98cb75d4, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x385b75d7, 98 },
    { 0x166b75d8, 23 },
    { 0x545b75d9, 99 },
    { 0xd18b75da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cb75de, 90 },
    { 0xe15b75df, 55 },
    { 0x492b75df, 95 },
    { 0xb27b75e1, 98 },
    { 0xa01b75e1, 33 },
    { 0xccfb75e3, 94 },
    { 0x4cbb75e2, 43 },
    { 0x8b75e5, 95 },
    { 0, 0 },
    { 0x117b75e7, 67 },
    { 0x57b75e8, 95 },
    { 0, 0 },
    { 0x304b75ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b75ee, 76 },
    { 0x85ab75ef, 85 },
    { 0xc6cb75ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71eb75f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eb75fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbb7601, 98 },
    { 0xc90b7601, 69 },
    { 0xf21b7601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79b7606, 83 },
    { 0, 0 },
    { 0x4f4b7608, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2b760f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb7613, 17 },
    { 0, 0 },
    { 0xa56b7615, 90 },
    { 0xd2db7616, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b761a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcb761e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409b7622, 90 },
    { 0x143b7623, 46 },
    { 0x176b7623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eb7628, 61 },
    { 0xe27b7629, 43 },
    { 0, 0 },
    { 0xaadb762b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67b7638, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab7647, 100 },
    { 0, 0 },
    { 0xc5cb7649, 33 },
    { 0xee4b7649, 90 },
    { 0, 0 },
    { 0x52db764c, 43 },
    { 0x585b764c, 61 },
    { 0xc15b764e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683b7652, 40 },
    { 0x7a3b7653, 98 },
    { 0x4d5b7654, 95 },
    { 0xed6b7655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ab765c, 98 },
    { 0, 0 },
    { 0x32fb765e, 34 },
    { 0x31ab765e, 90 },
    { 0xf76b765e, 70 },
    { 0xac7b765f, 93 },
    { 0x1b8b7662, 33 },
    { 0x1eeb7662, 17 },
    { 0, 0 },
    { 0x767b7665, 98 },
    { 0x4b9b7666, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x632b7669, 61 },
    { 0x17db766a, 91 },
    { 0x1e9b766b, 43 },
    { 0, 0 },
    { 0x5a2b766d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45db7674, 98 },
    { 0, 0 },
    { 0xe48b7676, 91 },
    { 0xa03b7677, 98 },
    { 0xaf3b7678, 95 },
    { 0x743b7677, 76 },
    { 0, 0 },
    { 0xfbbb767b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62b7681, 77 },
    { 0xd1eb7682, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975b7689, 98 },
    { 0xb91b768a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ab768d, 55 },
    { 0x7b3b768d, 90 },
    { 0xc76b768d, 95 },
    { 0x8a6b7690, 78 },
    { 0x371b7691, 55 },
    { 0x3cdb7692, 55 },
    { 0x7eb7693, 66 },
    { 0x70cb7693, 98 },
    { 0, 0 },
    { 0x30bb7696, 46 },
    { 0xf0bb7697, 61 },
    { 0xba3b7698, 95 },
    { 0x71ab7699, 66 },
    { 0x8d3b7697, 61 },
    { 0, 0 },
    { 0x710b769c, 23 },
    { 0, 0 },
    { 0x4afb769e, 61 },
    { 0xd0eb769f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cb76a3, 95 },
    { 0x1edb76a4, 96 },
    { 0x9d5b76a5, 23 },
    { 0xb0ab76a3, 90 },
    { 0x652b76a7, 76 },
    { 0x24cb76a7, 40 },
    { 0x989b76a4, 95 },
    { 0xef4b76aa, 33 },
    { 0, 0 },
    { 0x50db76ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eb76b0, 55 },
    { 0, 0 },
    { 0xc7ab76b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21b76b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x835b76bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6b76c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8b76c8, 90 },
    { 0x49fb76c9, 98 },
    { 0x1efb76ca, 23 },
    { 0x3f1b76cb, 40 },
    { 0x278b76cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210b76d3, 23 },
    { 0, 0 },
    { 0xa2db76d5, 95 },
    { 0, 0 },
    { 0xcddb76d7, 69 },
    { 0x277b76d8, 33 },
    { 0x76bb76d8, 98 },
    { 0x443b76da, 90 },
    { 0xf9bb76d7, 33 },
    { 0xd4db76db, 23 },
    { 0x98cb76db, 95 },
    { 0x323b76de, 34 },
    { 0x3d2b76df, 61 },
    { 0xe47b76e0, 66 },
    { 0x51eb76e0, 98 },
    { 0x321b76da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eb76e8, 52 },
    { 0x3e6b76e8, 34 },
    { 0xf25b76ea, 61 },
    { 0x813b76ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229b76f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863b76f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9b76fd, 23 },
    { 0xdf1b76fe, 98 },
    { 0xd77b76fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7703, 33 },
    { 0xec4b7704, 98 },
    { 0, 0 },
    { 0x969b7706, 46 },
    { 0xc63b7707, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1b770a, 90 },
    { 0, 0 },
    { 0x5dfb770c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ab770f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54b771c, 98 },
    { 0x75bb771d, 76 },
    { 0x9f3b771d, 23 },
    { 0x79eb771e, 76 },
    { 0xebeb7720, 33 },
    { 0xc76b7721, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbb7724, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x414b7727, 61 },
    { 0xfbb7728, 98 },
    { 0xaabb7729, 34 },
    { 0xcfdb772a, 98 },
    { 0xd6b7729, 34 },
    { 0xfc2b7728, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4b7736, 98 },
    { 0, 0 },
    { 0xd25b7738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41b773c, 55 },
    { 0, 0 },
    { 0x2beb773e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x898b7741, 33 },
    { 0x944b7742, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ab7748, 90 },
    { 0xf22b7748, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fab7752, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7b7756, 95 },
    { 0x8c8b7757, 93 },
    { 0xab7758, 98 },
    { 0xb8fb7757, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9beb775d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x579b7760, 87 },
    { 0, 0 },
    { 0xd0fb7762, 34 },
    { 0x831b7763, 61 },
    { 0x9c9b7762, 33 },
    { 0, 0 },
    { 0x129b7766, 90 },
    { 0x545b7767, 36 },
    { 0xbd9b7767, 33 },
    { 0xc90b7767, 84 },
    { 0x6a7b776a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209b776f, 33 },
    { 0x943b776f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edb777a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2b777f, 33 },
    { 0xa91b7780, 33 },
    { 0xa43b7780, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3b7785, 75 },
    { 0x744b7786, 55 },
    { 0xe97b7787, 34 },
    { 0x8a8b7788, 23 },
    { 0xf9b7787, 95 },
    { 0xb98b778a, 23 },
    { 0x21db7786, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4b778e, 65 },
    { 0x3b2b778e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524b7793, 55 },
    { 0, 0 },
    { 0x51cb7795, 33 },
    { 0xe6ab7795, 55 },
    { 0x833b7797, 23 },
    { 0, 0 },
    { 0xdbab7799, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5b779f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29b77a4, 61 },
    { 0x819b77a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39eb77a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecb77ac, 61 },
    { 0xf54b77ac, 61 },
    { 0x9beb77ad, 95 },
    { 0xe43b77ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadb77b4, 23 },
    { 0xee0b77b4, 43 },
    { 0xfa3b77b5, 33 },
    { 0xf6db77b6, 43 },
    { 0x393b77b8, 61 },
    { 0x684b77b9, 67 },
    { 0x821b77b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ab77be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b77c5, 52 },
    { 0xdb4b77c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44b77ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6b77ce, 98 },
    { 0xe51b77cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91b77d4, 87 },
    { 0x5e2b77d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ab77d8, 76 },
    { 0x43ab77d9, 90 },
    { 0x201b77da, 84 },
    { 0, 0 },
    { 0x234b77dc, 33 },
    { 0x946b77dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94b77e4, 55 },
    { 0x602b77e5, 23 },
    { 0x94fb77e5, 93 },
    { 0x26db77e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720b77ed, 65 },
    { 0xc0cb77ee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb77f2, 95 },
    { 0xedfb77f3, 93 },
    { 0, 0 },
    { 0x7edb77f5, 96 },
    { 0, 0 },
    { 0xe4cb77f7, 33 },
    { 0x98b77f8, 76 },
    { 0x61ab77f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x774b77fc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304b7800, 77 },
    { 0, 0 },
    { 0x413b7802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1db780a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1b781b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958b781f, 99 },
    { 0, 0 },
    { 0xd41b7821, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37db7827, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942b7833, 33 },
    { 0, 0 },
    { 0xbe8b7835, 77 },
    { 0xafeb7835, 23 },
    { 0x3a9b7837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8b783b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4b7843, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4b7847, 33 },
    { 0x9bfb7848, 95 },
    { 0, 0 },
    { 0xb54b784a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x492b784d, 67 },
    { 0x180b784d, 76 },
    { 0, 0 },
    { 0xa93b7850, 23 },
    { 0x3fab7851, 90 },
    { 0x398b7852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bb7859, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eb785d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fb7861, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43b7864, 90 },
    { 0, 0 },
    { 0xa2db7866, 76 },
    { 0, 0 },
    { 0x823b7868, 33 },
    { 0x55db7869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab786d, 52 },
    { 0, 0 },
    { 0xcffb786f, 66 },
    { 0xb47b7870, 34 },
    { 0, 0 },
    { 0xe87b7872, 34 },
    { 0xa0ab7872, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86b7877, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59b787d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7b7881, 76 },
    { 0xbb3b7882, 43 },
    { 0, 0 },
    { 0xbd4b7884, 76 },
    { 0, 0 },
    { 0x4afb7886, 33 },
    { 0, 0 },
    { 0x99ab7888, 67 },
    { 0, 0 },
    { 0x878b788a, 76 },
    { 0, 0 },
    { 0xdcdb788c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1db788f, 23 },
    { 0x1f6b7890, 99 },
    { 0x29bb7891, 76 },
    { 0x78fb7892, 43 },
    { 0xbf3b7891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x114b7896, 33 },
    { 0, 0 },
    { 0xf92b7898, 96 },
    { 0xe64b7898, 90 },
    { 0xf83b789a, 90 },
    { 0xa37b789b, 93 },
    { 0x6db789a, 95 },
    { 0xf7fb789d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fb78a0, 100 },
    { 0xb9cb78a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208b78a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906b78ab, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589b78af, 90 },
    { 0x8a2b78b0, 23 },
    { 0x2bfb78b1, 55 },
    { 0xd54b78b1, 95 },
    { 0x95db78b0, 87 },
    { 0xcf7b78b0, 76 },
    { 0, 0 },
    { 0xba4b78b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8b78ba, 90 },
    { 0, 0 },
    { 0xcab78bc, 90 },
    { 0x1e5b78bc, 90 },
    { 0xb31b78bc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262b78c2, 17 },
    { 0x256b78c3, 23 },
    { 0x322b78c4, 61 },
    { 0xa6ab78c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721b78ca, 95 },
    { 0x66bb78ca, 43 },
    { 0xf25b78cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fb78d0, 46 },
    { 0x31eb78d1, 17 },
    { 0x5d6b78d2, 90 },
    { 0xf44b78d3, 33 },
    { 0x7d3b78d3, 99 },
    { 0, 0 },
    { 0x207b78d6, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0b78de, 43 },
    { 0x6e7b78df, 99 },
    { 0x82bb78e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x338b78e3, 85 },
    { 0x800b78e4, 95 },
    { 0x4dbb78e5, 46 },
    { 0, 0 },
    { 0xcf8b78e7, 34 },
    { 0x3d5b78e8, 55 },
    { 0, 0 },
    { 0xc99b78ea, 93 },
    { 0, 0 },
    { 0x31cb78ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1b78f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1b78f6, 67 },
    { 0xcf2b78f7, 61 },
    { 0x48bb78f7, 95 },
    { 0xb22b78f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abb78fc, 98 },
    { 0x403b78fd, 78 },
    { 0x96eb78fe, 78 },
    { 0x3e7b78fc, 61 },
    { 0xee5b7900, 76 },
    { 0xee9b7901, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebb7904, 90 },
    { 0xcb3b7904, 98 },
    { 0xf13b7906, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5b7909, 61 },
    { 0, 0 },
    { 0x855b790b, 61 },
    { 0xc7cb790c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b7910, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eb7913, 55 },
    { 0x4bdb7913, 48 },
    { 0xa6db7914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eb7918, 34 },
    { 0, 0 },
    { 0xf9b791a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79eb7923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cb7926, 66 },
    { 0x7f9b7927, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0b792a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80b7930, 95 },
    { 0x794b7931, 95 },
    { 0, 0 },
    { 0xdc8b7933, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb7936, 90 },
    { 0x30eb7936, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab793e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cb7945, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb7948, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cb794b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2b794f, 78 },
    { 0x11fb794f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180b7954, 61 },
    { 0, 0 },
    { 0x927b7956, 91 },
    { 0x436b7956, 90 },
    { 0x57fb7957, 98 },
    { 0xb3b7959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03b795d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794b7961, 90 },
    { 0, 0 },
    { 0x5b5b7963, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40eb7967, 46 },
    { 0, 0 },
    { 0x8fbb7969, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4b7970, 34 },
    { 0, 0 },
    { 0x15eb7972, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3b7976, 91 },
    { 0xddb7977, 78 },
    { 0xc46b7978, 67 },
    { 0x138b7979, 61 },
    { 0, 0 },
    { 0xc83b797b, 100 },
    { 0, 0 },
    { 0x747b797d, 77 },
    { 0xfa2b797e, 33 },
    { 0, 0 },
    { 0x7a9b7980, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73b7984, 77 },
    { 0x84db7985, 43 },
    { 0x8a3b7986, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9db798a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b7993, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bb799a, 76 },
    { 0, 0 },
    { 0x585b799c, 93 },
    { 0, 0 },
    { 0x448b799e, 90 },
    { 0xef3b799f, 95 },
    { 0x743b799e, 95 },
    { 0x6d6b79a1, 95 },
    { 0xdacb79a2, 41 },
    { 0xd81b79a3, 98 },
    { 0x342b79a4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7b79aa, 99 },
    { 0x6dfb79ab, 34 },
    { 0x646b79ab, 95 },
    { 0, 0 },
    { 0xd82b79ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddab79b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3b79b9, 33 },
    { 0x594b79b9, 23 },
    { 0xa37b79b9, 34 },
    { 0x2f6b79bc, 76 },
    { 0xbdcb79bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcb79c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdb79c6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126b79d0, 98 },
    { 0xef0b79d1, 80 },
    { 0x569b79d1, 98 },
    { 0x5e3b79d3, 93 },
    { 0x6c1b79d4, 52 },
    { 0xe9b79d4, 90 },
    { 0, 0 },
    { 0xf5cb79d7, 52 },
    { 0xf19b79d7, 61 },
    { 0xa33b79d9, 90 },
    { 0x3d6b79d8, 99 },
    { 0, 0 },
    { 0x63b79dc, 67 },
    { 0xef9b79dd, 23 },
    { 0xd29b79dd, 90 },
    { 0, 0 },
    { 0x88db79e0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5b79e3, 91 },
    { 0x97fb79e4, 83 },
    { 0x131b79e4, 61 },
    { 0x89ab79e4, 61 },
    { 0x1fab79e5, 23 },
    { 0xf3b79e8, 95 },
    { 0x457b79e9, 90 },
    { 0x3a1b79ea, 87 },
    { 0x501b79e6, 98 },
    { 0x5ab79e6, 76 },
    { 0xc4bb79ed, 61 },
    { 0x200b79ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb79f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38b79f6, 95 },
    { 0xf59b79f7, 46 },
    { 0xdf8b79f8, 33 },
    { 0x96b79f9, 23 },
    { 0xff1b79f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22b79fd, 33 },
    { 0xbcab79fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ab7a02, 56 },
    { 0x2eb7a02, 95 },
    { 0x27fb7a04, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6db7a07, 55 },
    { 0, 0 },
    { 0x80cb7a09, 98 },
    { 0, 0 },
    { 0x4b4b7a0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeb7a10, 95 },
    { 0x9f5b7a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6b7a14, 90 },
    { 0xa90b7a15, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4b7a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2b7a29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adb7a2e, 90 },
    { 0, 0 },
    { 0x7eeb7a30, 98 },
    { 0x3f7b7a30, 34 },
    { 0xb84b7a32, 77 },
    { 0xa79b7a32, 43 },
    { 0x653b7a34, 95 },
    { 0xcd2b7a32, 90 },
    { 0, 0 },
    { 0x6ceb7a37, 98 },
    { 0x253b7a38, 33 },
    { 0, 0 },
    { 0x90cb7a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b7a3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7db7a40, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bb7a45, 98 },
    { 0, 0 },
    { 0x955b7a47, 75 },
    { 0, 0 },
    { 0x8b7a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3b7a4d, 67 },
    { 0x279b7a4e, 23 },
    { 0, 0 },
    { 0x548b7a50, 96 },
    { 0, 0 },
    { 0x6b7a52, 100 },
    { 0x6ccb7a53, 66 },
    { 0, 0 },
    { 0x6ab7a55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21b7a5d, 61 },
    { 0x9dfb7a5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab7a62, 95 },
    { 0xa1db7a62, 80 },
    { 0x29b7a64, 90 },
    { 0xa3cb7a65, 98 },
    { 0x408b7a65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccb7a70, 76 },
    { 0, 0 },
    { 0x726b7a72, 98 },
    { 0, 0 },
    { 0x702b7a74, 90 },
    { 0x941b7a74, 33 },
    { 0xb1db7a76, 95 },
    { 0x7b8b7a77, 61 },
    { 0x38db7a78, 55 },
    { 0x61cb7a78, 95 },
    { 0, 0 },
    { 0x283b7a7b, 67 },
    { 0, 0 },
    { 0xb0ab7a7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47b7a81, 34 },
    { 0x553b7a82, 61 },
    { 0x7cfb7a83, 33 },
    { 0, 0 },
    { 0xff1b7a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x927b7a88, 33 },
    { 0, 0 },
    { 0x50b7a8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbb7a94, 23 },
    { 0xc75b7a95, 76 },
    { 0xd19b7a96, 90 },
    { 0x3ecb7a96, 85 },
    { 0x186b7a98, 100 },
    { 0xbdab7a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfb7aa7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670b7aae, 90 },
    { 0x4afb7aae, 95 },
    { 0x1c4b7aaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fb7ab6, 98 },
    { 0x125b7ab7, 61 },
    { 0x36ab7ab7, 98 },
    { 0xfbab7ab7, 90 },
    { 0x343b7ab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b7ac4, 90 },
    { 0x333b7ac4, 90 },
    { 0x731b7ac6, 36 },
    { 0xd68b7ac6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bb7acb, 95 },
    { 0, 0 },
    { 0x846b7acd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832b7ad4, 93 },
    { 0x78b7ad4, 95 },
    { 0x574b7ad6, 49 },
    { 0xf94b7ad7, 42 },
    { 0x852b7ad6, 87 },
    { 0, 0 },
    { 0xe57b7ada, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3b7ae1, 77 },
    { 0xe7bb7ae2, 55 },
    { 0x85fb7ae3, 66 },
    { 0x867b7ae1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fb7aef, 61 },
    { 0x3dcb7aef, 55 },
    { 0xea3b7af1, 17 },
    { 0x7a2b7af2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bb7af5, 100 },
    { 0x733b7af5, 33 },
    { 0xed0b7af7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39b7afd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cb7b00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fb7b03, 99 },
    { 0xd9fb7b04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacb7b07, 66 },
    { 0xec4b7b08, 68 },
    { 0x72bb7b09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1db7b10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883b7b16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689b7b1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b7b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afb7b25, 33 },
    { 0x845b7b25, 90 },
    { 0x22db7b26, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923b7b2c, 95 },
    { 0x561b7b2d, 91 },
    { 0x76b7b2e, 33 },
    { 0x22db7b2f, 46 },
    { 0xab9b7b30, 61 },
    { 0xddeb7b2f, 95 },
    { 0x4aeb7b32, 90 },
    { 0xbb7b31, 23 },
    { 0x5b3b7b34, 90 },
    { 0xc0ab7b34, 43 },
    { 0x716b7b36, 95 },
    { 0x92b7b32, 61 },
    { 0xb1cb7b38, 55 },
    { 0x9c4b7b38, 95 },
    { 0x862b7b3a, 81 },
    { 0x14cb7b3b, 95 },
    { 0xfbbb7b2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ab7b42, 23 },
    { 0x651b7b43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9b7b47, 91 },
    { 0xc4b7b48, 90 },
    { 0x1d6b7b49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38db7b4d, 23 },
    { 0, 0 },
    { 0x66eb7b4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x616b7b52, 78 },
    { 0x344b7b53, 98 },
    { 0, 0 },
    { 0xc89b7b55, 69 },
    { 0, 0 },
    { 0xaf0b7b57, 84 },
    { 0, 0 },
    { 0xda4b7b59, 59 },
    { 0, 0 },
    { 0x5cb7b5b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb7b5e, 78 },
    { 0x262b7b5e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627b7b63, 61 },
    { 0x67fb7b63, 23 },
    { 0xfc8b7b64, 61 },
    { 0xbd1b7b66, 99 },
    { 0xadeb7b67, 95 },
    { 0x5acb7b66, 98 },
    { 0x861b7b69, 87 },
    { 0xa27b7b6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eeb7b71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0b7b77, 43 },
    { 0x673b7b77, 100 },
    { 0, 0 },
    { 0x455b7b7a, 61 },
    { 0xb38b7b7b, 93 },
    { 0, 0 },
    { 0xb79b7b7d, 65 },
    { 0x310b7b7e, 88 },
    { 0xeddb7b7e, 81 },
    { 0xb84b7b7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b7b85, 98 },
    { 0, 0 },
    { 0xa44b7b87, 55 },
    { 0xddab7b87, 52 },
    { 0x5e6b7b88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x218b7b8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x122b7b8f, 95 },
    { 0, 0 },
    { 0x3b9b7b91, 100 },
    { 0x25b7b92, 80 },
    { 0x22cb7b91, 87 },
    { 0x968b7b91, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3b7b9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb7ba3, 46 },
    { 0x1ceb7ba4, 33 },
    { 0x801b7ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edb7ba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68b7bac, 66 },
    { 0xc1b7bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb7bb4, 99 },
    { 0x62cb7bb5, 23 },
    { 0xa41b7bb6, 33 },
    { 0xf6bb7bb6, 81 },
    { 0xff6b7bb6, 61 },
    { 0x142b7bb9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62b7bbc, 33 },
    { 0x8bb7bbd, 95 },
    { 0x37fb7bbe, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fb7bc1, 95 },
    { 0xa6eb7bc2, 23 },
    { 0x328b7bc3, 33 },
    { 0x2d0b7bc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186b7bc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6b7bcd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fb7bd5, 85 },
    { 0xeb2b7bd5, 65 },
    { 0x880b7bd7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ab7bda, 33 },
    { 0, 0 },
    { 0x734b7bdc, 90 },
    { 0x430b7bdd, 23 },
    { 0x2b2b7bde, 55 },
    { 0xe7eb7bde, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cb7be3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32b7bea, 33 },
    { 0, 0 },
    { 0x4e0b7bec, 90 },
    { 0x481b7bec, 76 },
    { 0x39cb7bee, 33 },
    { 0x83db7bec, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67b7bf3, 90 },
    { 0xafab7bf4, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab7bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317b7c01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24b7c07, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x61db7c0a, 51 },
    { 0xfbeb7c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fb7c0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63b7c12, 61 },
    { 0xbf6b7c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5b7c17, 95 },
    { 0xd91b7c18, 90 },
    { 0x66b7c18, 91 },
    { 0x748b7c18, 43 },
    { 0xc07b7c18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566b7c20, 55 },
    { 0xebfb7c21, 95 },
    { 0, 0 },
    { 0xe15b7c23, 34 },
    { 0xa37b7c24, 33 },
    { 0, 0 },
    { 0x13eb7c26, 17 },
    { 0x131b7c27, 95 },
    { 0x49ab7c27, 95 },
    { 0xfb5b7c29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830b7c2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fb7c31, 78 },
    { 0, 0 },
    { 0xf1eb7c33, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452b7c39, 34 },
    { 0, 0 },
    { 0xcd8b7c3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54b7c42, 98 },
    { 0xe63b7c42, 90 },
    { 0, 0 },
    { 0x301b7c45, 98 },
    { 0x17eb7c45, 23 },
    { 0x95bb7c47, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b7c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51b7c4f, 61 },
    { 0x119b7c50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23b7c53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07b7c56, 65 },
    { 0, 0 },
    { 0xeb1b7c58, 95 },
    { 0x32eb7c59, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8b7c5d, 90 },
    { 0x7bfb7c5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bb7c61, 61 },
    { 0x6fcb7c62, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55b7c69, 34 },
    { 0x32b7c6a, 61 },
    { 0x5a7b7c6a, 67 },
    { 0x8a1b7c6c, 95 },
    { 0x9cbb7c6d, 61 },
    { 0xab6b7c6e, 38 },
    { 0x3eb7c6c, 95 },
    { 0xad5b7c70, 87 },
    { 0x638b7c70, 98 },
    { 0xb3bb7c72, 90 },
    { 0x60db7c73, 34 },
    { 0x566b7c73, 33 },
    { 0x9a3b7c6d, 61 },
    { 0x867b7c76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66db7c84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49b7c8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381b7c92, 33 },
    { 0x646b7c93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fb7c97, 33 },
    { 0x9d4b7c98, 90 },
    { 0xb41b7c99, 93 },
    { 0, 0 },
    { 0xf0db7c9b, 78 },
    { 0xb87b7c9b, 88 },
    { 0x88eb7c9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffb7ca4, 98 },
    { 0x547b7ca4, 23 },
    { 0x467b7ca5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfb7ca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdb7cac, 87 },
    { 0xf69b7cad, 61 },
    { 0xc5b7cac, 93 },
    { 0x675b7caf, 33 },
    { 0xd00b7cb0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eb7cba, 61 },
    { 0, 0 },
    { 0xe78b7cbc, 95 },
    { 0, 0 },
    { 0xec8b7cbe, 76 },
    { 0x2c6b7cbf, 90 },
    { 0xef6b7cbe, 23 },
    { 0, 0 },
    { 0x90db7cc2, 33 },
    { 0xffcb7cc2, 84 },
    { 0, 0 },
    { 0xf29b7cc5, 93 },
    { 0x50ab7cc6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ab7cd4, 34 },
    { 0, 0 },
    { 0x92db7cd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cb7cd9, 84 },
    { 0, 0 },
    { 0xa6ab7cdb, 99 },
    { 0x5f9b7cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ab7ce3, 98 },
    { 0x1a9b7ce4, 43 },
    { 0, 0 },
    { 0x7c2b7ce6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2b7cec, 78 },
    { 0, 0 },
    { 0x589b7cee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26b7cf1, 49 },
    { 0xac5b7cf1, 55 },
    { 0x1bfb7cf3, 33 },
    { 0xe29b7cf2, 61 },
    { 0, 0 },
    { 0xfb9b7cf6, 100 },
    { 0, 0 },
    { 0x14cb7cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14eb7d01, 55 },
    { 0xccb7d02, 43 },
    { 0xd1db7d01, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b7d06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3b7d09, 95 },
    { 0, 0 },
    { 0x92eb7d0b, 98 },
    { 0, 0 },
    { 0x81cb7d0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x675b7d10, 33 },
    { 0, 0 },
    { 0x8bfb7d12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cb7d15, 23 },
    { 0x548b7d16, 61 },
    { 0x37b7d17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0b7d1a, 61 },
    { 0xcab7d1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58b7d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6b7d2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2b7d2e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bb7d33, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfb7d36, 97 },
    { 0, 0 },
    { 0x54b7d38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbb7d3b, 33 },
    { 0, 0 },
    { 0xb45b7d3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b7d43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11b7d47, 76 },
    { 0xd2ab7d48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1b7d4b, 61 },
    { 0x827b7d4c, 68 },
    { 0x689b7d4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1b7d51, 61 },
    { 0xfb6b7d52, 77 },
    { 0x496b7d53, 33 },
    { 0xc0b7d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5b7d5a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308b7d5e, 48 },
    { 0x4f4b7d5f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50b7d62, 33 },
    { 0x855b7d63, 23 },
    { 0xc19b7d64, 91 },
    { 0x3d2b7d63, 33 },
    { 0x54cb7d66, 61 },
    { 0, 0 },
    { 0xdc0b7d68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6b7d6d, 76 },
    { 0xb13b7d6e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639b7d78, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccb7d7f, 43 },
    { 0x401b7d80, 34 },
    { 0x597b7d80, 61 },
    { 0, 0 },
    { 0xffcb7d83, 85 },
    { 0, 0 },
    { 0x81b7d85, 68 },
    { 0x92bb7d85, 33 },
    { 0xec8b7d85, 99 },
    { 0x510b7d88, 33 },
    { 0x28ab7d89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4b7d8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b7d92, 90 },
    { 0xa6eb7d93, 95 },
    { 0x38fb7d94, 75 },
    { 0, 0 },
    { 0xc12b7d96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1b7d9f, 90 },
    { 0x65b7da0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x948b7da3, 23 },
    { 0, 0 },
    { 0x749b7da5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818b7da9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb7dac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbb7daf, 34 },
    { 0x607b7db0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ab7db3, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cb7dbe, 95 },
    { 0, 0 },
    { 0xeeeb7dc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4b7dc3, 95 },
    { 0x5cdb7dc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36b7dcd, 76 },
    { 0, 0 },
    { 0x187b7dcf, 98 },
    { 0, 0 },
    { 0x55fb7dd1, 34 },
    { 0, 0 },
    { 0xe6cb7dd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbb7dda, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36b7de2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabb7de6, 61 },
    { 0x517b7de6, 23 },
    { 0xf95b7de8, 76 },
    { 0xaa6b7de8, 90 },
    { 0x831b7de9, 33 },
    { 0xcb7de9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83b7def, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb7df2, 23 },
    { 0x78eb7df2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495b7df8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ab7e01, 90 },
    { 0xb0b7e02, 90 },
    { 0xffab7e01, 17 },
    { 0x6f1b7e04, 93 },
    { 0xf85b7e05, 33 },
    { 0x8eb7e05, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x573b7e09, 90 },
    { 0x3e7b7e09, 99 },
    { 0, 0 },
    { 0xd2cb7e0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b7e10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3b7e13, 33 },
    { 0, 0 },
    { 0x7c5b7e15, 93 },
    { 0, 0 },
    { 0x6fdb7e17, 84 },
    { 0xeacb7e17, 23 },
    { 0x593b7e19, 61 },
    { 0, 0 },
    { 0x382b7e1b, 90 },
    { 0, 0 },
    { 0xf9b7e1d, 99 },
    { 0, 0 },
    { 0x89eb7e1f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfb7e22, 98 },
    { 0x6a6b7e22, 23 },
    { 0xac8b7e24, 100 },
    { 0xce5b7e25, 6 },
    { 0x9d2b7e24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382b7e2b, 61 },
    { 0xb7cb7e2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2b7e2f, 98 },
    { 0, 0 },
    { 0xdd0b7e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9b7e37, 99 },
    { 0x668b7e38, 55 },
    { 0xe3bb7e37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ab7e3c, 90 },
    { 0, 0 },
    { 0xbefb7e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b7e41, 98 },
    { 0, 0 },
    { 0xf4bb7e43, 60 },
    { 0x449b7e44, 90 },
    { 0x54bb7e45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883b7e4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39db7e50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b7e54, 95 },
    { 0, 0 },
    { 0xf93b7e56, 98 },
    { 0x6aeb7e56, 100 },
    { 0xc9bb7e56, 95 },
    { 0x4a7b7e59, 61 },
    { 0x4aeb7e59, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8db7e5e, 23 },
    { 0, 0 },
    { 0xb6eb7e60, 61 },
    { 0, 0 },
    { 0xfc8b7e62, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556b7e67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4b7e73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbb7e79, 90 },
    { 0x4cab7e7a, 68 },
    { 0, 0 },
    { 0xab6b7e7c, 61 },
    { 0, 0 },
    { 0xa9ab7e7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeb7e81, 99 },
    { 0x3cfb7e82, 66 },
    { 0x9a3b7e83, 84 },
    { 0xed8b7e83, 76 },
    { 0x7d6b7e85, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2b7e89, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361b7e8d, 95 },
    { 0xb77b7e8e, 90 },
    { 0x79db7e8e, 90 },
    { 0xa0fb7e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9b7e93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcb7e98, 52 },
    { 0x8c6b7e99, 98 },
    { 0x307b7e9a, 98 },
    { 0x711b7e9a, 55 },
    { 0x51db7e99, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ab7ea1, 61 },
    { 0x5c9b7ea2, 91 },
    { 0x961b7ea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacb7ea6, 81 },
    { 0x758b7ea6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eb7eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272b7eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941b7ebc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x112b7ebf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b7ec6, 87 },
    { 0x963b7ec7, 33 },
    { 0xa2bb7ec8, 52 },
    { 0, 0 },
    { 0x930b7eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6db7ecd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465b7ed1, 90 },
    { 0, 0 },
    { 0x71ab7ed3, 87 },
    { 0x292b7ed4, 34 },
    { 0, 0 },
    { 0x2c1b7ed6, 93 },
    { 0xad4b7ed6, 46 },
    { 0xcb4b7ed6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4b7edc, 90 },
    { 0xbfb7edd, 91 },
    { 0x45fb7edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2b7ee4, 43 },
    { 0xd0fb7ee5, 98 },
    { 0xc9db7ee5, 81 },
    { 0x9e4b7ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bb7eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b7eee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86eb7ef8, 90 },
    { 0, 0 },
    { 0x2d1b7efa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fb7f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cb7f08, 76 },
    { 0x7a3b7f09, 52 },
    { 0x1d2b7f0a, 98 },
    { 0x943b7f08, 90 },
    { 0, 0 },
    { 0x7e4b7f0d, 95 },
    { 0xbc2b7f0e, 85 },
    { 0x41fb7f0e, 61 },
    { 0, 0 },
    { 0x8f9b7f11, 93 },
    { 0xbebb7f11, 91 },
    { 0, 0 },
    { 0xa91b7f14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107b7f18, 81 },
    { 0, 0 },
    { 0x2b1b7f1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddb7f1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286b7f22, 96 },
    { 0xb2fb7f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ab7f26, 91 },
    { 0, 0 },
    { 0xa81b7f28, 99 },
    { 0xd84b7f29, 33 },
    { 0, 0 },
    { 0xd0bb7f2b, 81 },
    { 0xdf4b7f2b, 95 },
    { 0x40fb7f2d, 66 },
    { 0, 0 },
    { 0xb40b7f2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784b7f34, 67 },
    { 0x99db7f35, 90 },
    { 0, 0 },
    { 0xd22b7f37, 33 },
    { 0xaf5b7f37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bb7f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9db7f3f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ab7f4d, 34 },
    { 0x90fb7f4e, 81 },
    { 0xf4bb7f4e, 87 },
    { 0, 0 },
    { 0xbdbb7f51, 90 },
    { 0, 0 },
    { 0x7a2b7f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1b7f56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7b7f5b, 61 },
    { 0xecfb7f5b, 98 },
    { 0xa2bb7f5d, 77 },
    { 0xc90b7f5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546b7f64, 43 },
    { 0xb38b7f65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bb7f68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ab7f6c, 53 },
    { 0xa2cb7f6d, 95 },
    { 0x5a3b7f6e, 33 },
    { 0xda6b7f6f, 55 },
    { 0x280b7f6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fb7f79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb7f7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59b7f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cb7f87, 98 },
    { 0x413b7f88, 91 },
    { 0xa9db7f89, 36 },
    { 0x342b7f8a, 61 },
    { 0, 0 },
    { 0x74b7f8c, 90 },
    { 0xeefb7f8d, 55 },
    { 0xe72b7f8c, 23 },
    { 0, 0 },
    { 0xeb5b7f90, 34 },
    { 0xc9b7f91, 98 },
    { 0x844b7f91, 78 },
    { 0x675b7f90, 93 },
    { 0x729b7f90, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cb7f9d, 65 },
    { 0x9bb7f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40b7fa3, 33 },
    { 0, 0 },
    { 0xe32b7fa5, 93 },
    { 0, 0 },
    { 0xb95b7fa7, 98 },
    { 0x9fdb7fa7, 90 },
    { 0x312b7fa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6b7fb0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cb7fb3, 85 },
    { 0xc1eb7fb4, 23 },
    { 0x48b7fb5, 90 },
    { 0, 0 },
    { 0xa98b7fb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314b7fbb, 23 },
    { 0, 0 },
    { 0x35cb7fbd, 61 },
    { 0, 0 },
    { 0xe94b7fbf, 56 },
    { 0xfa0b7fbf, 33 },
    { 0x5db7fbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121b7fc5, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138b7fc9, 87 },
    { 0xf48b7fc9, 95 },
    { 0x545b7fcb, 43 },
    { 0x8dab7fcc, 55 },
    { 0x1bfb7fcd, 90 },
    { 0, 0 },
    { 0x9b1b7fcf, 66 },
    { 0xd2fb7fd0, 90 },
    { 0x959b7fd1, 38 },
    { 0xe90b7fd2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287b7fd7, 61 },
    { 0x824b7fd7, 98 },
    { 0, 0 },
    { 0xf32b7fda, 23 },
    { 0, 0 },
    { 0x277b7fdc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868b7fe6, 90 },
    { 0x616b7fe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05b7fee, 75 },
    { 0xe1b7fef, 90 },
    { 0x9f1b7ff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6b7ff5, 95 },
    { 0xdd1b7ff6, 33 },
    { 0x950b7ff7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ab7ffa, 95 },
    { 0, 0 },
    { 0x5fab7ffc, 33 },
    { 0xf12b7ffd, 30 },
    { 0x5bcb7ffe, 91 },
    { 0xbdab7fff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0b8005, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3b8009, 76 },
    { 0, 0 },
    { 0x549b800b, 34 },
    { 0x91b800c, 95 },
    { 0x50bb800d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b8012, 78 },
    { 0, 0 },
    { 0xa74b8014, 90 },
    { 0x139b8015, 61 },
    { 0xfc2b8016, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b8019, 33 },
    { 0, 0 },
    { 0x1e5b801b, 23 },
    { 0xfc0b801c, 95 },
    { 0x96cb801d, 93 },
    { 0xc91b801e, 33 },
    { 0xef0b801c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7b8022, 23 },
    { 0xdb2b8023, 33 },
    { 0x858b8023, 33 },
    { 0, 0 },
    { 0x698b8026, 34 },
    { 0, 0 },
    { 0x35ab8028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3b802e, 34 },
    { 0, 0 },
    { 0x381b8030, 68 },
    { 0xb51b8031, 90 },
    { 0x1c1b8031, 90 },
    { 0x408b8033, 93 },
    { 0x2a2b8034, 95 },
    { 0x2abb8034, 34 },
    { 0x6d4b8036, 98 },
    { 0x1e8b8036, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572b803c, 75 },
    { 0x1ceb803d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b8041, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x582b8044, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97b8047, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db804c, 98 },
    { 0, 0 },
    { 0x87cb804e, 76 },
    { 0x35b804e, 95 },
    { 0, 0 },
    { 0x85ab8051, 33 },
    { 0, 0 },
    { 0x5a6b8053, 90 },
    { 0, 0 },
    { 0xb4fb8055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abb8058, 33 },
    { 0x13eb8059, 23 },
    { 0, 0 },
    { 0x899b805b, 95 },
    { 0x3a9b805b, 61 },
    { 0, 0 },
    { 0x36b805e, 90 },
    { 0x4e9b805f, 99 },
    { 0x3b5b8060, 90 },
    { 0xa73b805f, 43 },
    { 0, 0 },
    { 0xaacb8063, 33 },
    { 0, 0 },
    { 0x572b8065, 61 },
    { 0x39fb8066, 95 },
    { 0x490b8065, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b806b, 93 },
    { 0xba9b806c, 98 },
    { 0x3ecb806c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdb807b, 90 },
    { 0x291b807c, 33 },
    { 0, 0 },
    { 0xb2b807e, 91 },
    { 0xf24b807f, 91 },
    { 0xa4eb8080, 55 },
    { 0, 0 },
    { 0x3ab8082, 66 },
    { 0xcc7b8082, 91 },
    { 0x643b8084, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128b8088, 87 },
    { 0, 0 },
    { 0xd03b808a, 23 },
    { 0, 0 },
    { 0xd4ab808c, 88 },
    { 0x49ab808c, 23 },
    { 0xafeb808e, 98 },
    { 0x7f2b808f, 66 },
    { 0x24bb8090, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91b8093, 23 },
    { 0xdabb8094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6b8097, 33 },
    { 0x88ab8097, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb809c, 43 },
    { 0x578b809d, 95 },
    { 0x31db809e, 93 },
    { 0x24cb809e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7b80a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6b80a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dab80a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1b80ad, 84 },
    { 0x7f7b80ad, 33 },
    { 0xcd6b80ae, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ab80b4, 81 },
    { 0x8bcb80b5, 23 },
    { 0, 0 },
    { 0xc30b80b7, 27 },
    { 0, 0 },
    { 0x50eb80b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bb80c4, 90 },
    { 0, 0 },
    { 0x7ecb80c6, 23 },
    { 0, 0 },
    { 0x6c0b80c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14b80cc, 90 },
    { 0, 0 },
    { 0xa20b80ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eb80d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58b80d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceb80da, 93 },
    { 0, 0 },
    { 0xa6bb80dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403b80e0, 82 },
    { 0x95db80e1, 40 },
    { 0, 0 },
    { 0xaa3b80e3, 33 },
    { 0, 0 },
    { 0x80cb80e5, 66 },
    { 0x840b80e6, 34 },
    { 0, 0 },
    { 0x2e5b80e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ab80ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccb80f0, 90 },
    { 0, 0 },
    { 0x30db80f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09b80f5, 38 },
    { 0xf3ab80f6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdb80fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4b8103, 61 },
    { 0x386b8104, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ab8107, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedb810a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x178b810d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cb8111, 90 },
    { 0x21fb8112, 34 },
    { 0, 0 },
    { 0x1b4b8114, 23 },
    { 0, 0 },
    { 0x671b8116, 61 },
    { 0x47bb8116, 95 },
    { 0xf78b8118, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceb811b, 36 },
    { 0, 0 },
    { 0x5bfb811d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcab8124, 33 },
    { 0, 0 },
    { 0x676b8126, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557b812a, 90 },
    { 0xdbdb812b, 66 },
    { 0x817b812a, 34 },
    { 0xde6b812a, 23 },
    { 0xc87b812e, 90 },
    { 0xfedb812e, 61 },
    { 0x8a2b8130, 33 },
    { 0xe98b8131, 98 },
    { 0xc5ab8130, 84 },
    { 0, 0 },
    { 0x41ab8134, 55 },
    { 0, 0 },
    { 0xa8cb8136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bb813a, 90 },
    { 0x4aab813b, 33 },
    { 0xe2eb813c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8b813f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb8144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953b8148, 66 },
    { 0xa24b8149, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cb814d, 52 },
    { 0, 0 },
    { 0xfd8b814f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebb8152, 91 },
    { 0x4d4b8153, 33 },
    { 0x57bb8152, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b815a, 61 },
    { 0xd93b815b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38b8162, 33 },
    { 0x614b8162, 90 },
    { 0x402b8164, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b8167, 76 },
    { 0xd6fb8168, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234b816c, 61 },
    { 0x53cb816c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5b8172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeb8177, 90 },
    { 0x8bbb8177, 33 },
    { 0, 0 },
    { 0x6f9b817a, 84 },
    { 0xfa0b817a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3b8180, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64b8184, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbb8188, 98 },
    { 0x223b8189, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2b818e, 90 },
    { 0, 0 },
    { 0xa3bb8190, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fb8193, 61 },
    { 0x199b8194, 34 },
    { 0, 0 },
    { 0xf7db8196, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb8199, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2b819f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x755b81a2, 61 },
    { 0x350b81a2, 17 },
    { 0xd9b81a4, 23 },
    { 0x294b81a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25db81ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x424b81af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52b81b3, 90 },
    { 0x23eb81b4, 23 },
    { 0xce8b81b5, 68 },
    { 0x5e4b81b5, 95 },
    { 0x42bb81b4, 61 },
    { 0x343b81b8, 23 },
    { 0xed6b81b5, 76 },
    { 0x883b81ba, 61 },
    { 0x1aeb81b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0b81c0, 91 },
    { 0x818b81c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dab81c4, 66 },
    { 0, 0 },
    { 0x186b81c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ab81cb, 98 },
    { 0x687b81cb, 17 },
    { 0x1f5b81cb, 67 },
    { 0, 0 },
    { 0xccab81cf, 95 },
    { 0x94b81cf, 76 },
    { 0x763b81d1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeb81db, 91 },
    { 0x2a8b81dc, 95 },
    { 0x62eb81dd, 61 },
    { 0x214b81de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16db81e4, 33 },
    { 0x88fb81e5, 90 },
    { 0x92db81e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcb81ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eb81fb, 100 },
    { 0x666b81fb, 43 },
    { 0xdab81fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40b8202, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94db8206, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523b820c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x247b820f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbb821b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93b821e, 95 },
    { 0x6a5b821f, 46 },
    { 0, 0 },
    { 0xcbdb8221, 93 },
    { 0x8e5b8221, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cb8225, 90 },
    { 0xe07b8226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6b822b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deb822e, 90 },
    { 0xeddb822e, 85 },
    { 0, 0 },
    { 0xa7bb8231, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbb8234, 33 },
    { 0xd79b8235, 23 },
    { 0xdfdb8236, 52 },
    { 0x906b8236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9b8240, 43 },
    { 0x1c5b8240, 97 },
    { 0x5eb8242, 95 },
    { 0xfc0b8242, 55 },
    { 0xb1eb8244, 64 },
    { 0, 0 },
    { 0xd6cb8246, 33 },
    { 0xd45b8246, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eb824c, 98 },
    { 0x30eb824c, 95 },
    { 0, 0 },
    { 0x500b824f, 34 },
    { 0xf63b8250, 90 },
    { 0xc44b824f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439b8256, 98 },
    { 0x5fb8257, 77 },
    { 0, 0 },
    { 0x8eab8259, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3b825c, 43 },
    { 0xde9b825d, 55 },
    { 0x7e1b825e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48b8262, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19b826a, 23 },
    { 0xf94b826b, 68 },
    { 0x567b826c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613b8270, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eb8274, 84 },
    { 0xfe2b8275, 46 },
    { 0x7a3b8276, 68 },
    { 0x5c1b8276, 95 },
    { 0xbd8b8278, 98 },
    { 0x517b8277, 99 },
    { 0, 0 },
    { 0x66fb827b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb827e, 43 },
    { 0, 0 },
    { 0xf74b8280, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373b8284, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903b8288, 78 },
    { 0xf24b8288, 61 },
    { 0, 0 },
    { 0x773b828b, 76 },
    { 0x846b828b, 90 },
    { 0x20eb828c, 99 },
    { 0x93ab828c, 61 },
    { 0xb20b828c, 33 },
    { 0x3e8b8290, 99 },
    { 0x4bbb8291, 76 },
    { 0x603b8292, 90 },
    { 0x702b8293, 75 },
    { 0x4b4b828c, 76 },
    { 0x9d2b828f, 23 },
    { 0, 0 },
    { 0xa50b8297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11b829b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87b82a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1b82ac, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689b82b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8b82b9, 95 },
    { 0, 0 },
    { 0xba2b82bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b82be, 23 },
    { 0xd8eb82bf, 34 },
    { 0xb9bb82c0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01b82c3, 46 },
    { 0x6ecb82c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0b82ce, 91 },
    { 0, 0 },
    { 0x9e6b82d0, 52 },
    { 0, 0 },
    { 0xab7b82d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b82d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69b82db, 90 },
    { 0xe07b82dc, 61 },
    { 0x887b82dd, 98 },
    { 0, 0 },
    { 0xdefb82df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x385b82e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bb82e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474b82e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceb82f0, 33 },
    { 0x302b82f0, 34 },
    { 0x69b82f2, 90 },
    { 0, 0 },
    { 0x63bb82f4, 52 },
    { 0x83fb82f5, 90 },
    { 0x6ddb82f6, 61 },
    { 0x3b8b82f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84b82fe, 95 },
    { 0x88b82fe, 76 },
    { 0, 0 },
    { 0xb42b8301, 90 },
    { 0x3e4b8301, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bb8309, 46 },
    { 0, 0 },
    { 0x306b830b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467b8310, 95 },
    { 0xb8cb8311, 23 },
    { 0xb0eb8310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb8316, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efb831f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2b832d, 95 },
    { 0xab6b832e, 34 },
    { 0, 0 },
    { 0xa4db8330, 90 },
    { 0x274b8331, 34 },
    { 0, 0 },
    { 0x5a2b8333, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6b8338, 81 },
    { 0x5e6b8339, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5b833c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8b8342, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb834b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ab835a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fab835d, 23 },
    { 0, 0 },
    { 0xbb9b835f, 91 },
    { 0x2c9b8360, 23 },
    { 0x21db835f, 95 },
    { 0, 0 },
    { 0xa53b8363, 23 },
    { 0x61cb8364, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eb8368, 76 },
    { 0x2c9b8368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62b836e, 61 },
    { 0xdf2b836f, 76 },
    { 0, 0 },
    { 0x75bb8371, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcb8375, 33 },
    { 0x509b8376, 76 },
    { 0x9db8377, 90 },
    { 0, 0 },
    { 0xbc6b8379, 76 },
    { 0x3cbb837a, 33 },
    { 0x877b837a, 100 },
    { 0x8c2b837a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x257b837f, 90 },
    { 0x241b8380, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2b8384, 55 },
    { 0xc22b8385, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcb838c, 87 },
    { 0xd8fb838d, 74 },
    { 0x315b838c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451b8394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35b839a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910b839e, 90 },
    { 0xb1cb839f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40b83a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4b83a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01b83aa, 34 },
    { 0x85fb83ab, 87 },
    { 0x5ddb83ab, 95 },
    { 0x264b83ad, 90 },
    { 0x52ab83ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1b83b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cb83b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb83b9, 43 },
    { 0x902b83ba, 76 },
    { 0xf8ab83ba, 100 },
    { 0xb5ab83bb, 33 },
    { 0, 0 },
    { 0x98bb83be, 84 },
    { 0, 0 },
    { 0xe61b83c0, 56 },
    { 0x78bb83c0, 23 },
    { 0xd10b83c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33b83c9, 99 },
    { 0x87cb83ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656b83ce, 36 },
    { 0x2d4b83ce, 65 },
    { 0xd21b83d0, 66 },
    { 0, 0 },
    { 0xe0bb83d2, 96 },
    { 0x80eb83d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56b83d7, 59 },
    { 0xcdab83d8, 33 },
    { 0x6edb83d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadb83dd, 77 },
    { 0xdb0b83dd, 66 },
    { 0x18ab83df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9b83e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133b83e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cb83eb, 81 },
    { 0, 0 },
    { 0x2b9b83ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ab83f1, 33 },
    { 0xe52b83f1, 43 },
    { 0x901b83f3, 91 },
    { 0, 0 },
    { 0xdcbb83f5, 99 },
    { 0x999b83f6, 46 },
    { 0x57db83f6, 34 },
    { 0x975b83f8, 61 },
    { 0, 0 },
    { 0x2f1b83fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0b83ff, 80 },
    { 0x3e8b8400, 99 },
    { 0x1c2b83ff, 43 },
    { 0x659b8402, 98 },
    { 0xeeab8400, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b8408, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922b840d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcb8411, 56 },
    { 0x8cb8412, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5b8417, 34 },
    { 0xeb3b8418, 52 },
    { 0x6b0b8418, 95 },
    { 0xc69b841a, 61 },
    { 0x1b6b8419, 99 },
    { 0xb80b841b, 33 },
    { 0xe31b841b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65b8423, 46 },
    { 0xd96b8424, 61 },
    { 0x60db8423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdb842a, 90 },
    { 0x775b842b, 90 },
    { 0xafeb842b, 95 },
    { 0x31cb842c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eb8432, 90 },
    { 0, 0 },
    { 0xb7db8434, 90 },
    { 0, 0 },
    { 0x9d2b8436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3b843c, 95 },
    { 0x934b843d, 90 },
    { 0x4b4b843c, 67 },
    { 0xeaeb843f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3b8442, 90 },
    { 0, 0 },
    { 0x258b8444, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691b8448, 76 },
    { 0x222b8449, 98 },
    { 0xb9bb844a, 90 },
    { 0xcfeb844b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812b8451, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b8454, 98 },
    { 0, 0 },
    { 0x814b8456, 66 },
    { 0x2b3b8457, 61 },
    { 0x36eb8458, 72 },
    { 0x9f2b8459, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeb845c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bab845f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x764b8462, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11b8465, 91 },
    { 0x40eb8465, 89 },
    { 0x967b8467, 96 },
    { 0xed6b8468, 95 },
    { 0xabfb8465, 87 },
    { 0xd66b8468, 61 },
    { 0x73bb8468, 23 },
    { 0, 0 },
    { 0xec7b846d, 33 },
    { 0x9e2b846e, 44 },
    { 0x960b846f, 67 },
    { 0xf05b8470, 76 },
    { 0, 0 },
    { 0xbfdb8472, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ab8477, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b847a, 77 },
    { 0x95bb847b, 80 },
    { 0, 0 },
    { 0xa36b847d, 76 },
    { 0x9fb847d, 33 },
    { 0x968b847f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52b8487, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b848c, 95 },
    { 0x16eb848d, 90 },
    { 0xfebb848d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79db8491, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x917b8494, 98 },
    { 0xc0fb8494, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x237b8498, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5b849e, 84 },
    { 0x956b849e, 61 },
    { 0x934b849e, 93 },
    { 0, 0 },
    { 0x46fb84a2, 95 },
    { 0x70fb84a3, 46 },
    { 0, 0 },
    { 0xbd8b84a5, 90 },
    { 0xd81b84a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9eb84aa, 98 },
    { 0xebb84aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x358b84ae, 98 },
    { 0xd6db84af, 90 },
    { 0x7a1b84ae, 61 },
    { 0xec9b84b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb84b6, 33 },
    { 0xf4eb84b6, 95 },
    { 0xa7bb84b8, 98 },
    { 0x3dbb84b9, 95 },
    { 0, 0 },
    { 0xb7db84bb, 85 },
    { 0xbefb84bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeab84c1, 34 },
    { 0xb6bb84c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bb84ca, 99 },
    { 0xac1b84cb, 9 },
    { 0, 0 },
    { 0xd82b84cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b84d0, 100 },
    { 0x179b84d1, 95 },
    { 0x9a1b84d1, 98 },
    { 0xac8b84d3, 76 },
    { 0, 0 },
    { 0x5c7b84d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ab84dd, 55 },
    { 0x73eb84de, 43 },
    { 0, 0 },
    { 0x3efb84e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb84e8, 95 },
    { 0, 0 },
    { 0xd31b84ea, 90 },
    { 0x837b84ea, 43 },
    { 0, 0 },
    { 0x8b9b84ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41b84f0, 75 },
    { 0, 0 },
    { 0x42db84f2, 75 },
    { 0, 0 },
    { 0xe12b84f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413b84fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77b84fe, 91 },
    { 0xac3b84ff, 92 },
    { 0xcccb8500, 98 },
    { 0x630b8500, 81 },
    { 0xcc4b84fe, 34 },
    { 0xd6ab84fe, 46 },
    { 0x902b8504, 91 },
    { 0x8acb8505, 80 },
    { 0xb57b8506, 46 },
    { 0, 0 },
    { 0x3beb8508, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cb850e, 61 },
    { 0xe66b850f, 23 },
    { 0xbf5b850e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ab8514, 98 },
    { 0x725b8515, 98 },
    { 0x2cfb8516, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x933b8519, 90 },
    { 0, 0 },
    { 0xc1bb851b, 23 },
    { 0x929b851c, 88 },
    { 0x8b8b851c, 76 },
    { 0x9eeb851e, 61 },
    { 0x67eb851e, 98 },
    { 0xb55b8520, 61 },
    { 0x974b8521, 33 },
    { 0x788b8521, 34 },
    { 0xdabb8520, 61 },
    { 0x73eb8524, 34 },
    { 0xc4fb8525, 99 },
    { 0x451b8526, 95 },
    { 0, 0 },
    { 0xafcb8528, 90 },
    { 0x18eb8529, 98 },
    { 0, 0 },
    { 0x9eb852b, 90 },
    { 0x961b852c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5b8530, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3b8534, 95 },
    { 0x614b8535, 33 },
    { 0xc77b8536, 61 },
    { 0, 0 },
    { 0x4a6b8538, 95 },
    { 0xba4b8538, 55 },
    { 0, 0 },
    { 0xe56b853b, 61 },
    { 0, 0 },
    { 0x239b853d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cb8541, 85 },
    { 0x3b4b8542, 84 },
    { 0, 0 },
    { 0x892b8544, 95 },
    { 0xbd7b8545, 55 },
    { 0xfc5b8544, 98 },
    { 0xa5eb8547, 98 },
    { 0xa29b8548, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cb854c, 43 },
    { 0x508b854c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b8554, 76 },
    { 0x2db8554, 61 },
    { 0, 0 },
    { 0x326b8557, 88 },
    { 0x691b8557, 90 },
    { 0, 0 },
    { 0xc63b855a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7b8565, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448b856e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b8573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323b857c, 33 },
    { 0, 0 },
    { 0xb44b857e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317b8582, 67 },
    { 0x352b8583, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eb8589, 61 },
    { 0xb1fb8589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ab8591, 61 },
    { 0x6adb8592, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08b8597, 90 },
    { 0, 0 },
    { 0x977b8599, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ab859d, 76 },
    { 0x57fb859e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b85a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327b85ad, 76 },
    { 0, 0 },
    { 0x87db85af, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742b85b3, 98 },
    { 0x25b85b4, 23 },
    { 0xadb85b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedb85bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x412b85be, 33 },
    { 0, 0 },
    { 0xed0b85c0, 99 },
    { 0x8b1b85c1, 98 },
    { 0x618b85c2, 98 },
    { 0xb4ab85c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cb85c8, 76 },
    { 0, 0 },
    { 0x539b85ca, 88 },
    { 0, 0 },
    { 0xdaeb85cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451b85d7, 95 },
    { 0xa37b85d8, 33 },
    { 0x64eb85d7, 90 },
    { 0, 0 },
    { 0x3c8b85db, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b85de, 33 },
    { 0, 0 },
    { 0xd75b85e0, 76 },
    { 0, 0 },
    { 0xf2eb85e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560b85ea, 23 },
    { 0x412b85eb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49b85ee, 76 },
    { 0x7e8b85ee, 95 },
    { 0x19b85ee, 95 },
    { 0x597b85f1, 61 },
    { 0x1c9b85f1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9b85f5, 33 },
    { 0, 0 },
    { 0x93db85f7, 95 },
    { 0x2fb85f8, 33 },
    { 0xa56b85f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb85fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb8602, 100 },
    { 0, 0 },
    { 0xea0b8604, 33 },
    { 0xa63b8605, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0b860b, 90 },
    { 0xd60b860c, 61 },
    { 0x91b860d, 99 },
    { 0, 0 },
    { 0xc09b860f, 46 },
    { 0x513b8610, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bb8613, 90 },
    { 0, 0 },
    { 0xf4cb8615, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0b861a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58b861e, 61 },
    { 0x781b861e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3b8626, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99b862a, 98 },
    { 0x2d3b862b, 78 },
    { 0xdfeb862c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9b8633, 99 },
    { 0xc05b8633, 87 },
    { 0x804b8635, 33 },
    { 0x402b8635, 100 },
    { 0x2c6b8636, 90 },
    { 0x499b8633, 90 },
    { 0x1ceb8639, 65 },
    { 0xa62b863a, 95 },
    { 0xb46b8637, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667b8640, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21b8644, 98 },
    { 0x6afb8644, 61 },
    { 0xedbb8646, 20 },
    { 0xb2bb8646, 23 },
    { 0xe76b8647, 99 },
    { 0xb09b8649, 61 },
    { 0x44eb864a, 67 },
    { 0x89db864b, 90 },
    { 0xda1b864c, 99 },
    { 0xb6cb864d, 92 },
    { 0x84bb864e, 90 },
    { 0xb99b864f, 92 },
    { 0xbabb8650, 67 },
    { 0x288b8650, 93 },
    { 0xe1eb8652, 33 },
    { 0x2f7b8653, 88 },
    { 0xdab8654, 88 },
    { 0xf4b8652, 55 },
    { 0x6c2b8656, 34 },
    { 0x7e4b8657, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7db865c, 44 },
    { 0x172b865d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eb866b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1b866f, 61 },
    { 0, 0 },
    { 0x54b8671, 81 },
    { 0, 0 },
    { 0x64bb8673, 75 },
    { 0x1eab8673, 98 },
    { 0x924b8674, 99 },
    { 0xa5eb8675, 90 },
    { 0, 0 },
    { 0xb6cb8678, 100 },
    { 0x79bb8678, 61 },
    { 0x9d3b8678, 95 },
    { 0x451b867b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71b867f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bb8685, 99 },
    { 0x13b8686, 99 },
    { 0, 0 },
    { 0xa94b8688, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb868b, 33 },
    { 0x18bb868c, 90 },
    { 0xc19b868d, 95 },
    { 0x9deb868d, 33 },
    { 0xb96b868f, 88 },
    { 0xc3b868f, 23 },
    { 0x690b868e, 61 },
    { 0xcfab8692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdb8697, 77 },
    { 0xd9cb8697, 55 },
    { 0x1fb8698, 98 },
    { 0x992b8699, 33 },
    { 0xecdb8697, 90 },
    { 0xd6db869c, 55 },
    { 0x17b8698, 78 },
    { 0, 0 },
    { 0xbb3b869f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83b86a2, 90 },
    { 0x955b86a3, 84 },
    { 0x479b86a2, 90 },
    { 0x477b86a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fab86a9, 77 },
    { 0, 0 },
    { 0xf16b86ab, 90 },
    { 0xc82b86ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2b86b3, 61 },
    { 0x7bfb86b3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bb86b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175b86bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cb86c2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x502b86c5, 33 },
    { 0x5fcb86c6, 34 },
    { 0x189b86c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ab86cc, 23 },
    { 0x45cb86cd, 99 },
    { 0x3b0b86cd, 34 },
    { 0xedb86cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bb86d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6b86d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30b86dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7b86df, 23 },
    { 0xdb3b86df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580b86e4, 98 },
    { 0, 0 },
    { 0x393b86e6, 76 },
    { 0x71bb86e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x751b86ea, 90 },
    { 0x3efb86ea, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0b86ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30b86f3, 23 },
    { 0x500b86f3, 95 },
    { 0x7ceb86f5, 61 },
    { 0x243b86f6, 33 },
    { 0x926b86f7, 100 },
    { 0x68db86f7, 61 },
    { 0, 0 },
    { 0xad6b86fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbb8701, 42 },
    { 0x9fcb8701, 77 },
    { 0x2c9b8701, 33 },
    { 0x951b8704, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b8708, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73b8717, 90 },
    { 0xfaeb8718, 33 },
    { 0, 0 },
    { 0x917b871a, 91 },
    { 0x235b871b, 55 },
    { 0, 0 },
    { 0xcd5b871d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbb8722, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4b872a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x905b872d, 52 },
    { 0x906b872e, 17 },
    { 0xe02b872f, 98 },
    { 0x62bb8730, 33 },
    { 0x5a5b872f, 33 },
    { 0xb11b8731, 34 },
    { 0x2a7b8730, 90 },
    { 0, 0 },
    { 0xdf7b8735, 65 },
    { 0x5b8735, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x335b8739, 41 },
    { 0x95eb873a, 33 },
    { 0x32db873b, 76 },
    { 0x4d0b873c, 23 },
    { 0, 0 },
    { 0x5eb873e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08b8741, 81 },
    { 0x394b8741, 81 },
    { 0xd96b8741, 100 },
    { 0xd3eb8744, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52eb874e, 78 },
    { 0x3c5b874f, 95 },
    { 0x685b8750, 33 },
    { 0, 0 },
    { 0x798b8752, 33 },
    { 0, 0 },
    { 0x865b8754, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12b875c, 90 },
    { 0x38bb875c, 33 },
    { 0, 0 },
    { 0x187b875f, 66 },
    { 0x467b8760, 88 },
    { 0xfbeb8761, 46 },
    { 0xb8cb8762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294b876c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffb8770, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecb8775, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1b8778, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeb877c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cb877f, 61 },
    { 0xc59b8780, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab8785, 65 },
    { 0x983b8786, 90 },
    { 0x8d3b8787, 95 },
    { 0x27db8785, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398b878d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bb8793, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x911b8796, 90 },
    { 0xf8eb8796, 34 },
    { 0x6d1b8798, 95 },
    { 0x1eab8799, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24b879d, 33 },
    { 0x3fbb879e, 55 },
    { 0xc10b879f, 23 },
    { 0x6fb879e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7b87a4, 90 },
    { 0x757b87a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2b87a8, 33 },
    { 0x63fb87a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccb87ac, 77 },
    { 0, 0 },
    { 0xae0b87ae, 66 },
    { 0xd3eb87ae, 23 },
    { 0x346b87b0, 34 },
    { 0xa49b87b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36b87b6, 23 },
    { 0xf4cb87b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1b87bc, 21 },
    { 0xd14b87bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdb87c1, 66 },
    { 0, 0 },
    { 0x4dbb87c3, 33 },
    { 0x47b87c3, 33 },
    { 0, 0 },
    { 0x63db87c6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15b87ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x163b87d1, 81 },
    { 0x896b87d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0b87d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282b87db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0b87de, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eb87e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1b87e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x628b87e9, 52 },
    { 0x146b87e9, 76 },
    { 0xa23b87e9, 90 },
    { 0xc41b87ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffb87f0, 55 },
    { 0xb8bb87f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bb87f6, 61 },
    { 0xbe0b87f7, 61 },
    { 0x6deb87f8, 23 },
    { 0x36cb87f9, 61 },
    { 0, 0 },
    { 0xc97b87fb, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x163b87fe, 93 },
    { 0xaf7b87ff, 93 },
    { 0x8e7b8800, 93 },
    { 0xafcb8801, 38 },
    { 0x313b8801, 61 },
    { 0, 0 },
    { 0xc88b8804, 98 },
    { 0x78eb8805, 55 },
    { 0x2dfb8804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83b880a, 99 },
    { 0xd14b880b, 46 },
    { 0x8adb880c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574b8810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3b8814, 90 },
    { 0x206b8815, 90 },
    { 0x1cb8816, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fb881a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7b881d, 23 },
    { 0xa9cb881e, 90 },
    { 0x6cbb881d, 4 },
    { 0x40b8820, 67 },
    { 0x2eeb8821, 100 },
    { 0, 0 },
    { 0x86db8823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5b8826, 61 },
    { 0x3c4b8826, 55 },
    { 0x943b8828, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b882b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddb882f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791b8837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db883b, 23 },
    { 0, 0 },
    { 0x427b883d, 95 },
    { 0xd54b883d, 23 },
    { 0xe4cb883e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cb8842, 98 },
    { 0, 0 },
    { 0xeb8b8844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1eb8847, 61 },
    { 0, 0 },
    { 0xb2ab8849, 94 },
    { 0, 0 },
    { 0xb4db884b, 55 },
    { 0xbc9b884c, 33 },
    { 0x7b3b884d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdb8850, 36 },
    { 0x131b8851, 90 },
    { 0, 0 },
    { 0xb2cb8853, 34 },
    { 0xe2b8854, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb8858, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beb885f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ab8864, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166b8868, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eb886b, 23 },
    { 0, 0 },
    { 0x8fdb886d, 61 },
    { 0x9b8b886e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3b8873, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1b8876, 33 },
    { 0x926b8877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8b887c, 84 },
    { 0xd70b887c, 23 },
    { 0x432b887d, 61 },
    { 0xb10b887f, 98 },
    { 0, 0 },
    { 0x64b8881, 33 },
    { 0x73eb8882, 23 },
    { 0x7f6b8882, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06b8888, 33 },
    { 0x647b8889, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992b888d, 75 },
    { 0, 0 },
    { 0x475b888f, 77 },
    { 0xdcfb888f, 93 },
    { 0xc90b8891, 68 },
    { 0, 0 },
    { 0xcceb8893, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56b8896, 95 },
    { 0x92fb8897, 61 },
    { 0, 0 },
    { 0x1f1b8899, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597b88a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7b88a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bb88aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb88b7, 98 },
    { 0xfc5b88b8, 99 },
    { 0, 0 },
    { 0xf47b88ba, 76 },
    { 0, 0 },
    { 0xac7b88bc, 91 },
    { 0xee9b88bc, 88 },
    { 0x114b88be, 65 },
    { 0x502b88be, 23 },
    { 0x5c2b88c0, 23 },
    { 0xce9b88be, 23 },
    { 0xc4eb88c2, 33 },
    { 0x67eb88c3, 81 },
    { 0, 0 },
    { 0xbc1b88c5, 55 },
    { 0, 0 },
    { 0x5d1b88c7, 95 },
    { 0x3c5b88c7, 33 },
    { 0x4f4b88c9, 23 },
    { 0, 0 },
    { 0xc27b88cb, 61 },
    { 0, 0 },
    { 0x954b88cd, 61 },
    { 0x5d9b88ce, 38 },
    { 0x9b8b88cd, 90 },
    { 0xbc5b88ce, 85 },
    { 0x38fb88d1, 33 },
    { 0, 0 },
    { 0xdeeb88d3, 98 },
    { 0, 0 },
    { 0xa30b88d5, 43 },
    { 0xdafb88d6, 98 },
    { 0x6a4b88d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4b88da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fb88e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0b88e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb88e7, 55 },
    { 0x37fb88e7, 34 },
    { 0x773b88e9, 46 },
    { 0x89eb88ea, 90 },
    { 0, 0 },
    { 0x774b88ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35b88f2, 93 },
    { 0x326b88f3, 85 },
    { 0xba9b88f4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bb88f9, 93 },
    { 0xb24b88fa, 33 },
    { 0xbf7b88fb, 23 },
    { 0, 0 },
    { 0xe74b88fd, 95 },
    { 0, 0 },
    { 0x15bb88ff, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88b8905, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14b8909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3b890d, 52 },
    { 0x402b890e, 33 },
    { 0xb52b890f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cb8914, 34 },
    { 0x9b1b8915, 33 },
    { 0xc7eb8916, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b891b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4b891e, 95 },
    { 0x69cb891f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bb8923, 90 },
    { 0x523b8923, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756b892e, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eeb8933, 90 },
    { 0x30b8934, 93 },
    { 0, 0 },
    { 0xe26b8936, 98 },
    { 0, 0 },
    { 0xe69b8938, 23 },
    { 0x242b8939, 61 },
    { 0x7bcb8939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afb893e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcb8943, 67 },
    { 0, 0 },
    { 0xff2b8945, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fb894a, 23 },
    { 0xd7eb894b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x950b894e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955b8952, 61 },
    { 0x26ab8953, 61 },
    { 0xf39b8954, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9b8957, 77 },
    { 0xefb8957, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5b895b, 43 },
    { 0, 0 },
    { 0x7f1b895d, 99 },
    { 0xe3eb895e, 98 },
    { 0, 0 },
    { 0xa9fb8960, 95 },
    { 0x42fb8961, 61 },
    { 0x433b8962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0b8966, 78 },
    { 0xc1b8967, 90 },
    { 0xd63b8966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b896b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x201b896e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645b897e, 91 },
    { 0xa6fb897f, 17 },
    { 0x273b8980, 98 },
    { 0x194b8981, 98 },
    { 0x1c8b8980, 53 },
    { 0xf52b8981, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311b8989, 67 },
    { 0x6b6b898a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4eb898d, 100 },
    { 0x773b898e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798b8992, 86 },
    { 0xa34b8993, 99 },
    { 0x260b8994, 76 },
    { 0, 0 },
    { 0xf5cb8996, 33 },
    { 0, 0 },
    { 0xd40b8998, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57b899c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeb899f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83b89a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534b89aa, 33 },
    { 0xcd4b89aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cb89ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296b89b2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fb89b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76db89bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212b89c8, 33 },
    { 0xb97b89c9, 98 },
    { 0xcd6b89ca, 98 },
    { 0x7bcb89c9, 34 },
    { 0x25b89cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2b89cf, 90 },
    { 0, 0 },
    { 0x1a7b89d1, 76 },
    { 0, 0 },
    { 0xb47b89d3, 80 },
    { 0xa00b89d4, 90 },
    { 0xd4cb89d5, 48 },
    { 0x5cfb89d6, 76 },
    { 0xf40b89d6, 33 },
    { 0x2c7b89d8, 2 },
    { 0, 0 },
    { 0xbefb89da, 33 },
    { 0, 0 },
    { 0x2eab89dc, 33 },
    { 0, 0 },
    { 0x61eb89de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdb89e9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11b89ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83db89f0, 33 },
    { 0, 0 },
    { 0x8dcb89f2, 34 },
    { 0x5ab89f3, 90 },
    { 0x171b89f3, 61 },
    { 0, 0 },
    { 0xafcb89f6, 33 },
    { 0, 0 },
    { 0x80bb89f8, 23 },
    { 0x3fb89f9, 67 },
    { 0x59fb89f9, 19 },
    { 0, 0 },
    { 0xc2eb89fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69b8a00, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5b8a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365b8a0a, 95 },
    { 0x945b8a0b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5b8a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeab8a11, 61 },
    { 0xfccb8a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328b8a17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x510b8a1a, 98 },
    { 0x9a1b8a1b, 46 },
    { 0x62ab8a1c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8db8a22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdb8a25, 90 },
    { 0x5cdb8a26, 33 },
    { 0xde6b8a26, 90 },
    { 0, 0 },
    { 0x3a7b8a29, 46 },
    { 0x54b8a2a, 9 },
    { 0x531b8a2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0b8a34, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b8a3b, 100 },
    { 0x4a7b8a3b, 90 },
    { 0x83db8a3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b8a45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03b8a4a, 90 },
    { 0xc66b8a4a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4db8a50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0b8a54, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ab8a57, 99 },
    { 0, 0 },
    { 0xb7ab8a59, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98b8a5f, 95 },
    { 0x122b8a60, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cb8a67, 84 },
    { 0x744b8a68, 33 },
    { 0xf02b8a69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbb8a74, 23 },
    { 0x451b8a75, 98 },
    { 0x598b8a75, 98 },
    { 0, 0 },
    { 0x435b8a78, 61 },
    { 0xa6ab8a79, 33 },
    { 0, 0 },
    { 0x1ccb8a7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9b8a85, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55b8a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1b8a8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2b8a91, 78 },
    { 0xce2b8a91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03b8a96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6b8a9b, 93 },
    { 0x682b8a9c, 34 },
    { 0x15fb8a9d, 33 },
    { 0x77ab8a9d, 55 },
    { 0, 0 },
    { 0x11cb8aa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9b8aab, 90 },
    { 0xaa6b8aac, 17 },
    { 0xd18b8aac, 61 },
    { 0xde8b8aac, 61 },
    { 0, 0 },
    { 0x366b8ab0, 95 },
    { 0xbd5b8ab0, 23 },
    { 0x4acb8ab2, 23 },
    { 0xbd2b8ab2, 94 },
    { 0, 0 },
    { 0x42db8ab5, 76 },
    { 0, 0 },
    { 0xcf4b8ab7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ab8aba, 81 },
    { 0x5cab8aba, 98 },
    { 0, 0 },
    { 0xf2bb8abd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab8ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dab8ac4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1b8ac8, 95 },
    { 0xc6db8ac9, 95 },
    { 0x567b8aca, 66 },
    { 0x4e9b8acb, 90 },
    { 0xf0ab8acb, 34 },
    { 0x5bbb8acd, 100 },
    { 0x436b8aca, 76 },
    { 0x21bb8acf, 90 },
    { 0xf2ab8acf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x715b8ad3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94b8ad6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90b8ade, 61 },
    { 0, 0 },
    { 0xe06b8ae0, 99 },
    { 0xe3db8ae1, 97 },
    { 0xf82b8ae2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256b8ae6, 66 },
    { 0xb45b8ae6, 87 },
    { 0x8d3b8ae7, 85 },
    { 0x1e9b8ae6, 90 },
    { 0, 0 },
    { 0x19bb8aeb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdb8aee, 95 },
    { 0, 0 },
    { 0xbd3b8af0, 98 },
    { 0x861b8af0, 33 },
    { 0x7eeb8af2, 90 },
    { 0x174b8af0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8b8af7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcb8afd, 91 },
    { 0xc81b8afe, 97 },
    { 0x93ab8afe, 33 },
    { 0, 0 },
    { 0xa77b8b01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7b8b07, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cb8b0c, 23 },
    { 0x89fb8b0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65eb8b10, 23 },
    { 0x342b8b11, 90 },
    { 0xb03b8b12, 52 },
    { 0xceb8b11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43b8b17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1b8b1b, 61 },
    { 0x3eab8b1b, 93 },
    { 0x7f4b8b1c, 33 },
    { 0xa89b8b1c, 99 },
    { 0xdceb8b1f, 55 },
    { 0xccfb8b20, 95 },
    { 0, 0 },
    { 0x839b8b22, 33 },
    { 0x4b7b8b23, 98 },
    { 0, 0 },
    { 0xee6b8b25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30b8b29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357b8b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38db8b36, 95 },
    { 0, 0 },
    { 0xe25b8b38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b8b40, 33 },
    { 0, 0 },
    { 0x1afb8b42, 90 },
    { 0x557b8b43, 61 },
    { 0x23db8b42, 61 },
    { 0x2b4b8b45, 76 },
    { 0x4afb8b45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cb8b4a, 46 },
    { 0x305b8b4b, 85 },
    { 0x92b8b4c, 66 },
    { 0, 0 },
    { 0xdccb8b4e, 90 },
    { 0, 0 },
    { 0xb42b8b50, 69 },
    { 0, 0 },
    { 0x74db8b52, 33 },
    { 0, 0 },
    { 0xdd3b8b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b8b57, 95 },
    { 0xf5fb8b58, 95 },
    { 0x2f1b8b58, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6b8b5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140b8b62, 33 },
    { 0x295b8b63, 90 },
    { 0x92fb8b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb8b6b, 98 },
    { 0, 0 },
    { 0xf22b8b6d, 91 },
    { 0, 0 },
    { 0x187b8b6f, 52 },
    { 0x974b8b6f, 90 },
    { 0x5b6b8b70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7b8b74, 76 },
    { 0x635b8b74, 90 },
    { 0x744b8b76, 95 },
    { 0x814b8b77, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adb8b7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bb8b81, 77 },
    { 0, 0 },
    { 0x5d4b8b83, 98 },
    { 0x461b8b83, 90 },
    { 0, 0 },
    { 0x9dcb8b86, 43 },
    { 0x1e8b8b87, 90 },
    { 0, 0 },
    { 0xdeeb8b89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efb8b8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ab8b92, 33 },
    { 0, 0 },
    { 0x5acb8b94, 55 },
    { 0, 0 },
    { 0x4d1b8b96, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2b8b99, 90 },
    { 0x23db8b9a, 64 },
    { 0x5a1b8b9b, 34 },
    { 0, 0 },
    { 0x83db8b9d, 78 },
    { 0x69b8b9e, 33 },
    { 0, 0 },
    { 0x183b8ba0, 95 },
    { 0x1a9b8ba1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b8ba4, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x582b8ba7, 43 },
    { 0xaa4b8ba7, 61 },
    { 0x4a9b8ba9, 66 },
    { 0x226b8ba9, 33 },
    { 0x68cb8ba9, 75 },
    { 0x209b8ba7, 52 },
    { 0x871b8ba8, 33 },
    { 0xeeeb8bae, 61 },
    { 0x6fbb8baf, 31 },
    { 0x793b8ba8, 76 },
    { 0x877b8bb1, 69 },
    { 0xbd4b8bb2, 87 },
    { 0xf58b8bb1, 98 },
    { 0xce7b8bb4, 75 },
    { 0x998b8bb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9b8bb8, 34 },
    { 0x3b8b8bb9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5db8bbe, 43 },
    { 0x22b8bbf, 87 },
    { 0xf29b8bc0, 91 },
    { 0, 0 },
    { 0x1ceb8bc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaab8bcb, 33 },
    { 0x725b8bcb, 46 },
    { 0, 0 },
    { 0xfeab8bce, 61 },
    { 0xb94b8bcf, 95 },
    { 0x218b8bd0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69b8bd4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cb8bdb, 98 },
    { 0, 0 },
    { 0x2f2b8bdd, 90 },
    { 0x3f6b8bdd, 23 },
    { 0x375b8bdf, 90 },
    { 0x9b8b8bdf, 90 },
    { 0xdfb8be1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608b8be6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22b8be9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdb8bec, 61 },
    { 0x81ab8bed, 61 },
    { 0, 0 },
    { 0x9b0b8bef, 61 },
    { 0, 0 },
    { 0x85bb8bf1, 92 },
    { 0xe93b8bf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ab8bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x358b8bf8, 46 },
    { 0x45fb8bf9, 98 },
    { 0x45fb8bfa, 95 },
    { 0, 0 },
    { 0xee1b8bfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3db8c00, 61 },
    { 0xa22b8c00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4b8c06, 61 },
    { 0, 0 },
    { 0xc13b8c08, 23 },
    { 0, 0 },
    { 0xcdb8c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866b8c0e, 90 },
    { 0, 0 },
    { 0x7b4b8c10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfb8c13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2b8c18, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x741b8c1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bb8c22, 23 },
    { 0x492b8c23, 80 },
    { 0xa98b8c23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444b8c2a, 88 },
    { 0x478b8c2a, 66 },
    { 0, 0 },
    { 0xf80b8c2d, 98 },
    { 0x577b8c2d, 46 },
    { 0x4dcb8c2f, 67 },
    { 0, 0 },
    { 0x7c4b8c31, 33 },
    { 0x42bb8c32, 31 },
    { 0x2cfb8c31, 90 },
    { 0, 0 },
    { 0xe3bb8c35, 85 },
    { 0x338b8c35, 84 },
    { 0x97ab8c35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bb8c3c, 55 },
    { 0x4cdb8c3d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ab8c41, 23 },
    { 0, 0 },
    { 0x591b8c43, 55 },
    { 0xb2bb8c44, 23 },
    { 0, 0 },
    { 0xd19b8c46, 91 },
    { 0, 0 },
    { 0x67fb8c48, 90 },
    { 0xfadb8c49, 34 },
    { 0, 0 },
    { 0x655b8c4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0b8c50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbb8c53, 68 },
    { 0x731b8c54, 33 },
    { 0x25ab8c55, 85 },
    { 0x207b8c55, 53 },
    { 0xd97b8c55, 43 },
    { 0x922b8c58, 35 },
    { 0x6e6b8c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1b8c5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138b8c60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1b8c68, 61 },
    { 0x84cb8c68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dab8c6d, 98 },
    { 0xdcab8c6e, 33 },
    { 0x603b8c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efb8c73, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04b8c7d, 36 },
    { 0, 0 },
    { 0xa74b8c7f, 95 },
    { 0xa73b8c80, 34 },
    { 0xc79b8c81, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59b8c8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fb8c92, 61 },
    { 0, 0 },
    { 0x223b8c94, 90 },
    { 0xd1b8c95, 90 },
    { 0x425b8c95, 87 },
    { 0x36cb8c96, 91 },
    { 0xfc4b8c98, 67 },
    { 0x74fb8c98, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecb8c9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38b8ca2, 99 },
    { 0xb89b8ca3, 99 },
    { 0x577b8ca4, 78 },
    { 0, 0 },
    { 0xb0ab8ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ab8cad, 61 },
    { 0x44eb8cae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64b8cb2, 90 },
    { 0x388b8cb2, 61 },
    { 0x777b8cb2, 23 },
    { 0x48b8cb5, 88 },
    { 0x66ab8cb6, 55 },
    { 0x968b8cb6, 87 },
    { 0x240b8cb7, 67 },
    { 0x3e2b8cb6, 23 },
    { 0x45b8cb7, 85 },
    { 0xb9bb8cba, 61 },
    { 0xbb4b8cbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9db8cc0, 98 },
    { 0, 0 },
    { 0xc7eb8cc2, 33 },
    { 0x845b8cc2, 33 },
    { 0x906b8cc2, 33 },
    { 0x588b8cc5, 61 },
    { 0x8d9b8cc6, 34 },
    { 0xf95b8cc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702b8ccb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x980b8cce, 61 },
    { 0xaeeb8ccf, 98 },
    { 0xfe5b8cd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabb8cd4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8b8cd9, 75 },
    { 0x579b8cda, 98 },
    { 0, 0 },
    { 0xc9fb8cdc, 61 },
    { 0x962b8cdc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abb8ce0, 53 },
    { 0x935b8ce1, 76 },
    { 0xef7b8ce2, 77 },
    { 0xbc9b8ce3, 61 },
    { 0x8cbb8ce1, 98 },
    { 0xe58b8ce2, 2 },
    { 0x9dab8ce3, 98 },
    { 0x7f7b8ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624b8cec, 90 },
    { 0x769b8ced, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67b8cf1, 90 },
    { 0xbdcb8cf1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bb8cf6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2b8cfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb8d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958b8d0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fb8d0e, 43 },
    { 0, 0 },
    { 0xbeab8d10, 34 },
    { 0, 0 },
    { 0xfd2b8d12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2b8d16, 33 },
    { 0, 0 },
    { 0x207b8d18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19b8d1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267b8d21, 81 },
    { 0xe1fb8d22, 33 },
    { 0x2bfb8d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x695b8d26, 55 },
    { 0, 0 },
    { 0x44cb8d28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7b8d2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab8d2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x55b8d31, 33 },
    { 0x8f0b8d31, 33 },
    { 0x59eb8d31, 90 },
    { 0x3fb8d34, 55 },
    { 0xf3db8d35, 23 },
    { 0x76eb8d35, 46 },
    { 0x27ab8d37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8b8d3e, 90 },
    { 0x6a8b8d3f, 91 },
    { 0, 0 },
    { 0x73b8d41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cab8d47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25b8d4a, 23 },
    { 0x8cfb8d4b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2db8d4e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9b8d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fb8d59, 76 },
    { 0, 0 },
    { 0x27cb8d5b, 61 },
    { 0xc6ab8d5c, 99 },
    { 0xd09b8d5c, 61 },
    { 0x5e3b8d5e, 46 },
    { 0xf77b8d5e, 93 },
    { 0xd64b8d60, 67 },
    { 0x1c3b8d61, 67 },
    { 0x810b8d5f, 33 },
    { 0x333b8d63, 66 },
    { 0xd6ab8d63, 33 },
    { 0, 0 },
    { 0xe9cb8d66, 90 },
    { 0x93bb8d67, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eb8d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eab8d6d, 55 },
    { 0xae8b8d6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911b8d72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffab8d78, 100 },
    { 0xdafb8d79, 23 },
    { 0xab8b8d79, 61 },
    { 0x6f8b8d7b, 95 },
    { 0x3bdb8d78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0b8d81, 75 },
    { 0x256b8d82, 65 },
    { 0xd0db8d83, 88 },
    { 0x6e1b8d84, 90 },
    { 0xf7eb8d85, 55 },
    { 0x9fcb8d85, 34 },
    { 0xf59b8d86, 52 },
    { 0x61db8d86, 90 },
    { 0xf09b8d89, 95 },
    { 0x819b8d81, 100 },
    { 0x9b9b8d8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b8d93, 93 },
    { 0xce7b8d94, 76 },
    { 0x371b8d95, 33 },
    { 0x813b8d96, 98 },
    { 0x225b8d97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1b8d9a, 52 },
    { 0, 0 },
    { 0xfd3b8d9c, 34 },
    { 0, 0 },
    { 0xbc7b8d9e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbb8da5, 77 },
    { 0xe5eb8da5, 33 },
    { 0xab6b8da7, 95 },
    { 0xe55b8da6, 95 },
    { 0xe5bb8da6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35b8dac, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243b8db1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0b8db5, 55 },
    { 0xcbcb8db5, 67 },
    { 0x168b8db7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9db8dbd, 100 },
    { 0x473b8dbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54b8dc6, 34 },
    { 0, 0 },
    { 0x269b8dc8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1db8dcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4b8dd1, 33 },
    { 0x98cb8dd2, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeb8dd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2b8dda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eeb8dde, 55 },
    { 0, 0 },
    { 0x6e1b8de0, 34 },
    { 0, 0 },
    { 0x8e6b8de2, 33 },
    { 0x8d3b8de3, 87 },
    { 0x762b8de3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514b8ded, 95 },
    { 0x6dbb8dee, 78 },
    { 0, 0 },
    { 0x6b2b8df0, 90 },
    { 0, 0 },
    { 0x856b8df2, 88 },
    { 0, 0 },
    { 0xc43b8df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272b8dfd, 76 },
    { 0xda2b8dfe, 34 },
    { 0xa05b8dff, 33 },
    { 0xd9bb8e00, 51 },
    { 0, 0 },
    { 0x627b8e02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b8e05, 98 },
    { 0x567b8e05, 98 },
    { 0x469b8e07, 33 },
    { 0, 0 },
    { 0x86db8e09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efb8e0d, 46 },
    { 0xd12b8e0e, 91 },
    { 0xd8b8e0e, 23 },
    { 0, 0 },
    { 0x190b8e11, 61 },
    { 0xf4ab8e12, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397b8e1b, 43 },
    { 0x3d4b8e1c, 61 },
    { 0, 0 },
    { 0xfa6b8e1e, 61 },
    { 0x7ceb8e1f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beb8e22, 23 },
    { 0, 0 },
    { 0x36b8e24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b8e27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb8e2e, 33 },
    { 0x54cb8e2f, 38 },
    { 0x957b8e30, 95 },
    { 0, 0 },
    { 0xc12b8e32, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2b8e38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364b8e3c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x194b8e3f, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x824b8e42, 52 },
    { 0xd67b8e43, 100 },
    { 0, 0 },
    { 0xa5eb8e45, 88 },
    { 0, 0 },
    { 0xc80b8e47, 66 },
    { 0xa1fb8e48, 61 },
    { 0x984b8e47, 33 },
    { 0xfd8b8e47, 95 },
    { 0x3bcb8e4b, 93 },
    { 0xd1bb8e4b, 98 },
    { 0xf08b8e4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422b8e52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0b8e55, 77 },
    { 0, 0 },
    { 0x8acb8e57, 23 },
    { 0xfa0b8e58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccb8e5b, 95 },
    { 0x2afb8e5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42b8e5f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbb8e63, 23 },
    { 0, 0 },
    { 0x7b2b8e65, 66 },
    { 0, 0 },
    { 0xdb1b8e67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569b8e71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4b8e77, 33 },
    { 0, 0 },
    { 0x8f8b8e79, 56 },
    { 0x11eb8e79, 61 },
    { 0, 0 },
    { 0xf27b8e7c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6b8e7f, 90 },
    { 0x25b8e80, 98 },
    { 0x9d0b8e81, 33 },
    { 0, 0 },
    { 0x3e3b8e83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eb8e86, 76 },
    { 0xed8b8e87, 34 },
    { 0x973b8e88, 78 },
    { 0x6e5b8e89, 34 },
    { 0, 0 },
    { 0xb27b8e8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20b8e93, 55 },
    { 0x6ccb8e94, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6b8e97, 43 },
    { 0x15fb8e98, 61 },
    { 0x331b8e97, 98 },
    { 0xa3bb8e97, 33 },
    { 0xb3fb8e98, 51 },
    { 0x7a4b8e9c, 33 },
    { 0xec8b8e9b, 90 },
    { 0x2a1b8e9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x316b8ea1, 98 },
    { 0, 0 },
    { 0x360b8ea3, 52 },
    { 0, 0 },
    { 0x630b8ea5, 61 },
    { 0, 0 },
    { 0xdc8b8ea7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4b8eae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b8eb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db8eb4, 55 },
    { 0xca5b8eb5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x870b8eb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5b8ebd, 95 },
    { 0, 0 },
    { 0x416b8ebf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x940b8ec2, 43 },
    { 0, 0 },
    { 0x13cb8ec4, 33 },
    { 0xde5b8ec5, 93 },
    { 0x91bb8ec4, 61 },
    { 0x67ab8ec7, 33 },
    { 0x1b6b8ec4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662b8ecc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40b8ecf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fb8ed2, 34 },
    { 0x151b8ed3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b8edc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5b8ee0, 95 },
    { 0, 0 },
    { 0x750b8ee2, 61 },
    { 0, 0 },
    { 0xa81b8ee4, 61 },
    { 0, 0 },
    { 0x14fb8ee6, 61 },
    { 0x6acb8ee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152b8eeb, 84 },
    { 0xda3b8eec, 33 },
    { 0x4d5b8eed, 90 },
    { 0, 0 },
    { 0xcf0b8eef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747b8ef6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b8efa, 95 },
    { 0xa51b8efb, 84 },
    { 0xc0cb8efb, 98 },
    { 0x74eb8efa, 33 },
    { 0, 0 },
    { 0xa5cb8eff, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4db8f02, 84 },
    { 0, 0 },
    { 0x357b8f04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x474b8f07, 23 },
    { 0xbbfb8f08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb8f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1b8f13, 90 },
    { 0x1fab8f13, 75 },
    { 0xfe4b8f13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ab8f1b, 91 },
    { 0x23ab8f1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559b8f20, 81 },
    { 0, 0 },
    { 0x2a2b8f22, 61 },
    { 0x66ab8f23, 90 },
    { 0x958b8f24, 34 },
    { 0xf59b8f25, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fb8f29, 85 },
    { 0x6ebb8f29, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0b8f2e, 43 },
    { 0x6d7b8f2f, 98 },
    { 0xa22b8f2f, 90 },
    { 0x1f0b8f30, 98 },
    { 0x345b8f2e, 46 },
    { 0x441b8f33, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9eb8f37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4b8f3a, 55 },
    { 0xa47b8f3b, 66 },
    { 0x2b8f3c, 61 },
    { 0x66fb8f3b, 99 },
    { 0, 0 },
    { 0xfe0b8f3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605b8f47, 88 },
    { 0xa15b8f47, 55 },
    { 0, 0 },
    { 0x4e8b8f4a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ab8f53, 90 },
    { 0x484b8f54, 78 },
    { 0xdccb8f55, 23 },
    { 0, 0 },
    { 0xc83b8f57, 78 },
    { 0xa04b8f57, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fb8f5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6b8f61, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8b8f67, 55 },
    { 0, 0 },
    { 0x702b8f69, 90 },
    { 0x52b8f6a, 61 },
    { 0, 0 },
    { 0x87b8f6c, 34 },
    { 0, 0 },
    { 0x9fab8f6e, 33 },
    { 0, 0 },
    { 0x83db8f70, 23 },
    { 0, 0 },
    { 0xdfeb8f72, 87 },
    { 0x3d3b8f73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbb8f76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cb8f7d, 61 },
    { 0x8e8b8f7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceb8f85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acb8f88, 61 },
    { 0x8d8b8f89, 85 },
    { 0xbfbb8f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebb8f8f, 98 },
    { 0, 0 },
    { 0x775b8f91, 98 },
    { 0x620b8f92, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcb8f98, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123b8f9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfb8fa5, 93 },
    { 0xe13b8fa5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fb8fa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fae, 46 },
    { 0x69fb8faf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5b8fb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb8fbc, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eb8fc2, 98 },
    { 0, 0 },
    { 0x96cb8fc4, 61 },
    { 0, 0 },
    { 0x237b8fc6, 93 },
    { 0x12fb8fc7, 61 },
    { 0xd31b8fc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2b8fcc, 90 },
    { 0xc65b8fcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777b8fd5, 46 },
    { 0x98bb8fd5, 55 },
    { 0, 0 },
    { 0x701b8fd8, 33 },
    { 0, 0 },
    { 0x1dfb8fda, 33 },
    { 0x7f2b8fdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567b8fdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898b8fe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51b8fec, 23 },
    { 0, 0 },
    { 0x369b8fee, 98 },
    { 0, 0 },
    { 0xb2cb8ff0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63b8ff3, 46 },
    { 0x5fcb8ff4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5b8ff7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fb9002, 95 },
    { 0, 0 },
    { 0x214b9004, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740b900b, 65 },
    { 0x9f7b900c, 99 },
    { 0xb08b900b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63b9010, 98 },
    { 0, 0 },
    { 0xc73b9012, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605b9017, 33 },
    { 0xdbeb9017, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62b901b, 99 },
    { 0, 0 },
    { 0xdbab901d, 90 },
    { 0x3edb901d, 66 },
    { 0xd68b901f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7b9023, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cb902a, 93 },
    { 0x37fb902a, 90 },
    { 0x20ab902a, 55 },
    { 0, 0 },
    { 0x7c5b902e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb9038, 76 },
    { 0x11fb9038, 76 },
    { 0x2e1b903a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbb9044, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b9047, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0eb904b, 98 },
    { 0xd1ab904b, 52 },
    { 0, 0 },
    { 0xeb4b904e, 93 },
    { 0x95cb904f, 33 },
    { 0xf00b904f, 95 },
    { 0x54ab9051, 66 },
    { 0xdf4b9052, 33 },
    { 0xe21b9051, 95 },
    { 0xb38b9054, 90 },
    { 0xdccb9055, 98 },
    { 0x98fb9050, 81 },
    { 0x3fcb9057, 78 },
    { 0x2a0b9058, 83 },
    { 0x719b9057, 85 },
    { 0xa8ab905a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330b9060, 90 },
    { 0xdc4b9061, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41b9064, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67b9067, 23 },
    { 0x29eb9068, 68 },
    { 0, 0 },
    { 0xb9eb906a, 90 },
    { 0x7a8b906b, 99 },
    { 0x180b906c, 61 },
    { 0xe59b906a, 95 },
    { 0xf74b906e, 76 },
    { 0x72eb906f, 33 },
    { 0x163b9070, 43 },
    { 0, 0 },
    { 0x65eb9072, 98 },
    { 0x283b9073, 95 },
    { 0x961b9074, 22 },
    { 0xceb9075, 33 },
    { 0x5bb9075, 95 },
    { 0xe64b9077, 33 },
    { 0xbfdb9077, 61 },
    { 0x702b9079, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eb907c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0b9084, 33 },
    { 0, 0 },
    { 0x381b9086, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9b908d, 84 },
    { 0x166b908e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6b9091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917b9095, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x821b9098, 22 },
    { 0xed4b9098, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7b909c, 23 },
    { 0x401b909d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650b90a2, 61 },
    { 0, 0 },
    { 0x500b90a4, 99 },
    { 0, 0 },
    { 0xd74b90a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfb90a9, 78 },
    { 0xf2bb90a9, 84 },
    { 0x9e0b90ab, 90 },
    { 0, 0 },
    { 0xc0db90ad, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0b90b2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1b90ba, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eb90c1, 61 },
    { 0xb2ab90c2, 98 },
    { 0xb7b90c1, 90 },
    { 0xc0cb90c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63eb90c9, 90 },
    { 0x279b90ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24db90ce, 84 },
    { 0x6d5b90cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3b90d3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8b90d7, 81 },
    { 0x3b5b90d7, 100 },
    { 0xa4bb90d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58b90dc, 23 },
    { 0x7bab90dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5b90e0, 23 },
    { 0, 0 },
    { 0x976b90e2, 98 },
    { 0x32eb90e3, 84 },
    { 0x898b90e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3b90e7, 75 },
    { 0x765b90e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4b90f1, 98 },
    { 0x7e7b90f1, 61 },
    { 0, 0 },
    { 0xdb6b90f4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ab90f9, 61 },
    { 0xc5ab90fa, 78 },
    { 0xc47b90f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589b9101, 61 },
    { 0xd74b9102, 90 },
    { 0x980b9103, 98 },
    { 0x2a7b9102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfb910b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2b9111, 90 },
    { 0x51bb9111, 98 },
    { 0x1e6b9113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31b911a, 61 },
    { 0xf4b911a, 67 },
    { 0x309b911c, 90 },
    { 0x9ccb911d, 76 },
    { 0x81eb911e, 100 },
    { 0x77ab911f, 95 },
    { 0, 0 },
    { 0x4a7b9121, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2b912d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692b9133, 23 },
    { 0, 0 },
    { 0xf7bb9135, 33 },
    { 0xc55b9135, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x43eb9139, 95 },
    { 0xbe6b913a, 98 },
    { 0, 0 },
    { 0xd10b913c, 95 },
    { 0, 0 },
    { 0xb10b913e, 34 },
    { 0, 0 },
    { 0x682b9140, 90 },
    { 0x464b9141, 99 },
    { 0, 0 },
    { 0x5bab9143, 98 },
    { 0x7bb9144, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b914c, 90 },
    { 0x467b914c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1b9158, 98 },
    { 0, 0 },
    { 0x3fcb915a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76b915d, 33 },
    { 0x4c0b915e, 23 },
    { 0x49cb915f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800b9164, 78 },
    { 0x248b9165, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92b9168, 95 },
    { 0x9f2b9168, 34 },
    { 0x4a0b916a, 33 },
    { 0x565b916a, 61 },
    { 0xabab9169, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ab916f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8b9172, 33 },
    { 0x79b9173, 33 },
    { 0, 0 },
    { 0x162b9175, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acb9178, 33 },
    { 0, 0 },
    { 0xa5ab917a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110b917f, 61 },
    { 0, 0 },
    { 0xfafb9181, 34 },
    { 0, 0 },
    { 0xbf1b9183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5b9187, 91 },
    { 0, 0 },
    { 0x2f4b9189, 87 },
    { 0x95cb9189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49b9190, 52 },
    { 0xadab9191, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0b9195, 33 },
    { 0xc97b9196, 90 },
    { 0x671b9197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87b919a, 95 },
    { 0xb34b919b, 95 },
    { 0x293b919c, 76 },
    { 0, 0 },
    { 0xd6bb919e, 43 },
    { 0, 0 },
    { 0xf06b91a0, 38 },
    { 0, 0 },
    { 0x10ab91a2, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895b91a7, 76 },
    { 0xe14b91a7, 46 },
    { 0xd31b91a9, 91 },
    { 0x7a8b91a9, 43 },
    { 0xbfab91ab, 33 },
    { 0x550b91aa, 55 },
    { 0xcfbb91ac, 95 },
    { 0x27cb91ab, 61 },
    { 0x55db91af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6b91b5, 90 },
    { 0xe4b91b6, 53 },
    { 0xb30b91b7, 55 },
    { 0x778b91b8, 38 },
    { 0x655b91b8, 33 },
    { 0, 0 },
    { 0xb11b91bb, 23 },
    { 0xaf0b91bb, 90 },
    { 0, 0 },
    { 0xae8b91be, 61 },
    { 0, 0 },
    { 0x44fb91c0, 91 },
    { 0x76ab91c0, 61 },
    { 0xc81b91c2, 90 },
    { 0x700b91c2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dab91c9, 90 },
    { 0xbeb91c9, 84 },
    { 0x2e3b91cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x72eb91ce, 61 },
    { 0x435b91cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadeb91d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8b91d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fb91db, 61 },
    { 0x5fbb91dc, 34 },
    { 0x4eab91dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbceb91e1, 90 },
    { 0x8b8b91e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x677b91e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3b91ea, 98 },
    { 0xe80b91ea, 98 },
    { 0, 0 },
    { 0x6a2b91ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22b91f1, 52 },
    { 0x36bb91f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bb91f5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460b91fa, 87 },
    { 0x1b9b91fb, 95 },
    { 0xf74b91fa, 19 },
    { 0, 0 },
    { 0x2e1b91fe, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b9201, 93 },
    { 0, 0 },
    { 0xc05b9203, 95 },
    { 0xdfeb9204, 23 },
    { 0x291b9203, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29b920c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cb9214, 76 },
    { 0, 0 },
    { 0x594b9216, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfb921e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x990b9221, 83 },
    { 0, 0 },
    { 0x140b9223, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x347b9226, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3b922f, 91 },
    { 0xaf6b9230, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42b9234, 34 },
    { 0, 0 },
    { 0x241b9236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb923a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46b923d, 78 },
    { 0x5d7b923e, 76 },
    { 0x6a5b923f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78db9244, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36b924e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9b9251, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646b9257, 87 },
    { 0, 0 },
    { 0x787b9259, 90 },
    { 0xec9b925a, 43 },
    { 0x4f7b925b, 23 },
    { 0x5cbb9259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecb9261, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51b9264, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617b9269, 91 },
    { 0, 0 },
    { 0x9e8b926b, 53 },
    { 0, 0 },
    { 0x51ab926d, 55 },
    { 0, 0 },
    { 0x298b926f, 68 },
    { 0x2c0b9270, 88 },
    { 0x1a5b9271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79db9275, 95 },
    { 0x860b9276, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86b9279, 93 },
    { 0x67ab927a, 33 },
    { 0xe86b927b, 33 },
    { 0xc4bb927b, 23 },
    { 0xadeb927d, 52 },
    { 0xae8b927d, 98 },
    { 0x6e9b927d, 61 },
    { 0xcf8b9280, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb9283, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9286, 95 },
    { 0, 0 },
    { 0xb5fb9288, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x950b928b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929b928f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171b9293, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7eb929e, 78 },
    { 0xc5ab929f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x324b92a2, 34 },
    { 0, 0 },
    { 0x866b92a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db92a7, 46 },
    { 0xa53b92a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b92ab, 23 },
    { 0, 0 },
    { 0x326b92ad, 90 },
    { 0, 0 },
    { 0x29b92af, 98 },
    { 0x8beb92b0, 40 },
    { 0x764b92b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x525b92b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01b92b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcb92bc, 43 },
    { 0, 0 },
    { 0xecb92be, 52 },
    { 0xfe1b92bf, 46 },
    { 0xcafb92c0, 43 },
    { 0xbb8b92c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebb92c6, 98 },
    { 0, 0 },
    { 0x85fb92c8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20b92cb, 43 },
    { 0x886b92cc, 61 },
    { 0x892b92cc, 23 },
    { 0x25fb92ce, 91 },
    { 0xa4bb92cf, 61 },
    { 0x124b92cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feb92d4, 61 },
    { 0xddcb92d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402b92db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7b92de, 84 },
    { 0, 0 },
    { 0xa89b92e0, 95 },
    { 0xc3ab92e1, 76 },
    { 0xea6b92e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17b92e5, 61 },
    { 0, 0 },
    { 0x579b92e7, 61 },
    { 0, 0 },
    { 0x534b92e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ab92f1, 34 },
    { 0x610b92f2, 34 },
    { 0, 0 },
    { 0x1d1b92f4, 34 },
    { 0xcd6b92f4, 40 },
    { 0x33db92f6, 95 },
    { 0x80eb92f5, 90 },
    { 0x297b92f8, 61 },
    { 0x534b92f7, 98 },
    { 0x321b92fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5b92fd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510b9301, 90 },
    { 0, 0 },
    { 0x98eb9303, 76 },
    { 0xf93b9304, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x447b9307, 33 },
    { 0xce9b9307, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bb930b, 61 },
    { 0xcc7b930c, 76 },
    { 0x13fb930b, 91 },
    { 0xdeab930b, 100 },
    { 0xd0bb930f, 33 },
    { 0, 0 },
    { 0x4dcb9311, 95 },
    { 0, 0 },
    { 0x75cb9313, 90 },
    { 0, 0 },
    { 0x7bab9315, 88 },
    { 0xa26b9316, 33 },
    { 0x506b9317, 43 },
    { 0xd08b9318, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46b931b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9322, 61 },
    { 0, 0 },
    { 0xb15b9324, 91 },
    { 0x53db9324, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383b932c, 17 },
    { 0x98bb932d, 78 },
    { 0, 0 },
    { 0x609b932f, 23 },
    { 0x1edb9330, 100 },
    { 0xca9b9330, 100 },
    { 0x73b9332, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79b9337, 96 },
    { 0, 0 },
    { 0xa80b9339, 81 },
    { 0x6f4b9339, 23 },
    { 0xeddb933b, 23 },
    { 0xa3cb933a, 34 },
    { 0xb8b933b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ab934a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bb9354, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fb935c, 95 },
    { 0xb1eb935d, 93 },
    { 0, 0 },
    { 0x1e6b935f, 90 },
    { 0xd67b9360, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383b9366, 87 },
    { 0xd39b9367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aab936d, 91 },
    { 0, 0 },
    { 0x22cb936f, 98 },
    { 0x30fb936f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4b9373, 95 },
    { 0xc98b9374, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eeb9378, 43 },
    { 0xb7db9379, 34 },
    { 0x30ab9379, 61 },
    { 0xe04b937b, 77 },
    { 0xe5fb937b, 98 },
    { 0x6bdb937a, 81 },
    { 0, 0 },
    { 0x4bfb937f, 23 },
    { 0x49ab9380, 33 },
    { 0x485b9381, 23 },
    { 0x715b9381, 61 },
    { 0x107b9380, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88b9387, 95 },
    { 0, 0 },
    { 0x981b9389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccb938d, 23 },
    { 0xecdb938e, 95 },
    { 0x9fdb938f, 61 },
    { 0x160b9390, 61 },
    { 0x17fb938f, 61 },
    { 0x5bb9392, 61 },
    { 0x837b9390, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7b9398, 98 },
    { 0xc70b9399, 87 },
    { 0xcd7b9399, 83 },
    { 0x31b9399, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75b93a1, 76 },
    { 0xa2b93a2, 90 },
    { 0xb30b93a2, 33 },
    { 0, 0 },
    { 0xc4fb93a5, 90 },
    { 0, 0 },
    { 0xe65b93a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab93ab, 76 },
    { 0xa53b93ac, 76 },
    { 0, 0 },
    { 0xdd4b93ae, 95 },
    { 0x2acb93af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1b93b4, 23 },
    { 0, 0 },
    { 0x86b93b6, 34 },
    { 0xca1b93b7, 98 },
    { 0x913b93b7, 23 },
    { 0xc54b93b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebb93bd, 76 },
    { 0, 0 },
    { 0x294b93bf, 90 },
    { 0x9dbb93c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36b93c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30b93c8, 84 },
    { 0, 0 },
    { 0x5eb93ca, 76 },
    { 0x3f4b93ca, 99 },
    { 0x2e9b93ca, 93 },
    { 0x5a3b93cd, 98 },
    { 0x189b93ca, 33 },
    { 0x78b93cc, 33 },
    { 0, 0 },
    { 0x59bb93d1, 95 },
    { 0x9eb93d2, 95 },
    { 0x4b4b93d1, 38 },
    { 0xff1b93d4, 76 },
    { 0, 0 },
    { 0x9a3b93d6, 33 },
    { 0, 0 },
    { 0xd8cb93d8, 33 },
    { 0, 0 },
    { 0x22cb93da, 33 },
    { 0, 0 },
    { 0x423b93dc, 61 },
    { 0xc20b93dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbb93e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ab93e8, 17 },
    { 0xeebb93e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740b93ee, 98 },
    { 0x961b93ef, 46 },
    { 0x752b93ef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3b93f4, 34 },
    { 0, 0 },
    { 0x4ab93f6, 95 },
    { 0x959b93f7, 36 },
    { 0xc35b93f8, 90 },
    { 0xafab93f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722b93ff, 33 },
    { 0xbc5b9400, 23 },
    { 0, 0 },
    { 0xe1ab9402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbb9411, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cab9414, 61 },
    { 0x4a2b9414, 77 },
    { 0xc79b9416, 17 },
    { 0xce1b9414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bab9421, 98 },
    { 0, 0 },
    { 0xe66b9423, 81 },
    { 0xa52b9423, 61 },
    { 0x222b9424, 33 },
    { 0, 0 },
    { 0xbc8b9427, 34 },
    { 0, 0 },
    { 0x73eb9429, 84 },
    { 0x1d8b942a, 33 },
    { 0xacab942a, 34 },
    { 0xdc1b9429, 98 },
    { 0xf9b942d, 98 },
    { 0x844b9429, 100 },
    { 0x7e9b942f, 61 },
    { 0xcdb9429, 98 },
    { 0x5cab9431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94b9434, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559b943b, 76 },
    { 0x6acb943b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fb943f, 78 },
    { 0, 0 },
    { 0x607b9441, 52 },
    { 0xd5fb9442, 96 },
    { 0x2dab9443, 88 },
    { 0x4c9b9441, 77 },
    { 0xd11b9445, 43 },
    { 0, 0 },
    { 0x5b4b9447, 33 },
    { 0xb8bb9448, 98 },
    { 0x94b9449, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4b944c, 98 },
    { 0, 0 },
    { 0x36cb944e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebb9453, 33 },
    { 0, 0 },
    { 0x6d0b9455, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4b9459, 23 },
    { 0xc9fb945a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0b945e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88b9465, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237b9469, 66 },
    { 0, 0 },
    { 0x72ab946b, 95 },
    { 0x5e3b946c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ab946f, 68 },
    { 0xce4b9470, 30 },
    { 0, 0 },
    { 0x970b9472, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bb9478, 91 },
    { 0xb4eb9479, 100 },
    { 0x3d9b947a, 100 },
    { 0xde0b947b, 90 },
    { 0, 0 },
    { 0x258b947d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551b9481, 23 },
    { 0x43b9481, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bb9485, 95 },
    { 0x660b9486, 61 },
    { 0, 0 },
    { 0x7d5b9488, 98 },
    { 0x103b9489, 90 },
    { 0x1e6b948a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0b9496, 90 },
    { 0x902b9496, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31b949a, 66 },
    { 0xb29b949b, 43 },
    { 0x551b949b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bb94a2, 93 },
    { 0x52ab94a3, 98 },
    { 0xca5b94a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ab94a8, 66 },
    { 0x369b94a9, 34 },
    { 0x468b94aa, 27 },
    { 0, 0 },
    { 0xd23b94ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3b94b0, 34 },
    { 0, 0 },
    { 0xc30b94b2, 61 },
    { 0x750b94b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eb94b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbb94b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0b94bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49b94c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbb94c4, 23 },
    { 0, 0 },
    { 0x6ecb94c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6b94c9, 95 },
    { 0, 0 },
    { 0xd71b94cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b94d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214b94d4, 84 },
    { 0x596b94d4, 43 },
    { 0, 0 },
    { 0x636b94d7, 33 },
    { 0xf4bb94d8, 61 },
    { 0x2a2b94d9, 90 },
    { 0x387b94da, 81 },
    { 0x3d1b94da, 76 },
    { 0x380b94d7, 55 },
    { 0, 0 },
    { 0x82fb94de, 43 },
    { 0, 0 },
    { 0x7dab94e0, 23 },
    { 0x314b94e0, 98 },
    { 0x356b94e2, 61 },
    { 0x638b94e2, 61 },
    { 0x84ab94e1, 95 },
    { 0x2e3b94e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5b94e8, 76 },
    { 0, 0 },
    { 0x9efb94ea, 61 },
    { 0, 0 },
    { 0x1d1b94ec, 90 },
    { 0x680b94ed, 85 },
    { 0x8e8b94ed, 90 },
    { 0, 0 },
    { 0x99db94f0, 85 },
    { 0xc4db94f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fb94f4, 88 },
    { 0x9c6b94f4, 84 },
    { 0x45cb94f4, 76 },
    { 0, 0 },
    { 0x601b94f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x741b94fb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5b94fe, 76 },
    { 0x2fbb94fe, 65 },
    { 0, 0 },
    { 0xfdcb9501, 95 },
    { 0, 0 },
    { 0x322b9503, 23 },
    { 0, 0 },
    { 0x572b9505, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4b9508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8b9512, 93 },
    { 0x5fab9513, 93 },
    { 0xddeb9514, 98 },
    { 0x5afb9515, 93 },
    { 0xac6b9512, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbb951c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edb9520, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5db9525, 61 },
    { 0x8dab9525, 55 },
    { 0x5c4b9527, 93 },
    { 0, 0 },
    { 0x416b9529, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15b9535, 68 },
    { 0x59bb9536, 56 },
    { 0x3a9b9537, 90 },
    { 0, 0 },
    { 0xcc7b9539, 90 },
    { 0xab0b953a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ab9540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1b954b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29db954f, 33 },
    { 0x29cb954f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753b9559, 99 },
    { 0x89db955a, 90 },
    { 0x335b9559, 17 },
    { 0, 0 },
    { 0xd22b955d, 81 },
    { 0, 0 },
    { 0x911b955f, 61 },
    { 0, 0 },
    { 0xd82b9561, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8b9564, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5db9567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3b956b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54b9573, 98 },
    { 0xed6b9574, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8b9578, 88 },
    { 0x506b9579, 95 },
    { 0x75cb9579, 46 },
    { 0xf89b957a, 61 },
    { 0x5bfb957a, 90 },
    { 0x8e4b9578, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703b9587, 76 },
    { 0x9ceb9587, 90 },
    { 0, 0 },
    { 0xda8b958a, 95 },
    { 0, 0 },
    { 0x77bb958c, 95 },
    { 0x5a4b958d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ab9590, 52 },
    { 0, 0 },
    { 0x168b9592, 90 },
    { 0x331b9592, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33b959a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20eb959e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb95a1, 33 },
    { 0, 0 },
    { 0x85cb95a3, 90 },
    { 0, 0 },
    { 0x93ab95a5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7b95aa, 36 },
    { 0xd65b95aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cb95ae, 90 },
    { 0, 0 },
    { 0xc5cb95b0, 66 },
    { 0, 0 },
    { 0x96fb95b2, 23 },
    { 0x494b95b3, 61 },
    { 0x9ceb95b4, 41 },
    { 0x707b95b3, 76 },
    { 0xf09b95b6, 84 },
    { 0, 0 },
    { 0xf77b95b8, 84 },
    { 0x253b95b9, 61 },
    { 0x3dfb95ba, 33 },
    { 0x1b6b95bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcb95c5, 46 },
    { 0x606b95c6, 91 },
    { 0xe22b95c7, 23 },
    { 0xa50b95c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5b95cc, 95 },
    { 0x790b95cc, 90 },
    { 0, 0 },
    { 0xcebb95cf, 76 },
    { 0xfd6b95d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2b95dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb95e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bb95e3, 33 },
    { 0, 0 },
    { 0x37b95e5, 55 },
    { 0, 0 },
    { 0x34b95e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xebb95ea, 81 },
    { 0x20fb95ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x501b95ee, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0b95f4, 90 },
    { 0xa3db95f5, 52 },
    { 0x965b95f4, 99 },
    { 0x8a0b95f7, 55 },
    { 0x141b95f8, 61 },
    { 0xf1b95f4, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fb95fc, 55 },
    { 0xa67b95fc, 95 },
    { 0x92db95fc, 61 },
    { 0x9d6b95fe, 23 },
    { 0, 0 },
    { 0x5c1b9601, 76 },
    { 0x4b9602, 61 },
    { 0, 0 },
    { 0x154b9604, 90 },
    { 0, 0 },
    { 0xe42b9606, 55 },
    { 0xdf7b9607, 33 },
    { 0x542b9608, 76 },
    { 0, 0 },
    { 0xe6ab960a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bb960d, 88 },
    { 0xc68b960e, 33 },
    { 0, 0 },
    { 0xd3fb9610, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517b9615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75b9618, 55 },
    { 0, 0 },
    { 0xc08b961a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6b9622, 90 },
    { 0x805b9623, 97 },
    { 0, 0 },
    { 0x697b9625, 84 },
    { 0x2f4b9626, 61 },
    { 0x4ebb9626, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfb962c, 66 },
    { 0xaf7b962c, 61 },
    { 0x3b2b962e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fb9634, 91 },
    { 0x98b9635, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2b9638, 98 },
    { 0xdf8b9638, 77 },
    { 0x131b963a, 23 },
    { 0, 0 },
    { 0x960b963c, 90 },
    { 0x5a4b963d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4b9644, 34 },
    { 0xf80b9645, 61 },
    { 0xd3eb9646, 84 },
    { 0x556b9647, 90 },
    { 0x747b9648, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2b964b, 75 },
    { 0, 0 },
    { 0x602b964d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356b9654, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebb965e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2b9662, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x829b9665, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b9668, 23 },
    { 0x44ab9669, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648b966d, 90 },
    { 0x3bdb966e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708b9672, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8b9678, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedb967c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ab9685, 98 },
    { 0x18db9686, 90 },
    { 0, 0 },
    { 0x1a4b9688, 55 },
    { 0, 0 },
    { 0x577b968a, 23 },
    { 0xac4b968b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9b9692, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdab96a4, 43 },
    { 0x574b96a4, 61 },
    { 0, 0 },
    { 0xa0ab96a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adb96ab, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb96b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ab96b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbb96b6, 98 },
    { 0x4e5b96b7, 78 },
    { 0, 0 },
    { 0xa48b96b9, 46 },
    { 0xf7b96b9, 34 },
    { 0, 0 },
    { 0x9eb96bc, 98 },
    { 0x846b96bd, 98 },
    { 0xc68b96be, 90 },
    { 0xab96bf, 23 },
    { 0x234b96c0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffb96c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5b96c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2b96cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7b96d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947b96e2, 43 },
    { 0x9f2b96e3, 98 },
    { 0x582b96e4, 96 },
    { 0x8d3b96e2, 61 },
    { 0, 0 },
    { 0xdbab96e7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6b96eb, 52 },
    { 0xbd6b96eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcb96f0, 91 },
    { 0x418b96f0, 33 },
    { 0x276b96f2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931b96f6, 55 },
    { 0x1dbb96f7, 76 },
    { 0xc7db96f8, 91 },
    { 0xc95b96f8, 61 },
    { 0xce2b96fa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37b96fe, 46 },
    { 0x893b96ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eb9702, 98 },
    { 0x5eb9702, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feb9707, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195b970c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380b9715, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfb971a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6b9721, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229b9727, 91 },
    { 0x4e5b9728, 61 },
    { 0xcd0b9727, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cb9732, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eb973b, 23 },
    { 0x747b973b, 95 },
    { 0xe2fb973b, 61 },
    { 0x4b7b973e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448b9749, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbb9752, 61 },
    { 0xa1bb9752, 75 },
    { 0, 0 },
    { 0x616b9755, 78 },
    { 0xf61b9756, 77 },
    { 0, 0 },
    { 0x11bb9758, 84 },
    { 0x306b9759, 33 },
    { 0xc0eb975a, 46 },
    { 0, 0 },
    { 0xff5b975c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fb9760, 98 },
    { 0, 0 },
    { 0x2cb9762, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x46b9765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdb976b, 77 },
    { 0, 0 },
    { 0x607b976d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cb9774, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdb977b, 43 },
    { 0xf14b977b, 61 },
    { 0x78db977c, 34 },
    { 0xe9db977e, 55 },
    { 0xd9bb977f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eb9785, 77 },
    { 0x1b6b9786, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3b978a, 46 },
    { 0x4c3b978a, 77 },
    { 0x5adb978b, 34 },
    { 0xa92b978d, 33 },
    { 0x278b978d, 52 },
    { 0xc75b978e, 34 },
    { 0, 0 },
    { 0x46eb9791, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2b9794, 23 },
    { 0x6eab9795, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61b9799, 46 },
    { 0x1bfb9799, 33 },
    { 0xf79b979a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4b979f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb97a7, 51 },
    { 0, 0 },
    { 0x265b97a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cb97ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0b97b4, 61 },
    { 0x8a2b97b4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cab97b9, 90 },
    { 0x95bb97ba, 91 },
    { 0x65b97bb, 65 },
    { 0xc55b97bc, 61 },
    { 0, 0 },
    { 0xe31b97be, 40 },
    { 0, 0 },
    { 0x85eb97c0, 33 },
    { 0xac8b97c0, 90 },
    { 0xb03b97c0, 33 },
    { 0x903b97c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bb97c7, 90 },
    { 0, 0 },
    { 0xbc0b97c9, 43 },
    { 0x22eb97ca, 61 },
    { 0x1beb97cb, 33 },
    { 0xbe6b97c9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x752b97cf, 33 },
    { 0xb40b97d0, 61 },
    { 0x67db97d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x311b97d4, 78 },
    { 0x235b97d5, 33 },
    { 0x31bb97d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abb97dc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207b97e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fb97e5, 95 },
    { 0xfbbb97e6, 61 },
    { 0x41eb97e6, 61 },
    { 0xd2cb97e5, 19 },
    { 0x426b97e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bb97ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5b97f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604b97f4, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113b97ff, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bb9803, 90 },
    { 0x50cb9803, 90 },
    { 0x413b9803, 87 },
    { 0x7fb9806, 95 },
    { 0, 0 },
    { 0xa4cb9808, 93 },
    { 0, 0 },
    { 0x9dcb980a, 61 },
    { 0xb8ab980b, 76 },
    { 0x9d5b980b, 65 },
    { 0xe69b980d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430b9818, 100 },
    { 0xee1b9818, 23 },
    { 0xce2b981a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddb981e, 33 },
    { 0xf48b981e, 67 },
    { 0x4ddb9820, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fb9824, 95 },
    { 0x185b9825, 23 },
    { 0xa84b9826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55b9829, 23 },
    { 0, 0 },
    { 0x463b982b, 23 },
    { 0, 0 },
    { 0x9ab982d, 61 },
    { 0x53bb982e, 55 },
    { 0x2dbb982e, 76 },
    { 0xd38b9830, 61 },
    { 0x67bb9830, 33 },
    { 0x7ab982f, 33 },
    { 0x81cb982f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2b9836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981b983e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929b9842, 91 },
    { 0x8abb9843, 95 },
    { 0xb8fb9844, 40 },
    { 0xd7cb9845, 67 },
    { 0xb90b9846, 76 },
    { 0xcbeb9846, 34 },
    { 0x584b9846, 56 },
    { 0, 0 },
    { 0x499b984a, 53 },
    { 0, 0 },
    { 0x978b984c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6b9851, 95 },
    { 0xef7b9852, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349b9858, 77 },
    { 0x313b9859, 87 },
    { 0x204b9859, 90 },
    { 0x470b985b, 17 },
    { 0xc6ab985a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2b985f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9b9863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2b9866, 61 },
    { 0xf01b9867, 76 },
    { 0, 0 },
    { 0xaa7b9869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adb986e, 43 },
    { 0xda3b986f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bb9872, 61 },
    { 0, 0 },
    { 0x80ab9874, 98 },
    { 0x8a2b9875, 23 },
    { 0, 0 },
    { 0x6c1b9877, 46 },
    { 0x951b9878, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cb987b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8b9883, 23 },
    { 0x6aeb9883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4db9887, 93 },
    { 0x850b9887, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21b988b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eb988e, 91 },
    { 0x431b988e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fb9894, 98 },
    { 0xd72b9895, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0b989a, 61 },
    { 0xe5fb989b, 61 },
    { 0xd47b989c, 90 },
    { 0x16fb989c, 61 },
    { 0x408b989c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378b98a2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0b98a6, 90 },
    { 0, 0 },
    { 0xa96b98a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5db98ab, 87 },
    { 0x977b98ab, 33 },
    { 0x7fcb98ad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35b98b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901b98b5, 76 },
    { 0xf9db98b6, 77 },
    { 0x47eb98b7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ab98bc, 84 },
    { 0xaf8b98bd, 43 },
    { 0x4dcb98be, 77 },
    { 0xa6b98bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573b98c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21b98c9, 95 },
    { 0x4e0b98c9, 90 },
    { 0x2cb98cb, 61 },
    { 0, 0 },
    { 0xc9b98cd, 98 },
    { 0x5dab98cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ab98d5, 23 },
    { 0x4deb98d5, 61 },
    { 0xb48b98d7, 95 },
    { 0x2a8b98d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45b98df, 90 },
    { 0xa8bb98e0, 46 },
    { 0xf3bb98e1, 98 },
    { 0x614b98df, 33 },
    { 0x959b98e3, 77 },
    { 0x2db98e4, 33 },
    { 0x460b98e5, 61 },
    { 0xbcab98e6, 93 },
    { 0x1d5b98e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabb98ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182b98f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ab98f8, 90 },
    { 0, 0 },
    { 0x837b98fa, 98 },
    { 0x835b98fa, 95 },
    { 0xd1ab98fc, 95 },
    { 0, 0 },
    { 0xc92b98fe, 91 },
    { 0, 0 },
    { 0x270b9900, 55 },
    { 0xeceb9901, 90 },
    { 0, 0 },
    { 0xfd3b9903, 65 },
    { 0xe92b9904, 34 },
    { 0x4a2b9905, 90 },
    { 0x963b9906, 23 },
    { 0x8e0b9907, 22 },
    { 0x526b9905, 55 },
    { 0xaf6b9909, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eb990f, 90 },
    { 0xe5fb990f, 95 },
    { 0xa98b9911, 61 },
    { 0x3f8b9910, 33 },
    { 0x67bb9913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8b9918, 23 },
    { 0, 0 },
    { 0x958b991a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x979b991d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58b9920, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4b9924, 55 },
    { 0x1d1b9924, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03b992a, 33 },
    { 0xc09b992a, 98 },
    { 0x701b992c, 61 },
    { 0x561b992c, 33 },
    { 0xc26b992a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x696b9931, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17b993b, 90 },
    { 0xf50b993c, 95 },
    { 0xd17b993c, 81 },
    { 0x20bb993c, 93 },
    { 0x4a8b993e, 98 },
    { 0x5dab993c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacb9949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb994d, 91 },
    { 0, 0 },
    { 0x2ffb994f, 98 },
    { 0, 0 },
    { 0x415b9951, 43 },
    { 0x2a2b9952, 33 },
    { 0x819b9951, 99 },
    { 0x5bdb9954, 95 },
    { 0xc8cb9955, 33 },
    { 0x21b9956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ab9966, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bb9969, 75 },
    { 0x29cb996a, 99 },
    { 0, 0 },
    { 0x5d4b996c, 34 },
    { 0x76cb996d, 100 },
    { 0, 0 },
    { 0x350b996f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49b9974, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0b9977, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x939b997a, 80 },
    { 0, 0 },
    { 0x554b997c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80b997f, 33 },
    { 0, 0 },
    { 0xb08b9981, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eab9984, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9b998b, 23 },
    { 0x542b998b, 95 },
    { 0x13db998d, 52 },
    { 0x911b998e, 90 },
    { 0, 0 },
    { 0x816b9990, 61 },
    { 0, 0 },
    { 0x94fb9992, 90 },
    { 0, 0 },
    { 0xff6b9994, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471b9999, 43 },
    { 0, 0 },
    { 0xe71b999b, 100 },
    { 0xf6eb999c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57b99a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34b99a8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3b99ac, 37 },
    { 0x574b99ad, 90 },
    { 0, 0 },
    { 0xa76b99af, 33 },
    { 0, 0 },
    { 0xb51b99b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fb99b6, 23 },
    { 0xb89b99b7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887b99c6, 61 },
    { 0, 0 },
    { 0x393b99c8, 90 },
    { 0xf9ab99c8, 95 },
    { 0x6c6b99c9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1b99cd, 90 },
    { 0xd60b99ce, 61 },
    { 0x848b99cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fb99d3, 43 },
    { 0x88cb99d4, 77 },
    { 0x3acb99d3, 100 },
    { 0x11eb99d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cb99db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84b99df, 95 },
    { 0x3b5b99e0, 80 },
    { 0xdf0b99df, 46 },
    { 0x4afb99e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2b99e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9b99eb, 98 },
    { 0xf7ab99ec, 66 },
    { 0xc45b99ed, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afb99f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64b99f9, 66 },
    { 0, 0 },
    { 0xa57b99fb, 91 },
    { 0x6c6b99fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486b9a04, 76 },
    { 0, 0 },
    { 0x949b9a06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40b9a0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37b9a0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbb9a1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03b9a22, 76 },
    { 0x53b9a23, 95 },
    { 0xe4db9a24, 64 },
    { 0, 0 },
    { 0x247b9a26, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cb9a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8b9a31, 66 },
    { 0xe96b9a32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bb9a39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2b9a3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00b9a44, 33 },
    { 0, 0 },
    { 0xb7fb9a46, 33 },
    { 0, 0 },
    { 0x10db9a48, 61 },
    { 0, 0 },
    { 0x7d4b9a4a, 98 },
    { 0x26fb9a4b, 66 },
    { 0x5cdb9a4b, 61 },
    { 0, 0 },
    { 0x2beb9a4e, 96 },
    { 0x349b9a4f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851b9a53, 23 },
    { 0xd0eb9a54, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65db9a59, 100 },
    { 0xbeb9a5a, 33 },
    { 0, 0 },
    { 0x4a6b9a5c, 76 },
    { 0x34db9a5d, 36 },
    { 0x5cb9a5e, 61 },
    { 0x57db9a5f, 95 },
    { 0x60bb9a60, 46 },
    { 0xc05b9a61, 95 },
    { 0, 0 },
    { 0x16b9a63, 98 },
    { 0xf18b9a64, 90 },
    { 0x3e3b9a65, 98 },
    { 0, 0 },
    { 0x388b9a67, 86 },
    { 0xeb9a68, 88 },
    { 0xef2b9a69, 95 },
    { 0x34fb9a6a, 87 },
    { 0x9bdb9a6a, 90 },
    { 0xac6b9a6b, 33 },
    { 0x6d7b9a6d, 61 },
    { 0xdb1b9a6b, 96 },
    { 0xd46b9a6f, 61 },
    { 0xaddb9a70, 36 },
    { 0x2ab9a71, 90 },
    { 0, 0 },
    { 0x5d9b9a73, 75 },
    { 0, 0 },
    { 0x77cb9a75, 34 },
    { 0xc3b9a75, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7b9a79, 33 },
    { 0x62bb9a7a, 100 },
    { 0x9b1b9a7b, 98 },
    { 0x5a8b9a7c, 100 },
    { 0x3fbb9a7c, 87 },
    { 0x2e0b9a7a, 78 },
    { 0xa8eb9a7c, 90 },
    { 0x816b9a80, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95b9a8f, 77 },
    { 0xa71b9a8f, 90 },
    { 0x6bbb9a91, 66 },
    { 0xaa1b9a92, 86 },
    { 0x5f3b9a92, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2b9a9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2eb9aa0, 76 },
    { 0xeebb9aa1, 95 },
    { 0x3e8b9aa2, 34 },
    { 0x93db9aa1, 95 },
    { 0xa36b9aa3, 55 },
    { 0xf03b9aa4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afb9aa9, 33 },
    { 0xf6bb9aaa, 52 },
    { 0x19b9aab, 43 },
    { 0x79eb9aac, 33 },
    { 0x692b9aac, 61 },
    { 0, 0 },
    { 0xccbb9aaf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bb9ab2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eab9ab5, 61 },
    { 0, 0 },
    { 0xf55b9ab7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6b9aba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3b9abe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaab9ac1, 98 },
    { 0, 0 },
    { 0x2ebb9ac3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645b9ac7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab9acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ab9ad6, 33 },
    { 0, 0 },
    { 0xa2cb9ad8, 61 },
    { 0x3a4b9ad9, 61 },
    { 0xad6b9ad9, 33 },
    { 0, 0 },
    { 0x906b9adc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acb9ae0, 91 },
    { 0xffbb9ae1, 98 },
    { 0xf0b9ae1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01b9aea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7b9af1, 95 },
    { 0, 0 },
    { 0x5e3b9af3, 61 },
    { 0x7e2b9af4, 40 },
    { 0, 0 },
    { 0xf29b9af6, 55 },
    { 0x8f9b9af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2b9afb, 33 },
    { 0x654b9afb, 90 },
    { 0x7bdb9afd, 85 },
    { 0xa6fb9afb, 90 },
    { 0x4a6b9aff, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bb9b02, 77 },
    { 0x287b9b02, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26b9b0b, 56 },
    { 0, 0 },
    { 0xd63b9b0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ab9b13, 90 },
    { 0x2db9b14, 85 },
    { 0x161b9b15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2b9b1c, 90 },
    { 0xbd4b9b1c, 66 },
    { 0x4c7b9b1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9b9b26, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bb9b2a, 90 },
    { 0, 0 },
    { 0xbb2b9b2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4b9b31, 98 },
    { 0x77db9b32, 40 },
    { 0x38fb9b32, 90 },
    { 0x793b9b34, 83 },
    { 0x2ecb9b34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47b9b3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15b9b3f, 67 },
    { 0, 0 },
    { 0x81bb9b41, 61 },
    { 0x444b9b42, 74 },
    { 0x760b9b42, 66 },
    { 0, 0 },
    { 0x490b9b45, 61 },
    { 0x5b5b9b46, 93 },
    { 0x389b9b47, 90 },
    { 0xc86b9b48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7b9b52, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801b9b56, 65 },
    { 0x2feb9b56, 78 },
    { 0, 0 },
    { 0x757b9b59, 76 },
    { 0, 0 },
    { 0xc1eb9b5b, 98 },
    { 0, 0 },
    { 0x4a0b9b5d, 61 },
    { 0x593b9b5e, 55 },
    { 0, 0 },
    { 0x89b9b60, 91 },
    { 0, 0 },
    { 0x948b9b62, 61 },
    { 0, 0 },
    { 0xb1cb9b64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cb9b69, 95 },
    { 0xac3b9b6a, 33 },
    { 0x797b9b6b, 68 },
    { 0xe21b9b69, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bb9b74, 97 },
    { 0, 0 },
    { 0xeb9b76, 48 },
    { 0x3eeb9b77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78b9b7f, 84 },
    { 0xf50b9b7f, 90 },
    { 0x806b9b80, 46 },
    { 0xc57b9b82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38b9b8a, 48 },
    { 0x929b9b8b, 90 },
    { 0x6fcb9b8a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902b9b90, 27 },
    { 0, 0 },
    { 0x511b9b92, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0b9b9c, 52 },
    { 0xb38b9b9d, 90 },
    { 0xaafb9b9e, 47 },
    { 0x698b9b9e, 46 },
    { 0x39bb9b9d, 93 },
    { 0x36fb9ba1, 33 },
    { 0x13b9ba1, 95 },
    { 0x1bcb9b9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221b9ba7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceab9bb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21db9bb9, 33 },
    { 0x245b9bba, 90 },
    { 0x142b9bbb, 33 },
    { 0x92ab9bba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfab9bc5, 90 },
    { 0xfa7b9bc5, 61 },
    { 0xdeeb9bc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ab9bca, 23 },
    { 0x3a4b9bcb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611b9bd7, 98 },
    { 0x539b9bd8, 63 },
    { 0x218b9bd7, 91 },
    { 0xddab9bd8, 100 },
    { 0, 0 },
    { 0x73b9bdc, 95 },
    { 0, 0 },
    { 0xae6b9bde, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5b9be3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x801b9be6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908b9bea, 76 },
    { 0xfd3b9bea, 33 },
    { 0x3feb9bec, 76 },
    { 0x94cb9beb, 46 },
    { 0x62b9bee, 66 },
    { 0, 0 },
    { 0x5d3b9bf0, 53 },
    { 0xa90b9bf1, 55 },
    { 0, 0 },
    { 0x542b9bf3, 84 },
    { 0x3d4b9bf3, 91 },
    { 0x78fb9bf3, 46 },
    { 0x862b9bf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x181b9bf9, 75 },
    { 0x68ab9bfa, 99 },
    { 0x6f3b9bfa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcb9bff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1b9c02, 33 },
    { 0x7f3b9c02, 90 },
    { 0x44eb9c04, 80 },
    { 0, 0 },
    { 0xacb9c06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620b9c0f, 95 },
    { 0x8d1b9c10, 93 },
    { 0x833b9c0f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28b9c15, 93 },
    { 0xc64b9c16, 61 },
    { 0x399b9c17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbb9c1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38db9c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11b9c21, 33 },
    { 0x76b9c21, 61 },
    { 0x19eb9c22, 78 },
    { 0x198b9c22, 34 },
    { 0xadb9c23, 33 },
    { 0x1f0b9c26, 33 },
    { 0x1b0b9c26, 90 },
    { 0xc18b9c28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4b9c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49b9c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb9c36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8b9c3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42b9c3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3b9c4e, 55 },
    { 0x4f8b9c4f, 85 },
    { 0, 0 },
    { 0xeb3b9c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290b9c5a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477b9c5e, 76 },
    { 0xb5cb9c5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdb9c67, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129b9c6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bb9c71, 23 },
    { 0xf4db9c71, 23 },
    { 0xf72b9c72, 88 },
    { 0, 0 },
    { 0x5b2b9c75, 33 },
    { 0, 0 },
    { 0xbe3b9c77, 90 },
    { 0x9ceb9c78, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355b9c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccb9c8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cb9c94, 34 },
    { 0x76cb9c95, 52 },
    { 0x51fb9c96, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7b9c9e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0b9ca2, 55 },
    { 0x47ab9ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bb9ca6, 66 },
    { 0, 0 },
    { 0x3aab9ca8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71b9cab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2b9cae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedb9cb1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x839b9cb4, 76 },
    { 0, 0 },
    { 0x412b9cb6, 46 },
    { 0xc3db9cb6, 95 },
    { 0x601b9cb8, 61 },
    { 0x9fb9cb9, 52 },
    { 0xd37b9cb9, 40 },
    { 0xefcb9cba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6b9cc0, 95 },
    { 0, 0 },
    { 0xb1b9cc2, 91 },
    { 0x5f3b9cc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bb9cc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350b9ccd, 61 },
    { 0xc0db9ccd, 23 },
    { 0, 0 },
    { 0xf2eb9cd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8db9cd3, 61 },
    { 0xe3eb9cd4, 46 },
    { 0, 0 },
    { 0x47bb9cd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cb9cdd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efb9ce0, 61 },
    { 0xd50b9ce1, 90 },
    { 0, 0 },
    { 0x386b9ce3, 61 },
    { 0xe70b9ce4, 88 },
    { 0xd10b9ce4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4b9ceb, 33 },
    { 0x2b0b9cec, 98 },
    { 0x709b9ced, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633b9cf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbb9cf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bb9cfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66b9cfd, 67 },
    { 0, 0 },
    { 0x685b9cff, 90 },
    { 0x348b9d00, 95 },
    { 0, 0 },
    { 0xb29b9d02, 34 },
    { 0x146b9d02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630b9d08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3db9d10, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbb9d15, 23 },
    { 0xf45b9d15, 55 },
    { 0x1b9d17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563b9d1b, 68 },
    { 0xbd3b9d1b, 52 },
    { 0x2d3b9d1d, 98 },
    { 0xb1eb9d1c, 55 },
    { 0x681b9d1f, 90 },
    { 0xdebb9d20, 43 },
    { 0xdf7b9d21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ab9d24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcb9d2a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ab9d2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x587b9d30, 23 },
    { 0x225b9d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ab9d34, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51b9d37, 78 },
    { 0, 0 },
    { 0xefdb9d39, 40 },
    { 0, 0 },
    { 0xb2cb9d3b, 65 },
    { 0x877b9d3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49b9d40, 55 },
    { 0xa15b9d41, 23 },
    { 0x78eb9d40, 55 },
    { 0x71bb9d41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226b9d48, 61 },
    { 0x238b9d49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24b9d4c, 95 },
    { 0xad2b9d4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5b9d59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57b9d5d, 67 },
    { 0xae7b9d5e, 90 },
    { 0xefdb9d5f, 33 },
    { 0x757b9d60, 61 },
    { 0x444b9d5f, 90 },
    { 0, 0 },
    { 0x9d4b9d63, 61 },
    { 0x4ceb9d63, 61 },
    { 0x7db9d65, 75 },
    { 0, 0 },
    { 0xfa4b9d67, 87 },
    { 0, 0 },
    { 0xdc6b9d69, 99 },
    { 0xee9b9d6a, 91 },
    { 0x8d0b9d6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61b9d6f, 76 },
    { 0xbe7b9d70, 43 },
    { 0xdd5b9d71, 90 },
    { 0x571b9d6f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcb9d78, 61 },
    { 0xf7fb9d79, 98 },
    { 0xd10b9d7a, 95 },
    { 0x5d9b9d78, 90 },
    { 0, 0 },
    { 0x497b9d7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bb9d80, 98 },
    { 0x957b9d80, 17 },
    { 0xa37b9d80, 55 },
    { 0xaa8b9d83, 98 },
    { 0x6e3b9d83, 98 },
    { 0x1fdb9d85, 61 },
    { 0x628b9d86, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ab9d8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eb9d8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877b9d92, 100 },
    { 0x297b9d93, 99 },
    { 0x532b9d93, 55 },
    { 0x1eab9d93, 98 },
    { 0x57b9d96, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eb9d9f, 93 },
    { 0xc4eb9da0, 65 },
    { 0xb01b9da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bb9da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9b9dab, 95 },
    { 0x20cb9dab, 33 },
    { 0, 0 },
    { 0x8e4b9dae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ab9db2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x496b9db5, 81 },
    { 0xc3bb9db5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccb9db9, 23 },
    { 0xe34b9db9, 55 },
    { 0xb02b9dbb, 90 },
    { 0x250b9dbb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6b9dc1, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ab9dc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7db9dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0b9dcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bb9dd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b9dd5, 43 },
    { 0xb89b9dd6, 43 },
    { 0x337b9dd7, 87 },
    { 0x9d8b9dd8, 93 },
    { 0x9ecb9dd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815b9dde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ab9dec, 51 },
    { 0x6a0b9ded, 17 },
    { 0xadbb9dee, 61 },
    { 0xa84b9def, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7b9df6, 61 },
    { 0x620b9df6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x647b9dfa, 33 },
    { 0x82cb9dfb, 85 },
    { 0xfbbb9dfc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eb9e02, 76 },
    { 0x156b9e03, 33 },
    { 0x468b9e04, 23 },
    { 0x182b9e03, 66 },
    { 0x37ab9e06, 85 },
    { 0x916b9e04, 93 },
    { 0x462b9e08, 76 },
    { 0x402b9e09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1b9e0d, 90 },
    { 0x948b9e0d, 77 },
    { 0xe6bb9e0f, 52 },
    { 0x6ddb9e0d, 17 },
    { 0xa1bb9e11, 23 },
    { 0xae9b9e0e, 76 },
    { 0x6e5b9e13, 61 },
    { 0x4b3b9e14, 43 },
    { 0xa84b9e0d, 76 },
    { 0x832b9e16, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6b9e1a, 33 },
    { 0xbbdb9e1a, 66 },
    { 0xdb4b9e1b, 34 },
    { 0x3abb9e1d, 90 },
    { 0xe56b9e1e, 95 },
    { 0, 0 },
    { 0x10b9e20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7b9e23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5b9e29, 33 },
    { 0x1ceb9e29, 98 },
    { 0x49db9e2b, 23 },
    { 0xa41b9e2b, 66 },
    { 0xf74b9e2c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00b9e31, 75 },
    { 0x406b9e32, 33 },
    { 0, 0 },
    { 0xb9cb9e34, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01b9e39, 76 },
    { 0, 0 },
    { 0x105b9e3b, 91 },
    { 0x29bb9e3c, 93 },
    { 0x314b9e3d, 24 },
    { 0x659b9e3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeb9e41, 100 },
    { 0xcd3b9e42, 34 },
    { 0, 0 },
    { 0xefcb9e44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcb9e49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67b9e4d, 17 },
    { 0x6b3b9e4d, 46 },
    { 0, 0 },
    { 0x273b9e50, 90 },
    { 0, 0 },
    { 0xf36b9e52, 43 },
    { 0x4b5b9e52, 64 },
    { 0x56eb9e52, 76 },
    { 0, 0 },
    { 0x81ab9e56, 90 },
    { 0, 0 },
    { 0x7ccb9e58, 33 },
    { 0x555b9e59, 34 },
    { 0xdfb9e59, 90 },
    { 0xbf2b9e5b, 23 },
    { 0x28eb9e5b, 69 },
    { 0xb43b9e5d, 81 },
    { 0x3d1b9e5e, 61 },
    { 0x8c0b9e5f, 91 },
    { 0xd26b9e60, 23 },
    { 0xadfb9e5e, 95 },
    { 0x691b9e62, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3b9e65, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3b9e6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eb9e6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cb9e75, 66 },
    { 0, 0 },
    { 0x54bb9e77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453b9e7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x846b9e7f, 98 },
    { 0x4deb9e80, 90 },
    { 0xc69b9e80, 81 },
    { 0x12bb9e80, 87 },
    { 0, 0 },
    { 0xbeb9e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28b9e8b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ab9e95, 90 },
    { 0, 0 },
    { 0x25b9e97, 95 },
    { 0x425b9e98, 90 },
    { 0xb32b9e97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fb9e9e, 76 },
    { 0xd31b9e9e, 33 },
    { 0, 0 },
    { 0x784b9ea1, 95 },
    { 0, 0 },
    { 0x87fb9ea3, 100 },
    { 0x6fdb9ea4, 67 },
    { 0x6eeb9ea4, 93 },
    { 0x2ab9ea4, 23 },
    { 0xe85b9ea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10b9eaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09b9ead, 61 },
    { 0x1cdb9eae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5b9eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x799b9eb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57db9eb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x972b9ebc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12b9ebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421b9ec6, 76 },
    { 0xc69b9ec7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4b9ece, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebb9ed1, 58 },
    { 0xc31b9ed1, 90 },
    { 0x844b9ed3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7b9ed6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x499b9ed9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647b9edd, 61 },
    { 0xd2b9edd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3b9ee1, 61 },
    { 0xcfbb9ee1, 100 },
    { 0xa70b9ee2, 70 },
    { 0xf3bb9ee4, 88 },
    { 0xc5b9ee4, 55 },
    { 0x84fb9ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eb9ee9, 61 },
    { 0, 0 },
    { 0x4dfb9eeb, 33 },
    { 0xb73b9eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8b9eef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfb9ef2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ab9efb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bb9eff, 93 },
    { 0x2a6b9f00, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561b9f06, 98 },
    { 0, 0 },
    { 0xc30b9f08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbb9f19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aeb9f1d, 97 },
    { 0x50db9f1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ab9f22, 99 },
    { 0, 0 },
    { 0x199b9f24, 76 },
    { 0xdf1b9f25, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7b9f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cb9f30, 55 },
    { 0, 0 },
    { 0xc67b9f32, 64 },
    { 0x2ebb9f32, 78 },
    { 0x86ab9f34, 55 },
    { 0xbab9f34, 90 },
    { 0x81bb9f36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3b9f39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50b9f3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1b9f3f, 23 },
    { 0xa29b9f40, 95 },
    { 0x93cb9f3f, 90 },
    { 0xda0b9f42, 90 },
    { 0x40fb9f3f, 17 },
    { 0xf7b9f44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23b9f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeb9f4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afb9f55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfb9f5a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09b9f5e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55b9f62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x253b9f65, 93 },
    { 0x441b9f66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0b9f6e, 90 },
    { 0, 0 },
    { 0xc0ab9f70, 48 },
    { 0, 0 },
    { 0x78db9f72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eb9f76, 84 },
    { 0, 0 },
    { 0xfcfb9f78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7b9f7b, 33 },
    { 0xc42b9f7c, 88 },
    { 0xb3fb9f7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52b9f80, 66 },
    { 0x28db9f81, 55 },
    { 0x1c3b9f82, 61 },
    { 0x4d3b9f83, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fb9f87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bb9f8b, 56 },
    { 0xf7ab9f8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fb9f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eb9f97, 55 },
    { 0x290b9f98, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117b9f9e, 95 },
    { 0x607b9f9e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09b9fa5, 90 },
    { 0xc49b9fa6, 80 },
    { 0, 0 },
    { 0xfeb9fa8, 55 },
    { 0xfc8b9fa8, 76 },
    { 0xc79b9faa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x905b9fad, 93 },
    { 0xf0cb9fae, 84 },
    { 0, 0 },
    { 0x41ab9fb0, 95 },
    { 0x7bcb9fb1, 23 },
    { 0x910b9fb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eeb9fbc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58b9fc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3b9fc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fb9fcc, 55 },
    { 0xa76b9fcc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74eb9fd2, 43 },
    { 0xe71b9fd3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4b9fd6, 66 },
    { 0x556b9fd7, 86 },
    { 0x48db9fd8, 61 },
    { 0x91db9fd9, 78 },
    { 0xf91b9fd8, 34 },
    { 0x277b9fda, 99 },
    { 0x5edb9fdc, 33 },
    { 0x314b9fdc, 90 },
    { 0x697b9fda, 90 },
    { 0x84b9fdb, 61 },
    { 0x1fbb9fdb, 61 },
    { 0x3f1b9fe1, 43 },
    { 0, 0 },
    { 0x2ceb9fe3, 87 },
    { 0xd17b9fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5b9feb, 61 },
    { 0x1a8b9feb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x940b9fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228b9ff3, 93 },
    { 0x919b9ff4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x48eb9ff7, 61 },
    { 0, 0 },
    { 0x265b9ff9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3b9ffc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bba001, 33 },
    { 0x6aaba002, 98 },
    { 0x38dba003, 96 },
    { 0xd9cba002, 98 },
    { 0, 0 },
    { 0xc80ba006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffba009, 98 },
    { 0x86aba00a, 64 },
    { 0x20bba00a, 98 },
    { 0x38cba00c, 43 },
    { 0x4ba009, 75 },
    { 0x6ecba00e, 33 },
    { 0x19fba00e, 66 },
    { 0, 0 },
    { 0x54aba011, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bba014, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cba018, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8ba020, 55 },
    { 0x282ba020, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bba024, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bba029, 55 },
    { 0, 0 },
    { 0x3eba02b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48ba02e, 23 },
    { 0, 0 },
    { 0x71ba030, 61 },
    { 0, 0 },
    { 0x401ba032, 40 },
    { 0, 0 },
    { 0xdf4ba034, 78 },
    { 0xf23ba035, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x540ba038, 43 },
    { 0xfc9ba039, 90 },
    { 0x40fba03a, 61 },
    { 0x73aba03b, 95 },
    { 0, 0 },
    { 0x620ba03d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x431ba040, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cba044, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bba048, 34 },
    { 0x3c4ba049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791ba04e, 33 },
    { 0, 0 },
    { 0x16eba050, 90 },
    { 0xc92ba051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535ba05d, 55 },
    { 0x48ba05e, 98 },
    { 0x75eba05f, 46 },
    { 0x3faba05f, 95 },
    { 0, 0 },
    { 0x855ba062, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8ba065, 93 },
    { 0x3eaba066, 33 },
    { 0xdf4ba067, 98 },
    { 0, 0 },
    { 0x8b1ba069, 98 },
    { 0xfd7ba06a, 93 },
    { 0x4c3ba06a, 82 },
    { 0x6aaba069, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dba070, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eba077, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602ba07b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973ba081, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3ba086, 85 },
    { 0x5c7ba087, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9ba090, 14 },
    { 0x7ccba091, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ba094, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaba098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8ba0a1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5ba0a4, 33 },
    { 0, 0 },
    { 0xc31ba0a6, 52 },
    { 0xe7bba0a7, 95 },
    { 0, 0 },
    { 0xed0ba0a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492ba0af, 52 },
    { 0xc97ba0af, 33 },
    { 0x321ba0b1, 76 },
    { 0xd74ba0b0, 9 },
    { 0, 0 },
    { 0x828ba0b4, 61 },
    { 0x3ceba0b4, 75 },
    { 0, 0 },
    { 0xaf2ba0b7, 33 },
    { 0xbf0ba0b8, 77 },
    { 0xc0cba0b7, 77 },
    { 0x170ba0ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ba0bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639ba0c4, 90 },
    { 0x46eba0c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89ba0cc, 61 },
    { 0, 0 },
    { 0x71eba0ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7ba0d4, 33 },
    { 0, 0 },
    { 0x1d5ba0d6, 68 },
    { 0, 0 },
    { 0xc07ba0d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cba0df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cba0e3, 91 },
    { 0xfc3ba0e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efba0e9, 23 },
    { 0x29dba0ea, 95 },
    { 0x1edba0eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8ba0ef, 23 },
    { 0xc20ba0f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaba0f3, 90 },
    { 0x91ba0f4, 61 },
    { 0xa77ba0f4, 98 },
    { 0xfa2ba0f3, 98 },
    { 0xe5dba0f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ba0fe, 33 },
    { 0xa94ba0fe, 33 },
    { 0xaf9ba100, 46 },
    { 0x759ba0fe, 40 },
    { 0xca0ba102, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x49aba105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eba10b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96eba110, 98 },
    { 0x367ba111, 81 },
    { 0, 0 },
    { 0x3adba113, 95 },
    { 0, 0 },
    { 0xae1ba115, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1ba119, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbba11e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eba121, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaba127, 46 },
    { 0xd69ba127, 90 },
    { 0xeba127, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6ba12d, 92 },
    { 0x574ba12e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ba131, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48ba134, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76ba139, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fba13c, 78 },
    { 0x2a4ba13d, 92 },
    { 0x450ba13d, 34 },
    { 0xe2aba13c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3ba145, 78 },
    { 0x725ba146, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dba14b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19ba14e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4ba153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ba15b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61ba15f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabba167, 33 },
    { 0, 0 },
    { 0x1d7ba169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ba171, 76 },
    { 0, 0 },
    { 0x213ba173, 43 },
    { 0xaccba174, 33 },
    { 0x2c4ba174, 69 },
    { 0x419ba175, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104ba17a, 68 },
    { 0xe8dba17a, 95 },
    { 0xa73ba17c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dba180, 34 },
    { 0xb6cba181, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daba184, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7ba189, 95 },
    { 0x7bfba18a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ba18f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbba192, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deba198, 19 },
    { 0x955ba198, 61 },
    { 0, 0 },
    { 0xf8ba19b, 95 },
    { 0x5e8ba19c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07ba1a0, 19 },
    { 0, 0 },
    { 0x27bba1a2, 98 },
    { 0, 0 },
    { 0xf1aba1a4, 95 },
    { 0x101ba1a5, 33 },
    { 0xa87ba1a6, 23 },
    { 0xf93ba1a7, 33 },
    { 0xdcba1a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dba1af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9ba1ba, 23 },
    { 0xc11ba1bb, 100 },
    { 0x33ba1ba, 90 },
    { 0x63ba1bd, 87 },
    { 0xe7cba1be, 33 },
    { 0xb90ba1bf, 61 },
    { 0x55bba1bd, 61 },
    { 0x596ba1c1, 90 },
    { 0xee8ba1c2, 98 },
    { 0x5ebba1c1, 34 },
    { 0, 0 },
    { 0x3edba1c5, 98 },
    { 0, 0 },
    { 0xb47ba1c7, 76 },
    { 0x486ba1c7, 46 },
    { 0xd9ba1c9, 61 },
    { 0xccba1ca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ba1cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cba1d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ba1d5, 23 },
    { 0xd88ba1d6, 23 },
    { 0x1c8ba1d7, 33 },
    { 0x664ba1d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fba1dc, 95 },
    { 0x632ba1dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88ba1e1, 95 },
    { 0xae3ba1e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409ba1e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ba1eb, 66 },
    { 0x42eba1ec, 100 },
    { 0x493ba1eb, 93 },
    { 0x8ffba1ee, 90 },
    { 0, 0 },
    { 0xb00ba1f0, 53 },
    { 0x537ba1f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x458ba1f4, 95 },
    { 0x1eeba1f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ba1f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ba1fe, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ba201, 90 },
    { 0xb14ba202, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919ba20c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x727ba20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x169ba212, 99 },
    { 0xd73ba213, 99 },
    { 0xf65ba213, 76 },
    { 0xddbba215, 76 },
    { 0x818ba216, 34 },
    { 0x25bba216, 66 },
    { 0x8a4ba218, 91 },
    { 0x836ba217, 98 },
    { 0x561ba218, 95 },
    { 0xa58ba215, 98 },
    { 0x166ba217, 33 },
    { 0x205ba219, 43 },
    { 0x4d4ba215, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59ba221, 90 },
    { 0x446ba221, 55 },
    { 0x214ba223, 33 },
    { 0x3ccba222, 61 },
    { 0xf62ba225, 33 },
    { 0x70ba225, 76 },
    { 0xaf7ba227, 23 },
    { 0xe9bba222, 83 },
    { 0xdd0ba226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109ba22e, 95 },
    { 0xa76ba22f, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0x513ba232, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857ba237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865ba23b, 33 },
    { 0, 0 },
    { 0xda5ba23d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9ba241, 83 },
    { 0, 0 },
    { 0x5f8ba243, 95 },
    { 0x74ba244, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4aba249, 90 },
    { 0x6ccba249, 33 },
    { 0x1cfba24b, 76 },
    { 0xa60ba24c, 33 },
    { 0, 0 },
    { 0xdcdba24e, 69 },
    { 0, 0 },
    { 0xd21ba250, 81 },
    { 0x7e5ba251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16ba254, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ba257, 61 },
    { 0x2afba258, 51 },
    { 0xa1fba257, 33 },
    { 0xc11ba25a, 65 },
    { 0x858ba25b, 87 },
    { 0, 0 },
    { 0xab1ba25d, 91 },
    { 0, 0 },
    { 0x56aba25f, 61 },
    { 0x649ba260, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7aba264, 61 },
    { 0x1e9ba264, 87 },
    { 0x373ba266, 52 },
    { 0x5e1ba267, 23 },
    { 0x89bba264, 33 },
    { 0xae4ba269, 90 },
    { 0x3adba26a, 33 },
    { 0, 0 },
    { 0xa31ba26c, 95 },
    { 0x4c1ba26d, 19 },
    { 0xabcba26e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdba27a, 23 },
    { 0x206ba27b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecba27e, 76 },
    { 0xae3ba27e, 93 },
    { 0xc50ba280, 90 },
    { 0xbe0ba281, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19ba284, 68 },
    { 0x9d6ba284, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192ba28b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bba294, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cba297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bba29a, 55 },
    { 0xee6ba29b, 90 },
    { 0xf88ba29b, 66 },
    { 0, 0 },
    { 0xddfba29e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ba2a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2ba2ac, 95 },
    { 0, 0 },
    { 0xc7cba2ae, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243ba2b2, 66 },
    { 0x7a8ba2b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7ba2b8, 43 },
    { 0xe43ba2b8, 19 },
    { 0x2ddba2ba, 23 },
    { 0x796ba2bb, 51 },
    { 0, 0 },
    { 0x532ba2bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82ba2c0, 90 },
    { 0xf48ba2c1, 43 },
    { 0x3fba2c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fba2c7, 90 },
    { 0xe28ba2c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838ba2cc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5ba2cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fba2d2, 61 },
    { 0, 0 },
    { 0x19dba2d4, 55 },
    { 0xfc1ba2d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacba2dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944ba2e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ba2e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bba2e9, 46 },
    { 0x48aba2e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121ba2ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ba2f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fba2fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940ba300, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ba304, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376ba30a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7ba30e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaba311, 82 },
    { 0xe80ba311, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09ba316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaba31b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3ba31e, 90 },
    { 0xd7cba31f, 95 },
    { 0xd96ba31e, 76 },
    { 0x1b8ba321, 34 },
    { 0, 0 },
    { 0x6b7ba323, 61 },
    { 0, 0 },
    { 0x11ba325, 33 },
    { 0x7b2ba325, 90 },
    { 0, 0 },
    { 0xf0aba328, 90 },
    { 0xb4cba329, 87 },
    { 0xd4dba32a, 55 },
    { 0x4b6ba32b, 59 },
    { 0xd86ba329, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ba331, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aba337, 34 },
    { 0, 0 },
    { 0x6d7ba339, 84 },
    { 0x21eba339, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ba33d, 33 },
    { 0x9d0ba33e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fba341, 95 },
    { 0, 0 },
    { 0x401ba343, 33 },
    { 0, 0 },
    { 0xaeeba345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8ba348, 38 },
    { 0, 0 },
    { 0x70bba34a, 38 },
    { 0, 0 },
    { 0xdebba34c, 98 },
    { 0x339ba34c, 76 },
    { 0xb5cba34e, 33 },
    { 0, 0 },
    { 0x83ba350, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2ba356, 95 },
    { 0xd5eba357, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714ba360, 77 },
    { 0x2e3ba361, 100 },
    { 0, 0 },
    { 0xe1aba363, 55 },
    { 0, 0 },
    { 0xe69ba365, 90 },
    { 0x981ba366, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x876ba369, 90 },
    { 0, 0 },
    { 0x38cba36b, 33 },
    { 0x345ba36c, 76 },
    { 0xa43ba36c, 23 },
    { 0x502ba36d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ba371, 68 },
    { 0x3c4ba372, 76 },
    { 0x27aba371, 55 },
    { 0xab4ba373, 33 },
    { 0x5baba375, 76 },
    { 0xf9fba371, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bba37a, 61 },
    { 0x8faba37a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebba37e, 96 },
    { 0x578ba37f, 61 },
    { 0xcc7ba380, 68 },
    { 0x840ba381, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ba384, 91 },
    { 0xfe9ba385, 95 },
    { 0, 0 },
    { 0x404ba387, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7ba391, 61 },
    { 0, 0 },
    { 0x83cba393, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ba397, 90 },
    { 0x329ba398, 90 },
    { 0, 0 },
    { 0x44bba39a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4ba39d, 93 },
    { 0x122ba39e, 91 },
    { 0xe37ba39d, 33 },
    { 0, 0 },
    { 0x51fba3a1, 61 },
    { 0x785ba3a2, 58 },
    { 0x36eba3a3, 46 },
    { 0, 0 },
    { 0xa84ba3a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403ba3aa, 98 },
    { 0x649ba3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x116ba3ae, 34 },
    { 0, 0 },
    { 0x23eba3b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468ba3bb, 34 },
    { 0x906ba3bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ba3c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2ba3c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ba3c8, 98 },
    { 0, 0 },
    { 0x382ba3ca, 69 },
    { 0xf9bba3cb, 33 },
    { 0xff3ba3cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4ba3d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48ba3d3, 85 },
    { 0x115ba3d4, 55 },
    { 0, 0 },
    { 0x9b2ba3d6, 77 },
    { 0x45eba3d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39ba3dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bba3e1, 98 },
    { 0x5acba3e1, 95 },
    { 0, 0 },
    { 0xe8fba3e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abba3ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7ba3ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5ba3f2, 61 },
    { 0, 0 },
    { 0x23eba3f4, 90 },
    { 0x1d2ba3f4, 90 },
    { 0xdc5ba3f6, 23 },
    { 0x1d4ba3f7, 55 },
    { 0xbeeba3f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ba401, 34 },
    { 0x286ba402, 90 },
    { 0x1deba402, 95 },
    { 0xec0ba401, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aba407, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bba40b, 23 },
    { 0, 0 },
    { 0x8d1ba40d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4ba412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ba415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5ba418, 23 },
    { 0x15dba419, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ba41d, 90 },
    { 0, 0 },
    { 0x37cba41f, 85 },
    { 0x705ba41f, 99 },
    { 0x77fba421, 68 },
    { 0x19aba422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888ba425, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3ba428, 95 },
    { 0x6a0ba429, 94 },
    { 0xc4dba42a, 93 },
    { 0, 0 },
    { 0x7a9ba42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8ba437, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bba43d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ba440, 17 },
    { 0xf51ba441, 61 },
    { 0xffbba441, 91 },
    { 0x5b3ba442, 90 },
    { 0xd03ba444, 93 },
    { 0, 0 },
    { 0x258ba446, 43 },
    { 0x6ba447, 98 },
    { 0x854ba448, 33 },
    { 0x12fba449, 23 },
    { 0, 0 },
    { 0x5c9ba44b, 33 },
    { 0x988ba44c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4ba44f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdba454, 75 },
    { 0x154ba454, 95 },
    { 0xbbeba455, 55 },
    { 0x2f0ba457, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4ba45b, 68 },
    { 0, 0 },
    { 0xea1ba45d, 95 },
    { 0, 0 },
    { 0x9b3ba45f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08ba464, 46 },
    { 0xe7eba465, 85 },
    { 0xe35ba466, 55 },
    { 0, 0 },
    { 0xaaba468, 55 },
    { 0x71fba468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ba46c, 61 },
    { 0x1acba46d, 98 },
    { 0x8abba46e, 33 },
    { 0x59ba46d, 76 },
    { 0, 0 },
    { 0xfa3ba471, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ba47b, 99 },
    { 0xc5fba47c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ba483, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9ba486, 66 },
    { 0x9c6ba486, 53 },
    { 0xba0ba488, 23 },
    { 0, 0 },
    { 0x824ba48a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2ba48d, 55 },
    { 0xe8aba48e, 90 },
    { 0, 0 },
    { 0x5aba490, 84 },
    { 0xac0ba490, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478ba498, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aba49c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ba4a0, 33 },
    { 0, 0 },
    { 0x633ba4a2, 98 },
    { 0xfb1ba4a3, 77 },
    { 0x1d6ba4a4, 77 },
    { 0x74bba4a4, 98 },
    { 0xaabba4a6, 95 },
    { 0x852ba4a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x801ba4aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fba4ad, 33 },
    { 0xbc5ba4ae, 66 },
    { 0, 0 },
    { 0x353ba4b0, 33 },
    { 0x26fba4b1, 33 },
    { 0xf25ba4b2, 44 },
    { 0, 0 },
    { 0xf25ba4b4, 55 },
    { 0x5d5ba4b5, 76 },
    { 0, 0 },
    { 0x106ba4b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ba4bc, 100 },
    { 0xd33ba4bd, 90 },
    { 0xef8ba4bc, 83 },
    { 0xf63ba4bf, 90 },
    { 0x9e2ba4bf, 90 },
    { 0, 0 },
    { 0x16eba4c2, 78 },
    { 0xbcdba4c2, 95 },
    { 0x5d5ba4c4, 23 },
    { 0x49eba4c5, 84 },
    { 0xc7bba4c5, 34 },
    { 0x9b3ba4c2, 90 },
    { 0xeeaba4c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaba4ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85ba4d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9ba4d5, 76 },
    { 0, 0 },
    { 0x376ba4d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8ba4df, 33 },
    { 0x943ba4e0, 43 },
    { 0x97bba4e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1aba4e4, 23 },
    { 0, 0 },
    { 0x44dba4e6, 97 },
    { 0x538ba4e7, 87 },
    { 0x9bcba4e7, 55 },
    { 0, 0 },
    { 0xb7fba4ea, 23 },
    { 0, 0 },
    { 0xb34ba4ec, 61 },
    { 0xa37ba4ed, 23 },
    { 0x2daba4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878ba4f5, 35 },
    { 0, 0 },
    { 0x5e7ba4f7, 87 },
    { 0xda3ba4f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293ba4fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898ba504, 61 },
    { 0xf76ba505, 33 },
    { 0x2ba506, 81 },
    { 0xf2ba507, 17 },
    { 0x8b4ba508, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fba50b, 90 },
    { 0xd64ba50c, 33 },
    { 0x7a3ba50d, 91 },
    { 0xbf0ba50e, 99 },
    { 0xdb6ba50d, 43 },
    { 0x22dba510, 43 },
    { 0x603ba511, 81 },
    { 0xda4ba512, 90 },
    { 0x5aba512, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178ba517, 84 },
    { 0, 0 },
    { 0xf51ba519, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215ba51d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2ba522, 78 },
    { 0x9edba522, 61 },
    { 0x6fcba523, 95 },
    { 0x4a4ba525, 33 },
    { 0xa5ba525, 33 },
    { 0x1d6ba526, 33 },
    { 0xd2eba522, 95 },
    { 0xc71ba529, 61 },
    { 0x37aba52a, 90 },
    { 0x84bba52b, 33 },
    { 0xa49ba52c, 78 },
    { 0xf32ba529, 61 },
    { 0, 0 },
    { 0x3c9ba52f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1ba533, 90 },
    { 0x775ba533, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ba539, 66 },
    { 0x9b2ba53a, 100 },
    { 0xacfba53b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eba53e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62ba541, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15ba546, 33 },
    { 0x978ba547, 23 },
    { 0x99eba547, 61 },
    { 0xc65ba549, 87 },
    { 0x608ba54a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fba54d, 55 },
    { 0, 0 },
    { 0xad7ba54f, 33 },
    { 0xed6ba54f, 95 },
    { 0x6e8ba550, 61 },
    { 0, 0 },
    { 0xb07ba553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630ba558, 100 },
    { 0xaa0ba559, 97 },
    { 0x1ebba55a, 23 },
    { 0x576ba55a, 33 },
    { 0, 0 },
    { 0x6ba55d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eba563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cba567, 98 },
    { 0, 0 },
    { 0x617ba569, 90 },
    { 0x72dba56a, 93 },
    { 0xb0dba56b, 55 },
    { 0x59ba56c, 90 },
    { 0x4f6ba56c, 98 },
    { 0xe8aba56e, 98 },
    { 0, 0 },
    { 0x942ba570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4caba573, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320ba57c, 90 },
    { 0x9d4ba57d, 76 },
    { 0, 0 },
    { 0x373ba57f, 23 },
    { 0, 0 },
    { 0xc5dba581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x976ba584, 66 },
    { 0x106ba585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76ba588, 90 },
    { 0xe0eba589, 95 },
    { 0x612ba589, 98 },
    { 0, 0 },
    { 0x47cba58c, 87 },
    { 0, 0 },
    { 0xef0ba58e, 46 },
    { 0x2b6ba58e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5ba594, 90 },
    { 0x924ba595, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44ba599, 99 },
    { 0x184ba599, 99 },
    { 0, 0 },
    { 0xd01ba59c, 95 },
    { 0, 0 },
    { 0xed1ba59e, 90 },
    { 0xac5ba59e, 33 },
    { 0xa29ba5a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ba5a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5ba5b5, 23 },
    { 0, 0 },
    { 0xc88ba5b7, 98 },
    { 0xb1bba5b8, 90 },
    { 0x7aba5b8, 23 },
    { 0xa4cba5ba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1ba5be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbba5c3, 98 },
    { 0x9efba5c4, 34 },
    { 0x89cba5c4, 33 },
    { 0xf7cba5c6, 55 },
    { 0, 0 },
    { 0xd22ba5c8, 33 },
    { 0, 0 },
    { 0x393ba5ca, 98 },
    { 0xbba5cb, 90 },
    { 0xf92ba5cc, 98 },
    { 0x2bdba5cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aba5d2, 61 },
    { 0x13fba5d3, 78 },
    { 0xd61ba5d4, 98 },
    { 0xea4ba5d4, 90 },
    { 0x457ba5d6, 76 },
    { 0x9d5ba5d6, 76 },
    { 0xf90ba5d5, 78 },
    { 0, 0 },
    { 0x64dba5da, 93 },
    { 0, 0 },
    { 0xc98ba5dc, 33 },
    { 0x7d3ba5dc, 85 },
    { 0x31fba5de, 33 },
    { 0x2a3ba5dd, 55 },
    { 0, 0 },
    { 0xc26ba5e1, 23 },
    { 0x529ba5e2, 99 },
    { 0xa7aba5e3, 33 },
    { 0xb8cba5e2, 34 },
    { 0, 0 },
    { 0x105ba5e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcba5eb, 87 },
    { 0, 0 },
    { 0x2ddba5ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bba5f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287ba5fa, 98 },
    { 0xadfba5fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0ba606, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53ba609, 61 },
    { 0x42fba60a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadba611, 77 },
    { 0, 0 },
    { 0x35ba613, 43 },
    { 0, 0 },
    { 0x559ba615, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cba61a, 90 },
    { 0xb50ba61b, 61 },
    { 0x58ba61c, 33 },
    { 0x6a0ba61d, 61 },
    { 0, 0 },
    { 0x5bcba61f, 88 },
    { 0x46fba61f, 33 },
    { 0, 0 },
    { 0x610ba622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x292ba625, 55 },
    { 0x6feba626, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x366ba629, 66 },
    { 0x215ba62a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72ba634, 34 },
    { 0x7d8ba635, 90 },
    { 0x919ba636, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32ba63b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x483ba63e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ba642, 55 },
    { 0x895ba642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ba648, 81 },
    { 0x2bfba648, 67 },
    { 0xcb8ba64a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10ba64d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1ba652, 83 },
    { 0x282ba652, 98 },
    { 0, 0 },
    { 0x24bba655, 78 },
    { 0xfbba655, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2ba659, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680ba660, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4ba664, 68 },
    { 0, 0 },
    { 0xb9cba666, 23 },
    { 0x8cbba667, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aeba677, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddba681, 76 },
    { 0xb51ba682, 23 },
    { 0, 0 },
    { 0x542ba684, 78 },
    { 0x771ba685, 23 },
    { 0x8a7ba685, 61 },
    { 0x8eeba687, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ba68a, 53 },
    { 0x535ba68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829ba68f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272ba693, 78 },
    { 0x8d6ba694, 65 },
    { 0, 0 },
    { 0x151ba696, 67 },
    { 0, 0 },
    { 0x457ba698, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295ba69c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fba6a0, 98 },
    { 0x5e3ba6a0, 46 },
    { 0xc1aba6a2, 90 },
    { 0xb99ba6a2, 95 },
    { 0xa4dba6a4, 33 },
    { 0, 0 },
    { 0x1bfba6a6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966ba6ac, 90 },
    { 0x128ba6ac, 95 },
    { 0x23aba6ae, 61 },
    { 0x911ba6af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7ba6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcba6bf, 81 },
    { 0, 0 },
    { 0xdb2ba6c1, 95 },
    { 0x3dba6c2, 98 },
    { 0x9feba6c2, 95 },
    { 0, 0 },
    { 0x715ba6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462ba6c9, 33 },
    { 0, 0 },
    { 0x954ba6cb, 42 },
    { 0xc9aba6cc, 98 },
    { 0x958ba6cb, 90 },
    { 0xc3bba6ce, 54 },
    { 0x769ba6ce, 98 },
    { 0xa13ba6d0, 40 },
    { 0, 0 },
    { 0xd1ba6d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4eba6d5, 90 },
    { 0, 0 },
    { 0x4efba6d7, 93 },
    { 0, 0 },
    { 0x7b9ba6d9, 78 },
    { 0x934ba6d9, 61 },
    { 0xfabba6da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfba6de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167ba6e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cba6e8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x640ba6eb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ba6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56ba6fa, 85 },
    { 0x73aba6fb, 34 },
    { 0x4a5ba6fa, 95 },
    { 0, 0 },
    { 0xe4cba6fe, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40ba701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x637ba704, 76 },
    { 0, 0 },
    { 0xf41ba706, 90 },
    { 0, 0 },
    { 0xd9bba708, 61 },
    { 0x655ba709, 90 },
    { 0x697ba70a, 26 },
    { 0xda1ba70b, 90 },
    { 0, 0 },
    { 0x941ba70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eba710, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39ba713, 46 },
    { 0xabcba714, 85 },
    { 0xc6fba713, 76 },
    { 0xd82ba714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fba71b, 33 },
    { 0, 0 },
    { 0xc2dba71d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00ba720, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52ba723, 76 },
    { 0x4a6ba724, 55 },
    { 0x23aba725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312ba729, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fba72c, 93 },
    { 0xb06ba72d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9ba731, 33 },
    { 0xd60ba732, 34 },
    { 0xac5ba733, 33 },
    { 0x43eba733, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1ba739, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3ba73c, 90 },
    { 0x323ba73d, 98 },
    { 0, 0 },
    { 0x5e0ba73f, 39 },
    { 0xd24ba73f, 95 },
    { 0x592ba741, 98 },
    { 0x63bba73f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cba747, 55 },
    { 0x2ecba748, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fba74d, 87 },
    { 0xfa5ba74e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3ba752, 61 },
    { 0, 0 },
    { 0x417ba754, 90 },
    { 0, 0 },
    { 0xcfcba756, 33 },
    { 0x3c7ba756, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256ba75b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ba763, 23 },
    { 0xd33ba764, 33 },
    { 0xf51ba765, 67 },
    { 0xc8fba766, 65 },
    { 0xf10ba767, 76 },
    { 0xc43ba768, 33 },
    { 0, 0 },
    { 0x230ba76a, 95 },
    { 0xf27ba76b, 95 },
    { 0x401ba76c, 33 },
    { 0, 0 },
    { 0xedba76e, 52 },
    { 0, 0 },
    { 0xd73ba770, 23 },
    { 0x8e9ba771, 17 },
    { 0xf82ba772, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138ba779, 23 },
    { 0xf04ba77a, 92 },
    { 0x5beba77a, 66 },
    { 0x38cba77a, 61 },
    { 0x841ba77d, 43 },
    { 0x5deba77e, 23 },
    { 0xffeba77d, 36 },
    { 0, 0 },
    { 0x91bba781, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bba785, 93 },
    { 0xb9eba785, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69ba78d, 23 },
    { 0x35fba78e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bba792, 90 },
    { 0x236ba792, 33 },
    { 0x267ba794, 33 },
    { 0xf18ba795, 46 },
    { 0x8bcba796, 95 },
    { 0xa2fba797, 33 },
    { 0x657ba797, 93 },
    { 0xcc8ba799, 61 },
    { 0x276ba79a, 90 },
    { 0, 0 },
    { 0xe5eba79c, 78 },
    { 0x861ba79d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ba7a1, 90 },
    { 0xcb6ba7a1, 66 },
    { 0, 0 },
    { 0x65bba7a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ba7a9, 100 },
    { 0, 0 },
    { 0x3acba7ab, 78 },
    { 0xcdeba7ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29ba7b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfba7b9, 90 },
    { 0x7e8ba7ba, 34 },
    { 0, 0 },
    { 0x3a1ba7bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ba7c1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fba7c4, 98 },
    { 0, 0 },
    { 0xd9dba7c6, 43 },
    { 0x9f8ba7c7, 23 },
    { 0xcfcba7c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ba7cc, 17 },
    { 0, 0 },
    { 0x4a6ba7ce, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x408ba7d1, 23 },
    { 0xf32ba7d1, 98 },
    { 0x2f9ba7d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfba7d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36ba7dc, 90 },
    { 0x807ba7dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9ba7e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ba7e3, 34 },
    { 0x408ba7e3, 23 },
    { 0xbdaba7e5, 97 },
    { 0xceba7e4, 23 },
    { 0x8cba7e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcba7ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2ba7ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdba7f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0ba7f9, 98 },
    { 0x6a1ba7f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868ba7fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51ba803, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eba808, 34 },
    { 0x52aba808, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69eba810, 98 },
    { 0xbe5ba811, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5ba815, 98 },
    { 0x855ba816, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eba824, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cba829, 46 },
    { 0xd4bba82a, 93 },
    { 0x663ba82a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18aba832, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fba836, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9ba839, 91 },
    { 0x484ba839, 23 },
    { 0, 0 },
    { 0x809ba83c, 90 },
    { 0xd1dba83d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499ba848, 90 },
    { 0x99dba848, 95 },
    { 0xba3ba848, 95 },
    { 0xfabba84b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x487ba84e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ba851, 33 },
    { 0xe09ba851, 55 },
    { 0, 0 },
    { 0xefbba854, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811ba85a, 67 },
    { 0, 0 },
    { 0xbcbba85c, 66 },
    { 0x418ba85c, 99 },
    { 0x653ba85e, 90 },
    { 0xe92ba85f, 98 },
    { 0, 0 },
    { 0x421ba861, 55 },
    { 0, 0 },
    { 0x23cba863, 23 },
    { 0xe79ba864, 43 },
    { 0, 0 },
    { 0x11aba866, 55 },
    { 0x4b8ba867, 81 },
    { 0xd26ba868, 23 },
    { 0, 0 },
    { 0x36dba86a, 90 },
    { 0xe6ba86a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115ba86f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x984ba872, 90 },
    { 0x915ba873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25ba876, 76 },
    { 0x3afba877, 61 },
    { 0, 0 },
    { 0x31bba879, 23 },
    { 0x87dba87a, 93 },
    { 0, 0 },
    { 0x2d7ba87c, 93 },
    { 0xda7ba87c, 55 },
    { 0x8b3ba87e, 87 },
    { 0x359ba87f, 61 },
    { 0, 0 },
    { 0x3ecba881, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dba887, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ba88e, 33 },
    { 0, 0 },
    { 0xf0bba890, 90 },
    { 0, 0 },
    { 0xe44ba892, 17 },
    { 0x677ba893, 76 },
    { 0, 0 },
    { 0x78ba895, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x215ba898, 61 },
    { 0x9c2ba899, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aba89c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1aba8a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdba8a3, 74 },
    { 0, 0 },
    { 0x8e3ba8a5, 76 },
    { 0xc09ba8a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0ba8ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dba8ae, 93 },
    { 0x9fdba8af, 93 },
    { 0, 0 },
    { 0x547ba8b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62eba8b6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cba8b9, 46 },
    { 0xe6eba8ba, 90 },
    { 0xa0bba8ba, 64 },
    { 0, 0 },
    { 0xa8dba8bd, 91 },
    { 0xd53ba8be, 66 },
    { 0xe41ba8bf, 90 },
    { 0xebeba8bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cba8cf, 33 },
    { 0, 0 },
    { 0xdb1ba8d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x732ba8d4, 46 },
    { 0x44dba8d5, 50 },
    { 0x4f0ba8d5, 48 },
    { 0, 0 },
    { 0x644ba8d8, 61 },
    { 0, 0 },
    { 0x7cba8da, 99 },
    { 0xea1ba8db, 95 },
    { 0xe70ba8dc, 33 },
    { 0xd4aba8dd, 23 },
    { 0x920ba8dd, 95 },
    { 0x2e3ba8df, 99 },
    { 0x906ba8da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7ba8e7, 99 },
    { 0x2eeba8e7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdba8f0, 91 },
    { 0xe4bba8f1, 61 },
    { 0x242ba8f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4ba8f5, 90 },
    { 0xd15ba8f6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ba8f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ba8fe, 46 },
    { 0x774ba8ff, 33 },
    { 0x9b1ba900, 98 },
    { 0x8ecba900, 95 },
    { 0x9bbba902, 23 },
    { 0x360ba903, 43 },
    { 0xaabba903, 25 },
    { 0xd7ba903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ba909, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680ba911, 55 },
    { 0, 0 },
    { 0x24dba913, 88 },
    { 0xd38ba913, 84 },
    { 0x7bfba913, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x773ba918, 95 },
    { 0x8b1ba919, 61 },
    { 0xc99ba919, 55 },
    { 0x8cbba91b, 69 },
    { 0x109ba91c, 55 },
    { 0, 0 },
    { 0x6c4ba91e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dba921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baba926, 90 },
    { 0, 0 },
    { 0xebfba928, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x342ba92b, 87 },
    { 0, 0 },
    { 0x6e8ba92d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ba934, 33 },
    { 0, 0 },
    { 0xd67ba936, 33 },
    { 0xb61ba937, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623ba93b, 70 },
    { 0, 0 },
    { 0xcfaba93d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ba941, 43 },
    { 0xcb4ba942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x586ba945, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ba94a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cba94f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ba952, 95 },
    { 0, 0 },
    { 0x945ba954, 76 },
    { 0xeaaba955, 84 },
    { 0xb8dba956, 61 },
    { 0x7e1ba954, 90 },
    { 0x53dba955, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eba95b, 67 },
    { 0x370ba95c, 95 },
    { 0xf9aba95b, 5 },
    { 0x56ba95b, 33 },
    { 0x4baba95f, 43 },
    { 0x599ba95e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dba964, 55 },
    { 0x367ba965, 85 },
    { 0xa7ba966, 87 },
    { 0xf2fba967, 61 },
    { 0, 0 },
    { 0x248ba969, 99 },
    { 0xe66ba969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ba972, 77 },
    { 0xa6fba972, 33 },
    { 0, 0 },
    { 0x906ba975, 34 },
    { 0x5b1ba976, 65 },
    { 0x380ba977, 19 },
    { 0x3a1ba977, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fba97b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcba97f, 43 },
    { 0xbd0ba97f, 52 },
    { 0x58aba981, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44ba988, 90 },
    { 0, 0 },
    { 0x57ba98a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eba993, 23 },
    { 0x9dfba994, 95 },
    { 0xb2cba995, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6ba99d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ba9a1, 90 },
    { 0xd43ba9a2, 95 },
    { 0x84eba9a3, 23 },
    { 0, 0 },
    { 0xdadba9a5, 90 },
    { 0, 0 },
    { 0xaedba9a7, 61 },
    { 0x56aba9a7, 24 },
    { 0x938ba9a7, 98 },
    { 0xe55ba9aa, 23 },
    { 0, 0 },
    { 0xb4ba9ac, 33 },
    { 0x170ba9ad, 55 },
    { 0xaa3ba9ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ba9b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dba9b9, 61 },
    { 0xc97ba9ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6ba9bf, 95 },
    { 0x54aba9bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2ba9c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17ba9c6, 90 },
    { 0x4d2ba9c6, 96 },
    { 0, 0 },
    { 0x286ba9c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799ba9cd, 34 },
    { 0xb2aba9ce, 90 },
    { 0, 0 },
    { 0xde6ba9d0, 61 },
    { 0xf78ba9d1, 40 },
    { 0x1ecba9d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfba9d7, 61 },
    { 0x79aba9d8, 76 },
    { 0x260ba9d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdba9dc, 33 },
    { 0, 0 },
    { 0x69ba9de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcba9e4, 23 },
    { 0xc65ba9e5, 55 },
    { 0, 0 },
    { 0x34ba9e7, 34 },
    { 0xacaba9e8, 91 },
    { 0xf96ba9e8, 76 },
    { 0, 0 },
    { 0xab1ba9eb, 52 },
    { 0xc4fba9ec, 76 },
    { 0xa0dba9ed, 90 },
    { 0x3d3ba9ee, 76 },
    { 0xa98ba9ed, 52 },
    { 0xd52ba9f0, 23 },
    { 0xd8dba9f1, 90 },
    { 0xda0ba9f2, 23 },
    { 0, 0 },
    { 0x9eba9f4, 76 },
    { 0x60aba9f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fba9f8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1ba9fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162baa01, 90 },
    { 0x4e0baa01, 80 },
    { 0x92dbaa03, 68 },
    { 0xb6cbaa03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbaa0b, 59 },
    { 0, 0 },
    { 0xd4ebaa0d, 47 },
    { 0, 0 },
    { 0xb61baa0f, 69 },
    { 0x81cbaa10, 87 },
    { 0, 0 },
    { 0x7b7baa12, 76 },
    { 0x58baa12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98baa17, 55 },
    { 0, 0 },
    { 0xe55baa19, 90 },
    { 0, 0 },
    { 0xaebbaa1b, 61 },
    { 0, 0 },
    { 0x63cbaa1d, 76 },
    { 0x784baa1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddbaa22, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x173baa25, 98 },
    { 0, 0 },
    { 0x92cbaa27, 85 },
    { 0, 0 },
    { 0xdcbbaa29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x911baa2c, 23 },
    { 0x676baa2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bebaa31, 40 },
    { 0x1eebaa32, 98 },
    { 0x19cbaa33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x844baa36, 90 },
    { 0, 0 },
    { 0x3c5baa38, 98 },
    { 0xd2dbaa39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dbaa3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x648baa42, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2baa46, 93 },
    { 0x732baa46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baa4d, 90 },
    { 0xce1baa4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0baa53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412baa58, 76 },
    { 0xdabbaa58, 98 },
    { 0x15baa58, 23 },
    { 0xa0baa5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefbaa5f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fbaa63, 52 },
    { 0xa75baa63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefbaa68, 33 },
    { 0, 0 },
    { 0x3adbaa6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dbaa6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbaa72, 78 },
    { 0xd9abaa72, 76 },
    { 0x38cbaa74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdbaa7f, 33 },
    { 0xfdfbaa7f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cbaa89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47baa8c, 61 },
    { 0, 0 },
    { 0xa5baa8e, 98 },
    { 0x6f9baa8f, 23 },
    { 0xb59baa8f, 33 },
    { 0x9d1baa8e, 76 },
    { 0xc6cbaa92, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baa95, 88 },
    { 0x80dbaa96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2baa99, 87 },
    { 0xad3baa9a, 55 },
    { 0xe3cbaa9a, 90 },
    { 0x30ebaa9a, 61 },
    { 0, 0 },
    { 0x5afbaa9e, 76 },
    { 0x295baa9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbaaa2, 87 },
    { 0xc32baaa3, 33 },
    { 0x109baaa4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52baaa7, 61 },
    { 0, 0 },
    { 0x787baaa9, 93 },
    { 0, 0 },
    { 0x919baaab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfbaab0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94baab5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbabaab8, 55 },
    { 0, 0 },
    { 0x47dbaaba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7baabd, 43 },
    { 0x6d4baabd, 69 },
    { 0x9fdbaabf, 90 },
    { 0xf71baabf, 23 },
    { 0xe2cbaac1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8baac9, 78 },
    { 0xec8baac9, 90 },
    { 0, 0 },
    { 0x6cebaacc, 90 },
    { 0, 0 },
    { 0x527baace, 55 },
    { 0, 0 },
    { 0xd32baad0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1abaad6, 23 },
    { 0x187baad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0baadc, 43 },
    { 0, 0 },
    { 0x24abaade, 33 },
    { 0xcffbaade, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fbaae3, 76 },
    { 0xe97baae4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51baae7, 23 },
    { 0x32dbaae8, 81 },
    { 0x150baae8, 90 },
    { 0x51baaea, 90 },
    { 0xf7cbaaeb, 76 },
    { 0x477baaec, 95 },
    { 0, 0 },
    { 0x9ffbaaee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9baaf5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fbaaf9, 38 },
    { 0x9cbbaaf9, 98 },
    { 0xafdbaafb, 67 },
    { 0x37baafb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87baaff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x153bab02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fbab05, 46 },
    { 0x97ebab06, 95 },
    { 0x2d8bab07, 93 },
    { 0xd84bab08, 91 },
    { 0x6efbab09, 61 },
    { 0, 0 },
    { 0xeffbab0b, 90 },
    { 0, 0 },
    { 0x467bab0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283bab12, 95 },
    { 0xb5dbab13, 33 },
    { 0x76fbab14, 33 },
    { 0xd62bab13, 84 },
    { 0x573bab16, 76 },
    { 0, 0 },
    { 0xa78bab18, 99 },
    { 0, 0 },
    { 0x4debab1a, 33 },
    { 0, 0 },
    { 0x3dbab1c, 55 },
    { 0x56fbab1d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864bab21, 81 },
    { 0x70bbab21, 75 },
    { 0x66cbab23, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bab29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bab2c, 90 },
    { 0x178bab2d, 85 },
    { 0, 0 },
    { 0x615bab2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917bab34, 46 },
    { 0x998bab35, 61 },
    { 0xadabab35, 90 },
    { 0xa3dbab35, 99 },
    { 0xd4bab36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ebab3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddbab42, 76 },
    { 0, 0 },
    { 0x7e3bab44, 23 },
    { 0, 0 },
    { 0x526bab46, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1bab4b, 90 },
    { 0xbe6bab4b, 98 },
    { 0, 0 },
    { 0xac1bab4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ebab52, 23 },
    { 0x563bab53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24bab58, 99 },
    { 0x621bab59, 98 },
    { 0, 0 },
    { 0x19bab5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebab5f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14bab65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x501bab68, 99 },
    { 0xc9cbab69, 61 },
    { 0x340bab69, 61 },
    { 0xd66bab6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464bab74, 98 },
    { 0x508bab75, 95 },
    { 0x7adbab74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfbab7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x155bab80, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6bab85, 95 },
    { 0x195bab85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadbab89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678bab96, 93 },
    { 0x657bab97, 100 },
    { 0x116bab98, 98 },
    { 0xdccbab99, 33 },
    { 0x4bdbab98, 23 },
    { 0x430bab96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x821bab9e, 43 },
    { 0x6afbab9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17baba4, 23 },
    { 0xff9baba5, 90 },
    { 0x107baba4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fbaba9, 95 },
    { 0, 0 },
    { 0xe35babab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58babaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39babb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x498babb6, 55 },
    { 0, 0 },
    { 0xed7babb8, 23 },
    { 0xe76babb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0babbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91babc1, 95 },
    { 0, 0 },
    { 0xdbbbabc3, 91 },
    { 0x754babc4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aababc7, 76 },
    { 0x112babc8, 84 },
    { 0x3cfbabc8, 98 },
    { 0x856babc9, 33 },
    { 0, 0 },
    { 0x7f3babcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaababd0, 34 },
    { 0, 0 },
    { 0x72ebabd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cbabd6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07babd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fbabde, 97 },
    { 0x108babde, 90 },
    { 0xe5babe0, 55 },
    { 0xf82babe1, 84 },
    { 0x8e2babe2, 76 },
    { 0, 0 },
    { 0xf4dbabe4, 33 },
    { 0x266babe5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610babe9, 78 },
    { 0x664babea, 23 },
    { 0xc7ababeb, 33 },
    { 0x39dbabec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaababf0, 90 },
    { 0xa8bbabf0, 36 },
    { 0x1ebbabf0, 33 },
    { 0x92bbabf2, 98 },
    { 0, 0 },
    { 0x2babf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dbabf8, 55 },
    { 0xb89babf9, 90 },
    { 0x6f1babf9, 61 },
    { 0x490babfb, 90 },
    { 0xaadbabfc, 46 },
    { 0xd27babfd, 61 },
    { 0x887babfd, 90 },
    { 0xa28babfb, 61 },
    { 0x243babfe, 90 },
    { 0xe23bac01, 33 },
    { 0x46fbac01, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdbac07, 100 },
    { 0x245bac08, 33 },
    { 0x8bdbac08, 76 },
    { 0, 0 },
    { 0x404bac0b, 52 },
    { 0x8b0bac0c, 23 },
    { 0xfa8bac0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bac11, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fbac15, 76 },
    { 0x3ebac15, 87 },
    { 0xd45bac17, 98 },
    { 0x422bac18, 33 },
    { 0x32bbac15, 55 },
    { 0, 0 },
    { 0x439bac1b, 55 },
    { 0xc9bac1b, 61 },
    { 0x635bac1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4bac21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96bac27, 56 },
    { 0xa0bac28, 23 },
    { 0, 0 },
    { 0xf4ebac2a, 68 },
    { 0x1f5bac2a, 90 },
    { 0x9f3bac2a, 61 },
    { 0x1d1bac2a, 94 },
    { 0xbebac2a, 61 },
    { 0xfebac2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dbac33, 61 },
    { 0x394bac33, 33 },
    { 0x958bac34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8bac38, 23 },
    { 0x42dbac38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8bac3c, 52 },
    { 0, 0 },
    { 0x1afbac3e, 61 },
    { 0x66bac3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9bac44, 46 },
    { 0xc28bac45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3bac4a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3bac50, 47 },
    { 0xbbac51, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0bac57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbebac60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bac65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44bac68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2bac6e, 33 },
    { 0, 0 },
    { 0xdaabac70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcabac77, 31 },
    { 0, 0 },
    { 0xb54bac79, 90 },
    { 0xd60bac79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x741bac7d, 61 },
    { 0xdf4bac7d, 61 },
    { 0x4fbac7f, 23 },
    { 0xf2dbac80, 52 },
    { 0, 0 },
    { 0x48dbac82, 23 },
    { 0xda8bac82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507bac88, 43 },
    { 0xf1abac89, 76 },
    { 0x344bac8a, 61 },
    { 0, 0 },
    { 0xebbbac8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38bac90, 98 },
    { 0x4dbbac91, 58 },
    { 0x85dbac91, 23 },
    { 0x693bac92, 55 },
    { 0, 0 },
    { 0xb9fbac95, 33 },
    { 0xd55bac95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeebac9c, 98 },
    { 0xdf1bac9d, 98 },
    { 0x8bdbac9e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68baca3, 52 },
    { 0xcc2baca3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44baca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3bacab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37bacae, 96 },
    { 0x954bacaf, 17 },
    { 0xf3dbacaf, 33 },
    { 0x498bacb0, 93 },
    { 0x1a7bacb2, 52 },
    { 0x219bacb3, 99 },
    { 0, 0 },
    { 0x872bacb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10bacbf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbacc3, 85 },
    { 0x651bacc4, 34 },
    { 0xe3bacc5, 61 },
    { 0x85bacc5, 33 },
    { 0x596bacc5, 88 },
    { 0, 0 },
    { 0x83bacc9, 23 },
    { 0xff0bacc9, 93 },
    { 0xc33bacc9, 33 },
    { 0xa68baccc, 34 },
    { 0, 0 },
    { 0xb26bacce, 33 },
    { 0x42abaccf, 84 },
    { 0xa80baccf, 61 },
    { 0x439bacd0, 90 },
    { 0x8d7bacd2, 61 },
    { 0x9f4bacd1, 23 },
    { 0x49bacd2, 43 },
    { 0, 0 },
    { 0xcbbacd6, 90 },
    { 0x4f6bacd6, 43 },
    { 0x1dabacd8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ebacdd, 23 },
    { 0x6aebacdd, 23 },
    { 0x346bacde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x139bace2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0bace7, 17 },
    { 0xc17bace7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dbaceb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fbacef, 61 },
    { 0x16bbacf0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25bacf3, 55 },
    { 0x69dbacf4, 95 },
    { 0xe2cbacf5, 76 },
    { 0, 0 },
    { 0x84abacf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabacfa, 78 },
    { 0xfe1bacfb, 90 },
    { 0x2d4bacfb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bad02, 33 },
    { 0x3b4bad03, 55 },
    { 0, 0 },
    { 0x464bad05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbad0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408bad10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0bad14, 43 },
    { 0xbbcbad15, 61 },
    { 0, 0 },
    { 0x9d0bad17, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fbad1d, 95 },
    { 0, 0 },
    { 0xbc6bad1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3bad22, 61 },
    { 0xf8ebad22, 61 },
    { 0x93cbad24, 84 },
    { 0, 0 },
    { 0x798bad26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x148bad29, 38 },
    { 0x1d9bad2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5bad2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0bad32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebad36, 75 },
    { 0xbcdbad36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74bad3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fbad42, 95 },
    { 0x239bad42, 61 },
    { 0xe81bad43, 98 },
    { 0xa69bad44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cbad48, 98 },
    { 0, 0 },
    { 0x5b2bad4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4bad4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cbad58, 76 },
    { 0xd9ebad59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43bad5f, 93 },
    { 0xba6bad5f, 61 },
    { 0xd91bad5f, 52 },
    { 0, 0 },
    { 0xe6bbad63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dbad67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bad6a, 61 },
    { 0, 0 },
    { 0x529bad6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbad6f, 61 },
    { 0x5febad70, 43 },
    { 0x890bad71, 78 },
    { 0x511bad71, 99 },
    { 0xfb6bad73, 40 },
    { 0x955bad71, 87 },
    { 0x422bad75, 100 },
    { 0xe50bad76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ebad7b, 78 },
    { 0x53ebad7c, 67 },
    { 0xccbbad7b, 46 },
    { 0xf34bad7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8bad82, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bad85, 33 },
    { 0x9a4bad86, 53 },
    { 0xd35bad87, 33 },
    { 0x4fcbad88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ebad8d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866bad93, 98 },
    { 0, 0 },
    { 0xdd9bad95, 95 },
    { 0xea6bad95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63abad99, 72 },
    { 0, 0 },
    { 0x208bad9b, 90 },
    { 0x475bad9c, 91 },
    { 0xa3bbad9d, 100 },
    { 0xce7bad9e, 81 },
    { 0xe51bad9e, 91 },
    { 0x688bad9e, 76 },
    { 0xa88bada1, 90 },
    { 0xc67bada2, 90 },
    { 0x118bada1, 33 },
    { 0xe96bada4, 33 },
    { 0xb93bada5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3badab, 90 },
    { 0x951badac, 77 },
    { 0xed6badad, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbbadb4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cbadbd, 92 },
    { 0xb2dbadbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9abadc7, 78 },
    { 0x59fbadc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ebadcd, 93 },
    { 0xcf7badcd, 90 },
    { 0x545badcf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fbadd2, 93 },
    { 0x2c7badd3, 95 },
    { 0x705badd4, 98 },
    { 0, 0 },
    { 0xd79badd6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8baddf, 88 },
    { 0, 0 },
    { 0x2cabade1, 33 },
    { 0, 0 },
    { 0x8a2bade3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5bade6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55badf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843badfe, 33 },
    { 0, 0 },
    { 0x476bae00, 90 },
    { 0x11dbae01, 84 },
    { 0x714bae00, 33 },
    { 0xdaebae01, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ebae09, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ebae0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cabae12, 95 },
    { 0, 0 },
    { 0x187bae14, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6bae1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbae1d, 90 },
    { 0x9b7bae1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03bae21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735bae25, 55 },
    { 0, 0 },
    { 0xbd8bae27, 98 },
    { 0x66cbae28, 43 },
    { 0xe48bae29, 90 },
    { 0, 0 },
    { 0xe37bae2b, 33 },
    { 0xaf3bae2b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bae33, 34 },
    { 0xdecbae33, 33 },
    { 0xed2bae35, 98 },
    { 0x9bbbae35, 61 },
    { 0x21bbae34, 98 },
    { 0, 0 },
    { 0xa34bae39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ebae3d, 90 },
    { 0, 0 },
    { 0x255bae3f, 98 },
    { 0x69bbae40, 95 },
    { 0xf72bae41, 95 },
    { 0x682bae41, 23 },
    { 0x457bae42, 100 },
    { 0x687bae44, 23 },
    { 0x47bbae43, 95 },
    { 0x9adbae46, 98 },
    { 0x4acbae47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cbae4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82bae4f, 61 },
    { 0xa1ebae50, 89 },
    { 0, 0 },
    { 0xb16bae52, 98 },
    { 0, 0 },
    { 0xf11bae54, 90 },
    { 0, 0 },
    { 0xf77bae56, 90 },
    { 0, 0 },
    { 0x8e9bae58, 93 },
    { 0xfc8bae59, 23 },
    { 0x441bae59, 61 },
    { 0, 0 },
    { 0x43dbae5c, 17 },
    { 0x69ebae5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57abae61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754bae6a, 98 },
    { 0, 0 },
    { 0x844bae6c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808bae77, 99 },
    { 0xa2abae78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bbae7b, 90 },
    { 0xab6bae7c, 76 },
    { 0, 0 },
    { 0x659bae7e, 34 },
    { 0x92bbae7f, 93 },
    { 0, 0 },
    { 0x154bae81, 33 },
    { 0xdfabae81, 90 },
    { 0x809bae83, 33 },
    { 0xd5bae83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cbae89, 23 },
    { 0x1adbae89, 98 },
    { 0x408bae8b, 77 },
    { 0x77abae8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16abae90, 61 },
    { 0, 0 },
    { 0xfdebae92, 88 },
    { 0, 0 },
    { 0x2ebbae94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bbae9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4baea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ebaea9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854baeaf, 43 },
    { 0x6d9baeb0, 23 },
    { 0x6c6baeb1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffbaeb6, 100 },
    { 0xf54baeb7, 84 },
    { 0xeedbaeb7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998baebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdbaec8, 99 },
    { 0x6a6baec9, 98 },
    { 0, 0 },
    { 0x9aebaecb, 81 },
    { 0xc06baecb, 90 },
    { 0, 0 },
    { 0x2ccbaece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbaed3, 61 },
    { 0x715baed3, 66 },
    { 0x30dbaed5, 66 },
    { 0x192baed6, 98 },
    { 0, 0 },
    { 0x959baed8, 46 },
    { 0x462baed8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x999baedc, 95 },
    { 0, 0 },
    { 0x761baede, 78 },
    { 0, 0 },
    { 0x4c6baee0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4babaee6, 95 },
    { 0x3a4baee6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37baef4, 90 },
    { 0x42fbaef5, 98 },
    { 0x598baef6, 90 },
    { 0x305baef5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eabaefb, 23 },
    { 0x88bbaefb, 95 },
    { 0xf7abaefd, 55 },
    { 0x225baefd, 66 },
    { 0xbc7baefb, 95 },
    { 0x13bbaefb, 95 },
    { 0x554baf00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77baf07, 90 },
    { 0, 0 },
    { 0xb72baf09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fbaf0e, 90 },
    { 0xfd8baf0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbaf12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868baf17, 78 },
    { 0x66dbaf18, 98 },
    { 0xb69baf17, 67 },
    { 0xa04baf18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x729baf1d, 33 },
    { 0, 0 },
    { 0xc3bbaf1f, 33 },
    { 0x4a9baf20, 34 },
    { 0x248baf20, 23 },
    { 0xfd1baf22, 98 },
    { 0xc7cbaf21, 52 },
    { 0, 0 },
    { 0x86abaf25, 33 },
    { 0, 0 },
    { 0x893baf27, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38baf2b, 76 },
    { 0, 0 },
    { 0x1dabaf2d, 78 },
    { 0, 0 },
    { 0x2a8baf2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81abaf33, 95 },
    { 0, 0 },
    { 0xf22baf35, 61 },
    { 0x45abaf36, 90 },
    { 0x1d4baf36, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2baf3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995baf40, 90 },
    { 0, 0 },
    { 0xbf3baf42, 33 },
    { 0xbb2baf43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x949baf46, 99 },
    { 0x22abaf47, 61 },
    { 0, 0 },
    { 0x8d3baf49, 76 },
    { 0, 0 },
    { 0xa10baf4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78abaf4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x256baf52, 33 },
    { 0x1c7baf52, 23 },
    { 0, 0 },
    { 0x97ebaf55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dbaf5a, 98 },
    { 0, 0 },
    { 0xf47baf5c, 90 },
    { 0xd2fbaf5d, 34 },
    { 0xa4abaf5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679baf63, 98 },
    { 0x2c4baf64, 61 },
    { 0x1cebaf63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x964baf68, 81 },
    { 0x86fbaf69, 95 },
    { 0x8b3baf69, 61 },
    { 0xd52baf6b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dbaf6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82abaf72, 71 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dbaf76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abaf79, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473baf7e, 46 },
    { 0xaf2baf7f, 66 },
    { 0x30baf80, 66 },
    { 0, 0 },
    { 0xea3baf82, 55 },
    { 0, 0 },
    { 0x157baf84, 34 },
    { 0, 0 },
    { 0x96abaf86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ebaf89, 66 },
    { 0xa01baf89, 91 },
    { 0xe81baf8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30baf96, 90 },
    { 0x10bbaf97, 87 },
    { 0xc61baf97, 95 },
    { 0x62bbaf99, 90 },
    { 0x972baf99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353baf9e, 40 },
    { 0xa22baf9f, 65 },
    { 0x3acbaf9e, 55 },
    { 0xc5cbafa1, 61 },
    { 0xda8bafa2, 76 },
    { 0xba2bafa2, 23 },
    { 0x314bafa4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79bafa9, 98 },
    { 0x49bbafaa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebafad, 90 },
    { 0, 0 },
    { 0x9febafaf, 99 },
    { 0x5d5bafb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684bafb9, 76 },
    { 0xfbebafb9, 55 },
    { 0, 0 },
    { 0xd55bafbc, 87 },
    { 0x9fdbafbc, 90 },
    { 0x4bdbafbe, 90 },
    { 0, 0 },
    { 0x55ebafc0, 88 },
    { 0xb06bafc1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dbafc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcbafcb, 98 },
    { 0x2e2bafcb, 23 },
    { 0, 0 },
    { 0xed2bafce, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59bafd9, 99 },
    { 0xc30bafd9, 95 },
    { 0x938bafdb, 84 },
    { 0, 0 },
    { 0x8b5bafdd, 23 },
    { 0xf8ebafde, 61 },
    { 0x9b8bafdd, 76 },
    { 0x268bafde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20bafe3, 34 },
    { 0, 0 },
    { 0xb48bafe5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbbafeb, 76 },
    { 0x99fbafeb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8baff0, 84 },
    { 0x43dbaff1, 67 },
    { 0x2bdbaff2, 52 },
    { 0x1bdbaff3, 55 },
    { 0x5f2baff0, 90 },
    { 0, 0 },
    { 0xc25baff6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dbaff9, 99 },
    { 0x6babaffa, 61 },
    { 0, 0 },
    { 0x2ecbaffc, 23 },
    { 0xe53baffd, 61 },
    { 0xe12baffe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcbb006, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67bb009, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0bb00e, 66 },
    { 0x654bb00e, 33 },
    { 0xae5bb00e, 90 },
    { 0x7c7bb011, 99 },
    { 0x1bb00e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83bb01a, 43 },
    { 0x1ddbb01b, 90 },
    { 0, 0 },
    { 0xcebbb01d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dbb020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587bb025, 90 },
    { 0x46bbb026, 66 },
    { 0xa5bbb026, 93 },
    { 0x30bb025, 61 },
    { 0, 0 },
    { 0x10bbb02a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x649bb02d, 66 },
    { 0xc42bb02d, 34 },
    { 0x825bb02e, 80 },
    { 0, 0 },
    { 0x765bb031, 78 },
    { 0x81dbb031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eebb039, 33 },
    { 0x4d9bb03a, 55 },
    { 0x463bb03a, 66 },
    { 0xca2bb039, 40 },
    { 0, 0 },
    { 0xa43bb03e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77bb047, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9bb04c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4bb051, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88bb055, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1bb059, 83 },
    { 0x176bb05a, 90 },
    { 0x833bb05b, 33 },
    { 0x59abb05b, 95 },
    { 0xfcbb05b, 95 },
    { 0x28fbb05e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bb061, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffbb06b, 34 },
    { 0xf0cbb06c, 23 },
    { 0x107bb06c, 99 },
    { 0x8abbb06d, 92 },
    { 0x9bcbb06f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53bb073, 87 },
    { 0xd9dbb074, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x764bb077, 83 },
    { 0, 0 },
    { 0x85ebb079, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efbb08b, 75 },
    { 0x301bb08c, 98 },
    { 0x881bb08c, 34 },
    { 0x868bb08e, 55 },
    { 0xde1bb08d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ebb094, 95 },
    { 0xc73bb095, 33 },
    { 0xe57bb095, 90 },
    { 0xfdfbb096, 98 },
    { 0x470bb098, 84 },
    { 0xa6ebb099, 61 },
    { 0x9aabb099, 33 },
    { 0x8abbb098, 90 },
    { 0xc94bb09c, 43 },
    { 0, 0 },
    { 0x740bb09e, 34 },
    { 0, 0 },
    { 0xe78bb0a0, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbbb0a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21bb0ab, 61 },
    { 0x5e8bb0ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebbb0b4, 78 },
    { 0xf98bb0b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6bb0be, 98 },
    { 0, 0 },
    { 0xc01bb0c0, 87 },
    { 0xcb6bb0c1, 52 },
    { 0, 0 },
    { 0xdddbb0c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62abb0c7, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acbb0ca, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860bb0ce, 98 },
    { 0, 0 },
    { 0xef0bb0d0, 90 },
    { 0xdc0bb0d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7bb0d4, 75 },
    { 0, 0 },
    { 0x286bb0d6, 46 },
    { 0, 0 },
    { 0x392bb0d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211bb0dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632bb0e2, 61 },
    { 0, 0 },
    { 0x709bb0e4, 95 },
    { 0, 0 },
    { 0xc21bb0e6, 98 },
    { 0x442bb0e7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2bb0ef, 34 },
    { 0x9dbbb0ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3abb0f3, 91 },
    { 0x98abb0f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccbb0fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bbb102, 65 },
    { 0x863bb103, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bb107, 76 },
    { 0x69ebb108, 36 },
    { 0x3d0bb109, 46 },
    { 0xc3fbb10a, 33 },
    { 0x555bb10a, 66 },
    { 0x86abb109, 98 },
    { 0x547bb108, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bebb112, 95 },
    { 0, 0 },
    { 0x2e7bb114, 98 },
    { 0xb85bb114, 84 },
    { 0, 0 },
    { 0x6c6bb117, 55 },
    { 0x19bb118, 93 },
    { 0xde2bb119, 95 },
    { 0xd18bb11a, 34 },
    { 0xc10bb118, 90 },
    { 0xe46bb11c, 43 },
    { 0x4bcbb11d, 90 },
    { 0x48ebb11e, 33 },
    { 0, 0 },
    { 0xc74bb120, 33 },
    { 0, 0 },
    { 0x97bbb122, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5bb126, 23 },
    { 0x124bb127, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63bb12d, 34 },
    { 0, 0 },
    { 0x97fbb12f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204bb134, 61 },
    { 0x80fbb135, 33 },
    { 0x74cbb135, 95 },
    { 0xba6bb134, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99bb13a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cbb13d, 61 },
    { 0, 0 },
    { 0x561bb13f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdbb142, 33 },
    { 0, 0 },
    { 0x7b7bb144, 43 },
    { 0, 0 },
    { 0x8a0bb146, 34 },
    { 0, 0 },
    { 0x25cbb148, 81 },
    { 0x72ebb149, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbbb14c, 33 },
    { 0x33bb14d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7bb150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5bb153, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fbb158, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93abb15d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcbb161, 47 },
    { 0x4eebb162, 98 },
    { 0xb9dbb163, 98 },
    { 0, 0 },
    { 0x4f8bb165, 33 },
    { 0x29fbb166, 33 },
    { 0x3d2bb167, 95 },
    { 0, 0 },
    { 0x34bb169, 98 },
    { 0x59ebb16a, 95 },
    { 0x2dbb16a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35bb16f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270bb172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bb17c, 76 },
    { 0, 0 },
    { 0xaafbb17e, 81 },
    { 0x737bb17f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5bb182, 87 },
    { 0, 0 },
    { 0xd4dbb184, 95 },
    { 0xa2cbb185, 95 },
    { 0xc44bb186, 23 },
    { 0, 0 },
    { 0xe50bb188, 76 },
    { 0, 0 },
    { 0xd87bb18a, 90 },
    { 0x472bb18b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x120bb18e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x978bb191, 46 },
    { 0x766bb191, 90 },
    { 0, 0 },
    { 0x98bb194, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827bb199, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83bb19f, 100 },
    { 0, 0 },
    { 0x8abbb1a1, 98 },
    { 0xe23bb1a2, 87 },
    { 0x53abb1a3, 95 },
    { 0xc5bbb1a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87abb1a7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bb1ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efbb1b2, 66 },
    { 0xfe1bb1b2, 84 },
    { 0x401bb1b4, 90 },
    { 0x99bbb1b5, 100 },
    { 0, 0 },
    { 0x799bb1b7, 61 },
    { 0x17bb1b7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9bb1c0, 55 },
    { 0x5fcbb1c1, 77 },
    { 0, 0 },
    { 0xf32bb1c3, 90 },
    { 0x8dbb1c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbb1c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5bb1ca, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400bb1d0, 90 },
    { 0xe5bbb1d1, 76 },
    { 0x445bb1d1, 95 },
    { 0, 0 },
    { 0x2c6bb1d4, 98 },
    { 0, 0 },
    { 0x354bb1d6, 95 },
    { 0x2e0bb1d7, 33 },
    { 0x6abbb1d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35bb1e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xccabb1e3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1bb1e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x457bb1ec, 55 },
    { 0x56cbb1ed, 46 },
    { 0xc9dbb1ee, 91 },
    { 0x73ebb1ef, 67 },
    { 0xde6bb1f0, 65 },
    { 0, 0 },
    { 0xb9bb1f2, 90 },
    { 0, 0 },
    { 0x716bb1f4, 87 },
    { 0x38bbb1f5, 101 },
    { 0x359bb1f5, 97 },
    { 0x44bbb1f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dbb1fa, 23 },
    { 0x6abbb1fa, 33 },
    { 0, 0 },
    { 0x9b9bb1fd, 33 },
    { 0, 0 },
    { 0xa7ebb1ff, 98 },
    { 0, 0 },
    { 0xd69bb201, 61 },
    { 0xeebb202, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476bb20b, 23 },
    { 0x598bb20b, 98 },
    { 0x31fbb20d, 61 },
    { 0xd32bb20e, 76 },
    { 0x213bb20f, 55 },
    { 0x1f5bb210, 78 },
    { 0x65fbb210, 61 },
    { 0x93ebb211, 33 },
    { 0, 0 },
    { 0x64bbb214, 76 },
    { 0x87cbb214, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810bb219, 76 },
    { 0x864bb21a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cbb21f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbb224, 61 },
    { 0x3bebb224, 98 },
    { 0, 0 },
    { 0xd0bbb227, 95 },
    { 0xa98bb228, 52 },
    { 0xc8ebb227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ebb22d, 93 },
    { 0x8a6bb22e, 23 },
    { 0x1b0bb22f, 99 },
    { 0x47fbb230, 23 },
    { 0x16fbb231, 43 },
    { 0x28cbb22f, 99 },
    { 0xbe6bb231, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddabb239, 23 },
    { 0x746bb23a, 23 },
    { 0xb45bb23b, 36 },
    { 0x25fbb23c, 76 },
    { 0, 0 },
    { 0x6adbb23e, 46 },
    { 0x7a5bb23f, 76 },
    { 0x84bb240, 77 },
    { 0xc51bb241, 76 },
    { 0, 0 },
    { 0x9c2bb243, 94 },
    { 0x6bcbb244, 100 },
    { 0, 0 },
    { 0x9afbb246, 40 },
    { 0x7c1bb246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dbb24a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cbb250, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bb253, 90 },
    { 0x768bb254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ebb257, 91 },
    { 0xebebb258, 44 },
    { 0x719bb257, 43 },
    { 0xdd2bb257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729bb25e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8bb261, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bb265, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41bb268, 55 },
    { 0, 0 },
    { 0x51bbb26a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fbb270, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634bb274, 98 },
    { 0, 0 },
    { 0x92cbb276, 33 },
    { 0x9abbb277, 93 },
    { 0x318bb277, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbb27b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506bb281, 23 },
    { 0xeb1bb282, 95 },
    { 0, 0 },
    { 0x2bb284, 43 },
    { 0x148bb285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370bb289, 98 },
    { 0x4bb28a, 52 },
    { 0xaebbb28b, 98 },
    { 0xadabb28b, 98 },
    { 0x8f6bb28d, 23 },
    { 0, 0 },
    { 0xaa0bb28f, 34 },
    { 0x1f3bb290, 91 },
    { 0x17fbb291, 98 },
    { 0x41bb292, 98 },
    { 0x97ebb293, 23 },
    { 0, 0 },
    { 0xc46bb295, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147bb29c, 78 },
    { 0x820bb29d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2abb2a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19bb2aa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bb2af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9bb2b7, 98 },
    { 0x11bb2b7, 34 },
    { 0x3f1bb2b9, 67 },
    { 0x1acbb2ba, 87 },
    { 0x76cbb2bb, 61 },
    { 0x2adbb2bb, 23 },
    { 0xe3bbb2bd, 61 },
    { 0xcc4bb2bc, 34 },
    { 0x9b4bb2bf, 98 },
    { 0x68fbb2bb, 100 },
    { 0x58bb2c1, 61 },
    { 0x253bb2c2, 52 },
    { 0xba3bb2bf, 99 },
    { 0x1bbbb2c0, 99 },
    { 0, 0 },
    { 0x388bb2c6, 90 },
    { 0x345bb2c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfbb2ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fbb2cd, 84 },
    { 0x317bb2cd, 98 },
    { 0xd35bb2cf, 90 },
    { 0xf75bb2cf, 23 },
    { 0, 0 },
    { 0x761bb2d2, 34 },
    { 0x3cdbb2d3, 66 },
    { 0x4e8bb2d4, 91 },
    { 0xcdebb2d3, 95 },
    { 0xb52bb2d6, 66 },
    { 0, 0 },
    { 0xbcdbb2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x743bb2db, 90 },
    { 0xec5bb2db, 99 },
    { 0xfc4bb2db, 2 },
    { 0xf29bb2de, 33 },
    { 0xe94bb2df, 61 },
    { 0x90ebb2de, 76 },
    { 0xecfbb2de, 33 },
    { 0, 0 },
    { 0xc2bbb2e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bb2e7, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0bb2f0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bbb2f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3bb2f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x863bb2fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbbb2fd, 85 },
    { 0xfcbb2fe, 85 },
    { 0xd31bb2ff, 90 },
    { 0x1d1bb2ff, 33 },
    { 0xd95bb301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8abb304, 100 },
    { 0x780bb304, 33 },
    { 0xf40bb306, 55 },
    { 0xc2fbb307, 61 },
    { 0xb40bb307, 55 },
    { 0, 0 },
    { 0xfaebb30a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254bb30f, 95 },
    { 0xf9ebb310, 90 },
    { 0xab0bb311, 67 },
    { 0x6eebb312, 90 },
    { 0x881bb311, 71 },
    { 0x640bb312, 85 },
    { 0x7b0bb315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848bb31d, 98 },
    { 0x212bb31e, 33 },
    { 0, 0 },
    { 0x89cbb320, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19bb324, 72 },
    { 0x238bb324, 98 },
    { 0, 0 },
    { 0xbcbb327, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x501bb32a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbb32d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2bb331, 90 },
    { 0, 0 },
    { 0xd3ebb333, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeebb336, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ebb33d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734bb347, 67 },
    { 0, 0 },
    { 0xcebbb349, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x693bb34c, 76 },
    { 0x995bb34d, 33 },
    { 0x40bbb34e, 61 },
    { 0xa84bb34f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38bb354, 85 },
    { 0, 0 },
    { 0x585bb356, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93bb359, 95 },
    { 0x474bb35a, 97 },
    { 0x3f3bb35b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329bb362, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5bb365, 91 },
    { 0x146bb366, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6bb369, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24bb36d, 76 },
    { 0, 0 },
    { 0x73dbb36f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fbb378, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bb37e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15bb381, 23 },
    { 0, 0 },
    { 0x855bb383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13bb387, 33 },
    { 0xff3bb388, 33 },
    { 0xf33bb388, 90 },
    { 0x8edbb387, 90 },
    { 0xfb5bb387, 55 },
    { 0, 0 },
    { 0x66fbb38d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efbb390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2bb398, 61 },
    { 0x974bb399, 90 },
    { 0xeb7bb398, 95 },
    { 0xe6fbb399, 76 },
    { 0x5e7bb39c, 99 },
    { 0x6a0bb39d, 66 },
    { 0xcabb39b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dbb3a5, 23 },
    { 0x25bb3a6, 98 },
    { 0xe2abb3a5, 33 },
    { 0xae1bb3a6, 55 },
    { 0x3c2bb3a8, 91 },
    { 0x805bb3aa, 23 },
    { 0x807bb3a7, 90 },
    { 0xcccbb3ab, 64 },
    { 0x121bb3ad, 61 },
    { 0xf27bb3ad, 23 },
    { 0x41dbb3ad, 87 },
    { 0x76bb3b0, 61 },
    { 0xc41bb3ad, 95 },
    { 0x7bbb3b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80bb3b7, 95 },
    { 0, 0 },
    { 0x24bb3b9, 76 },
    { 0xe69bb3ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dbb3bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28bb3c7, 34 },
    { 0x791bb3c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x906bb3cb, 55 },
    { 0x870bb3cb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0bb3d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724bb3d4, 61 },
    { 0, 0 },
    { 0xd7ebb3d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebb3db, 46 },
    { 0, 0 },
    { 0x8cbbb3dd, 76 },
    { 0, 0 },
    { 0xc89bb3df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60bb3e3, 55 },
    { 0x484bb3e4, 43 },
    { 0xb40bb3e5, 61 },
    { 0x7bbbb3e3, 33 },
    { 0x6f4bb3e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43bb3eb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ebb3ee, 76 },
    { 0x506bb3ef, 99 },
    { 0x1bdbb3ee, 55 },
    { 0x922bb3f1, 94 },
    { 0x1e4bb3f2, 23 },
    { 0x1f0bb3f1, 61 },
    { 0x127bb3ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295bb3fd, 90 },
    { 0x84ebb3fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfbb401, 90 },
    { 0x9bb401, 34 },
    { 0, 0 },
    { 0x20bb404, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40bb407, 98 },
    { 0xd8bb407, 33 },
    { 0xeebb409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09bb40c, 90 },
    { 0, 0 },
    { 0xbf3bb40e, 34 },
    { 0x69bb40f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368bb414, 34 },
    { 0x534bb414, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69bb418, 68 },
    { 0x338bb419, 90 },
    { 0, 0 },
    { 0x80cbb41b, 98 },
    { 0x12fbb41c, 76 },
    { 0x49abb41d, 101 },
    { 0, 0 },
    { 0xe87bb41f, 55 },
    { 0, 0 },
    { 0x43abb421, 100 },
    { 0xf9bb422, 90 },
    { 0x23dbb422, 52 },
    { 0x957bb421, 34 },
    { 0x5ffbb425, 23 },
    { 0xe00bb424, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68abb431, 100 },
    { 0x8f4bb432, 91 },
    { 0x6e1bb433, 81 },
    { 0xaa0bb434, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8bb43b, 64 },
    { 0, 0 },
    { 0x86fbb43d, 33 },
    { 0x74bb43e, 33 },
    { 0x8bbb43e, 78 },
    { 0x188bb43f, 98 },
    { 0xd1dbb440, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x289bb444, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fbb44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x224bb44e, 61 },
    { 0xa4abb44e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdbb457, 55 },
    { 0x19bbb457, 98 },
    { 0xd6ebb459, 55 },
    { 0x6a4bb458, 95 },
    { 0, 0 },
    { 0x28abb45c, 33 },
    { 0x127bb45d, 76 },
    { 0x9e2bb45d, 46 },
    { 0xef9bb45e, 43 },
    { 0, 0 },
    { 0xa5bb461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ebb468, 34 },
    { 0x293bb469, 43 },
    { 0, 0 },
    { 0x85fbb46b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93bb472, 90 },
    { 0, 0 },
    { 0x10fbb474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb47c, 43 },
    { 0x279bb47d, 33 },
    { 0x19bb47e, 76 },
    { 0xd63bb47f, 61 },
    { 0x165bb480, 93 },
    { 0x21bb47d, 76 },
    { 0x3f7bb47f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443bb488, 90 },
    { 0x4d8bb489, 98 },
    { 0x9b6bb48a, 43 },
    { 0x9dfbb489, 43 },
    { 0x26ebb489, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x612bb48f, 34 },
    { 0x7c0bb48f, 61 },
    { 0, 0 },
    { 0x795bb492, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bb495, 78 },
    { 0, 0 },
    { 0xd60bb497, 98 },
    { 0x1eebb497, 90 },
    { 0x4eebb498, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cbb49e, 46 },
    { 0xf94bb49f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbb4a2, 33 },
    { 0x74ebb4a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62bb4ab, 34 },
    { 0xfddbb4ac, 85 },
    { 0xb59bb4ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305bb4b4, 98 },
    { 0, 0 },
    { 0x127bb4b6, 81 },
    { 0, 0 },
    { 0x38cbb4b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663bb4bd, 90 },
    { 0, 0 },
    { 0x2dbbb4bf, 23 },
    { 0x158bb4bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2bb4c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753bb4c7, 33 },
    { 0x36dbb4c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8bb4cb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bb4d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602bb4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cbb4da, 87 },
    { 0, 0 },
    { 0x5c1bb4dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9abb4df, 43 },
    { 0, 0 },
    { 0xd2bbb4e1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9bb4e8, 34 },
    { 0x3fabb4e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5bb4ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09bb4f3, 66 },
    { 0, 0 },
    { 0x418bb4f5, 43 },
    { 0, 0 },
    { 0x631bb4f7, 33 },
    { 0x1f6bb4f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580bb4fc, 33 },
    { 0xacebb4fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288bb509, 61 },
    { 0xd4bbb50a, 91 },
    { 0xaccbb50b, 48 },
    { 0x35bbb509, 98 },
    { 0, 0 },
    { 0x54dbb50e, 75 },
    { 0x7f7bb50f, 95 },
    { 0xc27bb510, 100 },
    { 0x392bb511, 76 },
    { 0, 0 },
    { 0xc50bb513, 76 },
    { 0x6f0bb513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfbb520, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bbb523, 17 },
    { 0, 0 },
    { 0xe74bb525, 78 },
    { 0x17abb526, 95 },
    { 0, 0 },
    { 0xcfbbb528, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x878bb52b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212bb530, 61 },
    { 0x7c1bb530, 23 },
    { 0x457bb530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4bb535, 95 },
    { 0, 0 },
    { 0x29cbb537, 95 },
    { 0xcabb538, 61 },
    { 0x87fbb539, 90 },
    { 0x3acbb53a, 34 },
    { 0, 0 },
    { 0xa68bb53c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2bb53f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ebb542, 61 },
    { 0x537bb542, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dbb546, 33 },
    { 0, 0 },
    { 0x33abb548, 98 },
    { 0x466bb549, 95 },
    { 0, 0 },
    { 0x24bb54b, 93 },
    { 0x97ebb54b, 98 },
    { 0xc60bb54d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52bb552, 98 },
    { 0, 0 },
    { 0x286bb554, 33 },
    { 0x273bb554, 46 },
    { 0xa1abb556, 81 },
    { 0x7a9bb556, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bb564, 61 },
    { 0xa21bb565, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x697bb568, 33 },
    { 0, 0 },
    { 0x153bb56a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bb56d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dbb573, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5bb578, 69 },
    { 0xfeebb579, 90 },
    { 0xda8bb579, 43 },
    { 0x48bb57a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49bb584, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eebb587, 33 },
    { 0x100bb588, 95 },
    { 0x3cbb589, 61 },
    { 0x628bb587, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x713bb58d, 61 },
    { 0x37cbb58d, 95 },
    { 0xd79bb58e, 55 },
    { 0x701bb590, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbbb594, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71bb598, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96bb5a3, 75 },
    { 0xf4abb5a4, 76 },
    { 0x99dbb5a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34abb5ab, 33 },
    { 0x937bb5ab, 90 },
    { 0x7acbb5ad, 23 },
    { 0x66bb5ab, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cbb5b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98bb5b8, 88 },
    { 0x953bb5b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47bb5bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x191bb5bf, 61 },
    { 0x2b4bb5c0, 33 },
    { 0xf9abb5bf, 75 },
    { 0x911bb5c1, 76 },
    { 0xf32bb5c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736bb5cd, 95 },
    { 0x9a4bb5ce, 76 },
    { 0xb8bbb5cf, 65 },
    { 0x90bb5ce, 76 },
    { 0xf01bb5d1, 55 },
    { 0xe42bb5cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1bb5d7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0bb5e9, 95 },
    { 0xbfbb5ea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727bb5ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74bb5f3, 68 },
    { 0xccbb5f3, 17 },
    { 0, 0 },
    { 0x505bb5f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6bb5f9, 23 },
    { 0x7d8bb5fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279bb601, 88 },
    { 0, 0 },
    { 0x5ffbb603, 33 },
    { 0x61bbb603, 61 },
    { 0x78cbb604, 66 },
    { 0x121bb606, 95 },
    { 0x43bb607, 61 },
    { 0xd77bb608, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfbb614, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x760bb617, 90 },
    { 0x9b0bb617, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7bb620, 23 },
    { 0, 0 },
    { 0x396bb622, 68 },
    { 0, 0 },
    { 0x29cbb624, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bb629, 33 },
    { 0x9febb629, 98 },
    { 0xb25bb62a, 43 },
    { 0x1b0bb62c, 61 },
    { 0, 0 },
    { 0x960bb62e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424bb636, 61 },
    { 0, 0 },
    { 0xc3fbb638, 98 },
    { 0x40abb639, 52 },
    { 0xb3fbb63a, 76 },
    { 0x18ebb639, 100 },
    { 0x6e5bb63c, 90 },
    { 0, 0 },
    { 0xedabb63e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0bb641, 76 },
    { 0x70ebb641, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65bb645, 76 },
    { 0x467bb645, 34 },
    { 0, 0 },
    { 0x79abb648, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61bb64f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ebb652, 90 },
    { 0, 0 },
    { 0x7dabb654, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825bb65c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeabb660, 77 },
    { 0, 0 },
    { 0xf43bb662, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8bb665, 17 },
    { 0x74fbb666, 61 },
    { 0x381bb666, 95 },
    { 0, 0 },
    { 0x491bb669, 33 },
    { 0xc0bb66a, 33 },
    { 0xa1dbb66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dabb66e, 61 },
    { 0xf95bb66f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6bb674, 23 },
    { 0xa53bb675, 33 },
    { 0x288bb675, 33 },
    { 0x3fbb677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ebb67a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2bb681, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x283bb684, 84 },
    { 0, 0 },
    { 0x5b2bb686, 33 },
    { 0xa0dbb687, 26 },
    { 0x477bb687, 90 },
    { 0xe4cbb687, 90 },
    { 0xc9ebb687, 69 },
    { 0x200bb68b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172bb692, 55 },
    { 0x453bb693, 33 },
    { 0, 0 },
    { 0x1edbb695, 46 },
    { 0x72fbb696, 90 },
    { 0x2cbbb697, 98 },
    { 0xd46bb697, 91 },
    { 0x16dbb697, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9bb69d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bb6a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132bb6a6, 90 },
    { 0x2a7bb6a6, 33 },
    { 0x872bb6a7, 76 },
    { 0x76bbb6a9, 33 },
    { 0x2ebbb6aa, 95 },
    { 0x96fbb6aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34bb6b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dbb6b6, 61 },
    { 0, 0 },
    { 0x28abb6b8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbb6be, 90 },
    { 0x4cabb6bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8febb6ca, 98 },
    { 0xf72bb6cb, 98 },
    { 0, 0 },
    { 0x893bb6cd, 75 },
    { 0x10dbb6ce, 61 },
    { 0x42fbb6cf, 81 },
    { 0x8e5bb6ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939bb6d5, 100 },
    { 0x306bb6d6, 46 },
    { 0, 0 },
    { 0xf1fbb6d8, 90 },
    { 0, 0 },
    { 0xd7dbb6da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0abb6df, 90 },
    { 0xf01bb6e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892bb6e5, 91 },
    { 0x1aebb6e6, 38 },
    { 0x872bb6e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2bb6ec, 90 },
    { 0x2c5bb6ed, 90 },
    { 0x44bbb6ed, 33 },
    { 0xd08bb6ef, 33 },
    { 0xcb5bb6ef, 43 },
    { 0xd4cbb6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4bb6f4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bb6f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7bb6fb, 61 },
    { 0x5e1bb6fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xedabb6ff, 34 },
    { 0x8a5bb700, 90 },
    { 0x76cbb701, 90 },
    { 0, 0 },
    { 0x48abb703, 99 },
    { 0xf8bb703, 81 },
    { 0x478bb705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6bb70d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfbb718, 90 },
    { 0, 0 },
    { 0xd6ebb71a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bb71e, 43 },
    { 0x9ccbb71e, 90 },
    { 0x412bb71f, 90 },
    { 0xc4fbb721, 33 },
    { 0xeacbb722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeebb728, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38bb72c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0bb733, 46 },
    { 0x1e6bb734, 66 },
    { 0xa70bb735, 33 },
    { 0xd6bb735, 90 },
    { 0xfcfbb737, 98 },
    { 0x318bb738, 90 },
    { 0x787bb737, 46 },
    { 0x6d5bb73a, 100 },
    { 0, 0 },
    { 0x5e5bb73c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549bb740, 55 },
    { 0x313bb741, 68 },
    { 0x38dbb741, 98 },
    { 0xb95bb740, 40 },
    { 0x116bb744, 76 },
    { 0xb51bb742, 85 },
    { 0x21fbb746, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9bb74c, 98 },
    { 0, 0 },
    { 0x3aebb74e, 90 },
    { 0, 0 },
    { 0xe18bb750, 52 },
    { 0xe56bb751, 84 },
    { 0, 0 },
    { 0xdd2bb753, 34 },
    { 0, 0 },
    { 0x556bb755, 23 },
    { 0x932bb755, 33 },
    { 0x4a1bb757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bb75a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedbb764, 68 },
    { 0x13dbb764, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62bb768, 84 },
    { 0x1fdbb769, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x558bb76c, 95 },
    { 0x358bb76c, 76 },
    { 0xe1ebb76e, 90 },
    { 0x7d4bb76f, 40 },
    { 0x408bb76f, 33 },
    { 0x229bb76c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x539bb774, 23 },
    { 0xb63bb775, 76 },
    { 0x66dbb775, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0bb77c, 90 },
    { 0x46dbb77c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dbb781, 55 },
    { 0x2cdbb781, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31bb786, 43 },
    { 0xd5ebb787, 90 },
    { 0, 0 },
    { 0x6dfbb789, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6febb78c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecbb794, 98 },
    { 0x6c1bb795, 33 },
    { 0x9abbb795, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73bb799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5bb7a0, 34 },
    { 0x842bb7a1, 98 },
    { 0xd62bb7a2, 52 },
    { 0x11dbb7a2, 67 },
    { 0x815bb7a4, 43 },
    { 0x845bb7a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fabb7ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdbb7af, 93 },
    { 0x99bbb7b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x639bb7b3, 90 },
    { 0xf2cbb7b3, 89 },
    { 0x445bb7b5, 99 },
    { 0xbd5bb7b6, 43 },
    { 0, 0 },
    { 0x690bb7b8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fbb7bb, 61 },
    { 0x15bb7bc, 55 },
    { 0, 0 },
    { 0x559bb7be, 99 },
    { 0, 0 },
    { 0xafbbb7c0, 90 },
    { 0x825bb7c1, 99 },
    { 0xbcebb7c2, 52 },
    { 0x73bb7c2, 95 },
    { 0, 0 },
    { 0xeadbb7c5, 33 },
    { 0x29cbb7c6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6bb7c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bb7d0, 23 },
    { 0x3fdbb7d0, 85 },
    { 0x8afbb7d0, 33 },
    { 0xf36bb7d2, 61 },
    { 0, 0 },
    { 0xeaebb7d5, 70 },
    { 0, 0 },
    { 0x88ebb7d7, 99 },
    { 0xba6bb7d7, 34 },
    { 0, 0 },
    { 0x127bb7da, 85 },
    { 0xe7dbb7db, 95 },
    { 0, 0 },
    { 0x1d0bb7dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00bb7e4, 23 },
    { 0x453bb7e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdbb7ea, 33 },
    { 0xf00bb7eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558bb7f0, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871bb7f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbb7fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3bb7fd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fbb800, 61 },
    { 0x10fbb801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4bb807, 66 },
    { 0, 0 },
    { 0xd88bb809, 55 },
    { 0x94cbb809, 95 },
    { 0, 0 },
    { 0xfffbb80c, 55 },
    { 0, 0 },
    { 0xe6ebb80e, 93 },
    { 0xe41bb80e, 68 },
    { 0x890bb810, 90 },
    { 0, 0 },
    { 0x7febb812, 95 },
    { 0, 0 },
    { 0xa62bb814, 33 },
    { 0, 0 },
    { 0x1b2bb816, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303bb81a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bb81e, 23 },
    { 0x47dbb81f, 91 },
    { 0xb34bb81f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55bb828, 78 },
    { 0xb86bb829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2bb82c, 85 },
    { 0, 0 },
    { 0x895bb82e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bb833, 95 },
    { 0, 0 },
    { 0x466bb835, 33 },
    { 0xc0bb836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14bb840, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfebb843, 77 },
    { 0x5f1bb843, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79bb847, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fbb84b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15bb854, 95 },
    { 0xf01bb855, 93 },
    { 0x490bb855, 61 },
    { 0xfe9bb857, 23 },
    { 0x15ebb858, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ebb85e, 90 },
    { 0, 0 },
    { 0x715bb860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafabb865, 78 },
    { 0x5b8bb865, 61 },
    { 0, 0 },
    { 0x654bb868, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8bb86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cbb86f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cbb874, 46 },
    { 0xe44bb875, 66 },
    { 0, 0 },
    { 0xd3bb877, 33 },
    { 0, 0 },
    { 0xde1bb879, 91 },
    { 0, 0 },
    { 0xd4abb87b, 66 },
    { 0xfdebb87b, 23 },
    { 0x8d2bb87d, 94 },
    { 0x9a6bb87b, 23 },
    { 0, 0 },
    { 0xb59bb880, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x991bb883, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bbb887, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64bb88c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edbb890, 95 },
    { 0x3ddbb891, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0bb894, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efbb898, 95 },
    { 0, 0 },
    { 0xbdbbb89a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ebb89d, 33 },
    { 0x1b5bb89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cebb8a8, 43 },
    { 0xf55bb8a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ebb8af, 52 },
    { 0, 0 },
    { 0x3bcbb8b1, 85 },
    { 0xdbdbb8b2, 90 },
    { 0x542bb8b2, 52 },
    { 0, 0 },
    { 0x7cbbb8b5, 61 },
    { 0, 0 },
    { 0xb9cbb8b7, 90 },
    { 0, 0 },
    { 0xfd5bb8b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dabb8c0, 84 },
    { 0x747bb8c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09bb8c5, 61 },
    { 0x44ebb8c6, 76 },
    { 0, 0 },
    { 0x186bb8c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3abb8ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77bb8d2, 97 },
    { 0xc46bb8d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1bb8d9, 33 },
    { 0, 0 },
    { 0x84bb8db, 55 },
    { 0x78abb8db, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cbb8e4, 95 },
    { 0xd05bb8e5, 76 },
    { 0x5a2bb8e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbebb8eb, 61 },
    { 0x7cabb8ec, 81 },
    { 0x4cabb8ed, 85 },
    { 0, 0 },
    { 0x49abb8ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54bb8f2, 61 },
    { 0x7d7bb8f3, 92 },
    { 0x128bb8f3, 61 },
    { 0x9a9bb8f3, 61 },
    { 0xadbb8f3, 61 },
    { 0x6d8bb8f7, 95 },
    { 0, 0 },
    { 0x27cbb8f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31bb8fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bb903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5bb90a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x306bb90d, 33 },
    { 0xbcfbb90e, 33 },
    { 0xb1dbb90f, 34 },
    { 0x2ebb910, 93 },
    { 0x77abb911, 90 },
    { 0xef3bb90e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5bb917, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0bb91c, 33 },
    { 0x862bb91d, 23 },
    { 0, 0 },
    { 0xdfebb91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbb923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64abb926, 90 },
    { 0xb98bb927, 61 },
    { 0xa7dbb926, 61 },
    { 0xf42bb928, 46 },
    { 0x29ebb92a, 61 },
    { 0xdb7bb92b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eabb92e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285bb932, 93 },
    { 0x1b9bb932, 33 },
    { 0x228bb934, 61 },
    { 0, 0 },
    { 0xce6bb936, 61 },
    { 0x9e0bb937, 61 },
    { 0x4a5bb937, 86 },
    { 0x5cbbb938, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccbb940, 26 },
    { 0x995bb940, 66 },
    { 0x5e3bb940, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842bb946, 36 },
    { 0xac1bb947, 33 },
    { 0, 0 },
    { 0x7f7bb949, 24 },
    { 0xa9bbb949, 87 },
    { 0x90fbb949, 23 },
    { 0x532bb949, 40 },
    { 0x803bb94d, 78 },
    { 0xc0ebb94d, 85 },
    { 0, 0 },
    { 0xb75bb950, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9bb958, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebb95c, 33 },
    { 0, 0 },
    { 0x259bb95e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7bb961, 95 },
    { 0x8dbb962, 77 },
    { 0xd33bb963, 76 },
    { 0x8b2bb961, 23 },
    { 0x967bb965, 33 },
    { 0xddebb961, 55 },
    { 0x43ebb967, 90 },
    { 0x18fbb967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80bb970, 23 },
    { 0, 0 },
    { 0x154bb972, 61 },
    { 0xa4abb972, 95 },
    { 0x748bb974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fbb97e, 49 },
    { 0x8afbb97f, 95 },
    { 0, 0 },
    { 0xd00bb981, 87 },
    { 0x474bb982, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cbb988, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24bb98c, 91 },
    { 0x492bb98c, 66 },
    { 0x54cbb98e, 61 },
    { 0x2c7bb98f, 78 },
    { 0x268bb98f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940bb995, 43 },
    { 0xdbbb996, 90 },
    { 0, 0 },
    { 0x8a2bb998, 33 },
    { 0x9e4bb999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9bb99d, 95 },
    { 0xa9bb99d, 66 },
    { 0xdcebb99f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fbb9a5, 61 },
    { 0xc97bb9a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41bb9a9, 95 },
    { 0x710bb9a9, 75 },
    { 0, 0 },
    { 0x4bbbb9ac, 101 },
    { 0x19cbb9ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7abb9b0, 66 },
    { 0, 0 },
    { 0x2abbb9b2, 61 },
    { 0, 0 },
    { 0x9bdbb9b4, 33 },
    { 0x185bb9b5, 93 },
    { 0, 0 },
    { 0x6b7bb9b7, 55 },
    { 0x440bb9b8, 46 },
    { 0x8b6bb9b9, 23 },
    { 0, 0 },
    { 0x908bb9bb, 93 },
    { 0, 0 },
    { 0xd15bb9bd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673bb9c2, 61 },
    { 0xfbfbb9c3, 91 },
    { 0x54ebb9c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fbb9c7, 67 },
    { 0, 0 },
    { 0x58bb9c9, 98 },
    { 0, 0 },
    { 0x248bb9cb, 93 },
    { 0x374bb9cc, 43 },
    { 0x15abb9cb, 87 },
    { 0x6b3bb9cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eebb9d4, 90 },
    { 0xd32bb9d5, 33 },
    { 0, 0 },
    { 0xc7dbb9d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8bb9e0, 78 },
    { 0, 0 },
    { 0x892bb9e2, 90 },
    { 0, 0 },
    { 0xccabb9e4, 23 },
    { 0x201bb9e5, 34 },
    { 0, 0 },
    { 0xff1bb9e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bb9eb, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852bb9f4, 90 },
    { 0x6bfbb9f5, 96 },
    { 0x78bb9f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcbb9fd, 61 },
    { 0x9abbb9fe, 67 },
    { 0xe15bb9fe, 61 },
    { 0x340bba00, 76 },
    { 0xca8bba01, 23 },
    { 0x93cbba00, 76 },
    { 0x2e9bba01, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cbba0d, 76 },
    { 0x54cbba0e, 93 },
    { 0x56fbba0f, 95 },
    { 0xdfcbba0f, 90 },
    { 0x784bba0f, 95 },
    { 0x3aabba12, 90 },
    { 0x9b6bba0d, 95 },
    { 0, 0 },
    { 0x152bba15, 61 },
    { 0, 0 },
    { 0x898bba17, 99 },
    { 0x2a2bba17, 33 },
    { 0xe9cbba18, 23 },
    { 0x530bba1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37bba1d, 90 },
    { 0x3fbba1e, 66 },
    { 0xa8ebba1e, 33 },
    { 0xd0bbba1d, 69 },
    { 0xac7bba21, 76 },
    { 0xc9dbba21, 46 },
    { 0x67abba23, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7abba28, 91 },
    { 0x7ffbba28, 33 },
    { 0xa8cbba2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aebba2f, 34 },
    { 0, 0 },
    { 0xe33bba31, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183bba35, 93 },
    { 0, 0 },
    { 0xa39bba37, 34 },
    { 0x439bba38, 40 },
    { 0x1a7bba39, 34 },
    { 0xb70bba3a, 23 },
    { 0, 0 },
    { 0x923bba3c, 80 },
    { 0, 0 },
    { 0xd5bbba3e, 91 },
    { 0, 0 },
    { 0xd9ebba40, 76 },
    { 0x6d7bba41, 83 },
    { 0x4d2bba42, 33 },
    { 0xb56bba43, 43 },
    { 0x897bba41, 76 },
    { 0x859bba43, 95 },
    { 0x9fdbba46, 34 },
    { 0xc51bba46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6bba4b, 23 },
    { 0, 0 },
    { 0x617bba4d, 90 },
    { 0xe85bba4e, 98 },
    { 0x660bba4f, 55 },
    { 0xb98bba50, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bba55, 33 },
    { 0xf0abba56, 90 },
    { 0, 0 },
    { 0xb0bba58, 52 },
    { 0x44abba59, 61 },
    { 0xb05bba5a, 90 },
    { 0xaf9bba5b, 90 },
    { 0xc37bba5b, 33 },
    { 0xd5cbba59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x947bba60, 95 },
    { 0xc5fbba60, 90 },
    { 0xd0ebba61, 98 },
    { 0x9b9bba63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3bba66, 33 },
    { 0xce4bba67, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0dbba6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5abba6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606bba72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133bba76, 90 },
    { 0x941bba77, 33 },
    { 0x784bba77, 69 },
    { 0, 0 },
    { 0xe76bba7a, 93 },
    { 0, 0 },
    { 0x1f8bba7c, 33 },
    { 0x479bba7c, 77 },
    { 0, 0 },
    { 0x9c8bba7f, 36 },
    { 0x16cbba80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x941bba83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4abba87, 78 },
    { 0x4e8bba88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x184bba8b, 90 },
    { 0x1bbbba8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdbba91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6bba96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89bba99, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8bba9d, 77 },
    { 0x706bba9e, 87 },
    { 0x705bba9e, 90 },
    { 0x6e6bba9e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeabbaa6, 33 },
    { 0, 0 },
    { 0x6f6bbaa8, 67 },
    { 0xf5abbaa9, 33 },
    { 0x7eabbaaa, 91 },
    { 0x6dabbaaa, 61 },
    { 0x43fbbaaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992bbab0, 33 },
    { 0x34abbab1, 61 },
    { 0, 0 },
    { 0x103bbab3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816bbab8, 23 },
    { 0x7b4bbab9, 85 },
    { 0x2dbbbab9, 77 },
    { 0x583bbabb, 33 },
    { 0x97bbbabc, 23 },
    { 0x1d0bbabc, 88 },
    { 0xf2ebbabe, 17 },
    { 0x848bbabd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbac2, 61 },
    { 0xf63bbac3, 95 },
    { 0xd79bbac3, 98 },
    { 0xa55bbac5, 33 },
    { 0xc47bbac6, 55 },
    { 0x430bbac7, 98 },
    { 0, 0 },
    { 0x164bbac9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fbbacc, 64 },
    { 0xc94bbacd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98bbad0, 65 },
    { 0x4ddbbad1, 34 },
    { 0, 0 },
    { 0xebbbbad3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbbad9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x319bbadc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356bbae0, 61 },
    { 0xbd3bbae1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfbbae7, 61 },
    { 0x33dbbae8, 17 },
    { 0, 0 },
    { 0x8cdbbaea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2bbaf1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292bbaf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528bbb00, 61 },
    { 0, 0 },
    { 0xaddbbb02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7bbb07, 33 },
    { 0x822bbb08, 55 },
    { 0xca8bbb09, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bbbb1c, 90 },
    { 0xe8ebbb1d, 46 },
    { 0xc83bbb1e, 66 },
    { 0x299bbb1f, 61 },
    { 0xf95bbb20, 23 },
    { 0x487bbb1e, 80 },
    { 0xa59bbb22, 78 },
    { 0x10bbb23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbbbb27, 34 },
    { 0x72bbbb28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x159bbb2b, 33 },
    { 0, 0 },
    { 0x61abbb2d, 33 },
    { 0xc05bbb2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99bbb31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cbbb34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbbbb39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efbbb42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29bbb47, 98 },
    { 0x7c6bbb48, 98 },
    { 0xd8bbb49, 98 },
    { 0x7dfbbb49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dbbb4d, 77 },
    { 0x64dbbb4d, 90 },
    { 0x1dcbbb4d, 61 },
    { 0x5a5bbb50, 95 },
    { 0x60abbb4e, 90 },
    { 0x98fbbb52, 98 },
    { 0x4e6bbb53, 34 },
    { 0, 0 },
    { 0x1d0bbb55, 33 },
    { 0x9fbbb55, 76 },
    { 0xd33bbb56, 23 },
    { 0xf11bbb58, 91 },
    { 0xe01bbb58, 40 },
    { 0x2babbb5a, 90 },
    { 0xb7dbbb57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79bbb5f, 90 },
    { 0, 0 },
    { 0x7bebbb61, 55 },
    { 0x9babbb62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637bbb69, 90 },
    { 0xc7ebbb69, 95 },
    { 0xbe0bbb69, 34 },
    { 0, 0 },
    { 0x339bbb6d, 88 },
    { 0x371bbb6e, 85 },
    { 0xebbbbb6d, 33 },
    { 0, 0 },
    { 0xb0bbbb71, 33 },
    { 0xbcdbbb71, 61 },
    { 0, 0 },
    { 0xf62bbb74, 23 },
    { 0, 0 },
    { 0x9bbbbb76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fbbb80, 93 },
    { 0, 0 },
    { 0xc45bbb82, 23 },
    { 0xc95bbb83, 61 },
    { 0x945bbb84, 55 },
    { 0, 0 },
    { 0xb60bbb86, 23 },
    { 0xdb5bbb86, 61 },
    { 0, 0 },
    { 0x832bbb89, 90 },
    { 0, 0 },
    { 0x4b5bbb8b, 98 },
    { 0x4a5bbb8b, 33 },
    { 0xc17bbb8d, 76 },
    { 0x2f3bbb8e, 90 },
    { 0xac5bbb8b, 81 },
    { 0, 0 },
    { 0x15cbbb91, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cabbb96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bbbb9d, 29 },
    { 0, 0 },
    { 0x691bbb9f, 95 },
    { 0x7d8bbb9f, 33 },
    { 0xc76bbb9f, 61 },
    { 0x748bbba1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75bbba6, 23 },
    { 0xa38bbba7, 52 },
    { 0, 0 },
    { 0x888bbba9, 75 },
    { 0xe6dbbbaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ebbbad, 76 },
    { 0xe4bbbad, 90 },
    { 0, 0 },
    { 0xec5bbbb0, 61 },
    { 0x938bbbb0, 55 },
    { 0xafcbbbb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9bbbbb, 95 },
    { 0xc42bbbbb, 33 },
    { 0xdc0bbbbc, 90 },
    { 0x8cfbbbbb, 65 },
    { 0x5afbbbbf, 95 },
    { 0x46cbbbc0, 33 },
    { 0x379bbbc1, 52 },
    { 0xa62bbbbd, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5bbbc5, 90 },
    { 0, 0 },
    { 0x52cbbbc7, 91 },
    { 0x94dbbbc8, 33 },
    { 0x786bbbc8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662bbbcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bbbd1, 93 },
    { 0x7bcbbbd2, 93 },
    { 0xb5abbbd3, 66 },
    { 0x825bbbd3, 98 },
    { 0x3dcbbbd3, 34 },
    { 0x7d0bbbd1, 43 },
    { 0, 0 },
    { 0xb4bbbd8, 85 },
    { 0, 0 },
    { 0x3c6bbbda, 98 },
    { 0x7febbbdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bbbe7, 55 },
    { 0, 0 },
    { 0x95abbbe9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240bbbed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfbbbf3, 68 },
    { 0x68dbbbf4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbbbf7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bbbfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabbc04, 58 },
    { 0xaf4bbc05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09bbc09, 66 },
    { 0, 0 },
    { 0x47fbbc0b, 67 },
    { 0xd14bbc0c, 61 },
    { 0xfebbbc0c, 34 },
    { 0xc0fbbc0e, 90 },
    { 0x925bbc0e, 98 },
    { 0x910bbc10, 33 },
    { 0x341bbc11, 33 },
    { 0, 0 },
    { 0xf78bbc13, 33 },
    { 0x31bbbc13, 23 },
    { 0x23cbbc15, 95 },
    { 0xad2bbc16, 55 },
    { 0, 0 },
    { 0xc90bbc18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fbbc1c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ebbc25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1bbc28, 95 },
    { 0x356bbc29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fbbc2c, 90 },
    { 0, 0 },
    { 0x5c7bbc2e, 95 },
    { 0xd16bbc2f, 65 },
    { 0, 0 },
    { 0x20dbbc31, 77 },
    { 0x286bbc31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dbbc36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0bbc39, 87 },
    { 0xf79bbc3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cbbc3e, 78 },
    { 0x441bbc3f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363bbc44, 66 },
    { 0xfe4bbc45, 85 },
    { 0xe85bbc46, 59 },
    { 0x2e6bbc46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1bbc4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594bbc4f, 46 },
    { 0x3ecbbc50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214bbc54, 55 },
    { 0x7c1bbc55, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8bbc59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebbc5f, 61 },
    { 0x563bbc60, 61 },
    { 0, 0 },
    { 0x578bbc62, 85 },
    { 0x50fbbc62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ebbc69, 61 },
    { 0, 0 },
    { 0xef4bbc6b, 96 },
    { 0, 0 },
    { 0xc0cbbc6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946bbc72, 90 },
    { 0, 0 },
    { 0x9e7bbc74, 95 },
    { 0, 0 },
    { 0x5acbbc76, 23 },
    { 0, 0 },
    { 0x4b4bbc78, 90 },
    { 0, 0 },
    { 0x997bbc7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8bbc7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbbc84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22bbc8d, 52 },
    { 0xb81bbc8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cbbc96, 61 },
    { 0xed6bbc97, 23 },
    { 0xc4bbbc98, 33 },
    { 0x6debbc98, 87 },
    { 0xbc3bbc9a, 61 },
    { 0xc04bbc9b, 98 },
    { 0x84abbc97, 23 },
    { 0xc72bbc9d, 33 },
    { 0x679bbc98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5bbca4, 98 },
    { 0, 0 },
    { 0x256bbca6, 95 },
    { 0, 0 },
    { 0xa20bbca8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efbbcac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ebbcb0, 100 },
    { 0x336bbcb1, 90 },
    { 0x571bbcb1, 95 },
    { 0, 0 },
    { 0x833bbcb4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90bbcb7, 100 },
    { 0x5e2bbcb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbbbcbb, 95 },
    { 0x695bbcbc, 93 },
    { 0xb79bbcbd, 52 },
    { 0, 0 },
    { 0xdb2bbcbf, 90 },
    { 0, 0 },
    { 0x75bbbcc1, 73 },
    { 0x5acbbcc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbbcc7, 90 },
    { 0x714bbcc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fbbccc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912bbcd0, 43 },
    { 0xbdfbbcd1, 85 },
    { 0xce6bbcd1, 76 },
    { 0x608bbcd3, 43 },
    { 0xa1fbbcd4, 61 },
    { 0xea8bbcd0, 98 },
    { 0x707bbcd6, 98 },
    { 0x7ebbbcd4, 43 },
    { 0x299bbcd0, 61 },
    { 0x43cbbcd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975bbce0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cbbce3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48bbce6, 95 },
    { 0, 0 },
    { 0xa67bbce8, 78 },
    { 0x917bbce9, 96 },
    { 0x4f8bbce9, 33 },
    { 0, 0 },
    { 0x3bcbbcec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abbcf5, 61 },
    { 0x67dbbcf5, 76 },
    { 0x9b6bbcf5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dbbcfe, 91 },
    { 0, 0 },
    { 0x5debbd00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172bbd05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabbd0a, 87 },
    { 0x2f1bbd0b, 61 },
    { 0x81dbbd0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bbbd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x630bbd1d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95bbd28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993bbd2e, 34 },
    { 0x578bbd2f, 61 },
    { 0x23abbd30, 90 },
    { 0x6cdbbd31, 75 },
    { 0xa2fbbd31, 98 },
    { 0xecebbd31, 33 },
    { 0x436bbd34, 76 },
    { 0, 0 },
    { 0x8f3bbd36, 77 },
    { 0x9ddbbd37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527bbd3b, 23 },
    { 0x4e7bbd3b, 95 },
    { 0x15ebbd3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efbbd42, 17 },
    { 0, 0 },
    { 0x552bbd44, 33 },
    { 0x940bbd44, 98 },
    { 0x257bbd46, 55 },
    { 0, 0 },
    { 0xbb9bbd48, 98 },
    { 0x420bbd49, 34 },
    { 0, 0 },
    { 0x7a0bbd4b, 95 },
    { 0x324bbd4c, 17 },
    { 0, 0 },
    { 0xfabbd4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2bbd53, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x322bbd56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6bbd5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dbbd61, 67 },
    { 0x50ebbd62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0bbd6a, 38 },
    { 0x658bbd6b, 68 },
    { 0x247bbd6c, 99 },
    { 0xaabbbd6c, 17 },
    { 0x2b3bbd6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59bbd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964bbd79, 98 },
    { 0xc40bbd7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cabbd7e, 95 },
    { 0, 0 },
    { 0xc4bbd80, 78 },
    { 0xd8bbd81, 91 },
    { 0, 0 },
    { 0xfbebbd83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aebbd8d, 99 },
    { 0x469bbd8e, 84 },
    { 0xa84bbd8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bbbd92, 95 },
    { 0, 0 },
    { 0x1debbd94, 67 },
    { 0x8b0bbd95, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8bbda0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69bbdac, 77 },
    { 0xd9cbbdad, 91 },
    { 0x137bbdad, 93 },
    { 0x871bbdad, 90 },
    { 0x243bbdad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcebbdb3, 91 },
    { 0, 0 },
    { 0x45ebbdb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x837bbdb8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbbdbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439bbdc0, 61 },
    { 0x24bbbdc1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x374bbdc4, 61 },
    { 0xe0dbbdc4, 33 },
    { 0xb11bbdc6, 24 },
    { 0x102bbdc7, 46 },
    { 0, 0 },
    { 0xe92bbdc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69bbdcc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304bbdd3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180bbdda, 65 },
    { 0xdd6bbdda, 92 },
    { 0xfdbbddb, 96 },
    { 0x877bbddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98bbde0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bbbde4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bbdec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeebbdef, 90 },
    { 0xf8dbbdf0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x43abbdf3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5bbdfc, 69 },
    { 0x2a4bbdfd, 75 },
    { 0xe38bbdfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbe01, 90 },
    { 0x36fbbe02, 34 },
    { 0, 0 },
    { 0xa49bbe04, 95 },
    { 0x922bbe05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ebbe0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bbbe12, 90 },
    { 0x13bbbe12, 33 },
    { 0x693bbe14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x321bbe17, 95 },
    { 0, 0 },
    { 0xdf0bbe19, 98 },
    { 0x571bbe1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3bbe25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409bbe2a, 95 },
    { 0, 0 },
    { 0x1e2bbe2c, 23 },
    { 0x5abbbe2d, 33 },
    { 0x369bbe2c, 84 },
    { 0, 0 },
    { 0xcf1bbe30, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bbe34, 95 },
    { 0, 0 },
    { 0x244bbe36, 80 },
    { 0x8b0bbe36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ebbe41, 87 },
    { 0x697bbe41, 61 },
    { 0x7b2bbe43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3bbe46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x756bbe49, 34 },
    { 0x486bbe4a, 98 },
    { 0x6b7bbe4a, 95 },
    { 0x18bbe4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28bbe53, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fbbe57, 87 },
    { 0xf29bbe57, 91 },
    { 0x750bbe59, 33 },
    { 0xb0dbbe59, 43 },
    { 0xbc6bbe5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460bbe61, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26bbe64, 33 },
    { 0xe85bbe64, 61 },
    { 0x32fbbe66, 84 },
    { 0x969bbe66, 75 },
    { 0x11ebbe66, 95 },
    { 0x307bbe66, 93 },
    { 0, 0 },
    { 0x606bbe6b, 100 },
    { 0x96abbe6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdbbe72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3bbe75, 76 },
    { 0, 0 },
    { 0x5b1bbe77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38bbe7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fbbe7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08bbe80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8bbe89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fbbe8d, 55 },
    { 0x965bbe8d, 55 },
    { 0x83bbe8f, 98 },
    { 0x2dbbe8f, 90 },
    { 0x7bbbbe90, 55 },
    { 0xc4cbbe92, 88 },
    { 0x119bbe92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2bbe97, 76 },
    { 0xdb1bbe97, 46 },
    { 0x98bbbe99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5bbe9d, 34 },
    { 0, 0 },
    { 0xa95bbe9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1bbea2, 84 },
    { 0, 0 },
    { 0xc1bbea4, 55 },
    { 0xf00bbea5, 93 },
    { 0, 0 },
    { 0x119bbea7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x282bbeaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14bbeaf, 98 },
    { 0xba5bbeb0, 33 },
    { 0xdc2bbeb0, 92 },
    { 0x2c6bbeb2, 46 },
    { 0xd62bbeb1, 23 },
    { 0x8eebbeb2, 34 },
    { 0, 0 },
    { 0x6cdbbeb6, 90 },
    { 0x609bbeb6, 90 },
    { 0x4eabbeb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6bbec3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x135bbec6, 98 },
    { 0, 0 },
    { 0xe46bbec8, 23 },
    { 0, 0 },
    { 0x69ebbeca, 61 },
    { 0xe7cbbecb, 23 },
    { 0x463bbecc, 33 },
    { 0xc48bbecb, 23 },
    { 0x8a4bbece, 68 },
    { 0x9adbbecb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afbbed3, 90 },
    { 0x663bbed3, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81bbed7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abbbedc, 91 },
    { 0, 0 },
    { 0xfa2bbede, 66 },
    { 0x294bbedf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadbbee7, 66 },
    { 0x179bbee8, 46 },
    { 0xb1ebbee9, 76 },
    { 0x31ebbeea, 91 },
    { 0x857bbeeb, 76 },
    { 0x94ebbeec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8bbeef, 100 },
    { 0, 0 },
    { 0x678bbef1, 43 },
    { 0x894bbef2, 43 },
    { 0xc5bbef3, 76 },
    { 0x369bbef3, 77 },
    { 0, 0 },
    { 0xa07bbef6, 61 },
    { 0x776bbef6, 98 },
    { 0xdecbbef8, 61 },
    { 0xd33bbef9, 76 },
    { 0xf4abbef9, 87 },
    { 0, 0 },
    { 0x24abbefc, 61 },
    { 0, 0 },
    { 0xa7ebbefe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3bbf02, 46 },
    { 0xf63bbf02, 98 },
    { 0x241bbf03, 43 },
    { 0x4bdbbf02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dbbf08, 34 },
    { 0, 0 },
    { 0x206bbf0a, 99 },
    { 0xdebbf0b, 99 },
    { 0, 0 },
    { 0x179bbf0d, 53 },
    { 0, 0 },
    { 0x711bbf0f, 34 },
    { 0xf59bbf0f, 76 },
    { 0xc09bbf11, 67 },
    { 0x7f0bbf12, 23 },
    { 0xe3abbf12, 61 },
    { 0x602bbf14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x219bbf17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2bbf21, 33 },
    { 0xe91bbf22, 33 },
    { 0x1dbbf22, 91 },
    { 0x362bbf22, 33 },
    { 0, 0 },
    { 0xde7bbf26, 19 },
    { 0xe67bbf27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bbf2a, 64 },
    { 0, 0 },
    { 0x758bbf2c, 55 },
    { 0xb91bbf2d, 87 },
    { 0xe60bbf2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18bbf32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bbbf36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8bbf3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dbbf3f, 95 },
    { 0, 0 },
    { 0xb90bbf41, 98 },
    { 0, 0 },
    { 0x9d9bbf43, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142bbf47, 33 },
    { 0xd4bbbf48, 61 },
    { 0xb51bbf49, 55 },
    { 0, 0 },
    { 0x82ebbf4b, 65 },
    { 0x9c5bbf4b, 76 },
    { 0x5e4bbf4b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbbbf50, 61 },
    { 0xff1bbf50, 95 },
    { 0x60bbf52, 98 },
    { 0xd68bbf53, 95 },
    { 0xf70bbf53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3bbf58, 61 },
    { 0xe6bbbf58, 33 },
    { 0xfa0bbf59, 100 },
    { 0, 0 },
    { 0x5b8bbf5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bbf5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fbbf65, 23 },
    { 0xc0cbbf65, 90 },
    { 0x27bbbf65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcbbf6b, 91 },
    { 0xcadbbf6c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6bbf70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728bbf76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96bbf7a, 98 },
    { 0, 0 },
    { 0x613bbf7c, 90 },
    { 0xec4bbf7d, 95 },
    { 0x38bbf7e, 61 },
    { 0, 0 },
    { 0x244bbf80, 81 },
    { 0x750bbf80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67bbf86, 90 },
    { 0x2cbbbf87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0bbf8a, 98 },
    { 0xa0bbf8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1bbf8f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bbbf93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1bbf97, 95 },
    { 0, 0 },
    { 0x912bbf99, 98 },
    { 0x972bbf9a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cbbf9d, 93 },
    { 0xb9cbbf9e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cbbfa3, 76 },
    { 0, 0 },
    { 0xbc4bbfa5, 23 },
    { 0x81abbfa6, 95 },
    { 0x514bbfa5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29bbfaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2bbfad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bbfb0, 95 },
    { 0xa6abbfb1, 75 },
    { 0, 0 },
    { 0x101bbfb3, 76 },
    { 0x561bbfb4, 91 },
    { 0xc2bbfb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbbfb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1bbfbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5bbfc0, 76 },
    { 0x27cbbfc1, 85 },
    { 0x25cbbfc2, 99 },
    { 0x360bbfc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x941bbfc6, 33 },
    { 0x839bbfc7, 61 },
    { 0, 0 },
    { 0x3a9bbfc9, 43 },
    { 0, 0 },
    { 0xb24bbfcb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0bbfd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x793bbfd5, 90 },
    { 0x758bbfd6, 93 },
    { 0, 0 },
    { 0xbc0bbfd8, 67 },
    { 0x9a0bbfd9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fbbfdd, 98 },
    { 0x302bbfde, 95 },
    { 0, 0 },
    { 0xe78bbfe0, 97 },
    { 0x76cbbfe0, 55 },
    { 0x683bbfe1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5bbfe6, 68 },
    { 0, 0 },
    { 0xb6abbfe8, 55 },
    { 0x427bbfe9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392bbfee, 61 },
    { 0x97bbbfee, 98 },
    { 0x60fbbfee, 55 },
    { 0x842bbff1, 61 },
    { 0x16bbbfee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcebbff6, 76 },
    { 0x24bbbff7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fbbffe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00bc001, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x257bc004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbc007, 90 },
    { 0xadfbc008, 95 },
    { 0xf4fbc009, 61 },
    { 0xf00bc00a, 46 },
    { 0xf6bc00b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fbc013, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6bc016, 97 },
    { 0, 0 },
    { 0xd3cbc018, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2bc020, 33 },
    { 0, 0 },
    { 0x375bc022, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93bc027, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582bc02b, 23 },
    { 0xc46bc02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2bc034, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4bc038, 23 },
    { 0xc6ebc039, 33 },
    { 0xd49bc03a, 95 },
    { 0, 0 },
    { 0x8b8bc03c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92bc03f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebbc042, 33 },
    { 0x469bc043, 34 },
    { 0x3bbbc044, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bbc047, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cbc04c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83bc050, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fbc055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47bc058, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbc05d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcbc063, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17bc067, 23 },
    { 0xbc9bc068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26bc06f, 34 },
    { 0xde6bc06f, 90 },
    { 0x45dbc071, 77 },
    { 0, 0 },
    { 0x926bc073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04bc076, 23 },
    { 0x250bc077, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7bc07e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9bc083, 46 },
    { 0x617bc084, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dbc089, 33 },
    { 0, 0 },
    { 0x163bc08b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfbc097, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dabc09b, 23 },
    { 0, 0 },
    { 0x8e3bc09d, 55 },
    { 0xad4bc09e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517bc0a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fbc0aa, 55 },
    { 0xdb7bc0aa, 87 },
    { 0xba4bc0aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dbc0af, 98 },
    { 0xb1bc0af, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130bc0b4, 90 },
    { 0x61bbc0b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bc0ba, 93 },
    { 0xef0bc0bb, 23 },
    { 0xc11bc0bc, 90 },
    { 0, 0 },
    { 0x6d5bc0be, 99 },
    { 0x3d6bc0be, 33 },
    { 0x425bc0c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cabc0c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfbc0cc, 77 },
    { 0x54bbc0cc, 61 },
    { 0, 0 },
    { 0xff3bc0cf, 55 },
    { 0, 0 },
    { 0xeadbc0d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3bc0d5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaebc0dc, 77 },
    { 0, 0 },
    { 0xb68bc0de, 90 },
    { 0xfabbc0de, 81 },
    { 0xacdbc0e0, 33 },
    { 0x77bc0e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc0e5, 52 },
    { 0, 0 },
    { 0x180bc0e7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19bc0eb, 98 },
    { 0x2bbbc0ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19bc0f2, 43 },
    { 0xcb7bc0f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x821bc0f6, 23 },
    { 0, 0 },
    { 0x905bc0f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7babc104, 90 },
    { 0xac0bc104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37abc10c, 90 },
    { 0x281bc10d, 90 },
    { 0xee0bc10d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37bc111, 87 },
    { 0x44fbc112, 17 },
    { 0xea8bc112, 23 },
    { 0xe5bbc113, 76 },
    { 0x5a2bc115, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377bc11b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8bc129, 81 },
    { 0, 0 },
    { 0xc4bc12b, 61 },
    { 0xb98bc12c, 76 },
    { 0xd80bc12d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbc131, 91 },
    { 0, 0 },
    { 0x65bbc133, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391bc138, 90 },
    { 0, 0 },
    { 0x77fbc13a, 76 },
    { 0x234bc13b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbbc149, 81 },
    { 0x3bebc149, 88 },
    { 0x469bc14a, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4bc14e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4bc153, 23 },
    { 0xc3cbc154, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217bc15e, 90 },
    { 0, 0 },
    { 0xc95bc160, 33 },
    { 0x1bc161, 93 },
    { 0x66bc161, 100 },
    { 0xe45bc163, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x689bc166, 43 },
    { 0, 0 },
    { 0x31abc168, 23 },
    { 0x2adbc168, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fbc16d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4bc171, 34 },
    { 0x25fbc171, 74 },
    { 0x85cbc172, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bc176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x235bc179, 68 },
    { 0, 0 },
    { 0x103bc17b, 97 },
    { 0, 0 },
    { 0x82fbc17d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9bc186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cbc189, 95 },
    { 0, 0 },
    { 0x8d1bc18b, 42 },
    { 0x534bc18c, 78 },
    { 0, 0 },
    { 0x603bc18e, 95 },
    { 0x321bc18e, 33 },
    { 0x613bc18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34abc194, 61 },
    { 0x8afbc195, 61 },
    { 0xc4cbc195, 90 },
    { 0x1aebc197, 61 },
    { 0, 0 },
    { 0xf39bc199, 66 },
    { 0xc66bc199, 43 },
    { 0x15bc19b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57bc1a3, 23 },
    { 0x444bc1a3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4bc1aa, 95 },
    { 0, 0 },
    { 0x27ebc1ac, 90 },
    { 0x29dbc1ad, 76 },
    { 0, 0 },
    { 0xef4bc1af, 87 },
    { 0x852bc1b0, 84 },
    { 0x2cbbc1af, 76 },
    { 0xb6cbc1af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bbc1b5, 23 },
    { 0x270bc1b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bc1b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc1bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8bc1c3, 95 },
    { 0, 0 },
    { 0x5f1bc1c5, 43 },
    { 0, 0 },
    { 0xc07bc1c7, 98 },
    { 0x5fcbc1c8, 33 },
    { 0x88ebc1c9, 91 },
    { 0x8aabc1c9, 95 },
    { 0x533bc1cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fbc1ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbc1d2, 85 },
    { 0x7fcbc1d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9bc1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155bc1da, 33 },
    { 0x62ebc1db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07bc1df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863bc1e6, 90 },
    { 0xd98bc1e7, 90 },
    { 0x19ebc1e8, 49 },
    { 0xf1cbc1e9, 90 },
    { 0xb2bc1e9, 88 },
    { 0x305bc1e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96bc1f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63bc1f5, 23 },
    { 0xc22bc1f6, 99 },
    { 0, 0 },
    { 0xd8bbc1f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6bc200, 85 },
    { 0, 0 },
    { 0x5a2bc202, 90 },
    { 0xa04bc203, 33 },
    { 0x7e0bc203, 77 },
    { 0, 0 },
    { 0x4e5bc206, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bc20a, 33 },
    { 0xbbdbc20b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3bc20f, 17 },
    { 0xd58bc210, 91 },
    { 0xa43bc210, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ebc21b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3bc21f, 98 },
    { 0xe16bc220, 78 },
    { 0xb98bc220, 90 },
    { 0xa45bc221, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8bc225, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fbc229, 90 },
    { 0x86fbc22a, 23 },
    { 0xc02bc22b, 91 },
    { 0x136bc22c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bc230, 95 },
    { 0x852bc231, 55 },
    { 0x7f6bc232, 61 },
    { 0xe16bc232, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccbc236, 99 },
    { 0xa0ebc237, 33 },
    { 0, 0 },
    { 0xb03bc239, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dbc23c, 69 },
    { 0xd90bc23c, 77 },
    { 0, 0 },
    { 0x833bc23f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673bc24b, 76 },
    { 0xa7abc24c, 33 },
    { 0xfabc24d, 34 },
    { 0x5b6bc24c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86bc251, 61 },
    { 0x6f9bc251, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x572bc255, 68 },
    { 0x644bc256, 76 },
    { 0x1bebc257, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dbc25c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x202bc25f, 68 },
    { 0, 0 },
    { 0xcbc261, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54bc266, 95 },
    { 0x51abc267, 90 },
    { 0xd74bc268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdbc26e, 34 },
    { 0xe5bbc26e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfbc273, 34 },
    { 0x8c9bc274, 76 },
    { 0x582bc274, 96 },
    { 0, 0 },
    { 0xb94bc277, 76 },
    { 0, 0 },
    { 0x2e1bc279, 95 },
    { 0x603bc27a, 61 },
    { 0xf06bc27b, 64 },
    { 0x92dbc27b, 46 },
    { 0x1fbbc27d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9bc280, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcbc285, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81bc288, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ebc28b, 67 },
    { 0xad7bc28b, 93 },
    { 0, 0 },
    { 0x5aebc28e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfbc292, 90 },
    { 0xffcbc293, 55 },
    { 0x377bc294, 98 },
    { 0, 0 },
    { 0xb43bc296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1bc299, 90 },
    { 0xe3bbc29a, 61 },
    { 0xe80bc299, 33 },
    { 0xd46bc29a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7bc2a1, 87 },
    { 0x50fbc2a2, 66 },
    { 0xd5cbc2a3, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc2aa, 34 },
    { 0, 0 },
    { 0xa2dbc2ac, 95 },
    { 0xfdfbc2ad, 61 },
    { 0x1e5bc2ac, 90 },
    { 0, 0 },
    { 0xf0bbc2b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bc2b3, 76 },
    { 0, 0 },
    { 0xa7fbc2b5, 98 },
    { 0x4dbc2b5, 90 },
    { 0x687bc2b7, 95 },
    { 0x66fbc2b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905bc2c3, 77 },
    { 0x8b8bc2c4, 90 },
    { 0, 0 },
    { 0x393bc2c6, 49 },
    { 0, 0 },
    { 0xb09bc2c8, 88 },
    { 0x5e8bc2c9, 34 },
    { 0xfdcbc2ca, 36 },
    { 0xe53bc2cb, 98 },
    { 0, 0 },
    { 0x519bc2cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6bc2d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ebc2d9, 23 },
    { 0x473bc2d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48bc2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0bc2e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cbc2ec, 78 },
    { 0, 0 },
    { 0x507bc2ee, 99 },
    { 0, 0 },
    { 0x607bc2f0, 55 },
    { 0xdecbc2f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7bc2f7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bbc2fd, 90 },
    { 0x1d9bc2fd, 95 },
    { 0xd7bbc2ff, 87 },
    { 0xcadbc300, 88 },
    { 0, 0 },
    { 0x6ddbc302, 101 },
    { 0xcdbbc302, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30bc308, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336bc30c, 98 },
    { 0, 0 },
    { 0xaa8bc30e, 76 },
    { 0, 0 },
    { 0x71fbc310, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2bc318, 90 },
    { 0, 0 },
    { 0x8cebc31a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdbc31d, 33 },
    { 0xda5bc31e, 36 },
    { 0x69abc31e, 90 },
    { 0, 0 },
    { 0x6f8bc321, 87 },
    { 0, 0 },
    { 0x9b0bc323, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1bc32b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abbc32f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97bc338, 98 },
    { 0xe70bc339, 90 },
    { 0x721bc33a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ebc342, 33 },
    { 0xfdabc343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1bc346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdebc34b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dbc34f, 76 },
    { 0, 0 },
    { 0xc2fbc351, 95 },
    { 0x8d3bc352, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc357, 93 },
    { 0x6eabc358, 90 },
    { 0, 0 },
    { 0x1dbc35a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9bc35e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15bc361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4bc36c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ebc371, 76 },
    { 0, 0 },
    { 0xa23bc373, 90 },
    { 0x794bc374, 46 },
    { 0x2e7bc374, 90 },
    { 0x7bcbc376, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecbc380, 87 },
    { 0, 0 },
    { 0x154bc382, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bc385, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8bc388, 77 },
    { 0x2a7bc388, 90 },
    { 0, 0 },
    { 0x51bbc38b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4bc392, 78 },
    { 0xbfabc392, 66 },
    { 0x8b6bc394, 4 },
    { 0xd59bc394, 33 },
    { 0, 0 },
    { 0xb16bc397, 33 },
    { 0xa16bc398, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143bc39c, 98 },
    { 0x5d6bc39d, 84 },
    { 0, 0 },
    { 0xe62bc39f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x954bc3a2, 98 },
    { 0, 0 },
    { 0x8e2bc3a4, 98 },
    { 0x3acbc3a5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afbc3ac, 90 },
    { 0x79bbc3ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534bc3b1, 90 },
    { 0xfc4bc3b2, 55 },
    { 0xd60bc3b3, 23 },
    { 0xb6abc3b3, 34 },
    { 0xaffbc3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x179bc3b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69bc3bc, 33 },
    { 0x94fbc3bd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55bc3c4, 76 },
    { 0xe8fbc3c4, 85 },
    { 0x54bc3c6, 90 },
    { 0x2abbc3c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674bc3cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bc3ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58bc3d1, 23 },
    { 0, 0 },
    { 0x95dbc3d3, 90 },
    { 0, 0 },
    { 0xb07bc3d5, 98 },
    { 0, 0 },
    { 0xe2abc3d7, 61 },
    { 0x3c8bc3d8, 33 },
    { 0, 0 },
    { 0xf4cbc3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bbc3df, 78 },
    { 0x1fbc3e0, 23 },
    { 0x575bc3e1, 66 },
    { 0x357bc3e2, 55 },
    { 0xad7bc3e2, 76 },
    { 0, 0 },
    { 0xdbdbc3e5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49bc3e9, 90 },
    { 0xf78bc3ea, 55 },
    { 0, 0 },
    { 0x56fbc3ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390bc3f1, 98 },
    { 0, 0 },
    { 0xd34bc3f3, 55 },
    { 0x998bc3f4, 61 },
    { 0x9f5bc3f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bc3fc, 17 },
    { 0xc95bc3fd, 78 },
    { 0x3d2bc3fe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dbc402, 61 },
    { 0x927bc403, 90 },
    { 0xf21bc404, 34 },
    { 0, 0 },
    { 0x502bc406, 78 },
    { 0x512bc406, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66abc40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fbc413, 33 },
    { 0x1bc413, 61 },
    { 0x15fbc414, 61 },
    { 0x1a9bc416, 81 },
    { 0x7e6bc417, 46 },
    { 0, 0 },
    { 0x2e9bc419, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421bc41f, 87 },
    { 0x4a4bc420, 90 },
    { 0x517bc421, 23 },
    { 0x19cbc422, 46 },
    { 0xd5cbc41f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc428, 33 },
    { 0x234bc429, 61 },
    { 0x774bc42a, 46 },
    { 0, 0 },
    { 0xbf4bc42c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62bc433, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dbc437, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x818bc43a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fbc440, 101 },
    { 0, 0 },
    { 0xcdcbc442, 34 },
    { 0, 0 },
    { 0x8e7bc444, 33 },
    { 0x250bc444, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bc44b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32bc44e, 98 },
    { 0x90dbc44f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523bc455, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdabc458, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791bc45d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862bc461, 23 },
    { 0x3a0bc462, 55 },
    { 0x404bc463, 98 },
    { 0x1aabc464, 33 },
    { 0xf18bc465, 93 },
    { 0xfb8bc464, 55 },
    { 0x5ccbc466, 33 },
    { 0xa75bc461, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41bc46b, 90 },
    { 0, 0 },
    { 0x380bc46d, 61 },
    { 0xb0abc46d, 76 },
    { 0xf86bc46d, 90 },
    { 0x77cbc470, 84 },
    { 0, 0 },
    { 0x51ebc472, 90 },
    { 0x135bc472, 61 },
    { 0x9bcbc474, 23 },
    { 0x5cebc475, 34 },
    { 0x8b9bc475, 90 },
    { 0xde6bc476, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9bc47c, 100 },
    { 0x827bc47d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bc480, 34 },
    { 0, 0 },
    { 0xabcbc482, 95 },
    { 0x7f3bc483, 5 },
    { 0x6d6bc483, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bc48c, 95 },
    { 0, 0 },
    { 0xffebc48e, 61 },
    { 0xd55bc48e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69bc492, 85 },
    { 0xad1bc492, 23 },
    { 0x2dbbc494, 95 },
    { 0xa1fbc492, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bc4a0, 33 },
    { 0, 0 },
    { 0x625bc4a2, 34 },
    { 0xd7fbc4a3, 95 },
    { 0x957bc4a2, 43 },
    { 0x375bc4a5, 78 },
    { 0, 0 },
    { 0xd0cbc4a7, 90 },
    { 0x78bbc4a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96bc4ae, 95 },
    { 0x610bc4ae, 23 },
    { 0xccbc4b0, 90 },
    { 0, 0 },
    { 0x2c1bc4b2, 76 },
    { 0xfd3bc4b3, 98 },
    { 0x5e4bc4b4, 66 },
    { 0xd02bc4b5, 95 },
    { 0, 0 },
    { 0x6d2bc4b7, 55 },
    { 0xbb5bc4b8, 90 },
    { 0xaadbc4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4bc4be, 90 },
    { 0, 0 },
    { 0x200bc4c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x811bc4c3, 34 },
    { 0x4c6bc4c4, 90 },
    { 0, 0 },
    { 0x71fbc4c6, 99 },
    { 0x710bc4c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1bc4ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867bc4ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bc4d2, 90 },
    { 0, 0 },
    { 0x70bbc4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x679bc4d7, 33 },
    { 0, 0 },
    { 0xc8dbc4d9, 95 },
    { 0x279bc4da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ebc4de, 46 },
    { 0, 0 },
    { 0xbb4bc4e0, 46 },
    { 0xc71bc4e0, 95 },
    { 0, 0 },
    { 0x897bc4e3, 98 },
    { 0x306bc4e3, 95 },
    { 0xf0bc4e5, 90 },
    { 0x95dbc4e6, 99 },
    { 0x4eebc4e6, 34 },
    { 0x445bc4e8, 93 },
    { 0xb5abc4e5, 61 },
    { 0, 0 },
    { 0xa57bc4eb, 37 },
    { 0x5c0bc4eb, 98 },
    { 0xfb8bc4ed, 33 },
    { 0, 0 },
    { 0x820bc4ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x956bc4f2, 90 },
    { 0, 0 },
    { 0xa98bc4f4, 46 },
    { 0xd93bc4f5, 98 },
    { 0x3adbc4f6, 46 },
    { 0xc40bc4f7, 93 },
    { 0xc69bc4f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5bc4fd, 99 },
    { 0x344bc4fe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cbc503, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bbc506, 61 },
    { 0xf5abc507, 23 },
    { 0xb9abc508, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cbc50c, 98 },
    { 0x727bc50c, 67 },
    { 0, 0 },
    { 0x5a5bc50f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864bc515, 76 },
    { 0, 0 },
    { 0x1d1bc517, 40 },
    { 0x48bc518, 43 },
    { 0xdc1bc519, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40bc51c, 98 },
    { 0xd18bc51d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1bc525, 99 },
    { 0x1a4bc525, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bbc529, 95 },
    { 0x3afbc529, 95 },
    { 0x732bc52a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafbc53a, 80 },
    { 0, 0 },
    { 0xe16bc53c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fbc541, 90 },
    { 0x578bc541, 67 },
    { 0x83bc543, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46abc547, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4bc54a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46bc54e, 33 },
    { 0x4edbc54e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446bc553, 76 },
    { 0xae0bc554, 46 },
    { 0xc01bc555, 90 },
    { 0xafebc556, 43 },
    { 0x9ffbc555, 100 },
    { 0x24ebc558, 83 },
    { 0x13fbc558, 46 },
    { 0xf4dbc559, 68 },
    { 0x2dbbc55b, 90 },
    { 0x38fbc559, 33 },
    { 0x920bc55d, 33 },
    { 0xf5bbc55e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1bc562, 51 },
    { 0, 0 },
    { 0x708bc564, 23 },
    { 0, 0 },
    { 0x1d3bc566, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6bc56a, 23 },
    { 0x229bc56a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6bc56e, 90 },
    { 0x5acbc56f, 91 },
    { 0, 0 },
    { 0x9fcbc571, 23 },
    { 0xb56bc571, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1bc576, 55 },
    { 0, 0 },
    { 0xac9bc578, 33 },
    { 0x229bc578, 99 },
    { 0x9ebc57a, 46 },
    { 0, 0 },
    { 0x8a5bc57c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329bc580, 66 },
    { 0x7aebc581, 98 },
    { 0, 0 },
    { 0x3d2bc583, 76 },
    { 0x397bc584, 85 },
    { 0xe84bc585, 55 },
    { 0x4dfbc585, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68bc58e, 100 },
    { 0x7d2bc58f, 34 },
    { 0xc90bc590, 48 },
    { 0xa58bc591, 95 },
    { 0xccdbc592, 83 },
    { 0xa29bc593, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc59a, 90 },
    { 0xa21bc59b, 85 },
    { 0x87cbc59c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bc5a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713bc5a4, 33 },
    { 0xc5bbc5a4, 68 },
    { 0, 0 },
    { 0x66abc5a7, 61 },
    { 0xd66bc5a7, 23 },
    { 0xc4fbc5a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9bc5ad, 67 },
    { 0xb0fbc5ad, 33 },
    { 0x91abc5ad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919bc5b6, 76 },
    { 0xa79bc5b6, 85 },
    { 0x77bbc5b7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcbc5bd, 23 },
    { 0, 0 },
    { 0x12cbc5bf, 99 },
    { 0xdbc5c0, 98 },
    { 0, 0 },
    { 0xa86bc5c2, 95 },
    { 0x2c7bc5c2, 61 },
    { 0, 0 },
    { 0xb36bc5c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aebc5c9, 95 },
    { 0, 0 },
    { 0xe40bc5cb, 61 },
    { 0x33cbc5cc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xecebc5cf, 61 },
    { 0, 0 },
    { 0xac6bc5d1, 52 },
    { 0, 0 },
    { 0x64fbc5d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bbc5da, 23 },
    { 0, 0 },
    { 0x843bc5dc, 73 },
    { 0, 0 },
    { 0x1d2bc5de, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799bc5e3, 23 },
    { 0x4aebc5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bc5f4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636bc5f8, 62 },
    { 0, 0 },
    { 0xdabc5fa, 23 },
    { 0x3cabc5fb, 84 },
    { 0x758bc5fc, 23 },
    { 0x668bc5fd, 83 },
    { 0xd0cbc5fb, 95 },
    { 0x739bc5fe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x130bc602, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258bc608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x704bc60b, 43 },
    { 0x817bc60b, 61 },
    { 0xf81bc60d, 66 },
    { 0x115bc60e, 17 },
    { 0xf6bbc60e, 99 },
    { 0xbe3bc610, 23 },
    { 0x28cbc610, 90 },
    { 0xe31bc60d, 90 },
    { 0, 0 },
    { 0xc93bc614, 61 },
    { 0x576bc614, 55 },
    { 0x2abbc616, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97bc61c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdbc61f, 52 },
    { 0x3d1bc620, 55 },
    { 0, 0 },
    { 0x573bc622, 43 },
    { 0xe80bc622, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5bc629, 46 },
    { 0, 0 },
    { 0x3e0bc62b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x936bc62e, 61 },
    { 0x700bc62f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110bc635, 90 },
    { 0, 0 },
    { 0x550bc637, 95 },
    { 0x720bc637, 88 },
    { 0, 0 },
    { 0x588bc63a, 58 },
    { 0xb6ebc63b, 100 },
    { 0x95dbc63b, 46 },
    { 0xabbbc63d, 46 },
    { 0xee3bc63b, 55 },
    { 0, 0 },
    { 0xd7ebc640, 43 },
    { 0xaf5bc641, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05bc645, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5bc648, 44 },
    { 0x74ebc649, 23 },
    { 0x4c1bc648, 55 },
    { 0x354bc64b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6abc650, 61 },
    { 0, 0 },
    { 0x5e0bc652, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60abc656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeabc659, 61 },
    { 0xe0cbc659, 84 },
    { 0x461bc65a, 78 },
    { 0, 0 },
    { 0x425bc65d, 76 },
    { 0, 0 },
    { 0x8d4bc65f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623bc667, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4bc66c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fbc66f, 55 },
    { 0, 0 },
    { 0x1f6bc671, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92bc675, 55 },
    { 0, 0 },
    { 0x15cbc677, 61 },
    { 0xa52bc678, 99 },
    { 0x680bc679, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65abc67f, 90 },
    { 0x574bc680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3bc685, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dbc68a, 23 },
    { 0x206bc68b, 76 },
    { 0x42bc68c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1abc690, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7bc698, 98 },
    { 0x8ffbc699, 99 },
    { 0x5e6bc699, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bc6a2, 33 },
    { 0xa5bbc6a3, 90 },
    { 0, 0 },
    { 0x661bc6a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5bc6ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbbc6b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9bc6b4, 93 },
    { 0x6babc6b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63bc6bc, 88 },
    { 0x5ffbc6bc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefbc6c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x583bc6c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dbc6d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9bc6d7, 66 },
    { 0x996bc6d7, 66 },
    { 0x40abc6d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbbc6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbc6e1, 95 },
    { 0, 0 },
    { 0xa0bbc6e3, 43 },
    { 0, 0 },
    { 0x9f3bc6e5, 99 },
    { 0, 0 },
    { 0x60dbc6e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x348bc6ea, 34 },
    { 0x783bc6eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdbc6f5, 55 },
    { 0x267bc6f6, 93 },
    { 0x8e8bc6f6, 33 },
    { 0x2edbc6f6, 90 },
    { 0, 0 },
    { 0x3b2bc6fa, 61 },
    { 0xf5bc6fb, 23 },
    { 0xcdebc6fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bc700, 95 },
    { 0, 0 },
    { 0xa38bc702, 98 },
    { 0xd1abc703, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4bc70a, 34 },
    { 0x5c1bc70b, 90 },
    { 0x364bc70c, 77 },
    { 0, 0 },
    { 0xfa1bc70e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550bc713, 90 },
    { 0, 0 },
    { 0x657bc715, 95 },
    { 0, 0 },
    { 0xb17bc717, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33bc71e, 33 },
    { 0, 0 },
    { 0xa3bc720, 61 },
    { 0x93bc720, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364bc727, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08bc72b, 66 },
    { 0x1b7bc72b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648bc730, 93 },
    { 0x3ddbc731, 90 },
    { 0x6cdbc730, 87 },
    { 0xdbebc733, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28bc736, 33 },
    { 0xe83bc737, 55 },
    { 0xc92bc738, 90 },
    { 0, 0 },
    { 0x73bc73a, 33 },
    { 0, 0 },
    { 0x960bc73c, 90 },
    { 0x81ebc73c, 61 },
    { 0, 0 },
    { 0x52bbc73f, 33 },
    { 0, 0 },
    { 0x24fbc741, 90 },
    { 0, 0 },
    { 0x51fbc743, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc747, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dbc74a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2bc751, 23 },
    { 0x7fbbc751, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751bc757, 46 },
    { 0, 0 },
    { 0xf61bc759, 98 },
    { 0, 0 },
    { 0x7b0bc75b, 33 },
    { 0x515bc75c, 33 },
    { 0x17bc75d, 23 },
    { 0xe82bc75e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01bc763, 68 },
    { 0x6febc764, 90 },
    { 0x730bc765, 98 },
    { 0xf45bc766, 90 },
    { 0x6f5bc765, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x684bc76a, 90 },
    { 0xe9ebc76a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428bc771, 61 },
    { 0xafbbc772, 64 },
    { 0x56bc773, 66 },
    { 0x976bc774, 95 },
    { 0, 0 },
    { 0xcccbc776, 90 },
    { 0x169bc776, 95 },
    { 0x9ddbc778, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436bc781, 67 },
    { 0, 0 },
    { 0x85fbc783, 90 },
    { 0, 0 },
    { 0xd7cbc785, 33 },
    { 0, 0 },
    { 0x81bc787, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4bc78a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bc792, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x969bc795, 33 },
    { 0xf74bc796, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fbc79b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85bc7a2, 61 },
    { 0xd78bc7a2, 37 },
    { 0, 0 },
    { 0x820bc7a5, 33 },
    { 0xf14bc7a5, 33 },
    { 0, 0 },
    { 0x55bc7a8, 98 },
    { 0x8f2bc7a9, 90 },
    { 0, 0 },
    { 0xcf6bc7ab, 61 },
    { 0xdabbc7ab, 67 },
    { 0x4b0bc7ad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dbc7b1, 23 },
    { 0xf9fbc7b1, 33 },
    { 0x575bc7b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951bc7b7, 61 },
    { 0xce3bc7b8, 95 },
    { 0xaddbc7b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83bc7bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bc7c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84bc7c3, 61 },
    { 0x21ebc7c4, 43 },
    { 0x5cbc7c5, 76 },
    { 0x471bc7c3, 90 },
    { 0xaa3bc7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698bc7cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99abc7cf, 90 },
    { 0x997bc7d0, 23 },
    { 0, 0 },
    { 0x467bc7d2, 90 },
    { 0x309bc7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454bc7d8, 46 },
    { 0x8f9bc7d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cbc7e1, 68 },
    { 0, 0 },
    { 0xe01bc7e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3bc7e6, 95 },
    { 0, 0 },
    { 0x3e7bc7e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bc7eb, 76 },
    { 0x38dbc7ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeabc7f4, 33 },
    { 0, 0 },
    { 0x716bc7f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0bc7fb, 98 },
    { 0x377bc7fb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc802, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4bc80a, 93 },
    { 0xca1bc80a, 76 },
    { 0x300bc80c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fbc811, 33 },
    { 0, 0 },
    { 0xf12bc813, 38 },
    { 0xba0bc813, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbbc818, 61 },
    { 0xc7bc819, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72bc820, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cbc829, 46 },
    { 0, 0 },
    { 0xd57bc82b, 98 },
    { 0xdddbc82b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9bc831, 80 },
    { 0xa36bc832, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfbc83c, 98 },
    { 0x5fbc83c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfbc840, 43 },
    { 0xe69bc841, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ebc845, 98 },
    { 0x858bc846, 98 },
    { 0x25fbc847, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ebc84e, 55 },
    { 0xd0dbc84f, 61 },
    { 0x9dfbc850, 61 },
    { 0x152bc851, 46 },
    { 0x2f5bc851, 90 },
    { 0xce3bc84f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86bc856, 83 },
    { 0x75cbc857, 98 },
    { 0xf8ebc858, 98 },
    { 0xff4bc858, 98 },
    { 0x254bc859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2bc85e, 87 },
    { 0, 0 },
    { 0xe53bc860, 98 },
    { 0x4febc861, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x686bc864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbc86a, 90 },
    { 0x214bc86b, 46 },
    { 0x822bc86a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcbc872, 93 },
    { 0, 0 },
    { 0xe33bc874, 66 },
    { 0xe3bbc875, 61 },
    { 0x178bc876, 90 },
    { 0, 0 },
    { 0xe66bc878, 17 },
    { 0xea1bc878, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bbc87e, 90 },
    { 0, 0 },
    { 0x440bc880, 67 },
    { 0x245bc881, 100 },
    { 0x447bc881, 46 },
    { 0xc8abc882, 76 },
    { 0xc8bbc884, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733bc889, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bc88f, 100 },
    { 0x149bc88f, 98 },
    { 0, 0 },
    { 0x171bc892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfebc897, 61 },
    { 0x79cbc897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1bc89b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8abc8a0, 66 },
    { 0, 0 },
    { 0xd59bc8a2, 98 },
    { 0, 0 },
    { 0xf85bc8a4, 33 },
    { 0, 0 },
    { 0x5b1bc8a6, 95 },
    { 0, 0 },
    { 0xdfdbc8a8, 98 },
    { 0x403bc8a8, 95 },
    { 0x6e0bc8aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcbc8ae, 95 },
    { 0, 0 },
    { 0x4f8bc8b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ebc8b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350bc8bb, 33 },
    { 0, 0 },
    { 0x425bc8bd, 40 },
    { 0xdd1bc8bd, 33 },
    { 0xd25bc8bf, 99 },
    { 0, 0 },
    { 0x6fbc8c1, 61 },
    { 0x770bc8c2, 75 },
    { 0, 0 },
    { 0xb33bc8c4, 61 },
    { 0, 0 },
    { 0x7b0bc8c6, 23 },
    { 0x58bc8c6, 77 },
    { 0, 0 },
    { 0x6ccbc8c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bc8d1, 90 },
    { 0x871bc8d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251bc8d5, 61 },
    { 0, 0 },
    { 0x12bbc8d7, 78 },
    { 0, 0 },
    { 0x629bc8d9, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8bc8e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6bc8e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9bc8e9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385bc8f4, 61 },
    { 0xb0abc8f5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625bc8fa, 33 },
    { 0, 0 },
    { 0xa5cbc8fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964bc900, 99 },
    { 0x482bc901, 95 },
    { 0, 0 },
    { 0x601bc903, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dbc90d, 85 },
    { 0xd5ebc90d, 34 },
    { 0x648bc90e, 61 },
    { 0xf1fbc90d, 78 },
    { 0x20abc911, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97bc917, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553bc91d, 61 },
    { 0x5a2bc91e, 33 },
    { 0xa92bc91f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ebc927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fbc92c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67bc931, 90 },
    { 0x1c2bc932, 81 },
    { 0x299bc931, 78 },
    { 0x57ebc934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc938, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x178bc93b, 46 },
    { 0x243bc93c, 75 },
    { 0xaecbc93d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ebc942, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cbc947, 61 },
    { 0x4e8bc948, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bbc94e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3bc951, 66 },
    { 0xf2ebc952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bbc95b, 90 },
    { 0xd5cbc95c, 23 },
    { 0x38abc95b, 98 },
    { 0xdbabc95e, 76 },
    { 0xb45bc95f, 99 },
    { 0xd08bc960, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ebc966, 100 },
    { 0, 0 },
    { 0x3c2bc968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541bc96c, 76 },
    { 0, 0 },
    { 0x234bc96e, 98 },
    { 0xdd7bc96f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41bc972, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bbc975, 46 },
    { 0, 0 },
    { 0xd32bc977, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bc97d, 23 },
    { 0xa3bbc97e, 95 },
    { 0xe0abc97f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24bc986, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbc989, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5abc98c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1bc991, 61 },
    { 0xab8bc991, 95 },
    { 0, 0 },
    { 0x2debc994, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bc99a, 33 },
    { 0, 0 },
    { 0xbaabc99c, 98 },
    { 0xb3bbc99d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bbc9a0, 33 },
    { 0x1afbc9a1, 76 },
    { 0xe6bbc9a1, 43 },
    { 0xf4bbc9a3, 85 },
    { 0x324bc9a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afbc9a7, 61 },
    { 0xd1cbc9a8, 43 },
    { 0, 0 },
    { 0x572bc9aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06bc9ad, 33 },
    { 0x3c0bc9ae, 46 },
    { 0xb98bc9af, 33 },
    { 0x781bc9b0, 90 },
    { 0x76dbc9af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dbc9b4, 66 },
    { 0x8ebbc9b5, 33 },
    { 0xc8cbc9b4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dbc9bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bc9bf, 98 },
    { 0xf1cbc9c0, 52 },
    { 0, 0 },
    { 0x545bc9c2, 98 },
    { 0x67cbc9c3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ebc9cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adbc9d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x130bc9d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc9d8, 65 },
    { 0x48bc9d9, 95 },
    { 0xec3bc9da, 78 },
    { 0xebebc9da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490bc9df, 61 },
    { 0x40abc9e0, 88 },
    { 0x3d9bc9e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0bc9eb, 75 },
    { 0, 0 },
    { 0x1afbc9ed, 99 },
    { 0xe92bc9ee, 33 },
    { 0xce2bc9ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8bc9f4, 55 },
    { 0x81cbc9f4, 65 },
    { 0xf3abc9f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16bc9fd, 55 },
    { 0, 0 },
    { 0xd27bc9ff, 34 },
    { 0, 0 },
    { 0x39abca01, 45 },
    { 0, 0 },
    { 0xd3fbca03, 65 },
    { 0x22bbca03, 23 },
    { 0x5e7bca05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bbca08, 33 },
    { 0, 0 },
    { 0x5cfbca0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cbca18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60bca1e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11bca21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07bca25, 100 },
    { 0, 0 },
    { 0xabfbca27, 80 },
    { 0, 0 },
    { 0x791bca29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cbca2d, 33 },
    { 0x748bca2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dbca34, 75 },
    { 0, 0 },
    { 0xe5cbca36, 99 },
    { 0xb20bca37, 98 },
    { 0, 0 },
    { 0x46bca39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18bca3f, 100 },
    { 0, 0 },
    { 0x1b4bca41, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65bca45, 61 },
    { 0, 0 },
    { 0xdbbbca47, 43 },
    { 0, 0 },
    { 0xe9dbca49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9bca4c, 61 },
    { 0, 0 },
    { 0xeabca4e, 61 },
    { 0xb65bca4f, 46 },
    { 0xd56bca50, 38 },
    { 0x29cbca50, 33 },
    { 0xaa5bca52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bbca55, 93 },
    { 0xb5bca55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdbca5a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbbca5e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415bca62, 33 },
    { 0, 0 },
    { 0x55abca64, 55 },
    { 0, 0 },
    { 0x174bca66, 93 },
    { 0x4b1bca66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7bca6e, 78 },
    { 0, 0 },
    { 0x7b4bca70, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adbca74, 91 },
    { 0, 0 },
    { 0x450bca76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bca79, 98 },
    { 0x97ebca79, 38 },
    { 0, 0 },
    { 0xadabca7c, 88 },
    { 0, 0 },
    { 0x60dbca7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76bca81, 58 },
    { 0xeddbca82, 84 },
    { 0xbd9bca83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fbca86, 90 },
    { 0, 0 },
    { 0xb2bca88, 61 },
    { 0, 0 },
    { 0x1ebbca8a, 55 },
    { 0, 0 },
    { 0x5e7bca8c, 76 },
    { 0x878bca8d, 46 },
    { 0, 0 },
    { 0xf6ebca8f, 43 },
    { 0, 0 },
    { 0xb80bca91, 61 },
    { 0x869bca92, 84 },
    { 0xa76bca93, 90 },
    { 0xf0bca91, 95 },
    { 0xb2ebca95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2bca9b, 43 },
    { 0, 0 },
    { 0x24fbca9d, 93 },
    { 0xe19bca9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bbcaa4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81bcaaa, 95 },
    { 0, 0 },
    { 0x9cebcaac, 34 },
    { 0, 0 },
    { 0xe54bcaae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742bcab9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4bcabd, 33 },
    { 0x76abcabe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbcac1, 95 },
    { 0, 0 },
    { 0xe91bcac3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x709bcac6, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510bcacc, 66 },
    { 0x87dbcacc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39abcad0, 98 },
    { 0xe59bcad0, 52 },
    { 0x781bcad2, 61 },
    { 0xe16bcad3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bbcad6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96bcadb, 61 },
    { 0xf0abcadc, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7bcae5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434bcaed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0bcaf2, 23 },
    { 0, 0 },
    { 0x497bcaf4, 43 },
    { 0x41bcaf5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2bcaf9, 99 },
    { 0, 0 },
    { 0x2bfbcafb, 33 },
    { 0xff6bcafc, 69 },
    { 0, 0 },
    { 0xa79bcafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1bcb02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bbcb05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841bcb0b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x472bcb0e, 87 },
    { 0x81dbcb0f, 98 },
    { 0, 0 },
    { 0xbdfbcb11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddbcb14, 76 },
    { 0, 0 },
    { 0xa82bcb16, 61 },
    { 0, 0 },
    { 0x4cbbcb18, 81 },
    { 0xd0ebcb18, 23 },
    { 0x6fbbcb18, 33 },
    { 0, 0 },
    { 0xb46bcb1c, 95 },
    { 0xb4bbcb1d, 99 },
    { 0x312bcb1d, 90 },
    { 0, 0 },
    { 0x3fdbcb20, 55 },
    { 0xefebcb21, 93 },
    { 0x453bcb21, 33 },
    { 0x9d9bcb22, 100 },
    { 0, 0 },
    { 0xef0bcb25, 33 },
    { 0x84ebcb26, 33 },
    { 0xbb4bcb26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x505bcb2a, 33 },
    { 0x315bcb2a, 33 },
    { 0, 0 },
    { 0xbd6bcb2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebcb30, 98 },
    { 0, 0 },
    { 0xf60bcb32, 76 },
    { 0xd1ebcb32, 66 },
    { 0, 0 },
    { 0x6eebcb35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cbcb39, 98 },
    { 0x92fbcb39, 17 },
    { 0x3ccbcb39, 91 },
    { 0xe87bcb3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cbcb41, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ebcb49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edbcb53, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774bcb59, 91 },
    { 0x8debcb5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdbcb5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bcb66, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207bcb6a, 90 },
    { 0, 0 },
    { 0xed9bcb6c, 43 },
    { 0, 0 },
    { 0xbb7bcb6e, 52 },
    { 0x9c7bcb6f, 33 },
    { 0, 0 },
    { 0x61cbcb71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bcb75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2bcb7d, 91 },
    { 0, 0 },
    { 0x821bcb7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438bcb85, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cbcb8a, 52 },
    { 0x320bcb8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7bcb91, 87 },
    { 0, 0 },
    { 0x65bbcb93, 55 },
    { 0x5c6bcb93, 95 },
    { 0, 0 },
    { 0x673bcb96, 23 },
    { 0, 0 },
    { 0xfd0bcb98, 90 },
    { 0x73bcb99, 66 },
    { 0x37cbcb9a, 80 },
    { 0xadfbcb9a, 23 },
    { 0, 0 },
    { 0x76cbcb9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08bcba6, 43 },
    { 0x13abcba7, 33 },
    { 0xe5fbcba7, 61 },
    { 0xd54bcba9, 76 },
    { 0x407bcba9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ebcbaf, 78 },
    { 0, 0 },
    { 0xef6bcbb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfbcbb6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ebcbbc, 55 },
    { 0x8afbcbbd, 83 },
    { 0, 0 },
    { 0x260bcbbf, 33 },
    { 0, 0 },
    { 0x2babcbc1, 43 },
    { 0xa91bcbc1, 90 },
    { 0x181bcbc3, 67 },
    { 0x240bcbc1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabcbca, 95 },
    { 0x7f3bcbca, 98 },
    { 0, 0 },
    { 0xc8bbcbcd, 66 },
    { 0xf60bcbce, 99 },
    { 0x716bcbcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bbcbd2, 61 },
    { 0x3ffbcbd2, 99 },
    { 0xec5bcbd4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dbcbda, 61 },
    { 0x3ebcbdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7bcbe7, 17 },
    { 0x6eebcbe8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0abcbeb, 23 },
    { 0x734bcbec, 76 },
    { 0x65cbcbec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7bcbf1, 99 },
    { 0, 0 },
    { 0xa57bcbf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9abcbfa, 90 },
    { 0, 0 },
    { 0xe80bcbfc, 33 },
    { 0, 0 },
    { 0x601bcbfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bcc01, 76 },
    { 0x776bcc02, 61 },
    { 0xfa2bcc02, 87 },
    { 0x995bcc04, 90 },
    { 0xdc2bcc05, 88 },
    { 0xa0cbcc05, 87 },
    { 0x603bcc01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53bcc0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5bcc26, 78 },
    { 0x234bcc27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aebcc2c, 33 },
    { 0xc73bcc2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22bcc30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dbcc36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14bcc39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cebcc3d, 66 },
    { 0x4aabcc3e, 61 },
    { 0x201bcc3e, 90 },
    { 0xae8bcc3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51bcc46, 64 },
    { 0, 0 },
    { 0x6cdbcc48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fbcc4c, 98 },
    { 0x60ebcc4d, 61 },
    { 0x585bcc4e, 33 },
    { 0x388bcc4d, 68 },
    { 0x53dbcc50, 61 },
    { 0x3d7bcc4f, 43 },
    { 0, 0 },
    { 0x7a7bcc53, 55 },
    { 0xffebcc54, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721bcc58, 61 },
    { 0xc99bcc59, 90 },
    { 0x337bcc59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2bcc63, 67 },
    { 0x6e8bcc63, 61 },
    { 0, 0 },
    { 0x532bcc66, 61 },
    { 0xde9bcc67, 33 },
    { 0x231bcc67, 90 },
    { 0, 0 },
    { 0xe3fbcc6a, 76 },
    { 0x4ebbcc6b, 88 },
    { 0, 0 },
    { 0x4c6bcc6d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361bcc71, 17 },
    { 0xf8bbcc72, 23 },
    { 0x147bcc73, 95 },
    { 0x6b9bcc73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1bcc77, 33 },
    { 0x3c4bcc77, 33 },
    { 0x516bcc77, 61 },
    { 0x54ebcc79, 61 },
    { 0, 0 },
    { 0x71bcc7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbbcc7f, 88 },
    { 0xd8ebcc80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fbcc84, 90 },
    { 0x897bcc85, 23 },
    { 0x442bcc86, 77 },
    { 0, 0 },
    { 0xbc0bcc88, 95 },
    { 0, 0 },
    { 0x74ebcc8a, 98 },
    { 0, 0 },
    { 0x97bcc8c, 36 },
    { 0xa4cbcc8d, 78 },
    { 0xcebbcc8e, 61 },
    { 0, 0 },
    { 0x128bcc90, 98 },
    { 0, 0 },
    { 0xa7bbcc92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6bcc96, 61 },
    { 0, 0 },
    { 0xb9cbcc98, 90 },
    { 0x7d6bcc99, 98 },
    { 0x3b5bcc99, 85 },
    { 0, 0 },
    { 0x23bcc9c, 33 },
    { 0x40ebcc9c, 91 },
    { 0x5d8bcc9e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcbcca4, 52 },
    { 0x62bcca5, 67 },
    { 0x673bcca6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x954bcca9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bccac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fbccb0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cbccb4, 95 },
    { 0x882bccb5, 50 },
    { 0x64abccb5, 61 },
    { 0x978bccb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8bccbc, 34 },
    { 0x29abccbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3bccc6, 23 },
    { 0x6fabccc7, 67 },
    { 0, 0 },
    { 0xcb2bccc9, 85 },
    { 0x34dbccca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2bccd4, 90 },
    { 0, 0 },
    { 0xe4ebccd6, 23 },
    { 0xcaabccd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dabcce0, 55 },
    { 0x2e2bcce0, 61 },
    { 0, 0 },
    { 0x60fbcce3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677bcce8, 90 },
    { 0x991bcce9, 85 },
    { 0xafbbccea, 33 },
    { 0xe81bcceb, 33 },
    { 0, 0 },
    { 0x650bcced, 90 },
    { 0, 0 },
    { 0x3febccef, 61 },
    { 0, 0 },
    { 0x3debccf1, 66 },
    { 0, 0 },
    { 0xd87bccf3, 99 },
    { 0x2ecbccf3, 34 },
    { 0xc05bccf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2bcd01, 67 },
    { 0x35bcd02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x524bcd05, 33 },
    { 0xaa3bcd06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81bcd0d, 33 },
    { 0x830bcd0e, 23 },
    { 0x9eabcd0d, 33 },
    { 0, 0 },
    { 0xca9bcd11, 93 },
    { 0x497bcd11, 76 },
    { 0xd1abcd12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4bcd1b, 77 },
    { 0xb0ebcd1c, 38 },
    { 0x9c6bcd1d, 52 },
    { 0xcd6bcd1d, 33 },
    { 0x128bcd1f, 43 },
    { 0x775bcd1b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127bcd2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5bcd2e, 95 },
    { 0x476bcd2e, 68 },
    { 0x9c6bcd30, 61 },
    { 0x299bcd31, 61 },
    { 0x22cbcd30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ebcd38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0bcd44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97bcd47, 40 },
    { 0xb8ebcd48, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dbcd4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1bcd52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bbcd55, 100 },
    { 0x735bcd56, 55 },
    { 0, 0 },
    { 0x7a1bcd58, 95 },
    { 0, 0 },
    { 0x58ebcd5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cbcd5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117bcd65, 88 },
    { 0x2bcd65, 66 },
    { 0xa84bcd67, 61 },
    { 0x5c9bcd67, 33 },
    { 0xf39bcd67, 33 },
    { 0, 0 },
    { 0xe6abcd6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173bcd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x870bcd72, 52 },
    { 0xb4abcd72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x571bcd76, 88 },
    { 0xe10bcd76, 93 },
    { 0x351bcd77, 81 },
    { 0, 0 },
    { 0x204bcd7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689bcd7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefbcd84, 23 },
    { 0, 0 },
    { 0xe59bcd86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0bcd8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bcd8f, 55 },
    { 0xbe9bcd90, 78 },
    { 0x767bcd91, 93 },
    { 0x849bcd91, 84 },
    { 0xae9bcd92, 76 },
    { 0x146bcd93, 61 },
    { 0xb3abcd92, 33 },
    { 0xf99bcd96, 61 },
    { 0x559bcd97, 90 },
    { 0x2edbcd98, 46 },
    { 0x3febcd96, 95 },
    { 0x777bcd92, 46 },
    { 0xbdcbcd9b, 23 },
    { 0xabbbcd9c, 33 },
    { 0x5d9bcd9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x597bcda0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1abcdb9, 33 },
    { 0, 0 },
    { 0x269bcdbb, 23 },
    { 0x7b4bcdbb, 98 },
    { 0x182bcdbd, 33 },
    { 0, 0 },
    { 0xe31bcdbf, 34 },
    { 0xaffbcdbf, 98 },
    { 0xdcbcdc1, 52 },
    { 0x4aabcdbf, 34 },
    { 0x91ebcdc3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddbcdc6, 37 },
    { 0, 0 },
    { 0xde0bcdc8, 90 },
    { 0xa97bcdc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbcdcc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2bcdd1, 52 },
    { 0xd3bcdd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bcdd7, 68 },
    { 0x14dbcdd7, 90 },
    { 0, 0 },
    { 0xd19bcdda, 95 },
    { 0xae1bcdda, 100 },
    { 0, 0 },
    { 0x65ebcddd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251bcde0, 99 },
    { 0xe0bbcde1, 98 },
    { 0x54ebcde1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bcde5, 43 },
    { 0xab7bcde6, 90 },
    { 0xf37bcde7, 52 },
    { 0, 0 },
    { 0x33abcde9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cbcdfb, 83 },
    { 0xfcfbcdfb, 95 },
    { 0x77bbcdfb, 43 },
    { 0x1c1bcdfc, 95 },
    { 0xac8bcdfe, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedabce05, 93 },
    { 0x422bce06, 90 },
    { 0x54ebce06, 90 },
    { 0x3b3bce08, 95 },
    { 0xf5bce07, 80 },
    { 0x1e8bce09, 33 },
    { 0x4c6bce07, 93 },
    { 0x94bce0c, 23 },
    { 0xaa3bce09, 100 },
    { 0xccbbce0e, 100 },
    { 0, 0 },
    { 0x2dabce10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104bce16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3bce19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413bce1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcbce27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7bce2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bbce2d, 83 },
    { 0xdedbce2d, 28 },
    { 0, 0 },
    { 0x1a6bce30, 52 },
    { 0x62abce30, 61 },
    { 0x58dbce30, 61 },
    { 0x36fbce33, 43 },
    { 0xef1bce33, 33 },
    { 0x503bce30, 98 },
    { 0x808bce36, 46 },
    { 0x298bce36, 17 },
    { 0, 0 },
    { 0x5fabce39, 93 },
    { 0, 0 },
    { 0x885bce3b, 33 },
    { 0x272bce3c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8bce42, 33 },
    { 0, 0 },
    { 0x9adbce44, 95 },
    { 0xa4cbce45, 23 },
    { 0xbe4bce44, 61 },
    { 0, 0 },
    { 0x7eebce48, 33 },
    { 0x864bce49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ebce4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0bce52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afbce55, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dbce5a, 43 },
    { 0, 0 },
    { 0x9febce5c, 23 },
    { 0x204bce5d, 23 },
    { 0x964bce5e, 33 },
    { 0x583bce5c, 98 },
    { 0x91ebce5d, 87 },
    { 0x7bdbce61, 84 },
    { 0x4c0bce5c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739bce6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75bce71, 90 },
    { 0x805bce72, 98 },
    { 0xa21bce73, 98 },
    { 0x1d4bce73, 34 },
    { 0xd93bce75, 83 },
    { 0x602bce72, 33 },
    { 0x609bce77, 90 },
    { 0, 0 },
    { 0xd8ebce79, 52 },
    { 0x607bce7a, 34 },
    { 0x633bce7a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x707bce7e, 84 },
    { 0, 0 },
    { 0x8b2bce80, 90 },
    { 0, 0 },
    { 0xc4bbce82, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583bce86, 23 },
    { 0x94bce86, 95 },
    { 0x2ddbce87, 61 },
    { 0xb8ebce88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53bce8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23bce91, 93 },
    { 0, 0 },
    { 0x831bce93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x266bce96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adbce9b, 91 },
    { 0xd9fbce9c, 61 },
    { 0xffebce9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92bcea0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485bcea5, 90 },
    { 0xf63bcea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451bceab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bbceae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5bceb2, 98 },
    { 0xb7fbceb3, 90 },
    { 0x37abceb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x328bceb7, 33 },
    { 0x42cbceb7, 33 },
    { 0, 0 },
    { 0x177bceba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccbcebd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40abcec4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9bced0, 90 },
    { 0xe95bced0, 66 },
    { 0, 0 },
    { 0xc98bced3, 61 },
    { 0, 0 },
    { 0xdebced5, 75 },
    { 0xb8abced6, 61 },
    { 0x9abced7, 33 },
    { 0x86abced7, 98 },
    { 0, 0 },
    { 0xa8dbceda, 95 },
    { 0xdeebceda, 61 },
    { 0, 0 },
    { 0x5a2bcedd, 55 },
    { 0x24ebcede, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bcee7, 34 },
    { 0, 0 },
    { 0xcc4bcee9, 76 },
    { 0x71abceea, 61 },
    { 0x548bceea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdbcef0, 93 },
    { 0xb9fbcef1, 55 },
    { 0x1c5bcef0, 99 },
    { 0, 0 },
    { 0x1f6bcef4, 98 },
    { 0, 0 },
    { 0x514bcef6, 90 },
    { 0x326bcef7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913bcefb, 61 },
    { 0xdfabcefb, 98 },
    { 0x99abcefd, 33 },
    { 0x4cbcefe, 95 },
    { 0xe67bceff, 95 },
    { 0, 0 },
    { 0x45dbcf01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9bcf06, 81 },
    { 0, 0 },
    { 0x2cfbcf08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bcf0b, 83 },
    { 0x3e1bcf0b, 95 },
    { 0xfd0bcf0d, 23 },
    { 0xc29bcf0b, 43 },
    { 0x698bcf0f, 90 },
    { 0x506bcf0e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3abcf16, 87 },
    { 0xac0bcf17, 33 },
    { 0xba5bcf17, 56 },
    { 0xa7ebcf19, 90 },
    { 0x219bcf16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479bcf21, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676bcf26, 98 },
    { 0, 0 },
    { 0xfebbcf28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebcf2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ebcf2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0bcf32, 46 },
    { 0, 0 },
    { 0xe66bcf34, 69 },
    { 0, 0 },
    { 0x51dbcf36, 81 },
    { 0x9d6bcf36, 23 },
    { 0xeadbcf38, 33 },
    { 0xfaabcf37, 23 },
    { 0, 0 },
    { 0x62ebcf3b, 98 },
    { 0, 0 },
    { 0x89bcf3d, 79 },
    { 0x44bcf3d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cbcf43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2bcf4a, 17 },
    { 0x255bcf4b, 75 },
    { 0x1dabcf4c, 33 },
    { 0x564bcf4c, 95 },
    { 0xb93bcf4e, 43 },
    { 0xd8fbcf4f, 61 },
    { 0, 0 },
    { 0xc1bbcf51, 33 },
    { 0, 0 },
    { 0xebfbcf53, 33 },
    { 0, 0 },
    { 0x189bcf55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36bcf5a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bcf5e, 23 },
    { 0xcb3bcf5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdbcf63, 76 },
    { 0xe0dbcf64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837bcf68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43bcf6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12bcf71, 91 },
    { 0, 0 },
    { 0x949bcf73, 85 },
    { 0, 0 },
    { 0xf40bcf75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ebcf7a, 43 },
    { 0x61cbcf7b, 17 },
    { 0x144bcf7b, 93 },
    { 0x359bcf7d, 33 },
    { 0, 0 },
    { 0xd03bcf7f, 90 },
    { 0, 0 },
    { 0x499bcf81, 23 },
    { 0xe66bcf82, 34 },
    { 0xf76bcf83, 33 },
    { 0x327bcf82, 95 },
    { 0xe80bcf85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12bcf8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adbcf93, 52 },
    { 0, 0 },
    { 0xe76bcf95, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cbcf98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38bcf9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0bcfa2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918bcfa7, 55 },
    { 0x428bcfa7, 93 },
    { 0, 0 },
    { 0x49ebcfaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3bcfb3, 99 },
    { 0x141bcfb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59bcfb8, 75 },
    { 0x254bcfb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bbcfbc, 66 },
    { 0, 0 },
    { 0xe02bcfbe, 45 },
    { 0x62ebcfbf, 93 },
    { 0xbfbcfc0, 90 },
    { 0x913bcfbe, 33 },
    { 0x6edbcfbe, 95 },
    { 0, 0 },
    { 0x765bcfc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abbcfd0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135bcfda, 46 },
    { 0, 0 },
    { 0x5dfbcfdc, 33 },
    { 0x7efbcfdc, 61 },
    { 0x737bcfdc, 61 },
    { 0xdaebcfdf, 61 },
    { 0xb33bcfe0, 7 },
    { 0, 0 },
    { 0xf96bcfe2, 98 },
    { 0x707bcfe3, 23 },
    { 0x779bcfe4, 66 },
    { 0x444bcfe5, 46 },
    { 0x2c3bcfe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ebcfe9, 61 },
    { 0xf48bcfe9, 64 },
    { 0x3cdbcfeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cbcff1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13bcff5, 55 },
    { 0x46bcff6, 33 },
    { 0, 0 },
    { 0xfbabcff8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9bcffc, 61 },
    { 0x11bbcffd, 34 },
    { 0, 0 },
    { 0x229bcfff, 23 },
    { 0xd03bd000, 90 },
    { 0x615bd000, 99 },
    { 0, 0 },
    { 0x9c8bd003, 78 },
    { 0, 0 },
    { 0x7c1bd005, 90 },
    { 0xc5dbd006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5debd00c, 98 },
    { 0x669bd00d, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ebd010, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edbd020, 80 },
    { 0x1bebd021, 43 },
    { 0x47dbd022, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52bd029, 95 },
    { 0xf00bd02a, 84 },
    { 0xad1bd02b, 38 },
    { 0xeffbd02b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125bd030, 52 },
    { 0x17abd031, 33 },
    { 0xccfbd032, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88bd03b, 66 },
    { 0x1b3bd03c, 33 },
    { 0x82bbd03c, 55 },
    { 0x794bd03e, 88 },
    { 0xfdcbd03e, 90 },
    { 0xa72bd040, 76 },
    { 0xa1abd03f, 55 },
    { 0xd48bd040, 66 },
    { 0xd6dbd043, 33 },
    { 0x4bebd03b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396bd04b, 43 },
    { 0xaf1bd04c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16bd051, 98 },
    { 0x960bd052, 61 },
    { 0x2fbbd052, 87 },
    { 0x74cbd054, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x276bd057, 29 },
    { 0xf7dbd058, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725bd062, 88 },
    { 0xdc7bd063, 33 },
    { 0x62dbd064, 92 },
    { 0x66bbd063, 75 },
    { 0, 0 },
    { 0x4fdbd067, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8bd06b, 43 },
    { 0xf98bd06c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06bd070, 77 },
    { 0, 0 },
    { 0x97abd072, 33 },
    { 0x4f4bd072, 90 },
    { 0xc05bd074, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9bd079, 17 },
    { 0xe45bd07a, 91 },
    { 0xe65bd07a, 95 },
    { 0x36dbd07c, 23 },
    { 0x1fbbd07d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fbd080, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dbd089, 67 },
    { 0x356bd08a, 76 },
    { 0xea8bd089, 61 },
    { 0xc01bd08c, 61 },
    { 0xe54bd089, 33 },
    { 0x7a0bd08e, 76 },
    { 0x33bbd08f, 78 },
    { 0xa1dbd090, 55 },
    { 0x270bd08a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ebd096, 34 },
    { 0x4febd097, 46 },
    { 0x4a7bd098, 23 },
    { 0, 0 },
    { 0x442bd09a, 90 },
    { 0xaffbd09b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7bd0a4, 90 },
    { 0x8bebd0a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bd0af, 80 },
    { 0x191bd0b0, 90 },
    { 0x8bcbd0b1, 87 },
    { 0x67bbd0b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfbd0b8, 33 },
    { 0x602bd0b8, 33 },
    { 0x5efbd0b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435bd0c0, 77 },
    { 0x454bd0c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dbd0c6, 69 },
    { 0x34bd0c7, 55 },
    { 0xcebd0c6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615bd0cd, 34 },
    { 0x33dbd0ce, 23 },
    { 0, 0 },
    { 0x5b0bd0d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19abd0d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebd0df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x988bd0e4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dbd0e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x606bd0eb, 33 },
    { 0xd7fbd0ec, 17 },
    { 0, 0 },
    { 0x81bbd0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadbd0f1, 25 },
    { 0x1d5bd0f2, 93 },
    { 0x5c1bd0f3, 61 },
    { 0xbd0bd0f3, 33 },
    { 0xc09bd0f3, 91 },
    { 0xc1bd0f3, 76 },
    { 0x4ddbd0f7, 90 },
    { 0, 0 },
    { 0x9e7bd0f9, 23 },
    { 0x43cbd0f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16bd0fd, 95 },
    { 0, 0 },
    { 0x194bd0ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddbd103, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bd106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x537bd109, 55 },
    { 0, 0 },
    { 0xe91bd10b, 34 },
    { 0, 0 },
    { 0x837bd10d, 90 },
    { 0, 0 },
    { 0x9d7bd10f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805bd115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0bd118, 34 },
    { 0xd08bd119, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649bd121, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2bd124, 23 },
    { 0xa9fbd125, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10bd12e, 90 },
    { 0x984bd12e, 78 },
    { 0xf15bd130, 80 },
    { 0xce1bd130, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233bd138, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605bd13e, 87 },
    { 0x490bd13e, 61 },
    { 0xe6fbd13e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4bd143, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4bd14d, 90 },
    { 0, 0 },
    { 0x828bd14f, 55 },
    { 0x96dbd150, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x935bd153, 33 },
    { 0, 0 },
    { 0x34bd155, 52 },
    { 0, 0 },
    { 0x576bd157, 23 },
    { 0x55dbd157, 34 },
    { 0xae8bd159, 33 },
    { 0xd68bd15a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bbd165, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58bd16a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960bd170, 33 },
    { 0, 0 },
    { 0xeaabd172, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5bd178, 93 },
    { 0, 0 },
    { 0x1fbd17a, 33 },
    { 0xa67bd17a, 84 },
    { 0xbbbbd17c, 95 },
    { 0xc27bd17a, 90 },
    { 0, 0 },
    { 0x159bd17f, 90 },
    { 0x3c4bd180, 90 },
    { 0, 0 },
    { 0x726bd182, 95 },
    { 0x8d5bd183, 95 },
    { 0xeebbd183, 98 },
    { 0x4e6bd184, 23 },
    { 0x148bd186, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6bd18a, 61 },
    { 0, 0 },
    { 0x6fcbd18c, 68 },
    { 0x899bd18c, 55 },
    { 0x9fbbd18e, 87 },
    { 0x69bd18f, 100 },
    { 0x73bbd18f, 90 },
    { 0, 0 },
    { 0x589bd192, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ebd197, 98 },
    { 0, 0 },
    { 0x4c1bd199, 99 },
    { 0x276bd19a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3bd19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x751bd1a0, 77 },
    { 0x786bd1a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596bd1a6, 23 },
    { 0x1e2bd1a7, 23 },
    { 0, 0 },
    { 0xb4bbd1a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5bd1af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47bd1b3, 23 },
    { 0x938bd1b4, 34 },
    { 0, 0 },
    { 0x285bd1b6, 75 },
    { 0x2e2bd1b7, 23 },
    { 0, 0 },
    { 0xa67bd1b9, 98 },
    { 0xd15bd1ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54bd1c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868bd1c6, 93 },
    { 0, 0 },
    { 0x9debd1c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ebd1cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebd1d4, 61 },
    { 0x8e8bd1d5, 76 },
    { 0xd3dbd1d6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cbd1de, 17 },
    { 0x464bd1de, 100 },
    { 0, 0 },
    { 0xa32bd1e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ebd1ed, 61 },
    { 0, 0 },
    { 0x9f1bd1ef, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7bd1f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eebd1f8, 98 },
    { 0, 0 },
    { 0x748bd1fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bbd205, 87 },
    { 0, 0 },
    { 0x9cbbd207, 34 },
    { 0xee5bd208, 61 },
    { 0xd6abd209, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37bd20d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983bd213, 98 },
    { 0xa7dbd214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bbd219, 33 },
    { 0, 0 },
    { 0x912bd21b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1bd223, 91 },
    { 0x900bd223, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fbd227, 87 },
    { 0xcd0bd228, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2bd22d, 76 },
    { 0x638bd22e, 55 },
    { 0xe6dbd22f, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1bd233, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59bd236, 23 },
    { 0x3edbd237, 95 },
    { 0xf20bd238, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cebd23b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdbd240, 33 },
    { 0, 0 },
    { 0xc04bd242, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0bd245, 98 },
    { 0xa67bd245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dbd249, 98 },
    { 0, 0 },
    { 0xabbbd24b, 100 },
    { 0x7b9bd24c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80abd252, 66 },
    { 0x575bd253, 91 },
    { 0, 0 },
    { 0xee3bd255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505bd259, 34 },
    { 0x631bd25a, 17 },
    { 0x173bd25b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x794bd25e, 33 },
    { 0xaeabd25f, 88 },
    { 0, 0 },
    { 0x4cebd261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112bd265, 55 },
    { 0x25bd266, 99 },
    { 0xf6bd267, 17 },
    { 0x21bd267, 68 },
    { 0x469bd268, 66 },
    { 0x80dbd26a, 23 },
    { 0x896bd267, 65 },
    { 0x1e4bd269, 77 },
    { 0xb44bd26c, 34 },
    { 0x692bd26e, 23 },
    { 0x8c7bd26f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dbd272, 33 },
    { 0xe49bd273, 23 },
    { 0, 0 },
    { 0x8b2bd275, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560bd27c, 66 },
    { 0, 0 },
    { 0x1cfbd27e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bbd28a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92bd296, 46 },
    { 0x3debd296, 90 },
    { 0xf0bbd297, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efbd29b, 91 },
    { 0x8b6bd29c, 98 },
    { 0, 0 },
    { 0xe8bbd29e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1bd2a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafbd2a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28bd2ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ebd2b1, 98 },
    { 0x378bd2b2, 90 },
    { 0xc77bd2b2, 33 },
    { 0, 0 },
    { 0xcfabd2b5, 95 },
    { 0x999bd2b6, 98 },
    { 0, 0 },
    { 0x925bd2b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bd2bd, 34 },
    { 0xa3bbd2bd, 90 },
    { 0, 0 },
    { 0x88bd2c0, 23 },
    { 0x813bd2c1, 34 },
    { 0xcf4bd2c2, 43 },
    { 0xbc2bd2c2, 95 },
    { 0xf3abd2c4, 33 },
    { 0x503bd2c4, 93 },
    { 0x37cbd2c5, 33 },
    { 0xd0ebd2c7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x777bd2ca, 76 },
    { 0x500bd2cb, 46 },
    { 0x523bd2cb, 90 },
    { 0x544bd2ca, 95 },
    { 0xd04bd2ce, 95 },
    { 0, 0 },
    { 0x28abd2d0, 90 },
    { 0, 0 },
    { 0x30abd2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dbd2d7, 46 },
    { 0xa12bd2d8, 33 },
    { 0, 0 },
    { 0x21abd2da, 46 },
    { 0x5c2bd2da, 46 },
    { 0x5e1bd2dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3bd2e0, 61 },
    { 0x9dcbd2e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcebd2e6, 95 },
    { 0xb17bd2e7, 43 },
    { 0, 0 },
    { 0x6f4bd2e9, 34 },
    { 0x3e8bd2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6bd2f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704bd2f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcbd2fa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1bd2fe, 95 },
    { 0xc7abd2ff, 52 },
    { 0xcb3bd2ff, 40 },
    { 0x6f7bd2ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3bd305, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bbd308, 98 },
    { 0xb6cbd309, 68 },
    { 0xa82bd308, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cbd30d, 61 },
    { 0x73bbd30e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aebd317, 23 },
    { 0, 0 },
    { 0x1d5bd319, 100 },
    { 0xa0ebd31a, 95 },
    { 0x6a8bd31a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0bd31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6bd323, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102bd32a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364bd332, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x600bd335, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08bd340, 77 },
    { 0x631bd341, 23 },
    { 0xda8bd340, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10bd349, 90 },
    { 0, 0 },
    { 0xa43bd34b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62bd34e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524bd355, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27abd358, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3bd35b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd35f, 90 },
    { 0x49abd360, 33 },
    { 0xb33bd360, 95 },
    { 0x121bd361, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dbd367, 98 },
    { 0xa73bd368, 61 },
    { 0xdcdbd368, 40 },
    { 0x8a0bd36a, 93 },
    { 0, 0 },
    { 0x707bd36c, 61 },
    { 0xee9bd36d, 23 },
    { 0xebbbd36e, 55 },
    { 0, 0 },
    { 0x9b9bd370, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9abd37a, 99 },
    { 0xe74bd37b, 60 },
    { 0x8babd37b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2bd380, 90 },
    { 0x9f0bd381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x522bd384, 90 },
    { 0xad2bd385, 33 },
    { 0xd8bbd385, 33 },
    { 0xc1fbd387, 43 },
    { 0x541bd387, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71bd38b, 90 },
    { 0x407bd38c, 47 },
    { 0x234bd38c, 95 },
    { 0x4e7bd38e, 43 },
    { 0x18ebd38f, 34 },
    { 0xc2fbd390, 68 },
    { 0x92ebd38f, 76 },
    { 0, 0 },
    { 0x231bd393, 95 },
    { 0x164bd394, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x152bd397, 33 },
    { 0x4b4bd397, 61 },
    { 0xbd399, 43 },
    { 0, 0 },
    { 0x3fbbd39b, 99 },
    { 0xea9bd39c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddbd3a3, 98 },
    { 0xf0cbd3a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1babd3aa, 61 },
    { 0x9d9bd3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fbd3b0, 76 },
    { 0x1b7bd3b1, 23 },
    { 0x868bd3b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bbd3b6, 90 },
    { 0x57abd3b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161bd3be, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x977bd3c1, 43 },
    { 0, 0 },
    { 0x115bd3c3, 33 },
    { 0xc04bd3c4, 33 },
    { 0x126bd3c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1bd3cb, 98 },
    { 0x899bd3cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cbd3d1, 91 },
    { 0x214bd3d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0bd3d7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7bd3dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5bd3ea, 23 },
    { 0x708bd3eb, 98 },
    { 0x661bd3ec, 61 },
    { 0xffcbd3ed, 95 },
    { 0x65abd3ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efbd3f1, 90 },
    { 0, 0 },
    { 0x392bd3f3, 98 },
    { 0, 0 },
    { 0xb38bd3f5, 98 },
    { 0xf0bbd3f6, 55 },
    { 0xcf9bd3f7, 95 },
    { 0x7b3bd3f5, 34 },
    { 0xb00bd3f9, 98 },
    { 0xb46bd3f5, 33 },
    { 0, 0 },
    { 0xc8dbd3fc, 43 },
    { 0, 0 },
    { 0x9debd3fe, 61 },
    { 0x9ebd3ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4bd402, 55 },
    { 0, 0 },
    { 0xd75bd404, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389bd408, 33 },
    { 0, 0 },
    { 0x8f4bd40a, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5abd40e, 90 },
    { 0xdd2bd40f, 90 },
    { 0x485bd40f, 55 },
    { 0x8bfbd40e, 68 },
    { 0xfdebd412, 90 },
    { 0, 0 },
    { 0x137bd414, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbd41e, 90 },
    { 0, 0 },
    { 0x354bd420, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696bd426, 33 },
    { 0xec0bd427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd42f, 90 },
    { 0, 0 },
    { 0xb88bd431, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90bd439, 46 },
    { 0xd38bd439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fbd43d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x341bd440, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53bd446, 55 },
    { 0, 0 },
    { 0x301bd448, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13bd457, 46 },
    { 0x4abbd457, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68bd460, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56bd463, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70abd467, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x610bd46a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeebd46e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaabd474, 33 },
    { 0, 0 },
    { 0xd50bd476, 100 },
    { 0xb9dbd477, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dbd47e, 61 },
    { 0xac0bd47f, 23 },
    { 0x25bbd47f, 90 },
    { 0, 0 },
    { 0xb9fbd482, 33 },
    { 0xd3bd483, 76 },
    { 0xcb8bd484, 95 },
    { 0x14ebd485, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccbd494, 90 },
    { 0x2e0bd495, 76 },
    { 0x3e5bd496, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51bd499, 52 },
    { 0x2afbd49a, 32 },
    { 0x803bd499, 98 },
    { 0xa6ebd49c, 61 },
    { 0xfdbd499, 99 },
    { 0x231bd49e, 33 },
    { 0x105bd49f, 75 },
    { 0xb9bd499, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x901bd4a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bd4a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28bd4a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9abd4ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912bd4b8, 90 },
    { 0x6cbbd4b8, 100 },
    { 0xa44bd4ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aabd4c2, 40 },
    { 0x23cbd4c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1bd4c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5bd4cc, 91 },
    { 0xdd3bd4cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19bd4d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cbd4d4, 23 },
    { 0x801bd4d5, 61 },
    { 0xf62bd4d5, 90 },
    { 0xbaebd4d7, 61 },
    { 0xee3bd4d5, 66 },
    { 0xe0cbd4d9, 61 },
    { 0x52ebd4da, 33 },
    { 0x553bd4db, 98 },
    { 0x6ebd4d9, 66 },
    { 0x6fbbd4dd, 91 },
    { 0x30abd4de, 87 },
    { 0x676bd4df, 76 },
    { 0xe96bd4df, 76 },
    { 0xf12bd4e1, 90 },
    { 0, 0 },
    { 0x630bd4e3, 61 },
    { 0, 0 },
    { 0xaf5bd4e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4bd4e8, 91 },
    { 0x971bd4e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x944bd4ec, 33 },
    { 0x46bd4ed, 33 },
    { 0xdf8bd4ee, 90 },
    { 0x51fbd4ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dbd4f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dbd4f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd4f8, 88 },
    { 0, 0 },
    { 0x2f5bd4fa, 23 },
    { 0, 0 },
    { 0x7f7bd4fc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49bd502, 23 },
    { 0, 0 },
    { 0x744bd504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bbd507, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49abd50d, 95 },
    { 0xc51bd50e, 91 },
    { 0x467bd50f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2bd516, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dbd519, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3bd51d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19bd524, 33 },
    { 0, 0 },
    { 0xc6dbd526, 23 },
    { 0x322bd526, 95 },
    { 0xe5abd527, 66 },
    { 0x743bd526, 93 },
    { 0x7e8bd526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23bd52d, 23 },
    { 0, 0 },
    { 0xa87bd52f, 84 },
    { 0, 0 },
    { 0xe89bd531, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6bd535, 33 },
    { 0xc71bd536, 96 },
    { 0, 0 },
    { 0xad6bd538, 90 },
    { 0, 0 },
    { 0x521bd53a, 52 },
    { 0x1a2bd53b, 43 },
    { 0x949bd53c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9bd540, 34 },
    { 0x112bd540, 49 },
    { 0x16bd540, 78 },
    { 0xcc7bd542, 95 },
    { 0xdb9bd541, 99 },
    { 0x869bd545, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebbd549, 90 },
    { 0, 0 },
    { 0xf6ebd54b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77bd54e, 33 },
    { 0x11fbd54f, 78 },
    { 0, 0 },
    { 0x747bd551, 23 },
    { 0x9e3bd552, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbd557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cbd55b, 52 },
    { 0x4eebd55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289bd561, 66 },
    { 0, 0 },
    { 0x693bd563, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bd569, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fbd56c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8bd572, 91 },
    { 0x48dbd573, 55 },
    { 0xb87bd573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbd57b, 95 },
    { 0x86cbd57b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9abd581, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ebd584, 88 },
    { 0, 0 },
    { 0x1b6bd586, 67 },
    { 0x574bd587, 55 },
    { 0, 0 },
    { 0xab3bd589, 77 },
    { 0x83abd589, 84 },
    { 0, 0 },
    { 0x8bd58c, 93 },
    { 0x32bbd58d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2bd593, 52 },
    { 0, 0 },
    { 0xeb0bd595, 93 },
    { 0x66bbd596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dbd599, 66 },
    { 0, 0 },
    { 0xc4ebd59b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaebd5a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cbd5a9, 76 },
    { 0x3dfbd5aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61bd5ae, 77 },
    { 0x723bd5ae, 95 },
    { 0x3c2bd5ae, 76 },
    { 0x980bd5b1, 81 },
    { 0xc7ebd5b2, 90 },
    { 0x571bd5b3, 84 },
    { 0xdbbbd5b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191bd5ba, 90 },
    { 0xabebd5ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500bd5c1, 99 },
    { 0xbe8bd5c2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10bd5c6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1bd5d1, 34 },
    { 0x70bbd5d2, 33 },
    { 0x180bd5d3, 33 },
    { 0x2b4bd5d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dbd5db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965bd5e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd5e4, 38 },
    { 0xb3ebd5e5, 43 },
    { 0xb90bd5e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57bd5e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13bd5f0, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30bd5f5, 72 },
    { 0x8e2bd5f6, 34 },
    { 0xbcbbd5f7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adbd5fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5bd5fd, 65 },
    { 0x4e0bd5fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95bd603, 98 },
    { 0x4d5bd604, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5bd609, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37bd60d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ebd610, 55 },
    { 0xc20bd611, 93 },
    { 0, 0 },
    { 0x4fbbd613, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329bd61b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bd61e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4bd622, 66 },
    { 0, 0 },
    { 0xd05bd624, 85 },
    { 0xf15bd625, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261bd62a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd62f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353bd633, 95 },
    { 0x9d7bd634, 99 },
    { 0xe63bd634, 33 },
    { 0xe66bd636, 98 },
    { 0x2cabd633, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddbd63a, 81 },
    { 0xe8bd63b, 77 },
    { 0x4f0bd63b, 61 },
    { 0, 0 },
    { 0x118bd63e, 23 },
    { 0x1dcbd63f, 98 },
    { 0x49fbd640, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195bd64c, 90 },
    { 0xecabd64d, 33 },
    { 0xa9fbd64c, 85 },
    { 0xef8bd64f, 78 },
    { 0x11fbd64d, 93 },
    { 0xb48bd651, 61 },
    { 0x805bd651, 84 },
    { 0x3e8bd653, 76 },
    { 0x7debd654, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fbd659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8abd65d, 23 },
    { 0, 0 },
    { 0x882bd65f, 23 },
    { 0x961bd65f, 76 },
    { 0, 0 },
    { 0xf53bd662, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17bd665, 61 },
    { 0, 0 },
    { 0x3d3bd667, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbbd66b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0abd670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37bd675, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34bd678, 66 },
    { 0x989bd679, 61 },
    { 0, 0 },
    { 0x3d6bd67b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491bd682, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebbd686, 76 },
    { 0xbc1bd687, 46 },
    { 0x86abd687, 99 },
    { 0x345bd688, 98 },
    { 0xa9ebd68a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbd68d, 44 },
    { 0xf98bd68e, 87 },
    { 0x4fbd68f, 99 },
    { 0xbe4bd68f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ebd693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67bd696, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x213bd699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdbd69c, 76 },
    { 0x871bd69d, 23 },
    { 0, 0 },
    { 0xd51bd69f, 100 },
    { 0x46fbd69f, 76 },
    { 0, 0 },
    { 0x826bd6a2, 76 },
    { 0xa7cbd6a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fbd6a7, 61 },
    { 0xa25bd6a8, 33 },
    { 0, 0 },
    { 0x5d3bd6aa, 66 },
    { 0xd98bd6aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79bd6af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x438bd6b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x204bd6b5, 61 },
    { 0x3debd6b5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909bd6bc, 77 },
    { 0, 0 },
    { 0xf91bd6be, 61 },
    { 0, 0 },
    { 0x127bd6c0, 76 },
    { 0xa96bd6c1, 55 },
    { 0xfbd6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x244bd6c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53bd6ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x474bd6d1, 76 },
    { 0, 0 },
    { 0xee3bd6d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bbd6da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ebd6de, 34 },
    { 0x5ebd6df, 67 },
    { 0, 0 },
    { 0xc43bd6e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cebd6eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9bd6ee, 23 },
    { 0, 0 },
    { 0x52cbd6f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abd6f7, 33 },
    { 0x370bd6f8, 84 },
    { 0xdafbd6f8, 34 },
    { 0x7c1bd6fa, 23 },
    { 0x71bd6fb, 91 },
    { 0xaedbd6fc, 55 },
    { 0x311bd6fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0bd701, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973bd708, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7bd710, 34 },
    { 0, 0 },
    { 0x68fbd712, 23 },
    { 0xb61bd713, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72bd716, 17 },
    { 0, 0 },
    { 0x8d3bd718, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85bd71c, 81 },
    { 0x14fbd71c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461bd723, 78 },
    { 0, 0 },
    { 0x16ebd725, 98 },
    { 0x3aebd725, 61 },
    { 0xbbbbd726, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dbd72a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59bd72d, 99 },
    { 0x8f0bd72d, 23 },
    { 0, 0 },
    { 0x661bd730, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0bd733, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5febd737, 34 },
    { 0xb56bd738, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ebd73e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141bd744, 98 },
    { 0x334bd745, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43bd74b, 55 },
    { 0xddcbd74c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x140bd74f, 23 },
    { 0, 0 },
    { 0x828bd751, 90 },
    { 0xc01bd752, 88 },
    { 0, 0 },
    { 0xf74bd754, 85 },
    { 0x9f3bd755, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x977bd758, 95 },
    { 0, 0 },
    { 0x96bd75a, 90 },
    { 0x521bd75b, 98 },
    { 0, 0 },
    { 0xd25bd75d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2bd764, 95 },
    { 0, 0 },
    { 0x74ebd766, 87 },
    { 0, 0 },
    { 0xe0abd768, 76 },
    { 0, 0 },
    { 0xd83bd76a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950bd770, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adbd776, 90 },
    { 0, 0 },
    { 0xe3ebd778, 78 },
    { 0, 0 },
    { 0xc93bd77a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10bd780, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39bd783, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9bd788, 55 },
    { 0x28fbd789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124bd791, 46 },
    { 0xb4ebd792, 99 },
    { 0xfb2bd793, 95 },
    { 0xc84bd793, 33 },
    { 0x660bd795, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cbd799, 43 },
    { 0, 0 },
    { 0xa6fbd79b, 23 },
    { 0x560bd79b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bd7ac, 33 },
    { 0x6bbbd7ad, 43 },
    { 0, 0 },
    { 0xcfcbd7af, 78 },
    { 0, 0 },
    { 0x4bcbd7b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6bd7b4, 93 },
    { 0xd21bd7b5, 43 },
    { 0x143bd7b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9bd7c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9bd7c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838bd7ca, 98 },
    { 0xe3cbd7ca, 90 },
    { 0, 0 },
    { 0xab2bd7cd, 77 },
    { 0x4d2bd7ce, 61 },
    { 0xea0bd7cd, 95 },
    { 0xd1ebd7cd, 87 },
    { 0xbb7bd7ce, 24 },
    { 0xbe3bd7d0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bebd7d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dbd7dc, 43 },
    { 0xbf6bd7dd, 78 },
    { 0, 0 },
    { 0xef4bd7df, 66 },
    { 0x27fbd7e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0bd7e4, 90 },
    { 0x2e1bd7e4, 95 },
    { 0x100bd7e6, 88 },
    { 0x44fbd7e6, 43 },
    { 0xdafbd7e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86bd7eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761bd7ef, 99 },
    { 0x324bd7f0, 64 },
    { 0, 0 },
    { 0x4c9bd7f2, 40 },
    { 0x3bebd7f2, 46 },
    { 0x82bbd7f2, 23 },
    { 0x247bd7f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7bd806, 46 },
    { 0, 0 },
    { 0xc17bd808, 90 },
    { 0x433bd809, 61 },
    { 0xfd9bd80a, 33 },
    { 0x6bd809, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49bd811, 95 },
    { 0, 0 },
    { 0x8a9bd813, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bbd817, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8bd81c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x882bd81f, 67 },
    { 0xe1abd81f, 87 },
    { 0x454bd821, 90 },
    { 0x7cbbd822, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602bd826, 52 },
    { 0xa76bd827, 19 },
    { 0, 0 },
    { 0x7bcbd829, 23 },
    { 0x29ebd82a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12abd830, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x971bd833, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9bd837, 23 },
    { 0xd61bd837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccbd83f, 61 },
    { 0, 0 },
    { 0xa23bd841, 33 },
    { 0x2bd842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297bd846, 98 },
    { 0, 0 },
    { 0xa10bd848, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cbd84b, 92 },
    { 0x8adbd84c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dbd850, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4bd855, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x332bd858, 61 },
    { 0x614bd858, 95 },
    { 0, 0 },
    { 0x15cbd85b, 98 },
    { 0, 0 },
    { 0x211bd85d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fbd861, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564bd865, 33 },
    { 0x526bd866, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5bd869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7bd86e, 90 },
    { 0x328bd86f, 61 },
    { 0x14cbd86e, 55 },
    { 0xad7bd871, 55 },
    { 0x7cebd872, 90 },
    { 0x28ebd86e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53bd879, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaabd87c, 90 },
    { 0, 0 },
    { 0x4bbd87e, 76 },
    { 0, 0 },
    { 0x627bd880, 66 },
    { 0x84fbd881, 33 },
    { 0xed1bd881, 85 },
    { 0x7c1bd883, 78 },
    { 0, 0 },
    { 0x26dbd885, 66 },
    { 0xdabd885, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bbd88d, 98 },
    { 0, 0 },
    { 0x598bd88f, 61 },
    { 0x7dbbd890, 90 },
    { 0x4e0bd890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fbd896, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1bd899, 43 },
    { 0x219bd89a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbbd89d, 95 },
    { 0x3a7bd89e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bd8a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cbd8af, 91 },
    { 0, 0 },
    { 0x18fbd8b1, 17 },
    { 0xe7bbd8b2, 23 },
    { 0xc79bd8b2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bd8b6, 59 },
    { 0, 0 },
    { 0x5fabd8b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecbd8c0, 90 },
    { 0xe6abd8c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dbd8c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbd8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909bd8d9, 33 },
    { 0x97dbd8d9, 75 },
    { 0x4f3bd8db, 23 },
    { 0x3b3bd8db, 90 },
    { 0xbbcbd8dc, 84 },
    { 0x134bd8de, 33 },
    { 0x2e4bd8dd, 95 },
    { 0xed6bd8e0, 90 },
    { 0x4b0bd8de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58bd8e6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcbd8ed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bbd8f1, 100 },
    { 0xe0fbd8f2, 33 },
    { 0xc63bd8f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8bd8f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89abd8f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x836bd8fc, 33 },
    { 0, 0 },
    { 0x467bd8fe, 87 },
    { 0xa5fbd8fe, 90 },
    { 0x1b5bd900, 76 },
    { 0, 0 },
    { 0x37bbd902, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67bd906, 96 },
    { 0xfe5bd907, 61 },
    { 0, 0 },
    { 0x88bd909, 76 },
    { 0xf16bd909, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8bd90e, 76 },
    { 0, 0 },
    { 0x26bbd910, 33 },
    { 0, 0 },
    { 0xa00bd912, 78 },
    { 0, 0 },
    { 0x358bd914, 90 },
    { 0xec7bd915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5bd919, 61 },
    { 0x72abd91a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12bd91f, 56 },
    { 0xf3fbd920, 92 },
    { 0x448bd91f, 95 },
    { 0xda2bd922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adbd925, 36 },
    { 0xdfcbd925, 98 },
    { 0x60abd926, 95 },
    { 0xb2cbd927, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656bd92c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3bd932, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cbd939, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cbd93e, 100 },
    { 0x22ebd93e, 33 },
    { 0xb99bd940, 95 },
    { 0x8dbbd940, 61 },
    { 0, 0 },
    { 0x63bbd943, 23 },
    { 0, 0 },
    { 0x82dbd945, 100 },
    { 0xf9abd946, 90 },
    { 0, 0 },
    { 0xe9ebd948, 33 },
    { 0x4acbd948, 23 },
    { 0xe22bd94a, 93 },
    { 0xa9ebd94b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bd956, 95 },
    { 0xb14bd957, 76 },
    { 0x2b9bd958, 90 },
    { 0x2f5bd959, 90 },
    { 0, 0 },
    { 0x83fbd95b, 67 },
    { 0x72abd95c, 90 },
    { 0x138bd95d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4bd961, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0bd965, 33 },
    { 0x4dabd966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23bd96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763bd970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbbd973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cbd979, 94 },
    { 0xc68bd97a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2bd980, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dbd984, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215bd98a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88bd98d, 64 },
    { 0, 0 },
    { 0x80bd98f, 47 },
    { 0xcc5bd990, 90 },
    { 0xc14bd991, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322bd996, 17 },
    { 0, 0 },
    { 0xc2dbd998, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cbd99d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56bd9a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4bd9a9, 23 },
    { 0, 0 },
    { 0xe94bd9ab, 55 },
    { 0x86dbd9ac, 98 },
    { 0xd6cbd9ad, 23 },
    { 0x259bd9ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x736bd9b1, 61 },
    { 0xb1bd9b2, 76 },
    { 0x17ebd9b3, 56 },
    { 0x280bd9b3, 23 },
    { 0x326bd9b5, 90 },
    { 0xafbd9b6, 98 },
    { 0x804bd9b5, 87 },
    { 0x6a9bd9b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bd9bc, 33 },
    { 0x84abd9bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877bd9c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x263bd9c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfbd9cb, 98 },
    { 0x139bd9cc, 85 },
    { 0x642bd9cd, 46 },
    { 0x49fbd9ce, 88 },
    { 0x1bbbd9cb, 33 },
    { 0xbd7bd9d0, 95 },
    { 0x9c7bd9d1, 90 },
    { 0x60fbd9cb, 33 },
    { 0xd2bbd9d0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebd9d6, 33 },
    { 0, 0 },
    { 0xdd0bd9d8, 90 },
    { 0, 0 },
    { 0x53bd9da, 53 },
    { 0x5b8bd9db, 55 },
    { 0xfa3bd9dc, 61 },
    { 0, 0 },
    { 0xe41bd9de, 51 },
    { 0x7aabd9df, 66 },
    { 0xa3abd9de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920bd9e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12bd9ec, 95 },
    { 0x79fbd9ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9bd9f8, 98 },
    { 0xd44bd9f9, 95 },
    { 0xb6fbd9fa, 90 },
    { 0, 0 },
    { 0xe3cbd9fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bbda00, 90 },
    { 0xbc8bda01, 78 },
    { 0x6d9bda01, 46 },
    { 0xe2abda02, 55 },
    { 0x45bbda02, 90 },
    { 0xfe0bda00, 90 },
    { 0xcf3bda02, 43 },
    { 0x670bda07, 33 },
    { 0, 0 },
    { 0x2febda09, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfbda0c, 75 },
    { 0, 0 },
    { 0x81dbda0e, 33 },
    { 0, 0 },
    { 0x5a8bda10, 98 },
    { 0, 0 },
    { 0x3a2bda12, 61 },
    { 0x9ffbda12, 17 },
    { 0x18abda14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3bda18, 93 },
    { 0x9e2bda19, 61 },
    { 0xe45bda1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44bda1e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ebda21, 87 },
    { 0x1b2bda22, 90 },
    { 0, 0 },
    { 0x318bda24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x168bda27, 34 },
    { 0x1bdbda28, 66 },
    { 0xd7bbda28, 33 },
    { 0xbd6bda28, 98 },
    { 0x107bda2b, 23 },
    { 0x525bda2c, 95 },
    { 0xf37bda2b, 40 },
    { 0x4fabda2e, 61 },
    { 0xdecbda2f, 87 },
    { 0xdfebda29, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9bda33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7bda3d, 66 },
    { 0xaa7bda3e, 76 },
    { 0x35bbda3e, 33 },
    { 0xd82bda3f, 23 },
    { 0x7a7bda3f, 23 },
    { 0xa56bda3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acbda48, 61 },
    { 0, 0 },
    { 0xf1bbda4a, 23 },
    { 0x198bda4b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56bda50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764bda55, 90 },
    { 0, 0 },
    { 0xcf5bda57, 46 },
    { 0x418bda57, 98 },
    { 0x54bbda59, 67 },
    { 0xce2bda5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661bda60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05bda64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811bda6b, 34 },
    { 0xddcbda6c, 43 },
    { 0, 0 },
    { 0x78dbda6e, 95 },
    { 0, 0 },
    { 0xc01bda70, 75 },
    { 0, 0 },
    { 0x342bda72, 40 },
    { 0x77abda72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7bda78, 45 },
    { 0x4f2bda78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1bda83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216bda89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497bda8e, 87 },
    { 0x5ecbda8e, 87 },
    { 0x5afbda90, 90 },
    { 0x38bda8e, 78 },
    { 0xb28bda91, 98 },
    { 0xf0dbda93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78bda99, 92 },
    { 0x3eebda9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bda9d, 46 },
    { 0x4fcbda9d, 93 },
    { 0xf1abda9f, 33 },
    { 0xffbbda9d, 33 },
    { 0x89abda9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeabdaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bbdaab, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bdab0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bbdab3, 43 },
    { 0x792bdab3, 76 },
    { 0xcb5bdab3, 44 },
    { 0x676bdab6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958bdabb, 69 },
    { 0xf90bdabc, 87 },
    { 0x253bdabc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dbdac0, 100 },
    { 0xbb3bdac1, 33 },
    { 0x43ebdac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7bdac5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02bdac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x907bdacb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddbdace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ebdad2, 95 },
    { 0, 0 },
    { 0x4ebbdad4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33bdad8, 76 },
    { 0, 0 },
    { 0x1c1bdada, 100 },
    { 0xb9fbdadb, 33 },
    { 0, 0 },
    { 0x7e3bdadd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcebdae4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53bdae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496bdaed, 46 },
    { 0, 0 },
    { 0x71abdaef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977bdaf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x250bdafc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbdaff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bdb02, 90 },
    { 0, 0 },
    { 0xf72bdb04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5abdb07, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7bdb0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x724bdb0e, 23 },
    { 0, 0 },
    { 0xc42bdb10, 90 },
    { 0, 0 },
    { 0x214bdb12, 95 },
    { 0xe8abdb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1bdb17, 23 },
    { 0xa33bdb17, 98 },
    { 0xf36bdb17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0bdb1d, 33 },
    { 0xae1bdb1d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2abdb21, 36 },
    { 0xb08bdb22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbbdb29, 33 },
    { 0, 0 },
    { 0x836bdb2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6bdb30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bbdb36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x338bdb39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebdb3c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19bdb40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22abdb44, 23 },
    { 0, 0 },
    { 0x4a9bdb46, 95 },
    { 0, 0 },
    { 0x63cbdb48, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaabdb4c, 65 },
    { 0x57abdb4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bbdb51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bdb56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dbdb5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2bdb60, 90 },
    { 0x3f6bdb61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebbdb65, 90 },
    { 0x55bbdb66, 75 },
    { 0xb58bdb65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32bdb6c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2bdb70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67bdb7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638bdb86, 80 },
    { 0x7bcbdb87, 61 },
    { 0, 0 },
    { 0x4debdb89, 43 },
    { 0, 0 },
    { 0xf36bdb8b, 90 },
    { 0x650bdb8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197bdb92, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43bdb96, 61 },
    { 0x238bdb97, 23 },
    { 0x61ebdb97, 36 },
    { 0, 0 },
    { 0x2fabdb9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264bdba0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ebdba3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173bdbaa, 78 },
    { 0x5fbdbab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47bdbb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abbdbb8, 61 },
    { 0x43abdbb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82bdbc0, 93 },
    { 0, 0 },
    { 0x4ecbdbc2, 43 },
    { 0x183bdbc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626bdbc9, 100 },
    { 0x1bbdbca, 55 },
    { 0, 0 },
    { 0x7c9bdbcc, 90 },
    { 0, 0 },
    { 0x747bdbce, 66 },
    { 0, 0 },
    { 0x64fbdbd0, 98 },
    { 0x2e3bdbd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcbdbd8, 61 },
    { 0xd77bdbd9, 23 },
    { 0xf5abdbd9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158bdbe1, 43 },
    { 0xf95bdbe2, 43 },
    { 0x9dcbdbe1, 76 },
    { 0, 0 },
    { 0x638bdbe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bbdbf0, 23 },
    { 0, 0 },
    { 0x200bdbf2, 64 },
    { 0xfe8bdbf3, 67 },
    { 0, 0 },
    { 0xf63bdbf5, 95 },
    { 0x39bdbf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bdbf9, 81 },
    { 0x1b4bdbf9, 90 },
    { 0x75ebdbf9, 95 },
    { 0x12ebdbfc, 23 },
    { 0x154bdbfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bbdc00, 61 },
    { 0, 0 },
    { 0x60bdc02, 91 },
    { 0xf57bdc03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3bdc08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55bdc0b, 90 },
    { 0, 0 },
    { 0x52bbdc0d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dbdc10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddbdc13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8bdc19, 95 },
    { 0x9a2bdc1a, 90 },
    { 0, 0 },
    { 0x358bdc1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fbdc21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cbdc28, 90 },
    { 0xc2dbdc29, 81 },
    { 0x433bdc2a, 78 },
    { 0xa0bdc2b, 84 },
    { 0xa47bdc2b, 31 },
    { 0, 0 },
    { 0xd66bdc2e, 61 },
    { 0xe57bdc2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68bdc32, 33 },
    { 0x8e5bdc33, 90 },
    { 0xb13bdc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64bdc37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x795bdc3a, 93 },
    { 0xeb6bdc3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9bdc3e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9bdc41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207bdc47, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbbdc4d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83bdc51, 61 },
    { 0x71ebdc52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630bdc5a, 85 },
    { 0xc0dbdc5a, 78 },
    { 0x44bbdc5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9abdc60, 90 },
    { 0, 0 },
    { 0x8c2bdc62, 33 },
    { 0xdb2bdc62, 85 },
    { 0, 0 },
    { 0xdedbdc65, 55 },
    { 0, 0 },
    { 0x289bdc67, 55 },
    { 0xde6bdc68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5bdc6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfbdc72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535bdc76, 33 },
    { 0x51ebdc77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858bdc83, 17 },
    { 0x357bdc84, 95 },
    { 0, 0 },
    { 0x437bdc86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x441bdc89, 46 },
    { 0, 0 },
    { 0xb40bdc8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbbdc8f, 77 },
    { 0x2fcbdc90, 81 },
    { 0xf4cbdc90, 23 },
    { 0, 0 },
    { 0x108bdc93, 99 },
    { 0xe67bdc94, 33 },
    { 0xb0abdc95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6bdc98, 91 },
    { 0xa29bdc98, 93 },
    { 0, 0 },
    { 0x363bdc9b, 68 },
    { 0, 0 },
    { 0x10bdc9d, 61 },
    { 0, 0 },
    { 0x9e4bdc9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffbdca2, 33 },
    { 0, 0 },
    { 0x8b6bdca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4bdca7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eabdcaa, 34 },
    { 0xe93bdcab, 95 },
    { 0xef2bdcab, 33 },
    { 0x1a0bdcad, 61 },
    { 0x162bdcac, 66 },
    { 0xe33bdcaf, 90 },
    { 0x418bdcb0, 95 },
    { 0x702bdcb1, 67 },
    { 0x958bdcb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fbdcb5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291bdcb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1bdcbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fbdcc4, 87 },
    { 0, 0 },
    { 0xb64bdcc6, 83 },
    { 0x525bdcc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74bdcca, 46 },
    { 0xab4bdccb, 34 },
    { 0x7e5bdccc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x283bdccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ebdcd5, 33 },
    { 0x9efbdcd6, 76 },
    { 0x586bdcd7, 90 },
    { 0x7b7bdcd8, 91 },
    { 0xd42bdcd6, 67 },
    { 0, 0 },
    { 0x7bbbdcdb, 33 },
    { 0xd54bdcdc, 95 },
    { 0x3afbdcdd, 46 },
    { 0x5f8bdcdb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bbdceb, 33 },
    { 0x107bdcec, 55 },
    { 0x8cbbdced, 98 },
    { 0xe17bdced, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1bdcf3, 33 },
    { 0x900bdcf4, 98 },
    { 0x614bdcf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efbdcf9, 33 },
    { 0, 0 },
    { 0x357bdcfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179bdd02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ebdd05, 67 },
    { 0x73ebdd06, 87 },
    { 0xdc0bdd06, 61 },
    { 0xcedbdd08, 33 },
    { 0xa86bdd09, 34 },
    { 0, 0 },
    { 0xce6bdd0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671bdd11, 33 },
    { 0xdcebdd12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x251bdd15, 95 },
    { 0, 0 },
    { 0x9f5bdd17, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41bdd1c, 33 },
    { 0xe80bdd1d, 90 },
    { 0x8ffbdd1e, 95 },
    { 0x18bdd1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9bdd22, 98 },
    { 0x63abdd23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8bdd27, 98 },
    { 0, 0 },
    { 0x559bdd29, 95 },
    { 0, 0 },
    { 0x819bdd2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bbdd30, 95 },
    { 0x50cbdd31, 61 },
    { 0xf68bdd32, 90 },
    { 0, 0 },
    { 0x6abdd34, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99bdd3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebbdd43, 98 },
    { 0xaeebdd43, 33 },
    { 0, 0 },
    { 0x9f6bdd46, 100 },
    { 0xf47bdd47, 23 },
    { 0, 0 },
    { 0xc5ebdd49, 43 },
    { 0xbe2bdd4a, 23 },
    { 0xd52bdd49, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327bdd51, 77 },
    { 0x7a0bdd52, 91 },
    { 0xdebbdd53, 61 },
    { 0x236bdd54, 40 },
    { 0x3f1bdd55, 76 },
    { 0, 0 },
    { 0x45abdd57, 90 },
    { 0x6e5bdd58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fbdd5f, 33 },
    { 0x791bdd60, 90 },
    { 0xde5bdd60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0bdd6c, 33 },
    { 0xc82bdd6d, 95 },
    { 0xda9bdd6e, 33 },
    { 0xcf7bdd6f, 98 },
    { 0x948bdd70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741bdd75, 55 },
    { 0, 0 },
    { 0xe5bdd77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x544bdd7a, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603bdd7f, 61 },
    { 0x12bbdd7f, 90 },
    { 0, 0 },
    { 0x24cbdd82, 33 },
    { 0x7ddbdd83, 98 },
    { 0xdaabdd83, 23 },
    { 0, 0 },
    { 0x5bbbdd86, 77 },
    { 0x76ebdd87, 98 },
    { 0x5b5bdd88, 90 },
    { 0x159bdd89, 81 },
    { 0x8c1bdd87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53bdd8d, 19 },
    { 0xf7bdd8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82bdd93, 23 },
    { 0, 0 },
    { 0xfaabdd95, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5bdd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x992bdd9c, 100 },
    { 0x24bbdd9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x441bdda0, 61 },
    { 0xcb5bdda0, 19 },
    { 0x3d6bdda2, 90 },
    { 0x16ebdda3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412bdda8, 99 },
    { 0x302bdda8, 65 },
    { 0, 0 },
    { 0x13fbddab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ebddaf, 76 },
    { 0x12abddb0, 87 },
    { 0x929bddb1, 22 },
    { 0xe91bddb2, 33 },
    { 0x4cbddaf, 98 },
    { 0xad2bddb4, 93 },
    { 0x10ebddb3, 95 },
    { 0xa16bddb6, 23 },
    { 0, 0 },
    { 0x28abddb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867bddbc, 93 },
    { 0x7fabddbc, 61 },
    { 0, 0 },
    { 0xce0bddbf, 33 },
    { 0xadcbddbf, 23 },
    { 0, 0 },
    { 0xa3bbddc2, 95 },
    { 0, 0 },
    { 0x36bbddc4, 76 },
    { 0xb2fbddc5, 76 },
    { 0, 0 },
    { 0x5bcbddc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2bddca, 85 },
    { 0xfbbbddcb, 95 },
    { 0xd2dbddcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0bddd0, 61 },
    { 0, 0 },
    { 0xf4bbddd2, 23 },
    { 0x6dfbddd3, 33 },
    { 0x679bddd2, 100 },
    { 0, 0 },
    { 0xdf7bddd6, 90 },
    { 0, 0 },
    { 0xfd3bddd8, 23 },
    { 0x14abddd9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12bdddc, 23 },
    { 0, 0 },
    { 0xc86bddde, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bdde1, 90 },
    { 0x35bdde2, 76 },
    { 0, 0 },
    { 0x6f7bdde4, 90 },
    { 0xc55bdde5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3bdde8, 93 },
    { 0xb5bdde9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186bdded, 95 },
    { 0x168bddee, 66 },
    { 0xb1cbddef, 23 },
    { 0, 0 },
    { 0x1a8bddf1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0bddf4, 95 },
    { 0xb87bddf5, 61 },
    { 0x5a3bddf6, 90 },
    { 0x64bddf5, 43 },
    { 0, 0 },
    { 0xcc5bddf9, 90 },
    { 0x1e2bddf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7bddfe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57bde05, 90 },
    { 0x39fbde06, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdbde0e, 95 },
    { 0xd43bde0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2bde12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bbde16, 46 },
    { 0xf91bde17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510bde1b, 33 },
    { 0x419bde1c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6bde1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327bde23, 95 },
    { 0xe9fbde24, 23 },
    { 0x2c6bde25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20bde2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5bde30, 34 },
    { 0, 0 },
    { 0xfefbde32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7bde35, 98 },
    { 0xb22bde36, 23 },
    { 0xd41bde37, 100 },
    { 0x6b3bde38, 52 },
    { 0, 0 },
    { 0xf31bde3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4bde40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72bde43, 75 },
    { 0x99abde44, 77 },
    { 0x7a0bde45, 23 },
    { 0x86bde45, 76 },
    { 0, 0 },
    { 0x35dbde48, 77 },
    { 0x9debde49, 61 },
    { 0x8ebde48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8bde4f, 76 },
    { 0, 0 },
    { 0xb7dbde51, 84 },
    { 0xdf0bde52, 33 },
    { 0, 0 },
    { 0x52ebde54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cbde59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36abde5c, 66 },
    { 0, 0 },
    { 0xe85bde5e, 33 },
    { 0x7cebde5f, 66 },
    { 0xb1cbde5e, 90 },
    { 0xc62bde60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbde64, 23 },
    { 0, 0 },
    { 0xf7abde66, 33 },
    { 0x412bde67, 81 },
    { 0xbabde68, 61 },
    { 0xcfbbde69, 33 },
    { 0x4a8bde69, 95 },
    { 0x962bde67, 98 },
    { 0xa6bde6c, 90 },
    { 0x707bde67, 98 },
    { 0x71ebde6b, 85 },
    { 0x669bde6f, 7 },
    { 0x591bde70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fbde78, 33 },
    { 0xca5bde79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35bde80, 98 },
    { 0xd25bde80, 93 },
    { 0x2cbbde80, 90 },
    { 0x16abde83, 33 },
    { 0xfcbde84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06bde89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bebde8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0bde95, 90 },
    { 0, 0 },
    { 0x195bde97, 33 },
    { 0x841bde98, 64 },
    { 0x213bde98, 99 },
    { 0x42bde99, 33 },
    { 0x2a7bde97, 95 },
    { 0xfc4bde99, 61 },
    { 0, 0 },
    { 0x179bde9e, 90 },
    { 0x8febde9f, 90 },
    { 0xca9bde9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47bdea4, 34 },
    { 0x830bdea4, 43 },
    { 0, 0 },
    { 0xfb8bdea7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bdeab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bbdeb0, 90 },
    { 0xc88bdeb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cbdeba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0bdebd, 76 },
    { 0xf63bdebd, 39 },
    { 0x43abdebf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5bdec2, 61 },
    { 0xc3ebdec2, 75 },
    { 0x333bdec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbbdec8, 23 },
    { 0x3c4bdec9, 76 },
    { 0x499bdec9, 85 },
    { 0, 0 },
    { 0x5cbbdecc, 77 },
    { 0x1d0bdecc, 61 },
    { 0, 0 },
    { 0x8c7bdecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17abded2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bbded5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cbded8, 33 },
    { 0x813bded9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21bdedd, 52 },
    { 0xf3bdede, 78 },
    { 0, 0 },
    { 0x679bdee0, 98 },
    { 0x84bbdee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2bdee7, 43 },
    { 0x7cdbdee7, 64 },
    { 0xedfbdee9, 55 },
    { 0xf36bdeea, 66 },
    { 0xbd7bdeeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445bdeef, 34 },
    { 0xa56bdef0, 23 },
    { 0xa76bdef1, 66 },
    { 0x8d9bdef1, 65 },
    { 0xd7cbdef3, 90 },
    { 0x1b6bdef2, 76 },
    { 0, 0 },
    { 0x2d3bdef6, 100 },
    { 0xf38bdef7, 98 },
    { 0xd56bdef8, 85 },
    { 0, 0 },
    { 0x93abdefa, 61 },
    { 0x668bdefb, 76 },
    { 0x134bdefc, 61 },
    { 0x81cbdefc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5bdf01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05bdf0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x466bdf0e, 98 },
    { 0xababdf0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393bdf14, 64 },
    { 0, 0 },
    { 0x85cbdf16, 90 },
    { 0, 0 },
    { 0x64bbdf18, 33 },
    { 0x41ebdf18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81abdf21, 33 },
    { 0x61bbdf21, 61 },
    { 0xdfcbdf23, 33 },
    { 0x290bdf21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bdf27, 98 },
    { 0x10abdf28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ebdf2f, 61 },
    { 0xec8bdf2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4debdf33, 23 },
    { 0x473bdf33, 81 },
    { 0x85bbdf35, 65 },
    { 0xec1bdf34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cebdf3a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332bdf3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ebdf41, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1bdf47, 33 },
    { 0xb01bdf48, 33 },
    { 0, 0 },
    { 0x457bdf4a, 98 },
    { 0, 0 },
    { 0x323bdf4c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x338bdf4f, 95 },
    { 0, 0 },
    { 0x9c5bdf51, 98 },
    { 0x4febdf52, 61 },
    { 0x8f2bdf53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3bdf58, 95 },
    { 0, 0 },
    { 0xf8cbdf5a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffbdf5e, 85 },
    { 0x884bdf5f, 88 },
    { 0x420bdf60, 76 },
    { 0, 0 },
    { 0x5b7bdf62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2bdf65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58bdf68, 76 },
    { 0xd20bdf69, 90 },
    { 0, 0 },
    { 0x9c7bdf6b, 46 },
    { 0xf16bdf6b, 98 },
    { 0x112bdf6d, 85 },
    { 0xb67bdf6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfbdf73, 43 },
    { 0x6bbdf74, 93 },
    { 0x170bdf75, 88 },
    { 0xdebdf74, 95 },
    { 0xde1bdf74, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2bdf7a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x305bdf7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5bdf84, 95 },
    { 0xe10bdf85, 23 },
    { 0xe6bdf84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426bdf90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36bdf93, 98 },
    { 0x818bdf94, 99 },
    { 0xf86bdf95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ebdf9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ebdf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bdfa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931bdfa9, 34 },
    { 0, 0 },
    { 0xb56bdfab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bbdfb2, 54 },
    { 0xefdbdfb3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23bdfb6, 66 },
    { 0xf20bdfb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88abdfba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x968bdfbd, 71 },
    { 0x82bdfbe, 98 },
    { 0xaf1bdfbf, 23 },
    { 0, 0 },
    { 0x73abdfc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285bdfcc, 43 },
    { 0xea6bdfcd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cbdfd6, 66 },
    { 0x272bdfd6, 76 },
    { 0x628bdfd8, 61 },
    { 0x34dbdfd9, 78 },
    { 0, 0 },
    { 0x3e0bdfdb, 34 },
    { 0xa79bdfdc, 55 },
    { 0x538bdfdc, 99 },
    { 0x4f0bdfde, 95 },
    { 0, 0 },
    { 0xea0bdfe0, 61 },
    { 0xdf0bdfe1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acbdfe4, 93 },
    { 0xdc9bdfe5, 95 },
    { 0, 0 },
    { 0x35dbdfe7, 61 },
    { 0xfebdfe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131bdfed, 90 },
    { 0x80dbdfed, 46 },
    { 0x547bdfef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbbdff3, 46 },
    { 0xe20bdff4, 61 },
    { 0x47bdff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22be000, 93 },
    { 0, 0 },
    { 0x3f3be002, 77 },
    { 0, 0 },
    { 0xa2abe004, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775be009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4be00d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacebe011, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cbe016, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22abe01a, 90 },
    { 0, 0 },
    { 0x622be01c, 90 },
    { 0x42fbe01c, 23 },
    { 0x2a3be01e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432be024, 33 },
    { 0x31abe025, 33 },
    { 0x475be026, 23 },
    { 0xf1cbe025, 55 },
    { 0xd54be024, 61 },
    { 0, 0 },
    { 0xc03be02a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57be030, 52 },
    { 0xb3be031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddbe035, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948be039, 43 },
    { 0x6b8be03a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebe043, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865be047, 34 },
    { 0x5c2be047, 55 },
    { 0, 0 },
    { 0x5f6be04a, 100 },
    { 0x781be04b, 23 },
    { 0, 0 },
    { 0xb0ebe04d, 33 },
    { 0x8be04e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dbe051, 93 },
    { 0, 0 },
    { 0x69be053, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabe05c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83be060, 76 },
    { 0x724be061, 34 },
    { 0, 0 },
    { 0x91bbe063, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afbe067, 100 },
    { 0, 0 },
    { 0x438be069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cbe06e, 95 },
    { 0xf91be06e, 46 },
    { 0, 0 },
    { 0x5b3be071, 91 },
    { 0xfa0be071, 95 },
    { 0x72cbe072, 83 },
    { 0x1fabe074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfbe077, 98 },
    { 0, 0 },
    { 0x410be079, 43 },
    { 0xb83be07a, 98 },
    { 0, 0 },
    { 0xc11be07c, 31 },
    { 0x4b9be07d, 95 },
    { 0, 0 },
    { 0x686be07f, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fbe084, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161be08a, 17 },
    { 0x500be08b, 61 },
    { 0x50be08b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdabe091, 77 },
    { 0x69ebe091, 33 },
    { 0, 0 },
    { 0x4f1be094, 33 },
    { 0xbc8be095, 23 },
    { 0xe18be094, 61 },
    { 0xd16be097, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dbe09d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bbe0a0, 23 },
    { 0x1bbe0a0, 34 },
    { 0xd42be0a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccbe0a6, 90 },
    { 0x92fbe0a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bbe0ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cbe0b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abe0bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8be0c0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98be0c4, 99 },
    { 0, 0 },
    { 0xa73be0c6, 90 },
    { 0xd2ebe0c6, 33 },
    { 0xe36be0c6, 33 },
    { 0x42cbe0c9, 66 },
    { 0xda9be0ca, 90 },
    { 0xebbbe0cb, 34 },
    { 0x225be0cc, 95 },
    { 0x7f9be0c9, 61 },
    { 0x869be0ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56be0d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22be0d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfbe0e4, 90 },
    { 0, 0 },
    { 0x4cebe0e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fbe0ea, 33 },
    { 0x7dabe0eb, 90 },
    { 0xb28be0ec, 61 },
    { 0xedebe0ed, 76 },
    { 0x533be0ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929be0f2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbbe0f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1be101, 33 },
    { 0xb77be102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54be105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74be108, 43 },
    { 0x4dabe109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x787be10c, 90 },
    { 0xb6ebe10c, 23 },
    { 0xdd8be10e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x263be111, 33 },
    { 0xc62be112, 90 },
    { 0, 0 },
    { 0xd93be114, 34 },
    { 0x89cbe114, 98 },
    { 0x196be114, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cbe11b, 61 },
    { 0xcfebe11c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbbe11f, 95 },
    { 0x58fbe11f, 76 },
    { 0x17dbe120, 17 },
    { 0xd3be122, 98 },
    { 0x7b4be123, 33 },
    { 0x7eabe124, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35be127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254be12b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61be12e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3be136, 98 },
    { 0x93abe136, 33 },
    { 0xfccbe138, 33 },
    { 0x12be136, 90 },
    { 0xa3cbe13a, 57 },
    { 0x5ddbe139, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ebe13f, 43 },
    { 0x1d5be140, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacbe145, 98 },
    { 0xdb8be146, 33 },
    { 0x1c5be147, 98 },
    { 0x8e1be148, 76 },
    { 0xd28be148, 61 },
    { 0x12fbe147, 23 },
    { 0x7aebe14b, 99 },
    { 0x3aebe14c, 34 },
    { 0x68ebe14c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488be151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8be154, 67 },
    { 0x6a9be155, 23 },
    { 0x6cebe156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbe15a, 33 },
    { 0xc42be15b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8be15f, 33 },
    { 0, 0 },
    { 0xa19be161, 98 },
    { 0, 0 },
    { 0x12bbe163, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccbe166, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x592be169, 53 },
    { 0, 0 },
    { 0xa50be16b, 93 },
    { 0, 0 },
    { 0x7f2be16d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887be173, 43 },
    { 0xfe8be173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dbe17c, 98 },
    { 0xa9dbe17d, 33 },
    { 0x57dbe17d, 75 },
    { 0xe11be17e, 90 },
    { 0x9fabe17e, 96 },
    { 0xb5cbe17f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x136be184, 43 },
    { 0x310be185, 34 },
    { 0x4cbbe186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x473be189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ebe190, 90 },
    { 0x252be191, 91 },
    { 0xab8be192, 65 },
    { 0xc36be193, 61 },
    { 0x6a4be192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36be197, 76 },
    { 0xf8ebe198, 91 },
    { 0, 0 },
    { 0xe4bbe19a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ebe19d, 90 },
    { 0x542be19d, 53 },
    { 0xf8bbe19f, 23 },
    { 0x19abe1a0, 23 },
    { 0x1ffbe1a1, 98 },
    { 0x199be1a1, 61 },
    { 0x811be1a1, 33 },
    { 0xce8be1a4, 82 },
    { 0x295be1a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2be1a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ebe1ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169be1bc, 52 },
    { 0x66bbe1bd, 88 },
    { 0xb5fbe1bd, 23 },
    { 0x576be1bd, 43 },
    { 0xa4be1bf, 90 },
    { 0xb1be1c1, 98 },
    { 0xe18be1c2, 93 },
    { 0xe44be1c3, 66 },
    { 0x552be1bc, 34 },
    { 0x5cdbe1bc, 95 },
    { 0xc06be1c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86be1cb, 93 },
    { 0x677be1cb, 95 },
    { 0x158be1cd, 40 },
    { 0, 0 },
    { 0x876be1cf, 98 },
    { 0x99ebe1d0, 43 },
    { 0x5cebe1cf, 84 },
    { 0x204be1d2, 17 },
    { 0x219be1d3, 90 },
    { 0x694be1d3, 61 },
    { 0xdcbbe1d4, 87 },
    { 0x3debe1d6, 76 },
    { 0xa45be1d4, 34 },
    { 0x7c9be1d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145be1dc, 90 },
    { 0x867be1dc, 46 },
    { 0x236be1dc, 61 },
    { 0x602be1df, 61 },
    { 0xd77be1dc, 23 },
    { 0x3a3be1e1, 23 },
    { 0xaddbe1e2, 55 },
    { 0xed8be1e3, 90 },
    { 0xc58be1e4, 61 },
    { 0, 0 },
    { 0xe7bbe1e6, 33 },
    { 0xd05be1e7, 93 },
    { 0x792be1e8, 90 },
    { 0x65cbe1e9, 80 },
    { 0xc81be1e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35be1f3, 33 },
    { 0, 0 },
    { 0x33fbe1f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84be1fd, 33 },
    { 0xd96be1fe, 23 },
    { 0x165be1fd, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8be202, 23 },
    { 0xacbe203, 33 },
    { 0x458be204, 33 },
    { 0xd84be205, 46 },
    { 0xf0fbe203, 26 },
    { 0, 0 },
    { 0x8e6be208, 98 },
    { 0x930be209, 43 },
    { 0, 0 },
    { 0x573be20b, 98 },
    { 0, 0 },
    { 0x369be20d, 90 },
    { 0xa0fbe20e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187be212, 90 },
    { 0x692be213, 34 },
    { 0x49fbe212, 99 },
    { 0x38cbe215, 33 },
    { 0x964be216, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dbe21a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548be225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x32abe228, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18be22b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cbe22e, 90 },
    { 0xf0abe22e, 61 },
    { 0xa3cbe230, 100 },
    { 0x136be231, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75be235, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefbe238, 78 },
    { 0xeb0be238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fbe23c, 98 },
    { 0xe2abe23d, 61 },
    { 0x39ebe23c, 33 },
    { 0x51be23f, 95 },
    { 0x122be240, 34 },
    { 0xc32be241, 55 },
    { 0x85bbe242, 90 },
    { 0x49dbe243, 33 },
    { 0, 0 },
    { 0x83fbe245, 98 },
    { 0xb3bbe246, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddbe249, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88be24c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02be252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0be258, 76 },
    { 0x7aabe258, 91 },
    { 0, 0 },
    { 0x8cbbe25b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bbe260, 52 },
    { 0, 0 },
    { 0x164be262, 93 },
    { 0x54bbe262, 49 },
    { 0, 0 },
    { 0x857be265, 90 },
    { 0, 0 },
    { 0xd36be267, 98 },
    { 0, 0 },
    { 0x65abe269, 67 },
    { 0x389be26a, 93 },
    { 0x687be26b, 33 },
    { 0x431be269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597be271, 55 },
    { 0xf0abe272, 61 },
    { 0, 0 },
    { 0xd59be274, 100 },
    { 0, 0 },
    { 0xe6abe276, 23 },
    { 0, 0 },
    { 0xa46be278, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcbe27b, 33 },
    { 0, 0 },
    { 0xbb6be27d, 88 },
    { 0, 0 },
    { 0xbc0be27f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dbe283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4be288, 91 },
    { 0xac2be288, 90 },
    { 0x7fbbe28a, 90 },
    { 0, 0 },
    { 0xdf7be28c, 90 },
    { 0xe4fbe28d, 64 },
    { 0x94ebe28e, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fbe294, 34 },
    { 0x7fabe295, 76 },
    { 0xfacbe295, 61 },
    { 0, 0 },
    { 0x22dbe298, 56 },
    { 0, 0 },
    { 0xbf9be29a, 93 },
    { 0x74fbe29b, 55 },
    { 0, 0 },
    { 0x497be29d, 33 },
    { 0x397be29e, 33 },
    { 0, 0 },
    { 0x510be2a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be2a4, 34 },
    { 0, 0 },
    { 0xbc2be2a6, 50 },
    { 0x50dbe2a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755be2ab, 52 },
    { 0, 0 },
    { 0x89dbe2ad, 65 },
    { 0x2e3be2ae, 95 },
    { 0xf79be2ae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3be2b2, 61 },
    { 0x7febe2b2, 61 },
    { 0x32dbe2b4, 92 },
    { 0x7f3be2b2, 34 },
    { 0x2e5be2b3, 95 },
    { 0x43bbe2b2, 90 },
    { 0xcb4be2b8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7be2bb, 61 },
    { 0xd09be2bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0abe2c0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bbe2c4, 90 },
    { 0, 0 },
    { 0x7c3be2c6, 87 },
    { 0x164be2c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412be2cc, 23 },
    { 0xbc7be2cc, 55 },
    { 0xfebbe2ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636be2d4, 91 },
    { 0xcf4be2d5, 33 },
    { 0x65cbe2d6, 61 },
    { 0x34ebe2d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6abe2db, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x741be2de, 33 },
    { 0, 0 },
    { 0x3adbe2e0, 90 },
    { 0xe8ebe2e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dbe2e5, 93 },
    { 0x645be2e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eabe2e9, 34 },
    { 0xdebe2ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdbe2f0, 66 },
    { 0x25be2f1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832be2f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90be2f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccbe2fc, 93 },
    { 0, 0 },
    { 0xb66be2fe, 33 },
    { 0, 0 },
    { 0xb78be300, 34 },
    { 0, 0 },
    { 0x348be302, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688be30e, 34 },
    { 0x367be30f, 90 },
    { 0xe40be30e, 33 },
    { 0x1b9be311, 33 },
    { 0xc45be312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6be316, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35be319, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dbe31e, 61 },
    { 0, 0 },
    { 0x177be320, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x442be323, 33 },
    { 0x767be324, 95 },
    { 0, 0 },
    { 0x274be326, 101 },
    { 0, 0 },
    { 0x95be328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943be330, 24 },
    { 0xb2cbe331, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8be334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fbe33c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95abe342, 34 },
    { 0, 0 },
    { 0xa28be344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103be34e, 91 },
    { 0xe12be34f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29be355, 23 },
    { 0x562be355, 95 },
    { 0x535be355, 33 },
    { 0, 0 },
    { 0xb5dbe359, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4be35d, 23 },
    { 0, 0 },
    { 0xa25be35f, 93 },
    { 0xa9be360, 66 },
    { 0xb91be35f, 61 },
    { 0xcbbbe360, 65 },
    { 0, 0 },
    { 0xa29be364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abe368, 90 },
    { 0x61dbe369, 23 },
    { 0x38ebe36a, 90 },
    { 0x66bbe36b, 90 },
    { 0x92ebe368, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62be370, 23 },
    { 0, 0 },
    { 0xdf8be372, 43 },
    { 0xf79be373, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571be378, 95 },
    { 0x16ebe378, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8be37d, 95 },
    { 0, 0 },
    { 0x290be37f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740be386, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7be38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cbe398, 66 },
    { 0x783be398, 61 },
    { 0x760be39a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97be39e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90be3a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94be3ac, 78 },
    { 0x7c0be3ac, 23 },
    { 0, 0 },
    { 0x904be3af, 87 },
    { 0, 0 },
    { 0xd57be3b1, 61 },
    { 0xaadbe3b2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cbe3b5, 23 },
    { 0, 0 },
    { 0x5cebe3b7, 76 },
    { 0x8c5be3b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abe3bc, 47 },
    { 0x9dfbe3bd, 91 },
    { 0xf0abe3be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dbe3c3, 98 },
    { 0x754be3c4, 98 },
    { 0x9ecbe3c5, 84 },
    { 0x8c4be3c3, 33 },
    { 0xa8abe3c7, 55 },
    { 0x5eebe3c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe3cb, 98 },
    { 0xec8be3cc, 43 },
    { 0xaf5be3cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaabe3d0, 76 },
    { 0xe35be3d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x969be3d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3be3d9, 24 },
    { 0x12bbe3da, 95 },
    { 0, 0 },
    { 0xc48be3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddbe3df, 33 },
    { 0, 0 },
    { 0x7e9be3e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8be3e9, 98 },
    { 0xc16be3ea, 34 },
    { 0x238be3eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acbe3ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9be3f1, 67 },
    { 0xa9be3f1, 66 },
    { 0, 0 },
    { 0x6bfbe3f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ebe3fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146be401, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0be404, 90 },
    { 0xa3bbe405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3be410, 90 },
    { 0xa71be411, 17 },
    { 0xb7be412, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ebe416, 23 },
    { 0x76cbe416, 93 },
    { 0, 0 },
    { 0xba5be419, 23 },
    { 0, 0 },
    { 0xa2bbe41b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fbe41e, 66 },
    { 0, 0 },
    { 0xd1ebe420, 61 },
    { 0x289be421, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x764be424, 55 },
    { 0, 0 },
    { 0xccbbe426, 81 },
    { 0, 0 },
    { 0xa6bbe428, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5be42d, 23 },
    { 0x7b5be42e, 43 },
    { 0xd0fbe42f, 85 },
    { 0x900be430, 34 },
    { 0x273be431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2be435, 80 },
    { 0x79abe436, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87be43a, 84 },
    { 0xf10be43b, 95 },
    { 0xbe9be43a, 34 },
    { 0x9abe43b, 23 },
    { 0, 0 },
    { 0xe4bbe43f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcabe443, 90 },
    { 0xd61be443, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16abe44c, 87 },
    { 0, 0 },
    { 0x7babe44e, 90 },
    { 0xa92be44f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddbe455, 76 },
    { 0xd18be456, 76 },
    { 0x79abe456, 52 },
    { 0xa5ebe458, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81be45c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6dbe462, 86 },
    { 0x70dbe463, 23 },
    { 0xf3abe464, 90 },
    { 0x5b1be465, 33 },
    { 0, 0 },
    { 0x7abe467, 4 },
    { 0x5c6be468, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159be46c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65be470, 61 },
    { 0, 0 },
    { 0x588be472, 80 },
    { 0x2e6be473, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13be47d, 98 },
    { 0xd86be47e, 76 },
    { 0, 0 },
    { 0xdd8be480, 95 },
    { 0x7dabe481, 90 },
    { 0, 0 },
    { 0x23ebe483, 34 },
    { 0xaf1be483, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9be489, 47 },
    { 0x8ffbe489, 33 },
    { 0x76be48a, 95 },
    { 0, 0 },
    { 0x3c8be48d, 55 },
    { 0, 0 },
    { 0xb11be48f, 96 },
    { 0x5a8be490, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fbe499, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcbe49c, 34 },
    { 0x91dbe49d, 52 },
    { 0xd51be49e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x702be4a1, 33 },
    { 0, 0 },
    { 0xb57be4a3, 95 },
    { 0xe5ebe4a3, 66 },
    { 0, 0 },
    { 0x3d2be4a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bebe4aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282be4ae, 76 },
    { 0x5debe4ae, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabe4b7, 43 },
    { 0xdf7be4b7, 17 },
    { 0, 0 },
    { 0xd0bbe4ba, 33 },
    { 0, 0 },
    { 0x905be4bc, 95 },
    { 0x721be4bc, 55 },
    { 0xe27be4be, 61 },
    { 0xb01be4bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x67abe4c2, 33 },
    { 0x369be4c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fbe4c6, 96 },
    { 0xcb3be4c7, 84 },
    { 0x2be4c8, 99 },
    { 0x3b2be4c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dbe4cc, 55 },
    { 0x769be4cc, 76 },
    { 0xc99be4cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47be4d3, 98 },
    { 0x697be4d3, 23 },
    { 0x10dbe4d5, 46 },
    { 0, 0 },
    { 0x35ebe4d7, 46 },
    { 0x74abe4d7, 93 },
    { 0, 0 },
    { 0xb17be4da, 34 },
    { 0xcd3be4db, 99 },
    { 0x8afbe4dc, 93 },
    { 0x3f4be4dc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48be4e7, 95 },
    { 0x6b9be4e8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4be4eb, 61 },
    { 0xc1dbe4ec, 33 },
    { 0xf21be4ec, 95 },
    { 0xb96be4ed, 95 },
    { 0x25cbe4ee, 98 },
    { 0xb13be4f0, 61 },
    { 0x6dbe4ef, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cebe4f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97be4fa, 83 },
    { 0x7b7be4fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcebe500, 78 },
    { 0xd45be500, 61 },
    { 0xbdbe500, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7be505, 61 },
    { 0x83abe506, 76 },
    { 0x5f2be505, 91 },
    { 0, 0 },
    { 0x200be509, 33 },
    { 0x687be50a, 98 },
    { 0x6d5be50b, 55 },
    { 0x576be50c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23be50f, 33 },
    { 0x866be50f, 33 },
    { 0, 0 },
    { 0x7a9be512, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eebe518, 34 },
    { 0xc4dbe519, 43 },
    { 0, 0 },
    { 0xde3be51b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696be521, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6be526, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35be529, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356be531, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46be535, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbbe538, 77 },
    { 0xe09be539, 90 },
    { 0x8c5be538, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369be53f, 68 },
    { 0x764be540, 90 },
    { 0, 0 },
    { 0x9abbe542, 33 },
    { 0x286be543, 90 },
    { 0xf05be544, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1be547, 23 },
    { 0x340be548, 81 },
    { 0xbbdbe548, 43 },
    { 0xa9ebe547, 61 },
    { 0x71ebe548, 77 },
    { 0x516be54c, 95 },
    { 0xfebbe549, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cbe552, 90 },
    { 0, 0 },
    { 0x607be554, 77 },
    { 0x70fbe555, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe558, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x837be55b, 90 },
    { 0x9a9be55b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4be561, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bbe565, 93 },
    { 0x40dbe566, 61 },
    { 0, 0 },
    { 0x980be568, 76 },
    { 0x14bbe569, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395be574, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180be57d, 33 },
    { 0x635be57e, 86 },
    { 0xc7cbe57e, 95 },
    { 0, 0 },
    { 0x6d4be581, 33 },
    { 0x4ccbe581, 88 },
    { 0, 0 },
    { 0x787be584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1be589, 95 },
    { 0x104be589, 33 },
    { 0x768be589, 76 },
    { 0xd7bbe58c, 100 },
    { 0x90bbe58d, 98 },
    { 0x657be58c, 33 },
    { 0, 0 },
    { 0xb84be590, 23 },
    { 0x740be590, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x569be594, 46 },
    { 0xc42be594, 61 },
    { 0x163be594, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2be59e, 52 },
    { 0x7e2be59e, 95 },
    { 0xedfbe59e, 100 },
    { 0, 0 },
    { 0xb3bbe5a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05be5a6, 23 },
    { 0, 0 },
    { 0x11cbe5a8, 23 },
    { 0x662be5a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebbe5ac, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddebe5b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9be5bb, 66 },
    { 0xea1be5bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811be5c2, 33 },
    { 0, 0 },
    { 0x61ebe5c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1be5c9, 98 },
    { 0x357be5ca, 61 },
    { 0x5e2be5cb, 76 },
    { 0, 0 },
    { 0x1a2be5cd, 61 },
    { 0xa40be5ce, 23 },
    { 0, 0 },
    { 0x678be5d0, 23 },
    { 0x795be5d0, 88 },
    { 0xdfbe5d2, 61 },
    { 0x3b7be5d0, 90 },
    { 0, 0 },
    { 0x458be5d5, 46 },
    { 0x7d8be5d6, 98 },
    { 0x31bbe5d5, 100 },
    { 0x70fbe5d8, 67 },
    { 0xc53be5d8, 90 },
    { 0xf0dbe5da, 84 },
    { 0x4b5be5da, 93 },
    { 0x85fbe5da, 33 },
    { 0x6f2be5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214be5e3, 33 },
    { 0xf69be5e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x75abe5e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedbe5ea, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adbe5ed, 23 },
    { 0x9d2be5ee, 33 },
    { 0, 0 },
    { 0xc2bbe5f0, 61 },
    { 0x40abe5f1, 81 },
    { 0xffbbe5f2, 4 },
    { 0x722be5f3, 90 },
    { 0xcb1be5f2, 61 },
    { 0xa62be5f1, 61 },
    { 0, 0 },
    { 0xd5abe5f7, 88 },
    { 0x329be5f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcabe5fb, 90 },
    { 0, 0 },
    { 0xc2fbe5fd, 98 },
    { 0xe45be5fd, 59 },
    { 0, 0 },
    { 0xeb7be600, 43 },
    { 0x964be601, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabbe609, 90 },
    { 0xa1bbe60a, 95 },
    { 0x493be60b, 76 },
    { 0xfc4be60a, 96 },
    { 0x750be60b, 33 },
    { 0xee8be60e, 34 },
    { 0, 0 },
    { 0x2ebe610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbe613, 81 },
    { 0, 0 },
    { 0x24ebe615, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598be619, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63be621, 95 },
    { 0, 0 },
    { 0x899be623, 78 },
    { 0, 0 },
    { 0xa10be625, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be629, 87 },
    { 0, 0 },
    { 0x1d4be62b, 23 },
    { 0x527be62c, 98 },
    { 0x950be62d, 55 },
    { 0xf4be62e, 67 },
    { 0x880be62f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbbe632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80abe63b, 90 },
    { 0x838be63c, 33 },
    { 0x686be63d, 23 },
    { 0xac7be63e, 76 },
    { 0xed2be63f, 76 },
    { 0xa88be63f, 23 },
    { 0xb81be641, 23 },
    { 0xd34be642, 90 },
    { 0, 0 },
    { 0xdcfbe644, 23 },
    { 0x1fabe645, 33 },
    { 0, 0 },
    { 0x757be647, 46 },
    { 0x76cbe648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaebe656, 95 },
    { 0x920be657, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84be65e, 93 },
    { 0xcc0be65e, 36 },
    { 0, 0 },
    { 0x402be661, 84 },
    { 0, 0 },
    { 0x42cbe663, 55 },
    { 0, 0 },
    { 0x75ebe665, 95 },
    { 0, 0 },
    { 0x4fabe667, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cbe66f, 33 },
    { 0, 0 },
    { 0xae3be671, 33 },
    { 0x18fbe672, 23 },
    { 0x1f0be673, 91 },
    { 0, 0 },
    { 0x9c8be675, 75 },
    { 0x1d8be676, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bbe67a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x926be67d, 93 },
    { 0, 0 },
    { 0x970be67f, 43 },
    { 0, 0 },
    { 0x8adbe681, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36be684, 43 },
    { 0xb32be684, 91 },
    { 0, 0 },
    { 0xa37be687, 100 },
    { 0x646be688, 43 },
    { 0, 0 },
    { 0xe82be68a, 90 },
    { 0x11be68b, 61 },
    { 0x353be68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x822be68f, 61 },
    { 0, 0 },
    { 0xee2be691, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7be698, 52 },
    { 0x5f3be699, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6abe69c, 95 },
    { 0x989be69d, 61 },
    { 0xbccbe69e, 78 },
    { 0xba3be69f, 76 },
    { 0x649be6a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2be6a4, 81 },
    { 0xa21be6a5, 90 },
    { 0x7e7be6a6, 90 },
    { 0xfb2be6a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x531be6aa, 23 },
    { 0xfa1be6aa, 61 },
    { 0x5cdbe6aa, 52 },
    { 0x695be6aa, 99 },
    { 0xfefbe6ad, 78 },
    { 0xa46be6ae, 78 },
    { 0x703be6aa, 33 },
    { 0xe29be6ab, 23 },
    { 0xfc1be6ae, 55 },
    { 0x37be6ab, 90 },
    { 0, 0 },
    { 0xc67be6b5, 98 },
    { 0x1abe6b5, 61 },
    { 0xd35be6b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36be6be, 46 },
    { 0xc36be6bf, 56 },
    { 0xbebbe6bf, 76 },
    { 0x91bbe6c1, 91 },
    { 0xc86be6c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be6c6, 87 },
    { 0x745be6c7, 34 },
    { 0x7abe6c8, 61 },
    { 0xe8dbe6c9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48abe6cd, 33 },
    { 0, 0 },
    { 0xfacbe6cf, 33 },
    { 0, 0 },
    { 0xcc0be6d1, 95 },
    { 0, 0 },
    { 0xe85be6d3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823be6d8, 65 },
    { 0x8c9be6d9, 99 },
    { 0xc61be6da, 61 },
    { 0xa37be6db, 84 },
    { 0x93abe6db, 84 },
    { 0, 0 },
    { 0x28fbe6de, 66 },
    { 0x697be6df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787be6e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31abe6eb, 33 },
    { 0xd63be6ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25be6ef, 90 },
    { 0x320be6f0, 95 },
    { 0xe87be6f1, 66 },
    { 0x604be6f1, 95 },
    { 0xea6be6f3, 33 },
    { 0xaa0be6f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x697be6f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bbe6fa, 23 },
    { 0x958be6fb, 90 },
    { 0xc26be6fa, 90 },
    { 0, 0 },
    { 0xb5cbe6fe, 20 },
    { 0x72cbe6ff, 64 },
    { 0x7c0be700, 95 },
    { 0x566be701, 90 },
    { 0xc05be701, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03be708, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30abe712, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbe715, 90 },
    { 0, 0 },
    { 0x9e7be717, 61 },
    { 0x677be718, 33 },
    { 0xffebe719, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9be71c, 95 },
    { 0xdafbe71c, 34 },
    { 0, 0 },
    { 0xe49be71f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdbe723, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aabe728, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecbe72d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x872be730, 98 },
    { 0x2dbbe730, 90 },
    { 0, 0 },
    { 0x2cbe733, 44 },
    { 0, 0 },
    { 0x458be735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0be738, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fbe74a, 90 },
    { 0xa2fbe74a, 66 },
    { 0x96abe74a, 40 },
    { 0xb1abe74b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96be751, 84 },
    { 0x9bcbe752, 78 },
    { 0xf1ebe753, 84 },
    { 0x6a9be754, 90 },
    { 0xfa8be753, 33 },
    { 0xd9dbe756, 46 },
    { 0, 0 },
    { 0x742be758, 34 },
    { 0x4f9be759, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5be761, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12be768, 36 },
    { 0xef4be768, 34 },
    { 0x42ebe768, 93 },
    { 0x6aebe76b, 33 },
    { 0xff2be76c, 98 },
    { 0xaafbe76d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635be773, 78 },
    { 0x50abe774, 77 },
    { 0x6abbe773, 90 },
    { 0xd6fbe775, 33 },
    { 0x7b8be775, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4be77d, 90 },
    { 0x328be77e, 23 },
    { 0x25bbe77f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fbe786, 97 },
    { 0xdcebe786, 98 },
    { 0x137be788, 90 },
    { 0x668be789, 90 },
    { 0xfc0be78a, 76 },
    { 0x790be787, 93 },
    { 0xf01be78c, 34 },
    { 0xb16be787, 95 },
    { 0xc51be78a, 66 },
    { 0, 0 },
    { 0xdcdbe790, 90 },
    { 0x851be790, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73abe794, 90 },
    { 0xdf8be795, 98 },
    { 0x7d5be796, 34 },
    { 0x9cabe797, 46 },
    { 0, 0 },
    { 0xbc9be799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26be79e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdbe7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359be7ac, 98 },
    { 0, 0 },
    { 0x306be7ae, 52 },
    { 0x529be7af, 23 },
    { 0xf8abe7b0, 95 },
    { 0x9d5be7b0, 84 },
    { 0xe84be7af, 95 },
    { 0xe98be7af, 93 },
    { 0x724be7b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5be7b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacabe7bf, 95 },
    { 0, 0 },
    { 0x283be7c1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5be7c4, 78 },
    { 0, 0 },
    { 0xfa0be7c6, 61 },
    { 0x8c8be7c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbebe7ca, 77 },
    { 0, 0 },
    { 0x413be7cc, 34 },
    { 0x331be7cd, 98 },
    { 0x435be7cd, 78 },
    { 0x93fbe7cf, 66 },
    { 0x3be7cc, 76 },
    { 0x274be7cf, 90 },
    { 0x286be7d2, 95 },
    { 0x56dbe7d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x250be7d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774be7df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38be7e2, 23 },
    { 0xbc9be7e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115be7ea, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73be7ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3be7f2, 33 },
    { 0x32dbe7f3, 43 },
    { 0xd47be7f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602be7f8, 93 },
    { 0, 0 },
    { 0x418be7fa, 46 },
    { 0x600be7fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafbe806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dbe809, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x895be80c, 84 },
    { 0, 0 },
    { 0x5debe80e, 98 },
    { 0xf16be80f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x991be812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8be816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7be81b, 87 },
    { 0xdcbbe81c, 95 },
    { 0x4ccbe81b, 17 },
    { 0x105be81e, 90 },
    { 0xd25be81e, 46 },
    { 0x3cfbe820, 55 },
    { 0, 0 },
    { 0x496be822, 61 },
    { 0, 0 },
    { 0xdc5be824, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889be828, 95 },
    { 0xc23be829, 61 },
    { 0x455be82a, 84 },
    { 0x9eebe829, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fbe831, 91 },
    { 0xc4be831, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9be83b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dbe842, 98 },
    { 0, 0 },
    { 0xe09be844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddbe84a, 98 },
    { 0, 0 },
    { 0x517be84c, 76 },
    { 0xb5cbe84c, 99 },
    { 0x1f0be84e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40be851, 61 },
    { 0x69bbe851, 66 },
    { 0x2dabe853, 95 },
    { 0xb3cbe854, 95 },
    { 0x30bbe854, 95 },
    { 0x21ebe856, 43 },
    { 0, 0 },
    { 0xe48be858, 90 },
    { 0x6bebe859, 33 },
    { 0xda6be85a, 76 },
    { 0x403be85b, 91 },
    { 0x6c5be85a, 84 },
    { 0, 0 },
    { 0x5cfbe85e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72be867, 67 },
    { 0x28abe868, 77 },
    { 0xf9bbe869, 77 },
    { 0, 0 },
    { 0xad5be86b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80be871, 95 },
    { 0xc8be872, 61 },
    { 0x59be872, 61 },
    { 0x688be872, 34 },
    { 0x1dbbe875, 34 },
    { 0xb90be875, 90 },
    { 0xa1bbe877, 95 },
    { 0x597be876, 61 },
    { 0, 0 },
    { 0xcefbe87a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91be87e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ebe883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645be888, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5be88b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be88f, 61 },
    { 0x4c2be890, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdbe895, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551be89a, 43 },
    { 0xb1fbe89b, 61 },
    { 0xabbe89c, 98 },
    { 0xf96be89d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648be8a8, 75 },
    { 0xf85be8a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2be8b0, 66 },
    { 0x4c6be8b1, 55 },
    { 0x5f7be8b2, 23 },
    { 0, 0 },
    { 0xae4be8b4, 93 },
    { 0, 0 },
    { 0x87fbe8b6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafbe8bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bbe8c5, 42 },
    { 0x5be8c6, 96 },
    { 0x80be8c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904be8cb, 90 },
    { 0xf42be8cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be8cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cbe8d2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813be8d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6be8dd, 99 },
    { 0, 0 },
    { 0xdb1be8df, 98 },
    { 0xec4be8df, 87 },
    { 0, 0 },
    { 0xc84be8e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866be8e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x596be8ea, 90 },
    { 0x170be8ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5be8f2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183be8fc, 33 },
    { 0x33fbe8fd, 33 },
    { 0, 0 },
    { 0xa6ebe8ff, 95 },
    { 0xbf0be900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dbe905, 61 },
    { 0x744be905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63be909, 93 },
    { 0, 0 },
    { 0xdaabe90b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0be910, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbbe914, 89 },
    { 0x3b4be915, 99 },
    { 0x178be915, 33 },
    { 0x313be917, 85 },
    { 0xc6bbe918, 76 },
    { 0x34be919, 84 },
    { 0x153be916, 95 },
    { 0x533be916, 98 },
    { 0x4be91a, 100 },
    { 0xa16be91d, 23 },
    { 0xa01be914, 23 },
    { 0xb6dbe91b, 98 },
    { 0, 0 },
    { 0xebabe921, 43 },
    { 0, 0 },
    { 0xabe923, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639be927, 98 },
    { 0xf54be927, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x557be92b, 98 },
    { 0x321be92b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbe92f, 46 },
    { 0x2f7be92f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0be935, 88 },
    { 0xccebe936, 76 },
    { 0x926be935, 61 },
    { 0x66dbe935, 98 },
    { 0x8a7be935, 90 },
    { 0x2adbe935, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ebe93d, 95 },
    { 0x6dcbe93e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x977be941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccbe945, 55 },
    { 0, 0 },
    { 0xb35be947, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67be94a, 84 },
    { 0x8e8be94b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x927be94e, 85 },
    { 0, 0 },
    { 0xa3cbe950, 52 },
    { 0, 0 },
    { 0xbbbe952, 98 },
    { 0x7fbe952, 95 },
    { 0xac2be952, 100 },
    { 0xd1cbe953, 46 },
    { 0x856be956, 84 },
    { 0, 0 },
    { 0x8fdbe958, 67 },
    { 0xb46be959, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96be95d, 61 },
    { 0, 0 },
    { 0x193be95f, 99 },
    { 0, 0 },
    { 0x337be961, 88 },
    { 0, 0 },
    { 0x115be963, 95 },
    { 0, 0 },
    { 0xb16be965, 66 },
    { 0x50ebe966, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1be96f, 77 },
    { 0x6a0be970, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edbe973, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ebe976, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbe97b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cbe97f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91abe986, 66 },
    { 0x7d0be987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4be98b, 61 },
    { 0, 0 },
    { 0xf25be98d, 98 },
    { 0xec9be98d, 85 },
    { 0x8b8be98f, 67 },
    { 0x33ebe98f, 33 },
    { 0x450be991, 23 },
    { 0xfa7be992, 88 },
    { 0xf36be991, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453be997, 33 },
    { 0x9e9be997, 40 },
    { 0x813be999, 34 },
    { 0, 0 },
    { 0x999be99b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06be99f, 23 },
    { 0x5c2be9a0, 55 },
    { 0x187be99f, 90 },
    { 0x38dbe9a2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efbe9a5, 61 },
    { 0x541be9a5, 34 },
    { 0xb34be9a5, 33 },
    { 0x8cbbe9a8, 23 },
    { 0x8cabe9a9, 68 },
    { 0x5d8be9a8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecbe9ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12be9bd, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70be9c3, 98 },
    { 0xf3ebe9c3, 55 },
    { 0, 0 },
    { 0xe51be9c6, 85 },
    { 0, 0 },
    { 0xffcbe9c8, 33 },
    { 0x69abe9c9, 23 },
    { 0, 0 },
    { 0x437be9cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67abe9cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890be9d5, 81 },
    { 0, 0 },
    { 0xcfabe9d7, 90 },
    { 0, 0 },
    { 0xc78be9d9, 90 },
    { 0, 0 },
    { 0x1fbe9db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ebe9df, 61 },
    { 0, 0 },
    { 0xf87be9e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946be9e5, 61 },
    { 0x43dbe9e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdbe9ee, 88 },
    { 0, 0 },
    { 0x117be9f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bbe9f5, 76 },
    { 0, 0 },
    { 0x429be9f7, 87 },
    { 0xe5abe9f8, 33 },
    { 0xceebe9f7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ebe9fd, 33 },
    { 0x34fbe9fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621bea02, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469bea07, 95 },
    { 0, 0 },
    { 0xcbdbea09, 95 },
    { 0, 0 },
    { 0xbc8bea0b, 46 },
    { 0x56ebea0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67bea11, 52 },
    { 0x66bbea12, 61 },
    { 0x9dbbea13, 33 },
    { 0xdb6bea13, 61 },
    { 0x6e5bea12, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeabea19, 95 },
    { 0, 0 },
    { 0xc4cbea1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3bea1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcbea23, 52 },
    { 0, 0 },
    { 0xb41bea25, 90 },
    { 0, 0 },
    { 0x21dbea27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192bea33, 61 },
    { 0xa98bea33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcbea3a, 61 },
    { 0x13dbea3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57bea41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fbea47, 95 },
    { 0xc12bea47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6bea4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93bea50, 78 },
    { 0, 0 },
    { 0x53ebea52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dabea55, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0bea5c, 61 },
    { 0x64cbea5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dbea60, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740bea66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755bea6b, 66 },
    { 0x294bea6b, 61 },
    { 0xf39bea6d, 98 },
    { 0x689bea6c, 33 },
    { 0xfbabea6c, 85 },
    { 0x42bea6e, 33 },
    { 0x265bea71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7bea76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bbea7c, 95 },
    { 0, 0 },
    { 0x1edbea7e, 69 },
    { 0xbe1bea7f, 93 },
    { 0x7e1bea80, 93 },
    { 0x759bea7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fbea85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbea93, 85 },
    { 0xa3dbea93, 90 },
    { 0x554bea95, 87 },
    { 0, 0 },
    { 0x35dbea97, 95 },
    { 0x69dbea98, 90 },
    { 0x4a1bea98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdbea9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6beaa0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbeaa4, 23 },
    { 0xa8ebeaa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dabeaa8, 40 },
    { 0x5dbbeaa9, 61 },
    { 0xd51beaa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dbeaae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2beaba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fbeabd, 43 },
    { 0x997beabe, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8beac1, 95 },
    { 0x8fbeac1, 36 },
    { 0xf6cbeac3, 67 },
    { 0x10cbeac4, 67 },
    { 0xcf5beac4, 99 },
    { 0, 0 },
    { 0xdd2beac7, 61 },
    { 0xafbbeac7, 68 },
    { 0x8f3beac7, 61 },
    { 0xf15beaca, 98 },
    { 0x49dbeaca, 65 },
    { 0x2beacc, 33 },
    { 0xfc0beaca, 55 },
    { 0x48fbeacd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896bead3, 93 },
    { 0x8dbbead3, 90 },
    { 0xa5ebead5, 61 },
    { 0, 0 },
    { 0xfcebead7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x340beada, 46 },
    { 0xfb8beada, 46 },
    { 0xc0cbeada, 91 },
    { 0x128beadd, 77 },
    { 0xf6abeadd, 23 },
    { 0xe57beadf, 33 },
    { 0x22cbeadf, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509beae5, 61 },
    { 0xb76beae6, 76 },
    { 0xdc3beae7, 34 },
    { 0xb4ebeae7, 90 },
    { 0, 0 },
    { 0xc15beaea, 23 },
    { 0x144beaeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x110beaee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dbeaf1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57beafa, 91 },
    { 0, 0 },
    { 0x998beafc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847beb01, 61 },
    { 0xe13beb01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171beb0a, 93 },
    { 0xe68beb0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944beb13, 98 },
    { 0xe0dbeb13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecbeb18, 95 },
    { 0x8afbeb18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896beb23, 33 },
    { 0x653beb23, 93 },
    { 0x203beb23, 61 },
    { 0x9bbeb26, 90 },
    { 0, 0 },
    { 0x916beb28, 33 },
    { 0x631beb29, 52 },
    { 0x7e7beb28, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x129beb2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecbeb35, 93 },
    { 0xe05beb35, 33 },
    { 0x117beb36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02beb3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fbeb42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6beb45, 90 },
    { 0xd5ebeb45, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdbeb4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccbeb4f, 61 },
    { 0, 0 },
    { 0xb9bbeb51, 46 },
    { 0x556beb52, 76 },
    { 0, 0 },
    { 0x1d3beb54, 76 },
    { 0x5b3beb55, 66 },
    { 0xb3beb56, 90 },
    { 0x125beb54, 98 },
    { 0x906beb58, 33 },
    { 0, 0 },
    { 0xe97beb5a, 34 },
    { 0x6f6beb5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65beb5e, 61 },
    { 0x919beb5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831beb65, 90 },
    { 0, 0 },
    { 0xd6abeb67, 33 },
    { 0x973beb68, 43 },
    { 0xac1beb69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bbeb72, 34 },
    { 0x140beb73, 68 },
    { 0x82fbeb74, 61 },
    { 0x1d9beb75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fbeb78, 98 },
    { 0x59fbeb78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beb7f, 95 },
    { 0x4f7beb80, 95 },
    { 0, 0 },
    { 0x185beb82, 95 },
    { 0xbf0beb82, 61 },
    { 0xa3cbeb82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x224beb87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fbeb8b, 43 },
    { 0xa39beb8c, 65 },
    { 0, 0 },
    { 0x17bbeb8e, 34 },
    { 0x31beb8e, 90 },
    { 0x8f7beb90, 61 },
    { 0, 0 },
    { 0x6a1beb92, 98 },
    { 0x5e8beb92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43beb99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bbeba2, 46 },
    { 0x987beba2, 23 },
    { 0, 0 },
    { 0x9d6beba5, 76 },
    { 0x465beba5, 98 },
    { 0x5c8beba7, 78 },
    { 0, 0 },
    { 0x2f4beba9, 93 },
    { 0x8cbebaa, 66 },
    { 0xac7bebab, 98 },
    { 0x8d3bebac, 52 },
    { 0x53bbebad, 95 },
    { 0, 0 },
    { 0xcc2bebaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fbebb4, 76 },
    { 0xd6bebb4, 84 },
    { 0x28ebebb5, 33 },
    { 0x14fbebb4, 23 },
    { 0x6f9bebb4, 33 },
    { 0x9ecbebb8, 23 },
    { 0x39bebba, 43 },
    { 0, 0 },
    { 0x537bebbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1bebbf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195bebc3, 90 },
    { 0xba7bebc4, 98 },
    { 0x95fbebc3, 61 },
    { 0, 0 },
    { 0xb32bebc7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fbebd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5bebd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3bebdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddabebdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5bebe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fbebe8, 90 },
    { 0, 0 },
    { 0x191bebea, 43 },
    { 0xf85bebeb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bebee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5bebf2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31bebf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ebebfc, 43 },
    { 0x2ddbebfd, 52 },
    { 0x449bebfc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0bec02, 23 },
    { 0x1d2bec03, 66 },
    { 0x32fbec04, 68 },
    { 0x2c9bec05, 61 },
    { 0, 0 },
    { 0xc5ebec07, 33 },
    { 0, 0 },
    { 0xe6cbec09, 52 },
    { 0, 0 },
    { 0x348bec0b, 55 },
    { 0x417bec0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6bec12, 23 },
    { 0xd3bbec13, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x646bec16, 90 },
    { 0, 0 },
    { 0xf82bec18, 33 },
    { 0xdf2bec19, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dbec1d, 55 },
    { 0, 0 },
    { 0x2ebec1f, 98 },
    { 0x2b9bec1f, 52 },
    { 0, 0 },
    { 0x8f5bec22, 87 },
    { 0, 0 },
    { 0x9e2bec24, 90 },
    { 0xa33bec24, 59 },
    { 0x456bec26, 61 },
    { 0x786bec27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dbec34, 98 },
    { 0x72ebec34, 19 },
    { 0xc55bec36, 68 },
    { 0x301bec34, 61 },
    { 0x921bec38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x742bec3b, 98 },
    { 0, 0 },
    { 0x328bec3d, 61 },
    { 0x961bec3e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbec42, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157bec50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5bec53, 91 },
    { 0, 0 },
    { 0x2eebec55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34bec58, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2bec5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09bec60, 17 },
    { 0x1f6bec61, 84 },
    { 0xaebec61, 98 },
    { 0, 0 },
    { 0x321bec64, 55 },
    { 0, 0 },
    { 0x7e4bec66, 94 },
    { 0xcdfbec67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07bec6c, 80 },
    { 0, 0 },
    { 0x83bec6e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbebec74, 34 },
    { 0, 0 },
    { 0xfb3bec76, 34 },
    { 0xdbbbec77, 33 },
    { 0x665bec77, 99 },
    { 0x403bec79, 23 },
    { 0xfbdbec7a, 100 },
    { 0x35cbec79, 23 },
    { 0x6e3bec78, 84 },
    { 0x756bec7d, 76 },
    { 0, 0 },
    { 0x9e2bec7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01bec83, 23 },
    { 0x420bec84, 61 },
    { 0x6d7bec85, 81 },
    { 0xa4ebec85, 33 },
    { 0xe1bec86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9bec8d, 55 },
    { 0xa07bec8d, 33 },
    { 0xe72bec8f, 90 },
    { 0x93cbec90, 84 },
    { 0, 0 },
    { 0x9bebec92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633bec96, 23 },
    { 0, 0 },
    { 0xa8bec98, 61 },
    { 0, 0 },
    { 0x6c5bec9a, 66 },
    { 0, 0 },
    { 0x50dbec9c, 34 },
    { 0x11fbec9c, 90 },
    { 0x9e2bec9e, 85 },
    { 0, 0 },
    { 0x6cfbeca0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dbeca7, 76 },
    { 0, 0 },
    { 0xe03beca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ebecb0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462becb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18becb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bbecbd, 33 },
    { 0x9f4becbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68becc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ebeccb, 33 },
    { 0x32bbeccc, 90 },
    { 0, 0 },
    { 0xa1ebecce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccbecda, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdbecde, 98 },
    { 0xe69becdf, 90 },
    { 0xa4abece0, 55 },
    { 0x928becdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6febece5, 23 },
    { 0xc61bece6, 33 },
    { 0, 0 },
    { 0xa32bece8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cbecec, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x737becef, 78 },
    { 0x648becf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1bed02, 95 },
    { 0xdf7bed02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52bed0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fbed0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0bed14, 33 },
    { 0x106bed15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0bed18, 34 },
    { 0, 0 },
    { 0x21bed1a, 33 },
    { 0x365bed1b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x162bed1e, 43 },
    { 0xbccbed1e, 95 },
    { 0x1fbed20, 33 },
    { 0xeadbed20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0bed2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edbed38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830bed3c, 55 },
    { 0xa82bed3d, 90 },
    { 0xdabbed3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ebed45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806bed49, 34 },
    { 0xdaabed4a, 94 },
    { 0xb03bed49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56bed4e, 93 },
    { 0, 0 },
    { 0xf44bed50, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ebed56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbed5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32bed67, 67 },
    { 0, 0 },
    { 0xd12bed69, 33 },
    { 0, 0 },
    { 0x703bed6b, 68 },
    { 0xe63bed6c, 90 },
    { 0x21abed6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4bed70, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5bed75, 46 },
    { 0, 0 },
    { 0x82abed77, 95 },
    { 0xed0bed78, 90 },
    { 0x3fdbed78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bed7d, 81 },
    { 0x6d6bed7d, 46 },
    { 0, 0 },
    { 0xacabed80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6bed88, 90 },
    { 0xb0ebed89, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8bed8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72bed92, 43 },
    { 0xa7bed93, 76 },
    { 0x812bed94, 61 },
    { 0xeb4bed95, 77 },
    { 0, 0 },
    { 0x4b2bed97, 91 },
    { 0x1f0bed97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3febed9b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dbeda2, 69 },
    { 0x61cbeda2, 78 },
    { 0x9cfbeda3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664beda9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26bedac, 42 },
    { 0x8a7bedad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fbedb0, 23 },
    { 0, 0 },
    { 0x109bedb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54bedb8, 33 },
    { 0, 0 },
    { 0x252bedba, 90 },
    { 0, 0 },
    { 0xdebbedbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58bedc0, 98 },
    { 0x864bedc0, 61 },
    { 0x342bedc2, 65 },
    { 0x632bedc3, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149bedc8, 46 },
    { 0xa20bedc9, 98 },
    { 0xb3bbedca, 87 },
    { 0xf53bedcb, 33 },
    { 0x846bedc8, 98 },
    { 0x457bedcd, 61 },
    { 0xc48bedc8, 33 },
    { 0, 0 },
    { 0x3afbedd0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bedd6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07bede3, 55 },
    { 0, 0 },
    { 0xff0bede5, 95 },
    { 0x4b1bede6, 23 },
    { 0x8cbede7, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310bedeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0bedf0, 53 },
    { 0x517bedf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bbedf5, 67 },
    { 0x872bedf5, 43 },
    { 0xdc6bedf7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622bedfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926bee02, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bee08, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0bee0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbee10, 95 },
    { 0, 0 },
    { 0xda8bee12, 85 },
    { 0x391bee13, 84 },
    { 0x5edbee12, 78 },
    { 0x4d0bee15, 33 },
    { 0x3e0bee16, 23 },
    { 0xe76bee15, 98 },
    { 0xab1bee18, 43 },
    { 0x70abee17, 90 },
    { 0xad1bee13, 40 },
    { 0xc6ebee17, 34 },
    { 0, 0 },
    { 0x877bee1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ebee21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbee24, 43 },
    { 0xb7bee25, 58 },
    { 0x408bee24, 33 },
    { 0, 0 },
    { 0xc7bbee28, 90 },
    { 0x466bee29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9bee2e, 33 },
    { 0, 0 },
    { 0x395bee30, 90 },
    { 0x6bebee31, 90 },
    { 0, 0 },
    { 0xf68bee33, 90 },
    { 0x5c6bee33, 95 },
    { 0, 0 },
    { 0xcf3bee36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bbee39, 95 },
    { 0xeb9bee39, 33 },
    { 0x18bee3b, 23 },
    { 0x5dfbee39, 98 },
    { 0xa5bbee39, 40 },
    { 0xdcbee3e, 76 },
    { 0xde3bee3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03bee42, 98 },
    { 0x15dbee43, 67 },
    { 0xf34bee43, 33 },
    { 0xf19bee45, 90 },
    { 0, 0 },
    { 0xfa3bee47, 98 },
    { 0xcd7bee47, 98 },
    { 0xd0fbee47, 95 },
    { 0, 0 },
    { 0x2f7bee4b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388bee50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621bee55, 61 },
    { 0x614bee56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64bee59, 90 },
    { 0x8c5bee5a, 90 },
    { 0x376bee5b, 84 },
    { 0, 0 },
    { 0x6bbbee5d, 33 },
    { 0x90abee5e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8bee65, 98 },
    { 0xaf2bee66, 90 },
    { 0, 0 },
    { 0x1e1bee68, 90 },
    { 0xe4abee69, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9bee6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14bee71, 23 },
    { 0xa09bee72, 88 },
    { 0x971bee73, 99 },
    { 0xc4dbee74, 98 },
    { 0xeb2bee71, 90 },
    { 0xdc5bee73, 43 },
    { 0x171bee77, 95 },
    { 0, 0 },
    { 0x425bee79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1bee7e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9bee83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fbee8a, 61 },
    { 0, 0 },
    { 0xaa9bee8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561bee91, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783bee95, 17 },
    { 0x799bee96, 90 },
    { 0x3b8bee96, 90 },
    { 0, 0 },
    { 0xeabbee99, 98 },
    { 0xed4bee99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x419bee9d, 67 },
    { 0xdffbee9e, 43 },
    { 0xa4abee9f, 98 },
    { 0xe2cbee9d, 85 },
    { 0xefabee9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfbeea5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cbeeaf, 88 },
    { 0x3e5beeaf, 88 },
    { 0xfabbeeb0, 33 },
    { 0x3d0beeb2, 61 },
    { 0xae4beeb3, 66 },
    { 0x6acbeeb3, 61 },
    { 0x970beeb3, 43 },
    { 0, 0 },
    { 0x952beeb7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eebeebe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253beec3, 99 },
    { 0x98ebeec4, 23 },
    { 0x9a7beec3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4beec8, 100 },
    { 0, 0 },
    { 0xacabeeca, 78 },
    { 0xd10beeca, 93 },
    { 0xcfbbeecc, 35 },
    { 0x513beecd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x627beed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1beed3, 98 },
    { 0x4efbeed4, 95 },
    { 0xfeebeed5, 52 },
    { 0xf4cbeed3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06beed9, 98 },
    { 0x6cdbeed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777beede, 33 },
    { 0xe87beedf, 33 },
    { 0x9c4beee0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x620beee3, 78 },
    { 0x227beee4, 61 },
    { 0xe45beee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cbeeea, 61 },
    { 0x9c7beeea, 68 },
    { 0x883beeea, 76 },
    { 0, 0 },
    { 0x1a1beeee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71beef6, 52 },
    { 0x19fbeef6, 90 },
    { 0x117beef8, 76 },
    { 0x348beef8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bbeefd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfbef03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ebef07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88bef0c, 95 },
    { 0x4dbbef0d, 61 },
    { 0x8cabef0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbef11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609bef1c, 84 },
    { 0, 0 },
    { 0x49cbef1e, 46 },
    { 0x951bef1e, 98 },
    { 0, 0 },
    { 0x175bef21, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848bef25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83bef29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21bef2c, 52 },
    { 0x2d7bef2c, 90 },
    { 0, 0 },
    { 0x3b6bef2f, 90 },
    { 0, 0 },
    { 0x777bef31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ebef39, 90 },
    { 0xe19bef3a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5bef3e, 90 },
    { 0x131bef3f, 90 },
    { 0x3c9bef40, 76 },
    { 0, 0 },
    { 0xf5fbef42, 33 },
    { 0, 0 },
    { 0x23dbef44, 69 },
    { 0x769bef45, 84 },
    { 0x960bef44, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x45abef49, 95 },
    { 0, 0 },
    { 0x40ebef4b, 90 },
    { 0x43fbef4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ebef51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dbef56, 67 },
    { 0x918bef57, 46 },
    { 0x188bef58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3bef5b, 33 },
    { 0, 0 },
    { 0x717bef5d, 99 },
    { 0, 0 },
    { 0x7dbbef5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecbef63, 52 },
    { 0x4dbbef63, 55 },
    { 0x245bef64, 100 },
    { 0, 0 },
    { 0xd0bef67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07bef6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1bef7b, 90 },
    { 0xb15bef7b, 95 },
    { 0x459bef7d, 81 },
    { 0xe25bef7e, 76 },
    { 0x9c9bef7d, 33 },
    { 0x3c3bef80, 33 },
    { 0x858bef7c, 98 },
    { 0xf4bef7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1bef85, 98 },
    { 0x1c6bef86, 33 },
    { 0x764bef87, 90 },
    { 0x2debef87, 61 },
    { 0xaf0bef88, 81 },
    { 0, 0 },
    { 0xf0bef8b, 61 },
    { 0xe3bbef8b, 23 },
    { 0x871bef8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304bef93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111bef97, 84 },
    { 0, 0 },
    { 0x49ebef99, 99 },
    { 0, 0 },
    { 0xbe7bef9b, 90 },
    { 0xfc3bef9b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3befa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbbefa8, 33 },
    { 0x424befa9, 95 },
    { 0, 0 },
    { 0x51dbefab, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabefb0, 76 },
    { 0, 0 },
    { 0xe43befb2, 97 },
    { 0xababefb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffbefbb, 33 },
    { 0x316befbb, 55 },
    { 0xfbfbefbd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412befc2, 91 },
    { 0xe04befc3, 100 },
    { 0x1bdbefc3, 85 },
    { 0xf74befc5, 100 },
    { 0x665befc6, 23 },
    { 0x341befc4, 90 },
    { 0x90abefc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddbefcc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498befd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccbefd5, 98 },
    { 0, 0 },
    { 0xee7befd7, 78 },
    { 0, 0 },
    { 0x184befd9, 90 },
    { 0xe23befda, 34 },
    { 0x582befda, 100 },
    { 0x663befdc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecbefe0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30befe3, 95 },
    { 0x9b6befe4, 87 },
    { 0xb30befe3, 67 },
    { 0, 0 },
    { 0x3c8befe7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0befea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fbeff0, 36 },
    { 0, 0 },
    { 0xd2dbeff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21beff7, 79 },
    { 0, 0 },
    { 0x1ebeff9, 23 },
    { 0x679beffa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73beffd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02bf009, 95 },
    { 0xcd0bf00a, 55 },
    { 0xa60bf00b, 87 },
    { 0x330bf00a, 90 },
    { 0x2d7bf00c, 33 },
    { 0, 0 },
    { 0xc8fbf00f, 61 },
    { 0x74dbf010, 33 },
    { 0x4a1bf011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8bf014, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cbf017, 55 },
    { 0x629bf018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbbf01b, 98 },
    { 0xf1cbf01c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11bf01f, 43 },
    { 0xfc3bf020, 33 },
    { 0x4e0bf020, 77 },
    { 0x6fbf021, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ebf025, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbbf02f, 78 },
    { 0xd4cbf030, 8 },
    { 0xce6bf031, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5bf03a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dbf03e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810bf046, 84 },
    { 0x470bf047, 90 },
    { 0, 0 },
    { 0xe14bf049, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabf04c, 95 },
    { 0x9f4bf04d, 81 },
    { 0x2d4bf04e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x310bf051, 76 },
    { 0, 0 },
    { 0x25dbf053, 66 },
    { 0x36cbf054, 98 },
    { 0, 0 },
    { 0x8e4bf056, 33 },
    { 0x837bf057, 33 },
    { 0xecbbf058, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x608bf05b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20bf05e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7bf063, 52 },
    { 0xa76bf064, 87 },
    { 0x714bf065, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aabf069, 99 },
    { 0x41cbf069, 90 },
    { 0, 0 },
    { 0x7b4bf06c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dbf075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcbf078, 67 },
    { 0x933bf079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbf07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5bf080, 95 },
    { 0, 0 },
    { 0xc93bf082, 46 },
    { 0xd0cbf083, 75 },
    { 0xa62bf082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ebf088, 33 },
    { 0, 0 },
    { 0x30bf08a, 55 },
    { 0xf67bf08b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1bf08f, 55 },
    { 0x701bf090, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46bf095, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6bf09c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ebf0a3, 98 },
    { 0xb00bf0a3, 33 },
    { 0, 0 },
    { 0x5f6bf0a6, 90 },
    { 0x345bf0a6, 90 },
    { 0, 0 },
    { 0xeffbf0a9, 55 },
    { 0, 0 },
    { 0xe5cbf0ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabbf0b1, 98 },
    { 0, 0 },
    { 0xa43bf0b3, 23 },
    { 0, 0 },
    { 0x783bf0b5, 87 },
    { 0xed4bf0b6, 90 },
    { 0x120bf0b7, 61 },
    { 0xda9bf0b7, 68 },
    { 0x541bf0b7, 95 },
    { 0x454bf0b6, 90 },
    { 0xf1ebf0b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23bf0be, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99bf0c6, 76 },
    { 0x276bf0c6, 98 },
    { 0, 0 },
    { 0xb3fbf0c9, 68 },
    { 0, 0 },
    { 0x6acbf0cb, 90 },
    { 0, 0 },
    { 0xeedbf0cd, 91 },
    { 0x767bf0ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9bf0d2, 85 },
    { 0, 0 },
    { 0x963bf0d4, 23 },
    { 0x126bf0d5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58bf0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4bf0dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8bf0e4, 61 },
    { 0, 0 },
    { 0x2dbbf0e6, 23 },
    { 0x1d6bf0e7, 98 },
    { 0xd34bf0e8, 61 },
    { 0xba4bf0e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfbf0ed, 76 },
    { 0x19bf0ee, 76 },
    { 0xc60bf0ef, 61 },
    { 0x741bf0ee, 76 },
    { 0x83fbf0ef, 22 },
    { 0x64ebf0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aebf0fa, 55 },
    { 0, 0 },
    { 0x5a8bf0fc, 77 },
    { 0xd1cbf0fd, 52 },
    { 0, 0 },
    { 0x89bbf0ff, 78 },
    { 0x7a3bf100, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22bf103, 55 },
    { 0, 0 },
    { 0xa4ebf105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75bf10e, 90 },
    { 0x179bf10f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2bf112, 66 },
    { 0x868bf113, 90 },
    { 0, 0 },
    { 0x4acbf115, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2bf118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0bf11c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4bf124, 100 },
    { 0, 0 },
    { 0x16bbf126, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4bf12b, 23 },
    { 0x457bf12c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdbf133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bbf138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bbf13d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcbf141, 98 },
    { 0xb82bf142, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8bf14b, 33 },
    { 0, 0 },
    { 0x3c1bf14d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x122bf150, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bbf154, 95 },
    { 0xb0bbf154, 90 },
    { 0x7debf154, 43 },
    { 0x2f1bf155, 99 },
    { 0x82bf158, 48 },
    { 0xd31bf154, 82 },
    { 0x17bf15a, 36 },
    { 0xdd3bf15b, 33 },
    { 0, 0 },
    { 0x67bbf15d, 90 },
    { 0xd31bf15e, 90 },
    { 0xb91bf15f, 93 },
    { 0xa3bf160, 59 },
    { 0x84bf161, 33 },
    { 0x961bf161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64bf166, 34 },
    { 0xd6fbf166, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326bf16c, 33 },
    { 0, 0 },
    { 0x3eabf16e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100bf176, 90 },
    { 0xcedbf177, 97 },
    { 0xfa2bf176, 43 },
    { 0xc0cbf179, 76 },
    { 0xb10bf179, 66 },
    { 0x42fbf179, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736bf17f, 75 },
    { 0x41cbf180, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dbf18f, 43 },
    { 0x326bf18f, 96 },
    { 0xca8bf191, 34 },
    { 0xd89bf191, 90 },
    { 0x323bf193, 91 },
    { 0xf0fbf193, 98 },
    { 0xeddbf195, 88 },
    { 0xdc7bf194, 53 },
    { 0x1fcbf197, 55 },
    { 0, 0 },
    { 0x447bf199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dbf1a2, 17 },
    { 0x4a9bf1a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dbf1a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09bf1ab, 98 },
    { 0, 0 },
    { 0xc4cbf1ad, 99 },
    { 0, 0 },
    { 0x96fbf1af, 33 },
    { 0, 0 },
    { 0xa79bf1b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933bf1ba, 43 },
    { 0xbe3bf1bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8bf1be, 98 },
    { 0x7b7bf1be, 99 },
    { 0x5bbf1bf, 90 },
    { 0xacabf1c1, 55 },
    { 0xd9fbf1c1, 33 },
    { 0x6f0bf1c3, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8bf1c6, 76 },
    { 0x7cebf1c7, 52 },
    { 0x8b1bf1c8, 76 },
    { 0x4debf1c9, 34 },
    { 0x10abf1c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95bf1cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9abf1d9, 55 },
    { 0x7b4bf1da, 84 },
    { 0x6e7bf1d9, 61 },
    { 0x503bf1d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcbf1df, 90 },
    { 0, 0 },
    { 0x400bf1e1, 33 },
    { 0x52ebf1e2, 75 },
    { 0, 0 },
    { 0x7f6bf1e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bbf1e7, 95 },
    { 0x645bf1e8, 38 },
    { 0x38dbf1e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75bf1ef, 33 },
    { 0, 0 },
    { 0x886bf1f1, 90 },
    { 0x138bf1f2, 33 },
    { 0, 0 },
    { 0x32bbf1f4, 34 },
    { 0xadbf1f4, 43 },
    { 0x6b0bf1f6, 33 },
    { 0, 0 },
    { 0x397bf1f8, 33 },
    { 0x374bf1f8, 33 },
    { 0x446bf1f9, 90 },
    { 0xbd4bf1fb, 19 },
    { 0x12fbf1fc, 76 },
    { 0, 0 },
    { 0x26fbf1fe, 34 },
    { 0, 0 },
    { 0x3f2bf200, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3babf206, 33 },
    { 0xdc4bf207, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815bf20d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4bf210, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bbf213, 33 },
    { 0, 0 },
    { 0xa0ebf215, 61 },
    { 0x41ebf216, 23 },
    { 0xaebf217, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72bf21a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fbf21e, 33 },
    { 0x710bf21f, 61 },
    { 0, 0 },
    { 0x5f4bf221, 76 },
    { 0xf07bf222, 81 },
    { 0, 0 },
    { 0x4adbf224, 33 },
    { 0xf93bf224, 93 },
    { 0xea2bf226, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64bf22b, 55 },
    { 0xaf2bf22b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cbf231, 52 },
    { 0x631bf231, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32bf237, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ebf23a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x539bf23d, 23 },
    { 0, 0 },
    { 0xb6bf23f, 98 },
    { 0x9e4bf240, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0bf244, 46 },
    { 0x6f4bf245, 76 },
    { 0x52dbf246, 43 },
    { 0x26bbf247, 100 },
    { 0xe08bf247, 33 },
    { 0x161bf244, 67 },
    { 0x518bf245, 98 },
    { 0x9e3bf24b, 33 },
    { 0, 0 },
    { 0xe0bf24d, 90 },
    { 0, 0 },
    { 0x97fbf24f, 17 },
    { 0x790bf250, 67 },
    { 0xd51bf251, 55 },
    { 0xaa6bf251, 95 },
    { 0xa17bf253, 33 },
    { 0, 0 },
    { 0xe0bbf255, 98 },
    { 0x3b5bf256, 61 },
    { 0x7dabf255, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x988bf25a, 17 },
    { 0x69cbf25b, 90 },
    { 0x1c1bf25c, 95 },
    { 0x963bf25d, 93 },
    { 0, 0 },
    { 0xba4bf25f, 84 },
    { 0, 0 },
    { 0x9acbf261, 95 },
    { 0, 0 },
    { 0x55fbf263, 53 },
    { 0x416bf264, 90 },
    { 0x939bf265, 52 },
    { 0xad9bf266, 61 },
    { 0x29fbf267, 95 },
    { 0x58fbf267, 98 },
    { 0xe1ebf266, 37 },
    { 0x8bf263, 90 },
    { 0x38bf26b, 90 },
    { 0x232bf264, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0bf26f, 19 },
    { 0xb77bf270, 23 },
    { 0xefbf270, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bbf274, 78 },
    { 0xf37bf274, 53 },
    { 0x4a7bf274, 98 },
    { 0xa90bf277, 33 },
    { 0xe61bf275, 33 },
    { 0, 0 },
    { 0xbf8bf27a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bbf27d, 33 },
    { 0x4ebbf27e, 52 },
    { 0x6fbf27d, 90 },
    { 0xe66bf27f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3bf284, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17bf28d, 33 },
    { 0, 0 },
    { 0xcaebf28f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6bf292, 23 },
    { 0, 0 },
    { 0xd78bf294, 100 },
    { 0, 0 },
    { 0x75bbf296, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdbf29b, 76 },
    { 0x4b3bf29c, 90 },
    { 0xe52bf29b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fbf2a0, 33 },
    { 0, 0 },
    { 0xc29bf2a2, 96 },
    { 0, 0 },
    { 0x68dbf2a4, 76 },
    { 0x89ebf2a5, 93 },
    { 0xe1dbf2a5, 85 },
    { 0x1a8bf2a5, 98 },
    { 0x1eabf2a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4bf2ad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fbf2b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3bf2b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9bf2bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605bf2bf, 55 },
    { 0, 0 },
    { 0xdcabf2c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dbf2c7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40bf2cc, 90 },
    { 0x12cbf2cd, 61 },
    { 0x2f6bf2ce, 34 },
    { 0xb7bbf2cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79bf2d7, 67 },
    { 0xd89bf2d8, 98 },
    { 0xf74bf2d9, 46 },
    { 0x4c7bf2da, 61 },
    { 0x603bf2d9, 95 },
    { 0xbbbf2dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bbf2e0, 55 },
    { 0x38fbf2e1, 81 },
    { 0x5a7bf2e2, 77 },
    { 0xd64bf2e3, 90 },
    { 0, 0 },
    { 0xf62bf2e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3bf2ed, 91 },
    { 0x8dabf2ed, 76 },
    { 0xe64bf2ef, 23 },
    { 0, 0 },
    { 0xad9bf2f1, 99 },
    { 0, 0 },
    { 0x30abf2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cbf2f6, 95 },
    { 0x2e2bf2f7, 95 },
    { 0, 0 },
    { 0x929bf2f9, 77 },
    { 0x4a1bf2fa, 95 },
    { 0, 0 },
    { 0x2b1bf2fc, 55 },
    { 0x314bf2fc, 17 },
    { 0, 0 },
    { 0x3bbf2ff, 33 },
    { 0x72abf300, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fbf304, 55 },
    { 0xf0ebf305, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425bf30d, 95 },
    { 0x47abf30d, 90 },
    { 0, 0 },
    { 0x22dbf310, 52 },
    { 0x20ebf310, 95 },
    { 0, 0 },
    { 0x26dbf313, 23 },
    { 0x385bf313, 33 },
    { 0, 0 },
    { 0x178bf316, 88 },
    { 0x737bf317, 61 },
    { 0xd35bf318, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cbf31f, 90 },
    { 0x820bf320, 61 },
    { 0x8cbf31f, 33 },
    { 0x19bbf31f, 90 },
    { 0, 0 },
    { 0xe1abf324, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x66abf327, 46 },
    { 0x4dfbf327, 23 },
    { 0x5d6bf327, 80 },
    { 0xe88bf329, 23 },
    { 0x42cbf32b, 61 },
    { 0xb74bf32b, 87 },
    { 0x546bf32d, 77 },
    { 0x8e2bf32d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdbf334, 66 },
    { 0xc60bf335, 34 },
    { 0x514bf336, 95 },
    { 0xdd8bf337, 91 },
    { 0x647bf336, 61 },
    { 0xb35bf339, 98 },
    { 0x5a5bf33a, 84 },
    { 0x74dbf334, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fbf33f, 88 },
    { 0xb55bf340, 23 },
    { 0x4b2bf341, 52 },
    { 0x1cebf342, 90 },
    { 0xa5abf33f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22bf34f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dbf352, 76 },
    { 0xfbcbf352, 76 },
    { 0, 0 },
    { 0xa79bf355, 46 },
    { 0xca6bf356, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dbf360, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20bf363, 90 },
    { 0x8ebf364, 34 },
    { 0x6ebf364, 33 },
    { 0xc82bf363, 52 },
    { 0xa8ebf367, 99 },
    { 0x29bbf368, 90 },
    { 0x2a8bf368, 33 },
    { 0xadebf369, 93 },
    { 0, 0 },
    { 0x370bf36c, 43 },
    { 0, 0 },
    { 0x8b4bf36e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04bf371, 61 },
    { 0xb6dbf372, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932bf376, 33 },
    { 0xadbbf377, 46 },
    { 0x22abf378, 33 },
    { 0x505bf376, 95 },
    { 0xb7dbf37a, 40 },
    { 0, 0 },
    { 0xa92bf37c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7bf381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ebf388, 95 },
    { 0xabfbf388, 76 },
    { 0, 0 },
    { 0x518bf38b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9bf395, 33 },
    { 0x9b5bf396, 93 },
    { 0, 0 },
    { 0x57abf398, 99 },
    { 0x4babf399, 85 },
    { 0x3b7bf399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2bf39e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630bf3a4, 61 },
    { 0x7abbf3a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93bf3a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715bf3b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06bf3b5, 66 },
    { 0x465bf3b6, 65 },
    { 0x12ebf3b7, 90 },
    { 0xc56bf3b8, 61 },
    { 0, 0 },
    { 0xf99bf3ba, 61 },
    { 0x876bf3ba, 61 },
    { 0, 0 },
    { 0x690bf3bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x478bf3c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fbf3c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffbf3c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548bf3d1, 90 },
    { 0, 0 },
    { 0x8c1bf3d3, 90 },
    { 0xdbcbf3d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x841bf3d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54bf3dc, 84 },
    { 0x50cbf3dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59bf3e0, 23 },
    { 0x716bf3e1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66bf3e4, 100 },
    { 0xcadbf3e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206bf3e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffbf3ee, 52 },
    { 0, 0 },
    { 0x54abf3f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ebf3f3, 76 },
    { 0, 0 },
    { 0x5a2bf3f5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aebf3f8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3bf3fb, 23 },
    { 0xd13bf3fc, 98 },
    { 0xe2abf3fc, 93 },
    { 0xbc6bf3fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83bf403, 98 },
    { 0, 0 },
    { 0x6f4bf405, 61 },
    { 0xc6bbf406, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2bf409, 90 },
    { 0x49ebf409, 99 },
    { 0xba6bf40b, 90 },
    { 0x523bf40b, 81 },
    { 0xd55bf40d, 76 },
    { 0xab0bf40e, 61 },
    { 0x7f4bf40f, 66 },
    { 0xe3bbf410, 90 },
    { 0xd61bf411, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x667bf414, 90 },
    { 0xad5bf415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92bf419, 93 },
    { 0x635bf419, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670bf423, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fbf427, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1bf431, 38 },
    { 0x38bbf432, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2bf436, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2bf43f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x465bf442, 61 },
    { 0x2d2bf443, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601bf447, 90 },
    { 0xe4dbf448, 90 },
    { 0x857bf449, 52 },
    { 0xbeebf449, 23 },
    { 0x79fbf44a, 17 },
    { 0x563bf449, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4bf450, 61 },
    { 0xa71bf451, 55 },
    { 0xf91bf450, 61 },
    { 0x765bf452, 67 },
    { 0xbe9bf454, 52 },
    { 0xa7abf455, 61 },
    { 0xe9fbf455, 96 },
    { 0xda8bf455, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf45b, 95 },
    { 0x4d6bf45c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930bf461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790bf466, 61 },
    { 0, 0 },
    { 0xa6abf468, 95 },
    { 0x821bf469, 33 },
    { 0x8f7bf469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dabf46e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bbf472, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5bf47e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76bf483, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7bf486, 90 },
    { 0, 0 },
    { 0x70cbf488, 61 },
    { 0xbcfbf489, 76 },
    { 0x409bf48a, 33 },
    { 0xd08bf48a, 98 },
    { 0, 0 },
    { 0x916bf48d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92bf490, 33 },
    { 0x375bf490, 33 },
    { 0x4b6bf490, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410bf496, 90 },
    { 0, 0 },
    { 0x8a8bf498, 43 },
    { 0xfcabf499, 23 },
    { 0, 0 },
    { 0xe38bf49b, 61 },
    { 0x13ebf49b, 93 },
    { 0xd55bf49d, 52 },
    { 0x509bf49e, 34 },
    { 0xc96bf49e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3bf4a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bbf4ac, 76 },
    { 0x8fabf4ad, 61 },
    { 0xf6abf4ae, 55 },
    { 0x1bcbf4ae, 91 },
    { 0x3cebf4b0, 76 },
    { 0x365bf4b1, 84 },
    { 0x649bf4b1, 90 },
    { 0x8c7bf4b3, 77 },
    { 0, 0 },
    { 0x50ebf4b5, 46 },
    { 0, 0 },
    { 0xea1bf4b7, 99 },
    { 0xfebf4b8, 67 },
    { 0x2d8bf4b9, 76 },
    { 0x92fbf4b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141bf4bf, 100 },
    { 0, 0 },
    { 0x618bf4c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5bf4c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71bf4c9, 90 },
    { 0x3d0bf4c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf4ce, 43 },
    { 0x198bf4ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1bf4d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3bf4d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aebf4d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0bf4dc, 23 },
    { 0xf59bf4dc, 76 },
    { 0xafebf4dc, 95 },
    { 0x778bf4df, 95 },
    { 0x2bfbf4de, 95 },
    { 0x8a2bf4e0, 33 },
    { 0x5bfbf4e2, 33 },
    { 0xf53bf4e0, 61 },
    { 0x8d9bf4dd, 61 },
    { 0x4ebbf4e1, 61 },
    { 0xd60bf4e6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bbf4e9, 33 },
    { 0xe55bf4ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebbf4ed, 33 },
    { 0xd7bbf4ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaabf4f1, 99 },
    { 0x255bf4f1, 90 },
    { 0x450bf4f3, 91 },
    { 0, 0 },
    { 0x896bf4f5, 61 },
    { 0, 0 },
    { 0x8a4bf4f7, 77 },
    { 0x948bf4f7, 55 },
    { 0xb23bf4f9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02bf4fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647bf502, 80 },
    { 0x13bbf503, 9 },
    { 0x9efbf504, 61 },
    { 0xe96bf504, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1bf508, 43 },
    { 0x4c1bf508, 23 },
    { 0, 0 },
    { 0x3e7bf50b, 81 },
    { 0x6fbf50c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759bf510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bbf514, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbf517, 94 },
    { 0, 0 },
    { 0x9a4bf519, 23 },
    { 0xe38bf51a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4bf520, 90 },
    { 0, 0 },
    { 0xee5bf522, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56bf525, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964bf532, 46 },
    { 0, 0 },
    { 0x29ebf534, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341bf538, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01bf53b, 97 },
    { 0, 0 },
    { 0xd78bf53d, 61 },
    { 0xfddbf53e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf541, 90 },
    { 0, 0 },
    { 0x78cbf543, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0bf547, 78 },
    { 0x5e8bf547, 33 },
    { 0, 0 },
    { 0x29ebf54a, 98 },
    { 0xfddbf54b, 66 },
    { 0xacfbf54a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1bf54f, 95 },
    { 0xaa3bf54f, 43 },
    { 0x6f9bf551, 61 },
    { 0x76bf552, 90 },
    { 0x775bf54f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4bf55d, 88 },
    { 0xf54bf55e, 61 },
    { 0x539bf55f, 61 },
    { 0x906bf560, 34 },
    { 0x4a9bf55e, 95 },
    { 0x674bf562, 33 },
    { 0x538bf563, 98 },
    { 0xf33bf564, 90 },
    { 0x758bf563, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87bf569, 90 },
    { 0, 0 },
    { 0xf14bf56b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbbf56e, 34 },
    { 0x91cbf56f, 90 },
    { 0x5ecbf56f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7bf573, 90 },
    { 0xcc2bf573, 66 },
    { 0x2c3bf575, 66 },
    { 0, 0 },
    { 0x794bf577, 33 },
    { 0, 0 },
    { 0xc7abf579, 23 },
    { 0x1f0bf57a, 90 },
    { 0xc46bf57b, 78 },
    { 0x7b6bf57b, 76 },
    { 0xc95bf57d, 93 },
    { 0, 0 },
    { 0xb80bf57f, 95 },
    { 0xba3bf57f, 34 },
    { 0x287bf581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eebf585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7bf589, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fbf591, 76 },
    { 0x6a6bf592, 67 },
    { 0xa9bf592, 84 },
    { 0x679bf593, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbf597, 84 },
    { 0x926bf598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48bf59d, 76 },
    { 0xab9bf59e, 61 },
    { 0, 0 },
    { 0x146bf5a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664bf5a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cbf5a8, 43 },
    { 0, 0 },
    { 0x556bf5aa, 23 },
    { 0x300bf5aa, 46 },
    { 0x6efbf5ac, 91 },
    { 0x123bf5ad, 95 },
    { 0xee1bf5ac, 95 },
    { 0x67cbf5af, 90 },
    { 0x824bf5ae, 95 },
    { 0xf27bf5b1, 23 },
    { 0xc6dbf5b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5bf5b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5bf5bb, 66 },
    { 0, 0 },
    { 0xc33bf5bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4abf5c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22abf5c5, 76 },
    { 0, 0 },
    { 0x81bf5c7, 91 },
    { 0, 0 },
    { 0x814bf5c9, 87 },
    { 0x4abbf5c9, 95 },
    { 0x747bf5ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eebf5d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cbf5d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37bf5db, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ebf5de, 55 },
    { 0xd57bf5df, 61 },
    { 0, 0 },
    { 0xb8bbf5e1, 61 },
    { 0xe3dbf5e1, 91 },
    { 0x8b6bf5e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ebf5e6, 61 },
    { 0x65bf5e7, 61 },
    { 0, 0 },
    { 0xda5bf5e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33bf5ee, 90 },
    { 0x196bf5ee, 43 },
    { 0, 0 },
    { 0xd2bf5f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcbf5f5, 52 },
    { 0xe39bf5f5, 43 },
    { 0x7e3bf5f7, 33 },
    { 0xe1dbf5f7, 93 },
    { 0xc6bf5f9, 23 },
    { 0x4bbf5f9, 70 },
    { 0x177bf5fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedabf5ff, 61 },
    { 0xf38bf5ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42dbf603, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568bf607, 95 },
    { 0xc2dbf607, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cbf60d, 95 },
    { 0, 0 },
    { 0x8a8bf60f, 65 },
    { 0, 0 },
    { 0xebcbf611, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35bf616, 40 },
    { 0x768bf617, 33 },
    { 0, 0 },
    { 0x507bf619, 85 },
    { 0x5e5bf61a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1bf61f, 90 },
    { 0, 0 },
    { 0x56cbf621, 85 },
    { 0xc21bf622, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558bf62d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4bf632, 33 },
    { 0xde7bf633, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30bf636, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x934bf639, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14bf63d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dbf644, 90 },
    { 0, 0 },
    { 0x3fabf646, 61 },
    { 0, 0 },
    { 0x3d9bf648, 87 },
    { 0, 0 },
    { 0xfbbbf64a, 95 },
    { 0x5ecbf64a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737bf64f, 99 },
    { 0x846bf650, 33 },
    { 0xbffbf650, 23 },
    { 0x790bf652, 23 },
    { 0x5c4bf64f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x654bf656, 100 },
    { 0, 0 },
    { 0xf58bf658, 90 },
    { 0x278bf659, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dbf65d, 85 },
    { 0xc24bf65d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cbf663, 90 },
    { 0x763bf663, 87 },
    { 0x501bf665, 85 },
    { 0x21bbf666, 33 },
    { 0xd0abf666, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cebf66d, 23 },
    { 0xf0bbf66e, 90 },
    { 0x21bf66e, 95 },
    { 0x6fcbf66f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86bf675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10bf678, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88bf67d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3bf682, 76 },
    { 0, 0 },
    { 0xbc0bf684, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0bf689, 90 },
    { 0xa3dbf689, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cbf68d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aebf692, 65 },
    { 0, 0 },
    { 0xf84bf694, 99 },
    { 0x414bf695, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fabf698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87bf69b, 95 },
    { 0x63dbf69b, 80 },
    { 0xeebbf69d, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0bf6a6, 34 },
    { 0xdedbf6a7, 23 },
    { 0x6e1bf6a8, 33 },
    { 0x5c7bf6a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1bf6af, 61 },
    { 0, 0 },
    { 0x9b7bf6b1, 17 },
    { 0x3a9bf6b2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14abf6b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aabf6c1, 33 },
    { 0x3f5bf6c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x196bf6c5, 84 },
    { 0, 0 },
    { 0x471bf6c7, 91 },
    { 0xd5abf6c7, 37 },
    { 0, 0 },
    { 0xc78bf6ca, 93 },
    { 0, 0 },
    { 0xfabbf6cc, 55 },
    { 0xc40bf6cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfbf6d5, 33 },
    { 0xec3bf6d6, 76 },
    { 0x1c9bf6d7, 23 },
    { 0, 0 },
    { 0xcb9bf6d9, 61 },
    { 0xa27bf6da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cbf6e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccbf6e5, 34 },
    { 0x174bf6e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdbf6ef, 76 },
    { 0x10bf6f0, 76 },
    { 0, 0 },
    { 0x8f6bf6f2, 61 },
    { 0x924bf6f3, 80 },
    { 0xd01bf6f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4bf6fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63bf6fd, 61 },
    { 0xfb0bf6fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0bf703, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fbf710, 85 },
    { 0x6b5bf710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514bf716, 66 },
    { 0, 0 },
    { 0xbfbf718, 43 },
    { 0xf68bf719, 98 },
    { 0, 0 },
    { 0x62fbf71b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84bf723, 61 },
    { 0x4d5bf723, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8bf72b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7bf72e, 43 },
    { 0, 0 },
    { 0x104bf730, 38 },
    { 0x155bf731, 69 },
    { 0xffabf732, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831bf73e, 93 },
    { 0, 0 },
    { 0x1d3bf740, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5bf744, 23 },
    { 0xcdabf744, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0bf752, 53 },
    { 0xc3bf752, 69 },
    { 0x305bf752, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cbf757, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86bf75e, 85 },
    { 0x6cebf75f, 93 },
    { 0xaddbf75f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebbf764, 61 },
    { 0x25dbf764, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fbf76f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3abf772, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x965bf775, 33 },
    { 0, 0 },
    { 0xf4dbf777, 61 },
    { 0x6c6bf778, 90 },
    { 0x4cbbf777, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03bf781, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29abf787, 85 },
    { 0, 0 },
    { 0x5b4bf789, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6bf78d, 87 },
    { 0xf17bf78e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01bf791, 86 },
    { 0, 0 },
    { 0xc2ebf793, 81 },
    { 0x106bf793, 33 },
    { 0x852bf795, 68 },
    { 0x8ccbf795, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71bf79a, 95 },
    { 0x1d9bf79b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5bf7a3, 90 },
    { 0xbddbf7a3, 98 },
    { 0x2f4bf7a4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x136bf7a8, 43 },
    { 0, 0 },
    { 0x592bf7aa, 55 },
    { 0xb70bf7ab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bf7af, 33 },
    { 0x969bf7b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dbf7b5, 46 },
    { 0xebfbf7b5, 84 },
    { 0x150bf7b5, 23 },
    { 0xe1bbf7b8, 80 },
    { 0x860bf7b9, 90 },
    { 0x42fbf7b5, 33 },
    { 0x17bbf7bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95bf7c4, 98 },
    { 0xd08bf7c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffbf7c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bf7cd, 77 },
    { 0, 0 },
    { 0x7f0bf7cf, 23 },
    { 0x80fbf7d0, 87 },
    { 0x273bf7d1, 87 },
    { 0xc73bf7d0, 76 },
    { 0xf7bbf7d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05bf7d6, 90 },
    { 0x4b6bf7d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bbf7dc, 88 },
    { 0x167bf7dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91bf7e4, 23 },
    { 0, 0 },
    { 0x6bf7e6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10abf7ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bf7ef, 23 },
    { 0x83ebf7f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29abf7f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbbf7f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3bf7fc, 93 },
    { 0xc34bf7fc, 76 },
    { 0xe85bf7fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481bf802, 68 },
    { 0x6cbbf803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57bf806, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x746bf809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404bf80d, 68 },
    { 0x3f2bf80d, 90 },
    { 0x19fbf80f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481bf819, 90 },
    { 0, 0 },
    { 0x288bf81b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cbf823, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x141bf826, 23 },
    { 0x614bf826, 95 },
    { 0xffebf828, 93 },
    { 0, 0 },
    { 0x109bf82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922bf82e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bf838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bf83b, 98 },
    { 0xcd7bf83c, 61 },
    { 0x589bf83c, 91 },
    { 0x3f5bf83c, 61 },
    { 0x26cbf83e, 98 },
    { 0x4f9bf840, 23 },
    { 0xe94bf841, 85 },
    { 0x897bf842, 100 },
    { 0x8aebf843, 90 },
    { 0x946bf841, 55 },
    { 0x650bf842, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aebf848, 23 },
    { 0xa14bf849, 61 },
    { 0, 0 },
    { 0xd0dbf84b, 90 },
    { 0xa66bf84c, 76 },
    { 0x55bf84d, 33 },
    { 0, 0 },
    { 0xe5abf84f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x171bf852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bbf85b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0bf860, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fbf86c, 85 },
    { 0xfbcbf86c, 55 },
    { 0, 0 },
    { 0xd26bf86f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66abf874, 98 },
    { 0x4e3bf875, 34 },
    { 0x563bf875, 76 },
    { 0, 0 },
    { 0x1b2bf878, 8 },
    { 0x336bf878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05bf87c, 17 },
    { 0x51dbf87d, 76 },
    { 0x258bf87e, 98 },
    { 0, 0 },
    { 0x67fbf880, 61 },
    { 0, 0 },
    { 0x19bbf882, 90 },
    { 0, 0 },
    { 0x6edbf884, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985bf889, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04bf88c, 93 },
    { 0x340bf88d, 61 },
    { 0x26abf88d, 76 },
    { 0, 0 },
    { 0x2b2bf890, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bf895, 77 },
    { 0, 0 },
    { 0xbbbbf897, 100 },
    { 0x70abf898, 91 },
    { 0x50bbf897, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435bf89e, 88 },
    { 0x670bf89f, 55 },
    { 0x815bf8a0, 53 },
    { 0, 0 },
    { 0x364bf8a2, 77 },
    { 0, 0 },
    { 0x5dbbf8a4, 100 },
    { 0x359bf8a4, 88 },
    { 0x3fdbf8a5, 68 },
    { 0xcd2bf8a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9bf8af, 55 },
    { 0x968bf8b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x776bf8b3, 55 },
    { 0, 0 },
    { 0x1a8bf8b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ebf8b8, 85 },
    { 0, 0 },
    { 0x997bf8ba, 76 },
    { 0x42dbf8ba, 76 },
    { 0x7a9bf8bc, 95 },
    { 0x490bf8bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17bf8c3, 90 },
    { 0x183bf8c4, 78 },
    { 0x2a5bf8c5, 52 },
    { 0x2dfbf8c5, 76 },
    { 0x3c9bf8c7, 97 },
    { 0x776bf8c8, 52 },
    { 0x992bf8c9, 93 },
    { 0xfbdbf8c8, 34 },
    { 0xbbfbf8cb, 46 },
    { 0, 0 },
    { 0xe2fbf8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5bf8d4, 78 },
    { 0xbcfbf8d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18bf8d8, 55 },
    { 0xa3fbf8d9, 61 },
    { 0xa8cbf8da, 46 },
    { 0x5e5bf8db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cbf8e0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ebf8e8, 90 },
    { 0, 0 },
    { 0xfd0bf8ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4bf8ef, 55 },
    { 0x99bbf8ef, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x73abf8f3, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8bf8f6, 87 },
    { 0x8c2bf8f7, 34 },
    { 0xd8ebf8f8, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ebf8fe, 60 },
    { 0, 0 },
    { 0x6d6bf900, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bf904, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37bf90a, 68 },
    { 0xf2dbf90b, 23 },
    { 0x401bf90b, 84 },
    { 0x40bbf90c, 90 },
    { 0x796bf90c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9bf911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748bf91b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bf920, 99 },
    { 0x158bf921, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48bf92f, 33 },
    { 0x608bf92f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dbf934, 88 },
    { 0, 0 },
    { 0x4f4bf936, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35bf939, 66 },
    { 0x8ebbf939, 81 },
    { 0x56bf93a, 55 },
    { 0, 0 },
    { 0x539bf93d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffbf943, 98 },
    { 0x709bf944, 33 },
    { 0xb56bf944, 90 },
    { 0x44abf946, 77 },
    { 0x6c6bf946, 33 },
    { 0x22fbf945, 17 },
    { 0xe1bbf948, 23 },
    { 0x84dbf945, 100 },
    { 0xcc8bf94b, 55 },
    { 0x440bf94c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fbf950, 95 },
    { 0x912bf950, 43 },
    { 0x8a3bf951, 33 },
    { 0xbe9bf953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dbf956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1bf959, 90 },
    { 0, 0 },
    { 0x98bf95b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20bf95f, 61 },
    { 0, 0 },
    { 0x8efbf961, 55 },
    { 0x210bf961, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193bf967, 34 },
    { 0, 0 },
    { 0xfffbf969, 61 },
    { 0x6acbf96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675bf970, 90 },
    { 0x8c2bf971, 23 },
    { 0, 0 },
    { 0xfefbf973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf978, 85 },
    { 0, 0 },
    { 0x730bf97a, 61 },
    { 0xe69bf97b, 67 },
    { 0, 0 },
    { 0xd0dbf97d, 90 },
    { 0, 0 },
    { 0xf45bf97f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44bf986, 34 },
    { 0, 0 },
    { 0x8c6bf988, 33 },
    { 0x826bf989, 55 },
    { 0xd6abf98a, 48 },
    { 0x569bf989, 90 },
    { 0x1efbf98c, 61 },
    { 0, 0 },
    { 0x31fbf98e, 61 },
    { 0x596bf98f, 90 },
    { 0xbc1bf98f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874bf995, 98 },
    { 0, 0 },
    { 0xe5dbf997, 23 },
    { 0x750bf997, 67 },
    { 0, 0 },
    { 0xb20bf99a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565bf99e, 95 },
    { 0x13fbf99e, 43 },
    { 0x26cbf9a0, 78 },
    { 0xf9bbf9a1, 99 },
    { 0, 0 },
    { 0x5abf9a3, 61 },
    { 0, 0 },
    { 0x43dbf9a5, 94 },
    { 0x84ebf9a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1bf9a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf9ad, 38 },
    { 0x3debf9ad, 34 },
    { 0x6d1bf9ae, 95 },
    { 0xd9bbf9b0, 76 },
    { 0, 0 },
    { 0x2e4bf9b2, 76 },
    { 0x161bf9b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74bf9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3bf9bb, 48 },
    { 0xfdcbf9bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825bf9c2, 61 },
    { 0xe4fbf9c2, 66 },
    { 0xc2cbf9c4, 78 },
    { 0, 0 },
    { 0xc6bbf9c6, 23 },
    { 0, 0 },
    { 0x335bf9c8, 61 },
    { 0xa11bf9c9, 93 },
    { 0, 0 },
    { 0xd0dbf9cb, 90 },
    { 0, 0 },
    { 0x9b2bf9cd, 55 },
    { 0x161bf9ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1febf9d4, 95 },
    { 0xc37bf9d5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fbf9db, 34 },
    { 0x95bbf9db, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766bf9e0, 33 },
    { 0, 0 },
    { 0x9bfbf9e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8abf9e8, 87 },
    { 0xcb9bf9e8, 34 },
    { 0xacfbf9ea, 61 },
    { 0, 0 },
    { 0xfe6bf9ec, 78 },
    { 0xf9fbf9ed, 90 },
    { 0xbd1bf9ee, 23 },
    { 0, 0 },
    { 0x2bbbf9f0, 90 },
    { 0xd91bf9f1, 23 },
    { 0x8e3bf9f2, 33 },
    { 0, 0 },
    { 0xff2bf9f4, 66 },
    { 0x181bf9f5, 33 },
    { 0xb5ebf9f5, 99 },
    { 0, 0 },
    { 0x169bf9f8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8bf9ff, 77 },
    { 0x99fbfa00, 93 },
    { 0x7f8bfa01, 55 },
    { 0x701bfa02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbfa0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804bfa11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21abfa15, 76 },
    { 0, 0 },
    { 0xefdbfa17, 34 },
    { 0, 0 },
    { 0x529bfa19, 33 },
    { 0xf89bfa1a, 98 },
    { 0x3bfa1a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08bfa1e, 66 },
    { 0x623bfa1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65abfa2c, 81 },
    { 0, 0 },
    { 0x20fbfa2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848bfa33, 67 },
    { 0x758bfa33, 90 },
    { 0xb8cbfa35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dbfa3e, 33 },
    { 0x9dbfa3e, 33 },
    { 0x1c7bfa3f, 33 },
    { 0xf0fbfa3f, 98 },
    { 0xb2bfa42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13abfa45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6bfa4a, 100 },
    { 0xa2cbfa4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5bfa4e, 76 },
    { 0, 0 },
    { 0x6a1bfa50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0bfa53, 90 },
    { 0x161bfa54, 95 },
    { 0x87fbfa55, 98 },
    { 0, 0 },
    { 0x89bfa57, 17 },
    { 0x523bfa57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3bfa5e, 61 },
    { 0xc53bfa5f, 88 },
    { 0, 0 },
    { 0xf02bfa61, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fbfa68, 93 },
    { 0x521bfa69, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772bfa6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535bfa75, 33 },
    { 0x59bfa76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5bfa7b, 52 },
    { 0x395bfa7c, 90 },
    { 0, 0 },
    { 0x93ebfa7e, 52 },
    { 0xa77bfa7f, 95 },
    { 0xe76bfa80, 91 },
    { 0xd76bfa81, 76 },
    { 0, 0 },
    { 0x411bfa83, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29bfa8c, 76 },
    { 0xcdbbfa8d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940bfa94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bbfa97, 34 },
    { 0x313bfa98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efbfa9b, 55 },
    { 0x82abfa9c, 87 },
    { 0xbabbfa9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3bfaa0, 95 },
    { 0x51cbfaa1, 34 },
    { 0x798bfaa2, 95 },
    { 0x720bfaa3, 23 },
    { 0x6c5bfaa4, 33 },
    { 0x2f6bfaa5, 33 },
    { 0, 0 },
    { 0x3a8bfaa7, 76 },
    { 0, 0 },
    { 0x890bfaa9, 69 },
    { 0x49dbfaaa, 76 },
    { 0xc18bfaab, 77 },
    { 0x92abfaac, 46 },
    { 0x989bfaaa, 93 },
    { 0, 0 },
    { 0xdbbbfaaf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dbfabb, 76 },
    { 0x471bfabb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1bfabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2bfac2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42abfac7, 55 },
    { 0x85cbfac7, 23 },
    { 0x55fbfac7, 98 },
    { 0xe5ebfaca, 34 },
    { 0xaddbfacb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bbfacf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9bfad2, 53 },
    { 0, 0 },
    { 0xfa7bfad4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x175bfad7, 23 },
    { 0, 0 },
    { 0x8f2bfad9, 90 },
    { 0, 0 },
    { 0x50fbfadb, 91 },
    { 0xcc0bfadc, 33 },
    { 0xa93bfadc, 98 },
    { 0x297bfadc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccbfae6, 93 },
    { 0x16cbfae7, 43 },
    { 0xae2bfae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x224bfaeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609bfaef, 90 },
    { 0x440bfaf0, 34 },
    { 0x55cbfaf1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcbfaf6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92bfaf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4bfb01, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfbfb07, 61 },
    { 0x701bfb07, 98 },
    { 0xd2bbfb07, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbbfb0d, 95 },
    { 0x44fbfb0d, 61 },
    { 0x7e9bfb0f, 90 },
    { 0xee0bfb0d, 98 },
    { 0x61bfb11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2bfb17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97bfb1c, 23 },
    { 0, 0 },
    { 0x4b2bfb1e, 98 },
    { 0, 0 },
    { 0xe7cbfb20, 90 },
    { 0x6eebfb20, 90 },
    { 0x47bfb21, 55 },
    { 0xc62bfb20, 93 },
    { 0x2fdbfb24, 40 },
    { 0xcadbfb25, 95 },
    { 0xa2fbfb26, 52 },
    { 0x511bfb27, 95 },
    { 0x83cbfb28, 43 },
    { 0x9edbfb29, 33 },
    { 0x923bfb26, 34 },
    { 0xc09bfb2a, 61 },
    { 0xc7cbfb27, 98 },
    { 0x3acbfb26, 61 },
    { 0, 0 },
    { 0xd22bfb2f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafbfb33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357bfb37, 49 },
    { 0, 0 },
    { 0x84fbfb39, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dbfb3e, 93 },
    { 0x8aabfb3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cbfb43, 33 },
    { 0x1e7bfb43, 61 },
    { 0x9e0bfb45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afbfb50, 90 },
    { 0xbaabfb51, 75 },
    { 0x777bfb52, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8bfb58, 61 },
    { 0, 0 },
    { 0xce6bfb5a, 90 },
    { 0x87cbfb5b, 78 },
    { 0, 0 },
    { 0x37bfb5d, 95 },
    { 0x213bfb5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbfb64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42bfb67, 33 },
    { 0xedfbfb68, 23 },
    { 0xeaebfb68, 90 },
    { 0x6d1bfb68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868bfb6e, 46 },
    { 0xdc8bfb6f, 33 },
    { 0x1d7bfb70, 90 },
    { 0, 0 },
    { 0x53bbfb72, 46 },
    { 0xdb4bfb72, 74 },
    { 0x6a2bfb74, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3bfb7a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96bfb7f, 40 },
    { 0x1e0bfb80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69bfb87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bbfb8a, 76 },
    { 0x202bfb8b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dbfb8e, 60 },
    { 0, 0 },
    { 0xd6bbfb90, 23 },
    { 0, 0 },
    { 0x39abfb92, 90 },
    { 0x2f5bfb92, 98 },
    { 0x199bfb94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392bfb9f, 98 },
    { 0xf9bbfb9f, 60 },
    { 0, 0 },
    { 0x1fbfba2, 78 },
    { 0xba3bfba3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948bfba7, 34 },
    { 0x393bfba7, 33 },
    { 0x7b5bfba9, 98 },
    { 0x34cbfbaa, 33 },
    { 0x667bfbaa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4bfbae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bbfbb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36bfbb6, 95 },
    { 0, 0 },
    { 0x936bfbb8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cbfbbf, 90 },
    { 0, 0 },
    { 0x6b9bfbc1, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18bfbc4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9bfbcd, 98 },
    { 0, 0 },
    { 0x155bfbcf, 43 },
    { 0, 0 },
    { 0xe05bfbd1, 46 },
    { 0x8b8bfbd2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1bfbdf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x757bfbe2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbfbf0, 98 },
    { 0x91ebfbf1, 23 },
    { 0x34bbfbf2, 61 },
    { 0xacabfbf3, 87 },
    { 0xf5dbfbf4, 90 },
    { 0xbf6bfbf5, 23 },
    { 0x269bfbf3, 95 },
    { 0xd34bfbf2, 96 },
    { 0xf72bfbf8, 98 },
    { 0x895bfbf8, 76 },
    { 0, 0 },
    { 0xcfabfbfb, 33 },
    { 0, 0 },
    { 0xa4bfbfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56bfc00, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cbfc04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5febfc0a, 95 },
    { 0x366bfc0b, 23 },
    { 0xd59bfc0b, 98 },
    { 0xa33bfc0d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdbfc10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42bfc14, 55 },
    { 0xa22bfc15, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544bfc1f, 34 },
    { 0x126bfc1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6bfc24, 61 },
    { 0x165bfc25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104bfc2b, 98 },
    { 0xbc2bfc2c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x490bfc2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cbfc32, 93 },
    { 0x489bfc32, 65 },
    { 0xceebfc34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x390bfc37, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x726bfc3a, 38 },
    { 0x92cbfc3a, 95 },
    { 0x977bfc3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddbfc3f, 90 },
    { 0x351bfc40, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5bfc43, 90 },
    { 0x8ddbfc44, 90 },
    { 0x9bcbfc45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797bfc4a, 90 },
    { 0xf55bfc4a, 100 },
    { 0x662bfc4b, 93 },
    { 0, 0 },
    { 0x310bfc4e, 92 },
    { 0x6e5bfc4f, 87 },
    { 0x3d2bfc50, 36 },
    { 0x9ecbfc51, 81 },
    { 0xf5fbfc51, 34 },
    { 0xac7bfc53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67bfc56, 75 },
    { 0x7a1bfc57, 90 },
    { 0x339bfc58, 66 },
    { 0x643bfc59, 98 },
    { 0x6ddbfc5a, 34 },
    { 0x382bfc5a, 76 },
    { 0xfedbfc5a, 23 },
    { 0x7c4bfc59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874bfc62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x153bfc65, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bfc70, 61 },
    { 0xcabbfc71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24bfc76, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfbfc7f, 23 },
    { 0xa38bfc80, 90 },
    { 0xe55bfc81, 61 },
    { 0xe95bfc82, 76 },
    { 0, 0 },
    { 0x3f3bfc84, 76 },
    { 0x743bfc84, 95 },
    { 0xd48bfc86, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bfc8c, 87 },
    { 0, 0 },
    { 0xeb4bfc8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597bfc94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730bfc99, 43 },
    { 0x75bbfc9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48bfc9e, 34 },
    { 0, 0 },
    { 0x210bfca0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bfca4, 51 },
    { 0x7c1bfca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88bfcad, 23 },
    { 0, 0 },
    { 0x127bfcaf, 95 },
    { 0xe38bfcaf, 34 },
    { 0x21abfcb1, 33 },
    { 0xe2abfcb2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09bfcb8, 100 },
    { 0x441bfcb9, 90 },
    { 0x522bfcba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59bfcbf, 92 },
    { 0xed1bfcbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9bfcc7, 98 },
    { 0, 0 },
    { 0xc0cbfcc9, 98 },
    { 0, 0 },
    { 0x776bfccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1bfcce, 46 },
    { 0x1c9bfccf, 96 },
    { 0xbf6bfccf, 76 },
    { 0x721bfcd1, 12 },
    { 0x387bfccf, 90 },
    { 0x517bfcd3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbbfcd6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbbfcdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdbfce3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178bfce8, 94 },
    { 0xad1bfce9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2bfcec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740bfcf6, 52 },
    { 0xae7bfcf7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01bfcfe, 90 },
    { 0, 0 },
    { 0x150bfd00, 23 },
    { 0x826bfd01, 90 },
    { 0, 0 },
    { 0x163bfd03, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77bfd08, 55 },
    { 0x610bfd09, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99bfd15, 43 },
    { 0, 0 },
    { 0x8cbbfd17, 84 },
    { 0xf78bfd18, 85 },
    { 0x97fbfd19, 87 },
    { 0x3bfbfd18, 61 },
    { 0x572bfd1b, 81 },
    { 0xf4cbfd1a, 17 },
    { 0x9b9bfd1d, 19 },
    { 0, 0 },
    { 0x231bfd1f, 99 },
    { 0x1b9bfd1f, 75 },
    { 0x20abfd20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7febfd25, 61 },
    { 0x878bfd26, 96 },
    { 0, 0 },
    { 0x1d3bfd28, 84 },
    { 0xdbbbfd28, 90 },
    { 0x5aebfd29, 64 },
    { 0xe19bfd2b, 33 },
    { 0xd54bfd2b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52bfd2f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9bfd32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5bfd35, 90 },
    { 0x3fbbfd36, 55 },
    { 0x5cbfd36, 61 },
    { 0xa1fbfd38, 67 },
    { 0x908bfd37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dbfd3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0bfd45, 91 },
    { 0x19bfd46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddbfd49, 76 },
    { 0x966bfd4a, 61 },
    { 0, 0 },
    { 0x330bfd4c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0bfd51, 98 },
    { 0x7f8bfd51, 33 },
    { 0xa6dbfd53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dbfd57, 98 },
    { 0, 0 },
    { 0x7e6bfd59, 36 },
    { 0x5f5bfd59, 55 },
    { 0, 0 },
    { 0xd09bfd5c, 78 },
    { 0x40abfd5c, 55 },
    { 0xa54bfd5c, 90 },
    { 0x23bfd5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cbfd64, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fbfd67, 23 },
    { 0, 0 },
    { 0xf3abfd69, 61 },
    { 0x86fbfd6a, 76 },
    { 0x3e0bfd6b, 98 },
    { 0, 0 },
    { 0x298bfd6d, 43 },
    { 0, 0 },
    { 0xc45bfd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4bfd76, 33 },
    { 0xc9bfd76, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6bfd81, 23 },
    { 0x96cbfd82, 52 },
    { 0xbb6bfd83, 61 },
    { 0xf32bfd83, 35 },
    { 0, 0 },
    { 0x3f4bfd86, 98 },
    { 0x42bbfd87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aabfd8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdbfd8f, 85 },
    { 0x2f2bfd90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbebfd94, 33 },
    { 0x47abfd95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3bfd99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x813bfd9c, 93 },
    { 0x815bfd9c, 66 },
    { 0xfcbbfd9e, 90 },
    { 0xfe6bfd9f, 61 },
    { 0x456bfda0, 98 },
    { 0xc90bfda1, 95 },
    { 0x410bfda2, 87 },
    { 0x9c9bfd9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ebfda8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735bfdac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77bfdb4, 43 },
    { 0, 0 },
    { 0x911bfdb6, 23 },
    { 0x7b1bfdb7, 87 },
    { 0xabbfdb7, 33 },
    { 0xdebfdb9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcbfdc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766bfdc7, 75 },
    { 0xcaabfdc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9bfdcd, 90 },
    { 0xe67bfdcd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aabfdd1, 90 },
    { 0xd66bfdd2, 90 },
    { 0x518bfdd3, 93 },
    { 0xcf8bfdd1, 96 },
    { 0xc3cbfdd5, 43 },
    { 0xd02bfdd5, 43 },
    { 0x12bfdd7, 100 },
    { 0x60fbfdd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bbfddb, 84 },
    { 0x417bfddc, 87 },
    { 0xa73bfddc, 90 },
    { 0xffcbfdde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddbfde2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402bfde7, 76 },
    { 0, 0 },
    { 0xa62bfde9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbabfdf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44abfdf4, 78 },
    { 0xadfbfdf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebbfdf8, 61 },
    { 0x616bfdf8, 43 },
    { 0x831bfdfa, 61 },
    { 0x6b3bfdfb, 90 },
    { 0xe72bfdfc, 90 },
    { 0x566bfdf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80bfe01, 90 },
    { 0x9d9bfe02, 76 },
    { 0x94bbfe02, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbfe07, 46 },
    { 0, 0 },
    { 0x949bfe09, 56 },
    { 0, 0 },
    { 0x7febfe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7babfe0e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1bfe12, 61 },
    { 0xc60bfe12, 55 },
    { 0xb32bfe13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95bfe1a, 95 },
    { 0, 0 },
    { 0x979bfe1c, 67 },
    { 0x70fbfe1d, 61 },
    { 0x87fbfe1e, 23 },
    { 0x380bfe1e, 65 },
    { 0xfdcbfe1f, 91 },
    { 0, 0 },
    { 0xda4bfe22, 81 },
    { 0x1ccbfe22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69bfe27, 90 },
    { 0x506bfe27, 95 },
    { 0xe12bfe28, 56 },
    { 0xfb7bfe2a, 55 },
    { 0, 0 },
    { 0x65bbfe2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbbfe30, 76 },
    { 0xf5ebfe30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917bfe36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcbfe3a, 61 },
    { 0, 0 },
    { 0xb8abfe3c, 61 },
    { 0, 0 },
    { 0x84cbfe3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9bfe45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x808bfe48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26bfe4c, 43 },
    { 0, 0 },
    { 0x78dbfe4e, 33 },
    { 0, 0 },
    { 0xa20bfe50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47bfe59, 64 },
    { 0xc3fbfe5a, 34 },
    { 0xbd8bfe59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834bfe5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41bfe65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbbfe6b, 90 },
    { 0, 0 },
    { 0x208bfe6d, 34 },
    { 0x5ecbfe6e, 33 },
    { 0x1f1bfe6f, 33 },
    { 0x17dbfe6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cbfe73, 23 },
    { 0x80fbfe74, 46 },
    { 0xb66bfe75, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7bfe78, 99 },
    { 0xe81bfe79, 33 },
    { 0x2e4bfe7a, 90 },
    { 0xdfcbfe79, 85 },
    { 0x41dbfe7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0bfe7f, 99 },
    { 0, 0 },
    { 0x1ccbfe81, 98 },
    { 0, 0 },
    { 0x338bfe83, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcbfe86, 95 },
    { 0xfa2bfe86, 33 },
    { 0x309bfe88, 93 },
    { 0, 0 },
    { 0xf1dbfe8a, 90 },
    { 0xbe7bfe8b, 67 },
    { 0x249bfe8b, 75 },
    { 0x309bfe8c, 98 },
    { 0xad7bfe8d, 55 },
    { 0, 0 },
    { 0xb0ebfe90, 61 },
    { 0x5a5bfe91, 90 },
    { 0x4d3bfe92, 33 },
    { 0xa4dbfe93, 33 },
    { 0x703bfe94, 100 },
    { 0x772bfe93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dbfe98, 90 },
    { 0x9b1bfe99, 46 },
    { 0x699bfe99, 98 },
    { 0x92bbfe9b, 23 },
    { 0xdfdbfe9b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbfea1, 61 },
    { 0, 0 },
    { 0xf1cbfea3, 33 },
    { 0xeb9bfea4, 98 },
    { 0x8a1bfea5, 96 },
    { 0, 0 },
    { 0xf81bfea7, 88 },
    { 0, 0 },
    { 0x601bfea9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66bfead, 61 },
    { 0x8e9bfeae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64abfeb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4bfeb8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dbfebd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796bfec4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348bfecb, 34 },
    { 0xb24bfecc, 33 },
    { 0xcdfbfecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428bfed2, 93 },
    { 0xaa1bfed3, 33 },
    { 0xa49bfed2, 90 },
    { 0x642bfed5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8bfedc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242bfee0, 70 },
    { 0, 0 },
    { 0x6aabfee2, 84 },
    { 0, 0 },
    { 0x29abfee4, 55 },
    { 0x602bfee4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6bfee8, 95 },
    { 0, 0 },
    { 0xda9bfeea, 95 },
    { 0xf0bfeeb, 33 },
    { 0x25ebfeeb, 90 },
    { 0xb00bfeed, 66 },
    { 0xd94bfeed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85bfef6, 66 },
    { 0x82cbfef7, 43 },
    { 0, 0 },
    { 0x63ebfef9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26bfefd, 87 },
    { 0, 0 },
    { 0x592bfeff, 80 },
    { 0xc8abfeff, 95 },
    { 0x835bff01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100bff07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x717bff0a, 39 },
    { 0xe19bff0a, 23 },
    { 0x558bff0b, 77 },
    { 0x86bff0d, 61 },
    { 0x38dbff0a, 23 },
    { 0x525bff0f, 61 },
    { 0xceabff0b, 98 },
    { 0xe74bff11, 17 },
    { 0x922bff12, 33 },
    { 0, 0 },
    { 0xa8cbff14, 34 },
    { 0x4a2bff14, 33 },
    { 0x476bff16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dbff1a, 23 },
    { 0x6e3bff1b, 90 },
    { 0x942bff1c, 76 },
    { 0xd26bff1d, 78 },
    { 0xdf2bff1d, 34 },
    { 0x60dbff1f, 24 },
    { 0xea5bff20, 95 },
    { 0x234bff1b, 33 },
    { 0xde8bff22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacbff2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dbff30, 23 },
    { 0x642bff31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffbff36, 33 },
    { 0x841bff36, 61 },
    { 0xe41bff38, 61 },
    { 0xa2dbff39, 76 },
    { 0x3c4bff38, 61 },
    { 0xe20bff39, 68 },
    { 0x337bff39, 33 },
    { 0xefbbff3d, 67 },
    { 0xb47bff3d, 95 },
    { 0x214bff3b, 33 },
    { 0, 0 },
    { 0x5f0bff41, 26 },
    { 0xa72bff42, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbbff49, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9bff50, 61 },
    { 0x66bff51, 98 },
    { 0, 0 },
    { 0x985bff53, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabff56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cbff5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ebff5d, 90 },
    { 0x8d5bff5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1bff62, 90 },
    { 0x64abff63, 61 },
    { 0xe59bff64, 90 },
    { 0xab9bff62, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x522bff68, 98 },
    { 0xc8ebff69, 23 },
    { 0xa30bff68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bbff72, 98 },
    { 0, 0 },
    { 0xdb8bff74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0bff77, 61 },
    { 0xccfbff78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebbff7c, 23 },
    { 0xbb1bff7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4bff81, 91 },
    { 0, 0 },
    { 0x99bbff83, 34 },
    { 0x7b2bff83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93bff88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dbff8c, 95 },
    { 0x874bff8d, 17 },
    { 0xd64bff8e, 55 },
    { 0x9e6bff8f, 90 },
    { 0, 0 },
    { 0x3f5bff91, 34 },
    { 0, 0 },
    { 0x959bff93, 90 },
    { 0x407bff94, 78 },
    { 0, 0 },
    { 0x425bff96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cbff99, 23 },
    { 0xb98bff9a, 86 },
    { 0x3ecbff9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6debffa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138bffaa, 90 },
    { 0xc3abffaa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6bffb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1bffbc, 76 },
    { 0, 0 },
    { 0x135bffbe, 88 },
    { 0x74abffbf, 61 },
    { 0x289bffc0, 98 },
    { 0xfffbffbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ebffc4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fabffc7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebffd4, 87 },
    { 0x39bffd4, 33 },
    { 0xdfebffd4, 61 },
    { 0x5a2bffd7, 90 },
    { 0x419bffd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4bffdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fbffe2, 76 },
    { 0x4f3bffe2, 90 },
    { 0xaa2bffe2, 76 },
    { 0, 0 },
    { 0x67bbffe6, 83 },
    { 0, 0 },
    { 0xa5abffe8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7bfff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bbfff8, 34 },
    { 0xdb8bfff9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabfffd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc0000, 66 },
    { 0x1b5c0000, 84 },
    { 0xc80c0002, 95 },
    { 0x637c0001, 95 },
    { 0x242c0002, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5c0008, 93 },
    { 0xcd6c0009, 33 },
    { 0, 0 },
    { 0x138c000b, 95 },
    { 0, 0 },
    { 0x6ccc000d, 33 },
    { 0x4f7c000e, 76 },
    { 0xd18c000f, 98 },
    { 0x24ec000e, 77 },
    { 0x614c0011, 23 },
    { 0x160c0011, 99 },
    { 0x220c000e, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x361c0016, 96 },
    { 0xcf5c0017, 43 },
    { 0xbc5c0018, 33 },
    { 0xc69c0017, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496c0021, 90 },
    { 0x657c0022, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0c0026, 100 },
    { 0x753c0026, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c002d, 90 },
    { 0x687c002d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cc0033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dac0038, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cc0049, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eac004e, 61 },
    { 0, 0 },
    { 0xf6bc0050, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464c0062, 23 },
    { 0xa9ec0062, 23 },
    { 0, 0 },
    { 0xb27c0065, 46 },
    { 0xa4fc0066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c0069, 61 },
    { 0, 0 },
    { 0x4a8c006b, 34 },
    { 0xec9c006b, 78 },
    { 0xecbc006d, 46 },
    { 0xd83c006e, 95 },
    { 0xd92c006f, 34 },
    { 0x943c006e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3c0073, 43 },
    { 0, 0 },
    { 0xceac0075, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5c007c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614c008c, 90 },
    { 0x1abc008c, 66 },
    { 0, 0 },
    { 0x7ac008f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c0093, 34 },
    { 0x7ddc0093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc009c, 90 },
    { 0, 0 },
    { 0x700c009e, 52 },
    { 0xac3c009f, 34 },
    { 0, 0 },
    { 0x27cc00a1, 69 },
    { 0x88cc00a2, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503c00a6, 96 },
    { 0, 0 },
    { 0x5bcc00a8, 100 },
    { 0xc5bc00a8, 47 },
    { 0x398c00a9, 23 },
    { 0x3d4c00a9, 33 },
    { 0x30dc00ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cc00b0, 61 },
    { 0xffcc00b1, 95 },
    { 0x93cc00b1, 90 },
    { 0x48cc00b3, 46 },
    { 0x6d2c00b0, 90 },
    { 0xe57c00b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876c00be, 95 },
    { 0xc03c00be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44c00c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6c00ca, 95 },
    { 0x2e8c00cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06c00cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713c00d3, 98 },
    { 0, 0 },
    { 0x1ac00d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0c00dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c00e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6c00e5, 61 },
    { 0x3c5c00e6, 52 },
    { 0x198c00e6, 61 },
    { 0x53ec00e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63c00ef, 95 },
    { 0x92dc00f0, 76 },
    { 0x1bac00f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03c00f9, 30 },
    { 0, 0 },
    { 0x84ac00fb, 91 },
    { 0xc25c00fc, 90 },
    { 0x25ac00fd, 23 },
    { 0xc9dc00fc, 100 },
    { 0, 0 },
    { 0xd3bc0100, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eec0106, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6c0111, 90 },
    { 0, 0 },
    { 0x1c6c0113, 61 },
    { 0x362c0114, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x646c0117, 55 },
    { 0xb47c0117, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec011c, 78 },
    { 0, 0 },
    { 0xf27c011e, 46 },
    { 0, 0 },
    { 0x8c0120, 93 },
    { 0, 0 },
    { 0x192c0122, 33 },
    { 0xd62c0123, 23 },
    { 0x362c0123, 11 },
    { 0x6e7c0124, 95 },
    { 0x409c0126, 78 },
    { 0x295c0127, 95 },
    { 0xa1dc0128, 33 },
    { 0x954c0129, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dc012d, 100 },
    { 0x4dcc012e, 90 },
    { 0x199c012e, 55 },
    { 0x791c0130, 76 },
    { 0x52dc012e, 100 },
    { 0xf81c0132, 90 },
    { 0, 0 },
    { 0x72bc0134, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cc013a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60c0140, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x492c0143, 34 },
    { 0, 0 },
    { 0x4eac0145, 90 },
    { 0xc42c0146, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dac014d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c0150, 87 },
    { 0x3b9c0150, 55 },
    { 0xa3c0152, 33 },
    { 0, 0 },
    { 0x412c0154, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ec015a, 98 },
    { 0xc71c015a, 33 },
    { 0x839c015b, 95 },
    { 0x1b2c015d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dc0162, 90 },
    { 0xb27c0162, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc0166, 76 },
    { 0xb50c0167, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bc016b, 98 },
    { 0, 0 },
    { 0xb4fc016d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25c0172, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc017c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c0185, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x371c0188, 76 },
    { 0x6e9c0189, 67 },
    { 0xd7dc018a, 43 },
    { 0, 0 },
    { 0x560c018c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c0190, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec0193, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516c0197, 90 },
    { 0x28bc0198, 95 },
    { 0xa97c0197, 76 },
    { 0x751c0198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723c01a1, 93 },
    { 0x53c01a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478c01a6, 23 },
    { 0, 0 },
    { 0xd8c01a8, 48 },
    { 0, 0 },
    { 0x220c01aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x335c01ad, 98 },
    { 0x28fc01ae, 34 },
    { 0x621c01af, 76 },
    { 0x994c01ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dc01b7, 95 },
    { 0, 0 },
    { 0x510c01b9, 90 },
    { 0, 0 },
    { 0xfd1c01bb, 95 },
    { 0x978c01bb, 67 },
    { 0x9dbc01bd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1c01c0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375c01c4, 78 },
    { 0x3f6c01c4, 67 },
    { 0x68cc01c6, 33 },
    { 0xd6ec01c7, 78 },
    { 0x7bc01c4, 69 },
    { 0xea6c01c9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfc01d7, 33 },
    { 0x6ec01d8, 95 },
    { 0x31dc01d9, 65 },
    { 0x59bc01da, 33 },
    { 0, 0 },
    { 0x16dc01dc, 43 },
    { 0x459c01dc, 90 },
    { 0, 0 },
    { 0x8bc01df, 95 },
    { 0x11cc01e0, 66 },
    { 0x630c01e1, 93 },
    { 0xc63c01e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c01ff, 75 },
    { 0xae0c0200, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbc0203, 98 },
    { 0xb4cc0204, 33 },
    { 0x7c0c0203, 98 },
    { 0x555c0206, 100 },
    { 0xaaac0207, 90 },
    { 0, 0 },
    { 0x733c0209, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fc0210, 98 },
    { 0x15dc0211, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129c0216, 95 },
    { 0x4fec0217, 90 },
    { 0x7b2c0218, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c021f, 90 },
    { 0xd2fc021f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c0223, 90 },
    { 0x5e5c0224, 46 },
    { 0, 0 },
    { 0x208c0226, 46 },
    { 0x995c0227, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc022e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x704c0231, 52 },
    { 0xf4c0232, 36 },
    { 0x468c0233, 76 },
    { 0xd6ec0234, 34 },
    { 0, 0 },
    { 0x325c0236, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x932c0239, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5c0241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc0247, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec024d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfc0251, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeec0255, 61 },
    { 0xb00c0256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9c0259, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21c0262, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39c0268, 90 },
    { 0xaf5c0269, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66c026c, 95 },
    { 0xd9ec026d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dc0272, 67 },
    { 0x664c0273, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30c0283, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544c0288, 98 },
    { 0x8a4c0289, 33 },
    { 0x453c0289, 23 },
    { 0xbe4c028b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x386c028e, 55 },
    { 0x16ac028f, 90 },
    { 0x43c028e, 95 },
    { 0xc8ec028e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cac0294, 61 },
    { 0x1cbc0294, 93 },
    { 0x897c0295, 90 },
    { 0xc66c0297, 100 },
    { 0xfcc0297, 43 },
    { 0, 0 },
    { 0xf61c029a, 23 },
    { 0xb82c029a, 23 },
    { 0, 0 },
    { 0x547c029d, 46 },
    { 0x3ebc029d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92c02a1, 95 },
    { 0, 0 },
    { 0x93dc02a3, 76 },
    { 0x321c02a3, 46 },
    { 0, 0 },
    { 0xd49c02a6, 56 },
    { 0xdc8c02a6, 61 },
    { 0, 0 },
    { 0xd4cc02a9, 34 },
    { 0, 0 },
    { 0xb7bc02ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38c02ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adc02b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dc02b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9c02ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c02bd, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9c02c0, 61 },
    { 0x2e4c02c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10c02c4, 76 },
    { 0x3f7c02c5, 95 },
    { 0xb1ec02c5, 90 },
    { 0x5bac02c7, 100 },
    { 0x8b1c02c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1c02cb, 61 },
    { 0xb57c02cc, 87 },
    { 0, 0 },
    { 0x992c02ce, 90 },
    { 0, 0 },
    { 0xc3bc02d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2c02d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20c02d6, 90 },
    { 0x1d6c02d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c02dc, 65 },
    { 0x967c02dd, 61 },
    { 0xd2ac02de, 93 },
    { 0xde8c02df, 61 },
    { 0xebec02df, 66 },
    { 0x19ec02de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62c02e4, 78 },
    { 0, 0 },
    { 0xe80c02e6, 33 },
    { 0x6dac02e7, 90 },
    { 0x491c02e8, 93 },
    { 0xd66c02e9, 55 },
    { 0x5bfc02ea, 59 },
    { 0x322c02e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x396c02ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c02f2, 90 },
    { 0, 0 },
    { 0x2fc02f4, 98 },
    { 0x559c02f4, 67 },
    { 0xf8cc02f6, 33 },
    { 0x26c02f4, 61 },
    { 0x5e9c02f8, 66 },
    { 0x960c02f9, 33 },
    { 0xcacc02f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x364c02fd, 56 },
    { 0, 0 },
    { 0x5d5c02ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcc0304, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ac030b, 33 },
    { 0, 0 },
    { 0x551c030d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8c0313, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bc0317, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c031a, 34 },
    { 0x59ec031a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6c031e, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43c0322, 95 },
    { 0x68ac0322, 61 },
    { 0x687c0324, 76 },
    { 0x3fac0324, 69 },
    { 0x8e8c0326, 23 },
    { 0xba9c0324, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfc032b, 40 },
    { 0x737c032c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6c032f, 34 },
    { 0, 0 },
    { 0xb8c0331, 52 },
    { 0x171c0331, 33 },
    { 0xff9c0333, 100 },
    { 0x6d7c0332, 83 },
    { 0, 0 },
    { 0xe0cc0336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383c033e, 76 },
    { 0, 0 },
    { 0xccec0340, 76 },
    { 0x4d7c0340, 52 },
    { 0, 0 },
    { 0xe96c0343, 94 },
    { 0x454c0343, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c034a, 76 },
    { 0x1b6c034b, 90 },
    { 0x5cec034c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bc0352, 33 },
    { 0x618c0353, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06c035d, 34 },
    { 0, 0 },
    { 0x933c035f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c0367, 84 },
    { 0x326c0368, 76 },
    { 0xa4bc0369, 95 },
    { 0x538c0368, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1c036d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842c0371, 52 },
    { 0, 0 },
    { 0x423c0373, 78 },
    { 0, 0 },
    { 0xe00c0375, 90 },
    { 0xa73c0376, 38 },
    { 0x726c0376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ac037b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c037e, 95 },
    { 0x456c037f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdc0386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c0389, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44c0392, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fc0399, 23 },
    { 0xe36c0399, 93 },
    { 0xddec039b, 93 },
    { 0x60bc039c, 92 },
    { 0x5f7c039b, 33 },
    { 0x593c039e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1c03a6, 23 },
    { 0, 0 },
    { 0x919c03a8, 23 },
    { 0, 0 },
    { 0xaeac03aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3c03ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9c03b1, 33 },
    { 0xaedc03b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6c03b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5c03bf, 90 },
    { 0, 0 },
    { 0xa35c03c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c03c4, 43 },
    { 0x1b4c03c5, 67 },
    { 0x51fc03c6, 46 },
    { 0, 0 },
    { 0x54cc03c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825c03cf, 23 },
    { 0, 0 },
    { 0xbf7c03d1, 90 },
    { 0x98ec03d2, 51 },
    { 0xa86c03d2, 33 },
    { 0x518c03d4, 61 },
    { 0, 0 },
    { 0x80ac03d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bc03d9, 23 },
    { 0x3efc03da, 33 },
    { 0xc13c03db, 95 },
    { 0x45ec03dc, 34 },
    { 0x76c03dd, 90 },
    { 0x833c03de, 69 },
    { 0, 0 },
    { 0xe9dc03e0, 23 },
    { 0xf60c03e1, 77 },
    { 0xda6c03e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bc03ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69c03f1, 64 },
    { 0x6a8c03f1, 76 },
    { 0x9f1c03f2, 23 },
    { 0x9bdc03f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ac03fa, 84 },
    { 0xa10c03fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10c0400, 33 },
    { 0, 0 },
    { 0x316c0402, 90 },
    { 0, 0 },
    { 0xb9ac0404, 34 },
    { 0, 0 },
    { 0x1cec0406, 33 },
    { 0, 0 },
    { 0xe92c0408, 92 },
    { 0xdd1c0409, 34 },
    { 0x26fc040a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ac040f, 90 },
    { 0x16ec0410, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fc0415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fc0418, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac041b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84c041f, 81 },
    { 0, 0 },
    { 0xa69c0421, 43 },
    { 0, 0 },
    { 0xa2c0423, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c0428, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1c042b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ac0431, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ec0435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991c0439, 38 },
    { 0x256c043a, 33 },
    { 0, 0 },
    { 0x1dcc043c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0c0443, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c0447, 61 },
    { 0, 0 },
    { 0x318c0449, 52 },
    { 0x998c0449, 17 },
    { 0x3eec044b, 66 },
    { 0xc71c044b, 95 },
    { 0xfefc044b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27c0450, 61 },
    { 0x5d4c0451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47c0455, 53 },
    { 0x852c0456, 43 },
    { 0x694c0457, 61 },
    { 0x1acc0457, 23 },
    { 0x4eac0459, 43 },
    { 0x464c0457, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9c045e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5c0463, 69 },
    { 0, 0 },
    { 0x52dc0465, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dc0469, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4c0470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c0473, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bec047c, 91 },
    { 0xa6fc047d, 100 },
    { 0x189c047e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacac0484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9c0487, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477c0491, 98 },
    { 0, 0 },
    { 0xbcbc0493, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c0496, 66 },
    { 0xa3bc0497, 61 },
    { 0xa5ec0497, 91 },
    { 0xa9c0497, 23 },
    { 0xaabc0497, 52 },
    { 0xe66c049a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c049e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1c04a3, 55 },
    { 0x329c04a4, 61 },
    { 0xf0ac04a4, 95 },
    { 0, 0 },
    { 0x769c04a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6c04ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ec04ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17c04b2, 33 },
    { 0, 0 },
    { 0xc5bc04b4, 99 },
    { 0x7e4c04b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06c04ba, 90 },
    { 0, 0 },
    { 0xb85c04bc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85c04bf, 76 },
    { 0x456c04bf, 95 },
    { 0xfdbc04c0, 61 },
    { 0x2d2c04c2, 90 },
    { 0x619c04c3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfc04c6, 33 },
    { 0, 0 },
    { 0xf1bc04c8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ac04cb, 55 },
    { 0x1b5c04cc, 98 },
    { 0xde3c04cd, 96 },
    { 0, 0 },
    { 0x107c04cf, 40 },
    { 0x738c04d0, 90 },
    { 0xe1c04cf, 76 },
    { 0, 0 },
    { 0x33c04d3, 23 },
    { 0xd03c04d3, 23 },
    { 0, 0 },
    { 0x520c04d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8c04de, 43 },
    { 0xf9bc04de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bc04e4, 90 },
    { 0x644c04e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c04ea, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dc04ee, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ec04f9, 90 },
    { 0x9ccc04fa, 76 },
    { 0x30bc04fb, 36 },
    { 0xa89c04fb, 98 },
    { 0xcc6c04fd, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0c0502, 23 },
    { 0x849c0503, 95 },
    { 0x31ac0503, 34 },
    { 0xd77c0503, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4c0509, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5c050d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fc0511, 34 },
    { 0, 0 },
    { 0xee0c0513, 64 },
    { 0x4c3c0514, 90 },
    { 0x3d9c0514, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c051b, 90 },
    { 0x461c051c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951c0522, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fc0526, 98 },
    { 0, 0 },
    { 0x810c0528, 95 },
    { 0xa1fc0528, 98 },
    { 0xd1ac0528, 61 },
    { 0x936c052b, 61 },
    { 0x21cc052c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ac0532, 90 },
    { 0x9e9c0533, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fc0537, 46 },
    { 0, 0 },
    { 0xda2c0539, 90 },
    { 0xd75c0539, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0c053d, 33 },
    { 0xff2c053d, 98 },
    { 0x395c053f, 43 },
    { 0, 0 },
    { 0x6e9c0541, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2c0545, 55 },
    { 0x1f9c0545, 61 },
    { 0x6ecc0547, 33 },
    { 0xfd6c0548, 61 },
    { 0xdfbc0548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cac054c, 95 },
    { 0xee4c054d, 33 },
    { 0xac2c054e, 76 },
    { 0xc03c054c, 23 },
    { 0x539c0550, 61 },
    { 0x771c0550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c0556, 85 },
    { 0, 0 },
    { 0xbcfc0558, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebc055b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1c0560, 90 },
    { 0x6f2c0561, 23 },
    { 0x7c9c0561, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c0565, 33 },
    { 0x587c0566, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945c056b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90c0570, 90 },
    { 0x6b1c0571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dc0577, 90 },
    { 0x56ac0578, 95 },
    { 0x17bc0578, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedc057d, 23 },
    { 0x66bc057e, 33 },
    { 0x370c057f, 61 },
    { 0xec8c0580, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85c0584, 33 },
    { 0, 0 },
    { 0xd32c0586, 33 },
    { 0x6a7c0587, 61 },
    { 0, 0 },
    { 0x5c6c0589, 66 },
    { 0xf85c058a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6c0591, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fc0594, 84 },
    { 0xeabc0595, 66 },
    { 0xd36c0596, 100 },
    { 0x64ac0597, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ac059a, 61 },
    { 0xcbac059b, 90 },
    { 0, 0 },
    { 0x74bc059d, 93 },
    { 0x3ebc059e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77c05a9, 33 },
    { 0, 0 },
    { 0x8cc05ab, 95 },
    { 0, 0 },
    { 0xd3ec05ad, 95 },
    { 0x506c05ae, 76 },
    { 0x933c05ad, 23 },
    { 0xbd6c05b0, 77 },
    { 0x45bc05b1, 55 },
    { 0xed2c05b1, 76 },
    { 0x4b3c05ad, 34 },
    { 0xdb6c05b4, 55 },
    { 0x1adc05ad, 77 },
    { 0x3a0c05b5, 99 },
    { 0xc6ac05b7, 85 },
    { 0x49dc05b7, 95 },
    { 0x618c05b9, 90 },
    { 0x8b8c05ba, 61 },
    { 0x966c05b7, 23 },
    { 0x2a0c05bc, 82 },
    { 0x7edc05bc, 76 },
    { 0x6b9c05be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac05c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1c05cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50c05cf, 78 },
    { 0, 0 },
    { 0xf02c05d1, 33 },
    { 0x66dc05d2, 90 },
    { 0xfa0c05d2, 97 },
    { 0, 0 },
    { 0x98ec05d5, 68 },
    { 0xe09c05d5, 95 },
    { 0x2d2c05d7, 34 },
    { 0x8bec05d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9c05db, 61 },
    { 0x3a6c05dc, 93 },
    { 0, 0 },
    { 0x285c05de, 61 },
    { 0x73cc05df, 33 },
    { 0xaabc05e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9c05e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bc05e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcc05eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afc05f1, 90 },
    { 0x100c05f2, 91 },
    { 0x313c05f1, 61 },
    { 0x26c05f2, 75 },
    { 0x639c05f3, 55 },
    { 0, 0 },
    { 0x83ec05f7, 46 },
    { 0x20ec05f8, 100 },
    { 0x90cc05f9, 66 },
    { 0xbb8c05f9, 33 },
    { 0x616c05fb, 87 },
    { 0, 0 },
    { 0xf8dc05fd, 98 },
    { 0x4a0c05fe, 23 },
    { 0x22fc05ff, 33 },
    { 0x84cc0600, 78 },
    { 0xce8c05fe, 65 },
    { 0x9d4c0602, 23 },
    { 0x9fac0603, 23 },
    { 0x5a2c0604, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86c0609, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544c0610, 77 },
    { 0xdaac0610, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc0616, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ac061b, 98 },
    { 0xd95c061c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccac0620, 61 },
    { 0, 0 },
    { 0xe13c0622, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bc0625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adc062a, 23 },
    { 0, 0 },
    { 0x44ec062c, 66 },
    { 0x30bc062c, 98 },
    { 0x33dc062e, 77 },
    { 0xd27c062f, 68 },
    { 0x991c062e, 55 },
    { 0xee2c062d, 69 },
    { 0x1bfc0632, 62 },
    { 0xbccc0631, 95 },
    { 0x7ccc0634, 93 },
    { 0x58fc0634, 93 },
    { 0xe68c0636, 90 },
    { 0xec2c0636, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ec063d, 90 },
    { 0x9dac063e, 99 },
    { 0xca6c063f, 95 },
    { 0x3d3c0640, 97 },
    { 0x944c0641, 76 },
    { 0xa3c0642, 55 },
    { 0xc23c0643, 61 },
    { 0x6ccc0643, 95 },
    { 0x257c0641, 61 },
    { 0x47cc0641, 33 },
    { 0x980c0647, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcc064c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x957c064f, 98 },
    { 0x6cbc0650, 52 },
    { 0xe1fc0650, 23 },
    { 0xcc5c0651, 90 },
    { 0x4a2c0650, 46 },
    { 0xa88c064f, 90 },
    { 0xaebc0655, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9c065a, 43 },
    { 0, 0 },
    { 0xa44c065c, 90 },
    { 0xe2ec065d, 90 },
    { 0xfc4c065d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07c0663, 80 },
    { 0, 0 },
    { 0xae8c0665, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbc066d, 76 },
    { 0x5eec066e, 77 },
    { 0, 0 },
    { 0xc3c0670, 76 },
    { 0x532c0670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154c0676, 46 },
    { 0, 0 },
    { 0x7b8c0678, 76 },
    { 0x1bcc0679, 95 },
    { 0x986c0678, 90 },
    { 0xba2c0678, 43 },
    { 0x523c0679, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cc067f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bc0683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c0686, 33 },
    { 0xdcfc0687, 95 },
    { 0x658c0688, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc068c, 61 },
    { 0, 0 },
    { 0x669c068e, 98 },
    { 0xb80c068f, 76 },
    { 0x72bc068e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fc0694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2c0699, 98 },
    { 0, 0 },
    { 0x8efc069b, 66 },
    { 0xa56c069c, 98 },
    { 0x30ec069d, 43 },
    { 0x44fc069b, 33 },
    { 0xe38c069b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293c06a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac06a9, 91 },
    { 0, 0 },
    { 0x4fbc06ab, 81 },
    { 0, 0 },
    { 0x18dc06ad, 77 },
    { 0, 0 },
    { 0xbd1c06af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60c06b6, 55 },
    { 0x15dc06b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398c06bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x305c06c0, 96 },
    { 0x127c06c1, 34 },
    { 0x9a0c06c2, 23 },
    { 0x908c06c3, 65 },
    { 0xab9c06c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5c06ca, 76 },
    { 0x7afc06cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbc06ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fc06d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6c06d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912c06df, 95 },
    { 0, 0 },
    { 0x605c06e1, 90 },
    { 0, 0 },
    { 0xc34c06e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ec06e8, 55 },
    { 0x4a8c06e9, 33 },
    { 0x3fdc06e8, 98 },
    { 0x9aec06eb, 33 },
    { 0xc5c06ec, 46 },
    { 0, 0 },
    { 0xe95c06ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7c06f9, 93 },
    { 0, 0 },
    { 0x63bc06fb, 46 },
    { 0x7a8c06fc, 98 },
    { 0, 0 },
    { 0x45fc06fe, 61 },
    { 0, 0 },
    { 0x505c0700, 33 },
    { 0x589c0700, 98 },
    { 0x19dc0702, 37 },
    { 0x458c0702, 23 },
    { 0xe23c0701, 95 },
    { 0, 0 },
    { 0x565c0706, 95 },
    { 0, 0 },
    { 0x3acc0708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddc070b, 98 },
    { 0, 0 },
    { 0xeb0c070d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478c0711, 90 },
    { 0x5cac0711, 40 },
    { 0, 0 },
    { 0x278c0714, 93 },
    { 0xcd7c0715, 67 },
    { 0x610c0714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07c071a, 23 },
    { 0, 0 },
    { 0x8a9c071c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9c071f, 90 },
    { 0x8adc071f, 99 },
    { 0, 0 },
    { 0x52dc0722, 87 },
    { 0x7d1c0722, 85 },
    { 0x567c0724, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00c072a, 93 },
    { 0, 0 },
    { 0xeaac072c, 43 },
    { 0x628c072d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41c0734, 55 },
    { 0xab2c0735, 33 },
    { 0x40ac0736, 55 },
    { 0x24bc0737, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c073b, 61 },
    { 0x19dc073c, 100 },
    { 0, 0 },
    { 0xf03c073e, 100 },
    { 0x7bfc073f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfc0743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900c0748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c074d, 33 },
    { 0xdd9c074e, 81 },
    { 0x5eec074f, 68 },
    { 0x23ac074f, 90 },
    { 0x88dc0751, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ec0758, 100 },
    { 0x398c0759, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93c0760, 93 },
    { 0x553c0760, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c0764, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fec0768, 95 },
    { 0xf59c0769, 23 },
    { 0, 0 },
    { 0xb16c076b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0c0771, 87 },
    { 0, 0 },
    { 0x132c0773, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ac0776, 78 },
    { 0x174c0776, 52 },
    { 0x87ac0778, 33 },
    { 0, 0 },
    { 0x63ec077a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543c0787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ec078c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9c078f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2c0793, 66 },
    { 0x494c0794, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ac0799, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8c079c, 91 },
    { 0x75fc079d, 88 },
    { 0, 0 },
    { 0xb42c079f, 90 },
    { 0xacec07a0, 90 },
    { 0xe13c07a0, 36 },
    { 0x382c07a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5c07a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdc07b7, 90 },
    { 0xa9ec07b8, 33 },
    { 0xee4c07b9, 95 },
    { 0x1e1c07ba, 23 },
    { 0x438c07bb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfc07be, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30c07c4, 46 },
    { 0x218c07c5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1c07dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x286c07df, 17 },
    { 0x2a1c07e0, 87 },
    { 0, 0 },
    { 0x4efc07e2, 33 },
    { 0, 0 },
    { 0x2c0c07e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadac07e7, 33 },
    { 0x944c07e7, 61 },
    { 0x1f3c07e9, 72 },
    { 0, 0 },
    { 0x734c07eb, 77 },
    { 0x9a5c07ec, 66 },
    { 0xd9dc07ec, 95 },
    { 0x168c07ec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cc07f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bc07f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ac07fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7c0802, 33 },
    { 0xf86c0803, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262c080a, 76 },
    { 0xca1c080b, 90 },
    { 0x4d2c080c, 88 },
    { 0xa7fc080a, 76 },
    { 0, 0 },
    { 0xd97c080f, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3c0813, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9c0822, 88 },
    { 0, 0 },
    { 0x77fc0824, 90 },
    { 0, 0 },
    { 0x111c0826, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7c0829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fc0832, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c0835, 90 },
    { 0xc61c0835, 61 },
    { 0x1c3c0836, 98 },
    { 0x777c0836, 70 },
    { 0, 0 },
    { 0x8a7c083a, 23 },
    { 0x3f4c083a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6c083f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc0842, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc0846, 85 },
    { 0xcddc0847, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c0852, 90 },
    { 0x18c0853, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fc0856, 23 },
    { 0xd28c0857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dc085a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6c085d, 90 },
    { 0x8dac085e, 97 },
    { 0x954c085f, 88 },
    { 0xb21c085d, 61 },
    { 0x736c0861, 23 },
    { 0x2cfc085e, 100 },
    { 0x4bfc0863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cac086d, 98 },
    { 0, 0 },
    { 0x2d3c086f, 76 },
    { 0, 0 },
    { 0x217c0871, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ac0875, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ac087a, 90 },
    { 0x28bc087b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dc087f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6c0888, 90 },
    { 0, 0 },
    { 0x2ac088a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc088d, 23 },
    { 0x37fc088e, 23 },
    { 0x704c088d, 90 },
    { 0x886c088e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c0895, 33 },
    { 0xa1c0896, 85 },
    { 0xe94c0897, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4c089d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c08a3, 78 },
    { 0x2eec08a4, 84 },
    { 0, 0 },
    { 0xb99c08a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88c08ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34c08b0, 90 },
    { 0x5d6c08b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cc08b4, 90 },
    { 0, 0 },
    { 0x6d3c08b6, 68 },
    { 0, 0 },
    { 0x205c08b8, 33 },
    { 0, 0 },
    { 0x43cc08ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bc08c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1c08cc, 55 },
    { 0x11ac08cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ec08d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc08d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc08da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c08e1, 67 },
    { 0x553c08e2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fc08e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x507c08e8, 77 },
    { 0x451c08e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fc08ee, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c08f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7c08fc, 81 },
    { 0x404c08fd, 78 },
    { 0, 0 },
    { 0x177c08ff, 98 },
    { 0, 0 },
    { 0xca3c0901, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c090f, 66 },
    { 0xc58c0910, 90 },
    { 0x40ec0911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27c0914, 98 },
    { 0, 0 },
    { 0x3ebc0916, 33 },
    { 0xcdec0917, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x78c091a, 95 },
    { 0, 0 },
    { 0x7a5c091c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bac0921, 61 },
    { 0xf0fc0922, 91 },
    { 0x6c8c0921, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x273c0926, 67 },
    { 0x924c0927, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8c092c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2c0934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc093c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c093f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc0942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194c094a, 66 },
    { 0xd76c094b, 99 },
    { 0xc05c094b, 61 },
    { 0, 0 },
    { 0x25ac094e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc0952, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09c0955, 33 },
    { 0x502c0955, 33 },
    { 0x556c0957, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2c095f, 83 },
    { 0x531c0960, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec0963, 33 },
    { 0xda8c0964, 90 },
    { 0x229c0965, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904c096b, 33 },
    { 0xfe1c096b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ac096f, 17 },
    { 0x337c0970, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedc0973, 61 },
    { 0, 0 },
    { 0xcaec0975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928c097b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x271c097e, 76 },
    { 0x792c097e, 23 },
    { 0xdd0c097e, 34 },
    { 0x90ac0980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2c098a, 34 },
    { 0x78dc098b, 91 },
    { 0x7b8c098b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x717c098f, 55 },
    { 0xf03c098f, 46 },
    { 0x399c0991, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec0995, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42c0998, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fc099d, 34 },
    { 0x330c099e, 78 },
    { 0x17cc099e, 55 },
    { 0x6b4c099e, 76 },
    { 0x8d3c09a1, 61 },
    { 0xc4ac09a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493c09a7, 76 },
    { 0, 0 },
    { 0xcf1c09a9, 17 },
    { 0, 0 },
    { 0x62cc09ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c09af, 95 },
    { 0, 0 },
    { 0xcc2c09b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x718c09b4, 98 },
    { 0, 0 },
    { 0xd25c09b6, 51 },
    { 0xb4bc09b6, 25 },
    { 0x323c09b8, 23 },
    { 0x333c09b8, 43 },
    { 0x708c09b8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x928c09bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ec09c1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ac09c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ac09cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c09cf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61c09d6, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdc09d9, 64 },
    { 0xc54c09da, 91 },
    { 0x56ac09db, 55 },
    { 0x826c09dc, 90 },
    { 0x240c09dd, 90 },
    { 0, 0 },
    { 0x4a4c09df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcc09e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdc09e9, 76 },
    { 0, 0 },
    { 0xaa3c09eb, 90 },
    { 0x3c6c09eb, 76 },
    { 0, 0 },
    { 0x406c09ee, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc09f2, 33 },
    { 0x464c09f3, 44 },
    { 0xd2c09f4, 46 },
    { 0, 0 },
    { 0xd9bc09f6, 76 },
    { 0, 0 },
    { 0x1c5c09f8, 98 },
    { 0xe61c09f9, 95 },
    { 0xfb2c09fa, 95 },
    { 0xe4cc09fa, 90 },
    { 0xfd1c09fa, 17 },
    { 0, 0 },
    { 0x2afc09fe, 68 },
    { 0xa97c09ff, 61 },
    { 0xa7ac0a00, 98 },
    { 0x443c09ff, 90 },
    { 0x6a0c0a01, 87 },
    { 0x2bc0a03, 61 },
    { 0xf3ac0a03, 84 },
    { 0x381c0a03, 90 },
    { 0x980c0a06, 61 },
    { 0xdfac0a04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40c0a0e, 99 },
    { 0x7cfc0a0f, 98 },
    { 0x971c0a10, 98 },
    { 0, 0 },
    { 0xcdac0a12, 95 },
    { 0, 0 },
    { 0x2c0a14, 33 },
    { 0, 0 },
    { 0x39c0a16, 87 },
    { 0xda5c0a16, 61 },
    { 0xa15c0a18, 98 },
    { 0xcd2c0a16, 61 },
    { 0xb58c0a18, 34 },
    { 0xef1c0a1b, 76 },
    { 0xffbc0a18, 95 },
    { 0x4fac0a1d, 23 },
    { 0xb7cc0a1e, 61 },
    { 0x21cc0a1e, 33 },
    { 0xcfcc0a1e, 34 },
    { 0x561c0a19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984c0a25, 46 },
    { 0xe33c0a25, 61 },
    { 0, 0 },
    { 0x6eec0a28, 33 },
    { 0, 0 },
    { 0xef6c0a2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6c0a33, 36 },
    { 0, 0 },
    { 0x3a2c0a35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fc0a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c0a40, 76 },
    { 0xe7bc0a41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933c0a49, 91 },
    { 0x244c0a49, 23 },
    { 0x3fbc0a4b, 33 },
    { 0xf12c0a4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949c0a58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x213c0a5b, 76 },
    { 0xeedc0a5c, 34 },
    { 0x1d2c0a5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdec0a62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebc0a66, 61 },
    { 0xc75c0a66, 61 },
    { 0, 0 },
    { 0x987c0a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69c0a71, 61 },
    { 0, 0 },
    { 0x86bc0a73, 95 },
    { 0xeec0a73, 23 },
    { 0x629c0a75, 23 },
    { 0x3b5c0a74, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642c0a7b, 100 },
    { 0, 0 },
    { 0xd4ac0a7d, 87 },
    { 0, 0 },
    { 0xab9c0a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8c0a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64c0a8d, 33 },
    { 0xedbc0a8e, 91 },
    { 0x6acc0a8f, 33 },
    { 0x8fc0a90, 28 },
    { 0xe77c0a8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99c0a97, 61 },
    { 0x275c0a97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1c0a9e, 90 },
    { 0x6f7c0a9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc0aa9, 99 },
    { 0, 0 },
    { 0x4ecc0aab, 23 },
    { 0x33ec0aac, 23 },
    { 0xea2c0aac, 23 },
    { 0xe26c0aac, 46 },
    { 0x585c0aae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7c0ab6, 61 },
    { 0, 0 },
    { 0x80bc0ab8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fec0abd, 77 },
    { 0x456c0abd, 23 },
    { 0x36c0abe, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82c0ac5, 90 },
    { 0x43ac0ac5, 76 },
    { 0, 0 },
    { 0xddac0ac8, 90 },
    { 0x7a9c0ac9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec0acd, 55 },
    { 0xf5fc0ace, 93 },
    { 0x212c0acf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5c0ad2, 90 },
    { 0xc1ec0ad3, 90 },
    { 0xec9c0ad2, 33 },
    { 0x96ac0ad5, 95 },
    { 0x2d7c0ad2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311c0adc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778c0ae0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfc0ae4, 41 },
    { 0, 0 },
    { 0x9cc0ae6, 100 },
    { 0, 0 },
    { 0x524c0ae8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dc0aeb, 90 },
    { 0x6e2c0aec, 33 },
    { 0x35cc0aec, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101c0af3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edc0af6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44c0afa, 90 },
    { 0, 0 },
    { 0xa29c0afc, 23 },
    { 0, 0 },
    { 0x1d5c0afe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557c0b05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4c0b08, 98 },
    { 0x99ac0b09, 96 },
    { 0, 0 },
    { 0xa7ec0b0b, 77 },
    { 0, 0 },
    { 0x75fc0b0d, 33 },
    { 0x5c8c0b0e, 33 },
    { 0, 0 },
    { 0x285c0b10, 33 },
    { 0x1ebc0b11, 46 },
    { 0, 0 },
    { 0x9dcc0b13, 23 },
    { 0, 0 },
    { 0xff2c0b15, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12c0b18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730c0b1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c0b2a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdc0b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ac0b34, 95 },
    { 0, 0 },
    { 0xbd7c0b36, 23 },
    { 0x937c0b37, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbc0b3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516c0b42, 66 },
    { 0x11ec0b42, 90 },
    { 0, 0 },
    { 0x6dfc0b45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ac0b49, 46 },
    { 0xeb1c0b49, 90 },
    { 0x1a9c0b4b, 66 },
    { 0x845c0b49, 33 },
    { 0x96c0b4c, 33 },
    { 0xe15c0b4e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522c0b58, 34 },
    { 0xd74c0b59, 33 },
    { 0, 0 },
    { 0x60cc0b5b, 90 },
    { 0xcb4c0b5c, 33 },
    { 0x6dbc0b5d, 61 },
    { 0xc02c0b5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fc0b61, 23 },
    { 0x4acc0b62, 17 },
    { 0xe22c0b63, 99 },
    { 0, 0 },
    { 0xe12c0b65, 84 },
    { 0xf3dc0b66, 23 },
    { 0x739c0b66, 76 },
    { 0xec6c0b67, 76 },
    { 0xfedc0b68, 99 },
    { 0, 0 },
    { 0xf27c0b6b, 90 },
    { 0x50dc0b6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c0b71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ac0b74, 23 },
    { 0, 0 },
    { 0x642c0b76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99c0b7a, 61 },
    { 0, 0 },
    { 0x378c0b7c, 90 },
    { 0x783c0b7d, 61 },
    { 0xa48c0b7d, 23 },
    { 0xee5c0b7f, 67 },
    { 0xc1dc0b80, 33 },
    { 0xdc7c0b7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1c0b87, 66 },
    { 0, 0 },
    { 0x837c0b89, 91 },
    { 0, 0 },
    { 0x5c3c0b8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24c0b8e, 93 },
    { 0x6a3c0b8e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c0b93, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537c0b9b, 55 },
    { 0x97fc0b9c, 61 },
    { 0x5bcc0b9d, 86 },
    { 0, 0 },
    { 0x29bc0b9f, 66 },
    { 0x5e6c0b9f, 93 },
    { 0xaa6c0ba1, 95 },
    { 0, 0 },
    { 0xdd7c0ba3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x475c0ba6, 43 },
    { 0, 0 },
    { 0x900c0ba8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dc0bac, 95 },
    { 0x44ac0bad, 77 },
    { 0xa80c0bad, 95 },
    { 0x9a0c0bad, 31 },
    { 0x4bdc0bae, 61 },
    { 0, 0 },
    { 0x15dc0bb2, 55 },
    { 0xbf0c0bb2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ac0bb7, 40 },
    { 0, 0 },
    { 0xec2c0bb9, 64 },
    { 0xb9ec0bba, 61 },
    { 0x96ac0bbb, 48 },
    { 0xac5c0bbc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328c0bc0, 51 },
    { 0x590c0bc0, 33 },
    { 0x7d8c0bc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfc0bcd, 43 },
    { 0x73c0bce, 52 },
    { 0, 0 },
    { 0x2ebc0bd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4c0bd3, 90 },
    { 0xe9bc0bd4, 52 },
    { 0xac1c0bd5, 46 },
    { 0x2c9c0bd5, 23 },
    { 0xa61c0bd4, 75 },
    { 0x13fc0bd8, 90 },
    { 0x4e0c0bd9, 81 },
    { 0, 0 },
    { 0x85ec0bdb, 42 },
    { 0x8cc0bdc, 83 },
    { 0, 0 },
    { 0x35fc0bde, 90 },
    { 0x6afc0bde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5c0be7, 61 },
    { 0x3c6c0be8, 43 },
    { 0x71bc0be9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76c0bee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac0bf3, 95 },
    { 0x9cac0bf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9c0bf7, 95 },
    { 0x31c0bf8, 95 },
    { 0, 0 },
    { 0x21ec0bfa, 23 },
    { 0x6e3c0bfa, 90 },
    { 0xc02c0bfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x947c0bff, 90 },
    { 0, 0 },
    { 0x903c0c01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acc0c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fc0c0b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652c0c11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec0c18, 67 },
    { 0xc5bc0c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651c0c1f, 90 },
    { 0, 0 },
    { 0x76ac0c21, 33 },
    { 0xfc1c0c22, 90 },
    { 0x610c0c23, 23 },
    { 0x4ebc0c24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336c0c29, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4c0c2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554c0c32, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6c0c36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8c0c3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdc0c3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24c0c43, 55 },
    { 0x4f4c0c44, 76 },
    { 0xc14c0c44, 93 },
    { 0xd30c0c46, 60 },
    { 0, 0 },
    { 0x738c0c48, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c0c4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ec0c4f, 76 },
    { 0, 0 },
    { 0x4d1c0c51, 80 },
    { 0x14cc0c52, 100 },
    { 0x59ac0c53, 33 },
    { 0, 0 },
    { 0x438c0c55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6c0c5a, 42 },
    { 0x762c0c5b, 95 },
    { 0, 0 },
    { 0x4e3c0c5d, 23 },
    { 0xf5fc0c5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fc0c63, 90 },
    { 0, 0 },
    { 0x157c0c65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c0c68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810c0c6f, 17 },
    { 0, 0 },
    { 0x6f4c0c71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fc0c74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0c7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ec0c82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8c0c85, 66 },
    { 0, 0 },
    { 0x9e4c0c87, 23 },
    { 0, 0 },
    { 0x55ec0c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dc0c8d, 17 },
    { 0x8cbc0c8e, 98 },
    { 0x524c0c8f, 61 },
    { 0x668c0c8f, 38 },
    { 0x4d2c0c91, 94 },
    { 0x693c0c92, 76 },
    { 0x5b3c0c93, 93 },
    { 0x33fc0c94, 98 },
    { 0x9afc0c95, 98 },
    { 0xed8c0c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9c0c99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c0ca0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126c0ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ec0caa, 33 },
    { 0x9cec0caa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679c0cb0, 66 },
    { 0, 0 },
    { 0x2f2c0cb2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cc0cb7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719c0cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9c0cbe, 90 },
    { 0x23bc0cbe, 68 },
    { 0, 0 },
    { 0x2f4c0cc1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c0cc4, 66 },
    { 0x1cac0cc4, 55 },
    { 0x9dcc0cc4, 98 },
    { 0x3cdc0cc4, 99 },
    { 0x353c0cc4, 56 },
    { 0x7dcc0cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ec0cd8, 67 },
    { 0xb3bc0cd8, 61 },
    { 0x1c0c0cd8, 46 },
    { 0, 0 },
    { 0xdbbc0cdc, 46 },
    { 0x919c0cdd, 87 },
    { 0xfd0c0cdd, 23 },
    { 0x6acc0cdc, 76 },
    { 0xae6c0ce0, 55 },
    { 0x383c0ce1, 61 },
    { 0xd56c0ce1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x607c0ce5, 76 },
    { 0xef3c0ce6, 90 },
    { 0x125c0ce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x995c0cea, 87 },
    { 0x281c0ceb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59c0cf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04c0cf6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ec0d01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89c0d09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741c0d12, 95 },
    { 0, 0 },
    { 0x701c0d14, 43 },
    { 0x1b2c0d15, 75 },
    { 0x5e0c0d16, 98 },
    { 0x4c0d17, 90 },
    { 0x402c0d18, 98 },
    { 0, 0 },
    { 0xbe4c0d1a, 98 },
    { 0xcf8c0d1b, 17 },
    { 0x36cc0d1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c0d1f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5c0d22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c0d2a, 46 },
    { 0x720c0d2b, 67 },
    { 0x9ec0d2a, 84 },
    { 0x8a4c0d2d, 95 },
    { 0, 0 },
    { 0x5b8c0d2f, 56 },
    { 0xcffc0d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c0d36, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4c0d3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92c0d3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59c0d44, 34 },
    { 0x1a3c0d45, 33 },
    { 0xe77c0d44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3c0d49, 68 },
    { 0xf25c0d49, 66 },
    { 0, 0 },
    { 0x32bc0d4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc0d4f, 90 },
    { 0, 0 },
    { 0x875c0d51, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2c0d56, 61 },
    { 0, 0 },
    { 0x926c0d58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x202c0d5b, 99 },
    { 0x7aac0d5c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bc0d60, 95 },
    { 0, 0 },
    { 0x419c0d62, 61 },
    { 0x9d7c0d63, 98 },
    { 0xcadc0d64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1c0d6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac0d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465c0d74, 100 },
    { 0x5e8c0d75, 90 },
    { 0xc0cc0d76, 98 },
    { 0xe74c0d77, 98 },
    { 0x215c0d77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92c0d7b, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443c0d80, 23 },
    { 0, 0 },
    { 0xd47c0d82, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fac0d85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cc0d8c, 95 },
    { 0x5c3c0d8d, 90 },
    { 0, 0 },
    { 0x8d1c0d8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149c0d96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4c0d9e, 85 },
    { 0x1acc0d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92c0da2, 90 },
    { 0xdb7c0da3, 61 },
    { 0x627c0da2, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575c0daa, 65 },
    { 0xafbc0dab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c0db1, 90 },
    { 0xd11c0db2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ec0dbb, 23 },
    { 0x37fc0dbc, 46 },
    { 0xa62c0dbc, 76 },
    { 0, 0 },
    { 0xedfc0dbf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c0dc2, 93 },
    { 0, 0 },
    { 0xde4c0dc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66c0dcc, 23 },
    { 0x53ac0dcd, 23 },
    { 0x654c0dce, 68 },
    { 0xf4ac0dce, 34 },
    { 0x90ac0dce, 90 },
    { 0x7f0c0dce, 52 },
    { 0, 0 },
    { 0xe58c0dd3, 23 },
    { 0x81c0dd4, 67 },
    { 0, 0 },
    { 0x694c0dd6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ec0dd9, 43 },
    { 0, 0 },
    { 0xc20c0ddb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4c0dde, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442c0de2, 61 },
    { 0x549c0de2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398c0de8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716c0dec, 61 },
    { 0xe98c0dec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60c0df1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05c0df7, 98 },
    { 0, 0 },
    { 0x274c0df9, 90 },
    { 0, 0 },
    { 0x585c0dfb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c0e01, 33 },
    { 0, 0 },
    { 0x99cc0e03, 90 },
    { 0xd24c0e04, 90 },
    { 0, 0 },
    { 0x2eec0e06, 23 },
    { 0x36bc0e07, 90 },
    { 0, 0 },
    { 0x264c0e09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986c0e0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ec0e14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bac0e1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc0e25, 43 },
    { 0x1fcc0e26, 87 },
    { 0xca4c0e27, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c0e31, 81 },
    { 0xcbcc0e32, 91 },
    { 0xf78c0e32, 52 },
    { 0x9b1c0e31, 24 },
    { 0x5f0c0e33, 33 },
    { 0x88c0e36, 61 },
    { 0xa87c0e36, 33 },
    { 0x28dc0e38, 61 },
    { 0xd80c0e39, 95 },
    { 0x8dec0e34, 84 },
    { 0xf5fc0e3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6c0e41, 23 },
    { 0x51bc0e42, 90 },
    { 0xb0cc0e43, 81 },
    { 0xa5ac0e41, 43 },
    { 0xd7bc0e45, 90 },
    { 0x7f2c0e46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbc0e4c, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc0e52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6c0e5a, 61 },
    { 0x97ac0e5b, 100 },
    { 0x191c0e5b, 23 },
    { 0, 0 },
    { 0x574c0e5e, 55 },
    { 0, 0 },
    { 0x8c7c0e60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95c0e65, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8c0e6d, 76 },
    { 0, 0 },
    { 0xb49c0e6f, 33 },
    { 0x855c0e70, 90 },
    { 0x530c0e71, 78 },
    { 0xf1dc0e71, 23 },
    { 0x2b3c0e73, 33 },
    { 0x8c6c0e73, 100 },
    { 0, 0 },
    { 0xa86c0e76, 78 },
    { 0, 0 },
    { 0xb10c0e78, 95 },
    { 0, 0 },
    { 0xca1c0e7a, 98 },
    { 0x14ac0e7b, 23 },
    { 0xeb6c0e7c, 61 },
    { 0, 0 },
    { 0xba3c0e7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196c0e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6c0e88, 55 },
    { 0x249c0e88, 33 },
    { 0x469c0e88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c0e8f, 34 },
    { 0, 0 },
    { 0x452c0e91, 43 },
    { 0xe8cc0e91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acc0e95, 33 },
    { 0, 0 },
    { 0x31cc0e97, 66 },
    { 0xd3c0e98, 55 },
    { 0x6dbc0e98, 40 },
    { 0xe7fc0e97, 90 },
    { 0x998c0e9b, 61 },
    { 0x43c0e9b, 61 },
    { 0x297c0e9d, 98 },
    { 0x739c0e98, 61 },
    { 0x697c0e9e, 61 },
    { 0x337c0e9f, 90 },
    { 0, 0 },
    { 0xf32c0ea2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47c0ea9, 33 },
    { 0x679c0eaa, 61 },
    { 0x5f3c0eab, 95 },
    { 0x631c0eac, 23 },
    { 0xd1c0eac, 55 },
    { 0xd38c0eaa, 98 },
    { 0xfaac0eaf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85c0eb6, 93 },
    { 0, 0 },
    { 0x954c0eb8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68c0ebd, 34 },
    { 0, 0 },
    { 0xb0bc0ebf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c0ec2, 61 },
    { 0, 0 },
    { 0xcfec0ec4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9c0ecb, 46 },
    { 0x818c0ecc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6c0ecf, 78 },
    { 0x89cc0ed0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191c0ed5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dac0ed8, 95 },
    { 0, 0 },
    { 0x12ac0eda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c0ee4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25c0eec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ac0eef, 95 },
    { 0x187c0ef0, 23 },
    { 0x3d4c0ef1, 23 },
    { 0, 0 },
    { 0x259c0ef3, 33 },
    { 0x7ecc0ef4, 90 },
    { 0x5d8c0ef4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bc0ef9, 55 },
    { 0, 0 },
    { 0x49ac0efb, 98 },
    { 0, 0 },
    { 0xc96c0efd, 17 },
    { 0xe11c0efe, 48 },
    { 0x833c0efd, 55 },
    { 0xca9c0f00, 90 },
    { 0x6e0c0eff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ac0f04, 90 },
    { 0, 0 },
    { 0x6c1c0f06, 61 },
    { 0x882c0f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52c0f0e, 34 },
    { 0xe76c0f0f, 90 },
    { 0x168c0f10, 95 },
    { 0, 0 },
    { 0x837c0f12, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363c0f18, 61 },
    { 0xa87c0f18, 34 },
    { 0xc74c0f19, 98 },
    { 0x33dc0f1b, 90 },
    { 0xbfbc0f19, 67 },
    { 0x13ac0f18, 95 },
    { 0x43cc0f1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddc0f24, 90 },
    { 0, 0 },
    { 0x5ffc0f26, 98 },
    { 0x4c5c0f27, 34 },
    { 0x6c9c0f27, 17 },
    { 0x431c0f29, 43 },
    { 0xa54c0f29, 55 },
    { 0x1aac0f29, 22 },
    { 0xab8c0f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecc0f30, 61 },
    { 0xd3fc0f31, 43 },
    { 0x6c4c0f30, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213c0f37, 99 },
    { 0xdb7c0f38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fc0f42, 55 },
    { 0x91bc0f43, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80c0f4c, 52 },
    { 0x475c0f4d, 99 },
    { 0, 0 },
    { 0x31cc0f4f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dc0f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebc0f58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569c0f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c0f63, 23 },
    { 0xc7bc0f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3c0f6c, 23 },
    { 0, 0 },
    { 0xa61c0f6e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53c0f74, 80 },
    { 0x8a2c0f74, 61 },
    { 0x58ec0f76, 76 },
    { 0, 0 },
    { 0xb40c0f78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23c0f7f, 48 },
    { 0xcfec0f80, 76 },
    { 0, 0 },
    { 0xcb4c0f82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9c0f89, 90 },
    { 0, 0 },
    { 0xd99c0f8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afc0f92, 95 },
    { 0, 0 },
    { 0x80c0f94, 66 },
    { 0x983c0f95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824c0f99, 67 },
    { 0x1b5c0f9a, 67 },
    { 0xed5c0f9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253c0f9f, 90 },
    { 0xacac0fa0, 88 },
    { 0xa51c0f9f, 61 },
    { 0x27bc0fa2, 100 },
    { 0, 0 },
    { 0xa2dc0fa4, 33 },
    { 0, 0 },
    { 0x621c0fa6, 93 },
    { 0xbdfc0fa6, 93 },
    { 0, 0 },
    { 0x584c0fa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c0fb3, 91 },
    { 0, 0 },
    { 0x4e4c0fb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67c0fb9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986c0fbd, 61 },
    { 0, 0 },
    { 0x29fc0fbf, 98 },
    { 0, 0 },
    { 0x35bc0fc1, 90 },
    { 0xcd8c0fc2, 61 },
    { 0x73ac0fc3, 100 },
    { 0x71ec0fc1, 17 },
    { 0xf66c0fc5, 69 },
    { 0, 0 },
    { 0xe86c0fc7, 91 },
    { 0xca8c0fc8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c0fcc, 23 },
    { 0, 0 },
    { 0x1b6c0fce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33c0fd1, 33 },
    { 0x5b6c0fd1, 34 },
    { 0, 0 },
    { 0xc75c0fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6c0fdb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x822c0fde, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec0fe3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19c0fe9, 77 },
    { 0xf73c0fea, 23 },
    { 0x1c8c0feb, 90 },
    { 0x204c0feb, 33 },
    { 0xa77c0fea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dc0ff4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fc0ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c0fff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccc1003, 33 },
    { 0xc80c1004, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adc1007, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233c100c, 17 },
    { 0, 0 },
    { 0x757c100e, 22 },
    { 0xa07c100f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bc1015, 100 },
    { 0x53dc1015, 61 },
    { 0, 0 },
    { 0xdd4c1018, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dc1022, 61 },
    { 0x455c1023, 90 },
    { 0x565c1024, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62c102a, 43 },
    { 0x625c102b, 23 },
    { 0xecbc102c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464c1030, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bc1033, 17 },
    { 0xd5dc1034, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ac103d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8c1043, 61 },
    { 0xa11c1044, 43 },
    { 0xbb3c1045, 98 },
    { 0xc40c1044, 61 },
    { 0xe4c1044, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecc104b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c104e, 61 },
    { 0xee7c104e, 61 },
    { 0x3b6c1050, 78 },
    { 0x82ec1051, 40 },
    { 0xff5c1052, 61 },
    { 0x60cc1053, 33 },
    { 0x97fc1051, 90 },
    { 0x652c1055, 87 },
    { 0, 0 },
    { 0x80dc1057, 78 },
    { 0x72ec1057, 99 },
    { 0xa1ac1059, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27c105c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ec1063, 55 },
    { 0x758c1063, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4c1068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fc1071, 90 },
    { 0, 0 },
    { 0x986c1073, 76 },
    { 0x27cc1073, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fc107c, 98 },
    { 0x758c107d, 90 },
    { 0x1bdc107d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eac1085, 43 },
    { 0xff8c1086, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec108b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc108f, 80 },
    { 0xc18c108f, 75 },
    { 0, 0 },
    { 0xee1c1092, 61 },
    { 0x35ec1093, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dc1099, 95 },
    { 0x5ecc109a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4c109e, 91 },
    { 0x18ec109f, 59 },
    { 0x9b4c109e, 81 },
    { 0x53ac10a1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c10a7, 23 },
    { 0xeb8c10a8, 61 },
    { 0x5b3c10a7, 61 },
    { 0x3e9c10a8, 87 },
    { 0, 0 },
    { 0xf67c10ac, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fc10b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc10bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2c10c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaac10c6, 98 },
    { 0x7cbc10c7, 98 },
    { 0x3f9c10c8, 94 },
    { 0x179c10c9, 90 },
    { 0xa65c10c8, 95 },
    { 0, 0 },
    { 0x287c10cc, 61 },
    { 0, 0 },
    { 0xaffc10ce, 61 },
    { 0, 0 },
    { 0x521c10d0, 52 },
    { 0x5bfc10d1, 91 },
    { 0x41fc10d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x697c10d5, 90 },
    { 0x381c10d5, 61 },
    { 0x167c10d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476c10db, 33 },
    { 0x63cc10dc, 33 },
    { 0xb34c10dc, 90 },
    { 0xa42c10de, 95 },
    { 0x44c10dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5c10e3, 55 },
    { 0, 0 },
    { 0xbb0c10e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c10ea, 17 },
    { 0, 0 },
    { 0xdbac10ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cc10f0, 34 },
    { 0xdf5c10f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421c10f5, 80 },
    { 0xc00c10f5, 90 },
    { 0, 0 },
    { 0x278c10f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306c1103, 98 },
    { 0, 0 },
    { 0x8a9c1105, 98 },
    { 0xaeec1106, 90 },
    { 0xfaec1106, 87 },
    { 0xb61c1108, 61 },
    { 0xd41c1109, 52 },
    { 0, 0 },
    { 0xaa3c110b, 23 },
    { 0xf7ac110c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x531c110f, 99 },
    { 0, 0 },
    { 0xae5c1111, 61 },
    { 0x89cc1111, 90 },
    { 0xf39c1113, 76 },
    { 0x184c1111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c111e, 95 },
    { 0x166c111f, 43 },
    { 0xe9cc1120, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bc1124, 81 },
    { 0x97fc1125, 99 },
    { 0x86c1126, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364c112a, 90 },
    { 0xcb4c112b, 61 },
    { 0, 0 },
    { 0xb27c112d, 33 },
    { 0, 0 },
    { 0x8d7c112f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c1134, 33 },
    { 0x72cc1134, 40 },
    { 0x655c1136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712c113a, 33 },
    { 0x480c113b, 55 },
    { 0xb10c113b, 98 },
    { 0xadfc113c, 69 },
    { 0, 0 },
    { 0xbfdc113f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4c1142, 56 },
    { 0x12bc1142, 78 },
    { 0xb0ec1143, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603c1151, 90 },
    { 0xd25c1152, 90 },
    { 0x1c3c1153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc1156, 53 },
    { 0, 0 },
    { 0xb90c1158, 90 },
    { 0xd21c1159, 90 },
    { 0, 0 },
    { 0x962c115b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3c1160, 33 },
    { 0x391c1161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c1166, 61 },
    { 0x174c1166, 33 },
    { 0xec5c1166, 23 },
    { 0x120c1166, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fc116d, 43 },
    { 0x6b2c116e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c1177, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2c117b, 76 },
    { 0, 0 },
    { 0x34ac117d, 98 },
    { 0, 0 },
    { 0x417c117f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbc1182, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121c1186, 55 },
    { 0x744c1186, 99 },
    { 0, 0 },
    { 0xd61c1189, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fc118d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afc1198, 98 },
    { 0xc1198, 98 },
    { 0x104c119a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfec119d, 90 },
    { 0x25bc119e, 8 },
    { 0x67dc119f, 61 },
    { 0x809c119f, 95 },
    { 0xf00c11a1, 99 },
    { 0, 0 },
    { 0xc91c11a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc11a7, 33 },
    { 0x6ac11a7, 33 },
    { 0xe5ac11a9, 93 },
    { 0x97ac11aa, 91 },
    { 0x8c9c11a8, 33 },
    { 0xaf6c11ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x251c11af, 95 },
    { 0x32c11b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x776c11b3, 33 },
    { 0, 0 },
    { 0x232c11b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c11b8, 90 },
    { 0, 0 },
    { 0x18cc11ba, 68 },
    { 0, 0 },
    { 0x60ec11bc, 75 },
    { 0x6c11bd, 23 },
    { 0x405c11be, 98 },
    { 0, 0 },
    { 0x20fc11c0, 76 },
    { 0x141c11c0, 68 },
    { 0xf27c11c0, 23 },
    { 0xd15c11c2, 100 },
    { 0x73ec11c2, 43 },
    { 0xa09c11c5, 95 },
    { 0x5fdc11c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8c11cc, 33 },
    { 0xf36c11cd, 95 },
    { 0x35bc11cd, 33 },
    { 0, 0 },
    { 0x325c11d0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974c11d9, 95 },
    { 0x2c4c11da, 85 },
    { 0x9eac11d9, 90 },
    { 0xbd8c11dc, 34 },
    { 0xb29c11dd, 33 },
    { 0x2a0c11d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc11e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcc11f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43c11f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c11f9, 100 },
    { 0x995c11fa, 98 },
    { 0, 0 },
    { 0x29ac11fc, 95 },
    { 0x3fc11fd, 88 },
    { 0xaa4c11fe, 61 },
    { 0xb09c11ff, 90 },
    { 0x4ebc11ff, 23 },
    { 0x410c1200, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dc1205, 23 },
    { 0x755c1205, 100 },
    { 0x160c1206, 88 },
    { 0x6e8c1207, 33 },
    { 0x5d6c1209, 43 },
    { 0x41bc1209, 61 },
    { 0x221c1205, 96 },
    { 0xf32c1209, 61 },
    { 0, 0 },
    { 0xd6ec120e, 76 },
    { 0x539c120f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c1213, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ec121d, 95 },
    { 0xb7dc121d, 33 },
    { 0xf31c121f, 33 },
    { 0x48fc1220, 90 },
    { 0xb06c1221, 61 },
    { 0, 0 },
    { 0x36ec1223, 61 },
    { 0x942c1223, 95 },
    { 0xd14c1224, 61 },
    { 0, 0 },
    { 0xadec1227, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311c122d, 55 },
    { 0, 0 },
    { 0x8eac122f, 98 },
    { 0x602c1230, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c1233, 66 },
    { 0xd0ac1233, 95 },
    { 0x9e4c1235, 33 },
    { 0xca3c1236, 61 },
    { 0xcadc1233, 90 },
    { 0x268c1238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98c1240, 81 },
    { 0xe76c1241, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddc1247, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cc124a, 33 },
    { 0x704c124a, 55 },
    { 0x96ec124b, 76 },
    { 0xa8dc124d, 91 },
    { 0xbf0c124a, 75 },
    { 0x2ddc124f, 98 },
    { 0xdd0c124f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c1255, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86c1258, 46 },
    { 0xf01c1259, 46 },
    { 0xaddc1259, 61 },
    { 0xfd4c125b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac125f, 84 },
    { 0x8bac125f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7c1268, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c126d, 95 },
    { 0x231c126d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1c1272, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705c127b, 61 },
    { 0x44bc127c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283c1280, 95 },
    { 0, 0 },
    { 0x480c1282, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90c1285, 23 },
    { 0, 0 },
    { 0xd6bc1287, 90 },
    { 0x464c1287, 98 },
    { 0xe75c1289, 77 },
    { 0xf33c128a, 95 },
    { 0xba7c128a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc128e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x169c1291, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc1294, 90 },
    { 0, 0 },
    { 0x4bcc1296, 95 },
    { 0x1efc1296, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dc129f, 77 },
    { 0xde0c129f, 61 },
    { 0x1b9c12a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1c12a5, 90 },
    { 0, 0 },
    { 0x698c12a7, 55 },
    { 0xfb5c12a8, 98 },
    { 0xbfec12a8, 23 },
    { 0x87cc12aa, 33 },
    { 0x278c12a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5c12b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c12b8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436c12c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ec12ce, 81 },
    { 0, 0 },
    { 0x15bc12d0, 33 },
    { 0, 0 },
    { 0xfefc12d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471c12d8, 39 },
    { 0x5a6c12d8, 61 },
    { 0x7eec12da, 76 },
    { 0x352c12d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eec12e2, 90 },
    { 0xdf1c12e2, 33 },
    { 0x783c12e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687c12e8, 87 },
    { 0x141c12e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525c12ed, 55 },
    { 0x7b3c12ee, 76 },
    { 0xc0ac12ee, 52 },
    { 0xf1ec12f0, 84 },
    { 0x3aec12f1, 23 },
    { 0xb79c12f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ac12f7, 98 },
    { 0x1e8c12f8, 85 },
    { 0x465c12f9, 76 },
    { 0x681c12fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850c1300, 33 },
    { 0x6e0c1301, 33 },
    { 0x420c1302, 61 },
    { 0, 0 },
    { 0x30ec1304, 84 },
    { 0xe19c1304, 99 },
    { 0x644c1306, 90 },
    { 0xfbac1307, 61 },
    { 0x417c1308, 78 },
    { 0x2c1309, 90 },
    { 0xfdcc130a, 93 },
    { 0, 0 },
    { 0x41fc130c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c1312, 61 },
    { 0, 0 },
    { 0x9a6c1314, 84 },
    { 0x58fc1315, 90 },
    { 0x76dc1316, 33 },
    { 0, 0 },
    { 0xca4c1318, 76 },
    { 0x398c1319, 61 },
    { 0, 0 },
    { 0xb61c131b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fc131e, 75 },
    { 0x34bc131f, 68 },
    { 0xce2c1320, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc1324, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc132a, 23 },
    { 0xde2c132a, 90 },
    { 0x6dc132c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfc132f, 98 },
    { 0xf3dc1330, 33 },
    { 0, 0 },
    { 0x925c1332, 98 },
    { 0xbb6c1333, 33 },
    { 0x2e5c1334, 90 },
    { 0, 0 },
    { 0xf30c1336, 33 },
    { 0x4b8c1337, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ac133a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6c133f, 66 },
    { 0xcccc133f, 23 },
    { 0, 0 },
    { 0xfbc1342, 88 },
    { 0x2cc1343, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821c1349, 34 },
    { 0x49fc1349, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc1352, 55 },
    { 0x561c1353, 61 },
    { 0x9ec1354, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80c1357, 61 },
    { 0x788c1357, 46 },
    { 0x7f5c1357, 55 },
    { 0x3a0c135a, 95 },
    { 0, 0 },
    { 0xc39c135c, 90 },
    { 0xa1ec135d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c1360, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c1364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23c1367, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dc136a, 90 },
    { 0, 0 },
    { 0xc1fc136c, 53 },
    { 0x8a4c136d, 95 },
    { 0x227c136d, 61 },
    { 0xaf9c136f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68c1378, 55 },
    { 0xe14c1378, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bc137c, 88 },
    { 0xfd8c137d, 69 },
    { 0x828c137d, 90 },
    { 0xc56c137c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c1385, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c138d, 34 },
    { 0, 0 },
    { 0xfbdc138f, 61 },
    { 0x6a6c1390, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448c139b, 43 },
    { 0x46ac139c, 61 },
    { 0x43ec139d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dc13a0, 61 },
    { 0xa28c13a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x910c13a4, 98 },
    { 0x2c0c13a5, 99 },
    { 0x45c13a6, 34 },
    { 0x90c13a5, 100 },
    { 0x3cc13a8, 90 },
    { 0xbf5c13a5, 98 },
    { 0x6b4c13a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2c13ad, 23 },
    { 0xa68c13ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ec13b6, 33 },
    { 0x8ffc13b7, 55 },
    { 0x4f6c13b8, 76 },
    { 0xddac13b8, 90 },
    { 0xb68c13b7, 43 },
    { 0xa7ec13bb, 90 },
    { 0xee1c13bb, 43 },
    { 0x45dc13bd, 55 },
    { 0, 0 },
    { 0x921c13bf, 90 },
    { 0xbdcc13bf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c13c5, 23 },
    { 0xf65c13c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35c13ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x194c13cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245c13d1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x217c13d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x317c13d7, 90 },
    { 0, 0 },
    { 0x270c13d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ec13dc, 99 },
    { 0, 0 },
    { 0xd72c13de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc13e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c13ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdc13f2, 90 },
    { 0xa7ec13f3, 34 },
    { 0xb74c13f4, 52 },
    { 0xd08c13f5, 61 },
    { 0xd14c13f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x770c13f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3c13fc, 90 },
    { 0xecc13fc, 55 },
    { 0x5d7c13fe, 66 },
    { 0x4c3c13ff, 90 },
    { 0x67ac13fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dac1408, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbc140b, 93 },
    { 0x2ebc140b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x125c140f, 60 },
    { 0x73ec1410, 33 },
    { 0x783c1410, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4c1416, 34 },
    { 0x803c1417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x873c141a, 65 },
    { 0x93dc141b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2c141f, 87 },
    { 0x904c1420, 76 },
    { 0x9eec1421, 90 },
    { 0xa48c1422, 61 },
    { 0x72bc1423, 88 },
    { 0, 0 },
    { 0xe20c1425, 33 },
    { 0x58cc1426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05c142b, 98 },
    { 0x96ec142c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cac1432, 95 },
    { 0x2b7c1433, 91 },
    { 0, 0 },
    { 0x105c1435, 52 },
    { 0, 0 },
    { 0xfc4c1437, 65 },
    { 0, 0 },
    { 0xd42c1439, 84 },
    { 0, 0 },
    { 0x680c143b, 98 },
    { 0x413c143c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24c143f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504c1445, 90 },
    { 0x8eac1446, 87 },
    { 0xf6fc1447, 38 },
    { 0x2fac1448, 90 },
    { 0xe37c1448, 66 },
    { 0x2a9c144a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06c144d, 98 },
    { 0, 0 },
    { 0x7d2c144f, 90 },
    { 0xca0c1450, 85 },
    { 0x670c144f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6c1456, 67 },
    { 0x1a1c1457, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c145f, 90 },
    { 0xc7ac145f, 84 },
    { 0xbfcc1461, 23 },
    { 0, 0 },
    { 0xf4dc1463, 23 },
    { 0xd8cc1464, 90 },
    { 0, 0 },
    { 0x75ec1466, 23 },
    { 0x1c4c1467, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bc146a, 67 },
    { 0x2a1c146b, 23 },
    { 0xdadc146c, 76 },
    { 0x9c9c146d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fc1474, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6c1477, 95 },
    { 0, 0 },
    { 0x2bac1479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2c147c, 100 },
    { 0x582c147d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ac1484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545c148f, 100 },
    { 0x8e6c148f, 33 },
    { 0x776c1491, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08c149a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9c149e, 69 },
    { 0, 0 },
    { 0x5a7c14a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fc14a5, 55 },
    { 0xf46c14a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfc14af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbec14b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x278c14b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc14b9, 66 },
    { 0, 0 },
    { 0x9bfc14bb, 85 },
    { 0x1a6c14bc, 76 },
    { 0x6aec14bb, 91 },
    { 0x11fc14be, 48 },
    { 0xbd1c14bf, 23 },
    { 0x890c14be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58c14c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ec14c9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8c14cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327c14d3, 95 },
    { 0x25bc14d4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc14d7, 90 },
    { 0x92ac14d7, 77 },
    { 0x690c14d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95c14de, 95 },
    { 0xc38c14df, 33 },
    { 0x2f7c14e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c14e5, 90 },
    { 0, 0 },
    { 0x2e2c14e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1c14ed, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904c14f1, 83 },
    { 0x19ec14f1, 23 },
    { 0x10ac14f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491c14f7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9c14fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ac14ff, 90 },
    { 0, 0 },
    { 0xcd6c1501, 61 },
    { 0, 0 },
    { 0x751c1503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02c1507, 90 },
    { 0x2f0c1508, 55 },
    { 0xea6c1508, 87 },
    { 0, 0 },
    { 0x68dc150b, 33 },
    { 0, 0 },
    { 0xc77c150d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eec1510, 90 },
    { 0x2e0c1510, 61 },
    { 0, 0 },
    { 0x5c9c1513, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c1519, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84c1525, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcc152b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ac152f, 48 },
    { 0xef7c1530, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c1535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ac153d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2c1540, 61 },
    { 0x132c1541, 90 },
    { 0, 0 },
    { 0x505c1543, 36 },
    { 0, 0 },
    { 0x69fc1545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91c1548, 97 },
    { 0, 0 },
    { 0x3fdc154a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c154e, 61 },
    { 0xe30c154f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fc1554, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1c1557, 46 },
    { 0x360c1557, 90 },
    { 0x3b0c1557, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43c155d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c1561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec1564, 19 },
    { 0xcd5c1565, 76 },
    { 0x430c1566, 33 },
    { 0x39ec1567, 61 },
    { 0xf01c1565, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764c156c, 91 },
    { 0xd31c156d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3c1570, 23 },
    { 0x83ac1571, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12c1577, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dc157b, 66 },
    { 0, 0 },
    { 0x2bc157d, 43 },
    { 0xe59c157e, 23 },
    { 0xf7c157e, 33 },
    { 0x38fc1580, 83 },
    { 0xb8cc1580, 87 },
    { 0, 0 },
    { 0x9a2c1583, 84 },
    { 0xfebc1584, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0c1589, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4c158c, 98 },
    { 0, 0 },
    { 0x671c158e, 61 },
    { 0xd06c158e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cc1592, 55 },
    { 0, 0 },
    { 0x288c1594, 61 },
    { 0x582c1595, 90 },
    { 0x3cec1595, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cc1599, 85 },
    { 0, 0 },
    { 0x737c159b, 85 },
    { 0x4d1c159b, 95 },
    { 0x3f9c159d, 33 },
    { 0xb94c159b, 55 },
    { 0x963c159f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7c15a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630c15aa, 90 },
    { 0, 0 },
    { 0xebac15ac, 90 },
    { 0x69dc15ad, 98 },
    { 0xcacc15ae, 46 },
    { 0, 0 },
    { 0x742c15b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x895c15b3, 80 },
    { 0xba4c15b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25c15bc, 90 },
    { 0, 0 },
    { 0x21ac15be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513c15c2, 98 },
    { 0x1d2c15c3, 95 },
    { 0, 0 },
    { 0xc91c15c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fc15c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c15cb, 33 },
    { 0xd06c15cc, 23 },
    { 0x2c3c15cd, 90 },
    { 0x78ac15cc, 34 },
    { 0xc0fc15cb, 61 },
    { 0xd89c15d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26c15d3, 33 },
    { 0xafc15d4, 66 },
    { 0x511c15d5, 33 },
    { 0x40ec15d6, 52 },
    { 0x84cc15d6, 33 },
    { 0xe0dc15d6, 61 },
    { 0, 0 },
    { 0x3a8c15da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc15dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc15e0, 34 },
    { 0, 0 },
    { 0x3e9c15e2, 95 },
    { 0, 0 },
    { 0xd37c15e4, 46 },
    { 0x92cc15e5, 98 },
    { 0, 0 },
    { 0xfafc15e7, 87 },
    { 0xe2bc15e8, 23 },
    { 0x49ac15e9, 55 },
    { 0x693c15ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16c15ed, 93 },
    { 0x67bc15ee, 66 },
    { 0x6b2c15ee, 43 },
    { 0x371c15ed, 61 },
    { 0, 0 },
    { 0xeeac15f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57c15f5, 61 },
    { 0x79ec15f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155c1600, 90 },
    { 0x9d5c1600, 100 },
    { 0x75ac1602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfc1605, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc160f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2c1613, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bc1617, 69 },
    { 0x49cc1618, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ec161b, 98 },
    { 0xd8fc161b, 33 },
    { 0x905c161d, 33 },
    { 0, 0 },
    { 0xb73c161f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc1622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64c162a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14c1630, 61 },
    { 0x91c1631, 34 },
    { 0, 0 },
    { 0x11cc1633, 90 },
    { 0x9f4c1633, 23 },
    { 0x3c4c1633, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ac163b, 81 },
    { 0, 0 },
    { 0x4a4c163d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03c1641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x865c1644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9c1648, 90 },
    { 0xd30c1649, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdc164c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4c164f, 20 },
    { 0xf78c1650, 23 },
    { 0xef0c1651, 78 },
    { 0x829c1651, 99 },
    { 0x8ac1651, 95 },
    { 0xc79c1650, 93 },
    { 0x69bc1651, 90 },
    { 0xfd1c1656, 99 },
    { 0x29fc1657, 80 },
    { 0, 0 },
    { 0x499c1659, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dc1660, 67 },
    { 0x9fac1661, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcc1665, 95 },
    { 0x385c1666, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751c166a, 95 },
    { 0, 0 },
    { 0x677c166c, 93 },
    { 0xca8c166c, 90 },
    { 0, 0 },
    { 0x2c8c166f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c1672, 93 },
    { 0x100c1673, 90 },
    { 0x4b0c1674, 90 },
    { 0x4c3c1674, 76 },
    { 0x242c1675, 33 },
    { 0xd92c1675, 55 },
    { 0x2c3c1673, 34 },
    { 0, 0 },
    { 0xcc8c167a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756c1683, 90 },
    { 0, 0 },
    { 0x5bdc1685, 95 },
    { 0xe91c1685, 77 },
    { 0, 0 },
    { 0x11ec1688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100c1693, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fc1699, 34 },
    { 0, 0 },
    { 0x3f8c169b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac16a0, 98 },
    { 0xd25c16a1, 67 },
    { 0x96c16a0, 90 },
    { 0, 0 },
    { 0x1dcc16a4, 61 },
    { 0, 0 },
    { 0x2adc16a6, 55 },
    { 0x7d0c16a7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c16ab, 90 },
    { 0x9c7c16ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dc16b1, 55 },
    { 0x2a6c16b2, 61 },
    { 0x26bc16b3, 97 },
    { 0xc7fc16b4, 76 },
    { 0x62ac16b3, 33 },
    { 0x7b1c16b6, 90 },
    { 0xfb4c16b1, 61 },
    { 0, 0 },
    { 0x280c16b9, 61 },
    { 0x76cc16ba, 43 },
    { 0x11ac16ba, 98 },
    { 0x9cfc16bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bc16c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2c16c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7c16d6, 33 },
    { 0xb8ec16d7, 33 },
    { 0, 0 },
    { 0x485c16d9, 99 },
    { 0, 0 },
    { 0x4c8c16db, 95 },
    { 0x9e4c16dc, 78 },
    { 0x7f5c16db, 33 },
    { 0x8f3c16dc, 17 },
    { 0, 0 },
    { 0xd49c16e0, 61 },
    { 0xbfac16e1, 3 },
    { 0, 0 },
    { 0x510c16e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bc16e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x396c16eb, 33 },
    { 0xf39c16eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9c16ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac16f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ac16f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fc1700, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdc1706, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bc1710, 99 },
    { 0xc58c1711, 76 },
    { 0x543c1711, 33 },
    { 0, 0 },
    { 0x916c1714, 85 },
    { 0x67fc1715, 66 },
    { 0, 0 },
    { 0x918c1717, 97 },
    { 0x720c1718, 67 },
    { 0x31bc1719, 43 },
    { 0xc92c171a, 61 },
    { 0x86cc171b, 61 },
    { 0x19ec171c, 68 },
    { 0x80bc171d, 90 },
    { 0xaec171e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8c1723, 33 },
    { 0xe6c1724, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655c172a, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83c172d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347c1733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ec1736, 90 },
    { 0xc92c1736, 76 },
    { 0, 0 },
    { 0xfa9c1739, 95 },
    { 0x8f0c173a, 90 },
    { 0, 0 },
    { 0x79dc173c, 33 },
    { 0x346c173d, 90 },
    { 0, 0 },
    { 0xaddc173f, 43 },
    { 0xa10c173f, 43 },
    { 0x332c173f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37c1744, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec174c, 87 },
    { 0xeb5c174d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x296c1750, 46 },
    { 0, 0 },
    { 0x2b7c1752, 98 },
    { 0xb44c1753, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcc1759, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4c175c, 61 },
    { 0x662c175d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6c1766, 90 },
    { 0xcfdc1767, 61 },
    { 0xc7ec1768, 52 },
    { 0xd6c1768, 90 },
    { 0xd01c176a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35c176d, 85 },
    { 0xab9c176d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41c1771, 83 },
    { 0x84c1771, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eac1777, 34 },
    { 0x6b3c1777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfc177f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599c1783, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91c1786, 95 },
    { 0x3f7c1786, 33 },
    { 0, 0 },
    { 0x1a6c1789, 87 },
    { 0x6ec178a, 55 },
    { 0, 0 },
    { 0x901c178c, 78 },
    { 0, 0 },
    { 0x994c178e, 61 },
    { 0, 0 },
    { 0x8a4c1790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c1793, 98 },
    { 0xc52c1793, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7c1798, 23 },
    { 0x863c1799, 99 },
    { 0x417c179a, 80 },
    { 0xa00c179b, 33 },
    { 0x525c179c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0c179f, 95 },
    { 0x374c17a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c17a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc17a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583c17b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dc17b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96c17ba, 78 },
    { 0xb8dc17bb, 46 },
    { 0, 0 },
    { 0x699c17bd, 34 },
    { 0, 0 },
    { 0xb58c17bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cc17c2, 88 },
    { 0xc7c17c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdc17c8, 76 },
    { 0x5ec17c8, 76 },
    { 0xe03c17ca, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176c17ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c17d5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c17d8, 23 },
    { 0xefec17d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c17dd, 55 },
    { 0xc99c17dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686c17e4, 33 },
    { 0x2b2c17e4, 98 },
    { 0x183c17e6, 43 },
    { 0x67ac17e7, 23 },
    { 0x5d1c17e8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36c17ee, 33 },
    { 0x4ac17ef, 66 },
    { 0xe04c17f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbac17f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533c17fd, 61 },
    { 0, 0 },
    { 0xf33c17ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x607c1802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fc1806, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210c180d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaac181a, 17 },
    { 0xf7bc181a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c1820, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafc1829, 33 },
    { 0, 0 },
    { 0xe60c182b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9c1830, 95 },
    { 0x91ac1831, 61 },
    { 0, 0 },
    { 0x37fc1833, 77 },
    { 0x33bc1834, 90 },
    { 0x645c1833, 76 },
    { 0, 0 },
    { 0x450c1837, 55 },
    { 0, 0 },
    { 0x1e8c1839, 90 },
    { 0, 0 },
    { 0x23cc183b, 95 },
    { 0x1e9c183b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84c183f, 95 },
    { 0xc4ac183f, 33 },
    { 0x1d0c183f, 33 },
    { 0x810c1842, 93 },
    { 0x92ac1843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c184c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c1850, 52 },
    { 0x850c1851, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c1854, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c1858, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161c185d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c1869, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30c186e, 43 },
    { 0xac6c186f, 55 },
    { 0xebbc1870, 77 },
    { 0, 0 },
    { 0x90c1872, 95 },
    { 0x921c1873, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefc1879, 77 },
    { 0x6b6c1879, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4c187f, 46 },
    { 0x1a8c187f, 43 },
    { 0x3ec1881, 98 },
    { 0x9f7c187f, 95 },
    { 0xa94c1883, 90 },
    { 0, 0 },
    { 0xceec1885, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126c188a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc1891, 66 },
    { 0xc06c1892, 52 },
    { 0xf75c1891, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5c1896, 87 },
    { 0, 0 },
    { 0x219c1898, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dc189b, 73 },
    { 0x4d2c189c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c18a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c18aa, 69 },
    { 0xcbfc18aa, 76 },
    { 0x855c18ac, 84 },
    { 0x3b7c18ad, 68 },
    { 0x6b8c18ae, 95 },
    { 0xb7bc18ae, 23 },
    { 0x447c18af, 61 },
    { 0xb42c18ae, 23 },
    { 0x2afc18b1, 55 },
    { 0x3dec18b3, 69 },
    { 0, 0 },
    { 0x8a8c18b5, 66 },
    { 0xfb6c18b6, 43 },
    { 0x59cc18b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c18bc, 61 },
    { 0xd21c18bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0c18c0, 78 },
    { 0xa44c18c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567c18c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dc18ca, 81 },
    { 0xdb2c18cb, 34 },
    { 0x183c18cb, 61 },
    { 0x87fc18cc, 61 },
    { 0x3d1c18ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc18d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bc18d7, 90 },
    { 0x391c18d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5c18db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fc18e1, 61 },
    { 0, 0 },
    { 0x8b6c18e3, 76 },
    { 0x199c18e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52c18e7, 90 },
    { 0xfb2c18e7, 69 },
    { 0x989c18e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99c18f0, 33 },
    { 0x90bc18f1, 98 },
    { 0, 0 },
    { 0x977c18f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163c18fa, 90 },
    { 0x613c18fb, 92 },
    { 0x7b7c18fc, 85 },
    { 0, 0 },
    { 0x671c18fe, 90 },
    { 0x300c18fe, 33 },
    { 0xe5ac1900, 95 },
    { 0xd8ac18ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421c1909, 34 },
    { 0x450c190a, 61 },
    { 0xe74c1909, 95 },
    { 0, 0 },
    { 0xfcec190d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61c1912, 33 },
    { 0, 0 },
    { 0xf06c1914, 17 },
    { 0x768c1915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c1918, 87 },
    { 0x739c1919, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bc191f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cc1925, 33 },
    { 0xc4c1925, 90 },
    { 0, 0 },
    { 0xc56c1928, 100 },
    { 0xc7bc1928, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c192c, 95 },
    { 0, 0 },
    { 0x134c192e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c1931, 96 },
    { 0xe07c1931, 23 },
    { 0x850c1933, 61 },
    { 0, 0 },
    { 0x770c1935, 33 },
    { 0x7d1c1936, 76 },
    { 0x2d3c1937, 90 },
    { 0, 0 },
    { 0x15ec1939, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x859c193c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c1944, 95 },
    { 0, 0 },
    { 0x48ac1946, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0c194a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80c194d, 95 },
    { 0, 0 },
    { 0x5e5c194f, 33 },
    { 0xb1c1950, 93 },
    { 0xf90c194f, 33 },
    { 0x645c194f, 33 },
    { 0x561c1953, 17 },
    { 0xac8c1953, 61 },
    { 0xd9ec1955, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0c1958, 33 },
    { 0xaedc1958, 98 },
    { 0x134c1958, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7c195e, 61 },
    { 0x272c195e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x573c1962, 17 },
    { 0x51c1962, 33 },
    { 0, 0 },
    { 0x81bc1965, 17 },
    { 0, 0 },
    { 0x517c1967, 52 },
    { 0x12ec1968, 95 },
    { 0x98dc1968, 43 },
    { 0x9e5c196a, 90 },
    { 0x97dc1968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8c197b, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5c197e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x923c1981, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185c1985, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958c198c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeac1990, 67 },
    { 0, 0 },
    { 0xe3cc1992, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0c1995, 55 },
    { 0xda0c1996, 33 },
    { 0x27cc1996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c199e, 77 },
    { 0, 0 },
    { 0x162c19a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdc19a6, 90 },
    { 0xbe3c19a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c19b0, 81 },
    { 0, 0 },
    { 0x5b4c19b2, 90 },
    { 0, 0 },
    { 0x2b5c19b4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ac19b8, 55 },
    { 0xd54c19b9, 85 },
    { 0x259c19b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc19bf, 46 },
    { 0x43ec19c0, 91 },
    { 0, 0 },
    { 0xfc7c19c2, 84 },
    { 0, 0 },
    { 0x445c19c4, 76 },
    { 0, 0 },
    { 0x71dc19c6, 81 },
    { 0x81cc19c7, 90 },
    { 0, 0 },
    { 0xbbfc19c9, 61 },
    { 0, 0 },
    { 0xdc2c19cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5c19d4, 52 },
    { 0x3ebc19d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bc19da, 99 },
    { 0, 0 },
    { 0x825c19dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bc19df, 34 },
    { 0x4b8c19df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x300c19e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c19e9, 61 },
    { 0x2b0c19ea, 33 },
    { 0, 0 },
    { 0x735c19ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7c19f4, 90 },
    { 0x574c19f4, 43 },
    { 0x5ffc19f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dc19fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fc1a00, 95 },
    { 0x188c1a01, 81 },
    { 0xc9dc1a02, 90 },
    { 0xc53c1a02, 95 },
    { 0xc93c1a00, 99 },
    { 0xc1ec1a05, 35 },
    { 0xae4c1a01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c1a09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0c1a0c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c1a13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dc1a16, 69 },
    { 0xaaac1a17, 43 },
    { 0x872c1a17, 95 },
    { 0, 0 },
    { 0x47dc1a1a, 23 },
    { 0, 0 },
    { 0x229c1a1c, 61 },
    { 0x506c1a1d, 90 },
    { 0xe9fc1a1e, 66 },
    { 0x2a5c1a1f, 93 },
    { 0x78cc1a1f, 33 },
    { 0xfc1a1d, 38 },
    { 0x32c1a21, 23 },
    { 0xe8c1a23, 61 },
    { 0, 0 },
    { 0x9b2c1a25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550c1a2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cc1a2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbc1a32, 33 },
    { 0, 0 },
    { 0x30ec1a34, 4 },
    { 0, 0 },
    { 0xf01c1a36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecc1a3c, 48 },
    { 0x167c1a3d, 23 },
    { 0x783c1a3c, 90 },
    { 0, 0 },
    { 0xf38c1a40, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x146c1a43, 84 },
    { 0x6fbc1a44, 23 },
    { 0xc80c1a44, 78 },
    { 0x9efc1a45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fc1a54, 61 },
    { 0x389c1a54, 34 },
    { 0xdb2c1a55, 93 },
    { 0xb24c1a57, 98 },
    { 0x5a6c1a56, 55 },
    { 0x6fbc1a59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fc1a5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495c1a69, 46 },
    { 0xc7ec1a69, 61 },
    { 0xd39c1a6a, 61 },
    { 0xce7c1a6c, 46 },
    { 0x33dc1a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ac1a75, 100 },
    { 0xa7ec1a76, 40 },
    { 0xaac1a76, 95 },
    { 0x27ec1a78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ac1a7d, 90 },
    { 0x9bfc1a7d, 40 },
    { 0x821c1a7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacec1a85, 52 },
    { 0, 0 },
    { 0xe64c1a87, 61 },
    { 0x3fc1a88, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dc1a8b, 33 },
    { 0x980c1a8b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81c1a8f, 98 },
    { 0, 0 },
    { 0xd99c1a91, 90 },
    { 0, 0 },
    { 0x358c1a93, 55 },
    { 0, 0 },
    { 0xc77c1a95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76c1a99, 46 },
    { 0xfc7c1a9a, 90 },
    { 0, 0 },
    { 0x9a0c1a9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4c1a9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6c1aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1c1aa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c1aa9, 34 },
    { 0x280c1aaa, 23 },
    { 0x643c1aa9, 34 },
    { 0xae6c1aa9, 99 },
    { 0xd46c1aad, 90 },
    { 0xc1cc1aad, 61 },
    { 0xf2fc1aad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50c1ab3, 66 },
    { 0x391c1ab4, 61 },
    { 0xc43c1ab5, 91 },
    { 0x828c1ab3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359c1abb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec1abe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c1ac3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5c1ac6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7c1ada, 36 },
    { 0x58c1adb, 95 },
    { 0xddec1adc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c1adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc1ae3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8c1ae6, 55 },
    { 0xe39c1ae7, 40 },
    { 0xe96c1ae6, 90 },
    { 0, 0 },
    { 0xa3cc1aea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc1aed, 90 },
    { 0x825c1aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23c1af2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c1af7, 61 },
    { 0xbf8c1af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc1afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c1b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ec1b08, 90 },
    { 0x445c1b09, 91 },
    { 0x7e8c1b0a, 92 },
    { 0x89c1b0b, 87 },
    { 0x64fc1b0b, 90 },
    { 0, 0 },
    { 0x977c1b0e, 76 },
    { 0x2edc1b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3c1b16, 66 },
    { 0xe11c1b16, 98 },
    { 0x31bc1b18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cc1b1d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc1b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adc1b29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6c1b3b, 43 },
    { 0, 0 },
    { 0x11cc1b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cc1b40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31c1b46, 43 },
    { 0x62bc1b46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0c1b4b, 34 },
    { 0x60ec1b4c, 61 },
    { 0, 0 },
    { 0x50c1b4e, 97 },
    { 0, 0 },
    { 0xeb0c1b50, 33 },
    { 0, 0 },
    { 0xf4bc1b52, 61 },
    { 0x82fc1b53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfc1b58, 98 },
    { 0x676c1b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2c1b5c, 34 },
    { 0, 0 },
    { 0x322c1b5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2c1b61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908c1b67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bc1b6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fc1b70, 98 },
    { 0, 0 },
    { 0x78ec1b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c1b75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x741c1b78, 33 },
    { 0, 0 },
    { 0x487c1b7a, 98 },
    { 0, 0 },
    { 0x556c1b7c, 34 },
    { 0, 0 },
    { 0x103c1b7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bc1b85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7c1b88, 52 },
    { 0x348c1b88, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979c1b8f, 95 },
    { 0xc3fc1b90, 90 },
    { 0xc7cc1b90, 84 },
    { 0, 0 },
    { 0x913c1b93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3c1b97, 95 },
    { 0x24cc1b98, 33 },
    { 0xb4ac1b98, 61 },
    { 0x53ac1b98, 100 },
    { 0xe1ac1b9b, 55 },
    { 0x143c1b99, 61 },
    { 0xdfec1b9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cc1ba0, 85 },
    { 0xec7c1ba1, 90 },
    { 0xd8cc1ba2, 76 },
    { 0x19ac1ba3, 91 },
    { 0xc9ec1ba2, 98 },
    { 0xe86c1ba2, 76 },
    { 0, 0 },
    { 0x22ec1ba7, 66 },
    { 0, 0 },
    { 0xd5dc1ba9, 100 },
    { 0x9f2c1baa, 90 },
    { 0xfc8c1bab, 95 },
    { 0xdfbc1bab, 33 },
    { 0xe2fc1bac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafc1bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cc1bb4, 66 },
    { 0, 0 },
    { 0x7f3c1bb6, 61 },
    { 0x165c1bb7, 43 },
    { 0x3b6c1bb8, 23 },
    { 0xce2c1bb8, 98 },
    { 0xd1cc1bb6, 90 },
    { 0xc2fc1bbb, 95 },
    { 0, 0 },
    { 0xa6cc1bbd, 88 },
    { 0x62dc1bbe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c1bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac1bca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bc1bd2, 90 },
    { 0, 0 },
    { 0xd3c1bd4, 90 },
    { 0x654c1bd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c1bdf, 23 },
    { 0x32fc1bdf, 80 },
    { 0, 0 },
    { 0x5cac1be2, 90 },
    { 0x53fc1be3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5c1be6, 44 },
    { 0x79fc1be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6c1bf1, 55 },
    { 0, 0 },
    { 0x43bc1bf3, 67 },
    { 0, 0 },
    { 0x4ddc1bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec1bf8, 76 },
    { 0xdeac1bf9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc1c00, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0c1c03, 85 },
    { 0, 0 },
    { 0x2dec1c05, 95 },
    { 0x1edc1c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dc1c09, 52 },
    { 0, 0 },
    { 0x491c1c0b, 46 },
    { 0, 0 },
    { 0x399c1c0d, 99 },
    { 0, 0 },
    { 0xf1bc1c0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55c1c12, 90 },
    { 0xb87c1c12, 33 },
    { 0x891c1c14, 56 },
    { 0x83cc1c15, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8c1c18, 43 },
    { 0x4a0c1c19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dc1c1d, 55 },
    { 0x418c1c1e, 98 },
    { 0, 0 },
    { 0xf03c1c20, 23 },
    { 0xf72c1c20, 90 },
    { 0x3b8c1c20, 76 },
    { 0, 0 },
    { 0xa1ec1c24, 98 },
    { 0, 0 },
    { 0x562c1c26, 75 },
    { 0, 0 },
    { 0xfb7c1c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfc1c2e, 23 },
    { 0xddac1c2f, 98 },
    { 0, 0 },
    { 0x65dc1c31, 90 },
    { 0x11c1c32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fc1c35, 43 },
    { 0xa10c1c36, 33 },
    { 0, 0 },
    { 0xa53c1c38, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fc1c3b, 95 },
    { 0, 0 },
    { 0x210c1c3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ac1c40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdac1c43, 34 },
    { 0x584c1c43, 100 },
    { 0x69bc1c45, 91 },
    { 0x400c1c46, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bc1c4b, 90 },
    { 0x51c1c4b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8c1c50, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffc1c56, 38 },
    { 0, 0 },
    { 0x673c1c58, 43 },
    { 0x959c1c58, 61 },
    { 0, 0 },
    { 0x468c1c5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708c1c5f, 90 },
    { 0xbe0c1c60, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136c1c64, 23 },
    { 0, 0 },
    { 0xac2c1c66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9c1c6a, 33 },
    { 0, 0 },
    { 0x6cec1c6c, 90 },
    { 0xb83c1c6d, 78 },
    { 0xeafc1c6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460c1c73, 23 },
    { 0x178c1c73, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bc1c77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac1c7d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdc1c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577c1c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89c1c8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ac1c92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec1c95, 61 },
    { 0xd19c1c96, 34 },
    { 0x21dc1c96, 33 },
    { 0xf38c1c95, 43 },
    { 0x74bc1c95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c1c9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x584c1ca0, 61 },
    { 0, 0 },
    { 0x3cc1ca2, 90 },
    { 0, 0 },
    { 0x898c1ca4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292c1ca8, 93 },
    { 0xfc5c1ca9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25c1cad, 67 },
    { 0x292c1cae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5c1cb4, 96 },
    { 0xd3fc1cb4, 87 },
    { 0x21ac1cb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cc1cba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cc1cc2, 33 },
    { 0xbf2c1cc2, 61 },
    { 0, 0 },
    { 0xa4cc1cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c1cc8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fc1ccf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bc1cd2, 23 },
    { 0xdcdc1cd3, 66 },
    { 0, 0 },
    { 0x2f8c1cd5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bc1cda, 99 },
    { 0x133c1cda, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4c1cdf, 76 },
    { 0xe6cc1cdf, 99 },
    { 0, 0 },
    { 0xe69c1ce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c1ce6, 53 },
    { 0x8fac1ce7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c1ceb, 85 },
    { 0xc3dc1cec, 98 },
    { 0xe1dc1cec, 43 },
    { 0x6c2c1cec, 90 },
    { 0xd5ec1cef, 33 },
    { 0x8abc1cef, 52 },
    { 0xee5c1cf1, 46 },
    { 0xd78c1cf2, 65 },
    { 0x7a2c1cee, 33 },
    { 0xf13c1cf4, 98 },
    { 0xda2c1cee, 90 },
    { 0x4b1c1cf3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaec1cf9, 55 },
    { 0x228c1cfa, 95 },
    { 0x1a8c1cfa, 46 },
    { 0, 0 },
    { 0xfa9c1cfd, 98 },
    { 0x38c1cfe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62c1d02, 33 },
    { 0xf4dc1d02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c1d0a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cc1d0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c1d17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eec1d1d, 76 },
    { 0, 0 },
    { 0x9eec1d1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c1d22, 92 },
    { 0xcc1d23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x449c1d26, 95 },
    { 0xa02c1d27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c1d2b, 69 },
    { 0xebec1d2c, 87 },
    { 0, 0 },
    { 0xf36c1d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38c1d32, 98 },
    { 0, 0 },
    { 0x39cc1d34, 95 },
    { 0xac9c1d34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ec1d3e, 46 },
    { 0, 0 },
    { 0x285c1d40, 61 },
    { 0, 0 },
    { 0xc72c1d42, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387c1d46, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ec1d4a, 23 },
    { 0xeafc1d4b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6c1d4e, 61 },
    { 0xd84c1d4f, 84 },
    { 0, 0 },
    { 0xfcbc1d51, 33 },
    { 0xae7c1d52, 65 },
    { 0x949c1d53, 99 },
    { 0x91ac1d51, 95 },
    { 0xacac1d55, 95 },
    { 0xc27c1d56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02c1d5b, 90 },
    { 0, 0 },
    { 0x64dc1d5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c1d61, 46 },
    { 0xbfec1d62, 98 },
    { 0x9dc1d63, 33 },
    { 0x2e8c1d64, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43c1d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45dc1d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdc1d71, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x788c1d74, 90 },
    { 0x588c1d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59c1d7a, 40 },
    { 0, 0 },
    { 0x197c1d7c, 33 },
    { 0x45c1d7d, 76 },
    { 0xa2cc1d7e, 98 },
    { 0, 0 },
    { 0x17ac1d80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc1d83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6c1d87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x165c1d8a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dc1d90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c1d95, 34 },
    { 0, 0 },
    { 0x843c1d97, 90 },
    { 0, 0 },
    { 0x7ddc1d99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbc1d9d, 33 },
    { 0, 0 },
    { 0x3fdc1d9f, 95 },
    { 0, 0 },
    { 0x17ec1da1, 61 },
    { 0xfd8c1da1, 52 },
    { 0, 0 },
    { 0x4a0c1da4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7c1da7, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c1daa, 23 },
    { 0x5c6c1dab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8c1db0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1c1db8, 95 },
    { 0xca4c1db9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5c1dc0, 33 },
    { 0x73ac1dc0, 55 },
    { 0x7bdc1dc2, 67 },
    { 0x6c8c1dc1, 65 },
    { 0x9acc1dc3, 78 },
    { 0, 0 },
    { 0x1e1c1dc6, 99 },
    { 0x6c3c1dc6, 55 },
    { 0x514c1dc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889c1dcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c1dd0, 55 },
    { 0x14c1dd1, 61 },
    { 0, 0 },
    { 0xef2c1dd3, 98 },
    { 0, 0 },
    { 0x1bbc1dd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0c1dd8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509c1ddc, 76 },
    { 0xfa9c1ddc, 61 },
    { 0xeecc1dde, 33 },
    { 0, 0 },
    { 0xbecc1de0, 98 },
    { 0xbf7c1de1, 61 },
    { 0xcbdc1de1, 98 },
    { 0xd0ac1de1, 8 },
    { 0, 0 },
    { 0x4d8c1de5, 33 },
    { 0x59fc1de5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05c1dea, 61 },
    { 0, 0 },
    { 0x467c1dec, 33 },
    { 0x2d3c1dec, 59 },
    { 0xacec1dee, 99 },
    { 0xccec1def, 98 },
    { 0xb9fc1def, 98 },
    { 0xa43c1dec, 95 },
    { 0, 0 },
    { 0x8bcc1df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ec1dfc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c1e00, 98 },
    { 0, 0 },
    { 0x125c1e02, 66 },
    { 0, 0 },
    { 0xcd5c1e04, 55 },
    { 0x5e8c1e05, 51 },
    { 0x588c1e06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3c1e0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fc1e14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101c1e1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ac1e1d, 95 },
    { 0, 0 },
    { 0x6c0c1e1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cc1e22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5c1e26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0c1e34, 90 },
    { 0xfb8c1e35, 61 },
    { 0x5d2c1e36, 93 },
    { 0xc7c1e37, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731c1e3e, 33 },
    { 0xdccc1e3e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952c1e44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdc1e4c, 61 },
    { 0x4f3c1e4c, 90 },
    { 0xbc4c1e4c, 90 },
    { 0, 0 },
    { 0xdcdc1e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9c1e56, 90 },
    { 0, 0 },
    { 0x81bc1e58, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c1e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5c1e60, 61 },
    { 0x2dfc1e60, 98 },
    { 0, 0 },
    { 0x5f1c1e63, 90 },
    { 0x3a1c1e64, 33 },
    { 0, 0 },
    { 0xd23c1e66, 95 },
    { 0, 0 },
    { 0x65fc1e68, 23 },
    { 0x832c1e68, 46 },
    { 0xa33c1e6a, 78 },
    { 0xf48c1e68, 76 },
    { 0xaf6c1e68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x197c1e6f, 33 },
    { 0xa3cc1e6f, 23 },
    { 0x2d3c1e71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bc1e74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97c1e7d, 98 },
    { 0xf8fc1e7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ac1e82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5c1e85, 61 },
    { 0x300c1e86, 83 },
    { 0, 0 },
    { 0x8bfc1e88, 33 },
    { 0x85cc1e89, 33 },
    { 0x5f2c1e88, 84 },
    { 0, 0 },
    { 0xca5c1e8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ec1e98, 91 },
    { 0x1dac1e99, 98 },
    { 0xab3c1e9a, 55 },
    { 0x4d9c1e9b, 17 },
    { 0x5c4c1e9b, 95 },
    { 0, 0 },
    { 0xe26c1e9e, 68 },
    { 0, 0 },
    { 0x285c1ea0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6c1ea5, 33 },
    { 0, 0 },
    { 0xf21c1ea7, 23 },
    { 0x6b3c1ea8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6c1eab, 90 },
    { 0xf8ec1eac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfc1eaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc1eb4, 52 },
    { 0x8c9c1eb4, 61 },
    { 0x70c1eb4, 76 },
    { 0xe4cc1eb7, 67 },
    { 0x365c1eb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21c1ebf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfc1ec6, 76 },
    { 0x12fc1ec7, 34 },
    { 0xd3bc1ec7, 17 },
    { 0xe36c1ec9, 76 },
    { 0, 0 },
    { 0x50fc1ecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4c1ecf, 94 },
    { 0x1e9c1ecf, 40 },
    { 0x552c1ed1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404c1ed5, 61 },
    { 0x2ac1ed6, 99 },
    { 0, 0 },
    { 0xf8cc1ed8, 90 },
    { 0, 0 },
    { 0xbcc1eda, 98 },
    { 0, 0 },
    { 0xc1fc1edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cc1ee3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c1ee6, 98 },
    { 0x63c1ee7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c1eea, 84 },
    { 0x938c1eeb, 91 },
    { 0xa3dc1eeb, 61 },
    { 0x715c1eed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8c1ef9, 34 },
    { 0x7cec1efa, 55 },
    { 0, 0 },
    { 0x34cc1efc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6c1eff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20c1f03, 95 },
    { 0x3fbc1f03, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328c1f08, 91 },
    { 0x9bec1f09, 61 },
    { 0x450c1f08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc1f11, 75 },
    { 0x6cfc1f11, 87 },
    { 0xeaac1f11, 46 },
    { 0x2c3c1f13, 33 },
    { 0, 0 },
    { 0x135c1f16, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdc1f1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679c1f20, 93 },
    { 0xdebc1f21, 82 },
    { 0xf49c1f21, 90 },
    { 0xb68c1f22, 76 },
    { 0x54dc1f24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c1f2a, 33 },
    { 0xf51c1f2b, 33 },
    { 0x3eac1f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6c1f2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c1f34, 34 },
    { 0x801c1f34, 43 },
    { 0x52dc1f36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dc1f39, 99 },
    { 0, 0 },
    { 0xe7c1f3b, 93 },
    { 0, 0 },
    { 0x87dc1f3d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ec1f41, 55 },
    { 0x615c1f42, 65 },
    { 0, 0 },
    { 0x40bc1f44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8c1f47, 98 },
    { 0xa70c1f47, 33 },
    { 0, 0 },
    { 0xd7ac1f4a, 76 },
    { 0, 0 },
    { 0x776c1f4c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79c1f52, 23 },
    { 0xb4c1f53, 36 },
    { 0x56c1f52, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dec1f58, 95 },
    { 0xab8c1f59, 55 },
    { 0x77c1f5a, 34 },
    { 0x982c1f5b, 34 },
    { 0xe06c1f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fc1f64, 61 },
    { 0, 0 },
    { 0xd14c1f66, 98 },
    { 0xb1dc1f67, 78 },
    { 0x782c1f67, 95 },
    { 0x235c1f68, 85 },
    { 0x1bbc1f66, 95 },
    { 0x952c1f6b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x12c1f6e, 34 },
    { 0xca3c1f6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84c1f73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0c1f7d, 95 },
    { 0xeccc1f7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfc1f82, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcc1f86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517c1f8a, 34 },
    { 0, 0 },
    { 0x4c8c1f8c, 92 },
    { 0x8a5c1f8c, 98 },
    { 0x965c1f8e, 33 },
    { 0x9e8c1f8e, 33 },
    { 0x37c1f8e, 95 },
    { 0, 0 },
    { 0x78fc1f92, 61 },
    { 0x25dc1f93, 95 },
    { 0x66ac1f93, 90 },
    { 0x81bc1f95, 90 },
    { 0x7eec1f96, 46 },
    { 0x297c1f94, 22 },
    { 0x501c1f97, 33 },
    { 0xf9ac1f99, 34 },
    { 0xfd3c1f9a, 78 },
    { 0x118c1f95, 75 },
    { 0x88fc1f9c, 90 },
    { 0xc31c1f9c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ec1fa0, 76 },
    { 0, 0 },
    { 0x68c1fa2, 95 },
    { 0xf88c1fa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553c1fab, 98 },
    { 0x20bc1fac, 46 },
    { 0, 0 },
    { 0xea1c1fae, 87 },
    { 0xaf7c1faf, 81 },
    { 0, 0 },
    { 0xc37c1fb1, 34 },
    { 0xfeac1fb2, 61 },
    { 0xc97c1fb2, 78 },
    { 0x91fc1fb3, 76 },
    { 0x8afc1fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3c1fb8, 48 },
    { 0xc59c1fb9, 99 },
    { 0x18c1fba, 90 },
    { 0, 0 },
    { 0xb60c1fbc, 65 },
    { 0, 0 },
    { 0x188c1fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ac1fc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bc1fc7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fec1fce, 75 },
    { 0, 0 },
    { 0xa6ec1fd0, 61 },
    { 0, 0 },
    { 0xf0dc1fd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2c1fd5, 98 },
    { 0, 0 },
    { 0xd2bc1fd7, 23 },
    { 0x63bc1fd8, 61 },
    { 0xf1cc1fd8, 90 },
    { 0x27c1fda, 97 },
    { 0, 0 },
    { 0x4aac1fdc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cc1fe2, 52 },
    { 0x20fc1fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83c1fe6, 95 },
    { 0x2dec1fe7, 90 },
    { 0x156c1fe7, 46 },
    { 0, 0 },
    { 0x12bc1fea, 96 },
    { 0xb3ac1feb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ac1fef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dc1ff2, 91 },
    { 0x3cdc1ff3, 46 },
    { 0xdf5c1ff4, 61 },
    { 0xfbec1ff3, 90 },
    { 0xb01c1ff6, 90 },
    { 0x5ccc1ff7, 76 },
    { 0, 0 },
    { 0xebdc1ff9, 90 },
    { 0xee6c1ffa, 46 },
    { 0x2d5c1ffb, 90 },
    { 0, 0 },
    { 0x99ac1ffd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c2004, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc2009, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cc200c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffc2012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81c2015, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8c2023, 23 },
    { 0xa84c2024, 90 },
    { 0x420c2024, 46 },
    { 0x2c1c2025, 98 },
    { 0, 0 },
    { 0xc90c2028, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18c202c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fac2030, 78 },
    { 0x7f8c2031, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57c2036, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65c203a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bc203d, 23 },
    { 0xe79c203e, 96 },
    { 0x4f3c203f, 61 },
    { 0x264c203d, 90 },
    { 0x866c2040, 65 },
    { 0, 0 },
    { 0x578c2043, 61 },
    { 0xc7ec2044, 33 },
    { 0, 0 },
    { 0xe0c2046, 93 },
    { 0, 0 },
    { 0x7f3c2048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec204e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567c2053, 23 },
    { 0xa85c2053, 61 },
    { 0x874c2055, 64 },
    { 0, 0 },
    { 0xdcfc2057, 98 },
    { 0, 0 },
    { 0xc43c2059, 93 },
    { 0xdefc205a, 99 },
    { 0, 0 },
    { 0x503c205c, 98 },
    { 0xf65c205d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafc2066, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dc2069, 90 },
    { 0, 0 },
    { 0x42dc206b, 90 },
    { 0, 0 },
    { 0x5f7c206d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24c2072, 99 },
    { 0x767c2072, 90 },
    { 0x561c2074, 33 },
    { 0, 0 },
    { 0xa9ec2076, 66 },
    { 0, 0 },
    { 0x51dc2078, 84 },
    { 0x64cc2079, 100 },
    { 0x33c207a, 52 },
    { 0x773c207b, 90 },
    { 0xf4cc207c, 61 },
    { 0x443c207c, 85 },
    { 0xcd9c207d, 64 },
    { 0x853c207f, 33 },
    { 0x836c207f, 95 },
    { 0x9e8c2081, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7c2084, 66 },
    { 0x288c2085, 95 },
    { 0x1c5c2086, 95 },
    { 0xb7c2087, 76 },
    { 0x415c2086, 90 },
    { 0, 0 },
    { 0xce9c208a, 61 },
    { 0xbfbc208b, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fc208f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59c2093, 34 },
    { 0x145c2094, 43 },
    { 0x365c2093, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7c2099, 91 },
    { 0x820c209a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238c20a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc20a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9c20ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac20b2, 97 },
    { 0, 0 },
    { 0xc67c20b4, 61 },
    { 0, 0 },
    { 0x464c20b6, 43 },
    { 0xe3bc20b7, 95 },
    { 0xd49c20b8, 85 },
    { 0x722c20b9, 99 },
    { 0, 0 },
    { 0x210c20bb, 98 },
    { 0x40ec20bc, 91 },
    { 0xecbc20bc, 61 },
    { 0xb91c20bc, 98 },
    { 0xa05c20bd, 90 },
    { 0x185c20c0, 33 },
    { 0, 0 },
    { 0xbe2c20c2, 95 },
    { 0x110c20c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdec20c7, 87 },
    { 0x353c20c7, 76 },
    { 0x304c20c8, 95 },
    { 0, 0 },
    { 0x883c20cb, 84 },
    { 0, 0 },
    { 0x5eec20cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251c20d3, 69 },
    { 0, 0 },
    { 0x442c20d5, 23 },
    { 0, 0 },
    { 0x297c20d7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dc20da, 70 },
    { 0x8c7c20db, 61 },
    { 0x5fcc20dc, 100 },
    { 0x38ac20dd, 33 },
    { 0x535c20de, 95 },
    { 0x38ec20da, 95 },
    { 0x3efc20dc, 98 },
    { 0x4b8c20de, 98 },
    { 0, 0 },
    { 0x9a7c20e3, 61 },
    { 0x544c20e4, 93 },
    { 0x80dc20e3, 46 },
    { 0, 0 },
    { 0xd2fc20e7, 55 },
    { 0x1b4c20e8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c20f1, 23 },
    { 0, 0 },
    { 0xa29c20f3, 90 },
    { 0xc7ac20f4, 55 },
    { 0, 0 },
    { 0x5f3c20f6, 61 },
    { 0, 0 },
    { 0xdacc20f8, 95 },
    { 0x598c20f9, 61 },
    { 0, 0 },
    { 0xa04c20fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2c20ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78c2109, 90 },
    { 0, 0 },
    { 0x40ec210b, 91 },
    { 0x65fc210c, 33 },
    { 0x315c210d, 95 },
    { 0xddc210b, 34 },
    { 0x8bcc210f, 90 },
    { 0xaa9c210f, 69 },
    { 0x172c2111, 33 },
    { 0xa07c210d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc211c, 90 },
    { 0x832c211d, 52 },
    { 0xddbc211d, 98 },
    { 0, 0 },
    { 0x6a1c2120, 90 },
    { 0xbacc2121, 76 },
    { 0xb45c2122, 61 },
    { 0x15ac2123, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246c2127, 61 },
    { 0, 0 },
    { 0x1eec2129, 90 },
    { 0x2a0c212a, 61 },
    { 0x1a7c212b, 43 },
    { 0x504c212b, 95 },
    { 0xfdec212c, 81 },
    { 0xb4bc212e, 100 },
    { 0, 0 },
    { 0x1adc2130, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5c2133, 73 },
    { 0xa8bc2133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207c213a, 87 },
    { 0, 0 },
    { 0x5a8c213c, 98 },
    { 0xcf7c213d, 98 },
    { 0x8ccc213d, 98 },
    { 0, 0 },
    { 0x735c2140, 43 },
    { 0xf69c2140, 95 },
    { 0, 0 },
    { 0x187c2143, 36 },
    { 0, 0 },
    { 0x95c2145, 74 },
    { 0xa35c2145, 90 },
    { 0x34dc2147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7c214b, 51 },
    { 0xd5ac214c, 95 },
    { 0x2ffc214c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5c2152, 78 },
    { 0xc61c2153, 34 },
    { 0xe7ac2153, 90 },
    { 0xf90c2155, 98 },
    { 0x1cac2155, 66 },
    { 0x139c2155, 46 },
    { 0xb0dc2158, 55 },
    { 0, 0 },
    { 0x98c215a, 95 },
    { 0, 0 },
    { 0xb18c215c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dc215f, 33 },
    { 0xb62c2160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cc2163, 55 },
    { 0, 0 },
    { 0xe9cc2165, 91 },
    { 0x61ec2165, 46 },
    { 0xcc7c2167, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c216a, 52 },
    { 0xb94c216b, 23 },
    { 0xfb9c216c, 100 },
    { 0x213c216c, 95 },
    { 0, 0 },
    { 0x1e6c216f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48c2176, 87 },
    { 0xd16c2176, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cc217c, 23 },
    { 0x146c217d, 61 },
    { 0xad1c217d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c2182, 98 },
    { 0x351c2183, 34 },
    { 0xa75c2184, 76 },
    { 0xceac2185, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7c2189, 46 },
    { 0xd2bc218a, 98 },
    { 0xefc218b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302c218f, 23 },
    { 0, 0 },
    { 0x4fec2191, 17 },
    { 0x3dec2192, 33 },
    { 0xae7c2193, 34 },
    { 0xe9dc2192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26c219c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c21a0, 90 },
    { 0x766c21a1, 85 },
    { 0xb5fc21a1, 17 },
    { 0x637c21a3, 90 },
    { 0xf3ec21a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bc21a8, 91 },
    { 0, 0 },
    { 0xd8dc21aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52c21ad, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdc21b0, 55 },
    { 0x337c21b1, 33 },
    { 0x43ac21b2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7c21b5, 23 },
    { 0, 0 },
    { 0x8e4c21b7, 43 },
    { 0xe6ec21b8, 23 },
    { 0xcffc21b8, 61 },
    { 0x613c21ba, 90 },
    { 0x5e3c21ba, 76 },
    { 0x219c21b8, 33 },
    { 0x967c21bc, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72c21c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0c21c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecc21c8, 100 },
    { 0xa7dc21c9, 61 },
    { 0x940c21c9, 55 },
    { 0xae0c21ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363c21cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc21d3, 61 },
    { 0, 0 },
    { 0x3cec21d5, 93 },
    { 0, 0 },
    { 0x6e7c21d7, 88 },
    { 0x37c21d8, 90 },
    { 0x924c21d8, 87 },
    { 0x1dc21da, 61 },
    { 0xacec21db, 61 },
    { 0xee0c21dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac21e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bc21e6, 23 },
    { 0xbbbc21e7, 90 },
    { 0x6ddc21e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc21ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ac21f3, 61 },
    { 0, 0 },
    { 0x735c21f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x211c21f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ec21fb, 90 },
    { 0xd48c21fc, 61 },
    { 0xebfc21fd, 98 },
    { 0xe31c21fc, 90 },
    { 0xb26c21fe, 43 },
    { 0xdbc2200, 98 },
    { 0x81ec21fc, 91 },
    { 0, 0 },
    { 0x452c2203, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efc2207, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaac220b, 76 },
    { 0x396c220c, 91 },
    { 0xe07c220d, 33 },
    { 0x78fc220d, 23 },
    { 0, 0 },
    { 0x5dc2210, 61 },
    { 0x953c2210, 43 },
    { 0x6e4c2212, 55 },
    { 0, 0 },
    { 0xffcc2214, 52 },
    { 0x418c2215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76c221a, 38 },
    { 0, 0 },
    { 0xbaac221c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc2226, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c222a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0c222d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ac2230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136c2234, 98 },
    { 0x274c2235, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc223f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9c2242, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cc2252, 61 },
    { 0x170c2253, 61 },
    { 0xd00c2254, 81 },
    { 0xcb4c2252, 75 },
    { 0, 0 },
    { 0x13bc2257, 23 },
    { 0, 0 },
    { 0xbb4c2259, 61 },
    { 0x8d7c225a, 34 },
    { 0x5e4c225b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac225f, 84 },
    { 0xa2c2260, 100 },
    { 0, 0 },
    { 0x9adc2262, 90 },
    { 0, 0 },
    { 0x702c2264, 33 },
    { 0xdfac2265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec226b, 33 },
    { 0xb58c226c, 55 },
    { 0, 0 },
    { 0xba3c226e, 17 },
    { 0x18ec226e, 55 },
    { 0xe53c226f, 76 },
    { 0x2a3c2271, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8c2275, 90 },
    { 0, 0 },
    { 0xe52c2277, 61 },
    { 0, 0 },
    { 0x478c2279, 95 },
    { 0x624c2279, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50c227f, 98 },
    { 0xaeec2280, 66 },
    { 0x4f1c2281, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac2284, 33 },
    { 0x5bc2284, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x349c2288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c228d, 33 },
    { 0xe32c228e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x256c2291, 34 },
    { 0xb0bc2292, 99 },
    { 0x9e9c2293, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563c229a, 84 },
    { 0x7e7c229a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7c22a0, 93 },
    { 0x302c22a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54c22a4, 77 },
    { 0, 0 },
    { 0xf07c22a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbc22a9, 94 },
    { 0, 0 },
    { 0x6f3c22ab, 95 },
    { 0, 0 },
    { 0xca9c22ad, 81 },
    { 0, 0 },
    { 0xd28c22af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cc22b3, 74 },
    { 0x4b8c22b4, 67 },
    { 0, 0 },
    { 0x1fec22b6, 90 },
    { 0xc11c22b6, 98 },
    { 0x66bc22b8, 95 },
    { 0xd02c22b9, 34 },
    { 0xbccc22b9, 46 },
    { 0, 0 },
    { 0x8a8c22bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x430c22bf, 33 },
    { 0x428c22c0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843c22c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c22ca, 43 },
    { 0, 0 },
    { 0xe9bc22cc, 61 },
    { 0x602c22cc, 98 },
    { 0x206c22cd, 76 },
    { 0x4a5c22cc, 100 },
    { 0x7fbc22cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573c22d4, 93 },
    { 0xd1c22d5, 33 },
    { 0, 0 },
    { 0x781c22d7, 90 },
    { 0, 0 },
    { 0x6b8c22d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ec22e2, 33 },
    { 0x1b9c22e2, 12 },
    { 0xa0ac22e2, 55 },
    { 0x8a2c22e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ac22e8, 33 },
    { 0, 0 },
    { 0xc0fc22ea, 85 },
    { 0xfa4c22ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821c22f3, 61 },
    { 0, 0 },
    { 0xccdc22f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x666c22f8, 61 },
    { 0x8c9c22f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ac22fe, 22 },
    { 0x70bc22ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974c2304, 95 },
    { 0, 0 },
    { 0x4a3c2306, 44 },
    { 0, 0 },
    { 0x8c9c2308, 98 },
    { 0x76ac2309, 43 },
    { 0, 0 },
    { 0x513c230b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587c2310, 61 },
    { 0xe05c2310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8c2314, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02c2318, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dc231f, 68 },
    { 0x978c2320, 23 },
    { 0x3acc2321, 61 },
    { 0xe3bc2320, 99 },
    { 0x798c2320, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8c2328, 65 },
    { 0x6afc2329, 90 },
    { 0, 0 },
    { 0xf8ec232b, 85 },
    { 0x89bc232c, 75 },
    { 0xb8fc232d, 23 },
    { 0x852c232d, 76 },
    { 0xe82c232c, 95 },
    { 0xa91c232e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cc2337, 55 },
    { 0x50ac2338, 91 },
    { 0x2fac2339, 61 },
    { 0x626c233a, 98 },
    { 0, 0 },
    { 0x191c233c, 55 },
    { 0xf44c233d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13c2345, 33 },
    { 0x327c2345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ac234a, 33 },
    { 0, 0 },
    { 0x1a7c234c, 23 },
    { 0x731c234d, 52 },
    { 0xc00c234d, 88 },
    { 0, 0 },
    { 0xe86c2350, 98 },
    { 0xb60c2350, 11 },
    { 0x11bc2352, 46 },
    { 0xe8cc2352, 61 },
    { 0x16cc2352, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82c2358, 46 },
    { 0xf04c2359, 67 },
    { 0x560c235a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ac2361, 61 },
    { 0, 0 },
    { 0x6dfc2363, 33 },
    { 0, 0 },
    { 0x352c2365, 34 },
    { 0x16cc2366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acc236a, 95 },
    { 0, 0 },
    { 0x21c236c, 43 },
    { 0x472c236c, 76 },
    { 0, 0 },
    { 0xa46c236f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dec2372, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aac2375, 76 },
    { 0xd22c2375, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c2379, 33 },
    { 0, 0 },
    { 0x184c237b, 76 },
    { 0x1d1c237c, 95 },
    { 0x7eac237d, 46 },
    { 0xd8fc237c, 95 },
    { 0x85bc237d, 17 },
    { 0xf09c2380, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7c2384, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6c2387, 91 },
    { 0xaf5c2388, 36 },
    { 0xf77c2389, 61 },
    { 0x7e0c2389, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dc238f, 98 },
    { 0, 0 },
    { 0x214c2391, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6c2397, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bac239b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dc239f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c23a2, 36 },
    { 0, 0 },
    { 0x267c23a4, 99 },
    { 0, 0 },
    { 0xdf0c23a6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3c23aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dc23af, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaec23b4, 55 },
    { 0x4f1c23b4, 78 },
    { 0x20dc23b5, 70 },
    { 0xc06c23b4, 33 },
    { 0xc3c23b8, 81 },
    { 0x45dc23b8, 55 },
    { 0x8d5c23ba, 100 },
    { 0xd72c23b7, 95 },
    { 0x7e0c23b7, 93 },
    { 0, 0 },
    { 0x8bcc23be, 34 },
    { 0x3fdc23be, 34 },
    { 0x23ac23be, 93 },
    { 0x4f4c23c1, 33 },
    { 0, 0 },
    { 0x6fac23c3, 98 },
    { 0, 0 },
    { 0x9b6c23c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ac23ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5c23d4, 61 },
    { 0, 0 },
    { 0x498c23d6, 34 },
    { 0x644c23d6, 95 },
    { 0xcd7c23d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8c23db, 34 },
    { 0, 0 },
    { 0xafac23dd, 90 },
    { 0xa5c23de, 99 },
    { 0x5d1c23de, 69 },
    { 0xf29c23e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afc23e6, 43 },
    { 0x4d5c23e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162c23ec, 61 },
    { 0xbb4c23ec, 61 },
    { 0xf32c23ee, 77 },
    { 0x81bc23ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc23f2, 61 },
    { 0xf41c23f3, 95 },
    { 0, 0 },
    { 0x2c1c23f5, 85 },
    { 0, 0 },
    { 0xb1dc23f7, 76 },
    { 0xebfc23f7, 66 },
    { 0x163c23f9, 87 },
    { 0xa09c23fa, 85 },
    { 0x99ac23fa, 98 },
    { 0x75cc23f7, 98 },
    { 0xcd6c23fd, 67 },
    { 0xbbfc23fd, 90 },
    { 0, 0 },
    { 0x838c2400, 61 },
    { 0xca2c2401, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2c2404, 17 },
    { 0xf20c2405, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71c240a, 33 },
    { 0x6cdc240b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7c2416, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x154c2419, 99 },
    { 0, 0 },
    { 0xd95c241b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9c241e, 55 },
    { 0xbe4c241f, 67 },
    { 0x9f1c2420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c2425, 43 },
    { 0x3bcc2426, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c2431, 68 },
    { 0xfd3c2432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609c2438, 43 },
    { 0x214c2439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecc2442, 23 },
    { 0x7dac2443, 76 },
    { 0xbd2c2442, 98 },
    { 0, 0 },
    { 0xed0c2446, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0c244a, 90 },
    { 0x60ac244b, 33 },
    { 0xe4c244b, 90 },
    { 0, 0 },
    { 0xdf8c244e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82c2452, 69 },
    { 0xe4ec2453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ec2456, 55 },
    { 0, 0 },
    { 0x5cc2458, 90 },
    { 0x6d7c2459, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7c245c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66c2460, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bc2463, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33c246a, 98 },
    { 0, 0 },
    { 0x439c246c, 91 },
    { 0, 0 },
    { 0x4fc246e, 64 },
    { 0xa96c246e, 46 },
    { 0xa9bc2470, 90 },
    { 0x199c2471, 33 },
    { 0, 0 },
    { 0x27dc2473, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc2477, 90 },
    { 0xeedc2478, 34 },
    { 0x188c2479, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x199c247c, 69 },
    { 0, 0 },
    { 0x70fc247e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22c2482, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c2486, 61 },
    { 0x3fcc2486, 65 },
    { 0x53ec2488, 23 },
    { 0x41fc2488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3c248e, 98 },
    { 0, 0 },
    { 0x374c2490, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90c2498, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cc249e, 90 },
    { 0x65bc249f, 88 },
    { 0xd4ac249f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc24a3, 76 },
    { 0xa9ac24a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7c24a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x408c24aa, 33 },
    { 0x682c24aa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cc24af, 95 },
    { 0, 0 },
    { 0x396c24b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dc24b6, 33 },
    { 0x81ac24b6, 23 },
    { 0xdd5c24b7, 61 },
    { 0xb1c24b7, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452c24c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3c24c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fec24cc, 61 },
    { 0xe00c24cd, 87 },
    { 0, 0 },
    { 0xf98c24cf, 33 },
    { 0x6bcc24d0, 23 },
    { 0xd36c24d1, 95 },
    { 0x19dc24d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc24da, 91 },
    { 0x26cc24da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6c24e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x855c24e3, 43 },
    { 0xb86c24e4, 17 },
    { 0, 0 },
    { 0x838c24e6, 33 },
    { 0x932c24e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9c24ea, 98 },
    { 0, 0 },
    { 0xa6ac24ec, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4c24ef, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534c24f6, 98 },
    { 0, 0 },
    { 0x409c24f8, 43 },
    { 0, 0 },
    { 0xbd9c24fa, 33 },
    { 0xaabc24fa, 77 },
    { 0x1aec24fc, 65 },
    { 0xfe8c24fa, 46 },
    { 0, 0 },
    { 0x9afc24ff, 84 },
    { 0xe4ac2500, 40 },
    { 0x3f9c2500, 43 },
    { 0x4edc2500, 46 },
    { 0x864c2503, 34 },
    { 0x665c2504, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85c250d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dc2510, 84 },
    { 0xb41c2510, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176c2515, 61 },
    { 0xa75c2515, 85 },
    { 0xb1ac2516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c251b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212c251f, 9 },
    { 0x19fc2520, 90 },
    { 0, 0 },
    { 0xe67c2522, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719c2529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509c252e, 33 },
    { 0x684c252f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc2536, 61 },
    { 0, 0 },
    { 0x956c2538, 33 },
    { 0, 0 },
    { 0x755c253a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dec2542, 99 },
    { 0x847c2542, 61 },
    { 0, 0 },
    { 0x52bc2545, 77 },
    { 0xa63c2546, 93 },
    { 0xc6cc2546, 96 },
    { 0xbd5c2548, 76 },
    { 0xec3c2549, 66 },
    { 0xdeec2549, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x143c254d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1c2556, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8c255c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c255f, 87 },
    { 0, 0 },
    { 0x823c2561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bc2565, 33 },
    { 0, 0 },
    { 0xe4dc2567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x562c256a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ac2572, 67 },
    { 0, 0 },
    { 0x3cac2574, 66 },
    { 0xc91c2574, 95 },
    { 0, 0 },
    { 0xcfcc2577, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ac257a, 46 },
    { 0, 0 },
    { 0x3c9c257c, 61 },
    { 0, 0 },
    { 0xb33c257e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0c2584, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x906c2587, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0c258c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263c2593, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712c259a, 91 },
    { 0, 0 },
    { 0xc36c259c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebc25a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ac25a5, 8 },
    { 0x63dc25a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfc25b0, 98 },
    { 0xc46c25b0, 43 },
    { 0x628c25b2, 43 },
    { 0x38c25b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1c25b6, 55 },
    { 0x8bac25b7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbc25bb, 98 },
    { 0xef7c25bc, 98 },
    { 0x865c25bb, 55 },
    { 0xdb1c25be, 98 },
    { 0x972c25bd, 87 },
    { 0x18bc25c0, 55 },
    { 0xe09c25bf, 98 },
    { 0xe18c25c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50c25c9, 55 },
    { 0xc0c25ca, 90 },
    { 0, 0 },
    { 0xd45c25cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c25d5, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ac25de, 43 },
    { 0, 0 },
    { 0xfd3c25e0, 90 },
    { 0xd8ec25e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92c25e4, 78 },
    { 0x1b6c25e5, 84 },
    { 0x1e2c25e5, 23 },
    { 0, 0 },
    { 0x8b8c25e8, 93 },
    { 0xb4fc25e9, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc25ef, 88 },
    { 0xc16c25ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09c25f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc25f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dc25fd, 55 },
    { 0x31ec25fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10c2603, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650c2607, 33 },
    { 0x84bc2607, 61 },
    { 0, 0 },
    { 0x9a0c260a, 90 },
    { 0, 0 },
    { 0xc27c260c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffc2610, 83 },
    { 0x57ac2611, 33 },
    { 0xd09c2612, 43 },
    { 0x47cc2612, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553c2621, 90 },
    { 0xf3ac2621, 90 },
    { 0xdc6c2623, 97 },
    { 0x8aec2624, 33 },
    { 0xc7bc2624, 33 },
    { 0, 0 },
    { 0xed6c2627, 61 },
    { 0, 0 },
    { 0xdb1c2629, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6c262e, 61 },
    { 0xed6c262e, 61 },
    { 0xe69c262f, 98 },
    { 0xc1fc2631, 98 },
    { 0x1c1c262f, 52 },
    { 0, 0 },
    { 0x8f5c2634, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafc2638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54c263b, 99 },
    { 0xc9ac263c, 55 },
    { 0x16fc263d, 93 },
    { 0x851c263c, 61 },
    { 0x1d5c263f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc2642, 95 },
    { 0xa2ec2643, 84 },
    { 0x727c2644, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x542c2647, 61 },
    { 0, 0 },
    { 0xfafc2649, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43c2651, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756c2655, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569c265a, 91 },
    { 0x7ac265b, 90 },
    { 0xda1c265c, 87 },
    { 0x477c265d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29c2661, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97c2664, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fc266b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6c266e, 87 },
    { 0xf13c266e, 90 },
    { 0x809c266e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cc2673, 61 },
    { 0, 0 },
    { 0x2a2c2675, 85 },
    { 0xc5bc2676, 23 },
    { 0x3d8c2677, 66 },
    { 0x82bc2677, 95 },
    { 0x554c2679, 34 },
    { 0x314c2679, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991c267e, 95 },
    { 0, 0 },
    { 0xcb3c2680, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c2684, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07c2689, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c268d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec2690, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cc2693, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61c2698, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bc269b, 74 },
    { 0x3a3c269b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97c26a2, 78 },
    { 0x601c26a2, 33 },
    { 0x832c26a4, 43 },
    { 0xae8c26a4, 76 },
    { 0x8bcc26a4, 33 },
    { 0xe7bc26a6, 76 },
    { 0x517c26a8, 98 },
    { 0x6a6c26a9, 95 },
    { 0, 0 },
    { 0x223c26ab, 90 },
    { 0x1a8c26ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebc26af, 61 },
    { 0xff8c26b0, 55 },
    { 0, 0 },
    { 0x64cc26b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c26b5, 23 },
    { 0, 0 },
    { 0xf2dc26b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x469c26ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbc26bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcac26ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ac26d5, 95 },
    { 0xf0cc26d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5c26d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ec26dd, 90 },
    { 0x110c26dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bc26e1, 90 },
    { 0x810c26e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8c26e6, 55 },
    { 0, 0 },
    { 0x7edc26e8, 95 },
    { 0x435c26e9, 34 },
    { 0x2bdc26ea, 95 },
    { 0, 0 },
    { 0x443c26ec, 98 },
    { 0x41cc26ed, 68 },
    { 0xec4c26ec, 38 },
    { 0x798c26ef, 61 },
    { 0x5f2c26ec, 98 },
    { 0xa4ec26f1, 89 },
    { 0x8d9c26ec, 95 },
    { 0x478c26f1, 61 },
    { 0xa7c26f0, 95 },
    { 0x181c26f2, 101 },
    { 0xc7cc26ee, 93 },
    { 0x843c26f5, 43 },
    { 0x57bc26ec, 43 },
    { 0x6fcc26f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ac26ff, 36 },
    { 0x190c26ff, 34 },
    { 0x654c26ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bc2705, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdc2709, 93 },
    { 0x80ac2709, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16c270e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3c2712, 60 },
    { 0x854c2713, 96 },
    { 0xd9c2713, 65 },
    { 0x338c2713, 95 },
    { 0x531c2715, 61 },
    { 0x7fbc2717, 98 },
    { 0xe97c2718, 33 },
    { 0xf4ec2719, 95 },
    { 0xfdfc2718, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc271e, 90 },
    { 0x9adc271e, 85 },
    { 0, 0 },
    { 0x5cbc2721, 81 },
    { 0x1fec2721, 33 },
    { 0x2d5c2723, 33 },
    { 0xeeec2724, 68 },
    { 0x6e8c2724, 68 },
    { 0x248c2726, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c272f, 95 },
    { 0xe86c2730, 34 },
    { 0x892c2731, 61 },
    { 0xbb1c2730, 17 },
    { 0xa7dc2733, 61 },
    { 0x2c2c2734, 98 },
    { 0xd37c2735, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dc2739, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4c273c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbec2741, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0c2744, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c2748, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926c274e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fc2753, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcc275a, 33 },
    { 0x118c275b, 95 },
    { 0, 0 },
    { 0x9a2c275d, 94 },
    { 0xaa5c275e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54c2769, 98 },
    { 0xdd8c276a, 33 },
    { 0, 0 },
    { 0x9afc276c, 66 },
    { 0xcc8c276c, 78 },
    { 0x3a1c276d, 33 },
    { 0, 0 },
    { 0x8dbc2770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cc2773, 61 },
    { 0xd60c2774, 34 },
    { 0xacdc2775, 78 },
    { 0x917c2775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4c277e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8c2783, 43 },
    { 0x38cc2783, 61 },
    { 0x89ec2783, 95 },
    { 0, 0 },
    { 0x611c2787, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccc278d, 68 },
    { 0x591c278d, 37 },
    { 0x920c278f, 34 },
    { 0x692c2790, 52 },
    { 0, 0 },
    { 0xaa8c2792, 90 },
    { 0, 0 },
    { 0x3dac2794, 40 },
    { 0xe7cc2794, 55 },
    { 0x33cc2796, 34 },
    { 0xe8dc2797, 33 },
    { 0, 0 },
    { 0xb3dc2799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87c279c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269c27a3, 76 },
    { 0xd30c27a3, 33 },
    { 0xf90c27a5, 90 },
    { 0x47ec27a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4c27aa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dc27ad, 55 },
    { 0x9b7c27ae, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cc27b5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c27b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84c27bd, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c27c3, 76 },
    { 0x57dc27c3, 33 },
    { 0, 0 },
    { 0x43c27c6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7c27cb, 76 },
    { 0, 0 },
    { 0x2e1c27cd, 93 },
    { 0x9c5c27ce, 33 },
    { 0, 0 },
    { 0x793c27d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dc27d7, 95 },
    { 0, 0 },
    { 0xb15c27d9, 84 },
    { 0xeb0c27d9, 99 },
    { 0, 0 },
    { 0x4e6c27dc, 90 },
    { 0xb54c27dc, 78 },
    { 0xe9ac27de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8c27e2, 33 },
    { 0xed3c27e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0c27e9, 34 },
    { 0x6a1c27ea, 33 },
    { 0, 0 },
    { 0x210c27ec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cec27f4, 90 },
    { 0x4b0c27f4, 91 },
    { 0xbcc27f5, 95 },
    { 0, 0 },
    { 0xa5cc27f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0c27fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369c27ff, 95 },
    { 0xa3dc27ff, 23 },
    { 0, 0 },
    { 0x624c2802, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5c2805, 34 },
    { 0x20ac2805, 91 },
    { 0, 0 },
    { 0x255c2808, 34 },
    { 0, 0 },
    { 0x5a4c280a, 23 },
    { 0xd4dc280b, 47 },
    { 0x124c280c, 88 },
    { 0x115c280d, 33 },
    { 0xbecc280c, 99 },
    { 0, 0 },
    { 0xc0dc2810, 33 },
    { 0x2cec2811, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c2816, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ec2819, 91 },
    { 0x994c281a, 90 },
    { 0x8f0c281b, 23 },
    { 0xae5c281b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30c2820, 33 },
    { 0x59cc2821, 61 },
    { 0x188c2821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4c2827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2c282b, 66 },
    { 0, 0 },
    { 0xbe8c282d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaac2831, 52 },
    { 0, 0 },
    { 0x465c2833, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c2836, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721c283c, 77 },
    { 0xd91c283c, 76 },
    { 0, 0 },
    { 0xeedc283f, 95 },
    { 0x353c2840, 23 },
    { 0, 0 },
    { 0x23fc2842, 90 },
    { 0xfaec2842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ac2859, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x493c285c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14c285f, 61 },
    { 0, 0 },
    { 0x25ac2861, 61 },
    { 0x90c2861, 90 },
    { 0xd64c2863, 23 },
    { 0xfd5c2863, 12 },
    { 0, 0 },
    { 0xee9c2866, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bc286d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63c2872, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8c2876, 95 },
    { 0x16bc2877, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fc287f, 90 },
    { 0, 0 },
    { 0xb5fc2881, 90 },
    { 0xc70c2882, 33 },
    { 0x7f9c2883, 98 },
    { 0, 0 },
    { 0x85ac2885, 78 },
    { 0x3dac2885, 98 },
    { 0x26ec2887, 33 },
    { 0xe21c2888, 76 },
    { 0xfa9c2888, 33 },
    { 0xa4dc288a, 33 },
    { 0xdb0c2888, 33 },
    { 0xeeac2887, 100 },
    { 0xf50c288c, 90 },
    { 0x741c288e, 90 },
    { 0xf6c2888, 98 },
    { 0, 0 },
    { 0xd5bc2891, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39c289a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781c28a0, 95 },
    { 0, 0 },
    { 0x36ac28a2, 23 },
    { 0x3f2c28a2, 98 },
    { 0, 0 },
    { 0xff8c28a5, 55 },
    { 0xbd9c28a5, 85 },
    { 0xafc28a7, 61 },
    { 0, 0 },
    { 0xa04c28a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2c28ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcc28af, 98 },
    { 0, 0 },
    { 0x789c28b1, 46 },
    { 0x2fac28b2, 90 },
    { 0x39c28b3, 46 },
    { 0xb7fc28b4, 93 },
    { 0, 0 },
    { 0xc6bc28b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c28ba, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02c28bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c28c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c28cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec28d0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36c28d4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c28d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24c28dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0c28e2, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317c28e7, 85 },
    { 0, 0 },
    { 0x238c28e9, 33 },
    { 0xec0c28ea, 23 },
    { 0, 0 },
    { 0x95cc28ec, 22 },
    { 0x63bc28ed, 98 },
    { 0, 0 },
    { 0x6d2c28ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c28f3, 78 },
    { 0xffdc28f4, 78 },
    { 0x714c28f5, 88 },
    { 0, 0 },
    { 0xd1bc28f7, 90 },
    { 0xf81c28f8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ac28fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x403c28ff, 76 },
    { 0x200c28ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbec2908, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bc290f, 34 },
    { 0xa03c2910, 92 },
    { 0, 0 },
    { 0x937c2912, 91 },
    { 0, 0 },
    { 0xba1c2914, 61 },
    { 0x5eac2914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccc291b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ec2923, 98 },
    { 0x731c2924, 93 },
    { 0x949c2924, 68 },
    { 0x88cc2923, 23 },
    { 0x3c0c2927, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffc292d, 33 },
    { 0xda8c292d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c2934, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88c2938, 90 },
    { 0, 0 },
    { 0x464c293a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dec293d, 88 },
    { 0xf86c293e, 33 },
    { 0, 0 },
    { 0x207c2940, 52 },
    { 0x123c2940, 61 },
    { 0x610c2942, 76 },
    { 0xd4bc2943, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dc2948, 90 },
    { 0x6ccc2949, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc294c, 61 },
    { 0, 0 },
    { 0xfdfc294e, 76 },
    { 0xc65c294f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6c2956, 61 },
    { 0, 0 },
    { 0x3a6c2958, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197c295d, 100 },
    { 0, 0 },
    { 0x27bc295f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ec2962, 34 },
    { 0xc5bc2962, 98 },
    { 0, 0 },
    { 0xa0bc2965, 17 },
    { 0x75ec2966, 23 },
    { 0xd56c2966, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc2978, 23 },
    { 0xea8c2979, 46 },
    { 0, 0 },
    { 0xd56c297b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dec297e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79c2983, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224c2988, 95 },
    { 0, 0 },
    { 0x7a8c298a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1c298f, 23 },
    { 0x2a1c298f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c2993, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x157c2996, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1c299b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecc299e, 61 },
    { 0x8cdc299f, 98 },
    { 0x869c299f, 34 },
    { 0, 0 },
    { 0x2dac29a2, 34 },
    { 0xc72c29a3, 33 },
    { 0x774c29a2, 67 },
    { 0x34cc29a3, 87 },
    { 0, 0 },
    { 0xee6c29a7, 33 },
    { 0xaaec29a8, 67 },
    { 0x5c6c29a7, 23 },
    { 0x270c29aa, 96 },
    { 0x2fac29ab, 46 },
    { 0x399c29ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289c29b0, 81 },
    { 0, 0 },
    { 0x4ebc29b2, 90 },
    { 0, 0 },
    { 0x46ec29b4, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38c29b8, 95 },
    { 0x4edc29b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x206c29bc, 33 },
    { 0xc8dc29bc, 85 },
    { 0xa12c29be, 91 },
    { 0, 0 },
    { 0xe6c29c0, 33 },
    { 0x84c29c1, 98 },
    { 0x7ac29c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcc29c6, 65 },
    { 0x28cc29c7, 95 },
    { 0xf06c29c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bc29cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3c29d2, 99 },
    { 0xbe8c29d2, 33 },
    { 0xab4c29d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x135c29d7, 33 },
    { 0xa0c29d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9c29dc, 95 },
    { 0x9fec29dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbc29e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06c29e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2c29ec, 95 },
    { 0xa79c29ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ec29f0, 95 },
    { 0, 0 },
    { 0x94fc29f2, 90 },
    { 0x80fc29f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ec29f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adc29fa, 90 },
    { 0, 0 },
    { 0xa4ac29fc, 61 },
    { 0x97ec29fc, 90 },
    { 0xfdcc29fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efc2a03, 90 },
    { 0x273c2a04, 61 },
    { 0xd7cc2a05, 90 },
    { 0xb66c2a03, 99 },
    { 0x9f4c2a07, 33 },
    { 0x138c2a08, 86 },
    { 0xa65c2a05, 75 },
    { 0, 0 },
    { 0x98dc2a0b, 90 },
    { 0x7fbc2a0c, 90 },
    { 0x807c2a0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626c2a12, 61 },
    { 0xc2bc2a13, 23 },
    { 0xf61c2a14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fc2a1a, 88 },
    { 0xd62c2a1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4c2a1e, 23 },
    { 0x68bc2a1f, 77 },
    { 0x2f4c2a20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bc2a24, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec2a2e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ac2a36, 65 },
    { 0x46c2a37, 23 },
    { 0, 0 },
    { 0xac0c2a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x522c2a3c, 61 },
    { 0xa58c2a3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c2a40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6c2a48, 61 },
    { 0, 0 },
    { 0x1f3c2a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c2a4f, 98 },
    { 0x174c2a50, 40 },
    { 0x7e3c2a50, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9c2a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673c2a63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac2a67, 77 },
    { 0x14c2a68, 78 },
    { 0, 0 },
    { 0x32c2a6a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec2a6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabac2a72, 98 },
    { 0x30dc2a73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8c2a78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cc2a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc2a80, 100 },
    { 0x114c2a80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afc2a8c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x561c2a8f, 66 },
    { 0x980c2a90, 93 },
    { 0x2e4c2a8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bc2a95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ac2a99, 78 },
    { 0xbc1c2a9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22c2aa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707c2aa8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c2aab, 40 },
    { 0, 0 },
    { 0x8bec2aad, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619c2ab2, 90 },
    { 0x8f5c2ab3, 90 },
    { 0xe3ec2ab3, 55 },
    { 0xc55c2ab5, 95 },
    { 0, 0 },
    { 0x6e7c2ab7, 77 },
    { 0xc44c2ab7, 52 },
    { 0, 0 },
    { 0xc6bc2aba, 33 },
    { 0x9abc2abb, 61 },
    { 0xea5c2abb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc2abf, 85 },
    { 0xb96c2ac0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c2acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cc2ad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c2ada, 76 },
    { 0xd89c2adb, 23 },
    { 0, 0 },
    { 0x9fc2add, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6c2ae2, 71 },
    { 0xeaac2ae2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01c2ae8, 34 },
    { 0, 0 },
    { 0x24bc2aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7c2aed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569c2af3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53c2af8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec2afc, 43 },
    { 0xde2c2afd, 99 },
    { 0xb53c2afd, 23 },
    { 0x239c2afe, 90 },
    { 0x7f7c2b00, 33 },
    { 0x90dc2b00, 97 },
    { 0x67c2b02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c2b07, 95 },
    { 0xe81c2b08, 23 },
    { 0xfb4c2b09, 43 },
    { 0xb99c2b0a, 90 },
    { 0x711c2b0b, 66 },
    { 0, 0 },
    { 0x4c9c2b0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752c2b14, 33 },
    { 0x36cc2b15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c2b1c, 34 },
    { 0xaa5c2b1d, 95 },
    { 0x1ffc2b1d, 90 },
    { 0xd4bc2b1f, 33 },
    { 0xed6c2b20, 61 },
    { 0x4dc2b20, 83 },
    { 0, 0 },
    { 0x42ac2b23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07c2b27, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1c2b2a, 34 },
    { 0, 0 },
    { 0x56bc2b2c, 34 },
    { 0xe69c2b2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43c2b30, 95 },
    { 0, 0 },
    { 0x296c2b32, 33 },
    { 0xb6c2b33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c2b36, 90 },
    { 0xc5ac2b36, 61 },
    { 0, 0 },
    { 0x88c2b39, 34 },
    { 0xa4cc2b39, 43 },
    { 0x947c2b3b, 98 },
    { 0, 0 },
    { 0x94ac2b3d, 61 },
    { 0x66ec2b3d, 34 },
    { 0xcdbc2b3f, 23 },
    { 0x2cdc2b40, 26 },
    { 0x23ac2b41, 38 },
    { 0, 0 },
    { 0xc54c2b43, 76 },
    { 0x47fc2b44, 90 },
    { 0x43fc2b45, 93 },
    { 0x6f7c2b45, 95 },
    { 0, 0 },
    { 0x27ac2b48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2c2b4b, 33 },
    { 0x16cc2b4c, 96 },
    { 0xff7c2b4d, 29 },
    { 0xe4dc2b4d, 95 },
    { 0x826c2b4f, 55 },
    { 0xf4cc2b4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc2b53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecc2b58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ec2b5b, 88 },
    { 0xbafc2b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bc2b60, 90 },
    { 0xfc7c2b61, 55 },
    { 0, 0 },
    { 0xe24c2b63, 23 },
    { 0, 0 },
    { 0x677c2b65, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c2b69, 78 },
    { 0xd64c2b6a, 90 },
    { 0xc3fc2b69, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc2b71, 33 },
    { 0x348c2b72, 61 },
    { 0xcfc2b72, 84 },
    { 0x977c2b74, 98 },
    { 0xde3c2b74, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243c2b79, 95 },
    { 0, 0 },
    { 0xa79c2b7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40c2b7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c2b82, 46 },
    { 0xde6c2b82, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ec2b86, 90 },
    { 0, 0 },
    { 0xaf4c2b88, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c2b92, 33 },
    { 0x690c2b92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac2b99, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ec2ba5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x897c2ba8, 95 },
    { 0xd6ac2ba9, 66 },
    { 0, 0 },
    { 0xa2fc2bab, 98 },
    { 0xb13c2bab, 61 },
    { 0, 0 },
    { 0x1b2c2bae, 90 },
    { 0, 0 },
    { 0x9fac2bb0, 90 },
    { 0x16ec2bb0, 91 },
    { 0xb56c2bb0, 61 },
    { 0x406c2bb0, 34 },
    { 0, 0 },
    { 0xcd8c2bb5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cac2bb9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5c2bbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bc2bc1, 95 },
    { 0, 0 },
    { 0x7e6c2bc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c2bc6, 88 },
    { 0x4c8c2bc7, 66 },
    { 0, 0 },
    { 0x165c2bc9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228c2bce, 76 },
    { 0xdbac2bce, 55 },
    { 0x3f3c2bce, 47 },
    { 0xe57c2bd1, 55 },
    { 0xcf2c2bd2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c2bd6, 78 },
    { 0, 0 },
    { 0x175c2bd8, 52 },
    { 0xe45c2bd9, 43 },
    { 0x5f4c2bda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785c2bdf, 93 },
    { 0x49ac2be0, 82 },
    { 0x5d2c2be0, 66 },
    { 0xa7c2be2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95c2be7, 61 },
    { 0x2e7c2be8, 52 },
    { 0x210c2be9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x728c2bec, 95 },
    { 0x2aec2bed, 66 },
    { 0xdbcc2bee, 69 },
    { 0xd32c2bef, 43 },
    { 0, 0 },
    { 0xbd2c2bf1, 99 },
    { 0xc84c2bf2, 23 },
    { 0x8cbc2bf2, 98 },
    { 0xd19c2bf3, 90 },
    { 0x25bc2bf5, 78 },
    { 0, 0 },
    { 0x9d1c2bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97c2bfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc2bff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2c2c03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9c2c07, 90 },
    { 0x516c2c07, 61 },
    { 0xad2c2c07, 84 },
    { 0xd77c2c09, 34 },
    { 0, 0 },
    { 0xbe5c2c0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749c2c16, 55 },
    { 0, 0 },
    { 0x234c2c18, 93 },
    { 0xc8fc2c18, 88 },
    { 0x6ac2c1a, 34 },
    { 0x81ec2c1a, 55 },
    { 0x76dc2c1c, 95 },
    { 0, 0 },
    { 0x21cc2c1e, 99 },
    { 0x19dc2c1e, 66 },
    { 0, 0 },
    { 0x3cac2c21, 81 },
    { 0x49ac2c21, 61 },
    { 0, 0 },
    { 0x586c2c24, 95 },
    { 0xc76c2c24, 91 },
    { 0x19bc2c26, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19c2c2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffc2c2f, 95 },
    { 0x59c2c30, 98 },
    { 0, 0 },
    { 0x7fbc2c32, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7c2c37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdc2c3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cc2c40, 55 },
    { 0, 0 },
    { 0xf04c2c42, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeac2c48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c2c4b, 90 },
    { 0, 0 },
    { 0xa18c2c4d, 67 },
    { 0x6a2c2c4e, 76 },
    { 0x680c2c4e, 95 },
    { 0xb63c2c4f, 61 },
    { 0xd03c2c51, 61 },
    { 0x8b4c2c52, 23 },
    { 0x2dc2c4f, 100 },
    { 0xfcfc2c4e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x287c2c57, 36 },
    { 0, 0 },
    { 0x63cc2c59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ec2c5c, 33 },
    { 0, 0 },
    { 0x7d8c2c5e, 61 },
    { 0x21dc2c5e, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33c2c65, 61 },
    { 0x455c2c66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fc2c69, 84 },
    { 0, 0 },
    { 0x6c6c2c6b, 33 },
    { 0xbe1c2c6c, 23 },
    { 0x84cc2c6d, 46 },
    { 0xad8c2c6c, 96 },
    { 0x9ecc2c6f, 90 },
    { 0xa1dc2c70, 95 },
    { 0xefcc2c6d, 91 },
    { 0x14dc2c6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15c2c77, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ec2c7b, 61 },
    { 0, 0 },
    { 0xd05c2c7d, 95 },
    { 0xc46c2c7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89c2c81, 34 },
    { 0, 0 },
    { 0x402c2c83, 34 },
    { 0xad8c2c84, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48c2c87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474c2c8c, 46 },
    { 0x5bcc2c8c, 95 },
    { 0x62fc2c8e, 55 },
    { 0x8a6c2c8f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8c2c93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27c2c9c, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26c2c9f, 83 },
    { 0x54ec2ca0, 77 },
    { 0xe97c2c9f, 95 },
    { 0xba7c2ca2, 93 },
    { 0x614c2ca3, 98 },
    { 0xf73c2ca4, 95 },
    { 0x665c2c9f, 95 },
    { 0x523c2ca6, 85 },
    { 0x586c2ca7, 61 },
    { 0xd6c2ca7, 33 },
    { 0xc65c2ca9, 76 },
    { 0xff9c2ca9, 98 },
    { 0x1b2c2ca7, 61 },
    { 0xd16c2cac, 33 },
    { 0xd55c2cad, 33 },
    { 0xfcc2cac, 98 },
    { 0xadec2cae, 95 },
    { 0x8ffc2cb0, 61 },
    { 0x8b9c2ca3, 61 },
    { 0x9aac2cb2, 95 },
    { 0x361c2cb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacc2cb6, 87 },
    { 0x316c2cb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbc2cbd, 76 },
    { 0x516c2cbe, 17 },
    { 0xac1c2cbd, 99 },
    { 0x631c2cc0, 66 },
    { 0, 0 },
    { 0xbcac2cc2, 67 },
    { 0x48bc2cc2, 76 },
    { 0x9e2c2cc2, 93 },
    { 0x269c2cc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ac2ccc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0c2cd1, 98 },
    { 0, 0 },
    { 0x52ec2cd3, 78 },
    { 0x1bac2cd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xccc2cd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c2ce1, 90 },
    { 0, 0 },
    { 0x6b5c2ce3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10c2ce7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51c2cec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccec2cf4, 76 },
    { 0xa51c2cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1c2cf8, 90 },
    { 0, 0 },
    { 0x4d2c2cfa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc2d00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dc2d03, 98 },
    { 0x3d1c2d04, 55 },
    { 0xf70c2d05, 33 },
    { 0xc29c2d06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49c2d09, 33 },
    { 0x6b4c2d09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124c2d0e, 61 },
    { 0x214c2d0f, 76 },
    { 0xd74c2d0e, 90 },
    { 0xe03c2d11, 61 },
    { 0, 0 },
    { 0xc2cc2d13, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x376c2d16, 98 },
    { 0xcb5c2d17, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc2d1a, 23 },
    { 0x783c2d1a, 93 },
    { 0x60ec2d1c, 99 },
    { 0, 0 },
    { 0xc61c2d1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209c2d27, 75 },
    { 0, 0 },
    { 0xb4c2d29, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x530c2d2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9c2d33, 68 },
    { 0x9dbc2d34, 99 },
    { 0, 0 },
    { 0xe4ec2d36, 23 },
    { 0, 0 },
    { 0x615c2d38, 68 },
    { 0x8e1c2d38, 33 },
    { 0x790c2d3a, 61 },
    { 0x855c2d3a, 95 },
    { 0x132c2d3c, 90 },
    { 0x814c2d3c, 90 },
    { 0x9e6c2d3c, 90 },
    { 0xdf7c2d39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801c2d43, 23 },
    { 0x58fc2d44, 33 },
    { 0, 0 },
    { 0xc4dc2d46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09c2d4d, 34 },
    { 0xfeec2d4d, 99 },
    { 0xb65c2d4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc2d56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9c2d60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc2d6a, 90 },
    { 0x5dbc2d6a, 98 },
    { 0x9ac2d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4c2d6f, 33 },
    { 0x943c2d6f, 33 },
    { 0x37dc2d70, 61 },
    { 0xc53c2d72, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbc2d79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ec2d7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cc2d87, 43 },
    { 0x945c2d88, 66 },
    { 0, 0 },
    { 0x5b7c2d8a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7c2d90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebc2d95, 95 },
    { 0x7ecc2d95, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eac2d9b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c2da2, 33 },
    { 0x1c2c2da2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81c2da6, 33 },
    { 0x9ac2da6, 33 },
    { 0, 0 },
    { 0xf20c2da9, 33 },
    { 0x397c2da9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2dae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c2db1, 68 },
    { 0x819c2db2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ec2db7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fc2dbc, 66 },
    { 0xa28c2dbc, 76 },
    { 0xf4fc2dbd, 85 },
    { 0xb1ec2dbf, 23 },
    { 0x7e2c2dbf, 52 },
    { 0, 0 },
    { 0x366c2dc2, 61 },
    { 0x1dc2dc2, 33 },
    { 0xcd5c2dc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6c2dcb, 61 },
    { 0xf33c2dcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8c2dd1, 90 },
    { 0, 0 },
    { 0xf6c2dd3, 75 },
    { 0xd0ac2dd4, 55 },
    { 0, 0 },
    { 0xd8bc2dd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdec2de0, 69 },
    { 0xd8ac2de0, 81 },
    { 0xa30c2de2, 96 },
    { 0xf7c2de3, 100 },
    { 0, 0 },
    { 0xa2ac2de5, 23 },
    { 0xb63c2de6, 23 },
    { 0xf4ec2de5, 100 },
    { 0, 0 },
    { 0x48dc2de9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acc2ded, 33 },
    { 0x31c2ded, 23 },
    { 0x1b6c2def, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec2df2, 33 },
    { 0xffec2df3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bc2df8, 61 },
    { 0, 0 },
    { 0xeebc2dfa, 100 },
    { 0, 0 },
    { 0x5a7c2dfc, 84 },
    { 0x69bc2dfd, 90 },
    { 0x9b9c2dfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89c2e02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80c2e06, 66 },
    { 0, 0 },
    { 0xad4c2e08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dc2e0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x867c2e0e, 61 },
    { 0, 0 },
    { 0x35ac2e10, 33 },
    { 0x4b0c2e10, 33 },
    { 0x6f2c2e11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c2e19, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c2e1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59c2e1f, 61 },
    { 0xf81c2e20, 23 },
    { 0xfdfc2e1f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x62c2e24, 46 },
    { 0x1cc2e24, 43 },
    { 0, 0 },
    { 0x8aac2e27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fc2e2f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dc2e32, 100 },
    { 0xc1c2e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21c2e36, 100 },
    { 0, 0 },
    { 0xceec2e38, 55 },
    { 0xe6bc2e39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fec2e3c, 95 },
    { 0x8c4c2e3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4c2e40, 90 },
    { 0x26bc2e40, 98 },
    { 0, 0 },
    { 0xc1cc2e43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7c2e49, 98 },
    { 0x984c2e49, 61 },
    { 0x752c2e4b, 90 },
    { 0xea4c2e4c, 61 },
    { 0xe76c2e4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6c2e50, 55 },
    { 0x1dbc2e50, 90 },
    { 0, 0 },
    { 0x43ec2e53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cc2e57, 23 },
    { 0, 0 },
    { 0x243c2e59, 93 },
    { 0x42dc2e5a, 76 },
    { 0xbc3c2e5b, 98 },
    { 0x754c2e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26c2e61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fc2e64, 90 },
    { 0xdb2c2e64, 32 },
    { 0xf5cc2e66, 23 },
    { 0xa9c2e66, 38 },
    { 0, 0 },
    { 0xa60c2e69, 61 },
    { 0x89bc2e6a, 98 },
    { 0xde5c2e6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x856c2e6e, 33 },
    { 0x719c2e6f, 23 },
    { 0x19c2e70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87c2e74, 52 },
    { 0, 0 },
    { 0xebfc2e76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcc2e7b, 55 },
    { 0, 0 },
    { 0xd4fc2e7d, 78 },
    { 0, 0 },
    { 0x8d3c2e7f, 23 },
    { 0x2bec2e80, 34 },
    { 0x6aec2e81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244c2e85, 46 },
    { 0xafcc2e85, 61 },
    { 0, 0 },
    { 0xfb2c2e88, 61 },
    { 0xd5cc2e89, 55 },
    { 0xde3c2e8a, 55 },
    { 0x6fcc2e8b, 23 },
    { 0xf84c2e8c, 93 },
    { 0, 0 },
    { 0x4e9c2e8e, 52 },
    { 0x98ac2e8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cc2e92, 17 },
    { 0, 0 },
    { 0x964c2e94, 46 },
    { 0xa02c2e94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c2e99, 66 },
    { 0xea2c2e99, 33 },
    { 0xe36c2e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6c2e9e, 90 },
    { 0, 0 },
    { 0x9f5c2ea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500c2ea5, 34 },
    { 0x118c2ea6, 100 },
    { 0xc6dc2ea7, 95 },
    { 0, 0 },
    { 0x7afc2ea9, 48 },
    { 0, 0 },
    { 0x843c2eab, 87 },
    { 0x9b5c2eac, 84 },
    { 0x895c2eac, 23 },
    { 0xc88c2eae, 81 },
    { 0xcf5c2ead, 95 },
    { 0, 0 },
    { 0x341c2eb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bc2eb7, 33 },
    { 0x544c2eb8, 95 },
    { 0xe60c2eb7, 34 },
    { 0, 0 },
    { 0xf2bc2ebb, 88 },
    { 0xa5ac2ebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45c2ec0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2c2ec3, 61 },
    { 0x425c2ec4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68c2ec9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc2ed1, 34 },
    { 0, 0 },
    { 0x1efc2ed3, 33 },
    { 0x5cfc2ed3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c2ed7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ec2eda, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfc2ede, 33 },
    { 0xd73c2edf, 95 },
    { 0, 0 },
    { 0xfc0c2ee1, 61 },
    { 0, 0 },
    { 0x1d0c2ee3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1c2ee6, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x427c2ee9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x520c2eec, 95 },
    { 0x321c2eed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ac2ef2, 23 },
    { 0x354c2ef3, 61 },
    { 0x6fdc2ef4, 81 },
    { 0x6abc2ef3, 23 },
    { 0x7dbc2ef6, 95 },
    { 0xa1cc2ef3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59c2efb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c2efe, 33 },
    { 0xb58c2eff, 98 },
    { 0x802c2f00, 34 },
    { 0xa68c2eff, 67 },
    { 0xc71c2f02, 23 },
    { 0xad7c2efe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dc2f0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x279c2f10, 43 },
    { 0xcf2c2f11, 17 },
    { 0x401c2f12, 23 },
    { 0x54cc2f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc2f17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fc2f1d, 33 },
    { 0x59ac2f1d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffc2f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c2f2d, 98 },
    { 0xc15c2f2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3c2f38, 95 },
    { 0xc47c2f39, 61 },
    { 0x32ec2f38, 55 },
    { 0x1a1c2f3b, 95 },
    { 0, 0 },
    { 0x336c2f3d, 61 },
    { 0x490c2f3e, 33 },
    { 0xd53c2f3f, 96 },
    { 0, 0 },
    { 0x277c2f41, 55 },
    { 0, 0 },
    { 0xeeec2f43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697c2f49, 61 },
    { 0xa26c2f4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5c2f4d, 34 },
    { 0xe65c2f4e, 46 },
    { 0, 0 },
    { 0xf49c2f50, 84 },
    { 0, 0 },
    { 0xc79c2f52, 34 },
    { 0x87ec2f53, 65 },
    { 0x66bc2f54, 84 },
    { 0x106c2f55, 95 },
    { 0x483c2f56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18c2f5a, 98 },
    { 0x64bc2f5b, 61 },
    { 0, 0 },
    { 0x83cc2f5d, 61 },
    { 0, 0 },
    { 0x8fdc2f5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c2f63, 93 },
    { 0, 0 },
    { 0x243c2f65, 55 },
    { 0x91cc2f66, 91 },
    { 0x1ddc2f67, 23 },
    { 0x856c2f67, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbc2f6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc2f6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x738c2f72, 46 },
    { 0xf34c2f73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c2f79, 23 },
    { 0, 0 },
    { 0x343c2f7b, 90 },
    { 0xad4c2f7b, 64 },
    { 0, 0 },
    { 0xb22c2f7e, 23 },
    { 0x9abc2f7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71c2f83, 61 },
    { 0xa45c2f83, 95 },
    { 0xd92c2f85, 99 },
    { 0xe3fc2f86, 66 },
    { 0x6d8c2f87, 61 },
    { 0x8ffc2f83, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14c2f92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25c2f96, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ac2f99, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1c2fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83c2fab, 85 },
    { 0x575c2fab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68c2fb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c2fb3, 55 },
    { 0, 0 },
    { 0x25ac2fb5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac2fb8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ac2fbb, 23 },
    { 0xfc9c2fbc, 61 },
    { 0x92ec2fbb, 87 },
    { 0xe8cc2fbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cc2fc7, 36 },
    { 0xd9dc2fc7, 61 },
    { 0xaacc2fc9, 90 },
    { 0xc36c2fca, 76 },
    { 0x6ffc2fcb, 76 },
    { 0, 0 },
    { 0x84cc2fcd, 100 },
    { 0xd72c2fce, 98 },
    { 0, 0 },
    { 0xae5c2fd0, 61 },
    { 0, 0 },
    { 0xac0c2fd2, 95 },
    { 0xd0ac2fd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c2fd7, 90 },
    { 0, 0 },
    { 0x72dc2fd9, 98 },
    { 0xc46c2fda, 90 },
    { 0x754c2fd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc2fde, 90 },
    { 0, 0 },
    { 0x577c2fe0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c2fe4, 98 },
    { 0x4a7c2fe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85c2fee, 34 },
    { 0, 0 },
    { 0xf8bc2ff0, 76 },
    { 0x933c2ff0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59c2ff8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acc2ffe, 34 },
    { 0, 0 },
    { 0xd5bc3000, 61 },
    { 0x45cc3001, 34 },
    { 0xb5cc3002, 61 },
    { 0xf35c3003, 90 },
    { 0x630c3004, 61 },
    { 0, 0 },
    { 0x9a0c3006, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ec300a, 43 },
    { 0x6ec300b, 46 },
    { 0, 0 },
    { 0xa0c300d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fc3011, 91 },
    { 0xa5dc3011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac3017, 43 },
    { 0x65fc3018, 33 },
    { 0, 0 },
    { 0xe4fc301a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acc301d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dc3024, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20c302a, 90 },
    { 0xefbc302b, 99 },
    { 0, 0 },
    { 0xe45c302d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3c3030, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec3033, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fc3037, 98 },
    { 0, 0 },
    { 0x613c3039, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecc3043, 61 },
    { 0x968c3044, 67 },
    { 0xacec3043, 98 },
    { 0, 0 },
    { 0xbfdc3047, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cc3050, 61 },
    { 0x931c3051, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83c3054, 23 },
    { 0, 0 },
    { 0x95cc3056, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23c305d, 34 },
    { 0x575c305e, 55 },
    { 0xd1dc305d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1c3068, 78 },
    { 0x80ac3069, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c306d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dc3073, 34 },
    { 0, 0 },
    { 0xd3c3075, 43 },
    { 0, 0 },
    { 0xef6c3077, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55c3082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74c3089, 23 },
    { 0xae8c308a, 84 },
    { 0, 0 },
    { 0x77cc308c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c3090, 66 },
    { 0x697c3091, 95 },
    { 0x472c3092, 61 },
    { 0xddbc3091, 77 },
    { 0x958c3094, 52 },
    { 0x509c3092, 43 },
    { 0x9fec3096, 90 },
    { 0x312c3097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c309b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bc309f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90c30a3, 76 },
    { 0xc91c30a3, 61 },
    { 0, 0 },
    { 0xedbc30a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbc30ac, 84 },
    { 0x874c30ad, 98 },
    { 0xf5c30ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8c30b3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc30b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19c30bb, 76 },
    { 0x432c30bc, 97 },
    { 0x706c30bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bc30c0, 66 },
    { 0xd0ec30c1, 78 },
    { 0xce2c30c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc30d0, 88 },
    { 0, 0 },
    { 0xa89c30d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881c30d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adc30e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34c30e7, 61 },
    { 0, 0 },
    { 0x9f5c30e9, 34 },
    { 0xca6c30e9, 40 },
    { 0xf59c30eb, 33 },
    { 0xac9c30ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x564c30ef, 34 },
    { 0xfdac30f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1c30f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dc30f8, 4 },
    { 0xcdcc30f9, 91 },
    { 0xe10c30fa, 43 },
    { 0, 0 },
    { 0xb48c30fc, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8c30ff, 43 },
    { 0x68dc30ff, 90 },
    { 0, 0 },
    { 0x888c3102, 95 },
    { 0xcffc3103, 34 },
    { 0x908c3104, 67 },
    { 0xb1c3105, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ec3108, 95 },
    { 0xdb6c3109, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3c310c, 66 },
    { 0xc96c310d, 98 },
    { 0x43fc310e, 76 },
    { 0x16ac310e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ec3112, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843c3122, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcc3126, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c312f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3c3136, 76 },
    { 0, 0 },
    { 0x69ec3138, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adc313e, 76 },
    { 0x550c313e, 93 },
    { 0x99bc3140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956c3145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7c3148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89c314f, 33 },
    { 0x28ec3150, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978c3157, 55 },
    { 0xb74c3158, 91 },
    { 0x6afc3159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fc3161, 95 },
    { 0, 0 },
    { 0xee1c3163, 98 },
    { 0x14bc3164, 33 },
    { 0xff6c3165, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79c316d, 36 },
    { 0x804c316e, 61 },
    { 0xc2ec316e, 95 },
    { 0xfd9c316e, 43 },
    { 0xbe4c3171, 52 },
    { 0xc7ec3171, 23 },
    { 0x15cc3170, 23 },
    { 0xbfec316f, 90 },
    { 0x87fc316d, 85 },
    { 0, 0 },
    { 0xf97c3177, 84 },
    { 0x167c3177, 99 },
    { 0xaefc3179, 23 },
    { 0x389c3178, 77 },
    { 0x85dc317b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c317e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc3187, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686c318c, 90 },
    { 0, 0 },
    { 0xa87c318e, 23 },
    { 0, 0 },
    { 0x893c3190, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32c3194, 61 },
    { 0xb9cc3194, 61 },
    { 0xa54c3196, 91 },
    { 0x843c3197, 23 },
    { 0xb4dc3197, 90 },
    { 0x958c3199, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307c319f, 100 },
    { 0x8f1c31a0, 91 },
    { 0xcbac31a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bc31a6, 87 },
    { 0x6cec31a7, 98 },
    { 0x697c31a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5c31ac, 23 },
    { 0x1bfc31ad, 61 },
    { 0, 0 },
    { 0xc61c31af, 76 },
    { 0, 0 },
    { 0x330c31b1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0c31b5, 90 },
    { 0x690c31b6, 90 },
    { 0, 0 },
    { 0xcb2c31b8, 90 },
    { 0, 0 },
    { 0x301c31ba, 33 },
    { 0xad2c31bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dc31be, 88 },
    { 0x793c31bf, 33 },
    { 0x21c31c0, 43 },
    { 0xf3ec31c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4c31c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eec31c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7c31cc, 33 },
    { 0, 0 },
    { 0xc13c31ce, 53 },
    { 0x195c31ce, 93 },
    { 0, 0 },
    { 0x6ebc31d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0c31d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45c31d7, 95 },
    { 0xddbc31d8, 33 },
    { 0x5bbc31d8, 98 },
    { 0xb12c31da, 23 },
    { 0, 0 },
    { 0xcccc31dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc31e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c31e7, 99 },
    { 0, 0 },
    { 0x1dbc31e9, 95 },
    { 0x1fcc31ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c31ed, 90 },
    { 0xedc31ee, 33 },
    { 0x61ac31ef, 75 },
    { 0x56fc31f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34c31f4, 95 },
    { 0xfa1c31f5, 61 },
    { 0xde2c31f5, 34 },
    { 0xd27c31f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3c31fb, 46 },
    { 0xe3c31fb, 33 },
    { 0xfddc31fd, 91 },
    { 0x969c31fe, 36 },
    { 0x836c31fe, 98 },
    { 0, 0 },
    { 0x514c3201, 33 },
    { 0x5f2c3202, 91 },
    { 0x7d6c3202, 90 },
    { 0x8e5c3202, 66 },
    { 0x9cfc3205, 23 },
    { 0x6e2c3206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48c320d, 34 },
    { 0xb53c320d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcc3214, 65 },
    { 0, 0 },
    { 0x710c3216, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cc3221, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcac3225, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c322a, 36 },
    { 0x777c322a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bc322e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fc3232, 90 },
    { 0x150c3233, 61 },
    { 0, 0 },
    { 0x97dc3235, 90 },
    { 0xb7dc3236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4c323a, 90 },
    { 0xf23c323b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94c3240, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c3247, 61 },
    { 0, 0 },
    { 0x950c3249, 76 },
    { 0, 0 },
    { 0x736c324b, 67 },
    { 0x489c324b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356c3250, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ac3257, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78c325a, 23 },
    { 0xf2cc325a, 90 },
    { 0xb43c325c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343c3263, 17 },
    { 0x1d0c3264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797c3271, 90 },
    { 0x926c3272, 98 },
    { 0xd57c3272, 33 },
    { 0x469c3272, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8c3278, 33 },
    { 0, 0 },
    { 0x5a2c327a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1c327f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59c3283, 90 },
    { 0xdc9c3284, 33 },
    { 0x72bc3285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613c3291, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec3294, 93 },
    { 0x9c2c3295, 90 },
    { 0xc00c3294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670c329e, 90 },
    { 0xa63c329f, 55 },
    { 0xcf1c329f, 61 },
    { 0, 0 },
    { 0xa08c32a2, 69 },
    { 0xde2c32a3, 33 },
    { 0x7bcc32a4, 23 },
    { 0xf72c32a5, 61 },
    { 0x85ec32a5, 90 },
    { 0x20fc32a7, 65 },
    { 0x329c32a6, 55 },
    { 0, 0 },
    { 0x322c32aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c32b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c32b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2c32b8, 76 },
    { 0xf55c32b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dac32bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80c32c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5c32c5, 61 },
    { 0, 0 },
    { 0x928c32c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cc32cc, 33 },
    { 0x380c32cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x619c32d0, 52 },
    { 0x436c32d0, 101 },
    { 0x9eac32d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7c32d7, 43 },
    { 0, 0 },
    { 0xa70c32d9, 98 },
    { 0xf57c32da, 95 },
    { 0x35dc32db, 55 },
    { 0x8a2c32dc, 33 },
    { 0xb77c32dd, 90 },
    { 0x12bc32de, 90 },
    { 0x839c32df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692c32e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bc32e8, 46 },
    { 0x771c32e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5c32ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x797c32f1, 90 },
    { 0xb5fc32f1, 33 },
    { 0, 0 },
    { 0x7d7c32f4, 90 },
    { 0x1a0c32f5, 77 },
    { 0x5ec32f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bc32f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8c32fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc3302, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8c3308, 66 },
    { 0, 0 },
    { 0x8bc330a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3310, 84 },
    { 0x489c3311, 34 },
    { 0x7eec3311, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fc3318, 23 },
    { 0, 0 },
    { 0x345c331a, 61 },
    { 0x33dc331b, 95 },
    { 0x652c331c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2c3320, 90 },
    { 0x411c3321, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9c3324, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cc3327, 78 },
    { 0xc04c3328, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x926c332b, 76 },
    { 0x6f5c332b, 23 },
    { 0xc4ac332b, 93 },
    { 0xa0cc332e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c3331, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04c3335, 90 },
    { 0, 0 },
    { 0x230c3337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5c333c, 76 },
    { 0x82c333d, 98 },
    { 0, 0 },
    { 0xbd0c333f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911c3343, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56c3349, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498c334d, 95 },
    { 0x34bc334d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ac3358, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8c335b, 33 },
    { 0xd9ac335c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0c3362, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeec3366, 61 },
    { 0x132c3367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fc336d, 67 },
    { 0xa5fc336e, 60 },
    { 0xb48c336f, 55 },
    { 0x4fc3370, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99c3373, 52 },
    { 0xb5ac3373, 23 },
    { 0, 0 },
    { 0xa5ec3376, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc3379, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc337c, 61 },
    { 0x422c337d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fc3380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99c3383, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59c3387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ac338c, 33 },
    { 0xe00c338d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c3390, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbc3393, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3c3396, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10c3399, 91 },
    { 0x740c3399, 98 },
    { 0, 0 },
    { 0xf83c339c, 33 },
    { 0x2e9c339d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9dc33a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94c33a7, 33 },
    { 0x1a3c33a8, 17 },
    { 0, 0 },
    { 0x990c33aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c33ae, 22 },
    { 0, 0 },
    { 0x912c33b0, 43 },
    { 0x2e1c33b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x848c33b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bec33bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76c33c0, 76 },
    { 0xcf1c33c1, 95 },
    { 0x34ec33c2, 77 },
    { 0x865c33c3, 33 },
    { 0xbd6c33c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec33c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9c33d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694c33d6, 90 },
    { 0, 0 },
    { 0x107c33d8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cc33e6, 90 },
    { 0x1d3c33e7, 81 },
    { 0xf20c33e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85c33ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9c33ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc33fa, 55 },
    { 0xd82c33fa, 91 },
    { 0x3dfc33fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c3400, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c3404, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4c340a, 98 },
    { 0, 0 },
    { 0xa67c340c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689c3410, 95 },
    { 0xb8ec3411, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c3415, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x806c3418, 78 },
    { 0, 0 },
    { 0x22cc341a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdc341d, 84 },
    { 0, 0 },
    { 0xa7cc341f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcc3424, 98 },
    { 0xa18c3424, 90 },
    { 0xb6fc3424, 90 },
    { 0, 0 },
    { 0xb72c3428, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7c3432, 99 },
    { 0xe49c3433, 34 },
    { 0xb18c3434, 90 },
    { 0xeb1c3432, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c3439, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7c343f, 43 },
    { 0, 0 },
    { 0x732c3441, 33 },
    { 0, 0 },
    { 0xbe1c3443, 90 },
    { 0x4f0c3444, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc3448, 61 },
    { 0, 0 },
    { 0xbb0c344a, 78 },
    { 0, 0 },
    { 0x258c344c, 33 },
    { 0xc36c344c, 95 },
    { 0x1e1c344e, 46 },
    { 0xee0c344e, 61 },
    { 0x37bc344f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x177c3453, 61 },
    { 0x2d1c3454, 68 },
    { 0xc43c3455, 33 },
    { 0xb93c3456, 66 },
    { 0xc5fc3456, 61 },
    { 0x5a8c3458, 91 },
    { 0x82c3458, 55 },
    { 0x452c3458, 38 },
    { 0xe5ec3459, 91 },
    { 0xea7c3459, 95 },
    { 0xb00c3454, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68c3463, 23 },
    { 0, 0 },
    { 0x712c3465, 67 },
    { 0x523c3465, 67 },
    { 0x922c3466, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbc346a, 95 },
    { 0x393c346b, 61 },
    { 0x22dc346a, 95 },
    { 0, 0 },
    { 0x164c346e, 77 },
    { 0, 0 },
    { 0x8b9c3470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85c3474, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01c347b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280c347f, 90 },
    { 0x916c3480, 98 },
    { 0x172c3480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41c3484, 95 },
    { 0xc79c3484, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x679c3488, 78 },
    { 0xcddc3489, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcc348d, 93 },
    { 0, 0 },
    { 0xfe6c348f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888c3492, 46 },
    { 0x2fc3492, 23 },
    { 0xf7cc3494, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3c349b, 61 },
    { 0xfa5c349b, 61 },
    { 0, 0 },
    { 0x3cc349e, 33 },
    { 0x25c349f, 98 },
    { 0xf5ec34a0, 88 },
    { 0, 0 },
    { 0xc02c34a2, 83 },
    { 0xcdc34a3, 95 },
    { 0, 0 },
    { 0x1fec34a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887c34ab, 40 },
    { 0, 0 },
    { 0xd96c34ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12c34b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecc34b4, 52 },
    { 0x70ec34b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5c34bd, 61 },
    { 0xfd2c34be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c34c5, 43 },
    { 0, 0 },
    { 0xdc8c34c7, 85 },
    { 0, 0 },
    { 0xac0c34c9, 43 },
    { 0xb7cc34c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29c34d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc34dc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dc34e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecc34e6, 61 },
    { 0xdcfc34e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c34ef, 95 },
    { 0, 0 },
    { 0xf7ac34f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fc34f5, 95 },
    { 0xd22c34f6, 52 },
    { 0xe2bc34f7, 76 },
    { 0x24dc34f7, 33 },
    { 0, 0 },
    { 0x77fc34fa, 77 },
    { 0xfec34fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eec3506, 23 },
    { 0xc4cc3507, 95 },
    { 0xfec3507, 76 },
    { 0x60ec3509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fc350e, 55 },
    { 0xf0bc350e, 17 },
    { 0x2cac3510, 23 },
    { 0xde1c3511, 95 },
    { 0x4cbc3512, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1c3519, 76 },
    { 0, 0 },
    { 0x682c351b, 99 },
    { 0x4c1c351c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc3521, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdc3525, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ec3528, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4c352b, 99 },
    { 0x96ac352c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcc352f, 95 },
    { 0x9b8c3530, 85 },
    { 0x494c3531, 67 },
    { 0, 0 },
    { 0x637c3533, 85 },
    { 0x8e6c3534, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15c3537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c353a, 78 },
    { 0xb7c353b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52c3543, 93 },
    { 0xfd1c3543, 46 },
    { 0x2fbc3545, 90 },
    { 0xd69c3546, 84 },
    { 0xc70c3547, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adc354a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadc354e, 33 },
    { 0x80ec354e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c3555, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc3559, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c355f, 52 },
    { 0, 0 },
    { 0xf8c3561, 76 },
    { 0xb1c3561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44c3568, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c356b, 95 },
    { 0xcc6c356c, 61 },
    { 0, 0 },
    { 0x3cfc356e, 90 },
    { 0x61cc356e, 90 },
    { 0xcc6c3570, 33 },
    { 0, 0 },
    { 0x94cc3572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24c3575, 66 },
    { 0x9d2c3576, 90 },
    { 0xf33c3575, 67 },
    { 0x9c5c3576, 34 },
    { 0x15cc3578, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9c357d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x696c3580, 93 },
    { 0xc95c3581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbc3584, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfc3588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fc358b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c3592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554c3596, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443c359a, 95 },
    { 0x695c359b, 90 },
    { 0x92fc359c, 84 },
    { 0x757c359d, 66 },
    { 0x5c7c359e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ac35a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc35ac, 90 },
    { 0x498c35ad, 55 },
    { 0x1f6c35ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74c35b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeec35be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3c35c2, 98 },
    { 0x716c35c2, 90 },
    { 0x8c0c35c2, 90 },
    { 0x55c35c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cc35c8, 93 },
    { 0, 0 },
    { 0x561c35ca, 61 },
    { 0x5f2c35cb, 66 },
    { 0x943c35ca, 69 },
    { 0x2f8c35cd, 75 },
    { 0x43fc35cd, 100 },
    { 0, 0 },
    { 0x901c35d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ac35d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cc35d7, 90 },
    { 0, 0 },
    { 0x40c35d9, 33 },
    { 0xa0fc35da, 55 },
    { 0xfa9c35db, 90 },
    { 0x450c35dc, 33 },
    { 0xf8dc35db, 55 },
    { 0x920c35db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3c35e4, 61 },
    { 0x961c35e4, 90 },
    { 0x505c35e6, 99 },
    { 0, 0 },
    { 0xd1c35e8, 33 },
    { 0x86ac35e9, 33 },
    { 0xa0cc35ea, 43 },
    { 0x294c35eb, 55 },
    { 0x2dbc35e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628c35f5, 56 },
    { 0x437c35f6, 95 },
    { 0xe5cc35f6, 98 },
    { 0xb36c35f6, 61 },
    { 0x510c35f5, 76 },
    { 0xbf2c35fa, 84 },
    { 0xa15c35f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98c35fe, 65 },
    { 0xa49c35ff, 33 },
    { 0xc43c3600, 23 },
    { 0, 0 },
    { 0x33dc3602, 85 },
    { 0x357c3603, 100 },
    { 0xfd0c3604, 76 },
    { 0xd29c3603, 43 },
    { 0, 0 },
    { 0xd45c3607, 33 },
    { 0xe1cc3608, 87 },
    { 0xf59c3607, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fc360e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fc3611, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cc3614, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833c3619, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x347c361c, 34 },
    { 0, 0 },
    { 0xd8fc361e, 90 },
    { 0, 0 },
    { 0xdc7c3620, 90 },
    { 0x9a8c3621, 76 },
    { 0xc45c3622, 23 },
    { 0xab0c3622, 92 },
    { 0xdecc3624, 52 },
    { 0, 0 },
    { 0xae9c3626, 33 },
    { 0, 0 },
    { 0xf7cc3628, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ac362c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fc362f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x624c3632, 98 },
    { 0xccbc3633, 84 },
    { 0xc3dc3634, 78 },
    { 0xcb8c3632, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98c363c, 98 },
    { 0xc42c363d, 100 },
    { 0x62ac363e, 90 },
    { 0, 0 },
    { 0x5acc3640, 98 },
    { 0x335c3640, 61 },
    { 0xd24c3641, 66 },
    { 0x248c3643, 99 },
    { 0, 0 },
    { 0x750c3645, 56 },
    { 0x754c3645, 43 },
    { 0, 0 },
    { 0xf49c3648, 100 },
    { 0x8e9c3648, 33 },
    { 0xbf1c364a, 23 },
    { 0, 0 },
    { 0x458c364c, 98 },
    { 0, 0 },
    { 0x3bac364e, 33 },
    { 0x91ec364e, 61 },
    { 0x3c2c3650, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x300c3653, 90 },
    { 0xae8c3654, 84 },
    { 0, 0 },
    { 0x84ec3656, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317c365a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c365d, 84 },
    { 0, 0 },
    { 0xf2dc365f, 77 },
    { 0xa02c365f, 66 },
    { 0, 0 },
    { 0x644c3662, 46 },
    { 0x48dc3663, 61 },
    { 0xa34c3664, 23 },
    { 0x8c3c3665, 61 },
    { 0, 0 },
    { 0xd27c3667, 90 },
    { 0, 0 },
    { 0xacec3669, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5c366f, 43 },
    { 0x61c3670, 76 },
    { 0x797c3670, 90 },
    { 0x3bac3672, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ec3676, 33 },
    { 0x375c3677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dac367a, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cc368a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fc368d, 81 },
    { 0x54c368d, 23 },
    { 0, 0 },
    { 0x861c3690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8c3693, 78 },
    { 0xd14c3694, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547c369b, 76 },
    { 0xc08c369b, 98 },
    { 0x9eec369d, 85 },
    { 0x734c369c, 23 },
    { 0xd1bc369c, 23 },
    { 0x41fc369e, 90 },
    { 0x961c36a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ac36a6, 33 },
    { 0, 0 },
    { 0x4d4c36a8, 76 },
    { 0, 0 },
    { 0x484c36aa, 23 },
    { 0, 0 },
    { 0x212c36ac, 61 },
    { 0x807c36ac, 91 },
    { 0x806c36ad, 34 },
    { 0x4edc36af, 61 },
    { 0xdddc36af, 90 },
    { 0, 0 },
    { 0x5ccc36b2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337c36b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeac36bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878c36ca, 46 },
    { 0x31dc36cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557c36d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dc36d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29c36da, 90 },
    { 0x1c1c36db, 80 },
    { 0x341c36db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c36e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fc36e3, 33 },
    { 0, 0 },
    { 0x46cc36e5, 90 },
    { 0, 0 },
    { 0x79bc36e7, 23 },
    { 0x29dc36e8, 43 },
    { 0, 0 },
    { 0x2f9c36ea, 61 },
    { 0xa91c36eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cc36ee, 33 },
    { 0, 0 },
    { 0x1cfc36f0, 91 },
    { 0, 0 },
    { 0xb30c36f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8c36f5, 61 },
    { 0x365c36f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585c36fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c36ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec3702, 69 },
    { 0xab3c3702, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bc3707, 52 },
    { 0, 0 },
    { 0x6a9c3709, 90 },
    { 0, 0 },
    { 0x280c370b, 76 },
    { 0xd47c370c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88c370f, 95 },
    { 0xb22c370f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdc3714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfc3718, 90 },
    { 0xbd8c3719, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45c3720, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717c3728, 56 },
    { 0, 0 },
    { 0xb76c372a, 23 },
    { 0x1b0c372b, 68 },
    { 0x720c372a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adc372f, 46 },
    { 0xeb7c3730, 33 },
    { 0x97ec3731, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc3734, 95 },
    { 0xf0fc3734, 61 },
    { 0xb2fc3736, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc373b, 33 },
    { 0x5a8c373c, 61 },
    { 0, 0 },
    { 0x152c373e, 90 },
    { 0, 0 },
    { 0xe84c3740, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8c3748, 33 },
    { 0x660c3749, 61 },
    { 0x9a1c3749, 90 },
    { 0, 0 },
    { 0x663c374c, 33 },
    { 0x9aec374d, 90 },
    { 0xca5c374e, 23 },
    { 0, 0 },
    { 0x460c3750, 23 },
    { 0xa0c3751, 90 },
    { 0xe7c3751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3c3755, 95 },
    { 0x99c3755, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c375a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ec375e, 78 },
    { 0x9b9c375f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77c376c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ec3772, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc3775, 33 },
    { 0x8a5c3776, 33 },
    { 0, 0 },
    { 0xbecc3778, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ac377e, 66 },
    { 0xb0cc377e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c3784, 93 },
    { 0x7c8c3785, 90 },
    { 0xadac3786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cac378a, 33 },
    { 0xd64c378a, 84 },
    { 0x5a2c378c, 98 },
    { 0x19fc378c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98c3792, 61 },
    { 0x54ec3793, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1c3799, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2c379c, 95 },
    { 0, 0 },
    { 0x916c379e, 90 },
    { 0x410c379e, 66 },
    { 0, 0 },
    { 0xec2c37a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ac37a7, 52 },
    { 0x107c37a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x178c37ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51c37ae, 61 },
    { 0xaa4c37ae, 90 },
    { 0xdd5c37ae, 76 },
    { 0x3d1c37b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c37b6, 61 },
    { 0xe8cc37b6, 33 },
    { 0x603c37b8, 46 },
    { 0xef9c37b9, 90 },
    { 0xff8c37ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc37bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c37c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88c37c3, 90 },
    { 0x258c37c3, 33 },
    { 0x52cc37c4, 95 },
    { 0x9f0c37c6, 61 },
    { 0x5a3c37c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cc37ca, 78 },
    { 0xd19c37cb, 33 },
    { 0xf2dc37cc, 78 },
    { 0xc4c37cd, 90 },
    { 0, 0 },
    { 0xcd3c37cf, 23 },
    { 0x96ac37d0, 46 },
    { 0, 0 },
    { 0x4a9c37d2, 90 },
    { 0x74dc37d3, 23 },
    { 0x928c37d4, 76 },
    { 0x285c37d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42c37da, 98 },
    { 0xc65c37da, 85 },
    { 0x6b3c37da, 46 },
    { 0, 0 },
    { 0x953c37de, 66 },
    { 0xf14c37df, 90 },
    { 0x4cec37de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0c37e6, 95 },
    { 0x121c37e7, 43 },
    { 0x53dc37e8, 52 },
    { 0xfc3c37e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2c37ed, 93 },
    { 0x25dc37ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc37f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59c37fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460c3803, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c3806, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbc380a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6c380d, 76 },
    { 0xfaec380d, 61 },
    { 0, 0 },
    { 0x173c3810, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3c3814, 23 },
    { 0x378c3815, 33 },
    { 0xe65c3815, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327c381c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343c3822, 98 },
    { 0x811c3823, 98 },
    { 0x4bec3824, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66c3828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec382d, 85 },
    { 0x576c382e, 23 },
    { 0, 0 },
    { 0x3f8c3830, 76 },
    { 0x406c3830, 61 },
    { 0, 0 },
    { 0xa3ec3833, 77 },
    { 0, 0 },
    { 0xfb0c3835, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c3838, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613c383d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9c3841, 33 },
    { 0xcfbc3842, 90 },
    { 0xb2dc3842, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846c3847, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c384a, 67 },
    { 0xf1cc384a, 76 },
    { 0xc36c384a, 95 },
    { 0x5a5c384b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354c3852, 98 },
    { 0x7cbc3852, 43 },
    { 0x887c3853, 87 },
    { 0, 0 },
    { 0x289c3856, 90 },
    { 0x826c3857, 23 },
    { 0x3ecc3858, 61 },
    { 0xe48c3859, 95 },
    { 0, 0 },
    { 0x179c385b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec385e, 95 },
    { 0xa99c385f, 61 },
    { 0x3aac385f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16c3863, 81 },
    { 0x47ec3864, 78 },
    { 0xf92c3865, 23 },
    { 0x8eac3864, 33 },
    { 0xa4bc3866, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec386a, 67 },
    { 0, 0 },
    { 0xbeac386c, 23 },
    { 0xc13c386d, 61 },
    { 0, 0 },
    { 0x6b7c386f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c3873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6c3876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bc3879, 23 },
    { 0, 0 },
    { 0x941c387b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc387f, 78 },
    { 0x56ec3880, 61 },
    { 0xb13c3881, 64 },
    { 0x8c3881, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979c3887, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223c388b, 61 },
    { 0, 0 },
    { 0x7aec388d, 61 },
    { 0xa6bc388e, 61 },
    { 0xcf8c388e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec3892, 100 },
    { 0, 0 },
    { 0x213c3894, 23 },
    { 0xb92c3895, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acc3898, 34 },
    { 0xc02c3899, 61 },
    { 0, 0 },
    { 0x575c389b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c389e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc38a5, 98 },
    { 0, 0 },
    { 0x867c38a7, 23 },
    { 0x10ac38a8, 95 },
    { 0xef8c38a9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc38b0, 23 },
    { 0x657c38b1, 90 },
    { 0x8a9c38b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c38b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fc38be, 100 },
    { 0x528c38be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ac38c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aac38cd, 93 },
    { 0x66ac38ce, 61 },
    { 0xfacc38cd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79c38d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785c38d9, 76 },
    { 0xab3c38da, 95 },
    { 0, 0 },
    { 0x563c38dc, 67 },
    { 0, 0 },
    { 0x278c38de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aac38e2, 55 },
    { 0x565c38e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x881c38e6, 43 },
    { 0x9bbc38e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abc38ea, 75 },
    { 0, 0 },
    { 0x4d6c38ec, 95 },
    { 0xd5cc38ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c38f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c38f7, 61 },
    { 0, 0 },
    { 0x391c38f9, 98 },
    { 0x46fc38f9, 23 },
    { 0x8aec38fb, 61 },
    { 0x90cc38fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc3900, 55 },
    { 0x2eec3900, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0c3904, 33 },
    { 0, 0 },
    { 0x499c3906, 77 },
    { 0xf90c3907, 61 },
    { 0xeacc3907, 61 },
    { 0xa3cc3909, 53 },
    { 0x3b2c3908, 95 },
    { 0x822c390b, 43 },
    { 0xc48c390b, 61 },
    { 0x141c390c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fec3912, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4c3916, 85 },
    { 0xa92c3917, 23 },
    { 0xf20c3916, 65 },
    { 0xf27c3919, 61 },
    { 0x2cec3919, 61 },
    { 0x964c391b, 98 },
    { 0x449c3919, 55 },
    { 0xa1bc391d, 76 },
    { 0x2ddc391c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2c3921, 33 },
    { 0x25ec3921, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63c3926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c392a, 77 },
    { 0, 0 },
    { 0x8c2c392c, 61 },
    { 0xd55c392d, 33 },
    { 0xc1ec392e, 76 },
    { 0x61fc392c, 87 },
    { 0xb79c392e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67c3934, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14c3939, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8c393d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2c3940, 23 },
    { 0, 0 },
    { 0x417c3942, 33 },
    { 0x73ac3942, 83 },
    { 0xdf5c3943, 43 },
    { 0x2e3c3944, 75 },
    { 0xa82c3946, 61 },
    { 0x8fbc3947, 52 },
    { 0xe59c3948, 93 },
    { 0, 0 },
    { 0xe0ec394a, 98 },
    { 0x1e0c394b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c394e, 98 },
    { 0, 0 },
    { 0xf4cc3950, 23 },
    { 0x5dbc3950, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768c395b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5c3960, 90 },
    { 0, 0 },
    { 0xd0bc3962, 55 },
    { 0x69cc3963, 98 },
    { 0, 0 },
    { 0xabec3965, 55 },
    { 0, 0 },
    { 0x8b9c3967, 33 },
    { 0x493c3968, 95 },
    { 0xbd6c3968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdc396c, 61 },
    { 0, 0 },
    { 0x1f1c396e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cc3971, 90 },
    { 0, 0 },
    { 0x65cc3973, 90 },
    { 0xd5fc3974, 61 },
    { 0x95bc3973, 91 },
    { 0, 0 },
    { 0x3f7c3977, 90 },
    { 0xbbdc3977, 87 },
    { 0xe66c3979, 75 },
    { 0x9a6c3979, 55 },
    { 0xf1bc397b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642c397f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656c3983, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c3987, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfc398a, 53 },
    { 0, 0 },
    { 0x44c398c, 46 },
    { 0xf45c398d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ac3990, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bc3999, 91 },
    { 0x8c4c3999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x545c399d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdac39a5, 99 },
    { 0, 0 },
    { 0xb35c39a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecc39aa, 61 },
    { 0x662c39aa, 76 },
    { 0, 0 },
    { 0x33c39ad, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446c39b1, 33 },
    { 0, 0 },
    { 0x5dcc39b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c39c2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x645c39c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884c39d0, 68 },
    { 0xf90c39d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc39d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cc39d9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x252c39dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ec39e2, 43 },
    { 0xa83c39e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c39e9, 33 },
    { 0xf79c39ea, 23 },
    { 0x4b2c39e9, 96 },
    { 0xc08c39ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc39f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cc39fd, 34 },
    { 0x7a5c39fe, 33 },
    { 0x232c39ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40c3a02, 66 },
    { 0xf42c3a03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x183c3a06, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edc3a09, 34 },
    { 0, 0 },
    { 0x2dfc3a0b, 23 },
    { 0x19c3a0c, 33 },
    { 0xc68c3a0b, 66 },
    { 0, 0 },
    { 0xfd8c3a0f, 69 },
    { 0x738c3a10, 93 },
    { 0x2b6c3a11, 98 },
    { 0xa41c3a12, 81 },
    { 0xe05c3a13, 95 },
    { 0x1e2c3a13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35c3a17, 61 },
    { 0xd23c3a18, 46 },
    { 0xca5c3a19, 76 },
    { 0x4adc3a1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ac3a1e, 99 },
    { 0x410c3a1f, 94 },
    { 0x611c3a1f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdc3a24, 98 },
    { 0xe93c3a25, 90 },
    { 0xc21c3a25, 93 },
    { 0x33cc3a26, 61 },
    { 0xfb1c3a28, 61 },
    { 0x722c3a27, 61 },
    { 0xff2c3a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x398c3a2d, 78 },
    { 0, 0 },
    { 0xa87c3a2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3c3a37, 34 },
    { 0xd7fc3a37, 98 },
    { 0xabcc3a39, 61 },
    { 0, 0 },
    { 0x961c3a3b, 95 },
    { 0x488c3a3c, 33 },
    { 0x20c3a3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ec3a41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c3a44, 23 },
    { 0xb0dc3a45, 91 },
    { 0x920c3a45, 33 },
    { 0x130c3a46, 33 },
    { 0x3a6c3a47, 61 },
    { 0, 0 },
    { 0x93fc3a4a, 69 },
    { 0x1d7c3a4a, 55 },
    { 0xcc3a4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ac3a53, 98 },
    { 0x186c3a53, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8c3a57, 87 },
    { 0, 0 },
    { 0x75ac3a59, 95 },
    { 0xb8c3a5a, 17 },
    { 0xbddc3a59, 43 },
    { 0x5eac3a5c, 23 },
    { 0, 0 },
    { 0xd03c3a5e, 95 },
    { 0x14dc3a5f, 23 },
    { 0x802c3a60, 33 },
    { 0x43fc3a61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edc3a65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5c3a6d, 90 },
    { 0x3c1c3a6e, 37 },
    { 0, 0 },
    { 0x686c3a70, 17 },
    { 0, 0 },
    { 0x799c3a72, 99 },
    { 0x9ac3a73, 33 },
    { 0, 0 },
    { 0xc5fc3a75, 61 },
    { 0x455c3a75, 100 },
    { 0x5b2c3a77, 23 },
    { 0x196c3a78, 33 },
    { 0xbe4c3a79, 61 },
    { 0xcc7c3a7a, 77 },
    { 0x2b7c3a7a, 34 },
    { 0x2dbc3a7c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ec3a7f, 55 },
    { 0, 0 },
    { 0xa07c3a81, 67 },
    { 0xeb1c3a81, 33 },
    { 0x1dc3a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac3a86, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1c3a89, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac3a8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c3a90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79c3a93, 87 },
    { 0xa81c3a93, 61 },
    { 0x25cc3a95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcc3a98, 100 },
    { 0, 0 },
    { 0x33c3a9a, 98 },
    { 0xa36c3a9b, 95 },
    { 0, 0 },
    { 0x794c3a9d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1c3aa2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22c3aa5, 68 },
    { 0x6d8c3aa5, 55 },
    { 0xc2ec3aa7, 66 },
    { 0, 0 },
    { 0x2cac3aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184c3aae, 34 },
    { 0x457c3aae, 55 },
    { 0x3b5c3ab0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5c3ab3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3c3aba, 90 },
    { 0, 0 },
    { 0x99bc3abc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795c3ac3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2c3aca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82c3ace, 76 },
    { 0x8c8c3ace, 33 },
    { 0x2b3c3ad0, 61 },
    { 0, 0 },
    { 0x799c3ad2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2c3ad8, 98 },
    { 0x835c3ad9, 95 },
    { 0, 0 },
    { 0xbd6c3adb, 90 },
    { 0x6f6c3adb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6c3ae1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4c3ae5, 61 },
    { 0xdf0c3ae6, 90 },
    { 0xc68c3ae7, 90 },
    { 0x8d7c3ae8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cc3af1, 76 },
    { 0, 0 },
    { 0xefdc3af3, 90 },
    { 0xbe9c3af4, 61 },
    { 0, 0 },
    { 0xa3ec3af6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x709c3af9, 43 },
    { 0xf0fc3afa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2c3afd, 61 },
    { 0xd7fc3afd, 33 },
    { 0, 0 },
    { 0xe5ec3b00, 76 },
    { 0x21ac3b01, 84 },
    { 0, 0 },
    { 0x524c3b03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301c3b0a, 95 },
    { 0x70ec3b0b, 46 },
    { 0xcb7c3b0b, 67 },
    { 0, 0 },
    { 0xe91c3b0e, 90 },
    { 0, 0 },
    { 0x73dc3b10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244c3b15, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bc3b1c, 98 },
    { 0, 0 },
    { 0x4cdc3b1e, 52 },
    { 0x38bc3b1f, 78 },
    { 0x220c3b1f, 33 },
    { 0x7a3c3b21, 33 },
    { 0xe90c3b22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42c3b26, 66 },
    { 0xaa0c3b27, 99 },
    { 0x8fac3b26, 43 },
    { 0x886c3b29, 90 },
    { 0x65fc3b2a, 55 },
    { 0, 0 },
    { 0x860c3b2c, 43 },
    { 0, 0 },
    { 0xbf0c3b2e, 61 },
    { 0xde8c3b2e, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfec3b34, 65 },
    { 0x591c3b35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c3b3c, 90 },
    { 0x80fc3b3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x774c3b40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c3b43, 61 },
    { 0xe16c3b43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffc3b47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40c3b4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dc3b50, 69 },
    { 0xfe4c3b51, 90 },
    { 0x89ec3b51, 84 },
    { 0, 0 },
    { 0x231c3b54, 33 },
    { 0xc12c3b54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81c3b5a, 87 },
    { 0x12ac3b5b, 43 },
    { 0xa44c3b5a, 78 },
    { 0, 0 },
    { 0xc66c3b5e, 95 },
    { 0xaa6c3b5f, 95 },
    { 0, 0 },
    { 0x675c3b61, 23 },
    { 0x34fc3b62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc3b65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bc3b69, 85 },
    { 0xd69c3b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47c3b6d, 61 },
    { 0, 0 },
    { 0x9e9c3b6f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc3b72, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69c3b77, 83 },
    { 0x570c3b78, 67 },
    { 0x2f5c3b78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1c3b7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1dc3b81, 67 },
    { 0x577c3b82, 46 },
    { 0xdf4c3b83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x328c3b86, 95 },
    { 0x622c3b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dac3b91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617c3b97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c3b9c, 61 },
    { 0x59fc3b9d, 98 },
    { 0, 0 },
    { 0x574c3b9f, 96 },
    { 0, 0 },
    { 0x848c3ba1, 23 },
    { 0, 0 },
    { 0x9aec3ba3, 99 },
    { 0x480c3ba4, 76 },
    { 0x67ac3ba5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6c3bad, 61 },
    { 0x958c3bae, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdc3bb1, 93 },
    { 0x6f4c3bb2, 55 },
    { 0x99ec3bb3, 78 },
    { 0xfcc3bb4, 95 },
    { 0x276c3bb4, 80 },
    { 0x481c3bb6, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199c3bbc, 76 },
    { 0x1d5c3bbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ec3bc0, 34 },
    { 0x72c3bc1, 52 },
    { 0, 0 },
    { 0xca7c3bc3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ec3bcb, 81 },
    { 0x833c3bcc, 90 },
    { 0xda6c3bcd, 61 },
    { 0xb01c3bcd, 90 },
    { 0xb32c3bce, 75 },
    { 0xeabc3bd0, 61 },
    { 0x900c3bd0, 90 },
    { 0xe3dc3bd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97c3bd6, 61 },
    { 0x94ec3bd6, 90 },
    { 0xa9fc3bd6, 78 },
    { 0, 0 },
    { 0xd07c3bda, 43 },
    { 0xdfbc3bda, 90 },
    { 0x496c3bdc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9c3be0, 55 },
    { 0, 0 },
    { 0xbcc3be2, 37 },
    { 0x810c3be3, 98 },
    { 0x1d1c3be3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c3be9, 43 },
    { 0, 0 },
    { 0xc61c3beb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x117c3bee, 33 },
    { 0x267c3bef, 61 },
    { 0xa46c3bf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938c3bf6, 76 },
    { 0x149c3bf7, 33 },
    { 0x76c3bf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bc3bfb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2c3bfe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04c3c01, 90 },
    { 0, 0 },
    { 0x836c3c03, 23 },
    { 0, 0 },
    { 0xdd0c3c05, 61 },
    { 0x145c3c06, 61 },
    { 0xac6c3c06, 93 },
    { 0xf59c3c08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57c3c0e, 33 },
    { 0x399c3c0e, 33 },
    { 0x940c3c10, 69 },
    { 0, 0 },
    { 0x29fc3c12, 99 },
    { 0x875c3c13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dc3c17, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c3c23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49c3c2b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbc3c34, 99 },
    { 0x191c3c35, 95 },
    { 0x731c3c36, 93 },
    { 0x62cc3c37, 81 },
    { 0xd9dc3c36, 46 },
    { 0x699c3c34, 33 },
    { 0xe3fc3c3a, 46 },
    { 0xec8c3c3b, 90 },
    { 0x72ec3c3c, 97 },
    { 0xe88c3c3c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac3c41, 91 },
    { 0, 0 },
    { 0x68c3c43, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2c3c50, 23 },
    { 0, 0 },
    { 0xdd8c3c52, 91 },
    { 0, 0 },
    { 0x3c2c3c54, 87 },
    { 0, 0 },
    { 0x7fbc3c56, 98 },
    { 0, 0 },
    { 0x3cac3c58, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c3c63, 52 },
    { 0x9bec3c63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ac3c68, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0c3c6b, 55 },
    { 0, 0 },
    { 0x764c3c6d, 78 },
    { 0x304c3c6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x532c3c71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54c3c7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cc3c7d, 55 },
    { 0, 0 },
    { 0xb65c3c7f, 90 },
    { 0x431c3c80, 76 },
    { 0x5ccc3c7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c3c85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c3c89, 98 },
    { 0x14bc3c8a, 84 },
    { 0x3d4c3c8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6c3c8e, 90 },
    { 0x410c3c8f, 95 },
    { 0xcfac3c90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ac3c93, 76 },
    { 0xeb5c3c94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x919c3c97, 85 },
    { 0x75c3c97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666c3ca0, 66 },
    { 0, 0 },
    { 0xae0c3ca2, 90 },
    { 0x8d4c3ca2, 77 },
    { 0xd33c3ca3, 95 },
    { 0x9d2c3ca5, 95 },
    { 0, 0 },
    { 0x65fc3ca7, 87 },
    { 0xa09c3ca8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfc3cac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8c3cb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9c3cbe, 100 },
    { 0, 0 },
    { 0xcdfc3cc0, 33 },
    { 0xdfc3cc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cc3cc5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7c3cc8, 43 },
    { 0x471c3cc9, 64 },
    { 0, 0 },
    { 0xfeac3ccb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac3cce, 43 },
    { 0x4a4c3ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94c3cd4, 34 },
    { 0, 0 },
    { 0xb96c3cd6, 95 },
    { 0, 0 },
    { 0xad3c3cd8, 95 },
    { 0x45cc3cd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605c3cdd, 61 },
    { 0, 0 },
    { 0xd20c3cdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68c3ce3, 83 },
    { 0, 0 },
    { 0x3fac3ce5, 33 },
    { 0xdec3ce6, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c3cec, 43 },
    { 0, 0 },
    { 0x4dc3cee, 75 },
    { 0xfd4c3cef, 55 },
    { 0x1d5c3cf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfc3cf4, 23 },
    { 0x7b5c3cf4, 67 },
    { 0xb9cc3cf5, 78 },
    { 0xa2dc3cf5, 61 },
    { 0x979c3cf8, 75 },
    { 0xda6c3cf9, 43 },
    { 0x627c3cf9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ec3d00, 95 },
    { 0x4f5c3d01, 61 },
    { 0xc9ec3d02, 33 },
    { 0x414c3d03, 95 },
    { 0x7f6c3d04, 33 },
    { 0xe30c3d03, 61 },
    { 0x2fcc3d03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86c3d09, 90 },
    { 0, 0 },
    { 0xfd1c3d0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c3d11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bc3d16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc3d19, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbc3d24, 61 },
    { 0x74cc3d25, 33 },
    { 0, 0 },
    { 0x58dc3d27, 95 },
    { 0x2ac3d28, 23 },
    { 0x700c3d27, 23 },
    { 0, 0 },
    { 0xfc4c3d2b, 77 },
    { 0x996c3d2b, 93 },
    { 0xc1c3d2d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c3d30, 68 },
    { 0x91ec3d31, 90 },
    { 0x79c3d31, 34 },
    { 0x521c3d31, 90 },
    { 0x28bc3d34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bc3d38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c3d3c, 93 },
    { 0, 0 },
    { 0xc51c3d3e, 100 },
    { 0x416c3d3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897c3d43, 95 },
    { 0, 0 },
    { 0xe43c3d45, 55 },
    { 0, 0 },
    { 0x442c3d47, 34 },
    { 0, 0 },
    { 0x254c3d49, 90 },
    { 0xa5fc3d4a, 46 },
    { 0x3fbc3d4b, 23 },
    { 0x58ec3d4a, 73 },
    { 0x71dc3d4a, 99 },
    { 0x46fc3d4e, 84 },
    { 0, 0 },
    { 0x2c1c3d50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13c3d57, 98 },
    { 0, 0 },
    { 0xcbac3d59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87c3d5d, 87 },
    { 0xfa5c3d5d, 61 },
    { 0, 0 },
    { 0xac4c3d60, 98 },
    { 0x11cc3d61, 72 },
    { 0, 0 },
    { 0xe6ac3d63, 91 },
    { 0x720c3d63, 90 },
    { 0x305c3d64, 81 },
    { 0x9b8c3d66, 61 },
    { 0xfa1c3d67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28c3d6b, 90 },
    { 0x376c3d6c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c3d6f, 34 },
    { 0, 0 },
    { 0x79cc3d71, 46 },
    { 0, 0 },
    { 0x335c3d73, 98 },
    { 0x2d6c3d74, 46 },
    { 0xa1dc3d74, 23 },
    { 0xc52c3d76, 98 },
    { 0x3bfc3d77, 34 },
    { 0x91bc3d75, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaec3d7b, 90 },
    { 0, 0 },
    { 0x9cac3d7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150c3d81, 98 },
    { 0x82c3d82, 23 },
    { 0xf8ac3d83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adc3d8a, 90 },
    { 0x944c3d8b, 91 },
    { 0x5a2c3d8a, 23 },
    { 0x8e5c3d8a, 95 },
    { 0x32ec3d8e, 93 },
    { 0x31c3d8e, 55 },
    { 0x231c3d8c, 23 },
    { 0xef8c3d8b, 33 },
    { 0x62bc3d8f, 76 },
    { 0x5ac3d93, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bc3d96, 52 },
    { 0xaa6c3d97, 76 },
    { 0x1a9c3d96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c3d9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ec3d9f, 17 },
    { 0xcfc3da0, 42 },
    { 0x5eac3da1, 61 },
    { 0x33ec3da2, 84 },
    { 0x37c3da2, 76 },
    { 0x35cc3da2, 61 },
    { 0xb5ac3da3, 53 },
    { 0, 0 },
    { 0x655c3da7, 67 },
    { 0x1d6c3da8, 87 },
    { 0xceec3da8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4c3dac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4c3db0, 46 },
    { 0xe8ec3db1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dc3db4, 76 },
    { 0xb2dc3db4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac3dbc, 67 },
    { 0xd1ec3dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7c3dc6, 76 },
    { 0x257c3dc7, 90 },
    { 0x84c3dc7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93c3dcc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c3dcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16c3ddd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134c3de3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacec3de8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39c3dec, 17 },
    { 0xdafc3ded, 98 },
    { 0x567c3ded, 98 },
    { 0x8bcc3def, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1c3df7, 61 },
    { 0x53c3df8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2c3dfb, 98 },
    { 0x285c3dfc, 100 },
    { 0x1ffc3dfc, 95 },
    { 0x3bc3dfc, 55 },
    { 0xe1dc3dfe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c3e05, 23 },
    { 0, 0 },
    { 0x369c3e07, 33 },
    { 0, 0 },
    { 0xed7c3e09, 96 },
    { 0x41bc3e0a, 55 },
    { 0xcb2c3e0b, 85 },
    { 0xdd8c3e0c, 55 },
    { 0xabac3e0b, 98 },
    { 0xdbdc3e0e, 95 },
    { 0x752c3e0f, 34 },
    { 0, 0 },
    { 0x27ac3e11, 84 },
    { 0x3bc3e12, 23 },
    { 0xaa4c3e13, 90 },
    { 0x1a1c3e12, 95 },
    { 0xf79c3e15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10c3e1d, 98 },
    { 0x92ac3e1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2c3e22, 43 },
    { 0xf71c3e23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3e29, 80 },
    { 0x5d2c3e2a, 85 },
    { 0xa80c3e2b, 33 },
    { 0, 0 },
    { 0xc25c3e2d, 33 },
    { 0x97c3e2e, 46 },
    { 0x6bfc3e2f, 98 },
    { 0x785c3e30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63c3e35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x374c3e38, 30 },
    { 0x344c3e38, 55 },
    { 0x9ecc3e3a, 23 },
    { 0, 0 },
    { 0x804c3e3c, 65 },
    { 0x518c3e3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x112c3e40, 33 },
    { 0x7cc3e40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649c3e46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63c3e4d, 93 },
    { 0xfbdc3e4e, 33 },
    { 0xb7dc3e4f, 43 },
    { 0x22c3e4f, 93 },
    { 0xcaac3e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edc3e56, 55 },
    { 0, 0 },
    { 0x5b8c3e58, 95 },
    { 0, 0 },
    { 0x1c8c3e5a, 33 },
    { 0x478c3e5b, 90 },
    { 0x579c3e5c, 90 },
    { 0x2a8c3e5d, 33 },
    { 0, 0 },
    { 0x44cc3e5f, 90 },
    { 0x638c3e5f, 90 },
    { 0xd2bc3e61, 90 },
    { 0x146c3e60, 61 },
    { 0xe15c3e63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c3e6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c3e70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x619c3e73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x238c3e76, 33 },
    { 0xa41c3e76, 35 },
    { 0x284c3e77, 90 },
    { 0, 0 },
    { 0xaaec3e7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32c3e81, 61 },
    { 0x2dfc3e82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2c3e87, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6c3e8c, 90 },
    { 0xf0dc3e8c, 23 },
    { 0x84c3e8e, 34 },
    { 0x46bc3e8f, 95 },
    { 0x7ffc3e8d, 90 },
    { 0, 0 },
    { 0xe8bc3e92, 68 },
    { 0x1d7c3e92, 46 },
    { 0x5ac3e92, 100 },
    { 0, 0 },
    { 0xb37c3e96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc3ea2, 66 },
    { 0xc08c3ea3, 67 },
    { 0x885c3ea3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc3eaf, 23 },
    { 0, 0 },
    { 0xee2c3eb1, 98 },
    { 0xb15c3eb2, 33 },
    { 0xc37c3eb1, 61 },
    { 0x2ddc3eb4, 95 },
    { 0x4b8c3eb5, 23 },
    { 0, 0 },
    { 0x439c3eb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00c3ebf, 76 },
    { 0xb4c3ec0, 33 },
    { 0xb6dc3ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cc3ec4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fc3ec8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c3ecb, 55 },
    { 0xdac3ecc, 95 },
    { 0x4a1c3ecd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951c3ed1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bc3ed4, 61 },
    { 0xa8c3ed5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c3ed8, 61 },
    { 0x919c3ed9, 77 },
    { 0x4bdc3ed9, 100 },
    { 0, 0 },
    { 0x8fcc3edc, 33 },
    { 0x647c3edc, 65 },
    { 0x91bc3ede, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6c3ee2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ac3ee7, 52 },
    { 0x1cfc3ee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c3ef1, 66 },
    { 0, 0 },
    { 0x5f1c3ef3, 95 },
    { 0, 0 },
    { 0x3b7c3ef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ac3ef8, 95 },
    { 0xefac3ef9, 61 },
    { 0xc60c3efa, 76 },
    { 0x3eac3ef8, 95 },
    { 0x5d8c3efb, 90 },
    { 0xcb5c3ef9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ac3f00, 76 },
    { 0xf04c3f01, 80 },
    { 0x503c3f01, 33 },
    { 0x1dec3f03, 34 },
    { 0x1f8c3f03, 17 },
    { 0xa76c3f01, 81 },
    { 0xf88c3f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eec3f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22c3f0d, 90 },
    { 0xa30c3f0e, 93 },
    { 0x67cc3f0f, 33 },
    { 0x4fdc3f10, 23 },
    { 0x234c3f11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51c3f14, 87 },
    { 0, 0 },
    { 0x664c3f16, 66 },
    { 0, 0 },
    { 0x7d2c3f18, 61 },
    { 0x33c3f18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5c3f1f, 43 },
    { 0, 0 },
    { 0xe3ec3f21, 46 },
    { 0x6bdc3f21, 61 },
    { 0x10ac3f21, 34 },
    { 0x91fc3f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x230c3f27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ac3f2b, 68 },
    { 0x5dac3f2c, 55 },
    { 0x91fc3f2d, 77 },
    { 0x298c3f2d, 90 },
    { 0xc21c3f2f, 33 },
    { 0, 0 },
    { 0x161c3f31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7c3f34, 55 },
    { 0x32bc3f34, 98 },
    { 0x977c3f35, 61 },
    { 0, 0 },
    { 0xa6cc3f38, 98 },
    { 0xd01c3f39, 40 },
    { 0x8f8c3f39, 95 },
    { 0x347c3f38, 43 },
    { 0, 0 },
    { 0x408c3f3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4c3f44, 90 },
    { 0x9a3c3f45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cc3f49, 33 },
    { 0xcb3c3f49, 46 },
    { 0xc6bc3f4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192c3f50, 66 },
    { 0x8dcc3f51, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ac3f54, 83 },
    { 0x929c3f55, 81 },
    { 0x1e5c3f56, 98 },
    { 0x27bc3f56, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762c3f5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4c3f64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c3f67, 98 },
    { 0, 0 },
    { 0x355c3f69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc3f70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18c3f73, 95 },
    { 0x7aec3f74, 34 },
    { 0x11dc3f75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31c3f7c, 46 },
    { 0xf9cc3f7d, 98 },
    { 0x8f3c3f7c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789c3f82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bc3f87, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7c3f8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec3f8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413c3f92, 69 },
    { 0, 0 },
    { 0xf39c3f94, 90 },
    { 0xb96c3f95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec3f98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bc3f9c, 61 },
    { 0xfd3c3f9d, 61 },
    { 0x7ddc3f9d, 23 },
    { 0xb54c3f9d, 95 },
    { 0x76dc3f9d, 87 },
    { 0xbedc3fa1, 92 },
    { 0xdac3fa0, 46 },
    { 0xd40c3fa1, 69 },
    { 0xa99c3fa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c3faa, 93 },
    { 0xee9c3fab, 34 },
    { 0, 0 },
    { 0x9cdc3fad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ec3fb3, 33 },
    { 0xec1c3fb3, 99 },
    { 0, 0 },
    { 0xf10c3fb6, 99 },
    { 0xe40c3fb6, 36 },
    { 0xa82c3fb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dc3fbb, 98 },
    { 0x706c3fbc, 90 },
    { 0x838c3fbd, 61 },
    { 0, 0 },
    { 0x29ac3fbf, 46 },
    { 0x465c3fc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fc3fc9, 76 },
    { 0x1e4c3fca, 95 },
    { 0x87cc3fcb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703c3fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbc3fda, 90 },
    { 0xbacc3fdb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c3fe4, 90 },
    { 0xadc3fe5, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dc3fe8, 33 },
    { 0, 0 },
    { 0x64bc3fea, 66 },
    { 0x634c3fea, 66 },
    { 0x4bdc3fec, 33 },
    { 0x8c3fec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ac3ff8, 75 },
    { 0, 0 },
    { 0x7b0c3ffa, 33 },
    { 0, 0 },
    { 0xa24c3ffc, 96 },
    { 0x9bfc3ffd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ac4000, 90 },
    { 0x919c4001, 61 },
    { 0x4cbc4002, 90 },
    { 0x44bc4001, 33 },
    { 0x779c4004, 64 },
    { 0, 0 },
    { 0x5e9c4006, 55 },
    { 0, 0 },
    { 0xea0c4008, 93 },
    { 0xa6c4009, 33 },
    { 0x309c4008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c400d, 69 },
    { 0x8e7c400e, 90 },
    { 0x881c400d, 34 },
    { 0, 0 },
    { 0x806c4011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683c4016, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc401d, 93 },
    { 0xd7cc401e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7c4022, 76 },
    { 0, 0 },
    { 0xa97c4024, 52 },
    { 0xce3c4024, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fc402a, 98 },
    { 0, 0 },
    { 0x666c402c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc402f, 67 },
    { 0, 0 },
    { 0x153c4031, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x556c4034, 33 },
    { 0x85dc4035, 40 },
    { 0x11fc4036, 70 },
    { 0x79c4037, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216c403b, 76 },
    { 0x598c403b, 43 },
    { 0x978c403d, 23 },
    { 0, 0 },
    { 0x775c403f, 75 },
    { 0, 0 },
    { 0xa01c4041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41c4044, 91 },
    { 0x188c4045, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34c4048, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x699c404b, 34 },
    { 0xa76c404c, 66 },
    { 0x259c404d, 68 },
    { 0x71bc404c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c4056, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbec4059, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dc4065, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9c4068, 95 },
    { 0xe31c4068, 34 },
    { 0x3d7c406a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c406e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabac4074, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x703c4077, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96c407a, 99 },
    { 0xc76c407b, 98 },
    { 0x7f6c407c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x291c407f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2c4082, 95 },
    { 0xff2c4083, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75c4087, 66 },
    { 0xba2c4087, 95 },
    { 0x314c4088, 90 },
    { 0x6d6c408a, 55 },
    { 0x660c408a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc4092, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9c4095, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ac409b, 76 },
    { 0, 0 },
    { 0x2d3c409d, 90 },
    { 0xd53c409e, 55 },
    { 0xd07c409f, 33 },
    { 0x99c409f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c40a7, 33 },
    { 0xa4cc40a8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9c40af, 23 },
    { 0x2cac40b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23c40b4, 34 },
    { 0x133c40b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cac40ba, 66 },
    { 0, 0 },
    { 0xc58c40bc, 23 },
    { 0xdbcc40bd, 98 },
    { 0, 0 },
    { 0x8b6c40bf, 61 },
    { 0xe08c40bf, 33 },
    { 0x669c40c1, 90 },
    { 0, 0 },
    { 0x90bc40c3, 34 },
    { 0xfb1c40c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325c40ca, 77 },
    { 0, 0 },
    { 0x10fc40cc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c40d1, 33 },
    { 0x867c40d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3c40d6, 17 },
    { 0x9bdc40d7, 78 },
    { 0x55bc40d7, 90 },
    { 0x31fc40d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5c40de, 61 },
    { 0, 0 },
    { 0xda3c40e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350c40e5, 76 },
    { 0xa09c40e6, 33 },
    { 0xf63c40e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2c40ec, 90 },
    { 0xcf1c40ec, 90 },
    { 0x3a1c40ed, 34 },
    { 0x3bac40ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2c40f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49c40fc, 33 },
    { 0, 0 },
    { 0xa2bc40fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x354c4101, 23 },
    { 0xcbcc4102, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83c410a, 66 },
    { 0xbf4c410b, 90 },
    { 0x7c3c410b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918c4115, 43 },
    { 0, 0 },
    { 0x2fbc4117, 23 },
    { 0x78dc4117, 46 },
    { 0, 0 },
    { 0x29dc411a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1c4121, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc4124, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbc4127, 90 },
    { 0x95fc4127, 68 },
    { 0xd67c4129, 90 },
    { 0, 0 },
    { 0xcbcc412b, 61 },
    { 0, 0 },
    { 0x7f1c412d, 75 },
    { 0xae2c412e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdc4133, 88 },
    { 0x1d2c4134, 17 },
    { 0xc56c4135, 90 },
    { 0, 0 },
    { 0x486c4137, 87 },
    { 0xe74c4138, 95 },
    { 0, 0 },
    { 0x896c413a, 61 },
    { 0xd93c413b, 65 },
    { 0xbe5c413c, 98 },
    { 0, 0 },
    { 0xd68c413e, 23 },
    { 0, 0 },
    { 0x638c4140, 97 },
    { 0x801c4141, 95 },
    { 0xe1dc4142, 61 },
    { 0xf9bc4140, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dc4147, 100 },
    { 0x67dc4148, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76c414b, 33 },
    { 0x537c414c, 76 },
    { 0xe40c414c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517c4158, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc415c, 61 },
    { 0xa96c415d, 95 },
    { 0x9a6c415e, 90 },
    { 0, 0 },
    { 0xb5ec4160, 46 },
    { 0x678c4160, 90 },
    { 0, 0 },
    { 0x3c5c4163, 95 },
    { 0xb11c4164, 17 },
    { 0x90dc4164, 55 },
    { 0, 0 },
    { 0xae8c4167, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc416a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4c416f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431c4176, 23 },
    { 0, 0 },
    { 0xe66c4178, 98 },
    { 0xb22c4178, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc417f, 34 },
    { 0, 0 },
    { 0xce8c4181, 61 },
    { 0xce6c4182, 61 },
    { 0, 0 },
    { 0x2f8c4184, 67 },
    { 0x438c4184, 55 },
    { 0x1a7c4184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fc418a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac418e, 68 },
    { 0xd2cc418f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7c4192, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11c4197, 46 },
    { 0x8e3c4198, 43 },
    { 0x3a9c4197, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ec419c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c41a1, 61 },
    { 0x768c41a2, 23 },
    { 0xbdfc41a3, 90 },
    { 0xed7c41a3, 55 },
    { 0x32fc41a5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x828c41a8, 33 },
    { 0, 0 },
    { 0xe38c41aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4c41ae, 23 },
    { 0, 0 },
    { 0x17ec41b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ac41b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bc41b9, 23 },
    { 0x31cc41b9, 90 },
    { 0x7efc41bb, 78 },
    { 0xc08c41bc, 76 },
    { 0x237c41ba, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08c41c0, 93 },
    { 0x8c2c41c1, 66 },
    { 0x919c41c2, 95 },
    { 0x4e0c41c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66c41c6, 55 },
    { 0x105c41c7, 33 },
    { 0, 0 },
    { 0xfc8c41c9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5c41cd, 61 },
    { 0, 0 },
    { 0xd41c41cf, 95 },
    { 0x1efc41d0, 87 },
    { 0x9bfc41d1, 52 },
    { 0x5b6c41d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3c41d6, 33 },
    { 0x4d1c41d7, 91 },
    { 0x240c41d8, 93 },
    { 0x47dc41d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c41dc, 34 },
    { 0x684c41dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec41e2, 46 },
    { 0, 0 },
    { 0xb42c41e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacc41e9, 85 },
    { 0x8c2c41ea, 17 },
    { 0, 0 },
    { 0x733c41ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353c41f0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567c41f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cc41fc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac41ff, 66 },
    { 0, 0 },
    { 0xa73c4201, 98 },
    { 0, 0 },
    { 0x825c4203, 76 },
    { 0, 0 },
    { 0xde7c4205, 52 },
    { 0x9e2c4206, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8c420e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edc4213, 33 },
    { 0xd6c4214, 19 },
    { 0xb07c4215, 55 },
    { 0xffdc4214, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdc421a, 78 },
    { 0xf03c421a, 77 },
    { 0, 0 },
    { 0x7a2c421d, 33 },
    { 0x220c421e, 65 },
    { 0xab1c421f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61c4222, 95 },
    { 0x651c4223, 23 },
    { 0xf89c4224, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6c4228, 95 },
    { 0x49dc4229, 33 },
    { 0xd90c422a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1c422f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ac4233, 95 },
    { 0, 0 },
    { 0x12ac4235, 90 },
    { 0, 0 },
    { 0x340c4237, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968c4244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38c424a, 76 },
    { 0x19dc424b, 87 },
    { 0x873c424a, 93 },
    { 0xf11c424d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dec4250, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3c4257, 84 },
    { 0, 0 },
    { 0x948c4259, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccc4263, 98 },
    { 0, 0 },
    { 0x291c4265, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dc4269, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c426c, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88c426f, 34 },
    { 0x131c4270, 78 },
    { 0x226c4271, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ec4275, 34 },
    { 0xc70c4275, 23 },
    { 0, 0 },
    { 0x3c0c4278, 46 },
    { 0x798c4279, 23 },
    { 0xdac4279, 78 },
    { 0x963c427b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xadac427e, 33 },
    { 0x25dc427f, 81 },
    { 0, 0 },
    { 0x501c4281, 78 },
    { 0x5a1c4282, 75 },
    { 0x256c4282, 61 },
    { 0xbc1c4282, 98 },
    { 0xb22c4285, 52 },
    { 0x43c4285, 61 },
    { 0xb81c4287, 98 },
    { 0x33fc4288, 95 },
    { 0x21ac4287, 99 },
    { 0x53cc428a, 98 },
    { 0xed8c428b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x164c428e, 61 },
    { 0, 0 },
    { 0xbf3c4290, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaec4295, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c4298, 88 },
    { 0xf5ac4299, 33 },
    { 0x4ddc429a, 100 },
    { 0x8d8c429b, 91 },
    { 0xd9cc429c, 90 },
    { 0x29fc429d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381c42a4, 61 },
    { 0xeb0c42a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5c42a9, 98 },
    { 0x77dc42aa, 75 },
    { 0x3a1c42ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7c42b0, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bc42b9, 77 },
    { 0xd72c42ba, 90 },
    { 0x879c42bb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x765c42be, 76 },
    { 0, 0 },
    { 0xb73c42c0, 75 },
    { 0x156c42c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efc42c6, 66 },
    { 0, 0 },
    { 0x589c42c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02c42cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ac42ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5c42d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09c42d4, 34 },
    { 0x4d4c42d5, 100 },
    { 0x7f6c42d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c42d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48c42e1, 33 },
    { 0x4c3c42e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ac42e8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c42eb, 76 },
    { 0, 0 },
    { 0x6aac42ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c42f0, 33 },
    { 0x5bbc42f1, 98 },
    { 0xef9c42f2, 65 },
    { 0x5cac42f2, 95 },
    { 0xfd1c42f4, 33 },
    { 0, 0 },
    { 0xafec42f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46c42fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x879c42fd, 33 },
    { 0, 0 },
    { 0x442c42ff, 88 },
    { 0xb58c4300, 23 },
    { 0x5a1c4300, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c4307, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831c430b, 23 },
    { 0xb9fc430c, 33 },
    { 0x15cc430c, 90 },
    { 0, 0 },
    { 0xef5c430f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dc4313, 84 },
    { 0xe6ec4313, 69 },
    { 0x150c4315, 43 },
    { 0xdfac4314, 95 },
    { 0xfedc4317, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3c431a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4c431d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bc4322, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc4328, 43 },
    { 0x1e9c4329, 55 },
    { 0x855c432a, 74 },
    { 0x273c432b, 64 },
    { 0, 0 },
    { 0xb8ac432d, 76 },
    { 0, 0 },
    { 0x76bc432f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fc4332, 19 },
    { 0x7d4c4333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cc4336, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bc4340, 95 },
    { 0x422c4341, 76 },
    { 0x266c4341, 23 },
    { 0, 0 },
    { 0x6f6c4344, 88 },
    { 0, 0 },
    { 0xf5fc4346, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c4350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14c4355, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967c4359, 65 },
    { 0xd82c4359, 61 },
    { 0x87ac435b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c435e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c4364, 90 },
    { 0x941c4365, 23 },
    { 0x2f5c4366, 33 },
    { 0, 0 },
    { 0x4f3c4368, 55 },
    { 0x172c4368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bc436e, 95 },
    { 0, 0 },
    { 0x3c2c4370, 23 },
    { 0xacfc4370, 77 },
    { 0x5d4c4371, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374c437a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852c437f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97c4388, 99 },
    { 0xe24c4389, 99 },
    { 0x1dfc4388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17c438d, 98 },
    { 0x4fcc438e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c4392, 99 },
    { 0x655c4393, 95 },
    { 0xaa5c4394, 76 },
    { 0x7d7c4394, 77 },
    { 0x82fc4395, 23 },
    { 0xb9c4393, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337c439c, 90 },
    { 0, 0 },
    { 0x6fac439e, 84 },
    { 0x579c439f, 90 },
    { 0x3e8c439f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4c43a6, 91 },
    { 0xbd6c43a6, 34 },
    { 0x639c43a6, 91 },
    { 0x2f3c43a6, 98 },
    { 0x4a5c43a9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86c43ad, 95 },
    { 0x913c43ae, 98 },
    { 0x85cc43af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13c43b3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45c43b6, 98 },
    { 0, 0 },
    { 0xdbc43b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3c43bc, 90 },
    { 0x6adc43bc, 67 },
    { 0x8a3c43be, 90 },
    { 0x2a3c43be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcc43c6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88c43cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66c43d0, 98 },
    { 0x306c43d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c43d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fc43e0, 85 },
    { 0xdfbc43e0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cc43e5, 23 },
    { 0xb95c43e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570c43ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c43f2, 77 },
    { 0x946c43f3, 23 },
    { 0x396c43f4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103c43fc, 33 },
    { 0, 0 },
    { 0xc90c43fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c4408, 23 },
    { 0, 0 },
    { 0x312c440a, 33 },
    { 0x633c440a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc4411, 23 },
    { 0x4adc4412, 40 },
    { 0x931c4413, 98 },
    { 0x40dc4412, 52 },
    { 0x1e7c4412, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ec4423, 93 },
    { 0xce1c4423, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c4429, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960c442f, 88 },
    { 0x333c442f, 100 },
    { 0x9f1c4431, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x422c4434, 90 },
    { 0xed6c4435, 95 },
    { 0xc19c4435, 61 },
    { 0, 0 },
    { 0x53fc4438, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x777c443b, 33 },
    { 0x111c443b, 76 },
    { 0x355c443d, 33 },
    { 0xfa2c443c, 61 },
    { 0x2f0c443d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ac4446, 95 },
    { 0xd65c4446, 43 },
    { 0x9bec4448, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89c4454, 76 },
    { 0xedec4455, 98 },
    { 0xe9bc4456, 23 },
    { 0xaa8c4457, 67 },
    { 0x1c7c4455, 55 },
    { 0, 0 },
    { 0x9fcc445a, 95 },
    { 0, 0 },
    { 0xb2fc445c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc4460, 76 },
    { 0x4e5c4461, 23 },
    { 0x236c4462, 90 },
    { 0xcfbc4463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ac4468, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x736c446b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ec446e, 23 },
    { 0, 0 },
    { 0xa64c4470, 78 },
    { 0x6e9c4471, 90 },
    { 0x125c4471, 93 },
    { 0, 0 },
    { 0xbe0c4474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0c4478, 95 },
    { 0x8c9c4479, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ac447e, 61 },
    { 0x412c447f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x573c4482, 46 },
    { 0x3f3c4483, 33 },
    { 0x5a3c4483, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bc4489, 76 },
    { 0x71ec448a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422c4491, 90 },
    { 0xfbac4491, 61 },
    { 0x9d7c4493, 90 },
    { 0xf32c4494, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9c4497, 34 },
    { 0, 0 },
    { 0xfa3c4499, 61 },
    { 0xaccc449a, 84 },
    { 0x2dfc449b, 23 },
    { 0x519c449c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529c44a4, 61 },
    { 0xc92c44a4, 98 },
    { 0x6c6c44a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8c44ad, 98 },
    { 0x690c44ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dc44b2, 55 },
    { 0x43ac44b3, 90 },
    { 0x70fc44b2, 33 },
    { 0xc31c44b5, 95 },
    { 0xadbc44b6, 61 },
    { 0x153c44b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4c44c0, 33 },
    { 0, 0 },
    { 0xb8bc44c2, 61 },
    { 0, 0 },
    { 0x65c44c4, 100 },
    { 0xd3dc44c5, 84 },
    { 0x54ec44c4, 46 },
    { 0x62cc44c7, 96 },
    { 0x85c44c8, 61 },
    { 0x816c44c9, 91 },
    { 0x436c44ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9c44ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ac44d2, 33 },
    { 0xe88c44d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fc44d7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98c44da, 23 },
    { 0, 0 },
    { 0x156c44dc, 93 },
    { 0x9f6c44dc, 90 },
    { 0, 0 },
    { 0x7e4c44df, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ac44e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3c44e6, 23 },
    { 0x90cc44e7, 76 },
    { 0x308c44e8, 90 },
    { 0, 0 },
    { 0x2a1c44ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395c44ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc44f2, 34 },
    { 0, 0 },
    { 0xf33c44f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9c44f9, 33 },
    { 0, 0 },
    { 0xfa8c44fb, 23 },
    { 0, 0 },
    { 0x42cc44fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42c4503, 98 },
    { 0x4dec4503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24c4509, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305c450f, 98 },
    { 0x252c4510, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfac4515, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fac4519, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25c451c, 99 },
    { 0xb0fc451c, 33 },
    { 0x7c4c451c, 52 },
    { 0xc1c451f, 17 },
    { 0xee7c4520, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c4526, 55 },
    { 0, 0 },
    { 0x7e7c4528, 43 },
    { 0xbd1c4529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834c452e, 40 },
    { 0, 0 },
    { 0xdedc4530, 34 },
    { 0, 0 },
    { 0x821c4532, 76 },
    { 0, 0 },
    { 0xa53c4534, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc453c, 33 },
    { 0x2eec453d, 32 },
    { 0x9c0c453c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0c4544, 46 },
    { 0, 0 },
    { 0xaddc4546, 66 },
    { 0x515c4546, 90 },
    { 0x531c4547, 83 },
    { 0, 0 },
    { 0x99dc454a, 67 },
    { 0, 0 },
    { 0x7b8c454c, 90 },
    { 0x36ac454c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbc4557, 99 },
    { 0x8f3c4557, 52 },
    { 0x15bc4557, 23 },
    { 0x171c455a, 61 },
    { 0xe12c4559, 43 },
    { 0xd60c4557, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcc4561, 34 },
    { 0, 0 },
    { 0xf5bc4563, 84 },
    { 0xb4c4564, 61 },
    { 0x84bc4564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38c4568, 86 },
    { 0x223c4569, 33 },
    { 0x491c4568, 84 },
    { 0, 0 },
    { 0x2eac456c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967c4570, 33 },
    { 0x84bc4570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891c4575, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6c457d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dc4586, 76 },
    { 0x48ec4586, 67 },
    { 0, 0 },
    { 0x3b2c4589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc458d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41c4593, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc4596, 95 },
    { 0, 0 },
    { 0x712c4598, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47c459d, 97 },
    { 0x6dbc459d, 17 },
    { 0x6c4c459f, 66 },
    { 0x7bdc459f, 67 },
    { 0xf1bc459e, 33 },
    { 0x228c45a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ec45a6, 76 },
    { 0x564c45a7, 61 },
    { 0x4abc45a8, 76 },
    { 0xbacc45a9, 65 },
    { 0xb8ac45aa, 61 },
    { 0, 0 },
    { 0x426c45ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453c45b0, 78 },
    { 0xcb3c45b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543c45b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ec45b9, 46 },
    { 0x778c45b9, 93 },
    { 0x891c45bb, 34 },
    { 0xffcc45bb, 99 },
    { 0x6b5c45b9, 64 },
    { 0x994c45be, 77 },
    { 0x91bc45bc, 98 },
    { 0x8bcc45b9, 33 },
    { 0xfffc45c1, 84 },
    { 0x48dc45b9, 90 },
    { 0x532c45c3, 23 },
    { 0x4e6c45c3, 95 },
    { 0, 0 },
    { 0x397c45c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ec45cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dc45d2, 61 },
    { 0x627c45d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0c45d6, 23 },
    { 0xbd3c45d6, 33 },
    { 0xcc1c45d6, 23 },
    { 0xc6bc45d6, 66 },
    { 0, 0 },
    { 0x557c45db, 99 },
    { 0x512c45dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ac45df, 55 },
    { 0, 0 },
    { 0x8e4c45e1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabc45e5, 98 },
    { 0, 0 },
    { 0xec5c45e7, 52 },
    { 0x2f1c45e8, 95 },
    { 0x724c45e9, 90 },
    { 0, 0 },
    { 0x107c45eb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c45ee, 55 },
    { 0xc2ec45ef, 77 },
    { 0xbe1c45ef, 90 },
    { 0x3bbc45ee, 55 },
    { 0x413c45f2, 87 },
    { 0x88c45ef, 76 },
    { 0, 0 },
    { 0xf6fc45f5, 95 },
    { 0x5d5c45f5, 55 },
    { 0x759c45f5, 61 },
    { 0x82bc45f7, 90 },
    { 0xa3dc45f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ec45fc, 33 },
    { 0, 0 },
    { 0xd5bc45fe, 61 },
    { 0xc11c45fe, 81 },
    { 0x446c4600, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932c4604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dc4607, 90 },
    { 0x18ac4608, 46 },
    { 0xb06c4609, 76 },
    { 0xb6dc4609, 61 },
    { 0, 0 },
    { 0x1e6c460c, 2 },
    { 0x469c460d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9c4610, 88 },
    { 0x4c3c4610, 90 },
    { 0x696c4611, 88 },
    { 0, 0 },
    { 0x65ec4614, 52 },
    { 0x42ec4614, 33 },
    { 0x46dc4616, 78 },
    { 0x6d8c4615, 34 },
    { 0x936c4617, 98 },
    { 0xe6dc4614, 23 },
    { 0x6e3c4615, 77 },
    { 0xf96c461b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc461e, 55 },
    { 0x86ec461f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc4625, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7c462a, 23 },
    { 0x32ac462b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc462e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2c4632, 95 },
    { 0x6dfc4633, 23 },
    { 0, 0 },
    { 0xa45c4635, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5c463f, 55 },
    { 0, 0 },
    { 0x33dc4641, 17 },
    { 0x89dc4642, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebc464b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eac464f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccc4653, 76 },
    { 0x3d7c4653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec4658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c465d, 75 },
    { 0xf4ec465d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x692c4661, 98 },
    { 0x2e9c4662, 90 },
    { 0xc12c4663, 90 },
    { 0x6b7c4664, 91 },
    { 0xa2ac4664, 90 },
    { 0x36bc4664, 85 },
    { 0x256c4666, 34 },
    { 0x3d4c4664, 100 },
    { 0xbc5c4669, 84 },
    { 0x7a2c466a, 61 },
    { 0x32ec4664, 100 },
    { 0x8e5c4664, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c466f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dc4673, 61 },
    { 0xd0c4674, 76 },
    { 0, 0 },
    { 0xa06c4676, 98 },
    { 0x9a7c4677, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeac467c, 95 },
    { 0x63ac467c, 95 },
    { 0x111c467e, 23 },
    { 0, 0 },
    { 0xfcdc4680, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6c4683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cc4686, 34 },
    { 0x363c4687, 34 },
    { 0xb0cc4688, 33 },
    { 0xce6c4686, 93 },
    { 0xb4c468a, 61 },
    { 0xf22c468b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x757c468e, 46 },
    { 0, 0 },
    { 0x39dc4690, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc4694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fc469a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec469e, 90 },
    { 0x93dc469f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504c46a4, 68 },
    { 0xd97c46a4, 43 },
    { 0xe63c46a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c46ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c46b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3c46b4, 61 },
    { 0xc17c46b5, 66 },
    { 0, 0 },
    { 0x4bdc46b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc46bd, 95 },
    { 0x629c46bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0c46c3, 87 },
    { 0x3b4c46c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec46ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccc46cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491c46d5, 98 },
    { 0, 0 },
    { 0xbd5c46d7, 33 },
    { 0x1bdc46d7, 68 },
    { 0, 0 },
    { 0x9cc46da, 34 },
    { 0x2f4c46db, 33 },
    { 0, 0 },
    { 0x17ec46dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cc46e3, 34 },
    { 0x969c46e4, 90 },
    { 0, 0 },
    { 0x53fc46e6, 43 },
    { 0x80ec46e7, 66 },
    { 0, 0 },
    { 0x23dc46e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cc46ed, 76 },
    { 0x40dc46ee, 90 },
    { 0x9cc46ee, 33 },
    { 0xc80c46f0, 76 },
    { 0xf85c46f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c46f6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cc46f9, 91 },
    { 0xc4bc46fa, 61 },
    { 0x4a2c46fa, 95 },
    { 0, 0 },
    { 0x8dac46fd, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bc4702, 95 },
    { 0xe9fc4703, 95 },
    { 0xdbdc4704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930c4708, 77 },
    { 0x1d5c4709, 33 },
    { 0, 0 },
    { 0x821c470b, 96 },
    { 0xae9c470b, 23 },
    { 0xe55c470d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cac4710, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x676c4713, 43 },
    { 0xadec4713, 90 },
    { 0x7ddc4715, 77 },
    { 0x119c4716, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ec471b, 90 },
    { 0, 0 },
    { 0xa86c471d, 93 },
    { 0x312c471e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371c4723, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36c4726, 33 },
    { 0x4b4c4726, 93 },
    { 0xe57c4728, 55 },
    { 0xe76c4729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276c4735, 90 },
    { 0xc72c4735, 17 },
    { 0, 0 },
    { 0x42fc4738, 61 },
    { 0, 0 },
    { 0x37cc473a, 34 },
    { 0xf60c473b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fc473f, 90 },
    { 0x345c473f, 33 },
    { 0x2acc4741, 61 },
    { 0, 0 },
    { 0x9f1c4743, 95 },
    { 0x84fc4744, 95 },
    { 0xe3fc4743, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabc474d, 33 },
    { 0, 0 },
    { 0xddbc474f, 100 },
    { 0x3d4c4750, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5c4754, 23 },
    { 0x4f6c4755, 55 },
    { 0, 0 },
    { 0x895c4757, 33 },
    { 0, 0 },
    { 0xa07c4759, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8c475f, 85 },
    { 0, 0 },
    { 0xc5c4761, 61 },
    { 0, 0 },
    { 0x8d0c4763, 33 },
    { 0, 0 },
    { 0xbd4c4765, 90 },
    { 0x1bac4765, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdc4769, 99 },
    { 0x508c4769, 98 },
    { 0xc4bc476b, 55 },
    { 0, 0 },
    { 0x544c476d, 33 },
    { 0xf9ec476d, 23 },
    { 0xed3c476f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87c4774, 33 },
    { 0x46dc4774, 93 },
    { 0xa4c4775, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fc4779, 83 },
    { 0, 0 },
    { 0x81ac477b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cc477e, 23 },
    { 0x33c477f, 98 },
    { 0x4b9c477f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c4784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4788, 90 },
    { 0, 0 },
    { 0xee8c478a, 98 },
    { 0x849c478b, 97 },
    { 0, 0 },
    { 0xa7cc478d, 67 },
    { 0, 0 },
    { 0xab3c478f, 90 },
    { 0xe2fc4790, 95 },
    { 0x1e3c4791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6c4798, 43 },
    { 0x6fcc4798, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676c479e, 33 },
    { 0xa94c479e, 23 },
    { 0x145c47a0, 90 },
    { 0x91ec47a1, 99 },
    { 0x6d5c47a2, 97 },
    { 0x548c47a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327c47ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x965c47b1, 76 },
    { 0x17ec47b2, 23 },
    { 0xa0fc47b3, 61 },
    { 0x71bc47b1, 95 },
    { 0x777c47b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bc47b8, 33 },
    { 0xd1ec47b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9c47bd, 95 },
    { 0, 0 },
    { 0x1fac47bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3c47c2, 55 },
    { 0x951c47c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78c47c7, 61 },
    { 0x837c47c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154c47d2, 90 },
    { 0xfc1c47d3, 34 },
    { 0, 0 },
    { 0x2bac47d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafec47de, 33 },
    { 0x95cc47df, 90 },
    { 0x45ec47df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c47e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98c47e6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9c47e9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4c47ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8c47ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc47fd, 33 },
    { 0, 0 },
    { 0x69dc47ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9c480a, 61 },
    { 0x231c480b, 33 },
    { 0x2d4c480c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9c4810, 91 },
    { 0x74ec4811, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c4819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31c481c, 61 },
    { 0, 0 },
    { 0xe57c481e, 64 },
    { 0x36dc481f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1c4823, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40c4827, 95 },
    { 0, 0 },
    { 0xd15c4829, 46 },
    { 0, 0 },
    { 0x920c482b, 43 },
    { 0xe96c482b, 34 },
    { 0, 0 },
    { 0x6dcc482e, 61 },
    { 0, 0 },
    { 0xa99c4830, 61 },
    { 0x672c4830, 23 },
    { 0xf54c4830, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adc4838, 87 },
    { 0x663c4839, 90 },
    { 0x85fc4838, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673c483f, 90 },
    { 0x398c4840, 90 },
    { 0xc82c4840, 23 },
    { 0xf44c4841, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5c4845, 90 },
    { 0, 0 },
    { 0x91dc4847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1c484c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcc4853, 33 },
    { 0x31ec4854, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ac485a, 98 },
    { 0x97ac485b, 90 },
    { 0x21bc485b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376c4865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x577c4868, 78 },
    { 0x5d7c4869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac486d, 55 },
    { 0xafdc486d, 61 },
    { 0xd30c486f, 90 },
    { 0x908c4870, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec4876, 34 },
    { 0x810c4877, 43 },
    { 0, 0 },
    { 0xc0cc4879, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c487e, 91 },
    { 0, 0 },
    { 0xeb3c4880, 76 },
    { 0xac4881, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66c4889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740c488e, 61 },
    { 0, 0 },
    { 0x18bc4890, 34 },
    { 0x5b4c4891, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09c4894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713c4898, 90 },
    { 0xac9c4898, 93 },
    { 0x7a8c489a, 61 },
    { 0xfe3c489a, 23 },
    { 0x81bc489c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0c48a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846c48a9, 43 },
    { 0x20fc48aa, 55 },
    { 0, 0 },
    { 0x1bec48ac, 75 },
    { 0xf56c48ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380c48b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c48ba, 91 },
    { 0x780c48bb, 55 },
    { 0xe16c48bb, 23 },
    { 0, 0 },
    { 0x695c48be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674c48c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x551c48c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0c48c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dc48ce, 61 },
    { 0, 0 },
    { 0x1b9c48d0, 68 },
    { 0xd61c48d1, 52 },
    { 0xc01c48d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8c48d8, 23 },
    { 0x321c48d8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8c48dc, 66 },
    { 0, 0 },
    { 0xe3c48de, 76 },
    { 0xceac48df, 33 },
    { 0x7d4c48de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8c48e4, 55 },
    { 0, 0 },
    { 0x66ac48e6, 34 },
    { 0x49cc48e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9c48ef, 90 },
    { 0x82fc48ef, 23 },
    { 0xf69c48f1, 90 },
    { 0, 0 },
    { 0xd95c48f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc48f6, 43 },
    { 0, 0 },
    { 0xacac48f8, 95 },
    { 0x62ac48f9, 91 },
    { 0xd1ac48f9, 17 },
    { 0, 0 },
    { 0xf99c48fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318c4901, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fc4904, 95 },
    { 0x3a2c4905, 33 },
    { 0x179c4904, 43 },
    { 0xc0ac4904, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c490a, 76 },
    { 0x622c490b, 33 },
    { 0, 0 },
    { 0xefc490d, 46 },
    { 0, 0 },
    { 0xabac490f, 90 },
    { 0xbfac490f, 76 },
    { 0xd62c490f, 33 },
    { 0xd91c4912, 23 },
    { 0x4ebc4912, 90 },
    { 0xe23c4914, 34 },
    { 0xf68c4914, 76 },
    { 0x48bc4916, 33 },
    { 0, 0 },
    { 0x53ac4918, 90 },
    { 0, 0 },
    { 0xefcc491a, 95 },
    { 0x6bcc491a, 99 },
    { 0xfc4c491c, 99 },
    { 0x41dc491d, 61 },
    { 0xe44c491e, 90 },
    { 0xa36c491e, 95 },
    { 0x4c4920, 90 },
    { 0x965c4920, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526c492b, 90 },
    { 0, 0 },
    { 0x6c2c492d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bc4934, 17 },
    { 0x3c2c4935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c4939, 93 },
    { 0x4bbc493a, 33 },
    { 0xabcc493b, 24 },
    { 0x55c4939, 85 },
    { 0, 0 },
    { 0x530c493e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5c4949, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25c494c, 78 },
    { 0x333c494d, 23 },
    { 0xd5fc494e, 90 },
    { 0x17c494d, 90 },
    { 0xdc7c4950, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x698c4953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaec4957, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c495a, 90 },
    { 0, 0 },
    { 0x491c495c, 90 },
    { 0, 0 },
    { 0xa09c495e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacc4961, 90 },
    { 0x80dc4962, 43 },
    { 0xfaac4963, 101 },
    { 0, 0 },
    { 0xe28c4965, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0c496a, 76 },
    { 0x487c496a, 90 },
    { 0x29c496c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cc4973, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8c4979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc497d, 100 },
    { 0xf43c497d, 95 },
    { 0, 0 },
    { 0x579c4980, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631c4995, 77 },
    { 0, 0 },
    { 0x651c4997, 90 },
    { 0x5c4997, 95 },
    { 0, 0 },
    { 0x192c499a, 52 },
    { 0xd46c499b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fc49a1, 95 },
    { 0xee1c49a2, 46 },
    { 0xaacc49a3, 99 },
    { 0, 0 },
    { 0x4dfc49a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24c49a8, 61 },
    { 0xb28c49a9, 95 },
    { 0x85ac49aa, 90 },
    { 0x7e7c49ab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc49af, 23 },
    { 0x586c49af, 33 },
    { 0, 0 },
    { 0x799c49b2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c49b8, 66 },
    { 0, 0 },
    { 0xb13c49ba, 61 },
    { 0, 0 },
    { 0xa22c49bc, 99 },
    { 0x2cec49bc, 96 },
    { 0x175c49be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x557c49c1, 95 },
    { 0, 0 },
    { 0xd79c49c3, 33 },
    { 0, 0 },
    { 0x3dfc49c5, 43 },
    { 0, 0 },
    { 0x411c49c7, 95 },
    { 0x432c49c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520c49ce, 61 },
    { 0x9bcc49ce, 33 },
    { 0x9bfc49d0, 90 },
    { 0x61fc49d1, 55 },
    { 0x331c49d0, 34 },
    { 0x281c49cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ac49d8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bc49e0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cc49e4, 69 },
    { 0x2c1c49e5, 98 },
    { 0xd5fc49e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bc49e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc49ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bc49f0, 76 },
    { 0x5a2c49f1, 55 },
    { 0x45ac49f0, 23 },
    { 0x352c49f2, 61 },
    { 0, 0 },
    { 0x82cc49f5, 99 },
    { 0xd13c49f5, 61 },
    { 0xc7c49f7, 91 },
    { 0, 0 },
    { 0x5fc49f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cc49fd, 99 },
    { 0x489c49fd, 33 },
    { 0xdddc49fd, 68 },
    { 0, 0 },
    { 0x2d6c4a01, 75 },
    { 0xf9c4a02, 98 },
    { 0x811c4a02, 33 },
    { 0x3bdc4a04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc4a0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6c4a0f, 87 },
    { 0, 0 },
    { 0x2d8c4a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1c4a17, 46 },
    { 0, 0 },
    { 0xd7c4a19, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5c4a1e, 98 },
    { 0, 0 },
    { 0x9a4c4a20, 99 },
    { 0, 0 },
    { 0xff9c4a22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c4a27, 23 },
    { 0, 0 },
    { 0xfb1c4a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x796c4a2c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ac4a31, 33 },
    { 0, 0 },
    { 0x3e2c4a33, 34 },
    { 0xa44c4a34, 61 },
    { 0x751c4a35, 87 },
    { 0x2d9c4a35, 33 },
    { 0, 0 },
    { 0x4fc4a38, 98 },
    { 0xda7c4a39, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667c4a40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6c4a43, 96 },
    { 0x62ec4a44, 98 },
    { 0, 0 },
    { 0x6f3c4a46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfc4a4a, 33 },
    { 0, 0 },
    { 0x8bec4a4c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ac4a4f, 33 },
    { 0x4dbc4a50, 76 },
    { 0xf25c4a50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec4a55, 98 },
    { 0x6f5c4a56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc4a59, 84 },
    { 0x980c4a59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dc4a5d, 98 },
    { 0, 0 },
    { 0x62ec4a5f, 33 },
    { 0xb0bc4a60, 23 },
    { 0x58ec4a60, 23 },
    { 0xe54c4a62, 55 },
    { 0xaecc4a62, 61 },
    { 0xd2cc4a64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0c4a68, 55 },
    { 0x3a6c4a68, 34 },
    { 0x82cc4a6a, 90 },
    { 0xf49c4a6b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc4a6e, 84 },
    { 0x132c4a6f, 61 },
    { 0x1f9c4a70, 34 },
    { 0xd90c4a70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925c4a75, 23 },
    { 0x142c4a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171c4a7c, 43 },
    { 0, 0 },
    { 0x177c4a7e, 87 },
    { 0xfac4a7e, 90 },
    { 0x974c4a80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c4a85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfc4a89, 61 },
    { 0xe0c4a89, 98 },
    { 0x657c4a8a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1c4a93, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8c4a96, 90 },
    { 0, 0 },
    { 0x1bac4a98, 82 },
    { 0xd75c4a99, 66 },
    { 0x195c4a99, 61 },
    { 0x919c4a9b, 52 },
    { 0, 0 },
    { 0x136c4a9d, 93 },
    { 0x391c4a9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ec4aa9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dc4aac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ec4aaf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4c4ab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90c4ab8, 66 },
    { 0, 0 },
    { 0x520c4aba, 61 },
    { 0x701c4aba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4c4abf, 46 },
    { 0, 0 },
    { 0xfd1c4ac1, 33 },
    { 0, 0 },
    { 0x3c6c4ac3, 98 },
    { 0x988c4ac4, 66 },
    { 0x626c4ac5, 34 },
    { 0, 0 },
    { 0x66cc4ac7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fc4acb, 55 },
    { 0x4c1c4acb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47c4acf, 67 },
    { 0xfb2c4ad0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546c4ad5, 98 },
    { 0xfb0c4ad6, 90 },
    { 0x914c4ad7, 33 },
    { 0xee7c4ad6, 98 },
    { 0xb4bc4ad8, 46 },
    { 0xa35c4ada, 34 },
    { 0x5dbc4adb, 90 },
    { 0, 0 },
    { 0xfd8c4add, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dc4ae1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999c4ae9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898c4af6, 90 },
    { 0x44ec4af7, 93 },
    { 0x406c4af7, 90 },
    { 0x61ec4af7, 91 },
    { 0xa4bc4af6, 90 },
    { 0xb0dc4afa, 100 },
    { 0x695c4af7, 61 },
    { 0xc83c4afc, 67 },
    { 0x59ac4afe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc4b02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82c4b0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fc4b11, 78 },
    { 0, 0 },
    { 0xf64c4b13, 61 },
    { 0xab4c4b14, 95 },
    { 0x770c4b15, 43 },
    { 0xd5cc4b16, 100 },
    { 0, 0 },
    { 0xa85c4b18, 97 },
    { 0x1c6c4b19, 76 },
    { 0, 0 },
    { 0xcf5c4b1b, 61 },
    { 0xcac4b1b, 98 },
    { 0xfe2c4b1b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ec4b20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758c4b24, 23 },
    { 0, 0 },
    { 0x20fc4b26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627c4b2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec4b34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc4b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x486c4b3d, 81 },
    { 0, 0 },
    { 0x57dc4b3f, 34 },
    { 0x356c4b3f, 52 },
    { 0x1d1c4b41, 90 },
    { 0xa2c4b42, 76 },
    { 0x681c4b43, 46 },
    { 0xdc1c4b43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3c4b47, 40 },
    { 0, 0 },
    { 0x1dbc4b49, 98 },
    { 0x51dc4b4a, 95 },
    { 0x74ac4b4a, 61 },
    { 0x457c4b4b, 90 },
    { 0xf6ac4b4b, 76 },
    { 0xad7c4b4e, 33 },
    { 0x829c4b4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dc4b52, 91 },
    { 0, 0 },
    { 0x2ebc4b54, 76 },
    { 0, 0 },
    { 0xcbac4b56, 34 },
    { 0, 0 },
    { 0xa50c4b58, 76 },
    { 0xdc6c4b58, 55 },
    { 0x98cc4b5a, 98 },
    { 0x557c4b59, 95 },
    { 0, 0 },
    { 0x572c4b5d, 33 },
    { 0xc58c4b5d, 33 },
    { 0, 0 },
    { 0x17c4b60, 56 },
    { 0x407c4b61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec4b69, 40 },
    { 0xe75c4b6a, 93 },
    { 0x168c4b6b, 87 },
    { 0xd6cc4b69, 33 },
    { 0, 0 },
    { 0xfdc4b6e, 98 },
    { 0x888c4b6e, 61 },
    { 0x2bc4b6e, 23 },
    { 0x936c4b70, 66 },
    { 0x280c4b72, 98 },
    { 0x6cec4b71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fc4b78, 90 },
    { 0xbb5c4b78, 56 },
    { 0x61bc4b7a, 98 },
    { 0x641c4b7b, 33 },
    { 0xa1cc4b7c, 77 },
    { 0xb2fc4b7d, 84 },
    { 0x26ac4b7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45c4b83, 87 },
    { 0, 0 },
    { 0xeaac4b85, 46 },
    { 0x9fdc4b86, 93 },
    { 0xafcc4b87, 86 },
    { 0xa6bc4b88, 23 },
    { 0x8b0c4b89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9c4b96, 78 },
    { 0xec6c4b97, 90 },
    { 0, 0 },
    { 0x73fc4b99, 61 },
    { 0x964c4b9a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2c4b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7c4ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382c4ba5, 85 },
    { 0x548c4ba6, 33 },
    { 0x825c4ba6, 61 },
    { 0x4fc4ba8, 98 },
    { 0x32dc4ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ac4bae, 98 },
    { 0xb1bc4baf, 52 },
    { 0xe0dc4bb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ec4bb8, 34 },
    { 0x525c4bb9, 67 },
    { 0xae3c4bb8, 80 },
    { 0, 0 },
    { 0xc28c4bbc, 55 },
    { 0x430c4bbd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dc4bc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0c4bc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ac4bc9, 76 },
    { 0xda6c4bca, 84 },
    { 0, 0 },
    { 0xb5cc4bcc, 90 },
    { 0, 0 },
    { 0x899c4bce, 90 },
    { 0xe1ec4bcf, 85 },
    { 0x7fac4bd0, 88 },
    { 0xb2ec4bd0, 95 },
    { 0xa33c4bd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9c4bd5, 96 },
    { 0x42ac4bd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7c4bd9, 95 },
    { 0xf54c4bda, 46 },
    { 0xacdc4bdb, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28c4be0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdc4be5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553c4beb, 93 },
    { 0xef9c4bec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dc4bf1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6dc4bf4, 76 },
    { 0xa8ac4bf5, 90 },
    { 0x311c4bf6, 88 },
    { 0x318c4bf7, 95 },
    { 0x21fc4bf8, 33 },
    { 0x7fac4bf8, 93 },
    { 0x7c4c4bf8, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cc4bff, 77 },
    { 0xd22c4c00, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dc4c06, 43 },
    { 0x5acc4c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec4c0d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc4c14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccac4c19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bc4c25, 90 },
    { 0x74bc4c25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc4c2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fc4c2f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cc4c35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ac4c38, 23 },
    { 0x789c4c38, 33 },
    { 0x7cc4c3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cc4c3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc4c45, 33 },
    { 0xbc7c4c45, 43 },
    { 0x4bcc4c47, 53 },
    { 0, 0 },
    { 0x703c4c49, 46 },
    { 0x96ec4c49, 46 },
    { 0x77fc4c4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebc4c4e, 95 },
    { 0, 0 },
    { 0x1efc4c50, 78 },
    { 0, 0 },
    { 0x65fc4c52, 61 },
    { 0xb7ac4c53, 66 },
    { 0, 0 },
    { 0xb50c4c55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acc4c5f, 61 },
    { 0x4b2c4c60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25c4c65, 100 },
    { 0, 0 },
    { 0x4aac4c67, 61 },
    { 0xecec4c68, 95 },
    { 0x993c4c69, 90 },
    { 0, 0 },
    { 0xd20c4c6b, 77 },
    { 0x7cbc4c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ec4c6f, 95 },
    { 0x42fc4c70, 61 },
    { 0, 0 },
    { 0x62dc4c72, 23 },
    { 0xfdfc4c72, 33 },
    { 0x85fc4c74, 90 },
    { 0xafc4c73, 91 },
    { 0, 0 },
    { 0x24ec4c77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4c4c7a, 95 },
    { 0, 0 },
    { 0x446c4c7c, 61 },
    { 0x505c4c7c, 80 },
    { 0xd23c4c7c, 23 },
    { 0, 0 },
    { 0xfac4c80, 61 },
    { 0x3eec4c81, 46 },
    { 0xe08c4c82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876c4c8d, 98 },
    { 0, 0 },
    { 0xb53c4c8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4c4c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c4c9d, 98 },
    { 0x372c4c9e, 91 },
    { 0, 0 },
    { 0xe8bc4ca0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2c4cad, 95 },
    { 0x451c4cae, 33 },
    { 0x6ebc4caf, 95 },
    { 0xfecc4cb0, 90 },
    { 0x281c4cb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc4cb4, 90 },
    { 0xbb0c4cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bc4cc1, 76 },
    { 0x143c4cc1, 98 },
    { 0xc43c4cc2, 99 },
    { 0xff3c4cc4, 33 },
    { 0, 0 },
    { 0x9e6c4cc6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6c4cc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73c4cce, 36 },
    { 0, 0 },
    { 0xf56c4cd0, 90 },
    { 0xabac4cd0, 90 },
    { 0, 0 },
    { 0xaf6c4cd3, 95 },
    { 0x52dc4cd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881c4cd9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dc4cdf, 33 },
    { 0x1a7c4ce0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c4ce5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0c4ced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80c4cf0, 33 },
    { 0xd55c4cf1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbc4cf7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c4d00, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01c4d03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c4d12, 98 },
    { 0x5ddc4d12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1c4d16, 61 },
    { 0, 0 },
    { 0x4e4c4d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1c4d1f, 33 },
    { 0xaddc4d1f, 95 },
    { 0, 0 },
    { 0x1cbc4d22, 33 },
    { 0, 0 },
    { 0xcc7c4d24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc4d2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07c4d32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7c4d36, 33 },
    { 0x600c4d37, 33 },
    { 0x627c4d38, 87 },
    { 0x95dc4d39, 90 },
    { 0, 0 },
    { 0x1b6c4d3b, 93 },
    { 0x447c4d3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebc4d49, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c4d4f, 61 },
    { 0, 0 },
    { 0x848c4d51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcc4d54, 78 },
    { 0x847c4d55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fc4d5c, 23 },
    { 0x134c4d5c, 53 },
    { 0xf2cc4d5e, 34 },
    { 0xebfc4d5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6c4d62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c4d68, 77 },
    { 0, 0 },
    { 0xaa3c4d6a, 95 },
    { 0xdcfc4d6b, 23 },
    { 0xafcc4d6c, 99 },
    { 0x4fc4d6b, 75 },
    { 0, 0 },
    { 0xbe1c4d6f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc4d74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72c4d78, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc4d80, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14c4d85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c4d8c, 61 },
    { 0xf02c4d8d, 43 },
    { 0, 0 },
    { 0x821c4d8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72c4d93, 34 },
    { 0x1abc4d93, 33 },
    { 0xa5fc4d95, 97 },
    { 0x1c7c4d93, 93 },
    { 0xc1ec4d95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac4d9c, 23 },
    { 0x589c4d9d, 33 },
    { 0, 0 },
    { 0x6aac4d9f, 33 },
    { 0x238c4da0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc4da4, 78 },
    { 0xe29c4da5, 90 },
    { 0x67ac4da4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9c4daa, 99 },
    { 0x679c4daa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cc4dae, 43 },
    { 0, 0 },
    { 0x68fc4db0, 61 },
    { 0x20ac4db0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50c4db4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28c4dba, 84 },
    { 0x1f7c4dbb, 90 },
    { 0x55fc4dbc, 78 },
    { 0xb74c4dbc, 23 },
    { 0x733c4dbe, 61 },
    { 0xb5c4dbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27c4dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ac4dc7, 100 },
    { 0xe81c4dc8, 33 },
    { 0x9aac4dc8, 93 },
    { 0x26ec4dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fc4dce, 65 },
    { 0x474c4dcf, 61 },
    { 0, 0 },
    { 0xd8fc4dd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c4dd4, 9 },
    { 0, 0 },
    { 0x20cc4dd6, 33 },
    { 0x1e3c4dd7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc4ddb, 33 },
    { 0, 0 },
    { 0xb03c4ddd, 78 },
    { 0x54ac4dde, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcc4de2, 67 },
    { 0xa1bc4de3, 95 },
    { 0x4e3c4de2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c4de7, 99 },
    { 0x93cc4de8, 100 },
    { 0x71bc4de8, 90 },
    { 0xc6ec4de8, 23 },
    { 0, 0 },
    { 0xbd2c4dec, 17 },
    { 0x6a3c4ded, 43 },
    { 0, 0 },
    { 0x67ec4def, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fc4df7, 43 },
    { 0x1d1c4df8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eac4dfb, 61 },
    { 0x366c4dfc, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec4e01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198c4e05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ac4e0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dc4e12, 99 },
    { 0xc25c4e13, 38 },
    { 0, 0 },
    { 0x273c4e15, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68c4e1c, 55 },
    { 0xedac4e1d, 98 },
    { 0x6c3c4e1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dc4e25, 55 },
    { 0x6a8c4e26, 98 },
    { 0xfbcc4e27, 55 },
    { 0xbbec4e28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63c4e2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423c4e34, 76 },
    { 0xb95c4e35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722c4e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x758c4e3d, 76 },
    { 0xa72c4e3e, 67 },
    { 0xad6c4e3e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac4e44, 91 },
    { 0x929c4e45, 66 },
    { 0xad9c4e46, 23 },
    { 0, 0 },
    { 0x576c4e48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bc4e4b, 90 },
    { 0xf29c4e4b, 66 },
    { 0xcdac4e4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c4e51, 56 },
    { 0, 0 },
    { 0x1b6c4e53, 34 },
    { 0, 0 },
    { 0xea2c4e55, 90 },
    { 0, 0 },
    { 0x1c6c4e57, 55 },
    { 0x4f6c4e57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c4e5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1c4e61, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dc4e64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc4e67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420c4e6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc4e6f, 87 },
    { 0xf2cc4e6f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75c4e73, 23 },
    { 0x109c4e74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53c4e77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fc4e80, 84 },
    { 0xac4c4e81, 52 },
    { 0, 0 },
    { 0x6c2c4e83, 98 },
    { 0xeb4c4e84, 95 },
    { 0, 0 },
    { 0x357c4e86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887c4e8a, 90 },
    { 0xba3c4e8a, 33 },
    { 0x54fc4e8c, 53 },
    { 0x3f2c4e8a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x145c4e90, 46 },
    { 0, 0 },
    { 0xf26c4e92, 85 },
    { 0, 0 },
    { 0x80ec4e94, 90 },
    { 0xe56c4e95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x242c4e98, 33 },
    { 0x455c4e99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x515c4e9c, 34 },
    { 0, 0 },
    { 0x29fc4e9e, 90 },
    { 0, 0 },
    { 0x4bdc4ea0, 76 },
    { 0x452c4ea0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5c4ea7, 94 },
    { 0, 0 },
    { 0x798c4ea9, 84 },
    { 0xe28c4ea9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ac4ead, 99 },
    { 0xe41c4ead, 36 },
    { 0xc43c4eaf, 85 },
    { 0, 0 },
    { 0x343c4eb1, 99 },
    { 0xe97c4eb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200c4ebb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddc4ebe, 55 },
    { 0x896c4ebf, 90 },
    { 0, 0 },
    { 0xde2c4ec1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c4ec7, 98 },
    { 0, 0 },
    { 0xee0c4ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c4ecf, 76 },
    { 0x3d8c4ed0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57c4ed5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc4ed8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41c4ee2, 61 },
    { 0x3b8c4ee3, 76 },
    { 0xe04c4ee4, 98 },
    { 0xb1bc4ee5, 61 },
    { 0xe9cc4ee6, 17 },
    { 0xd73c4ee7, 90 },
    { 0x64ec4ee7, 61 },
    { 0, 0 },
    { 0x6e6c4eea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbac4eee, 95 },
    { 0xcaac4eef, 68 },
    { 0xe7ec4ef0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdc4ef6, 36 },
    { 0x459c4ef7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9c4efa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c4f03, 68 },
    { 0xb58c4f03, 98 },
    { 0xc36c4f05, 23 },
    { 0xe5ec4f06, 33 },
    { 0x77dc4f06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125c4f0f, 90 },
    { 0, 0 },
    { 0x3abc4f11, 61 },
    { 0xbf5c4f12, 67 },
    { 0, 0 },
    { 0x9a9c4f14, 85 },
    { 0, 0 },
    { 0xe70c4f16, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabec4f19, 76 },
    { 0, 0 },
    { 0x848c4f1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c4f1e, 61 },
    { 0xba8c4f1e, 98 },
    { 0, 0 },
    { 0x867c4f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25c4f29, 55 },
    { 0x356c4f29, 80 },
    { 0x635c4f2b, 61 },
    { 0x97dc4f2b, 90 },
    { 0xb81c4f2c, 90 },
    { 0xe1ac4f2c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591c4f32, 46 },
    { 0x968c4f33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09c4f36, 80 },
    { 0, 0 },
    { 0x4fc4f38, 43 },
    { 0xd9cc4f39, 88 },
    { 0, 0 },
    { 0xb24c4f3b, 23 },
    { 0, 0 },
    { 0x8dac4f3d, 61 },
    { 0, 0 },
    { 0x78ac4f3f, 55 },
    { 0xdf7c4f3f, 95 },
    { 0, 0 },
    { 0xd56c4f42, 33 },
    { 0x626c4f43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeec4f47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76c4f4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x494c4f4f, 70 },
    { 0x8bbc4f50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fc4f54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ec4f58, 61 },
    { 0x878c4f59, 23 },
    { 0x6bcc4f59, 90 },
    { 0x948c4f5b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cc4f5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dc4f63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92c4f66, 84 },
    { 0x44c4f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc4f6a, 95 },
    { 0x5f5c4f6b, 33 },
    { 0x880c4f6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c4f70, 95 },
    { 0x317c4f70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dc4f75, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x133c4f78, 19 },
    { 0, 0 },
    { 0xee4c4f7a, 33 },
    { 0x106c4f7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c4f80, 90 },
    { 0xd83c4f81, 95 },
    { 0x31ac4f82, 34 },
    { 0, 0 },
    { 0xd6cc4f84, 43 },
    { 0x857c4f85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10c4f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752c4f8d, 77 },
    { 0, 0 },
    { 0x3cc4f8f, 55 },
    { 0x425c4f8f, 80 },
    { 0x715c4f91, 33 },
    { 0x6aec4f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0c4f95, 90 },
    { 0, 0 },
    { 0x63ec4f97, 90 },
    { 0x4bec4f97, 55 },
    { 0xc46c4f99, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cc4f9e, 95 },
    { 0x3d2c4f9f, 75 },
    { 0xc34c4fa0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc4fa3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85c4fa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51c4fab, 46 },
    { 0xde6c4fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc4fb6, 78 },
    { 0xa8bc4fb7, 45 },
    { 0x808c4fb7, 68 },
    { 0xb73c4fb9, 34 },
    { 0x54dc4fb6, 33 },
    { 0xd96c4fb7, 30 },
    { 0x315c4fbc, 46 },
    { 0x82dc4fbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c4fc1, 61 },
    { 0, 0 },
    { 0x5fac4fc3, 96 },
    { 0x99dc4fc4, 91 },
    { 0, 0 },
    { 0x8c1c4fc6, 78 },
    { 0x91cc4fc6, 78 },
    { 0x66bc4fc8, 23 },
    { 0x435c4fc9, 91 },
    { 0xbd5c4fca, 43 },
    { 0, 0 },
    { 0x5aac4fcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cc4fd0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195c4fda, 93 },
    { 0x894c4fdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c4fde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8c4fe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ac4fe6, 90 },
    { 0xa8c4fe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bc4fee, 96 },
    { 0x17dc4fef, 46 },
    { 0xb23c4ff0, 33 },
    { 0x980c4ff1, 98 },
    { 0, 0 },
    { 0x6dcc4ff3, 90 },
    { 0x108c4ff4, 90 },
    { 0, 0 },
    { 0x243c4ff6, 83 },
    { 0x1d7c4ff6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x537c4ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594c4ffe, 76 },
    { 0x464c4fff, 98 },
    { 0, 0 },
    { 0xd47c5001, 90 },
    { 0, 0 },
    { 0xddac5003, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c500c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57c5010, 76 },
    { 0xb09c5011, 55 },
    { 0, 0 },
    { 0x7bfc5013, 90 },
    { 0x20ac5014, 78 },
    { 0x291c5015, 33 },
    { 0xe64c5015, 90 },
    { 0x5c0c5016, 90 },
    { 0, 0 },
    { 0xe33c5019, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272c501e, 33 },
    { 0, 0 },
    { 0x9ec5020, 55 },
    { 0, 0 },
    { 0xe0ec5022, 68 },
    { 0x6cdc5023, 95 },
    { 0x893c5023, 61 },
    { 0, 0 },
    { 0x983c5026, 33 },
    { 0x26cc5027, 43 },
    { 0xb8dc5028, 66 },
    { 0, 0 },
    { 0xb9bc502a, 90 },
    { 0x813c502b, 91 },
    { 0x5bbc502c, 23 },
    { 0x6c8c502d, 84 },
    { 0x24dc502d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2c5031, 68 },
    { 0x6ec5032, 98 },
    { 0x2d0c5032, 46 },
    { 0xeccc5034, 90 },
    { 0x89dc5035, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7c5038, 34 },
    { 0x79c5038, 61 },
    { 0xa32c5038, 75 },
    { 0x2b2c5038, 98 },
    { 0x1e2c5038, 23 },
    { 0xab1c503a, 55 },
    { 0, 0 },
    { 0x8d6c503f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ac5043, 84 },
    { 0x4f0c5044, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7c5047, 91 },
    { 0x8d1c5048, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627c5051, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebec5056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec505e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adc5061, 48 },
    { 0, 0 },
    { 0x641c5063, 33 },
    { 0, 0 },
    { 0xcb8c5065, 33 },
    { 0x99fc5065, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1c506f, 43 },
    { 0x245c506f, 61 },
    { 0x1aec5071, 90 },
    { 0x24ac5072, 90 },
    { 0x912c5071, 61 },
    { 0xb18c5074, 95 },
    { 0xfd9c5075, 95 },
    { 0x192c506f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94c507e, 56 },
    { 0xfc507f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc5082, 61 },
    { 0xf46c5082, 23 },
    { 0x611c5084, 43 },
    { 0xfbec5084, 77 },
    { 0xa57c5086, 90 },
    { 0, 0 },
    { 0x358c5088, 33 },
    { 0xd11c5088, 61 },
    { 0xac1c508a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5dc508f, 46 },
    { 0x465c5090, 90 },
    { 0x495c5091, 90 },
    { 0xcaec5091, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2c5099, 68 },
    { 0x9d7c5099, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c50a2, 93 },
    { 0x105c50a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bc50a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6c50a9, 17 },
    { 0x9cfc50aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871c50ae, 76 },
    { 0xf57c50af, 90 },
    { 0x9fc50b0, 61 },
    { 0xd9c50ae, 43 },
    { 0x35ac50ae, 95 },
    { 0x442c50b2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c50b9, 34 },
    { 0, 0 },
    { 0x533c50bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c50be, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4c50c1, 76 },
    { 0xfd9c50c1, 100 },
    { 0xf5fc50c2, 87 },
    { 0x443c50c2, 90 },
    { 0xe72c50c3, 33 },
    { 0x361c50c6, 43 },
    { 0x901c50c7, 95 },
    { 0, 0 },
    { 0x943c50c9, 87 },
    { 0x1b0c50c9, 65 },
    { 0, 0 },
    { 0xb1c50cc, 61 },
    { 0x2adc50cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3c50d6, 98 },
    { 0, 0 },
    { 0xf4dc50d8, 61 },
    { 0xfbac50d9, 34 },
    { 0x275c50da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fec50dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8c50e1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ac50e4, 34 },
    { 0x21cc50e5, 66 },
    { 0x79cc50e5, 90 },
    { 0, 0 },
    { 0x651c50e8, 67 },
    { 0, 0 },
    { 0xf74c50ea, 76 },
    { 0x338c50ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684c50f5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c50f8, 79 },
    { 0x25ac50f9, 75 },
    { 0x702c50fa, 76 },
    { 0xe0ec50fb, 77 },
    { 0x4edc50fa, 23 },
    { 0x876c50f8, 84 },
    { 0, 0 },
    { 0xbafc50ff, 95 },
    { 0, 0 },
    { 0x168c5101, 76 },
    { 0x762c5102, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbc5106, 90 },
    { 0xe0dc5107, 95 },
    { 0x24ac5108, 43 },
    { 0x458c5108, 90 },
    { 0x712c510a, 90 },
    { 0x6cdc5108, 90 },
    { 0x544c5109, 46 },
    { 0x508c5106, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x120c5110, 61 },
    { 0x7f4c5111, 55 },
    { 0xe91c5112, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adc5115, 99 },
    { 0x337c5116, 90 },
    { 0xa5fc5115, 98 },
    { 0x978c5118, 90 },
    { 0, 0 },
    { 0x59ec511a, 78 },
    { 0x597c511b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48c5122, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cc5128, 23 },
    { 0x4f1c5129, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9c512c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29c5131, 93 },
    { 0, 0 },
    { 0xa47c5133, 61 },
    { 0xf58c5134, 98 },
    { 0, 0 },
    { 0x904c5136, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620c513b, 95 },
    { 0xb5fc513b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48c5144, 61 },
    { 0x301c5144, 33 },
    { 0x241c5146, 98 },
    { 0xf0ac5144, 66 },
    { 0, 0 },
    { 0xe7ec5149, 52 },
    { 0xb47c514a, 68 },
    { 0x288c514a, 98 },
    { 0x4adc514c, 88 },
    { 0, 0 },
    { 0xee8c514e, 90 },
    { 0x363c514e, 43 },
    { 0xcc514e, 23 },
    { 0, 0 },
    { 0xf72c5152, 68 },
    { 0xe31c5153, 91 },
    { 0xc9ac5153, 99 },
    { 0x525c5152, 34 },
    { 0x84fc5156, 84 },
    { 0xf06c5154, 55 },
    { 0xde7c5158, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5c5161, 95 },
    { 0xcafc5161, 90 },
    { 0x556c5163, 91 },
    { 0xcd5c5164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcec5168, 34 },
    { 0x2ec5168, 55 },
    { 0x8e7c5168, 52 },
    { 0xe06c516b, 46 },
    { 0x218c516c, 67 },
    { 0xd92c516a, 23 },
    { 0, 0 },
    { 0x5e9c516f, 98 },
    { 0, 0 },
    { 0xd75c5171, 76 },
    { 0, 0 },
    { 0xccac5173, 34 },
    { 0xbb9c5174, 98 },
    { 0x930c5174, 90 },
    { 0x911c5175, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48c517b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6c518b, 23 },
    { 0xb8bc518c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dac518f, 90 },
    { 0x70ac5190, 33 },
    { 0xb0fc5190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558c5195, 96 },
    { 0x349c5196, 46 },
    { 0xc3bc5195, 93 },
    { 0, 0 },
    { 0xa10c5199, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61c519d, 61 },
    { 0xb42c519e, 95 },
    { 0x284c519e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798c51a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c51ad, 23 },
    { 0, 0 },
    { 0x25c51af, 52 },
    { 0x62dc51af, 90 },
    { 0x185c51b1, 95 },
    { 0x59c51b1, 61 },
    { 0, 0 },
    { 0x540c51b4, 23 },
    { 0xdcac51b5, 76 },
    { 0x220c51b6, 23 },
    { 0x9d3c51b6, 99 },
    { 0x51cc51b8, 81 },
    { 0x87c51b7, 23 },
    { 0xc2ec51ba, 43 },
    { 0, 0 },
    { 0x5c51bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c51c1, 90 },
    { 0x42dc51c2, 59 },
    { 0xeecc51c3, 55 },
    { 0, 0 },
    { 0xc8cc51c5, 61 },
    { 0x13fc51c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c51ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecac51d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafac51d5, 87 },
    { 0x6b4c51d6, 23 },
    { 0xadcc51d5, 90 },
    { 0x924c51d8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x111c51db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3c51e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65c51e9, 67 },
    { 0, 0 },
    { 0xee6c51eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172c51f6, 55 },
    { 0x2e4c51f6, 46 },
    { 0x7ebc51f8, 76 },
    { 0xaa4c51f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc51ff, 61 },
    { 0xaf9c5200, 95 },
    { 0, 0 },
    { 0x68cc5202, 76 },
    { 0x22c5202, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c520a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78c520d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8c5213, 52 },
    { 0xc76c5214, 33 },
    { 0x2a8c5214, 61 },
    { 0x806c5214, 95 },
    { 0x8bec5213, 61 },
    { 0x944c5218, 95 },
    { 0x92c5217, 95 },
    { 0xa2bc521a, 90 },
    { 0x890c521b, 66 },
    { 0xac1c521b, 61 },
    { 0, 0 },
    { 0x849c521e, 90 },
    { 0xa47c521f, 55 },
    { 0xb6dc5220, 76 },
    { 0, 0 },
    { 0x9ecc5222, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dc5225, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc522a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fac5231, 98 },
    { 0xe3c5232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c5235, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bc523c, 91 },
    { 0, 0 },
    { 0xfe2c523e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3c5241, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fc5245, 76 },
    { 0, 0 },
    { 0xedfc5247, 55 },
    { 0x33ac5248, 33 },
    { 0, 0 },
    { 0xb3fc524a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fc524e, 95 },
    { 0x16fc524f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cc5252, 23 },
    { 0, 0 },
    { 0x5d7c5254, 99 },
    { 0xebbc5255, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513c5260, 90 },
    { 0, 0 },
    { 0x14dc5262, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cc5265, 23 },
    { 0xf8c5266, 23 },
    { 0, 0 },
    { 0x58dc5268, 33 },
    { 0x579c5269, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bc526d, 93 },
    { 0x683c526d, 33 },
    { 0x650c526d, 93 },
    { 0, 0 },
    { 0x37c5271, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713c5277, 78 },
    { 0xd8ec5278, 66 },
    { 0xc48c5279, 61 },
    { 0x1e7c527a, 55 },
    { 0x14cc527b, 91 },
    { 0xa23c527c, 61 },
    { 0xf99c527c, 90 },
    { 0x5d6c5278, 98 },
    { 0, 0 },
    { 0xc11c5280, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6c5285, 92 },
    { 0x7f9c5285, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26c528d, 67 },
    { 0xfa4c528e, 61 },
    { 0x2e5c528e, 90 },
    { 0xfaac528f, 72 },
    { 0x4b5c5291, 23 },
    { 0, 0 },
    { 0xe5dc5293, 61 },
    { 0x7cc5294, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644c5299, 33 },
    { 0x6e6c529a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ac529e, 90 },
    { 0x43c529f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c52a5, 61 },
    { 0x87c52a5, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c52a9, 33 },
    { 0, 0 },
    { 0xef5c52ab, 85 },
    { 0x7abc52ac, 95 },
    { 0, 0 },
    { 0x1f1c52ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bc52b2, 76 },
    { 0x485c52b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc52b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8c52bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cc52c0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9c52c3, 17 },
    { 0x206c52c3, 68 },
    { 0, 0 },
    { 0x72dc52c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc52cd, 95 },
    { 0, 0 },
    { 0xdf6c52cf, 61 },
    { 0xe86c52cf, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x643c52d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec52db, 90 },
    { 0xe51c52db, 33 },
    { 0xbe4c52dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bc52e1, 55 },
    { 0x500c52e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c52e6, 98 },
    { 0xd01c52e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acc52eb, 53 },
    { 0, 0 },
    { 0x95fc52ed, 90 },
    { 0xc48c52ee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5c52f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x413c52f5, 33 },
    { 0x28ec52f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cc52fa, 90 },
    { 0x301c52fb, 61 },
    { 0x2bec52fc, 49 },
    { 0x575c52fd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbc5303, 97 },
    { 0x1a6c5304, 33 },
    { 0x105c5305, 90 },
    { 0x93c5306, 81 },
    { 0x8c5307, 98 },
    { 0x19cc5308, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7c530c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ec530f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec5312, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8c5316, 95 },
    { 0xad0c5317, 81 },
    { 0, 0 },
    { 0xf46c5319, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x461c531c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4c5328, 43 },
    { 0x65bc5329, 90 },
    { 0xb56c532a, 95 },
    { 0, 0 },
    { 0x6e0c532c, 66 },
    { 0, 0 },
    { 0xc61c532e, 33 },
    { 0, 0 },
    { 0x3f3c5330, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c5333, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bc533b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20c533e, 90 },
    { 0, 0 },
    { 0x892c5340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc5343, 33 },
    { 0x413c5343, 95 },
    { 0x372c5345, 33 },
    { 0, 0 },
    { 0x23dc5347, 96 },
    { 0, 0 },
    { 0xb1fc5349, 76 },
    { 0x583c5349, 23 },
    { 0, 0 },
    { 0x582c534c, 90 },
    { 0x1a9c534c, 93 },
    { 0x6a0c534c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9c5353, 100 },
    { 0, 0 },
    { 0x43dc5355, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ac535b, 33 },
    { 0x177c535c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x591c535f, 61 },
    { 0xb88c535f, 96 },
    { 0x9c6c535f, 90 },
    { 0xa58c5360, 35 },
    { 0xac5c5363, 39 },
    { 0xad4c5364, 43 },
    { 0xd61c5365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac5369, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0c536c, 55 },
    { 0x6d3c536d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7c5373, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9c537e, 30 },
    { 0, 0 },
    { 0x568c5380, 65 },
    { 0xbe7c5380, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfc5387, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ec538a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3c538e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49c5391, 98 },
    { 0x663c5392, 33 },
    { 0, 0 },
    { 0x9aec5394, 33 },
    { 0x6e1c5395, 99 },
    { 0x753c5396, 61 },
    { 0xcc8c5397, 33 },
    { 0x7f2c5397, 66 },
    { 0xc1cc5399, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc539c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c53a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c53a3, 61 },
    { 0x5e2c53a4, 34 },
    { 0x9f0c53a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ac53a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bc53ac, 90 },
    { 0, 0 },
    { 0xf5cc53ae, 43 },
    { 0x481c53af, 90 },
    { 0, 0 },
    { 0x11fc53b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5c53b5, 43 },
    { 0, 0 },
    { 0x781c53b7, 81 },
    { 0x640c53b8, 85 },
    { 0x113c53b9, 98 },
    { 0, 0 },
    { 0xe2cc53bb, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1c53be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c53c3, 43 },
    { 0x962c53c3, 87 },
    { 0xfb2c53c3, 90 },
    { 0x2bc53c6, 33 },
    { 0x9a8c53c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bc53ca, 95 },
    { 0x707c53ca, 97 },
    { 0, 0 },
    { 0xbc4c53cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325c53d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc53d5, 85 },
    { 0xa1cc53d5, 61 },
    { 0xa1ec53d5, 98 },
    { 0xd59c53d8, 76 },
    { 0x13fc53d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x341c53dc, 61 },
    { 0x809c53dd, 95 },
    { 0x73c53de, 95 },
    { 0x4a7c53dd, 95 },
    { 0xf08c53e0, 66 },
    { 0xd58c53e1, 77 },
    { 0x6a4c53de, 38 },
    { 0, 0 },
    { 0x7f4c53e4, 78 },
    { 0, 0 },
    { 0xffec53e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2c53ea, 95 },
    { 0, 0 },
    { 0xce5c53ec, 96 },
    { 0xe19c53ed, 74 },
    { 0x8d2c53ee, 61 },
    { 0x48fc53ef, 99 },
    { 0, 0 },
    { 0x4f6c53f1, 88 },
    { 0xeddc53f2, 91 },
    { 0, 0 },
    { 0x9c5c53f4, 90 },
    { 0, 0 },
    { 0xbb1c53f6, 90 },
    { 0xb99c53f7, 33 },
    { 0x722c53f6, 66 },
    { 0x414c53f9, 90 },
    { 0x277c53f7, 55 },
    { 0x808c53f6, 100 },
    { 0x217c53fc, 100 },
    { 0x2fec53f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x536c5400, 94 },
    { 0x59c5401, 46 },
    { 0xc57c5400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65c5407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c5411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dc5414, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x208c5417, 33 },
    { 0x942c5418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340c541c, 37 },
    { 0x3bcc541d, 90 },
    { 0x681c541d, 61 },
    { 0, 0 },
    { 0x9f1c5420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2c5428, 61 },
    { 0xebfc5428, 78 },
    { 0x93fc5428, 90 },
    { 0x5bcc542b, 100 },
    { 0x7c8c542b, 33 },
    { 0xd1cc5428, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec5430, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c5435, 95 },
    { 0x9a1c5435, 34 },
    { 0x3f5c5437, 90 },
    { 0x5aac5437, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0c543e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cc5445, 95 },
    { 0x99cc5445, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146c544a, 83 },
    { 0, 0 },
    { 0xa73c544c, 33 },
    { 0x8cac544c, 90 },
    { 0xd05c544c, 84 },
    { 0, 0 },
    { 0x3e5c5450, 99 },
    { 0x47ac5451, 88 },
    { 0xa31c5452, 95 },
    { 0x907c5453, 34 },
    { 0, 0 },
    { 0x8e2c5455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bc5458, 69 },
    { 0x84dc5459, 91 },
    { 0x4b3c545a, 46 },
    { 0xc2cc5458, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c545e, 87 },
    { 0xc2bc545f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6c5465, 66 },
    { 0xe00c5465, 33 },
    { 0x322c5466, 33 },
    { 0x204c5467, 55 },
    { 0, 0 },
    { 0x53ac546a, 66 },
    { 0xb73c546a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64c546e, 95 },
    { 0xb23c546f, 33 },
    { 0x48fc5470, 65 },
    { 0x412c5471, 66 },
    { 0x249c546e, 61 },
    { 0xcd3c5471, 46 },
    { 0xc6cc5474, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08c5479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfc547f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264c5483, 55 },
    { 0, 0 },
    { 0x4f9c5485, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x208c5488, 85 },
    { 0x5d4c5489, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4c548e, 61 },
    { 0x9f7c548f, 95 },
    { 0, 0 },
    { 0x3d7c5491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dc549a, 98 },
    { 0xdffc549a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1c549e, 33 },
    { 0, 0 },
    { 0xa53c54a0, 40 },
    { 0, 0 },
    { 0xea4c54a2, 93 },
    { 0x869c54a3, 33 },
    { 0x510c54a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c54a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fc54ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cc54b1, 61 },
    { 0xe22c54b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11c54bc, 61 },
    { 0x595c54bd, 23 },
    { 0, 0 },
    { 0x3a7c54bf, 90 },
    { 0x858c54c0, 95 },
    { 0, 0 },
    { 0xe06c54c2, 33 },
    { 0x65ec54c2, 61 },
    { 0, 0 },
    { 0xfe8c54c5, 90 },
    { 0x990c54c5, 90 },
    { 0, 0 },
    { 0x293c54c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebc54cb, 43 },
    { 0, 0 },
    { 0x8dcc54cd, 61 },
    { 0, 0 },
    { 0x143c54cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc54d2, 64 },
    { 0, 0 },
    { 0x105c54d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3c54db, 23 },
    { 0x328c54dc, 66 },
    { 0, 0 },
    { 0x716c54de, 31 },
    { 0xfafc54df, 23 },
    { 0x6bbc54e0, 95 },
    { 0x634c54de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c54e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bc54e9, 98 },
    { 0, 0 },
    { 0x6aac54eb, 88 },
    { 0, 0 },
    { 0x484c54ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775c54f2, 67 },
    { 0x7c1c54f3, 23 },
    { 0x472c54f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9c54f7, 76 },
    { 0x9eec54f8, 95 },
    { 0, 0 },
    { 0xc1ec54fa, 90 },
    { 0, 0 },
    { 0xe8c54fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fc54ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c5505, 93 },
    { 0x97fc5505, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4c550f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfc5516, 61 },
    { 0x4f9c5517, 61 },
    { 0, 0 },
    { 0x1c2c5519, 61 },
    { 0, 0 },
    { 0xa2cc551b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6c551e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25c5523, 67 },
    { 0xa2ac5524, 33 },
    { 0, 0 },
    { 0xb22c5526, 23 },
    { 0, 0 },
    { 0x75cc5528, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ec552e, 64 },
    { 0, 0 },
    { 0xd88c5530, 33 },
    { 0x17cc5531, 98 },
    { 0x462c5532, 61 },
    { 0x1acc5533, 34 },
    { 0x6bc5534, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4c5538, 33 },
    { 0x4eec5539, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2c553c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358c5541, 55 },
    { 0, 0 },
    { 0x8abc5543, 43 },
    { 0, 0 },
    { 0x4e0c5545, 98 },
    { 0xbe6c5546, 90 },
    { 0xb09c5546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342c554c, 90 },
    { 0, 0 },
    { 0x968c554e, 33 },
    { 0xf29c554f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c5558, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x975c555b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915c5561, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326c5565, 34 },
    { 0xb91c5566, 84 },
    { 0xcf6c5567, 52 },
    { 0x2a1c5567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2c556c, 33 },
    { 0, 0 },
    { 0x4dcc556e, 61 },
    { 0x652c556f, 98 },
    { 0x1c3c556e, 98 },
    { 0x799c556f, 99 },
    { 0, 0 },
    { 0x4ecc5573, 98 },
    { 0xc27c5574, 53 },
    { 0, 0 },
    { 0x64ec5576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422c557a, 98 },
    { 0, 0 },
    { 0x79ac557c, 90 },
    { 0xf32c557c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ac5582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994c5588, 100 },
    { 0, 0 },
    { 0x58ec558a, 91 },
    { 0x329c558a, 66 },
    { 0x713c558a, 90 },
    { 0x327c558d, 33 },
    { 0x6d1c558c, 98 },
    { 0, 0 },
    { 0x367c5590, 61 },
    { 0x990c5591, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cc5594, 33 },
    { 0xb3dc5595, 95 },
    { 0x47ec5596, 34 },
    { 0xbd9c5597, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bc559a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63c55ab, 34 },
    { 0, 0 },
    { 0x394c55ad, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x190c55b0, 90 },
    { 0xe21c55b0, 90 },
    { 0xcf4c55b2, 33 },
    { 0x3b9c55b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c55b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x945c55bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892c55c1, 90 },
    { 0xdaec55c2, 61 },
    { 0, 0 },
    { 0x7a1c55c4, 95 },
    { 0xdeec55c5, 55 },
    { 0x384c55c5, 95 },
    { 0xe8fc55c6, 61 },
    { 0x918c55c7, 95 },
    { 0x7bec55c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ec55cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bc55d3, 63 },
    { 0, 0 },
    { 0xff0c55d5, 90 },
    { 0, 0 },
    { 0x171c55d7, 33 },
    { 0xe32c55d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c55dc, 100 },
    { 0x850c55dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c55e3, 33 },
    { 0xe9cc55e3, 90 },
    { 0xafec55e5, 90 },
    { 0x166c55e5, 33 },
    { 0x761c55e7, 95 },
    { 0x127c55e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fc55ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699c55f0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fc55f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c55fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90c55fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x673c5600, 98 },
    { 0x656c5600, 61 },
    { 0xa2c5601, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ec560b, 43 },
    { 0x2c6c560b, 95 },
    { 0xb41c560d, 95 },
    { 0x8b6c560b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912c5616, 80 },
    { 0, 0 },
    { 0xe05c5618, 61 },
    { 0xa3cc5619, 46 },
    { 0xebcc5618, 98 },
    { 0x2a4c561b, 85 },
    { 0x412c5619, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dc5621, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c5629, 17 },
    { 0, 0 },
    { 0x322c562b, 77 },
    { 0x9f0c562b, 61 },
    { 0x29c562d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c5632, 67 },
    { 0, 0 },
    { 0x493c5634, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101c563e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161c5643, 61 },
    { 0x7c3c5644, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310c564a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x16c564d, 95 },
    { 0, 0 },
    { 0x883c564f, 61 },
    { 0x2f3c5650, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372c5654, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7c5658, 76 },
    { 0, 0 },
    { 0x3f0c565a, 88 },
    { 0x2e3c565b, 67 },
    { 0x8f0c565b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06c5660, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25c5666, 33 },
    { 0xa8cc5667, 93 },
    { 0x807c5668, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58c5670, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243c5675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x204c5678, 55 },
    { 0, 0 },
    { 0x521c567a, 90 },
    { 0x8d0c567a, 81 },
    { 0xbadc567c, 61 },
    { 0xda9c567d, 91 },
    { 0x4c4c567d, 95 },
    { 0x49dc567f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c5683, 55 },
    { 0x41cc5684, 61 },
    { 0x429c5684, 61 },
    { 0, 0 },
    { 0x7dac5687, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ac568b, 90 },
    { 0x669c568c, 52 },
    { 0x4e2c568d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dc5697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c569b, 90 },
    { 0x383c569b, 55 },
    { 0x251c569b, 98 },
    { 0x848c569e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1c56a4, 61 },
    { 0, 0 },
    { 0x899c56a6, 76 },
    { 0xac9c56a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813c56ad, 61 },
    { 0, 0 },
    { 0x830c56af, 33 },
    { 0xe46c56af, 90 },
    { 0xe6ec56b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245c56b9, 33 },
    { 0, 0 },
    { 0xc76c56bb, 33 },
    { 0x970c56bb, 90 },
    { 0xe68c56bd, 98 },
    { 0xe41c56bb, 98 },
    { 0x294c56bf, 61 },
    { 0x9a1c56bf, 33 },
    { 0x917c56c1, 98 },
    { 0xbc0c56c2, 23 },
    { 0, 0 },
    { 0xa41c56c4, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2c56c8, 23 },
    { 0x9f8c56c9, 76 },
    { 0xafc56c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dc56cd, 33 },
    { 0x1e9c56cd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacc56d3, 76 },
    { 0x25bc56d4, 90 },
    { 0xcc6c56d5, 90 },
    { 0x1cac56d6, 98 },
    { 0, 0 },
    { 0xd3cc56d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74c56dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91c56e0, 23 },
    { 0, 0 },
    { 0x958c56e2, 93 },
    { 0xd7dc56e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c56e7, 33 },
    { 0x88dc56e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0c56eb, 23 },
    { 0xd49c56ec, 61 },
    { 0xab1c56ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ac56f2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17c56f5, 98 },
    { 0xf75c56f6, 76 },
    { 0x1d2c56f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadc56ff, 52 },
    { 0x318c5700, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1c5703, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cc5707, 90 },
    { 0x9b9c5708, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83c570c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22c570f, 95 },
    { 0x86cc570f, 33 },
    { 0xeecc5711, 43 },
    { 0x768c5711, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c5715, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc5718, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac571d, 66 },
    { 0, 0 },
    { 0x402c571f, 95 },
    { 0xf9fc5720, 61 },
    { 0xdc2c5721, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3c572a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8c572e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ac5731, 61 },
    { 0, 0 },
    { 0xe0bc5733, 53 },
    { 0x8cec5734, 33 },
    { 0xbb3c5734, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7c5738, 34 },
    { 0xd5dc5739, 23 },
    { 0x651c5739, 61 },
    { 0x756c573a, 33 },
    { 0x7d6c573c, 91 },
    { 0x9eac573c, 33 },
    { 0xf43c573e, 91 },
    { 0xe51c5739, 33 },
    { 0x836c5738, 33 },
    { 0xa6ac573d, 53 },
    { 0xf4c5742, 98 },
    { 0xe5dc5738, 95 },
    { 0xb43c5744, 43 },
    { 0xe83c5745, 61 },
    { 0x2f2c5744, 98 },
    { 0xd6dc5744, 76 },
    { 0xc35c5748, 87 },
    { 0, 0 },
    { 0x27bc574a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78c574e, 43 },
    { 0x5eec574e, 90 },
    { 0x613c5750, 23 },
    { 0x34ec5751, 55 },
    { 0x11fc5751, 61 },
    { 0x63ec5752, 61 },
    { 0xdd0c5751, 98 },
    { 0xaaac5754, 66 },
    { 0, 0 },
    { 0x9ec5757, 66 },
    { 0x5fac5758, 40 },
    { 0x739c5759, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x508c575c, 77 },
    { 0x96dc575c, 84 },
    { 0x89cc575c, 33 },
    { 0x73bc575c, 90 },
    { 0x5f9c575e, 91 },
    { 0, 0 },
    { 0x472c5762, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fc5765, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bc5768, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14c5778, 61 },
    { 0x188c5779, 43 },
    { 0x1a7c5779, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6c577d, 43 },
    { 0x4c0c577e, 84 },
    { 0x414c577e, 76 },
    { 0, 0 },
    { 0xf9ec5781, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bac5787, 55 },
    { 0x47cc5788, 77 },
    { 0x189c5787, 90 },
    { 0, 0 },
    { 0x87cc578b, 68 },
    { 0, 0 },
    { 0xc52c578d, 98 },
    { 0, 0 },
    { 0x59ac578f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x690c5792, 33 },
    { 0x524c5792, 98 },
    { 0, 0 },
    { 0xfaec5795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579c5799, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x611c579c, 84 },
    { 0x5a6c579c, 95 },
    { 0x566c579e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8c57a2, 23 },
    { 0x66cc57a2, 33 },
    { 0x1f6c57a4, 61 },
    { 0xdadc57a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ac57a8, 61 },
    { 0x943c57a9, 90 },
    { 0, 0 },
    { 0xa15c57ab, 78 },
    { 0xa68c57ac, 33 },
    { 0x577c57ad, 96 },
    { 0xfc3c57ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x535c57b1, 55 },
    { 0, 0 },
    { 0xc35c57b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9c57b6, 91 },
    { 0, 0 },
    { 0xf21c57b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdc57c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dc57c6, 90 },
    { 0x62cc57c7, 98 },
    { 0x447c57c7, 90 },
    { 0xcac57c8, 33 },
    { 0, 0 },
    { 0x987c57cb, 61 },
    { 0x4a5c57cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3c57cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bac57d3, 76 },
    { 0x6b7c57d3, 90 },
    { 0xf1ac57d4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880c57da, 34 },
    { 0xd2ac57db, 52 },
    { 0x82c57dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fac57e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ec57e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc57e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9c57eb, 66 },
    { 0, 0 },
    { 0xa00c57ed, 65 },
    { 0x168c57ee, 90 },
    { 0, 0 },
    { 0x34cc57f0, 98 },
    { 0x767c57f1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c57f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0c57f8, 61 },
    { 0x4b0c57f9, 90 },
    { 0xb16c57f9, 46 },
    { 0, 0 },
    { 0x6b3c57fc, 90 },
    { 0xd4fc57fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x258c5800, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1c5806, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ac580b, 98 },
    { 0x2d0c580c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ec5810, 55 },
    { 0, 0 },
    { 0xae1c5812, 40 },
    { 0x6e4c5813, 90 },
    { 0xbd3c5814, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x207c5817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966c581c, 33 },
    { 0, 0 },
    { 0xe74c581e, 95 },
    { 0xf9fc581f, 95 },
    { 0x373c581f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ec5823, 77 },
    { 0x2b0c5823, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9c5828, 55 },
    { 0, 0 },
    { 0xd75c582a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ac582e, 46 },
    { 0xd42c582f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33c5833, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cc5836, 76 },
    { 0x9c7c5837, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372c583b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cc5843, 95 },
    { 0xe28c5844, 90 },
    { 0x382c5845, 38 },
    { 0xce5c5846, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600c584d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8c5850, 43 },
    { 0, 0 },
    { 0x779c5852, 98 },
    { 0xe0fc5853, 90 },
    { 0x2d1c5853, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0c585a, 90 },
    { 0, 0 },
    { 0x324c585c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c5863, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7c5868, 33 },
    { 0x8d7c5868, 55 },
    { 0, 0 },
    { 0x457c586b, 43 },
    { 0, 0 },
    { 0x427c586d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cec5872, 90 },
    { 0xb91c5873, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43c5878, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c587b, 64 },
    { 0, 0 },
    { 0xec4c587d, 98 },
    { 0, 0 },
    { 0x835c587f, 95 },
    { 0, 0 },
    { 0xe8ac5881, 77 },
    { 0xd8cc5882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7c5885, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dc588b, 23 },
    { 0x4f4c588b, 61 },
    { 0x288c588c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dc5897, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cc589b, 95 },
    { 0, 0 },
    { 0x5bcc589d, 99 },
    { 0, 0 },
    { 0xbb1c589f, 34 },
    { 0x90ec58a0, 88 },
    { 0xf28c58a1, 78 },
    { 0x6cfc58a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcc58a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4c58a9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922c58af, 23 },
    { 0x6abc58af, 90 },
    { 0xa6c58b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06c58b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bc58b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6c58bb, 33 },
    { 0, 0 },
    { 0x2acc58bd, 100 },
    { 0, 0 },
    { 0xfdc58bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32c58c3, 93 },
    { 0, 0 },
    { 0xa54c58c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdc58ca, 61 },
    { 0x9b6c58cb, 90 },
    { 0x5edc58cb, 98 },
    { 0x313c58cc, 90 },
    { 0xd58c58cc, 48 },
    { 0, 0 },
    { 0x6b3c58d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79c58d3, 55 },
    { 0, 0 },
    { 0xe7bc58d5, 90 },
    { 0, 0 },
    { 0x5e4c58d7, 95 },
    { 0x498c58d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cec58de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0c58e4, 77 },
    { 0x757c58e5, 23 },
    { 0x709c58e6, 67 },
    { 0x105c58e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3c58ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7c58f4, 33 },
    { 0xf4fc58f5, 99 },
    { 0x27ac58f5, 90 },
    { 0x7edc58f7, 99 },
    { 0xcc3c58f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3c58fd, 38 },
    { 0xe74c58fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8c5903, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdc5906, 96 },
    { 0xf05c5907, 33 },
    { 0xa71c5908, 95 },
    { 0xff5c5907, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dac590f, 90 },
    { 0xea3c590f, 43 },
    { 0x197c5910, 33 },
    { 0xf85c5910, 4 },
    { 0xedec5913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc5916, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9c5919, 76 },
    { 0, 0 },
    { 0x3e3c591b, 95 },
    { 0x42bc591c, 98 },
    { 0, 0 },
    { 0x6dbc591e, 99 },
    { 0xbd8c591e, 23 },
    { 0xd32c591e, 65 },
    { 0xe65c5921, 67 },
    { 0x2e9c5920, 80 },
    { 0x42ec5923, 90 },
    { 0x157c5923, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dc5927, 90 },
    { 0x4efc5928, 61 },
    { 0x4efc5929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dc592c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299c5930, 76 },
    { 0, 0 },
    { 0x912c5932, 40 },
    { 0x486c5933, 55 },
    { 0, 0 },
    { 0x9ddc5935, 100 },
    { 0xfc0c5936, 90 },
    { 0x3a0c5937, 61 },
    { 0, 0 },
    { 0x89c5939, 90 },
    { 0xd0bc593a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c5941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74c5944, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362c5948, 90 },
    { 0x7e5c5949, 23 },
    { 0xb08c594a, 98 },
    { 0x264c5948, 95 },
    { 0x2aec5948, 36 },
    { 0, 0 },
    { 0x72ac594e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fc5959, 98 },
    { 0x526c595a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bc595e, 90 },
    { 0xddbc595f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977c5965, 91 },
    { 0, 0 },
    { 0x667c5967, 55 },
    { 0x7cbc5968, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadc596d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cc5971, 91 },
    { 0xdfdc5972, 78 },
    { 0x524c5971, 43 },
    { 0xe56c5974, 90 },
    { 0xdd8c5975, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c597b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2c597f, 95 },
    { 0x618c5980, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ac5984, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24c5987, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8c598f, 93 },
    { 0x26dc598f, 34 },
    { 0xfafc5991, 33 },
    { 0x7bec5991, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dc5995, 55 },
    { 0, 0 },
    { 0x975c5997, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0x211c599a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dc59a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2c59a6, 98 },
    { 0, 0 },
    { 0xeefc59a8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212c59ac, 94 },
    { 0xed3c59ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cc59b3, 101 },
    { 0x3b4c59b4, 95 },
    { 0xad1c59b5, 55 },
    { 0xfc0c59b5, 47 },
    { 0x5f1c59b7, 90 },
    { 0xca7c59b6, 90 },
    { 0x888c59b4, 95 },
    { 0x953c59b3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6c59bf, 90 },
    { 0, 0 },
    { 0x3a6c59c1, 55 },
    { 0x19c59c2, 43 },
    { 0x2a3c59c1, 90 },
    { 0x8ac59c4, 52 },
    { 0xb1ac59c5, 76 },
    { 0xba6c59c6, 98 },
    { 0x66ac59c5, 83 },
    { 0xa1bc59c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13c59cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6c59d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5c59d7, 34 },
    { 0x60ec59d8, 90 },
    { 0x372c59d9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dc59dc, 46 },
    { 0x498c59dd, 93 },
    { 0, 0 },
    { 0x99ec59df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8c59e2, 100 },
    { 0xb05c59e3, 76 },
    { 0x8e1c59e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc59ec, 75 },
    { 0xb80c59ed, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c59f2, 34 },
    { 0xc59c59f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93c59fd, 61 },
    { 0xc81c59fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c5a03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ac5a07, 85 },
    { 0x3c6c5a07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618c5a14, 34 },
    { 0, 0 },
    { 0xc97c5a16, 55 },
    { 0, 0 },
    { 0x12dc5a18, 43 },
    { 0x4a7c5a18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6c5a1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc5a21, 98 },
    { 0x94dc5a22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc5a27, 90 },
    { 0x1efc5a28, 84 },
    { 0xa11c5a29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ac5a35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85c5a39, 23 },
    { 0xdecc5a3a, 90 },
    { 0xa61c5a39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ec5a3f, 33 },
    { 0xf38c5a3f, 95 },
    { 0xf4c5a3f, 98 },
    { 0x344c5a42, 52 },
    { 0x9dac5a43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2c5a49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c5a4e, 93 },
    { 0, 0 },
    { 0xf86c5a50, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x256c5a53, 23 },
    { 0xc2c5a54, 78 },
    { 0x2b6c5a53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1c5a59, 84 },
    { 0x6c5c5a59, 33 },
    { 0x685c5a5a, 99 },
    { 0x76fc5a5a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ac5a5f, 55 },
    { 0xcf0c5a5f, 90 },
    { 0, 0 },
    { 0xaf2c5a62, 88 },
    { 0x232c5a63, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7c5a68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242c5a6c, 40 },
    { 0xabcc5a6d, 33 },
    { 0xd7fc5a6d, 33 },
    { 0xeb4c5a6f, 76 },
    { 0x1acc5a70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c5a73, 61 },
    { 0x7bcc5a73, 99 },
    { 0x27dc5a74, 95 },
    { 0x749c5a76, 33 },
    { 0x7c7c5a75, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19c5a7d, 78 },
    { 0x435c5a7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ac5a81, 33 },
    { 0x968c5a82, 24 },
    { 0x53ec5a82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31c5a87, 55 },
    { 0x7aac5a88, 23 },
    { 0xf9ac5a88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1c5a8c, 52 },
    { 0x754c5a8c, 84 },
    { 0x3bcc5a8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c5a91, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeac5a96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc5a99, 23 },
    { 0x296c5a99, 90 },
    { 0xd15c5a99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23c5a9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47c5aa1, 76 },
    { 0x210c5aa1, 64 },
    { 0x795c5aa3, 90 },
    { 0, 0 },
    { 0x641c5aa5, 55 },
    { 0, 0 },
    { 0x9cfc5aa7, 85 },
    { 0xadbc5aa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x716c5aab, 23 },
    { 0x803c5aac, 95 },
    { 0x1dfc5aab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769c5ab2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec5aba, 33 },
    { 0xe72c5abb, 95 },
    { 0, 0 },
    { 0xa6dc5abd, 33 },
    { 0xd46c5abe, 66 },
    { 0xed1c5abf, 69 },
    { 0, 0 },
    { 0xeafc5ac1, 17 },
    { 0xf30c5ac2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fc5ac8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5c5acf, 90 },
    { 0, 0 },
    { 0x951c5ad1, 23 },
    { 0x39bc5ad2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74c5ad6, 61 },
    { 0, 0 },
    { 0x8fac5ad8, 67 },
    { 0x689c5ad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ac5ade, 55 },
    { 0x8dac5adf, 52 },
    { 0xf58c5ae0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acc5ae3, 90 },
    { 0xc21c5ae3, 98 },
    { 0, 0 },
    { 0x7e8c5ae6, 93 },
    { 0x68fc5ae7, 88 },
    { 0x684c5ae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c5af2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c5aff, 61 },
    { 0x721c5b00, 55 },
    { 0x671c5b01, 23 },
    { 0x820c5b01, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125c5b06, 33 },
    { 0, 0 },
    { 0x7c0c5b08, 23 },
    { 0, 0 },
    { 0x54dc5b0a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bc5b0e, 46 },
    { 0x91ec5b0f, 93 },
    { 0x879c5b10, 100 },
    { 0x77c5b11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc5b14, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ac5b17, 78 },
    { 0, 0 },
    { 0x36fc5b19, 34 },
    { 0x2e3c5b19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34c5b1d, 66 },
    { 0xbcac5b1e, 98 },
    { 0xacc5b1f, 93 },
    { 0, 0 },
    { 0xc41c5b21, 90 },
    { 0, 0 },
    { 0xc9c5b23, 90 },
    { 0, 0 },
    { 0x41c5b25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cc5b31, 93 },
    { 0x585c5b32, 33 },
    { 0x73bc5b32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbc5b37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391c5b40, 95 },
    { 0xa2fc5b41, 95 },
    { 0xd09c5b42, 46 },
    { 0x85dc5b43, 33 },
    { 0x110c5b43, 61 },
    { 0x11c5b45, 23 },
    { 0x218c5b43, 90 },
    { 0xd37c5b47, 34 },
    { 0, 0 },
    { 0x4d8c5b49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65c5b4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63c5b5c, 98 },
    { 0x1f1c5b5d, 90 },
    { 0, 0 },
    { 0x7c2c5b5f, 98 },
    { 0x217c5b5f, 33 },
    { 0x4d6c5b60, 97 },
    { 0x174c5b62, 66 },
    { 0x88fc5b62, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341c5b67, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69c5b6a, 66 },
    { 0x314c5b6a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x451c5b6e, 90 },
    { 0, 0 },
    { 0x93bc5b70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdc5b74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9c5b7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7c5b80, 61 },
    { 0x9f4c5b81, 61 },
    { 0x523c5b80, 66 },
    { 0x38ec5b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcac5b88, 43 },
    { 0xbedc5b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419c5b8e, 98 },
    { 0x2bbc5b8f, 34 },
    { 0xbafc5b90, 81 },
    { 0, 0 },
    { 0xa59c5b92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3c5b96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c5b9a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cc5b9e, 55 },
    { 0x289c5b9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5c5ba2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbc5ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4c5ba8, 33 },
    { 0x733c5ba9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10c5bac, 46 },
    { 0x1cac5bad, 23 },
    { 0, 0 },
    { 0x1ecc5baf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8c5bb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67c5bbb, 61 },
    { 0xde3c5bbb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c5bc4, 55 },
    { 0xa6ec5bc4, 90 },
    { 0, 0 },
    { 0xcedc5bc7, 98 },
    { 0x390c5bc8, 61 },
    { 0, 0 },
    { 0x3a0c5bca, 34 },
    { 0x61c5bca, 33 },
    { 0x1a1c5bcc, 60 },
    { 0x1afc5bcd, 98 },
    { 0x9a2c5bce, 76 },
    { 0xb53c5bcd, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c5bd8, 23 },
    { 0x807c5bd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c5bdc, 40 },
    { 0, 0 },
    { 0x28cc5bde, 24 },
    { 0xe1bc5bdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8c5be2, 95 },
    { 0x4ffc5be3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dec5be6, 93 },
    { 0x222c5be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c5beb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec5bef, 95 },
    { 0x16c5bf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1c5bf4, 34 },
    { 0xd9c5bf5, 100 },
    { 0, 0 },
    { 0xb35c5bf7, 55 },
    { 0xd9ac5bf8, 95 },
    { 0, 0 },
    { 0x617c5bfa, 66 },
    { 0xd0cc5bfb, 43 },
    { 0, 0 },
    { 0xd95c5bfd, 33 },
    { 0x7aac5bfe, 90 },
    { 0x989c5bfe, 95 },
    { 0, 0 },
    { 0x4fcc5c01, 80 },
    { 0, 0 },
    { 0x648c5c03, 66 },
    { 0x41bc5c03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc5c07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c5c0d, 33 },
    { 0xc8bc5c0e, 90 },
    { 0xb47c5c0d, 66 },
    { 0, 0 },
    { 0x11cc5c11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cc5c14, 95 },
    { 0, 0 },
    { 0xb37c5c16, 95 },
    { 0xb41c5c17, 33 },
    { 0x347c5c18, 61 },
    { 0xef2c5c19, 78 },
    { 0xed2c5c19, 95 },
    { 0x2f5c5c17, 90 },
    { 0, 0 },
    { 0xe90c5c1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c5c20, 99 },
    { 0, 0 },
    { 0xc0ac5c22, 98 },
    { 0xa3c5c23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efc5c28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dc5c2d, 93 },
    { 0x861c5c2e, 40 },
    { 0x7f6c5c2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34c5c33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdc5c37, 23 },
    { 0x81ec5c38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x586c5c3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586c5c44, 55 },
    { 0x358c5c44, 91 },
    { 0, 0 },
    { 0x42cc5c47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102c5c4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfc5c54, 95 },
    { 0, 0 },
    { 0x544c5c56, 95 },
    { 0x580c5c57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0c5c5c, 99 },
    { 0x970c5c5d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1c5c60, 90 },
    { 0xaa0c5c61, 33 },
    { 0, 0 },
    { 0x227c5c63, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fc5c6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c5c6e, 88 },
    { 0xa45c5c6f, 98 },
    { 0x5c8c5c70, 61 },
    { 0, 0 },
    { 0x954c5c72, 61 },
    { 0xa9c5c73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811c5c7a, 95 },
    { 0xa68c5c7b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bc5c7e, 100 },
    { 0x1c9c5c7e, 76 },
    { 0x68fc5c7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39c5c83, 99 },
    { 0x665c5c83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fc5c87, 61 },
    { 0x7f3c5c88, 34 },
    { 0xb69c5c88, 23 },
    { 0, 0 },
    { 0xcbdc5c8b, 78 },
    { 0x760c5c8b, 95 },
    { 0, 0 },
    { 0x931c5c8e, 95 },
    { 0x7b1c5c8e, 51 },
    { 0, 0 },
    { 0xe3ac5c91, 81 },
    { 0, 0 },
    { 0x74c5c93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239c5c97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x105c5c9a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746c5c9f, 93 },
    { 0x1cac5ca0, 76 },
    { 0x32dc5ca1, 61 },
    { 0x7a6c5ca2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edc5ca6, 67 },
    { 0x368c5ca7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7c5cac, 98 },
    { 0, 0 },
    { 0x9aec5cae, 95 },
    { 0x42cc5caf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dc5cb2, 61 },
    { 0x7aec5cb2, 84 },
    { 0x658c5cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x394c5cb7, 90 },
    { 0x324c5cb7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cc5cbb, 46 },
    { 0x113c5cbc, 46 },
    { 0xa57c5cbc, 23 },
    { 0xc09c5cbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597c5cc2, 90 },
    { 0x612c5cc3, 98 },
    { 0x248c5cc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbec5cc9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec5ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0c5cd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3c5cd9, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62c5cdd, 61 },
    { 0xa1bc5cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c5ce2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c5ce9, 93 },
    { 0, 0 },
    { 0x7c5ceb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec5cef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c5cf2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x277c5cf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bec5cfc, 95 },
    { 0x294c5cfd, 95 },
    { 0, 0 },
    { 0xc61c5cff, 90 },
    { 0, 0 },
    { 0x800c5d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c5d06, 34 },
    { 0xa7cc5d07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c5d0b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fc5d0e, 67 },
    { 0xc60c5d0f, 33 },
    { 0, 0 },
    { 0x3d1c5d11, 61 },
    { 0xeebc5d11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fc5d1b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c5d1e, 90 },
    { 0x710c5d1f, 98 },
    { 0x617c5d20, 33 },
    { 0, 0 },
    { 0xd3fc5d22, 95 },
    { 0xb4cc5d23, 23 },
    { 0x6e8c5d23, 93 },
    { 0x311c5d25, 90 },
    { 0x755c5d26, 33 },
    { 0xb2dc5d27, 17 },
    { 0xcf1c5d26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82c5d2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945c5d30, 84 },
    { 0x8ccc5d30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4c5d34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c5d38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926c5d3d, 33 },
    { 0, 0 },
    { 0xde1c5d3f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c5d43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839c5d47, 90 },
    { 0x99ac5d48, 96 },
    { 0, 0 },
    { 0x64dc5d4a, 95 },
    { 0x55cc5d4b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x213c5d4e, 90 },
    { 0x5d8c5d4e, 98 },
    { 0xe1c5d50, 33 },
    { 0xb67c5d51, 43 },
    { 0, 0 },
    { 0x451c5d53, 95 },
    { 0x62ac5d53, 98 },
    { 0, 0 },
    { 0xaac5d56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650c5d5b, 61 },
    { 0, 0 },
    { 0x22fc5d5d, 61 },
    { 0xd1ac5d5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc5d62, 53 },
    { 0xf2bc5d63, 98 },
    { 0xc9ec5d63, 98 },
    { 0x801c5d65, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2c5d6e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386c5d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ec5d7c, 55 },
    { 0x599c5d7d, 33 },
    { 0xe56c5d7e, 76 },
    { 0xeb5c5d7f, 76 },
    { 0x8e2c5d7c, 95 },
    { 0xc53c5d81, 98 },
    { 0, 0 },
    { 0x822c5d83, 98 },
    { 0, 0 },
    { 0x74c5d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ac5d8c, 17 },
    { 0xc21c5d8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c5d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ac5d96, 90 },
    { 0x21cc5d97, 98 },
    { 0x347c5d98, 23 },
    { 0xc72c5d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbc5d9c, 95 },
    { 0xe4dc5d9c, 90 },
    { 0, 0 },
    { 0x5cac5d9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30c5da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dc5da8, 78 },
    { 0x8a5c5da8, 23 },
    { 0x621c5daa, 33 },
    { 0x606c5dab, 98 },
    { 0xd06c5dac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46c5daf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ac5db2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x750c5db5, 75 },
    { 0x667c5db6, 52 },
    { 0xed7c5db5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64c5dbe, 95 },
    { 0xce1c5dbe, 34 },
    { 0xb34c5dbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684c5dc4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x927c5dc7, 98 },
    { 0, 0 },
    { 0x7f8c5dc9, 46 },
    { 0, 0 },
    { 0x4a5c5dcb, 61 },
    { 0x1bfc5dcc, 98 },
    { 0, 0 },
    { 0x91c5dce, 98 },
    { 0x45fc5dcf, 90 },
    { 0x2d2c5dd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c5dd7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59c5ddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ec5de2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbc5def, 61 },
    { 0x645c5df0, 33 },
    { 0x41ec5df1, 65 },
    { 0, 0 },
    { 0xa99c5df3, 98 },
    { 0x144c5df4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3c5df8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x799c5dfb, 61 },
    { 0x56c5dfb, 69 },
    { 0xa46c5dfd, 61 },
    { 0, 0 },
    { 0xf6ac5dff, 61 },
    { 0, 0 },
    { 0x376c5e01, 99 },
    { 0x86c5e02, 98 },
    { 0x3f9c5e01, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ec5e0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ac5e11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c5e14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c5e17, 34 },
    { 0, 0 },
    { 0x103c5e19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636c5e26, 61 },
    { 0x2b7c5e27, 23 },
    { 0, 0 },
    { 0x63ac5e29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c5e2c, 75 },
    { 0, 0 },
    { 0xe19c5e2e, 34 },
    { 0x2b6c5e2e, 84 },
    { 0x7dc5e30, 76 },
    { 0xfa5c5e30, 23 },
    { 0x843c5e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc5e35, 33 },
    { 0xd7fc5e36, 66 },
    { 0xa76c5e37, 76 },
    { 0, 0 },
    { 0x6e4c5e39, 95 },
    { 0xd73c5e3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fc5e3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bc5e40, 46 },
    { 0, 0 },
    { 0x3eac5e42, 33 },
    { 0x889c5e43, 90 },
    { 0, 0 },
    { 0xc82c5e45, 61 },
    { 0x72c5e45, 34 },
    { 0xf92c5e47, 93 },
    { 0x48bc5e48, 98 },
    { 0, 0 },
    { 0xbeec5e4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfc5e51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6c5e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeac5e59, 55 },
    { 0x5d6c5e59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdc5e5d, 46 },
    { 0x617c5e5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c5e62, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6c5e66, 95 },
    { 0, 0 },
    { 0x6a7c5e68, 91 },
    { 0, 0 },
    { 0xc99c5e6a, 23 },
    { 0x702c5e6b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cc5e70, 90 },
    { 0xde3c5e70, 98 },
    { 0xdd5c5e72, 23 },
    { 0x52ac5e71, 67 },
    { 0, 0 },
    { 0xde6c5e75, 77 },
    { 0xcf8c5e76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ac5e7b, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935c5e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59c5e8d, 98 },
    { 0, 0 },
    { 0xfdc5e8f, 55 },
    { 0, 0 },
    { 0x4a9c5e91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc5e96, 84 },
    { 0x1bac5e96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ac5ea1, 43 },
    { 0, 0 },
    { 0xc5fc5ea3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabc5ea7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x285c5eaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ac5ead, 98 },
    { 0, 0 },
    { 0x7cc5eaf, 98 },
    { 0x41ac5eb0, 23 },
    { 0x66ec5eb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fc5eb6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804c5ebc, 33 },
    { 0, 0 },
    { 0x5c6c5ebe, 46 },
    { 0x3a4c5ebe, 90 },
    { 0, 0 },
    { 0xf45c5ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x694c5ec4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ac5ec7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x603c5eca, 100 },
    { 0, 0 },
    { 0x9fbc5ecc, 61 },
    { 0xa33c5ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c5ed2, 90 },
    { 0, 0 },
    { 0xc23c5ed4, 84 },
    { 0xe6ac5ed5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6c5ed9, 61 },
    { 0x976c5eda, 61 },
    { 0xaf1c5eda, 95 },
    { 0x73c5edc, 76 },
    { 0x743c5ed9, 95 },
    { 0x148c5ede, 98 },
    { 0xd88c5ede, 33 },
    { 0xcd3c5edf, 61 },
    { 0, 0 },
    { 0xac5c5ee2, 90 },
    { 0x743c5ee2, 44 },
    { 0xe7ec5ee2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dc5ee7, 90 },
    { 0, 0 },
    { 0xd56c5ee9, 90 },
    { 0xce7c5eea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfc5eee, 98 },
    { 0x962c5eee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dc5ef6, 61 },
    { 0x311c5ef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759c5efb, 23 },
    { 0xc0fc5efb, 90 },
    { 0x589c5efc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72c5f02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5f0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x765c5f0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5c5f12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc5f1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1c5f1f, 43 },
    { 0xfb7c5f20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdc5f23, 77 },
    { 0x3f7c5f24, 99 },
    { 0xaebc5f25, 98 },
    { 0, 0 },
    { 0xc80c5f27, 23 },
    { 0xec1c5f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119c5f2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86c5f32, 66 },
    { 0xaeec5f33, 98 },
    { 0x308c5f34, 78 },
    { 0xc26c5f35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c5f38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7c5f3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34c5f48, 55 },
    { 0x29ac5f49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dc5f52, 95 },
    { 0x670c5f53, 95 },
    { 0x34c5f53, 90 },
    { 0, 0 },
    { 0x2abc5f56, 91 },
    { 0x2c9c5f56, 23 },
    { 0, 0 },
    { 0xda5c5f59, 88 },
    { 0xc10c5f5a, 52 },
    { 0xb1ac5f5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccc5f63, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4c5f66, 90 },
    { 0x2cbc5f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c5f6a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dc5f70, 64 },
    { 0x4bc5f70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c5f74, 33 },
    { 0xa74c5f75, 17 },
    { 0xc36c5f76, 85 },
    { 0x533c5f77, 90 },
    { 0x7a2c5f76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdc5f7b, 88 },
    { 0x83dc5f7b, 34 },
    { 0x23cc5f7d, 90 },
    { 0xbb1c5f7c, 96 },
    { 0, 0 },
    { 0xd25c5f80, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cc5f83, 53 },
    { 0, 0 },
    { 0xe3fc5f85, 46 },
    { 0x89ec5f86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9c5f8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5c5f90, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9dc5f99, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ac5f9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dc5fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x987c5faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c5fb2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3c5fb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5fbf, 43 },
    { 0x9dcc5fc0, 99 },
    { 0x148c5fc1, 93 },
    { 0x411c5fc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716c5fc9, 90 },
    { 0xe95c5fca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc5fd5, 61 },
    { 0x384c5fd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580c5fdb, 100 },
    { 0xa80c5fdc, 90 },
    { 0x24bc5fdc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87c5fe1, 77 },
    { 0, 0 },
    { 0x6e8c5fe3, 90 },
    { 0x714c5fe3, 33 },
    { 0xb82c5fe5, 74 },
    { 0xd0dc5fe6, 84 },
    { 0, 0 },
    { 0x896c5fe8, 23 },
    { 0, 0 },
    { 0xfc9c5fea, 98 },
    { 0x924c5feb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ec5fef, 76 },
    { 0xa5bc5ff0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbc5ff3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4c5ff8, 33 },
    { 0x5c5ff9, 83 },
    { 0, 0 },
    { 0x3fec5ffb, 23 },
    { 0, 0 },
    { 0x4ebc5ffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adc6007, 66 },
    { 0xbe0c6008, 80 },
    { 0x1e0c6009, 33 },
    { 0x29ac6007, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcc600d, 78 },
    { 0x954c600d, 61 },
    { 0xff2c600e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8c6012, 33 },
    { 0, 0 },
    { 0x75fc6014, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec6017, 34 },
    { 0xdd1c6018, 95 },
    { 0x3abc6019, 61 },
    { 0x17dc601a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fc601d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378c6022, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27c6028, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ec602c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c6033, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99c6042, 61 },
    { 0x9f7c6042, 90 },
    { 0x89dc6044, 36 },
    { 0, 0 },
    { 0x84c6046, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2c6049, 94 },
    { 0x247c6049, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c604e, 87 },
    { 0x8d1c604f, 33 },
    { 0x6f8c6050, 90 },
    { 0xe77c604e, 33 },
    { 0xbb4c6052, 98 },
    { 0, 0 },
    { 0xe14c6054, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84c6059, 84 },
    { 0, 0 },
    { 0x737c605b, 84 },
    { 0, 0 },
    { 0x84ec605d, 101 },
    { 0xfd0c605d, 98 },
    { 0, 0 },
    { 0xf30c6060, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2c6068, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20c606e, 84 },
    { 0, 0 },
    { 0x7bc6070, 90 },
    { 0, 0 },
    { 0x27dc6072, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x320c6075, 98 },
    { 0, 0 },
    { 0xf22c6077, 90 },
    { 0x1bc6078, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8c607c, 78 },
    { 0x169c607c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8c6080, 98 },
    { 0, 0 },
    { 0x4cec6082, 33 },
    { 0, 0 },
    { 0xb94c6084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cc6088, 61 },
    { 0xcdfc6089, 56 },
    { 0xd04c608a, 56 },
    { 0, 0 },
    { 0xdecc608c, 61 },
    { 0x15ec608d, 33 },
    { 0x2b5c608c, 33 },
    { 0xa84c608d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6c6095, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ac609b, 95 },
    { 0x142c609c, 81 },
    { 0x230c609c, 33 },
    { 0x51c609c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2c60a1, 55 },
    { 0xdfcc60a2, 61 },
    { 0, 0 },
    { 0x96c60a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c60ac, 61 },
    { 0x3eac60ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c60b0, 34 },
    { 0x31cc60b0, 67 },
    { 0, 0 },
    { 0xc56c60b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac60b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131c60ba, 61 },
    { 0x704c60bb, 52 },
    { 0, 0 },
    { 0xccec60bd, 33 },
    { 0x786c60bd, 91 },
    { 0x24ec60bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414c60c9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc60cc, 94 },
    { 0x9cbc60cd, 48 },
    { 0x490c60ce, 98 },
    { 0x9a6c60ce, 76 },
    { 0, 0 },
    { 0xde5c60d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982c60d8, 43 },
    { 0x142c60d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddc60dd, 76 },
    { 0xb5cc60de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02c60e4, 43 },
    { 0xfb9c60e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304c60e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66c60f2, 100 },
    { 0xd9ac60f3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83c60f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9c60fe, 34 },
    { 0, 0 },
    { 0xadcc6100, 27 },
    { 0xe04c6101, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ac6104, 46 },
    { 0, 0 },
    { 0xe7dc6106, 95 },
    { 0, 0 },
    { 0x891c6108, 95 },
    { 0x992c6109, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acc610f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x487c6112, 91 },
    { 0x300c6112, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cc6116, 61 },
    { 0, 0 },
    { 0x54c6118, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c6120, 61 },
    { 0xca8c6121, 90 },
    { 0xfcfc6120, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24c6125, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddc6128, 76 },
    { 0, 0 },
    { 0x1c1c612a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9c6135, 33 },
    { 0xb15c6136, 34 },
    { 0xc35c6137, 90 },
    { 0x626c6138, 67 },
    { 0xe69c6137, 88 },
    { 0, 0 },
    { 0xf9dc613b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcc613f, 55 },
    { 0, 0 },
    { 0xf10c6141, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987c6147, 43 },
    { 0xeeec6147, 95 },
    { 0xd5ec6147, 87 },
    { 0, 0 },
    { 0x31ec614b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9c614f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bc6152, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ec6155, 66 },
    { 0x154c6156, 98 },
    { 0x7f1c6157, 90 },
    { 0x8f7c6157, 100 },
    { 0x20c6159, 61 },
    { 0x78bc615a, 91 },
    { 0xc4dc615b, 23 },
    { 0xb83c6159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1c6160, 33 },
    { 0, 0 },
    { 0xc9fc6162, 34 },
    { 0xf5c6162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2c6166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cc6169, 90 },
    { 0x5ffc6169, 90 },
    { 0xfbc616b, 41 },
    { 0x2a5c616c, 90 },
    { 0xb3bc616b, 52 },
    { 0xacac616e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52c6171, 43 },
    { 0xd0dc6172, 93 },
    { 0, 0 },
    { 0x279c6174, 90 },
    { 0, 0 },
    { 0xb1ac6176, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fc6179, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15c617c, 90 },
    { 0x84bc617d, 65 },
    { 0x63ac617d, 61 },
    { 0, 0 },
    { 0x595c6180, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acc6184, 61 },
    { 0, 0 },
    { 0x488c6186, 95 },
    { 0xd84c6187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99c618a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aec618d, 61 },
    { 0xfdc618e, 90 },
    { 0x2d0c618f, 90 },
    { 0x409c6190, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4c6194, 90 },
    { 0, 0 },
    { 0xd9dc6196, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdc619a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6c619f, 76 },
    { 0x55bc61a0, 76 },
    { 0, 0 },
    { 0xb33c61a2, 23 },
    { 0x56ec61a2, 61 },
    { 0x782c61a4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387c61af, 78 },
    { 0, 0 },
    { 0x764c61b1, 91 },
    { 0xcc8c61b1, 88 },
    { 0, 0 },
    { 0x225c61b4, 95 },
    { 0xacdc61b4, 33 },
    { 0, 0 },
    { 0xf86c61b7, 61 },
    { 0x287c61b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6c61be, 55 },
    { 0, 0 },
    { 0x2c5c61c0, 87 },
    { 0, 0 },
    { 0x22c61c2, 84 },
    { 0xfa4c61c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x156c61c6, 23 },
    { 0, 0 },
    { 0xa9cc61c8, 76 },
    { 0xb46c61c8, 95 },
    { 0xe4bc61ca, 76 },
    { 0x2ddc61cb, 85 },
    { 0, 0 },
    { 0xa0dc61cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x896c61d0, 98 },
    { 0x586c61d0, 67 },
    { 0x1d6c61d2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871c61d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168c61df, 46 },
    { 0, 0 },
    { 0xf44c61e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbc61e5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03c61e8, 90 },
    { 0, 0 },
    { 0xcc5c61ea, 61 },
    { 0, 0 },
    { 0xed8c61ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4c61ef, 90 },
    { 0x8ebc61f0, 46 },
    { 0, 0 },
    { 0x44cc61f2, 90 },
    { 0, 0 },
    { 0xb4bc61f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ac61f9, 97 },
    { 0x980c61fa, 34 },
    { 0x799c61fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57c6201, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c6204, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x562c6207, 61 },
    { 0x1aac6208, 99 },
    { 0xcdbc6209, 61 },
    { 0xcb0c6209, 79 },
    { 0x6f0c6208, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455c6211, 65 },
    { 0x64dc6211, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4c6217, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bc621c, 90 },
    { 0x75ec621d, 90 },
    { 0xb8cc621d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bc6225, 98 },
    { 0x15ac6226, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c622b, 43 },
    { 0xfb2c622b, 98 },
    { 0xf6bc622c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbac6230, 95 },
    { 0x35cc6230, 99 },
    { 0xc24c6232, 66 },
    { 0x4c8c6232, 99 },
    { 0xf84c6234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x899c6237, 61 },
    { 0x777c6238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3c623e, 48 },
    { 0x83dc623f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ac6242, 61 },
    { 0x1b0c6243, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8c6248, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53c624c, 76 },
    { 0x150c624d, 75 },
    { 0x1b7c624e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9c6252, 100 },
    { 0, 0 },
    { 0x3a1c6254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c625a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc625d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ac6262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8c6267, 61 },
    { 0x956c6268, 95 },
    { 0x46dc6269, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32c626d, 33 },
    { 0, 0 },
    { 0xf5cc626f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45dc6273, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896c6278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cac6282, 61 },
    { 0, 0 },
    { 0x8edc6284, 33 },
    { 0, 0 },
    { 0x629c6286, 55 },
    { 0x118c6286, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ac628a, 52 },
    { 0, 0 },
    { 0x12ac628c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830c6294, 46 },
    { 0x82ec6294, 46 },
    { 0xc72c6296, 95 },
    { 0x7d2c6294, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c629f, 91 },
    { 0xef6c62a0, 98 },
    { 0xfabc62a1, 61 },
    { 0x6a1c62a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10c62a5, 98 },
    { 0x1ec62a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819c62ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c62ae, 34 },
    { 0, 0 },
    { 0x924c62b0, 93 },
    { 0x952c62b0, 33 },
    { 0x287c62b1, 34 },
    { 0, 0 },
    { 0x98fc62b4, 23 },
    { 0xd1fc62b4, 33 },
    { 0xf51c62b5, 23 },
    { 0xf91c62b6, 76 },
    { 0x956c62b7, 95 },
    { 0xc86c62b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c62bd, 61 },
    { 0x7cc62bd, 23 },
    { 0, 0 },
    { 0x6efc62c0, 33 },
    { 0, 0 },
    { 0xf74c62c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cc62c5, 93 },
    { 0, 0 },
    { 0xeabc62c7, 95 },
    { 0x22ac62c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335c62ce, 95 },
    { 0x55cc62ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0c62d8, 66 },
    { 0x13cc62d9, 87 },
    { 0xd16c62da, 61 },
    { 0x99dc62db, 61 },
    { 0xa4ec62db, 95 },
    { 0x71bc62dd, 61 },
    { 0, 0 },
    { 0x5c6c62df, 95 },
    { 0, 0 },
    { 0xe83c62e1, 94 },
    { 0, 0 },
    { 0x1f7c62e3, 90 },
    { 0, 0 },
    { 0xdd5c62e5, 93 },
    { 0x564c62e5, 23 },
    { 0x318c62e7, 23 },
    { 0, 0 },
    { 0x29c62e9, 90 },
    { 0x211c62ea, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfc62ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x366c62f2, 23 },
    { 0x905c62f3, 98 },
    { 0x5d4c62f4, 76 },
    { 0xe81c62f5, 34 },
    { 0xa02c62f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fac62f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9c62fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60c6302, 99 },
    { 0, 0 },
    { 0xa61c6304, 95 },
    { 0, 0 },
    { 0x481c6306, 91 },
    { 0xe36c6307, 91 },
    { 0xf77c6307, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c630b, 100 },
    { 0x5bac630b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dc6313, 34 },
    { 0x6c8c6314, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5c6319, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c631e, 98 },
    { 0x297c631f, 77 },
    { 0xcc4c6320, 33 },
    { 0xaafc6320, 90 },
    { 0, 0 },
    { 0x1a0c6323, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9c632a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc632d, 23 },
    { 0x4b0c632d, 55 },
    { 0xb94c632d, 34 },
    { 0xfcac6330, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ac633f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac6342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785c634d, 95 },
    { 0, 0 },
    { 0xcfac634f, 99 },
    { 0xd43c6350, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcc6353, 85 },
    { 0x18fc6353, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdc6357, 66 },
    { 0, 0 },
    { 0x71c6359, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7c635d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9c6361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bc6367, 78 },
    { 0xa07c6368, 43 },
    { 0x592c6367, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c6370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59c6375, 34 },
    { 0, 0 },
    { 0xdd2c6377, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbc637a, 98 },
    { 0x371c637b, 95 },
    { 0x22ec637b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dc6381, 90 },
    { 0xd32c6382, 88 },
    { 0xdb3c6383, 61 },
    { 0x14bc6382, 46 },
    { 0x2f4c6384, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6c6388, 93 },
    { 0xc54c6389, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bc638c, 88 },
    { 0, 0 },
    { 0xe98c638e, 33 },
    { 0x673c638f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dc6395, 98 },
    { 0, 0 },
    { 0xd3dc6397, 46 },
    { 0xc0fc6398, 95 },
    { 0xc22c6398, 34 },
    { 0x26fc6397, 95 },
    { 0xf00c6397, 43 },
    { 0xbe4c639c, 34 },
    { 0x3ccc639d, 93 },
    { 0x5e2c639c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4c63a2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fc63a8, 90 },
    { 0xa9fc63a9, 86 },
    { 0, 0 },
    { 0xf28c63ab, 95 },
    { 0x93bc63ac, 98 },
    { 0xb1bc63ad, 90 },
    { 0x1b3c63ae, 76 },
    { 0, 0 },
    { 0x636c63b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2c63b5, 67 },
    { 0x532c63b6, 55 },
    { 0xa4ec63b7, 93 },
    { 0x3f1c63b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc63bb, 69 },
    { 0xbf3c63bc, 90 },
    { 0, 0 },
    { 0x324c63be, 90 },
    { 0xa00c63be, 46 },
    { 0, 0 },
    { 0x7aec63c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06c63c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc63cc, 61 },
    { 0x2f3c63cd, 17 },
    { 0x1fbc63ce, 65 },
    { 0xd90c63cf, 33 },
    { 0x2abc63d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dc63d3, 33 },
    { 0x120c63d4, 61 },
    { 0x190c63d4, 38 },
    { 0, 0 },
    { 0xcfec63d7, 93 },
    { 0, 0 },
    { 0x8b7c63d9, 91 },
    { 0, 0 },
    { 0xf0cc63db, 43 },
    { 0xc97c63dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dc63df, 88 },
    { 0x7c4c63e0, 98 },
    { 0, 0 },
    { 0x939c63e2, 76 },
    { 0x33c63e3, 33 },
    { 0x86c63e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c63e7, 83 },
    { 0xfadc63e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x843c63eb, 52 },
    { 0x1a3c63ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93c63f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5c63f6, 55 },
    { 0xb6ac63f7, 95 },
    { 0x548c63f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22c6402, 66 },
    { 0, 0 },
    { 0xbe2c6404, 76 },
    { 0xd27c6404, 87 },
    { 0x11fc6405, 81 },
    { 0xafac6407, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97c640b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0c640f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8c6416, 33 },
    { 0xd47c6416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376c641c, 91 },
    { 0, 0 },
    { 0x12c641e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5c6422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4c6425, 95 },
    { 0x360c6425, 55 },
    { 0, 0 },
    { 0xc8ac6428, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c642e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4c6433, 78 },
    { 0xc31c6433, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bac6437, 61 },
    { 0xff7c6438, 90 },
    { 0x69c6439, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899c643d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651c6442, 34 },
    { 0, 0 },
    { 0xddcc6444, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefec644a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c644e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147c6454, 76 },
    { 0x286c6455, 91 },
    { 0x666c6455, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x350c6459, 33 },
    { 0x59cc6459, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41c645e, 22 },
    { 0x2fec645f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc6462, 33 },
    { 0xfdbc6462, 76 },
    { 0xa35c6464, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fc646a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19c646d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03c6471, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ac6474, 65 },
    { 0xb62c6474, 76 },
    { 0x9e9c6475, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dc647c, 55 },
    { 0, 0 },
    { 0x359c647e, 78 },
    { 0, 0 },
    { 0x5f2c6480, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc6484, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x306c6487, 61 },
    { 0x74ac6487, 46 },
    { 0xf87c6488, 90 },
    { 0xc90c6487, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cec648d, 65 },
    { 0, 0 },
    { 0x5fbc648f, 66 },
    { 0x431c6490, 43 },
    { 0x920c648f, 36 },
    { 0xdcdc6492, 65 },
    { 0x681c6493, 90 },
    { 0xba0c6492, 98 },
    { 0, 0 },
    { 0xe45c6496, 34 },
    { 0x1fbc6497, 55 },
    { 0xc5ac6498, 34 },
    { 0x3e2c6498, 87 },
    { 0xe67c649a, 90 },
    { 0, 0 },
    { 0x2f9c649c, 33 },
    { 0x58ec649d, 33 },
    { 0x73bc649e, 77 },
    { 0x1bc649f, 43 },
    { 0x848c64a0, 46 },
    { 0xc1bc649c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48c64a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1c64ad, 84 },
    { 0x54ec64ae, 61 },
    { 0, 0 },
    { 0x75fc64b0, 34 },
    { 0, 0 },
    { 0xc2dc64b2, 61 },
    { 0x50c64b3, 85 },
    { 0xfbcc64b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11c64b7, 61 },
    { 0, 0 },
    { 0x89ac64b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc64bc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x847c64bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aec64c2, 90 },
    { 0xa5ec64c2, 33 },
    { 0xa9fc64c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8c64c7, 90 },
    { 0x3dec64c7, 61 },
    { 0x984c64c7, 61 },
    { 0xf19c64c8, 23 },
    { 0x169c64cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64c64d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343c64d6, 90 },
    { 0x9a5c64d7, 90 },
    { 0x846c64d7, 61 },
    { 0, 0 },
    { 0x7f1c64da, 34 },
    { 0, 0 },
    { 0x653c64dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec64e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6c64e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969c64eb, 61 },
    { 0, 0 },
    { 0xedac64ed, 98 },
    { 0x23bc64ed, 33 },
    { 0, 0 },
    { 0xdb7c64f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41c64f8, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0c6500, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26c650a, 98 },
    { 0x702c650a, 34 },
    { 0xadac650a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fc6511, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ac6518, 95 },
    { 0x20ec6518, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecc651d, 90 },
    { 0xdbdc651d, 33 },
    { 0x76cc651f, 98 },
    { 0x934c651d, 90 },
    { 0xe1ac651e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ec6526, 46 },
    { 0xfcdc6526, 96 },
    { 0xf56c6526, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eec652f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cc6532, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aec6536, 43 },
    { 0x3ec6537, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ac653a, 50 },
    { 0x16ac653a, 61 },
    { 0xb60c653b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7c653f, 36 },
    { 0x719c6540, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fc6546, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40c654a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c654d, 84 },
    { 0, 0 },
    { 0xac9c654f, 43 },
    { 0, 0 },
    { 0xec5c6551, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1c6556, 36 },
    { 0x63fc6557, 55 },
    { 0x132c6558, 33 },
    { 0x4b6c6559, 95 },
    { 0x608c655a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ec6564, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c6568, 43 },
    { 0xed0c6569, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfac656f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5c6573, 90 },
    { 0x352c6573, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afc6577, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c657a, 91 },
    { 0, 0 },
    { 0xa81c657c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6c6583, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec6587, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fc658b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c658e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04c6594, 33 },
    { 0, 0 },
    { 0x2cbc6596, 78 },
    { 0xaddc6597, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9c659b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbc659f, 90 },
    { 0x400c659f, 61 },
    { 0xe0c65a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801c65a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cc65a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c65af, 76 },
    { 0xfbfc65b0, 46 },
    { 0x53dc65b0, 95 },
    { 0, 0 },
    { 0x8bfc65b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927c65b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750c65bd, 33 },
    { 0x259c65be, 55 },
    { 0, 0 },
    { 0x307c65c0, 93 },
    { 0xe01c65c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5c65ca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92c65cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715c65d5, 90 },
    { 0xd3ec65d6, 80 },
    { 0, 0 },
    { 0xefec65d8, 95 },
    { 0x5d9c65d9, 55 },
    { 0x26ec65da, 52 },
    { 0x187c65db, 93 },
    { 0x5afc65dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x743c65df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x386c65e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c65e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc65e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50c65f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70c65f7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c65fe, 61 },
    { 0x4d7c65ff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48c6602, 66 },
    { 0x19ac6602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x746c6606, 98 },
    { 0x52bc6606, 90 },
    { 0x121c6608, 90 },
    { 0x442c6609, 23 },
    { 0xa01c6608, 55 },
    { 0x2d6c6609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc660f, 90 },
    { 0xb9dc6610, 33 },
    { 0, 0 },
    { 0x442c6612, 23 },
    { 0x99bc6613, 33 },
    { 0x5e1c6613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bc6619, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c6620, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cc662e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fc6634, 90 },
    { 0xd51c6635, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5c6638, 43 },
    { 0, 0 },
    { 0x183c663a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cc663d, 90 },
    { 0xafac663e, 90 },
    { 0xa9bc663e, 67 },
    { 0x5b2c663d, 17 },
    { 0x143c6641, 76 },
    { 0xdf0c6641, 61 },
    { 0x84fc6643, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8c6648, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464c664c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dc6650, 61 },
    { 0, 0 },
    { 0x33fc6652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbc6657, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251c665b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc665e, 100 },
    { 0x9ffc665f, 99 },
    { 0xd4fc6660, 85 },
    { 0x315c6661, 90 },
    { 0x305c6662, 61 },
    { 0, 0 },
    { 0xc11c6664, 90 },
    { 0, 0 },
    { 0x99fc6666, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8c666a, 91 },
    { 0, 0 },
    { 0x7fbc666c, 33 },
    { 0, 0 },
    { 0x7b4c666e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856c6673, 61 },
    { 0x4eec6674, 34 },
    { 0x89cc6674, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8c667b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cc6682, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fc668b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c668e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac6693, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c6698, 61 },
    { 0xc87c6699, 61 },
    { 0, 0 },
    { 0xcbac669b, 34 },
    { 0, 0 },
    { 0x256c669d, 23 },
    { 0xaf3c669e, 55 },
    { 0xd24c669d, 84 },
    { 0, 0 },
    { 0x4e4c66a1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c66a4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554c66a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28c66ae, 95 },
    { 0xf1bc66af, 61 },
    { 0x83bc66af, 61 },
    { 0xa4c66b1, 34 },
    { 0x5f8c66ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c66b5, 90 },
    { 0x326c66b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32c66bb, 95 },
    { 0xd0fc66bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934c66c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6c66c6, 23 },
    { 0x5f2c66c6, 61 },
    { 0, 0 },
    { 0xa5cc66c9, 76 },
    { 0xc09c66c9, 98 },
    { 0, 0 },
    { 0xf4bc66cc, 95 },
    { 0, 0 },
    { 0xae7c66ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bac66d5, 90 },
    { 0xbd5c66d6, 98 },
    { 0x22bc66d7, 66 },
    { 0x5c9c66d8, 65 },
    { 0xd01c66d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bc66e0, 43 },
    { 0x2a0c66e1, 61 },
    { 0xc44c66e1, 100 },
    { 0x7bc66e3, 91 },
    { 0xa4fc66e4, 90 },
    { 0, 0 },
    { 0xc45c66e6, 66 },
    { 0x7dec66e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dc66ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05c66ee, 66 },
    { 0xafac66ef, 61 },
    { 0x902c66f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7c66f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dc66fb, 46 },
    { 0, 0 },
    { 0xf17c66fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bac6704, 33 },
    { 0, 0 },
    { 0xf4ec6706, 95 },
    { 0xd46c6707, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44c670b, 78 },
    { 0x545c670c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fc6710, 33 },
    { 0x43cc6711, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeac6715, 85 },
    { 0x80dc6715, 61 },
    { 0, 0 },
    { 0x810c6718, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bc671e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c6725, 95 },
    { 0, 0 },
    { 0x90cc6727, 90 },
    { 0x8ccc6727, 55 },
    { 0xe8bc6729, 17 },
    { 0x438c672a, 33 },
    { 0, 0 },
    { 0x647c672c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc6736, 61 },
    { 0x6eac6736, 88 },
    { 0x2e6c6738, 95 },
    { 0xdb3c6739, 34 },
    { 0xfc5c673a, 40 },
    { 0x34dc673b, 61 },
    { 0x952c6738, 66 },
    { 0x3b0c673c, 76 },
    { 0xe2c673e, 95 },
    { 0x5bfc673f, 90 },
    { 0xd53c673f, 90 },
    { 0x790c6741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833c6745, 33 },
    { 0, 0 },
    { 0x87ac6747, 88 },
    { 0, 0 },
    { 0xdf9c6749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6c674e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256c6755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3c6758, 38 },
    { 0x919c6759, 55 },
    { 0x823c6758, 95 },
    { 0x1a7c6758, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dc675f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6c6766, 95 },
    { 0xcd3c6767, 90 },
    { 0, 0 },
    { 0x132c6769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bac677c, 90 },
    { 0, 0 },
    { 0x94dc677e, 34 },
    { 0, 0 },
    { 0xdd2c6780, 72 },
    { 0, 0 },
    { 0xd9fc6782, 84 },
    { 0x177c6783, 93 },
    { 0x344c6784, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7c6788, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2c678c, 33 },
    { 0xe8dc678c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fc6790, 76 },
    { 0xa1bc6790, 46 },
    { 0x645c6792, 61 },
    { 0, 0 },
    { 0x859c6794, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c6797, 76 },
    { 0, 0 },
    { 0x9a7c6799, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bc679d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac67a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12c67a4, 61 },
    { 0, 0 },
    { 0xf6ec67a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722c67ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac67b5, 95 },
    { 0xa99c67b6, 61 },
    { 0xb72c67b7, 93 },
    { 0xe9bc67b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2c67bd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bec67c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357c67c4, 64 },
    { 0x725c67c4, 90 },
    { 0xd0c67c4, 34 },
    { 0x2c9c67c7, 95 },
    { 0, 0 },
    { 0xc04c67c9, 93 },
    { 0x993c67ca, 43 },
    { 0xed3c67cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5c67ce, 90 },
    { 0, 0 },
    { 0x268c67d0, 81 },
    { 0x2aac67d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c67d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc67de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5c67e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c67e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31c67eb, 61 },
    { 0x71c67ec, 95 },
    { 0xc85c67ed, 55 },
    { 0x5adc67ee, 23 },
    { 0x28ac67ef, 34 },
    { 0, 0 },
    { 0xc8fc67f1, 95 },
    { 0, 0 },
    { 0x4b2c67f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bc67f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edc67fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5c6801, 33 },
    { 0x453c6801, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23c6806, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91c6809, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cc6812, 48 },
    { 0x7ec6813, 97 },
    { 0x482c6813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ec6818, 43 },
    { 0xe3cc6819, 76 },
    { 0xbf1c681a, 61 },
    { 0x503c681b, 66 },
    { 0, 0 },
    { 0x350c681d, 46 },
    { 0, 0 },
    { 0x808c681f, 85 },
    { 0x661c6820, 55 },
    { 0x6c2c681f, 76 },
    { 0x1bac6822, 61 },
    { 0, 0 },
    { 0xd9dc6824, 43 },
    { 0, 0 },
    { 0xa2cc6826, 65 },
    { 0, 0 },
    { 0xef7c6828, 33 },
    { 0xc4ec6829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71c6831, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265c6837, 98 },
    { 0xeacc6838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cc6843, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527c684e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ac6851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8c685c, 65 },
    { 0, 0 },
    { 0x4f5c685e, 95 },
    { 0x49ac685f, 90 },
    { 0xde6c6860, 43 },
    { 0x497c6861, 34 },
    { 0xec2c6861, 90 },
    { 0xe2ac6861, 23 },
    { 0x9ec6864, 23 },
    { 0xa8cc6861, 82 },
    { 0x17dc6865, 33 },
    { 0xda8c6867, 66 },
    { 0x4dc6868, 61 },
    { 0xf0fc6869, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaac686d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x362c6870, 85 },
    { 0x5d2c6871, 61 },
    { 0, 0 },
    { 0x3e1c6873, 100 },
    { 0x4fdc6874, 33 },
    { 0x581c6875, 23 },
    { 0x3e3c6873, 98 },
    { 0x62ac6877, 100 },
    { 0x4c6c6876, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c6880, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x977c6883, 55 },
    { 0xb27c6883, 61 },
    { 0, 0 },
    { 0x35fc6886, 53 },
    { 0x1f0c6886, 95 },
    { 0x382c6888, 61 },
    { 0x458c6889, 91 },
    { 0xbd6c688a, 95 },
    { 0, 0 },
    { 0x3a9c688c, 76 },
    { 0, 0 },
    { 0xbc688e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8c6891, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889c6896, 23 },
    { 0, 0 },
    { 0x56ac6898, 55 },
    { 0, 0 },
    { 0xe06c689a, 95 },
    { 0x35ec689b, 61 },
    { 0xca7c689b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146c68a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45c68b0, 95 },
    { 0, 0 },
    { 0x88dc68b2, 61 },
    { 0xbd0c68b2, 76 },
    { 0xa4ac68b4, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212c68b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7c68bd, 61 },
    { 0x16fc68be, 23 },
    { 0, 0 },
    { 0xc75c68c0, 76 },
    { 0x94c68c1, 46 },
    { 0, 0 },
    { 0xdd3c68c3, 61 },
    { 0, 0 },
    { 0xbf4c68c5, 78 },
    { 0xc9dc68c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609c68cc, 76 },
    { 0xf51c68cd, 90 },
    { 0xf51c68cc, 95 },
    { 0x476c68cd, 37 },
    { 0, 0 },
    { 0xa9ac68d1, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x799c68d4, 90 },
    { 0, 0 },
    { 0xcabc68d6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8c68d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbc68dc, 90 },
    { 0, 0 },
    { 0xfa5c68de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4c68e2, 90 },
    { 0, 0 },
    { 0x5c2c68e4, 55 },
    { 0xb02c68e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9c68e9, 55 },
    { 0x53bc68ea, 43 },
    { 0, 0 },
    { 0xfa5c68ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc68f2, 68 },
    { 0xcd2c68f2, 90 },
    { 0, 0 },
    { 0x270c68f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x825c68f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fc68fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8c6908, 20 },
    { 0, 0 },
    { 0x45bc690a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136c6912, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6c691b, 55 },
    { 0x4cac691c, 98 },
    { 0xd4c691d, 99 },
    { 0x8edc691c, 61 },
    { 0x6a1c691d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9c6924, 95 },
    { 0xaedc6925, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c692b, 90 },
    { 0x465c692c, 66 },
    { 0x3edc692d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17c6932, 55 },
    { 0x550c6933, 33 },
    { 0xfe6c6933, 55 },
    { 0, 0 },
    { 0xf4ac6936, 61 },
    { 0xa37c6937, 69 },
    { 0xf59c6938, 56 },
    { 0, 0 },
    { 0x921c693a, 100 },
    { 0, 0 },
    { 0x4cbc693c, 88 },
    { 0, 0 },
    { 0x1cac693e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c6946, 43 },
    { 0xef5c6946, 98 },
    { 0x307c6948, 93 },
    { 0, 0 },
    { 0xb0ac694a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2c6952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ac695d, 100 },
    { 0x79c695e, 88 },
    { 0, 0 },
    { 0x86fc6960, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3c6966, 88 },
    { 0, 0 },
    { 0x2cec6968, 33 },
    { 0, 0 },
    { 0x4f5c696a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cc6971, 87 },
    { 0, 0 },
    { 0xbe3c6973, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967c6977, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbc6984, 23 },
    { 0x7ccc6984, 95 },
    { 0x718c6986, 90 },
    { 0xc00c6987, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edc698b, 52 },
    { 0xb2cc698c, 91 },
    { 0x574c698b, 95 },
    { 0, 0 },
    { 0x236c698f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ec6992, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168c6999, 55 },
    { 0, 0 },
    { 0x552c699b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc699e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ec69a3, 43 },
    { 0, 0 },
    { 0xe4ec69a5, 38 },
    { 0, 0 },
    { 0xf22c69a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ec69aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894c69b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c69b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9c69bd, 33 },
    { 0xc6bc69be, 55 },
    { 0xfd3c69bf, 61 },
    { 0x90dc69c0, 93 },
    { 0x980c69c1, 90 },
    { 0x6d4c69c1, 98 },
    { 0, 0 },
    { 0xbec69c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641c69c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897c69cc, 55 },
    { 0, 0 },
    { 0xb17c69ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5c69d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54c69d9, 66 },
    { 0xf99c69da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9c69de, 33 },
    { 0xd00c69de, 61 },
    { 0x8f5c69e0, 68 },
    { 0xce4c69e0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adc69e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09c69ef, 55 },
    { 0x9b2c69f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bc69f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c69f9, 23 },
    { 0xb48c69fa, 76 },
    { 0x54ac69fa, 23 },
    { 0x98bc69fc, 87 },
    { 0, 0 },
    { 0xefbc69fe, 76 },
    { 0, 0 },
    { 0xa9c6a00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x297c6a03, 61 },
    { 0x60dc6a04, 61 },
    { 0, 0 },
    { 0x9aac6a06, 100 },
    { 0x420c6a06, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac6a0a, 101 },
    { 0x67dc6a0a, 90 },
    { 0xccdc6a0c, 95 },
    { 0xf7ec6a0d, 61 },
    { 0x2eac6a0d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867c6a13, 78 },
    { 0x56c6a14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2c6a19, 95 },
    { 0x391c6a1a, 90 },
    { 0xa99c6a19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990c6a1f, 46 },
    { 0x3d6c6a20, 65 },
    { 0xc04c6a20, 76 },
    { 0xb65c6a22, 19 },
    { 0x382c6a23, 98 },
    { 0x401c6a24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865c6a2e, 61 },
    { 0xd3dc6a2f, 76 },
    { 0x5a8c6a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263c6a40, 98 },
    { 0xf80c6a40, 34 },
    { 0xa3ac6a42, 43 },
    { 0x50fc6a43, 23 },
    { 0xf46c6a44, 33 },
    { 0xfdcc6a44, 66 },
    { 0xd8cc6a44, 88 },
    { 0x5b4c6a44, 91 },
    { 0x943c6a44, 65 },
    { 0x445c6a49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bc6a4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5c6a50, 42 },
    { 0xc73c6a51, 17 },
    { 0xbf4c6a51, 23 },
    { 0x714c6a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cc6a56, 98 },
    { 0x159c6a57, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7c6a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5c6a61, 33 },
    { 0xb72c6a62, 95 },
    { 0x906c6a61, 100 },
    { 0, 0 },
    { 0xc73c6a65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609c6a6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23c6a70, 98 },
    { 0x85bc6a71, 75 },
    { 0xc23c6a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42c6a7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbc6a7d, 95 },
    { 0, 0 },
    { 0x54bc6a7f, 90 },
    { 0, 0 },
    { 0x90c6a81, 98 },
    { 0, 0 },
    { 0xd5dc6a83, 61 },
    { 0x425c6a83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cec6a87, 99 },
    { 0x832c6a88, 85 },
    { 0x703c6a89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74c6a8d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0c6a92, 95 },
    { 0, 0 },
    { 0xfd1c6a94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ac6a97, 95 },
    { 0, 0 },
    { 0x60ec6a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15c6a9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353c6aa3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0c6aa6, 93 },
    { 0xe3ac6aa7, 87 },
    { 0xa0fc6aa7, 95 },
    { 0xce0c6aa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fc6aaf, 76 },
    { 0xd1bc6ab0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5c6ab3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c6ab6, 91 },
    { 0x7f6c6ab7, 23 },
    { 0xfa2c6ab8, 61 },
    { 0x874c6ab7, 61 },
    { 0xad2c6aba, 33 },
    { 0xe80c6aba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502c6abf, 23 },
    { 0, 0 },
    { 0xb7cc6ac1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4c6ac4, 84 },
    { 0x1f8c6ac4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ec6ace, 95 },
    { 0, 0 },
    { 0xfdfc6ad0, 46 },
    { 0xc13c6ad1, 90 },
    { 0xeac6ad1, 33 },
    { 0, 0 },
    { 0x8e2c6ad4, 87 },
    { 0, 0 },
    { 0x543c6ad6, 90 },
    { 0xed4c6ad7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efc6add, 100 },
    { 0, 0 },
    { 0x388c6adf, 33 },
    { 0x18c6ae0, 90 },
    { 0x25dc6ae1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffc6ae4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ec6ae9, 90 },
    { 0x8e8c6aea, 98 },
    { 0, 0 },
    { 0xb9dc6aec, 90 },
    { 0, 0 },
    { 0x44c6aee, 76 },
    { 0x4bcc6aef, 76 },
    { 0xad0c6af0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6c6af3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ec6af6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1c6afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37c6b00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ac6b09, 34 },
    { 0, 0 },
    { 0x17fc6b0b, 68 },
    { 0x9f7c6b0c, 55 },
    { 0, 0 },
    { 0x127c6b0e, 61 },
    { 0x962c6b0f, 23 },
    { 0x209c6b10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c6b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fc6b16, 98 },
    { 0x2b2c6b17, 61 },
    { 0xd71c6b18, 34 },
    { 0x99bc6b17, 68 },
    { 0x63ec6b16, 65 },
    { 0x7e9c6b1b, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64c6b1e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecc6b21, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fc6b28, 17 },
    { 0x2b1c6b29, 95 },
    { 0x67fc6b28, 78 },
    { 0x16ec6b2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc6b2e, 85 },
    { 0x3d7c6b2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521c6b33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffc6b3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc6b40, 95 },
    { 0xa41c6b41, 90 },
    { 0x85c6b41, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89c6b49, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abc6b51, 98 },
    { 0, 0 },
    { 0x2adc6b53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1c6b59, 93 },
    { 0, 0 },
    { 0x266c6b5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdc6b66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddc6b6c, 77 },
    { 0x9b3c6b6d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bc6b72, 23 },
    { 0x3f9c6b73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439c6b77, 76 },
    { 0x5bec6b77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0c6b7d, 23 },
    { 0x3a0c6b7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71c6b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3c6b89, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c6b8c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8c6b8f, 85 },
    { 0x97ec6b8f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dc6b95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x112c6b98, 23 },
    { 0xce2c6b99, 65 },
    { 0x89bc6b99, 91 },
    { 0x7d0c6b9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bc6b9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdc6ba5, 70 },
    { 0, 0 },
    { 0xa09c6ba7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4c6bb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64c6bb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c6bba, 66 },
    { 0x733c6bba, 33 },
    { 0, 0 },
    { 0x484c6bbd, 33 },
    { 0xeffc6bbe, 76 },
    { 0, 0 },
    { 0x9f2c6bc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778c6bc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eac6bcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4c6bd9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cc6bdc, 91 },
    { 0xc55c6bdc, 55 },
    { 0xd45c6bdc, 75 },
    { 0, 0 },
    { 0x3a0c6be0, 52 },
    { 0, 0 },
    { 0x820c6be2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6c6be6, 33 },
    { 0xf3cc6be7, 77 },
    { 0xceec6be6, 52 },
    { 0x3e3c6be9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc6bed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87c6bf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbec6bf5, 23 },
    { 0xb7cc6bf6, 53 },
    { 0x439c6bf6, 95 },
    { 0, 0 },
    { 0x1cbc6bf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525c6c00, 61 },
    { 0x7ac6c01, 61 },
    { 0x993c6c02, 61 },
    { 0xce9c6c03, 23 },
    { 0xdeec6c03, 33 },
    { 0x96dc6c05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225c6c0a, 23 },
    { 0, 0 },
    { 0x57fc6c0c, 33 },
    { 0x58ec6c0d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4c6c10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x403c6c13, 61 },
    { 0xa2bc6c14, 99 },
    { 0, 0 },
    { 0xbd9c6c16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ec6c19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfc6c21, 23 },
    { 0, 0 },
    { 0x3f3c6c23, 78 },
    { 0, 0 },
    { 0xa55c6c25, 61 },
    { 0x693c6c26, 17 },
    { 0, 0 },
    { 0x89fc6c28, 43 },
    { 0x987c6c29, 52 },
    { 0x394c6c2a, 76 },
    { 0xb7c6c2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cc6c2f, 53 },
    { 0xcc6c30, 91 },
    { 0xbc2c6c31, 55 },
    { 0x3d6c6c30, 78 },
    { 0x26ec6c33, 90 },
    { 0xdadc6c33, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9c6c37, 76 },
    { 0xf86c6c37, 61 },
    { 0xef5c6c39, 76 },
    { 0x567c6c3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198c6c40, 76 },
    { 0x52c6c41, 58 },
    { 0x8e5c6c40, 90 },
    { 0, 0 },
    { 0x2e9c6c44, 95 },
    { 0x85cc6c45, 90 },
    { 0, 0 },
    { 0xb26c6c47, 90 },
    { 0xccbc6c48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbc6c51, 75 },
    { 0xb9dc6c52, 98 },
    { 0xc5c6c53, 40 },
    { 0x817c6c54, 61 },
    { 0x396c6c54, 23 },
    { 0x95fc6c56, 90 },
    { 0xbb1c6c57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x685c6c5a, 81 },
    { 0xbecc6c5a, 95 },
    { 0x989c6c5c, 90 },
    { 0xfeac6c5d, 99 },
    { 0x162c6c5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8c6c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffc6c6e, 95 },
    { 0xc91c6c6f, 33 },
    { 0x2c3c6c70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dc6c73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddc6c7b, 55 },
    { 0x7a0c6c7b, 33 },
    { 0, 0 },
    { 0x106c6c7e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c6c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61c6c87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3c6c8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6c6c90, 55 },
    { 0xa50c6c91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dc6c98, 33 },
    { 0x1d1c6c99, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87c6c9c, 83 },
    { 0xa91c6c9c, 55 },
    { 0x86fc6c9c, 43 },
    { 0x134c6c9f, 34 },
    { 0x4c3c6ca0, 55 },
    { 0xc2c6ca1, 90 },
    { 0x66bc6c9f, 33 },
    { 0, 0 },
    { 0x623c6ca4, 95 },
    { 0x1ddc6ca5, 94 },
    { 0xdcdc6ca6, 98 },
    { 0xeebc6ca7, 95 },
    { 0x68bc6ca8, 76 },
    { 0x72c6ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ec6cac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8c6caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ec6cb5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc6cb8, 75 },
    { 0, 0 },
    { 0x8c2c6cba, 90 },
    { 0xd5ac6cba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46c6cbf, 34 },
    { 0xe33c6cc0, 97 },
    { 0x115c6cc1, 66 },
    { 0x103c6cc2, 33 },
    { 0, 0 },
    { 0xcfdc6cc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1c6ccd, 87 },
    { 0x34fc6cce, 93 },
    { 0, 0 },
    { 0x1dc6cd0, 33 },
    { 0, 0 },
    { 0x273c6cd2, 98 },
    { 0xb1cc6cd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeec6cd8, 98 },
    { 0xc1ec6cd8, 61 },
    { 0xba8c6cda, 33 },
    { 0xc32c6cda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155c6cdf, 92 },
    { 0x2d7c6cdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afc6ce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ec6ce8, 78 },
    { 0xc23c6ce9, 55 },
    { 0x2dbc6cea, 95 },
    { 0x47cc6cea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3c6cf4, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacc6cfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fc6cfe, 80 },
    { 0x83ec6cff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266c6d03, 87 },
    { 0xc4bc6d04, 98 },
    { 0x2c3c6d05, 93 },
    { 0, 0 },
    { 0xe05c6d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9c6d0b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbc6d0e, 91 },
    { 0, 0 },
    { 0x8f3c6d10, 88 },
    { 0x94ac6d10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabc6d15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbec6d1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393c6d1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ac6d24, 61 },
    { 0, 0 },
    { 0x5bec6d26, 77 },
    { 0xf21c6d27, 84 },
    { 0xb0fc6d26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1c6d2b, 23 },
    { 0x1b6c6d2c, 76 },
    { 0xde9c6d2b, 33 },
    { 0x9dec6d2d, 77 },
    { 0xfa2c6d2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cc6d35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885c6d39, 91 },
    { 0xfb5c6d3a, 96 },
    { 0x30cc6d3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0c6d4b, 98 },
    { 0x38ec6d4c, 98 },
    { 0, 0 },
    { 0x362c6d4e, 23 },
    { 0xa22c6d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27c6d54, 95 },
    { 0xe42c6d55, 95 },
    { 0, 0 },
    { 0xfb3c6d57, 33 },
    { 0, 0 },
    { 0x3c6d59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1c6d5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872c6d62, 98 },
    { 0xd5ac6d63, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406c6d6a, 52 },
    { 0x1c8c6d6a, 93 },
    { 0xa9dc6d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885c6d70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecc6d73, 65 },
    { 0, 0 },
    { 0xd7dc6d75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521c6d79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ec6d85, 98 },
    { 0xec8c6d86, 47 },
    { 0, 0 },
    { 0x535c6d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fc6d8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc6d8f, 40 },
    { 0, 0 },
    { 0x78bc6d91, 61 },
    { 0x6edc6d92, 90 },
    { 0, 0 },
    { 0xd07c6d94, 75 },
    { 0x337c6d94, 43 },
    { 0, 0 },
    { 0x6cac6d97, 46 },
    { 0x3b1c6d97, 100 },
    { 0x5e5c6d99, 33 },
    { 0, 0 },
    { 0xe90c6d9b, 85 },
    { 0x78ec6d9c, 98 },
    { 0x7cbc6d9c, 55 },
    { 0x6bcc6d9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148c6da8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bc6dac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95c6db0, 90 },
    { 0x407c6db1, 55 },
    { 0x8cc6db2, 43 },
    { 0xbfac6db3, 66 },
    { 0xd42c6db4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcc6db7, 99 },
    { 0, 0 },
    { 0x352c6db9, 90 },
    { 0, 0 },
    { 0x9d0c6dbb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26c6dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5c6dc1, 91 },
    { 0x276c6dc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffc6dc5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4c6dc8, 23 },
    { 0, 0 },
    { 0xc57c6dca, 90 },
    { 0x73ac6dcb, 96 },
    { 0x5ddc6dcc, 67 },
    { 0xcbfc6dcd, 36 },
    { 0x184c6dce, 76 },
    { 0x1e8c6dcf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x719c6dd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c6dd7, 76 },
    { 0x983c6dd8, 98 },
    { 0xddcc6dd8, 33 },
    { 0x717c6dd8, 61 },
    { 0x3c3c6ddb, 23 },
    { 0x5c6c6dda, 98 },
    { 0x146c6dda, 99 },
    { 0xe33c6dde, 67 },
    { 0x2cac6ddf, 66 },
    { 0, 0 },
    { 0x44fc6de1, 80 },
    { 0x465c6de1, 87 },
    { 0, 0 },
    { 0xfbec6de4, 52 },
    { 0, 0 },
    { 0xca1c6de6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181c6dee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c6df3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fc6df8, 48 },
    { 0xf8cc6df9, 61 },
    { 0xc51c6dfa, 34 },
    { 0xacec6dfb, 33 },
    { 0x22ec6dfa, 95 },
    { 0x646c6dfc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c6e02, 33 },
    { 0, 0 },
    { 0xae1c6e04, 61 },
    { 0, 0 },
    { 0xd4ac6e06, 88 },
    { 0x4b8c6e07, 100 },
    { 0x391c6e08, 76 },
    { 0x9a7c6e09, 90 },
    { 0xe78c6e0a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47c6e0d, 23 },
    { 0xeefc6e0e, 95 },
    { 0xdf8c6e0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbc6e12, 17 },
    { 0x272c6e12, 33 },
    { 0x11fc6e14, 90 },
    { 0, 0 },
    { 0xff0c6e16, 67 },
    { 0x3d6c6e16, 75 },
    { 0x522c6e18, 76 },
    { 0xb4dc6e19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14c6e20, 98 },
    { 0, 0 },
    { 0xd4ac6e22, 90 },
    { 0, 0 },
    { 0x146c6e24, 61 },
    { 0xc4dc6e24, 55 },
    { 0x7a6c6e25, 33 },
    { 0x728c6e24, 34 },
    { 0, 0 },
    { 0x3fbc6e29, 69 },
    { 0, 0 },
    { 0xba8c6e2b, 76 },
    { 0x21dc6e2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bc6e2f, 55 },
    { 0xb3dc6e2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x237c6e33, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcc6e36, 95 },
    { 0xac6c6e37, 23 },
    { 0xa6c6e37, 77 },
    { 0x8e5c6e39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeec6e3c, 90 },
    { 0x92c6e3d, 55 },
    { 0x5ebc6e3e, 90 },
    { 0x539c6e3f, 55 },
    { 0x72dc6e40, 98 },
    { 0x54dc6e3d, 46 },
    { 0x907c6e3e, 23 },
    { 0x952c6e43, 61 },
    { 0xddbc6e42, 23 },
    { 0x906c6e41, 90 },
    { 0x119c6e46, 95 },
    { 0xef8c6e47, 84 },
    { 0xb5dc6e48, 56 },
    { 0x21bc6e49, 33 },
    { 0xea3c6e43, 43 },
    { 0, 0 },
    { 0x5f9c6e4c, 84 },
    { 0x627c6e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bec6e50, 90 },
    { 0xe4cc6e51, 90 },
    { 0, 0 },
    { 0x18cc6e53, 95 },
    { 0, 0 },
    { 0x497c6e55, 78 },
    { 0, 0 },
    { 0xc38c6e57, 34 },
    { 0x302c6e57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcc6e5b, 61 },
    { 0x64fc6e5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c6e62, 55 },
    { 0xfc2c6e63, 67 },
    { 0x5c4c6e63, 90 },
    { 0x6dbc6e65, 76 },
    { 0x584c6e66, 98 },
    { 0x12bc6e67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ac6e6b, 23 },
    { 0x54ac6e6c, 99 },
    { 0, 0 },
    { 0xe2ac6e6e, 23 },
    { 0xd1dc6e6f, 33 },
    { 0, 0 },
    { 0xf6ac6e71, 75 },
    { 0x770c6e72, 66 },
    { 0xbb7c6e72, 95 },
    { 0, 0 },
    { 0x350c6e75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0c6e78, 46 },
    { 0xc88c6e78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793c6e7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460c6e87, 95 },
    { 0, 0 },
    { 0x18cc6e89, 90 },
    { 0xd73c6e8a, 96 },
    { 0x266c6e8a, 99 },
    { 0, 0 },
    { 0x95ec6e8d, 93 },
    { 0x2f9c6e8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60c6e94, 23 },
    { 0xa07c6e95, 33 },
    { 0x7c3c6e94, 95 },
    { 0x5c9c6e96, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512c6e9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x652c6e9e, 93 },
    { 0x7e7c6e9f, 90 },
    { 0xd1dc6ea0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fec6ea5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648c6eae, 33 },
    { 0xb87c6eaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eec6eb8, 34 },
    { 0x325c6eb9, 87 },
    { 0x715c6eb9, 61 },
    { 0xb39c6ebb, 90 },
    { 0, 0 },
    { 0x58c6ebd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5c6ec0, 33 },
    { 0, 0 },
    { 0xa1fc6ec2, 33 },
    { 0x2d3c6ec3, 95 },
    { 0x6fac6ec3, 95 },
    { 0xf13c6ec5, 33 },
    { 0x433c6ec6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501c6ecd, 46 },
    { 0, 0 },
    { 0xe16c6ecf, 76 },
    { 0, 0 },
    { 0x76bc6ed1, 88 },
    { 0x28dc6ed1, 61 },
    { 0x6a3c6ed3, 76 },
    { 0xc1cc6ed4, 33 },
    { 0, 0 },
    { 0x3a5c6ed6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31c6edb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecc6ee0, 61 },
    { 0x62fc6ee1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94c6ee5, 90 },
    { 0, 0 },
    { 0x9b2c6ee7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1c6eea, 95 },
    { 0x1dcc6eea, 90 },
    { 0x99ac6eea, 33 },
    { 0, 0 },
    { 0xb95c6eee, 33 },
    { 0x59dc6eee, 91 },
    { 0x548c6ef0, 95 },
    { 0x730c6ef1, 90 },
    { 0xb2ec6ef2, 98 },
    { 0, 0 },
    { 0xffcc6ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c6ef7, 12 },
    { 0xecfc6ef8, 61 },
    { 0xa2ec6ef9, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x888c6efc, 98 },
    { 0, 0 },
    { 0x452c6efe, 98 },
    { 0x1c6efe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efc6f02, 76 },
    { 0x2f6c6f02, 90 },
    { 0x6b9c6f03, 67 },
    { 0, 0 },
    { 0xaf9c6f06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x564c6f09, 90 },
    { 0x3edc6f09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bc6f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9c6f11, 46 },
    { 0x773c6f12, 56 },
    { 0x912c6f13, 75 },
    { 0xae9c6f14, 93 },
    { 0x21fc6f14, 61 },
    { 0xe5dc6f13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fc6f1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882c6f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448c6f2c, 34 },
    { 0x807c6f2d, 90 },
    { 0x181c6f2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfc6f32, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcec6f37, 84 },
    { 0x168c6f38, 95 },
    { 0x1acc6f38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7c6f45, 93 },
    { 0xfccc6f45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c6f49, 55 },
    { 0xb94c6f49, 90 },
    { 0x5d6c6f49, 40 },
    { 0x6c1c6f4a, 23 },
    { 0x396c6f4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcc6f53, 34 },
    { 0xf6c6f54, 46 },
    { 0x2b7c6f55, 61 },
    { 0xbc5c6f56, 90 },
    { 0x6c6f57, 95 },
    { 0xd0ac6f54, 33 },
    { 0, 0 },
    { 0x544c6f5a, 23 },
    { 0xb84c6f5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703c6f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbc6f66, 46 },
    { 0, 0 },
    { 0xa21c6f68, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978c6f6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470c6f71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aec6f7a, 85 },
    { 0x5c9c6f7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afc6f7e, 53 },
    { 0, 0 },
    { 0xb19c6f80, 33 },
    { 0x389c6f80, 55 },
    { 0, 0 },
    { 0xb1fc6f83, 90 },
    { 0x87bc6f83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cec6f87, 95 },
    { 0x6f0c6f87, 43 },
    { 0, 0 },
    { 0xcb2c6f8a, 78 },
    { 0x515c6f8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fc6f90, 84 },
    { 0xb56c6f91, 34 },
    { 0, 0 },
    { 0xed3c6f93, 90 },
    { 0x5b0c6f94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46c6f99, 33 },
    { 0, 0 },
    { 0xd8c6f9b, 43 },
    { 0xe2ec6f9b, 90 },
    { 0, 0 },
    { 0xbfec6f9e, 76 },
    { 0, 0 },
    { 0xb0bc6fa0, 61 },
    { 0xa31c6fa0, 76 },
    { 0xcb0c6fa2, 84 },
    { 0x244c6fa1, 99 },
    { 0x212c6fa4, 95 },
    { 0xd3fc6fa5, 52 },
    { 0x5b3c6fa2, 61 },
    { 0xd1fc6fa7, 90 },
    { 0, 0 },
    { 0x111c6fa9, 34 },
    { 0, 0 },
    { 0xbb3c6fab, 61 },
    { 0x676c6fab, 98 },
    { 0, 0 },
    { 0xaf4c6fae, 98 },
    { 0, 0 },
    { 0x8b9c6fb0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2c6fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dc6fb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664c6fc0, 95 },
    { 0x343c6fc1, 33 },
    { 0x78bc6fc0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x806c6fc5, 40 },
    { 0xb4fc6fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dc6fcc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cc6fd0, 78 },
    { 0, 0 },
    { 0x282c6fd2, 95 },
    { 0, 0 },
    { 0x291c6fd4, 100 },
    { 0, 0 },
    { 0xcaec6fd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914c6fdc, 33 },
    { 0xfcc6fdd, 98 },
    { 0xe39c6fdd, 95 },
    { 0x188c6fdf, 87 },
    { 0x3bfc6fe0, 90 },
    { 0, 0 },
    { 0xf53c6fe2, 76 },
    { 0xc43c6fe2, 34 },
    { 0x106c6fe4, 90 },
    { 0x6cbc6fe5, 61 },
    { 0xdabc6fe5, 23 },
    { 0xbbec6fe7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dc6fea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524c6fef, 43 },
    { 0, 0 },
    { 0xb13c6ff1, 78 },
    { 0x684c6ff2, 98 },
    { 0x4fec6ff2, 90 },
    { 0x871c6ff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfc6ffb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6c6ffe, 75 },
    { 0x677c6fff, 46 },
    { 0, 0 },
    { 0xd22c7001, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfc7005, 46 },
    { 0xe2fc7006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ec700d, 98 },
    { 0xa22c700e, 90 },
    { 0x7a4c700d, 90 },
    { 0xf19c700d, 95 },
    { 0xffec7011, 98 },
    { 0x995c7012, 77 },
    { 0x326c700e, 34 },
    { 0xbcac7011, 33 },
    { 0x213c700e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352c7019, 68 },
    { 0, 0 },
    { 0x5ccc701b, 55 },
    { 0x3e4c701c, 90 },
    { 0, 0 },
    { 0xd00c701e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50c7021, 78 },
    { 0x43bc7022, 93 },
    { 0xd1c7022, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690c702a, 61 },
    { 0, 0 },
    { 0x806c702c, 33 },
    { 0x521c702d, 34 },
    { 0xcc6c702e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84c703a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84c703d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cc7040, 33 },
    { 0x78cc7041, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56c7046, 23 },
    { 0x332c7047, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x426c704a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fc704f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832c7053, 61 },
    { 0xcb8c7053, 90 },
    { 0, 0 },
    { 0x1fcc7056, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc705c, 76 },
    { 0, 0 },
    { 0x459c705e, 33 },
    { 0xd6ec705e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xccac7062, 43 },
    { 0, 0 },
    { 0xdd7c7064, 43 },
    { 0xbcac7065, 61 },
    { 0x402c7066, 85 },
    { 0x3fcc7067, 33 },
    { 0x4f4c7066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x958c706b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684c7070, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4c7075, 95 },
    { 0xb73c7076, 84 },
    { 0x7cac7077, 23 },
    { 0, 0 },
    { 0x299c7079, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fec707f, 98 },
    { 0xdefc7080, 95 },
    { 0x14c7081, 98 },
    { 0xdcbc7082, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63c708f, 95 },
    { 0, 0 },
    { 0x25fc7091, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77c709c, 98 },
    { 0xc69c709d, 55 },
    { 0x522c709d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x742c70a1, 34 },
    { 0, 0 },
    { 0x903c70a3, 90 },
    { 0xdf0c70a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6c70a7, 90 },
    { 0x999c70a7, 95 },
    { 0xcbac70a9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29c70ac, 90 },
    { 0x329c70ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92c70b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x530c70b9, 61 },
    { 0x430c70ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2c70bd, 46 },
    { 0xf43c70bd, 98 },
    { 0, 0 },
    { 0x39fc70c0, 91 },
    { 0xd9ec70c0, 78 },
    { 0x2e5c70c0, 93 },
    { 0x109c70c2, 90 },
    { 0xae5c70c4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45c70c8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68c70cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc70ce, 61 },
    { 0x269c70ce, 76 },
    { 0x43ec70ce, 90 },
    { 0xff3c70d1, 98 },
    { 0xf0fc70d1, 34 },
    { 0x4dec70d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ec70d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bc70d9, 98 },
    { 0x9dc70da, 90 },
    { 0x19dc70db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc70df, 90 },
    { 0x9a3c70df, 76 },
    { 0, 0 },
    { 0x216c70e2, 98 },
    { 0, 0 },
    { 0x73bc70e4, 98 },
    { 0, 0 },
    { 0x711c70e6, 55 },
    { 0, 0 },
    { 0xf76c70e8, 34 },
    { 0x922c70e8, 61 },
    { 0xedac70ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x261c70ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0c70f1, 23 },
    { 0xc6dc70f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114c70fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ac70ff, 40 },
    { 0xa48c70ff, 34 },
    { 0x52bc7101, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x388c7104, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47c7109, 59 },
    { 0, 0 },
    { 0xaa9c710b, 99 },
    { 0, 0 },
    { 0x315c710d, 80 },
    { 0, 0 },
    { 0x30dc710f, 34 },
    { 0xee7c7110, 84 },
    { 0x5cdc7111, 98 },
    { 0xac4c7110, 61 },
    { 0x74ec7113, 66 },
    { 0x93ec7110, 98 },
    { 0x9a8c7110, 55 },
    { 0xc7ec7112, 90 },
    { 0, 0 },
    { 0xadac7118, 98 },
    { 0, 0 },
    { 0x7e5c711a, 33 },
    { 0, 0 },
    { 0x7c6c711c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabec7129, 17 },
    { 0x6d1c712a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c712e, 77 },
    { 0xd53c712f, 23 },
    { 0xe7ec7130, 88 },
    { 0, 0 },
    { 0xa3bc7132, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67c7137, 34 },
    { 0x8b5c7137, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcc713c, 52 },
    { 0xa69c713c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ac7143, 98 },
    { 0xd0fc7143, 95 },
    { 0x5e7c7144, 33 },
    { 0xfcec7146, 61 },
    { 0, 0 },
    { 0xcc0c7148, 61 },
    { 0xae6c7149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4c714d, 65 },
    { 0xf50c714d, 98 },
    { 0xf6c714f, 85 },
    { 0, 0 },
    { 0x112c7151, 15 },
    { 0x780c7152, 61 },
    { 0xa21c7153, 34 },
    { 0x287c7154, 33 },
    { 0xd5ac7152, 61 },
    { 0x39ac7151, 90 },
    { 0x248c7153, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cc715b, 76 },
    { 0xea7c715b, 33 },
    { 0, 0 },
    { 0x233c715e, 23 },
    { 0x272c715f, 33 },
    { 0xadec7160, 87 },
    { 0x91dc7161, 95 },
    { 0xa29c7162, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4c7165, 95 },
    { 0x598c7165, 76 },
    { 0, 0 },
    { 0x8f1c7168, 61 },
    { 0, 0 },
    { 0xbc4c716a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6c716d, 93 },
    { 0xb33c716e, 23 },
    { 0, 0 },
    { 0x82ec7170, 78 },
    { 0x828c7171, 55 },
    { 0x3aec7172, 90 },
    { 0xed2c7173, 34 },
    { 0xea1c7174, 55 },
    { 0, 0 },
    { 0x332c7176, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1c7179, 46 },
    { 0x923c717a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x300c717d, 98 },
    { 0x7f9c717e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbc7182, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cc7185, 61 },
    { 0, 0 },
    { 0x410c7187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51c718a, 85 },
    { 0, 0 },
    { 0xb33c718c, 33 },
    { 0xa58c718d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5c7190, 64 },
    { 0x9e3c7190, 34 },
    { 0x863c7192, 33 },
    { 0x3c7193, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57c7199, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fc719e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfc71a3, 66 },
    { 0x74c71a3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefac71a9, 93 },
    { 0, 0 },
    { 0xd91c71ab, 23 },
    { 0, 0 },
    { 0xd2dc71ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cc71b0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ac71b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfc71b9, 95 },
    { 0, 0 },
    { 0x33cc71bb, 33 },
    { 0, 0 },
    { 0x810c71bd, 33 },
    { 0x4d7c71bd, 77 },
    { 0, 0 },
    { 0xe04c71c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135c71c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec71cc, 95 },
    { 0x27c71cd, 61 },
    { 0xecfc71cc, 33 },
    { 0xb94c71ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x352c71d2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x68c71d5, 90 },
    { 0x3a3c71d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6c71d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64c71dc, 23 },
    { 0x8d7c71dd, 77 },
    { 0, 0 },
    { 0xab6c71df, 33 },
    { 0, 0 },
    { 0x9d3c71e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5c71e9, 55 },
    { 0x6b7c71ea, 55 },
    { 0xa3cc71e9, 23 },
    { 0x54cc71ec, 93 },
    { 0, 0 },
    { 0x779c71ee, 78 },
    { 0, 0 },
    { 0xafdc71f0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2c71f3, 90 },
    { 0, 0 },
    { 0x422c71f5, 95 },
    { 0xe3cc71f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760c71fa, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287c71ff, 33 },
    { 0xb5c71ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfec7205, 61 },
    { 0, 0 },
    { 0x31bc7207, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2c720b, 100 },
    { 0, 0 },
    { 0x6cac720d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dc7213, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4c7217, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4c721c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5c721f, 95 },
    { 0xabac7220, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c7224, 23 },
    { 0x1afc7224, 77 },
    { 0x9d5c7226, 90 },
    { 0xb70c7226, 100 },
    { 0x904c7228, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ac722f, 46 },
    { 0x5d9c7230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c7233, 23 },
    { 0, 0 },
    { 0x879c7235, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6c7242, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532c7245, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cc724a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ac724d, 27 },
    { 0, 0 },
    { 0x257c724f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcc7254, 55 },
    { 0x77ec7255, 34 },
    { 0x893c7255, 98 },
    { 0x6ffc7255, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760c725b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdc725e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cc7261, 61 },
    { 0x509c7262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fc7267, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebc726b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x405c726e, 60 },
    { 0xf7ec726f, 84 },
    { 0, 0 },
    { 0x221c7271, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4c7275, 98 },
    { 0xbb8c7276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bc7279, 55 },
    { 0, 0 },
    { 0xc2ec727b, 90 },
    { 0x72cc727c, 67 },
    { 0x96bc727d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753c7285, 61 },
    { 0x1a7c7285, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4c7289, 91 },
    { 0xeccc728a, 34 },
    { 0, 0 },
    { 0x79bc728c, 87 },
    { 0x200c728d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87c7291, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532c7294, 90 },
    { 0x329c7294, 33 },
    { 0x742c7294, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10c7299, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cac729e, 23 },
    { 0xe48c729f, 93 },
    { 0, 0 },
    { 0x35bc72a1, 90 },
    { 0, 0 },
    { 0xea3c72a3, 98 },
    { 0, 0 },
    { 0x7ec72a5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9c72a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4c72b1, 90 },
    { 0x528c72b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc72b8, 23 },
    { 0xd0cc72b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88c72c1, 33 },
    { 0xe96c72c2, 77 },
    { 0x1f4c72c3, 67 },
    { 0, 0 },
    { 0x6fdc72c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bc72c8, 90 },
    { 0x8f6c72c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc72ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eac72d5, 66 },
    { 0x206c72d6, 23 },
    { 0x72c72d5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29c72dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adc72e0, 95 },
    { 0, 0 },
    { 0x553c72e2, 98 },
    { 0x3a2c72e3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc72e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c72ec, 46 },
    { 0xd59c72ed, 23 },
    { 0xe8cc72ec, 43 },
    { 0xc8ac72ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc72f3, 81 },
    { 0x79ac72f4, 61 },
    { 0x495c72f4, 23 },
    { 0x252c72f6, 99 },
    { 0x2cbc72f7, 33 },
    { 0, 0 },
    { 0xad1c72f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c72fc, 68 },
    { 0x5c2c72fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399c7306, 23 },
    { 0xac3c7307, 55 },
    { 0, 0 },
    { 0x1afc7309, 95 },
    { 0xef3c730a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8c730f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891c7313, 34 },
    { 0xb88c7314, 98 },
    { 0, 0 },
    { 0x94bc7316, 93 },
    { 0x382c7317, 90 },
    { 0x237c7318, 55 },
    { 0, 0 },
    { 0xf0dc731a, 90 },
    { 0, 0 },
    { 0x665c731c, 98 },
    { 0x408c731d, 95 },
    { 0, 0 },
    { 0xab7c731f, 98 },
    { 0x2e1c731f, 78 },
    { 0x800c7321, 93 },
    { 0xb11c7320, 85 },
    { 0x991c7323, 61 },
    { 0x79dc7322, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39c7328, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c732b, 67 },
    { 0x1d2c732c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3c7330, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952c733a, 23 },
    { 0xc82c733a, 98 },
    { 0xcf1c733c, 95 },
    { 0xa24c733d, 23 },
    { 0xe19c733d, 23 },
    { 0, 0 },
    { 0x7e7c7340, 34 },
    { 0, 0 },
    { 0x1cc7342, 61 },
    { 0, 0 },
    { 0x659c7344, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17c7347, 33 },
    { 0, 0 },
    { 0x9f4c7349, 33 },
    { 0xc2bc734a, 98 },
    { 0xd92c7349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c7350, 23 },
    { 0xcf7c7351, 67 },
    { 0, 0 },
    { 0x89ac7353, 61 },
    { 0x4bdc7354, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bc735e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c7361, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6c7364, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbc7368, 95 },
    { 0, 0 },
    { 0xa3dc736a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c736f, 78 },
    { 0x70ac7370, 98 },
    { 0, 0 },
    { 0xe7fc7372, 90 },
    { 0x6eac7373, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ac7376, 23 },
    { 0x969c7376, 55 },
    { 0x37ec7378, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cc737c, 61 },
    { 0xa42c737d, 90 },
    { 0x5ac737c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bc7382, 33 },
    { 0x504c7383, 34 },
    { 0, 0 },
    { 0x792c7385, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cc738c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cc7390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bec7394, 33 },
    { 0x152c7394, 53 },
    { 0x863c7394, 78 },
    { 0x35cc7396, 98 },
    { 0xba5c7398, 33 },
    { 0x8bbc7398, 90 },
    { 0x3b6c739a, 33 },
    { 0x54dc739a, 43 },
    { 0xaf7c739c, 55 },
    { 0, 0 },
    { 0x8dc739e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dc73a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x965c73a6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccc73ab, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2c73b0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfc73b3, 43 },
    { 0x5c4c73b3, 90 },
    { 0, 0 },
    { 0x6e2c73b6, 57 },
    { 0x3fac73b7, 90 },
    { 0x280c73b8, 61 },
    { 0x93dc73b9, 95 },
    { 0, 0 },
    { 0x697c73bb, 90 },
    { 0xfd1c73bc, 78 },
    { 0x9f9c73bd, 61 },
    { 0xcacc73be, 61 },
    { 0xc64c73be, 90 },
    { 0, 0 },
    { 0x192c73c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adc73c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc73c8, 90 },
    { 0x7b1c73c8, 23 },
    { 0x905c73c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x841c73cd, 99 },
    { 0xcf3c73cd, 78 },
    { 0x512c73cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc73d6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264c73da, 23 },
    { 0x316c73da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17c73e0, 43 },
    { 0xb64c73e0, 55 },
    { 0x7ec73e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612c73ec, 33 },
    { 0x6dbc73ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dc73f2, 91 },
    { 0xde7c73f3, 33 },
    { 0xfb3c73f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9c73f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c73fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbc73fe, 68 },
    { 0x7d9c73ff, 61 },
    { 0xebdc7400, 52 },
    { 0x5e8c73fe, 67 },
    { 0xf1c7401, 100 },
    { 0x4a5c7402, 88 },
    { 0xd81c7404, 69 },
    { 0xc42c7405, 98 },
    { 0, 0 },
    { 0x3bcc7407, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c740a, 76 },
    { 0x145c740b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfc740e, 100 },
    { 0x32c740f, 90 },
    { 0, 0 },
    { 0x3b6c7411, 55 },
    { 0x744c7412, 61 },
    { 0, 0 },
    { 0x577c7414, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456c741a, 95 },
    { 0x901c741b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x284c741e, 78 },
    { 0, 0 },
    { 0xc5ac7420, 40 },
    { 0x601c7421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73c7424, 90 },
    { 0xc8cc7425, 17 },
    { 0x373c7425, 34 },
    { 0, 0 },
    { 0xed2c7428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fc742c, 99 },
    { 0x24c742c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188c7432, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x377c7435, 90 },
    { 0x281c7436, 95 },
    { 0, 0 },
    { 0x999c7438, 23 },
    { 0, 0 },
    { 0x33dc743a, 33 },
    { 0x90dc743a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c7446, 23 },
    { 0x3efc7447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617c744b, 61 },
    { 0xe85c744b, 85 },
    { 0x6d6c744c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ec7450, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81c7454, 95 },
    { 0, 0 },
    { 0xef4c7456, 99 },
    { 0, 0 },
    { 0x202c7458, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3c745b, 33 },
    { 0, 0 },
    { 0x9a4c745d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fc7461, 93 },
    { 0xaf4c7462, 88 },
    { 0xca9c7462, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76c7467, 87 },
    { 0x70c7467, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1c746d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dc7471, 23 },
    { 0, 0 },
    { 0x724c7473, 97 },
    { 0xadcc7474, 90 },
    { 0xd8ac7475, 98 },
    { 0, 0 },
    { 0xfffc7477, 43 },
    { 0x229c7478, 91 },
    { 0, 0 },
    { 0xef8c747a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796c7481, 33 },
    { 0xf1cc7481, 98 },
    { 0xc00c7483, 91 },
    { 0xeac7484, 90 },
    { 0, 0 },
    { 0x6b6c7486, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc748a, 90 },
    { 0, 0 },
    { 0x3fcc748c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3c7493, 85 },
    { 0xfc7c7493, 85 },
    { 0xbc0c7493, 23 },
    { 0, 0 },
    { 0x8edc7497, 75 },
    { 0x459c7498, 55 },
    { 0xbb1c7499, 98 },
    { 0xc0bc749a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266c749e, 33 },
    { 0xca4c749f, 90 },
    { 0xe04c74a0, 95 },
    { 0xcbdc74a1, 76 },
    { 0xf54c74a2, 90 },
    { 0, 0 },
    { 0xc2bc74a4, 98 },
    { 0x573c74a4, 46 },
    { 0x5cfc74a6, 100 },
    { 0, 0 },
    { 0x58ac74a8, 90 },
    { 0x80fc74a9, 93 },
    { 0xca6c74aa, 23 },
    { 0xe84c74a9, 34 },
    { 0, 0 },
    { 0xd88c74ad, 98 },
    { 0xb4fc74ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bc74b9, 81 },
    { 0xf47c74b9, 33 },
    { 0xeecc74b9, 90 },
    { 0x194c74ba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fc74c0, 23 },
    { 0x515c74c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2c74ca, 33 },
    { 0x664c74cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bc74ce, 88 },
    { 0x6c4c74ce, 61 },
    { 0x71ec74ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972c74d9, 17 },
    { 0xf36c74da, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83c74df, 90 },
    { 0xe23c74e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326c74e8, 85 },
    { 0xaf4c74e9, 23 },
    { 0x7adc74e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7c74ed, 46 },
    { 0, 0 },
    { 0x13fc74ef, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ec74f4, 61 },
    { 0, 0 },
    { 0xfb7c74f6, 76 },
    { 0x3c5c74f7, 95 },
    { 0x868c74f8, 91 },
    { 0xe5dc74f7, 78 },
    { 0xe23c74fa, 23 },
    { 0x551c74fb, 46 },
    { 0x784c74fc, 96 },
    { 0xa11c74fd, 68 },
    { 0xcfac74fe, 34 },
    { 0x2bdc74f7, 43 },
    { 0x1e2c7500, 33 },
    { 0xaf8c7501, 23 },
    { 0xf73c74f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621c7508, 91 },
    { 0x1e8c7509, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99c750c, 52 },
    { 0x714c750d, 33 },
    { 0xd72c750e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92c7511, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4c7514, 90 },
    { 0x6fc7514, 43 },
    { 0x70cc7514, 55 },
    { 0, 0 },
    { 0x206c7518, 55 },
    { 0x6fec7519, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c751e, 90 },
    { 0, 0 },
    { 0x356c7520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c7523, 33 },
    { 0x4a0c7524, 93 },
    { 0x5bcc7524, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595c752a, 95 },
    { 0x63fc752b, 91 },
    { 0x19ac752c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3c752f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bc7537, 90 },
    { 0x131c7538, 95 },
    { 0x9f3c7539, 33 },
    { 0, 0 },
    { 0xcacc753b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cc7542, 98 },
    { 0xc88c7543, 61 },
    { 0xab5c7544, 33 },
    { 0x2c0c7543, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dc7550, 78 },
    { 0, 0 },
    { 0xaf7c7552, 76 },
    { 0xd32c7552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ec7558, 95 },
    { 0xe7ec7558, 98 },
    { 0, 0 },
    { 0x109c755b, 98 },
    { 0xa59c755b, 33 },
    { 0, 0 },
    { 0x68ec755e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4c7564, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4c7567, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383c7576, 33 },
    { 0x825c7576, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86c757a, 33 },
    { 0, 0 },
    { 0xd8dc757c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x882c757f, 81 },
    { 0, 0 },
    { 0xf46c7581, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23c7584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95c7588, 76 },
    { 0xb79c7589, 36 },
    { 0x7e0c758a, 66 },
    { 0x51ec758b, 75 },
    { 0x618c758c, 76 },
    { 0x40cc758a, 76 },
    { 0xa7c7589, 88 },
    { 0, 0 },
    { 0x19ec7590, 67 },
    { 0, 0 },
    { 0x1a2c7592, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507c7598, 99 },
    { 0x9ec7598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc759c, 33 },
    { 0, 0 },
    { 0x994c759e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fc75a4, 43 },
    { 0x94dc75a5, 17 },
    { 0x442c75a5, 75 },
    { 0, 0 },
    { 0xa51c75a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c75b1, 61 },
    { 0, 0 },
    { 0xa85c75b3, 46 },
    { 0, 0 },
    { 0x8c3c75b5, 100 },
    { 0, 0 },
    { 0x736c75b7, 38 },
    { 0x8dc75b8, 90 },
    { 0x983c75b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fc75bd, 90 },
    { 0, 0 },
    { 0xfaec75bf, 23 },
    { 0, 0 },
    { 0x61fc75c1, 98 },
    { 0xa9fc75c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309c75c7, 98 },
    { 0x8f3c75c8, 95 },
    { 0x183c75c8, 90 },
    { 0x6b9c75c8, 95 },
    { 0, 0 },
    { 0xf40c75cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cc75cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dc75d2, 90 },
    { 0xaa2c75d2, 33 },
    { 0x622c75d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94c75d9, 19 },
    { 0xdbcc75da, 88 },
    { 0xbaac75da, 87 },
    { 0xb3ec75db, 23 },
    { 0x1cdc75dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9c75e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0c75e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c75e7, 85 },
    { 0xb3bc75e7, 61 },
    { 0x3eec75e8, 23 },
    { 0xecac75e8, 61 },
    { 0x3c75eb, 23 },
    { 0x76c75ec, 90 },
    { 0, 0 },
    { 0x4f0c75ee, 66 },
    { 0x45dc75ef, 34 },
    { 0xd36c75f0, 99 },
    { 0x5f6c75f1, 98 },
    { 0x79cc75f2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aec75f6, 55 },
    { 0x212c75f7, 90 },
    { 0xa2c75f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315c75ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3c7602, 23 },
    { 0x9c6c7603, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccc760b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94c760e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495c7613, 43 },
    { 0xe35c7613, 91 },
    { 0, 0 },
    { 0xf87c7616, 98 },
    { 0x3f8c7616, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ec761a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x128c761d, 61 },
    { 0xf66c761e, 76 },
    { 0x9d9c761d, 90 },
    { 0xa2ac7620, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc7623, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac7626, 76 },
    { 0xcb7c7627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7c762a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x555c762d, 76 },
    { 0xb53c762d, 98 },
    { 0xa13c762f, 34 },
    { 0xb2fc762f, 33 },
    { 0xa13c7630, 95 },
    { 0x13bc762d, 23 },
    { 0xdc1c7633, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8c7639, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571c763e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5c7642, 75 },
    { 0xf58c7643, 23 },
    { 0xc7ec7642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7c7648, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71c7650, 33 },
    { 0, 0 },
    { 0xd09c7652, 67 },
    { 0xd79c7653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6c7656, 98 },
    { 0x766c7656, 66 },
    { 0x297c7658, 95 },
    { 0x5fbc7659, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fc765e, 90 },
    { 0x937c765f, 90 },
    { 0x857c7660, 78 },
    { 0x80dc7661, 61 },
    { 0, 0 },
    { 0xa44c7663, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c7667, 87 },
    { 0xf4c7668, 85 },
    { 0, 0 },
    { 0x27fc766a, 23 },
    { 0x496c766b, 61 },
    { 0, 0 },
    { 0xc2fc766d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac7673, 88 },
    { 0x28ec7674, 33 },
    { 0x2eac7674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bc768e, 55 },
    { 0xa3fc768f, 85 },
    { 0xdd6c7690, 61 },
    { 0xcc1c7690, 68 },
    { 0x4fcc7692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x383c7695, 99 },
    { 0xa6dc7696, 90 },
    { 0xa0ac7697, 75 },
    { 0x3f6c7697, 77 },
    { 0x773c7699, 61 },
    { 0x9ec769a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dc769d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bc76a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c76a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c76a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3c76ab, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c76b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9c76b4, 66 },
    { 0xccac76b4, 67 },
    { 0x2d8c76b6, 61 },
    { 0, 0 },
    { 0x817c76b8, 91 },
    { 0x177c76b9, 33 },
    { 0x4c8c76b9, 33 },
    { 0x989c76ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabc76be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aac76c3, 95 },
    { 0xd0dc76c3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40c76c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41c76cb, 52 },
    { 0x8b5c76cb, 43 },
    { 0xec2c76cd, 98 },
    { 0xdccc76ce, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bc76d1, 56 },
    { 0, 0 },
    { 0xd1cc76d3, 95 },
    { 0xae4c76d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71c76d8, 93 },
    { 0x5dec76d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fc76dc, 23 },
    { 0, 0 },
    { 0x1a4c76de, 61 },
    { 0xa32c76de, 43 },
    { 0xb42c76de, 7 },
    { 0xfd1c76e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c76e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10c76e9, 23 },
    { 0x397c76e9, 36 },
    { 0, 0 },
    { 0xe23c76ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x418c76ef, 98 },
    { 0, 0 },
    { 0xcc76f1, 34 },
    { 0, 0 },
    { 0xe03c76f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdc76f6, 46 },
    { 0x548c76f6, 68 },
    { 0x58ec76f8, 90 },
    { 0x6c0c76f6, 95 },
    { 0, 0 },
    { 0xc1c76fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88c76fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fc7707, 78 },
    { 0x19ec7708, 98 },
    { 0x7adc7707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc770c, 90 },
    { 0xb50c770d, 55 },
    { 0x862c770c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9c7711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319c7716, 90 },
    { 0x4c6c7717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cc7721, 84 },
    { 0, 0 },
    { 0x200c7723, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74c7728, 46 },
    { 0, 0 },
    { 0x294c772a, 33 },
    { 0x841c772a, 95 },
    { 0, 0 },
    { 0x69fc772d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171c7731, 52 },
    { 0xaa7c7732, 90 },
    { 0xb1fc7733, 34 },
    { 0, 0 },
    { 0x2acc7735, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77c773d, 95 },
    { 0, 0 },
    { 0x870c773f, 23 },
    { 0, 0 },
    { 0xb94c7741, 55 },
    { 0, 0 },
    { 0xadc7743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41c774b, 76 },
    { 0, 0 },
    { 0x4c9c774d, 55 },
    { 0xaedc774d, 98 },
    { 0xc27c774d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc7753, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50c775b, 98 },
    { 0, 0 },
    { 0x225c775d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bc7760, 95 },
    { 0x79c7760, 76 },
    { 0xc0bc7762, 73 },
    { 0, 0 },
    { 0xa66c7764, 76 },
    { 0, 0 },
    { 0x8a0c7766, 76 },
    { 0, 0 },
    { 0xec0c7768, 98 },
    { 0xacec7768, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1c7770, 78 },
    { 0, 0 },
    { 0x8e4c7772, 98 },
    { 0, 0 },
    { 0x581c7774, 98 },
    { 0xbb9c7774, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x190c7778, 95 },
    { 0x7a3c7779, 90 },
    { 0xc98c7778, 98 },
    { 0xeb1c777b, 61 },
    { 0x5a4c777c, 95 },
    { 0xd0cc777c, 61 },
    { 0x13ac777c, 23 },
    { 0xc1bc777d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05c7782, 23 },
    { 0, 0 },
    { 0xf0dc7784, 46 },
    { 0x786c7785, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bec7790, 90 },
    { 0xaa2c7790, 23 },
    { 0, 0 },
    { 0x999c7793, 52 },
    { 0x61fc7794, 61 },
    { 0, 0 },
    { 0xd39c7796, 98 },
    { 0xc88c7797, 61 },
    { 0x808c7797, 90 },
    { 0, 0 },
    { 0x710c779a, 70 },
    { 0, 0 },
    { 0xfb9c779c, 98 },
    { 0x7c779d, 98 },
    { 0xb94c779d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90c77a1, 34 },
    { 0x1c0c77a2, 88 },
    { 0xe50c77a3, 90 },
    { 0xc6c77a4, 23 },
    { 0xb77c77a5, 90 },
    { 0x2c0c77a2, 98 },
    { 0x44ec77a6, 76 },
    { 0x8c2c77a5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0c77ab, 52 },
    { 0x160c77ac, 61 },
    { 0x171c77ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccc77b4, 61 },
    { 0, 0 },
    { 0x608c77b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72c77bc, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787c77c2, 55 },
    { 0x2dec77c3, 2 },
    { 0x8e2c77c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbc77c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96c77ce, 90 },
    { 0x233c77ce, 76 },
    { 0, 0 },
    { 0x2e8c77d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc77d5, 33 },
    { 0xf0fc77d6, 88 },
    { 0xea8c77d6, 85 },
    { 0xd87c77d8, 98 },
    { 0x6b4c77d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dc77e0, 55 },
    { 0, 0 },
    { 0xea8c77e2, 34 },
    { 0xfa0c77e3, 78 },
    { 0x115c77e2, 90 },
    { 0x7edc77e2, 23 },
    { 0x9c7c77e6, 90 },
    { 0x355c77e2, 98 },
    { 0x4e6c77e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c77ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3c77f9, 90 },
    { 0x682c77fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4c7800, 23 },
    { 0xb5cc7800, 90 },
    { 0x560c7802, 85 },
    { 0, 0 },
    { 0xe32c7804, 61 },
    { 0, 0 },
    { 0x3e4c7806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbc7809, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dac780f, 33 },
    { 0, 0 },
    { 0xe3ac7811, 68 },
    { 0x87ac7811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934c7816, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bec7819, 23 },
    { 0x518c781a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4c7828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbc782b, 76 },
    { 0xb9bc782b, 76 },
    { 0, 0 },
    { 0xd3c782e, 34 },
    { 0xbec782f, 38 },
    { 0, 0 },
    { 0x840c7831, 33 },
    { 0, 0 },
    { 0x552c7833, 81 },
    { 0x53fc7834, 61 },
    { 0, 0 },
    { 0x37ac7836, 61 },
    { 0x3acc7836, 61 },
    { 0x708c7837, 61 },
    { 0xaebc7839, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cc783f, 33 },
    { 0x5c5c7840, 98 },
    { 0xf9ac7840, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfc7844, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc7847, 87 },
    { 0x5f3c7848, 33 },
    { 0xbb9c7849, 23 },
    { 0x959c784a, 95 },
    { 0, 0 },
    { 0x46ec784c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdc784f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fc7854, 100 },
    { 0xa56c7855, 78 },
    { 0x796c7854, 33 },
    { 0xb95c7855, 34 },
    { 0xdf0c7857, 90 },
    { 0, 0 },
    { 0x9a3c785a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59c785d, 96 },
    { 0xfb4c785e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346c7864, 98 },
    { 0, 0 },
    { 0xc53c7866, 95 },
    { 0, 0 },
    { 0xf4cc7868, 99 },
    { 0xd4cc7869, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5c786c, 33 },
    { 0x103c786d, 99 },
    { 0x472c786e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6c7874, 95 },
    { 0, 0 },
    { 0x845c7876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c787c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dc787f, 43 },
    { 0x506c7880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ec7887, 99 },
    { 0xcbec7887, 76 },
    { 0x53ec7889, 95 },
    { 0xda2c788a, 100 },
    { 0xf73c788a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869c788f, 33 },
    { 0x3d1c788f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37c7894, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbc789e, 61 },
    { 0x6efc789f, 33 },
    { 0x2ac789e, 90 },
    { 0xd36c789e, 90 },
    { 0, 0 },
    { 0xe56c78a3, 95 },
    { 0, 0 },
    { 0x19bc78a5, 90 },
    { 0x1bbc78a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bc78aa, 84 },
    { 0x263c78ab, 55 },
    { 0, 0 },
    { 0x9d4c78ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6c78b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55c78b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13c78b8, 61 },
    { 0x90ac78b9, 84 },
    { 0xdbec78b9, 23 },
    { 0xe3bc78bb, 84 },
    { 0x723c78bc, 23 },
    { 0xcdac78bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45c78c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x537c78c6, 95 },
    { 0x13ec78c6, 90 },
    { 0x166c78c8, 61 },
    { 0x207c78c8, 55 },
    { 0x515c78ca, 90 },
    { 0x890c78c6, 44 },
    { 0x667c78cc, 76 },
    { 0x2c6c78c9, 33 },
    { 0x815c78ce, 33 },
    { 0xf31c78cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8c78d6, 33 },
    { 0x1f4c78d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0c78da, 98 },
    { 0xa14c78db, 91 },
    { 0xe96c78da, 98 },
    { 0x78cc78dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6c78e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72c78e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8c78e6, 98 },
    { 0, 0 },
    { 0x3d8c78e8, 33 },
    { 0xa1c78e9, 91 },
    { 0xd10c78ea, 48 },
    { 0x8f7c78ea, 87 },
    { 0, 0 },
    { 0x1cc78ed, 23 },
    { 0, 0 },
    { 0x697c78ef, 23 },
    { 0x26ac78f0, 99 },
    { 0x5b1c78f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bac78f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fc78fc, 61 },
    { 0, 0 },
    { 0x38dc78fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fc7904, 98 },
    { 0x7d7c7905, 98 },
    { 0x199c7905, 90 },
    { 0x6adc7907, 81 },
    { 0x966c7906, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90c790b, 68 },
    { 0xe02c790c, 68 },
    { 0xc5ac790c, 55 },
    { 0, 0 },
    { 0xbb4c790f, 86 },
    { 0xdafc7910, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666c7914, 76 },
    { 0, 0 },
    { 0xc8ac7916, 98 },
    { 0xc07c7917, 95 },
    { 0xa78c7917, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44dc7922, 61 },
    { 0, 0 },
    { 0xe60c7924, 76 },
    { 0x524c7925, 23 },
    { 0x785c7926, 90 },
    { 0x9d9c7927, 95 },
    { 0xa20c7925, 46 },
    { 0x11ac7929, 17 },
    { 0xdfc792a, 61 },
    { 0x6d7c792b, 89 },
    { 0xdfec792c, 93 },
    { 0x6cbc792b, 98 },
    { 0xcebc792e, 23 },
    { 0xdb1c792f, 95 },
    { 0, 0 },
    { 0x3afc7931, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624c7935, 90 },
    { 0, 0 },
    { 0x1d1c7937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5c793b, 85 },
    { 0, 0 },
    { 0x190c793d, 78 },
    { 0xe72c793e, 98 },
    { 0xd29c793f, 76 },
    { 0x5a2c793f, 17 },
    { 0x73bc7941, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeac7949, 77 },
    { 0xcd0c7949, 61 },
    { 0xe70c794b, 43 },
    { 0xe45c794c, 95 },
    { 0, 0 },
    { 0xfebc794e, 98 },
    { 0, 0 },
    { 0x754c7950, 46 },
    { 0x8b6c7951, 34 },
    { 0x727c7952, 33 },
    { 0, 0 },
    { 0x4fac7954, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fc795b, 55 },
    { 0x1b9c795c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bec7969, 93 },
    { 0x5d4c7969, 61 },
    { 0, 0 },
    { 0xccbc796c, 23 },
    { 0x213c796d, 76 },
    { 0, 0 },
    { 0xe41c796f, 23 },
    { 0x6e1c7970, 90 },
    { 0, 0 },
    { 0xcec7972, 84 },
    { 0, 0 },
    { 0xb25c7974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c7977, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983c797b, 93 },
    { 0x6bc797b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdc7980, 23 },
    { 0xcecc7981, 95 },
    { 0xf10c7982, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c7986, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205c798b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbac798e, 58 },
    { 0x97ec798f, 33 },
    { 0, 0 },
    { 0xc9bc7991, 23 },
    { 0, 0 },
    { 0x7a8c7993, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8c7998, 98 },
    { 0x2bec7999, 33 },
    { 0xfddc799a, 95 },
    { 0, 0 },
    { 0xdbc799c, 88 },
    { 0, 0 },
    { 0x718c799e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcc79a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3c79aa, 33 },
    { 0xd08c79ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedac79b7, 90 },
    { 0, 0 },
    { 0x9e7c79b9, 90 },
    { 0, 0 },
    { 0xd62c79bb, 95 },
    { 0, 0 },
    { 0x660c79bd, 95 },
    { 0x142c79be, 90 },
    { 0, 0 },
    { 0x121c79c0, 84 },
    { 0, 0 },
    { 0xe90c79c2, 90 },
    { 0xb01c79c2, 33 },
    { 0x91c79c4, 90 },
    { 0x3a4c79c5, 95 },
    { 0xf1c79c6, 23 },
    { 0xf37c79c4, 61 },
    { 0, 0 },
    { 0xc64c79c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ec79cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32c79d0, 90 },
    { 0x93cc79d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ec79d4, 99 },
    { 0, 0 },
    { 0x6fec79d6, 76 },
    { 0x9c9c79d6, 88 },
    { 0xa7dc79d6, 33 },
    { 0xfefc79d8, 99 },
    { 0, 0 },
    { 0xffac79db, 95 },
    { 0xb35c79dc, 61 },
    { 0x25cc79dd, 100 },
    { 0xdf8c79dd, 66 },
    { 0xcd7c79dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52c79e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabc79ea, 33 },
    { 0, 0 },
    { 0x6ddc79ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cc79f1, 90 },
    { 0, 0 },
    { 0xaa0c79f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ac79f7, 95 },
    { 0xebac79f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13c7a00, 95 },
    { 0x1a7c7a01, 90 },
    { 0xd46c7a02, 98 },
    { 0xe3ac7a03, 98 },
    { 0xd83c7a04, 23 },
    { 0xdc9c7a02, 83 },
    { 0xa88c7a06, 87 },
    { 0xa5fc7a01, 55 },
    { 0, 0 },
    { 0xc5ac7a09, 43 },
    { 0, 0 },
    { 0x6dec7a0b, 90 },
    { 0x644c7a0b, 23 },
    { 0xa7ac7a0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c7a11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x681c7a14, 34 },
    { 0x801c7a15, 98 },
    { 0xd52c7a16, 78 },
    { 0, 0 },
    { 0x173c7a18, 23 },
    { 0x93ac7a18, 55 },
    { 0xd92c7a18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bc7a1f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0c7a26, 93 },
    { 0x4f2c7a26, 55 },
    { 0, 0 },
    { 0x4e2c7a29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x995c7a2c, 98 },
    { 0xc6dc7a2d, 43 },
    { 0xa56c7a2d, 33 },
    { 0x4fec7a2e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462c7a33, 52 },
    { 0x491c7a33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bc7a3b, 90 },
    { 0, 0 },
    { 0xa6cc7a3d, 43 },
    { 0, 0 },
    { 0xcfc7a3f, 87 },
    { 0x811c7a40, 90 },
    { 0x2a7c7a41, 76 },
    { 0x93bc7a41, 90 },
    { 0xbaec7a43, 85 },
    { 0xb3cc7a43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x155c7a47, 61 },
    { 0xf0fc7a47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914c7a4e, 46 },
    { 0x7bac7a4f, 34 },
    { 0xccfc7a4e, 55 },
    { 0x475c7a51, 99 },
    { 0x7a8c7a52, 33 },
    { 0xb02c7a53, 33 },
    { 0, 0 },
    { 0x3acc7a55, 95 },
    { 0xae9c7a56, 61 },
    { 0, 0 },
    { 0xecbc7a58, 61 },
    { 0xa91c7a59, 23 },
    { 0x20dc7a59, 95 },
    { 0xacac7a5b, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2c7a5e, 87 },
    { 0, 0 },
    { 0x56fc7a60, 90 },
    { 0x8e1c7a60, 33 },
    { 0, 0 },
    { 0xb89c7a63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7c7a6b, 88 },
    { 0xc0ec7a6c, 33 },
    { 0xbbcc7a6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4c7a75, 23 },
    { 0, 0 },
    { 0xd90c7a77, 90 },
    { 0xdf8c7a77, 83 },
    { 0, 0 },
    { 0xda9c7a7a, 61 },
    { 0x57bc7a7b, 98 },
    { 0xd71c7a7a, 95 },
    { 0xeebc7a7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52c7a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecc7a8e, 33 },
    { 0, 0 },
    { 0x4d2c7a90, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadc7a9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306c7a9e, 23 },
    { 0xae5c7a9f, 95 },
    { 0xd14c7aa0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29c7aa4, 90 },
    { 0x43cc7aa5, 61 },
    { 0xa07c7aa4, 33 },
    { 0x9fec7aa7, 52 },
    { 0xb44c7aa5, 23 },
    { 0x9a1c7aa9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9c7ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3c7abc, 64 },
    { 0x27dc7abd, 98 },
    { 0x23fc7abe, 56 },
    { 0x3b0c7abf, 91 },
    { 0x492c7ac0, 81 },
    { 0x5e7c7abe, 86 },
    { 0x4d8c7abe, 61 },
    { 0x744c7abf, 91 },
    { 0xc17c7ac2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec7ac7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37c7aca, 61 },
    { 0x36ac7aca, 33 },
    { 0xf17c7acc, 23 },
    { 0xa63c7aca, 33 },
    { 0xc56c7ace, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5c7ad1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c7ad4, 77 },
    { 0, 0 },
    { 0x791c7ad6, 67 },
    { 0, 0 },
    { 0x7d0c7ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0c7adb, 99 },
    { 0x7dcc7adc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599c7ae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c7ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413c7aed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0c7af0, 84 },
    { 0, 0 },
    { 0x672c7af2, 87 },
    { 0xa08c7af3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621c7af7, 90 },
    { 0x8ccc7af7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82c7afb, 33 },
    { 0x64fc7afc, 99 },
    { 0, 0 },
    { 0x47fc7afe, 52 },
    { 0xed0c7aff, 33 },
    { 0x2ccc7aff, 33 },
    { 0x3d2c7b01, 17 },
    { 0x158c7b02, 98 },
    { 0xf7c7b02, 55 },
    { 0x408c7b04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dac7b08, 23 },
    { 0x7d2c7b09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5c7b0c, 98 },
    { 0x9acc7b0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x997c7b10, 100 },
    { 0, 0 },
    { 0x6c3c7b12, 33 },
    { 0x83c7b12, 55 },
    { 0xabcc7b14, 98 },
    { 0x96c7b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bc7b1b, 61 },
    { 0x1bdc7b1c, 90 },
    { 0x3bdc7b1d, 90 },
    { 0x30c7b1e, 33 },
    { 0xabac7b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cac7b22, 34 },
    { 0, 0 },
    { 0xe17c7b24, 23 },
    { 0, 0 },
    { 0xc1ac7b26, 61 },
    { 0xe82c7b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fc7b2a, 34 },
    { 0xb30c7b2b, 52 },
    { 0xfcbc7b2b, 95 },
    { 0xfb3c7b2d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9c7b30, 55 },
    { 0x24cc7b31, 99 },
    { 0xa4cc7b32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1c7b37, 85 },
    { 0xc0ec7b38, 76 },
    { 0x4f7c7b38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecc7b3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184c7b42, 33 },
    { 0xa7cc7b42, 88 },
    { 0, 0 },
    { 0x60fc7b45, 66 },
    { 0, 0 },
    { 0x267c7b47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cc7b4b, 93 },
    { 0, 0 },
    { 0x626c7b4d, 81 },
    { 0x2a7c7b4d, 90 },
    { 0, 0 },
    { 0xd1c7b50, 23 },
    { 0xcfac7b50, 66 },
    { 0x781c7b52, 97 },
    { 0xcb9c7b53, 33 },
    { 0x87fc7b52, 55 },
    { 0, 0 },
    { 0x471c7b56, 65 },
    { 0xd37c7b57, 90 },
    { 0x5aac7b58, 77 },
    { 0, 0 },
    { 0xb88c7b5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427c7b5e, 40 },
    { 0x9e3c7b5e, 93 },
    { 0, 0 },
    { 0xf36c7b61, 23 },
    { 0, 0 },
    { 0x76ec7b63, 61 },
    { 0x607c7b64, 19 },
    { 0x6a5c7b63, 33 },
    { 0, 0 },
    { 0xc8ac7b67, 61 },
    { 0x661c7b68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35c7b6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xddac7b6e, 34 },
    { 0, 0 },
    { 0xbbec7b70, 95 },
    { 0, 0 },
    { 0xea3c7b72, 76 },
    { 0, 0 },
    { 0xc4cc7b74, 98 },
    { 0x6e6c7b75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30c7b78, 46 },
    { 0x4ecc7b78, 33 },
    { 0xbd5c7b7a, 52 },
    { 0xd54c7b7b, 77 },
    { 0, 0 },
    { 0xdcdc7b7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c7b87, 55 },
    { 0x30c7b88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564c7b8f, 61 },
    { 0xd34c7b90, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7c7b95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689c7b9e, 76 },
    { 0, 0 },
    { 0x90c7ba0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dc7ba3, 53 },
    { 0, 0 },
    { 0xd74c7ba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37c7ba9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x590c7bac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650c7bb0, 23 },
    { 0xb9cc7bb1, 23 },
    { 0, 0 },
    { 0xe50c7bb3, 100 },
    { 0xebec7bb4, 33 },
    { 0xfc8c7bb4, 91 },
    { 0x30fc7bb3, 55 },
    { 0xb19c7bb6, 96 },
    { 0x69fc7bb8, 95 },
    { 0, 0 },
    { 0x45c7bba, 23 },
    { 0, 0 },
    { 0x727c7bbc, 55 },
    { 0x8fdc7bbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eac7bc6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1c7bc9, 43 },
    { 0, 0 },
    { 0x656c7bcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3c7bce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1c7bd7, 61 },
    { 0x485c7bd8, 100 },
    { 0x58bc7bd7, 76 },
    { 0xf17c7bda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ac7bea, 91 },
    { 0x67fc7beb, 84 },
    { 0xec7beb, 66 },
    { 0, 0 },
    { 0xc19c7bee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9c7bf6, 66 },
    { 0x138c7bf7, 61 },
    { 0, 0 },
    { 0xd44c7bf9, 89 },
    { 0, 0 },
    { 0x4dac7bfb, 55 },
    { 0x10ec7bfc, 85 },
    { 0, 0 },
    { 0x178c7bfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc7c03, 91 },
    { 0xd1ec7c04, 48 },
    { 0xe34c7c05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec7c09, 61 },
    { 0x8bac7c0a, 52 },
    { 0x4edc7c09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ac7c0e, 66 },
    { 0, 0 },
    { 0x51bc7c10, 61 },
    { 0, 0 },
    { 0xe47c7c12, 64 },
    { 0, 0 },
    { 0x395c7c14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869c7c1d, 88 },
    { 0xdec7c1e, 90 },
    { 0x8b5c7c1e, 55 },
    { 0x159c7c1e, 82 },
    { 0x23cc7c21, 95 },
    { 0xf9c7c21, 61 },
    { 0xd4cc7c21, 44 },
    { 0x2c0c7c24, 68 },
    { 0xff1c7c24, 61 },
    { 0x230c7c26, 55 },
    { 0, 0 },
    { 0x753c7c28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c7c2b, 61 },
    { 0, 0 },
    { 0x266c7c2d, 87 },
    { 0xfc0c7c2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc7c32, 52 },
    { 0, 0 },
    { 0x8cdc7c34, 95 },
    { 0x81ec7c35, 95 },
    { 0xfe5c7c36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2c7c43, 87 },
    { 0x670c7c43, 34 },
    { 0xcb1c7c43, 98 },
    { 0, 0 },
    { 0x143c7c47, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947c7c4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8c7c4f, 99 },
    { 0x21bc7c50, 61 },
    { 0x941c7c51, 98 },
    { 0xe6ac7c52, 61 },
    { 0x18fc7c53, 90 },
    { 0x928c7c53, 67 },
    { 0xd94c7c55, 90 },
    { 0, 0 },
    { 0x83fc7c57, 90 },
    { 0xc2ec7c58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x971c7c5b, 95 },
    { 0, 0 },
    { 0x8b8c7c5d, 61 },
    { 0xff3c7c5e, 23 },
    { 0x830c7c5f, 91 },
    { 0x62cc7c60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fc7c65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adc7c73, 52 },
    { 0x1eec7c73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38c7c79, 90 },
    { 0xb49c7c7a, 98 },
    { 0x45ac7c7a, 100 },
    { 0x22ac7c7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cc7c7f, 91 },
    { 0, 0 },
    { 0xbb1c7c81, 90 },
    { 0x9dac7c82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fc7c89, 66 },
    { 0xd4c7c8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dc7c8e, 55 },
    { 0, 0 },
    { 0xef8c7c90, 55 },
    { 0x909c7c91, 98 },
    { 0xebcc7c91, 85 },
    { 0xab6c7c93, 76 },
    { 0x1eec7c94, 23 },
    { 0x149c7c91, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cc7c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585c7ca4, 84 },
    { 0, 0 },
    { 0xd54c7ca6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981c7cac, 93 },
    { 0xf25c7cad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1c7cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02c7cb8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ac7cc0, 55 },
    { 0x2d0c7cc1, 67 },
    { 0x1b7c7cc2, 95 },
    { 0x5d2c7cc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc7cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508c7ccc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eac7cd2, 33 },
    { 0x82dc7cd3, 61 },
    { 0, 0 },
    { 0xc1c7cd5, 46 },
    { 0x2fac7cd6, 55 },
    { 0xbb7c7cd5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bc7ce2, 85 },
    { 0x4b3c7ce3, 66 },
    { 0x4dcc7ce4, 99 },
    { 0x917c7ce3, 90 },
    { 0x7fcc7ce6, 68 },
    { 0x4a2c7ce3, 55 },
    { 0xdd6c7ce3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8c7ceb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9c7cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bc7cf7, 23 },
    { 0, 0 },
    { 0xf45c7cf9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c7cfd, 33 },
    { 0xbdac7cfd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12c7d06, 46 },
    { 0xf46c7d06, 23 },
    { 0, 0 },
    { 0x847c7d09, 33 },
    { 0x18cc7d0a, 46 },
    { 0x3aac7d0a, 76 },
    { 0x863c7d0c, 53 },
    { 0xcc8c7d0d, 33 },
    { 0xa41c7d0c, 90 },
    { 0xcffc7d0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fec7d12, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecc7d16, 61 },
    { 0xd1fc7d17, 61 },
    { 0x19cc7d16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799c7d1d, 61 },
    { 0xd47c7d1e, 98 },
    { 0, 0 },
    { 0xe6ec7d20, 90 },
    { 0x692c7d20, 66 },
    { 0xa3ac7d22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x287c7d25, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9c7d28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9c7d32, 98 },
    { 0x569c7d32, 67 },
    { 0xc6ac7d34, 34 },
    { 0xf52c7d33, 85 },
    { 0x92fc7d36, 78 },
    { 0x835c7d37, 33 },
    { 0x525c7d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c7d3b, 23 },
    { 0x7d2c7d3b, 45 },
    { 0x39dc7d3b, 95 },
    { 0, 0 },
    { 0x1d9c7d3f, 23 },
    { 0xa4ac7d40, 23 },
    { 0x2d8c7d41, 61 },
    { 0xd29c7d42, 61 },
    { 0xe54c7d43, 23 },
    { 0xb3bc7d43, 90 },
    { 0xcbac7d44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1c7d4f, 84 },
    { 0x1ddc7d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75c7d56, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x729c7d59, 33 },
    { 0, 0 },
    { 0x887c7d5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d60, 33 },
    { 0, 0 },
    { 0x579c7d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4c7d66, 33 },
    { 0x54ac7d66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2c7d6a, 88 },
    { 0xff4c7d6b, 33 },
    { 0, 0 },
    { 0x89dc7d6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fc7d72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x908c7d75, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28c7d78, 61 },
    { 0x83bc7d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5c7d7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779c7d80, 100 },
    { 0x432c7d81, 61 },
    { 0x26dc7d80, 98 },
    { 0x7bc7d81, 90 },
    { 0x577c7d84, 33 },
    { 0xd9bc7d85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cc7d8d, 55 },
    { 0x9a3c7d8e, 67 },
    { 0xe10c7d8f, 93 },
    { 0x622c7d90, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbc7d94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c7d97, 85 },
    { 0x97c7d98, 55 },
    { 0xfb2c7d99, 43 },
    { 0x83ec7d99, 99 },
    { 0xf2bc7d99, 88 },
    { 0x197c7d9c, 76 },
    { 0x394c7d9d, 98 },
    { 0x632c7d9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dc7da2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432c7da9, 23 },
    { 0xb5dc7daa, 52 },
    { 0, 0 },
    { 0xe34c7dac, 61 },
    { 0xf80c7dad, 37 },
    { 0x385c7dae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20c7db3, 76 },
    { 0x31c7db3, 60 },
    { 0xc35c7db5, 84 },
    { 0xb8dc7db6, 91 },
    { 0xabcc7db3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ac7dc0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c7dc5, 76 },
    { 0x3b2c7dc6, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ec7dcd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1c7dd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45c7dd6, 55 },
    { 0x6c5c7dd6, 98 },
    { 0x2b2c7dd8, 46 },
    { 0x89bc7dd9, 98 },
    { 0xd1bc7dd6, 68 },
    { 0x587c7dd8, 91 },
    { 0, 0 },
    { 0x512c7ddd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x810c7de0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadac7de4, 95 },
    { 0x2cdc7de5, 61 },
    { 0, 0 },
    { 0x2cbc7de7, 77 },
    { 0, 0 },
    { 0xcc3c7de9, 85 },
    { 0xdfec7dea, 55 },
    { 0x215c7deb, 34 },
    { 0xb50c7dec, 98 },
    { 0xf56c7dec, 100 },
    { 0xdf2c7dee, 43 },
    { 0xaf2c7def, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c7df2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dc7df6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fc7dfc, 68 },
    { 0x392c7dfd, 93 },
    { 0, 0 },
    { 0x1e2c7dff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72c7e03, 91 },
    { 0x89fc7e03, 95 },
    { 0x3bc7e05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763c7e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ec7e0d, 61 },
    { 0, 0 },
    { 0x20cc7e0f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c7e13, 83 },
    { 0, 0 },
    { 0xdddc7e15, 61 },
    { 0, 0 },
    { 0x43fc7e17, 23 },
    { 0x25ec7e18, 55 },
    { 0, 0 },
    { 0x7b5c7e1a, 91 },
    { 0x666c7e1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2c7e20, 76 },
    { 0x2dac7e21, 77 },
    { 0x374c7e21, 78 },
    { 0x97fc7e23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbc7e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8c7e32, 98 },
    { 0x535c7e33, 55 },
    { 0xbfc7e34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ec7e38, 95 },
    { 0xb8fc7e39, 52 },
    { 0, 0 },
    { 0x155c7e3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccc7e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0c7e42, 67 },
    { 0, 0 },
    { 0xe87c7e44, 76 },
    { 0xa5fc7e44, 95 },
    { 0x400c7e45, 33 },
    { 0xf4dc7e47, 65 },
    { 0x878c7e46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05c7e59, 98 },
    { 0x6bac7e5a, 95 },
    { 0xd38c7e5a, 66 },
    { 0x699c7e5c, 98 },
    { 0, 0 },
    { 0x116c7e5e, 90 },
    { 0x463c7e5f, 95 },
    { 0, 0 },
    { 0x8bcc7e61, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33c7e66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cec7e69, 33 },
    { 0, 0 },
    { 0xa3bc7e6b, 34 },
    { 0, 0 },
    { 0xc6dc7e6d, 76 },
    { 0, 0 },
    { 0xc7ac7e6f, 23 },
    { 0xa17c7e70, 46 },
    { 0, 0 },
    { 0x95cc7e72, 95 },
    { 0x991c7e73, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88c7e77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94c7e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3c7e85, 61 },
    { 0, 0 },
    { 0x6f7c7e87, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c7e8c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ec7e92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2c7e97, 91 },
    { 0xfe9c7e98, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x578c7e9b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3c7ea6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856c7eaa, 98 },
    { 0xa17c7eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2c7eb1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fc7eb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fc7eb9, 55 },
    { 0x69c7eba, 98 },
    { 0x643c7ebb, 98 },
    { 0xad8c7ebc, 98 },
    { 0x843c7ebc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0c7ec0, 17 },
    { 0, 0 },
    { 0xa25c7ec2, 46 },
    { 0xc63c7ec2, 88 },
    { 0xa2ec7ec2, 99 },
    { 0x9dfc7ec3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4c7ec9, 84 },
    { 0, 0 },
    { 0xba5c7ecb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bac7ece, 95 },
    { 0, 0 },
    { 0xb5fc7ed0, 39 },
    { 0x296c7ed0, 93 },
    { 0xad7c7ed1, 90 },
    { 0x32c7ed3, 67 },
    { 0, 0 },
    { 0xb8ec7ed5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408c7edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69c7ee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1c7eea, 61 },
    { 0, 0 },
    { 0x90ec7eec, 90 },
    { 0xa0ac7eed, 93 },
    { 0xd68c7eed, 43 },
    { 0xf29c7eef, 90 },
    { 0x345c7eed, 99 },
    { 0xa6cc7eee, 98 },
    { 0x1f4c7eef, 95 },
    { 0x6a3c7ef3, 80 },
    { 0x796c7ef4, 66 },
    { 0xec1c7ef5, 33 },
    { 0, 0 },
    { 0x803c7ef7, 33 },
    { 0x57fc7ef7, 23 },
    { 0xaf2c7ef9, 99 },
    { 0x519c7efa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccc7efe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637c7f03, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5c7f07, 23 },
    { 0x210c7f08, 33 },
    { 0, 0 },
    { 0x25fc7f0a, 99 },
    { 0x196c7f0b, 95 },
    { 0x9ddc7f0b, 90 },
    { 0xd2fc7f0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x321c7f10, 59 },
    { 0xc04c7f11, 61 },
    { 0x91c7f10, 61 },
    { 0x13ac7f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dc7f17, 98 },
    { 0x21ac7f18, 95 },
    { 0, 0 },
    { 0x590c7f1a, 98 },
    { 0x26c7f1b, 90 },
    { 0, 0 },
    { 0x9bc7f1d, 93 },
    { 0x2b8c7f1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3c7f21, 61 },
    { 0xaa0c7f22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3c7f25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393c7f2b, 95 },
    { 0x119c7f2b, 69 },
    { 0xe54c7f2d, 98 },
    { 0x423c7f2b, 95 },
    { 0x6c4c7f2f, 61 },
    { 0x297c7f30, 98 },
    { 0xe99c7f30, 43 },
    { 0x8bec7f32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8c7f3c, 99 },
    { 0, 0 },
    { 0xaacc7f3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3c7f46, 90 },
    { 0xec7c7f46, 52 },
    { 0xff8c7f48, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bc7f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6c7f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c7f59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x536c7f5c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x614c7f5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1c7f64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x96c7f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3c7f6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9c7f72, 33 },
    { 0, 0 },
    { 0xf34c7f74, 90 },
    { 0xa1ac7f75, 23 },
    { 0xd1ac7f74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1c7f7c, 90 },
    { 0, 0 },
    { 0x38ac7f7e, 81 },
    { 0, 0 },
    { 0x590c7f80, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295c7f8a, 93 },
    { 0xc9c7f8b, 98 },
    { 0xae7c7f8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81c7f94, 87 },
    { 0x64ac7f95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc7f9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ec7f9e, 33 },
    { 0x656c7f9e, 23 },
    { 0x2f3c7fa0, 23 },
    { 0xb48c7f9f, 52 },
    { 0xa24c7fa2, 87 },
    { 0xd65c7fa2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bc7fa6, 83 },
    { 0xcd2c7fa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6c7fab, 33 },
    { 0x817c7fab, 98 },
    { 0x30dc7fab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7c7fb0, 93 },
    { 0xdfcc7fb1, 90 },
    { 0xa54c7fb2, 55 },
    { 0x488c7fb3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75c7fb7, 33 },
    { 0x748c7fb8, 98 },
    { 0, 0 },
    { 0x1e5c7fba, 34 },
    { 0, 0 },
    { 0xa85c7fbc, 98 },
    { 0x892c7fbd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cec7fc0, 61 },
    { 0, 0 },
    { 0x876c7fc2, 87 },
    { 0x10dc7fc3, 90 },
    { 0x64ac7fc4, 61 },
    { 0xee1c7fc5, 90 },
    { 0x71ac7fc6, 46 },
    { 0x6f0c7fc5, 90 },
    { 0, 0 },
    { 0xd68c7fc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7c7fcc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc7fd0, 90 },
    { 0xb59c7fd0, 66 },
    { 0, 0 },
    { 0x412c7fd3, 67 },
    { 0, 0 },
    { 0x353c7fd5, 33 },
    { 0x935c7fd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbac7fdf, 61 },
    { 0, 0 },
    { 0x5ecc7fe1, 38 },
    { 0x504c7fe2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ac7fea, 76 },
    { 0, 0 },
    { 0x9d5c7fec, 85 },
    { 0, 0 },
    { 0xc82c7fee, 17 },
    { 0xac7fef, 98 },
    { 0x5afc7fef, 98 },
    { 0xba3c7ff1, 34 },
    { 0x789c7ff2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27c7ff8, 98 },
    { 0, 0 },
    { 0xa75c7ffa, 78 },
    { 0x496c7ffb, 83 },
    { 0x433c7ffb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dc8005, 85 },
    { 0x1b6c8006, 68 },
    { 0x818c8006, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c800a, 33 },
    { 0, 0 },
    { 0x39bc800c, 95 },
    { 0, 0 },
    { 0xaf7c800e, 95 },
    { 0, 0 },
    { 0x7bec8010, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abc8016, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c801d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1c8026, 34 },
    { 0xb72c8026, 95 },
    { 0x67dc8028, 33 },
    { 0, 0 },
    { 0x5fcc802a, 97 },
    { 0, 0 },
    { 0x17cc802c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40c8033, 52 },
    { 0, 0 },
    { 0x469c8035, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec8038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cc803c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x414c803f, 66 },
    { 0xb1ac8040, 33 },
    { 0xbd6c8041, 90 },
    { 0x6d0c8040, 67 },
    { 0x22fc8043, 52 },
    { 0x152c803f, 95 },
    { 0x4b5c803f, 61 },
    { 0x932c8046, 95 },
    { 0x13ac8047, 76 },
    { 0x2a7c8044, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eec804b, 61 },
    { 0x8bdc804b, 61 },
    { 0x1b3c804d, 23 },
    { 0x6a7c804e, 43 },
    { 0xbc7c804f, 33 },
    { 0x60dc8050, 90 },
    { 0xac9c8050, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4c8054, 91 },
    { 0x10bc8054, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0c8060, 33 },
    { 0x759c8060, 78 },
    { 0xb49c8062, 90 },
    { 0xe63c8061, 90 },
    { 0, 0 },
    { 0xe42c8065, 52 },
    { 0x37c8066, 58 },
    { 0xe64c8066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bc806c, 33 },
    { 0xe38c806c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c8075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566c807d, 33 },
    { 0, 0 },
    { 0x34ac807f, 66 },
    { 0, 0 },
    { 0xfddc8081, 61 },
    { 0x92cc8081, 98 },
    { 0xac8083, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebec808f, 95 },
    { 0xbe1c8090, 77 },
    { 0x9cdc8091, 91 },
    { 0xb0fc8092, 95 },
    { 0xf0fc808f, 85 },
    { 0x83cc808f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0c8098, 95 },
    { 0, 0 },
    { 0x6e4c809a, 101 },
    { 0x921c809b, 81 },
    { 0x7abc809c, 53 },
    { 0, 0 },
    { 0x937c809e, 12 },
    { 0, 0 },
    { 0xa86c80a0, 76 },
    { 0x2acc80a1, 43 },
    { 0xb29c80a1, 23 },
    { 0xcefc80a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ec80a6, 95 },
    { 0, 0 },
    { 0xec8c80a8, 91 },
    { 0x403c80a9, 75 },
    { 0, 0 },
    { 0x5c5c80ab, 98 },
    { 0x4b9c80ac, 43 },
    { 0x553c80ab, 98 },
    { 0x492c80ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x40c80b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdec80b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1c80c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943c80ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653c80cf, 99 },
    { 0xaabc80d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ec80d7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86c80db, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90c80df, 61 },
    { 0xc49c80df, 33 },
    { 0x6ddc80e1, 100 },
    { 0xa92c80e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc80f1, 33 },
    { 0x5a2c80f2, 99 },
    { 0, 0 },
    { 0xfeac80f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34c80f8, 90 },
    { 0x8b5c80f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cc80fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefc80ff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1c8107, 66 },
    { 0, 0 },
    { 0x8dac8109, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46c810f, 52 },
    { 0x600c810f, 76 },
    { 0x250c810f, 98 },
    { 0xe08c8112, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70c811a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ec8125, 90 },
    { 0, 0 },
    { 0x493c8127, 95 },
    { 0xfdac8128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5c812b, 61 },
    { 0xb22c812c, 95 },
    { 0x5fcc812c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7c8135, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9c813a, 85 },
    { 0x853c813b, 43 },
    { 0x749c813b, 36 },
    { 0x7c5c813b, 93 },
    { 0, 0 },
    { 0x296c813f, 93 },
    { 0, 0 },
    { 0x8ddc8141, 98 },
    { 0, 0 },
    { 0xad0c8143, 90 },
    { 0xdccc8144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0c8149, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdc814e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21c8151, 91 },
    { 0x963c8152, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfc8155, 55 },
    { 0x7dbc8155, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48c815c, 55 },
    { 0xe80c815d, 92 },
    { 0x4dac815e, 77 },
    { 0x7b4c815f, 76 },
    { 0x9dcc8160, 55 },
    { 0xedbc8160, 23 },
    { 0x286c815f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdc816d, 61 },
    { 0xbf7c816e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8c8172, 61 },
    { 0x1b4c8173, 81 },
    { 0x52ec8173, 76 },
    { 0, 0 },
    { 0x33dc8176, 76 },
    { 0, 0 },
    { 0x779c8178, 98 },
    { 0xc1ec8178, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848c817d, 87 },
    { 0xfedc817d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2c8183, 34 },
    { 0xb18c8183, 55 },
    { 0xf5fc8183, 55 },
    { 0xa6dc8186, 78 },
    { 0xc4dc8186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc818a, 33 },
    { 0, 0 },
    { 0x4c0c818c, 43 },
    { 0xf1bc818d, 43 },
    { 0xf52c818d, 43 },
    { 0x5ffc818c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c8192, 78 },
    { 0xf6ac8193, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3c819f, 61 },
    { 0x321c81a0, 55 },
    { 0xefac81a1, 90 },
    { 0x411c81a2, 43 },
    { 0xa3ec81a2, 23 },
    { 0xce1c81a4, 90 },
    { 0x7ddc81a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc81ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc81b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabc81b6, 96 },
    { 0, 0 },
    { 0x71dc81b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88c81bc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ec81c1, 95 },
    { 0, 0 },
    { 0x2c8c81c3, 98 },
    { 0x504c81c4, 33 },
    { 0x5b0c81c5, 90 },
    { 0, 0 },
    { 0x5f0c81c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bec81cb, 23 },
    { 0x209c81cb, 43 },
    { 0xc69c81cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7c81d1, 61 },
    { 0xc2ec81d2, 25 },
    { 0x316c81d3, 83 },
    { 0x37dc81d4, 98 },
    { 0, 0 },
    { 0x391c81d6, 23 },
    { 0x964c81d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc81df, 85 },
    { 0, 0 },
    { 0x159c81e1, 65 },
    { 0xe29c81e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbc81ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ec81ef, 76 },
    { 0, 0 },
    { 0x1e5c81f1, 90 },
    { 0xa59c81f2, 61 },
    { 0x4edc81f1, 61 },
    { 0x852c81f4, 23 },
    { 0xf8bc81f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec8200, 97 },
    { 0x236c8201, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x866c8204, 98 },
    { 0x3fac8204, 55 },
    { 0x4cfc8206, 90 },
    { 0x545c8206, 76 },
    { 0, 0 },
    { 0x72cc8209, 90 },
    { 0, 0 },
    { 0x701c820b, 98 },
    { 0, 0 },
    { 0x80fc820d, 61 },
    { 0xd4dc820d, 93 },
    { 0x7c820f, 61 },
    { 0x10c820f, 93 },
    { 0, 0 },
    { 0x278c8212, 61 },
    { 0, 0 },
    { 0xea6c8214, 90 },
    { 0x109c8215, 66 },
    { 0x51c8214, 55 },
    { 0xf24c8215, 93 },
    { 0x969c8218, 67 },
    { 0x78ec8219, 80 },
    { 0xccec8218, 61 },
    { 0x364c821b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ac8220, 78 },
    { 0x5aec8221, 95 },
    { 0, 0 },
    { 0xec0c8223, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ec8228, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bc822f, 68 },
    { 0, 0 },
    { 0x7a9c8231, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6c8237, 55 },
    { 0xe3bc8238, 90 },
    { 0x937c8239, 61 },
    { 0, 0 },
    { 0x45ec823b, 17 },
    { 0xbd1c823c, 95 },
    { 0, 0 },
    { 0xeeec823e, 61 },
    { 0x315c823f, 34 },
    { 0x577c8240, 61 },
    { 0x4e7c8241, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38c8246, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9c824c, 76 },
    { 0, 0 },
    { 0x541c824e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8c8252, 76 },
    { 0x5f3c8253, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbec8257, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24c825b, 78 },
    { 0, 0 },
    { 0x9c0c825d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2c8267, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bc826f, 100 },
    { 0x93ac8270, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bc8275, 33 },
    { 0x64dc8275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85c827e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5c8283, 95 },
    { 0x8d7c8284, 90 },
    { 0x570c8285, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7c8289, 76 },
    { 0xf38c828a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac828d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcc8290, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x326c8293, 61 },
    { 0x413c8294, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50c8297, 77 },
    { 0x2bbc8298, 33 },
    { 0xc10c8297, 55 },
    { 0x46ec829a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c82a6, 84 },
    { 0, 0 },
    { 0xd6fc82a8, 46 },
    { 0x6d2c82a9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5c82af, 69 },
    { 0x200c82af, 90 },
    { 0x6c1c82b1, 61 },
    { 0x7cac82b1, 46 },
    { 0x66dc82b3, 84 },
    { 0xc28c82b4, 23 },
    { 0x6a4c82b4, 43 },
    { 0x8a9c82b6, 61 },
    { 0, 0 },
    { 0x13c82b8, 33 },
    { 0, 0 },
    { 0x2cec82ba, 23 },
    { 0, 0 },
    { 0xa44c82bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fc82c0, 61 },
    { 0x822c82c0, 94 },
    { 0xf0bc82c2, 94 },
    { 0xbecc82c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c82c7, 31 },
    { 0xc65c82c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dac82cd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41c82d3, 55 },
    { 0x348c82d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdc82d8, 33 },
    { 0xeb2c82d8, 67 },
    { 0, 0 },
    { 0x74cc82db, 46 },
    { 0, 0 },
    { 0x573c82dd, 91 },
    { 0x5fbc82de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0c82e1, 90 },
    { 0xfbcc82e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295c82e9, 52 },
    { 0xb3fc82ea, 76 },
    { 0x889c82ea, 66 },
    { 0xc90c82ea, 76 },
    { 0, 0 },
    { 0x39ec82ee, 17 },
    { 0xaa0c82ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ac82f2, 61 },
    { 0xfbcc82f3, 80 },
    { 0xf4cc82f3, 34 },
    { 0x108c82f5, 46 },
    { 0, 0 },
    { 0x3bc82f7, 42 },
    { 0xd07c82f8, 90 },
    { 0x1a5c82f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x779c82fc, 55 },
    { 0, 0 },
    { 0x80bc82fe, 61 },
    { 0xb35c82ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x401c8302, 98 },
    { 0x9a3c8303, 90 },
    { 0x5c4c8302, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4c8308, 61 },
    { 0xb12c8309, 100 },
    { 0, 0 },
    { 0xd68c830b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fc8313, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bc8319, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cc831d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4c8322, 34 },
    { 0xc1c8323, 23 },
    { 0xbb7c8322, 56 },
    { 0x912c8324, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x889c8328, 95 },
    { 0xb6ec8329, 77 },
    { 0x39dc8329, 61 },
    { 0x26bc832b, 34 },
    { 0xcebc832c, 98 },
    { 0xf4bc832d, 90 },
    { 0, 0 },
    { 0x55dc832f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fc8333, 55 },
    { 0, 0 },
    { 0x396c8335, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0c8338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cc833c, 90 },
    { 0xe74c833c, 76 },
    { 0xa63c833e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771c8344, 98 },
    { 0xe84c8344, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7c8349, 95 },
    { 0, 0 },
    { 0xfccc834b, 46 },
    { 0x37cc834c, 53 },
    { 0xe1ac834d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514c8351, 95 },
    { 0xb3ec8352, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3c8358, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7c835e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5c8363, 61 },
    { 0xdf7c8364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ac836c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2c8375, 95 },
    { 0xc8ac8376, 33 },
    { 0xa66c8377, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c837b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bc837f, 98 },
    { 0x8c7c8380, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12c8386, 100 },
    { 0, 0 },
    { 0xfa9c8388, 33 },
    { 0, 0 },
    { 0xfd8c838a, 98 },
    { 0xceac838a, 98 },
    { 0, 0 },
    { 0xda5c838d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56c8392, 99 },
    { 0x264c8392, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcc8396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11c839b, 95 },
    { 0xa81c839c, 28 },
    { 0xfaec839c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ac83a4, 23 },
    { 0x2d0c83a4, 90 },
    { 0x722c83a6, 46 },
    { 0x2bec83a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81c83ac, 94 },
    { 0x88bc83ad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fc83b5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8c83ba, 90 },
    { 0xf8cc83bb, 91 },
    { 0xe10c83bb, 90 },
    { 0, 0 },
    { 0xdc2c83be, 33 },
    { 0xe30c83bf, 88 },
    { 0, 0 },
    { 0xcd8c83c1, 91 },
    { 0, 0 },
    { 0x11ec83c3, 43 },
    { 0x36fc83c4, 52 },
    { 0xe2fc83c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7c83cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abc83d1, 100 },
    { 0xcccc83d1, 33 },
    { 0xc2ac83d1, 95 },
    { 0x6f1c83d4, 23 },
    { 0xdb1c83d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5c83de, 76 },
    { 0, 0 },
    { 0x73ec83e0, 85 },
    { 0x856c83e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bc83e6, 51 },
    { 0xa29c83e7, 45 },
    { 0x7edc83e8, 66 },
    { 0xfd7c83e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbc83f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec83f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaac83fa, 81 },
    { 0x1dbc83fb, 23 },
    { 0, 0 },
    { 0x76dc83fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x244c8400, 33 },
    { 0x7c3c8401, 34 },
    { 0, 0 },
    { 0x8c3c8403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847c8408, 95 },
    { 0x1b1c8409, 46 },
    { 0x2ffc840a, 23 },
    { 0xb51c840b, 65 },
    { 0x630c840b, 23 },
    { 0, 0 },
    { 0xd3dc840e, 98 },
    { 0, 0 },
    { 0x95bc8410, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254c8415, 90 },
    { 0, 0 },
    { 0x67c8417, 33 },
    { 0xb5bc8418, 98 },
    { 0x771c8419, 98 },
    { 0x6dec841a, 78 },
    { 0x1bc841b, 95 },
    { 0x1d1c841a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5c8421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253c8425, 93 },
    { 0xfdfc8425, 85 },
    { 0xd01c8427, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7c842b, 98 },
    { 0, 0 },
    { 0xdcc842d, 55 },
    { 0x7ebc842e, 33 },
    { 0xcbac842f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ec8433, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751c8437, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6c843a, 61 },
    { 0, 0 },
    { 0x544c843c, 53 },
    { 0x8afc843d, 75 },
    { 0x8b1c843d, 33 },
    { 0xd92c843f, 17 },
    { 0, 0 },
    { 0xbd6c8441, 66 },
    { 0x11c8442, 40 },
    { 0, 0 },
    { 0x8edc8444, 33 },
    { 0xcdcc8445, 75 },
    { 0, 0 },
    { 0xea3c8447, 43 },
    { 0xfc9c8448, 65 },
    { 0xc9dc8449, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c8452, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fc8457, 23 },
    { 0, 0 },
    { 0xf4cc8459, 90 },
    { 0x35ec845a, 80 },
    { 0x604c845b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392c845f, 90 },
    { 0xd16c8460, 98 },
    { 0x21ac8460, 87 },
    { 0x3aec845f, 95 },
    { 0xeac8463, 100 },
    { 0x4e9c8464, 55 },
    { 0, 0 },
    { 0x80fc8466, 33 },
    { 0, 0 },
    { 0xd21c8468, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x566c846c, 23 },
    { 0x8bdc846c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bc8478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcc8480, 93 },
    { 0, 0 },
    { 0x850c8482, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28c8486, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c8489, 66 },
    { 0, 0 },
    { 0xbdfc848b, 33 },
    { 0xdccc848b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc848f, 52 },
    { 0x686c848f, 95 },
    { 0, 0 },
    { 0xcf5c8492, 81 },
    { 0x712c8492, 99 },
    { 0xdf4c8494, 90 },
    { 0xb62c8495, 95 },
    { 0xf1c8492, 23 },
    { 0x30bc8497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456c849d, 87 },
    { 0x7eac849e, 99 },
    { 0x367c849f, 87 },
    { 0, 0 },
    { 0x1d7c84a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cec84a4, 55 },
    { 0xeb7c84a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9c84a9, 93 },
    { 0, 0 },
    { 0xc9bc84ab, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bc84ae, 81 },
    { 0x343c84ae, 95 },
    { 0, 0 },
    { 0x13ac84b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417c84b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x985c84b9, 84 },
    { 0, 0 },
    { 0x3ecc84bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bc84c0, 23 },
    { 0x3a4c84c1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc84c4, 95 },
    { 0x76bc84c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79c84ca, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0c84ce, 34 },
    { 0x375c84cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6c84d4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3c84d7, 99 },
    { 0x6bec84d8, 33 },
    { 0xe22c84d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bc84dc, 90 },
    { 0xcaec84dd, 38 },
    { 0x37dc84dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x434c84e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302c84e9, 76 },
    { 0x4c84ea, 100 },
    { 0, 0 },
    { 0xf6fc84ec, 91 },
    { 0xff3c84ec, 61 },
    { 0x751c84ed, 90 },
    { 0, 0 },
    { 0x705c84f0, 95 },
    { 0x23bc84f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3c84f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc84ff, 85 },
    { 0xa75c8500, 90 },
    { 0xacac84ff, 95 },
    { 0xddac8502, 95 },
    { 0xe12c8503, 33 },
    { 0x4b9c8504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aec8507, 46 },
    { 0x5e5c8507, 75 },
    { 0x7eec8509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x929c850c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aec8511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2c8518, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5c851c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589c8520, 90 },
    { 0xc60c8521, 98 },
    { 0xc77c8520, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x379c8525, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925c852a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72c852d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c8530, 90 },
    { 0x877c8531, 90 },
    { 0, 0 },
    { 0xa25c8533, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c8539, 90 },
    { 0x35bc853a, 90 },
    { 0xe72c853b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485c853f, 61 },
    { 0xcf1c8540, 65 },
    { 0x2e8c8541, 33 },
    { 0x8d1c8541, 61 },
    { 0x103c8542, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968c8548, 33 },
    { 0x1e6c8549, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcc854c, 48 },
    { 0, 0 },
    { 0x60dc854e, 90 },
    { 0x55fc854f, 98 },
    { 0xcd5c8550, 55 },
    { 0x16ac8551, 61 },
    { 0x74c8552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94c855b, 23 },
    { 0x695c855c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ec855f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65c8569, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c8573, 43 },
    { 0x2edc8574, 98 },
    { 0xaacc8575, 84 },
    { 0, 0 },
    { 0x12dc8577, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786c857e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfc8584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91c858c, 33 },
    { 0x5e7c858d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103c8593, 33 },
    { 0xbbfc8593, 90 },
    { 0x5bfc8595, 34 },
    { 0, 0 },
    { 0x486c8597, 76 },
    { 0xd0ac8598, 34 },
    { 0x1f1c8599, 99 },
    { 0, 0 },
    { 0x120c859b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bc859e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fec85a4, 61 },
    { 0xbabc85a4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25c85aa, 23 },
    { 0xc1ec85ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc85ae, 76 },
    { 0, 0 },
    { 0xbc85b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc85b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcc85bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228c85c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2c85c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fc85ce, 76 },
    { 0x4afc85ce, 98 },
    { 0, 0 },
    { 0x109c85d1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aec85d5, 23 },
    { 0x6b2c85d6, 84 },
    { 0, 0 },
    { 0xebbc85d8, 34 },
    { 0x54cc85d9, 33 },
    { 0, 0 },
    { 0x18c85db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68c85de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9c85e2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50c85e7, 48 },
    { 0x149c85e7, 95 },
    { 0xed5c85e9, 46 },
    { 0xc29c85ea, 95 },
    { 0xa22c85ea, 100 },
    { 0x91fc85e8, 34 },
    { 0x518c85ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacc85f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87c85f4, 99 },
    { 0xdb5c85f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63c85fc, 98 },
    { 0x806c85fd, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bc8603, 95 },
    { 0xc34c8603, 95 },
    { 0, 0 },
    { 0x3e8c8606, 33 },
    { 0x2abc8607, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775c860d, 46 },
    { 0x6f3c860e, 33 },
    { 0x811c860d, 55 },
    { 0xdbdc8610, 95 },
    { 0x473c8611, 76 },
    { 0, 0 },
    { 0xc14c8613, 90 },
    { 0, 0 },
    { 0xd56c8615, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3c8618, 43 },
    { 0x68ac8619, 90 },
    { 0, 0 },
    { 0xcc6c861b, 98 },
    { 0xa37c861b, 23 },
    { 0x8f0c861d, 95 },
    { 0xe40c861b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757c8625, 66 },
    { 0x379c8626, 90 },
    { 0, 0 },
    { 0x98ac8628, 33 },
    { 0xed2c8628, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568c862f, 98 },
    { 0, 0 },
    { 0xccdc8631, 43 },
    { 0xa2c8631, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcc8637, 95 },
    { 0xdb7c8637, 76 },
    { 0x567c8639, 43 },
    { 0x153c8637, 23 },
    { 0x2dc8638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440c8643, 61 },
    { 0x8ecc8644, 80 },
    { 0x8ecc8645, 90 },
    { 0x90bc8644, 34 },
    { 0x4c5c8646, 91 },
    { 0, 0 },
    { 0xf6fc8649, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ac864e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865c8653, 76 },
    { 0x57dc8654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56c8658, 61 },
    { 0xc98c8658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4c865f, 90 },
    { 0, 0 },
    { 0x422c8661, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fc866d, 46 },
    { 0, 0 },
    { 0xd57c866f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddc8674, 90 },
    { 0x3eac8675, 34 },
    { 0, 0 },
    { 0xfcbc8677, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x550c867a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c867f, 23 },
    { 0xca8c8680, 99 },
    { 0xf0ec8680, 33 },
    { 0, 0 },
    { 0x56cc8683, 61 },
    { 0, 0 },
    { 0x88c8685, 33 },
    { 0xfe8c8686, 98 },
    { 0x8bfc8687, 87 },
    { 0xc5bc8685, 33 },
    { 0xacfc8689, 33 },
    { 0xa61c8687, 38 },
    { 0, 0 },
    { 0x106c868c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4c8691, 93 },
    { 0xef1c8692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fc8696, 90 },
    { 0x482c8697, 46 },
    { 0, 0 },
    { 0x5fcc8699, 34 },
    { 0, 0 },
    { 0xaffc869b, 77 },
    { 0, 0 },
    { 0x484c869d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bc86a0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ec86a3, 76 },
    { 0x33bc86a4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c86a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bc86b5, 75 },
    { 0xcf2c86b5, 55 },
    { 0, 0 },
    { 0xd6cc86b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c86c2, 95 },
    { 0xb1fc86c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dc86c6, 61 },
    { 0x318c86c7, 93 },
    { 0x812c86c8, 23 },
    { 0, 0 },
    { 0x647c86ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abc86cd, 78 },
    { 0x3aec86ce, 88 },
    { 0, 0 },
    { 0x293c86d0, 43 },
    { 0xd9c86d1, 61 },
    { 0x7a6c86d1, 76 },
    { 0x278c86d1, 33 },
    { 0, 0 },
    { 0x5a2c86d5, 43 },
    { 0xcb9c86d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1c86db, 52 },
    { 0x56fc86db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30c86df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x858c86e2, 90 },
    { 0x3ecc86e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7c86e8, 85 },
    { 0xa65c86e8, 61 },
    { 0x836c86ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0c86ef, 43 },
    { 0xd6dc86f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3c86f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253c86f8, 98 },
    { 0x6b3c86f9, 33 },
    { 0, 0 },
    { 0xebc86fb, 33 },
    { 0x5a9c86fc, 78 },
    { 0xca9c86fb, 85 },
    { 0x215c86fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8c8704, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c8708, 23 },
    { 0x56ec8709, 55 },
    { 0x424c8709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc8710, 23 },
    { 0x865c8711, 68 },
    { 0xcfbc8711, 92 },
    { 0x2c0c8713, 95 },
    { 0xd45c8714, 55 },
    { 0xdec8715, 33 },
    { 0, 0 },
    { 0xd82c8717, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c871b, 53 },
    { 0xf99c871c, 85 },
    { 0x7bbc871c, 34 },
    { 0xf96c871c, 61 },
    { 0, 0 },
    { 0xfe3c8720, 17 },
    { 0x80dc8720, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7c8725, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcec872f, 78 },
    { 0xb6dc8730, 98 },
    { 0x41fc8731, 75 },
    { 0, 0 },
    { 0x363c8733, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bc8738, 23 },
    { 0xef0c8739, 33 },
    { 0xae1c8738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375c873e, 76 },
    { 0, 0 },
    { 0xd2bc8740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11c8747, 33 },
    { 0x2e6c8748, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fc874b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16c8755, 90 },
    { 0, 0 },
    { 0x86ac8757, 23 },
    { 0x5e7c8758, 46 },
    { 0x4b3c8757, 91 },
    { 0x276c8757, 43 },
    { 0xb5bc875b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cc875e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x733c8761, 87 },
    { 0xc64c8762, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bc8765, 90 },
    { 0xd54c8766, 98 },
    { 0xa38c8766, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ec8775, 33 },
    { 0x71bc8775, 46 },
    { 0x2afc8777, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770c877b, 61 },
    { 0xafac877b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9c8780, 55 },
    { 0xe70c8780, 33 },
    { 0x2d0c8782, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bc8787, 61 },
    { 0x1bbc8788, 33 },
    { 0x625c8789, 90 },
    { 0x62fc8789, 95 },
    { 0x7cbc8789, 90 },
    { 0, 0 },
    { 0xc5bc878d, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cc8790, 43 },
    { 0x6c2c8791, 61 },
    { 0x9b6c8792, 55 },
    { 0x38ec8791, 76 },
    { 0x855c8790, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9c8799, 61 },
    { 0, 0 },
    { 0xa68c879b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cc879e, 33 },
    { 0, 0 },
    { 0x5b5c87a0, 77 },
    { 0x4cec87a1, 34 },
    { 0x82ec87a0, 33 },
    { 0, 0 },
    { 0x135c87a4, 91 },
    { 0x9d2c87a5, 77 },
    { 0xa1c87a4, 95 },
    { 0x8e7c87a5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c87b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcc87bb, 61 },
    { 0xd03c87bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dec87c1, 77 },
    { 0xe6cc87c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bac87c5, 90 },
    { 0x911c87c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c87c9, 90 },
    { 0, 0 },
    { 0x197c87cb, 57 },
    { 0xb07c87cc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5c87d1, 33 },
    { 0x3cbc87d2, 98 },
    { 0x4dc87d2, 95 },
    { 0x87bc87d2, 34 },
    { 0x768c87d5, 34 },
    { 0xa89c87d6, 95 },
    { 0xf16c87d6, 61 },
    { 0x9c9c87d8, 23 },
    { 0, 0 },
    { 0x40c87da, 96 },
    { 0xa2bc87db, 87 },
    { 0xf7dc87db, 90 },
    { 0x79dc87dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88c87e0, 90 },
    { 0x428c87e1, 55 },
    { 0x5e0c87e2, 84 },
    { 0x68c87e2, 61 },
    { 0x491c87e0, 34 },
    { 0x4edc87e5, 78 },
    { 0x898c87e6, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0x540c87e9, 99 },
    { 0x58fc87ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3c87ed, 23 },
    { 0, 0 },
    { 0xd9dc87ef, 33 },
    { 0, 0 },
    { 0xec87f1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc87f4, 68 },
    { 0x1f6c87f5, 43 },
    { 0x97ec87f6, 98 },
    { 0xb4fc87f4, 33 },
    { 0xdcec87f8, 38 },
    { 0x8d2c87f8, 62 },
    { 0xd49c87fa, 52 },
    { 0xbc3c87fa, 98 },
    { 0x280c87fa, 98 },
    { 0x5efc87fd, 33 },
    { 0xe8ec87fe, 93 },
    { 0, 0 },
    { 0x8acc8800, 91 },
    { 0, 0 },
    { 0x911c8802, 55 },
    { 0xb02c8803, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1c8806, 99 },
    { 0, 0 },
    { 0x7a9c8808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3c8812, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9c881a, 61 },
    { 0, 0 },
    { 0xaa4c881c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304c8821, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fc8828, 78 },
    { 0xf0cc8828, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97c882c, 84 },
    { 0xcecc882c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cec8832, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aac8836, 33 },
    { 0xb8bc8836, 34 },
    { 0x4bbc8836, 98 },
    { 0x257c8839, 84 },
    { 0x1ecc883a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc883d, 34 },
    { 0x43fc883e, 33 },
    { 0xef7c883f, 100 },
    { 0x649c8840, 93 },
    { 0xfa1c8840, 95 },
    { 0x9c8c883f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54c8845, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec8852, 33 },
    { 0x30ac8852, 95 },
    { 0x57fc8854, 87 },
    { 0, 0 },
    { 0x4c8c8856, 95 },
    { 0xdcec8856, 23 },
    { 0x7b9c8858, 90 },
    { 0x558c8859, 95 },
    { 0, 0 },
    { 0xa14c885b, 90 },
    { 0x92ac885c, 33 },
    { 0, 0 },
    { 0xd97c885e, 43 },
    { 0x672c885f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258c8864, 90 },
    { 0x2e8c8864, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677c886b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901c8878, 61 },
    { 0x48fc8879, 98 },
    { 0xa42c887a, 90 },
    { 0x49fc8878, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1c8880, 100 },
    { 0x41cc8880, 90 },
    { 0x43ac8880, 52 },
    { 0, 0 },
    { 0x5e7c8884, 76 },
    { 0x61bc8885, 98 },
    { 0xaa4c8886, 93 },
    { 0xae7c8887, 98 },
    { 0x8c6c8888, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9c8890, 90 },
    { 0x570c8890, 99 },
    { 0xef9c8892, 76 },
    { 0xab3c8893, 33 },
    { 0x78cc8894, 23 },
    { 0xe22c8892, 91 },
    { 0xf2fc8893, 33 },
    { 0, 0 },
    { 0x94dc8898, 55 },
    { 0, 0 },
    { 0x841c889a, 76 },
    { 0, 0 },
    { 0x337c889c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982c88a0, 81 },
    { 0x4e3c88a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2c88a4, 46 },
    { 0, 0 },
    { 0x551c88a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128c88ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c88b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bc88b9, 75 },
    { 0xdb5c88b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbac88be, 67 },
    { 0xdb0c88bf, 98 },
    { 0x993c88bf, 61 },
    { 0x5d0c88c0, 23 },
    { 0x9c4c88c2, 98 },
    { 0x9d1c88c3, 84 },
    { 0xe6ec88c4, 100 },
    { 0xf8ac88be, 98 },
    { 0x5c9c88c6, 17 },
    { 0xccfc88c7, 33 },
    { 0x76ec88c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0c88ce, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1c88d6, 23 },
    { 0x586c88d7, 90 },
    { 0x2c7c88d7, 90 },
    { 0xc0dc88d9, 74 },
    { 0x3e9c88da, 90 },
    { 0xb24c88db, 98 },
    { 0xb9bc88dc, 90 },
    { 0x70dc88dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6c88e2, 33 },
    { 0x9f6c88e3, 90 },
    { 0, 0 },
    { 0xa2fc88e5, 33 },
    { 0xea4c88e6, 33 },
    { 0x92ac88e5, 100 },
    { 0x725c88e8, 98 },
    { 0, 0 },
    { 0xf0dc88ea, 40 },
    { 0xd9dc88eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715c88ef, 61 },
    { 0x2b2c88f0, 55 },
    { 0, 0 },
    { 0xdacc88f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6c88f7, 55 },
    { 0x6bfc88f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75c88fb, 33 },
    { 0xdc4c88fc, 90 },
    { 0x20c88fb, 33 },
    { 0x13cc88fe, 90 },
    { 0, 0 },
    { 0xe07c8900, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cc8903, 90 },
    { 0xd54c8903, 95 },
    { 0, 0 },
    { 0xfcec8906, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55c890a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4c890d, 95 },
    { 0xd17c890e, 55 },
    { 0x435c890f, 76 },
    { 0xaa7c890e, 23 },
    { 0, 0 },
    { 0x692c8912, 66 },
    { 0, 0 },
    { 0x20fc8914, 90 },
    { 0xa9c8914, 95 },
    { 0x7f9c8916, 61 },
    { 0x3cec8917, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6c891b, 90 },
    { 0xdddc891b, 55 },
    { 0x96bc891d, 95 },
    { 0x989c891e, 100 },
    { 0xfac891e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3c8922, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c8925, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5c892c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8c8936, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9c893b, 95 },
    { 0, 0 },
    { 0x72cc893d, 90 },
    { 0x75ac893e, 96 },
    { 0x4f7c893e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53c8945, 93 },
    { 0, 0 },
    { 0x9c8c8947, 55 },
    { 0x3bfc8948, 98 },
    { 0x67bc8949, 41 },
    { 0, 0 },
    { 0xa35c894b, 33 },
    { 0x12ac894b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dc8950, 90 },
    { 0, 0 },
    { 0x43cc8952, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11c8956, 33 },
    { 0x7c9c8956, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738c895e, 81 },
    { 0xdf0c895f, 53 },
    { 0xbbec8960, 90 },
    { 0, 0 },
    { 0x467c8962, 6 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844c8967, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fc896a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14c8970, 52 },
    { 0xaddc8971, 98 },
    { 0x5f2c8972, 88 },
    { 0xc82c8971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458c8979, 43 },
    { 0x31c897a, 96 },
    { 0xd39c897b, 55 },
    { 0xa3dc897b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2c8983, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ec898e, 61 },
    { 0x4cac898e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafc8994, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289c8999, 99 },
    { 0x7b8c8999, 75 },
    { 0x8dbc899b, 80 },
    { 0x5cbc899b, 52 },
    { 0x187c8999, 100 },
    { 0xaffc899e, 55 },
    { 0xf8c899e, 61 },
    { 0x135c89a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ac89a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ac89a7, 98 },
    { 0x157c89a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fec89af, 75 },
    { 0, 0 },
    { 0x57dc89b1, 61 },
    { 0x7ac89b2, 75 },
    { 0, 0 },
    { 0x425c89b4, 33 },
    { 0x4acc89b5, 33 },
    { 0x8bdc89b6, 61 },
    { 0x300c89b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8c89ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fc89be, 44 },
    { 0, 0 },
    { 0x5ac89c0, 34 },
    { 0xba2c89c1, 90 },
    { 0, 0 },
    { 0xa65c89c3, 88 },
    { 0, 0 },
    { 0x150c89c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1c89cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dc89d0, 90 },
    { 0x4ec89d1, 95 },
    { 0x94ac89d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9c89d6, 33 },
    { 0, 0 },
    { 0xf22c89d8, 98 },
    { 0xebbc89d9, 33 },
    { 0x6f3c89d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fc89df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc89e2, 36 },
    { 0x9bec89e3, 23 },
    { 0, 0 },
    { 0x8b2c89e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dc89ed, 93 },
    { 0, 0 },
    { 0x1aac89ef, 77 },
    { 0xe85c89f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ac89f3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3c89f8, 90 },
    { 0xe66c89f9, 55 },
    { 0, 0 },
    { 0xb28c89fb, 83 },
    { 0x8aac89fb, 76 },
    { 0, 0 },
    { 0x8f8c89fe, 17 },
    { 0, 0 },
    { 0xf90c8a00, 33 },
    { 0xbd5c8a01, 46 },
    { 0xbe2c8a02, 33 },
    { 0, 0 },
    { 0x8b2c8a04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ec8a07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x528c8a0a, 23 },
    { 0, 0 },
    { 0x673c8a0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cc8a0f, 33 },
    { 0xee5c8a10, 66 },
    { 0xc0fc8a10, 90 },
    { 0x895c8a11, 34 },
    { 0x22c8a10, 98 },
    { 0xeb1c8a14, 90 },
    { 0xebec8a15, 33 },
    { 0x623c8a14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ac8a1c, 76 },
    { 0x7a6c8a1c, 90 },
    { 0, 0 },
    { 0x7e4c8a1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadc8a25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48c8a2a, 76 },
    { 0xff2c8a2b, 23 },
    { 0x150c8a2c, 33 },
    { 0xb95c8a2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9c8a33, 23 },
    { 0xc6dc8a34, 100 },
    { 0, 0 },
    { 0xb33c8a36, 81 },
    { 0x6d1c8a36, 23 },
    { 0x84cc8a36, 55 },
    { 0x8f1c8a39, 76 },
    { 0, 0 },
    { 0x6cc8a3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1c8a3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98c8a41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dc8a45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5c8a49, 61 },
    { 0, 0 },
    { 0xaa1c8a4b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1c8a51, 95 },
    { 0x837c8a51, 99 },
    { 0x8d9c8a53, 47 },
    { 0xfa1c8a54, 55 },
    { 0x185c8a55, 55 },
    { 0xfcec8a55, 75 },
    { 0xd91c8a53, 17 },
    { 0x8dcc8a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373c8a5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbc8a60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bc8a68, 95 },
    { 0x273c8a68, 78 },
    { 0x4a2c8a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcc8a6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefc8a71, 36 },
    { 0x2d2c8a72, 33 },
    { 0xcaec8a73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c8a76, 65 },
    { 0x4d2c8a76, 88 },
    { 0xc6cc8a77, 78 },
    { 0xc65c8a78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dc8a82, 90 },
    { 0xcdc8a83, 33 },
    { 0xc2c8a84, 76 },
    { 0, 0 },
    { 0x33ec8a86, 95 },
    { 0, 0 },
    { 0xd0dc8a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bc8a8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5c8a90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c8a97, 33 },
    { 0xb9bc8a98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5c8aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbac8aa4, 61 },
    { 0x4afc8aa5, 23 },
    { 0, 0 },
    { 0x21dc8aa7, 33 },
    { 0x18bc8aa7, 90 },
    { 0xb50c8aa9, 61 },
    { 0, 0 },
    { 0xe71c8aab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x370c8aae, 90 },
    { 0, 0 },
    { 0x434c8ab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ac8ab3, 90 },
    { 0, 0 },
    { 0xf3ac8ab5, 61 },
    { 0xcbfc8ab6, 76 },
    { 0x1d6c8ab7, 66 },
    { 0x7d5c8ab7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284c8abd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481c8ac3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbc8ac6, 55 },
    { 0x322c8ac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ec8aca, 95 },
    { 0, 0 },
    { 0xc38c8acc, 98 },
    { 0, 0 },
    { 0xd2bc8ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72c8ad3, 98 },
    { 0x58dc8ad4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x518c8ad7, 95 },
    { 0, 0 },
    { 0x3d3c8ad9, 95 },
    { 0, 0 },
    { 0x77fc8adb, 95 },
    { 0, 0 },
    { 0x380c8add, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58c8ae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x397c8aea, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764c8af1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccc8af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x219c8af8, 88 },
    { 0xf3bc8af9, 55 },
    { 0x374c8afa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60c8aff, 95 },
    { 0, 0 },
    { 0x714c8b01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70c8b04, 66 },
    { 0x68dc8b05, 34 },
    { 0, 0 },
    { 0xca4c8b07, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cc8b0b, 65 },
    { 0xd84c8b0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x663c8b0f, 33 },
    { 0x253c8b10, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1c8b13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dc8b18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ac8b1d, 61 },
    { 0x5a9c8b1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ac8b25, 65 },
    { 0x935c8b25, 76 },
    { 0x4aac8b27, 61 },
    { 0, 0 },
    { 0x83fc8b29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x893c8b2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1c8b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21c8b34, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8c8b3a, 99 },
    { 0xa6c8b3b, 40 },
    { 0x20bc8b3c, 98 },
    { 0x8bac8b3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57c8b43, 98 },
    { 0x470c8b43, 90 },
    { 0x360c8b43, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afc8b48, 98 },
    { 0xd3cc8b48, 23 },
    { 0x1e0c8b48, 95 },
    { 0xbf7c8b4b, 55 },
    { 0xa4bc8b4b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbc8b4f, 90 },
    { 0, 0 },
    { 0x435c8b51, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x99c8b54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ec8b59, 90 },
    { 0x466c8b5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1c8b60, 95 },
    { 0x18bc8b61, 93 },
    { 0xa35c8b61, 98 },
    { 0x2ecc8b63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x401c8b66, 81 },
    { 0x6c1c8b67, 34 },
    { 0x8cac8b67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11c8b6c, 78 },
    { 0xb2ac8b6d, 33 },
    { 0, 0 },
    { 0xcddc8b6f, 84 },
    { 0xdbdc8b70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x310c8b73, 98 },
    { 0x6efc8b73, 98 },
    { 0, 0 },
    { 0x7c0c8b76, 91 },
    { 0, 0 },
    { 0xec3c8b78, 66 },
    { 0xd37c8b79, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66c8b7c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0c8b7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cc8b83, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x651c8b86, 55 },
    { 0xf72c8b87, 90 },
    { 0, 0 },
    { 0x52fc8b89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23c8b8c, 98 },
    { 0, 0 },
    { 0x540c8b8e, 85 },
    { 0, 0 },
    { 0x49dc8b90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75c8b93, 61 },
    { 0xc7c8b94, 34 },
    { 0x33dc8b94, 95 },
    { 0x48fc8b94, 94 },
    { 0, 0 },
    { 0x76bc8b98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c8b9c, 23 },
    { 0x202c8b9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4c8ba0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367c8ba4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46c8ba8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900c8baf, 78 },
    { 0xd81c8bb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fc8bb5, 55 },
    { 0x20bc8bb6, 48 },
    { 0x52c8bb7, 88 },
    { 0x84ec8bb8, 81 },
    { 0x97cc8bb6, 55 },
    { 0xf45c8bb8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2c8bbe, 61 },
    { 0, 0 },
    { 0x1ecc8bc0, 67 },
    { 0xdc3c8bc0, 90 },
    { 0x3cac8bc2, 33 },
    { 0xac6c8bc3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0c8bce, 34 },
    { 0x5bbc8bcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1c8bd2, 91 },
    { 0, 0 },
    { 0xcacc8bd4, 33 },
    { 0, 0 },
    { 0x12dc8bd6, 98 },
    { 0xf79c8bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x291c8bda, 85 },
    { 0xb17c8bda, 61 },
    { 0x51c8bdb, 55 },
    { 0xecdc8bdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78c8be8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70c8bec, 17 },
    { 0, 0 },
    { 0x109c8bee, 88 },
    { 0x3c8c8bee, 65 },
    { 0x74bc8bf0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5c8bf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4c8bfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c8c00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130c8c04, 95 },
    { 0x2d3c8c05, 98 },
    { 0xa1ac8c05, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92c8c09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839c8c0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99c8c11, 67 },
    { 0xcb5c8c12, 91 },
    { 0xf09c8c11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cc8c17, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3c8c1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c8c21, 33 },
    { 0xa5cc8c22, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512c8c2b, 61 },
    { 0x7d9c8c2c, 98 },
    { 0, 0 },
    { 0xceac8c2e, 23 },
    { 0x1c5c8c2f, 33 },
    { 0xdb1c8c2e, 95 },
    { 0, 0 },
    { 0xe32c8c32, 43 },
    { 0, 0 },
    { 0xba7c8c34, 76 },
    { 0x6fbc8c35, 43 },
    { 0xfe4c8c36, 95 },
    { 0x6d4c8c37, 77 },
    { 0x318c8c37, 33 },
    { 0x979c8c38, 77 },
    { 0x1c7c8c34, 84 },
    { 0xd81c8c3b, 90 },
    { 0, 0 },
    { 0x792c8c3d, 22 },
    { 0x8f8c8c3e, 99 },
    { 0x57ec8c3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bc8c4d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x852c8c50, 36 },
    { 0, 0 },
    { 0x261c8c52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecc8c56, 33 },
    { 0x949c8c57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ec8c5a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbc8c60, 90 },
    { 0xe34c8c61, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c8c65, 55 },
    { 0x4b5c8c65, 87 },
    { 0x4a7c8c67, 75 },
    { 0xc37c8c68, 85 },
    { 0xe92c8c69, 91 },
    { 0xb9cc8c68, 90 },
    { 0xe29c8c6b, 64 },
    { 0xeb8c8c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7c8c70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7c8c78, 34 },
    { 0xfbac8c79, 98 },
    { 0, 0 },
    { 0xfabc8c7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5c8c7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcc8c83, 84 },
    { 0x951c8c84, 17 },
    { 0x1efc8c85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbc8c91, 78 },
    { 0x67ec8c91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0c8c96, 2 },
    { 0xec7c8c97, 98 },
    { 0xc68c8c96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x269c8c9b, 23 },
    { 0, 0 },
    { 0x3fdc8c9d, 83 },
    { 0x842c8c9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x776c8ca1, 38 },
    { 0xe47c8ca1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14c8ca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818c8ca9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac8cac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1c8cb1, 23 },
    { 0xa9fc8cb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4c8cb7, 23 },
    { 0, 0 },
    { 0x1c0c8cb9, 63 },
    { 0, 0 },
    { 0x611c8cbb, 66 },
    { 0x19dc8cbb, 55 },
    { 0, 0 },
    { 0x7e9c8cbe, 95 },
    { 0, 0 },
    { 0x926c8cc0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2c8cc3, 76 },
    { 0, 0 },
    { 0xbb4c8cc5, 33 },
    { 0, 0 },
    { 0x5e8c8cc7, 91 },
    { 0, 0 },
    { 0xd24c8cc9, 33 },
    { 0, 0 },
    { 0xe1fc8ccb, 61 },
    { 0x2b2c8ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7c8ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3c8cd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16c8cde, 90 },
    { 0, 0 },
    { 0xaf8c8ce0, 68 },
    { 0, 0 },
    { 0x149c8ce2, 36 },
    { 0, 0 },
    { 0x6e5c8ce4, 95 },
    { 0x485c8ce5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7c8cec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbac8cf3, 43 },
    { 0, 0 },
    { 0x5a9c8cf5, 98 },
    { 0, 0 },
    { 0x56c8cf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6c8cfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1c8cfe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dc8d06, 90 },
    { 0x264c8d07, 95 },
    { 0, 0 },
    { 0x673c8d09, 93 },
    { 0xe17c8d0a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccc8d0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8c8d10, 98 },
    { 0xeecc8d10, 95 },
    { 0, 0 },
    { 0x8aec8d13, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749c8d18, 93 },
    { 0xf40c8d18, 95 },
    { 0, 0 },
    { 0xdfac8d1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dc8d22, 23 },
    { 0, 0 },
    { 0x90bc8d24, 78 },
    { 0xb9cc8d24, 69 },
    { 0x32fc8d24, 95 },
    { 0, 0 },
    { 0xebdc8d28, 43 },
    { 0x79ac8d28, 23 },
    { 0xeb8c8d29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efc8d2e, 23 },
    { 0x69bc8d2f, 61 },
    { 0xb05c8d30, 98 },
    { 0x66c8d31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaec8d36, 61 },
    { 0, 0 },
    { 0xd39c8d38, 61 },
    { 0xa44c8d39, 95 },
    { 0x36ec8d39, 90 },
    { 0, 0 },
    { 0xfaec8d3c, 90 },
    { 0x512c8d3c, 23 },
    { 0xb5c8d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9c8d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdc8d45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edc8d4b, 90 },
    { 0x8dec8d4b, 61 },
    { 0xc6cc8d4b, 90 },
    { 0, 0 },
    { 0xd6bc8d4f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ec8d52, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bc8d55, 91 },
    { 0x7ccc8d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ec8d59, 78 },
    { 0x48fc8d5a, 69 },
    { 0x738c8d5b, 95 },
    { 0x67fc8d5b, 61 },
    { 0x5fec8d5b, 93 },
    { 0x424c8d59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906c8d6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbc8d6e, 98 },
    { 0xc2c8d6e, 33 },
    { 0x716c8d6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x533c8d73, 99 },
    { 0xefc8d74, 77 },
    { 0, 0 },
    { 0xb19c8d76, 98 },
    { 0xb3fc8d76, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x788c8d7a, 61 },
    { 0, 0 },
    { 0xc34c8d7c, 34 },
    { 0xf64c8d7d, 23 },
    { 0, 0 },
    { 0x337c8d7f, 34 },
    { 0, 0 },
    { 0xec8d81, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919c8d87, 88 },
    { 0xca8c8d88, 33 },
    { 0x61cc8d88, 78 },
    { 0x8c6c8d87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7c8d8f, 61 },
    { 0, 0 },
    { 0x757c8d91, 84 },
    { 0, 0 },
    { 0x661c8d93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45c8d96, 95 },
    { 0xa9cc8d97, 23 },
    { 0xf54c8d98, 61 },
    { 0, 0 },
    { 0xeb7c8d9a, 90 },
    { 0, 0 },
    { 0x282c8d9c, 98 },
    { 0x427c8d9d, 61 },
    { 0x55ec8d9e, 90 },
    { 0xc61c8d9e, 99 },
    { 0x7e6c8da0, 61 },
    { 0, 0 },
    { 0x1d7c8da2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02c8da9, 95 },
    { 0x43c8daa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dec8dad, 98 },
    { 0, 0 },
    { 0x3f8c8daf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cac8db4, 93 },
    { 0xfc7c8db5, 61 },
    { 0xfc0c8db6, 95 },
    { 0x9c7c8db7, 23 },
    { 0x970c8db5, 90 },
    { 0x8f7c8db9, 23 },
    { 0x789c8db6, 23 },
    { 0, 0 },
    { 0x6c6c8dbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471c8dc0, 33 },
    { 0x8c2c8dc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914c8dc9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805c8dd3, 33 },
    { 0xe84c8dd3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32c8dd8, 77 },
    { 0, 0 },
    { 0xd34c8dda, 76 },
    { 0xd27c8ddb, 76 },
    { 0x90dc8ddc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3c8ddf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658c8de4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6c8de8, 33 },
    { 0xfb0c8de8, 33 },
    { 0x669c8dea, 95 },
    { 0x279c8dea, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fc8df3, 90 },
    { 0, 0 },
    { 0x572c8df5, 93 },
    { 0, 0 },
    { 0x400c8df7, 98 },
    { 0xe68c8df7, 90 },
    { 0x50ec8df9, 23 },
    { 0xcadc8df9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x733c8dfd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31c8e01, 43 },
    { 0x5f6c8e02, 90 },
    { 0x6ccc8e02, 98 },
    { 0, 0 },
    { 0x43c8e05, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31c8e08, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c8e0e, 76 },
    { 0, 0 },
    { 0x3bec8e10, 95 },
    { 0x3bcc8e11, 23 },
    { 0xa90c8e10, 23 },
    { 0xf36c8e13, 98 },
    { 0x156c8e14, 98 },
    { 0, 0 },
    { 0xce0c8e16, 95 },
    { 0xbbbc8e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcc8e1a, 33 },
    { 0xbeec8e1b, 95 },
    { 0x5a2c8e1c, 34 },
    { 0x516c8e1a, 94 },
    { 0x89fc8e1a, 91 },
    { 0xae8c8e1f, 90 },
    { 0x5b3c8e1d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680c8e24, 98 },
    { 0xa45c8e24, 90 },
    { 0xdbbc8e26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106c8e2a, 66 },
    { 0x1c2c8e2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5c8e2f, 46 },
    { 0, 0 },
    { 0x3c5c8e31, 33 },
    { 0, 0 },
    { 0x9e9c8e33, 23 },
    { 0, 0 },
    { 0x9a8c8e35, 31 },
    { 0x2fac8e36, 23 },
    { 0x448c8e36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac8e3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cc8e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7c8e4b, 90 },
    { 0x15bc8e4b, 90 },
    { 0x8e4c8e4d, 99 },
    { 0x2efc8e4c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4c8e52, 52 },
    { 0xf94c8e53, 95 },
    { 0x293c8e54, 77 },
    { 0x74c8e53, 33 },
    { 0xc82c8e56, 90 },
    { 0xacdc8e56, 33 },
    { 0x271c8e56, 98 },
    { 0xcdcc8e56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708c8e5f, 98 },
    { 0, 0 },
    { 0x893c8e61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ac8e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620c8e6a, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x137c8e6d, 90 },
    { 0x6dec8e6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dc8e71, 93 },
    { 0x431c8e71, 98 },
    { 0x4d0c8e72, 33 },
    { 0xf32c8e74, 68 },
    { 0, 0 },
    { 0xd02c8e76, 34 },
    { 0x9d0c8e76, 95 },
    { 0x787c8e78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dc8e7b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc8e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24c8e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x998c8e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89c8e90, 43 },
    { 0, 0 },
    { 0x6c8e92, 52 },
    { 0, 0 },
    { 0x7c6c8e94, 93 },
    { 0x44fc8e95, 98 },
    { 0, 0 },
    { 0x45cc8e97, 61 },
    { 0, 0 },
    { 0xae1c8e99, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dc8e9f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bc8ea2, 55 },
    { 0, 0 },
    { 0xd50c8ea4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12c8ea9, 33 },
    { 0, 0 },
    { 0xf0fc8eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc8eae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26c8eb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552c8eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x874c8eb9, 84 },
    { 0xc7ac8eba, 43 },
    { 0xacfc8eba, 95 },
    { 0x125c8ebc, 66 },
    { 0x647c8eba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6c8ec3, 95 },
    { 0, 0 },
    { 0x575c8ec5, 100 },
    { 0x4bec8ec6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20c8ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3c8ecd, 93 },
    { 0, 0 },
    { 0xd4ac8ecf, 34 },
    { 0x52c8ecf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6c8ed4, 76 },
    { 0xbdc8ed5, 52 },
    { 0, 0 },
    { 0x937c8ed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0c8eda, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58c8edd, 100 },
    { 0x360c8ede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fc8ee4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdac8ee8, 91 },
    { 0x4a5c8ee9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94c8eec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923c8ef1, 33 },
    { 0xbfec8ef2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9c8ef5, 33 },
    { 0xa23c8ef6, 55 },
    { 0, 0 },
    { 0x167c8ef8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bc8eff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceac8f07, 90 },
    { 0, 0 },
    { 0xf1cc8f09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabec8f0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765c8f10, 84 },
    { 0x44cc8f10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ac8f15, 81 },
    { 0x34bc8f16, 23 },
    { 0xfa4c8f17, 95 },
    { 0, 0 },
    { 0x962c8f19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0c8f1e, 99 },
    { 0x856c8f1f, 61 },
    { 0xd8cc8f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x888c8f23, 53 },
    { 0x752c8f24, 61 },
    { 0x8afc8f25, 90 },
    { 0xa2ec8f24, 31 },
    { 0x22fc8f27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ec8f2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bc8f2d, 98 },
    { 0xba3c8f2e, 98 },
    { 0x372c8f2f, 34 },
    { 0, 0 },
    { 0xfc3c8f31, 90 },
    { 0x9f5c8f31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39c8f36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3c8f39, 84 },
    { 0x391c8f3a, 90 },
    { 0, 0 },
    { 0x62bc8f3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ec8f41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886c8f46, 80 },
    { 0x8f3c8f46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7c8f4f, 93 },
    { 0, 0 },
    { 0x733c8f51, 90 },
    { 0, 0 },
    { 0xe17c8f53, 95 },
    { 0xe21c8f54, 33 },
    { 0, 0 },
    { 0x25dc8f56, 43 },
    { 0x7e2c8f57, 61 },
    { 0xc4ac8f58, 81 },
    { 0xca0c8f59, 36 },
    { 0x732c8f5a, 23 },
    { 0x2a0c8f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95c8f5f, 43 },
    { 0, 0 },
    { 0x2fdc8f61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dc8f69, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7c8f6c, 66 },
    { 0, 0 },
    { 0x148c8f6e, 65 },
    { 0xd83c8f6f, 55 },
    { 0x789c8f6f, 53 },
    { 0x69ac8f71, 43 },
    { 0, 0 },
    { 0x24ac8f73, 33 },
    { 0xe7cc8f74, 34 },
    { 0xeecc8f74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73c8f7a, 98 },
    { 0x550c8f7b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445c8f81, 90 },
    { 0, 0 },
    { 0x5f3c8f83, 23 },
    { 0x457c8f84, 76 },
    { 0, 0 },
    { 0x5bcc8f86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4c8f89, 61 },
    { 0, 0 },
    { 0x814c8f8b, 95 },
    { 0, 0 },
    { 0x6b1c8f8d, 90 },
    { 0x32cc8f8e, 78 },
    { 0x551c8f8f, 97 },
    { 0x51ac8f90, 40 },
    { 0x93dc8f8e, 98 },
    { 0, 0 },
    { 0x67cc8f93, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8f96, 99 },
    { 0xc3ec8f97, 90 },
    { 0xd38c8f98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983c8fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bc8fa4, 33 },
    { 0xdcbc8fa5, 33 },
    { 0xa17c8fa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dc8fac, 61 },
    { 0x218c8fac, 23 },
    { 0x5ccc8fae, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecc8fba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dc8fc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5c8fc7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dc8fca, 52 },
    { 0, 0 },
    { 0x177c8fcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fc8fd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ac8fd5, 99 },
    { 0, 0 },
    { 0x250c8fd7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439c8fdb, 78 },
    { 0x86bc8fdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33c8fe0, 76 },
    { 0xd6ec8fe0, 85 },
    { 0x691c8fe0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ac8fe9, 23 },
    { 0, 0 },
    { 0x6f7c8feb, 23 },
    { 0, 0 },
    { 0xd41c8fed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdac8ff0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9c8ff4, 90 },
    { 0x807c8ff4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbc8ff9, 68 },
    { 0x65fc8ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dc9001, 87 },
    { 0, 0 },
    { 0xc8ec9003, 43 },
    { 0xa4ac9004, 90 },
    { 0, 0 },
    { 0x57fc9006, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883c900a, 52 },
    { 0x92c900b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdc900f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dc9014, 93 },
    { 0x601c9015, 84 },
    { 0xc15c9016, 33 },
    { 0xe1c9015, 33 },
    { 0xa27c9018, 66 },
    { 0x87fc9017, 90 },
    { 0xa19c9018, 23 },
    { 0x650c901b, 33 },
    { 0x801c9016, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fc9020, 95 },
    { 0, 0 },
    { 0xd13c9022, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2c9025, 43 },
    { 0, 0 },
    { 0xa33c9027, 95 },
    { 0x2c7c9028, 100 },
    { 0x367c9029, 61 },
    { 0x660c9028, 61 },
    { 0xdd0c9029, 46 },
    { 0xb61c902c, 95 },
    { 0x641c902b, 33 },
    { 0x79fc902e, 67 },
    { 0x1aac902f, 34 },
    { 0, 0 },
    { 0x2bcc9031, 90 },
    { 0xf9c9032, 55 },
    { 0xae0c9031, 17 },
    { 0xbefc9031, 34 },
    { 0x2bfc9034, 33 },
    { 0, 0 },
    { 0x179c9037, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bc903a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95c9040, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44c9043, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5c9046, 93 },
    { 0xf12c9047, 35 },
    { 0x5c5c9048, 23 },
    { 0x77bc9049, 61 },
    { 0x451c904a, 98 },
    { 0x67bc9048, 88 },
    { 0, 0 },
    { 0x898c904d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97c9052, 98 },
    { 0x9ecc9053, 33 },
    { 0xc82c9054, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebc9057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x691c905a, 61 },
    { 0xdb3c905a, 23 },
    { 0x706c905b, 95 },
    { 0, 0 },
    { 0xc05c905e, 87 },
    { 0x70ac905e, 78 },
    { 0, 0 },
    { 0x4d6c9061, 43 },
    { 0x8c8c9061, 23 },
    { 0xa6bc9062, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c9066, 90 },
    { 0x97bc9067, 81 },
    { 0, 0 },
    { 0xb52c9069, 85 },
    { 0x7f2c906a, 98 },
    { 0, 0 },
    { 0x3c5c906c, 61 },
    { 0x19dc906c, 90 },
    { 0x438c906e, 34 },
    { 0x273c906f, 33 },
    { 0x734c9070, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ac9073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70c9077, 55 },
    { 0, 0 },
    { 0x44ac9079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18c907c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ac9082, 95 },
    { 0x972c9082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3c9088, 90 },
    { 0xffcc9089, 90 },
    { 0x5bc9088, 40 },
    { 0x507c908a, 61 },
    { 0x530c908c, 98 },
    { 0xba8c908d, 26 },
    { 0x941c908d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58c9093, 52 },
    { 0x548c9094, 33 },
    { 0xdc5c9095, 33 },
    { 0xe8c9096, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebc90a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bc90a6, 61 },
    { 0xefc90a7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc90aa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5c90ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x718c90b0, 91 },
    { 0xd73c90b1, 90 },
    { 0x627c90b2, 88 },
    { 0, 0 },
    { 0xf2bc90b4, 88 },
    { 0, 0 },
    { 0x1ecc90b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153c90bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0c90be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cc90c3, 33 },
    { 0xe85c90c3, 87 },
    { 0, 0 },
    { 0xd78c90c6, 55 },
    { 0xb5cc90c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x860c90ca, 43 },
    { 0, 0 },
    { 0x540c90cc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60c90cf, 23 },
    { 0, 0 },
    { 0xe5cc90d1, 56 },
    { 0, 0 },
    { 0x277c90d3, 61 },
    { 0x73cc90d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d8c90d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafc90db, 88 },
    { 0x865c90db, 34 },
    { 0xf59c90dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4c90e2, 33 },
    { 0, 0 },
    { 0x481c90e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6c90ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bc90ef, 61 },
    { 0x7e9c90f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edc90f7, 61 },
    { 0, 0 },
    { 0x595c90f9, 61 },
    { 0, 0 },
    { 0xf6ec90fb, 61 },
    { 0, 0 },
    { 0x78ec90fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x280c9100, 85 },
    { 0, 0 },
    { 0xb57c9102, 61 },
    { 0, 0 },
    { 0x6c2c9104, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x47c9107, 95 },
    { 0x7bbc9107, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734c910d, 34 },
    { 0x9c8c910e, 61 },
    { 0x804c910f, 33 },
    { 0xd13c910d, 23 },
    { 0xa33c9111, 85 },
    { 0x9fac9112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcc911b, 90 },
    { 0x485c911c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bc912f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ec913b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17c9140, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10c9144, 76 },
    { 0xc4bc9144, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9c9148, 61 },
    { 0x8a0c9148, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fc914c, 93 },
    { 0x779c914d, 90 },
    { 0x71ac914d, 90 },
    { 0x49fc914e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ec915f, 85 },
    { 0x29cc915f, 55 },
    { 0x2b4c9161, 78 },
    { 0, 0 },
    { 0x611c9163, 23 },
    { 0xad6c9164, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4c9167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaec916c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356c917b, 90 },
    { 0xcdcc917b, 23 },
    { 0x38bc917d, 68 },
    { 0x703c917e, 23 },
    { 0x24ac917f, 33 },
    { 0x3c4c9180, 75 },
    { 0, 0 },
    { 0x7adc9182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fc9187, 95 },
    { 0xbe8c9187, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7c918c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c918f, 90 },
    { 0x91c9190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60c9193, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccc9196, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x282c9199, 17 },
    { 0, 0 },
    { 0x814c919b, 100 },
    { 0x4c7c919b, 33 },
    { 0xa6dc919c, 93 },
    { 0xfffc919e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bc91a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906c91a5, 66 },
    { 0xcb3c91a6, 90 },
    { 0x4a5c91a7, 61 },
    { 0xb43c91a6, 98 },
    { 0, 0 },
    { 0x1d0c91aa, 55 },
    { 0xf99c91aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ec91b4, 61 },
    { 0x98c91b4, 90 },
    { 0x1c8c91b4, 99 },
    { 0xcf2c91b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88c91ba, 34 },
    { 0, 0 },
    { 0x6f1c91bc, 99 },
    { 0xe51c91bc, 46 },
    { 0x279c91be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00c91c3, 33 },
    { 0xdc3c91c3, 33 },
    { 0x17c91c4, 55 },
    { 0x4c3c91c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x222c91c9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0c91cd, 55 },
    { 0x27ac91ce, 61 },
    { 0x3d6c91cd, 23 },
    { 0x129c91d0, 49 },
    { 0xa66c91d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907c91d7, 81 },
    { 0x285c91d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2c91de, 87 },
    { 0x6d8c91de, 33 },
    { 0x764c91de, 96 },
    { 0, 0 },
    { 0x56cc91e2, 90 },
    { 0x37ac91e3, 61 },
    { 0x96ec91e3, 55 },
    { 0xdcc91e5, 34 },
    { 0x6e6c91e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39c91eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c91ef, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450c91f7, 76 },
    { 0, 0 },
    { 0x256c91f9, 52 },
    { 0, 0 },
    { 0xeecc91fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ec9201, 98 },
    { 0x546c9201, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c920a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6c920d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51c9210, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444c9217, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6c921a, 95 },
    { 0, 0 },
    { 0xe79c921c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c9222, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddc9227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5c922a, 93 },
    { 0xeafc922a, 61 },
    { 0x9cac922b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcec922f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43c9233, 33 },
    { 0x4f2c9233, 33 },
    { 0x2fac9234, 34 },
    { 0x881c9235, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7c9239, 99 },
    { 0, 0 },
    { 0xee4c923b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbc923e, 61 },
    { 0, 0 },
    { 0x2a0c9240, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705c9244, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ac9247, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8c924b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3c9250, 33 },
    { 0, 0 },
    { 0x14dc9252, 46 },
    { 0, 0 },
    { 0xbedc9254, 23 },
    { 0xadbc9255, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc925e, 34 },
    { 0x3ddc925f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c9267, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816c926e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c9271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dc9275, 17 },
    { 0x363c9276, 90 },
    { 0, 0 },
    { 0x6c2c9278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3c927b, 34 },
    { 0, 0 },
    { 0x509c927d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7c9280, 61 },
    { 0xb13c9281, 76 },
    { 0xcec9282, 33 },
    { 0x403c9281, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbc928c, 61 },
    { 0x576c928d, 19 },
    { 0x346c928c, 61 },
    { 0xb1dc928f, 33 },
    { 0xb08c928d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ac929c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc929f, 23 },
    { 0, 0 },
    { 0x43ec92a1, 95 },
    { 0xf5ec92a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cc92a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebc92ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3c92af, 90 },
    { 0, 0 },
    { 0x5c0c92b1, 65 },
    { 0xb0ac92b2, 77 },
    { 0x361c92b2, 61 },
    { 0x26bc92b4, 100 },
    { 0, 0 },
    { 0x5eac92b6, 61 },
    { 0xbf2c92b7, 67 },
    { 0, 0 },
    { 0xdaac92b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcc92bd, 23 },
    { 0x646c92be, 61 },
    { 0xea0c92bd, 88 },
    { 0x304c92c0, 23 },
    { 0, 0 },
    { 0xd1dc92c2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709c92c6, 90 },
    { 0x16cc92c7, 91 },
    { 0x9fac92c8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94c92cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cc92ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3c92d3, 90 },
    { 0x897c92d3, 33 },
    { 0x662c92d3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x79c92d8, 33 },
    { 0x584c92d8, 67 },
    { 0, 0 },
    { 0x58dc92db, 95 },
    { 0x642c92db, 23 },
    { 0x80fc92dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x704c92e0, 98 },
    { 0x9bbc92e1, 23 },
    { 0xfafc92e2, 34 },
    { 0xeafc92e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6c92e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58c92eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1c92ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4c92f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323c92fe, 90 },
    { 0x872c92ff, 33 },
    { 0x5ddc9300, 99 },
    { 0x287c9301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8c9305, 46 },
    { 0xf57c9305, 43 },
    { 0x5a2c9307, 33 },
    { 0x7c2c9305, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eac930d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfc9313, 90 },
    { 0x7f0c9314, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ec9318, 61 },
    { 0x315c9319, 84 },
    { 0xfa4c931a, 61 },
    { 0x5b1c9318, 93 },
    { 0x233c931c, 77 },
    { 0x621c931d, 90 },
    { 0xcf7c931e, 98 },
    { 0x55ec931d, 85 },
    { 0, 0 },
    { 0x2a9c9321, 23 },
    { 0xf2bc9322, 34 },
    { 0, 0 },
    { 0x79ac9324, 98 },
    { 0, 0 },
    { 0x94fc9326, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59c932e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202c9334, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbc933b, 90 },
    { 0, 0 },
    { 0x545c933d, 23 },
    { 0x75ac933d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dc9344, 61 },
    { 0xc39c9345, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68c934b, 55 },
    { 0xa77c934c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x77c934f, 34 },
    { 0x579c934f, 69 },
    { 0xa36c9350, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103c9357, 43 },
    { 0, 0 },
    { 0x815c9359, 90 },
    { 0, 0 },
    { 0x623c935b, 75 },
    { 0xdc7c935c, 87 },
    { 0x553c935b, 48 },
    { 0x92cc935b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ec9365, 61 },
    { 0, 0 },
    { 0x454c9367, 90 },
    { 0, 0 },
    { 0x70dc9369, 98 },
    { 0x291c936a, 66 },
    { 0xe7cc936b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec936e, 98 },
    { 0x8e9c936f, 85 },
    { 0x291c936e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98c9373, 76 },
    { 0, 0 },
    { 0x18cc9375, 23 },
    { 0x34c9375, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ac937c, 98 },
    { 0, 0 },
    { 0xecec937e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1c9381, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x601c9384, 33 },
    { 0x39ac9385, 43 },
    { 0xa80c9386, 66 },
    { 0xa5ec9387, 90 },
    { 0, 0 },
    { 0x45fc9389, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x680c938c, 23 },
    { 0, 0 },
    { 0x236c938e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86c9396, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899c939a, 61 },
    { 0x43cc939b, 9 },
    { 0x373c939b, 95 },
    { 0x8cac939d, 66 },
    { 0x3b5c939e, 90 },
    { 0xb8ec939f, 46 },
    { 0x938c93a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0c93a3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497c93a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc93ab, 33 },
    { 0, 0 },
    { 0xb0ac93ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dc93b0, 88 },
    { 0x4c3c93b0, 55 },
    { 0, 0 },
    { 0x317c93b3, 100 },
    { 0xb71c93b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0c93bb, 33 },
    { 0, 0 },
    { 0xd76c93bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113c93c7, 55 },
    { 0x6bdc93c8, 95 },
    { 0, 0 },
    { 0xc20c93ca, 95 },
    { 0xe53c93cb, 61 },
    { 0, 0 },
    { 0x8f3c93cd, 95 },
    { 0xc57c93ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cc93d1, 84 },
    { 0, 0 },
    { 0x4c0c93d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924c93d7, 95 },
    { 0, 0 },
    { 0x652c93d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c93dc, 43 },
    { 0, 0 },
    { 0xf9fc93de, 33 },
    { 0x73cc93df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x472c93e2, 91 },
    { 0, 0 },
    { 0xda0c93e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bc93e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x316c93ef, 46 },
    { 0x378c93f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40c93f5, 33 },
    { 0x5d4c93f6, 23 },
    { 0xba5c93f7, 46 },
    { 0, 0 },
    { 0x380c93f9, 93 },
    { 0x669c93fa, 90 },
    { 0xe47c93fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530c9402, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4c940c, 33 },
    { 0xa12c940c, 90 },
    { 0, 0 },
    { 0x640c940f, 23 },
    { 0xb62c940f, 98 },
    { 0, 0 },
    { 0xfa4c9412, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97c9415, 36 },
    { 0, 0 },
    { 0x231c9417, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cc9421, 43 },
    { 0, 0 },
    { 0x343c9423, 76 },
    { 0xc32c9424, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13c9428, 95 },
    { 0x508c9429, 90 },
    { 0xe73c942a, 66 },
    { 0xbc3c9428, 17 },
    { 0xed6c942c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x303c942f, 91 },
    { 0, 0 },
    { 0x141c9431, 33 },
    { 0x9ebc9432, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0c9435, 76 },
    { 0xb95c9436, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefc9439, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacc943c, 33 },
    { 0, 0 },
    { 0x994c943e, 33 },
    { 0x209c943f, 53 },
    { 0xb21c943e, 46 },
    { 0xa8cc943e, 95 },
    { 0xbd5c943f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x827c9445, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502c944d, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0c9451, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fc9454, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c945d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x961c9464, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88c946c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27c9470, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7c9476, 90 },
    { 0x845c9477, 95 },
    { 0xb0c9478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x139c947b, 66 },
    { 0x7b2c947c, 99 },
    { 0xf85c947d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665c9482, 23 },
    { 0x283c9483, 91 },
    { 0xad4c9483, 90 },
    { 0x3c9483, 46 },
    { 0x239c9485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5c9489, 55 },
    { 0x149c9489, 100 },
    { 0xe81c9489, 98 },
    { 0x63bc948b, 100 },
    { 0xe61c948a, 66 },
    { 0x751c948c, 91 },
    { 0, 0 },
    { 0x3fec9490, 33 },
    { 0x6f6c9490, 90 },
    { 0x3c8c9490, 76 },
    { 0x856c9490, 76 },
    { 0x9f1c9494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0c9497, 33 },
    { 0, 0 },
    { 0xee6c9499, 55 },
    { 0x10ec949a, 95 },
    { 0, 0 },
    { 0xeeac949c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211c94a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebac94a9, 61 },
    { 0xa38c94aa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5c94ad, 92 },
    { 0xa4ac94ae, 90 },
    { 0x5f2c94af, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4c94b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862c94bd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ac94c1, 23 },
    { 0xae9c94c2, 95 },
    { 0, 0 },
    { 0x554c94c4, 43 },
    { 0, 0 },
    { 0xb38c94c6, 55 },
    { 0x4c2c94c7, 61 },
    { 0x86c94c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcc94cb, 95 },
    { 0xfc7c94cb, 90 },
    { 0xc45c94cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1c94d0, 23 },
    { 0, 0 },
    { 0xfebc94d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbc94da, 46 },
    { 0xc33c94db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5c94e0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x390c94e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebc94e6, 98 },
    { 0xaa7c94e7, 55 },
    { 0xa80c94e8, 33 },
    { 0, 0 },
    { 0x9a4c94ea, 52 },
    { 0x76ac94ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbc94f1, 100 },
    { 0, 0 },
    { 0xbe7c94f3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cc94f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dc9501, 95 },
    { 0, 0 },
    { 0x5dc9503, 61 },
    { 0xbb6c9503, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1c9507, 95 },
    { 0xe74c9508, 66 },
    { 0x25bc9508, 61 },
    { 0x317c9508, 99 },
    { 0x95fc950b, 91 },
    { 0, 0 },
    { 0x5d7c950d, 90 },
    { 0xf9ac950e, 90 },
    { 0x728c950f, 90 },
    { 0, 0 },
    { 0xf24c9511, 98 },
    { 0, 0 },
    { 0x1b1c9513, 95 },
    { 0x4a3c9514, 61 },
    { 0xb02c9514, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7c951d, 61 },
    { 0x3d5c951d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83c9524, 55 },
    { 0, 0 },
    { 0xaddc9526, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bc952a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91c9530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17c953a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7c953e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbc9541, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dc9544, 90 },
    { 0, 0 },
    { 0x5a4c9546, 55 },
    { 0x304c9546, 85 },
    { 0x548c9548, 90 },
    { 0x1a9c9549, 46 },
    { 0x1f7c954a, 61 },
    { 0xbc7c954b, 95 },
    { 0, 0 },
    { 0xa99c954d, 76 },
    { 0x536c954e, 90 },
    { 0xeeac954e, 76 },
    { 0x894c9550, 90 },
    { 0, 0 },
    { 0xa7bc9552, 46 },
    { 0x476c9553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffc9557, 23 },
    { 0x888c9558, 99 },
    { 0x11c9559, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22c9560, 90 },
    { 0, 0 },
    { 0xe1ac9562, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86c9566, 100 },
    { 0, 0 },
    { 0xec6c9568, 33 },
    { 0x6f0c9569, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9c956d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75c9570, 34 },
    { 0, 0 },
    { 0x9fdc9572, 90 },
    { 0x651c9572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35c9578, 98 },
    { 0xddbc9578, 46 },
    { 0xf69c9578, 61 },
    { 0x59bc957b, 87 },
    { 0, 0 },
    { 0x9e0c957d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898c9582, 85 },
    { 0x481c9582, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428c9588, 98 },
    { 0xaeac9589, 61 },
    { 0xe3cc9589, 52 },
    { 0x9bbc9589, 33 },
    { 0x3c9c958b, 87 },
    { 0x3fcc958c, 76 },
    { 0xa93c9588, 53 },
    { 0, 0 },
    { 0xe0ac9590, 91 },
    { 0, 0 },
    { 0xd1dc9592, 33 },
    { 0, 0 },
    { 0x57fc9594, 80 },
    { 0, 0 },
    { 0xfc4c9596, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6c959b, 98 },
    { 0, 0 },
    { 0x272c959d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cc95a2, 61 },
    { 0x54fc95a3, 93 },
    { 0, 0 },
    { 0xb82c95a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52c95a8, 95 },
    { 0x785c95a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec95ae, 33 },
    { 0, 0 },
    { 0x48bc95b0, 90 },
    { 0x509c95b1, 95 },
    { 0x2d8c95b2, 87 },
    { 0, 0 },
    { 0x1bac95b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0c95ba, 61 },
    { 0, 0 },
    { 0xa69c95bc, 61 },
    { 0, 0 },
    { 0x796c95be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c95c3, 43 },
    { 0x107c95c4, 69 },
    { 0x41dc95c5, 95 },
    { 0x6e8c95c6, 75 },
    { 0x639c95c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0c95cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7c95d6, 81 },
    { 0, 0 },
    { 0xb07c95d8, 61 },
    { 0, 0 },
    { 0xdcac95da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4c95de, 85 },
    { 0, 0 },
    { 0x86bc95e0, 76 },
    { 0, 0 },
    { 0xcbbc95e2, 34 },
    { 0x9ebc95e3, 87 },
    { 0xbe5c95e3, 75 },
    { 0x868c95e4, 66 },
    { 0x64cc95e6, 88 },
    { 0x568c95e7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1c95ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x774c95ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c95f2, 33 },
    { 0xe5dc95f2, 84 },
    { 0, 0 },
    { 0x90ec95f5, 81 },
    { 0x7c5c95f5, 98 },
    { 0, 0 },
    { 0x3cac95f8, 33 },
    { 0, 0 },
    { 0x6edc95fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9c95fe, 67 },
    { 0xcf5c95ff, 61 },
    { 0xc58c9600, 90 },
    { 0x878c9601, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3c9608, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32c960b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15c960e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacc9611, 91 },
    { 0xeebc9612, 90 },
    { 0x8c7c9613, 34 },
    { 0x7fdc9613, 84 },
    { 0x1a1c9615, 78 },
    { 0x10bc9612, 61 },
    { 0, 0 },
    { 0xa99c9618, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5c961b, 23 },
    { 0xbbec961c, 66 },
    { 0xf3ac961d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x314c9620, 90 },
    { 0x3aec9621, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407c962d, 33 },
    { 0x134c962d, 34 },
    { 0xc19c962f, 98 },
    { 0, 0 },
    { 0xf9dc9631, 33 },
    { 0xbe9c9632, 90 },
    { 0x51c9631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7c9637, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6c963a, 33 },
    { 0x38ac963a, 55 },
    { 0, 0 },
    { 0x387c963d, 33 },
    { 0xb1ec963e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96c9642, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabc9645, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bc9648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecac964e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2c9658, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bc9661, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6c9668, 87 },
    { 0x1ddc9668, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495c9670, 80 },
    { 0x330c9671, 43 },
    { 0xd9fc9670, 95 },
    { 0xa47c9673, 90 },
    { 0, 0 },
    { 0x968c9675, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cc9679, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x255c967c, 55 },
    { 0xd28c967d, 52 },
    { 0x18ac967d, 33 },
    { 0xa4fc967f, 88 },
    { 0xe20c967f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cc9683, 98 },
    { 0, 0 },
    { 0x1dc9685, 55 },
    { 0, 0 },
    { 0xb43c9687, 43 },
    { 0xf7c9687, 100 },
    { 0, 0 },
    { 0xfa6c968a, 67 },
    { 0, 0 },
    { 0xf18c968c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5c968f, 90 },
    { 0x122c9690, 100 },
    { 0x9ecc9690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dc9697, 66 },
    { 0x94cc9698, 17 },
    { 0x800c9699, 69 },
    { 0x885c969a, 34 },
    { 0x2d2c969b, 87 },
    { 0, 0 },
    { 0x733c969d, 43 },
    { 0x21ec969e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2c96a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2c96a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903c96ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cc96b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecc96bf, 33 },
    { 0x8fac96c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227c96c7, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8c96cc, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fc96cf, 33 },
    { 0, 0 },
    { 0xc5ec96d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801c96e6, 92 },
    { 0, 0 },
    { 0x312c96e8, 52 },
    { 0x408c96e9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170c96ee, 46 },
    { 0, 0 },
    { 0xa74c96f0, 40 },
    { 0xfd5c96f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x653c96f4, 98 },
    { 0x7edc96f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479c96fa, 55 },
    { 0x815c96fb, 33 },
    { 0x17bc96fc, 80 },
    { 0x4e8c96fd, 78 },
    { 0x425c96fd, 61 },
    { 0x68dc96fd, 33 },
    { 0xe26c96fd, 23 },
    { 0xb1ec9701, 84 },
    { 0x3bbc9702, 23 },
    { 0x42cc9700, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4c9707, 77 },
    { 0xa1c9707, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc970e, 98 },
    { 0xaaec970f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927c9719, 61 },
    { 0xedac9719, 90 },
    { 0, 0 },
    { 0xdbfc971c, 90 },
    { 0, 0 },
    { 0x66dc971e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836c9724, 61 },
    { 0xb3dc9724, 90 },
    { 0x20ac9726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44c972c, 33 },
    { 0, 0 },
    { 0xc19c972e, 33 },
    { 0, 0 },
    { 0x66ac9730, 81 },
    { 0x72ec9730, 61 },
    { 0x850c9732, 93 },
    { 0x335c9730, 38 },
    { 0, 0 },
    { 0x6cec9735, 23 },
    { 0, 0 },
    { 0x5a5c9737, 34 },
    { 0x1fcc9738, 90 },
    { 0, 0 },
    { 0x548c973a, 93 },
    { 0x205c973a, 55 },
    { 0x98fc973a, 33 },
    { 0x4fc973a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bc9741, 43 },
    { 0x7efc9741, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24c9745, 88 },
    { 0, 0 },
    { 0x598c9747, 87 },
    { 0x596c9747, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bc974d, 33 },
    { 0, 0 },
    { 0x39fc974f, 76 },
    { 0x58ac9750, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcac9753, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ec9758, 78 },
    { 0xb05c9758, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9c9761, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4c9768, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ac9773, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2c9777, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fc977b, 76 },
    { 0xcb1c977c, 95 },
    { 0, 0 },
    { 0xcc6c977e, 61 },
    { 0x541c977f, 55 },
    { 0, 0 },
    { 0xe0c9781, 85 },
    { 0x429c9782, 90 },
    { 0xa6cc9781, 75 },
    { 0xd79c9782, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4c9787, 68 },
    { 0xdcec9788, 23 },
    { 0, 0 },
    { 0x9a4c978a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c978d, 90 },
    { 0x90fc978e, 88 },
    { 0x21bc978f, 92 },
    { 0x48bc9790, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430c9797, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfc979a, 52 },
    { 0x3b6c979a, 43 },
    { 0x1e7c979c, 95 },
    { 0xdc1c979d, 75 },
    { 0x7b0c979e, 33 },
    { 0, 0 },
    { 0xb67c97a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640c97a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2c97aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fc97ad, 61 },
    { 0x2e0c97ad, 23 },
    { 0, 0 },
    { 0xf6bc97b0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ec97b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307c97b9, 55 },
    { 0x3e9c97ba, 90 },
    { 0xe54c97ba, 33 },
    { 0x669c97bc, 33 },
    { 0x9afc97bd, 23 },
    { 0x2e0c97bc, 33 },
    { 0x1eec97bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cc97c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209c97c8, 76 },
    { 0x10fc97c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62c97d1, 76 },
    { 0xa88c97d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62c97d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bc97db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50c97df, 66 },
    { 0xd30c97e0, 90 },
    { 0xa0c97e1, 99 },
    { 0x7a6c97e1, 90 },
    { 0x142c97df, 33 },
    { 0xc75c97e4, 97 },
    { 0x33dc97e5, 55 },
    { 0xc9ec97e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dc97eb, 23 },
    { 0x39ac97ec, 98 },
    { 0xd4cc97eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991c97f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x152c97f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4c97f9, 23 },
    { 0, 0 },
    { 0x3b5c97fb, 78 },
    { 0xfb0c97fc, 90 },
    { 0x140c97fd, 34 },
    { 0xb77c97fd, 61 },
    { 0xf1bc97fc, 23 },
    { 0x404c97fe, 34 },
    { 0, 0 },
    { 0x76ac9802, 33 },
    { 0x3b2c9803, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ac980a, 34 },
    { 0x72ec980a, 87 },
    { 0xd6c980c, 46 },
    { 0x51dc980d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ec981b, 100 },
    { 0xcc981c, 33 },
    { 0x870c981b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933c9821, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7c9824, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbc9827, 17 },
    { 0xc2bc9828, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x412c982b, 91 },
    { 0x63ec982c, 33 },
    { 0, 0 },
    { 0x243c982e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c9836, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x235c9839, 23 },
    { 0xbc1c983a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecc983d, 43 },
    { 0x764c983e, 34 },
    { 0xec5c983f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0c9844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x549c9847, 84 },
    { 0x651c9848, 33 },
    { 0, 0 },
    { 0xf59c984a, 95 },
    { 0xc38c984a, 95 },
    { 0x5ccc984c, 33 },
    { 0x6b3c984b, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bc9853, 95 },
    { 0x122c9854, 43 },
    { 0x476c9854, 61 },
    { 0x7e9c9856, 95 },
    { 0, 0 },
    { 0x5a3c9858, 95 },
    { 0x5cec9858, 85 },
    { 0, 0 },
    { 0xb2ec985b, 68 },
    { 0x6d9c985c, 77 },
    { 0x629c985c, 78 },
    { 0xd2ec985b, 23 },
    { 0xa3cc985f, 76 },
    { 0x43dc985b, 75 },
    { 0x257c9861, 87 },
    { 0x614c9862, 95 },
    { 0xc45c9860, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afc9873, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x879c9876, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccc987b, 84 },
    { 0x5b4c987c, 98 },
    { 0x17bc987d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5c9882, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ec9887, 46 },
    { 0x5d7c9887, 52 },
    { 0x3b3c9889, 33 },
    { 0xa3dc9888, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafac988d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aac9892, 33 },
    { 0x599c9892, 61 },
    { 0x7e2c9894, 100 },
    { 0x367c9892, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3c989a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcc989d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edc98a2, 90 },
    { 0x8d3c98a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62c98a7, 33 },
    { 0xdcac98a8, 66 },
    { 0x97fc98a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313c98b4, 88 },
    { 0, 0 },
    { 0x169c98b6, 90 },
    { 0, 0 },
    { 0xe42c98b8, 76 },
    { 0x67dc98b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69c98bd, 90 },
    { 0xb1ec98be, 61 },
    { 0x793c98bf, 43 },
    { 0x820c98c0, 33 },
    { 0, 0 },
    { 0x482c98c2, 90 },
    { 0xc48c98c2, 34 },
    { 0x72fc98c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39c98c7, 61 },
    { 0, 0 },
    { 0xe1cc98c9, 33 },
    { 0x1a3c98c9, 52 },
    { 0x1d9c98cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986c98d1, 90 },
    { 0x5dbc98d2, 81 },
    { 0x922c98d3, 91 },
    { 0, 0 },
    { 0x2fcc98d5, 61 },
    { 0x54ec98d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x598c98d9, 33 },
    { 0x5c8c98d9, 76 },
    { 0x7a2c98db, 34 },
    { 0xd08c98db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7c98e1, 17 },
    { 0xb6dc98e2, 95 },
    { 0xbc4c98e3, 90 },
    { 0xfc8c98e4, 77 },
    { 0x358c98e5, 61 },
    { 0xeaec98e3, 93 },
    { 0x600c98e3, 84 },
    { 0x972c98e8, 67 },
    { 0x4e3c98e4, 33 },
    { 0xb2dc98ea, 90 },
    { 0xa4fc98e9, 95 },
    { 0xbe1c98e9, 98 },
    { 0x9fdc98e2, 85 },
    { 0xd43c98ee, 98 },
    { 0x9c6c98ef, 33 },
    { 0xfa5c98ec, 33 },
    { 0, 0 },
    { 0x8e2c98f2, 61 },
    { 0, 0 },
    { 0x932c98f4, 98 },
    { 0xafac98f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cc98fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739c9900, 43 },
    { 0, 0 },
    { 0x690c9902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9906, 52 },
    { 0x1d4c9907, 76 },
    { 0xae3c9908, 23 },
    { 0xefac9907, 95 },
    { 0x501c990a, 98 },
    { 0x7b1c990b, 23 },
    { 0x292c990a, 61 },
    { 0x64bc990d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724c9911, 95 },
    { 0x483c9912, 65 },
    { 0, 0 },
    { 0xed4c9914, 99 },
    { 0x2d2c9915, 65 },
    { 0x60ac9916, 65 },
    { 0x7eec9917, 23 },
    { 0xfc4c9915, 33 },
    { 0xb14c9915, 85 },
    { 0xb46c9916, 34 },
    { 0xff7c991b, 61 },
    { 0xfb3c991c, 33 },
    { 0xb3fc9917, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ec9922, 61 },
    { 0x5a3c9923, 100 },
    { 0, 0 },
    { 0x9a0c9925, 55 },
    { 0xe5fc9926, 56 },
    { 0x9f2c9927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837c992b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cc992f, 90 },
    { 0x305c9930, 17 },
    { 0xb8bc9931, 61 },
    { 0xdbbc9932, 99 },
    { 0x486c9932, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dc993c, 98 },
    { 0, 0 },
    { 0x6ffc993e, 95 },
    { 0x8fc993e, 33 },
    { 0x2dec993e, 17 },
    { 0, 0 },
    { 0x424c9942, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ac9945, 99 },
    { 0, 0 },
    { 0xcd1c9947, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1c994d, 38 },
    { 0xf40c994e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574c9954, 23 },
    { 0x332c9954, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x657c9958, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ec995b, 23 },
    { 0x48fc995c, 90 },
    { 0x388c995b, 61 },
    { 0x584c995c, 52 },
    { 0x674c995f, 90 },
    { 0x28ac995f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0c9964, 33 },
    { 0, 0 },
    { 0xa14c9966, 61 },
    { 0x5dac9967, 43 },
    { 0xd9c9968, 95 },
    { 0x223c9967, 23 },
    { 0, 0 },
    { 0x781c996b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734c9971, 78 },
    { 0, 0 },
    { 0x497c9973, 52 },
    { 0xca2c9973, 87 },
    { 0x6aec9974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4c9979, 33 },
    { 0xb57c9979, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805c997e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc9981, 33 },
    { 0x63ac9981, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddc9985, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bc9988, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6c998d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7c9990, 33 },
    { 0xd37c9991, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963c9995, 55 },
    { 0x4ac9996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fc999a, 43 },
    { 0x3aec999b, 61 },
    { 0x9e4c999b, 33 },
    { 0xd66c999d, 90 },
    { 0xe7cc999e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439c99a4, 61 },
    { 0xc1ac99a5, 26 },
    { 0x22bc99a6, 100 },
    { 0xd2ac99a6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39c99ad, 84 },
    { 0x955c99ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16c99b1, 61 },
    { 0x8a2c99b2, 98 },
    { 0, 0 },
    { 0x52bc99b4, 76 },
    { 0, 0 },
    { 0x1f5c99b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3c99bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06c99be, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3c99c2, 90 },
    { 0x919c99c3, 40 },
    { 0, 0 },
    { 0x2f9c99c5, 78 },
    { 0x155c99c6, 77 },
    { 0xf20c99c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dc99ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632c99ce, 93 },
    { 0xf95c99cf, 98 },
    { 0xd80c99d0, 90 },
    { 0x5a6c99d0, 98 },
    { 0xc90c99d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x755c99d5, 93 },
    { 0, 0 },
    { 0xae3c99d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174c99de, 90 },
    { 0x823c99df, 90 },
    { 0, 0 },
    { 0x438c99e1, 17 },
    { 0x10ac99e2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440c99e7, 98 },
    { 0, 0 },
    { 0x386c99e9, 84 },
    { 0, 0 },
    { 0x639c99eb, 33 },
    { 0, 0 },
    { 0xdd9c99ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7c99f6, 66 },
    { 0x7b1c99f6, 95 },
    { 0x113c99f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1c99fd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1c9a00, 66 },
    { 0x8a0c9a00, 43 },
    { 0x86c9a02, 33 },
    { 0xd27c9a03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880c9a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cc9a0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7c9a14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bc9a20, 61 },
    { 0x4c1c9a21, 87 },
    { 0x6b1c9a22, 61 },
    { 0x2fac9a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eac9a26, 23 },
    { 0x693c9a26, 55 },
    { 0xbe4c9a26, 55 },
    { 0x87cc9a29, 90 },
    { 0, 0 },
    { 0xfb8c9a2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3c9a33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbec9a36, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15c9a3e, 76 },
    { 0x8d4c9a3f, 98 },
    { 0x53ac9a3f, 23 },
    { 0x162c9a41, 90 },
    { 0xd5cc9a42, 33 },
    { 0x5bcc9a43, 87 },
    { 0x43ec9a43, 34 },
    { 0x2d4c9a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eac9a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afc9a50, 91 },
    { 0, 0 },
    { 0xdb2c9a52, 98 },
    { 0, 0 },
    { 0x9d7c9a54, 55 },
    { 0x52fc9a55, 34 },
    { 0, 0 },
    { 0x67ac9a57, 61 },
    { 0x542c9a58, 67 },
    { 0x779c9a57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294c9a5e, 33 },
    { 0x289c9a5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fc9a6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c9a6f, 33 },
    { 0x97dc9a70, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267c9a77, 33 },
    { 0xbdfc9a78, 61 },
    { 0xdacc9a77, 88 },
    { 0, 0 },
    { 0x546c9a7b, 90 },
    { 0, 0 },
    { 0x5e9c9a7d, 43 },
    { 0x6f3c9a7e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bc9a82, 66 },
    { 0xb3dc9a82, 61 },
    { 0, 0 },
    { 0x611c9a85, 98 },
    { 0, 0 },
    { 0x18ec9a87, 67 },
    { 0, 0 },
    { 0x5a0c9a89, 23 },
    { 0, 0 },
    { 0x1d3c9a8b, 98 },
    { 0xb83c9a8c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02c9a93, 65 },
    { 0xa58c9a94, 33 },
    { 0x24fc9a95, 61 },
    { 0xf2dc9a96, 34 },
    { 0x9acc9a97, 81 },
    { 0xdbdc9a93, 33 },
    { 0x1afc9a99, 95 },
    { 0x5a7c9a9a, 33 },
    { 0x470c9a98, 23 },
    { 0x1b0c9a9c, 99 },
    { 0x520c9a9d, 95 },
    { 0x7c7c9a9c, 95 },
    { 0xbabc9a9f, 90 },
    { 0, 0 },
    { 0x7c5c9aa1, 90 },
    { 0x58c9aa1, 90 },
    { 0xff4c9aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cc9aa6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fc9aa9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7c9aae, 98 },
    { 0, 0 },
    { 0x2e5c9ab0, 59 },
    { 0, 0 },
    { 0x3b0c9ab2, 97 },
    { 0x929c9ab3, 95 },
    { 0xf87c9ab4, 96 },
    { 0xb0cc9ab3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefc9ab8, 34 },
    { 0xce5c9ab9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141c9ac0, 87 },
    { 0, 0 },
    { 0x5e6c9ac2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8c9ac8, 33 },
    { 0x425c9ac9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1c9acc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ec9ad1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ec9ad4, 34 },
    { 0, 0 },
    { 0x207c9ad6, 23 },
    { 0x8afc9ad6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5c9ae6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafc9aec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2c9af0, 33 },
    { 0xda2c9af1, 90 },
    { 0, 0 },
    { 0x389c9af3, 68 },
    { 0x3f2c9af4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dc9af9, 90 },
    { 0x80dc9afa, 98 },
    { 0x435c9afb, 34 },
    { 0, 0 },
    { 0x82fc9afd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadc9b01, 23 },
    { 0x1ddc9b02, 95 },
    { 0xc54c9b01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6c9b07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bc9b0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06c9b0d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661c9b11, 87 },
    { 0xc2dc9b12, 78 },
    { 0x271c9b11, 61 },
    { 0, 0 },
    { 0xd18c9b15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6c9b19, 75 },
    { 0, 0 },
    { 0xe10c9b1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcc9b1f, 33 },
    { 0, 0 },
    { 0x82dc9b21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c9b24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x644c9b27, 95 },
    { 0, 0 },
    { 0x717c9b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbc9b2e, 90 },
    { 0, 0 },
    { 0xdb7c9b30, 23 },
    { 0, 0 },
    { 0x440c9b32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18c9b39, 34 },
    { 0xe51c9b3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0c9b3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870c9b42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2c9b48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ec9b4b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8c9b4e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x430c9b51, 76 },
    { 0, 0 },
    { 0xc5bc9b53, 68 },
    { 0x8d4c9b53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10c9b59, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750c9b5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22c9b6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2c9b78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445c9b7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdc9b85, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345c9b8b, 33 },
    { 0, 0 },
    { 0x586c9b8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x218c9b90, 33 },
    { 0x3c0c9b90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6c9b94, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbc9b97, 76 },
    { 0, 0 },
    { 0x424c9b99, 93 },
    { 0, 0 },
    { 0xf5c9b9b, 55 },
    { 0x6b6c9b9c, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2c9ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbc9ba4, 33 },
    { 0x348c9ba5, 55 },
    { 0x2ddc9ba6, 87 },
    { 0x3f5c9ba5, 98 },
    { 0xcb0c9ba5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5c9bb1, 90 },
    { 0x81c9bb2, 90 },
    { 0, 0 },
    { 0xf54c9bb4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x437c9bb7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bac9bbb, 52 },
    { 0xa6cc9bbc, 33 },
    { 0x4dbc9bbc, 75 },
    { 0x39bc9bbe, 90 },
    { 0x88fc9bbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dc9bc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccc9bc8, 40 },
    { 0xd02c9bc9, 76 },
    { 0, 0 },
    { 0x135c9bcb, 76 },
    { 0, 0 },
    { 0x8c0c9bcd, 75 },
    { 0xde6c9bcd, 61 },
    { 0xe48c9bcd, 81 },
    { 0, 0 },
    { 0x2abc9bd1, 34 },
    { 0x7cfc9bd2, 87 },
    { 0, 0 },
    { 0xe11c9bd4, 94 },
    { 0xac1c9bd5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206c9bdb, 61 },
    { 0, 0 },
    { 0x69cc9bdd, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cc9be2, 64 },
    { 0xdf5c9be2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446c9be7, 90 },
    { 0x594c9be8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09c9bef, 23 },
    { 0x238c9bf0, 43 },
    { 0, 0 },
    { 0x144c9bf2, 95 },
    { 0x548c9bf3, 75 },
    { 0x5bdc9bf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ec9bfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15c9c01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25c9c05, 100 },
    { 0, 0 },
    { 0xf45c9c07, 87 },
    { 0x54dc9c08, 23 },
    { 0, 0 },
    { 0x999c9c0a, 90 },
    { 0xb25c9c0b, 33 },
    { 0, 0 },
    { 0xe91c9c0d, 91 },
    { 0x983c9c0e, 91 },
    { 0, 0 },
    { 0x194c9c10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15c9c17, 90 },
    { 0, 0 },
    { 0x681c9c19, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963c9c1e, 98 },
    { 0xdfbc9c1e, 55 },
    { 0x957c9c20, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dc9c23, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3c9c29, 98 },
    { 0x47ec9c2a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cc9c2e, 33 },
    { 0x810c9c2e, 23 },
    { 0, 0 },
    { 0x89ac9c31, 94 },
    { 0, 0 },
    { 0xccbc9c33, 90 },
    { 0x15dc9c34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940c9c38, 90 },
    { 0xe49c9c38, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6c9c3e, 33 },
    { 0x113c9c3f, 46 },
    { 0xfdbc9c40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc9c43, 43 },
    { 0xde4c9c44, 90 },
    { 0x9aec9c44, 23 },
    { 0, 0 },
    { 0x6eec9c47, 23 },
    { 0x413c9c47, 52 },
    { 0x571c9c49, 90 },
    { 0x254c9c4a, 87 },
    { 0xa58c9c47, 33 },
    { 0, 0 },
    { 0x34cc9c4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcc9c54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3c9c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6c9c5c, 47 },
    { 0, 0 },
    { 0xc21c9c5e, 61 },
    { 0, 0 },
    { 0x916c9c60, 85 },
    { 0x3fec9c61, 98 },
    { 0x61c9c62, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946c9c69, 68 },
    { 0, 0 },
    { 0xcb8c9c6b, 68 },
    { 0xd4ac9c6b, 85 },
    { 0xed6c9c6b, 34 },
    { 0x93c9c6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acc9c71, 90 },
    { 0x267c9c71, 61 },
    { 0xb59c9c71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78c9c76, 33 },
    { 0x8cdc9c77, 93 },
    { 0x17ec9c78, 76 },
    { 0x805c9c77, 61 },
    { 0, 0 },
    { 0xcedc9c7b, 100 },
    { 0, 0 },
    { 0x466c9c7d, 93 },
    { 0x71c9c7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bc9c81, 64 },
    { 0x718c9c82, 33 },
    { 0, 0 },
    { 0xbe8c9c84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc9c88, 23 },
    { 0xac2c9c88, 33 },
    { 0, 0 },
    { 0x980c9c8b, 95 },
    { 0xe89c9c8c, 100 },
    { 0, 0 },
    { 0xbc0c9c8e, 90 },
    { 0x6cfc9c8e, 87 },
    { 0, 0 },
    { 0x166c9c91, 34 },
    { 0x58ec9c91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91c9c98, 61 },
    { 0x3b6c9c98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8c9c9c, 55 },
    { 0x22ac9c9d, 23 },
    { 0xa8c9c9e, 55 },
    { 0x53fc9c9e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64c9ca2, 33 },
    { 0x9e2c9ca3, 23 },
    { 0x7aec9ca2, 76 },
    { 0x3eac9ca5, 95 },
    { 0xa26c9ca5, 90 },
    { 0x22fc9ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fc9caa, 100 },
    { 0, 0 },
    { 0x2b2c9cac, 69 },
    { 0xcbcc9cad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8c9cb0, 90 },
    { 0x42cc9cb1, 8 },
    { 0xf58c9cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511c9cb8, 33 },
    { 0xdb8c9cb8, 23 },
    { 0, 0 },
    { 0x2cbc9cbb, 93 },
    { 0xa7dc9cbc, 91 },
    { 0, 0 },
    { 0x4ccc9cbe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x634c9cc1, 83 },
    { 0xd72c9cc1, 61 },
    { 0, 0 },
    { 0xba2c9cc4, 100 },
    { 0x8dec9cc5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ec9cca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48c9ccd, 90 },
    { 0, 0 },
    { 0x916c9ccf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x118c9cd2, 33 },
    { 0x215c9cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836c9cd7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x266c9cda, 90 },
    { 0, 0 },
    { 0xe8bc9cdc, 90 },
    { 0, 0 },
    { 0xf63c9cde, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ec9ce2, 55 },
    { 0, 0 },
    { 0xb93c9ce4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffc9cea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0c9cee, 95 },
    { 0xd82c9cef, 76 },
    { 0xfd8c9cf0, 81 },
    { 0x47bc9cf1, 95 },
    { 0x35fc9cf2, 33 },
    { 0x490c9cf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefc9cf6, 46 },
    { 0, 0 },
    { 0xeec9cf8, 61 },
    { 0, 0 },
    { 0x4b0c9cfa, 65 },
    { 0x7edc9cfa, 98 },
    { 0x929c9cfc, 90 },
    { 0x771c9cfd, 78 },
    { 0xd3ec9cfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82c9d05, 46 },
    { 0, 0 },
    { 0xb53c9d07, 98 },
    { 0xf42c9d08, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dc9d0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1c9d10, 90 },
    { 0x844c9d11, 80 },
    { 0x766c9d12, 61 },
    { 0xbe7c9d13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ec9d17, 95 },
    { 0x84ac9d17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bc9d1d, 33 },
    { 0x963c9d1e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbc9d21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cc9d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafc9d2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30c9d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaac9d31, 66 },
    { 0x158c9d32, 33 },
    { 0xce8c9d33, 98 },
    { 0xb6c9d33, 90 },
    { 0, 0 },
    { 0x7f6c9d36, 61 },
    { 0x40ec9d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x73c9d3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7c9d46, 84 },
    { 0, 0 },
    { 0x5b6c9d48, 84 },
    { 0x3e5c9d48, 61 },
    { 0x88ac9d4a, 61 },
    { 0xc38c9d48, 23 },
    { 0x1aec9d4b, 90 },
    { 0x422c9d4b, 98 },
    { 0xea7c9d4e, 78 },
    { 0xecdc9d4b, 27 },
    { 0, 0 },
    { 0x2e8c9d51, 98 },
    { 0, 0 },
    { 0xc01c9d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615c9d5a, 33 },
    { 0xdc1c9d5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2c9d60, 78 },
    { 0x82bc9d60, 33 },
    { 0x206c9d61, 13 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ac9d69, 90 },
    { 0x9acc9d69, 100 },
    { 0xf4ac9d6b, 33 },
    { 0, 0 },
    { 0x15bc9d6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2c9d71, 95 },
    { 0x55ac9d72, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62c9d76, 85 },
    { 0, 0 },
    { 0x54cc9d78, 90 },
    { 0x90fc9d79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfc9d82, 43 },
    { 0xfc8c9d82, 95 },
    { 0xa10c9d82, 91 },
    { 0xb6ac9d83, 23 },
    { 0x5dbc9d86, 98 },
    { 0xacac9d82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x389c9d8a, 100 },
    { 0x58ec9d8b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8c9d8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2c9d99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272c9d9d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x807c9da0, 23 },
    { 0x774c9da1, 23 },
    { 0, 0 },
    { 0xd80c9da3, 33 },
    { 0x94ec9da3, 55 },
    { 0x3c5c9da4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0c9daa, 90 },
    { 0, 0 },
    { 0x171c9dac, 95 },
    { 0x723c9dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71c9db2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68c9db7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37c9dbb, 90 },
    { 0, 0 },
    { 0xa40c9dbd, 33 },
    { 0, 0 },
    { 0xc5dc9dbf, 25 },
    { 0, 0 },
    { 0xeefc9dc1, 46 },
    { 0x2a1c9dc2, 85 },
    { 0x9fdc9dc2, 61 },
    { 0x74ec9dc3, 90 },
    { 0xfdc9dc2, 52 },
    { 0x338c9dc6, 55 },
    { 0xa09c9dc7, 34 },
    { 0, 0 },
    { 0x105c9dc9, 61 },
    { 0xf4cc9dca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ec9dcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dc9dd1, 98 },
    { 0xd4bc9dd2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72c9dd8, 98 },
    { 0, 0 },
    { 0x702c9dda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9ddf, 76 },
    { 0x30dc9de0, 34 },
    { 0xdbbc9de0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0c9de7, 95 },
    { 0, 0 },
    { 0x6d2c9de9, 93 },
    { 0x749c9de9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadc9def, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754c9df5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ec9df8, 43 },
    { 0xc52c9df8, 33 },
    { 0xdec9dfa, 98 },
    { 0x825c9dfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9c9e00, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dc9e04, 23 },
    { 0, 0 },
    { 0xb53c9e06, 33 },
    { 0x291c9e07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebc9e0a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648c9e14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bc9e17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0c9e1a, 33 },
    { 0x388c9e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fc9e1e, 33 },
    { 0xcc4c9e1e, 67 },
    { 0, 0 },
    { 0x6e1c9e21, 34 },
    { 0xd5cc9e22, 91 },
    { 0, 0 },
    { 0x15c9e24, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x513c9e27, 65 },
    { 0xa76c9e28, 17 },
    { 0xe90c9e28, 75 },
    { 0xfe8c9e2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abc9e2e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbc9e31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dc9e34, 75 },
    { 0xf2bc9e34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29c9e3d, 23 },
    { 0xebc9e3d, 90 },
    { 0xba6c9e3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298c9e44, 75 },
    { 0, 0 },
    { 0x430c9e46, 95 },
    { 0x24ec9e46, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4c9e4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fc9e50, 93 },
    { 0, 0 },
    { 0xcb7c9e52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bc9e55, 76 },
    { 0x333c9e56, 78 },
    { 0, 0 },
    { 0xe85c9e58, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1c9e5c, 91 },
    { 0x2d5c9e5d, 33 },
    { 0, 0 },
    { 0x668c9e5f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fc9e65, 95 },
    { 0x97bc9e66, 90 },
    { 0x160c9e67, 90 },
    { 0x585c9e67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2c9e6d, 88 },
    { 0xcf6c9e6d, 33 },
    { 0x90ac9e6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ec9e72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5c9e76, 61 },
    { 0xf11c9e77, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0c9e7a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ec9e7e, 90 },
    { 0xa87c9e7f, 76 },
    { 0, 0 },
    { 0xf73c9e81, 43 },
    { 0, 0 },
    { 0x1e0c9e83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ac9e86, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dc9e89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0c9e8f, 98 },
    { 0xf4bc9e90, 78 },
    { 0xbefc9e91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7c9e94, 76 },
    { 0xf7cc9e95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156c9e9a, 95 },
    { 0, 0 },
    { 0x454c9e9c, 76 },
    { 0x84dc9e9d, 76 },
    { 0x1dc9e9e, 34 },
    { 0x447c9e9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815c9ea4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcc9ea7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344c9ead, 51 },
    { 0x3e3c9eae, 95 },
    { 0x256c9eaf, 98 },
    { 0xa21c9ead, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x918c9eb3, 46 },
    { 0xe02c9eb4, 85 },
    { 0x63fc9eb5, 83 },
    { 0x439c9eb6, 76 },
    { 0xdac9eb5, 90 },
    { 0x302c9eb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7c9ec4, 98 },
    { 0x7fcc9ec4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfc9ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdc9ecc, 90 },
    { 0x484c9ecd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdc9ed0, 33 },
    { 0x4e7c9ed1, 52 },
    { 0, 0 },
    { 0x42ec9ed3, 55 },
    { 0x25bc9ed3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ac9edb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02c9ede, 33 },
    { 0x3fc9ede, 38 },
    { 0x3b5c9ede, 76 },
    { 0x509c9ee1, 90 },
    { 0xa9fc9ee2, 34 },
    { 0x659c9ee1, 33 },
    { 0xa88c9ee1, 90 },
    { 0x341c9ee1, 33 },
    { 0xb3c9ee6, 98 },
    { 0x657c9ee4, 38 },
    { 0xcdec9ee8, 90 },
    { 0, 0 },
    { 0x535c9eea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6c9ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295c9efa, 33 },
    { 0x9f0c9efb, 95 },
    { 0, 0 },
    { 0xd13c9efd, 61 },
    { 0, 0 },
    { 0x465c9eff, 90 },
    { 0x26ac9eff, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ac9f03, 93 },
    { 0x541c9f04, 43 },
    { 0xaa0c9f05, 61 },
    { 0x6b0c9f05, 75 },
    { 0x3d2c9f07, 67 },
    { 0, 0 },
    { 0xa81c9f09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654c9f11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7c9f17, 61 },
    { 0x9e2c9f17, 53 },
    { 0, 0 },
    { 0xff4c9f1a, 95 },
    { 0xdecc9f1a, 33 },
    { 0xaa6c9f1c, 34 },
    { 0x98ac9f1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8c9f20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbc9f24, 61 },
    { 0, 0 },
    { 0x5c8c9f26, 80 },
    { 0x228c9f27, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814c9f3a, 36 },
    { 0, 0 },
    { 0x8bbc9f3c, 55 },
    { 0xc72c9f3c, 95 },
    { 0x355c9f3d, 33 },
    { 0xab2c9f3e, 34 },
    { 0x37fc9f40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bc9f47, 95 },
    { 0x163c9f48, 61 },
    { 0, 0 },
    { 0x96cc9f4a, 95 },
    { 0xc63c9f4a, 98 },
    { 0, 0 },
    { 0xd4ec9f4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42c9f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebc9f56, 84 },
    { 0xe8ac9f57, 90 },
    { 0x5bbc9f58, 87 },
    { 0xb60c9f58, 55 },
    { 0xf47c9f59, 77 },
    { 0x453c9f59, 33 },
    { 0xfbdc9f5a, 43 },
    { 0xcefc9f5b, 33 },
    { 0, 0 },
    { 0x22cc9f5f, 90 },
    { 0xc93c9f60, 66 },
    { 0x567c9f60, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efc9f64, 67 },
    { 0xdc4c9f64, 33 },
    { 0, 0 },
    { 0x4e8c9f67, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bc9f6d, 33 },
    { 0, 0 },
    { 0x44dc9f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x229c9f72, 78 },
    { 0xb8fc9f73, 87 },
    { 0xee4c9f73, 23 },
    { 0xc75c9f75, 85 },
    { 0xde0c9f76, 100 },
    { 0x265c9f76, 23 },
    { 0, 0 },
    { 0xd89c9f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400c9f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecc9f85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ec9f8e, 77 },
    { 0x8f1c9f8e, 99 },
    { 0xc88c9f90, 23 },
    { 0x2cfc9f90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55c9f95, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182c9f9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23c9f9d, 90 },
    { 0xf1fc9f9e, 61 },
    { 0, 0 },
    { 0x8cec9fa0, 90 },
    { 0, 0 },
    { 0x719c9fa2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2c9fae, 40 },
    { 0x40ec9fae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ec9fb2, 95 },
    { 0xc41c9fb3, 90 },
    { 0xd1fc9fb4, 98 },
    { 0x626c9fb5, 55 },
    { 0, 0 },
    { 0x7e1c9fb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82c9fba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7c9fc1, 34 },
    { 0xefcc9fc2, 33 },
    { 0x88ec9fc2, 91 },
    { 0xfc7c9fc4, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72c9fcb, 91 },
    { 0xeb3c9fcc, 88 },
    { 0x31c9fcc, 43 },
    { 0x55cc9fce, 100 },
    { 0xd46c9fcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x483c9fd2, 95 },
    { 0, 0 },
    { 0xad0c9fd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ac9fd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9c9fe0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255c9fe9, 100 },
    { 0x870c9fea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x622c9fed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2c9ff0, 43 },
    { 0, 0 },
    { 0xd07c9ff2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330c9ff9, 36 },
    { 0x63ac9ff9, 76 },
    { 0xa7ac9ff9, 34 },
    { 0, 0 },
    { 0x5b3c9ffd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398ca001, 95 },
    { 0xdc3ca002, 75 },
    { 0xfa3ca003, 61 },
    { 0x9d5ca003, 93 },
    { 0, 0 },
    { 0xc9ca006, 93 },
    { 0, 0 },
    { 0x9ecca008, 46 },
    { 0, 0 },
    { 0x438ca00a, 23 },
    { 0xd27ca00b, 52 },
    { 0xafca00c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7ca00f, 78 },
    { 0, 0 },
    { 0xb69ca011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eca016, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15ca019, 34 },
    { 0, 0 },
    { 0xf62ca01b, 23 },
    { 0xd2fca01c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02ca01f, 95 },
    { 0x55cca020, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58ca028, 33 },
    { 0, 0 },
    { 0x164ca02a, 95 },
    { 0x807ca02b, 99 },
    { 0x2fcca02b, 77 },
    { 0x98cca02b, 33 },
    { 0x60cca02b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490ca037, 90 },
    { 0x942ca037, 61 },
    { 0x1b0ca039, 100 },
    { 0x2d2ca039, 90 },
    { 0, 0 },
    { 0xa93ca03c, 33 },
    { 0x8deca03d, 85 },
    { 0x7fcca03e, 90 },
    { 0x307ca03d, 98 },
    { 0, 0 },
    { 0x8f1ca041, 66 },
    { 0x116ca042, 98 },
    { 0x704ca042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ca049, 43 },
    { 0xfe5ca04a, 23 },
    { 0, 0 },
    { 0x364ca04c, 88 },
    { 0x57ca04d, 61 },
    { 0xa0cca04e, 43 },
    { 0, 0 },
    { 0xecbca050, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ca054, 85 },
    { 0x38dca055, 97 },
    { 0x2b2ca055, 23 },
    { 0xad3ca057, 61 },
    { 0, 0 },
    { 0x77eca059, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6ca066, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dca06a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ca06f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bca074, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ca07c, 52 },
    { 0, 0 },
    { 0x850ca07e, 46 },
    { 0x1c3ca07f, 23 },
    { 0, 0 },
    { 0x4d8ca081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebca084, 98 },
    { 0x88dca085, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dca089, 40 },
    { 0x532ca08a, 33 },
    { 0xc6fca08b, 43 },
    { 0xc00ca08c, 61 },
    { 0xb6aca08c, 34 },
    { 0xe53ca08a, 46 },
    { 0xb34ca08f, 93 },
    { 0xaf4ca08e, 66 },
    { 0, 0 },
    { 0x3efca092, 87 },
    { 0xcf4ca093, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eca097, 61 },
    { 0, 0 },
    { 0xf2bca099, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172ca09d, 90 },
    { 0x10fca09e, 66 },
    { 0, 0 },
    { 0xc1eca0a0, 98 },
    { 0xe2ca0a0, 84 },
    { 0x259ca0a2, 61 },
    { 0x6ebca0a3, 43 },
    { 0x39bca0a4, 33 },
    { 0x4c7ca0a5, 77 },
    { 0xd5eca0a6, 95 },
    { 0xc79ca0a2, 33 },
    { 0x186ca0a3, 43 },
    { 0xb85ca0a9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57ca0ad, 77 },
    { 0, 0 },
    { 0x6eca0af, 76 },
    { 0x28fca0b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dca0b7, 78 },
    { 0x4bca0b7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4ca0bd, 84 },
    { 0xf68ca0bd, 98 },
    { 0, 0 },
    { 0x635ca0c0, 76 },
    { 0x52eca0c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaca0cc, 61 },
    { 0xdbcca0cc, 46 },
    { 0x4bbca0ce, 77 },
    { 0, 0 },
    { 0x7baca0d0, 95 },
    { 0x1f2ca0d0, 55 },
    { 0, 0 },
    { 0x45ca0d3, 90 },
    { 0x995ca0d4, 43 },
    { 0, 0 },
    { 0x8b5ca0d6, 23 },
    { 0, 0 },
    { 0xe2bca0d8, 95 },
    { 0, 0 },
    { 0xbf4ca0da, 95 },
    { 0xc3dca0db, 95 },
    { 0, 0 },
    { 0xac8ca0dd, 95 },
    { 0x220ca0de, 33 },
    { 0, 0 },
    { 0x155ca0e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcca0e3, 33 },
    { 0x230ca0e4, 84 },
    { 0, 0 },
    { 0xc94ca0e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877ca0ea, 65 },
    { 0xcafca0ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x987ca0ee, 91 },
    { 0, 0 },
    { 0xb40ca0f0, 34 },
    { 0xbfdca0f1, 61 },
    { 0, 0 },
    { 0x53eca0f3, 98 },
    { 0, 0 },
    { 0x539ca0f5, 90 },
    { 0xe6dca0f5, 99 },
    { 0xf1aca0f7, 55 },
    { 0xcdcca0f5, 98 },
    { 0, 0 },
    { 0xfebca0fa, 93 },
    { 0xa74ca0fb, 95 },
    { 0x6f2ca0fa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cca0ff, 100 },
    { 0, 0 },
    { 0xa99ca101, 88 },
    { 0, 0 },
    { 0x11aca103, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81ca106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ca109, 23 },
    { 0xf0fca10a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649ca110, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ca115, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634ca11a, 85 },
    { 0, 0 },
    { 0x3f5ca11c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228ca120, 23 },
    { 0, 0 },
    { 0xdd3ca122, 61 },
    { 0, 0 },
    { 0x44bca124, 95 },
    { 0x9b5ca124, 76 },
    { 0x3bfca126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ca129, 95 },
    { 0, 0 },
    { 0xbd1ca12b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ca12e, 88 },
    { 0, 0 },
    { 0x6e3ca130, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ca135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafca139, 33 },
    { 0xe1ca13a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bca13e, 23 },
    { 0x3eeca13f, 33 },
    { 0x24eca13e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0ca144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaca147, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5ca14a, 98 },
    { 0, 0 },
    { 0x5d0ca14c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ca14f, 93 },
    { 0x1fbca150, 61 },
    { 0x685ca150, 78 },
    { 0x5e8ca150, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1ca157, 61 },
    { 0, 0 },
    { 0x9f0ca159, 55 },
    { 0x963ca159, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ca15e, 95 },
    { 0xa21ca15f, 98 },
    { 0x549ca160, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aca164, 100 },
    { 0, 0 },
    { 0xd93ca166, 23 },
    { 0x9bdca167, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7ca16a, 61 },
    { 0x251ca16b, 61 },
    { 0x1c7ca16c, 90 },
    { 0, 0 },
    { 0x65eca16e, 33 },
    { 0xe34ca16f, 99 },
    { 0x4d9ca170, 95 },
    { 0x99bca171, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fca17b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfca17e, 95 },
    { 0, 0 },
    { 0xedcca180, 90 },
    { 0, 0 },
    { 0xf49ca182, 98 },
    { 0x7bdca182, 61 },
    { 0, 0 },
    { 0xa35ca185, 66 },
    { 0, 0 },
    { 0xcbeca187, 33 },
    { 0x3aca188, 78 },
    { 0xc5bca189, 95 },
    { 0x4d8ca18a, 90 },
    { 0xa56ca18a, 34 },
    { 0xcedca18c, 65 },
    { 0, 0 },
    { 0xba6ca18e, 23 },
    { 0xdf7ca18e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536ca194, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ca19b, 24 },
    { 0, 0 },
    { 0x957ca19d, 23 },
    { 0xb7dca19d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ca1a2, 91 },
    { 0x2a9ca1a3, 55 },
    { 0xddbca1a4, 33 },
    { 0x4eca1a4, 43 },
    { 0xf51ca1a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4ca1b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfca1b3, 76 },
    { 0, 0 },
    { 0x356ca1b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ca1b9, 88 },
    { 0xaf4ca1ba, 66 },
    { 0x28bca1bb, 99 },
    { 0x52eca1bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2ca1c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3ca1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653ca1c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3ca1cb, 90 },
    { 0x68ca1cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8ca1cf, 23 },
    { 0x723ca1d0, 61 },
    { 0xd9cca1d1, 99 },
    { 0, 0 },
    { 0x4f9ca1d3, 43 },
    { 0x832ca1d4, 98 },
    { 0x240ca1d4, 76 },
    { 0x91bca1d6, 46 },
    { 0x3b9ca1d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5ca1da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6ca1de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ca1e1, 95 },
    { 0, 0 },
    { 0xeb2ca1e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cca1ee, 17 },
    { 0, 0 },
    { 0x46ca1f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8ca1f3, 33 },
    { 0x3b9ca1f3, 9 },
    { 0, 0 },
    { 0xe60ca1f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dca1ff, 90 },
    { 0x5f0ca200, 87 },
    { 0xa5aca1ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aca204, 77 },
    { 0, 0 },
    { 0xcedca206, 80 },
    { 0xe6fca207, 33 },
    { 0x620ca208, 95 },
    { 0xf5ca209, 98 },
    { 0xdbeca20a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fca20d, 61 },
    { 0xfa0ca20e, 33 },
    { 0, 0 },
    { 0xa74ca210, 84 },
    { 0xf70ca211, 40 },
    { 0xc12ca212, 95 },
    { 0x1daca213, 98 },
    { 0, 0 },
    { 0x884ca215, 98 },
    { 0x224ca216, 99 },
    { 0x393ca215, 98 },
    { 0x9a7ca218, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dca221, 33 },
    { 0, 0 },
    { 0x190ca223, 52 },
    { 0x7f3ca223, 87 },
    { 0, 0 },
    { 0x69dca226, 90 },
    { 0x861ca227, 90 },
    { 0x9e0ca227, 90 },
    { 0xe35ca227, 95 },
    { 0x752ca22a, 34 },
    { 0, 0 },
    { 0x127ca22c, 17 },
    { 0x65fca22d, 93 },
    { 0xcd3ca22d, 44 },
    { 0xbcfca22d, 76 },
    { 0xd9cca230, 33 },
    { 0x18fca231, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05ca234, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acca239, 90 },
    { 0x998ca23a, 61 },
    { 0, 0 },
    { 0x536ca23c, 61 },
    { 0, 0 },
    { 0xc9dca23e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x737ca241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9ca246, 61 },
    { 0x74eca246, 33 },
    { 0x977ca248, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bca24c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dca253, 96 },
    { 0xf7fca253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca25b, 90 },
    { 0xc4fca25b, 90 },
    { 0x236ca25b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642ca264, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ca26d, 84 },
    { 0x54ca26e, 90 },
    { 0, 0 },
    { 0xabbca270, 90 },
    { 0, 0 },
    { 0x68dca272, 100 },
    { 0, 0 },
    { 0x2b7ca274, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93ca27b, 34 },
    { 0xd77ca27b, 98 },
    { 0, 0 },
    { 0xbf1ca27e, 36 },
    { 0x11fca27e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcca282, 33 },
    { 0xf35ca282, 95 },
    { 0x527ca284, 61 },
    { 0xf5ca284, 33 },
    { 0xd7aca286, 99 },
    { 0x66aca287, 99 },
    { 0x8a2ca286, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9ca28b, 23 },
    { 0xe71ca28c, 90 },
    { 0xd09ca28c, 34 },
    { 0x9acca28e, 67 },
    { 0x30bca28e, 76 },
    { 0xddca28b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cca294, 23 },
    { 0x76eca295, 23 },
    { 0, 0 },
    { 0xc54ca297, 87 },
    { 0xb1cca298, 85 },
    { 0xbfbca299, 33 },
    { 0xc2aca29a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ca2a1, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312ca2a6, 76 },
    { 0, 0 },
    { 0x897ca2a8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3eca2ab, 95 },
    { 0, 0 },
    { 0xb2bca2ad, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ca2b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eca2b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5baca2b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cca2bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfca2c2, 91 },
    { 0x8b2ca2c3, 55 },
    { 0, 0 },
    { 0x8e6ca2c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eca2c9, 88 },
    { 0xe50ca2c9, 19 },
    { 0, 0 },
    { 0xb8dca2cc, 87 },
    { 0xf1fca2cd, 61 },
    { 0, 0 },
    { 0x338ca2cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08ca2d2, 34 },
    { 0x6a6ca2d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757ca2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486ca2dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cca2e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572ca2e6, 61 },
    { 0x76bca2e7, 34 },
    { 0xec7ca2e6, 98 },
    { 0xd3fca2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1ca2ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8eca2f2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdca2f6, 95 },
    { 0x40fca2f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ca2fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ca303, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aca307, 76 },
    { 0, 0 },
    { 0xb65ca309, 17 },
    { 0, 0 },
    { 0x75dca30b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05ca30f, 55 },
    { 0, 0 },
    { 0xb16ca311, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddca314, 91 },
    { 0x490ca314, 84 },
    { 0x381ca314, 61 },
    { 0xe47ca314, 23 },
    { 0x2b4ca318, 23 },
    { 0x18cca317, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5ca31e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2ca321, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ca324, 90 },
    { 0, 0 },
    { 0xcd0ca326, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedca32a, 91 },
    { 0, 0 },
    { 0x23bca32c, 33 },
    { 0, 0 },
    { 0xba8ca32e, 98 },
    { 0xd08ca32e, 67 },
    { 0x7dcca330, 33 },
    { 0x449ca330, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274ca336, 95 },
    { 0x1d4ca337, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x26aca33a, 61 },
    { 0x428ca33a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cca340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x335ca343, 93 },
    { 0xd7cca344, 33 },
    { 0xd4eca345, 87 },
    { 0xf4cca345, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x101ca349, 67 },
    { 0xbc2ca34a, 23 },
    { 0x224ca349, 75 },
    { 0xfa2ca34c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7ca34f, 90 },
    { 0xbc5ca350, 61 },
    { 0x2f0ca351, 99 },
    { 0xcc0ca352, 95 },
    { 0xe1dca353, 91 },
    { 0, 0 },
    { 0xbbeca355, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aca35e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x236ca361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eca368, 23 },
    { 0xf0eca369, 17 },
    { 0x1f2ca36a, 65 },
    { 0xcdcca369, 56 },
    { 0x1a3ca36c, 33 },
    { 0xcb9ca36b, 93 },
    { 0, 0 },
    { 0x2d9ca36f, 33 },
    { 0x798ca36f, 46 },
    { 0, 0 },
    { 0x7a0ca372, 84 },
    { 0xa96ca373, 66 },
    { 0, 0 },
    { 0x8d2ca375, 61 },
    { 0, 0 },
    { 0x436ca377, 99 },
    { 0xd94ca378, 99 },
    { 0x70ca379, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2ca37e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677ca387, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcca38a, 95 },
    { 0x66dca38b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cca38f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343ca393, 98 },
    { 0, 0 },
    { 0x201ca395, 55 },
    { 0, 0 },
    { 0x74cca397, 98 },
    { 0xbc0ca397, 95 },
    { 0, 0 },
    { 0x51aca39a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aca3a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaca3a3, 33 },
    { 0xb8cca3a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646ca3b0, 85 },
    { 0, 0 },
    { 0xd00ca3b2, 78 },
    { 0, 0 },
    { 0x418ca3b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcca3bb, 55 },
    { 0x15bca3bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879ca3c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79ca3cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77ca3d2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ca3d7, 23 },
    { 0xf18ca3d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bca3de, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cca3e2, 76 },
    { 0xed0ca3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eca3ea, 17 },
    { 0xf20ca3ea, 98 },
    { 0, 0 },
    { 0x106ca3ed, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283ca3f3, 61 },
    { 0, 0 },
    { 0xc70ca3f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8beca3fd, 93 },
    { 0xef1ca3fe, 61 },
    { 0xb9fca3fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fca408, 95 },
    { 0, 0 },
    { 0x5e2ca40a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65ca40f, 95 },
    { 0, 0 },
    { 0x803ca411, 33 },
    { 0, 0 },
    { 0xd55ca413, 33 },
    { 0, 0 },
    { 0x49bca415, 23 },
    { 0x3d9ca416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8ca419, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdca428, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0ca42c, 66 },
    { 0x9f1ca42d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31ca431, 90 },
    { 0x29bca432, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecca435, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7ca438, 61 },
    { 0, 0 },
    { 0xa79ca43a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dca43e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83ca443, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ca446, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11ca44f, 91 },
    { 0, 0 },
    { 0x298ca451, 100 },
    { 0, 0 },
    { 0x51eca453, 95 },
    { 0x12fca454, 61 },
    { 0x91dca455, 46 },
    { 0x450ca455, 98 },
    { 0x1c1ca454, 100 },
    { 0xd8ca458, 78 },
    { 0x3feca456, 23 },
    { 0x34ca459, 33 },
    { 0x7e3ca454, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ca460, 100 },
    { 0x8b7ca461, 33 },
    { 0xc77ca462, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aca467, 33 },
    { 0xa63ca468, 33 },
    { 0, 0 },
    { 0xc5fca46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148ca471, 33 },
    { 0, 0 },
    { 0xf34ca473, 92 },
    { 0x5e7ca473, 51 },
    { 0x9bdca475, 46 },
    { 0xeadca476, 46 },
    { 0xc6bca475, 61 },
    { 0xf95ca474, 55 },
    { 0xdc0ca479, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8ca47d, 23 },
    { 0, 0 },
    { 0xbceca47f, 53 },
    { 0xc5cca47f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ca485, 76 },
    { 0x64bca486, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ca48a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbca48e, 33 },
    { 0xe00ca48f, 55 },
    { 0xea4ca490, 61 },
    { 0x40bca490, 76 },
    { 0x811ca48e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9ca499, 85 },
    { 0xc13ca499, 90 },
    { 0x320ca49b, 61 },
    { 0x7b6ca49c, 76 },
    { 0x974ca49d, 33 },
    { 0xcb7ca49d, 92 },
    { 0, 0 },
    { 0x1b0ca4a0, 33 },
    { 0x5b5ca4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6ca4a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794ca4ab, 75 },
    { 0, 0 },
    { 0xe9cca4ad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28eca4b6, 34 },
    { 0x97dca4b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ca4bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ca4c3, 61 },
    { 0xd88ca4c4, 61 },
    { 0, 0 },
    { 0x8b2ca4c6, 90 },
    { 0xfeca4c7, 90 },
    { 0xc5cca4c7, 23 },
    { 0x831ca4c7, 34 },
    { 0x881ca4ca, 66 },
    { 0xf8bca4c6, 99 },
    { 0xa2cca4cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5ca4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56ca4d4, 95 },
    { 0, 0 },
    { 0x25ca4d6, 66 },
    { 0xf1ca4d7, 100 },
    { 0x8dcca4d7, 33 },
    { 0xf61ca4d9, 93 },
    { 0x33cca4d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497ca4e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeca4e4, 43 },
    { 0x889ca4e4, 33 },
    { 0x8cca4e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ca4e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623ca4f1, 46 },
    { 0x368ca4f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ca4f6, 78 },
    { 0, 0 },
    { 0x92ca4f8, 33 },
    { 0x9c2ca4f9, 95 },
    { 0, 0 },
    { 0xdbeca4fb, 34 },
    { 0x2d7ca4fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edca500, 61 },
    { 0x98aca501, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcca504, 95 },
    { 0xdb4ca504, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cca50a, 95 },
    { 0xaf4ca50a, 90 },
    { 0xc2bca50c, 17 },
    { 0, 0 },
    { 0xf81ca50e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9ca511, 61 },
    { 0x48dca511, 90 },
    { 0, 0 },
    { 0xfafca514, 91 },
    { 0, 0 },
    { 0x98dca516, 36 },
    { 0, 0 },
    { 0xcabca518, 90 },
    { 0xaf0ca518, 66 },
    { 0x3d3ca51a, 76 },
    { 0xd52ca51b, 98 },
    { 0, 0 },
    { 0xe82ca51d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fca521, 90 },
    { 0, 0 },
    { 0x126ca523, 66 },
    { 0x88eca524, 33 },
    { 0x745ca525, 61 },
    { 0xc4ca525, 76 },
    { 0xb79ca523, 23 },
    { 0x3ddca525, 76 },
    { 0, 0 },
    { 0xb07ca52a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42ca52d, 61 },
    { 0, 0 },
    { 0xcb7ca52f, 76 },
    { 0xf3cca530, 53 },
    { 0x70cca531, 46 },
    { 0xab4ca532, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203ca536, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dca53e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398ca54c, 98 },
    { 0x836ca54d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ca551, 76 },
    { 0xd8eca552, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4deca555, 90 },
    { 0, 0 },
    { 0x6fdca557, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x487ca55a, 95 },
    { 0x5c7ca55a, 46 },
    { 0x1eaca55c, 98 },
    { 0xc64ca55d, 90 },
    { 0xbb9ca55e, 87 },
    { 0x60cca55f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27ca569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cca56e, 33 },
    { 0x332ca56e, 55 },
    { 0x960ca56f, 78 },
    { 0xd19ca571, 61 },
    { 0xf13ca570, 61 },
    { 0x25ca572, 61 },
    { 0, 0 },
    { 0x85aca575, 33 },
    { 0x101ca576, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251ca57b, 34 },
    { 0x2f6ca57c, 33 },
    { 0, 0 },
    { 0xe18ca57e, 75 },
    { 0x3adca57e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266ca588, 31 },
    { 0x320ca589, 33 },
    { 0xdf4ca589, 33 },
    { 0xe3bca58b, 65 },
    { 0xee4ca58c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1ca592, 33 },
    { 0, 0 },
    { 0x8b7ca594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfca597, 33 },
    { 0, 0 },
    { 0x57ca599, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bca59c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75eca59f, 61 },
    { 0x95fca59f, 55 },
    { 0x7e9ca59f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ca5a8, 61 },
    { 0x97ca5a8, 90 },
    { 0, 0 },
    { 0x501ca5ab, 95 },
    { 0x198ca5ac, 61 },
    { 0xc1eca5ad, 76 },
    { 0x64eca5ac, 98 },
    { 0xa72ca5ac, 33 },
    { 0x29ca5b0, 90 },
    { 0x336ca5b1, 43 },
    { 0, 0 },
    { 0xa5bca5b3, 33 },
    { 0x7a4ca5b4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2ca5b8, 34 },
    { 0x119ca5b9, 78 },
    { 0, 0 },
    { 0xf48ca5bb, 68 },
    { 0x39ca5bb, 100 },
    { 0xcb0ca5bd, 76 },
    { 0x153ca5bd, 61 },
    { 0, 0 },
    { 0xf5aca5c0, 56 },
    { 0xc62ca5c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4ca5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2ca5c8, 99 },
    { 0x31aca5c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadca5d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5ca5d7, 23 },
    { 0x6c3ca5d8, 95 },
    { 0x65cca5d9, 61 },
    { 0x9cca5d9, 76 },
    { 0, 0 },
    { 0x1efca5dc, 33 },
    { 0x2e9ca5dc, 52 },
    { 0xa1dca5de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514ca5e2, 90 },
    { 0, 0 },
    { 0xdfeca5e4, 93 },
    { 0xb29ca5e5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbca5e9, 68 },
    { 0x732ca5e9, 33 },
    { 0x470ca5e9, 76 },
    { 0x23eca5ec, 61 },
    { 0x63eca5ed, 95 },
    { 0x69aca5ee, 90 },
    { 0xd51ca5ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x572ca5f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6ca5f5, 46 },
    { 0x9ccca5f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eca603, 76 },
    { 0xf4bca603, 34 },
    { 0, 0 },
    { 0x94eca606, 99 },
    { 0x634ca607, 90 },
    { 0xac1ca606, 33 },
    { 0x4bca609, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ceca60d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x48aca610, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2ca618, 23 },
    { 0x6c7ca619, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46ca61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccca620, 90 },
    { 0x8ddca621, 70 },
    { 0x569ca622, 23 },
    { 0xfa4ca622, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53ca626, 55 },
    { 0xcb1ca627, 67 },
    { 0xa84ca628, 90 },
    { 0x4e5ca627, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1ca62c, 94 },
    { 0xb16ca62c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x728ca630, 90 },
    { 0x552ca630, 46 },
    { 0, 0 },
    { 0x4c6ca633, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x574ca636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ca639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eca63c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02ca63f, 98 },
    { 0xcadca63f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6ca643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9ca646, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ca64b, 99 },
    { 0xe3dca64c, 33 },
    { 0x48bca64d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3ca653, 76 },
    { 0, 0 },
    { 0xbedca655, 66 },
    { 0xec5ca655, 55 },
    { 0x188ca657, 23 },
    { 0x27dca656, 55 },
    { 0xbf9ca659, 95 },
    { 0, 0 },
    { 0xd2dca65b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6ca65f, 90 },
    { 0x629ca65f, 93 },
    { 0xb88ca65f, 83 },
    { 0xf96ca662, 95 },
    { 0xc51ca663, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7ca667, 33 },
    { 0x2b9ca668, 23 },
    { 0x1e4ca669, 100 },
    { 0x969ca667, 55 },
    { 0xe92ca66b, 100 },
    { 0x9c0ca66c, 40 },
    { 0x337ca66b, 98 },
    { 0x5e5ca66d, 85 },
    { 0xf49ca66a, 23 },
    { 0, 0 },
    { 0x3f6ca671, 61 },
    { 0, 0 },
    { 0xb3aca673, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eca677, 61 },
    { 0xcd5ca677, 66 },
    { 0, 0 },
    { 0xedfca67a, 99 },
    { 0x8afca67b, 33 },
    { 0x7d1ca67a, 99 },
    { 0x73aca67d, 33 },
    { 0xe35ca67d, 98 },
    { 0xd7ca67f, 33 },
    { 0, 0 },
    { 0xc48ca681, 98 },
    { 0, 0 },
    { 0x85cca683, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8ca688, 95 },
    { 0x68dca689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7ca68c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x288ca68f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x20aca692, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3ca697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fca69c, 46 },
    { 0xd80ca69d, 67 },
    { 0x82ca69e, 65 },
    { 0xacbca69f, 99 },
    { 0x3e2ca6a0, 23 },
    { 0x40aca6a1, 34 },
    { 0xc00ca6a2, 23 },
    { 0x9c4ca69d, 23 },
    { 0xa63ca6a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4ca6a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cca6ab, 90 },
    { 0xf8eca6ab, 43 },
    { 0, 0 },
    { 0x5f9ca6ae, 66 },
    { 0, 0 },
    { 0x574ca6b0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127ca6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3ca6bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2ca6c1, 77 },
    { 0x5deca6c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95ca6c7, 90 },
    { 0x19cca6c8, 36 },
    { 0x4b7ca6c9, 61 },
    { 0x9a9ca6ca, 43 },
    { 0, 0 },
    { 0x29cca6cc, 76 },
    { 0xd64ca6cc, 34 },
    { 0x4e9ca6cc, 90 },
    { 0x3cbca6cd, 61 },
    { 0x628ca6cf, 90 },
    { 0x64aca6d0, 61 },
    { 0xa99ca6d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ca6d6, 90 },
    { 0x85cca6d7, 23 },
    { 0, 0 },
    { 0x3d2ca6d9, 34 },
    { 0x1cbca6da, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9ca6dd, 76 },
    { 0, 0 },
    { 0x67bca6df, 98 },
    { 0x83ca6e0, 61 },
    { 0x8acca6e1, 76 },
    { 0x74cca6e2, 10 },
    { 0, 0 },
    { 0x1e0ca6e4, 99 },
    { 0, 0 },
    { 0xab4ca6e6, 83 },
    { 0xd16ca6e6, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bca6eb, 90 },
    { 0xa31ca6ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ca6ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9ca6f2, 23 },
    { 0, 0 },
    { 0xdd2ca6f4, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ca6fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9ca6fe, 60 },
    { 0, 0 },
    { 0xa51ca700, 75 },
    { 0xe35ca701, 33 },
    { 0xc50ca702, 34 },
    { 0xa5cca703, 33 },
    { 0xd75ca704, 23 },
    { 0x4f8ca703, 95 },
    { 0x978ca706, 98 },
    { 0x1ccca700, 38 },
    { 0xb6bca708, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925ca70c, 90 },
    { 0xf07ca70d, 95 },
    { 0xe78ca70e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ca712, 34 },
    { 0x817ca713, 93 },
    { 0, 0 },
    { 0x76eca715, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649ca719, 61 },
    { 0x40dca71a, 68 },
    { 0x1c4ca719, 76 },
    { 0x8c3ca719, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2ca720, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeca723, 34 },
    { 0x94bca724, 61 },
    { 0, 0 },
    { 0x4eeca726, 46 },
    { 0, 0 },
    { 0x316ca728, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3ca72c, 46 },
    { 0x2cfca72d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00ca731, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaca734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ca738, 61 },
    { 0x3fcca738, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2ca743, 33 },
    { 0, 0 },
    { 0x757ca745, 61 },
    { 0x5acca746, 55 },
    { 0x685ca745, 69 },
    { 0xc00ca748, 76 },
    { 0xddaca749, 93 },
    { 0x8bcca74a, 23 },
    { 0xdbbca745, 33 },
    { 0x3c4ca74c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dca753, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560ca75e, 46 },
    { 0xa0bca75f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589ca763, 95 },
    { 0, 0 },
    { 0xc6bca765, 33 },
    { 0x316ca766, 55 },
    { 0x9f9ca767, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x391ca76a, 43 },
    { 0xf23ca76b, 61 },
    { 0x80ca76c, 43 },
    { 0xcc9ca76d, 90 },
    { 0x392ca76c, 84 },
    { 0, 0 },
    { 0x8fdca770, 61 },
    { 0xd3aca771, 33 },
    { 0x367ca771, 95 },
    { 0x161ca773, 89 },
    { 0x123ca774, 98 },
    { 0x619ca772, 23 },
    { 0x1b4ca776, 98 },
    { 0x5f4ca774, 61 },
    { 0xb1bca778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cca77d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7ca780, 90 },
    { 0xafeca781, 84 },
    { 0, 0 },
    { 0x73bca783, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fca787, 67 },
    { 0, 0 },
    { 0xe8dca789, 65 },
    { 0xb32ca78a, 90 },
    { 0x5c4ca78b, 23 },
    { 0xda4ca789, 98 },
    { 0xcf6ca789, 95 },
    { 0, 0 },
    { 0x34aca78f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbca794, 33 },
    { 0, 0 },
    { 0xb8dca796, 55 },
    { 0xf5ca797, 78 },
    { 0xdb7ca798, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bca79f, 23 },
    { 0, 0 },
    { 0x7fca7a1, 61 },
    { 0xffbca7a1, 93 },
    { 0xbfdca7a3, 90 },
    { 0xc8bca7a3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeca7a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ca7ac, 33 },
    { 0x610ca7ad, 90 },
    { 0, 0 },
    { 0x106ca7af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585ca7b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aca7ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cca7bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ca7c3, 34 },
    { 0x82bca7c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2ca7c7, 95 },
    { 0xb92ca7c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47ca7cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156ca7d3, 95 },
    { 0, 0 },
    { 0x986ca7d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ca7dd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x165ca7e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1dca7e4, 33 },
    { 0xa0bca7e5, 55 },
    { 0x84dca7e4, 90 },
    { 0xc60ca7e7, 61 },
    { 0xd29ca7e8, 90 },
    { 0xaeca7e5, 23 },
    { 0x338ca7ea, 61 },
    { 0x48fca7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23aca7ee, 95 },
    { 0xee0ca7ef, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x144ca7f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcca7f5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89ca7fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cca801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71ca804, 76 },
    { 0x1ddca805, 95 },
    { 0xe53ca805, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eca80b, 55 },
    { 0xc26ca80c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37ca80f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edca815, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5ca818, 66 },
    { 0xbd8ca819, 23 },
    { 0x316ca81a, 61 },
    { 0x5f3ca81b, 97 },
    { 0xe2cca819, 34 },
    { 0x44dca81b, 76 },
    { 0x1acca81a, 29 },
    { 0x419ca81f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ca822, 61 },
    { 0, 0 },
    { 0xfbdca824, 33 },
    { 0x1fbca825, 34 },
    { 0x653ca826, 67 },
    { 0xd95ca826, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x590ca82a, 100 },
    { 0x63dca82a, 76 },
    { 0x886ca82c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafca830, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ca833, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ca836, 61 },
    { 0x868ca837, 33 },
    { 0x101ca838, 64 },
    { 0, 0 },
    { 0x38bca83a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75ca841, 95 },
    { 0x350ca841, 52 },
    { 0xf21ca843, 90 },
    { 0x2c7ca844, 90 },
    { 0xef3ca845, 23 },
    { 0xedbca845, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5ca84b, 95 },
    { 0x85bca84c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bca84f, 76 },
    { 0xa38ca850, 90 },
    { 0, 0 },
    { 0x9f6ca852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4ca855, 61 },
    { 0x25ca855, 33 },
    { 0x3edca855, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ca85a, 90 },
    { 0xdf4ca85b, 98 },
    { 0x955ca85a, 17 },
    { 0, 0 },
    { 0x94fca85e, 52 },
    { 0, 0 },
    { 0xa7dca860, 90 },
    { 0x13ca861, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ca86c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275ca87a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fca87f, 93 },
    { 0xcfdca880, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdca883, 34 },
    { 0xaa4ca884, 76 },
    { 0, 0 },
    { 0xb9fca886, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbca88a, 90 },
    { 0, 0 },
    { 0xb3fca88c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdca893, 61 },
    { 0xb0bca894, 76 },
    { 0x1dbca893, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726ca89a, 98 },
    { 0x24aca89b, 61 },
    { 0x754ca89b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebca8a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ca8a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceca8a6, 34 },
    { 0xf46ca8a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbca8b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0ca8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cca8c0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3ca8cd, 76 },
    { 0xc5cca8cd, 95 },
    { 0x909ca8cf, 23 },
    { 0, 0 },
    { 0x2d1ca8d1, 61 },
    { 0x1dca8d2, 33 },
    { 0, 0 },
    { 0xd6bca8d4, 98 },
    { 0xb03ca8d4, 61 },
    { 0xa1fca8d6, 98 },
    { 0xb41ca8d7, 33 },
    { 0xca2ca8d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffca8dc, 95 },
    { 0, 0 },
    { 0x7d0ca8de, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dca8e7, 44 },
    { 0, 0 },
    { 0xbbdca8e9, 98 },
    { 0, 0 },
    { 0xdd2ca8eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29ca8ef, 76 },
    { 0x661ca8ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ca8f3, 23 },
    { 0x6ceca8f4, 34 },
    { 0xdcfca8f4, 95 },
    { 0xe7fca8f6, 33 },
    { 0x997ca8f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28ca8fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aca900, 99 },
    { 0, 0 },
    { 0x563ca902, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ca907, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ca90a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ca913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aca92d, 61 },
    { 0xf38ca92e, 76 },
    { 0x18ca92e, 34 },
    { 0xa40ca92e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ca935, 90 },
    { 0, 0 },
    { 0x39aca937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afca93c, 90 },
    { 0, 0 },
    { 0x23dca93e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44ca941, 33 },
    { 0x149ca941, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0eca94a, 76 },
    { 0x7fca94a, 76 },
    { 0xc66ca94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80ca950, 90 },
    { 0, 0 },
    { 0x28bca952, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5ca956, 90 },
    { 0xcf7ca956, 23 },
    { 0x49dca958, 69 },
    { 0x40fca959, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43ca960, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccca964, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5ca96a, 75 },
    { 0x657ca96b, 95 },
    { 0x316ca96c, 61 },
    { 0x610ca96d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x13aca970, 55 },
    { 0, 0 },
    { 0xe04ca972, 84 },
    { 0x2d6ca972, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ca978, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dca97c, 91 },
    { 0x208ca97d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84ca982, 76 },
    { 0xba7ca982, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eca986, 90 },
    { 0, 0 },
    { 0xcf3ca988, 90 },
    { 0, 0 },
    { 0xdc7ca98a, 91 },
    { 0, 0 },
    { 0xedcca98c, 77 },
    { 0x615ca98d, 38 },
    { 0x312ca98d, 81 },
    { 0x378ca98f, 98 },
    { 0xf92ca98c, 23 },
    { 0x4d5ca98e, 95 },
    { 0x7aeca991, 23 },
    { 0, 0 },
    { 0xcb6ca994, 95 },
    { 0x2e2ca995, 98 },
    { 0xcb5ca994, 67 },
    { 0x33ca997, 90 },
    { 0xb10ca996, 90 },
    { 0xa9bca999, 80 },
    { 0xd51ca99a, 81 },
    { 0, 0 },
    { 0x5afca99c, 33 },
    { 0xed8ca99c, 95 },
    { 0xb7ca99e, 98 },
    { 0x81ca99d, 23 },
    { 0, 0 },
    { 0x87aca9a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65ca9a4, 76 },
    { 0x387ca9a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eca9a8, 96 },
    { 0, 0 },
    { 0x90dca9aa, 17 },
    { 0, 0 },
    { 0x8f0ca9ac, 98 },
    { 0, 0 },
    { 0xc2dca9ae, 52 },
    { 0xf2dca9af, 95 },
    { 0x8cdca9b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898ca9b7, 55 },
    { 0x466ca9b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bca9bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cca9bf, 90 },
    { 0x928ca9bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bca9c4, 76 },
    { 0xe8ca9c4, 98 },
    { 0x830ca9c6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4ca9ce, 33 },
    { 0, 0 },
    { 0xb2ca9d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcca9d3, 95 },
    { 0, 0 },
    { 0x227ca9d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bca9d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dca9dd, 17 },
    { 0xb12ca9dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dca9e2, 33 },
    { 0x393ca9e3, 77 },
    { 0x67eca9e2, 23 },
    { 0xf74ca9e2, 76 },
    { 0x64eca9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825ca9ed, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5ca9f0, 61 },
    { 0x80aca9f1, 76 },
    { 0xba3ca9f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cca9f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bca9fe, 99 },
    { 0x429ca9ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625caa05, 90 },
    { 0xb44caa06, 90 },
    { 0xb3fcaa06, 93 },
    { 0x724caa08, 93 },
    { 0, 0 },
    { 0x559caa0a, 61 },
    { 0, 0 },
    { 0xd85caa0c, 23 },
    { 0xb4ccaa0c, 98 },
    { 0x878caa0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4caa12, 33 },
    { 0x635caa13, 100 },
    { 0, 0 },
    { 0x8e9caa15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecaa18, 61 },
    { 0, 0 },
    { 0xad5caa1a, 65 },
    { 0, 0 },
    { 0x401caa1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4caa1f, 76 },
    { 0x1c7caa20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecaa24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6caa2d, 33 },
    { 0xd72caa2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fcaa36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53caa3f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374caa45, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bcaa4f, 78 },
    { 0x42dcaa50, 98 },
    { 0x662caa4f, 61 },
    { 0xd9bcaa50, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25caa59, 43 },
    { 0xbb9caa5a, 90 },
    { 0, 0 },
    { 0x8d9caa5c, 61 },
    { 0x801caa5c, 36 },
    { 0, 0 },
    { 0x19fcaa5f, 90 },
    { 0, 0 },
    { 0xedbcaa61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6caa67, 87 },
    { 0x505caa67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512caa70, 91 },
    { 0x8decaa71, 33 },
    { 0, 0 },
    { 0xca3caa73, 23 },
    { 0, 0 },
    { 0x7b3caa75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ccaa7b, 100 },
    { 0x5e1caa7c, 33 },
    { 0, 0 },
    { 0xf90caa7e, 33 },
    { 0x31bcaa7f, 34 },
    { 0, 0 },
    { 0x7bacaa81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970caa8a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8acaa90, 61 },
    { 0x6a2caa90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3caa97, 33 },
    { 0x88fcaa97, 98 },
    { 0, 0 },
    { 0xd46caa9a, 90 },
    { 0, 0 },
    { 0x538caa9c, 77 },
    { 0x78caa9d, 61 },
    { 0x93acaa9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dcaaa5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fcaaa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0caab0, 46 },
    { 0, 0 },
    { 0xb17caab2, 61 },
    { 0x66ecaab3, 34 },
    { 0x64fcaab3, 34 },
    { 0, 0 },
    { 0x6e8caab6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1caabb, 90 },
    { 0xa13caabc, 61 },
    { 0x172caabc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6caaca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfcaace, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124caad4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69caad9, 55 },
    { 0, 0 },
    { 0xa6fcaadb, 87 },
    { 0x8dcaadb, 33 },
    { 0x42bcaadd, 98 },
    { 0xc3ecaade, 90 },
    { 0x665caade, 91 },
    { 0xe88caae0, 46 },
    { 0x92fcaadc, 90 },
    { 0xe85caae2, 78 },
    { 0x2aecaae2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1caae6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3caaea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9caaee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467caaf5, 90 },
    { 0xedccaaf6, 12 },
    { 0, 0 },
    { 0xc9bcaaf8, 90 },
    { 0x715caaf9, 34 },
    { 0, 0 },
    { 0x7bccaafb, 90 },
    { 0x8dccaafb, 23 },
    { 0xdd3caafd, 65 },
    { 0x969caafe, 88 },
    { 0x1facaaff, 23 },
    { 0xeeacaafb, 90 },
    { 0xd44cab01, 76 },
    { 0xf27caafb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ccab07, 90 },
    { 0, 0 },
    { 0x247cab09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5cab0d, 95 },
    { 0, 0 },
    { 0xde7cab0f, 23 },
    { 0xef3cab10, 91 },
    { 0xe7fcab11, 36 },
    { 0x2fdcab11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222cab19, 22 },
    { 0, 0 },
    { 0x7fecab1b, 55 },
    { 0x408cab1c, 77 },
    { 0x9a8cab1b, 55 },
    { 0x64ecab1e, 66 },
    { 0x159cab1f, 23 },
    { 0x8a0cab20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132cab24, 67 },
    { 0x7afcab25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfcab2a, 61 },
    { 0x51bcab2b, 90 },
    { 0xef5cab2c, 95 },
    { 0xa8ccab2d, 77 },
    { 0x161cab2e, 33 },
    { 0xe69cab2e, 33 },
    { 0x25ecab2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x486cab33, 76 },
    { 0x407cab33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02cab38, 76 },
    { 0xf0bcab39, 33 },
    { 0x384cab3a, 17 },
    { 0xc39cab3b, 90 },
    { 0xf6ccab3a, 98 },
    { 0x80dcab3a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ccab43, 77 },
    { 0x3f4cab43, 61 },
    { 0xefccab43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcab49, 98 },
    { 0x4e8cab4a, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4cab50, 33 },
    { 0, 0 },
    { 0xe1fcab52, 23 },
    { 0, 0 },
    { 0xc0dcab54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaccab5c, 61 },
    { 0x2ecab5d, 95 },
    { 0, 0 },
    { 0x38bcab5f, 83 },
    { 0, 0 },
    { 0xca4cab61, 33 },
    { 0x351cab62, 90 },
    { 0x102cab62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03cab66, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecab6b, 23 },
    { 0xc8cab6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aecab6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6cab74, 95 },
    { 0xc35cab75, 33 },
    { 0x609cab76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5cab79, 43 },
    { 0, 0 },
    { 0x7d4cab7b, 90 },
    { 0x5e1cab7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361cab84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26acab88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fcab8d, 93 },
    { 0xa90cab8e, 98 },
    { 0x18ccab8d, 95 },
    { 0, 0 },
    { 0x905cab91, 90 },
    { 0xb27cab92, 17 },
    { 0xb2cab92, 33 },
    { 0xf38cab94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794cab99, 61 },
    { 0xc0ccab99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7caba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacccaba6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62caba9, 61 },
    { 0xd32cabaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634cabaf, 85 },
    { 0x9ecabb0, 78 },
    { 0x8bfcabb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dcabb4, 88 },
    { 0, 0 },
    { 0x722cabb6, 62 },
    { 0, 0 },
    { 0xd89cabb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbcabbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181cabc1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x386cabc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5cabca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1cabcd, 52 },
    { 0, 0 },
    { 0xbe3cabcf, 95 },
    { 0xf85cabd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94cabd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28cabd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133cabe0, 55 },
    { 0x759cabe1, 61 },
    { 0xa5dcabe0, 34 },
    { 0x1dbcabe3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dcabe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09cabe9, 55 },
    { 0x811cabe9, 100 },
    { 0xbefcabea, 87 },
    { 0x655cabec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ecabfd, 55 },
    { 0, 0 },
    { 0xb95cabff, 90 },
    { 0xb12cac00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9cac03, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3cac08, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2cac0e, 33 },
    { 0x6e6cac0e, 61 },
    { 0x91fcac10, 23 },
    { 0xfaecac0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x318cac14, 52 },
    { 0x3cfcac14, 61 },
    { 0xbcbcac16, 46 },
    { 0xdebcac14, 98 },
    { 0x116cac18, 78 },
    { 0x543cac19, 23 },
    { 0, 0 },
    { 0x4f9cac1b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacac1e, 90 },
    { 0xd65cac1e, 61 },
    { 0xfffcac20, 46 },
    { 0, 0 },
    { 0xd7ecac22, 34 },
    { 0, 0 },
    { 0x465cac24, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bcac28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbcac34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbcac37, 77 },
    { 0xbaccac37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2cac3b, 95 },
    { 0xacac3c, 56 },
    { 0x7dacac3c, 98 },
    { 0x360cac3e, 91 },
    { 0xd95cac3f, 98 },
    { 0x7afcac40, 33 },
    { 0xf81cac3c, 95 },
    { 0xec7cac42, 90 },
    { 0x798cac3e, 98 },
    { 0x3d0cac42, 98 },
    { 0x37ecac45, 78 },
    { 0, 0 },
    { 0xe74cac47, 23 },
    { 0xd46cac47, 65 },
    { 0x560cac47, 76 },
    { 0xbc7cac47, 61 },
    { 0xf98cac4b, 90 },
    { 0x168cac4a, 76 },
    { 0, 0 },
    { 0xd15cac4e, 55 },
    { 0x5cecac4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19cac54, 98 },
    { 0, 0 },
    { 0x93ccac56, 77 },
    { 0xec0cac56, 90 },
    { 0x328cac56, 33 },
    { 0xa14cac59, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dcac62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bcac6b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43cac6f, 23 },
    { 0xf42cac70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ccac7b, 34 },
    { 0x229cac7c, 95 },
    { 0, 0 },
    { 0xef0cac7e, 100 },
    { 0x545cac7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbcac85, 75 },
    { 0xbd3cac85, 85 },
    { 0, 0 },
    { 0x18cac88, 90 },
    { 0x904cac89, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68cac8e, 33 },
    { 0, 0 },
    { 0xfbdcac90, 46 },
    { 0x73bcac90, 95 },
    { 0x246cac92, 75 },
    { 0x2e4cac93, 36 },
    { 0xdb4cac93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cac97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x478cac9a, 23 },
    { 0, 0 },
    { 0x45ecac9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfcac9f, 90 },
    { 0xf77caca0, 95 },
    { 0, 0 },
    { 0x34caca2, 40 },
    { 0, 0 },
    { 0x8ccaca4, 52 },
    { 0, 0 },
    { 0x610caca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ecaca9, 90 },
    { 0xe0dcacaa, 76 },
    { 0x7bfcacab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fcacae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6cacb1, 61 },
    { 0, 0 },
    { 0xc7acacb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5cacba, 55 },
    { 0x1c5cacba, 33 },
    { 0x3acacba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14cacc0, 61 },
    { 0xd84cacc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ccacc4, 95 },
    { 0xe85cacc4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fcacd0, 23 },
    { 0x96bcacd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x378cacd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ccacd9, 93 },
    { 0x464cacd9, 78 },
    { 0, 0 },
    { 0xaa3cacdc, 33 },
    { 0xeecacdd, 33 },
    { 0x9b1cacde, 95 },
    { 0xb3acacdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18acace2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4cace5, 76 },
    { 0xa88cace6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69cacf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942cacf5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7cacfd, 76 },
    { 0x53ccacfe, 98 },
    { 0xefacacff, 33 },
    { 0x425cad00, 61 },
    { 0x242cad00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3cad07, 23 },
    { 0x7b9cad08, 90 },
    { 0x40cad09, 23 },
    { 0, 0 },
    { 0xc5fcad0b, 76 },
    { 0xe2ccad0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12cad10, 34 },
    { 0x36cad11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efcad15, 55 },
    { 0x3ffcad16, 78 },
    { 0xd3ecad17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7cad1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbcad20, 95 },
    { 0x6d2cad21, 97 },
    { 0x38ccad20, 99 },
    { 0x3d5cad23, 90 },
    { 0, 0 },
    { 0x7e9cad25, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5cad2b, 96 },
    { 0x211cad2c, 100 },
    { 0xb63cad2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bccad31, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x331cad34, 100 },
    { 0x473cad34, 90 },
    { 0x74fcad35, 90 },
    { 0x35ccad34, 76 },
    { 0x143cad38, 33 },
    { 0, 0 },
    { 0xab8cad3a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79cad3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64cad43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47cad4a, 65 },
    { 0x26dcad4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4cad4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x671cad52, 33 },
    { 0x189cad53, 55 },
    { 0xf27cad54, 90 },
    { 0, 0 },
    { 0x4f7cad56, 55 },
    { 0x428cad57, 55 },
    { 0, 0 },
    { 0x2bbcad59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cad5d, 87 },
    { 0xe8bcad5e, 90 },
    { 0, 0 },
    { 0x3c9cad60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57cad64, 61 },
    { 0x541cad65, 95 },
    { 0xc44cad64, 67 },
    { 0xff8cad67, 52 },
    { 0x1a9cad64, 100 },
    { 0xff6cad64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cad71, 66 },
    { 0, 0 },
    { 0x732cad73, 65 },
    { 0xd3bcad74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0cad7c, 99 },
    { 0xbaacad7d, 33 },
    { 0xf40cad7d, 34 },
    { 0x96dcad7e, 81 },
    { 0x3b2cad80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5cad84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15cad8b, 61 },
    { 0x981cad8c, 23 },
    { 0, 0 },
    { 0x7e6cad8e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9acad95, 67 },
    { 0xc8ecad95, 66 },
    { 0, 0 },
    { 0x60cad98, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0cad9f, 93 },
    { 0x420cada0, 33 },
    { 0xe21cada1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36cada4, 34 },
    { 0x796cada5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bcada8, 93 },
    { 0x67acada9, 33 },
    { 0x481cada9, 33 },
    { 0xccacadab, 98 },
    { 0xcfecadac, 17 },
    { 0xaa7cadac, 55 },
    { 0xdefcadaa, 98 },
    { 0xd1dcadaf, 23 },
    { 0xde1cadaf, 84 },
    { 0x99acadb1, 93 },
    { 0xe2cadb1, 43 },
    { 0x2e9cadb1, 61 },
    { 0, 0 },
    { 0xc66cadb5, 95 },
    { 0, 0 },
    { 0xe3ecadb7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78cadba, 90 },
    { 0x9c2cadba, 23 },
    { 0x635cadbc, 23 },
    { 0x6c5cadbc, 61 },
    { 0x5bdcadbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624cadc2, 65 },
    { 0xbe9cadc3, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187cadc9, 55 },
    { 0, 0 },
    { 0x90dcadcb, 42 },
    { 0xa05cadcc, 98 },
    { 0xecfcadcc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ccadd3, 95 },
    { 0xfbbcadd3, 90 },
    { 0x577cadd5, 91 },
    { 0xebacadd6, 91 },
    { 0xaccadd7, 23 },
    { 0x743cadd6, 93 },
    { 0x402cadd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11caddc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721cade0, 61 },
    { 0x694cade1, 95 },
    { 0x25ecade0, 95 },
    { 0, 0 },
    { 0x591cade4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4cade9, 33 },
    { 0xeabcadea, 90 },
    { 0xf59cade9, 33 },
    { 0x1cdcadec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fcadf1, 67 },
    { 0xa35cadf2, 98 },
    { 0x967cadf2, 84 },
    { 0, 0 },
    { 0x9bccadf5, 67 },
    { 0, 0 },
    { 0x4e8cadf7, 95 },
    { 0, 0 },
    { 0x2c6cadf9, 95 },
    { 0x72cadfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845cae01, 61 },
    { 0, 0 },
    { 0x3b8cae03, 52 },
    { 0, 0 },
    { 0x9f1cae05, 33 },
    { 0x3bccae05, 76 },
    { 0, 0 },
    { 0x51acae08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5cae0c, 46 },
    { 0, 0 },
    { 0x42ecae0e, 23 },
    { 0xee0cae0e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16cae14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cae18, 33 },
    { 0x5f4cae19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cae1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99cae21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0acae24, 56 },
    { 0x74dcae24, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9cae29, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4cae2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76cae34, 90 },
    { 0, 0 },
    { 0x71ccae36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ecae3b, 61 },
    { 0x73cae3c, 90 },
    { 0x381cae3b, 95 },
    { 0x14cae3d, 33 },
    { 0x3f2cae3f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ccae42, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cae48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6cae4b, 61 },
    { 0xeedcae4b, 90 },
    { 0x6b2cae4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ccae50, 90 },
    { 0x13ecae51, 61 },
    { 0x5b3cae52, 90 },
    { 0xa1cae53, 95 },
    { 0x600cae53, 34 },
    { 0x1f9cae54, 88 },
    { 0x6fecae55, 33 },
    { 0x704cae57, 90 },
    { 0x6a7cae58, 33 },
    { 0, 0 },
    { 0x980cae5a, 81 },
    { 0, 0 },
    { 0x2a1cae5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecae60, 55 },
    { 0x80ccae61, 98 },
    { 0x87ccae61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ccae65, 90 },
    { 0, 0 },
    { 0x34ccae67, 33 },
    { 0xf85cae67, 40 },
    { 0x12cae69, 95 },
    { 0, 0 },
    { 0xa14cae6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8cae73, 33 },
    { 0x455cae74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbcae77, 55 },
    { 0xdbdcae78, 90 },
    { 0x14ecae79, 90 },
    { 0xabfcae7a, 97 },
    { 0, 0 },
    { 0x9cdcae7c, 76 },
    { 0xc1ccae7c, 23 },
    { 0, 0 },
    { 0x6dcae7f, 68 },
    { 0xbabcae80, 65 },
    { 0x7dfcae7f, 76 },
    { 0x9a9cae7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877cae88, 98 },
    { 0x139cae89, 55 },
    { 0x9fecae8a, 78 },
    { 0x2efcae8b, 90 },
    { 0x634cae8c, 90 },
    { 0x85ecae8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x727cae90, 43 },
    { 0x105cae91, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dcae94, 91 },
    { 0, 0 },
    { 0x27bcae96, 85 },
    { 0x1e6cae97, 34 },
    { 0x310cae98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dcae9f, 34 },
    { 0, 0 },
    { 0xb9dcaea1, 95 },
    { 0x843caea2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x463caea5, 95 },
    { 0, 0 },
    { 0x860caea7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x933caeaa, 55 },
    { 0, 0 },
    { 0x355caeac, 33 },
    { 0xc10caeac, 23 },
    { 0x128caeae, 76 },
    { 0x746caeaf, 84 },
    { 0xc0dcaeb0, 90 },
    { 0, 0 },
    { 0x448caeb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbcaeb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62caebb, 90 },
    { 0xac5caebc, 52 },
    { 0x94caebb, 98 },
    { 0, 0 },
    { 0x1eecaebf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71caec4, 55 },
    { 0xe6bcaec5, 94 },
    { 0xda5caec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55caec9, 92 },
    { 0x7decaeca, 23 },
    { 0x316caeca, 74 },
    { 0, 0 },
    { 0x329caecd, 98 },
    { 0xce2caecd, 33 },
    { 0x170caecf, 68 },
    { 0xfd6caecf, 77 },
    { 0, 0 },
    { 0xfa7caed2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32caed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x505caeda, 33 },
    { 0xb86caedb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9caede, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1caee1, 93 },
    { 0, 0 },
    { 0x87dcaee3, 85 },
    { 0xb55caee3, 23 },
    { 0xfeccaee5, 55 },
    { 0xd44caee3, 98 },
    { 0xed5caee7, 98 },
    { 0x21ccaee5, 76 },
    { 0x414caee6, 95 },
    { 0x417caeea, 90 },
    { 0xefcaeeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aecaeee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677caef6, 65 },
    { 0x3b3caef7, 17 },
    { 0x2bfcaef8, 78 },
    { 0x503caef8, 43 },
    { 0xca0caefa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18caeff, 66 },
    { 0x93ccaf00, 90 },
    { 0, 0 },
    { 0x41caf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1caf0a, 61 },
    { 0x735caf0b, 61 },
    { 0xb11caf0c, 95 },
    { 0x516caf0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdcaf14, 33 },
    { 0x289caf14, 95 },
    { 0, 0 },
    { 0xd1fcaf17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547caf1c, 28 },
    { 0xc4acaf1c, 61 },
    { 0xc48caf1c, 23 },
    { 0xc18caf1f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80caf29, 87 },
    { 0xf42caf2a, 66 },
    { 0x7b4caf2b, 98 },
    { 0x136caf2a, 88 },
    { 0xdf1caf2d, 33 },
    { 0xaeacaf2e, 90 },
    { 0, 0 },
    { 0x8d9caf30, 98 },
    { 0, 0 },
    { 0x6bccaf32, 46 },
    { 0, 0 },
    { 0x520caf34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3caf37, 98 },
    { 0x4f6caf37, 66 },
    { 0x770caf39, 77 },
    { 0xbb0caf39, 55 },
    { 0x6e9caf3b, 42 },
    { 0x53dcaf39, 98 },
    { 0x80dcaf3a, 98 },
    { 0xaf4caf3a, 33 },
    { 0x376caf3f, 90 },
    { 0xfb6caf3f, 98 },
    { 0x7b9caf41, 23 },
    { 0xdadcaf42, 98 },
    { 0x592caf40, 34 },
    { 0xb94caf37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7caf47, 84 },
    { 0x700caf47, 98 },
    { 0x81acaf47, 90 },
    { 0xfc8caf4a, 90 },
    { 0xa2bcaf47, 90 },
    { 0, 0 },
    { 0x1a6caf4d, 33 },
    { 0x2c1caf4e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4caf53, 90 },
    { 0xce8caf54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ccaf58, 23 },
    { 0x1e6caf59, 77 },
    { 0, 0 },
    { 0xa2fcaf5b, 61 },
    { 0, 0 },
    { 0xe20caf5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6caf62, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ccaf6a, 61 },
    { 0xef4caf6b, 23 },
    { 0, 0 },
    { 0xca3caf6d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41caf72, 92 },
    { 0, 0 },
    { 0x731caf74, 90 },
    { 0x75bcaf75, 64 },
    { 0, 0 },
    { 0xcdcaf77, 87 },
    { 0x224caf78, 98 },
    { 0xa46caf79, 61 },
    { 0xb6caf78, 91 },
    { 0, 0 },
    { 0xe0dcaf7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacaf85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6caf88, 48 },
    { 0x8c5caf89, 78 },
    { 0x8eecaf8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddcaf8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccaf91, 90 },
    { 0xb2acaf92, 33 },
    { 0, 0 },
    { 0x33ecaf94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2caf9e, 90 },
    { 0x6d0caf9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x681cafa2, 61 },
    { 0x3c1cafa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5cafa6, 98 },
    { 0x682cafa7, 52 },
    { 0xde4cafa6, 33 },
    { 0x28bcafa9, 67 },
    { 0, 0 },
    { 0xaafcafab, 95 },
    { 0xde2cafac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30cafaf, 66 },
    { 0xab8cafaf, 95 },
    { 0x60ecafb0, 88 },
    { 0, 0 },
    { 0x87dcafb3, 20 },
    { 0xe65cafb3, 91 },
    { 0, 0 },
    { 0x4bdcafb6, 95 },
    { 0, 0 },
    { 0x3b5cafb8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bcafbf, 33 },
    { 0x2a4cafc0, 98 },
    { 0, 0 },
    { 0xcedcafc2, 98 },
    { 0, 0 },
    { 0xf12cafc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528cafc9, 52 },
    { 0x5eacafca, 33 },
    { 0, 0 },
    { 0x7e3cafcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecafdc, 52 },
    { 0xeb1cafdd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x797cafe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaecafe4, 90 },
    { 0x958cafe4, 97 },
    { 0x667cafe6, 90 },
    { 0, 0 },
    { 0xd82cafe8, 98 },
    { 0, 0 },
    { 0x470cafea, 61 },
    { 0xd33cafeb, 33 },
    { 0, 0 },
    { 0xcdfcafed, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196caff6, 87 },
    { 0xfabcaff6, 99 },
    { 0x2f2caff6, 98 },
    { 0, 0 },
    { 0x317caffa, 99 },
    { 0x421caffa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23cb004, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3cb00a, 98 },
    { 0, 0 },
    { 0xfffcb00c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7cb011, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508cb016, 76 },
    { 0xb69cb017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541cb022, 90 },
    { 0x447cb022, 93 },
    { 0, 0 },
    { 0xae2cb025, 33 },
    { 0xe2ccb026, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eccb02a, 55 },
    { 0xc9fcb02b, 66 },
    { 0x674cb02a, 43 },
    { 0, 0 },
    { 0x6b6cb02e, 61 },
    { 0xa89cb02f, 66 },
    { 0xc4fcb02f, 66 },
    { 0x668cb031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cb034, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7cb038, 85 },
    { 0, 0 },
    { 0x17fcb03a, 46 },
    { 0x49dcb03a, 90 },
    { 0xa32cb03a, 75 },
    { 0xdffcb03d, 81 },
    { 0x9f0cb03d, 33 },
    { 0xf42cb03f, 84 },
    { 0, 0 },
    { 0x693cb041, 80 },
    { 0xcc6cb042, 97 },
    { 0x1fbcb042, 61 },
    { 0x4bacb042, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x680cb047, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dacb04a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638cb052, 90 },
    { 0, 0 },
    { 0x640cb054, 33 },
    { 0x3b5cb055, 55 },
    { 0xcf5cb054, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaacb05b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ccb05e, 90 },
    { 0x4f5cb05e, 90 },
    { 0x767cb060, 61 },
    { 0xeaccb061, 43 },
    { 0x3d7cb060, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23cb066, 95 },
    { 0, 0 },
    { 0x715cb068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01cb06b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edcb06f, 61 },
    { 0, 0 },
    { 0x88dcb071, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xceccb074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dcb078, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9cb07b, 67 },
    { 0, 0 },
    { 0x67cb07d, 76 },
    { 0, 0 },
    { 0x6f8cb07f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x780cb082, 99 },
    { 0, 0 },
    { 0x515cb084, 90 },
    { 0x24fcb085, 84 },
    { 0, 0 },
    { 0x989cb087, 90 },
    { 0x380cb087, 35 },
    { 0, 0 },
    { 0xd44cb08a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ccb08d, 61 },
    { 0xe70cb08d, 55 },
    { 0x8a1cb08f, 38 },
    { 0x4c9cb08f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x937cb093, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4cb096, 67 },
    { 0x6cccb096, 95 },
    { 0, 0 },
    { 0xd0fcb099, 52 },
    { 0x9a8cb099, 98 },
    { 0xcaacb09b, 33 },
    { 0, 0 },
    { 0x732cb09d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361cb0a2, 43 },
    { 0x3efcb0a3, 15 },
    { 0xb17cb0a4, 33 },
    { 0x983cb0a2, 98 },
    { 0xe52cb0a6, 91 },
    { 0x6b2cb0a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccb0aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cb0ad, 76 },
    { 0x35ecb0ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08cb0bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fcb0bf, 23 },
    { 0x55ccb0bf, 78 },
    { 0x574cb0c1, 33 },
    { 0xe68cb0c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x937cb0c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeccb0c9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcb0ce, 34 },
    { 0x33dcb0ce, 61 },
    { 0, 0 },
    { 0x27cb0d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5cb0d9, 90 },
    { 0x223cb0d9, 66 },
    { 0xb57cb0d9, 61 },
    { 0x57bcb0dc, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcb0e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ecb0e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151cb0f6, 98 },
    { 0xd49cb0f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7cb0fa, 68 },
    { 0, 0 },
    { 0x650cb0fc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cb103, 33 },
    { 0, 0 },
    { 0xad1cb105, 76 },
    { 0x66ccb106, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cb10c, 46 },
    { 0xcbcb10d, 76 },
    { 0xe83cb10c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccb111, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ecb115, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5cb11b, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998cb120, 33 },
    { 0xccfcb121, 34 },
    { 0, 0 },
    { 0x80ecb123, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77cb129, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68cb130, 23 },
    { 0, 0 },
    { 0x3ddcb132, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebacb13a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdcb144, 101 },
    { 0xb75cb145, 85 },
    { 0xca9cb146, 95 },
    { 0xf9ecb146, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x32dcb14a, 66 },
    { 0x4ecb14b, 23 },
    { 0x493cb14c, 66 },
    { 0x434cb14d, 34 },
    { 0, 0 },
    { 0x9c0cb14f, 98 },
    { 0x1cccb150, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x240cb153, 90 },
    { 0, 0 },
    { 0x344cb155, 23 },
    { 0, 0 },
    { 0xdbfcb157, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74acb15c, 36 },
    { 0x50ecb15d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0cb160, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164cb165, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x900cb168, 68 },
    { 0xbd4cb169, 61 },
    { 0xdf9cb169, 90 },
    { 0xdf1cb169, 33 },
    { 0xe40cb16c, 78 },
    { 0x3dcb169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92cb172, 93 },
    { 0xb21cb173, 95 },
    { 0x64acb174, 98 },
    { 0x3b7cb174, 34 },
    { 0xeb6cb172, 66 },
    { 0xcf1cb177, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x148cb17a, 52 },
    { 0xfdacb17b, 61 },
    { 0xe57cb17b, 55 },
    { 0x763cb17b, 76 },
    { 0xd10cb17b, 100 },
    { 0x26ecb17f, 77 },
    { 0x7d4cb17d, 84 },
    { 0x9edcb180, 23 },
    { 0x9fbcb182, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ccb185, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddcb18e, 55 },
    { 0x3d6cb18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdcb193, 93 },
    { 0, 0 },
    { 0xb0cb195, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907cb19c, 61 },
    { 0, 0 },
    { 0x4a8cb19e, 76 },
    { 0xf30cb19e, 43 },
    { 0x872cb1a0, 23 },
    { 0x7aacb1a1, 46 },
    { 0xececb1a2, 78 },
    { 0x429cb1a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7cb1ab, 99 },
    { 0x908cb1ac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ccb1af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadcb1b3, 23 },
    { 0x2d0cb1b4, 33 },
    { 0, 0 },
    { 0xbf2cb1b6, 34 },
    { 0x57ecb1b7, 90 },
    { 0x9c2cb1b6, 96 },
    { 0xc64cb1b9, 95 },
    { 0xf81cb1b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffacb1c4, 61 },
    { 0xe5ccb1c4, 17 },
    { 0, 0 },
    { 0xc7cb1c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689cb1cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9cb1d3, 76 },
    { 0, 0 },
    { 0x901cb1d5, 55 },
    { 0x452cb1d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcacb1da, 77 },
    { 0xe6ecb1db, 76 },
    { 0x1c5cb1db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5cb1e0, 101 },
    { 0xa8dcb1e0, 33 },
    { 0x5dacb1e2, 100 },
    { 0x361cb1e3, 64 },
    { 0x9f8cb1e4, 66 },
    { 0xa14cb1e5, 33 },
    { 0x177cb1e6, 61 },
    { 0x85acb1e0, 90 },
    { 0xb8bcb1e8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8cb1ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3cb1f3, 98 },
    { 0, 0 },
    { 0xd65cb1f5, 61 },
    { 0x68dcb1f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fcb1fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbccb1fe, 23 },
    { 0x5f0cb1ff, 99 },
    { 0, 0 },
    { 0x477cb201, 38 },
    { 0xa28cb201, 61 },
    { 0x2d4cb201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1cb206, 98 },
    { 0xda2cb207, 99 },
    { 0x259cb207, 99 },
    { 0x456cb206, 67 },
    { 0x81cb20a, 99 },
    { 0x23bcb20b, 90 },
    { 0x528cb206, 43 },
    { 0x161cb20d, 34 },
    { 0xa79cb20e, 23 },
    { 0xf90cb20b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4cb215, 93 },
    { 0x781cb215, 87 },
    { 0x45dcb217, 90 },
    { 0, 0 },
    { 0xe6ccb219, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dcb21c, 93 },
    { 0x6d2cb21d, 23 },
    { 0x98cb21e, 90 },
    { 0, 0 },
    { 0x144cb220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12cb226, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5cb22c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ecb234, 100 },
    { 0x339cb235, 90 },
    { 0, 0 },
    { 0xd4bcb237, 95 },
    { 0x841cb238, 85 },
    { 0xd90cb239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7cb23e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefccb245, 61 },
    { 0x901cb246, 43 },
    { 0x944cb247, 90 },
    { 0x5a7cb248, 66 },
    { 0xb20cb247, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8cb251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514cb255, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fcb258, 95 },
    { 0, 0 },
    { 0xc54cb25a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fccb25d, 33 },
    { 0x8dbcb25e, 90 },
    { 0xa89cb25f, 95 },
    { 0xa65cb260, 61 },
    { 0x710cb25e, 34 },
    { 0x7accb25d, 75 },
    { 0, 0 },
    { 0xab9cb264, 76 },
    { 0x8dacb264, 91 },
    { 0, 0 },
    { 0x70fcb267, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cccb26a, 33 },
    { 0, 0 },
    { 0x943cb26c, 61 },
    { 0xc5fcb26c, 61 },
    { 0, 0 },
    { 0x7b4cb26f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3cb274, 68 },
    { 0x957cb275, 33 },
    { 0, 0 },
    { 0x60dcb277, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x586cb27a, 43 },
    { 0xb35cb27b, 40 },
    { 0x9edcb27c, 34 },
    { 0, 0 },
    { 0x7aecb27e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23cb282, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11cb285, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0acb28a, 23 },
    { 0x3e7cb28b, 98 },
    { 0, 0 },
    { 0xe1acb28d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ecb297, 90 },
    { 0xb34cb298, 66 },
    { 0xa13cb299, 98 },
    { 0x439cb299, 90 },
    { 0x4c2cb29b, 65 },
    { 0x824cb299, 66 },
    { 0x5cb29d, 33 },
    { 0x38bcb29e, 78 },
    { 0, 0 },
    { 0xaaecb2a0, 90 },
    { 0xaeecb2a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75cb2a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdcb2ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fcb2b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fcb2ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04cb2be, 61 },
    { 0, 0 },
    { 0xcdecb2c0, 95 },
    { 0xb0cb2c0, 34 },
    { 0, 0 },
    { 0xf44cb2c3, 33 },
    { 0xe28cb2c4, 85 },
    { 0, 0 },
    { 0xcdacb2c6, 51 },
    { 0xd0cb2c7, 90 },
    { 0xf9dcb2c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x822cb2cb, 78 },
    { 0x896cb2cc, 46 },
    { 0x50bcb2cd, 17 },
    { 0x918cb2cd, 91 },
    { 0x279cb2cd, 43 },
    { 0x9adcb2d0, 88 },
    { 0x7bdcb2cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bccb2dc, 17 },
    { 0x884cb2dc, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9cb2e1, 90 },
    { 0, 0 },
    { 0xdecb2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1cb2ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbcb2ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4cb2f7, 90 },
    { 0xb25cb2f7, 93 },
    { 0, 0 },
    { 0x50fcb2fa, 75 },
    { 0xfa3cb2fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dcb300, 34 },
    { 0x2cdcb301, 66 },
    { 0xdc0cb302, 90 },
    { 0x12cb303, 76 },
    { 0, 0 },
    { 0x383cb305, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ccb311, 90 },
    { 0x8f6cb311, 55 },
    { 0x7fdcb313, 61 },
    { 0x77acb313, 98 },
    { 0xa6ecb315, 78 },
    { 0xe4fcb316, 99 },
    { 0, 0 },
    { 0x96ccb318, 69 },
    { 0, 0 },
    { 0x5e0cb31a, 98 },
    { 0xfc2cb31b, 29 },
    { 0xe62cb31b, 95 },
    { 0x834cb31d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fcb321, 78 },
    { 0x6e4cb321, 55 },
    { 0xdfacb323, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263cb329, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dcb32c, 33 },
    { 0xd0cb32d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0cb335, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fcb33a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bcb33d, 87 },
    { 0xb21cb33e, 78 },
    { 0x17dcb33f, 33 },
    { 0xf95cb33e, 87 },
    { 0xab6cb340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23acb348, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2cb34b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bcb350, 61 },
    { 0xacecb350, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31cb354, 76 },
    { 0xbb7cb355, 91 },
    { 0, 0 },
    { 0xf9bcb357, 90 },
    { 0x81ecb358, 95 },
    { 0, 0 },
    { 0x346cb35a, 98 },
    { 0x8becb35b, 95 },
    { 0xa6dcb35a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xabecb35f, 84 },
    { 0xb47cb35f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bcb364, 90 },
    { 0, 0 },
    { 0x31acb366, 34 },
    { 0x790cb367, 90 },
    { 0xf4bcb368, 67 },
    { 0x5dfcb366, 61 },
    { 0xefacb368, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09cb370, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7cb37a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5cb37d, 61 },
    { 0x80ccb37d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9cb387, 93 },
    { 0x2ebcb388, 23 },
    { 0, 0 },
    { 0xb8fcb38a, 85 },
    { 0x453cb38b, 61 },
    { 0x5a7cb38c, 68 },
    { 0xb44cb38c, 34 },
    { 0x1fcb38e, 91 },
    { 0x7e2cb38f, 23 },
    { 0, 0 },
    { 0x12dcb391, 76 },
    { 0x43acb391, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dcb39b, 90 },
    { 0xf68cb39c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22cb3a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163cb3a8, 55 },
    { 0x57cb3a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165cb3b0, 52 },
    { 0, 0 },
    { 0xd56cb3b2, 55 },
    { 0, 0 },
    { 0x4f5cb3b4, 90 },
    { 0xfb7cb3b5, 84 },
    { 0xd85cb3b6, 88 },
    { 0x6d1cb3b7, 61 },
    { 0x553cb3b7, 95 },
    { 0xddcb3b5, 90 },
    { 0x964cb3ba, 90 },
    { 0x6cdcb3b6, 98 },
    { 0x76bcb3bc, 98 },
    { 0x99acb3bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42cb3c0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123cb3c7, 33 },
    { 0x879cb3c8, 99 },
    { 0xa98cb3c7, 90 },
    { 0x4b0cb3c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92cb3cd, 93 },
    { 0x5dacb3ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593cb3d4, 76 },
    { 0, 0 },
    { 0xbdbcb3d6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5cb3e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952cb3e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45cb3ed, 95 },
    { 0x80cb3ee, 34 },
    { 0, 0 },
    { 0xaaccb3f0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37cb3f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962cb3f8, 93 },
    { 0, 0 },
    { 0xd3cb3fa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9cb3fd, 55 },
    { 0, 0 },
    { 0xeb2cb3ff, 23 },
    { 0, 0 },
    { 0xc89cb401, 61 },
    { 0x84cb401, 84 },
    { 0, 0 },
    { 0xb70cb404, 98 },
    { 0x781cb404, 84 },
    { 0x3edcb405, 98 },
    { 0xa60cb407, 23 },
    { 0x1b7cb407, 99 },
    { 0x742cb409, 34 },
    { 0x542cb409, 90 },
    { 0xe74cb40b, 43 },
    { 0xcb5cb40a, 90 },
    { 0x690cb407, 46 },
    { 0, 0 },
    { 0x9d6cb40f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32cb412, 34 },
    { 0xc2fcb412, 34 },
    { 0, 0 },
    { 0x9c9cb415, 61 },
    { 0, 0 },
    { 0xf73cb417, 55 },
    { 0, 0 },
    { 0x3c6cb419, 93 },
    { 0, 0 },
    { 0x29acb41b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5cb41f, 76 },
    { 0, 0 },
    { 0x4f2cb421, 76 },
    { 0x2c0cb422, 39 },
    { 0xa68cb423, 68 },
    { 0, 0 },
    { 0x388cb425, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbcb428, 76 },
    { 0xd78cb429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adcb42d, 28 },
    { 0x773cb42e, 40 },
    { 0x4ddcb42e, 90 },
    { 0, 0 },
    { 0xd30cb431, 55 },
    { 0x7dccb431, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fcb435, 95 },
    { 0, 0 },
    { 0x103cb437, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726cb43d, 88 },
    { 0x5b4cb43e, 84 },
    { 0x86dcb43f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cb442, 77 },
    { 0, 0 },
    { 0xafecb444, 98 },
    { 0xdbfcb445, 88 },
    { 0, 0 },
    { 0x599cb447, 91 },
    { 0x9bcb448, 23 },
    { 0xb38cb448, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53cb44c, 90 },
    { 0xff6cb44c, 100 },
    { 0x651cb44d, 98 },
    { 0x472cb44f, 46 },
    { 0xa07cb450, 98 },
    { 0xeadcb451, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedcb454, 23 },
    { 0xe17cb455, 84 },
    { 0xeafcb454, 23 },
    { 0, 0 },
    { 0x637cb458, 23 },
    { 0x11ecb459, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1cb45e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ecb462, 34 },
    { 0xf4ccb462, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x949cb466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1cb46d, 33 },
    { 0xf6cb46e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7cb471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ecb478, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x365cb47b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63cb483, 93 },
    { 0x3ccb484, 33 },
    { 0x3acb485, 90 },
    { 0, 0 },
    { 0xd83cb487, 90 },
    { 0, 0 },
    { 0x39bcb489, 90 },
    { 0x4a8cb48a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0cb492, 61 },
    { 0x30cb492, 90 },
    { 0xb94cb494, 95 },
    { 0x29cb495, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fcb49f, 34 },
    { 0xf3fcb4a0, 86 },
    { 0x80ecb4a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6cb4a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1cb4b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31cb4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdcb4bb, 55 },
    { 0xa85cb4bb, 98 },
    { 0x66cb4bd, 33 },
    { 0, 0 },
    { 0xe6fcb4bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efcb4c3, 90 },
    { 0x976cb4c4, 98 },
    { 0xf42cb4c5, 61 },
    { 0xc4ccb4c6, 33 },
    { 0x536cb4c6, 76 },
    { 0xacbcb4c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893cb4cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fcb4cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dacb4d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5cb4d9, 98 },
    { 0x728cb4da, 98 },
    { 0x5eecb4da, 55 },
    { 0x817cb4da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0cb4e0, 95 },
    { 0, 0 },
    { 0x521cb4e2, 33 },
    { 0x473cb4e3, 55 },
    { 0, 0 },
    { 0xba7cb4e5, 99 },
    { 0, 0 },
    { 0xcdecb4e7, 98 },
    { 0x84dcb4e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2cb4ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeecb4f1, 90 },
    { 0x915cb4f1, 95 },
    { 0x41fcb4f3, 23 },
    { 0xc0bcb4f4, 90 },
    { 0x7c4cb4f3, 90 },
    { 0x3e6cb4f4, 76 },
    { 0xadccb4f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1cb4fa, 98 },
    { 0, 0 },
    { 0x51acb4fc, 61 },
    { 0, 0 },
    { 0x42bcb4fe, 100 },
    { 0x31bcb4ff, 90 },
    { 0, 0 },
    { 0x113cb501, 98 },
    { 0x41fcb502, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edcb50a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45cb50e, 90 },
    { 0, 0 },
    { 0x834cb510, 90 },
    { 0, 0 },
    { 0xc7fcb512, 93 },
    { 0, 0 },
    { 0x5c5cb514, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecb51b, 43 },
    { 0x3f7cb51c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbacb521, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01cb529, 95 },
    { 0x41fcb52a, 90 },
    { 0x8bacb52b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cb52e, 46 },
    { 0, 0 },
    { 0x173cb530, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33cb53a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4cb53d, 98 },
    { 0, 0 },
    { 0x5afcb53f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554cb550, 99 },
    { 0, 0 },
    { 0x206cb552, 95 },
    { 0xab4cb553, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fcb556, 76 },
    { 0, 0 },
    { 0x7c8cb558, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3cb55e, 61 },
    { 0x227cb55f, 34 },
    { 0xed1cb55f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cb567, 34 },
    { 0x537cb568, 55 },
    { 0xeddcb569, 90 },
    { 0x822cb569, 67 },
    { 0xa38cb569, 46 },
    { 0x10ecb56c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dcb572, 33 },
    { 0, 0 },
    { 0xb30cb574, 81 },
    { 0, 0 },
    { 0x9fccb576, 65 },
    { 0xd38cb576, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00cb57c, 33 },
    { 0x938cb57d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7cb582, 55 },
    { 0x9a5cb583, 61 },
    { 0x6a5cb583, 33 },
    { 0xacecb585, 84 },
    { 0x3f9cb582, 90 },
    { 0, 0 },
    { 0x8d9cb588, 43 },
    { 0, 0 },
    { 0x209cb58a, 76 },
    { 0, 0 },
    { 0x1b2cb58c, 90 },
    { 0x2d4cb58d, 90 },
    { 0x7e2cb58d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeecb591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768cb599, 98 },
    { 0x2d4cb59a, 90 },
    { 0x1cecb59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7cb5a0, 90 },
    { 0xf99cb5a1, 91 },
    { 0xd5acb5a2, 46 },
    { 0xe26cb5a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eccb5a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adcb5ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x169cb5b0, 33 },
    { 0xf4acb5b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980cb5b6, 90 },
    { 0x918cb5b7, 98 },
    { 0x65fcb5b8, 91 },
    { 0x428cb5b9, 95 },
    { 0xa5ccb5ba, 61 },
    { 0xb7ecb5bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8cb5be, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ecb5cd, 23 },
    { 0, 0 },
    { 0xd33cb5cf, 90 },
    { 0, 0 },
    { 0x781cb5d1, 98 },
    { 0, 0 },
    { 0xe2ecb5d3, 90 },
    { 0, 0 },
    { 0x488cb5d5, 98 },
    { 0x61cb5d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cb5db, 66 },
    { 0x4acb5db, 61 },
    { 0x379cb5dd, 95 },
    { 0x8d3cb5de, 99 },
    { 0, 0 },
    { 0x2e0cb5e0, 33 },
    { 0xb48cb5e1, 75 },
    { 0xb92cb5e2, 90 },
    { 0x2b3cb5e3, 95 },
    { 0, 0 },
    { 0xdc9cb5e5, 85 },
    { 0x9b8cb5e6, 19 },
    { 0, 0 },
    { 0xd3cb5e8, 97 },
    { 0x3c5cb5e9, 98 },
    { 0x701cb5ea, 33 },
    { 0, 0 },
    { 0xf40cb5ec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ccb5f2, 61 },
    { 0xb42cb5f2, 66 },
    { 0, 0 },
    { 0x1becb5f5, 55 },
    { 0, 0 },
    { 0xe40cb5f7, 93 },
    { 0, 0 },
    { 0x899cb5f9, 65 },
    { 0x54cb5f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bcb5ff, 55 },
    { 0, 0 },
    { 0x908cb601, 23 },
    { 0xad1cb602, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63cb607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cb610, 23 },
    { 0xbd8cb610, 90 },
    { 0xbf8cb611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dcb61a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828cb623, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dcb626, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8cb62a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2cb635, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ccb638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88cb63f, 99 },
    { 0xbf7cb63f, 34 },
    { 0, 0 },
    { 0x393cb642, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413cb647, 33 },
    { 0xe12cb648, 75 },
    { 0x858cb649, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512cb653, 69 },
    { 0, 0 },
    { 0x263cb655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425cb65c, 23 },
    { 0xf0bcb65d, 40 },
    { 0xcffcb65e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772cb667, 61 },
    { 0xdb7cb667, 61 },
    { 0, 0 },
    { 0xe94cb66a, 90 },
    { 0xd92cb66b, 52 },
    { 0x7facb66c, 98 },
    { 0xc83cb66c, 80 },
    { 0xb92cb66c, 33 },
    { 0x35acb66f, 55 },
    { 0x339cb66b, 91 },
    { 0, 0 },
    { 0x937cb672, 33 },
    { 0x967cb672, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82cb677, 33 },
    { 0x99bcb677, 55 },
    { 0xf9cb679, 95 },
    { 0x7c4cb679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7cb67d, 98 },
    { 0, 0 },
    { 0x2b6cb67f, 34 },
    { 0xadfcb680, 67 },
    { 0x9f2cb681, 87 },
    { 0xf0fcb682, 85 },
    { 0x68bcb682, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62cb68b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1cb68f, 76 },
    { 0xa4ecb68f, 61 },
    { 0, 0 },
    { 0x834cb692, 61 },
    { 0x28ecb693, 43 },
    { 0x6aacb694, 90 },
    { 0x16fcb693, 33 },
    { 0xfcbcb695, 90 },
    { 0x695cb696, 61 },
    { 0x96fcb698, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2cb69c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ccb6a0, 99 },
    { 0x818cb6a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ecb6a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9cb6ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4decb6ba, 76 },
    { 0x422cb6bb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cccb6be, 76 },
    { 0x7e6cb6bf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6facb6c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587cb6cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69cb6d3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bcb6d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91cb6df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cb6e6, 61 },
    { 0x7e5cb6e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ccb6ea, 66 },
    { 0, 0 },
    { 0xe80cb6ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bccb6f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7cb6f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cb6fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861cb701, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6cb706, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2cb70a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ecb70d, 98 },
    { 0x641cb70d, 33 },
    { 0x8d5cb70f, 34 },
    { 0x59dcb70d, 87 },
    { 0x79dcb711, 90 },
    { 0xcb2cb70f, 23 },
    { 0x46ccb710, 24 },
    { 0x959cb714, 66 },
    { 0, 0 },
    { 0x7fbcb716, 61 },
    { 0, 0 },
    { 0xfb9cb718, 46 },
    { 0xe84cb718, 98 },
    { 0xa3fcb719, 42 },
    { 0x495cb71b, 95 },
    { 0x50acb71c, 46 },
    { 0x431cb71d, 76 },
    { 0xd2fcb71d, 95 },
    { 0x369cb71f, 76 },
    { 0x9e4cb720, 34 },
    { 0x1cbcb71f, 33 },
    { 0xde3cb720, 33 },
    { 0x968cb71a, 61 },
    { 0, 0 },
    { 0x4f7cb725, 23 },
    { 0x693cb726, 33 },
    { 0xff4cb727, 91 },
    { 0x207cb727, 52 },
    { 0x57ccb727, 76 },
    { 0x3c7cb727, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0cb72d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7cb734, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacb73b, 90 },
    { 0x126cb73b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3cb745, 88 },
    { 0x804cb746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8cb74a, 33 },
    { 0x9acb74a, 55 },
    { 0xb7ecb74c, 88 },
    { 0xf1bcb74d, 98 },
    { 0xd5ccb74d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fcb752, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ccb758, 61 },
    { 0xe05cb759, 76 },
    { 0xd63cb75a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x669cb75d, 43 },
    { 0x49fcb75e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacccb765, 67 },
    { 0x2e6cb766, 34 },
    { 0x433cb766, 23 },
    { 0xb75cb768, 90 },
    { 0x341cb769, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4cb76d, 78 },
    { 0, 0 },
    { 0x50dcb76f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x481cb772, 95 },
    { 0xdbccb773, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x247cb776, 95 },
    { 0x7a0cb777, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x643cb77a, 97 },
    { 0xbb5cb77a, 98 },
    { 0x3e7cb77b, 61 },
    { 0x5b4cb77d, 90 },
    { 0, 0 },
    { 0x518cb77f, 98 },
    { 0x3adcb780, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83cb783, 23 },
    { 0x424cb784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bcb789, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867cb798, 23 },
    { 0, 0 },
    { 0xde6cb79a, 84 },
    { 0, 0 },
    { 0x54ecb79c, 87 },
    { 0, 0 },
    { 0xac8cb79e, 33 },
    { 0x3c1cb79e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55cb7a4, 95 },
    { 0, 0 },
    { 0x72ccb7a6, 87 },
    { 0x799cb7a7, 95 },
    { 0x43fcb7a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cb7ab, 95 },
    { 0x6b4cb7ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdcb7b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ecb7b8, 23 },
    { 0, 0 },
    { 0x17dcb7ba, 77 },
    { 0, 0 },
    { 0xedcb7bc, 17 },
    { 0xf2fcb7bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x994cb7c0, 98 },
    { 0x903cb7c1, 90 },
    { 0x327cb7c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3cb7c8, 90 },
    { 0x323cb7c9, 76 },
    { 0xd32cb7c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bcb7cd, 40 },
    { 0x9edcb7ce, 82 },
    { 0xa3acb7cf, 90 },
    { 0x16dcb7d0, 33 },
    { 0x71fcb7d0, 33 },
    { 0x20fcb7d0, 96 },
    { 0xc81cb7d3, 33 },
    { 0x2bccb7d4, 95 },
    { 0xaeecb7d5, 90 },
    { 0x30cb7d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3cb7da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5cb7df, 90 },
    { 0x41ecb7e0, 98 },
    { 0x391cb7df, 87 },
    { 0, 0 },
    { 0x6ddcb7e3, 90 },
    { 0, 0 },
    { 0xbc8cb7e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2cb7e9, 23 },
    { 0x9e0cb7ea, 33 },
    { 0x27acb7eb, 33 },
    { 0xfaecb7ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x675cb7ef, 61 },
    { 0x859cb7ef, 96 },
    { 0xb9cb7f1, 55 },
    { 0, 0 },
    { 0x2f3cb7f3, 95 },
    { 0x839cb7f3, 76 },
    { 0x24ccb7f5, 33 },
    { 0x55cb7f3, 95 },
    { 0, 0 },
    { 0x817cb7f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7cb7fb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649cb7ff, 96 },
    { 0, 0 },
    { 0x235cb801, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70cb804, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7cb80b, 52 },
    { 0xddfcb80c, 61 },
    { 0x24dcb80d, 46 },
    { 0, 0 },
    { 0x3a1cb80f, 90 },
    { 0xa0bcb810, 33 },
    { 0x970cb811, 100 },
    { 0xd72cb812, 98 },
    { 0xacb813, 88 },
    { 0x1eacb812, 90 },
    { 0x5cccb813, 34 },
    { 0x4ebcb816, 90 },
    { 0xf9bcb817, 85 },
    { 0xde7cb814, 95 },
    { 0x42bcb819, 33 },
    { 0xca2cb81a, 76 },
    { 0x419cb81b, 34 },
    { 0xdfccb819, 25 },
    { 0xa2ccb81d, 55 },
    { 0x1fecb81d, 61 },
    { 0x5b5cb81e, 43 },
    { 0x902cb820, 23 },
    { 0xf38cb81c, 90 },
    { 0x33fcb81e, 98 },
    { 0xe67cb823, 23 },
    { 0, 0 },
    { 0xfa1cb825, 61 },
    { 0x885cb826, 98 },
    { 0, 0 },
    { 0x461cb828, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08cb831, 95 },
    { 0xb71cb832, 81 },
    { 0xb9dcb833, 43 },
    { 0x888cb832, 46 },
    { 0x174cb835, 33 },
    { 0, 0 },
    { 0x9accb837, 95 },
    { 0xc02cb837, 23 },
    { 0, 0 },
    { 0xa47cb83a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81cb840, 90 },
    { 0x5b3cb840, 60 },
    { 0xbd6cb842, 95 },
    { 0xc87cb840, 23 },
    { 0xfe4cb843, 90 },
    { 0, 0 },
    { 0xf0ccb846, 88 },
    { 0, 0 },
    { 0xe4ccb848, 90 },
    { 0xc1acb849, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0cb84e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddcb856, 98 },
    { 0, 0 },
    { 0x28ecb858, 48 },
    { 0xac7cb859, 90 },
    { 0x863cb85a, 33 },
    { 0, 0 },
    { 0x65cb85c, 98 },
    { 0x3d2cb85d, 61 },
    { 0x682cb85c, 61 },
    { 0x9d4cb85f, 90 },
    { 0x162cb860, 81 },
    { 0xd0fcb861, 93 },
    { 0xf6acb862, 100 },
    { 0x937cb863, 33 },
    { 0x7c0cb860, 90 },
    { 0, 0 },
    { 0xacecb866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fcb86d, 95 },
    { 0, 0 },
    { 0x90bcb86f, 90 },
    { 0x542cb870, 83 },
    { 0xfb3cb871, 85 },
    { 0xdd6cb872, 34 },
    { 0xf86cb871, 84 },
    { 0, 0 },
    { 0xe4bcb875, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8cb879, 66 },
    { 0, 0 },
    { 0xcefcb87b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cb880, 23 },
    { 0x668cb881, 95 },
    { 0x8bacb881, 55 },
    { 0x62acb883, 90 },
    { 0xbe7cb881, 55 },
    { 0xc73cb882, 43 },
    { 0, 0 },
    { 0x1ebcb887, 66 },
    { 0, 0 },
    { 0xa6ecb889, 33 },
    { 0x418cb88a, 90 },
    { 0xab8cb889, 89 },
    { 0, 0 },
    { 0x92ccb88d, 23 },
    { 0x343cb88e, 61 },
    { 0, 0 },
    { 0xfe9cb890, 46 },
    { 0xaa3cb890, 53 },
    { 0xfdbcb892, 98 },
    { 0, 0 },
    { 0x875cb894, 95 },
    { 0xcd4cb895, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dcb898, 88 },
    { 0xd56cb899, 85 },
    { 0x951cb89a, 99 },
    { 0xadecb89b, 46 },
    { 0x7bccb89a, 87 },
    { 0xe1acb89c, 34 },
    { 0x140cb899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7cb8a2, 64 },
    { 0, 0 },
    { 0x88ecb8a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccb8a7, 33 },
    { 0, 0 },
    { 0x161cb8a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fcb8ad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1cb8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x773cb8b5, 52 },
    { 0xc43cb8b5, 83 },
    { 0x2cacb8b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463cb8c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ecb8c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbcb8cc, 33 },
    { 0, 0 },
    { 0x3d5cb8ce, 95 },
    { 0xccacb8cf, 90 },
    { 0, 0 },
    { 0xf2acb8d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6cb8d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731cb8dd, 23 },
    { 0x6b7cb8de, 76 },
    { 0xd0fcb8df, 95 },
    { 0xdc9cb8e0, 61 },
    { 0x48acb8e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12cb8eb, 76 },
    { 0xc6bcb8ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47cb8f1, 67 },
    { 0xb33cb8f1, 34 },
    { 0x2f1cb8f3, 43 },
    { 0, 0 },
    { 0xe0acb8f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bcb8f9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941cb8fe, 93 },
    { 0x433cb8ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaacb908, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dcb90b, 59 },
    { 0, 0 },
    { 0x733cb90d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x873cb910, 34 },
    { 0, 0 },
    { 0x8ccb912, 84 },
    { 0x605cb913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236cb919, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fcb921, 80 },
    { 0x78ecb922, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x962cb925, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bcb928, 23 },
    { 0x471cb929, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fcb92e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ecb932, 43 },
    { 0x9a7cb933, 61 },
    { 0x45ccb934, 61 },
    { 0x47ecb932, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ccb93a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdcb93d, 95 },
    { 0, 0 },
    { 0xc80cb93f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520cb943, 81 },
    { 0x41acb944, 99 },
    { 0xa01cb945, 33 },
    { 0x1afcb946, 33 },
    { 0x2b1cb944, 61 },
    { 0x6b4cb946, 61 },
    { 0x33acb949, 43 },
    { 0x7cecb945, 84 },
    { 0xeaecb944, 23 },
    { 0xd7dcb947, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dcb94f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cccb952, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ecb958, 95 },
    { 0, 0 },
    { 0x29ccb95a, 23 },
    { 0x184cb95b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacb966, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241cb96c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cb970, 75 },
    { 0xf62cb971, 95 },
    { 0x3d6cb971, 23 },
    { 0xc7ecb970, 98 },
    { 0, 0 },
    { 0x34ccb975, 90 },
    { 0x163cb975, 33 },
    { 0x9c7cb977, 65 },
    { 0xbbcb978, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adcb97b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ecb97e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x787cb981, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06cb989, 98 },
    { 0x117cb98a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3cb990, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8cb993, 91 },
    { 0xa9dcb994, 23 },
    { 0x46dcb995, 66 },
    { 0xfabcb993, 90 },
    { 0x8e9cb997, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6cb99a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fcb99e, 90 },
    { 0x16bcb99f, 98 },
    { 0x54fcb9a0, 90 },
    { 0xeb3cb99f, 76 },
    { 0x86bcb99e, 61 },
    { 0x5cccb99e, 23 },
    { 0xfe1cb9a4, 43 },
    { 0x346cb99e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fcb9ab, 33 },
    { 0, 0 },
    { 0x804cb9ad, 99 },
    { 0x877cb9ae, 61 },
    { 0x665cb9af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x245cb9b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2cb9b7, 100 },
    { 0x3becb9b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x116cb9bb, 77 },
    { 0, 0 },
    { 0x15bcb9bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484cb9c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767cb9cf, 23 },
    { 0x5d2cb9cf, 98 },
    { 0, 0 },
    { 0xe92cb9d2, 33 },
    { 0xdc2cb9d3, 33 },
    { 0x9a3cb9d4, 34 },
    { 0x202cb9d5, 98 },
    { 0x13ecb9d5, 61 },
    { 0xe3fcb9d7, 93 },
    { 0x41ccb9d8, 95 },
    { 0x2eacb9d9, 75 },
    { 0xb12cb9d3, 55 },
    { 0x446cb9db, 33 },
    { 0xeb6cb9dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6cb9df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68cb9e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942cb9e9, 90 },
    { 0, 0 },
    { 0xf8cb9eb, 66 },
    { 0, 0 },
    { 0x7afcb9ed, 56 },
    { 0xefcb9ee, 98 },
    { 0, 0 },
    { 0x49bcb9f0, 98 },
    { 0xa87cb9f1, 95 },
    { 0x811cb9f2, 90 },
    { 0xe2ccb9f1, 61 },
    { 0x99bcb9f4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8cb9f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4cb9fc, 33 },
    { 0x2cccb9fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x272cba00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5cba04, 23 },
    { 0, 0 },
    { 0x13fcba06, 61 },
    { 0xa02cba06, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bcba0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cba0f, 90 },
    { 0xd90cba0f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598cba15, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ecba19, 46 },
    { 0x4afcba1a, 90 },
    { 0, 0 },
    { 0xe0bcba1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcba20, 55 },
    { 0x1c8cba21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0cba26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fcba29, 98 },
    { 0x88ccba2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9cba2d, 24 },
    { 0xe4dcba2e, 95 },
    { 0x969cba2f, 23 },
    { 0x35cba2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fcba3a, 90 },
    { 0xa87cba3b, 100 },
    { 0xf38cba3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6becba3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15cba42, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83cba45, 100 },
    { 0, 0 },
    { 0xa72cba47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63dcba4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ecba4e, 23 },
    { 0, 0 },
    { 0xb54cba50, 33 },
    { 0xa2cba51, 66 },
    { 0x879cba52, 99 },
    { 0, 0 },
    { 0x20ecba54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14cba57, 33 },
    { 0, 0 },
    { 0xeb2cba59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cba61, 43 },
    { 0xa83cba62, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20cba66, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x235cba69, 46 },
    { 0, 0 },
    { 0x982cba6b, 33 },
    { 0x86bcba6c, 17 },
    { 0xddecba6d, 61 },
    { 0, 0 },
    { 0x5d9cba6f, 95 },
    { 0xa9ecba70, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ccba74, 23 },
    { 0xa8bcba74, 99 },
    { 0, 0 },
    { 0x58ecba77, 23 },
    { 0xc38cba77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fcba7e, 46 },
    { 0xe0dcba7e, 33 },
    { 0x1f1cba7e, 98 },
    { 0x15ecba81, 93 },
    { 0xc0ecba7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1cba85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7cba88, 33 },
    { 0xb06cba88, 95 },
    { 0xaffcba8a, 43 },
    { 0x73fcba8b, 61 },
    { 0x14ccba88, 69 },
    { 0xa19cba89, 17 },
    { 0x876cba8b, 33 },
    { 0xe6fcba8e, 95 },
    { 0x608cba88, 95 },
    { 0xde3cba8d, 93 },
    { 0xcdccba92, 95 },
    { 0xa37cba8a, 98 },
    { 0x72acba8f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341cba9d, 95 },
    { 0, 0 },
    { 0xb06cba9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccbaa3, 46 },
    { 0x709cbaa3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736cbaa8, 95 },
    { 0xee2cbaa9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bcbaad, 33 },
    { 0, 0 },
    { 0xc16cbaaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fccbab9, 34 },
    { 0, 0 },
    { 0xac9cbabb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638cbac1, 55 },
    { 0, 0 },
    { 0x415cbac3, 90 },
    { 0xe73cbac3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x910cbac7, 93 },
    { 0, 0 },
    { 0xb72cbac9, 99 },
    { 0xb7ccbac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ccbad0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbccbad5, 98 },
    { 0x546cbad6, 98 },
    { 0x9d8cbad7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dcbadc, 38 },
    { 0x681cbadd, 34 },
    { 0x39ecbade, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4decbae2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fcbae5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7cbaea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fcbaed, 98 },
    { 0, 0 },
    { 0x8bdcbaef, 33 },
    { 0x6b4cbaef, 90 },
    { 0x6ddcbaf0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cbaf6, 61 },
    { 0, 0 },
    { 0xb9ccbaf8, 46 },
    { 0x855cbaf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470cbb01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9cbb0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x915cbb12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x959cbb15, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8cbb18, 33 },
    { 0x840cbb19, 75 },
    { 0, 0 },
    { 0xbd9cbb1b, 23 },
    { 0x68bcbb1b, 85 },
    { 0xebccbb1d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594cbb24, 93 },
    { 0x714cbb25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60cbb28, 40 },
    { 0xd4acbb29, 95 },
    { 0xf8ecbb29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956cbb2f, 88 },
    { 0x513cbb2f, 76 },
    { 0xfa2cbb31, 98 },
    { 0x25bcbb31, 46 },
    { 0x5e8cbb33, 87 },
    { 0x52fcbb31, 76 },
    { 0x7c9cbb33, 95 },
    { 0xe2dcbb2f, 76 },
    { 0xb7ccbb37, 90 },
    { 0, 0 },
    { 0x5c9cbb39, 78 },
    { 0x74ccbb3a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ecbb3f, 61 },
    { 0x6cbcbb3f, 90 },
    { 0xd4cbb41, 93 },
    { 0, 0 },
    { 0x2bcbb43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4cbb46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16cbb4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41cbb4f, 90 },
    { 0x502cbb50, 23 },
    { 0xd02cbb51, 52 },
    { 0xf2fcbb52, 87 },
    { 0xb74cbb53, 55 },
    { 0xe2bcbb52, 55 },
    { 0x3d4cbb52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x195cbb58, 69 },
    { 0x199cbb58, 40 },
    { 0x298cbb5a, 33 },
    { 0xae6cbb58, 95 },
    { 0x228cbb5c, 76 },
    { 0x9dcbb5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eccbb64, 93 },
    { 0x6aacbb64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52acbb6c, 90 },
    { 0x6c1cbb6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dcbb73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dcbb79, 33 },
    { 0x614cbb7a, 17 },
    { 0, 0 },
    { 0xc21cbb7c, 93 },
    { 0xcdbcbb7c, 67 },
    { 0x734cbb7e, 23 },
    { 0x723cbb7f, 90 },
    { 0xec0cbb7f, 55 },
    { 0xf43cbb7d, 90 },
    { 0xe5ccbb82, 23 },
    { 0xa0ccbb80, 99 },
    { 0xb1ecbb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dcbb8b, 90 },
    { 0, 0 },
    { 0x84fcbb8d, 34 },
    { 0x728cbb8e, 24 },
    { 0, 0 },
    { 0x23fcbb90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbcbb9a, 23 },
    { 0, 0 },
    { 0xfb8cbb9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0cbba1, 95 },
    { 0xfdcbba2, 23 },
    { 0xd16cbba2, 34 },
    { 0xea0cbba4, 61 },
    { 0x709cbba5, 55 },
    { 0xe40cbba3, 95 },
    { 0xfe6cbba6, 84 },
    { 0x6b4cbba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06cbbb0, 95 },
    { 0xfe4cbbb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2cbbb5, 84 },
    { 0x211cbbb5, 33 },
    { 0, 0 },
    { 0xb23cbbb8, 55 },
    { 0x8f6cbbb8, 34 },
    { 0x632cbbba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88cbbc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1cbbc8, 23 },
    { 0, 0 },
    { 0xb67cbbca, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aacbbcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5cbbd2, 61 },
    { 0x2e8cbbd3, 61 },
    { 0x3b0cbbd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ecbbd7, 90 },
    { 0xa2cbbd8, 33 },
    { 0, 0 },
    { 0xf5fcbbda, 95 },
    { 0x7fbcbbda, 98 },
    { 0xad3cbbda, 98 },
    { 0x18ecbbdd, 85 },
    { 0x897cbbdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbbe9, 55 },
    { 0, 0 },
    { 0x87cbbeb, 90 },
    { 0x50acbbec, 97 },
    { 0x1bacbbed, 90 },
    { 0, 0 },
    { 0xec7cbbef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bcbbf2, 95 },
    { 0x6b2cbbf2, 90 },
    { 0, 0 },
    { 0xf84cbbf5, 90 },
    { 0xf38cbbf6, 38 },
    { 0xd21cbbf7, 68 },
    { 0xb5cbbf8, 61 },
    { 0x1fcbbf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfccbc08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4cbc0b, 61 },
    { 0x82fcbc0c, 23 },
    { 0, 0 },
    { 0x738cbc0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ccbc11, 33 },
    { 0x15fcbc12, 93 },
    { 0, 0 },
    { 0x976cbc14, 99 },
    { 0x72ecbc14, 33 },
    { 0xf89cbc16, 34 },
    { 0x3b1cbc15, 90 },
    { 0, 0 },
    { 0x848cbc19, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8cbc1c, 92 },
    { 0xf45cbc1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39cbc20, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458cbc26, 52 },
    { 0xaccbc27, 81 },
    { 0x9dfcbc28, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561cbc2d, 98 },
    { 0x9cbc2e, 84 },
    { 0x4bbcbc2e, 43 },
    { 0, 0 },
    { 0xd0cbc31, 77 },
    { 0x575cbc31, 90 },
    { 0, 0 },
    { 0x9fccbc34, 61 },
    { 0x47dcbc34, 90 },
    { 0x125cbc36, 91 },
    { 0x9a4cbc37, 43 },
    { 0, 0 },
    { 0x603cbc39, 98 },
    { 0x2eccbc3a, 61 },
    { 0xfeecbc3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1cbc42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bcbc46, 43 },
    { 0xffdcbc46, 90 },
    { 0xa0ecbc46, 61 },
    { 0, 0 },
    { 0xef0cbc4a, 90 },
    { 0x9a0cbc4b, 99 },
    { 0x395cbc4b, 99 },
    { 0x4f8cbc4d, 33 },
    { 0x826cbc4d, 34 },
    { 0xd20cbc4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bcbc55, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4cbc59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554cbc5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0cbc64, 33 },
    { 0, 0 },
    { 0xa6ecbc66, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05cbc69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fcbc6d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66cbc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5cbc75, 33 },
    { 0x96ecbc76, 95 },
    { 0x80fcbc76, 76 },
    { 0x7d4cbc78, 85 },
    { 0x45acbc75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772cbc7d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95cbc81, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ecbc88, 95 },
    { 0, 0 },
    { 0x420cbc8a, 100 },
    { 0x15cbc8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x771cbc8e, 33 },
    { 0, 0 },
    { 0xbf7cbc90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bcbc93, 23 },
    { 0, 0 },
    { 0x14fcbc95, 95 },
    { 0xfc8cbc96, 81 },
    { 0xf89cbc97, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x813cbc9a, 61 },
    { 0, 0 },
    { 0xf01cbc9c, 33 },
    { 0, 0 },
    { 0xa7dcbc9e, 93 },
    { 0x32fcbc9e, 61 },
    { 0xddcbc9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67cbca3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dcbca9, 67 },
    { 0, 0 },
    { 0xeb4cbcab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ccbcb1, 95 },
    { 0x7a3cbcb2, 23 },
    { 0x2b6cbcb3, 98 },
    { 0x614cbcb4, 33 },
    { 0, 0 },
    { 0xda1cbcb6, 73 },
    { 0x20dcbcb6, 90 },
    { 0x959cbcb8, 98 },
    { 0xb34cbcb9, 23 },
    { 0x393cbcb8, 33 },
    { 0, 0 },
    { 0x2e5cbcbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18cbcc0, 85 },
    { 0, 0 },
    { 0x8aacbcc2, 61 },
    { 0x75dcbcc3, 53 },
    { 0xacacbcc4, 33 },
    { 0xc30cbcc5, 77 },
    { 0x9afcbcc5, 61 },
    { 0x54dcbcc4, 90 },
    { 0x922cbcc6, 52 },
    { 0x5dbcbcc2, 55 },
    { 0, 0 },
    { 0xa56cbccb, 87 },
    { 0x3edcbccc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447cbcd3, 40 },
    { 0, 0 },
    { 0xe99cbcd5, 61 },
    { 0xaeecbcd5, 98 },
    { 0, 0 },
    { 0xa51cbcd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ecbcde, 95 },
    { 0x94cbcdf, 95 },
    { 0, 0 },
    { 0x135cbce1, 61 },
    { 0xc80cbce2, 77 },
    { 0x745cbce3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cbcee, 90 },
    { 0x6dacbcef, 60 },
    { 0x42ccbcee, 99 },
    { 0, 0 },
    { 0xd66cbcf2, 61 },
    { 0x7a8cbcf3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6cbcfa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90cbcfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcbd03, 98 },
    { 0, 0 },
    { 0x353cbd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8cbd0c, 67 },
    { 0x8f7cbd0d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3cbd13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9cbd17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfcbd1f, 90 },
    { 0x282cbd20, 87 },
    { 0xe48cbd20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fccbd24, 76 },
    { 0x80ccbd25, 34 },
    { 0xeacbd26, 98 },
    { 0xba3cbd27, 87 },
    { 0, 0 },
    { 0xc36cbd29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ecbd2c, 34 },
    { 0x53cbd2c, 90 },
    { 0x993cbd2e, 43 },
    { 0x423cbd2d, 55 },
    { 0xf8ecbd2f, 95 },
    { 0xb16cbd30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdcbd36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266cbd3d, 90 },
    { 0xc0ecbd3e, 33 },
    { 0xd4cbd3d, 23 },
    { 0x136cbd40, 61 },
    { 0x63cbd41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x749cbd44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8facbd49, 93 },
    { 0xadccbd4a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54cbd4e, 33 },
    { 0x682cbd4f, 91 },
    { 0xbdecbd50, 33 },
    { 0x767cbd51, 76 },
    { 0xe54cbd4f, 61 },
    { 0xb45cbd53, 76 },
    { 0x9ecbd54, 23 },
    { 0xa81cbd55, 61 },
    { 0xf6bcbd55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7cbd5a, 87 },
    { 0x894cbd5b, 33 },
    { 0xd48cbd5c, 99 },
    { 0x689cbd5d, 33 },
    { 0x3bbcbd5a, 98 },
    { 0xfd8cbd5c, 48 },
    { 0x5dacbd60, 33 },
    { 0x356cbd61, 34 },
    { 0x934cbd62, 90 },
    { 0x10ccbd60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dcbd66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adcbd72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1cbd75, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9cbd7b, 90 },
    { 0, 0 },
    { 0x747cbd7d, 52 },
    { 0x5c0cbd7e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53cbd81, 95 },
    { 0, 0 },
    { 0x278cbd83, 52 },
    { 0, 0 },
    { 0x141cbd85, 61 },
    { 0, 0 },
    { 0x4bfcbd87, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07cbd8a, 46 },
    { 0x1d7cbd8b, 90 },
    { 0x51acbd8c, 95 },
    { 0x43ecbd8d, 76 },
    { 0x588cbd8d, 90 },
    { 0x72acbd8a, 84 },
    { 0xd48cbd90, 68 },
    { 0, 0 },
    { 0xe16cbd92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdcbd95, 90 },
    { 0x417cbd95, 95 },
    { 0x88ecbd96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bcbd9d, 85 },
    { 0xd14cbd9d, 76 },
    { 0xd28cbd9d, 90 },
    { 0x1acbd9f, 88 },
    { 0, 0 },
    { 0xdc2cbda2, 23 },
    { 0xfaecbda2, 33 },
    { 0x51cbda3, 91 },
    { 0, 0 },
    { 0x4dbcbda6, 97 },
    { 0x705cbda7, 55 },
    { 0x372cbda8, 91 },
    { 0x8a3cbda8, 33 },
    { 0x280cbda9, 85 },
    { 0, 0 },
    { 0x6a1cbdac, 61 },
    { 0xac1cbdad, 100 },
    { 0x92fcbdac, 33 },
    { 0x919cbdad, 98 },
    { 0x5b0cbdb0, 90 },
    { 0x1e4cbdb1, 66 },
    { 0, 0 },
    { 0xd2fcbdb3, 98 },
    { 0x159cbdb3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fcbdbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bacbdc5, 23 },
    { 0x257cbdc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x892cbdc9, 33 },
    { 0x4efcbdca, 95 },
    { 0xa59cbdcb, 61 },
    { 0, 0 },
    { 0x592cbdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98acbdd1, 95 },
    { 0xc38cbdd1, 61 },
    { 0xadbcbdd3, 98 },
    { 0x887cbdd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470cbdda, 61 },
    { 0x4c6cbdda, 76 },
    { 0xf02cbddc, 61 },
    { 0x74ccbdda, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cbde3, 87 },
    { 0x99cbde4, 23 },
    { 0x49bcbde3, 55 },
    { 0xa97cbde6, 34 },
    { 0xa4dcbde6, 84 },
    { 0xca6cbde8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06cbdef, 90 },
    { 0x304cbdef, 43 },
    { 0, 0 },
    { 0x7a6cbdf2, 90 },
    { 0xf13cbdf2, 33 },
    { 0, 0 },
    { 0x72fcbdf5, 61 },
    { 0, 0 },
    { 0x1a6cbdf7, 33 },
    { 0x9e8cbdf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3cbdfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cbdfe, 33 },
    { 0, 0 },
    { 0x2aecbe00, 55 },
    { 0, 0 },
    { 0xffcbe02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7cbe05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50cbe0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x619cbe0e, 46 },
    { 0x9fbcbe0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397cbe14, 34 },
    { 0x44fcbe15, 90 },
    { 0x4d6cbe15, 90 },
    { 0, 0 },
    { 0x459cbe18, 90 },
    { 0, 0 },
    { 0x99fcbe1a, 87 },
    { 0xe47cbe1a, 33 },
    { 0x9a2cbe1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bcbe20, 33 },
    { 0x7c3cbe21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cbe24, 66 },
    { 0x52bcbe24, 76 },
    { 0x4d1cbe26, 61 },
    { 0x55ccbe27, 55 },
    { 0, 0 },
    { 0xb12cbe29, 98 },
    { 0, 0 },
    { 0x685cbe2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621cbe30, 69 },
    { 0x367cbe31, 55 },
    { 0, 0 },
    { 0xe21cbe33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324cbe39, 23 },
    { 0, 0 },
    { 0x84acbe3b, 33 },
    { 0xa10cbe3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3cbe43, 100 },
    { 0xf2fcbe44, 23 },
    { 0, 0 },
    { 0x33dcbe46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25cbe4a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeecbe4d, 55 },
    { 0xb9acbe4e, 98 },
    { 0x2d9cbe4f, 95 },
    { 0x650cbe4d, 90 },
    { 0x29bcbe51, 46 },
    { 0xf7ccbe52, 98 },
    { 0xd92cbe52, 76 },
    { 0xe34cbe54, 55 },
    { 0x7c4cbe53, 87 },
    { 0x677cbe56, 93 },
    { 0x7f1cbe57, 76 },
    { 0xf78cbe57, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x371cbe5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757cbe62, 22 },
    { 0, 0 },
    { 0xba0cbe64, 43 },
    { 0x209cbe64, 78 },
    { 0x632cbe66, 98 },
    { 0, 0 },
    { 0xcecbe68, 23 },
    { 0xa7dcbe69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0cbe6d, 43 },
    { 0x429cbe6d, 98 },
    { 0x8e3cbe6f, 43 },
    { 0x696cbe6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dcbe79, 85 },
    { 0, 0 },
    { 0xb09cbe7b, 98 },
    { 0, 0 },
    { 0xe77cbe7d, 61 },
    { 0xdf2cbe7d, 78 },
    { 0xbe3cbe7f, 90 },
    { 0xec4cbe80, 61 },
    { 0x2a9cbe80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8cbe85, 99 },
    { 0xd1dcbe86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ccbe8c, 33 },
    { 0xc59cbe8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cbe94, 34 },
    { 0x2cecbe94, 34 },
    { 0xc21cbe96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dcbe9a, 55 },
    { 0x8bcbe9b, 95 },
    { 0x644cbe9c, 95 },
    { 0xfcdcbe9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cbea4, 99 },
    { 0x4cbea5, 61 },
    { 0x7accbea4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcccbea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5cbeae, 90 },
    { 0x554cbeaf, 87 },
    { 0xbf9cbeaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4cbeb4, 55 },
    { 0x231cbeb5, 88 },
    { 0x8d7cbeb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438cbebd, 98 },
    { 0, 0 },
    { 0xececbebf, 33 },
    { 0xe01cbebf, 93 },
    { 0xb59cbec1, 76 },
    { 0x363cbec2, 61 },
    { 0, 0 },
    { 0xdaccbec4, 98 },
    { 0, 0 },
    { 0xbfcbec6, 90 },
    { 0xb1ecbec6, 33 },
    { 0xa42cbec6, 84 },
    { 0x811cbec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65cbecc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20cbed0, 34 },
    { 0x3bacbed1, 63 },
    { 0, 0 },
    { 0xdf3cbed3, 23 },
    { 0x210cbed4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223cbed8, 90 },
    { 0xde6cbed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbcbedd, 84 },
    { 0, 0 },
    { 0x3b7cbedf, 55 },
    { 0x488cbedf, 58 },
    { 0xf9cbee1, 33 },
    { 0xe44cbee1, 55 },
    { 0x4c4cbee3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50cbee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dcbeec, 98 },
    { 0x8e4cbeed, 87 },
    { 0xe75cbeed, 95 },
    { 0x3c8cbeef, 95 },
    { 0, 0 },
    { 0xda4cbef1, 90 },
    { 0, 0 },
    { 0x734cbef3, 22 },
    { 0x68dcbef4, 23 },
    { 0x1f2cbef5, 23 },
    { 0, 0 },
    { 0x200cbef7, 43 },
    { 0xbd9cbef8, 23 },
    { 0x3bcbef8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0cbefc, 85 },
    { 0x6c4cbefd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2becbf00, 98 },
    { 0, 0 },
    { 0xa17cbf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08cbf06, 97 },
    { 0x10acbf07, 23 },
    { 0, 0 },
    { 0xd5ecbf09, 99 },
    { 0x867cbf0a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2cbf0e, 52 },
    { 0, 0 },
    { 0x3a5cbf10, 66 },
    { 0xf3ecbf11, 90 },
    { 0, 0 },
    { 0x2bbcbf13, 67 },
    { 0x3f8cbf13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eacbf17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11cbf1d, 90 },
    { 0xac2cbf1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833cbf26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5cbf31, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dcbf39, 98 },
    { 0, 0 },
    { 0xcc4cbf3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2cbf3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40cbf4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3cbf52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606cbf57, 81 },
    { 0x6aecbf57, 95 },
    { 0, 0 },
    { 0x8f3cbf5a, 46 },
    { 0x9a5cbf5a, 43 },
    { 0x2e6cbf5c, 33 },
    { 0x4cecbf5d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcbf63, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59cbf67, 98 },
    { 0, 0 },
    { 0x45bcbf69, 95 },
    { 0x5b9cbf6a, 33 },
    { 0xae4cbf6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5cbf75, 81 },
    { 0xce7cbf75, 33 },
    { 0xd20cbf77, 23 },
    { 0x60ccbf78, 90 },
    { 0x871cbf78, 55 },
    { 0x2accbf79, 66 },
    { 0, 0 },
    { 0x924cbf7c, 76 },
    { 0x9c0cbf7d, 95 },
    { 0, 0 },
    { 0x763cbf7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346cbf86, 90 },
    { 0x96acbf86, 23 },
    { 0x4bacbf88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1cbf8b, 34 },
    { 0, 0 },
    { 0xbbdcbf8d, 92 },
    { 0x108cbf8e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36cbf92, 93 },
    { 0xadcbf93, 55 },
    { 0x4d8cbf94, 92 },
    { 0xf2ccbf95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17acbf98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fcbf9b, 90 },
    { 0, 0 },
    { 0x52cbf9d, 34 },
    { 0x60acbf9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4cbfa4, 99 },
    { 0x2aacbfa5, 98 },
    { 0x9eecbfa6, 90 },
    { 0, 0 },
    { 0x4bccbfa8, 93 },
    { 0, 0 },
    { 0xc3fcbfaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cbfad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5cbfb7, 33 },
    { 0x677cbfb8, 80 },
    { 0, 0 },
    { 0x28fcbfba, 97 },
    { 0x1b2cbfba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ccbfbe, 90 },
    { 0, 0 },
    { 0xf6ccbfc0, 61 },
    { 0x1dacbfc1, 55 },
    { 0xb80cbfc2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577cbfc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ecbfcc, 34 },
    { 0x10bcbfcc, 98 },
    { 0, 0 },
    { 0x86dcbfcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8cbfd3, 23 },
    { 0, 0 },
    { 0x700cbfd5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cbfdb, 98 },
    { 0, 0 },
    { 0x6e6cbfdd, 52 },
    { 0, 0 },
    { 0x3b8cbfdf, 95 },
    { 0, 0 },
    { 0x90fcbfe1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cbfe9, 43 },
    { 0, 0 },
    { 0x27fcbfeb, 38 },
    { 0, 0 },
    { 0xcf5cbfed, 61 },
    { 0x58acbfee, 90 },
    { 0x7accbfee, 91 },
    { 0xf1fcbfee, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ccbff7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86cbffa, 61 },
    { 0, 0 },
    { 0xf7cbffc, 90 },
    { 0, 0 },
    { 0x102cbffe, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44acc002, 90 },
    { 0x71ecc002, 99 },
    { 0xb7bcc002, 95 },
    { 0x30dcc005, 98 },
    { 0x23bcc006, 40 },
    { 0x6d9cc005, 33 },
    { 0xafecc005, 55 },
    { 0x97ccc009, 90 },
    { 0x5b7cc00a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233cc010, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3cc015, 61 },
    { 0, 0 },
    { 0x40ccc017, 23 },
    { 0x7e7cc017, 34 },
    { 0, 0 },
    { 0x3cc01a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519cc022, 90 },
    { 0xf24cc023, 91 },
    { 0xcb6cc023, 34 },
    { 0x7e6cc025, 23 },
    { 0x633cc025, 85 },
    { 0x372cc027, 77 },
    { 0xa9ccc028, 90 },
    { 0, 0 },
    { 0xf4dcc02a, 78 },
    { 0xac6cc02b, 33 },
    { 0, 0 },
    { 0x87ecc02d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954cc031, 76 },
    { 0x98ecc031, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbcc036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cacc03c, 95 },
    { 0xaaccc03d, 91 },
    { 0x2abcc03e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597cc043, 61 },
    { 0x1a6cc044, 95 },
    { 0x1b4cc044, 99 },
    { 0x933cc046, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfcc051, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x766cc054, 33 },
    { 0x2b8cc055, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fcc059, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cc05c, 61 },
    { 0, 0 },
    { 0x2cdcc05e, 33 },
    { 0xe0fcc05f, 81 },
    { 0x7cbcc05e, 75 },
    { 0, 0 },
    { 0x3c7cc062, 91 },
    { 0x8cfcc063, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x290cc066, 95 },
    { 0x7a3cc067, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8cc06e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaecc074, 78 },
    { 0x864cc074, 95 },
    { 0xb59cc075, 34 },
    { 0x950cc075, 17 },
    { 0x2afcc075, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x180cc07e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49cc084, 93 },
    { 0xf74cc085, 85 },
    { 0x7cbcc085, 99 },
    { 0xed2cc086, 98 },
    { 0xf5bcc088, 17 },
    { 0x445cc089, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7cc08f, 95 },
    { 0xb0bcc090, 43 },
    { 0x50fcc091, 91 },
    { 0, 0 },
    { 0xea5cc093, 98 },
    { 0x6b5cc094, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20cc098, 46 },
    { 0, 0 },
    { 0x7d8cc09a, 78 },
    { 0x40dcc09b, 53 },
    { 0x5facc09b, 100 },
    { 0, 0 },
    { 0xe05cc09e, 98 },
    { 0x41acc09f, 55 },
    { 0x22cc0a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dcc0a3, 33 },
    { 0x812cc0a4, 90 },
    { 0x175cc0a5, 61 },
    { 0x109cc0a5, 61 },
    { 0xd78cc0a5, 55 },
    { 0xb45cc0a8, 23 },
    { 0x9eccc0a4, 87 },
    { 0xc78cc0aa, 33 },
    { 0xb52cc0aa, 95 },
    { 0xbafcc0ac, 91 },
    { 0xd25cc0ad, 98 },
    { 0x5e8cc0aa, 98 },
    { 0x2bdcc0af, 98 },
    { 0xd97cc0af, 87 },
    { 0xbc8cc0a6, 76 },
    { 0, 0 },
    { 0xf49cc0b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fcc0b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ecc0ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbcc0bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eecc0c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7cc0cf, 95 },
    { 0xc74cc0d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dcc0d7, 90 },
    { 0x89ccc0d8, 66 },
    { 0x72dcc0d9, 61 },
    { 0x15fcc0da, 52 },
    { 0x2efcc0db, 56 },
    { 0x5b3cc0d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbcc0e4, 93 },
    { 0xdc8cc0e5, 69 },
    { 0xa00cc0e6, 76 },
    { 0xefecc0e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fcc0ee, 99 },
    { 0x76ecc0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdcc0f2, 55 },
    { 0xb0ecc0f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4cc0f8, 98 },
    { 0xf61cc0f9, 91 },
    { 0xf5fcc0fa, 23 },
    { 0x104cc0fb, 78 },
    { 0xc06cc0fc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cc106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247cc10d, 90 },
    { 0x27cc10e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bcc111, 33 },
    { 0xac9cc112, 90 },
    { 0x877cc113, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5cc117, 9 },
    { 0xeccc117, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67cc11d, 23 },
    { 0, 0 },
    { 0x308cc11f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdcc124, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4cc127, 33 },
    { 0x9fccc127, 61 },
    { 0x69acc128, 23 },
    { 0xbcbcc129, 61 },
    { 0x3dacc129, 61 },
    { 0x159cc12c, 33 },
    { 0, 0 },
    { 0x609cc12e, 98 },
    { 0xfb9cc12e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382cc135, 95 },
    { 0, 0 },
    { 0x8c2cc137, 65 },
    { 0, 0 },
    { 0x3a1cc139, 76 },
    { 0x192cc139, 23 },
    { 0x290cc139, 90 },
    { 0, 0 },
    { 0xc74cc13d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70cc142, 90 },
    { 0, 0 },
    { 0xb98cc144, 34 },
    { 0x8f9cc145, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9cc148, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494cc14e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61cc155, 23 },
    { 0xf7ccc155, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dcc159, 46 },
    { 0x7efcc15a, 81 },
    { 0x822cc15a, 95 },
    { 0x753cc15a, 23 },
    { 0xc53cc15b, 61 },
    { 0, 0 },
    { 0x8d4cc15f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x663cc162, 68 },
    { 0xe92cc163, 23 },
    { 0, 0 },
    { 0xd75cc165, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edcc168, 76 },
    { 0x4ffcc169, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2cc16e, 66 },
    { 0, 0 },
    { 0x73bcc170, 90 },
    { 0x699cc171, 33 },
    { 0x8cccc172, 61 },
    { 0x21fcc173, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eecc179, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6cc17d, 23 },
    { 0, 0 },
    { 0xaa0cc17f, 23 },
    { 0x8fbcc17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x117cc183, 98 },
    { 0x98ecc184, 91 },
    { 0, 0 },
    { 0xa04cc186, 91 },
    { 0x9a8cc187, 100 },
    { 0x5f7cc188, 98 },
    { 0, 0 },
    { 0x301cc18a, 66 },
    { 0xb5cc18b, 76 },
    { 0xec7cc18c, 91 },
    { 0xcb0cc18c, 98 },
    { 0x697cc18e, 95 },
    { 0xdebcc18f, 90 },
    { 0x5b2cc18b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35cc198, 95 },
    { 0, 0 },
    { 0x2bccc19a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcc19d, 98 },
    { 0, 0 },
    { 0x801cc19f, 61 },
    { 0, 0 },
    { 0x887cc1a1, 98 },
    { 0, 0 },
    { 0x9d8cc1a3, 93 },
    { 0xa39cc1a4, 38 },
    { 0xd6cc1a5, 33 },
    { 0x8bacc1a3, 47 },
    { 0, 0 },
    { 0x511cc1a8, 52 },
    { 0x3e6cc1a9, 94 },
    { 0x7b9cc1a9, 65 },
    { 0x5afcc1a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137cc1b1, 61 },
    { 0, 0 },
    { 0x53bcc1b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9cc1b6, 95 },
    { 0x570cc1b6, 81 },
    { 0, 0 },
    { 0xbb2cc1b9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfcc1bd, 61 },
    { 0xd68cc1be, 23 },
    { 0x468cc1bd, 90 },
    { 0x7a0cc1be, 90 },
    { 0x7e6cc1c1, 23 },
    { 0x436cc1c2, 33 },
    { 0, 0 },
    { 0x7f2cc1c4, 33 },
    { 0xc46cc1c5, 33 },
    { 0x433cc1c6, 33 },
    { 0xc38cc1c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175cc1cf, 61 },
    { 0xdd7cc1cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fcc1d5, 98 },
    { 0, 0 },
    { 0x10fcc1d7, 43 },
    { 0x468cc1d8, 43 },
    { 0x445cc1d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ecc1dc, 90 },
    { 0, 0 },
    { 0xbefcc1de, 17 },
    { 0xc13cc1df, 33 },
    { 0xd1fcc1df, 76 },
    { 0, 0 },
    { 0x100cc1e2, 40 },
    { 0x7b3cc1e3, 90 },
    { 0xe6cc1e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485cc1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5cc1eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edcc1f0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bacc1f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fcc1f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffcc201, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1cc206, 93 },
    { 0, 0 },
    { 0x2dacc208, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdcc20b, 33 },
    { 0x4ffcc20c, 90 },
    { 0xfbcc20d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609cc211, 93 },
    { 0xf94cc212, 55 },
    { 0x359cc213, 76 },
    { 0xd1acc214, 61 },
    { 0x93dcc213, 41 },
    { 0x7ccc216, 52 },
    { 0x8fccc216, 76 },
    { 0xa75cc218, 55 },
    { 0xb6ecc215, 90 },
    { 0x9f5cc21a, 55 },
    { 0x85acc21b, 78 },
    { 0xdb9cc21c, 91 },
    { 0x2a1cc21d, 55 },
    { 0x9d7cc21e, 53 },
    { 0xa91cc21c, 95 },
    { 0xfd5cc220, 52 },
    { 0xab1cc215, 93 },
    { 0xb40cc21d, 44 },
    { 0x8bfcc215, 90 },
    { 0, 0 },
    { 0x2d6cc225, 95 },
    { 0, 0 },
    { 0x6bccc227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x104cc22a, 23 },
    { 0x65fcc22b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddcc231, 90 },
    { 0, 0 },
    { 0x2f2cc233, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47acc239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17cc240, 23 },
    { 0, 0 },
    { 0xf1ecc242, 90 },
    { 0, 0 },
    { 0x99bcc244, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623cc251, 90 },
    { 0x873cc252, 61 },
    { 0x54acc252, 90 },
    { 0x530cc253, 98 },
    { 0x7ecc251, 100 },
    { 0x867cc256, 43 },
    { 0x353cc254, 75 },
    { 0x545cc258, 91 },
    { 0x978cc259, 61 },
    { 0, 0 },
    { 0x7adcc25b, 61 },
    { 0, 0 },
    { 0xc34cc25d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9cc260, 23 },
    { 0xeaecc260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ecc264, 95 },
    { 0x1abcc265, 95 },
    { 0x9d8cc266, 90 },
    { 0, 0 },
    { 0xd8acc268, 95 },
    { 0x325cc268, 98 },
    { 0, 0 },
    { 0x872cc26b, 46 },
    { 0, 0 },
    { 0x8c4cc26d, 61 },
    { 0x88dcc26e, 98 },
    { 0xdc5cc26f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912cc273, 33 },
    { 0xfbbcc274, 33 },
    { 0, 0 },
    { 0xc53cc276, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72cc27e, 80 },
    { 0x5eccc27f, 98 },
    { 0x9f8cc280, 23 },
    { 0x7dccc280, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228cc285, 33 },
    { 0xf1ecc285, 33 },
    { 0, 0 },
    { 0xc0ecc288, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x830cc28b, 75 },
    { 0x8efcc28c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cc295, 93 },
    { 0xf2cc296, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc29e, 33 },
    { 0, 0 },
    { 0x4eccc2a0, 61 },
    { 0x8c9cc2a0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618cc2ac, 76 },
    { 0xc8ccc2ad, 61 },
    { 0xc12cc2ae, 90 },
    { 0x1c3cc2ad, 90 },
    { 0xd08cc2b0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945cc2be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28acc2c5, 23 },
    { 0x28dcc2c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998cc2cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0cc2ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8cc2d1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14cc2d4, 56 },
    { 0xd23cc2d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178cc2de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cc2e3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eacc2e6, 46 },
    { 0x954cc2e7, 33 },
    { 0x696cc2e8, 97 },
    { 0xa7ecc2e9, 34 },
    { 0x76cc2e8, 93 },
    { 0, 0 },
    { 0x4fdcc2ec, 88 },
    { 0, 0 },
    { 0xfa3cc2ee, 98 },
    { 0x4e7cc2ee, 33 },
    { 0x840cc2f0, 33 },
    { 0x691cc2f1, 76 },
    { 0xa18cc2ef, 90 },
    { 0xf43cc2f3, 98 },
    { 0x7cfcc2f3, 69 },
    { 0, 0 },
    { 0xe5bcc2f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fcc2f9, 33 },
    { 0x915cc2fa, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7cc2fe, 61 },
    { 0x1cbcc2fe, 55 },
    { 0, 0 },
    { 0x7e4cc301, 90 },
    { 0, 0 },
    { 0x9ddcc303, 96 },
    { 0x216cc304, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440cc30f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67cc312, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x252cc315, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bcc318, 95 },
    { 0x95dcc319, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23cc31d, 61 },
    { 0x72ecc31d, 66 },
    { 0, 0 },
    { 0x8e5cc320, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917cc327, 23 },
    { 0xfe2cc328, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90cc32b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0cc332, 52 },
    { 0x1e9cc333, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ecc337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bcc33c, 90 },
    { 0, 0 },
    { 0x658cc33e, 90 },
    { 0, 0 },
    { 0x9c9cc340, 90 },
    { 0x46bcc340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269cc34a, 33 },
    { 0, 0 },
    { 0x275cc34c, 33 },
    { 0, 0 },
    { 0x685cc34e, 90 },
    { 0x408cc34f, 66 },
    { 0, 0 },
    { 0x7e5cc351, 69 },
    { 0xa4acc351, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852cc35e, 34 },
    { 0, 0 },
    { 0xd0bcc360, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaccc363, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x428cc366, 46 },
    { 0, 0 },
    { 0xa9ccc368, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11cc36b, 61 },
    { 0xdfdcc36c, 23 },
    { 0x877cc36d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3cc370, 17 },
    { 0, 0 },
    { 0x64cc372, 95 },
    { 0xf0bcc372, 59 },
    { 0xc9ccc374, 33 },
    { 0xcbacc375, 93 },
    { 0x83ecc373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7cc379, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3cc37e, 98 },
    { 0xa00cc37e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x224cc382, 95 },
    { 0, 0 },
    { 0xa2acc384, 95 },
    { 0x6f3cc385, 33 },
    { 0x323cc385, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24cc38a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4cc38e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1cc393, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcc396, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cc39a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02cc39f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcc3a4, 55 },
    { 0xe3bcc3a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85cc3a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72cc3ae, 98 },
    { 0x46ccc3ae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ccc3b2, 90 },
    { 0, 0 },
    { 0x32ecc3b4, 70 },
    { 0, 0 },
    { 0x531cc3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46cc3b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7cc3c1, 55 },
    { 0x55bcc3c1, 90 },
    { 0x611cc3c3, 89 },
    { 0x1ecc3c4, 91 },
    { 0xe04cc3c4, 98 },
    { 0x536cc3c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1cc3ca, 33 },
    { 0xbfacc3ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dcc3d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284cc3d9, 46 },
    { 0x3f6cc3da, 76 },
    { 0x948cc3d9, 55 },
    { 0xd1dcc3da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5cc3e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cc3e9, 33 },
    { 0, 0 },
    { 0x8e4cc3eb, 55 },
    { 0x9f7cc3ec, 98 },
    { 0x1a6cc3ed, 61 },
    { 0xf42cc3ee, 95 },
    { 0x4d6cc3ef, 23 },
    { 0, 0 },
    { 0xbd2cc3f1, 23 },
    { 0x3f1cc3f1, 95 },
    { 0x620cc3f3, 23 },
    { 0x870cc3f4, 91 },
    { 0x2c2cc3f3, 61 },
    { 0x850cc3f2, 87 },
    { 0xefdcc3f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ecc3fb, 86 },
    { 0xb28cc3fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdcc403, 17 },
    { 0xd39cc404, 34 },
    { 0x3fccc405, 61 },
    { 0xba7cc406, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04cc409, 95 },
    { 0x6eacc409, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5cc40d, 78 },
    { 0x264cc40e, 76 },
    { 0xca5cc40e, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ecc414, 78 },
    { 0x30ecc414, 90 },
    { 0xcdbcc416, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14cc419, 55 },
    { 0x936cc41a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84cc41d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1cc420, 33 },
    { 0x796cc421, 61 },
    { 0x310cc422, 23 },
    { 0x845cc422, 23 },
    { 0x76cc424, 33 },
    { 0xb0bcc425, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61cc42d, 46 },
    { 0, 0 },
    { 0xba2cc42f, 61 },
    { 0, 0 },
    { 0xf17cc431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcc436, 8 },
    { 0x67ccc437, 23 },
    { 0x592cc438, 76 },
    { 0x927cc438, 34 },
    { 0x924cc43a, 61 },
    { 0, 0 },
    { 0x718cc43c, 33 },
    { 0x7b8cc43c, 3 },
    { 0x900cc43e, 23 },
    { 0xf29cc43e, 99 },
    { 0, 0 },
    { 0x404cc441, 34 },
    { 0x2a7cc442, 98 },
    { 0xcd0cc443, 33 },
    { 0xf41cc442, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcc449, 81 },
    { 0xf74cc44a, 90 },
    { 0, 0 },
    { 0xa67cc44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fcc453, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4cc456, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3cc45b, 34 },
    { 0, 0 },
    { 0xfdbcc45d, 88 },
    { 0xacacc45e, 90 },
    { 0xfa1cc45d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377cc465, 46 },
    { 0xc1bcc465, 56 },
    { 0, 0 },
    { 0x3c0cc468, 43 },
    { 0x612cc469, 17 },
    { 0xdb4cc46a, 78 },
    { 0x5b9cc46a, 90 },
    { 0x98dcc46c, 76 },
    { 0x415cc46b, 33 },
    { 0, 0 },
    { 0x4fcc46f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdcc473, 90 },
    { 0x775cc474, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12cc477, 60 },
    { 0, 0 },
    { 0x5e6cc479, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bcc47f, 33 },
    { 0x5ffcc480, 55 },
    { 0, 0 },
    { 0xe5acc482, 23 },
    { 0, 0 },
    { 0xb25cc484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cc487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599cc48b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dcc48f, 98 },
    { 0x228cc490, 61 },
    { 0x8eacc491, 23 },
    { 0x7a7cc492, 55 },
    { 0x9c8cc493, 33 },
    { 0x742cc494, 98 },
    { 0x1c6cc495, 91 },
    { 0x958cc496, 98 },
    { 0x380cc493, 43 },
    { 0x4ebcc492, 95 },
    { 0, 0 },
    { 0x3cacc49a, 34 },
    { 0x829cc49a, 33 },
    { 0x4c6cc49c, 66 },
    { 0x1c4cc49d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fecc4a9, 88 },
    { 0, 0 },
    { 0xca7cc4ab, 61 },
    { 0x9b8cc4ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x427cc4af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccc4b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eacc4b7, 43 },
    { 0x1b8cc4b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4cc4c1, 93 },
    { 0, 0 },
    { 0xe6dcc4c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ecc4ca, 98 },
    { 0x6ccc4cb, 34 },
    { 0xb7bcc4cc, 98 },
    { 0, 0 },
    { 0x2b6cc4ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99cc4d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50cc4d6, 76 },
    { 0x273cc4d7, 84 },
    { 0xfc2cc4d7, 17 },
    { 0x2fccc4d9, 99 },
    { 0x30ccc4da, 77 },
    { 0x59fcc4d9, 81 },
    { 0xf79cc4da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dcc4e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739cc4e7, 55 },
    { 0x845cc4e7, 65 },
    { 0xbcccc4e9, 61 },
    { 0x3c6cc4ea, 33 },
    { 0xd32cc4eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bcc4f9, 66 },
    { 0xd5acc4fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858cc4fe, 17 },
    { 0x69fcc4ff, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655cc503, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1acc506, 90 },
    { 0x5e8cc506, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x914cc50a, 90 },
    { 0x577cc50b, 91 },
    { 0xb70cc50b, 33 },
    { 0x20ecc50b, 98 },
    { 0x432cc50e, 91 },
    { 0, 0 },
    { 0x5edcc510, 48 },
    { 0, 0 },
    { 0x1cacc512, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dccc515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604cc523, 90 },
    { 0, 0 },
    { 0x275cc525, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554cc529, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ccc52c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5cc52f, 95 },
    { 0x181cc530, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5cc533, 85 },
    { 0, 0 },
    { 0x4cacc535, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496cc53a, 90 },
    { 0, 0 },
    { 0x609cc53c, 91 },
    { 0xa3ecc53c, 91 },
    { 0, 0 },
    { 0xdd6cc53f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868cc543, 100 },
    { 0xd8bcc543, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47cc54c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02cc550, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cc553, 77 },
    { 0x305cc554, 61 },
    { 0x4cecc553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cc559, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ecc560, 76 },
    { 0x961cc561, 81 },
    { 0x96bcc560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39cc565, 95 },
    { 0xce5cc566, 33 },
    { 0xf3bcc567, 55 },
    { 0xe0ecc567, 33 },
    { 0x722cc568, 46 },
    { 0x148cc56a, 55 },
    { 0x83acc56b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dcc56e, 55 },
    { 0xb11cc56e, 84 },
    { 0x649cc56e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6cc573, 34 },
    { 0x5c3cc574, 90 },
    { 0x274cc575, 23 },
    { 0xcefcc575, 90 },
    { 0xdaecc577, 90 },
    { 0xff7cc578, 98 },
    { 0, 0 },
    { 0xa0fcc57a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6cc583, 76 },
    { 0xf68cc584, 98 },
    { 0xe2ecc583, 61 },
    { 0, 0 },
    { 0x247cc587, 43 },
    { 0x321cc587, 76 },
    { 0x4e3cc589, 98 },
    { 0x927cc589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cacc58f, 85 },
    { 0x69ecc590, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367cc596, 55 },
    { 0x8f9cc597, 61 },
    { 0, 0 },
    { 0x6a5cc599, 33 },
    { 0xd33cc59a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7acc59d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384cc5a1, 90 },
    { 0, 0 },
    { 0x70dcc5a3, 90 },
    { 0x98bcc5a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4cc5ab, 23 },
    { 0, 0 },
    { 0x8e0cc5ad, 33 },
    { 0, 0 },
    { 0x5cccc5af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197cc5b3, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336cc5b7, 90 },
    { 0xdb1cc5b7, 55 },
    { 0x1cecc5b8, 23 },
    { 0xb80cc5b9, 33 },
    { 0xfbfcc5bb, 90 },
    { 0xc3dcc5bb, 76 },
    { 0x97fcc5bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cc5c1, 90 },
    { 0xa63cc5c2, 90 },
    { 0, 0 },
    { 0xad3cc5c4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dcc5c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16cc5ca, 96 },
    { 0, 0 },
    { 0xdedcc5cc, 90 },
    { 0xe91cc5cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fecc5d5, 55 },
    { 0x12dcc5d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3cc5dd, 98 },
    { 0x766cc5de, 90 },
    { 0xf08cc5df, 93 },
    { 0x5efcc5e0, 90 },
    { 0, 0 },
    { 0x70cc5e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630cc5e8, 98 },
    { 0x7f5cc5e9, 76 },
    { 0, 0 },
    { 0x9cdcc5eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670cc5f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5cc5fb, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86cc603, 90 },
    { 0, 0 },
    { 0x639cc605, 61 },
    { 0xc97cc606, 98 },
    { 0xd3ccc605, 90 },
    { 0x70fcc608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537cc611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8acc616, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249cc61a, 95 },
    { 0, 0 },
    { 0x3fecc61c, 66 },
    { 0, 0 },
    { 0xc04cc61e, 90 },
    { 0x51dcc61e, 90 },
    { 0, 0 },
    { 0x42acc621, 34 },
    { 0, 0 },
    { 0x212cc623, 95 },
    { 0x659cc624, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cc627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75cc632, 77 },
    { 0x3cacc633, 61 },
    { 0x960cc633, 34 },
    { 0xf3fcc634, 98 },
    { 0, 0 },
    { 0x13acc637, 80 },
    { 0xa6ecc638, 95 },
    { 0x7e4cc639, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dcc63f, 90 },
    { 0, 0 },
    { 0x4bbcc641, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x989cc644, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ecc649, 78 },
    { 0x3aecc64a, 68 },
    { 0xdfcc64b, 61 },
    { 0x4accc64a, 17 },
    { 0x9f4cc64d, 64 },
    { 0xf90cc64e, 23 },
    { 0xbd5cc64f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x742cc652, 76 },
    { 0x8bccc653, 33 },
    { 0, 0 },
    { 0x1a5cc655, 85 },
    { 0xbaecc655, 98 },
    { 0x668cc657, 98 },
    { 0x94dcc657, 76 },
    { 0x5c4cc659, 76 },
    { 0, 0 },
    { 0x7dcc65b, 90 },
    { 0x9bbcc65c, 23 },
    { 0, 0 },
    { 0x2a3cc65e, 55 },
    { 0xd6dcc65e, 95 },
    { 0xe86cc65e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ccc664, 23 },
    { 0x518cc664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecc66b, 100 },
    { 0, 0 },
    { 0xbedcc66d, 33 },
    { 0x5e6cc66e, 34 },
    { 0x8b9cc66f, 90 },
    { 0x44cc66f, 91 },
    { 0x340cc671, 77 },
    { 0xb6dcc670, 95 },
    { 0xf21cc671, 55 },
    { 0x93dcc674, 55 },
    { 0xb88cc675, 33 },
    { 0, 0 },
    { 0xf91cc677, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775cc67b, 84 },
    { 0, 0 },
    { 0xa09cc67d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dcc684, 43 },
    { 0, 0 },
    { 0xf8dcc686, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ccc68a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5cc68d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cc690, 33 },
    { 0x8eecc691, 90 },
    { 0, 0 },
    { 0x240cc693, 23 },
    { 0x1fcc694, 61 },
    { 0x5a3cc695, 33 },
    { 0xf82cc696, 99 },
    { 0x5fecc696, 46 },
    { 0, 0 },
    { 0xb32cc699, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbcc6a5, 33 },
    { 0, 0 },
    { 0x7f1cc6a7, 55 },
    { 0x174cc6a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32cc6ab, 23 },
    { 0xb7bcc6ac, 98 },
    { 0x290cc6ac, 93 },
    { 0x52dcc6ad, 95 },
    { 0xe71cc6ac, 38 },
    { 0, 0 },
    { 0x509cc6b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fcc6b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293cc6bc, 90 },
    { 0, 0 },
    { 0xdb7cc6be, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fcc6c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadecc6c9, 23 },
    { 0x102cc6ca, 78 },
    { 0, 0 },
    { 0xb4acc6cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43cc6d0, 95 },
    { 0xa01cc6d1, 33 },
    { 0, 0 },
    { 0xa7cc6d3, 95 },
    { 0xdb9cc6d4, 98 },
    { 0x967cc6d5, 34 },
    { 0x27cc6d3, 90 },
    { 0, 0 },
    { 0x346cc6d8, 61 },
    { 0x28bcc6d9, 99 },
    { 0, 0 },
    { 0xdefcc6db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bcc6e0, 90 },
    { 0x7eecc6e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2cc6e4, 90 },
    { 0, 0 },
    { 0xe40cc6e6, 34 },
    { 0, 0 },
    { 0x152cc6e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12cc6f0, 55 },
    { 0x21cc6f1, 98 },
    { 0x2f3cc6f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x235cc6f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x789cc6f8, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ecc70e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecc711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cc716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfcc71c, 100 },
    { 0x6abcc71d, 78 },
    { 0xf3dcc71e, 90 },
    { 0, 0 },
    { 0xecacc720, 23 },
    { 0, 0 },
    { 0xd7ccc722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72acc727, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fcc72d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7cc730, 34 },
    { 0, 0 },
    { 0xf69cc732, 55 },
    { 0, 0 },
    { 0x374cc734, 33 },
    { 0x6dfcc735, 23 },
    { 0x7cc735, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93cc739, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fcc741, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89cc746, 67 },
    { 0x951cc747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1cc74d, 55 },
    { 0x26fcc74e, 17 },
    { 0x3c6cc74e, 95 },
    { 0, 0 },
    { 0xf14cc751, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4cc755, 33 },
    { 0x20acc756, 66 },
    { 0x91cc757, 46 },
    { 0x57fcc757, 61 },
    { 0x795cc756, 98 },
    { 0x81ecc758, 34 },
    { 0x953cc759, 93 },
    { 0x307cc75c, 98 },
    { 0, 0 },
    { 0xfc7cc75e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9cc764, 90 },
    { 0, 0 },
    { 0xb3ecc766, 98 },
    { 0x802cc767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fcc76b, 66 },
    { 0x790cc76c, 61 },
    { 0x619cc76d, 98 },
    { 0x23fcc76e, 55 },
    { 0xd23cc76b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfcc775, 23 },
    { 0x1e0cc775, 61 },
    { 0, 0 },
    { 0xdafcc778, 23 },
    { 0, 0 },
    { 0x88dcc77a, 87 },
    { 0x48ecc77a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96acc786, 61 },
    { 0x5c8cc787, 90 },
    { 0x48acc788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecc78f, 47 },
    { 0x1bbcc790, 33 },
    { 0x82ccc791, 55 },
    { 0, 0 },
    { 0xc5ccc793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4cc79c, 33 },
    { 0x5fbcc79d, 61 },
    { 0, 0 },
    { 0x28ccc79f, 85 },
    { 0xc9dcc7a0, 61 },
    { 0x536cc7a1, 75 },
    { 0xdc6cc7a2, 33 },
    { 0x43dcc7a2, 61 },
    { 0x47ecc7a2, 43 },
    { 0x401cc7a5, 90 },
    { 0x1accc79f, 78 },
    { 0xcaacc7a3, 34 },
    { 0x792cc7a8, 35 },
    { 0xe96cc7a9, 34 },
    { 0x7ebcc7aa, 78 },
    { 0, 0 },
    { 0xb4dcc7ac, 90 },
    { 0xf26cc7ad, 33 },
    { 0, 0 },
    { 0x75ecc7af, 33 },
    { 0, 0 },
    { 0xbdbcc7b1, 55 },
    { 0xa5bcc7b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x141cc7b5, 76 },
    { 0x676cc7b5, 95 },
    { 0xccccc7b7, 17 },
    { 0x5becc7b6, 56 },
    { 0x8d1cc7b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337cc7c0, 61 },
    { 0, 0 },
    { 0x35acc7c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x243cc7c5, 81 },
    { 0, 0 },
    { 0xdf6cc7c7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5cc7ca, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cc7ce, 90 },
    { 0x53bcc7ce, 61 },
    { 0, 0 },
    { 0xec8cc7d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dcc7d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99cc7da, 98 },
    { 0x6d5cc7db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55cc7e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13cc7e6, 33 },
    { 0, 0 },
    { 0xd1acc7e8, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdcc7ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84cc7f5, 43 },
    { 0xf07cc7f5, 95 },
    { 0x6b7cc7f5, 33 },
    { 0, 0 },
    { 0xd96cc7f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62cc802, 77 },
    { 0x76fcc803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaccc807, 34 },
    { 0x6a4cc808, 61 },
    { 0x450cc809, 55 },
    { 0, 0 },
    { 0x661cc80b, 88 },
    { 0, 0 },
    { 0x4c6cc80d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4cc812, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2cc817, 90 },
    { 0x572cc817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313cc81d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8cc822, 93 },
    { 0, 0 },
    { 0x7fbcc824, 34 },
    { 0x5a1cc825, 78 },
    { 0, 0 },
    { 0x579cc827, 88 },
    { 0xe7ecc828, 67 },
    { 0x13acc828, 90 },
    { 0x429cc828, 98 },
    { 0xaa7cc82b, 61 },
    { 0x384cc82a, 17 },
    { 0xfa3cc827, 33 },
    { 0, 0 },
    { 0x733cc82f, 68 },
    { 0x5a2cc82f, 88 },
    { 0xf09cc830, 91 },
    { 0x29ccc832, 33 },
    { 0x68dcc82f, 77 },
    { 0x58dcc833, 33 },
    { 0xe20cc835, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebcc838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8cc83b, 78 },
    { 0, 0 },
    { 0x92ccc83d, 98 },
    { 0x7dccc83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37cc841, 61 },
    { 0x768cc841, 95 },
    { 0xfd3cc843, 65 },
    { 0xb3dcc843, 61 },
    { 0xaeacc844, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ccc84a, 98 },
    { 0xdb4cc84a, 78 },
    { 0xda1cc84c, 91 },
    { 0xaedcc84c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37cc852, 33 },
    { 0x64cc853, 80 },
    { 0xe81cc854, 68 },
    { 0x767cc854, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cc859, 33 },
    { 0x125cc859, 34 },
    { 0x698cc85b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x715cc85e, 90 },
    { 0x24fcc85e, 93 },
    { 0x2ecc860, 90 },
    { 0xee9cc861, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54cc865, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcc869, 61 },
    { 0, 0 },
    { 0xb34cc86b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeccc86f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498cc873, 66 },
    { 0x21ccc873, 84 },
    { 0, 0 },
    { 0xd70cc876, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9cc879, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedcc87e, 61 },
    { 0, 0 },
    { 0x8e4cc880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x697cc883, 77 },
    { 0x53dcc884, 98 },
    { 0x4afcc885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81cc88a, 93 },
    { 0x210cc88a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ccc897, 43 },
    { 0x71acc898, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2cc89c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cc89f, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ecc8a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279cc8a8, 95 },
    { 0x71cc8a8, 98 },
    { 0x552cc8aa, 46 },
    { 0, 0 },
    { 0x92ecc8ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6cc8b0, 76 },
    { 0x3e2cc8b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bcc8b5, 85 },
    { 0xb10cc8b6, 76 },
    { 0xf9ecc8b7, 97 },
    { 0, 0 },
    { 0x8c8cc8b9, 23 },
    { 0x9dccc8ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cc8bf, 91 },
    { 0xbd5cc8c0, 69 },
    { 0x35cc8c1, 43 },
    { 0, 0 },
    { 0x2e3cc8c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eccc8cb, 99 },
    { 0x7fecc8cc, 67 },
    { 0xc81cc8cd, 98 },
    { 0, 0 },
    { 0x4e3cc8cf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210cc8d5, 23 },
    { 0x8e6cc8d6, 99 },
    { 0xe7ccc8d5, 66 },
    { 0x3d5cc8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcc8dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64cc8e1, 98 },
    { 0x21bcc8e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dcc8e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bcc8ec, 98 },
    { 0xffbcc8ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7cc8f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edcc8fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bcc905, 93 },
    { 0x54cc906, 76 },
    { 0xeedcc907, 90 },
    { 0xca3cc908, 33 },
    { 0xa21cc907, 55 },
    { 0, 0 },
    { 0x531cc90b, 99 },
    { 0, 0 },
    { 0x36ecc90d, 95 },
    { 0xcd3cc90e, 55 },
    { 0x9a3cc90f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dcc914, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17cc917, 95 },
    { 0, 0 },
    { 0x35acc919, 76 },
    { 0, 0 },
    { 0xb21cc91b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2cc91f, 23 },
    { 0xe2cc920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3cc923, 96 },
    { 0xdc2cc924, 85 },
    { 0xbcc924, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cc928, 61 },
    { 0xee1cc929, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dcc92e, 23 },
    { 0xc1cc92f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cc935, 55 },
    { 0x436cc936, 94 },
    { 0xe6ecc937, 34 },
    { 0x6f7cc937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cc93b, 33 },
    { 0x7c5cc93b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cc93f, 87 },
    { 0x9a8cc940, 83 },
    { 0, 0 },
    { 0x4facc942, 55 },
    { 0xdadcc943, 33 },
    { 0, 0 },
    { 0x3bbcc945, 95 },
    { 0x928cc946, 90 },
    { 0x304cc947, 64 },
    { 0x646cc946, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56acc94e, 78 },
    { 0, 0 },
    { 0xde3cc950, 95 },
    { 0x208cc951, 2 },
    { 0x48ecc951, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cc956, 55 },
    { 0x611cc956, 61 },
    { 0, 0 },
    { 0x21bcc959, 23 },
    { 0, 0 },
    { 0xa4ecc95b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657cc95f, 34 },
    { 0x6cdcc960, 43 },
    { 0, 0 },
    { 0x480cc962, 47 },
    { 0x7b7cc963, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3accc968, 34 },
    { 0, 0 },
    { 0x9b0cc96a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19cc970, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fccc973, 90 },
    { 0x338cc974, 97 },
    { 0x98fcc974, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dcc979, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579cc97d, 90 },
    { 0xebfcc97e, 68 },
    { 0x693cc97e, 33 },
    { 0xf5acc97d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cc989, 98 },
    { 0x752cc98a, 61 },
    { 0, 0 },
    { 0xdffcc98c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bcc994, 61 },
    { 0x1c0cc995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cc999, 55 },
    { 0, 0 },
    { 0x345cc99b, 95 },
    { 0, 0 },
    { 0x89dcc99d, 23 },
    { 0x22fcc99e, 23 },
    { 0, 0 },
    { 0xec0cc9a0, 100 },
    { 0x454cc9a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x445cc9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880cc9ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cc9b4, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41cc9be, 96 },
    { 0xe72cc9be, 33 },
    { 0, 0 },
    { 0xc9acc9c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55cc9cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aecc9d1, 34 },
    { 0, 0 },
    { 0xf46cc9d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667cc9de, 93 },
    { 0, 0 },
    { 0x432cc9e0, 23 },
    { 0xb44cc9e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6cc9e4, 95 },
    { 0, 0 },
    { 0x4f3cc9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x272cc9e9, 76 },
    { 0, 0 },
    { 0xac0cc9eb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4acc9f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40acc9f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403cca03, 48 },
    { 0xe22cca03, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27cca0a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ccca0d, 98 },
    { 0x296cca0e, 43 },
    { 0x178cca0f, 61 },
    { 0x9a4cca0f, 91 },
    { 0, 0 },
    { 0xf8cca12, 95 },
    { 0x11cca12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930cca19, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328cca20, 55 },
    { 0xc3ecca21, 76 },
    { 0, 0 },
    { 0x12acca23, 52 },
    { 0xd6ecca23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xabacca27, 95 },
    { 0x8a9cca28, 95 },
    { 0x1f0cca27, 23 },
    { 0x708cca2a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x557cca2d, 67 },
    { 0x286cca2e, 90 },
    { 0x5e9cca2d, 61 },
    { 0xe8fcca2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cca33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xacacca36, 49 },
    { 0x8dacca37, 43 },
    { 0xd53cca38, 76 },
    { 0xf06cca36, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874cca3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628cca43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffcca48, 76 },
    { 0x5eacca48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5cca4d, 90 },
    { 0x14acca4e, 61 },
    { 0, 0 },
    { 0xaaacca50, 98 },
    { 0, 0 },
    { 0xd72cca52, 95 },
    { 0x583cca53, 81 },
    { 0x9a1cca52, 33 },
    { 0x113cca55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4cca58, 88 },
    { 0x280cca59, 33 },
    { 0xd94cca58, 98 },
    { 0x378cca5b, 49 },
    { 0x755cca5b, 90 },
    { 0, 0 },
    { 0xcd4cca5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acca64, 77 },
    { 0x44acca65, 84 },
    { 0x3c1cca65, 33 },
    { 0x86fcca64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64cca6b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x100cca6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dcca71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2cca77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3facca7a, 23 },
    { 0, 0 },
    { 0x6a0cca7c, 91 },
    { 0x49cca7d, 90 },
    { 0xd37cca7e, 69 },
    { 0x3c9cca7f, 98 },
    { 0x74dcca7f, 94 },
    { 0xc86cca81, 34 },
    { 0x53fcca82, 90 },
    { 0x1acca83, 78 },
    { 0x1b7cca84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x703cca87, 95 },
    { 0, 0 },
    { 0xd25cca89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdcca8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfcca93, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36cca99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fccaa1, 43 },
    { 0x76ccaa2, 95 },
    { 0xb9accaa1, 84 },
    { 0x4deccaa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1ccaab, 90 },
    { 0x9caccaac, 99 },
    { 0x244ccaab, 55 },
    { 0xeb7ccaae, 85 },
    { 0xceaccaaf, 91 },
    { 0x7faccaad, 23 },
    { 0x9b3ccab1, 34 },
    { 0xb3ccab2, 90 },
    { 0xbaaccab2, 61 },
    { 0x3c6ccab3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972ccabc, 95 },
    { 0, 0 },
    { 0xb91ccabe, 93 },
    { 0x15eccabf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbccac3, 23 },
    { 0xabdccac4, 33 },
    { 0, 0 },
    { 0x601ccac6, 33 },
    { 0x71cccac6, 77 },
    { 0x59cccac8, 17 },
    { 0, 0 },
    { 0x13cccaca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929ccace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eeccad4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4ccade, 98 },
    { 0, 0 },
    { 0xbb9ccae0, 98 },
    { 0, 0 },
    { 0x778ccae2, 93 },
    { 0xae1ccae3, 76 },
    { 0, 0 },
    { 0xe0dccae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6ccae8, 65 },
    { 0x103ccae9, 23 },
    { 0x6daccae9, 61 },
    { 0x5ebccaeb, 90 },
    { 0x766ccae9, 76 },
    { 0x64eccaeb, 61 },
    { 0x339ccaee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3ccaf1, 87 },
    { 0x576ccaf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276ccaf7, 95 },
    { 0xdedccaf7, 90 },
    { 0x509ccaf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dccafc, 61 },
    { 0x512ccafd, 34 },
    { 0, 0 },
    { 0x16cccaff, 33 },
    { 0, 0 },
    { 0xfdbccb01, 55 },
    { 0, 0 },
    { 0x1deccb03, 90 },
    { 0, 0 },
    { 0x19cccb05, 43 },
    { 0xe9ccb06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93ccb0a, 93 },
    { 0, 0 },
    { 0xe38ccb0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ccb12, 90 },
    { 0, 0 },
    { 0xf40ccb14, 33 },
    { 0xef3ccb15, 77 },
    { 0x572ccb15, 98 },
    { 0xab5ccb15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedccb1d, 33 },
    { 0x2abccb1e, 52 },
    { 0x6eeccb1f, 23 },
    { 0x354ccb20, 23 },
    { 0x69dccb21, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dccb27, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1ccb2d, 90 },
    { 0xdbcccb2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0ccb32, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8ccb37, 90 },
    { 0xd32ccb38, 33 },
    { 0xa3bccb39, 90 },
    { 0x80dccb3a, 34 },
    { 0xfcbccb39, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357ccb42, 93 },
    { 0xad6ccb43, 90 },
    { 0x178ccb43, 61 },
    { 0, 0 },
    { 0x2c6ccb46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439ccb4a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18accb54, 23 },
    { 0x512ccb55, 90 },
    { 0xfadccb56, 87 },
    { 0x27bccb55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cccb60, 33 },
    { 0, 0 },
    { 0x9bbccb62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefccb66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x746ccb69, 34 },
    { 0, 0 },
    { 0x3d6ccb6b, 43 },
    { 0x6daccb6c, 81 },
    { 0xf65ccb6d, 98 },
    { 0x715ccb6d, 98 },
    { 0, 0 },
    { 0xbfcccb70, 23 },
    { 0x8e6ccb70, 90 },
    { 0xfecccb72, 75 },
    { 0x28accb73, 61 },
    { 0x11ccb73, 98 },
    { 0xaafccb75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5ccb79, 61 },
    { 0, 0 },
    { 0x764ccb7b, 53 },
    { 0, 0 },
    { 0x222ccb7d, 84 },
    { 0x17eccb7e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63ccb83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccccb91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ccb96, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2accb9c, 90 },
    { 0xc25ccb9d, 100 },
    { 0x509ccb9c, 34 },
    { 0x6d6ccb9f, 33 },
    { 0x77bccb9f, 87 },
    { 0, 0 },
    { 0xca6ccba2, 34 },
    { 0xffdccba3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cccbab, 75 },
    { 0xe0cccbab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6ccbb0, 93 },
    { 0x546ccbb1, 34 },
    { 0x5e5ccbb1, 23 },
    { 0, 0 },
    { 0x40cccbb4, 61 },
    { 0xb19ccbb5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bccbb9, 93 },
    { 0x875ccbb9, 34 },
    { 0x7d9ccbb9, 95 },
    { 0x1d8ccbb9, 78 },
    { 0x9aeccbbd, 34 },
    { 0x2f5ccbbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ccbc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78eccbc7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ccbcd, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0x801ccbd0, 61 },
    { 0, 0 },
    { 0x8f2ccbd2, 90 },
    { 0x28accbd3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bccbd8, 78 },
    { 0, 0 },
    { 0x666ccbda, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51accbde, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674ccbe2, 98 },
    { 0x42dccbe3, 33 },
    { 0, 0 },
    { 0x7dfccbe5, 98 },
    { 0xd88ccbe5, 88 },
    { 0xbb8ccbe6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55ccbec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x179ccbef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0ccbf2, 87 },
    { 0xf7ccbf3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ccbf7, 90 },
    { 0xb54ccbf8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfccbfb, 93 },
    { 0xec9ccbfc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ccc01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ccc04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4caccc0d, 38 },
    { 0xe1dccc0e, 78 },
    { 0, 0 },
    { 0x41eccc10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9accc14, 66 },
    { 0x9a2ccc15, 98 },
    { 0, 0 },
    { 0x7e8ccc17, 95 },
    { 0xfb0ccc17, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0accc1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ccc1f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8ccc22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73ccc29, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaccc2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x595ccc30, 55 },
    { 0x995ccc30, 98 },
    { 0x3ccc31, 95 },
    { 0, 0 },
    { 0x16ccc34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efccc38, 91 },
    { 0x8adccc39, 23 },
    { 0x32bccc3a, 90 },
    { 0xee0ccc3a, 34 },
    { 0xd67ccc3b, 23 },
    { 0x8f5ccc3d, 90 },
    { 0x6d4ccc3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ccc44, 61 },
    { 0xff1ccc45, 43 },
    { 0x1d7ccc46, 46 },
    { 0, 0 },
    { 0x1cfccc48, 95 },
    { 0x55cccc49, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccc55, 90 },
    { 0xd2bccc55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bccc59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2ccc5d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadccc62, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3ccc66, 90 },
    { 0, 0 },
    { 0x229ccc68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1ccc6c, 33 },
    { 0, 0 },
    { 0xb8accc6e, 90 },
    { 0x318ccc6f, 94 },
    { 0x2b5ccc6e, 64 },
    { 0xd74ccc71, 98 },
    { 0xf0fccc71, 33 },
    { 0x1e8ccc73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ccc76, 33 },
    { 0x29dccc76, 33 },
    { 0, 0 },
    { 0xc13ccc79, 95 },
    { 0x5e7ccc7a, 61 },
    { 0, 0 },
    { 0xc4cccc7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4ccc80, 90 },
    { 0xc08ccc81, 33 },
    { 0, 0 },
    { 0x8e5ccc83, 98 },
    { 0x911ccc84, 34 },
    { 0xd8dccc84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6ccc97, 98 },
    { 0x62eccc97, 61 },
    { 0x344ccc98, 98 },
    { 0xbe5ccc99, 90 },
    { 0, 0 },
    { 0xbccccc9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacccca1, 23 },
    { 0x741ccca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccca5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6ccca9, 55 },
    { 0xdafccca9, 46 },
    { 0x7c3cccab, 99 },
    { 0x9dbcccac, 95 },
    { 0x39dcccac, 55 },
    { 0x63cccae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1cccb2, 90 },
    { 0, 0 },
    { 0x63bcccb4, 61 },
    { 0, 0 },
    { 0x929cccb6, 55 },
    { 0xee2cccb7, 36 },
    { 0x245cccb8, 61 },
    { 0x82ecccb8, 95 },
    { 0x807cccb9, 99 },
    { 0xf70cccbb, 61 },
    { 0x8aecccb6, 98 },
    { 0xa52cccbd, 90 },
    { 0x13cccbe, 33 },
    { 0x732cccbf, 98 },
    { 0x124cccc0, 90 },
    { 0xd81cccb6, 95 },
    { 0x4cecccc2, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aecccc8, 48 },
    { 0, 0 },
    { 0x534cccca, 85 },
    { 0, 0 },
    { 0x51cccccc, 34 },
    { 0, 0 },
    { 0xca0cccce, 81 },
    { 0xc8acccce, 34 },
    { 0, 0 },
    { 0xac7cccd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcccd4, 84 },
    { 0xae2cccd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bcccdb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9ccce0, 98 },
    { 0x472ccce1, 91 },
    { 0xb08ccce2, 90 },
    { 0xe0eccce3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4ccce7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdcccea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x811ccced, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcccf0, 61 },
    { 0x713cccf1, 90 },
    { 0x69fcccf1, 61 },
    { 0, 0 },
    { 0x661cccf4, 23 },
    { 0x744cccf4, 43 },
    { 0, 0 },
    { 0x398cccf7, 67 },
    { 0, 0 },
    { 0x23acccf9, 95 },
    { 0xd01cccfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8cccfd, 23 },
    { 0, 0 },
    { 0x9bdcccff, 23 },
    { 0xf8accd00, 95 },
    { 0, 0 },
    { 0x189ccd02, 85 },
    { 0xd2dccd03, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ccd07, 81 },
    { 0, 0 },
    { 0xe7accd09, 90 },
    { 0, 0 },
    { 0xae4ccd0b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeccd11, 61 },
    { 0xc07ccd12, 95 },
    { 0x838ccd11, 85 },
    { 0, 0 },
    { 0x9a0ccd15, 81 },
    { 0x76fccd16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40ccd1b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906ccd21, 61 },
    { 0xcacccd21, 33 },
    { 0, 0 },
    { 0xd3dccd24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8ccd27, 23 },
    { 0x683ccd28, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92ccd2b, 52 },
    { 0x874ccd2c, 52 },
    { 0x294ccd2c, 33 },
    { 0x9b7ccd2d, 88 },
    { 0xec4ccd2b, 90 },
    { 0xcecccd30, 98 },
    { 0, 0 },
    { 0x3ceccd32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7ccd36, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0accd3a, 33 },
    { 0x41bccd3b, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fccd43, 33 },
    { 0x579ccd44, 43 },
    { 0xf7cccd44, 98 },
    { 0, 0 },
    { 0xf94ccd47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4ccd4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ccd57, 90 },
    { 0x8f6ccd58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580ccd60, 90 },
    { 0xd14ccd60, 33 },
    { 0x1beccd61, 33 },
    { 0xdcfccd63, 78 },
    { 0xaaaccd64, 61 },
    { 0x124ccd65, 98 },
    { 0xe2accd66, 33 },
    { 0, 0 },
    { 0x42dccd68, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cccd6e, 23 },
    { 0xa42ccd6f, 76 },
    { 0xb66ccd6f, 90 },
    { 0x31ccd71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ccd75, 98 },
    { 0x969ccd76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84ccd7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ccd81, 98 },
    { 0x489ccd82, 33 },
    { 0xebbccd83, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92ccd87, 90 },
    { 0x6a3ccd87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68ccd8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7ccd8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebccd91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ccd98, 87 },
    { 0xaf2ccd98, 92 },
    { 0x98accd9a, 100 },
    { 0, 0 },
    { 0xe51ccd9c, 77 },
    { 0x877ccd9c, 90 },
    { 0, 0 },
    { 0xabfccd9f, 65 },
    { 0x14dccd9f, 61 },
    { 0, 0 },
    { 0xda5ccda2, 61 },
    { 0xaadccda2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50ccdab, 76 },
    { 0x5dccdab, 98 },
    { 0xe78ccdad, 33 },
    { 0x881ccdab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3ccdb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fccdb8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bccdbc, 90 },
    { 0xe0cccdbd, 23 },
    { 0xe9accdbd, 95 },
    { 0xfe7ccdbf, 98 },
    { 0, 0 },
    { 0x124ccdc1, 33 },
    { 0, 0 },
    { 0xf01ccdc3, 72 },
    { 0, 0 },
    { 0xeb6ccdc5, 98 },
    { 0, 0 },
    { 0x283ccdc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3ccdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x419ccdd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dccdd3, 90 },
    { 0x7acccdd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bccddc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ccddf, 90 },
    { 0x6ceccde0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcccde5, 95 },
    { 0xb68ccde5, 61 },
    { 0x99fccde6, 33 },
    { 0xa0cccde8, 95 },
    { 0x9c4ccde5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62ccdee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68ccdf2, 61 },
    { 0, 0 },
    { 0xbadccdf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ccdf9, 40 },
    { 0xf5ccdf9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37cce02, 76 },
    { 0, 0 },
    { 0x700cce04, 61 },
    { 0x3a5cce04, 95 },
    { 0x22cce04, 23 },
    { 0x398cce07, 87 },
    { 0xca6cce08, 90 },
    { 0x603cce09, 98 },
    { 0x673cce08, 33 },
    { 0xa2fcce0b, 67 },
    { 0, 0 },
    { 0x629cce0d, 90 },
    { 0x9cdcce0d, 95 },
    { 0xeb4cce0f, 55 },
    { 0x11cce10, 66 },
    { 0, 0 },
    { 0xd40cce12, 100 },
    { 0x291cce13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafccce16, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602cce1b, 95 },
    { 0x671cce1b, 97 },
    { 0, 0 },
    { 0x433cce1e, 33 },
    { 0xa57cce1e, 33 },
    { 0xb0ecce1e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dcce24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bcce2d, 91 },
    { 0, 0 },
    { 0xb97cce2f, 43 },
    { 0xa6dcce30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0cce37, 90 },
    { 0x69fcce37, 91 },
    { 0, 0 },
    { 0xb46cce3a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1cce40, 95 },
    { 0x7d2cce41, 23 },
    { 0xc99cce41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82cce47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2cce4d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x556cce50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665cce5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffcce62, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993cce66, 90 },
    { 0x898cce66, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ecce72, 33 },
    { 0x9cccce73, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fcce76, 23 },
    { 0xe53cce77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbcce7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bcce7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x516cce80, 99 },
    { 0x27bcce81, 61 },
    { 0xadecce80, 17 },
    { 0, 0 },
    { 0xefdcce84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbacce8a, 33 },
    { 0, 0 },
    { 0x7bcce8c, 95 },
    { 0x58ecce8d, 34 },
    { 0x886cce8e, 76 },
    { 0, 0 },
    { 0xe05cce90, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264cce9a, 90 },
    { 0x134cce9a, 61 },
    { 0, 0 },
    { 0xcf1cce9d, 55 },
    { 0xa6ccce9e, 90 },
    { 0xecacce9f, 61 },
    { 0x24accea0, 90 },
    { 0x16ecce9d, 33 },
    { 0x663ccea2, 90 },
    { 0xb2acce9d, 23 },
    { 0, 0 },
    { 0xe42ccea5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x135ccea8, 33 },
    { 0x1ddccea9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49cceac, 34 },
    { 0, 0 },
    { 0xfe1cceae, 61 },
    { 0x8ffcceae, 78 },
    { 0, 0 },
    { 0x30bcceb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dcceb8, 90 },
    { 0x25bcceb9, 23 },
    { 0, 0 },
    { 0x2f3ccebb, 90 },
    { 0x8c3ccebc, 99 },
    { 0x230ccebc, 97 },
    { 0xe43ccebe, 60 },
    { 0x187ccebf, 90 },
    { 0, 0 },
    { 0xae9ccec1, 61 },
    { 0xb6eccec2, 33 },
    { 0x2daccec1, 43 },
    { 0, 0 },
    { 0x42fccec5, 69 },
    { 0x7b0ccec6, 95 },
    { 0x72eccec5, 61 },
    { 0xe09ccec6, 43 },
    { 0, 0 },
    { 0x175cceca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cccecf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31cced2, 46 },
    { 0xc93cced3, 33 },
    { 0, 0 },
    { 0x70acced5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ecced9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x455ccedc, 23 },
    { 0, 0 },
    { 0x567ccede, 46 },
    { 0x65bccede, 23 },
    { 0x318ccede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944ccee5, 76 },
    { 0xe8accee6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994cceea, 98 },
    { 0xd39cceeb, 90 },
    { 0x5dfcceea, 33 },
    { 0x6d1cceed, 90 },
    { 0x68dcceed, 90 },
    { 0, 0 },
    { 0x227ccef0, 90 },
    { 0x91cccef0, 33 },
    { 0, 0 },
    { 0xaa7ccef3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dccefd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3ccf00, 90 },
    { 0x3dbccf01, 98 },
    { 0xeccf00, 99 },
    { 0xc0ccf03, 23 },
    { 0x4c0ccf02, 61 },
    { 0, 0 },
    { 0xb38ccf06, 33 },
    { 0x3b2ccf07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeccf0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dccf0e, 90 },
    { 0x5b0ccf0f, 77 },
    { 0, 0 },
    { 0x2ccf11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2ccf1d, 65 },
    { 0, 0 },
    { 0x73accf1f, 67 },
    { 0xd81ccf1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36eccf24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195ccf2a, 61 },
    { 0, 0 },
    { 0x7d3ccf2c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ccf32, 91 },
    { 0x6cbccf32, 33 },
    { 0, 0 },
    { 0x585ccf35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97ccf39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x220ccf3c, 99 },
    { 0, 0 },
    { 0x35fccf3e, 61 },
    { 0, 0 },
    { 0xa78ccf40, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5ccf47, 87 },
    { 0xa67ccf48, 23 },
    { 0, 0 },
    { 0xd84ccf4a, 33 },
    { 0x384ccf4a, 66 },
    { 0x11ccf4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ccf4f, 77 },
    { 0x906ccf50, 90 },
    { 0, 0 },
    { 0xa02ccf52, 66 },
    { 0xa95ccf53, 66 },
    { 0x5abccf54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dccf58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfccf5b, 67 },
    { 0x7b9ccf5c, 33 },
    { 0x2edccf5b, 95 },
    { 0, 0 },
    { 0xcbeccf5f, 84 },
    { 0xe53ccf60, 55 },
    { 0xe10ccf60, 85 },
    { 0x647ccf62, 95 },
    { 0x6c1ccf63, 95 },
    { 0, 0 },
    { 0x6c1ccf65, 70 },
    { 0xfdccf66, 85 },
    { 0x7aeccf66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ccf6b, 98 },
    { 0x868ccf6c, 23 },
    { 0x74cccf6d, 90 },
    { 0xcf3ccf6c, 94 },
    { 0, 0 },
    { 0xa79ccf70, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583ccf74, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ccf78, 34 },
    { 0xb1cccf79, 90 },
    { 0x6c0ccf7a, 76 },
    { 0, 0 },
    { 0xae4ccf7c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ccf7f, 23 },
    { 0xa42ccf80, 76 },
    { 0x7dbccf81, 17 },
    { 0xf2fccf82, 33 },
    { 0x4ddccf83, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eccf8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542ccf91, 61 },
    { 0x8a1ccf91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bccf99, 90 },
    { 0x1d4ccf9a, 93 },
    { 0x832ccf9b, 61 },
    { 0xc0eccf99, 66 },
    { 0x917ccf9d, 88 },
    { 0xb55ccf9e, 67 },
    { 0x1deccf9b, 33 },
    { 0xd2accf99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x709ccfa3, 98 },
    { 0x3dccfa4, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ccfa7, 95 },
    { 0, 0 },
    { 0xe5ccfa9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ccfac, 95 },
    { 0, 0 },
    { 0x794ccfae, 90 },
    { 0, 0 },
    { 0x698ccfb0, 98 },
    { 0x89cccfb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ccfb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cccfbe, 46 },
    { 0x671ccfbf, 28 },
    { 0xc22ccfbf, 33 },
    { 0xe48ccfc1, 61 },
    { 0xbf8ccfbf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6ccfc6, 55 },
    { 0xda5ccfc7, 76 },
    { 0x145ccfc8, 76 },
    { 0xcabccfc9, 33 },
    { 0xb9dccfca, 90 },
    { 0xf1eccfca, 76 },
    { 0xd0eccfcc, 98 },
    { 0xd39ccfc7, 88 },
    { 0x386ccfc9, 98 },
    { 0x13eccfce, 91 },
    { 0x74fccfc7, 76 },
    { 0xdb7ccfc9, 96 },
    { 0x32ccfd2, 55 },
    { 0x1e3ccfd3, 90 },
    { 0x1f2ccfd4, 98 },
    { 0, 0 },
    { 0x5d4ccfd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850ccfda, 46 },
    { 0xf57ccfda, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abccfdf, 90 },
    { 0xb1eccfe0, 61 },
    { 0xd9bccfe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bccfe4, 98 },
    { 0x11fccfe5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91ccfe9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5ccff0, 95 },
    { 0xa92ccff0, 33 },
    { 0, 0 },
    { 0xee8ccff3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94ccffb, 75 },
    { 0x851ccffb, 84 },
    { 0xa5fccffd, 91 },
    { 0x78eccffe, 98 },
    { 0x1fbccffe, 90 },
    { 0x352ccfff, 90 },
    { 0xd8dccfff, 90 },
    { 0, 0 },
    { 0xc8acd003, 95 },
    { 0, 0 },
    { 0x57bcd005, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7becd009, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dcd011, 33 },
    { 0x43cd011, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20cd017, 76 },
    { 0, 0 },
    { 0x182cd019, 99 },
    { 0x28bcd01a, 98 },
    { 0x58acd019, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bcd01e, 98 },
    { 0, 0 },
    { 0x124cd020, 90 },
    { 0xb24cd021, 33 },
    { 0, 0 },
    { 0x477cd023, 90 },
    { 0x38bcd023, 23 },
    { 0xb3dcd023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cd028, 33 },
    { 0xf8ecd029, 61 },
    { 0x822cd029, 33 },
    { 0, 0 },
    { 0xed0cd02c, 94 },
    { 0x1dacd02d, 90 },
    { 0xf10cd02d, 17 },
    { 0, 0 },
    { 0x14dcd030, 61 },
    { 0, 0 },
    { 0x194cd032, 98 },
    { 0x9f4cd032, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80cd037, 95 },
    { 0x99acd038, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ecd03b, 93 },
    { 0x3b5cd03c, 91 },
    { 0xaf9cd03c, 88 },
    { 0xfa2cd03d, 95 },
    { 0x7fbcd03f, 81 },
    { 0xb55cd03d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2accd044, 91 },
    { 0x8ccd045, 95 },
    { 0xae9cd046, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdcd049, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893cd04d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1cd051, 95 },
    { 0, 0 },
    { 0xadbcd053, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664cd058, 95 },
    { 0xb61cd059, 80 },
    { 0x879cd059, 61 },
    { 0x8f0cd05b, 33 },
    { 0xdb7cd05c, 66 },
    { 0x382cd059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbcd061, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bcd065, 61 },
    { 0, 0 },
    { 0x8b1cd067, 76 },
    { 0xb41cd067, 90 },
    { 0xdd3cd068, 34 },
    { 0xf38cd068, 99 },
    { 0, 0 },
    { 0xd63cd06c, 95 },
    { 0xcecd06c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41cd071, 66 },
    { 0, 0 },
    { 0x5a4cd073, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cd077, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fccd07b, 90 },
    { 0x12acd07b, 99 },
    { 0, 0 },
    { 0xf98cd07e, 90 },
    { 0, 0 },
    { 0x75ccd080, 55 },
    { 0x39ecd081, 23 },
    { 0x15fcd081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bcd086, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2cd089, 76 },
    { 0x655cd08a, 91 },
    { 0, 0 },
    { 0x15dcd08c, 80 },
    { 0, 0 },
    { 0x991cd08e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2cd09a, 61 },
    { 0xa72cd09b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51acd09e, 96 },
    { 0x559cd09f, 90 },
    { 0xaeacd0a0, 98 },
    { 0xff9cd0a1, 93 },
    { 0, 0 },
    { 0x71cd0a3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196cd0ac, 23 },
    { 0xfc8cd0ac, 93 },
    { 0x761cd0ae, 90 },
    { 0xc4dcd0af, 93 },
    { 0xb1fcd0b0, 95 },
    { 0x385cd0ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5cd0b6, 76 },
    { 0, 0 },
    { 0xad9cd0b8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970cd0bc, 99 },
    { 0, 0 },
    { 0xc45cd0be, 95 },
    { 0xa20cd0bf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2cd0c7, 98 },
    { 0x192cd0c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8cd0cc, 76 },
    { 0, 0 },
    { 0xa47cd0ce, 23 },
    { 0x7d0cd0cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cecd0d2, 40 },
    { 0, 0 },
    { 0xdd1cd0d4, 55 },
    { 0, 0 },
    { 0x157cd0d6, 98 },
    { 0, 0 },
    { 0x269cd0d8, 61 },
    { 0, 0 },
    { 0xf73cd0da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6cd0e0, 95 },
    { 0xe1bcd0e0, 36 },
    { 0xbabcd0e2, 91 },
    { 0x5bdcd0e2, 61 },
    { 0x8f2cd0e0, 23 },
    { 0, 0 },
    { 0xb6ecd0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacd0ea, 33 },
    { 0x166cd0ea, 23 },
    { 0xd66cd0ec, 23 },
    { 0xa73cd0ea, 61 },
    { 0xbf9cd0ea, 100 },
    { 0, 0 },
    { 0x36cd0f0, 33 },
    { 0, 0 },
    { 0x6facd0f2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8cd0f6, 90 },
    { 0xadecd0f6, 61 },
    { 0xf55cd0f8, 33 },
    { 0x66acd0f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9cd108, 95 },
    { 0, 0 },
    { 0xa75cd10a, 61 },
    { 0, 0 },
    { 0x576cd10c, 92 },
    { 0x2e7cd10c, 90 },
    { 0x7bcd10d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5cd114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9cd117, 61 },
    { 0, 0 },
    { 0x784cd119, 61 },
    { 0x1cccd11a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0cd11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fcd120, 76 },
    { 0xf21cd121, 33 },
    { 0x2dbcd122, 46 },
    { 0, 0 },
    { 0x938cd124, 98 },
    { 0x7adcd125, 95 },
    { 0xa4acd126, 25 },
    { 0x958cd124, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ccd12a, 90 },
    { 0, 0 },
    { 0x1d0cd12c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45cd130, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328cd134, 95 },
    { 0xb39cd135, 91 },
    { 0xd4bcd136, 95 },
    { 0xbccd135, 46 },
    { 0xf99cd137, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02cd13b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650cd140, 33 },
    { 0, 0 },
    { 0x90cd142, 61 },
    { 0x4cccd143, 23 },
    { 0xc14cd144, 33 },
    { 0x777cd145, 61 },
    { 0x10bcd146, 67 },
    { 0xd82cd147, 76 },
    { 0, 0 },
    { 0xc6fcd149, 75 },
    { 0xd62cd14a, 94 },
    { 0, 0 },
    { 0xd13cd14c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ccd14f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2cd152, 34 },
    { 0x9cacd153, 43 },
    { 0xb2bcd153, 83 },
    { 0x635cd154, 61 },
    { 0x86fcd156, 34 },
    { 0xba7cd157, 78 },
    { 0xd58cd158, 95 },
    { 0x89bcd156, 76 },
    { 0x54ecd159, 100 },
    { 0x72dcd152, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ecd161, 75 },
    { 0xf99cd162, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebcd169, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cd16d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cd170, 95 },
    { 0x80ccd171, 64 },
    { 0xc6fcd172, 90 },
    { 0x3d4cd171, 33 },
    { 0x634cd174, 34 },
    { 0, 0 },
    { 0x9f1cd176, 76 },
    { 0xb99cd176, 98 },
    { 0x8dacd178, 33 },
    { 0x244cd179, 33 },
    { 0, 0 },
    { 0x26bcd17b, 46 },
    { 0x8d1cd17c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4cd181, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edcd184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dcd187, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5cd18a, 46 },
    { 0x206cd18b, 61 },
    { 0xf73cd18a, 33 },
    { 0xfefcd18b, 43 },
    { 0x461cd18d, 95 },
    { 0x533cd18f, 61 },
    { 0xef4cd190, 23 },
    { 0, 0 },
    { 0x56ccd192, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1cd197, 90 },
    { 0, 0 },
    { 0x4ffcd199, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fcd1a4, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5cd1ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4cd1b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746cd1b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fcd1bd, 33 },
    { 0x65bcd1be, 34 },
    { 0xcbccd1bf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9cd1c2, 23 },
    { 0xccfcd1c3, 85 },
    { 0x8efcd1c4, 80 },
    { 0x110cd1c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808cd1cc, 95 },
    { 0xd1acd1cd, 90 },
    { 0x2b8cd1ce, 93 },
    { 0xffdcd1ce, 23 },
    { 0, 0 },
    { 0x830cd1d1, 61 },
    { 0xa6ecd1d2, 51 },
    { 0xba9cd1d2, 33 },
    { 0xd2bcd1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25cd1d9, 61 },
    { 0x4afcd1da, 84 },
    { 0xf5ecd1db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7cd1df, 78 },
    { 0xadcd1e0, 91 },
    { 0x1a6cd1df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05cd1e7, 90 },
    { 0x6ccd1e8, 99 },
    { 0xa74cd1e7, 67 },
    { 0, 0 },
    { 0x4b9cd1eb, 33 },
    { 0, 0 },
    { 0xbcecd1ed, 33 },
    { 0, 0 },
    { 0xb54cd1ef, 61 },
    { 0xdffcd1f0, 33 },
    { 0xbd3cd1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bcd1f4, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48cd1ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94cd202, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ecd205, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bccd208, 76 },
    { 0x88ecd209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78acd213, 90 },
    { 0xea7cd213, 98 },
    { 0x64fcd215, 23 },
    { 0x738cd216, 43 },
    { 0xf3fcd217, 55 },
    { 0, 0 },
    { 0x98fcd219, 80 },
    { 0, 0 },
    { 0xbffcd21b, 98 },
    { 0x543cd21c, 77 },
    { 0x1d0cd21b, 90 },
    { 0, 0 },
    { 0xb15cd21f, 55 },
    { 0x5b8cd220, 34 },
    { 0xccacd220, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9cd226, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27cd22a, 84 },
    { 0, 0 },
    { 0x4a3cd22c, 61 },
    { 0x80cd22c, 94 },
    { 0, 0 },
    { 0xe8bcd22f, 23 },
    { 0, 0 },
    { 0xb3cd231, 34 },
    { 0x68bcd231, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9cd236, 61 },
    { 0x7b2cd237, 33 },
    { 0x8cfcd238, 90 },
    { 0x789cd238, 98 },
    { 0x2aacd239, 95 },
    { 0x615cd23b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bcd23e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366cd248, 98 },
    { 0, 0 },
    { 0x27ecd24a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fcd24d, 61 },
    { 0, 0 },
    { 0x217cd24f, 33 },
    { 0xbccd24f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95cd253, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0cd256, 33 },
    { 0xe25cd257, 87 },
    { 0x6a1cd257, 93 },
    { 0x781cd259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81acd262, 77 },
    { 0xf0acd262, 43 },
    { 0, 0 },
    { 0x43dcd265, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bcd26b, 61 },
    { 0xf87cd26c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142cd270, 46 },
    { 0xf3dcd271, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938cd275, 100 },
    { 0x800cd276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5acd279, 93 },
    { 0, 0 },
    { 0x176cd27b, 66 },
    { 0x3facd27b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72acd283, 33 },
    { 0xa94cd283, 99 },
    { 0, 0 },
    { 0xc64cd286, 33 },
    { 0x98dcd287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7cd28b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x517cd28e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48cd295, 76 },
    { 0, 0 },
    { 0x2eccd297, 98 },
    { 0xb40cd297, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfcd2a0, 33 },
    { 0xd5cd2a1, 34 },
    { 0x440cd2a2, 17 },
    { 0xbdcd2a2, 61 },
    { 0xb3acd2a2, 34 },
    { 0x129cd2a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2cd2aa, 98 },
    { 0xff8cd2ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bcd2b0, 93 },
    { 0x711cd2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1cd2b6, 95 },
    { 0xd47cd2b7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbcd2c0, 90 },
    { 0x9b6cd2c1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19cd2c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633cd2ca, 95 },
    { 0, 0 },
    { 0x938cd2cc, 66 },
    { 0, 0 },
    { 0x8a8cd2ce, 98 },
    { 0, 0 },
    { 0x5ecd2d0, 51 },
    { 0x37bcd2d1, 33 },
    { 0, 0 },
    { 0x507cd2d3, 95 },
    { 0xe98cd2d3, 61 },
    { 0xf88cd2d5, 69 },
    { 0x157cd2d6, 90 },
    { 0xb7cd2d4, 55 },
    { 0x37ecd2d8, 52 },
    { 0xc84cd2d7, 23 },
    { 0xa10cd2d3, 61 },
    { 0x6e0cd2db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8cd2e0, 33 },
    { 0x631cd2e1, 17 },
    { 0, 0 },
    { 0xf1fcd2e3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830cd2e8, 95 },
    { 0x826cd2e9, 36 },
    { 0x30ecd2ea, 68 },
    { 0x875cd2ea, 55 },
    { 0x873cd2ec, 99 },
    { 0x604cd2ea, 55 },
    { 0x770cd2ee, 33 },
    { 0, 0 },
    { 0x5bbcd2f0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1acd2f6, 90 },
    { 0, 0 },
    { 0x3bfcd2f8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99cd2fb, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2cd2ff, 8 },
    { 0x952cd300, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fccd303, 33 },
    { 0x1efcd303, 61 },
    { 0, 0 },
    { 0xccdcd306, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcd309, 75 },
    { 0x952cd30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffcd30f, 23 },
    { 0x63acd310, 90 },
    { 0xe8fcd311, 34 },
    { 0x4cecd311, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595cd319, 76 },
    { 0xe6bcd31a, 95 },
    { 0x287cd31a, 23 },
    { 0x3decd31c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5cd320, 84 },
    { 0x464cd321, 33 },
    { 0, 0 },
    { 0xca2cd323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dcd328, 61 },
    { 0x59ecd328, 33 },
    { 0xe90cd32a, 76 },
    { 0xc41cd32a, 76 },
    { 0x3e5cd32b, 56 },
    { 0xa5cd32d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eecd333, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6cd336, 90 },
    { 0, 0 },
    { 0xa38cd338, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7decd33b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96cd340, 95 },
    { 0xb12cd341, 95 },
    { 0x97cd342, 67 },
    { 0, 0 },
    { 0x6f7cd344, 61 },
    { 0x35dcd345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2cd34b, 65 },
    { 0xc88cd34c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621cd351, 61 },
    { 0x2e9cd352, 99 },
    { 0x203cd352, 61 },
    { 0xbe8cd353, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd357, 90 },
    { 0, 0 },
    { 0xd92cd359, 98 },
    { 0, 0 },
    { 0x68ecd35b, 90 },
    { 0, 0 },
    { 0x7f5cd35d, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dcd360, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xceacd363, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcd366, 61 },
    { 0x9f1cd367, 61 },
    { 0x650cd368, 85 },
    { 0x443cd368, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315cd36d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7cd374, 46 },
    { 0x80ccd375, 78 },
    { 0, 0 },
    { 0x9c9cd377, 66 },
    { 0xf32cd378, 95 },
    { 0xd01cd377, 23 },
    { 0x554cd37a, 33 },
    { 0x816cd37b, 33 },
    { 0, 0 },
    { 0x7d2cd37d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532cd381, 52 },
    { 0x7d9cd382, 90 },
    { 0x319cd382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bcd38c, 93 },
    { 0, 0 },
    { 0xc5ccd38e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x18cd391, 61 },
    { 0xe6bcd391, 95 },
    { 0x6e7cd393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41cd397, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617cd39d, 98 },
    { 0, 0 },
    { 0xdc0cd39f, 95 },
    { 0x1a6cd39f, 93 },
    { 0x3e7cd39f, 23 },
    { 0, 0 },
    { 0x9cbcd3a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32cd3a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bcd3af, 78 },
    { 0, 0 },
    { 0xb68cd3b1, 90 },
    { 0xc7cd3b2, 64 },
    { 0x2a6cd3b1, 52 },
    { 0xecdcd3b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3cd3bd, 34 },
    { 0xeddcd3be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18cd3c5, 33 },
    { 0xd31cd3c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaecd3ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ccd3cd, 90 },
    { 0x8c7cd3cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793cd3d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599cd3db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10cd3e4, 95 },
    { 0x1d0cd3e5, 23 },
    { 0x619cd3e6, 34 },
    { 0x758cd3e7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3acd3ee, 77 },
    { 0x80ecd3ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbcd3f7, 98 },
    { 0, 0 },
    { 0xc96cd3f9, 61 },
    { 0xb24cd3fa, 23 },
    { 0x4edcd3fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebcd3fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacd407, 43 },
    { 0xadbcd408, 23 },
    { 0x613cd409, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73cd40c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bcd410, 55 },
    { 0x2edcd411, 84 },
    { 0x70dcd411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65acd41d, 87 },
    { 0x95cd41d, 90 },
    { 0x8bfcd41f, 34 },
    { 0xa59cd420, 76 },
    { 0x87ccd421, 98 },
    { 0, 0 },
    { 0xab1cd423, 38 },
    { 0, 0 },
    { 0xd95cd425, 98 },
    { 0x4d1cd426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82cd42d, 61 },
    { 0x157cd42d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x135cd431, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ccd434, 33 },
    { 0x291cd435, 90 },
    { 0xadfcd436, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234cd43f, 34 },
    { 0xc9ccd440, 76 },
    { 0, 0 },
    { 0x157cd442, 98 },
    { 0x910cd443, 61 },
    { 0, 0 },
    { 0xf2bcd445, 43 },
    { 0x3c1cd445, 61 },
    { 0xececd447, 34 },
    { 0x8cfcd447, 33 },
    { 0, 0 },
    { 0xc2ccd44a, 55 },
    { 0x48cd44b, 100 },
    { 0, 0 },
    { 0x2fcd44d, 88 },
    { 0x12dcd44e, 61 },
    { 0xf5ccd44f, 97 },
    { 0xb9cd450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaecd453, 33 },
    { 0xab8cd454, 98 },
    { 0x230cd455, 25 },
    { 0xdafcd456, 52 },
    { 0xaf3cd455, 23 },
    { 0x29dcd456, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3cd45e, 98 },
    { 0x9ddcd45f, 76 },
    { 0, 0 },
    { 0x9e5cd461, 90 },
    { 0, 0 },
    { 0xf88cd463, 76 },
    { 0x52bcd463, 98 },
    { 0x451cd465, 32 },
    { 0, 0 },
    { 0x732cd467, 34 },
    { 0xff9cd468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05cd472, 33 },
    { 0, 0 },
    { 0xa29cd474, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x156cd477, 23 },
    { 0, 0 },
    { 0xd2acd479, 61 },
    { 0x74dcd47a, 75 },
    { 0xb3ecd47b, 33 },
    { 0, 0 },
    { 0x4b8cd47d, 23 },
    { 0xb41cd47e, 95 },
    { 0, 0 },
    { 0xc8ccd480, 46 },
    { 0xe9cd480, 34 },
    { 0xb90cd482, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedcd486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0cd489, 35 },
    { 0xb5cd489, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96cd492, 98 },
    { 0, 0 },
    { 0xbd2cd494, 95 },
    { 0x719cd495, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59cd498, 34 },
    { 0xf86cd499, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63acd49f, 33 },
    { 0, 0 },
    { 0xc2ecd4a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ccd4a5, 98 },
    { 0x833cd4a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49acd4ac, 56 },
    { 0xec3cd4ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296cd4b2, 43 },
    { 0, 0 },
    { 0xdb2cd4b4, 43 },
    { 0x151cd4b4, 91 },
    { 0x480cd4b6, 87 },
    { 0, 0 },
    { 0x998cd4b8, 88 },
    { 0, 0 },
    { 0xca1cd4ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57cd4c1, 98 },
    { 0x193cd4c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bcd4c9, 17 },
    { 0x544cd4ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cd4d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8cd4d4, 61 },
    { 0, 0 },
    { 0x889cd4d6, 95 },
    { 0x695cd4d7, 23 },
    { 0x81ecd4d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9cd4de, 95 },
    { 0, 0 },
    { 0x890cd4e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342cd4e4, 40 },
    { 0x868cd4e5, 85 },
    { 0xde2cd4e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacacd4ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fcd4f2, 55 },
    { 0xa5fcd4f3, 98 },
    { 0xb39cd4f4, 81 },
    { 0xcb0cd4f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25cd4f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fcd4fe, 90 },
    { 0x6a3cd4ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ccd502, 66 },
    { 0xfefcd503, 34 },
    { 0xd1dcd504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4cd507, 33 },
    { 0x587cd508, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaefcd50e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fcd513, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0cd516, 90 },
    { 0x8a5cd517, 55 },
    { 0x9a2cd518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccd51d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03cd52b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37cd52e, 100 },
    { 0, 0 },
    { 0x28ccd530, 90 },
    { 0xe5bcd530, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70cd541, 75 },
    { 0x7d4cd542, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x533cd54b, 90 },
    { 0xf2fcd54b, 61 },
    { 0xa21cd54c, 78 },
    { 0xe40cd54b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x751cd551, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58cd554, 84 },
    { 0x5d3cd555, 95 },
    { 0, 0 },
    { 0xae7cd557, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbccd55c, 61 },
    { 0x7edcd55c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1cd561, 23 },
    { 0x407cd562, 90 },
    { 0x4a6cd563, 95 },
    { 0x174cd561, 95 },
    { 0xb56cd565, 98 },
    { 0xdfcd566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x614cd569, 95 },
    { 0x8a5cd56a, 90 },
    { 0, 0 },
    { 0xf66cd56c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceacd570, 85 },
    { 0x98dcd570, 87 },
    { 0x30dcd571, 46 },
    { 0, 0 },
    { 0x7c8cd574, 43 },
    { 0x851cd574, 34 },
    { 0xd66cd574, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6cd582, 87 },
    { 0x6e1cd582, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x889cd586, 91 },
    { 0x424cd586, 43 },
    { 0xd71cd588, 75 },
    { 0, 0 },
    { 0x22ecd58a, 90 },
    { 0xb43cd58b, 91 },
    { 0, 0 },
    { 0xe37cd58d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68cd590, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bcd597, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82cd5a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbcd5ae, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5cd5b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8cd5bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfcd5bf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1cd5c7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ecd5cc, 77 },
    { 0, 0 },
    { 0xcb2cd5ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbcd5d3, 23 },
    { 0x3dcd5d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31acd5d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ecd5dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58cd5e2, 81 },
    { 0x8a6cd5e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310cd5ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805cd5f4, 95 },
    { 0x553cd5f4, 81 },
    { 0, 0 },
    { 0xc94cd5f7, 34 },
    { 0xc89cd5f8, 99 },
    { 0xf54cd5f9, 17 },
    { 0x90cd5f9, 90 },
    { 0, 0 },
    { 0x4eacd5fc, 55 },
    { 0, 0 },
    { 0xd85cd5fe, 17 },
    { 0, 0 },
    { 0x36ecd600, 86 },
    { 0, 0 },
    { 0x7d8cd602, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492cd606, 98 },
    { 0x9b3cd607, 55 },
    { 0x990cd608, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfcd60c, 52 },
    { 0x928cd60d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dccd610, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xececd615, 46 },
    { 0xc6acd616, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dcd619, 90 },
    { 0x118cd619, 95 },
    { 0xefdcd61b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccd620, 95 },
    { 0x1aecd621, 91 },
    { 0, 0 },
    { 0xc74cd623, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dcd628, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cd62d, 23 },
    { 0xc89cd62d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630cd633, 46 },
    { 0x142cd633, 23 },
    { 0x8f2cd634, 61 },
    { 0xf98cd636, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcd639, 61 },
    { 0xb55cd63a, 61 },
    { 0x8f0cd63b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x751cd63e, 61 },
    { 0, 0 },
    { 0x203cd640, 23 },
    { 0x3dcd640, 100 },
    { 0xd9fcd641, 93 },
    { 0x4ebcd643, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600cd647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307cd64f, 43 },
    { 0x216cd64f, 52 },
    { 0x271cd650, 61 },
    { 0xc7bcd64f, 38 },
    { 0x3f6cd653, 90 },
    { 0x8b8cd654, 61 },
    { 0x650cd654, 95 },
    { 0x661cd656, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58cd65e, 23 },
    { 0x48fcd65f, 66 },
    { 0, 0 },
    { 0x6f4cd661, 46 },
    { 0x430cd661, 66 },
    { 0x127cd662, 61 },
    { 0x424cd663, 99 },
    { 0, 0 },
    { 0xe9fcd666, 61 },
    { 0x212cd667, 55 },
    { 0x2e6cd668, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcd66b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccd670, 52 },
    { 0xb40cd671, 99 },
    { 0x12acd670, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcd67e, 98 },
    { 0xa15cd67f, 65 },
    { 0, 0 },
    { 0xc70cd681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413cd687, 33 },
    { 0x637cd688, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75cd68b, 55 },
    { 0, 0 },
    { 0x376cd68d, 61 },
    { 0x20bcd68e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4cd692, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aacd696, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28acd699, 66 },
    { 0x8afcd699, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6cd69f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474cd6a4, 90 },
    { 0, 0 },
    { 0x5cbcd6a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffecd6ac, 61 },
    { 0x9a5cd6ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1cd6b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624cd6b5, 61 },
    { 0xb6fcd6b6, 87 },
    { 0x16ecd6b6, 76 },
    { 0, 0 },
    { 0xfe6cd6b9, 90 },
    { 0xee9cd6ba, 99 },
    { 0xe60cd6bb, 34 },
    { 0xc87cd6bc, 99 },
    { 0x5c5cd6bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbcd6c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9cd6c7, 95 },
    { 0x2cacd6c8, 23 },
    { 0x947cd6c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838cd6cf, 76 },
    { 0x32ecd6d0, 95 },
    { 0, 0 },
    { 0xa8acd6d2, 61 },
    { 0x12dcd6d2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7cd6db, 23 },
    { 0x69dcd6db, 98 },
    { 0x18ecd6db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152cd6e9, 90 },
    { 0x526cd6e9, 90 },
    { 0x625cd6ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x593cd6ee, 95 },
    { 0x499cd6ef, 61 },
    { 0x975cd6ef, 85 },
    { 0, 0 },
    { 0x947cd6f2, 61 },
    { 0xe4acd6f3, 98 },
    { 0xddcd6f3, 98 },
    { 0x467cd6f5, 46 },
    { 0x140cd6f4, 46 },
    { 0xaf1cd6f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3cd6fb, 90 },
    { 0xb44cd6fc, 33 },
    { 0xf51cd6fd, 46 },
    { 0x7eacd6fd, 95 },
    { 0xcf3cd6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36cd709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20cd70d, 95 },
    { 0x844cd70e, 76 },
    { 0xb4acd70f, 98 },
    { 0xa6ecd710, 90 },
    { 0xa35cd70e, 65 },
    { 0, 0 },
    { 0x1b1cd713, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5cd718, 93 },
    { 0x42ecd718, 33 },
    { 0, 0 },
    { 0xbfcd71b, 61 },
    { 0xb12cd71b, 23 },
    { 0x9fccd71c, 34 },
    { 0xb0fcd71d, 98 },
    { 0xa06cd71b, 75 },
    { 0xbbfcd720, 61 },
    { 0xb77cd720, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bcd731, 52 },
    { 0, 0 },
    { 0xba7cd733, 93 },
    { 0x9becd734, 95 },
    { 0x67ecd733, 100 },
    { 0x47acd736, 22 },
    { 0x56bcd736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ecd73d, 66 },
    { 0x472cd73d, 55 },
    { 0xad2cd73d, 98 },
    { 0, 0 },
    { 0x7f9cd741, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5cd744, 67 },
    { 0xc6bcd745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8cd749, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3cd74f, 91 },
    { 0, 0 },
    { 0x62ecd751, 93 },
    { 0xae5cd752, 76 },
    { 0xec3cd752, 90 },
    { 0x492cd754, 33 },
    { 0x30cd755, 46 },
    { 0, 0 },
    { 0x2aacd757, 98 },
    { 0, 0 },
    { 0xf5cd759, 23 },
    { 0, 0 },
    { 0xb03cd75b, 48 },
    { 0xc65cd75c, 66 },
    { 0x92bcd75d, 33 },
    { 0, 0 },
    { 0x129cd75f, 33 },
    { 0xa7dcd760, 33 },
    { 0x7cdcd75f, 95 },
    { 0xedecd762, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2cd767, 55 },
    { 0, 0 },
    { 0x6bbcd769, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847cd772, 100 },
    { 0x7a0cd772, 55 },
    { 0xa94cd774, 61 },
    { 0xa02cd775, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efcd780, 69 },
    { 0, 0 },
    { 0x2f8cd782, 48 },
    { 0x241cd783, 80 },
    { 0xc73cd783, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745cd788, 93 },
    { 0x97acd788, 61 },
    { 0x1dacd789, 33 },
    { 0xafcd78b, 33 },
    { 0x794cd78c, 61 },
    { 0x566cd788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fcd790, 66 },
    { 0x1adcd790, 90 },
    { 0x400cd792, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcd795, 85 },
    { 0xf43cd796, 90 },
    { 0x782cd797, 33 },
    { 0xe89cd795, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22cd7a3, 95 },
    { 0xffcd7a3, 61 },
    { 0x313cd7a5, 66 },
    { 0x829cd7a4, 90 },
    { 0xc24cd7a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x356cd7aa, 23 },
    { 0x9b1cd7aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8cd7b0, 98 },
    { 0, 0 },
    { 0x4e4cd7b2, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92cd7bd, 33 },
    { 0, 0 },
    { 0xecd7bf, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7cd7c5, 98 },
    { 0xaabcd7c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbcd7c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60cd7cf, 66 },
    { 0xb22cd7d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbcd7d3, 55 },
    { 0xce7cd7d3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffcd7db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8decd7e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8cd7e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x358cd7e9, 33 },
    { 0xce8cd7ea, 23 },
    { 0, 0 },
    { 0xa4bcd7ec, 55 },
    { 0xd53cd7ec, 43 },
    { 0xc0dcd7ec, 76 },
    { 0, 0 },
    { 0x4acd7f0, 61 },
    { 0x7fecd7f1, 90 },
    { 0, 0 },
    { 0xc82cd7f3, 76 },
    { 0x40fcd7f4, 33 },
    { 0x773cd7f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4bcd7f9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccd801, 52 },
    { 0x1b7cd802, 61 },
    { 0, 0 },
    { 0xbcfcd804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x368cd807, 43 },
    { 0xddecd808, 64 },
    { 0xcd2cd808, 67 },
    { 0xef4cd80a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7cd80d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dcd810, 87 },
    { 0x1accd811, 34 },
    { 0x562cd811, 23 },
    { 0xe68cd813, 93 },
    { 0x566cd814, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84cd81a, 33 },
    { 0x732cd81a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd81e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26cd821, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33cd824, 99 },
    { 0, 0 },
    { 0xfc1cd826, 68 },
    { 0x3aecd827, 33 },
    { 0xa42cd828, 85 },
    { 0x83ccd827, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd82d, 90 },
    { 0x88bcd82e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6acd831, 90 },
    { 0xfc7cd832, 46 },
    { 0xb00cd833, 33 },
    { 0x109cd833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfecd838, 90 },
    { 0x146cd838, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcd83c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x581cd83f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831cd844, 84 },
    { 0xbd7cd844, 33 },
    { 0, 0 },
    { 0x358cd847, 61 },
    { 0xdadcd848, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955cd84c, 90 },
    { 0, 0 },
    { 0x5eecd84e, 61 },
    { 0x674cd84e, 23 },
    { 0xcc5cd84e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447cd858, 23 },
    { 0xa72cd858, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd85c, 61 },
    { 0x78bcd85d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adcd863, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0cd86c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9cd870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8cd87a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47cd87d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfcd882, 90 },
    { 0xf42cd883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dcd886, 98 },
    { 0, 0 },
    { 0xf56cd888, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbcd88b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fcd88f, 43 },
    { 0, 0 },
    { 0xb4ecd891, 78 },
    { 0xf96cd892, 23 },
    { 0x586cd893, 33 },
    { 0, 0 },
    { 0x272cd895, 99 },
    { 0x5c6cd896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x145cd899, 98 },
    { 0x7ffcd899, 23 },
    { 0, 0 },
    { 0x31acd89c, 95 },
    { 0, 0 },
    { 0x62ccd89e, 38 },
    { 0x844cd89f, 98 },
    { 0, 0 },
    { 0x303cd8a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bcd8a5, 95 },
    { 0, 0 },
    { 0x9becd8a7, 23 },
    { 0x54dcd8a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14cd8ac, 95 },
    { 0, 0 },
    { 0xfb9cd8ae, 84 },
    { 0xdaacd8ae, 98 },
    { 0x590cd8ae, 87 },
    { 0xb39cd8b1, 81 },
    { 0, 0 },
    { 0x4cacd8b3, 55 },
    { 0xd20cd8b3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949cd8c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x442cd8c6, 33 },
    { 0x8d3cd8c7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14cd8d1, 23 },
    { 0, 0 },
    { 0xe36cd8d3, 23 },
    { 0xee3cd8d4, 69 },
    { 0, 0 },
    { 0x879cd8d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fcd8db, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cd8de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2cd8e2, 90 },
    { 0, 0 },
    { 0x5eecd8e4, 33 },
    { 0, 0 },
    { 0x7fbcd8e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaccd8ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2cd8f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bcd8f7, 23 },
    { 0x3e1cd8f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495cd900, 61 },
    { 0, 0 },
    { 0xce6cd902, 99 },
    { 0x387cd903, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abcd907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ecd90a, 61 },
    { 0x5b3cd90a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcd90f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dcd912, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ccd915, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75cd919, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x274cd91c, 33 },
    { 0xc15cd91d, 90 },
    { 0x5facd91e, 90 },
    { 0, 0 },
    { 0xfe8cd920, 95 },
    { 0xc7fcd921, 76 },
    { 0xd02cd921, 95 },
    { 0x6abcd923, 78 },
    { 0xb39cd923, 34 },
    { 0x90bcd925, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883cd929, 38 },
    { 0xfa4cd929, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bcd931, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abcd938, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38cd93f, 33 },
    { 0xbfbcd940, 98 },
    { 0xe2cd941, 34 },
    { 0, 0 },
    { 0x7ddcd943, 72 },
    { 0x3facd944, 61 },
    { 0xa4acd943, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5cd948, 61 },
    { 0x500cd949, 95 },
    { 0, 0 },
    { 0x680cd94b, 78 },
    { 0xc6acd94b, 61 },
    { 0x34bcd94d, 100 },
    { 0xa7cd94e, 61 },
    { 0x6d6cd94e, 98 },
    { 0x28ecd950, 95 },
    { 0x987cd951, 61 },
    { 0xf47cd952, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46cd957, 33 },
    { 0x505cd958, 23 },
    { 0xce2cd957, 43 },
    { 0x900cd95a, 23 },
    { 0, 0 },
    { 0x759cd95c, 23 },
    { 0, 0 },
    { 0xc20cd95e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bcd963, 38 },
    { 0xe8cd964, 76 },
    { 0xddccd965, 77 },
    { 0, 0 },
    { 0xe8cd967, 98 },
    { 0x305cd968, 84 },
    { 0x8c5cd969, 98 },
    { 0xde5cd96a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6cd96f, 98 },
    { 0, 0 },
    { 0x7ffcd971, 55 },
    { 0x79bcd971, 61 },
    { 0, 0 },
    { 0x163cd974, 61 },
    { 0x4bccd975, 95 },
    { 0xe95cd975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x71acd979, 90 },
    { 0, 0 },
    { 0xfe3cd97b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20cd981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dcd984, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75cd988, 43 },
    { 0, 0 },
    { 0x5f1cd98a, 93 },
    { 0x22dcd98b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaecd991, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41cd995, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x571cd998, 34 },
    { 0, 0 },
    { 0x791cd99a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48cd99d, 44 },
    { 0, 0 },
    { 0xe3ccd99f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x919cd9a2, 66 },
    { 0, 0 },
    { 0xe6ecd9a4, 90 },
    { 0, 0 },
    { 0x9c8cd9a6, 100 },
    { 0xdb6cd9a7, 87 },
    { 0, 0 },
    { 0x21acd9a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9cd9af, 84 },
    { 0, 0 },
    { 0x8fccd9b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ecd9b4, 55 },
    { 0, 0 },
    { 0xf3dcd9b6, 87 },
    { 0x3bfcd9b7, 34 },
    { 0x85bcd9b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27cd9bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68acd9c4, 43 },
    { 0, 0 },
    { 0x888cd9c6, 90 },
    { 0xb37cd9c6, 33 },
    { 0x896cd9c7, 61 },
    { 0, 0 },
    { 0xebfcd9ca, 85 },
    { 0, 0 },
    { 0x52ccd9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564cd9d0, 61 },
    { 0, 0 },
    { 0xd49cd9d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cd9d5, 85 },
    { 0x40bcd9d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebcd9d9, 61 },
    { 0x1ebcd9da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953cd9de, 33 },
    { 0xa76cd9df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09cd9e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44cd9e7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x768cd9ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5cd9ed, 23 },
    { 0x15acd9ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3cd9f5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cd9fa, 93 },
    { 0xc81cd9fb, 74 },
    { 0, 0 },
    { 0xc7ccd9fd, 66 },
    { 0xa17cd9fd, 93 },
    { 0x7bccd9ff, 33 },
    { 0x2a8cd9fd, 61 },
    { 0, 0 },
    { 0xcabcda02, 61 },
    { 0x3a5cda03, 51 },
    { 0x809cda03, 98 },
    { 0xf5ecda05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfcda0a, 61 },
    { 0x309cda0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfcda0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54cda11, 55 },
    { 0, 0 },
    { 0xdfbcda13, 61 },
    { 0xccda14, 61 },
    { 0, 0 },
    { 0xe7bcda16, 95 },
    { 0, 0 },
    { 0x9fecda18, 33 },
    { 0, 0 },
    { 0x9a7cda1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800cda1e, 77 },
    { 0xebbcda1f, 55 },
    { 0, 0 },
    { 0x227cda21, 46 },
    { 0, 0 },
    { 0x1e0cda23, 23 },
    { 0x609cda23, 61 },
    { 0x5bfcda23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dcda28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x176cda2b, 43 },
    { 0xd70cda2c, 61 },
    { 0x1d7cda2d, 23 },
    { 0, 0 },
    { 0xe56cda2f, 77 },
    { 0x9d1cda2f, 34 },
    { 0xfbbcda31, 33 },
    { 0xdf5cda31, 61 },
    { 0x7e6cda33, 61 },
    { 0x62ccda34, 90 },
    { 0x7d6cda34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5cda38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dcda3f, 76 },
    { 0, 0 },
    { 0xc5dcda41, 24 },
    { 0, 0 },
    { 0xe80cda43, 80 },
    { 0, 0 },
    { 0x3dbcda45, 95 },
    { 0xc2bcda46, 76 },
    { 0x54bcda46, 83 },
    { 0x534cda46, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bcda4c, 61 },
    { 0xd35cda4d, 23 },
    { 0, 0 },
    { 0x8fdcda4f, 98 },
    { 0x897cda50, 36 },
    { 0, 0 },
    { 0xf22cda52, 33 },
    { 0xeb0cda53, 61 },
    { 0x70cda53, 90 },
    { 0xd63cda55, 61 },
    { 0, 0 },
    { 0x6c4cda57, 66 },
    { 0x2a2cda58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcda5c, 90 },
    { 0x1c3cda5c, 23 },
    { 0xd4ecda5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2cda61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bcda66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809cda6d, 52 },
    { 0, 0 },
    { 0x9cfcda6f, 90 },
    { 0x2cfcda70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08cda73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415cda7d, 90 },
    { 0xcecda7d, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bcda82, 33 },
    { 0xdaccda82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2cda8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11cda8e, 98 },
    { 0, 0 },
    { 0xa4bcda90, 37 },
    { 0x8e3cda91, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6cda98, 23 },
    { 0xdd6cda99, 23 },
    { 0x711cda98, 93 },
    { 0x7cfcda98, 98 },
    { 0, 0 },
    { 0xadbcda9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2cdaa3, 98 },
    { 0xe15cdaa3, 76 },
    { 0, 0 },
    { 0xbe7cdaa6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81cdaac, 61 },
    { 0, 0 },
    { 0xc51cdaae, 33 },
    { 0, 0 },
    { 0xecdab0, 98 },
    { 0x364cdab1, 90 },
    { 0x541cdab1, 61 },
    { 0xfc1cdab3, 55 },
    { 0x29ccdab4, 88 },
    { 0x9f8cdab5, 90 },
    { 0, 0 },
    { 0xb4fcdab7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bcdac3, 90 },
    { 0xd9bcdac3, 90 },
    { 0xb03cdac5, 74 },
    { 0, 0 },
    { 0xa9fcdac7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x255cdaca, 55 },
    { 0x155cdacb, 44 },
    { 0x5eacdacb, 100 },
    { 0x75fcdacd, 33 },
    { 0, 0 },
    { 0xb6cdacf, 98 },
    { 0, 0 },
    { 0xfadcdad1, 33 },
    { 0x872cdad2, 100 },
    { 0x2c0cdad2, 85 },
    { 0, 0 },
    { 0xbcfcdad5, 46 },
    { 0xc24cdad6, 84 },
    { 0x73ccdad6, 100 },
    { 0x42cdad8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42cdadc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdae6, 90 },
    { 0x612cdae7, 66 },
    { 0x15fcdae7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473cdaec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3cdaf7, 34 },
    { 0x4cccdaf8, 78 },
    { 0x8cacdaf8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cdaff, 81 },
    { 0x4f6cdb00, 91 },
    { 0xc87cdb00, 66 },
    { 0x1ecdb01, 93 },
    { 0x635cdb03, 76 },
    { 0xa25cdb04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ccdb07, 61 },
    { 0x9cdb08, 33 },
    { 0x68bcdb09, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ccdb10, 61 },
    { 0xbfdcdb10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644cdb1b, 76 },
    { 0, 0 },
    { 0xc32cdb1d, 100 },
    { 0xf0bcdb1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ecdb25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ccdb28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67cdb2f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edcdb32, 61 },
    { 0x370cdb33, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cdb3a, 95 },
    { 0xc97cdb3b, 61 },
    { 0, 0 },
    { 0x3d4cdb3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2cdb41, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ecdb44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2cdb47, 23 },
    { 0xf66cdb47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903cdb51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508cdb58, 33 },
    { 0xffecdb59, 55 },
    { 0, 0 },
    { 0xdf6cdb5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x715cdb5e, 100 },
    { 0, 0 },
    { 0xbeccdb60, 87 },
    { 0xa4acdb60, 33 },
    { 0x13ecdb60, 81 },
    { 0x9e1cdb63, 43 },
    { 0xea0cdb63, 90 },
    { 0x61acdb65, 33 },
    { 0x541cdb66, 52 },
    { 0x80acdb66, 33 },
    { 0x733cdb67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcdb6d, 55 },
    { 0xf4acdb6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1cdb71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8cdb75, 33 },
    { 0, 0 },
    { 0xccfcdb77, 90 },
    { 0, 0 },
    { 0x380cdb79, 95 },
    { 0x7ddcdb7a, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8cdb7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21cdb83, 86 },
    { 0, 0 },
    { 0xcd6cdb85, 61 },
    { 0x4c5cdb85, 43 },
    { 0xc3fcdb87, 52 },
    { 0x3b3cdb88, 100 },
    { 0x144cdb89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x769cdb8c, 66 },
    { 0xb3fcdb8c, 23 },
    { 0x77ecdb8c, 34 },
    { 0x7d0cdb8f, 66 },
    { 0xcc9cdb8f, 33 },
    { 0xbc3cdb90, 95 },
    { 0x55cdb92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32cdb97, 90 },
    { 0xb12cdb98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x361cdb9b, 99 },
    { 0x95dcdb9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93cdba1, 33 },
    { 0x9abcdba2, 90 },
    { 0x744cdba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84cdbac, 55 },
    { 0xcfdcdbad, 55 },
    { 0, 0 },
    { 0x456cdbaf, 61 },
    { 0xea5cdbb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3cdbb3, 90 },
    { 0xd0fcdbb4, 88 },
    { 0xf86cdbb5, 90 },
    { 0x1a5cdbb6, 84 },
    { 0x39ccdbb4, 46 },
    { 0x74ccdbb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6cdbc7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924cdbcd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08cdbd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382cdbe0, 23 },
    { 0xa7acdbe0, 33 },
    { 0xef7cdbe2, 90 },
    { 0x421cdbe3, 61 },
    { 0x782cdbe2, 69 },
    { 0xc5bcdbe5, 17 },
    { 0x7b8cdbe1, 76 },
    { 0xaebcdbe4, 80 },
    { 0x821cdbe8, 91 },
    { 0x1b5cdbe9, 34 },
    { 0, 0 },
    { 0x6eacdbeb, 40 },
    { 0x90ccdbec, 91 },
    { 0, 0 },
    { 0x131cdbee, 87 },
    { 0x9e8cdbef, 75 },
    { 0x55ccdbef, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9cdbf4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60cdbfa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620cdbfe, 46 },
    { 0xba5cdbfe, 34 },
    { 0, 0 },
    { 0xe38cdc01, 93 },
    { 0, 0 },
    { 0xb94cdc03, 61 },
    { 0x177cdc03, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2cdc0c, 98 },
    { 0x5f7cdc0c, 55 },
    { 0, 0 },
    { 0xbe2cdc0f, 67 },
    { 0x3a6cdc10, 33 },
    { 0x89acdc11, 91 },
    { 0x8b1cdc11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x403cdc15, 61 },
    { 0xffcdc16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x630cdc19, 90 },
    { 0xea2cdc1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aacdc1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9cdc21, 61 },
    { 0xe90cdc21, 33 },
    { 0, 0 },
    { 0xd1acdc24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63cdc27, 22 },
    { 0x989cdc28, 95 },
    { 0, 0 },
    { 0x799cdc2a, 61 },
    { 0xadccdc2b, 84 },
    { 0xa6fcdc2c, 34 },
    { 0x2bfcdc2b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ccdc35, 90 },
    { 0, 0 },
    { 0x42acdc37, 23 },
    { 0, 0 },
    { 0x186cdc39, 61 },
    { 0xf5dcdc3a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cdc44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3cdc49, 66 },
    { 0xea5cdc49, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccdc52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ecdc59, 99 },
    { 0x1dbcdc5a, 84 },
    { 0x4b7cdc5b, 76 },
    { 0xd5ccdc59, 46 },
    { 0x350cdc5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ccdc63, 46 },
    { 0, 0 },
    { 0xfb8cdc65, 33 },
    { 0, 0 },
    { 0xa30cdc67, 33 },
    { 0xfdfcdc68, 75 },
    { 0x2a0cdc67, 33 },
    { 0xbeacdc6a, 80 },
    { 0x50ccdc6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bcdc6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3cdc71, 90 },
    { 0x7a5cdc71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19cdc75, 83 },
    { 0x1dbcdc76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ecdc79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105cdc7f, 23 },
    { 0x428cdc7f, 33 },
    { 0x3fbcdc81, 61 },
    { 0xab0cdc80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cdc88, 43 },
    { 0, 0 },
    { 0xb53cdc8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8cdc8d, 95 },
    { 0, 0 },
    { 0x83ecdc8f, 61 },
    { 0x9cfcdc8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ecdc9e, 91 },
    { 0, 0 },
    { 0x8dccdca0, 61 },
    { 0x518cdca0, 33 },
    { 0x47acdca2, 84 },
    { 0xba5cdca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcdca7, 90 },
    { 0x424cdca8, 55 },
    { 0xd01cdca8, 55 },
    { 0x735cdcaa, 61 },
    { 0xd4dcdcab, 56 },
    { 0xcf7cdcab, 55 },
    { 0xf68cdcad, 95 },
    { 0x2ddcdcae, 90 },
    { 0xa22cdcad, 52 },
    { 0xe40cdca8, 23 },
    { 0xbbdcdcb0, 33 },
    { 0x6c7cdcb2, 81 },
    { 0x949cdcb2, 95 },
    { 0xeeccdcb4, 76 },
    { 0xc6ccdcb1, 43 },
    { 0, 0 },
    { 0xf70cdcb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ccdcbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7cdcc0, 88 },
    { 0xf73cdcc1, 33 },
    { 0x95ecdcc2, 23 },
    { 0x1a0cdcc3, 90 },
    { 0xceacdcc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x710cdcc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dacdccd, 61 },
    { 0x59cdccd, 90 },
    { 0xfe2cdcce, 90 },
    { 0, 0 },
    { 0x6e9cdcd1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83acdcd5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bcdcd9, 55 },
    { 0xa15cdcd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18cdce0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ecdce7, 90 },
    { 0, 0 },
    { 0x19ecdce9, 61 },
    { 0, 0 },
    { 0x7fbcdceb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54cdcee, 95 },
    { 0x54bcdcee, 33 },
    { 0, 0 },
    { 0xb3bcdcf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562cdcf5, 98 },
    { 0xb16cdcf6, 95 },
    { 0, 0 },
    { 0xaf1cdcf8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909cdcfc, 93 },
    { 0xd01cdcfd, 91 },
    { 0, 0 },
    { 0x283cdcff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9cdd03, 55 },
    { 0xfe8cdd04, 78 },
    { 0x1a3cdd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cdd08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42cdd0f, 78 },
    { 0x896cdd10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222cdd17, 83 },
    { 0, 0 },
    { 0xa42cdd19, 95 },
    { 0x92acdd1a, 90 },
    { 0x434cdd1b, 52 },
    { 0x432cdd1c, 91 },
    { 0xf4ccdd1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45cdd24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871cdd2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62cdd33, 34 },
    { 0xb5bcdd34, 88 },
    { 0x417cdd35, 98 },
    { 0x7e3cdd35, 90 },
    { 0xa40cdd37, 99 },
    { 0x6fcdd38, 95 },
    { 0xa8ecdd38, 33 },
    { 0, 0 },
    { 0xd9ecdd3b, 90 },
    { 0xf32cdd3c, 33 },
    { 0x1a4cdd3d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x324cdd40, 46 },
    { 0, 0 },
    { 0x12dcdd42, 78 },
    { 0x27fcdd43, 90 },
    { 0, 0 },
    { 0x39ccdd45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x859cdd48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ccdd4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6cdd50, 76 },
    { 0xc7bcdd51, 91 },
    { 0xdc7cdd52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ecdd55, 33 },
    { 0, 0 },
    { 0x452cdd57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcdd5b, 34 },
    { 0, 0 },
    { 0xba1cdd5d, 90 },
    { 0x273cdd5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ecdd62, 43 },
    { 0x2abcdd63, 61 },
    { 0x574cdd64, 33 },
    { 0xd5cdd65, 84 },
    { 0xff6cdd66, 90 },
    { 0x541cdd65, 33 },
    { 0xf85cdd65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7cdd6b, 33 },
    { 0x15bcdd6c, 76 },
    { 0x2fdcdd6d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8cdd79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcdd7c, 43 },
    { 0, 0 },
    { 0xbb1cdd7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1facdd85, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80cdd89, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23cdd90, 98 },
    { 0x4e8cdd91, 81 },
    { 0x88cdd91, 43 },
    { 0xf5ccdd93, 90 },
    { 0x9dccdd92, 51 },
    { 0, 0 },
    { 0xd40cdd96, 95 },
    { 0, 0 },
    { 0x605cdd98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaecdd9e, 95 },
    { 0x20acdd9f, 95 },
    { 0xdfcdda0, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcdda4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ccddaa, 23 },
    { 0x169cddaa, 34 },
    { 0x332cddac, 52 },
    { 0x6bccddab, 87 },
    { 0x378cddab, 93 },
    { 0x825cddaf, 76 },
    { 0x5efcddaf, 98 },
    { 0x7bbcddb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90cddb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4cddc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8cddcd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45cddd0, 23 },
    { 0, 0 },
    { 0x77fcddd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3cddd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dcdddb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00cdddf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x893cdde4, 77 },
    { 0xaf7cdde5, 61 },
    { 0, 0 },
    { 0xbc8cdde7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bcddea, 66 },
    { 0, 0 },
    { 0xaf7cddec, 55 },
    { 0, 0 },
    { 0x2dccddee, 65 },
    { 0x32acddef, 98 },
    { 0x355cddef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960cddf5, 52 },
    { 0xc6dcddf6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121cddf9, 84 },
    { 0xf44cddfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954cde01, 68 },
    { 0xf48cde02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2cde05, 95 },
    { 0xc1acde06, 90 },
    { 0, 0 },
    { 0x47acde08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8cde0b, 67 },
    { 0xde0cde0c, 33 },
    { 0x884cde0b, 61 },
    { 0x16bcde0c, 65 },
    { 0x419cde0f, 85 },
    { 0x4a1cde10, 34 },
    { 0xfe8cde11, 95 },
    { 0x5c2cde12, 52 },
    { 0xa1cde13, 94 },
    { 0x2e4cde13, 68 },
    { 0x21dcde13, 78 },
    { 0xf68cde15, 84 },
    { 0x8bcde14, 75 },
    { 0, 0 },
    { 0xe70cde19, 76 },
    { 0xfc4cde1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6cde22, 90 },
    { 0x829cde22, 90 },
    { 0, 0 },
    { 0xd8acde25, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41cde2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388cde2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16cde37, 52 },
    { 0x448cde37, 90 },
    { 0, 0 },
    { 0xa3acde3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fcde43, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccde46, 90 },
    { 0, 0 },
    { 0x7afcde48, 23 },
    { 0, 0 },
    { 0x985cde4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1cde4d, 87 },
    { 0xd1cde4e, 90 },
    { 0xba8cde4f, 90 },
    { 0xd8bcde50, 34 },
    { 0x153cde51, 80 },
    { 0x1fecde4e, 52 },
    { 0x8b4cde4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82cde57, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aacde5a, 98 },
    { 0, 0 },
    { 0x5e8cde5c, 34 },
    { 0xa3cde5d, 33 },
    { 0x68cde5e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bcde61, 61 },
    { 0x28acde62, 78 },
    { 0x20cde63, 23 },
    { 0x1dfcde64, 100 },
    { 0x1bbcde65, 61 },
    { 0x297cde65, 33 },
    { 0, 0 },
    { 0xb48cde68, 84 },
    { 0, 0 },
    { 0x11bcde6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3cde6e, 96 },
    { 0x547cde6f, 34 },
    { 0xcdbcde70, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ccde78, 90 },
    { 0x618cde78, 77 },
    { 0xb4ecde7a, 90 },
    { 0, 0 },
    { 0x67bcde7c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cde83, 96 },
    { 0, 0 },
    { 0x1decde85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92bcde89, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4cde8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4cde91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6cde94, 95 },
    { 0x2fcde95, 33 },
    { 0xc5ecde96, 90 },
    { 0xe53cde97, 61 },
    { 0x3f6cde97, 91 },
    { 0xf9ccde99, 98 },
    { 0x6cdcde98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaecde9d, 33 },
    { 0x1bacde9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbcdea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110cdea6, 23 },
    { 0x7ebcdea6, 95 },
    { 0x40dcdea8, 46 },
    { 0x414cdea8, 90 },
    { 0, 0 },
    { 0xecdcdeab, 23 },
    { 0x2a9cdeac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05cdeb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x446cdeb7, 95 },
    { 0xf6dcdeb8, 43 },
    { 0xd37cdeb9, 78 },
    { 0, 0 },
    { 0x11bcdebb, 75 },
    { 0xb30cdebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fcdec0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23cdec3, 33 },
    { 0x9d5cdec3, 33 },
    { 0xfd9cdec5, 34 },
    { 0x715cdec4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ccded0, 90 },
    { 0, 0 },
    { 0xf58cded2, 33 },
    { 0xb39cded2, 37 },
    { 0, 0 },
    { 0x7a4cded5, 90 },
    { 0, 0 },
    { 0x91ecded7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6cdeda, 23 },
    { 0, 0 },
    { 0x77dcdedc, 91 },
    { 0xb52cdedd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x823cdee0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eecdee4, 90 },
    { 0, 0 },
    { 0x28fcdee6, 33 },
    { 0xef8cdee7, 99 },
    { 0, 0 },
    { 0x58bcdee9, 76 },
    { 0xd24cdeea, 77 },
    { 0, 0 },
    { 0x8fbcdeec, 61 },
    { 0, 0 },
    { 0x925cdeee, 23 },
    { 0xfb7cdeef, 90 },
    { 0xfacdef0, 46 },
    { 0, 0 },
    { 0x41cdef2, 76 },
    { 0x927cdef3, 91 },
    { 0x6fccdef3, 61 },
    { 0, 0 },
    { 0x9f7cdef6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524cdefd, 34 },
    { 0xca1cdefd, 43 },
    { 0x213cdefd, 34 },
    { 0xd1acdefd, 61 },
    { 0x35dcdf00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192cdf07, 40 },
    { 0, 0 },
    { 0xe99cdf09, 33 },
    { 0, 0 },
    { 0xd41cdf0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cacdf0f, 95 },
    { 0, 0 },
    { 0x480cdf11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5cdf16, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecccdf1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2cdf22, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7cdf29, 90 },
    { 0x2dacdf2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6cdf2f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11acdf33, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1cdf36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ecdf4c, 33 },
    { 0x4a3cdf4d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ccdf50, 67 },
    { 0, 0 },
    { 0xf99cdf52, 61 },
    { 0, 0 },
    { 0x211cdf54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7cdf59, 90 },
    { 0x18bcdf5a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ccdf5e, 87 },
    { 0, 0 },
    { 0x358cdf60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5cdf65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41acdf6f, 93 },
    { 0x98dcdf70, 81 },
    { 0xa07cdf71, 99 },
    { 0xcf2cdf72, 95 },
    { 0xb62cdf70, 33 },
    { 0x5e6cdf74, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36cdf77, 77 },
    { 0, 0 },
    { 0xecfcdf79, 36 },
    { 0, 0 },
    { 0x42dcdf7b, 55 },
    { 0, 0 },
    { 0x21fcdf7d, 91 },
    { 0, 0 },
    { 0xa34cdf7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dcdf83, 90 },
    { 0xf54cdf83, 81 },
    { 0x616cdf84, 98 },
    { 0x881cdf86, 90 },
    { 0x7cdcdf87, 93 },
    { 0, 0 },
    { 0x977cdf89, 66 },
    { 0xd12cdf8a, 43 },
    { 0xfe1cdf8a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbcdf90, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4cdf94, 34 },
    { 0, 0 },
    { 0x4d9cdf96, 33 },
    { 0xc3acdf96, 89 },
    { 0, 0 },
    { 0xc46cdf99, 90 },
    { 0, 0 },
    { 0x4b1cdf9b, 33 },
    { 0xe2acdf9c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x761cdf9f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fcdfa2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fcdfa6, 34 },
    { 0x297cdfa7, 66 },
    { 0x530cdfa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bcdfab, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acdfae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7cdfb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662cdfbe, 65 },
    { 0, 0 },
    { 0x6f6cdfc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bcdfc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bcdfce, 61 },
    { 0x548cdfcf, 33 },
    { 0, 0 },
    { 0x763cdfd1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecdfe0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ccdfe4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20cdfe7, 33 },
    { 0xbedcdfe8, 68 },
    { 0x30acdfe7, 84 },
    { 0xda2cdfe9, 33 },
    { 0x1f6cdfeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3cdfee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4cdff1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ccdff5, 55 },
    { 0x9accdff5, 33 },
    { 0x95ccdff6, 65 },
    { 0xa13cdff8, 98 },
    { 0x155cdff9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x32acdffc, 52 },
    { 0xe2ecdffc, 98 },
    { 0x341cdffe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843ce005, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ace00d, 76 },
    { 0x2fcce00e, 40 },
    { 0xe9dce00d, 59 },
    { 0xea8ce010, 90 },
    { 0xab9ce010, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x777ce014, 88 },
    { 0xa37ce015, 93 },
    { 0x9cfce016, 23 },
    { 0xa1bce017, 90 },
    { 0x1c2ce014, 61 },
    { 0x5ffce018, 43 },
    { 0x476ce017, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63ce01f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5ce023, 33 },
    { 0xa09ce024, 61 },
    { 0x688ce025, 46 },
    { 0xcf9ce026, 34 },
    { 0x68ece027, 68 },
    { 0x5efce024, 98 },
    { 0xce1ce029, 98 },
    { 0xcfbce029, 84 },
    { 0x854ce02a, 55 },
    { 0xdcfce02c, 33 },
    { 0x130ce029, 95 },
    { 0xcbdce02a, 90 },
    { 0, 0 },
    { 0x209ce030, 78 },
    { 0xbcece030, 90 },
    { 0x236ce030, 91 },
    { 0x388ce033, 33 },
    { 0x407ce034, 43 },
    { 0x784ce032, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14ce03b, 98 },
    { 0xc7bce03c, 76 },
    { 0x855ce03c, 95 },
    { 0xc71ce03e, 61 },
    { 0, 0 },
    { 0x5f6ce040, 98 },
    { 0x641ce041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03ce045, 33 },
    { 0x174ce046, 80 },
    { 0x918ce046, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddce04c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ece058, 65 },
    { 0, 0 },
    { 0x2a4ce05a, 55 },
    { 0x19cce05a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cce061, 90 },
    { 0x5c8ce062, 87 },
    { 0xdc9ce062, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efce068, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fce079, 93 },
    { 0x662ce079, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ce07f, 55 },
    { 0x4acce07f, 93 },
    { 0x72ace081, 76 },
    { 0x80ece082, 98 },
    { 0, 0 },
    { 0x11ece084, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdce088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559ce08c, 23 },
    { 0xb77ce08c, 91 },
    { 0x597ce08c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5ce092, 67 },
    { 0x7f7ce093, 66 },
    { 0x294ce094, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3ce098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ce09b, 76 },
    { 0x299ce09b, 61 },
    { 0xc72ce09d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504ce0a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8ce0a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccce0aa, 76 },
    { 0, 0 },
    { 0xd9ece0ac, 46 },
    { 0, 0 },
    { 0xf24ce0ae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6ce0b5, 53 },
    { 0xc2ce0b6, 33 },
    { 0, 0 },
    { 0xd5bce0b8, 65 },
    { 0x960ce0b8, 87 },
    { 0xb5ace0ba, 43 },
    { 0xbc2ce0bb, 99 },
    { 0xa6ece0ba, 55 },
    { 0x2b4ce0ba, 61 },
    { 0x647ce0bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21ce0c1, 93 },
    { 0, 0 },
    { 0x73ace0c3, 33 },
    { 0xfece0c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fce0ca, 98 },
    { 0x55bce0cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ce0cf, 23 },
    { 0, 0 },
    { 0x15ece0d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dece0d8, 69 },
    { 0xef5ce0d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542ce0dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6face0e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7ce0e3, 91 },
    { 0x9a7ce0e4, 62 },
    { 0, 0 },
    { 0xb2ace0e6, 43 },
    { 0x940ce0e6, 33 },
    { 0, 0 },
    { 0x44ce0e9, 55 },
    { 0x89ce0e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ce0ee, 98 },
    { 0, 0 },
    { 0x69ece0f0, 99 },
    { 0x82ace0f1, 77 },
    { 0x865ce0f1, 53 },
    { 0x4a3ce0f1, 98 },
    { 0, 0 },
    { 0x597ce0f5, 33 },
    { 0x3aace0f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fce0fa, 66 },
    { 0, 0 },
    { 0x386ce0fc, 34 },
    { 0, 0 },
    { 0x6b9ce0fe, 90 },
    { 0, 0 },
    { 0xa4dce100, 76 },
    { 0xb11ce100, 90 },
    { 0x8bece102, 85 },
    { 0x41bce103, 98 },
    { 0xce6ce104, 95 },
    { 0xf54ce104, 93 },
    { 0xc77ce106, 61 },
    { 0x2e0ce107, 69 },
    { 0, 0 },
    { 0x1efce109, 90 },
    { 0x183ce10a, 98 },
    { 0, 0 },
    { 0x734ce10c, 66 },
    { 0xe0bce10d, 90 },
    { 0x38bce10d, 90 },
    { 0x2a1ce10f, 34 },
    { 0x20ace110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcce113, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71ce117, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bce11b, 61 },
    { 0xc33ce11c, 55 },
    { 0, 0 },
    { 0x582ce11e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508ce122, 95 },
    { 0, 0 },
    { 0xdb9ce124, 33 },
    { 0x7bcce125, 98 },
    { 0x1d3ce126, 85 },
    { 0x87fce126, 33 },
    { 0, 0 },
    { 0x52dce129, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41ce130, 99 },
    { 0x43dce131, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x526ce134, 61 },
    { 0xb76ce135, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578ce139, 61 },
    { 0xfc8ce139, 93 },
    { 0xb5cce13a, 61 },
    { 0xb76ce139, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dce142, 33 },
    { 0xca8ce143, 90 },
    { 0x340ce144, 90 },
    { 0x6bbce145, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dce14b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adce152, 85 },
    { 0x788ce153, 81 },
    { 0x6d2ce154, 91 },
    { 0x1fcce154, 61 },
    { 0x9edce154, 67 },
    { 0, 0 },
    { 0x79dce158, 33 },
    { 0x24bce159, 78 },
    { 0x84fce15a, 99 },
    { 0x354ce15a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30ce160, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2ce164, 61 },
    { 0xacfce165, 81 },
    { 0x90ce165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9ce16a, 61 },
    { 0xe35ce16b, 95 },
    { 0x861ce16b, 87 },
    { 0xf90ce16c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cce170, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8ce177, 52 },
    { 0, 0 },
    { 0xe27ce179, 100 },
    { 0, 0 },
    { 0x335ce17b, 33 },
    { 0, 0 },
    { 0x91ce17d, 78 },
    { 0xec5ce17d, 61 },
    { 0, 0 },
    { 0xc41ce180, 99 },
    { 0x9dcce181, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ce185, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ace188, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fce18f, 95 },
    { 0, 0 },
    { 0xbebce191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731ce195, 95 },
    { 0, 0 },
    { 0x15ace197, 66 },
    { 0x539ce198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fce1a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bce1a3, 88 },
    { 0x585ce1a4, 61 },
    { 0xc20ce1a4, 90 },
    { 0x917ce1a4, 33 },
    { 0, 0 },
    { 0x173ce1a8, 36 },
    { 0x725ce1a9, 46 },
    { 0x638ce1aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x816ce1ad, 33 },
    { 0x122ce1ad, 43 },
    { 0x262ce1af, 90 },
    { 0xb1fce1b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ece1b5, 61 },
    { 0x9ece1b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7ce1b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68ce1bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ce1bf, 90 },
    { 0, 0 },
    { 0x565ce1c1, 76 },
    { 0x94bce1c1, 17 },
    { 0x26cce1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cce1c7, 81 },
    { 0, 0 },
    { 0xacece1c9, 87 },
    { 0x82bce1ca, 66 },
    { 0xb49ce1c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502ce1cf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8ce1d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762ce1dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dce1e2, 61 },
    { 0, 0 },
    { 0xaf4ce1e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96ce1e7, 23 },
    { 0x7abce1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1ce1ed, 3 },
    { 0, 0 },
    { 0xadfce1ef, 90 },
    { 0xd43ce1f0, 99 },
    { 0x93bce1f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ce1f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eece1fb, 33 },
    { 0x40ece1fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ace1ff, 90 },
    { 0xb29ce200, 23 },
    { 0x806ce1ff, 33 },
    { 0x5dace202, 38 },
    { 0, 0 },
    { 0xdccce204, 61 },
    { 0, 0 },
    { 0x34bce206, 23 },
    { 0, 0 },
    { 0xc63ce208, 52 },
    { 0xf4cce209, 33 },
    { 0x7acce209, 74 },
    { 0x466ce208, 33 },
    { 0xed6ce209, 33 },
    { 0x954ce20d, 33 },
    { 0x8eece208, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ece218, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0ce21d, 52 },
    { 0x1a8ce21d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171ce22a, 61 },
    { 0, 0 },
    { 0xeddce22c, 61 },
    { 0x57bce22d, 81 },
    { 0xefbce22d, 43 },
    { 0xc45ce22d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73ce235, 90 },
    { 0xeece236, 33 },
    { 0, 0 },
    { 0x152ce238, 87 },
    { 0, 0 },
    { 0xbb0ce23a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87ce243, 81 },
    { 0x14dce243, 33 },
    { 0x2e2ce244, 33 },
    { 0xf28ce245, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ce249, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0ce24e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ce251, 85 },
    { 0, 0 },
    { 0xe4ace253, 98 },
    { 0, 0 },
    { 0x57dce255, 61 },
    { 0xfd0ce255, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ce25b, 99 },
    { 0x6ccce25b, 61 },
    { 0, 0 },
    { 0xdbece25e, 36 },
    { 0x90ece25f, 17 },
    { 0, 0 },
    { 0x635ce261, 55 },
    { 0xe38ce262, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998ce269, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbce26c, 95 },
    { 0xe0bce26d, 94 },
    { 0x878ce26d, 55 },
    { 0x8bfce26f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114ce275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09ce27b, 23 },
    { 0x1e3ce27c, 90 },
    { 0x922ce27b, 61 },
    { 0x64ece27e, 52 },
    { 0x638ce27f, 85 },
    { 0xf28ce27f, 46 },
    { 0, 0 },
    { 0x7b4ce282, 43 },
    { 0x107ce283, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4ce288, 87 },
    { 0x772ce289, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ece28d, 99 },
    { 0, 0 },
    { 0xdeece28f, 95 },
    { 0, 0 },
    { 0x294ce291, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2ce294, 33 },
    { 0xdfdce295, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fce29b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abce2a3, 76 },
    { 0x96ece2a4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cce2a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ce2af, 78 },
    { 0x162ce2af, 65 },
    { 0x898ce2b1, 56 },
    { 0xe8bce2b2, 84 },
    { 0xb9cce2b3, 88 },
    { 0x99bce2b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cce2bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ce2c0, 78 },
    { 0x731ce2c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcce2cc, 84 },
    { 0xaccce2cd, 40 },
    { 0x36dce2ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8ce2d3, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ce2d7, 23 },
    { 0xd08ce2d8, 95 },
    { 0x4bcce2d9, 61 },
    { 0x5d1ce2d9, 76 },
    { 0x708ce2d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cce2e3, 23 },
    { 0xdb5ce2e4, 55 },
    { 0x15cce2e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459ce2eb, 55 },
    { 0x6c2ce2ec, 77 },
    { 0, 0 },
    { 0xfcbce2ee, 84 },
    { 0x6ace2ee, 95 },
    { 0x10ce2f0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fce2f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x664ce2fb, 98 },
    { 0xd66ce2fc, 98 },
    { 0x26bce2fc, 90 },
    { 0x2b2ce2fe, 36 },
    { 0xb2fce2ff, 33 },
    { 0xff5ce2fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bce307, 77 },
    { 0xab3ce308, 61 },
    { 0, 0 },
    { 0x5c6ce30a, 76 },
    { 0, 0 },
    { 0x555ce30c, 98 },
    { 0xfbace30d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2ce313, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ece316, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17ce31e, 76 },
    { 0, 0 },
    { 0x905ce320, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dce323, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adce32b, 34 },
    { 0x658ce32c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ce332, 80 },
    { 0x967ce332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ce337, 33 },
    { 0xa78ce338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bce33b, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141ce34b, 93 },
    { 0x3fcce34b, 33 },
    { 0xf4ece34d, 55 },
    { 0xb73ce34e, 91 },
    { 0x28ece34d, 90 },
    { 0, 0 },
    { 0xcd0ce351, 66 },
    { 0x90bce352, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6ce358, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74ce35c, 76 },
    { 0xb18ce35d, 93 },
    { 0x1d6ce35e, 67 },
    { 0xe34ce35d, 42 },
    { 0xc13ce35c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6ce364, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ce367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ce370, 84 },
    { 0, 0 },
    { 0xbe5ce372, 76 },
    { 0, 0 },
    { 0x55fce374, 69 },
    { 0x643ce374, 95 },
    { 0x266ce376, 84 },
    { 0x9bfce377, 95 },
    { 0xb67ce378, 61 },
    { 0x6f5ce379, 98 },
    { 0xb06ce37a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66ce37d, 43 },
    { 0x511ce37d, 95 },
    { 0, 0 },
    { 0x654ce380, 61 },
    { 0xeace381, 76 },
    { 0, 0 },
    { 0xd50ce383, 78 },
    { 0xeaace383, 88 },
    { 0, 0 },
    { 0xbd6ce386, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44ce38a, 90 },
    { 0xdaece38a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60ce38f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2ce393, 23 },
    { 0, 0 },
    { 0xdccce395, 43 },
    { 0, 0 },
    { 0xe5ce397, 33 },
    { 0xb3bce398, 77 },
    { 0x84dce398, 90 },
    { 0, 0 },
    { 0x7fece39b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dce3a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bace3a6, 90 },
    { 0xdc9ce3a7, 90 },
    { 0x2a9ce3a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974ce3ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13ce3b1, 95 },
    { 0, 0 },
    { 0x8ece3b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874ce3bd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbece3c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ce3c7, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfce3cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ace3ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddce3d5, 95 },
    { 0xfd0ce3d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87ce3de, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ce3e1, 77 },
    { 0, 0 },
    { 0x5a0ce3e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1ce3e6, 87 },
    { 0x13dce3e6, 84 },
    { 0xa8bce3e8, 78 },
    { 0x289ce3e8, 90 },
    { 0x78ce3e9, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfce3ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bce3f5, 98 },
    { 0x7d7ce3f6, 90 },
    { 0x950ce3f7, 100 },
    { 0x7f6ce3f5, 33 },
    { 0x81cce3f5, 90 },
    { 0xd27ce3fa, 61 },
    { 0x4ce3f5, 93 },
    { 0, 0 },
    { 0x2fbce3fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73ce401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322ce409, 66 },
    { 0, 0 },
    { 0x739ce40b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabce40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1ce413, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1ce41b, 98 },
    { 0xb27ce41c, 98 },
    { 0, 0 },
    { 0x40cce41e, 76 },
    { 0x73ece41f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce42f, 99 },
    { 0x22ace430, 90 },
    { 0x674ce431, 93 },
    { 0x6bce431, 91 },
    { 0, 0 },
    { 0x450ce434, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2ce439, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ece43c, 98 },
    { 0, 0 },
    { 0xf26ce43e, 40 },
    { 0x8a7ce43e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ace444, 23 },
    { 0x4e9ce445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bce44a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dce44d, 46 },
    { 0xa12ce44e, 75 },
    { 0x491ce44f, 91 },
    { 0x754ce44f, 90 },
    { 0x2d9ce450, 56 },
    { 0x306ce44d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556ce456, 98 },
    { 0xe15ce456, 67 },
    { 0, 0 },
    { 0x9bce459, 52 },
    { 0xdcdce45a, 95 },
    { 0, 0 },
    { 0x64ce45c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fce45f, 23 },
    { 0x63ece460, 66 },
    { 0, 0 },
    { 0xb44ce462, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xdface465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fce468, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cce46b, 95 },
    { 0, 0 },
    { 0x4fbce46d, 98 },
    { 0, 0 },
    { 0x91ce46f, 55 },
    { 0, 0 },
    { 0xfcfce471, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bace477, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82ce47b, 61 },
    { 0xdc5ce47b, 61 },
    { 0, 0 },
    { 0x787ce47e, 66 },
    { 0, 0 },
    { 0x18ece480, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186ce485, 90 },
    { 0, 0 },
    { 0x583ce487, 98 },
    { 0xc3ce488, 76 },
    { 0, 0 },
    { 0x636ce48a, 23 },
    { 0xb79ce48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6ce48e, 61 },
    { 0x357ce48f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388ce495, 33 },
    { 0x8ccce496, 98 },
    { 0, 0 },
    { 0xa5dce498, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ce4a1, 61 },
    { 0xa69ce4a1, 52 },
    { 0x85ace4a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x911ce4a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ace4a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4ce4ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45ce4af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409ce4b8, 61 },
    { 0, 0 },
    { 0x58bce4ba, 17 },
    { 0xb08ce4bb, 98 },
    { 0, 0 },
    { 0x51ce4bd, 33 },
    { 0xc5ace4bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ce4c2, 95 },
    { 0, 0 },
    { 0xacdce4c4, 43 },
    { 0, 0 },
    { 0xde9ce4c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bce4c9, 90 },
    { 0x2e3ce4ca, 90 },
    { 0x45fce4cb, 98 },
    { 0xe0ace4ca, 33 },
    { 0x257ce4ca, 95 },
    { 0x43bce4ca, 98 },
    { 0x3d2ce4cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ce4d3, 23 },
    { 0x429ce4d4, 95 },
    { 0x40ace4d4, 99 },
    { 0, 0 },
    { 0x1e7ce4d7, 23 },
    { 0, 0 },
    { 0x58cce4d9, 76 },
    { 0xc66ce4da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818ce4e2, 84 },
    { 0x4e1ce4e3, 33 },
    { 0x78ece4e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ce4e8, 76 },
    { 0x2a9ce4e9, 43 },
    { 0xadace4ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ce4ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637ce4f1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x129ce4f4, 33 },
    { 0xfd6ce4f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x425ce4f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x453ce4fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468ce507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511ce50c, 23 },
    { 0x2d9ce50d, 80 },
    { 0x83dce50e, 90 },
    { 0, 0 },
    { 0xff2ce510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fce514, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x654ce517, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ce51e, 65 },
    { 0, 0 },
    { 0x87ace520, 90 },
    { 0xda2ce520, 88 },
    { 0x4f3ce522, 97 },
    { 0x29bce523, 98 },
    { 0xba8ce521, 76 },
    { 0xc0dce523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8ce529, 90 },
    { 0, 0 },
    { 0xdfcce52b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dce52e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dce531, 76 },
    { 0x12ece532, 33 },
    { 0xbe8ce533, 33 },
    { 0x636ce534, 76 },
    { 0xacece533, 95 },
    { 0, 0 },
    { 0x1efce537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688ce53b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2ce541, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ece544, 46 },
    { 0x6b2ce545, 55 },
    { 0x1e5ce546, 69 },
    { 0xb72ce544, 90 },
    { 0, 0 },
    { 0x396ce549, 100 },
    { 0xea3ce54a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ece54e, 95 },
    { 0, 0 },
    { 0xe26ce550, 61 },
    { 0x223ce551, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbace557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1ce55a, 90 },
    { 0x96fce55b, 90 },
    { 0, 0 },
    { 0x3d1ce55d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cace562, 61 },
    { 0, 0 },
    { 0x8ddce564, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7ce572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x689ce575, 33 },
    { 0xe2ece576, 66 },
    { 0xbd7ce576, 80 },
    { 0x6dbce578, 98 },
    { 0x1a7ce579, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6ce57f, 98 },
    { 0x999ce580, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2ce586, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffce58b, 34 },
    { 0, 0 },
    { 0x589ce58d, 61 },
    { 0xb7ce58e, 43 },
    { 0xc1ace58f, 85 },
    { 0x8d0ce590, 23 },
    { 0xbf8ce590, 34 },
    { 0x7a7ce592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ce596, 33 },
    { 0x62ace596, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebce59a, 93 },
    { 0, 0 },
    { 0xf86ce59c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327ce5a1, 91 },
    { 0xc7dce5a1, 33 },
    { 0x31bce5a3, 68 },
    { 0x650ce5a4, 46 },
    { 0x184ce5a5, 76 },
    { 0xbeece5a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7ce5aa, 66 },
    { 0xf5ace5ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1ce5b0, 43 },
    { 0x453ce5b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbace5b6, 43 },
    { 0, 0 },
    { 0xf10ce5b8, 95 },
    { 0xdcece5b9, 69 },
    { 0xb4fce5b8, 99 },
    { 0x53fce5b8, 98 },
    { 0x9c4ce5b9, 55 },
    { 0, 0 },
    { 0x204ce5be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6ce5c4, 77 },
    { 0xe8cce5c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9ce5c9, 87 },
    { 0x773ce5ca, 34 },
    { 0xdfcce5cb, 33 },
    { 0xe8cce5cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ce5d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x404ce5d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ce5db, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cce5e0, 90 },
    { 0xa04ce5e0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x143ce5e4, 43 },
    { 0x82ace5e5, 23 },
    { 0x2e1ce5e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dce5f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8ce5f6, 34 },
    { 0xbd8ce5f7, 17 },
    { 0xb93ce5f6, 52 },
    { 0, 0 },
    { 0x378ce5fa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2ce600, 90 },
    { 0, 0 },
    { 0xaa3ce602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dce606, 17 },
    { 0xb82ce607, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefece60f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4ce617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fce629, 90 },
    { 0, 0 },
    { 0x75ce62b, 76 },
    { 0, 0 },
    { 0xd4dce62d, 46 },
    { 0, 0 },
    { 0x804ce62f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fce635, 77 },
    { 0x3c1ce636, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x695ce639, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ce63c, 95 },
    { 0x861ce63c, 67 },
    { 0xbacce63e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ce641, 55 },
    { 0xaa8ce642, 55 },
    { 0x7fdce643, 33 },
    { 0x559ce644, 90 },
    { 0x909ce644, 90 },
    { 0x45ece646, 43 },
    { 0, 0 },
    { 0x6a6ce648, 68 },
    { 0x142ce649, 98 },
    { 0xe1ce64a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ce64d, 90 },
    { 0x5dcce64d, 78 },
    { 0xa8ace64f, 52 },
    { 0, 0 },
    { 0x433ce651, 95 },
    { 0, 0 },
    { 0xcfece653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9ce656, 33 },
    { 0x520ce657, 33 },
    { 0xebfce657, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75ce65d, 65 },
    { 0x753ce65e, 46 },
    { 0, 0 },
    { 0xe71ce660, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645ce664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x524ce667, 90 },
    { 0, 0 },
    { 0xcb0ce669, 77 },
    { 0xdb1ce669, 93 },
    { 0x624ce66b, 98 },
    { 0xb9ace66b, 93 },
    { 0xd0dce66d, 90 },
    { 0, 0 },
    { 0xccbce66f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43ce674, 55 },
    { 0, 0 },
    { 0xf0ece676, 90 },
    { 0x874ce677, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11ce67a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x921ce67d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bce682, 42 },
    { 0x759ce682, 36 },
    { 0, 0 },
    { 0x67bce685, 34 },
    { 0x280ce685, 95 },
    { 0, 0 },
    { 0x585ce688, 98 },
    { 0xb6bce688, 98 },
    { 0x5c0ce68a, 33 },
    { 0x4fbce689, 91 },
    { 0, 0 },
    { 0xde5ce68d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dce691, 99 },
    { 0x9c4ce692, 90 },
    { 0x6a8ce693, 66 },
    { 0x571ce694, 55 },
    { 0x329ce695, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345ce69e, 23 },
    { 0xe46ce69e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dce6a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17ce6a6, 84 },
    { 0xaf4ce6a6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bce6ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28ce6af, 88 },
    { 0x48ce6b0, 76 },
    { 0x8fcce6b1, 98 },
    { 0x94ece6af, 74 },
    { 0xf32ce6b3, 23 },
    { 0xcbce6b3, 76 },
    { 0x6b6ce6b3, 61 },
    { 0xcf0ce6b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87ce6be, 90 },
    { 0x532ce6bf, 66 },
    { 0xceace6be, 34 },
    { 0x880ce6c1, 33 },
    { 0xae0ce6c2, 76 },
    { 0xba8ce6c2, 96 },
    { 0, 0 },
    { 0x3a4ce6c5, 33 },
    { 0, 0 },
    { 0xeebce6c7, 91 },
    { 0x17ece6c8, 61 },
    { 0x2cdce6c9, 98 },
    { 0x8f9ce6c8, 61 },
    { 0x132ce6cb, 100 },
    { 0xf50ce6cc, 76 },
    { 0xe3bce6cd, 33 },
    { 0xa65ce6ce, 61 },
    { 0x4bece6cf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bce6d3, 33 },
    { 0x43ace6d4, 90 },
    { 0xc17ce6d3, 33 },
    { 0x6d4ce6d4, 61 },
    { 0, 0 },
    { 0x820ce6d8, 84 },
    { 0, 0 },
    { 0x831ce6da, 90 },
    { 0x38ece6db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaace6df, 95 },
    { 0, 0 },
    { 0x271ce6e1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x808ce6e4, 33 },
    { 0x8b2ce6e5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x181ce6e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ece6ec, 76 },
    { 0, 0 },
    { 0x7a2ce6ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dce6f7, 90 },
    { 0x8d1ce6f7, 23 },
    { 0, 0 },
    { 0x5f9ce6fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ce6fd, 90 },
    { 0, 0 },
    { 0x686ce6ff, 23 },
    { 0x2c7ce700, 23 },
    { 0x574ce701, 93 },
    { 0, 0 },
    { 0xeaace703, 90 },
    { 0, 0 },
    { 0xcdece705, 90 },
    { 0, 0 },
    { 0x60bce707, 95 },
    { 0xf3ace708, 95 },
    { 0xf89ce708, 78 },
    { 0x21ce709, 90 },
    { 0, 0 },
    { 0x9b8ce70c, 91 },
    { 0xfc1ce70d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901ce711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4ce716, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2ce719, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727ce71d, 95 },
    { 0xc8fce71e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ce721, 33 },
    { 0, 0 },
    { 0xc2bce723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ece727, 46 },
    { 0x5cace728, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09ce72c, 33 },
    { 0xac7ce72d, 90 },
    { 0x272ce72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ce732, 98 },
    { 0x3edce733, 23 },
    { 0, 0 },
    { 0x553ce735, 90 },
    { 0xe03ce736, 61 },
    { 0, 0 },
    { 0xda2ce738, 34 },
    { 0x6bfce739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ce73c, 34 },
    { 0x2dfce73c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cce742, 43 },
    { 0x1adce742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ce747, 68 },
    { 0x78ce748, 23 },
    { 0x965ce749, 95 },
    { 0x575ce749, 95 },
    { 0x6b2ce74b, 43 },
    { 0x61ece74c, 76 },
    { 0x4d6ce74b, 61 },
    { 0xcbfce74c, 17 },
    { 0x4f9ce74a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dce759, 90 },
    { 0x3a9ce759, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x511ce75d, 96 },
    { 0xf9ce75d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bce761, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0ce764, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15ce76a, 93 },
    { 0, 0 },
    { 0x443ce76c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ace76f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fce774, 95 },
    { 0, 0 },
    { 0xb8ace776, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebce779, 34 },
    { 0, 0 },
    { 0xfb2ce77b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ece781, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ce785, 66 },
    { 0, 0 },
    { 0x4d4ce787, 23 },
    { 0x289ce788, 98 },
    { 0, 0 },
    { 0x508ce78a, 98 },
    { 0xcb8ce78a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543ce78f, 46 },
    { 0, 0 },
    { 0x488ce791, 100 },
    { 0x18dce791, 33 },
    { 0x4b7ce792, 83 },
    { 0x65fce794, 98 },
    { 0x66fce794, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bce798, 93 },
    { 0x5c3ce799, 33 },
    { 0x48fce79a, 23 },
    { 0xd9bce798, 61 },
    { 0xf07ce799, 61 },
    { 0, 0 },
    { 0xc2bce79e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ce7a5, 33 },
    { 0x212ce7a6, 23 },
    { 0xc05ce7a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x413ce7aa, 43 },
    { 0xb2fce7ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ce7af, 91 },
    { 0x584ce7b0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ace7b3, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ece7b6, 77 },
    { 0, 0 },
    { 0x4f4ce7b8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96ce7bb, 55 },
    { 0x651ce7bc, 76 },
    { 0x479ce7bd, 46 },
    { 0x3e1ce7bd, 81 },
    { 0xe60ce7bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eace7c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adce7c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ce7cb, 90 },
    { 0x72ace7cc, 90 },
    { 0, 0 },
    { 0x5ccce7ce, 34 },
    { 0, 0 },
    { 0xf46ce7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaece7d5, 65 },
    { 0, 0 },
    { 0x369ce7d7, 91 },
    { 0xf7ace7d8, 90 },
    { 0x1d6ce7d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0ce7dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adce7e5, 23 },
    { 0xdadce7e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9ce7f1, 95 },
    { 0, 0 },
    { 0x7ace7f3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6ce7fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ace802, 33 },
    { 0xd5ece802, 33 },
    { 0xe14ce804, 61 },
    { 0x264ce802, 90 },
    { 0, 0 },
    { 0xc40ce807, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eece80a, 55 },
    { 0xb87ce80b, 68 },
    { 0xa6ce80b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39ce814, 95 },
    { 0, 0 },
    { 0xcf5ce816, 34 },
    { 0xe6ece817, 90 },
    { 0xeadce818, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2ce81d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50ce821, 90 },
    { 0, 0 },
    { 0xa98ce823, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130ce827, 61 },
    { 0, 0 },
    { 0x1c6ce829, 34 },
    { 0, 0 },
    { 0xbc6ce82b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fce831, 61 },
    { 0xa7ece832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cce835, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ce83a, 75 },
    { 0, 0 },
    { 0x18bce83c, 93 },
    { 0, 0 },
    { 0x2b3ce83e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcace841, 100 },
    { 0x781ce841, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62ce84c, 43 },
    { 0, 0 },
    { 0xb6ece84e, 61 },
    { 0, 0 },
    { 0xf66ce850, 34 },
    { 0, 0 },
    { 0xad5ce852, 78 },
    { 0, 0 },
    { 0x620ce854, 95 },
    { 0, 0 },
    { 0x6e4ce856, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x249ce859, 23 },
    { 0, 0 },
    { 0xadece85b, 33 },
    { 0x39ce85c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bce85f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229ce863, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dce86a, 90 },
    { 0xab3ce86a, 77 },
    { 0x5f4ce86c, 97 },
    { 0xb45ce86c, 33 },
    { 0, 0 },
    { 0xd0fce86f, 61 },
    { 0x2f1ce86f, 95 },
    { 0, 0 },
    { 0x6ddce872, 87 },
    { 0xf94ce873, 64 },
    { 0x829ce872, 33 },
    { 0xb6cce872, 76 },
    { 0xc2ece876, 82 },
    { 0x3a0ce876, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16ce87a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ace87d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffce880, 96 },
    { 0x16ce881, 90 },
    { 0x62fce882, 98 },
    { 0x8fbce883, 95 },
    { 0x6cfce882, 33 },
    { 0x8bce882, 25 },
    { 0xfd0ce882, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979ce88a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4ce88f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960ce893, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cce89a, 88 },
    { 0xcce89b, 88 },
    { 0, 0 },
    { 0xa74ce89d, 23 },
    { 0, 0 },
    { 0x635ce89f, 33 },
    { 0xe3dce8a0, 95 },
    { 0x997ce8a1, 99 },
    { 0x3d8ce8a2, 61 },
    { 0x3d4ce8a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dce8ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ace8ae, 99 },
    { 0xcd8ce8ae, 85 },
    { 0xbdece8af, 98 },
    { 0x918ce8b1, 95 },
    { 0x3fdce8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdce8b7, 43 },
    { 0x7a6ce8b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8ce8bb, 98 },
    { 0, 0 },
    { 0xdb7ce8bd, 33 },
    { 0, 0 },
    { 0xc97ce8bf, 76 },
    { 0x2c6ce8c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfce8c5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24ce8c8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01ce8d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52ce8db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8ce8de, 33 },
    { 0, 0 },
    { 0xef7ce8e0, 55 },
    { 0xfbdce8e1, 33 },
    { 0xb86ce8e2, 33 },
    { 0x298ce8e3, 75 },
    { 0xf31ce8e2, 98 },
    { 0, 0 },
    { 0xdb0ce8e6, 46 },
    { 0, 0 },
    { 0xd95ce8e8, 23 },
    { 0, 0 },
    { 0xc8ece8ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ce8f6, 61 },
    { 0, 0 },
    { 0x945ce8f8, 40 },
    { 0x64ace8f9, 61 },
    { 0xa05ce8fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfce8ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abce904, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6ce907, 33 },
    { 0x6cace907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x312ce90b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266ce90f, 95 },
    { 0x7d8ce910, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fce915, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dce918, 23 },
    { 0x428ce918, 33 },
    { 0x2d9ce919, 36 },
    { 0xd39ce91b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1ce91e, 93 },
    { 0xc5ace91e, 90 },
    { 0xbc3ce920, 88 },
    { 0x382ce921, 98 },
    { 0, 0 },
    { 0x4dcce923, 46 },
    { 0xa60ce924, 36 },
    { 0x4bce923, 23 },
    { 0x223ce926, 88 },
    { 0x6dace927, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4ce92a, 77 },
    { 0x6bbce92b, 98 },
    { 0x279ce92b, 95 },
    { 0, 0 },
    { 0x9aece92e, 39 },
    { 0xf03ce92f, 98 },
    { 0x2a0ce930, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce933, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0ce936, 61 },
    { 0, 0 },
    { 0xca9ce938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fce93b, 96 },
    { 0, 0 },
    { 0x7c8ce93d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ce942, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bce94b, 23 },
    { 0xdaece94b, 76 },
    { 0, 0 },
    { 0xc1ce94e, 61 },
    { 0xeefce94e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634ce953, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30ce958, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ce95e, 33 },
    { 0x607ce95f, 99 },
    { 0x625ce960, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bce965, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9ce969, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7ce96c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadce974, 98 },
    { 0xc43ce975, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7ce978, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0ce97f, 61 },
    { 0xe5ece980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbce985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfce990, 33 },
    { 0x59fce991, 33 },
    { 0x10ece992, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80ce99f, 33 },
    { 0, 0 },
    { 0xa38ce9a1, 34 },
    { 0, 0 },
    { 0x194ce9a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12ce9a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9ce9aa, 61 },
    { 0, 0 },
    { 0x62fce9ac, 98 },
    { 0xe94ce9ad, 61 },
    { 0, 0 },
    { 0xa2ece9af, 52 },
    { 0x4c3ce9b0, 76 },
    { 0xf49ce9b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7ce9be, 90 },
    { 0x27bce9bf, 90 },
    { 0xcf8ce9c0, 76 },
    { 0xc92ce9c1, 33 },
    { 0x938ce9c1, 12 },
    { 0xf7dce9c3, 23 },
    { 0xb52ce9c3, 43 },
    { 0x4c5ce9c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8ce9cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x619ce9cf, 34 },
    { 0xdce9d0, 76 },
    { 0x1f5ce9d1, 95 },
    { 0x93fce9d0, 34 },
    { 0x802ce9d0, 61 },
    { 0, 0 },
    { 0xd0ece9d5, 87 },
    { 0x3c2ce9d6, 23 },
    { 0x3c4ce9d5, 76 },
    { 0, 0 },
    { 0x5e9ce9d9, 95 },
    { 0, 0 },
    { 0x200ce9db, 90 },
    { 0, 0 },
    { 0xe44ce9dd, 95 },
    { 0, 0 },
    { 0x7a9ce9df, 67 },
    { 0, 0 },
    { 0x909ce9e1, 34 },
    { 0, 0 },
    { 0x8face9e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bce9e6, 34 },
    { 0xb22ce9e7, 90 },
    { 0x391ce9e8, 100 },
    { 0x5d1ce9e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ece9ec, 90 },
    { 0xafece9ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bce9f0, 91 },
    { 0x27ece9f1, 33 },
    { 0x248ce9f1, 90 },
    { 0x69cce9f2, 95 },
    { 0xf73ce9f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ce9f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fecea00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dcea04, 70 },
    { 0x694cea05, 43 },
    { 0xaf0cea04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddcea09, 78 },
    { 0, 0 },
    { 0xf52cea0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3cea11, 95 },
    { 0x17acea12, 52 },
    { 0, 0 },
    { 0x9dfcea14, 67 },
    { 0xb49cea14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfcea18, 83 },
    { 0x76ccea18, 55 },
    { 0x28fcea1a, 84 },
    { 0xaa0cea1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15cea23, 66 },
    { 0xd7fcea23, 61 },
    { 0xb95cea25, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8cea29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57acea2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06cea34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55cea39, 33 },
    { 0xba4cea3a, 88 },
    { 0x7a4cea3a, 95 },
    { 0x8fbcea3b, 90 },
    { 0xe6ccea3d, 76 },
    { 0, 0 },
    { 0xf33cea3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5cea42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3cea46, 33 },
    { 0, 0 },
    { 0xe1dcea48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8cea4c, 34 },
    { 0, 0 },
    { 0xf1acea4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2cea52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbcea5a, 40 },
    { 0, 0 },
    { 0x93ccea5c, 99 },
    { 0x427cea5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edcea60, 17 },
    { 0x350cea60, 97 },
    { 0x563cea62, 101 },
    { 0xd3bcea63, 43 },
    { 0x8ccea62, 9 },
    { 0x422cea65, 90 },
    { 0, 0 },
    { 0x963cea67, 55 },
    { 0x1cacea68, 93 },
    { 0xb81cea69, 98 },
    { 0, 0 },
    { 0x7b3cea6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfcea6e, 61 },
    { 0, 0 },
    { 0x73dcea70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5cea75, 61 },
    { 0, 0 },
    { 0x7c3cea77, 33 },
    { 0, 0 },
    { 0xb96cea79, 90 },
    { 0xaaacea79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375cea7e, 34 },
    { 0xaeecea7f, 90 },
    { 0xe9bcea7f, 23 },
    { 0, 0 },
    { 0x5b8cea82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x426cea85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7cea89, 46 },
    { 0xfa0cea8a, 95 },
    { 0xd25cea89, 85 },
    { 0xe5ecea8c, 84 },
    { 0x81cea8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0cea90, 23 },
    { 0x3efcea91, 90 },
    { 0x172cea90, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fcea96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0acea99, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92acea9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2ceaa0, 23 },
    { 0xbfceaa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bceaa4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x748ceaa7, 87 },
    { 0x153ceaa8, 91 },
    { 0xacfceaa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dceaaf, 33 },
    { 0xf7aceab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42ceab4, 80 },
    { 0, 0 },
    { 0xd27ceab6, 38 },
    { 0, 0 },
    { 0x3e9ceab8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99dceabc, 90 },
    { 0x452ceabd, 91 },
    { 0x529ceabd, 81 },
    { 0xb14ceabe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797cead2, 52 },
    { 0x4dacead2, 98 },
    { 0x50acead4, 90 },
    { 0xae6cead5, 34 },
    { 0xf0ccead6, 90 },
    { 0x48fcead4, 53 },
    { 0x25dcead8, 23 },
    { 0xf8ecead9, 84 },
    { 0x76ecead9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6ceadf, 80 },
    { 0x90dceae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2ceae4, 61 },
    { 0xbdcceae5, 61 },
    { 0x655ceae5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84eceaf3, 85 },
    { 0xbb2ceaf4, 46 },
    { 0xbccceaf5, 43 },
    { 0xbeceaf6, 66 },
    { 0xf00ceaf5, 23 },
    { 0x66ceaf5, 55 },
    { 0x567ceaf3, 93 },
    { 0x15bceafa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2eceaff, 23 },
    { 0xb0cceb00, 61 },
    { 0, 0 },
    { 0xb67ceb02, 90 },
    { 0xc93ceb02, 55 },
    { 0x136ceb03, 23 },
    { 0xad0ceb05, 61 },
    { 0x4e6ceb06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11ceb0a, 61 },
    { 0x1ecceb0b, 23 },
    { 0x94aceb0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758ceb10, 75 },
    { 0xe6bceb11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ceb15, 76 },
    { 0, 0 },
    { 0x628ceb17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ceb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fceb1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eceb27, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfceb2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x719ceb2e, 33 },
    { 0xfabceb2e, 33 },
    { 0x14bceb2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadceb3a, 61 },
    { 0x83fceb3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7ceb42, 95 },
    { 0x170ceb43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15ceb47, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ceb50, 23 },
    { 0x33dceb50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778ceb55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982ceb58, 95 },
    { 0xee4ceb59, 90 },
    { 0x94cceb5a, 76 },
    { 0, 0 },
    { 0x605ceb5c, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ceb64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bceb76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8ceb79, 34 },
    { 0, 0 },
    { 0x429ceb7b, 95 },
    { 0x4eeceb7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bceb80, 88 },
    { 0x1efceb81, 78 },
    { 0x2ebceb82, 33 },
    { 0x90eceb83, 23 },
    { 0x7e1ceb80, 23 },
    { 0xd3eceb85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cceb88, 65 },
    { 0, 0 },
    { 0x6b6ceb8a, 33 },
    { 0x908ceb8b, 98 },
    { 0xb40ceb8c, 90 },
    { 0x786ceb8b, 93 },
    { 0x397ceb8e, 33 },
    { 0, 0 },
    { 0xf1dceb90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02ceb97, 75 },
    { 0x532ceb97, 33 },
    { 0x234ceb97, 95 },
    { 0, 0 },
    { 0x45fceb9b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ceb9e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652ceba9, 76 },
    { 0xe8dceba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfcebae, 34 },
    { 0x927cebaf, 23 },
    { 0xa20cebae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cebb3, 61 },
    { 0x4cdcebb4, 61 },
    { 0xc2ecebb4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63cebb9, 46 },
    { 0, 0 },
    { 0x52ecebbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8cebbe, 90 },
    { 0, 0 },
    { 0x56acebc0, 23 },
    { 0x7abcebc1, 61 },
    { 0x5bcebc2, 61 },
    { 0xe0ccebc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3cebc7, 93 },
    { 0xba5cebc8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfcebcb, 88 },
    { 0xb90cebcc, 76 },
    { 0x93bcebcc, 91 },
    { 0, 0 },
    { 0xc27cebcf, 34 },
    { 0xbbfcebd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfccebd6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dcebdb, 98 },
    { 0, 0 },
    { 0x726cebdd, 61 },
    { 0xd55cebde, 23 },
    { 0xa9fcebdd, 95 },
    { 0x3c1cebdd, 17 },
    { 0xaa5cebe1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fcebe7, 43 },
    { 0, 0 },
    { 0xac4cebe9, 76 },
    { 0x6ecebea, 88 },
    { 0, 0 },
    { 0x702cebec, 61 },
    { 0x5dcebed, 90 },
    { 0x631cebee, 61 },
    { 0xbd1cebed, 83 },
    { 0xc74cebf0, 61 },
    { 0x775cebf1, 90 },
    { 0xe4dcebf1, 23 },
    { 0x256cebf1, 52 },
    { 0x88ccebf3, 55 },
    { 0xd0ecebf3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdcebfd, 49 },
    { 0x84cebfd, 61 },
    { 0x66dcebfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3becec02, 46 },
    { 0, 0 },
    { 0x95acec04, 33 },
    { 0x92ecec05, 77 },
    { 0x3b5cec06, 64 },
    { 0x93ecec07, 23 },
    { 0xb9acec05, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fcec0c, 98 },
    { 0x906cec0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ecec12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42cec1c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x988cec1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9cec22, 99 },
    { 0, 0 },
    { 0xcabcec24, 95 },
    { 0x548cec25, 98 },
    { 0, 0 },
    { 0x774cec27, 34 },
    { 0x9e5cec27, 33 },
    { 0, 0 },
    { 0xb0bcec2a, 55 },
    { 0, 0 },
    { 0xa01cec2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36cec2f, 96 },
    { 0, 0 },
    { 0x14fcec31, 61 },
    { 0, 0 },
    { 0x684cec33, 66 },
    { 0x186cec33, 34 },
    { 0, 0 },
    { 0x7c6cec36, 99 },
    { 0x7d8cec37, 98 },
    { 0x8b7cec37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323cec3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40cec42, 61 },
    { 0xae4cec43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x788cec46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5cec4e, 64 },
    { 0xc15cec4f, 33 },
    { 0, 0 },
    { 0x925cec51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x877cec54, 88 },
    { 0x516cec55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213cec59, 99 },
    { 0x162cec5a, 87 },
    { 0x2f3cec59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29cec5f, 96 },
    { 0x1dcec60, 68 },
    { 0, 0 },
    { 0xae9cec62, 33 },
    { 0xdbecec63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dcec67, 61 },
    { 0x3a7cec68, 90 },
    { 0, 0 },
    { 0xab4cec6a, 66 },
    { 0xe4ecec6a, 33 },
    { 0x60acec6a, 67 },
    { 0xa64cec6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbcec70, 20 },
    { 0x5f2cec70, 90 },
    { 0x290cec71, 90 },
    { 0x8f9cec73, 23 },
    { 0xa0cec73, 76 },
    { 0x4bacec74, 95 },
    { 0x6a1cec72, 87 },
    { 0, 0 },
    { 0x98ecec78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03cec7d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cacec80, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddcec86, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4cec89, 90 },
    { 0x4facec89, 33 },
    { 0xaacec8a, 98 },
    { 0x51ccec8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6cec90, 53 },
    { 0xf74cec90, 23 },
    { 0x1c4cec92, 33 },
    { 0x341cec93, 91 },
    { 0x5cecec94, 33 },
    { 0xed3cec95, 61 },
    { 0x7bccec96, 40 },
    { 0x79cec96, 33 },
    { 0, 0 },
    { 0xadccec99, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ccec9c, 33 },
    { 0, 0 },
    { 0x417cec9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85fceca1, 23 },
    { 0x44dceca1, 75 },
    { 0x779ceca3, 67 },
    { 0x8f5ceca3, 61 },
    { 0x172ceca2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5ceca9, 90 },
    { 0x3e9ceca9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ececad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dcecb2, 98 },
    { 0, 0 },
    { 0x25dcecb4, 67 },
    { 0xd72cecb5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dcecb8, 33 },
    { 0x38acecb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x997cecbc, 90 },
    { 0, 0 },
    { 0xb14cecbe, 81 },
    { 0xa57cecbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x108cecc2, 61 },
    { 0x2edcecc2, 98 },
    { 0xf85cecc3, 66 },
    { 0x1cececc5, 90 },
    { 0xd86cecc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715cecca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9cecd1, 33 },
    { 0xbbacecd2, 95 },
    { 0xd1acecd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62cecd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26cecd9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6cecdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bcece4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5cece8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aceceb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bcecee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bcecf1, 61 },
    { 0xe4acecf2, 99 },
    { 0xebacecf2, 91 },
    { 0, 0 },
    { 0x85bcecf5, 91 },
    { 0x95ccecf5, 95 },
    { 0xc45cecf5, 61 },
    { 0x708cecf8, 99 },
    { 0x25cecf5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bced02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4ced0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bced15, 61 },
    { 0x8bbced16, 76 },
    { 0x1b5ced17, 90 },
    { 0xc09ced18, 61 },
    { 0xcd7ced19, 66 },
    { 0xf32ced1a, 61 },
    { 0, 0 },
    { 0x725ced1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691ced24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ced2a, 23 },
    { 0, 0 },
    { 0xd38ced2c, 33 },
    { 0x258ced2d, 92 },
    { 0xda2ced2d, 36 },
    { 0x6adced2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3ced32, 55 },
    { 0xd1ced32, 78 },
    { 0x9a7ced33, 33 },
    { 0xb33ced34, 98 },
    { 0, 0 },
    { 0xa34ced37, 53 },
    { 0xa5aced37, 43 },
    { 0x9c4ced38, 84 },
    { 0, 0 },
    { 0xef6ced3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80aced44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6eced47, 61 },
    { 0xdbced48, 95 },
    { 0, 0 },
    { 0xc5bced4a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ced4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cced51, 23 },
    { 0xff2ced52, 95 },
    { 0x871ced52, 98 },
    { 0x7c2ced52, 95 },
    { 0x664ced55, 98 },
    { 0xd49ced55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ced5b, 87 },
    { 0xe8aced5c, 95 },
    { 0xdf0ced5b, 95 },
    { 0, 0 },
    { 0x396ced5f, 33 },
    { 0xd1aced60, 23 },
    { 0, 0 },
    { 0x6cbced62, 33 },
    { 0, 0 },
    { 0xef2ced64, 90 },
    { 0, 0 },
    { 0x450ced66, 43 },
    { 0xf1fced67, 66 },
    { 0x6f0ced68, 61 },
    { 0x53ced68, 90 },
    { 0x9f0ced6a, 87 },
    { 0xcd9ced69, 90 },
    { 0x22fced6c, 91 },
    { 0x349ced6d, 93 },
    { 0xb30ced6e, 67 },
    { 0, 0 },
    { 0xb60ced70, 52 },
    { 0x13ced70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fced75, 35 },
    { 0xfc3ced76, 66 },
    { 0xbfcced77, 23 },
    { 0xc1ced78, 33 },
    { 0xa19ced78, 93 },
    { 0, 0 },
    { 0x2abced7b, 43 },
    { 0, 0 },
    { 0x482ced7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9ced80, 98 },
    { 0x3fced81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x979ced84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeced8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89ced91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bced94, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ced97, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdced9d, 23 },
    { 0xfd0ced9d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bceda1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bcedac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06cedb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8cedb6, 23 },
    { 0, 0 },
    { 0xb00cedb8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccedbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11cedc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebcedc6, 98 },
    { 0xd8cedc6, 84 },
    { 0xe6bcedc7, 99 },
    { 0, 0 },
    { 0xf4ccedca, 33 },
    { 0x81cedca, 33 },
    { 0xbd9cedcb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcedcf, 46 },
    { 0x9adcedcf, 55 },
    { 0, 0 },
    { 0x2eccedd2, 89 },
    { 0x768cedd3, 95 },
    { 0x3e3cedd4, 33 },
    { 0xbc4cedd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4caceddb, 98 },
    { 0x940ceddc, 34 },
    { 0x966ceddd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353cede1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ecede6, 23 },
    { 0, 0 },
    { 0xb0ccede8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5acedec, 33 },
    { 0, 0 },
    { 0x670cedee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6cedf9, 61 },
    { 0x250cedfa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53cedfd, 33 },
    { 0, 0 },
    { 0x8f6cedff, 87 },
    { 0x7fbcee00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fcee07, 93 },
    { 0x325cee08, 98 },
    { 0x142cee09, 61 },
    { 0x61ccee0a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3cee0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefcee10, 33 },
    { 0xa20cee11, 93 },
    { 0x674cee12, 43 },
    { 0x2eacee13, 90 },
    { 0, 0 },
    { 0xe88cee15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cecee18, 33 },
    { 0x90ccee19, 98 },
    { 0x6ffcee1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2cee24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89cee29, 43 },
    { 0x50fcee29, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x175cee2d, 95 },
    { 0, 0 },
    { 0xabcee2f, 93 },
    { 0x309cee30, 76 },
    { 0xef4cee2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1cee36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155cee3a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5cee3d, 34 },
    { 0x85bcee3e, 90 },
    { 0xe7ecee3f, 90 },
    { 0x1e4cee3f, 23 },
    { 0xdbfcee3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3cee47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6cee4a, 95 },
    { 0x845cee4b, 61 },
    { 0xdd9cee4c, 55 },
    { 0xe8bcee4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557cee52, 98 },
    { 0xb2ccee52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0cee57, 34 },
    { 0xc3cee57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ecee62, 34 },
    { 0, 0 },
    { 0x2f4cee64, 90 },
    { 0, 0 },
    { 0xc03cee66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdcee69, 99 },
    { 0, 0 },
    { 0x2fbcee6b, 95 },
    { 0xa13cee6c, 90 },
    { 0x1e5cee6d, 76 },
    { 0xac9cee6e, 90 },
    { 0x5fdcee6f, 100 },
    { 0xa4dcee70, 23 },
    { 0xf97cee71, 95 },
    { 0x3b1cee70, 91 },
    { 0x407cee73, 87 },
    { 0xaeecee6d, 61 },
    { 0x4cee75, 52 },
    { 0xcd4cee76, 68 },
    { 0x3e4cee72, 90 },
    { 0x31bcee78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9cee7b, 84 },
    { 0xab0cee7b, 98 },
    { 0x7c2cee7d, 65 },
    { 0x33dcee7e, 33 },
    { 0xc29cee7f, 46 },
    { 0, 0 },
    { 0xeb2cee81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376cee85, 61 },
    { 0xa4fcee86, 90 },
    { 0xcc8cee85, 23 },
    { 0, 0 },
    { 0x20ecee89, 90 },
    { 0, 0 },
    { 0x6efcee8b, 43 },
    { 0xc3fcee8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99cee8f, 33 },
    { 0, 0 },
    { 0x438cee91, 23 },
    { 0, 0 },
    { 0x802cee93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cee96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fecee99, 66 },
    { 0x4a0cee9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fceea4, 61 },
    { 0x2b5ceea5, 90 },
    { 0x2d7ceea6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cceeaa, 61 },
    { 0xb7eceeab, 61 },
    { 0x659ceeab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640ceeb2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9ceeb7, 34 },
    { 0, 0 },
    { 0x2b1ceeb9, 95 },
    { 0x69ceeb9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3aceec7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bceecb, 90 },
    { 0xf2dceecb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65ceed2, 52 },
    { 0xcefceed2, 75 },
    { 0x68eceed4, 84 },
    { 0x488ceed5, 55 },
    { 0x5b4ceed5, 33 },
    { 0x48cceed7, 46 },
    { 0x582ceed8, 90 },
    { 0x17aceed9, 91 },
    { 0x2dbceed7, 95 },
    { 0x530ceedb, 68 },
    { 0xc8bceedc, 23 },
    { 0x77bceedd, 34 },
    { 0xa4cceedb, 61 },
    { 0x8ddceed9, 90 },
    { 0, 0 },
    { 0x964ceee1, 76 },
    { 0xfc2ceee2, 98 },
    { 0, 0 },
    { 0xaf0ceee4, 95 },
    { 0x9cceee5, 66 },
    { 0, 0 },
    { 0xf57ceee7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4ceeea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fceeee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61ceef4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cceefa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9ceefe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0cef01, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999cef05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aacef0a, 99 },
    { 0x499cef0b, 34 },
    { 0x5ebcef0a, 67 },
    { 0, 0 },
    { 0x24bcef0e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdcef11, 84 },
    { 0x43fcef12, 23 },
    { 0x193cef11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dcef16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159cef1a, 90 },
    { 0, 0 },
    { 0x74cef1c, 98 },
    { 0x6a7cef1d, 90 },
    { 0x808cef1d, 93 },
    { 0xf7acef1d, 69 },
    { 0xc0cef1d, 61 },
    { 0xe17cef21, 23 },
    { 0, 0 },
    { 0x17bcef23, 100 },
    { 0x920cef24, 90 },
    { 0, 0 },
    { 0xbe2cef26, 23 },
    { 0x6eacef27, 43 },
    { 0x50ccef27, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8cef2f, 33 },
    { 0xb8ccef2f, 33 },
    { 0xef1cef31, 33 },
    { 0, 0 },
    { 0xb9cef33, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40cef3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116cef3e, 33 },
    { 0xa82cef3f, 66 },
    { 0x5ffcef3e, 55 },
    { 0x9b2cef41, 90 },
    { 0xea7cef42, 61 },
    { 0xa82cef43, 90 },
    { 0x627cef44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3cef47, 95 },
    { 0, 0 },
    { 0xc2dcef49, 76 },
    { 0, 0 },
    { 0x848cef4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492cef4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85cef56, 95 },
    { 0x131cef57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754cef5b, 95 },
    { 0x1cef5b, 95 },
    { 0x856cef5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eecef61, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5cef64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cef68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefcef6c, 52 },
    { 0, 0 },
    { 0x955cef6e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x281cef71, 33 },
    { 0, 0 },
    { 0x852cef73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfcef7b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x546cef7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbcef83, 43 },
    { 0x1e8cef83, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cef87, 61 },
    { 0, 0 },
    { 0xa40cef89, 77 },
    { 0x5fcef8a, 90 },
    { 0, 0 },
    { 0xc9bcef8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x545cef8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4cef92, 84 },
    { 0, 0 },
    { 0x3c9cef94, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108cef98, 23 },
    { 0xf97cef99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26cef9e, 23 },
    { 0, 0 },
    { 0x178cefa0, 90 },
    { 0, 0 },
    { 0xd9cefa2, 90 },
    { 0x203cefa3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5cefa7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cefb0, 34 },
    { 0x2eecefb0, 23 },
    { 0x649cefb1, 17 },
    { 0x109cefb3, 78 },
    { 0x6bacefb4, 84 },
    { 0x319cefb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbcefb8, 90 },
    { 0, 0 },
    { 0x761cefba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04cefbd, 84 },
    { 0x54bcefbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cefc3, 98 },
    { 0x5c9cefc4, 98 },
    { 0xfa4cefc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bccefc8, 38 },
    { 0x705cefc9, 33 },
    { 0, 0 },
    { 0x189cefcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246cefd6, 43 },
    { 0xb9cefd7, 29 },
    { 0x5f1cefd8, 90 },
    { 0x768cefd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65cefdd, 78 },
    { 0xd29cefde, 90 },
    { 0xb3fcefde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766cefe3, 91 },
    { 0, 0 },
    { 0x4e7cefe5, 37 },
    { 0x114cefe6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7cefeb, 93 },
    { 0x48ecefec, 98 },
    { 0x9dccefed, 61 },
    { 0x61acefee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ceff2, 33 },
    { 0x100ceff3, 88 },
    { 0xfc3ceff4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cceff7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ceffa, 23 },
    { 0xe95ceffb, 76 },
    { 0, 0 },
    { 0xc27ceffd, 76 },
    { 0x15cceffe, 95 },
    { 0x4abcefff, 34 },
    { 0xe5acefff, 93 },
    { 0xe74cefff, 90 },
    { 0x94dcf002, 87 },
    { 0x1aacf003, 33 },
    { 0, 0 },
    { 0x43dcf005, 95 },
    { 0x19bcf006, 43 },
    { 0x61cf006, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270cf00b, 46 },
    { 0x2b3cf00c, 61 },
    { 0x4b4cf00d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf014, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92cf018, 95 },
    { 0xb6cf018, 61 },
    { 0, 0 },
    { 0xf5bcf01b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x33acf01e, 90 },
    { 0xa13cf01f, 52 },
    { 0x57dcf020, 34 },
    { 0xbf9cf021, 84 },
    { 0xf01cf021, 76 },
    { 0x8d9cf023, 90 },
    { 0xa17cf024, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ecf029, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9cf031, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ecf035, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdcf038, 101 },
    { 0, 0 },
    { 0xe53cf03a, 76 },
    { 0, 0 },
    { 0xae8cf03c, 61 },
    { 0xa53cf03d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2cf044, 69 },
    { 0x68acf045, 61 },
    { 0, 0 },
    { 0x564cf047, 43 },
    { 0xb2cf048, 94 },
    { 0, 0 },
    { 0x5c2cf04a, 19 },
    { 0, 0 },
    { 0xfdcf04c, 93 },
    { 0x690cf04d, 33 },
    { 0, 0 },
    { 0xa33cf04f, 36 },
    { 0, 0 },
    { 0x61fcf051, 95 },
    { 0, 0 },
    { 0xb08cf053, 23 },
    { 0x94fcf054, 23 },
    { 0x78fcf055, 100 },
    { 0xb50cf056, 23 },
    { 0, 0 },
    { 0xa43cf058, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2cf062, 55 },
    { 0, 0 },
    { 0xd03cf064, 55 },
    { 0x34acf065, 98 },
    { 0, 0 },
    { 0x201cf067, 98 },
    { 0x405cf068, 90 },
    { 0xe39cf067, 91 },
    { 0xde4cf06a, 95 },
    { 0xa8acf06a, 90 },
    { 0, 0 },
    { 0xad0cf06d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdcf072, 23 },
    { 0, 0 },
    { 0xfcccf074, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dcf077, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dcf07c, 98 },
    { 0xed6cf07d, 33 },
    { 0, 0 },
    { 0x2cbcf07f, 64 },
    { 0x96acf080, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbcf084, 95 },
    { 0x69ccf085, 23 },
    { 0x92bcf085, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54cf08b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9cf094, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eccf09d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758cf0a3, 52 },
    { 0x3f4cf0a4, 100 },
    { 0x4d1cf0a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791cf0ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881cf0b2, 83 },
    { 0xf85cf0b2, 98 },
    { 0x889cf0b4, 90 },
    { 0xd70cf0b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x874cf0b8, 23 },
    { 0xf23cf0b9, 95 },
    { 0xf79cf0b9, 100 },
    { 0xff9cf0bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bacf0c7, 92 },
    { 0x45ccf0c8, 98 },
    { 0x2b5cf0c8, 98 },
    { 0, 0 },
    { 0x979cf0cb, 98 },
    { 0, 0 },
    { 0x1d7cf0cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105cf0da, 82 },
    { 0x309cf0db, 23 },
    { 0xbeacf0dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787cf0e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06cf0ec, 76 },
    { 0xa4cf0ed, 33 },
    { 0xfc2cf0ee, 53 },
    { 0xae4cf0ef, 55 },
    { 0, 0 },
    { 0x724cf0f1, 87 },
    { 0x1f3cf0f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71cf0f6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8cf0fb, 91 },
    { 0x86bcf0fb, 59 },
    { 0x344cf0fd, 98 },
    { 0x3f0cf0fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8cf102, 46 },
    { 0, 0 },
    { 0xf84cf104, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bacf110, 96 },
    { 0, 0 },
    { 0x9c5cf112, 91 },
    { 0, 0 },
    { 0xc54cf114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dcf11a, 48 },
    { 0, 0 },
    { 0x562cf11c, 23 },
    { 0xbaacf11d, 52 },
    { 0x679cf11e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575cf123, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84cf128, 95 },
    { 0xc7acf129, 78 },
    { 0x3cdcf12a, 23 },
    { 0x234cf12a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07cf132, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ecf136, 56 },
    { 0, 0 },
    { 0x8a3cf138, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4cf13c, 46 },
    { 0x54dcf13c, 95 },
    { 0x77fcf13e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06cf143, 34 },
    { 0x98ccf144, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373cf14f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fecf154, 55 },
    { 0xf7acf155, 77 },
    { 0, 0 },
    { 0x2c2cf157, 76 },
    { 0x7b4cf158, 66 },
    { 0xb79cf159, 61 },
    { 0xab1cf158, 99 },
    { 0x566cf157, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fcf15f, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0cf162, 98 },
    { 0x5e9cf162, 46 },
    { 0, 0 },
    { 0xce1cf165, 90 },
    { 0, 0 },
    { 0xf99cf167, 95 },
    { 0x9fecf168, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54cf171, 93 },
    { 0, 0 },
    { 0xcc2cf173, 61 },
    { 0xadbcf173, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcf178, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ccf17c, 33 },
    { 0x453cf17c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeecf182, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1cf186, 98 },
    { 0xd93cf187, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x945cf18a, 33 },
    { 0, 0 },
    { 0x8f0cf18c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9cf192, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfacf196, 76 },
    { 0xb4bcf197, 75 },
    { 0x9b4cf197, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790cf19d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbcf1a0, 23 },
    { 0x337cf1a1, 95 },
    { 0, 0 },
    { 0x33fcf1a3, 95 },
    { 0x966cf1a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbcf1a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03cf1ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585cf1b1, 67 },
    { 0x2d8cf1b2, 33 },
    { 0x841cf1b3, 33 },
    { 0xc42cf1b4, 90 },
    { 0x121cf1b5, 61 },
    { 0xb09cf1b5, 66 },
    { 0xa46cf1b7, 46 },
    { 0, 0 },
    { 0xe20cf1b9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ecf1be, 90 },
    { 0xc54cf1be, 90 },
    { 0xc2ecf1c0, 64 },
    { 0, 0 },
    { 0xdbcf1c2, 90 },
    { 0xdc8cf1c2, 33 },
    { 0xf27cf1c4, 61 },
    { 0x6d0cf1c5, 34 },
    { 0xb8ccf1c4, 90 },
    { 0xf5ecf1c6, 93 },
    { 0xc93cf1c4, 76 },
    { 0x8d9cf1c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96cf1ce, 23 },
    { 0x53acf1ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fcf1d2, 61 },
    { 0x57ccf1d3, 95 },
    { 0, 0 },
    { 0x158cf1d5, 78 },
    { 0, 0 },
    { 0xb9cf1d7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90cf1dc, 76 },
    { 0xb09cf1dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeacf1e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8cf1e4, 23 },
    { 0x983cf1e5, 17 },
    { 0xa68cf1e4, 90 },
    { 0x6c7cf1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54cf1ef, 61 },
    { 0, 0 },
    { 0x5efcf1f1, 34 },
    { 0x7b5cf1f1, 52 },
    { 0, 0 },
    { 0x5efcf1f4, 95 },
    { 0xed1cf1f5, 97 },
    { 0, 0 },
    { 0xb9bcf1f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dcf1fc, 61 },
    { 0xd96cf1fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x998cf200, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x570cf203, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9cf208, 76 },
    { 0x30acf209, 61 },
    { 0xdd6cf20a, 93 },
    { 0x46bcf20b, 33 },
    { 0x55dcf209, 95 },
    { 0xb55cf20a, 76 },
    { 0xb0ecf20e, 61 },
    { 0xe0fcf20f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dcf212, 33 },
    { 0x63fcf213, 95 },
    { 0xf57cf213, 74 },
    { 0x951cf214, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fcf219, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xecccf21c, 92 },
    { 0x831cf21d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x878cf220, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628cf227, 98 },
    { 0x12acf228, 67 },
    { 0x617cf229, 76 },
    { 0, 0 },
    { 0x598cf22b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x446cf22e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7cf232, 80 },
    { 0xd6fcf232, 55 },
    { 0xb62cf234, 33 },
    { 0x403cf235, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dcf238, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5cf23b, 95 },
    { 0x339cf23c, 66 },
    { 0, 0 },
    { 0x31ccf23e, 95 },
    { 0x4becf23f, 33 },
    { 0xeffcf23e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16cf247, 77 },
    { 0, 0 },
    { 0xffdcf249, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fcf24c, 92 },
    { 0x135cf24c, 90 },
    { 0x490cf24e, 33 },
    { 0xb11cf24f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18cf256, 95 },
    { 0xa5acf256, 33 },
    { 0, 0 },
    { 0xa7dcf259, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cf265, 33 },
    { 0x5c2cf266, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20cf26c, 34 },
    { 0xb71cf26d, 43 },
    { 0xeccf26e, 100 },
    { 0xd72cf26c, 78 },
    { 0xb6ccf270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291cf277, 57 },
    { 0xcb5cf277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0cf27e, 61 },
    { 0x5c3cf27f, 33 },
    { 0x7cdcf280, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58cf288, 77 },
    { 0x7e3cf289, 76 },
    { 0x8fccf288, 61 },
    { 0xc6bcf28b, 78 },
    { 0x84ccf28c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57acf28f, 61 },
    { 0x70ccf290, 33 },
    { 0, 0 },
    { 0xe55cf292, 76 },
    { 0x581cf292, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8cf299, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf29d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x242cf2a0, 87 },
    { 0x4b5cf2a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bcf2a6, 38 },
    { 0x926cf2a6, 23 },
    { 0x551cf2a8, 33 },
    { 0, 0 },
    { 0x3c2cf2aa, 85 },
    { 0x8c0cf2ab, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8cf2ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86cf2b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3cf2be, 33 },
    { 0x7b7cf2be, 90 },
    { 0xeb5cf2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bcf2c3, 64 },
    { 0x75ccf2c3, 33 },
    { 0x5cdcf2c3, 80 },
    { 0, 0 },
    { 0x4d3cf2c7, 82 },
    { 0x31dcf2c8, 90 },
    { 0xfeacf2c9, 61 },
    { 0x6bacf2c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8acf2d4, 34 },
    { 0, 0 },
    { 0xdd1cf2d6, 91 },
    { 0x698cf2d6, 96 },
    { 0x898cf2d7, 95 },
    { 0x4dacf2d9, 95 },
    { 0x746cf2da, 55 },
    { 0xe6cf2d9, 87 },
    { 0x6e6cf2dc, 77 },
    { 0xa8fcf2dc, 23 },
    { 0, 0 },
    { 0x82acf2df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6cf2e6, 23 },
    { 0x11acf2e7, 55 },
    { 0x103cf2e8, 66 },
    { 0, 0 },
    { 0x24cf2ea, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8cf2f1, 33 },
    { 0x69ccf2f2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffcf2f5, 40 },
    { 0x283cf2f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09cf2fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ecf300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2cf30a, 66 },
    { 0xc0acf30a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60cf30f, 90 },
    { 0xf70cf310, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcf318, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x760cf31b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91cf31e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582cf328, 40 },
    { 0x8e0cf328, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ecf32d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fcf337, 61 },
    { 0x3c7cf338, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72cf33d, 43 },
    { 0x560cf33e, 88 },
    { 0x60bcf33e, 95 },
    { 0, 0 },
    { 0x773cf341, 43 },
    { 0x9d6cf342, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abcf346, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffcf34e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45cf353, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cf357, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5cf35b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fcf360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0cf366, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40cf36b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9cf374, 61 },
    { 0, 0 },
    { 0xa75cf376, 61 },
    { 0xefbcf377, 98 },
    { 0x9f5cf378, 33 },
    { 0x68ccf377, 33 },
    { 0, 0 },
    { 0x208cf37b, 98 },
    { 0xafdcf37c, 43 },
    { 0xe21cf37d, 23 },
    { 0xe13cf37e, 90 },
    { 0x387cf37c, 23 },
    { 0x599cf380, 90 },
    { 0xdc6cf381, 90 },
    { 0x9c6cf382, 90 },
    { 0x41ccf37c, 55 },
    { 0x521cf384, 76 },
    { 0x99acf382, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bccf390, 95 },
    { 0, 0 },
    { 0x531cf392, 65 },
    { 0xc5ecf393, 33 },
    { 0, 0 },
    { 0x182cf395, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee6cf39a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351cf39e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ccf3a1, 33 },
    { 0x4b2cf3a2, 77 },
    { 0, 0 },
    { 0xefacf3a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1cf3ac, 33 },
    { 0, 0 },
    { 0x436cf3ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367cf3b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dcf3bf, 43 },
    { 0, 0 },
    { 0x621cf3c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bcf3c7, 55 },
    { 0x564cf3c8, 90 },
    { 0x74acf3c7, 95 },
    { 0x718cf3ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32cf3d0, 90 },
    { 0, 0 },
    { 0xa6fcf3d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188cf3da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746cf3de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ccf3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9dcf3ea, 90 },
    { 0x823cf3eb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7cf3ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16cf3f4, 99 },
    { 0, 0 },
    { 0xe72cf3f6, 90 },
    { 0x663cf3f7, 81 },
    { 0, 0 },
    { 0x560cf3f9, 90 },
    { 0, 0 },
    { 0xe70cf3fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32cf3ff, 23 },
    { 0xff2cf3ff, 95 },
    { 0, 0 },
    { 0xaf8cf402, 33 },
    { 0, 0 },
    { 0x59ecf404, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840cf408, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3cf40c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302cf411, 95 },
    { 0, 0 },
    { 0xf7ecf413, 34 },
    { 0xdcccf414, 90 },
    { 0xe2cf415, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9cf419, 61 },
    { 0xc69cf41a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x106cf41d, 79 },
    { 0, 0 },
    { 0xac0cf41f, 98 },
    { 0x446cf41f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533cf426, 96 },
    { 0xfcecf427, 47 },
    { 0xe86cf428, 90 },
    { 0, 0 },
    { 0xfb5cf42a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fcf434, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bcf438, 34 },
    { 0x631cf439, 90 },
    { 0x276cf43a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ecf443, 43 },
    { 0xc00cf444, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0cf449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7cf44d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeecf454, 80 },
    { 0xee0cf455, 23 },
    { 0xf4ecf456, 35 },
    { 0xed5cf456, 98 },
    { 0xfbdcf458, 33 },
    { 0xbd4cf456, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3cf45d, 84 },
    { 0x4fccf45e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63cf463, 95 },
    { 0, 0 },
    { 0xdcfcf465, 87 },
    { 0xa73cf465, 95 },
    { 0x2cccf467, 33 },
    { 0, 0 },
    { 0xe1cf469, 84 },
    { 0x545cf46a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x960cf46d, 61 },
    { 0xa21cf46d, 90 },
    { 0x411cf46f, 66 },
    { 0xbb0cf46f, 61 },
    { 0, 0 },
    { 0xadfcf472, 69 },
    { 0xf9fcf472, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401cf478, 88 },
    { 0x66cf478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3cf47f, 95 },
    { 0x3afcf480, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4cf484, 74 },
    { 0, 0 },
    { 0x54acf486, 43 },
    { 0xbdccf486, 43 },
    { 0, 0 },
    { 0xc07cf489, 75 },
    { 0xa18cf48a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9cf48f, 90 },
    { 0xc7ecf490, 76 },
    { 0xc3fcf491, 69 },
    { 0, 0 },
    { 0xfc7cf493, 98 },
    { 0x2acf494, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2cf497, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992cf49f, 98 },
    { 0, 0 },
    { 0xf94cf4a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbacf4a5, 93 },
    { 0, 0 },
    { 0x6a7cf4a7, 98 },
    { 0x6b9cf4a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeccf4ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cf4b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcecf4b4, 66 },
    { 0, 0 },
    { 0x739cf4b6, 95 },
    { 0, 0 },
    { 0xf64cf4b8, 33 },
    { 0xdb0cf4b8, 65 },
    { 0xfacf4b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8cf4bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339cf4c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7cf4ca, 33 },
    { 0, 0 },
    { 0xd20cf4cc, 68 },
    { 0x568cf4cd, 33 },
    { 0x49bcf4ce, 95 },
    { 0xc1ecf4cf, 76 },
    { 0x6f5cf4d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303cf4d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3cf4d7, 90 },
    { 0x369cf4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebcf4dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ccf4e4, 98 },
    { 0, 0 },
    { 0x60dcf4e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98cf4ed, 80 },
    { 0, 0 },
    { 0xbd7cf4ef, 52 },
    { 0xdd4cf4f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8cf4f7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4cf4fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3acf502, 83 },
    { 0xccf503, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bccf50a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aecf50e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3cf511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdccf516, 65 },
    { 0xaf2cf516, 98 },
    { 0, 0 },
    { 0xf0acf519, 95 },
    { 0x763cf519, 78 },
    { 0, 0 },
    { 0x6d3cf51c, 90 },
    { 0, 0 },
    { 0x8bccf51e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ccf52e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76cf531, 61 },
    { 0, 0 },
    { 0x2cbcf533, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40cf536, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904cf53f, 99 },
    { 0x3c5cf540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6cf547, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5cf54a, 99 },
    { 0, 0 },
    { 0x385cf54c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fcf550, 66 },
    { 0, 0 },
    { 0xa1dcf552, 52 },
    { 0xd46cf553, 61 },
    { 0x542cf554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fcf558, 91 },
    { 0xe72cf558, 95 },
    { 0x579cf559, 84 },
    { 0xadcf55a, 61 },
    { 0xdb7cf559, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0cf55f, 88 },
    { 0xc3acf55f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cf567, 98 },
    { 0xbc4cf567, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940cf56c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5accf56f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989cf57a, 33 },
    { 0xe3dcf57b, 95 },
    { 0, 0 },
    { 0x75fcf57d, 68 },
    { 0, 0 },
    { 0x489cf57f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ccf582, 83 },
    { 0, 0 },
    { 0x485cf584, 33 },
    { 0xff4cf584, 84 },
    { 0, 0 },
    { 0x11bcf587, 52 },
    { 0xd9bcf588, 66 },
    { 0, 0 },
    { 0xeeecf58a, 17 },
    { 0, 0 },
    { 0xa93cf58c, 67 },
    { 0, 0 },
    { 0x4cacf58e, 90 },
    { 0x65acf58f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197cf595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cf59a, 33 },
    { 0x8b3cf59b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ccf59f, 61 },
    { 0xc79cf5a0, 95 },
    { 0xc11cf59f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x126cf5a7, 66 },
    { 0x68cf5a8, 87 },
    { 0xf94cf5a9, 23 },
    { 0x81ecf5a9, 95 },
    { 0x181cf5ab, 46 },
    { 0x87cf5ac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cf5b0, 23 },
    { 0x6bccf5b1, 85 },
    { 0, 0 },
    { 0x6f4cf5b3, 23 },
    { 0x185cf5b3, 95 },
    { 0, 0 },
    { 0xd05cf5b6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0cf5bb, 99 },
    { 0x539cf5bb, 43 },
    { 0, 0 },
    { 0x7a6cf5be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7cf5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cccf5c5, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dccf5cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4cf5d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44cf5d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0cf5db, 52 },
    { 0x85acf5dc, 43 },
    { 0x436cf5dc, 61 },
    { 0x85cf5dd, 34 },
    { 0x42fcf5df, 95 },
    { 0x48bcf5df, 33 },
    { 0xea6cf5e1, 99 },
    { 0x544cf5e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1cf5e5, 98 },
    { 0x474cf5e5, 33 },
    { 0x54bcf5e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x750cf5ea, 76 },
    { 0xe70cf5eb, 93 },
    { 0x9a5cf5eb, 90 },
    { 0, 0 },
    { 0xbc1cf5ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484cf5f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cf5f7, 61 },
    { 0xea1cf5f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9cf5fb, 33 },
    { 0xdc6cf5fc, 95 },
    { 0xe65cf5fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ccf600, 76 },
    { 0, 0 },
    { 0x1c8cf602, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x127cf605, 23 },
    { 0x904cf606, 98 },
    { 0x68dcf607, 76 },
    { 0xe06cf607, 61 },
    { 0, 0 },
    { 0xbfacf60a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306cf612, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77cf617, 33 },
    { 0xbbecf618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0cf61c, 78 },
    { 0, 0 },
    { 0x800cf61e, 98 },
    { 0x6d3cf61f, 90 },
    { 0, 0 },
    { 0x255cf621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ecf626, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440cf62d, 75 },
    { 0, 0 },
    { 0xc34cf62f, 90 },
    { 0xb65cf62f, 34 },
    { 0xeaacf631, 67 },
    { 0x7f2cf632, 90 },
    { 0xaefcf633, 55 },
    { 0x3a3cf62f, 90 },
    { 0xf94cf630, 23 },
    { 0, 0 },
    { 0xae4cf637, 52 },
    { 0x85ccf638, 68 },
    { 0x5a1cf639, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4acf642, 61 },
    { 0x74ecf642, 97 },
    { 0x5fecf644, 23 },
    { 0x4a1cf644, 98 },
    { 0xbc3cf646, 61 },
    { 0, 0 },
    { 0xe25cf648, 99 },
    { 0xe00cf648, 23 },
    { 0x423cf64a, 66 },
    { 0xc9acf64b, 23 },
    { 0x21fcf64a, 34 },
    { 0xb5fcf64a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bcf653, 34 },
    { 0xa45cf654, 33 },
    { 0xf10cf654, 90 },
    { 0xc4acf654, 84 },
    { 0xaebcf657, 90 },
    { 0xaaacf658, 90 },
    { 0, 0 },
    { 0xcffcf65a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12acf65f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84cf668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339cf670, 76 },
    { 0, 0 },
    { 0xcddcf672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bcf679, 68 },
    { 0, 0 },
    { 0x3a4cf67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf67e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ecf681, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cf688, 95 },
    { 0x297cf689, 87 },
    { 0x686cf68a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ecf68d, 95 },
    { 0x2eacf68e, 33 },
    { 0xf06cf68e, 23 },
    { 0x680cf690, 43 },
    { 0xb66cf68e, 90 },
    { 0xbd4cf692, 85 },
    { 0x7efcf692, 36 },
    { 0, 0 },
    { 0xabdcf695, 93 },
    { 0xd2fcf695, 23 },
    { 0xd30cf697, 17 },
    { 0x597cf698, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fcf69c, 68 },
    { 0x2a2cf69c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdcf6a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ecf6b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ccf6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bcf6b7, 91 },
    { 0, 0 },
    { 0x894cf6b9, 95 },
    { 0, 0 },
    { 0xdb5cf6bb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596cf6c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bcf6d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16cf6da, 99 },
    { 0x289cf6db, 98 },
    { 0, 0 },
    { 0xf9cf6dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191cf6e1, 90 },
    { 0x970cf6e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905cf6e6, 33 },
    { 0x5fbcf6e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422cf6ec, 68 },
    { 0, 0 },
    { 0x490cf6ee, 67 },
    { 0x409cf6ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9cf6f4, 90 },
    { 0x28acf6f5, 61 },
    { 0x9fdcf6f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73cf6f9, 55 },
    { 0x8d8cf6fa, 34 },
    { 0x47fcf6fa, 95 },
    { 0, 0 },
    { 0xd0acf6fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48cf701, 93 },
    { 0, 0 },
    { 0x691cf703, 67 },
    { 0xd06cf703, 52 },
    { 0x1eecf704, 95 },
    { 0xab6cf705, 87 },
    { 0, 0 },
    { 0x789cf708, 61 },
    { 0x69fcf708, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bcf70d, 61 },
    { 0x6cacf70d, 34 },
    { 0, 0 },
    { 0x894cf710, 98 },
    { 0x41cf710, 65 },
    { 0, 0 },
    { 0xab3cf713, 95 },
    { 0, 0 },
    { 0x5becf715, 23 },
    { 0x88ecf716, 46 },
    { 0x88dcf716, 81 },
    { 0x61ccf717, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadcf71b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363cf71f, 95 },
    { 0x6b0cf720, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541cf724, 90 },
    { 0xef5cf724, 46 },
    { 0xbdbcf725, 95 },
    { 0, 0 },
    { 0x208cf728, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x332cf72b, 33 },
    { 0xda7cf72c, 76 },
    { 0, 0 },
    { 0xb4fcf72e, 61 },
    { 0xecbcf72e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ecf733, 85 },
    { 0x8b6cf734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eecf738, 34 },
    { 0x493cf738, 34 },
    { 0, 0 },
    { 0xae0cf73b, 23 },
    { 0xa3ecf73c, 100 },
    { 0xf9acf73c, 99 },
    { 0x522cf73e, 100 },
    { 0x5a8cf73c, 65 },
    { 0x69ecf73b, 95 },
    { 0x766cf73d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7cf744, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bcf747, 67 },
    { 0, 0 },
    { 0xd40cf749, 95 },
    { 0x4a8cf749, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dcf74d, 52 },
    { 0xc9fcf74e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7cf754, 90 },
    { 0xbc9cf755, 75 },
    { 0x18ecf756, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708cf75d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21cf761, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfecf764, 23 },
    { 0x3ddcf765, 55 },
    { 0, 0 },
    { 0xb0fcf767, 61 },
    { 0x40bcf767, 90 },
    { 0, 0 },
    { 0x739cf76a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x839cf76d, 98 },
    { 0x87dcf76e, 66 },
    { 0xe5ccf76f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8cf773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cf776, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdcf77c, 89 },
    { 0, 0 },
    { 0x8d8cf77e, 33 },
    { 0x50bcf77e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efcf782, 95 },
    { 0x56fcf783, 61 },
    { 0x567cf783, 33 },
    { 0xb71cf785, 91 },
    { 0, 0 },
    { 0x6d0cf787, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560cf78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96cf793, 78 },
    { 0xdd8cf793, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71cf7a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x607cf7a4, 23 },
    { 0, 0 },
    { 0xeb2cf7a6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cecf7a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27cf7ae, 78 },
    { 0xddccf7af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67cf7b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x605cf7b5, 97 },
    { 0x918cf7b5, 80 },
    { 0xe3dcf7b5, 93 },
    { 0, 0 },
    { 0x3eacf7b9, 52 },
    { 0x535cf7b9, 23 },
    { 0, 0 },
    { 0x73dcf7bc, 90 },
    { 0x6f1cf7bc, 90 },
    { 0xb71cf7bd, 95 },
    { 0, 0 },
    { 0xe89cf7c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50cf7c4, 98 },
    { 0, 0 },
    { 0x396cf7c6, 43 },
    { 0, 0 },
    { 0xc7acf7c8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x197cf7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afcf7d2, 33 },
    { 0x36ccf7d3, 98 },
    { 0x12acf7d4, 55 },
    { 0x672cf7d5, 91 },
    { 0x7e4cf7d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67cf7dd, 43 },
    { 0x310cf7de, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afcf7e3, 23 },
    { 0x1a0cf7e4, 95 },
    { 0, 0 },
    { 0x67acf7e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8cf7e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25cf7ec, 93 },
    { 0x20ccf7ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dcf7f0, 90 },
    { 0x23cf7f1, 100 },
    { 0, 0 },
    { 0x659cf7f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x706cf7f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ccf7fb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ecf801, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62cf809, 91 },
    { 0xa91cf80a, 33 },
    { 0x891cf80a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x707cf80e, 61 },
    { 0xdddcf80f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7cf812, 55 },
    { 0xbe8cf813, 76 },
    { 0, 0 },
    { 0xa9ccf815, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ccf81a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcf81f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fcf827, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fcf82d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4cf833, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ecf843, 81 },
    { 0, 0 },
    { 0x3eecf845, 33 },
    { 0x751cf846, 90 },
    { 0, 0 },
    { 0x585cf848, 52 },
    { 0xb18cf849, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836cf84d, 100 },
    { 0x935cf84e, 33 },
    { 0x11bcf84f, 98 },
    { 0x888cf84f, 88 },
    { 0, 0 },
    { 0x36dcf852, 34 },
    { 0x944cf852, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dcf860, 95 },
    { 0x671cf860, 43 },
    { 0xf38cf862, 84 },
    { 0x3bacf863, 61 },
    { 0x92ecf862, 33 },
    { 0, 0 },
    { 0x839cf866, 80 },
    { 0, 0 },
    { 0xf1cf868, 23 },
    { 0xc8ecf869, 55 },
    { 0x555cf868, 33 },
    { 0, 0 },
    { 0xe0fcf86c, 34 },
    { 0, 0 },
    { 0x92bcf86e, 95 },
    { 0, 0 },
    { 0x90cf870, 23 },
    { 0x52acf871, 34 },
    { 0x49bcf872, 34 },
    { 0x404cf872, 33 },
    { 0xabdcf873, 85 },
    { 0x899cf873, 87 },
    { 0x50bcf876, 61 },
    { 0x49cf877, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x306cf87a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87cf87f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf883, 84 },
    { 0, 0 },
    { 0x1fccf885, 98 },
    { 0x922cf886, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cecf88b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacecf890, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bcf893, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bcf8a1, 98 },
    { 0x48ecf8a2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59acf8a7, 33 },
    { 0, 0 },
    { 0x421cf8a9, 90 },
    { 0, 0 },
    { 0xb1dcf8ab, 55 },
    { 0, 0 },
    { 0x2a3cf8ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dcf8b2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8cf8bd, 100 },
    { 0xbf1cf8be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9cf8c5, 43 },
    { 0xce3cf8c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8cf8cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcecf8d2, 33 },
    { 0x6dfcf8d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x784cf8d6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fcf8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438cf8de, 90 },
    { 0xcacf8df, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824cf8e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dcf8e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82cf8eb, 87 },
    { 0xb80cf8ec, 98 },
    { 0xa11cf8eb, 90 },
    { 0x50dcf8ee, 23 },
    { 0, 0 },
    { 0x250cf8f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ccf8f6, 33 },
    { 0x76dcf8f7, 91 },
    { 0xe7cf8f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422cf8fc, 46 },
    { 0xee6cf8fd, 23 },
    { 0x59acf8fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6cf905, 55 },
    { 0xfddcf905, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdcf909, 95 },
    { 0x75acf909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x621cf90d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dcf914, 66 },
    { 0x8b7cf915, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8cf918, 78 },
    { 0xcfecf918, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248cf91e, 87 },
    { 0, 0 },
    { 0xda9cf920, 61 },
    { 0x885cf921, 98 },
    { 0x66fcf922, 17 },
    { 0, 0 },
    { 0xc0fcf924, 90 },
    { 0x7d7cf925, 52 },
    { 0x9f7cf926, 88 },
    { 0x36ccf927, 98 },
    { 0x10bcf928, 90 },
    { 0, 0 },
    { 0x21ecf92a, 90 },
    { 0, 0 },
    { 0xffccf92c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437cf931, 95 },
    { 0, 0 },
    { 0x8becf933, 33 },
    { 0, 0 },
    { 0xfa3cf935, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52cf938, 90 },
    { 0, 0 },
    { 0xdecf93a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ecf944, 85 },
    { 0x904cf944, 66 },
    { 0x84acf946, 55 },
    { 0xc7bcf947, 95 },
    { 0xbeccf947, 93 },
    { 0xde0cf949, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6cf953, 34 },
    { 0xbaacf953, 33 },
    { 0x294cf955, 99 },
    { 0x737cf956, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbcf962, 98 },
    { 0x613cf963, 90 },
    { 0xd77cf963, 52 },
    { 0, 0 },
    { 0x6d7cf966, 17 },
    { 0x6e3cf966, 95 },
    { 0xcf8cf967, 46 },
    { 0x22acf969, 93 },
    { 0xcbccf96a, 43 },
    { 0x38ccf96a, 97 },
    { 0x108cf966, 55 },
    { 0xe7dcf966, 84 },
    { 0x2d8cf96e, 55 },
    { 0x171cf96f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6cf975, 34 },
    { 0xe11cf976, 95 },
    { 0, 0 },
    { 0xcc3cf978, 52 },
    { 0x735cf979, 34 },
    { 0x225cf97a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bcf97d, 55 },
    { 0xba5cf97d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfcf986, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x774cf989, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bcf98d, 55 },
    { 0xfcacf98e, 33 },
    { 0x22ccf98f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640cf993, 33 },
    { 0xeb4cf994, 33 },
    { 0xae5cf993, 61 },
    { 0x815cf994, 33 },
    { 0, 0 },
    { 0x970cf998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424cf99e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x609cf9a1, 34 },
    { 0xf45cf9a2, 61 },
    { 0xd77cf9a3, 33 },
    { 0xab5cf9a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17bcf9ab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7cf9b2, 33 },
    { 0, 0 },
    { 0x778cf9b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eccf9b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591cf9c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x971cf9c6, 99 },
    { 0xf68cf9c7, 33 },
    { 0x8b9cf9c8, 33 },
    { 0x89cf9c8, 87 },
    { 0x663cf9ca, 34 },
    { 0x45bcf9c9, 33 },
    { 0xdfcf9cc, 76 },
    { 0x316cf9ca, 43 },
    { 0x4eecf9cb, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26cf9d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74cf9d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbcf9dc, 23 },
    { 0x87ccf9dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87cf9e0, 78 },
    { 0xe8fcf9e0, 61 },
    { 0x999cf9e0, 93 },
    { 0xf7acf9e3, 33 },
    { 0x347cf9e4, 55 },
    { 0, 0 },
    { 0x75bcf9e6, 100 },
    { 0x3efcf9e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350cf9f6, 76 },
    { 0, 0 },
    { 0x298cf9f8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x433cf9fb, 55 },
    { 0, 0 },
    { 0x541cf9fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bcfa03, 43 },
    { 0x47acfa04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7cfa08, 23 },
    { 0, 0 },
    { 0xcffcfa0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15cfa0d, 84 },
    { 0x2adcfa0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ecfa13, 88 },
    { 0x592cfa13, 23 },
    { 0x17acfa15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabecfa19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13cfa1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfacfa24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976cfa35, 90 },
    { 0, 0 },
    { 0x88cfa37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2cfa3a, 68 },
    { 0xa1dcfa3b, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebcfa40, 23 },
    { 0x795cfa40, 34 },
    { 0, 0 },
    { 0xa81cfa43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fcfa46, 61 },
    { 0xf10cfa47, 98 },
    { 0, 0 },
    { 0xf9cfa49, 66 },
    { 0, 0 },
    { 0x3e7cfa4b, 61 },
    { 0, 0 },
    { 0xc4bcfa4d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbcfa50, 23 },
    { 0xdccfa51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8cfa54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3acfa5a, 61 },
    { 0xf0ccfa5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8acfa5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cfa69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fcfa79, 23 },
    { 0, 0 },
    { 0xf7ccfa7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3cfa81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1cfa84, 23 },
    { 0xcfcfa84, 66 },
    { 0xd4bcfa85, 23 },
    { 0, 0 },
    { 0xa78cfa88, 90 },
    { 0xc3ecfa89, 99 },
    { 0x4f7cfa8a, 55 },
    { 0x5a8cfa8b, 97 },
    { 0xb95cfa8b, 90 },
    { 0, 0 },
    { 0xe8cfa8e, 52 },
    { 0x1f1cfa8e, 98 },
    { 0x6e5cfa8e, 80 },
    { 0x643cfa91, 100 },
    { 0x380cfa91, 67 },
    { 0xe67cfa92, 90 },
    { 0xd85cfa93, 43 },
    { 0x9b2cfa95, 90 },
    { 0xb5bcfa96, 34 },
    { 0xbb3cfa92, 61 },
    { 0xb23cfa96, 33 },
    { 0x899cfa99, 61 },
    { 0, 0 },
    { 0xe9dcfa9b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7cfaa0, 9 },
    { 0x27dcfaa0, 23 },
    { 0x3e4cfaa2, 23 },
    { 0x971cfaa3, 43 },
    { 0x4c0cfaa4, 34 },
    { 0x9f7cfaa5, 98 },
    { 0x582cfaa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ccfaab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59cfab1, 33 },
    { 0x907cfab1, 33 },
    { 0x6e4cfab3, 17 },
    { 0x1bdcfab1, 76 },
    { 0xc06cfab4, 95 },
    { 0, 0 },
    { 0x289cfab7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8cfabb, 76 },
    { 0x860cfabb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x292cfabf, 100 },
    { 0, 0 },
    { 0xfa5cfac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44cfac4, 95 },
    { 0xb8fcfac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ecfac8, 33 },
    { 0, 0 },
    { 0x680cfaca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dcfad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17cfad4, 90 },
    { 0, 0 },
    { 0xffdcfad6, 61 },
    { 0, 0 },
    { 0x308cfad8, 91 },
    { 0xf4cfad9, 90 },
    { 0xa1bcfada, 78 },
    { 0xe64cfad8, 23 },
    { 0xdaccfadc, 66 },
    { 0, 0 },
    { 0x6e7cfade, 33 },
    { 0, 0 },
    { 0x9b6cfae0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95cfae7, 95 },
    { 0x936cfae7, 67 },
    { 0x699cfae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57acfaed, 75 },
    { 0x9cdcfaed, 23 },
    { 0x334cfaed, 23 },
    { 0x7e0cfaf0, 33 },
    { 0, 0 },
    { 0x254cfaf2, 23 },
    { 0, 0 },
    { 0x41ecfaf4, 95 },
    { 0xfbccfaf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496cfafa, 23 },
    { 0xc03cfafb, 61 },
    { 0xc7cfafc, 95 },
    { 0, 0 },
    { 0xf13cfafe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5cfb04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbacfb08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x417cfb0b, 84 },
    { 0x83dcfb0c, 23 },
    { 0, 0 },
    { 0x506cfb0e, 61 },
    { 0x88fcfb0e, 33 },
    { 0xd90cfb10, 33 },
    { 0x59bcfb11, 23 },
    { 0, 0 },
    { 0x19acfb13, 33 },
    { 0, 0 },
    { 0x694cfb15, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532cfb1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3cfb1e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fcfb22, 33 },
    { 0xd4ccfb23, 66 },
    { 0xb27cfb22, 100 },
    { 0x17ccfb25, 23 },
    { 0x710cfb26, 69 },
    { 0xe7dcfb24, 99 },
    { 0x116cfb22, 69 },
    { 0, 0 },
    { 0x5dbcfb2a, 33 },
    { 0, 0 },
    { 0xbdbcfb2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddcfb30, 66 },
    { 0x160cfb31, 61 },
    { 0, 0 },
    { 0xcc5cfb33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86acfb39, 90 },
    { 0, 0 },
    { 0xc6dcfb3b, 61 },
    { 0x34fcfb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x577cfb3f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6cfb42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6cfb48, 55 },
    { 0, 0 },
    { 0x912cfb4a, 93 },
    { 0, 0 },
    { 0x391cfb4c, 78 },
    { 0xe3ccfb4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2cfb53, 95 },
    { 0x2e8cfb54, 61 },
    { 0xf96cfb54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x745cfb58, 61 },
    { 0x219cfb58, 67 },
    { 0, 0 },
    { 0xf4fcfb5b, 90 },
    { 0x3ebcfb5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cfb61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2cfb64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925cfb68, 90 },
    { 0xb56cfb68, 78 },
    { 0xf37cfb68, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbcfb6d, 34 },
    { 0x1aecfb6d, 98 },
    { 0xeb3cfb6f, 33 },
    { 0x188cfb70, 91 },
    { 0x466cfb6f, 33 },
    { 0xddcfb72, 60 },
    { 0xa57cfb73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfb76, 76 },
    { 0x67acfb77, 75 },
    { 0xfc6cfb78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4cfb7c, 43 },
    { 0, 0 },
    { 0x6dcfb7e, 61 },
    { 0x9f3cfb7e, 87 },
    { 0x883cfb80, 33 },
    { 0, 0 },
    { 0x751cfb82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11cfb86, 84 },
    { 0xf70cfb87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5acfb8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3cfb91, 67 },
    { 0, 0 },
    { 0xd1cfb93, 76 },
    { 0, 0 },
    { 0xcbacfb95, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefcfb9b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ecfba1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750cfbac, 98 },
    { 0x1c0cfbac, 33 },
    { 0xd80cfbae, 91 },
    { 0x637cfbaf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940cfbb8, 88 },
    { 0x4e9cfbb9, 23 },
    { 0x41bcfbb9, 90 },
    { 0, 0 },
    { 0x421cfbbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dcfbc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957cfbc7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854cfbcb, 34 },
    { 0x57cfbcb, 95 },
    { 0, 0 },
    { 0xdc2cfbce, 95 },
    { 0, 0 },
    { 0x16dcfbd0, 65 },
    { 0, 0 },
    { 0x6acfbd2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808cfbd9, 52 },
    { 0, 0 },
    { 0xdeacfbdb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ccfbe1, 46 },
    { 0x278cfbe2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cfbe7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39cfbef, 46 },
    { 0x48bcfbf0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfcfbfa, 61 },
    { 0, 0 },
    { 0x1fcfbfc, 61 },
    { 0xf2ccfbfd, 98 },
    { 0x7c5cfbfc, 88 },
    { 0x653cfbff, 43 },
    { 0xf3ecfbff, 33 },
    { 0x9edcfc01, 61 },
    { 0x793cfbff, 46 },
    { 0x16cfbfc, 95 },
    { 0, 0 },
    { 0x315cfc05, 90 },
    { 0x848cfc06, 34 },
    { 0xe6ccfc06, 90 },
    { 0, 0 },
    { 0xdb6cfc09, 61 },
    { 0x837cfc0a, 33 },
    { 0x5f0cfc0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efcfc0e, 46 },
    { 0x1e1cfc0e, 55 },
    { 0xbc7cfc10, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0cfc1a, 34 },
    { 0xef6cfc1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04cfc1e, 34 },
    { 0x6d8cfc1e, 61 },
    { 0xd29cfc1e, 46 },
    { 0xaafcfc21, 23 },
    { 0, 0 },
    { 0x89acfc23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fcfc2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696cfc33, 87 },
    { 0x67ccfc34, 91 },
    { 0x72acfc34, 61 },
    { 0x666cfc36, 34 },
    { 0xa45cfc33, 95 },
    { 0xaffcfc38, 61 },
    { 0xf86cfc33, 33 },
    { 0xfa6cfc39, 55 },
    { 0x97cfc3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x821cfc3e, 61 },
    { 0, 0 },
    { 0x25acfc40, 98 },
    { 0x4fcfc40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x682cfc44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76acfc49, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84cfc4c, 23 },
    { 0, 0 },
    { 0xbabcfc4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00cfc60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538cfc64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fcfc68, 23 },
    { 0, 0 },
    { 0xe42cfc6a, 66 },
    { 0x409cfc6b, 33 },
    { 0x8a8cfc6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5cfc6f, 90 },
    { 0x35ecfc70, 23 },
    { 0xbf3cfc71, 33 },
    { 0x50acfc71, 61 },
    { 0x12bcfc70, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8cfc76, 95 },
    { 0x4d9cfc77, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9cfc7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fcfc7e, 34 },
    { 0x6b9cfc7f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ecfc83, 98 },
    { 0, 0 },
    { 0x396cfc85, 90 },
    { 0xa94cfc86, 33 },
    { 0, 0 },
    { 0xa89cfc88, 33 },
    { 0xbd0cfc88, 76 },
    { 0, 0 },
    { 0x1fdcfc8b, 87 },
    { 0x5e5cfc8c, 44 },
    { 0x577cfc8d, 93 },
    { 0x446cfc8e, 36 },
    { 0x76bcfc8e, 98 },
    { 0x73fcfc90, 90 },
    { 0x4c5cfc90, 95 },
    { 0x91dcfc92, 61 },
    { 0x6a3cfc93, 46 },
    { 0x234cfc92, 90 },
    { 0, 0 },
    { 0xd5ccfc96, 95 },
    { 0x9fcfc96, 98 },
    { 0x4cfcfc97, 95 },
    { 0xd8fcfc99, 61 },
    { 0, 0 },
    { 0x44fcfc9b, 83 },
    { 0, 0 },
    { 0xe58cfc9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28cfca4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efcfcaa, 100 },
    { 0xea5cfcab, 33 },
    { 0, 0 },
    { 0x84ecfcad, 95 },
    { 0, 0 },
    { 0x1ccfcaf, 90 },
    { 0, 0 },
    { 0x192cfcb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffcfcb4, 33 },
    { 0, 0 },
    { 0x1c9cfcb6, 52 },
    { 0xd1acfcb6, 100 },
    { 0, 0 },
    { 0xceecfcb9, 98 },
    { 0, 0 },
    { 0x57fcfcbb, 23 },
    { 0x27fcfcbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0cfcbf, 61 },
    { 0x56acfcbf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404cfcc8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8acfccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9cfcd2, 23 },
    { 0x2e5cfcd3, 67 },
    { 0x459cfcd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8accfcd9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bcfce2, 90 },
    { 0, 0 },
    { 0x504cfce4, 100 },
    { 0x581cfce5, 61 },
    { 0xaa4cfce4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4cfce9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bcfced, 76 },
    { 0x580cfced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3cfcf4, 95 },
    { 0, 0 },
    { 0x978cfcf6, 61 },
    { 0xd6fcfcf7, 67 },
    { 0xba5cfcf8, 84 },
    { 0, 0 },
    { 0xbd0cfcfa, 76 },
    { 0x8d8cfcfb, 90 },
    { 0x2b9cfcfa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09cfd01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7cfd04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71cfd07, 78 },
    { 0x5e2cfd08, 23 },
    { 0xfeacfd07, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251cfd12, 61 },
    { 0xbd7cfd13, 90 },
    { 0, 0 },
    { 0xbb3cfd15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571cfd1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7cfd24, 99 },
    { 0x982cfd25, 90 },
    { 0, 0 },
    { 0xab6cfd27, 23 },
    { 0x410cfd28, 66 },
    { 0x169cfd28, 93 },
    { 0x1d1cfd2a, 67 },
    { 0x7e8cfd29, 90 },
    { 0xee4cfd2c, 33 },
    { 0xd3cfd2d, 55 },
    { 0x781cfd28, 55 },
    { 0x71dcfd2c, 98 },
    { 0x15fcfd2d, 43 },
    { 0, 0 },
    { 0x5b7cfd32, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56cfd39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bcfd3c, 23 },
    { 0x5d1cfd3c, 33 },
    { 0, 0 },
    { 0xa8ccfd3f, 66 },
    { 0xc28cfd3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155cfd45, 61 },
    { 0x54ecfd46, 33 },
    { 0x589cfd47, 76 },
    { 0xc11cfd48, 90 },
    { 0x91cfd45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0cfd4d, 90 },
    { 0xc2acfd4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1decfd51, 33 },
    { 0xd6cfd52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47cfd55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192cfd59, 98 },
    { 0xda6cfd5a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ecfd5f, 95 },
    { 0xdfacfd5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3cfd67, 90 },
    { 0x305cfd67, 95 },
    { 0, 0 },
    { 0xd5dcfd6a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eccfd70, 33 },
    { 0x23bcfd70, 61 },
    { 0x9d0cfd71, 98 },
    { 0, 0 },
    { 0xef8cfd74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4cfd78, 87 },
    { 0xb8bcfd79, 11 },
    { 0x3eccfd7a, 52 },
    { 0, 0 },
    { 0x848cfd7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910cfd83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307cfd8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1dcfd90, 95 },
    { 0x626cfd91, 77 },
    { 0xb72cfd90, 61 },
    { 0xa2dcfd93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bcfd98, 95 },
    { 0xbbbcfd99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77cfd9c, 61 },
    { 0x371cfd9d, 48 },
    { 0xdf0cfd9e, 33 },
    { 0, 0 },
    { 0x9a1cfda0, 33 },
    { 0xe29cfda1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3cfda5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bcfda8, 43 },
    { 0xc94cfda9, 91 },
    { 0xb0bcfdaa, 34 },
    { 0xa21cfdab, 33 },
    { 0xeb1cfdaa, 55 },
    { 0x4cbcfdad, 61 },
    { 0, 0 },
    { 0x737cfdaf, 43 },
    { 0x5c3cfdb0, 55 },
    { 0xf0cfdb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcfdb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x876cfdb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941cfdbd, 95 },
    { 0x716cfdbe, 46 },
    { 0x13dcfdbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdcfdc2, 33 },
    { 0, 0 },
    { 0xaeecfdc4, 67 },
    { 0xdc3cfdc5, 23 },
    { 0, 0 },
    { 0x613cfdc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dcfdca, 77 },
    { 0x55fcfdcb, 90 },
    { 0x8bfcfdcb, 100 },
    { 0xd27cfdcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fcfdd0, 33 },
    { 0x5becfdd1, 90 },
    { 0, 0 },
    { 0x527cfdd3, 94 },
    { 0, 0 },
    { 0xd44cfdd5, 33 },
    { 0x82bcfdd6, 23 },
    { 0x8b5cfdd5, 23 },
    { 0xa00cfdd5, 34 },
    { 0xec2cfdd9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58cfde5, 66 },
    { 0x892cfde6, 61 },
    { 0xf03cfde5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98cfdeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7cfdee, 90 },
    { 0x505cfdef, 90 },
    { 0xe4ecfdef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dcfdf6, 69 },
    { 0x411cfdf6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53cfdfc, 43 },
    { 0x691cfdfc, 91 },
    { 0xf05cfdfe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ccfe05, 95 },
    { 0x9e2cfe05, 34 },
    { 0xec5cfe05, 34 },
    { 0xd67cfe06, 98 },
    { 0, 0 },
    { 0xa61cfe0a, 55 },
    { 0x896cfe0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371cfe0f, 99 },
    { 0xc6fcfe0f, 91 },
    { 0xec9cfe11, 95 },
    { 0, 0 },
    { 0xb42cfe13, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4cfe16, 97 },
    { 0x6eccfe17, 34 },
    { 0x8f1cfe18, 76 },
    { 0x563cfe18, 87 },
    { 0, 0 },
    { 0xb8cfe1b, 61 },
    { 0, 0 },
    { 0x2ebcfe1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3cfe22, 33 },
    { 0x1a0cfe22, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52cfe2b, 55 },
    { 0, 0 },
    { 0x4f3cfe2d, 87 },
    { 0xe05cfe2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535cfe38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7cfe3e, 90 },
    { 0, 0 },
    { 0x940cfe40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2cfe45, 23 },
    { 0x795cfe46, 90 },
    { 0x8bcfe47, 55 },
    { 0x87ecfe48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834cfe4f, 40 },
    { 0, 0 },
    { 0x857cfe51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36acfe56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fecfe5e, 90 },
    { 0, 0 },
    { 0xaa4cfe60, 95 },
    { 0x3e8cfe61, 90 },
    { 0xa57cfe62, 98 },
    { 0, 0 },
    { 0x19bcfe64, 90 },
    { 0, 0 },
    { 0x15ecfe66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ecfe6c, 90 },
    { 0, 0 },
    { 0x1e0cfe6e, 81 },
    { 0, 0 },
    { 0xf59cfe70, 90 },
    { 0x206cfe71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606cfe79, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x296cfe7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0acfe83, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36cfe86, 99 },
    { 0, 0 },
    { 0xc1bcfe88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1cfe8f, 90 },
    { 0xc78cfe8f, 90 },
    { 0xae7cfe91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ecfe94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcccfe9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05cfe9e, 96 },
    { 0xba3cfe9f, 61 },
    { 0x705cfe9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dcfea4, 90 },
    { 0, 0 },
    { 0x543cfea6, 61 },
    { 0xe4acfea7, 23 },
    { 0x8b4cfea8, 55 },
    { 0x14fcfea9, 78 },
    { 0x7ddcfeaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfcfeb7, 76 },
    { 0xc3fcfeb8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5cfebd, 87 },
    { 0x30bcfebe, 61 },
    { 0x6edcfebe, 33 },
    { 0, 0 },
    { 0xd87cfec1, 98 },
    { 0x56dcfec1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27cfec7, 55 },
    { 0x703cfec8, 34 },
    { 0xe9dcfec7, 33 },
    { 0, 0 },
    { 0x6b4cfecb, 85 },
    { 0x8d1cfecb, 68 },
    { 0x1acfecd, 98 },
    { 0xe14cfece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dcfed1, 46 },
    { 0x20ccfed2, 98 },
    { 0xf4fcfed2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411cfed9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2cfedc, 90 },
    { 0x465cfedc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463cfee3, 76 },
    { 0x5e7cfee4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xefecfee7, 98 },
    { 0, 0 },
    { 0x3c2cfee9, 90 },
    { 0x2adcfeea, 92 },
    { 0x5d7cfee9, 33 },
    { 0x198cfeeb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8cfef1, 90 },
    { 0, 0 },
    { 0xa4cfef3, 76 },
    { 0xe72cfef4, 43 },
    { 0xeb1cfef5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5cfef8, 76 },
    { 0, 0 },
    { 0xf56cfefa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89cfefd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dcff02, 90 },
    { 0x3cacff03, 76 },
    { 0x8d9cff03, 66 },
    { 0x4d3cff05, 76 },
    { 0xb4ecff03, 46 },
    { 0, 0 },
    { 0x4cccff08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ccff0c, 34 },
    { 0x990cff0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1cff12, 98 },
    { 0x844cff13, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33cff17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4cff1a, 33 },
    { 0, 0 },
    { 0x973cff1c, 76 },
    { 0, 0 },
    { 0xdfcff1e, 100 },
    { 0, 0 },
    { 0xce8cff20, 98 },
    { 0x12bcff21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cff24, 90 },
    { 0xcfecff24, 90 },
    { 0xa91cff26, 61 },
    { 0, 0 },
    { 0x1c8cff28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fcff2b, 95 },
    { 0, 0 },
    { 0x917cff2d, 55 },
    { 0, 0 },
    { 0xba5cff2f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dcff36, 34 },
    { 0x404cff37, 56 },
    { 0xe27cff38, 91 },
    { 0x777cff38, 100 },
    { 0xf10cff39, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28cff3e, 33 },
    { 0x645cff3e, 97 },
    { 0, 0 },
    { 0x474cff41, 90 },
    { 0xa84cff41, 33 },
    { 0x27acff42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcff50, 90 },
    { 0, 0 },
    { 0xaf9cff52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346cff5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93acff5e, 17 },
    { 0xbb6cff5f, 76 },
    { 0xf0cff5f, 23 },
    { 0x420cff61, 98 },
    { 0x1cfcff62, 61 },
    { 0xc3dcff60, 81 },
    { 0, 0 },
    { 0x765cff65, 76 },
    { 0x5a6cff66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4cff6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dcff77, 78 },
    { 0, 0 },
    { 0xc8ccff79, 95 },
    { 0x541cff79, 95 },
    { 0, 0 },
    { 0x78ccff7c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x193cff7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353cff84, 100 },
    { 0, 0 },
    { 0xa5dcff86, 40 },
    { 0xa98cff87, 90 },
    { 0xececff86, 95 },
    { 0, 0 },
    { 0x33bcff8a, 81 },
    { 0xb92cff8b, 95 },
    { 0xf04cff8c, 98 },
    { 0xf50cff8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4cff94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x603cff97, 64 },
    { 0x54acff98, 98 },
    { 0, 0 },
    { 0x411cff9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2cffa0, 98 },
    { 0xe7bcffa0, 33 },
    { 0x3adcffa1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0cffa5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fcffa8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174cffac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbcffb2, 95 },
    { 0x35bcffb2, 33 },
    { 0x62bcffb3, 76 },
    { 0, 0 },
    { 0x33dcffb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117cffbc, 90 },
    { 0x4cffbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2cffc2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38cffc7, 77 },
    { 0xaa4cffc8, 55 },
    { 0x8eacffc9, 97 },
    { 0x2c1cffc9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcffcf, 99 },
    { 0x2c3cffd0, 98 },
    { 0, 0 },
    { 0xcafcffd2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512cffd6, 23 },
    { 0, 0 },
    { 0x38dcffd8, 93 },
    { 0xc77cffd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cffdd, 61 },
    { 0x859cffdd, 75 },
    { 0x66ccffdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48cffe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17cffe7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3cffec, 95 },
    { 0, 0 },
    { 0xc9cffee, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445cfff5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x445cfff8, 93 },
    { 0xdc7cfff9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bcfffc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cd0000, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bd0004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6d0008, 33 },
    { 0xc64d0009, 69 },
    { 0x953d000a, 90 },
    { 0x76fd000b, 95 },
    { 0xaf2d000a, 94 },
    { 0xad4d0009, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dd0011, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94d0018, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d001b, 98 },
    { 0x6cdd001b, 76 },
    { 0, 0 },
    { 0x856d001e, 98 },
    { 0x19fd001f, 93 },
    { 0x2b5d0020, 66 },
    { 0xfa9d0021, 99 },
    { 0x66ed0021, 92 },
    { 0x3fad0022, 81 },
    { 0x720d0024, 76 },
    { 0x7d0d0023, 33 },
    { 0x167d0026, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd0029, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8d002d, 34 },
    { 0, 0 },
    { 0x2e7d002f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1d0033, 43 },
    { 0xa3ed0034, 98 },
    { 0x93d0034, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bd0039, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17d003d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabd0041, 88 },
    { 0xd2d0041, 17 },
    { 0x728d0043, 95 },
    { 0x979d0044, 33 },
    { 0x652d0044, 90 },
    { 0x287d0041, 100 },
    { 0xd71d0047, 34 },
    { 0x718d0047, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x516d004b, 23 },
    { 0, 0 },
    { 0xd2fd004d, 90 },
    { 0xb68d004d, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6d0058, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64d005f, 95 },
    { 0, 0 },
    { 0x545d0061, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d0068, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5d006b, 33 },
    { 0xa3dd006c, 33 },
    { 0x47dd006d, 98 },
    { 0, 0 },
    { 0xc30d006f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47d0074, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d0078, 98 },
    { 0x26fd0078, 99 },
    { 0x709d007a, 33 },
    { 0x6c2d007b, 43 },
    { 0xac7d007c, 46 },
    { 0x67cd0078, 23 },
    { 0x3dcd007e, 90 },
    { 0xe38d007f, 55 },
    { 0x371d0080, 23 },
    { 0xe3ad0081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d0084, 33 },
    { 0x202d0084, 98 },
    { 0x766d0086, 33 },
    { 0x67d0087, 77 },
    { 0, 0 },
    { 0xd57d0089, 95 },
    { 0, 0 },
    { 0xe07d008b, 36 },
    { 0, 0 },
    { 0x334d008d, 76 },
    { 0, 0 },
    { 0xcbfd008f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986d0093, 98 },
    { 0x147d0094, 78 },
    { 0, 0 },
    { 0xd99d0096, 95 },
    { 0, 0 },
    { 0x4d1d0098, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d00a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4d00ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xabad00af, 66 },
    { 0x741d00b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48d00b7, 90 },
    { 0, 0 },
    { 0xd92d00b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6d00bc, 37 },
    { 0xb40d00bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cad00c2, 87 },
    { 0, 0 },
    { 0x36fd00c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25d00d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccd00d6, 55 },
    { 0, 0 },
    { 0xac9d00d8, 23 },
    { 0xfedd00d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdd00dd, 77 },
    { 0, 0 },
    { 0x4f2d00df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dd00e8, 61 },
    { 0, 0 },
    { 0xe0ad00ea, 92 },
    { 0, 0 },
    { 0xef5d00ec, 61 },
    { 0x2d3d00ec, 99 },
    { 0x86cd00ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725d00f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d00fa, 33 },
    { 0, 0 },
    { 0x75d00fc, 23 },
    { 0, 0 },
    { 0x103d00fe, 76 },
    { 0xfc4d00fe, 23 },
    { 0xb3cd0100, 78 },
    { 0x806d0101, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04d0106, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fd010a, 43 },
    { 0, 0 },
    { 0x5e4d010c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x455d010f, 61 },
    { 0, 0 },
    { 0xcc8d0111, 61 },
    { 0xe8bd0112, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7d0116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ed011b, 90 },
    { 0x807d011b, 95 },
    { 0x4c7d011d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319d0124, 95 },
    { 0x445d0124, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fad0128, 95 },
    { 0x630d0128, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ad0130, 76 },
    { 0xb13d0130, 61 },
    { 0xadbd0132, 100 },
    { 0xa91d0130, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5d0138, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d0141, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x213d0144, 90 },
    { 0xd83d0145, 95 },
    { 0xbcdd0146, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x601d0149, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5d015b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dd0163, 76 },
    { 0, 0 },
    { 0x252d0165, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd0168, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8d016b, 23 },
    { 0, 0 },
    { 0x81ad016d, 61 },
    { 0xfaad016e, 87 },
    { 0xa15d016d, 98 },
    { 0x6ed016e, 55 },
    { 0xda5d0170, 76 },
    { 0, 0 },
    { 0xf42d0173, 95 },
    { 0, 0 },
    { 0x5b1d0175, 95 },
    { 0x4dcd0176, 78 },
    { 0x62dd0176, 55 },
    { 0xf3bd0176, 76 },
    { 0x670d0179, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d017e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100d0183, 61 },
    { 0x714d0183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52d0187, 61 },
    { 0, 0 },
    { 0xad2d0189, 23 },
    { 0xb2ad018a, 98 },
    { 0xa27d018a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644d0190, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d0194, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fd019b, 33 },
    { 0xc1ed019b, 46 },
    { 0x4ad019c, 90 },
    { 0x30d019e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdd01a5, 23 },
    { 0x511d01a6, 85 },
    { 0xe5d01a5, 95 },
    { 0, 0 },
    { 0x261d01a9, 78 },
    { 0xd4bd01a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccd01b2, 95 },
    { 0x90ed01b2, 55 },
    { 0, 0 },
    { 0xc0dd01b5, 100 },
    { 0, 0 },
    { 0xdebd01b7, 61 },
    { 0, 0 },
    { 0x1d4d01b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fd01c1, 98 },
    { 0xd62d01c2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d01c5, 48 },
    { 0x33ed01c6, 33 },
    { 0x28cd01c6, 61 },
    { 0xa4ad01c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d01cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594d01cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cd01d2, 90 },
    { 0, 0 },
    { 0x6f4d01d4, 33 },
    { 0x798d01d5, 34 },
    { 0x894d01d6, 61 },
    { 0x35dd01d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dd01de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d01e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6add01e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fd01eb, 78 },
    { 0, 0 },
    { 0x94bd01ed, 46 },
    { 0xb84d01ee, 77 },
    { 0x6b2d01ef, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xafed01f2, 87 },
    { 0xcdfd01f3, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28d01ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7d0204, 46 },
    { 0x3c3d0204, 55 },
    { 0x73bd0206, 87 },
    { 0xc6ad0207, 75 },
    { 0x881d0206, 90 },
    { 0xcc7d0205, 85 },
    { 0xf3ad0204, 99 },
    { 0, 0 },
    { 0x6bad020c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffd020f, 78 },
    { 0xb21d020f, 90 },
    { 0, 0 },
    { 0x5f9d0212, 67 },
    { 0x924d0213, 76 },
    { 0, 0 },
    { 0x87ad0215, 98 },
    { 0xb83d0216, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cd0219, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3d0221, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cd0225, 90 },
    { 0, 0 },
    { 0xb1ad0227, 90 },
    { 0xfad0228, 67 },
    { 0, 0 },
    { 0xc32d022a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9d0230, 98 },
    { 0xfedd0231, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebd0234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853d0239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bd023c, 61 },
    { 0xa1cd023d, 33 },
    { 0x890d023c, 66 },
    { 0x7c7d023c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bd0242, 43 },
    { 0x9d2d0243, 55 },
    { 0, 0 },
    { 0x703d0245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4d0249, 23 },
    { 0, 0 },
    { 0x844d024b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ad024e, 98 },
    { 0x3a7d024f, 33 },
    { 0xa3ed024f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15d0254, 74 },
    { 0xe04d0255, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4d0258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322d025d, 98 },
    { 0x209d025e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfd0264, 78 },
    { 0x7fbd0264, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ed0269, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d026d, 33 },
    { 0xd31d026e, 23 },
    { 0x7a0d026e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d0275, 90 },
    { 0x441d0276, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07d027c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fd0280, 95 },
    { 0xa16d0280, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bd0285, 55 },
    { 0, 0 },
    { 0xdb2d0287, 55 },
    { 0xcbfd0288, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cd028c, 95 },
    { 0, 0 },
    { 0x14fd028e, 65 },
    { 0xa9ed028f, 90 },
    { 0xe6ad028f, 23 },
    { 0, 0 },
    { 0x58cd0292, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88d0295, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x524d0298, 65 },
    { 0xe19d0298, 76 },
    { 0x6f5d029a, 61 },
    { 0x464d0298, 94 },
    { 0x50bd029c, 46 },
    { 0, 0 },
    { 0xb88d029e, 98 },
    { 0x899d029f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66d02a2, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fd02a5, 66 },
    { 0x4ead02a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdd02ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edd02ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00d02b5, 84 },
    { 0, 0 },
    { 0xd65d02b7, 33 },
    { 0x66ed02b8, 90 },
    { 0, 0 },
    { 0x936d02ba, 33 },
    { 0xc53d02bb, 95 },
    { 0xd47d02bc, 90 },
    { 0xd93d02bb, 23 },
    { 0xa25d02be, 52 },
    { 0xe6cd02ba, 90 },
    { 0, 0 },
    { 0xee2d02c1, 23 },
    { 0, 0 },
    { 0x196d02c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecd02c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72d02cc, 67 },
    { 0x968d02cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964d02d3, 90 },
    { 0xa6ad02d4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7d02d8, 93 },
    { 0x8bdd02d9, 33 },
    { 0xa2bd02da, 61 },
    { 0x645d02da, 95 },
    { 0xc4d02d8, 33 },
    { 0x8dd02dc, 23 },
    { 0xd1fd02de, 33 },
    { 0x92ad02de, 33 },
    { 0x9e4d02df, 95 },
    { 0xfb1d02e1, 52 },
    { 0xc2ad02e0, 23 },
    { 0x8a2d02e3, 55 },
    { 0x1f5d02df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d02ec, 90 },
    { 0, 0 },
    { 0x500d02ee, 95 },
    { 0xbb3d02ef, 95 },
    { 0x9b2d02ee, 99 },
    { 0, 0 },
    { 0xbeed02f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0d02f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd02fa, 67 },
    { 0, 0 },
    { 0x60d02fc, 99 },
    { 0, 0 },
    { 0xcdad02fe, 95 },
    { 0, 0 },
    { 0x2b1d0300, 90 },
    { 0, 0 },
    { 0xf2d0302, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ed030a, 33 },
    { 0, 0 },
    { 0x985d030c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d0310, 91 },
    { 0, 0 },
    { 0xa73d0312, 33 },
    { 0x3fbd0312, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429d0318, 90 },
    { 0x519d0319, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dd031d, 64 },
    { 0x58fd031e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3d0321, 98 },
    { 0, 0 },
    { 0x38bd0323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5d0326, 23 },
    { 0, 0 },
    { 0x7cdd0328, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d032d, 46 },
    { 0, 0 },
    { 0x228d032f, 66 },
    { 0, 0 },
    { 0xe05d0331, 61 },
    { 0xcb9d0331, 95 },
    { 0, 0 },
    { 0x1d6d0334, 43 },
    { 0x4e7d0334, 95 },
    { 0xb49d0336, 90 },
    { 0xf91d0337, 33 },
    { 0, 0 },
    { 0x7bad0339, 55 },
    { 0xb0fd033a, 93 },
    { 0xbdad033b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cd033e, 76 },
    { 0xb9bd033e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2d0342, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddd0347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecd034e, 95 },
    { 0, 0 },
    { 0x35dd0350, 86 },
    { 0x8c7d0351, 23 },
    { 0, 0 },
    { 0xe37d0353, 33 },
    { 0, 0 },
    { 0xa6dd0355, 55 },
    { 0x530d0356, 95 },
    { 0x801d0356, 66 },
    { 0xda3d0356, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3d035b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21d0360, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7d0364, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26d0367, 65 },
    { 0x374d0368, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0370, 61 },
    { 0x3e9d0371, 93 },
    { 0xe6cd0371, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6d0375, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363d037b, 55 },
    { 0, 0 },
    { 0xfddd037d, 95 },
    { 0x12ed037d, 95 },
    { 0, 0 },
    { 0x23d0380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9d0383, 55 },
    { 0xf72d0384, 23 },
    { 0xc8d0384, 90 },
    { 0xcfed0384, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201d038b, 93 },
    { 0, 0 },
    { 0x125d038d, 33 },
    { 0, 0 },
    { 0x98ad038f, 33 },
    { 0xcadd0390, 100 },
    { 0x24dd0390, 98 },
    { 0xe1fd0392, 90 },
    { 0x7cfd0393, 98 },
    { 0xa85d0394, 100 },
    { 0x95d0394, 95 },
    { 0x8d8d0396, 78 },
    { 0x597d0397, 78 },
    { 0xce0d0396, 61 },
    { 0x7a4d0396, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ed039c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5d039f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dd03a2, 77 },
    { 0x1add03a3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834d03a8, 93 },
    { 0, 0 },
    { 0x709d03aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41d03b0, 90 },
    { 0xe81d03b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dd03b5, 78 },
    { 0x1cad03b5, 76 },
    { 0xa5cd03b7, 99 },
    { 0x279d03b8, 55 },
    { 0, 0 },
    { 0x73ad03ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d03bf, 80 },
    { 0, 0 },
    { 0x24cd03c1, 23 },
    { 0xb4fd03c2, 43 },
    { 0xf5bd03c3, 23 },
    { 0x45fd03c3, 46 },
    { 0x891d03c5, 98 },
    { 0x4d4d03c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4d03c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3d03d2, 84 },
    { 0, 0 },
    { 0xa6d03d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227d03d8, 31 },
    { 0xe0ad03d9, 33 },
    { 0xed6d03d9, 95 },
    { 0xca6d03db, 91 },
    { 0, 0 },
    { 0x88fd03dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abd03e1, 98 },
    { 0x875d03e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d03e5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaed03ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7d03f2, 36 },
    { 0x795d03f3, 43 },
    { 0, 0 },
    { 0x2efd03f5, 87 },
    { 0, 0 },
    { 0x9bbd03f7, 87 },
    { 0x7f4d03f8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ed03fc, 38 },
    { 0xd8fd03fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ad0400, 90 },
    { 0x55ed0400, 33 },
    { 0xfa9d0402, 61 },
    { 0x435d0401, 33 },
    { 0xc0d0404, 90 },
    { 0x7e8d0401, 84 },
    { 0xbbcd0401, 99 },
    { 0, 0 },
    { 0xa1dd0408, 99 },
    { 0x748d0408, 90 },
    { 0, 0 },
    { 0x4fcd040b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd0421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7d0424, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980d0428, 55 },
    { 0x3bd0429, 90 },
    { 0x565d042a, 66 },
    { 0x7d3d042a, 33 },
    { 0, 0 },
    { 0xdcad042d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccd0430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809d0434, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd043a, 93 },
    { 0x5b3d043b, 33 },
    { 0x3d4d043c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d043f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d0442, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded0447, 33 },
    { 0x1c5d0448, 43 },
    { 0, 0 },
    { 0xd67d044a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x129d044d, 34 },
    { 0, 0 },
    { 0xa2bd044f, 90 },
    { 0xa02d0450, 78 },
    { 0xdb6d0451, 56 },
    { 0xd56d0450, 23 },
    { 0x24dd0450, 90 },
    { 0x55dd0454, 23 },
    { 0x983d0450, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344d045a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffd045d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525d0466, 95 },
    { 0x9ccd0466, 43 },
    { 0, 0 },
    { 0xfa9d0469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1d046f, 61 },
    { 0x3e4d0470, 95 },
    { 0x745d0471, 93 },
    { 0x163d0470, 23 },
    { 0x269d0473, 60 },
    { 0x727d0473, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80d0478, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4d047d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46d0483, 23 },
    { 0x125d0484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989d048b, 23 },
    { 0x66ed048c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ced0491, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bd0494, 23 },
    { 0x89fd0494, 34 },
    { 0x1b5d0496, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8d04a0, 55 },
    { 0x92cd04a1, 94 },
    { 0, 0 },
    { 0x366d04a3, 67 },
    { 0xf98d04a4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfd04ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d04b4, 16 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227d04bb, 23 },
    { 0xdded04bb, 95 },
    { 0, 0 },
    { 0xf37d04be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed04c4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x751d04c7, 61 },
    { 0xff4d04c8, 61 },
    { 0x20dd04c9, 99 },
    { 0xbd3d04ca, 95 },
    { 0x9f0d04cb, 78 },
    { 0x44ad04cc, 23 },
    { 0xe75d04cc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd04d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd04d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51d04d6, 95 },
    { 0xc8d04d7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd04da, 66 },
    { 0x135d04db, 33 },
    { 0x159d04dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad04e0, 84 },
    { 0, 0 },
    { 0xca3d04e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ad04e6, 43 },
    { 0, 0 },
    { 0xfc5d04e8, 98 },
    { 0x89ad04e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ad04f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5d04fb, 46 },
    { 0x741d04fc, 100 },
    { 0xb79d04fd, 76 },
    { 0x1bfd04fd, 61 },
    { 0, 0 },
    { 0x60fd0500, 33 },
    { 0x8e8d0500, 90 },
    { 0x938d0502, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd0507, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137d050d, 95 },
    { 0xbaad050d, 67 },
    { 0x67cd050f, 87 },
    { 0, 0 },
    { 0x6e8d0511, 95 },
    { 0x44ed0511, 55 },
    { 0x5b0d0511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed0519, 66 },
    { 0, 0 },
    { 0x749d051b, 33 },
    { 0, 0 },
    { 0xf13d051d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd0521, 33 },
    { 0, 0 },
    { 0x335d0523, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd0528, 98 },
    { 0xcecd0529, 75 },
    { 0x9b8d052a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72d0533, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ad0536, 98 },
    { 0xc02d0537, 95 },
    { 0x36dd0537, 23 },
    { 0xb36d0539, 76 },
    { 0x752d0539, 46 },
    { 0x2c9d053b, 85 },
    { 0x985d053c, 52 },
    { 0x7f0d053c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503d0541, 61 },
    { 0x534d0541, 87 },
    { 0x22dd0543, 33 },
    { 0x5bbd0541, 33 },
    { 0, 0 },
    { 0x986d0546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x872d054c, 78 },
    { 0, 0 },
    { 0x4d2d054e, 23 },
    { 0, 0 },
    { 0x1a9d0550, 33 },
    { 0xfccd0551, 98 },
    { 0x35fd0551, 33 },
    { 0x337d0553, 76 },
    { 0, 0 },
    { 0xccad0555, 84 },
    { 0xee0d0555, 55 },
    { 0, 0 },
    { 0xe55d0558, 46 },
    { 0, 0 },
    { 0x752d055a, 90 },
    { 0xcdd055a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bd0561, 76 },
    { 0xeeed0562, 90 },
    { 0xeb5d0562, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bd0567, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177d056b, 33 },
    { 0xf4dd056c, 95 },
    { 0x835d056d, 91 },
    { 0x361d056e, 66 },
    { 0xcc6d056e, 100 },
    { 0xf56d0570, 90 },
    { 0x5e8d056f, 76 },
    { 0x281d0572, 88 },
    { 0x6cdd0570, 46 },
    { 0, 0 },
    { 0xd5dd0575, 95 },
    { 0xc00d0576, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8d0579, 33 },
    { 0x527d057a, 33 },
    { 0x4b0d0579, 23 },
    { 0, 0 },
    { 0x33cd057d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512d0583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4d0587, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43d058a, 90 },
    { 0, 0 },
    { 0x270d058c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d058f, 64 },
    { 0x34fd058f, 34 },
    { 0, 0 },
    { 0xa61d0592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5d0597, 100 },
    { 0x519d0598, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d059d, 23 },
    { 0, 0 },
    { 0x6bdd059f, 33 },
    { 0x37bd05a0, 23 },
    { 0x82ad05a1, 66 },
    { 0x96d05a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2d05a7, 34 },
    { 0xbc7d05a8, 61 },
    { 0xafd05a9, 76 },
    { 0, 0 },
    { 0x6a0d05ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd05ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cd05b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d05b4, 61 },
    { 0, 0 },
    { 0xdb2d05b6, 90 },
    { 0, 0 },
    { 0x3b1d05b8, 93 },
    { 0, 0 },
    { 0x27ed05ba, 33 },
    { 0, 0 },
    { 0xfd5d05bc, 61 },
    { 0, 0 },
    { 0x62d05be, 99 },
    { 0x3e4d05bf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x461d05c2, 78 },
    { 0x56ad05c3, 76 },
    { 0x9d6d05c4, 23 },
    { 0x69bd05c4, 33 },
    { 0, 0 },
    { 0xc34d05c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1d05ca, 76 },
    { 0x914d05ca, 64 },
    { 0, 0 },
    { 0xc58d05cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfd05d2, 90 },
    { 0, 0 },
    { 0xfbd05d4, 46 },
    { 0, 0 },
    { 0xdfdd05d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502d05da, 77 },
    { 0xa89d05db, 90 },
    { 0xecdd05dc, 98 },
    { 0x5dbd05dd, 87 },
    { 0x59d05dc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d05e3, 56 },
    { 0xd48d05e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303d05eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ed05ef, 61 },
    { 0xdd05f0, 61 },
    { 0xf15d05f0, 23 },
    { 0xf16d05f0, 55 },
    { 0x76cd05f3, 90 },
    { 0, 0 },
    { 0xf27d05f5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad05fa, 90 },
    { 0x3bcd05fa, 55 },
    { 0x546d05fc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738d0602, 23 },
    { 0, 0 },
    { 0xb5fd0604, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2d0607, 90 },
    { 0x45fd0608, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfd060e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7d0614, 46 },
    { 0xae4d0614, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2d061c, 33 },
    { 0xb1ad061d, 76 },
    { 0xa0bd061e, 90 },
    { 0x6fed061d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x773d0622, 61 },
    { 0x659d0623, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfd0626, 95 },
    { 0x4d9d0627, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64d062a, 55 },
    { 0x606d062b, 61 },
    { 0xd34d062c, 95 },
    { 0xb2ad062c, 33 },
    { 0x702d062e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d0632, 33 },
    { 0x932d0633, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcd0637, 95 },
    { 0xff6d0638, 77 },
    { 0x848d0639, 34 },
    { 0xb27d063a, 33 },
    { 0x214d063b, 90 },
    { 0xa70d063b, 95 },
    { 0xcedd063a, 33 },
    { 0xd4dd0639, 90 },
    { 0x26ed063f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d0647, 98 },
    { 0xe16d0648, 85 },
    { 0x2f8d0649, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd064d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ed0650, 99 },
    { 0xab5d0651, 43 },
    { 0xe44d0650, 95 },
    { 0x5ccd0653, 87 },
    { 0xb08d0654, 33 },
    { 0, 0 },
    { 0x967d0656, 90 },
    { 0, 0 },
    { 0x515d0658, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x190d065b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cd065f, 98 },
    { 0, 0 },
    { 0x9f1d0661, 29 },
    { 0x70ed0661, 95 },
    { 0, 0 },
    { 0xc62d0664, 100 },
    { 0x360d0664, 98 },
    { 0xe36d0664, 98 },
    { 0x119d0666, 51 },
    { 0xc47d0668, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d066d, 76 },
    { 0, 0 },
    { 0xf02d066f, 23 },
    { 0, 0 },
    { 0xc2fd0671, 76 },
    { 0x5b7d0671, 61 },
    { 0x608d0671, 100 },
    { 0xc1ad0674, 95 },
    { 0xa6ad0675, 78 },
    { 0, 0 },
    { 0x96cd0677, 93 },
    { 0xe3dd0678, 61 },
    { 0xfadd0677, 98 },
    { 0xffd067a, 95 },
    { 0x939d067a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d067e, 90 },
    { 0x37ad067e, 23 },
    { 0, 0 },
    { 0x4b4d0681, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ed0684, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaad068a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119d068f, 34 },
    { 0x178d0690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fd0693, 90 },
    { 0, 0 },
    { 0x7eed0695, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9d069b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fd069f, 99 },
    { 0, 0 },
    { 0x7f2d06a1, 55 },
    { 0, 0 },
    { 0xb65d06a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d06a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d06b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593d06ba, 61 },
    { 0x7e7d06bb, 93 },
    { 0x6ecd06bc, 90 },
    { 0xcb7d06bd, 98 },
    { 0xd4d06be, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0d06c4, 76 },
    { 0xb7dd06c4, 98 },
    { 0xeb1d06c6, 98 },
    { 0, 0 },
    { 0x9dfd06c8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4d06cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714d06d5, 90 },
    { 0xeaed06d6, 33 },
    { 0x7f6d06d7, 100 },
    { 0x423d06d8, 23 },
    { 0x692d06d7, 85 },
    { 0, 0 },
    { 0xa26d06db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603d06df, 23 },
    { 0, 0 },
    { 0x886d06e1, 76 },
    { 0, 0 },
    { 0x223d06e3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d06e6, 40 },
    { 0, 0 },
    { 0x483d06e8, 33 },
    { 0xc5bd06e9, 33 },
    { 0x526d06ea, 77 },
    { 0xf5ed06eb, 23 },
    { 0x860d06ec, 52 },
    { 0x39fd06ec, 90 },
    { 0, 0 },
    { 0x2f7d06ef, 46 },
    { 0xfeed06ef, 87 },
    { 0, 0 },
    { 0x3edd06f2, 95 },
    { 0xbddd06f3, 61 },
    { 0xec4d06f4, 78 },
    { 0x188d06f3, 55 },
    { 0, 0 },
    { 0xfbdd06f7, 99 },
    { 0, 0 },
    { 0x3b8d06f9, 23 },
    { 0xc80d06fa, 87 },
    { 0x183d06f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abd06ff, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188d070a, 33 },
    { 0x979d070b, 76 },
    { 0, 0 },
    { 0x8f8d070d, 84 },
    { 0x12fd070e, 12 },
    { 0xd75d070f, 98 },
    { 0xdddd0710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743d0714, 90 },
    { 0x483d0715, 33 },
    { 0x503d0716, 87 },
    { 0xf07d0717, 90 },
    { 0xe13d0716, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7d071c, 33 },
    { 0xab4d071d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ed0720, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bd072a, 95 },
    { 0x23cd072a, 66 },
    { 0, 0 },
    { 0xc1d072d, 76 },
    { 0x9bfd072e, 77 },
    { 0x6e5d072f, 76 },
    { 0, 0 },
    { 0x69bd0731, 68 },
    { 0, 0 },
    { 0xb8d0733, 76 },
    { 0x2c5d0734, 33 },
    { 0x352d0734, 93 },
    { 0, 0 },
    { 0x738d0737, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d073f, 76 },
    { 0x407d0740, 43 },
    { 0x248d0741, 33 },
    { 0x54ed0742, 23 },
    { 0, 0 },
    { 0x830d0744, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d0749, 61 },
    { 0xa3cd0749, 33 },
    { 0, 0 },
    { 0x63bd074c, 23 },
    { 0, 0 },
    { 0xaa0d074e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d0752, 23 },
    { 0x4f2d0753, 91 },
    { 0x3a3d0753, 95 },
    { 0xd01d0755, 76 },
    { 0, 0 },
    { 0x4d9d0757, 98 },
    { 0, 0 },
    { 0x805d0759, 90 },
    { 0xe3fd0759, 90 },
    { 0, 0 },
    { 0xcdd075c, 75 },
    { 0xbd2d075d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55d0760, 98 },
    { 0x508d0761, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608d0768, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cd076e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x266d0771, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87d077a, 33 },
    { 0x72d077a, 53 },
    { 0x35fd077c, 66 },
    { 0x710d077c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cd0784, 91 },
    { 0x867d0785, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99d0788, 61 },
    { 0x85bd0789, 98 },
    { 0x932d0789, 55 },
    { 0xde3d078b, 61 },
    { 0x157d078a, 85 },
    { 0x5d2d078d, 33 },
    { 0xbd3d078a, 99 },
    { 0x9e9d078b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8d0792, 34 },
    { 0x53ad0793, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0798, 98 },
    { 0x4c3d0799, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91d079c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3d079f, 84 },
    { 0, 0 },
    { 0xae3d07a1, 100 },
    { 0x10d07a1, 93 },
    { 0x810d07a3, 95 },
    { 0x25d07a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x215d07a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d07ae, 78 },
    { 0x874d07ae, 61 },
    { 0xfa3d07af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d07b5, 98 },
    { 0, 0 },
    { 0x823d07b7, 84 },
    { 0, 0 },
    { 0x1bdd07b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4d07bd, 46 },
    { 0, 0 },
    { 0x487d07bf, 78 },
    { 0x9edd07c0, 33 },
    { 0x592d07c1, 95 },
    { 0x127d07bf, 36 },
    { 0x1b1d07c3, 90 },
    { 0x61bd07c4, 90 },
    { 0xbdcd07c3, 33 },
    { 0xd5cd07c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d07cc, 33 },
    { 0, 0 },
    { 0x86bd07ce, 90 },
    { 0x487d07ce, 33 },
    { 0, 0 },
    { 0x7b6d07d1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd07d9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7d07e1, 43 },
    { 0x1d07e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd07e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66d07ed, 95 },
    { 0x11cd07ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad07fe, 23 },
    { 0xe41d07ff, 84 },
    { 0, 0 },
    { 0x46ad0801, 90 },
    { 0xd0d0802, 85 },
    { 0, 0 },
    { 0x2d2d0804, 33 },
    { 0xf02d0805, 61 },
    { 0xa5dd0806, 33 },
    { 0x4cdd0805, 33 },
    { 0x13fd0808, 98 },
    { 0xf6bd0808, 98 },
    { 0xecfd080a, 95 },
    { 0x3a6d080a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6d0813, 95 },
    { 0xffd0813, 98 },
    { 0x71bd0815, 87 },
    { 0, 0 },
    { 0x635d0817, 77 },
    { 0, 0 },
    { 0x5ded0819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bed081d, 84 },
    { 0x113d081e, 84 },
    { 0xf2ad081f, 33 },
    { 0x9d6d081d, 90 },
    { 0x661d0821, 43 },
    { 0xe82d0822, 55 },
    { 0xc42d0822, 77 },
    { 0, 0 },
    { 0x78fd0825, 43 },
    { 0x7c0d0825, 98 },
    { 0x588d0827, 68 },
    { 0xb34d0827, 95 },
    { 0x248d0827, 61 },
    { 0x897d082a, 77 },
    { 0x75fd082a, 75 },
    { 0xea5d082c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ad0833, 33 },
    { 0x943d0834, 61 },
    { 0x93cd0834, 99 },
    { 0xb2d0836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5d0839, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd083c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359d0844, 23 },
    { 0x6dfd0844, 91 },
    { 0xc60d0844, 33 },
    { 0x9d5d0845, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ed084a, 98 },
    { 0x3bed084a, 90 },
    { 0, 0 },
    { 0xae2d084d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cd0851, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d0855, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8d085a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad086b, 33 },
    { 0xfa3d086b, 65 },
    { 0x132d086c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdd0873, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15d0877, 33 },
    { 0x620d0878, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bd087b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ced087e, 66 },
    { 0, 0 },
    { 0x581d0880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d0884, 95 },
    { 0xc25d0885, 98 },
    { 0xc05d0884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2d088d, 23 },
    { 0x62cd088e, 76 },
    { 0x1fd088d, 90 },
    { 0x91d0890, 100 },
    { 0x47d0891, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x548d0894, 43 },
    { 0, 0 },
    { 0xdeed0896, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689d089a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77d08a5, 49 },
    { 0, 0 },
    { 0x37dd08a7, 33 },
    { 0xafdd08a8, 67 },
    { 0xeb7d08a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaed08ac, 95 },
    { 0xbe7d08ad, 46 },
    { 0x337d08ae, 65 },
    { 0x1c0d08af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acd08b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670d08bf, 90 },
    { 0x99cd08bf, 68 },
    { 0, 0 },
    { 0x805d08c2, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aad08c5, 95 },
    { 0, 0 },
    { 0x8cdd08c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d08cc, 55 },
    { 0xaed08cc, 55 },
    { 0x958d08cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x942d08d1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efd08da, 61 },
    { 0xf9cd08db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ed08e0, 85 },
    { 0xd8cd08e1, 93 },
    { 0x64ad08e1, 98 },
    { 0xe2cd08e2, 33 },
    { 0xb5ad08e4, 98 },
    { 0x31ed08e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x985d08e8, 33 },
    { 0x18fd08e8, 95 },
    { 0x421d08e9, 98 },
    { 0x7a3d08eb, 33 },
    { 0, 0 },
    { 0x8f6d08ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749d08f4, 34 },
    { 0x77fd08f5, 95 },
    { 0x978d08f6, 90 },
    { 0x54dd08f6, 76 },
    { 0x3c9d08f8, 33 },
    { 0xfcad08f6, 90 },
    { 0xebed08fa, 53 },
    { 0x607d08fa, 33 },
    { 0xe8ad08f5, 95 },
    { 0xf1dd08f5, 55 },
    { 0xf32d08fe, 78 },
    { 0xaa9d08fe, 95 },
    { 0x33ed08ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06d0903, 61 },
    { 0x7aed0904, 61 },
    { 0xe36d0904, 91 },
    { 0x362d0904, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacd0913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5d0917, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3d091e, 61 },
    { 0x2aad091e, 38 },
    { 0xb7ed091e, 98 },
    { 0x656d0921, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cd0924, 99 },
    { 0x715d0925, 40 },
    { 0, 0 },
    { 0x33d0927, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d092b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfd092f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130d0933, 84 },
    { 0, 0 },
    { 0x484d0935, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77d0939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933d093e, 78 },
    { 0xef1d093e, 88 },
    { 0x3add0940, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fd0943, 23 },
    { 0x15d0944, 100 },
    { 0x624d0945, 100 },
    { 0x8f8d0946, 80 },
    { 0xea5d0947, 87 },
    { 0, 0 },
    { 0xbd3d0949, 72 },
    { 0x6b8d094a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbed094f, 33 },
    { 0x27dd0950, 33 },
    { 0, 0 },
    { 0x449d0952, 34 },
    { 0, 0 },
    { 0x8abd0954, 33 },
    { 0, 0 },
    { 0xebad0956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cd095c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x421d095f, 33 },
    { 0xf8bd0960, 33 },
    { 0, 0 },
    { 0xf7bd0962, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dd096b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bd096e, 61 },
    { 0x507d096f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ed0972, 98 },
    { 0xaad0973, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5d0977, 23 },
    { 0xb09d0977, 23 },
    { 0, 0 },
    { 0xb68d097a, 100 },
    { 0x7e7d097a, 87 },
    { 0xec8d097b, 88 },
    { 0, 0 },
    { 0x811d097e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x242d0981, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d0988, 61 },
    { 0xfead0988, 61 },
    { 0xd67d0988, 76 },
    { 0x51cd098a, 83 },
    { 0xac4d098c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144d0998, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39d09a3, 90 },
    { 0x2a9d09a4, 93 },
    { 0x373d09a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdd09ac, 77 },
    { 0x6e3d09ad, 90 },
    { 0x310d09ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d09b6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcd09bd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xacd09c0, 34 },
    { 0x9aed09c1, 33 },
    { 0x5b0d09c2, 67 },
    { 0, 0 },
    { 0xa92d09c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad09c7, 98 },
    { 0xa15d09c7, 76 },
    { 0xd40d09c9, 99 },
    { 0xa5ed09ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ad09d1, 90 },
    { 0xf16d09d1, 93 },
    { 0xe0fd09d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fd09d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688d09e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d09e6, 46 },
    { 0x3fed09e6, 61 },
    { 0x39dd09e6, 90 },
    { 0, 0 },
    { 0x482d09ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbd09f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d09f4, 89 },
    { 0xb5dd09f5, 98 },
    { 0x426d09f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d09f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4d09fc, 46 },
    { 0xb92d09fd, 66 },
    { 0x6d3d09fd, 55 },
    { 0x9bbd09ff, 66 },
    { 0, 0 },
    { 0x767d0a01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd0a07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad0a0a, 46 },
    { 0, 0 },
    { 0xf2dd0a0c, 90 },
    { 0x157d0a0c, 10 },
    { 0xcd0d0a0e, 61 },
    { 0x327d0a0e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5d0a16, 23 },
    { 0x820d0a17, 33 },
    { 0xc43d0a16, 52 },
    { 0x50dd0a18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bd0a29, 61 },
    { 0, 0 },
    { 0x3b9d0a2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570d0a32, 61 },
    { 0x1f4d0a32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65d0a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x889d0a3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdd0a40, 90 },
    { 0x466d0a41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8d0a47, 91 },
    { 0x3cd0a48, 23 },
    { 0, 0 },
    { 0x2aad0a4a, 36 },
    { 0, 0 },
    { 0xbbd0a4c, 88 },
    { 0x955d0a4c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35d0a51, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d0a56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabad0a5a, 23 },
    { 0x858d0a5a, 90 },
    { 0x474d0a5c, 61 },
    { 0x53ed0a5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dd0a61, 75 },
    { 0x5dd0a61, 43 },
    { 0xc4cd0a63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5d0a6a, 33 },
    { 0xeacd0a6b, 64 },
    { 0x6eed0a6c, 85 },
    { 0xdf7d0a6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd0a72, 88 },
    { 0, 0 },
    { 0xa6ed0a74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d0a7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20d0a82, 100 },
    { 0x3cd0a83, 61 },
    { 0xb3cd0a84, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885d0a88, 33 },
    { 0xc70d0a89, 36 },
    { 0, 0 },
    { 0xb36d0a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63d0a98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d0a9c, 93 },
    { 0xf58d0a9d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5d0aa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad0aa6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d0aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x846d0ab2, 76 },
    { 0x495d0ab3, 76 },
    { 0x11cd0ab4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d0ac4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0d0ac7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188d0acd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2d0ad7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4d0adc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8d0adf, 33 },
    { 0x749d0ae0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49d0ae4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cd0ae8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d0aed, 85 },
    { 0x6a6d0aee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d0af1, 95 },
    { 0xd5cd0af2, 23 },
    { 0, 0 },
    { 0x346d0af4, 98 },
    { 0x925d0af5, 83 },
    { 0x44bd0af6, 78 },
    { 0, 0 },
    { 0x11fd0af8, 95 },
    { 0x16dd0af9, 98 },
    { 0x454d0afa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5d0aff, 61 },
    { 0xde1d0aff, 43 },
    { 0, 0 },
    { 0xaead0b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766d0b07, 90 },
    { 0, 0 },
    { 0x9ded0b09, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87d0b0d, 90 },
    { 0x63ad0b0d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed0b11, 24 },
    { 0x85bd0b12, 76 },
    { 0x1b1d0b13, 95 },
    { 0, 0 },
    { 0xa15d0b15, 90 },
    { 0, 0 },
    { 0x71ad0b17, 61 },
    { 0xe53d0b18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d0b1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10d0b2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ad0b35, 23 },
    { 0xe18d0b36, 90 },
    { 0x7d9d0b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355d0b3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x687d0b3e, 81 },
    { 0xae0d0b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fd0b43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d0b48, 93 },
    { 0x6a8d0b48, 76 },
    { 0x5c2d0b48, 33 },
    { 0x481d0b48, 23 },
    { 0xff6d0b4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdd0b50, 69 },
    { 0x6d1d0b51, 93 },
    { 0, 0 },
    { 0x70cd0b53, 90 },
    { 0, 0 },
    { 0xf57d0b55, 90 },
    { 0x87ad0b55, 80 },
    { 0x4f9d0b57, 74 },
    { 0x65ad0b56, 34 },
    { 0, 0 },
    { 0x766d0b5a, 33 },
    { 0, 0 },
    { 0xb4cd0b5c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ad0b60, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58d0b65, 23 },
    { 0x8add0b66, 98 },
    { 0x2dfd0b65, 52 },
    { 0xa50d0b66, 61 },
    { 0xa86d0b67, 52 },
    { 0x895d0b6a, 99 },
    { 0x88fd0b6b, 78 },
    { 0xc05d0b6b, 90 },
    { 0x8d4d0b6c, 90 },
    { 0xd30d0b6d, 76 },
    { 0xe6bd0b6f, 93 },
    { 0xb61d0b6f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111d0b74, 90 },
    { 0x5d5d0b75, 98 },
    { 0, 0 },
    { 0xb09d0b77, 78 },
    { 0x704d0b78, 52 },
    { 0x66ad0b77, 34 },
    { 0, 0 },
    { 0x8fbd0b7b, 61 },
    { 0xca4d0b7b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40d0b86, 90 },
    { 0, 0 },
    { 0x2b0d0b88, 90 },
    { 0x808d0b89, 69 },
    { 0x77fd0b8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46d0b91, 90 },
    { 0x972d0b91, 55 },
    { 0x5a8d0b91, 33 },
    { 0xca1d0b93, 33 },
    { 0xd25d0b95, 87 },
    { 0x1c8d0b95, 99 },
    { 0x3e6d0b92, 34 },
    { 0, 0 },
    { 0xfacd0b99, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5d0ba1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545d0ba6, 99 },
    { 0x9d8d0ba7, 66 },
    { 0, 0 },
    { 0xeb9d0ba9, 78 },
    { 0xd96d0baa, 46 },
    { 0x470d0bab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5d0baf, 23 },
    { 0, 0 },
    { 0xe94d0bb1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d0bba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8d0bbd, 33 },
    { 0xa8bd0bbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d0bc3, 90 },
    { 0, 0 },
    { 0x6e8d0bc5, 52 },
    { 0xcb6d0bc5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47d0bcb, 90 },
    { 0x18ed0bcc, 95 },
    { 0, 0 },
    { 0x6ebd0bce, 90 },
    { 0x3ded0bcf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ed0bd5, 40 },
    { 0xc18d0bd6, 23 },
    { 0xc1ad0bd5, 99 },
    { 0x184d0bd5, 46 },
    { 0, 0 },
    { 0x8ddd0bda, 43 },
    { 0xd09d0bdb, 61 },
    { 0xb9dd0bdb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dd0be5, 95 },
    { 0x747d0be6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cd0bea, 61 },
    { 0xe40d0beb, 90 },
    { 0xc92d0beb, 98 },
    { 0, 0 },
    { 0x237d0bee, 90 },
    { 0, 0 },
    { 0xb5ad0bf0, 61 },
    { 0x806d0bf1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fd0bf4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ad0bfb, 22 },
    { 0x727d0bfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47d0bff, 76 },
    { 0xffbd0c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d0c03, 90 },
    { 0x5c5d0c04, 33 },
    { 0xb93d0c05, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15d0c08, 78 },
    { 0xc91d0c09, 33 },
    { 0x15bd0c0a, 55 },
    { 0, 0 },
    { 0x66cd0c0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4d0c10, 46 },
    { 0x107d0c11, 33 },
    { 0, 0 },
    { 0x944d0c13, 77 },
    { 0x6f0d0c14, 38 },
    { 0xb74d0c15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d0c22, 95 },
    { 0x73d0c23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d0c26, 78 },
    { 0x364d0c27, 23 },
    { 0xb58d0c26, 98 },
    { 0, 0 },
    { 0xed6d0c2a, 55 },
    { 0x6e0d0c2b, 77 },
    { 0x5d2d0c2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4d0c2f, 61 },
    { 0x25bd0c2f, 33 },
    { 0xca4d0c31, 55 },
    { 0x3a4d0c32, 23 },
    { 0x5ced0c33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd0c37, 76 },
    { 0x4f7d0c38, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3d0c3b, 33 },
    { 0xe55d0c3c, 61 },
    { 0x543d0c3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d0c42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3d0c49, 84 },
    { 0, 0 },
    { 0x137d0c4b, 90 },
    { 0x6bbd0c4b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cd0c51, 76 },
    { 0x410d0c52, 78 },
    { 0xc7ed0c52, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0d0c56, 61 },
    { 0x12cd0c57, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ded0c5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467d0c5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad0c62, 34 },
    { 0xe56d0c63, 90 },
    { 0x795d0c62, 55 },
    { 0, 0 },
    { 0xcadd0c66, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114d0c6a, 91 },
    { 0x457d0c6b, 98 },
    { 0x484d0c6c, 33 },
    { 0x884d0c6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ed0c77, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d0c7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d0c85, 90 },
    { 0, 0 },
    { 0x15dd0c87, 100 },
    { 0x6c5d0c88, 61 },
    { 0x707d0c88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2d0c92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8d0c96, 55 },
    { 0xd19d0c96, 55 },
    { 0x44cd0c98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d0c9d, 66 },
    { 0xdc6d0c9e, 34 },
    { 0xbb8d0c9f, 61 },
    { 0x1a9d0c9f, 61 },
    { 0x383d0ca0, 93 },
    { 0xebcd0c9e, 99 },
    { 0xea1d0ca3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccd0ca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d0cb4, 46 },
    { 0x143d0cb5, 61 },
    { 0x493d0cb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0d0cba, 52 },
    { 0xd07d0cba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd0cbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3d0cc5, 85 },
    { 0x860d0cc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x236d0cc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcded0ccd, 33 },
    { 0xceed0cce, 61 },
    { 0xb57d0ccf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75d0cd6, 23 },
    { 0xaefd0cd7, 33 },
    { 0xdd5d0cd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320d0cdf, 90 },
    { 0xc7d0ce0, 88 },
    { 0x40d0ce1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9d0ce5, 61 },
    { 0, 0 },
    { 0xc99d0ce7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fd0cea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dd0cef, 46 },
    { 0x68cd0cf0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49d0cf6, 90 },
    { 0, 0 },
    { 0xef6d0cf8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd0cfb, 91 },
    { 0, 0 },
    { 0x7fed0cfd, 61 },
    { 0x68ad0cfe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979d0d05, 46 },
    { 0, 0 },
    { 0x444d0d07, 61 },
    { 0, 0 },
    { 0xfcfd0d09, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad0d11, 33 },
    { 0, 0 },
    { 0x33dd0d13, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efd0d17, 33 },
    { 0x64cd0d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d0d1c, 61 },
    { 0x263d0d1d, 90 },
    { 0x145d0d1e, 76 },
    { 0, 0 },
    { 0x4e4d0d20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x624d0d23, 67 },
    { 0, 0 },
    { 0xb37d0d25, 84 },
    { 0x64d0d26, 33 },
    { 0x3c6d0d25, 95 },
    { 0, 0 },
    { 0xce6d0d29, 34 },
    { 0xd90d0d2a, 61 },
    { 0x5ded0d2b, 23 },
    { 0x891d0d2c, 91 },
    { 0, 0 },
    { 0x928d0d2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d0d38, 90 },
    { 0, 0 },
    { 0xa68d0d3a, 23 },
    { 0xe6fd0d3b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719d0d42, 23 },
    { 0xe4d0d43, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7d0d46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bd0d4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2d0d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d0d53, 34 },
    { 0x1eed0d54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabd0d5e, 33 },
    { 0, 0 },
    { 0x147d0d60, 90 },
    { 0x5c1d0d60, 69 },
    { 0, 0 },
    { 0x239d0d63, 42 },
    { 0x8c2d0d63, 92 },
    { 0x301d0d63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cd0d6d, 90 },
    { 0x7bed0d6e, 92 },
    { 0xbfcd0d6f, 17 },
    { 0x525d0d6f, 69 },
    { 0x997d0d70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39d0d78, 76 },
    { 0x73fd0d79, 89 },
    { 0x7bad0d78, 85 },
    { 0, 0 },
    { 0x7bbd0d7c, 98 },
    { 0xafd0d7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857d0d83, 81 },
    { 0xc19d0d84, 99 },
    { 0xde3d0d84, 33 },
    { 0, 0 },
    { 0x2b6d0d87, 36 },
    { 0, 0 },
    { 0x59d0d89, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190d0d90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cd0d93, 61 },
    { 0, 0 },
    { 0xcb5d0d95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x541d0d98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d0d9b, 90 },
    { 0xc25d0d9c, 91 },
    { 0xa98d0d9d, 23 },
    { 0xac9d0d9e, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ad0da3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cd0da6, 76 },
    { 0, 0 },
    { 0xbfd0da8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52d0dab, 76 },
    { 0xcded0dac, 63 },
    { 0x9c4d0dab, 93 },
    { 0x27ed0dac, 90 },
    { 0, 0 },
    { 0x58bd0db0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd0db3, 90 },
    { 0, 0 },
    { 0x8bfd0db5, 99 },
    { 0, 0 },
    { 0x3abd0db7, 90 },
    { 0, 0 },
    { 0xa05d0db9, 66 },
    { 0xe6dd0dba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746d0dc2, 87 },
    { 0xf82d0dc3, 77 },
    { 0x6d5d0dc4, 64 },
    { 0xb53d0dc3, 98 },
    { 0xa9d0dc6, 65 },
    { 0xae0d0dc7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cd0dcc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ed0dd0, 68 },
    { 0x4bbd0dd1, 23 },
    { 0xe0dd0dd0, 68 },
    { 0x5c5d0dd3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ad0dd6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ad0de0, 43 },
    { 0xd7ed0de0, 17 },
    { 0, 0 },
    { 0x7e0d0de3, 90 },
    { 0x77fd0de3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dd0de9, 23 },
    { 0x775d0dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3d0def, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4d0df2, 67 },
    { 0xd03d0df3, 61 },
    { 0x975d0df4, 90 },
    { 0x639d0df4, 95 },
    { 0xe94d0df2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74d0df9, 61 },
    { 0, 0 },
    { 0x83cd0dfb, 90 },
    { 0xc5ed0dfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x397d0dff, 95 },
    { 0, 0 },
    { 0xea8d0e01, 83 },
    { 0x9d6d0e01, 95 },
    { 0, 0 },
    { 0xf34d0e04, 76 },
    { 0xb6bd0e05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9d0e09, 23 },
    { 0x8ad0e0a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d0e0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17d0e10, 90 },
    { 0, 0 },
    { 0xca0d0e12, 97 },
    { 0, 0 },
    { 0x68ad0e14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bd0e18, 33 },
    { 0xcad0e18, 23 },
    { 0xb48d0e1a, 75 },
    { 0x240d0e1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bd0e1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d0e27, 68 },
    { 0, 0 },
    { 0xcf7d0e29, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d0e2e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ad0e32, 64 },
    { 0, 0 },
    { 0x665d0e34, 90 },
    { 0x71cd0e35, 90 },
    { 0xb5d0e36, 93 },
    { 0xa1bd0e35, 55 },
    { 0xc35d0e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38d0e3e, 87 },
    { 0x60fd0e3f, 23 },
    { 0xbeed0e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846d0e47, 100 },
    { 0x512d0e47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d0e54, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d0e58, 23 },
    { 0x7b4d0e58, 98 },
    { 0x5d3d0e59, 95 },
    { 0xe46d0e5b, 90 },
    { 0x506d0e5c, 61 },
    { 0xcc4d0e5a, 61 },
    { 0x948d0e5e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3d0e61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x248d0e64, 33 },
    { 0, 0 },
    { 0x256d0e66, 99 },
    { 0, 0 },
    { 0xc2ad0e68, 61 },
    { 0x55bd0e69, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37d0e77, 33 },
    { 0x2ebd0e78, 61 },
    { 0x5e4d0e79, 61 },
    { 0, 0 },
    { 0x7afd0e7b, 61 },
    { 0xbded0e7b, 74 },
    { 0x265d0e7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243d0e83, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bd0e89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd0e8d, 84 },
    { 0xb00d0e8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ad0e94, 52 },
    { 0, 0 },
    { 0xe4fd0e96, 90 },
    { 0xf15d0e97, 90 },
    { 0xf52d0e98, 61 },
    { 0, 0 },
    { 0x7fbd0e9a, 95 },
    { 0xa8dd0e9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cd0e9e, 87 },
    { 0, 0 },
    { 0xf43d0ea0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d0ea5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34d0ea9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d0eac, 90 },
    { 0x488d0eac, 23 },
    { 0xa9ad0eac, 61 },
    { 0, 0 },
    { 0xb79d0eb0, 34 },
    { 0x104d0eb0, 61 },
    { 0x48ad0eb2, 95 },
    { 0, 0 },
    { 0xb47d0eb4, 23 },
    { 0x8a8d0eb5, 61 },
    { 0x632d0eb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x793d0eb9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89d0ebd, 33 },
    { 0x14d0ebe, 90 },
    { 0xb43d0ebe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4d0ec3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccd0ec9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dd0ecc, 76 },
    { 0xb15d0ecd, 90 },
    { 0x38cd0ecd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d0ed3, 87 },
    { 0, 0 },
    { 0xc81d0ed5, 99 },
    { 0xe69d0ed5, 65 },
    { 0xe0d0ed7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ed0edd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabd0ee1, 85 },
    { 0, 0 },
    { 0x72cd0ee3, 78 },
    { 0xe5cd0ee4, 87 },
    { 0x9b9d0ee3, 52 },
    { 0x4b9d0ee4, 78 },
    { 0xce4d0ee6, 33 },
    { 0, 0 },
    { 0x4b3d0ee9, 95 },
    { 0, 0 },
    { 0x3d0eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcd0ef3, 93 },
    { 0x87dd0ef4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cad0ef8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbd0efb, 61 },
    { 0x509d0efc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd0f00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0d0f04, 90 },
    { 0xdaed0f05, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd0f0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dd0f10, 66 },
    { 0xdb9d0f10, 95 },
    { 0x91d0f12, 90 },
    { 0xac4d0f13, 90 },
    { 0xdedd0f10, 33 },
    { 0xe75d0f15, 33 },
    { 0xecbd0f16, 95 },
    { 0, 0 },
    { 0xf82d0f18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1d0f1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x247d0f20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802d0f25, 76 },
    { 0x238d0f25, 95 },
    { 0x7dd0f25, 90 },
    { 0xda9d0f28, 61 },
    { 0xf04d0f29, 95 },
    { 0xc0bd0f28, 52 },
    { 0x1fbd0f27, 95 },
    { 0xa83d0f29, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0d0f30, 64 },
    { 0x873d0f31, 61 },
    { 0, 0 },
    { 0x451d0f33, 90 },
    { 0xe88d0f33, 23 },
    { 0x42fd0f34, 90 },
    { 0x795d0f36, 98 },
    { 0, 0 },
    { 0x8e5d0f38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd0f3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dd0f3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fd0f41, 67 },
    { 0xf6dd0f42, 61 },
    { 0x342d0f43, 33 },
    { 0, 0 },
    { 0x98fd0f45, 90 },
    { 0x591d0f46, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3d0f49, 90 },
    { 0, 0 },
    { 0x446d0f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6d0f4f, 90 },
    { 0xf93d0f50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404d0f54, 61 },
    { 0x7cd0f55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd0f5d, 88 },
    { 0, 0 },
    { 0xa79d0f5f, 34 },
    { 0xb55d0f60, 52 },
    { 0, 0 },
    { 0xf1dd0f62, 23 },
    { 0x868d0f62, 61 },
    { 0xcffd0f64, 46 },
    { 0x2ead0f62, 67 },
    { 0, 0 },
    { 0xabbd0f67, 85 },
    { 0x198d0f68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1d0f6b, 61 },
    { 0, 0 },
    { 0x826d0f6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2d0f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2d0f73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99d0f77, 33 },
    { 0x3ead0f78, 90 },
    { 0x3ced0f79, 33 },
    { 0xa25d0f7a, 91 },
    { 0xaccd0f78, 90 },
    { 0x9aed0f7a, 91 },
    { 0x296d0f7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fd0f82, 98 },
    { 0xeccd0f82, 90 },
    { 0x907d0f84, 96 },
    { 0xe6ed0f85, 76 },
    { 0x73fd0f85, 55 },
    { 0xe9bd0f83, 90 },
    { 0xcc7d0f86, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ed0f8b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ad0f8e, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cd0f91, 67 },
    { 0x778d0f92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ad0f99, 90 },
    { 0x417d0f99, 99 },
    { 0xe05d0f9a, 93 },
    { 0x276d0f9c, 98 },
    { 0x98ed0f9c, 78 },
    { 0x2cfd0f9e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x240d0fa1, 100 },
    { 0x212d0fa1, 98 },
    { 0xddad0fa3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d0fa6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dd0fae, 99 },
    { 0x1d9d0fae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194d0fb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3d0fb7, 76 },
    { 0, 0 },
    { 0x325d0fb9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65d0fbc, 61 },
    { 0xc45d0fbd, 90 },
    { 0x397d0fbe, 96 },
    { 0, 0 },
    { 0xb2ad0fc0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d0fc8, 93 },
    { 0xeced0fc9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9d0fd0, 98 },
    { 0xf3bd0fd0, 69 },
    { 0x1fcd0fd2, 39 },
    { 0x3ced0fd3, 34 },
    { 0x6e2d0fd4, 76 },
    { 0x36dd0fd5, 90 },
    { 0xa33d0fd3, 90 },
    { 0, 0 },
    { 0xe2ad0fd8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850d0fdc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2d0fe1, 83 },
    { 0x2cad0fe2, 90 },
    { 0x40fd0fe2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd0fe6, 23 },
    { 0xfc5d0fe7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x520d0fea, 93 },
    { 0, 0 },
    { 0x8b6d0fec, 90 },
    { 0x42bd0fed, 34 },
    { 0, 0 },
    { 0xf7ad0fef, 90 },
    { 0, 0 },
    { 0x19fd0ff1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ed0ff4, 52 },
    { 0x8a4d0ff4, 61 },
    { 0, 0 },
    { 0xefdd0ff7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x798d0ffa, 90 },
    { 0xdfbd0ffb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683d0fff, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd1002, 81 },
    { 0xdded1003, 46 },
    { 0x41cd1004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91d1007, 76 },
    { 0x2ced1008, 52 },
    { 0x564d1007, 61 },
    { 0x51bd1009, 40 },
    { 0x1d2d1007, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bd1010, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1d1016, 61 },
    { 0x68fd1017, 88 },
    { 0xfe3d1017, 67 },
    { 0, 0 },
    { 0x2bdd101a, 90 },
    { 0xa70d101a, 61 },
    { 0, 0 },
    { 0x817d101d, 93 },
    { 0x170d101e, 75 },
    { 0xce9d101e, 65 },
    { 0xa0bd101f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953d1029, 76 },
    { 0x826d102a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554d1031, 23 },
    { 0, 0 },
    { 0xcccd1033, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d1036, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x909d1039, 85 },
    { 0xf0dd103a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd103e, 34 },
    { 0xf1bd103f, 98 },
    { 0, 0 },
    { 0xe46d1041, 98 },
    { 0xf86d1042, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d1045, 93 },
    { 0x8edd1046, 98 },
    { 0xb8ad1047, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ed104b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bd1050, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed1055, 40 },
    { 0x742d1055, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d1063, 76 },
    { 0, 0 },
    { 0x909d1065, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2d1068, 67 },
    { 0, 0 },
    { 0x4fed106a, 61 },
    { 0, 0 },
    { 0x9d7d106c, 61 },
    { 0xd4fd106d, 67 },
    { 0, 0 },
    { 0x3f6d106f, 46 },
    { 0xcb8d1070, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d1075, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ded1079, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd107c, 43 },
    { 0, 0 },
    { 0xb6cd107e, 84 },
    { 0x28bd107e, 90 },
    { 0xbedd1080, 95 },
    { 0x43cd1080, 90 },
    { 0xa36d1082, 33 },
    { 0x3acd1083, 90 },
    { 0xed5d1084, 40 },
    { 0xf85d1084, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51d108c, 87 },
    { 0xa92d108d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67d1090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dd1093, 61 },
    { 0xec8d1094, 95 },
    { 0, 0 },
    { 0x9abd1096, 34 },
    { 0, 0 },
    { 0xc58d1098, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d109e, 91 },
    { 0x87ed109e, 69 },
    { 0, 0 },
    { 0xe4dd10a1, 90 },
    { 0, 0 },
    { 0xfa2d10a3, 95 },
    { 0xc74d10a4, 61 },
    { 0xfb4d10a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cd10af, 90 },
    { 0x552d10b0, 69 },
    { 0xa34d10af, 85 },
    { 0x9acd10b2, 77 },
    { 0xc13d10b3, 76 },
    { 0x3c2d10b1, 34 },
    { 0, 0 },
    { 0x342d10b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338d10be, 90 },
    { 0xa2fd10be, 90 },
    { 0x510d10be, 61 },
    { 0x14ad10c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x39d10c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x972d10c7, 23 },
    { 0, 0 },
    { 0x489d10c9, 55 },
    { 0xf50d10c9, 81 },
    { 0x708d10cb, 55 },
    { 0x3b4d10c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x797d10cf, 81 },
    { 0x5ced10d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ed10d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x361d10dc, 93 },
    { 0x688d10dd, 76 },
    { 0, 0 },
    { 0x18fd10df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8d10e4, 61 },
    { 0, 0 },
    { 0xb7fd10e6, 96 },
    { 0x466d10e7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3d10ee, 55 },
    { 0x7a8d10ef, 90 },
    { 0x874d10f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ad10f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cd1100, 61 },
    { 0, 0 },
    { 0xdcbd1102, 46 },
    { 0x2f5d1103, 78 },
    { 0, 0 },
    { 0x67bd1105, 61 },
    { 0xc0dd1106, 77 },
    { 0xba2d1107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d1110, 95 },
    { 0xc52d1111, 61 },
    { 0x81dd1110, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d1116, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cd1119, 100 },
    { 0x2d7d1119, 95 },
    { 0x7f6d111b, 66 },
    { 0xc7ad111c, 43 },
    { 0, 0 },
    { 0x6aad111e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d1122, 23 },
    { 0xd93d1123, 43 },
    { 0x970d1124, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1127, 34 },
    { 0, 0 },
    { 0xd9dd1129, 90 },
    { 0x2bed112a, 33 },
    { 0xd44d112b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add112e, 46 },
    { 0x583d112f, 52 },
    { 0, 0 },
    { 0xca5d1131, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x673d1134, 68 },
    { 0xb37d1135, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aed113b, 38 },
    { 0xac8d113b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9d1146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d114a, 76 },
    { 0x98cd114b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd1152, 90 },
    { 0xa35d1152, 61 },
    { 0, 0 },
    { 0xf46d1155, 98 },
    { 0x381d1156, 84 },
    { 0x7f2d1157, 90 },
    { 0x6fad1157, 76 },
    { 0x438d1157, 95 },
    { 0, 0 },
    { 0x1a5d115b, 91 },
    { 0, 0 },
    { 0xd0bd115d, 77 },
    { 0, 0 },
    { 0xdb4d115f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4d1165, 76 },
    { 0x335d1166, 98 },
    { 0xb1d1167, 61 },
    { 0x43cd1168, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad116d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdd1170, 90 },
    { 0x505d1171, 85 },
    { 0x323d1171, 90 },
    { 0x7e8d1171, 46 },
    { 0xd8ad1173, 90 },
    { 0xc80d1175, 43 },
    { 0, 0 },
    { 0x1f8d1177, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d1183, 23 },
    { 0, 0 },
    { 0xcccd1185, 34 },
    { 0x64ad1186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffd118d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d1191, 77 },
    { 0x5bbd1192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568d1197, 43 },
    { 0x4b0d1198, 66 },
    { 0, 0 },
    { 0x8bcd119a, 98 },
    { 0, 0 },
    { 0xd17d119c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d11a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9d11a6, 76 },
    { 0x6dd11a7, 61 },
    { 0x7d7d11a7, 90 },
    { 0x63ad11a9, 34 },
    { 0x7ed11a8, 23 },
    { 0, 0 },
    { 0xa55d11ac, 33 },
    { 0x33bd11ad, 43 },
    { 0x539d11ad, 61 },
    { 0, 0 },
    { 0xfb4d11b0, 95 },
    { 0xc3cd11b1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51d11bc, 76 },
    { 0, 0 },
    { 0x9d0d11be, 34 },
    { 0x406d11bf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d11c2, 46 },
    { 0xdafd11c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4d11ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad11cf, 93 },
    { 0x977d11cf, 95 },
    { 0, 0 },
    { 0x9b5d11d2, 95 },
    { 0x99ad11d3, 65 },
    { 0, 0 },
    { 0xed8d11d5, 66 },
    { 0x30cd11d5, 23 },
    { 0x6a7d11d7, 77 },
    { 0x6d3d11d7, 98 },
    { 0x5aed11d5, 23 },
    { 0x6c1d11d8, 46 },
    { 0xc28d11d5, 98 },
    { 0, 0 },
    { 0x6a7d11dd, 33 },
    { 0xd42d11de, 93 },
    { 0x91bd11df, 90 },
    { 0x3d11df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cd11e3, 78 },
    { 0x988d11e4, 17 },
    { 0xae5d11e5, 76 },
    { 0xbdfd11e6, 93 },
    { 0, 0 },
    { 0x6bdd11e8, 61 },
    { 0x5bfd11e8, 76 },
    { 0x62fd11e8, 61 },
    { 0, 0 },
    { 0xfa5d11ec, 98 },
    { 0, 0 },
    { 0xb81d11ee, 98 },
    { 0x51ed11ef, 33 },
    { 0xc1bd11ef, 33 },
    { 0x366d11ee, 76 },
    { 0x54d11f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd11f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35d11f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x797d11fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54d11fe, 91 },
    { 0xf16d11fe, 46 },
    { 0x1f4d1200, 98 },
    { 0xf2fd1201, 90 },
    { 0x9ced11fe, 65 },
    { 0xbf0d1203, 76 },
    { 0x801d1203, 95 },
    { 0xed6d1205, 55 },
    { 0x239d1206, 93 },
    { 0xe4ad1206, 95 },
    { 0xbc8d1208, 98 },
    { 0xbc1d1201, 33 },
    { 0x5c6d120a, 80 },
    { 0xc30d1205, 40 },
    { 0xec7d1201, 76 },
    { 0x35bd120d, 34 },
    { 0x4b0d120c, 95 },
    { 0x3c0d1205, 61 },
    { 0x43d1210, 87 },
    { 0xeeed120c, 90 },
    { 0x490d1205, 66 },
    { 0x38d1207, 90 },
    { 0x220d1214, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ed1219, 46 },
    { 0, 0 },
    { 0x4d2d121b, 61 },
    { 0, 0 },
    { 0x432d121d, 90 },
    { 0x382d121d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32d1221, 61 },
    { 0x2c0d1222, 93 },
    { 0x82dd1222, 55 },
    { 0x45d1224, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1d122a, 46 },
    { 0xad8d122b, 86 },
    { 0x763d122c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70d1231, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2d1239, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d123d, 23 },
    { 0x328d123e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792d124a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x409d124d, 34 },
    { 0x21dd124e, 98 },
    { 0x9cd124f, 55 },
    { 0xde5d1250, 61 },
    { 0x313d124f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cd1256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fd1259, 95 },
    { 0, 0 },
    { 0x16dd125b, 93 },
    { 0xbafd125c, 33 },
    { 0x2c1d125d, 61 },
    { 0xc9d125d, 96 },
    { 0xab3d125d, 90 },
    { 0x5e0d125f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0d1266, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d126c, 98 },
    { 0xed8d126c, 23 },
    { 0xdfed126e, 99 },
    { 0, 0 },
    { 0x511d1270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d1274, 81 },
    { 0, 0 },
    { 0xa78d1276, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0d1279, 95 },
    { 0xeb5d127a, 61 },
    { 0xdaad127a, 33 },
    { 0, 0 },
    { 0xc4cd127d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2d1284, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd1288, 87 },
    { 0, 0 },
    { 0x263d128a, 23 },
    { 0, 0 },
    { 0x890d128c, 68 },
    { 0x23ed128d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ed1295, 99 },
    { 0xe71d1296, 46 },
    { 0, 0 },
    { 0xc87d1298, 23 },
    { 0, 0 },
    { 0x5d0d129a, 95 },
    { 0xa42d129b, 77 },
    { 0, 0 },
    { 0x879d129d, 90 },
    { 0x567d129e, 92 },
    { 0, 0 },
    { 0x5f8d12a0, 95 },
    { 0x33ed12a0, 90 },
    { 0xa15d12a0, 76 },
    { 0x28cd12a0, 52 },
    { 0, 0 },
    { 0x4d0d12a5, 99 },
    { 0x3d4d12a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336d12aa, 61 },
    { 0, 0 },
    { 0x48ad12ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12d12b5, 85 },
    { 0x3c2d12b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d12b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16d12bd, 33 },
    { 0, 0 },
    { 0xa3ed12bf, 100 },
    { 0, 0 },
    { 0x96dd12c1, 98 },
    { 0xb41d12c1, 87 },
    { 0x7d4d12c3, 100 },
    { 0, 0 },
    { 0x2b5d12c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ed12c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd12cb, 46 },
    { 0xceed12cc, 98 },
    { 0xbecd12cc, 33 },
    { 0, 0 },
    { 0x29d12cf, 61 },
    { 0, 0 },
    { 0xf59d12d1, 61 },
    { 0x38fd12d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dd12d5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514d12da, 65 },
    { 0x89fd12db, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdd12df, 76 },
    { 0xa6ed12e0, 49 },
    { 0x57bd12e0, 66 },
    { 0xd9bd12df, 61 },
    { 0xca3d12e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fd12e7, 61 },
    { 0xc3cd12e8, 98 },
    { 0, 0 },
    { 0xf0d12ea, 93 },
    { 0xefdd12eb, 52 },
    { 0x192d12ec, 95 },
    { 0x6d5d12ed, 61 },
    { 0xed3d12eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3d12f4, 52 },
    { 0x729d12f5, 95 },
    { 0xcf9d12f6, 95 },
    { 0x307d12f7, 33 },
    { 0xd8fd12f8, 76 },
    { 0x794d12f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfd12fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cd12ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d1308, 23 },
    { 0x7ddd1308, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fd130d, 33 },
    { 0x4fcd130e, 87 },
    { 0x8b5d130f, 55 },
    { 0x28d130d, 61 },
    { 0x42d1311, 76 },
    { 0, 0 },
    { 0xfa2d1313, 90 },
    { 0xcfed1313, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75d131d, 83 },
    { 0, 0 },
    { 0xd25d131f, 33 },
    { 0x81cd1320, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d1323, 23 },
    { 0x7cfd1324, 98 },
    { 0x3bad1323, 75 },
    { 0x199d1326, 43 },
    { 0x2a3d1327, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebd132c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911d1330, 61 },
    { 0x30fd1330, 33 },
    { 0, 0 },
    { 0x16fd1333, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8d1339, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18d1343, 61 },
    { 0, 0 },
    { 0x952d1345, 61 },
    { 0xab4d1346, 101 },
    { 0x523d1347, 55 },
    { 0xc60d1348, 23 },
    { 0, 0 },
    { 0xb50d134a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d1353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d1357, 90 },
    { 0xfe7d1358, 66 },
    { 0x3ffd1359, 98 },
    { 0x428d1358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57d135d, 43 },
    { 0, 0 },
    { 0x21fd135f, 67 },
    { 0, 0 },
    { 0x44ed1361, 78 },
    { 0xc39d1362, 61 },
    { 0x25dd1362, 98 },
    { 0xbfd1362, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afd1368, 90 },
    { 0x6f1d1369, 99 },
    { 0x12ad136a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1371, 61 },
    { 0xd8ad1371, 43 },
    { 0x8dbd1371, 33 },
    { 0, 0 },
    { 0xb89d1375, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd1379, 23 },
    { 0x55ed1379, 43 },
    { 0xee3d1379, 95 },
    { 0, 0 },
    { 0xb66d137d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47d1382, 85 },
    { 0xaffd1383, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68d1387, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4d138d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fd1392, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fd1395, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d139d, 95 },
    { 0x837d139e, 98 },
    { 0, 0 },
    { 0x7ed13a0, 87 },
    { 0xcc2d13a1, 76 },
    { 0, 0 },
    { 0x725d13a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ead13a6, 23 },
    { 0, 0 },
    { 0x8afd13a8, 61 },
    { 0x48d13a9, 90 },
    { 0x1ded13aa, 23 },
    { 0, 0 },
    { 0x580d13ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2d13b1, 33 },
    { 0x3add13b1, 90 },
    { 0x823d13b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ead13b7, 90 },
    { 0, 0 },
    { 0x1e9d13b9, 87 },
    { 0x3add13b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7d13bd, 34 },
    { 0xe7bd13be, 66 },
    { 0x9a4d13bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ed13c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645d13c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ad13cc, 52 },
    { 0, 0 },
    { 0x20fd13ce, 33 },
    { 0x10dd13cf, 91 },
    { 0, 0 },
    { 0x1d13d1, 33 },
    { 0, 0 },
    { 0x167d13d3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x150d13d6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ed13db, 90 },
    { 0, 0 },
    { 0x1a8d13dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x231d13e0, 55 },
    { 0x7fd13e0, 33 },
    { 0x42fd13e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800d13e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92d13f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681d13fe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ead1401, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8d1405, 61 },
    { 0x23bd1406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d140b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bd140e, 95 },
    { 0xe6cd140e, 43 },
    { 0x530d1410, 90 },
    { 0, 0 },
    { 0x722d1412, 90 },
    { 0, 0 },
    { 0xae3d1414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7d1417, 93 },
    { 0xea4d1418, 90 },
    { 0, 0 },
    { 0x7eed141a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198d1421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15d1424, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcd1432, 91 },
    { 0, 0 },
    { 0xa54d1434, 63 },
    { 0xdf1d1435, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4d143c, 90 },
    { 0x2acd143c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x163d1440, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd1447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cd144e, 100 },
    { 0, 0 },
    { 0xf87d1450, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdd1454, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fd1457, 61 },
    { 0, 0 },
    { 0x37ad1459, 95 },
    { 0xa0d145a, 84 },
    { 0, 0 },
    { 0x89bd145c, 95 },
    { 0xf15d145d, 93 },
    { 0xb38d145e, 91 },
    { 0x299d145f, 33 },
    { 0xdb2d145c, 34 },
    { 0xc73d1461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd1467, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6d146a, 61 },
    { 0, 0 },
    { 0x891d146c, 98 },
    { 0, 0 },
    { 0xad4d146e, 23 },
    { 0, 0 },
    { 0x3fcd1470, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ad1474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ced147a, 39 },
    { 0x13cd147a, 90 },
    { 0, 0 },
    { 0xc73d147d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8d1480, 33 },
    { 0xa12d1481, 43 },
    { 0, 0 },
    { 0x942d1483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d1486, 33 },
    { 0xf43d1486, 68 },
    { 0x9a4d1487, 49 },
    { 0x647d1489, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60d1490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151d1496, 23 },
    { 0, 0 },
    { 0x3e9d1498, 94 },
    { 0, 0 },
    { 0xd55d149a, 95 },
    { 0xd1d149b, 17 },
    { 0x231d149b, 95 },
    { 0xdcd149d, 90 },
    { 0x738d149e, 23 },
    { 0xf23d149e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7d14a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbad14ac, 93 },
    { 0x888d14ac, 90 },
    { 0x92cd14ae, 23 },
    { 0x825d14af, 90 },
    { 0x90dd14b0, 66 },
    { 0x458d14b1, 46 },
    { 0xcd2d14b0, 95 },
    { 0, 0 },
    { 0xf75d14b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ead14ba, 98 },
    { 0, 0 },
    { 0x463d14bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759d14c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d14cb, 95 },
    { 0xaed14cc, 61 },
    { 0xd27d14cc, 99 },
    { 0, 0 },
    { 0x759d14cf, 76 },
    { 0x5d8d14d0, 55 },
    { 0x88d14d1, 61 },
    { 0x185d14d2, 88 },
    { 0x2f6d14d2, 23 },
    { 0x667d14d4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79d14d7, 75 },
    { 0xf98d14d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cd14dc, 88 },
    { 0x608d14dc, 87 },
    { 0xf3ad14de, 55 },
    { 0x904d14dc, 98 },
    { 0x596d14e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530d14e9, 33 },
    { 0xe9cd14e9, 81 },
    { 0xcf4d14eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89d14ef, 33 },
    { 0x91ad14f0, 61 },
    { 0xcd7d14f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x323d14f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d14f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962d14fb, 43 },
    { 0x3e9d14fc, 95 },
    { 0x5cfd14fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bd1505, 98 },
    { 0xea7d1505, 90 },
    { 0x6d6d1507, 77 },
    { 0x66bd1508, 61 },
    { 0x5ecd1505, 90 },
    { 0xb3d150a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bd150f, 99 },
    { 0x784d1510, 46 },
    { 0x5c5d1511, 90 },
    { 0xdadd150f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507d1516, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdd1519, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dd151d, 90 },
    { 0x894d151d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d1522, 33 },
    { 0xed5d1522, 90 },
    { 0x3e4d1524, 87 },
    { 0x930d1525, 23 },
    { 0xfbfd1524, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3d1529, 33 },
    { 0xb5bd152a, 77 },
    { 0x1e3d152b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cd1532, 98 },
    { 0xcecd1532, 61 },
    { 0x64cd1533, 95 },
    { 0, 0 },
    { 0xce8d1536, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898d153b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd1544, 33 },
    { 0x4b0d1544, 82 },
    { 0xa58d1546, 23 },
    { 0xafbd1547, 42 },
    { 0xeaad1544, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3d154e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5d1552, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cd1557, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8d155b, 98 },
    { 0x363d155b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74d155f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd1563, 61 },
    { 0, 0 },
    { 0xaf0d1565, 33 },
    { 0xad3d1565, 69 },
    { 0x5c8d1567, 93 },
    { 0x2e5d1568, 77 },
    { 0x6f6d1565, 98 },
    { 0xd61d156a, 67 },
    { 0xd4ed1567, 33 },
    { 0, 0 },
    { 0x9c9d156d, 61 },
    { 0, 0 },
    { 0x37ad156f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14d1572, 23 },
    { 0x586d1573, 33 },
    { 0x20ad1572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68d1578, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2d157e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775d1588, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd1592, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d1598, 55 },
    { 0x878d1599, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdead159f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eed15a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65d15a8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fd15ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d15af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240d15c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dd15c9, 23 },
    { 0x278d15ca, 61 },
    { 0xe01d15c9, 93 },
    { 0x63ed15cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cd15d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d15d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54d15e1, 34 },
    { 0, 0 },
    { 0x725d15e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38d15e7, 38 },
    { 0x8ecd15e8, 17 },
    { 0xd65d15e9, 90 },
    { 0xefdd15ea, 61 },
    { 0xee2d15e8, 87 },
    { 0xf9cd15ec, 78 },
    { 0, 0 },
    { 0x6ebd15ee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149d15f2, 33 },
    { 0x74d15f2, 95 },
    { 0x8b0d15f4, 98 },
    { 0, 0 },
    { 0xf50d15f6, 46 },
    { 0x2a9d15f7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67d15fb, 43 },
    { 0, 0 },
    { 0x6d8d15fd, 46 },
    { 0, 0 },
    { 0x1b9d15ff, 90 },
    { 0xbdbd1600, 100 },
    { 0x331d15ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dd1605, 34 },
    { 0xc42d1606, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8d160b, 76 },
    { 0, 0 },
    { 0x72ad160d, 90 },
    { 0x77d160e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x159d1611, 98 },
    { 0x147d1612, 91 },
    { 0xb72d1612, 43 },
    { 0x9b5d1614, 34 },
    { 0x889d1615, 98 },
    { 0xe69d1614, 99 },
    { 0x84ed1617, 90 },
    { 0x553d1618, 23 },
    { 0xcc0d1618, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712d1622, 98 },
    { 0xbf1d1622, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9d1628, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4d162c, 23 },
    { 0x40ed162c, 84 },
    { 0x5a5d162e, 61 },
    { 0x640d162c, 33 },
    { 0, 0 },
    { 0x740d1631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760d163b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d1642, 78 },
    { 0xa36d1642, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223d1648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2d164c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3cd164f, 55 },
    { 0xf34d1650, 90 },
    { 0xe27d1651, 23 },
    { 0, 0 },
    { 0x82ed1653, 61 },
    { 0xab0d1653, 76 },
    { 0xf64d1655, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345d1659, 94 },
    { 0x7f1d165a, 90 },
    { 0xdced165b, 84 },
    { 0, 0 },
    { 0x8fbd165d, 23 },
    { 0, 0 },
    { 0x1e1d165f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed1668, 95 },
    { 0, 0 },
    { 0x3c1d166a, 36 },
    { 0x9f0d166b, 40 },
    { 0x927d166b, 80 },
    { 0xca0d166a, 76 },
    { 0x550d166b, 98 },
    { 0x11dd166d, 55 },
    { 0, 0 },
    { 0x298d1671, 91 },
    { 0x3e6d1671, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27d1675, 44 },
    { 0xa98d1676, 55 },
    { 0, 0 },
    { 0xc3ad1678, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1d167b, 90 },
    { 0x5f2d167c, 33 },
    { 0xb0bd167c, 55 },
    { 0x75dd167e, 34 },
    { 0xd40d167f, 98 },
    { 0, 0 },
    { 0x535d1681, 95 },
    { 0x7d6d1682, 33 },
    { 0xbf0d1682, 98 },
    { 0xe24d1684, 55 },
    { 0xc11d1684, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcd168a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8dd168e, 61 },
    { 0, 0 },
    { 0x1bed1690, 36 },
    { 0xabad1691, 95 },
    { 0xb96d1692, 61 },
    { 0x326d1693, 90 },
    { 0, 0 },
    { 0xd8ed1695, 33 },
    { 0x2ded1696, 90 },
    { 0x476d1695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482d16a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77d16a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d16ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bd16b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d16b5, 33 },
    { 0, 0 },
    { 0xb97d16b7, 23 },
    { 0x558d16b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d16bc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4d16c2, 23 },
    { 0x7fdd16c2, 88 },
    { 0, 0 },
    { 0x485d16c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cd16d0, 99 },
    { 0, 0 },
    { 0x363d16d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ed16d5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d16d9, 33 },
    { 0xc27d16da, 93 },
    { 0x3cad16d9, 53 },
    { 0xdb9d16dc, 90 },
    { 0, 0 },
    { 0x9c6d16de, 61 },
    { 0x339d16de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead16e7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9d16eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71d16ee, 61 },
    { 0, 0 },
    { 0x21cd16f0, 90 },
    { 0, 0 },
    { 0xf23d16f2, 55 },
    { 0x9e8d16f2, 81 },
    { 0xc32d16f3, 61 },
    { 0, 0 },
    { 0xbb1d16f6, 34 },
    { 0x5c2d16f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239d16fb, 87 },
    { 0xeb1d16fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9d1702, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cd1705, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ad170d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd1711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d1714, 46 },
    { 0xd23d1715, 78 },
    { 0, 0 },
    { 0xeb1d1717, 98 },
    { 0x964d1717, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8d171e, 90 },
    { 0, 0 },
    { 0xaf5d1720, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd1727, 98 },
    { 0, 0 },
    { 0x5c0d1729, 96 },
    { 0x74bd172a, 90 },
    { 0, 0 },
    { 0xad5d172c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x784d172f, 90 },
    { 0, 0 },
    { 0x6ad1731, 97 },
    { 0x2bdd1731, 75 },
    { 0, 0 },
    { 0xf43d1734, 93 },
    { 0x3bcd1734, 33 },
    { 0x147d1735, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffd1739, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9dd173f, 98 },
    { 0xefcd1740, 34 },
    { 0xd45d173f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68d1744, 43 },
    { 0, 0 },
    { 0xe9d1746, 52 },
    { 0x271d1747, 34 },
    { 0x68ed1748, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aed174e, 61 },
    { 0xf71d174f, 52 },
    { 0xa52d1750, 90 },
    { 0x942d174f, 66 },
    { 0x7b0d1752, 33 },
    { 0xd29d1753, 34 },
    { 0, 0 },
    { 0x16bd1755, 90 },
    { 0, 0 },
    { 0x2ced1757, 95 },
    { 0xaefd1757, 78 },
    { 0xe42d1759, 66 },
    { 0, 0 },
    { 0xe9d175b, 61 },
    { 0xe62d175c, 46 },
    { 0x6cfd175c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cad1760, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26d1766, 98 },
    { 0xd47d1767, 61 },
    { 0, 0 },
    { 0x589d1769, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85d176c, 98 },
    { 0xd97d176c, 23 },
    { 0x2f4d176c, 88 },
    { 0x981d176d, 43 },
    { 0xffed1770, 98 },
    { 0x95ed1771, 95 },
    { 0xd90d1772, 33 },
    { 0x9eed1772, 101 },
    { 0xa7ad1774, 43 },
    { 0x929d1773, 33 },
    { 0, 0 },
    { 0xff9d1777, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fd177c, 33 },
    { 0x5cbd177d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86d1780, 61 },
    { 0, 0 },
    { 0x9b6d1782, 100 },
    { 0x363d1783, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136d1788, 96 },
    { 0xef0d1789, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8d1790, 46 },
    { 0xd91d1791, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d1799, 46 },
    { 0xb27d1799, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cd179f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d17a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6d17b3, 34 },
    { 0x686d17b3, 23 },
    { 0, 0 },
    { 0x50bd17b6, 53 },
    { 0, 0 },
    { 0x54fd17b8, 84 },
    { 0, 0 },
    { 0x4add17ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9d17c1, 91 },
    { 0x40bd17c1, 55 },
    { 0x130d17c3, 33 },
    { 0x77dd17c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbd17c7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfd17d0, 46 },
    { 0, 0 },
    { 0x23dd17d2, 95 },
    { 0x8bcd17d2, 95 },
    { 0x12bd17d2, 23 },
    { 0xfd1d17d2, 61 },
    { 0, 0 },
    { 0x860d17d7, 95 },
    { 0x463d17d8, 65 },
    { 0x56d17d9, 46 },
    { 0xdb8d17da, 85 },
    { 0x479d17db, 97 },
    { 0xc19d17d9, 91 },
    { 0xf07d17d9, 43 },
    { 0x2e5d17d7, 98 },
    { 0x799d17d7, 85 },
    { 0x98ed17da, 34 },
    { 0xd07d17e1, 34 },
    { 0x74d17e0, 95 },
    { 0x490d17da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dd17eb, 23 },
    { 0x920d17ec, 87 },
    { 0xb3dd17ec, 23 },
    { 0xc9fd17ed, 33 },
    { 0xbe0d17ef, 66 },
    { 0x1f7d17f0, 17 },
    { 0xc3bd17f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x595d17f4, 38 },
    { 0, 0 },
    { 0xc21d17f6, 77 },
    { 0xfd4d17f7, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314d17fc, 91 },
    { 0xcd8d17fd, 23 },
    { 0xbfed17fe, 101 },
    { 0x5b1d17ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dd1806, 33 },
    { 0x6b8d1807, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cd180a, 95 },
    { 0x7a5d180b, 90 },
    { 0x539d180c, 23 },
    { 0xdc7d180c, 91 },
    { 0xb81d180e, 33 },
    { 0xbcad180a, 90 },
    { 0xaa3d1810, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8d1813, 100 },
    { 0x8a7d1814, 43 },
    { 0, 0 },
    { 0x746d1816, 90 },
    { 0xcd2d1816, 95 },
    { 0x53d1818, 95 },
    { 0xf4d1816, 33 },
    { 0, 0 },
    { 0xd55d181b, 33 },
    { 0, 0 },
    { 0x885d181d, 23 },
    { 0x796d181d, 98 },
    { 0x52fd181f, 61 },
    { 0xd59d181e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fad1828, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8d182f, 56 },
    { 0, 0 },
    { 0xab2d1831, 19 },
    { 0xa08d1832, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531d1838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53d183d, 55 },
    { 0x2cbd183d, 95 },
    { 0xd46d183d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50d1844, 61 },
    { 0x1c9d1845, 55 },
    { 0xf2bd1846, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959d1853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x459d1856, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65d185a, 61 },
    { 0xb6ad185b, 94 },
    { 0x7d2d185c, 55 },
    { 0x3ead185d, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918d1868, 76 },
    { 0, 0 },
    { 0x55ed186a, 95 },
    { 0xe22d186a, 98 },
    { 0xe5cd186c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd1870, 98 },
    { 0x4cd1871, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20d1877, 23 },
    { 0xc4cd1878, 71 },
    { 0xe96d1879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cd187c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129d1882, 53 },
    { 0x61cd1882, 61 },
    { 0xbb2d1884, 66 },
    { 0xe7dd1882, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1888, 23 },
    { 0x6bad1888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccd1890, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd1895, 61 },
    { 0xc8ad1896, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad1899, 65 },
    { 0x9b3d189a, 76 },
    { 0, 0 },
    { 0xccbd189c, 78 },
    { 0x3aad189d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60d18a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02d18aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccd18b0, 91 },
    { 0x504d18b0, 87 },
    { 0xc9fd18b1, 90 },
    { 0, 0 },
    { 0x652d18b4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61d18b8, 34 },
    { 0, 0 },
    { 0x754d18ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406d18c2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1d18c9, 68 },
    { 0x7ffd18ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x551d18cd, 33 },
    { 0x62cd18cd, 93 },
    { 0, 0 },
    { 0xc45d18d0, 60 },
    { 0xd02d18d1, 98 },
    { 0x59ed18d2, 33 },
    { 0x872d18d3, 61 },
    { 0x70ad18d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7d18d7, 84 },
    { 0x76dd18d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bd18dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9d18e0, 66 },
    { 0, 0 },
    { 0x461d18e2, 33 },
    { 0x43d18e2, 61 },
    { 0xaa8d18e4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d18e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459d18ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0d18f7, 34 },
    { 0x506d18f8, 78 },
    { 0xff1d18f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262d18fe, 91 },
    { 0x7add18fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x378d1902, 90 },
    { 0, 0 },
    { 0x5ddd1904, 76 },
    { 0x834d1904, 95 },
    { 0xdb0d1904, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1d1917, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666d191b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbd1920, 46 },
    { 0xa7cd1921, 99 },
    { 0x938d1922, 52 },
    { 0xc2fd1923, 98 },
    { 0xfefd1923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x964d1927, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577d192f, 90 },
    { 0xe68d1930, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfd1935, 61 },
    { 0xccbd1935, 33 },
    { 0xafcd1935, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdd193a, 76 },
    { 0, 0 },
    { 0x889d193c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dd1949, 61 },
    { 0x167d1949, 90 },
    { 0, 0 },
    { 0x959d194c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x739d194f, 90 },
    { 0xd96d1950, 33 },
    { 0x236d194f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594d1955, 98 },
    { 0x51d1955, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0d195c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d1961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74d1965, 98 },
    { 0x7fcd1966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd196e, 33 },
    { 0x6e1d196f, 34 },
    { 0x1a5d1970, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528d1977, 23 },
    { 0x93cd1978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d197f, 98 },
    { 0x756d1980, 88 },
    { 0x46ad1981, 77 },
    { 0x543d197f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0d1985, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8d198b, 78 },
    { 0, 0 },
    { 0xf9bd198d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172d1996, 76 },
    { 0x664d1997, 23 },
    { 0xbd1998, 33 },
    { 0xee1d1997, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d199c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2d19a0, 93 },
    { 0xc0ad19a1, 33 },
    { 0xfa7d19a0, 77 },
    { 0xcdfd19a2, 23 },
    { 0x839d19a4, 98 },
    { 0xc26d19a5, 41 },
    { 0x448d19a6, 66 },
    { 0x4a9d19a7, 46 },
    { 0xf6ed19a8, 52 },
    { 0x21fd19a8, 55 },
    { 0x504d19a4, 69 },
    { 0x89d19a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335d19af, 23 },
    { 0x666d19af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d19b3, 33 },
    { 0x199d19b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebd19b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fd19bd, 93 },
    { 0, 0 },
    { 0x64ad19bf, 76 },
    { 0x62ad19c0, 90 },
    { 0xf21d19c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d19c4, 43 },
    { 0xbf7d19c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04d19c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2d19cd, 33 },
    { 0, 0 },
    { 0xb1fd19cf, 98 },
    { 0xdf7d19cf, 61 },
    { 0, 0 },
    { 0xacfd19d2, 95 },
    { 0, 0 },
    { 0x9b2d19d4, 77 },
    { 0x36dd19d4, 91 },
    { 0x223d19d6, 93 },
    { 0x155d19d6, 95 },
    { 0x161d19d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd19e3, 95 },
    { 0xdd3d19e4, 43 },
    { 0x374d19e5, 91 },
    { 0x4d1d19e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d19f0, 61 },
    { 0x6bed19f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbd19f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d19f8, 52 },
    { 0x947d19f8, 76 },
    { 0x1e4d19fa, 90 },
    { 0xaf8d19fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947d1a02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175d1a06, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dd1a0e, 34 },
    { 0x7bed1a0f, 95 },
    { 0xbf4d1a0e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd1a17, 23 },
    { 0x1bad1a18, 90 },
    { 0, 0 },
    { 0x3b1d1a1a, 98 },
    { 0x14fd1a1b, 76 },
    { 0x1e0d1a1c, 33 },
    { 0, 0 },
    { 0xd1bd1a1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07d1a22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d1a29, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x268d1a2c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d1a2f, 90 },
    { 0, 0 },
    { 0xed9d1a31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ad1a34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cd1a38, 36 },
    { 0, 0 },
    { 0xde0d1a3a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d1a41, 66 },
    { 0x7e5d1a42, 34 },
    { 0x555d1a42, 75 },
    { 0, 0 },
    { 0x8e2d1a45, 68 },
    { 0xcd8d1a46, 91 },
    { 0, 0 },
    { 0x998d1a48, 78 },
    { 0, 0 },
    { 0x883d1a4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebd1a4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5d1a54, 76 },
    { 0, 0 },
    { 0x8e5d1a56, 61 },
    { 0x147d1a57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x53d1a5a, 33 },
    { 0x544d1a5b, 84 },
    { 0, 0 },
    { 0x7cad1a5d, 93 },
    { 0, 0 },
    { 0x20ad1a5f, 42 },
    { 0x2e7d1a5f, 92 },
    { 0x49bd1a5f, 90 },
    { 0x2bd1a60, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37d1a69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d1a6c, 61 },
    { 0xa55d1a6d, 61 },
    { 0, 0 },
    { 0xaf7d1a6f, 95 },
    { 0x1e2d1a70, 61 },
    { 0x5fbd1a71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cd1a76, 33 },
    { 0xf97d1a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2d1a7e, 23 },
    { 0x1e4d1a7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4d1a8b, 61 },
    { 0x4b3d1a8b, 84 },
    { 0xa3ed1a8d, 55 },
    { 0x327d1a8d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dd1a91, 33 },
    { 0xdb4d1a92, 77 },
    { 0xbdad1a93, 61 },
    { 0xbf8d1a91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ed1a9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d1aa9, 90 },
    { 0xa4d1aaa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5d1aae, 95 },
    { 0, 0 },
    { 0xf02d1ab0, 23 },
    { 0x6d8d1ab1, 43 },
    { 0x399d1ab2, 90 },
    { 0x44cd1ab2, 33 },
    { 0x579d1ab2, 40 },
    { 0, 0 },
    { 0xf06d1ab6, 76 },
    { 0xa65d1ab7, 76 },
    { 0x9dad1ab8, 90 },
    { 0, 0 },
    { 0xa92d1aba, 76 },
    { 0, 0 },
    { 0xbb8d1abc, 98 },
    { 0x1b4d1abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acd1ac1, 99 },
    { 0, 0 },
    { 0xe47d1ac3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3d1ac6, 34 },
    { 0, 0 },
    { 0x27ed1ac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533d1acc, 70 },
    { 0x2bcd1acc, 67 },
    { 0x5ed1ace, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4d1ad5, 93 },
    { 0x203d1ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6d1add, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ad1ae6, 90 },
    { 0xb08d1ae7, 66 },
    { 0x922d1ae8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34d1aee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ed1af1, 90 },
    { 0, 0 },
    { 0xb23d1af3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91d1af8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bd1afb, 61 },
    { 0x18bd1afb, 61 },
    { 0x5cdd1afd, 46 },
    { 0, 0 },
    { 0xb6fd1aff, 91 },
    { 0, 0 },
    { 0x651d1b01, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fd1b08, 100 },
    { 0xb47d1b09, 76 },
    { 0, 0 },
    { 0x280d1b0b, 98 },
    { 0x823d1b0b, 93 },
    { 0, 0 },
    { 0x371d1b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7d1b13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ad1b16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26d1b19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ed1b1d, 95 },
    { 0xbe4d1b1e, 33 },
    { 0, 0 },
    { 0xce5d1b20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716d1b27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9d1b2a, 61 },
    { 0xb34d1b2a, 92 },
    { 0xf42d1b2c, 87 },
    { 0x909d1b2d, 90 },
    { 0, 0 },
    { 0x80bd1b2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ed1b32, 90 },
    { 0x581d1b32, 23 },
    { 0xe33d1b34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d1b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849d1b3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2d1b3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d1b45, 91 },
    { 0xa32d1b46, 77 },
    { 0xd2fd1b47, 90 },
    { 0xaead1b46, 98 },
    { 0xeccd1b48, 23 },
    { 0xe6bd1b46, 90 },
    { 0x17d1b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efd1b50, 61 },
    { 0x5cd1b51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8d1b54, 83 },
    { 0, 0 },
    { 0x28d1b56, 68 },
    { 0x78fd1b57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d1b5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad1b62, 61 },
    { 0, 0 },
    { 0x562d1b64, 78 },
    { 0, 0 },
    { 0x5dd1b66, 34 },
    { 0x9a9d1b67, 100 },
    { 0, 0 },
    { 0xe67d1b69, 98 },
    { 0xf15d1b6a, 23 },
    { 0x206d1b6b, 38 },
    { 0xa9d1b6c, 91 },
    { 0, 0 },
    { 0x12fd1b6e, 49 },
    { 0x83d1b6e, 95 },
    { 0, 0 },
    { 0xd2cd1b71, 95 },
    { 0x686d1b71, 76 },
    { 0, 0 },
    { 0xa42d1b74, 61 },
    { 0xae4d1b75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9d1b79, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18d1b7e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37d1b82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d1b85, 61 },
    { 0, 0 },
    { 0x592d1b87, 23 },
    { 0xb32d1b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53d1b8e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d1b91, 98 },
    { 0x5ebd1b91, 61 },
    { 0, 0 },
    { 0xf1ed1b94, 75 },
    { 0xb01d1b95, 33 },
    { 0x9ead1b95, 69 },
    { 0x574d1b95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7d1b9c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d1ba6, 95 },
    { 0, 0 },
    { 0x832d1ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fd1bb0, 76 },
    { 0x94d1bb1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93d1bbb, 88 },
    { 0x2cad1bbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ed1bbf, 76 },
    { 0x330d1bc0, 75 },
    { 0, 0 },
    { 0x34cd1bc2, 97 },
    { 0x5d3d1bc3, 76 },
    { 0, 0 },
    { 0xca3d1bc5, 33 },
    { 0x344d1bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2d1bca, 84 },
    { 0xe30d1bcb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2d1bcf, 91 },
    { 0x61d1bd0, 61 },
    { 0x208d1bd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556d1bd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0d1bdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d1be4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d1beb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fad1bf0, 99 },
    { 0x2abd1bf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984d1bfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06d1bfd, 85 },
    { 0x30ad1bfe, 76 },
    { 0, 0 },
    { 0x4e6d1c00, 78 },
    { 0xf46d1c00, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ad1c05, 91 },
    { 0x648d1c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372d1c0c, 87 },
    { 0xf99d1c0c, 98 },
    { 0, 0 },
    { 0xb39d1c0f, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cd1c12, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340d1c28, 45 },
    { 0x81d1c29, 90 },
    { 0x620d1c2a, 90 },
    { 0x3ead1c2b, 46 },
    { 0x8b9d1c2c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977d1c30, 61 },
    { 0x105d1c30, 46 },
    { 0xdbbd1c32, 95 },
    { 0, 0 },
    { 0xe16d1c34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd1c38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bd1c3c, 90 },
    { 0x4a9d1c3c, 78 },
    { 0x840d1c3c, 90 },
    { 0xb07d1c3f, 46 },
    { 0xae5d1c40, 23 },
    { 0x95dd1c41, 95 },
    { 0x6d1d1c42, 23 },
    { 0xf5ed1c40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3d1c50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937d1c59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x211d1c5c, 55 },
    { 0xf87d1c5d, 23 },
    { 0x46ad1c5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103d1c6b, 98 },
    { 0x5bad1c6c, 90 },
    { 0xead1c6d, 55 },
    { 0x2c2d1c6e, 90 },
    { 0xd71d1c6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d1c76, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd1c79, 77 },
    { 0x543d1c79, 61 },
    { 0xcd4d1c7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d1c7e, 100 },
    { 0x773d1c7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05d1c83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ad1c89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ad1c8d, 23 },
    { 0xf8ad1c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efd1c91, 34 },
    { 0x663d1c91, 95 },
    { 0xda0d1c93, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x582d1c96, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdd1c9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4d1ca2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24d1ca6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839d1caa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ed1caf, 66 },
    { 0, 0 },
    { 0x759d1cb1, 84 },
    { 0, 0 },
    { 0x150d1cb3, 17 },
    { 0x9dbd1cb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dd1cbf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd1cc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d1cc5, 93 },
    { 0xbfed1cc6, 81 },
    { 0xc6ed1cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d1cca, 90 },
    { 0, 0 },
    { 0x185d1ccc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288d1cd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278d1cda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485d1ce1, 95 },
    { 0x49ed1ce1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579d1ce7, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efd1cec, 90 },
    { 0x1f9d1ced, 23 },
    { 0xa76d1cec, 53 },
    { 0x253d1cef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x411d1cf2, 92 },
    { 0x15fd1cf3, 100 },
    { 0x142d1cf3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d1cf9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78d1d00, 40 },
    { 0xa58d1d01, 23 },
    { 0xabad1d01, 90 },
    { 0x4e4d1d01, 84 },
    { 0, 0 },
    { 0x26d1d05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3d1d0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13d1d10, 46 },
    { 0, 0 },
    { 0xd67d1d12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdd1d17, 22 },
    { 0, 0 },
    { 0x5d9d1d19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcded1d1d, 76 },
    { 0xef6d1d1d, 95 },
    { 0x1bed1d1d, 100 },
    { 0x53bd1d1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260d1d28, 76 },
    { 0xb05d1d29, 23 },
    { 0xa47d1d28, 33 },
    { 0x60ed1d29, 95 },
    { 0x42ed1d2b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d1d35, 95 },
    { 0, 0 },
    { 0x3dd1d37, 55 },
    { 0x1f2d1d37, 94 },
    { 0x387d1d38, 33 },
    { 0x3dd1d39, 84 },
    { 0, 0 },
    { 0x4ed1d3c, 17 },
    { 0, 0 },
    { 0xbedd1d3e, 52 },
    { 0, 0 },
    { 0xf2ad1d40, 76 },
    { 0x656d1d41, 65 },
    { 0xc78d1d40, 101 },
    { 0, 0 },
    { 0x9f1d1d44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabd1d4c, 43 },
    { 0xbfbd1d4d, 90 },
    { 0xa61d1d4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ad1d51, 94 },
    { 0, 0 },
    { 0xb40d1d53, 95 },
    { 0xe47d1d54, 76 },
    { 0xb79d1d55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cd1d59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d1d5f, 61 },
    { 0x53ed1d5f, 46 },
    { 0x224d1d5f, 36 },
    { 0, 0 },
    { 0xf20d1d63, 51 },
    { 0, 0 },
    { 0xb3bd1d65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57d1d6a, 33 },
    { 0x70bd1d6a, 93 },
    { 0xd70d1d6c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddd1d73, 66 },
    { 0x5f3d1d73, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94d1d78, 61 },
    { 0, 0 },
    { 0x5edd1d7a, 17 },
    { 0x77bd1d7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d1d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118d1d85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d1d8c, 93 },
    { 0x766d1d8d, 55 },
    { 0, 0 },
    { 0xfc1d1d8f, 61 },
    { 0, 0 },
    { 0xe99d1d91, 23 },
    { 0, 0 },
    { 0x748d1d93, 33 },
    { 0, 0 },
    { 0x89dd1d95, 55 },
    { 0, 0 },
    { 0x446d1d97, 66 },
    { 0xde5d1d98, 52 },
    { 0, 0 },
    { 0xb7d1d9a, 95 },
    { 0xc75d1d9b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ed1da0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d1da3, 33 },
    { 0, 0 },
    { 0xcded1da5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fd1da9, 61 },
    { 0, 0 },
    { 0x3d0d1dab, 61 },
    { 0xe17d1dac, 34 },
    { 0x4c2d1dad, 55 },
    { 0, 0 },
    { 0x946d1daf, 98 },
    { 0xceed1db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851d1db5, 61 },
    { 0xf4fd1db6, 76 },
    { 0x443d1db7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d1dba, 46 },
    { 0, 0 },
    { 0xf03d1dbc, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd1dc0, 90 },
    { 0x997d1dc0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37d1dca, 66 },
    { 0x70ad1dca, 90 },
    { 0, 0 },
    { 0x228d1dcd, 33 },
    { 0x824d1dce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8add1dd2, 99 },
    { 0x8c2d1dd3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd1dd9, 95 },
    { 0x6cbd1dda, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5d1ddd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8d1de8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48d1dec, 23 },
    { 0, 0 },
    { 0x10ed1dee, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ad1df2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4d1dfa, 41 },
    { 0xd79d1dfa, 90 },
    { 0x609d1dfc, 52 },
    { 0xff9d1dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93d1e01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cd1e07, 100 },
    { 0x2fad1e08, 55 },
    { 0x458d1e09, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9d1e14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x233d1e17, 76 },
    { 0x2ed1e18, 34 },
    { 0, 0 },
    { 0x3ead1e1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2d1e1e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58d1e22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eed1e28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d1e2c, 55 },
    { 0, 0 },
    { 0x60fd1e2e, 95 },
    { 0x607d1e2f, 76 },
    { 0xc25d1e2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x863d1e33, 91 },
    { 0x585d1e34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x249d1e3a, 98 },
    { 0x139d1e3b, 75 },
    { 0x69d1e3c, 65 },
    { 0xac6d1e3d, 61 },
    { 0x88dd1e3b, 95 },
    { 0xe78d1e3f, 80 },
    { 0xcdd1e3f, 55 },
    { 0x85fd1e41, 100 },
    { 0xf49d1e3d, 95 },
    { 0xfbed1e43, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ed1e57, 90 },
    { 0x66dd1e58, 90 },
    { 0x1c0d1e59, 98 },
    { 0xe39d1e57, 43 },
    { 0xf5fd1e5b, 90 },
    { 0xe0d1e5c, 95 },
    { 0xe10d1e5d, 61 },
    { 0xef8d1e59, 95 },
    { 0x29d1e5a, 46 },
    { 0xf53d1e5c, 33 },
    { 0x77ed1e61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2d1e67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddd1e6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd1e75, 66 },
    { 0xa02d1e76, 52 },
    { 0xcaed1e75, 61 },
    { 0, 0 },
    { 0x724d1e79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ad1e7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x977d1e82, 84 },
    { 0x1d9d1e83, 84 },
    { 0xf8ed1e84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0d1e88, 90 },
    { 0, 0 },
    { 0xc87d1e8a, 90 },
    { 0x72ad1e8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75d1e8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x962d1e91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0d1e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x737d1e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eed1ea3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x560d1ea6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fd1eaa, 95 },
    { 0x858d1eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84d1eae, 98 },
    { 0, 0 },
    { 0xdc1d1eb0, 65 },
    { 0, 0 },
    { 0x3add1eb2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d1eb6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfd1ebb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfd1ebe, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ed1ec2, 95 },
    { 0, 0 },
    { 0x376d1ec4, 43 },
    { 0, 0 },
    { 0xe58d1ec6, 91 },
    { 0x549d1ec7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d1ecd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afd1ed4, 23 },
    { 0xa72d1ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacd1eda, 36 },
    { 0, 0 },
    { 0x446d1edc, 43 },
    { 0x855d1edd, 55 },
    { 0x997d1edc, 95 },
    { 0xf2dd1edc, 61 },
    { 0x5dfd1ee0, 61 },
    { 0xcdad1ee0, 81 },
    { 0xf42d1ee1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09d1eec, 9 },
    { 0, 0 },
    { 0x187d1eee, 61 },
    { 0xfebd1eef, 36 },
    { 0x329d1ef0, 78 },
    { 0xb39d1ef1, 76 },
    { 0, 0 },
    { 0x5ead1ef3, 98 },
    { 0xe07d1ef4, 93 },
    { 0x14cd1ef5, 33 },
    { 0, 0 },
    { 0xe2d1ef7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd1efe, 90 },
    { 0, 0 },
    { 0xdefd1f00, 95 },
    { 0xe0bd1f01, 66 },
    { 0xff8d1f02, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfd1f06, 68 },
    { 0, 0 },
    { 0xc88d1f08, 90 },
    { 0xfb6d1f09, 87 },
    { 0, 0 },
    { 0x594d1f0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9d1f11, 38 },
    { 0x9b5d1f12, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d1f17, 98 },
    { 0x2b6d1f18, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4d1f1d, 90 },
    { 0x5f7d1f1e, 52 },
    { 0, 0 },
    { 0x464d1f20, 90 },
    { 0x48bd1f21, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94d1f27, 61 },
    { 0x294d1f27, 76 },
    { 0xb0d1f29, 33 },
    { 0x8add1f2a, 90 },
    { 0x1b9d1f2b, 61 },
    { 0x4dad1f2b, 94 },
    { 0xeefd1f2b, 55 },
    { 0x1b7d1f2b, 61 },
    { 0xadd1f2f, 55 },
    { 0xc5bd1f28, 61 },
    { 0x9bed1f29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d1f34, 34 },
    { 0xc0bd1f35, 68 },
    { 0x640d1f36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cd1f40, 90 },
    { 0xae9d1f41, 90 },
    { 0xddcd1f41, 23 },
    { 0xefbd1f40, 23 },
    { 0x2c3d1f44, 85 },
    { 0xc9d1f45, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abd1f49, 78 },
    { 0, 0 },
    { 0x53ed1f4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaed1f4f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfed1f56, 52 },
    { 0x599d1f56, 33 },
    { 0x9b4d1f58, 76 },
    { 0xfad1f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697d1f5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133d1f63, 68 },
    { 0xaddd1f64, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18d1f68, 23 },
    { 0x139d1f69, 46 },
    { 0xe62d1f69, 23 },
    { 0x201d1f6b, 23 },
    { 0, 0 },
    { 0x92fd1f6d, 43 },
    { 0, 0 },
    { 0xfe7d1f6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded1f72, 78 },
    { 0x223d1f73, 100 },
    { 0xa01d1f73, 90 },
    { 0xb3dd1f73, 33 },
    { 0, 0 },
    { 0x54bd1f77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d1f7b, 61 },
    { 0, 0 },
    { 0xbedd1f7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cd1f88, 55 },
    { 0, 0 },
    { 0xf64d1f8a, 98 },
    { 0xe14d1f8a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fd1f91, 90 },
    { 0xc7fd1f91, 95 },
    { 0, 0 },
    { 0x619d1f94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44d1f9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79d1fa0, 33 },
    { 0xbddd1fa0, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ad1fa5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd1fa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585d1fae, 75 },
    { 0x6f8d1faf, 61 },
    { 0x68dd1fb0, 95 },
    { 0, 0 },
    { 0x46bd1fb2, 98 },
    { 0x473d1fb2, 98 },
    { 0x191d1fb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bd1fb7, 98 },
    { 0xa0ed1fb8, 33 },
    { 0x184d1fb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ad1fbe, 95 },
    { 0xc2ad1fbf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ad1fc5, 23 },
    { 0x2b3d1fc5, 98 },
    { 0, 0 },
    { 0x21dd1fc8, 23 },
    { 0xd31d1fc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408d1fcf, 55 },
    { 0xc70d1fd0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffd1fd6, 90 },
    { 0, 0 },
    { 0xe6fd1fd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x970d1fdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd1fe1, 61 },
    { 0x205d1fe2, 34 },
    { 0xe3dd1fe3, 85 },
    { 0x86ad1fe4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7d1fe9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed1fed, 90 },
    { 0x29ed1fed, 95 },
    { 0xc62d1fef, 90 },
    { 0x514d1ff0, 98 },
    { 0xa9bd1fef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8d1ff4, 34 },
    { 0xd35d1ff5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47d1ff8, 80 },
    { 0xdded1ff9, 93 },
    { 0, 0 },
    { 0x91ad1ffb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bd1ffe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ad2005, 77 },
    { 0x849d2005, 33 },
    { 0xf6bd2007, 90 },
    { 0x151d2008, 95 },
    { 0x1f7d2009, 93 },
    { 0xd91d2008, 55 },
    { 0x7e6d200a, 90 },
    { 0x1fcd200c, 76 },
    { 0x2b0d200d, 95 },
    { 0xbe8d2008, 95 },
    { 0xf4fd2005, 83 },
    { 0, 0 },
    { 0xa70d2011, 67 },
    { 0, 0 },
    { 0xefad2013, 99 },
    { 0, 0 },
    { 0x46d2015, 95 },
    { 0, 0 },
    { 0x4aed2017, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836d201e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd2028, 95 },
    { 0xc93d2029, 43 },
    { 0xcced2028, 40 },
    { 0, 0 },
    { 0xc33d202c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd202f, 23 },
    { 0, 0 },
    { 0x7a5d2031, 78 },
    { 0xa43d2032, 46 },
    { 0x1cd2032, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d203c, 23 },
    { 0x8d6d203c, 55 },
    { 0, 0 },
    { 0x9acd203f, 98 },
    { 0, 0 },
    { 0xcd2d2041, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd2044, 98 },
    { 0, 0 },
    { 0xb08d2046, 33 },
    { 0x6dad2047, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd204b, 98 },
    { 0, 0 },
    { 0x91ad204d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cd2050, 23 },
    { 0xe60d2050, 98 },
    { 0, 0 },
    { 0xe37d2053, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdd2056, 76 },
    { 0x3a1d2057, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbd205e, 43 },
    { 0, 0 },
    { 0x12ed2060, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d2066, 81 },
    { 0, 0 },
    { 0x97dd2068, 91 },
    { 0x950d2069, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0d2071, 78 },
    { 0x797d2071, 91 },
    { 0, 0 },
    { 0x85bd2074, 33 },
    { 0xf8bd2075, 90 },
    { 0xb0cd2075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bd207d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d2080, 33 },
    { 0x569d2081, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad2084, 95 },
    { 0x4dbd2084, 85 },
    { 0, 0 },
    { 0x845d2087, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28d208a, 52 },
    { 0x9dfd208b, 93 },
    { 0x4dd208c, 90 },
    { 0x75ed208d, 34 },
    { 0x55dd208e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d2096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7d2099, 43 },
    { 0x7ccd2099, 93 },
    { 0x31ad209b, 76 },
    { 0x11ad209b, 23 },
    { 0x1cd209c, 43 },
    { 0x255d209d, 53 },
    { 0xbc9d209f, 98 },
    { 0x45bd209a, 84 },
    { 0, 0 },
    { 0x208d20a2, 90 },
    { 0, 0 },
    { 0x645d20a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ad20a9, 76 },
    { 0x3bcd20a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504d20b1, 61 },
    { 0x8b2d20b2, 88 },
    { 0x56ad20b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ed20b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bd20b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bad20bd, 95 },
    { 0xaffd20be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fd20c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6d20c5, 95 },
    { 0x274d20c5, 95 },
    { 0x3b3d20c7, 98 },
    { 0x68cd20c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bd20cb, 69 },
    { 0x559d20cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebd20d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3d20d6, 61 },
    { 0x5acd20d7, 33 },
    { 0xea5d20d7, 87 },
    { 0, 0 },
    { 0xaaed20da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09d20dd, 40 },
    { 0x6b6d20de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd20e4, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d20ec, 23 },
    { 0, 0 },
    { 0x61ad20ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cd20f2, 34 },
    { 0, 0 },
    { 0xa72d20f4, 43 },
    { 0x133d20f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dd20fc, 85 },
    { 0xf72d20fc, 90 },
    { 0x18dd20fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fd2103, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04d2107, 90 },
    { 0xa70d2107, 94 },
    { 0xc1d2107, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ed2111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x463d2114, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3d2117, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ad211f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd2124, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ed2127, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25d212b, 61 },
    { 0x130d212c, 90 },
    { 0xb01d212c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8d2132, 95 },
    { 0xffdd2132, 98 },
    { 0xf47d2134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d213b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461d213f, 95 },
    { 0, 0 },
    { 0x95fd2141, 90 },
    { 0, 0 },
    { 0x9dfd2143, 98 },
    { 0x9e9d2144, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dd2149, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4d214c, 61 },
    { 0xbe6d214d, 55 },
    { 0, 0 },
    { 0x226d214f, 90 },
    { 0x525d2150, 66 },
    { 0x668d2151, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7d2158, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d215b, 17 },
    { 0x92d215c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdd2160, 98 },
    { 0x6e5d2161, 61 },
    { 0x890d2162, 67 },
    { 0, 0 },
    { 0x519d2164, 52 },
    { 0x7ead2165, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x287d2168, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944d216d, 88 },
    { 0x6d9d216d, 33 },
    { 0x11ed216e, 65 },
    { 0, 0 },
    { 0x510d2171, 55 },
    { 0, 0 },
    { 0x411d2173, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37d2177, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x124d217a, 33 },
    { 0xca8d217b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ad2185, 90 },
    { 0x11dd2186, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x101d2189, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d2191, 99 },
    { 0, 0 },
    { 0xccad2193, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x416d2196, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624d219d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcd21a1, 84 },
    { 0x816d21a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dd21a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746d21ae, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6d21b1, 23 },
    { 0x1ded21b2, 33 },
    { 0xe74d21b3, 61 },
    { 0xc65d21b1, 4 },
    { 0, 0 },
    { 0xabfd21b6, 69 },
    { 0x59d21b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dd21c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18d21c3, 34 },
    { 0xb46d21c4, 61 },
    { 0xf63d21c5, 56 },
    { 0xf2cd21c6, 67 },
    { 0, 0 },
    { 0xdb8d21c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81d21cd, 77 },
    { 0x281d21ce, 76 },
    { 0xd06d21cf, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d21d4, 93 },
    { 0x4f7d21d5, 90 },
    { 0xe59d21d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1d21da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afd21df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaad21e2, 98 },
    { 0x5ecd21e2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4d21e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156d21ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5d21f1, 52 },
    { 0xa3ad21f2, 23 },
    { 0x1cad21f3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513d21f8, 65 },
    { 0x773d21f8, 90 },
    { 0, 0 },
    { 0x1f8d21fb, 77 },
    { 0xd7ed21fc, 66 },
    { 0xe17d21fd, 17 },
    { 0, 0 },
    { 0x5c5d21ff, 17 },
    { 0x9b9d2200, 91 },
    { 0x56d2201, 17 },
    { 0xa2ed2202, 61 },
    { 0, 0 },
    { 0x714d2204, 90 },
    { 0x580d2204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814d220d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3d2212, 90 },
    { 0x7f9d2213, 88 },
    { 0xd37d2212, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d2218, 55 },
    { 0x82ed2219, 33 },
    { 0x71ad221a, 61 },
    { 0xd9cd221b, 23 },
    { 0x408d221b, 33 },
    { 0x28ed221d, 90 },
    { 0xeaed221d, 69 },
    { 0x49fd221f, 23 },
    { 0x119d221b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd2223, 61 },
    { 0x5f0d2224, 90 },
    { 0x1e4d2225, 55 },
    { 0x79bd2225, 55 },
    { 0xfd7d2223, 40 },
    { 0, 0 },
    { 0x70ad2229, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd222f, 76 },
    { 0x641d2230, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acd2239, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dd223c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcd223f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d2243, 34 },
    { 0, 0 },
    { 0xc34d2245, 76 },
    { 0xf21d2246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x215d2249, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed224e, 75 },
    { 0, 0 },
    { 0x52d2250, 90 },
    { 0, 0 },
    { 0x45dd2252, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82d2256, 78 },
    { 0, 0 },
    { 0xc0ed2258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273d225c, 76 },
    { 0x827d225d, 90 },
    { 0xf1ed225e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07d2266, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x801d2269, 99 },
    { 0x82fd226a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d226d, 55 },
    { 0xfe0d226e, 55 },
    { 0x8cbd226d, 61 },
    { 0, 0 },
    { 0xe9cd2271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1d2276, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4d227b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96d227e, 98 },
    { 0xd6dd227f, 87 },
    { 0x853d2280, 55 },
    { 0xb67d2280, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d2285, 61 },
    { 0x8d0d2285, 95 },
    { 0x481d2285, 61 },
    { 0, 0 },
    { 0x7dad2289, 90 },
    { 0x8ecd2289, 23 },
    { 0x39ed228b, 98 },
    { 0x75dd228c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x562d228f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcd2293, 33 },
    { 0x7d7d2294, 23 },
    { 0xc56d2294, 61 },
    { 0x651d2296, 98 },
    { 0xfe3d2294, 61 },
    { 0, 0 },
    { 0x7afd2299, 76 },
    { 0x596d2299, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d229d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138d22a2, 90 },
    { 0x627d22a3, 95 },
    { 0x33ed22a4, 34 },
    { 0xa14d22a5, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadd22a9, 23 },
    { 0, 0 },
    { 0x3e8d22ab, 23 },
    { 0xdafd22ac, 23 },
    { 0x961d22ad, 93 },
    { 0x2ad22ac, 33 },
    { 0, 0 },
    { 0xccdd22b0, 23 },
    { 0, 0 },
    { 0xf49d22b2, 34 },
    { 0x258d22b2, 34 },
    { 0x854d22b2, 98 },
    { 0xef3d22b5, 34 },
    { 0, 0 },
    { 0x2acd22b7, 33 },
    { 0, 0 },
    { 0x3f3d22b9, 47 },
    { 0x315d22ba, 61 },
    { 0xa6cd22bb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925d22c0, 75 },
    { 0xc8d22c1, 76 },
    { 0x257d22c2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbd22c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51d22cc, 68 },
    { 0x2a4d22cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ed22d1, 76 },
    { 0x2ddd22d2, 55 },
    { 0xbe1d22d3, 76 },
    { 0xecad22d2, 61 },
    { 0x7f6d22d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x359d22d8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42d22e1, 100 },
    { 0, 0 },
    { 0xc77d22e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdd22ea, 65 },
    { 0x61cd22eb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cd22f1, 76 },
    { 0, 0 },
    { 0x9d3d22f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452d22fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d22ff, 95 },
    { 0, 0 },
    { 0x3d2d2301, 23 },
    { 0x50d2301, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ed2307, 65 },
    { 0xa6ed2307, 95 },
    { 0x702d2308, 43 },
    { 0x8fed230a, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x438d230d, 34 },
    { 0x531d230e, 81 },
    { 0x648d230d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bad2315, 61 },
    { 0xbb1d2316, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffd231a, 83 },
    { 0xc68d231b, 61 },
    { 0xffad231c, 55 },
    { 0x244d231b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837d2321, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551d2325, 43 },
    { 0, 0 },
    { 0x55d2327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfed232c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add232f, 80 },
    { 0x4ecd2330, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cd2333, 99 },
    { 0xd5dd2334, 90 },
    { 0x56fd2335, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7d233a, 23 },
    { 0xbf6d233a, 61 },
    { 0x751d233c, 55 },
    { 0xb2fd233c, 23 },
    { 0xbb2d233e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4d2342, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd234b, 90 },
    { 0, 0 },
    { 0xe9d234d, 90 },
    { 0, 0 },
    { 0xc5d234f, 93 },
    { 0x1ecd2350, 61 },
    { 0x395d2351, 98 },
    { 0x1f5d234f, 90 },
    { 0x8c0d2353, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fed2358, 90 },
    { 0xc4cd2359, 98 },
    { 0, 0 },
    { 0x81dd235b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8d235e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x353d2361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766d2366, 73 },
    { 0, 0 },
    { 0x7add2368, 98 },
    { 0xf2bd2369, 34 },
    { 0x2abd2369, 87 },
    { 0, 0 },
    { 0xc90d236c, 46 },
    { 0, 0 },
    { 0x619d236e, 33 },
    { 0xaa0d236e, 42 },
    { 0x4c8d2370, 88 },
    { 0x87ed2370, 46 },
    { 0, 0 },
    { 0x651d2373, 100 },
    { 0x283d2374, 90 },
    { 0, 0 },
    { 0xbdad2376, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecd2379, 99 },
    { 0x7fcd2379, 23 },
    { 0, 0 },
    { 0x703d237c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ed2381, 43 },
    { 0x828d2382, 61 },
    { 0x803d2383, 85 },
    { 0xe2ed2384, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5d238b, 95 },
    { 0xccbd238c, 76 },
    { 0, 0 },
    { 0x97ed238e, 98 },
    { 0xaffd238e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ed2392, 2 },
    { 0, 0 },
    { 0x1b2d2394, 98 },
    { 0x430d2395, 33 },
    { 0x1d2394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35d2399, 95 },
    { 0xe83d2399, 90 },
    { 0xf5bd2399, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4d23a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1d23a3, 95 },
    { 0, 0 },
    { 0x45fd23a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d23a9, 95 },
    { 0x25bd23aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d23ae, 33 },
    { 0x9d9d23ae, 33 },
    { 0xccbd23af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcd23b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6d23ba, 33 },
    { 0, 0 },
    { 0xa20d23bc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453d23c0, 98 },
    { 0, 0 },
    { 0x199d23c2, 33 },
    { 0xe3dd23c2, 60 },
    { 0, 0 },
    { 0x56bd23c5, 23 },
    { 0x742d23c5, 90 },
    { 0, 0 },
    { 0xc16d23c8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d23d0, 23 },
    { 0x1bed23d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9d23d4, 99 },
    { 0xe7fd23d4, 43 },
    { 0xa2cd23d6, 61 },
    { 0x311d23d7, 46 },
    { 0x808d23d8, 90 },
    { 0xe55d23d8, 61 },
    { 0x538d23d4, 76 },
    { 0x370d23db, 61 },
    { 0x545d23d4, 33 },
    { 0x439d23dd, 98 },
    { 0x4c0d23de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaad23e2, 33 },
    { 0xb62d23e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad23e8, 87 },
    { 0xa92d23e8, 95 },
    { 0x9f5d23ea, 23 },
    { 0xb7fd23eb, 40 },
    { 0, 0 },
    { 0xfcbd23ed, 90 },
    { 0, 0 },
    { 0xad4d23ef, 23 },
    { 0x7e0d23f0, 55 },
    { 0x56fd23ef, 43 },
    { 0, 0 },
    { 0x993d23f3, 61 },
    { 0, 0 },
    { 0x53ed23f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dd23f9, 55 },
    { 0xb67d23fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cd2401, 78 },
    { 0x9f5d2402, 100 },
    { 0x396d2403, 90 },
    { 0xe17d2403, 33 },
    { 0x31d2405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7d2408, 90 },
    { 0xa66d2409, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xadad240c, 95 },
    { 0x406d240d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794d2413, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x989d2416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fd2419, 23 },
    { 0x80ad241a, 34 },
    { 0x5d7d241b, 90 },
    { 0x67bd241c, 85 },
    { 0, 0 },
    { 0x33d241e, 66 },
    { 0, 0 },
    { 0xb7fd2420, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dd2424, 90 },
    { 0x5dd2425, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1d242a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9d242d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67d2434, 88 },
    { 0, 0 },
    { 0x2f4d2436, 96 },
    { 0x9fdd2437, 61 },
    { 0x7ddd2436, 61 },
    { 0x666d2439, 35 },
    { 0, 0 },
    { 0x483d243b, 99 },
    { 0xe2ed243c, 90 },
    { 0xfc8d243c, 85 },
    { 0xc7ed243c, 61 },
    { 0x80cd243b, 91 },
    { 0xa91d243e, 55 },
    { 0x136d2440, 52 },
    { 0x6dcd2442, 98 },
    { 0x382d2443, 78 },
    { 0, 0 },
    { 0x1bfd2445, 2 },
    { 0x230d2446, 34 },
    { 0x5cad2446, 90 },
    { 0, 0 },
    { 0x37dd2449, 98 },
    { 0x4f6d244a, 76 },
    { 0, 0 },
    { 0x2f4d244c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915d2451, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0d2454, 23 },
    { 0x813d2454, 65 },
    { 0xdb2d2454, 98 },
    { 0x58d2456, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d245d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x494d2460, 52 },
    { 0xa4bd2460, 55 },
    { 0xe27d2461, 3 },
    { 0x68fd2463, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300d246a, 55 },
    { 0x24ed246b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bed246f, 100 },
    { 0, 0 },
    { 0x2cd2471, 17 },
    { 0x881d2472, 61 },
    { 0, 0 },
    { 0x96dd2474, 98 },
    { 0xa1dd2475, 90 },
    { 0xb68d2476, 61 },
    { 0x2e8d2474, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bd247f, 91 },
    { 0x694d2480, 65 },
    { 0x967d247f, 33 },
    { 0xd86d2482, 55 },
    { 0xc3dd247f, 36 },
    { 0x597d2484, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07d248a, 81 },
    { 0, 0 },
    { 0x3dcd248c, 66 },
    { 0x267d248c, 98 },
    { 0x2f5d248e, 95 },
    { 0x25fd248d, 98 },
    { 0x5d9d248d, 33 },
    { 0x8d5d2491, 53 },
    { 0xd0dd2492, 65 },
    { 0x650d2493, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ad2496, 85 },
    { 0xaeed2497, 33 },
    { 0x9a9d2496, 33 },
    { 0x23d2499, 34 },
    { 0, 0 },
    { 0x83ed249b, 23 },
    { 0xf8bd249c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688d24a3, 46 },
    { 0, 0 },
    { 0x2fcd24a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5d24b2, 23 },
    { 0x1b3d24b3, 43 },
    { 0xafdd24b4, 52 },
    { 0, 0 },
    { 0x823d24b6, 43 },
    { 0, 0 },
    { 0x4c2d24b8, 23 },
    { 0x747d24b9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7d24be, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d24c3, 23 },
    { 0x45bd24c3, 34 },
    { 0x4d8d24c3, 55 },
    { 0x266d24c6, 99 },
    { 0xaed24c7, 33 },
    { 0xc5cd24c8, 34 },
    { 0xe5bd24c7, 100 },
    { 0xd8ed24c8, 76 },
    { 0x236d24cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbd24ce, 95 },
    { 0x25bd24ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d24d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d24d5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7d24df, 93 },
    { 0x60ad24e0, 95 },
    { 0x6e4d24df, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32d24e4, 40 },
    { 0xf35d24e5, 33 },
    { 0xfa1d24e6, 90 },
    { 0x79ad24e5, 95 },
    { 0xaa6d24e8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5d24ec, 61 },
    { 0x5acd24ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d24f0, 98 },
    { 0xb76d24f1, 98 },
    { 0x5aad24f2, 32 },
    { 0xb39d24f3, 43 },
    { 0x960d24f4, 95 },
    { 0xe95d24f4, 33 },
    { 0, 0 },
    { 0x9c4d24f7, 90 },
    { 0, 0 },
    { 0x297d24f9, 33 },
    { 0x84ad24fa, 95 },
    { 0x71d24fb, 23 },
    { 0x38fd24fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d2501, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128d250d, 87 },
    { 0xfb2d250d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd2512, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd2518, 87 },
    { 0xc60d2519, 43 },
    { 0, 0 },
    { 0xd51d251b, 61 },
    { 0xea8d251c, 61 },
    { 0xd58d251c, 34 },
    { 0x3bfd251e, 90 },
    { 0xca4d251e, 95 },
    { 0xc64d2520, 61 },
    { 0x950d2521, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbd2526, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d252b, 23 },
    { 0xaf9d252c, 90 },
    { 0x6f4d252d, 90 },
    { 0x6b9d252e, 93 },
    { 0, 0 },
    { 0x81ed2530, 23 },
    { 0xb37d2531, 95 },
    { 0xb8ad2531, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4d2537, 33 },
    { 0x5d6d2537, 100 },
    { 0x333d2539, 33 },
    { 0, 0 },
    { 0xcad253b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d2543, 96 },
    { 0x5c4d2544, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329d2549, 91 },
    { 0xc1bd254a, 46 },
    { 0, 0 },
    { 0xc61d254c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xceed254f, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd2553, 38 },
    { 0xfb0d2554, 36 },
    { 0xf37d2555, 55 },
    { 0x5f4d2556, 76 },
    { 0x975d2557, 90 },
    { 0x3a3d2558, 38 },
    { 0x8e6d2555, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97d255d, 95 },
    { 0x436d255e, 93 },
    { 0, 0 },
    { 0x6b4d2560, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6d2563, 90 },
    { 0, 0 },
    { 0x21fd2565, 93 },
    { 0xa4d2566, 85 },
    { 0, 0 },
    { 0x82dd2568, 43 },
    { 0, 0 },
    { 0x3d9d256a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640d2572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d2579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x697d257c, 76 },
    { 0, 0 },
    { 0x306d257e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd2582, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fd258e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6d2593, 52 },
    { 0xf38d2593, 90 },
    { 0x1c7d2595, 68 },
    { 0x492d2596, 98 },
    { 0x128d2597, 76 },
    { 0xda6d2598, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99d259f, 90 },
    { 0x826d259f, 95 },
    { 0x183d25a1, 23 },
    { 0x48ad25a1, 76 },
    { 0xf8d25a2, 93 },
    { 0x730d25a3, 66 },
    { 0, 0 },
    { 0x491d25a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835d25aa, 84 },
    { 0, 0 },
    { 0x57d25ac, 33 },
    { 0, 0 },
    { 0x895d25ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddd25b2, 23 },
    { 0x3c7d25b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aad25c5, 83 },
    { 0x50cd25c6, 90 },
    { 0xf40d25c5, 61 },
    { 0xe28d25c8, 33 },
    { 0xbed25c8, 51 },
    { 0xcebd25c6, 61 },
    { 0x602d25cb, 95 },
    { 0xb89d25c9, 56 },
    { 0x76cd25cd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdd25d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1d25d6, 43 },
    { 0, 0 },
    { 0xab0d25d8, 90 },
    { 0x46dd25d8, 90 },
    { 0, 0 },
    { 0x3fed25db, 84 },
    { 0x673d25dc, 33 },
    { 0xdced25db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dd25e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540d25e4, 95 },
    { 0xd8ad25e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99d25e9, 95 },
    { 0, 0 },
    { 0x751d25eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d25f0, 76 },
    { 0x5ecd25f1, 90 },
    { 0x332d25f1, 23 },
    { 0xad9d25f3, 90 },
    { 0xf4ed25f2, 98 },
    { 0x51cd25f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cd25f8, 90 },
    { 0x923d25f9, 23 },
    { 0x1cfd25f8, 75 },
    { 0x180d25fb, 33 },
    { 0x230d25fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65d2601, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85d2604, 61 },
    { 0xf08d2605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5d2609, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cd2611, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383d2621, 99 },
    { 0, 0 },
    { 0x3a7d2623, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45d262b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900d262f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bd2632, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fd263f, 94 },
    { 0, 0 },
    { 0x1c9d2641, 90 },
    { 0x522d2641, 75 },
    { 0x9bcd2643, 52 },
    { 0x1b7d2644, 34 },
    { 0xa30d2645, 33 },
    { 0xf57d2646, 55 },
    { 0xfc9d2647, 95 },
    { 0xb59d2648, 66 },
    { 0x203d2649, 90 },
    { 0xd7d2642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78d2650, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd2656, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38d265b, 81 },
    { 0, 0 },
    { 0x48dd265d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x595d2660, 46 },
    { 0x677d2661, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd2664, 33 },
    { 0x492d2665, 61 },
    { 0, 0 },
    { 0xae9d2667, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedd266a, 77 },
    { 0x348d266b, 90 },
    { 0, 0 },
    { 0x8fdd266d, 90 },
    { 0x696d266d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2672, 33 },
    { 0x680d2673, 90 },
    { 0xad8d2674, 67 },
    { 0xc5fd2675, 61 },
    { 0x8d4d2675, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd267d, 33 },
    { 0xbdcd267e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecd2681, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabd2685, 85 },
    { 0xb3dd2685, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcced268b, 37 },
    { 0, 0 },
    { 0x198d268d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed2695, 33 },
    { 0xa6ed2696, 33 },
    { 0x7cdd2697, 76 },
    { 0x590d2696, 55 },
    { 0x2c4d2699, 64 },
    { 0x38d2695, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dd26a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cd26a4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33d26a7, 63 },
    { 0x683d26a8, 90 },
    { 0x447d26a9, 33 },
    { 0x238d26aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71dd26b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3d26bb, 95 },
    { 0x42ed26bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154d26c3, 91 },
    { 0x61ed26c3, 93 },
    { 0, 0 },
    { 0x500d26c6, 52 },
    { 0x688d26c7, 100 },
    { 0x300d26c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3d26cf, 34 },
    { 0, 0 },
    { 0x9ad26d1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9d26d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98d26da, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7d26e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd26e5, 88 },
    { 0, 0 },
    { 0x7dcd26e7, 23 },
    { 0, 0 },
    { 0xa75d26e9, 61 },
    { 0xb13d26ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bd26f0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303d26f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19d26fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x381d26fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333d2708, 33 },
    { 0x88bd2708, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ad270c, 87 },
    { 0xcd9d270c, 52 },
    { 0x2a8d270c, 33 },
    { 0x2e6d270f, 34 },
    { 0, 0 },
    { 0xf9d2711, 59 },
    { 0, 0 },
    { 0x77fd2713, 98 },
    { 0x147d2714, 33 },
    { 0xa7d2715, 23 },
    { 0, 0 },
    { 0xaa3d2717, 66 },
    { 0x81fd2717, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d271e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcd2722, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0d2725, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259d2729, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7d272d, 85 },
    { 0x797d272d, 98 },
    { 0, 0 },
    { 0x618d2730, 87 },
    { 0xd8ed2731, 33 },
    { 0x8e8d2732, 33 },
    { 0xcc8d2732, 95 },
    { 0, 0 },
    { 0x13fd2735, 61 },
    { 0xcd4d2736, 46 },
    { 0x6eed2736, 61 },
    { 0, 0 },
    { 0x9e4d2739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ed273c, 87 },
    { 0xd6fd273d, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fd2740, 23 },
    { 0, 0 },
    { 0x6b5d2742, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed2750, 67 },
    { 0, 0 },
    { 0x666d2752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03d275c, 55 },
    { 0xb0bd275d, 98 },
    { 0x675d275d, 93 },
    { 0x6b3d275e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8d2763, 52 },
    { 0xd36d2763, 95 },
    { 0, 0 },
    { 0xf24d2766, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28d2769, 95 },
    { 0x6b7d2769, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x319d276d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeed2775, 84 },
    { 0x466d2775, 84 },
    { 0x783d2777, 91 },
    { 0x249d2777, 66 },
    { 0x77bd2777, 100 },
    { 0xc42d2778, 90 },
    { 0xc1ad2778, 76 },
    { 0x87fd277c, 33 },
    { 0x1c6d277a, 95 },
    { 0, 0 },
    { 0x8f0d277f, 81 },
    { 0x532d2780, 61 },
    { 0, 0 },
    { 0xc4ad2782, 23 },
    { 0x4dd2783, 90 },
    { 0xba1d2784, 91 },
    { 0x6fcd2785, 43 },
    { 0xfcbd2786, 90 },
    { 0xfb0d2784, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bd278a, 84 },
    { 0x36cd278a, 95 },
    { 0, 0 },
    { 0x618d278d, 55 },
    { 0, 0 },
    { 0x698d278f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14d2793, 22 },
    { 0, 0 },
    { 0xc75d2795, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358d279b, 43 },
    { 0x362d279c, 77 },
    { 0xe28d279d, 87 },
    { 0xc22d279d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x90fd27a1, 98 },
    { 0xcb3d27a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fd27a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9d27aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91d27ad, 76 },
    { 0x118d27ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fd27b1, 33 },
    { 0x317d27b1, 23 },
    { 0xaa9d27b2, 66 },
    { 0x65bd27b4, 43 },
    { 0, 0 },
    { 0x961d27b6, 55 },
    { 0x617d27b7, 34 },
    { 0, 0 },
    { 0x179d27b9, 76 },
    { 0xb6cd27b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdd27c1, 95 },
    { 0x927d27c2, 98 },
    { 0xefd27c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d27c9, 95 },
    { 0x712d27ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x935d27cd, 90 },
    { 0xe7ad27ce, 90 },
    { 0x176d27cf, 61 },
    { 0x6e5d27cd, 33 },
    { 0xd20d27ce, 95 },
    { 0x96cd27cf, 90 },
    { 0, 0 },
    { 0x611d27d4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49d27d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6d27df, 100 },
    { 0, 0 },
    { 0x5bad27e1, 46 },
    { 0, 0 },
    { 0x540d27e3, 98 },
    { 0xa87d27e3, 61 },
    { 0xf94d27e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x443d27e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97d27ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431d27f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1d27f4, 43 },
    { 0xa69d27f4, 66 },
    { 0xad3d27f6, 55 },
    { 0xdd5d27f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d27fb, 61 },
    { 0, 0 },
    { 0xb38d27fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6d2800, 90 },
    { 0xddd2801, 90 },
    { 0xe6d2801, 33 },
    { 0, 0 },
    { 0xd4d2804, 98 },
    { 0xc51d2805, 23 },
    { 0x165d2806, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x744d2809, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbd280f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebd2817, 34 },
    { 0xc32d2818, 95 },
    { 0x6c6d2819, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38d281c, 34 },
    { 0, 0 },
    { 0x916d281e, 46 },
    { 0xa69d281e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd2823, 98 },
    { 0xda7d2823, 77 },
    { 0, 0 },
    { 0x756d2826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eed282c, 40 },
    { 0, 0 },
    { 0x5ffd282e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cd2832, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ed2835, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2d2839, 61 },
    { 0xaead283a, 65 },
    { 0, 0 },
    { 0x613d283c, 52 },
    { 0, 0 },
    { 0xd8d283e, 91 },
    { 0x9b2d283f, 33 },
    { 0xa68d2840, 95 },
    { 0x42ad2841, 33 },
    { 0x4fad283e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986d2846, 98 },
    { 0x4e8d2847, 78 },
    { 0xc54d2847, 55 },
    { 0x852d2849, 66 },
    { 0x368d284a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed284e, 33 },
    { 0xa44d284f, 93 },
    { 0xbbd284e, 90 },
    { 0x5dbd2850, 23 },
    { 0x6aad2852, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d2856, 52 },
    { 0x600d2856, 93 },
    { 0x910d2856, 98 },
    { 0x47ad2859, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd2865, 87 },
    { 0xdead2865, 90 },
    { 0x23cd2867, 23 },
    { 0xba9d2868, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331d286f, 98 },
    { 0, 0 },
    { 0x65ed2871, 78 },
    { 0xe09d2872, 66 },
    { 0xbad2872, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778d287f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x521d2882, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x444d2885, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1d288a, 60 },
    { 0xd81d288a, 61 },
    { 0xea8d288c, 33 },
    { 0, 0 },
    { 0x361d288e, 55 },
    { 0xb0fd288f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ed2893, 95 },
    { 0, 0 },
    { 0xfa9d2895, 23 },
    { 0x7bcd2896, 91 },
    { 0, 0 },
    { 0xf77d2898, 87 },
    { 0, 0 },
    { 0x742d289a, 95 },
    { 0x1bfd289b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386d289f, 43 },
    { 0xd6fd28a0, 98 },
    { 0, 0 },
    { 0x851d28a2, 55 },
    { 0x573d28a3, 98 },
    { 0xf87d28a4, 91 },
    { 0, 0 },
    { 0x723d28a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6d28b0, 91 },
    { 0, 0 },
    { 0xce1d28b2, 68 },
    { 0x97bd28b3, 90 },
    { 0xe97d28b4, 93 },
    { 0xf01d28b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51d28bb, 52 },
    { 0x2a4d28bc, 61 },
    { 0, 0 },
    { 0x4a0d28be, 77 },
    { 0, 0 },
    { 0xfc1d28c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969d28c5, 95 },
    { 0xa33d28c5, 55 },
    { 0xbcad28c7, 33 },
    { 0xf7dd28c8, 95 },
    { 0x6dbd28c8, 23 },
    { 0x763d28ca, 78 },
    { 0x19ed28c9, 55 },
    { 0xa88d28cc, 55 },
    { 0x1cdd28cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x434d28d0, 33 },
    { 0, 0 },
    { 0x7f1d28d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d28d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccd28da, 87 },
    { 0, 0 },
    { 0x6dfd28dc, 55 },
    { 0xd2ed28dc, 63 },
    { 0, 0 },
    { 0x72ed28df, 84 },
    { 0xcf6d28e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725d28e5, 61 },
    { 0, 0 },
    { 0xd25d28e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d28ef, 90 },
    { 0xd6ad28f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dd28f3, 95 },
    { 0xda5d28f4, 93 },
    { 0xe3ad28f4, 43 },
    { 0, 0 },
    { 0x386d28f7, 33 },
    { 0xf00d28f8, 95 },
    { 0x966d28f9, 91 },
    { 0xb86d28f9, 61 },
    { 0x9a3d28f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed28fe, 36 },
    { 0, 0 },
    { 0xd55d2900, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x297d2903, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbd2906, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd2909, 98 },
    { 0x722d290a, 90 },
    { 0x5ecd2909, 91 },
    { 0x27ed290c, 61 },
    { 0x48dd290d, 95 },
    { 0x510d290c, 93 },
    { 0, 0 },
    { 0x8d2910, 55 },
    { 0xdefd2911, 33 },
    { 0x71cd2912, 23 },
    { 0, 0 },
    { 0x3fbd2914, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ed2919, 78 },
    { 0x477d2919, 99 },
    { 0x61ad2919, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63d2923, 67 },
    { 0xbc5d2924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d2927, 61 },
    { 0xef0d2928, 66 },
    { 0x152d2929, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbd292c, 23 },
    { 0, 0 },
    { 0x43ad292e, 46 },
    { 0x863d292f, 95 },
    { 0x2a9d2930, 90 },
    { 0x243d2931, 64 },
    { 0x8a1d2932, 94 },
    { 0x106d2932, 76 },
    { 0xdb1d292e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9d2937, 75 },
    { 0xd56d2938, 43 },
    { 0x32d2939, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x203d293c, 23 },
    { 0x563d293d, 33 },
    { 0x728d293e, 61 },
    { 0x18cd293e, 61 },
    { 0x2d3d293c, 23 },
    { 0x358d2941, 33 },
    { 0x8c6d2942, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6d2947, 46 },
    { 0x1d4d2947, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad2952, 91 },
    { 0, 0 },
    { 0x282d2954, 66 },
    { 0, 0 },
    { 0x915d2956, 61 },
    { 0, 0 },
    { 0xebed2958, 90 },
    { 0x405d2958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314d295e, 95 },
    { 0x361d295e, 78 },
    { 0, 0 },
    { 0x2fad2961, 93 },
    { 0xccad2962, 90 },
    { 0xd3fd2963, 61 },
    { 0, 0 },
    { 0xa22d2965, 55 },
    { 0, 0 },
    { 0xdcad2967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d296a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd296d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bed2971, 23 },
    { 0x5f5d2971, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ed2976, 76 },
    { 0xb93d2976, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618d297d, 93 },
    { 0xb03d297e, 98 },
    { 0, 0 },
    { 0x4c4d2980, 33 },
    { 0, 0 },
    { 0x20bd2982, 53 },
    { 0x9bed2983, 34 },
    { 0x719d2984, 61 },
    { 0xb6ed2985, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2d298a, 23 },
    { 0xa58d298b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852d2990, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1dd2994, 52 },
    { 0, 0 },
    { 0x83dd2996, 95 },
    { 0x5e6d2996, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cd299d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x371d29a6, 33 },
    { 0xd59d29a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88d29aa, 52 },
    { 0x8e8d29ab, 100 },
    { 0x56ad29aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd29b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed29b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d29b8, 83 },
    { 0, 0 },
    { 0x500d29ba, 76 },
    { 0, 0 },
    { 0x8a6d29bc, 90 },
    { 0x643d29bd, 83 },
    { 0xcd6d29be, 67 },
    { 0xeabd29bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d29c2, 98 },
    { 0xdbdd29c3, 33 },
    { 0x124d29c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fd29c7, 33 },
    { 0x917d29c8, 85 },
    { 0x35bd29c7, 98 },
    { 0xbb9d29c8, 46 },
    { 0, 0 },
    { 0xd1fd29cc, 55 },
    { 0xd84d29cc, 33 },
    { 0x29cd29ce, 23 },
    { 0x699d29cc, 55 },
    { 0x848d29ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d29d3, 98 },
    { 0x2b9d29d4, 90 },
    { 0xaead29d3, 93 },
    { 0xd65d29d4, 90 },
    { 0x5dd29d7, 81 },
    { 0, 0 },
    { 0x951d29d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fed29e0, 52 },
    { 0x708d29e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d29e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadd29e9, 23 },
    { 0xb7cd29e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0d29f3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d29f8, 23 },
    { 0, 0 },
    { 0x175d29fa, 85 },
    { 0xe29d29fb, 100 },
    { 0xa13d29fc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49d2a00, 95 },
    { 0x15d2a01, 78 },
    { 0x8f6d2a01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58d2a06, 93 },
    { 0xcf0d2a06, 55 },
    { 0, 0 },
    { 0x4b3d2a09, 84 },
    { 0xb65d2a09, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ad2a0f, 61 },
    { 0x127d2a10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9d2a14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2d2a17, 96 },
    { 0, 0 },
    { 0xc6ed2a19, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d2a22, 23 },
    { 0, 0 },
    { 0x8e0d2a24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d2a2b, 38 },
    { 0x91ad2a2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614d2a30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1d2a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x364d2a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11d2a3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277d2a43, 95 },
    { 0xdc5d2a44, 66 },
    { 0xd87d2a44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddd2a48, 61 },
    { 0x42dd2a49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ed2a4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebd2a55, 90 },
    { 0, 0 },
    { 0xd3bd2a57, 90 },
    { 0xc35d2a58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6d2a60, 80 },
    { 0, 0 },
    { 0x5dad2a62, 61 },
    { 0xbf3d2a62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cd2a67, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66d2a6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92d2a6f, 76 },
    { 0x88ad2a70, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bd2a74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ad2a7f, 43 },
    { 0, 0 },
    { 0x323d2a81, 95 },
    { 0x632d2a82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6d2a85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19d2a8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750d2a8f, 93 },
    { 0xfacd2a8f, 95 },
    { 0, 0 },
    { 0x65fd2a92, 34 },
    { 0x4f0d2a93, 23 },
    { 0x4efd2a93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cd2a9e, 90 },
    { 0, 0 },
    { 0xad4d2aa0, 66 },
    { 0x6a5d2aa1, 46 },
    { 0, 0 },
    { 0xd40d2aa3, 78 },
    { 0xecbd2aa3, 55 },
    { 0xcd3d2aa5, 33 },
    { 0x33d2aa3, 98 },
    { 0x72ad2aa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89d2aad, 33 },
    { 0x995d2aad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9d2ab1, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7add2ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82d2aba, 61 },
    { 0x59fd2aba, 23 },
    { 0x437d2abc, 34 },
    { 0xd58d2abd, 33 },
    { 0x536d2abe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bd2ac2, 84 },
    { 0x845d2ac3, 77 },
    { 0xc1bd2ac3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cd2ac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd2aca, 76 },
    { 0, 0 },
    { 0xfbfd2acc, 90 },
    { 0xc96d2acd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5d2ad4, 61 },
    { 0xa70d2ad4, 95 },
    { 0xb83d2ad4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1d2ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dd2ae7, 93 },
    { 0x79dd2ae8, 90 },
    { 0xc36d2ae8, 90 },
    { 0x982d2aea, 98 },
    { 0, 0 },
    { 0x399d2aec, 17 },
    { 0x594d2aed, 90 },
    { 0x19ad2aee, 90 },
    { 0, 0 },
    { 0x87cd2af0, 99 },
    { 0xd04d2af1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d2af4, 34 },
    { 0x599d2af5, 95 },
    { 0x68bd2af6, 76 },
    { 0xb93d2af5, 97 },
    { 0xb8d2af4, 67 },
    { 0, 0 },
    { 0xcf8d2afa, 61 },
    { 0, 0 },
    { 0x942d2afc, 93 },
    { 0x7f6d2afd, 19 },
    { 0xd5bd2afc, 95 },
    { 0xb9dd2aff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d2b07, 95 },
    { 0xd42d2b08, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d2b0d, 93 },
    { 0xdabd2b0d, 55 },
    { 0xab2d2b0f, 43 },
    { 0, 0 },
    { 0x99ed2b11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x510d2b14, 90 },
    { 0x2b7d2b15, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafd2b19, 52 },
    { 0x1d1d2b19, 95 },
    { 0xb57d2b1b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded2b1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ed2b22, 43 },
    { 0x1e3d2b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4d2b28, 98 },
    { 0x7afd2b29, 90 },
    { 0x569d2b29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ed2b2d, 96 },
    { 0x2bcd2b2e, 23 },
    { 0xe8d2b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bd2b32, 98 },
    { 0x994d2b33, 61 },
    { 0, 0 },
    { 0x33d2b35, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d2b3a, 43 },
    { 0x38ad2b3b, 34 },
    { 0x31dd2b3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ed2b40, 77 },
    { 0xc58d2b40, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7d2b45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270d2b4b, 99 },
    { 0xefcd2b4c, 90 },
    { 0x751d2b4d, 61 },
    { 0x33ed2b4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147d2b53, 90 },
    { 0xa50d2b53, 77 },
    { 0, 0 },
    { 0x809d2b56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77d2b5b, 90 },
    { 0xe66d2b5c, 61 },
    { 0xbb5d2b5c, 61 },
    { 0x383d2b5e, 43 },
    { 0, 0 },
    { 0xfe9d2b60, 33 },
    { 0, 0 },
    { 0x975d2b62, 23 },
    { 0x68ed2b63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aed2b66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142d2b6b, 61 },
    { 0xfafd2b6c, 23 },
    { 0xcffd2b6b, 61 },
    { 0x610d2b6d, 90 },
    { 0xd2ed2b6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4d2b74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545d2b79, 53 },
    { 0x9d8d2b7a, 23 },
    { 0xc2fd2b7b, 90 },
    { 0, 0 },
    { 0x5f5d2b7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153d2b82, 80 },
    { 0x28bd2b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bd2b87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bd2b8d, 55 },
    { 0xafbd2b8e, 61 },
    { 0xaaed2b8f, 95 },
    { 0x33fd2b8f, 61 },
    { 0xcefd2b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aad2b95, 90 },
    { 0x2e2d2b95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d2b9a, 33 },
    { 0x1ced2b9a, 33 },
    { 0xa52d2b9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11d2ba0, 95 },
    { 0x185d2ba1, 85 },
    { 0x5c7d2ba2, 91 },
    { 0x858d2ba3, 98 },
    { 0x6f9d2ba3, 61 },
    { 0x5b7d2ba5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x855d2ba8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ed2bac, 33 },
    { 0x9dad2bad, 67 },
    { 0xf8cd2bae, 95 },
    { 0xc7bd2bad, 55 },
    { 0, 0 },
    { 0x5fbd2bb1, 69 },
    { 0xe4cd2bb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483d2bb7, 23 },
    { 0x7e7d2bb8, 93 },
    { 0x3dd2bb8, 55 },
    { 0xe6ad2bb9, 95 },
    { 0, 0 },
    { 0x245d2bbc, 69 },
    { 0xd2ad2bbd, 23 },
    { 0x73bd2bbe, 81 },
    { 0, 0 },
    { 0xd9ed2bc0, 38 },
    { 0x767d2bc1, 90 },
    { 0x176d2bc2, 77 },
    { 0x36ad2bc1, 85 },
    { 0x3b8d2bc3, 56 },
    { 0x223d2bc3, 90 },
    { 0xd6cd2bc6, 33 },
    { 0x2efd2bc2, 61 },
    { 0x72d2bc3, 75 },
    { 0xb35d2bc9, 78 },
    { 0x10cd2bca, 61 },
    { 0, 0 },
    { 0x83ed2bcc, 93 },
    { 0xd8fd2bcd, 55 },
    { 0x6a6d2bce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99d2bd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9d2bd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d2bdf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddd2beb, 33 },
    { 0x1efd2beb, 80 },
    { 0xe39d2bed, 55 },
    { 0xc6ed2bee, 65 },
    { 0xd25d2bef, 91 },
    { 0x79fd2bef, 100 },
    { 0xcfcd2bf1, 64 },
    { 0x2eed2beb, 90 },
    { 0, 0 },
    { 0x6b4d2bf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41d2bfa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24d2bff, 33 },
    { 0x614d2c00, 43 },
    { 0x5f0d2c01, 76 },
    { 0xa8ed2c02, 76 },
    { 0xc09d2c02, 61 },
    { 0x72ed2c04, 34 },
    { 0x289d2c05, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3d2c08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1d2c0c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d2c12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd2c18, 84 },
    { 0, 0 },
    { 0x3eed2c1a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d2c1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d2c26, 95 },
    { 0x72fd2c27, 34 },
    { 0x20cd2c28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d2c31, 77 },
    { 0, 0 },
    { 0xa9fd2c33, 61 },
    { 0x15ed2c33, 90 },
    { 0, 0 },
    { 0x65bd2c36, 95 },
    { 0x4a7d2c37, 75 },
    { 0x649d2c38, 90 },
    { 0x194d2c39, 55 },
    { 0x246d2c3a, 65 },
    { 0xb31d2c3b, 67 },
    { 0xbebd2c36, 61 },
    { 0, 0 },
    { 0x354d2c3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aad2c42, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd2c49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1d2c4c, 90 },
    { 0x19cd2c4d, 46 },
    { 0x64ed2c4e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0d2c53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9d2c57, 95 },
    { 0xe6ad2c58, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8d2c5b, 90 },
    { 0xd01d2c5c, 61 },
    { 0xd20d2c5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2d2c61, 33 },
    { 0x3c2d2c62, 34 },
    { 0xfebd2c63, 61 },
    { 0, 0 },
    { 0x76fd2c65, 56 },
    { 0xe36d2c65, 98 },
    { 0xa22d2c67, 33 },
    { 0, 0 },
    { 0x54ed2c69, 100 },
    { 0xe54d2c6a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edd2c76, 98 },
    { 0xdd6d2c76, 33 },
    { 0xfbed2c76, 23 },
    { 0x20d2c78, 90 },
    { 0, 0 },
    { 0xa77d2c7b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08d2c7f, 61 },
    { 0xcf3d2c80, 87 },
    { 0x3b4d2c81, 77 },
    { 0, 0 },
    { 0xf9bd2c83, 34 },
    { 0x70ed2c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbd2c88, 61 },
    { 0, 0 },
    { 0xba3d2c8a, 76 },
    { 0x606d2c8b, 33 },
    { 0, 0 },
    { 0x115d2c8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9d2c90, 98 },
    { 0x4bfd2c90, 90 },
    { 0x42fd2c92, 61 },
    { 0xfe9d2c93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d2c96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbd2ca5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9d2caa, 33 },
    { 0, 0 },
    { 0xf1dd2cac, 78 },
    { 0xaefd2cad, 61 },
    { 0xdccd2cad, 55 },
    { 0x271d2cac, 61 },
    { 0, 0 },
    { 0xc52d2cb1, 55 },
    { 0xc29d2cb2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9d2cb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8d2cbb, 91 },
    { 0xd99d2cbc, 90 },
    { 0x17d2cbd, 84 },
    { 0, 0 },
    { 0x7d2d2cbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bd2cc3, 46 },
    { 0, 0 },
    { 0xa50d2cc5, 75 },
    { 0, 0 },
    { 0x41dd2cc7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x622d2cca, 66 },
    { 0x8afd2cca, 95 },
    { 0xf61d2ccc, 95 },
    { 0x737d2ccc, 61 },
    { 0, 0 },
    { 0x414d2ccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbed2cd3, 98 },
    { 0x852d2cd4, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737d2ce0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcd2ce3, 78 },
    { 0x894d2ce4, 91 },
    { 0x41ad2ce5, 55 },
    { 0xdd1d2ce6, 91 },
    { 0, 0 },
    { 0x633d2ce8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d2cec, 93 },
    { 0x86ed2ced, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2d2cf1, 33 },
    { 0, 0 },
    { 0x994d2cf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7d2cf6, 85 },
    { 0xbf3d2cf7, 78 },
    { 0x86cd2cf7, 55 },
    { 0xff2d2cf9, 23 },
    { 0x3d6d2cfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8d2d01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83d2d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50d2d0d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432d2d14, 66 },
    { 0x55ad2d15, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82d2d18, 33 },
    { 0xb2ad2d19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96d2d1f, 87 },
    { 0x1e7d2d20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ad2d24, 33 },
    { 0x485d2d24, 61 },
    { 0, 0 },
    { 0xced2d27, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dd2d2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebd2d30, 33 },
    { 0, 0 },
    { 0xcb2d2d32, 76 },
    { 0xabed2d33, 91 },
    { 0x717d2d34, 100 },
    { 0x9e1d2d34, 90 },
    { 0x3b1d2d34, 84 },
    { 0xfefd2d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2d2d3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bd2d45, 61 },
    { 0x298d2d46, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd2d4c, 95 },
    { 0xe8d2d4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678d2d53, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d2d58, 95 },
    { 0x2bd2d59, 77 },
    { 0, 0 },
    { 0x7e3d2d5b, 90 },
    { 0, 0 },
    { 0x27cd2d5d, 93 },
    { 0x82bd2d5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93d2d61, 77 },
    { 0x785d2d61, 95 },
    { 0x738d2d63, 43 },
    { 0x677d2d61, 66 },
    { 0x1f7d2d65, 43 },
    { 0xc2fd2d66, 98 },
    { 0xcf2d2d67, 90 },
    { 0x77cd2d68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd2d6b, 33 },
    { 0, 0 },
    { 0x34ed2d6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616d2d76, 93 },
    { 0x9b5d2d77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7d2d7a, 33 },
    { 0x429d2d7b, 76 },
    { 0xfd3d2d7c, 34 },
    { 0xfb6d2d7c, 90 },
    { 0xb43d2d7a, 45 },
    { 0xd45d2d7e, 95 },
    { 0x450d2d7b, 33 },
    { 0xe2cd2d7b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78d2d84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326d2d89, 2 },
    { 0, 0 },
    { 0xaa5d2d8b, 23 },
    { 0, 0 },
    { 0x11fd2d8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d2d92, 66 },
    { 0, 0 },
    { 0xe72d2d94, 98 },
    { 0xaced2d95, 61 },
    { 0x902d2d96, 61 },
    { 0xa04d2d94, 55 },
    { 0x1efd2d98, 76 },
    { 0xf3bd2d98, 87 },
    { 0xa0d2d99, 23 },
    { 0xcfbd2d9b, 33 },
    { 0x5a5d2d9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed2da0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ed2da3, 55 },
    { 0x9b3d2da4, 33 },
    { 0xc62d2da5, 55 },
    { 0x7d7d2da6, 90 },
    { 0xd49d2da7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x621d2daa, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11d2db5, 55 },
    { 0xe0bd2db5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d2dba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686d2dbf, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4d2dc2, 23 },
    { 0x36bd2dc2, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2d2dc7, 33 },
    { 0, 0 },
    { 0x418d2dc9, 77 },
    { 0x2bad2dc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x589d2dcd, 87 },
    { 0x65fd2dce, 90 },
    { 0xd5d2dcd, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1d2dd2, 76 },
    { 0, 0 },
    { 0xdcad2dd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0d2dda, 90 },
    { 0xc3d2ddb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd2ddf, 96 },
    { 0x144d2ddf, 90 },
    { 0xe84d2de1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32d2de7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfd2deb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571d2df0, 23 },
    { 0xe3fd2df1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5d2df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ed2df9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50d2e00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dd2e04, 61 },
    { 0xd1bd2e05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedd2e0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ad2e11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6d2e14, 61 },
    { 0x37ed2e15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647d2e19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569d2e1d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd2e24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d2e2c, 61 },
    { 0, 0 },
    { 0xa2d2e2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2e31, 33 },
    { 0, 0 },
    { 0x9a6d2e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39d2e3a, 67 },
    { 0xe54d2e3b, 95 },
    { 0x918d2e3c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd2e40, 95 },
    { 0x82ad2e41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2d2e46, 95 },
    { 0x237d2e47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d2e4a, 34 },
    { 0xbf9d2e4a, 75 },
    { 0, 0 },
    { 0x928d2e4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6d2e57, 77 },
    { 0, 0 },
    { 0xc88d2e59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x683d2e5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffd2e64, 90 },
    { 0x5d2e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bd2e6b, 33 },
    { 0xbd8d2e6c, 98 },
    { 0, 0 },
    { 0xe2dd2e6e, 53 },
    { 0xfbd2e6f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x207d2e72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0d2e78, 99 },
    { 0x3c7d2e78, 55 },
    { 0x8d3d2e7a, 33 },
    { 0x97ed2e7b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d2e80, 61 },
    { 0, 0 },
    { 0x6bd2e82, 90 },
    { 0xefdd2e83, 78 },
    { 0x772d2e84, 98 },
    { 0, 0 },
    { 0x37ad2e86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0d2e8a, 98 },
    { 0x4e6d2e8b, 95 },
    { 0xf1ad2e8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d2e90, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cd2e93, 95 },
    { 0, 0 },
    { 0x280d2e95, 97 },
    { 0, 0 },
    { 0xb99d2e97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4d2e9b, 98 },
    { 0x7aed2e9b, 90 },
    { 0xde5d2e9d, 33 },
    { 0x461d2e9b, 40 },
    { 0, 0 },
    { 0xde5d2ea0, 95 },
    { 0x9d9d2ea0, 95 },
    { 0, 0 },
    { 0x7a1d2ea3, 98 },
    { 0x187d2ea4, 90 },
    { 0xad2ea3, 76 },
    { 0x943d2ea6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd2eaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4d2ead, 46 },
    { 0x431d2ead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443d2ebc, 84 },
    { 0xfcd2ebd, 39 },
    { 0xbabd2ebe, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bd2ec4, 46 },
    { 0x2a0d2ec5, 90 },
    { 0x6dcd2ec6, 93 },
    { 0, 0 },
    { 0xe6fd2ec8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x658d2ecb, 40 },
    { 0x40d2ecc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9d2ed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74d2ed4, 84 },
    { 0x69cd2ed5, 90 },
    { 0xb18d2ed6, 34 },
    { 0x369d2ed7, 87 },
    { 0x151d2ed5, 99 },
    { 0, 0 },
    { 0x24d2eda, 90 },
    { 0, 0 },
    { 0x4b5d2edc, 66 },
    { 0, 0 },
    { 0x293d2ede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105d2ee6, 52 },
    { 0xb85d2ee7, 43 },
    { 0x1e4d2ee8, 55 },
    { 0x4e2d2ee7, 90 },
    { 0x308d2ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59d2ef4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bd2efa, 33 },
    { 0x432d2efb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddd2efe, 90 },
    { 0xfbd2efe, 61 },
    { 0xadad2f00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x353d2f10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456d2f14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dd2f17, 55 },
    { 0x5f6d2f18, 100 },
    { 0x437d2f19, 77 },
    { 0x8aed2f18, 34 },
    { 0xf84d2f1b, 90 },
    { 0x8bed2f18, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914d2f25, 61 },
    { 0xa98d2f26, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cd2f2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ed2f30, 55 },
    { 0x8f8d2f30, 34 },
    { 0xc0cd2f30, 55 },
    { 0xfcad2f33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6d2f36, 99 },
    { 0x51d2f36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d2f3d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7d2f41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154d2f47, 61 },
    { 0x40d2f48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ed2f4f, 90 },
    { 0xd71d2f4f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x621d2f53, 88 },
    { 0x81dd2f54, 76 },
    { 0x987d2f53, 64 },
    { 0x163d2f56, 76 },
    { 0xab6d2f57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d2f5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfd2f5f, 95 },
    { 0x6b3d2f60, 90 },
    { 0xb84d2f60, 98 },
    { 0x827d2f62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fd2f69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9d2f6c, 61 },
    { 0x81d2f6d, 67 },
    { 0xdafd2f6d, 90 },
    { 0xcf1d2f6f, 61 },
    { 0x7f8d2f70, 98 },
    { 0x3f1d2f71, 90 },
    { 0x9c3d2f70, 76 },
    { 0x822d2f73, 61 },
    { 0x825d2f73, 76 },
    { 0, 0 },
    { 0xda1d2f76, 77 },
    { 0x739d2f76, 55 },
    { 0xa8d2f78, 95 },
    { 0x485d2f78, 61 },
    { 0x213d2f7a, 33 },
    { 0x90cd2f7b, 91 },
    { 0xaf5d2f7a, 90 },
    { 0x7dbd2f7d, 98 },
    { 0x4bdd2f7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d2f82, 99 },
    { 0, 0 },
    { 0x40dd2f84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbd2f87, 95 },
    { 0x3cad2f88, 61 },
    { 0x462d2f89, 99 },
    { 0, 0 },
    { 0xf87d2f8b, 33 },
    { 0xd21d2f8b, 33 },
    { 0xf8dd2f8d, 98 },
    { 0x53fd2f8e, 33 },
    { 0, 0 },
    { 0x9b5d2f90, 93 },
    { 0x6e1d2f91, 33 },
    { 0xe45d2f92, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bd2f97, 61 },
    { 0, 0 },
    { 0x9d1d2f99, 46 },
    { 0xa8fd2f9a, 33 },
    { 0xec6d2f9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dd2fa0, 90 },
    { 0x571d2fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dd2fa6, 61 },
    { 0x6e2d2fa7, 90 },
    { 0x306d2fa7, 91 },
    { 0, 0 },
    { 0xa17d2faa, 66 },
    { 0xcd6d2faa, 43 },
    { 0x822d2fac, 61 },
    { 0xa27d2fad, 95 },
    { 0xbed2fae, 33 },
    { 0xd82d2faf, 98 },
    { 0x533d2fae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2d2fbb, 33 },
    { 0x2bbd2fbc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8d2fc2, 90 },
    { 0, 0 },
    { 0x702d2fc4, 33 },
    { 0xdcd2fc4, 90 },
    { 0x110d2fc5, 55 },
    { 0xdbad2fc7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x704d2fcf, 23 },
    { 0, 0 },
    { 0x1a0d2fd1, 55 },
    { 0xd22d2fd1, 61 },
    { 0, 0 },
    { 0xeed2fd4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9d2fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1d2fe0, 88 },
    { 0, 0 },
    { 0x942d2fe2, 78 },
    { 0, 0 },
    { 0xc32d2fe4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6d2ff1, 75 },
    { 0xa4cd2ff2, 61 },
    { 0xd69d2ff1, 84 },
    { 0x1b0d2ff4, 95 },
    { 0x14cd2ff1, 33 },
    { 0x874d2ff6, 61 },
    { 0xe8bd2ff7, 43 },
    { 0xc09d2ff4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d2ffb, 61 },
    { 0x5e0d2ffb, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd2fff, 66 },
    { 0, 0 },
    { 0x5dcd3001, 85 },
    { 0xeb8d3001, 85 },
    { 0, 0 },
    { 0x12cd3004, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bd3008, 98 },
    { 0, 0 },
    { 0x2f9d300a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7add300e, 95 },
    { 0xc85d300f, 99 },
    { 0x731d3010, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ad3014, 90 },
    { 0x81dd3014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836d3019, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904d301f, 90 },
    { 0, 0 },
    { 0x85ad3021, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04d3024, 76 },
    { 0, 0 },
    { 0x9a9d3026, 98 },
    { 0x2cbd3027, 98 },
    { 0, 0 },
    { 0x7efd3029, 81 },
    { 0xb53d302a, 95 },
    { 0xccdd302a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75d302f, 55 },
    { 0, 0 },
    { 0x821d3031, 23 },
    { 0, 0 },
    { 0x5e5d3033, 33 },
    { 0xc88d3034, 91 },
    { 0x6c5d3035, 53 },
    { 0xe04d3035, 75 },
    { 0xfded3034, 100 },
    { 0xb10d3038, 76 },
    { 0x7dad3038, 43 },
    { 0x1e0d303a, 34 },
    { 0x1bd3035, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefd303f, 46 },
    { 0, 0 },
    { 0xd47d3041, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cd3044, 76 },
    { 0, 0 },
    { 0x5d2d3046, 99 },
    { 0, 0 },
    { 0x7d7d3048, 43 },
    { 0, 0 },
    { 0x858d304a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bd3050, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d3053, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36d3056, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43d3059, 98 },
    { 0x23cd305a, 75 },
    { 0, 0 },
    { 0x7c2d305c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd305f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466d3063, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8d3066, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2d306b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d306e, 81 },
    { 0x7a9d306e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0d3073, 95 },
    { 0x61ad3074, 65 },
    { 0x346d3075, 99 },
    { 0, 0 },
    { 0xa5ad3077, 76 },
    { 0xb41d3078, 76 },
    { 0x199d3079, 33 },
    { 0x4fad3079, 33 },
    { 0x9d7d307b, 76 },
    { 0xfa9d3079, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fd307f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7d3084, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730d3089, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd3090, 61 },
    { 0, 0 },
    { 0xc8bd3092, 93 },
    { 0x6a9d3093, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5d3096, 91 },
    { 0xfe1d3097, 23 },
    { 0x2a5d3098, 23 },
    { 0x46fd3097, 23 },
    { 0xa12d3098, 76 },
    { 0xb8bd309a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65d30a1, 34 },
    { 0x908d30a2, 76 },
    { 0, 0 },
    { 0x640d30a4, 33 },
    { 0xc74d30a5, 69 },
    { 0x4d8d30a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487d30ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853d30b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985d30bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cd30c1, 61 },
    { 0xa12d30c1, 93 },
    { 0x5d2d30c2, 98 },
    { 0x94dd30c4, 98 },
    { 0, 0 },
    { 0x14dd30c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d30cd, 36 },
    { 0xc65d30cd, 90 },
    { 0x890d30cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd30d2, 90 },
    { 0, 0 },
    { 0x78ed30d4, 40 },
    { 0, 0 },
    { 0x50bd30d6, 33 },
    { 0x156d30d7, 23 },
    { 0, 0 },
    { 0x6a1d30d9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fd30dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976d30e6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d30eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406d30ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d30f6, 87 },
    { 0x776d30f7, 52 },
    { 0xb66d30f8, 84 },
    { 0, 0 },
    { 0x368d30fa, 90 },
    { 0, 0 },
    { 0x841d30fc, 78 },
    { 0xbc1d30fd, 76 },
    { 0x4e9d30fd, 61 },
    { 0x3ccd30fc, 19 },
    { 0x8a6d3100, 23 },
    { 0xd4dd3101, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfd3104, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80d3107, 61 },
    { 0x795d3108, 76 },
    { 0x461d3108, 24 },
    { 0x8f5d310a, 51 },
    { 0xe72d310b, 46 },
    { 0x96fd310c, 95 },
    { 0xac8d310c, 76 },
    { 0x438d310a, 33 },
    { 0x6c7d310d, 69 },
    { 0xcdcd3110, 34 },
    { 0, 0 },
    { 0x809d3112, 33 },
    { 0x48fd3113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6d311a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d3123, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472d312a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdd3130, 61 },
    { 0, 0 },
    { 0xbefd3132, 61 },
    { 0x70cd3133, 98 },
    { 0, 0 },
    { 0x5abd3135, 91 },
    { 0x8edd3135, 90 },
    { 0x442d3137, 67 },
    { 0x114d3138, 33 },
    { 0xf15d3139, 76 },
    { 0xcfed3139, 34 },
    { 0x8edd313b, 98 },
    { 0xd2cd3137, 34 },
    { 0x4a6d313d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ad3142, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ed3145, 43 },
    { 0xc58d3146, 95 },
    { 0x4bed3146, 82 },
    { 0x963d3146, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81d314c, 23 },
    { 0, 0 },
    { 0x5fad314e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30d3151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbd3154, 69 },
    { 0x62ad3155, 46 },
    { 0xcbed3154, 43 },
    { 0, 0 },
    { 0xeb1d3158, 33 },
    { 0x6c7d3159, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x668d315c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ad3162, 90 },
    { 0, 0 },
    { 0xed3164, 33 },
    { 0xa72d3165, 90 },
    { 0xa7d3165, 95 },
    { 0x7d3166, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d316d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87d3171, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61d3174, 98 },
    { 0, 0 },
    { 0x528d3176, 36 },
    { 0x925d3177, 61 },
    { 0x608d3178, 61 },
    { 0, 0 },
    { 0x61fd317a, 90 },
    { 0x4e8d317b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0d317e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d3183, 52 },
    { 0, 0 },
    { 0x528d3185, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ded318b, 33 },
    { 0x7c9d318c, 93 },
    { 0x580d318c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9d3194, 55 },
    { 0, 0 },
    { 0xce4d3196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d319c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad31a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25d31a9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68d31ac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd31b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd31b5, 66 },
    { 0x44ad31b5, 95 },
    { 0xeb9d31b6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ed31ba, 23 },
    { 0xacdd31bb, 33 },
    { 0x978d31ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4d31c0, 46 },
    { 0, 0 },
    { 0x174d31c2, 40 },
    { 0x749d31c2, 91 },
    { 0x8cdd31c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7d31ce, 95 },
    { 0xda0d31cf, 52 },
    { 0xa0ad31d0, 23 },
    { 0xea7d31d1, 34 },
    { 0x89d31d2, 88 },
    { 0xee9d31d1, 23 },
    { 0xc34d31d1, 34 },
    { 0xd28d31d5, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdd31dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687d31e1, 33 },
    { 0xef9d31e2, 33 },
    { 0, 0 },
    { 0x69fd31e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfd31e9, 23 },
    { 0xd07d31ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2d31ef, 33 },
    { 0, 0 },
    { 0x721d31f1, 77 },
    { 0x2b0d31f2, 93 },
    { 0, 0 },
    { 0x3b1d31f4, 61 },
    { 0, 0 },
    { 0x3b5d31f6, 33 },
    { 0, 0 },
    { 0x5ebd31f8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ad31fb, 33 },
    { 0x4f7d31fc, 23 },
    { 0x657d31fd, 87 },
    { 0x438d31fe, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd3201, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad3204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6d320b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16d320e, 34 },
    { 0x747d320f, 61 },
    { 0x6acd3210, 23 },
    { 0x8a8d3211, 33 },
    { 0xa90d3212, 95 },
    { 0x7b7d3213, 76 },
    { 0x7bcd3214, 76 },
    { 0, 0 },
    { 0xf6fd3216, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fd3219, 95 },
    { 0, 0 },
    { 0xdf6d321b, 61 },
    { 0xff7d321c, 90 },
    { 0xb2fd321d, 46 },
    { 0xb83d321d, 23 },
    { 0xc05d321f, 33 },
    { 0xd91d3220, 33 },
    { 0x762d3221, 92 },
    { 0xf18d3222, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7d3227, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5d322a, 33 },
    { 0, 0 },
    { 0xb5bd322c, 55 },
    { 0xd7ad322c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bd3232, 55 },
    { 0xe2dd3232, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bd3236, 68 },
    { 0xf0fd3236, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0d323c, 98 },
    { 0xaafd323d, 99 },
    { 0x525d323e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d3241, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad3246, 61 },
    { 0x671d3247, 98 },
    { 0x358d3246, 46 },
    { 0xdcbd3247, 91 },
    { 0xf19d324a, 43 },
    { 0, 0 },
    { 0x61dd324c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1d324f, 61 },
    { 0xa09d3250, 61 },
    { 0x465d3251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362d3255, 93 },
    { 0, 0 },
    { 0xbcbd3257, 76 },
    { 0x30cd3258, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39d325d, 33 },
    { 0x368d325e, 61 },
    { 0xdb0d325d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdd326a, 68 },
    { 0, 0 },
    { 0x1f1d326c, 98 },
    { 0x5ddd326d, 33 },
    { 0x3ded326d, 33 },
    { 0xb6bd326f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x706d3272, 61 },
    { 0xf69d3272, 90 },
    { 0x2a3d3272, 61 },
    { 0x8efd3275, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecd327b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bed3283, 95 },
    { 0, 0 },
    { 0x4bbd3285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd3288, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cd328c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d3290, 95 },
    { 0x52ed3290, 23 },
    { 0x234d3292, 46 },
    { 0x78cd3292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5d3297, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d329d, 93 },
    { 0x1c7d329e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50d32a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3d32a6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dad32b1, 98 },
    { 0xa9ad32b2, 55 },
    { 0, 0 },
    { 0x7c9d32b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaed32be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71d32c2, 90 },
    { 0, 0 },
    { 0x1fdd32c4, 53 },
    { 0xb40d32c5, 98 },
    { 0, 0 },
    { 0x541d32c7, 76 },
    { 0x6fed32c7, 46 },
    { 0, 0 },
    { 0x2dd32ca, 90 },
    { 0x996d32ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbd32d2, 17 },
    { 0xa96d32d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89d32d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd32dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99d32df, 90 },
    { 0x4fdd32df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85d32e7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44d32ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d32ed, 61 },
    { 0x1e7d32ed, 61 },
    { 0x12bd32ed, 36 },
    { 0x4f7d32ef, 17 },
    { 0x89ad32f1, 17 },
    { 0xa5d32f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41d32fb, 95 },
    { 0, 0 },
    { 0x220d32fd, 66 },
    { 0, 0 },
    { 0xa84d32ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x676d3302, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dd331b, 66 },
    { 0x4e3d331b, 33 },
    { 0x84d331d, 33 },
    { 0, 0 },
    { 0xad0d331f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2d3323, 78 },
    { 0xf83d3324, 55 },
    { 0xdb7d3325, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49d3328, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764d332f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd3333, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d3336, 68 },
    { 0xeddd3336, 61 },
    { 0x738d3338, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65d333b, 23 },
    { 0, 0 },
    { 0xbb1d333d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x312d3340, 90 },
    { 0x112d3341, 99 },
    { 0xb1ed3340, 23 },
    { 0x473d3343, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0d3347, 98 },
    { 0xa3ad3347, 33 },
    { 0, 0 },
    { 0x76ad334a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21d334d, 33 },
    { 0x60ed334e, 98 },
    { 0, 0 },
    { 0xa09d3350, 61 },
    { 0x609d3351, 33 },
    { 0x180d3352, 53 },
    { 0, 0 },
    { 0x74fd3354, 98 },
    { 0xd53d3355, 43 },
    { 0x91dd3355, 90 },
    { 0x58dd3357, 46 },
    { 0xdabd3355, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426d335e, 76 },
    { 0xd6dd335e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0d3362, 85 },
    { 0xf59d3363, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead3366, 90 },
    { 0xde9d3367, 34 },
    { 0xb72d3366, 61 },
    { 0, 0 },
    { 0x18d336a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43d336d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6d3370, 34 },
    { 0, 0 },
    { 0xde9d3372, 33 },
    { 0xa19d3373, 90 },
    { 0x291d3374, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd3377, 33 },
    { 0xa1ed3377, 55 },
    { 0, 0 },
    { 0x5bcd337a, 99 },
    { 0x155d337b, 23 },
    { 0x309d337c, 61 },
    { 0x84dd337a, 84 },
    { 0xe46d337a, 95 },
    { 0xf34d337b, 90 },
    { 0, 0 },
    { 0xef8d3381, 55 },
    { 0, 0 },
    { 0x379d3383, 66 },
    { 0xb6d3384, 33 },
    { 0x55bd3384, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22d338c, 23 },
    { 0x2e9d338d, 69 },
    { 0xc8ad338d, 93 },
    { 0xaa0d338f, 33 },
    { 0x372d3390, 83 },
    { 0x368d3391, 88 },
    { 0x15fd3390, 23 },
    { 0x2b9d3391, 93 },
    { 0x87cd3394, 87 },
    { 0, 0 },
    { 0xf33d3396, 46 },
    { 0, 0 },
    { 0x9b8d3398, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707d339d, 36 },
    { 0, 0 },
    { 0x944d339f, 67 },
    { 0x3ffd33a0, 43 },
    { 0xf84d33a1, 61 },
    { 0, 0 },
    { 0xdf6d33a3, 61 },
    { 0x4f6d33a4, 90 },
    { 0x47fd33a5, 87 },
    { 0x733d33a6, 93 },
    { 0, 0 },
    { 0x28d33a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d33b0, 93 },
    { 0xf64d33b1, 95 },
    { 0x58cd33b2, 66 },
    { 0xdc3d33b1, 33 },
    { 0xe4dd33b4, 78 },
    { 0x13cd33b4, 40 },
    { 0x267d33b1, 33 },
    { 0x2d4d33b7, 87 },
    { 0x6f0d33b8, 34 },
    { 0x577d33b9, 65 },
    { 0xd73d33ba, 43 },
    { 0xa10d33bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efd33bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabd33c2, 76 },
    { 0, 0 },
    { 0xfb6d33c4, 55 },
    { 0xb0fd33c5, 98 },
    { 0, 0 },
    { 0x415d33c7, 38 },
    { 0xf90d33c7, 23 },
    { 0xa6ed33c9, 93 },
    { 0x816d33c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0d33cd, 88 },
    { 0xcc2d33cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d33d6, 61 },
    { 0, 0 },
    { 0xb84d33d8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ad33db, 34 },
    { 0, 0 },
    { 0x4a0d33dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d33e0, 23 },
    { 0x99ad33e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124d33e5, 76 },
    { 0x47fd33e6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573d33ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ed33f2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96d33f7, 90 },
    { 0x2e9d33f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd33fc, 90 },
    { 0xbced33fd, 61 },
    { 0, 0 },
    { 0x2dcd33ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d3405, 52 },
    { 0xbfcd3406, 91 },
    { 0xf8fd3407, 98 },
    { 0xc1cd3407, 98 },
    { 0xe7bd3409, 76 },
    { 0xc39d3405, 33 },
    { 0x73d340a, 95 },
    { 0x60dd340c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34d3410, 61 },
    { 0xc1dd3411, 61 },
    { 0xf4cd3412, 33 },
    { 0x699d3411, 93 },
    { 0x497d3414, 100 },
    { 0xeabd3412, 34 },
    { 0x3fd3413, 88 },
    { 0x8b7d3417, 61 },
    { 0x68bd3412, 98 },
    { 0x13ad3419, 87 },
    { 0xf40d341a, 78 },
    { 0xf21d3416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11d341e, 23 },
    { 0xba2d341e, 85 },
    { 0, 0 },
    { 0x27bd3421, 23 },
    { 0x7ed3421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254d342c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x444d342f, 95 },
    { 0x803d3430, 78 },
    { 0, 0 },
    { 0x50bd3432, 90 },
    { 0x80ed3433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbd3437, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7d3440, 90 },
    { 0x60d3441, 85 },
    { 0x553d3441, 23 },
    { 0x86ad3440, 43 },
    { 0xc01d3444, 66 },
    { 0x93ed3442, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975d344b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81d344e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9d3451, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed3456, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0d345a, 66 },
    { 0xe97d345a, 36 },
    { 0, 0 },
    { 0xcad345d, 75 },
    { 0x659d345e, 93 },
    { 0xc4ed345e, 33 },
    { 0x3a7d3460, 78 },
    { 0, 0 },
    { 0xc70d3462, 66 },
    { 0x111d3463, 90 },
    { 0xadfd3464, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ad346c, 76 },
    { 0x191d346c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4d3471, 65 },
    { 0x209d3471, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77d3477, 61 },
    { 0x2cfd3477, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdd347c, 33 },
    { 0xbd3d347d, 90 },
    { 0, 0 },
    { 0xdb1d347f, 76 },
    { 0, 0 },
    { 0xc06d3481, 61 },
    { 0, 0 },
    { 0xa4dd3483, 95 },
    { 0xe9ad3484, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6d3487, 76 },
    { 0, 0 },
    { 0xaad3489, 87 },
    { 0x56ed348a, 90 },
    { 0xaa0d348b, 61 },
    { 0x9c0d348b, 33 },
    { 0xf64d348d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d3491, 66 },
    { 0x812d3492, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194d3498, 88 },
    { 0xb4cd3499, 23 },
    { 0, 0 },
    { 0xb73d349b, 95 },
    { 0, 0 },
    { 0xb76d349d, 34 },
    { 0, 0 },
    { 0x558d349f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fed34a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed34a6, 91 },
    { 0x5a8d34a6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fd34aa, 98 },
    { 0x7a4d34ab, 52 },
    { 0xf01d34aa, 23 },
    { 0, 0 },
    { 0x67ad34ae, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bd34b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d34b9, 75 },
    { 0, 0 },
    { 0xd3ed34bb, 91 },
    { 0xa6d34bc, 43 },
    { 0, 0 },
    { 0x64d34be, 98 },
    { 0, 0 },
    { 0xef0d34c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9d34c5, 23 },
    { 0, 0 },
    { 0x39d34c7, 80 },
    { 0x202d34c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacd34cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ed34cf, 53 },
    { 0, 0 },
    { 0x7b0d34d1, 19 },
    { 0x896d34d2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4d34d6, 84 },
    { 0xc7d34d6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x116d34da, 61 },
    { 0x538d34db, 33 },
    { 0, 0 },
    { 0x222d34dd, 43 },
    { 0x68ad34de, 33 },
    { 0xa89d34df, 98 },
    { 0xd0dd34de, 40 },
    { 0x5c4d34e1, 23 },
    { 0xde2d34df, 55 },
    { 0xc6dd34e3, 34 },
    { 0x1c6d34e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bd34e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4d34f0, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605d34f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebd34f7, 95 },
    { 0, 0 },
    { 0x165d34f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79d34fc, 68 },
    { 0, 0 },
    { 0x80fd34fe, 33 },
    { 0xbb1d34ff, 90 },
    { 0xba9d34ff, 23 },
    { 0xbd0d3501, 90 },
    { 0, 0 },
    { 0x30dd3503, 90 },
    { 0xbf5d3504, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d350a, 55 },
    { 0, 0 },
    { 0xbd9d350c, 99 },
    { 0x29ad350d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2d3512, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94d3517, 95 },
    { 0xb2dd3517, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbded351c, 33 },
    { 0, 0 },
    { 0x90dd351e, 61 },
    { 0x6f4d351f, 90 },
    { 0, 0 },
    { 0xf5dd3521, 61 },
    { 0x8e3d3522, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22d3527, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dd352c, 98 },
    { 0x35bd352c, 55 },
    { 0x376d352d, 95 },
    { 0x793d352f, 46 },
    { 0, 0 },
    { 0x5fed3531, 91 },
    { 0xc16d3531, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125d353d, 98 },
    { 0xc2d353e, 90 },
    { 0x929d353e, 77 },
    { 0xcbd353e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x312d3543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd3546, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809d354a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbd3550, 95 },
    { 0x11bd3550, 55 },
    { 0x9d8d3552, 90 },
    { 0x38d3553, 90 },
    { 0, 0 },
    { 0x769d3555, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d355a, 61 },
    { 0x516d355b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71d3562, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479d3566, 55 },
    { 0, 0 },
    { 0xb60d3568, 84 },
    { 0, 0 },
    { 0x1ecd356a, 78 },
    { 0, 0 },
    { 0x467d356c, 95 },
    { 0, 0 },
    { 0x6a1d356e, 33 },
    { 0x526d356f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x50d3572, 33 },
    { 0x3a5d3572, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd3576, 23 },
    { 0xd01d3577, 61 },
    { 0xe3dd3576, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1d357b, 55 },
    { 0xb28d357b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ed3584, 74 },
    { 0x837d3585, 61 },
    { 0x91ed3584, 61 },
    { 0, 0 },
    { 0x58bd3588, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d358c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebd3591, 61 },
    { 0xc3fd3591, 33 },
    { 0x462d3593, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ad359c, 23 },
    { 0x1d4d359d, 95 },
    { 0x9e2d359e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8d35a3, 34 },
    { 0, 0 },
    { 0xdf3d35a5, 33 },
    { 0xfe0d35a5, 61 },
    { 0x663d35a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcd35b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fd35b9, 87 },
    { 0xc5d35b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8d35be, 76 },
    { 0, 0 },
    { 0x118d35c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4d35c5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcd35d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429d35d5, 33 },
    { 0x3c2d35d6, 55 },
    { 0x9a8d35d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d35da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ad35de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d35e3, 43 },
    { 0, 0 },
    { 0x5e8d35e5, 85 },
    { 0xd97d35e6, 90 },
    { 0xfcfd35e7, 34 },
    { 0xf09d35e8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60d35ec, 46 },
    { 0xa80d35ed, 90 },
    { 0, 0 },
    { 0xcecd35ef, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69d35f6, 33 },
    { 0xa94d35f6, 61 },
    { 0x95d35f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49d35fc, 93 },
    { 0xfeed35fd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7d3600, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x740d3603, 23 },
    { 0x674d3604, 23 },
    { 0x7f9d3604, 23 },
    { 0xcecd3604, 95 },
    { 0x646d3606, 55 },
    { 0x381d3608, 90 },
    { 0xd6ed3609, 91 },
    { 0xf13d3609, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecd360e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9d3611, 98 },
    { 0x5c1d3611, 33 },
    { 0, 0 },
    { 0xbf1d3614, 93 },
    { 0x2dad3614, 47 },
    { 0xb7dd3614, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6d361b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fd3622, 61 },
    { 0x137d3623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad3628, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1d362e, 34 },
    { 0x102d362f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad3632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196d3637, 95 },
    { 0x7b6d3638, 90 },
    { 0xd4cd3638, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d363c, 76 },
    { 0, 0 },
    { 0x229d363e, 95 },
    { 0xe9ad363f, 93 },
    { 0x38dd3640, 33 },
    { 0x266d3641, 61 },
    { 0x101d3640, 33 },
    { 0xf38d363e, 94 },
    { 0xaa4d3641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9d364a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dd364f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2d3653, 23 },
    { 0x9b4d3653, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xecad3657, 98 },
    { 0, 0 },
    { 0x66d3659, 67 },
    { 0xb6fd365a, 55 },
    { 0xe79d365a, 52 },
    { 0x6e0d365c, 52 },
    { 0x851d365b, 66 },
    { 0xd19d365a, 84 },
    { 0, 0 },
    { 0xdc2d3660, 61 },
    { 0x571d3661, 23 },
    { 0x657d3662, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd366d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ad3670, 90 },
    { 0, 0 },
    { 0x9f4d3672, 90 },
    { 0, 0 },
    { 0x927d3674, 61 },
    { 0x8ded3675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d367a, 93 },
    { 0xdc0d367a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d3682, 55 },
    { 0, 0 },
    { 0x727d3684, 33 },
    { 0, 0 },
    { 0x387d3686, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95d3689, 34 },
    { 0xd1cd368a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d368d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd3690, 34 },
    { 0x509d3690, 84 },
    { 0, 0 },
    { 0xdb0d3693, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aed369c, 46 },
    { 0x10d369d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acd36a1, 91 },
    { 0x905d36a1, 90 },
    { 0xc4fd36a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x793d36a6, 33 },
    { 0, 0 },
    { 0xa1bd36a8, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d36ab, 23 },
    { 0, 0 },
    { 0x1b0d36ad, 90 },
    { 0x5fdd36ad, 61 },
    { 0xcf6d36af, 95 },
    { 0x17ed36b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87d36b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3d36b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418d36bc, 90 },
    { 0x652d36bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493d36c4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9d36c9, 66 },
    { 0, 0 },
    { 0xa06d36cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bd36ce, 76 },
    { 0x24d36ce, 98 },
    { 0x180d36d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ad36d3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636d36d7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ad36dc, 90 },
    { 0xbed36dd, 33 },
    { 0, 0 },
    { 0xaed36df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fd36e4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803d36eb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x568d36ee, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3d36f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd36f9, 90 },
    { 0, 0 },
    { 0x6f5d36fb, 23 },
    { 0x7cfd36fb, 84 },
    { 0xbf0d36fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd3705, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafad3709, 90 },
    { 0x7efd370a, 61 },
    { 0x508d3709, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40d370e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed3711, 95 },
    { 0, 0 },
    { 0xb25d3713, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x272d3716, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dd371a, 53 },
    { 0x604d371a, 23 },
    { 0xdf3d371c, 43 },
    { 0, 0 },
    { 0xb40d371e, 77 },
    { 0x7c2d371f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9d3725, 95 },
    { 0, 0 },
    { 0xdffd3727, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165d372b, 98 },
    { 0xbaad372c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83d372f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933d373c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904d3740, 95 },
    { 0, 0 },
    { 0xc38d3742, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x988d3745, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d3748, 23 },
    { 0xaa2d3749, 83 },
    { 0x55ed374a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd374d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ad3752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fd3756, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27d3759, 65 },
    { 0x65d375a, 61 },
    { 0x6bcd375a, 38 },
    { 0x77dd375c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0d3761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349d3766, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0d376a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d376d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345d3771, 61 },
    { 0, 0 },
    { 0x787d3773, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fd3776, 34 },
    { 0xba6d3777, 33 },
    { 0x7e6d3778, 75 },
    { 0, 0 },
    { 0xce8d377a, 23 },
    { 0, 0 },
    { 0x79d377c, 90 },
    { 0xc67d377d, 23 },
    { 0xced377e, 68 },
    { 0x272d377d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52d3788, 98 },
    { 0x355d3789, 43 },
    { 0x230d378a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33d378e, 23 },
    { 0, 0 },
    { 0xc78d3790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19d3796, 55 },
    { 0x6cd3797, 90 },
    { 0xe37d3798, 76 },
    { 0, 0 },
    { 0x5b4d379a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2d37a7, 95 },
    { 0x571d37a7, 24 },
    { 0x4fd37a9, 78 },
    { 0xd97d37aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944d37af, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78d37b2, 77 },
    { 0xe91d37b2, 67 },
    { 0xd04d37b4, 93 },
    { 0, 0 },
    { 0x98ed37b6, 23 },
    { 0x81ed37b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654d37bc, 33 },
    { 0x3afd37bd, 98 },
    { 0x8ecd37be, 95 },
    { 0xae4d37bf, 90 },
    { 0x9e7d37bd, 61 },
    { 0xda0d37c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54d37c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01d37cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ed37cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad37d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd37da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fd37df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4d37ea, 88 },
    { 0x79bd37eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeded37f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bed37f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d37fb, 90 },
    { 0xc3ed37fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4d380a, 61 },
    { 0x29bd380b, 80 },
    { 0xf5fd380c, 55 },
    { 0, 0 },
    { 0xa39d380e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd3812, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded3815, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedd3819, 95 },
    { 0, 0 },
    { 0xd1cd381b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d3824, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01d3828, 33 },
    { 0, 0 },
    { 0x8f4d382a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d3835, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94d383b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d383f, 61 },
    { 0, 0 },
    { 0x23ad3841, 90 },
    { 0xb08d3842, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7d3851, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d3856, 80 },
    { 0x217d3856, 55 },
    { 0, 0 },
    { 0x36dd3859, 90 },
    { 0, 0 },
    { 0xf37d385b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5d3860, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d3867, 23 },
    { 0x901d3867, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634d386d, 23 },
    { 0xd84d386e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cd3872, 40 },
    { 0xf91d3872, 61 },
    { 0, 0 },
    { 0x6b0d3875, 90 },
    { 0x5aed3876, 34 },
    { 0x305d3876, 40 },
    { 0xb93d3878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6d387d, 78 },
    { 0x96dd387e, 95 },
    { 0x5b1d387e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed3883, 90 },
    { 0x439d3884, 95 },
    { 0x3fbd3885, 33 },
    { 0x641d3886, 76 },
    { 0x6bdd3887, 40 },
    { 0x69fd3887, 90 },
    { 0, 0 },
    { 0x924d388a, 61 },
    { 0xe22d388b, 91 },
    { 0, 0 },
    { 0xc3dd388d, 23 },
    { 0xa85d388d, 95 },
    { 0, 0 },
    { 0x990d3890, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dd3894, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dad3898, 100 },
    { 0x8fad3899, 56 },
    { 0, 0 },
    { 0xe54d389b, 98 },
    { 0, 0 },
    { 0x2dfd389d, 90 },
    { 0x3bed389e, 23 },
    { 0xacdd389e, 61 },
    { 0, 0 },
    { 0xf4dd38a1, 46 },
    { 0, 0 },
    { 0x3a9d38a3, 98 },
    { 0xb22d38a4, 98 },
    { 0xa2dd38a5, 46 },
    { 0, 0 },
    { 0x5d9d38a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179d38ae, 33 },
    { 0xd97d38af, 90 },
    { 0x4ddd38b0, 90 },
    { 0x52dd38b0, 95 },
    { 0, 0 },
    { 0x94d38b3, 61 },
    { 0x990d38b4, 90 },
    { 0xea9d38b4, 61 },
    { 0, 0 },
    { 0x695d38b7, 91 },
    { 0x595d38b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bd38bd, 61 },
    { 0, 0 },
    { 0x8cbd38bf, 93 },
    { 0xf70d38bf, 99 },
    { 0, 0 },
    { 0x5bed38c2, 90 },
    { 0xeb1d38c3, 98 },
    { 0, 0 },
    { 0x7ded38c5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaed38cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2d38d0, 98 },
    { 0x82cd38d1, 61 },
    { 0xb1ad38d1, 33 },
    { 0x354d38d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d38d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33d38db, 23 },
    { 0, 0 },
    { 0xc52d38dd, 85 },
    { 0, 0 },
    { 0x203d38df, 33 },
    { 0, 0 },
    { 0xcb3d38e1, 61 },
    { 0, 0 },
    { 0x8ad38e3, 95 },
    { 0xed4d38e4, 23 },
    { 0xe81d38e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed38e8, 98 },
    { 0, 0 },
    { 0xa52d38ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffd38ed, 95 },
    { 0xa9cd38ed, 67 },
    { 0x839d38ef, 90 },
    { 0, 0 },
    { 0xfd2d38f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9d38f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d38fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837d3900, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72d3904, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64d390a, 98 },
    { 0, 0 },
    { 0xf53d390c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90d3910, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d3913, 76 },
    { 0xf78d3914, 98 },
    { 0xedbd3915, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd3918, 33 },
    { 0x218d3918, 23 },
    { 0xb51d391a, 33 },
    { 0x49dd391a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d3926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4d3929, 66 },
    { 0x587d3929, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x17d392d, 76 },
    { 0x16ed392e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462d3937, 98 },
    { 0xb29d3938, 43 },
    { 0, 0 },
    { 0xcb5d393a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55d393f, 55 },
    { 0xced393f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad3944, 33 },
    { 0x763d3945, 90 },
    { 0xc8d3946, 76 },
    { 0xeb3d3947, 36 },
    { 0x15ed3945, 90 },
    { 0xf9ad3944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9d394c, 98 },
    { 0x8aad394d, 95 },
    { 0, 0 },
    { 0xf40d394f, 83 },
    { 0, 0 },
    { 0x3f2d3951, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ed3956, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaed3959, 23 },
    { 0, 0 },
    { 0x327d395b, 33 },
    { 0, 0 },
    { 0x5b9d395d, 87 },
    { 0xacfd395e, 95 },
    { 0x28fd395f, 33 },
    { 0x5bfd395f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297d3967, 75 },
    { 0xcb0d3967, 96 },
    { 0x65dd3969, 33 },
    { 0x334d3969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86d3975, 80 },
    { 0xbf7d3976, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ed397a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd397e, 51 },
    { 0x9a6d397f, 76 },
    { 0xfe4d3980, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1d3985, 75 },
    { 0xbdd3986, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1d398c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04d398f, 46 },
    { 0x9bfd3990, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ad3994, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d399a, 95 },
    { 0, 0 },
    { 0xf94d399c, 76 },
    { 0, 0 },
    { 0xa7bd399e, 98 },
    { 0, 0 },
    { 0x3cad39a0, 61 },
    { 0, 0 },
    { 0x27d39a2, 23 },
    { 0, 0 },
    { 0x63fd39a4, 33 },
    { 0x4e9d39a5, 23 },
    { 0xc43d39a6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed39ad, 84 },
    { 0xb06d39ae, 95 },
    { 0xcbfd39ae, 46 },
    { 0x9c6d39ae, 90 },
    { 0, 0 },
    { 0x1cbd39b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769d39bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455d39cb, 33 },
    { 0x796d39cc, 85 },
    { 0xefbd39cd, 33 },
    { 0x866d39cc, 55 },
    { 0, 0 },
    { 0xb9ed39d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6d39d3, 23 },
    { 0, 0 },
    { 0xc2d39d5, 95 },
    { 0, 0 },
    { 0xa3bd39d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27d39dc, 78 },
    { 0x987d39dd, 46 },
    { 0xd8ed39de, 100 },
    { 0x87cd39df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d39e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31d39e5, 88 },
    { 0, 0 },
    { 0x91ed39e7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fd39ec, 98 },
    { 0, 0 },
    { 0x246d39ee, 61 },
    { 0, 0 },
    { 0xf5ad39f0, 61 },
    { 0xb54d39f1, 99 },
    { 0x9dd39f0, 66 },
    { 0xf5ed39f3, 23 },
    { 0xd3cd39f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d39fd, 70 },
    { 0xb7cd39fe, 98 },
    { 0, 0 },
    { 0x5bcd3a00, 93 },
    { 0xbabd3a01, 93 },
    { 0x246d3a00, 43 },
    { 0x731d3a03, 95 },
    { 0xeb1d3a04, 77 },
    { 0x25d3a05, 90 },
    { 0x44fd3a01, 90 },
    { 0xf58d3a07, 23 },
    { 0xf72d3a07, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6d3a0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d3a13, 90 },
    { 0, 0 },
    { 0x74bd3a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5d3a1c, 61 },
    { 0xd99d3a1d, 78 },
    { 0, 0 },
    { 0x400d3a1f, 17 },
    { 0, 0 },
    { 0x499d3a21, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fd3a25, 33 },
    { 0xd7d3a25, 90 },
    { 0xb7ad3a27, 53 },
    { 0, 0 },
    { 0x7d2d3a29, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210d3a2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x906d3a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cd3a33, 95 },
    { 0x85ed3a34, 98 },
    { 0xafcd3a35, 23 },
    { 0xda7d3a33, 90 },
    { 0xe8cd3a37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aad3a3a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cd3a3e, 33 },
    { 0, 0 },
    { 0x50ad3a40, 23 },
    { 0x667d3a41, 100 },
    { 0x589d3a40, 78 },
    { 0xa03d3a43, 91 },
    { 0xfe4d3a43, 98 },
    { 0x2ecd3a45, 84 },
    { 0, 0 },
    { 0xe3fd3a47, 97 },
    { 0, 0 },
    { 0xadbd3a49, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69d3a4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132d3a55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12d3a58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435d3a61, 90 },
    { 0x707d3a61, 98 },
    { 0x416d3a63, 93 },
    { 0x685d3a64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x724d3a67, 95 },
    { 0x4a9d3a68, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0d3a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667d3a75, 90 },
    { 0x94dd3a76, 76 },
    { 0x112d3a77, 24 },
    { 0, 0 },
    { 0x21bd3a79, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd3a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3d3a7f, 99 },
    { 0, 0 },
    { 0x1efd3a81, 91 },
    { 0, 0 },
    { 0x4dbd3a83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d3a89, 100 },
    { 0, 0 },
    { 0xe5bd3a8b, 61 },
    { 0x7a8d3a8c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fed3a90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fd3a97, 90 },
    { 0xeb4d3a97, 65 },
    { 0, 0 },
    { 0x244d3a9a, 61 },
    { 0, 0 },
    { 0x630d3a9c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x300d3a9f, 75 },
    { 0x1e4d3aa0, 77 },
    { 0x8d1d3aa0, 34 },
    { 0x316d3aa1, 100 },
    { 0xacad3aa2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cd3aa6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfd3aaa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ad3aad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d3ab1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29d3ab8, 90 },
    { 0x6e6d3ab8, 98 },
    { 0x71fd3aba, 76 },
    { 0x199d3abb, 93 },
    { 0xd47d3abc, 91 },
    { 0x6d6d3ab9, 95 },
    { 0, 0 },
    { 0x593d3abf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdd3ac2, 90 },
    { 0xab8d3ac3, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bad3ac8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd3acf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3d3ad5, 95 },
    { 0x66dd3ad5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfd3ada, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x649d3add, 86 },
    { 0x98cd3ade, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadd3ae4, 100 },
    { 0x4afd3ae4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1d3ae8, 33 },
    { 0x907d3ae9, 66 },
    { 0x40ad3aea, 98 },
    { 0xebfd3aeb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8d3aee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd3af7, 80 },
    { 0x9f3d3af8, 87 },
    { 0xd3d3af7, 95 },
    { 0, 0 },
    { 0x7a6d3afb, 90 },
    { 0xee0d3afc, 87 },
    { 0x9ddd3afc, 34 },
    { 0, 0 },
    { 0xb4fd3aff, 90 },
    { 0x2ead3b00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d3b04, 61 },
    { 0, 0 },
    { 0x2a9d3b06, 17 },
    { 0x1add3b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d3b0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d3b0f, 61 },
    { 0xfa3d3b10, 23 },
    { 0, 0 },
    { 0xd81d3b12, 61 },
    { 0x23bd3b13, 56 },
    { 0, 0 },
    { 0x65bd3b15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d3b19, 95 },
    { 0xe4cd3b1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2d3b1d, 90 },
    { 0x638d3b1e, 95 },
    { 0xcbcd3b1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fad3b24, 98 },
    { 0xad3b25, 61 },
    { 0, 0 },
    { 0x513d3b27, 98 },
    { 0x61d3b28, 88 },
    { 0, 0 },
    { 0x8cad3b2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dd3b2d, 98 },
    { 0xfa9d3b2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cd3b32, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855d3b36, 34 },
    { 0x191d3b37, 33 },
    { 0x75dd3b38, 33 },
    { 0x1f2d3b39, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96d3b3e, 67 },
    { 0, 0 },
    { 0x4e4d3b40, 90 },
    { 0xc99d3b41, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d3b44, 91 },
    { 0x83bd3b45, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35d3b48, 53 },
    { 0x42dd3b49, 95 },
    { 0xe10d3b49, 91 },
    { 0xab1d3b4b, 78 },
    { 0x438d3b4c, 67 },
    { 0xbc6d3b4b, 67 },
    { 0x9a8d3b4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270d3b53, 33 },
    { 0xf16d3b54, 95 },
    { 0x662d3b54, 95 },
    { 0xe4cd3b56, 98 },
    { 0xb90d3b57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7d3b5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfd3b64, 95 },
    { 0x8f3d3b65, 91 },
    { 0x7c2d3b66, 23 },
    { 0x45bd3b67, 33 },
    { 0xa46d3b68, 95 },
    { 0x56ad3b68, 55 },
    { 0x670d3b6a, 61 },
    { 0xd48d3b6a, 55 },
    { 0x289d3b6c, 33 },
    { 0x3ebd3b6a, 76 },
    { 0xd3cd3b69, 90 },
    { 0x808d3b6b, 95 },
    { 0, 0 },
    { 0x8b7d3b71, 90 },
    { 0xc3ed3b72, 61 },
    { 0x3fad3b73, 61 },
    { 0xb36d3b74, 61 },
    { 0x67fd3b72, 98 },
    { 0, 0 },
    { 0x5add3b77, 95 },
    { 0x3e5d3b78, 90 },
    { 0x443d3b79, 76 },
    { 0xc3d3b77, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbd3b7f, 88 },
    { 0xcded3b80, 33 },
    { 0x59dd3b80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x614d3b84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59d3b87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277d3b90, 96 },
    { 0xd8bd3b91, 61 },
    { 0, 0 },
    { 0xa8fd3b93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211d3b9a, 98 },
    { 0x843d3b9b, 90 },
    { 0x95cd3b9a, 90 },
    { 0, 0 },
    { 0xdedd3b9e, 91 },
    { 0, 0 },
    { 0xe28d3ba0, 33 },
    { 0xc7ed3ba0, 33 },
    { 0x2d3ba2, 43 },
    { 0, 0 },
    { 0x37dd3ba4, 34 },
    { 0x6f4d3ba5, 61 },
    { 0xf17d3ba6, 84 },
    { 0, 0 },
    { 0x7c1d3ba8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bd3bab, 46 },
    { 0x348d3bac, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9add3baf, 55 },
    { 0, 0 },
    { 0x45cd3bb1, 98 },
    { 0xcdfd3bb2, 23 },
    { 0x73cd3bb3, 76 },
    { 0x6c9d3bb3, 61 },
    { 0xb27d3bb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dd3bb8, 33 },
    { 0x71cd3bb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7d3bbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x527d3bc0, 33 },
    { 0x15bd3bc0, 23 },
    { 0x54bd3bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345d3bc8, 78 },
    { 0x7a5d3bc8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd3bcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2d3bd4, 46 },
    { 0, 0 },
    { 0x36bd3bd6, 61 },
    { 0x63d3bd7, 95 },
    { 0x23dd3bd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bd3be2, 33 },
    { 0x674d3be3, 23 },
    { 0, 0 },
    { 0x371d3be5, 23 },
    { 0xb69d3be6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffd3bed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffd3bf1, 85 },
    { 0x1cad3bf1, 34 },
    { 0x524d3bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed3bfa, 55 },
    { 0x93ed3bfb, 67 },
    { 0, 0 },
    { 0x780d3bfd, 33 },
    { 0, 0 },
    { 0x71ed3bff, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x733d3c02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bd3c05, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd3c0b, 61 },
    { 0, 0 },
    { 0x4ead3c0d, 98 },
    { 0x966d3c0e, 61 },
    { 0, 0 },
    { 0x835d3c10, 87 },
    { 0xc96d3c11, 95 },
    { 0x699d3c12, 55 },
    { 0xfcfd3c13, 43 },
    { 0x61d3c14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccd3c1a, 77 },
    { 0x6a3d3c1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30d3c25, 97 },
    { 0x24ed3c25, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02d3c29, 76 },
    { 0x8c0d3c2a, 90 },
    { 0xbb4d3c2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d3c2f, 23 },
    { 0xbad3c2f, 61 },
    { 0x340d3c31, 43 },
    { 0xcd6d3c31, 98 },
    { 0, 0 },
    { 0x2edd3c34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d3c38, 95 },
    { 0xeb3d3c38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd3c3e, 74 },
    { 0xebbd3c3f, 76 },
    { 0xc63d3c3f, 90 },
    { 0, 0 },
    { 0x3ecd3c42, 99 },
    { 0, 0 },
    { 0xdc8d3c44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2d3c4a, 78 },
    { 0x600d3c4b, 34 },
    { 0xb27d3c4c, 61 },
    { 0, 0 },
    { 0xb4bd3c4e, 17 },
    { 0xacad3c4f, 78 },
    { 0x776d3c50, 69 },
    { 0x370d3c4f, 46 },
    { 0x203d3c52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x829d3c55, 93 },
    { 0xce2d3c56, 90 },
    { 0, 0 },
    { 0x4ccd3c58, 77 },
    { 0x493d3c59, 90 },
    { 0x681d3c59, 90 },
    { 0, 0 },
    { 0x2a8d3c5c, 90 },
    { 0x2c4d3c5d, 55 },
    { 0xdc1d3c5e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72d3c6a, 61 },
    { 0x714d3c6b, 55 },
    { 0xb99d3c6a, 23 },
    { 0xdbad3c6c, 23 },
    { 0, 0 },
    { 0x349d3c6f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71d3c78, 33 },
    { 0x4f4d3c79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cd3c81, 22 },
    { 0xa56d3c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6d3c85, 95 },
    { 0xaded3c85, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89d3c8a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475d3c8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecd3c91, 95 },
    { 0x1acd3c92, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bd3c96, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebd3c9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dad3ca3, 33 },
    { 0xcbcd3ca3, 61 },
    { 0x235d3ca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ed3cae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fad3cb1, 33 },
    { 0x370d3cb1, 43 },
    { 0x80fd3cb3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5d3cb7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddd3cbc, 76 },
    { 0x2c1d3cbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363d3cc2, 75 },
    { 0x959d3cc3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d3cca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x440d3ccd, 95 },
    { 0xe0cd3ccd, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ed3cd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7d3cd8, 90 },
    { 0x72dd3cd9, 55 },
    { 0x9a0d3cda, 90 },
    { 0x6c0d3cdb, 23 },
    { 0xbfed3cd9, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05d3ce0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ed3ce3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaed3ce8, 98 },
    { 0x449d3ce8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ad3cee, 38 },
    { 0xbcfd3cef, 23 },
    { 0x48fd3cf0, 34 },
    { 0x8aad3cf1, 93 },
    { 0xf5ad3cf2, 69 },
    { 0xd41d3cf3, 95 },
    { 0x424d3cf4, 84 },
    { 0xf89d3cf3, 23 },
    { 0x272d3cf6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ad3cf9, 55 },
    { 0xa60d3cfa, 55 },
    { 0x249d3cfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6d3cff, 98 },
    { 0, 0 },
    { 0xfa4d3d01, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719d3d0a, 61 },
    { 0, 0 },
    { 0x937d3d0c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fd3d10, 95 },
    { 0xca2d3d11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57d3d15, 52 },
    { 0x29dd3d15, 75 },
    { 0xe3fd3d16, 33 },
    { 0x8aad3d16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83d3d1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x980d3d1e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bed3d23, 76 },
    { 0xb5cd3d24, 83 },
    { 0x33fd3d25, 81 },
    { 0x4b2d3d25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21d3d2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bd3d32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbd3d35, 78 },
    { 0xf60d3d36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5d3d40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfd3d44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931d3d4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbd3d4f, 90 },
    { 0x922d3d50, 95 },
    { 0x750d3d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bd3d54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6d3d60, 69 },
    { 0x1a9d3d61, 61 },
    { 0xee0d3d62, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ad3d66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ed3d70, 95 },
    { 0, 0 },
    { 0x5c7d3d72, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935d3d76, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82d3d83, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8d3d89, 43 },
    { 0x53dd3d8a, 95 },
    { 0x3ccd3d8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d3d8f, 77 },
    { 0xb24d3d90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d3d93, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d3d96, 43 },
    { 0, 0 },
    { 0x853d3d98, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfd3d9b, 84 },
    { 0, 0 },
    { 0xe5bd3d9d, 23 },
    { 0, 0 },
    { 0x153d3d9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dd3daa, 87 },
    { 0x529d3dab, 76 },
    { 0, 0 },
    { 0x7d3d3dad, 33 },
    { 0x6e8d3dae, 65 },
    { 0xaadd3daf, 23 },
    { 0x7cfd3dae, 91 },
    { 0x549d3db1, 78 },
    { 0xcad3db2, 47 },
    { 0xc99d3db3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25d3db8, 90 },
    { 0x99d3db9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01d3dbd, 23 },
    { 0xed1d3dbd, 88 },
    { 0, 0 },
    { 0xb3ad3dc0, 34 },
    { 0x612d3dc1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad3dc4, 33 },
    { 0xf1d3dc5, 23 },
    { 0xc7bd3dc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x722d3dc9, 61 },
    { 0, 0 },
    { 0xd52d3dcb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341d3dcf, 66 },
    { 0xe93d3dd0, 95 },
    { 0x943d3dd1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d3dd7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55d3ddb, 66 },
    { 0x317d3ddb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554d3de0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8d3de9, 83 },
    { 0x70dd3dea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ad3ded, 90 },
    { 0xf3d3dee, 33 },
    { 0x1d7d3def, 98 },
    { 0x2cfd3ded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfd3df3, 95 },
    { 0xa47d3df4, 67 },
    { 0xb41d3df5, 55 },
    { 0x691d3df4, 95 },
    { 0xf79d3df7, 43 },
    { 0x3d0d3df7, 23 },
    { 0xfb3d3df8, 76 },
    { 0x698d3df5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x498d3dfd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993d3e04, 55 },
    { 0xb25d3e05, 61 },
    { 0xf1fd3e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2d3e0d, 55 },
    { 0x9abd3e0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189d3e12, 66 },
    { 0x1a6d3e12, 95 },
    { 0, 0 },
    { 0xdedd3e15, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2d3e18, 69 },
    { 0x8b1d3e19, 81 },
    { 0xacd3e18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bd3e29, 33 },
    { 0, 0 },
    { 0xb8fd3e2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbad3e2e, 81 },
    { 0x759d3e2e, 68 },
    { 0x173d3e2e, 90 },
    { 0, 0 },
    { 0x3ad3e32, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ad3e35, 23 },
    { 0x69cd3e36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bd3e3c, 52 },
    { 0x39fd3e3d, 55 },
    { 0x9c5d3e3e, 43 },
    { 0xd60d3e3f, 98 },
    { 0, 0 },
    { 0x8c6d3e41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x895d3e44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6d3e48, 95 },
    { 0, 0 },
    { 0x5add3e4a, 42 },
    { 0x104d3e4a, 90 },
    { 0xa68d3e4c, 78 },
    { 0, 0 },
    { 0x83dd3e4e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5d3e56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3d3e5a, 97 },
    { 0x75dd3e5b, 69 },
    { 0xd84d3e5c, 95 },
    { 0xb86d3e5d, 23 },
    { 0x5c4d3e5e, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcad3e61, 90 },
    { 0, 0 },
    { 0x4e9d3e63, 95 },
    { 0x6a5d3e64, 90 },
    { 0x8fed3e65, 91 },
    { 0x52ad3e64, 98 },
    { 0, 0 },
    { 0x982d3e68, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x815d3e6b, 85 },
    { 0, 0 },
    { 0xb29d3e6d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8d3e70, 65 },
    { 0x736d3e71, 98 },
    { 0x5b7d3e70, 100 },
    { 0xf53d3e73, 61 },
    { 0x9ced3e74, 33 },
    { 0x4b1d3e74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d3e78, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffd3e7b, 61 },
    { 0xcb3d3e7c, 98 },
    { 0, 0 },
    { 0x528d3e7e, 23 },
    { 0x724d3e7e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d3e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d3e88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x699d3e8b, 34 },
    { 0x5cbd3e8c, 43 },
    { 0x392d3e8d, 66 },
    { 0x10cd3e8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239d3e92, 33 },
    { 0, 0 },
    { 0xd73d3e94, 87 },
    { 0x9ead3e95, 77 },
    { 0x98cd3e94, 90 },
    { 0, 0 },
    { 0x595d3e98, 43 },
    { 0xd94d3e98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9d3e9d, 78 },
    { 0xd7bd3e9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x614d3ea1, 17 },
    { 0, 0 },
    { 0xeaed3ea3, 61 },
    { 0x975d3ea4, 40 },
    { 0x1c8d3ea5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cd3eaa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47d3eb7, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9d3ebc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x651d3ebf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cd3ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad3ec7, 75 },
    { 0, 0 },
    { 0xa04d3ec9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d3ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64d3ed2, 33 },
    { 0x3fad3ed3, 64 },
    { 0x70ad3ed3, 23 },
    { 0x5cd3ed5, 23 },
    { 0x917d3ed2, 23 },
    { 0x984d3ed7, 87 },
    { 0x236d3ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7d3edb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabd3ede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ed3ee1, 76 },
    { 0xb58d3ee1, 55 },
    { 0x735d3ee3, 55 },
    { 0, 0 },
    { 0xc9d3ee5, 40 },
    { 0, 0 },
    { 0x411d3ee7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d3eed, 95 },
    { 0xfced3eee, 88 },
    { 0x2acd3eef, 90 },
    { 0x9e3d3ef0, 100 },
    { 0, 0 },
    { 0x12ad3ef2, 90 },
    { 0x692d3ef2, 87 },
    { 0x585d3ef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x231d3ef7, 21 },
    { 0xaad3ef7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5d3eff, 23 },
    { 0x19fd3f00, 90 },
    { 0x5f4d3f01, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7d3f07, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbd3f0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827d3f10, 90 },
    { 0x7d5d3f11, 33 },
    { 0x7d6d3f11, 61 },
    { 0xf39d3f11, 76 },
    { 0xc39d3f14, 99 },
    { 0xdfed3f12, 33 },
    { 0, 0 },
    { 0xf77d3f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bd3f1a, 34 },
    { 0x582d3f1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d3f20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6d3f24, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d3f2a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afd3f32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629d3f37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fd3f3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441d3f42, 90 },
    { 0, 0 },
    { 0x989d3f44, 90 },
    { 0, 0 },
    { 0x574d3f46, 46 },
    { 0, 0 },
    { 0x7e5d3f48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1d3f4c, 68 },
    { 0xbbdd3f4c, 55 },
    { 0, 0 },
    { 0xd0fd3f4f, 23 },
    { 0x511d3f50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216d3f58, 98 },
    { 0x606d3f58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ed3f5d, 23 },
    { 0, 0 },
    { 0x107d3f5f, 85 },
    { 0, 0 },
    { 0xe4bd3f61, 88 },
    { 0x709d3f62, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05d3f66, 98 },
    { 0x133d3f67, 55 },
    { 0x38fd3f68, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101d3f6d, 87 },
    { 0x7ead3f6e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530d3f72, 33 },
    { 0, 0 },
    { 0xdfcd3f74, 65 },
    { 0x64fd3f74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483d3f81, 90 },
    { 0x8a7d3f81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4d3f88, 37 },
    { 0x9b9d3f89, 76 },
    { 0x94dd3f8a, 93 },
    { 0x8b2d3f8b, 90 },
    { 0xb1bd3f8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433d3f98, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x76d3f9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ad3faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53d3fb0, 77 },
    { 0xbedd3fb1, 98 },
    { 0x487d3fb2, 90 },
    { 0x139d3fb2, 61 },
    { 0xd88d3fb4, 98 },
    { 0xef5d3fb3, 98 },
    { 0, 0 },
    { 0x60ed3fb7, 61 },
    { 0x49ed3fb8, 95 },
    { 0x99fd3fb7, 33 },
    { 0xfd8d3fba, 69 },
    { 0xae6d3fba, 23 },
    { 0x6f5d3fbc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7d3fc1, 55 },
    { 0, 0 },
    { 0x9f4d3fc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad3fcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ed3fd2, 46 },
    { 0x11bd3fd3, 33 },
    { 0x2f4d3fd4, 93 },
    { 0x433d3fd4, 98 },
    { 0x970d3fd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8d3fde, 40 },
    { 0, 0 },
    { 0x1add3fe0, 53 },
    { 0xa4ed3fe1, 33 },
    { 0, 0 },
    { 0x66d3fe3, 46 },
    { 0xd15d3fe4, 95 },
    { 0x6a3d3fe5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160d3feb, 91 },
    { 0xeebd3feb, 95 },
    { 0x6b0d3fec, 98 },
    { 0, 0 },
    { 0xb6ed3fef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ad3ff4, 61 },
    { 0x359d3ff5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536d3ff9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5dd3ffe, 43 },
    { 0x75bd3ffe, 61 },
    { 0x58fd3fff, 76 },
    { 0x1b6d4001, 23 },
    { 0, 0 },
    { 0x37fd4003, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dd4007, 33 },
    { 0xdd4008, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280d400f, 33 },
    { 0x3f6d400f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabd4015, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7d401b, 91 },
    { 0x630d401c, 81 },
    { 0x5e2d401d, 98 },
    { 0, 0 },
    { 0x5d3d401f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ed4022, 23 },
    { 0xdd7d4022, 61 },
    { 0x8a0d4023, 17 },
    { 0, 0 },
    { 0xf1fd4026, 55 },
    { 0xb6fd4027, 95 },
    { 0x8f3d4027, 33 },
    { 0xfb4d4029, 90 },
    { 0x19dd4029, 100 },
    { 0x4fd402b, 98 },
    { 0x3cdd402c, 33 },
    { 0x286d402d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ed4030, 51 },
    { 0x70fd4031, 33 },
    { 0xaead4030, 23 },
    { 0xeb2d4033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341d4037, 61 },
    { 0, 0 },
    { 0xae4d4039, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331d4046, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacd4049, 90 },
    { 0x8f5d404a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8d404e, 43 },
    { 0x2fd404e, 91 },
    { 0x8ced404e, 90 },
    { 0x9c3d4050, 43 },
    { 0x5b1d4052, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2d4055, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85d4059, 91 },
    { 0xdcd405a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ed405d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42d4060, 61 },
    { 0, 0 },
    { 0xa1fd4062, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ad406b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867d406f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255d4073, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d4076, 83 },
    { 0xb0dd4077, 33 },
    { 0x4f4d4077, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd407e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641d4086, 87 },
    { 0, 0 },
    { 0xe85d4088, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x963d408b, 33 },
    { 0xfc3d408c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482d4091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded4096, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bd409c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabd40a2, 52 },
    { 0x552d40a3, 95 },
    { 0x5e7d40a4, 98 },
    { 0x17ed40a5, 23 },
    { 0x6fd40a6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94d40aa, 95 },
    { 0x74cd40ab, 23 },
    { 0, 0 },
    { 0xff9d40ad, 61 },
    { 0xfdfd40ae, 33 },
    { 0xf0ed40ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fd40b2, 33 },
    { 0, 0 },
    { 0x15fd40b4, 100 },
    { 0, 0 },
    { 0xd55d40b6, 56 },
    { 0, 0 },
    { 0x896d40b8, 33 },
    { 0xd2cd40b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0d40c3, 61 },
    { 0x6d40c3, 52 },
    { 0x29d40c5, 43 },
    { 0x211d40c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887d40ce, 95 },
    { 0x96d40ce, 67 },
    { 0x8d3d40ce, 76 },
    { 0, 0 },
    { 0x6aed40d2, 33 },
    { 0, 0 },
    { 0x597d40d4, 77 },
    { 0x26ad40d4, 33 },
    { 0xb44d40d6, 90 },
    { 0, 0 },
    { 0x6f4d40d8, 90 },
    { 0x85ad40d8, 98 },
    { 0x972d40da, 33 },
    { 0xbaed40d8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ad40df, 17 },
    { 0xb8ed40df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491d40e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4d40e9, 33 },
    { 0x95d40ea, 61 },
    { 0x290d40ea, 95 },
    { 0xae4d40ec, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d40f4, 43 },
    { 0x87fd40f5, 90 },
    { 0xdeed40f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbad40fa, 76 },
    { 0x14d40fb, 43 },
    { 0x271d40fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6d40ff, 90 },
    { 0x20ad40ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ad4105, 87 },
    { 0xe64d4106, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8d4109, 90 },
    { 0, 0 },
    { 0x5b8d410b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916d4110, 33 },
    { 0xd0bd4111, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ded4116, 34 },
    { 0, 0 },
    { 0xf6fd4118, 33 },
    { 0xca4d4119, 90 },
    { 0x524d411a, 61 },
    { 0xe99d411b, 61 },
    { 0xc52d411b, 90 },
    { 0x4c9d411d, 33 },
    { 0x637d411e, 33 },
    { 0, 0 },
    { 0xaead4120, 98 },
    { 0xd8cd4120, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4d4125, 76 },
    { 0xb37d4126, 61 },
    { 0xc4bd4127, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fd4133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fd4136, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72d4139, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10d413d, 61 },
    { 0xacdd413d, 76 },
    { 0x6fbd413d, 61 },
    { 0, 0 },
    { 0x903d4141, 95 },
    { 0xfe8d4142, 93 },
    { 0x93bd4143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574d414d, 33 },
    { 0xc6ed414d, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5d4154, 46 },
    { 0x77ad4155, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d4158, 91 },
    { 0x7e1d4159, 76 },
    { 0x379d4158, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ad4161, 90 },
    { 0x405d4161, 33 },
    { 0xbebd4161, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdd4166, 85 },
    { 0x9bcd4167, 91 },
    { 0x913d4166, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d416b, 23 },
    { 0xd6ad416c, 76 },
    { 0, 0 },
    { 0xd18d416e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619d4172, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41d4175, 52 },
    { 0x2f7d4176, 98 },
    { 0x99cd4177, 23 },
    { 0x80d4178, 43 },
    { 0x3b7d4179, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d417c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91d417f, 99 },
    { 0x267d4180, 98 },
    { 0x6d0d4181, 81 },
    { 0x127d4181, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61dd4186, 90 },
    { 0x1d8d4187, 23 },
    { 0x44d4188, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0d4190, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8d4194, 46 },
    { 0x3bfd4195, 33 },
    { 0x4ead4196, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5d419c, 76 },
    { 0x41ed419d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acd41a0, 80 },
    { 0, 0 },
    { 0xf8fd41a2, 61 },
    { 0xce6d41a3, 87 },
    { 0x191d41a4, 61 },
    { 0xe28d41a5, 46 },
    { 0x946d41a6, 95 },
    { 0x79d41a7, 23 },
    { 0x3ebd41a7, 61 },
    { 0x450d41a3, 55 },
    { 0xd82d41aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ed41ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x287d41b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299d41b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7d41bb, 99 },
    { 0xad8d41bc, 23 },
    { 0x47fd41bc, 33 },
    { 0x76dd41be, 61 },
    { 0x764d41be, 77 },
    { 0x3e6d41c0, 43 },
    { 0xb7fd41c1, 85 },
    { 0x165d41c2, 43 },
    { 0xafed41c3, 34 },
    { 0x359d41c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2d41cf, 61 },
    { 0xe7ad41d0, 90 },
    { 0x6ccd41d1, 61 },
    { 0x764d41d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd41d5, 100 },
    { 0x17ad41d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfd41db, 99 },
    { 0, 0 },
    { 0xa42d41dd, 23 },
    { 0x204d41de, 55 },
    { 0, 0 },
    { 0x99dd41e0, 23 },
    { 0x5c6d41e1, 93 },
    { 0xc8ad41e2, 23 },
    { 0xb4bd41e3, 23 },
    { 0x265d41e4, 95 },
    { 0x8e6d41e5, 23 },
    { 0xb2bd41e6, 68 },
    { 0x349d41e4, 76 },
    { 0x8c3d41e8, 81 },
    { 0xc50d41e8, 85 },
    { 0xb7ed41e8, 98 },
    { 0x854d41e7, 90 },
    { 0, 0 },
    { 0x37ed41ed, 68 },
    { 0x837d41ee, 61 },
    { 0xfbbd41ee, 66 },
    { 0xf2ad41ef, 55 },
    { 0xbced41f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520d41f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add41f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad41ff, 66 },
    { 0x904d4200, 76 },
    { 0, 0 },
    { 0x2b0d4202, 87 },
    { 0x863d4203, 87 },
    { 0x10d4204, 88 },
    { 0xc70d4205, 87 },
    { 0xd91d4206, 33 },
    { 0x4e6d4207, 43 },
    { 0x8a1d4208, 46 },
    { 0x2f6d4209, 93 },
    { 0x13bd4207, 98 },
    { 0, 0 },
    { 0x422d420c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfd4210, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3d4219, 95 },
    { 0, 0 },
    { 0x2cdd421b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bd4220, 87 },
    { 0, 0 },
    { 0x7ddd4222, 34 },
    { 0x1c4d4223, 90 },
    { 0x5a6d4222, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aed4228, 23 },
    { 0, 0 },
    { 0x44cd422a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbd422d, 90 },
    { 0x80ad422e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcd4231, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x539d4234, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x478d4237, 43 },
    { 0xab4d4237, 95 },
    { 0xb8ad4238, 76 },
    { 0xbf4d4238, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7d423e, 90 },
    { 0x694d423e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bd4242, 93 },
    { 0, 0 },
    { 0xc40d4244, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed424a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fd424d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05d425a, 98 },
    { 0, 0 },
    { 0x319d425c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9d4263, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85d4269, 98 },
    { 0x7d3d4269, 87 },
    { 0x995d426b, 61 },
    { 0, 0 },
    { 0x82bd426d, 87 },
    { 0, 0 },
    { 0x4c2d426f, 23 },
    { 0, 0 },
    { 0xdccd4271, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fd4274, 91 },
    { 0x6c2d4274, 95 },
    { 0xab1d4276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5d427a, 90 },
    { 0, 0 },
    { 0x5bcd427c, 87 },
    { 0, 0 },
    { 0xd90d427e, 77 },
    { 0xae3d427f, 33 },
    { 0x527d4280, 43 },
    { 0x75bd4281, 95 },
    { 0x952d4280, 95 },
    { 0, 0 },
    { 0xba8d4284, 75 },
    { 0, 0 },
    { 0x87ed4286, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90d428a, 61 },
    { 0x890d428b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630d428f, 95 },
    { 0, 0 },
    { 0xfd0d4291, 23 },
    { 0, 0 },
    { 0xddbd4293, 100 },
    { 0xedad4294, 99 },
    { 0xe04d4295, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4299, 34 },
    { 0, 0 },
    { 0x6b2d429b, 66 },
    { 0x9cad429c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14d429f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651d42a8, 81 },
    { 0x694d42a9, 85 },
    { 0, 0 },
    { 0x325d42ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63d42ae, 90 },
    { 0xb42d42af, 98 },
    { 0x359d42af, 76 },
    { 0, 0 },
    { 0xf9ed42b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34d42b6, 68 },
    { 0xbffd42b7, 61 },
    { 0xca3d42b8, 23 },
    { 0xfead42b8, 33 },
    { 0, 0 },
    { 0xc25d42bb, 95 },
    { 0x93ad42bc, 23 },
    { 0x6e6d42bc, 34 },
    { 0x360d42be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbed42c3, 83 },
    { 0, 0 },
    { 0xa45d42c5, 34 },
    { 0xbdd42c5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965d42ce, 81 },
    { 0x2d9d42ce, 98 },
    { 0xba8d42d0, 66 },
    { 0x9fed42cf, 90 },
    { 0xe21d42d1, 80 },
    { 0x5ad42d3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacd42d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9d42db, 33 },
    { 0, 0 },
    { 0xe18d42dd, 46 },
    { 0x95d42de, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bd42e3, 55 },
    { 0, 0 },
    { 0x626d42e5, 61 },
    { 0, 0 },
    { 0x214d42e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0dd42ec, 61 },
    { 0, 0 },
    { 0x8d8d42ee, 34 },
    { 0x7b8d42ef, 61 },
    { 0x222d42f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bd42f3, 46 },
    { 0x333d42f4, 90 },
    { 0xf22d42f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ed42f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41d42fc, 33 },
    { 0x212d42fd, 23 },
    { 0, 0 },
    { 0x2c0d42ff, 61 },
    { 0x3c4d4300, 23 },
    { 0xe16d4300, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bd4305, 24 },
    { 0, 0 },
    { 0x8d2d4307, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd430a, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9d431c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814d4321, 90 },
    { 0xec1d4322, 76 },
    { 0x315d4323, 55 },
    { 0x993d4323, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3d4327, 33 },
    { 0, 0 },
    { 0xc0dd4329, 46 },
    { 0xa8dd432a, 76 },
    { 0, 0 },
    { 0x37ad432c, 66 },
    { 0xac2d432c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x529d4330, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2add4334, 43 },
    { 0, 0 },
    { 0x12ad4336, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cd433b, 98 },
    { 0, 0 },
    { 0x31d433d, 66 },
    { 0xd3ad433e, 90 },
    { 0x28bd433e, 98 },
    { 0x243d433d, 100 },
    { 0xc5cd433f, 33 },
    { 0x506d433f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddd4345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3d4349, 78 },
    { 0xe59d434a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174d434f, 85 },
    { 0x509d434f, 61 },
    { 0x163d4351, 43 },
    { 0x43ed4350, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd4356, 98 },
    { 0xc96d4356, 84 },
    { 0, 0 },
    { 0x56ad4359, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2d435c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142d4362, 95 },
    { 0xbccd4362, 33 },
    { 0x55dd4364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dd4367, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d436a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaed436d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351d4374, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31d437e, 84 },
    { 0, 0 },
    { 0xfdd4380, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78d4383, 90 },
    { 0x242d4383, 76 },
    { 0, 0 },
    { 0x92ed4386, 88 },
    { 0xd9d4386, 38 },
    { 0xf99d4388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d438c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ad4394, 98 },
    { 0x74dd4395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6d4399, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add439d, 90 },
    { 0x515d439e, 23 },
    { 0x518d439d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226d43af, 95 },
    { 0xdb1d43b0, 33 },
    { 0x876d43b0, 40 },
    { 0xe41d43b2, 85 },
    { 0, 0 },
    { 0x4b2d43b4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1d43b7, 76 },
    { 0, 0 },
    { 0x713d43b9, 43 },
    { 0xc98d43ba, 66 },
    { 0x3bfd43b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499d43c5, 98 },
    { 0, 0 },
    { 0xc46d43c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ad43ca, 81 },
    { 0x2f5d43cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbad43ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x582d43d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d43d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcd43e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cd43e5, 85 },
    { 0x50bd43e6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4d43ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d43ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bd43f4, 46 },
    { 0x8f1d43f5, 46 },
    { 0x764d43f5, 87 },
    { 0xe47d43f6, 61 },
    { 0x284d43f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdd43fb, 90 },
    { 0xd2ad43fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccd4400, 95 },
    { 0xe12d4400, 33 },
    { 0xf21d4400, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8d4405, 78 },
    { 0x39ad4406, 23 },
    { 0x155d4407, 98 },
    { 0xa92d4408, 43 },
    { 0x3cfd4405, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfed440d, 100 },
    { 0x4f3d440d, 98 },
    { 0x5e4d440d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x958d4412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523d441a, 95 },
    { 0x6a5d441b, 76 },
    { 0xd72d441b, 90 },
    { 0, 0 },
    { 0x5f8d441e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x119d4421, 19 },
    { 0, 0 },
    { 0x254d4423, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8d4426, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40d442b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ed4432, 66 },
    { 0xfa1d4432, 23 },
    { 0x9b8d4432, 93 },
    { 0x55cd4435, 66 },
    { 0xa06d4436, 61 },
    { 0xf17d4437, 23 },
    { 0x34cd4435, 87 },
    { 0xe37d4439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8d4440, 46 },
    { 0x513d4440, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974d4447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cd444b, 98 },
    { 0x1bed444c, 66 },
    { 0x200d444c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d4453, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8d4456, 98 },
    { 0x441d4457, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd445c, 65 },
    { 0xf00d445d, 23 },
    { 0x310d445e, 33 },
    { 0x4f0d445f, 52 },
    { 0x31ed445c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd4463, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd4466, 95 },
    { 0, 0 },
    { 0x198d4468, 91 },
    { 0, 0 },
    { 0xf18d446a, 33 },
    { 0xd34d446b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ad446e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6d4475, 93 },
    { 0xcfad4475, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcad4479, 55 },
    { 0xd67d447a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2d447d, 90 },
    { 0x299d447e, 23 },
    { 0x8b2d447e, 95 },
    { 0x833d4480, 93 },
    { 0x1ed4481, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15d4485, 55 },
    { 0, 0 },
    { 0x560d4487, 17 },
    { 0, 0 },
    { 0x14d4489, 33 },
    { 0xd1fd448a, 88 },
    { 0x476d448b, 87 },
    { 0x57bd448b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6dd448f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcd4497, 98 },
    { 0x46ed4498, 23 },
    { 0xfd8d4498, 61 },
    { 0x62dd449a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x508d449d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204d44a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xebed44a6, 66 },
    { 0x90ad44a6, 100 },
    { 0x9bed44a8, 66 },
    { 0xd04d44a6, 95 },
    { 0, 0 },
    { 0xca1d44ab, 90 },
    { 0x3bcd44ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3d44b0, 23 },
    { 0xcf4d44b1, 95 },
    { 0x78d44b1, 77 },
    { 0x4aad44b3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ad44b9, 76 },
    { 0x579d44ba, 88 },
    { 0x24cd44b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aad44c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fd44d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x867d44d4, 33 },
    { 0x3d6d44d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8d44d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490d44dc, 98 },
    { 0x3fdd44dc, 33 },
    { 0x89cd44dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7d44e3, 55 },
    { 0x514d44e4, 33 },
    { 0x258d44e3, 61 },
    { 0x701d44e6, 90 },
    { 0xbbdd44e7, 46 },
    { 0, 0 },
    { 0x26ad44e9, 95 },
    { 0, 0 },
    { 0xccdd44eb, 55 },
    { 0x391d44ec, 95 },
    { 0xa5d44ec, 61 },
    { 0x7b2d44ed, 84 },
    { 0x92d44ee, 33 },
    { 0x1fd44ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08d44f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18d44f9, 34 },
    { 0xee0d44f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d44fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad4501, 100 },
    { 0, 0 },
    { 0x47d4503, 85 },
    { 0xfe2d4504, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75d4507, 61 },
    { 0x7ddd4508, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cd450c, 46 },
    { 0x168d450d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00d4515, 23 },
    { 0xbefd4516, 55 },
    { 0, 0 },
    { 0x5e5d4518, 61 },
    { 0x30dd4518, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d451c, 90 },
    { 0x801d451c, 98 },
    { 0xd56d451e, 98 },
    { 0x34ad451f, 90 },
    { 0xbb3d4520, 90 },
    { 0x146d4521, 91 },
    { 0xb78d4521, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x881d4525, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2d452b, 94 },
    { 0x2aed452c, 98 },
    { 0xb7dd452d, 90 },
    { 0x494d452b, 23 },
    { 0x2f9d452f, 98 },
    { 0x668d4530, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899d4537, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebd453a, 77 },
    { 0, 0 },
    { 0x2b2d453c, 33 },
    { 0xe8ad453d, 33 },
    { 0x650d453d, 91 },
    { 0xf06d453f, 93 },
    { 0, 0 },
    { 0x945d4541, 95 },
    { 0, 0 },
    { 0x8d9d4543, 90 },
    { 0x3fed4543, 23 },
    { 0xb87d4545, 100 },
    { 0x795d4544, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d454b, 98 },
    { 0, 0 },
    { 0x70bd454d, 36 },
    { 0x45fd454e, 23 },
    { 0x430d454d, 61 },
    { 0xc9cd454f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbed4553, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68d455a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ad455f, 95 },
    { 0xc39d4560, 23 },
    { 0, 0 },
    { 0x81cd4562, 33 },
    { 0x518d4562, 93 },
    { 0x5e4d4564, 91 },
    { 0x9e5d4565, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09d4568, 43 },
    { 0x57ad4569, 33 },
    { 0x921d4569, 40 },
    { 0x7b6d456a, 85 },
    { 0xe61d456b, 23 },
    { 0xaf3d456c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894d4572, 91 },
    { 0x82ad4573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abd4576, 61 },
    { 0x5ead4576, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ad457a, 76 },
    { 0xb0ed457a, 61 },
    { 0x84bd457b, 23 },
    { 0xd2ad457d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedd4586, 80 },
    { 0xc48d4587, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9d458c, 90 },
    { 0, 0 },
    { 0xaead458e, 90 },
    { 0xdffd458f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d4594, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9d459b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd45a3, 68 },
    { 0xadd45a4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3d45a7, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcd45aa, 43 },
    { 0x50ed45ab, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d45b1, 33 },
    { 0, 0 },
    { 0xa07d45b3, 90 },
    { 0, 0 },
    { 0xc9fd45b5, 43 },
    { 0x5a3d45b6, 34 },
    { 0x29fd45b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dd45c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bd45c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dd45c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5d45cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7d45cf, 55 },
    { 0x304d45d0, 98 },
    { 0x5efd45d1, 55 },
    { 0xa0dd45d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ad45d6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1d45da, 23 },
    { 0xd5ed45da, 98 },
    { 0x1e7d45dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d45df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d45e3, 67 },
    { 0xe40d45e4, 98 },
    { 0x7bfd45e4, 90 },
    { 0x2d8d45e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d45eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bd45ee, 76 },
    { 0xabd45ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbd45f3, 90 },
    { 0, 0 },
    { 0xd10d45f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edd45ff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9d4606, 23 },
    { 0x37cd4607, 90 },
    { 0xf87d4607, 61 },
    { 0xbfbd4609, 34 },
    { 0, 0 },
    { 0xe6ad460b, 90 },
    { 0, 0 },
    { 0xf47d460d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ed4610, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cd4614, 23 },
    { 0xa78d4614, 33 },
    { 0x305d4616, 23 },
    { 0x684d4614, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8d461a, 98 },
    { 0, 0 },
    { 0x9d7d461c, 43 },
    { 0xcc7d461d, 78 },
    { 0x5a2d461d, 87 },
    { 0x710d461f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261d462d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6d4633, 93 },
    { 0xad1d4634, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebd463d, 76 },
    { 0xffed463d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97d4647, 90 },
    { 0xb6fd4648, 34 },
    { 0xc80d4649, 33 },
    { 0, 0 },
    { 0xeafd464b, 99 },
    { 0, 0 },
    { 0xb13d464d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805d4651, 76 },
    { 0x427d4652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145d4658, 33 },
    { 0xcccd4659, 52 },
    { 0x57ad4658, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fd465f, 4 },
    { 0, 0 },
    { 0x9dad4661, 52 },
    { 0, 0 },
    { 0xd10d4663, 90 },
    { 0, 0 },
    { 0xcb0d4665, 90 },
    { 0, 0 },
    { 0xea3d4667, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd466a, 46 },
    { 0, 0 },
    { 0x7e9d466c, 99 },
    { 0x1f7d466c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x23d4670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69d4675, 90 },
    { 0, 0 },
    { 0x3c3d4677, 90 },
    { 0x749d4677, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2d467d, 55 },
    { 0x4c4d467e, 76 },
    { 0, 0 },
    { 0x885d4680, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43d4686, 61 },
    { 0, 0 },
    { 0xf4ad4688, 61 },
    { 0, 0 },
    { 0xbcfd468a, 34 },
    { 0x510d468b, 23 },
    { 0, 0 },
    { 0x35cd468d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d4690, 95 },
    { 0x5ccd4691, 33 },
    { 0xd0ed4691, 43 },
    { 0x6bd4692, 84 },
    { 0xe3fd4693, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736d4699, 76 },
    { 0xf52d4699, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d469f, 61 },
    { 0x21dd46a0, 33 },
    { 0x342d469f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd46a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x146d46a9, 23 },
    { 0, 0 },
    { 0x70dd46ab, 87 },
    { 0x511d46ac, 95 },
    { 0x28cd46ab, 33 },
    { 0x8a7d46ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d46b5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ad46c2, 46 },
    { 0x8e4d46c3, 61 },
    { 0, 0 },
    { 0x904d46c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aad46c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06d46cc, 80 },
    { 0x5dcd46cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8d46d5, 61 },
    { 0, 0 },
    { 0x581d46d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d46db, 23 },
    { 0, 0 },
    { 0x297d46dd, 55 },
    { 0xaa7d46de, 76 },
    { 0xc29d46df, 55 },
    { 0x8ffd46e0, 90 },
    { 0x752d46e0, 93 },
    { 0x81ad46e2, 95 },
    { 0x7a6d46e2, 33 },
    { 0x269d46e1, 95 },
    { 0x422d46e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd46ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d46ed, 43 },
    { 0, 0 },
    { 0xca5d46ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x714d46f2, 84 },
    { 0, 0 },
    { 0xd68d46f4, 65 },
    { 0x581d46f5, 9 },
    { 0x5bfd46f5, 61 },
    { 0x930d46f7, 90 },
    { 0, 0 },
    { 0x6ded46f9, 88 },
    { 0, 0 },
    { 0xfccd46fb, 33 },
    { 0x6a4d46fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244d4704, 33 },
    { 0x8ed4704, 95 },
    { 0xed3d4704, 68 },
    { 0xd0fd4707, 61 },
    { 0, 0 },
    { 0x1c5d4709, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bd470c, 61 },
    { 0x62ed470d, 34 },
    { 0x3c7d470d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7d4711, 97 },
    { 0xf9fd4712, 95 },
    { 0x5f3d4713, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510d4717, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21d471e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5d4722, 61 },
    { 0xa25d4722, 95 },
    { 0xcd6d4724, 90 },
    { 0x809d4724, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fd472a, 98 },
    { 0xbcd472b, 76 },
    { 0xfe0d472b, 33 },
    { 0, 0 },
    { 0x8a0d472e, 61 },
    { 0, 0 },
    { 0x5b6d4730, 77 },
    { 0, 0 },
    { 0x71ad4732, 77 },
    { 0xc4bd4733, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9d4737, 49 },
    { 0, 0 },
    { 0xf27d4739, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbd4740, 95 },
    { 0x4dfd4740, 76 },
    { 0xdb2d4742, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bd4747, 84 },
    { 0xf6ad4748, 55 },
    { 0xaf7d4749, 68 },
    { 0xd2ed474a, 55 },
    { 0, 0 },
    { 0x677d474c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838d4753, 77 },
    { 0xd71d4754, 87 },
    { 0x478d4755, 90 },
    { 0x758d4754, 61 },
    { 0xaebd4757, 23 },
    { 0xcaed4758, 78 },
    { 0x2add4758, 34 },
    { 0xac2d4759, 98 },
    { 0xb37d475b, 87 },
    { 0xe13d475b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ad4762, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5d4766, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbd4769, 33 },
    { 0, 0 },
    { 0xb2ed476b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711d476f, 34 },
    { 0x7fad4770, 95 },
    { 0xf3dd4770, 95 },
    { 0xe5cd4772, 76 },
    { 0x572d4773, 33 },
    { 0x112d4773, 98 },
    { 0xdcad476f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x961d4778, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7d477d, 76 },
    { 0, 0 },
    { 0x30dd477f, 88 },
    { 0x66fd4780, 33 },
    { 0x2d1d4781, 68 },
    { 0x55d4782, 91 },
    { 0x2bbd4782, 61 },
    { 0xe77d4782, 23 },
    { 0x5e9d4785, 76 },
    { 0, 0 },
    { 0xb0ed4787, 93 },
    { 0x1a5d4787, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d4790, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bd4794, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4d4798, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73d479c, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82d479f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fd47a3, 88 },
    { 0xaf2d47a4, 23 },
    { 0, 0 },
    { 0x5e6d47a6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07d47ac, 76 },
    { 0, 0 },
    { 0xb17d47ae, 90 },
    { 0xb15d47af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1add47b2, 90 },
    { 0xd94d47b3, 52 },
    { 0xc36d47b4, 78 },
    { 0x9dd47b5, 61 },
    { 0x9d6d47b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fd47bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59d47c3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d47cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8d47d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19d47d5, 77 },
    { 0x2dfd47d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ed47dc, 83 },
    { 0x1b3d47dd, 46 },
    { 0xf14d47dc, 56 },
    { 0xd84d47dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24d47e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x817d47e6, 91 },
    { 0x800d47e7, 90 },
    { 0xd40d47e7, 66 },
    { 0, 0 },
    { 0x51bd47ea, 90 },
    { 0xb25d47eb, 92 },
    { 0xdf0d47ec, 23 },
    { 0x1c6d47ec, 68 },
    { 0x220d47ee, 66 },
    { 0x43ed47ed, 76 },
    { 0xb6fd47ea, 61 },
    { 0x4d8d47f1, 61 },
    { 0x4d0d47f0, 23 },
    { 0x5f7d47f3, 33 },
    { 0, 0 },
    { 0xa3dd47f5, 23 },
    { 0xeb0d47f6, 61 },
    { 0x4ed47f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3d47fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36d47fe, 93 },
    { 0x4c6d47ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3d4803, 55 },
    { 0, 0 },
    { 0x246d4805, 55 },
    { 0, 0 },
    { 0xfd7d4807, 52 },
    { 0x89bd4807, 33 },
    { 0x9ccd4807, 90 },
    { 0x1e1d480a, 46 },
    { 0xcc3d480b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d480e, 61 },
    { 0, 0 },
    { 0x1fdd4810, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2d4813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x499d4816, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d481d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cd4820, 33 },
    { 0, 0 },
    { 0x54ad4822, 61 },
    { 0, 0 },
    { 0xd08d4824, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540d4828, 93 },
    { 0x78bd4829, 46 },
    { 0x873d482a, 95 },
    { 0xbcd4829, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x711d482e, 33 },
    { 0x349d482f, 95 },
    { 0x549d482f, 84 },
    { 0x3ccd4830, 33 },
    { 0, 0 },
    { 0x915d4833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadd4836, 77 },
    { 0x1dbd4836, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6d4840, 95 },
    { 0, 0 },
    { 0xf50d4842, 95 },
    { 0, 0 },
    { 0x9f5d4844, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dd4847, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8d484a, 76 },
    { 0x728d484b, 90 },
    { 0xe21d484b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0d484f, 53 },
    { 0x9b5d484f, 68 },
    { 0xe24d4851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3d4856, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ed485a, 96 },
    { 0x174d485a, 55 },
    { 0x2ecd485c, 23 },
    { 0x382d485d, 95 },
    { 0x37cd485d, 61 },
    { 0xcfbd485d, 90 },
    { 0xaccd485e, 33 },
    { 0xc14d485d, 84 },
    { 0xb57d4862, 93 },
    { 0xcc8d4863, 93 },
    { 0, 0 },
    { 0x43ad4865, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3d4868, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x641d486b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fd4874, 95 },
    { 0xe7ed4875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ed487e, 33 },
    { 0, 0 },
    { 0x404d4880, 93 },
    { 0xb32d4881, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bd4884, 95 },
    { 0x4ced4885, 61 },
    { 0xfbed4886, 97 },
    { 0xe29d4886, 90 },
    { 0xf9ad4888, 78 },
    { 0xe59d4889, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72d4891, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331d4899, 85 },
    { 0xb1d489a, 81 },
    { 0x4ded489b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed489e, 98 },
    { 0x188d489f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd48a2, 61 },
    { 0x5fed48a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4d48a6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dd48aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81d48ae, 90 },
    { 0x475d48af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d48b5, 52 },
    { 0x8d9d48b6, 33 },
    { 0xa49d48b7, 34 },
    { 0, 0 },
    { 0x7f5d48b9, 76 },
    { 0, 0 },
    { 0x362d48bb, 90 },
    { 0, 0 },
    { 0x23ad48bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0d48c5, 90 },
    { 0xa22d48c5, 52 },
    { 0x62bd48c5, 91 },
    { 0x537d48c7, 61 },
    { 0x293d48c9, 33 },
    { 0xd61d48c9, 61 },
    { 0x850d48c6, 43 },
    { 0x12ad48c8, 34 },
    { 0xa6fd48cd, 55 },
    { 0x311d48ce, 23 },
    { 0x394d48cc, 98 },
    { 0x6e7d48d0, 93 },
    { 0xfb2d48d1, 52 },
    { 0x70d48d2, 61 },
    { 0x53fd48d2, 55 },
    { 0x764d48d1, 34 },
    { 0x471d48d5, 76 },
    { 0xd9bd48d2, 99 },
    { 0x5b8d48cb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bd48db, 76 },
    { 0xb5fd48dc, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x997d48df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d48e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fd48e8, 90 },
    { 0xf60d48e9, 56 },
    { 0xe1bd48ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74d48ed, 33 },
    { 0, 0 },
    { 0x7bd48ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d48f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2d48f8, 90 },
    { 0x3bbd48f9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d48fe, 23 },
    { 0xf6ed48ff, 23 },
    { 0, 0 },
    { 0x6bed4901, 55 },
    { 0x92ad4902, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25d4906, 23 },
    { 0xe6fd4907, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7d490b, 90 },
    { 0x9bd490c, 87 },
    { 0, 0 },
    { 0x83dd490e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d4914, 23 },
    { 0xc74d4915, 90 },
    { 0, 0 },
    { 0x1a8d4917, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x1add491a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420d4923, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44d492d, 61 },
    { 0xe69d492e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29d4939, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdd493f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d4942, 23 },
    { 0, 0 },
    { 0x4a9d4944, 34 },
    { 0x42cd4945, 52 },
    { 0x6e5d4944, 95 },
    { 0x639d4947, 46 },
    { 0, 0 },
    { 0xa5bd4949, 95 },
    { 0x97ed494a, 56 },
    { 0, 0 },
    { 0x122d494c, 95 },
    { 0x259d494c, 87 },
    { 0xdc2d494d, 68 },
    { 0x35dd494f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ed4954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29d495a, 61 },
    { 0, 0 },
    { 0x45fd495c, 61 },
    { 0x446d495d, 61 },
    { 0, 0 },
    { 0xc80d495f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3d496e, 23 },
    { 0xa37d496f, 42 },
    { 0x479d496e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bd4973, 43 },
    { 0xa03d4973, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cd4979, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365d497e, 23 },
    { 0x505d497f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0d4983, 90 },
    { 0xc57d4984, 66 },
    { 0x71dd4985, 91 },
    { 0x826d4986, 87 },
    { 0x676d4986, 33 },
    { 0xd64d4988, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd498e, 98 },
    { 0x94fd498f, 98 },
    { 0xa0ad4990, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaad4997, 28 },
    { 0, 0 },
    { 0x350d4999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x156d499c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d49a1, 52 },
    { 0x33ed49a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56d49a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5d49b4, 85 },
    { 0, 0 },
    { 0x208d49b6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819d49ba, 65 },
    { 0xc0ad49ba, 52 },
    { 0xf87d49bb, 34 },
    { 0x7e4d49bd, 33 },
    { 0xf63d49be, 98 },
    { 0x39ed49be, 34 },
    { 0xa15d49bf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271d49cc, 68 },
    { 0, 0 },
    { 0x6d6d49ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576d49d3, 34 },
    { 0x574d49d4, 95 },
    { 0, 0 },
    { 0xf68d49d6, 43 },
    { 0, 0 },
    { 0x46cd49d8, 91 },
    { 0x582d49d9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548d49e2, 61 },
    { 0x401d49e3, 61 },
    { 0xea7d49e4, 61 },
    { 0, 0 },
    { 0x583d49e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86d49ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d49ed, 76 },
    { 0x814d49ee, 23 },
    { 0x459d49ef, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd49f4, 33 },
    { 0xf7cd49f4, 23 },
    { 0x82cd49f4, 98 },
    { 0x466d49f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd49fb, 66 },
    { 0, 0 },
    { 0x329d49fd, 98 },
    { 0x44ed49fd, 77 },
    { 0x846d49fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0d4a04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78d4a09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945d4a13, 55 },
    { 0, 0 },
    { 0x634d4a15, 61 },
    { 0xaded4a16, 90 },
    { 0x281d4a17, 61 },
    { 0x816d4a18, 90 },
    { 0xc27d4a17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x131d4a1c, 76 },
    { 0x94fd4a1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2d4a2b, 90 },
    { 0x889d4a2c, 23 },
    { 0x5bcd4a2d, 75 },
    { 0x422d4a2d, 91 },
    { 0xaa1d4a2d, 55 },
    { 0x14fd4a2e, 19 },
    { 0xd7bd4a31, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487d4a36, 91 },
    { 0x5bfd4a37, 66 },
    { 0x41d4a37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d4a43, 55 },
    { 0, 0 },
    { 0xfd3d4a45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd4a4a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x42dd4a4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fd4a51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6d4a54, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bd4a5a, 76 },
    { 0x26fd4a5a, 33 },
    { 0xbb7d4a5c, 93 },
    { 0, 0 },
    { 0x6dcd4a5e, 84 },
    { 0xb67d4a5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93d4a6b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728d4a70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0d4a74, 98 },
    { 0, 0 },
    { 0x76bd4a76, 23 },
    { 0, 0 },
    { 0x7dcd4a78, 55 },
    { 0xf3fd4a79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fd4a7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccd4a88, 90 },
    { 0x8d7d4a88, 55 },
    { 0, 0 },
    { 0x4aed4a8b, 95 },
    { 0x6c6d4a8c, 77 },
    { 0x85cd4a8b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427d4a92, 98 },
    { 0xd05d4a93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fd4a97, 78 },
    { 0x654d4a97, 80 },
    { 0xe0ad4a99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d4a9e, 46 },
    { 0xc38d4a9f, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255d4aaa, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dd4ab2, 98 },
    { 0x131d4ab2, 76 },
    { 0, 0 },
    { 0xcabd4ab5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d4aba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d4ac1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdd4ac4, 33 },
    { 0x9a1d4ac5, 75 },
    { 0xc74d4ac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0d4acb, 23 },
    { 0x53d4acb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0d4ad4, 69 },
    { 0x8ffd4ad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d4ad8, 90 },
    { 0xbf0d4ad9, 55 },
    { 0, 0 },
    { 0x7aed4adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17d4ade, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28d4ae8, 84 },
    { 0xa13d4ae9, 46 },
    { 0, 0 },
    { 0x36d4aeb, 34 },
    { 0x1f0d4aec, 34 },
    { 0xcefd4aed, 23 },
    { 0, 0 },
    { 0xbb4d4aef, 43 },
    { 0, 0 },
    { 0x185d4af1, 99 },
    { 0x259d4af2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edd4af8, 95 },
    { 0x3c9d4af9, 78 },
    { 0, 0 },
    { 0x61cd4afb, 23 },
    { 0x7f3d4afb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82d4aff, 43 },
    { 0xa85d4aff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0d4b04, 77 },
    { 0, 0 },
    { 0x87d4b06, 95 },
    { 0x399d4b07, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51d4b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d4b0d, 95 },
    { 0x898d4b0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ad4b11, 77 },
    { 0x5bfd4b12, 99 },
    { 0xe1fd4b13, 61 },
    { 0x53ed4b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56d4b19, 33 },
    { 0, 0 },
    { 0xa28d4b1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bd4b1f, 49 },
    { 0xfffd4b20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2d4b23, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3d4b26, 17 },
    { 0xcbcd4b27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d4b2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ed4b2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dfd4b34, 33 },
    { 0, 0 },
    { 0xa53d4b36, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9d4b3a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31d4b3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdd4b47, 90 },
    { 0xb5ad4b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239d4b4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40d4b4f, 83 },
    { 0, 0 },
    { 0x632d4b51, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8d4b57, 69 },
    { 0x721d4b58, 98 },
    { 0x478d4b59, 55 },
    { 0, 0 },
    { 0x15ed4b5b, 34 },
    { 0, 0 },
    { 0x19bd4b5d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d4b65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10d4b68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804d4b6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cd4b77, 23 },
    { 0xaded4b78, 33 },
    { 0, 0 },
    { 0xe40d4b7a, 98 },
    { 0x956d4b7a, 84 },
    { 0x7fbd4b7b, 34 },
    { 0x193d4b7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9d4b80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d4b83, 81 },
    { 0x484d4b83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fd4b87, 67 },
    { 0x109d4b87, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ed4b90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8d4b99, 70 },
    { 0x506d4b9a, 90 },
    { 0x41cd4b9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ad4ba2, 33 },
    { 0xecad4ba3, 61 },
    { 0xb7cd4ba4, 67 },
    { 0x34fd4ba5, 90 },
    { 0xa9ad4ba6, 33 },
    { 0, 0 },
    { 0xe3ad4ba8, 90 },
    { 0x742d4ba8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7d4bac, 90 },
    { 0x709d4bac, 55 },
    { 0x70dd4bad, 46 },
    { 0x7c2d4bad, 43 },
    { 0x826d4bb0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d4bb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fd4bba, 90 },
    { 0x6bad4bbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552d4bc2, 76 },
    { 0x377d4bc3, 34 },
    { 0x4cad4bc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4d4bc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eed4bcc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94d4bd8, 33 },
    { 0x669d4bd9, 23 },
    { 0x51ed4bda, 85 },
    { 0x2c9d4bdb, 95 },
    { 0xf7ad4bdc, 85 },
    { 0, 0 },
    { 0xfad4bde, 91 },
    { 0x9a2d4bdf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcd4be3, 80 },
    { 0, 0 },
    { 0xfd3d4be5, 98 },
    { 0x227d4be5, 23 },
    { 0, 0 },
    { 0xfd7d4be8, 61 },
    { 0, 0 },
    { 0x768d4bea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d4bf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147d4bf5, 61 },
    { 0x843d4bf5, 66 },
    { 0x4abd4bf5, 95 },
    { 0x9dbd4bf8, 33 },
    { 0x8b0d4bf8, 33 },
    { 0x65bd4bf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x587d4bfd, 83 },
    { 0, 0 },
    { 0xe1bd4bff, 87 },
    { 0x9fad4c00, 23 },
    { 0x596d4c00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d4c04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cd4c09, 90 },
    { 0, 0 },
    { 0x1cdd4c0b, 66 },
    { 0, 0 },
    { 0xe32d4c0d, 61 },
    { 0, 0 },
    { 0x3b7d4c0f, 95 },
    { 0x457d4c10, 95 },
    { 0x1e7d4c11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840d4c19, 90 },
    { 0xd7fd4c1a, 95 },
    { 0x502d4c19, 76 },
    { 0x6b1d4c1c, 38 },
    { 0x74fd4c19, 85 },
    { 0x2e4d4c1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cd4c21, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330d4c26, 69 },
    { 0x5a2d4c26, 90 },
    { 0x24ad4c28, 98 },
    { 0x8dcd4c29, 98 },
    { 0x9d4c27, 55 },
    { 0xdd2d4c28, 70 },
    { 0xec8d4c2c, 95 },
    { 0x8e0d4c26, 90 },
    { 0x6a8d4c2e, 98 },
    { 0x6e2d4c2f, 61 },
    { 0x529d4c30, 88 },
    { 0xa25d4c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d4c34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97d4c39, 76 },
    { 0x107d4c3a, 95 },
    { 0x3d9d4c3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6d4c40, 81 },
    { 0x81cd4c41, 23 },
    { 0x154d4c42, 95 },
    { 0xc1ad4c43, 76 },
    { 0x201d4c42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d4c48, 52 },
    { 0x2e8d4c49, 90 },
    { 0x5a1d4c48, 23 },
    { 0xfdcd4c4b, 55 },
    { 0xfdcd4c4c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3d4c51, 23 },
    { 0x2e8d4c51, 86 },
    { 0x657d4c53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d4c57, 17 },
    { 0x414d4c57, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccd4c5d, 95 },
    { 0, 0 },
    { 0xa51d4c5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3d4c62, 90 },
    { 0xa41d4c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d4c73, 48 },
    { 0, 0 },
    { 0x93bd4c75, 90 },
    { 0xdd9d4c75, 98 },
    { 0x622d4c75, 100 },
    { 0xf4ad4c78, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1d4c7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ad4c81, 43 },
    { 0xb58d4c81, 83 },
    { 0xf66d4c83, 90 },
    { 0x51cd4c83, 90 },
    { 0x847d4c85, 95 },
    { 0, 0 },
    { 0xc8fd4c87, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d4c8e, 61 },
    { 0xf69d4c8f, 67 },
    { 0x7edd4c8e, 98 },
    { 0xf0ed4c8f, 23 },
    { 0x60cd4c91, 23 },
    { 0, 0 },
    { 0x50d4c94, 98 },
    { 0x9d7d4c95, 95 },
    { 0x4e7d4c95, 95 },
    { 0x10d4c97, 43 },
    { 0x5ebd4c97, 33 },
    { 0x980d4c99, 98 },
    { 0x381d4c9a, 67 },
    { 0x53bd4c9a, 68 },
    { 0xb9cd4c9a, 61 },
    { 0x299d4c97, 100 },
    { 0xd0bd4c9e, 85 },
    { 0, 0 },
    { 0xabdd4ca0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8d4ca4, 93 },
    { 0, 0 },
    { 0xe6dd4ca6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343d4cab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148d4cb2, 55 },
    { 0xc5dd4cb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd4cb6, 100 },
    { 0x1e8d4cb7, 98 },
    { 0xda7d4cb8, 95 },
    { 0xb3cd4cb9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ed4cc3, 78 },
    { 0x2ffd4cc4, 88 },
    { 0xe7ad4cc5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dd4cc9, 87 },
    { 0, 0 },
    { 0xd65d4ccb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3d4cd1, 98 },
    { 0x371d4cd2, 24 },
    { 0, 0 },
    { 0x143d4cd4, 23 },
    { 0, 0 },
    { 0x9bdd4cd6, 33 },
    { 0x315d4cd7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fd4cda, 23 },
    { 0x245d4cdb, 98 },
    { 0xd2d4cda, 90 },
    { 0, 0 },
    { 0x333d4cde, 98 },
    { 0x41ad4cdf, 17 },
    { 0x2dd4ce0, 93 },
    { 0xc43d4cde, 55 },
    { 0x2a5d4ce2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ed4ce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadd4cf0, 33 },
    { 0, 0 },
    { 0x858d4cf2, 34 },
    { 0xd36d4cf3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6d4cf8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d4cfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8d4d06, 91 },
    { 0x8a0d4d06, 90 },
    { 0x80ad4d08, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5d4d0c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441d4d12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d4d15, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x58d4d18, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384d4d1c, 61 },
    { 0x5b5d4d1c, 91 },
    { 0x947d4d1e, 46 },
    { 0xd09d4d1e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3d4d23, 34 },
    { 0, 0 },
    { 0x63ed4d25, 95 },
    { 0xfc2d4d26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x721d4d29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bd4d2e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x774d4d31, 61 },
    { 0x6b0d4d32, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ced4d39, 33 },
    { 0x104d4d3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd4d3f, 90 },
    { 0xaccd4d3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263d4d44, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690d4d48, 76 },
    { 0xf93d4d49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187d4d50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d4d54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fd4d58, 33 },
    { 0x28cd4d59, 33 },
    { 0, 0 },
    { 0xf35d4d5b, 90 },
    { 0x826d4d5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d4d60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcced4d63, 90 },
    { 0x953d4d63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87d4d68, 76 },
    { 0xd63d4d69, 46 },
    { 0xba6d4d6a, 78 },
    { 0x483d4d6b, 90 },
    { 0x74fd4d6b, 95 },
    { 0x5c1d4d6d, 84 },
    { 0xa55d4d6d, 93 },
    { 0, 0 },
    { 0xf02d4d70, 33 },
    { 0, 0 },
    { 0x65bd4d72, 52 },
    { 0, 0 },
    { 0x32d4d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3d4d78, 33 },
    { 0x935d4d79, 61 },
    { 0xe6cd4d7a, 76 },
    { 0xf47d4d78, 55 },
    { 0xd4ed4d7c, 33 },
    { 0xc6cd4d79, 98 },
    { 0x72fd4d7d, 43 },
    { 0xd6ad4d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303d4d84, 33 },
    { 0x3cfd4d84, 43 },
    { 0, 0 },
    { 0xfffd4d87, 93 },
    { 0xf78d4d87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d4d8b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddd4d90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fd4d93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fd4d98, 23 },
    { 0x6aed4d99, 43 },
    { 0, 0 },
    { 0xd16d4d9b, 90 },
    { 0, 0 },
    { 0xa58d4d9d, 61 },
    { 0xc7dd4d9e, 66 },
    { 0x6c2d4d9e, 91 },
    { 0x65ed4d9e, 98 },
    { 0x491d4da1, 23 },
    { 0, 0 },
    { 0x301d4da3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ed4da9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520d4dad, 34 },
    { 0x372d4dae, 33 },
    { 0xee5d4daf, 33 },
    { 0x9dcd4db0, 90 },
    { 0x9bed4db0, 87 },
    { 0xae3d4db1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428d4db9, 33 },
    { 0, 0 },
    { 0xa7d4dbb, 52 },
    { 0xa9dd4dbc, 65 },
    { 0, 0 },
    { 0xae3d4dbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdd4dc3, 95 },
    { 0x808d4dc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dad4dc9, 98 },
    { 0xac1d4dca, 33 },
    { 0xf16d4dcb, 61 },
    { 0xa31d4dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd4dd2, 33 },
    { 0, 0 },
    { 0xa67d4dd4, 34 },
    { 0x652d4dd4, 95 },
    { 0, 0 },
    { 0x927d4dd7, 95 },
    { 0x472d4dd8, 23 },
    { 0x3ffd4dd8, 80 },
    { 0xb16d4dda, 90 },
    { 0xa57d4dd7, 98 },
    { 0, 0 },
    { 0xf7dd4ddd, 91 },
    { 0x731d4dde, 87 },
    { 0x26ad4ddf, 33 },
    { 0, 0 },
    { 0xeffd4de1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ed4de6, 23 },
    { 0xb80d4de7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5d4dea, 61 },
    { 0, 0 },
    { 0x11d4dec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d4df6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144d4dfa, 55 },
    { 0xdc0d4dfb, 100 },
    { 0x2e4d4dfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55d4dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d4e09, 90 },
    { 0x937d4e09, 87 },
    { 0x322d4e09, 33 },
    { 0x4eed4e0c, 33 },
    { 0xe23d4e0a, 36 },
    { 0x698d4e0e, 98 },
    { 0xa6ad4e0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948d4e16, 98 },
    { 0, 0 },
    { 0x995d4e18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d4e24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260d4e29, 90 },
    { 0x942d4e2a, 61 },
    { 0xd69d4e2b, 98 },
    { 0x47bd4e2b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ed4e31, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1d4e34, 33 },
    { 0x325d4e34, 76 },
    { 0, 0 },
    { 0x462d4e37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17d4e3c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279d4e40, 99 },
    { 0x180d4e40, 76 },
    { 0, 0 },
    { 0x5cd4e43, 61 },
    { 0xaf1d4e44, 33 },
    { 0, 0 },
    { 0x60ed4e46, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x334d4e49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24d4e4d, 99 },
    { 0, 0 },
    { 0x8aad4e4f, 98 },
    { 0, 0 },
    { 0xfe4d4e51, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570d4e56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbd4e5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dd4e5f, 46 },
    { 0, 0 },
    { 0x2a4d4e61, 66 },
    { 0, 0 },
    { 0x583d4e63, 90 },
    { 0xd70d4e64, 23 },
    { 0x29bd4e65, 64 },
    { 0x174d4e64, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aed4e69, 95 },
    { 0x71dd4e69, 75 },
    { 0x8b8d4e69, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56d4e6e, 94 },
    { 0x318d4e6f, 69 },
    { 0xe0ed4e6f, 23 },
    { 0x3b3d4e71, 100 },
    { 0x28ad4e72, 33 },
    { 0x3b8d4e73, 76 },
    { 0x2d4e70, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad4e77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x528d4e7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40d4e7f, 23 },
    { 0xe9fd4e80, 90 },
    { 0, 0 },
    { 0xee1d4e82, 95 },
    { 0, 0 },
    { 0xb4cd4e84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ed4e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ad4e8d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4d4e90, 61 },
    { 0, 0 },
    { 0xa9ed4e92, 33 },
    { 0, 0 },
    { 0xdddd4e94, 98 },
    { 0x813d4e94, 23 },
    { 0, 0 },
    { 0x2aed4e97, 8 },
    { 0x33bd4e98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876d4ea0, 95 },
    { 0xae9d4ea1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x125d4ea4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510d4eac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7d4eb6, 98 },
    { 0, 0 },
    { 0x621d4eb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd4ebd, 90 },
    { 0x6e2d4ebe, 33 },
    { 0xa79d4ebf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d4ec2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x112d4ec5, 90 },
    { 0x8fbd4ec6, 90 },
    { 0x65ed4ec6, 90 },
    { 0xbbad4ec8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852d4ecf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d4ed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cd4eda, 90 },
    { 0, 0 },
    { 0xa89d4edc, 98 },
    { 0xf1d4edc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcd4ee4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d4ee7, 90 },
    { 0x5aad4ee8, 46 },
    { 0x111d4ee9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4d4eef, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ed4ef5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5d4efd, 66 },
    { 0, 0 },
    { 0x798d4eff, 93 },
    { 0xf0ad4eff, 23 },
    { 0x5ddd4f01, 34 },
    { 0xe7bd4f02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82d4f05, 55 },
    { 0, 0 },
    { 0xa43d4f07, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbd4f0d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed4f15, 93 },
    { 0x87fd4f16, 23 },
    { 0x63cd4f15, 61 },
    { 0xe17d4f18, 90 },
    { 0x5b7d4f19, 33 },
    { 0xfd6d4f18, 95 },
    { 0xc11d4f16, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522d4f1f, 98 },
    { 0, 0 },
    { 0x83bd4f21, 98 },
    { 0x82cd4f22, 33 },
    { 0x18ed4f22, 33 },
    { 0x609d4f22, 90 },
    { 0xbc3d4f24, 46 },
    { 0x4f0d4f26, 95 },
    { 0, 0 },
    { 0xf37d4f28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970d4f2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x744d4f31, 84 },
    { 0, 0 },
    { 0x90ad4f33, 66 },
    { 0x211d4f33, 43 },
    { 0x371d4f33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d4f39, 34 },
    { 0x40ed4f39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cd4f3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1d4f41, 90 },
    { 0xc2bd4f42, 77 },
    { 0xd6bd4f42, 93 },
    { 0xdf6d4f42, 34 },
    { 0x813d4f45, 90 },
    { 0x96bd4f44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143d4f52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d4f57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174d4f5d, 90 },
    { 0, 0 },
    { 0x48ad4f5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6d4f65, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80d4f70, 61 },
    { 0, 0 },
    { 0xa2cd4f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd4f75, 33 },
    { 0xbb5d4f75, 55 },
    { 0x666d4f77, 96 },
    { 0x2afd4f78, 95 },
    { 0x418d4f79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bd4f7c, 95 },
    { 0x40d4f7d, 85 },
    { 0, 0 },
    { 0x36ed4f7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729d4f87, 34 },
    { 0x146d4f87, 95 },
    { 0x1a7d4f89, 99 },
    { 0xdfd4f8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6d4f8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628d4f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cd4f96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x128d4f99, 55 },
    { 0, 0 },
    { 0xe44d4f9b, 33 },
    { 0, 0 },
    { 0x3e3d4f9d, 78 },
    { 0xfe3d4f9e, 46 },
    { 0xbbad4f9f, 90 },
    { 0xc1fd4fa0, 61 },
    { 0xb53d4fa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced4fa8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35d4fad, 90 },
    { 0, 0 },
    { 0x162d4faf, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919d4fb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718d4fb9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ed4fbf, 98 },
    { 0, 0 },
    { 0xa04d4fc1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd4fd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd4fd5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5d4fd9, 93 },
    { 0xee0d4fda, 98 },
    { 0x260d4fdb, 99 },
    { 0, 0 },
    { 0x50bd4fdd, 93 },
    { 0, 0 },
    { 0xac0d4fdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cd4fe3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xead4fe9, 90 },
    { 0x365d4fea, 33 },
    { 0, 0 },
    { 0x89ad4fec, 55 },
    { 0xec3d4fed, 61 },
    { 0xc29d4fec, 90 },
    { 0x989d4fef, 23 },
    { 0x97d4fee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ed4ff5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbd4ff8, 99 },
    { 0x517d4ff8, 55 },
    { 0x957d4ffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad4ffd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72d5001, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264d5006, 34 },
    { 0xeadd5007, 55 },
    { 0x91dd5008, 43 },
    { 0, 0 },
    { 0x5bed500a, 76 },
    { 0x12dd500b, 46 },
    { 0x356d500c, 93 },
    { 0x555d500c, 23 },
    { 0x1acd500e, 33 },
    { 0, 0 },
    { 0x2dd5010, 98 },
    { 0x6fd5011, 78 },
    { 0x68d5012, 100 },
    { 0xbeed5013, 61 },
    { 0xe28d5014, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x892d5017, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x672d501a, 98 },
    { 0xb87d501a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12d501e, 52 },
    { 0x193d501e, 90 },
    { 0, 0 },
    { 0x413d5021, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cd5025, 23 },
    { 0, 0 },
    { 0x67fd5027, 95 },
    { 0, 0 },
    { 0x3fd5029, 61 },
    { 0x5fdd502a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2d5032, 61 },
    { 0, 0 },
    { 0xf8dd5034, 23 },
    { 0x489d5035, 95 },
    { 0, 0 },
    { 0x241d5037, 61 },
    { 0x351d5037, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1d503b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6d5041, 55 },
    { 0x6b5d5042, 93 },
    { 0, 0 },
    { 0xde7d5044, 96 },
    { 0, 0 },
    { 0x41fd5046, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ad5049, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d504d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92d5057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ed505a, 33 },
    { 0x583d505b, 95 },
    { 0xd40d505a, 61 },
    { 0x1dcd505d, 23 },
    { 0, 0 },
    { 0xaed505f, 34 },
    { 0, 0 },
    { 0x54fd5061, 52 },
    { 0x7f7d5061, 99 },
    { 0x238d5063, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd5069, 61 },
    { 0x3c7d5069, 100 },
    { 0, 0 },
    { 0xf1cd506c, 65 },
    { 0x5f5d506d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329d5074, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5d5078, 23 },
    { 0x3cd5079, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37d507c, 23 },
    { 0x35ed507c, 76 },
    { 0xc9ad507c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x369d5081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37d5085, 55 },
    { 0xcc3d5086, 98 },
    { 0, 0 },
    { 0x2bd5088, 33 },
    { 0, 0 },
    { 0x5fbd508a, 98 },
    { 0xae1d508a, 66 },
    { 0, 0 },
    { 0x9f9d508d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7d5092, 33 },
    { 0, 0 },
    { 0x469d5094, 33 },
    { 0, 0 },
    { 0xf30d5096, 61 },
    { 0xc04d5097, 76 },
    { 0xa10d5098, 76 },
    { 0x421d5096, 55 },
    { 0xffdd5097, 33 },
    { 0, 0 },
    { 0xa10d509c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dd50a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820d50ac, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61d50b3, 55 },
    { 0x3a2d50b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed50b8, 90 },
    { 0, 0 },
    { 0x475d50ba, 33 },
    { 0xf4d50bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x163d50be, 95 },
    { 0xab3d50bf, 65 },
    { 0xca6d50c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1d50c3, 95 },
    { 0, 0 },
    { 0xd9bd50c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x729d50c8, 96 },
    { 0xa1ed50c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bad50cc, 98 },
    { 0xfc7d50cc, 33 },
    { 0x2ddd50ce, 90 },
    { 0x3bbd50cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7d50d4, 43 },
    { 0x886d50d5, 33 },
    { 0xb04d50d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fd50da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x305d50df, 55 },
    { 0x15cd50e0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ad50e3, 76 },
    { 0, 0 },
    { 0x34ed50e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8d50e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2d50f1, 66 },
    { 0x8b4d50f1, 23 },
    { 0x7f9d50f3, 95 },
    { 0xcced50f4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885d50f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0d50fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x196d50ff, 43 },
    { 0x5dd5100, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ad5107, 96 },
    { 0x447d5107, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abd510e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59d5113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x449d5116, 90 },
    { 0xadcd5116, 69 },
    { 0xb0dd5118, 61 },
    { 0x587d5119, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fd511c, 55 },
    { 0, 0 },
    { 0x142d511e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3d5122, 99 },
    { 0, 0 },
    { 0x215d5124, 42 },
    { 0x25d5124, 61 },
    { 0xf5d5124, 67 },
    { 0x538d5126, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499d512c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320d5134, 100 },
    { 0xe64d5134, 23 },
    { 0x9c5d5136, 93 },
    { 0, 0 },
    { 0xde2d5138, 40 },
    { 0x1edd5138, 23 },
    { 0x7dbd5138, 68 },
    { 0x8acd513a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d5146, 55 },
    { 0x6d5d5146, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ad514a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d514e, 43 },
    { 0xec8d514f, 78 },
    { 0, 0 },
    { 0x9dfd5151, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650d515c, 83 },
    { 0x6c8d515c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfd5168, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdd5171, 55 },
    { 0xdfed5171, 93 },
    { 0xd90d5173, 40 },
    { 0x2b7d5173, 95 },
    { 0x8c7d5172, 61 },
    { 0x472d5171, 90 },
    { 0xdf7d5177, 43 },
    { 0x303d5174, 43 },
    { 0, 0 },
    { 0x831d517a, 98 },
    { 0xc6fd517b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cd5180, 98 },
    { 0x4f4d5180, 90 },
    { 0x882d5181, 33 },
    { 0, 0 },
    { 0xd34d5184, 61 },
    { 0xf8bd5184, 76 },
    { 0x8c4d5186, 37 },
    { 0x6d8d5187, 55 },
    { 0x138d5188, 100 },
    { 0x237d5189, 90 },
    { 0x7a6d518a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886d5190, 61 },
    { 0x55ad5190, 55 },
    { 0xa87d5192, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59d5195, 90 },
    { 0x10bd5196, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x325d5199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01d519c, 93 },
    { 0, 0 },
    { 0x4e2d519e, 90 },
    { 0x89ed519e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6d51a3, 61 },
    { 0, 0 },
    { 0x18dd51a5, 55 },
    { 0x500d51a6, 95 },
    { 0xf21d51a6, 90 },
    { 0, 0 },
    { 0x7f7d51a9, 61 },
    { 0x38ad51aa, 61 },
    { 0xfb3d51aa, 95 },
    { 0x9a5d51ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07d51af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46d51b2, 23 },
    { 0x341d51b3, 61 },
    { 0, 0 },
    { 0xe48d51b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedad51b9, 23 },
    { 0xa6ad51b9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d51bf, 61 },
    { 0, 0 },
    { 0xe51d51c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x636d51c4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72d51c7, 93 },
    { 0, 0 },
    { 0xcd3d51c9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x504d51cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cd51cf, 33 },
    { 0xd60d51d0, 33 },
    { 0xdc7d51d1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd51d5, 80 },
    { 0x538d51d5, 99 },
    { 0x197d51d7, 95 },
    { 0xdacd51d8, 66 },
    { 0, 0 },
    { 0x96bd51da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810d51de, 98 },
    { 0x188d51df, 95 },
    { 0x18ad51e0, 52 },
    { 0xa59d51df, 98 },
    { 0x1dbd51df, 23 },
    { 0x158d51e1, 98 },
    { 0, 0 },
    { 0x412d51e5, 61 },
    { 0x5cad51e5, 33 },
    { 0x753d51e7, 23 },
    { 0, 0 },
    { 0x72fd51e9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead51f1, 33 },
    { 0xc5cd51f2, 90 },
    { 0xddcd51f3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d51ff, 95 },
    { 0, 0 },
    { 0xd1fd5201, 98 },
    { 0x543d5201, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfd5209, 91 },
    { 0xb4ed520a, 90 },
    { 0x36bd520a, 97 },
    { 0x2f8d520c, 90 },
    { 0xba4d520c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597d5211, 46 },
    { 0xc92d5212, 33 },
    { 0, 0 },
    { 0x78ed5214, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfd521b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fd521e, 98 },
    { 0xb77d521e, 23 },
    { 0xd4ad5220, 98 },
    { 0x104d5220, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd5226, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dd5229, 33 },
    { 0xd54d522a, 33 },
    { 0, 0 },
    { 0x3d6d522c, 55 },
    { 0xc39d522d, 68 },
    { 0x391d522d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d5234, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x838d523a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x481d523d, 90 },
    { 0xdd5d523e, 33 },
    { 0xb36d523f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fd5242, 61 },
    { 0xf28d5243, 76 },
    { 0, 0 },
    { 0xa85d5245, 98 },
    { 0x3f2d5246, 90 },
    { 0, 0 },
    { 0x60cd5248, 77 },
    { 0x67bd5249, 33 },
    { 0xe4ad524a, 61 },
    { 0xbded524b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199d524f, 87 },
    { 0x766d5250, 81 },
    { 0x808d5250, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4d5256, 55 },
    { 0, 0 },
    { 0xe02d5258, 61 },
    { 0xbb2d5259, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd525c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813d5261, 98 },
    { 0x2a9d5261, 33 },
    { 0xbf3d5262, 75 },
    { 0, 0 },
    { 0x453d5265, 91 },
    { 0x1aed5265, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dd526b, 93 },
    { 0xbe7d526c, 76 },
    { 0xf20d526c, 95 },
    { 0x981d526b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ed5271, 23 },
    { 0xf0ad5272, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ed5276, 76 },
    { 0x63cd5276, 90 },
    { 0, 0 },
    { 0x1a8d5279, 34 },
    { 0x19ad527a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bd527d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d5282, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22d528a, 66 },
    { 0, 0 },
    { 0x3d2d528c, 33 },
    { 0x8dd528c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ced5292, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bd529b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d52a3, 76 },
    { 0x5a9d52a3, 46 },
    { 0, 0 },
    { 0xa0bd52a6, 98 },
    { 0x479d52a6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x742d52aa, 52 },
    { 0xe6d52ab, 34 },
    { 0x93fd52ac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884d52b0, 90 },
    { 0xab4d52b1, 61 },
    { 0, 0 },
    { 0xed2d52b3, 78 },
    { 0, 0 },
    { 0x13fd52b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ed52b8, 67 },
    { 0x13dd52b9, 70 },
    { 0x59fd52ba, 40 },
    { 0xef8d52b9, 66 },
    { 0x4e2d52bb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90d52bf, 98 },
    { 0x93fd52c0, 66 },
    { 0x220d52bf, 61 },
    { 0x219d52c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649d52cb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d52ce, 55 },
    { 0x285d52cf, 95 },
    { 0xbe5d52d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01d52d4, 33 },
    { 0, 0 },
    { 0x4b3d52d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad52dd, 93 },
    { 0xc79d52de, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd52e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9d52e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bd52ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0d52ef, 33 },
    { 0x6c4d52ef, 91 },
    { 0x525d52ef, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3d52fc, 34 },
    { 0x9dbd52fd, 53 },
    { 0x62d52fe, 61 },
    { 0x86fd52ff, 97 },
    { 0, 0 },
    { 0x909d5301, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7d5304, 33 },
    { 0xd4ed5305, 87 },
    { 0x5c6d5306, 43 },
    { 0x25ad5306, 77 },
    { 0x772d5305, 82 },
    { 0x25d5307, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2d530c, 98 },
    { 0x804d530d, 90 },
    { 0x673d530e, 67 },
    { 0x4cd530c, 80 },
    { 0, 0 },
    { 0x4dbd5311, 23 },
    { 0, 0 },
    { 0x239d5313, 61 },
    { 0xe95d5313, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957d5318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ad5320, 19 },
    { 0, 0 },
    { 0xf93d5322, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccd5328, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d532b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105d532f, 98 },
    { 0x9ecd532f, 100 },
    { 0x2aad5331, 80 },
    { 0x93cd532f, 91 },
    { 0, 0 },
    { 0x6b3d5334, 76 },
    { 0x480d5335, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74d5338, 23 },
    { 0x2e7d5339, 52 },
    { 0, 0 },
    { 0x7edd533b, 23 },
    { 0x52ed533b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702d5340, 97 },
    { 0, 0 },
    { 0x9bcd5342, 23 },
    { 0xe41d5342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45dd534d, 33 },
    { 0, 0 },
    { 0x113d534f, 98 },
    { 0, 0 },
    { 0xf26d5351, 98 },
    { 0, 0 },
    { 0x1b5d5353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6d5356, 98 },
    { 0, 0 },
    { 0xfbad5358, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd535c, 90 },
    { 0, 0 },
    { 0x4e8d535e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542d5362, 99 },
    { 0x32fd5363, 68 },
    { 0xe46d5364, 23 },
    { 0x150d5365, 88 },
    { 0xc04d5362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941d536b, 46 },
    { 0x37bd536c, 46 },
    { 0xd15d536b, 98 },
    { 0xf42d536e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd5371, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fed5376, 91 },
    { 0xcccd5377, 55 },
    { 0x15ed5377, 95 },
    { 0xf2cd5377, 76 },
    { 0xa2d5378, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd537d, 55 },
    { 0, 0 },
    { 0xf5d537f, 55 },
    { 0xacbd5380, 90 },
    { 0x5d4d5381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91d5386, 23 },
    { 0x75d5387, 93 },
    { 0, 0 },
    { 0x129d5389, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06d538c, 55 },
    { 0x1d8d538c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cd5390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5393, 55 },
    { 0x11d5393, 33 },
    { 0x5a7d5393, 100 },
    { 0x647d5396, 33 },
    { 0x49cd5394, 76 },
    { 0x22cd5398, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6d539c, 85 },
    { 0, 0 },
    { 0x2ecd539e, 23 },
    { 0x94cd539f, 52 },
    { 0x438d53a0, 61 },
    { 0, 0 },
    { 0x3f2d53a2, 99 },
    { 0x328d53a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7d53a9, 61 },
    { 0x7c8d53a9, 84 },
    { 0x55dd53ab, 33 },
    { 0xe4bd53ab, 98 },
    { 0, 0 },
    { 0x48ed53ae, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d53b1, 66 },
    { 0xc1ed53b1, 93 },
    { 0xc59d53b2, 46 },
    { 0, 0 },
    { 0xb9bd53b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29d53b8, 23 },
    { 0, 0 },
    { 0x417d53ba, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2d53bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45d53c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4d53c6, 33 },
    { 0, 0 },
    { 0xa06d53c8, 66 },
    { 0xedcd53c9, 81 },
    { 0x590d53c9, 33 },
    { 0xd94d53cb, 33 },
    { 0x94bd53c9, 80 },
    { 0xb88d53c8, 61 },
    { 0x106d53cb, 80 },
    { 0xa9d53ce, 99 },
    { 0xfa5d53cc, 90 },
    { 0x4c1d53d1, 61 },
    { 0xe62d53d2, 33 },
    { 0x6b0d53d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d53d9, 90 },
    { 0x73cd53da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1d53e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ad53f1, 65 },
    { 0xbdfd53f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0d53f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ad5401, 95 },
    { 0, 0 },
    { 0x33cd5403, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974d540a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbd540e, 91 },
    { 0, 0 },
    { 0x878d5410, 98 },
    { 0, 0 },
    { 0xb97d5412, 77 },
    { 0x963d5413, 99 },
    { 0x8fdd5414, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2d5417, 95 },
    { 0x130d5417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63d5420, 66 },
    { 0, 0 },
    { 0x5ccd5422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafd5429, 90 },
    { 0xbaed5429, 63 },
    { 0x4d542b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdd542f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27d5432, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55d543a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bd5442, 55 },
    { 0x727d5442, 81 },
    { 0x150d5444, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5cd5448, 34 },
    { 0x24ad5449, 95 },
    { 0x486d544a, 77 },
    { 0, 0 },
    { 0xcf6d544c, 93 },
    { 0, 0 },
    { 0xae1d544e, 90 },
    { 0xc21d544f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ed5452, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5d5455, 33 },
    { 0x968d5456, 88 },
    { 0, 0 },
    { 0x384d5458, 77 },
    { 0xe8ad5458, 23 },
    { 0xbb9d5458, 76 },
    { 0x473d5458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4d5461, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dd5465, 77 },
    { 0x2f9d5465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0d546b, 93 },
    { 0x14cd546b, 90 },
    { 0, 0 },
    { 0x42d546e, 99 },
    { 0xcbbd546f, 90 },
    { 0x83cd5470, 78 },
    { 0x716d5471, 93 },
    { 0x89cd5470, 33 },
    { 0xb37d5472, 99 },
    { 0x6c9d546e, 87 },
    { 0xa0dd546f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd547c, 76 },
    { 0xfb0d547d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cd5483, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99d548a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x366d548d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2cd5490, 85 },
    { 0xdbdd5490, 55 },
    { 0xd33d5490, 33 },
    { 0, 0 },
    { 0xd4fd5494, 75 },
    { 0x373d5495, 23 },
    { 0xcd5496, 84 },
    { 0, 0 },
    { 0xcf7d5498, 33 },
    { 0xd86d5499, 33 },
    { 0x42bd5498, 61 },
    { 0x214d549b, 98 },
    { 0x846d549c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fad54a1, 98 },
    { 0xda4d54a2, 95 },
    { 0xa07d54a3, 98 },
    { 0x87cd54a1, 95 },
    { 0, 0 },
    { 0xa1ad54a6, 61 },
    { 0xd6d54a7, 91 },
    { 0x83cd54a8, 23 },
    { 0x794d54a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dd54b4, 90 },
    { 0x887d54b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedd54b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d54bd, 100 },
    { 0x564d54bd, 98 },
    { 0x138d54bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcd54c6, 90 },
    { 0x953d54c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcd54d3, 99 },
    { 0x72bd54d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8d54d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ad54da, 90 },
    { 0x32ad54db, 33 },
    { 0, 0 },
    { 0x610d54dd, 52 },
    { 0x489d54de, 38 },
    { 0x30fd54dd, 98 },
    { 0x8dcd54de, 88 },
    { 0xbebd54e1, 61 },
    { 0x707d54e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6d54e6, 34 },
    { 0x8cd54e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664d54eb, 43 },
    { 0, 0 },
    { 0x9ecd54ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596d54f1, 90 },
    { 0, 0 },
    { 0x8b9d54f3, 76 },
    { 0, 0 },
    { 0xc2ad54f5, 93 },
    { 0x14bd54f6, 55 },
    { 0x6ed54f7, 64 },
    { 0xe63d54f8, 51 },
    { 0x3b2d54f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58d54fe, 67 },
    { 0, 0 },
    { 0xd2ad5500, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafed5509, 33 },
    { 0x528d550a, 91 },
    { 0x8d4d550b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd550e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59d5515, 33 },
    { 0, 0 },
    { 0x535d5517, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dd551a, 33 },
    { 0xf2ed551b, 95 },
    { 0, 0 },
    { 0xc5d551d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x180d5520, 61 },
    { 0x6cdd5521, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x891d5528, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d5530, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19d553a, 95 },
    { 0, 0 },
    { 0xab1d553c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdd5549, 90 },
    { 0x7c6d554a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9d5551, 95 },
    { 0xe0d5552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3d5556, 76 },
    { 0x91bd5557, 90 },
    { 0, 0 },
    { 0xe5dd5559, 66 },
    { 0xead555a, 8 },
    { 0xecfd5559, 33 },
    { 0, 0 },
    { 0x504d555d, 98 },
    { 0, 0 },
    { 0x124d555f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffd5562, 33 },
    { 0xf75d5563, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d5566, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcd556a, 33 },
    { 0xcd5d556b, 98 },
    { 0xa3ed556a, 76 },
    { 0xb96d556d, 67 },
    { 0x541d556c, 95 },
    { 0x331d556f, 33 },
    { 0xc2dd556a, 90 },
    { 0x74cd5571, 78 },
    { 0x115d5572, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ad5578, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1d557b, 85 },
    { 0x92dd557c, 68 },
    { 0x2a9d557c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d5582, 23 },
    { 0x1a4d5583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676d558b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ad558e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868d5592, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd5597, 61 },
    { 0x8bfd5598, 96 },
    { 0x838d5598, 52 },
    { 0x273d559a, 55 },
    { 0x1e2d559b, 55 },
    { 0x6bd5597, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330d55a2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4d55a7, 93 },
    { 0, 0 },
    { 0xbd3d55a9, 76 },
    { 0x6b3d55aa, 76 },
    { 0, 0 },
    { 0x4c0d55ac, 23 },
    { 0x5e5d55ad, 55 },
    { 0x863d55ae, 61 },
    { 0x15dd55ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06d55b6, 33 },
    { 0xccad55b7, 78 },
    { 0, 0 },
    { 0x38bd55b9, 33 },
    { 0x346d55ba, 52 },
    { 0xd9cd55bb, 23 },
    { 0, 0 },
    { 0xf51d55bd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbed55c0, 46 },
    { 0xc7bd55c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x993d55c4, 55 },
    { 0x391d55c4, 33 },
    { 0x5f0d55c6, 95 },
    { 0x6dcd55c6, 90 },
    { 0xa70d55c7, 23 },
    { 0x296d55c4, 61 },
    { 0x296d55ca, 81 },
    { 0xef4d55ca, 23 },
    { 0, 0 },
    { 0x684d55cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ad55d0, 33 },
    { 0x179d55d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9d55d8, 90 },
    { 0x3d5d55d9, 33 },
    { 0, 0 },
    { 0xa66d55db, 93 },
    { 0xa1fd55dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed55e2, 23 },
    { 0xff0d55e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679d55eb, 98 },
    { 0x6a1d55ec, 43 },
    { 0, 0 },
    { 0xa10d55ee, 33 },
    { 0, 0 },
    { 0x701d55f0, 95 },
    { 0x590d55f1, 98 },
    { 0x7f6d55f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ded55f6, 87 },
    { 0, 0 },
    { 0x203d55f8, 95 },
    { 0x7fed55f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ed5601, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493d5606, 80 },
    { 0xa45d5607, 90 },
    { 0, 0 },
    { 0x5b9d5609, 90 },
    { 0, 0 },
    { 0x62bd560b, 78 },
    { 0x937d560c, 98 },
    { 0x4cd560d, 46 },
    { 0x349d560d, 23 },
    { 0x464d560e, 95 },
    { 0, 0 },
    { 0xe79d5611, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fad5614, 55 },
    { 0, 0 },
    { 0xad2d5616, 90 },
    { 0, 0 },
    { 0x633d5618, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103d561f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612d5623, 76 },
    { 0, 0 },
    { 0x710d5625, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64d5629, 23 },
    { 0xa51d562a, 98 },
    { 0xd29d562a, 24 },
    { 0x572d5629, 98 },
    { 0, 0 },
    { 0x5f9d562e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x400d5631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2d5635, 33 },
    { 0x48fd5635, 93 },
    { 0, 0 },
    { 0x5bed5638, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x393d563b, 33 },
    { 0x402d563c, 90 },
    { 0x110d563b, 91 },
    { 0xfffd563b, 98 },
    { 0xd8ed563f, 34 },
    { 0x364d5640, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcd5643, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3d5646, 75 },
    { 0, 0 },
    { 0x9bfd5648, 23 },
    { 0xee6d5648, 90 },
    { 0, 0 },
    { 0xbf4d564b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdd564e, 61 },
    { 0, 0 },
    { 0x8dbd5650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cd5657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bd565a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968d5662, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535d5666, 97 },
    { 0, 0 },
    { 0x692d5668, 61 },
    { 0, 0 },
    { 0x82d566a, 90 },
    { 0, 0 },
    { 0x667d566c, 90 },
    { 0x229d566c, 23 },
    { 0xbdcd566d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bd5673, 90 },
    { 0x1b0d5673, 23 },
    { 0x6fad5675, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73d567a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x501d5680, 52 },
    { 0, 0 },
    { 0x930d5682, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5d5685, 90 },
    { 0xa27d5686, 43 },
    { 0xa74d5687, 34 },
    { 0x1c8d5685, 95 },
    { 0xbf2d5689, 90 },
    { 0xbfbd568a, 52 },
    { 0x20ed5689, 69 },
    { 0xd29d5689, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252d5691, 76 },
    { 0, 0 },
    { 0x46cd5693, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9d5699, 90 },
    { 0, 0 },
    { 0x820d569b, 90 },
    { 0x156d569c, 95 },
    { 0x888d569d, 46 },
    { 0x530d569e, 98 },
    { 0xa6fd569e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d56a7, 76 },
    { 0, 0 },
    { 0xb6bd56a9, 88 },
    { 0, 0 },
    { 0xdc9d56ab, 95 },
    { 0, 0 },
    { 0x84ad56ad, 33 },
    { 0x703d56ad, 98 },
    { 0, 0 },
    { 0xab1d56b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d56b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d56bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62d56c3, 23 },
    { 0, 0 },
    { 0x982d56c5, 91 },
    { 0x956d56c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993d56cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532d56da, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73d56de, 61 },
    { 0x7e0d56de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x86d56e2, 100 },
    { 0xd98d56e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8d56e6, 91 },
    { 0, 0 },
    { 0xe87d56e8, 46 },
    { 0, 0 },
    { 0x25dd56ea, 92 },
    { 0, 0 },
    { 0xdbd56ec, 88 },
    { 0x13fd56ed, 23 },
    { 0, 0 },
    { 0x212d56ef, 85 },
    { 0x2ffd56f0, 98 },
    { 0xd54d56f1, 99 },
    { 0x72ad56f2, 23 },
    { 0x712d56f3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadd56f7, 78 },
    { 0x9ced56f7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xded56fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x332d56ff, 55 },
    { 0x11dd56ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cd5707, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fed570a, 95 },
    { 0x3c2d570b, 40 },
    { 0x8c8d570c, 61 },
    { 0, 0 },
    { 0xd11d570e, 61 },
    { 0x6a8d570e, 55 },
    { 0x7d6d5710, 96 },
    { 0, 0 },
    { 0x524d5712, 61 },
    { 0, 0 },
    { 0xe1ad5714, 23 },
    { 0x672d5715, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8d5718, 40 },
    { 0xb82d5719, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cd571c, 78 },
    { 0x8ded571c, 95 },
    { 0x60dd571c, 66 },
    { 0, 0 },
    { 0xfd5720, 61 },
    { 0xa6ed5721, 95 },
    { 0xe7fd5722, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ed5726, 76 },
    { 0x9e4d5727, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d572a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250d572f, 78 },
    { 0x762d5730, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3d573a, 33 },
    { 0x8fad573a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692d573f, 90 },
    { 0x533d5740, 87 },
    { 0xd93d5740, 93 },
    { 0x311d5740, 34 },
    { 0x1ed5741, 100 },
    { 0x786d5744, 90 },
    { 0x832d5741, 65 },
    { 0x5d8d5746, 78 },
    { 0x345d5743, 90 },
    { 0x871d5748, 46 },
    { 0xa71d5749, 88 },
    { 0x948d574a, 33 },
    { 0xc75d574b, 95 },
    { 0x205d5742, 33 },
    { 0x8f2d574a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8d5754, 95 },
    { 0x1cbd5754, 76 },
    { 0x757d5756, 91 },
    { 0x7fad5757, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d5765, 76 },
    { 0, 0 },
    { 0x2a6d5767, 68 },
    { 0x48ad5768, 87 },
    { 0x86dd5769, 61 },
    { 0x68cd5768, 34 },
    { 0, 0 },
    { 0xd32d576c, 33 },
    { 0, 0 },
    { 0xfc5d576e, 90 },
    { 0x132d576f, 23 },
    { 0x9a4d5770, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56d5774, 52 },
    { 0xcead5775, 90 },
    { 0xc68d5776, 90 },
    { 0x13fd5777, 36 },
    { 0x407d5777, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92d577e, 75 },
    { 0xaa4d577f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9d5782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dd5785, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338d5789, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dd578c, 90 },
    { 0xda3d578d, 55 },
    { 0x95ed578e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaed5793, 77 },
    { 0x715d5794, 61 },
    { 0x453d5795, 61 },
    { 0, 0 },
    { 0x961d5797, 23 },
    { 0x8add5798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd57a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88d57a3, 23 },
    { 0, 0 },
    { 0x28ed57a5, 33 },
    { 0xb96d57a5, 40 },
    { 0xe58d57a5, 93 },
    { 0x243d57a6, 90 },
    { 0x19dd57a9, 76 },
    { 0, 0 },
    { 0xdb5d57ab, 90 },
    { 0x822d57ab, 33 },
    { 0, 0 },
    { 0xd17d57ae, 95 },
    { 0x1a2d57ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fd57b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x760d57b6, 33 },
    { 0x96ed57b7, 98 },
    { 0, 0 },
    { 0x9c9d57b9, 23 },
    { 0x18fd57b9, 95 },
    { 0x903d57b9, 87 },
    { 0x25dd57bc, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8d57bf, 44 },
    { 0x56ed57c0, 90 },
    { 0xe3ed57c1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x259d57c4, 81 },
    { 0x208d57c5, 61 },
    { 0, 0 },
    { 0xcdcd57c7, 85 },
    { 0, 0 },
    { 0xf76d57c9, 66 },
    { 0xed8d57ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fd57cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dd57d4, 33 },
    { 0, 0 },
    { 0xefed57d6, 66 },
    { 0xd81d57d7, 33 },
    { 0xdcfd57d7, 23 },
    { 0, 0 },
    { 0xbb9d57da, 93 },
    { 0x745d57db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2d57e0, 55 },
    { 0xd5d57e1, 43 },
    { 0xb42d57e1, 33 },
    { 0xd6dd57e3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dd57e6, 100 },
    { 0x121d57e6, 65 },
    { 0, 0 },
    { 0x549d57e9, 90 },
    { 0, 0 },
    { 0xe44d57eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d57ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed57f1, 66 },
    { 0x6d0d57f1, 23 },
    { 0x809d57f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x969d57f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1d57fc, 85 },
    { 0xad1d57fd, 52 },
    { 0xa6d57fd, 75 },
    { 0x688d57ff, 81 },
    { 0x305d57ff, 99 },
    { 0x31dd57ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cd5804, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acd5807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559d580b, 76 },
    { 0, 0 },
    { 0x6dad580d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648d5813, 66 },
    { 0, 0 },
    { 0x6e0d5815, 61 },
    { 0, 0 },
    { 0xf5dd5817, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x157d581a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d5822, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x766d5825, 43 },
    { 0, 0 },
    { 0xf94d5827, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567d582c, 98 },
    { 0x581d582c, 2 },
    { 0, 0 },
    { 0x5cbd582f, 33 },
    { 0, 0 },
    { 0xeb1d5831, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dd5834, 33 },
    { 0x868d5835, 25 },
    { 0x2f3d5836, 55 },
    { 0, 0 },
    { 0x3bbd5838, 95 },
    { 0, 0 },
    { 0x904d583a, 76 },
    { 0x173d583a, 55 },
    { 0xc7ed583b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611d5840, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbd5848, 76 },
    { 0xeecd5849, 61 },
    { 0x95dd584a, 46 },
    { 0xa52d584b, 33 },
    { 0, 0 },
    { 0xcaad584d, 23 },
    { 0xfd7d584d, 90 },
    { 0, 0 },
    { 0x7aed5850, 98 },
    { 0x326d5851, 33 },
    { 0xbfd5852, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd585c, 90 },
    { 0x8e4d585c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ed5864, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4d586a, 64 },
    { 0, 0 },
    { 0xcafd586c, 77 },
    { 0xf7cd586c, 23 },
    { 0xb00d586e, 88 },
    { 0xea7d586e, 95 },
    { 0, 0 },
    { 0xc4d5871, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdd5874, 98 },
    { 0xc8cd5875, 23 },
    { 0, 0 },
    { 0xc5fd5877, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efd587c, 52 },
    { 0x85fd587d, 95 },
    { 0x2cd587d, 90 },
    { 0, 0 },
    { 0x2a1d5880, 95 },
    { 0x6e6d5881, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03d5886, 23 },
    { 0x94cd5887, 90 },
    { 0xb25d5888, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd588d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19d5890, 90 },
    { 0xcb5d5891, 90 },
    { 0x28d5890, 55 },
    { 0xe69d5893, 33 },
    { 0x39bd5893, 90 },
    { 0xf98d5895, 90 },
    { 0x57ed5890, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d5899, 91 },
    { 0x285d589a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d589d, 55 },
    { 0, 0 },
    { 0xe5fd589f, 33 },
    { 0x15fd58a0, 33 },
    { 0xcfed58a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6d58a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07d58b0, 17 },
    { 0x658d58b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x251d58b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5d58b8, 90 },
    { 0x378d58b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21d58c0, 46 },
    { 0, 0 },
    { 0x1f6d58c2, 100 },
    { 0x2f5d58c3, 78 },
    { 0, 0 },
    { 0x66fd58c5, 23 },
    { 0x6ffd58c6, 61 },
    { 0x966d58c7, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473d58cc, 95 },
    { 0x7bfd58cd, 98 },
    { 0xfc9d58ce, 95 },
    { 0xb18d58cf, 90 },
    { 0x433d58cc, 33 },
    { 0, 0 },
    { 0xb55d58d2, 23 },
    { 0x45bd58d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75d58dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd58e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9d58e4, 90 },
    { 0x93dd58e5, 98 },
    { 0, 0 },
    { 0x988d58e7, 95 },
    { 0xc63d58e7, 75 },
    { 0, 0 },
    { 0xdbdd58ea, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eed58ee, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ad58f3, 33 },
    { 0, 0 },
    { 0x5ecd58f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7d58f9, 76 },
    { 0x878d58fa, 95 },
    { 0x490d58fb, 24 },
    { 0x78bd58fb, 88 },
    { 0, 0 },
    { 0x19fd58fe, 33 },
    { 0x335d58ff, 90 },
    { 0, 0 },
    { 0x740d5901, 55 },
    { 0x4edd5902, 55 },
    { 0x912d5903, 55 },
    { 0xb57d5904, 78 },
    { 0xf4bd5902, 33 },
    { 0, 0 },
    { 0x7ddd5907, 76 },
    { 0x6bed5907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77d590c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d5910, 43 },
    { 0xb94d5910, 90 },
    { 0x3ddd5911, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3d5915, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcd591d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7d5923, 4 },
    { 0xe8ad5924, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x66d5927, 46 },
    { 0x7cfd5928, 23 },
    { 0x441d5929, 95 },
    { 0xd2ad5929, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713d592f, 66 },
    { 0, 0 },
    { 0x3c9d5931, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243d5936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17d593a, 93 },
    { 0x82ed593a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d593e, 76 },
    { 0, 0 },
    { 0xcddd5940, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280d5945, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fd594b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391d594f, 99 },
    { 0xa59d5950, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3d5955, 33 },
    { 0, 0 },
    { 0x556d5957, 33 },
    { 0xe26d5958, 61 },
    { 0x198d5959, 95 },
    { 0, 0 },
    { 0x64dd595b, 99 },
    { 0x59dd595b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bd5960, 99 },
    { 0x3efd5960, 23 },
    { 0x824d5962, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96d5965, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d5969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbd596e, 61 },
    { 0x3bdd596f, 76 },
    { 0, 0 },
    { 0xc47d5971, 99 },
    { 0x72dd5972, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dd5978, 93 },
    { 0, 0 },
    { 0x553d597a, 67 },
    { 0xf56d597b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d5980, 23 },
    { 0x3dad5981, 23 },
    { 0xd16d5982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1d5987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfd598c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bd5991, 87 },
    { 0xf08d5992, 78 },
    { 0, 0 },
    { 0x838d5994, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49d5997, 61 },
    { 0, 0 },
    { 0x96fd5999, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacd59a0, 76 },
    { 0x732d59a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd59a8, 61 },
    { 0, 0 },
    { 0x5f9d59aa, 23 },
    { 0x18bd59ab, 100 },
    { 0xe8ed59ab, 34 },
    { 0x152d59ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cd59b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75d59bd, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdd59c5, 90 },
    { 0x9a2d59c5, 33 },
    { 0, 0 },
    { 0xa29d59c8, 43 },
    { 0, 0 },
    { 0xaa4d59ca, 98 },
    { 0, 0 },
    { 0x35cd59cc, 84 },
    { 0xdfdd59cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fd59d7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88d59dc, 95 },
    { 0xb68d59dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x645d59e0, 61 },
    { 0xd2cd59e1, 76 },
    { 0, 0 },
    { 0xf5d59e3, 73 },
    { 0, 0 },
    { 0, 0 },
    { 0xfed59e6, 33 },
    { 0xd78d59e7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cd59ec, 77 },
    { 0, 0 },
    { 0x704d59ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d59f6, 84 },
    { 0, 0 },
    { 0x51cd59f8, 61 },
    { 0xac9d59f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d59fe, 92 },
    { 0xa52d59ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fd5a04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd5a07, 99 },
    { 0, 0 },
    { 0xfd9d5a09, 43 },
    { 0, 0 },
    { 0x844d5a0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d5a0e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5d5a16, 77 },
    { 0xd6bd5a16, 51 },
    { 0xb09d5a18, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ed5a1c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d5a21, 55 },
    { 0x84bd5a22, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfd5a27, 23 },
    { 0xf20d5a27, 43 },
    { 0x7f4d5a29, 90 },
    { 0xa50d5a2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5a2f, 23 },
    { 0, 0 },
    { 0x586d5a31, 98 },
    { 0, 0 },
    { 0x488d5a33, 78 },
    { 0, 0 },
    { 0x39cd5a35, 84 },
    { 0, 0 },
    { 0x7a3d5a37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d5a3a, 90 },
    { 0xa02d5a3b, 75 },
    { 0, 0 },
    { 0x5cdd5a3d, 87 },
    { 0x30d5a3d, 61 },
    { 0, 0 },
    { 0xa1d5a40, 91 },
    { 0xb47d5a41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x419d5a44, 84 },
    { 0x1c6d5a45, 61 },
    { 0xdfed5a46, 95 },
    { 0x760d5a47, 98 },
    { 0x512d5a46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abd5a4b, 55 },
    { 0xc1d5a4c, 95 },
    { 0x134d5a4d, 33 },
    { 0x4bad5a4e, 90 },
    { 0x71d5a4f, 75 },
    { 0x729d5a4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dd5a53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fd5a58, 90 },
    { 0xf08d5a59, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfd5a60, 68 },
    { 0x902d5a60, 76 },
    { 0x1b9d5a61, 33 },
    { 0xfd6d5a63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fd5a67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9d5a6c, 61 },
    { 0, 0 },
    { 0x8c4d5a6e, 90 },
    { 0x7ddd5a6f, 75 },
    { 0x917d5a6f, 90 },
    { 0x197d5a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8d5a75, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45d5a78, 33 },
    { 0x177d5a78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcd5a7c, 90 },
    { 0x4ffd5a7c, 76 },
    { 0, 0 },
    { 0x82bd5a7f, 61 },
    { 0xd80d5a7f, 98 },
    { 0xc30d5a81, 46 },
    { 0xf9d5a82, 91 },
    { 0xffcd5a83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28d5a8c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fd5a93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x654d5a96, 61 },
    { 0x28ed5a97, 33 },
    { 0xc91d5a97, 61 },
    { 0, 0 },
    { 0xdd7d5a9a, 34 },
    { 0x43ed5a9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764d5aa0, 46 },
    { 0xaead5aa0, 80 },
    { 0, 0 },
    { 0xa06d5aa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5aa8, 61 },
    { 0xcf1d5aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d5aac, 75 },
    { 0x58ad5aac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbd5ab6, 99 },
    { 0, 0 },
    { 0x77bd5ab8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccd5abb, 98 },
    { 0xcf6d5abb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cad5ac0, 95 },
    { 0x3c4d5ac1, 98 },
    { 0, 0 },
    { 0x4c7d5ac3, 98 },
    { 0xde5d5ac3, 95 },
    { 0xc8bd5ac5, 43 },
    { 0, 0 },
    { 0x1cdd5ac7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418d5ace, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bed5ad1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fd5ad6, 90 },
    { 0, 0 },
    { 0x8edd5ad8, 91 },
    { 0, 0 },
    { 0xf85d5ada, 88 },
    { 0, 0 },
    { 0x444d5adc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdd5ae3, 23 },
    { 0xa20d5ae4, 90 },
    { 0, 0 },
    { 0xc7ed5ae6, 98 },
    { 0xc34d5ae7, 98 },
    { 0x9bed5ae7, 46 },
    { 0x273d5ae7, 76 },
    { 0xce0d5aea, 33 },
    { 0x948d5aeb, 90 },
    { 0xb22d5aec, 85 },
    { 0x4d4d5aec, 95 },
    { 0xf80d5aee, 90 },
    { 0x9bad5aec, 95 },
    { 0xc76d5aec, 34 },
    { 0xe04d5af1, 95 },
    { 0, 0 },
    { 0x266d5af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd5af6, 90 },
    { 0x2e9d5af7, 98 },
    { 0x6d3d5af6, 33 },
    { 0x16dd5af9, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4d5afd, 55 },
    { 0, 0 },
    { 0xdd5aff, 39 },
    { 0x838d5b00, 76 },
    { 0x8c6d5b00, 55 },
    { 0x8e4d5aff, 25 },
    { 0x1b4d5b03, 61 },
    { 0, 0 },
    { 0x4cd5b05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eed5b09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fd5b0c, 23 },
    { 0, 0 },
    { 0x116d5b0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ed5b19, 61 },
    { 0x997d5b19, 33 },
    { 0x387d5b19, 33 },
    { 0, 0 },
    { 0x2cad5b1d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bd5b21, 96 },
    { 0, 0 },
    { 0x5cdd5b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacd5b27, 34 },
    { 0xe07d5b27, 95 },
    { 0xc2dd5b29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d5b2e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bd5b36, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbd5b39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2d5b40, 99 },
    { 0x513d5b40, 91 },
    { 0xc88d5b42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bd5b45, 90 },
    { 0x24dd5b45, 95 },
    { 0xc66d5b47, 81 },
    { 0xcb3d5b48, 33 },
    { 0xc13d5b46, 23 },
    { 0xfbdd5b4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcded5b4f, 90 },
    { 0x66ad5b4f, 99 },
    { 0x21dd5b50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9d5b54, 85 },
    { 0xaacd5b55, 90 },
    { 0xc57d5b55, 66 },
    { 0x561d5b55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecd5b5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636d5b60, 78 },
    { 0x8fbd5b61, 61 },
    { 0x30ed5b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d5b65, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4d5b68, 76 },
    { 0x8efd5b69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d5b6c, 36 },
    { 0x247d5b6c, 61 },
    { 0xeb7d5b6e, 46 },
    { 0x40fd5b6c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x479d5b72, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x514d5b75, 76 },
    { 0x619d5b76, 95 },
    { 0xecfd5b77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0d5b7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x755d5b82, 33 },
    { 0x14bd5b83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415d5b8f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dd5b93, 33 },
    { 0x47d5b94, 33 },
    { 0x5a4d5b95, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d5b99, 90 },
    { 0xf3dd5b9a, 23 },
    { 0xd95d5b9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170d5ba6, 98 },
    { 0x3e9d5ba7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd5bab, 55 },
    { 0xbcbd5bab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63d5bb1, 61 },
    { 0x221d5bb1, 95 },
    { 0, 0 },
    { 0xabdd5bb4, 81 },
    { 0xe89d5bb4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802d5bbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4d5bc0, 99 },
    { 0xdcfd5bc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857d5bd2, 98 },
    { 0xaa8d5bd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3d5bd9, 88 },
    { 0xfbad5bda, 98 },
    { 0, 0 },
    { 0x550d5bdc, 85 },
    { 0, 0 },
    { 0x7abd5bde, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71d5be8, 93 },
    { 0, 0 },
    { 0xf4ad5bea, 33 },
    { 0, 0 },
    { 0xff1d5bec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60d5bf2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678d5bf6, 95 },
    { 0x35cd5bf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6d5bfa, 90 },
    { 0x9f7d5bfa, 78 },
    { 0xebdd5bfc, 34 },
    { 0x3e8d5bfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38d5c00, 23 },
    { 0x270d5c01, 100 },
    { 0xc52d5c02, 90 },
    { 0xc2d5c00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ad5c06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x734d5c09, 61 },
    { 0, 0 },
    { 0x813d5c0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cd5c15, 95 },
    { 0x24cd5c15, 90 },
    { 0x294d5c15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05d5c1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add5c23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ad5c27, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ad5c2f, 61 },
    { 0, 0 },
    { 0x2a2d5c31, 66 },
    { 0xb92d5c32, 61 },
    { 0, 0 },
    { 0x3f1d5c34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ad5c39, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dd5c3c, 61 },
    { 0x13dd5c3c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfd5c43, 23 },
    { 0, 0 },
    { 0xefdd5c45, 95 },
    { 0x3afd5c46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbd5c49, 95 },
    { 0x137d5c49, 90 },
    { 0xf6cd5c4b, 23 },
    { 0x9e5d5c4c, 93 },
    { 0x42bd5c4d, 95 },
    { 0, 0 },
    { 0xe79d5c4f, 84 },
    { 0, 0 },
    { 0x93ad5c51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75d5c55, 93 },
    { 0x4a5d5c55, 77 },
    { 0, 0 },
    { 0xd71d5c58, 76 },
    { 0x417d5c58, 90 },
    { 0, 0 },
    { 0x311d5c5b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7d5c60, 33 },
    { 0xd5ad5c60, 69 },
    { 0, 0 },
    { 0x13bd5c63, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6d5c66, 55 },
    { 0, 0 },
    { 0x526d5c68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cad5c72, 61 },
    { 0xfc4d5c73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554d5c78, 23 },
    { 0x786d5c78, 52 },
    { 0xb79d5c7a, 55 },
    { 0, 0 },
    { 0x7e1d5c7c, 23 },
    { 0x8ead5c7d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31d5c80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4d5c85, 55 },
    { 0xec3d5c86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfad5c8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7d5c90, 90 },
    { 0xcc2d5c91, 43 },
    { 0x68ad5c90, 98 },
    { 0xfc3d5c90, 98 },
    { 0x2d8d5c93, 98 },
    { 0, 0 },
    { 0xdf4d5c96, 96 },
    { 0xc3cd5c97, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ad5ca3, 95 },
    { 0xb04d5ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d5ca8, 39 },
    { 0x3ebd5ca9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd5cac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d5cb0, 98 },
    { 0xb84d5cb0, 95 },
    { 0xdafd5cb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bd5cb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477d5cbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fd5cc0, 90 },
    { 0xc91d5cc1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44d5cc5, 76 },
    { 0x2b2d5cc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fd5ccb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x389d5cce, 95 },
    { 0x939d5ccf, 98 },
    { 0x969d5cd0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34d5cd5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fd5cd8, 76 },
    { 0xde9d5cd9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdd5cdc, 46 },
    { 0x361d5cdd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bed5ce0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d5ce9, 33 },
    { 0, 0 },
    { 0x775d5ceb, 84 },
    { 0x207d5cec, 88 },
    { 0x709d5ced, 78 },
    { 0xef6d5cec, 33 },
    { 0, 0 },
    { 0xc77d5cf0, 33 },
    { 0xcedd5cf1, 78 },
    { 0xc9cd5cf2, 43 },
    { 0xa74d5cf1, 90 },
    { 0xaaad5cf4, 61 },
    { 0x4aad5cf3, 23 },
    { 0x3d2d5cf1, 90 },
    { 0xcc5d5cf3, 84 },
    { 0xf61d5cf8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69d5cfb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1d5cff, 61 },
    { 0, 0 },
    { 0x95cd5d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cd5d04, 98 },
    { 0, 0 },
    { 0x391d5d06, 87 },
    { 0xc71d5d07, 61 },
    { 0x208d5d08, 76 },
    { 0, 0 },
    { 0x28ed5d0a, 95 },
    { 0x77ed5d0b, 90 },
    { 0, 0 },
    { 0x13dd5d0d, 33 },
    { 0x4edd5d0e, 76 },
    { 0x7eed5d0f, 95 },
    { 0, 0 },
    { 0xb3ed5d11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecd5d1c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebd5d23, 95 },
    { 0xed6d5d23, 95 },
    { 0x99fd5d25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304d5d2c, 95 },
    { 0, 0 },
    { 0xad0d5d2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad5d31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d5d34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fd5d3c, 90 },
    { 0x8b4d5d3d, 23 },
    { 0x6d5d3d, 61 },
    { 0xf38d5d3f, 90 },
    { 0x6c1d5d40, 55 },
    { 0xf94d5d41, 87 },
    { 0x554d5d41, 90 },
    { 0xaf8d5d40, 34 },
    { 0x302d5d43, 87 },
    { 0xf84d5d45, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdd5d4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d5d51, 66 },
    { 0x9dfd5d51, 55 },
    { 0x6c9d5d53, 34 },
    { 0x797d5d54, 95 },
    { 0xb13d5d55, 66 },
    { 0x7ead5d54, 61 },
    { 0x841d5d57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceed5d5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ed5d5e, 76 },
    { 0, 0 },
    { 0xfbed5d60, 99 },
    { 0x975d5d61, 76 },
    { 0x134d5d61, 93 },
    { 0xf12d5d63, 46 },
    { 0xa23d5d61, 34 },
    { 0x4c3d5d61, 90 },
    { 0x141d5d66, 89 },
    { 0x6d3d5d67, 33 },
    { 0, 0 },
    { 0x798d5d69, 23 },
    { 0, 0 },
    { 0xeded5d6b, 46 },
    { 0x264d5d6c, 61 },
    { 0, 0 },
    { 0x2d1d5d6e, 33 },
    { 0x790d5d6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544d5d73, 90 },
    { 0xa5fd5d74, 91 },
    { 0xd66d5d74, 46 },
    { 0x99fd5d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d5d7b, 61 },
    { 0xa3ed5d7c, 46 },
    { 0x1b1d5d7d, 43 },
    { 0x1ead5d7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68d5d81, 61 },
    { 0xe27d5d82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd5d9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7d5da0, 33 },
    { 0, 0 },
    { 0xf84d5da2, 98 },
    { 0, 0 },
    { 0x3c1d5da4, 76 },
    { 0, 0 },
    { 0x15bd5da6, 34 },
    { 0x99ed5da7, 40 },
    { 0xb8dd5da7, 33 },
    { 0x127d5da6, 98 },
    { 0x660d5daa, 43 },
    { 0, 0 },
    { 0x462d5dac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76d5daf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d5db5, 66 },
    { 0xca4d5db6, 55 },
    { 0xedd5db7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1d5dc2, 43 },
    { 0, 0 },
    { 0xd4dd5dc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fd5dc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0d5dca, 46 },
    { 0x78dd5dcb, 23 },
    { 0x819d5dcc, 43 },
    { 0x612d5dcc, 98 },
    { 0, 0 },
    { 0xacad5dcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cd5dd3, 95 },
    { 0x771d5dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9d5dd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x10d5ddb, 33 },
    { 0x346d5ddc, 33 },
    { 0, 0 },
    { 0x5c8d5dde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2d5de1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0d5dea, 23 },
    { 0, 0 },
    { 0x5b7d5dec, 98 },
    { 0x3b0d5ded, 99 },
    { 0xfefd5dee, 90 },
    { 0x90cd5dee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68d5df4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3d5dfe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x718d5e01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ead5e04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845d5e09, 90 },
    { 0, 0 },
    { 0xa51d5e0b, 75 },
    { 0xa73d5e0c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58d5e0f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7d5e12, 34 },
    { 0x911d5e13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706d5e19, 97 },
    { 0x35cd5e19, 68 },
    { 0xbced5e19, 84 },
    { 0, 0 },
    { 0x766d5e1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x139d5e20, 33 },
    { 0x48d5e21, 23 },
    { 0xa49d5e22, 77 },
    { 0x921d5e23, 46 },
    { 0x79ad5e24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bed5e28, 66 },
    { 0, 0 },
    { 0x5b2d5e2a, 85 },
    { 0, 0 },
    { 0x70ad5e2c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23d5e30, 76 },
    { 0xf71d5e30, 61 },
    { 0, 0 },
    { 0x3f4d5e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eed5e3b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06d5e42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d5e47, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfd5e4d, 97 },
    { 0xfcd5e4e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715d5e56, 61 },
    { 0, 0 },
    { 0xe40d5e58, 90 },
    { 0x712d5e58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279d5e61, 96 },
    { 0xee3d5e61, 43 },
    { 0x495d5e63, 95 },
    { 0x779d5e64, 78 },
    { 0xe6bd5e64, 15 },
    { 0x7cbd5e66, 90 },
    { 0xba6d5e65, 78 },
    { 0x481d5e62, 40 },
    { 0x54fd5e69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfd5e6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81d5e6f, 77 },
    { 0, 0 },
    { 0x464d5e71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0d5e82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9d5e87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d5e8d, 55 },
    { 0x1fcd5e8d, 61 },
    { 0x376d5e8f, 33 },
    { 0xa1cd5e8f, 46 },
    { 0, 0 },
    { 0x9c8d5e92, 61 },
    { 0x985d5e92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d5e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fd5e9c, 55 },
    { 0, 0 },
    { 0x2ed5e9e, 17 },
    { 0, 0 },
    { 0x85ad5ea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bd5ea8, 34 },
    { 0, 0 },
    { 0x903d5eaa, 23 },
    { 0x9c3d5eaa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40d5eaf, 23 },
    { 0, 0 },
    { 0xcfcd5eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad5eb6, 61 },
    { 0, 0 },
    { 0xe07d5eb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cd5ebd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8d5ec2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbed5ec6, 61 },
    { 0xcc3d5ec7, 95 },
    { 0, 0 },
    { 0xbbbd5ec9, 95 },
    { 0xa0ed5ec9, 90 },
    { 0x776d5eca, 68 },
    { 0xf6cd5ecc, 99 },
    { 0xc6d5ecd, 93 },
    { 0, 0 },
    { 0xc62d5ecf, 43 },
    { 0x20cd5ed0, 90 },
    { 0, 0 },
    { 0x9e7d5ed2, 77 },
    { 0x94ed5ed3, 95 },
    { 0xed1d5ed4, 98 },
    { 0, 0 },
    { 0xe98d5ed6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d5eda, 33 },
    { 0, 0 },
    { 0xf24d5edc, 33 },
    { 0x9f1d5edd, 82 },
    { 0x2d2d5edd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3d5ee2, 33 },
    { 0x2a2d5ee3, 95 },
    { 0xf33d5ee3, 33 },
    { 0, 0 },
    { 0xcaed5ee6, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ad5ef0, 87 },
    { 0, 0 },
    { 0x711d5ef2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fd5ef7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x705d5efa, 76 },
    { 0x39d5efb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92d5eff, 98 },
    { 0xbf0d5eff, 90 },
    { 0x52cd5f01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3d5f06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd5f0b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ed5f11, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cd5f14, 24 },
    { 0xe45d5f15, 90 },
    { 0x9a2d5f15, 23 },
    { 0x88ed5f17, 17 },
    { 0x8c4d5f18, 95 },
    { 0xa4cd5f19, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20d5f1c, 34 },
    { 0xa4dd5f1c, 33 },
    { 0x971d5f1e, 85 },
    { 0x774d5f1f, 93 },
    { 0x5b8d5f20, 43 },
    { 0xc5ed5f21, 91 },
    { 0xd0bd5f22, 96 },
    { 0x4d5d5f1f, 91 },
    { 0xb9dd5f1d, 46 },
    { 0x2d2d5f1e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d5f29, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2d5f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95d5f30, 34 },
    { 0xac9d5f31, 98 },
    { 0x77fd5f30, 43 },
    { 0x5ead5f30, 33 },
    { 0, 0 },
    { 0x501d5f35, 23 },
    { 0xedd5f36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabd5f39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d5f3d, 69 },
    { 0xaf8d5f3d, 95 },
    { 0, 0 },
    { 0xa29d5f40, 90 },
    { 0xbc3d5f41, 91 },
    { 0x2d2d5f42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504d5f46, 33 },
    { 0xef5d5f46, 61 },
    { 0xc6ad5f48, 61 },
    { 0, 0 },
    { 0xdd9d5f4a, 97 },
    { 0xdffd5f4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987d5f4f, 61 },
    { 0x775d5f50, 61 },
    { 0x676d5f50, 85 },
    { 0, 0 },
    { 0x18cd5f53, 33 },
    { 0, 0 },
    { 0x419d5f55, 81 },
    { 0xa9bd5f56, 90 },
    { 0xebcd5f57, 52 },
    { 0x459d5f57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786d5f5c, 99 },
    { 0, 0 },
    { 0xbdd5f5e, 95 },
    { 0, 0 },
    { 0x8a0d5f60, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eed5f68, 33 },
    { 0x559d5f69, 34 },
    { 0, 0 },
    { 0x7a8d5f6b, 98 },
    { 0x919d5f6c, 98 },
    { 0x838d5f6d, 55 },
    { 0x7dcd5f6d, 23 },
    { 0xdfcd5f6e, 33 },
    { 0xfd1d5f6e, 87 },
    { 0x271d5f71, 95 },
    { 0x875d5f71, 90 },
    { 0x299d5f70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cd5f77, 55 },
    { 0, 0 },
    { 0xc0bd5f79, 67 },
    { 0, 0 },
    { 0x961d5f7b, 81 },
    { 0, 0 },
    { 0xb95d5f7d, 76 },
    { 0, 0 },
    { 0x456d5f7f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43d5f83, 78 },
    { 0x830d5f83, 98 },
    { 0, 0 },
    { 0xc90d5f86, 33 },
    { 0xd8d5f87, 43 },
    { 0x607d5f88, 33 },
    { 0xdf0d5f88, 61 },
    { 0x301d5f88, 93 },
    { 0, 0 },
    { 0x3c9d5f8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20d5f8f, 61 },
    { 0x2c1d5f90, 90 },
    { 0xfd9d5f8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d5f95, 78 },
    { 0, 0 },
    { 0x29dd5f97, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75d5f9c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38d5f9f, 98 },
    { 0, 0 },
    { 0xb8fd5fa1, 99 },
    { 0xad9d5fa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1d5fa7, 33 },
    { 0, 0 },
    { 0xea5d5fa9, 77 },
    { 0xbe9d5fa9, 33 },
    { 0xd1cd5faa, 98 },
    { 0xe5cd5fac, 84 },
    { 0xa32d5fa9, 90 },
    { 0xbfed5faa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fad5fb3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbd5fb6, 61 },
    { 0, 0 },
    { 0x76ad5fb8, 95 },
    { 0, 0 },
    { 0x891d5fba, 23 },
    { 0xc02d5fba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd5fbe, 95 },
    { 0x5a9d5fbe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad5fc3, 52 },
    { 0x528d5fc3, 96 },
    { 0, 0 },
    { 0xb61d5fc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x711d5fc9, 98 },
    { 0xd4d5fca, 23 },
    { 0x34bd5fcb, 61 },
    { 0, 0 },
    { 0xb0dd5fcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7d5fd6, 90 },
    { 0, 0 },
    { 0xe3d5fd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfed5fdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d5fdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd5fe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57d5fe8, 95 },
    { 0xa93d5fe9, 91 },
    { 0xe2dd5fe8, 66 },
    { 0x11dd5fe8, 90 },
    { 0x5c1d5fec, 61 },
    { 0xf49d5fec, 90 },
    { 0xce3d5fe9, 88 },
    { 0x248d5feb, 90 },
    { 0xcd3d5ff0, 33 },
    { 0x588d5ff1, 23 },
    { 0x16ad5ff0, 17 },
    { 0xf2ed5fe8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xced5ff6, 34 },
    { 0, 0 },
    { 0x14cd5ff8, 34 },
    { 0x35ed5ff8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d6001, 33 },
    { 0xae4d6002, 95 },
    { 0x7edd6003, 33 },
    { 0x584d6003, 23 },
    { 0xd96d6002, 36 },
    { 0xd93d6006, 46 },
    { 0xb40d6007, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d600c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddd6010, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37d6017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd601c, 55 },
    { 0xcaad601d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d6023, 99 },
    { 0xe10d6023, 93 },
    { 0, 0 },
    { 0x438d6026, 85 },
    { 0x2b5d6027, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbd6035, 94 },
    { 0x4c4d6036, 90 },
    { 0x70bd6037, 23 },
    { 0x65ad6038, 85 },
    { 0xba3d6036, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd603c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcd6040, 43 },
    { 0x73fd6041, 76 },
    { 0xf65d6040, 23 },
    { 0, 0 },
    { 0x94ed6044, 75 },
    { 0x92cd6045, 95 },
    { 0xd9bd6044, 98 },
    { 0, 0 },
    { 0x8f7d6048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53d6052, 90 },
    { 0x300d6052, 69 },
    { 0x54fd6053, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d6057, 34 },
    { 0x47bd6057, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bd605c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48d6064, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0d606c, 76 },
    { 0, 0 },
    { 0xf01d606e, 23 },
    { 0x7c5d606f, 90 },
    { 0x99cd606f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1d6078, 61 },
    { 0, 0 },
    { 0x9c1d607a, 87 },
    { 0xda7d607b, 76 },
    { 0x901d607c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fd6085, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924d6089, 93 },
    { 0, 0 },
    { 0xfd7d608b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70d6090, 40 },
    { 0x9bbd6091, 78 },
    { 0x7bad6092, 34 },
    { 0x5f6d6092, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3d6098, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d609b, 61 },
    { 0x552d609c, 98 },
    { 0x442d609d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd60a0, 61 },
    { 0x326d60a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x262d60a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0d60aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d60b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdd60b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6d60ba, 33 },
    { 0x8a1d60bb, 76 },
    { 0xd81d60bc, 98 },
    { 0x1fcd60bc, 95 },
    { 0xe32d60be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9d60c2, 91 },
    { 0x863d60c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dd60c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34d60cd, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914d60d3, 88 },
    { 0, 0 },
    { 0x69d60d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x153d60d8, 34 },
    { 0xe93d60d9, 49 },
    { 0, 0 },
    { 0x350d60db, 98 },
    { 0xb63d60db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7d60e2, 99 },
    { 0xd36d60e3, 61 },
    { 0x2c3d60e4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2d60f6, 43 },
    { 0xaf6d60f7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d60fa, 76 },
    { 0xbc8d60fa, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565d6101, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefd6104, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeed610e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dd6113, 90 },
    { 0x479d6113, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392d6118, 90 },
    { 0x30ed6119, 85 },
    { 0, 0 },
    { 0xed9d611b, 23 },
    { 0x5d8d611c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278d6122, 91 },
    { 0x571d6122, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ed6126, 95 },
    { 0, 0 },
    { 0xaafd6128, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd6130, 23 },
    { 0xa74d6131, 90 },
    { 0xd97d6131, 93 },
    { 0x40cd6132, 98 },
    { 0xc0dd6131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502d6139, 61 },
    { 0, 0 },
    { 0xdebd613b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdd6143, 61 },
    { 0xb97d6143, 95 },
    { 0x2c1d6144, 82 },
    { 0x3e2d6146, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eed614f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6d6152, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5d6155, 52 },
    { 0, 0 },
    { 0xfe5d6157, 23 },
    { 0x512d6158, 33 },
    { 0x8cad6159, 33 },
    { 0x2f4d615a, 98 },
    { 0xdb2d615b, 78 },
    { 0x26cd615b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dd6160, 90 },
    { 0, 0 },
    { 0x405d6162, 100 },
    { 0xb8cd6162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260d616d, 96 },
    { 0x193d616e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733d6172, 90 },
    { 0, 0 },
    { 0xa8ad6174, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd6177, 77 },
    { 0x7a8d6178, 34 },
    { 0x308d6179, 91 },
    { 0x628d6178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d617d, 33 },
    { 0x4f6d617e, 34 },
    { 0x9b0d617f, 95 },
    { 0, 0 },
    { 0xb4ed6181, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ad6186, 23 },
    { 0x9fbd6187, 78 },
    { 0x712d6188, 90 },
    { 0, 0 },
    { 0xfe0d618a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d6191, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485d6198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95d619b, 98 },
    { 0x231d619b, 88 },
    { 0x710d619d, 43 },
    { 0x1acd619e, 100 },
    { 0, 0 },
    { 0xc5ad61a0, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aed61ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dd61af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd61b2, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ed61be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0d61c1, 90 },
    { 0, 0 },
    { 0x9d8d61c3, 33 },
    { 0x5a3d61c3, 98 },
    { 0xa0cd61c5, 90 },
    { 0x2d5d61c4, 34 },
    { 0, 0 },
    { 0x189d61c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1d61cb, 23 },
    { 0, 0 },
    { 0x6cad61cd, 66 },
    { 0, 0 },
    { 0x3fbd61cf, 55 },
    { 0x8d61cf, 23 },
    { 0xb51d61cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503d61d6, 76 },
    { 0x50ad61d7, 90 },
    { 0x148d61d8, 34 },
    { 0, 0 },
    { 0x98ad61da, 77 },
    { 0x95d61db, 61 },
    { 0xa13d61db, 76 },
    { 0x8d4d61da, 34 },
    { 0xb1ad61de, 34 },
    { 0xe25d61db, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7d61e3, 52 },
    { 0xbaad61e4, 61 },
    { 0xaffd61e5, 78 },
    { 0xbf7d61e6, 90 },
    { 0, 0 },
    { 0x9e1d61e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d61f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add61f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203d61f8, 55 },
    { 0, 0 },
    { 0x14cd61fa, 23 },
    { 0, 0 },
    { 0x6e6d61fc, 23 },
    { 0, 0 },
    { 0xeccd61fe, 90 },
    { 0xe65d61fe, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdd620f, 33 },
    { 0x8cbd6210, 99 },
    { 0x5a9d6211, 76 },
    { 0xc79d620f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ad621a, 98 },
    { 0xe77d621b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88d621f, 55 },
    { 0, 0 },
    { 0x143d6221, 95 },
    { 0xbad6221, 93 },
    { 0x407d6223, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddd6229, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1d622d, 66 },
    { 0x8bfd622e, 77 },
    { 0xf51d622f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dd6238, 23 },
    { 0xd17d6238, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0d6240, 55 },
    { 0x338d6241, 68 },
    { 0xd2d6241, 55 },
    { 0x3f5d6243, 23 },
    { 0xf54d6244, 88 },
    { 0xe54d6245, 55 },
    { 0x29ed6244, 95 },
    { 0x9afd6244, 84 },
    { 0x76fd6244, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add624d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6d6250, 95 },
    { 0xaabd6251, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5d6254, 91 },
    { 0xa5bd6255, 98 },
    { 0x9b7d6256, 61 },
    { 0x10cd6255, 98 },
    { 0xb8d6258, 61 },
    { 0xe7d6258, 91 },
    { 0x467d6256, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9d625f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18d6263, 87 },
    { 0x8fbd6263, 66 },
    { 0x8ed6263, 61 },
    { 0x41fd6265, 99 },
    { 0x4dcd6264, 76 },
    { 0, 0 },
    { 0x406d6269, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cd626e, 96 },
    { 0x875d626e, 93 },
    { 0, 0 },
    { 0xa2cd6271, 46 },
    { 0x8ecd6272, 95 },
    { 0xdddd6273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d6278, 84 },
    { 0xe9dd6278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d627e, 95 },
    { 0x540d627f, 87 },
    { 0xb72d6280, 67 },
    { 0x966d6281, 55 },
    { 0xd94d6280, 100 },
    { 0x9c2d6283, 90 },
    { 0, 0 },
    { 0x55bd6285, 98 },
    { 0x1c9d6286, 61 },
    { 0x531d6285, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbd628c, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7d628f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x467d6292, 61 },
    { 0x18ed6292, 94 },
    { 0xe2fd6293, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaad629d, 91 },
    { 0xefbd629e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7d62a1, 95 },
    { 0xad9d62a2, 95 },
    { 0xd96d62a3, 23 },
    { 0x8ebd62a2, 95 },
    { 0x906d62a3, 90 },
    { 0x47ad62a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbd62ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9d62b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bd62b4, 70 },
    { 0xb19d62b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8d62ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaed62c2, 23 },
    { 0x5b4d62c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dd62c6, 43 },
    { 0xc38d62c6, 61 },
    { 0x5c4d62c8, 33 },
    { 0, 0 },
    { 0xf00d62ca, 90 },
    { 0x4ecd62ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9d62ce, 76 },
    { 0x702d62cf, 67 },
    { 0x8d0d62cf, 97 },
    { 0x956d62d1, 52 },
    { 0, 0 },
    { 0x59fd62d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd62d9, 33 },
    { 0x2e8d62da, 95 },
    { 0x599d62db, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d62df, 78 },
    { 0x364d62df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73d62ea, 90 },
    { 0x46d62eb, 64 },
    { 0x1ecd62eb, 98 },
    { 0x952d62ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895d62f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d62fa, 23 },
    { 0xe4fd62fb, 91 },
    { 0, 0 },
    { 0x37ad62fd, 84 },
    { 0xe79d62fe, 33 },
    { 0, 0 },
    { 0x1e6d6300, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401d6306, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d6309, 46 },
    { 0x652d6309, 43 },
    { 0x1eed630b, 77 },
    { 0x7cfd6309, 33 },
    { 0x945d630d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cd6310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08d6315, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4d6318, 99 },
    { 0xc25d6319, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd631c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97d631f, 66 },
    { 0x500d631f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x373d6323, 33 },
    { 0x9fad6324, 43 },
    { 0, 0 },
    { 0xc3fd6326, 90 },
    { 0, 0 },
    { 0xead6328, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647d632c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2d6330, 36 },
    { 0x704d6330, 23 },
    { 0x27cd6332, 61 },
    { 0, 0 },
    { 0xd82d6334, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fd633c, 76 },
    { 0, 0 },
    { 0x6bed633e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d6343, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649d6347, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cd634a, 61 },
    { 0x39ad634a, 80 },
    { 0x667d634c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7d634f, 98 },
    { 0x217d6350, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315d6356, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0d635a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x234d635d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467d6364, 98 },
    { 0, 0 },
    { 0x36cd6366, 87 },
    { 0, 0 },
    { 0x464d6368, 33 },
    { 0xfb7d6369, 66 },
    { 0x4f5d636a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd636e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5d6372, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x549d6375, 95 },
    { 0xc89d6376, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fd6379, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68d637c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ed637f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5d6382, 33 },
    { 0x3a7d6383, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2d6386, 78 },
    { 0xd6ad6387, 43 },
    { 0, 0 },
    { 0xb0fd6389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ad638c, 43 },
    { 0, 0 },
    { 0xd74d638e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccd6392, 67 },
    { 0, 0 },
    { 0xe78d6394, 88 },
    { 0xdd2d6395, 55 },
    { 0xc47d6394, 61 },
    { 0x1d4d6397, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d639a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0d639e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982d63a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7d63a9, 33 },
    { 0xf35d63aa, 90 },
    { 0xcafd63ab, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbd63b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d63b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67d63b6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x126d63b9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9d63bd, 91 },
    { 0, 0 },
    { 0x2b8d63bf, 23 },
    { 0x6ffd63c0, 98 },
    { 0, 0 },
    { 0xf11d63c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d63ca, 61 },
    { 0, 0 },
    { 0x49fd63cc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4d63cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157d63d4, 23 },
    { 0x1f3d63d5, 95 },
    { 0, 0 },
    { 0xeb5d63d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6d63db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17d63df, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7d63e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724d63e9, 61 },
    { 0xbd1d63e9, 87 },
    { 0, 0 },
    { 0x7c7d63ec, 34 },
    { 0x82fd63ec, 95 },
    { 0x3e5d63ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611d63f3, 23 },
    { 0, 0 },
    { 0x357d63f5, 81 },
    { 0x4cdd63f5, 46 },
    { 0x837d63f7, 98 },
    { 0xb3ad63f8, 23 },
    { 0x90cd63f9, 68 },
    { 0x96d63f9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88d6406, 55 },
    { 0xdd5d6406, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d640a, 90 },
    { 0, 0 },
    { 0x5d3d640c, 95 },
    { 0, 0 },
    { 0x19ed640e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffd6414, 52 },
    { 0x9a0d6415, 98 },
    { 0xb2dd6415, 23 },
    { 0x31dd6417, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd641a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeed641d, 23 },
    { 0x869d641e, 23 },
    { 0xb4d641e, 33 },
    { 0x93fd6420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x516d6423, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d6429, 93 },
    { 0xa8d6429, 75 },
    { 0x58dd642b, 34 },
    { 0x6b6d642c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8d642f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437d6436, 77 },
    { 0xf7cd6436, 23 },
    { 0x726d6437, 70 },
    { 0x33dd6439, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75d6442, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded6449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dd6450, 23 },
    { 0x248d6451, 61 },
    { 0x4efd6451, 90 },
    { 0xfc1d6451, 90 },
    { 0x48fd6450, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d645c, 61 },
    { 0, 0 },
    { 0xa29d645e, 33 },
    { 0x916d645f, 93 },
    { 0x591d6460, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd6463, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59d6468, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3d6470, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4d6473, 33 },
    { 0x3dd6474, 68 },
    { 0xeb0d6475, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bd6478, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618d647c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21fd6480, 52 },
    { 0, 0 },
    { 0x94ad6482, 23 },
    { 0xf66d6482, 61 },
    { 0, 0 },
    { 0x116d6485, 98 },
    { 0, 0 },
    { 0x24cd6487, 76 },
    { 0x6fbd6488, 90 },
    { 0x1b1d6488, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eed648c, 23 },
    { 0, 0 },
    { 0xfccd648e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30d6492, 17 },
    { 0x3f0d6492, 17 },
    { 0xe7fd6494, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bd6499, 90 },
    { 0, 0 },
    { 0x48dd649b, 66 },
    { 0x7d9d649c, 61 },
    { 0xa5fd649d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1d64a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66d64a6, 98 },
    { 0xa95d64a7, 77 },
    { 0, 0 },
    { 0x861d64a9, 95 },
    { 0x6e5d64aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9d64af, 47 },
    { 0xd5d64b0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6d64b3, 90 },
    { 0x7cd64b3, 23 },
    { 0x4f1d64b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ad64b9, 67 },
    { 0x884d64ba, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945d64be, 78 },
    { 0xadd64bf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ed64c5, 43 },
    { 0x569d64c5, 23 },
    { 0xfbd64c5, 33 },
    { 0xc7cd64c8, 93 },
    { 0xcddd64c9, 33 },
    { 0x3bad64c8, 99 },
    { 0, 0 },
    { 0x85d64cc, 61 },
    { 0x32ad64cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0d64d2, 100 },
    { 0x30fd64d3, 76 },
    { 0xcd2d64d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x811d64d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4d64db, 98 },
    { 0, 0 },
    { 0x399d64dd, 95 },
    { 0xe8ad64dd, 98 },
    { 0x4c2d64dd, 33 },
    { 0xa75d64de, 95 },
    { 0xe44d64df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfd64e8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5d64eb, 90 },
    { 0, 0 },
    { 0x60bd64ed, 67 },
    { 0, 0 },
    { 0x2bbd64ef, 67 },
    { 0x9ddd64ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x234d64f3, 90 },
    { 0x4cdd64f3, 95 },
    { 0, 0 },
    { 0xf20d64f6, 23 },
    { 0, 0 },
    { 0x789d64f8, 78 },
    { 0x6c4d64f9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x111d64fc, 23 },
    { 0x10fd64fd, 61 },
    { 0x3a3d64fe, 34 },
    { 0x967d64ff, 98 },
    { 0x600d64fe, 33 },
    { 0, 0 },
    { 0x579d6502, 95 },
    { 0xb48d6503, 90 },
    { 0, 0 },
    { 0x849d6505, 91 },
    { 0xac1d6506, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed6509, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ad650f, 77 },
    { 0x503d650f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bd6516, 66 },
    { 0x93d6517, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fd6521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fd652a, 95 },
    { 0xde5d652a, 95 },
    { 0x6a6d652c, 95 },
    { 0x272d652d, 77 },
    { 0x90cd652d, 40 },
    { 0xb64d652c, 61 },
    { 0x4f5d652e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09d6533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62d6536, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5dd653b, 66 },
    { 0x476d653c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4d6541, 76 },
    { 0x535d6541, 90 },
    { 0xc0d6543, 93 },
    { 0xc69d6542, 98 },
    { 0x5ffd6542, 23 },
    { 0xf1d6546, 95 },
    { 0x118d6546, 91 },
    { 0x403d6548, 98 },
    { 0xfd9d6546, 90 },
    { 0x2d1d654a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53d6551, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150d6556, 90 },
    { 0, 0 },
    { 0x24dd6558, 98 },
    { 0, 0 },
    { 0x20bd655a, 83 },
    { 0xebbd655b, 61 },
    { 0x2a9d655b, 33 },
    { 0xfcbd655d, 55 },
    { 0x706d655c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5d6564, 52 },
    { 0x73fd6564, 33 },
    { 0xd9d6566, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd6569, 61 },
    { 0xc81d656a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cd656e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dd6577, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7d657a, 59 },
    { 0x8d8d657a, 81 },
    { 0x2d2d657b, 77 },
    { 0x4a3d657d, 23 },
    { 0x302d657a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13d6582, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975d658a, 88 },
    { 0x837d658a, 98 },
    { 0xf27d658c, 34 },
    { 0xa3ad658d, 87 },
    { 0xd0fd658c, 67 },
    { 0xb78d658b, 61 },
    { 0xc4bd6590, 95 },
    { 0x9c6d6591, 31 },
    { 0x27d6592, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4d6596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d6599, 43 },
    { 0x3bfd6599, 95 },
    { 0xb4ed659b, 88 },
    { 0xa06d659c, 91 },
    { 0x5a7d659b, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bad65a0, 90 },
    { 0x13bd65a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9d65a4, 61 },
    { 0xf30d65a4, 99 },
    { 0x817d65a6, 99 },
    { 0, 0 },
    { 0x5aed65a8, 40 },
    { 0x4dfd65a8, 55 },
    { 0, 0 },
    { 0xc37d65ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d65ae, 23 },
    { 0x1d4d65af, 98 },
    { 0x1a4d65ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ad65b4, 78 },
    { 0xcfdd65b5, 23 },
    { 0x865d65b6, 33 },
    { 0x8f3d65b7, 90 },
    { 0, 0 },
    { 0x876d65b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aad65bd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382d65c2, 23 },
    { 0x470d65c3, 100 },
    { 0, 0 },
    { 0xad2d65c5, 88 },
    { 0x5cfd65c6, 98 },
    { 0x5d3d65c6, 61 },
    { 0x52bd65c8, 95 },
    { 0x210d65c8, 98 },
    { 0x8bbd65c5, 61 },
    { 0, 0 },
    { 0x747d65cc, 90 },
    { 0, 0 },
    { 0x9c7d65ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d65d5, 90 },
    { 0x290d65d6, 90 },
    { 0xdc9d65d6, 95 },
    { 0x7c2d65d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0d65dd, 61 },
    { 0xb6fd65de, 90 },
    { 0, 0 },
    { 0xbbbd65e0, 61 },
    { 0x1a1d65e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7d65e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399d65eb, 95 },
    { 0x85bd65ec, 85 },
    { 0x34bd65ec, 75 },
    { 0x124d65ee, 91 },
    { 0xab7d65ef, 61 },
    { 0x958d65f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfd65f6, 67 },
    { 0xbbed65f7, 55 },
    { 0, 0 },
    { 0xbfad65f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94d65fd, 46 },
    { 0xddd65fd, 33 },
    { 0x3d8d65ff, 90 },
    { 0x5ecd65ff, 95 },
    { 0xeb4d65fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937d6606, 52 },
    { 0, 0 },
    { 0x3dbd6608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6d660e, 61 },
    { 0xc9bd660e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70d661a, 36 },
    { 0x211d661b, 34 },
    { 0x314d661a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcd6620, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d6629, 55 },
    { 0, 0 },
    { 0x13bd662b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157d6632, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726d6637, 34 },
    { 0, 0 },
    { 0x6a8d6639, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28d663d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7d6640, 61 },
    { 0x4c1d6641, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5bd6648, 43 },
    { 0, 0 },
    { 0x575d664a, 101 },
    { 0, 0 },
    { 0x856d664c, 98 },
    { 0xa4bd664d, 95 },
    { 0x197d664c, 93 },
    { 0, 0 },
    { 0xad6650, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dd6653, 90 },
    { 0x6ebd6654, 76 },
    { 0x2b4d6653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ad6659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bd665d, 100 },
    { 0xe3ed665e, 84 },
    { 0xdbfd665f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67dd6663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dd666b, 84 },
    { 0x8d4d666c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fd6671, 34 },
    { 0, 0 },
    { 0xd5d6673, 33 },
    { 0x1a4d6673, 68 },
    { 0x768d6675, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cd6679, 52 },
    { 0x499d6679, 43 },
    { 0, 0 },
    { 0x31cd667c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe23d667f, 64 },
    { 0, 0 },
    { 0x1bad6681, 23 },
    { 0xd8d6682, 34 },
    { 0x193d6683, 34 },
    { 0x7c4d6683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedd6687, 76 },
    { 0, 0 },
    { 0x5acd6689, 61 },
    { 0x81d668a, 43 },
    { 0x364d6689, 77 },
    { 0x555d668c, 98 },
    { 0xfa5d668c, 43 },
    { 0xad7d6689, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbed6691, 90 },
    { 0xcc7d6691, 76 },
    { 0x2efd6693, 66 },
    { 0xc39d6694, 90 },
    { 0x524d6695, 17 },
    { 0x681d6695, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ced669e, 90 },
    { 0xe53d669f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339d66a4, 55 },
    { 0x233d66a5, 61 },
    { 0xc94d66a6, 98 },
    { 0, 0 },
    { 0xb24d66a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d66ad, 46 },
    { 0, 0 },
    { 0x432d66af, 43 },
    { 0xb36d66b0, 95 },
    { 0, 0 },
    { 0x897d66b2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6d66b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecd66bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fd66c1, 55 },
    { 0x2a0d66c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d66c5, 66 },
    { 0, 0 },
    { 0x991d66c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d66ce, 23 },
    { 0x20bd66cf, 23 },
    { 0x137d66cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79d66d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98d66dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdd66e2, 95 },
    { 0x76bd66e3, 84 },
    { 0x787d66e3, 84 },
    { 0, 0 },
    { 0x90bd66e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfd66ec, 23 },
    { 0xd2ad66ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dad66f2, 88 },
    { 0xdf2d66f2, 23 },
    { 0xd69d66f4, 33 },
    { 0, 0 },
    { 0xa5d66f6, 66 },
    { 0x9b8d66f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d66fa, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x226d66fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33d6702, 33 },
    { 0x7d4d6703, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92d6709, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd670e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9d6712, 61 },
    { 0x7ead6713, 34 },
    { 0x6e4d6712, 56 },
    { 0, 0 },
    { 0x112d6716, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d6719, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0d671e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x141d6721, 61 },
    { 0, 0 },
    { 0x53cd6723, 55 },
    { 0xc64d6723, 23 },
    { 0x32dd6725, 61 },
    { 0x774d6726, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x313d6729, 87 },
    { 0, 0 },
    { 0xfced672b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdd672e, 90 },
    { 0x6add672e, 61 },
    { 0x3aed672f, 43 },
    { 0xb6d672f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09d6735, 61 },
    { 0, 0 },
    { 0x4bed6737, 90 },
    { 0x4acd6738, 17 },
    { 0xa03d6738, 87 },
    { 0xb33d6738, 24 },
    { 0x2d6737, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952d673f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecd674d, 98 },
    { 0xc7dd674e, 91 },
    { 0x3b1d674f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x756d6752, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8d6756, 52 },
    { 0xbe5d6757, 74 },
    { 0x1a3d6757, 85 },
    { 0, 0 },
    { 0xb54d675a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95d6769, 90 },
    { 0xa4ad676a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbd6770, 98 },
    { 0x613d6771, 90 },
    { 0x5b6d6771, 33 },
    { 0xadfd6771, 95 },
    { 0x38d6774, 55 },
    { 0, 0 },
    { 0x779d6776, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ed677d, 61 },
    { 0, 0 },
    { 0xb86d677f, 84 },
    { 0x455d6780, 98 },
    { 0, 0 },
    { 0xf3d6782, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640d6788, 98 },
    { 0x9f0d6788, 76 },
    { 0x43ed678a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ad678d, 33 },
    { 0xb18d678e, 43 },
    { 0x8c4d678e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8d6792, 43 },
    { 0, 0 },
    { 0xa5ad6794, 46 },
    { 0x709d6795, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xecad6798, 98 },
    { 0, 0 },
    { 0xe48d679a, 75 },
    { 0xe9ed679b, 90 },
    { 0xabdd679b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ed679f, 95 },
    { 0, 0 },
    { 0x3a1d67a1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2d67a8, 76 },
    { 0x85fd67a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722d67af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5d67b3, 98 },
    { 0x217d67b4, 76 },
    { 0, 0 },
    { 0xb28d67b6, 90 },
    { 0, 0 },
    { 0xd7cd67b8, 17 },
    { 0xf66d67b9, 55 },
    { 0x178d67ba, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6d67bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d67c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04d67cc, 88 },
    { 0x95dd67cc, 61 },
    { 0x311d67ce, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6d67d1, 33 },
    { 0x1a6d67d2, 33 },
    { 0, 0 },
    { 0xb4ed67d4, 83 },
    { 0x338d67d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2d67d8, 98 },
    { 0xdefd67d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ed67dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dd67e5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d67e9, 61 },
    { 0x923d67ea, 91 },
    { 0x8b8d67eb, 46 },
    { 0xfc2d67ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ed67ef, 76 },
    { 0x4a3d67f0, 33 },
    { 0, 0 },
    { 0xce8d67f2, 84 },
    { 0x83ed67f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cd67f7, 76 },
    { 0x46bd67f7, 90 },
    { 0, 0 },
    { 0x708d67fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304d67ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444d6806, 78 },
    { 0x3fbd6807, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d680a, 90 },
    { 0xd4d680b, 53 },
    { 0xcded680b, 33 },
    { 0xf62d680d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1d6812, 61 },
    { 0, 0 },
    { 0x9fd6814, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24d6819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1d681c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3d681f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13d6822, 55 },
    { 0xa56d6822, 95 },
    { 0, 0 },
    { 0xce4d6825, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ad682a, 90 },
    { 0x411d682b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706d682f, 93 },
    { 0x820d682f, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd6833, 84 },
    { 0x8fbd6833, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790d6840, 90 },
    { 0, 0 },
    { 0x3d0d6842, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19d6845, 93 },
    { 0x2b5d6846, 52 },
    { 0x3d3d6847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d684f, 33 },
    { 0x352d6850, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ad6854, 33 },
    { 0x151d6854, 61 },
    { 0xad0d6854, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356d685a, 33 },
    { 0x956d685a, 33 },
    { 0, 0 },
    { 0xd66d685d, 98 },
    { 0xeb7d685d, 75 },
    { 0x676d685f, 55 },
    { 0x32ad685e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd6863, 34 },
    { 0, 0 },
    { 0xc42d6865, 98 },
    { 0x455d6865, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cad686a, 23 },
    { 0x615d686a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314d6874, 40 },
    { 0x511d6874, 61 },
    { 0x1fbd6876, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd6879, 93 },
    { 0, 0 },
    { 0x2afd687b, 42 },
    { 0, 0 },
    { 0xb55d687d, 87 },
    { 0, 0 },
    { 0x139d687f, 90 },
    { 0x20ad687f, 76 },
    { 0xa26d6881, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218d6887, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ed688e, 76 },
    { 0x6dcd688f, 100 },
    { 0x25dd688e, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acd689b, 66 },
    { 0xa8ad689c, 46 },
    { 0, 0 },
    { 0x185d689e, 98 },
    { 0x771d689f, 33 },
    { 0x427d68a0, 33 },
    { 0xe9ed689f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8d68a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efd68a8, 33 },
    { 0xce7d68a8, 90 },
    { 0x40ad68aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d68b0, 61 },
    { 0x7dcd68b0, 23 },
    { 0xdd1d68b2, 98 },
    { 0x39dd68b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6d68ba, 92 },
    { 0, 0 },
    { 0x924d68bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25d68c7, 48 },
    { 0, 0 },
    { 0x84bd68c9, 46 },
    { 0x6f2d68c9, 90 },
    { 0x82d68cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43d68ce, 95 },
    { 0x429d68cf, 33 },
    { 0x79ed68d0, 98 },
    { 0, 0 },
    { 0x812d68d2, 33 },
    { 0, 0 },
    { 0x5bad68d4, 99 },
    { 0x9ddd68d4, 94 },
    { 0x264d68d5, 100 },
    { 0, 0 },
    { 0x60bd68d8, 17 },
    { 0x6f9d68d8, 90 },
    { 0xa14d68da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bd68de, 23 },
    { 0x77cd68de, 34 },
    { 0xb81d68e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644d68e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bd68eb, 61 },
    { 0x5b8d68eb, 43 },
    { 0, 0 },
    { 0x5e2d68ee, 95 },
    { 0, 0 },
    { 0xf69d68f0, 33 },
    { 0x50bd68f1, 23 },
    { 0xf8cd68f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152d68f6, 90 },
    { 0xcaed68f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5d68fd, 84 },
    { 0xdf3d68fd, 43 },
    { 0x56ed68ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x655d6902, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3d690d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cd6918, 90 },
    { 0xd75d6918, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d691f, 61 },
    { 0xc1d6920, 76 },
    { 0xb7ed6921, 66 },
    { 0, 0 },
    { 0x558d6923, 95 },
    { 0x136d6924, 33 },
    { 0x22fd6925, 93 },
    { 0x876d6925, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78d6929, 61 },
    { 0xd79d6929, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88d692d, 61 },
    { 0xa02d692d, 76 },
    { 0xd99d692f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d6932, 88 },
    { 0, 0 },
    { 0xd7cd6934, 90 },
    { 0x9e4d6935, 90 },
    { 0xdb1d6936, 55 },
    { 0xa9ad6935, 76 },
    { 0, 0 },
    { 0xfe9d6939, 98 },
    { 0xc60d693a, 43 },
    { 0xc8ad693a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ed693f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d694e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24d6952, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77d6955, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd6958, 81 },
    { 0xdbfd6959, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dd6962, 33 },
    { 0xba8d6963, 61 },
    { 0xc13d6964, 95 },
    { 0x419d6965, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed696f, 23 },
    { 0x6b8d6970, 61 },
    { 0x8fd6970, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd6977, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fed697b, 66 },
    { 0x244d697c, 88 },
    { 0, 0 },
    { 0x277d697e, 23 },
    { 0, 0 },
    { 0x533d6980, 55 },
    { 0x4f3d6981, 23 },
    { 0xb38d6982, 67 },
    { 0xe2fd6983, 76 },
    { 0x3bcd6983, 43 },
    { 0x68d6985, 23 },
    { 0x453d6986, 33 },
    { 0xc5dd6983, 36 },
    { 0x789d6988, 33 },
    { 0, 0 },
    { 0xd0d698a, 61 },
    { 0xb75d698a, 61 },
    { 0xdcad698c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69d6990, 61 },
    { 0, 0 },
    { 0xf17d6992, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733d6996, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcd699a, 78 },
    { 0, 0 },
    { 0x619d699c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90d699f, 33 },
    { 0x4d4d699f, 23 },
    { 0xa45d69a1, 46 },
    { 0, 0 },
    { 0xb4ed69a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ed69a8, 90 },
    { 0x8ed69a8, 33 },
    { 0xb3dd69a9, 55 },
    { 0x20cd69ab, 93 },
    { 0x69fd69ab, 90 },
    { 0x7cbd69ad, 23 },
    { 0x24bd69ae, 2 },
    { 0xd0d69ae, 99 },
    { 0xb75d69b0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x167d69b4, 95 },
    { 0x128d69b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8d69b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622d69bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244d69c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2d69c7, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fd69cb, 90 },
    { 0xaccd69cc, 23 },
    { 0, 0 },
    { 0x287d69ce, 89 },
    { 0, 0 },
    { 0x40d69d0, 33 },
    { 0, 0 },
    { 0x6bad69d2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ad69d7, 78 },
    { 0xdd2d69d7, 55 },
    { 0x491d69d8, 90 },
    { 0xf6fd69da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd69de, 95 },
    { 0, 0 },
    { 0x210d69e0, 23 },
    { 0, 0 },
    { 0xcf5d69e2, 76 },
    { 0, 0 },
    { 0x656d69e4, 90 },
    { 0, 0 },
    { 0x166d69e6, 99 },
    { 0x8cfd69e7, 95 },
    { 0xf6d69e7, 23 },
    { 0, 0 },
    { 0x8afd69ea, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7d69f1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccd69f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684d69fa, 23 },
    { 0xce0d69fb, 33 },
    { 0x8ddd69fb, 67 },
    { 0, 0 },
    { 0xf5ed69fe, 52 },
    { 0x4b8d69ff, 99 },
    { 0, 0 },
    { 0x5fed6a01, 90 },
    { 0xcfcd6a02, 43 },
    { 0xcf1d6a02, 87 },
    { 0x96ed6a04, 90 },
    { 0xce0d6a05, 61 },
    { 0, 0 },
    { 0x3f0d6a07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5d6a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cd6a0f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cd6a12, 76 },
    { 0, 0 },
    { 0x8a0d6a14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5d6a1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1d6a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957d6a23, 84 },
    { 0x4e8d6a23, 33 },
    { 0, 0 },
    { 0x5dbd6a26, 100 },
    { 0x998d6a27, 34 },
    { 0x5ad6a28, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9d6a2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6d6a31, 98 },
    { 0x386d6a32, 88 },
    { 0, 0 },
    { 0x68fd6a34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfad6a39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74d6a3e, 33 },
    { 0, 0 },
    { 0x3afd6a40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09d6a46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5dd6a49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23d6a4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x346d6a50, 98 },
    { 0x9e4d6a51, 81 },
    { 0x3d1d6a52, 95 },
    { 0x7e8d6a53, 90 },
    { 0x90bd6a51, 90 },
    { 0x58ed6a50, 55 },
    { 0xd4d6a56, 95 },
    { 0x419d6a55, 86 },
    { 0, 0 },
    { 0xbb5d6a59, 98 },
    { 0x4add6a59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x533d6a5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8d6a66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8d6a69, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57d6a6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaad6a73, 95 },
    { 0xa1ed6a74, 78 },
    { 0x88dd6a75, 36 },
    { 0x94ad6a76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bd6a80, 33 },
    { 0x224d6a80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3d6a84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3d6a88, 90 },
    { 0x956d6a89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8d6a8d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9d6a92, 75 },
    { 0x72ed6a92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dd6a96, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d6a9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9d6aa0, 46 },
    { 0xb79d6aa1, 61 },
    { 0x32ad6aa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ed6aa6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x704d6aa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dd6aac, 76 },
    { 0xd7ad6aad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d6ab0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61d6ab3, 84 },
    { 0, 0 },
    { 0xa39d6ab5, 76 },
    { 0x41fd6ab6, 33 },
    { 0, 0 },
    { 0x41d6ab8, 61 },
    { 0xd6d6ab9, 68 },
    { 0x216d6aba, 93 },
    { 0x42bd6ab8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ad6abe, 33 },
    { 0, 0 },
    { 0xa21d6ac0, 33 },
    { 0x9f9d6ac1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192d6ac9, 37 },
    { 0x2e8d6aca, 33 },
    { 0x9e4d6aca, 98 },
    { 0x190d6acc, 100 },
    { 0xd4dd6acd, 76 },
    { 0xba9d6ace, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6d6ad1, 40 },
    { 0x2c7d6ad1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7d6ad7, 90 },
    { 0x40ed6ad8, 76 },
    { 0x1e8d6ad9, 90 },
    { 0x8e9d6ada, 78 },
    { 0xe3ad6ad7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbd6ade, 76 },
    { 0xd09d6adf, 23 },
    { 0x42bd6ae0, 33 },
    { 0, 0 },
    { 0x66ed6ae2, 76 },
    { 0x4a8d6ae2, 4 },
    { 0x953d6ae3, 33 },
    { 0, 0 },
    { 0x551d6ae6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23d6ae9, 66 },
    { 0x992d6ae9, 34 },
    { 0xf06d6aea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbad6aef, 55 },
    { 0, 0 },
    { 0xd97d6af1, 61 },
    { 0x9dbd6af2, 98 },
    { 0, 0 },
    { 0xe66d6af4, 76 },
    { 0, 0 },
    { 0xadcd6af6, 90 },
    { 0, 0 },
    { 0x6ecd6af8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5d6afd, 61 },
    { 0x982d6afe, 90 },
    { 0xd02d6aff, 33 },
    { 0x5c3d6afe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740d6b06, 46 },
    { 0xf74d6b07, 95 },
    { 0x7aed6b08, 76 },
    { 0, 0 },
    { 0xdd0d6b0a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cd6b0d, 88 },
    { 0xdf3d6b0d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ad6b12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4d6b19, 66 },
    { 0x2c7d6b1a, 61 },
    { 0x81ad6b1b, 90 },
    { 0xdf4d6b19, 76 },
    { 0xed8d6b1d, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7d6b20, 84 },
    { 0x9f1d6b21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d6b27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ed6b2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aed6b2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x139d6b32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefd6b36, 33 },
    { 0x396d6b36, 76 },
    { 0xf85d6b38, 85 },
    { 0xef7d6b38, 80 },
    { 0x81ad6b38, 90 },
    { 0x2a1d6b3b, 40 },
    { 0x515d6b3c, 95 },
    { 0xc3bd6b3c, 43 },
    { 0xf0d6b3b, 61 },
    { 0x578d6b3f, 98 },
    { 0x868d6b40, 98 },
    { 0xc3ed6b41, 91 },
    { 0x9b8d6b40, 93 },
    { 0x6a6d6b3c, 33 },
    { 0x2fd6b40, 34 },
    { 0x2e5d6b45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883d6b4a, 81 },
    { 0x78fd6b4b, 93 },
    { 0x781d6b4a, 33 },
    { 0xd04d6b4a, 91 },
    { 0x882d6b4e, 66 },
    { 0xeabd6b4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffd6b55, 98 },
    { 0xd27d6b55, 90 },
    { 0x59ad6b57, 90 },
    { 0x23d6b57, 43 },
    { 0xb65d6b59, 98 },
    { 0xa0ed6b5a, 94 },
    { 0xd7cd6b59, 23 },
    { 0x94cd6b5c, 98 },
    { 0xb58d6b5c, 33 },
    { 0x616d6b5e, 95 },
    { 0xb7cd6b5c, 90 },
    { 0x66bd6b5a, 98 },
    { 0x8fad6b61, 95 },
    { 0xeb7d6b5a, 90 },
    { 0x295d6b58, 99 },
    { 0xb01d6b60, 33 },
    { 0x73ed6b5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edd6b6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848d6b72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ad6b7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7d6b7f, 95 },
    { 0x9a1d6b7f, 55 },
    { 0, 0 },
    { 0x2aed6b82, 33 },
    { 0xf48d6b83, 66 },
    { 0xe35d6b83, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bed6b87, 33 },
    { 0, 0 },
    { 0x2e1d6b89, 23 },
    { 0xa86d6b89, 61 },
    { 0x8b8d6b8b, 90 },
    { 0, 0 },
    { 0xdd2d6b8d, 90 },
    { 0, 0 },
    { 0x3c6d6b8f, 23 },
    { 0xbead6b8f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13d6b95, 84 },
    { 0x331d6b96, 90 },
    { 0x7a9d6b97, 98 },
    { 0x579d6b97, 81 },
    { 0x9a2d6b97, 98 },
    { 0, 0 },
    { 0x6f8d6b9b, 81 },
    { 0x61dd6b9b, 75 },
    { 0x651d6b9c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10d6ba3, 61 },
    { 0x5a8d6ba4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967d6baa, 90 },
    { 0xfcdd6baa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2d6bae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8d6bb4, 90 },
    { 0xbecd6bb4, 17 },
    { 0xd2fd6bb6, 61 },
    { 0, 0 },
    { 0xb3dd6bb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x102d6bbb, 98 },
    { 0x2edd6bbc, 67 },
    { 0x18bd6bbb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafd6bc0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x947d6bc4, 34 },
    { 0, 0 },
    { 0x62fd6bc6, 48 },
    { 0x4f7d6bc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952d6bcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cd6bd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecd6bdb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84d6bdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0d6be2, 95 },
    { 0, 0 },
    { 0x785d6be4, 23 },
    { 0, 0 },
    { 0x1bad6be6, 33 },
    { 0x958d6be7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x456d6bea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760d6bf1, 90 },
    { 0x8d6bf1, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cd6bf5, 90 },
    { 0xcfad6bf5, 61 },
    { 0, 0 },
    { 0xe35d6bf8, 85 },
    { 0, 0 },
    { 0xfafd6bfa, 33 },
    { 0, 0 },
    { 0xc10d6bfc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x274d6bff, 46 },
    { 0x90fd6bff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x105d6c03, 75 },
    { 0x80ad6c03, 66 },
    { 0xcf9d6c05, 33 },
    { 0x629d6c05, 33 },
    { 0x5ded6c07, 90 },
    { 0xc84d6c08, 33 },
    { 0xa40d6c09, 84 },
    { 0x88fd6c09, 33 },
    { 0xe64d6c0b, 43 },
    { 0xf9dd6c08, 34 },
    { 0x423d6c08, 80 },
    { 0x496d6c0e, 95 },
    { 0x19dd6c0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dad6c19, 96 },
    { 0xaf0d6c1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbd6c1e, 76 },
    { 0xadcd6c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4d6c22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cd6c30, 90 },
    { 0x4acd6c30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7d6c38, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ad6c3f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797d6c43, 90 },
    { 0, 0 },
    { 0xadbd6c45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fd6c4c, 33 },
    { 0, 0 },
    { 0x45cd6c4e, 61 },
    { 0, 0 },
    { 0x2b8d6c50, 90 },
    { 0, 0 },
    { 0x88ed6c52, 93 },
    { 0xa66d6c53, 43 },
    { 0x319d6c53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dd6c57, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d6c5a, 90 },
    { 0xa88d6c5a, 98 },
    { 0x480d6c5c, 99 },
    { 0x5f1d6c5d, 77 },
    { 0x2ebd6c5d, 99 },
    { 0xfebd6c5a, 76 },
    { 0, 0 },
    { 0x6c6d6c61, 98 },
    { 0x804d6c62, 46 },
    { 0x510d6c63, 95 },
    { 0xb42d6c64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0d6c67, 91 },
    { 0, 0 },
    { 0x3f7d6c69, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbad6c6c, 98 },
    { 0xcedd6c6d, 61 },
    { 0x1ced6c6e, 33 },
    { 0xb27d6c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68d6c77, 61 },
    { 0, 0 },
    { 0xeded6c79, 33 },
    { 0x8c7d6c7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05d6c7f, 91 },
    { 0x3c2d6c7f, 23 },
    { 0xc02d6c7f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ed6c84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4d6c88, 100 },
    { 0xb9cd6c89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cd6c8c, 91 },
    { 0xe16d6c8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbd6c92, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad6c99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d6c9c, 90 },
    { 0xf87d6c9d, 33 },
    { 0xb2ed6c9c, 100 },
    { 0, 0 },
    { 0xb77d6ca0, 95 },
    { 0xe70d6ca1, 23 },
    { 0xf1dd6ca1, 90 },
    { 0xbbdd6ca3, 86 },
    { 0x4c5d6ca4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcd6cac, 23 },
    { 0x41fd6cac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad6cb4, 99 },
    { 0, 0 },
    { 0x4b5d6cb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6d6cbe, 23 },
    { 0xe00d6cbf, 90 },
    { 0xfe2d6cc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d6cc3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bd6cc6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0d6ccb, 61 },
    { 0xb3dd6ccb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37d6ccf, 98 },
    { 0xef4d6cd0, 33 },
    { 0x6ed6cd1, 46 },
    { 0x773d6cd2, 23 },
    { 0, 0 },
    { 0x630d6cd4, 99 },
    { 0x787d6cd4, 91 },
    { 0, 0 },
    { 0xce7d6cd7, 33 },
    { 0, 0 },
    { 0x4e4d6cd9, 61 },
    { 0x59ed6cd9, 33 },
    { 0x65fd6cdb, 94 },
    { 0, 0 },
    { 0xa63d6cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310d6ce2, 33 },
    { 0x1e3d6ce3, 95 },
    { 0x622d6ce4, 55 },
    { 0xebad6ce2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x578d6ce8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214d6ced, 95 },
    { 0xbe0d6ced, 98 },
    { 0xbbed6cef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x454d6cf2, 55 },
    { 0x295d6cf3, 95 },
    { 0x8bfd6cf3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72d6cfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67d6d00, 90 },
    { 0, 0 },
    { 0xd8d6d02, 95 },
    { 0xc5ed6d02, 90 },
    { 0x20ed6d04, 43 },
    { 0xc6bd6d05, 76 },
    { 0xb4bd6d06, 90 },
    { 0xe27d6d06, 99 },
    { 0x725d6d08, 23 },
    { 0xf94d6d09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12d6d12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d6d1e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbd6d24, 85 },
    { 0, 0 },
    { 0x28d6d26, 90 },
    { 0, 0 },
    { 0x23dd6d28, 38 },
    { 0x9e6d6d28, 34 },
    { 0, 0 },
    { 0x536d6d2b, 98 },
    { 0, 0 },
    { 0xa12d6d2d, 43 },
    { 0xec5d6d2e, 33 },
    { 0xe18d6d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abd6d34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed6d37, 98 },
    { 0x394d6d38, 68 },
    { 0x50bd6d37, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bd6d3c, 23 },
    { 0x15ad6d3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d6d41, 26 },
    { 0x220d6d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaded6d47, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cd6d50, 98 },
    { 0, 0 },
    { 0xa47d6d52, 23 },
    { 0x8d4d6d52, 40 },
    { 0x89ad6d54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60d6d57, 33 },
    { 0x7b5d6d58, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dd6d5c, 84 },
    { 0, 0 },
    { 0x9a1d6d5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537d6d63, 76 },
    { 0x39d6d63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994d6d6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cad6d71, 99 },
    { 0x4e3d6d72, 33 },
    { 0xdb1d6d72, 95 },
    { 0x935d6d74, 33 },
    { 0x9bad6d73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x317d6d78, 34 },
    { 0, 0 },
    { 0xcbdd6d7a, 38 },
    { 0, 0 },
    { 0xf12d6d7c, 98 },
    { 0x465d6d7d, 33 },
    { 0xe2ad6d7d, 93 },
    { 0x648d6d7c, 33 },
    { 0, 0 },
    { 0xe14d6d81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddd6d87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0d6d8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eed6d8f, 23 },
    { 0, 0 },
    { 0x450d6d91, 87 },
    { 0, 0 },
    { 0xe59d6d93, 33 },
    { 0x37dd6d93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fd6d98, 46 },
    { 0, 0 },
    { 0x9b1d6d9a, 43 },
    { 0x6c4d6d9b, 100 },
    { 0x7b6d6d9b, 90 },
    { 0xe38d6d9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943d6da4, 38 },
    { 0xad7d6da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398d6dab, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710d6db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cd6db3, 78 },
    { 0x885d6db3, 95 },
    { 0x6b7d6db5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x759d6db8, 84 },
    { 0, 0 },
    { 0x491d6dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dd6dc0, 92 },
    { 0, 0 },
    { 0x822d6dc2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cd6dc6, 67 },
    { 0xed2d6dc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7d6dcf, 90 },
    { 0, 0 },
    { 0x3ccd6dd1, 97 },
    { 0xf67d6dd1, 55 },
    { 0x43cd6dd3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fad6de1, 95 },
    { 0xb58d6de2, 58 },
    { 0xbbed6de2, 90 },
    { 0xb24d6de4, 43 },
    { 0xce6d6de5, 66 },
    { 0, 0 },
    { 0xcd3d6de7, 61 },
    { 0, 0 },
    { 0x26bd6de9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68d6df6, 98 },
    { 0xa72d6df6, 90 },
    { 0x597d6df8, 91 },
    { 0xe42d6df9, 91 },
    { 0x59d6df7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ad6dfe, 76 },
    { 0x879d6dff, 55 },
    { 0, 0 },
    { 0x46bd6e01, 53 },
    { 0xc06d6e01, 73 },
    { 0, 0 },
    { 0x847d6e04, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bd6e07, 55 },
    { 0xc19d6e08, 77 },
    { 0x311d6e09, 91 },
    { 0x10cd6e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235d6e11, 95 },
    { 0xf38d6e11, 93 },
    { 0x428d6e12, 33 },
    { 0x3bad6e14, 93 },
    { 0, 0 },
    { 0xbaad6e16, 78 },
    { 0, 0 },
    { 0x5d6e18, 90 },
    { 0, 0 },
    { 0x354d6e1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x225d6e1d, 23 },
    { 0, 0 },
    { 0x1c5d6e1f, 34 },
    { 0, 0 },
    { 0x7edd6e21, 90 },
    { 0xc80d6e22, 67 },
    { 0x8dcd6e23, 99 },
    { 0xf54d6e23, 56 },
    { 0, 0 },
    { 0xc2fd6e26, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbd6e2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d6e2f, 61 },
    { 0x583d6e2f, 96 },
    { 0x356d6e2f, 95 },
    { 0, 0 },
    { 0x4f3d6e33, 87 },
    { 0, 0 },
    { 0xe80d6e35, 90 },
    { 0, 0 },
    { 0x73dd6e37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd6e3d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bd6e41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1d6e45, 61 },
    { 0, 0 },
    { 0xf99d6e47, 80 },
    { 0x8a5d6e47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56d6e4e, 34 },
    { 0xa17d6e4e, 33 },
    { 0x977d6e50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ad6e54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288d6e5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308d6e63, 23 },
    { 0x241d6e63, 90 },
    { 0xa78d6e65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bd6e6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fd6e6e, 66 },
    { 0xf2fd6e6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20d6e72, 81 },
    { 0xb58d6e72, 33 },
    { 0xa04d6e74, 68 },
    { 0x3ded6e75, 88 },
    { 0x93fd6e76, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ad6e7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08d6e81, 23 },
    { 0xf85d6e82, 61 },
    { 0x5e4d6e83, 90 },
    { 0xef7d6e84, 78 },
    { 0x5efd6e84, 34 },
    { 0x236d6e85, 23 },
    { 0x7b8d6e87, 38 },
    { 0x6b5d6e87, 90 },
    { 0, 0 },
    { 0xa8ad6e8a, 90 },
    { 0x1d4d6e8b, 95 },
    { 0x3fdd6e8c, 36 },
    { 0xb5ed6e8d, 76 },
    { 0x29ad6e8a, 90 },
    { 0x897d6e8a, 46 },
    { 0x579d6e8b, 34 },
    { 0, 0 },
    { 0x22ad6e92, 43 },
    { 0xb42d6e92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94d6e9e, 84 },
    { 0, 0 },
    { 0x387d6ea0, 33 },
    { 0x913d6ea1, 33 },
    { 0, 0 },
    { 0x4b3d6ea3, 55 },
    { 0, 0 },
    { 0x5bad6ea5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d6ea8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffd6eab, 55 },
    { 0, 0 },
    { 0x9c3d6ead, 98 },
    { 0xa24d6eae, 23 },
    { 0x24fd6eae, 33 },
    { 0x17d6eb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afd6eb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fd6ebc, 61 },
    { 0x667d6ebd, 34 },
    { 0x562d6ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8d6ec3, 95 },
    { 0x1d8d6ec3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dd6ec7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbd6ecb, 95 },
    { 0x15d6ecc, 84 },
    { 0x255d6ecc, 90 },
    { 0xbf3d6ece, 90 },
    { 0xe5ad6ece, 17 },
    { 0, 0 },
    { 0xa52d6ed1, 100 },
    { 0, 0 },
    { 0xa39d6ed3, 77 },
    { 0xd5dd6ed3, 90 },
    { 0x7c9d6ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d6edd, 55 },
    { 0, 0 },
    { 0x935d6edf, 61 },
    { 0x138d6ee0, 91 },
    { 0xb8ed6ee1, 23 },
    { 0x2c3d6ee2, 64 },
    { 0x8dbd6ee3, 90 },
    { 0x941d6ee2, 52 },
    { 0x1e9d6ee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d6ee8, 43 },
    { 0, 0 },
    { 0xac8d6eea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7d6eed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d6ef1, 90 },
    { 0xba9d6ef2, 23 },
    { 0, 0 },
    { 0x54cd6ef4, 33 },
    { 0, 0 },
    { 0x60dd6ef6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415d6efa, 90 },
    { 0x7e4d6efb, 95 },
    { 0x256d6efa, 33 },
    { 0, 0 },
    { 0xa9bd6efe, 94 },
    { 0x7dcd6eff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ed6f02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85d6f0d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fd6f18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621d6f1f, 85 },
    { 0x6d3d6f20, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdd6f24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5d6f27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92d6f2b, 76 },
    { 0x366d6f2c, 61 },
    { 0x3e5d6f2d, 33 },
    { 0xb71d6f2e, 100 },
    { 0xb1cd6f2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbd6f33, 90 },
    { 0x4f0d6f34, 76 },
    { 0xacdd6f35, 43 },
    { 0, 0 },
    { 0x94d6f37, 23 },
    { 0x7add6f38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6d6f3f, 61 },
    { 0x8c5d6f40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x349d6f43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7d6f47, 81 },
    { 0x23d6f47, 90 },
    { 0x1e4d6f49, 86 },
    { 0x1e7d6f4a, 90 },
    { 0xfdfd6f4b, 95 },
    { 0x98dd6f4c, 90 },
    { 0, 0 },
    { 0xfdbd6f4e, 93 },
    { 0x395d6f4e, 55 },
    { 0x781d6f50, 75 },
    { 0x24ed6f51, 90 },
    { 0, 0 },
    { 0xddad6f53, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0d6f56, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4d6f5e, 90 },
    { 0x2dfd6f5e, 34 },
    { 0, 0 },
    { 0x2aad6f61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3d6f65, 85 },
    { 0, 0 },
    { 0x3ed6f67, 23 },
    { 0xaa2d6f68, 61 },
    { 0, 0 },
    { 0xfa3d6f6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2d6f72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x714d6f75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ad6f78, 90 },
    { 0x9dbd6f79, 33 },
    { 0x29ad6f79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713d6f7e, 34 },
    { 0xd30d6f7f, 33 },
    { 0x4ed6f80, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18d6f89, 34 },
    { 0xd23d6f8a, 46 },
    { 0x83d6f89, 61 },
    { 0x7ced6f89, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d6f92, 95 },
    { 0, 0 },
    { 0x7dbd6f94, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fd6f98, 95 },
    { 0xaf2d6f99, 61 },
    { 0, 0 },
    { 0x958d6f9b, 90 },
    { 0x13ed6f9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd6fa0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x321d6fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fd6fa7, 87 },
    { 0xc87d6fa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89d6fab, 95 },
    { 0x2ecd6fac, 23 },
    { 0x456d6fad, 90 },
    { 0, 0 },
    { 0xd80d6faf, 95 },
    { 0, 0 },
    { 0xd94d6fb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ed6fb4, 34 },
    { 0x15d6fb5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fd6fb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30d6fbb, 92 },
    { 0xbfbd6fbb, 61 },
    { 0x379d6fbc, 33 },
    { 0x154d6fbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed6fc5, 98 },
    { 0, 0 },
    { 0x9fbd6fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9add6fcc, 98 },
    { 0x534d6fcd, 85 },
    { 0xf90d6fcd, 66 },
    { 0, 0 },
    { 0x3e2d6fd0, 87 },
    { 0, 0 },
    { 0xb1bd6fd2, 95 },
    { 0x56ad6fd2, 91 },
    { 0, 0 },
    { 0x773d6fd5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900d6fde, 61 },
    { 0x290d6fdf, 95 },
    { 0x44d6fdf, 76 },
    { 0x12ad6fde, 99 },
    { 0x240d6fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dd6fe7, 85 },
    { 0x94ad6fe7, 55 },
    { 0, 0 },
    { 0x872d6fea, 84 },
    { 0x279d6feb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbd6fee, 23 },
    { 0x68fd6fef, 91 },
    { 0, 0 },
    { 0x5f4d6ff1, 33 },
    { 0x772d6ff2, 95 },
    { 0, 0 },
    { 0xe0fd6ff4, 74 },
    { 0xe35d6ff4, 99 },
    { 0x76dd6ff5, 90 },
    { 0x358d6ff5, 93 },
    { 0x15bd6ff8, 55 },
    { 0, 0 },
    { 0x7d1d6ffa, 61 },
    { 0x75cd6ffb, 17 },
    { 0xbb2d6ffc, 23 },
    { 0x879d6ffb, 64 },
    { 0x813d6ffc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4d7006, 66 },
    { 0, 0 },
    { 0x824d7008, 34 },
    { 0x9afd7009, 90 },
    { 0, 0 },
    { 0x7edd700b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4d700f, 95 },
    { 0xb85d7010, 17 },
    { 0xad2d7011, 76 },
    { 0x780d7012, 90 },
    { 0xfb8d7012, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66d7019, 98 },
    { 0xeb6d701a, 66 },
    { 0xef9d701a, 98 },
    { 0xcddd7019, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ad701f, 66 },
    { 0, 0 },
    { 0x994d7021, 94 },
    { 0x7fcd7021, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cd7031, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6d7036, 95 },
    { 0xefcd7037, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20d703d, 66 },
    { 0xa11d703e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcd7041, 95 },
    { 0x9c8d7042, 87 },
    { 0, 0 },
    { 0x640d7044, 90 },
    { 0xef0d7045, 61 },
    { 0, 0 },
    { 0x741d7047, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x495d704a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6d7051, 66 },
    { 0x9bbd7052, 33 },
    { 0x73ed7052, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ad705b, 24 },
    { 0, 0 },
    { 0xaf5d705d, 34 },
    { 0, 0 },
    { 0x37cd705f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d7063, 34 },
    { 0, 0 },
    { 0x64d7065, 93 },
    { 0, 0 },
    { 0xa3bd7067, 95 },
    { 0xc45d7068, 55 },
    { 0, 0 },
    { 0x432d706a, 91 },
    { 0, 0 },
    { 0x81bd706c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd7071, 95 },
    { 0xd09d7072, 34 },
    { 0, 0 },
    { 0xa15d7074, 76 },
    { 0x606d7074, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ad707a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x900d707d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ed7083, 91 },
    { 0xd86d7083, 98 },
    { 0, 0 },
    { 0xe02d7086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26d7092, 17 },
    { 0, 0 },
    { 0xc17d7094, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228d7099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ad709c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cd709f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dd70a2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19d70a6, 98 },
    { 0xc16d70a6, 33 },
    { 0xf7dd70a8, 99 },
    { 0x90fd70a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707d70b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fd70b3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ad70b8, 61 },
    { 0xb00d70b9, 55 },
    { 0xdf2d70ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d70c2, 33 },
    { 0, 0 },
    { 0x3e5d70c4, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92d70c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad70cb, 88 },
    { 0x850d70cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed70cf, 90 },
    { 0x577d70d0, 95 },
    { 0, 0 },
    { 0xe7fd70d2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593d70d6, 33 },
    { 0xbf0d70d7, 90 },
    { 0x7ffd70d6, 81 },
    { 0xb2ed70d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x320d70dc, 90 },
    { 0x811d70dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbd70e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x532d70e3, 34 },
    { 0xd8cd70e4, 61 },
    { 0xaced70e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fd70ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289d70f2, 23 },
    { 0, 0 },
    { 0x653d70f4, 95 },
    { 0x7aad70f4, 95 },
    { 0xe4bd70f6, 61 },
    { 0xc0ed70f6, 33 },
    { 0xb4d70f6, 8 },
    { 0, 0 },
    { 0x1cfd70fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1d7100, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dd7107, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d7111, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcd7116, 33 },
    { 0, 0 },
    { 0x72bd7118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cd711b, 85 },
    { 0xcb8d711c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfd7126, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bd7129, 95 },
    { 0xbaed712a, 93 },
    { 0xbd712a, 52 },
    { 0xd81d712c, 52 },
    { 0xebad712d, 95 },
    { 0, 0 },
    { 0x2d712f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06d7132, 61 },
    { 0x4ebd7133, 84 },
    { 0, 0 },
    { 0x669d7135, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebd713b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1d7141, 34 },
    { 0, 0 },
    { 0x649d7143, 90 },
    { 0x115d7144, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x927d7147, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2d714c, 23 },
    { 0xc63d714d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ed7155, 91 },
    { 0xfedd7156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84d715d, 98 },
    { 0, 0 },
    { 0xe9d715f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ad7166, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45d716a, 76 },
    { 0xca5d716b, 76 },
    { 0xe6dd716c, 98 },
    { 0x9e7d716d, 90 },
    { 0xbded716b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15d7174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x759d7177, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dd717c, 90 },
    { 0, 0 },
    { 0x73d717e, 75 },
    { 0xc30d717e, 87 },
    { 0x572d717f, 23 },
    { 0x72cd7181, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d7185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9d718b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cd7190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dd7198, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04d719b, 90 },
    { 0xf5fd719c, 33 },
    { 0x7f0d719c, 55 },
    { 0x6b5d719e, 91 },
    { 0xe3fd719f, 90 },
    { 0x17ed71a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0d71a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6d71a9, 61 },
    { 0x4b5d71aa, 55 },
    { 0, 0 },
    { 0xba5d71ac, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7d71b0, 98 },
    { 0x19cd71b0, 88 },
    { 0x55dd71b1, 67 },
    { 0x38dd71b3, 55 },
    { 0xe32d71b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fd71ba, 95 },
    { 0x2d6d71ba, 95 },
    { 0xf55d71bc, 95 },
    { 0, 0 },
    { 0x4fbd71be, 90 },
    { 0xbd4d71be, 90 },
    { 0x327d71c0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dd71c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x691d71ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ed71cd, 65 },
    { 0, 0 },
    { 0x272d71cf, 90 },
    { 0x1c3d71d0, 99 },
    { 0, 0 },
    { 0xc13d71d2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6d71d5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cd71d8, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9d71db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267d71e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd71e3, 33 },
    { 0xb08d71e4, 23 },
    { 0x208d71e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7d71e9, 23 },
    { 0xedcd71ea, 100 },
    { 0x60cd71eb, 76 },
    { 0x1dcd71ec, 90 },
    { 0x545d71ed, 43 },
    { 0x640d71ee, 61 },
    { 0x3d2d71ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x653d71f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78d71f5, 77 },
    { 0, 0 },
    { 0x6fd71f7, 38 },
    { 0xbd7d71f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfd71fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ed71fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301d7204, 77 },
    { 0xf7ed7205, 76 },
    { 0xf20d7206, 100 },
    { 0x52d7205, 43 },
    { 0xe59d7208, 90 },
    { 0x6f0d7209, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5d720d, 95 },
    { 0xdb5d720e, 90 },
    { 0x76ad720d, 99 },
    { 0x817d720e, 23 },
    { 0x17bd7210, 33 },
    { 0x813d7212, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbd7215, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9d7219, 46 },
    { 0xdeed721a, 46 },
    { 0xaafd721a, 90 },
    { 0xc9dd721b, 61 },
    { 0x543d721d, 84 },
    { 0xe93d721e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dd7222, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfd7225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d722a, 46 },
    { 0xf36d722a, 85 },
    { 0x4dad722c, 76 },
    { 0x3a5d722b, 23 },
    { 0xf0fd722e, 10 },
    { 0xb8ed722d, 43 },
    { 0x9a1d722e, 33 },
    { 0x1ebd722c, 84 },
    { 0xeb0d7232, 78 },
    { 0xc18d722e, 61 },
    { 0xd94d7234, 61 },
    { 0xeefd7234, 61 },
    { 0x8bad722e, 90 },
    { 0x888d7237, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321d723b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afd723e, 93 },
    { 0xfcd723e, 95 },
    { 0x6c0d723e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1d7244, 43 },
    { 0xfcfd7244, 78 },
    { 0x261d7244, 55 },
    { 0x15ad7247, 95 },
    { 0x79d7246, 24 },
    { 0xda7d7249, 33 },
    { 0x516d7247, 55 },
    { 0x8c2d724b, 68 },
    { 0, 0 },
    { 0xaa4d724d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dad7250, 42 },
    { 0xe8cd7250, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5d7256, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbced725b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518d725f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5d7262, 61 },
    { 0, 0 },
    { 0xa72d7264, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xebad7267, 33 },
    { 0xab8d7268, 90 },
    { 0x376d7268, 48 },
    { 0, 0 },
    { 0x92bd726b, 61 },
    { 0x63bd726c, 76 },
    { 0x77ed726c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101d7271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4d7275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1d7278, 93 },
    { 0x82dd7278, 75 },
    { 0, 0 },
    { 0x1acd727b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8d727e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bd7281, 95 },
    { 0x16d7282, 95 },
    { 0x721d7283, 84 },
    { 0x596d7281, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2d7289, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18d728c, 90 },
    { 0, 0 },
    { 0x842d728e, 95 },
    { 0x38dd728f, 95 },
    { 0x9c1d7290, 96 },
    { 0xc8ad7291, 33 },
    { 0x553d728f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92d7295, 61 },
    { 0x855d7296, 95 },
    { 0, 0 },
    { 0x370d7298, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01d729b, 61 },
    { 0, 0 },
    { 0x891d729d, 91 },
    { 0xa74d729e, 95 },
    { 0x946d729f, 66 },
    { 0x3ffd72a0, 93 },
    { 0, 0 },
    { 0x63cd72a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6d72a9, 36 },
    { 0xcfd72aa, 33 },
    { 0x5aad72ab, 93 },
    { 0xb91d72ab, 88 },
    { 0xe23d72ad, 95 },
    { 0x6d9d72ae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7d72b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0d72b7, 33 },
    { 0, 0 },
    { 0x534d72b9, 53 },
    { 0xc55d72ba, 55 },
    { 0, 0 },
    { 0xcdcd72bc, 98 },
    { 0x781d72bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad72c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4d72c7, 80 },
    { 0, 0 },
    { 0xe11d72c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ad72ce, 66 },
    { 0xb5ed72ce, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47d72d4, 98 },
    { 0xcb8d72d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4d72d8, 40 },
    { 0x39dd72d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cd72e0, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dd72e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ed72e8, 99 },
    { 0, 0 },
    { 0xe5ad72ea, 90 },
    { 0xe6d72eb, 78 },
    { 0, 0 },
    { 0x550d72ed, 90 },
    { 0, 0 },
    { 0x64bd72ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9d72f3, 98 },
    { 0x4fad72f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ad72f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ad72fc, 61 },
    { 0x87bd72fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456d7303, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4d7307, 90 },
    { 0xe39d7308, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfd730e, 87 },
    { 0, 0 },
    { 0xd07d7310, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd7314, 55 },
    { 0xf76d7314, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x865d7318, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadd731d, 2 },
    { 0x49ad731e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ad7322, 43 },
    { 0x814d7323, 46 },
    { 0x917d7322, 61 },
    { 0xf15d7323, 97 },
    { 0xe39d7326, 75 },
    { 0, 0 },
    { 0xb8dd7328, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28d732c, 42 },
    { 0x35cd732d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d7332, 87 },
    { 0, 0 },
    { 0x245d7334, 100 },
    { 0x20d7335, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x391d7338, 61 },
    { 0x5e9d7339, 90 },
    { 0xf32d7338, 61 },
    { 0, 0 },
    { 0x6c2d733c, 76 },
    { 0, 0 },
    { 0x51ed733e, 33 },
    { 0x833d733f, 98 },
    { 0xfd3d7340, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81d7346, 23 },
    { 0, 0 },
    { 0xb82d7348, 98 },
    { 0xadfd7348, 23 },
    { 0x152d7349, 98 },
    { 0, 0 },
    { 0x7a5d734c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878d7352, 33 },
    { 0, 0 },
    { 0x1c5d7354, 17 },
    { 0, 0 },
    { 0xa6bd7356, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x261d7359, 76 },
    { 0x6a5d7359, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2d735d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x666d7364, 91 },
    { 0xde0d7365, 90 },
    { 0, 0 },
    { 0x3c9d7367, 98 },
    { 0, 0 },
    { 0xa5cd7369, 95 },
    { 0x551d736a, 64 },
    { 0xd16d736b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cd736e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfd7371, 98 },
    { 0, 0 },
    { 0x5dbd7373, 76 },
    { 0x3f7d7373, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31d7380, 66 },
    { 0xeced7381, 75 },
    { 0, 0 },
    { 0x21cd7383, 90 },
    { 0x2bad7384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40d7387, 100 },
    { 0x8b8d7387, 95 },
    { 0xc5cd7388, 90 },
    { 0xe5dd738a, 90 },
    { 0x1a2d738b, 55 },
    { 0x7f3d738c, 45 },
    { 0xe8cd738d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3d7391, 23 },
    { 0x68dd7392, 52 },
    { 0x28ed7392, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cad7396, 33 },
    { 0xaaad7396, 90 },
    { 0x9dd7397, 95 },
    { 0x466d7399, 95 },
    { 0x6bdd739a, 90 },
    { 0x933d739b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd739f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75d73a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dd73a5, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ad73aa, 61 },
    { 0x36dd73ab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8d73ae, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835d73b2, 78 },
    { 0, 0 },
    { 0xbbd73b4, 76 },
    { 0x8cad73b4, 23 },
    { 0xd26d73b6, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcd73b9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dd73c1, 84 },
    { 0x236d73c1, 61 },
    { 0x49d73c2, 90 },
    { 0, 0 },
    { 0x137d73c5, 61 },
    { 0x8e6d73c6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9d73ce, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291d73e4, 23 },
    { 0xa5cd73e5, 76 },
    { 0, 0 },
    { 0xbc8d73e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9d73ea, 98 },
    { 0, 0 },
    { 0x4d3d73ec, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8d73ef, 69 },
    { 0x332d73f0, 90 },
    { 0, 0 },
    { 0x264d73f2, 43 },
    { 0x4e2d73f2, 52 },
    { 0x909d73f4, 77 },
    { 0x9a4d73f3, 33 },
    { 0, 0 },
    { 0x571d73f7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342d73fd, 98 },
    { 0xd04d73fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6d7401, 33 },
    { 0x892d7402, 88 },
    { 0xbbed7402, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71d7406, 33 },
    { 0, 0 },
    { 0x9f4d7408, 55 },
    { 0, 0 },
    { 0x40ed740a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbd7410, 98 },
    { 0xa14d7411, 99 },
    { 0x91fd7411, 98 },
    { 0, 0 },
    { 0x6c3d7414, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0d7417, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6d741c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cd7420, 17 },
    { 0, 0 },
    { 0xd0bd7422, 98 },
    { 0, 0 },
    { 0x530d7424, 23 },
    { 0, 0 },
    { 0xe05d7426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fd742c, 81 },
    { 0, 0 },
    { 0xbc4d742e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ed7437, 33 },
    { 0x9f9d7438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbd743b, 68 },
    { 0xcd5d743b, 33 },
    { 0, 0 },
    { 0x962d743e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741d7443, 90 },
    { 0x7fcd7444, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557d7448, 33 },
    { 0x80ed7448, 95 },
    { 0x56ed744a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ded7452, 95 },
    { 0xd62d7452, 76 },
    { 0xa89d7454, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3d7457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774d745b, 76 },
    { 0xe1fd745c, 33 },
    { 0xb58d745d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28d7461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39d7466, 98 },
    { 0, 0 },
    { 0xb0cd7468, 43 },
    { 0, 0 },
    { 0x51ad746a, 52 },
    { 0x991d746a, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dd7472, 95 },
    { 0xfeed7472, 66 },
    { 0x924d7474, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7d7477, 95 },
    { 0x956d7478, 33 },
    { 0x7d1d7479, 98 },
    { 0x893d7478, 95 },
    { 0x4e0d7477, 75 },
    { 0xc89d747c, 98 },
    { 0xb02d747d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6d7483, 61 },
    { 0x3ffd7483, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6d748d, 43 },
    { 0xbdcd748d, 90 },
    { 0xeb5d748f, 55 },
    { 0, 0 },
    { 0xa0ed7491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753d7495, 95 },
    { 0xd91d7496, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57d7499, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74dd749c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x363d749f, 90 },
    { 0, 0 },
    { 0x148d74a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5d74a5, 55 },
    { 0xd2bd74a6, 33 },
    { 0xee9d74a6, 61 },
    { 0, 0 },
    { 0x41fd74a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328d74ae, 90 },
    { 0xa30d74af, 40 },
    { 0x5b4d74b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01d74ba, 95 },
    { 0xbe8d74bb, 90 },
    { 0xed3d74ba, 23 },
    { 0xb27d74bd, 61 },
    { 0xac0d74be, 76 },
    { 0xe2d74bf, 61 },
    { 0x90dd74c0, 33 },
    { 0xae6d74c1, 61 },
    { 0xaa1d74c2, 95 },
    { 0xeccd74c3, 76 },
    { 0xf8d74c4, 95 },
    { 0xad5d74ba, 33 },
    { 0xcf6d74c4, 93 },
    { 0, 0 },
    { 0x12cd74c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x807d74cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362d74d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7d74d3, 34 },
    { 0, 0 },
    { 0xb27d74d5, 33 },
    { 0, 0 },
    { 0x4ced74d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6d74db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ad74e5, 87 },
    { 0, 0 },
    { 0x5e3d74e7, 43 },
    { 0, 0 },
    { 0xc65d74e9, 85 },
    { 0, 0 },
    { 0x469d74eb, 43 },
    { 0, 0 },
    { 0x1f6d74ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x250d74f0, 90 },
    { 0xa1ed74f0, 61 },
    { 0xbf9d74f2, 98 },
    { 0, 0 },
    { 0x988d74f4, 90 },
    { 0, 0 },
    { 0xa48d74f6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dd74f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cad7507, 79 },
    { 0xb7bd7508, 65 },
    { 0x1a8d7509, 98 },
    { 0xd86d7509, 33 },
    { 0x10d750b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x751d750e, 61 },
    { 0, 0 },
    { 0xeb5d7510, 61 },
    { 0, 0 },
    { 0x71ad7512, 78 },
    { 0, 0 },
    { 0x7d4d7514, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7d751d, 33 },
    { 0, 0 },
    { 0x4f1d751f, 98 },
    { 0, 0 },
    { 0x9b7d7521, 33 },
    { 0x2b5d7521, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x865d7525, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd752e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x541d7531, 17 },
    { 0x241d7532, 61 },
    { 0x8ad7532, 33 },
    { 0xf48d7534, 23 },
    { 0, 0 },
    { 0xbadd7536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad753d, 98 },
    { 0x82bd753e, 76 },
    { 0, 0 },
    { 0xe50d7540, 88 },
    { 0x6efd7540, 90 },
    { 0, 0 },
    { 0xfa6d7543, 93 },
    { 0x780d7543, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375d7548, 95 },
    { 0x93dd7548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcd7550, 55 },
    { 0xbe0d7551, 55 },
    { 0, 0 },
    { 0x7d8d7553, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79d755b, 78 },
    { 0x94fd755c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54d7567, 90 },
    { 0, 0 },
    { 0x8acd7569, 55 },
    { 0xdc7d756a, 90 },
    { 0xdb4d756a, 95 },
    { 0xa0ad756b, 98 },
    { 0, 0 },
    { 0x16ed756e, 90 },
    { 0x388d756f, 67 },
    { 0xa44d7570, 97 },
    { 0x82fd7570, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0d7576, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fd757a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4d757f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ed7585, 55 },
    { 0x895d7586, 52 },
    { 0x673d7587, 61 },
    { 0x1aad7588, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x803d758b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bd758f, 33 },
    { 0x9efd7590, 55 },
    { 0xeb8d7590, 67 },
    { 0x13cd7592, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dd7599, 55 },
    { 0x6c0d759a, 91 },
    { 0xda0d759a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcd759e, 61 },
    { 0, 0 },
    { 0x28d75a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0d75a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x229d75a9, 76 },
    { 0, 0 },
    { 0xefd75ab, 90 },
    { 0x837d75ac, 87 },
    { 0x2add75ad, 84 },
    { 0, 0 },
    { 0x663d75af, 33 },
    { 0, 0 },
    { 0xcedd75b1, 33 },
    { 0x708d75b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602d75b6, 100 },
    { 0x48dd75b7, 33 },
    { 0xa0ad75b8, 95 },
    { 0, 0 },
    { 0x66cd75ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79d75bd, 76 },
    { 0x967d75be, 93 },
    { 0, 0 },
    { 0x59ed75c0, 84 },
    { 0, 0 },
    { 0x5fbd75c2, 84 },
    { 0x65bd75c3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ad75c6, 76 },
    { 0xc4d75c7, 55 },
    { 0xb03d75c8, 61 },
    { 0xafcd75c8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fd75cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8d75d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5d75db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9d75e0, 90 },
    { 0x7f1d75e1, 55 },
    { 0xc1ad75e2, 33 },
    { 0x313d75e3, 98 },
    { 0x5bfd75e3, 95 },
    { 0xda6d75e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefd75ea, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6d75ee, 68 },
    { 0x4b3d75ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13d75f8, 61 },
    { 0x820d75f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8d75fe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6d7607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d760e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175d7613, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cd7617, 90 },
    { 0x433d7618, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985d7625, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fd762a, 64 },
    { 0xfb7d762b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bd762e, 90 },
    { 0x8d6d762f, 23 },
    { 0, 0 },
    { 0xf14d7631, 55 },
    { 0, 0 },
    { 0xb4d7633, 98 },
    { 0, 0 },
    { 0x47ad7635, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced7638, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadd763e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42d7642, 55 },
    { 0x515d7642, 34 },
    { 0xde3d7644, 98 },
    { 0x931d7642, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d7648, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cd764d, 55 },
    { 0xc9fd764d, 84 },
    { 0, 0 },
    { 0x1efd7650, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8d7655, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcd7658, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd765b, 61 },
    { 0x4ebd765c, 33 },
    { 0xe6bd765b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95dd7660, 87 },
    { 0x69ad7660, 90 },
    { 0xb57d7660, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x765d7665, 61 },
    { 0x1dd7666, 43 },
    { 0x9fed7665, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd766e, 93 },
    { 0x9d8d766f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0d7675, 61 },
    { 0x308d7675, 34 },
    { 0xa8d7675, 61 },
    { 0xfb0d7678, 69 },
    { 0x40fd7675, 98 },
    { 0x303d767a, 33 },
    { 0xd05d7675, 90 },
    { 0, 0 },
    { 0x28ad767d, 33 },
    { 0xc8cd767e, 90 },
    { 0xdd2d767e, 33 },
    { 0x3d3d7680, 90 },
    { 0xaeed767f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead7684, 90 },
    { 0x3fbd7685, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cd7688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcd7690, 78 },
    { 0xc81d7690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38d7694, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x539d7697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912d769c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6dd76a0, 95 },
    { 0, 0 },
    { 0x33dd76a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9d76a6, 68 },
    { 0, 0 },
    { 0xd41d76a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0d76b5, 77 },
    { 0x910d76b6, 33 },
    { 0x369d76b7, 61 },
    { 0x7c8d76b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dd76bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dd76c1, 46 },
    { 0xcd2d76c2, 68 },
    { 0xf57d76c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fd76c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d76cf, 99 },
    { 0x967d76d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdced76d5, 64 },
    { 0xd9bd76d6, 66 },
    { 0x322d76d7, 98 },
    { 0, 0 },
    { 0x819d76d9, 93 },
    { 0xf23d76da, 76 },
    { 0xe69d76d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53d76de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21d76e1, 84 },
    { 0x254d76e2, 87 },
    { 0x16d76e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425d76ed, 33 },
    { 0, 0 },
    { 0xd20d76ef, 61 },
    { 0xf68d76f0, 98 },
    { 0, 0 },
    { 0xa7cd76f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bd76f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fd76fc, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753d7700, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcd7704, 73 },
    { 0x8e7d7705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdd7709, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d770d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bd7710, 77 },
    { 0x3b9d7711, 88 },
    { 0x537d7710, 95 },
    { 0, 0 },
    { 0x69dd7714, 67 },
    { 0xcf6d7715, 87 },
    { 0x5aad7714, 33 },
    { 0xbb5d7717, 23 },
    { 0x5ddd7717, 96 },
    { 0xca4d7719, 90 },
    { 0, 0 },
    { 0x975d771b, 33 },
    { 0x80dd771b, 34 },
    { 0, 0 },
    { 0xbe4d771e, 24 },
    { 0, 0 },
    { 0xe35d7720, 77 },
    { 0, 0 },
    { 0x4ed7722, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cd7725, 23 },
    { 0x703d7726, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead772a, 46 },
    { 0x17ad772a, 90 },
    { 0xba5d772c, 99 },
    { 0x82fd772d, 84 },
    { 0x63d772e, 94 },
    { 0x630d772f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2d7735, 61 },
    { 0, 0 },
    { 0xcbed7737, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516d7740, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d7744, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x770d7747, 33 },
    { 0xc5fd7747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5d774c, 95 },
    { 0xcafd774d, 34 },
    { 0x7fbd774d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1d7751, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0d7755, 78 },
    { 0xc6fd7755, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x889d7759, 61 },
    { 0xfa6d775a, 98 },
    { 0xec9d775b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74d775f, 76 },
    { 0, 0 },
    { 0xc9ad7761, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cd7766, 90 },
    { 0x99dd7766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed776b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53d776f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cd7772, 23 },
    { 0xd32d7773, 88 },
    { 0x900d7772, 90 },
    { 0x3fad7774, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d7779, 77 },
    { 0x1fed777a, 66 },
    { 0xcb6d777b, 61 },
    { 0x27bd777c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed7781, 84 },
    { 0x18bd7782, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7d7785, 46 },
    { 0x4acd7785, 61 },
    { 0x56ed7785, 55 },
    { 0x394d7788, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfd778b, 61 },
    { 0xd5fd778c, 98 },
    { 0, 0 },
    { 0xd37d778e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d7795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edd779a, 76 },
    { 0x59d779b, 76 },
    { 0x25dd779a, 34 },
    { 0, 0 },
    { 0xfe4d779e, 61 },
    { 0xfbad779f, 95 },
    { 0x262d77a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79d77a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38d77a8, 46 },
    { 0x335d77a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84d77ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aad77b2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1d77b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdd77ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cd77be, 76 },
    { 0xf9dd77bf, 93 },
    { 0, 0 },
    { 0x1a7d77c1, 76 },
    { 0x508d77c2, 90 },
    { 0x822d77c1, 93 },
    { 0xc2dd77c4, 23 },
    { 0xcb5d77c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3d77c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfead77cc, 98 },
    { 0xddad77cd, 91 },
    { 0xb85d77ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7d77d2, 93 },
    { 0x81fd77d3, 53 },
    { 0, 0 },
    { 0xe66d77d5, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d77db, 91 },
    { 0, 0 },
    { 0x858d77dd, 91 },
    { 0x2fad77de, 61 },
    { 0xbebd77df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x460d77e2, 93 },
    { 0, 0 },
    { 0x733d77e4, 55 },
    { 0xc4dd77e5, 90 },
    { 0xa6bd77e6, 90 },
    { 0x922d77e7, 78 },
    { 0x67bd77e5, 33 },
    { 0xb14d77e9, 90 },
    { 0x848d77ea, 75 },
    { 0xbb1d77eb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23d77ee, 52 },
    { 0, 0 },
    { 0xa20d77f0, 55 },
    { 0x1edd77f1, 95 },
    { 0, 0 },
    { 0xbccd77f3, 90 },
    { 0x9b3d77f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9d77f7, 61 },
    { 0xad2d77f8, 2 },
    { 0x723d77f8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dd77fd, 100 },
    { 0x97ed77fe, 46 },
    { 0x367d77fd, 99 },
    { 0x103d7800, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72d7804, 98 },
    { 0, 0 },
    { 0x122d7806, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd780b, 76 },
    { 0x502d780c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55d7812, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfd7817, 90 },
    { 0xb78d7818, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3d7820, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7823, 61 },
    { 0x76bd7824, 46 },
    { 0x25dd7825, 76 },
    { 0x5d9d7826, 76 },
    { 0, 0 },
    { 0x9c9d7828, 61 },
    { 0xef9d7829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d782e, 99 },
    { 0x619d782f, 61 },
    { 0xa06d782e, 81 },
    { 0xdf0d7831, 76 },
    { 0xa4cd7832, 81 },
    { 0x805d7833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7d7838, 38 },
    { 0x82dd7839, 43 },
    { 0x796d783a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abd783e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4d7845, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cd784b, 98 },
    { 0xf55d784b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1d7854, 90 },
    { 0xa60d7855, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4d7859, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d785c, 61 },
    { 0x7f0d785c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccd7862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceed7866, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bd7869, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1d786e, 78 },
    { 0xf9fd786f, 95 },
    { 0, 0 },
    { 0x24fd7871, 33 },
    { 0xf15d7872, 33 },
    { 0xc74d7872, 55 },
    { 0x25d7873, 77 },
    { 0x6ffd7874, 75 },
    { 0xa63d7876, 98 },
    { 0x17cd7877, 76 },
    { 0, 0 },
    { 0xd9bd7879, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350d787f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9d7883, 91 },
    { 0, 0 },
    { 0x81fd7885, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4d788a, 93 },
    { 0x50cd788a, 64 },
    { 0, 0 },
    { 0x324d788d, 55 },
    { 0x6c8d788e, 80 },
    { 0, 0 },
    { 0xdb0d7890, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cd7894, 90 },
    { 0, 0 },
    { 0x4d0d7896, 55 },
    { 0, 0 },
    { 0xa20d7898, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x384d789b, 52 },
    { 0, 0 },
    { 0x2bdd789d, 23 },
    { 0x26cd789e, 90 },
    { 0, 0 },
    { 0xbd2d78a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbd78a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaed78a8, 33 },
    { 0x54dd78a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdd78ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ed78b1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bd78b7, 100 },
    { 0, 0 },
    { 0x350d78b9, 61 },
    { 0, 0 },
    { 0x7bed78bb, 23 },
    { 0, 0 },
    { 0x1c2d78bd, 46 },
    { 0xeeed78bd, 33 },
    { 0xc62d78bf, 90 },
    { 0x8f0d78bd, 55 },
    { 0, 0 },
    { 0x7abd78c2, 86 },
    { 0x407d78c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x623d78c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fd78cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfd78d3, 76 },
    { 0, 0 },
    { 0xfddd78d5, 90 },
    { 0, 0 },
    { 0xf52d78d7, 90 },
    { 0, 0 },
    { 0xf43d78d9, 55 },
    { 0x7d7d78da, 55 },
    { 0, 0 },
    { 0xfadd78dc, 61 },
    { 0xbdbd78dd, 90 },
    { 0x544d78de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03d78e3, 33 },
    { 0x9e4d78e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ad78e7, 75 },
    { 0, 0 },
    { 0xb03d78e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615d78ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ad78f2, 23 },
    { 0x5ad78f3, 61 },
    { 0xa8ad78f3, 55 },
    { 0xfe4d78f5, 100 },
    { 0xcacd78f5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bad78fa, 95 },
    { 0x953d78fa, 85 },
    { 0, 0 },
    { 0xfe4d78fd, 43 },
    { 0x60bd78fe, 66 },
    { 0xc70d78fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11d7903, 61 },
    { 0x342d7903, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d790a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x945d790d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abd7910, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd791d, 55 },
    { 0xc00d791d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecd7921, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2d7928, 83 },
    { 0, 0 },
    { 0x66cd792a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6d792d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868d7931, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0d7934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ed7937, 55 },
    { 0xfdfd7937, 43 },
    { 0, 0 },
    { 0xe6ad793a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd793e, 70 },
    { 0, 0 },
    { 0xfc1d7940, 34 },
    { 0x74ad7940, 61 },
    { 0x375d7942, 23 },
    { 0xeed7943, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x316d7947, 23 },
    { 0, 0 },
    { 0x1e0d7949, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x598d794c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad7953, 95 },
    { 0xe3bd7953, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802d795c, 90 },
    { 0, 0 },
    { 0x565d795e, 55 },
    { 0, 0 },
    { 0x467d7960, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcd7964, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cd7968, 65 },
    { 0x57ad7968, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1d796c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6d7971, 61 },
    { 0, 0 },
    { 0x6dcd7973, 64 },
    { 0xd8fd7974, 43 },
    { 0x5bad7975, 77 },
    { 0, 0 },
    { 0x22ed7977, 34 },
    { 0xff8d7978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd797c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ed7982, 95 },
    { 0x31ed7982, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34d7987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd798b, 46 },
    { 0, 0 },
    { 0x6f8d798d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ed7991, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07d7994, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cd799a, 90 },
    { 0, 0 },
    { 0x308d799c, 61 },
    { 0x86ed799d, 95 },
    { 0x17ed799e, 95 },
    { 0x982d799f, 61 },
    { 0x3ccd799d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8d79a6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22d79a9, 69 },
    { 0xeddd79aa, 24 },
    { 0xa2ad79aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337d79af, 98 },
    { 0x683d79b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0d79b4, 90 },
    { 0xb81d79b5, 68 },
    { 0xccbd79b6, 98 },
    { 0, 0 },
    { 0x6f6d79b8, 97 },
    { 0x755d79b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bd79c1, 93 },
    { 0xb55d79c1, 52 },
    { 0x1e2d79c3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03d79c6, 95 },
    { 0xe38d79c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405d79d0, 55 },
    { 0x84d79d1, 14 },
    { 0x9bed79d2, 34 },
    { 0x29ad79d3, 84 },
    { 0x3d9d79d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d79d8, 98 },
    { 0x226d79d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05d79dc, 81 },
    { 0xa1ed79dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fd79e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfd79e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826d79ed, 34 },
    { 0xbced79ee, 90 },
    { 0, 0 },
    { 0x672d79f0, 90 },
    { 0x249d79f1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0d79f8, 34 },
    { 0, 0 },
    { 0xa06d79fa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38d79ff, 33 },
    { 0xebbd7a00, 61 },
    { 0x216d7a01, 90 },
    { 0x8ed7a02, 91 },
    { 0x1d7d7a01, 61 },
    { 0, 0 },
    { 0x906d7a05, 98 },
    { 0x93d7a05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x968d7a09, 61 },
    { 0x8b5d7a0a, 87 },
    { 0x614d7a0b, 33 },
    { 0x3ead7a0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcd7a10, 100 },
    { 0, 0 },
    { 0x45cd7a12, 94 },
    { 0, 0 },
    { 0xa5d7a14, 90 },
    { 0x3aed7a15, 78 },
    { 0x52fd7a15, 38 },
    { 0, 0 },
    { 0x9acd7a18, 23 },
    { 0x3c1d7a19, 98 },
    { 0x895d7a18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x40d7a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecd7a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d7a27, 61 },
    { 0xd9cd7a28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x793d7a2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4d7a33, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45d7a36, 33 },
    { 0x345d7a37, 90 },
    { 0x2e9d7a37, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d7a41, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ad7a48, 46 },
    { 0, 0 },
    { 0x278d7a4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d7a4d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dcd7a53, 90 },
    { 0x99d7a54, 52 },
    { 0xc12d7a54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3d7a5a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0d7a60, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbd7a63, 98 },
    { 0, 0 },
    { 0xf15d7a65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61d7a69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4d7a6c, 93 },
    { 0xf65d7a6d, 23 },
    { 0xa73d7a6e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437d7a78, 33 },
    { 0, 0 },
    { 0x94fd7a7a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94d7a7f, 95 },
    { 0xc62d7a80, 23 },
    { 0x74dd7a80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x906d7a84, 17 },
    { 0xb8dd7a85, 23 },
    { 0x136d7a84, 19 },
    { 0x3dfd7a84, 99 },
    { 0, 0 },
    { 0xffdd7a89, 76 },
    { 0xb8d7a89, 84 },
    { 0x4c5d7a8a, 88 },
    { 0xb3ed7a8c, 95 },
    { 0x4c8d7a8d, 65 },
    { 0xd65d7a8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edd7a94, 90 },
    { 0x75dd7a94, 33 },
    { 0, 0 },
    { 0x3efd7a97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d7a9a, 90 },
    { 0x147d7a9a, 56 },
    { 0xd89d7a9a, 43 },
    { 0xbc3d7a9d, 76 },
    { 0xf8ed7a9b, 90 },
    { 0xcdfd7a9f, 90 },
    { 0x84d7a9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38d7aa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308d7aa9, 76 },
    { 0x7ced7aaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cd7ab1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d7ab4, 87 },
    { 0xe28d7ab4, 95 },
    { 0x279d7ab6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fd7abd, 61 },
    { 0x808d7abe, 84 },
    { 0x4ded7abd, 67 },
    { 0xbc7d7abe, 33 },
    { 0, 0 },
    { 0x3b7d7ac2, 76 },
    { 0x8c0d7ac3, 66 },
    { 0xd6dd7ac3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66d7ac8, 76 },
    { 0x57cd7ac8, 55 },
    { 0, 0 },
    { 0xae0d7acb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ad7ace, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21d7ad3, 33 },
    { 0x4a8d7ad4, 23 },
    { 0x8edd7ad5, 95 },
    { 0xf8dd7ad6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45d7ada, 95 },
    { 0x8d7adb, 95 },
    { 0xeb6d7adb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabd7ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9d7aeb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6d7aee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bd7af1, 60 },
    { 0x73dd7af2, 95 },
    { 0x85ed7af3, 78 },
    { 0x12bd7af4, 98 },
    { 0x2d8d7af5, 76 },
    { 0xacd7af3, 23 },
    { 0x797d7af7, 98 },
    { 0x4dfd7af3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8d7aff, 43 },
    { 0, 0 },
    { 0x1cd7b01, 98 },
    { 0x9e8d7b02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x575d7b05, 33 },
    { 0x50dd7b06, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23d7b0c, 55 },
    { 0xc98d7b0d, 68 },
    { 0x40d7b0e, 95 },
    { 0xa0dd7b0f, 53 },
    { 0, 0 },
    { 0x756d7b11, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdd7b14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1d7b17, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fed7b1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x211d7b1d, 90 },
    { 0xafcd7b1e, 95 },
    { 0x7f1d7b1e, 66 },
    { 0, 0 },
    { 0xe7bd7b21, 78 },
    { 0xf78d7b21, 77 },
    { 0x3dad7b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ed7b27, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x410d7b2a, 98 },
    { 0x388d7b2b, 85 },
    { 0x151d7b2a, 95 },
    { 0x1b6d7b2d, 78 },
    { 0x195d7b2e, 33 },
    { 0x123d7b2c, 66 },
    { 0xf1ad7b2f, 100 },
    { 0xb6dd7b31, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x854d7b34, 90 },
    { 0, 0 },
    { 0xf15d7b36, 34 },
    { 0x4abd7b37, 90 },
    { 0, 0 },
    { 0x3d5d7b39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9d7b3d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553d7b42, 93 },
    { 0x4d1d7b43, 61 },
    { 0xfe6d7b44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddcd7b4c, 61 },
    { 0xf73d7b4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d7b53, 55 },
    { 0x775d7b53, 95 },
    { 0x2d4d7b55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5add7b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8d7b5f, 95 },
    { 0x3a6d7b60, 93 },
    { 0xb41d7b61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03d7b64, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293d7b69, 34 },
    { 0, 0 },
    { 0xd15d7b6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273d7b74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682d7b7d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd7b80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d7b87, 100 },
    { 0xe0ed7b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446d7b96, 95 },
    { 0, 0 },
    { 0x2aad7b98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bd7b9b, 43 },
    { 0, 0 },
    { 0xc1dd7b9d, 33 },
    { 0x1c5d7b9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7d7ba5, 55 },
    { 0x291d7ba6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bd7baf, 85 },
    { 0xc16d7bb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194d7bb7, 23 },
    { 0x17ad7bb8, 95 },
    { 0, 0 },
    { 0xc8cd7bba, 77 },
    { 0, 0 },
    { 0x453d7bbc, 76 },
    { 0xb4ed7bbd, 76 },
    { 0, 0 },
    { 0xe1bd7bbf, 66 },
    { 0, 0 },
    { 0xb69d7bc1, 61 },
    { 0xd14d7bc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcd7bca, 33 },
    { 0x1d3d7bcb, 90 },
    { 0x881d7bcb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201d7bd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bd7bd5, 52 },
    { 0, 0 },
    { 0xfdfd7bd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ed7bdc, 33 },
    { 0x9e4d7bdd, 90 },
    { 0, 0 },
    { 0x850d7bdf, 93 },
    { 0x3e9d7be0, 95 },
    { 0x159d7be0, 85 },
    { 0x532d7be2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8d7be5, 61 },
    { 0x587d7be5, 23 },
    { 0xab3d7be7, 77 },
    { 0, 0 },
    { 0xda1d7be9, 88 },
    { 0xb78d7be9, 61 },
    { 0xe77d7be9, 46 },
    { 0, 0 },
    { 0x753d7bed, 33 },
    { 0xd98d7bee, 91 },
    { 0, 0 },
    { 0xe63d7bf0, 101 },
    { 0, 0 },
    { 0xe5fd7bf2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08d7bf5, 78 },
    { 0x8a1d7bf6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7d7bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcd7bfc, 90 },
    { 0, 0 },
    { 0x566d7bfe, 34 },
    { 0, 0 },
    { 0xdd6d7c00, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38d7c03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47d7c07, 61 },
    { 0xdccd7c08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cd7c10, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73d7c13, 68 },
    { 0x128d7c14, 77 },
    { 0x2b1d7c14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147d7c19, 55 },
    { 0x81fd7c1a, 46 },
    { 0x7d0d7c1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1d7c21, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7c24, 98 },
    { 0x3dcd7c25, 34 },
    { 0x898d7c26, 52 },
    { 0, 0 },
    { 0xcf6d7c28, 98 },
    { 0xb38d7c29, 95 },
    { 0, 0 },
    { 0xa76d7c2b, 23 },
    { 0x108d7c2c, 33 },
    { 0xc1ad7c2c, 95 },
    { 0xf79d7c2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ed7c32, 61 },
    { 0x74ad7c33, 52 },
    { 0x7a7d7c33, 55 },
    { 0x3c6d7c35, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x856d7c38, 34 },
    { 0xc99d7c39, 52 },
    { 0, 0 },
    { 0x84ed7c3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bd7c42, 98 },
    { 0x7c4d7c43, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0d7c46, 76 },
    { 0xa4dd7c47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ed7c4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dd7c4f, 88 },
    { 0x602d7c50, 88 },
    { 0x673d7c51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643d7c5d, 76 },
    { 0, 0 },
    { 0x50fd7c5f, 23 },
    { 0xf31d7c5f, 61 },
    { 0xd5d7c61, 91 },
    { 0x45cd7c61, 61 },
    { 0x7abd7c62, 43 },
    { 0xe43d7c61, 55 },
    { 0x31d7c65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d7c6d, 76 },
    { 0x34d7c6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9d7c72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x242d7c75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cd7c78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d7c7d, 95 },
    { 0xd44d7c7e, 77 },
    { 0xcbcd7c7e, 95 },
    { 0, 0 },
    { 0x687d7c81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd7c8b, 98 },
    { 0, 0 },
    { 0xe5dd7c8d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcd7c92, 76 },
    { 0xa71d7c92, 55 },
    { 0xa38d7c94, 95 },
    { 0, 0 },
    { 0x338d7c96, 76 },
    { 0, 0 },
    { 0xfb6d7c98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d7c9b, 76 },
    { 0x748d7c9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84d7ca1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad7ca6, 55 },
    { 0xf6dd7ca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d7cae, 90 },
    { 0xb75d7caf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2d7cb7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d7cbb, 61 },
    { 0x6b8d7cbc, 84 },
    { 0x239d7cbc, 34 },
    { 0x495d7cbe, 61 },
    { 0xd1dd7cbe, 76 },
    { 0x54d7cbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd7cc6, 23 },
    { 0x91fd7cc7, 55 },
    { 0x496d7cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0d7ccc, 77 },
    { 0xafad7ccc, 93 },
    { 0, 0 },
    { 0xe00d7ccf, 77 },
    { 0x2d2d7cd0, 52 },
    { 0x4dcd7cd1, 23 },
    { 0xc41d7cd0, 95 },
    { 0xb0cd7cd3, 61 },
    { 0x8ecd7cd1, 61 },
    { 0x7b6d7cd5, 98 },
    { 0x29cd7cd6, 34 },
    { 0x982d7cd7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dd7cde, 23 },
    { 0x4ffd7cde, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x647d7ce2, 98 },
    { 0xf81d7ce3, 33 },
    { 0x49cd7ce4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53d7ce8, 98 },
    { 0xad6d7ce9, 98 },
    { 0x5a3d7ce8, 95 },
    { 0x352d7ceb, 33 },
    { 0xc9d7cec, 90 },
    { 0xd08d7ced, 85 },
    { 0, 0 },
    { 0xbb7d7cef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ad7cf3, 91 },
    { 0x9d0d7cf4, 76 },
    { 0, 0 },
    { 0x73dd7cf6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791d7cfa, 23 },
    { 0x964d7cfb, 90 },
    { 0xe3ed7cfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dd7cff, 100 },
    { 0xf96d7d00, 33 },
    { 0, 0 },
    { 0xaf6d7d02, 76 },
    { 0xa3fd7d02, 76 },
    { 0x645d7d02, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d7d07, 84 },
    { 0x6fbd7d08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6d7d0b, 23 },
    { 0x96bd7d0c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d7d10, 69 },
    { 0, 0 },
    { 0x87cd7d12, 33 },
    { 0xfa4d7d13, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69d7d16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d7d19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90d7d1d, 65 },
    { 0x2a0d7d1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeed7d21, 95 },
    { 0x613d7d22, 46 },
    { 0x2cfd7d22, 52 },
    { 0x96ed7d23, 33 },
    { 0x4c4d7d24, 61 },
    { 0x96ad7d26, 90 },
    { 0xe20d7d27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbced7d2c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187d7d30, 75 },
    { 0x72dd7d31, 90 },
    { 0xe7bd7d32, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dd7d43, 33 },
    { 0xc62d7d44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bad7d4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ed7d4f, 90 },
    { 0x643d7d4f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dd7d56, 98 },
    { 0, 0 },
    { 0xd90d7d58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7d5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69d7d64, 52 },
    { 0x59bd7d64, 95 },
    { 0xa48d7d66, 61 },
    { 0xb61d7d67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24d7d72, 90 },
    { 0x890d7d72, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd7d78, 43 },
    { 0x26cd7d78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149d7d81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d7d88, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4d7d8c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d7d8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdad7d93, 84 },
    { 0x7cbd7d94, 33 },
    { 0xc2cd7d94, 61 },
    { 0xd22d7d96, 36 },
    { 0x270d7d94, 100 },
    { 0, 0 },
    { 0xdcdd7d99, 33 },
    { 0x6efd7d9a, 95 },
    { 0, 0 },
    { 0x8dd7d9c, 65 },
    { 0xc7dd7d9d, 33 },
    { 0xa0cd7d9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4d7da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1d7da7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fd7dab, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343d7daf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acd7db4, 20 },
    { 0x6e5d7db4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6d7dba, 96 },
    { 0x7ecd7dbb, 33 },
    { 0x22dd7dbc, 61 },
    { 0x44bd7dbd, 88 },
    { 0x94fd7dbe, 98 },
    { 0xa25d7dbf, 98 },
    { 0xf66d7dc0, 33 },
    { 0, 0 },
    { 0x417d7dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770d7dc6, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ad7dc9, 98 },
    { 0, 0 },
    { 0xc8d7dcb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bd7dd2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248d7dd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cd7ddd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3d7de2, 33 },
    { 0xfcfd7de3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x598d7de6, 90 },
    { 0x1c8d7de7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fd7dea, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bd7df0, 93 },
    { 0xde1d7df1, 34 },
    { 0xe2bd7df1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8d7dfd, 61 },
    { 0, 0 },
    { 0xaed7dff, 33 },
    { 0xd06d7e00, 88 },
    { 0xb55d7e00, 23 },
    { 0, 0 },
    { 0x1c1d7e03, 61 },
    { 0xd83d7e04, 65 },
    { 0xe85d7e05, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x335d7e08, 68 },
    { 0xdddd7e08, 33 },
    { 0x99fd7e08, 84 },
    { 0x392d7e09, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfd7e0e, 72 },
    { 0, 0 },
    { 0xf8d7e10, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9d7e14, 34 },
    { 0x841d7e15, 78 },
    { 0x831d7e15, 61 },
    { 0x9bfd7e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x546d7e1a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eed7e1e, 90 },
    { 0, 0 },
    { 0xedfd7e20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1d7e24, 61 },
    { 0, 0 },
    { 0xbcbd7e26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcd7e29, 33 },
    { 0, 0 },
    { 0xc19d7e2b, 23 },
    { 0x345d7e2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ed7e32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x708d7e35, 33 },
    { 0x668d7e36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ad7e39, 98 },
    { 0, 0 },
    { 0xf16d7e3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64d7e3e, 76 },
    { 0, 0 },
    { 0x3ffd7e40, 76 },
    { 0, 0 },
    { 0x4d7d7e42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53d7e45, 98 },
    { 0x767d7e46, 90 },
    { 0x332d7e45, 43 },
    { 0, 0 },
    { 0x8f3d7e49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0d7e52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0d7e56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2d7e5a, 98 },
    { 0xc77d7e5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x651d7e5e, 81 },
    { 0xfb4d7e5e, 98 },
    { 0xb4dd7e60, 95 },
    { 0xd4fd7e61, 91 },
    { 0x947d7e61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646d7e66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x290d7e69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d7e6c, 61 },
    { 0x65ad7e6d, 33 },
    { 0x5f3d7e6d, 34 },
    { 0x99cd7e6f, 23 },
    { 0xa38d7e70, 43 },
    { 0, 0 },
    { 0x86ad7e72, 95 },
    { 0x62bd7e73, 90 },
    { 0, 0 },
    { 0x477d7e75, 90 },
    { 0x210d7e76, 38 },
    { 0x695d7e75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88d7e7b, 61 },
    { 0, 0 },
    { 0xf46d7e7d, 34 },
    { 0x685d7e7e, 61 },
    { 0x55bd7e7f, 61 },
    { 0xdc0d7e80, 34 },
    { 0xae7d7e7f, 90 },
    { 0xb3bd7e82, 52 },
    { 0x36ed7e83, 98 },
    { 0xfe9d7e84, 90 },
    { 0xc2ed7e84, 76 },
    { 0xebfd7e86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cad7e8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43d7e9b, 61 },
    { 0xa30d7e9c, 90 },
    { 0xdcad7e9c, 61 },
    { 0x24ad7e9e, 91 },
    { 0x8cdd7e9d, 95 },
    { 0x9ded7ea0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d7ea7, 76 },
    { 0x132d7ea8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7eab, 61 },
    { 0x9a6d7eac, 90 },
    { 0, 0 },
    { 0x7aad7eae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd7eb1, 33 },
    { 0x89d7eb2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcd7eb5, 97 },
    { 0, 0 },
    { 0x491d7eb7, 78 },
    { 0xf39d7eb8, 33 },
    { 0x4a1d7eb9, 76 },
    { 0xbcfd7eba, 61 },
    { 0x9f0d7ebb, 90 },
    { 0x93ed7ebc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd7ec1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8d7ec5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d7ecb, 52 },
    { 0x77ad7ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4d7ecf, 36 },
    { 0, 0 },
    { 0x2c4d7ed1, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62d7ed4, 26 },
    { 0xc6cd7ed5, 90 },
    { 0, 0 },
    { 0x668d7ed7, 98 },
    { 0xa73d7ed8, 23 },
    { 0, 0 },
    { 0xeed7eda, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8d7edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdcd7ee0, 90 },
    { 0, 0 },
    { 0x441d7ee2, 90 },
    { 0x1e4d7ee3, 33 },
    { 0xeefd7ee2, 33 },
    { 0x52d7ee3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x753d7ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dd7ef0, 23 },
    { 0, 0 },
    { 0xf03d7ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7ef9, 84 },
    { 0x89ed7ef9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cd7efe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6d7f0e, 90 },
    { 0, 0 },
    { 0x2bad7f10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1d7f14, 91 },
    { 0x79fd7f14, 76 },
    { 0xfb2d7f16, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0d7f19, 33 },
    { 0x87d7f1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62d7f20, 33 },
    { 0x23cd7f21, 17 },
    { 0, 0 },
    { 0x6c8d7f23, 64 },
    { 0x6b1d7f23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fd7f27, 98 },
    { 0x128d7f27, 61 },
    { 0xe54d7f28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71d7f30, 90 },
    { 0, 0 },
    { 0xf3d7f32, 90 },
    { 0, 0 },
    { 0xda0d7f34, 33 },
    { 0x5fed7f35, 33 },
    { 0xe88d7f36, 55 },
    { 0x67ad7f37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7dd7f3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdd7f41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4d7f44, 34 },
    { 0, 0 },
    { 0x663d7f46, 61 },
    { 0x4c2d7f46, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dd7f4c, 23 },
    { 0, 0 },
    { 0xcbdd7f4e, 98 },
    { 0, 0 },
    { 0x807d7f50, 55 },
    { 0, 0 },
    { 0x500d7f52, 61 },
    { 0xbc5d7f53, 76 },
    { 0x88cd7f53, 34 },
    { 0, 0 },
    { 0xd34d7f56, 95 },
    { 0x136d7f57, 55 },
    { 0, 0 },
    { 0xa41d7f59, 93 },
    { 0, 0 },
    { 0xdc9d7f5b, 98 },
    { 0x834d7f5c, 23 },
    { 0, 0 },
    { 0x234d7f5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x230d7f61, 23 },
    { 0x2dd7f62, 95 },
    { 0x933d7f61, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83d7f6d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0d7f74, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873d7f7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd7f7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834d7f83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3d7f8c, 43 },
    { 0xa87d7f8d, 61 },
    { 0, 0 },
    { 0xe23d7f8f, 61 },
    { 0xc84d7f8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abd7f94, 95 },
    { 0x4e5d7f94, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddd7f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ad7f9e, 90 },
    { 0, 0 },
    { 0x730d7fa0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3d7fa5, 77 },
    { 0, 0 },
    { 0x41cd7fa7, 98 },
    { 0, 0 },
    { 0x64dd7fa9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbd7fad, 34 },
    { 0, 0 },
    { 0xdedd7faf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeead7fb6, 84 },
    { 0x7ccd7fb6, 34 },
    { 0x240d7fb7, 100 },
    { 0x59d7fb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1d7fbc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed7fc4, 66 },
    { 0x2fed7fc5, 33 },
    { 0xf09d7fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x783d7fc9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x552d7fcc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bd7fd2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75d7fd7, 33 },
    { 0x5b1d7fd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ed7fef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4d7ff4, 38 },
    { 0xe01d7ff5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x934d7ff8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bd7ffe, 91 },
    { 0xa26d7fff, 93 },
    { 0x270d8000, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0d8005, 33 },
    { 0x747d8006, 84 },
    { 0xd28d8007, 69 },
    { 0, 0 },
    { 0x1bd8009, 85 },
    { 0xe02d800a, 76 },
    { 0x3c6d800a, 61 },
    { 0, 0 },
    { 0x938d800d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bd8014, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdd8017, 46 },
    { 0xa68d8017, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x562d801b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74d801e, 90 },
    { 0, 0 },
    { 0x43d8020, 52 },
    { 0xd5cd8021, 98 },
    { 0, 0 },
    { 0xa7d8023, 76 },
    { 0x455d8024, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cd8028, 23 },
    { 0, 0 },
    { 0x714d802a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471d8030, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cd803a, 98 },
    { 0x1eed803b, 90 },
    { 0x999d803b, 93 },
    { 0, 0 },
    { 0x72ed803e, 33 },
    { 0, 0 },
    { 0x500d8040, 46 },
    { 0, 0 },
    { 0x4cbd8042, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1d8045, 95 },
    { 0, 0 },
    { 0xdefd8047, 90 },
    { 0x27d8047, 23 },
    { 0x7eed8049, 78 },
    { 0xb1fd8048, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xacad804d, 90 },
    { 0xa46d804d, 67 },
    { 0xe95d804e, 98 },
    { 0x96ed804f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ad8055, 90 },
    { 0xf2ad8055, 34 },
    { 0, 0 },
    { 0xe17d8058, 61 },
    { 0x348d8059, 36 },
    { 0x51ed8059, 66 },
    { 0, 0 },
    { 0xe9ad805c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d8062, 78 },
    { 0x7bad8063, 98 },
    { 0, 0 },
    { 0x9b4d8065, 23 },
    { 0x8f7d8066, 33 },
    { 0x4e6d8067, 90 },
    { 0x187d8066, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d8071, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d8078, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd807b, 75 },
    { 0x308d807c, 90 },
    { 0xc5ad807c, 95 },
    { 0xa8d807e, 33 },
    { 0x9e3d807c, 95 },
    { 0, 0 },
    { 0x307d8081, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7d8085, 78 },
    { 0xdc1d8086, 95 },
    { 0, 0 },
    { 0x854d8088, 95 },
    { 0xb8d8089, 23 },
    { 0x942d808a, 55 },
    { 0x8e7d808a, 23 },
    { 0x2c5d808a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfd8091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59d80a2, 55 },
    { 0xd05d80a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x58dd80a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddad80a9, 90 },
    { 0, 0 },
    { 0x293d80ab, 68 },
    { 0x69dd80ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x861d80af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880d80b3, 33 },
    { 0xc38d80b4, 67 },
    { 0x91bd80b5, 43 },
    { 0x7ccd80b5, 23 },
    { 0x740d80b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cd80bd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7d80c0, 77 },
    { 0, 0 },
    { 0x39ed80c2, 65 },
    { 0x4a4d80c3, 68 },
    { 0x61ed80c4, 23 },
    { 0, 0 },
    { 0xd66d80c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84d80cc, 65 },
    { 0xd7bd80cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fd80d9, 76 },
    { 0x3aad80d9, 23 },
    { 0, 0 },
    { 0x918d80dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3d80e2, 52 },
    { 0x9a2d80e3, 95 },
    { 0xaf2d80e4, 76 },
    { 0x5abd80e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad80e9, 90 },
    { 0, 0 },
    { 0xa9d80eb, 23 },
    { 0xed9d80ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549d80f1, 66 },
    { 0x10fd80f1, 95 },
    { 0x45fd80f3, 43 },
    { 0x38bd80f4, 23 },
    { 0x94bd80f5, 52 },
    { 0x8a1d80f6, 61 },
    { 0x1ced80f4, 95 },
    { 0xda1d80f8, 90 },
    { 0xad4d80f2, 61 },
    { 0xa2ed80f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cd80fe, 76 },
    { 0, 0 },
    { 0x8dad8100, 91 },
    { 0xc61d8100, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dd8111, 78 },
    { 0x71cd8112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x404d8115, 95 },
    { 0, 0 },
    { 0xf4cd8117, 23 },
    { 0, 0 },
    { 0xa75d8119, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbd811c, 90 },
    { 0, 0 },
    { 0x252d811e, 33 },
    { 0, 0 },
    { 0xc4fd8120, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60d8125, 93 },
    { 0, 0 },
    { 0x697d8127, 33 },
    { 0, 0 },
    { 0x245d8129, 40 },
    { 0xcfd8129, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7d812d, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9d8131, 43 },
    { 0xb0cd8132, 23 },
    { 0x62ed8133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fad8138, 23 },
    { 0x7d7d8139, 33 },
    { 0x5edd813a, 84 },
    { 0x650d8139, 90 },
    { 0xa79d813c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ad813f, 61 },
    { 0x927d8140, 90 },
    { 0x6a7d813f, 93 },
    { 0xb08d8142, 33 },
    { 0xf4dd813f, 33 },
    { 0xb32d8144, 91 },
    { 0x268d8144, 33 },
    { 0x9cbd8146, 99 },
    { 0x549d8141, 95 },
    { 0xc79d8148, 87 },
    { 0x9c8d8148, 85 },
    { 0xaa0d8143, 85 },
    { 0, 0 },
    { 0xd91d814c, 91 },
    { 0, 0 },
    { 0x7d2d814e, 33 },
    { 0x7a4d814f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddd8152, 98 },
    { 0xd45d8153, 23 },
    { 0xae2d8154, 95 },
    { 0xff2d8153, 95 },
    { 0, 0 },
    { 0xa30d8157, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfd8161, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfd8164, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20d8167, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132d816b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdd8174, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadd8179, 23 },
    { 0x461d817a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31d817d, 55 },
    { 0x199d817d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4d8181, 68 },
    { 0x5a3d8181, 34 },
    { 0xd08d8181, 46 },
    { 0xbb9d8184, 61 },
    { 0xdf5d8185, 34 },
    { 0xe95d8186, 84 },
    { 0, 0 },
    { 0x50fd8188, 52 },
    { 0, 0 },
    { 0x3b8d818a, 52 },
    { 0, 0 },
    { 0x76fd818c, 84 },
    { 0, 0 },
    { 0x544d818e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x154d8191, 76 },
    { 0xcd0d8191, 98 },
    { 0x3c8d8191, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacd8198, 76 },
    { 0xebfd8199, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ed81a7, 33 },
    { 0x779d81a7, 33 },
    { 0x157d81a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cd81ac, 33 },
    { 0, 0 },
    { 0x3ecd81ae, 78 },
    { 0xb47d81af, 95 },
    { 0x707d81b0, 33 },
    { 0xc8ed81b1, 98 },
    { 0xa8bd81b0, 23 },
    { 0, 0 },
    { 0x636d81b4, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dad81b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dd81be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7d81c1, 98 },
    { 0x8a2d81c1, 68 },
    { 0, 0 },
    { 0x6f5d81c4, 78 },
    { 0x2ed81c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398d81cb, 93 },
    { 0x70fd81cc, 90 },
    { 0x33ad81cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04d81d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490d81d8, 90 },
    { 0x6fdd81d9, 67 },
    { 0x6acd81da, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ed81e3, 61 },
    { 0x376d81e4, 55 },
    { 0x347d81e5, 98 },
    { 0, 0 },
    { 0xd12d81e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d81ef, 90 },
    { 0xb79d81f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x394d81f3, 75 },
    { 0x32d81f4, 34 },
    { 0xf83d81f5, 65 },
    { 0x889d81f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3d81f9, 90 },
    { 0, 0 },
    { 0x344d81fb, 98 },
    { 0, 0 },
    { 0xfbed81fd, 67 },
    { 0, 0 },
    { 0x374d81ff, 33 },
    { 0, 0 },
    { 0xae4d8201, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdd8205, 78 },
    { 0x226d8205, 90 },
    { 0x921d8207, 33 },
    { 0xbd8205, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332d8210, 98 },
    { 0x50cd8211, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ed8218, 33 },
    { 0x799d8218, 23 },
    { 0, 0 },
    { 0xa39d821b, 90 },
    { 0xec1d821c, 85 },
    { 0xdc9d821d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0d8222, 23 },
    { 0xe63d8223, 90 },
    { 0x4a9d8222, 78 },
    { 0x68bd8225, 87 },
    { 0x765d8225, 61 },
    { 0x85d8227, 61 },
    { 0xbdad8227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x843d822b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942d8231, 33 },
    { 0xd1fd8231, 95 },
    { 0x73fd8232, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cd8236, 51 },
    { 0, 0 },
    { 0x8a8d8238, 98 },
    { 0, 0 },
    { 0xcc1d823a, 43 },
    { 0, 0 },
    { 0x5ad823c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f5d8243, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd8246, 100 },
    { 0, 0 },
    { 0x273d8248, 34 },
    { 0x1f8d8249, 100 },
    { 0xe65d8248, 95 },
    { 0xdb2d8249, 93 },
    { 0xe59d824c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dd8251, 61 },
    { 0x245d8252, 90 },
    { 0xab2d8252, 55 },
    { 0x8a9d8254, 33 },
    { 0xe90d8255, 61 },
    { 0x685d8256, 84 },
    { 0xd04d8257, 90 },
    { 0xa3bd8254, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999d825f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24d8269, 98 },
    { 0xd17d8269, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ad8271, 90 },
    { 0x400d8271, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617d8276, 85 },
    { 0xd9ed8276, 83 },
    { 0xc13d8278, 100 },
    { 0, 0 },
    { 0x429d827a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619d8280, 90 },
    { 0x21cd8281, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54d8286, 33 },
    { 0, 0 },
    { 0xf97d8288, 90 },
    { 0, 0 },
    { 0xe4ed828a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x538d828d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2d8290, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdced8293, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x318d8296, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fd829e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863d82a6, 83 },
    { 0x356d82a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x21d82aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9d82b1, 76 },
    { 0, 0 },
    { 0x53fd82b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d82b8, 77 },
    { 0, 0 },
    { 0xafcd82ba, 61 },
    { 0, 0 },
    { 0xea4d82bc, 98 },
    { 0, 0 },
    { 0xb26d82be, 76 },
    { 0xff5d82bf, 87 },
    { 0, 0 },
    { 0x4d1d82c1, 90 },
    { 0xf5bd82c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32d82c6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dd82cf, 77 },
    { 0x6b6d82cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edd82d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ed82d9, 33 },
    { 0x4ecd82d9, 34 },
    { 0xd45d82db, 85 },
    { 0x27cd82db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02d82df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137d82e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cad82e7, 33 },
    { 0, 0 },
    { 0x73cd82e9, 98 },
    { 0xecdd82ea, 55 },
    { 0, 0 },
    { 0x3f1d82ec, 61 },
    { 0xa5cd82ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d82f0, 95 },
    { 0, 0 },
    { 0x6e4d82f2, 76 },
    { 0, 0 },
    { 0xc35d82f4, 55 },
    { 0xbd4d82f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfafd82f8, 34 },
    { 0xbe4d82f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cd82fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9d8304, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2d8309, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d830e, 43 },
    { 0x59bd830f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbd8317, 93 },
    { 0x92dd8318, 46 },
    { 0, 0 },
    { 0xcefd831a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1d831d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d8320, 96 },
    { 0x87ad8321, 55 },
    { 0xa60d8322, 23 },
    { 0, 0 },
    { 0x43d8324, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6d8329, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7d8330, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcd8341, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922d8345, 61 },
    { 0, 0 },
    { 0x52dd8347, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339d834d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3d8352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2d8356, 33 },
    { 0x83bd8356, 98 },
    { 0x613d8358, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cd835c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8d8362, 76 },
    { 0x1a8d8363, 69 },
    { 0, 0 },
    { 0x8bbd8365, 33 },
    { 0x9a4d8366, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0d8369, 33 },
    { 0xb4cd836a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3d836f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cd8374, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602d837b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461d8380, 87 },
    { 0xa6ed8381, 95 },
    { 0x33ad8382, 61 },
    { 0, 0 },
    { 0x62cd8384, 87 },
    { 0, 0 },
    { 0x483d8386, 87 },
    { 0x195d8386, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76d838a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabd838e, 34 },
    { 0xfd3d838f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fd839c, 34 },
    { 0xa34d839d, 61 },
    { 0x2f9d839e, 23 },
    { 0x8e6d839f, 23 },
    { 0, 0 },
    { 0x77ed83a1, 95 },
    { 0, 0 },
    { 0x305d83a3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad83ae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x31d83b1, 55 },
    { 0x274d83b1, 98 },
    { 0x135d83b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424d83bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724d83c8, 61 },
    { 0x999d83c9, 76 },
    { 0, 0 },
    { 0xf97d83cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85bd83d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecd83d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbd83db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x423d83de, 90 },
    { 0xb53d83df, 80 },
    { 0xd45d83e0, 55 },
    { 0xd42d83e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ad83e4, 95 },
    { 0, 0 },
    { 0xfa6d83e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1d83e9, 93 },
    { 0, 0 },
    { 0x752d83eb, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ed83f0, 98 },
    { 0x6b3d83f0, 81 },
    { 0x23bd83f2, 77 },
    { 0xf98d83f0, 34 },
    { 0, 0 },
    { 0x55fd83f5, 95 },
    { 0x61fd83f6, 33 },
    { 0xdb2d83f5, 95 },
    { 0xfced83f8, 95 },
    { 0, 0 },
    { 0xa79d83fa, 55 },
    { 0, 0 },
    { 0x2bad83fc, 17 },
    { 0x101d83fd, 23 },
    { 0xc51d83fc, 75 },
    { 0xee4d83ff, 90 },
    { 0, 0 },
    { 0x13ed8401, 98 },
    { 0x7fed8402, 98 },
    { 0, 0 },
    { 0x1fed8404, 95 },
    { 0, 0 },
    { 0xcdcd8406, 33 },
    { 0xcb2d8407, 98 },
    { 0xf3d8408, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafd840f, 90 },
    { 0x750d8410, 84 },
    { 0x3fcd8411, 98 },
    { 0xfddd8411, 34 },
    { 0x8e9d8410, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9d841a, 93 },
    { 0xd0ed841a, 90 },
    { 0x2dbd841c, 98 },
    { 0xdd9d841c, 33 },
    { 0xafad841e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fd8421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd8425, 98 },
    { 0, 0 },
    { 0xa8fd8427, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36d842a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdd842e, 75 },
    { 0xdd0d842f, 93 },
    { 0x5a0d8430, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5d8437, 35 },
    { 0x75ad8437, 95 },
    { 0xb02d8439, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205d8442, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8d8446, 23 },
    { 0x43bd8447, 90 },
    { 0x5d8d8448, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7d844c, 61 },
    { 0x808d844d, 98 },
    { 0xf3cd844e, 23 },
    { 0, 0 },
    { 0x486d8450, 88 },
    { 0xf8cd8451, 38 },
    { 0, 0 },
    { 0x949d8453, 38 },
    { 0x1c4d8454, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234d8458, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fd845e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62d8462, 93 },
    { 0, 0 },
    { 0xd7cd8464, 90 },
    { 0x6dbd8465, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfded846b, 88 },
    { 0x8dfd846c, 55 },
    { 0xe7cd846d, 91 },
    { 0x780d846e, 33 },
    { 0x803d846f, 34 },
    { 0xebdd8470, 23 },
    { 0xe2d8471, 61 },
    { 0, 0 },
    { 0xb43d8473, 61 },
    { 0x87bd8474, 33 },
    { 0, 0 },
    { 0xbdbd8476, 69 },
    { 0, 0 },
    { 0x329d8478, 95 },
    { 0x5d2d8479, 88 },
    { 0xaf0d847a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723d847e, 46 },
    { 0xeccd847f, 43 },
    { 0x551d847f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75d848e, 78 },
    { 0xd4dd848f, 78 },
    { 0x6e8d848f, 90 },
    { 0x52fd8491, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4d849a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd849d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2d84a0, 98 },
    { 0x88dd84a1, 33 },
    { 0x4d2d84a1, 90 },
    { 0, 0 },
    { 0xabd84a4, 33 },
    { 0, 0 },
    { 0xfbd84a6, 88 },
    { 0, 0 },
    { 0x254d84a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6d84af, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdd84b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ad84be, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd84c7, 61 },
    { 0, 0 },
    { 0xbbfd84c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8d84d1, 61 },
    { 0, 0 },
    { 0x909d84d3, 42 },
    { 0, 0 },
    { 0x38d84d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59d84db, 33 },
    { 0x3e7d84dc, 61 },
    { 0, 0 },
    { 0x4ced84de, 38 },
    { 0, 0 },
    { 0x111d84e0, 61 },
    { 0x8d2d84e1, 33 },
    { 0, 0 },
    { 0x435d84e3, 93 },
    { 0x6aed84e4, 55 },
    { 0xc8cd84e3, 93 },
    { 0, 0 },
    { 0x430d84e7, 61 },
    { 0xbefd84e8, 23 },
    { 0x5e1d84e9, 33 },
    { 0xef0d84e7, 95 },
    { 0xcc1d84e8, 61 },
    { 0x2a2d84ec, 23 },
    { 0x1e3d84ed, 55 },
    { 0xcdad84ee, 23 },
    { 0x947d84ee, 95 },
    { 0x5a8d84e9, 98 },
    { 0x10bd84f1, 91 },
    { 0x39ed84f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48d84f5, 90 },
    { 0x271d84f6, 33 },
    { 0x6acd84f7, 76 },
    { 0x76ed84f8, 33 },
    { 0xb59d84f7, 95 },
    { 0x86fd84fa, 61 },
    { 0x2e4d84f6, 87 },
    { 0x107d84fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x789d84ff, 91 },
    { 0, 0 },
    { 0x12fd8501, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x380d8504, 95 },
    { 0, 0 },
    { 0x277d8506, 95 },
    { 0x8eed8507, 23 },
    { 0xacd8507, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20d850e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7d8515, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8d8518, 17 },
    { 0x972d8519, 61 },
    { 0, 0 },
    { 0x516d851b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d8521, 43 },
    { 0, 0 },
    { 0xb1d8523, 76 },
    { 0, 0 },
    { 0x59bd8525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ad852a, 93 },
    { 0x7ed852a, 55 },
    { 0x622d852c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dd8533, 77 },
    { 0xd9d8534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fad8538, 98 },
    { 0x83dd8539, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd853c, 40 },
    { 0, 0 },
    { 0xd85d853e, 61 },
    { 0xaecd853e, 33 },
    { 0, 0 },
    { 0x739d8541, 61 },
    { 0, 0 },
    { 0xa76d8543, 95 },
    { 0, 0 },
    { 0xb4bd8545, 98 },
    { 0x12d8545, 34 },
    { 0x8a0d8545, 46 },
    { 0xf6d8547, 98 },
    { 0xc30d8549, 24 },
    { 0x363d854a, 46 },
    { 0xfb2d854b, 90 },
    { 0xeecd854a, 90 },
    { 0x40ed854d, 33 },
    { 0x375d8546, 61 },
    { 0x7f4d854f, 90 },
    { 0x27ad8550, 90 },
    { 0x685d8551, 44 },
    { 0xac0d854f, 99 },
    { 0xf90d8548, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ed8559, 93 },
    { 0xf5ed855a, 90 },
    { 0xc6ad8559, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d855e, 61 },
    { 0xb05d855f, 96 },
    { 0, 0 },
    { 0xd99d8561, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ed8564, 61 },
    { 0xd9ad8565, 34 },
    { 0x60bd8566, 91 },
    { 0xf1fd8567, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60d856e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bd8573, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x191d8576, 55 },
    { 0, 0 },
    { 0x488d8578, 76 },
    { 0xe79d8578, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dd857c, 61 },
    { 0x575d857c, 95 },
    { 0x9bdd857e, 33 },
    { 0x2e3d857f, 61 },
    { 0xa8fd8580, 66 },
    { 0, 0 },
    { 0x90dd8582, 75 },
    { 0x5f6d8583, 75 },
    { 0xb8d8584, 33 },
    { 0xcfcd8584, 98 },
    { 0xae0d8585, 90 },
    { 0x71ad8587, 90 },
    { 0xfed8587, 23 },
    { 0x1c3d8588, 84 },
    { 0, 0 },
    { 0xc5fd858b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fd8591, 61 },
    { 0, 0 },
    { 0x753d8593, 52 },
    { 0x226d8594, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64d8599, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3d859e, 61 },
    { 0, 0 },
    { 0x28ad85a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71d85a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0d85ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97d85b0, 99 },
    { 0, 0 },
    { 0xf2dd85b2, 33 },
    { 0, 0 },
    { 0x242d85b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x920d85b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7d85ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fd85c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bad85cb, 52 },
    { 0xb46d85cc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8d85d5, 90 },
    { 0x953d85d6, 33 },
    { 0x391d85d7, 61 },
    { 0x8ccd85d8, 66 },
    { 0x162d85d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd85e1, 77 },
    { 0x5e2d85e2, 34 },
    { 0xa75d85e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d85e6, 67 },
    { 0x245d85e7, 90 },
    { 0, 0 },
    { 0xcf0d85e9, 98 },
    { 0, 0 },
    { 0xb64d85eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfd85ef, 43 },
    { 0xe70d85f0, 84 },
    { 0, 0 },
    { 0x97ad85f2, 98 },
    { 0x9fdd85f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1d85f6, 87 },
    { 0xf4ad85f6, 90 },
    { 0xe13d85f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbd85fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318d8604, 90 },
    { 0xd16d8604, 55 },
    { 0x9b1d8605, 80 },
    { 0xc3fd8607, 33 },
    { 0xef6d8608, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2d860b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad860e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49d8613, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x11d8616, 33 },
    { 0, 0 },
    { 0xaf6d8618, 90 },
    { 0, 0 },
    { 0x7f4d861a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963d8625, 43 },
    { 0xfb1d8626, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d862a, 33 },
    { 0xab2d862b, 95 },
    { 0xb4cd862c, 90 },
    { 0, 0 },
    { 0x746d862e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x488d8631, 17 },
    { 0, 0 },
    { 0xb69d8633, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fd8639, 23 },
    { 0xf89d863a, 81 },
    { 0xb24d8639, 34 },
    { 0x876d863c, 98 },
    { 0x33d863b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacd8640, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ed8643, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfd864e, 96 },
    { 0, 0 },
    { 0x751d8650, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524d865a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d865d, 33 },
    { 0x66d865d, 40 },
    { 0x814d865d, 61 },
    { 0x8e5d865e, 65 },
    { 0xd53d865f, 43 },
    { 0x728d8661, 55 },
    { 0x1d8663, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00d8667, 23 },
    { 0, 0 },
    { 0xf06d8669, 55 },
    { 0x133d866a, 52 },
    { 0xac9d866b, 77 },
    { 0x385d866c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774d8672, 98 },
    { 0, 0 },
    { 0x988d8674, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8d8678, 33 },
    { 0, 0 },
    { 0xeecd867a, 23 },
    { 0x6b2d867a, 61 },
    { 0xf15d867c, 46 },
    { 0x82fd867d, 36 },
    { 0x8a5d867e, 78 },
    { 0x7e0d867c, 92 },
    { 0xfcd8680, 46 },
    { 0x184d867c, 43 },
    { 0x673d8680, 23 },
    { 0, 0 },
    { 0xf32d8684, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3d8688, 55 },
    { 0x49bd8689, 95 },
    { 0x987d868a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbd868f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ad8693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fd8697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550d869e, 90 },
    { 0x18bd869e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7d86a3, 55 },
    { 0, 0 },
    { 0xc57d86a5, 40 },
    { 0xdd86a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0d86a9, 100 },
    { 0, 0 },
    { 0x76ed86ab, 33 },
    { 0, 0 },
    { 0xf3fd86ad, 76 },
    { 0x7fcd86ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3d86b1, 33 },
    { 0xd5fd86b2, 93 },
    { 0x8fad86b3, 55 },
    { 0x212d86b2, 90 },
    { 0, 0 },
    { 0x85d86b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80d86bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cd86cb, 61 },
    { 0x4aed86cc, 76 },
    { 0x9e7d86cb, 80 },
    { 0x122d86cb, 90 },
    { 0xb59d86cb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959d86d9, 55 },
    { 0xd86d86da, 67 },
    { 0xc3bd86d9, 95 },
    { 0xb18d86dc, 98 },
    { 0x5fd86dd, 90 },
    { 0x46cd86dc, 48 },
    { 0, 0 },
    { 0xd33d86e0, 34 },
    { 0x956d86e1, 91 },
    { 0x428d86e2, 46 },
    { 0x196d86e3, 23 },
    { 0x161d86e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x453d86e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcd86ea, 95 },
    { 0xc1cd86ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3d86ee, 33 },
    { 0xee9d86ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fd86f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x631d86f7, 34 },
    { 0, 0 },
    { 0x7d1d86f9, 46 },
    { 0, 0 },
    { 0x3aed86fb, 61 },
    { 0, 0 },
    { 0x427d86fd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fd8702, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bd8705, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46d8710, 33 },
    { 0xe73d8710, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96d8714, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd8719, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dd8720, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5d8723, 23 },
    { 0xda0d8724, 95 },
    { 0xde9d8725, 55 },
    { 0x5c0d8725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x567d8729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46d872c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7d8732, 17 },
    { 0x7d7d8733, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539d8739, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ed873c, 55 },
    { 0, 0 },
    { 0xa2fd873e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x106d8741, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8d8746, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9d874b, 92 },
    { 0xf24d874c, 67 },
    { 0xc9dd874c, 90 },
    { 0xe71d874e, 95 },
    { 0x5f7d874f, 33 },
    { 0, 0 },
    { 0x799d8751, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffd8754, 33 },
    { 0, 0 },
    { 0xed0d8756, 91 },
    { 0xaeed8757, 34 },
    { 0xf54d8757, 46 },
    { 0xd47d8759, 99 },
    { 0, 0 },
    { 0xa21d875b, 90 },
    { 0x2e8d875c, 55 },
    { 0x75cd875d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32d8764, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4d8767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd876b, 90 },
    { 0, 0 },
    { 0x846d876d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446d8778, 90 },
    { 0, 0 },
    { 0xfbfd877a, 95 },
    { 0x4abd877b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1d8781, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdd878a, 23 },
    { 0, 0 },
    { 0x64d878c, 55 },
    { 0xdbbd878d, 61 },
    { 0x169d878e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dd8795, 46 },
    { 0, 0 },
    { 0xa50d8797, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ad879b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x358d879e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcd87a1, 52 },
    { 0xb56d87a1, 61 },
    { 0, 0 },
    { 0x25ad87a4, 46 },
    { 0xab5d87a5, 61 },
    { 0x59d87a4, 33 },
    { 0x842d87a7, 85 },
    { 0xfbbd87a8, 56 },
    { 0xc0bd87a6, 55 },
    { 0x765d87aa, 77 },
    { 0x410d87ab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77d87b0, 25 },
    { 0xd23d87b1, 33 },
    { 0x847d87b2, 51 },
    { 0x5d9d87b3, 90 },
    { 0, 0 },
    { 0xe49d87b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34d87b8, 99 },
    { 0x8bdd87b9, 90 },
    { 0xdf7d87ba, 33 },
    { 0xa11d87bb, 76 },
    { 0x6e3d87bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ad87bf, 78 },
    { 0xf3fd87bf, 95 },
    { 0x114d87c0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04d87c6, 33 },
    { 0x237d87c7, 99 },
    { 0x556d87c8, 95 },
    { 0x278d87c6, 98 },
    { 0xe6ed87ca, 90 },
    { 0x729d87c9, 76 },
    { 0x38fd87cc, 61 },
    { 0x885d87cb, 80 },
    { 0x9d0d87ce, 95 },
    { 0x435d87cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd87d3, 43 },
    { 0xfcd87d4, 85 },
    { 0x6fed87d5, 99 },
    { 0xefbd87d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccd87d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4d87dc, 95 },
    { 0xa58d87dd, 61 },
    { 0x1d6d87de, 34 },
    { 0x334d87de, 95 },
    { 0, 0 },
    { 0xfa0d87e1, 95 },
    { 0x3fed87e2, 85 },
    { 0xf2bd87e2, 98 },
    { 0, 0 },
    { 0x68d87e5, 75 },
    { 0, 0 },
    { 0x8add87e7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d87ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8d87ed, 17 },
    { 0xbbcd87ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9d87f4, 33 },
    { 0, 0 },
    { 0x538d87f6, 90 },
    { 0xdb3d87f7, 76 },
    { 0x70d87f8, 61 },
    { 0xb32d87f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecd87fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91d8803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x221d8806, 33 },
    { 0, 0 },
    { 0x453d8808, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0d880b, 23 },
    { 0, 0 },
    { 0x291d880d, 85 },
    { 0, 0 },
    { 0x3a2d880f, 98 },
    { 0xacdd8810, 55 },
    { 0x17bd8811, 61 },
    { 0x430d8812, 55 },
    { 0, 0 },
    { 0x4f4d8814, 43 },
    { 0, 0 },
    { 0x5dad8816, 87 },
    { 0, 0 },
    { 0x73d8818, 90 },
    { 0xe00d8818, 75 },
    { 0, 0 },
    { 0xfad881b, 33 },
    { 0xf9ad881c, 93 },
    { 0x223d881c, 95 },
    { 0xc46d881e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0d8821, 81 },
    { 0xccd8822, 95 },
    { 0x128d8823, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d8828, 67 },
    { 0x81ed8829, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8d882e, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123d8832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbd8842, 98 },
    { 0, 0 },
    { 0xf60d8844, 33 },
    { 0xaf7d8845, 33 },
    { 0xc7ed8846, 95 },
    { 0xdddd8844, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94d884a, 52 },
    { 0, 0 },
    { 0x912d884c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d884f, 66 },
    { 0x532d8850, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237d8854, 94 },
    { 0x2b2d8855, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x255d8858, 33 },
    { 0x97d8859, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27d8860, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0d8863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8d8868, 80 },
    { 0x661d8869, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4d886c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203d8872, 90 },
    { 0x860d8873, 61 },
    { 0, 0 },
    { 0xb9ad8875, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fd8878, 91 },
    { 0, 0 },
    { 0x586d887a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfd887e, 46 },
    { 0x7aad887f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaced8883, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207d888c, 98 },
    { 0xfdbd888d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x595d8894, 90 },
    { 0, 0 },
    { 0x954d8896, 90 },
    { 0x493d8896, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fad889d, 90 },
    { 0x29bd889d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbd88a5, 23 },
    { 0xf10d88a6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8d88ac, 61 },
    { 0xbedd88ac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61d88b0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84d88b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93d88bb, 78 },
    { 0x176d88bc, 61 },
    { 0xa36d88bd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedd88c0, 87 },
    { 0x191d88c0, 61 },
    { 0x5cbd88c2, 43 },
    { 0xaefd88c3, 90 },
    { 0x9ecd88c4, 34 },
    { 0x910d88c0, 90 },
    { 0xd52d88c6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x844d88c9, 48 },
    { 0x296d88ca, 90 },
    { 0xd41d88cb, 33 },
    { 0x727d88cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468d88d3, 90 },
    { 0x89bd88d3, 33 },
    { 0x4d2d88d3, 61 },
    { 0x18fd88d6, 76 },
    { 0x709d88d6, 34 },
    { 0xead88d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799d88dd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479d88e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffd88e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8d88f4, 98 },
    { 0x5cfd88f4, 55 },
    { 0, 0 },
    { 0x5d3d88f7, 90 },
    { 0xfa8d88f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcd88ff, 61 },
    { 0xcfbd88ff, 76 },
    { 0x583d8901, 85 },
    { 0x24d8902, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cd890a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5dd890d, 100 },
    { 0x7dbd890e, 88 },
    { 0, 0 },
    { 0xbdbd8910, 66 },
    { 0x107d8910, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87d8915, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51d8921, 61 },
    { 0, 0 },
    { 0x383d8923, 23 },
    { 0, 0 },
    { 0xf7ad8925, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd892a, 23 },
    { 0x8a8d892b, 46 },
    { 0, 0 },
    { 0x1d9d892d, 33 },
    { 0x7f2d892e, 99 },
    { 0xf9cd892e, 34 },
    { 0, 0 },
    { 0x479d8931, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41d893c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98d8942, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcd8949, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2d894f, 95 },
    { 0xf88d8950, 19 },
    { 0x4d6d8951, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9d8954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395d8958, 83 },
    { 0xbbad8959, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448d895d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d8961, 90 },
    { 0x347d8962, 33 },
    { 0, 0 },
    { 0xd68d8964, 93 },
    { 0xb38d8965, 77 },
    { 0x29ed8966, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03d8969, 87 },
    { 0x6bed8969, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417d8972, 90 },
    { 0x59bd8973, 44 },
    { 0xdc7d8974, 75 },
    { 0xa57d8975, 52 },
    { 0x24cd8974, 91 },
    { 0xa0ad8973, 23 },
    { 0x5e8d8974, 90 },
    { 0x371d8976, 33 },
    { 0, 0 },
    { 0xd89d897b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87d8980, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3d8987, 55 },
    { 0xd17d8988, 95 },
    { 0, 0 },
    { 0xc65d898a, 33 },
    { 0x875d898b, 55 },
    { 0x7edd898c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d8993, 93 },
    { 0x953d8993, 84 },
    { 0xa5bd8995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbd8999, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddd89a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7d89a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d89aa, 43 },
    { 0x91d89ab, 61 },
    { 0xa21d89ac, 98 },
    { 0, 0 },
    { 0x80cd89ae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1d89b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed89b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03d89ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d89c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fd89c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ed89ca, 90 },
    { 0xfdcd89ca, 76 },
    { 0xb7dd89cb, 75 },
    { 0x7d8d89cd, 33 },
    { 0xd3ed89cd, 33 },
    { 0xc59d89cf, 33 },
    { 0, 0 },
    { 0x2fad89d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680d89d7, 33 },
    { 0x42cd89d8, 34 },
    { 0x8d89d9, 90 },
    { 0x20ad89da, 77 },
    { 0x327d89db, 86 },
    { 0xa3d89dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1d89e0, 95 },
    { 0xdbed89e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fd89e9, 33 },
    { 0x7a1d89e9, 81 },
    { 0x25fd89e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03d89ef, 77 },
    { 0x495d89f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dd89f5, 61 },
    { 0, 0 },
    { 0x45ad89f7, 91 },
    { 0x808d89f8, 90 },
    { 0x7fd89f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9d89ff, 93 },
    { 0, 0 },
    { 0x72ed8a01, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bd8a04, 43 },
    { 0x799d8a04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bd8a0a, 95 },
    { 0, 0 },
    { 0x81ed8a0c, 46 },
    { 0, 0 },
    { 0x2d8d8a0e, 95 },
    { 0xd30d8a0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302d8a17, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2d8a1b, 90 },
    { 0x529d8a1c, 33 },
    { 0x6c4d8a1d, 43 },
    { 0xb05d8a1e, 76 },
    { 0x529d8a1f, 85 },
    { 0xf6cd8a1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5d8a26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72d8a2b, 90 },
    { 0, 0 },
    { 0x550d8a2d, 55 },
    { 0xcd8a2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0d8a31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754d8a35, 88 },
    { 0x19ed8a35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1d8a39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dd8a3e, 91 },
    { 0x3b7d8a3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29d8a42, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13d8a46, 92 },
    { 0x4c5d8a46, 90 },
    { 0xc81d8a46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dd8a4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64d8a57, 90 },
    { 0x267d8a58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3d8a5e, 23 },
    { 0, 0 },
    { 0x37ad8a60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80d8a65, 43 },
    { 0x21d8a66, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ad8a6c, 34 },
    { 0xd47d8a6c, 90 },
    { 0xbdcd8a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0d8a71, 98 },
    { 0x128d8a72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00d8a79, 95 },
    { 0x822d8a79, 46 },
    { 0xa7d8a7b, 46 },
    { 0xac5d8a7c, 76 },
    { 0xbf6d8a7d, 81 },
    { 0x28bd8a7e, 95 },
    { 0, 0 },
    { 0x818d8a80, 95 },
    { 0x4ebd8a81, 66 },
    { 0xcc1d8a82, 87 },
    { 0x6ccd8a82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd8a87, 46 },
    { 0xca5d8a88, 33 },
    { 0xb99d8a88, 76 },
    { 0x521d8a89, 76 },
    { 0xedad8a8b, 34 },
    { 0x903d8a8b, 61 },
    { 0x9cdd8a8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bd8a97, 2 },
    { 0x7f2d8a97, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bd8a9d, 98 },
    { 0xa0fd8a9e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8d8aa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ed8aac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6d8ab4, 33 },
    { 0x36ad8ab5, 91 },
    { 0x33cd8ab6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ead8abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ad8ac1, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59d8ac4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2d8ac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3add8ad1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d8ad5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cd8ad8, 23 },
    { 0, 0 },
    { 0xfa8d8ada, 90 },
    { 0, 0 },
    { 0xa88d8adc, 98 },
    { 0, 0 },
    { 0x500d8ade, 55 },
    { 0x15ed8adf, 43 },
    { 0xc5cd8adf, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86d8ae6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ced8ae9, 98 },
    { 0, 0 },
    { 0xbd2d8aeb, 95 },
    { 0x3b0d8aec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdd8aef, 98 },
    { 0, 0 },
    { 0x6bed8af1, 45 },
    { 0xc3ed8af2, 76 },
    { 0xd61d8af3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1d8af7, 85 },
    { 0xb16d8af7, 100 },
    { 0xdbfd8af9, 33 },
    { 0xf6fd8afa, 34 },
    { 0xac0d8afa, 34 },
    { 0x657d8afa, 61 },
    { 0x5b5d8afc, 90 },
    { 0xdc3d8afa, 88 },
    { 0, 0 },
    { 0xdded8b00, 84 },
    { 0xc10d8b00, 34 },
    { 0, 0 },
    { 0xd27d8b03, 95 },
    { 0, 0 },
    { 0xc70d8b05, 91 },
    { 0xd8ad8b06, 95 },
    { 0, 0 },
    { 0xfc1d8b08, 61 },
    { 0, 0 },
    { 0xdbd8b0a, 90 },
    { 0xe5fd8b0b, 45 },
    { 0, 0 },
    { 0x31d8b0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x593d8b10, 93 },
    { 0xb71d8b11, 52 },
    { 0xa69d8b12, 98 },
    { 0xa06d8b13, 61 },
    { 0x34ed8b13, 78 },
    { 0x53fd8b15, 23 },
    { 0xde0d8b13, 84 },
    { 0x267d8b15, 98 },
    { 0xfbd8b18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32d8b1b, 90 },
    { 0x620d8b1b, 90 },
    { 0, 0 },
    { 0x2b3d8b1e, 61 },
    { 0x817d8b1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92d8b23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9d8b28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dd8b2d, 34 },
    { 0xaeed8b2e, 43 },
    { 0, 0 },
    { 0xd41d8b30, 46 },
    { 0xbe9d8b30, 93 },
    { 0x982d8b30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16d8b35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efd8b3a, 98 },
    { 0x5dfd8b3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dd8b3f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9d8b43, 78 },
    { 0x13dd8b44, 66 },
    { 0xa8bd8b44, 33 },
    { 0, 0 },
    { 0x75ad8b47, 81 },
    { 0xe30d8b48, 98 },
    { 0x7eed8b49, 61 },
    { 0x49ed8b4a, 34 },
    { 0xb7fd8b4b, 76 },
    { 0, 0 },
    { 0xf92d8b4d, 23 },
    { 0x66dd8b4e, 93 },
    { 0x3ced8b4f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73d8b5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x156d8b5e, 33 },
    { 0x6bfd8b5f, 98 },
    { 0xafcd8b60, 8 },
    { 0x634d8b61, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x210d8b64, 68 },
    { 0, 0 },
    { 0x6efd8b66, 33 },
    { 0x631d8b66, 46 },
    { 0xc06d8b68, 90 },
    { 0x29fd8b68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30d8b6d, 90 },
    { 0, 0 },
    { 0x6fbd8b6f, 85 },
    { 0x34fd8b70, 84 },
    { 0x808d8b71, 33 },
    { 0x579d8b71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991d8b77, 33 },
    { 0x65dd8b77, 34 },
    { 0, 0 },
    { 0x998d8b7a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d8b7d, 95 },
    { 0xb9ad8b7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadd8b86, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfd8b89, 33 },
    { 0, 0 },
    { 0x71dd8b8b, 61 },
    { 0, 0 },
    { 0xe40d8b8d, 34 },
    { 0, 0 },
    { 0x72fd8b8f, 77 },
    { 0xda5d8b8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6d8b99, 93 },
    { 0xe00d8b99, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ad8ba0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3d8ba7, 93 },
    { 0x994d8ba7, 90 },
    { 0x665d8ba7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ed8bad, 34 },
    { 0x1b6d8bae, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bd8bb1, 76 },
    { 0x271d8bb2, 66 },
    { 0xd89d8bb2, 61 },
    { 0x799d8bb3, 33 },
    { 0xcadd8bb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9d8bba, 98 },
    { 0x5d9d8bba, 90 },
    { 0x5d8d8bbc, 33 },
    { 0x177d8bbb, 33 },
    { 0, 0 },
    { 0x883d8bbf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151d8bc5, 55 },
    { 0, 0 },
    { 0xd16d8bc7, 90 },
    { 0x37ed8bc7, 55 },
    { 0x1d4d8bc9, 23 },
    { 0x894d8bc9, 87 },
    { 0xcb8d8bcb, 90 },
    { 0x8c7d8bcb, 91 },
    { 0xeced8bc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ad8bd0, 34 },
    { 0x5fcd8bd0, 93 },
    { 0, 0 },
    { 0x4c6d8bd3, 23 },
    { 0, 0 },
    { 0x8dbd8bd5, 90 },
    { 0xf8fd8bd6, 87 },
    { 0x3c0d8bd5, 93 },
    { 0x478d8bd8, 93 },
    { 0x83ad8bd9, 98 },
    { 0x108d8bd9, 95 },
    { 0xb66d8bdb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282d8be2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22d8be8, 95 },
    { 0x360d8be9, 61 },
    { 0, 0 },
    { 0x12ed8beb, 98 },
    { 0xcc4d8beb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x921d8bef, 61 },
    { 0x519d8bf0, 98 },
    { 0xad4d8bf1, 87 },
    { 0, 0 },
    { 0x14ad8bf3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cd8bf6, 33 },
    { 0x8cdd8bf7, 61 },
    { 0xbbdd8bf8, 81 },
    { 0, 0 },
    { 0x49ad8bfa, 80 },
    { 0xb7dd8bfb, 76 },
    { 0x188d8bfc, 100 },
    { 0, 0 },
    { 0x1ed8bfe, 95 },
    { 0x2dbd8bff, 95 },
    { 0x15ed8bfe, 95 },
    { 0, 0 },
    { 0xc11d8c02, 90 },
    { 0x89d8c03, 23 },
    { 0xf88d8c03, 90 },
    { 0xe76d8c05, 43 },
    { 0xb0d8c06, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91d8c09, 34 },
    { 0xd86d8c0a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986d8c11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8d8c15, 23 },
    { 0x322d8c15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719d8c1c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3d8c25, 95 },
    { 0x215d8c25, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x236d8c2c, 90 },
    { 0xb7dd8c2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfd8c30, 95 },
    { 0x7f3d8c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813d8c35, 76 },
    { 0x150d8c36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1d8c39, 23 },
    { 0x29ed8c3a, 95 },
    { 0x2b5d8c3a, 61 },
    { 0, 0 },
    { 0x6e4d8c3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3d8c42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bd8c49, 99 },
    { 0, 0 },
    { 0x8d6d8c4b, 34 },
    { 0x405d8c4c, 34 },
    { 0x64d8c4c, 90 },
    { 0x91fd8c4b, 98 },
    { 0xc28d8c4f, 91 },
    { 0x26ad8c4c, 98 },
    { 0xefbd8c51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcd8c55, 23 },
    { 0, 0 },
    { 0xdfcd8c57, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd8c5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41d8c62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ed8c66, 33 },
    { 0x2ccd8c67, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2d8c6e, 33 },
    { 0xc4fd8c6e, 84 },
    { 0x2cad8c6f, 61 },
    { 0, 0 },
    { 0x3b2d8c72, 61 },
    { 0, 0 },
    { 0x6fd8c74, 84 },
    { 0, 0 },
    { 0x36cd8c76, 91 },
    { 0, 0 },
    { 0x2d8d8c78, 95 },
    { 0x97fd8c78, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0d8c7c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bd8c86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768d8c8b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdd8c90, 93 },
    { 0x376d8c90, 95 },
    { 0x45fd8c90, 84 },
    { 0x78d8c93, 98 },
    { 0, 0 },
    { 0x3d8d8c95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bd8c9a, 95 },
    { 0x20bd8c9a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x387d8c9e, 61 },
    { 0xbc6d8c9f, 33 },
    { 0xa9dd8c9e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393d8ca6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383d8cab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5d8cb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ad8cb4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8d8cb8, 61 },
    { 0, 0 },
    { 0xb67d8cba, 23 },
    { 0xa47d8cbb, 90 },
    { 0x46ad8cbb, 55 },
    { 0xeb9d8cbb, 81 },
    { 0x239d8cba, 95 },
    { 0, 0 },
    { 0x5d4d8cc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ed8cc5, 88 },
    { 0x78d8cc5, 52 },
    { 0x807d8cc6, 95 },
    { 0xbdcd8cc8, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcd8ccc, 95 },
    { 0, 0 },
    { 0x1ad8cce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82d8cd5, 95 },
    { 0, 0 },
    { 0xd6d8cd7, 23 },
    { 0x7f2d8cd7, 75 },
    { 0x32d8cd9, 67 },
    { 0xf78d8cd7, 90 },
    { 0, 0 },
    { 0xafcd8cdc, 23 },
    { 0xc11d8cdd, 23 },
    { 0xbcdd8cde, 90 },
    { 0, 0 },
    { 0x7d2d8ce0, 40 },
    { 0xa9dd8ce1, 98 },
    { 0x3a4d8ce1, 52 },
    { 0x2b0d8ce3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79d8ce7, 93 },
    { 0, 0 },
    { 0x20cd8ce9, 68 },
    { 0xc47d8ce9, 43 },
    { 0xca7d8ceb, 61 },
    { 0xcd8d8cea, 98 },
    { 0xbc5d8cec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8d8cf0, 94 },
    { 0, 0 },
    { 0x430d8cf2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd8cf6, 34 },
    { 0x672d8cf7, 23 },
    { 0x8fbd8cf8, 33 },
    { 0, 0 },
    { 0x532d8cfa, 98 },
    { 0xcc0d8cfb, 98 },
    { 0x540d8cfb, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bd8d03, 95 },
    { 0x88ed8d04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadd8d08, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ad8d0b, 90 },
    { 0, 0 },
    { 0xe62d8d0d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cd8d16, 99 },
    { 0xabcd8d16, 87 },
    { 0xdeed8d18, 100 },
    { 0x9eed8d19, 90 },
    { 0, 0 },
    { 0xbdcd8d1b, 67 },
    { 0xfa8d8d1c, 61 },
    { 0x825d8d1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561d8d22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fd8d27, 90 },
    { 0xabcd8d28, 23 },
    { 0xad8d8d29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7d8d2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd8d34, 84 },
    { 0, 0 },
    { 0xab4d8d36, 64 },
    { 0x331d8d37, 95 },
    { 0xe58d8d38, 33 },
    { 0x57dd8d37, 33 },
    { 0, 0 },
    { 0x1a6d8d3b, 98 },
    { 0x343d8d3b, 34 },
    { 0x3a8d8d3d, 61 },
    { 0x61bd8d3e, 67 },
    { 0x1a7d8d3e, 98 },
    { 0, 0 },
    { 0x896d8d41, 67 },
    { 0xd3cd8d41, 33 },
    { 0, 0 },
    { 0x361d8d44, 85 },
    { 0xadad8d45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910d8d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44d8d4f, 98 },
    { 0xccad8d4f, 87 },
    { 0, 0 },
    { 0xff4d8d52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d8d56, 98 },
    { 0, 0 },
    { 0x2b8d8d58, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7d8d5c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ad8d64, 33 },
    { 0x9b6d8d65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cd8d68, 61 },
    { 0x1c4d8d69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5d8d71, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aed8d74, 46 },
    { 0xd40d8d75, 23 },
    { 0xfdfd8d75, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebd8d7c, 95 },
    { 0xcd1d8d7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdad8d80, 23 },
    { 0x2cfd8d81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313d8d85, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71d8d88, 98 },
    { 0x232d8d89, 95 },
    { 0xac1d8d8a, 43 },
    { 0, 0 },
    { 0x8e6d8d8c, 23 },
    { 0, 0 },
    { 0xbf5d8d8e, 34 },
    { 0x4a9d8d8f, 46 },
    { 0x95fd8d90, 90 },
    { 0xc2ad8d91, 69 },
    { 0xba3d8d92, 95 },
    { 0, 0 },
    { 0x160d8d94, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2d8d97, 23 },
    { 0xa9bd8d98, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63d8d9c, 76 },
    { 0xa0ad8d9c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228d8da6, 98 },
    { 0, 0 },
    { 0x145d8da8, 23 },
    { 0x986d8da9, 33 },
    { 0x32fd8daa, 34 },
    { 0xc17d8dab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3d8daf, 34 },
    { 0x306d8db0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x185d8db3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1d8db8, 56 },
    { 0x17bd8db9, 81 },
    { 0x467d8dba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5d8dcb, 90 },
    { 0xdc2d8dcc, 61 },
    { 0x962d8dcb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3d8dd2, 90 },
    { 0, 0 },
    { 0x7c8d8dd4, 40 },
    { 0xded8dd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2d8dd8, 61 },
    { 0xd3fd8dd9, 87 },
    { 0x5f3d8dd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ded8ddd, 90 },
    { 0xf4d8dde, 33 },
    { 0x7b7d8ddf, 76 },
    { 0, 0 },
    { 0x85fd8de1, 65 },
    { 0x749d8de2, 93 },
    { 0, 0 },
    { 0xd1ed8de4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd8de7, 46 },
    { 0x6dfd8de7, 33 },
    { 0xcbbd8de9, 95 },
    { 0, 0 },
    { 0xb0dd8deb, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffd8df0, 60 },
    { 0, 0 },
    { 0x29bd8df2, 61 },
    { 0x6aad8df2, 95 },
    { 0xf22d8df4, 95 },
    { 0x3f8d8df2, 101 },
    { 0x65ed8df6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fd8dfa, 90 },
    { 0, 0 },
    { 0x46d8dfc, 33 },
    { 0x6aed8dfc, 98 },
    { 0, 0 },
    { 0x2bad8dff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cd8e02, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9d8e08, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6d8e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cd8e0e, 33 },
    { 0x144d8e0f, 43 },
    { 0xfa5d8e10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aed8e16, 92 },
    { 0x73fd8e16, 43 },
    { 0x8cfd8e18, 95 },
    { 0, 0 },
    { 0x321d8e1a, 99 },
    { 0xdd8e1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98d8e1e, 69 },
    { 0x528d8e1e, 46 },
    { 0x552d8e1f, 76 },
    { 0xf95d8e21, 99 },
    { 0x654d8e22, 98 },
    { 0xd7ad8e22, 95 },
    { 0, 0 },
    { 0x79fd8e25, 33 },
    { 0xc90d8e26, 43 },
    { 0, 0 },
    { 0x150d8e28, 90 },
    { 0xfbad8e29, 33 },
    { 0xa95d8e28, 90 },
    { 0xe1d8e2a, 91 },
    { 0x58ad8e28, 80 },
    { 0xcb7d8e2d, 61 },
    { 0x29ad8e29, 98 },
    { 0x881d8e2f, 76 },
    { 0x1fbd8e30, 55 },
    { 0xe5dd8e2b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bd8e41, 59 },
    { 0, 0 },
    { 0xb4ad8e43, 91 },
    { 0xb6ed8e44, 61 },
    { 0x46fd8e45, 98 },
    { 0, 0 },
    { 0xe1bd8e47, 95 },
    { 0xdcfd8e48, 33 },
    { 0, 0 },
    { 0xde9d8e4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d8e4f, 78 },
    { 0x453d8e50, 33 },
    { 0, 0 },
    { 0x38cd8e52, 43 },
    { 0x1acd8e53, 100 },
    { 0xb4dd8e53, 90 },
    { 0x460d8e55, 98 },
    { 0x313d8e55, 61 },
    { 0x467d8e57, 76 },
    { 0xb84d8e57, 66 },
    { 0, 0 },
    { 0x727d8e5a, 87 },
    { 0xb37d8e5a, 87 },
    { 0xcf7d8e5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aad8e5f, 98 },
    { 0x48fd8e5f, 77 },
    { 0x93bd8e60, 23 },
    { 0, 0 },
    { 0x80fd8e63, 61 },
    { 0x2ddd8e64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26d8e68, 91 },
    { 0xc5cd8e69, 61 },
    { 0xd20d8e69, 90 },
    { 0x88cd8e6b, 52 },
    { 0x90dd8e69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4d8e6f, 95 },
    { 0xe36d8e6f, 90 },
    { 0x3d4d8e71, 88 },
    { 0x6bdd8e72, 84 },
    { 0xad5d8e71, 98 },
    { 0x33bd8e71, 33 },
    { 0x710d8e70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d8e7c, 97 },
    { 0x158d8e7d, 61 },
    { 0x47d8e7e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2d8e81, 61 },
    { 0xb36d8e82, 61 },
    { 0x2b8d8e83, 95 },
    { 0, 0 },
    { 0x41dd8e85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ad8e89, 55 },
    { 0, 0 },
    { 0xd23d8e8b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2d8e91, 33 },
    { 0x12dd8e91, 95 },
    { 0xeabd8e93, 91 },
    { 0xa20d8e93, 61 },
    { 0xcc6d8e95, 33 },
    { 0xa1fd8e95, 33 },
    { 0, 0 },
    { 0x375d8e98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cd8e9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fd8ea0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfd8ea3, 100 },
    { 0x4b3d8ea3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dd8eab, 68 },
    { 0x1aed8eab, 23 },
    { 0xf1d8eab, 98 },
    { 0, 0 },
    { 0xc42d8eaf, 97 },
    { 0, 0 },
    { 0x79fd8eb1, 40 },
    { 0, 0 },
    { 0x8afd8eb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67d8eb6, 33 },
    { 0xd3dd8eb6, 61 },
    { 0x71ed8eb8, 61 },
    { 0, 0 },
    { 0xfd3d8eba, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2d8ebd, 98 },
    { 0x461d8ebd, 43 },
    { 0x91d8ebf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41d8ec2, 93 },
    { 0, 0 },
    { 0xa6ed8ec4, 61 },
    { 0xd1bd8ec5, 77 },
    { 0x76fd8ec4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbd8eca, 61 },
    { 0x7e6d8ecb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x492d8ece, 90 },
    { 0x9b5d8ecf, 98 },
    { 0xaecd8ed0, 78 },
    { 0x8e2d8ed0, 70 },
    { 0x354d8ed0, 33 },
    { 0x33fd8ed3, 84 },
    { 0xb5bd8ece, 88 },
    { 0x203d8ed5, 77 },
    { 0x7add8ed6, 61 },
    { 0x580d8ed7, 76 },
    { 0xcb1d8ed8, 91 },
    { 0xafed8ed8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbd8edd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1d8ee0, 52 },
    { 0x8bad8ee1, 90 },
    { 0xf63d8ee1, 33 },
    { 0, 0 },
    { 0xcecd8ee4, 95 },
    { 0x118d8ee5, 61 },
    { 0x4cdd8ee6, 90 },
    { 0xe87d8ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347d8eed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cd8ef0, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x36dd8ef3, 95 },
    { 0x47ad8ef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708d8efb, 52 },
    { 0, 0 },
    { 0xf36d8efd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dd8f01, 33 },
    { 0, 0 },
    { 0x552d8f03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ed8f0a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x371d8f0d, 76 },
    { 0x759d8f0e, 76 },
    { 0xc46d8f0f, 90 },
    { 0, 0 },
    { 0x928d8f11, 81 },
    { 0, 0 },
    { 0x872d8f13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939d8f18, 90 },
    { 0x59ed8f19, 95 },
    { 0xc12d8f18, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88d8f1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccd8f26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x966d8f2a, 84 },
    { 0xb2bd8f2b, 88 },
    { 0xe49d8f2a, 46 },
    { 0xe51d8f2c, 36 },
    { 0xa1ed8f2e, 52 },
    { 0xb79d8f2f, 56 },
    { 0x81cd8f2e, 98 },
    { 0xf30d8f2f, 100 },
    { 0, 0 },
    { 0x2c1d8f33, 85 },
    { 0xacad8f34, 34 },
    { 0xd49d8f35, 84 },
    { 0x5fbd8f35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ad8f3a, 98 },
    { 0x3afd8f3b, 34 },
    { 0xd12d8f3c, 61 },
    { 0, 0 },
    { 0x4c9d8f3e, 98 },
    { 0, 0 },
    { 0x166d8f40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6d8f44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cd8f49, 61 },
    { 0, 0 },
    { 0x20fd8f4b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1d8f4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cd8f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bd8f55, 98 },
    { 0xd9bd8f56, 98 },
    { 0xe18d8f57, 91 },
    { 0, 0 },
    { 0x488d8f59, 52 },
    { 0xa60d8f59, 43 },
    { 0xe0ad8f5b, 33 },
    { 0, 0 },
    { 0x930d8f5d, 95 },
    { 0x5a5d8f5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaead8f65, 87 },
    { 0x9fcd8f66, 61 },
    { 0, 0 },
    { 0x1abd8f68, 23 },
    { 0, 0 },
    { 0xff1d8f6a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d8f6d, 25 },
    { 0, 0 },
    { 0x76dd8f6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15d8f72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0d8f77, 95 },
    { 0, 0 },
    { 0xda4d8f79, 90 },
    { 0x8cad8f79, 90 },
    { 0x3e0d8f7b, 90 },
    { 0x353d8f7c, 33 },
    { 0x23ad8f7a, 61 },
    { 0, 0 },
    { 0xc8ad8f7f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dd8f8e, 61 },
    { 0, 0 },
    { 0x801d8f90, 90 },
    { 0x538d8f91, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aed8f97, 95 },
    { 0x62fd8f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796d8f9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4d8f9f, 33 },
    { 0xb1ed8fa0, 47 },
    { 0xcad8fa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecd8fa6, 33 },
    { 0xbfad8fa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68d8fab, 43 },
    { 0, 0 },
    { 0x170d8fad, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5d8fb0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ed8fb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8d8fb7, 76 },
    { 0x87bd8fb7, 61 },
    { 0, 0 },
    { 0x411d8fba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354d8fc0, 23 },
    { 0, 0 },
    { 0x7efd8fc2, 55 },
    { 0x368d8fc3, 44 },
    { 0x255d8fc3, 93 },
    { 0x60dd8fc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcd8fcf, 66 },
    { 0x4f7d8fd0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999d8fdb, 68 },
    { 0, 0 },
    { 0x1f6d8fdd, 34 },
    { 0x58dd8fdd, 23 },
    { 0x16fd8fdf, 56 },
    { 0xe51d8fe0, 23 },
    { 0x58ad8fdf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7d8fe4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x925d8fe8, 90 },
    { 0xf61d8fe9, 61 },
    { 0, 0 },
    { 0xe2dd8feb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67dd8ffa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x837d8ffd, 90 },
    { 0, 0 },
    { 0x589d8fff, 33 },
    { 0, 0 },
    { 0x49d9001, 95 },
    { 0x4a5d9001, 75 },
    { 0x2ead9002, 95 },
    { 0xf5fd9001, 61 },
    { 0x6c5d9005, 91 },
    { 0x203d9006, 91 },
    { 0, 0 },
    { 0xd76d9008, 90 },
    { 0, 0 },
    { 0x9fd900a, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ed900e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1d9016, 52 },
    { 0x8ead9016, 61 },
    { 0x939d9018, 23 },
    { 0, 0 },
    { 0x652d901a, 93 },
    { 0x771d901b, 34 },
    { 0xb7ad901c, 33 },
    { 0x86d901c, 23 },
    { 0x42dd901d, 55 },
    { 0xc42d901b, 90 },
    { 0xdead901c, 53 },
    { 0, 0 },
    { 0x534d9022, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd9026, 33 },
    { 0x63ad9027, 23 },
    { 0x837d9028, 34 },
    { 0xa4ed9027, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0d902e, 53 },
    { 0, 0 },
    { 0x82ad9030, 66 },
    { 0x5ead9031, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25d9036, 91 },
    { 0x22fd9037, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d9041, 99 },
    { 0x4d4d9042, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ed9045, 98 },
    { 0x629d9046, 61 },
    { 0, 0 },
    { 0xe26d9048, 95 },
    { 0, 0 },
    { 0xae3d904a, 100 },
    { 0xeb7d904b, 68 },
    { 0, 0 },
    { 0xc1fd904d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fd9051, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1d9058, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xafd905b, 34 },
    { 0x213d905c, 61 },
    { 0, 0 },
    { 0x1fdd905e, 55 },
    { 0x4dfd905f, 99 },
    { 0xa68d9060, 43 },
    { 0x639d9060, 78 },
    { 0, 0 },
    { 0xd79d9063, 61 },
    { 0x893d9063, 61 },
    { 0, 0 },
    { 0x442d9066, 87 },
    { 0, 0 },
    { 0x80cd9068, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x349d906b, 95 },
    { 0xbad906b, 98 },
    { 0, 0 },
    { 0x5b0d906e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2d9074, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ed9078, 98 },
    { 0x99fd9079, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8d907c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfd9080, 46 },
    { 0x9a6d9081, 78 },
    { 0x427d9082, 90 },
    { 0, 0 },
    { 0x2d6d9084, 90 },
    { 0xb6fd9084, 76 },
    { 0xb84d9084, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786d908a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3d908e, 98 },
    { 0xbc5d908e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcd9094, 23 },
    { 0xd05d9095, 33 },
    { 0xd9d9095, 77 },
    { 0xc0dd9095, 33 },
    { 0x733d9095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cad909e, 33 },
    { 0xe38d909f, 95 },
    { 0xd5ed90a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ad90a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84d90a8, 33 },
    { 0xbe0d90a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84d90b2, 90 },
    { 0, 0 },
    { 0xb84d90b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841d90bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687d90c3, 23 },
    { 0x118d90c4, 64 },
    { 0xbb9d90c4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22d90c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52fd90ce, 91 },
    { 0, 0 },
    { 0xc67d90d0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07d90d5, 98 },
    { 0, 0 },
    { 0xb9ed90d7, 95 },
    { 0x17cd90d8, 96 },
    { 0, 0 },
    { 0xc9fd90da, 52 },
    { 0, 0 },
    { 0x190d90dc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66d90e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bd90e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1d90ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x257d90f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fd90f4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabd9100, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x16d9103, 100 },
    { 0xb6ed9104, 69 },
    { 0xaa7d9105, 95 },
    { 0, 0 },
    { 0x3a6d9107, 93 },
    { 0xc6d9108, 90 },
    { 0x907d9109, 33 },
    { 0x2a0d910a, 90 },
    { 0x9add910b, 33 },
    { 0xdbcd9109, 33 },
    { 0, 0 },
    { 0x577d910e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fd9111, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9d9117, 61 },
    { 0xdced9117, 96 },
    { 0xf3fd9119, 40 },
    { 0x523d911a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd911f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2d9123, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7d9126, 98 },
    { 0xbf3d9127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0d912c, 33 },
    { 0, 0 },
    { 0x51fd912e, 93 },
    { 0x8afd912f, 97 },
    { 0x4d0d9130, 90 },
    { 0xfd9d9130, 90 },
    { 0x3c6d912e, 90 },
    { 0xb11d9130, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fd9137, 46 },
    { 0x650d9137, 90 },
    { 0x26dd9137, 69 },
    { 0, 0 },
    { 0xe05d913b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ed913f, 33 },
    { 0xbedd913f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x936d9143, 76 },
    { 0xf65d9144, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x509d9147, 95 },
    { 0x9e9d9148, 23 },
    { 0x237d9149, 95 },
    { 0x2b1d9149, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3d914d, 80 },
    { 0, 0 },
    { 0x879d914f, 98 },
    { 0x6edd9150, 43 },
    { 0x166d9151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4d915f, 61 },
    { 0x7c6d915f, 69 },
    { 0x782d9161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ad9168, 95 },
    { 0, 0 },
    { 0x5fad916a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9d9172, 40 },
    { 0xa6ed9173, 68 },
    { 0x7efd9172, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed9178, 93 },
    { 0x64ed9179, 84 },
    { 0x601d917a, 67 },
    { 0x604d9179, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bd9180, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dd9184, 98 },
    { 0xe97d9185, 90 },
    { 0x424d9186, 33 },
    { 0xeb1d9187, 66 },
    { 0x77fd9184, 34 },
    { 0x5cfd9189, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ded918c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9d9190, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7d9194, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cd919a, 95 },
    { 0x7fcd919b, 34 },
    { 0x5bdd919c, 34 },
    { 0, 0 },
    { 0xdced919e, 55 },
    { 0, 0 },
    { 0xf67d91a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x195d91a3, 90 },
    { 0, 0 },
    { 0xb27d91a5, 76 },
    { 0xa36d91a5, 95 },
    { 0xe81d91a7, 23 },
    { 0x36ad91a7, 61 },
    { 0, 0 },
    { 0x97fd91aa, 90 },
    { 0x432d91ab, 90 },
    { 0, 0 },
    { 0xee9d91ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657d91b1, 39 },
    { 0xe0dd91b2, 95 },
    { 0xe47d91b3, 23 },
    { 0x5b3d91b1, 46 },
    { 0, 0 },
    { 0x157d91b6, 95 },
    { 0xb66d91b7, 90 },
    { 0, 0 },
    { 0x338d91b9, 61 },
    { 0x282d91ba, 93 },
    { 0x137d91ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fd91c4, 76 },
    { 0x26fd91c5, 95 },
    { 0x7ead91c6, 23 },
    { 0, 0 },
    { 0x1d5d91c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72d91cb, 33 },
    { 0, 0 },
    { 0x205d91cd, 77 },
    { 0xbbbd91ce, 46 },
    { 0x5e4d91ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfd91d3, 43 },
    { 0x3b9d91d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229d91d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89d91dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4d91df, 76 },
    { 0x115d91e0, 23 },
    { 0x1f6d91e1, 98 },
    { 0xcdad91e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fd91e9, 43 },
    { 0x43d91ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4d91ee, 34 },
    { 0x3d3d91ef, 46 },
    { 0xedfd91f0, 76 },
    { 0x80ed91f0, 95 },
    { 0x75d91f0, 90 },
    { 0xf04d91f3, 93 },
    { 0, 0 },
    { 0x30d91f5, 23 },
    { 0x999d91f6, 76 },
    { 0xc35d91f5, 95 },
    { 0, 0 },
    { 0xeccd91f9, 98 },
    { 0x5a9d91fa, 98 },
    { 0, 0 },
    { 0xa84d91fc, 61 },
    { 0, 0 },
    { 0x245d91fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94dd9202, 95 },
    { 0x5f7d9202, 96 },
    { 0xc9bd9203, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bd9209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ed9212, 98 },
    { 0x7e7d9213, 61 },
    { 0x890d9212, 61 },
    { 0xa4cd9215, 98 },
    { 0xed1d9212, 84 },
    { 0, 0 },
    { 0x627d9218, 91 },
    { 0x9c4d9218, 33 },
    { 0x2aed9219, 33 },
    { 0x719d9219, 78 },
    { 0x246d921c, 67 },
    { 0x880d921c, 23 },
    { 0x5b3d921d, 23 },
    { 0x726d921b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ed9223, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dd9226, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04d9235, 95 },
    { 0, 0 },
    { 0xb7fd9237, 99 },
    { 0x302d9238, 95 },
    { 0x403d9239, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3d923c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ead9242, 55 },
    { 0, 0 },
    { 0x8f1d9244, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ed9247, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x191d924a, 34 },
    { 0xabfd924b, 76 },
    { 0x943d924c, 78 },
    { 0x8fad924c, 90 },
    { 0xe60d924c, 55 },
    { 0x6e7d924f, 61 },
    { 0, 0 },
    { 0x41dd9251, 34 },
    { 0x5a0d9252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x632d9255, 93 },
    { 0xdbbd9255, 55 },
    { 0xa08d9256, 55 },
    { 0xcbdd9258, 95 },
    { 0, 0 },
    { 0xb35d925a, 99 },
    { 0, 0 },
    { 0xe4dd925c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ad925f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfd9268, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871d926e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ad9271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81d9275, 61 },
    { 0x7a6d9276, 90 },
    { 0xf89d9277, 90 },
    { 0x5f6d9278, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bd927e, 99 },
    { 0, 0 },
    { 0xa40d9280, 23 },
    { 0x228d9281, 66 },
    { 0xf37d9282, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ed928f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4d9296, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfd929a, 33 },
    { 0x692d929a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x541d929e, 90 },
    { 0, 0 },
    { 0x37cd92a0, 93 },
    { 0xbbbd92a0, 34 },
    { 0, 0 },
    { 0x825d92a3, 87 },
    { 0x9fcd92a4, 52 },
    { 0x14dd92a5, 90 },
    { 0, 0 },
    { 0xd76d92a7, 33 },
    { 0xf80d92a8, 23 },
    { 0, 0 },
    { 0x97d92aa, 88 },
    { 0x438d92aa, 61 },
    { 0x860d92aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2bd92b1, 46 },
    { 0, 0 },
    { 0x26bd92b3, 93 },
    { 0, 0 },
    { 0x5e4d92b5, 43 },
    { 0, 0 },
    { 0xa24d92b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dd92bb, 76 },
    { 0x8fed92bc, 95 },
    { 0, 0 },
    { 0x98ad92be, 88 },
    { 0x270d92be, 90 },
    { 0x9ccd92c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x830d92c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640d92c7, 61 },
    { 0x45dd92c8, 33 },
    { 0xeb3d92c8, 95 },
    { 0xf73d92c7, 98 },
    { 0x736d92cb, 93 },
    { 0x722d92cc, 81 },
    { 0x67cd92cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1d92d3, 93 },
    { 0x6c1d92d4, 33 },
    { 0x84dd92d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531d92d9, 66 },
    { 0, 0 },
    { 0x82dd92db, 98 },
    { 0, 0 },
    { 0xdc0d92dd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cd92e4, 77 },
    { 0, 0 },
    { 0x9dbd92e6, 33 },
    { 0, 0 },
    { 0xd3dd92e8, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fd92ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x243d92ef, 91 },
    { 0x1a5d92f0, 61 },
    { 0x226d92ef, 76 },
    { 0xd10d92f2, 90 },
    { 0x259d92f3, 65 },
    { 0x161d92f2, 90 },
    { 0x8ccd92f3, 33 },
    { 0x3cad92ef, 95 },
    { 0xf9ed92f7, 95 },
    { 0xff1d92f8, 80 },
    { 0xc59d92f9, 91 },
    { 0x4dad92f9, 100 },
    { 0x382d92f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3d9306, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x217d930b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32d930f, 46 },
    { 0, 0 },
    { 0x611d9311, 76 },
    { 0, 0 },
    { 0x36d9313, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42d9318, 33 },
    { 0, 0 },
    { 0x86cd931a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856d931e, 84 },
    { 0xcf0d931e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229d9325, 98 },
    { 0, 0 },
    { 0x21dd9327, 55 },
    { 0xc70d9328, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2d932b, 100 },
    { 0x25fd932b, 99 },
    { 0, 0 },
    { 0xda0d932e, 90 },
    { 0xf06d932f, 95 },
    { 0xc34d9330, 95 },
    { 0xac8d9330, 75 },
    { 0, 0 },
    { 0xfbd9333, 23 },
    { 0x451d9333, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11d9337, 95 },
    { 0x84ed9338, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafd933c, 61 },
    { 0xb1cd933c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bd9342, 91 },
    { 0x3e7d9342, 23 },
    { 0x667d9344, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d9348, 85 },
    { 0xb59d9349, 90 },
    { 0x7bbd9348, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fd934d, 34 },
    { 0x21dd934e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x924d9351, 49 },
    { 0xdc9d9352, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707d935a, 61 },
    { 0xdbfd935b, 76 },
    { 0xa96d935a, 23 },
    { 0xfe9d935d, 96 },
    { 0x4f8d935c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00d9364, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29dd9369, 33 },
    { 0, 0 },
    { 0x496d936b, 43 },
    { 0x7d5d936c, 90 },
    { 0x178d936d, 33 },
    { 0xfb9d936c, 95 },
    { 0x4a2d936f, 95 },
    { 0xab8d9370, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3d9375, 93 },
    { 0xcacd9376, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ad9381, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1d9386, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9d9392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737d939f, 39 },
    { 0xea2d93a0, 23 },
    { 0x800d93a0, 64 },
    { 0, 0 },
    { 0xa62d93a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639d93a9, 46 },
    { 0x626d93a9, 55 },
    { 0x1d6d93ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276d93af, 90 },
    { 0x63cd93b0, 98 },
    { 0x7f6d93b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6d93b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919d93b8, 46 },
    { 0x8d93b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8d93c1, 34 },
    { 0xcccd93c1, 23 },
    { 0x9ccd93c1, 75 },
    { 0xdb1d93c4, 34 },
    { 0x62ed93c5, 61 },
    { 0xdffd93c6, 83 },
    { 0x779d93c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8d93ca, 95 },
    { 0x1b2d93cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59d93ce, 98 },
    { 0xfcbd93cf, 34 },
    { 0, 0 },
    { 0x3d2d93d1, 100 },
    { 0xe81d93d2, 61 },
    { 0xfcfd93d2, 90 },
    { 0x326d93d4, 33 },
    { 0xf61d93d3, 55 },
    { 0, 0 },
    { 0xc31d93d7, 33 },
    { 0xce2d93d8, 88 },
    { 0x49bd93d7, 75 },
    { 0, 0 },
    { 0x571d93db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bd93e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cd93e7, 23 },
    { 0x989d93e8, 55 },
    { 0x137d93e9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ed93f1, 95 },
    { 0x5aad93f2, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ed93f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8d93fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cd9400, 98 },
    { 0x7ed9400, 98 },
    { 0x990d9402, 76 },
    { 0x473d9402, 98 },
    { 0x1bd9402, 90 },
    { 0, 0 },
    { 0x21d9406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0d9409, 98 },
    { 0, 0 },
    { 0x709d940b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ad9411, 80 },
    { 0x39bd9412, 91 },
    { 0, 0 },
    { 0x3e1d9414, 61 },
    { 0xaf1d9415, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdd9422, 76 },
    { 0, 0 },
    { 0x5cd9424, 91 },
    { 0x379d9425, 95 },
    { 0, 0 },
    { 0x425d9427, 17 },
    { 0x8e3d9427, 76 },
    { 0xf5ed9429, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bd942c, 77 },
    { 0xb5cd942d, 23 },
    { 0, 0 },
    { 0xc68d942f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0d9432, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7d9436, 55 },
    { 0, 0 },
    { 0x7efd9438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fd943b, 95 },
    { 0, 0 },
    { 0x9f8d943d, 76 },
    { 0x663d943e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730d9446, 61 },
    { 0xee9d9447, 95 },
    { 0xb21d9446, 95 },
    { 0, 0 },
    { 0xb1d944a, 55 },
    { 0x856d944a, 90 },
    { 0x1e8d944c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cd9452, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233d945b, 78 },
    { 0x158d945c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1d945f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fd9466, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97dd946c, 98 },
    { 0x7c6d946d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9d9470, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408d9478, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39d9480, 33 },
    { 0x74bd9481, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6d9485, 91 },
    { 0x27cd9486, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1d9489, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x388d948c, 46 },
    { 0x6d6d948d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19d9490, 81 },
    { 0x787d9490, 99 },
    { 0, 0 },
    { 0xa4cd9493, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d9499, 52 },
    { 0xd0dd9499, 61 },
    { 0, 0 },
    { 0xecbd949c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x192d94a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d94ab, 69 },
    { 0, 0 },
    { 0x2dd94ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x890d94b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6d94b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafd94ba, 95 },
    { 0xf1dd94ba, 90 },
    { 0x8f7d94bc, 38 },
    { 0x5c3d94bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4d94c0, 67 },
    { 0x5cdd94c1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584d94c5, 33 },
    { 0x338d94c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x376d94c9, 34 },
    { 0x1bdd94ca, 45 },
    { 0, 0 },
    { 0x6a9d94cc, 33 },
    { 0xfa6d94cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x171d94d0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ed94d5, 76 },
    { 0x710d94d6, 91 },
    { 0xc32d94d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1d94db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cd94e0, 67 },
    { 0x46ad94e1, 17 },
    { 0xb80d94e2, 91 },
    { 0xd85d94e3, 87 },
    { 0, 0 },
    { 0xd0ad94e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34d94eb, 23 },
    { 0, 0 },
    { 0xb02d94ed, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ad94f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fd94f3, 61 },
    { 0x970d94f4, 33 },
    { 0, 0 },
    { 0x45ed94f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd94fc, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecd9505, 33 },
    { 0xc00d9505, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43d950f, 23 },
    { 0, 0 },
    { 0xdf0d9511, 61 },
    { 0x5efd9512, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfd9515, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dd951c, 67 },
    { 0x39dd951d, 95 },
    { 0xe0cd951d, 61 },
    { 0x3bad951d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141d9524, 67 },
    { 0xbe7d9525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x948d9528, 33 },
    { 0xc78d9528, 98 },
    { 0xf25d952a, 98 },
    { 0x17dd9528, 43 },
    { 0x1a7d9529, 61 },
    { 0xdad952a, 61 },
    { 0x99dd952e, 23 },
    { 0xa40d9528, 95 },
    { 0xf05d952d, 96 },
    { 0, 0 },
    { 0x14cd9532, 88 },
    { 0, 0 },
    { 0x6fdd9534, 90 },
    { 0x832d9534, 76 },
    { 0xe29d9534, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215d953e, 99 },
    { 0x223d953f, 55 },
    { 0, 0 },
    { 0x995d9541, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0d9544, 81 },
    { 0, 0 },
    { 0xfe4d9546, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dd954e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ad9553, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x448d9556, 33 },
    { 0, 0 },
    { 0x2dbd9558, 55 },
    { 0, 0 },
    { 0xdb0d955a, 98 },
    { 0xafd955b, 90 },
    { 0xe10d955c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedd955f, 55 },
    { 0x104d955f, 46 },
    { 0x689d9560, 43 },
    { 0xc0cd955f, 91 },
    { 0xf0fd955f, 98 },
    { 0xead9564, 93 },
    { 0, 0 },
    { 0xe30d9566, 90 },
    { 0, 0 },
    { 0xdd0d9568, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73d956e, 55 },
    { 0x1dbd956f, 52 },
    { 0x7f6d9570, 76 },
    { 0, 0 },
    { 0x53cd9572, 93 },
    { 0xdeed9573, 90 },
    { 0x27ed9574, 33 },
    { 0x424d9575, 78 },
    { 0x68bd9575, 68 },
    { 0xf99d9575, 90 },
    { 0x3e2d9573, 77 },
    { 0x57bd9579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2d957d, 23 },
    { 0x930d957e, 75 },
    { 0x5b0d957f, 33 },
    { 0x6cd957f, 25 },
    { 0, 0 },
    { 0x60bd9582, 33 },
    { 0x8d7d9582, 33 },
    { 0, 0 },
    { 0x41dd9585, 34 },
    { 0, 0 },
    { 0x76cd9587, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x63d958a, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8d958f, 93 },
    { 0xc03d9590, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcad9593, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ad9598, 88 },
    { 0x494d9598, 23 },
    { 0x702d959a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fd959f, 90 },
    { 0x1bbd95a0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6d95a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5d95a9, 61 },
    { 0, 0 },
    { 0x441d95ab, 38 },
    { 0x9e6d95ab, 95 },
    { 0x17ad95ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86d95b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70d95bd, 100 },
    { 0, 0 },
    { 0x532d95bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ed95c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00d95c6, 33 },
    { 0x2efd95c7, 33 },
    { 0xd1d95c8, 77 },
    { 0, 0 },
    { 0x34cd95ca, 98 },
    { 0x9c4d95cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52d95d0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48d95d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ed95d9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2d95de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cd95e1, 95 },
    { 0x11cd95e2, 52 },
    { 0, 0 },
    { 0x863d95e4, 95 },
    { 0, 0 },
    { 0x6ced95e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dd95ec, 95 },
    { 0xe9d95ed, 93 },
    { 0x49bd95ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11d95f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309d95f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x73d95f9, 56 },
    { 0, 0 },
    { 0x690d95fb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6d9602, 76 },
    { 0x5c5d9602, 61 },
    { 0xbc3d9602, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3d960c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14d960f, 89 },
    { 0, 0 },
    { 0xc00d9611, 23 },
    { 0xda4d9612, 95 },
    { 0x5b2d9613, 77 },
    { 0, 0 },
    { 0x4dad9615, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x30dd9618, 98 },
    { 0x14ad9619, 33 },
    { 0, 0 },
    { 0x296d961b, 92 },
    { 0x667d961c, 77 },
    { 0x81cd961c, 75 },
    { 0xb80d961e, 61 },
    { 0xc9dd961d, 12 },
    { 0x3f4d9620, 76 },
    { 0x5d4d9621, 76 },
    { 0, 0 },
    { 0xa7ad9623, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x624d9626, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43d962c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2d962f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133d9636, 76 },
    { 0, 0 },
    { 0x165d9638, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825d9643, 90 },
    { 0, 0 },
    { 0x204d9645, 95 },
    { 0x5ded9646, 33 },
    { 0x4bad9646, 100 },
    { 0x767d9645, 78 },
    { 0xdddd9649, 95 },
    { 0, 0 },
    { 0xeead964b, 90 },
    { 0x35cd964c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674d9650, 36 },
    { 0, 0 },
    { 0x90cd9652, 33 },
    { 0x76d9653, 46 },
    { 0x358d9654, 61 },
    { 0xd03d9652, 78 },
    { 0xb5dd9652, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x650d9659, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83d965d, 81 },
    { 0xf3d965e, 67 },
    { 0xf08d965e, 52 },
    { 0xa22d9660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1d9663, 46 },
    { 0xd49d9664, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dd9669, 87 },
    { 0x224d966a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696d966f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bd9672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08d9679, 94 },
    { 0xf88d967a, 77 },
    { 0, 0 },
    { 0xcd3d967c, 95 },
    { 0, 0 },
    { 0x263d967e, 43 },
    { 0, 0 },
    { 0x83d9680, 76 },
    { 0, 0 },
    { 0x260d9682, 61 },
    { 0x1f3d9683, 90 },
    { 0xd42d9682, 34 },
    { 0x430d9685, 95 },
    { 0x804d9686, 99 },
    { 0, 0 },
    { 0xdc5d9688, 61 },
    { 0x8cd9689, 95 },
    { 0x8a4d9689, 33 },
    { 0, 0 },
    { 0x15ed968c, 90 },
    { 0, 0 },
    { 0xf90d968e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34bd9692, 76 },
    { 0x698d9692, 34 },
    { 0xce8d9694, 33 },
    { 0, 0 },
    { 0x604d9696, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d969b, 90 },
    { 0x7e2d969c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ad969f, 77 },
    { 0x6cad96a0, 76 },
    { 0xa2cd96a1, 99 },
    { 0xd6fd969f, 46 },
    { 0, 0 },
    { 0x476d96a4, 76 },
    { 0x184d96a5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edd96ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5d96b1, 84 },
    { 0xc51d96b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0d96b9, 23 },
    { 0xbe1d96b9, 33 },
    { 0x9add96bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dd96c9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11d96cc, 90 },
    { 0, 0 },
    { 0xe9d96ce, 90 },
    { 0x93ed96ce, 33 },
    { 0, 0 },
    { 0xdcad96d1, 61 },
    { 0xb5d96d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9d96d9, 43 },
    { 0x89dd96d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858d96de, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78d96e3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fd96e7, 90 },
    { 0x680d96e8, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675d96f1, 38 },
    { 0x979d96f2, 46 },
    { 0xfc9d96f3, 81 },
    { 0, 0 },
    { 0xfb1d96f5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced96f8, 95 },
    { 0x21fd96f9, 85 },
    { 0x452d96fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cd96fe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60d9702, 52 },
    { 0, 0 },
    { 0x940d9704, 91 },
    { 0, 0 },
    { 0x11dd9706, 95 },
    { 0, 0 },
    { 0x391d9708, 95 },
    { 0, 0 },
    { 0x47fd970a, 100 },
    { 0, 0 },
    { 0xc8cd970c, 67 },
    { 0x620d970c, 67 },
    { 0x557d970e, 33 },
    { 0xdbad970e, 23 },
    { 0xe8ed970d, 90 },
    { 0x286d9711, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ed9714, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ced9719, 98 },
    { 0x87dd971a, 34 },
    { 0x572d971b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5d971e, 61 },
    { 0xdbfd971f, 61 },
    { 0x1bbd971f, 33 },
    { 0x48ed9721, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xfad9724, 33 },
    { 0x4d3d9725, 95 },
    { 0x58d9726, 61 },
    { 0x1b7d9727, 61 },
    { 0x904d9727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7d972b, 98 },
    { 0xf8fd972c, 61 },
    { 0, 0 },
    { 0xa3fd972e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ed973a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x473d973d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1d9740, 33 },
    { 0x304d9741, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ed9745, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0d9752, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x412d9755, 77 },
    { 0x194d9756, 52 },
    { 0, 0 },
    { 0xf43d9758, 98 },
    { 0, 0 },
    { 0xf27d975a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18d975e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134d9762, 55 },
    { 0x307d9763, 34 },
    { 0, 0 },
    { 0xaf0d9765, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47d9768, 95 },
    { 0xff7d9768, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0d9775, 98 },
    { 0, 0 },
    { 0x6c5d9777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933d977b, 76 },
    { 0x98dd977b, 61 },
    { 0x383d977d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dd9783, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7d978a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ed978f, 84 },
    { 0, 0 },
    { 0x13bd9791, 80 },
    { 0xbd2d9792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fd9798, 61 },
    { 0x63cd9799, 95 },
    { 0x195d979a, 90 },
    { 0xc16d979b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51d97a0, 95 },
    { 0x246d97a1, 68 },
    { 0x45fd97a2, 65 },
    { 0xc6ed97a2, 100 },
    { 0xe3d97a2, 61 },
    { 0x1cfd97a5, 23 },
    { 0x9acd97a5, 90 },
    { 0xcd0d97a7, 55 },
    { 0xa40d97a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedd97ab, 69 },
    { 0x5e5d97ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd97af, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0d97b3, 78 },
    { 0x7e0d97b4, 65 },
    { 0x66ad97b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52d97ba, 98 },
    { 0xa66d97ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568d97bf, 76 },
    { 0xbb6d97c0, 87 },
    { 0x291d97c1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bd97c4, 93 },
    { 0xad1d97c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3d97c9, 90 },
    { 0x8fad97ca, 61 },
    { 0, 0 },
    { 0x6b1d97cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6d97cf, 61 },
    { 0, 0 },
    { 0x98fd97d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaced97d4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2d97d8, 55 },
    { 0x2b5d97d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9d97e0, 99 },
    { 0x4add97e1, 34 },
    { 0x1a8d97e2, 99 },
    { 0x71d97e3, 83 },
    { 0x45cd97e2, 53 },
    { 0, 0 },
    { 0x7d97e6, 66 },
    { 0, 0 },
    { 0x73d97e8, 61 },
    { 0x684d97e9, 90 },
    { 0x2b7d97e9, 93 },
    { 0, 0 },
    { 0x7cad97ec, 65 },
    { 0x24dd97ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cd97f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55d97f7, 94 },
    { 0, 0 },
    { 0x5cbd97f9, 91 },
    { 0, 0 },
    { 0x4dbd97fb, 90 },
    { 0x784d97fc, 91 },
    { 0xfdad97fd, 55 },
    { 0xaa7d97fe, 33 },
    { 0, 0 },
    { 0xa5dd9800, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46d9803, 95 },
    { 0xb51d9804, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x432d9807, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10d980a, 34 },
    { 0x7d2d980b, 61 },
    { 0x623d980c, 90 },
    { 0x2a1d980c, 95 },
    { 0, 0 },
    { 0xd01d980f, 33 },
    { 0, 0 },
    { 0xbb0d9811, 90 },
    { 0, 0 },
    { 0x29ed9813, 33 },
    { 0x815d9814, 36 },
    { 0xb50d9815, 43 },
    { 0x1b9d9814, 46 },
    { 0x321d9816, 33 },
    { 0x9bad9818, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbd981b, 90 },
    { 0, 0 },
    { 0xf20d981d, 55 },
    { 0x3a0d981e, 55 },
    { 0, 0 },
    { 0x9b7d9820, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2d9829, 76 },
    { 0x431d982a, 93 },
    { 0, 0 },
    { 0x19fd982c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ad9834, 33 },
    { 0xab5d9834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96d983a, 100 },
    { 0x95ed983b, 93 },
    { 0, 0 },
    { 0xeb9d983d, 100 },
    { 0xb8cd983e, 91 },
    { 0x78cd983d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fd9843, 90 },
    { 0, 0 },
    { 0xf4ed9845, 33 },
    { 0x56fd9845, 33 },
    { 0, 0 },
    { 0xa2ed9848, 98 },
    { 0, 0 },
    { 0xd4d984a, 23 },
    { 0xb0dd984b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7d9853, 90 },
    { 0, 0 },
    { 0xd21d9855, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606d985b, 90 },
    { 0x283d985b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465d9861, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4d9864, 34 },
    { 0xe75d9865, 98 },
    { 0xcb4d9866, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d986b, 78 },
    { 0xeb4d986c, 46 },
    { 0x922d986d, 61 },
    { 0xa6dd986d, 23 },
    { 0x580d986e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613d9873, 34 },
    { 0x97ed9874, 76 },
    { 0x4bbd9873, 23 },
    { 0x31d9876, 93 },
    { 0xa7d9873, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4d987a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2d987e, 96 },
    { 0x423d987e, 23 },
    { 0xfefd9880, 76 },
    { 0x8bfd9881, 61 },
    { 0, 0 },
    { 0xea3d9883, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8d9887, 61 },
    { 0, 0 },
    { 0x60ad9889, 23 },
    { 0, 0 },
    { 0x465d988b, 95 },
    { 0, 0 },
    { 0xab3d988d, 77 },
    { 0x230d988e, 14 },
    { 0x256d988f, 33 },
    { 0x52bd988f, 23 },
    { 0x6a4d9891, 99 },
    { 0xbd7d988e, 55 },
    { 0x639d9893, 23 },
    { 0, 0 },
    { 0xc43d9895, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3dd9899, 98 },
    { 0xc00d989a, 89 },
    { 0xe06d989b, 90 },
    { 0x169d9899, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fd989f, 91 },
    { 0xe11d989f, 90 },
    { 0x2fdd98a1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d98a5, 33 },
    { 0x729d98a6, 80 },
    { 0x515d98a6, 76 },
    { 0x6acd98a8, 23 },
    { 0x445d98a9, 76 },
    { 0, 0 },
    { 0x164d98ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d98b1, 33 },
    { 0, 0 },
    { 0x8bad98b3, 65 },
    { 0x672d98b4, 61 },
    { 0x7b4d98b4, 23 },
    { 0x9dbd98b4, 76 },
    { 0xafcd98b7, 23 },
    { 0x42ed98b8, 95 },
    { 0x413d98b9, 90 },
    { 0xacbd98b6, 46 },
    { 0, 0 },
    { 0xd77d98bc, 66 },
    { 0xe33d98bc, 55 },
    { 0, 0 },
    { 0x72dd98bf, 33 },
    { 0, 0 },
    { 0x857d98c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5d98c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd98ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38d98ce, 95 },
    { 0, 0 },
    { 0x87bd98d0, 95 },
    { 0x2a3d98d1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fd98d9, 91 },
    { 0x197d98d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56d98dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8add98e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6d98e8, 98 },
    { 0x59cd98e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefd98ec, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddd98f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283d98f4, 23 },
    { 0x2e4d98f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725d98fc, 98 },
    { 0x114d98fc, 17 },
    { 0xfa1d98fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bad9907, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabed990d, 76 },
    { 0, 0 },
    { 0x6bfd990f, 95 },
    { 0x97bd9910, 90 },
    { 0, 0 },
    { 0x788d9912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cd9917, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecd991a, 91 },
    { 0xb7bd991b, 76 },
    { 0x375d991b, 90 },
    { 0xb5d991b, 33 },
    { 0xf02d991e, 33 },
    { 0, 0 },
    { 0xfddd9920, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198d9924, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74d9928, 33 },
    { 0xbbcd9928, 61 },
    { 0xe53d9929, 90 },
    { 0, 0 },
    { 0x96d992c, 61 },
    { 0, 0 },
    { 0x93cd992e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bd9931, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486d9936, 90 },
    { 0xfd0d9937, 90 },
    { 0x457d9936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1d993b, 61 },
    { 0xd5ed993b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ad993f, 23 },
    { 0, 0 },
    { 0x149d9941, 33 },
    { 0x997d9941, 95 },
    { 0x694d9942, 98 },
    { 0, 0 },
    { 0xd70d9945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcd994a, 98 },
    { 0xa1dd994a, 8 },
    { 0, 0 },
    { 0x4c8d994d, 76 },
    { 0, 0 },
    { 0xe57d994f, 55 },
    { 0, 0 },
    { 0x5bbd9951, 23 },
    { 0xd6dd9952, 90 },
    { 0x38dd9951, 99 },
    { 0, 0 },
    { 0x828d9955, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40d995a, 98 },
    { 0, 0 },
    { 0x2d3d995c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1d995f, 23 },
    { 0, 0 },
    { 0x229d9961, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efd996a, 85 },
    { 0x938d996b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13d996f, 13 },
    { 0xbfed996f, 33 },
    { 0x4c2d9971, 100 },
    { 0x12ed9972, 55 },
    { 0x37bd9972, 76 },
    { 0xbcdd9972, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11d9977, 33 },
    { 0, 0 },
    { 0x4b3d9979, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4d997e, 69 },
    { 0x6d4d997f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27d9983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83d9988, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67d998b, 23 },
    { 0x9e3d998c, 60 },
    { 0, 0 },
    { 0xb0ed998e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26d9992, 90 },
    { 0x644d9993, 34 },
    { 0x75fd9993, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95d999e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572d99a4, 23 },
    { 0, 0 },
    { 0x108d99a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65d99b1, 95 },
    { 0x912d99b2, 55 },
    { 0x2b0d99b3, 68 },
    { 0xf15d99b2, 90 },
    { 0x4dad99b5, 76 },
    { 0x14ed99b6, 43 },
    { 0, 0 },
    { 0xdc7d99b8, 61 },
    { 0, 0 },
    { 0x5a5d99ba, 46 },
    { 0, 0 },
    { 0x12ed99bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dd99bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15cd99c3, 33 },
    { 0x1a2d99c3, 23 },
    { 0x424d99c4, 93 },
    { 0x824d99c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bd99ca, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120d99d2, 101 },
    { 0, 0 },
    { 0xa40d99d4, 52 },
    { 0xceed99d5, 91 },
    { 0x940d99d6, 23 },
    { 0x824d99d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x906d99da, 98 },
    { 0x14d99db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2d99de, 98 },
    { 0, 0 },
    { 0x8b7d99e0, 34 },
    { 0, 0 },
    { 0x735d99e2, 99 },
    { 0x897d99e2, 66 },
    { 0x5efd99e4, 78 },
    { 0x13dd99e5, 33 },
    { 0x7a1d99e4, 43 },
    { 0x9efd99e6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ed99ea, 61 },
    { 0, 0 },
    { 0xa11d99ec, 90 },
    { 0xa1d99ed, 23 },
    { 0x16cd99ee, 66 },
    { 0, 0 },
    { 0xd38d99f0, 61 },
    { 0x919d99f1, 42 },
    { 0x74ed99f2, 33 },
    { 0x137d99f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8d99f8, 78 },
    { 0x11ed99f8, 90 },
    { 0xfa3d99f9, 91 },
    { 0xe4dd99fb, 23 },
    { 0x511d99f8, 76 },
    { 0x7cfd99fd, 55 },
    { 0x755d99fb, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3d9a04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7d9a09, 61 },
    { 0, 0 },
    { 0x582d9a0b, 55 },
    { 0, 0 },
    { 0x7b2d9a0d, 95 },
    { 0, 0 },
    { 0x34dd9a0f, 23 },
    { 0x3ffd9a0f, 76 },
    { 0xeead9a11, 91 },
    { 0, 0 },
    { 0x264d9a13, 23 },
    { 0xe14d9a14, 66 },
    { 0xa24d9a15, 96 },
    { 0x164d9a14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874d9a1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dd9a22, 67 },
    { 0, 0 },
    { 0x6d5d9a24, 77 },
    { 0xeb7d9a25, 97 },
    { 0x441d9a25, 52 },
    { 0xce8d9a27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3d9a2c, 61 },
    { 0x69dd9a2c, 23 },
    { 0xff6d9a2c, 33 },
    { 0xa03d9a2e, 33 },
    { 0x515d9a2f, 61 },
    { 0x7efd9a2d, 66 },
    { 0x101d9a31, 61 },
    { 0x78ed9a31, 98 },
    { 0x5b9d9a32, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3d9a3a, 87 },
    { 0, 0 },
    { 0x971d9a3c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81d9a3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5d9a42, 76 },
    { 0x488d9a43, 33 },
    { 0, 0 },
    { 0xe73d9a45, 90 },
    { 0xf81d9a45, 81 },
    { 0xb62d9a47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dd9a4c, 61 },
    { 0, 0 },
    { 0x645d9a4e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cd9a53, 91 },
    { 0x868d9a53, 46 },
    { 0xc95d9a55, 100 },
    { 0x4dfd9a54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401d9a5d, 95 },
    { 0xb2d9a5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cd9a63, 76 },
    { 0xe87d9a64, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2d9a6b, 17 },
    { 0xfb4d9a6b, 34 },
    { 0xdddd9a6d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377d9a74, 34 },
    { 0, 0 },
    { 0xa88d9a76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3d9a7d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7d9a86, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7d9a8d, 61 },
    { 0xbccd9a8e, 61 },
    { 0xda2d9a8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645d9a94, 76 },
    { 0xb27d9a94, 23 },
    { 0, 0 },
    { 0x6d0d9a97, 91 },
    { 0xc8bd9a97, 98 },
    { 0x4a4d9a97, 90 },
    { 0x71cd9a9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fd9a9d, 87 },
    { 0x8dd9a9e, 91 },
    { 0x3fbd9a9e, 95 },
    { 0x34bd9aa0, 93 },
    { 0x39ed9aa1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67d9aa5, 90 },
    { 0xc2ad9aa6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ed9aaa, 81 },
    { 0x643d9aaa, 55 },
    { 0xe8fd9aaa, 43 },
    { 0x38cd9aad, 95 },
    { 0x602d9aad, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824d9ab3, 34 },
    { 0xbd8d9ab4, 52 },
    { 0x476d9ab5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ad9abc, 98 },
    { 0x2b3d9abd, 53 },
    { 0x672d9abe, 33 },
    { 0x1add9abf, 87 },
    { 0x46ed9ac0, 52 },
    { 0, 0 },
    { 0xa96d9ac2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6d9acf, 75 },
    { 0x820d9acf, 76 },
    { 0xb85d9ad1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93d9ae0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dd9ae3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56d9ae8, 88 },
    { 0, 0 },
    { 0xeaed9aea, 91 },
    { 0xcd5d9aeb, 23 },
    { 0xe9ed9aec, 66 },
    { 0, 0 },
    { 0xc36d9aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08d9af1, 76 },
    { 0x4b2d9af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4d9b02, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2d9b05, 87 },
    { 0x3d5d9b06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694d9b0a, 65 },
    { 0x753d9b0b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490d9b11, 46 },
    { 0x6fdd9b12, 61 },
    { 0x17ed9b13, 93 },
    { 0x942d9b11, 66 },
    { 0xe9fd9b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85d9b1c, 95 },
    { 0, 0 },
    { 0x666d9b1e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x788d9b21, 46 },
    { 0xb72d9b22, 61 },
    { 0x2c1d9b22, 90 },
    { 0x293d9b22, 61 },
    { 0x74ed9b23, 67 },
    { 0x6a2d9b22, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55d9b29, 90 },
    { 0, 0 },
    { 0xf1dd9b2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978d9b35, 90 },
    { 0xc71d9b36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9b3b, 75 },
    { 0xe7dd9b3c, 23 },
    { 0, 0 },
    { 0x6a2d9b3e, 95 },
    { 0x27bd9b3f, 77 },
    { 0x507d9b3f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x108d9b43, 87 },
    { 0xc7ed9b44, 46 },
    { 0, 0 },
    { 0xe8bd9b46, 95 },
    { 0x7c7d9b46, 23 },
    { 0x66ed9b47, 90 },
    { 0x70bd9b49, 87 },
    { 0xcd7d9b4a, 33 },
    { 0x3fcd9b4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36d9b56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aed9b5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x690d9b5d, 85 },
    { 0x85fd9b5e, 78 },
    { 0x1afd9b5f, 77 },
    { 0x4c3d9b5f, 61 },
    { 0xc6d9b61, 90 },
    { 0x638d9b5f, 61 },
    { 0x240d9b60, 84 },
    { 0x976d9b64, 95 },
    { 0x9f5d9b65, 34 },
    { 0x997d9b5e, 61 },
    { 0xa15d9b67, 23 },
    { 0xbfdd9b64, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dd9b6b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3d9b6e, 40 },
    { 0x869d9b6e, 95 },
    { 0xbfed9b70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2d9b75, 61 },
    { 0, 0 },
    { 0x5f7d9b77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28d9b7e, 23 },
    { 0xf6ed9b7f, 33 },
    { 0xfe3d9b80, 93 },
    { 0xd7ed9b81, 61 },
    { 0x776d9b82, 77 },
    { 0x7e2d9b7e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97d9b86, 76 },
    { 0x6a9d9b87, 41 },
    { 0xf9ed9b87, 98 },
    { 0xd8cd9b88, 90 },
    { 0xffcd9b8a, 98 },
    { 0x7ebd9b87, 90 },
    { 0, 0 },
    { 0x910d9b8d, 33 },
    { 0, 0 },
    { 0x2d8d9b8f, 66 },
    { 0, 0 },
    { 0x374d9b91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bd9b94, 85 },
    { 0, 0 },
    { 0xb92d9b96, 55 },
    { 0x72fd9b96, 58 },
    { 0x370d9b98, 23 },
    { 0x6cbd9b98, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181d9b9e, 23 },
    { 0x1dad9b9e, 33 },
    { 0, 0 },
    { 0x746d9ba1, 98 },
    { 0x2ced9ba2, 98 },
    { 0x5ded9ba1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206d9bab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9d9bb0, 78 },
    { 0xbcd9bb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cd9bb4, 52 },
    { 0x91ed9bb5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d0d9bc3, 61 },
    { 0x9e7d9bc4, 98 },
    { 0xeead9bc4, 90 },
    { 0x671d9bc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22d9bca, 33 },
    { 0x81bd9bcb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6d9bcf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92d9bd8, 61 },
    { 0, 0 },
    { 0xa4ad9bda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cd9be0, 66 },
    { 0x8d7d9be1, 76 },
    { 0xdadd9be2, 23 },
    { 0x133d9be3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556d9be9, 23 },
    { 0x284d9bea, 93 },
    { 0x86ed9beb, 95 },
    { 0xdbcd9bec, 33 },
    { 0xa59d9bec, 33 },
    { 0xfb0d9bed, 43 },
    { 0xb69d9bef, 90 },
    { 0x6dfd9bf0, 95 },
    { 0, 0 },
    { 0x3a8d9bf2, 23 },
    { 0x49d9bf2, 40 },
    { 0xea6d9bf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44d9bf7, 76 },
    { 0, 0 },
    { 0x675d9bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x756d9bfc, 91 },
    { 0xe07d9bfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6d9c00, 85 },
    { 0xd62d9c00, 95 },
    { 0, 0 },
    { 0x58dd9c03, 90 },
    { 0xff5d9c04, 23 },
    { 0x80dd9c04, 95 },
    { 0, 0 },
    { 0x6bed9c07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655d9c0e, 98 },
    { 0, 0 },
    { 0xa32d9c10, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823d9c15, 90 },
    { 0xbfed9c16, 78 },
    { 0x5b9d9c15, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fd9c1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x388d9c1d, 99 },
    { 0, 0 },
    { 0x867d9c1f, 23 },
    { 0xd7fd9c20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2d9c25, 23 },
    { 0xe8ad9c25, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03d9c29, 55 },
    { 0x218d9c2a, 33 },
    { 0x57ad9c29, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2d9c2f, 80 },
    { 0x347d9c2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477d9c35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38d9c3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735d9c46, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x257d9c4a, 98 },
    { 0, 0 },
    { 0x232d9c4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bd9c52, 87 },
    { 0x2c4d9c53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cd9c5b, 98 },
    { 0xa5d9c5c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7d9c5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fd9c62, 88 },
    { 0x1f4d9c62, 90 },
    { 0xc2fd9c64, 66 },
    { 0x3c4d9c65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80d9c6b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfd9c6f, 77 },
    { 0, 0 },
    { 0xc8bd9c71, 52 },
    { 0xe39d9c72, 33 },
    { 0xe59d9c73, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x917d9c76, 53 },
    { 0x1f3d9c77, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x879d9c7a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x929d9c7d, 90 },
    { 0x545d9c7e, 91 },
    { 0xa1ad9c7f, 76 },
    { 0, 0 },
    { 0x98ed9c81, 95 },
    { 0x9c5d9c82, 88 },
    { 0x60dd9c81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x896d9c86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4d9c89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7d9c8e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3d9c91, 61 },
    { 0, 0 },
    { 0x26dd9c93, 95 },
    { 0, 0 },
    { 0xdd5d9c95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181d9c99, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x656d9c9c, 78 },
    { 0xa65d9c9d, 33 },
    { 0x1c7d9c9e, 84 },
    { 0x290d9c9c, 96 },
    { 0x52ed9ca0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ded9ca4, 43 },
    { 0xb94d9ca4, 76 },
    { 0, 0 },
    { 0x1fd9ca7, 34 },
    { 0x2ebd9ca8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23d9cb0, 90 },
    { 0x8f3d9cb1, 67 },
    { 0x9ecd9cb2, 33 },
    { 0xd81d9cb1, 98 },
    { 0xcc1d9cb4, 33 },
    { 0x703d9cb5, 33 },
    { 0x55dd9cb6, 36 },
    { 0x79dd9cb6, 90 },
    { 0x616d9cb4, 23 },
    { 0xb34d9cb4, 90 },
    { 0xfc2d9cb0, 95 },
    { 0, 0 },
    { 0xeb3d9cbc, 33 },
    { 0x53bd9cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71d9cc0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3d9cc4, 23 },
    { 0x16cd9cc4, 90 },
    { 0x81dd9cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82d9cca, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bd9cd4, 90 },
    { 0xba8d9cd5, 99 },
    { 0x657d9cd4, 46 },
    { 0x2f6d9cd5, 55 },
    { 0, 0 },
    { 0x14ed9cd9, 90 },
    { 0x478d9cd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2d9cdd, 33 },
    { 0xa87d9cde, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15d9ce2, 23 },
    { 0xcdfd9ce2, 90 },
    { 0x5b6d9ce4, 82 },
    { 0x72bd9ce5, 76 },
    { 0x220d9ce6, 93 },
    { 0, 0 },
    { 0xb47d9ce8, 34 },
    { 0xf74d9ce9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6d9cec, 98 },
    { 0xaf3d9ced, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12d9cf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ad9cf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8d9d02, 23 },
    { 0x303d9d03, 34 },
    { 0x708d9d03, 98 },
    { 0, 0 },
    { 0x92cd9d06, 61 },
    { 0xe52d9d06, 93 },
    { 0xd2ad9d08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0d9d0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bd9d12, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3d9d21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccd9d25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdd9d2b, 95 },
    { 0xd9fd9d2c, 95 },
    { 0x597d9d2b, 23 },
    { 0xf95d9d2d, 55 },
    { 0xdacd9d2c, 90 },
    { 0x25fd9d30, 33 },
    { 0x79ad9d2e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4d9d39, 98 },
    { 0x4abd9d3a, 36 },
    { 0x7c5d9d3a, 46 },
    { 0, 0 },
    { 0xf88d9d3d, 76 },
    { 0x14d9d3e, 33 },
    { 0, 0 },
    { 0x694d9d40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cd9d46, 55 },
    { 0xdeed9d46, 76 },
    { 0xbaad9d47, 90 },
    { 0x7f7d9d49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34d9d4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98fd9d52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ed9d58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbd9d65, 33 },
    { 0xc09d9d66, 23 },
    { 0, 0 },
    { 0x240d9d68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bed9d6c, 23 },
    { 0xfcd9d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9d9d70, 46 },
    { 0x75bd9d71, 90 },
    { 0x35fd9d72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85d9d75, 33 },
    { 0, 0 },
    { 0x9e9d9d77, 55 },
    { 0x81ad9d78, 33 },
    { 0x736d9d79, 95 },
    { 0x2d9d9d7a, 33 },
    { 0, 0 },
    { 0x46ed9d7c, 34 },
    { 0, 0 },
    { 0xf15d9d7e, 76 },
    { 0xec9d9d7f, 90 },
    { 0x66ed9d80, 88 },
    { 0x83fd9d81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fd9d8b, 61 },
    { 0x222d9d8c, 66 },
    { 0x63fd9d8d, 90 },
    { 0, 0 },
    { 0x7b2d9d8f, 76 },
    { 0x86fd9d90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ad9d94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43d9d97, 43 },
    { 0, 0 },
    { 0x96ed9d99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x176d9d9d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232d9da7, 95 },
    { 0x110d9da8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fd9dad, 23 },
    { 0xc8dd9dae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35d9db2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3d9dbb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469d9dc0, 46 },
    { 0x951d9dc0, 100 },
    { 0x85fd9dc1, 76 },
    { 0xb11d9dc3, 95 },
    { 0x377d9dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cd9dc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992d9dcd, 85 },
    { 0, 0 },
    { 0x82fd9dcf, 33 },
    { 0x801d9dcf, 90 },
    { 0x1b3d9dd0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bd9dd4, 34 },
    { 0, 0 },
    { 0x82cd9dd6, 93 },
    { 0, 0 },
    { 0x7e7d9dd8, 33 },
    { 0, 0 },
    { 0xf73d9dda, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5d9ddf, 33 },
    { 0x670d9de0, 34 },
    { 0x767d9de0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155d9de7, 33 },
    { 0, 0 },
    { 0x9eed9de9, 90 },
    { 0xa6ad9dea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfd9df3, 65 },
    { 0x8c9d9df4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2d9dfb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49d9e00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884d9e05, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73d9e0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5d9e15, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dd9e1c, 61 },
    { 0, 0 },
    { 0x6aad9e1e, 33 },
    { 0x445d9e1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aed9e23, 33 },
    { 0xb1ad9e24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bd9e27, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0d9e2d, 85 },
    { 0xdf4d9e2e, 90 },
    { 0x5b2d9e2d, 76 },
    { 0x7d7d9e30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbd9e37, 61 },
    { 0x4f3d9e37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdd9e3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0d9e40, 43 },
    { 0x6b3d9e41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9d9e46, 75 },
    { 0xcadd9e47, 91 },
    { 0, 0 },
    { 0x3b5d9e49, 33 },
    { 0, 0 },
    { 0xc8fd9e4b, 23 },
    { 0xd86d9e4c, 61 },
    { 0xbc2d9e4d, 33 },
    { 0xe8cd9e4e, 55 },
    { 0x42dd9e4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbced9e54, 95 },
    { 0x1bdd9e55, 33 },
    { 0x440d9e56, 91 },
    { 0, 0 },
    { 0xe01d9e58, 34 },
    { 0xe95d9e59, 66 },
    { 0xc15d9e5a, 78 },
    { 0x9cad9e5a, 98 },
    { 0x3b7d9e5b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbd9e5f, 23 },
    { 0xcb3d9e60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bd9e66, 61 },
    { 0, 0 },
    { 0xf1dd9e68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edd9e6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128d9e6f, 83 },
    { 0x15ad9e70, 95 },
    { 0, 0 },
    { 0x52dd9e72, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cd9e75, 61 },
    { 0x651d9e76, 33 },
    { 0, 0 },
    { 0xb28d9e78, 23 },
    { 0xb18d9e78, 95 },
    { 0x186d9e78, 98 },
    { 0x67cd9e7b, 100 },
    { 0x537d9e79, 87 },
    { 0, 0 },
    { 0x3dad9e7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3d9e81, 85 },
    { 0x891d9e81, 84 },
    { 0xd61d9e83, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dd9e87, 43 },
    { 0x62ed9e87, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bd9e8f, 95 },
    { 0xbf5d9e90, 99 },
    { 0, 0 },
    { 0xf50d9e92, 61 },
    { 0xb1ed9e93, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4d9e96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241d9ea0, 43 },
    { 0x8c4d9ea1, 52 },
    { 0xffad9ea2, 90 },
    { 0, 0 },
    { 0x25d9ea4, 33 },
    { 0x96d9ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x190d9ea8, 78 },
    { 0x1bfd9ea9, 97 },
    { 0xded9eaa, 90 },
    { 0x6dad9eaa, 90 },
    { 0, 0 },
    { 0x96fd9ead, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecd9eb1, 66 },
    { 0xfa8d9eb2, 46 },
    { 0x1fcd9eb2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xded9eb6, 34 },
    { 0, 0 },
    { 0x4b8d9eb8, 17 },
    { 0, 0 },
    { 0x25dd9eba, 33 },
    { 0xbc2d9ebb, 90 },
    { 0xa4cd9ebb, 23 },
    { 0xa41d9ebd, 81 },
    { 0xe2d9ebe, 33 },
    { 0x736d9ebf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677d9ec5, 43 },
    { 0, 0 },
    { 0x6abd9ec7, 90 },
    { 0x4c8d9ec7, 65 },
    { 0x6cd9ec9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1d9ecd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35d9ed1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x384d9ed4, 17 },
    { 0, 0 },
    { 0xdb5d9ed6, 33 },
    { 0x7e1d9ed7, 33 },
    { 0, 0 },
    { 0x1b8d9ed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebd9edf, 55 },
    { 0, 0 },
    { 0x74cd9ee1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cd9ee5, 69 },
    { 0, 0 },
    { 0xd4ad9ee7, 65 },
    { 0xa75d9ee8, 90 },
    { 0xdd4d9ee9, 76 },
    { 0x445d9ee9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9d9ef0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cd9ef3, 100 },
    { 0, 0 },
    { 0xdced9ef5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06d9ef8, 95 },
    { 0, 0 },
    { 0x9dbd9efa, 90 },
    { 0xd8bd9efb, 90 },
    { 0x12cd9efb, 34 },
    { 0xe2ed9efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ad9f01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fd9f07, 55 },
    { 0, 0 },
    { 0x3b0d9f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcead9f14, 36 },
    { 0, 0 },
    { 0x10ed9f16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8d9f19, 95 },
    { 0, 0 },
    { 0xed9d9f1b, 61 },
    { 0xd3fd9f1c, 95 },
    { 0x6e9d9f1d, 99 },
    { 0xaced9f1e, 95 },
    { 0x700d9f1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6d9f24, 95 },
    { 0x217d9f25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbd9f28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0d9f2f, 23 },
    { 0x440d9f2f, 66 },
    { 0x3c8d9f30, 23 },
    { 0xae8d9f30, 55 },
    { 0x8bcd9f33, 78 },
    { 0x47cd9f33, 34 },
    { 0, 0 },
    { 0xbe3d9f36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9d9f3c, 23 },
    { 0, 0 },
    { 0x9a2d9f3e, 91 },
    { 0xf00d9f3f, 98 },
    { 0x4c6d9f40, 90 },
    { 0x89bd9f40, 23 },
    { 0x826d9f40, 23 },
    { 0x347d9f43, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fd9f47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1d9f4c, 98 },
    { 0x385d9f4c, 23 },
    { 0xab9d9f4e, 61 },
    { 0xee6d9f4f, 23 },
    { 0x402d9f50, 33 },
    { 0, 0 },
    { 0x9f3d9f52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36d9f55, 34 },
    { 0x304d9f56, 85 },
    { 0x1bfd9f55, 61 },
    { 0xf34d9f57, 43 },
    { 0x1a9d9f59, 33 },
    { 0xdedd9f59, 87 },
    { 0x208d9f55, 98 },
    { 0x928d9f5c, 43 },
    { 0x6d6d9f5d, 33 },
    { 0x73dd9f5e, 76 },
    { 0x412d9f5e, 95 },
    { 0x20cd9f5f, 46 },
    { 0xb50d9f61, 23 },
    { 0xaded9f62, 90 },
    { 0xf7fd9f63, 66 },
    { 0x7a8d9f62, 52 },
    { 0x36cd9f5e, 23 },
    { 0x6aad9f66, 77 },
    { 0x702d9f65, 23 },
    { 0x434d9f60, 98 },
    { 0xb52d9f63, 95 },
    { 0xfc4d9f6a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164d9f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825d9f75, 90 },
    { 0, 0 },
    { 0xcf7d9f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762d9f80, 90 },
    { 0, 0 },
    { 0x13dd9f82, 98 },
    { 0xd0cd9f83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x546d9f86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26d9f8c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x809d9f8f, 90 },
    { 0xd8ed9f90, 98 },
    { 0x48fd9f8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efd9f94, 95 },
    { 0, 0 },
    { 0x2ccd9f96, 61 },
    { 0, 0 },
    { 0xf48d9f98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71d9f9c, 90 },
    { 0x6cdd9f9c, 99 },
    { 0xaf1d9f9e, 65 },
    { 0xe23d9f9f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824d9fa3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0d9fa6, 76 },
    { 0xa95d9fa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564d9fab, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ead9fb1, 76 },
    { 0, 0 },
    { 0x64fd9fb3, 33 },
    { 0x8c7d9fb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dd9fb8, 33 },
    { 0x6b7d9fb9, 95 },
    { 0x173d9fba, 66 },
    { 0x937d9fbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7d9fc5, 76 },
    { 0, 0 },
    { 0x274d9fc7, 98 },
    { 0, 0 },
    { 0x545d9fc9, 98 },
    { 0x113d9fca, 95 },
    { 0, 0 },
    { 0x2cdd9fcc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ad9fd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfd9fd3, 98 },
    { 0x1b9d9fd4, 33 },
    { 0x3ecd9fd5, 77 },
    { 0xe3dd9fd5, 34 },
    { 0xbe0d9fd7, 76 },
    { 0xe11d9fd6, 55 },
    { 0xc6dd9fd6, 90 },
    { 0x524d9fd3, 78 },
    { 0x76fd9fdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4d9fde, 90 },
    { 0xbebd9fdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5d9fe3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6d9fe7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4d9fec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x698d9fef, 65 },
    { 0, 0 },
    { 0xe95d9ff1, 77 },
    { 0, 0 },
    { 0xb6fd9ff3, 76 },
    { 0x78ad9ff4, 90 },
    { 0xb84d9ff5, 34 },
    { 0xda0d9ff6, 90 },
    { 0xa83d9ff7, 76 },
    { 0xafad9ff8, 55 },
    { 0x30bd9ff9, 34 },
    { 0x278d9ff4, 23 },
    { 0, 0 },
    { 0x5ebd9ffc, 61 },
    { 0, 0 },
    { 0xded9ffe, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877da007, 33 },
    { 0, 0 },
    { 0xdc5da009, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628da00e, 33 },
    { 0, 0 },
    { 0xdc6da010, 99 },
    { 0, 0 },
    { 0xd86da012, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fda015, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251da01c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cda026, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73da029, 46 },
    { 0xd09da02a, 61 },
    { 0x9bbda02b, 90 },
    { 0xad9da02a, 98 },
    { 0x277da02b, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583da031, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aeda037, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26da03a, 33 },
    { 0, 0 },
    { 0xb37da03c, 95 },
    { 0x492da03c, 61 },
    { 0xe4da03e, 43 },
    { 0, 0 },
    { 0x6d5da040, 90 },
    { 0xd43da041, 23 },
    { 0x12cda040, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cda047, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcda04c, 33 },
    { 0x12da04d, 95 },
    { 0xdbeda04e, 95 },
    { 0xe48da04f, 91 },
    { 0x52dda04d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcda057, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9da05a, 33 },
    { 0, 0 },
    { 0x49fda05c, 46 },
    { 0xe82da05c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1feda060, 55 },
    { 0, 0 },
    { 0xeb6da062, 90 },
    { 0, 0 },
    { 0x9c8da064, 52 },
    { 0xdadda065, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38da06a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eda06d, 87 },
    { 0x18fda06e, 90 },
    { 0xce5da06e, 98 },
    { 0, 0 },
    { 0x4f8da071, 46 },
    { 0xef0da071, 83 },
    { 0x61eda073, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aada078, 95 },
    { 0, 0 },
    { 0x21bda07a, 23 },
    { 0x7c9da07b, 61 },
    { 0x439da07c, 33 },
    { 0, 0 },
    { 0x43ada07e, 97 },
    { 0, 0 },
    { 0x3c9da080, 33 },
    { 0x3f9da081, 95 },
    { 0xb33da080, 61 },
    { 0, 0 },
    { 0xfc6da084, 33 },
    { 0, 0 },
    { 0xf91da086, 61 },
    { 0, 0 },
    { 0xe31da088, 33 },
    { 0x62bda089, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cda08c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fda091, 91 },
    { 0, 0 },
    { 0x47fda093, 76 },
    { 0x4a3da093, 98 },
    { 0x43bda095, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cda09c, 90 },
    { 0, 0 },
    { 0x88fda09e, 76 },
    { 0, 0 },
    { 0x2c5da0a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8da0a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907da0a9, 40 },
    { 0x822da0aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8da0b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2da0b4, 53 },
    { 0, 0 },
    { 0x123da0b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65da0b9, 90 },
    { 0xf4fda0ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eda0c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aeda0c8, 52 },
    { 0, 0 },
    { 0x473da0ca, 65 },
    { 0, 0 },
    { 0x7ddda0cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4da0d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2da0d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdada0da, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x783da0dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbda0e1, 90 },
    { 0xd6fda0e2, 33 },
    { 0, 0 },
    { 0x12eda0e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccda0e7, 90 },
    { 0, 0 },
    { 0x6c3da0e9, 98 },
    { 0x8beda0ea, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449da0f0, 23 },
    { 0x47da0f1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x491da0f4, 100 },
    { 0xb26da0f5, 98 },
    { 0, 0 },
    { 0x115da0f7, 23 },
    { 0xb6bda0f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39da0fe, 96 },
    { 0x91ada0ff, 33 },
    { 0xf45da100, 33 },
    { 0x514da101, 68 },
    { 0x369da102, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x608da105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268da109, 65 },
    { 0xd1fda109, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfda10d, 33 },
    { 0x824da10e, 98 },
    { 0, 0 },
    { 0x8a3da110, 62 },
    { 0, 0 },
    { 0x9b3da112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fda117, 98 },
    { 0x8a6da118, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabda11e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64da125, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cada128, 95 },
    { 0, 0 },
    { 0xd31da12a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960da131, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bda138, 43 },
    { 0x64ada138, 61 },
    { 0x38da13a, 23 },
    { 0xeeada13b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbeda144, 90 },
    { 0xa51da145, 23 },
    { 0, 0 },
    { 0x3cfda147, 20 },
    { 0xbb4da148, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136da14c, 93 },
    { 0x9d3da14d, 52 },
    { 0x74da14e, 34 },
    { 0x847da14f, 85 },
    { 0x666da150, 78 },
    { 0x17fda14f, 61 },
    { 0xd02da14f, 40 },
    { 0x10dda14d, 17 },
    { 0xcedda154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fda158, 66 },
    { 0xaf9da159, 98 },
    { 0, 0 },
    { 0x641da15b, 61 },
    { 0, 0 },
    { 0x3f7da15d, 66 },
    { 0, 0 },
    { 0xf89da15f, 98 },
    { 0xb6eda160, 66 },
    { 0xb0fda161, 33 },
    { 0, 0 },
    { 0x75ada163, 33 },
    { 0x17dda163, 23 },
    { 0x8f4da163, 55 },
    { 0xe5da166, 90 },
    { 0x31bda167, 23 },
    { 0x4aeda165, 55 },
    { 0xb2bda169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43da16c, 34 },
    { 0x172da16d, 61 },
    { 0x566da16e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ada171, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feda178, 93 },
    { 0x4c0da178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6da17c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bda17f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf2da182, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987da186, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cda18f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223da198, 98 },
    { 0, 0 },
    { 0xa5cda19a, 90 },
    { 0, 0 },
    { 0xb3fda19c, 23 },
    { 0x9dfda19c, 76 },
    { 0, 0 },
    { 0xef8da19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda1a4, 55 },
    { 0xb85da1a5, 23 },
    { 0, 0 },
    { 0x119da1a7, 68 },
    { 0xba3da1a7, 43 },
    { 0xff0da1a9, 23 },
    { 0xab0da1aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68da1ae, 95 },
    { 0x3acda1af, 95 },
    { 0, 0 },
    { 0x70fda1b1, 76 },
    { 0, 0 },
    { 0xc39da1b3, 23 },
    { 0x77fda1b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbda1b9, 33 },
    { 0x43fda1ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eda1bf, 23 },
    { 0x523da1c0, 33 },
    { 0xa86da1bf, 61 },
    { 0, 0 },
    { 0x390da1c3, 91 },
    { 0xcaada1c3, 52 },
    { 0xdceda1c3, 90 },
    { 0x4bda1c5, 90 },
    { 0xf28da1c7, 33 },
    { 0, 0 },
    { 0x38cda1c9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341da1cd, 61 },
    { 0, 0 },
    { 0xf3ada1cf, 91 },
    { 0x428da1cf, 98 },
    { 0x438da1d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44eda1d5, 93 },
    { 0xfddda1d6, 52 },
    { 0, 0 },
    { 0x62eda1d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94da1df, 88 },
    { 0xdf8da1e0, 33 },
    { 0x375da1e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42da1e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08da1e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eda1f3, 98 },
    { 0, 0 },
    { 0xf95da1f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdda1f9, 90 },
    { 0x2b2da1f9, 31 },
    { 0x6e7da1fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb6da1fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990da202, 33 },
    { 0x7f6da202, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beda207, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7da211, 88 },
    { 0, 0 },
    { 0xc70da213, 66 },
    { 0x677da213, 61 },
    { 0xb31da213, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201da21b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dda220, 53 },
    { 0xda4da220, 23 },
    { 0x928da222, 61 },
    { 0x1b7da223, 90 },
    { 0, 0 },
    { 0xb7bda225, 95 },
    { 0x36da226, 52 },
    { 0xf1cda226, 95 },
    { 0x83da228, 64 },
    { 0, 0 },
    { 0x11bda22a, 97 },
    { 0x60cda22a, 61 },
    { 0x4dcda22b, 95 },
    { 0x759da22d, 101 },
    { 0x813da22e, 76 },
    { 0x20da22f, 76 },
    { 0, 0 },
    { 0x2dfda231, 65 },
    { 0x545da232, 98 },
    { 0x93bda231, 93 },
    { 0x9feda234, 61 },
    { 0x575da235, 98 },
    { 0xbcdda236, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9da239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45da23c, 67 },
    { 0x348da23d, 76 },
    { 0xcfada23d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16da241, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4bda244, 90 },
    { 0x89fda244, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ada249, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bda24c, 91 },
    { 0x2edda24c, 95 },
    { 0, 0 },
    { 0xaadda24f, 46 },
    { 0xa6eda250, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fda254, 99 },
    { 0, 0 },
    { 0xd8eda256, 52 },
    { 0, 0 },
    { 0xdfdda258, 33 },
    { 0x77ada258, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447da25d, 33 },
    { 0, 0 },
    { 0x9eeda25f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4da265, 43 },
    { 0xa80da266, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddda26a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6da26e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6da276, 98 },
    { 0, 0 },
    { 0xfb5da278, 61 },
    { 0x342da279, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afda27c, 61 },
    { 0xf70da27d, 76 },
    { 0x64bda27e, 90 },
    { 0x27da27f, 61 },
    { 0x4e4da27f, 95 },
    { 0xb7eda280, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7da284, 98 },
    { 0xd0dda285, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137da289, 23 },
    { 0, 0 },
    { 0xf81da28b, 68 },
    { 0, 0 },
    { 0x94eda28d, 90 },
    { 0x80fda28e, 61 },
    { 0x90eda28f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeda293, 55 },
    { 0xba3da293, 23 },
    { 0x591da295, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dda299, 23 },
    { 0, 0 },
    { 0x681da29b, 61 },
    { 0xe6da29c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4da2a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6da2a4, 90 },
    { 0, 0 },
    { 0x25bda2a6, 100 },
    { 0xcacda2a7, 33 },
    { 0xb02da2a7, 76 },
    { 0x81fda2a9, 90 },
    { 0, 0 },
    { 0xdecda2ab, 90 },
    { 0x3c8da2ac, 95 },
    { 0x9c5da2ab, 43 },
    { 0xe76da2ae, 95 },
    { 0xbcdda2af, 55 },
    { 0xe4bda2ad, 90 },
    { 0xaecda2b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963da2b9, 34 },
    { 0x2aada2ba, 23 },
    { 0xa8bda2bb, 33 },
    { 0x1dfda2bb, 33 },
    { 0x474da2bd, 90 },
    { 0xab2da2be, 23 },
    { 0x19bda2bd, 33 },
    { 0, 0 },
    { 0xd8cda2c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4da2c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xffda2ca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5da2cd, 95 },
    { 0x810da2ce, 61 },
    { 0x37dda2cf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcda2d5, 93 },
    { 0x21bda2d6, 23 },
    { 0xf6da2d5, 43 },
    { 0, 0 },
    { 0x3afda2d9, 76 },
    { 0xaf4da2d9, 77 },
    { 0xdc6da2d9, 33 },
    { 0x5edda2db, 26 },
    { 0, 0 },
    { 0x6a6da2de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeda2e4, 46 },
    { 0x88eda2e5, 76 },
    { 0, 0 },
    { 0xb7cda2e7, 55 },
    { 0xe6cda2e8, 98 },
    { 0, 0 },
    { 0xcf2da2ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887da2ef, 55 },
    { 0x920da2ef, 53 },
    { 0x86fda2f1, 90 },
    { 0xf5eda2f2, 35 },
    { 0x5e6da2f0, 66 },
    { 0, 0 },
    { 0x3dbda2f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8da2fa, 90 },
    { 0x2b1da2fa, 95 },
    { 0x817da2fa, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6da302, 61 },
    { 0, 0 },
    { 0x84eda304, 52 },
    { 0xbf6da305, 23 },
    { 0x71fda304, 77 },
    { 0x835da306, 33 },
    { 0xa9dda308, 23 },
    { 0x2fbda308, 43 },
    { 0x753da307, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x53eda30d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0da311, 76 },
    { 0, 0 },
    { 0x1b0da313, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fda316, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234da31c, 90 },
    { 0xf02da31c, 66 },
    { 0x259da31e, 61 },
    { 0x5c7da31c, 90 },
    { 0x6f2da320, 68 },
    { 0x6cda320, 61 },
    { 0xf89da322, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddda328, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8da331, 85 },
    { 0, 0 },
    { 0x34ada333, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798da337, 99 },
    { 0x2a0da337, 95 },
    { 0x5c7da337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7da33c, 33 },
    { 0x5d8da33d, 55 },
    { 0x349da33d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3da341, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecda34a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9deda34d, 61 },
    { 0x221da34d, 98 },
    { 0xab7da34f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5da358, 93 },
    { 0xededa359, 91 },
    { 0x1dda35a, 76 },
    { 0, 0 },
    { 0x9e3da35c, 34 },
    { 0x60ada35d, 96 },
    { 0, 0 },
    { 0x49dda35f, 90 },
    { 0, 0 },
    { 0x211da361, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dda36d, 23 },
    { 0xc99da36e, 34 },
    { 0x83bda36e, 95 },
    { 0x76bda370, 85 },
    { 0xafda370, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379da37c, 68 },
    { 0xeb8da37d, 67 },
    { 0xbfbda37e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39da382, 56 },
    { 0xa40da383, 90 },
    { 0, 0 },
    { 0x959da385, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60dda38a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5da38d, 55 },
    { 0xf91da38d, 76 },
    { 0x7abda38f, 33 },
    { 0x738da390, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459da394, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbeda3a1, 33 },
    { 0, 0 },
    { 0x6eada3a3, 33 },
    { 0xa96da3a4, 76 },
    { 0xf53da3a4, 33 },
    { 0, 0 },
    { 0xd81da3a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4da3ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807da3b3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575da3b7, 66 },
    { 0x1cada3b8, 33 },
    { 0x5f3da3b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ada3c5, 90 },
    { 0, 0 },
    { 0x1cada3c7, 61 },
    { 0, 0 },
    { 0xbccda3c9, 90 },
    { 0x7d3da3ca, 61 },
    { 0xafeda3cb, 33 },
    { 0xdc5da3cc, 90 },
    { 0, 0 },
    { 0x458da3ce, 17 },
    { 0x398da3ce, 98 },
    { 0xfcdda3d0, 46 },
    { 0x2bfda3cf, 23 },
    { 0, 0 },
    { 0x426da3d3, 69 },
    { 0, 0 },
    { 0x34fda3d5, 95 },
    { 0x803da3d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fda3dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1da3e2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x904da3e5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cada3ec, 61 },
    { 0xadcda3ed, 33 },
    { 0xbb5da3ed, 98 },
    { 0xd98da3ef, 55 },
    { 0xa59da3ed, 91 },
    { 0xff4da3f1, 55 },
    { 0x60dda3f1, 17 },
    { 0x7d8da3f3, 90 },
    { 0xe21da3f3, 93 },
    { 0, 0 },
    { 0x13eda3f6, 61 },
    { 0x64da3f7, 76 },
    { 0x4e2da3f8, 46 },
    { 0, 0 },
    { 0xbfcda3fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x581da3fd, 78 },
    { 0xbf1da3fe, 87 },
    { 0x144da3ff, 33 },
    { 0xc5eda3fe, 90 },
    { 0xeabda401, 61 },
    { 0xf62da402, 23 },
    { 0, 0 },
    { 0x595da404, 95 },
    { 0xd33da404, 23 },
    { 0xc22da406, 33 },
    { 0xf5fda404, 75 },
    { 0, 0 },
    { 0x84ada409, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50da40c, 43 },
    { 0xcd6da40d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cda410, 61 },
    { 0xf4ada411, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693da416, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfda41e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cda421, 77 },
    { 0x9cada421, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2da425, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99da42b, 33 },
    { 0xf8eda42c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bda431, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeda436, 98 },
    { 0, 0 },
    { 0xcc7da438, 46 },
    { 0, 0 },
    { 0xc4dda43a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4da43d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x280da440, 43 },
    { 0x6beda441, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43dda447, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733da44b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bda450, 23 },
    { 0x235da450, 52 },
    { 0, 0 },
    { 0xdc0da453, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x157da456, 61 },
    { 0x52cda457, 46 },
    { 0x109da457, 34 },
    { 0xc7ada459, 65 },
    { 0x71fda45a, 85 },
    { 0xdf3da45a, 33 },
    { 0x1ada45c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ada461, 33 },
    { 0x2c8da461, 90 },
    { 0xe6bda461, 76 },
    { 0x4f3da464, 61 },
    { 0xcf4da463, 75 },
    { 0x1bdda462, 61 },
    { 0, 0 },
    { 0x70dda468, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x846da46b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72da46e, 98 },
    { 0x725da46f, 61 },
    { 0xae9da470, 23 },
    { 0x4b3da46e, 64 },
    { 0x13cda472, 95 },
    { 0x844da473, 52 },
    { 0, 0 },
    { 0xa1da475, 90 },
    { 0, 0 },
    { 0xfe3da477, 61 },
    { 0xe33da478, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7da47b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4da47f, 33 },
    { 0, 0 },
    { 0xd46da481, 28 },
    { 0, 0 },
    { 0x96eda483, 87 },
    { 0xa45da484, 95 },
    { 0xea6da485, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffda48b, 23 },
    { 0x1afda48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3da48f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230da498, 99 },
    { 0, 0 },
    { 0x7f2da49a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x666da49d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fda4a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x533da4a6, 55 },
    { 0, 0 },
    { 0xd4cda4a8, 98 },
    { 0xaddda4a9, 78 },
    { 0xa28da4a8, 65 },
    { 0x355da4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5da4b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820da4b6, 95 },
    { 0x510da4b7, 93 },
    { 0xe90da4b8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9da4bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80da4c3, 90 },
    { 0, 0 },
    { 0x222da4c5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3dda4c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82da4cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627da4d0, 46 },
    { 0, 0 },
    { 0xa4dda4d2, 98 },
    { 0xb8dda4d3, 87 },
    { 0xd22da4d4, 76 },
    { 0x31fda4d5, 75 },
    { 0xefda4d2, 78 },
    { 0, 0 },
    { 0xf34da4d8, 23 },
    { 0x90fda4d9, 88 },
    { 0, 0 },
    { 0xdc7da4db, 76 },
    { 0x30fda4dc, 99 },
    { 0, 0 },
    { 0xeedda4de, 34 },
    { 0xccdda4df, 99 },
    { 0x5d8da4e0, 90 },
    { 0, 0 },
    { 0x8fda4e2, 32 },
    { 0xdb5da4e3, 61 },
    { 0x7b8da4e4, 89 },
    { 0xd52da4e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4da4eb, 61 },
    { 0xf0ada4eb, 33 },
    { 0, 0 },
    { 0x342da4ee, 95 },
    { 0x9eda4ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8da4fa, 33 },
    { 0xbf0da4fb, 90 },
    { 0x4b7da4fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83da4ff, 33 },
    { 0, 0 },
    { 0xb37da501, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfda504, 43 },
    { 0x7edda504, 98 },
    { 0xb06da505, 43 },
    { 0, 0 },
    { 0x466da508, 38 },
    { 0x640da509, 55 },
    { 0xfb8da508, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17da50f, 33 },
    { 0x569da510, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218da517, 95 },
    { 0xd8dda518, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866da51f, 93 },
    { 0, 0 },
    { 0xddbda521, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3da528, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eda52c, 90 },
    { 0, 0 },
    { 0x2f2da52e, 52 },
    { 0xb8bda52f, 88 },
    { 0xe09da52e, 46 },
    { 0x739da52f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x143da534, 61 },
    { 0x807da535, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eda53b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6da53e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879da542, 77 },
    { 0, 0 },
    { 0xa9cda544, 67 },
    { 0xce3da545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896da549, 66 },
    { 0x91eda54a, 34 },
    { 0x800da54b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5eda554, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213da558, 90 },
    { 0x506da558, 95 },
    { 0, 0 },
    { 0x2b1da55b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6da561, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fda567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eda56b, 85 },
    { 0x7d4da56b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6da570, 87 },
    { 0xcaada571, 76 },
    { 0x187da572, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2da575, 95 },
    { 0xf24da576, 100 },
    { 0xaf3da577, 95 },
    { 0xf64da576, 98 },
    { 0x9d8da576, 99 },
    { 0, 0 },
    { 0xff7da57b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3da57e, 90 },
    { 0x78eda57f, 95 },
    { 0xcbcda580, 55 },
    { 0x89cda581, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfada586, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38da591, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280da595, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x981da598, 61 },
    { 0x91cda599, 87 },
    { 0x87eda59a, 33 },
    { 0, 0 },
    { 0xa81da59c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461da5a1, 78 },
    { 0x5abda5a2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1da5ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3da5b2, 98 },
    { 0x746da5b2, 55 },
    { 0xcb5da5b4, 66 },
    { 0xddcda5b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268da5b9, 90 },
    { 0x32eda5ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68da5c2, 55 },
    { 0, 0 },
    { 0x762da5c4, 96 },
    { 0xa5cda5c4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9da5d2, 100 },
    { 0x2c6da5d2, 34 },
    { 0x997da5d4, 52 },
    { 0xb76da5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0da5db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7da5e0, 33 },
    { 0x5a5da5e1, 77 },
    { 0xfe8da5e1, 23 },
    { 0x182da5e0, 98 },
    { 0x2fda5e1, 93 },
    { 0xfe6da5e4, 33 },
    { 0xba2da5e6, 61 },
    { 0xe32da5e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ada5ef, 43 },
    { 0x542da5f0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71da5f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582da5fb, 84 },
    { 0x6aada5fc, 76 },
    { 0x392da5fc, 90 },
    { 0xefbda5fe, 55 },
    { 0, 0 },
    { 0x5e4da600, 93 },
    { 0, 0 },
    { 0x2c1da602, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0da605, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4da60a, 53 },
    { 0, 0 },
    { 0x309da60c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x716da60f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fda616, 85 },
    { 0xa57da617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55eda61b, 76 },
    { 0x444da61b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x936da61f, 34 },
    { 0x204da620, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eda628, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4da62e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ada634, 77 },
    { 0xd31da635, 82 },
    { 0, 0 },
    { 0x194da637, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6da63c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fda640, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58da644, 78 },
    { 0x4b7da644, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72da648, 46 },
    { 0x64da649, 53 },
    { 0x8fcda648, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bada64e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812da653, 66 },
    { 0xe61da653, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9da65b, 55 },
    { 0, 0 },
    { 0xbbda65d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3da661, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05da665, 76 },
    { 0, 0 },
    { 0xe07da667, 61 },
    { 0xa69da668, 88 },
    { 0xbd7da669, 87 },
    { 0x362da668, 95 },
    { 0x743da66b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffada675, 98 },
    { 0x37ada676, 93 },
    { 0x858da675, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3da67d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cda680, 55 },
    { 0, 0 },
    { 0x9b3da682, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785da687, 61 },
    { 0xc6dda688, 90 },
    { 0xb85da688, 43 },
    { 0, 0 },
    { 0xef9da68b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86da68f, 90 },
    { 0x881da68f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aada699, 91 },
    { 0xc60da69a, 76 },
    { 0x1d3da69a, 61 },
    { 0x176da69c, 33 },
    { 0x953da69d, 40 },
    { 0, 0 },
    { 0xe30da69f, 33 },
    { 0x90fda69f, 33 },
    { 0x99dda6a1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228da6a8, 95 },
    { 0x89da6a8, 95 },
    { 0, 0 },
    { 0x272da6ab, 66 },
    { 0x427da6ac, 84 },
    { 0x933da6ac, 88 },
    { 0x212da6ae, 61 },
    { 0xe8eda6af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fda6b2, 98 },
    { 0x426da6b3, 34 },
    { 0x1d3da6b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989da6be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ada6c1, 23 },
    { 0x963da6c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedda6c6, 90 },
    { 0x23bda6c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ada6cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8da6cf, 90 },
    { 0, 0 },
    { 0xc39da6d1, 64 },
    { 0x85bda6d1, 90 },
    { 0, 0 },
    { 0x7bada6d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fda6d9, 43 },
    { 0x7ceda6da, 87 },
    { 0x544da6db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3da6e1, 91 },
    { 0x7a2da6e2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69da6e5, 77 },
    { 0, 0 },
    { 0xd58da6e7, 87 },
    { 0x472da6e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ada6f2, 33 },
    { 0x519da6f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434da6f8, 34 },
    { 0x41ada6f9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2da6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69dda702, 23 },
    { 0x4a0da703, 34 },
    { 0x86ada702, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9da708, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787da70e, 55 },
    { 0xa1da70f, 52 },
    { 0x51bda710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfda713, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbda718, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200da71c, 46 },
    { 0xa8bda71d, 46 },
    { 0x8d1da71c, 84 },
    { 0xa0bda71c, 95 },
    { 0xa07da720, 23 },
    { 0xedda721, 98 },
    { 0x12eda722, 46 },
    { 0x950da723, 17 },
    { 0xa7eda721, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16da728, 77 },
    { 0, 0 },
    { 0x31cda72a, 46 },
    { 0x2c0da72b, 98 },
    { 0x5deda72b, 95 },
    { 0xce4da72c, 85 },
    { 0xae8da72e, 46 },
    { 0x635da72f, 33 },
    { 0x87eda72c, 95 },
    { 0, 0 },
    { 0x7cda732, 34 },
    { 0xd4eda733, 99 },
    { 0x889da734, 76 },
    { 0x685da734, 43 },
    { 0x6eada736, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169da73c, 90 },
    { 0, 0 },
    { 0x10cda73e, 99 },
    { 0x9d6da73e, 23 },
    { 0x9d5da740, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8da743, 90 },
    { 0x5b9da743, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105da749, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1da750, 61 },
    { 0xa7da751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2da754, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcda75b, 87 },
    { 0x9d2da75c, 95 },
    { 0, 0 },
    { 0xd0ada75e, 76 },
    { 0, 0 },
    { 0xee3da760, 84 },
    { 0, 0 },
    { 0xeb7da762, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cada765, 33 },
    { 0, 0 },
    { 0xafbda767, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69da76c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cda76f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60da773, 90 },
    { 0xad9da773, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cda777, 95 },
    { 0, 0 },
    { 0x736da779, 78 },
    { 0xe8dda77a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3da77e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x34da781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427da787, 81 },
    { 0xa82da787, 23 },
    { 0xadcda789, 65 },
    { 0, 0 },
    { 0xa8cda78b, 66 },
    { 0x683da78c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bda790, 23 },
    { 0x4fada790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5da794, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadda797, 23 },
    { 0xb42da797, 43 },
    { 0x45ada798, 95 },
    { 0x190da799, 34 },
    { 0, 0 },
    { 0x82bda79c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70eda7a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfda7aa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8da7ae, 100 },
    { 0xb8bda7af, 67 },
    { 0x4cda7af, 91 },
    { 0, 0 },
    { 0x958da7b2, 55 },
    { 0xcdfda7b2, 23 },
    { 0, 0 },
    { 0x7e7da7b5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bda7b9, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dda7be, 95 },
    { 0, 0 },
    { 0x2f7da7c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cda7c7, 84 },
    { 0x8beda7c8, 76 },
    { 0xd32da7c7, 61 },
    { 0, 0 },
    { 0xd19da7cb, 83 },
    { 0, 0 },
    { 0x722da7cd, 90 },
    { 0, 0 },
    { 0x159da7cf, 61 },
    { 0x733da7d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6da7d4, 95 },
    { 0x4c5da7d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cda7d8, 91 },
    { 0x99cda7d8, 61 },
    { 0, 0 },
    { 0xc65da7db, 93 },
    { 0x770da7dc, 9 },
    { 0x813da7dd, 43 },
    { 0, 0 },
    { 0x9c1da7df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cada7e3, 90 },
    { 0, 0 },
    { 0xbf9da7e5, 33 },
    { 0xfc8da7e6, 99 },
    { 0x50bda7e7, 76 },
    { 0x49ada7e6, 98 },
    { 0, 0 },
    { 0x483da7ea, 91 },
    { 0xbd5da7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50da7f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7da7fb, 33 },
    { 0, 0 },
    { 0x938da7fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fda800, 90 },
    { 0xa4dda801, 95 },
    { 0, 0 },
    { 0x610da803, 95 },
    { 0xffcda804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfda807, 90 },
    { 0x6bdda808, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90da80b, 67 },
    { 0x748da80c, 61 },
    { 0, 0 },
    { 0x4c1da80e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4da811, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fda815, 76 },
    { 0xa5eda816, 23 },
    { 0x250da815, 95 },
    { 0, 0 },
    { 0x684da819, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615da81f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298da824, 76 },
    { 0x909da825, 23 },
    { 0x8ebda825, 31 },
    { 0xe65da827, 95 },
    { 0xc8ada828, 61 },
    { 0x48da828, 98 },
    { 0xd0eda82a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fda82f, 33 },
    { 0x27bda82f, 101 },
    { 0, 0 },
    { 0xa2fda832, 95 },
    { 0, 0 },
    { 0xdb9da834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78da839, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171da83d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12da840, 98 },
    { 0xaecda841, 95 },
    { 0xaa8da842, 99 },
    { 0xed6da843, 34 },
    { 0x4feda844, 76 },
    { 0x3dada841, 61 },
    { 0xafcda842, 33 },
    { 0xf53da847, 61 },
    { 0x360da848, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x631da84b, 98 },
    { 0xf5dda84b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339da850, 61 },
    { 0xa02da850, 93 },
    { 0, 0 },
    { 0x94dda853, 33 },
    { 0x9bbda854, 66 },
    { 0x1b5da855, 68 },
    { 0, 0 },
    { 0x9d3da857, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6da85b, 90 },
    { 0, 0 },
    { 0x791da85d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x753da860, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9da864, 34 },
    { 0xf0ada865, 91 },
    { 0xfd9da865, 93 },
    { 0x54da867, 55 },
    { 0xc40da868, 34 },
    { 0x2f6da865, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fda86d, 55 },
    { 0xc23da86e, 43 },
    { 0x543da86e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5da872, 76 },
    { 0x4c0da872, 61 },
    { 0xa8bda874, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2da87c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adda880, 95 },
    { 0xc03da881, 61 },
    { 0, 0 },
    { 0x366da883, 55 },
    { 0, 0 },
    { 0xb58da885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84da88f, 33 },
    { 0xb43da88f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1da893, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcda897, 33 },
    { 0xf34da897, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ada89b, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afda89e, 90 },
    { 0xf58da89e, 33 },
    { 0xe43da89f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6da8a5, 51 },
    { 0xd92da8a5, 23 },
    { 0x1b2da8a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ada8aa, 23 },
    { 0x209da8ab, 34 },
    { 0, 0 },
    { 0x1feda8ad, 33 },
    { 0, 0 },
    { 0xcabda8af, 61 },
    { 0x893da8b0, 33 },
    { 0xf44da8b1, 90 },
    { 0x70bda8b1, 91 },
    { 0x6ebda8b1, 33 },
    { 0xe94da8b2, 90 },
    { 0x640da8b4, 61 },
    { 0x9fdda8b0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1da8b9, 61 },
    { 0, 0 },
    { 0x4d6da8bb, 76 },
    { 0, 0 },
    { 0xde5da8bd, 61 },
    { 0, 0 },
    { 0x5e9da8bf, 91 },
    { 0, 0 },
    { 0xdd4da8c1, 90 },
    { 0x467da8c2, 100 },
    { 0x60eda8c2, 33 },
    { 0x2bfda8c4, 68 },
    { 0x63ada8c2, 33 },
    { 0x76eda8c6, 61 },
    { 0x4f9da8c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822da8cf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeda8d8, 34 },
    { 0xef1da8d9, 88 },
    { 0x2afda8da, 40 },
    { 0x46ada8db, 95 },
    { 0xe5ada8dc, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ada8e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84da8e6, 33 },
    { 0x279da8e7, 95 },
    { 0xe70da8e7, 52 },
    { 0xc6bda8e9, 90 },
    { 0, 0 },
    { 0xbda8eb, 87 },
    { 0x617da8ec, 23 },
    { 0xbd2da8ed, 33 },
    { 0, 0 },
    { 0x1feda8ef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15da8f6, 52 },
    { 0xafada8f6, 43 },
    { 0x848da8f6, 82 },
    { 0xe1fda8f7, 34 },
    { 0, 0 },
    { 0x26ada8fb, 76 },
    { 0xeccda8fc, 34 },
    { 0xbd9da8fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeda904, 23 },
    { 0x7cda905, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x223da908, 76 },
    { 0, 0 },
    { 0x2fbda90a, 90 },
    { 0, 0 },
    { 0xd45da90c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6da913, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0da917, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ada91a, 55 },
    { 0x44fda91b, 34 },
    { 0x49ada91b, 67 },
    { 0xf23da91d, 95 },
    { 0x30bda91d, 95 },
    { 0x1d7da91a, 23 },
    { 0x9d6da91a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bda924, 85 },
    { 0x828da924, 98 },
    { 0x37ada926, 93 },
    { 0, 0 },
    { 0x628da928, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8da92b, 80 },
    { 0xc87da92c, 93 },
    { 0xfd6da92d, 46 },
    { 0x505da92b, 91 },
    { 0, 0 },
    { 0xc0bda930, 17 },
    { 0, 0 },
    { 0x462da932, 77 },
    { 0, 0 },
    { 0xca1da934, 78 },
    { 0x31dda934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9da939, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedada945, 61 },
    { 0xf56da946, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810da94a, 90 },
    { 0x91cda94a, 61 },
    { 0, 0 },
    { 0x296da94d, 91 },
    { 0, 0 },
    { 0x5eada94f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56da952, 90 },
    { 0, 0 },
    { 0x159da954, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x530da957, 78 },
    { 0x8bcda957, 61 },
    { 0, 0 },
    { 0x447da95a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8da960, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585da966, 80 },
    { 0, 0 },
    { 0x6edda968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x241da96b, 88 },
    { 0, 0 },
    { 0xc73da96d, 95 },
    { 0x67eda96e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bda973, 67 },
    { 0x3ddda973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56da977, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98da97c, 33 },
    { 0x7d4da97d, 46 },
    { 0x298da97c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0da983, 43 },
    { 0xc08da984, 61 },
    { 0x4f3da983, 55 },
    { 0x472da986, 98 },
    { 0xc6eda987, 98 },
    { 0x801da988, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dda98b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597da991, 68 },
    { 0x2edda991, 33 },
    { 0x592da992, 66 },
    { 0, 0 },
    { 0x3dfda995, 43 },
    { 0xcda996, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131da99b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6da9ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53da9b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fda9b5, 98 },
    { 0xf0ada9b6, 43 },
    { 0xabada9b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684da9be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x387da9c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1da9ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2da9cd, 43 },
    { 0xcada9ce, 98 },
    { 0x899da9cf, 78 },
    { 0, 0 },
    { 0x89cda9d1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1da9d9, 93 },
    { 0, 0 },
    { 0xbb1da9db, 23 },
    { 0, 0 },
    { 0x1cbda9dd, 34 },
    { 0, 0 },
    { 0xbd5da9df, 33 },
    { 0x89dda9e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790da9e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13da9ec, 61 },
    { 0xabada9ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11da9f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394da9f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23da9f7, 91 },
    { 0x89eda9f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0da9ff, 91 },
    { 0xe68daa00, 90 },
    { 0x7a2da9ff, 67 },
    { 0, 0 },
    { 0x616daa03, 90 },
    { 0x4c9daa04, 61 },
    { 0, 0 },
    { 0x8f7daa06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69daa0c, 67 },
    { 0, 0 },
    { 0xdd0daa0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62daa11, 43 },
    { 0, 0 },
    { 0xcb9daa13, 90 },
    { 0x6b9daa14, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07daa17, 61 },
    { 0x8f7daa18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241daa1e, 33 },
    { 0x2cedaa1e, 33 },
    { 0, 0 },
    { 0x108daa21, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935daa27, 76 },
    { 0x5f9daa28, 43 },
    { 0xe02daa28, 90 },
    { 0x29edaa29, 98 },
    { 0x6c1daa2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6daa31, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46daa35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ddaa38, 98 },
    { 0, 0 },
    { 0x264daa3a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813daa41, 23 },
    { 0xfe1daa42, 99 },
    { 0x8e5daa43, 76 },
    { 0xbeddaa42, 55 },
    { 0, 0 },
    { 0xf78daa46, 93 },
    { 0, 0 },
    { 0x462daa48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69adaa4c, 93 },
    { 0x558daa4d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bdaa50, 33 },
    { 0, 0 },
    { 0x276daa52, 23 },
    { 0xb21daa52, 33 },
    { 0, 0 },
    { 0x62edaa55, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66adaa5c, 61 },
    { 0, 0 },
    { 0x7ebdaa5e, 46 },
    { 0, 0 },
    { 0xd4edaa60, 75 },
    { 0x1f7daa60, 95 },
    { 0xa98daa62, 23 },
    { 0x9cbdaa63, 34 },
    { 0, 0 },
    { 0xae0daa65, 52 },
    { 0x53daa66, 90 },
    { 0, 0 },
    { 0xcffdaa68, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bdaa6b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5daa6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2edaa71, 95 },
    { 0xb00daa72, 52 },
    { 0, 0 },
    { 0x8dedaa74, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fdaa77, 90 },
    { 0, 0 },
    { 0xfa8daa79, 46 },
    { 0, 0 },
    { 0xa59daa7b, 55 },
    { 0, 0 },
    { 0x5b4daa7d, 95 },
    { 0x915daa7e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3daa84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77daa87, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfdaa8a, 40 },
    { 0, 0 },
    { 0xfaedaa8c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4daa90, 33 },
    { 0xbc2daa91, 33 },
    { 0, 0 },
    { 0x639daa93, 33 },
    { 0x62daa94, 97 },
    { 0x51cdaa94, 95 },
    { 0x2fddaa96, 23 },
    { 0xa1adaa97, 61 },
    { 0x5cadaa98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45daa9f, 90 },
    { 0x81daaa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ddaaa8, 76 },
    { 0x9b5daaa9, 46 },
    { 0x8f7daaaa, 81 },
    { 0xf41daaa9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efdaab0, 90 },
    { 0x314daab0, 23 },
    { 0x465daab0, 33 },
    { 0x17ddaab1, 98 },
    { 0, 0 },
    { 0xdf9daab5, 95 },
    { 0xd8edaab5, 90 },
    { 0xd35daab5, 91 },
    { 0, 0 },
    { 0xa55daab9, 23 },
    { 0xa1cdaab9, 46 },
    { 0xe28daabb, 90 },
    { 0xd29daabb, 43 },
    { 0, 0 },
    { 0x97edaabe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85daac1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x275daac4, 90 },
    { 0x843daac4, 93 },
    { 0x4f8daac6, 61 },
    { 0xe47daac4, 51 },
    { 0x3eddaac8, 90 },
    { 0xe2fdaac7, 61 },
    { 0xcb8daac4, 55 },
    { 0x2a5daaca, 33 },
    { 0, 0 },
    { 0x777daacd, 91 },
    { 0x83daacd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730daad2, 61 },
    { 0xd85daad3, 33 },
    { 0xbfedaad2, 33 },
    { 0x637daad5, 90 },
    { 0x831daad5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191daadc, 98 },
    { 0, 0 },
    { 0xf89daade, 66 },
    { 0xed1daadf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x914daae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9daaed, 90 },
    { 0xe4ddaaed, 61 },
    { 0xcb1daaef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4daaf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94daaf9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699dab04, 43 },
    { 0x3c3dab05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45dab08, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6adab0d, 46 },
    { 0x1fcdab0e, 34 },
    { 0x98cdab0e, 95 },
    { 0x9f8dab0f, 93 },
    { 0x538dab11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0dab15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908dab1b, 49 },
    { 0x334dab1b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x853dab1f, 33 },
    { 0xd72dab20, 33 },
    { 0xff5dab1f, 76 },
    { 0xbbedab1f, 61 },
    { 0x1cbdab1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8dab26, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bdab2b, 95 },
    { 0x6a1dab2b, 61 },
    { 0x1b1dab2d, 88 },
    { 0x9afdab2e, 95 },
    { 0x488dab2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x86edab32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fdab36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527dab3a, 23 },
    { 0x35adab3a, 95 },
    { 0, 0 },
    { 0xa20dab3d, 81 },
    { 0x239dab3e, 23 },
    { 0x89cdab3f, 98 },
    { 0x1cbdab3d, 17 },
    { 0x2dadab3e, 51 },
    { 0x74edab42, 61 },
    { 0xdfdab42, 23 },
    { 0x37bdab42, 66 },
    { 0, 0 },
    { 0x913dab46, 95 },
    { 0, 0 },
    { 0xecddab48, 93 },
    { 0xa4edab49, 61 },
    { 0x723dab48, 33 },
    { 0, 0 },
    { 0x4fcdab4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3dab4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7dab54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cdab57, 90 },
    { 0, 0 },
    { 0xfd4dab59, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2dab5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15adab61, 43 },
    { 0x4cbdab62, 98 },
    { 0, 0 },
    { 0x30dab64, 84 },
    { 0x62edab65, 90 },
    { 0, 0 },
    { 0xfc2dab67, 33 },
    { 0, 0 },
    { 0x688dab69, 61 },
    { 0xf4bdab6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfdab6d, 46 },
    { 0, 0 },
    { 0x564dab6f, 61 },
    { 0, 0 },
    { 0x1badab71, 93 },
    { 0, 0 },
    { 0x952dab73, 90 },
    { 0, 0 },
    { 0x817dab75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8edab78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0dab7b, 23 },
    { 0x972dab7b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4badab80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6dab83, 61 },
    { 0x859dab83, 93 },
    { 0xc6bdab85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cdab88, 33 },
    { 0xe6ddab88, 61 },
    { 0xf71dab88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebdab8d, 23 },
    { 0x1ebdab8e, 61 },
    { 0, 0 },
    { 0x9f4dab90, 61 },
    { 0x14edab91, 90 },
    { 0xce9dab92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26dab95, 87 },
    { 0x7b7dab96, 98 },
    { 0, 0 },
    { 0x401dab98, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdab9f, 98 },
    { 0x914daba0, 88 },
    { 0, 0 },
    { 0xcd0daba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97daba6, 61 },
    { 0, 0 },
    { 0x395daba8, 55 },
    { 0, 0 },
    { 0x697dabaa, 61 },
    { 0xb90dabab, 91 },
    { 0, 0 },
    { 0xd1bdabad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82dabb2, 61 },
    { 0x82adabb2, 23 },
    { 0, 0 },
    { 0x454dabb5, 95 },
    { 0, 0 },
    { 0xf3adabb7, 33 },
    { 0xbfbdabb8, 95 },
    { 0, 0 },
    { 0x2c3dabba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801dabbf, 61 },
    { 0, 0 },
    { 0x60fdabc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74dabc5, 95 },
    { 0x490dabc6, 66 },
    { 0, 0 },
    { 0x528dabc8, 76 },
    { 0xf6cdabc9, 33 },
    { 0xb63dabca, 68 },
    { 0x72adabca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bdabce, 23 },
    { 0x329dabcf, 52 },
    { 0xbb2dabcf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453dabd8, 61 },
    { 0xb41dabd9, 33 },
    { 0xa8ddabda, 98 },
    { 0x62dabd8, 76 },
    { 0, 0 },
    { 0x3a9dabdd, 34 },
    { 0x228dabdd, 61 },
    { 0x525dabdf, 80 },
    { 0xe16dabe0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3dabe5, 91 },
    { 0, 0 },
    { 0x450dabe7, 61 },
    { 0xd94dabe8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x920dabeb, 85 },
    { 0, 0 },
    { 0xb47dabed, 93 },
    { 0xdd6dabee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bdabf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8dabf4, 23 },
    { 0xfa1dabf4, 91 },
    { 0x40dabf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefdabf9, 98 },
    { 0, 0 },
    { 0x755dabfb, 33 },
    { 0x980dabfb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cdac01, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dac07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x695dac0a, 76 },
    { 0x4ffdac0b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3dac10, 97 },
    { 0x570dac10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x879dac14, 23 },
    { 0, 0 },
    { 0xe50dac16, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dac1b, 68 },
    { 0x2dadac1c, 76 },
    { 0, 0 },
    { 0xfdddac1e, 61 },
    { 0xf0cdac1f, 46 },
    { 0, 0 },
    { 0x5afdac21, 98 },
    { 0xa2fdac22, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadac26, 66 },
    { 0x65adac27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x889dac2a, 36 },
    { 0xb8bdac2b, 55 },
    { 0, 0 },
    { 0x930dac2d, 78 },
    { 0x153dac2e, 60 },
    { 0, 0 },
    { 0xf70dac30, 61 },
    { 0x678dac31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdddac35, 93 },
    { 0xfa9dac36, 100 },
    { 0, 0 },
    { 0x599dac38, 90 },
    { 0x8a6dac39, 76 },
    { 0, 0 },
    { 0xb3bdac3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbdac3e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bdac43, 55 },
    { 0x193dac43, 98 },
    { 0xf4fdac44, 99 },
    { 0x749dac46, 95 },
    { 0xaf6dac43, 98 },
    { 0x10adac44, 99 },
    { 0x3d5dac49, 65 },
    { 0, 0 },
    { 0xe48dac4b, 61 },
    { 0xf31dac4b, 55 },
    { 0, 0 },
    { 0x113dac4e, 95 },
    { 0, 0 },
    { 0x79bdac50, 68 },
    { 0xec2dac51, 61 },
    { 0, 0 },
    { 0xd85dac53, 61 },
    { 0x3b0dac53, 68 },
    { 0x9d5dac55, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacedac5f, 23 },
    { 0, 0 },
    { 0x8edac61, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660dac67, 100 },
    { 0xfd7dac68, 84 },
    { 0xbe8dac69, 76 },
    { 0xd5cdac6a, 33 },
    { 0xf99dac67, 95 },
    { 0, 0 },
    { 0xaadac6d, 95 },
    { 0, 0 },
    { 0x170dac6f, 34 },
    { 0, 0 },
    { 0xbbcdac71, 46 },
    { 0, 0 },
    { 0xf4cdac73, 55 },
    { 0xb95dac73, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4dac78, 61 },
    { 0, 0 },
    { 0x1dedac7a, 95 },
    { 0x28dac7a, 33 },
    { 0x771dac7c, 23 },
    { 0xf94dac7c, 67 },
    { 0x545dac7e, 43 },
    { 0x9f3dac7f, 94 },
    { 0, 0 },
    { 0xe1adac81, 98 },
    { 0x7b4dac82, 46 },
    { 0x707dac83, 33 },
    { 0x71cdac81, 95 },
    { 0xa82dac83, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dac88, 61 },
    { 0x5fcdac89, 93 },
    { 0xdd5dac89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4dac8d, 95 },
    { 0x8e9dac8e, 59 },
    { 0x34ddac8f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647dac95, 93 },
    { 0x5c2dac96, 100 },
    { 0, 0 },
    { 0xd89dac98, 46 },
    { 0xef9dac98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4addac9f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3daca3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x840daca6, 52 },
    { 0x372daca6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfdacad, 33 },
    { 0xd38dacad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21dacb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1dacb7, 53 },
    { 0xdc4dacb8, 98 },
    { 0x311dacb9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fdacbc, 33 },
    { 0x40cdacbd, 33 },
    { 0xa1adacbd, 61 },
    { 0xcd7dacbf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398dacca, 75 },
    { 0xd69daccb, 95 },
    { 0xd9daccb, 90 },
    { 0x11bdaccc, 61 },
    { 0x44bdaccb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57edacd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948dace1, 64 },
    { 0x292dace2, 98 },
    { 0x453dace2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ddace7, 61 },
    { 0, 0 },
    { 0xa8fdace9, 23 },
    { 0xeddacea, 23 },
    { 0xd89dacea, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164dacf2, 43 },
    { 0x19ddacf2, 46 },
    { 0x73fdacf4, 76 },
    { 0x26edacf5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327dacfd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fddad03, 33 },
    { 0, 0 },
    { 0xbe7dad05, 55 },
    { 0x403dad06, 81 },
    { 0xce2dad07, 76 },
    { 0xb62dad08, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbdad0c, 98 },
    { 0x244dad0d, 90 },
    { 0xd83dad0c, 93 },
    { 0x32ddad0f, 95 },
    { 0x65dad0f, 98 },
    { 0x719dad11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2dad16, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1adad1e, 67 },
    { 0x4fcdad1e, 99 },
    { 0x815dad1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30dad27, 75 },
    { 0x6dddad28, 33 },
    { 0, 0 },
    { 0x8badad2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2dad2d, 33 },
    { 0xfeadad2e, 90 },
    { 0, 0 },
    { 0x14cdad30, 90 },
    { 0x161dad31, 43 },
    { 0xd07dad32, 76 },
    { 0x51cdad31, 33 },
    { 0x9ddad30, 43 },
    { 0xfa5dad35, 95 },
    { 0, 0 },
    { 0xc91dad37, 76 },
    { 0, 0 },
    { 0x773dad39, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aedad3d, 99 },
    { 0x75cdad3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x686dad41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7dad44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afdad4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35dad50, 23 },
    { 0xed6dad50, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964dad57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ddad5d, 61 },
    { 0x992dad5e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573dad65, 33 },
    { 0x3e8dad65, 76 },
    { 0, 0 },
    { 0xb16dad68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x985dad6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51edad6e, 77 },
    { 0x5d4dad6f, 98 },
    { 0xad4dad6e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77edad77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cddad7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28dad7d, 98 },
    { 0x5fcdad7e, 90 },
    { 0x5b4dad7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40dad89, 33 },
    { 0xd1fdad8a, 78 },
    { 0xe51dad8a, 78 },
    { 0, 0 },
    { 0xcfadad8d, 23 },
    { 0xbdddad8e, 61 },
    { 0x17dad8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88dad93, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602dad99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4dad9f, 43 },
    { 0, 0 },
    { 0xd1cdada1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ddada4, 90 },
    { 0, 0 },
    { 0xa98dada6, 34 },
    { 0x597dada7, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bdadaa, 23 },
    { 0xf82dadab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x127dadae, 88 },
    { 0, 0 },
    { 0x25bdadb0, 90 },
    { 0x518dadb1, 55 },
    { 0xc6bdadb0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1dadb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15dadb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcdadbc, 61 },
    { 0x83adadbc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fedadc2, 66 },
    { 0, 0 },
    { 0x3e6dadc4, 90 },
    { 0, 0 },
    { 0x76cdadc6, 23 },
    { 0x234dadc7, 25 },
    { 0x1dbdadc8, 76 },
    { 0x8e2dadc8, 93 },
    { 0x4c4dadca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95edadd8, 76 },
    { 0, 0 },
    { 0x58adadda, 23 },
    { 0, 0 },
    { 0xbc1daddc, 87 },
    { 0xd9adaddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbdade3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6dade9, 66 },
    { 0xbebdadea, 100 },
    { 0x6dadade9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01dadf1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdadf4, 100 },
    { 0, 0 },
    { 0x5d4dadf6, 61 },
    { 0x3a1dadf7, 61 },
    { 0x539dadf8, 98 },
    { 0x29bdadf9, 95 },
    { 0, 0 },
    { 0xae5dadfb, 90 },
    { 0xc2bdadfc, 34 },
    { 0x2c8dadfd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fdae02, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38dae05, 90 },
    { 0x82cdae06, 90 },
    { 0, 0 },
    { 0xebbdae08, 90 },
    { 0x223dae09, 95 },
    { 0x3d7dae09, 95 },
    { 0x9b8dae09, 95 },
    { 0x439dae08, 90 },
    { 0, 0 },
    { 0x672dae0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111dae12, 94 },
    { 0xa57dae13, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63edae18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x627dae1b, 33 },
    { 0x232dae1b, 46 },
    { 0xc3adae1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfdae21, 66 },
    { 0, 0 },
    { 0x8b7dae23, 33 },
    { 0, 0 },
    { 0xa3bdae25, 98 },
    { 0xa5edae25, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbdae2b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7dae33, 90 },
    { 0xe48dae34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2dae39, 61 },
    { 0, 0 },
    { 0x888dae3b, 61 },
    { 0x2aedae3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49dae41, 75 },
    { 0xd95dae42, 77 },
    { 0, 0 },
    { 0x5fadae44, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cdae49, 52 },
    { 0x4bdae4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46dae52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187dae58, 65 },
    { 0, 0 },
    { 0xa24dae5a, 61 },
    { 0xf1fdae5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461dae61, 61 },
    { 0, 0 },
    { 0x4dddae63, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9dae69, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6dae6d, 90 },
    { 0x596dae6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x825dae71, 77 },
    { 0xb3dae72, 85 },
    { 0xaaadae71, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaedae76, 91 },
    { 0xa85dae76, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6badae7d, 33 },
    { 0xafddae7d, 33 },
    { 0x5fedae7e, 77 },
    { 0xc6cdae7f, 34 },
    { 0, 0 },
    { 0x637dae82, 98 },
    { 0xe60dae83, 90 },
    { 0, 0 },
    { 0x5f7dae85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aadae88, 90 },
    { 0, 0 },
    { 0x28dae8a, 33 },
    { 0x5c0dae8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacddae8e, 61 },
    { 0, 0 },
    { 0x9adae90, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afdae93, 78 },
    { 0x540dae94, 77 },
    { 0xccadae94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ddae9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82daea1, 17 },
    { 0xbccdaea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878daea9, 43 },
    { 0x2f4daeaa, 33 },
    { 0xa1bdaea9, 76 },
    { 0x213daeac, 95 },
    { 0x29bdaead, 97 },
    { 0x362daeae, 76 },
    { 0x401daea9, 93 },
    { 0xd2fdaeb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6edaeb4, 90 },
    { 0, 0 },
    { 0x3cadaeb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89daeba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5daebd, 98 },
    { 0x71fdaebe, 76 },
    { 0x53fdaebf, 36 },
    { 0x9abdaebe, 93 },
    { 0x9d4daebe, 95 },
    { 0, 0 },
    { 0x2aadaec3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0daec9, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00daecf, 98 },
    { 0x565daed0, 40 },
    { 0xcbdaecf, 98 },
    { 0, 0 },
    { 0xfffdaed3, 23 },
    { 0, 0 },
    { 0xdf5daed5, 75 },
    { 0x8f6daed6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7fdaeda, 90 },
    { 0x424daedb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36fdaedf, 55 },
    { 0, 0 },
    { 0xf9daee1, 34 },
    { 0, 0 },
    { 0x8f9daee3, 90 },
    { 0, 0 },
    { 0x30ddaee5, 95 },
    { 0xcbedaee6, 98 },
    { 0xceedaee7, 86 },
    { 0xba1daee6, 33 },
    { 0x861daee9, 75 },
    { 0x8abdaee9, 33 },
    { 0x8dddaeeb, 55 },
    { 0xa6ddaeea, 93 },
    { 0x5d4daee6, 56 },
    { 0x688daeee, 95 },
    { 0x49cdaeef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34daef7, 90 },
    { 0xd04daef7, 34 },
    { 0x68bdaef7, 33 },
    { 0, 0 },
    { 0xb84daefb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcdaf00, 55 },
    { 0xf35daf01, 55 },
    { 0xce0daf01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1cdaf06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9daf0b, 23 },
    { 0x24daf0c, 66 },
    { 0xf8bdaf0c, 76 },
    { 0x7bdaf0b, 67 },
    { 0xea2daf0f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4daf12, 95 },
    { 0xb87daf12, 81 },
    { 0x1d0daf13, 90 },
    { 0x3adaf15, 61 },
    { 0, 0 },
    { 0xd01daf17, 93 },
    { 0xa6fdaf18, 34 },
    { 0, 0 },
    { 0x1fcdaf1a, 67 },
    { 0, 0 },
    { 0xe60daf1c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32daf21, 43 },
    { 0xd11daf22, 90 },
    { 0xffddaf22, 93 },
    { 0x687daf21, 34 },
    { 0x88bdaf25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158daf2a, 98 },
    { 0xd10daf2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11edaf30, 23 },
    { 0x232daf30, 90 },
    { 0x829daf30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x129daf36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbedaf40, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2daf49, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02daf4d, 98 },
    { 0, 0 },
    { 0x5aedaf4f, 61 },
    { 0x6bcdaf50, 33 },
    { 0xb2fdaf51, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08daf55, 55 },
    { 0x7addaf55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ddaf5a, 33 },
    { 0x612daf5a, 76 },
    { 0, 0 },
    { 0x70fdaf5d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fdaf60, 76 },
    { 0x286daf61, 96 },
    { 0xeefdaf60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bdaf65, 90 },
    { 0x758daf66, 34 },
    { 0xe47daf65, 55 },
    { 0, 0 },
    { 0xab2daf69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0daf70, 33 },
    { 0, 0 },
    { 0x156daf72, 66 },
    { 0x110daf73, 23 },
    { 0x555daf74, 67 },
    { 0, 0 },
    { 0xeb9daf76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14daf7b, 17 },
    { 0, 0 },
    { 0x990daf7d, 65 },
    { 0, 0 },
    { 0xec2daf7f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67daf85, 34 },
    { 0, 0 },
    { 0x92cdaf87, 55 },
    { 0x82bdaf88, 23 },
    { 0x1e1daf89, 17 },
    { 0x381daf8a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5daf93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bedaf9f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abdafa5, 43 },
    { 0x660dafa6, 98 },
    { 0x53adafa7, 99 },
    { 0x83dafa6, 43 },
    { 0xf44dafa9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfdafb2, 23 },
    { 0, 0 },
    { 0x2c8dafb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210dafb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4dafc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x966dafc5, 99 },
    { 0, 0 },
    { 0xdc4dafc7, 88 },
    { 0x11ddafc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999dafcd, 98 },
    { 0xdc0dafcd, 98 },
    { 0xd91dafcf, 78 },
    { 0xcd3dafce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fdafd3, 33 },
    { 0x555dafd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x38bdafd7, 90 },
    { 0x18dafd8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ddafdf, 23 },
    { 0x56bdafe0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x783dafe3, 98 },
    { 0x752dafe4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcdafea, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cdafed, 23 },
    { 0, 0 },
    { 0x612dafef, 61 },
    { 0x108daff0, 99 },
    { 0, 0 },
    { 0x60edaff2, 98 },
    { 0x785daff2, 90 },
    { 0xedcdaff4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x772daff7, 81 },
    { 0x331daff7, 76 },
    { 0xb3edaff9, 43 },
    { 0x5ffdaff7, 95 },
    { 0xd2edaffb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28daffe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91db002, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9db005, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30db009, 93 },
    { 0x8dfdb00a, 67 },
    { 0xecddb00a, 17 },
    { 0x695db00c, 33 },
    { 0x23edb00d, 90 },
    { 0x6ccdb00e, 61 },
    { 0x5a6db00c, 78 },
    { 0xae9db00c, 90 },
    { 0x97ddb00e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eadb01d, 98 },
    { 0x2bedb01e, 23 },
    { 0x3e1db01f, 65 },
    { 0xd89db01f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbdb023, 90 },
    { 0xfd2db023, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ddb027, 68 },
    { 0x32edb027, 33 },
    { 0x5fbdb027, 83 },
    { 0xa70db028, 98 },
    { 0, 0 },
    { 0x5c7db02c, 90 },
    { 0x94cdb02c, 88 },
    { 0x399db02e, 98 },
    { 0x7bdb02d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9db032, 98 },
    { 0xa71db033, 94 },
    { 0x1dedb033, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6db038, 90 },
    { 0xaf4db039, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efdb040, 84 },
    { 0xff2db041, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57db044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cdb057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x65db05a, 23 },
    { 0, 0 },
    { 0xbd9db05c, 23 },
    { 0x49cdb05d, 33 },
    { 0xffedb05e, 33 },
    { 0xeebdb05f, 33 },
    { 0x3a6db060, 76 },
    { 0x8f7db061, 43 },
    { 0, 0 },
    { 0x32db063, 76 },
    { 0x780db064, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2db06d, 80 },
    { 0, 0 },
    { 0x4abdb06f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99db076, 33 },
    { 0x936db077, 34 },
    { 0x234db076, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cdb07b, 23 },
    { 0xe61db07c, 3 },
    { 0xb45db07d, 90 },
    { 0, 0 },
    { 0x970db07f, 46 },
    { 0x4adb07f, 90 },
    { 0xf11db07f, 98 },
    { 0xbb9db080, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165db088, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89db08b, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebdb08f, 46 },
    { 0xdadb090, 61 },
    { 0, 0 },
    { 0x37db092, 61 },
    { 0x940db092, 69 },
    { 0x41ddb094, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35db098, 59 },
    { 0x2a6db099, 33 },
    { 0x23bdb099, 24 },
    { 0x4f9db09b, 33 },
    { 0x416db09b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ddb0a2, 90 },
    { 0xee8db0a3, 90 },
    { 0, 0 },
    { 0xd5fdb0a5, 98 },
    { 0x4f5db0a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fadb0aa, 33 },
    { 0x24edb0ab, 33 },
    { 0, 0 },
    { 0xbecdb0ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abdb0b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fdb0b5, 77 },
    { 0x674db0b6, 43 },
    { 0x75fdb0b6, 33 },
    { 0, 0 },
    { 0xc83db0b9, 66 },
    { 0x933db0ba, 68 },
    { 0x5a7db0b9, 33 },
    { 0xc0fdb0bc, 23 },
    { 0x36fdb0b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfdb0c2, 34 },
    { 0, 0 },
    { 0x6e4db0c4, 33 },
    { 0xb1cdb0c4, 43 },
    { 0xc2db0c6, 98 },
    { 0xc34db0c6, 90 },
    { 0, 0 },
    { 0x857db0c9, 61 },
    { 0xaf1db0c9, 100 },
    { 0x246db0ca, 78 },
    { 0x9dcdb0cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7db0cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410db0d4, 61 },
    { 0, 0 },
    { 0x2bfdb0d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2db0e0, 17 },
    { 0xfeddb0e1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ddb0e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ddb0e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63db0ec, 90 },
    { 0x716db0ed, 90 },
    { 0x672db0ec, 52 },
    { 0xcefdb0ef, 52 },
    { 0, 0 },
    { 0xc68db0f1, 98 },
    { 0, 0 },
    { 0xae5db0f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0db0f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98db103, 91 },
    { 0, 0 },
    { 0x1b3db105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12db108, 46 },
    { 0, 0 },
    { 0xd10db10a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42adb10e, 90 },
    { 0, 0 },
    { 0x2b1db110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6db11b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3db121, 65 },
    { 0xa50db122, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edb125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbdb129, 77 },
    { 0xf93db12a, 33 },
    { 0xfddb12b, 78 },
    { 0x1edb12b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7db12f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fdb133, 98 },
    { 0x28cdb134, 94 },
    { 0x6efdb135, 33 },
    { 0x4dadb136, 33 },
    { 0x8a7db137, 85 },
    { 0xb19db134, 76 },
    { 0x334db139, 90 },
    { 0x72edb13a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb13d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7db140, 33 },
    { 0, 0 },
    { 0x2b0db142, 78 },
    { 0x354db143, 52 },
    { 0x1b8db144, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77bdb148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x492db14b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x867db14e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00db153, 98 },
    { 0x271db154, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06db158, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cdb15c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccdb15f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x859db162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284db166, 77 },
    { 0x475db166, 34 },
    { 0, 0 },
    { 0x571db169, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2db170, 33 },
    { 0, 0 },
    { 0x9cadb172, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x967db175, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1db179, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aedb17c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00db180, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fadb183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1db188, 61 },
    { 0x635db189, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x76db18c, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5db191, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19db196, 43 },
    { 0, 0 },
    { 0x338db198, 94 },
    { 0, 0 },
    { 0x984db19a, 77 },
    { 0xd81db19a, 90 },
    { 0x235db19c, 98 },
    { 0xa3cdb19d, 61 },
    { 0xc07db19d, 90 },
    { 0x848db19d, 95 },
    { 0x2d2db1a0, 34 },
    { 0xf82db1a1, 55 },
    { 0, 0 },
    { 0xa74db1a3, 61 },
    { 0, 0 },
    { 0x901db1a5, 93 },
    { 0, 0 },
    { 0x6cbdb1a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fddb1ab, 61 },
    { 0x380db1ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71edb1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518db1b6, 87 },
    { 0, 0 },
    { 0xd1edb1b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafdb1bb, 98 },
    { 0xb4ddb1bc, 95 },
    { 0x734db1bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x634db1c0, 61 },
    { 0xab6db1c1, 55 },
    { 0, 0 },
    { 0xc81db1c3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3cdb1c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcdb1cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ddb1cf, 61 },
    { 0x889db1d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49adb1d7, 84 },
    { 0x3d6db1d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28db1e2, 23 },
    { 0, 0 },
    { 0x8eedb1e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72edb1e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805db1ee, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedb1f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe6db1f8, 67 },
    { 0xa0bdb1f9, 52 },
    { 0xce2db1fa, 76 },
    { 0, 0 },
    { 0x7bdb1fc, 95 },
    { 0x415db1fd, 23 },
    { 0xc01db1fe, 33 },
    { 0, 0 },
    { 0x8aadb200, 90 },
    { 0xe66db201, 33 },
    { 0x390db201, 46 },
    { 0, 0 },
    { 0x83cdb204, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4db209, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bddb20f, 61 },
    { 0xf29db210, 23 },
    { 0xa2fdb20f, 90 },
    { 0x66db210, 67 },
    { 0, 0 },
    { 0x85edb214, 55 },
    { 0xc7cdb214, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x594db218, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfdb229, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34db22c, 51 },
    { 0xbf8db22c, 55 },
    { 0, 0 },
    { 0xed6db22f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bdb232, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fdb237, 90 },
    { 0, 0 },
    { 0xbeddb239, 61 },
    { 0x61db23a, 33 },
    { 0xb7cdb23b, 77 },
    { 0xef1db23b, 33 },
    { 0, 0 },
    { 0x40adb23e, 61 },
    { 0xf34db23e, 98 },
    { 0x1edb23e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdb244, 66 },
    { 0, 0 },
    { 0x9eedb246, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ddb249, 95 },
    { 0, 0 },
    { 0xfc9db24b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedddb253, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ddb256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7db259, 81 },
    { 0x64db25a, 90 },
    { 0, 0 },
    { 0x3b5db25c, 61 },
    { 0x7db25d, 98 },
    { 0xc5edb25e, 98 },
    { 0x277db25e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345db265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1edb269, 76 },
    { 0xb53db269, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6edb26f, 33 },
    { 0, 0 },
    { 0x83ddb271, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2db275, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84adb27c, 34 },
    { 0x5a0db27d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bdb284, 90 },
    { 0xc75db284, 61 },
    { 0x339db286, 93 },
    { 0xe7db287, 76 },
    { 0x8a1db288, 23 },
    { 0xe0bdb289, 81 },
    { 0x3f0db288, 90 },
    { 0x9dddb289, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ddb296, 98 },
    { 0, 0 },
    { 0x4efdb298, 34 },
    { 0x104db298, 33 },
    { 0x763db298, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16db29d, 100 },
    { 0, 0 },
    { 0x52bdb29f, 98 },
    { 0xba3db29f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504db2a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cdb2ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cdb2b0, 33 },
    { 0, 0 },
    { 0x73bdb2b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bddb2b5, 46 },
    { 0, 0 },
    { 0xec4db2b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6db2c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6db2c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692db2cc, 33 },
    { 0x1b1db2cd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94db2d1, 52 },
    { 0xed5db2d1, 98 },
    { 0x33db2d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82db2d9, 61 },
    { 0, 0 },
    { 0xd91db2db, 95 },
    { 0, 0 },
    { 0x778db2dd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15db2e0, 33 },
    { 0x25fdb2e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59adb2ec, 98 },
    { 0x157db2ed, 64 },
    { 0x477db2ec, 91 },
    { 0xb0adb2ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7db2f2, 85 },
    { 0, 0 },
    { 0x602db2f4, 95 },
    { 0x3b6db2f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x201db2f8, 17 },
    { 0xc44db2f9, 33 },
    { 0xe72db2fa, 93 },
    { 0x406db2fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6badb301, 95 },
    { 0x8aadb302, 99 },
    { 0x98adb301, 33 },
    { 0x41fdb304, 33 },
    { 0x5ddb304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503db30b, 90 },
    { 0x576db30c, 95 },
    { 0, 0 },
    { 0x18edb30e, 17 },
    { 0x690db30e, 90 },
    { 0xeb9db30f, 33 },
    { 0x42edb30f, 61 },
    { 0x18edb312, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45db315, 98 },
    { 0x9e1db316, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2db319, 17 },
    { 0, 0 },
    { 0x196db31b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87db323, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bdb32c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01db332, 33 },
    { 0x751db333, 34 },
    { 0x654db334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fdb337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21db33a, 69 },
    { 0x7d8db33b, 67 },
    { 0x16adb33c, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27db341, 61 },
    { 0x21db341, 90 },
    { 0x940db343, 52 },
    { 0x25edb344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cddb347, 66 },
    { 0xdb4db348, 66 },
    { 0x7eedb348, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28db34c, 100 },
    { 0x417db34c, 77 },
    { 0x70cdb34e, 98 },
    { 0, 0 },
    { 0x927db350, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5db354, 33 },
    { 0xc0edb355, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fdb35d, 61 },
    { 0x46adb35d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a5db366, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83db36e, 90 },
    { 0, 0 },
    { 0x895db370, 99 },
    { 0xbb1db371, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cdb374, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bdb377, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360db37d, 90 },
    { 0x425db37e, 93 },
    { 0, 0 },
    { 0x3edb380, 61 },
    { 0, 0 },
    { 0x55fdb382, 66 },
    { 0, 0 },
    { 0xe6cdb384, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9db38c, 23 },
    { 0xd50db38d, 95 },
    { 0xa4adb38e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0db391, 61 },
    { 0x3cedb391, 55 },
    { 0x697db393, 76 },
    { 0, 0 },
    { 0x4bdb395, 84 },
    { 0x45adb395, 93 },
    { 0x90edb395, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ddb39d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7db3a1, 98 },
    { 0x2c3db3a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73db3a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fdb3a9, 78 },
    { 0xe54db3a9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991db3ae, 67 },
    { 0x523db3af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8db3b7, 66 },
    { 0x9fadb3b8, 61 },
    { 0, 0 },
    { 0x959db3ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183db3bf, 33 },
    { 0x7d3db3bf, 85 },
    { 0, 0 },
    { 0xb5db3c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ddb3c5, 87 },
    { 0xf40db3c6, 90 },
    { 0x36edb3c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682db3cd, 95 },
    { 0x5c9db3ce, 98 },
    { 0x4abdb3ce, 43 },
    { 0x895db3d0, 98 },
    { 0xad4db3d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x857db3d4, 61 },
    { 0, 0 },
    { 0x6b7db3d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffdb3d9, 100 },
    { 0xe9db3da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81db3ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0db3f0, 98 },
    { 0xacdb3f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fadb3fb, 66 },
    { 0x575db3fc, 98 },
    { 0xa90db3fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x497db400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eddb403, 46 },
    { 0xe57db404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29db409, 61 },
    { 0xa5cdb409, 85 },
    { 0xeb8db40b, 55 },
    { 0x91cdb40b, 12 },
    { 0xbaadb40d, 61 },
    { 0x11db40e, 23 },
    { 0x94bdb40d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd50db413, 33 },
    { 0x9c2db413, 61 },
    { 0xbe1db415, 23 },
    { 0xe86db414, 75 },
    { 0xd16db417, 23 },
    { 0xca7db418, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6db420, 99 },
    { 0x588db421, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ddb428, 87 },
    { 0x5b9db428, 87 },
    { 0xc8edb429, 91 },
    { 0, 0 },
    { 0xec1db42c, 76 },
    { 0xa03db42d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487db432, 66 },
    { 0xf7adb433, 95 },
    { 0, 0 },
    { 0x3a0db435, 77 },
    { 0xd59db436, 84 },
    { 0xdafdb437, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23db443, 60 },
    { 0x664db444, 88 },
    { 0, 0 },
    { 0x670db446, 61 },
    { 0, 0 },
    { 0x597db448, 23 },
    { 0x5d8db449, 23 },
    { 0, 0 },
    { 0x3fadb44b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79edb44e, 95 },
    { 0x788db44e, 67 },
    { 0, 0 },
    { 0x73cdb451, 76 },
    { 0x1e5db452, 91 },
    { 0x56fdb452, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4db456, 91 },
    { 0xebddb457, 81 },
    { 0x6b4db457, 33 },
    { 0, 0 },
    { 0x17edb45a, 90 },
    { 0x102db45b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9db45f, 90 },
    { 0xb12db45f, 90 },
    { 0x446db461, 84 },
    { 0, 0 },
    { 0xc5fdb463, 61 },
    { 0x57cdb463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfdb469, 95 },
    { 0x10fdb46a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04db46d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x353db470, 34 },
    { 0x271db471, 90 },
    { 0x55ddb472, 93 },
    { 0, 0 },
    { 0x3d2db474, 17 },
    { 0x68bdb474, 55 },
    { 0x9e8db476, 61 },
    { 0x1badb477, 88 },
    { 0, 0 },
    { 0xa87db479, 98 },
    { 0xf27db47a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x488db47d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401db482, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7db485, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dedb488, 95 },
    { 0x5f2db488, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415db491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6db496, 90 },
    { 0xb54db497, 8 },
    { 0x966db496, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91db49e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9db4aa, 90 },
    { 0xac6db4aa, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebdb4ae, 88 },
    { 0xdebdb4ae, 95 },
    { 0x5e2db4b0, 61 },
    { 0x5b8db4b0, 90 },
    { 0, 0 },
    { 0xccedb4b3, 33 },
    { 0, 0 },
    { 0x7b9db4b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4db4ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3edb4bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdb4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eddb4c3, 95 },
    { 0xd09db4c4, 80 },
    { 0, 0 },
    { 0xcf1db4c6, 76 },
    { 0x1a6db4c6, 95 },
    { 0, 0 },
    { 0xd34db4c9, 55 },
    { 0x718db4ca, 90 },
    { 0xcb2db4cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245db4d6, 95 },
    { 0x547db4d7, 90 },
    { 0, 0 },
    { 0x488db4d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601db4dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbddb4e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63db4ea, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4db4ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6db4f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362db4f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527db4fb, 23 },
    { 0, 0 },
    { 0x8eadb4fd, 33 },
    { 0xdd2db4fe, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9addb507, 34 },
    { 0x6b8db507, 55 },
    { 0x262db509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478db510, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335db51b, 43 },
    { 0x4addb51c, 33 },
    { 0xa48db51b, 23 },
    { 0x906db51e, 95 },
    { 0, 0 },
    { 0x996db520, 91 },
    { 0x99adb521, 23 },
    { 0x388db522, 24 },
    { 0, 0 },
    { 0x723db524, 95 },
    { 0x405db525, 66 },
    { 0xe3ddb524, 98 },
    { 0x9c4db526, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcdb52a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x257db52d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1db534, 61 },
    { 0xb6edb534, 76 },
    { 0, 0 },
    { 0xb05db537, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50db53a, 55 },
    { 0x8aadb53b, 43 },
    { 0, 0 },
    { 0x624db53d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05db541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42adb548, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfadb54d, 88 },
    { 0, 0 },
    { 0xe58db54f, 90 },
    { 0, 0 },
    { 0x742db551, 76 },
    { 0x207db552, 99 },
    { 0x932db552, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7db55b, 55 },
    { 0, 0 },
    { 0x6a5db55d, 93 },
    { 0x819db55d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbdb564, 40 },
    { 0x2dedb565, 98 },
    { 0x35ddb566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcdb56a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52db56e, 34 },
    { 0x36ddb56f, 23 },
    { 0x65edb56f, 98 },
    { 0x60edb56f, 67 },
    { 0x7f9db572, 91 },
    { 0x79edb572, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8db57a, 90 },
    { 0x7cddb57b, 36 },
    { 0x332db57a, 43 },
    { 0x3a1db57d, 81 },
    { 0xd9bdb57e, 90 },
    { 0x65adb57d, 33 },
    { 0xf85db57e, 90 },
    { 0, 0 },
    { 0xb9fdb582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ddb588, 33 },
    { 0xf14db589, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbdb58c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137db590, 23 },
    { 0x262db590, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x283db594, 46 },
    { 0xb97db594, 23 },
    { 0xb3bdb596, 95 },
    { 0, 0 },
    { 0x2cbdb598, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1adb5a9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdb5ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cedb5b2, 67 },
    { 0xfe4db5b3, 78 },
    { 0x9c0db5b3, 91 },
    { 0, 0 },
    { 0xd0fdb5b6, 65 },
    { 0xa00db5b6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7addb5ba, 90 },
    { 0x42db5bb, 98 },
    { 0xe05db5bc, 23 },
    { 0xd18db5bc, 61 },
    { 0xbd8db5be, 61 },
    { 0x70fdb5be, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8db5c5, 23 },
    { 0, 0 },
    { 0xf44db5c7, 65 },
    { 0, 0 },
    { 0x903db5c9, 46 },
    { 0x406db5c9, 48 },
    { 0, 0 },
    { 0xa40db5cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x758db5cf, 78 },
    { 0, 0 },
    { 0x372db5d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bdb5d5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fdb5d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83db5de, 81 },
    { 0xd6edb5df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929db5e3, 61 },
    { 0x71db5e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdb5e7, 33 },
    { 0xd63db5e7, 75 },
    { 0xba2db5e7, 75 },
    { 0x979db5e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76db5ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601db5f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efdb5f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0db603, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4db607, 90 },
    { 0x586db607, 95 },
    { 0x281db609, 76 },
    { 0x585db60a, 93 },
    { 0x799db60b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ddb60e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296db612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36db615, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0db619, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fdb61e, 33 },
    { 0x36cdb61f, 52 },
    { 0x6d3db61f, 23 },
    { 0, 0 },
    { 0x9d8db622, 55 },
    { 0xac6db622, 91 },
    { 0, 0 },
    { 0xa7adb625, 90 },
    { 0x318db626, 75 },
    { 0, 0 },
    { 0xb84db628, 100 },
    { 0x27db629, 90 },
    { 0xa03db629, 33 },
    { 0x574db629, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0db62e, 85 },
    { 0xfa7db62e, 90 },
    { 0x12ddb62e, 76 },
    { 0, 0 },
    { 0xb12db632, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0db63a, 33 },
    { 0, 0 },
    { 0x771db63c, 87 },
    { 0xcf2db63c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x36db640, 100 },
    { 0x5dddb641, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adb645, 66 },
    { 0x8e0db646, 91 },
    { 0xc48db647, 67 },
    { 0xd8adb646, 95 },
    { 0xdd8db649, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0db64f, 23 },
    { 0xaa5db650, 90 },
    { 0, 0 },
    { 0xa1edb652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492db65b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3db65f, 76 },
    { 0, 0 },
    { 0x1cadb661, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabdb66c, 61 },
    { 0, 0 },
    { 0x9a2db66e, 46 },
    { 0xddbdb66f, 87 },
    { 0x527db66f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdb678, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddb67b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5db680, 95 },
    { 0, 0 },
    { 0xbbbdb682, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66db686, 95 },
    { 0x8d5db687, 43 },
    { 0xa44db688, 80 },
    { 0, 0 },
    { 0xbf1db68a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556db68e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7db691, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6adb695, 90 },
    { 0x3c0db696, 76 },
    { 0x9d1db696, 43 },
    { 0x7d4db698, 94 },
    { 0xdddb698, 46 },
    { 0xa31db69a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556db6a2, 90 },
    { 0x4b3db6a3, 61 },
    { 0x34edb6a2, 95 },
    { 0x1d9db6a5, 93 },
    { 0x68bdb6a6, 90 },
    { 0xfa3db6a4, 55 },
    { 0, 0 },
    { 0xf6adb6a9, 90 },
    { 0, 0 },
    { 0xc9edb6ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319db6af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cddb6b2, 90 },
    { 0, 0 },
    { 0x17adb6b4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5db6b7, 95 },
    { 0, 0 },
    { 0x473db6b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133db6c0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb6c3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6db6c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebdb6d2, 95 },
    { 0x3bfdb6d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbdb6d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5db6de, 33 },
    { 0xb6cdb6df, 91 },
    { 0x102db6df, 61 },
    { 0x6bddb6e1, 66 },
    { 0x771db6e0, 33 },
    { 0, 0 },
    { 0x1fddb6e4, 100 },
    { 0, 0 },
    { 0x478db6e6, 90 },
    { 0xcd8db6e7, 23 },
    { 0x9ecdb6e7, 23 },
    { 0xb5fdb6e7, 95 },
    { 0x6c1db6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bdb6ed, 33 },
    { 0x589db6ed, 61 },
    { 0xd98db6ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bdb6f2, 90 },
    { 0, 0 },
    { 0xd5adb6f4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55adb6f8, 98 },
    { 0, 0 },
    { 0xd61db6fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cdb700, 76 },
    { 0, 0 },
    { 0xcc4db702, 90 },
    { 0xeb4db703, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201db70b, 33 },
    { 0, 0 },
    { 0xf20db70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebdb710, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6db715, 93 },
    { 0xd75db716, 88 },
    { 0x66edb715, 17 },
    { 0xf9fdb718, 56 },
    { 0, 0 },
    { 0x4c9db71a, 85 },
    { 0x8fbdb71b, 91 },
    { 0x243db71b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cdb72a, 95 },
    { 0, 0 },
    { 0x68ddb72c, 76 },
    { 0x96edb72d, 76 },
    { 0x666db72e, 40 },
    { 0x728db72f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fdb737, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4adb73a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60adb73d, 77 },
    { 0, 0 },
    { 0x568db73f, 33 },
    { 0x34db740, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddb743, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614db747, 84 },
    { 0xf63db748, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859db74e, 23 },
    { 0, 0 },
    { 0x8db750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x250db753, 85 },
    { 0xde9db754, 98 },
    { 0xe0db755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75db75a, 52 },
    { 0x522db75b, 91 },
    { 0x89fdb75c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x806db75f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2db76c, 81 },
    { 0x101db76c, 52 },
    { 0x5cdb76e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968db772, 43 },
    { 0x1e2db773, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921db779, 98 },
    { 0x953db77a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4db77f, 90 },
    { 0, 0 },
    { 0x289db781, 23 },
    { 0x409db782, 90 },
    { 0xdc1db783, 23 },
    { 0x974db784, 100 },
    { 0xc07db784, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cdb78c, 84 },
    { 0x7b2db78d, 38 },
    { 0, 0 },
    { 0xd9db78f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fdb794, 97 },
    { 0x4b2db794, 75 },
    { 0, 0 },
    { 0xf5edb797, 23 },
    { 0x5e7db798, 43 },
    { 0x62bdb798, 90 },
    { 0x528db79a, 55 },
    { 0xcfddb797, 43 },
    { 0x99bdb79c, 99 },
    { 0xc18db79d, 93 },
    { 0xd22db79e, 76 },
    { 0xe92db79a, 90 },
    { 0xafdb79f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72db7a3, 90 },
    { 0x19ddb7a4, 23 },
    { 0x274db7a5, 68 },
    { 0x2bfdb7a3, 55 },
    { 0x5c5db7a7, 61 },
    { 0x6edb7a8, 88 },
    { 0xa3bdb7a5, 61 },
    { 0, 0 },
    { 0x9dddb7ab, 76 },
    { 0x324db7ac, 46 },
    { 0x238db7ab, 76 },
    { 0xa0fdb7ab, 76 },
    { 0xcbddb7ab, 66 },
    { 0x69cdb7af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2db7b6, 55 },
    { 0x81ddb7b7, 61 },
    { 0x407db7b8, 76 },
    { 0x5fcdb7b8, 90 },
    { 0xb44db7b8, 61 },
    { 0x3e1db7ba, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x79adb7be, 75 },
    { 0x4ebdb7bf, 90 },
    { 0x10cdb7c0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7db7c3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdb7cc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9db7cf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53db7d2, 23 },
    { 0xc83db7d3, 90 },
    { 0x7a4db7d2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcdb7da, 90 },
    { 0x655db7da, 36 },
    { 0x316db7da, 84 },
    { 0xcbfdb7dd, 90 },
    { 0xa83db7de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffdb7e6, 55 },
    { 0x54db7e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cadb7ea, 33 },
    { 0xa6cdb7ea, 67 },
    { 0, 0 },
    { 0x26fdb7ed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240db7f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fedb7fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44db7fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28db800, 33 },
    { 0x764db801, 84 },
    { 0, 0 },
    { 0x6badb803, 92 },
    { 0x666db803, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66db809, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bdb811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdb816, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddb81d, 33 },
    { 0x6ffdb81e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x141db821, 23 },
    { 0xe0fdb822, 90 },
    { 0x425db823, 90 },
    { 0x6f4db823, 34 },
    { 0x776db824, 87 },
    { 0xeedb826, 34 },
    { 0, 0 },
    { 0xdc8db828, 87 },
    { 0xa91db829, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dadb82d, 33 },
    { 0x6b6db82e, 23 },
    { 0xdaddb82e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5db833, 99 },
    { 0xbfdb834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702db83c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700db844, 48 },
    { 0xd66db845, 23 },
    { 0x721db846, 90 },
    { 0xb3bdb845, 55 },
    { 0x2d9db848, 95 },
    { 0x3d7db845, 55 },
    { 0xcc7db849, 101 },
    { 0x38db848, 91 },
    { 0x322db84b, 64 },
    { 0, 0 },
    { 0x1fdb84e, 90 },
    { 0x1f2db84f, 91 },
    { 0xbc5db850, 91 },
    { 0xc29db851, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabdb854, 23 },
    { 0x24edb855, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbedb858, 99 },
    { 0xc21db859, 61 },
    { 0x1aadb85a, 33 },
    { 0, 0 },
    { 0x994db85c, 99 },
    { 0x54adb85d, 95 },
    { 0xa5edb85e, 84 },
    { 0x75bdb85f, 95 },
    { 0x576db860, 46 },
    { 0x726db860, 56 },
    { 0x656db862, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bdb865, 90 },
    { 0x77db866, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ddb872, 98 },
    { 0, 0 },
    { 0x168db874, 98 },
    { 0x81fdb875, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbdb879, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855db88e, 98 },
    { 0xedbdb88f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdb893, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0edb896, 87 },
    { 0xb69db897, 23 },
    { 0, 0 },
    { 0x486db899, 55 },
    { 0, 0 },
    { 0xfcbdb89b, 87 },
    { 0xfa6db89c, 90 },
    { 0x55edb89c, 61 },
    { 0x7d6db89d, 95 },
    { 0x455db89f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cdb8a4, 67 },
    { 0, 0 },
    { 0xda2db8a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacdb8a9, 67 },
    { 0xf1edb8a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cdb8af, 90 },
    { 0xfacdb8b0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376db8bb, 80 },
    { 0x3addb8bc, 61 },
    { 0xc86db8bc, 65 },
    { 0, 0 },
    { 0x11adb8bf, 33 },
    { 0xa8db8c0, 75 },
    { 0, 0 },
    { 0xefadb8c2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x37adb8c5, 95 },
    { 0xbcddb8c6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xecfdb8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffdb8cd, 34 },
    { 0xbe5db8ce, 66 },
    { 0x340db8cf, 95 },
    { 0xbe3db8cf, 90 },
    { 0x78ddb8cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4db8d7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4db8e4, 23 },
    { 0, 0 },
    { 0x343db8e6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbdb8eb, 23 },
    { 0, 0 },
    { 0x466db8ed, 84 },
    { 0x31ddb8ed, 33 },
    { 0xe57db8ef, 98 },
    { 0x4d0db8f0, 76 },
    { 0x57ddb8f0, 94 },
    { 0x8c2db8f1, 55 },
    { 0xd2db8f1, 23 },
    { 0xcf0db8f4, 100 },
    { 0, 0 },
    { 0x98db8f6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcdb8f9, 55 },
    { 0, 0 },
    { 0x68ddb8fb, 33 },
    { 0x671db8fb, 99 },
    { 0, 0 },
    { 0x86cdb8fe, 95 },
    { 0xfcddb8ff, 33 },
    { 0, 0 },
    { 0xb32db901, 85 },
    { 0xe95db902, 100 },
    { 0, 0 },
    { 0x2eedb904, 99 },
    { 0x2b4db905, 87 },
    { 0x821db906, 46 },
    { 0x94ddb906, 87 },
    { 0, 0 },
    { 0x9cbdb909, 40 },
    { 0, 0 },
    { 0x810db90b, 78 },
    { 0xa92db90b, 84 },
    { 0x921db90d, 98 },
    { 0x9c8db90c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8db914, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edb91d, 95 },
    { 0xe2adb91e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ddb921, 98 },
    { 0x13ddb922, 55 },
    { 0x773db921, 96 },
    { 0x957db923, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x990db927, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3db92a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9db92e, 23 },
    { 0x4bfdb92e, 43 },
    { 0x8b2db92e, 76 },
    { 0x65adb92e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23db93a, 23 },
    { 0xfa8db93b, 99 },
    { 0x27db93a, 90 },
    { 0x2dadb93c, 46 },
    { 0x1f5db93e, 23 },
    { 0x28edb93f, 53 },
    { 0xbc4db940, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbdb946, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9db949, 95 },
    { 0x1edb94a, 33 },
    { 0xafddb94b, 4 },
    { 0xb4db94a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20db950, 95 },
    { 0, 0 },
    { 0xb88db952, 76 },
    { 0x15db953, 91 },
    { 0xae2db952, 90 },
    { 0x37cdb955, 90 },
    { 0x8c0db952, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fdb959, 90 },
    { 0, 0 },
    { 0x84adb95b, 33 },
    { 0, 0 },
    { 0x5c0db95d, 75 },
    { 0, 0 },
    { 0xae9db95f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7db965, 66 },
    { 0xd00db966, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fdb96d, 90 },
    { 0, 0 },
    { 0x1d6db96f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0db972, 33 },
    { 0x1a1db973, 43 },
    { 0xd67db974, 67 },
    { 0x25bdb975, 95 },
    { 0x690db976, 68 },
    { 0x9dbdb972, 23 },
    { 0x43bdb978, 95 },
    { 0xab5db978, 80 },
    { 0xd9cdb97a, 46 },
    { 0x915db979, 25 },
    { 0x1a2db97c, 98 },
    { 0, 0 },
    { 0x640db97e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efdb981, 95 },
    { 0xafddb982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98db987, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484db99c, 95 },
    { 0, 0 },
    { 0x56bdb99e, 75 },
    { 0, 0 },
    { 0x247db9a0, 46 },
    { 0xcb7db9a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57db9a4, 46 },
    { 0xd08db9a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecadb9b3, 90 },
    { 0xddbdb9b4, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83edb9b9, 95 },
    { 0x3ffdb9ba, 78 },
    { 0xde8db9bb, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862db9c0, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74db9c4, 100 },
    { 0xac1db9c5, 87 },
    { 0, 0 },
    { 0xa61db9c7, 65 },
    { 0xe50db9c8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64db9d2, 23 },
    { 0xdaddb9d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdb9d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fdb9d9, 17 },
    { 0x15fdb9d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20db9dd, 33 },
    { 0xb5fdb9de, 61 },
    { 0, 0 },
    { 0x1a6db9e0, 95 },
    { 0xabcdb9e1, 66 },
    { 0x121db9e2, 91 },
    { 0, 0 },
    { 0xa41db9e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9db9e7, 80 },
    { 0x824db9e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fadb9eb, 90 },
    { 0xe2bdb9ec, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8adb9ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71db9f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91edb9fe, 34 },
    { 0x9f0db9ff, 93 },
    { 0x687dba00, 61 },
    { 0, 0 },
    { 0xc43dba02, 33 },
    { 0x7cfdba03, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecdba07, 23 },
    { 0x77adba07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8dba0b, 85 },
    { 0x1dbdba0c, 100 },
    { 0x9ecdba0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409dba11, 95 },
    { 0, 0 },
    { 0x9d4dba13, 61 },
    { 0x5a2dba14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bdba17, 23 },
    { 0xa98dba18, 90 },
    { 0xc2adba17, 33 },
    { 0xe17dba18, 94 },
    { 0xcbcdba1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4adba21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bdba25, 90 },
    { 0x22edba25, 61 },
    { 0, 0 },
    { 0xfd3dba28, 90 },
    { 0xf66dba29, 84 },
    { 0xdaadba2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cdba2d, 68 },
    { 0, 0 },
    { 0x59cdba2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8dba32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fdba36, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x390dba39, 76 },
    { 0, 0 },
    { 0x39fdba3b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99dba3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ddba42, 95 },
    { 0x40cdba42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dba46, 33 },
    { 0x2f3dba47, 98 },
    { 0xcf8dba47, 90 },
    { 0xc3ddba49, 76 },
    { 0x1bedba48, 34 },
    { 0xf5fdba49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9dba4e, 78 },
    { 0x566dba4f, 55 },
    { 0x2ebdba4e, 43 },
    { 0x4b0dba51, 75 },
    { 0x6a5dba52, 23 },
    { 0x152dba53, 99 },
    { 0x4b6dba54, 33 },
    { 0xec5dba55, 61 },
    { 0x9b3dba56, 43 },
    { 0x6b9dba53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2dba5c, 95 },
    { 0x992dba5c, 87 },
    { 0x777dba5d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4dba61, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49dba66, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x33dba69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebdba6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3dba75, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdba79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cadba7d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731dba81, 90 },
    { 0xdcfdba82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901dba89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7dba8e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6addba91, 53 },
    { 0xfe6dba92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3adba95, 90 },
    { 0xeaddba96, 46 },
    { 0xf71dba97, 77 },
    { 0x2b2dba97, 33 },
    { 0x7e1dba99, 66 },
    { 0x961dba98, 23 },
    { 0x456dba97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765dbaa1, 52 },
    { 0xc1adbaa2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1edbaa5, 61 },
    { 0, 0 },
    { 0x23fdbaa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810dbaaf, 76 },
    { 0, 0 },
    { 0x4d3dbab1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69dbab6, 52 },
    { 0x80adbab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76edbaba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ddbabd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fdbac0, 17 },
    { 0xcb6dbac1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5dbac5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x602dbac8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffdbacc, 100 },
    { 0x987dbacd, 17 },
    { 0xb1edbacd, 23 },
    { 0xcbadbacd, 93 },
    { 0, 0 },
    { 0xabdbad1, 61 },
    { 0xf73dbad2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ddbad7, 43 },
    { 0, 0 },
    { 0xa60dbad9, 98 },
    { 0, 0 },
    { 0x3c8dbadb, 55 },
    { 0x93dbadc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42dbadf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebdbae4, 88 },
    { 0xd42dbae4, 90 },
    { 0xd78dbae5, 95 },
    { 0x472dbae6, 76 },
    { 0x617dbae8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cdbaf0, 85 },
    { 0xfa0dbaf0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1addbaf4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dbaf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945dbafc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3edbaff, 98 },
    { 0x123dbb00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25dbb07, 93 },
    { 0, 0 },
    { 0xaf1dbb09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3adbb11, 68 },
    { 0xc37dbb11, 95 },
    { 0x68fdbb11, 95 },
    { 0xccddbb12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1dbb18, 33 },
    { 0, 0 },
    { 0xf61dbb1a, 95 },
    { 0xd2cdbb1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68edbb1e, 56 },
    { 0x116dbb1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4dbb26, 93 },
    { 0x6d9dbb27, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadedbb2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538dbb31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abdbb34, 93 },
    { 0, 0 },
    { 0x39adbb36, 65 },
    { 0, 0 },
    { 0x986dbb38, 33 },
    { 0x3c4dbb39, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11dbb3c, 68 },
    { 0x55dbb3d, 23 },
    { 0x71ddbb3d, 98 },
    { 0x708dbb3f, 91 },
    { 0xd4edbb40, 36 },
    { 0x1efdbb41, 43 },
    { 0, 0 },
    { 0xd71dbb43, 66 },
    { 0, 0 },
    { 0x13edbb45, 55 },
    { 0, 0 },
    { 0x483dbb47, 76 },
    { 0xf42dbb48, 61 },
    { 0xf1adbb49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbdbb4c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdbb52, 76 },
    { 0, 0 },
    { 0x35cdbb54, 61 },
    { 0, 0 },
    { 0x180dbb56, 23 },
    { 0xf8cdbb57, 90 },
    { 0x698dbb58, 87 },
    { 0, 0 },
    { 0x9e2dbb5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08dbb63, 93 },
    { 0x9a4dbb63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6dbb67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fdbb6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fdbb6f, 90 },
    { 0x950dbb70, 90 },
    { 0xd4bdbb71, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ddbb76, 34 },
    { 0, 0 },
    { 0x6a7dbb78, 33 },
    { 0x6dbb79, 93 },
    { 0xc22dbb7a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9dbb81, 61 },
    { 0, 0 },
    { 0xa51dbb83, 88 },
    { 0, 0 },
    { 0x1d9dbb85, 43 },
    { 0x621dbb86, 23 },
    { 0, 0 },
    { 0xa25dbb88, 82 },
    { 0, 0 },
    { 0x620dbb8a, 23 },
    { 0xd6adbb8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5dbb90, 55 },
    { 0xcd8dbb91, 23 },
    { 0x9bdbb91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefedbb9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212dbba2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x472dbba5, 99 },
    { 0, 0 },
    { 0x992dbba7, 76 },
    { 0x9c3dbba7, 84 },
    { 0, 0 },
    { 0x5e3dbbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868dbbaf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460dbbb5, 23 },
    { 0x20bdbbb6, 61 },
    { 0xb73dbbb7, 95 },
    { 0, 0 },
    { 0x66ddbbb9, 66 },
    { 0x6dfdbbb9, 25 },
    { 0, 0 },
    { 0x620dbbbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499dbbc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784dbbc6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cdbbcc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4dbbd3, 77 },
    { 0xc72dbbd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x963dbbd7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82adbbdf, 90 },
    { 0, 0 },
    { 0x9b0dbbe1, 95 },
    { 0, 0 },
    { 0x623dbbe3, 33 },
    { 0xd69dbbe3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x416dbbe7, 33 },
    { 0x371dbbe8, 68 },
    { 0x7f1dbbe8, 55 },
    { 0x538dbbe9, 99 },
    { 0xbddbbe7, 84 },
    { 0x4b4dbbec, 36 },
    { 0x96dbbed, 43 },
    { 0xc96dbbe9, 40 },
    { 0x276dbbea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32dbbf4, 68 },
    { 0x2b5dbbf5, 23 },
    { 0x8d8dbbf6, 34 },
    { 0xf38dbbf7, 85 },
    { 0x5e2dbbf4, 99 },
    { 0x4abdbbf9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfcdbbfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816dbc00, 61 },
    { 0, 0 },
    { 0x6a4dbc02, 77 },
    { 0, 0 },
    { 0x548dbc04, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38dbc0b, 98 },
    { 0x343dbc0c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451dbc12, 92 },
    { 0x273dbc13, 99 },
    { 0x1f0dbc13, 23 },
    { 0x18bdbc15, 76 },
    { 0x1e8dbc16, 33 },
    { 0x846dbc17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dbc1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625dbc20, 76 },
    { 0x280dbc20, 34 },
    { 0x249dbc21, 78 },
    { 0x850dbc21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6dbc27, 33 },
    { 0xe26dbc27, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edbc2c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x823dbc2f, 76 },
    { 0, 0 },
    { 0x990dbc31, 90 },
    { 0, 0 },
    { 0x9f8dbc33, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3badbc38, 96 },
    { 0, 0 },
    { 0x960dbc3a, 93 },
    { 0x80cdbc3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386dbc41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336dbc46, 23 },
    { 0xf85dbc46, 61 },
    { 0x2c7dbc48, 87 },
    { 0xa4edbc49, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58adbc4e, 34 },
    { 0xd7dbc4f, 33 },
    { 0xebadbc4e, 43 },
    { 0x263dbc51, 52 },
    { 0, 0 },
    { 0x206dbc53, 80 },
    { 0x1dfdbc53, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2adbc65, 99 },
    { 0xdaddbc65, 95 },
    { 0, 0 },
    { 0x65fdbc68, 95 },
    { 0xe9bdbc68, 66 },
    { 0xe34dbc69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1dbc6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36dbc75, 90 },
    { 0, 0 },
    { 0x9f3dbc77, 33 },
    { 0, 0 },
    { 0xdf2dbc79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x887dbc7c, 76 },
    { 0, 0 },
    { 0xa4bdbc7e, 77 },
    { 0x69cdbc7f, 33 },
    { 0x861dbc80, 81 },
    { 0x48edbc81, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2dbc8c, 76 },
    { 0x160dbc8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcdbc91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938dbc99, 96 },
    { 0x2addbc9a, 76 },
    { 0x57dbc99, 55 },
    { 0x86adbc9b, 46 },
    { 0xc6cdbc9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadbca3, 95 },
    { 0xec3dbca3, 98 },
    { 0x706dbca5, 61 },
    { 0x912dbca6, 61 },
    { 0, 0 },
    { 0x6b9dbca8, 33 },
    { 0xc45dbca8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cdbcaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985dbcb4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabdbcbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cddbcbe, 95 },
    { 0xe09dbcbe, 46 },
    { 0xcddbcc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77dbcc3, 61 },
    { 0x8c3dbcc4, 90 },
    { 0, 0 },
    { 0x672dbcc6, 34 },
    { 0x53cdbcc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48fdbcd1, 55 },
    { 0x4aadbcd1, 93 },
    { 0x61edbcd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7dbcd9, 34 },
    { 0x5ddbcda, 85 },
    { 0, 0 },
    { 0xf57dbcdc, 85 },
    { 0x93cdbcdd, 93 },
    { 0x8d8dbcde, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450dbce2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17dbceb, 94 },
    { 0, 0 },
    { 0x7b3dbced, 78 },
    { 0x51edbced, 95 },
    { 0x814dbced, 23 },
    { 0xa42dbcee, 100 },
    { 0xaf9dbcf1, 23 },
    { 0x4ddbcef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56edbcf6, 33 },
    { 0x147dbcf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ddbcfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bdbcfd, 91 },
    { 0, 0 },
    { 0xce0dbcff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffdbd04, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fdbd0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2dbd10, 84 },
    { 0xe2bdbd10, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cdbd15, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bddbd1a, 36 },
    { 0x4dbdbd1b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48dbd1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cdbd23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cadbd29, 91 },
    { 0x60adbd2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73dbd2d, 91 },
    { 0x8cdbd2e, 47 },
    { 0, 0 },
    { 0x4ccdbd30, 33 },
    { 0xd98dbd31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eadbd34, 90 },
    { 0xbdcdbd34, 61 },
    { 0x752dbd36, 76 },
    { 0x660dbd37, 66 },
    { 0x86adbd38, 23 },
    { 0, 0 },
    { 0x452dbd3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30dbd3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dbd42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x660dbd45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4dbd4b, 99 },
    { 0, 0 },
    { 0x1e5dbd4d, 95 },
    { 0x389dbd4e, 43 },
    { 0x344dbd4f, 33 },
    { 0xfefdbd50, 95 },
    { 0, 0 },
    { 0xd8fdbd52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fdbd58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bdbd61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ddbd66, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9dbd69, 55 },
    { 0, 0 },
    { 0x383dbd6b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbddbd70, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38dbd7a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x239dbd7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afdbd81, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4dbd86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86adbd8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995dbd90, 90 },
    { 0xa32dbd90, 90 },
    { 0, 0 },
    { 0xf23dbd93, 61 },
    { 0xedbdbd93, 65 },
    { 0xeaadbd95, 98 },
    { 0, 0 },
    { 0x329dbd97, 68 },
    { 0xdeedbd97, 61 },
    { 0x3c2dbd98, 61 },
    { 0x989dbd9a, 33 },
    { 0x6f6dbd9b, 34 },
    { 0x968dbd9c, 52 },
    { 0x3a9dbd9d, 23 },
    { 0xb4fdbd9a, 17 },
    { 0xdc3dbd9f, 33 },
    { 0x69adbd9f, 90 },
    { 0, 0 },
    { 0xb9bdbda2, 61 },
    { 0x81cdbda3, 33 },
    { 0x597dbda3, 61 },
    { 0x62edbda2, 99 },
    { 0, 0 },
    { 0xce8dbda7, 33 },
    { 0xc60dbda8, 91 },
    { 0xc44dbda9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808dbdb5, 43 },
    { 0x658dbdb5, 61 },
    { 0, 0 },
    { 0xa65dbdb8, 33 },
    { 0, 0 },
    { 0x92ddbdba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9dbdbd, 95 },
    { 0x7bddbdbd, 43 },
    { 0x37edbdbe, 98 },
    { 0xd62dbdbe, 95 },
    { 0xbffdbdbd, 93 },
    { 0x7d9dbdc2, 88 },
    { 0x32fdbdc2, 55 },
    { 0xe02dbdc3, 90 },
    { 0x6acdbdc5, 90 },
    { 0, 0 },
    { 0xf73dbdc7, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2dbdcb, 98 },
    { 0x3dedbdcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x43edbdcf, 33 },
    { 0, 0 },
    { 0x952dbdd1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bddbdd6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8dbdda, 34 },
    { 0, 0 },
    { 0x5e8dbddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58adbde0, 23 },
    { 0, 0 },
    { 0x1bdbde2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dbdea, 84 },
    { 0, 0 },
    { 0x292dbdec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79adbdf0, 33 },
    { 0x6c1dbdf0, 100 },
    { 0x288dbdf1, 98 },
    { 0xbdedbdf3, 90 },
    { 0x863dbdf0, 90 },
    { 0x239dbdf1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8dbdf8, 83 },
    { 0x423dbdf8, 85 },
    { 0, 0 },
    { 0xa65dbdfb, 95 },
    { 0x57fdbdfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x853dbdff, 93 },
    { 0x71adbe00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4dbe04, 87 },
    { 0, 0 },
    { 0xdbdbe06, 98 },
    { 0x5ffdbe07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffdbe0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x94edbe0d, 46 },
    { 0, 0 },
    { 0xacfdbe0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459dbe1c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddbe23, 23 },
    { 0, 0 },
    { 0xdc5dbe25, 43 },
    { 0x7e7dbe26, 23 },
    { 0x18cdbe26, 34 },
    { 0x755dbe28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x154dbe2b, 76 },
    { 0, 0 },
    { 0x527dbe2d, 90 },
    { 0x817dbe2e, 90 },
    { 0, 0 },
    { 0x31bdbe30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3dbe35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aedbe38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bdbe3b, 33 },
    { 0x8d4dbe3c, 40 },
    { 0x49adbe3b, 99 },
    { 0xd7bdbe3e, 61 },
    { 0, 0 },
    { 0x6d9dbe40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4dbe48, 90 },
    { 0, 0 },
    { 0xcc0dbe4a, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40edbe52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50dbe58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7dbe5b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x693dbe5e, 34 },
    { 0, 0 },
    { 0x872dbe60, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fdbe64, 33 },
    { 0xc2adbe65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cdbe6d, 90 },
    { 0x141dbe6d, 33 },
    { 0xa32dbe6f, 76 },
    { 0x5addbe6f, 23 },
    { 0x7d8dbe71, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dbe76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46edbe79, 84 },
    { 0x6c1dbe7a, 87 },
    { 0, 0 },
    { 0x98bdbe7c, 95 },
    { 0, 0 },
    { 0x588dbe7e, 90 },
    { 0x9b9dbe7f, 61 },
    { 0xe31dbe80, 90 },
    { 0xb5cdbe81, 99 },
    { 0xfc1dbe82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49dbe89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabddbe8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8dbe93, 98 },
    { 0xd6ddbe94, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708dbe99, 56 },
    { 0x9c9dbe9a, 33 },
    { 0x7a5dbe9b, 90 },
    { 0xd0fdbe9c, 100 },
    { 0xb40dbe9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193dbea1, 33 },
    { 0x716dbea2, 23 },
    { 0, 0 },
    { 0x152dbea4, 33 },
    { 0x3a4dbea5, 76 },
    { 0x52bdbea5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34dbeaa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9dbeb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0dbeb6, 46 },
    { 0x32adbeb7, 90 },
    { 0xe24dbeb8, 93 },
    { 0x1b4dbeb9, 90 },
    { 0x185dbeb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4bdbebe, 65 },
    { 0x9d5dbebf, 85 },
    { 0, 0 },
    { 0x40adbec1, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66dbec9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabddbecf, 40 },
    { 0, 0 },
    { 0x144dbed1, 96 },
    { 0, 0 },
    { 0xf67dbed3, 93 },
    { 0, 0 },
    { 0x9c7dbed5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x231dbed8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434dbedd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcdbeeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x554dbeee, 98 },
    { 0x938dbeee, 76 },
    { 0xe1dbef0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafedbef3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437dbefb, 88 },
    { 0xc0bdbefc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3dbeff, 34 },
    { 0, 0 },
    { 0x8d6dbf01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12dbf04, 96 },
    { 0x608dbf04, 36 },
    { 0, 0 },
    { 0x8d6dbf07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fdbf0d, 93 },
    { 0, 0 },
    { 0x683dbf0f, 100 },
    { 0, 0 },
    { 0xa34dbf11, 95 },
    { 0x383dbf12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1dbf17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ddbf1b, 65 },
    { 0, 0 },
    { 0x97dbf1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaedbf20, 67 },
    { 0x4eddbf21, 95 },
    { 0x1c8dbf21, 38 },
    { 0, 0 },
    { 0x670dbf24, 43 },
    { 0, 0 },
    { 0xe80dbf26, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dadbf29, 92 },
    { 0x47ddbf2a, 98 },
    { 0x878dbf2a, 85 },
    { 0xc8bdbf2c, 95 },
    { 0x1efdbf2d, 85 },
    { 0x8dbdbf2d, 66 },
    { 0x7b6dbf2f, 85 },
    { 0x250dbf29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645dbf34, 76 },
    { 0x9d6dbf35, 99 },
    { 0x3acdbf36, 90 },
    { 0x8c0dbf37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x126dbf3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91dbf3f, 98 },
    { 0, 0 },
    { 0xe1dbf41, 95 },
    { 0, 0 },
    { 0x2f3dbf43, 99 },
    { 0, 0 },
    { 0x924dbf45, 61 },
    { 0x36bdbf46, 85 },
    { 0x66bdbf47, 52 },
    { 0x25edbf48, 33 },
    { 0, 0 },
    { 0x7c7dbf4a, 77 },
    { 0xf1fdbf4a, 53 },
    { 0xf19dbf4b, 61 },
    { 0x96adbf4b, 61 },
    { 0xfd4dbf4d, 90 },
    { 0x8f8dbf4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8dbf52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5dbf55, 38 },
    { 0x607dbf56, 76 },
    { 0, 0 },
    { 0xd04dbf58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166dbf5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737dbf61, 95 },
    { 0xa04dbf62, 77 },
    { 0, 0 },
    { 0xf8cdbf64, 61 },
    { 0x3f0dbf65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28dbf68, 23 },
    { 0x365dbf68, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadbf6c, 84 },
    { 0xe7dbf6d, 33 },
    { 0, 0 },
    { 0xa22dbf6f, 98 },
    { 0x9b1dbf70, 95 },
    { 0x42cdbf71, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cdbf76, 85 },
    { 0, 0 },
    { 0xc6fdbf78, 98 },
    { 0x70edbf79, 76 },
    { 0xe75dbf7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bdbf7d, 98 },
    { 0, 0 },
    { 0x830dbf7f, 90 },
    { 0x465dbf7f, 68 },
    { 0xe01dbf81, 100 },
    { 0x860dbf82, 90 },
    { 0, 0 },
    { 0xbe2dbf84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17dbf8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1dbf90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ddbf99, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0x703dbf9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62dbf9f, 61 },
    { 0, 0 },
    { 0x6bfdbfa1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fdbfa6, 90 },
    { 0, 0 },
    { 0x5ccdbfa8, 85 },
    { 0x915dbfa9, 76 },
    { 0x8e1dbfaa, 23 },
    { 0x6addbfaa, 78 },
    { 0, 0 },
    { 0xe91dbfad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3dbfb7, 33 },
    { 0xf15dbfb8, 98 },
    { 0x19bdbfb9, 55 },
    { 0xadedbfba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76dbfbd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x864dbfc0, 76 },
    { 0, 0 },
    { 0x459dbfc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aadbfc9, 23 },
    { 0x8c3dbfca, 55 },
    { 0, 0 },
    { 0xaa8dbfcc, 55 },
    { 0, 0 },
    { 0x484dbfce, 96 },
    { 0, 0 },
    { 0x3e8dbfd0, 90 },
    { 0, 0 },
    { 0xb2ddbfd2, 76 },
    { 0, 0 },
    { 0xf42dbfd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0dbfd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfdbfe2, 78 },
    { 0x30dbfe3, 34 },
    { 0x5d7dbfe2, 98 },
    { 0x5c5dbfe5, 98 },
    { 0xf05dbfe6, 66 },
    { 0x28dbfe5, 43 },
    { 0x54cdbfe8, 98 },
    { 0xdb0dbfe9, 90 },
    { 0x8dddbfea, 95 },
    { 0xed9dbfe9, 90 },
    { 0x5b5dbfec, 90 },
    { 0xa2fdbfed, 99 },
    { 0x94adbfed, 23 },
    { 0xcf7dbfee, 100 },
    { 0x74cdbfed, 61 },
    { 0xb13dbfee, 98 },
    { 0x560dbff2, 61 },
    { 0x2c3dbff3, 90 },
    { 0x630dbff4, 78 },
    { 0xf7adbfee, 23 },
    { 0x3e0dbff0, 34 },
    { 0xc32dbff7, 90 },
    { 0x8e3dbff2, 98 },
    { 0xa65dbfee, 65 },
    { 0xa56dbff3, 99 },
    { 0x7d6dbffb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x100dbffe, 95 },
    { 0xc42dbffe, 95 },
    { 0xf90dbffe, 46 },
    { 0x703dc001, 43 },
    { 0x2fcdc002, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2adc006, 90 },
    { 0xb88dc006, 55 },
    { 0x268dc007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bdc00f, 87 },
    { 0, 0 },
    { 0xe59dc011, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8dc014, 55 },
    { 0, 0 },
    { 0x244dc016, 95 },
    { 0, 0 },
    { 0xb70dc018, 43 },
    { 0xd40dc019, 33 },
    { 0x663dc018, 90 },
    { 0, 0 },
    { 0x614dc01c, 90 },
    { 0xc87dc01d, 61 },
    { 0x7bfdc01e, 98 },
    { 0x5f8dc01f, 22 },
    { 0x801dc01f, 61 },
    { 0x57ddc021, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0dc027, 95 },
    { 0x6eadc027, 67 },
    { 0x1f2dc028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefdc02d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6dc030, 66 },
    { 0xef3dc031, 90 },
    { 0, 0 },
    { 0x23ddc033, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00dc036, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdc039, 98 },
    { 0, 0 },
    { 0xacfdc03b, 75 },
    { 0, 0 },
    { 0x709dc03d, 55 },
    { 0x76edc03e, 90 },
    { 0, 0 },
    { 0x31adc040, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124dc048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdc04b, 75 },
    { 0, 0 },
    { 0xcb5dc04d, 61 },
    { 0x946dc04e, 90 },
    { 0x613dc04f, 98 },
    { 0, 0 },
    { 0x93cdc051, 98 },
    { 0xa05dc052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cdc057, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17adc05d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1dc060, 33 },
    { 0xd1bdc061, 61 },
    { 0, 0 },
    { 0xb3dc063, 76 },
    { 0xe48dc064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0dc067, 95 },
    { 0xa1ddc068, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9dc06f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4dc073, 33 },
    { 0, 0 },
    { 0x4c5dc075, 87 },
    { 0xe6edc075, 90 },
    { 0xf15dc077, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bdc07b, 98 },
    { 0x32dc07c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401dc087, 34 },
    { 0xab1dc087, 76 },
    { 0x256dc089, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dc08d, 33 },
    { 0, 0 },
    { 0x28ddc08f, 66 },
    { 0x9ebdc090, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x458dc093, 23 },
    { 0, 0 },
    { 0x7d3dc095, 90 },
    { 0xce4dc096, 100 },
    { 0x3a3dc096, 76 },
    { 0, 0 },
    { 0xf2ddc099, 43 },
    { 0x3c3dc09a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476dc09f, 87 },
    { 0xfd4dc0a0, 33 },
    { 0xdefdc09f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cdc0a7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0dc0b0, 85 },
    { 0xec5dc0b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06dc0b4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91dc0b7, 65 },
    { 0x7fbdc0b8, 34 },
    { 0xc7cdc0b9, 61 },
    { 0, 0 },
    { 0xa52dc0bb, 95 },
    { 0x9f1dc0bc, 23 },
    { 0x37bdc0bb, 23 },
    { 0x1c3dc0bb, 33 },
    { 0xeeedc0bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddc0c5, 23 },
    { 0x70ddc0c6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x558dc0c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5dc0cc, 87 },
    { 0x73dc0cc, 61 },
    { 0xd05dc0cc, 23 },
    { 0x77dc0cc, 55 },
    { 0xd9ddc0d0, 90 },
    { 0x486dc0d1, 95 },
    { 0x810dc0d1, 78 },
    { 0, 0 },
    { 0x1f0dc0d4, 93 },
    { 0x285dc0d4, 33 },
    { 0xe6ddc0d6, 61 },
    { 0xd78dc0d7, 95 },
    { 0xb9dc0d7, 33 },
    { 0x47fdc0d9, 77 },
    { 0xc90dc0d4, 76 },
    { 0x4f1dc0da, 96 },
    { 0x205dc0db, 98 },
    { 0x334dc0dd, 66 },
    { 0xe71dc0dd, 43 },
    { 0x3bdc0dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3dc0e2, 90 },
    { 0, 0 },
    { 0x2b2dc0e4, 33 },
    { 0xdaedc0e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x44adc0e8, 94 },
    { 0x249dc0e9, 95 },
    { 0x21fdc0ea, 31 },
    { 0xccedc0e8, 61 },
    { 0xc72dc0ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90dc0f2, 90 },
    { 0xb04dc0f3, 96 },
    { 0x785dc0f4, 88 },
    { 0x718dc0f5, 90 },
    { 0, 0 },
    { 0xc2edc0f7, 76 },
    { 0, 0 },
    { 0xb70dc0f9, 100 },
    { 0, 0 },
    { 0x5c4dc0fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ddc0fe, 61 },
    { 0xda2dc0ff, 96 },
    { 0x36ddc0ff, 90 },
    { 0xf69dc0ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bdc109, 33 },
    { 0, 0 },
    { 0xdf5dc10b, 23 },
    { 0x35adc10c, 90 },
    { 0x85cdc10d, 61 },
    { 0xb89dc10e, 33 },
    { 0xc8cdc10e, 55 },
    { 0x861dc10f, 69 },
    { 0xf96dc10b, 93 },
    { 0x862dc112, 98 },
    { 0x1d7dc113, 34 },
    { 0xc79dc110, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3dc118, 95 },
    { 0, 0 },
    { 0xb40dc11a, 90 },
    { 0, 0 },
    { 0x931dc11c, 61 },
    { 0xfcddc11c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ddc120, 95 },
    { 0, 0 },
    { 0x1dddc122, 33 },
    { 0x208dc123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dc132, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbedc136, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x78edc139, 55 },
    { 0x2c9dc139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x203dc13d, 52 },
    { 0x61dc13e, 98 },
    { 0, 0 },
    { 0xc6fdc140, 33 },
    { 0x6f9dc140, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fedc146, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5dc14b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cdc14e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x890dc151, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859dc15a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x105dc15d, 90 },
    { 0xa22dc15d, 93 },
    { 0x4a5dc15f, 90 },
    { 0x556dc160, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744dc165, 17 },
    { 0x9cddc166, 95 },
    { 0xbf4dc167, 68 },
    { 0x314dc166, 34 },
    { 0, 0 },
    { 0x6e3dc16a, 61 },
    { 0x1b6dc16b, 90 },
    { 0, 0 },
    { 0x6cddc16d, 23 },
    { 0x2bcdc16e, 90 },
    { 0x3d1dc16e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x33adc172, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10adc176, 33 },
    { 0xd0cdc177, 91 },
    { 0xc5edc178, 90 },
    { 0x139dc179, 34 },
    { 0xceedc17a, 98 },
    { 0x145dc178, 85 },
    { 0x8c8dc17c, 95 },
    { 0xe8fdc17c, 33 },
    { 0x540dc17e, 90 },
    { 0xe14dc17f, 84 },
    { 0x83edc17f, 23 },
    { 0x7bcdc17e, 33 },
    { 0x99fdc180, 43 },
    { 0xae8dc183, 33 },
    { 0xfefdc184, 98 },
    { 0x2c0dc183, 61 },
    { 0xa78dc186, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3dc18b, 23 },
    { 0xcd4dc18b, 90 },
    { 0, 0 },
    { 0xed4dc18e, 91 },
    { 0xde6dc18f, 61 },
    { 0x643dc190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1dc19d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87dc1a1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xafddc1a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45dc1a7, 90 },
    { 0x6a7dc1a8, 85 },
    { 0x777dc1a9, 61 },
    { 0x7dddc1a7, 33 },
    { 0x527dc1ab, 90 },
    { 0xdddc1a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdc1af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0dc1b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80edc1b8, 90 },
    { 0xa89dc1b9, 34 },
    { 0x8f0dc1ba, 95 },
    { 0, 0 },
    { 0x48cdc1bc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2dc1bf, 95 },
    { 0xeedc1c0, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1dc1c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dc1cd, 75 },
    { 0, 0 },
    { 0xe92dc1cf, 55 },
    { 0, 0 },
    { 0xa0dc1d1, 33 },
    { 0xed1dc1d2, 46 },
    { 0xa18dc1d3, 46 },
    { 0x567dc1d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bdc1d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648dc1e0, 23 },
    { 0x67adc1e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cdc1e7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fdc1eb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fedc1ee, 90 },
    { 0, 0 },
    { 0x5e8dc1f0, 61 },
    { 0x7a9dc1f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fdc1f4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x463dc1f7, 95 },
    { 0, 0 },
    { 0x17fdc1f9, 98 },
    { 0x4addc1f9, 65 },
    { 0xe8edc1fa, 98 },
    { 0, 0 },
    { 0xda6dc1fd, 34 },
    { 0x49adc1fe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230dc204, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2dc207, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1dc20c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1dc211, 61 },
    { 0x6d6dc211, 90 },
    { 0x818dc213, 52 },
    { 0x93bdc213, 61 },
    { 0x8a8dc214, 98 },
    { 0x187dc214, 91 },
    { 0x69ddc214, 98 },
    { 0x99bdc215, 76 },
    { 0x165dc218, 95 },
    { 0x7eadc21a, 43 },
    { 0x4f7dc215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fddc21f, 90 },
    { 0, 0 },
    { 0x359dc221, 33 },
    { 0x1a1dc222, 95 },
    { 0x9dbdc223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4dc226, 66 },
    { 0x1dddc227, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xffadc22a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03dc22e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cdc232, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4dc236, 46 },
    { 0x546dc237, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2edc243, 98 },
    { 0, 0 },
    { 0xd5bdc245, 17 },
    { 0, 0 },
    { 0xf41dc247, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cdc254, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4dc25b, 33 },
    { 0, 0 },
    { 0x63ddc25d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942dc26d, 75 },
    { 0x54adc26e, 23 },
    { 0, 0 },
    { 0x58fdc270, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424dc277, 43 },
    { 0x40fdc278, 95 },
    { 0x170dc277, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecdc280, 95 },
    { 0x481dc281, 85 },
    { 0x2a6dc282, 95 },
    { 0x71cdc283, 76 },
    { 0xc41dc283, 98 },
    { 0x87fdc283, 61 },
    { 0xbfedc286, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x963dc289, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8dc291, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fdc294, 90 },
    { 0, 0 },
    { 0xaeedc296, 52 },
    { 0xf71dc296, 95 },
    { 0x255dc297, 61 },
    { 0xe6dc297, 33 },
    { 0, 0 },
    { 0x3d5dc29b, 90 },
    { 0xcf0dc29c, 66 },
    { 0x13cdc29d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bedc2a0, 93 },
    { 0x573dc2a1, 90 },
    { 0x77fdc2a2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130dc2a6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616dc2ab, 90 },
    { 0xc12dc2ac, 76 },
    { 0x752dc2ad, 68 },
    { 0xb06dc2ac, 76 },
    { 0x4eadc2ae, 43 },
    { 0x9d0dc2ac, 93 },
    { 0x110dc2b0, 98 },
    { 0x232dc2b2, 46 },
    { 0x14cdc2b2, 91 },
    { 0x352dc2b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948dc2b9, 84 },
    { 0, 0 },
    { 0x72bdc2bb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dc2bf, 55 },
    { 0xb86dc2c0, 61 },
    { 0, 0 },
    { 0xf03dc2c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7dc2c9, 61 },
    { 0, 0 },
    { 0x3d5dc2cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecdc2ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77edc2d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cdc2d9, 46 },
    { 0x6d6dc2da, 33 },
    { 0xd95dc2db, 34 },
    { 0x789dc2d9, 98 },
    { 0x891dc2dd, 61 },
    { 0x2e9dc2de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefdc2e1, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x365dc2e4, 67 },
    { 0x219dc2e4, 93 },
    { 0x1bbdc2e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24dc2ed, 68 },
    { 0xf1dc2ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929dc2f2, 61 },
    { 0, 0 },
    { 0x831dc2f4, 55 },
    { 0, 0 },
    { 0x804dc2f6, 95 },
    { 0xf82dc2f7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913dc2ff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dc303, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1dc312, 23 },
    { 0xf49dc313, 76 },
    { 0x8fddc312, 33 },
    { 0, 0 },
    { 0xf24dc316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bdc31a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9dc31d, 23 },
    { 0x7b4dc31e, 87 },
    { 0, 0 },
    { 0xf1fdc320, 23 },
    { 0, 0 },
    { 0x2f4dc322, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ddc329, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdedc32f, 65 },
    { 0xf34dc330, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6badc333, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5dc336, 55 },
    { 0xe5bdc337, 23 },
    { 0xf09dc337, 77 },
    { 0x11dc339, 55 },
    { 0, 0 },
    { 0x5c1dc33b, 90 },
    { 0, 0 },
    { 0xda6dc33d, 33 },
    { 0x545dc33e, 46 },
    { 0, 0 },
    { 0x24adc340, 93 },
    { 0, 0 },
    { 0x4eedc342, 98 },
    { 0xf1ddc342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535dc347, 90 },
    { 0, 0 },
    { 0x344dc349, 55 },
    { 0xf18dc349, 98 },
    { 0x1fddc34b, 66 },
    { 0x207dc34b, 91 },
    { 0xb4adc34b, 23 },
    { 0x660dc34e, 98 },
    { 0x8b6dc34f, 61 },
    { 0x733dc34f, 46 },
    { 0x1fcdc34c, 12 },
    { 0xbd8dc352, 95 },
    { 0x5dfdc34b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29dc357, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70dc360, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4dc365, 81 },
    { 0x3dadc365, 47 },
    { 0xadbdc367, 33 },
    { 0x9bedc368, 95 },
    { 0x580dc369, 78 },
    { 0x11fdc369, 61 },
    { 0, 0 },
    { 0x222dc36c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93dc372, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120dc378, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79dc37e, 33 },
    { 0x553dc37f, 23 },
    { 0xcb8dc37f, 23 },
    { 0x638dc381, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7dc387, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4dc38a, 67 },
    { 0x85edc38a, 61 },
    { 0x214dc38a, 55 },
    { 0x2cfdc38d, 61 },
    { 0, 0 },
    { 0xcb7dc38f, 23 },
    { 0x37fdc38f, 23 },
    { 0, 0 },
    { 0x398dc392, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04dc395, 23 },
    { 0x686dc396, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7dc39c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466dc3a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1dc3a8, 95 },
    { 0, 0 },
    { 0x8efdc3aa, 34 },
    { 0x426dc3aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cdc3ae, 61 },
    { 0x33ddc3ae, 98 },
    { 0xde4dc3af, 33 },
    { 0x6a2dc3b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cdc3b8, 33 },
    { 0x681dc3b9, 76 },
    { 0xea2dc3b8, 94 },
    { 0x685dc3bb, 52 },
    { 0x214dc3b8, 46 },
    { 0x62fdc3b8, 100 },
    { 0, 0 },
    { 0x1b9dc3bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ddc3cc, 69 },
    { 0xd24dc3cd, 36 },
    { 0x4aedc3ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9dc3d4, 93 },
    { 0, 0 },
    { 0x3e3dc3d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbdc3de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dc3e1, 98 },
    { 0x989dc3e2, 61 },
    { 0x496dc3e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cdc3e6, 76 },
    { 0xc99dc3e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ddc3ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cdc3f0, 81 },
    { 0x5fddc3f0, 90 },
    { 0xfeadc3f2, 95 },
    { 0x567dc3f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470dc3f7, 66 },
    { 0xa88dc3f8, 76 },
    { 0x743dc3f9, 90 },
    { 0xa13dc3f9, 61 },
    { 0xdd0dc3f9, 33 },
    { 0, 0 },
    { 0x5e1dc3fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcdc402, 61 },
    { 0x158dc403, 90 },
    { 0, 0 },
    { 0xb28dc405, 90 },
    { 0x432dc406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586dc40c, 23 },
    { 0x665dc40c, 43 },
    { 0, 0 },
    { 0x3d9dc40f, 55 },
    { 0, 0 },
    { 0x3a0dc411, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbdc419, 56 },
    { 0x46bdc41a, 17 },
    { 0x67edc41b, 95 },
    { 0xac1dc41a, 90 },
    { 0xa83dc41d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3dc421, 81 },
    { 0, 0 },
    { 0xc42dc423, 33 },
    { 0, 0 },
    { 0xe86dc425, 23 },
    { 0x591dc426, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bdc42a, 76 },
    { 0x15dc42b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3dc42f, 88 },
    { 0x743dc430, 76 },
    { 0x7a4dc431, 90 },
    { 0, 0 },
    { 0x67adc433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbdc436, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcdc43e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x49edc441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6dc444, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488dc44e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199dc453, 90 },
    { 0, 0 },
    { 0x8b4dc455, 33 },
    { 0, 0 },
    { 0x42dc457, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611dc45b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1dc45e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25dc462, 98 },
    { 0, 0 },
    { 0x3bfdc464, 90 },
    { 0xbb9dc464, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27edc468, 90 },
    { 0xd76dc469, 95 },
    { 0x72dc46a, 76 },
    { 0xa8cdc46a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fdc46f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1dc472, 98 },
    { 0x6e7dc473, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cadc477, 98 },
    { 0xbc9dc478, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51dc482, 52 },
    { 0xfe6dc482, 87 },
    { 0, 0 },
    { 0xc40dc485, 42 },
    { 0x8eedc486, 98 },
    { 0xacfdc485, 99 },
    { 0xf60dc487, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6dc48c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbedc490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabedc494, 76 },
    { 0xb19dc495, 90 },
    { 0, 0 },
    { 0x58bdc497, 66 },
    { 0x726dc498, 23 },
    { 0xc6dc499, 55 },
    { 0, 0 },
    { 0x1eadc49b, 61 },
    { 0xb9cdc49b, 98 },
    { 0, 0 },
    { 0xa07dc49e, 23 },
    { 0x37edc49f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efdc4a3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6dc4a6, 88 },
    { 0xf17dc4a6, 77 },
    { 0x70fdc4a8, 95 },
    { 0x954dc4a9, 98 },
    { 0x3dadc4aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0dc4ad, 76 },
    { 0x7d9dc4ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cadc4b2, 90 },
    { 0xafbdc4b3, 69 },
    { 0x72fdc4b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ddc4b8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233dc4c3, 61 },
    { 0xbb0dc4c3, 77 },
    { 0xd2dc4c5, 55 },
    { 0x5fbdc4c5, 98 },
    { 0xcf4dc4c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8dc4ca, 66 },
    { 0, 0 },
    { 0xfd7dc4cc, 40 },
    { 0x6efdc4cd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bdc4d3, 90 },
    { 0xd9cdc4d4, 33 },
    { 0x9bbdc4d3, 90 },
    { 0xa5edc4d5, 78 },
    { 0x407dc4d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00dc4e3, 87 },
    { 0x4f2dc4e3, 46 },
    { 0, 0 },
    { 0xd53dc4e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4addc4ec, 95 },
    { 0x92adc4ed, 23 },
    { 0x8b8dc4ee, 55 },
    { 0x1eadc4ed, 61 },
    { 0xa13dc4ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85dc4f3, 56 },
    { 0x8f9dc4f4, 43 },
    { 0x287dc4f3, 95 },
    { 0x28dc4f5, 55 },
    { 0x47dc4f7, 61 },
    { 0x8e5dc4f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bddc4fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccdc4ff, 95 },
    { 0x49edc4ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809dc504, 85 },
    { 0x688dc504, 98 },
    { 0x883dc506, 90 },
    { 0x2c5dc507, 23 },
    { 0xe05dc507, 98 },
    { 0x430dc509, 17 },
    { 0xb6bdc506, 77 },
    { 0xaebdc50b, 90 },
    { 0x190dc50b, 98 },
    { 0x657dc50d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0dc511, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98dc517, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216dc51b, 61 },
    { 0, 0 },
    { 0xe7fdc51d, 91 },
    { 0, 0 },
    { 0xaa8dc51f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fdc529, 63 },
    { 0x2fbdc52a, 23 },
    { 0, 0 },
    { 0x91fdc52c, 23 },
    { 0x93adc52d, 23 },
    { 0x371dc52c, 98 },
    { 0, 0 },
    { 0xd82dc530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcdc537, 61 },
    { 0, 0 },
    { 0xac6dc539, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55dc53e, 100 },
    { 0, 0 },
    { 0x90adc540, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fdc543, 65 },
    { 0xd9dc544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acdc548, 90 },
    { 0x6fedc549, 66 },
    { 0xa80dc549, 23 },
    { 0x4dadc54b, 90 },
    { 0x40cdc548, 90 },
    { 0xc93dc549, 55 },
    { 0x591dc54e, 76 },
    { 0xc00dc54f, 90 },
    { 0xf93dc54f, 23 },
    { 0x86bdc54e, 90 },
    { 0x120dc54d, 75 },
    { 0x5dcdc553, 78 },
    { 0x738dc54d, 55 },
    { 0x532dc54c, 23 },
    { 0x280dc552, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fdc55c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4dc569, 81 },
    { 0xb8bdc569, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdc56e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94dc571, 34 },
    { 0x96edc572, 55 },
    { 0xdf6dc573, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cdc576, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5dc57a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50dc57e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefdc58a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8dc58d, 33 },
    { 0x46ddc58e, 74 },
    { 0x9d4dc58f, 65 },
    { 0xc38dc590, 100 },
    { 0xc6dc590, 85 },
    { 0xf40dc592, 52 },
    { 0x9dedc58d, 98 },
    { 0xfa6dc594, 90 },
    { 0x8f4dc594, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25edc598, 61 },
    { 0, 0 },
    { 0x437dc59a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49dc59d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27dc5a0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4dc5a3, 90 },
    { 0, 0 },
    { 0xc36dc5a5, 61 },
    { 0xfb3dc5a6, 90 },
    { 0xf60dc5a7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9edc5aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2dc5ae, 98 },
    { 0, 0 },
    { 0x575dc5b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29dc5b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9dc5b6, 90 },
    { 0xb5cdc5b6, 81 },
    { 0x344dc5b8, 61 },
    { 0xa0fdc5b8, 43 },
    { 0, 0 },
    { 0xff8dc5bb, 98 },
    { 0x6abdc5bc, 34 },
    { 0, 0 },
    { 0xa23dc5be, 23 },
    { 0xbe1dc5bf, 61 },
    { 0x2e2dc5be, 17 },
    { 0xfe7dc5c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fedc5c4, 67 },
    { 0x106dc5c4, 34 },
    { 0, 0 },
    { 0xe3dc5c7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bdc5cb, 100 },
    { 0x555dc5cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebdc5d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7dc5dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dddc5e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58dc5e3, 46 },
    { 0, 0 },
    { 0xc07dc5e5, 90 },
    { 0x7d8dc5e6, 43 },
    { 0x173dc5e6, 34 },
    { 0, 0 },
    { 0x10dc5e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1dc5ee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500dc5f2, 33 },
    { 0, 0 },
    { 0x562dc5f4, 34 },
    { 0xaf2dc5f5, 100 },
    { 0xe58dc5f6, 95 },
    { 0x55adc5f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4dc5fb, 33 },
    { 0, 0 },
    { 0x499dc5fd, 64 },
    { 0x273dc5fe, 23 },
    { 0x541dc5ff, 23 },
    { 0x132dc600, 67 },
    { 0x3dcdc601, 90 },
    { 0x9bddc5fd, 95 },
    { 0, 0 },
    { 0xc72dc604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefdc60a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0edc612, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55dc617, 78 },
    { 0x25fdc618, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dc61d, 95 },
    { 0x139dc61e, 52 },
    { 0x181dc61f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8dc622, 34 },
    { 0xaccdc623, 90 },
    { 0x3eedc624, 95 },
    { 0x9b7dc625, 76 },
    { 0, 0 },
    { 0x2f3dc627, 61 },
    { 0xd5fdc628, 23 },
    { 0x6a8dc629, 98 },
    { 0x7b7dc62a, 87 },
    { 0x4dadc629, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38dc631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcdc635, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7dc638, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37dc63b, 17 },
    { 0xc76dc63c, 90 },
    { 0xfbadc63c, 95 },
    { 0, 0 },
    { 0x776dc63f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7dc644, 43 },
    { 0x8fddc645, 43 },
    { 0x6d2dc644, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bedc64d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9dc657, 90 },
    { 0x9ddc658, 100 },
    { 0x68ddc659, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226dc65e, 95 },
    { 0x1eddc65f, 33 },
    { 0, 0 },
    { 0xdddc661, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcdc66b, 95 },
    { 0, 0 },
    { 0x64bdc66d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467dc671, 33 },
    { 0x169dc672, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2dc678, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40dc67c, 98 },
    { 0, 0 },
    { 0x715dc67e, 98 },
    { 0x5aedc67f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fdc684, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668dc68b, 33 },
    { 0x76cdc68b, 95 },
    { 0xea4dc68d, 98 },
    { 0x232dc68e, 33 },
    { 0, 0 },
    { 0x885dc690, 33 },
    { 0x48cdc691, 66 },
    { 0xba3dc692, 91 },
    { 0x8f5dc692, 100 },
    { 0x27bdc693, 78 },
    { 0x526dc695, 90 },
    { 0xd36dc694, 90 },
    { 0x1cddc697, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfdc69a, 66 },
    { 0x51bdc69b, 90 },
    { 0, 0 },
    { 0x17bdc69d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x883dc6a0, 90 },
    { 0xce4dc6a1, 91 },
    { 0xf79dc6a1, 97 },
    { 0x68dc6a1, 87 },
    { 0xcb5dc6a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589dc6a8, 34 },
    { 0, 0 },
    { 0x32cdc6aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dc6ae, 34 },
    { 0x22fdc6af, 66 },
    { 0xc2adc6b0, 33 },
    { 0x68bdc6b1, 90 },
    { 0xca6dc6b2, 61 },
    { 0x883dc6b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dc6b7, 98 },
    { 0xe94dc6b7, 33 },
    { 0, 0 },
    { 0x19adc6ba, 33 },
    { 0, 0 },
    { 0xb49dc6bc, 76 },
    { 0x353dc6bc, 78 },
    { 0x46dc6be, 61 },
    { 0x362dc6bd, 66 },
    { 0x115dc6bc, 43 },
    { 0, 0 },
    { 0xc97dc6c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70dc6c7, 61 },
    { 0x2f4dc6c8, 61 },
    { 0, 0 },
    { 0xe8cdc6ca, 17 },
    { 0xa99dc6cb, 98 },
    { 0xbd3dc6cb, 81 },
    { 0, 0 },
    { 0xf72dc6ce, 33 },
    { 0x27ddc6cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1dc6d6, 46 },
    { 0xb1fdc6d7, 46 },
    { 0xa67dc6d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dc6dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dc6df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x805dc6e2, 77 },
    { 0xd3edc6e2, 66 },
    { 0x236dc6e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2dc6e7, 87 },
    { 0, 0 },
    { 0xa07dc6e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241dc6ed, 61 },
    { 0x1adc6ee, 33 },
    { 0x258dc6ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26fdc6f2, 90 },
    { 0xd74dc6f3, 33 },
    { 0x32fdc6f4, 100 },
    { 0x90edc6f2, 23 },
    { 0x57edc6f6, 17 },
    { 0x406dc6f7, 61 },
    { 0x1e3dc6f8, 93 },
    { 0x339dc6f6, 90 },
    { 0, 0 },
    { 0xe4ddc6fb, 90 },
    { 0, 0 },
    { 0x6c0dc6fd, 78 },
    { 0x879dc6fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866dc70a, 95 },
    { 0xfbdc70b, 3 },
    { 0, 0 },
    { 0x184dc70d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115dc712, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x79edc715, 66 },
    { 0xc82dc716, 33 },
    { 0xc65dc717, 98 },
    { 0x111dc717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfdc71b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bdc71f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x945dc722, 61 },
    { 0xe09dc722, 91 },
    { 0x28dc724, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bdc728, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827dc72f, 90 },
    { 0, 0 },
    { 0x4f2dc731, 34 },
    { 0x705dc732, 55 },
    { 0, 0 },
    { 0xadcdc734, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdc737, 78 },
    { 0x847dc738, 43 },
    { 0x3c5dc739, 33 },
    { 0xa18dc73a, 61 },
    { 0xb6cdc73b, 90 },
    { 0x9fadc73c, 65 },
    { 0x367dc73c, 93 },
    { 0, 0 },
    { 0xe56dc73f, 88 },
    { 0x62edc73f, 61 },
    { 0x7f9dc740, 66 },
    { 0x6b0dc742, 34 },
    { 0x3dddc742, 88 },
    { 0, 0 },
    { 0x74dc745, 91 },
    { 0, 0 },
    { 0x619dc747, 33 },
    { 0xd99dc748, 61 },
    { 0x3a7dc749, 90 },
    { 0, 0 },
    { 0x5ccdc74b, 98 },
    { 0, 0 },
    { 0x87fdc74d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722dc751, 86 },
    { 0, 0 },
    { 0x5f3dc753, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x629dc756, 34 },
    { 0, 0 },
    { 0x445dc758, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5dc75d, 88 },
    { 0, 0 },
    { 0x64adc75f, 66 },
    { 0xe1ddc75f, 95 },
    { 0x943dc761, 43 },
    { 0x160dc762, 33 },
    { 0x82cdc75f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cdc766, 90 },
    { 0x5d7dc767, 55 },
    { 0x773dc768, 76 },
    { 0, 0 },
    { 0x68ddc76a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddc76d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81edc771, 33 },
    { 0x9c2dc772, 40 },
    { 0x88cdc771, 77 },
    { 0x800dc774, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdc778, 34 },
    { 0, 0 },
    { 0xa33dc77a, 98 },
    { 0xd5ddc77a, 90 },
    { 0, 0 },
    { 0xa0adc77d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5dc783, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ddc78c, 90 },
    { 0xf2bdc78c, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacedc791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25dc796, 90 },
    { 0, 0 },
    { 0x8fddc798, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584dc79c, 96 },
    { 0, 0 },
    { 0x110dc79e, 96 },
    { 0xe58dc79f, 52 },
    { 0, 0 },
    { 0x1eedc7a1, 90 },
    { 0xa61dc7a2, 92 },
    { 0x682dc7a2, 76 },
    { 0x414dc7a4, 78 },
    { 0xe59dc7a5, 90 },
    { 0x51ddc7a5, 77 },
    { 0xf80dc7a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2dc7ac, 90 },
    { 0x40dc7ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bdc7b3, 33 },
    { 0, 0 },
    { 0x59edc7b5, 93 },
    { 0, 0 },
    { 0xf34dc7b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936dc7c1, 61 },
    { 0x924dc7c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90adc7c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfdc7d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bdc7d4, 87 },
    { 0xa97dc7d4, 95 },
    { 0xc13dc7d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x338dc7d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x311dc7dc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99dc7e2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ddc7e5, 98 },
    { 0xb0cdc7e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcdc7ea, 33 },
    { 0, 0 },
    { 0x2d2dc7ec, 34 },
    { 0x3c2dc7ed, 33 },
    { 0xb09dc7ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdc7f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89adc7f5, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607dc7fa, 33 },
    { 0x818dc7fb, 66 },
    { 0x27bdc7fc, 78 },
    { 0x1ddc7fd, 76 },
    { 0xf5bdc7fe, 81 },
    { 0x4b2dc7fe, 61 },
    { 0xa16dc7fb, 61 },
    { 0x6a7dc7ff, 80 },
    { 0xfb7dc802, 90 },
    { 0, 0 },
    { 0x858dc804, 75 },
    { 0xd5bdc804, 98 },
    { 0xc06dc805, 95 },
    { 0x7cbdc807, 61 },
    { 0x46ddc805, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x626dc80b, 43 },
    { 0xe6cdc80c, 85 },
    { 0xb97dc80d, 95 },
    { 0x881dc80e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2addc812, 61 },
    { 0, 0 },
    { 0xce8dc814, 58 },
    { 0xc99dc814, 90 },
    { 0xf91dc814, 95 },
    { 0, 0 },
    { 0x554dc818, 90 },
    { 0, 0 },
    { 0x9a6dc81a, 98 },
    { 0xfc7dc81a, 90 },
    { 0xe8fdc81b, 90 },
    { 0x258dc81d, 76 },
    { 0x622dc81e, 33 },
    { 0x703dc81a, 75 },
    { 0xd32dc81f, 65 },
    { 0, 0 },
    { 0x1a2dc822, 33 },
    { 0, 0 },
    { 0xc25dc824, 78 },
    { 0x722dc825, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfdc82d, 67 },
    { 0, 0 },
    { 0x4fdc82f, 55 },
    { 0x6dfdc830, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efdc833, 23 },
    { 0x9cfdc833, 23 },
    { 0x585dc835, 95 },
    { 0x135dc836, 46 },
    { 0xbcedc836, 55 },
    { 0x23edc835, 34 },
    { 0x849dc839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7dc83e, 55 },
    { 0x870dc83f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x603dc843, 55 },
    { 0, 0 },
    { 0x87adc845, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02dc84c, 87 },
    { 0x60edc84d, 23 },
    { 0x54ddc84e, 33 },
    { 0x573dc84d, 95 },
    { 0xbb5dc84e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679dc855, 61 },
    { 0x729dc856, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x696dc85a, 76 },
    { 0x8e5dc85a, 100 },
    { 0xdb4dc85c, 98 },
    { 0x922dc85d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x510dc860, 42 },
    { 0xf27dc861, 61 },
    { 0, 0 },
    { 0x73cdc863, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fddc866, 76 },
    { 0x26edc867, 61 },
    { 0xbe2dc868, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x554dc86b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8dc86f, 96 },
    { 0x994dc870, 99 },
    { 0, 0 },
    { 0x9dddc872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdedc877, 90 },
    { 0x7bedc878, 75 },
    { 0x26dc879, 69 },
    { 0xecdc879, 33 },
    { 0xe73dc87b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x427dc87e, 78 },
    { 0xacddc87e, 93 },
    { 0x9e6dc87e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aadc884, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaddc887, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x425dc88a, 88 },
    { 0xdd9dc88a, 90 },
    { 0, 0 },
    { 0x6b4dc88d, 90 },
    { 0x6f9dc88e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766dc894, 76 },
    { 0x6f5dc895, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6adc898, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc46dc89e, 93 },
    { 0xe16dc89f, 67 },
    { 0x967dc89f, 33 },
    { 0xf2bdc89e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140dc8a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x691dc8ab, 90 },
    { 0x969dc8ab, 98 },
    { 0, 0 },
    { 0x9cadc8ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586dc8b3, 80 },
    { 0x75adc8b3, 99 },
    { 0, 0 },
    { 0x562dc8b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3dc8b9, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90dc8bc, 87 },
    { 0x8cbdc8bc, 23 },
    { 0x482dc8be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dc8c4, 95 },
    { 0, 0 },
    { 0xc02dc8c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4edc8d0, 95 },
    { 0, 0 },
    { 0x7d0dc8d2, 66 },
    { 0xad3dc8d3, 43 },
    { 0, 0 },
    { 0x1e9dc8d5, 23 },
    { 0x4f4dc8d5, 23 },
    { 0x5f8dc8d6, 33 },
    { 0x226dc8d8, 90 },
    { 0xccddc8d9, 90 },
    { 0xa4adc8d8, 66 },
    { 0x92edc8d7, 23 },
    { 0x7d4dc8d8, 36 },
    { 0x39fdc8da, 33 },
    { 0, 0 },
    { 0x290dc8df, 98 },
    { 0x4f1dc8df, 87 },
    { 0xe58dc8e1, 34 },
    { 0, 0 },
    { 0x69adc8e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaadc8ed, 78 },
    { 0xecdc8ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1addc8f1, 33 },
    { 0, 0 },
    { 0x342dc8f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcadc902, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649dc907, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfdc90b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x473dc90e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dc91a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140dc91e, 34 },
    { 0, 0 },
    { 0x277dc920, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x718dc924, 76 },
    { 0, 0 },
    { 0x968dc926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0dc92a, 77 },
    { 0x94bdc92a, 98 },
    { 0xb22dc92c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17dc933, 33 },
    { 0xeeedc933, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ddc937, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9edc93a, 84 },
    { 0, 0 },
    { 0x687dc93c, 90 },
    { 0x4c1dc93d, 90 },
    { 0x8cbdc93e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abdc944, 100 },
    { 0x111dc945, 61 },
    { 0x195dc945, 67 },
    { 0x394dc944, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaedc94b, 65 },
    { 0x4dadc94c, 34 },
    { 0x4a3dc94b, 98 },
    { 0xaf1dc94e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fdc952, 61 },
    { 0xb8fdc953, 55 },
    { 0x676dc954, 91 },
    { 0xaaadc952, 90 },
    { 0x4a1dc956, 84 },
    { 0xfb4dc957, 55 },
    { 0xbeddc958, 100 },
    { 0xbc5dc957, 33 },
    { 0, 0 },
    { 0x167dc95b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dddc962, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8adc968, 23 },
    { 0xd00dc968, 95 },
    { 0xe71dc96a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ddc96e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aedc974, 48 },
    { 0xf64dc975, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fddc97b, 34 },
    { 0xcfbdc97b, 90 },
    { 0x127dc97d, 66 },
    { 0xbbedc97e, 61 },
    { 0x74ddc97f, 89 },
    { 0, 0 },
    { 0x63fdc981, 33 },
    { 0x7f5dc982, 90 },
    { 0xb9dc982, 55 },
    { 0x374dc982, 34 },
    { 0, 0 },
    { 0xda0dc986, 52 },
    { 0x9f8dc986, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaedc98c, 33 },
    { 0, 0 },
    { 0x96adc98e, 61 },
    { 0, 0 },
    { 0x9dc990, 43 },
    { 0xbd5dc991, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaedc995, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66edc999, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85dc99c, 76 },
    { 0x627dc99d, 87 },
    { 0xc03dc99e, 78 },
    { 0x8c3dc99e, 34 },
    { 0, 0 },
    { 0x2e0dc9a1, 52 },
    { 0x38bdc9a2, 90 },
    { 0x15adc9a2, 61 },
    { 0, 0 },
    { 0x378dc9a5, 60 },
    { 0, 0 },
    { 0x8e6dc9a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x255dc9aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69edc9ad, 55 },
    { 0x2b1dc9ae, 61 },
    { 0x3a3dc9af, 90 },
    { 0, 0 },
    { 0xba7dc9b1, 33 },
    { 0xa27dc9b2, 93 },
    { 0xff0dc9b3, 67 },
    { 0x7dedc9b4, 23 },
    { 0x25bdc9b5, 95 },
    { 0xb28dc9b6, 46 },
    { 0xa88dc9b7, 33 },
    { 0xf28dc9b8, 90 },
    { 0, 0 },
    { 0x5b1dc9ba, 91 },
    { 0x8f9dc9bb, 95 },
    { 0x403dc9bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64dc9c2, 61 },
    { 0x424dc9c2, 61 },
    { 0xd7ddc9c3, 43 },
    { 0, 0 },
    { 0x35edc9c6, 98 },
    { 0xf95dc9c7, 23 },
    { 0xc18dc9c6, 90 },
    { 0, 0 },
    { 0xdf3dc9ca, 90 },
    { 0xb11dc9cb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bdc9ce, 95 },
    { 0xfd0dc9ce, 95 },
    { 0xc3ddc9d0, 75 },
    { 0, 0 },
    { 0x84bdc9d2, 96 },
    { 0x7c6dc9d3, 55 },
    { 0x671dc9d4, 97 },
    { 0xbbddc9d5, 90 },
    { 0xd2dc9d5, 91 },
    { 0x29ddc9d7, 93 },
    { 0, 0 },
    { 0x8adc9d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6dc9de, 61 },
    { 0xc69dc9df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cdc9e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bddc9ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2dc9f4, 51 },
    { 0x4b6dc9f5, 95 },
    { 0x3c0dc9f5, 69 },
    { 0x860dc9f7, 87 },
    { 0xa6cdc9f4, 61 },
    { 0xcfcdc9f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53dc9fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7dc9ff, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1dca03, 95 },
    { 0x55cdca04, 93 },
    { 0xeefdca05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dca13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25dca1c, 90 },
    { 0x200dca1c, 91 },
    { 0x8c2dca1c, 33 },
    { 0, 0 },
    { 0x8d5dca20, 33 },
    { 0, 0 },
    { 0x37ddca22, 92 },
    { 0xe98dca22, 88 },
    { 0, 0 },
    { 0x9e9dca25, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0dca29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1dca2d, 90 },
    { 0xf33dca2e, 33 },
    { 0x9d3dca2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ddca36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdca3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7dca45, 82 },
    { 0, 0 },
    { 0x91edca47, 61 },
    { 0x6cadca48, 81 },
    { 0, 0 },
    { 0x298dca4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddca4d, 52 },
    { 0xb04dca4e, 61 },
    { 0, 0 },
    { 0x9b5dca50, 98 },
    { 0x942dca51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7dca5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edca5f, 95 },
    { 0x94bdca60, 84 },
    { 0, 0 },
    { 0x1f7dca62, 43 },
    { 0xd15dca63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241dca68, 98 },
    { 0, 0 },
    { 0xa9edca6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0dca77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dca7a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9dca7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffadca83, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88dca87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fedca8d, 76 },
    { 0x701dca8d, 67 },
    { 0xfcfdca8d, 55 },
    { 0, 0 },
    { 0xe6adca91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4dca94, 98 },
    { 0xabddca95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edca9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bdca9e, 84 },
    { 0x47bdca9f, 64 },
    { 0x49dca9f, 46 },
    { 0xc62dca9f, 95 },
    { 0, 0 },
    { 0x2e3dcaa3, 76 },
    { 0, 0 },
    { 0xac2dcaa5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306dcaaa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x599dcaad, 90 },
    { 0, 0 },
    { 0xf8bdcaaf, 33 },
    { 0, 0 },
    { 0x2a2dcab1, 34 },
    { 0x7b5dcab2, 90 },
    { 0x2fedcab3, 98 },
    { 0, 0 },
    { 0xc01dcab5, 34 },
    { 0x1bedcab5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9dcab9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ddcabd, 84 },
    { 0x2b0dcabe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97dcac2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bdcac5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3dcac8, 33 },
    { 0xb4dcac9, 76 },
    { 0xa99dcaca, 41 },
    { 0xc90dcac8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64dcad5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fdcad9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bdcae2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ddcae6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4dcaf2, 43 },
    { 0x56dcaf3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cdcaf6, 78 },
    { 0x25adcaf7, 93 },
    { 0xa34dcaf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ddcafc, 33 },
    { 0, 0 },
    { 0x203dcafe, 84 },
    { 0, 0 },
    { 0xab1dcb00, 34 },
    { 0x465dcb01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acdcb05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x46edcb08, 63 },
    { 0x9dcdcb09, 72 },
    { 0, 0 },
    { 0x20bdcb0b, 76 },
    { 0x866dcb0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bddcb0f, 85 },
    { 0, 0 },
    { 0xc78dcb11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343dcb1c, 55 },
    { 0xe23dcb1d, 46 },
    { 0, 0 },
    { 0xf93dcb1f, 78 },
    { 0x4b9dcb1f, 75 },
    { 0x893dcb21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892dcb25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdcb2a, 23 },
    { 0x437dcb2b, 61 },
    { 0x415dcb2c, 98 },
    { 0xbe6dcb2d, 84 },
    { 0x520dcb2b, 90 },
    { 0, 0 },
    { 0x67cdcb30, 89 },
    { 0x368dcb30, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6dcb39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ddcb40, 95 },
    { 0, 0 },
    { 0xce3dcb42, 95 },
    { 0x20bdcb43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fedcb48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbdcb4b, 34 },
    { 0xafcdcb4c, 93 },
    { 0xe0edcb4d, 76 },
    { 0, 0 },
    { 0x3c3dcb4f, 33 },
    { 0xe49dcb50, 33 },
    { 0x864dcb4f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10dcb56, 52 },
    { 0x36dcb57, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1dcb5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9dcb5f, 33 },
    { 0x21edcb5f, 34 },
    { 0xfe7dcb61, 90 },
    { 0xa7dcb62, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddcb68, 33 },
    { 0x8c2dcb69, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95dcb6f, 33 },
    { 0x867dcb70, 81 },
    { 0x644dcb70, 98 },
    { 0xb12dcb70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3dcb77, 33 },
    { 0x113dcb78, 90 },
    { 0x3edcb77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482dcb7d, 95 },
    { 0, 0 },
    { 0x344dcb7f, 33 },
    { 0xf74dcb80, 98 },
    { 0x8d5dcb81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8dcb84, 90 },
    { 0, 0 },
    { 0x297dcb86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9adcb89, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bdcb8e, 33 },
    { 0, 0 },
    { 0xa5dcb90, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467dcb9a, 33 },
    { 0x931dcb9a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0dcba3, 95 },
    { 0x244dcba3, 61 },
    { 0xb67dcba5, 93 },
    { 0x515dcba4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6dcbad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8adcbb0, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cdcbbc, 76 },
    { 0xb1bdcbbd, 33 },
    { 0x162dcbbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969dcbc3, 90 },
    { 0xcb5dcbc3, 34 },
    { 0xa48dcbc5, 34 },
    { 0xe08dcbc5, 75 },
    { 0, 0 },
    { 0xdc8dcbc8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x751dcbcb, 33 },
    { 0xb65dcbcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x24edcbcf, 65 },
    { 0xca9dcbcf, 95 },
    { 0xea3dcbd1, 55 },
    { 0x1dbdcbd2, 23 },
    { 0x99edcbd1, 80 },
    { 0xc5fdcbd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119dcbdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7dcbe4, 98 },
    { 0xe9adcbe5, 55 },
    { 0xa1dcbe5, 61 },
    { 0x9bddcbe7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x67dcbea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7dcbf4, 67 },
    { 0, 0 },
    { 0x841dcbf6, 75 },
    { 0xe5dcbf7, 76 },
    { 0x9fbdcbf7, 95 },
    { 0xe7bdcbf7, 98 },
    { 0xc40dcbfa, 100 },
    { 0xbc7dcbf6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ddcbfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4dcc04, 88 },
    { 0xd48dcc05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588dcc0c, 81 },
    { 0xb75dcc0c, 91 },
    { 0x5d4dcc0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x381dcc11, 98 },
    { 0, 0 },
    { 0x1a3dcc13, 43 },
    { 0x263dcc14, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4dcc18, 91 },
    { 0, 0 },
    { 0xf2ddcc1a, 90 },
    { 0xcd2dcc1b, 98 },
    { 0, 0 },
    { 0xd23dcc1d, 61 },
    { 0x814dcc1e, 80 },
    { 0x367dcc1e, 76 },
    { 0x5e8dcc1e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78edcc24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6dcc27, 46 },
    { 0xb8cdcc28, 65 },
    { 0x7cedcc29, 23 },
    { 0x9ccdcc2a, 90 },
    { 0xcfcdcc29, 17 },
    { 0, 0 },
    { 0xe40dcc2d, 52 },
    { 0xa5adcc2e, 91 },
    { 0, 0 },
    { 0x937dcc30, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984dcc3a, 61 },
    { 0x11ddcc3b, 33 },
    { 0x98edcc3b, 98 },
    { 0xf4bdcc3d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0adcc49, 23 },
    { 0xae6dcc4a, 90 },
    { 0x8c6dcc4a, 100 },
    { 0, 0 },
    { 0x80cdcc4d, 61 },
    { 0, 0 },
    { 0xadfdcc4f, 64 },
    { 0xe24dcc4f, 65 },
    { 0xcaddcc51, 33 },
    { 0x152dcc51, 76 },
    { 0x32cdcc51, 100 },
    { 0x12dcc54, 95 },
    { 0xfdcdcc55, 55 },
    { 0x197dcc53, 66 },
    { 0x97fdcc57, 90 },
    { 0xeedcc55, 98 },
    { 0xab0dcc58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fdcc5d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59dcc65, 43 },
    { 0x3f1dcc65, 75 },
    { 0xc6adcc65, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521dcc6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438dcc73, 34 },
    { 0, 0 },
    { 0xa3edcc75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62dcc7f, 33 },
    { 0x109dcc80, 66 },
    { 0xe44dcc81, 43 },
    { 0x89adcc82, 95 },
    { 0x933dcc81, 55 },
    { 0x721dcc84, 61 },
    { 0x9e0dcc85, 95 },
    { 0x7b5dcc86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3dcc8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49dcc8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534dcc93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6adcc99, 78 },
    { 0, 0 },
    { 0xf2fdcc9b, 33 },
    { 0x852dcc9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ddcca1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4dcca6, 52 },
    { 0, 0 },
    { 0x45cdcca8, 55 },
    { 0, 0 },
    { 0xa2bdccaa, 61 },
    { 0x376dccaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6dccb7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x373dccba, 93 },
    { 0xc9fdccbb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88dccbe, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76edccc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64dccc5, 61 },
    { 0xcedccc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x907dcccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5adccce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x264dccd1, 95 },
    { 0xbedccd2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebdccda, 82 },
    { 0x11edccdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x996dccde, 52 },
    { 0xbf6dccdf, 23 },
    { 0x9b3dcce0, 43 },
    { 0x8f0dccdf, 23 },
    { 0xcdadcce2, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83adcced, 61 },
    { 0, 0 },
    { 0x382dccef, 36 },
    { 0xd75dccf0, 33 },
    { 0, 0 },
    { 0x67edccf2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cedccf9, 67 },
    { 0xa6cdccfa, 93 },
    { 0xc4adccfa, 46 },
    { 0xd7bdccfc, 23 },
    { 0, 0 },
    { 0xc2ddccfe, 55 },
    { 0xbfedccfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcdcd08, 98 },
    { 0x4e4dcd08, 95 },
    { 0x5e8dcd0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5dcd15, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90dcd1a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dcd1e, 97 },
    { 0, 0 },
    { 0x221dcd20, 52 },
    { 0xb5edcd21, 23 },
    { 0xa17dcd21, 90 },
    { 0x958dcd20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4dcd27, 76 },
    { 0, 0 },
    { 0xefedcd29, 34 },
    { 0x167dcd2a, 33 },
    { 0x459dcd2b, 100 },
    { 0, 0 },
    { 0xfc0dcd2d, 23 },
    { 0xe4bdcd2e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22dcd32, 23 },
    { 0, 0 },
    { 0x32ddcd34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcdcd37, 66 },
    { 0x79fdcd37, 90 },
    { 0, 0 },
    { 0x93edcd3a, 88 },
    { 0x999dcd3b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfdcd3f, 99 },
    { 0xe00dcd40, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x966dcd43, 98 },
    { 0x792dcd44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47dcd4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402dcd50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddcd54, 23 },
    { 0xfb8dcd55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5edcd58, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacdcd5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcdcd62, 76 },
    { 0, 0 },
    { 0x583dcd64, 23 },
    { 0, 0 },
    { 0x97edcd66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9dcd69, 76 },
    { 0, 0 },
    { 0xbc2dcd6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57adcd70, 33 },
    { 0, 0 },
    { 0x5a9dcd72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0dcd79, 46 },
    { 0xba1dcd7a, 90 },
    { 0x111dcd7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7dcd7e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaedcd81, 67 },
    { 0, 0 },
    { 0xe7edcd83, 76 },
    { 0x706dcd84, 33 },
    { 0x4cadcd84, 89 },
    { 0xd06dcd86, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15edcd8e, 23 },
    { 0xf2fdcd8e, 33 },
    { 0xd73dcd8e, 98 },
    { 0xbd2dcd91, 46 },
    { 0x217dcd91, 90 },
    { 0x978dcd93, 33 },
    { 0x6fddcd93, 34 },
    { 0x140dcd95, 33 },
    { 0x26ddcd96, 23 },
    { 0x852dcd95, 33 },
    { 0x81fdcd92, 98 },
    { 0x8c0dcd99, 98 },
    { 0x4cdcd9a, 17 },
    { 0xaebdcd93, 55 },
    { 0, 0 },
    { 0x27fdcd9d, 85 },
    { 0, 0 },
    { 0x534dcd9f, 98 },
    { 0x669dcda0, 88 },
    { 0, 0 },
    { 0x21ddcda2, 100 },
    { 0x5a9dcda3, 55 },
    { 0x8ffdcda4, 90 },
    { 0x1bfdcda4, 90 },
    { 0x279dcda4, 90 },
    { 0xccfdcda7, 52 },
    { 0x28ddcda7, 90 },
    { 0xd76dcda6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x536dcdad, 33 },
    { 0, 0 },
    { 0x1d8dcdaf, 90 },
    { 0x547dcdaf, 43 },
    { 0, 0 },
    { 0xea1dcdb2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0dcdbc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14dcdc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ddcdc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3adcdcf, 33 },
    { 0, 0 },
    { 0x7fbdcdd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eedcdd6, 90 },
    { 0, 0 },
    { 0x960dcdd8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23dcddc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1dcde1, 93 },
    { 0, 0 },
    { 0x962dcde3, 33 },
    { 0xa69dcde4, 61 },
    { 0x4b7dcde5, 90 },
    { 0xcafdcde6, 61 },
    { 0xdafdcde5, 98 },
    { 0xb6ddcde8, 100 },
    { 0, 0 },
    { 0x8a0dcdea, 90 },
    { 0, 0 },
    { 0x3dadcdec, 66 },
    { 0x57adcdec, 90 },
    { 0xd3edcdee, 95 },
    { 0x211dcded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fdcdf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ddcdf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3dcdfa, 23 },
    { 0x894dcdfa, 98 },
    { 0, 0 },
    { 0x661dcdfd, 98 },
    { 0xaccdcdfe, 100 },
    { 0x949dcdfe, 9 },
    { 0, 0 },
    { 0xe8bdce01, 90 },
    { 0x439dce02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fdce0a, 33 },
    { 0xd4edce0b, 46 },
    { 0x5e3dce0a, 90 },
    { 0x190dce0d, 95 },
    { 0x963dce0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342dce12, 61 },
    { 0x703dce13, 90 },
    { 0, 0 },
    { 0x3d6dce15, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9dce19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9dce1d, 46 },
    { 0x7cbdce1e, 63 },
    { 0, 0 },
    { 0xa43dce20, 34 },
    { 0x9a9dce20, 67 },
    { 0, 0 },
    { 0x63edce23, 61 },
    { 0x88fdce24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdce2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7dce36, 88 },
    { 0, 0 },
    { 0x489dce38, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576dce42, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806dce46, 55 },
    { 0x254dce47, 52 },
    { 0, 0 },
    { 0x7bddce49, 76 },
    { 0xe0fdce4a, 90 },
    { 0, 0 },
    { 0xacedce4c, 84 },
    { 0x5c8dce4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdce50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10dce53, 76 },
    { 0x19adce54, 23 },
    { 0x505dce53, 37 },
    { 0x81cdce55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45dce5a, 98 },
    { 0x569dce5b, 46 },
    { 0, 0 },
    { 0x30fdce5d, 34 },
    { 0, 0 },
    { 0x6fbdce5f, 33 },
    { 0x1e9dce5f, 33 },
    { 0x8b2dce5f, 84 },
    { 0x759dce62, 46 },
    { 0, 0 },
    { 0x8aadce64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93dce69, 34 },
    { 0xb66dce6a, 33 },
    { 0x2d5dce69, 76 },
    { 0, 0 },
    { 0x385dce6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105dce72, 90 },
    { 0x2dbdce73, 90 },
    { 0xaa9dce73, 34 },
    { 0xd1adce75, 65 },
    { 0, 0 },
    { 0x16fdce77, 61 },
    { 0x413dce78, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15dce7c, 85 },
    { 0xe22dce7d, 52 },
    { 0x9f0dce7e, 61 },
    { 0x8a5dce7f, 90 },
    { 0, 0 },
    { 0xd26dce81, 99 },
    { 0x605dce82, 61 },
    { 0x990dce83, 46 },
    { 0x943dce84, 33 },
    { 0xcaedce85, 98 },
    { 0x875dce84, 84 },
    { 0xdffdce84, 90 },
    { 0xf5fdce83, 99 },
    { 0x821dce87, 55 },
    { 0xa28dce8a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fdce8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27edce93, 90 },
    { 0xf06dce93, 33 },
    { 0x3dcdce95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0dce99, 33 },
    { 0x849dce9a, 99 },
    { 0xda0dce9b, 17 },
    { 0x6fdce9b, 33 },
    { 0x513dce9a, 100 },
    { 0xe2ddce9b, 55 },
    { 0, 0 },
    { 0x82dcea0, 76 },
    { 0, 0 },
    { 0x458dcea2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6dcea8, 33 },
    { 0x33edcea8, 98 },
    { 0x1d6dcea9, 98 },
    { 0xe7edcea9, 34 },
    { 0x3e2dcea8, 55 },
    { 0x29bdcea8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415dceb1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50cdceb4, 61 },
    { 0, 0 },
    { 0x8d6dceb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1dcebc, 33 },
    { 0xbb6dcebc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0dcec1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bdcec7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecdcecc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1dced0, 98 },
    { 0xc25dced1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbdced9, 98 },
    { 0x165dceda, 43 },
    { 0x652dcedb, 23 },
    { 0xb7bdcedb, 76 },
    { 0, 0 },
    { 0x1c7dcede, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522dcee3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fdcee9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1dceef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x213dcef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bdcef7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06dcefa, 43 },
    { 0, 0 },
    { 0x66adcefc, 77 },
    { 0, 0 },
    { 0x954dcefe, 90 },
    { 0x36cdceff, 34 },
    { 0xbacdcefe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55dcf07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77dcf0c, 33 },
    { 0xf2edcf0d, 98 },
    { 0xfaddcf0e, 76 },
    { 0xa71dcf0f, 46 },
    { 0xf45dcf10, 100 },
    { 0x89cdcf0c, 34 },
    { 0xed5dcf12, 98 },
    { 0xcf2dcf13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadcf1f, 75 },
    { 0xa33dcf20, 52 },
    { 0x62cdcf21, 23 },
    { 0, 0 },
    { 0x515dcf23, 67 },
    { 0xbe5dcf24, 99 },
    { 0xecedcf25, 90 },
    { 0x4f5dcf25, 43 },
    { 0x95adcf25, 95 },
    { 0, 0 },
    { 0x673dcf29, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1dcf2d, 76 },
    { 0, 0 },
    { 0x447dcf2f, 33 },
    { 0, 0 },
    { 0x494dcf31, 43 },
    { 0, 0 },
    { 0x16cdcf33, 76 },
    { 0xaacdcf34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4dcf37, 76 },
    { 0, 0 },
    { 0xb66dcf39, 87 },
    { 0, 0 },
    { 0xa77dcf3b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361dcf43, 55 },
    { 0x8b9dcf44, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdadcf49, 34 },
    { 0x333dcf49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286dcf4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274dcf55, 52 },
    { 0, 0 },
    { 0x2c2dcf57, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4dcf5a, 76 },
    { 0xc9bdcf5b, 43 },
    { 0xec9dcf5c, 98 },
    { 0, 0 },
    { 0x8dadcf5e, 99 },
    { 0x6fdcf5e, 84 },
    { 0x29cdcf60, 98 },
    { 0xb6adcf61, 98 },
    { 0x570dcf62, 33 },
    { 0x255dcf5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0dcf69, 61 },
    { 0xf9dcf69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x576dcf6d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8dcf73, 84 },
    { 0x33ddcf74, 66 },
    { 0, 0 },
    { 0x233dcf76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6dcf79, 90 },
    { 0x265dcf7a, 23 },
    { 0x6b2dcf7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaedcf81, 52 },
    { 0x5a9dcf82, 87 },
    { 0xfd5dcf83, 61 },
    { 0x707dcf84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7dcf88, 23 },
    { 0, 0 },
    { 0x75cdcf8a, 76 },
    { 0, 0 },
    { 0x48cdcf8c, 98 },
    { 0xe29dcf8c, 90 },
    { 0, 0 },
    { 0x8ccdcf8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52dcf92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfdcf95, 98 },
    { 0x15ddcf96, 87 },
    { 0, 0 },
    { 0xb3ddcf98, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdadcf9e, 55 },
    { 0x79ddcf9f, 23 },
    { 0, 0 },
    { 0x376dcfa1, 43 },
    { 0x9f3dcfa2, 93 },
    { 0, 0 },
    { 0xefedcfa4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcdcfa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bedcfad, 34 },
    { 0x29fdcfae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7dcfb2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193dcfba, 23 },
    { 0x4bddcfbb, 90 },
    { 0xe2edcfbb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98adcfc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9dcfd0, 52 },
    { 0xc7edcfd0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9dcfd5, 61 },
    { 0xacddcfd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecdcfdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553dcfdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cadcfe2, 34 },
    { 0x408dcfe2, 33 },
    { 0, 0 },
    { 0x959dcfe5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7dcfe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1edcfee, 34 },
    { 0x477dcfef, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcddcff4, 98 },
    { 0, 0 },
    { 0x6c5dcff6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3dcffd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cdd004, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdd00a, 91 },
    { 0x426dd00a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434dd013, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15dd017, 33 },
    { 0x419dd018, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ddd020, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fdd025, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18dd02c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541dd031, 33 },
    { 0xf8dd032, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x169dd035, 34 },
    { 0xc20dd035, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5dd03a, 90 },
    { 0xa20dd03b, 55 },
    { 0, 0 },
    { 0x4fdd03d, 95 },
    { 0x65ddd03d, 76 },
    { 0xdbadd03f, 97 },
    { 0x1d2dd03f, 84 },
    { 0xf2dd041, 43 },
    { 0xcc4dd03e, 95 },
    { 0, 0 },
    { 0xb44dd044, 23 },
    { 0x586dd045, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6dd048, 98 },
    { 0, 0 },
    { 0x3d9dd04a, 98 },
    { 0x45bdd04b, 33 },
    { 0x3bbdd04b, 95 },
    { 0x1dd04d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fedd057, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x679dd05a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4dd062, 95 },
    { 0x209dd062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734dd068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddd06c, 66 },
    { 0, 0 },
    { 0xc2edd06e, 23 },
    { 0x636dd06f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dd072, 61 },
    { 0, 0 },
    { 0x7f1dd074, 33 },
    { 0x50ddd074, 61 },
    { 0x7e4dd075, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6edd080, 33 },
    { 0, 0 },
    { 0x86dd082, 76 },
    { 0x150dd083, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317dd089, 55 },
    { 0x5dbdd089, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fdd08e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dd095, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8dd099, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cdd09f, 55 },
    { 0x4cddd0a0, 55 },
    { 0x6a5dd0a1, 76 },
    { 0, 0 },
    { 0xb05dd0a3, 90 },
    { 0xb23dd0a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x538dd0a7, 77 },
    { 0x9b8dd0a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x735dd0ab, 52 },
    { 0xa92dd0ab, 61 },
    { 0xfa5dd0ab, 76 },
    { 0x5e5dd0ae, 33 },
    { 0x7e1dd0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cdd0b6, 61 },
    { 0x569dd0b6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65dd0bd, 23 },
    { 0x15add0bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722dd0c4, 23 },
    { 0x1dfdd0c5, 76 },
    { 0xb25dd0c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271dd0ca, 61 },
    { 0xeb1dd0ca, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52dd0ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51dd0d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1dd0d9, 43 },
    { 0, 0 },
    { 0xee2dd0db, 92 },
    { 0x256dd0db, 23 },
    { 0xf39dd0dd, 76 },
    { 0, 0 },
    { 0xb7add0df, 98 },
    { 0x200dd0e0, 100 },
    { 0x1afdd0e1, 98 },
    { 0xbd2dd0e1, 33 },
    { 0x12fdd0e3, 61 },
    { 0x698dd0e1, 90 },
    { 0x24ddd0e1, 43 },
    { 0, 0 },
    { 0xaf5dd0e7, 34 },
    { 0, 0 },
    { 0x846dd0e9, 87 },
    { 0xd84dd0ea, 46 },
    { 0x35dd0e9, 93 },
    { 0xa43dd0eb, 90 },
    { 0xf25dd0ed, 33 },
    { 0x24bdd0ea, 99 },
    { 0, 0 },
    { 0x610dd0f0, 9 },
    { 0xcc0dd0f1, 95 },
    { 0xe01dd0f1, 98 },
    { 0, 0 },
    { 0x1fcdd0f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51dd0f8, 72 },
    { 0x698dd0f9, 75 },
    { 0xa24dd0fa, 95 },
    { 0x1c6dd0fb, 23 },
    { 0x50cdd0fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6dd102, 33 },
    { 0, 0 },
    { 0xa74dd104, 95 },
    { 0x6dadd104, 98 },
    { 0x5d5dd105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67dd10a, 99 },
    { 0x2e5dd10b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fdd119, 91 },
    { 0xddfdd11a, 81 },
    { 0x3e3dd119, 34 },
    { 0x52edd11b, 67 },
    { 0, 0 },
    { 0x119dd11e, 100 },
    { 0, 0 },
    { 0x7ddd120, 100 },
    { 0x94fdd121, 55 },
    { 0x214dd120, 98 },
    { 0x89ddd123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd127, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fdd12c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98dd130, 66 },
    { 0x53fdd131, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbdd138, 91 },
    { 0xd59dd138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0dd13c, 90 },
    { 0xb43dd13d, 90 },
    { 0xb93dd13e, 98 },
    { 0x2f1dd13f, 95 },
    { 0x12add13f, 61 },
    { 0xb4edd141, 23 },
    { 0x45fdd142, 40 },
    { 0x93add141, 23 },
    { 0xafadd142, 66 },
    { 0, 0 },
    { 0x799dd146, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9dd152, 23 },
    { 0x979dd153, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x457dd156, 46 },
    { 0x2c4dd157, 100 },
    { 0, 0 },
    { 0xd68dd159, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91dd15c, 33 },
    { 0x963dd15d, 55 },
    { 0x47cdd15e, 55 },
    { 0xfabdd15f, 81 },
    { 0xe36dd15e, 34 },
    { 0x399dd160, 38 },
    { 0x136dd15d, 36 },
    { 0xc04dd163, 85 },
    { 0, 0 },
    { 0x1b4dd165, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8dd168, 72 },
    { 0, 0 },
    { 0xc54dd16a, 34 },
    { 0x746dd16b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcdd170, 99 },
    { 0xf90dd170, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ddd178, 81 },
    { 0x8d3dd178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecdd17d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6dd180, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18dd186, 33 },
    { 0, 0 },
    { 0x678dd188, 85 },
    { 0, 0 },
    { 0xa27dd18a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7dd18f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cadd192, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7dd195, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0dd198, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ddd19f, 98 },
    { 0, 0 },
    { 0xb5cdd1a1, 33 },
    { 0xfd4dd1a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x640dd1a5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880dd1ab, 100 },
    { 0, 0 },
    { 0x434dd1ad, 78 },
    { 0x75bdd1ae, 98 },
    { 0, 0 },
    { 0xeeddd1b0, 33 },
    { 0xf64dd1b1, 66 },
    { 0xa88dd1b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53dd1b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310dd1b9, 85 },
    { 0, 0 },
    { 0x31fdd1bb, 33 },
    { 0, 0 },
    { 0x7abdd1bd, 100 },
    { 0xcc6dd1bd, 90 },
    { 0xf10dd1bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331dd1c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x755dd1c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133dd1cd, 90 },
    { 0x79bdd1ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x75edd1d1, 98 },
    { 0xea6dd1d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ddd1d5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ddd1dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ddd1e4, 76 },
    { 0x242dd1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4dd1f0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efdd1f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfdd1fb, 67 },
    { 0, 0 },
    { 0x686dd1fd, 33 },
    { 0, 0 },
    { 0x75edd1ff, 78 },
    { 0x8dddd200, 61 },
    { 0xa3fdd200, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6dd20a, 84 },
    { 0xf46dd20b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928dd211, 95 },
    { 0, 0 },
    { 0xe03dd213, 47 },
    { 0, 0 },
    { 0x7a0dd215, 23 },
    { 0x5c0dd215, 85 },
    { 0xcb3dd215, 90 },
    { 0x500dd218, 34 },
    { 0, 0 },
    { 0x3ddd21a, 34 },
    { 0x373dd21b, 91 },
    { 0xd1add21a, 93 },
    { 0x41edd21d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8addd220, 61 },
    { 0, 0 },
    { 0x5ddd222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606dd22a, 98 },
    { 0xffddd22a, 91 },
    { 0x21edd22c, 77 },
    { 0xadedd22c, 93 },
    { 0x292dd22a, 33 },
    { 0xf4fdd22f, 43 },
    { 0xc64dd230, 47 },
    { 0x9aadd231, 43 },
    { 0x8eedd232, 55 },
    { 0x7fcdd233, 23 },
    { 0xbb8dd234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3dd237, 23 },
    { 0xdf1dd238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7dd241, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6edd244, 33 },
    { 0x3b5dd245, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92dd248, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15dd24c, 75 },
    { 0, 0 },
    { 0x7cbdd24e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cdd251, 61 },
    { 0xed4dd251, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08dd255, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857dd259, 90 },
    { 0x598dd25a, 90 },
    { 0xc8fdd25a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x882dd25e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8dd261, 23 },
    { 0x879dd261, 90 },
    { 0xdbcdd261, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ddd26c, 98 },
    { 0, 0 },
    { 0xf43dd26e, 61 },
    { 0x36cdd26f, 95 },
    { 0x2f2dd270, 97 },
    { 0xfa0dd270, 76 },
    { 0xa70dd271, 95 },
    { 0x8addd272, 98 },
    { 0xc40dd26e, 55 },
    { 0xb49dd275, 93 },
    { 0x26fdd270, 90 },
    { 0xf55dd274, 66 },
    { 0x8e8dd278, 61 },
    { 0x6b1dd278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x674dd27c, 90 },
    { 0xe6fdd27d, 90 },
    { 0xebfdd27e, 33 },
    { 0x4a2dd27f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2dd286, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218dd28b, 55 },
    { 0, 0 },
    { 0x5dd28d, 52 },
    { 0xa78dd28e, 46 },
    { 0xd85dd28e, 76 },
    { 0x7a6dd28e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34dd294, 61 },
    { 0x380dd294, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ddd298, 90 },
    { 0, 0 },
    { 0x4aedd29a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15dd29d, 33 },
    { 0, 0 },
    { 0xc05dd29f, 33 },
    { 0x210dd29f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16dd2a3, 23 },
    { 0x653dd2a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719dd2ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1dd2b0, 43 },
    { 0x5dadd2b0, 33 },
    { 0, 0 },
    { 0x58bdd2b3, 98 },
    { 0x71ddd2b3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85dd2b7, 33 },
    { 0xcf4dd2b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22dd2bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bdd2c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fadd2c8, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74dd2d9, 61 },
    { 0x566dd2d9, 34 },
    { 0xdedd2db, 64 },
    { 0xb4cdd2d9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1dd2e1, 23 },
    { 0xa3edd2e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x103dd2e5, 66 },
    { 0x501dd2e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x240dd2e9, 90 },
    { 0xb33dd2ea, 98 },
    { 0x620dd2e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21dd2ee, 33 },
    { 0xa27dd2ee, 90 },
    { 0, 0 },
    { 0x9f6dd2f1, 33 },
    { 0, 0 },
    { 0x9fcdd2f3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x95add2f6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1dd2f9, 33 },
    { 0x83bdd2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19dd2fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0dd301, 24 },
    { 0, 0 },
    { 0xe7bdd303, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fdd306, 33 },
    { 0, 0 },
    { 0x1c9dd308, 76 },
    { 0, 0 },
    { 0x2c7dd30a, 93 },
    { 0xb8add30b, 33 },
    { 0x506dd30c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeedd312, 76 },
    { 0x82bdd313, 65 },
    { 0xe8edd314, 90 },
    { 0x4c0dd315, 61 },
    { 0x2dadd315, 93 },
    { 0x2cedd314, 78 },
    { 0xd14dd318, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8dd31f, 38 },
    { 0x2bfdd320, 99 },
    { 0, 0 },
    { 0x174dd322, 88 },
    { 0x426dd323, 87 },
    { 0, 0 },
    { 0x6dadd325, 34 },
    { 0, 0 },
    { 0xcdadd327, 33 },
    { 0x314dd327, 65 },
    { 0x95fdd329, 38 },
    { 0x22add32a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0dd330, 85 },
    { 0x580dd331, 23 },
    { 0, 0 },
    { 0xbc0dd333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ddd33d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5dd344, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ddd349, 90 },
    { 0x2cbdd349, 23 },
    { 0x151dd34a, 33 },
    { 0x8cdd34c, 90 },
    { 0x882dd34c, 34 },
    { 0xc7add34e, 33 },
    { 0xbf4dd34f, 59 },
    { 0xe48dd34e, 90 },
    { 0, 0 },
    { 0xe79dd352, 99 },
    { 0, 0 },
    { 0xbb3dd354, 98 },
    { 0x185dd355, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90dd35a, 90 },
    { 0xebfdd35b, 33 },
    { 0x8a8dd35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x905dd35f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237dd363, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619dd368, 90 },
    { 0x41fdd369, 95 },
    { 0xd44dd36a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3dd36f, 76 },
    { 0xe6dd370, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7dd373, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cdd377, 61 },
    { 0, 0 },
    { 0x2a8dd379, 23 },
    { 0x964dd37a, 61 },
    { 0x942dd37b, 78 },
    { 0x49edd379, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ddd37f, 90 },
    { 0x178dd37f, 91 },
    { 0x9d0dd381, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667dd38b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ddd391, 78 },
    { 0, 0 },
    { 0xa47dd393, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10dd396, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dd39b, 34 },
    { 0xad1dd39c, 93 },
    { 0, 0 },
    { 0x27edd39e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdd3a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557dd3b2, 23 },
    { 0, 0 },
    { 0xa0fdd3b4, 61 },
    { 0xad2dd3b5, 23 },
    { 0, 0 },
    { 0x9cedd3b7, 90 },
    { 0x8dbdd3b7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ddd3bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24dd3c0, 61 },
    { 0x85ddd3c0, 55 },
    { 0x22bdd3c0, 61 },
    { 0, 0 },
    { 0x8aedd3c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x696dd3c7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1dd3cc, 90 },
    { 0xe4edd3cc, 33 },
    { 0, 0 },
    { 0xcdadd3cf, 88 },
    { 0x814dd3cf, 100 },
    { 0x7b3dd3d0, 24 },
    { 0xacdd3d2, 55 },
    { 0, 0 },
    { 0x127dd3d4, 76 },
    { 0xfbddd3d5, 90 },
    { 0x85add3d6, 34 },
    { 0x6f9dd3d7, 88 },
    { 0x914dd3d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0dd3db, 61 },
    { 0, 0 },
    { 0xedd3dd, 98 },
    { 0, 0 },
    { 0x152dd3df, 68 },
    { 0xe4fdd3df, 33 },
    { 0, 0 },
    { 0xf2ddd3e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x458dd3e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7dd3e8, 40 },
    { 0x83ddd3e8, 23 },
    { 0, 0 },
    { 0x98ddd3eb, 99 },
    { 0x354dd3ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddd3f2, 95 },
    { 0x521dd3f3, 90 },
    { 0xe70dd3f4, 87 },
    { 0x5dddd3f3, 68 },
    { 0xf18dd3f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574dd3fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fddd3fe, 46 },
    { 0xd2ddd3ff, 46 },
    { 0xca8dd3fe, 61 },
    { 0, 0 },
    { 0x27ddd402, 33 },
    { 0, 0 },
    { 0xad5dd404, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65dd407, 33 },
    { 0x743dd408, 91 },
    { 0xbebdd409, 61 },
    { 0x8aedd409, 90 },
    { 0x633dd40b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ddd40e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131dd415, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86dd418, 76 },
    { 0xf18dd419, 61 },
    { 0xb0dd41a, 33 },
    { 0x926dd418, 61 },
    { 0x34ddd418, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x778dd41f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63dd425, 100 },
    { 0x447dd426, 90 },
    { 0x492dd427, 55 },
    { 0xb48dd428, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332dd42c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8dd432, 84 },
    { 0x3e1dd432, 68 },
    { 0x646dd432, 33 },
    { 0x861dd435, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ddd438, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202dd43c, 43 },
    { 0x434dd43c, 90 },
    { 0x8b0dd43e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2dd443, 43 },
    { 0, 0 },
    { 0xdd445, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63dd44a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325dd452, 98 },
    { 0xee6dd452, 95 },
    { 0x388dd454, 55 },
    { 0x366dd455, 61 },
    { 0, 0 },
    { 0x3a8dd457, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ddd45e, 23 },
    { 0, 0 },
    { 0x1dfdd460, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6dd463, 85 },
    { 0xcf1dd463, 95 },
    { 0xabbdd465, 98 },
    { 0, 0 },
    { 0x45bdd467, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cddd46a, 77 },
    { 0, 0 },
    { 0x73bdd46c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabdd471, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7dd474, 90 },
    { 0x5dcdd475, 90 },
    { 0, 0 },
    { 0x2e2dd477, 46 },
    { 0x80fdd478, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574dd482, 99 },
    { 0xd2add483, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287dd488, 34 },
    { 0, 0 },
    { 0x676dd48a, 55 },
    { 0x156dd48b, 78 },
    { 0x28cdd48c, 66 },
    { 0, 0 },
    { 0x52add48e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x840dd491, 98 },
    { 0, 0 },
    { 0xddcdd493, 90 },
    { 0, 0 },
    { 0xd01dd495, 4 },
    { 0x5d6dd496, 91 },
    { 0x6b1dd497, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0dd49e, 34 },
    { 0x88cdd49f, 90 },
    { 0xe77dd4a0, 98 },
    { 0x49bdd4a1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cdd4a7, 43 },
    { 0, 0 },
    { 0x9d4dd4a9, 90 },
    { 0xdc9dd4aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440dd4b0, 52 },
    { 0x902dd4b1, 95 },
    { 0xf69dd4b2, 90 },
    { 0xaeedd4b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cedd4b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09dd4bc, 55 },
    { 0, 0 },
    { 0x1cadd4be, 78 },
    { 0, 0 },
    { 0x8ebdd4c0, 46 },
    { 0xf2cdd4c0, 93 },
    { 0xbefdd4c2, 87 },
    { 0x386dd4c0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccdd4cb, 90 },
    { 0xb0edd4cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbdd4d0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aedd4d7, 78 },
    { 0xc64dd4d8, 76 },
    { 0xbadd4d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162dd4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50add4e4, 81 },
    { 0x2bbdd4e5, 61 },
    { 0xb1fdd4e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5edd4e9, 61 },
    { 0, 0 },
    { 0x3e2dd4eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebdd4ee, 23 },
    { 0x271dd4ef, 76 },
    { 0x71dd4f0, 98 },
    { 0x228dd4f0, 33 },
    { 0, 0 },
    { 0xe96dd4f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98dd4f6, 95 },
    { 0x2b5dd4f7, 23 },
    { 0x4a2dd4f8, 93 },
    { 0, 0 },
    { 0x2dd4fa, 95 },
    { 0x837dd4fb, 52 },
    { 0x1acdd4fa, 23 },
    { 0xb2dd4fc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20dd506, 46 },
    { 0x4f7dd507, 9 },
    { 0, 0 },
    { 0xcf3dd509, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31dd50e, 90 },
    { 0x94add50f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69dd516, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174dd51b, 61 },
    { 0x89edd51b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16edd525, 95 },
    { 0xdaedd526, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edd530, 84 },
    { 0xb9fdd530, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cdd534, 90 },
    { 0, 0 },
    { 0xc3bdd536, 95 },
    { 0, 0 },
    { 0x786dd538, 33 },
    { 0x6fddd538, 33 },
    { 0x55cdd538, 34 },
    { 0x2a4dd53b, 98 },
    { 0x2d7dd53b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4dd540, 43 },
    { 0x552dd541, 95 },
    { 0, 0 },
    { 0x811dd543, 95 },
    { 0x18ddd543, 34 },
    { 0xd41dd543, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4dd548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63dd54e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0dd551, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51dd556, 55 },
    { 0x6dfdd557, 98 },
    { 0xf89dd557, 90 },
    { 0x61dd556, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68dd55f, 95 },
    { 0, 0 },
    { 0xb6add561, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ddd566, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1afdd56c, 95 },
    { 0x615dd56d, 67 },
    { 0x351dd56e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0dd575, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926dd580, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fdd585, 33 },
    { 0, 0 },
    { 0x3d1dd587, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30edd58c, 61 },
    { 0, 0 },
    { 0xd5fdd58e, 47 },
    { 0x538dd58e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadddd594, 76 },
    { 0x803dd595, 58 },
    { 0x901dd596, 61 },
    { 0x421dd597, 95 },
    { 0xa84dd595, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12dd59c, 61 },
    { 0xa44dd59d, 43 },
    { 0x444dd59d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fdd5a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68edd5a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x253dd5a7, 90 },
    { 0x833dd5a7, 23 },
    { 0, 0 },
    { 0xac1dd5aa, 85 },
    { 0xadddd5aa, 87 },
    { 0x9cedd5ac, 76 },
    { 0x288dd5ab, 91 },
    { 0xcd5dd5ae, 76 },
    { 0, 0 },
    { 0x19cdd5b0, 61 },
    { 0, 0 },
    { 0xf9add5b2, 75 },
    { 0xd51dd5b2, 98 },
    { 0x49dd5b4, 33 },
    { 0x2c1dd5b4, 61 },
    { 0xf4bdd5b6, 90 },
    { 0x66ddd5b5, 43 },
    { 0x40cdd5b5, 76 },
    { 0x670dd5b9, 33 },
    { 0x68cdd5b7, 90 },
    { 0x3e1dd5bb, 23 },
    { 0x4d7dd5bc, 88 },
    { 0xf34dd5b9, 84 },
    { 0x6f3dd5b8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08dd5c3, 66 },
    { 0xfe8dd5c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336dd5cc, 33 },
    { 0x6c1dd5cd, 90 },
    { 0, 0 },
    { 0x7b5dd5cf, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cddd5d4, 90 },
    { 0x9f3dd5d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x540dd5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bdd5df, 98 },
    { 0x748dd5e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x164dd5e3, 76 },
    { 0x299dd5e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x907dd5e7, 52 },
    { 0x28fdd5e8, 43 },
    { 0x4eedd5e9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ddd5ed, 76 },
    { 0, 0 },
    { 0x2d4dd5ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2dd5f2, 90 },
    { 0xd01dd5f2, 96 },
    { 0xea2dd5f2, 78 },
    { 0x9c6dd5f5, 76 },
    { 0, 0 },
    { 0x357dd5f7, 90 },
    { 0x927dd5f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133dd5fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9dd605, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x44add608, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6dd60b, 34 },
    { 0x93edd60c, 55 },
    { 0x7e6dd60b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dd610, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1dd617, 84 },
    { 0x48add618, 98 },
    { 0x67fdd618, 33 },
    { 0x76add61a, 55 },
    { 0x8a2dd61b, 90 },
    { 0, 0 },
    { 0xe38dd61d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dd621, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dd624, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4dd62c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fdd62f, 33 },
    { 0, 0 },
    { 0x5cadd631, 46 },
    { 0xd73dd631, 50 },
    { 0xf57dd631, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54dd638, 34 },
    { 0x70fdd639, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593dd63d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfdd646, 33 },
    { 0x57ddd647, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0add64a, 95 },
    { 0, 0 },
    { 0x8fadd64c, 90 },
    { 0, 0 },
    { 0xcb5dd64e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3dd653, 33 },
    { 0x8d9dd654, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadddd659, 69 },
    { 0x62cdd65a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7dd660, 8 },
    { 0, 0 },
    { 0x714dd662, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92dd669, 66 },
    { 0xa7cdd669, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dd66d, 81 },
    { 0x742dd66d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84dd671, 90 },
    { 0x1dadd672, 61 },
    { 0x5cedd671, 34 },
    { 0xeeedd673, 76 },
    { 0xe93dd675, 53 },
    { 0xc2bdd676, 43 },
    { 0xb2bdd676, 61 },
    { 0x628dd675, 90 },
    { 0, 0 },
    { 0x6d5dd67a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x282dd67d, 75 },
    { 0x1c1dd67d, 84 },
    { 0, 0 },
    { 0x32bdd680, 95 },
    { 0, 0 },
    { 0x4a1dd682, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fdd686, 61 },
    { 0, 0 },
    { 0xfc0dd688, 76 },
    { 0, 0 },
    { 0xb12dd68a, 78 },
    { 0x33cdd68a, 78 },
    { 0x3acdd68b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190dd690, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04dd693, 52 },
    { 0x233dd693, 33 },
    { 0, 0 },
    { 0xf6bdd696, 90 },
    { 0xb8edd697, 98 },
    { 0xfdcdd697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cdd69b, 90 },
    { 0x73edd69b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x785dd69f, 55 },
    { 0, 0 },
    { 0x498dd6a1, 61 },
    { 0x2e6dd6a2, 52 },
    { 0xf94dd6a1, 98 },
    { 0, 0 },
    { 0xa14dd6a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dedd6aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24dd6ad, 33 },
    { 0xbe1dd6ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159dd6b4, 83 },
    { 0xd94dd6b4, 67 },
    { 0x773dd6b6, 33 },
    { 0x9f7dd6b6, 23 },
    { 0x9fadd6b7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549dd6bd, 92 },
    { 0x16dd6be, 33 },
    { 0x34ddd6bf, 43 },
    { 0, 0 },
    { 0x58ddd6c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfdd6c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9dd6cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83dd6d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ddd6da, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cdd6dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2dd6e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310dd6ee, 90 },
    { 0, 0 },
    { 0x1b1dd6f0, 84 },
    { 0x2efdd6f1, 46 },
    { 0x998dd6f1, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916dd6f8, 43 },
    { 0x1badd6f9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fdd6fc, 98 },
    { 0xf37dd6fd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4dd701, 76 },
    { 0xedbdd702, 17 },
    { 0xaf6dd703, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbdd709, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dcdd70c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dd711, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68edd715, 99 },
    { 0, 0 },
    { 0xd05dd717, 46 },
    { 0x82ddd717, 98 },
    { 0, 0 },
    { 0x23bdd71a, 90 },
    { 0, 0 },
    { 0x61dd71c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fdd723, 87 },
    { 0x607dd724, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524dd72c, 95 },
    { 0x66add72d, 33 },
    { 0x672dd72d, 98 },
    { 0xc71dd72e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cdd732, 33 },
    { 0, 0 },
    { 0x8dcdd734, 43 },
    { 0x2b1dd734, 66 },
    { 0x8fcdd736, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8dd73d, 98 },
    { 0, 0 },
    { 0xfaadd73f, 23 },
    { 0x4d6dd740, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208dd749, 77 },
    { 0, 0 },
    { 0x8b9dd74b, 90 },
    { 0, 0 },
    { 0x636dd74d, 90 },
    { 0xb8edd74e, 43 },
    { 0x479dd74e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59add754, 85 },
    { 0, 0 },
    { 0xab1dd756, 61 },
    { 0, 0 },
    { 0x5f0dd758, 61 },
    { 0xd8edd758, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5dd763, 90 },
    { 0x67cdd763, 95 },
    { 0x310dd765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dd769, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x352dd76c, 61 },
    { 0, 0 },
    { 0x243dd76e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fdd771, 23 },
    { 0xc59dd771, 98 },
    { 0, 0 },
    { 0xd33dd774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbadd778, 68 },
    { 0x4ccdd779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fdd77c, 61 },
    { 0xd8ddd77d, 98 },
    { 0x61cdd77e, 33 },
    { 0, 0 },
    { 0x31fdd780, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fdd786, 23 },
    { 0x241dd787, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9dd78a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bdd78d, 76 },
    { 0xe79dd78d, 33 },
    { 0x815dd78f, 61 },
    { 0xfa1dd790, 91 },
    { 0xddfdd790, 61 },
    { 0xecdd792, 87 },
    { 0x67dd792, 93 },
    { 0x798dd790, 90 },
    { 0x2f4dd793, 23 },
    { 0x76edd790, 46 },
    { 0xe0dd793, 55 },
    { 0x19ddd798, 93 },
    { 0xdacdd792, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cdd79d, 90 },
    { 0, 0 },
    { 0x3ecdd79f, 98 },
    { 0xfc8dd7a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9dd7a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bdd7a7, 90 },
    { 0, 0 },
    { 0xc39dd7a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddd7ae, 36 },
    { 0xa58dd7af, 90 },
    { 0xde2dd7af, 55 },
    { 0x69dd7b1, 98 },
    { 0xc3cdd7b2, 61 },
    { 0x4d0dd7b2, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10add7b8, 52 },
    { 0xdc9dd7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44dd7bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabdd7c0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9dd7c4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x185dd7ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ddd7cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60dd7d0, 78 },
    { 0x169dd7d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfadd7d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x637dd7d9, 17 },
    { 0, 0 },
    { 0x4a4dd7db, 33 },
    { 0xecfdd7db, 91 },
    { 0xa9add7dd, 97 },
    { 0x39add7dd, 33 },
    { 0x8dddd7df, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cdd7e5, 90 },
    { 0x17edd7e6, 43 },
    { 0xddbdd7e7, 23 },
    { 0xb87dd7e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469dd7ec, 61 },
    { 0xba2dd7ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfddd7f6, 34 },
    { 0x241dd7f7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0dd7fd, 78 },
    { 0xaf6dd7fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fdd806, 90 },
    { 0x26bdd806, 87 },
    { 0xdebdd806, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cdd80c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5dd813, 95 },
    { 0x211dd814, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5dd817, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2dd81e, 95 },
    { 0xfe2dd81e, 44 },
    { 0xfa6dd820, 55 },
    { 0x23fdd820, 95 },
    { 0x3ffdd822, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409dd827, 33 },
    { 0, 0 },
    { 0xf1bdd829, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x265dd82c, 94 },
    { 0x2cedd82d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cddd830, 33 },
    { 0x4badd831, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6dd835, 46 },
    { 0xfb8dd836, 91 },
    { 0xa58dd835, 34 },
    { 0x3eadd835, 93 },
    { 0x417dd838, 90 },
    { 0x28dd83a, 66 },
    { 0x907dd83a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9dd843, 98 },
    { 0xfb1dd844, 98 },
    { 0x720dd844, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9dd848, 90 },
    { 0, 0 },
    { 0x492dd84a, 95 },
    { 0x141dd84a, 83 },
    { 0, 0 },
    { 0x509dd84d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45dd856, 46 },
    { 0, 0 },
    { 0x4afdd858, 90 },
    { 0xdabdd859, 95 },
    { 0xd4ddd858, 8 },
    { 0, 0 },
    { 0x615dd85c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bdd860, 90 },
    { 0, 0 },
    { 0x6a5dd862, 67 },
    { 0xfb3dd862, 98 },
    { 0x51fdd864, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06dd867, 33 },
    { 0xc3fdd868, 81 },
    { 0x5f6dd869, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfdd86e, 46 },
    { 0xa4ddd86f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781dd87a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3dd87e, 40 },
    { 0x43cdd87f, 87 },
    { 0x76dd880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bdd883, 33 },
    { 0xfdfdd884, 90 },
    { 0x374dd885, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x863dd888, 95 },
    { 0x396dd889, 33 },
    { 0xb67dd88a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdd88e, 61 },
    { 0xfc2dd88e, 98 },
    { 0xc51dd890, 90 },
    { 0x98add890, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ddd895, 33 },
    { 0x3e8dd895, 33 },
    { 0, 0 },
    { 0xe83dd898, 90 },
    { 0x65add899, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81dd89c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30dd8a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724dd8b0, 51 },
    { 0x395dd8b0, 33 },
    { 0xe1bdd8b1, 93 },
    { 0x313dd8b0, 98 },
    { 0xf4fdd8b4, 76 },
    { 0x51add8b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cadd8b8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6dd8be, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9dd8c4, 90 },
    { 0, 0 },
    { 0x502dd8c6, 33 },
    { 0x5e1dd8c7, 53 },
    { 0x89cdd8c6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8dd8cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94dd8cf, 90 },
    { 0x689dd8d0, 95 },
    { 0xadd8d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39dd8d4, 23 },
    { 0, 0 },
    { 0xa30dd8d6, 95 },
    { 0x593dd8d7, 61 },
    { 0, 0 },
    { 0xc9add8d9, 95 },
    { 0x335dd8da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x615dd8dd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5dd8e5, 61 },
    { 0x2ccdd8e6, 95 },
    { 0xbaadd8e7, 34 },
    { 0xfeddd8e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157dd8ed, 46 },
    { 0xc0cdd8ee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663dd8f8, 90 },
    { 0x46cdd8f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0dd8fd, 95 },
    { 0x7a7dd8fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4dd902, 61 },
    { 0x2b3dd903, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dd90c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bdd90f, 93 },
    { 0, 0 },
    { 0xcbcdd911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3dd915, 65 },
    { 0xb5add915, 40 },
    { 0x514dd915, 98 },
    { 0xfbddd917, 90 },
    { 0x9e9dd919, 61 },
    { 0x8c6dd919, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1dd921, 49 },
    { 0, 0 },
    { 0x214dd923, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8dd927, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x738dd92a, 33 },
    { 0x6f1dd92a, 33 },
    { 0x673dd92c, 33 },
    { 0x525dd92c, 90 },
    { 0x5eedd92c, 34 },
    { 0x3fbdd92c, 91 },
    { 0x21fdd930, 95 },
    { 0x40dd92f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81dd934, 90 },
    { 0xd27dd934, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35dd939, 90 },
    { 0x64cdd93a, 99 },
    { 0x5e1dd93b, 98 },
    { 0x2f5dd93b, 90 },
    { 0x73ddd93d, 95 },
    { 0x6ccdd93d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1dd943, 90 },
    { 0x721dd944, 90 },
    { 0, 0 },
    { 0xf08dd946, 55 },
    { 0x296dd946, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910dd955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fdd958, 98 },
    { 0xf99dd959, 93 },
    { 0, 0 },
    { 0xcf7dd95b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3dd95f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39dd962, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57dd965, 34 },
    { 0, 0 },
    { 0x802dd967, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x456dd96a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480dd973, 81 },
    { 0xd7ddd973, 23 },
    { 0xf08dd975, 85 },
    { 0xc7edd975, 90 },
    { 0x5dddd974, 81 },
    { 0x9eddd978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0edd97e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5dd98a, 23 },
    { 0xe0add98b, 95 },
    { 0x819dd98b, 95 },
    { 0xbd5dd98c, 95 },
    { 0x37fdd98e, 61 },
    { 0xfc3dd98f, 95 },
    { 0, 0 },
    { 0x5bcdd991, 93 },
    { 0x6fbdd992, 55 },
    { 0, 0 },
    { 0x8c5dd994, 99 },
    { 0x3ecdd995, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0dd999, 46 },
    { 0x9d4dd999, 34 },
    { 0, 0 },
    { 0xf6fdd99c, 33 },
    { 0x579dd99c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0dd9a7, 98 },
    { 0x347dd9a8, 23 },
    { 0xbd9dd9a9, 76 },
    { 0x8aadd9aa, 37 },
    { 0x808dd9a7, 98 },
    { 0, 0 },
    { 0x261dd9ad, 23 },
    { 0x506dd9ae, 61 },
    { 0, 0 },
    { 0xb67dd9b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5dd9b7, 52 },
    { 0x485dd9b8, 66 },
    { 0xffadd9b9, 66 },
    { 0x1f9dd9b9, 95 },
    { 0, 0 },
    { 0xccfdd9bc, 98 },
    { 0x144dd9bd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32dd9cc, 90 },
    { 0, 0 },
    { 0x24edd9ce, 98 },
    { 0x30cdd9ce, 34 },
    { 0, 0 },
    { 0x913dd9d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38dd9d4, 90 },
    { 0, 0 },
    { 0xf29dd9d6, 90 },
    { 0xea3dd9d6, 68 },
    { 0xddedd9d8, 33 },
    { 0x3b4dd9d9, 33 },
    { 0x298dd9da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0edd9dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5dd9e0, 84 },
    { 0xc54dd9e1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16dd9ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9dd9ef, 33 },
    { 0x283dd9f0, 33 },
    { 0xf85dd9f1, 90 },
    { 0xbc7dd9f1, 90 },
    { 0xb28dd9f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffdd9f6, 92 },
    { 0, 0 },
    { 0x5a0dd9f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2dd9fe, 33 },
    { 0x702dd9ff, 55 },
    { 0xfc3dd9ff, 95 },
    { 0xa99dd9ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1dda05, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbdda09, 90 },
    { 0x904dda0a, 77 },
    { 0, 0 },
    { 0xa60dda0c, 78 },
    { 0x775dda0c, 55 },
    { 0xbe6dda0e, 95 },
    { 0x3f4dda0e, 55 },
    { 0xfabdda0d, 55 },
    { 0x14edda0d, 90 },
    { 0xe23dda0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bdda19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53bdda1c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535dda20, 46 },
    { 0, 0 },
    { 0x14adda22, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7dda29, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8dda2f, 33 },
    { 0xbfcdda30, 67 },
    { 0xdc7dda30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cdda39, 33 },
    { 0x6b6dda3a, 78 },
    { 0xca0dda3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490dda44, 90 },
    { 0x857dda45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x643dda48, 99 },
    { 0, 0 },
    { 0xe5dda4a, 33 },
    { 0x49adda4a, 95 },
    { 0x2a8dda4c, 87 },
    { 0x4c4dda4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91dda52, 33 },
    { 0, 0 },
    { 0x363dda54, 87 },
    { 0, 0 },
    { 0x82bdda56, 99 },
    { 0xcd1dda57, 90 },
    { 0, 0 },
    { 0xcbddda59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331dda5f, 76 },
    { 0xa5dda60, 33 },
    { 0xb5edda61, 78 },
    { 0xb22dda60, 84 },
    { 0, 0 },
    { 0xd31dda64, 90 },
    { 0xd16dda64, 33 },
    { 0xf56dda65, 55 },
    { 0, 0 },
    { 0xe40dda68, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5dda6d, 23 },
    { 0, 0 },
    { 0xeeddda6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35dda73, 61 },
    { 0, 0 },
    { 0x35edda75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54dda79, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x826dda7c, 68 },
    { 0, 0 },
    { 0xcabdda7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933dda82, 23 },
    { 0xd9bdda82, 95 },
    { 0x895dda84, 93 },
    { 0xc08dda84, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573dda97, 17 },
    { 0x674dda97, 95 },
    { 0xf91dda99, 55 },
    { 0x46edda98, 95 },
    { 0x9c0dda99, 46 },
    { 0x5e9dda9b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4ddaa1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48ddaa6, 61 },
    { 0xa47ddaa6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57ddaab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8ddab2, 76 },
    { 0xa49ddab3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cddab6, 100 },
    { 0x7e0ddab7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fddabb, 76 },
    { 0xd6cddabb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3ddac3, 61 },
    { 0, 0 },
    { 0xac1ddac5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaddac8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcddacf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1ddad7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1eddadd, 9 },
    { 0x6c7ddadd, 23 },
    { 0, 0 },
    { 0x5b6ddae0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fddae5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3ddaec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x182ddaef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x735ddaf2, 98 },
    { 0xdf4ddaf3, 33 },
    { 0, 0 },
    { 0xa42ddaf5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9ddaf9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2ddb00, 52 },
    { 0xbdcddb01, 91 },
    { 0x478ddb02, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ddb08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ddb0b, 90 },
    { 0x789ddb0c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdddb11, 90 },
    { 0, 0 },
    { 0xd4fddb13, 78 },
    { 0, 0 },
    { 0xb54ddb15, 43 },
    { 0xbbddb16, 91 },
    { 0xc31ddb17, 90 },
    { 0, 0 },
    { 0x196ddb19, 33 },
    { 0x14addb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cddb1d, 66 },
    { 0x5fdddb1e, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x47dddb21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfddb24, 85 },
    { 0, 0 },
    { 0x2e0ddb26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442ddb2b, 61 },
    { 0x9ebddb2c, 98 },
    { 0xd57ddb2d, 87 },
    { 0, 0 },
    { 0x648ddb2f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eddb33, 99 },
    { 0xb73ddb34, 33 },
    { 0, 0 },
    { 0x54bddb36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882ddb3b, 76 },
    { 0x790ddb3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fddb40, 76 },
    { 0x4aeddb40, 93 },
    { 0x395ddb42, 23 },
    { 0xbf7ddb42, 98 },
    { 0x363ddb44, 98 },
    { 0x913ddb44, 99 },
    { 0x4feddb44, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6ddb51, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0ddb56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12ddb5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fddb5e, 23 },
    { 0, 0 },
    { 0xa6addb60, 61 },
    { 0, 0 },
    { 0x218ddb62, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08ddb68, 36 },
    { 0x160ddb69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbddb71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bddb75, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fddb7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850ddb86, 43 },
    { 0x616ddb87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbddb8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adddb8e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fddb95, 95 },
    { 0x872ddb96, 33 },
    { 0xe79ddb97, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bddb9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afddb9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dddba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3ddba5, 90 },
    { 0, 0 },
    { 0xc89ddba7, 76 },
    { 0, 0 },
    { 0xba0ddba9, 76 },
    { 0x5f1ddba9, 98 },
    { 0xfdcddba9, 33 },
    { 0xe8fddbac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cddbb2, 52 },
    { 0x38dddbb2, 43 },
    { 0x2a1ddbb4, 95 },
    { 0x499ddbb5, 33 },
    { 0xafddbb6, 33 },
    { 0xfc4ddbb7, 43 },
    { 0x843ddbb8, 43 },
    { 0, 0 },
    { 0xe08ddbba, 61 },
    { 0xe4dddbbb, 23 },
    { 0xfbaddbbc, 52 },
    { 0x95fddbbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2ddbc0, 61 },
    { 0xf65ddbc0, 99 },
    { 0x36bddbc2, 95 },
    { 0xc24ddbc3, 88 },
    { 0x4d4ddbc4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fddbc8, 87 },
    { 0xf4addbc9, 95 },
    { 0xc81ddbca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34cddbcf, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974ddbda, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ddbde, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ddbe9, 61 },
    { 0xaafddbea, 90 },
    { 0, 0 },
    { 0xb54ddbec, 77 },
    { 0x361ddbed, 33 },
    { 0, 0 },
    { 0x8cdddbef, 61 },
    { 0, 0 },
    { 0xae7ddbf1, 55 },
    { 0xd2cddbf2, 81 },
    { 0xe2fddbf2, 91 },
    { 0xbd6ddbf2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12ddbf8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bddbfc, 66 },
    { 0, 0 },
    { 0xb2dddbfe, 99 },
    { 0x41cddbff, 76 },
    { 0x72addc00, 61 },
    { 0xcd4ddc01, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ddc08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09ddc0b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4addc0f, 23 },
    { 0x1a2ddc10, 93 },
    { 0x888ddc10, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840ddc18, 93 },
    { 0, 0 },
    { 0xaa7ddc1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0ddc1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fddc22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20ddc2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3ddc2e, 90 },
    { 0xbc3ddc2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6ddc36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9addc39, 55 },
    { 0, 0 },
    { 0xbc8ddc3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422ddc3f, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaddc44, 23 },
    { 0x1c5ddc45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ddc49, 23 },
    { 0x7a2ddc4a, 23 },
    { 0x9bfddc4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ddc4f, 81 },
    { 0x14ddc50, 76 },
    { 0x2fcddc50, 64 },
    { 0xf5bddc52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1ddc55, 66 },
    { 0xdc3ddc56, 33 },
    { 0x956ddc57, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abddc5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ddc5e, 95 },
    { 0, 0 },
    { 0x244ddc60, 90 },
    { 0xfdeddc61, 90 },
    { 0, 0 },
    { 0x56eddc63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ddc66, 90 },
    { 0, 0 },
    { 0x9d3ddc68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefddc6c, 95 },
    { 0xb6cddc6c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5ddc70, 61 },
    { 0xdf0ddc71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43ddc75, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7ddc79, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fddc7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799ddc87, 23 },
    { 0xf7ddc87, 34 },
    { 0x6afddc89, 33 },
    { 0x6aeddc8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cddc8d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eddc91, 98 },
    { 0, 0 },
    { 0x56dddc93, 100 },
    { 0x2dddc94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ddc9a, 52 },
    { 0xae7ddc9b, 98 },
    { 0x3a0ddc9c, 76 },
    { 0, 0 },
    { 0xfe3ddc9e, 98 },
    { 0x7fcddc9f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261ddca5, 95 },
    { 0x35cddca6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8ddcaa, 65 },
    { 0xa3addcab, 43 },
    { 0x3a5ddcac, 33 },
    { 0xec9ddcad, 55 },
    { 0, 0 },
    { 0x638ddcaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503ddcb3, 95 },
    { 0x881ddcb4, 95 },
    { 0x519ddcb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67ddcbb, 55 },
    { 0, 0 },
    { 0x782ddcbd, 55 },
    { 0xcddddcbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381ddcc6, 23 },
    { 0, 0 },
    { 0xf22ddcc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6ddccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37ddcd2, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8ddcdd, 33 },
    { 0x956ddcde, 66 },
    { 0x201ddcde, 23 },
    { 0, 0 },
    { 0xf98ddce1, 33 },
    { 0x270ddce2, 43 },
    { 0, 0 },
    { 0x8aeddce4, 90 },
    { 0xbe3ddce5, 90 },
    { 0xff0ddce6, 61 },
    { 0x7c0ddce5, 40 },
    { 0x6c4ddce4, 88 },
    { 0x510ddce6, 55 },
    { 0x5ebddcea, 95 },
    { 0xb63ddce9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6ddcf0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8ddcf4, 88 },
    { 0xd53ddcf4, 90 },
    { 0xa7bddcf5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02ddcfe, 98 },
    { 0x73fddcfe, 34 },
    { 0x574ddd00, 33 },
    { 0xca5ddd01, 52 },
    { 0x458ddd01, 76 },
    { 0xeceddd02, 61 },
    { 0x97bddd03, 76 },
    { 0xc6dddd01, 76 },
    { 0x37eddd05, 95 },
    { 0x64dddd06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3ddd0e, 85 },
    { 0x636ddd0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fddd14, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0ddd17, 17 },
    { 0xac7ddd17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eddd20, 98 },
    { 0, 0 },
    { 0x4afddd22, 95 },
    { 0x6f0ddd23, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cddd28, 61 },
    { 0x3e2ddd28, 78 },
    { 0, 0 },
    { 0xf5cddd2b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67ddd2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ddd33, 4 },
    { 0x23bddd34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2ddd37, 78 },
    { 0x2dbddd38, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890ddd3e, 68 },
    { 0, 0 },
    { 0x420ddd40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdddd48, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x269ddd4b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ddd4f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dddd55, 33 },
    { 0, 0 },
    { 0xda7ddd57, 23 },
    { 0x10addd58, 23 },
    { 0xac7ddd59, 61 },
    { 0xe29ddd5a, 66 },
    { 0xd9ddd59, 34 },
    { 0x27eddd5b, 61 },
    { 0x7a2ddd5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70ddd60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffddd66, 76 },
    { 0x171ddd67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ddd73, 23 },
    { 0x517ddd74, 90 },
    { 0x77fddd75, 61 },
    { 0xe6cddd76, 99 },
    { 0x627ddd77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dddd7b, 61 },
    { 0x5c9ddd7c, 23 },
    { 0xe8eddd7c, 34 },
    { 0, 0 },
    { 0x66eddd7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x291ddd82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24ddd85, 90 },
    { 0, 0 },
    { 0xdabddd87, 76 },
    { 0, 0 },
    { 0x336ddd89, 52 },
    { 0x574ddd8a, 52 },
    { 0x4e1ddd89, 52 },
    { 0x8ceddd8c, 90 },
    { 0, 0 },
    { 0xd26ddd8e, 33 },
    { 0xdb0ddd8e, 98 },
    { 0x9bcddd90, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4ddd97, 83 },
    { 0x387ddd98, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7ddd9d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bddda3, 95 },
    { 0x208ddda4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9fddda9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5dddaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefdddb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0adddb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8dddba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4dddbe, 61 },
    { 0, 0 },
    { 0xd87dddc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35adddc5, 76 },
    { 0x1bbdddc5, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8edddca, 87 },
    { 0xba3dddca, 66 },
    { 0x95adddcb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fdddcf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512dddd6, 90 },
    { 0x6e9dddd7, 61 },
    { 0xf96dddd8, 85 },
    { 0x424dddd9, 33 },
    { 0xc4bdddda, 87 },
    { 0xc08ddddb, 23 },
    { 0xa6bddddc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3ddddf, 23 },
    { 0x887ddddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cddde3, 71 },
    { 0x112ddde4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1ddde7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6dddeb, 76 },
    { 0x91dddec, 77 },
    { 0xa2fddded, 90 },
    { 0xb50dddeb, 98 },
    { 0xa95ddded, 90 },
    { 0, 0 },
    { 0xa27dddf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35dddf5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bdddf8, 87 },
    { 0x9adddf9, 66 },
    { 0xa55dddf8, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bdddfd, 95 },
    { 0, 0 },
    { 0x685dddff, 61 },
    { 0x6b5dde00, 61 },
    { 0x389dde01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7dde06, 93 },
    { 0xe6fdde06, 61 },
    { 0, 0 },
    { 0xfbcdde09, 98 },
    { 0xb7fdde0a, 33 },
    { 0x8e2dde09, 93 },
    { 0xa94dde09, 100 },
    { 0x354dde0d, 52 },
    { 0xab9dde0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70adde17, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fdde1a, 95 },
    { 0xccedde1b, 95 },
    { 0x54cdde1c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bdde23, 93 },
    { 0x44ddde24, 81 },
    { 0x27dde25, 93 },
    { 0xa4edde25, 61 },
    { 0, 0 },
    { 0xedde28, 88 },
    { 0x457dde29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bdde2d, 98 },
    { 0x617dde2e, 98 },
    { 0, 0 },
    { 0xfcdde30, 61 },
    { 0x98dde30, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1dde38, 90 },
    { 0x89edde39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bddde3c, 98 },
    { 0, 0 },
    { 0xe50dde3e, 23 },
    { 0x14adde3f, 46 },
    { 0xbfedde3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1badde47, 33 },
    { 0xd7adde47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25dde4e, 98 },
    { 0, 0 },
    { 0x22cdde50, 34 },
    { 0x1e7dde50, 91 },
    { 0, 0 },
    { 0xdaadde53, 61 },
    { 0x163dde53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cadde5b, 23 },
    { 0, 0 },
    { 0x4fdde5d, 55 },
    { 0x57adde5e, 75 },
    { 0xf48dde5f, 33 },
    { 0x824dde5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ddde64, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdde68, 77 },
    { 0x5ecdde68, 95 },
    { 0, 0 },
    { 0x5b5dde6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x574dde6e, 55 },
    { 0, 0 },
    { 0x4ccdde70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fdde76, 6 },
    { 0xa71dde77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde7c, 61 },
    { 0x913dde7c, 33 },
    { 0, 0 },
    { 0xfbedde7f, 23 },
    { 0, 0 },
    { 0x589dde81, 76 },
    { 0x1f1dde81, 78 },
    { 0xb1bdde82, 90 },
    { 0x31bdde84, 98 },
    { 0xd70dde81, 64 },
    { 0x581dde85, 90 },
    { 0x510dde87, 90 },
    { 0xb66dde88, 98 },
    { 0x443dde81, 90 },
    { 0xaeddde84, 90 },
    { 0xad3dde87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7dde8e, 86 },
    { 0x6efdde8f, 99 },
    { 0xff5dde90, 100 },
    { 0xff6dde8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826dde9b, 34 },
    { 0, 0 },
    { 0xb76dde9d, 95 },
    { 0x338dde9d, 61 },
    { 0, 0 },
    { 0x2bdddea0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ddea3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81addeab, 23 },
    { 0, 0 },
    { 0x513ddead, 76 },
    { 0x9aeddeae, 95 },
    { 0xc70ddeae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ddeba, 95 },
    { 0xc17ddeba, 99 },
    { 0x73dddebc, 98 },
    { 0, 0 },
    { 0x37fddebe, 38 },
    { 0xfb8ddebf, 33 },
    { 0x1daddec0, 91 },
    { 0xb97ddec1, 88 },
    { 0x864ddec1, 43 },
    { 0xf81ddec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5ddec7, 55 },
    { 0, 0 },
    { 0x190ddec9, 90 },
    { 0x68cddec9, 87 },
    { 0x7c6ddec9, 34 },
    { 0x7a7ddecc, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdddedb, 33 },
    { 0x99fddedc, 76 },
    { 0x99bddedb, 93 },
    { 0xe8addedb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80ddeea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dddef1, 95 },
    { 0x896ddef2, 40 },
    { 0x66fddef3, 92 },
    { 0x439ddef3, 98 },
    { 0, 0 },
    { 0x678ddef6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eddefe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceaddf02, 95 },
    { 0x89ddf03, 91 },
    { 0xcffddf04, 78 },
    { 0, 0 },
    { 0xb69ddf06, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389ddf0a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fddf10, 61 },
    { 0x6f3ddf10, 33 },
    { 0x297ddf12, 88 },
    { 0x6e4ddf13, 98 },
    { 0x780ddf14, 93 },
    { 0xc7eddf15, 53 },
    { 0xd4fddf14, 43 },
    { 0, 0 },
    { 0x16bddf18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ddf1c, 33 },
    { 0, 0 },
    { 0x1afddf1e, 95 },
    { 0x10eddf1f, 95 },
    { 0x95fddf20, 95 },
    { 0, 0 },
    { 0xc1bddf22, 34 },
    { 0, 0 },
    { 0x6bfddf24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ddf2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2ddf2e, 55 },
    { 0, 0 },
    { 0xb4fddf30, 47 },
    { 0, 0 },
    { 0xa2dddf32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49ddf36, 33 },
    { 0, 0 },
    { 0x701ddf38, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5addf3d, 61 },
    { 0xd48ddf3e, 78 },
    { 0x238ddf3f, 90 },
    { 0xcaeddf3d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224ddf44, 55 },
    { 0xf5ddf44, 33 },
    { 0xf70ddf44, 33 },
    { 0xcdaddf47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fddf4e, 90 },
    { 0, 0 },
    { 0xd92ddf50, 91 },
    { 0x19ddf50, 52 },
    { 0, 0 },
    { 0x1e8ddf53, 68 },
    { 0, 0 },
    { 0x17cddf55, 97 },
    { 0x14dddf56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8ddf59, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ddf60, 55 },
    { 0x95bddf61, 33 },
    { 0, 0 },
    { 0xa47ddf63, 98 },
    { 0, 0 },
    { 0x725ddf65, 34 },
    { 0, 0 },
    { 0x5e2ddf67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddddf6a, 68 },
    { 0xf00ddf6a, 98 },
    { 0x337ddf6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7ddf70, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0ddf77, 61 },
    { 0x62cddf77, 90 },
    { 0xad5ddf79, 23 },
    { 0, 0 },
    { 0xc8eddf7b, 61 },
    { 0x1f0ddf7b, 76 },
    { 0, 0 },
    { 0x1d8ddf7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x991ddf81, 67 },
    { 0xd05ddf82, 55 },
    { 0, 0 },
    { 0xc11ddf84, 46 },
    { 0xfddf85, 52 },
    { 0x8a9ddf84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ddf8a, 33 },
    { 0xdf8ddf8b, 98 },
    { 0xc10ddf8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x594ddf8f, 99 },
    { 0x1a0ddf90, 46 },
    { 0xe3fddf90, 90 },
    { 0x545ddf92, 55 },
    { 0, 0 },
    { 0x996ddf94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3ddf97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2feddfa3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ddfa9, 100 },
    { 0x2bddfaa, 61 },
    { 0x452ddfaa, 61 },
    { 0x4a3ddfac, 66 },
    { 0xe58ddfad, 95 },
    { 0xf74ddfaa, 61 },
    { 0x602ddfa9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114ddfb4, 52 },
    { 0x379ddfb5, 33 },
    { 0, 0 },
    { 0x93ddfb7, 95 },
    { 0x430ddfb8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160ddfc0, 76 },
    { 0x9dcddfc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ddfc8, 55 },
    { 0, 0 },
    { 0x631ddfca, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf73ddfcf, 46 },
    { 0x2d9ddfd0, 34 },
    { 0x47fddfd1, 93 },
    { 0, 0 },
    { 0x92dddfd3, 95 },
    { 0x525ddfd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ddfd8, 61 },
    { 0x1cddfd9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ddfe0, 98 },
    { 0x104ddfe1, 95 },
    { 0, 0 },
    { 0xda3ddfe3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eddfe6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635ddfed, 84 },
    { 0xb4eddfee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19ddff1, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1ddff4, 33 },
    { 0x660ddff5, 90 },
    { 0, 0 },
    { 0x71addff7, 23 },
    { 0, 0 },
    { 0xb94ddff9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02ddffe, 76 },
    { 0, 0 },
    { 0x64cde000, 91 },
    { 0xc2bde000, 76 },
    { 0xe3dde001, 33 },
    { 0x7dede003, 88 },
    { 0, 0 },
    { 0x440de005, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdde00a, 78 },
    { 0x2dcde00b, 98 },
    { 0xa5bde00b, 55 },
    { 0x55ade00a, 100 },
    { 0x61dde00e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1de011, 76 },
    { 0x78de012, 53 },
    { 0, 0 },
    { 0x859de014, 95 },
    { 0xa26de014, 95 },
    { 0xe71de014, 90 },
    { 0, 0 },
    { 0x5a6de018, 46 },
    { 0xe17de018, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2de01d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fde021, 93 },
    { 0xdd9de022, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694de026, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dde029, 55 },
    { 0x113de02a, 61 },
    { 0x8cde02b, 90 },
    { 0x46cde02c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dde033, 88 },
    { 0, 0 },
    { 0x2a8de035, 91 },
    { 0x7a0de036, 34 },
    { 0, 0 },
    { 0x8cde038, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffde03e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51de043, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x453de04a, 38 },
    { 0, 0 },
    { 0x793de04c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bde052, 100 },
    { 0x132de053, 23 },
    { 0x740de054, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dde057, 55 },
    { 0x7f7de058, 83 },
    { 0x169de059, 91 },
    { 0x1dade05a, 66 },
    { 0xf14de05a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59de06c, 93 },
    { 0xf60de06d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0de070, 44 },
    { 0, 0 },
    { 0xebade072, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7de075, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83de078, 66 },
    { 0, 0 },
    { 0x5dfde07a, 34 },
    { 0, 0 },
    { 0x6eade07c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ede07f, 61 },
    { 0xeeade080, 90 },
    { 0x6b6de081, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ade086, 95 },
    { 0x7c3de087, 61 },
    { 0, 0 },
    { 0x453de089, 23 },
    { 0x116de089, 80 },
    { 0, 0 },
    { 0xf83de08c, 80 },
    { 0, 0 },
    { 0x38bde08e, 90 },
    { 0, 0 },
    { 0xadcde090, 77 },
    { 0x42de090, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x523de094, 90 },
    { 0, 0 },
    { 0xc06de096, 34 },
    { 0, 0 },
    { 0xade098, 33 },
    { 0x27dde099, 55 },
    { 0, 0 },
    { 0xa0cde09b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ade09e, 61 },
    { 0, 0 },
    { 0x8ede0a0, 90 },
    { 0x749de0a1, 95 },
    { 0x42cde0a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3de0a5, 61 },
    { 0xc19de0a5, 95 },
    { 0, 0 },
    { 0x457de0a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48de0ab, 76 },
    { 0xb56de0ac, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393de0b0, 90 },
    { 0, 0 },
    { 0x1aede0b2, 23 },
    { 0xc26de0b3, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x457de0b9, 61 },
    { 0xa9ade0ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ede0c4, 61 },
    { 0xe9ade0c5, 90 },
    { 0x13fde0c4, 61 },
    { 0xe60de0c5, 33 },
    { 0x619de0c7, 55 },
    { 0x3c3de0c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5de0ce, 61 },
    { 0x19fde0ce, 98 },
    { 0, 0 },
    { 0xbbade0d1, 90 },
    { 0xbc3de0d2, 33 },
    { 0x22ade0d2, 23 },
    { 0x5f5de0d4, 55 },
    { 0x1c7de0d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x715de0d8, 86 },
    { 0x7e8de0d8, 34 },
    { 0xbcdde0d9, 61 },
    { 0, 0 },
    { 0x1edde0dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27de0e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849de0e8, 33 },
    { 0x3bde0e9, 61 },
    { 0xe42de0e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1de0ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3de0f5, 91 },
    { 0, 0 },
    { 0x66ade0f7, 98 },
    { 0xd6dde0f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de0fb, 43 },
    { 0, 0 },
    { 0x7e6de0fd, 33 },
    { 0x3dede0fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7de104, 90 },
    { 0x21de104, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81de10a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x608de10d, 76 },
    { 0x918de10e, 33 },
    { 0x7a1de10f, 90 },
    { 0x12dde10f, 98 },
    { 0x528de111, 61 },
    { 0xea2de111, 66 },
    { 0x40dde110, 76 },
    { 0xc9bde10e, 55 },
    { 0x19bde115, 33 },
    { 0, 0 },
    { 0x3ffde117, 52 },
    { 0, 0 },
    { 0xd3ade119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ade121, 99 },
    { 0x656de122, 90 },
    { 0, 0 },
    { 0xecde124, 90 },
    { 0x9fcde125, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1de128, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6de12c, 34 },
    { 0xdb3de12d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7de131, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ade134, 98 },
    { 0x8a4de135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ade13b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6de142, 43 },
    { 0x73ede142, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829de148, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ede14b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9de14f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51de156, 98 },
    { 0x964de157, 67 },
    { 0x9aede156, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cde15b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fde15e, 90 },
    { 0xb39de15f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8de16c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4de170, 55 },
    { 0x5f3de170, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7de175, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390de17f, 84 },
    { 0xddfde180, 93 },
    { 0, 0 },
    { 0x42dde182, 23 },
    { 0x8b5de183, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319de187, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544de18c, 90 },
    { 0x62fde18d, 78 },
    { 0, 0 },
    { 0xf2dde18f, 23 },
    { 0xf1fde18f, 100 },
    { 0xea9de191, 81 },
    { 0, 0 },
    { 0xbcfde193, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420de198, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742de19e, 61 },
    { 0xf2fde19f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbde1a4, 88 },
    { 0x2e7de1a5, 34 },
    { 0x296de1a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x26dde1a9, 62 },
    { 0x5cfde1a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fde1ad, 85 },
    { 0x6d4de1ae, 77 },
    { 0xfdde1af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79de1b2, 81 },
    { 0xe0bde1b3, 95 },
    { 0x19ede1b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x798de1be, 95 },
    { 0xb1ede1bf, 95 },
    { 0xd1ede1c0, 77 },
    { 0xc97de1c0, 90 },
    { 0xb9dde1be, 75 },
    { 0xcd5de1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641de1c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2dde1cd, 99 },
    { 0xf29de1ce, 61 },
    { 0xa43de1cf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9de1d3, 76 },
    { 0xde0de1d3, 43 },
    { 0xe51de1d3, 99 },
    { 0, 0 },
    { 0xef4de1d7, 43 },
    { 0, 0 },
    { 0xe3ade1d9, 91 },
    { 0x95cde1da, 96 },
    { 0, 0 },
    { 0xbcede1dc, 43 },
    { 0xb69de1dd, 61 },
    { 0xce1de1de, 23 },
    { 0x102de1de, 34 },
    { 0x452de1e0, 33 },
    { 0xe3de1e0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414de1e8, 33 },
    { 0, 0 },
    { 0xf7ede1ea, 43 },
    { 0x6dcde1eb, 34 },
    { 0, 0 },
    { 0xa17de1ed, 98 },
    { 0, 0 },
    { 0xdb6de1ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcde1f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ade1fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348de202, 33 },
    { 0x958de203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdde206, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13de212, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fde217, 23 },
    { 0xf04de217, 100 },
    { 0x88ede219, 61 },
    { 0x78fde21a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x438de21d, 98 },
    { 0x5c2de21e, 34 },
    { 0, 0 },
    { 0x54ade220, 23 },
    { 0x4f2de220, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791de22e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6de231, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfde235, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bde23c, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2de240, 68 },
    { 0xa3ede241, 23 },
    { 0x134de242, 55 },
    { 0x95dde240, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23de24c, 87 },
    { 0, 0 },
    { 0x72ade24e, 65 },
    { 0x616de24e, 61 },
    { 0, 0 },
    { 0x6bcde251, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81de259, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0cde25c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dde25f, 90 },
    { 0, 0 },
    { 0x968de261, 95 },
    { 0x635de261, 76 },
    { 0, 0 },
    { 0xe54de264, 23 },
    { 0x250de264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5de26b, 87 },
    { 0x8f7de26c, 23 },
    { 0, 0 },
    { 0x128de26e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24de271, 100 },
    { 0, 0 },
    { 0x7ffde273, 88 },
    { 0x8b5de274, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4de278, 33 },
    { 0xd28de279, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecde27c, 66 },
    { 0xcc7de27d, 55 },
    { 0, 0 },
    { 0x2e1de27f, 55 },
    { 0x211de280, 88 },
    { 0x5b2de281, 98 },
    { 0, 0 },
    { 0x55ade283, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ede287, 33 },
    { 0xad6de287, 90 },
    { 0xd08de289, 93 },
    { 0, 0 },
    { 0xc8ede28b, 90 },
    { 0, 0 },
    { 0xf45de28d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfde296, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5de29a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687de29e, 55 },
    { 0, 0 },
    { 0xda0de2a0, 23 },
    { 0xf37de2a0, 46 },
    { 0xde1de2a2, 76 },
    { 0x818de2a3, 78 },
    { 0x839de2a4, 61 },
    { 0x9edde2a5, 93 },
    { 0, 0 },
    { 0xfe8de2a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954de2ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62de2b2, 80 },
    { 0, 0 },
    { 0xf60de2b4, 43 },
    { 0, 0 },
    { 0x195de2b6, 61 },
    { 0x9d7de2b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03de2bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cde2be, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dde2c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83de2c4, 95 },
    { 0, 0 },
    { 0x6bde2c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1de2cc, 90 },
    { 0x2ecde2cd, 85 },
    { 0xf59de2ce, 85 },
    { 0x3c4de2cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30de2d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dde2d9, 33 },
    { 0x2cfde2da, 98 },
    { 0x3efde2da, 76 },
    { 0x773de2d9, 33 },
    { 0xdcdde2dd, 61 },
    { 0xcade2d9, 95 },
    { 0, 0 },
    { 0x137de2e0, 65 },
    { 0xd15de2e0, 87 },
    { 0, 0 },
    { 0xa09de2e3, 34 },
    { 0, 0 },
    { 0x84ade2e5, 91 },
    { 0x8a4de2e6, 98 },
    { 0xbffde2e5, 61 },
    { 0x4c4de2e8, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacde2eb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dde2ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afde2f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ede2f8, 61 },
    { 0, 0 },
    { 0x741de2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68de2fd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fede300, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0de305, 52 },
    { 0x455de306, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8de309, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6de30c, 76 },
    { 0x9e5de30c, 76 },
    { 0, 0 },
    { 0x4fbde30f, 61 },
    { 0xbe2de310, 100 },
    { 0, 0 },
    { 0x57cde312, 46 },
    { 0x907de313, 90 },
    { 0x697de314, 84 },
    { 0x304de312, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x844de318, 99 },
    { 0xe2cde319, 98 },
    { 0, 0 },
    { 0xd9cde31b, 34 },
    { 0xa98de31c, 23 },
    { 0, 0 },
    { 0x2bcde31e, 90 },
    { 0x63de31f, 17 },
    { 0x8a0de31e, 23 },
    { 0xce7de321, 87 },
    { 0x600de31e, 61 },
    { 0, 0 },
    { 0x90ede324, 95 },
    { 0, 0 },
    { 0xf81de326, 84 },
    { 0xda4de327, 32 },
    { 0xa0ade328, 33 },
    { 0, 0 },
    { 0xb26de32a, 61 },
    { 0, 0 },
    { 0x70ade32c, 23 },
    { 0x425de32c, 66 },
    { 0x67de32e, 34 },
    { 0xdffde32f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575de336, 90 },
    { 0x1f8de337, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3de33a, 90 },
    { 0x2abde33a, 98 },
    { 0x2bede33c, 66 },
    { 0x8bde33c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0de340, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767de349, 55 },
    { 0x904de34a, 33 },
    { 0xc60de34b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690de350, 98 },
    { 0, 0 },
    { 0x934de352, 78 },
    { 0xd9bde352, 99 },
    { 0x7dcde354, 34 },
    { 0x914de355, 33 },
    { 0, 0 },
    { 0x18bde357, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62de35a, 55 },
    { 0xa7bde35b, 33 },
    { 0, 0 },
    { 0x1f8de35d, 33 },
    { 0x6f5de35e, 95 },
    { 0, 0 },
    { 0x38dde360, 87 },
    { 0x25ede361, 23 },
    { 0xc78de361, 34 },
    { 0x520de363, 66 },
    { 0x23cde364, 61 },
    { 0xf1de363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fde368, 95 },
    { 0, 0 },
    { 0xdf4de36a, 90 },
    { 0, 0 },
    { 0xdecde36c, 90 },
    { 0x365de36d, 90 },
    { 0xbc5de36e, 90 },
    { 0xb5de36f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbede376, 90 },
    { 0x999de377, 61 },
    { 0x136de378, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64de37b, 93 },
    { 0, 0 },
    { 0xdde37d, 61 },
    { 0xc77de37e, 90 },
    { 0, 0 },
    { 0x63ade380, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de388, 76 },
    { 0xf93de388, 76 },
    { 0, 0 },
    { 0x609de38b, 33 },
    { 0xb6ede38b, 90 },
    { 0xb0bde38c, 43 },
    { 0x3d9de38e, 98 },
    { 0xc5cde38b, 33 },
    { 0xe78de38b, 33 },
    { 0x2d2de391, 52 },
    { 0x28fde392, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecde395, 78 },
    { 0, 0 },
    { 0x9acde397, 98 },
    { 0x67bde398, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7de3a1, 98 },
    { 0, 0 },
    { 0x81de3a3, 95 },
    { 0x73cde3a4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282de3aa, 61 },
    { 0, 0 },
    { 0x80bde3ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cde3b4, 61 },
    { 0xa7de3b4, 90 },
    { 0, 0 },
    { 0xff6de3b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956de3bb, 90 },
    { 0x2a9de3bb, 95 },
    { 0xcf7de3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3de3c0, 81 },
    { 0xbc5de3c0, 61 },
    { 0x602de3c0, 64 },
    { 0x742de3c3, 61 },
    { 0x4bede3c3, 95 },
    { 0, 0 },
    { 0x287de3c6, 95 },
    { 0x564de3c6, 75 },
    { 0xfe9de3c8, 84 },
    { 0, 0 },
    { 0x478de3ca, 95 },
    { 0, 0 },
    { 0xb7ede3cc, 98 },
    { 0, 0 },
    { 0xad1de3ce, 90 },
    { 0xc22de3ce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7de3d4, 91 },
    { 0xb5fde3d5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9de3d9, 43 },
    { 0x650de3da, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4de3df, 33 },
    { 0xf73de3e0, 66 },
    { 0x6bfde3e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x438de3e4, 95 },
    { 0xefade3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ede3e8, 95 },
    { 0x7dbde3e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ade3ec, 34 },
    { 0, 0 },
    { 0x4dcde3ee, 30 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ede3f1, 95 },
    { 0, 0 },
    { 0x8f0de3f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x297de3f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0de3f9, 98 },
    { 0, 0 },
    { 0xc86de3fb, 76 },
    { 0xbeade3fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0de3ff, 43 },
    { 0x5b8de400, 98 },
    { 0x214de401, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6de406, 66 },
    { 0xe02de407, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366de40c, 95 },
    { 0, 0 },
    { 0xedde40e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14de412, 76 },
    { 0, 0 },
    { 0xefde414, 76 },
    { 0, 0 },
    { 0x740de416, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bde419, 95 },
    { 0x7edde41a, 33 },
    { 0, 0 },
    { 0x881de41c, 52 },
    { 0xc87de41d, 61 },
    { 0x299de41c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fade421, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635de425, 55 },
    { 0xd7cde426, 61 },
    { 0xe1fde426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3de42c, 90 },
    { 0x5fde42d, 76 },
    { 0xb22de42e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dde433, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fde436, 95 },
    { 0, 0 },
    { 0xddfde438, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20de444, 52 },
    { 0x25ade444, 87 },
    { 0, 0 },
    { 0x371de447, 33 },
    { 0x1abde447, 90 },
    { 0xb40de448, 23 },
    { 0xc0ede44a, 40 },
    { 0, 0 },
    { 0x1e7de44c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8de450, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5de453, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fde456, 33 },
    { 0x1eede456, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93de45e, 69 },
    { 0xae7de45f, 90 },
    { 0, 0 },
    { 0xb44de461, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35de467, 33 },
    { 0x98ade468, 90 },
    { 0xfc3de469, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2de46d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cde471, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1de474, 56 },
    { 0, 0 },
    { 0xcd1de476, 61 },
    { 0xe6dde477, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73de47a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41de47d, 52 },
    { 0x9c0de47e, 66 },
    { 0x934de47f, 95 },
    { 0x272de47e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518de48c, 23 },
    { 0x415de48d, 95 },
    { 0, 0 },
    { 0xf47de48f, 90 },
    { 0, 0 },
    { 0xc35de491, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0de497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdde4a3, 23 },
    { 0, 0 },
    { 0x1a6de4a5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de4aa, 55 },
    { 0xec6de4aa, 23 },
    { 0, 0 },
    { 0x8a0de4ad, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22fde4b6, 59 },
    { 0xad9de4b7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382de4bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1de4c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520de4c7, 46 },
    { 0x665de4c8, 23 },
    { 0x66bde4c8, 43 },
    { 0x931de4c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62dde4ce, 83 },
    { 0x207de4ce, 23 },
    { 0, 0 },
    { 0x918de4d1, 33 },
    { 0x725de4d2, 61 },
    { 0xd78de4d2, 85 },
    { 0x642de4d4, 33 },
    { 0x21fde4d5, 61 },
    { 0x4b0de4d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73de4d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafade4dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eede4e4, 33 },
    { 0, 0 },
    { 0xd00de4e6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10de4eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1de4f1, 88 },
    { 0x745de4f1, 91 },
    { 0x9ebde4f3, 75 },
    { 0xa34de4f4, 77 },
    { 0xe09de4f4, 68 },
    { 0, 0 },
    { 0xa06de4f7, 87 },
    { 0xe8dde4f8, 78 },
    { 0x2c7de4f9, 100 },
    { 0, 0 },
    { 0x36cde4fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebde4ff, 96 },
    { 0x7f1de4ff, 95 },
    { 0, 0 },
    { 0x467de502, 33 },
    { 0x795de503, 69 },
    { 0x220de502, 61 },
    { 0xf25de504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ade509, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333de50d, 61 },
    { 0, 0 },
    { 0xdb9de50f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfde512, 90 },
    { 0, 0 },
    { 0xaa3de514, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ede518, 52 },
    { 0x470de519, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53de51f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6de523, 93 },
    { 0xfc4de524, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1de528, 33 },
    { 0, 0 },
    { 0xab8de52a, 23 },
    { 0, 0 },
    { 0x78dde52c, 43 },
    { 0x669de52d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ede531, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bde53b, 66 },
    { 0, 0 },
    { 0x9f4de53d, 52 },
    { 0, 0 },
    { 0x543de53f, 98 },
    { 0, 0 },
    { 0x52ede541, 61 },
    { 0xa0fde542, 33 },
    { 0, 0 },
    { 0x482de544, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bde54c, 61 },
    { 0xdc6de54d, 23 },
    { 0x1b2de54e, 78 },
    { 0xdedde54f, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fde552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477de557, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6de55a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x532de561, 87 },
    { 0x55dde562, 33 },
    { 0x815de563, 88 },
    { 0x86bde563, 75 },
    { 0x2bbde561, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedde568, 98 },
    { 0x705de569, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37fde56d, 90 },
    { 0x87dde56d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66de571, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34de578, 98 },
    { 0xa2cde578, 95 },
    { 0x1b2de57a, 90 },
    { 0xa7de57a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37de57e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26de582, 43 },
    { 0, 0 },
    { 0xa67de584, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfde589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97de58c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621de591, 76 },
    { 0, 0 },
    { 0x7f4de593, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cde59a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7de5a2, 90 },
    { 0, 0 },
    { 0x8c0de5a4, 93 },
    { 0xed9de5a5, 61 },
    { 0x114de5a4, 61 },
    { 0, 0 },
    { 0x43ede5a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479de5ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607de5b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x553de5b9, 99 },
    { 0x73dde5ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cde5bd, 67 },
    { 0, 0 },
    { 0x713de5bf, 38 },
    { 0xde0de5bf, 98 },
    { 0x484de5c1, 23 },
    { 0xb81de5bf, 65 },
    { 0xe11de5c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254de5c8, 98 },
    { 0x3b7de5c9, 93 },
    { 0xede5c9, 99 },
    { 0x6e8de5cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5de5d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4de5d3, 43 },
    { 0, 0 },
    { 0x26ade5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8de5dd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3de5e3, 95 },
    { 0, 0 },
    { 0x632de5e5, 66 },
    { 0xcb2de5e6, 61 },
    { 0, 0 },
    { 0x43cde5e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bde5ee, 23 },
    { 0x46de5ef, 34 },
    { 0x315de5f0, 66 },
    { 0, 0 },
    { 0x352de5f2, 33 },
    { 0x2b7de5f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfde5fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fede5ff, 67 },
    { 0x502de600, 23 },
    { 0xd0de600, 61 },
    { 0xfa5de602, 23 },
    { 0x2a8de603, 81 },
    { 0xb23de603, 90 },
    { 0xd0de603, 95 },
    { 0xbf5de606, 98 },
    { 0xa7bde607, 46 },
    { 0, 0 },
    { 0x6b2de609, 47 },
    { 0xf84de60a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ede60e, 93 },
    { 0x5adde60f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0de612, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5de616, 61 },
    { 0xfc0de616, 23 },
    { 0xb24de618, 55 },
    { 0, 0 },
    { 0x9de61a, 76 },
    { 0, 0 },
    { 0xb3ade61c, 61 },
    { 0x2fcde61c, 84 },
    { 0xec7de61e, 95 },
    { 0, 0 },
    { 0x114de620, 80 },
    { 0xddde621, 87 },
    { 0x27fde621, 33 },
    { 0xc9ade620, 23 },
    { 0, 0 },
    { 0x2d0de625, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x988de629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ede62d, 76 },
    { 0x972de62e, 98 },
    { 0x2c9de62f, 61 },
    { 0x71cde62f, 98 },
    { 0xc9fde62e, 33 },
    { 0x194de632, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4de63a, 98 },
    { 0xb86de63a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39de640, 33 },
    { 0x597de641, 97 },
    { 0x397de642, 33 },
    { 0, 0 },
    { 0xe8bde644, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cde64b, 33 },
    { 0xa5cde64c, 90 },
    { 0, 0 },
    { 0x158de64e, 90 },
    { 0, 0 },
    { 0xc24de650, 90 },
    { 0xe93de650, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321de65e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1de663, 23 },
    { 0, 0 },
    { 0x417de665, 91 },
    { 0x399de666, 95 },
    { 0, 0 },
    { 0xda7de668, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaade66c, 33 },
    { 0, 0 },
    { 0x13ede66e, 95 },
    { 0, 0 },
    { 0x359de670, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50de673, 100 },
    { 0, 0 },
    { 0x823de675, 61 },
    { 0xc7dde675, 34 },
    { 0x72de677, 95 },
    { 0x453de678, 55 },
    { 0, 0 },
    { 0x490de67a, 61 },
    { 0xe50de67b, 55 },
    { 0xd0ade67b, 100 },
    { 0xe23de67a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fde682, 91 },
    { 0xbcfde683, 34 },
    { 0xedde682, 55 },
    { 0xc6dde685, 76 },
    { 0, 0 },
    { 0x8de687, 77 },
    { 0, 0 },
    { 0xebede689, 84 },
    { 0x58fde68a, 55 },
    { 0, 0 },
    { 0xf46de68c, 76 },
    { 0xd5ede68d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5de690, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ede693, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ade69d, 93 },
    { 0, 0 },
    { 0xce0de69f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5bde6a3, 47 },
    { 0xed2de6a4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede6a9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fade6ac, 90 },
    { 0xc44de6ad, 90 },
    { 0x6a8de6ae, 98 },
    { 0x905de6ae, 61 },
    { 0x99ede6b0, 46 },
    { 0x130de6b0, 91 },
    { 0, 0 },
    { 0x2eede6b3, 33 },
    { 0xb8cde6b4, 90 },
    { 0x338de6b5, 43 },
    { 0x8b9de6b4, 99 },
    { 0xbbde6b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74de6bc, 98 },
    { 0xdcfde6bc, 84 },
    { 0xb6de6be, 90 },
    { 0x1f5de6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x319de6c2, 98 },
    { 0x9d8de6c2, 95 },
    { 0x15ade6c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cde6c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeede6ca, 33 },
    { 0x545de6ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadde6ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x267de6d1, 23 },
    { 0x1bfde6d1, 66 },
    { 0x572de6d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67de6d8, 31 },
    { 0x11cde6d9, 100 },
    { 0, 0 },
    { 0x6a2de6db, 33 },
    { 0x158de6dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99de6e3, 90 },
    { 0x295de6e4, 34 },
    { 0xbf4de6e5, 61 },
    { 0x7e4de6e6, 85 },
    { 0x8a8de6e7, 61 },
    { 0xefede6e8, 23 },
    { 0xbd2de6e7, 95 },
    { 0xb94de6e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cde6fb, 90 },
    { 0, 0 },
    { 0xbb6de6fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adde700, 98 },
    { 0x87bde701, 61 },
    { 0, 0 },
    { 0xbc2de703, 61 },
    { 0xc7de704, 90 },
    { 0, 0 },
    { 0xe31de706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x772de709, 90 },
    { 0xf4bde709, 61 },
    { 0x1d1de70b, 55 },
    { 0xf60de70c, 33 },
    { 0x9acde70d, 34 },
    { 0x474de70e, 84 },
    { 0x34de70e, 23 },
    { 0x6edde710, 98 },
    { 0, 0 },
    { 0xb52de712, 17 },
    { 0xb89de712, 64 },
    { 0, 0 },
    { 0x57bde715, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfde718, 33 },
    { 0x92dde719, 90 },
    { 0, 0 },
    { 0x469de71b, 76 },
    { 0, 0 },
    { 0xacbde71d, 61 },
    { 0x971de71d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1de721, 90 },
    { 0, 0 },
    { 0x27ade723, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352de72e, 76 },
    { 0xb02de72f, 43 },
    { 0xbf9de72f, 90 },
    { 0, 0 },
    { 0x81cde732, 98 },
    { 0x126de733, 98 },
    { 0xc07de734, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173de738, 98 },
    { 0xa0de739, 33 },
    { 0x5d5de73a, 52 },
    { 0x6c6de73a, 33 },
    { 0x214de73c, 76 },
    { 0xd86de73b, 33 },
    { 0, 0 },
    { 0x39ede73f, 82 },
    { 0xf6de740, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x737de749, 90 },
    { 0x487de74a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258de750, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477de754, 33 },
    { 0x66cde754, 98 },
    { 0xb6fde756, 55 },
    { 0x428de757, 33 },
    { 0x74fde758, 43 },
    { 0x6dbde758, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505de75f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146de76b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fde76e, 78 },
    { 0x2dcde76e, 90 },
    { 0x7a4de770, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4de775, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ede77a, 76 },
    { 0xf2bde77a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7de77e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x442de781, 23 },
    { 0, 0 },
    { 0xebade783, 95 },
    { 0x9fdde783, 33 },
    { 0, 0 },
    { 0xd41de786, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1de78a, 66 },
    { 0, 0 },
    { 0xd9bde78c, 61 },
    { 0x206de78d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376de793, 53 },
    { 0x143de793, 68 },
    { 0, 0 },
    { 0xe8ade796, 95 },
    { 0x38bde796, 33 },
    { 0x4adde798, 99 },
    { 0xfa0de799, 67 },
    { 0xe8ade79a, 95 },
    { 0xec8de79a, 34 },
    { 0xadbde79c, 36 },
    { 0xf72de79d, 33 },
    { 0x92cde79e, 98 },
    { 0xd2de79f, 87 },
    { 0xb97de7a0, 84 },
    { 0, 0 },
    { 0xce7de7a2, 43 },
    { 0x315de7a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1de7a6, 66 },
    { 0, 0 },
    { 0x31fde7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ade7b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ede7b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61de7bb, 33 },
    { 0x88ede7bc, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdde7c0, 34 },
    { 0, 0 },
    { 0x20dde7c2, 40 },
    { 0xefbde7c3, 46 },
    { 0, 0 },
    { 0xaf2de7c5, 64 },
    { 0xd90de7c6, 99 },
    { 0x842de7c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfde7ca, 33 },
    { 0x655de7ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fde7cf, 95 },
    { 0, 0 },
    { 0xdc0de7d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694de7dd, 98 },
    { 0xe37de7de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8de7e8, 95 },
    { 0, 0 },
    { 0x8b2de7ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7de7f1, 91 },
    { 0, 0 },
    { 0x8cde7f3, 43 },
    { 0x7c9de7f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20de7f7, 68 },
    { 0xe43de7f8, 55 },
    { 0x853de7f8, 61 },
    { 0, 0 },
    { 0x91fde7fb, 23 },
    { 0x732de7fb, 66 },
    { 0xb0ede7fc, 33 },
    { 0xfeede7fc, 23 },
    { 0x9b8de7ff, 9 },
    { 0xf62de7fe, 90 },
    { 0x34ede801, 91 },
    { 0xcbde802, 90 },
    { 0x542de803, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0de80a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2de80f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707de819, 96 },
    { 0, 0 },
    { 0xd0dde81b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8de81f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdede828, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x825de82b, 23 },
    { 0, 0 },
    { 0x6d9de82d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9de836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ede83a, 52 },
    { 0xfa8de83a, 34 },
    { 0xb79de83c, 55 },
    { 0x8bfde83d, 66 },
    { 0x199de83d, 99 },
    { 0x6a8de83f, 42 },
    { 0xbde840, 76 },
    { 0, 0 },
    { 0x1a0de842, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ade847, 23 },
    { 0xfbbde848, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ade84b, 33 },
    { 0x7e2de84b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804de852, 61 },
    { 0, 0 },
    { 0x124de854, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcde85c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9de860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1de865, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5de869, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de86d, 95 },
    { 0x4b2de86e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbade872, 95 },
    { 0xb49de873, 61 },
    { 0xc14de873, 34 },
    { 0, 0 },
    { 0x97ede876, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebde882, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcde88a, 95 },
    { 0x98fde88a, 23 },
    { 0x100de88a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37de893, 90 },
    { 0xf50de893, 33 },
    { 0x9c7de895, 97 },
    { 0x986de893, 43 },
    { 0xaa8de897, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12de89b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6de89e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ade8a6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345de8ab, 98 },
    { 0, 0 },
    { 0x42ade8ad, 95 },
    { 0xe6bde8ae, 75 },
    { 0xb99de8af, 46 },
    { 0x217de8b0, 52 },
    { 0xfa6de8b1, 43 },
    { 0, 0 },
    { 0x138de8b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfde8b9, 61 },
    { 0x674de8ba, 95 },
    { 0x96ade8bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ade8be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbede8c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2de8c7, 77 },
    { 0, 0 },
    { 0xdb2de8c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10de8cc, 76 },
    { 0, 0 },
    { 0x58bde8ce, 85 },
    { 0x536de8ce, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfde8d3, 99 },
    { 0, 0 },
    { 0x26cde8d5, 98 },
    { 0, 0 },
    { 0x3abde8d7, 69 },
    { 0x4b2de8d7, 33 },
    { 0x57fde8d9, 55 },
    { 0x3de8da, 46 },
    { 0xa0fde8d8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8de8df, 76 },
    { 0x3bfde8df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eede8e7, 76 },
    { 0xb60de8e7, 81 },
    { 0x207de8e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2bde8ec, 33 },
    { 0x79dde8ed, 43 },
    { 0x6a1de8ec, 23 },
    { 0x47de8ef, 95 },
    { 0, 0 },
    { 0xb3ede8f1, 44 },
    { 0x50de8f2, 33 },
    { 0, 0 },
    { 0xc1dde8f4, 91 },
    { 0xcacde8f5, 69 },
    { 0x4c6de8f6, 23 },
    { 0x5d0de8f4, 98 },
    { 0, 0 },
    { 0x98dde8f9, 23 },
    { 0x6dade8f9, 95 },
    { 0, 0 },
    { 0x3cfde8fc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ade902, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x809de905, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x696de908, 90 },
    { 0x334de909, 95 },
    { 0x759de90a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ede911, 76 },
    { 0x8b9de912, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbade917, 90 },
    { 0xa54de918, 55 },
    { 0x8a8de918, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412de91d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ade923, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527de927, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29de92b, 100 },
    { 0x910de92c, 43 },
    { 0x280de92d, 61 },
    { 0x40bde92e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x327de931, 43 },
    { 0xa37de932, 98 },
    { 0x368de932, 61 },
    { 0xa87de934, 98 },
    { 0xaa0de934, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26de93b, 99 },
    { 0x273de93b, 34 },
    { 0, 0 },
    { 0x652de93e, 46 },
    { 0, 0 },
    { 0xff2de940, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164de944, 95 },
    { 0x1aede945, 87 },
    { 0x8e2de945, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ede94c, 23 },
    { 0x68ede94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56de951, 33 },
    { 0x20dde951, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bde957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeede95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ede961, 61 },
    { 0x24de962, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5de967, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889de96f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abde972, 17 },
    { 0x938de973, 90 },
    { 0x653de974, 23 },
    { 0x92fde975, 95 },
    { 0xc18de976, 90 },
    { 0x190de976, 23 },
    { 0xfacde978, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eade97b, 90 },
    { 0, 0 },
    { 0x2edde97d, 61 },
    { 0xaa0de97e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256de982, 33 },
    { 0xc09de983, 76 },
    { 0x5c3de984, 90 },
    { 0x307de985, 34 },
    { 0, 0 },
    { 0x503de987, 98 },
    { 0, 0 },
    { 0xa28de989, 4 },
    { 0xe22de98a, 23 },
    { 0x295de98b, 23 },
    { 0xb57de98a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39de98f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de997, 61 },
    { 0xd5ade998, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97de99b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28de99f, 95 },
    { 0x13cde99f, 61 },
    { 0x49ade9a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06de9a6, 76 },
    { 0, 0 },
    { 0x569de9a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdde9ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcde9ae, 55 },
    { 0, 0 },
    { 0x590de9b0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ade9b3, 95 },
    { 0x410de9b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3de9b8, 95 },
    { 0xbe8de9b8, 99 },
    { 0, 0 },
    { 0xe4ede9bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13de9be, 33 },
    { 0xe3ade9be, 33 },
    { 0x388de9c0, 81 },
    { 0xfdade9c1, 33 },
    { 0, 0 },
    { 0x98de9c3, 95 },
    { 0x9f0de9c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccde9c7, 93 },
    { 0, 0 },
    { 0x555de9c9, 23 },
    { 0xa93de9ca, 62 },
    { 0xdf4de9c9, 76 },
    { 0x376de9cc, 23 },
    { 0, 0 },
    { 0x97dde9ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108de9d3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1de9d7, 75 },
    { 0xa56de9d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145de9de, 98 },
    { 0, 0 },
    { 0x84dde9e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63de9e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6de9ec, 100 },
    { 0x1b9de9ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2de9f1, 55 },
    { 0xf79de9f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9de9f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3de9f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7bde9fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1adea00, 33 },
    { 0xdb5dea00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1dea04, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6dea07, 78 },
    { 0xacdea07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3badea10, 33 },
    { 0, 0 },
    { 0xb2bdea12, 93 },
    { 0, 0 },
    { 0x525dea14, 23 },
    { 0x84ddea15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5dea1a, 98 },
    { 0xf64dea1b, 33 },
    { 0x7cbdea1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dea1f, 55 },
    { 0x32dea1f, 23 },
    { 0x414dea1f, 23 },
    { 0, 0 },
    { 0xa93dea23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931dea28, 87 },
    { 0, 0 },
    { 0xda1dea2a, 90 },
    { 0xbf6dea2a, 90 },
    { 0xff6dea2c, 96 },
    { 0x379dea2c, 98 },
    { 0x4d3dea2c, 100 },
    { 0x434dea2b, 96 },
    { 0x3aedea2d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dea33, 80 },
    { 0x601dea33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cbdea39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940dea3d, 34 },
    { 0x649dea3d, 17 },
    { 0x7d8dea3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63edea49, 91 },
    { 0, 0 },
    { 0xdc0dea4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3dea51, 100 },
    { 0, 0 },
    { 0x312dea53, 93 },
    { 0, 0 },
    { 0x195dea55, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fdea5c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dea61, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ddea65, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x685dea68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32dea6b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3dea71, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223dea76, 81 },
    { 0x718dea76, 33 },
    { 0x6b3dea78, 98 },
    { 0x9b8dea79, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80edea7f, 90 },
    { 0, 0 },
    { 0x431dea81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1dea84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338dea8a, 34 },
    { 0xa12dea8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9dea8f, 33 },
    { 0xb59dea8f, 82 },
    { 0x1ffdea91, 52 },
    { 0xd66dea92, 83 },
    { 0xb27dea91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3dea96, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ddea9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8dea9d, 76 },
    { 0, 0 },
    { 0x75edea9f, 98 },
    { 0xbdea9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4deaa8, 90 },
    { 0xfcbdeaa8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4deaad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273deab3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7deab6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6deabb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x863deabe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7deac1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x462deac4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8deac9, 98 },
    { 0x248deac9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89deacd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cdead8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeedeadf, 23 },
    { 0xa49deadf, 67 },
    { 0xcdfdeadf, 17 },
    { 0x1dbdeae2, 55 },
    { 0, 0 },
    { 0xc73deae4, 23 },
    { 0xe3deae5, 78 },
    { 0xd8adeae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ddeafd, 46 },
    { 0, 0 },
    { 0x67cdeaff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ddeb04, 85 },
    { 0xe61deb04, 55 },
    { 0xd3adeb05, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506deb11, 33 },
    { 0x7adeb11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762deb16, 33 },
    { 0, 0 },
    { 0x182deb18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79deb1c, 95 },
    { 0x839deb1d, 98 },
    { 0, 0 },
    { 0xee2deb1f, 23 },
    { 0xf27deb20, 95 },
    { 0x120deb21, 87 },
    { 0, 0 },
    { 0x7dadeb23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bdeb26, 95 },
    { 0xfe0deb27, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeedeb2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdddeb32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdeb35, 61 },
    { 0, 0 },
    { 0x342deb37, 40 },
    { 0x99fdeb37, 23 },
    { 0x1d5deb38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3deb3c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510deb40, 95 },
    { 0x19deb41, 90 },
    { 0, 0 },
    { 0x333deb43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bdeb47, 93 },
    { 0x5d9deb48, 49 },
    { 0xbffdeb49, 90 },
    { 0, 0 },
    { 0x82cdeb4b, 83 },
    { 0xb27deb4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abdeb50, 33 },
    { 0, 0 },
    { 0x26bdeb52, 55 },
    { 0xa23deb52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1deb56, 76 },
    { 0xa56deb56, 66 },
    { 0x3b9deb58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579deb5e, 33 },
    { 0x639deb5f, 61 },
    { 0xc02deb60, 99 },
    { 0x4e7deb61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774deb67, 55 },
    { 0, 0 },
    { 0xf43deb69, 55 },
    { 0x18fdeb6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64deb6d, 99 },
    { 0, 0 },
    { 0x7afdeb6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98deb77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ddeb7b, 34 },
    { 0, 0 },
    { 0xbccdeb7d, 61 },
    { 0x520deb7e, 90 },
    { 0x4a9deb7d, 99 },
    { 0x677deb7d, 91 },
    { 0xf4adeb81, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394deb85, 33 },
    { 0x496deb86, 85 },
    { 0x180deb86, 98 },
    { 0xf58deb85, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2deb91, 93 },
    { 0xbfddeb91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23deb9c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0deba2, 55 },
    { 0, 0 },
    { 0x1c2deba4, 23 },
    { 0x531deba4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabdeba8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1badebab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53adebaf, 33 },
    { 0x8ffdebaf, 100 },
    { 0x830debaf, 33 },
    { 0xc67debb2, 90 },
    { 0x60cdebb3, 98 },
    { 0xdb6debb3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77debba, 88 },
    { 0xb70debba, 46 },
    { 0x9c6debbc, 75 },
    { 0, 0 },
    { 0x39ddebbe, 23 },
    { 0, 0 },
    { 0x5cfdebc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x293debc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcdebc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9debcc, 66 },
    { 0, 0 },
    { 0x77edebce, 61 },
    { 0x267debce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33debd2, 85 },
    { 0, 0 },
    { 0xb13debd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2adebda, 99 },
    { 0x83debdb, 40 },
    { 0, 0 },
    { 0x3b1debdd, 98 },
    { 0x5c5debde, 61 },
    { 0xdb0debdf, 90 },
    { 0x74cdebdd, 43 },
    { 0xe4bdebde, 33 },
    { 0xa2edebe0, 99 },
    { 0xe7ddebe0, 84 },
    { 0x790debe4, 90 },
    { 0x90bdebe5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1debe8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69debec, 75 },
    { 0xdebec, 98 },
    { 0, 0 },
    { 0xb23debef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732debf6, 76 },
    { 0xea1debf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684debfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15debfe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74adec06, 98 },
    { 0, 0 },
    { 0xb46dec08, 90 },
    { 0, 0 },
    { 0x287dec0a, 52 },
    { 0x3d5dec0b, 98 },
    { 0, 0 },
    { 0x826dec0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x258dec10, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x5addec13, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4edec1c, 23 },
    { 0xa74dec1c, 91 },
    { 0x1a0dec1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbdec21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bdec24, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efdec28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4dec30, 95 },
    { 0, 0 },
    { 0xb7adec32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8dec38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7dec3b, 75 },
    { 0, 0 },
    { 0x945dec3d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7dec40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4dec44, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34dec47, 93 },
    { 0x5dec48, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0dec4e, 23 },
    { 0xa09dec4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcdec53, 90 },
    { 0xd4fdec54, 90 },
    { 0x40edec53, 87 },
    { 0x82ddec55, 90 },
    { 0x6d8dec54, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322dec5d, 91 },
    { 0, 0 },
    { 0xe55dec5f, 94 },
    { 0xddddec60, 76 },
    { 0x719dec61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78dec65, 90 },
    { 0xa64dec65, 56 },
    { 0xa5cdec67, 90 },
    { 0x4f6dec67, 23 },
    { 0, 0 },
    { 0x2e0dec6a, 93 },
    { 0, 0 },
    { 0xad9dec6c, 61 },
    { 0x8bcdec6d, 33 },
    { 0xf9dec6e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fddec72, 78 },
    { 0xad4dec72, 101 },
    { 0xd05dec72, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdec79, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x371dec7c, 33 },
    { 0x878dec7d, 76 },
    { 0xecdec7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffdec82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbedec87, 23 },
    { 0xa7fdec88, 93 },
    { 0xc7fdec87, 46 },
    { 0, 0 },
    { 0xd8dec8b, 90 },
    { 0, 0 },
    { 0x7d2dec8d, 23 },
    { 0x5bcdec8e, 81 },
    { 0x68ddec8f, 23 },
    { 0x7addec8f, 17 },
    { 0xd26dec91, 33 },
    { 0x2c1dec8f, 98 },
    { 0x4fcdec92, 23 },
    { 0xf43dec94, 91 },
    { 0x2e8dec94, 90 },
    { 0x477dec96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ddec99, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7dec9e, 91 },
    { 0x975dec9f, 77 },
    { 0x7fddeca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ddeca3, 90 },
    { 0xc6ddeca4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fdecb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ddecb5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2decbd, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7decc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x199deccb, 86 },
    { 0, 0 },
    { 0x67deccd, 33 },
    { 0, 0 },
    { 0xa9adeccf, 61 },
    { 0xb81decd0, 43 },
    { 0, 0 },
    { 0x32fdecd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7decd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660decdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edece2, 52 },
    { 0, 0 },
    { 0xf80dece4, 33 },
    { 0, 0 },
    { 0x684dece6, 23 },
    { 0x854dece7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdeceb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ddecf5, 61 },
    { 0x9d8decf6, 61 },
    { 0xdb2decf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8decfd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7ded00, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ded03, 23 },
    { 0, 0 },
    { 0x842ded05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7ded0c, 55 },
    { 0x1f2ded0d, 85 },
    { 0xaacded0e, 66 },
    { 0xd0bded0f, 90 },
    { 0x1fdded0f, 90 },
    { 0x3bdded0e, 98 },
    { 0xfe0ded12, 46 },
    { 0x6ffded0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ded19, 76 },
    { 0x909ded1a, 67 },
    { 0xe27ded1a, 99 },
    { 0xdc9ded19, 77 },
    { 0xbbfded1d, 46 },
    { 0x1e8ded1d, 100 },
    { 0xd2bded1c, 34 },
    { 0x705ded1a, 95 },
    { 0xd2ded1d, 93 },
    { 0x423ded1d, 43 },
    { 0x310ded1b, 81 },
    { 0, 0 },
    { 0xcd1ded25, 43 },
    { 0x836ded26, 33 },
    { 0, 0 },
    { 0x633ded28, 76 },
    { 0xcb1ded29, 98 },
    { 0xf50ded2a, 95 },
    { 0x714ded2b, 90 },
    { 0x33fded2b, 55 },
    { 0xc83ded2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aded36, 98 },
    { 0x33fded37, 90 },
    { 0xa66ded38, 46 },
    { 0x564ded39, 100 },
    { 0xe7fded38, 95 },
    { 0x41dded3b, 76 },
    { 0x499ded3c, 95 },
    { 0x226ded3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ded44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cded49, 76 },
    { 0x758ded4a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bded4e, 33 },
    { 0, 0 },
    { 0x8edded50, 33 },
    { 0xa57ded51, 52 },
    { 0, 0 },
    { 0x63dded53, 95 },
    { 0xe30ded53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ded57, 61 },
    { 0, 0 },
    { 0x75fded59, 95 },
    { 0xef5ded5a, 61 },
    { 0xdaaded5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3ded5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eded62, 23 },
    { 0xa43ded63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73ded66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cded6c, 95 },
    { 0xa05ded6c, 98 },
    { 0xbdfded6e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9ded72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ded79, 33 },
    { 0xda5ded79, 99 },
    { 0xe9fded79, 90 },
    { 0, 0 },
    { 0x908ded7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x786ded80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ded83, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361ded87, 77 },
    { 0, 0 },
    { 0x53dded89, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86ded8d, 23 },
    { 0x9e9ded8e, 60 },
    { 0xa9dded8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cded97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992ded9f, 66 },
    { 0x396deda0, 61 },
    { 0xfb8deda1, 33 },
    { 0xd3ededa2, 33 },
    { 0x12adeda1, 23 },
    { 0xa27deda1, 98 },
    { 0x994deda2, 61 },
    { 0x3b1deda6, 61 },
    { 0, 0 },
    { 0x9afdeda8, 87 },
    { 0, 0 },
    { 0x231dedaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84dedad, 91 },
    { 0x30bdedae, 55 },
    { 0, 0 },
    { 0x420dedb0, 90 },
    { 0x4d6dedb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944dedb5, 61 },
    { 0xcf0dedb6, 61 },
    { 0, 0 },
    { 0x49ddedb8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6dedbd, 61 },
    { 0x8d0dedbe, 90 },
    { 0, 0 },
    { 0x2c0dedc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3dedc4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884dedcb, 23 },
    { 0xe53dedcc, 90 },
    { 0xc25dedcc, 33 },
    { 0x99ededcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83dedd1, 55 },
    { 0x706dedd2, 95 },
    { 0, 0 },
    { 0xa1dedd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ededd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bededdb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38adeddf, 76 },
    { 0, 0 },
    { 0xf7cdede1, 61 },
    { 0xf2ddede2, 33 },
    { 0, 0 },
    { 0xb3bdede4, 33 },
    { 0x843dede5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727dede9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97dedef, 33 },
    { 0x17cdedef, 33 },
    { 0, 0 },
    { 0xf68dedf2, 84 },
    { 0xe1dedf3, 55 },
    { 0x823dedf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdcdedf8, 98 },
    { 0x6addedf9, 33 },
    { 0xb8fdedfa, 90 },
    { 0xc30dedfb, 46 },
    { 0x969dedfb, 56 },
    { 0xf0bdedfa, 98 },
    { 0xdebdedfe, 75 },
    { 0, 0 },
    { 0xcf0dee00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2dee04, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cedee0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0dee16, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2dee19, 61 },
    { 0xdafdee19, 98 },
    { 0x6eddee1b, 33 },
    { 0x3bdee19, 99 },
    { 0, 0 },
    { 0xc6ddee1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782dee22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24dee25, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5dee2a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3dee2f, 65 },
    { 0xf08dee30, 33 },
    { 0, 0 },
    { 0xd7adee32, 59 },
    { 0xddbdee33, 23 },
    { 0xb56dee33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bdee37, 23 },
    { 0, 0 },
    { 0x39ddee39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cdee3e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dee41, 91 },
    { 0xadedee42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245dee47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32dee4e, 45 },
    { 0, 0 },
    { 0x71ddee50, 34 },
    { 0x51ddee51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664dee55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09dee5c, 99 },
    { 0x3e5dee5d, 90 },
    { 0, 0 },
    { 0xd61dee5f, 95 },
    { 0xba1dee60, 91 },
    { 0xe2bdee61, 90 },
    { 0x315dee5f, 76 },
    { 0, 0 },
    { 0x832dee64, 23 },
    { 0xc6fdee65, 90 },
    { 0x80bdee66, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adee6a, 100 },
    { 0, 0 },
    { 0xefedee6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaadee74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8dee77, 43 },
    { 0x8a8dee77, 34 },
    { 0, 0 },
    { 0xbcddee7a, 61 },
    { 0, 0 },
    { 0x1bddee7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630dee87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608dee8c, 68 },
    { 0, 0 },
    { 0xdddee8e, 95 },
    { 0, 0 },
    { 0x80edee90, 23 },
    { 0x21edee91, 17 },
    { 0x445dee92, 43 },
    { 0, 0 },
    { 0xe19dee94, 76 },
    { 0, 0 },
    { 0x5a0dee96, 61 },
    { 0xefadee97, 78 },
    { 0xeddee96, 61 },
    { 0xcbbdee99, 61 },
    { 0, 0 },
    { 0x47adee9b, 98 },
    { 0x30adee9b, 46 },
    { 0x4aedee9c, 34 },
    { 0, 0 },
    { 0x514dee9f, 34 },
    { 0x7bfdeea0, 33 },
    { 0, 0 },
    { 0x4c4deea2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bdeea5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449deeac, 67 },
    { 0, 0 },
    { 0xa70deeae, 98 },
    { 0x470deeae, 34 },
    { 0xad4deeae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86deeb5, 93 },
    { 0xb5adeeb6, 34 },
    { 0xfa2deeb7, 91 },
    { 0, 0 },
    { 0x245deeb9, 95 },
    { 0, 0 },
    { 0x5bddeebb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66deebe, 61 },
    { 0, 0 },
    { 0x221deec0, 67 },
    { 0, 0 },
    { 0x3d5deec2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ddeec7, 75 },
    { 0x395deec8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fdeecb, 33 },
    { 0xe21deecb, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16deed0, 90 },
    { 0xba9deed1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5deed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833deeda, 61 },
    { 0xf39deedb, 43 },
    { 0, 0 },
    { 0x98fdeedd, 36 },
    { 0x69ddeede, 23 },
    { 0x1bddeede, 55 },
    { 0xcebdeede, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddddeee6, 52 },
    { 0, 0 },
    { 0xfa1deee8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7deeeb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2deef2, 33 },
    { 0, 0 },
    { 0xd7bdeef4, 76 },
    { 0, 0 },
    { 0x8d9deef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ddeefa, 91 },
    { 0, 0 },
    { 0x19adeefc, 23 },
    { 0x470deefc, 95 },
    { 0x1f1deefe, 17 },
    { 0xe2bdeeff, 100 },
    { 0xeb5def00, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697def07, 23 },
    { 0x323def08, 88 },
    { 0xd8fdef09, 33 },
    { 0, 0 },
    { 0x5afdef0b, 55 },
    { 0xffbdef0b, 95 },
    { 0x45fdef0d, 98 },
    { 0x759def0e, 81 },
    { 0x3fbdef0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11def1b, 33 },
    { 0x523def1c, 77 },
    { 0, 0 },
    { 0x2b8def1e, 77 },
    { 0x41adef1f, 87 },
    { 0x845def20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88adef24, 65 },
    { 0x5b7def25, 33 },
    { 0, 0 },
    { 0x5aadef27, 43 },
    { 0xbcfdef28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467def2d, 88 },
    { 0xb81def2e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3def31, 76 },
    { 0x3d7def32, 90 },
    { 0x3bdef33, 95 },
    { 0xde9def34, 98 },
    { 0x6e8def31, 33 },
    { 0, 0 },
    { 0xd1bdef37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbdef43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71adef49, 69 },
    { 0, 0 },
    { 0xd76def4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6def4f, 61 },
    { 0, 0 },
    { 0x73edef51, 23 },
    { 0xe38def51, 98 },
    { 0x437def53, 76 },
    { 0x6b4def52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fdef57, 90 },
    { 0, 0 },
    { 0x485def59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcdef5e, 60 },
    { 0, 0 },
    { 0x9a7def60, 65 },
    { 0x72adef61, 33 },
    { 0xb84def62, 100 },
    { 0x588def63, 55 },
    { 0, 0 },
    { 0xfa1def65, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eadef69, 40 },
    { 0x3c3def6a, 23 },
    { 0xc80def6b, 90 },
    { 0x71cdef6b, 80 },
    { 0x50ddef6a, 75 },
    { 0xda7def6c, 90 },
    { 0x77fdef6f, 55 },
    { 0x4f7def6f, 43 },
    { 0x49ddef71, 43 },
    { 0x8b7def72, 76 },
    { 0xdc4def73, 61 },
    { 0xd55def73, 98 },
    { 0x5bddef75, 68 },
    { 0xa01def76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x962def79, 33 },
    { 0x1bcdef79, 90 },
    { 0x2e7def7b, 95 },
    { 0, 0 },
    { 0xf35def7d, 23 },
    { 0, 0 },
    { 0xe36def7f, 38 },
    { 0, 0 },
    { 0x7c9def81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x622def84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddef8a, 90 },
    { 0x5b9def8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26adef92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb8def96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdafdef9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x92bdefa1, 98 },
    { 0x73edefa2, 95 },
    { 0, 0 },
    { 0x303defa4, 33 },
    { 0x930defa5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefdefa9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519defad, 61 },
    { 0, 0 },
    { 0x98edefaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35defb2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x245defb5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x10defb8, 66 },
    { 0, 0 },
    { 0x61defba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x945defc1, 33 },
    { 0x433defc2, 52 },
    { 0xc8cdefc2, 91 },
    { 0x291defc3, 66 },
    { 0x513defc2, 55 },
    { 0x835defc6, 66 },
    { 0xd00defc6, 23 },
    { 0x637defc2, 76 },
    { 0xf4edefc9, 34 },
    { 0x80cdefca, 84 },
    { 0xb38defcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x930defd3, 77 },
    { 0x698defd4, 46 },
    { 0, 0 },
    { 0x6dadefd6, 33 },
    { 0xc7edefd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x653defda, 96 },
    { 0x8d5defdb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cadefe0, 100 },
    { 0, 0 },
    { 0x54defe2, 98 },
    { 0x9c1defe3, 33 },
    { 0xd00defe3, 40 },
    { 0, 0 },
    { 0x4badefe6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcdefe9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365deff4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x529deff7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62deffe, 76 },
    { 0xa35defff, 95 },
    { 0, 0 },
    { 0x79df001, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831df007, 97 },
    { 0xf61df008, 90 },
    { 0x3d0df009, 61 },
    { 0xe56df00a, 90 },
    { 0, 0 },
    { 0x7e2df00c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896df023, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccdf02a, 32 },
    { 0xe99df02b, 100 },
    { 0, 0 },
    { 0x3dcdf02d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258df032, 98 },
    { 0x20ddf033, 100 },
    { 0xe4edf034, 66 },
    { 0, 0 },
    { 0x7c8df036, 33 },
    { 0x82ddf037, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02df03c, 95 },
    { 0x83edf03c, 19 },
    { 0xe07df03e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55edf041, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6df045, 23 },
    { 0x9edf046, 61 },
    { 0x48ddf046, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6df051, 55 },
    { 0xc8edf052, 17 },
    { 0xe8cdf053, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cdf058, 61 },
    { 0x1bdf058, 95 },
    { 0, 0 },
    { 0xd11df05b, 91 },
    { 0xba9df05b, 69 },
    { 0x14bdf05d, 68 },
    { 0xeabdf05c, 78 },
    { 0xd99df05e, 47 },
    { 0x5b8df060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98edf067, 61 },
    { 0x2d3df068, 76 },
    { 0x1aadf069, 23 },
    { 0x80cdf067, 33 },
    { 0xf0cdf069, 61 },
    { 0x1c1df06a, 77 },
    { 0, 0 },
    { 0x1c5df06e, 52 },
    { 0, 0 },
    { 0xdebdf070, 95 },
    { 0, 0 },
    { 0xebddf072, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x852df075, 98 },
    { 0xeadf076, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2df07a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58df07e, 76 },
    { 0x1aadf07f, 95 },
    { 0x8b5df07f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840df084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8df089, 90 },
    { 0x2b7df089, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1df08f, 85 },
    { 0, 0 },
    { 0xf56df091, 23 },
    { 0x2a2df092, 91 },
    { 0xf30df093, 33 },
    { 0x3b9df094, 90 },
    { 0x7d5df095, 90 },
    { 0xaa8df094, 90 },
    { 0, 0 },
    { 0x50df098, 90 },
    { 0, 0 },
    { 0x2b8df09a, 23 },
    { 0, 0 },
    { 0x263df09c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7df09f, 46 },
    { 0x4aadf09f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810df0a6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9df0ab, 61 },
    { 0x7e8df0ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3df0b2, 80 },
    { 0xcfddf0b3, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bdf0b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29df0b9, 23 },
    { 0, 0 },
    { 0x3c6df0bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2df0c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acdf0ca, 23 },
    { 0xcc3df0cb, 100 },
    { 0, 0 },
    { 0x646df0cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9df0d5, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbddf0da, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8df0de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x808df0e1, 98 },
    { 0x2fadf0e2, 17 },
    { 0xae0df0e2, 33 },
    { 0xbaadf0e4, 93 },
    { 0x1fdf0e4, 61 },
    { 0xa4ddf0e6, 78 },
    { 0x1f1df0e6, 93 },
    { 0x84edf0e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abdf0ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x620df0ef, 34 },
    { 0x246df0f0, 43 },
    { 0x478df0f0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cdf0f5, 42 },
    { 0x208df0f5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5df0fe, 77 },
    { 0xf21df0fe, 66 },
    { 0xd16df100, 78 },
    { 0x310df100, 61 },
    { 0x733df102, 55 },
    { 0, 0 },
    { 0x7e6df104, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4df10c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcadf110, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fdf114, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695df11b, 46 },
    { 0, 0 },
    { 0xb81df11d, 76 },
    { 0x2c6df11d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01df122, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x182df125, 77 },
    { 0, 0 },
    { 0x99cdf127, 44 },
    { 0xf31df128, 55 },
    { 0x34fdf129, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45adf12d, 95 },
    { 0x89fdf12e, 23 },
    { 0, 0 },
    { 0xfe2df130, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x384df133, 46 },
    { 0x15ddf133, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530df13c, 61 },
    { 0x933df13d, 61 },
    { 0x1dbdf13e, 98 },
    { 0xe39df13f, 34 },
    { 0x7d3df13d, 34 },
    { 0xa48df141, 34 },
    { 0x5cedf13f, 90 },
    { 0x86cdf143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8bdf149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5df156, 23 },
    { 0xad8df157, 78 },
    { 0xa9fdf157, 98 },
    { 0, 0 },
    { 0xbe9df15a, 93 },
    { 0x565df15b, 76 },
    { 0xbb5df15a, 95 },
    { 0xbfbdf15d, 33 },
    { 0xe8adf15d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cdf161, 23 },
    { 0x3c6df162, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x425df166, 90 },
    { 0, 0 },
    { 0x343df168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fdf16b, 61 },
    { 0, 0 },
    { 0x23fdf16d, 100 },
    { 0, 0 },
    { 0x11fdf16f, 46 },
    { 0, 0 },
    { 0x323df171, 95 },
    { 0, 0 },
    { 0x385df173, 90 },
    { 0, 0 },
    { 0x8f5df175, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801df17e, 33 },
    { 0xc59df17f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832df184, 23 },
    { 0x7cedf185, 33 },
    { 0, 0 },
    { 0xfe0df187, 99 },
    { 0xb90df188, 95 },
    { 0x1d1df189, 90 },
    { 0, 0 },
    { 0x2efdf18b, 91 },
    { 0, 0 },
    { 0xedfdf18d, 23 },
    { 0, 0 },
    { 0xe07df18f, 93 },
    { 0, 0 },
    { 0x4a9df191, 33 },
    { 0x8a1df192, 17 },
    { 0, 0 },
    { 0xa81df194, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18df199, 98 },
    { 0x231df19a, 61 },
    { 0, 0 },
    { 0xcfbdf19c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x787df19f, 90 },
    { 0x2aedf1a0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4df1a4, 81 },
    { 0, 0 },
    { 0x78cdf1a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7df1aa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bdf1ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffdf1b1, 55 },
    { 0x479df1b2, 95 },
    { 0x758df1b1, 90 },
    { 0, 0 },
    { 0xf99df1b5, 61 },
    { 0xc82df1b6, 33 },
    { 0x8dddf1b7, 90 },
    { 0xe43df1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadf1bb, 98 },
    { 0x6badf1bc, 34 },
    { 0xdb6df1bd, 34 },
    { 0, 0 },
    { 0x2e5df1bf, 33 },
    { 0x2cfdf1c0, 66 },
    { 0x41adf1bf, 33 },
    { 0xfeadf1c2, 90 },
    { 0xcfcdf1bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddf1c6, 77 },
    { 0x363df1c7, 23 },
    { 0, 0 },
    { 0x405df1c9, 95 },
    { 0xa7fdf1c9, 93 },
    { 0x8d6df1c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1df1ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6df1d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08df1d6, 33 },
    { 0, 0 },
    { 0xa65df1d8, 77 },
    { 0, 0 },
    { 0xd09df1da, 90 },
    { 0x9adf1db, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffdf1df, 55 },
    { 0x4f0df1df, 23 },
    { 0x7dfdf1e1, 53 },
    { 0xccfdf1df, 61 },
    { 0x665df1e3, 95 },
    { 0xf8bdf1e4, 61 },
    { 0xc0fdf1e5, 90 },
    { 0xcc6df1e6, 55 },
    { 0x697df1e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5df1ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4df1f5, 87 },
    { 0x786df1f6, 99 },
    { 0x21edf1f6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4df1fd, 98 },
    { 0, 0 },
    { 0x7b0df1ff, 100 },
    { 0, 0 },
    { 0x25adf201, 90 },
    { 0x8f6df202, 90 },
    { 0xf84df202, 76 },
    { 0x41adf204, 43 },
    { 0x1c3df205, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329df20a, 91 },
    { 0xfc4df20b, 98 },
    { 0, 0 },
    { 0xaafdf20d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2df210, 43 },
    { 0xc4adf211, 56 },
    { 0x3a5df211, 61 },
    { 0, 0 },
    { 0x2ebdf214, 95 },
    { 0, 0 },
    { 0x20edf216, 76 },
    { 0xaeddf217, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x571df21a, 34 },
    { 0x465df21b, 95 },
    { 0x415df21c, 61 },
    { 0x24cdf21d, 95 },
    { 0x974df21e, 55 },
    { 0, 0 },
    { 0x62ddf220, 33 },
    { 0x79ddf221, 87 },
    { 0x604df222, 66 },
    { 0x395df223, 23 },
    { 0, 0 },
    { 0x118df225, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7df22a, 80 },
    { 0x8aedf22a, 23 },
    { 0x6e7df22c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afdf231, 33 },
    { 0xfa8df232, 100 },
    { 0xe69df233, 61 },
    { 0x873df231, 23 },
    { 0xca8df234, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34df239, 33 },
    { 0x98adf23a, 85 },
    { 0xa4adf239, 95 },
    { 0x803df23a, 95 },
    { 0, 0 },
    { 0xf86df23e, 93 },
    { 0xb30df23f, 95 },
    { 0x559df23e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccdf243, 43 },
    { 0x290df244, 66 },
    { 0x453df243, 85 },
    { 0x160df244, 95 },
    { 0, 0 },
    { 0x151df248, 75 },
    { 0xac1df249, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32df24d, 33 },
    { 0xe04df24d, 95 },
    { 0x681df24f, 33 },
    { 0x2dddf24e, 33 },
    { 0xd6df251, 55 },
    { 0xb65df252, 19 },
    { 0xaa8df253, 84 },
    { 0x76adf24e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45df25a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x573df25d, 90 },
    { 0x51cdf25d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0df261, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13df264, 17 },
    { 0xeffdf265, 43 },
    { 0x6edf264, 33 },
    { 0, 0 },
    { 0xcb1df268, 40 },
    { 0x286df268, 99 },
    { 0x550df26a, 90 },
    { 0x6b1df26b, 33 },
    { 0xa34df26b, 61 },
    { 0, 0 },
    { 0x43ddf26e, 90 },
    { 0xbc5df26f, 98 },
    { 0xb5df270, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1df277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf27e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fdf282, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bcdf288, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbdf28b, 61 },
    { 0, 0 },
    { 0x46bdf28d, 83 },
    { 0, 0 },
    { 0x20cdf28f, 99 },
    { 0, 0 },
    { 0x2e7df291, 76 },
    { 0x583df292, 52 },
    { 0xbbddf293, 95 },
    { 0, 0 },
    { 0x90cdf295, 63 },
    { 0, 0 },
    { 0x3f4df297, 98 },
    { 0x782df298, 81 },
    { 0x76bdf298, 33 },
    { 0x267df29a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4df29e, 66 },
    { 0x2dedf29e, 78 },
    { 0xef9df29f, 90 },
    { 0, 0 },
    { 0xe05df2a2, 61 },
    { 0x21adf2a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3df2a9, 55 },
    { 0x3a6df2aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x95df2ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1edf2b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cdf2b7, 76 },
    { 0x99edf2b8, 55 },
    { 0x6f0df2b9, 23 },
    { 0, 0 },
    { 0xd6bdf2bb, 23 },
    { 0, 0 },
    { 0x870df2bd, 61 },
    { 0x358df2be, 34 },
    { 0, 0 },
    { 0x14edf2c0, 33 },
    { 0x800df2c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4df2c9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33cdf2d1, 98 },
    { 0x5ecdf2d1, 23 },
    { 0x83df2d2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfadf2d6, 33 },
    { 0x952df2d7, 84 },
    { 0x7abdf2d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfadf2db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391df2df, 90 },
    { 0, 0 },
    { 0xb0df2e1, 46 },
    { 0x6d9df2e1, 90 },
    { 0x6f8df2e3, 91 },
    { 0xad2df2e4, 43 },
    { 0xf5bdf2e5, 91 },
    { 0x753df2e6, 91 },
    { 0, 0 },
    { 0x8f7df2e8, 98 },
    { 0xc66df2e9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x708df2ec, 61 },
    { 0, 0 },
    { 0xd33df2ee, 95 },
    { 0, 0 },
    { 0x803df2f0, 95 },
    { 0xe9fdf2f1, 90 },
    { 0, 0 },
    { 0x74ddf2f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1df2f9, 33 },
    { 0, 0 },
    { 0x621df2fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8edf301, 61 },
    { 0x89ddf301, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eddf305, 23 },
    { 0xc7adf306, 76 },
    { 0, 0 },
    { 0xce3df308, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0df30b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edf30f, 76 },
    { 0x7aedf310, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bdf313, 34 },
    { 0x4d0df314, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1df317, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ddf31a, 90 },
    { 0x863df31a, 90 },
    { 0xffcdf31b, 33 },
    { 0x520df31d, 90 },
    { 0x731df31a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cdf327, 98 },
    { 0, 0 },
    { 0xf87df329, 87 },
    { 0xe0bdf32a, 61 },
    { 0xd29df32b, 33 },
    { 0x897df32c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fdf32f, 77 },
    { 0x15df330, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53df333, 76 },
    { 0, 0 },
    { 0xa31df335, 33 },
    { 0xa8df336, 33 },
    { 0, 0 },
    { 0xe62df338, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36df33b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0df340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42df343, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cdf346, 55 },
    { 0x5e0df346, 61 },
    { 0x6f8df348, 33 },
    { 0x129df347, 95 },
    { 0, 0 },
    { 0x387df34b, 95 },
    { 0, 0 },
    { 0x4d8df34d, 46 },
    { 0, 0 },
    { 0xf40df34f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ddf358, 52 },
    { 0, 0 },
    { 0x133df35a, 90 },
    { 0x709df35a, 33 },
    { 0xe0ddf35b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343df360, 90 },
    { 0x9fddf361, 33 },
    { 0, 0 },
    { 0x21ddf363, 55 },
    { 0x8a1df364, 98 },
    { 0, 0 },
    { 0xdb0df366, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332df36a, 87 },
    { 0, 0 },
    { 0x6e4df36c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dadf370, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240df374, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19df377, 75 },
    { 0xaedf378, 98 },
    { 0, 0 },
    { 0x8b1df37a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53df37d, 95 },
    { 0, 0 },
    { 0xf28df37f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf382, 81 },
    { 0xce1df382, 90 },
    { 0x3bddf384, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1df389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1df38d, 77 },
    { 0, 0 },
    { 0xb3cdf38f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943df393, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbedf39a, 33 },
    { 0x610df39a, 84 },
    { 0x9e3df39c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8df3a1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10df3a9, 61 },
    { 0, 0 },
    { 0x712df3ab, 90 },
    { 0x811df3ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bdf3b1, 65 },
    { 0xcb1df3b2, 33 },
    { 0xe26df3b2, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6df3b6, 97 },
    { 0, 0 },
    { 0x8a0df3b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffdf3bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bdf3c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08df3c6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2df3cb, 90 },
    { 0x345df3cb, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208df3d1, 61 },
    { 0xc52df3d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806df3d6, 88 },
    { 0xa8edf3d7, 95 },
    { 0xe14df3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51df3dd, 90 },
    { 0, 0 },
    { 0x26df3df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35df3e3, 46 },
    { 0, 0 },
    { 0xd9df3e5, 93 },
    { 0xa13df3e5, 33 },
    { 0x929df3e7, 55 },
    { 0xef3df3e6, 90 },
    { 0x895df3e9, 69 },
    { 0x6a4df3ea, 95 },
    { 0, 0 },
    { 0xc44df3ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1adf3ef, 33 },
    { 0, 0 },
    { 0xdc9df3f1, 33 },
    { 0x5dadf3f2, 50 },
    { 0x97fdf3f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49df3f6, 95 },
    { 0xcfadf3f7, 55 },
    { 0x567df3f8, 77 },
    { 0x993df3f8, 61 },
    { 0x64bdf3fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fdf405, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88df40d, 99 },
    { 0, 0 },
    { 0x2b3df40f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56df412, 84 },
    { 0, 0 },
    { 0xf33df414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33df418, 99 },
    { 0xe18df418, 76 },
    { 0xdffdf419, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fdf425, 93 },
    { 0, 0 },
    { 0xf82df427, 23 },
    { 0x48df428, 26 },
    { 0x68adf428, 61 },
    { 0x2cddf428, 90 },
    { 0, 0 },
    { 0xc23df42c, 98 },
    { 0, 0 },
    { 0x85ddf42e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7edf432, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x58edf435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ddf43c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63df440, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdedf443, 65 },
    { 0x5d4df443, 55 },
    { 0x622df445, 91 },
    { 0x7b1df446, 23 },
    { 0x5aadf447, 87 },
    { 0x9a3df443, 90 },
    { 0xa2cdf447, 98 },
    { 0x7e8df44a, 40 },
    { 0x9ccdf44a, 66 },
    { 0x96cdf44c, 33 },
    { 0xc3cdf44c, 76 },
    { 0x992df44d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0df451, 84 },
    { 0xdfdf451, 98 },
    { 0x544df452, 98 },
    { 0x2e3df451, 90 },
    { 0xe22df453, 55 },
    { 0x5ecdf454, 66 },
    { 0x8e2df452, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2df460, 98 },
    { 0, 0 },
    { 0x8a5df462, 99 },
    { 0, 0 },
    { 0x649df464, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdedf468, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fdf46d, 98 },
    { 0x60df46e, 98 },
    { 0x8b4df46e, 61 },
    { 0xdecdf46f, 90 },
    { 0x1c8df46e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdf474, 95 },
    { 0xa6edf474, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ddf478, 17 },
    { 0, 0 },
    { 0x1c1df47a, 90 },
    { 0xd47df47b, 34 },
    { 0x166df47c, 78 },
    { 0x94edf47c, 61 },
    { 0x383df47d, 23 },
    { 0, 0 },
    { 0x45ddf480, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcdf488, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28df48e, 68 },
    { 0x9ffdf48e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4df496, 95 },
    { 0x78ddf497, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fedf49b, 93 },
    { 0, 0 },
    { 0xfa5df49d, 76 },
    { 0, 0 },
    { 0x537df49f, 88 },
    { 0x288df4a0, 55 },
    { 0x6f5df4a0, 100 },
    { 0x4ccdf4a1, 61 },
    { 0x30edf4a0, 95 },
    { 0xa7ddf4a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6df4a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7df4b0, 91 },
    { 0x4f8df4b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4addf4b5, 61 },
    { 0xe05df4b5, 52 },
    { 0xa8fdf4b7, 69 },
    { 0x5a2df4b6, 93 },
    { 0, 0 },
    { 0xc72df4ba, 23 },
    { 0x24df4bb, 65 },
    { 0xfd1df4bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bdf4bf, 90 },
    { 0x581df4bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4df4c9, 95 },
    { 0xcacdf4ca, 90 },
    { 0xd76df4cb, 91 },
    { 0, 0 },
    { 0xfdbdf4cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68df4d0, 49 },
    { 0x2e6df4d0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bdf4d4, 76 },
    { 0x337df4d4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578df4d9, 33 },
    { 0x5ecdf4da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166df4e0, 78 },
    { 0x9cfdf4e0, 34 },
    { 0xc60df4e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x456df4e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x994df4e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcdf4eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35edf4f2, 90 },
    { 0x3d2df4f2, 68 },
    { 0, 0 },
    { 0xf05df4f5, 100 },
    { 0, 0 },
    { 0xd6adf4f7, 95 },
    { 0, 0 },
    { 0xc3adf4f9, 88 },
    { 0xf9bdf4fa, 65 },
    { 0x3b6df4fa, 34 },
    { 0, 0 },
    { 0x1cdf4fd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cadf500, 55 },
    { 0, 0 },
    { 0x3fddf502, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27edf507, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fdf50c, 78 },
    { 0x51bdf50d, 91 },
    { 0x734df50e, 99 },
    { 0x122df50e, 61 },
    { 0xebdf50d, 36 },
    { 0x586df511, 90 },
    { 0xae3df50c, 61 },
    { 0x914df50c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26df516, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7df519, 91 },
    { 0x7d4df519, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x196df51d, 99 },
    { 0, 0 },
    { 0xb8fdf51f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6df528, 52 },
    { 0xd6fdf529, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1df52d, 90 },
    { 0xf25df52d, 76 },
    { 0xeb6df52f, 76 },
    { 0, 0 },
    { 0xcf4df531, 17 },
    { 0x764df531, 81 },
    { 0, 0 },
    { 0x619df534, 33 },
    { 0x213df535, 98 },
    { 0xa55df536, 33 },
    { 0, 0 },
    { 0xed9df538, 52 },
    { 0, 0 },
    { 0xdfbdf53a, 78 },
    { 0, 0 },
    { 0x652df53c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154df543, 84 },
    { 0x780df544, 91 },
    { 0xcedf545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2df54e, 84 },
    { 0, 0 },
    { 0x829df550, 98 },
    { 0x426df551, 61 },
    { 0x7f1df550, 33 },
    { 0x2c7df553, 98 },
    { 0xc9adf554, 98 },
    { 0xc19df554, 33 },
    { 0x79cdf556, 100 },
    { 0x10ddf553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfdf55c, 76 },
    { 0x6f4df55d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bdf564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42df569, 33 },
    { 0x82adf56a, 93 },
    { 0, 0 },
    { 0x79bdf56c, 33 },
    { 0x8dedf56d, 75 },
    { 0xf02df56c, 95 },
    { 0x27ddf56f, 33 },
    { 0x6c2df56f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bdf574, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345df578, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7adf57c, 90 },
    { 0xe97df57d, 33 },
    { 0x3f4df57d, 61 },
    { 0x281df57f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fdf583, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90df587, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bdf58a, 90 },
    { 0, 0 },
    { 0x9bedf58c, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5edf58f, 76 },
    { 0x2c1df58f, 46 },
    { 0xe24df591, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x59adf594, 55 },
    { 0x8bcdf595, 76 },
    { 0x99df595, 95 },
    { 0xddedf597, 99 },
    { 0xee5df598, 76 },
    { 0x7d3df599, 90 },
    { 0xd74df59a, 61 },
    { 0x101df594, 61 },
    { 0x14edf59c, 33 },
    { 0x72ddf59d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x282df5a0, 61 },
    { 0xbabdf5a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x365df5a4, 61 },
    { 0x841df5a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbdf5aa, 90 },
    { 0x1dfdf5aa, 33 },
    { 0x3f2df5ac, 95 },
    { 0x5bedf5aa, 23 },
    { 0, 0 },
    { 0xc58df5af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985df5b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8df5b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45df5ba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fdf5c4, 99 },
    { 0x24df5c5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628df5ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabdf5d4, 82 },
    { 0x6d4df5d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eadf5d9, 88 },
    { 0x2cfdf5da, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bdf5de, 55 },
    { 0, 0 },
    { 0x878df5e0, 90 },
    { 0x76bdf5e1, 46 },
    { 0x7dadf5e1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60edf5e8, 43 },
    { 0, 0 },
    { 0x781df5ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffdf5ed, 53 },
    { 0x30df5ee, 34 },
    { 0x9eadf5ef, 33 },
    { 0x5e9df5f0, 49 },
    { 0x791df5f0, 98 },
    { 0x1fadf5f2, 90 },
    { 0xfe9df5ef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cdf5f6, 33 },
    { 0x821df5f6, 33 },
    { 0xbe4df5f8, 34 },
    { 0x65fdf5f9, 61 },
    { 0, 0 },
    { 0xac3df5fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x625df5fe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x813df601, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcdf606, 46 },
    { 0, 0 },
    { 0x4c2df608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bdf60d, 68 },
    { 0x149df60e, 90 },
    { 0x4c7df60f, 36 },
    { 0xb0adf60f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230df615, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334df619, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cdf61e, 66 },
    { 0xea5df61e, 34 },
    { 0xe74df61e, 61 },
    { 0x8f7df621, 33 },
    { 0xe67df622, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebdf627, 95 },
    { 0, 0 },
    { 0xeafdf629, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffdf62c, 33 },
    { 0x755df62c, 55 },
    { 0x858df62e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fdf631, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4df638, 43 },
    { 0x6f2df639, 33 },
    { 0, 0 },
    { 0x1ccdf63b, 90 },
    { 0x387df63b, 76 },
    { 0x715df63b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748df641, 33 },
    { 0x222df642, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818df648, 23 },
    { 0x5df649, 78 },
    { 0, 0 },
    { 0x727df64b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x825df64e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15df652, 33 },
    { 0x90adf652, 90 },
    { 0xc05df654, 90 },
    { 0, 0 },
    { 0xec9df656, 76 },
    { 0xbe0df657, 90 },
    { 0xcfcdf656, 46 },
    { 0, 0 },
    { 0x9bcdf65a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x658df65d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1df663, 46 },
    { 0x7b8df664, 90 },
    { 0x328df665, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5df66a, 61 },
    { 0xeb3df66b, 69 },
    { 0xa3ddf66c, 76 },
    { 0xf87df66d, 61 },
    { 0x11df66e, 55 },
    { 0x8fadf66f, 23 },
    { 0xb2df66c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bddf675, 55 },
    { 0x63edf676, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0adf67a, 17 },
    { 0x5d4df67b, 99 },
    { 0xb5ddf67b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0df680, 49 },
    { 0xd5edf681, 61 },
    { 0xbb7df682, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83df686, 81 },
    { 0xe7fdf686, 59 },
    { 0, 0 },
    { 0xbd4df689, 61 },
    { 0x1fcdf68a, 78 },
    { 0x2e9df68a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8df691, 90 },
    { 0x5d5df692, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418df698, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36df69d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fdf6a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15df6ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x192df6b0, 75 },
    { 0x543df6b1, 23 },
    { 0x88ddf6b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1df6b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00df6bb, 32 },
    { 0, 0 },
    { 0xb37df6bd, 55 },
    { 0xd5df6be, 77 },
    { 0xe05df6bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1df6c3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ddf6c8, 33 },
    { 0xec7df6c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afdf6cd, 99 },
    { 0, 0 },
    { 0xa2edf6cf, 88 },
    { 0, 0 },
    { 0x718df6d1, 33 },
    { 0xa03df6d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcdf6d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x844df6dc, 80 },
    { 0, 0 },
    { 0x6f2df6de, 61 },
    { 0xfb1df6df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2df6e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78adf6ed, 90 },
    { 0xf8adf6ee, 88 },
    { 0, 0 },
    { 0xb19df6f0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3df6f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fdf6f6, 61 },
    { 0x9fddf6f7, 95 },
    { 0, 0 },
    { 0x391df6f9, 100 },
    { 0xb72df6f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x132df6fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cadf702, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x818df705, 61 },
    { 0, 0 },
    { 0xa76df707, 87 },
    { 0x7e5df707, 90 },
    { 0xb4bdf709, 61 },
    { 0xfabdf70a, 90 },
    { 0x6c6df70a, 67 },
    { 0x855df70c, 23 },
    { 0xf06df708, 95 },
    { 0xd4fdf70b, 66 },
    { 0x2dddf709, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9df719, 93 },
    { 0, 0 },
    { 0xc1bdf71b, 33 },
    { 0xc87df71c, 95 },
    { 0x73edf71c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19df723, 23 },
    { 0x1eadf723, 76 },
    { 0xc22df725, 98 },
    { 0x1cadf725, 33 },
    { 0xafedf725, 61 },
    { 0, 0 },
    { 0x205df729, 80 },
    { 0x34df729, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2df730, 23 },
    { 0, 0 },
    { 0x831df732, 46 },
    { 0x228df733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fdf737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cedf73e, 91 },
    { 0x19df73e, 95 },
    { 0x17bdf740, 33 },
    { 0x138df73e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3df746, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bdf74c, 61 },
    { 0, 0 },
    { 0xf4ddf74e, 90 },
    { 0xb0fdf74e, 90 },
    { 0, 0 },
    { 0x48edf751, 76 },
    { 0x96ddf752, 78 },
    { 0x575df751, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5df756, 91 },
    { 0xebbdf757, 90 },
    { 0x70df757, 82 },
    { 0x84ddf759, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbdf75f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6df764, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cadf767, 98 },
    { 0x608df767, 34 },
    { 0x610df769, 99 },
    { 0xe8df768, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fdf774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cdf778, 55 },
    { 0xd94df779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3df77e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2df781, 84 },
    { 0x876df782, 95 },
    { 0xa3df782, 23 },
    { 0xd62df781, 38 },
    { 0x886df781, 33 },
    { 0, 0 },
    { 0x392df787, 46 },
    { 0xc33df788, 34 },
    { 0xf5edf789, 90 },
    { 0x3f5df789, 33 },
    { 0x333df787, 91 },
    { 0, 0 },
    { 0x1e8df78d, 55 },
    { 0x4addf78e, 93 },
    { 0xa3bdf78f, 76 },
    { 0x571df790, 95 },
    { 0xd9bdf791, 98 },
    { 0xf0bdf78e, 90 },
    { 0x1f0df78f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7df797, 61 },
    { 0xeecdf797, 95 },
    { 0x3f5df798, 93 },
    { 0xb5fdf79a, 90 },
    { 0, 0 },
    { 0xe4bdf79c, 93 },
    { 0, 0 },
    { 0xeffdf79e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fdf7a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8df7ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71df7b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4df7b6, 90 },
    { 0, 0 },
    { 0x160df7b8, 81 },
    { 0x757df7b9, 33 },
    { 0, 0 },
    { 0xe0bdf7bb, 46 },
    { 0x9dfdf7bc, 61 },
    { 0xfe4df7bd, 95 },
    { 0xc58df7be, 33 },
    { 0x28adf7bf, 23 },
    { 0x942df7c0, 55 },
    { 0x42adf7bb, 61 },
    { 0x572df7c2, 78 },
    { 0x24bdf7c1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732df7c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fdf7ca, 90 },
    { 0x932df7cb, 80 },
    { 0xbbfdf7cc, 100 },
    { 0xecadf7cd, 33 },
    { 0x1bdf7ce, 61 },
    { 0x2dadf7cf, 64 },
    { 0x9f3df7d0, 93 },
    { 0x2c0df7d1, 67 },
    { 0xeb0df7d2, 98 },
    { 0x838df7d0, 61 },
    { 0xf26df7d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58df7d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fddf7da, 91 },
    { 0xbb0df7db, 100 },
    { 0xf17df7dc, 55 },
    { 0, 0 },
    { 0x8fadf7de, 23 },
    { 0x398df7de, 46 },
    { 0, 0 },
    { 0xfaedf7e1, 66 },
    { 0xb1edf7e2, 33 },
    { 0x542df7e3, 43 },
    { 0x66ddf7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4df7e8, 61 },
    { 0xadcdf7e9, 76 },
    { 0, 0 },
    { 0x88edf7eb, 75 },
    { 0xd24df7ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576df7f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0df7f4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0edf7f7, 46 },
    { 0, 0 },
    { 0xa00df7f9, 52 },
    { 0xa6edf7f9, 90 },
    { 0, 0 },
    { 0xa09df7fc, 61 },
    { 0x114df7fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687df801, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x855df804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3df80b, 95 },
    { 0xc96df80c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x726df80f, 95 },
    { 0x58cdf810, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849df815, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cddf818, 67 },
    { 0x38df819, 61 },
    { 0x61df81a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3edf820, 84 },
    { 0xf24df821, 88 },
    { 0xff7df821, 33 },
    { 0x1dedf820, 33 },
    { 0xc7df824, 90 },
    { 0x4d2df821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2df828, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67adf82c, 33 },
    { 0x4b8df82d, 43 },
    { 0, 0 },
    { 0x9ecdf82f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645df835, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52df838, 43 },
    { 0, 0 },
    { 0xbcedf83a, 55 },
    { 0x750df83a, 23 },
    { 0x2cedf83c, 43 },
    { 0xf0fdf83b, 95 },
    { 0xed0df83e, 46 },
    { 0xb82df83f, 46 },
    { 0xe1edf83d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2df843, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3df848, 91 },
    { 0x5dcdf849, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfdf84e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbdf851, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x183df854, 95 },
    { 0x536df855, 33 },
    { 0x5cedf855, 33 },
    { 0xbfdf857, 77 },
    { 0x9bedf857, 98 },
    { 0x1d6df854, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96df85e, 90 },
    { 0, 0 },
    { 0xbdcdf860, 95 },
    { 0, 0 },
    { 0x662df862, 95 },
    { 0x21bdf862, 95 },
    { 0x746df864, 33 },
    { 0, 0 },
    { 0x8dcdf866, 78 },
    { 0, 0 },
    { 0xd42df868, 33 },
    { 0x4cdf868, 66 },
    { 0x33ddf86a, 76 },
    { 0xd4fdf86a, 33 },
    { 0xc02df86c, 76 },
    { 0x2e5df86a, 23 },
    { 0xde9df868, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8df872, 23 },
    { 0xd99df873, 23 },
    { 0x45edf873, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbdf879, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683df87f, 43 },
    { 0x6b4df880, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adf883, 23 },
    { 0, 0 },
    { 0xbe3df885, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fdf890, 90 },
    { 0x633df891, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0df89d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268df8a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2df8a5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4df8aa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4df8ae, 51 },
    { 0x261df8af, 84 },
    { 0x8ddf8ae, 90 },
    { 0x29cdf8b1, 33 },
    { 0x7cdf8b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102df8b6, 55 },
    { 0, 0 },
    { 0x7d7df8b8, 98 },
    { 0, 0 },
    { 0x507df8ba, 90 },
    { 0x2aadf8bb, 81 },
    { 0xf6adf8bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fdf8c0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778df8c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2badf8ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6df8d4, 90 },
    { 0x372df8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aadf8d8, 33 },
    { 0xaf9df8d8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322df8dd, 83 },
    { 0x841df8de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cdf8e1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5df8e4, 88 },
    { 0x130df8e4, 65 },
    { 0xc3cdf8e4, 99 },
    { 0xb53df8e7, 52 },
    { 0xf31df8e7, 23 },
    { 0x607df8e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92df8f0, 61 },
    { 0xba7df8f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5df8f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51df8f7, 65 },
    { 0x8acdf8f7, 90 },
    { 0x79df8f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1df8fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67df902, 69 },
    { 0, 0 },
    { 0xabddf904, 95 },
    { 0x349df905, 95 },
    { 0, 0 },
    { 0x6dcdf907, 90 },
    { 0xcd1df908, 90 },
    { 0, 0 },
    { 0x6bedf90a, 98 },
    { 0, 0 },
    { 0x900df90c, 98 },
    { 0x824df90d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86df918, 33 },
    { 0, 0 },
    { 0x25df91a, 90 },
    { 0, 0 },
    { 0xf5edf91c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5df92d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fdf935, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686df93e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3df943, 84 },
    { 0x7b8df944, 67 },
    { 0, 0 },
    { 0x8cbdf946, 78 },
    { 0, 0 },
    { 0xb86df948, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8df94b, 61 },
    { 0, 0 },
    { 0xa5edf94d, 100 },
    { 0, 0 },
    { 0x45bdf94f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15df955, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdf95a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bdf95d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df963, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ddf966, 43 },
    { 0xd9adf966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428df96c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fdf96f, 52 },
    { 0x7afdf970, 68 },
    { 0x451df971, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2df975, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ddf97d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730df981, 33 },
    { 0x301df982, 90 },
    { 0xa2edf981, 33 },
    { 0x953df984, 33 },
    { 0x8b1df985, 61 },
    { 0xc63df985, 87 },
    { 0xa81df985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65df98b, 90 },
    { 0x289df98c, 35 },
    { 0xbeedf98d, 97 },
    { 0xe0adf98e, 23 },
    { 0xf7bdf98f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffdf999, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88df99e, 61 },
    { 0, 0 },
    { 0x3a8df9a0, 87 },
    { 0, 0 },
    { 0x52df9a2, 33 },
    { 0x28df9a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x923df9a6, 61 },
    { 0xf70df9a7, 61 },
    { 0x1b9df9a7, 101 },
    { 0x760df9a6, 55 },
    { 0x3d7df9a7, 34 },
    { 0x8a6df9a7, 95 },
    { 0x827df9ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7df9b3, 76 },
    { 0, 0 },
    { 0x15bdf9b5, 98 },
    { 0x455df9b6, 23 },
    { 0xdf0df9b5, 100 },
    { 0xfd1df9b7, 99 },
    { 0, 0 },
    { 0x679df9ba, 90 },
    { 0x554df9ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3df9be, 61 },
    { 0xe33df9be, 23 },
    { 0xbb0df9bf, 76 },
    { 0x2f3df9c1, 88 },
    { 0x4c8df9c2, 98 },
    { 0x96cdf9c3, 95 },
    { 0xffcdf9c4, 43 },
    { 0x55adf9c4, 65 },
    { 0x4eddf9c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x30df9c9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ddf9cf, 90 },
    { 0, 0 },
    { 0x793df9d1, 61 },
    { 0x6acdf9d1, 76 },
    { 0xfa7df9d3, 33 },
    { 0, 0 },
    { 0xbfdf9d5, 77 },
    { 0xc0ddf9d6, 95 },
    { 0xdfedf9d7, 52 },
    { 0x254df9d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6df9db, 61 },
    { 0, 0 },
    { 0x788df9dd, 98 },
    { 0x23fdf9de, 33 },
    { 0xc0df9dd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455df9e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6df9e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88df9f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bdf9f6, 23 },
    { 0, 0 },
    { 0x7a7df9f8, 33 },
    { 0xd8ddf9f8, 61 },
    { 0x295df9f8, 61 },
    { 0, 0 },
    { 0xeefdf9fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdfa00, 98 },
    { 0x256dfa01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9dfa05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5fdfa08, 96 },
    { 0xf88dfa09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2dfa11, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810dfa17, 88 },
    { 0xc03dfa18, 61 },
    { 0x979dfa18, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63dfa1d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x430dfa20, 43 },
    { 0x1d0dfa20, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ddfa24, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199dfa28, 33 },
    { 0, 0 },
    { 0xdc5dfa2a, 90 },
    { 0x953dfa2a, 40 },
    { 0xc36dfa2c, 99 },
    { 0x8fdfa2c, 33 },
    { 0x117dfa2d, 95 },
    { 0x9acdfa2f, 87 },
    { 0x40ddfa2f, 34 },
    { 0xce4dfa2e, 17 },
    { 0x60cdfa2a, 33 },
    { 0, 0 },
    { 0x3fedfa34, 61 },
    { 0x5b5dfa35, 23 },
    { 0xbeadfa36, 95 },
    { 0x62cdfa37, 61 },
    { 0xc62dfa38, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcdfa41, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801dfa4d, 65 },
    { 0x6cddfa4e, 99 },
    { 0x8fedfa4f, 98 },
    { 0x1ccdfa50, 87 },
    { 0x536dfa4d, 43 },
    { 0, 0 },
    { 0x9b2dfa53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132dfa59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8dfa5c, 52 },
    { 0, 0 },
    { 0xf06dfa5e, 85 },
    { 0, 0 },
    { 0x978dfa60, 33 },
    { 0x6b0dfa61, 93 },
    { 0x17adfa62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eddfa6b, 98 },
    { 0, 0 },
    { 0x29cdfa6d, 95 },
    { 0, 0 },
    { 0x3d3dfa6f, 85 },
    { 0xd7dfa70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x272dfa73, 91 },
    { 0x4efdfa74, 61 },
    { 0xc51dfa74, 40 },
    { 0xab5dfa73, 84 },
    { 0x172dfa77, 90 },
    { 0, 0 },
    { 0xe73dfa79, 43 },
    { 0x30edfa7a, 90 },
    { 0xf3fdfa7a, 98 },
    { 0, 0 },
    { 0xefadfa7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x203dfa80, 90 },
    { 0, 0 },
    { 0x9a9dfa82, 98 },
    { 0x151dfa82, 66 },
    { 0xd6fdfa84, 90 },
    { 0, 0 },
    { 0xda0dfa86, 98 },
    { 0x717dfa86, 90 },
    { 0x107dfa86, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfdfa8b, 23 },
    { 0x8e3dfa8b, 90 },
    { 0, 0 },
    { 0xd4bdfa8e, 55 },
    { 0xdbdfa8f, 87 },
    { 0xc0edfa8f, 61 },
    { 0xcecdfa8e, 99 },
    { 0, 0 },
    { 0x968dfa93, 53 },
    { 0, 0 },
    { 0xa46dfa95, 76 },
    { 0x574dfa96, 87 },
    { 0, 0 },
    { 0xd8fdfa98, 23 },
    { 0x34bdfa98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x221dfa9c, 23 },
    { 0x2a5dfa9d, 23 },
    { 0x698dfa9e, 75 },
    { 0xc8dfa9f, 23 },
    { 0xc3adfa9c, 61 },
    { 0x604dfaa1, 67 },
    { 0x59fdfa9f, 34 },
    { 0xa00dfaa3, 46 },
    { 0x2fcdfaa3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcddfaa7, 23 },
    { 0, 0 },
    { 0x85cdfaa9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99edfaac, 98 },
    { 0xd67dfaac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x306dfab0, 55 },
    { 0x3aedfab1, 43 },
    { 0, 0 },
    { 0xb08dfab3, 90 },
    { 0x7a0dfab4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x814dfab7, 90 },
    { 0x32edfab8, 23 },
    { 0xb61dfab9, 84 },
    { 0xa3dfaba, 34 },
    { 0xc4fdfabb, 40 },
    { 0, 0 },
    { 0xddddfabd, 61 },
    { 0x5b8dfabe, 93 },
    { 0xfbedfabf, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5dfac9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ddfad0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacedfad5, 33 },
    { 0x4e9dfad5, 67 },
    { 0xa46dfad7, 100 },
    { 0, 0 },
    { 0xf01dfad9, 98 },
    { 0x716dfada, 61 },
    { 0x637dfadb, 61 },
    { 0xfc4dfadb, 95 },
    { 0x578dfadc, 92 },
    { 0xebcdfade, 33 },
    { 0x6aadfadf, 33 },
    { 0xcdfade, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecdfae6, 95 },
    { 0x64cdfae7, 99 },
    { 0xcf2dfae8, 75 },
    { 0x5dedfae9, 90 },
    { 0x8f8dfae9, 100 },
    { 0x9c0dfaeb, 98 },
    { 0x412dfae6, 33 },
    { 0x79cdfaed, 56 },
    { 0x21adfaed, 88 },
    { 0xa4edfaed, 95 },
    { 0x67dfaf0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9dfaf5, 98 },
    { 0xbf3dfaf5, 69 },
    { 0, 0 },
    { 0xd0bdfaf8, 36 },
    { 0xb09dfaf9, 65 },
    { 0, 0 },
    { 0x120dfafb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21dfb02, 66 },
    { 0x273dfb03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311dfb0a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5dfb12, 34 },
    { 0xab3dfb13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9dfb16, 33 },
    { 0, 0 },
    { 0x5fdfb18, 23 },
    { 0xe37dfb19, 90 },
    { 0x614dfb1a, 61 },
    { 0xe44dfb18, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72dfb23, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x175dfb26, 66 },
    { 0x650dfb26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267dfb2b, 76 },
    { 0, 0 },
    { 0x7aedfb2d, 33 },
    { 0x40ddfb2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4dfb34, 46 },
    { 0x7abdfb34, 95 },
    { 0x99fdfb36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfdfb39, 65 },
    { 0x34edfb3a, 90 },
    { 0x59cdfb3b, 95 },
    { 0x46cdfb3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebddfb40, 98 },
    { 0, 0 },
    { 0x587dfb42, 34 },
    { 0xa87dfb42, 33 },
    { 0xb29dfb44, 33 },
    { 0x899dfb45, 90 },
    { 0xadddfb43, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84adfb4d, 91 },
    { 0xb08dfb4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50dfb52, 78 },
    { 0xd3fdfb53, 61 },
    { 0x5d0dfb53, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fdfb5a, 90 },
    { 0, 0 },
    { 0xb1ddfb5c, 33 },
    { 0x76ddfb5c, 43 },
    { 0xac8dfb5d, 40 },
    { 0xc5adfb5e, 61 },
    { 0x619dfb60, 69 },
    { 0, 0 },
    { 0x93fdfb62, 43 },
    { 0x159dfb63, 33 },
    { 0x810dfb63, 61 },
    { 0x233dfb65, 61 },
    { 0, 0 },
    { 0xe37dfb67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7adfb6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75dfb73, 61 },
    { 0xdccdfb74, 76 },
    { 0xe15dfb74, 23 },
    { 0x2d1dfb76, 77 },
    { 0x1fddfb73, 61 },
    { 0xbc4dfb78, 43 },
    { 0x837dfb79, 84 },
    { 0xb73dfb73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfb84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fcdfb87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dfb8a, 33 },
    { 0xc49dfb8b, 93 },
    { 0xd88dfb8a, 23 },
    { 0xd9edfb8d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x601dfb90, 95 },
    { 0, 0 },
    { 0x527dfb92, 95 },
    { 0x72dfb93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8dfb96, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fdfb9b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x458dfb9e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cdfba1, 34 },
    { 0, 0 },
    { 0x5d2dfba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655dfba7, 91 },
    { 0xa5bdfba8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695dfbae, 46 },
    { 0x5a1dfbaf, 46 },
    { 0x909dfbb0, 43 },
    { 0xedadfbb0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5dfbb9, 46 },
    { 0x69bdfbb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5dfbc0, 95 },
    { 0x8bdfbc1, 90 },
    { 0xd55dfbc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbadfbd3, 66 },
    { 0xaadfbd4, 76 },
    { 0xd96dfbd5, 95 },
    { 0xd97dfbd6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6dfbdc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x875dfbe3, 55 },
    { 0, 0 },
    { 0xfc8dfbe5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x924dfbf0, 95 },
    { 0xe89dfbf0, 43 },
    { 0x362dfbf2, 98 },
    { 0, 0 },
    { 0x770dfbf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acdfbf7, 76 },
    { 0x22fdfbf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3dfbfb, 75 },
    { 0x752dfbfc, 53 },
    { 0x14dfbfc, 87 },
    { 0, 0 },
    { 0x1d5dfbff, 34 },
    { 0x2ddfc00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dfc03, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cdfc0a, 52 },
    { 0xb31dfc0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81dfc13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858dfc18, 33 },
    { 0x586dfc18, 78 },
    { 0xd51dfc1a, 55 },
    { 0, 0 },
    { 0x2bfdfc1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3dfc24, 90 },
    { 0, 0 },
    { 0x33fdfc26, 46 },
    { 0x9d5dfc27, 40 },
    { 0x61dfc28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74dfc2d, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83dfc31, 95 },
    { 0, 0 },
    { 0xfe9dfc33, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fdfc36, 91 },
    { 0x1a8dfc37, 61 },
    { 0xcd6dfc38, 95 },
    { 0x1eddfc39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x291dfc40, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8dfc43, 55 },
    { 0, 0 },
    { 0x17ddfc45, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96dfc4a, 95 },
    { 0x1efdfc4b, 33 },
    { 0, 0 },
    { 0x64bdfc4d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6dfc50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2dfc53, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24adfc59, 23 },
    { 0x749dfc59, 91 },
    { 0xe97dfc5a, 61 },
    { 0xe01dfc5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fdfc61, 23 },
    { 0x1d8dfc62, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37dfc66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37dfc69, 98 },
    { 0x34edfc6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaddfc71, 93 },
    { 0, 0 },
    { 0x961dfc73, 97 },
    { 0x4f0dfc73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1adfc77, 95 },
    { 0x5b2dfc78, 61 },
    { 0xd00dfc79, 76 },
    { 0x5a1dfc78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477dfc7f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03dfc82, 46 },
    { 0xa36dfc83, 61 },
    { 0x3a0dfc84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cdfc87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fdfc8a, 34 },
    { 0, 0 },
    { 0x765dfc8c, 98 },
    { 0xea3dfc8d, 61 },
    { 0, 0 },
    { 0x8d1dfc8f, 98 },
    { 0xccddfc90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3adfc93, 90 },
    { 0, 0 },
    { 0xa38dfc95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fdfca4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8dfca9, 98 },
    { 0xad4dfcaa, 98 },
    { 0x368dfcab, 76 },
    { 0xc41dfcab, 40 },
    { 0xe75dfcab, 26 },
    { 0xfdddfcae, 97 },
    { 0x42cdfcaf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ddfcb9, 84 },
    { 0x212dfcb9, 90 },
    { 0x901dfcbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7dfcbe, 61 },
    { 0, 0 },
    { 0x5e6dfcc0, 95 },
    { 0xccfdfcc1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0dfcc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcedfcc7, 100 },
    { 0x3dcdfcc8, 33 },
    { 0x147dfcc9, 23 },
    { 0x910dfcca, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ddfcd0, 52 },
    { 0xb9cdfcd0, 84 },
    { 0x429dfcd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dfcd7, 61 },
    { 0x2f6dfcd8, 95 },
    { 0x6e5dfcd9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4dfcdd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50dfce6, 78 },
    { 0x897dfce7, 95 },
    { 0, 0 },
    { 0x235dfce9, 90 },
    { 0x20adfcea, 93 },
    { 0x92dfcea, 95 },
    { 0, 0 },
    { 0x615dfced, 90 },
    { 0x76bdfcee, 33 },
    { 0x134dfced, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ddfcf3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248dfcf7, 90 },
    { 0xf4edfcf8, 61 },
    { 0xb37dfcf7, 98 },
    { 0x5d0dfcf7, 34 },
    { 0xc72dfcfb, 95 },
    { 0x7bddfcfc, 33 },
    { 0, 0 },
    { 0xf7cdfcfe, 23 },
    { 0, 0 },
    { 0x6b1dfd00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46dfd09, 61 },
    { 0x5dedfd0a, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777dfd10, 67 },
    { 0, 0 },
    { 0xad6dfd12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x930dfd15, 17 },
    { 0, 0 },
    { 0x80edfd17, 55 },
    { 0xf8adfd17, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9dfd1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706dfd1f, 33 },
    { 0xa7cdfd20, 76 },
    { 0xcb8dfd21, 98 },
    { 0x915dfd22, 78 },
    { 0, 0 },
    { 0x89edfd24, 46 },
    { 0xfeddfd24, 42 },
    { 0xbaadfd26, 84 },
    { 0x860dfd26, 98 },
    { 0x23edfd27, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdfd2b, 43 },
    { 0xb5adfd2c, 84 },
    { 0xdd4dfd2d, 98 },
    { 0x9b8dfd2e, 46 },
    { 0, 0 },
    { 0x49ddfd30, 34 },
    { 0, 0 },
    { 0xfafdfd32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdfd36, 33 },
    { 0, 0 },
    { 0x34adfd38, 61 },
    { 0, 0 },
    { 0x879dfd3a, 23 },
    { 0xfeadfd3b, 61 },
    { 0x4b3dfd3c, 91 },
    { 0xfb1dfd3a, 33 },
    { 0xf66dfd3e, 55 },
    { 0x9b5dfd3a, 95 },
    { 0x17edfd3b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fddfd44, 33 },
    { 0x3a9dfd45, 100 },
    { 0xd03dfd46, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xadedfd49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0dfd50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561dfd57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594dfd5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fdfd60, 84 },
    { 0x395dfd61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486dfd65, 33 },
    { 0x931dfd65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x47adfd69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dddfd75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccddfd7a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1dfd7d, 61 },
    { 0, 0 },
    { 0x344dfd7f, 95 },
    { 0, 0 },
    { 0x705dfd81, 95 },
    { 0x76edfd82, 17 },
    { 0x2a7dfd82, 55 },
    { 0, 0 },
    { 0x1e2dfd85, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4dfd88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32dfd8b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20dfd92, 90 },
    { 0xcfdfd92, 100 },
    { 0, 0 },
    { 0x277dfd95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaddfd9b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fdfda1, 90 },
    { 0x6fdfda2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160dfdaa, 98 },
    { 0xf68dfdaa, 58 },
    { 0xbdcdfdaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bdfdb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952dfdb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x895dfdb9, 33 },
    { 0x9aedfdba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaadfdbf, 52 },
    { 0x5e5dfdbf, 61 },
    { 0x603dfdbf, 33 },
    { 0, 0 },
    { 0xce8dfdc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493dfdc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cdfdcb, 34 },
    { 0x335dfdcc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23adfdd6, 23 },
    { 0x60edfdd7, 33 },
    { 0, 0 },
    { 0x9d5dfdd9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6edfddc, 33 },
    { 0xf9edfddd, 23 },
    { 0, 0 },
    { 0xcfcdfddf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fdfde4, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adfde7, 46 },
    { 0x311dfde8, 93 },
    { 0xcf3dfde9, 95 },
    { 0xdcfdfdea, 55 },
    { 0x4e1dfde7, 65 },
    { 0xac6dfdec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116dfdf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x630dfdf9, 23 },
    { 0, 0 },
    { 0xd6ddfdfb, 95 },
    { 0x52bdfdfc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x668dfe03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4dfe08, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bdfe0b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178dfe0f, 61 },
    { 0, 0 },
    { 0x760dfe11, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacddfe15, 33 },
    { 0x98dfe16, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bdfe1a, 61 },
    { 0xcd8dfe1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94dfe22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x640dfe25, 23 },
    { 0, 0 },
    { 0xea4dfe27, 90 },
    { 0x874dfe27, 83 },
    { 0x62fdfe29, 23 },
    { 0x4e7dfe2a, 90 },
    { 0x1a5dfe28, 33 },
    { 0xdb0dfe2c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690dfe32, 33 },
    { 0x189dfe33, 90 },
    { 0xe67dfe34, 61 },
    { 0xc2bdfe34, 95 },
    { 0x310dfe34, 76 },
    { 0xccbdfe34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324dfe3c, 46 },
    { 0x323dfe3d, 98 },
    { 0xc9bdfe3e, 40 },
    { 0x1a1dfe3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496dfe45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dfe48, 23 },
    { 0x3a1dfe49, 90 },
    { 0x4c9dfe4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42edfe50, 95 },
    { 0xe67dfe50, 77 },
    { 0xf31dfe50, 61 },
    { 0x8d9dfe53, 33 },
    { 0xe23dfe53, 95 },
    { 0x999dfe55, 90 },
    { 0x738dfe56, 77 },
    { 0xab7dfe55, 66 },
    { 0x9bdfe58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x519dfe5b, 90 },
    { 0xd71dfe5c, 23 },
    { 0xf84dfe5d, 61 },
    { 0, 0 },
    { 0x37edfe5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4edfe62, 55 },
    { 0x7dedfe63, 61 },
    { 0x46ddfe64, 84 },
    { 0xd6ddfe63, 90 },
    { 0, 0 },
    { 0x470dfe67, 55 },
    { 0, 0 },
    { 0x225dfe69, 55 },
    { 0x24bdfe6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bdfe6d, 61 },
    { 0xefdfe6d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82edfe72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfdfe75, 90 },
    { 0xdbddfe75, 33 },
    { 0x2f7dfe77, 87 },
    { 0x829dfe76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7dfe7b, 33 },
    { 0, 0 },
    { 0x213dfe7d, 43 },
    { 0xecbdfe7e, 61 },
    { 0xe7bdfe7f, 78 },
    { 0xdb0dfe80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0dfe83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaadfe87, 90 },
    { 0x471dfe88, 66 },
    { 0xe10dfe87, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09dfe8c, 90 },
    { 0, 0 },
    { 0x48dfe8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cdfe95, 52 },
    { 0, 0 },
    { 0xce9dfe97, 84 },
    { 0x8c5dfe98, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fadfea2, 67 },
    { 0, 0 },
    { 0xc60dfea4, 95 },
    { 0xbd6dfea5, 95 },
    { 0x2a5dfea4, 93 },
    { 0xfcfdfea7, 98 },
    { 0x9e3dfea8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbdfeae, 68 },
    { 0, 0 },
    { 0xef5dfeb0, 33 },
    { 0, 0 },
    { 0x21edfeb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6dfebe, 90 },
    { 0, 0 },
    { 0x4f2dfec0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200dfec7, 87 },
    { 0xa10dfec7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90dfece, 76 },
    { 0x3f2dfecf, 61 },
    { 0, 0 },
    { 0x9a9dfed1, 46 },
    { 0, 0 },
    { 0x483dfed3, 95 },
    { 0xf5bdfed4, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeadfed8, 23 },
    { 0xbefdfed9, 61 },
    { 0, 0 },
    { 0x911dfedb, 24 },
    { 0, 0 },
    { 0x385dfedd, 43 },
    { 0x950dfede, 95 },
    { 0x33cdfedf, 98 },
    { 0xe9dfedd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3dfeea, 98 },
    { 0, 0 },
    { 0xe3ddfeec, 61 },
    { 0, 0 },
    { 0x928dfeee, 37 },
    { 0, 0 },
    { 0xaf2dfef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcdfef7, 90 },
    { 0xf54dfef8, 98 },
    { 0x5b0dfef8, 96 },
    { 0xf85dfefa, 76 },
    { 0, 0 },
    { 0xeebdfefc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x643dfeff, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cdff05, 81 },
    { 0xe5cdff06, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6dff0b, 95 },
    { 0x392dff0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x407dff0f, 93 },
    { 0x1fadff10, 76 },
    { 0xd24dff0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9dff17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5dff1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4dff20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0dff28, 34 },
    { 0xaa4dff29, 88 },
    { 0x380dff2a, 90 },
    { 0x148dff2a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85adff34, 46 },
    { 0, 0 },
    { 0x52adff36, 98 },
    { 0xddfdff37, 78 },
    { 0x55fdff38, 85 },
    { 0xf19dff39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08dff3f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bdff43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21dff47, 33 },
    { 0, 0 },
    { 0x12cdff49, 90 },
    { 0, 0 },
    { 0xafadff4b, 90 },
    { 0xef1dff4c, 76 },
    { 0, 0 },
    { 0xd02dff4e, 85 },
    { 0xd8fdff4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2dff52, 99 },
    { 0x7f4dff52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20dff57, 78 },
    { 0xab7dff57, 17 },
    { 0x72edff59, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafdff5c, 61 },
    { 0, 0 },
    { 0xa84dff5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadff66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04dff69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99dff6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dff72, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cdff7b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x65edff7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1dff81, 95 },
    { 0x7badff81, 98 },
    { 0x2b3dff83, 23 },
    { 0x667dff83, 95 },
    { 0, 0 },
    { 0x550dff86, 33 },
    { 0x996dff87, 85 },
    { 0x615dff87, 90 },
    { 0, 0 },
    { 0x406dff8a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823dff91, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54edff94, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacddff9a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22dff9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6fdffa4, 98 },
    { 0x1c5dffa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777dffab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cdffb0, 33 },
    { 0xa5fdffb0, 33 },
    { 0, 0 },
    { 0xab0dffb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45dffb7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634dffbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cdffbe, 55 },
    { 0x3f2dffbf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x817dffc2, 95 },
    { 0x952dffc3, 85 },
    { 0xe39dffc2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x318dffc7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aedffca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549dffcf, 95 },
    { 0x615dffd0, 34 },
    { 0x9bedffd0, 43 },
    { 0, 0 },
    { 0x6c1dffd3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2dffd6, 23 },
    { 0x15bdffd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9dffde, 66 },
    { 0x226dffdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212dffe5, 80 },
    { 0x92bdffe5, 55 },
    { 0, 0 },
    { 0xa5bdffe8, 61 },
    { 0xffedffe9, 98 },
    { 0xebadffe9, 68 },
    { 0x1a6dffea, 46 },
    { 0x14dffec, 33 },
    { 0x9bddffed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80dfffc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45edffff, 95 },
    { 0, 0 },
    { 0xf57e0001, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e0005, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517e0009, 98 },
    { 0xf64e0009, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xface0010, 90 },
    { 0, 0 },
    { 0x1d1e0012, 33 },
    { 0, 0 },
    { 0xc83e0014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x968e0017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e001b, 61 },
    { 0, 0 },
    { 0x1dfe001d, 95 },
    { 0xe74e001d, 33 },
    { 0x29e001f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9e002b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141e002f, 52 },
    { 0xf8ae002f, 84 },
    { 0xba6e002f, 90 },
    { 0xfc2e0030, 90 },
    { 0x1e7e0030, 98 },
    { 0xab9e0032, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51e003c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ae003f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebe0042, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e0047, 40 },
    { 0, 0 },
    { 0x824e0049, 78 },
    { 0xfc7e004a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ae0052, 36 },
    { 0, 0 },
    { 0xfaae0054, 23 },
    { 0x6cae0055, 61 },
    { 0xeece0054, 23 },
    { 0x1e5e0057, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2e005a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce005e, 34 },
    { 0x5dde005f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898e0063, 78 },
    { 0xb5ae0064, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e0069, 77 },
    { 0x2c3e006a, 78 },
    { 0x559e006b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e0074, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37e0077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ee007a, 33 },
    { 0x434e007b, 61 },
    { 0xda8e007b, 96 },
    { 0, 0 },
    { 0x92fe007e, 33 },
    { 0x11ce007f, 34 },
    { 0, 0 },
    { 0x617e0081, 100 },
    { 0xb4ce0081, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55e0087, 23 },
    { 0, 0 },
    { 0x646e0089, 55 },
    { 0x5dce0089, 76 },
    { 0, 0 },
    { 0x461e008c, 91 },
    { 0x6c0e008c, 95 },
    { 0xa42e008e, 33 },
    { 0xe1be008f, 43 },
    { 0x8afe0090, 90 },
    { 0x67e0091, 98 },
    { 0x17de0092, 85 },
    { 0xe01e0093, 43 },
    { 0xac5e008e, 65 },
    { 0x35de0095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35e009c, 95 },
    { 0x544e009d, 26 },
    { 0x492e009e, 23 },
    { 0xe11e009d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8e00a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x723e00ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9e00b0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e00b5, 90 },
    { 0x291e00b5, 95 },
    { 0x15ae00b7, 43 },
    { 0x762e00b8, 55 },
    { 0xa4be00b9, 23 },
    { 0, 0 },
    { 0xff1e00bb, 33 },
    { 0, 0 },
    { 0xef4e00bd, 23 },
    { 0, 0 },
    { 0x98ee00bf, 46 },
    { 0x28ae00bf, 67 },
    { 0x740e00bf, 93 },
    { 0x757e00c2, 98 },
    { 0xa07e00c0, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27de00cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7e00d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eae00d7, 98 },
    { 0x7c5e00d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dce00db, 93 },
    { 0xb03e00dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de00df, 23 },
    { 0x249e00e0, 33 },
    { 0, 0 },
    { 0x51ee00e2, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49e00e5, 34 },
    { 0xb48e00e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3e00ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfece00ef, 90 },
    { 0x873e00f0, 98 },
    { 0xc73e00f1, 95 },
    { 0x3d2e00f2, 95 },
    { 0x8fee00f1, 23 },
    { 0x9d6e00f1, 23 },
    { 0x4bde00f0, 23 },
    { 0, 0 },
    { 0xa62e00f7, 90 },
    { 0xb39e00f8, 93 },
    { 0xb3ee00f9, 61 },
    { 0x618e00f9, 43 },
    { 0xb7e00fb, 23 },
    { 0x675e00fb, 50 },
    { 0, 0 },
    { 0x858e00fe, 98 },
    { 0, 0 },
    { 0x83ce0100, 61 },
    { 0, 0 },
    { 0xc19e0102, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee0107, 99 },
    { 0x7b9e0108, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x732e010b, 55 },
    { 0x8c6e010b, 33 },
    { 0, 0 },
    { 0x7dee010e, 43 },
    { 0, 0 },
    { 0x894e0110, 33 },
    { 0x4cfe0110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ce0114, 34 },
    { 0, 0 },
    { 0x5d7e0116, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dee0119, 98 },
    { 0, 0 },
    { 0x41de011b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afe011f, 80 },
    { 0x727e0120, 67 },
    { 0x43ee0121, 33 },
    { 0, 0 },
    { 0x839e0123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x454e0126, 46 },
    { 0, 0 },
    { 0x107e0128, 95 },
    { 0x498e0129, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x969e012c, 33 },
    { 0x889e012d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2e0130, 65 },
    { 0x2bae0131, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x303e0134, 69 },
    { 0x4b6e0135, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91de013b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce014d, 61 },
    { 0x4abe014d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcae0153, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de0158, 95 },
    { 0x2c0e0159, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5de015c, 55 },
    { 0xaa0e015d, 66 },
    { 0xf91e015d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ce0161, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce0166, 76 },
    { 0x5bee0167, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ce0171, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dee0174, 34 },
    { 0, 0 },
    { 0x8f0e0176, 91 },
    { 0x9d2e0177, 90 },
    { 0x65e0176, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ce017b, 98 },
    { 0x96ce017c, 98 },
    { 0xc2be017b, 95 },
    { 0x7a3e017c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6e0181, 61 },
    { 0x9b8e0181, 52 },
    { 0xf35e0183, 33 },
    { 0x5e4e0184, 98 },
    { 0xdc7e0184, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x270e0188, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480e018e, 99 },
    { 0xd3de018e, 98 },
    { 0, 0 },
    { 0x42ee0191, 33 },
    { 0x665e0192, 55 },
    { 0xac4e0192, 95 },
    { 0, 0 },
    { 0xf63e0195, 23 },
    { 0x954e0196, 76 },
    { 0, 0 },
    { 0xb03e0198, 84 },
    { 0x945e0199, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675e019d, 66 },
    { 0xf90e019e, 84 },
    { 0x860e019f, 19 },
    { 0xe45e01a0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x267e01a3, 33 },
    { 0, 0 },
    { 0xba0e01a5, 77 },
    { 0, 0 },
    { 0xc29e01a7, 56 },
    { 0x63de01a7, 33 },
    { 0x9bfe01a9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ce01ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafae01b2, 55 },
    { 0, 0 },
    { 0xb5ee01b4, 90 },
    { 0xb03e01b5, 98 },
    { 0x5a4e01b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7e01bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4e01c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412e01c9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5be01d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbe01d6, 68 },
    { 0x580e01d7, 61 },
    { 0x9fae01d8, 61 },
    { 0x319e01d8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589e01dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11be01e0, 61 },
    { 0x33be01e1, 85 },
    { 0x57fe01e1, 23 },
    { 0, 0 },
    { 0xad8e01e4, 46 },
    { 0x16ee01e4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfe01eb, 87 },
    { 0, 0 },
    { 0x869e01ed, 36 },
    { 0xd02e01ee, 95 },
    { 0, 0 },
    { 0x928e01f0, 98 },
    { 0, 0 },
    { 0x17ce01f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886e01f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14e01f9, 93 },
    { 0x664e01fa, 23 },
    { 0xb48e01fb, 81 },
    { 0x2abe01fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57e01ff, 76 },
    { 0xb34e0200, 76 },
    { 0x98de0201, 99 },
    { 0x855e01ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2e0206, 43 },
    { 0, 0 },
    { 0x6c1e0208, 90 },
    { 0, 0 },
    { 0x10ae020a, 78 },
    { 0x7a4e020b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12e0211, 98 },
    { 0, 0 },
    { 0x389e0213, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdce0216, 90 },
    { 0, 0 },
    { 0x5b5e0218, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7e021c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0e0220, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939e0224, 76 },
    { 0x8a5e0225, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacee0229, 61 },
    { 0, 0 },
    { 0xa49e022b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8e022e, 61 },
    { 0x49ee022f, 34 },
    { 0xceae0230, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdce023b, 55 },
    { 0x9bae023c, 33 },
    { 0x249e023d, 67 },
    { 0x805e023d, 90 },
    { 0x545e023d, 99 },
    { 0, 0 },
    { 0x6dee0241, 90 },
    { 0x8b4e0242, 78 },
    { 0x8f4e0241, 23 },
    { 0xd71e0244, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482e024d, 90 },
    { 0x5c3e024e, 95 },
    { 0, 0 },
    { 0xcc4e0250, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ae0257, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fe025a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6e0260, 95 },
    { 0, 0 },
    { 0xf82e0262, 101 },
    { 0x744e0263, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6e0270, 68 },
    { 0xa93e0271, 87 },
    { 0xe0ce0272, 99 },
    { 0, 0 },
    { 0xede0274, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735e0278, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ae027c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00e0284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9be028b, 33 },
    { 0, 0 },
    { 0xa7de028d, 76 },
    { 0xa6e028e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x221e0291, 61 },
    { 0xc5be0292, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e0296, 98 },
    { 0, 0 },
    { 0x8dde0298, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e029b, 90 },
    { 0xc0ce029b, 23 },
    { 0xc2fe029c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee02a3, 87 },
    { 0x48ae02a4, 95 },
    { 0xf5e02a5, 65 },
    { 0, 0 },
    { 0xab2e02a7, 33 },
    { 0, 0 },
    { 0x678e02a9, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be02ae, 96 },
    { 0, 0 },
    { 0x790e02b0, 76 },
    { 0x739e02b1, 33 },
    { 0xe04e02b1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cde02b5, 90 },
    { 0xd0ae02b6, 40 },
    { 0x89fe02b7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e02ba, 55 },
    { 0xfcbe02bb, 95 },
    { 0, 0 },
    { 0x7bce02bd, 85 },
    { 0, 0 },
    { 0xd4ee02bf, 76 },
    { 0xd0ce02bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2e02c5, 40 },
    { 0x290e02c5, 100 },
    { 0x392e02c7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbe02cb, 95 },
    { 0xedde02cc, 88 },
    { 0xdc7e02cd, 61 },
    { 0xa85e02cc, 67 },
    { 0x88ae02cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3e02d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbe02d7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694e02db, 91 },
    { 0xde7e02dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707e02e3, 78 },
    { 0x8e0e02e3, 90 },
    { 0, 0 },
    { 0xbde02e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ce02ee, 98 },
    { 0, 0 },
    { 0xa2ae02f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2e02f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ce02f7, 33 },
    { 0, 0 },
    { 0x175e02f9, 34 },
    { 0x94de02fa, 85 },
    { 0xaf2e02f9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ae0301, 93 },
    { 0x767e0302, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23e030d, 76 },
    { 0xd31e030e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e0314, 45 },
    { 0xfd6e0314, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4be0318, 91 },
    { 0x7d1e0319, 61 },
    { 0, 0 },
    { 0xcbde031b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3de0320, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ace0325, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48be0329, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbee032d, 55 },
    { 0x657e032e, 76 },
    { 0xece032e, 44 },
    { 0, 0 },
    { 0x904e0331, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5e0335, 33 },
    { 0, 0 },
    { 0x712e0337, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e033a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ee033e, 95 },
    { 0x3f9e033e, 55 },
    { 0x29be033f, 23 },
    { 0x569e033f, 55 },
    { 0, 0 },
    { 0xef0e0343, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87be0347, 46 },
    { 0, 0 },
    { 0xa58e0349, 98 },
    { 0x976e034a, 68 },
    { 0xde034a, 61 },
    { 0x5b4e034c, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3e034f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ece0352, 61 },
    { 0, 0 },
    { 0xa20e0354, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ae0359, 61 },
    { 0, 0 },
    { 0x74fe035b, 61 },
    { 0xb31e035c, 90 },
    { 0x60de035c, 90 },
    { 0x830e035b, 90 },
    { 0x498e035f, 23 },
    { 0x47de0360, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917e0364, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ce0368, 23 },
    { 0x978e0369, 87 },
    { 0, 0 },
    { 0xd5ae036b, 98 },
    { 0, 0 },
    { 0xf7ce036d, 61 },
    { 0, 0 },
    { 0x619e036f, 33 },
    { 0xab3e0370, 43 },
    { 0, 0 },
    { 0xca0e0372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626e0376, 43 },
    { 0xe6be0377, 66 },
    { 0x80fe0378, 34 },
    { 0x4e5e0378, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ee0380, 61 },
    { 0xda3e0380, 23 },
    { 0xcede0380, 76 },
    { 0xd57e0381, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe0386, 98 },
    { 0, 0 },
    { 0x2fae0388, 23 },
    { 0xb52e0389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4e038c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x758e038f, 99 },
    { 0xf44e0390, 90 },
    { 0x109e038f, 53 },
    { 0x377e0392, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ae0396, 78 },
    { 0x1ebe0396, 67 },
    { 0xa9ee0398, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23e039c, 68 },
    { 0x66ee039d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fe03a0, 61 },
    { 0, 0 },
    { 0xefe03a2, 43 },
    { 0, 0 },
    { 0xeae03a4, 66 },
    { 0xbdce03a5, 46 },
    { 0, 0 },
    { 0xbc0e03a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e03ad, 98 },
    { 0xb24e03ae, 84 },
    { 0xd4de03ae, 90 },
    { 0x2c5e03af, 23 },
    { 0x874e03b1, 90 },
    { 0xd17e03b1, 61 },
    { 0, 0 },
    { 0x656e03b4, 52 },
    { 0, 0 },
    { 0x46be03b6, 23 },
    { 0xdfe03b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fe03c3, 98 },
    { 0xdbde03c4, 76 },
    { 0x3be03c3, 23 },
    { 0x265e03c6, 95 },
    { 0xa95e03c7, 98 },
    { 0, 0 },
    { 0x167e03c9, 100 },
    { 0xfc7e03ca, 78 },
    { 0xefee03cb, 76 },
    { 0x524e03cc, 90 },
    { 0x819e03c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ee03d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4e03e2, 90 },
    { 0, 0 },
    { 0xa20e03e4, 83 },
    { 0x1c0e03e5, 61 },
    { 0xaede03e5, 34 },
    { 0, 0 },
    { 0x10de03e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809e03ec, 33 },
    { 0x948e03ed, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e03f3, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4e03f6, 76 },
    { 0xcabe03f7, 93 },
    { 0, 0 },
    { 0xe81e03f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7e0401, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ede0406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e040c, 76 },
    { 0x61ae040d, 34 },
    { 0x5d6e040d, 90 },
    { 0x9c8e040f, 93 },
    { 0x58ce0410, 90 },
    { 0x38be0411, 46 },
    { 0xaade0410, 43 },
    { 0x287e0410, 98 },
    { 0x79be0414, 90 },
    { 0x2a2e0411, 61 },
    { 0, 0 },
    { 0xd2be0417, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfe041c, 90 },
    { 0, 0 },
    { 0xfd8e041e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ae0421, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ae0426, 98 },
    { 0, 0 },
    { 0xafbe0428, 90 },
    { 0xc4de0429, 33 },
    { 0x67ee042a, 90 },
    { 0x6d6e042a, 93 },
    { 0xff1e042c, 33 },
    { 0x2fde042c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbe0432, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805e0436, 90 },
    { 0x220e0437, 55 },
    { 0, 0 },
    { 0x15be0439, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796e043e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fee0441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251e0446, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ae044b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e0455, 55 },
    { 0x334e0456, 76 },
    { 0xb98e0457, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8fe045a, 90 },
    { 0xf94e045b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fe0460, 100 },
    { 0xa1be0461, 61 },
    { 0x303e0462, 17 },
    { 0, 0 },
    { 0x4a1e0464, 98 },
    { 0x5aae0464, 81 },
    { 0xa26e0466, 61 },
    { 0xa8ee0466, 93 },
    { 0x1dce0467, 78 },
    { 0xb14e0467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ae046d, 90 },
    { 0xb43e046e, 93 },
    { 0, 0 },
    { 0x68fe0470, 61 },
    { 0x78ae0471, 95 },
    { 0xaeae0470, 34 },
    { 0x91be0472, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2e047b, 93 },
    { 0x768e047c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ae0484, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e0487, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52e048b, 84 },
    { 0xface048b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2e048f, 99 },
    { 0xf84e048f, 33 },
    { 0x701e0490, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x951e0498, 76 },
    { 0x881e0498, 90 },
    { 0xc5de049a, 95 },
    { 0x51fe049a, 93 },
    { 0xe9ce049c, 61 },
    { 0x217e049c, 98 },
    { 0x4f1e049e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e04a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fe04a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9e04a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358e04b2, 90 },
    { 0xbf8e04b3, 98 },
    { 0, 0 },
    { 0x34e04b5, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e04bf, 46 },
    { 0, 0 },
    { 0x6e3e04c1, 61 },
    { 0x250e04c2, 90 },
    { 0, 0 },
    { 0xb63e04c4, 91 },
    { 0xcf2e04c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fce04cb, 33 },
    { 0x76ce04cc, 23 },
    { 0, 0 },
    { 0xc6ee04ce, 98 },
    { 0x74be04ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1e04d4, 76 },
    { 0, 0 },
    { 0xc2ee04d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41e04d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e04dc, 84 },
    { 0xe67e04dd, 95 },
    { 0, 0 },
    { 0x99ee04df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ae04e2, 33 },
    { 0, 0 },
    { 0x9fae04e4, 55 },
    { 0x24fe04e4, 95 },
    { 0x6f1e04e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e04ea, 23 },
    { 0x311e04eb, 77 },
    { 0xc3e04eb, 34 },
    { 0, 0 },
    { 0xcbee04ee, 33 },
    { 0, 0 },
    { 0xbf5e04f0, 43 },
    { 0, 0 },
    { 0x208e04f2, 95 },
    { 0x88ee04f3, 33 },
    { 0xb20e04f4, 61 },
    { 0x890e04f2, 66 },
    { 0x6ae04f6, 34 },
    { 0, 0 },
    { 0x39de04f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29e0500, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90e050a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9e050d, 76 },
    { 0xbb0e050d, 61 },
    { 0x21de050f, 33 },
    { 0, 0 },
    { 0x6b2e0511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2de0517, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae051c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce0523, 23 },
    { 0, 0 },
    { 0xb49e0525, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348e0529, 12 },
    { 0, 0 },
    { 0x60ae052b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4e052e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x766e0531, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfe0538, 33 },
    { 0x967e0539, 60 },
    { 0x974e0538, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fe053d, 33 },
    { 0xfe4e053e, 90 },
    { 0xd4ae053f, 68 },
    { 0, 0 },
    { 0xc6be0541, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x760e0544, 23 },
    { 0, 0 },
    { 0x6f4e0546, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cee054a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfe0551, 23 },
    { 0x7aee0552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868e0556, 17 },
    { 0, 0 },
    { 0xbe2e0558, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384e055e, 33 },
    { 0, 0 },
    { 0xec7e0560, 34 },
    { 0x65ce0560, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e0567, 95 },
    { 0x8a5e0568, 55 },
    { 0x93ce0569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9e0571, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e0578, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1e057e, 84 },
    { 0x701e057f, 98 },
    { 0, 0 },
    { 0xa28e0581, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ae0587, 80 },
    { 0, 0 },
    { 0x5ace0589, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152e058f, 76 },
    { 0x594e058f, 90 },
    { 0x16de0591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862e0598, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e059b, 61 },
    { 0x3dde059c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde059f, 23 },
    { 0xc65e05a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475e05a4, 23 },
    { 0xa0e05a5, 33 },
    { 0, 0 },
    { 0x3a5e05a7, 46 },
    { 0xd07e05a7, 69 },
    { 0xb73e05a7, 90 },
    { 0x5b3e05a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae05b2, 43 },
    { 0x4bde05b3, 98 },
    { 0, 0 },
    { 0x4d0e05b5, 98 },
    { 0xd95e05b6, 98 },
    { 0xcdae05b7, 61 },
    { 0xb24e05b8, 99 },
    { 0xd57e05b6, 55 },
    { 0, 0 },
    { 0x8fee05bb, 99 },
    { 0x273e05bc, 76 },
    { 0x7abe05bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9e05c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e05c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fe05cc, 55 },
    { 0, 0 },
    { 0x4d0e05ce, 85 },
    { 0x408e05cf, 66 },
    { 0x2fae05ce, 96 },
    { 0xc6ce05cf, 33 },
    { 0xd96e05d2, 88 },
    { 0x29e05d2, 76 },
    { 0x674e05d1, 46 },
    { 0, 0 },
    { 0xf70e05d6, 95 },
    { 0x2c4e05d7, 93 },
    { 0xc36e05d8, 66 },
    { 0xb5ce05d9, 52 },
    { 0x20ee05d9, 90 },
    { 0, 0 },
    { 0xabde05dc, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ace05e0, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae05e3, 46 },
    { 0, 0 },
    { 0x989e05e5, 95 },
    { 0x7e7e05e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5e05ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4e05f0, 55 },
    { 0x9a3e05f0, 90 },
    { 0xc59e05f2, 23 },
    { 0x741e05f3, 61 },
    { 0xfb4e05f2, 55 },
    { 0, 0 },
    { 0x3aae05f6, 90 },
    { 0, 0 },
    { 0x534e05f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ace05fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6de05ff, 95 },
    { 0xebce05ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x993e0603, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363e0607, 90 },
    { 0xede0608, 17 },
    { 0x680e0607, 76 },
    { 0x880e060a, 90 },
    { 0x3a4e060b, 61 },
    { 0xb88e060c, 93 },
    { 0x481e0607, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2e0610, 87 },
    { 0xcbae0610, 95 },
    { 0x2cfe0612, 52 },
    { 0x9d0e0613, 98 },
    { 0x6c9e0614, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fe0617, 61 },
    { 0xee5e0618, 90 },
    { 0xc48e0619, 85 },
    { 0x5a6e061a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24e061f, 95 },
    { 0x346e061f, 66 },
    { 0x774e0620, 95 },
    { 0xf7ae0621, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2e0628, 34 },
    { 0x9fde0628, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce062e, 33 },
    { 0xf65e062f, 61 },
    { 0x4c4e0630, 90 },
    { 0xc52e0630, 33 },
    { 0x947e0632, 90 },
    { 0x81de0631, 95 },
    { 0xf4ee0634, 36 },
    { 0xe54e0633, 90 },
    { 0x752e062e, 100 },
    { 0xa95e0637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0e063d, 87 },
    { 0xa5be063e, 43 },
    { 0xfc2e063f, 61 },
    { 0, 0 },
    { 0xe6ee0641, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e0644, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ee0647, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ae064d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dee0653, 51 },
    { 0xbd0e0654, 61 },
    { 0x93e0655, 61 },
    { 0x9a6e0653, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe065a, 33 },
    { 0, 0 },
    { 0xf8de065c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581e0661, 84 },
    { 0x7de0662, 90 },
    { 0x7d7e0661, 90 },
    { 0xde5e0661, 98 },
    { 0xc62e0661, 33 },
    { 0x3a8e0664, 90 },
    { 0x130e0665, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e066c, 90 },
    { 0xf49e066c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896e0673, 61 },
    { 0x5d4e0674, 43 },
    { 0xd48e0673, 96 },
    { 0xb48e0676, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de0679, 66 },
    { 0x154e067a, 91 },
    { 0x922e067a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ce0682, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ee0688, 34 },
    { 0x348e0689, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x506e068c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1e068f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270e0696, 90 },
    { 0x590e0697, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726e06a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4de06a7, 23 },
    { 0x535e06a8, 67 },
    { 0x410e06a7, 90 },
    { 0xe99e06aa, 90 },
    { 0, 0 },
    { 0x96be06ac, 95 },
    { 0xff3e06ad, 33 },
    { 0x138e06ac, 90 },
    { 0xa6ae06af, 52 },
    { 0x2bee06af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78e06b9, 52 },
    { 0, 0 },
    { 0x91de06bb, 33 },
    { 0x21ce06bb, 61 },
    { 0xcf8e06bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522e06c2, 98 },
    { 0, 0 },
    { 0xd2ae06c4, 98 },
    { 0, 0 },
    { 0xe7ae06c6, 47 },
    { 0xbf5e06c7, 23 },
    { 0xd51e06c8, 95 },
    { 0xc33e06c7, 52 },
    { 0xd75e06c9, 23 },
    { 0, 0 },
    { 0xc4e06cc, 66 },
    { 0x6a9e06cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e06d2, 77 },
    { 0x438e06d3, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53de06d7, 90 },
    { 0, 0 },
    { 0x45ce06d9, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x584e06dc, 98 },
    { 0xd64e06dd, 61 },
    { 0xba1e06dc, 40 },
    { 0, 0 },
    { 0x118e06e0, 85 },
    { 0, 0 },
    { 0xea5e06e2, 78 },
    { 0x9be06e3, 23 },
    { 0, 0 },
    { 0x1ece06e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x295e06e8, 84 },
    { 0, 0 },
    { 0xefee06ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e06ed, 90 },
    { 0x23ce06ed, 59 },
    { 0x371e06ed, 100 },
    { 0xbece06f0, 23 },
    { 0x3dbe06f1, 66 },
    { 0x54ee06f1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e0e06f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e06f8, 43 },
    { 0xc73e06f8, 33 },
    { 0xb8de06fa, 90 },
    { 0xf0de06f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9e0702, 76 },
    { 0x2be0703, 17 },
    { 0x370e0702, 66 },
    { 0x659e0704, 61 },
    { 0, 0 },
    { 0xbf2e0707, 93 },
    { 0xec7e0707, 46 },
    { 0x9dae0709, 33 },
    { 0x938e070a, 95 },
    { 0x744e0708, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe0710, 33 },
    { 0, 0 },
    { 0x969e0712, 33 },
    { 0xf2be0713, 87 },
    { 0, 0 },
    { 0xc4ae0715, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2de0718, 99 },
    { 0, 0 },
    { 0x955e071a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e0723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410e0729, 33 },
    { 0, 0 },
    { 0xa40e072b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88e072e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5e0731, 90 },
    { 0xad4e0731, 98 },
    { 0x4cbe0733, 98 },
    { 0x9efe0734, 98 },
    { 0, 0 },
    { 0xabe0736, 38 },
    { 0x8ace0737, 90 },
    { 0x32e0738, 83 },
    { 0xe6be0737, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x471e073d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x814e0742, 46 },
    { 0x72ee0743, 61 },
    { 0x4a5e0744, 33 },
    { 0xf77e0743, 67 },
    { 0x94e0746, 95 },
    { 0x99fe0746, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e074a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82e074e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48e0751, 88 },
    { 0x19de0752, 87 },
    { 0x10ee0752, 90 },
    { 0x3aae0752, 23 },
    { 0x4d4e0755, 93 },
    { 0x351e0752, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7be0759, 90 },
    { 0x63fe0759, 90 },
    { 0, 0 },
    { 0x6c1e075c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0de0760, 88 },
    { 0, 0 },
    { 0x7e9e0762, 98 },
    { 0x6ade0763, 23 },
    { 0x5afe0764, 99 },
    { 0x3cfe0764, 98 },
    { 0x159e0766, 66 },
    { 0x97e0764, 55 },
    { 0x4bfe0767, 61 },
    { 0xfafe0769, 33 },
    { 0xfde0769, 61 },
    { 0, 0 },
    { 0x4c2e076c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32e0773, 34 },
    { 0, 0 },
    { 0xac6e0775, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ae0779, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e0780, 33 },
    { 0x89de0781, 46 },
    { 0xd98e0782, 90 },
    { 0xf12e0782, 95 },
    { 0, 0 },
    { 0xbbde0785, 55 },
    { 0x995e0786, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0e078c, 17 },
    { 0xdb0e078d, 33 },
    { 0x6fae078e, 90 },
    { 0, 0 },
    { 0x7e3e0790, 90 },
    { 0xa66e0791, 81 },
    { 0, 0 },
    { 0x104e0793, 23 },
    { 0xea4e0794, 91 },
    { 0, 0 },
    { 0x6dae0796, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e079f, 81 },
    { 0xde9e07a0, 76 },
    { 0x605e079f, 101 },
    { 0x6a2e07a2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e07a7, 33 },
    { 0, 0 },
    { 0x916e07a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ade07ae, 40 },
    { 0xd2de07ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ee07b2, 95 },
    { 0x5e3e07b3, 88 },
    { 0xb17e07b2, 78 },
    { 0x7b4e07b5, 90 },
    { 0x6f4e07b5, 34 },
    { 0xc0fe07b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e07ba, 91 },
    { 0xeb9e07ba, 90 },
    { 0xdcfe07bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x754e07bf, 61 },
    { 0, 0 },
    { 0x64ae07c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e07c9, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e07ce, 93 },
    { 0, 0 },
    { 0x8b2e07d0, 23 },
    { 0xed1e07d1, 78 },
    { 0x496e07d0, 76 },
    { 0, 0 },
    { 0x1ffe07d4, 90 },
    { 0, 0 },
    { 0x8e4e07d6, 95 },
    { 0x962e07d6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27e07da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7e07de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ae07e1, 61 },
    { 0, 0 },
    { 0x5b0e07e3, 90 },
    { 0x25de07e4, 38 },
    { 0xf36e07e5, 99 },
    { 0, 0 },
    { 0xa54e07e7, 91 },
    { 0x16ae07e8, 90 },
    { 0xa33e07e8, 43 },
    { 0x8f2e07e7, 67 },
    { 0xba1e07eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de07f1, 90 },
    { 0xb4e07f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ce07f9, 98 },
    { 0xb2ee07fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be0801, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e0804, 61 },
    { 0xa6be0805, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de0808, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689e080e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8e0811, 23 },
    { 0x196e0812, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47fe081b, 55 },
    { 0, 0 },
    { 0x428e081d, 23 },
    { 0x2ffe081e, 93 },
    { 0xec3e081f, 95 },
    { 0x132e081f, 98 },
    { 0, 0 },
    { 0x4dce0822, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926e0828, 55 },
    { 0, 0 },
    { 0xec5e082a, 61 },
    { 0, 0 },
    { 0x7abe082c, 34 },
    { 0, 0 },
    { 0xa4ee082e, 52 },
    { 0xf7ae082e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619e0834, 46 },
    { 0, 0 },
    { 0x362e0836, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4e083a, 61 },
    { 0x979e083b, 34 },
    { 0, 0 },
    { 0xde1e083d, 90 },
    { 0, 0 },
    { 0xd38e083f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156e0845, 90 },
    { 0x330e0846, 93 },
    { 0xf0fe0846, 55 },
    { 0xfa7e0848, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e0851, 23 },
    { 0xad7e0852, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36e0858, 33 },
    { 0, 0 },
    { 0xc9ce085a, 43 },
    { 0x9f8e085b, 23 },
    { 0x44ee085b, 61 },
    { 0x817e085d, 61 },
    { 0xbbe085d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7e0862, 61 },
    { 0, 0 },
    { 0x2b8e0864, 77 },
    { 0x379e0865, 61 },
    { 0, 0 },
    { 0x9e6e0867, 43 },
    { 0, 0 },
    { 0x202e0869, 69 },
    { 0xf3ee086a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e0872, 98 },
    { 0x9e7e0872, 90 },
    { 0xd60e0872, 23 },
    { 0, 0 },
    { 0xdfce0876, 77 },
    { 0xbbfe0877, 33 },
    { 0x132e0878, 98 },
    { 0x1b9e0876, 33 },
    { 0x40ee087a, 78 },
    { 0x151e087b, 23 },
    { 0x8ee087b, 90 },
    { 0x41ae087d, 36 },
    { 0, 0 },
    { 0x624e087f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e0885, 95 },
    { 0, 0 },
    { 0x67ce0887, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69e088a, 90 },
    { 0, 0 },
    { 0xd07e088c, 90 },
    { 0x372e088d, 99 },
    { 0xf2fe088e, 95 },
    { 0xf6fe088e, 34 },
    { 0, 0 },
    { 0x14ee0891, 55 },
    { 0x46fe0892, 55 },
    { 0xf8de0893, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e0896, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977e089a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47be089e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7e08a3, 61 },
    { 0, 0 },
    { 0xe27e08a5, 66 },
    { 0, 0 },
    { 0xee9e08a7, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe08ad, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1e08b0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4e08b3, 66 },
    { 0xe62e08b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254e08bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4e08c0, 84 },
    { 0, 0 },
    { 0xc21e08c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ace08c8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e08ce, 87 },
    { 0x366e08ce, 83 },
    { 0x587e08cf, 23 },
    { 0xf6ee08d1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x187e08d4, 34 },
    { 0, 0 },
    { 0x594e08d6, 33 },
    { 0x448e08d6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4e08dc, 77 },
    { 0x754e08dd, 93 },
    { 0, 0 },
    { 0xae0e08df, 33 },
    { 0, 0 },
    { 0xee7e08e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7de08e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1e08e8, 87 },
    { 0x588e08e9, 61 },
    { 0x757e08ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35e08f3, 33 },
    { 0x9b2e08f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x794e08f7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e08fb, 61 },
    { 0, 0 },
    { 0x28e08fd, 66 },
    { 0x3a2e08fd, 33 },
    { 0x977e08ff, 90 },
    { 0x65ce0900, 90 },
    { 0x76ee08fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329e0907, 90 },
    { 0, 0 },
    { 0xed5e0909, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fae090d, 33 },
    { 0x7e9e090e, 23 },
    { 0xe67e090f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe0912, 55 },
    { 0, 0 },
    { 0xebe0914, 28 },
    { 0, 0 },
    { 0x21de0916, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xace0919, 43 },
    { 0xb9ee0919, 33 },
    { 0x100e0919, 76 },
    { 0xa01e091c, 61 },
    { 0xf0be091d, 95 },
    { 0x577e091e, 23 },
    { 0x311e091f, 34 },
    { 0xeb9e0920, 90 },
    { 0x70fe0921, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e0925, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7e092b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e092f, 98 },
    { 0x208e0930, 93 },
    { 0xbf6e0931, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde0937, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8be093a, 100 },
    { 0x8dce093b, 66 },
    { 0xa38e093c, 95 },
    { 0x332e093d, 90 },
    { 0xc83e093e, 55 },
    { 0x93e093f, 61 },
    { 0xcc2e0940, 23 },
    { 0x4b9e0941, 98 },
    { 0xbe7e0941, 33 },
    { 0, 0 },
    { 0xdb7e0944, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dee094d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fe0951, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeae0957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3e095d, 95 },
    { 0, 0 },
    { 0x174e095f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e0966, 81 },
    { 0x320e0967, 33 },
    { 0x533e0968, 33 },
    { 0, 0 },
    { 0xfa2e096a, 66 },
    { 0, 0 },
    { 0x549e096c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96be0970, 61 },
    { 0, 0 },
    { 0xb24e0972, 34 },
    { 0, 0 },
    { 0x653e0974, 66 },
    { 0xe11e0974, 33 },
    { 0, 0 },
    { 0xa2be0977, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ee097a, 33 },
    { 0x99be097b, 98 },
    { 0, 0 },
    { 0xd59e097d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243e0981, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbe0989, 99 },
    { 0x747e0989, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1e0991, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4e0995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ae099c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce09a0, 55 },
    { 0x617e09a0, 83 },
    { 0x2ade09a2, 33 },
    { 0x97be09a3, 95 },
    { 0xb94e09a3, 46 },
    { 0xf37e09a2, 76 },
    { 0x557e09a5, 78 },
    { 0xb3ee09a7, 84 },
    { 0x63ee09a8, 34 },
    { 0x449e09a9, 33 },
    { 0, 0 },
    { 0x7dde09ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38e09af, 98 },
    { 0xf20e09af, 78 },
    { 0x645e09b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9be09b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ae09bb, 98 },
    { 0xcb6e09bc, 46 },
    { 0x4aee09bd, 98 },
    { 0x469e09bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ee09c2, 55 },
    { 0xecae09c2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8e09c8, 90 },
    { 0x20fe09c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ae09d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645e09d6, 95 },
    { 0, 0 },
    { 0x346e09d8, 34 },
    { 0x23be09d9, 23 },
    { 0, 0 },
    { 0x263e09db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869e09e3, 95 },
    { 0, 0 },
    { 0x158e09e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63e09e8, 55 },
    { 0x56be09e9, 23 },
    { 0x3d8e09ea, 66 },
    { 0xdae09eb, 91 },
    { 0x27ee09ea, 96 },
    { 0xa12e09ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5e09f2, 95 },
    { 0xdf0e09f3, 90 },
    { 0x8b2e09f2, 52 },
    { 0xed8e09f3, 95 },
    { 0x481e09f6, 90 },
    { 0, 0 },
    { 0xbbce09f8, 36 },
    { 0x1e2e09f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae09fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772e0a03, 98 },
    { 0x1bbe0a04, 98 },
    { 0xefae0a03, 98 },
    { 0x5b9e0a04, 84 },
    { 0x6c0e0a03, 93 },
    { 0, 0 },
    { 0x212e0a09, 98 },
    { 0x9efe0a0a, 61 },
    { 0, 0 },
    { 0xfe4e0a0c, 90 },
    { 0x425e0a0d, 55 },
    { 0xea9e0a0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fe0a12, 43 },
    { 0x7b4e0a13, 98 },
    { 0x9ede0a14, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8de0a19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08e0a1c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6e0a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x690e0a24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x957e0a27, 98 },
    { 0x711e0a28, 95 },
    { 0x748e0a28, 93 },
    { 0xab0e0a28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbe0a2d, 85 },
    { 0x3b3e0a2e, 98 },
    { 0xc8ae0a2f, 93 },
    { 0xaf4e0a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfe0a36, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92e0a39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace0a3d, 23 },
    { 0xa6ae0a3e, 52 },
    { 0x395e0a3e, 46 },
    { 0x52ae0a40, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ae0a43, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae0a46, 99 },
    { 0xeace0a47, 75 },
    { 0xf9de0a48, 61 },
    { 0x4d3e0a49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90e0a4c, 33 },
    { 0x431e0a4d, 61 },
    { 0x177e0a4e, 33 },
    { 0xbb1e0a4e, 55 },
    { 0xa45e0a4e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e0e0a56, 87 },
    { 0xe64e0a56, 90 },
    { 0x364e0a56, 61 },
    { 0, 0 },
    { 0xc64e0a5a, 34 },
    { 0xa3be0a5b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x141e0a5e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be0a62, 46 },
    { 0xae7e0a63, 43 },
    { 0, 0 },
    { 0x34ae0a65, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e0a69, 61 },
    { 0, 0 },
    { 0xccce0a6b, 55 },
    { 0xa0ee0a6c, 23 },
    { 0x263e0a6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577e0a71, 90 },
    { 0, 0 },
    { 0x353e0a73, 81 },
    { 0xd8ae0a74, 34 },
    { 0, 0 },
    { 0xee0a76, 55 },
    { 0xf82e0a77, 33 },
    { 0x2de0a78, 76 },
    { 0x955e0a79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466e0a7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4e0a81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce0a85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3be0a88, 90 },
    { 0x34de0a88, 95 },
    { 0x66fe0a88, 23 },
    { 0xff4e0a8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1e0a92, 33 },
    { 0xdcee0a92, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1e0a96, 99 },
    { 0, 0 },
    { 0x979e0a98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ce0a9d, 90 },
    { 0, 0 },
    { 0x344e0a9f, 98 },
    { 0, 0 },
    { 0x735e0aa1, 94 },
    { 0x73de0aa2, 98 },
    { 0, 0 },
    { 0xeb4e0aa4, 61 },
    { 0x88ee0aa5, 90 },
    { 0x1dbe0aa6, 33 },
    { 0x2fbe0aa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbe0aaa, 64 },
    { 0x5a1e0aaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622e0ab1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fce0ab5, 61 },
    { 0x5c9e0ab5, 93 },
    { 0x251e0ab5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0e0abb, 66 },
    { 0xe6fe0abc, 33 },
    { 0xf68e0abb, 33 },
    { 0, 0 },
    { 0x46de0abf, 61 },
    { 0, 0 },
    { 0x33e0ac1, 34 },
    { 0xb2e0ac1, 33 },
    { 0xe2fe0ac3, 90 },
    { 0x85ce0ac3, 91 },
    { 0x484e0ac3, 85 },
    { 0xb81e0ac6, 90 },
    { 0, 0 },
    { 0xc7be0ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ce0acb, 69 },
    { 0xfabe0acb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ee0ad3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444e0adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2e0adf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8e0ae4, 33 },
    { 0x9e4e0ae4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74be0ae8, 81 },
    { 0, 0 },
    { 0xbe6e0aea, 34 },
    { 0x5cce0aeb, 90 },
    { 0x86ae0aec, 61 },
    { 0x3f8e0aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e0af2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4e0af7, 90 },
    { 0x81e0af8, 34 },
    { 0x823e0af8, 33 },
    { 0xbd3e0afa, 85 },
    { 0x7d5e0af7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71e0b00, 95 },
    { 0x164e0b00, 23 },
    { 0x43be0b01, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ae0b06, 78 },
    { 0x7b8e0b06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e0b14, 88 },
    { 0xecbe0b14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8e0b1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ce0b1f, 55 },
    { 0, 0 },
    { 0x718e0b21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813e0b2b, 90 },
    { 0, 0 },
    { 0xae3e0b2d, 90 },
    { 0x1fde0b2e, 98 },
    { 0x544e0b2f, 23 },
    { 0x227e0b30, 61 },
    { 0x496e0b31, 61 },
    { 0x9bfe0b31, 61 },
    { 0xd73e0b33, 66 },
    { 0x70be0b34, 95 },
    { 0x66ae0b34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e0b39, 91 },
    { 0x1d2e0b3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee0b3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e0b40, 100 },
    { 0xbd7e0b41, 23 },
    { 0, 0 },
    { 0xfeae0b43, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eae0b47, 93 },
    { 0xa49e0b48, 98 },
    { 0xf6be0b49, 60 },
    { 0xcd9e0b4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835e0b4f, 95 },
    { 0, 0 },
    { 0x270e0b51, 55 },
    { 0x3e9e0b52, 61 },
    { 0, 0 },
    { 0x947e0b54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68be0b58, 75 },
    { 0x3c6e0b59, 78 },
    { 0xf21e0b5a, 91 },
    { 0xb53e0b59, 23 },
    { 0xca8e0b5a, 33 },
    { 0xfeae0b5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebe0b62, 33 },
    { 0xb36e0b62, 43 },
    { 0x3ce0b64, 81 },
    { 0x6fde0b65, 34 },
    { 0, 0 },
    { 0xf39e0b67, 87 },
    { 0x82fe0b67, 46 },
    { 0x53e0b67, 33 },
    { 0x952e0b67, 55 },
    { 0, 0 },
    { 0x26ce0b6c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615e0b72, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfe0b7a, 90 },
    { 0xcf7e0b7b, 95 },
    { 0xc8ae0b7c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ae0b81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe0b85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48e0b88, 98 },
    { 0, 0 },
    { 0xe35e0b8a, 98 },
    { 0, 0 },
    { 0xfbfe0b8c, 84 },
    { 0x7d0e0b8d, 55 },
    { 0xcf6e0b8e, 52 },
    { 0, 0 },
    { 0xefee0b90, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde0b97, 55 },
    { 0, 0 },
    { 0xc8ae0b99, 95 },
    { 0x2fe0b9a, 77 },
    { 0x9fe0b9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e0ba3, 23 },
    { 0x75ee0ba3, 98 },
    { 0x8eee0ba5, 93 },
    { 0xaf8e0ba6, 98 },
    { 0x854e0ba6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffee0bb1, 87 },
    { 0, 0 },
    { 0xc8ce0bb3, 78 },
    { 0xf64e0bb4, 61 },
    { 0xc1ee0bb5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8e0bc2, 90 },
    { 0x43fe0bc3, 67 },
    { 0, 0 },
    { 0xa05e0bc5, 76 },
    { 0, 0 },
    { 0xbd2e0bc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6e0bca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1e0bcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2e0bd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cde0bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fe0bd9, 61 },
    { 0x8d4e0bda, 33 },
    { 0, 0 },
    { 0x20ce0bdc, 87 },
    { 0xef0e0bdd, 95 },
    { 0x2f8e0bde, 33 },
    { 0x14e0bdf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7e0be5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0be0be9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffe0bec, 46 },
    { 0x7dee0bed, 55 },
    { 0x176e0bec, 53 },
    { 0, 0 },
    { 0xc8ae0bf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x786e0bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556e0bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4e0bfa, 61 },
    { 0x336e0bfa, 84 },
    { 0, 0 },
    { 0xf20e0bfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e0c01, 33 },
    { 0, 0 },
    { 0x608e0c03, 76 },
    { 0x7bde0c03, 90 },
    { 0, 0 },
    { 0x1d5e0c06, 76 },
    { 0x76be0c06, 98 },
    { 0x90ee0c07, 34 },
    { 0x98ce0c09, 86 },
    { 0x603e0c0a, 69 },
    { 0xcf9e0c09, 85 },
    { 0xcfae0c0c, 43 },
    { 0xe74e0c0c, 90 },
    { 0x5dae0c0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6be0c16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfe0c1d, 93 },
    { 0x430e0c1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0c21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x486e0c24, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6e0c28, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ee0c2e, 94 },
    { 0xa9be0c2e, 33 },
    { 0, 0 },
    { 0xdd8e0c31, 98 },
    { 0, 0 },
    { 0x1c6e0c33, 91 },
    { 0xf76e0c34, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e0c38, 33 },
    { 0, 0 },
    { 0xe8de0c3a, 90 },
    { 0x55ae0c3b, 61 },
    { 0xd5ae0c3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8e0c3f, 55 },
    { 0x7f5e0c40, 95 },
    { 0, 0 },
    { 0x83fe0c42, 61 },
    { 0x2dee0c43, 33 },
    { 0x683e0c44, 95 },
    { 0x7a8e0c45, 90 },
    { 0xbfee0c46, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4e0c49, 90 },
    { 0x95ee0c4a, 93 },
    { 0x87ae0c4a, 98 },
    { 0x7c8e0c4c, 98 },
    { 0xc9ee0c4d, 33 },
    { 0x981e0c4d, 55 },
    { 0xedbe0c4f, 87 },
    { 0x700e0c4f, 90 },
    { 0x349e0c4f, 90 },
    { 0xa90e0c4f, 23 },
    { 0x35be0c4d, 61 },
    { 0x4b8e0c54, 52 },
    { 0x8c8e0c55, 43 },
    { 0x32ee0c55, 23 },
    { 0, 0 },
    { 0xce8e0c58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5e0c5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x849e0c61, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x869e0c64, 78 },
    { 0x87ce0c65, 43 },
    { 0xaf0e0c65, 61 },
    { 0xe3be0c67, 23 },
    { 0, 0 },
    { 0xb4ce0c69, 98 },
    { 0x775e0c6a, 40 },
    { 0x6ee0c69, 90 },
    { 0x79fe0c6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaee0c6f, 95 },
    { 0x6f8e0c70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608e0c74, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce0c78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12e0c7b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde0c7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x668e0c81, 17 },
    { 0x97de0c82, 23 },
    { 0x764e0c83, 46 },
    { 0x5b9e0c83, 23 },
    { 0x7cae0c84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73e0c89, 87 },
    { 0x609e0c8a, 95 },
    { 0x307e0c8b, 98 },
    { 0, 0 },
    { 0xbfbe0c8d, 98 },
    { 0, 0 },
    { 0x60ae0c8f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x313e0c92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ce0c98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3e0c9b, 50 },
    { 0, 0 },
    { 0x54de0c9d, 76 },
    { 0, 0 },
    { 0xa03e0c9f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x381e0ca2, 95 },
    { 0x394e0ca3, 100 },
    { 0xd37e0ca4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ce0ca7, 23 },
    { 0x440e0ca7, 90 },
    { 0xf45e0ca7, 61 },
    { 0x714e0caa, 52 },
    { 0x340e0cab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ce0cb2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558e0cb9, 78 },
    { 0, 0 },
    { 0x3cfe0cbb, 95 },
    { 0, 0 },
    { 0xbeae0cbd, 95 },
    { 0, 0 },
    { 0xad4e0cbf, 55 },
    { 0x6a4e0cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fe0cc4, 95 },
    { 0x12ee0cc5, 95 },
    { 0xba6e0cc6, 95 },
    { 0xfe8e0cc7, 98 },
    { 0xd8ae0cc8, 91 },
    { 0xd03e0cc8, 68 },
    { 0xca8e0cca, 76 },
    { 0x95fe0cc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae0cd8, 90 },
    { 0x568e0cd9, 90 },
    { 0x9ee0cda, 61 },
    { 0x9d9e0cda, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e0cde, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e0ce6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ae0ce9, 99 },
    { 0x92ee0cea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1e0ced, 90 },
    { 0xb2ee0cee, 95 },
    { 0xfb4e0cef, 33 },
    { 0xc3ee0cf0, 66 },
    { 0, 0 },
    { 0x5c4e0cf2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07e0cf6, 87 },
    { 0, 0 },
    { 0xf21e0cf8, 23 },
    { 0x7dae0cf9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e0cfd, 98 },
    { 0x699e0cfe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e0d01, 34 },
    { 0xd69e0d02, 61 },
    { 0, 0 },
    { 0xc50e0d04, 64 },
    { 0, 0 },
    { 0x8c3e0d06, 95 },
    { 0x3ace0d07, 43 },
    { 0xa2fe0d07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e0d0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccde0d13, 33 },
    { 0xddee0d14, 90 },
    { 0x2c6e0d15, 61 },
    { 0x44be0d16, 55 },
    { 0x1a9e0d16, 34 },
    { 0, 0 },
    { 0xb14e0d19, 78 },
    { 0xc0be0d19, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3e0d1d, 33 },
    { 0xffde0d1e, 33 },
    { 0x45ce0d1f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e0d22, 95 },
    { 0xbdbe0d23, 84 },
    { 0, 0 },
    { 0xe5fe0d25, 90 },
    { 0x520e0d26, 100 },
    { 0x42ce0d27, 61 },
    { 0xd70e0d28, 78 },
    { 0xeb2e0d29, 76 },
    { 0x1b4e0d26, 61 },
    { 0x7ede0d2b, 61 },
    { 0, 0 },
    { 0xa8de0d2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605e0d31, 33 },
    { 0, 0 },
    { 0xbe0d33, 93 },
    { 0, 0 },
    { 0xe35e0d35, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de0d3c, 98 },
    { 0x4b6e0d3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e0d44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe0d48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04e0d4b, 76 },
    { 0x4b4e0d4c, 61 },
    { 0xd7ae0d4d, 61 },
    { 0, 0 },
    { 0x8fe0d4f, 85 },
    { 0xc03e0d4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61e0d53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x453e0d56, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4be0d5a, 61 },
    { 0, 0 },
    { 0xc3ae0d5c, 90 },
    { 0x18ce0d5d, 43 },
    { 0xa16e0d5c, 33 },
    { 0x1dfe0d5f, 23 },
    { 0x34ce0d60, 85 },
    { 0x51ce0d61, 91 },
    { 0xbdde0d61, 33 },
    { 0, 0 },
    { 0xd4ae0d64, 95 },
    { 0xdc6e0d65, 17 },
    { 0x608e0d66, 84 },
    { 0, 0 },
    { 0x5a8e0d68, 24 },
    { 0xed1e0d69, 93 },
    { 0, 0 },
    { 0x507e0d6b, 34 },
    { 0xb65e0d6c, 99 },
    { 0x4ebe0d6c, 84 },
    { 0x1c7e0d6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ee0d72, 99 },
    { 0, 0 },
    { 0x51ee0d74, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35e0d78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18de0d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8e0d80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aae0d85, 90 },
    { 0, 0 },
    { 0x9c0e0d87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ae0d8a, 69 },
    { 0, 0 },
    { 0x794e0d8c, 98 },
    { 0x937e0d8d, 43 },
    { 0x56ae0d8e, 90 },
    { 0x36ce0d8d, 90 },
    { 0xc71e0d8f, 43 },
    { 0x8f4e0d8f, 61 },
    { 0x7f3e0d92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfce0d96, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e0d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e0d9e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3e0da7, 99 },
    { 0x6a6e0da7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e0dad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397e0db2, 90 },
    { 0xca8e0db3, 33 },
    { 0xf7e0db3, 87 },
    { 0xe4ae0db5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e0dba, 43 },
    { 0xfcae0dba, 34 },
    { 0, 0 },
    { 0x952e0dbd, 33 },
    { 0x927e0dbd, 90 },
    { 0xccde0dbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fce0dc4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46e0dcd, 93 },
    { 0x15fe0dce, 84 },
    { 0x2a2e0dce, 36 },
    { 0xfb0e0dd0, 90 },
    { 0x132e0dcd, 76 },
    { 0x35de0dd2, 76 },
    { 0, 0 },
    { 0xd88e0dd4, 87 },
    { 0x7f3e0dd4, 95 },
    { 0xee4e0dd6, 34 },
    { 0x54fe0dd5, 90 },
    { 0x282e0dd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be0ddd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e0de4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18e0de7, 75 },
    { 0xbcee0de8, 87 },
    { 0xa6e0de7, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424e0df0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307e0df7, 93 },
    { 0, 0 },
    { 0xe6e0df9, 95 },
    { 0, 0 },
    { 0x93e0dfb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85e0dff, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9e0e03, 84 },
    { 0xd7ae0e03, 33 },
    { 0x68e0e05, 23 },
    { 0x885e0e03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e0e09, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fce0e0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35e0e16, 69 },
    { 0x103e0e17, 90 },
    { 0xf00e0e17, 90 },
    { 0x15ae0e19, 33 },
    { 0x7d6e0e19, 55 },
    { 0xb8ae0e1b, 61 },
    { 0xb1ce0e1c, 34 },
    { 0xc6be0e1a, 90 },
    { 0x6b5e0e1e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0be0e22, 36 },
    { 0xe69e0e22, 33 },
    { 0, 0 },
    { 0xff7e0e25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1e0e28, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee0e32, 33 },
    { 0x78be0e32, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe0e3d, 33 },
    { 0x24de0e3e, 90 },
    { 0, 0 },
    { 0xec0e0e40, 36 },
    { 0, 0 },
    { 0xd95e0e42, 98 },
    { 0x229e0e42, 61 },
    { 0x3b3e0e44, 43 },
    { 0x166e0e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ee0e49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1e0e4f, 55 },
    { 0, 0 },
    { 0x4a3e0e51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28e0e55, 95 },
    { 0x54ae0e55, 23 },
    { 0xc12e0e57, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe0e5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe0e5e, 93 },
    { 0x41ce0e5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ae0e62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74e0e66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0e6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8be0e73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ae0e76, 90 },
    { 0, 0 },
    { 0xfc3e0e78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689e0e7e, 98 },
    { 0, 0 },
    { 0xb52e0e80, 61 },
    { 0x4c4e0e80, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae0e88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7e0e8d, 66 },
    { 0xb2e0e8d, 76 },
    { 0x940e0e8f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cde0e94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4e0e98, 34 },
    { 0xe81e0e98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ee0e9c, 98 },
    { 0x360e0e9d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1e0ea0, 43 },
    { 0xb1ce0ea1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e0ea5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ae0eaa, 78 },
    { 0xaade0eab, 90 },
    { 0xfd3e0eac, 23 },
    { 0x84be0eac, 61 },
    { 0x9d2e0ead, 91 },
    { 0xfd9e0eaf, 61 },
    { 0x92ae0eb0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ae0eb3, 78 },
    { 0x6afe0eb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60e0eb7, 43 },
    { 0xdfae0eb8, 61 },
    { 0x979e0eb9, 90 },
    { 0x2ede0eba, 52 },
    { 0, 0 },
    { 0x3d1e0ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3e0ebf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81e0ec4, 68 },
    { 0, 0 },
    { 0xce2e0ec6, 95 },
    { 0xbb5e0ec7, 90 },
    { 0, 0 },
    { 0x5f4e0ec9, 90 },
    { 0x800e0eca, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ee0ece, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732e0ed4, 77 },
    { 0x96ce0ed4, 98 },
    { 0, 0 },
    { 0x1fe0ed7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57e0eda, 34 },
    { 0x648e0edb, 61 },
    { 0, 0 },
    { 0x5ce0edd, 61 },
    { 0x357e0ede, 68 },
    { 0, 0 },
    { 0x7b7e0ee0, 98 },
    { 0x643e0ee1, 85 },
    { 0, 0 },
    { 0x882e0ee3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de0eeb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x682e0ef1, 66 },
    { 0x86de0ef2, 33 },
    { 0x385e0ef2, 95 },
    { 0x943e0ef4, 33 },
    { 0x545e0ef4, 95 },
    { 0x612e0ef4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe0efc, 23 },
    { 0x3d1e0efc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ce0f01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06e0f09, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x49be0f0c, 90 },
    { 0xd45e0f0c, 61 },
    { 0, 0 },
    { 0xae3e0f0f, 98 },
    { 0xd54e0f0f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ee0f13, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20e0f18, 76 },
    { 0, 0 },
    { 0x117e0f1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe0f1e, 90 },
    { 0x5e8e0f1e, 23 },
    { 0xd73e0f20, 33 },
    { 0, 0 },
    { 0x553e0f22, 33 },
    { 0xcede0f22, 98 },
    { 0x408e0f23, 34 },
    { 0x570e0f25, 34 },
    { 0, 0 },
    { 0xd78e0f27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebe0f2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ae0f30, 88 },
    { 0xeefe0f31, 69 },
    { 0, 0 },
    { 0xe35e0f33, 77 },
    { 0, 0 },
    { 0xcbee0f35, 90 },
    { 0, 0 },
    { 0xfede0f37, 33 },
    { 0xebee0f38, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efe0f3b, 98 },
    { 0, 0 },
    { 0x438e0f3d, 78 },
    { 0x47ce0f3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394e0f42, 90 },
    { 0xdb6e0f42, 68 },
    { 0x6fce0f44, 98 },
    { 0x597e0f42, 90 },
    { 0xe8e0f45, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e0f4c, 23 },
    { 0xb7fe0f4d, 34 },
    { 0, 0 },
    { 0xc10e0f4f, 98 },
    { 0, 0 },
    { 0xc05e0f51, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02e0f56, 33 },
    { 0x1b1e0f56, 85 },
    { 0x2c2e0f57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e0f5b, 33 },
    { 0x2b7e0f5b, 93 },
    { 0x999e0f5b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce0f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3e0f67, 95 },
    { 0x7c8e0f67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbede0f71, 93 },
    { 0xef0e0f71, 67 },
    { 0, 0 },
    { 0x1b6e0f74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7e0f80, 61 },
    { 0x99e0f80, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103e0f8d, 90 },
    { 0x844e0f8e, 46 },
    { 0x420e0f8f, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9e0f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a9e0f9c, 98 },
    { 0, 0 },
    { 0xebce0f9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9e0faa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9be0fb0, 97 },
    { 0xe29e0fb0, 98 },
    { 0x4aee0fb1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ee0fb5, 98 },
    { 0x18be0fb5, 76 },
    { 0xa66e0fb7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280e0fc2, 60 },
    { 0xfe9e0fc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81e0fca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fe0fcd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e0fd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x177e0fd5, 33 },
    { 0x9e0e0fd5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39e0fda, 33 },
    { 0, 0 },
    { 0xd5ee0fdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6e0fe2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741e0fe8, 95 },
    { 0xcabe0fe9, 33 },
    { 0, 0 },
    { 0x39ce0feb, 35 },
    { 0x348e0feb, 33 },
    { 0x2c3e0fec, 43 },
    { 0, 0 },
    { 0xc95e0fef, 43 },
    { 0, 0 },
    { 0x934e0ff1, 61 },
    { 0xc9fe0ff2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12e0ff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x986e0ff8, 67 },
    { 0x999e0ff9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84e0fff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20e1002, 90 },
    { 0, 0 },
    { 0xfd5e1004, 75 },
    { 0, 0 },
    { 0xa9fe1006, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74e1009, 100 },
    { 0x279e1009, 71 },
    { 0x292e100b, 61 },
    { 0xfa3e100b, 95 },
    { 0x150e100d, 55 },
    { 0x1d2e100e, 93 },
    { 0xf81e100c, 98 },
    { 0xb0de1010, 98 },
    { 0xa15e1010, 81 },
    { 0, 0 },
    { 0x223e1013, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511e1017, 98 },
    { 0, 0 },
    { 0x485e1019, 77 },
    { 0x6aee101a, 98 },
    { 0x240e101b, 85 },
    { 0x848e1019, 75 },
    { 0, 0 },
    { 0xdcfe101e, 38 },
    { 0xe38e101f, 98 },
    { 0x748e1020, 23 },
    { 0x461e101e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520e1025, 98 },
    { 0x42de1025, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fe102c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6e1030, 90 },
    { 0xd3de1031, 95 },
    { 0, 0 },
    { 0x650e1033, 76 },
    { 0xcaae1034, 61 },
    { 0x4ece1035, 99 },
    { 0x9e2e1036, 95 },
    { 0x9a6e1037, 98 },
    { 0x2f5e1038, 35 },
    { 0xe5ce1037, 90 },
    { 0xee0e1036, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581e1044, 23 },
    { 0x52be1044, 43 },
    { 0x3bde1044, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e104e, 33 },
    { 0x735e104e, 61 },
    { 0, 0 },
    { 0x1ae1051, 55 },
    { 0, 0 },
    { 0xa02e1053, 95 },
    { 0xafe1053, 34 },
    { 0x90de1054, 23 },
    { 0x4d9e1056, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7e105c, 33 },
    { 0x114e105d, 88 },
    { 0xa62e105e, 90 },
    { 0xd70e105d, 61 },
    { 0xd10e1060, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e1065, 66 },
    { 0x42de1066, 61 },
    { 0x77ce1067, 76 },
    { 0x904e1065, 46 },
    { 0xfc7e1066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e1070, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x897e1073, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82e1076, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0e107c, 77 },
    { 0x8b7e107c, 23 },
    { 0x777e107e, 99 },
    { 0xc53e107f, 90 },
    { 0xbc6e1080, 98 },
    { 0, 0 },
    { 0xfb2e1082, 33 },
    { 0xe43e1082, 34 },
    { 0x5cae1084, 76 },
    { 0xa0fe1085, 78 },
    { 0x78fe1086, 61 },
    { 0xbc7e1083, 36 },
    { 0x739e1088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4e108c, 75 },
    { 0x5fe108d, 76 },
    { 0, 0 },
    { 0x9fae108f, 95 },
    { 0x3d1e1090, 61 },
    { 0, 0 },
    { 0xb7e1092, 55 },
    { 0x711e1093, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfe1099, 90 },
    { 0x2b3e109a, 61 },
    { 0x405e109b, 98 },
    { 0x731e109b, 91 },
    { 0xa5de109a, 76 },
    { 0x3ede109b, 61 },
    { 0xbf6e109e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67e10a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66e10a9, 46 },
    { 0x85ae10aa, 34 },
    { 0x64de10aa, 98 },
    { 0, 0 },
    { 0x651e10ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4e10b5, 66 },
    { 0xf80e10b6, 90 },
    { 0x904e10b7, 100 },
    { 0xbb5e10b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x121e10bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497e10c4, 90 },
    { 0, 0 },
    { 0x4a6e10c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fe10c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7e10cc, 33 },
    { 0x82be10cc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ce10d3, 67 },
    { 0, 0 },
    { 0xbade10d5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e10db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583e10e2, 46 },
    { 0, 0 },
    { 0x35be10e4, 90 },
    { 0xcd4e10e5, 95 },
    { 0x684e10e6, 61 },
    { 0xd0de10e7, 81 },
    { 0x67fe10e8, 98 },
    { 0x11e10e9, 95 },
    { 0xa25e10e9, 90 },
    { 0xf34e10e9, 68 },
    { 0x963e10eb, 76 },
    { 0x770e10ed, 42 },
    { 0xfdce10ee, 33 },
    { 0xf16e10e5, 43 },
    { 0x76be10f0, 61 },
    { 0x4d4e10f0, 34 },
    { 0xb2be10f2, 66 },
    { 0x495e10ee, 33 },
    { 0x7aae10f2, 61 },
    { 0x7c9e10f5, 61 },
    { 0xf2e10f6, 33 },
    { 0x29ae10f6, 33 },
    { 0, 0 },
    { 0x2e1e10f9, 33 },
    { 0xda4e10fa, 95 },
    { 0, 0 },
    { 0x64de10fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fae10ff, 33 },
    { 0, 0 },
    { 0x12ce1101, 61 },
    { 0, 0 },
    { 0x5d2e1103, 33 },
    { 0x1a2e1103, 95 },
    { 0, 0 },
    { 0x1e9e1106, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60e110d, 33 },
    { 0x3ede110d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865e1118, 96 },
    { 0x941e1118, 90 },
    { 0x7c7e1119, 34 },
    { 0xa77e1118, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae1123, 90 },
    { 0xd75e1124, 93 },
    { 0x6f2e1124, 55 },
    { 0x623e1126, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de1129, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72e112f, 95 },
    { 0, 0 },
    { 0x27ae1131, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aae1138, 76 },
    { 0xc9ee1138, 95 },
    { 0xe84e113a, 55 },
    { 0x75ee1138, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fe113e, 23 },
    { 0xec6e113f, 61 },
    { 0x53ce113f, 86 },
    { 0xc3ee1141, 23 },
    { 0x6fe1140, 33 },
    { 0, 0 },
    { 0x767e1144, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ce1149, 75 },
    { 0xc58e114a, 43 },
    { 0xc54e114a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde114f, 55 },
    { 0xcabe1150, 61 },
    { 0x386e114f, 90 },
    { 0xfb9e1152, 33 },
    { 0xf10e1153, 64 },
    { 0, 0 },
    { 0x761e1155, 91 },
    { 0xb07e1156, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664e115b, 52 },
    { 0xd21e115b, 34 },
    { 0x5f6e115c, 61 },
    { 0, 0 },
    { 0xe79e115f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42e1163, 33 },
    { 0xd2e1163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e1168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56e116c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174e1171, 43 },
    { 0x523e1172, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e1179, 90 },
    { 0xef5e117a, 91 },
    { 0xb89e117a, 76 },
    { 0x18ee117c, 90 },
    { 0xf7fe117c, 23 },
    { 0x74de117a, 55 },
    { 0x17ee117f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fce118e, 61 },
    { 0x59be118f, 90 },
    { 0xf0fe118e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee119a, 93 },
    { 0xa99e119b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe119e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7be11a2, 90 },
    { 0xcc4e11a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e11a7, 95 },
    { 0x138e11a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeae11ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72e11b1, 23 },
    { 0, 0 },
    { 0xff4e11b3, 98 },
    { 0x125e11b3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68de11b8, 33 },
    { 0x64ee11b9, 75 },
    { 0xac2e11ba, 61 },
    { 0xe53e11ba, 95 },
    { 0x8d5e11bc, 12 },
    { 0, 0 },
    { 0xc7fe11be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x585e11c1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992e11c6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391e11cb, 23 },
    { 0, 0 },
    { 0xbcfe11cd, 64 },
    { 0x188e11ce, 76 },
    { 0xca9e11ce, 90 },
    { 0xb4ce11d0, 55 },
    { 0, 0 },
    { 0x207e11d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73be11d6, 34 },
    { 0xa47e11d6, 93 },
    { 0x308e11d6, 98 },
    { 0xb81e11d7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116e11de, 95 },
    { 0xbdce11df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35e11e4, 33 },
    { 0x158e11e5, 33 },
    { 0x2fce11e4, 95 },
    { 0, 0 },
    { 0x98be11e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe11ee, 78 },
    { 0, 0 },
    { 0x249e11f0, 75 },
    { 0xac4e11f0, 90 },
    { 0x84ee11f2, 22 },
    { 0xb2be11f0, 90 },
    { 0, 0 },
    { 0xba4e11f5, 61 },
    { 0x57de11f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728e11fa, 98 },
    { 0, 0 },
    { 0x3c8e11fc, 90 },
    { 0, 0 },
    { 0xcb4e11fe, 55 },
    { 0xedde11ff, 90 },
    { 0xef3e11ff, 61 },
    { 0, 0 },
    { 0x10ce1202, 66 },
    { 0, 0 },
    { 0x230e1204, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cce1207, 43 },
    { 0x43ee1207, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23e121c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e121f, 76 },
    { 0, 0 },
    { 0xee0e1221, 90 },
    { 0, 0 },
    { 0x297e1223, 65 },
    { 0xc43e1224, 95 },
    { 0, 0 },
    { 0xde8e1226, 43 },
    { 0, 0 },
    { 0xc26e1228, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x143e122b, 65 },
    { 0x6e5e122c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17e122f, 68 },
    { 0, 0 },
    { 0x25de1231, 95 },
    { 0, 0 },
    { 0x2eae1233, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ee1240, 88 },
    { 0xe66e1240, 84 },
    { 0x5b1e1242, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fe1248, 90 },
    { 0xb21e1249, 84 },
    { 0xea9e1248, 61 },
    { 0x5f0e124b, 87 },
    { 0xc53e1248, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ae124f, 43 },
    { 0, 0 },
    { 0xa3ae1251, 23 },
    { 0xbf1e1252, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ce1256, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38e1259, 34 },
    { 0xf31e1259, 33 },
    { 0x33ae1259, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e125f, 98 },
    { 0xc8ae1260, 95 },
    { 0x10ee1261, 34 },
    { 0x1bfe1262, 90 },
    { 0x2c1e125f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73e1268, 23 },
    { 0xcc6e1268, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ae126c, 33 },
    { 0x196e126d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58e1270, 61 },
    { 0xc76e1270, 90 },
    { 0x2fde1272, 23 },
    { 0x682e1272, 95 },
    { 0xdd9e1272, 24 },
    { 0xc8ee1275, 23 },
    { 0, 0 },
    { 0x60ee1277, 23 },
    { 0xf3e1278, 43 },
    { 0, 0 },
    { 0xffde127a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x893e127d, 76 },
    { 0xd72e127e, 34 },
    { 0xfa9e127d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867e1283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ce1286, 40 },
    { 0x45e1287, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be128b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ce1290, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7e1295, 90 },
    { 0xd3ae1296, 95 },
    { 0xf89e1297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8e129a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede129e, 33 },
    { 0x9e6e129f, 98 },
    { 0, 0 },
    { 0x37ee12a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdde12a4, 55 },
    { 0x81ee12a5, 98 },
    { 0xcebe12a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbe12b5, 43 },
    { 0x241e12b6, 23 },
    { 0x19ce12b6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214e12c4, 55 },
    { 0xe18e12c5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2e12c8, 98 },
    { 0x702e12c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71e12cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x482e12cf, 95 },
    { 0xa4ce12d0, 23 },
    { 0x9b3e12d1, 65 },
    { 0x5bce12d2, 90 },
    { 0x6a4e12d3, 87 },
    { 0x4cee12d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee12d9, 90 },
    { 0, 0 },
    { 0xca6e12db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6e12e1, 67 },
    { 0xa13e12e1, 36 },
    { 0x950e12e1, 33 },
    { 0, 0 },
    { 0x7a5e12e5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dae12eb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae12ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce12f6, 99 },
    { 0, 0 },
    { 0x884e12f8, 61 },
    { 0, 0 },
    { 0x761e12fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e12fe, 90 },
    { 0, 0 },
    { 0xe34e1300, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e1306, 23 },
    { 0xbbe1307, 100 },
    { 0x99ce1308, 61 },
    { 0x38ce1309, 90 },
    { 0xc5be130a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e130e, 90 },
    { 0, 0 },
    { 0x650e1310, 32 },
    { 0x51ee1311, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e1314, 95 },
    { 0x626e1314, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x209e1318, 77 },
    { 0xe18e1319, 76 },
    { 0x48fe1319, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9e131d, 55 },
    { 0xf7e131e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7e1323, 33 },
    { 0, 0 },
    { 0xe77e1325, 90 },
    { 0x3c0e1325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fe132c, 90 },
    { 0, 0 },
    { 0x53fe132e, 67 },
    { 0xf0ce132e, 61 },
    { 0xdebe132e, 99 },
    { 0, 0 },
    { 0x3afe1332, 90 },
    { 0x8cee1333, 61 },
    { 0, 0 },
    { 0x183e1335, 68 },
    { 0xc61e1336, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e9e133b, 88 },
    { 0x482e133c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ae133f, 98 },
    { 0xeabe1340, 95 },
    { 0, 0 },
    { 0x4abe1342, 82 },
    { 0x12ee1343, 98 },
    { 0x344e1342, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81de1348, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25e134b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x489e134e, 95 },
    { 0x29e134f, 46 },
    { 0x5fe134f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeee1353, 46 },
    { 0, 0 },
    { 0x53de1355, 93 },
    { 0xf0be1356, 33 },
    { 0xd0ee1357, 90 },
    { 0, 0 },
    { 0xf34e1359, 23 },
    { 0xd69e135a, 84 },
    { 0x87fe135b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17e135f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e1362, 23 },
    { 0xac4e1362, 23 },
    { 0, 0 },
    { 0x52ae1365, 91 },
    { 0x67fe1366, 85 },
    { 0x81ce1366, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cee136e, 56 },
    { 0xb75e136f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7e1372, 33 },
    { 0x937e1373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654e137e, 85 },
    { 0x1fe137f, 81 },
    { 0xcc5e1380, 37 },
    { 0xb84e137f, 33 },
    { 0x108e1382, 96 },
    { 0xc78e1383, 76 },
    { 0x524e1384, 91 },
    { 0xfb8e1385, 95 },
    { 0x157e1382, 46 },
    { 0x939e1381, 93 },
    { 0xb44e1388, 90 },
    { 0xe0e1389, 51 },
    { 0x1dbe138a, 23 },
    { 0xd79e138b, 81 },
    { 0xcce1389, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fe1390, 76 },
    { 0xfa0e1391, 33 },
    { 0xa01e1392, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efe1396, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98e139a, 23 },
    { 0x69de139a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x376e139e, 78 },
    { 0xab0e139f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffe13aa, 34 },
    { 0, 0 },
    { 0x6d2e13ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfe13b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7e13b7, 95 },
    { 0xda1e13b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeae13bc, 23 },
    { 0x57ee13bc, 98 },
    { 0x87e13be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659e13c2, 33 },
    { 0xfb8e13c3, 61 },
    { 0xd7ee13c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fe13c7, 99 },
    { 0xfe5e13c8, 12 },
    { 0x4c6e13c9, 55 },
    { 0, 0 },
    { 0x604e13cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ce13d3, 66 },
    { 0x972e13d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e13d8, 76 },
    { 0xf45e13d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9e13e2, 23 },
    { 0x125e13e2, 90 },
    { 0x584e13e4, 77 },
    { 0x2fee13e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9e13e8, 90 },
    { 0, 0 },
    { 0xf34e13ea, 76 },
    { 0, 0 },
    { 0xdc5e13ec, 61 },
    { 0, 0 },
    { 0x9dde13ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8be13f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6e13f8, 61 },
    { 0x7ece13f8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84e1404, 66 },
    { 0x31ce1405, 87 },
    { 0xe1be1405, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21e1409, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3e140c, 55 },
    { 0x428e140d, 55 },
    { 0x29ae140d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e1414, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7e141a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95e1421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e1428, 55 },
    { 0, 0 },
    { 0x3cce142a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18e1432, 95 },
    { 0x75e1433, 78 },
    { 0xc9ee1434, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x791e1437, 61 },
    { 0x5e4e1438, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ae143d, 95 },
    { 0x839e143d, 33 },
    { 0xe5de143f, 33 },
    { 0x506e143d, 91 },
    { 0x9e0e143d, 17 },
    { 0, 0 },
    { 0x75e1443, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16e1449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9e144c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32e1452, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658e1460, 93 },
    { 0x2c9e1461, 33 },
    { 0x33ae1462, 61 },
    { 0x241e1463, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x887e1466, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e146c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efe1474, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x568e1477, 87 },
    { 0xb94e1477, 23 },
    { 0xd51e1478, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2e147c, 78 },
    { 0x3dde147d, 55 },
    { 0, 0 },
    { 0x642e147f, 68 },
    { 0x65ae1480, 52 },
    { 0, 0 },
    { 0xc95e1482, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8e1486, 33 },
    { 0, 0 },
    { 0x332e1488, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441e1493, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222e1499, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0be149e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ee14a3, 90 },
    { 0x4ce14a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ae14a7, 43 },
    { 0xebfe14a8, 23 },
    { 0xc26e14a9, 75 },
    { 0x55fe14a8, 55 },
    { 0xb67e14ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e14ae, 90 },
    { 0, 0 },
    { 0xb4ce14b0, 95 },
    { 0x71ce14b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4e14b6, 90 },
    { 0x4b4e14b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x290e14bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3e14c2, 23 },
    { 0x60ce14c3, 23 },
    { 0xa58e14c4, 76 },
    { 0xaf8e14c5, 98 },
    { 0xf1ae14c2, 95 },
    { 0x1f4e14c2, 61 },
    { 0x948e14c3, 23 },
    { 0, 0 },
    { 0x4c3e14ca, 61 },
    { 0x1b1e14cb, 98 },
    { 0xa07e14cb, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ce14d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78be14d3, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fe14df, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbe14e8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e14ee, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0x61de14f1, 33 },
    { 0x6fee14f1, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3e14f7, 33 },
    { 0, 0 },
    { 0x4d8e14f9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ce14fd, 33 },
    { 0x687e14fe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bee1503, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fe1508, 90 },
    { 0xdcde1508, 88 },
    { 0x242e1508, 75 },
    { 0x910e150b, 52 },
    { 0x24e150c, 93 },
    { 0x2a8e150c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9e1511, 100 },
    { 0x269e1512, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce1515, 88 },
    { 0x9a4e1516, 43 },
    { 0, 0 },
    { 0xb4ee1518, 33 },
    { 0xc06e1518, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54e151e, 33 },
    { 0x157e151f, 98 },
    { 0x94e151f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afe1526, 90 },
    { 0xd47e1526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4e1535, 98 },
    { 0x557e1535, 95 },
    { 0x959e1537, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ae153b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ede153e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53e1541, 91 },
    { 0x8fae1542, 99 },
    { 0xa19e1541, 83 },
    { 0x21ae1543, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fe1548, 98 },
    { 0xb13e1549, 91 },
    { 0xab4e154a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7e154d, 40 },
    { 0, 0 },
    { 0x681e154f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49e1552, 93 },
    { 0xf33e1553, 77 },
    { 0x96e1552, 98 },
    { 0x390e1555, 90 },
    { 0xfb0e1552, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e155d, 95 },
    { 0, 0 },
    { 0x577e155f, 76 },
    { 0x5eae155f, 90 },
    { 0x9e1560, 23 },
    { 0, 0 },
    { 0x247e1563, 33 },
    { 0xc8fe1563, 90 },
    { 0x106e1565, 40 },
    { 0, 0 },
    { 0xb3ce1567, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e156b, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78e156e, 95 },
    { 0xf13e156f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde1576, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e1579, 97 },
    { 0x84ae1579, 61 },
    { 0xe03e1579, 23 },
    { 0, 0 },
    { 0x52e157d, 84 },
    { 0, 0 },
    { 0x2ece157f, 98 },
    { 0x6bae157f, 61 },
    { 0xfe4e1581, 61 },
    { 0, 0 },
    { 0xd57e1583, 61 },
    { 0, 0 },
    { 0x29ae1585, 90 },
    { 0xaa2e1586, 96 },
    { 0x370e1585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ade158a, 34 },
    { 0, 0 },
    { 0xe64e158c, 95 },
    { 0x461e158c, 85 },
    { 0, 0 },
    { 0x6bae158f, 95 },
    { 0xe9ae158f, 43 },
    { 0xfe6e1591, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fe1597, 76 },
    { 0xf8fe1597, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee15a2, 33 },
    { 0xf74e15a3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1e15a6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21e15a9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e15b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476e15b7, 33 },
    { 0, 0 },
    { 0x9a3e15b9, 23 },
    { 0x2a3e15ba, 95 },
    { 0, 0 },
    { 0xad6e15bc, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89e15bf, 98 },
    { 0x73de15bf, 95 },
    { 0xed4e15c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efe15c4, 33 },
    { 0xfe1e15c5, 90 },
    { 0x17fe15c6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754e15ce, 33 },
    { 0x64ce15cf, 76 },
    { 0x27ae15d0, 58 },
    { 0x110e15d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91e15d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e15db, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829e15e2, 84 },
    { 0xdf1e15e2, 36 },
    { 0, 0 },
    { 0x58fe15e5, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ee15e9, 33 },
    { 0xb20e15ea, 95 },
    { 0x381e15eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560e15f0, 95 },
    { 0x1bae15f1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15e15fc, 84 },
    { 0, 0 },
    { 0x136e15fe, 98 },
    { 0, 0 },
    { 0x9dae1600, 91 },
    { 0x532e1601, 34 },
    { 0, 0 },
    { 0xfc0e1603, 90 },
    { 0xaf1e1604, 90 },
    { 0xb65e1605, 46 },
    { 0x7b6e1604, 23 },
    { 0x813e1606, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe160b, 55 },
    { 0x227e160c, 90 },
    { 0xeb4e160d, 93 },
    { 0x948e160d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ee1613, 23 },
    { 0xd15e1614, 90 },
    { 0xec7e1614, 38 },
    { 0, 0 },
    { 0x838e1617, 33 },
    { 0, 0 },
    { 0x4aae1619, 55 },
    { 0x2cfe161a, 23 },
    { 0, 0 },
    { 0x4f9e161c, 54 },
    { 0, 0 },
    { 0x5fee161e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4e1621, 95 },
    { 0, 0 },
    { 0xaf2e1623, 98 },
    { 0, 0 },
    { 0xc9ee1625, 90 },
    { 0, 0 },
    { 0x25ce1627, 23 },
    { 0, 0 },
    { 0x8e1e1629, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ee162f, 76 },
    { 0, 0 },
    { 0x92be1631, 90 },
    { 0xbb8e1632, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x139e1635, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e163a, 34 },
    { 0, 0 },
    { 0x5fde163c, 90 },
    { 0xb8de163c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee1646, 91 },
    { 0, 0 },
    { 0xccee1648, 68 },
    { 0x7ffe1649, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fee1650, 35 },
    { 0x724e1650, 99 },
    { 0, 0 },
    { 0x482e1653, 76 },
    { 0, 0 },
    { 0xf54e1655, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8e1659, 33 },
    { 0xef6e165a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579e165e, 34 },
    { 0, 0 },
    { 0x5c8e1660, 23 },
    { 0, 0 },
    { 0xb18e1662, 100 },
    { 0, 0 },
    { 0xf10e1664, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x985e1667, 91 },
    { 0x148e1667, 61 },
    { 0, 0 },
    { 0x333e166a, 33 },
    { 0x7cae166b, 61 },
    { 0xa6de166c, 78 },
    { 0xa0be166b, 90 },
    { 0xca5e166d, 46 },
    { 0xc67e166f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72e1672, 55 },
    { 0, 0 },
    { 0x93ae1674, 2 },
    { 0x4f8e1675, 33 },
    { 0x70be1676, 95 },
    { 0x2aae1677, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86e1681, 55 },
    { 0, 0 },
    { 0xb8ae1683, 43 },
    { 0x42fe1684, 34 },
    { 0x286e1685, 88 },
    { 0x33be1685, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x663e1689, 76 },
    { 0, 0 },
    { 0x513e168b, 98 },
    { 0xede168b, 99 },
    { 0x9f6e168b, 61 },
    { 0, 0 },
    { 0x5d6e168f, 34 },
    { 0x848e1690, 99 },
    { 0x1c3e1691, 95 },
    { 0, 0 },
    { 0xf72e1693, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739e1697, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3e169c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7e16a2, 90 },
    { 0, 0 },
    { 0x983e16a4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe16a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ee16ab, 95 },
    { 0, 0 },
    { 0xdc9e16ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79e16b2, 90 },
    { 0x9cde16b3, 33 },
    { 0x509e16b4, 23 },
    { 0x942e16b4, 90 },
    { 0xbbbe16b6, 87 },
    { 0xc4fe16b6, 90 },
    { 0x862e16b8, 46 },
    { 0x81ce16b8, 67 },
    { 0, 0 },
    { 0x1fde16bb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6e16be, 55 },
    { 0xde8e16bf, 40 },
    { 0xf69e16be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30e16c3, 95 },
    { 0x236e16c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2be16cb, 76 },
    { 0x7c4e16cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x275e16cf, 84 },
    { 0x852e16cf, 90 },
    { 0, 0 },
    { 0x80de16d2, 68 },
    { 0x50ee16d3, 61 },
    { 0xb7be16d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbae16d7, 91 },
    { 0xd6e16d8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afe16db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1e16df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6e16e5, 61 },
    { 0, 0 },
    { 0x27be16e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x267e16ea, 61 },
    { 0, 0 },
    { 0xa54e16ec, 78 },
    { 0, 0 },
    { 0x6c7e16ee, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ce16f1, 76 },
    { 0xe1e16f2, 55 },
    { 0x9cae16f3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4e16f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x16de16fb, 81 },
    { 0x6f5e16fc, 23 },
    { 0x5e16fd, 90 },
    { 0xf95e16fd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae1701, 84 },
    { 0, 0 },
    { 0xb75e1703, 78 },
    { 0x773e1704, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5e1707, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ce1715, 90 },
    { 0, 0 },
    { 0xa9fe1717, 61 },
    { 0x946e1718, 33 },
    { 0x5c0e1719, 90 },
    { 0xff8e171a, 90 },
    { 0xcd5e171b, 90 },
    { 0xe7be171b, 23 },
    { 0x8c1e171c, 76 },
    { 0x421e1719, 84 },
    { 0x955e171f, 98 },
    { 0x849e1720, 99 },
    { 0xf30e171f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e1725, 69 },
    { 0xcf6e1726, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7e172a, 84 },
    { 0xb2ae172b, 90 },
    { 0x8ebe172c, 76 },
    { 0, 0 },
    { 0xc1de172e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773e1732, 23 },
    { 0x64de1733, 95 },
    { 0xa5ce1734, 33 },
    { 0x491e1732, 90 },
    { 0xda0e1736, 34 },
    { 0xe0ee1737, 47 },
    { 0xb48e1738, 91 },
    { 0xd60e1738, 91 },
    { 0x500e1738, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6e1740, 46 },
    { 0x2aae1741, 84 },
    { 0, 0 },
    { 0xe43e1743, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xecae1746, 95 },
    { 0x594e1746, 17 },
    { 0, 0 },
    { 0xebde1749, 98 },
    { 0x84be174a, 33 },
    { 0xd25e174a, 78 },
    { 0x6fde174c, 61 },
    { 0xeb3e174a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fe1753, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946e1757, 66 },
    { 0, 0 },
    { 0x8afe1759, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974e1762, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58e1767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9e176a, 61 },
    { 0x9cde176a, 67 },
    { 0x1fee176b, 95 },
    { 0x768e176c, 33 },
    { 0x181e176b, 95 },
    { 0xefbe176a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x676e1772, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15de1779, 100 },
    { 0x6e2e177a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09e177e, 90 },
    { 0x7cce177f, 23 },
    { 0, 0 },
    { 0x7e5e1781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebe1785, 86 },
    { 0x5afe1786, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69de178a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388e1793, 23 },
    { 0, 0 },
    { 0x913e1795, 76 },
    { 0, 0 },
    { 0x36ee1797, 61 },
    { 0xe75e1797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31e17a1, 77 },
    { 0x158e17a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ce17a6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x425e17a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec9e17ae, 33 },
    { 0x5c4e17af, 23 },
    { 0x803e17b0, 90 },
    { 0x20fe17b0, 33 },
    { 0x91be17b2, 23 },
    { 0x440e17b3, 76 },
    { 0x40ce17b4, 90 },
    { 0xf70e17ae, 43 },
    { 0x9eae17b0, 44 },
    { 0, 0 },
    { 0x70ae17b8, 93 },
    { 0, 0 },
    { 0xfc7e17ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fe17c3, 97 },
    { 0x991e17c4, 55 },
    { 0x5c5e17c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde17c9, 43 },
    { 0xbce17ca, 99 },
    { 0x6f0e17cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59e17d1, 66 },
    { 0x482e17d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5e17d8, 95 },
    { 0, 0 },
    { 0xc5e17da, 80 },
    { 0x2d0e17db, 90 },
    { 0x8fae17dc, 46 },
    { 0xd69e17db, 87 },
    { 0xc13e17da, 33 },
    { 0x3fbe17dd, 43 },
    { 0xc0fe17de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317e17e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x764e17e7, 78 },
    { 0x8f4e17e8, 99 },
    { 0, 0 },
    { 0x8f3e17ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e17ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x33de17f0, 85 },
    { 0x852e17f1, 43 },
    { 0x623e17f0, 95 },
    { 0x4cee17f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e17fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ade1804, 61 },
    { 0x63de1804, 93 },
    { 0xa57e1806, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe180a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cce1810, 46 },
    { 0xb07e1811, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4e1815, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8e181b, 17 },
    { 0xd53e181b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b5e1825, 33 },
    { 0xd21e1826, 98 },
    { 0xbc8e1826, 84 },
    { 0xd83e1828, 87 },
    { 0, 0 },
    { 0x273e182a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47e182e, 90 },
    { 0, 0 },
    { 0x975e1830, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288e1837, 33 },
    { 0x18ee1837, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22be183e, 61 },
    { 0x59e183f, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33e1845, 61 },
    { 0, 0 },
    { 0x1fce1847, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e184e, 61 },
    { 0x183e184f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ce1855, 61 },
    { 0, 0 },
    { 0x35ce1857, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ae1860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975e1865, 81 },
    { 0x1ce1866, 95 },
    { 0x5fbe1866, 61 },
    { 0, 0 },
    { 0x339e1869, 98 },
    { 0x3d8e186a, 90 },
    { 0x7fce186a, 67 },
    { 0xdbde1869, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x259e186f, 34 },
    { 0, 0 },
    { 0x76ae1871, 98 },
    { 0xd1be1872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e187f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdae1884, 90 },
    { 0xcc6e1885, 76 },
    { 0xd4e1884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1e1890, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ee1897, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cee189a, 90 },
    { 0xa3ee189b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2e189f, 46 },
    { 0xca4e189f, 23 },
    { 0, 0 },
    { 0x16e18a2, 61 },
    { 0x887e18a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ce18ac, 40 },
    { 0x912e18ad, 90 },
    { 0x290e18ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x45e18b1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871e18bd, 33 },
    { 0xf02e18bd, 43 },
    { 0x7bde18bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ce18c2, 66 },
    { 0x482e18c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce18c9, 95 },
    { 0xa8fe18ca, 55 },
    { 0x1dce18cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fe18d2, 81 },
    { 0, 0 },
    { 0x740e18d4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828e18d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87be18dc, 34 },
    { 0x674e18dd, 93 },
    { 0xa4be18de, 23 },
    { 0, 0 },
    { 0xbb4e18e0, 90 },
    { 0x6d5e18e0, 66 },
    { 0xd2ee18e2, 77 },
    { 0, 0 },
    { 0xd0fe18e4, 66 },
    { 0x932e18e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bce18ea, 95 },
    { 0, 0 },
    { 0x293e18ec, 78 },
    { 0, 0 },
    { 0x2cae18ee, 17 },
    { 0xc31e18ef, 93 },
    { 0x199e18f0, 55 },
    { 0xf25e18f0, 22 },
    { 0, 0 },
    { 0xabde18f3, 61 },
    { 0xe42e18f4, 77 },
    { 0xbfe18f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x35e18f8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e18fb, 34 },
    { 0xb5fe18fc, 52 },
    { 0x207e18fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x872e1900, 90 },
    { 0, 0 },
    { 0xe7de1902, 33 },
    { 0xedfe1903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e1906, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75e190a, 55 },
    { 0xf8ae190b, 46 },
    { 0xfaae190a, 76 },
    { 0, 0 },
    { 0x10ae190e, 55 },
    { 0x215e190f, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e1912, 23 },
    { 0x397e1913, 76 },
    { 0xeee1913, 78 },
    { 0, 0 },
    { 0xb28e1916, 90 },
    { 0, 0 },
    { 0xfe8e1918, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09e1921, 88 },
    { 0x386e1921, 55 },
    { 0x84be1923, 34 },
    { 0x300e1923, 46 },
    { 0xc21e1925, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x594e1928, 93 },
    { 0x6ace1928, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaae192d, 23 },
    { 0, 0 },
    { 0xde5e192f, 65 },
    { 0x137e1930, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2de1934, 99 },
    { 0x280e1935, 33 },
    { 0xdabe1936, 46 },
    { 0xa26e1937, 90 },
    { 0, 0 },
    { 0x15fe1939, 81 },
    { 0, 0 },
    { 0x18be193b, 66 },
    { 0xc14e193c, 100 },
    { 0x42fe193c, 95 },
    { 0xb82e193e, 33 },
    { 0, 0 },
    { 0x38be1940, 98 },
    { 0x886e1941, 55 },
    { 0x75de1942, 52 },
    { 0x100e1940, 34 },
    { 0x343e1944, 95 },
    { 0x726e1945, 33 },
    { 0x508e1941, 99 },
    { 0xcade1940, 33 },
    { 0xf6ce1948, 23 },
    { 0xeeee1948, 84 },
    { 0x5cfe1942, 95 },
    { 0xd14e1942, 76 },
    { 0xdcee1941, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e194f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be1952, 97 },
    { 0x9dee1953, 33 },
    { 0xefae1953, 95 },
    { 0xacae1954, 33 },
    { 0, 0 },
    { 0x40ee1957, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63e195c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe51e1961, 81 },
    { 0x692e1961, 33 },
    { 0xf12e1963, 78 },
    { 0xe95e1964, 52 },
    { 0xe5be1963, 87 },
    { 0x32be1964, 95 },
    { 0x66e1965, 23 },
    { 0x632e1968, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x197e196b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ae1970, 95 },
    { 0, 0 },
    { 0xa43e1972, 55 },
    { 0, 0 },
    { 0x935e1974, 46 },
    { 0xaf1e1974, 61 },
    { 0x4ee1975, 43 },
    { 0, 0 },
    { 0xcd1e1978, 46 },
    { 0x58be1979, 17 },
    { 0x158e1979, 90 },
    { 0x952e197a, 67 },
    { 0, 0 },
    { 0x3d6e197d, 33 },
    { 0x296e197e, 33 },
    { 0xbb9e197f, 68 },
    { 0, 0 },
    { 0x7b6e1981, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ce1984, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cde1987, 66 },
    { 0x36ce1988, 68 },
    { 0x107e1989, 95 },
    { 0x77ce1987, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592e198e, 90 },
    { 0x946e198e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bce199b, 34 },
    { 0, 0 },
    { 0x9ce199d, 98 },
    { 0x903e199e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607e19a4, 46 },
    { 0x5cde19a4, 61 },
    { 0x402e19a6, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200e19ae, 90 },
    { 0x1ae19af, 69 },
    { 0xfae19b0, 90 },
    { 0xf9ce19b1, 99 },
    { 0x3e6e19b2, 95 },
    { 0, 0 },
    { 0xe02e19b4, 85 },
    { 0x67ae19b5, 75 },
    { 0x86ae19b5, 33 },
    { 0x9e6e19b7, 23 },
    { 0x12de19b8, 61 },
    { 0xa58e19b9, 95 },
    { 0xb45e19b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be19bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x758e19c0, 95 },
    { 0xc2be19c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79e19c7, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94e19cb, 78 },
    { 0, 0 },
    { 0x5e5e19cd, 98 },
    { 0, 0 },
    { 0xe63e19cf, 98 },
    { 0, 0 },
    { 0xd9de19d1, 33 },
    { 0x208e19d2, 55 },
    { 0x467e19d3, 90 },
    { 0x57be19d4, 90 },
    { 0, 0 },
    { 0x4b4e19d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e19db, 23 },
    { 0x8cfe19dc, 98 },
    { 0x752e19dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be19e2, 98 },
    { 0x817e19e3, 77 },
    { 0, 0 },
    { 0x636e19e5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x405e19e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2e19eb, 23 },
    { 0x7b7e19ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e19ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee19f2, 61 },
    { 0x80be19f3, 95 },
    { 0x32fe19f2, 99 },
    { 0x5f5e19f5, 78 },
    { 0, 0 },
    { 0x55e19f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3e19fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ae1a04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0e1a08, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e1a0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ce1a14, 90 },
    { 0x560e1a15, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97e1a18, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e1a1d, 93 },
    { 0, 0 },
    { 0xf9be1a1f, 23 },
    { 0, 0 },
    { 0xa64e1a21, 100 },
    { 0, 0 },
    { 0x9dae1a23, 33 },
    { 0x232e1a24, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce1a2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550e1a30, 90 },
    { 0x262e1a31, 55 },
    { 0xf42e1a32, 98 },
    { 0, 0 },
    { 0xd84e1a34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33de1a3d, 84 },
    { 0xea7e1a3e, 76 },
    { 0x48fe1a3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509e1a46, 61 },
    { 0, 0 },
    { 0x493e1a48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe1a52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde1a56, 90 },
    { 0, 0 },
    { 0xf9ae1a58, 99 },
    { 0xd7de1a58, 98 },
    { 0xb14e1a5a, 85 },
    { 0x6a0e1a5a, 95 },
    { 0x91ee1a5c, 33 },
    { 0x89be1a5d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2e1a64, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e1a73, 46 },
    { 0xfbde1a74, 90 },
    { 0xa2e1a73, 93 },
    { 0xeffe1a76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607e1a7d, 61 },
    { 0x3a6e1a7e, 95 },
    { 0xc1ce1a7d, 55 },
    { 0x676e1a80, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71e1a83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95be1a86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63e1a8a, 61 },
    { 0, 0 },
    { 0x667e1a8c, 61 },
    { 0x721e1a8d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e1a91, 33 },
    { 0x1b0e1a92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6e1a95, 61 },
    { 0x35de1a96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33be1a9e, 55 },
    { 0, 0 },
    { 0xba4e1aa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83de1aa5, 33 },
    { 0x215e1aa6, 43 },
    { 0, 0 },
    { 0x2a7e1aa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ae1aab, 87 },
    { 0x2fae1aac, 43 },
    { 0xad6e1aab, 61 },
    { 0xe41e1aae, 33 },
    { 0, 0 },
    { 0x819e1ab0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500e1ab4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe1ab7, 56 },
    { 0xcf4e1ab8, 46 },
    { 0x2ade1ab9, 61 },
    { 0x4d6e1ab8, 66 },
    { 0x97ce1ab8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x533e1abe, 66 },
    { 0x305e1abf, 90 },
    { 0, 0 },
    { 0xc4ce1ac1, 93 },
    { 0, 0 },
    { 0x548e1ac3, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ae1ac6, 55 },
    { 0xffce1ac6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x808e1aca, 23 },
    { 0xbebe1acb, 98 },
    { 0x4b6e1acb, 33 },
    { 0x4b9e1acd, 61 },
    { 0xbc8e1ace, 76 },
    { 0xf7ae1acf, 90 },
    { 0, 0 },
    { 0xd94e1ad1, 61 },
    { 0xd27e1ad2, 90 },
    { 0xbb4e1ad3, 98 },
    { 0xfd5e1ad3, 33 },
    { 0, 0 },
    { 0xa2ae1ad6, 95 },
    { 0, 0 },
    { 0xe04e1ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28be1ade, 36 },
    { 0x7f4e1ade, 46 },
    { 0x9e6e1ae0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aae1ae5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace1aea, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3be1aed, 34 },
    { 0x5d4e1aee, 38 },
    { 0x3cce1aef, 61 },
    { 0, 0 },
    { 0xa02e1af1, 23 },
    { 0, 0 },
    { 0x18de1af3, 43 },
    { 0xbace1af4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e1afa, 76 },
    { 0x145e1afa, 43 },
    { 0, 0 },
    { 0xdc7e1afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57e1b02, 96 },
    { 0, 0 },
    { 0xbf7e1b04, 33 },
    { 0x75e1b05, 23 },
    { 0xb3ce1b04, 23 },
    { 0xeeae1b07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebe1b0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899e1b14, 33 },
    { 0xe28e1b14, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fe1b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0e1b20, 52 },
    { 0x1cfe1b20, 46 },
    { 0, 0 },
    { 0xc95e1b23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ce1b28, 61 },
    { 0xf26e1b29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75e1b2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2e1b2f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5e1b32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626e1b37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x105e1b3a, 34 },
    { 0, 0 },
    { 0x879e1b3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43e1b3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10e1b44, 95 },
    { 0x31e1b45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ce1b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadee1b4c, 90 },
    { 0x53ae1b4c, 80 },
    { 0x5b8e1b4e, 90 },
    { 0, 0 },
    { 0x35ce1b50, 84 },
    { 0x270e1b51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeae1b55, 61 },
    { 0x508e1b56, 93 },
    { 0x8cde1b57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fe1b5f, 36 },
    { 0, 0 },
    { 0x42ae1b61, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53e1b68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be1b6b, 99 },
    { 0, 0 },
    { 0xa89e1b6d, 52 },
    { 0, 0 },
    { 0xf65e1b6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e1b73, 55 },
    { 0x2b6e1b73, 95 },
    { 0xe24e1b73, 46 },
    { 0x706e1b76, 98 },
    { 0xf21e1b77, 90 },
    { 0, 0 },
    { 0xd6e1b79, 23 },
    { 0, 0 },
    { 0xb74e1b7b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58e1b82, 90 },
    { 0x233e1b83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedae1b87, 55 },
    { 0xe83e1b88, 67 },
    { 0x9a1e1b89, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5e1b8d, 95 },
    { 0, 0 },
    { 0xb37e1b8f, 90 },
    { 0x7a9e1b8f, 33 },
    { 0xf8fe1b8f, 67 },
    { 0x365e1b92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e1b95, 61 },
    { 0xc4e1b96, 34 },
    { 0x2d3e1b97, 33 },
    { 0x944e1b98, 90 },
    { 0xdd4e1b95, 99 },
    { 0xe97e1b9a, 95 },
    { 0x6f2e1b9b, 90 },
    { 0x4c8e1b9c, 98 },
    { 0x13ae1b9a, 95 },
    { 0x73e1b9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620e1ba8, 23 },
    { 0x4e7e1ba8, 23 },
    { 0, 0 },
    { 0xee8e1bab, 98 },
    { 0, 0 },
    { 0xf83e1bad, 77 },
    { 0x388e1bad, 98 },
    { 0x84de1bae, 61 },
    { 0xa20e1bb0, 33 },
    { 0x361e1bb0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91e1bb4, 33 },
    { 0, 0 },
    { 0x9bbe1bb6, 23 },
    { 0xc44e1bb7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732e1bc3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79e1bc6, 81 },
    { 0xc46e1bc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ae1bca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e1bcd, 98 },
    { 0xf52e1bce, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e1bd1, 98 },
    { 0xae9e1bd2, 87 },
    { 0xd49e1bd3, 23 },
    { 0, 0 },
    { 0xbb7e1bd5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e1bd8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53e1bdc, 87 },
    { 0x9f9e1bdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6e1be1, 34 },
    { 0xab3e1be2, 23 },
    { 0x6fce1be3, 90 },
    { 0, 0 },
    { 0xd71e1be5, 61 },
    { 0, 0 },
    { 0xb90e1be7, 55 },
    { 0x5b2e1be8, 98 },
    { 0x854e1be9, 90 },
    { 0x56ce1be7, 38 },
    { 0, 0 },
    { 0x8dee1bec, 91 },
    { 0xae7e1bec, 55 },
    { 0x16ae1bec, 100 },
    { 0, 0 },
    { 0x953e1bf0, 90 },
    { 0x978e1bf0, 38 },
    { 0x81fe1bf0, 46 },
    { 0x145e1bf3, 95 },
    { 0x415e1bf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x60fe1bf7, 55 },
    { 0, 0 },
    { 0xc82e1bf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94e1bfc, 23 },
    { 0x51ae1bfc, 55 },
    { 0xb8ae1bfd, 23 },
    { 0xfefe1bff, 55 },
    { 0x68ae1bfd, 23 },
    { 0x31ae1c01, 95 },
    { 0x7cbe1c00, 95 },
    { 0, 0 },
    { 0xa32e1c04, 33 },
    { 0x37de1c04, 33 },
    { 0x1be1c06, 67 },
    { 0, 0 },
    { 0xec6e1c08, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6e1c0c, 17 },
    { 0x96ae1c0c, 88 },
    { 0x3bfe1c0e, 85 },
    { 0, 0 },
    { 0xc3fe1c10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1e1c18, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aae1c1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07e1c25, 100 },
    { 0xa23e1c26, 40 },
    { 0x89de1c26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243e1c2c, 90 },
    { 0xe8fe1c2d, 67 },
    { 0x725e1c2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1fe1c33, 98 },
    { 0x98fe1c34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146e1c38, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ade1c3b, 90 },
    { 0x138e1c3c, 46 },
    { 0x251e1c3d, 85 },
    { 0x9b2e1c3c, 61 },
    { 0x136e1c3e, 99 },
    { 0xb43e1c40, 33 },
    { 0x3e9e1c40, 88 },
    { 0x38de1c3e, 55 },
    { 0x388e1c43, 33 },
    { 0xd2ee1c44, 55 },
    { 0x464e1c45, 95 },
    { 0x9c1e1c46, 55 },
    { 0xce2e1c45, 98 },
    { 0, 0 },
    { 0x96de1c49, 95 },
    { 0, 0 },
    { 0x4e1c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbee1c50, 69 },
    { 0, 0 },
    { 0xc89e1c52, 61 },
    { 0xf23e1c53, 23 },
    { 0x9a9e1c54, 98 },
    { 0x661e1c54, 95 },
    { 0, 0 },
    { 0xd7de1c57, 43 },
    { 0x3d6e1c58, 99 },
    { 0xbaae1c57, 90 },
    { 0x597e1c5a, 61 },
    { 0xb75e1c5a, 33 },
    { 0x315e1c58, 76 },
    { 0x26ae1c5a, 90 },
    { 0xdabe1c5e, 17 },
    { 0x289e1c5e, 80 },
    { 0x8bde1c60, 34 },
    { 0x3ae1c61, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de1c67, 96 },
    { 0xfe1c68, 101 },
    { 0x91be1c67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69e1c6e, 76 },
    { 0, 0 },
    { 0x73be1c70, 40 },
    { 0xe2ee1c71, 33 },
    { 0xcffe1c72, 65 },
    { 0x361e1c71, 98 },
    { 0x779e1c74, 61 },
    { 0x398e1c74, 98 },
    { 0x73ae1c74, 33 },
    { 0, 0 },
    { 0x580e1c78, 55 },
    { 0xf59e1c79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07e1c83, 99 },
    { 0, 0 },
    { 0x4d7e1c85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce1c8a, 23 },
    { 0x1e0e1c8b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78be1c8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79e1c93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eee1c98, 87 },
    { 0, 0 },
    { 0x736e1c9a, 90 },
    { 0x6bfe1c9a, 61 },
    { 0x49e1c9c, 95 },
    { 0xf44e1c9d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8e1ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159e1ca8, 66 },
    { 0, 0 },
    { 0xf0ee1caa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4e1cb0, 12 },
    { 0x39ce1cb1, 23 },
    { 0xa2e1cb1, 55 },
    { 0x690e1cb3, 23 },
    { 0x8f9e1cb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6de1cba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7e1cbf, 61 },
    { 0xabfe1cbf, 90 },
    { 0x2cae1cc1, 43 },
    { 0xcf3e1cc2, 52 },
    { 0x729e1cc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265e1cc7, 76 },
    { 0, 0 },
    { 0xa93e1cc9, 23 },
    { 0x15be1cca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37e1ccd, 98 },
    { 0, 0 },
    { 0x3ee1ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68e1cd3, 98 },
    { 0x3d1e1cd3, 77 },
    { 0x8aae1cd3, 90 },
    { 0, 0 },
    { 0xd5fe1cd7, 33 },
    { 0xbae1cd8, 76 },
    { 0x607e1cd9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88de1cdc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1e1ce5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e1ce9, 33 },
    { 0x836e1cea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aae1ced, 91 },
    { 0xee8e1cee, 46 },
    { 0, 0 },
    { 0x777e1cf0, 61 },
    { 0x9bde1cf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4be1cf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77e1cfb, 33 },
    { 0x6f7e1cfc, 93 },
    { 0x1eee1cfd, 90 },
    { 0xd6fe1cfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80e1d07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe1d0c, 95 },
    { 0xf75e1d0d, 76 },
    { 0xacfe1d0e, 98 },
    { 0xd13e1d0f, 61 },
    { 0x9f6e1d10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47e1d15, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624e1d1b, 23 },
    { 0, 0 },
    { 0x9c1e1d1d, 90 },
    { 0x1a2e1d1e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15e1d23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e1d2c, 78 },
    { 0, 0 },
    { 0x818e1d2e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6e1d37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65e1d3a, 95 },
    { 0xf5e1d3b, 66 },
    { 0xfc6e1d3a, 33 },
    { 0xbd1e1d3d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551e1d41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69e1d50, 34 },
    { 0x2c3e1d51, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ae1d54, 34 },
    { 0, 0 },
    { 0xaaee1d56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebe1d5a, 33 },
    { 0, 0 },
    { 0x69ae1d5c, 90 },
    { 0xd44e1d5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x156e1d61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce1d65, 95 },
    { 0, 0 },
    { 0x696e1d67, 85 },
    { 0x7fce1d67, 66 },
    { 0x7e0e1d67, 76 },
    { 0x19ce1d67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44e1d74, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5de1d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713e1d80, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ae1d83, 61 },
    { 0, 0 },
    { 0x74be1d85, 98 },
    { 0x70de1d85, 23 },
    { 0x333e1d87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847e1d8c, 90 },
    { 0, 0 },
    { 0xaafe1d8e, 90 },
    { 0, 0 },
    { 0xa01e1d90, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58e1d93, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e1d97, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ce1da3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5e1da7, 23 },
    { 0xcf6e1da8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabce1db1, 61 },
    { 0xc47e1db2, 33 },
    { 0x439e1db2, 100 },
    { 0, 0 },
    { 0xa55e1db5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abe1db9, 95 },
    { 0x988e1db9, 98 },
    { 0x329e1dba, 99 },
    { 0, 0 },
    { 0x15ae1dbd, 23 },
    { 0, 0 },
    { 0x8a8e1dbf, 93 },
    { 0xc94e1dc0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaade1dc6, 90 },
    { 0x4e1dc6, 61 },
    { 0, 0 },
    { 0x294e1dc9, 94 },
    { 0xdf0e1dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93e1dce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e1dd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e1ddd, 90 },
    { 0x586e1dde, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ce1de1, 85 },
    { 0x9fee1de2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e1de6, 98 },
    { 0x52fe1de6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbe1dec, 33 },
    { 0x5aae1ded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3e1df4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x859e1df7, 33 },
    { 0, 0 },
    { 0xbcbe1df9, 61 },
    { 0, 0 },
    { 0xb69e1dfb, 33 },
    { 0, 0 },
    { 0x46fe1dfd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x837e1e03, 61 },
    { 0x70be1e04, 9 },
    { 0x9d2e1e04, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ae1e09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ce1e0f, 23 },
    { 0xadae1e0f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8e1e16, 100 },
    { 0x7afe1e17, 61 },
    { 0x965e1e18, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbe1e24, 90 },
    { 0xb77e1e25, 90 },
    { 0xaece1e26, 61 },
    { 0x929e1e27, 95 },
    { 0xad3e1e28, 90 },
    { 0x54fe1e29, 90 },
    { 0x893e1e27, 61 },
    { 0x74be1e25, 95 },
    { 0xf25e1e2c, 95 },
    { 0x572e1e2d, 61 },
    { 0x30e1e2e, 84 },
    { 0x382e1e2d, 34 },
    { 0xa2ee1e29, 76 },
    { 0x767e1e2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16e1e34, 33 },
    { 0x28e1e34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x459e1e38, 98 },
    { 0x33fe1e39, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee1e43, 61 },
    { 0x7fe1e43, 98 },
    { 0x433e1e44, 22 },
    { 0x3ede1e46, 61 },
    { 0, 0 },
    { 0x316e1e48, 61 },
    { 0, 0 },
    { 0x33ae1e4a, 53 },
    { 0x62fe1e4a, 23 },
    { 0x9f7e1e4c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fe1e50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3e1e56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabfe1e59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x352e1e5c, 23 },
    { 0, 0 },
    { 0x917e1e5e, 98 },
    { 0x74ae1e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e1e65, 33 },
    { 0x16ae1e66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4e1e73, 93 },
    { 0xa81e1e74, 90 },
    { 0, 0 },
    { 0xe79e1e76, 95 },
    { 0x282e1e76, 98 },
    { 0, 0 },
    { 0xd8de1e79, 97 },
    { 0x3dce1e79, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee1e86, 90 },
    { 0xbd7e1e86, 87 },
    { 0x4c8e1e88, 90 },
    { 0x590e1e86, 23 },
    { 0x14ee1e86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437e1e8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6de1e98, 90 },
    { 0, 0 },
    { 0x26fe1e9a, 43 },
    { 0x940e1e9b, 81 },
    { 0x449e1e9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41e1ea0, 88 },
    { 0x312e1ea0, 93 },
    { 0x451e1ea0, 55 },
    { 0xcb2e1ea3, 78 },
    { 0, 0 },
    { 0xad6e1ea5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12de1eac, 93 },
    { 0x9eae1eac, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ece1eb0, 90 },
    { 0x733e1eb1, 33 },
    { 0x10fe1eb2, 43 },
    { 0xb88e1eb3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606e1eba, 76 },
    { 0x4d2e1eba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6e1ebe, 96 },
    { 0xb3be1ebe, 85 },
    { 0, 0 },
    { 0xa73e1ec1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e1ec6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ce1ecb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ee1ecf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0be1ed3, 93 },
    { 0xcc0e1ed4, 95 },
    { 0x8bce1ed5, 95 },
    { 0, 0 },
    { 0x982e1ed7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ce1edd, 95 },
    { 0x9dce1ede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e1ee1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33de1ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492e1eef, 34 },
    { 0, 0 },
    { 0x9f0e1ef1, 90 },
    { 0, 0 },
    { 0x268e1ef3, 61 },
    { 0, 0 },
    { 0x18ee1ef5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be1f03, 90 },
    { 0x5a8e1f03, 43 },
    { 0xcd0e1f05, 99 },
    { 0xb55e1f06, 89 },
    { 0xe91e1f06, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xdade1f0a, 33 },
    { 0xd65e1f0b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ee1f10, 43 },
    { 0, 0 },
    { 0xb12e1f12, 76 },
    { 0, 0 },
    { 0xeaee1f14, 85 },
    { 0x99e1f15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ae1f1a, 34 },
    { 0x665e1f1b, 33 },
    { 0xe90e1f1a, 38 },
    { 0x2a4e1f1d, 98 },
    { 0, 0 },
    { 0x66ce1f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ee1f22, 52 },
    { 0xa64e1f23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fe1f26, 95 },
    { 0x2ade1f27, 98 },
    { 0xe87e1f28, 33 },
    { 0x798e1f26, 23 },
    { 0xf2e1f2a, 90 },
    { 0xcf0e1f2b, 95 },
    { 0, 0 },
    { 0x7f0e1f2d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x725e1f31, 34 },
    { 0, 0 },
    { 0x1ebe1f33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413e1f3b, 43 },
    { 0x576e1f3c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e1f3f, 100 },
    { 0x7bde1f3f, 34 },
    { 0xa4fe1f41, 55 },
    { 0x4e2e1f41, 99 },
    { 0x500e1f43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x426e1f46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e1f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x628e1f4d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ce1f51, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8e1f57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1e1f5e, 95 },
    { 0x32e1f5f, 61 },
    { 0, 0 },
    { 0x83fe1f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1e1f64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bde1f67, 75 },
    { 0x7fae1f68, 90 },
    { 0, 0 },
    { 0x50e1f6a, 61 },
    { 0, 0 },
    { 0x252e1f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e1f6f, 33 },
    { 0x3e3e1f70, 76 },
    { 0xba4e1f6f, 91 },
    { 0x738e1f70, 92 },
    { 0x7c5e1f71, 61 },
    { 0x97ae1f74, 93 },
    { 0xbfce1f73, 33 },
    { 0x99ae1f72, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ae1f7a, 67 },
    { 0, 0 },
    { 0x40be1f7c, 84 },
    { 0xe08e1f7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0e1f80, 23 },
    { 0, 0 },
    { 0x653e1f82, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235e1f88, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde1f8c, 23 },
    { 0x80de1f8d, 95 },
    { 0, 0 },
    { 0x8fe1f8f, 34 },
    { 0xcefe1f90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efe1f93, 52 },
    { 0x72be1f94, 46 },
    { 0xfe1e1f93, 98 },
    { 0x72e1f93, 90 },
    { 0xda5e1f95, 78 },
    { 0, 0 },
    { 0x59fe1f99, 84 },
    { 0, 0 },
    { 0x12fe1f9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e1f9f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x836e1fa2, 90 },
    { 0x319e1fa3, 33 },
    { 0x9abe1fa4, 43 },
    { 0xadae1fa4, 78 },
    { 0x4b4e1fa6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494e1faa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97e1fb2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6e1fb5, 23 },
    { 0x463e1fb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x217e1fb9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bee1fbc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e1fc3, 66 },
    { 0, 0 },
    { 0x6eee1fc5, 81 },
    { 0x340e1fc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5e1fc9, 68 },
    { 0, 0 },
    { 0x684e1fcb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ee1fd2, 33 },
    { 0, 0 },
    { 0x264e1fd4, 43 },
    { 0xac7e1fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3e1fd9, 90 },
    { 0x526e1fda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96e1fe0, 84 },
    { 0x3a6e1fe1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4e1fe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834e1ff1, 55 },
    { 0x470e1ff1, 43 },
    { 0x4a2e1ff2, 61 },
    { 0, 0 },
    { 0x5fde1ff5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7e1ffb, 90 },
    { 0, 0 },
    { 0x248e1ffd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfe2006, 98 },
    { 0xde7e2006, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce200a, 91 },
    { 0x5aae200a, 33 },
    { 0, 0 },
    { 0x25ee200d, 100 },
    { 0xbdbe200e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64e2011, 95 },
    { 0x6fee2012, 90 },
    { 0xe7ee2011, 61 },
    { 0x1c7e2014, 93 },
    { 0, 0 },
    { 0x781e2016, 95 },
    { 0xf0de2017, 61 },
    { 0x4e4e2018, 96 },
    { 0xcbee2019, 76 },
    { 0x9ace2018, 90 },
    { 0xe8fe201b, 43 },
    { 0x8cce201a, 61 },
    { 0x988e201b, 90 },
    { 0xd5ae201e, 61 },
    { 0x903e201e, 85 },
    { 0, 0 },
    { 0x7d1e2021, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaae2027, 90 },
    { 0x175e2027, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9e2030, 95 },
    { 0x43e2031, 66 },
    { 0x7ede2032, 98 },
    { 0x7f4e2033, 98 },
    { 0xb53e2033, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fe203b, 95 },
    { 0x746e203b, 61 },
    { 0, 0 },
    { 0x970e203e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7be204a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7e204f, 95 },
    { 0xb0de2050, 98 },
    { 0, 0 },
    { 0x102e2052, 61 },
    { 0xac0e2052, 61 },
    { 0, 0 },
    { 0x443e2055, 89 },
    { 0x364e2056, 78 },
    { 0x983e2055, 33 },
    { 0x324e2058, 61 },
    { 0, 0 },
    { 0x212e205a, 61 },
    { 0x9ebe205a, 61 },
    { 0x480e205c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900e2063, 33 },
    { 0x1d6e2064, 88 },
    { 0x699e2064, 61 },
    { 0x73e2066, 85 },
    { 0, 0 },
    { 0xe70e2068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x522e206b, 65 },
    { 0xb6ce206c, 17 },
    { 0x705e206b, 90 },
    { 0x4bbe206b, 93 },
    { 0xd1fe206f, 33 },
    { 0, 0 },
    { 0x934e2071, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265e207e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58e2084, 43 },
    { 0xa12e2085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x674e208a, 33 },
    { 0x70e208b, 67 },
    { 0, 0 },
    { 0x30de208d, 90 },
    { 0x8d2e208e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fe2093, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfde209a, 46 },
    { 0x52e209b, 33 },
    { 0x99de209a, 87 },
    { 0xc05e209d, 33 },
    { 0x70ee209e, 76 },
    { 0, 0 },
    { 0x4e6e20a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x981e20a3, 33 },
    { 0, 0 },
    { 0xc54e20a5, 95 },
    { 0x88de20a6, 43 },
    { 0x743e20a7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ce20ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ce20af, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x344e20b2, 99 },
    { 0x470e20b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705e20b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6e20c5, 91 },
    { 0, 0 },
    { 0xc3be20c7, 90 },
    { 0, 0 },
    { 0x83be20c9, 55 },
    { 0, 0 },
    { 0xf22e20cb, 93 },
    { 0, 0 },
    { 0x6cbe20cd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cde20d0, 95 },
    { 0, 0 },
    { 0x7c4e20d2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe20d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ae20dc, 61 },
    { 0x910e20dd, 85 },
    { 0xb2de20dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589e20e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb9e20e7, 76 },
    { 0x933e20e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9e20eb, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9e20ee, 37 },
    { 0x961e20ee, 61 },
    { 0xd13e20f0, 97 },
    { 0x66ae20f1, 95 },
    { 0x4aee20f2, 23 },
    { 0xd2e20f3, 43 },
    { 0x9ae20f4, 66 },
    { 0x73be20f1, 43 },
    { 0x7d8e20f6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de20f9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47e20fd, 23 },
    { 0x366e20fe, 61 },
    { 0xa70e20fd, 34 },
    { 0x2bae2100, 46 },
    { 0x520e2100, 93 },
    { 0xb6ae2102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2e2107, 46 },
    { 0, 0 },
    { 0xdafe2109, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e2110, 90 },
    { 0x3f1e2110, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfe2118, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x346e211b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e211e, 33 },
    { 0xd69e211f, 33 },
    { 0x4f3e2120, 55 },
    { 0x7a1e211f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0e2127, 33 },
    { 0xc2ce2128, 61 },
    { 0x8e7e2127, 23 },
    { 0x73ae212a, 98 },
    { 0x63ee212b, 40 },
    { 0xe07e212b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e212f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe2134, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdee213a, 76 },
    { 0, 0 },
    { 0xa5de213c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e213f, 33 },
    { 0xdfee2140, 65 },
    { 0x5aee2140, 61 },
    { 0, 0 },
    { 0x738e2143, 33 },
    { 0xa22e2144, 98 },
    { 0xa88e2145, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13e2149, 46 },
    { 0xc9ae214a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8e214d, 40 },
    { 0x37ae214d, 99 },
    { 0x97de214f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce2153, 99 },
    { 0x831e2153, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cde2159, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0e215c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283e2163, 61 },
    { 0x378e2163, 33 },
    { 0xe76e2165, 76 },
    { 0x566e2166, 33 },
    { 0x2d2e2164, 55 },
    { 0, 0 },
    { 0x5d7e2169, 33 },
    { 0, 0 },
    { 0x801e216b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e2170, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2e2176, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4e217c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e2181, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407e2188, 84 },
    { 0, 0 },
    { 0x504e218a, 33 },
    { 0xd01e218b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ee218e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65e2191, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x509e2194, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820e2198, 43 },
    { 0x5e3e2199, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e21a2, 23 },
    { 0xbc1e21a3, 95 },
    { 0x2a3e21a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0e21a8, 90 },
    { 0, 0 },
    { 0x53ee21aa, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e21ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52be21b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe21b8, 43 },
    { 0x8a4e21b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184e21c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458e21c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8e21cc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4e21d2, 66 },
    { 0x500e21d2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e21d8, 23 },
    { 0, 0 },
    { 0x6f4e21da, 61 },
    { 0x56e21db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbe21e6, 95 },
    { 0xf75e21e7, 36 },
    { 0x9b8e21e8, 61 },
    { 0x351e21e9, 23 },
    { 0xc3be21e9, 100 },
    { 0x760e21e8, 91 },
    { 0xe89e21ec, 78 },
    { 0, 0 },
    { 0xf11e21ee, 23 },
    { 0x73ee21ee, 98 },
    { 0, 0 },
    { 0xee9e21f1, 43 },
    { 0, 0 },
    { 0xd11e21f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e21f8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x518e21fb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ade21fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbe2202, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5e2209, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71e220f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88be221a, 33 },
    { 0x608e221a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfe221f, 61 },
    { 0x3b0e2220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ae2226, 78 },
    { 0x11e2226, 90 },
    { 0x5d8e2228, 33 },
    { 0x9d4e2229, 90 },
    { 0x2ede2229, 23 },
    { 0xf86e222a, 61 },
    { 0xe16e222c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91e222f, 90 },
    { 0x60ae222f, 100 },
    { 0xc9ae2231, 17 },
    { 0, 0 },
    { 0xf91e2233, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67be2236, 93 },
    { 0x17fe2237, 33 },
    { 0x269e2238, 76 },
    { 0x739e2238, 33 },
    { 0x7bae2239, 61 },
    { 0, 0 },
    { 0x989e223c, 61 },
    { 0, 0 },
    { 0x720e223e, 23 },
    { 0, 0 },
    { 0xab6e2240, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x420e2243, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751e2247, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e224a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292e224e, 33 },
    { 0x61de224e, 95 },
    { 0xc8be2250, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919e2254, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710e225c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e226a, 23 },
    { 0, 0 },
    { 0x9f3e226c, 84 },
    { 0, 0 },
    { 0xf69e226e, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e2271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9e2276, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3e227a, 40 },
    { 0x278e227a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29e227e, 100 },
    { 0, 0 },
    { 0x43de2280, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3e2288, 61 },
    { 0, 0 },
    { 0xed2e228a, 33 },
    { 0x8e4e228b, 33 },
    { 0, 0 },
    { 0x1b1e228d, 99 },
    { 0, 0 },
    { 0x168e228f, 78 },
    { 0x902e2290, 95 },
    { 0, 0 },
    { 0xc21e2292, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ee2296, 43 },
    { 0xbdfe2296, 23 },
    { 0x2dde2297, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e229f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ee22a2, 98 },
    { 0x361e22a3, 69 },
    { 0x3a1e22a2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7e22a8, 61 },
    { 0x83de22a8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3e22ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e22b1, 34 },
    { 0xd94e22b2, 61 },
    { 0, 0 },
    { 0x4f2e22b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cde22b8, 68 },
    { 0xc8ee22b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54be22bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fe22c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce22c5, 33 },
    { 0, 0 },
    { 0x523e22c7, 53 },
    { 0x85fe22c8, 67 },
    { 0x282e22c9, 90 },
    { 0, 0 },
    { 0x20ae22cb, 99 },
    { 0xbc4e22cc, 76 },
    { 0, 0 },
    { 0xd2e22ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbae22d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee22d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709e22e1, 85 },
    { 0, 0 },
    { 0x8ebe22e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41e22e6, 98 },
    { 0x899e22e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae22ea, 33 },
    { 0, 0 },
    { 0x20be22ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e22f2, 66 },
    { 0x4b9e22f3, 43 },
    { 0x71e22f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e22f8, 17 },
    { 0, 0 },
    { 0xe1ce22fa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e2300, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4be2304, 34 },
    { 0xef1e2304, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4e230a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3e2310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1e2314, 99 },
    { 0xe73e2315, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89de2318, 34 },
    { 0xd4fe2319, 90 },
    { 0xcd4e231a, 33 },
    { 0x3b5e231a, 85 },
    { 0, 0 },
    { 0xcd5e231d, 84 },
    { 0xeb8e231d, 90 },
    { 0xc43e231e, 95 },
    { 0, 0 },
    { 0x73ae2321, 64 },
    { 0x718e2322, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e2325, 93 },
    { 0x9d0e2326, 98 },
    { 0x9e5e2326, 33 },
    { 0x44ce2328, 91 },
    { 0, 0 },
    { 0x2bfe232a, 55 },
    { 0xb55e232b, 90 },
    { 0x6b5e232c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ade2336, 93 },
    { 0x8e5e2337, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefe233a, 95 },
    { 0, 0 },
    { 0x33fe233c, 55 },
    { 0, 0 },
    { 0x47ce233e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20e2342, 98 },
    { 0x4e2343, 17 },
    { 0xe17e2344, 91 },
    { 0xd94e2345, 39 },
    { 0, 0 },
    { 0x2c4e2347, 33 },
    { 0x5c6e2348, 23 },
    { 0, 0 },
    { 0x786e234a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564e2353, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fe2359, 33 },
    { 0xa3ee235a, 95 },
    { 0xdb3e235b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae235f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1e2366, 76 },
    { 0x25ae2366, 33 },
    { 0xb81e2368, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9e236e, 53 },
    { 0, 0 },
    { 0x759e2370, 66 },
    { 0, 0 },
    { 0xf3e2372, 98 },
    { 0, 0 },
    { 0x511e2374, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7afe2379, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679e2382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6e238b, 76 },
    { 0xc4de238b, 34 },
    { 0x3e8e238d, 61 },
    { 0xdcbe238c, 98 },
    { 0, 0 },
    { 0x27be2390, 93 },
    { 0xcce2390, 33 },
    { 0xbd3e2392, 61 },
    { 0x5ffe2391, 55 },
    { 0x64ae2393, 89 },
    { 0, 0 },
    { 0xc37e2396, 95 },
    { 0xfcce2396, 76 },
    { 0x60be2396, 98 },
    { 0, 0 },
    { 0x75ae239a, 95 },
    { 0x1cfe239b, 65 },
    { 0, 0 },
    { 0x71ee239d, 88 },
    { 0xc91e239e, 33 },
    { 0x9a4e239e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade23a5, 43 },
    { 0, 0 },
    { 0xae1e23a7, 91 },
    { 0xebfe23a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6e23ac, 95 },
    { 0, 0 },
    { 0x9e7e23ae, 55 },
    { 0xc81e23af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e23b2, 77 },
    { 0, 0 },
    { 0x472e23b4, 33 },
    { 0xcfde23b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dce23b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40e23bd, 75 },
    { 0, 0 },
    { 0x9bce23bf, 90 },
    { 0, 0 },
    { 0xf93e23c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9e23c5, 95 },
    { 0x587e23c6, 76 },
    { 0x317e23c7, 90 },
    { 0x49de23c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fe23ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dce23d1, 95 },
    { 0xbd7e23d2, 93 },
    { 0x6aae23d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5de23d6, 76 },
    { 0xf66e23d6, 33 },
    { 0x51ce23d6, 55 },
    { 0x1e0e23d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8be23e2, 76 },
    { 0, 0 },
    { 0xfafe23e4, 80 },
    { 0xe3de23e4, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e23e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8de23ef, 61 },
    { 0x8cae23f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7e23f3, 98 },
    { 0x8ade23f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ee23fb, 60 },
    { 0x59fe23fc, 23 },
    { 0xa78e23fb, 66 },
    { 0x87fe23fe, 90 },
    { 0x6c6e23fe, 55 },
    { 0x3a9e2400, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e2403, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90e2406, 43 },
    { 0, 0 },
    { 0xb3ce2408, 98 },
    { 0x307e2409, 28 },
    { 0xa70e2409, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e2411, 55 },
    { 0x9bbe2412, 76 },
    { 0x998e2413, 23 },
    { 0, 0 },
    { 0x8cae2415, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78e2418, 34 },
    { 0xfade2419, 93 },
    { 0x6f7e241a, 90 },
    { 0x163e241b, 90 },
    { 0, 0 },
    { 0x676e241d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x316e2420, 55 },
    { 0x68e2421, 46 },
    { 0xf50e2422, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5e2425, 43 },
    { 0, 0 },
    { 0x4f8e2427, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e242d, 46 },
    { 0x5fae242d, 67 },
    { 0xbbfe242d, 90 },
    { 0x7b9e2430, 87 },
    { 0xb9de2431, 52 },
    { 0x7f7e2431, 34 },
    { 0xd14e2433, 98 },
    { 0xa1de2434, 46 },
    { 0x6d3e2435, 84 },
    { 0x4d3e2432, 17 },
    { 0xa41e2437, 93 },
    { 0, 0 },
    { 0xad5e2439, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cae2441, 88 },
    { 0, 0 },
    { 0xd6be2443, 17 },
    { 0x952e2443, 76 },
    { 0x79de2443, 75 },
    { 0, 0 },
    { 0x868e2447, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17de244a, 47 },
    { 0, 0 },
    { 0x42ae244c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaee2450, 90 },
    { 0x7cfe2450, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cde2455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26be2459, 34 },
    { 0x6b4e245a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x200e245e, 66 },
    { 0, 0 },
    { 0x824e2460, 33 },
    { 0xf97e2461, 67 },
    { 0x720e2461, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2e2466, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe246b, 95 },
    { 0, 0 },
    { 0x593e246d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1e2471, 95 },
    { 0x85ae2472, 85 },
    { 0xb75e2473, 95 },
    { 0, 0 },
    { 0x669e2475, 76 },
    { 0xb07e2476, 90 },
    { 0xb5ce2476, 61 },
    { 0x24ee2478, 76 },
    { 0, 0 },
    { 0x9abe247a, 66 },
    { 0, 0 },
    { 0x52e247c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dee2483, 23 },
    { 0x763e2484, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x399e2487, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ce248b, 61 },
    { 0x3f2e248c, 93 },
    { 0, 0 },
    { 0xb5be248e, 43 },
    { 0xf06e248e, 76 },
    { 0x1c4e2490, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e2493, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36de2498, 90 },
    { 0xf8be2499, 77 },
    { 0, 0 },
    { 0x764e249b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbe249e, 99 },
    { 0, 0 },
    { 0xc7ae24a0, 61 },
    { 0xe5be24a1, 5 },
    { 0, 0 },
    { 0x48ae24a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e24a8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbee24ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ce24b0, 78 },
    { 0x677e24b1, 34 },
    { 0, 0 },
    { 0x38e24b3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbe24b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfade24bc, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647e24c2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255e24c6, 87 },
    { 0xe6ee24c7, 76 },
    { 0x6dbe24c8, 68 },
    { 0, 0 },
    { 0x566e24ca, 92 },
    { 0x932e24cb, 23 },
    { 0xec1e24cc, 99 },
    { 0x441e24cd, 90 },
    { 0xab5e24ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6e24d2, 33 },
    { 0, 0 },
    { 0x16ee24d4, 76 },
    { 0xb17e24d4, 68 },
    { 0xdbde24d5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135e24db, 76 },
    { 0xabfe24db, 90 },
    { 0x9a6e24dd, 23 },
    { 0, 0 },
    { 0x24ae24df, 99 },
    { 0, 0 },
    { 0x2afe24e1, 55 },
    { 0, 0 },
    { 0x1f1e24e3, 81 },
    { 0, 0 },
    { 0x821e24e5, 90 },
    { 0, 0 },
    { 0xfeae24e7, 61 },
    { 0x7c2e24e8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x560e24eb, 76 },
    { 0, 0 },
    { 0xb1ae24ed, 61 },
    { 0, 0 },
    { 0x45ce24ef, 8 },
    { 0, 0 },
    { 0xa99e24f1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cce24f5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfe24f9, 33 },
    { 0x3a8e24fa, 90 },
    { 0, 0 },
    { 0xe8e24fc, 23 },
    { 0xe37e24fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ee2501, 93 },
    { 0x114e2502, 76 },
    { 0, 0 },
    { 0xe09e2504, 33 },
    { 0xcd1e2504, 95 },
    { 0x6b7e2506, 33 },
    { 0xd52e2507, 55 },
    { 0x4f6e2508, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x466e250b, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe2514, 95 },
    { 0xa9ae2515, 88 },
    { 0xf76e2516, 89 },
    { 0x79de2517, 91 },
    { 0x53e2516, 35 },
    { 0xfd2e2519, 52 },
    { 0x754e251a, 87 },
    { 0xf85e2515, 33 },
    { 0x548e251c, 33 },
    { 0xe74e251d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ee2524, 61 },
    { 0xc8de2525, 91 },
    { 0x479e2526, 84 },
    { 0x92ee2527, 78 },
    { 0xfd9e2528, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x315e252b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ee2536, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16e253c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330e2542, 97 },
    { 0, 0 },
    { 0x412e2544, 99 },
    { 0, 0 },
    { 0xdb4e2546, 33 },
    { 0xfd8e2547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abe254e, 90 },
    { 0x7ee254f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97e2552, 90 },
    { 0x6bae2552, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ce255a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ae255f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280e256d, 98 },
    { 0, 0 },
    { 0xc4ce256f, 90 },
    { 0, 0 },
    { 0x504e2571, 95 },
    { 0x4ece2571, 33 },
    { 0, 0 },
    { 0x959e2574, 33 },
    { 0, 0 },
    { 0x6ebe2576, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x402e2579, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ae257f, 91 },
    { 0x790e257f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3e2586, 23 },
    { 0x459e2587, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ce258b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97e258f, 61 },
    { 0, 0 },
    { 0xb0e2591, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ce2595, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xace259d, 55 },
    { 0xf54e259d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e25a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9e25ad, 23 },
    { 0x4aae25ae, 61 },
    { 0xba7e25af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409e25b5, 52 },
    { 0xd2fe25b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x57e25b9, 90 },
    { 0, 0 },
    { 0xc20e25bb, 85 },
    { 0xcbee25bc, 43 },
    { 0, 0 },
    { 0xd1e25be, 52 },
    { 0x1b7e25be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6de25c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48de25ca, 87 },
    { 0, 0 },
    { 0x737e25cc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e25d1, 43 },
    { 0x858e25d2, 23 },
    { 0x3e4e25d1, 33 },
    { 0, 0 },
    { 0xcafe25d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7e25d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9e25e2, 33 },
    { 0xf0ee25e3, 90 },
    { 0, 0 },
    { 0x575e25e5, 17 },
    { 0x27e25e6, 78 },
    { 0x781e25e6, 90 },
    { 0, 0 },
    { 0x927e25e9, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3e25ec, 67 },
    { 0xe50e25ed, 95 },
    { 0xb8ce25ee, 91 },
    { 0x500e25ed, 23 },
    { 0xd4ae25f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebe25f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x26be25f9, 33 },
    { 0xffee25fa, 87 },
    { 0x89e25fb, 23 },
    { 0x714e25f9, 95 },
    { 0x58be25fd, 61 },
    { 0, 0 },
    { 0x8dae25ff, 55 },
    { 0x1ece25ff, 43 },
    { 0, 0 },
    { 0xecee2602, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11e2605, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e2608, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852e260f, 99 },
    { 0x1a3e260f, 33 },
    { 0xe92e2611, 33 },
    { 0x765e260f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9e2615, 23 },
    { 0, 0 },
    { 0x323e2617, 90 },
    { 0xd6fe2618, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61e261d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e2627, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbe262a, 40 },
    { 0xf85e262b, 80 },
    { 0x7c5e262c, 66 },
    { 0xbb3e262d, 33 },
    { 0x34ee262e, 93 },
    { 0xcc8e262d, 76 },
    { 0, 0 },
    { 0x8f1e2631, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76e2634, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6e2637, 66 },
    { 0, 0 },
    { 0xf93e2639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5e263c, 33 },
    { 0x3e9e263c, 68 },
    { 0, 0 },
    { 0x620e263f, 33 },
    { 0x6a8e2640, 90 },
    { 0x7d0e2641, 93 },
    { 0x646e2641, 46 },
    { 0, 0 },
    { 0xcb0e2644, 34 },
    { 0, 0 },
    { 0xf6be2646, 23 },
    { 0x6c0e2646, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ee264b, 66 },
    { 0, 0 },
    { 0x210e264d, 23 },
    { 0xbbe264e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc97e2654, 33 },
    { 0, 0 },
    { 0x819e2656, 95 },
    { 0, 0 },
    { 0x8d0e2658, 87 },
    { 0x592e2658, 99 },
    { 0xfc4e2659, 87 },
    { 0x40ee2658, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7e265e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e2662, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde2665, 61 },
    { 0, 0 },
    { 0xbe9e2667, 34 },
    { 0x6fee2668, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254e266d, 23 },
    { 0, 0 },
    { 0xd6ae266f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48e2672, 100 },
    { 0x7efe2672, 33 },
    { 0x355e2674, 98 },
    { 0x367e2675, 55 },
    { 0xddbe2676, 95 },
    { 0xaee2674, 90 },
    { 0xcc3e2677, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x224e267d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86e2682, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7e2686, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ce268b, 95 },
    { 0, 0 },
    { 0x353e268d, 90 },
    { 0xf56e268d, 90 },
    { 0x6a1e268e, 34 },
    { 0x730e2690, 43 },
    { 0x98fe2691, 33 },
    { 0, 0 },
    { 0x8a9e2693, 33 },
    { 0xa3ce2694, 65 },
    { 0x41de2695, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0e2698, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624e269e, 75 },
    { 0x6ce269f, 98 },
    { 0x885e269e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e26a3, 66 },
    { 0x3fbe26a3, 61 },
    { 0xf19e26a5, 90 },
    { 0xbc1e26a5, 33 },
    { 0x39ee26a7, 23 },
    { 0xac6e26a4, 98 },
    { 0xc6ae26a5, 67 },
    { 0xc11e26aa, 77 },
    { 0x1fbe26aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x845e26af, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e26b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53e26bb, 95 },
    { 0xe01e26bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702e26c1, 61 },
    { 0xb00e26c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e26c6, 61 },
    { 0, 0 },
    { 0xf67e26c8, 61 },
    { 0, 0 },
    { 0x301e26ca, 46 },
    { 0x711e26ca, 23 },
    { 0xebde26ca, 55 },
    { 0xf7be26cd, 99 },
    { 0, 0 },
    { 0x941e26cf, 90 },
    { 0, 0 },
    { 0x659e26d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81e26d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ae26d8, 99 },
    { 0x6e9e26d9, 98 },
    { 0, 0 },
    { 0x854e26db, 95 },
    { 0, 0 },
    { 0x347e26dd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4e26e3, 98 },
    { 0xcb1e26e4, 55 },
    { 0xeece26e4, 98 },
    { 0xd9ee26e6, 61 },
    { 0x1fe26e4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5e26eb, 90 },
    { 0xf4de26ec, 76 },
    { 0xa15e26ec, 90 },
    { 0x719e26ee, 66 },
    { 0x451e26ed, 67 },
    { 0xec9e26ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ee26f6, 56 },
    { 0x402e26f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5e26ff, 90 },
    { 0x381e26ff, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19e2709, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0e270c, 90 },
    { 0x9c2e270d, 90 },
    { 0x42de270d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x317e2711, 23 },
    { 0xd9ce2712, 98 },
    { 0x47ae2713, 95 },
    { 0x18e2713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x807e2717, 100 },
    { 0x5a3e2718, 101 },
    { 0x37ae2719, 61 },
    { 0xbc1e271a, 69 },
    { 0, 0 },
    { 0x61ee271c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e271f, 95 },
    { 0xa4ee2720, 90 },
    { 0, 0 },
    { 0x84de2722, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7e2725, 90 },
    { 0, 0 },
    { 0xa62e2727, 90 },
    { 0x5b8e2727, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbe2731, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7e2734, 23 },
    { 0xebde2734, 90 },
    { 0x295e2734, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8e273b, 61 },
    { 0xcede273c, 46 },
    { 0x968e273d, 33 },
    { 0x859e273e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3de2741, 17 },
    { 0x4eee2741, 76 },
    { 0x97ee2743, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14e2746, 100 },
    { 0xe2be2747, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbe274c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06e2751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e2755, 93 },
    { 0x46e2756, 37 },
    { 0xaa5e2757, 34 },
    { 0xed3e2758, 67 },
    { 0xa95e2759, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172e275f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2e2762, 76 },
    { 0, 0 },
    { 0x921e2764, 98 },
    { 0x71e2765, 77 },
    { 0x8ffe2766, 76 },
    { 0x1b0e2767, 33 },
    { 0xaa1e2767, 61 },
    { 0x354e2768, 23 },
    { 0x2f4e276a, 61 },
    { 0xf47e276b, 36 },
    { 0x443e276c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e2773, 88 },
    { 0x6fbe2774, 34 },
    { 0x942e2775, 65 },
    { 0x128e2775, 98 },
    { 0xb57e2774, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9e277b, 90 },
    { 0xbebe277c, 34 },
    { 0xaace277b, 52 },
    { 0x163e277e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3e2782, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ce2785, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573e2789, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e278c, 45 },
    { 0x6e0e278c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2de2790, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685e2795, 23 },
    { 0xd2de2795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e2799, 61 },
    { 0xfbbe279a, 66 },
    { 0x27e279b, 33 },
    { 0x699e279c, 90 },
    { 0, 0 },
    { 0x780e279e, 90 },
    { 0, 0 },
    { 0x8bbe27a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00e27a4, 61 },
    { 0xc6ee27a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756e27ac, 66 },
    { 0, 0 },
    { 0x707e27ae, 84 },
    { 0xe58e27ae, 40 },
    { 0xb4ce27af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e27b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5be27bf, 55 },
    { 0, 0 },
    { 0x786e27c1, 90 },
    { 0x6ade27c2, 61 },
    { 0x418e27c3, 33 },
    { 0, 0 },
    { 0x68be27c5, 90 },
    { 0xbcbe27c5, 90 },
    { 0, 0 },
    { 0xa5ae27c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e27cb, 34 },
    { 0, 0 },
    { 0x6b2e27cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2e27d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788e27d6, 43 },
    { 0x2dde27d7, 61 },
    { 0x5eee27d7, 33 },
    { 0, 0 },
    { 0xce2e27da, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e27dd, 87 },
    { 0x76be27dd, 66 },
    { 0xfb8e27df, 90 },
    { 0x895e27e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abe27e3, 90 },
    { 0x82fe27e4, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53e27e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65e27eb, 98 },
    { 0x4e9e27ec, 85 },
    { 0x8b9e27ec, 96 },
    { 0x49fe27ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e27f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bee2800, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e2803, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2806, 67 },
    { 0x609e2807, 83 },
    { 0, 0 },
    { 0x6a1e2809, 84 },
    { 0x2bbe2809, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443e280e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1e2819, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbae281c, 33 },
    { 0, 0 },
    { 0xf3e281e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14e2822, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e2827, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5e282a, 100 },
    { 0, 0 },
    { 0xed0e282c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fe282f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ce2834, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce2838, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4e283e, 91 },
    { 0x5d9e283e, 90 },
    { 0x441e2840, 81 },
    { 0xdbe2840, 43 },
    { 0xe82e2840, 65 },
    { 0xaae2843, 66 },
    { 0xc8ee2844, 98 },
    { 0xf7ce2845, 95 },
    { 0xf57e2842, 96 },
    { 0x6dae2842, 98 },
    { 0, 0 },
    { 0x48ee2849, 23 },
    { 0x2d1e2849, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128e284e, 90 },
    { 0, 0 },
    { 0x11de2850, 61 },
    { 0xc25e2851, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dae2854, 97 },
    { 0xf5ee2854, 95 },
    { 0xf1ae2856, 84 },
    { 0x33ae2855, 90 },
    { 0xc2ce2858, 90 },
    { 0x7d3e2859, 90 },
    { 0x942e285a, 61 },
    { 0, 0 },
    { 0xa7ae285c, 90 },
    { 0xd85e285c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x99e2860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ce2865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eee286c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fe2872, 84 },
    { 0x7e2e2873, 33 },
    { 0x155e2873, 98 },
    { 0xaf1e2875, 96 },
    { 0x99be2876, 77 },
    { 0xea2e2877, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7e287a, 43 },
    { 0xbb1e287a, 61 },
    { 0x179e287b, 95 },
    { 0x7a9e287a, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109e2883, 81 },
    { 0x517e2884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ae288a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fde288e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230e2892, 23 },
    { 0x34be2892, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93e2896, 33 },
    { 0x4b5e2897, 52 },
    { 0x5cae2897, 23 },
    { 0x904e2898, 95 },
    { 0x9a9e2899, 87 },
    { 0xb7be289b, 61 },
    { 0x5a1e289b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57de28a0, 82 },
    { 0, 0 },
    { 0x417e28a2, 43 },
    { 0x1fde28a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ce28a6, 90 },
    { 0, 0 },
    { 0xab2e28a8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e28ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x621e28b0, 95 },
    { 0xe0fe28b0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fde28b4, 98 },
    { 0, 0 },
    { 0x1f3e28b6, 33 },
    { 0x538e28b7, 33 },
    { 0, 0 },
    { 0x8fde28b9, 61 },
    { 0xf33e28ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8e28bd, 91 },
    { 0x701e28be, 87 },
    { 0x4f4e28be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dde28c6, 98 },
    { 0xf86e28c6, 90 },
    { 0, 0 },
    { 0x168e28c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e28cc, 95 },
    { 0x59de28cd, 91 },
    { 0x49e28ce, 91 },
    { 0x9eae28cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50e28d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x114e28d6, 61 },
    { 0xa1ce28d7, 90 },
    { 0x379e28d7, 90 },
    { 0xf0ce28d9, 77 },
    { 0x4ece28da, 91 },
    { 0x270e28db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e28df, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e28e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbe28e9, 98 },
    { 0x543e28ea, 90 },
    { 0, 0 },
    { 0x232e28ec, 61 },
    { 0x68e28ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0de28f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0e28f3, 49 },
    { 0x328e28f4, 55 },
    { 0x559e28f5, 33 },
    { 0x658e28f6, 61 },
    { 0xa77e28f7, 46 },
    { 0x95fe28f5, 90 },
    { 0x5e2e28f7, 46 },
    { 0xd12e28fa, 61 },
    { 0xfe3e28fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ce28fe, 36 },
    { 0x21be28ff, 90 },
    { 0xc1fe2900, 75 },
    { 0, 0 },
    { 0xc38e2902, 24 },
    { 0x3cfe2903, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8de2906, 61 },
    { 0x687e2906, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00e290a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7e2915, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e291f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3e2923, 95 },
    { 0x240e2924, 96 },
    { 0x604e2925, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cae2929, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476e292e, 61 },
    { 0xfd5e292f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ee2934, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56e2938, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e293b, 23 },
    { 0x564e293b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57e293f, 87 },
    { 0, 0 },
    { 0x388e2941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3e2948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6e294e, 93 },
    { 0, 0 },
    { 0xd8e2950, 90 },
    { 0, 0 },
    { 0x37ce2952, 96 },
    { 0, 0 },
    { 0x8c6e2954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ee2957, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42e295d, 98 },
    { 0x89ae295e, 23 },
    { 0x927e295e, 33 },
    { 0x3e1e2960, 61 },
    { 0xf49e2961, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e2965, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4fe296f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94e2975, 23 },
    { 0x38be2976, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675e297a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57e2985, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72e2989, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37e298c, 23 },
    { 0, 0 },
    { 0x9cee298e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffe2991, 23 },
    { 0xa26e2992, 95 },
    { 0x904e2991, 84 },
    { 0, 0 },
    { 0x857e2995, 23 },
    { 0, 0 },
    { 0xb8be2997, 95 },
    { 0x39be2997, 61 },
    { 0x57fe2998, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6e299d, 33 },
    { 0x3f1e299d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fe29a7, 23 },
    { 0x98ce29a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e29ac, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2e29b0, 93 },
    { 0x11de29b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59e29b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963e29bc, 33 },
    { 0xf03e29bd, 67 },
    { 0x4c3e29bd, 95 },
    { 0, 0 },
    { 0xce3e29c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294e29c4, 34 },
    { 0x579e29c5, 90 },
    { 0xf14e29c4, 98 },
    { 0, 0 },
    { 0xe5ae29c8, 61 },
    { 0xafae29c9, 23 },
    { 0xe63e29ca, 76 },
    { 0x5b4e29cb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee29cf, 90 },
    { 0x17ce29cf, 78 },
    { 0x365e29d1, 98 },
    { 0xcd0e29d2, 55 },
    { 0, 0 },
    { 0x9b4e29d4, 46 },
    { 0xeb5e29d5, 23 },
    { 0, 0 },
    { 0xd72e29d7, 33 },
    { 0xc90e29d7, 93 },
    { 0xb4be29d9, 61 },
    { 0x344e29da, 98 },
    { 0xc13e29db, 93 },
    { 0x78de29dc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee29e0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987e29e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5e29e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862e29ec, 23 },
    { 0, 0 },
    { 0x661e29ee, 34 },
    { 0xdb4e29ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2e29f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1e29f9, 95 },
    { 0, 0 },
    { 0xb15e29fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x469e29fe, 23 },
    { 0x514e29fe, 90 },
    { 0x8c2e2a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ae2a04, 17 },
    { 0xb88e2a05, 19 },
    { 0x2eae2a06, 76 },
    { 0x787e2a07, 98 },
    { 0x7c1e2a04, 93 },
    { 0x65ce2a09, 78 },
    { 0x25de2a04, 95 },
    { 0x908e2a08, 55 },
    { 0x1be2a0c, 23 },
    { 0, 0 },
    { 0x266e2a0e, 90 },
    { 0x114e2a0f, 87 },
    { 0x165e2a0e, 90 },
    { 0xeb6e2a0f, 46 },
    { 0xd8ae2a0e, 55 },
    { 0xdb1e2a10, 95 },
    { 0x4bbe2a13, 76 },
    { 0x37fe2a15, 95 },
    { 0, 0 },
    { 0x38ee2a17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ce2a1f, 90 },
    { 0xde4e2a20, 98 },
    { 0x665e2a21, 61 },
    { 0x605e2a22, 66 },
    { 0xddfe2a23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efe2a27, 61 },
    { 0, 0 },
    { 0x396e2a29, 46 },
    { 0x94ce2a2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84be2a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2e2a34, 46 },
    { 0xbd7e2a35, 68 },
    { 0x151e2a35, 55 },
    { 0x25ce2a34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfe2a3f, 95 },
    { 0x89ae2a40, 55 },
    { 0xd3e2a40, 65 },
    { 0, 0 },
    { 0xc7be2a43, 61 },
    { 0xde8e2a44, 98 },
    { 0x799e2a45, 91 },
    { 0xd15e2a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeee2a4a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fe2a4e, 90 },
    { 0xfbbe2a4e, 90 },
    { 0x73ee2a50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2e2a54, 98 },
    { 0x897e2a55, 61 },
    { 0xe9de2a56, 76 },
    { 0xee7e2a57, 33 },
    { 0xf0ee2a57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e2a5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfe2a60, 33 },
    { 0, 0 },
    { 0x656e2a62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5e2a6b, 43 },
    { 0xa27e2a6c, 100 },
    { 0x6d3e2a6b, 33 },
    { 0xde0e2a6d, 90 },
    { 0x521e2a6f, 34 },
    { 0xf94e2a6f, 94 },
    { 0, 0 },
    { 0xfe9e2a72, 100 },
    { 0xbdee2a72, 90 },
    { 0x645e2a74, 90 },
    { 0x8b5e2a73, 90 },
    { 0x414e2a74, 98 },
    { 0x931e2a77, 90 },
    { 0xb31e2a77, 33 },
    { 0, 0 },
    { 0x5f4e2a7a, 23 },
    { 0, 0 },
    { 0x917e2a7c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e2a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ece2a8b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3e2a8e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe2a93, 90 },
    { 0xbe7e2a94, 90 },
    { 0x484e2a95, 95 },
    { 0, 0 },
    { 0x8fbe2a97, 76 },
    { 0x23e2a98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade2a9c, 90 },
    { 0, 0 },
    { 0xbece2a9e, 90 },
    { 0, 0 },
    { 0xf14e2aa0, 61 },
    { 0x1f1e2aa1, 46 },
    { 0x71be2aa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91be2aa7, 78 },
    { 0x632e2aa7, 90 },
    { 0xc76e2aa9, 100 },
    { 0, 0 },
    { 0x103e2aab, 90 },
    { 0x98fe2aab, 67 },
    { 0x9ace2aad, 90 },
    { 0, 0 },
    { 0xf94e2aaf, 43 },
    { 0x57ce2ab0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ee2ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5e2ab6, 69 },
    { 0x77be2ab6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20e2ac0, 61 },
    { 0, 0 },
    { 0xaafe2ac2, 43 },
    { 0, 0 },
    { 0x3c3e2ac4, 40 },
    { 0, 0 },
    { 0x7bce2ac6, 34 },
    { 0, 0 },
    { 0x2d3e2ac8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686e2acd, 61 },
    { 0x87ee2ace, 34 },
    { 0x210e2acf, 34 },
    { 0, 0 },
    { 0x468e2ad1, 61 },
    { 0x478e2ad2, 23 },
    { 0x211e2ad3, 99 },
    { 0xb32e2ad1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38be2add, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x154e2ae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x500e2ae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9e2ae9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x270e2aec, 95 },
    { 0xab0e2aed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81e2af0, 33 },
    { 0x328e2af0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e2b01, 34 },
    { 0, 0 },
    { 0x406e2b03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xddee2b06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567e2b0f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751e2b14, 76 },
    { 0xd5fe2b14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x742e2b1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87e2b20, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40de2b24, 99 },
    { 0x182e2b24, 23 },
    { 0x228e2b25, 67 },
    { 0x57ae2b27, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e2b31, 76 },
    { 0, 0 },
    { 0x768e2b33, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe2b36, 83 },
    { 0x11e2b37, 34 },
    { 0x6d8e2b38, 23 },
    { 0x9e6e2b38, 90 },
    { 0x7f7e2b3a, 84 },
    { 0xeece2b3a, 90 },
    { 0, 0 },
    { 0x5f1e2b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dde2b44, 95 },
    { 0x7e8e2b45, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e2b4b, 23 },
    { 0x4ebe2b4b, 84 },
    { 0x91ce2b4d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977e2b52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efe2b57, 90 },
    { 0xe8ee2b58, 93 },
    { 0xd26e2b59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ce2b5c, 81 },
    { 0xf9ae2b5d, 81 },
    { 0x27be2b5c, 98 },
    { 0x44ee2b5f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e2b62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ee2b65, 90 },
    { 0, 0 },
    { 0x93e2b67, 90 },
    { 0x68be2b68, 85 },
    { 0xecee2b68, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9e2b70, 8 },
    { 0, 0 },
    { 0x73be2b72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608e2b79, 95 },
    { 0, 0 },
    { 0x727e2b7b, 90 },
    { 0xf82e2b7c, 98 },
    { 0, 0 },
    { 0x76ce2b7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e2b82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31e2b88, 33 },
    { 0x3fbe2b89, 23 },
    { 0, 0 },
    { 0x86fe2b8b, 23 },
    { 0x9fae2b8c, 90 },
    { 0x472e2b8d, 68 },
    { 0xffbe2b8e, 69 },
    { 0x821e2b8f, 95 },
    { 0x55de2b8d, 98 },
    { 0xf18e2b91, 33 },
    { 0x513e2b8b, 95 },
    { 0x30ee2b93, 88 },
    { 0x6b9e2b93, 23 },
    { 0, 0 },
    { 0x504e2b96, 98 },
    { 0x449e2b96, 33 },
    { 0xa2be2b98, 33 },
    { 0xc30e2b98, 61 },
    { 0x9cce2b9a, 34 },
    { 0x2dbe2b9b, 66 },
    { 0x231e2b9c, 61 },
    { 0xb04e2b9d, 95 },
    { 0xdd2e2b9e, 43 },
    { 0x587e2b9f, 33 },
    { 0x383e2ba0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9be2ba9, 99 },
    { 0x137e2baa, 43 },
    { 0, 0 },
    { 0x17fe2bac, 90 },
    { 0x4be2bad, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e2bb0, 34 },
    { 0, 0 },
    { 0xf41e2bb2, 34 },
    { 0, 0 },
    { 0x89de2bb4, 33 },
    { 0x24ce2bb5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e2bb9, 55 },
    { 0, 0 },
    { 0x521e2bbb, 46 },
    { 0x576e2bbc, 61 },
    { 0x752e2bbc, 61 },
    { 0xc0ce2bbe, 101 },
    { 0x99ce2bbd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e2bc6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fe2bca, 18 },
    { 0, 0 },
    { 0xb7de2bcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361e2bdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbce2be2, 61 },
    { 0xaee2be3, 96 },
    { 0, 0 },
    { 0x3f0e2be5, 98 },
    { 0x523e2be5, 33 },
    { 0x1ade2be6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de2bf2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52de2bf9, 23 },
    { 0x142e2bfa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24de2c00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65de2c0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fe2c0d, 95 },
    { 0x727e2c0d, 95 },
    { 0x869e2c0f, 76 },
    { 0x2f4e2c10, 43 },
    { 0xdbee2c11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61e2c1a, 90 },
    { 0, 0 },
    { 0x88ee2c1c, 90 },
    { 0, 0 },
    { 0x91ce2c1e, 98 },
    { 0xb1de2c1f, 100 },
    { 0x83ce2c1f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe2c27, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2e2c2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28e2c33, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592e2c38, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56e2c3b, 61 },
    { 0xeeae2c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6e2c40, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c7e2c48, 33 },
    { 0xd5be2c49, 95 },
    { 0x31e2c4a, 90 },
    { 0x3cae2c4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e2c50, 17 },
    { 0x1e4e2c51, 61 },
    { 0x147e2c50, 33 },
    { 0x28ce2c52, 61 },
    { 0, 0 },
    { 0x585e2c55, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3e2c58, 76 },
    { 0x637e2c58, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x664e2c5c, 85 },
    { 0, 0 },
    { 0xf2ee2c5e, 34 },
    { 0, 0 },
    { 0xb10e2c60, 81 },
    { 0xb3de2c61, 52 },
    { 0xbae2c62, 90 },
    { 0xf05e2c62, 43 },
    { 0x961e2c64, 99 },
    { 0x9f2e2c63, 23 },
    { 0x9fee2c66, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e2c6b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42e2c6f, 88 },
    { 0x6ee2c70, 90 },
    { 0x844e2c71, 42 },
    { 0xb6ae2c72, 66 },
    { 0x73ce2c6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4e2c77, 76 },
    { 0xad2e2c78, 84 },
    { 0x506e2c78, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93e2c7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aee2c83, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fde2c86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e2c89, 43 },
    { 0x194e2c8a, 61 },
    { 0x93de2c8b, 55 },
    { 0x225e2c8a, 75 },
    { 0, 0 },
    { 0xd5ae2c8e, 67 },
    { 0x49de2c8f, 34 },
    { 0x23ae2c8f, 43 },
    { 0xaaae2c91, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496e2c95, 76 },
    { 0x49be2c96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ae2c99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745e2c9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ae2ca3, 98 },
    { 0, 0 },
    { 0x712e2ca5, 55 },
    { 0x488e2ca6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cee2cad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5e2cb3, 90 },
    { 0xb3e2cb3, 33 },
    { 0x7c9e2cb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412e2cb9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962e2cc1, 23 },
    { 0xe59e2cc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x52be2cc5, 90 },
    { 0xa54e2cc5, 98 },
    { 0x405e2cc5, 43 },
    { 0xd31e2cc6, 90 },
    { 0, 0 },
    { 0xf6ae2cca, 76 },
    { 0x627e2ccb, 90 },
    { 0xa6de2ccc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fe2cda, 98 },
    { 0xcd1e2cdb, 61 },
    { 0xb66e2cdb, 90 },
    { 0, 0 },
    { 0x711e2cde, 95 },
    { 0, 0 },
    { 0x328e2ce0, 98 },
    { 0, 0 },
    { 0xf08e2ce2, 23 },
    { 0, 0 },
    { 0x183e2ce4, 90 },
    { 0x707e2ce4, 99 },
    { 0, 0 },
    { 0xe1e2ce7, 95 },
    { 0x1cfe2ce8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390e2ced, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcce2cf4, 61 },
    { 0, 0 },
    { 0x3fe2cf6, 33 },
    { 0xaabe2cf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60e2cfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde2d07, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78e2d0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57e2d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27e2d20, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615e2d26, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e2d2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482e2d35, 91 },
    { 0xcb8e2d35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce2d39, 90 },
    { 0x33fe2d3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8e2d3e, 77 },
    { 0x239e2d3e, 99 },
    { 0, 0 },
    { 0xd9ee2d41, 23 },
    { 0xce0e2d42, 95 },
    { 0xfe5e2d41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8e2d48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e2d4b, 33 },
    { 0, 0 },
    { 0xca2e2d4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfe2d53, 98 },
    { 0, 0 },
    { 0xb50e2d55, 90 },
    { 0x69ce2d56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x487e2d59, 90 },
    { 0, 0 },
    { 0xfbce2d5b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ace2d62, 98 },
    { 0x4fae2d63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e2d68, 52 },
    { 0, 0 },
    { 0xa7de2d6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x221e2d6d, 91 },
    { 0, 0 },
    { 0xad1e2d6f, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e2d79, 23 },
    { 0x73e2d79, 80 },
    { 0, 0 },
    { 0xe4ae2d7c, 100 },
    { 0xa7ae2d7d, 33 },
    { 0xdfbe2d7d, 33 },
    { 0x43ee2d7f, 90 },
    { 0xd56e2d7f, 90 },
    { 0xbfe2d80, 90 },
    { 0x632e2d82, 58 },
    { 0xd6ce2d83, 61 },
    { 0xa03e2d84, 78 },
    { 0x62ce2d83, 23 },
    { 0xf51e2d86, 93 },
    { 0xe5ae2d86, 33 },
    { 0x544e2d87, 95 },
    { 0x604e2d89, 34 },
    { 0x448e2d83, 38 },
    { 0xc4fe2d8b, 34 },
    { 0x9a3e2d8c, 52 },
    { 0x764e2d8b, 36 },
    { 0x73ce2d8d, 61 },
    { 0x2e0e2d80, 43 },
    { 0x1c5e2d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4e2d95, 33 },
    { 0xc3ce2d96, 33 },
    { 0xe92e2d96, 43 },
    { 0x8c0e2d97, 34 },
    { 0x4a2e2d99, 87 },
    { 0xec2e2d97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e6e2d9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03e2da3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7e2da7, 76 },
    { 0x126e2da8, 90 },
    { 0xbd4e2da7, 99 },
    { 0xbf1e2daa, 84 },
    { 0xeece2daa, 34 },
    { 0, 0 },
    { 0x1c4e2dad, 66 },
    { 0xba3e2dae, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7be2db1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543e2db5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ce2dbd, 87 },
    { 0xc13e2dbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e2dca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e2dcd, 55 },
    { 0, 0 },
    { 0xdee2dcf, 98 },
    { 0x39fe2dcf, 33 },
    { 0xcd4e2dd1, 65 },
    { 0, 0 },
    { 0x6f9e2dd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e2dd9, 33 },
    { 0x649e2dda, 46 },
    { 0x9aee2ddb, 77 },
    { 0, 0 },
    { 0x40ae2ddd, 90 },
    { 0, 0 },
    { 0x473e2ddf, 95 },
    { 0x843e2de0, 56 },
    { 0xd19e2de0, 61 },
    { 0x36ae2de2, 90 },
    { 0x7ade2ddf, 55 },
    { 0x7be2de1, 46 },
    { 0, 0 },
    { 0x5d4e2de6, 23 },
    { 0x19fe2de6, 46 },
    { 0xa6ae2de7, 67 },
    { 0x370e2de7, 61 },
    { 0xd9be2de9, 95 },
    { 0, 0 },
    { 0x300e2dec, 91 },
    { 0x7dee2ded, 87 },
    { 0x9cde2ded, 98 },
    { 0x2fde2def, 87 },
    { 0x385e2dec, 99 },
    { 0, 0 },
    { 0x2b0e2df2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e2df6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e2dfd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ae2e03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ce2e06, 99 },
    { 0, 0 },
    { 0x26ee2e08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4e2e0e, 61 },
    { 0xabde2e0f, 33 },
    { 0x4fe2e0e, 90 },
    { 0xe19e2e0e, 90 },
    { 0x37e2e10, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31e2e16, 90 },
    { 0xf82e2e17, 98 },
    { 0x3b1e2e17, 87 },
    { 0x4d3e2e19, 94 },
    { 0x4abe2e1a, 90 },
    { 0xf96e2e1a, 98 },
    { 0x73ee2e1b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x313e2e25, 90 },
    { 0, 0 },
    { 0xd32e2e27, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccce2e2f, 76 },
    { 0x943e2e30, 90 },
    { 0x9b3e2e31, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x952e2e34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5e2e3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e2e41, 93 },
    { 0xea6e2e42, 87 },
    { 0x7b7e2e43, 96 },
    { 0, 0 },
    { 0x681e2e45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8e2e48, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fe2e57, 23 },
    { 0xebe2e58, 33 },
    { 0x284e2e59, 43 },
    { 0x627e2e5a, 23 },
    { 0x13e2e59, 85 },
    { 0x30ee2e5b, 80 },
    { 0xe7be2e5b, 67 },
    { 0x41ae2e5e, 61 },
    { 0xf57e2e5a, 95 },
    { 0xa98e2e60, 96 },
    { 0xe7ae2e61, 76 },
    { 0x503e2e61, 98 },
    { 0x6e0e2e60, 66 },
    { 0x80ae2e5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffe2e67, 43 },
    { 0x941e2e68, 55 },
    { 0xa60e2e69, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e2e6c, 76 },
    { 0, 0 },
    { 0xfc8e2e6e, 61 },
    { 0x8e8e2e6f, 61 },
    { 0x965e2e6e, 69 },
    { 0, 0 },
    { 0xc53e2e72, 2 },
    { 0, 0 },
    { 0x142e2e74, 43 },
    { 0x54be2e75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aee2e7e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e2e82, 33 },
    { 0xaf7e2e82, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9e2e86, 33 },
    { 0, 0 },
    { 0xf74e2e88, 84 },
    { 0x504e2e88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fe2e8d, 33 },
    { 0, 0 },
    { 0x669e2e8f, 76 },
    { 0x3a4e2e8f, 46 },
    { 0xfd7e2e90, 61 },
    { 0xf76e2e92, 33 },
    { 0x37fe2e8f, 90 },
    { 0xbc6e2e94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be2e98, 47 },
    { 0x2e7e2e99, 95 },
    { 0, 0 },
    { 0x1e2e2e9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310e2ea1, 23 },
    { 0x350e2ea2, 95 },
    { 0xa9fe2ea3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6e2eaa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8e2eae, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712e2eb5, 23 },
    { 0x568e2eb6, 90 },
    { 0xb70e2eb7, 95 },
    { 0xc30e2eb8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e2ec0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39e2ec5, 90 },
    { 0xe1be2ec6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fae2ec9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e2ecc, 33 },
    { 0x8a7e2ecc, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09e2ed2, 43 },
    { 0x69ce2ed3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e2eda, 94 },
    { 0, 0 },
    { 0x20fe2edc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41e2ee0, 76 },
    { 0, 0 },
    { 0xba0e2ee2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeee2ee5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1e2ee9, 23 },
    { 0x4d7e2eea, 85 },
    { 0, 0 },
    { 0xd10e2eec, 93 },
    { 0x5cfe2eed, 85 },
    { 0xadee2eed, 33 },
    { 0xb01e2eef, 46 },
    { 0xfbde2ef0, 34 },
    { 0x47e2eee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59e2ef7, 98 },
    { 0, 0 },
    { 0xd97e2ef9, 98 },
    { 0, 0 },
    { 0xd96e2efb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ece2efe, 68 },
    { 0xe90e2eff, 76 },
    { 0x5c4e2eff, 95 },
    { 0x409e2f01, 90 },
    { 0, 0 },
    { 0xb0e2f03, 93 },
    { 0, 0 },
    { 0x148e2f05, 60 },
    { 0, 0 },
    { 0xc75e2f07, 52 },
    { 0x1e0e2f08, 55 },
    { 0x4c1e2f07, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04e2f0e, 77 },
    { 0xdf5e2f0f, 33 },
    { 0x49fe2f10, 34 },
    { 0xaace2f11, 80 },
    { 0x46de2f12, 90 },
    { 0xb9ce2f13, 95 },
    { 0x960e2f13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7e2f1a, 34 },
    { 0, 0 },
    { 0xb21e2f1c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6e2f20, 90 },
    { 0xa50e2f21, 91 },
    { 0x174e2f22, 96 },
    { 0x87be2f21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26e2f27, 33 },
    { 0x3dee2f27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ae2f34, 23 },
    { 0x9d0e2f35, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88be2f3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x375e2f3d, 67 },
    { 0x747e2f3e, 76 },
    { 0xe1ee2f3f, 51 },
    { 0x186e2f3d, 23 },
    { 0xce7e2f41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94e2f47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2e2f4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ce2f52, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31de2f58, 61 },
    { 0x28ae2f59, 23 },
    { 0x5ace2f5a, 90 },
    { 0x649e2f58, 98 },
    { 0, 0 },
    { 0x279e2f5d, 91 },
    { 0, 0 },
    { 0x7aee2f5f, 61 },
    { 0xb8fe2f5f, 66 },
    { 0, 0 },
    { 0xd11e2f62, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bae2f6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3e2f76, 78 },
    { 0xf7ce2f76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafde2f7c, 98 },
    { 0x595e2f7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28e2f84, 95 },
    { 0x422e2f85, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5e2f8c, 61 },
    { 0, 0 },
    { 0xd06e2f8e, 76 },
    { 0x3a4e2f8f, 76 },
    { 0xfcee2f90, 64 },
    { 0x894e2f91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26e2f94, 81 },
    { 0x112e2f95, 33 },
    { 0, 0 },
    { 0x735e2f97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e2f9d, 98 },
    { 0xf80e2f9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfce2fa8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be2fb3, 23 },
    { 0, 0 },
    { 0x1dce2fb5, 48 },
    { 0x430e2fb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x223e2fb9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ce2fbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91de2fc3, 90 },
    { 0xc98e2fc4, 52 },
    { 0x666e2fc5, 78 },
    { 0x1dce2fc4, 90 },
    { 0x6ade2fc4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ce2fcf, 34 },
    { 0, 0 },
    { 0x328e2fd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x405e2fd4, 36 },
    { 0xec9e2fd5, 93 },
    { 0xd82e2fd6, 91 },
    { 0xcf5e2fd7, 40 },
    { 0x2f8e2fd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349e2fe0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ce2fe3, 55 },
    { 0x467e2fe4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dde2fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74e2ff0, 23 },
    { 0x195e2ff1, 96 },
    { 0x2b8e2ff2, 90 },
    { 0x9b4e2ff1, 33 },
    { 0, 0 },
    { 0x330e2ff5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685e3006, 76 },
    { 0xa4ee3006, 33 },
    { 0x287e3008, 66 },
    { 0, 0 },
    { 0xb16e300a, 74 },
    { 0xe76e300b, 43 },
    { 0xf56e300c, 61 },
    { 0xe6e300c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102e301c, 75 },
    { 0, 0 },
    { 0xe7ae301e, 68 },
    { 0x7a1e301f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15e3022, 23 },
    { 0xda3e3023, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade3027, 67 },
    { 0, 0 },
    { 0x101e3029, 93 },
    { 0, 0 },
    { 0x92ae302b, 33 },
    { 0x8f2e302c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fe3032, 23 },
    { 0xfc9e3032, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe3036, 88 },
    { 0xa01e3036, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2e303e, 33 },
    { 0, 0 },
    { 0xb20e3040, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5e3047, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250e304c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e304f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79de3056, 90 },
    { 0, 0 },
    { 0x73ee3058, 90 },
    { 0x7fce3058, 94 },
    { 0x14ce305a, 90 },
    { 0x76de305b, 67 },
    { 0xb78e3059, 90 },
    { 0, 0 },
    { 0x6e4e305e, 88 },
    { 0xc76e305f, 87 },
    { 0, 0 },
    { 0x755e3061, 23 },
    { 0x9f0e3062, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eee3068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3e306c, 33 },
    { 0xa2ce306c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ee3071, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffce3075, 38 },
    { 0, 0 },
    { 0x8d2e3077, 34 },
    { 0x8c4e3078, 95 },
    { 0x350e3078, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58be307f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590e3084, 98 },
    { 0, 0 },
    { 0xf3fe3086, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676e3091, 66 },
    { 0x440e3092, 61 },
    { 0xbdae3091, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ce3097, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee30a7, 61 },
    { 0x9ace30a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3e30af, 33 },
    { 0xb61e30b0, 61 },
    { 0x15e30b1, 77 },
    { 0xe58e30b0, 98 },
    { 0x4b6e30b3, 53 },
    { 0, 0 },
    { 0xcf9e30b5, 90 },
    { 0, 0 },
    { 0x837e30b7, 33 },
    { 0x6fde30b8, 55 },
    { 0xae7e30b9, 90 },
    { 0xdeee30ba, 37 },
    { 0x682e30ba, 90 },
    { 0x7f2e30ba, 61 },
    { 0x3e2e30bc, 61 },
    { 0, 0 },
    { 0xc2de30bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x726e30c8, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae30cc, 43 },
    { 0xa4ae30cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e30d0, 80 },
    { 0x998e30d1, 95 },
    { 0x8a5e30d2, 23 },
    { 0x5bae30d3, 52 },
    { 0x3b9e30d2, 90 },
    { 0x3aee30d2, 33 },
    { 0, 0 },
    { 0x7f2e30d7, 33 },
    { 0x879e30d7, 95 },
    { 0, 0 },
    { 0xbd4e30da, 61 },
    { 0x6cbe30da, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8e30df, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dee30e2, 93 },
    { 0x7dde30e3, 90 },
    { 0xa6de30e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ce30ea, 33 },
    { 0x32ee30eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e30ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ee30f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ee30f8, 33 },
    { 0xd8de30f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99e30fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0e3101, 55 },
    { 0, 0 },
    { 0xe3ce3103, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fe3108, 61 },
    { 0xca5e3109, 76 },
    { 0xbd1e3108, 68 },
    { 0xd8e310b, 43 },
    { 0xed7e310b, 77 },
    { 0, 0 },
    { 0x869e310e, 46 },
    { 0xc99e310e, 90 },
    { 0, 0 },
    { 0x603e3111, 43 },
    { 0x701e3112, 95 },
    { 0, 0 },
    { 0x295e3114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8e311b, 33 },
    { 0xc9ce311c, 90 },
    { 0, 0 },
    { 0x416e311e, 66 },
    { 0, 0 },
    { 0xd9de3120, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe3124, 61 },
    { 0x182e3125, 95 },
    { 0, 0 },
    { 0xb11e3127, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e312a, 34 },
    { 0, 0 },
    { 0x72ae312c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138e3133, 91 },
    { 0x6d4e3134, 23 },
    { 0, 0 },
    { 0x192e3136, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6e3139, 76 },
    { 0, 0 },
    { 0x36ae313b, 95 },
    { 0, 0 },
    { 0x519e313d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace3142, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162e3148, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ae314c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x422e314f, 100 },
    { 0, 0 },
    { 0x935e3151, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ee3154, 61 },
    { 0, 0 },
    { 0xffbe3156, 95 },
    { 0xe93e3157, 93 },
    { 0x57be3158, 90 },
    { 0x3f0e3158, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15de315f, 61 },
    { 0, 0 },
    { 0x68e3161, 95 },
    { 0, 0 },
    { 0xebbe3163, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ae3166, 90 },
    { 0x6d0e3166, 76 },
    { 0x3aae3168, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9e316b, 90 },
    { 0xe28e316c, 98 },
    { 0xad9e316c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357e3172, 61 },
    { 0, 0 },
    { 0xdb2e3174, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fde3177, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecce317b, 34 },
    { 0xea1e317c, 99 },
    { 0, 0 },
    { 0x3b8e317e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994e3182, 33 },
    { 0, 0 },
    { 0x2f7e3184, 66 },
    { 0x92de3185, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8e318a, 98 },
    { 0, 0 },
    { 0x146e318c, 76 },
    { 0, 0 },
    { 0xc84e318e, 100 },
    { 0x71ee318f, 90 },
    { 0x362e3190, 95 },
    { 0x12be3190, 46 },
    { 0x656e3190, 23 },
    { 0x9cce318e, 76 },
    { 0x555e3194, 95 },
    { 0x7a6e3194, 61 },
    { 0x2fce3191, 95 },
    { 0xf2be3193, 90 },
    { 0, 0 },
    { 0xb1ae3199, 67 },
    { 0x278e319a, 98 },
    { 0x321e319b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9e31a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e31a6, 43 },
    { 0x786e31a7, 77 },
    { 0xf6be31a8, 92 },
    { 0xa62e31a7, 61 },
    { 0, 0 },
    { 0xe76e31ab, 66 },
    { 0x7b4e31ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ae31b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be31b6, 61 },
    { 0x8de31b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89be31bc, 61 },
    { 0x315e31bd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e31c6, 33 },
    { 0x72ae31c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58fe31cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e31ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ce31da, 43 },
    { 0xdbce31db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e31df, 100 },
    { 0, 0 },
    { 0xcdfe31e1, 34 },
    { 0x97fe31e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ce31e5, 61 },
    { 0, 0 },
    { 0xd0e31e7, 98 },
    { 0x3bfe31e7, 61 },
    { 0xa39e31e8, 33 },
    { 0, 0 },
    { 0x13ee31eb, 66 },
    { 0x759e31eb, 61 },
    { 0x799e31ed, 55 },
    { 0x5c7e31ed, 95 },
    { 0xe2ee31ee, 55 },
    { 0x82e31eb, 100 },
    { 0x640e31f1, 69 },
    { 0x5ee31f2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fe31f5, 84 },
    { 0, 0 },
    { 0x216e31f7, 61 },
    { 0xf0fe31f8, 23 },
    { 0x302e31f9, 68 },
    { 0x6a3e31f7, 40 },
    { 0, 0 },
    { 0xf63e31fc, 90 },
    { 0x7ebe31fd, 76 },
    { 0x42ee31fe, 95 },
    { 0xda0e31fc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357e3207, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0xb03e320a, 33 },
    { 0, 0 },
    { 0x428e320c, 61 },
    { 0x245e320d, 93 },
    { 0xe36e320e, 23 },
    { 0xe52e320e, 84 },
    { 0x750e320e, 95 },
    { 0x109e3211, 88 },
    { 0xe64e3212, 43 },
    { 0, 0 },
    { 0x349e3214, 52 },
    { 0x7cce3214, 40 },
    { 0xf9ee3216, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7e321a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4e3220, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x719e3223, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xabde3226, 46 },
    { 0x220e3227, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6e322b, 33 },
    { 0x201e322c, 23 },
    { 0xc70e322d, 93 },
    { 0, 0 },
    { 0xb19e322f, 66 },
    { 0x2cbe3230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e323d, 34 },
    { 0, 0 },
    { 0x9cae323f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebe3244, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2be3248, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbe3250, 88 },
    { 0xf64e3250, 94 },
    { 0, 0 },
    { 0x78fe3253, 60 },
    { 0x97fe3254, 98 },
    { 0, 0 },
    { 0xa62e3256, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e325b, 90 },
    { 0, 0 },
    { 0xc16e325d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8e3261, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05e3264, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d5e3268, 95 },
    { 0x33ae3268, 61 },
    { 0xae8e326a, 76 },
    { 0x568e326a, 85 },
    { 0x80ce326c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08e3270, 99 },
    { 0x1dae3271, 93 },
    { 0, 0 },
    { 0xdae3273, 95 },
    { 0x3d1e3273, 84 },
    { 0, 0 },
    { 0xf95e3276, 33 },
    { 0, 0 },
    { 0xf4ce3278, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fe3280, 76 },
    { 0x31e3281, 87 },
    { 0xf29e3281, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ede3287, 90 },
    { 0xd61e3288, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafce328d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ee3291, 95 },
    { 0xf02e3291, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98e3295, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ae329b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6be329f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20be32a5, 33 },
    { 0, 0 },
    { 0x445e32a7, 33 },
    { 0xa67e32a8, 81 },
    { 0x1c7e32a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8e32b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19de32b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861e32c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ae32ca, 33 },
    { 0x782e32cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e32d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaade32e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01e32e9, 95 },
    { 0, 0 },
    { 0x5f4e32eb, 90 },
    { 0x558e32ec, 46 },
    { 0, 0 },
    { 0x1e9e32ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x998e32f2, 17 },
    { 0x562e32f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e32fb, 93 },
    { 0xc2ce32fb, 84 },
    { 0xa6be32fd, 61 },
    { 0x13e32fe, 90 },
    { 0, 0 },
    { 0x5d2e3300, 40 },
    { 0x69ae3301, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17fe3304, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30e330b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e330e, 93 },
    { 0x1b0e330e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe3314, 80 },
    { 0xefee3315, 4 },
    { 0, 0 },
    { 0x90ce3317, 93 },
    { 0x516e3317, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cee331b, 23 },
    { 0x977e331b, 98 },
    { 0xea3e331d, 76 },
    { 0, 0 },
    { 0xecce331f, 61 },
    { 0x84ce331f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8be3329, 91 },
    { 0xe5ee332a, 76 },
    { 0x5a5e332b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807e332f, 33 },
    { 0, 0 },
    { 0xea9e3331, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e3335, 17 },
    { 0, 0 },
    { 0x470e3337, 95 },
    { 0, 0 },
    { 0x504e3339, 43 },
    { 0xce6e3339, 55 },
    { 0xe16e3339, 93 },
    { 0, 0 },
    { 0xa8ae333d, 61 },
    { 0x8dae333e, 23 },
    { 0xdafe333e, 98 },
    { 0xdf6e333e, 33 },
    { 0xb3de3341, 61 },
    { 0, 0 },
    { 0xa88e3343, 34 },
    { 0xc51e3344, 98 },
    { 0x8fde3345, 23 },
    { 0xc23e3346, 95 },
    { 0, 0 },
    { 0xa89e3348, 85 },
    { 0xadee3349, 56 },
    { 0x489e3349, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7e334d, 60 },
    { 0xa5fe334e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e3352, 33 },
    { 0x5a7e3353, 61 },
    { 0x4ae3352, 55 },
    { 0x76fe3354, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecae335b, 80 },
    { 0, 0 },
    { 0x2b1e335d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x29de3360, 76 },
    { 0, 0 },
    { 0xb5e3362, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6e3369, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bee336e, 95 },
    { 0xeb9e336e, 95 },
    { 0x8a8e336f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4e3374, 61 },
    { 0x14e3375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e3379, 93 },
    { 0, 0 },
    { 0x251e337b, 33 },
    { 0x851e337c, 66 },
    { 0, 0 },
    { 0xd7be337e, 61 },
    { 0x6dae337e, 23 },
    { 0xfdde3380, 23 },
    { 0x490e337e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fe3387, 90 },
    { 0x73ce3387, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41e338b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e3392, 23 },
    { 0, 0 },
    { 0x86fe3394, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce339b, 33 },
    { 0x5d6e339b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3de339f, 33 },
    { 0xd20e33a0, 97 },
    { 0x56fe33a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x294e33a4, 95 },
    { 0xa08e33a5, 61 },
    { 0x7bce33a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6e33ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9e33b3, 33 },
    { 0x118e33b4, 95 },
    { 0x35fe33b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e33b8, 76 },
    { 0xe39e33b8, 33 },
    { 0x494e33ba, 61 },
    { 0, 0 },
    { 0x9b2e33bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68e33c1, 90 },
    { 0x2f8e33c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fe33c7, 33 },
    { 0, 0 },
    { 0x395e33c9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141e33ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e33d2, 43 },
    { 0, 0 },
    { 0x690e33d4, 67 },
    { 0x4cfe33d5, 61 },
    { 0xb7ce33d6, 90 },
    { 0x50ce33d7, 66 },
    { 0x8f1e33d8, 33 },
    { 0x686e33d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x158e33dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58be33e1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e33e5, 61 },
    { 0x3c0e33e5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2e33e9, 98 },
    { 0, 0 },
    { 0x734e33eb, 61 },
    { 0xfd4e33eb, 95 },
    { 0x1bae33ed, 61 },
    { 0xa17e33ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde33f2, 84 },
    { 0x2d1e33f3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27e33f9, 33 },
    { 0, 0 },
    { 0xf3ee33fb, 75 },
    { 0x544e33fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19e33ff, 33 },
    { 0xce6e3400, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e3404, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939e340e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x745e3411, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5e341a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce3422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64e342a, 46 },
    { 0, 0 },
    { 0x9f8e342c, 69 },
    { 0xa47e342d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64e3434, 99 },
    { 0xc19e3434, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4e3438, 61 },
    { 0x30e3439, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0e3441, 93 },
    { 0x1c2e3442, 91 },
    { 0x39de3441, 33 },
    { 0x2f8e3441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x289e3447, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7e344b, 61 },
    { 0x494e344c, 76 },
    { 0xb5fe344d, 90 },
    { 0x457e344e, 76 },
    { 0xa3de344c, 43 },
    { 0, 0 },
    { 0xe2be3451, 81 },
    { 0xe9de3451, 23 },
    { 0xbd6e3453, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202e3458, 85 },
    { 0, 0 },
    { 0x387e345a, 66 },
    { 0xfe1e345b, 95 },
    { 0x390e345c, 33 },
    { 0xdabe345d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1e3461, 55 },
    { 0xd4ee3461, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2e3469, 90 },
    { 0, 0 },
    { 0x3d5e346b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14e346f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e3472, 90 },
    { 0xebae3473, 90 },
    { 0x37de3474, 64 },
    { 0x9cfe3473, 78 },
    { 0, 0 },
    { 0xefe3477, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703e347e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ce3481, 95 },
    { 0x4c5e3482, 23 },
    { 0xdcfe3483, 98 },
    { 0, 0 },
    { 0xbebe3485, 23 },
    { 0, 0 },
    { 0x3fee3487, 95 },
    { 0xbb6e3488, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1e348b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdee3491, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2e3494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e3498, 84 },
    { 0xff1e3499, 95 },
    { 0xecbe349a, 55 },
    { 0x912e349b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e34a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae34ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e34af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7e34b3, 23 },
    { 0, 0 },
    { 0x25be34b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65e34b8, 33 },
    { 0xff9e34b9, 61 },
    { 0, 0 },
    { 0xe14e34bb, 34 },
    { 0xc7ce34bb, 34 },
    { 0x381e34bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4e34c7, 78 },
    { 0, 0 },
    { 0x780e34c9, 77 },
    { 0x3d5e34c9, 90 },
    { 0x1be34c9, 33 },
    { 0x9f1e34cc, 76 },
    { 0xfa0e34cc, 61 },
    { 0x3e0e34cc, 66 },
    { 0, 0 },
    { 0xb87e34d0, 61 },
    { 0xd05e34d0, 98 },
    { 0x9a2e34d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ade34da, 68 },
    { 0x2c9e34db, 78 },
    { 0, 0 },
    { 0xbd6e34dd, 90 },
    { 0x1ece34de, 95 },
    { 0x1e2e34df, 90 },
    { 0x6c0e34e0, 61 },
    { 0x236e34e1, 95 },
    { 0x78e34e0, 52 },
    { 0x629e34e3, 46 },
    { 0, 0 },
    { 0x1d7e34e5, 90 },
    { 0x1a3e34e5, 33 },
    { 0x7a7e34e6, 98 },
    { 0xf3e34e5, 93 },
    { 0xe1e34e5, 84 },
    { 0x7ebe34e8, 100 },
    { 0xe00e34eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e34ee, 69 },
    { 0xe17e34ef, 90 },
    { 0, 0 },
    { 0xdffe34f1, 87 },
    { 0xed7e34f1, 91 },
    { 0x755e34f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eae34f6, 66 },
    { 0x7c6e34f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaede34fb, 90 },
    { 0, 0 },
    { 0xc4ce34fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37e3501, 93 },
    { 0x8ee3501, 90 },
    { 0x6c9e3503, 55 },
    { 0xd8fe3504, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272e3508, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e350b, 95 },
    { 0xa64e350c, 81 },
    { 0x103e350b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcee3512, 98 },
    { 0x9a8e3512, 90 },
    { 0, 0 },
    { 0xdcee3515, 46 },
    { 0x7d2e3516, 53 },
    { 0x35de3515, 100 },
    { 0xe62e3515, 90 },
    { 0xa27e3519, 33 },
    { 0x185e351a, 33 },
    { 0x90fe351a, 61 },
    { 0xb64e351c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60e3520, 90 },
    { 0, 0 },
    { 0x28ae3522, 67 },
    { 0, 0 },
    { 0x921e3524, 23 },
    { 0xe03e3524, 86 },
    { 0x362e3525, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bee3529, 98 },
    { 0, 0 },
    { 0x11ae352b, 61 },
    { 0xf9be352c, 88 },
    { 0x814e352c, 33 },
    { 0xe72e352c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e3531, 48 },
    { 0, 0 },
    { 0x687e3533, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x402e3536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6e3539, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffe353d, 23 },
    { 0x58ce353e, 66 },
    { 0x7d4e353e, 98 },
    { 0, 0 },
    { 0xe6be3541, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2be3545, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x590e3548, 17 },
    { 0, 0 },
    { 0x786e354a, 69 },
    { 0, 0 },
    { 0x663e354c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e354f, 88 },
    { 0x44ee3550, 34 },
    { 0xff1e3551, 93 },
    { 0x803e3552, 98 },
    { 0x4d0e3552, 95 },
    { 0x3fde3554, 33 },
    { 0x550e3553, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35e3559, 33 },
    { 0x7cce3559, 23 },
    { 0, 0 },
    { 0x2de355c, 33 },
    { 0x162e355c, 33 },
    { 0xf19e355e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1e356d, 23 },
    { 0, 0 },
    { 0x63be356f, 90 },
    { 0x5ade3570, 90 },
    { 0x467e3570, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8e3576, 33 },
    { 0x181e3577, 33 },
    { 0, 0 },
    { 0xcc3e3579, 84 },
    { 0xdf9e357a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfe357d, 23 },
    { 0x9c8e357e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce3581, 61 },
    { 0x2e4e3581, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e3589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9e358d, 76 },
    { 0xcbae358d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68de359c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x962e359f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fe35a3, 33 },
    { 0x74fe35a4, 23 },
    { 0xec6e35a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x891e35a8, 55 },
    { 0, 0 },
    { 0x4a5e35aa, 23 },
    { 0x916e35aa, 23 },
    { 0xa2ae35ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9e35b4, 65 },
    { 0xe10e35b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ee35ba, 78 },
    { 0, 0 },
    { 0x732e35bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee35bf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbe35c2, 90 },
    { 0x41de35c3, 76 },
    { 0xadee35c3, 55 },
    { 0xb11e35c5, 61 },
    { 0xf51e35c6, 37 },
    { 0x5abe35c5, 90 },
    { 0xadde35c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fe35ce, 33 },
    { 0xefae35cf, 33 },
    { 0, 0 },
    { 0x11de35d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbe35d4, 84 },
    { 0x49de35d5, 98 },
    { 0, 0 },
    { 0xdc8e35d7, 77 },
    { 0x9c6e35d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1e35db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dee35e0, 76 },
    { 0x320e35e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9e35eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78e35ee, 67 },
    { 0, 0 },
    { 0xee0e35f0, 33 },
    { 0x6ebe35f1, 33 },
    { 0x296e35f2, 85 },
    { 0x9e5e35f2, 76 },
    { 0x2fe35f4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ede35f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e35fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48be3603, 61 },
    { 0x5cae3604, 98 },
    { 0, 0 },
    { 0xa1ae3606, 33 },
    { 0, 0 },
    { 0x400e3608, 90 },
    { 0xd72e3609, 85 },
    { 0xf4ae3608, 67 },
    { 0x7fce3609, 36 },
    { 0xe30e360c, 61 },
    { 0x89ce360d, 61 },
    { 0xc7e360e, 95 },
    { 0xe58e360e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x404e3612, 66 },
    { 0xefe3612, 36 },
    { 0x12fe3613, 61 },
    { 0, 0 },
    { 0xa04e3616, 98 },
    { 0x58ce3617, 76 },
    { 0xc63e3617, 17 },
    { 0xb99e3619, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792e3620, 61 },
    { 0x319e3621, 33 },
    { 0x2dde3621, 99 },
    { 0x2efe3623, 76 },
    { 0, 0 },
    { 0x3cae3625, 78 },
    { 0x1fee3626, 34 },
    { 0x653e3625, 66 },
    { 0x82ee3628, 34 },
    { 0xac3e3628, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfe362f, 19 },
    { 0, 0 },
    { 0x296e3631, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79e3634, 51 },
    { 0, 0 },
    { 0xbdce3636, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffe363b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e3644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fe3648, 36 },
    { 0x33ae3649, 90 },
    { 0xf07e364a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34e364d, 93 },
    { 0x79ae364e, 90 },
    { 0x712e364e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4e3652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111e3656, 33 },
    { 0x8f5e3657, 34 },
    { 0, 0 },
    { 0xd84e3659, 90 },
    { 0xb93e365a, 90 },
    { 0x6a0e3659, 43 },
    { 0, 0 },
    { 0x747e365d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8e3661, 67 },
    { 0, 0 },
    { 0xd3ee3663, 95 },
    { 0xdb0e3664, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bde3667, 34 },
    { 0, 0 },
    { 0xfece3669, 43 },
    { 0xfdce366a, 90 },
    { 0xac2e366a, 95 },
    { 0x398e366a, 34 },
    { 0x995e366d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01e3673, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x540e367c, 87 },
    { 0xb5e367c, 33 },
    { 0xf7ee367e, 38 },
    { 0xec5e367f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdee3682, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xefe3685, 55 },
    { 0, 0 },
    { 0x7eae3687, 52 },
    { 0x7bde3688, 61 },
    { 0x23ce3688, 67 },
    { 0xd31e3689, 78 },
    { 0xd51e3687, 76 },
    { 0xdcce368b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5e3690, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e3694, 33 },
    { 0xbf2e3695, 23 },
    { 0x67be3694, 98 },
    { 0x444e3697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11e36a4, 31 },
    { 0x932e36a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71e36aa, 93 },
    { 0x5dce36ab, 55 },
    { 0x525e36ac, 46 },
    { 0x3c1e36ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7e36b1, 61 },
    { 0xbb9e36b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7e36b7, 23 },
    { 0x9c5e36b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1e36bc, 77 },
    { 0xbcce36bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93e36c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae36ca, 34 },
    { 0, 0 },
    { 0xa8be36cc, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce36d0, 67 },
    { 0x64e36d0, 43 },
    { 0, 0 },
    { 0x21ce36d3, 98 },
    { 0, 0 },
    { 0x65ce36d5, 81 },
    { 0x1c7e36d5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eee36e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e36e3, 98 },
    { 0xb8fe36e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0e36e7, 34 },
    { 0, 0 },
    { 0x8d7e36e9, 93 },
    { 0x16be36ea, 93 },
    { 0xde0e36eb, 17 },
    { 0, 0 },
    { 0x7b4e36ed, 61 },
    { 0, 0 },
    { 0x672e36ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506e36f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e36f7, 61 },
    { 0x3d2e36f8, 33 },
    { 0, 0 },
    { 0xbc5e36fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x882e36fd, 88 },
    { 0x2eae36fe, 98 },
    { 0x645e36ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xbece3702, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76e3705, 90 },
    { 0x766e3706, 77 },
    { 0xdcae3707, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8e370b, 52 },
    { 0x4c1e370c, 90 },
    { 0xb92e370d, 100 },
    { 0x16e370e, 76 },
    { 0x38fe370f, 90 },
    { 0x4ebe370c, 33 },
    { 0xf6be3711, 17 },
    { 0x1ede3712, 91 },
    { 0xbf3e3712, 76 },
    { 0x245e3711, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ce3717, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6be371d, 85 },
    { 0xbcbe371e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3e3721, 33 },
    { 0xe39e3722, 90 },
    { 0xbd6e3723, 93 },
    { 0x28ae3723, 61 },
    { 0x623e3725, 61 },
    { 0x356e3723, 76 },
    { 0xa00e3726, 34 },
    { 0xbf3e3722, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ce372b, 33 },
    { 0x50ae372c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465e3731, 55 },
    { 0x3f8e3732, 84 },
    { 0xfede3731, 55 },
    { 0xbffe3734, 98 },
    { 0, 0 },
    { 0xfe4e3736, 33 },
    { 0x1d6e3736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x819e373a, 33 },
    { 0x293e373b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12e3741, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60e374b, 98 },
    { 0, 0 },
    { 0xc9fe374d, 76 },
    { 0, 0 },
    { 0xa00e374f, 90 },
    { 0x99de3750, 84 },
    { 0x657e3751, 61 },
    { 0xb76e3752, 33 },
    { 0, 0 },
    { 0x458e3754, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3be3765, 43 },
    { 0, 0 },
    { 0x55fe3767, 33 },
    { 0x7a4e3767, 23 },
    { 0xf47e3769, 76 },
    { 0x88fe376a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307e376e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3e3771, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e3775, 66 },
    { 0x6bee3776, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dae3779, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e377c, 98 },
    { 0x71be377d, 34 },
    { 0xa29e377d, 81 },
    { 0x63ee377f, 61 },
    { 0x922e377f, 76 },
    { 0xbe2e377f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fe3789, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43e378f, 98 },
    { 0xdcbe3790, 52 },
    { 0x6d1e3791, 66 },
    { 0xd58e3792, 33 },
    { 0x75ce3793, 84 },
    { 0x131e3794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7e379f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246e37a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830e37ab, 53 },
    { 0x180e37ac, 91 },
    { 0, 0 },
    { 0xb31e37ae, 90 },
    { 0xaa7e37af, 61 },
    { 0x553e37b0, 100 },
    { 0x343e37b0, 33 },
    { 0x8c4e37b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7e37b9, 90 },
    { 0x23ee37ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13e37bd, 95 },
    { 0, 0 },
    { 0xd85e37bf, 33 },
    { 0xa01e37c0, 90 },
    { 0, 0 },
    { 0xe54e37c2, 100 },
    { 0, 0 },
    { 0xb53e37c4, 88 },
    { 0, 0 },
    { 0xdc8e37c6, 90 },
    { 0xe27e37c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e37cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2e37d0, 90 },
    { 0x4a1e37d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e37da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900e37df, 85 },
    { 0x85be37e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ce37e4, 95 },
    { 0, 0 },
    { 0x5e4e37e6, 33 },
    { 0x60fe37e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279e37eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e37ee, 95 },
    { 0x836e37ef, 95 },
    { 0xe66e37ef, 67 },
    { 0x373e37ee, 95 },
    { 0xf96e37f2, 98 },
    { 0xcece37f0, 90 },
    { 0x3a6e37f0, 23 },
    { 0x5c5e37f4, 36 },
    { 0xd5fe37f0, 95 },
    { 0, 0 },
    { 0x511e37f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0e37fb, 46 },
    { 0, 0 },
    { 0xb73e37fd, 17 },
    { 0xdc9e37fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506e3803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e3806, 95 },
    { 0x1c4e3807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4e380a, 47 },
    { 0, 0 },
    { 0x920e380c, 61 },
    { 0xc7ce380d, 23 },
    { 0x4d3e380e, 66 },
    { 0xb5de380f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0e3812, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2e3816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535e381f, 61 },
    { 0x568e3820, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549e3824, 34 },
    { 0xa0ae3825, 33 },
    { 0x74ae3825, 76 },
    { 0x8f9e3827, 98 },
    { 0x687e3828, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5e3830, 61 },
    { 0, 0 },
    { 0xa7be3832, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ee3838, 23 },
    { 0x6e2e3839, 33 },
    { 0xae7e3839, 34 },
    { 0x43de3839, 98 },
    { 0, 0 },
    { 0x3ade383d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74e3840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143e3844, 95 },
    { 0xeefe3845, 23 },
    { 0, 0 },
    { 0xa4ae3847, 46 },
    { 0xa70e3848, 91 },
    { 0xc3ee3848, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x744e384e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854e3855, 55 },
    { 0xf5be3856, 90 },
    { 0, 0 },
    { 0x25ee3858, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e385e, 46 },
    { 0x722e385e, 93 },
    { 0, 0 },
    { 0x65be3861, 77 },
    { 0x169e3862, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e3866, 90 },
    { 0x618e3867, 90 },
    { 0, 0 },
    { 0x309e3869, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e386c, 98 },
    { 0x3b6e386c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70e3870, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3be3876, 65 },
    { 0x8d4e3877, 95 },
    { 0, 0 },
    { 0x342e3879, 95 },
    { 0, 0 },
    { 0xa43e387b, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9e387e, 46 },
    { 0xb27e387f, 87 },
    { 0x92de387f, 91 },
    { 0x23be387f, 23 },
    { 0xbfce3882, 66 },
    { 0x370e3883, 43 },
    { 0x167e3884, 75 },
    { 0x933e3885, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bbe388c, 33 },
    { 0x193e388d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6e3891, 61 },
    { 0x299e3892, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9e3897, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9e38a2, 95 },
    { 0, 0 },
    { 0x8a6e38a4, 23 },
    { 0, 0 },
    { 0x913e38a6, 34 },
    { 0xfb4e38a7, 43 },
    { 0x12fe38a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292e38ac, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96e38af, 61 },
    { 0x95ce38b0, 23 },
    { 0, 0 },
    { 0x4fee38b2, 23 },
    { 0x2c6e38b3, 91 },
    { 0x4aee38b2, 76 },
    { 0x68ce38b5, 93 },
    { 0, 0 },
    { 0x4cfe38b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffe38bb, 95 },
    { 0xa41e38bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5e38bf, 90 },
    { 0, 0 },
    { 0x14ce38c1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdde38c4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8fe38cf, 75 },
    { 0, 0 },
    { 0xbe0e38d1, 90 },
    { 0, 0 },
    { 0x928e38d3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ae38da, 90 },
    { 0, 0 },
    { 0xdcbe38dc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe38df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9e38e3, 98 },
    { 0xf04e38e4, 90 },
    { 0, 0 },
    { 0x908e38e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0e38eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdede38f2, 98 },
    { 0x1efe38f3, 23 },
    { 0x136e38f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09e38fd, 46 },
    { 0, 0 },
    { 0x12e38ff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66be3905, 81 },
    { 0x9f2e3906, 76 },
    { 0xa51e3906, 90 },
    { 0x7c8e3908, 90 },
    { 0x64ae3906, 58 },
    { 0xe8ce390a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7e3911, 61 },
    { 0xb34e3912, 93 },
    { 0x2f5e3913, 91 },
    { 0x5f1e3914, 61 },
    { 0xd53e3915, 33 },
    { 0xaee3916, 90 },
    { 0x6b5e3911, 75 },
    { 0, 0 },
    { 0x736e3919, 38 },
    { 0, 0 },
    { 0x8afe391b, 95 },
    { 0x7a2e391c, 95 },
    { 0, 0 },
    { 0x9ece391e, 76 },
    { 0x16e391e, 90 },
    { 0x118e3920, 84 },
    { 0x757e3920, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2e3925, 87 },
    { 0xe82e3926, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ae392a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19e392d, 90 },
    { 0x30be392e, 52 },
    { 0, 0 },
    { 0x439e3930, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713e3934, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ce3937, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7e393c, 58 },
    { 0xe69e393d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67e3941, 43 },
    { 0xe81e3942, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90e3946, 90 },
    { 0xe33e3947, 98 },
    { 0xabee3947, 90 },
    { 0x57e3948, 90 },
    { 0x371e394a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e394e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x244e3951, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bce3955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4e395c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1e3968, 33 },
    { 0, 0 },
    { 0x56ce396a, 76 },
    { 0, 0 },
    { 0x8e4e396c, 76 },
    { 0x98be396c, 68 },
    { 0xf94e396d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ce3972, 23 },
    { 0, 0 },
    { 0xfbee3974, 43 },
    { 0x2b0e3975, 85 },
    { 0x586e3974, 100 },
    { 0x25de3974, 95 },
    { 0, 0 },
    { 0x482e3979, 98 },
    { 0, 0 },
    { 0xfdae397b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcee3981, 95 },
    { 0x3c1e3982, 61 },
    { 0, 0 },
    { 0xfe9e3984, 78 },
    { 0x860e3984, 98 },
    { 0x163e3984, 36 },
    { 0, 0 },
    { 0xcfbe3988, 97 },
    { 0x8f3e3989, 33 },
    { 0, 0 },
    { 0x758e398b, 90 },
    { 0x82de398c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e3992, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e3998, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ce399c, 46 },
    { 0xa94e399d, 52 },
    { 0x5b0e399e, 95 },
    { 0, 0 },
    { 0x59e39a0, 87 },
    { 0x8a9e39a1, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48e39a7, 23 },
    { 0xe28e39a7, 90 },
    { 0, 0 },
    { 0x4f0e39aa, 23 },
    { 0x256e39aa, 80 },
    { 0, 0 },
    { 0x30be39ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fe39b0, 68 },
    { 0x28de39b0, 93 },
    { 0xc5e39b1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe39b5, 81 },
    { 0x25ce39b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcae39bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721e39c3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ae39c7, 96 },
    { 0xd2fe39c8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ae39cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27e39d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fe39d3, 98 },
    { 0xa5ee39d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78de39d9, 33 },
    { 0x7ebe39da, 99 },
    { 0x984e39d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e39e1, 90 },
    { 0x3dce39e1, 46 },
    { 0x16ce39e2, 93 },
    { 0xb03e39e4, 90 },
    { 0, 0 },
    { 0x566e39e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e39ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69e39f6, 90 },
    { 0x14be39f6, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71de39fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824e3a00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e3a0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e3a13, 90 },
    { 0, 0 },
    { 0x4e5e3a15, 36 },
    { 0xe96e3a16, 77 },
    { 0x19ae3a17, 95 },
    { 0x775e3a18, 33 },
    { 0x9bbe3a19, 90 },
    { 0, 0 },
    { 0x5f9e3a1b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e3a20, 98 },
    { 0x9e2e3a21, 90 },
    { 0, 0 },
    { 0x1e5e3a23, 65 },
    { 0xbb5e3a23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89e3a27, 98 },
    { 0xf1ae3a28, 90 },
    { 0x8e4e3a29, 90 },
    { 0x159e3a28, 61 },
    { 0x7d6e3a28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x256e3a2e, 8 },
    { 0, 0 },
    { 0x6b0e3a30, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66e3a37, 61 },
    { 0, 0 },
    { 0xa8ee3a39, 61 },
    { 0x61ce3a3a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe3a3d, 90 },
    { 0xa36e3a3e, 95 },
    { 0xd0fe3a3f, 85 },
    { 0xe95e3a40, 61 },
    { 0x531e3a40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ee3a46, 46 },
    { 0, 0 },
    { 0x90ee3a48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5e3a4c, 91 },
    { 0, 0 },
    { 0x178e3a4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3a51, 61 },
    { 0x9abe3a51, 93 },
    { 0x165e3a53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72e3a57, 90 },
    { 0x6f0e3a58, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ce3a5b, 61 },
    { 0, 0 },
    { 0xcf4e3a5d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e3a61, 33 },
    { 0x893e3a62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x413e3a65, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x232e3a6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938e3a70, 46 },
    { 0x5a1e3a71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcee3a77, 33 },
    { 0xe0ee3a78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e3a7d, 33 },
    { 0x663e3a7d, 33 },
    { 0, 0 },
    { 0x9efe3a80, 76 },
    { 0xc9ae3a80, 52 },
    { 0, 0 },
    { 0x1abe3a83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x44de3a86, 78 },
    { 0x2a7e3a87, 98 },
    { 0x48ce3a86, 95 },
    { 0, 0 },
    { 0xd39e3a8a, 34 },
    { 0x377e3a8b, 81 },
    { 0x360e3a8c, 90 },
    { 0x38ae3a8a, 33 },
    { 0x9ae3a8e, 68 },
    { 0x130e3a8d, 23 },
    { 0, 0 },
    { 0x530e3a91, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6e3a96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ae3a99, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x395e3a9c, 23 },
    { 0x940e3a9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ce3aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e3aa3, 91 },
    { 0x90ce3aa4, 23 },
    { 0xdfbe3aa3, 33 },
    { 0x36e3aa3, 90 },
    { 0x954e3aa3, 90 },
    { 0x9f0e3aa8, 76 },
    { 0x4ade3aa9, 23 },
    { 0, 0 },
    { 0x1ece3aab, 65 },
    { 0x8b6e3aac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fce3ab1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ee3ab6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ee3ac0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ade3ac3, 76 },
    { 0, 0 },
    { 0x54de3ac5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92be3ac9, 46 },
    { 0x7e8e3aca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbe3ace, 23 },
    { 0xbb3e3acf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bce3ad2, 15 },
    { 0x472e3ad3, 61 },
    { 0xb3ee3ad4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fe3ad8, 27 },
    { 0xd6ae3ad8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ae3ae0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9de3ae4, 55 },
    { 0x23fe3ae5, 61 },
    { 0xe1e3ae6, 23 },
    { 0x859e3ae7, 55 },
    { 0xe6ee3ae8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ae3aee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538e3af4, 88 },
    { 0, 0 },
    { 0x228e3af6, 61 },
    { 0xae1e3af7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3afc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56e3b01, 52 },
    { 0xf04e3b02, 90 },
    { 0xcf1e3b03, 72 },
    { 0, 0 },
    { 0xdf2e3b05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336e3b0d, 100 },
    { 0, 0 },
    { 0xe3e3b0f, 43 },
    { 0x162e3b10, 33 },
    { 0x7b4e3b11, 77 },
    { 0xe64e3b12, 93 },
    { 0, 0 },
    { 0x198e3b14, 33 },
    { 0xb0ee3b14, 61 },
    { 0, 0 },
    { 0x93fe3b17, 33 },
    { 0x7dfe3b17, 93 },
    { 0xf14e3b18, 91 },
    { 0xcd2e3b1a, 33 },
    { 0x697e3b1a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfe3b21, 95 },
    { 0x5b0e3b21, 95 },
    { 0, 0 },
    { 0x377e3b24, 33 },
    { 0, 0 },
    { 0x9f9e3b26, 33 },
    { 0x14fe3b27, 90 },
    { 0x2f7e3b28, 33 },
    { 0xe2de3b28, 17 },
    { 0x23fe3b2a, 61 },
    { 0xabde3b2b, 23 },
    { 0xa07e3b2c, 90 },
    { 0x4e3b2a, 34 },
    { 0xf69e3b2d, 90 },
    { 0, 0 },
    { 0x372e3b30, 93 },
    { 0xbb3e3b31, 23 },
    { 0x9d5e3b32, 95 },
    { 0xd50e3b33, 61 },
    { 0, 0 },
    { 0x75de3b35, 68 },
    { 0x9e5e3b36, 38 },
    { 0xcf7e3b35, 91 },
    { 0xd45e3b38, 61 },
    { 0, 0 },
    { 0x7dce3b3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147e3b3e, 90 },
    { 0x207e3b3f, 95 },
    { 0x701e3b40, 23 },
    { 0x966e3b40, 67 },
    { 0xaffe3b42, 76 },
    { 0x1d1e3b43, 33 },
    { 0xfd8e3b44, 95 },
    { 0xc9ee3b44, 55 },
    { 0x553e3b44, 98 },
    { 0x39fe3b47, 76 },
    { 0xeb4e3b43, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8e3b51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77e3b54, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557e3b58, 77 },
    { 0x84de3b59, 23 },
    { 0x157e3b59, 90 },
    { 0xdf4e3b5b, 33 },
    { 0xc15e3b5c, 33 },
    { 0xf6ce3b59, 93 },
    { 0xd95e3b5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e3b66, 65 },
    { 0, 0 },
    { 0x34fe3b68, 61 },
    { 0, 0 },
    { 0xaece3b6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee3b6e, 33 },
    { 0, 0 },
    { 0xfc7e3b70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffe3b74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e3b77, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bce3b7f, 76 },
    { 0, 0 },
    { 0x6f3e3b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x371e3b84, 33 },
    { 0xc07e3b84, 43 },
    { 0x9e0e3b86, 34 },
    { 0x4e9e3b87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e3b8d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34e3b97, 61 },
    { 0xf5ae3b98, 34 },
    { 0, 0 },
    { 0xdb6e3b9a, 43 },
    { 0, 0 },
    { 0x9fde3b9c, 55 },
    { 0x790e3b9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ade3ba2, 76 },
    { 0x884e3ba3, 61 },
    { 0xd1ae3ba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca4e3ba8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aae3bab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e3bbb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59e3bbe, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdace3bc7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e3bcb, 77 },
    { 0xee3e3bcc, 33 },
    { 0x842e3bcc, 66 },
    { 0, 0 },
    { 0x2b3e3bcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25e3bd6, 93 },
    { 0, 0 },
    { 0x79de3bd8, 99 },
    { 0, 0 },
    { 0xc9e3bda, 33 },
    { 0xabe3bdb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5e3bdf, 87 },
    { 0xeafe3bdf, 90 },
    { 0x659e3bdf, 23 },
    { 0x74e3be2, 55 },
    { 0, 0 },
    { 0x60fe3be4, 87 },
    { 0x6e5e3be5, 61 },
    { 0xe9ee3be4, 93 },
    { 0x5e2e3be6, 52 },
    { 0, 0 },
    { 0x195e3be9, 55 },
    { 0x7b2e3be9, 90 },
    { 0x8f2e3beb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfe3bee, 69 },
    { 0x8fbe3bef, 95 },
    { 0xcece3bf0, 77 },
    { 0x5a8e3bef, 90 },
    { 0, 0 },
    { 0xd9fe3bf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0e3bf7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce3bfb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48e3bfe, 76 },
    { 0x888e3bfe, 61 },
    { 0x75fe3c00, 68 },
    { 0x868e3c01, 77 },
    { 0, 0 },
    { 0x7c5e3c03, 90 },
    { 0x11ce3c04, 76 },
    { 0, 0 },
    { 0xa71e3c06, 61 },
    { 0x1f0e3c07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817e3c0b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326e3c1a, 33 },
    { 0x142e3c1b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4e3c20, 85 },
    { 0x3d1e3c20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89e3c25, 90 },
    { 0x80ee3c26, 76 },
    { 0xa12e3c26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74e3c2b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e3c2f, 55 },
    { 0xbf0e3c30, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42e3c35, 55 },
    { 0, 0 },
    { 0xcdbe3c37, 23 },
    { 0xde5e3c38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87e3c40, 23 },
    { 0xfe8e3c40, 94 },
    { 0xb1be3c42, 81 },
    { 0, 0 },
    { 0x676e3c44, 55 },
    { 0xf64e3c45, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ce3c4b, 91 },
    { 0x369e3c4c, 61 },
    { 0x45e3c4d, 41 },
    { 0x772e3c4d, 23 },
    { 0xb53e3c4e, 61 },
    { 0xfd3e3c4e, 90 },
    { 0x113e3c51, 23 },
    { 0x20ce3c52, 61 },
    { 0x254e3c53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee3c56, 90 },
    { 0x8bce3c57, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ee3c5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0e3c65, 61 },
    { 0x7be3c66, 95 },
    { 0, 0 },
    { 0x9fde3c68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61e3c6c, 33 },
    { 0xa6ae3c6d, 90 },
    { 0x5dde3c6e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x96be3c71, 95 },
    { 0xf61e3c72, 34 },
    { 0, 0 },
    { 0xad8e3c74, 61 },
    { 0xdbfe3c75, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e3c7a, 98 },
    { 0xfdbe3c7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ce3c7e, 55 },
    { 0xecde3c7f, 61 },
    { 0x3bde3c7e, 46 },
    { 0x507e3c7e, 99 },
    { 0x941e3c7f, 61 },
    { 0x2c5e3c83, 90 },
    { 0xc64e3c7f, 33 },
    { 0, 0 },
    { 0x84fe3c86, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11e3c90, 33 },
    { 0x139e3c90, 90 },
    { 0xe3ae3c90, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ce3c96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fe3c9b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fe3c9e, 93 },
    { 0x58ee3c9f, 76 },
    { 0xb94e3ca0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9e3ca4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fe3ca9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27e3cae, 91 },
    { 0x653e3caf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ae3cb2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e3cb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13be3cbc, 78 },
    { 0, 0 },
    { 0x7bae3cbe, 33 },
    { 0x7f6e3cbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x603e3cc2, 46 },
    { 0x6ce3cc3, 55 },
    { 0x850e3cc3, 53 },
    { 0x1a3e3cc5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ee3cd0, 95 },
    { 0xc6fe3cd1, 61 },
    { 0, 0 },
    { 0x1abe3cd3, 52 },
    { 0xdcee3cd4, 33 },
    { 0x57de3cd4, 100 },
    { 0, 0 },
    { 0x8c7e3cd7, 33 },
    { 0xa7be3cd8, 95 },
    { 0, 0 },
    { 0x5f2e3cda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ae3cde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x850e3ce1, 46 },
    { 0, 0 },
    { 0xc5ce3ce3, 100 },
    { 0xca6e3ce4, 98 },
    { 0xefe3ce5, 25 },
    { 0x2f3e3ce6, 33 },
    { 0xc2de3ce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9de3cec, 95 },
    { 0x507e3ced, 23 },
    { 0x68ee3cec, 67 },
    { 0xfdfe3cef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e3cf3, 23 },
    { 0x60ce3cf3, 100 },
    { 0xa57e3cf5, 98 },
    { 0x44e3cf3, 91 },
    { 0x70e3cf7, 43 },
    { 0x48ae3cf8, 61 },
    { 0x1d4e3cf9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x157e3cfc, 92 },
    { 0x2bee3cfd, 76 },
    { 0xa5fe3cfe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x81be3d01, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3e3d05, 23 },
    { 0x4bbe3d05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96e3d09, 61 },
    { 0xb9ae3d09, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3e3d0e, 88 },
    { 0x881e3d0f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4e3d14, 90 },
    { 0x630e3d14, 98 },
    { 0x150e3d16, 90 },
    { 0xbc5e3d14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ae3d1d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702e3d21, 98 },
    { 0, 0 },
    { 0x4dee3d23, 91 },
    { 0xb83e3d24, 61 },
    { 0xa52e3d23, 90 },
    { 0, 0 },
    { 0xf93e3d27, 90 },
    { 0xfc1e3d28, 98 },
    { 0x6c3e3d28, 61 },
    { 0, 0 },
    { 0xc0ce3d2b, 99 },
    { 0, 0 },
    { 0x5c6e3d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18be3d30, 76 },
    { 0xeb3e3d31, 34 },
    { 0x1fe3d30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1e3d37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404e3d3b, 84 },
    { 0x451e3d3b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807e3d40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2e3d45, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14e3d4f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6de3d59, 31 },
    { 0, 0 },
    { 0x57e3d5b, 46 },
    { 0x80e3d5b, 52 },
    { 0, 0 },
    { 0xba0e3d5e, 78 },
    { 0x828e3d5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xddce3d62, 46 },
    { 0xf96e3d62, 95 },
    { 0, 0 },
    { 0xebce3d65, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06e3d68, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973e3d6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4e3d72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300e3d76, 61 },
    { 0xc7ee3d77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499e3d7e, 76 },
    { 0, 0 },
    { 0x66ee3d80, 61 },
    { 0x40e3d81, 97 },
    { 0, 0 },
    { 0x4cce3d83, 33 },
    { 0, 0 },
    { 0x241e3d85, 61 },
    { 0xc57e3d85, 76 },
    { 0xb92e3d86, 23 },
    { 0x311e3d88, 98 },
    { 0x182e3d89, 72 },
    { 0xec1e3d8a, 46 },
    { 0x6b8e3d8a, 100 },
    { 0, 0 },
    { 0x63ce3d8d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56be3d93, 98 },
    { 0, 0 },
    { 0xea9e3d95, 33 },
    { 0x71fe3d96, 95 },
    { 0x98e3d97, 23 },
    { 0x3cae3d97, 55 },
    { 0x25fe3d95, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e3d9c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ee3da1, 88 },
    { 0xf55e3da2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffe3da8, 33 },
    { 0x83ee3da9, 61 },
    { 0x218e3daa, 33 },
    { 0x71be3dab, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbae3db3, 95 },
    { 0, 0 },
    { 0xb9e3db5, 33 },
    { 0xc42e3db6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0e3dba, 90 },
    { 0x606e3dbb, 33 },
    { 0x86de3dba, 90 },
    { 0x3e5e3dbb, 90 },
    { 0, 0 },
    { 0x518e3dbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e3dc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e3dca, 68 },
    { 0, 0 },
    { 0xc9de3dcc, 90 },
    { 0x351e3dcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04e3dd6, 61 },
    { 0xb3ae3dd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fe3ddd, 46 },
    { 0, 0 },
    { 0x555e3ddf, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7de3de3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13e3df0, 77 },
    { 0, 0 },
    { 0x979e3df2, 98 },
    { 0x268e3df2, 43 },
    { 0x470e3df3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1e3df7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31e3dfb, 95 },
    { 0x868e3dfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafe3e01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2e3e09, 43 },
    { 0, 0 },
    { 0xa44e3e0b, 91 },
    { 0xbebe3e0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e3e0f, 61 },
    { 0, 0 },
    { 0xbc5e3e11, 85 },
    { 0, 0 },
    { 0xfb3e3e13, 34 },
    { 0x402e3e13, 76 },
    { 0xcf7e3e15, 23 },
    { 0xd6ae3e14, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e3e1a, 55 },
    { 0x7a3e3e1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1be3e20, 42 },
    { 0x33ee3e21, 93 },
    { 0x326e3e22, 90 },
    { 0, 0 },
    { 0xf9e3e24, 33 },
    { 0xbd5e3e25, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eee3e28, 98 },
    { 0x5f4e3e28, 33 },
    { 0x79fe3e2a, 23 },
    { 0x5ebe3e2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce3e30, 23 },
    { 0, 0 },
    { 0x393e3e32, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9e3e35, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02e3e38, 90 },
    { 0xe2ee3e39, 23 },
    { 0x4dae3e3a, 61 },
    { 0x6b0e3e3b, 90 },
    { 0, 0 },
    { 0xeb2e3e3d, 64 },
    { 0xcd0e3e3e, 33 },
    { 0x3c9e3e3e, 100 },
    { 0x4f6e3e40, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee3e43, 66 },
    { 0xe09e3e44, 23 },
    { 0, 0 },
    { 0xa4e3e46, 34 },
    { 0xc76e3e47, 33 },
    { 0x8c4e3e48, 90 },
    { 0xb9e3e48, 90 },
    { 0xcabe3e49, 88 },
    { 0x5e3e3e4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29e3e4e, 98 },
    { 0x7b1e3e4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769e3e53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x900e3e56, 95 },
    { 0x4bde3e57, 70 },
    { 0x3a2e3e57, 23 },
    { 0x635e3e59, 95 },
    { 0xbdae3e5a, 55 },
    { 0, 0 },
    { 0xa25e3e5c, 78 },
    { 0xa5ee3e5d, 90 },
    { 0, 0 },
    { 0x205e3e5f, 98 },
    { 0xfd1e3e60, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e3e63, 90 },
    { 0x50ce3e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e3e67, 95 },
    { 0xe64e3e68, 90 },
    { 0x24e3e68, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72e3e6f, 23 },
    { 0x33de3e70, 61 },
    { 0xdd4e3e71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2e3e77, 33 },
    { 0xe36e3e78, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77e3e7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3e3e8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ee3e92, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x801e3e95, 90 },
    { 0xe37e3e96, 85 },
    { 0xe1ae3e96, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce3e9c, 98 },
    { 0xcd1e3e9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79de3ea0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x357e3ea3, 87 },
    { 0x24ae3ea3, 46 },
    { 0xf85e3ea5, 33 },
    { 0xa1be3ea5, 61 },
    { 0xf54e3ea7, 55 },
    { 0xc3fe3ea4, 55 },
    { 0xe5ce3ea3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5e3eae, 23 },
    { 0x5a2e3eaf, 83 },
    { 0x917e3eaf, 95 },
    { 0x9a1e3eb1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712e3eb7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2e3ebc, 90 },
    { 0x793e3ebd, 69 },
    { 0, 0 },
    { 0xce1e3ebf, 33 },
    { 0xc8ae3ec0, 23 },
    { 0, 0 },
    { 0x89de3ec2, 98 },
    { 0x345e3ec3, 61 },
    { 0x7d9e3ec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144e3ec9, 99 },
    { 0x58ee3ec9, 75 },
    { 0, 0 },
    { 0x1b6e3ecc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ee3ecf, 93 },
    { 0xb25e3ecf, 23 },
    { 0, 0 },
    { 0x2fe3ed2, 100 },
    { 0xda8e3ed3, 90 },
    { 0, 0 },
    { 0x524e3ed5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeae3edf, 34 },
    { 0x968e3ee0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03e3ee8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773e3eed, 88 },
    { 0x38e3eee, 90 },
    { 0, 0 },
    { 0x39fe3ef0, 99 },
    { 0xe0ce3ef1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afe3ef8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31e3efb, 95 },
    { 0x1d8e3efb, 95 },
    { 0, 0 },
    { 0xc5fe3efe, 46 },
    { 0x102e3eff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7de3f02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e3f05, 95 },
    { 0xe3e3f05, 34 },
    { 0x10e3f07, 23 },
    { 0xa47e3f08, 99 },
    { 0x941e3f07, 23 },
    { 0x57be3f0a, 98 },
    { 0x241e3f0a, 33 },
    { 0x2f5e3f0a, 98 },
    { 0x40ae3f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x228e3f10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ee3f13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e3f17, 61 },
    { 0, 0 },
    { 0x3f8e3f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe3f1f, 33 },
    { 0x12be3f20, 23 },
    { 0x6afe3f1f, 43 },
    { 0xe39e3f1f, 85 },
    { 0, 0 },
    { 0xd40e3f24, 76 },
    { 0x41ee3f25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aee3f29, 100 },
    { 0, 0 },
    { 0xd8de3f2b, 90 },
    { 0, 0 },
    { 0xba6e3f2d, 23 },
    { 0xc04e3f2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45e3f33, 98 },
    { 0xacae3f34, 23 },
    { 0x1bbe3f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e3f42, 55 },
    { 0xe2e3f43, 65 },
    { 0, 0 },
    { 0xdc7e3f45, 98 },
    { 0, 0 },
    { 0x85fe3f47, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4e3f4b, 76 },
    { 0, 0 },
    { 0x68e3f4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8e3f51, 76 },
    { 0, 0 },
    { 0x4dbe3f53, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278e3f59, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x556e3f5c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe3f63, 61 },
    { 0x4b6e3f64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x213e3f67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298e3f6b, 52 },
    { 0, 0 },
    { 0x122e3f6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76fe3f70, 87 },
    { 0x4d8e3f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834e3f75, 90 },
    { 0x916e3f76, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876e3f7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0e3f81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81e3f84, 40 },
    { 0, 0 },
    { 0xcfce3f86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x901e3f8a, 33 },
    { 0, 0 },
    { 0x6a1e3f8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932e3f90, 98 },
    { 0x8c2e3f90, 23 },
    { 0, 0 },
    { 0xee3f93, 33 },
    { 0, 0 },
    { 0xd93e3f95, 91 },
    { 0x6ece3f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01e3f9b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae3fa0, 90 },
    { 0x1b7e3fa0, 93 },
    { 0xe22e3fa0, 55 },
    { 0x132e3fa3, 95 },
    { 0x71fe3fa2, 90 },
    { 0xddce3fa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ee3fa9, 33 },
    { 0xb04e3faa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be3fad, 78 },
    { 0, 0 },
    { 0xffe3faf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e3fb3, 100 },
    { 0xcdee3fb4, 49 },
    { 0x777e3fb4, 76 },
    { 0x33ae3fb5, 76 },
    { 0x6ede3fb6, 66 },
    { 0x6e5e3fb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4e3fbf, 33 },
    { 0, 0 },
    { 0xf50e3fc1, 95 },
    { 0, 0 },
    { 0x67ce3fc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7be3fc9, 85 },
    { 0x4bbe3fca, 61 },
    { 0xcdce3fca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x87de3fce, 70 },
    { 0xfc9e3fcf, 33 },
    { 0xa7ce3fce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e3fd4, 34 },
    { 0xa55e3fd5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7e3fda, 55 },
    { 0xfece3fdb, 33 },
    { 0x5b1e3fdb, 23 },
    { 0x25de3fdd, 9 },
    { 0, 0 },
    { 0x7b3e3fdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1be3fe8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e3ff0, 40 },
    { 0, 0 },
    { 0xaa5e3ff2, 65 },
    { 0xe1ce3ff2, 61 },
    { 0xd2ce3ff4, 23 },
    { 0, 0 },
    { 0x90be3ff6, 88 },
    { 0x140e3ff7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ae3ffb, 52 },
    { 0xf94e3ffb, 46 },
    { 0xdd1e3ffd, 78 },
    { 0xf3ce3ffe, 93 },
    { 0x7fae3fff, 33 },
    { 0x8dde3fff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146e4008, 90 },
    { 0, 0 },
    { 0x5e400a, 34 },
    { 0, 0 },
    { 0x295e400c, 90 },
    { 0x5cde400d, 98 },
    { 0x81fe400e, 98 },
    { 0xfafe400d, 33 },
    { 0xd9ee4010, 90 },
    { 0xe75e4010, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0e4014, 90 },
    { 0, 0 },
    { 0x513e4016, 66 },
    { 0, 0 },
    { 0xa1ee4018, 90 },
    { 0xe35e4018, 76 },
    { 0x8ece4019, 95 },
    { 0, 0 },
    { 0x77be401c, 34 },
    { 0x76fe401d, 95 },
    { 0x1abe401e, 34 },
    { 0x70be401e, 90 },
    { 0x647e401c, 69 },
    { 0xf14e401d, 93 },
    { 0x162e4022, 43 },
    { 0xde7e401e, 40 },
    { 0, 0 },
    { 0x2dee4025, 56 },
    { 0, 0 },
    { 0xd46e4027, 93 },
    { 0x333e4028, 85 },
    { 0xed2e4028, 90 },
    { 0x43e4029, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee402e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ee4034, 33 },
    { 0x1e2e4034, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e4038, 98 },
    { 0, 0 },
    { 0x880e403a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x762e4041, 66 },
    { 0x195e4042, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2be4046, 61 },
    { 0xed3e4047, 75 },
    { 0, 0 },
    { 0xba4e4049, 23 },
    { 0x294e4049, 23 },
    { 0x545e4049, 43 },
    { 0x652e404c, 78 },
    { 0xf81e404c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90e4050, 23 },
    { 0x8d2e4051, 98 },
    { 0x201e4052, 88 },
    { 0xb5de4051, 81 },
    { 0x206e4054, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x387e4057, 60 },
    { 0x1a2e4058, 90 },
    { 0, 0 },
    { 0xbc9e405a, 90 },
    { 0x1cae405b, 98 },
    { 0, 0 },
    { 0x410e405d, 96 },
    { 0x444e405e, 40 },
    { 0xcf3e405e, 55 },
    { 0xdf2e405e, 95 },
    { 0x187e405e, 17 },
    { 0xcc6e405e, 55 },
    { 0, 0 },
    { 0xa66e4064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505e406b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6e406e, 90 },
    { 0xc2ce406f, 66 },
    { 0x91ae406f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ee4073, 87 },
    { 0xbae4074, 17 },
    { 0x97ce4073, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ee4079, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0e407c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce407f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae4085, 81 },
    { 0, 0 },
    { 0x726e4087, 61 },
    { 0x16fe4088, 33 },
    { 0x785e4089, 61 },
    { 0x404e4087, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e408e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12e4092, 98 },
    { 0x111e4093, 55 },
    { 0xa57e4094, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36e4098, 61 },
    { 0xf1de4098, 90 },
    { 0xd1de409a, 61 },
    { 0, 0 },
    { 0x982e409c, 98 },
    { 0x95ae409d, 23 },
    { 0xe92e409e, 23 },
    { 0x786e409d, 61 },
    { 0x5dfe40a0, 55 },
    { 0xe6fe40a1, 61 },
    { 0x1ee40a1, 61 },
    { 0xa93e40a2, 43 },
    { 0xf9ee40a4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959e40aa, 33 },
    { 0xdd8e40aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x279e40b2, 34 },
    { 0xd88e40b2, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94be40bb, 46 },
    { 0, 0 },
    { 0xf6ee40bd, 61 },
    { 0, 0 },
    { 0x56ae40bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147e40c3, 34 },
    { 0xa1fe40c4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334e40cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804e40cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ee40d2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14de40da, 91 },
    { 0xa3ae40db, 86 },
    { 0x879e40da, 55 },
    { 0x8f5e40db, 75 },
    { 0, 0 },
    { 0xdcee40df, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3e40e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107e40e7, 34 },
    { 0, 0 },
    { 0xaf9e40e9, 23 },
    { 0x49e40ea, 95 },
    { 0xb31e40ea, 33 },
    { 0x2e5e40ec, 69 },
    { 0, 0 },
    { 0x665e40ee, 90 },
    { 0xde0e40ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ae40f3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x794e40f6, 91 },
    { 0, 0 },
    { 0x7dce40f8, 98 },
    { 0, 0 },
    { 0xf02e40fa, 95 },
    { 0x112e40fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2e4106, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee410c, 90 },
    { 0, 0 },
    { 0x8f2e410e, 90 },
    { 0xc7ae410e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e4115, 87 },
    { 0x61be4115, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce411b, 67 },
    { 0xf32e411c, 78 },
    { 0x556e411c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497e4121, 98 },
    { 0, 0 },
    { 0xe6be4123, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x530e4126, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcee4139, 43 },
    { 0, 0 },
    { 0x7a9e413b, 98 },
    { 0x182e413b, 33 },
    { 0, 0 },
    { 0xfaee413e, 78 },
    { 0x840e413e, 91 },
    { 0xb05e413f, 46 },
    { 0x8a4e4141, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5e4147, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fde414e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35e4153, 55 },
    { 0, 0 },
    { 0x607e4155, 33 },
    { 0, 0 },
    { 0xbe7e4157, 61 },
    { 0xbf4e4157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e415b, 95 },
    { 0x10e415b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7e4165, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109e4169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e416d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fde4172, 23 },
    { 0x9a0e4173, 84 },
    { 0x7e5e4173, 34 },
    { 0x5f7e4175, 90 },
    { 0xa61e4176, 76 },
    { 0xb84e4175, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x170e417a, 95 },
    { 0x627e417b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849e4180, 90 },
    { 0xf0de4181, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e4184, 33 },
    { 0x4b3e4185, 95 },
    { 0x2d0e4186, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fe418f, 23 },
    { 0, 0 },
    { 0x9a8e4191, 87 },
    { 0, 0 },
    { 0xbf0e4193, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e4198, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x747e419b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efe41a0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e41a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218e41ac, 64 },
    { 0xa9fe41ad, 87 },
    { 0x28ae41ad, 46 },
    { 0x46ae41ae, 66 },
    { 0xea4e41b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990e41b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85e41c6, 12 },
    { 0xd4e41c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ce41cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94de41d3, 33 },
    { 0x193e41d4, 23 },
    { 0xc10e41d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x326e41d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfe41db, 33 },
    { 0xf08e41dc, 33 },
    { 0xa66e41dd, 61 },
    { 0x132e41de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15e41e1, 100 },
    { 0x51de41e2, 78 },
    { 0x20fe41e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x92e41e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63e41ee, 52 },
    { 0x650e41ef, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x719e41f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6e41f5, 61 },
    { 0x728e41f5, 34 },
    { 0xeede41f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0e41fb, 43 },
    { 0, 0 },
    { 0x2dde41fd, 76 },
    { 0, 0 },
    { 0xebe41ff, 33 },
    { 0x802e41ff, 85 },
    { 0x96ee41ff, 21 },
    { 0xa89e4202, 61 },
    { 0, 0 },
    { 0x805e4204, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627e420f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4212, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efe4215, 61 },
    { 0xc71e4216, 93 },
    { 0x1c4e4217, 75 },
    { 0, 0 },
    { 0xd3ce4219, 95 },
    { 0xeefe421a, 90 },
    { 0xc46e421b, 85 },
    { 0x681e421a, 33 },
    { 0, 0 },
    { 0xfdfe421e, 33 },
    { 0xb40e421e, 66 },
    { 0x3e9e421e, 23 },
    { 0x389e421e, 91 },
    { 0, 0 },
    { 0xb61e4223, 98 },
    { 0, 0 },
    { 0xe5e4225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccee422e, 76 },
    { 0xd3ae422e, 95 },
    { 0xc6fe422e, 36 },
    { 0x283e4231, 95 },
    { 0x700e4232, 33 },
    { 0xf15e4231, 33 },
    { 0x7cae4230, 66 },
    { 0x8a2e4231, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dae423a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90e4240, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459e4244, 46 },
    { 0x6ffe4244, 61 },
    { 0, 0 },
    { 0x30ce4247, 95 },
    { 0x1ede4248, 77 },
    { 0xa22e4249, 33 },
    { 0x38ee4249, 67 },
    { 0x2e5e424a, 77 },
    { 0x1cee424c, 76 },
    { 0, 0 },
    { 0xd50e424e, 40 },
    { 0, 0 },
    { 0x953e4250, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731e4258, 61 },
    { 0x5ebe4258, 98 },
    { 0xeb0e425a, 90 },
    { 0xa82e425b, 76 },
    { 0xb14e425b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1e4261, 46 },
    { 0, 0 },
    { 0x5f3e4263, 67 },
    { 0xc45e4264, 36 },
    { 0x645e4265, 61 },
    { 0x942e4265, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48e426c, 90 },
    { 0x1d7e426c, 23 },
    { 0x4f2e426c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1e4272, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07e4275, 23 },
    { 0xf61e4275, 77 },
    { 0xab0e4275, 23 },
    { 0xd2e4278, 61 },
    { 0xfaee4276, 91 },
    { 0x568e427a, 77 },
    { 0xe7ce427b, 90 },
    { 0x54ee427c, 40 },
    { 0x27e427d, 23 },
    { 0x7c7e427e, 46 },
    { 0x9b1e427f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28be4283, 52 },
    { 0, 0 },
    { 0x179e4285, 98 },
    { 0, 0 },
    { 0x4c5e4287, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6efe428d, 23 },
    { 0, 0 },
    { 0xe9fe428f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e4292, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136e429c, 68 },
    { 0xf01e429d, 95 },
    { 0x396e429e, 90 },
    { 0x45fe429f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416e42a7, 17 },
    { 0, 0 },
    { 0x39fe42a9, 23 },
    { 0x2f1e42a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9e42ad, 76 },
    { 0x637e42ae, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefe42b2, 23 },
    { 0, 0 },
    { 0xdd9e42b4, 52 },
    { 0xb48e42b5, 34 },
    { 0xfae42b6, 76 },
    { 0, 0 },
    { 0xac4e42b8, 34 },
    { 0, 0 },
    { 0x7e7e42ba, 37 },
    { 0, 0 },
    { 0xc8ae42bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766e42c5, 90 },
    { 0xbe7e42c6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ee42cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0e42cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bce42d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121e42db, 61 },
    { 0x898e42db, 84 },
    { 0x8d5e42dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaee42e3, 33 },
    { 0, 0 },
    { 0x413e42e5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ee42ed, 90 },
    { 0xf1ae42ee, 61 },
    { 0xa0ae42ef, 93 },
    { 0xd60e42ef, 100 },
    { 0xd44e42ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3e42f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e42fc, 23 },
    { 0xb1ce42fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x720e4300, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dce4308, 61 },
    { 0xe3de4308, 91 },
    { 0, 0 },
    { 0x22de430b, 95 },
    { 0, 0 },
    { 0xb1de430d, 61 },
    { 0xcf7e430e, 95 },
    { 0x7ebe430f, 33 },
    { 0, 0 },
    { 0x18be4311, 66 },
    { 0, 0 },
    { 0xf11e4313, 90 },
    { 0xb37e4313, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158e4319, 61 },
    { 0xf04e4319, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93e4328, 23 },
    { 0, 0 },
    { 0x3a2e432a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8e4333, 95 },
    { 0, 0 },
    { 0xb1fe4335, 66 },
    { 0, 0 },
    { 0xfcfe4337, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7e433c, 40 },
    { 0, 0 },
    { 0x27fe433e, 88 },
    { 0xf58e433e, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226e4345, 98 },
    { 0x584e4346, 90 },
    { 0x753e4345, 90 },
    { 0x524e4348, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63e434b, 76 },
    { 0xdbae434c, 33 },
    { 0x5fae434d, 33 },
    { 0xc89e434c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ae4352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3e4355, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88e435c, 90 },
    { 0x8cee435d, 34 },
    { 0xc1ee435e, 34 },
    { 0x164e435f, 61 },
    { 0xe8e4360, 23 },
    { 0x488e4361, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300e436e, 46 },
    { 0, 0 },
    { 0xce2e4370, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ae4374, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e4378, 98 },
    { 0x741e4379, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675e437f, 90 },
    { 0x296e437f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce4385, 34 },
    { 0, 0 },
    { 0x3cbe4387, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ae438a, 33 },
    { 0, 0 },
    { 0x40be438c, 43 },
    { 0, 0 },
    { 0x4dde438e, 34 },
    { 0x43de438f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde4392, 40 },
    { 0x95fe4393, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6e439b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5e439e, 84 },
    { 0, 0 },
    { 0xd4ee43a0, 87 },
    { 0, 0 },
    { 0x1eae43a2, 55 },
    { 0x22ce43a3, 81 },
    { 0x808e43a3, 90 },
    { 0x8ade43a5, 33 },
    { 0x3cbe43a3, 33 },
    { 0x5efe43a7, 90 },
    { 0xf42e43a8, 90 },
    { 0x71e43a6, 90 },
    { 0x90be43aa, 90 },
    { 0xae43ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x217e43ae, 90 },
    { 0, 0 },
    { 0xcf0e43b0, 99 },
    { 0, 0 },
    { 0x9e6e43b2, 33 },
    { 0xb79e43b3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ae43b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ae43ba, 95 },
    { 0, 0 },
    { 0x200e43bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e43bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e43c3, 98 },
    { 0xbdde43c4, 98 },
    { 0x5f2e43c3, 61 },
    { 0xc57e43c6, 68 },
    { 0xb57e43c6, 98 },
    { 0x78ce43c8, 90 },
    { 0xfffe43c9, 90 },
    { 0x6a9e43c3, 34 },
    { 0xfbfe43cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336e43cf, 90 },
    { 0x58ae43d0, 90 },
    { 0xbb5e43d1, 90 },
    { 0x44e43d2, 61 },
    { 0, 0 },
    { 0x107e43d4, 84 },
    { 0, 0 },
    { 0x5be43d6, 66 },
    { 0, 0 },
    { 0x30ae43d8, 95 },
    { 0, 0 },
    { 0xcebe43da, 90 },
    { 0xb7fe43db, 23 },
    { 0xadde43dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997e43e0, 61 },
    { 0, 0 },
    { 0xac7e43e2, 78 },
    { 0, 0 },
    { 0x1ede43e4, 90 },
    { 0xe12e43e4, 90 },
    { 0xdd9e43e6, 75 },
    { 0x3f4e43e7, 66 },
    { 0x755e43e7, 95 },
    { 0, 0 },
    { 0x24e43ea, 90 },
    { 0, 0 },
    { 0x6dee43ec, 61 },
    { 0x5a1e43ec, 90 },
    { 0, 0 },
    { 0x524e43ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ae43f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bee43f5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00e43fa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7e43fd, 95 },
    { 0xe0ce43fd, 53 },
    { 0x600e43fd, 43 },
    { 0x877e43ff, 93 },
    { 0, 0 },
    { 0x2c5e4402, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aae4405, 76 },
    { 0xb3de4405, 98 },
    { 0x302e4407, 61 },
    { 0, 0 },
    { 0xb04e4409, 23 },
    { 0x6be440a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e440e, 52 },
    { 0, 0 },
    { 0x45be4410, 98 },
    { 0x3a0e4410, 98 },
    { 0, 0 },
    { 0xda0e4413, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e4416, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaace441c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fe4421, 33 },
    { 0xc49e4422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234e4429, 98 },
    { 0xe34e442a, 90 },
    { 0, 0 },
    { 0xe78e442c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5e4431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90e4434, 98 },
    { 0xcafe4435, 90 },
    { 0, 0 },
    { 0x3dde4437, 90 },
    { 0x1ce4438, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956e443e, 61 },
    { 0, 0 },
    { 0x2f7e4440, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fe4447, 95 },
    { 0, 0 },
    { 0x20e4449, 95 },
    { 0x687e4449, 69 },
    { 0x627e444b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e4452, 78 },
    { 0x236e4452, 77 },
    { 0, 0 },
    { 0x66fe4455, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be445b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x197e445e, 90 },
    { 0xc15e445f, 95 },
    { 0xf98e445f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3e4463, 38 },
    { 0x1a5e4464, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde4467, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e446a, 70 },
    { 0xc48e446b, 33 },
    { 0, 0 },
    { 0x30ce446d, 52 },
    { 0xc87e446e, 95 },
    { 0xf33e446f, 95 },
    { 0x1c6e4470, 76 },
    { 0x369e4471, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ce4475, 100 },
    { 0x6e0e4476, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae4479, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984e448c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e4491, 45 },
    { 0, 0 },
    { 0xaf4e4493, 94 },
    { 0, 0 },
    { 0xfc7e4495, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee449f, 95 },
    { 0, 0 },
    { 0x157e44a1, 95 },
    { 0x91de44a2, 40 },
    { 0, 0 },
    { 0x283e44a4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656e44a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e44b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6e44b4, 23 },
    { 0x490e44b4, 76 },
    { 0, 0 },
    { 0xf74e44b7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ee44ba, 90 },
    { 0x60ce44ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cae44be, 98 },
    { 0, 0 },
    { 0x731e44c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35be44c5, 23 },
    { 0x5e5e44c6, 95 },
    { 0x8b3e44c7, 23 },
    { 0, 0 },
    { 0x881e44c9, 88 },
    { 0xcbe44c9, 81 },
    { 0, 0 },
    { 0xd75e44cc, 90 },
    { 0x599e44cd, 23 },
    { 0x787e44ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11e44d1, 33 },
    { 0x4ce44d1, 90 },
    { 0, 0 },
    { 0x413e44d4, 95 },
    { 0, 0 },
    { 0xd4de44d6, 90 },
    { 0x63ce44d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ee44da, 33 },
    { 0, 0 },
    { 0x23ce44dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ae44e0, 95 },
    { 0x48ce44e0, 93 },
    { 0x511e44e2, 40 },
    { 0x354e44e2, 98 },
    { 0, 0 },
    { 0x41de44e5, 55 },
    { 0xf70e44e6, 39 },
    { 0x898e44e7, 43 },
    { 0, 0 },
    { 0x719e44e9, 52 },
    { 0xbe0e44e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242e44f1, 83 },
    { 0x44ae44f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35de44f6, 98 },
    { 0, 0 },
    { 0x358e44f8, 99 },
    { 0xd3ce44f9, 98 },
    { 0, 0 },
    { 0x67ee44fb, 95 },
    { 0, 0 },
    { 0xf5ce44fd, 93 },
    { 0x7b3e44fe, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x573e4501, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe4511, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ee4514, 46 },
    { 0xb3ee4515, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e4518, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963e451e, 90 },
    { 0, 0 },
    { 0xbc6e4520, 95 },
    { 0, 0 },
    { 0x9bce4522, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e4528, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ee452c, 90 },
    { 0, 0 },
    { 0xb93e452e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8e4533, 65 },
    { 0, 0 },
    { 0x82e4535, 77 },
    { 0x8e1e4536, 76 },
    { 0x930e4537, 66 },
    { 0x888e4538, 33 },
    { 0, 0 },
    { 0x5dfe453a, 46 },
    { 0, 0 },
    { 0xfbee453c, 33 },
    { 0x4f5e453c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e4540, 17 },
    { 0x879e4541, 98 },
    { 0, 0 },
    { 0xba3e4543, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41e454a, 91 },
    { 0x2ce454b, 33 },
    { 0x4dae454b, 52 },
    { 0xb75e454d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ee4550, 78 },
    { 0x964e4550, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce4557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e455c, 46 },
    { 0, 0 },
    { 0xba5e455e, 90 },
    { 0x1c7e455f, 95 },
    { 0xd1ee4560, 33 },
    { 0x4a3e4561, 81 },
    { 0x13be4561, 33 },
    { 0xb2e4563, 90 },
    { 0xc2be4560, 95 },
    { 0x938e4565, 43 },
    { 0x562e4566, 78 },
    { 0xba3e4561, 33 },
    { 0x4fce4568, 61 },
    { 0xea7e4569, 66 },
    { 0x2b5e456a, 23 },
    { 0x4b1e455e, 87 },
    { 0x5e4e4561, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x777e456f, 98 },
    { 0xd2ae456f, 95 },
    { 0, 0 },
    { 0x1b9e4572, 33 },
    { 0, 0 },
    { 0xc11e4574, 90 },
    { 0, 0 },
    { 0xf68e4576, 90 },
    { 0x5c0e4577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fbe457a, 91 },
    { 0xb7e457b, 90 },
    { 0x602e457c, 34 },
    { 0, 0 },
    { 0x93be457e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x533e4581, 90 },
    { 0x5bbe4582, 90 },
    { 0x840e4581, 90 },
    { 0xa9be4584, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8e458c, 33 },
    { 0x4dfe458d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e4594, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9be45a4, 90 },
    { 0xd8de45a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e45a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e45ad, 33 },
    { 0, 0 },
    { 0x974e45af, 100 },
    { 0x854e45af, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7e45b4, 98 },
    { 0, 0 },
    { 0xa5de45b6, 90 },
    { 0x6f3e45b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3e45be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5e45cc, 90 },
    { 0xa43e45cd, 61 },
    { 0, 0 },
    { 0x72be45cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731e45d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520e45dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40e45e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e45e3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542e45e7, 90 },
    { 0x636e45e8, 78 },
    { 0, 0 },
    { 0xef5e45ea, 34 },
    { 0xf12e45eb, 55 },
    { 0x8efe45ec, 99 },
    { 0x8a1e45ec, 95 },
    { 0x1bbe45ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae45f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde45f6, 46 },
    { 0x3fde45f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95be45fb, 33 },
    { 0x18ce45fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e45ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa72e4602, 61 },
    { 0x561e4603, 95 },
    { 0, 0 },
    { 0x8b1e4605, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fde4608, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x109e460c, 23 },
    { 0, 0 },
    { 0xb31e460e, 23 },
    { 0x82de460f, 33 },
    { 0x666e4610, 17 },
    { 0xfede4611, 33 },
    { 0x262e4610, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5e4617, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e461d, 43 },
    { 0xb8ae461e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103e4626, 91 },
    { 0xd25e4627, 46 },
    { 0xc47e4627, 66 },
    { 0x743e4628, 84 },
    { 0x3d7e4629, 61 },
    { 0, 0 },
    { 0x428e462c, 46 },
    { 0x522e462c, 33 },
    { 0xf2e462e, 95 },
    { 0, 0 },
    { 0x3cae4630, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14e4633, 98 },
    { 0xe34e4633, 43 },
    { 0, 0 },
    { 0x8cfe4636, 95 },
    { 0, 0 },
    { 0x810e4638, 34 },
    { 0, 0 },
    { 0xbd6e463a, 91 },
    { 0x9c0e463b, 76 },
    { 0xbf6e463c, 33 },
    { 0x5e0e463d, 95 },
    { 0x324e463e, 99 },
    { 0x5b3e463f, 80 },
    { 0xf32e4640, 98 },
    { 0, 0 },
    { 0x5e7e4642, 46 },
    { 0xca1e4643, 61 },
    { 0xd76e4644, 90 },
    { 0x8afe4645, 91 },
    { 0, 0 },
    { 0xa1ae4647, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1e464b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x778e464e, 33 },
    { 0xe6fe464f, 66 },
    { 0, 0 },
    { 0xb7e4651, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5de4654, 61 },
    { 0xb98e4654, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e4661, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90de466a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ce466d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e4671, 23 },
    { 0xbcde4672, 98 },
    { 0x4a8e4672, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8de4677, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x69be467a, 46 },
    { 0x140e467b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206e4681, 33 },
    { 0x432e4681, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bee4686, 17 },
    { 0x7a8e4686, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fce468b, 61 },
    { 0xa16e468c, 95 },
    { 0, 0 },
    { 0x6efe468e, 33 },
    { 0x110e468e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ae4693, 43 },
    { 0x144e4694, 55 },
    { 0x852e4694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7e469d, 95 },
    { 0, 0 },
    { 0x95de469f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8e46a2, 90 },
    { 0xb4de46a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6e46a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbe46ac, 99 },
    { 0, 0 },
    { 0x7c4e46ae, 61 },
    { 0x8a9e46af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffe46b5, 61 },
    { 0x1cce46b6, 33 },
    { 0, 0 },
    { 0xbeee46b8, 87 },
    { 0, 0 },
    { 0xaa1e46ba, 67 },
    { 0x586e46bb, 52 },
    { 0, 0 },
    { 0xffde46bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345e46c1, 84 },
    { 0, 0 },
    { 0xccfe46c3, 61 },
    { 0, 0 },
    { 0xd75e46c5, 33 },
    { 0, 0 },
    { 0x634e46c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e46ce, 78 },
    { 0x36ee46cf, 33 },
    { 0x7bae46d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddce46d6, 98 },
    { 0, 0 },
    { 0x4e7e46d8, 90 },
    { 0, 0 },
    { 0xe31e46da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33e46dd, 34 },
    { 0, 0 },
    { 0x86e46df, 34 },
    { 0, 0 },
    { 0x8bee46e1, 95 },
    { 0x46e46e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ae46f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe46f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e46f8, 98 },
    { 0xfede46f8, 5 },
    { 0, 0 },
    { 0x7dee46fb, 95 },
    { 0, 0 },
    { 0xd02e46fd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fae4703, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5e470b, 90 },
    { 0x79ae470c, 99 },
    { 0xdeee470d, 66 },
    { 0x59ae470e, 90 },
    { 0xe5de470f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9e4715, 68 },
    { 0x51be4715, 93 },
    { 0, 0 },
    { 0x8c8e4718, 90 },
    { 0x8fee4719, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ee471d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e4727, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x274e472a, 95 },
    { 0, 0 },
    { 0xacae472c, 33 },
    { 0x917e472d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdede4731, 67 },
    { 0, 0 },
    { 0x5fe4733, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9e4738, 33 },
    { 0, 0 },
    { 0x36e473a, 95 },
    { 0x290e473a, 17 },
    { 0, 0 },
    { 0xffce473d, 98 },
    { 0x510e473e, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15de4743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0e4746, 66 },
    { 0, 0 },
    { 0x8d6e4748, 43 },
    { 0x5b5e4749, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fe474c, 90 },
    { 0, 0 },
    { 0x238e474e, 67 },
    { 0x50ce474f, 33 },
    { 0xc7ce4750, 33 },
    { 0x737e474e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ce4756, 76 },
    { 0xf7ce4757, 90 },
    { 0x28fe4758, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916e475c, 90 },
    { 0x8f9e475c, 43 },
    { 0x54ae475e, 61 },
    { 0xf27e475f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84e4762, 55 },
    { 0x4dee4763, 46 },
    { 0xeaee4763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6e476d, 98 },
    { 0xe76e476e, 90 },
    { 0xd32e476f, 90 },
    { 0xe78e476f, 90 },
    { 0xd08e4770, 23 },
    { 0x90de476f, 95 },
    { 0x541e4773, 98 },
    { 0x391e476e, 55 },
    { 0x852e4775, 43 },
    { 0xd40e4776, 33 },
    { 0x81be4771, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ae477a, 98 },
    { 0x5e4e477a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fe477f, 61 },
    { 0xa4fe4780, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0e4784, 90 },
    { 0, 0 },
    { 0x2e8e4786, 52 },
    { 0, 0 },
    { 0x1d2e4788, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274e478f, 46 },
    { 0x1efe4790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6e4796, 33 },
    { 0xb51e4797, 33 },
    { 0x579e4798, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ce479b, 95 },
    { 0x78de479b, 98 },
    { 0xcd7e479d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe47a0, 34 },
    { 0, 0 },
    { 0x4a1e47a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22e47ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59e47c1, 95 },
    { 0x32ce47c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991e47c7, 100 },
    { 0, 0 },
    { 0x45ee47c9, 93 },
    { 0x94be47ca, 90 },
    { 0xca7e47cb, 78 },
    { 0, 0 },
    { 0x83de47cd, 98 },
    { 0xf76e47ce, 33 },
    { 0xbd5e47ce, 34 },
    { 0x2d4e47ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1e47d8, 23 },
    { 0, 0 },
    { 0x8dce47da, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74be47e0, 90 },
    { 0x131e47e1, 90 },
    { 0x841e47e2, 55 },
    { 0xec7e47e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e47e9, 49 },
    { 0xbd1e47ea, 76 },
    { 0x636e47e9, 43 },
    { 0, 0 },
    { 0xd9ae47ed, 93 },
    { 0x60be47ee, 55 },
    { 0xbe3e47ef, 85 },
    { 0x331e47f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe47f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x327e47ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e480e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59e4812, 33 },
    { 0x564e4813, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95e4819, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x896e481c, 95 },
    { 0x94ce481d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0e4826, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443e482b, 78 },
    { 0, 0 },
    { 0x362e482d, 90 },
    { 0x29ce482e, 98 },
    { 0x644e482d, 90 },
    { 0xc68e4830, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40e4834, 95 },
    { 0, 0 },
    { 0x5a3e4836, 34 },
    { 0x519e4837, 95 },
    { 0x547e4836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9e4841, 98 },
    { 0x67ee4841, 36 },
    { 0, 0 },
    { 0xc57e4844, 98 },
    { 0x9f8e4845, 66 },
    { 0xe7ce4846, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefde4849, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1e484e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf25e4852, 61 },
    { 0xa38e4853, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e4857, 95 },
    { 0x700e4857, 46 },
    { 0x81fe4858, 33 },
    { 0, 0 },
    { 0x29be485b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x359e485e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bce4861, 91 },
    { 0x45fe4862, 98 },
    { 0x15be4862, 61 },
    { 0xaa1e4864, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88e486a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29e486f, 56 },
    { 0, 0 },
    { 0x34ee4871, 76 },
    { 0x12be4872, 98 },
    { 0x9fce4873, 90 },
    { 0, 0 },
    { 0x34de4875, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23de4879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4e487c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1e4880, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81be4884, 101 },
    { 0, 0 },
    { 0xb30e4886, 61 },
    { 0x820e4887, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22e488c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2e4891, 98 },
    { 0xcd4e4892, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4e4895, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e4899, 61 },
    { 0x8dfe4899, 95 },
    { 0x3c8e489b, 76 },
    { 0, 0 },
    { 0xe95e489d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255e48a7, 23 },
    { 0x411e48a7, 34 },
    { 0x17fe48a9, 87 },
    { 0x374e48a9, 90 },
    { 0x3c7e48a8, 95 },
    { 0x9ace48ac, 34 },
    { 0x9eae48ad, 23 },
    { 0xed5e48ae, 90 },
    { 0x495e48af, 55 },
    { 0, 0 },
    { 0x4f9e48b1, 76 },
    { 0, 0 },
    { 0x104e48b3, 78 },
    { 0xe63e48b4, 95 },
    { 0xb71e48b5, 34 },
    { 0, 0 },
    { 0xf77e48b7, 55 },
    { 0, 0 },
    { 0xc16e48b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35e48bd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05e48c7, 91 },
    { 0xabee48c8, 33 },
    { 0xf4ce48c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207e48ce, 55 },
    { 0xa3fe48ce, 61 },
    { 0xd3de48d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe48db, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296e48df, 98 },
    { 0x734e48e0, 61 },
    { 0x40e48e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46e48e6, 61 },
    { 0x539e48e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6e48ec, 95 },
    { 0, 0 },
    { 0xcf0e48ee, 14 },
    { 0, 0 },
    { 0x72ae48f0, 75 },
    { 0, 0 },
    { 0xb82e48f2, 98 },
    { 0, 0 },
    { 0xabde48f4, 17 },
    { 0x857e48f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95e48f9, 99 },
    { 0xffce48fa, 64 },
    { 0xba8e48fb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90be48ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95e4907, 100 },
    { 0x673e4907, 90 },
    { 0x8aae4907, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5e4911, 33 },
    { 0, 0 },
    { 0xc8ce4913, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfbe4916, 52 },
    { 0x824e4917, 68 },
    { 0x530e4918, 23 },
    { 0x259e4917, 90 },
    { 0, 0 },
    { 0x1afe491b, 61 },
    { 0x2bde491b, 95 },
    { 0xdd8e491d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18be4921, 33 },
    { 0xe31e4922, 98 },
    { 0xb43e4923, 23 },
    { 0, 0 },
    { 0xd8ce4925, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55e4929, 55 },
    { 0x900e4929, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e4933, 100 },
    { 0x745e4934, 52 },
    { 0xf34e4935, 95 },
    { 0x265e4933, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82e493a, 67 },
    { 0, 0 },
    { 0xf32e493c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ee493f, 61 },
    { 0x62ae4940, 95 },
    { 0, 0 },
    { 0xe4ae4942, 23 },
    { 0x425e4942, 93 },
    { 0, 0 },
    { 0xf3fe4945, 36 },
    { 0, 0 },
    { 0x875e4947, 90 },
    { 0xe00e4947, 90 },
    { 0x68de4949, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ae494d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe4957, 94 },
    { 0xe81e4957, 34 },
    { 0xd91e4958, 98 },
    { 0, 0 },
    { 0x897e495b, 78 },
    { 0x115e495c, 23 },
    { 0, 0 },
    { 0x484e495e, 33 },
    { 0xf83e495f, 55 },
    { 0x696e495e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x727e4964, 46 },
    { 0x780e4965, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x344e4968, 61 },
    { 0, 0 },
    { 0xeae496a, 61 },
    { 0, 0 },
    { 0x599e496c, 61 },
    { 0xe5ce496c, 90 },
    { 0, 0 },
    { 0xa71e496f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683e4975, 90 },
    { 0x70fe4976, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e497d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ce4980, 93 },
    { 0xf7be4980, 55 },
    { 0x695e4982, 33 },
    { 0x707e4983, 23 },
    { 0x1b3e4983, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7e4989, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x337e498d, 76 },
    { 0x55de498e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402e4993, 98 },
    { 0x71de4994, 33 },
    { 0x4fce4993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ce499c, 43 },
    { 0, 0 },
    { 0x614e499e, 65 },
    { 0xa78e499f, 23 },
    { 0x52ce49a0, 90 },
    { 0xfaae49a0, 61 },
    { 0xbdae49a1, 95 },
    { 0, 0 },
    { 0xc7e49a4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95e49a8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eae49ab, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x83de49ae, 43 },
    { 0x2ffe49ae, 67 },
    { 0x9aee49b0, 33 },
    { 0x6bae49b1, 40 },
    { 0x85e49b2, 77 },
    { 0xbd6e49b0, 93 },
    { 0x67be49b4, 97 },
    { 0, 0 },
    { 0x2c2e49b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7e49b9, 52 },
    { 0x331e49ba, 90 },
    { 0, 0 },
    { 0x2c7e49bc, 33 },
    { 0xed6e49bc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbe49c0, 98 },
    { 0x609e49c0, 95 },
    { 0, 0 },
    { 0xc71e49c3, 98 },
    { 0x658e49c4, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440e49cb, 87 },
    { 0, 0 },
    { 0x41be49cd, 61 },
    { 0x1e5e49cd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x248e49d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29e49d6, 76 },
    { 0xbade49d7, 61 },
    { 0x764e49d8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15be49df, 61 },
    { 0, 0 },
    { 0x635e49e1, 90 },
    { 0x73ce49e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ae49e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce49eb, 23 },
    { 0x6dfe49ec, 98 },
    { 0x2dae49ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e49f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1e49f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ede49f8, 33 },
    { 0xb58e49f9, 95 },
    { 0xe6ce49fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde49fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bce4a00, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9be4a03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ee4a06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e4a0c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41e4a11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ee4a15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fe4a1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54e4a23, 90 },
    { 0xfdce4a24, 23 },
    { 0x121e4a24, 46 },
    { 0xb7de4a26, 52 },
    { 0xa89e4a27, 90 },
    { 0, 0 },
    { 0x271e4a29, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213e4a3b, 90 },
    { 0, 0 },
    { 0x938e4a3d, 33 },
    { 0, 0 },
    { 0x90de4a3f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fe4a44, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2e4a48, 55 },
    { 0x5de4a48, 95 },
    { 0x5b0e4a4a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21e4a4d, 90 },
    { 0x6a1e4a4e, 98 },
    { 0, 0 },
    { 0x886e4a50, 23 },
    { 0, 0 },
    { 0x2c8e4a52, 43 },
    { 0, 0 },
    { 0x4bde4a54, 78 },
    { 0x46ee4a54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237e4a60, 33 },
    { 0xb75e4a61, 90 },
    { 0x303e4a62, 77 },
    { 0xa7fe4a63, 33 },
    { 0x4dfe4a64, 52 },
    { 0xf82e4a65, 33 },
    { 0xb19e4a65, 76 },
    { 0xd36e4a66, 61 },
    { 0x502e4a63, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2be4a6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439e4a71, 77 },
    { 0x3b7e4a71, 96 },
    { 0x771e4a72, 95 },
    { 0xb15e4a74, 33 },
    { 0xdf8e4a75, 95 },
    { 0xa4ae4a76, 52 },
    { 0x54e4a77, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x667e4a7a, 83 },
    { 0, 0 },
    { 0xc24e4a7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fe4a84, 89 },
    { 0x124e4a84, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3e4a89, 84 },
    { 0xa5fe4a8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62de4a8f, 33 },
    { 0x85e4a90, 23 },
    { 0x259e4a90, 33 },
    { 0xeb5e4a92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19e4a96, 61 },
    { 0x607e4a97, 83 },
    { 0xcc8e4a98, 80 },
    { 0xb97e4a99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186e4a9e, 87 },
    { 0xa7be4a9f, 33 },
    { 0, 0 },
    { 0x412e4aa1, 100 },
    { 0xe61e4aa2, 64 },
    { 0, 0 },
    { 0x8b5e4aa4, 61 },
    { 0xd27e4aa4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9e4aaa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ce4aaf, 95 },
    { 0x166e4aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50e4ab3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8e4abc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8e4ac0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe4ac4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabe4acd, 34 },
    { 0x4a8e4ace, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ce4ad1, 76 },
    { 0xe62e4ad1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24e4ad5, 23 },
    { 0x3fee4ad6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ae4adb, 55 },
    { 0x29ee4adb, 33 },
    { 0x125e4adb, 61 },
    { 0xaf5e4ade, 93 },
    { 0x228e4adf, 55 },
    { 0x6c5e4adb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3e4ae6, 61 },
    { 0xa57e4ae7, 93 },
    { 0x376e4ae7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fde4aec, 90 },
    { 0, 0 },
    { 0x736e4aee, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0e4af1, 95 },
    { 0xdce4af2, 45 },
    { 0xe4ae4af2, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77e4af6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e4afd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641e4b01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e4b08, 84 },
    { 0xedce4b09, 55 },
    { 0, 0 },
    { 0x9c0e4b0b, 98 },
    { 0xeee4b0c, 98 },
    { 0xedae4b0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271e4b12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe4b16, 98 },
    { 0xf4ae4b17, 33 },
    { 0x908e4b18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace4b1b, 61 },
    { 0xe93e4b1c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7e4b25, 34 },
    { 0x11ae4b26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497e4b2b, 34 },
    { 0x6c4e4b2c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce4b2f, 33 },
    { 0xba3e4b30, 61 },
    { 0x9e5e4b31, 33 },
    { 0xcee4b31, 98 },
    { 0, 0 },
    { 0xa0de4b34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9de4b37, 34 },
    { 0x7b5e4b38, 52 },
    { 0xc8ce4b38, 33 },
    { 0x256e4b37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ece4b3e, 61 },
    { 0, 0 },
    { 0xe2de4b40, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x447e4b43, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91e4b4c, 36 },
    { 0x53de4b4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8e4b50, 98 },
    { 0xbd9e4b50, 90 },
    { 0, 0 },
    { 0xd1fe4b53, 43 },
    { 0x153e4b54, 90 },
    { 0x825e4b55, 95 },
    { 0, 0 },
    { 0x3e0e4b57, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6e4b5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fe4b5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bfe4b63, 95 },
    { 0x299e4b64, 67 },
    { 0x539e4b65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32be4b6a, 33 },
    { 0xb6ee4b6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895e4b71, 90 },
    { 0xb91e4b72, 66 },
    { 0x22fe4b73, 90 },
    { 0x6b5e4b74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91e4b79, 46 },
    { 0xe87e4b7a, 95 },
    { 0xae3e4b7a, 98 },
    { 0, 0 },
    { 0x4f4e4b7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffe4b82, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436e4b8a, 95 },
    { 0x279e4b8b, 43 },
    { 0x647e4b8c, 87 },
    { 0xd81e4b8d, 76 },
    { 0x6fde4b8c, 90 },
    { 0x56e4b8f, 87 },
    { 0xdfce4b90, 95 },
    { 0x63fe4b91, 36 },
    { 0xf97e4b8f, 55 },
    { 0x21ee4b8a, 81 },
    { 0x296e4b94, 95 },
    { 0, 0 },
    { 0xc9e4b96, 90 },
    { 0xf28e4b96, 90 },
    { 0x162e4b98, 95 },
    { 0x9b9e4b99, 93 },
    { 0xe4fe4b9a, 68 },
    { 0, 0 },
    { 0xd97e4b9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ce4ba0, 43 },
    { 0x16de4ba1, 88 },
    { 0, 0 },
    { 0x3ece4ba3, 61 },
    { 0xa56e4ba4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633e4baa, 97 },
    { 0, 0 },
    { 0x2bce4bac, 33 },
    { 0xae9e4bac, 98 },
    { 0xb2e4bae, 90 },
    { 0x8bae4bad, 52 },
    { 0, 0 },
    { 0xc07e4bb1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ece4bba, 55 },
    { 0x381e4bbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4e4bbe, 88 },
    { 0x23be4bbf, 55 },
    { 0x383e4bc0, 55 },
    { 0, 0 },
    { 0xe05e4bc2, 23 },
    { 0x96be4bc3, 90 },
    { 0x1be4bc4, 76 },
    { 0x627e4bc5, 33 },
    { 0x5e7e4bc5, 65 },
    { 0, 0 },
    { 0xf9ae4bc8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de4bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce4bd2, 23 },
    { 0x687e4bd3, 33 },
    { 0x1d7e4bd3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e4bd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5e4bdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fe4be3, 23 },
    { 0x553e4be3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65e4be8, 95 },
    { 0xcabe4be9, 81 },
    { 0, 0 },
    { 0xf35e4beb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09e4bef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e4bf2, 4 },
    { 0x23de4bf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ae4bf9, 90 },
    { 0, 0 },
    { 0xacde4bfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621e4c00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10e4c03, 90 },
    { 0x2c2e4c04, 90 },
    { 0xea1e4c05, 43 },
    { 0x555e4c04, 90 },
    { 0x6f4e4c05, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ae4c0a, 91 },
    { 0, 0 },
    { 0xcc3e4c0c, 91 },
    { 0, 0 },
    { 0xcd3e4c0e, 98 },
    { 0xcfee4c0f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8e4c19, 82 },
    { 0xa3fe4c19, 88 },
    { 0x7c6e4c19, 23 },
    { 0xb02e4c1c, 100 },
    { 0xbc3e4c1c, 87 },
    { 0x9bbe4c1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x782e4c21, 33 },
    { 0, 0 },
    { 0x234e4c23, 34 },
    { 0xf9ee4c24, 33 },
    { 0x8ede4c25, 99 },
    { 0xcebe4c25, 76 },
    { 0x372e4c25, 90 },
    { 0x247e4c28, 67 },
    { 0x1e5e4c29, 90 },
    { 0xa8ee4c27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dee4c2d, 40 },
    { 0x3b6e4c2e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed9e4c32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917e4c36, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356e4c3a, 48 },
    { 0x88be4c3b, 43 },
    { 0x8b4e4c3c, 90 },
    { 0x906e4c3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e4c46, 95 },
    { 0, 0 },
    { 0xc5ee4c48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfce4c4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aee4c55, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x77e4c58, 90 },
    { 0x2dde4c59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75be4c5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1e4c63, 61 },
    { 0x125e4c63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871e4c6a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7e4c6e, 17 },
    { 0xbdae4c6f, 34 },
    { 0xfe3e4c6e, 90 },
    { 0, 0 },
    { 0x17be4c72, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe4c75, 61 },
    { 0x342e4c76, 90 },
    { 0xe24e4c77, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19be4c7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26e4c7f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdae4c83, 61 },
    { 0x217e4c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcefe4c87, 99 },
    { 0x1de4c87, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8e4c8f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ce4c93, 8 },
    { 0x254e4c94, 77 },
    { 0, 0 },
    { 0x786e4c96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61e4c99, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835e4c9f, 81 },
    { 0x7aae4c9f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce4ca3, 61 },
    { 0x243e4ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afe4caa, 65 },
    { 0, 0 },
    { 0xfece4cac, 95 },
    { 0, 0 },
    { 0xa00e4cae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e4cb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x515e4cb4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x474e4cb7, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x143e4cba, 90 },
    { 0, 0 },
    { 0xd11e4cbc, 95 },
    { 0x495e4cbd, 23 },
    { 0, 0 },
    { 0xd83e4cbf, 46 },
    { 0xec1e4cbf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7e4cc4, 33 },
    { 0x5dde4cc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e4cce, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e4cd1, 92 },
    { 0x209e4cd2, 98 },
    { 0x44de4cd1, 80 },
    { 0xfb3e4cd4, 36 },
    { 0x9d8e4cd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e4ce2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e4ce6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7e4ceb, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82e4cef, 76 },
    { 0xbbde4cf0, 91 },
    { 0xd7e4cef, 93 },
    { 0x46ee4cf2, 95 },
    { 0x41ee4cf3, 90 },
    { 0, 0 },
    { 0xd28e4cf5, 33 },
    { 0xbc5e4cf6, 43 },
    { 0x329e4cf6, 87 },
    { 0, 0 },
    { 0x63ce4cf9, 90 },
    { 0, 0 },
    { 0xb4e4cfb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e4cfe, 61 },
    { 0xf4ce4cfe, 61 },
    { 0x373e4d00, 84 },
    { 0xd9e4d01, 99 },
    { 0, 0 },
    { 0x469e4d03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473e4d07, 99 },
    { 0, 0 },
    { 0x71ee4d09, 55 },
    { 0, 0 },
    { 0xc3e4d0b, 61 },
    { 0, 0 },
    { 0x1d2e4d0d, 43 },
    { 0, 0 },
    { 0x20be4d0f, 34 },
    { 0x264e4d10, 88 },
    { 0x3bbe4d10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612e4d18, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05e4d1e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbe4d23, 98 },
    { 0x91fe4d24, 43 },
    { 0xe53e4d24, 76 },
    { 0x790e4d25, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50e4d29, 98 },
    { 0x54be4d2a, 93 },
    { 0, 0 },
    { 0xbffe4d2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912e4d33, 76 },
    { 0xdb8e4d34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12e4d38, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2e4d3c, 98 },
    { 0, 0 },
    { 0x3f9e4d3e, 36 },
    { 0x987e4d3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124e4d44, 93 },
    { 0x5ffe4d45, 90 },
    { 0, 0 },
    { 0xe02e4d47, 95 },
    { 0, 0 },
    { 0xcc9e4d49, 23 },
    { 0x77ce4d4a, 98 },
    { 0x432e4d4b, 61 },
    { 0xc56e4d4c, 23 },
    { 0, 0 },
    { 0x932e4d4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce4d51, 98 },
    { 0, 0 },
    { 0xb15e4d53, 87 },
    { 0x8dae4d53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1e4d57, 66 },
    { 0, 0 },
    { 0xdb3e4d59, 91 },
    { 0x645e4d5a, 55 },
    { 0x3c8e4d5b, 33 },
    { 0, 0 },
    { 0xd93e4d5d, 46 },
    { 0x7a3e4d5e, 46 },
    { 0x44e4d5e, 91 },
    { 0x7c5e4d5f, 90 },
    { 0, 0 },
    { 0xc88e4d62, 95 },
    { 0xa26e4d63, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ae4d6d, 68 },
    { 0xeeee4d6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebe4d71, 34 },
    { 0x87fe4d72, 34 },
    { 0xcd8e4d71, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1e4d76, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4e4d7a, 94 },
    { 0xf73e4d7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208e4d80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887e4d84, 66 },
    { 0xb5ae4d84, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82e4d88, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4e4d93, 90 },
    { 0x207e4d94, 61 },
    { 0, 0 },
    { 0x721e4d96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5e4d9a, 90 },
    { 0, 0 },
    { 0x65fe4d9c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x333e4d9f, 61 },
    { 0x902e4da0, 93 },
    { 0, 0 },
    { 0x5be4da2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5e4da6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32e4da9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de4daf, 33 },
    { 0x1f3e4db0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02e4db4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ce4db7, 61 },
    { 0x714e4db7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840e4dbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25de4dc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69e4dc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaade4dc8, 67 },
    { 0x3f9e4dc9, 90 },
    { 0, 0 },
    { 0x162e4dcb, 95 },
    { 0x43be4dcc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1e4dcf, 43 },
    { 0x88ee4dcf, 33 },
    { 0x6a8e4dd1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dee4dd6, 99 },
    { 0x2aae4dd7, 43 },
    { 0x7cfe4dd8, 81 },
    { 0xd4de4dd8, 34 },
    { 0xdc8e4dda, 87 },
    { 0x18ce4dd8, 23 },
    { 0x8fe4dd7, 34 },
    { 0x91de4dd9, 90 },
    { 0xa9de4dde, 43 },
    { 0x8dde4ddf, 68 },
    { 0xb9de4ddf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19e4de8, 61 },
    { 0xa2e4de9, 46 },
    { 0xb84e4dea, 46 },
    { 0x460e4deb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4e4df1, 98 },
    { 0, 0 },
    { 0x2dfe4df3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58e4df6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314e4dfa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bae4dfd, 55 },
    { 0xeb0e4dfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33e4e04, 95 },
    { 0xe08e4e05, 87 },
    { 0xb04e4e05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ce4e0b, 90 },
    { 0x6cfe4e0b, 61 },
    { 0x6b9e4e0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccce4e11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dee4e14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4e4e1c, 17 },
    { 0xa18e4e1d, 95 },
    { 0xfe9e4e1e, 65 },
    { 0, 0 },
    { 0xe9fe4e20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25e4e23, 93 },
    { 0, 0 },
    { 0xd29e4e25, 61 },
    { 0x93ee4e26, 46 },
    { 0xc59e4e27, 95 },
    { 0x65ae4e28, 53 },
    { 0xd67e4e29, 78 },
    { 0x8f1e4e27, 95 },
    { 0xd83e4e2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f8e4e30, 61 },
    { 0x93fe4e30, 84 },
    { 0x507e4e31, 46 },
    { 0x5e6e4e33, 61 },
    { 0x5e6e4e34, 52 },
    { 0x5cfe4e33, 55 },
    { 0, 0 },
    { 0x9e5e4e37, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260e4e3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ce4e3f, 33 },
    { 0xe52e4e3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95e4e4e, 90 },
    { 0x324e4e4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4e4e52, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce4e58, 52 },
    { 0x720e4e59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x790e4e5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625e4e60, 81 },
    { 0x294e4e61, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e4e64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e4e69, 46 },
    { 0xcaae4e6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10be4e70, 33 },
    { 0xb83e4e71, 23 },
    { 0xb01e4e72, 17 },
    { 0x6a8e4e70, 90 },
    { 0xffe4e74, 61 },
    { 0x3f9e4e72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x59de4e78, 46 },
    { 0x760e4e78, 33 },
    { 0x547e4e79, 33 },
    { 0, 0 },
    { 0xc01e4e7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbe4e7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe4e82, 33 },
    { 0xe50e4e82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113e4e88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce4e8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8e4e91, 90 },
    { 0x32ce4e92, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x600e4e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810e4e9d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e4ea1, 65 },
    { 0, 0 },
    { 0xd0ee4ea3, 46 },
    { 0xed3e4ea3, 61 },
    { 0, 0 },
    { 0x213e4ea6, 99 },
    { 0, 0 },
    { 0x517e4ea8, 88 },
    { 0, 0 },
    { 0xa76e4eaa, 69 },
    { 0, 0 },
    { 0xc29e4eac, 85 },
    { 0, 0 },
    { 0x4cae4eae, 33 },
    { 0x8a5e4eaf, 98 },
    { 0xd44e4eb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cae4eb6, 85 },
    { 0xf8de4eb7, 65 },
    { 0xf8ee4eb8, 34 },
    { 0x918e4eb9, 23 },
    { 0xbb3e4eba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ae4ebd, 23 },
    { 0x38be4ebd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1e4ec1, 96 },
    { 0xb3ee4ec1, 33 },
    { 0x7a8e4ec3, 98 },
    { 0x16be4ec4, 90 },
    { 0x108e4ec1, 61 },
    { 0xbdde4ec6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e4ecc, 55 },
    { 0xdcee4ecc, 69 },
    { 0x487e4ece, 33 },
    { 0xc41e4ecf, 90 },
    { 0x6f0e4ecf, 33 },
    { 0xd8fe4ecc, 33 },
    { 0x880e4ed1, 33 },
    { 0x50de4ecd, 43 },
    { 0xed4e4ed4, 90 },
    { 0x7b8e4ed5, 68 },
    { 0x2a7e4ed6, 23 },
    { 0x197e4ed6, 76 },
    { 0, 0 },
    { 0x773e4ed9, 95 },
    { 0x9c7e4ed9, 33 },
    { 0x2a3e4edb, 87 },
    { 0x323e4edc, 23 },
    { 0x560e4edc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dde4ee0, 100 },
    { 0x9fbe4ee0, 23 },
    { 0xe97e4ee2, 90 },
    { 0x2e0e4ee1, 95 },
    { 0x5abe4ee2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ee4ee8, 76 },
    { 0x6c4e4ee9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ae4eee, 93 },
    { 0, 0 },
    { 0x90ae4ef0, 43 },
    { 0xd4e4ef1, 23 },
    { 0x867e4ef2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be4ef5, 61 },
    { 0, 0 },
    { 0xc46e4ef7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38e4efa, 84 },
    { 0xbfae4efa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ee4efe, 95 },
    { 0x9bee4efe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8e4f0c, 67 },
    { 0xa19e4f0d, 33 },
    { 0x4dee4f0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefce4f12, 98 },
    { 0, 0 },
    { 0x48e4f14, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9e4f17, 90 },
    { 0, 0 },
    { 0xb8ce4f19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3de4f1e, 23 },
    { 0x75ae4f1f, 81 },
    { 0xd68e4f1f, 95 },
    { 0x94be4f21, 98 },
    { 0, 0 },
    { 0x1dbe4f23, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cae4f2a, 34 },
    { 0x3aae4f2b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf6e4f2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbde4f3b, 90 },
    { 0x259e4f3c, 78 },
    { 0x2eee4f3c, 61 },
    { 0x834e4f3e, 76 },
    { 0xcdce4f3c, 33 },
    { 0, 0 },
    { 0x40ae4f41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x694e4f44, 90 },
    { 0x126e4f45, 46 },
    { 0x1a0e4f46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e4f4b, 23 },
    { 0, 0 },
    { 0x1dae4f4d, 55 },
    { 0, 0 },
    { 0x7d0e4f4f, 99 },
    { 0x3a9e4f50, 66 },
    { 0xcdbe4f4f, 65 },
    { 0x450e4f52, 55 },
    { 0xa8ee4f51, 43 },
    { 0xa3de4f54, 52 },
    { 0, 0 },
    { 0x23fe4f56, 95 },
    { 0, 0 },
    { 0x58be4f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x885e4f5b, 47 },
    { 0x316e4f5b, 91 },
    { 0x98e4f5d, 23 },
    { 0x852e4f5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2e4f63, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7e4f67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ce4f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91e4f70, 95 },
    { 0, 0 },
    { 0x885e4f72, 36 },
    { 0, 0 },
    { 0xba1e4f74, 61 },
    { 0, 0 },
    { 0x99be4f76, 84 },
    { 0, 0 },
    { 0x286e4f78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6e4f7c, 34 },
    { 0xe4fe4f7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42e4f80, 33 },
    { 0xf31e4f81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2e4f84, 83 },
    { 0, 0 },
    { 0x8b9e4f86, 46 },
    { 0, 0 },
    { 0xf29e4f88, 99 },
    { 0x2cce4f88, 55 },
    { 0x4ae4f8a, 90 },
    { 0, 0 },
    { 0xd0ce4f8c, 95 },
    { 0, 0 },
    { 0xe90e4f8e, 90 },
    { 0xbf5e4f8f, 90 },
    { 0x56de4f90, 79 },
    { 0, 0 },
    { 0x86ce4f92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x107e4f95, 33 },
    { 0x8a6e4f96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9e4f99, 43 },
    { 0x899e4f9a, 98 },
    { 0, 0 },
    { 0x61e4f9c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ce4fa1, 61 },
    { 0, 0 },
    { 0xfa9e4fa3, 90 },
    { 0xc4ae4fa4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x677e4fa7, 93 },
    { 0x72be4fa8, 61 },
    { 0x639e4fa9, 93 },
    { 0x271e4fa9, 90 },
    { 0xb20e4fab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748e4fb0, 12 },
    { 0x211e4fb1, 90 },
    { 0xece4fb0, 95 },
    { 0xaa6e4fb3, 33 },
    { 0, 0 },
    { 0x89fe4fb5, 95 },
    { 0, 0 },
    { 0xe47e4fb7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ae4fbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40be4fc2, 95 },
    { 0x59ce4fc3, 55 },
    { 0, 0 },
    { 0x9e4fc5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ee4fcd, 34 },
    { 0xfafe4fce, 80 },
    { 0x20e4fcf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ce4fd2, 55 },
    { 0, 0 },
    { 0xaeee4fd4, 90 },
    { 0xc8de4fd5, 95 },
    { 0xee0e4fd4, 80 },
    { 0x407e4fd5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56be4fdc, 23 },
    { 0x79fe4fdc, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372e4fe4, 78 },
    { 0xc2fe4fe5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24be4fea, 33 },
    { 0x52fe4feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350e4ff0, 90 },
    { 0xdb3e4ff0, 93 },
    { 0x86e4ff2, 90 },
    { 0xa87e4ff2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ff7, 33 },
    { 0x910e4ff8, 78 },
    { 0x763e4ff9, 76 },
    { 0x1f7e4ffa, 46 },
    { 0x62be4ffb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x464e4ffe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e5001, 45 },
    { 0xe7ce5002, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fe500b, 65 },
    { 0, 0 },
    { 0xf0ae500d, 61 },
    { 0x77de500e, 61 },
    { 0, 0 },
    { 0xe65e5010, 46 },
    { 0, 0 },
    { 0xa1ee5012, 90 },
    { 0, 0 },
    { 0x9c0e5014, 100 },
    { 0, 0 },
    { 0x1d0e5016, 33 },
    { 0, 0 },
    { 0xdbbe5018, 65 },
    { 0x4d7e5018, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e501c, 43 },
    { 0, 0 },
    { 0x21fe501e, 53 },
    { 0x83ce501f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cee5026, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x786e5029, 46 },
    { 0x663e502a, 61 },
    { 0x103e502a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68be5031, 93 },
    { 0x8d6e5031, 55 },
    { 0xbf2e5032, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71e5036, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e503a, 52 },
    { 0x734e503a, 85 },
    { 0, 0 },
    { 0x890e503d, 90 },
    { 0, 0 },
    { 0x557e503f, 33 },
    { 0x8d7e5040, 23 },
    { 0xaefe5040, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee5044, 33 },
    { 0xbe5e5044, 90 },
    { 0, 0 },
    { 0x2d2e5047, 98 },
    { 0, 0 },
    { 0x59be5049, 88 },
    { 0, 0 },
    { 0x364e504b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3e5058, 52 },
    { 0, 0 },
    { 0xeede505a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e505d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678e5062, 93 },
    { 0xa76e5062, 95 },
    { 0x39e5064, 68 },
    { 0xa76e5064, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1e5069, 90 },
    { 0x404e5069, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4e506d, 88 },
    { 0, 0 },
    { 0x10e506f, 68 },
    { 0x19ae5070, 42 },
    { 0xee1e5071, 77 },
    { 0xd46e5072, 96 },
    { 0, 0 },
    { 0x925e5074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88e5080, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4e5084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d5e5087, 98 },
    { 0xc11e5088, 83 },
    { 0xad0e5088, 90 },
    { 0xdc4e508a, 95 },
    { 0xf55e508a, 95 },
    { 0x192e5087, 43 },
    { 0x1e5e508a, 23 },
    { 0x88e508e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ae5092, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ee5097, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cee509b, 23 },
    { 0xb20e509c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae509f, 43 },
    { 0x5c6e50a0, 33 },
    { 0x1d3e50a0, 33 },
    { 0x6ebe509f, 61 },
    { 0, 0 },
    { 0xef1e50a4, 33 },
    { 0xb4be50a5, 66 },
    { 0x899e50a6, 90 },
    { 0x832e50a7, 90 },
    { 0xa31e50a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481e50ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae50af, 61 },
    { 0xd38e50b0, 69 },
    { 0xf9ae50b1, 78 },
    { 0x6bee50b1, 33 },
    { 0x1bee50b2, 67 },
    { 0x70ae50b0, 90 },
    { 0, 0 },
    { 0x6f4e50b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0e50b9, 75 },
    { 0x276e50ba, 23 },
    { 0xff2e50b9, 90 },
    { 0x48ee50bb, 90 },
    { 0, 0 },
    { 0x3a5e50be, 33 },
    { 0xea7e50be, 61 },
    { 0x64ae50c0, 90 },
    { 0, 0 },
    { 0xd84e50c2, 55 },
    { 0xe5ae50c3, 93 },
    { 0xd3fe50c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ce50c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaae50cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424e50d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741e50d6, 43 },
    { 0x356e50d6, 36 },
    { 0, 0 },
    { 0x263e50d9, 95 },
    { 0, 0 },
    { 0x973e50db, 34 },
    { 0, 0 },
    { 0xb85e50dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9e50e2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74be50e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e50f2, 33 },
    { 0x63fe50f3, 98 },
    { 0x22e50f4, 66 },
    { 0xf98e50f5, 76 },
    { 0x3c6e50f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e50fc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164e5103, 91 },
    { 0, 0 },
    { 0x97de5105, 67 },
    { 0xbcae5106, 33 },
    { 0xc0fe5107, 61 },
    { 0x864e5108, 23 },
    { 0, 0 },
    { 0x569e510a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dee5111, 98 },
    { 0x4ffe5111, 66 },
    { 0, 0 },
    { 0xddfe5114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4be5117, 90 },
    { 0x452e5118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1fe5121, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x39de5124, 78 },
    { 0xd29e5124, 23 },
    { 0xd67e5125, 61 },
    { 0x438e5127, 55 },
    { 0xbd9e5128, 90 },
    { 0x88ee5128, 55 },
    { 0x865e5129, 61 },
    { 0xd97e512b, 55 },
    { 0x96be5128, 90 },
    { 0, 0 },
    { 0x342e512e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2e5131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e5134, 91 },
    { 0x8e2e5135, 88 },
    { 0xa2ce5136, 91 },
    { 0, 0 },
    { 0x2e6e5138, 90 },
    { 0xc7de5139, 78 },
    { 0x7dee5139, 65 },
    { 0xce6e5139, 61 },
    { 0x88fe5139, 90 },
    { 0x3fae513b, 23 },
    { 0x886e5139, 43 },
    { 0xac7e513f, 46 },
    { 0x297e5139, 33 },
    { 0x8d2e5141, 98 },
    { 0x9b0e5142, 76 },
    { 0xd19e5142, 61 },
    { 0x53ce5144, 23 },
    { 0x285e5144, 100 },
    { 0, 0 },
    { 0x35e5147, 90 },
    { 0xd77e5148, 78 },
    { 0x5efe5147, 95 },
    { 0xb52e514a, 23 },
    { 0x13ee514b, 91 },
    { 0x584e5148, 95 },
    { 0x5dae514c, 43 },
    { 0, 0 },
    { 0x89de514f, 95 },
    { 0x9a5e5150, 33 },
    { 0, 0 },
    { 0xf18e5152, 38 },
    { 0x23ce5153, 100 },
    { 0, 0 },
    { 0x5e1e5155, 95 },
    { 0x753e5156, 90 },
    { 0xb42e5156, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6e515a, 23 },
    { 0x22be515a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ee515e, 87 },
    { 0x9ffe515f, 66 },
    { 0x9eae5160, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6e5165, 90 },
    { 0x250e5166, 95 },
    { 0x15ae5166, 76 },
    { 0, 0 },
    { 0xf0e5169, 62 },
    { 0x1be516a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1e516d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e5172, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0e5179, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdace517c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662e5180, 98 },
    { 0, 0 },
    { 0x332e5182, 69 },
    { 0x840e5183, 99 },
    { 0xbc1e5183, 99 },
    { 0xbfae5185, 90 },
    { 0x1ade5186, 33 },
    { 0xea2e5182, 33 },
    { 0xb14e5188, 67 },
    { 0xc58e5189, 98 },
    { 0xa96e518a, 85 },
    { 0, 0 },
    { 0x81ae518c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9e518f, 98 },
    { 0xe3ae518f, 33 },
    { 0x768e5191, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96e519b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8e519e, 36 },
    { 0x4bee519f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8e51a8, 46 },
    { 0x39de51a9, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x861e51ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae51b4, 99 },
    { 0x302e51b5, 23 },
    { 0, 0 },
    { 0x3e6e51b7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ae51bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17e51c2, 55 },
    { 0xdf2e51c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07e51c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2e51ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6e51d1, 85 },
    { 0, 0 },
    { 0xe0fe51d3, 23 },
    { 0x22ee51d4, 52 },
    { 0, 0 },
    { 0x9c9e51d6, 93 },
    { 0x92be51d6, 85 },
    { 0, 0 },
    { 0xadbe51d9, 90 },
    { 0xf2ee51da, 64 },
    { 0xe31e51d9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde51de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bde51e2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541e51e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ce51ea, 90 },
    { 0x344e51ea, 90 },
    { 0xa6de51ea, 95 },
    { 0, 0 },
    { 0xd7fe51ee, 34 },
    { 0x51ee51ef, 61 },
    { 0, 0 },
    { 0xc57e51f1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ee51f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e51fa, 61 },
    { 0x384e51fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafe51fe, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65be5204, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e520b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x208e5211, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30e5214, 33 },
    { 0x119e5215, 98 },
    { 0xa9e5216, 55 },
    { 0, 0 },
    { 0xbf1e5218, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ee521b, 95 },
    { 0, 0 },
    { 0xfaee521d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2e5220, 67 },
    { 0, 0 },
    { 0xe11e5222, 61 },
    { 0x89fe5223, 33 },
    { 0xb6de5224, 90 },
    { 0x99ae5225, 87 },
    { 0x3e4e5224, 90 },
    { 0x2ace5224, 95 },
    { 0x8d3e5223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7e522d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6e5232, 33 },
    { 0xbf6e5232, 76 },
    { 0, 0 },
    { 0xb86e5235, 55 },
    { 0x2b9e5236, 95 },
    { 0, 0 },
    { 0x20fe5238, 43 },
    { 0x3d9e5239, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x525e523c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x171e523f, 93 },
    { 0x14e5240, 99 },
    { 0xa15e523f, 33 },
    { 0x6e2e5242, 49 },
    { 0, 0 },
    { 0xb4ae5244, 33 },
    { 0x115e5244, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaae5248, 87 },
    { 0x76ee5248, 95 },
    { 0x269e524a, 17 },
    { 0, 0 },
    { 0x886e524c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ee524f, 33 },
    { 0xe2ce5250, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66e5253, 55 },
    { 0x7ece5254, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fe525b, 98 },
    { 0x6a4e525b, 76 },
    { 0x871e525d, 84 },
    { 0xc5e525d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4e5261, 90 },
    { 0xf62e5261, 23 },
    { 0x5eae5262, 81 },
    { 0x449e5262, 61 },
    { 0, 0 },
    { 0x16ce5266, 33 },
    { 0, 0 },
    { 0x684e5268, 61 },
    { 0xa2ae5269, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e526e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ae5274, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36e5277, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120e527d, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3e5288, 95 },
    { 0x986e5288, 46 },
    { 0, 0 },
    { 0x71be528b, 87 },
    { 0x47ce528c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce5292, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae5295, 90 },
    { 0xb16e5296, 90 },
    { 0, 0 },
    { 0x626e5298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee529f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffe52a3, 98 },
    { 0, 0 },
    { 0x112e52a5, 36 },
    { 0x998e52a6, 46 },
    { 0x41be52a7, 76 },
    { 0x498e52a8, 100 },
    { 0x68e52a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ee52ac, 99 },
    { 0xee6e52ad, 55 },
    { 0x3cee52ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56e52b2, 33 },
    { 0x579e52b3, 78 },
    { 0, 0 },
    { 0xb52e52b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e52b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ece52bc, 46 },
    { 0xcfe52bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe52c2, 33 },
    { 0x256e52c3, 69 },
    { 0x11ce52c3, 61 },
    { 0xe1be52c2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede52c9, 61 },
    { 0xc45e52ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8e52cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5e52d2, 99 },
    { 0xf3fe52d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114e52d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ce52df, 91 },
    { 0x773e52e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944e52e6, 33 },
    { 0xcb7e52e7, 33 },
    { 0x137e52e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e52ec, 92 },
    { 0x854e52ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94e52f1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae52f7, 77 },
    { 0x280e52f7, 34 },
    { 0x91ee52f7, 90 },
    { 0, 0 },
    { 0x665e52fb, 61 },
    { 0x16be52fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4e5304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aee5308, 64 },
    { 0xf67e5309, 76 },
    { 0xa0fe5308, 33 },
    { 0x6e7e530b, 61 },
    { 0x42e530b, 95 },
    { 0x13e530d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e5314, 40 },
    { 0xc89e5315, 95 },
    { 0x70ae5316, 34 },
    { 0xa4ce5316, 90 },
    { 0x362e5315, 40 },
    { 0x3a9e5319, 43 },
    { 0xe37e531a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fe531e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x591e5321, 61 },
    { 0xd57e5321, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8e532c, 76 },
    { 0x9f2e532c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585e5333, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e5336, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de533a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362e533e, 69 },
    { 0xd22e533f, 33 },
    { 0x206e5340, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9e5347, 23 },
    { 0xd38e5348, 98 },
    { 0, 0 },
    { 0xbd3e534a, 55 },
    { 0x459e534b, 90 },
    { 0x944e534b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948e5350, 23 },
    { 0x48ae5350, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dce5354, 90 },
    { 0, 0 },
    { 0x44e5356, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5359, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x706e535c, 23 },
    { 0, 0 },
    { 0xfe1e535e, 98 },
    { 0x985e535f, 33 },
    { 0x649e5360, 90 },
    { 0x681e5361, 23 },
    { 0, 0 },
    { 0x7ede5363, 77 },
    { 0x715e5363, 90 },
    { 0, 0 },
    { 0xf73e5366, 99 },
    { 0x1be5367, 61 },
    { 0xd4be5368, 76 },
    { 0x988e5369, 61 },
    { 0x791e536a, 61 },
    { 0, 0 },
    { 0x93be536c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae5370, 33 },
    { 0, 0 },
    { 0x1b7e5372, 61 },
    { 0xb68e5373, 61 },
    { 0, 0 },
    { 0xac7e5375, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9be537a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39e537f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x795e5382, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e5385, 65 },
    { 0xd73e5386, 33 },
    { 0, 0 },
    { 0xf29e5388, 85 },
    { 0x19e5388, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee5391, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc16e5396, 93 },
    { 0xa0de5397, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626e539d, 33 },
    { 0x98ee539e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3e53a2, 101 },
    { 0x93ce53a2, 91 },
    { 0x425e53a4, 23 },
    { 0xce8e53a5, 61 },
    { 0x466e53a3, 43 },
    { 0xcd3e53a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31e53aa, 90 },
    { 0x822e53aa, 51 },
    { 0xd48e53ac, 61 },
    { 0x308e53ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54e53b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9e53ba, 34 },
    { 0, 0 },
    { 0x668e53bc, 93 },
    { 0, 0 },
    { 0xa97e53be, 95 },
    { 0, 0 },
    { 0x76ce53c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7e53c6, 43 },
    { 0xdb4e53c6, 23 },
    { 0x532e53c8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fe53cb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeae53d3, 39 },
    { 0x410e53d3, 85 },
    { 0xe22e53d5, 55 },
    { 0x11be53d5, 90 },
    { 0x72e53d4, 36 },
    { 0x7e8e53d5, 34 },
    { 0x9b2e53d9, 90 },
    { 0x9cce53d9, 34 },
    { 0xe1fe53da, 34 },
    { 0x4fde53dc, 52 },
    { 0xb6ae53d8, 95 },
    { 0x39ae53de, 77 },
    { 0xaa4e53de, 76 },
    { 0xabee53de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ee53e8, 52 },
    { 0, 0 },
    { 0x1cbe53ea, 61 },
    { 0x524e53eb, 52 },
    { 0xb38e53ec, 98 },
    { 0xa52e53ea, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73e53f0, 90 },
    { 0, 0 },
    { 0xab4e53f2, 47 },
    { 0x56be53f2, 33 },
    { 0x74fe53f3, 33 },
    { 0, 0 },
    { 0x524e53f6, 17 },
    { 0x559e53f7, 90 },
    { 0x1efe53f8, 66 },
    { 0xb26e53f9, 85 },
    { 0x3fce53f7, 95 },
    { 0x320e53fa, 33 },
    { 0xf9be53fc, 60 },
    { 0x8c0e53fb, 95 },
    { 0x22ce53f7, 67 },
    { 0x708e53f8, 34 },
    { 0x679e5400, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dae5406, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730e540b, 99 },
    { 0x5fce540c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1e540f, 23 },
    { 0x6dae540f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x570e5413, 90 },
    { 0xca2e5413, 55 },
    { 0, 0 },
    { 0x191e5416, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5419, 33 },
    { 0x69be541a, 98 },
    { 0x29de541b, 61 },
    { 0x7c7e541c, 46 },
    { 0xcafe541d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x637e5420, 90 },
    { 0, 0 },
    { 0xb9ce5422, 33 },
    { 0x5ffe5423, 90 },
    { 0x44ae5424, 77 },
    { 0xa07e5424, 43 },
    { 0x63e5426, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ee542c, 95 },
    { 0, 0 },
    { 0xcd7e542e, 95 },
    { 0x166e542e, 98 },
    { 0x7cee542f, 55 },
    { 0, 0 },
    { 0x8e1e5432, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ee5435, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de5439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49e543d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x26be5440, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efe5449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533e5450, 33 },
    { 0, 0 },
    { 0x864e5452, 61 },
    { 0x88be5453, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29e5457, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bee545c, 76 },
    { 0xa39e545d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23e5461, 90 },
    { 0x848e5462, 52 },
    { 0xfd3e5462, 43 },
    { 0, 0 },
    { 0xc22e5465, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9e5468, 90 },
    { 0x824e5469, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd49e546c, 98 },
    { 0, 0 },
    { 0xdfe546e, 100 },
    { 0x10e546e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65de547c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2e5484, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e5487, 61 },
    { 0x9c9e5488, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ce548b, 91 },
    { 0, 0 },
    { 0xbbce548d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83e5494, 98 },
    { 0x480e5494, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ee54a0, 98 },
    { 0, 0 },
    { 0xd29e54a2, 95 },
    { 0x78ce54a3, 23 },
    { 0xb2de54a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ee54a7, 90 },
    { 0x24ce54a8, 52 },
    { 0xa7ce54a7, 95 },
    { 0xb5e54aa, 99 },
    { 0x7b4e54aa, 33 },
    { 0, 0 },
    { 0xb7ee54ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8e54b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e54b7, 78 },
    { 0x19ee54b8, 90 },
    { 0xd5ae54b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe54bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x80de54bf, 78 },
    { 0, 0 },
    { 0x12e54c1, 52 },
    { 0, 0 },
    { 0xed8e54c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x579e54c6, 33 },
    { 0x4c0e54c7, 84 },
    { 0xa2ae54c7, 76 },
    { 0x742e54c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abe54cc, 95 },
    { 0, 0 },
    { 0xe80e54ce, 97 },
    { 0x653e54cf, 34 },
    { 0, 0 },
    { 0xae54d1, 69 },
    { 0, 0 },
    { 0xa7fe54d3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57e54d8, 23 },
    { 0x9fe54d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ce54e2, 88 },
    { 0x87ae54e2, 90 },
    { 0xf34e54e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e54e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226e54ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dce54f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31e54f7, 55 },
    { 0x482e54f8, 91 },
    { 0x6a9e54f8, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ee54ff, 90 },
    { 0xd79e5500, 52 },
    { 0, 0 },
    { 0x682e5502, 33 },
    { 0x528e5503, 23 },
    { 0xe39e5502, 95 },
    { 0xdb7e5503, 34 },
    { 0, 0 },
    { 0xeb3e5507, 23 },
    { 0x21be5507, 23 },
    { 0x530e5507, 61 },
    { 0x5e8e5509, 55 },
    { 0xdf3e550b, 90 },
    { 0x165e550c, 17 },
    { 0, 0 },
    { 0xf7ae550e, 33 },
    { 0x24de550f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbe5514, 95 },
    { 0x6ace5515, 61 },
    { 0xb04e5516, 100 },
    { 0xac0e5516, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56e551e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36e5521, 90 },
    { 0, 0 },
    { 0x503e5523, 33 },
    { 0xfd7e5524, 90 },
    { 0, 0 },
    { 0x1dae5526, 75 },
    { 0, 0 },
    { 0x925e5528, 19 },
    { 0x19be5529, 100 },
    { 0, 0 },
    { 0xdd6e552b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94be5534, 90 },
    { 0x4b9e5535, 61 },
    { 0x93fe5536, 49 },
    { 0xd5fe5537, 23 },
    { 0x984e5535, 90 },
    { 0xb51e5536, 33 },
    { 0xe31e5535, 23 },
    { 0xf04e553b, 95 },
    { 0x14fe553b, 33 },
    { 0xb62e553d, 84 },
    { 0xcd4e553e, 95 },
    { 0x825e5536, 34 },
    { 0x174e553a, 95 },
    { 0xb1ae553c, 90 },
    { 0x73fe5540, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e5547, 88 },
    { 0x2dde5547, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e554b, 61 },
    { 0, 0 },
    { 0x1c8e554d, 62 },
    { 0xb78e554e, 99 },
    { 0x5ae554f, 90 },
    { 0x222e554d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4e5555, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ce555a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e5561, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6e5564, 52 },
    { 0x4dee5565, 61 },
    { 0, 0 },
    { 0x87fe5567, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dce556f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549e5574, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbe5577, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceae557a, 90 },
    { 0x896e557a, 97 },
    { 0x84e557b, 33 },
    { 0x531e557d, 90 },
    { 0xf38e557d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1e5584, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ce5587, 85 },
    { 0, 0 },
    { 0x585e5589, 78 },
    { 0x613e558a, 95 },
    { 0x6dce5589, 98 },
    { 0x540e558a, 90 },
    { 0xfa6e558b, 65 },
    { 0x2bbe558b, 98 },
    { 0, 0 },
    { 0x65ce5590, 76 },
    { 0x959e5590, 85 },
    { 0x808e5592, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4e5595, 76 },
    { 0xd66e5595, 76 },
    { 0, 0 },
    { 0x2fde5598, 33 },
    { 0x5e0e5598, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce559d, 33 },
    { 0, 0 },
    { 0xa05e559f, 33 },
    { 0xd2ee559f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50e55a7, 23 },
    { 0x3e7e55a8, 90 },
    { 0xcc2e55a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x24de55ac, 34 },
    { 0xae1e55ad, 61 },
    { 0xb92e55ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201e55b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36e55b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7e55be, 98 },
    { 0, 0 },
    { 0x10ce55c0, 46 },
    { 0, 0 },
    { 0x552e55c2, 98 },
    { 0x4a8e55c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392e55c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1e55d4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce55d9, 98 },
    { 0xcd8e55da, 99 },
    { 0xb11e55db, 55 },
    { 0x548e55d9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebce55e1, 68 },
    { 0x246e55e2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecde55e8, 33 },
    { 0x5de55e9, 95 },
    { 0, 0 },
    { 0x167e55eb, 23 },
    { 0x657e55eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e55f1, 90 },
    { 0, 0 },
    { 0x5e4e55f3, 76 },
    { 0, 0 },
    { 0x495e55f5, 99 },
    { 0, 0 },
    { 0x967e55f7, 95 },
    { 0, 0 },
    { 0xa8be55f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9e55fc, 93 },
    { 0xf9e55fc, 90 },
    { 0xa3de55fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x493e5601, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94e5605, 34 },
    { 0x54ce5605, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173e560e, 88 },
    { 0xcbfe560f, 52 },
    { 0x5b7e560f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dae5616, 61 },
    { 0x623e5617, 93 },
    { 0xc75e5618, 55 },
    { 0x41ee5617, 61 },
    { 0xda5e5616, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcee5621, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ade5625, 61 },
    { 0, 0 },
    { 0xebce5627, 33 },
    { 0xa4e5628, 33 },
    { 0xf8fe5629, 61 },
    { 0xc0de562a, 88 },
    { 0x481e562a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0be5634, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e5638, 76 },
    { 0x9ebe5638, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be5640, 100 },
    { 0x509e5641, 97 },
    { 0x81e5641, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe564c, 88 },
    { 0, 0 },
    { 0x5bbe564e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975e5652, 98 },
    { 0xb04e5653, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdee5658, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e5661, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11e5669, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ce566c, 93 },
    { 0xd62e566c, 34 },
    { 0xf05e566e, 55 },
    { 0xbebe566c, 67 },
    { 0, 0 },
    { 0x8cbe5671, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46e5674, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73e5680, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e5684, 43 },
    { 0xbf8e5684, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e568d, 90 },
    { 0x44fe568e, 76 },
    { 0xd6ce568d, 90 },
    { 0xa49e5690, 51 },
    { 0xd3de568f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7e5697, 34 },
    { 0xe2ce5697, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ae56a0, 87 },
    { 0xa39e56a1, 61 },
    { 0, 0 },
    { 0xbb5e56a3, 33 },
    { 0, 0 },
    { 0xa65e56a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43e56a9, 98 },
    { 0x421e56aa, 95 },
    { 0, 0 },
    { 0xd9fe56ac, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87fe56b4, 81 },
    { 0x26fe56b5, 99 },
    { 0x6eee56b6, 34 },
    { 0, 0 },
    { 0x829e56b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x785e56bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ae56c1, 90 },
    { 0, 0 },
    { 0xc37e56c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedce56c8, 90 },
    { 0x529e56c8, 55 },
    { 0, 0 },
    { 0xfeae56cb, 85 },
    { 0x741e56cc, 90 },
    { 0xba1e56cd, 40 },
    { 0xdaae56cc, 78 },
    { 0xf7fe56cf, 93 },
    { 0xee3e56d0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98e56d9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122e56de, 46 },
    { 0, 0 },
    { 0xc34e56e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956e56e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ee56ee, 52 },
    { 0x527e56ef, 23 },
    { 0, 0 },
    { 0x5f4e56f1, 23 },
    { 0x106e56f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceae56f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34de56fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5e5701, 55 },
    { 0xb2e5702, 33 },
    { 0x202e5702, 43 },
    { 0x8bae5704, 61 },
    { 0xa27e5705, 76 },
    { 0, 0 },
    { 0xcb7e5707, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8de570e, 43 },
    { 0xbb9e570e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1e5714, 46 },
    { 0x77ce5715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5e571b, 76 },
    { 0, 0 },
    { 0xd1ee571d, 98 },
    { 0xf58e571d, 76 },
    { 0, 0 },
    { 0x6d1e5720, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e5726, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77fe5729, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47de572d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9e5738, 100 },
    { 0x717e5738, 90 },
    { 0, 0 },
    { 0x10e573b, 85 },
    { 0xf5ae573c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x985e573f, 85 },
    { 0x976e5740, 95 },
    { 0x542e5740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82e5749, 98 },
    { 0x87ee574a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82e574d, 11 },
    { 0, 0 },
    { 0x628e574f, 88 },
    { 0xb45e5750, 84 },
    { 0, 0 },
    { 0x291e5752, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x718e5755, 23 },
    { 0, 0 },
    { 0x439e5757, 98 },
    { 0xcc6e5758, 61 },
    { 0xc2e5759, 99 },
    { 0xc9e5759, 23 },
    { 0x494e5758, 98 },
    { 0x3dce575c, 90 },
    { 0x5fee5759, 75 },
    { 0xc7e575c, 90 },
    { 0xa8de575f, 33 },
    { 0x650e575f, 90 },
    { 0xc48e5761, 66 },
    { 0x83e5761, 33 },
    { 0xc9fe5763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1e5766, 55 },
    { 0x48fe5767, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x503e576f, 61 },
    { 0, 0 },
    { 0xb8be5771, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66be5777, 95 },
    { 0xfcee5778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9e577d, 91 },
    { 0xa20e577e, 93 },
    { 0, 0 },
    { 0x528e5780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x537e5783, 52 },
    { 0, 0 },
    { 0xf40e5785, 98 },
    { 0xd7ce5786, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817e578b, 61 },
    { 0x382e578c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ace578f, 88 },
    { 0, 0 },
    { 0xea7e5791, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dde5797, 23 },
    { 0, 0 },
    { 0x381e5799, 52 },
    { 0x67de5799, 90 },
    { 0, 0 },
    { 0xaabe579c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ae57a9, 76 },
    { 0x604e57a9, 23 },
    { 0, 0 },
    { 0x90fe57ac, 78 },
    { 0, 0 },
    { 0xf6ce57ae, 61 },
    { 0x139e57ae, 23 },
    { 0, 0 },
    { 0x195e57b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x532e57b4, 55 },
    { 0x389e57b5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x633e57b8, 78 },
    { 0x112e57b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eae57c1, 85 },
    { 0x7f4e57c2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5e57c5, 98 },
    { 0xd28e57c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7e57cb, 48 },
    { 0, 0 },
    { 0xef2e57cd, 88 },
    { 0, 0 },
    { 0x454e57cf, 61 },
    { 0x179e57d0, 101 },
    { 0xc7de57cf, 90 },
    { 0x18fe57d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827e57da, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ee57e0, 90 },
    { 0, 0 },
    { 0xa02e57e2, 98 },
    { 0, 0 },
    { 0x90e57e4, 65 },
    { 0, 0 },
    { 0x79fe57e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ce57e9, 91 },
    { 0xe53e57ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4e57ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1e57f0, 96 },
    { 0xc5ee57f1, 23 },
    { 0x86ae57f1, 65 },
    { 0x194e57f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x383e57f6, 93 },
    { 0, 0 },
    { 0xf7fe57f8, 48 },
    { 0, 0 },
    { 0x7d2e57fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289e57fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e5803, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8de5808, 43 },
    { 0x50de5808, 61 },
    { 0x5cae5809, 23 },
    { 0x761e580a, 33 },
    { 0x2cde580c, 23 },
    { 0, 0 },
    { 0x804e580e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68e5814, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ece5819, 36 },
    { 0x6cce581a, 98 },
    { 0, 0 },
    { 0x540e581c, 88 },
    { 0xfa3e581d, 87 },
    { 0x3b1e581e, 90 },
    { 0xac0e581f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fe5822, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbce5829, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbee582f, 98 },
    { 0, 0 },
    { 0x861e5831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0e5837, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e583b, 95 },
    { 0xd96e583c, 23 },
    { 0x499e583c, 99 },
    { 0, 0 },
    { 0x95ee583f, 46 },
    { 0x90ae5840, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ee5845, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae5848, 78 },
    { 0xa4fe5849, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6e584c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x86be584f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1e5852, 95 },
    { 0, 0 },
    { 0x1f8e5854, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76e5858, 61 },
    { 0xfa8e5859, 98 },
    { 0x610e585a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ee585e, 55 },
    { 0, 0 },
    { 0x2a6e5860, 90 },
    { 0, 0 },
    { 0xbece5862, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x325e5865, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e5869, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ede586f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9e5874, 98 },
    { 0x35de5874, 98 },
    { 0x3fbe5875, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fe587a, 55 },
    { 0xf07e587b, 33 },
    { 0x2bce587a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460e5884, 33 },
    { 0, 0 },
    { 0x55ce5886, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e5889, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8e588c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e5893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e5897, 61 },
    { 0, 0 },
    { 0x5cfe5899, 99 },
    { 0x8c2e589a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x35fe589d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01e58a8, 52 },
    { 0x35de58a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17e58af, 33 },
    { 0x4fae58b0, 55 },
    { 0x651e58b0, 61 },
    { 0xff8e58b2, 41 },
    { 0, 0 },
    { 0xa9e58b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10e58b8, 95 },
    { 0xd1ee58b9, 91 },
    { 0xd41e58b9, 90 },
    { 0x10de58ba, 33 },
    { 0, 0 },
    { 0x581e58bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4e58c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x860e58c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8e58c9, 95 },
    { 0x59ae58ca, 76 },
    { 0, 0 },
    { 0xb3be58cc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x251e58cf, 88 },
    { 0x49de58cf, 78 },
    { 0x4d9e58d1, 90 },
    { 0xd7be58d2, 55 },
    { 0x5bee58d2, 34 },
    { 0x8a5e58d4, 66 },
    { 0xd13e58d4, 4 },
    { 0, 0 },
    { 0x1bae58d7, 90 },
    { 0x729e58d8, 52 },
    { 0, 0 },
    { 0xb97e58da, 85 },
    { 0, 0 },
    { 0xae7e58dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0be58df, 90 },
    { 0x768e58e0, 34 },
    { 0x959e58e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4e58e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5e58e9, 100 },
    { 0xa00e58ea, 61 },
    { 0x6f7e58ea, 33 },
    { 0, 0 },
    { 0xf1e58ed, 77 },
    { 0x21ee58ee, 33 },
    { 0x319e58ef, 91 },
    { 0x69be58ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9e58f3, 46 },
    { 0, 0 },
    { 0x989e58f5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18be58f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253e5901, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0e5904, 88 },
    { 0, 0 },
    { 0x55ce5906, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e590a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4e590d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61e5910, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36e5918, 87 },
    { 0x30fe5918, 55 },
    { 0xd53e591a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x428e591d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0e5921, 3 },
    { 0x33de5922, 84 },
    { 0xb50e5923, 61 },
    { 0xf5be5923, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce5928, 34 },
    { 0x487e5928, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66e592e, 98 },
    { 0xe35e592f, 61 },
    { 0x438e5930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee5933, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318e5938, 90 },
    { 0x351e5939, 23 },
    { 0x880e593a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0be593e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9e5943, 78 },
    { 0xf9ee5944, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ee5947, 61 },
    { 0xfade5948, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9e594b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00e5951, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6be5955, 33 },
    { 0, 0 },
    { 0xce8e5957, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9e595e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1e5963, 90 },
    { 0xc2ae5964, 55 },
    { 0, 0 },
    { 0x4ace5966, 23 },
    { 0xb70e5966, 33 },
    { 0x7d2e5968, 61 },
    { 0xdfe5969, 43 },
    { 0xbb5e596a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe596e, 76 },
    { 0, 0 },
    { 0xa4fe5970, 95 },
    { 0, 0 },
    { 0x4b0e5972, 85 },
    { 0x1dee5973, 23 },
    { 0x449e5973, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfe5978, 44 },
    { 0xafe5978, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fe597c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfe597f, 43 },
    { 0xe3ee5980, 55 },
    { 0xb83e5981, 23 },
    { 0x9efe5982, 95 },
    { 0xb6ce5982, 91 },
    { 0xbe3e5984, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78e5991, 95 },
    { 0xb08e5991, 34 },
    { 0x273e5991, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ae599b, 93 },
    { 0x49fe599b, 90 },
    { 0x297e599b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6e59a0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x895e59a3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e59a8, 55 },
    { 0x276e59a9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6de59ad, 90 },
    { 0, 0 },
    { 0xcb2e59af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe59b4, 81 },
    { 0x228e59b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde59b9, 33 },
    { 0xbe7e59ba, 87 },
    { 0x1f2e59ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae59c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934e59c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7e59cc, 95 },
    { 0x569e59cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2e59d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74e59dd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960e59e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ce59e9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2e59ee, 61 },
    { 0xca8e59ef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ae59fa, 43 },
    { 0, 0 },
    { 0x67ce59fc, 61 },
    { 0, 0 },
    { 0xc1be59fe, 90 },
    { 0x966e59ff, 23 },
    { 0, 0 },
    { 0x25de5a01, 93 },
    { 0x61ee5a02, 98 },
    { 0x613e5a03, 33 },
    { 0x688e5a04, 37 },
    { 0xc00e5a05, 91 },
    { 0x1d0e5a03, 43 },
    { 0xdb8e5a01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aae5a0b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be5a12, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4e5a15, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dde5a18, 46 },
    { 0x627e5a19, 23 },
    { 0x3eae5a1a, 91 },
    { 0x4fee5a1b, 23 },
    { 0, 0 },
    { 0xc8be5a1d, 46 },
    { 0, 0 },
    { 0xac7e5a1f, 95 },
    { 0x7d6e5a20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9e5a24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ae5a27, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6e5a2a, 90 },
    { 0, 0 },
    { 0xa7e5a2c, 91 },
    { 0, 0 },
    { 0x72ce5a2e, 33 },
    { 0, 0 },
    { 0xe49e5a30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04e5a33, 76 },
    { 0, 0 },
    { 0xc87e5a35, 34 },
    { 0x843e5a36, 98 },
    { 0xc0fe5a36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6e5a3a, 93 },
    { 0x4d3e5a3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e5a46, 98 },
    { 0x6c8e5a47, 23 },
    { 0, 0 },
    { 0x83fe5a49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafae5a50, 98 },
    { 0, 0 },
    { 0x672e5a52, 61 },
    { 0, 0 },
    { 0xb1de5a54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82e5a57, 46 },
    { 0x100e5a57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafe5a5c, 33 },
    { 0xd17e5a5d, 40 },
    { 0, 0 },
    { 0x500e5a5f, 88 },
    { 0x663e5a60, 76 },
    { 0x449e5a60, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57e5a6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7e5a72, 90 },
    { 0x820e5a72, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fe5a77, 33 },
    { 0xeaae5a78, 43 },
    { 0x681e5a77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5e5a7c, 95 },
    { 0x1fae5a7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e5a82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe5a86, 76 },
    { 0xaf2e5a87, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44be5a8b, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e5a8e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e5a91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ae5a94, 90 },
    { 0, 0 },
    { 0x443e5a96, 46 },
    { 0xa50e5a96, 76 },
    { 0, 0 },
    { 0x3fbe5a99, 95 },
    { 0, 0 },
    { 0x234e5a9b, 90 },
    { 0, 0 },
    { 0x4aae5a9d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2e5aa0, 33 },
    { 0, 0 },
    { 0xb07e5aa2, 34 },
    { 0, 0 },
    { 0xd64e5aa4, 95 },
    { 0xaf2e5aa4, 98 },
    { 0x323e5aa5, 95 },
    { 0x3ace5aa6, 98 },
    { 0xc30e5aa8, 90 },
    { 0, 0 },
    { 0x475e5aaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fe5aad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8e5ab1, 55 },
    { 0xd3ee5ab2, 33 },
    { 0x971e5ab3, 76 },
    { 0, 0 },
    { 0xa56e5ab5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26e5ab8, 98 },
    { 0x901e5ab9, 44 },
    { 0x6b4e5aba, 78 },
    { 0xb6ce5abb, 99 },
    { 0x2a7e5abb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee5ac4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4e5aca, 61 },
    { 0x852e5aca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191e5ad6, 55 },
    { 0x3fde5ad7, 55 },
    { 0x6c9e5ad8, 90 },
    { 0x6d7e5ad9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x84be5adc, 77 },
    { 0x417e5add, 83 },
    { 0x3b0e5add, 61 },
    { 0x20ee5adf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e5ae2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7e5aea, 61 },
    { 0x1dee5aeb, 34 },
    { 0x48be5aec, 33 },
    { 0xa00e5aed, 85 },
    { 0xc51e5aed, 90 },
    { 0xe5de5aeb, 52 },
    { 0xaa1e5af0, 43 },
    { 0x2be5af1, 98 },
    { 0xd31e5af2, 93 },
    { 0xd45e5af3, 23 },
    { 0xbbee5aed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e5af7, 52 },
    { 0x7dfe5af7, 90 },
    { 0x92e5af9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dee5afc, 98 },
    { 0xe02e5afd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae5b04, 33 },
    { 0x462e5b05, 78 },
    { 0x955e5b05, 90 },
    { 0x354e5b06, 96 },
    { 0xba5e5b08, 91 },
    { 0x200e5b09, 55 },
    { 0xa36e5b0a, 29 },
    { 0xea5e5b0a, 91 },
    { 0x336e5b08, 55 },
    { 0x71ee5b0d, 98 },
    { 0xad4e5b0d, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e5b12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e5b16, 61 },
    { 0x44ae5b17, 61 },
    { 0x5ede5b18, 90 },
    { 0x6b8e5b19, 76 },
    { 0xd2ce5b18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdae5b1e, 78 },
    { 0, 0 },
    { 0x3c8e5b20, 23 },
    { 0x74fe5b21, 84 },
    { 0xf8fe5b20, 95 },
    { 0x74ce5b23, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8e5b2b, 46 },
    { 0x697e5b2c, 98 },
    { 0x1cee5b2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301e5b34, 33 },
    { 0, 0 },
    { 0x4d0e5b36, 78 },
    { 0, 0 },
    { 0x24ee5b38, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3e5b3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e5b40, 40 },
    { 0x74ee5b40, 76 },
    { 0x763e5b42, 36 },
    { 0x667e5b43, 55 },
    { 0xddfe5b40, 91 },
    { 0xf51e5b45, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93e5b48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8e5b4c, 98 },
    { 0xa9fe5b4d, 89 },
    { 0x29ee5b4c, 76 },
    { 0xae8e5b4f, 76 },
    { 0, 0 },
    { 0x383e5b51, 85 },
    { 0x394e5b52, 73 },
    { 0, 0 },
    { 0xd3be5b54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5e5b57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724e5b5b, 55 },
    { 0xb63e5b5c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e5b62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e5b66, 90 },
    { 0, 0 },
    { 0x2f1e5b68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0e5b6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309e5b72, 95 },
    { 0x75fe5b73, 90 },
    { 0x311e5b73, 69 },
    { 0x1b8e5b74, 34 },
    { 0x8dae5b76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902e5b7e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e5b81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ae5b86, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801e5b8b, 61 },
    { 0x1b2e5b8c, 95 },
    { 0x132e5b8b, 66 },
    { 0x86de5b8d, 76 },
    { 0xf01e5b8f, 78 },
    { 0xcb4e5b8f, 93 },
    { 0, 0 },
    { 0x39fe5b92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ede5b99, 61 },
    { 0, 0 },
    { 0xfefe5b9b, 95 },
    { 0, 0 },
    { 0x7f4e5b9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5e5ba0, 61 },
    { 0xdbee5ba1, 36 },
    { 0x60ce5ba2, 61 },
    { 0, 0 },
    { 0xca0e5ba4, 83 },
    { 0x6fbe5ba5, 33 },
    { 0xf4e5ba6, 85 },
    { 0xc82e5ba4, 76 },
    { 0xff0e5ba8, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ee5bab, 34 },
    { 0xc80e5bab, 91 },
    { 0x60e5bad, 100 },
    { 0x4d9e5bad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54be5bb5, 76 },
    { 0x979e5bb6, 61 },
    { 0x9a6e5bb6, 98 },
    { 0x45ce5bb8, 61 },
    { 0xa84e5bb6, 25 },
    { 0x5f4e5bba, 95 },
    { 0x565e5bb8, 61 },
    { 0x370e5bbb, 90 },
    { 0xcaee5bbd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x153e5bc0, 81 },
    { 0xfd9e5bc1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e5bc4, 66 },
    { 0, 0 },
    { 0x829e5bc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50e5bca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90e5bcd, 91 },
    { 0x723e5bcd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8e5bd1, 87 },
    { 0, 0 },
    { 0x80fe5bd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8e5bd8, 23 },
    { 0, 0 },
    { 0xf22e5bda, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fe5be0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcede5be5, 33 },
    { 0x36fe5be6, 17 },
    { 0, 0 },
    { 0xdf0e5be8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59e5bed, 17 },
    { 0x80de5bee, 98 },
    { 0x81e5bef, 95 },
    { 0x9d6e5bf0, 61 },
    { 0x808e5bf0, 84 },
    { 0x283e5bf1, 90 },
    { 0xa5ce5bf3, 33 },
    { 0x5cfe5bf3, 38 },
    { 0x6bee5bf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ee5bfd, 76 },
    { 0x342e5bfe, 46 },
    { 0x50ae5bfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493e5c04, 98 },
    { 0x77ae5c05, 100 },
    { 0, 0 },
    { 0x915e5c07, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x876e5c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ae5c15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4e5c18, 91 },
    { 0x287e5c18, 93 },
    { 0xf1e5c1a, 61 },
    { 0, 0 },
    { 0xf3e5c1c, 95 },
    { 0, 0 },
    { 0x9dce5c1e, 95 },
    { 0xd9ce5c1f, 77 },
    { 0xcdce5c20, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8be5c23, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00e5c29, 85 },
    { 0xe24e5c29, 83 },
    { 0xa4ee5c2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e5c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409e5c3a, 90 },
    { 0x7d3e5c3b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb82e5c43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6e5c48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334e5c50, 95 },
    { 0, 0 },
    { 0xde6e5c52, 23 },
    { 0, 0 },
    { 0x7a1e5c54, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x775e5c57, 34 },
    { 0xdfae5c58, 23 },
    { 0x5a5e5c59, 98 },
    { 0xc1de5c5a, 46 },
    { 0, 0 },
    { 0x4ee5c5c, 70 },
    { 0x981e5c5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae5c6a, 95 },
    { 0, 0 },
    { 0x88fe5c6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde5c70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91e5c73, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ce5c77, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3e5c7f, 33 },
    { 0, 0 },
    { 0xfdce5c81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96e5c85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbede5c89, 91 },
    { 0, 0 },
    { 0xf63e5c8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14e5c90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16e5c97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ce5c9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918e5ca2, 33 },
    { 0, 0 },
    { 0xcd5e5ca4, 90 },
    { 0x8a7e5ca5, 78 },
    { 0x976e5ca6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5ca9, 90 },
    { 0x372e5caa, 61 },
    { 0, 0 },
    { 0xadde5cac, 33 },
    { 0, 0 },
    { 0xb8fe5cae, 65 },
    { 0xbc5e5cae, 34 },
    { 0xca5e5caf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e5cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3e5cbb, 78 },
    { 0, 0 },
    { 0x10ee5cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ce5cc1, 52 },
    { 0, 0 },
    { 0x925e5cc3, 84 },
    { 0, 0 },
    { 0x38e5cc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x376e5cc8, 33 },
    { 0x38fe5cc9, 52 },
    { 0, 0 },
    { 0x9f4e5ccb, 52 },
    { 0x8ebe5ccb, 78 },
    { 0, 0 },
    { 0xa56e5cce, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfece5cd4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bde5cd9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692e5cde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce5ce1, 90 },
    { 0, 0 },
    { 0xac4e5ce3, 95 },
    { 0, 0 },
    { 0x589e5ce5, 51 },
    { 0xde5ce5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70e5ce9, 95 },
    { 0, 0 },
    { 0xd56e5ceb, 75 },
    { 0x2dbe5cec, 55 },
    { 0, 0 },
    { 0x98e5cee, 33 },
    { 0, 0 },
    { 0x6a6e5cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x877e5cf5, 98 },
    { 0, 0 },
    { 0x4ebe5cf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591e5d00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde5d04, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3e5d08, 33 },
    { 0x5c3e5d09, 95 },
    { 0, 0 },
    { 0x7be5d0b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e5d11, 4 },
    { 0x801e5d12, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x291e5d15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x308e5d19, 65 },
    { 0xf1fe5d1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829e5d1f, 90 },
    { 0xc29e5d20, 90 },
    { 0x25be5d1f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300e5d26, 61 },
    { 0x927e5d26, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de5d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x486e5d2d, 33 },
    { 0, 0 },
    { 0x60ee5d2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76e5d32, 33 },
    { 0xcd6e5d33, 76 },
    { 0x10fe5d34, 23 },
    { 0x1d3e5d34, 61 },
    { 0x19fe5d36, 23 },
    { 0, 0 },
    { 0x3fde5d38, 95 },
    { 0x15e5d38, 68 },
    { 0x8e8e5d3a, 33 },
    { 0xf74e5d39, 61 },
    { 0xee4e5d3c, 85 },
    { 0x46e5d3d, 95 },
    { 0x95e5d3e, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2e5d41, 67 },
    { 0x7a1e5d42, 91 },
    { 0xaaae5d42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dae5d47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cae5d4d, 33 },
    { 0, 0 },
    { 0x506e5d4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ee5d52, 67 },
    { 0xf4fe5d53, 95 },
    { 0xb2ce5d52, 90 },
    { 0x93ae5d55, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7e5d59, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51e5d61, 55 },
    { 0, 0 },
    { 0x1e6e5d63, 67 },
    { 0, 0 },
    { 0x3d2e5d65, 76 },
    { 0, 0 },
    { 0x421e5d67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5e5d6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x860e5d6e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be5d71, 91 },
    { 0xeee5d71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331e5d76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e5d79, 83 },
    { 0x324e5d7a, 61 },
    { 0xdce5d7b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ee5d85, 66 },
    { 0, 0 },
    { 0x935e5d87, 95 },
    { 0xd94e5d88, 84 },
    { 0xa44e5d89, 33 },
    { 0xd7ee5d8a, 90 },
    { 0, 0 },
    { 0x1d8e5d8c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3e5d8f, 90 },
    { 0, 0 },
    { 0xd5ce5d91, 23 },
    { 0, 0 },
    { 0x39ce5d93, 61 },
    { 0x9b6e5d93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44e5d99, 61 },
    { 0x4b6e5d99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ce5d9d, 98 },
    { 0x4a7e5d9d, 90 },
    { 0x5d8e5d9e, 81 },
    { 0xe87e5da0, 90 },
    { 0xab6e5d9e, 90 },
    { 0x8aee5da2, 91 },
    { 0xdfce5d9e, 75 },
    { 0, 0 },
    { 0xc8fe5da5, 43 },
    { 0, 0 },
    { 0x21ee5da7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e5db1, 78 },
    { 0x9cce5db1, 33 },
    { 0xb6be5db2, 90 },
    { 0x44fe5db4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dde5db8, 90 },
    { 0x1f1e5db8, 43 },
    { 0xad5e5dba, 61 },
    { 0x227e5db9, 34 },
    { 0xa40e5dbc, 34 },
    { 0xf47e5dbd, 92 },
    { 0xe0e5db8, 96 },
    { 0, 0 },
    { 0x618e5dc0, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede5dc4, 23 },
    { 0x378e5dc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdde5dca, 99 },
    { 0x8c2e5dcb, 61 },
    { 0xd9be5dca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafce5dd0, 90 },
    { 0xadde5dd0, 33 },
    { 0, 0 },
    { 0x8b1e5dd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de5dd7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3e5dda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ae5de2, 84 },
    { 0x16fe5de2, 69 },
    { 0, 0 },
    { 0xbd6e5de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58e5de8, 93 },
    { 0, 0 },
    { 0x382e5dea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54ce5df0, 23 },
    { 0, 0 },
    { 0x4a3e5df2, 36 },
    { 0, 0 },
    { 0xbaae5df4, 90 },
    { 0xef8e5df4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64e5df9, 85 },
    { 0x4f4e5dfa, 23 },
    { 0x47e5dfb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcae5dff, 90 },
    { 0xeee5dff, 33 },
    { 0xcbce5e01, 61 },
    { 0xad7e5e01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e5e06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be5e0c, 19 },
    { 0xa68e5e0d, 98 },
    { 0, 0 },
    { 0x39fe5e0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78e5e13, 33 },
    { 0x339e5e14, 98 },
    { 0x877e5e15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9de5e1b, 61 },
    { 0x292e5e1c, 84 },
    { 0x2b2e5e1d, 64 },
    { 0x8cbe5e1e, 33 },
    { 0, 0 },
    { 0xbc8e5e20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1e5e2b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fe5e2f, 61 },
    { 0x5ece5e30, 66 },
    { 0x762e5e31, 23 },
    { 0x3c8e5e31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57e5e37, 34 },
    { 0xf00e5e37, 23 },
    { 0x53be5e39, 90 },
    { 0x2a8e5e3a, 66 },
    { 0xaf8e5e39, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e5e44, 33 },
    { 0xd83e5e45, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5de5e48, 33 },
    { 0, 0 },
    { 0xf84e5e4a, 61 },
    { 0, 0 },
    { 0xfeae5e4c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3e5e52, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e5e56, 90 },
    { 0xdfae5e56, 91 },
    { 0xee2e5e58, 46 },
    { 0xaa3e5e58, 23 },
    { 0xf61e5e56, 90 },
    { 0x36e5e5b, 75 },
    { 0x71be5e5c, 61 },
    { 0x940e5e5d, 33 },
    { 0xabce5e5d, 43 },
    { 0x656e5e5c, 98 },
    { 0x634e5e60, 33 },
    { 0xcdde5e56, 43 },
    { 0xf93e5e5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91e5e66, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x136e5e69, 90 },
    { 0, 0 },
    { 0x54ae5e6b, 34 },
    { 0, 0 },
    { 0x410e5e6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fce5e70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70e5e76, 90 },
    { 0x340e5e77, 98 },
    { 0x70ce5e78, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6e5e7c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ee5e7f, 33 },
    { 0, 0 },
    { 0xd9ce5e81, 34 },
    { 0x5fe5e82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9e5e85, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852e5e8e, 33 },
    { 0x992e5e8f, 61 },
    { 0, 0 },
    { 0xde2e5e91, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31e5e94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40de5e9a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248e5ea2, 98 },
    { 0xb8fe5ea3, 96 },
    { 0, 0 },
    { 0x838e5ea5, 85 },
    { 0x198e5ea5, 78 },
    { 0xd0fe5ea5, 95 },
    { 0x44be5ea8, 81 },
    { 0, 0 },
    { 0x2dee5eaa, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122e5eae, 33 },
    { 0x7b2e5eae, 93 },
    { 0, 0 },
    { 0xc73e5eb1, 46 },
    { 0x32ce5eb2, 90 },
    { 0xffbe5eb3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ae5ebe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x425e5ec1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee5ec4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fe5ec7, 95 },
    { 0, 0 },
    { 0x229e5ec9, 98 },
    { 0, 0 },
    { 0x827e5ecb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ae5ed0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee5ed3, 98 },
    { 0x9d5e5ed3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad6e5ed8, 90 },
    { 0, 0 },
    { 0x492e5eda, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413e5ede, 61 },
    { 0, 0 },
    { 0x956e5ee0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ece5ee4, 93 },
    { 0x843e5ee5, 33 },
    { 0, 0 },
    { 0x7f1e5ee7, 61 },
    { 0, 0 },
    { 0x62e5ee9, 61 },
    { 0, 0 },
    { 0xb4fe5eeb, 94 },
    { 0x5afe5eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ce5ef2, 61 },
    { 0x314e5ef3, 33 },
    { 0, 0 },
    { 0xdcfe5ef5, 101 },
    { 0, 0 },
    { 0xa6be5ef7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ee5efb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e5efe, 95 },
    { 0x96de5eff, 33 },
    { 0xff3e5f00, 98 },
    { 0xc4ce5f00, 46 },
    { 0xe8ee5f01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1de5f05, 88 },
    { 0x143e5f06, 90 },
    { 0x662e5f07, 61 },
    { 0, 0 },
    { 0xcf2e5f09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fe5f0d, 33 },
    { 0xe3fe5f0e, 66 },
    { 0xae5f0f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8be5f14, 33 },
    { 0x626e5f15, 52 },
    { 0xecbe5f15, 99 },
    { 0x7dae5f17, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ce5f1d, 87 },
    { 0x442e5f1e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecde5f22, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcde5f2b, 23 },
    { 0x71ae5f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee5f30, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2e5f36, 43 },
    { 0xe06e5f37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e5f3a, 90 },
    { 0x8ffe5f3a, 99 },
    { 0x507e5f3c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ee5f3f, 98 },
    { 0x492e5f3f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261e5f45, 90 },
    { 0x175e5f45, 90 },
    { 0x541e5f45, 23 },
    { 0x8a2e5f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2e5f4f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x189e5f52, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18e5f55, 34 },
    { 0x327e5f56, 12 },
    { 0, 0 },
    { 0xcd4e5f58, 76 },
    { 0xa9ee5f58, 98 },
    { 0, 0 },
    { 0x2e5e5f5b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ee5f5e, 33 },
    { 0, 0 },
    { 0x9eae5f60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304e5f64, 61 },
    { 0x2aae5f64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63e5f68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x198e5f6b, 87 },
    { 0xa53e5f6b, 52 },
    { 0x2ade5f6b, 90 },
    { 0x25de5f6e, 76 },
    { 0x917e5f6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee5f73, 61 },
    { 0, 0 },
    { 0xa5e5f75, 66 },
    { 0, 0 },
    { 0xf9de5f77, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fe5f7b, 95 },
    { 0xc8ee5f7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x561e5f7f, 61 },
    { 0xbc2e5f80, 92 },
    { 0xce8e5f80, 92 },
    { 0x8e3e5f7f, 55 },
    { 0xc6de5f81, 76 },
    { 0x42be5f84, 98 },
    { 0x82be5f84, 90 },
    { 0, 0 },
    { 0x522e5f87, 61 },
    { 0, 0 },
    { 0x38e5f89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5e5f92, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x40be5f95, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3e5f9e, 23 },
    { 0x493e5f9f, 66 },
    { 0x3fbe5fa0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfee5fa3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4e5fa9, 90 },
    { 0x213e5faa, 56 },
    { 0x14be5fab, 95 },
    { 0x96ce5fac, 33 },
    { 0x1f0e5fac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4e5fb4, 34 },
    { 0xe6ae5fb4, 61 },
    { 0xdd1e5fb4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ae5fb9, 23 },
    { 0xd3de5fba, 61 },
    { 0x444e5fba, 84 },
    { 0x800e5fbc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e5fc0, 61 },
    { 0x575e5fc1, 95 },
    { 0, 0 },
    { 0x972e5fc3, 33 },
    { 0, 0 },
    { 0x4c2e5fc5, 33 },
    { 0xc86e5fc6, 34 },
    { 0x242e5fc7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde5fca, 61 },
    { 0xa7ce5fcb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710e5fcf, 43 },
    { 0, 0 },
    { 0x42e5fd1, 61 },
    { 0, 0 },
    { 0xa6ce5fd3, 90 },
    { 0x7d4e5fd3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fe5fe9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fce5fed, 33 },
    { 0, 0 },
    { 0xe25e5fef, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28e5ff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e5ff6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ee6001, 61 },
    { 0, 0 },
    { 0x9afe6003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee600b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99e600e, 38 },
    { 0x6a1e600f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1e6016, 90 },
    { 0, 0 },
    { 0x9afe6018, 93 },
    { 0xac7e6018, 98 },
    { 0x72e6019, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5e601f, 61 },
    { 0xf7ce6020, 100 },
    { 0, 0 },
    { 0x10be6022, 95 },
    { 0, 0 },
    { 0xde6024, 98 },
    { 0xa67e6024, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdae6028, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ee602b, 81 },
    { 0xae8e602b, 90 },
    { 0x298e602b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84e6032, 61 },
    { 0x749e6032, 61 },
    { 0x6bde6033, 95 },
    { 0xee0e6034, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ee603b, 68 },
    { 0x600e603b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5e6041, 23 },
    { 0, 0 },
    { 0xb33e6043, 90 },
    { 0xcc4e6043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfe6047, 91 },
    { 0, 0 },
    { 0xce8e6049, 76 },
    { 0x234e6049, 90 },
    { 0x104e6049, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e6052, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaece6058, 98 },
    { 0xae6059, 23 },
    { 0x18be6058, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fe605f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833e6065, 34 },
    { 0x1bbe6066, 98 },
    { 0x9be6067, 23 },
    { 0, 0 },
    { 0xb5fe6069, 33 },
    { 0, 0 },
    { 0xd33e606b, 33 },
    { 0, 0 },
    { 0x6a3e606d, 90 },
    { 0, 0 },
    { 0xb05e606f, 76 },
    { 0xc0e606f, 86 },
    { 0x370e6071, 38 },
    { 0xb3be6072, 46 },
    { 0, 0 },
    { 0xf0ee6074, 80 },
    { 0, 0 },
    { 0x56ce6076, 100 },
    { 0, 0 },
    { 0x86ae6078, 90 },
    { 0, 0 },
    { 0x35ee607a, 90 },
    { 0xb79e607b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2e608b, 34 },
    { 0x5b3e608c, 99 },
    { 0xb54e608d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6e6091, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ee6099, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae609d, 46 },
    { 0x6dde609d, 46 },
    { 0, 0 },
    { 0x91ee60a0, 90 },
    { 0, 0 },
    { 0x100e60a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48de60a8, 33 },
    { 0xdd8e60a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880e60b3, 66 },
    { 0, 0 },
    { 0x82ae60b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552e60b9, 90 },
    { 0x751e60b9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2e60c0, 34 },
    { 0xdf3e60c0, 95 },
    { 0xdc3e60c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7e60c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x513e60c9, 100 },
    { 0, 0 },
    { 0x4c6e60cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbe60ce, 67 },
    { 0x33ae60cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x772e60d2, 76 },
    { 0x447e60d2, 33 },
    { 0x5e7e60d3, 33 },
    { 0x116e60d2, 93 },
    { 0x4d1e60d6, 87 },
    { 0xc5be60d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dae60db, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96e60de, 66 },
    { 0, 0 },
    { 0x4e1e60e0, 52 },
    { 0, 0 },
    { 0x41de60e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fe60e8, 43 },
    { 0x82be60e9, 61 },
    { 0x619e60e8, 61 },
    { 0, 0 },
    { 0x6bee60ec, 90 },
    { 0x782e60ed, 23 },
    { 0, 0 },
    { 0x777e60ef, 86 },
    { 0xefee60f0, 40 },
    { 0, 0 },
    { 0xcd5e60f2, 48 },
    { 0, 0 },
    { 0xaece60f4, 95 },
    { 0, 0 },
    { 0x40ee60f6, 46 },
    { 0, 0 },
    { 0x9fe60f8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1e60fe, 90 },
    { 0x2a2e60ff, 85 },
    { 0x481e6100, 85 },
    { 0, 0 },
    { 0xbafe6102, 67 },
    { 0, 0 },
    { 0x41e6104, 76 },
    { 0xfd3e6105, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee6108, 61 },
    { 0, 0 },
    { 0xae0e610a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4e6110, 90 },
    { 0x95ae6111, 55 },
    { 0x766e6112, 95 },
    { 0xc43e6112, 95 },
    { 0x44ce6114, 55 },
    { 0, 0 },
    { 0xd9fe6116, 23 },
    { 0x62be6116, 76 },
    { 0x370e6118, 76 },
    { 0x1fae6116, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e611e, 95 },
    { 0x58fe611f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfe6122, 48 },
    { 0, 0 },
    { 0x9d5e6124, 88 },
    { 0xaace6124, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebe6129, 55 },
    { 0, 0 },
    { 0x455e612b, 23 },
    { 0x62ae612b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fe6135, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4e6139, 85 },
    { 0xa1fe613a, 60 },
    { 0xf4de6139, 33 },
    { 0xaf3e6139, 43 },
    { 0xad5e613b, 93 },
    { 0x9f6e613e, 77 },
    { 0x85be613f, 78 },
    { 0x8a6e613f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97e6145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cae614c, 90 },
    { 0xd32e614d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dde6151, 23 },
    { 0xc5de6151, 23 },
    { 0x7fae6153, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3e6156, 46 },
    { 0, 0 },
    { 0xb10e6158, 81 },
    { 0, 0 },
    { 0x1d6e615a, 34 },
    { 0, 0 },
    { 0x811e615c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebe615f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ece6163, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e6168, 43 },
    { 0, 0 },
    { 0xcae616a, 33 },
    { 0, 0 },
    { 0xc74e616c, 55 },
    { 0x46ee616d, 90 },
    { 0xce8e616e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5fe6171, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbe617b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2e6184, 66 },
    { 0xebfe6184, 95 },
    { 0xfe3e6184, 90 },
    { 0xa07e6185, 19 },
    { 0, 0 },
    { 0x851e6189, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e618e, 55 },
    { 0xb11e618f, 83 },
    { 0, 0 },
    { 0xb27e6191, 33 },
    { 0xbf6e6191, 61 },
    { 0, 0 },
    { 0x2ebe6194, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17e6199, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552e61a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78e61a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fe61a7, 98 },
    { 0x698e61a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e61ad, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xaace61b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5e61b3, 61 },
    { 0xecde61b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842e61b9, 23 },
    { 0x9b5e61ba, 84 },
    { 0x5f5e61b9, 95 },
    { 0x2d0e61bc, 98 },
    { 0xadbe61ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7e61c0, 90 },
    { 0x906e61c1, 23 },
    { 0x81ae61c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e61c5, 23 },
    { 0xe9ce61c5, 91 },
    { 0x577e61c7, 90 },
    { 0x64de61c8, 90 },
    { 0x260e61c9, 95 },
    { 0, 0 },
    { 0x76fe61cb, 23 },
    { 0, 0 },
    { 0x309e61cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67e61d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x90e61d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04e61e4, 55 },
    { 0, 0 },
    { 0x3be61e6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4e61e9, 34 },
    { 0, 0 },
    { 0xdfe61eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ce61f5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61e6200, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52e6203, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e6209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44e6211, 95 },
    { 0xf48e6212, 33 },
    { 0x66ce6212, 33 },
    { 0x2b3e6212, 8 },
    { 0, 0 },
    { 0x614e6216, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223e621c, 33 },
    { 0x850e621d, 43 },
    { 0x78de621c, 90 },
    { 0x142e621f, 90 },
    { 0, 0 },
    { 0x98fe6221, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823e6225, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660e622c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847e6230, 77 },
    { 0x6c9e6230, 23 },
    { 0x306e6232, 84 },
    { 0x8f8e6233, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6e623a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ee623f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ae6245, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841e624b, 23 },
    { 0x789e624c, 43 },
    { 0xf6de624d, 65 },
    { 0x2bfe624e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cce6252, 98 },
    { 0, 0 },
    { 0x5a9e6254, 33 },
    { 0, 0 },
    { 0xf82e6256, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x137e6259, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e625f, 98 },
    { 0xfc2e6260, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7e6264, 90 },
    { 0x7a1e6264, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6de6268, 90 },
    { 0xdcae6268, 61 },
    { 0x33e626a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348e6270, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bae6273, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57e627f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ae6282, 100 },
    { 0xed3e6283, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e6288, 33 },
    { 0xc40e6289, 98 },
    { 0, 0 },
    { 0x407e628b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e628e, 43 },
    { 0xdeae628f, 90 },
    { 0xeb3e6290, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5e6295, 76 },
    { 0xf0de6296, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e629e, 65 },
    { 0xe14e629f, 61 },
    { 0xcd4e62a0, 66 },
    { 0, 0 },
    { 0x3c5e62a2, 81 },
    { 0x1d4e62a2, 90 },
    { 0xf9be62a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843e62a9, 33 },
    { 0x931e62aa, 23 },
    { 0x71be62a9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbe62b3, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705e62ba, 61 },
    { 0xf3ce62ba, 90 },
    { 0x98fe62bc, 33 },
    { 0x11fe62bd, 33 },
    { 0, 0 },
    { 0xd52e62bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58e62c3, 90 },
    { 0, 0 },
    { 0x380e62c5, 61 },
    { 0x8dfe62c6, 43 },
    { 0x5f2e62c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x682e62ca, 95 },
    { 0xa30e62ca, 34 },
    { 0xc3fe62cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ce62d8, 34 },
    { 0, 0 },
    { 0x7e3e62da, 91 },
    { 0x200e62db, 33 },
    { 0xeafe62dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26e62e4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e62e7, 55 },
    { 0x195e62e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4e62ee, 88 },
    { 0x6d7e62ee, 40 },
    { 0x77ae62ef, 85 },
    { 0x825e62f1, 91 },
    { 0xa42e62f2, 34 },
    { 0, 0 },
    { 0xa16e62f4, 99 },
    { 0x41de62f5, 66 },
    { 0x78ce62f6, 95 },
    { 0xccae62f4, 61 },
    { 0xdeee62f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fe62fe, 93 },
    { 0, 0 },
    { 0xdc9e6300, 66 },
    { 0x342e6301, 95 },
    { 0xd61e6302, 23 },
    { 0xc13e6303, 46 },
    { 0, 0 },
    { 0xb9e6305, 90 },
    { 0x2dbe6306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdee630f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1e6313, 90 },
    { 0xe05e6314, 33 },
    { 0xdf6e6313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0e631a, 90 },
    { 0x196e631b, 76 },
    { 0, 0 },
    { 0x56fe631d, 95 },
    { 0x848e631d, 95 },
    { 0x157e631f, 46 },
    { 0x144e631d, 98 },
    { 0, 0 },
    { 0x45de6322, 76 },
    { 0, 0 },
    { 0xd4de6324, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fe632a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03e632f, 55 },
    { 0, 0 },
    { 0x6e9e6331, 78 },
    { 0x22e6331, 61 },
    { 0x2ffe6333, 61 },
    { 0x15ae6334, 33 },
    { 0x83ee6335, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498e633b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x622e633e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1e6341, 33 },
    { 0x4fbe6342, 17 },
    { 0x849e6343, 19 },
    { 0, 0 },
    { 0x9ace6345, 61 },
    { 0x444e6346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74e6349, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46fe634f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5de6357, 90 },
    { 0xc95e6358, 67 },
    { 0x20ee6359, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e635e, 95 },
    { 0x477e635f, 84 },
    { 0xf30e635f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee6366, 76 },
    { 0xfd3e6367, 34 },
    { 0, 0 },
    { 0xf15e6369, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3e6370, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbae6374, 85 },
    { 0, 0 },
    { 0x373e6376, 61 },
    { 0, 0 },
    { 0x166e6378, 95 },
    { 0xd6de6379, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fe6381, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66e6384, 78 },
    { 0x53fe6385, 59 },
    { 0xef6e6386, 66 },
    { 0x213e6387, 95 },
    { 0xe3e6388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffe638c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfe6394, 53 },
    { 0xcd8e6395, 8 },
    { 0x21ce6394, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88e639a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3e63a0, 46 },
    { 0xb1fe63a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bde63a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbde63a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5e63aa, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919e63b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x383e63b7, 88 },
    { 0xf59e63b7, 33 },
    { 0xc8fe63b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25e63bd, 87 },
    { 0, 0 },
    { 0x660e63bf, 95 },
    { 0xc3be63c0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce63c3, 33 },
    { 0x1a5e63c4, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3e63c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0de63ca, 23 },
    { 0x5ebe63cb, 61 },
    { 0x4e9e63cb, 55 },
    { 0xaa4e63cd, 33 },
    { 0, 0 },
    { 0x26ce63cf, 95 },
    { 0x4a9e63d0, 76 },
    { 0xbb9e63d0, 95 },
    { 0xfc7e63d2, 34 },
    { 0xbe6e63d3, 43 },
    { 0x6b1e63d0, 61 },
    { 0x199e63d1, 95 },
    { 0x4a8e63d0, 53 },
    { 0x47ee63d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe63da, 76 },
    { 0, 0 },
    { 0x84ae63dc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6e63e4, 2 },
    { 0x2c8e63e5, 43 },
    { 0x7a0e63e6, 52 },
    { 0xc3ee63e7, 85 },
    { 0x4de63e8, 55 },
    { 0x26ee63e6, 61 },
    { 0xbaee63ea, 95 },
    { 0x42e63e5, 43 },
    { 0x853e63e9, 98 },
    { 0xb5de63ed, 23 },
    { 0xece63ec, 66 },
    { 0xb31e63e6, 46 },
    { 0xf65e63f0, 90 },
    { 0xc95e63e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7e63f8, 61 },
    { 0xa02e63f8, 43 },
    { 0x383e63fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37e63fd, 61 },
    { 0x2fe63fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fe6403, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e6406, 61 },
    { 0x3cbe6407, 46 },
    { 0xa89e6407, 61 },
    { 0x3cde6409, 43 },
    { 0, 0 },
    { 0x70fe640b, 23 },
    { 0xe67e640c, 77 },
    { 0x4d5e640c, 76 },
    { 0xca3e640e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x82be6411, 33 },
    { 0, 0 },
    { 0xc04e6413, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369e6417, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5e641b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1e641e, 33 },
    { 0, 0 },
    { 0x4bbe6420, 76 },
    { 0, 0 },
    { 0xbe1e6422, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ce6426, 55 },
    { 0, 0 },
    { 0x993e6428, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x134e642b, 93 },
    { 0x595e642b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x24e642f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e6434, 90 },
    { 0xa6ae6434, 33 },
    { 0, 0 },
    { 0xc1e6437, 23 },
    { 0xfcbe6438, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4e643d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe6443, 67 },
    { 0x8b3e6444, 76 },
    { 0, 0 },
    { 0x293e6446, 34 },
    { 0xb2be6447, 76 },
    { 0, 0 },
    { 0xc79e6449, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57e644c, 52 },
    { 0, 0 },
    { 0xcae644e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x744e6451, 98 },
    { 0x594e6452, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ee645c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x980e645f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76be6463, 78 },
    { 0, 0 },
    { 0x59e6465, 90 },
    { 0, 0 },
    { 0x862e6467, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5e646a, 100 },
    { 0x9b6e646b, 100 },
    { 0, 0 },
    { 0x9cce646d, 61 },
    { 0xbd9e646d, 61 },
    { 0x4bde646d, 90 },
    { 0x189e646e, 43 },
    { 0, 0 },
    { 0x260e6472, 33 },
    { 0, 0 },
    { 0xe9ae6474, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85be6478, 33 },
    { 0x963e6479, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63e6481, 95 },
    { 0xae9e6482, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70e6488, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3e648c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ce6496, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95e649a, 33 },
    { 0, 0 },
    { 0x16de649c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe64a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38de64a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee64a8, 67 },
    { 0xac2e64a8, 23 },
    { 0x4bee64aa, 98 },
    { 0xf58e64ab, 33 },
    { 0, 0 },
    { 0xb8ce64ad, 66 },
    { 0x635e64ad, 91 },
    { 0xc38e64af, 98 },
    { 0x45fe64ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88e64b3, 83 },
    { 0x8e0e64b3, 23 },
    { 0x621e64b5, 61 },
    { 0xa40e64b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ae64bf, 95 },
    { 0, 0 },
    { 0x10ce64c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5e64c4, 61 },
    { 0xa18e64c5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36e64cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256e64d0, 23 },
    { 0xdbce64d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e64d6, 33 },
    { 0, 0 },
    { 0x5f7e64d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e64dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebe64e2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63e64e6, 23 },
    { 0x4d0e64e7, 99 },
    { 0x8cae64e8, 90 },
    { 0x7a5e64e8, 33 },
    { 0x2fae64ea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57de64f1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce64f7, 76 },
    { 0, 0 },
    { 0x1dee64f9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe64fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fe6504, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8de650f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95e6513, 34 },
    { 0, 0 },
    { 0xb15e6515, 55 },
    { 0x50be6516, 93 },
    { 0xb58e6516, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4e651a, 43 },
    { 0xa90e651b, 23 },
    { 0, 0 },
    { 0x1e3e651d, 91 },
    { 0x30e651e, 23 },
    { 0, 0 },
    { 0x379e6520, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x655e6523, 56 },
    { 0xeaae6524, 33 },
    { 0xce7e6524, 94 },
    { 0, 0 },
    { 0xb1ce6527, 91 },
    { 0xbffe6528, 90 },
    { 0, 0 },
    { 0x709e652a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5e652f, 88 },
    { 0x865e6530, 33 },
    { 0x3f2e6531, 76 },
    { 0x817e6530, 76 },
    { 0x7c9e652f, 33 },
    { 0, 0 },
    { 0x6b0e6535, 23 },
    { 0, 0 },
    { 0xea9e6537, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dae653a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ee653f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e6543, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6546, 61 },
    { 0xba9e6546, 23 },
    { 0, 0 },
    { 0x936e6549, 43 },
    { 0x2cbe654a, 90 },
    { 0x4a2e654b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118e6550, 76 },
    { 0x423e6551, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3e6554, 90 },
    { 0x724e6555, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15e6558, 86 },
    { 0xdd5e6559, 90 },
    { 0xc5be655a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce6561, 90 },
    { 0x4dfe6562, 33 },
    { 0xe16e6562, 46 },
    { 0x7f8e6561, 95 },
    { 0xb8de6565, 61 },
    { 0xa9fe6563, 90 },
    { 0x515e6567, 61 },
    { 0x9a5e6567, 33 },
    { 0xf7be6569, 98 },
    { 0x8de6563, 98 },
    { 0x918e656b, 34 },
    { 0, 0 },
    { 0x20ee656d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e6571, 87 },
    { 0, 0 },
    { 0xf1de6573, 86 },
    { 0x1b6e6574, 75 },
    { 0xd77e6575, 78 },
    { 0xc23e6575, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e6579, 33 },
    { 0xf6ee657a, 100 },
    { 0x775e657b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03e6582, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfe6587, 76 },
    { 0, 0 },
    { 0x86ae6589, 95 },
    { 0xca7e658a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131e658e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32de6598, 52 },
    { 0xe6be6599, 99 },
    { 0x9cbe6599, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147e659f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6e65aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aae65ad, 90 },
    { 0x6d2e65ae, 90 },
    { 0x9a6e65af, 42 },
    { 0xd0de65b0, 76 },
    { 0, 0 },
    { 0x95be65b2, 46 },
    { 0, 0 },
    { 0x7ade65b4, 99 },
    { 0x56ce65b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e65b8, 76 },
    { 0, 0 },
    { 0xbeae65ba, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdce65be, 61 },
    { 0x1d3e65bf, 61 },
    { 0x9ebe65bf, 81 },
    { 0xf85e65be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5e65c4, 66 },
    { 0, 0 },
    { 0x954e65c6, 23 },
    { 0x688e65c6, 78 },
    { 0x53be65c6, 83 },
    { 0xea6e65c7, 43 },
    { 0, 0 },
    { 0xcb2e65cb, 61 },
    { 0x479e65cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbe65cf, 68 },
    { 0, 0 },
    { 0x7d2e65d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffe65d7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ae65dc, 67 },
    { 0x611e65dd, 43 },
    { 0xa31e65de, 84 },
    { 0x666e65de, 33 },
    { 0xdcee65de, 90 },
    { 0x400e65e1, 23 },
    { 0xf41e65e0, 66 },
    { 0, 0 },
    { 0xffae65e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ae65e7, 61 },
    { 0x9c5e65e8, 98 },
    { 0xb14e65e7, 76 },
    { 0x602e65ea, 77 },
    { 0, 0 },
    { 0xef2e65ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e65f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e65f3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fe65fb, 33 },
    { 0xb67e65fb, 92 },
    { 0x758e65fd, 94 },
    { 0xd8fe65fb, 44 },
    { 0, 0 },
    { 0xf77e6600, 76 },
    { 0x3ee6601, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x304e6604, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515e6608, 64 },
    { 0xd24e6609, 74 },
    { 0xff3e660a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60e6612, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de6617, 33 },
    { 0x80ae6618, 51 },
    { 0xc2fe6617, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ae661f, 88 },
    { 0xe83e6620, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaae6626, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcae662d, 61 },
    { 0x33be662e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53e6631, 76 },
    { 0x676e6631, 91 },
    { 0x758e6633, 90 },
    { 0x32fe6634, 98 },
    { 0x512e6634, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e6638, 33 },
    { 0xd6de6639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60e6644, 77 },
    { 0x52ce6645, 95 },
    { 0x60e6645, 58 },
    { 0xf5de6644, 43 },
    { 0xa57e6648, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18e664c, 43 },
    { 0x678e664d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75de6650, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2e6655, 90 },
    { 0, 0 },
    { 0xc66e6657, 23 },
    { 0x965e6657, 34 },
    { 0, 0 },
    { 0xdd5e665a, 90 },
    { 0xbaae665b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3e665e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ce6661, 46 },
    { 0, 0 },
    { 0xa29e6663, 100 },
    { 0xe8fe6664, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1be6668, 68 },
    { 0x1f0e6668, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aee6671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbe6677, 95 },
    { 0xc49e6678, 90 },
    { 0x844e6679, 98 },
    { 0x9b3e6679, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x17de667d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36e6683, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ae6686, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e6689, 33 },
    { 0x698e668a, 46 },
    { 0x2be668b, 61 },
    { 0xc09e668a, 76 },
    { 0x5b6e668d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7e6690, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0de6694, 93 },
    { 0xad2e6694, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eae669f, 33 },
    { 0xdf7e66a0, 37 },
    { 0xe3ce66a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x788e66a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616e66a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e66ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5e66af, 98 },
    { 0, 0 },
    { 0x2fbe66b1, 98 },
    { 0x419e66b2, 43 },
    { 0x89e66b3, 61 },
    { 0xcafe66b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58e66b7, 33 },
    { 0x516e66b8, 100 },
    { 0xbd8e66b7, 33 },
    { 0, 0 },
    { 0x913e66bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956e66c4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950e66c8, 77 },
    { 0, 0 },
    { 0xd9ae66ca, 90 },
    { 0xd56e66ca, 61 },
    { 0x34de66cc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9e66cf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffe66d5, 90 },
    { 0x573e66d5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22e66d9, 55 },
    { 0x9a5e66da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aae66e0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0e66e6, 98 },
    { 0xedfe66e7, 95 },
    { 0x823e66e6, 98 },
    { 0x7d5e66e9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x913e66ec, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e66f6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410e6702, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeae6705, 33 },
    { 0xa11e6705, 98 },
    { 0x961e6707, 61 },
    { 0, 0 },
    { 0xea8e6709, 76 },
    { 0x4a4e670a, 94 },
    { 0, 0 },
    { 0x687e670c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e670f, 66 },
    { 0xf41e6710, 95 },
    { 0x33de6711, 61 },
    { 0, 0 },
    { 0xd8ce6713, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750e6718, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ce671b, 90 },
    { 0xf12e671c, 90 },
    { 0x75e671d, 55 },
    { 0x98e671e, 90 },
    { 0, 0 },
    { 0x7e4e6720, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0e6723, 90 },
    { 0x155e6723, 95 },
    { 0xd4ae6725, 95 },
    { 0x5b2e6723, 98 },
    { 0x859e6727, 96 },
    { 0x137e6724, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ee6730, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa63e6735, 84 },
    { 0x8f2e6736, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5de6739, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e6742, 95 },
    { 0, 0 },
    { 0x312e6744, 68 },
    { 0, 0 },
    { 0x41de6746, 78 },
    { 0x58e6747, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e674e, 90 },
    { 0x8cfe674f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fe6755, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ce675c, 33 },
    { 0, 0 },
    { 0x9e7e675e, 43 },
    { 0xb89e675e, 61 },
    { 0, 0 },
    { 0xa67e6761, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7e6768, 33 },
    { 0xc3e6768, 90 },
    { 0, 0 },
    { 0x827e676b, 33 },
    { 0, 0 },
    { 0xc68e676d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469e6773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4e677f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bde6784, 52 },
    { 0xe1ce6784, 95 },
    { 0x3b1e6786, 23 },
    { 0xd80e6787, 95 },
    { 0x90fe6786, 61 },
    { 0, 0 },
    { 0xc1ee678a, 90 },
    { 0x6ade678a, 76 },
    { 0, 0 },
    { 0x9eae678d, 90 },
    { 0x763e678d, 98 },
    { 0x9cae678f, 78 },
    { 0x6bae6790, 33 },
    { 0xab8e6791, 23 },
    { 0xae7e6791, 90 },
    { 0xb65e6793, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ce6798, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xede67a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8e67a6, 95 },
    { 0x994e67a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72e67aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151e67af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9e67b5, 85 },
    { 0xc7ae67b6, 66 },
    { 0xd31e67b7, 99 },
    { 0x70e67b8, 61 },
    { 0x3f1e67b9, 76 },
    { 0x1a5e67b5, 98 },
    { 0x92ae67b7, 90 },
    { 0xe99e67b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21e67c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ee67c3, 99 },
    { 0xa44e67c3, 90 },
    { 0xb80e67c5, 98 },
    { 0x585e67c6, 76 },
    { 0xa0ce67c6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711e67cb, 33 },
    { 0, 0 },
    { 0xe7ce67cd, 55 },
    { 0x9e4e67ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfe67d2, 90 },
    { 0, 0 },
    { 0x60ae67d4, 33 },
    { 0xc1ee67d4, 61 },
    { 0x183e67d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255e67da, 90 },
    { 0xc88e67db, 90 },
    { 0x2f7e67dc, 23 },
    { 0x2abe67dd, 81 },
    { 0x71ee67dd, 23 },
    { 0xe0ce67df, 66 },
    { 0, 0 },
    { 0x72e67e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1e67e8, 66 },
    { 0x8dde67e9, 55 },
    { 0xc9e67e9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ece67ed, 90 },
    { 0xd91e67ee, 61 },
    { 0xfcbe67ef, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ce67f3, 75 },
    { 0x603e67f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642e67f8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffe67fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928e6801, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4be6804, 90 },
    { 0, 0 },
    { 0x4d5e6806, 95 },
    { 0, 0 },
    { 0x750e6808, 55 },
    { 0xfcee6809, 23 },
    { 0x30fe680a, 66 },
    { 0x5c3e680b, 88 },
    { 0xbcae680a, 95 },
    { 0x32ee6809, 23 },
    { 0x64ee680e, 69 },
    { 0xa71e680f, 61 },
    { 0x209e680f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609e681a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x10de681d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ee6820, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3de6826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50e682f, 95 },
    { 0, 0 },
    { 0xc52e6831, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24e6835, 61 },
    { 0, 0 },
    { 0x9ace6837, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ce683e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e6841, 93 },
    { 0x9f0e6842, 93 },
    { 0x5bde6843, 61 },
    { 0x10e6844, 55 },
    { 0x5a2e6841, 81 },
    { 0x324e6846, 61 },
    { 0x627e6846, 61 },
    { 0xe14e6848, 52 },
    { 0x8e6849, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e684e, 33 },
    { 0xf5fe684f, 33 },
    { 0, 0 },
    { 0x856e6851, 98 },
    { 0, 0 },
    { 0x932e6853, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbe6857, 78 },
    { 0x9b9e6857, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x79de685b, 90 },
    { 0xbb1e685c, 90 },
    { 0xd10e685c, 98 },
    { 0, 0 },
    { 0x46be685f, 84 },
    { 0xe2be6860, 76 },
    { 0x7e3e6860, 98 },
    { 0xa9ce6862, 36 },
    { 0x37be6863, 61 },
    { 0x2e9e6862, 75 },
    { 0x87de6865, 76 },
    { 0x904e6866, 66 },
    { 0xf7de6867, 90 },
    { 0x696e6868, 52 },
    { 0x29ce6861, 33 },
    { 0, 0 },
    { 0x162e686b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655e686f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e6872, 34 },
    { 0xf6ce6873, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dae6879, 55 },
    { 0x6a5e687a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ee687e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922e6883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ce6887, 98 },
    { 0xe62e6888, 43 },
    { 0x260e6889, 33 },
    { 0, 0 },
    { 0x36ee688b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ce688e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ae6894, 76 },
    { 0x3dbe6894, 95 },
    { 0xd15e6895, 33 },
    { 0xaf2e6896, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0e689a, 46 },
    { 0x57fe689b, 55 },
    { 0x955e689a, 76 },
    { 0x513e689a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44e68a0, 67 },
    { 0x80ee68a0, 90 },
    { 0, 0 },
    { 0x5a1e68a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647e68a7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e68ad, 46 },
    { 0x86ae68ae, 61 },
    { 0x1c2e68af, 76 },
    { 0xdb8e68ad, 67 },
    { 0xd86e68b1, 75 },
    { 0xa52e68b1, 90 },
    { 0x283e68b2, 33 },
    { 0xc0e68b2, 61 },
    { 0xd55e68b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e68b8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4e68bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x479e68bf, 99 },
    { 0, 0 },
    { 0x266e68c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bae68c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5e68ca, 48 },
    { 0xd99e68cb, 43 },
    { 0xa6fe68cc, 33 },
    { 0xe44e68ca, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x224e68d0, 98 },
    { 0xa84e68d1, 55 },
    { 0x9cde68d2, 61 },
    { 0xb88e68d2, 46 },
    { 0, 0 },
    { 0xafae68d5, 90 },
    { 0, 0 },
    { 0x17ce68d7, 95 },
    { 0, 0 },
    { 0x8bbe68d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e68de, 23 },
    { 0x49de68df, 61 },
    { 0x687e68de, 90 },
    { 0, 0 },
    { 0x1d9e68e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1e68e6, 23 },
    { 0, 0 },
    { 0x1b4e68e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dde68ed, 90 },
    { 0x5cde68ee, 78 },
    { 0xc33e68ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29be68f3, 55 },
    { 0x2ede68f4, 99 },
    { 0x7b9e68f4, 69 },
    { 0x728e68f4, 76 },
    { 0xacee68f6, 46 },
    { 0, 0 },
    { 0x6fce68f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89e6902, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3be690e, 61 },
    { 0x7a2e690f, 98 },
    { 0xcfde690e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfee6914, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de6919, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bce691c, 90 },
    { 0x674e691c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03e6922, 34 },
    { 0xb02e6923, 61 },
    { 0x5a6e6923, 61 },
    { 0x8b9e6925, 56 },
    { 0x32be6926, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdee692a, 95 },
    { 0x218e692b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x554e692f, 33 },
    { 0x76fe6930, 46 },
    { 0x46be6930, 61 },
    { 0, 0 },
    { 0x380e6933, 95 },
    { 0x5e2e6934, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbae6939, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeafe693e, 46 },
    { 0xaede693f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcce6943, 61 },
    { 0x3cae6944, 85 },
    { 0, 0 },
    { 0x8b1e6946, 43 },
    { 0, 0 },
    { 0x81de6948, 95 },
    { 0xfc7e6949, 53 },
    { 0xf46e694a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee694e, 90 },
    { 0, 0 },
    { 0x730e6950, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e6953, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8e6956, 99 },
    { 0xb96e6957, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802e695c, 95 },
    { 0x819e695d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe6963, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63e6966, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccce6971, 46 },
    { 0x691e6972, 77 },
    { 0x8dce6973, 60 },
    { 0xe31e6974, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ae6978, 61 },
    { 0x86ae6978, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888e697d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ce6980, 93 },
    { 0xd89e6981, 90 },
    { 0x754e6981, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68e6986, 19 },
    { 0x59ae6987, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaee698b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ee6993, 85 },
    { 0xffbe6994, 98 },
    { 0, 0 },
    { 0x8c7e6996, 23 },
    { 0x566e6996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1e699a, 61 },
    { 0, 0 },
    { 0x15e699c, 61 },
    { 0xc67e699d, 61 },
    { 0, 0 },
    { 0x6b0e699f, 99 },
    { 0x3e4e69a0, 97 },
    { 0x77ce69a0, 90 },
    { 0x9dbe699f, 95 },
    { 0xbe7e69a3, 90 },
    { 0x94ee69a4, 99 },
    { 0xf2e69a3, 53 },
    { 0x82de69a2, 98 },
    { 0, 0 },
    { 0x24fe69a8, 77 },
    { 0x7e3e69a8, 98 },
    { 0, 0 },
    { 0x694e69ab, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74e69ae, 91 },
    { 0xd82e69af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ae69b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6e69b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x531e69bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238e69c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e69cb, 99 },
    { 0x3d1e69cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9e69d0, 88 },
    { 0x17ae69d0, 95 },
    { 0x1c0e69d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe69d9, 61 },
    { 0x6ae69d9, 61 },
    { 0x897e69db, 90 },
    { 0x47e69dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3e69df, 23 },
    { 0, 0 },
    { 0x60fe69e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fe69e6, 81 },
    { 0, 0 },
    { 0xd95e69e8, 90 },
    { 0, 0 },
    { 0x7fce69ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445e69f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfe69f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79e69ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960e6a06, 42 },
    { 0xe1e6a06, 98 },
    { 0x7bbe6a07, 95 },
    { 0x5f5e6a09, 95 },
    { 0xe7de6a0a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de6a0d, 33 },
    { 0x80fe6a0e, 85 },
    { 0xe53e6a0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce6a12, 100 },
    { 0xec8e6a12, 90 },
    { 0x79ee6a12, 61 },
    { 0x4c5e6a15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358e6a1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5e6a22, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1e6a25, 34 },
    { 0xfe3e6a26, 84 },
    { 0, 0 },
    { 0x546e6a28, 98 },
    { 0xdd1e6a28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7e6a2e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75e6a31, 33 },
    { 0, 0 },
    { 0xdf7e6a33, 56 },
    { 0, 0 },
    { 0x53ee6a35, 90 },
    { 0x822e6a36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe6a3c, 52 },
    { 0xd2fe6a3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ae6a41, 33 },
    { 0x6cbe6a42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591e6a47, 33 },
    { 0x2dde6a47, 23 },
    { 0xbd4e6a49, 95 },
    { 0xbaee6a4a, 76 },
    { 0xe2ee6a4b, 61 },
    { 0x66fe6a49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5e6a4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1e6a56, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917e6a5f, 90 },
    { 0xe61e6a5f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ee6a63, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5e6a67, 95 },
    { 0x790e6a68, 78 },
    { 0xf6be6a68, 33 },
    { 0x59ae6a6a, 33 },
    { 0, 0 },
    { 0x91be6a6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fce6a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40e6a76, 34 },
    { 0x8eae6a77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245e6a7d, 99 },
    { 0x4cbe6a7e, 90 },
    { 0xe6a7f, 34 },
    { 0x66de6a7f, 80 },
    { 0x84ce6a81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ce6a84, 40 },
    { 0, 0 },
    { 0x5c7e6a86, 33 },
    { 0x4b5e6a87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee6a8a, 23 },
    { 0xe80e6a8b, 90 },
    { 0x5c5e6a8c, 69 },
    { 0x46ce6a8d, 61 },
    { 0x507e6a8d, 78 },
    { 0x17e6a8f, 91 },
    { 0, 0 },
    { 0x593e6a91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0e6a97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde6aa0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ee6aa4, 61 },
    { 0x719e6aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2e6aaa, 33 },
    { 0x4b6e6aab, 100 },
    { 0x79e6aab, 90 },
    { 0, 0 },
    { 0x6f6e6aae, 66 },
    { 0xc63e6aaf, 61 },
    { 0, 0 },
    { 0xb1be6ab1, 90 },
    { 0x450e6ab2, 95 },
    { 0x289e6ab3, 33 },
    { 0x8aae6ab3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110e6ab9, 23 },
    { 0x894e6ab9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989e6abe, 99 },
    { 0, 0 },
    { 0x5f3e6ac0, 33 },
    { 0x5e7e6ac1, 68 },
    { 0, 0 },
    { 0x1d8e6ac3, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x586e6ac7, 90 },
    { 0x1ee6ac7, 90 },
    { 0x808e6ac9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90e6acd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ae6ad6, 50 },
    { 0, 0 },
    { 0x92fe6ad8, 61 },
    { 0xa99e6ad9, 61 },
    { 0x54de6ada, 68 },
    { 0xdb2e6ad9, 66 },
    { 0, 0 },
    { 0x48ee6add, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4e6ae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x761e6aea, 61 },
    { 0xdc3e6aeb, 74 },
    { 0xf20e6aeb, 66 },
    { 0xeffe6aed, 46 },
    { 0xe5be6aee, 61 },
    { 0xb09e6aec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28e6af2, 33 },
    { 0, 0 },
    { 0xd16e6af4, 17 },
    { 0x3e5e6af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6e6afa, 33 },
    { 0, 0 },
    { 0x46be6afc, 90 },
    { 0, 0 },
    { 0xbf1e6afe, 67 },
    { 0x870e6aff, 61 },
    { 0x4a5e6aff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2e6b04, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5e6b08, 98 },
    { 0x127e6b09, 34 },
    { 0xe08e6b0a, 33 },
    { 0x97fe6b09, 97 },
    { 0x8dbe6b0c, 98 },
    { 0x67ee6b09, 91 },
    { 0xc45e6b08, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ce6b12, 61 },
    { 0xc57e6b13, 81 },
    { 0x540e6b14, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64be6b18, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e6b21, 43 },
    { 0x7dfe6b21, 87 },
    { 0x8d9e6b23, 23 },
    { 0xf7e6b24, 90 },
    { 0x801e6b23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cee6b2a, 85 },
    { 0, 0 },
    { 0x8b4e6b2c, 91 },
    { 0, 0 },
    { 0x554e6b2e, 28 },
    { 0x501e6b2e, 23 },
    { 0, 0 },
    { 0xaede6b31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7e6b34, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x80de6b37, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x972e6b3a, 43 },
    { 0x926e6b3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cae6b40, 90 },
    { 0xa7ae6b41, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1e6b4d, 33 },
    { 0xc7de6b4e, 61 },
    { 0x1d0e6b4f, 76 },
    { 0, 0 },
    { 0x24be6b51, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdae6b56, 90 },
    { 0, 0 },
    { 0x37ce6b58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfe6b5c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbe6b5f, 34 },
    { 0, 0 },
    { 0x671e6b61, 95 },
    { 0x14fe6b62, 98 },
    { 0xd28e6b63, 33 },
    { 0xe50e6b63, 65 },
    { 0, 0 },
    { 0x6a3e6b66, 33 },
    { 0, 0 },
    { 0x25ee6b68, 90 },
    { 0xaee6b68, 17 },
    { 0xd4ee6b6a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2e6b6e, 50 },
    { 0xb80e6b6e, 76 },
    { 0xab0e6b6e, 90 },
    { 0x793e6b70, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53e6b77, 90 },
    { 0, 0 },
    { 0x6a4e6b79, 61 },
    { 0x242e6b79, 33 },
    { 0x14ae6b7b, 100 },
    { 0, 0 },
    { 0x25ae6b7d, 65 },
    { 0xb81e6b7d, 95 },
    { 0x9ae6b7f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x210e6b82, 61 },
    { 0xf0ae6b83, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14e6b88, 23 },
    { 0xbcbe6b88, 88 },
    { 0, 0 },
    { 0x8f7e6b8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbce6b9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30e6b9e, 75 },
    { 0, 0 },
    { 0xe28e6ba0, 84 },
    { 0, 0 },
    { 0xed9e6ba2, 90 },
    { 0, 0 },
    { 0x87de6ba4, 95 },
    { 0x915e6ba5, 99 },
    { 0xeb5e6ba6, 93 },
    { 0xd0ae6ba5, 95 },
    { 0x7c3e6ba8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x981e6bab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afe6bae, 91 },
    { 0x9b1e6baf, 33 },
    { 0xede6bb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6de6bb4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652e6bb8, 90 },
    { 0x130e6bb9, 61 },
    { 0, 0 },
    { 0x84fe6bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e6bbe, 90 },
    { 0x6b0e6bbf, 95 },
    { 0, 0 },
    { 0x856e6bc1, 68 },
    { 0, 0 },
    { 0xdbee6bc3, 76 },
    { 0, 0 },
    { 0x4ade6bc5, 23 },
    { 0xb0be6bc6, 78 },
    { 0x759e6bc7, 65 },
    { 0x59e6bc7, 87 },
    { 0xf0de6bc9, 33 },
    { 0xfc2e6bca, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0e6bcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ae6bd0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3e6bd4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81fe6bd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x899e6bdb, 55 },
    { 0x487e6bdc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119e6be1, 33 },
    { 0xac4e6be1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46e6be5, 67 },
    { 0xec4e6be6, 61 },
    { 0xb4e6be7, 90 },
    { 0x782e6be6, 90 },
    { 0xabee6be5, 33 },
    { 0x2e3e6be5, 40 },
    { 0, 0 },
    { 0xb2ae6bec, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244e6bf1, 23 },
    { 0x746e6bf2, 98 },
    { 0x12ee6bf2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ae6bf6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4de6bf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x168e6bfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823e6c00, 33 },
    { 0x7d8e6c01, 99 },
    { 0xcfae6c02, 93 },
    { 0xd10e6c03, 61 },
    { 0x7f7e6c02, 61 },
    { 0x7c2e6c02, 95 },
    { 0x1c4e6c06, 90 },
    { 0xb5ae6c04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5e6c0a, 23 },
    { 0xc19e6c0a, 23 },
    { 0x5b4e6c0c, 93 },
    { 0x41ae6c0c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598e6c12, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x373e6c15, 33 },
    { 0xa62e6c16, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63de6c1c, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ae6c20, 87 },
    { 0, 0 },
    { 0x832e6c22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e6c28, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9e6c2e, 98 },
    { 0x2ace6c2e, 78 },
    { 0, 0 },
    { 0x5f4e6c31, 90 },
    { 0xc0be6c32, 76 },
    { 0xfbee6c33, 52 },
    { 0, 0 },
    { 0x665e6c35, 55 },
    { 0xb6ee6c35, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfae6c40, 61 },
    { 0xf14e6c41, 52 },
    { 0x834e6c41, 33 },
    { 0x5e1e6c43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e6c46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e6c49, 90 },
    { 0xfe1e6c4a, 87 },
    { 0xd41e6c4b, 99 },
    { 0, 0 },
    { 0xbc6e6c4d, 58 },
    { 0xb52e6c4e, 90 },
    { 0xe70e6c4e, 99 },
    { 0x694e6c4e, 98 },
    { 0x4b5e6c4e, 95 },
    { 0, 0 },
    { 0xb1de6c53, 90 },
    { 0x591e6c54, 43 },
    { 0xfb7e6c54, 33 },
    { 0xa2ce6c55, 77 },
    { 0x2c9e6c53, 76 },
    { 0, 0 },
    { 0xb87e6c59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e6c5c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272e6c62, 90 },
    { 0x3d5e6c63, 90 },
    { 0x433e6c64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2e6c69, 43 },
    { 0x2aee6c6a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ce6c6e, 55 },
    { 0xc54e6c6f, 85 },
    { 0x35be6c6f, 64 },
    { 0x557e6c6e, 90 },
    { 0xa04e6c70, 99 },
    { 0xd2be6c73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ee6c7a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x51de6c7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e6c89, 55 },
    { 0x175e6c8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0e6c90, 33 },
    { 0x333e6c91, 23 },
    { 0xe40e6c92, 93 },
    { 0xfe6e6c93, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3e6c96, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36de6c9a, 69 },
    { 0x999e6c9b, 33 },
    { 0xa0de6c9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7de6ca5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbe6ca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9e6cad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127e6cb2, 98 },
    { 0x9bee6cb3, 34 },
    { 0, 0 },
    { 0x113e6cb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e6cb8, 91 },
    { 0, 0 },
    { 0xcfce6cba, 66 },
    { 0x620e6cbb, 52 },
    { 0xfabe6cbc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x14de6cbf, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de6cc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfee6cca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303e6cce, 61 },
    { 0, 0 },
    { 0xd15e6cd0, 90 },
    { 0x1c5e6cd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2e6cdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e6ce0, 69 },
    { 0xcb7e6ce1, 85 },
    { 0x80e6ce2, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x91de6ce5, 98 },
    { 0, 0 },
    { 0x3e4e6ce7, 98 },
    { 0xf10e6ce8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2e6ced, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9e6cf0, 90 },
    { 0xb59e6cf1, 33 },
    { 0x5d2e6cf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x70be6cf5, 34 },
    { 0xd1fe6cf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4e6cf9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940e6d00, 98 },
    { 0x400e6d01, 33 },
    { 0x27ae6d01, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9e6d09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffe6d0c, 33 },
    { 0x5d0e6d0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5e6d11, 84 },
    { 0x92fe6d11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e6d1c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1e6d21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24e6d2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17e6d38, 76 },
    { 0xf00e6d38, 33 },
    { 0x7b3e6d3a, 34 },
    { 0x137e6d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fe6d3e, 33 },
    { 0xb47e6d3f, 40 },
    { 0, 0 },
    { 0x5e7e6d41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855e6d46, 76 },
    { 0xc37e6d47, 46 },
    { 0xa38e6d47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276e6d4c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0e6d50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9e6d53, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24de6d5f, 90 },
    { 0x29ee6d5f, 33 },
    { 0xd8be6d61, 90 },
    { 0xbfae6d62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621e6d67, 55 },
    { 0xb44e6d68, 76 },
    { 0x904e6d69, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eae6d70, 78 },
    { 0, 0 },
    { 0xf45e6d72, 84 },
    { 0x1c8e6d73, 61 },
    { 0xdc7e6d74, 90 },
    { 0x2a7e6d75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cee6d78, 33 },
    { 0, 0 },
    { 0x39fe6d7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9e6d7d, 90 },
    { 0, 0 },
    { 0x96fe6d7f, 23 },
    { 0x352e6d80, 98 },
    { 0x574e6d81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8be6d84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747e6d88, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23e6d8e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c4e6d93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce6d96, 68 },
    { 0, 0 },
    { 0x575e6d98, 99 },
    { 0x21de6d99, 76 },
    { 0x7d5e6d9a, 23 },
    { 0xd27e6d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x99be6d9e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffe6da5, 61 },
    { 0xbe3e6da6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x951e6da9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73e6dad, 61 },
    { 0xff9e6dae, 100 },
    { 0x6d3e6daf, 40 },
    { 0x870e6dae, 84 },
    { 0xd25e6daf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ae6db4, 95 },
    { 0, 0 },
    { 0x659e6db6, 33 },
    { 0xca8e6db7, 90 },
    { 0x7efe6db8, 98 },
    { 0xec4e6db9, 33 },
    { 0xa8be6db7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800e6dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce6dc2, 43 },
    { 0xe36e6dc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33e6dc7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fe6dcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e6dd3, 61 },
    { 0x84be6dd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ae6de1, 52 },
    { 0x9ce6de2, 66 },
    { 0x99ce6de3, 95 },
    { 0x3bfe6de3, 46 },
    { 0x773e6de4, 61 },
    { 0x822e6de5, 90 },
    { 0xb9ae6de6, 43 },
    { 0, 0 },
    { 0x97be6de9, 65 },
    { 0x203e6dea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98e6ded, 100 },
    { 0x79de6dee, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ce6df2, 90 },
    { 0, 0 },
    { 0x3a1e6df4, 33 },
    { 0xe6ee6df5, 61 },
    { 0xa79e6df6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91e6dfc, 43 },
    { 0, 0 },
    { 0xf20e6dfe, 65 },
    { 0, 0 },
    { 0xf64e6e00, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4e6e08, 38 },
    { 0, 0 },
    { 0xd39e6e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ee6e0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70e6e15, 23 },
    { 0xf09e6e16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ce6e19, 68 },
    { 0xc00e6e19, 90 },
    { 0x6d7e6e1b, 95 },
    { 0x65fe6e1b, 61 },
    { 0x33ce6e1d, 33 },
    { 0xa7e6e1e, 87 },
    { 0xc81e6e1f, 78 },
    { 0x454e6e1f, 88 },
    { 0x402e6e21, 34 },
    { 0xe65e6e1a, 55 },
    { 0, 0 },
    { 0x39be6e24, 61 },
    { 0x8c8e6e25, 23 },
    { 0x9aee6e26, 95 },
    { 0xceae6e27, 66 },
    { 0x8bce6e28, 38 },
    { 0x83fe6e29, 23 },
    { 0x496e6e27, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ee6e35, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x877e6e38, 43 },
    { 0, 0 },
    { 0xab6e6e3a, 78 },
    { 0, 0 },
    { 0xbfde6e3c, 33 },
    { 0x2e7e6e3d, 90 },
    { 0x201e6e3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09e6e41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518e6e45, 23 },
    { 0x799e6e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88be6e49, 98 },
    { 0x12fe6e49, 33 },
    { 0x9cae6e4b, 43 },
    { 0x4abe6e4c, 80 },
    { 0xeaae6e4d, 33 },
    { 0xe3e6e4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ae6e55, 95 },
    { 0x9fe6e55, 75 },
    { 0x86e6e57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e6e5c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x288e6e5f, 95 },
    { 0, 0 },
    { 0x73fe6e61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23e6e6c, 87 },
    { 0x9eae6e6c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe6e78, 76 },
    { 0x7a2e6e79, 91 },
    { 0x436e6e79, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88e6e7f, 61 },
    { 0, 0 },
    { 0x9e4e6e81, 98 },
    { 0x209e6e81, 77 },
    { 0x3f3e6e83, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ee6e87, 34 },
    { 0x772e6e88, 99 },
    { 0xbc8e6e87, 36 },
    { 0x44de6e8a, 23 },
    { 0x500e6e8b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ae6e90, 46 },
    { 0xf02e6e90, 34 },
    { 0x85fe6e92, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fe6e95, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae6e9a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56e6e9d, 33 },
    { 0, 0 },
    { 0x704e6e9f, 99 },
    { 0, 0 },
    { 0xdce6ea1, 33 },
    { 0, 0 },
    { 0xffee6ea3, 23 },
    { 0xf46e6ea4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85e6ea7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e6eab, 55 },
    { 0, 0 },
    { 0x2e1e6ead, 90 },
    { 0, 0 },
    { 0x8aee6eaf, 43 },
    { 0xcb5e6eb0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6be6eba, 93 },
    { 0xd16e6ebb, 17 },
    { 0xe0ae6ebc, 61 },
    { 0x1d9e6ebd, 93 },
    { 0x40ee6ebe, 95 },
    { 0, 0 },
    { 0xf85e6ec0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ae6ec5, 61 },
    { 0xe3de6ec5, 61 },
    { 0xd03e6ec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ee6ecc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6de6ed5, 56 },
    { 0, 0 },
    { 0x420e6ed7, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x799e6eda, 98 },
    { 0, 0 },
    { 0x728e6edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42be6edf, 95 },
    { 0xd8e6ee0, 66 },
    { 0, 0 },
    { 0xcc1e6ee2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4e6ee5, 99 },
    { 0, 0 },
    { 0xde8e6ee7, 33 },
    { 0, 0 },
    { 0x742e6ee9, 90 },
    { 0x7fe6eea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4e6eed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82e6ef4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433e6efa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4e6eff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263e6f04, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa7e6f07, 91 },
    { 0x1f5e6f08, 95 },
    { 0x19de6f08, 99 },
    { 0, 0 },
    { 0x8c7e6f0b, 23 },
    { 0xb09e6f0c, 90 },
    { 0, 0 },
    { 0x83fe6f0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219e6f13, 91 },
    { 0x838e6f14, 95 },
    { 0x67fe6f14, 95 },
    { 0x8dee6f15, 98 },
    { 0xa2be6f15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bee6f1c, 74 },
    { 0x1ce6f1c, 99 },
    { 0, 0 },
    { 0x8ae6f1f, 23 },
    { 0, 0 },
    { 0xffde6f21, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593e6f26, 97 },
    { 0xa98e6f27, 98 },
    { 0x4d4e6f27, 76 },
    { 0xbb0e6f29, 33 },
    { 0, 0 },
    { 0x193e6f2b, 100 },
    { 0x95de6f2c, 98 },
    { 0x546e6f2d, 85 },
    { 0x996e6f2d, 78 },
    { 0x5e7e6f2f, 23 },
    { 0x3e3e6f30, 23 },
    { 0x155e6f2e, 90 },
    { 0x968e6f32, 84 },
    { 0x4b5e6f32, 76 },
    { 0x1dce6f30, 99 },
    { 0x80ce6f35, 90 },
    { 0xb3ae6f30, 95 },
    { 0xe4be6f37, 36 },
    { 0xae1e6f33, 66 },
    { 0x6cae6f37, 36 },
    { 0x4eee6f3a, 90 },
    { 0x270e6f3b, 98 },
    { 0xc2ae6f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ae6f3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289e6f50, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1e6f55, 61 },
    { 0x1aee6f56, 90 },
    { 0, 0 },
    { 0x229e6f58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6e6f64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7de6f69, 90 },
    { 0xed9e6f69, 98 },
    { 0x930e6f6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbe6f70, 76 },
    { 0x5b3e6f71, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933e6f75, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fae6f7b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15e6f7e, 55 },
    { 0, 0 },
    { 0xf93e6f80, 95 },
    { 0x365e6f81, 95 },
    { 0x77be6f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72e6f86, 93 },
    { 0xd26e6f87, 69 },
    { 0xd71e6f87, 99 },
    { 0x83e6f88, 34 },
    { 0x1dfe6f8a, 77 },
    { 0, 0 },
    { 0xc45e6f8c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9e6f92, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ae6f95, 61 },
    { 0, 0 },
    { 0xfd6e6f97, 61 },
    { 0xdaae6f98, 23 },
    { 0x6de6f98, 33 },
    { 0, 0 },
    { 0xf8be6f9b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7e6fa0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe6fa6, 34 },
    { 0, 0 },
    { 0xebde6fa8, 38 },
    { 0xa7ae6fa9, 87 },
    { 0xef9e6faa, 95 },
    { 0x14fe6fa9, 93 },
    { 0xfee6faa, 100 },
    { 0x98be6faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce6fb1, 99 },
    { 0x911e6fb2, 98 },
    { 0xfcee6fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5e6fb9, 90 },
    { 0xfabe6fb9, 84 },
    { 0, 0 },
    { 0x8c8e6fbc, 90 },
    { 0, 0 },
    { 0xebbe6fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e6fc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9e6fc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0e6fce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e6fd4, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9e6fd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ee6fda, 61 },
    { 0x1dde6fdb, 34 },
    { 0xfe6fdc, 90 },
    { 0, 0 },
    { 0x8dae6fde, 33 },
    { 0, 0 },
    { 0x23ee6fe0, 88 },
    { 0x62ee6fe0, 90 },
    { 0, 0 },
    { 0x749e6fe3, 75 },
    { 0, 0 },
    { 0x864e6fe5, 85 },
    { 0x79ce6fe6, 99 },
    { 0x64ee6fe7, 98 },
    { 0, 0 },
    { 0xb1fe6fe9, 61 },
    { 0x839e6fea, 76 },
    { 0x609e6feb, 33 },
    { 0, 0 },
    { 0x518e6fed, 61 },
    { 0xde2e6fee, 34 },
    { 0xb70e6fee, 46 },
    { 0x92ce6fef, 66 },
    { 0xa94e6ff1, 55 },
    { 0x2c4e6ff1, 98 },
    { 0x284e6ff3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd20e6ffa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x166e6ffd, 84 },
    { 0, 0 },
    { 0xb97e6fff, 95 },
    { 0xade6fff, 33 },
    { 0x5ade7001, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xade7005, 90 },
    { 0x19ae7005, 95 },
    { 0xb14e7007, 55 },
    { 0xeade7008, 93 },
    { 0xa0be7009, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8e700f, 33 },
    { 0, 0 },
    { 0x619e7011, 43 },
    { 0xb78e7011, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ae7017, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64e701f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76e7022, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94e7027, 100 },
    { 0x3dbe7028, 78 },
    { 0xe90e7029, 99 },
    { 0, 0 },
    { 0xd63e702b, 61 },
    { 0, 0 },
    { 0x7efe702d, 90 },
    { 0xf97e702e, 90 },
    { 0, 0 },
    { 0x3b7e7030, 33 },
    { 0x8d6e7030, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e7034, 99 },
    { 0, 0 },
    { 0x671e7036, 88 },
    { 0xd49e7036, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe703a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ee703d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfee7045, 53 },
    { 0, 0 },
    { 0xfa1e7047, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81be704b, 90 },
    { 0x3f8e704b, 85 },
    { 0x2b9e704d, 52 },
    { 0x4cae704c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80e7052, 46 },
    { 0x79fe7053, 87 },
    { 0x7b0e7054, 23 },
    { 0xa2fe7053, 84 },
    { 0x6ae7056, 33 },
    { 0x8bbe7053, 23 },
    { 0xae3e7058, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74e705f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310e7067, 98 },
    { 0xb1fe7067, 40 },
    { 0xb62e7069, 43 },
    { 0, 0 },
    { 0xd63e706b, 95 },
    { 0x1aee706c, 98 },
    { 0xdc6e706d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ce7072, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93e7076, 46 },
    { 0, 0 },
    { 0x79fe7078, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbe707e, 98 },
    { 0x8cfe707f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545e7089, 23 },
    { 0, 0 },
    { 0xa3ee708b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5de708e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16e7093, 46 },
    { 0xc4de7093, 85 },
    { 0, 0 },
    { 0x849e7096, 91 },
    { 0x830e7096, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ade709b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9e709e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748e70ae, 99 },
    { 0xbcbe70ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25de70b4, 90 },
    { 0x300e70b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2e70b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x290e70bb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571e70bf, 43 },
    { 0x5d4e70c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafee70c8, 23 },
    { 0xa8ce70c9, 34 },
    { 0x370e70ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3e70cd, 66 },
    { 0, 0 },
    { 0xf48e70cf, 76 },
    { 0, 0 },
    { 0xebae70d1, 61 },
    { 0x456e70d1, 84 },
    { 0, 0 },
    { 0x15ae70d4, 34 },
    { 0, 0 },
    { 0x88e70d6, 83 },
    { 0, 0 },
    { 0xcebe70d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7e70db, 76 },
    { 0x20e70dc, 84 },
    { 0xd4ae70dd, 90 },
    { 0, 0 },
    { 0xc0fe70df, 43 },
    { 0x6aae70df, 33 },
    { 0, 0 },
    { 0x34de70e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x176e70e5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47e70e8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ace70eb, 90 },
    { 0xe01e70ec, 91 },
    { 0x6aee70ed, 78 },
    { 0x540e70ee, 76 },
    { 0x35fe70eb, 55 },
    { 0x85be70f0, 85 },
    { 0x75de70f1, 90 },
    { 0, 0 },
    { 0x28de70f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aee70f7, 61 },
    { 0x893e70f7, 95 },
    { 0, 0 },
    { 0xdb8e70fa, 95 },
    { 0x8afe70fb, 93 },
    { 0x44e70fc, 55 },
    { 0, 0 },
    { 0xbfee70fe, 46 },
    { 0xf36e70fe, 90 },
    { 0x37be70ff, 98 },
    { 0x8dfe7101, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81e7106, 95 },
    { 0xc8ce7106, 55 },
    { 0x124e7107, 23 },
    { 0x30ee7109, 61 },
    { 0x971e7108, 61 },
    { 0xc0de710b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e710f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87e7114, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04e7118, 93 },
    { 0x637e7118, 90 },
    { 0x6a5e711a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697e7120, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cbe7124, 90 },
    { 0, 0 },
    { 0xfb2e7126, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5e712a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3e7132, 76 },
    { 0xafbe7133, 95 },
    { 0x651e7134, 33 },
    { 0xad2e7135, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2e7139, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ae713c, 90 },
    { 0x3c5e713d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7e7141, 95 },
    { 0x8c1e7142, 33 },
    { 0x444e7143, 61 },
    { 0x90de7144, 33 },
    { 0x76e7145, 99 },
    { 0xb39e7146, 61 },
    { 0xdae7147, 61 },
    { 0x2d8e7148, 46 },
    { 0xa4e7149, 33 },
    { 0xb2ee714a, 90 },
    { 0xf36e714b, 23 },
    { 0xae5e714c, 90 },
    { 0xa2be714b, 90 },
    { 0x2a2e714e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abe7151, 23 },
    { 0x4aee7151, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70e7155, 23 },
    { 0, 0 },
    { 0xd74e7157, 38 },
    { 0x976e7158, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4e715c, 55 },
    { 0, 0 },
    { 0x50ce715e, 90 },
    { 0xed5e715f, 76 },
    { 0x11ce7160, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e716d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e7170, 61 },
    { 0xe6de7171, 33 },
    { 0xd2fe7170, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9e7175, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40e7178, 76 },
    { 0, 0 },
    { 0x21e717a, 87 },
    { 0x682e717a, 23 },
    { 0xea0e717c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x543e717f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0e7188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e718c, 90 },
    { 0, 0 },
    { 0xb89e718e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e7193, 61 },
    { 0x27be7194, 33 },
    { 0xb1e7194, 90 },
    { 0x4cfe7196, 67 },
    { 0, 0 },
    { 0xc18e7198, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841e719c, 46 },
    { 0, 0 },
    { 0x52ce719e, 23 },
    { 0x8b6e719f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61be71a3, 93 },
    { 0, 0 },
    { 0x275e71a5, 4 },
    { 0, 0 },
    { 0x6a0e71a7, 61 },
    { 0, 0 },
    { 0x2e5e71a9, 91 },
    { 0, 0 },
    { 0x570e71ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26e71b4, 61 },
    { 0, 0 },
    { 0xee5e71b6, 76 },
    { 0x90e71b6, 91 },
    { 0xe99e71b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ce71bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86e71ce, 34 },
    { 0, 0 },
    { 0x68e71d0, 90 },
    { 0x43fe71d1, 95 },
    { 0xfa5e71d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9e71d9, 87 },
    { 0, 0 },
    { 0x5c2e71db, 87 },
    { 0, 0 },
    { 0xd08e71dd, 85 },
    { 0xc4de71de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ee71e1, 61 },
    { 0xb5be71e2, 61 },
    { 0, 0 },
    { 0x370e71e4, 77 },
    { 0x63de71e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce71e9, 76 },
    { 0x14de71ea, 33 },
    { 0xbc9e71eb, 77 },
    { 0xc69e71eb, 23 },
    { 0x34be71ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x283e71f0, 78 },
    { 0xd3ae71f1, 98 },
    { 0xfbce71f2, 90 },
    { 0x646e71f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51e71f8, 90 },
    { 0xc8be71f9, 33 },
    { 0xdbee71f8, 90 },
    { 0x427e71fb, 76 },
    { 0, 0 },
    { 0xff9e71fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee7200, 90 },
    { 0, 0 },
    { 0xc99e7202, 95 },
    { 0xe2ae7203, 95 },
    { 0x97be7203, 95 },
    { 0x624e7203, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x628e720b, 91 },
    { 0x965e720b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3e7213, 33 },
    { 0, 0 },
    { 0xe78e7215, 76 },
    { 0xcefe7216, 34 },
    { 0x1dde7217, 100 },
    { 0, 0 },
    { 0x827e7219, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99e721c, 95 },
    { 0, 0 },
    { 0xe9be721e, 84 },
    { 0, 0 },
    { 0x1dee7220, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74e722c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x475e722f, 98 },
    { 0, 0 },
    { 0x4b2e7231, 66 },
    { 0x505e7231, 33 },
    { 0, 0 },
    { 0x515e7234, 90 },
    { 0xa79e7234, 99 },
    { 0x1fe7236, 55 },
    { 0xa05e7237, 98 },
    { 0x5aae7238, 61 },
    { 0x98ce7236, 98 },
    { 0x6cee723a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bee7240, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e7244, 77 },
    { 0x853e7244, 90 },
    { 0, 0 },
    { 0x57ae7247, 33 },
    { 0, 0 },
    { 0xf26e7249, 90 },
    { 0, 0 },
    { 0xc09e724b, 33 },
    { 0x1e9e724c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2be7258, 38 },
    { 0, 0 },
    { 0xe51e725a, 55 },
    { 0x829e725a, 76 },
    { 0x3ece725c, 90 },
    { 0x2fbe725d, 51 },
    { 0, 0 },
    { 0x3ffe725f, 91 },
    { 0, 0 },
    { 0xbace7261, 98 },
    { 0, 0 },
    { 0x688e7263, 95 },
    { 0x37de7263, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a7e7268, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e7272, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32de7276, 43 },
    { 0, 0 },
    { 0xabbe7278, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce727d, 93 },
    { 0, 0 },
    { 0xd41e727f, 90 },
    { 0xec8e727f, 61 },
    { 0xad0e7281, 23 },
    { 0x4bbe7282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4e728c, 35 },
    { 0x296e728d, 97 },
    { 0xf4ee728c, 95 },
    { 0xda0e728f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96e7292, 90 },
    { 0, 0 },
    { 0x183e7294, 33 },
    { 0x55be7294, 33 },
    { 0x462e7295, 23 },
    { 0xa09e7297, 43 },
    { 0xe70e7295, 34 },
    { 0x705e7299, 33 },
    { 0xf60e7294, 76 },
    { 0x83ee7294, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e72a4, 90 },
    { 0xcd2e72a5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcde72ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41be72b5, 56 },
    { 0xf14e72b6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fae72b9, 52 },
    { 0x5f8e72ba, 43 },
    { 0, 0 },
    { 0xd56e72bc, 77 },
    { 0, 0 },
    { 0x958e72be, 33 },
    { 0, 0 },
    { 0x75e72c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e72c7, 90 },
    { 0x7a0e72c8, 33 },
    { 0x468e72c7, 87 },
    { 0xdce72ca, 34 },
    { 0xb20e72cb, 33 },
    { 0xfe4e72cc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54e72d2, 88 },
    { 0, 0 },
    { 0x847e72d4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe38e72da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08e72dd, 85 },
    { 0x4dde72de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae72e3, 61 },
    { 0, 0 },
    { 0x340e72e5, 95 },
    { 0, 0 },
    { 0x7dae72e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e72ec, 66 },
    { 0, 0 },
    { 0xb8ce72ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68e72f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ae72fb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2be7302, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728e7308, 90 },
    { 0, 0 },
    { 0x1a1e730a, 33 },
    { 0xf58e730a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4e7310, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13e7314, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8e7317, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cee731a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3e731d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229e7321, 34 },
    { 0xb63e7321, 33 },
    { 0x595e7323, 90 },
    { 0xd6de7324, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258e7328, 46 },
    { 0, 0 },
    { 0x52be732a, 69 },
    { 0, 0 },
    { 0xdc6e732c, 95 },
    { 0x2b1e732c, 90 },
    { 0xae6e732d, 95 },
    { 0x128e732f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fe7335, 98 },
    { 0x854e7336, 33 },
    { 0, 0 },
    { 0x261e7338, 78 },
    { 0, 0 },
    { 0x1dee733a, 98 },
    { 0x9c1e733b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35be7341, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e7344, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x695e7347, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ee7350, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xede7353, 76 },
    { 0xdf6e7354, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894e735b, 23 },
    { 0xe5ee735c, 90 },
    { 0x7fce735d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825e7369, 98 },
    { 0, 0 },
    { 0xb1ee736b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5e736f, 61 },
    { 0x6fce736f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ee7377, 23 },
    { 0xb7ce7377, 76 },
    { 0xc0e7378, 90 },
    { 0, 0 },
    { 0x1d3e737b, 93 },
    { 0xf9fe737c, 61 },
    { 0x26de737d, 76 },
    { 0, 0 },
    { 0x3d8e737f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ae7382, 55 },
    { 0x674e7383, 48 },
    { 0, 0 },
    { 0x2cee7385, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e738a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0e738d, 61 },
    { 0x5dce738e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdfe7394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e739b, 23 },
    { 0xf77e739c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ce73a0, 99 },
    { 0xd50e73a1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ce73a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e73b0, 43 },
    { 0, 0 },
    { 0x7dae73b2, 55 },
    { 0xa3ee73b2, 42 },
    { 0x4c2e73b2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3de73bc, 33 },
    { 0x485e73bd, 61 },
    { 0xb95e73be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x682e73c1, 23 },
    { 0, 0 },
    { 0xf8be73c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43e73c7, 55 },
    { 0xc29e73c8, 94 },
    { 0x427e73c9, 78 },
    { 0x9d0e73ca, 95 },
    { 0, 0 },
    { 0xb7de73cc, 76 },
    { 0x564e73cd, 23 },
    { 0x872e73ce, 38 },
    { 0, 0 },
    { 0xa16e73d0, 61 },
    { 0xfbee73d1, 98 },
    { 0x9cae73d2, 34 },
    { 0x7b6e73d1, 90 },
    { 0x3e7e73d4, 33 },
    { 0xf05e73d4, 98 },
    { 0x231e73d4, 36 },
    { 0xbbbe73d6, 36 },
    { 0x65e73d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x90be73db, 98 },
    { 0x3b1e73dc, 95 },
    { 0x410e73dd, 59 },
    { 0x24ae73de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e73e3, 88 },
    { 0, 0 },
    { 0xb0ce73e5, 98 },
    { 0x2e6e73e6, 90 },
    { 0xeb5e73e7, 81 },
    { 0x7b4e73e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26e73ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06e73f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9e73f8, 85 },
    { 0xc9ae73f8, 98 },
    { 0x8de73fa, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ae73fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328e7408, 90 },
    { 0xdeae7409, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0e740d, 33 },
    { 0xe23e740e, 55 },
    { 0xb02e740f, 90 },
    { 0, 0 },
    { 0xfcfe7411, 36 },
    { 0x8ebe7411, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbde741d, 77 },
    { 0xb3de741d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e7422, 23 },
    { 0x18be7422, 43 },
    { 0x2d2e7422, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe7429, 90 },
    { 0, 0 },
    { 0x587e742b, 33 },
    { 0, 0 },
    { 0xbfee742d, 91 },
    { 0x987e742e, 98 },
    { 0x2e7e742e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fe7433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7be7436, 61 },
    { 0x63e7437, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7e743a, 95 },
    { 0xcfee743b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aee7441, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52e7444, 43 },
    { 0x1f5e7445, 23 },
    { 0x8bee7446, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14e744a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5e7450, 65 },
    { 0x45ae7451, 43 },
    { 0x94e7451, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bae7457, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94de745d, 33 },
    { 0, 0 },
    { 0x178e745f, 95 },
    { 0xe32e745f, 91 },
    { 0xbc7e7461, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ce7466, 67 },
    { 0, 0 },
    { 0x9a2e7468, 61 },
    { 0x45ce7468, 95 },
    { 0x99e7468, 77 },
    { 0x4d3e7469, 90 },
    { 0x78ee746c, 91 },
    { 0x73fe746d, 46 },
    { 0xf70e746d, 33 },
    { 0xfcfe746e, 76 },
    { 0xcdfe746f, 66 },
    { 0x99de746e, 66 },
    { 0x919e746e, 61 },
    { 0, 0 },
    { 0x38ae7474, 23 },
    { 0x368e7475, 90 },
    { 0xeb6e7476, 98 },
    { 0x2cce7476, 100 },
    { 0xa6be7477, 98 },
    { 0xa43e7478, 95 },
    { 0xb0ce747a, 91 },
    { 0, 0 },
    { 0x1a4e747c, 96 },
    { 0, 0 },
    { 0xe08e747e, 34 },
    { 0xf43e747f, 67 },
    { 0xe6ae7480, 24 },
    { 0, 0 },
    { 0xef5e7482, 61 },
    { 0x5cfe7483, 27 },
    { 0x7e7e7484, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8e7489, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81e748d, 90 },
    { 0xab1e748e, 95 },
    { 0x172e748f, 85 },
    { 0x887e7490, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48e7497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606e749b, 90 },
    { 0, 0 },
    { 0xacee749d, 94 },
    { 0xa3ee749d, 33 },
    { 0x4b4e749f, 91 },
    { 0x3ae74a0, 92 },
    { 0, 0 },
    { 0xf6e74a2, 93 },
    { 0xeaee74a3, 95 },
    { 0x1b9e74a3, 90 },
    { 0x901e74a5, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e74aa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213e74ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e74b1, 100 },
    { 0x2ebe74b2, 90 },
    { 0, 0 },
    { 0xef4e74b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e74b7, 33 },
    { 0, 0 },
    { 0x624e74b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32e74bc, 61 },
    { 0, 0 },
    { 0xa0e74be, 90 },
    { 0x179e74bf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce74c5, 95 },
    { 0x654e74c6, 23 },
    { 0, 0 },
    { 0x802e74c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916e74cf, 61 },
    { 0x925e74d0, 88 },
    { 0x5fbe74d1, 90 },
    { 0xfc3e74d0, 95 },
    { 0xb2de74d2, 90 },
    { 0xf6be74d3, 95 },
    { 0x8e8e74d5, 61 },
    { 0x74ae74d0, 98 },
    { 0x186e74d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ee74da, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4e74e0, 66 },
    { 0x30e74e0, 85 },
    { 0, 0 },
    { 0xcb9e74e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499e74e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ae74ed, 55 },
    { 0x9b5e74ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ce74f1, 95 },
    { 0, 0 },
    { 0x6a0e74f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x943e74f6, 95 },
    { 0x563e74f7, 46 },
    { 0x43ce74f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717e74fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ee7502, 67 },
    { 0x88ae7502, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7e7507, 90 },
    { 0, 0 },
    { 0xe67e7509, 95 },
    { 0xe85e7509, 90 },
    { 0x4a8e750b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3de750f, 46 },
    { 0x266e7510, 33 },
    { 0x1cce7510, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae9e7515, 98 },
    { 0x8e4e7516, 90 },
    { 0, 0 },
    { 0x3bee7518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5be7520, 93 },
    { 0, 0 },
    { 0x4a9e7522, 61 },
    { 0, 0 },
    { 0x668e7524, 76 },
    { 0x3dfe7525, 23 },
    { 0x9bbe7526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3e752a, 23 },
    { 0x90ce752b, 23 },
    { 0, 0 },
    { 0x860e752d, 78 },
    { 0xf9de752e, 95 },
    { 0xda6e752f, 61 },
    { 0xe5ce7530, 33 },
    { 0xb48e7530, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1e7539, 95 },
    { 0x250e753a, 34 },
    { 0x4dee753a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ce753e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe7541, 98 },
    { 0, 0 },
    { 0xa4be7543, 61 },
    { 0xbc1e7544, 32 },
    { 0, 0 },
    { 0x659e7546, 34 },
    { 0xddde7546, 33 },
    { 0xb15e7547, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e754e, 84 },
    { 0x345e754f, 84 },
    { 0, 0 },
    { 0xe6ce7551, 33 },
    { 0x55ce7552, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9e7558, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7e755b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08e7561, 61 },
    { 0x725e7561, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee756c, 43 },
    { 0, 0 },
    { 0xef9e756e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e7576, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e757a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98e757d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1e7582, 87 },
    { 0x284e7583, 87 },
    { 0xb69e7584, 52 },
    { 0xa85e7583, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938e758e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e7592, 23 },
    { 0x1f2e7592, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x232e7596, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320e759c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x370e759f, 100 },
    { 0, 0 },
    { 0x7f4e75a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e75a7, 61 },
    { 0, 0 },
    { 0x7efe75a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fe75b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148e75b4, 33 },
    { 0x598e75b5, 78 },
    { 0x78be75b6, 61 },
    { 0x801e75b6, 34 },
    { 0xda3e75b8, 95 },
    { 0xdf6e75b5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cde75c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3be75c3, 23 },
    { 0x70ae75c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6de75c8, 53 },
    { 0, 0 },
    { 0x455e75ca, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ee75ce, 87 },
    { 0, 0 },
    { 0x7d4e75d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae75d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6e75d7, 93 },
    { 0x830e75d8, 95 },
    { 0, 0 },
    { 0xd40e75da, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24e75de, 90 },
    { 0xaace75df, 23 },
    { 0x185e75de, 76 },
    { 0x505e75df, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81e75e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909e75ec, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ee75f2, 81 },
    { 0, 0 },
    { 0x1e5e75f4, 33 },
    { 0xd2de75f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7e75f8, 33 },
    { 0x589e75f9, 44 },
    { 0xb8ee75fa, 53 },
    { 0, 0 },
    { 0x4c4e75fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bce7600, 46 },
    { 0x18de7600, 61 },
    { 0xb10e7602, 33 },
    { 0x43be7602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbe7614, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ce7619, 23 },
    { 0, 0 },
    { 0x8a4e761b, 52 },
    { 0x98ce761c, 84 },
    { 0xca2e761c, 61 },
    { 0xf45e761c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979e7623, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0e7630, 55 },
    { 0, 0 },
    { 0xcece7632, 66 },
    { 0x9f8e7633, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x202e7636, 46 },
    { 0x6dde7637, 52 },
    { 0xdc9e7638, 98 },
    { 0xe7639, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ee763c, 61 },
    { 0xcfae763d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67e7641, 24 },
    { 0x2cde7642, 33 },
    { 0, 0 },
    { 0x352e7644, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ae7648, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87e764e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ede7652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ee7655, 23 },
    { 0x9bde7656, 34 },
    { 0x107e7655, 33 },
    { 0x58ce7658, 55 },
    { 0x9ae7658, 61 },
    { 0x6d3e7657, 76 },
    { 0xba8e765b, 95 },
    { 0xb77e765c, 90 },
    { 0, 0 },
    { 0xf83e765e, 98 },
    { 0xdde765e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416e7664, 90 },
    { 0xa4ce7664, 90 },
    { 0x39ae7665, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9e766a, 43 },
    { 0, 0 },
    { 0x5aae766c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccae7672, 33 },
    { 0xe50e7673, 61 },
    { 0xb67e7673, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04e7677, 33 },
    { 0xd64e7677, 95 },
    { 0x784e7679, 76 },
    { 0xd0fe767a, 61 },
    { 0xa31e767a, 11 },
    { 0x7aee767c, 66 },
    { 0x475e767d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2de7682, 61 },
    { 0, 0 },
    { 0xd65e7684, 33 },
    { 0x5ce7685, 65 },
    { 0xa35e7684, 87 },
    { 0x792e7687, 33 },
    { 0, 0 },
    { 0x1abe7689, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8e768e, 100 },
    { 0xa62e768e, 91 },
    { 0, 0 },
    { 0xd40e7691, 95 },
    { 0, 0 },
    { 0x19fe7693, 76 },
    { 0, 0 },
    { 0x1abe7695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eee769c, 76 },
    { 0x580e769c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6e76a2, 52 },
    { 0x56de76a3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530e76aa, 61 },
    { 0xf79e76aa, 76 },
    { 0xb9ae76aa, 33 },
    { 0, 0 },
    { 0xea5e76ae, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e76b4, 76 },
    { 0x8ffe76b5, 95 },
    { 0x373e76b6, 55 },
    { 0x922e76b6, 84 },
    { 0xd3ae76b5, 33 },
    { 0xa99e76b6, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ae76c7, 98 },
    { 0, 0 },
    { 0x66fe76c9, 33 },
    { 0, 0 },
    { 0x460e76cb, 83 },
    { 0xeefe76cc, 93 },
    { 0x813e76cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1de76d0, 23 },
    { 0x258e76d1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x738e76d4, 17 },
    { 0x552e76d4, 23 },
    { 0x321e76d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8e76db, 61 },
    { 0xa60e76dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5e76e2, 61 },
    { 0, 0 },
    { 0x7dce76e4, 61 },
    { 0xd60e76e5, 33 },
    { 0, 0 },
    { 0x369e76e7, 47 },
    { 0, 0 },
    { 0x4c2e76e9, 95 },
    { 0, 0 },
    { 0xee9e76eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282e76f0, 88 },
    { 0x762e76f1, 90 },
    { 0x8bbe76f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94de76f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6e76fe, 95 },
    { 0, 0 },
    { 0x1a4e7700, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e7703, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be7706, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1e7709, 61 },
    { 0xa13e770a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe770d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76e7713, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc42e7723, 77 },
    { 0x412e7723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44e7728, 61 },
    { 0, 0 },
    { 0xc1ee772a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4e772f, 17 },
    { 0x31e772f, 94 },
    { 0xd83e7731, 23 },
    { 0x7cce772f, 78 },
    { 0xd53e7730, 87 },
    { 0, 0 },
    { 0x9d4e7735, 78 },
    { 0x54ee7736, 61 },
    { 0x2c1e7737, 100 },
    { 0, 0 },
    { 0x95ee7739, 61 },
    { 0, 0 },
    { 0xb7ce773b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb6e773e, 98 },
    { 0xf50e773e, 46 },
    { 0xc0ae7740, 98 },
    { 0xee7e7740, 33 },
    { 0xac4e773f, 61 },
    { 0xc74e7740, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e7746, 61 },
    { 0x664e7747, 61 },
    { 0x21ae7748, 90 },
    { 0x97e7749, 76 },
    { 0xfd4e7749, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7e774d, 85 },
    { 0xa3be774d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6e775a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ee775d, 17 },
    { 0xa31e775d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245e7764, 40 },
    { 0x627e7765, 90 },
    { 0x4c4e7766, 69 },
    { 0xebe7765, 93 },
    { 0x49fe7764, 33 },
    { 0x7aae7769, 98 },
    { 0x423e7765, 98 },
    { 0, 0 },
    { 0x8d0e776c, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1de7770, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314e7775, 98 },
    { 0xa1be7776, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312e777b, 87 },
    { 0x887e777c, 33 },
    { 0xc91e777d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853e7781, 33 },
    { 0, 0 },
    { 0x7c3e7783, 93 },
    { 0x5e5e7784, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8e7787, 36 },
    { 0x762e7788, 55 },
    { 0xbb5e7789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1e778d, 23 },
    { 0, 0 },
    { 0xeb2e778f, 33 },
    { 0xf52e7790, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971e7797, 43 },
    { 0x113e7797, 87 },
    { 0xffde7799, 61 },
    { 0, 0 },
    { 0xa15e779b, 87 },
    { 0x24fe779b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ce77a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bae77a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22e77a9, 23 },
    { 0x53ee77aa, 61 },
    { 0xc2e77aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722e77af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x74de77b2, 91 },
    { 0xc2de77b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24e77b8, 23 },
    { 0x9c5e77b8, 84 },
    { 0, 0 },
    { 0xaa1e77bb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3e77bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6e77c6, 33 },
    { 0xb62e77c6, 76 },
    { 0, 0 },
    { 0x993e77c9, 33 },
    { 0x457e77ca, 87 },
    { 0xa32e77cb, 61 },
    { 0x8c7e77ca, 90 },
    { 0, 0 },
    { 0x5efe77ce, 98 },
    { 0x5b5e77cf, 98 },
    { 0xdd7e77d0, 46 },
    { 0, 0 },
    { 0xf69e77d2, 95 },
    { 0, 0 },
    { 0x309e77d4, 88 },
    { 0xa1e77d5, 78 },
    { 0, 0 },
    { 0x226e77d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5e77da, 98 },
    { 0, 0 },
    { 0xa9ce77dc, 95 },
    { 0, 0 },
    { 0x67ce77de, 90 },
    { 0xce4e77df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1e77e2, 19 },
    { 0, 0 },
    { 0xe2e77e4, 55 },
    { 0x99ae77e5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4e77ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ce77f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42de77fb, 33 },
    { 0xecfe77fc, 91 },
    { 0x6d5e77fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2e7804, 93 },
    { 0xc03e7805, 81 },
    { 0xeb9e7806, 93 },
    { 0x296e7807, 23 },
    { 0xda5e7808, 81 },
    { 0x757e7804, 99 },
    { 0xc8ce7808, 23 },
    { 0, 0 },
    { 0xb6ee780c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04e7814, 36 },
    { 0x792e7815, 33 },
    { 0xc11e7815, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98e781d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884e7822, 76 },
    { 0x7fae7823, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfe7829, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8efe782c, 23 },
    { 0, 0 },
    { 0xa00e782e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2e7838, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2e7841, 92 },
    { 0x20e7841, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54e7847, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29e784f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3de7856, 98 },
    { 0x966e7857, 93 },
    { 0xfdde7858, 95 },
    { 0xed2e7858, 33 },
    { 0xadce7858, 33 },
    { 0x10ae785b, 66 },
    { 0xc17e7859, 66 },
    { 0x603e785d, 38 },
    { 0x39ee785e, 88 },
    { 0x41be785e, 33 },
    { 0x86fe7860, 61 },
    { 0x6a0e7861, 93 },
    { 0x313e7858, 34 },
    { 0x7fce7863, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6e7866, 90 },
    { 0xfd6e7866, 36 },
    { 0xef1e7868, 87 },
    { 0xd1e7869, 76 },
    { 0x30ce7868, 61 },
    { 0xf20e786b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cfe786e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2e7878, 55 },
    { 0xd52e7879, 75 },
    { 0x19fe787a, 99 },
    { 0x1bce787b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1e787f, 78 },
    { 0x812e7880, 34 },
    { 0xbe7881, 33 },
    { 0xdc7e7882, 33 },
    { 0x91ee7883, 95 },
    { 0xf53e7884, 23 },
    { 0x6f9e7885, 93 },
    { 0x2b0e7886, 100 },
    { 0x2a0e7887, 95 },
    { 0x869e7883, 23 },
    { 0x2fae7880, 95 },
    { 0xfde7883, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x512e788e, 95 },
    { 0x548e788f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27e7894, 90 },
    { 0, 0 },
    { 0x971e7896, 76 },
    { 0x61ce7897, 87 },
    { 0, 0 },
    { 0x8cbe7899, 43 },
    { 0xdabe789a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17e78a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5e78a9, 33 },
    { 0x62ce78aa, 23 },
    { 0x240e78ab, 48 },
    { 0xf84e78ac, 90 },
    { 0, 0 },
    { 0x6f6e78ae, 95 },
    { 0, 0 },
    { 0x15ae78b0, 23 },
    { 0xac9e78b1, 98 },
    { 0, 0 },
    { 0x99ce78b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96e78b7, 67 },
    { 0, 0 },
    { 0x81ae78b9, 90 },
    { 0, 0 },
    { 0xab9e78bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfce78c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98e78c3, 43 },
    { 0x334e78c4, 33 },
    { 0x65ae78c5, 90 },
    { 0x875e78c3, 33 },
    { 0, 0 },
    { 0x9ede78c8, 46 },
    { 0x2efe78c8, 61 },
    { 0x699e78c9, 34 },
    { 0x2d8e78c9, 99 },
    { 0xab6e78ca, 76 },
    { 0xf88e78cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e78d2, 95 },
    { 0, 0 },
    { 0xdf3e78d4, 33 },
    { 0, 0 },
    { 0xdffe78d6, 46 },
    { 0x399e78d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949e78df, 87 },
    { 0, 0 },
    { 0x541e78e1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e78e4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe78ef, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69e78f3, 46 },
    { 0xf6e78f4, 99 },
    { 0x4b9e78f5, 43 },
    { 0xfbee78f5, 34 },
    { 0xa71e78f7, 33 },
    { 0x26ce78f6, 61 },
    { 0, 0 },
    { 0x8fbe78fa, 33 },
    { 0xdc9e78fa, 61 },
    { 0x903e78fc, 95 },
    { 0x603e78fd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e7902, 76 },
    { 0x323e7903, 65 },
    { 0x729e7902, 100 },
    { 0xf49e7905, 23 },
    { 0, 0 },
    { 0x2a8e7907, 95 },
    { 0, 0 },
    { 0xc54e7909, 90 },
    { 0x2a9e790a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9be790d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2e7912, 33 },
    { 0x7b5e7913, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49e7917, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4e791c, 61 },
    { 0x413e791d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88e7920, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6de7926, 43 },
    { 0xb3fe7927, 33 },
    { 0xabae7927, 34 },
    { 0x8c2e7929, 33 },
    { 0x652e792a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819e7931, 84 },
    { 0, 0 },
    { 0x76ee7933, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cae7936, 76 },
    { 0xf0be7937, 61 },
    { 0x4b0e7937, 68 },
    { 0x6e5e7937, 66 },
    { 0x44fe7938, 80 },
    { 0x42ce7936, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8e7942, 76 },
    { 0xdfde7942, 43 },
    { 0, 0 },
    { 0xd19e7945, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bfe7948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87e794d, 99 },
    { 0xe69e794e, 66 },
    { 0x89ee794f, 55 },
    { 0xbcde794e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ae7953, 66 },
    { 0x584e7953, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84e7957, 98 },
    { 0, 0 },
    { 0x9e9e7959, 95 },
    { 0, 0 },
    { 0x96de795b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5e795e, 61 },
    { 0x4dbe795f, 98 },
    { 0, 0 },
    { 0xa73e7961, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37be7967, 76 },
    { 0xf27e7968, 33 },
    { 0x131e7967, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe7974, 17 },
    { 0, 0 },
    { 0xa64e7976, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670e797a, 34 },
    { 0, 0 },
    { 0x830e797c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedae797f, 55 },
    { 0xdace797f, 95 },
    { 0x76ce797f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8e7985, 77 },
    { 0, 0 },
    { 0xa6fe7987, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97e798f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e7992, 90 },
    { 0x978e7993, 76 },
    { 0x8efe7992, 90 },
    { 0x2d8e7995, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ae799c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbe79a2, 78 },
    { 0x94ee79a2, 55 },
    { 0x460e79a3, 23 },
    { 0x798e79a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2e79a9, 23 },
    { 0x29be79aa, 84 },
    { 0, 0 },
    { 0xcd2e79ac, 61 },
    { 0xb1ce79ad, 61 },
    { 0x5c9e79ae, 33 },
    { 0xb13e79af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cde79b3, 55 },
    { 0, 0 },
    { 0xfe0e79b5, 23 },
    { 0, 0 },
    { 0x1ce79b7, 95 },
    { 0xf36e79b7, 91 },
    { 0, 0 },
    { 0xda0e79ba, 55 },
    { 0, 0 },
    { 0xb01e79bc, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ee79bf, 95 },
    { 0, 0 },
    { 0x14e79c1, 91 },
    { 0x4bee79c2, 90 },
    { 0x182e79c2, 83 },
    { 0, 0 },
    { 0xce9e79c5, 95 },
    { 0x821e79c6, 90 },
    { 0x6cee79c7, 100 },
    { 0xd4be79c8, 40 },
    { 0xc8ee79c8, 87 },
    { 0xe9de79ca, 98 },
    { 0xe40e79cb, 98 },
    { 0x7a7e79cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1e79d4, 77 },
    { 0x9e9e79d5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ee79da, 38 },
    { 0xe49e79db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4e79e1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852e79e7, 95 },
    { 0, 0 },
    { 0xb59e79e9, 95 },
    { 0x5ade79e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ce79f1, 81 },
    { 0x867e79f2, 95 },
    { 0x7c8e79f3, 95 },
    { 0xa83e79f4, 84 },
    { 0xb45e79f4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48e79f8, 23 },
    { 0, 0 },
    { 0xc33e79fa, 33 },
    { 0xac6e79fb, 98 },
    { 0, 0 },
    { 0x67ae79fd, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x974e7a00, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x899e7a03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54e7a07, 90 },
    { 0x6a3e7a08, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4e7a0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x728e7a0f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15e7a14, 95 },
    { 0x75ae7a15, 95 },
    { 0xa37e7a16, 95 },
    { 0xb45e7a15, 76 },
    { 0x2a4e7a18, 66 },
    { 0x3b8e7a19, 46 },
    { 0x57ae7a18, 75 },
    { 0, 0 },
    { 0x27ee7a1c, 90 },
    { 0x1fbe7a1c, 61 },
    { 0x3ae7a1e, 90 },
    { 0, 0 },
    { 0x401e7a20, 94 },
    { 0xcf6e7a20, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7e7a24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510e7a29, 23 },
    { 0, 0 },
    { 0x62ce7a2b, 86 },
    { 0, 0 },
    { 0xf18e7a2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x64de7a30, 55 },
    { 0x65de7a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20de7a34, 33 },
    { 0xe64e7a34, 77 },
    { 0, 0 },
    { 0xda8e7a37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745e7a3c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31e7a41, 95 },
    { 0x270e7a42, 90 },
    { 0xe2fe7a43, 39 },
    { 0x366e7a43, 61 },
    { 0x71ee7a44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965e7a4c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7e7a57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20e7a5b, 84 },
    { 0, 0 },
    { 0x75be7a5d, 100 },
    { 0x510e7a5e, 95 },
    { 0x9fe7a5f, 98 },
    { 0xe86e7a5d, 33 },
    { 0xca3e7a61, 85 },
    { 0x764e7a62, 61 },
    { 0xeb3e7a63, 90 },
    { 0xbcce7a64, 33 },
    { 0x6e1e7a61, 93 },
    { 0xc69e7a66, 76 },
    { 0x121e7a66, 17 },
    { 0xa69e7a65, 69 },
    { 0xfbde7a67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0e7a6d, 76 },
    { 0, 0 },
    { 0xec8e7a6f, 91 },
    { 0xa78e7a6f, 90 },
    { 0, 0 },
    { 0x551e7a72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4e7a75, 87 },
    { 0, 0 },
    { 0xcd7e7a77, 90 },
    { 0x161e7a78, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199e7a7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ae7a87, 46 },
    { 0xe7be7a87, 61 },
    { 0x5eee7a89, 100 },
    { 0x1ce7a8a, 55 },
    { 0x6a1e7a8b, 95 },
    { 0xbe5e7a8c, 76 },
    { 0, 0 },
    { 0x8b1e7a8e, 85 },
    { 0x40ce7a8f, 33 },
    { 0x6cce7a8e, 46 },
    { 0x4e7e7a8f, 66 },
    { 0, 0 },
    { 0x5a5e7a93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fe7a99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e7a9d, 52 },
    { 0xeb7e7a9e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ae7aa1, 91 },
    { 0, 0 },
    { 0xaae7aa3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ee7aaa, 61 },
    { 0x803e7aaa, 66 },
    { 0x616e7aac, 61 },
    { 0x443e7aac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467e7ab2, 55 },
    { 0, 0 },
    { 0x17de7ab4, 99 },
    { 0, 0 },
    { 0xbebe7ab6, 90 },
    { 0x930e7ab7, 61 },
    { 0xef5e7ab8, 33 },
    { 0, 0 },
    { 0xc2ae7aba, 33 },
    { 0x30de7abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ece7ac3, 17 },
    { 0, 0 },
    { 0xe18e7ac5, 33 },
    { 0xf84e7ac6, 81 },
    { 0x987e7ac6, 77 },
    { 0x5ffe7ac6, 95 },
    { 0, 0 },
    { 0x146e7aca, 98 },
    { 0, 0 },
    { 0xb9fe7acc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bce7ad0, 23 },
    { 0xe23e7ad0, 76 },
    { 0xd2be7ad0, 23 },
    { 0xf13e7ad2, 17 },
    { 0x6cde7ad4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be7ad7, 50 },
    { 0x68de7ad8, 61 },
    { 0x48ee7ad9, 43 },
    { 0x6f2e7ad8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1be7adf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaae7ae2, 98 },
    { 0x2cce7ae3, 99 },
    { 0x597e7ae4, 61 },
    { 0x224e7ae5, 43 },
    { 0xc9be7ae6, 61 },
    { 0xf06e7ae6, 23 },
    { 0, 0 },
    { 0x16ce7ae9, 98 },
    { 0xfaee7ae9, 34 },
    { 0xf25e7aeb, 33 },
    { 0x38be7ae9, 76 },
    { 0xc58e7aed, 61 },
    { 0xc74e7aed, 98 },
    { 0xf7fe7aeb, 98 },
    { 0x701e7af0, 43 },
    { 0xd4ae7af1, 52 },
    { 0x4c0e7af2, 34 },
    { 0, 0 },
    { 0xb20e7af4, 17 },
    { 0x978e7af5, 33 },
    { 0x5c4e7af6, 90 },
    { 0xef5e7af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ee7afd, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fe7b0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4be7b0d, 49 },
    { 0, 0 },
    { 0xb51e7b0f, 98 },
    { 0x3d9e7b10, 88 },
    { 0x8d3e7b11, 66 },
    { 0x1b3e7b12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae7b17, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ee7b1c, 23 },
    { 0x273e7b1c, 90 },
    { 0xca2e7b1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769e7b23, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e7b29, 95 },
    { 0x349e7b2a, 23 },
    { 0xb39e7b29, 52 },
    { 0x416e7b2c, 88 },
    { 0, 0 },
    { 0x875e7b2e, 61 },
    { 0xe9be7b2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3e7b32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aee7b37, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be7b3a, 23 },
    { 0, 0 },
    { 0x6b0e7b3c, 95 },
    { 0, 0 },
    { 0x301e7b3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709e7b44, 76 },
    { 0xd2ee7b45, 23 },
    { 0, 0 },
    { 0xdf0e7b47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9e7b4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x844e7b4e, 95 },
    { 0, 0 },
    { 0xd27e7b50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x447e7b56, 76 },
    { 0xe0ae7b57, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7e7b5a, 78 },
    { 0xd48e7b5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e7b69, 61 },
    { 0x35fe7b6a, 93 },
    { 0x1f7e7b6b, 90 },
    { 0x33de7b6c, 43 },
    { 0, 0 },
    { 0xa60e7b6e, 66 },
    { 0xf82e7b6f, 90 },
    { 0xd77e7b6e, 43 },
    { 0x54be7b71, 61 },
    { 0x915e7b71, 55 },
    { 0, 0 },
    { 0xf0be7b74, 68 },
    { 0x88ce7b75, 76 },
    { 0x291e7b74, 33 },
    { 0x8ace7b77, 84 },
    { 0, 0 },
    { 0x8e8e7b79, 33 },
    { 0, 0 },
    { 0x2dee7b7b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13e7b80, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e7b83, 69 },
    { 0x8b6e7b84, 90 },
    { 0, 0 },
    { 0x828e7b86, 90 },
    { 0xfe7b86, 90 },
    { 0, 0 },
    { 0x174e7b89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x513e7b8c, 85 },
    { 0xba3e7b8c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35e7b91, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52e7b97, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ce7b9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfae7b9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e7ba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5e7ba9, 61 },
    { 0x25e7baa, 90 },
    { 0x2fbe7bab, 95 },
    { 0x108e7bac, 75 },
    { 0x475e7bac, 61 },
    { 0x61de7bad, 87 },
    { 0xc23e7baf, 76 },
    { 0, 0 },
    { 0xd88e7bb1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x329e7bb4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e7bb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37de7bbd, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcade7bc1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815e7bc5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde7bd1, 90 },
    { 0, 0 },
    { 0x275e7bd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87e7bd6, 90 },
    { 0, 0 },
    { 0xd46e7bd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae7bdb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12e7bde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffe7be3, 90 },
    { 0x69ae7be3, 33 },
    { 0x8ebe7be3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895e7be9, 98 },
    { 0, 0 },
    { 0xf6ce7beb, 93 },
    { 0, 0 },
    { 0x661e7bed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10de7bf3, 33 },
    { 0xc1ae7bf4, 33 },
    { 0xdd4e7bf3, 91 },
    { 0xe7fe7bf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ce7bfa, 90 },
    { 0x5eee7bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1e7bff, 23 },
    { 0xc98e7c00, 76 },
    { 0xd2ee7c00, 90 },
    { 0, 0 },
    { 0x88ee7c03, 33 },
    { 0x73fe7c03, 33 },
    { 0xc1ce7c05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e7c09, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee7c0c, 90 },
    { 0x7f6e7c0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2e7c11, 95 },
    { 0x65ce7c12, 19 },
    { 0, 0 },
    { 0xbde7c14, 55 },
    { 0xbb4e7c15, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ee7c1c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161e7c21, 76 },
    { 0x950e7c22, 90 },
    { 0x50fe7c22, 88 },
    { 0x399e7c24, 39 },
    { 0xacbe7c25, 43 },
    { 0x6a8e7c25, 43 },
    { 0x3b9e7c21, 69 },
    { 0xa47e7c28, 61 },
    { 0xc52e7c29, 55 },
    { 0x357e7c2a, 78 },
    { 0xe8fe7c27, 33 },
    { 0xaf2e7c2c, 98 },
    { 0x35e7c2d, 77 },
    { 0x565e7c2e, 94 },
    { 0x2d7e7c2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e7c32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47e7c35, 34 },
    { 0x81be7c36, 100 },
    { 0x766e7c37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27e7c3d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ee7c41, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9e7c49, 98 },
    { 0xd54e7c49, 55 },
    { 0x79de7c49, 95 },
    { 0x5c5e7c4b, 76 },
    { 0x893e7c4d, 91 },
    { 0, 0 },
    { 0x8d9e7c4f, 61 },
    { 0, 0 },
    { 0xaa5e7c51, 90 },
    { 0x693e7c52, 61 },
    { 0x6c2e7c53, 95 },
    { 0x97fe7c52, 93 },
    { 0x295e7c55, 61 },
    { 0, 0 },
    { 0xf85e7c57, 101 },
    { 0x664e7c57, 23 },
    { 0x6ee7c58, 33 },
    { 0xcb0e7c5a, 98 },
    { 0x7f2e7c57, 46 },
    { 0xbcde7c5c, 97 },
    { 0x13be7c5d, 90 },
    { 0x77e7c58, 90 },
    { 0x92be7c5f, 43 },
    { 0x7d0e7c60, 43 },
    { 0x799e7c59, 85 },
    { 0x3cfe7c62, 23 },
    { 0x939e7c63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbe7c66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffe7c6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fe7c71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe7c74, 56 },
    { 0x439e7c74, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660e7c7a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdae7c7e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0fe7c82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbe7c87, 90 },
    { 0x7b4e7c88, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ee7c8d, 98 },
    { 0x69ee7c8e, 43 },
    { 0x2abe7c8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08e7c92, 33 },
    { 0, 0 },
    { 0x67ee7c94, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x385e7c97, 33 },
    { 0, 0 },
    { 0x287e7c99, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7e7c9d, 40 },
    { 0x35e7c9e, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653e7ca5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfae7ca9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24e7cac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ae7cb2, 65 },
    { 0x217e7cb3, 90 },
    { 0, 0 },
    { 0xf9ee7cb5, 33 },
    { 0x6c3e7cb6, 80 },
    { 0xd6fe7cb7, 23 },
    { 0x20ee7cb5, 99 },
    { 0, 0 },
    { 0x9ae7cba, 95 },
    { 0xb8ce7cbb, 34 },
    { 0xfd6e7cbb, 93 },
    { 0x8fe7cbb, 90 },
    { 0x64de7cbe, 33 },
    { 0xdafe7cbf, 90 },
    { 0x7c3e7cc0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x809e7cc4, 90 },
    { 0x113e7cc4, 10 },
    { 0x9b7e7cc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebe7cc9, 95 },
    { 0, 0 },
    { 0x82ae7ccb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914e7cd5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3de7cd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe7cdf, 33 },
    { 0x506e7ce0, 98 },
    { 0xa82e7cdf, 23 },
    { 0x451e7ce2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ce7ce7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdade7ceb, 23 },
    { 0x101e7cec, 91 },
    { 0x4d2e7cec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x217e7cf0, 33 },
    { 0xcb5e7cf1, 98 },
    { 0x4e7cf1, 98 },
    { 0, 0 },
    { 0x4dae7cf4, 68 },
    { 0xdc8e7cf4, 81 },
    { 0, 0 },
    { 0xdd0e7cf7, 62 },
    { 0x482e7cf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdffe7cfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ce7d00, 80 },
    { 0x697e7d00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08e7d04, 55 },
    { 0x3ee7d05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x31e7d08, 84 },
    { 0xd54e7d09, 34 },
    { 0, 0 },
    { 0xc66e7d0b, 23 },
    { 0x422e7d0b, 43 },
    { 0x249e7d0b, 95 },
    { 0x278e7d0e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e7d11, 91 },
    { 0, 0 },
    { 0xe1ce7d13, 90 },
    { 0, 0 },
    { 0x3e9e7d15, 23 },
    { 0x5fde7d16, 61 },
    { 0xdbbe7d17, 43 },
    { 0xd63e7d16, 90 },
    { 0, 0 },
    { 0x2ffe7d1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84e7d1d, 76 },
    { 0x2a1e7d1e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71e7d22, 23 },
    { 0xe98e7d23, 95 },
    { 0x348e7d24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340e7d29, 75 },
    { 0x463e7d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3e7d2d, 98 },
    { 0x66be7d2e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x925e7d31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1e7d39, 33 },
    { 0x68de7d39, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98e7d3e, 33 },
    { 0, 0 },
    { 0xae5e7d40, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ee7d43, 95 },
    { 0x430e7d43, 33 },
    { 0xaa3e7d45, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bde7d48, 81 },
    { 0xd5ee7d48, 96 },
    { 0x5abe7d49, 33 },
    { 0, 0 },
    { 0xeb8e7d4c, 98 },
    { 0, 0 },
    { 0x7cfe7d4e, 93 },
    { 0xdd6e7d4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584e7d54, 90 },
    { 0x956e7d54, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e7d5b, 23 },
    { 0x686e7d5c, 55 },
    { 0, 0 },
    { 0xcaae7d5e, 76 },
    { 0xd8ee7d5f, 23 },
    { 0, 0 },
    { 0x63ee7d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3e7d64, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e7d6a, 90 },
    { 0xfc5e7d6b, 61 },
    { 0xe2de7d6c, 61 },
    { 0x4ee7d6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594e7d71, 65 },
    { 0xdabe7d72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00e7d76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x837e7d79, 34 },
    { 0x61ae7d79, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4e7d82, 95 },
    { 0xd1fe7d83, 33 },
    { 0, 0 },
    { 0xa5de7d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0e7d8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dee7d90, 95 },
    { 0x6a5e7d90, 61 },
    { 0xecde7d92, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ee7d95, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbe7d9b, 23 },
    { 0, 0 },
    { 0x45be7d9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36e7da1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ae7da5, 90 },
    { 0xe8ee7da5, 90 },
    { 0x805e7da7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ae7dab, 90 },
    { 0, 0 },
    { 0x17de7dad, 90 },
    { 0x111e7dae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62be7db4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ae7db7, 46 },
    { 0x58be7db8, 90 },
    { 0xde4e7db8, 67 },
    { 0x85e7dba, 46 },
    { 0x2eae7db8, 90 },
    { 0xc4be7dbc, 100 },
    { 0x669e7db9, 17 },
    { 0x8bae7db9, 76 },
    { 0x8b2e7dbf, 34 },
    { 0, 0 },
    { 0x2ae7dc1, 90 },
    { 0x3d2e7dc2, 33 },
    { 0, 0 },
    { 0x59ae7dc4, 55 },
    { 0, 0 },
    { 0xe9ee7dc6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3e7dcd, 67 },
    { 0x4e4e7dce, 98 },
    { 0xe94e7dcf, 43 },
    { 0x3cbe7dcf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cce7dd3, 23 },
    { 0, 0 },
    { 0x781e7dd5, 46 },
    { 0xd1ae7dd6, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354e7dda, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e7ddd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9e7de7, 61 },
    { 0x82be7de7, 95 },
    { 0x94e7de8, 36 },
    { 0, 0 },
    { 0x99ce7deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e7def, 95 },
    { 0, 0 },
    { 0x9f2e7df1, 93 },
    { 0xf5de7df1, 95 },
    { 0xcc5e7df2, 61 },
    { 0xeb4e7df1, 43 },
    { 0, 0 },
    { 0x92ce7df6, 90 },
    { 0x4c1e7df7, 33 },
    { 0x178e7df6, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780e7dff, 98 },
    { 0x74ae7e00, 93 },
    { 0x9fce7e01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5e7e08, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8e7e0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bee7e12, 23 },
    { 0, 0 },
    { 0xcdae7e14, 75 },
    { 0x80e7e15, 95 },
    { 0, 0 },
    { 0x50fe7e17, 90 },
    { 0, 0 },
    { 0xfbde7e19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68e7e1d, 23 },
    { 0x1cfe7e1d, 98 },
    { 0, 0 },
    { 0x11ee7e20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56e7e29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe7e31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e7e35, 90 },
    { 0x776e7e35, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e7e3d, 66 },
    { 0, 0 },
    { 0xc3ee7e3f, 23 },
    { 0, 0 },
    { 0xd91e7e41, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xafee7e44, 33 },
    { 0, 0 },
    { 0x504e7e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11e7e4b, 33 },
    { 0x8bfe7e4c, 33 },
    { 0, 0 },
    { 0xdabe7e4e, 90 },
    { 0, 0 },
    { 0x9a8e7e50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460e7e55, 98 },
    { 0x3ece7e55, 33 },
    { 0x975e7e55, 61 },
    { 0xc8de7e58, 98 },
    { 0xab5e7e58, 76 },
    { 0x1d4e7e5a, 23 },
    { 0x674e7e56, 33 },
    { 0x57de7e5b, 98 },
    { 0x113e7e5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5e7e62, 33 },
    { 0x63ae7e63, 33 },
    { 0x789e7e64, 96 },
    { 0x9bbe7e65, 34 },
    { 0xb94e7e66, 95 },
    { 0x902e7e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfe7e6a, 67 },
    { 0, 0 },
    { 0xfafe7e6c, 23 },
    { 0x8cae7e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcee7e71, 61 },
    { 0x56be7e72, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3e7e77, 52 },
    { 0x6e3e7e77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3e7e7d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0e7e81, 90 },
    { 0x5a2e7e82, 46 },
    { 0x18de7e83, 68 },
    { 0xd6e7e82, 90 },
    { 0xb9fe7e82, 95 },
    { 0x64ae7e86, 35 },
    { 0x1d8e7e87, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be7e8a, 93 },
    { 0xdc7e7e8b, 90 },
    { 0x12be7e8c, 65 },
    { 0x9e7e7e8d, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ce7e92, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71e7e99, 33 },
    { 0x5fee7e9a, 95 },
    { 0x6a4e7e9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e7ea3, 90 },
    { 0xc89e7ea4, 76 },
    { 0, 0 },
    { 0xb3ae7ea6, 2 },
    { 0x3c5e7ea6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2e7eab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x764e7eae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593e7eb2, 76 },
    { 0x6de7eb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x138e7eb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130e7ebe, 23 },
    { 0, 0 },
    { 0x776e7ec0, 95 },
    { 0x1eae7ec1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4e7ec4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0e7ec7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612e7ece, 61 },
    { 0, 0 },
    { 0x754e7ed0, 23 },
    { 0x982e7ed0, 43 },
    { 0x744e7ed1, 95 },
    { 0xed5e7ed3, 77 },
    { 0x3dee7ed4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55be7ed7, 40 },
    { 0x6b7e7ed8, 90 },
    { 0, 0 },
    { 0xa2e7eda, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbe7ee0, 76 },
    { 0xbaae7ee0, 90 },
    { 0, 0 },
    { 0xe18e7ee3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afe7eea, 23 },
    { 0x1e8e7eea, 75 },
    { 0x31ce7eec, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfe7efe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e7f03, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebe7f07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9e7f0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee7f13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fe7f17, 67 },
    { 0x179e7f18, 33 },
    { 0xde7f18, 99 },
    { 0, 0 },
    { 0xa2ae7f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x347e7f1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00e7f21, 43 },
    { 0xfd1e7f21, 61 },
    { 0, 0 },
    { 0x1ae7f24, 90 },
    { 0x910e7f24, 43 },
    { 0x580e7f26, 52 },
    { 0, 0 },
    { 0x928e7f28, 98 },
    { 0x353e7f28, 90 },
    { 0x7d2e7f28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abe7f30, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16be7f39, 61 },
    { 0xb94e7f3a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410e7f43, 43 },
    { 0xb37e7f44, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88e7f48, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984e7f53, 93 },
    { 0xdc2e7f53, 61 },
    { 0xbfce7f55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ce7f58, 77 },
    { 0x7c2e7f59, 67 },
    { 0xff9e7f5a, 93 },
    { 0, 0 },
    { 0x554e7f5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99e7f62, 55 },
    { 0x776e7f63, 83 },
    { 0x396e7f63, 34 },
    { 0xcf3e7f63, 90 },
    { 0x24ee7f66, 33 },
    { 0x90de7f66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69de7f73, 61 },
    { 0xc62e7f73, 61 },
    { 0, 0 },
    { 0x265e7f76, 90 },
    { 0x509e7f77, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x260e7f7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2e7f7e, 33 },
    { 0x7bae7f7f, 52 },
    { 0x739e7f80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eee7f83, 23 },
    { 0x117e7f84, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161e7f90, 61 },
    { 0, 0 },
    { 0x9ae7f92, 98 },
    { 0xb6fe7f92, 61 },
    { 0x54ee7f94, 95 },
    { 0xe4ee7f94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7e7f99, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e7f9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x13e7fa0, 61 },
    { 0xeebe7fa1, 98 },
    { 0, 0 },
    { 0x508e7fa3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2be7fa7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x665e7fad, 35 },
    { 0x453e7fae, 43 },
    { 0xf91e7faf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1e7fb2, 85 },
    { 0xe4e7fb3, 65 },
    { 0x47ae7fb4, 95 },
    { 0x32ae7fb4, 98 },
    { 0, 0 },
    { 0xee0e7fb7, 23 },
    { 0x12ce7fb8, 33 },
    { 0xbafe7fb9, 61 },
    { 0, 0 },
    { 0xe0e7fbb, 17 },
    { 0xb95e7fbc, 91 },
    { 0x532e7fbc, 61 },
    { 0x66fe7fbe, 66 },
    { 0, 0 },
    { 0xcc2e7fc0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9e7fcb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3e7fd0, 68 },
    { 0x6e5e7fd1, 90 },
    { 0, 0 },
    { 0x468e7fd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e7fd7, 33 },
    { 0, 0 },
    { 0x2e3e7fd9, 95 },
    { 0x8e2e7fda, 66 },
    { 0xdb3e7fdb, 93 },
    { 0, 0 },
    { 0x446e7fdd, 43 },
    { 0, 0 },
    { 0x7c4e7fdf, 34 },
    { 0, 0 },
    { 0x176e7fe1, 80 },
    { 0x850e7fe2, 90 },
    { 0x7f2e7fe1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fe7fe8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e7fec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bee7ff0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4de7ff5, 95 },
    { 0x6e4e7ff5, 23 },
    { 0xcbae7ff7, 33 },
    { 0, 0 },
    { 0x390e7ff9, 46 },
    { 0x591e7ffa, 76 },
    { 0, 0 },
    { 0x208e7ffc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395e8001, 90 },
    { 0x132e8001, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e8006, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfde800e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee8014, 90 },
    { 0x795e8014, 98 },
    { 0xc35e8016, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ce801a, 33 },
    { 0, 0 },
    { 0x97be801c, 33 },
    { 0x962e801d, 78 },
    { 0, 0 },
    { 0x985e801f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0e8025, 69 },
    { 0x1b4e8026, 33 },
    { 0, 0 },
    { 0x270e8028, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98e802f, 34 },
    { 0xbe8030, 61 },
    { 0, 0 },
    { 0x9d2e8032, 33 },
    { 0x5b5e8032, 66 },
    { 0, 0 },
    { 0xd5fe8035, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7e803a, 76 },
    { 0x95fe803b, 95 },
    { 0x412e803c, 33 },
    { 0, 0 },
    { 0xbb1e803e, 43 },
    { 0xb5ce803f, 76 },
    { 0x72de8040, 33 },
    { 0x263e8041, 77 },
    { 0xee8e8042, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9e8045, 100 },
    { 0x48be8046, 34 },
    { 0, 0 },
    { 0x373e8048, 61 },
    { 0xd7e8049, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fde804e, 77 },
    { 0x7dce804e, 28 },
    { 0xe13e804e, 34 },
    { 0x524e8051, 76 },
    { 0x75be8052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eee8055, 90 },
    { 0xaf3e8055, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583e805a, 90 },
    { 0x69ee805b, 90 },
    { 0xbd2e805b, 55 },
    { 0x904e805b, 76 },
    { 0x7ece805a, 33 },
    { 0x941e805e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e8062, 95 },
    { 0xee9e8063, 90 },
    { 0x86de8063, 43 },
    { 0xacce8065, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47e806b, 23 },
    { 0xf15e806c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7de806f, 83 },
    { 0, 0 },
    { 0x8aee8071, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x175e8074, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ae8078, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ce8082, 84 },
    { 0x319e8083, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ee8086, 90 },
    { 0, 0 },
    { 0x79ce8088, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ce808d, 99 },
    { 0, 0 },
    { 0xd24e808f, 95 },
    { 0x254e8090, 76 },
    { 0, 0 },
    { 0xa19e8092, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fce8095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760e80a0, 55 },
    { 0xee7e80a1, 61 },
    { 0x8d1e80a0, 61 },
    { 0x4d0e80a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x398e80a6, 68 },
    { 0, 0 },
    { 0xb4be80a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399e80ad, 78 },
    { 0, 0 },
    { 0x988e80af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5be80b3, 98 },
    { 0, 0 },
    { 0x46be80b5, 90 },
    { 0xf4ae80b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267e80ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85de80be, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x359e80c2, 98 },
    { 0x96ae80c3, 33 },
    { 0, 0 },
    { 0x8ce80c5, 43 },
    { 0, 0 },
    { 0x4abe80c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x222e80ca, 91 },
    { 0, 0 },
    { 0x86fe80cc, 33 },
    { 0x97ce80cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5e80d6, 98 },
    { 0x86de80d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fe80da, 90 },
    { 0x68fe80da, 33 },
    { 0, 0 },
    { 0x815e80dd, 90 },
    { 0x4dbe80de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x927e80e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6e80e4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83e80ed, 33 },
    { 0xe30e80ee, 61 },
    { 0, 0 },
    { 0x9c5e80f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fe80f8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49e80fc, 55 },
    { 0x7dfe80fd, 61 },
    { 0, 0 },
    { 0x6dbe80ff, 61 },
    { 0x539e8100, 98 },
    { 0x42de8101, 100 },
    { 0, 0 },
    { 0x835e8103, 34 },
    { 0xf8fe8103, 61 },
    { 0xe5e8105, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1e810a, 52 },
    { 0x9afe810a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11e8112, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x587e8115, 93 },
    { 0, 0 },
    { 0x22de8117, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2e811c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1e811f, 90 },
    { 0x5e1e811f, 34 },
    { 0x960e8121, 80 },
    { 0x92ee8122, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5e8125, 90 },
    { 0x3f5e8125, 43 },
    { 0xef2e8125, 90 },
    { 0x3c1e8128, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44e812c, 61 },
    { 0x264e812d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2de8130, 90 },
    { 0xdc8e8131, 98 },
    { 0xbb2e8132, 23 },
    { 0x313e8133, 33 },
    { 0x89be8132, 66 },
    { 0x715e8134, 98 },
    { 0x81be8136, 91 },
    { 0, 0 },
    { 0x340e8138, 42 },
    { 0xacae8139, 95 },
    { 0x822e813a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ee8140, 90 },
    { 0xcede8140, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ce8144, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3e8148, 23 },
    { 0, 0 },
    { 0xa84e814a, 90 },
    { 0xa4de814a, 91 },
    { 0x3ece814c, 33 },
    { 0, 0 },
    { 0xcf0e814e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84e8151, 46 },
    { 0x774e8152, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805e8156, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x264e8159, 95 },
    { 0x859e815a, 61 },
    { 0, 0 },
    { 0x72be815c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ee8162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0e8168, 91 },
    { 0x534e8169, 98 },
    { 0xd33e816a, 61 },
    { 0xb3de8169, 34 },
    { 0x845e816c, 53 },
    { 0x8e6e816d, 95 },
    { 0xd70e816e, 33 },
    { 0x3cae816e, 61 },
    { 0x7cde8170, 61 },
    { 0x1d8e8171, 43 },
    { 0xf1fe8168, 65 },
    { 0, 0 },
    { 0xa48e8174, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ce817f, 55 },
    { 0, 0 },
    { 0x369e8181, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8de8189, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0e818d, 76 },
    { 0x678e818d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0e8191, 55 },
    { 0xb1fe8192, 61 },
    { 0xb7de8193, 77 },
    { 0x683e8194, 17 },
    { 0xfde8193, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace819a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30e819e, 33 },
    { 0x177e819f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfe81a3, 33 },
    { 0x84ae81a3, 65 },
    { 0x742e81a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3e81a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee81ad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3e81b1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15e81b4, 43 },
    { 0x77ce81b4, 61 },
    { 0xe99e81b6, 100 },
    { 0x7f3e81b5, 95 },
    { 0x8ace81b8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6e81bd, 95 },
    { 0, 0 },
    { 0x12fe81bf, 91 },
    { 0x921e81c0, 88 },
    { 0x3b0e81c1, 33 },
    { 0, 0 },
    { 0x93e81c3, 98 },
    { 0x17ee81c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbee81c8, 33 },
    { 0, 0 },
    { 0x459e81ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20e81d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x68de81d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68e81da, 55 },
    { 0xd23e81da, 90 },
    { 0, 0 },
    { 0x51fe81dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x220e81e0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ce81e4, 76 },
    { 0x658e81e5, 61 },
    { 0, 0 },
    { 0x897e81e7, 33 },
    { 0x978e81e7, 99 },
    { 0xc5fe81e8, 95 },
    { 0x981e81e9, 90 },
    { 0xb0ce81eb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0e81ee, 61 },
    { 0x45de81ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8e81f4, 95 },
    { 0x8bde81f5, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4e81fb, 43 },
    { 0x27ce81fc, 61 },
    { 0, 0 },
    { 0x84ce81fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x146e8201, 90 },
    { 0xc92e8202, 34 },
    { 0xdece8203, 33 },
    { 0xdb7e8204, 55 },
    { 0x66fe8205, 37 },
    { 0x3f1e8206, 33 },
    { 0, 0 },
    { 0x319e8208, 90 },
    { 0, 0 },
    { 0xea7e820a, 23 },
    { 0, 0 },
    { 0x95ae820c, 85 },
    { 0x696e820d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105e8213, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xface8216, 93 },
    { 0x37de8216, 90 },
    { 0, 0 },
    { 0xe5ae8219, 90 },
    { 0xbdbe821a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x780e821d, 46 },
    { 0x13ae821e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abe8222, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05e822a, 55 },
    { 0x23de822b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ae822f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921e8233, 100 },
    { 0x21be8234, 61 },
    { 0x36be8235, 67 },
    { 0x348e8234, 90 },
    { 0xe8de8237, 23 },
    { 0xd00e8237, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52e8242, 90 },
    { 0x78ee8243, 95 },
    { 0x8bee8244, 33 },
    { 0x5bce8243, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2e8248, 34 },
    { 0xb5ae8248, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaee824c, 98 },
    { 0xbaee824d, 90 },
    { 0, 0 },
    { 0xff3e824f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913e8259, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dce825c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae825f, 90 },
    { 0, 0 },
    { 0xa71e8261, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4e8266, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xefde8269, 33 },
    { 0, 0 },
    { 0xd97e826b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ce8273, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffe8276, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971e827b, 98 },
    { 0, 0 },
    { 0x726e827d, 61 },
    { 0x286e827d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8be8283, 61 },
    { 0x911e8283, 46 },
    { 0x78e8285, 98 },
    { 0, 0 },
    { 0xc1ee8287, 55 },
    { 0, 0 },
    { 0xbd1e8289, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612e828e, 43 },
    { 0xf2fe828f, 61 },
    { 0, 0 },
    { 0xc85e8291, 36 },
    { 0, 0 },
    { 0x7b6e8293, 95 },
    { 0x70be8294, 95 },
    { 0, 0 },
    { 0x5b0e8296, 76 },
    { 0, 0 },
    { 0xca3e8298, 76 },
    { 0x112e8299, 33 },
    { 0xe3ee829a, 98 },
    { 0xd19e8299, 61 },
    { 0x149e8298, 61 },
    { 0xfebe829d, 95 },
    { 0x2cce829e, 23 },
    { 0xd4de829f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dce82a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4de82ab, 61 },
    { 0x45fe82ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e82b1, 95 },
    { 0x9ffe82b2, 83 },
    { 0x969e82b3, 91 },
    { 0x5eee82b1, 87 },
    { 0xa37e82b2, 33 },
    { 0x507e82b6, 90 },
    { 0x315e82b7, 98 },
    { 0xdd1e82b6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220e82bd, 93 },
    { 0, 0 },
    { 0x81e82bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97e82c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ae82c9, 43 },
    { 0, 0 },
    { 0x9cbe82cb, 93 },
    { 0xc45e82cc, 43 },
    { 0, 0 },
    { 0x1afe82ce, 46 },
    { 0x232e82cf, 23 },
    { 0, 0 },
    { 0x15fe82d1, 23 },
    { 0x5aae82d2, 34 },
    { 0x756e82d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33be82d9, 61 },
    { 0x592e82d9, 87 },
    { 0x59e82db, 61 },
    { 0xe67e82da, 61 },
    { 0x31be82d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7e82e2, 43 },
    { 0x6bae82e2, 93 },
    { 0x22fe82e4, 76 },
    { 0xb99e82e5, 33 },
    { 0x8d9e82e6, 23 },
    { 0xec7e82e7, 76 },
    { 0xb82e82e7, 78 },
    { 0x658e82e7, 66 },
    { 0, 0 },
    { 0x6e9e82eb, 90 },
    { 0xc59e82ec, 99 },
    { 0x7b5e82ed, 99 },
    { 0x548e82eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdee82f8, 33 },
    { 0, 0 },
    { 0xda1e82fa, 34 },
    { 0, 0 },
    { 0xd17e82fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e8303, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9e8307, 95 },
    { 0xaf9e8308, 55 },
    { 0x231e8308, 34 },
    { 0xc30e830a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8e830d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9e8317, 90 },
    { 0x6e5e8317, 84 },
    { 0, 0 },
    { 0xb81e831a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x232e831d, 78 },
    { 0, 0 },
    { 0x78de831f, 85 },
    { 0x821e831f, 91 },
    { 0xf66e8321, 90 },
    { 0xf70e8322, 61 },
    { 0, 0 },
    { 0x488e8324, 67 },
    { 0xcc3e8325, 90 },
    { 0x679e8325, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e832a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5e832f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98e8334, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae833c, 91 },
    { 0x746e833d, 23 },
    { 0, 0 },
    { 0x1bee833f, 95 },
    { 0, 0 },
    { 0x937e8341, 76 },
    { 0x42e8341, 33 },
    { 0x1d7e8342, 76 },
    { 0xf1fe8344, 98 },
    { 0, 0 },
    { 0x96ce8346, 55 },
    { 0x758e8347, 81 },
    { 0, 0 },
    { 0x889e8349, 76 },
    { 0x12ce834a, 93 },
    { 0x6c9e834b, 88 },
    { 0x9fae834c, 33 },
    { 0xcbce834a, 90 },
    { 0xec7e834e, 23 },
    { 0xce0e834f, 90 },
    { 0x8c1e8350, 43 },
    { 0x6f9e8351, 23 },
    { 0x3bde8350, 55 },
    { 0x5fee8350, 90 },
    { 0x4c5e8354, 75 },
    { 0, 0 },
    { 0xdbce8356, 98 },
    { 0xd84e8357, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01e835a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xabe835d, 76 },
    { 0x577e835e, 60 },
    { 0x4d3e835e, 81 },
    { 0x4f6e835e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117e836b, 80 },
    { 0xb4e836b, 95 },
    { 0x8a2e836b, 93 },
    { 0, 0 },
    { 0x687e836f, 90 },
    { 0, 0 },
    { 0xadde8371, 61 },
    { 0x468e8372, 93 },
    { 0x50be8371, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80de837e, 98 },
    { 0x698e837f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463e8386, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xffce8389, 90 },
    { 0xee6e8389, 85 },
    { 0x88ae838b, 99 },
    { 0x2a5e838c, 90 },
    { 0x318e838d, 90 },
    { 0x354e838e, 90 },
    { 0x4ffe838f, 98 },
    { 0xf65e838f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x250e8393, 40 },
    { 0x4abe8394, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x80be8397, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4e839c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7e839f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6e83a2, 61 },
    { 0x255e83a3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7e83a7, 55 },
    { 0x130e83a8, 84 },
    { 0xd7fe83a7, 98 },
    { 0, 0 },
    { 0x29e83ab, 90 },
    { 0, 0 },
    { 0xdde83ad, 99 },
    { 0xbc2e83ae, 90 },
    { 0xf79e83ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1e83b7, 90 },
    { 0, 0 },
    { 0xb03e83b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefe83be, 33 },
    { 0x17ce83bf, 98 },
    { 0, 0 },
    { 0x76ae83c1, 33 },
    { 0xfe3e83c2, 61 },
    { 0x722e83c2, 95 },
    { 0x8cee83c4, 90 },
    { 0xf46e83c5, 78 },
    { 0xbcbe83c6, 90 },
    { 0xcfce83c1, 98 },
    { 0x24e83c8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903e83cc, 85 },
    { 0x5c8e83cd, 76 },
    { 0x85fe83ce, 46 },
    { 0x2a4e83cf, 33 },
    { 0xe4ee83d0, 23 },
    { 0x2a8e83d1, 33 },
    { 0x9e0e83ce, 33 },
    { 0x59ce83cd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfce83d6, 75 },
    { 0, 0 },
    { 0x711e83d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e83df, 90 },
    { 0, 0 },
    { 0x80fe83e1, 33 },
    { 0xb0ce83e2, 23 },
    { 0xdbce83e2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dee83e9, 95 },
    { 0xe1ee83e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60e83f1, 43 },
    { 0xbe4e83f2, 76 },
    { 0, 0 },
    { 0xae7e83f4, 88 },
    { 0x445e83f5, 52 },
    { 0, 0 },
    { 0xba6e83f7, 68 },
    { 0xc17e83f8, 46 },
    { 0, 0 },
    { 0x500e83fa, 91 },
    { 0, 0 },
    { 0xc6ae83fc, 33 },
    { 0xac6e83fd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3e8400, 55 },
    { 0xc11e8401, 33 },
    { 0x9aae8402, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e8406, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dde8411, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dae8416, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e8420, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2e842a, 76 },
    { 0xc81e842b, 98 },
    { 0x3aee842b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979e8435, 34 },
    { 0x13fe8436, 33 },
    { 0xe4be8435, 98 },
    { 0, 0 },
    { 0xa52e8439, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x969e843c, 34 },
    { 0, 0 },
    { 0xd9fe843e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670e8443, 61 },
    { 0x86fe8444, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe0e8447, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3be844a, 78 },
    { 0, 0 },
    { 0xafee844c, 95 },
    { 0xc3e844d, 88 },
    { 0xd83e844e, 61 },
    { 0x933e844e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9e8456, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbe845a, 95 },
    { 0xcf9e845b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85e8467, 90 },
    { 0x48fe8468, 55 },
    { 0xd4ce8469, 33 },
    { 0xa6ce846a, 67 },
    { 0xb9ee8468, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x19de846e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421e8472, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28e8478, 82 },
    { 0xe1ae8479, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2e847f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ece8490, 93 },
    { 0xb30e8491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x700e8495, 61 },
    { 0, 0 },
    { 0xbf3e8497, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573e849b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x742e849e, 33 },
    { 0xf1be849f, 98 },
    { 0x7c3e849e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1e84a3, 61 },
    { 0xb15e84a4, 98 },
    { 0xdfde84a4, 78 },
    { 0x57ae84a4, 34 },
    { 0x802e84a7, 97 },
    { 0, 0 },
    { 0x640e84a9, 99 },
    { 0x256e84a9, 95 },
    { 0, 0 },
    { 0x7a2e84ac, 95 },
    { 0x697e84ac, 61 },
    { 0x375e84ae, 55 },
    { 0x7d3e84af, 61 },
    { 0x600e84b0, 90 },
    { 0x1e7e84ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae84be, 98 },
    { 0x305e84be, 98 },
    { 0xf7ae84be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05e84c4, 98 },
    { 0, 0 },
    { 0x332e84c6, 61 },
    { 0xf89e84c7, 61 },
    { 0x7e6e84c6, 90 },
    { 0xa2ee84c9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06e84cf, 78 },
    { 0xabce84d0, 52 },
    { 0xc6e84d0, 95 },
    { 0, 0 },
    { 0x227e84d3, 87 },
    { 0x8f2e84d4, 90 },
    { 0, 0 },
    { 0xafae84d6, 88 },
    { 0x7b6e84d7, 33 },
    { 0, 0 },
    { 0x9ebe84d9, 95 },
    { 0xf1de84da, 90 },
    { 0x7f9e84da, 33 },
    { 0xcabe84dc, 76 },
    { 0xcdce84d9, 67 },
    { 0x9ade84da, 61 },
    { 0xb88e84dc, 55 },
    { 0xff0e84e0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9be84e3, 95 },
    { 0x1d1e84e4, 76 },
    { 0x28be84e5, 94 },
    { 0x669e84e4, 95 },
    { 0, 0 },
    { 0xb9ae84e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36de84ed, 67 },
    { 0xbcde84ee, 90 },
    { 0x818e84ee, 33 },
    { 0x956e84f0, 95 },
    { 0x324e84f0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2e84f5, 34 },
    { 0, 0 },
    { 0x850e84f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278e84ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x837e8502, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813e850c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692e8510, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54e8519, 91 },
    { 0, 0 },
    { 0xadfe851b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce8520, 87 },
    { 0x56e8521, 61 },
    { 0xf15e8522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6e8526, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2e852a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839e852e, 33 },
    { 0x7d5e852f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8e8535, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8e8538, 75 },
    { 0x284e8538, 87 },
    { 0, 0 },
    { 0x3c0e853b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7e853f, 84 },
    { 0, 0 },
    { 0x680e8541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50e8545, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x450e8548, 98 },
    { 0, 0 },
    { 0x558e854a, 34 },
    { 0xcefe854a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6e854f, 90 },
    { 0x466e854f, 76 },
    { 0, 0 },
    { 0x211e8552, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e8556, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3e855c, 98 },
    { 0x734e855c, 76 },
    { 0xbc6e855e, 53 },
    { 0, 0 },
    { 0xb3ee8560, 33 },
    { 0, 0 },
    { 0x768e8562, 33 },
    { 0, 0 },
    { 0x6dfe8564, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77e8567, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9e856a, 76 },
    { 0x324e856b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106e856f, 88 },
    { 0x584e8570, 23 },
    { 0x3c5e8571, 85 },
    { 0x7fee8571, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528e857a, 87 },
    { 0xdfe857b, 17 },
    { 0xb85e857c, 84 },
    { 0x948e857a, 61 },
    { 0xa45e857a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e0e8582, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x225e8585, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887e858a, 33 },
    { 0, 0 },
    { 0xa6ae858c, 66 },
    { 0x96ce858c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05e8590, 95 },
    { 0xeb2e8591, 76 },
    { 0xb47e8592, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3e8595, 95 },
    { 0xfd9e8595, 66 },
    { 0, 0 },
    { 0x85e8598, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e859c, 90 },
    { 0xa60e859c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e85a1, 90 },
    { 0, 0 },
    { 0x105e85a3, 94 },
    { 0x2b4e85a3, 95 },
    { 0, 0 },
    { 0x8efe85a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbe85a9, 90 },
    { 0x345e85a9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1be85ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898e85bc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ee85c0, 98 },
    { 0x190e85c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ae85c4, 90 },
    { 0, 0 },
    { 0x2c2e85c6, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ce85cc, 40 },
    { 0, 0 },
    { 0x776e85ce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbee85d2, 61 },
    { 0xfb5e85d3, 90 },
    { 0x3dae85d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e85d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ece85e4, 51 },
    { 0x26ae85e5, 91 },
    { 0, 0 },
    { 0x3cee85e7, 76 },
    { 0x915e85e7, 61 },
    { 0x213e85e9, 33 },
    { 0xd43e85e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e85f3, 23 },
    { 0x593e85f4, 17 },
    { 0x83ae85f4, 98 },
    { 0xad8e85f6, 23 },
    { 0x3bbe85f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31de85fb, 34 },
    { 0, 0 },
    { 0xe9ae85fd, 78 },
    { 0, 0 },
    { 0x247e85ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e8605, 77 },
    { 0x6d5e8606, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544e860c, 90 },
    { 0x692e860d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4e8610, 91 },
    { 0x6b7e8611, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeee8617, 76 },
    { 0x800e8618, 23 },
    { 0, 0 },
    { 0x73de861a, 46 },
    { 0xa9ae861b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbe861e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46de8625, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x86de8628, 87 },
    { 0x5dbe8629, 23 },
    { 0x3c7e862a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x418e862d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1e8631, 76 },
    { 0x614e8632, 98 },
    { 0, 0 },
    { 0x1c2e8634, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284e8638, 17 },
    { 0, 0 },
    { 0x8efe863a, 76 },
    { 0x312e863b, 98 },
    { 0x8fbe863a, 74 },
    { 0x3dce863d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xddae8640, 95 },
    { 0x770e8641, 76 },
    { 0x40e8642, 33 },
    { 0, 0 },
    { 0x25fe8644, 33 },
    { 0xe7ae8645, 98 },
    { 0xb57e8645, 95 },
    { 0x317e8647, 87 },
    { 0x21e8647, 95 },
    { 0xf24e8649, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0de864c, 95 },
    { 0x2a5e864d, 43 },
    { 0, 0 },
    { 0x380e864f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ce8652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x678e8655, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8e8659, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1e865d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2e8660, 36 },
    { 0, 0 },
    { 0xc58e8662, 95 },
    { 0, 0 },
    { 0x5cfe8664, 33 },
    { 0, 0 },
    { 0xd92e8666, 81 },
    { 0xd87e8667, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18e866e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x452e8671, 90 },
    { 0x70be8672, 95 },
    { 0, 0 },
    { 0xfcde8674, 76 },
    { 0x173e8675, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x699e867a, 90 },
    { 0xe07e867a, 46 },
    { 0, 0 },
    { 0x69ce867d, 43 },
    { 0xd71e867e, 33 },
    { 0x63fe867f, 34 },
    { 0, 0 },
    { 0x952e8681, 43 },
    { 0x829e8681, 90 },
    { 0xd33e8682, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8e8687, 90 },
    { 0xe88e8688, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27e8690, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fee8693, 98 },
    { 0, 0 },
    { 0xc9be8695, 61 },
    { 0x35e8695, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466e869a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e86a4, 98 },
    { 0xa15e86a5, 23 },
    { 0x369e86a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e86af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ce86b4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbde86b9, 61 },
    { 0, 0 },
    { 0xf8be86bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5e86c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75de86c7, 33 },
    { 0x91fe86c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5e86cd, 33 },
    { 0x576e86cd, 77 },
    { 0x409e86cf, 23 },
    { 0xf3ee86d0, 95 },
    { 0, 0 },
    { 0xb3ae86d2, 76 },
    { 0x2fee86d3, 40 },
    { 0x682e86d4, 90 },
    { 0x328e86d5, 23 },
    { 0xc20e86d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe86df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebce86e2, 96 },
    { 0, 0 },
    { 0xedce86e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aee86ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ce86ed, 43 },
    { 0, 0 },
    { 0x2dee86ef, 90 },
    { 0x822e86ef, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ee86f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60e86fc, 43 },
    { 0xdc3e86fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7e8701, 43 },
    { 0x619e8702, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1e870a, 33 },
    { 0xcf1e870b, 90 },
    { 0x634e870a, 61 },
    { 0x44fe870c, 17 },
    { 0x90ee870e, 36 },
    { 0x57ce870f, 90 },
    { 0x903e870e, 99 },
    { 0x189e870e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556e8718, 98 },
    { 0, 0 },
    { 0xaa0e871a, 90 },
    { 0x17de871a, 61 },
    { 0xb52e871c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be871f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e8722, 93 },
    { 0xdd7e8723, 33 },
    { 0xc5ce8724, 90 },
    { 0x89e8725, 34 },
    { 0, 0 },
    { 0xccfe8727, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2e872c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e872f, 34 },
    { 0xce8e8730, 95 },
    { 0, 0 },
    { 0x7b6e8732, 80 },
    { 0, 0 },
    { 0x5b1e8734, 90 },
    { 0, 0 },
    { 0x1b5e8736, 61 },
    { 0xf9de8737, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e873b, 61 },
    { 0x228e873c, 33 },
    { 0, 0 },
    { 0xdfde873e, 84 },
    { 0, 0 },
    { 0x76fe8740, 95 },
    { 0, 0 },
    { 0x22de8742, 36 },
    { 0x35ce8743, 43 },
    { 0xdee8743, 85 },
    { 0x3d3e8743, 76 },
    { 0x43ee8746, 90 },
    { 0xfc5e8747, 78 },
    { 0xaf6e8748, 61 },
    { 0xb67e8746, 90 },
    { 0x278e874a, 52 },
    { 0xe65e874b, 55 },
    { 0x8d9e874c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x901e874f, 61 },
    { 0xa2ce8750, 61 },
    { 0, 0 },
    { 0xdd9e8752, 96 },
    { 0, 0 },
    { 0x638e8754, 33 },
    { 0xf7ce8755, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x983e8758, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5e8761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee8765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abe8768, 68 },
    { 0, 0 },
    { 0xec1e876a, 91 },
    { 0, 0 },
    { 0xcc5e876c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabe8770, 38 },
    { 0, 0 },
    { 0xb0be8772, 46 },
    { 0x2a9e8772, 23 },
    { 0xacce8774, 88 },
    { 0x2ace8775, 33 },
    { 0x8c3e8776, 42 },
    { 0, 0 },
    { 0x2e6e8778, 90 },
    { 0x5e1e8779, 61 },
    { 0, 0 },
    { 0x80be877b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaee8781, 90 },
    { 0, 0 },
    { 0x35be8783, 99 },
    { 0, 0 },
    { 0x552e8785, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9e878f, 77 },
    { 0x9a6e878f, 95 },
    { 0, 0 },
    { 0x509e8792, 23 },
    { 0, 0 },
    { 0x941e8794, 98 },
    { 0x556e8795, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34de8799, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x714e87a3, 93 },
    { 0x9c8e87a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21e87aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85e87b0, 33 },
    { 0x862e87b1, 78 },
    { 0xadce87b0, 61 },
    { 0, 0 },
    { 0xd6ae87b4, 46 },
    { 0xa64e87b5, 95 },
    { 0x21e87b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5de87bd, 33 },
    { 0x7eae87be, 85 },
    { 0x402e87bf, 55 },
    { 0x662e87be, 95 },
    { 0xcd9e87be, 76 },
    { 0, 0 },
    { 0xcbee87c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e87cc, 90 },
    { 0x28fe87cc, 91 },
    { 0x78de87ce, 91 },
    { 0xb22e87cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8e87d3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae87d6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x572e87d9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1e87de, 33 },
    { 0xa5be87df, 85 },
    { 0x95fe87e0, 84 },
    { 0x992e87e0, 90 },
    { 0, 0 },
    { 0xa7fe87e3, 56 },
    { 0x406e87e4, 87 },
    { 0xd35e87e4, 95 },
    { 0xfa8e87e6, 95 },
    { 0x3bbe87e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fe87ec, 43 },
    { 0xad2e87ed, 33 },
    { 0x9dbe87ed, 61 },
    { 0x37de87ee, 61 },
    { 0xc6fe87f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6e87f5, 99 },
    { 0xd8be87f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96e87f9, 52 },
    { 0, 0 },
    { 0xc6ce87fb, 33 },
    { 0x35ce87fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688e8802, 98 },
    { 0x817e8802, 95 },
    { 0xe79e8804, 98 },
    { 0xc60e8805, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e880e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2e8813, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x341e8823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fe8827, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5e882f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e8833, 90 },
    { 0x73de8834, 85 },
    { 0xef5e8833, 55 },
    { 0x55de8836, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e883c, 99 },
    { 0x3f6e883d, 76 },
    { 0x54ee883c, 46 },
    { 0xb0de883f, 33 },
    { 0, 0 },
    { 0x4d3e8841, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ee8846, 52 },
    { 0x988e8847, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5e884b, 99 },
    { 0xcf4e884c, 74 },
    { 0x7cde884c, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7e8854, 23 },
    { 0x8c5e8854, 90 },
    { 0xcb7e8855, 77 },
    { 0x1ace8857, 61 },
    { 0x5a1e8857, 66 },
    { 0xc17e8859, 101 },
    { 0xae1e8858, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1e8861, 23 },
    { 0xfcee8861, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6e886a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dee886d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585e8871, 55 },
    { 0x50ee8872, 90 },
    { 0x715e8872, 34 },
    { 0, 0 },
    { 0x660e8875, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce887a, 33 },
    { 0x8c2e887b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ae887f, 55 },
    { 0, 0 },
    { 0xa3e8881, 76 },
    { 0, 0 },
    { 0xfa0e8883, 75 },
    { 0x598e8883, 38 },
    { 0xd7ce8883, 23 },
    { 0, 0 },
    { 0xb0de8887, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x95de888a, 34 },
    { 0x34e888a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ce888f, 61 },
    { 0x586e8890, 85 },
    { 0xf85e888f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2e8897, 43 },
    { 0x2dbe8897, 76 },
    { 0x6bbe8899, 34 },
    { 0x688e889a, 61 },
    { 0x87be889a, 76 },
    { 0, 0 },
    { 0x207e889d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7e88a1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x423e88a4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ee88a7, 34 },
    { 0xc5ae88a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x122e88ab, 69 },
    { 0, 0 },
    { 0x1e9e88ad, 61 },
    { 0, 0 },
    { 0x169e88af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbe88b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527e88ba, 88 },
    { 0x8c0e88bb, 33 },
    { 0x432e88ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6e88c1, 33 },
    { 0x47e88c2, 83 },
    { 0x8a4e88c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ae88c6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce88cb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x962e88ce, 99 },
    { 0, 0 },
    { 0x367e88d0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfe88d4, 96 },
    { 0xfe1e88d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x935e88d8, 90 },
    { 0, 0 },
    { 0xa48e88da, 33 },
    { 0xeace88db, 70 },
    { 0x98be88da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ce88e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f7e88ec, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eee88f1, 99 },
    { 0, 0 },
    { 0x5d0e88f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ce88f9, 61 },
    { 0xe17e88fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942e8904, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329e8911, 93 },
    { 0, 0 },
    { 0xbdde8913, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce8916, 23 },
    { 0, 0 },
    { 0xc6e8918, 90 },
    { 0, 0 },
    { 0x849e891a, 96 },
    { 0x154e891b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1e891e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5e8921, 55 },
    { 0xdb7e8922, 98 },
    { 0xe55e8923, 90 },
    { 0x5cde8921, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe8928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65e892b, 55 },
    { 0xb95e892c, 33 },
    { 0, 0 },
    { 0x75be892e, 76 },
    { 0x555e892e, 55 },
    { 0, 0 },
    { 0x48ee8931, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x521e8938, 33 },
    { 0x40ae8939, 61 },
    { 0, 0 },
    { 0x869e893b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x204e8941, 90 },
    { 0xa89e8942, 90 },
    { 0x2f0e8943, 90 },
    { 0xfade8943, 61 },
    { 0xb71e8942, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdce8948, 87 },
    { 0x7f4e8949, 33 },
    { 0, 0 },
    { 0xce4e894b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64e894e, 87 },
    { 0, 0 },
    { 0xd1fe8950, 68 },
    { 0x859e8951, 85 },
    { 0x673e8950, 99 },
    { 0x4c3e8953, 33 },
    { 0, 0 },
    { 0x383e8955, 90 },
    { 0, 0 },
    { 0x96ee8957, 85 },
    { 0xefbe8958, 76 },
    { 0xa81e8959, 84 },
    { 0x4c9e8957, 61 },
    { 0xdbe895b, 93 },
    { 0x432e8959, 76 },
    { 0x3e1e8958, 66 },
    { 0x370e8958, 95 },
    { 0xc71e8957, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fe8962, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4e8965, 99 },
    { 0x696e8965, 61 },
    { 0, 0 },
    { 0x541e8968, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504e896d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x39e8970, 95 },
    { 0, 0 },
    { 0x1abe8972, 87 },
    { 0xe36e8972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85e8976, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ee897a, 53 },
    { 0xf4ee897b, 100 },
    { 0xd5ae897c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684e8983, 78 },
    { 0x962e8984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7e8987, 33 },
    { 0x4bfe8987, 31 },
    { 0x84ee8987, 95 },
    { 0x903e898a, 93 },
    { 0xcc5e898a, 55 },
    { 0x1b7e898c, 61 },
    { 0x956e898c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbee8990, 90 },
    { 0, 0 },
    { 0x33be8992, 78 },
    { 0x1dae8993, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6e8996, 43 },
    { 0, 0 },
    { 0x154e8998, 61 },
    { 0xdc6e8999, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2e899c, 55 },
    { 0, 0 },
    { 0xc70e899e, 90 },
    { 0x1a3e899f, 66 },
    { 0x671e899e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17de89a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc02e89a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272e89ad, 95 },
    { 0xdb8e89ad, 90 },
    { 0xcf5e89ae, 61 },
    { 0x93ce89b0, 76 },
    { 0, 0 },
    { 0xdc4e89b2, 46 },
    { 0x5a9e89b3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9e89b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28e89ba, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179e89c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ede89cb, 100 },
    { 0, 0 },
    { 0xc75e89cd, 87 },
    { 0x987e89cd, 90 },
    { 0xaf2e89cf, 95 },
    { 0xc07e89cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68e89d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61be89d7, 61 },
    { 0x3b0e89d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1de89dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x916e89e2, 43 },
    { 0xfa0e89e2, 33 },
    { 0, 0 },
    { 0x2e2e89e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1e89e9, 69 },
    { 0, 0 },
    { 0x7dbe89eb, 98 },
    { 0x178e89ec, 61 },
    { 0xbcae89eb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x211e89f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818e89f8, 43 },
    { 0x21fe89f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbe89fc, 43 },
    { 0, 0 },
    { 0xd6be89fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1e8a03, 90 },
    { 0xede8a04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x246e8a0b, 61 },
    { 0x43e8a0b, 33 },
    { 0x615e8a0d, 23 },
    { 0, 0 },
    { 0x439e8a0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0be8a14, 75 },
    { 0, 0 },
    { 0x39ae8a16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693e8a1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae8a22, 98 },
    { 0xbb7e8a23, 95 },
    { 0, 0 },
    { 0x365e8a25, 17 },
    { 0, 0 },
    { 0x3cce8a27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63e8a2b, 23 },
    { 0xc7fe8a2b, 81 },
    { 0xd49e8a2c, 90 },
    { 0xd55e8a2e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeae8a38, 94 },
    { 0x6c4e8a39, 33 },
    { 0, 0 },
    { 0x199e8a3b, 61 },
    { 0xf96e8a3c, 23 },
    { 0, 0 },
    { 0x41ee8a3e, 23 },
    { 0xa70e8a3f, 98 },
    { 0xb8ce8a40, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9e8a47, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6e8a4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaee8a51, 61 },
    { 0xc41e8a51, 98 },
    { 0x9d4e8a52, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ae8a57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ee8a61, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7e8a66, 78 },
    { 0x8fae8a67, 68 },
    { 0xdaae8a67, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2e8a6c, 91 },
    { 0xe99e8a6d, 88 },
    { 0x3c1e8a6c, 61 },
    { 0x78be8a6c, 95 },
    { 0xa2fe8a70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6e8a74, 64 },
    { 0xbf2e8a75, 61 },
    { 0x775e8a75, 82 },
    { 0xff3e8a76, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72e8a7a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0e8a7f, 33 },
    { 0xab7e8a7f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fce8a84, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ae8a87, 61 },
    { 0xe0ae8a87, 95 },
    { 0x566e8a89, 55 },
    { 0xce2e8a8a, 61 },
    { 0x6b2e8a88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76e8a90, 55 },
    { 0x57be8a90, 46 },
    { 0, 0 },
    { 0x93ae8a93, 94 },
    { 0xa86e8a94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ae8a9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27e8a9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2e8aa1, 33 },
    { 0x820e8aa2, 55 },
    { 0xee8e8aa1, 98 },
    { 0xa46e8aa4, 78 },
    { 0x70de8aa5, 90 },
    { 0x6ede8aa6, 77 },
    { 0x381e8aa1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493e8aac, 33 },
    { 0x1d1e8aac, 79 },
    { 0, 0 },
    { 0x3ede8aaf, 98 },
    { 0x3fe8aaf, 34 },
    { 0, 0 },
    { 0x953e8ab2, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ee8ab5, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ee8abb, 88 },
    { 0x61ee8abc, 55 },
    { 0x606e8abb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7e8ac3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ede8ac6, 67 },
    { 0x2b2e8ac7, 55 },
    { 0x48ce8ac7, 46 },
    { 0x456e8ac8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801e8ace, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7ee8ad3, 65 },
    { 0, 0 },
    { 0xefae8ad5, 76 },
    { 0xd6fe8ad6, 95 },
    { 0xb32e8ad7, 55 },
    { 0x20e8ad8, 17 },
    { 0xe98e8ad8, 52 },
    { 0xca4e8ad7, 33 },
    { 0xe2ce8adb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7e8ade, 95 },
    { 0, 0 },
    { 0x69ce8ae0, 61 },
    { 0xf6de8ae1, 95 },
    { 0, 0 },
    { 0x96ce8ae3, 52 },
    { 0xcee8ae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e8ae7, 66 },
    { 0xb57e8ae8, 23 },
    { 0xc0ae8ae9, 98 },
    { 0x20be8ae9, 55 },
    { 0x25ce8ae8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fce8af1, 67 },
    { 0, 0 },
    { 0x860e8af3, 98 },
    { 0xa51e8af4, 69 },
    { 0, 0 },
    { 0x794e8af6, 98 },
    { 0x33ce8af7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ee8afa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975e8afe, 55 },
    { 0x8e7e8aff, 34 },
    { 0x1d9e8b00, 97 },
    { 0xb98e8b00, 33 },
    { 0x848e8b02, 61 },
    { 0, 0 },
    { 0x9d8e8b04, 98 },
    { 0xa20e8b04, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75e8b0b, 98 },
    { 0x6bce8b0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3e8b11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fe8b15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1de8b19, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de8b1c, 46 },
    { 0x895e8b1d, 75 },
    { 0xe54e8b1d, 34 },
    { 0xb81e8b1f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ee8b22, 61 },
    { 0x90de8b23, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fe8b29, 90 },
    { 0x231e8b2a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ee8b2e, 95 },
    { 0x816e8b2e, 76 },
    { 0x955e8b30, 33 },
    { 0xf19e8b31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce8b35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4e8b3b, 67 },
    { 0, 0 },
    { 0x557e8b3d, 34 },
    { 0x2dbe8b3e, 98 },
    { 0x2efe8b3d, 90 },
    { 0, 0 },
    { 0xb5ee8b41, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dee8b49, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fe8b4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7e8b51, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92e8b55, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386e8b59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a9e8b5e, 55 },
    { 0x457e8b5f, 33 },
    { 0xb09e8b60, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2e8b69, 77 },
    { 0x91fe8b6a, 43 },
    { 0xc6fe8b6a, 66 },
    { 0, 0 },
    { 0x11ae8b6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2e8b72, 42 },
    { 0x6e2e8b73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be8b76, 93 },
    { 0x24e8b76, 76 },
    { 0x3e0e8b76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98de8b7d, 61 },
    { 0x2dde8b7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51e8b86, 90 },
    { 0xe51e8b87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x496e8b8a, 98 },
    { 0x3bae8b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ce8b8e, 90 },
    { 0, 0 },
    { 0x75ae8b90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8e8b93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x904e8b96, 88 },
    { 0x9a4e8b96, 95 },
    { 0, 0 },
    { 0x458e8b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfee8b9e, 76 },
    { 0x7ede8b9e, 33 },
    { 0xc17e8ba0, 23 },
    { 0xf2ce8ba1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6e8ba5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ce8ba8, 67 },
    { 0xd29e8ba9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9e8bad, 61 },
    { 0x311e8bae, 43 },
    { 0xeaee8bad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830e8bb5, 95 },
    { 0x237e8bb6, 36 },
    { 0x6e4e8bb7, 98 },
    { 0x9a4e8bb5, 85 },
    { 0, 0 },
    { 0xde8bba, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219e8bbe, 66 },
    { 0x60ce8bbf, 61 },
    { 0, 0 },
    { 0x6b9e8bc1, 61 },
    { 0x61ce8bc2, 23 },
    { 0x153e8bc3, 61 },
    { 0, 0 },
    { 0xa68e8bc5, 98 },
    { 0, 0 },
    { 0xa39e8bc7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81e8bcb, 17 },
    { 0x6afe8bcb, 99 },
    { 0x158e8bcd, 78 },
    { 0x3e2e8bce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd34e8bd2, 90 },
    { 0, 0 },
    { 0x50be8bd4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x71de8bd7, 85 },
    { 0xb26e8bd8, 23 },
    { 0x13ee8bd7, 34 },
    { 0xd77e8bd8, 95 },
    { 0x640e8bd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7e8be7, 75 },
    { 0, 0 },
    { 0x36de8be9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17e8bed, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49e8bf1, 95 },
    { 0xa74e8bf1, 23 },
    { 0x8f1e8bf3, 81 },
    { 0xe41e8bf3, 61 },
    { 0xf78e8bf5, 23 },
    { 0xe7fe8bf6, 94 },
    { 0x2bee8bf7, 33 },
    { 0x3dce8bf7, 23 },
    { 0xd5fe8bf9, 46 },
    { 0x48ee8bfa, 78 },
    { 0xf6de8bfb, 95 },
    { 0x67e8bfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa83e8c00, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672e8c05, 23 },
    { 0xc42e8c05, 96 },
    { 0x2e9e8c06, 33 },
    { 0x3eee8c08, 76 },
    { 0, 0 },
    { 0x7de8c0a, 61 },
    { 0x403e8c0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abe8c0e, 33 },
    { 0x37ce8c0f, 90 },
    { 0x63ee8c10, 46 },
    { 0x4ede8c11, 33 },
    { 0x4f4e8c10, 17 },
    { 0x169e8c0f, 33 },
    { 0xe9e8c14, 93 },
    { 0, 0 },
    { 0xeb1e8c16, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265e8c1d, 90 },
    { 0, 0 },
    { 0x3e6e8c1f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4e8c23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1e8c2a, 48 },
    { 0, 0 },
    { 0x612e8c2c, 33 },
    { 0x945e8c2c, 95 },
    { 0x3a8e8c2d, 61 },
    { 0xdf0e8c2f, 95 },
    { 0xd11e8c2d, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5e8c38, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62e8c3e, 88 },
    { 0x52ee8c3f, 90 },
    { 0, 0 },
    { 0x908e8c41, 76 },
    { 0x375e8c41, 90 },
    { 0, 0 },
    { 0xa4ae8c44, 61 },
    { 0x50be8c45, 81 },
    { 0xce2e8c45, 61 },
    { 0xd3ce8c47, 23 },
    { 0xe4fe8c45, 61 },
    { 0xf47e8c44, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456e8c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79e8c5f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x761e8c64, 100 },
    { 0xa97e8c65, 61 },
    { 0x96be8c66, 61 },
    { 0xe51e8c67, 87 },
    { 0x62de8c65, 43 },
    { 0x50ae8c67, 33 },
    { 0x72de8c6a, 98 },
    { 0x78ae8c6b, 33 },
    { 0x9fae8c6c, 90 },
    { 0x594e8c6b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91e8c70, 23 },
    { 0xa3ae8c70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21e8c74, 98 },
    { 0x21be8c75, 61 },
    { 0xe4ce8c76, 43 },
    { 0x131e8c75, 61 },
    { 0, 0 },
    { 0x2d7e8c79, 84 },
    { 0xa0fe8c7a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7e8c7e, 61 },
    { 0xc9e8c7f, 90 },
    { 0x864e8c7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeae8c85, 76 },
    { 0xacde8c86, 90 },
    { 0x3c7e8c87, 90 },
    { 0x5c0e8c88, 23 },
    { 0x5f5e8c85, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e8c8f, 33 },
    { 0x2d0e8c90, 98 },
    { 0, 0 },
    { 0xdbce8c92, 78 },
    { 0x9e2e8c93, 84 },
    { 0, 0 },
    { 0xfe0e8c95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49e8c98, 61 },
    { 0, 0 },
    { 0xa22e8c9a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368e8ca0, 66 },
    { 0, 0 },
    { 0x6f7e8ca2, 93 },
    { 0x200e8ca3, 23 },
    { 0xf5fe8ca2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970e8cab, 33 },
    { 0xf3be8cab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bae8caf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42be8cb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2e8cb7, 98 },
    { 0xac5e8cb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0be8cbe, 23 },
    { 0, 0 },
    { 0x212e8cc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964e8cc4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x735e8cc7, 93 },
    { 0, 0 },
    { 0xb97e8cc9, 95 },
    { 0x718e8cca, 68 },
    { 0x561e8ccb, 90 },
    { 0xf75e8cc9, 55 },
    { 0x3e4e8ccb, 95 },
    { 0x712e8cce, 90 },
    { 0x7ffe8ccc, 90 },
    { 0x81fe8cd0, 46 },
    { 0x189e8cd1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x640e8cd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fe8ce1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cee8ce5, 46 },
    { 0xb97e8ce6, 85 },
    { 0, 0 },
    { 0xc5be8ce8, 93 },
    { 0xf3ee8ce9, 66 },
    { 0xbade8ce8, 65 },
    { 0xf41e8ceb, 23 },
    { 0xfb4e8cec, 99 },
    { 0xd96e8ced, 61 },
    { 0xf5be8cee, 33 },
    { 0xe30e8cef, 33 },
    { 0xc38e8ceb, 23 },
    { 0xbcfe8ceb, 95 },
    { 0x729e8cf1, 55 },
    { 0, 0 },
    { 0x5ebe8cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce8cf8, 66 },
    { 0, 0 },
    { 0x9fee8cfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3e8d00, 90 },
    { 0x478e8d01, 61 },
    { 0xe26e8d02, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ee8d09, 90 },
    { 0x259e8d0a, 55 },
    { 0xf4e8d0a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2e8d0e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ee8d14, 76 },
    { 0, 0 },
    { 0x450e8d16, 23 },
    { 0xbb9e8d17, 55 },
    { 0x5e3e8d18, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3e8d1c, 61 },
    { 0xed3e8d1c, 78 },
    { 0x739e8d1d, 76 },
    { 0xd4ae8d1d, 76 },
    { 0x36be8d20, 52 },
    { 0xa83e8d21, 93 },
    { 0, 0 },
    { 0x20ee8d23, 43 },
    { 0xe04e8d23, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8e8d27, 98 },
    { 0, 0 },
    { 0x854e8d29, 34 },
    { 0xa07e8d29, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7e8d2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5e8d37, 90 },
    { 0x36de8d37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2e8d3c, 55 },
    { 0, 0 },
    { 0x92de8d3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1e8d42, 67 },
    { 0, 0 },
    { 0x533e8d44, 67 },
    { 0, 0 },
    { 0x674e8d46, 95 },
    { 0, 0 },
    { 0xb87e8d48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ee8d50, 43 },
    { 0x930e8d51, 91 },
    { 0x485e8d52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ae8d57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x643e8d5a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ee8d61, 23 },
    { 0xeae8d62, 46 },
    { 0x55be8d62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae8d68, 93 },
    { 0x844e8d69, 65 },
    { 0xfce8d6a, 33 },
    { 0xee6e8d6b, 95 },
    { 0xa6ae8d6c, 93 },
    { 0xc50e8d6d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3e8d70, 52 },
    { 0x47e8d71, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743e8d75, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08e8d79, 90 },
    { 0, 0 },
    { 0xfa5e8d7b, 100 },
    { 0xddfe8d7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325e8d84, 96 },
    { 0, 0 },
    { 0x953e8d86, 90 },
    { 0x965e8d86, 33 },
    { 0xa89e8d87, 90 },
    { 0x711e8d89, 84 },
    { 0, 0 },
    { 0x9f7e8d8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42de8d91, 95 },
    { 0x221e8d91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478e8d96, 33 },
    { 0x582e8d97, 78 },
    { 0, 0 },
    { 0xc74e8d99, 33 },
    { 0x31ce8d9a, 85 },
    { 0xcc2e8d9b, 33 },
    { 0xbe8d9a, 90 },
    { 0xa6be8d9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dce8da2, 43 },
    { 0x1fee8da3, 23 },
    { 0x24ee8da4, 98 },
    { 0x39ee8da2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebde8dab, 33 },
    { 0, 0 },
    { 0x224e8dad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89e8db6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6e8dc0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6e8dc5, 83 },
    { 0xdebe8dc6, 98 },
    { 0xc5be8dc6, 61 },
    { 0xdcbe8dc8, 90 },
    { 0xcfe8dc8, 76 },
    { 0x819e8dca, 80 },
    { 0x761e8dca, 85 },
    { 0xe00e8dca, 84 },
    { 0xa48e8dcd, 76 },
    { 0xb6fe8dc7, 99 },
    { 0x7d9e8dcf, 17 },
    { 0xaede8dd0, 100 },
    { 0x666e8dd1, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x969e8dd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x882e8dd7, 61 },
    { 0x7bfe8dd8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc34e8ddc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10e8de5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bae8dea, 34 },
    { 0x857e8dea, 98 },
    { 0x2ebe8dec, 98 },
    { 0x1a4e8ded, 34 },
    { 0x711e8dee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aee8df2, 95 },
    { 0xe9e8df3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3e8df6, 76 },
    { 0, 0 },
    { 0x448e8df8, 61 },
    { 0xf86e8df9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ce8dfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11de8e02, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bce8e06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39de8e0a, 90 },
    { 0x91be8e0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4e8e11, 90 },
    { 0x3cbe8e12, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e8e18, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2e8e1b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812e8e21, 33 },
    { 0, 0 },
    { 0xeb5e8e23, 95 },
    { 0, 0 },
    { 0x9b3e8e25, 38 },
    { 0xf4fe8e25, 23 },
    { 0, 0 },
    { 0xe7e8e28, 23 },
    { 0x7bfe8e28, 23 },
    { 0, 0 },
    { 0x723e8e2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4e8e30, 99 },
    { 0xd9ee8e31, 92 },
    { 0, 0 },
    { 0x75de8e33, 80 },
    { 0x90ce8e33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dae8e37, 61 },
    { 0xfbfe8e37, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x820e8e3b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694e8e46, 77 },
    { 0xcf1e8e47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9e8e4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00e8e52, 95 },
    { 0xa2ce8e53, 98 },
    { 0xfe6e8e53, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e8e5a, 67 },
    { 0x97fe8e5b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46de8e5f, 23 },
    { 0xdc8e8e60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x784e8e63, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afe8e66, 61 },
    { 0, 0 },
    { 0x7d9e8e68, 78 },
    { 0x252e8e68, 33 },
    { 0xa2fe8e6a, 76 },
    { 0x5ae8e6b, 23 },
    { 0xff7e8e6c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24e8e75, 68 },
    { 0x6b2e8e76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90de8e7c, 93 },
    { 0, 0 },
    { 0x4f8e8e7e, 23 },
    { 0xe99e8e7f, 87 },
    { 0xe1be8e80, 76 },
    { 0x9cce8e81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfe8e89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfe8e8e, 87 },
    { 0xcd4e8e8e, 90 },
    { 0xce8e8f, 76 },
    { 0, 0 },
    { 0xadfe8e92, 90 },
    { 0x120e8e93, 99 },
    { 0xd1fe8e94, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97e8e97, 33 },
    { 0x103e8e97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68be8e9f, 76 },
    { 0, 0 },
    { 0x76ce8ea1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92be8ea5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22de8ead, 55 },
    { 0xa78e8ead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79e8eb3, 93 },
    { 0, 0 },
    { 0x9c5e8eb5, 98 },
    { 0xb6ee8eb6, 33 },
    { 0xc04e8eb7, 91 },
    { 0x515e8eb8, 95 },
    { 0x670e8eb6, 34 },
    { 0x5e8e8eba, 68 },
    { 0x5a5e8eba, 76 },
    { 0x6eee8ebc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85de8ec3, 88 },
    { 0x3c9e8ec3, 61 },
    { 0x812e8ec4, 95 },
    { 0xe2be8ec6, 77 },
    { 0xba6e8ec6, 33 },
    { 0xc3ee8ec7, 90 },
    { 0xc82e8ec7, 98 },
    { 0xbeae8eca, 95 },
    { 0xf2ee8ec8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2e8ed2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e8ed8, 46 },
    { 0xf2fe8ed9, 61 },
    { 0x2d4e8ed8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xed6e8edd, 55 },
    { 0x85de8ede, 90 },
    { 0x8f3e8edd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ce8ee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fe8ee7, 99 },
    { 0xb5e8ee8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x574e8eeb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ae8ef3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7de8ef7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dce8efa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e8f02, 87 },
    { 0, 0 },
    { 0xef8e8f04, 90 },
    { 0xf50e8f05, 52 },
    { 0x8b7e8f06, 61 },
    { 0, 0 },
    { 0xad7e8f08, 43 },
    { 0x3e2e8f08, 46 },
    { 0, 0 },
    { 0x4e9e8f0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959e8f11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415e8f16, 95 },
    { 0x2a4e8f16, 98 },
    { 0xd79e8f18, 76 },
    { 0x245e8f17, 23 },
    { 0x80ce8f1a, 90 },
    { 0x89be8f1b, 90 },
    { 0xfa8e8f1c, 90 },
    { 0xd2de8f1c, 99 },
    { 0x28e8f1e, 93 },
    { 0x346e8f1f, 90 },
    { 0x261e8f20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71de8f26, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbe8f29, 23 },
    { 0xb8e8f2a, 23 },
    { 0x7f7e8f2a, 95 },
    { 0xd45e8f2c, 65 },
    { 0xcb5e8f2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0e8f32, 77 },
    { 0x856e8f33, 69 },
    { 0x433e8f34, 55 },
    { 0x73ae8f33, 23 },
    { 0xb4fe8f36, 87 },
    { 0, 0 },
    { 0xce1e8f38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158e8f3e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01e8f47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509e8f4d, 61 },
    { 0, 0 },
    { 0x382e8f4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37e8f53, 90 },
    { 0x65e8f53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x511e8f5e, 77 },
    { 0, 0 },
    { 0xaace8f60, 78 },
    { 0xf6ee8f61, 17 },
    { 0, 0 },
    { 0x358e8f63, 23 },
    { 0, 0 },
    { 0x33e8f65, 23 },
    { 0, 0 },
    { 0xe2de8f67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ce8f6b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261e8f6f, 55 },
    { 0x42de8f70, 87 },
    { 0xbde8f71, 76 },
    { 0x146e8f72, 38 },
    { 0x377e8f73, 90 },
    { 0x89ce8f71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0e8f7a, 95 },
    { 0, 0 },
    { 0xdc4e8f7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02e8f7f, 43 },
    { 0x98e8f80, 33 },
    { 0x8b0e8f81, 81 },
    { 0x943e8f80, 84 },
    { 0xdd0e8f7f, 88 },
    { 0xa98e8f84, 33 },
    { 0x3b9e8f85, 84 },
    { 0xce3e8f86, 95 },
    { 0x5dce8f87, 85 },
    { 0x1c7e8f87, 23 },
    { 0xbb5e8f89, 95 },
    { 0x805e8f8a, 87 },
    { 0x10fe8f8b, 33 },
    { 0x775e8f86, 23 },
    { 0, 0 },
    { 0xa0ee8f8e, 52 },
    { 0x1d8e8f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95e8f96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfe8f99, 76 },
    { 0, 0 },
    { 0xa45e8f9b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98e8fa2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ee8fa5, 23 },
    { 0x75ee8fa5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123e8faa, 55 },
    { 0x8a2e8fab, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63e8fae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6e8fb2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x21e8fb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8e8fba, 93 },
    { 0x40ce8fbb, 76 },
    { 0xe26e8fbb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcee8fbf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92e8fc2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dee8fc7, 91 },
    { 0, 0 },
    { 0x865e8fc9, 90 },
    { 0, 0 },
    { 0x880e8fcb, 46 },
    { 0, 0 },
    { 0x763e8fcd, 85 },
    { 0x1bee8fce, 99 },
    { 0xd28e8fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe44e8fd3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1e8fd6, 23 },
    { 0xee8e8fd7, 98 },
    { 0x482e8fd6, 52 },
    { 0x404e8fd7, 23 },
    { 0x45ce8fda, 91 },
    { 0x593e8fdb, 34 },
    { 0x3e8e8fda, 95 },
    { 0x869e8fdd, 53 },
    { 0x9efe8fde, 84 },
    { 0xf03e8fde, 61 },
    { 0, 0 },
    { 0xc7ee8fe1, 33 },
    { 0xed7e8fe2, 95 },
    { 0x46ce8fe3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe8fe7, 61 },
    { 0xd88e8fe8, 95 },
    { 0x82ae8fe9, 43 },
    { 0x610e8fea, 61 },
    { 0x2bee8feb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426e8fef, 61 },
    { 0x9a3e8fef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x247e8ff3, 81 },
    { 0xe84e8ff4, 99 },
    { 0, 0 },
    { 0x7e8e8ff6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ae8ffa, 81 },
    { 0xc05e8ffb, 23 },
    { 0x8c8e8ffa, 98 },
    { 0x37de8ffd, 87 },
    { 0x40ce8ffe, 61 },
    { 0x315e8fff, 61 },
    { 0x68ae9000, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x488e9003, 46 },
    { 0, 0 },
    { 0xad5e9005, 85 },
    { 0x44fe9006, 33 },
    { 0, 0 },
    { 0x8cbe9008, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2e9011, 90 },
    { 0xde7e9012, 23 },
    { 0xa0be9013, 23 },
    { 0x38fe9014, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91e9018, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fe901e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39e9021, 23 },
    { 0xeae9021, 99 },
    { 0x237e9022, 76 },
    { 0x6e1e9022, 77 },
    { 0xa2ae9022, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51de902d, 90 },
    { 0x8cfe902d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cde9034, 78 },
    { 0, 0 },
    { 0xd53e9036, 77 },
    { 0x80ce9037, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x21de903a, 87 },
    { 0, 0 },
    { 0x73ee903c, 38 },
    { 0x5a4e903d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14de9043, 33 },
    { 0, 0 },
    { 0xb09e9045, 93 },
    { 0, 0 },
    { 0xa1ee9047, 33 },
    { 0xf6ae9048, 23 },
    { 0x965e9048, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189e904e, 84 },
    { 0x1abe904e, 90 },
    { 0xa14e904e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefe9053, 95 },
    { 0x95ce9054, 61 },
    { 0, 0 },
    { 0xcb0e9056, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ee905d, 90 },
    { 0x78de905e, 61 },
    { 0, 0 },
    { 0xefe9060, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e9064, 65 },
    { 0x278e9064, 90 },
    { 0x725e9066, 90 },
    { 0, 0 },
    { 0xfe1e9068, 33 },
    { 0xa96e9068, 33 },
    { 0xb57e9069, 34 },
    { 0, 0 },
    { 0x3dbe906c, 34 },
    { 0x83be906d, 90 },
    { 0xe33e906e, 33 },
    { 0, 0 },
    { 0xb1de9070, 33 },
    { 0, 0 },
    { 0x37e9072, 55 },
    { 0x389e9072, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7e9076, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60e907c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dce907f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6fe9084, 33 },
    { 0, 0 },
    { 0x43de9086, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bee908b, 33 },
    { 0xab3e908b, 93 },
    { 0x96e908d, 98 },
    { 0xb19e908c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86e9093, 61 },
    { 0xc9be9093, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23e9099, 43 },
    { 0x7a2e909a, 76 },
    { 0x55e909a, 99 },
    { 0, 0 },
    { 0x626e909d, 46 },
    { 0x5bde909e, 23 },
    { 0xf06e909f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cfe90a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x165e90aa, 33 },
    { 0, 0 },
    { 0x42fe90ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29e90b1, 34 },
    { 0x125e90b1, 33 },
    { 0x38ce90b1, 95 },
    { 0x80be90b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ee90bd, 17 },
    { 0x574e90be, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59e90c3, 88 },
    { 0x98fe90c4, 53 },
    { 0xae0e90c5, 64 },
    { 0x7c2e90c5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae90ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ee90ce, 98 },
    { 0x8f8e90cf, 28 },
    { 0xb4ee90cf, 98 },
    { 0xb3ae90d0, 91 },
    { 0xade90d1, 95 },
    { 0x3bde90d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fe90d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bae90dd, 76 },
    { 0, 0 },
    { 0x58ce90df, 90 },
    { 0xc5ce90df, 76 },
    { 0x91fe90e1, 93 },
    { 0x2b3e90e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9be90e5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19e90ea, 98 },
    { 0, 0 },
    { 0xc23e90ec, 55 },
    { 0, 0 },
    { 0x496e90ee, 33 },
    { 0x8b1e90ee, 36 },
    { 0x6b1e90f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45e90f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7e90f9, 63 },
    { 0, 0 },
    { 0xe94e90fb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19e90ff, 23 },
    { 0x2d8e9100, 38 },
    { 0x239e9100, 76 },
    { 0xf54e9100, 33 },
    { 0xe0be9103, 23 },
    { 0x2dce9102, 98 },
    { 0, 0 },
    { 0xa64e9106, 95 },
    { 0x308e9106, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1e910e, 32 },
    { 0x900e910f, 33 },
    { 0xafce910f, 67 },
    { 0xf87e9111, 99 },
    { 0x9dee9111, 55 },
    { 0xf89e9113, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x216e9116, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbe911b, 90 },
    { 0x57ce911b, 61 },
    { 0xb8ae911c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ce9121, 43 },
    { 0x706e9122, 65 },
    { 0xc1be9123, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840e9129, 43 },
    { 0x3fee912a, 81 },
    { 0x6abe9129, 34 },
    { 0x38ce912c, 87 },
    { 0x9bfe912d, 55 },
    { 0xdfee912e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995e9133, 61 },
    { 0xadde9133, 95 },
    { 0xd15e9135, 61 },
    { 0x3cae9136, 23 },
    { 0x21be9133, 68 },
    { 0x2c8e9136, 76 },
    { 0xc97e9138, 90 },
    { 0xccee913a, 33 },
    { 0, 0 },
    { 0x2d2e913c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40e9144, 61 },
    { 0x4e3e9145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2e9148, 100 },
    { 0, 0 },
    { 0x8d4e914a, 51 },
    { 0x367e914b, 66 },
    { 0xe19e914b, 34 },
    { 0x9dfe914c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463e9157, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7e915b, 55 },
    { 0xc80e915b, 90 },
    { 0x963e915c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ee9161, 85 },
    { 0, 0 },
    { 0x788e9163, 75 },
    { 0, 0 },
    { 0x8d3e9165, 95 },
    { 0, 0 },
    { 0xc83e9167, 69 },
    { 0, 0 },
    { 0x18ae9169, 76 },
    { 0xa93e916a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ee916e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733e9172, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68e917a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5e917e, 100 },
    { 0xa75e917e, 34 },
    { 0xc35e9180, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e9183, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763e9187, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fae918a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3e918d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19e9192, 55 },
    { 0xaa4e9193, 100 },
    { 0xed5e9192, 61 },
    { 0, 0 },
    { 0x6c7e9196, 98 },
    { 0x604e9196, 55 },
    { 0x961e9197, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931e919c, 43 },
    { 0, 0 },
    { 0xa3ae919e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbe91a2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2de91a6, 95 },
    { 0x42be91a6, 34 },
    { 0, 0 },
    { 0x357e91a9, 61 },
    { 0xd19e91a9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749e91ae, 90 },
    { 0x83e91ae, 90 },
    { 0, 0 },
    { 0x65ae91b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47e91b4, 34 },
    { 0, 0 },
    { 0x820e91b6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76e91b9, 91 },
    { 0, 0 },
    { 0xd60e91bb, 66 },
    { 0, 0 },
    { 0x6de91bd, 46 },
    { 0xfe91bd, 55 },
    { 0, 0 },
    { 0x8fe91c0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccde91ce, 82 },
    { 0xba0e91cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x596e91d2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0e91d7, 95 },
    { 0x41e91d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7e91de, 85 },
    { 0x1e3e91df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66e91ed, 33 },
    { 0x634e91ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738e91f4, 23 },
    { 0, 0 },
    { 0xf3de91f6, 78 },
    { 0x8d9e91f7, 61 },
    { 0xd40e91f6, 33 },
    { 0, 0 },
    { 0x9d7e91fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45e9201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25de9207, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ce920d, 90 },
    { 0, 0 },
    { 0x247e920f, 61 },
    { 0xcb5e9210, 84 },
    { 0x190e9211, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc8e9217, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e921a, 23 },
    { 0x3bfe921b, 99 },
    { 0, 0 },
    { 0xde6e921d, 88 },
    { 0xd6be921e, 76 },
    { 0x1d2e921f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ae9226, 91 },
    { 0xcafe9227, 98 },
    { 0xf74e9228, 90 },
    { 0xb95e9229, 76 },
    { 0, 0 },
    { 0x617e922b, 23 },
    { 0x544e922c, 99 },
    { 0x405e922d, 61 },
    { 0xffde922e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fae9231, 95 },
    { 0, 0 },
    { 0x197e9233, 55 },
    { 0, 0 },
    { 0xd1ce9235, 61 },
    { 0, 0 },
    { 0xf8fe9237, 78 },
    { 0x3b6e9238, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8e923b, 97 },
    { 0xdfbe923c, 95 },
    { 0xb16e923d, 55 },
    { 0xf8e923c, 95 },
    { 0x965e923c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dae9242, 23 },
    { 0, 0 },
    { 0xa7de9244, 23 },
    { 0xc28e9245, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ae9248, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8e925a, 76 },
    { 0xa44e925b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5e9260, 46 },
    { 0xd0fe9261, 90 },
    { 0xfa6e9262, 55 },
    { 0xd27e9261, 33 },
    { 0, 0 },
    { 0x936e9265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9e9269, 43 },
    { 0, 0 },
    { 0x2de926b, 91 },
    { 0, 0 },
    { 0x1a4e926d, 65 },
    { 0, 0 },
    { 0x4e2e926f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66e9273, 34 },
    { 0xb32e9274, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41de9278, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4e927f, 61 },
    { 0x3a1e927f, 33 },
    { 0, 0 },
    { 0xec9e9282, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8e9287, 90 },
    { 0x6ce9287, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14e928b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67e928e, 46 },
    { 0x4a8e928e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0e9292, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6e9296, 99 },
    { 0x1c2e9297, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcece929b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452e92a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6e92a5, 46 },
    { 0x5ace92a6, 90 },
    { 0, 0 },
    { 0x3d8e92a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635e92ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ee92af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90e92b2, 23 },
    { 0xc03e92b3, 55 },
    { 0, 0 },
    { 0x6f1e92b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0e92b9, 99 },
    { 0xa1ce92ba, 17 },
    { 0x204e92bb, 90 },
    { 0x7bfe92bb, 67 },
    { 0xae0e92bc, 52 },
    { 0x97ce92bb, 23 },
    { 0x2f3e92bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779e92c9, 87 },
    { 0, 0 },
    { 0x38ae92cb, 33 },
    { 0, 0 },
    { 0xc4de92cd, 43 },
    { 0xf8de92ce, 78 },
    { 0x70ce92cf, 91 },
    { 0xb3ce92d0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844e92e3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31fe92e7, 95 },
    { 0, 0 },
    { 0x924e92e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8e92ee, 23 },
    { 0xe68e92ef, 95 },
    { 0xad1e92f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ae92f3, 76 },
    { 0, 0 },
    { 0x885e92f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x999e92f8, 90 },
    { 0xfb9e92f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdce92fd, 95 },
    { 0x43fe92fd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78e9303, 95 },
    { 0x8a4e9304, 95 },
    { 0x81fe9305, 90 },
    { 0x533e9306, 95 },
    { 0xf5ae9306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x990e930a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c8e9313, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597e9317, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6e931c, 95 },
    { 0x6a1e931c, 98 },
    { 0x5fce931d, 90 },
    { 0xd44e931f, 90 },
    { 0x45be931d, 90 },
    { 0xa8ee9321, 90 },
    { 0x6aae9322, 43 },
    { 0xd90e9323, 34 },
    { 0x82ee9324, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dae9334, 90 },
    { 0xfa1e9335, 23 },
    { 0x19ae9336, 34 },
    { 0x450e9337, 23 },
    { 0x369e9337, 90 },
    { 0x992e9339, 81 },
    { 0x3f1e933a, 87 },
    { 0, 0 },
    { 0xa52e933c, 34 },
    { 0x147e933d, 91 },
    { 0xe22e933d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ee9345, 90 },
    { 0x214e9345, 90 },
    { 0x4b6e9347, 55 },
    { 0, 0 },
    { 0xd82e9349, 23 },
    { 0, 0 },
    { 0x202e934b, 33 },
    { 0x6d6e934b, 58 },
    { 0x5d9e934b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7e9351, 61 },
    { 0, 0 },
    { 0xbb2e9353, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6e9356, 46 },
    { 0, 0 },
    { 0x50de9358, 52 },
    { 0, 0 },
    { 0x37be935a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3e935d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4e9368, 76 },
    { 0xd52e9369, 76 },
    { 0xc43e936a, 98 },
    { 0x74e9369, 90 },
    { 0x886e936c, 55 },
    { 0, 0 },
    { 0xb15e936e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1e9373, 61 },
    { 0xd65e9373, 76 },
    { 0x7dbe9375, 34 },
    { 0xdc7e9375, 68 },
    { 0x645e9377, 43 },
    { 0x7c2e9375, 76 },
    { 0xa2de9379, 100 },
    { 0, 0 },
    { 0x1b3e937b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ee937f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ce9382, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946e9388, 61 },
    { 0xb66e9389, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12e938c, 90 },
    { 0x476e938c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc91e9390, 78 },
    { 0xd47e9390, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84e9398, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aee93a0, 88 },
    { 0x8c5e93a1, 81 },
    { 0xac9e93a0, 97 },
    { 0x29e93a1, 23 },
    { 0xdf7e93a0, 33 },
    { 0x77ce93a2, 83 },
    { 0, 0 },
    { 0x35ee93a7, 23 },
    { 0x94ee93a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4e93ab, 56 },
    { 0x190e93ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6de93b2, 84 },
    { 0xd30e93b3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18e93b9, 95 },
    { 0x405e93ba, 40 },
    { 0xaa2e93bb, 98 },
    { 0x67ee93bb, 61 },
    { 0, 0 },
    { 0x5b9e93be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1e93c2, 87 },
    { 0, 0 },
    { 0x734e93c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0e93c7, 98 },
    { 0, 0 },
    { 0xac4e93c9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30e93d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1e93d5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbace93df, 81 },
    { 0xa36e93df, 46 },
    { 0xe7be93e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbe93e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4e93e8, 40 },
    { 0, 0 },
    { 0x310e93ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ee93ee, 33 },
    { 0, 0 },
    { 0xf88e93f0, 98 },
    { 0, 0 },
    { 0xfc8e93f2, 90 },
    { 0xb8ee93f3, 99 },
    { 0x374e93f4, 90 },
    { 0xcc6e93f2, 31 },
    { 0x145e93f6, 33 },
    { 0x1fae93f7, 61 },
    { 0x980e93f8, 64 },
    { 0x92ce93f9, 75 },
    { 0x3c2e93fa, 34 },
    { 0xe4de93f7, 98 },
    { 0xe65e93fb, 95 },
    { 0xef1e93fd, 36 },
    { 0xd96e93f7, 34 },
    { 0, 0 },
    { 0xcdae9400, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4e9404, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9e9408, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ae940c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1de940f, 46 },
    { 0, 0 },
    { 0xa34e9411, 68 },
    { 0, 0 },
    { 0x251e9413, 76 },
    { 0, 0 },
    { 0x12de9415, 99 },
    { 0xb98e9415, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03e941a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85e941f, 81 },
    { 0x65fe9420, 42 },
    { 0, 0 },
    { 0xa31e9422, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ae9426, 33 },
    { 0, 0 },
    { 0xa5de9428, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0e942b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aee942f, 96 },
    { 0x212e9430, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9e9433, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb87e9439, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2e943c, 90 },
    { 0, 0 },
    { 0x27ee943e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x780e9441, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51be9447, 91 },
    { 0xe62e9447, 34 },
    { 0x60ce9449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0e944c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0e9452, 93 },
    { 0, 0 },
    { 0xfcde9454, 55 },
    { 0, 0 },
    { 0x1cae9456, 84 },
    { 0xad5e9457, 87 },
    { 0xc47e9458, 98 },
    { 0x954e9459, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7e945d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38e9461, 55 },
    { 0x8aee9461, 90 },
    { 0x258e9462, 59 },
    { 0xec3e9464, 76 },
    { 0xdd9e9462, 95 },
    { 0x7a9e9466, 34 },
    { 0xd6be9466, 90 },
    { 0x89ce9468, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x742e946b, 17 },
    { 0, 0 },
    { 0xb3de946d, 93 },
    { 0xc9de946d, 61 },
    { 0xfe3e946e, 23 },
    { 0xcf0e9470, 55 },
    { 0xa48e9470, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fe947b, 33 },
    { 0x4e4e947b, 61 },
    { 0, 0 },
    { 0x746e947e, 95 },
    { 0, 0 },
    { 0x178e9480, 61 },
    { 0x33ae9480, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4e948c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285e9490, 95 },
    { 0xbd4e9491, 61 },
    { 0, 0 },
    { 0x230e9493, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2e9497, 98 },
    { 0x941e9498, 23 },
    { 0x9ce9498, 95 },
    { 0xd87e9498, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7e949f, 87 },
    { 0xa5ee94a0, 61 },
    { 0xd06e94a1, 33 },
    { 0x3f7e94a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08e94a5, 97 },
    { 0x132e94a6, 33 },
    { 0x78fe94a6, 84 },
    { 0x30ce94a8, 58 },
    { 0, 0 },
    { 0xc46e94aa, 90 },
    { 0x48ce94ab, 61 },
    { 0, 0 },
    { 0xb2de94ad, 43 },
    { 0, 0 },
    { 0x81fe94af, 34 },
    { 0, 0 },
    { 0xe76e94b1, 33 },
    { 0, 0 },
    { 0xb3ee94b3, 98 },
    { 0x6ede94b4, 61 },
    { 0, 0 },
    { 0xaf5e94b6, 61 },
    { 0, 0 },
    { 0xbece94b8, 33 },
    { 0x750e94b9, 90 },
    { 0x45e94ba, 65 },
    { 0x466e94bb, 43 },
    { 0xc6fe94bb, 33 },
    { 0x864e94bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1e94c0, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46e94c3, 34 },
    { 0, 0 },
    { 0x8d5e94c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183e94ca, 55 },
    { 0x2bbe94ca, 95 },
    { 0, 0 },
    { 0x9c4e94cd, 17 },
    { 0xb0de94ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbe94d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aee94dd, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x140e94e1, 100 },
    { 0xdcae94e2, 23 },
    { 0xb8fe94e3, 66 },
    { 0, 0 },
    { 0x8eae94e5, 87 },
    { 0x3dbe94e6, 95 },
    { 0xa2de94e7, 33 },
    { 0xd92e94e8, 98 },
    { 0x8fee94e9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53e94ee, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186e94f3, 52 },
    { 0x3ae94f4, 61 },
    { 0x805e94f4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ce94f9, 38 },
    { 0xb09e94f9, 23 },
    { 0, 0 },
    { 0x9cbe94fc, 56 },
    { 0xf93e94fd, 87 },
    { 0x501e94fe, 55 },
    { 0xaae94fd, 69 },
    { 0x573e94fe, 87 },
    { 0xa65e94ff, 33 },
    { 0, 0 },
    { 0xfe7e9503, 61 },
    { 0x113e9503, 61 },
    { 0x85ce9505, 61 },
    { 0, 0 },
    { 0x16ee9507, 33 },
    { 0, 0 },
    { 0x432e9509, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x640e950e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883e9512, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7e9517, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75e951a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ce951d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7be9521, 98 },
    { 0x805e9522, 55 },
    { 0x1a8e9523, 90 },
    { 0x52fe9524, 66 },
    { 0xbdfe9525, 61 },
    { 0x8f1e9521, 90 },
    { 0, 0 },
    { 0x416e9528, 46 },
    { 0xd77e9528, 61 },
    { 0, 0 },
    { 0x3dfe952b, 75 },
    { 0xbc2e952c, 84 },
    { 0x115e952d, 61 },
    { 0x97de952d, 75 },
    { 0x416e952f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50e9532, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ce9539, 98 },
    { 0x951e953a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54e953f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45be9544, 76 },
    { 0x8fce9544, 46 },
    { 0xe8ae9544, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8e954a, 85 },
    { 0x5c2e954b, 78 },
    { 0x723e954c, 23 },
    { 0, 0 },
    { 0xc57e954e, 61 },
    { 0x928e954e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4e9552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0e9558, 87 },
    { 0, 0 },
    { 0xcbe955a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6e955e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fe9565, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9e956d, 33 },
    { 0, 0 },
    { 0x193e956f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14e9575, 52 },
    { 0xcf4e9576, 17 },
    { 0x9c4e9576, 95 },
    { 0xb19e9578, 84 },
    { 0, 0 },
    { 0x19ee957a, 100 },
    { 0x33fe957b, 23 },
    { 0x54ee957c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ade9580, 52 },
    { 0x570e9581, 23 },
    { 0xec0e9581, 33 },
    { 0xc1ee9583, 77 },
    { 0xb82e9583, 43 },
    { 0x81ae9585, 43 },
    { 0x952e9586, 61 },
    { 0x599e9587, 66 },
    { 0xf91e9588, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59be958d, 43 },
    { 0x538e958e, 46 },
    { 0x1a7e958f, 38 },
    { 0x2f8e958f, 61 },
    { 0x50e9590, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aae9597, 17 },
    { 0xf8de9597, 34 },
    { 0xb84e9599, 33 },
    { 0, 0 },
    { 0xb3e959b, 90 },
    { 0, 0 },
    { 0x977e959d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x378e95a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4e95a3, 98 },
    { 0xc7de95a4, 52 },
    { 0x1dde95a4, 52 },
    { 0, 0 },
    { 0x6f1e95a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde95aa, 91 },
    { 0x7ece95ab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xecae95ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e95b7, 90 },
    { 0xf15e95b8, 99 },
    { 0xf39e95b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdece95bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950e95c1, 55 },
    { 0xfade95c1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbe95c7, 91 },
    { 0, 0 },
    { 0xdb9e95c9, 77 },
    { 0x6c7e95ca, 68 },
    { 0x5e6e95c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x134e95ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86e95d3, 61 },
    { 0, 0 },
    { 0x798e95d5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x285e95d8, 90 },
    { 0x34fe95d9, 90 },
    { 0x8e9e95d8, 23 },
    { 0x3a7e95db, 23 },
    { 0x692e95dc, 85 },
    { 0xf9ee95dd, 99 },
    { 0, 0 },
    { 0x75be95df, 85 },
    { 0x207e95df, 95 },
    { 0xf01e95e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523e95e8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12e95ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75e95f1, 87 },
    { 0, 0 },
    { 0x4ce95f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x653e95f6, 43 },
    { 0xe95e95f6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02e95fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9e95fe, 87 },
    { 0x702e95fe, 33 },
    { 0, 0 },
    { 0x174e9601, 61 },
    { 0, 0 },
    { 0x3e9603, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfe960a, 61 },
    { 0, 0 },
    { 0xce7e960c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ce9611, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4e961c, 90 },
    { 0xda0e961d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659e9623, 23 },
    { 0, 0 },
    { 0xc6fe9625, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3be9632, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x412e9635, 90 },
    { 0, 0 },
    { 0x64ce9637, 88 },
    { 0x8d4e9638, 17 },
    { 0x41de9639, 23 },
    { 0x47de963a, 34 },
    { 0, 0 },
    { 0xa5be963c, 93 },
    { 0x4ee963d, 98 },
    { 0x8be963d, 77 },
    { 0xfbe963f, 95 },
    { 0xc28e963c, 61 },
    { 0, 0 },
    { 0xf39e9642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ee9649, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8e964d, 33 },
    { 0x711e964d, 36 },
    { 0x2f1e964f, 98 },
    { 0x786e964d, 98 },
    { 0, 0 },
    { 0xe13e9652, 90 },
    { 0xe76e9653, 76 },
    { 0, 0 },
    { 0xc8e9655, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643e965a, 61 },
    { 0xf50e965a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0e965e, 61 },
    { 0x6aae965e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15e9662, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06e9667, 95 },
    { 0xfcae9668, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0e966c, 98 },
    { 0x443e966c, 61 },
    { 0x253e966e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7e9672, 61 },
    { 0, 0 },
    { 0xa90e9674, 90 },
    { 0xf7ae9675, 85 },
    { 0x751e9675, 33 },
    { 0xcf0e9677, 17 },
    { 0xebfe9678, 34 },
    { 0x9f1e9675, 61 },
    { 0x729e967a, 90 },
    { 0xe94e967a, 61 },
    { 0x47e967c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ae9680, 55 },
    { 0xe59e9681, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01e968a, 61 },
    { 0, 0 },
    { 0xf70e968c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ee968f, 95 },
    { 0xbae9690, 33 },
    { 0, 0 },
    { 0x849e9692, 93 },
    { 0xc04e9693, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1e96a3, 90 },
    { 0, 0 },
    { 0xc71e96a5, 88 },
    { 0x294e96a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x227e96a9, 95 },
    { 0x725e96a9, 75 },
    { 0xac9e96ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ee96ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2e96b1, 76 },
    { 0, 0 },
    { 0x449e96b3, 91 },
    { 0x109e96b3, 61 },
    { 0x809e96b5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0be96bb, 61 },
    { 0xea1e96bc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1e96c3, 43 },
    { 0, 0 },
    { 0xd06e96c5, 95 },
    { 0, 0 },
    { 0xda8e96c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263e96cd, 98 },
    { 0x42e96cd, 98 },
    { 0, 0 },
    { 0x692e96d0, 33 },
    { 0, 0 },
    { 0x8a7e96d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93e96d6, 88 },
    { 0x99fe96d6, 55 },
    { 0xbd5e96d8, 33 },
    { 0xc38e96d8, 98 },
    { 0xe95e96d6, 66 },
    { 0, 0 },
    { 0xb4ce96dc, 98 },
    { 0xf81e96dc, 84 },
    { 0x8bde96dd, 55 },
    { 0xf95e96df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39fe96e3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01e96e9, 38 },
    { 0, 0 },
    { 0x60fe96eb, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x734e96ee, 85 },
    { 0x4e7e96ef, 90 },
    { 0xb7ae96f0, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45e96f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494e96f7, 46 },
    { 0x24de96f8, 98 },
    { 0xfa7e96f8, 95 },
    { 0, 0 },
    { 0xc22e96fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719e96ff, 95 },
    { 0xe99e9700, 90 },
    { 0, 0 },
    { 0x5e8e9702, 90 },
    { 0, 0 },
    { 0x83ae9704, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ae9707, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7e970a, 90 },
    { 0, 0 },
    { 0x6fde970c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7e9711, 95 },
    { 0, 0 },
    { 0x408e9713, 90 },
    { 0, 0 },
    { 0xa67e9715, 34 },
    { 0, 0 },
    { 0x5a6e9717, 23 },
    { 0x55ce9718, 90 },
    { 0x998e9719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5e971c, 78 },
    { 0, 0 },
    { 0x517e971e, 33 },
    { 0x809e971e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7e9723, 46 },
    { 0, 0 },
    { 0x1fce9725, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24e9728, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8e972f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbe9732, 95 },
    { 0, 0 },
    { 0xcb7e9734, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24e9739, 97 },
    { 0x6e9e9739, 85 },
    { 0, 0 },
    { 0xadee973c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadde9741, 23 },
    { 0x7bee9742, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbe9747, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9e974e, 46 },
    { 0x551e974f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbe9754, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2e975a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8e9760, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ae976c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43e9773, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x429e9776, 23 },
    { 0x87ae9777, 33 },
    { 0x4d8e9778, 33 },
    { 0, 0 },
    { 0x5cde977a, 95 },
    { 0x8a6e977a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ee977f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5e9782, 46 },
    { 0xbdbe9783, 84 },
    { 0xf58e9784, 99 },
    { 0x5f4e9784, 61 },
    { 0, 0 },
    { 0x197e9787, 33 },
    { 0x4d9e9788, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73de978c, 61 },
    { 0, 0 },
    { 0xa19e978e, 61 },
    { 0x535e978f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0e9793, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbe9797, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88e979e, 85 },
    { 0xe31e979f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4e97a4, 75 },
    { 0x68ce97a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeee97a9, 33 },
    { 0x9c9e97a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de97b6, 95 },
    { 0, 0 },
    { 0x9f2e97b8, 23 },
    { 0xb4de97b9, 95 },
    { 0, 0 },
    { 0x2f8e97bb, 97 },
    { 0x6a5e97bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783e97c4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ce97c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74e97cb, 34 },
    { 0, 0 },
    { 0xfebe97cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8e97d2, 93 },
    { 0, 0 },
    { 0xc18e97d4, 98 },
    { 0, 0 },
    { 0xe5be97d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd43e97da, 33 },
    { 0xb3fe97db, 88 },
    { 0x9efe97da, 61 },
    { 0xe8be97dd, 90 },
    { 0x948e97de, 43 },
    { 0xceee97df, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468e97e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ce97e8, 93 },
    { 0x285e97e9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2e97ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534e97f0, 27 },
    { 0xc96e97f1, 76 },
    { 0x799e97f2, 90 },
    { 0x30de97f2, 55 },
    { 0xc5ae97f4, 23 },
    { 0x99e97f4, 95 },
    { 0, 0 },
    { 0x10ee97f7, 98 },
    { 0xf33e97f7, 33 },
    { 0, 0 },
    { 0x5abe97fa, 91 },
    { 0x878e97fb, 33 },
    { 0xc7fe97fb, 33 },
    { 0x7e9e97fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2e9801, 61 },
    { 0xf43e9802, 61 },
    { 0x478e9803, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9e9809, 61 },
    { 0x66ee9809, 90 },
    { 0xe67e980b, 98 },
    { 0x370e980c, 33 },
    { 0x649e980d, 98 },
    { 0xc9e980c, 90 },
    { 0xcd2e980e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287e9813, 87 },
    { 0xf09e9814, 33 },
    { 0xb64e9813, 90 },
    { 0, 0 },
    { 0x348e9817, 40 },
    { 0x2fbe9817, 34 },
    { 0, 0 },
    { 0x645e981a, 90 },
    { 0, 0 },
    { 0x687e981c, 55 },
    { 0x934e981c, 61 },
    { 0x7f2e981c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356e9825, 61 },
    { 0x2d0e9826, 23 },
    { 0x2b0e9825, 43 },
    { 0x7ede9828, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcfe982b, 95 },
    { 0xbade982c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21e9830, 61 },
    { 0x3d5e9830, 23 },
    { 0x9e9832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ae9835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccde9838, 23 },
    { 0x179e9839, 99 },
    { 0xfbfe9839, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6e983e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9844, 55 },
    { 0xd90e9845, 81 },
    { 0x89fe9845, 76 },
    { 0, 0 },
    { 0x3fae9848, 95 },
    { 0x2b8e9849, 100 },
    { 0xbf5e984a, 33 },
    { 0, 0 },
    { 0x7f2e984c, 33 },
    { 0xb63e984d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4e9851, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ee9858, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69be9860, 61 },
    { 0x508e9860, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddae9865, 90 },
    { 0, 0 },
    { 0xf03e9867, 45 },
    { 0x702e9867, 90 },
    { 0xfcbe9869, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ce986c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cce9873, 81 },
    { 0, 0 },
    { 0x171e9875, 90 },
    { 0x770e9876, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfffe987e, 95 },
    { 0, 0 },
    { 0xd51e9880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659e9886, 55 },
    { 0xb74e9887, 84 },
    { 0, 0 },
    { 0xa9be9889, 33 },
    { 0xb79e988a, 23 },
    { 0xf6ce988b, 33 },
    { 0x87ae988b, 67 },
    { 0, 0 },
    { 0xd73e988e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe9891, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x984e9894, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01e9897, 66 },
    { 0xc9ee9898, 95 },
    { 0x823e9899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393e98a0, 55 },
    { 0x2cce98a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fe98a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25e98a8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5e98ab, 84 },
    { 0xfcae98ac, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892e98b4, 93 },
    { 0x4ede98b5, 52 },
    { 0, 0 },
    { 0xe3be98b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa1e98bb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x642e98be, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6e98c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ae98ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30e98d3, 90 },
    { 0, 0 },
    { 0x6f1e98d5, 17 },
    { 0xec1e98d6, 98 },
    { 0x7bee98d7, 76 },
    { 0x820e98d8, 55 },
    { 0xd45e98d9, 78 },
    { 0x262e98d9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fce98dd, 95 },
    { 0xf73e98de, 23 },
    { 0x3d9e98df, 90 },
    { 0, 0 },
    { 0xfabe98e1, 52 },
    { 0x698e98e1, 19 },
    { 0x656e98e1, 66 },
    { 0xbe98e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x787e98e7, 34 },
    { 0, 0 },
    { 0x5afe98e9, 84 },
    { 0x855e98ea, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85e98ee, 95 },
    { 0, 0 },
    { 0xca5e98f0, 33 },
    { 0x366e98f0, 87 },
    { 0x838e98f2, 55 },
    { 0x9b1e98f1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26e98f6, 84 },
    { 0x891e98f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4e98fc, 76 },
    { 0, 0 },
    { 0xaf9e98fe, 98 },
    { 0, 0 },
    { 0x89fe9900, 55 },
    { 0, 0 },
    { 0x4aae9902, 34 },
    { 0, 0 },
    { 0xb54e9904, 76 },
    { 0x499e9905, 90 },
    { 0, 0 },
    { 0xbc2e9907, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cce990d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881e9913, 67 },
    { 0, 0 },
    { 0xc0be9915, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbde9919, 23 },
    { 0x541e991a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x658e991d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ae9921, 32 },
    { 0x9d6e9922, 98 },
    { 0, 0 },
    { 0x87e9924, 61 },
    { 0, 0 },
    { 0x2e3e9926, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9929, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bce992c, 95 },
    { 0x3c6e992c, 95 },
    { 0x520e992e, 90 },
    { 0xbcce992d, 100 },
    { 0x9dfe9930, 90 },
    { 0xb5be9931, 76 },
    { 0, 0 },
    { 0x175e9933, 61 },
    { 0x198e9934, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x448e9937, 96 },
    { 0x575e9938, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37e993d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x661e9940, 33 },
    { 0, 0 },
    { 0x6e3e9942, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4e9945, 24 },
    { 0x178e9946, 43 },
    { 0, 0 },
    { 0x7d5e9948, 23 },
    { 0xa41e9949, 99 },
    { 0x793e994a, 90 },
    { 0x48fe9948, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ece994f, 23 },
    { 0x5ae994f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68e9954, 33 },
    { 0x51de9955, 65 },
    { 0x9e7e9954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8e9959, 61 },
    { 0x8eee995a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ce995f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7e9963, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922e9969, 61 },
    { 0x38e996a, 90 },
    { 0xde8e9969, 33 },
    { 0xe65e996a, 90 },
    { 0x405e996d, 17 },
    { 0x2ebe996d, 66 },
    { 0x93e996d, 34 },
    { 0x410e996d, 69 },
    { 0x20de996f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1e9978, 55 },
    { 0, 0 },
    { 0x6dfe997a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112e9988, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0e998e, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcde9998, 90 },
    { 0, 0 },
    { 0x49ee999a, 61 },
    { 0xec2e999b, 61 },
    { 0, 0 },
    { 0x753e999d, 53 },
    { 0x8a4e999e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1e99a4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55e99aa, 34 },
    { 0, 0 },
    { 0xd4ae99ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08e99af, 55 },
    { 0, 0 },
    { 0x166e99b1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x277e99b4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987e99bc, 46 },
    { 0xfffe99bc, 48 },
    { 0xb0ce99be, 90 },
    { 0x67ce99bf, 17 },
    { 0xd84e99bf, 61 },
    { 0x3afe99c1, 23 },
    { 0, 0 },
    { 0x83fe99c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdede99c6, 93 },
    { 0x110e99c7, 67 },
    { 0xc1ae99c8, 55 },
    { 0x5dae99c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08e99ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78fe99d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3be99db, 66 },
    { 0xc6e99dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e99df, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fe99e2, 90 },
    { 0xe29e99e3, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469e99e8, 61 },
    { 0xface99e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1e99ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bde99f2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaece99f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x88e99f9, 95 },
    { 0x9e6e99fa, 100 },
    { 0, 0 },
    { 0xe63e99fc, 33 },
    { 0x650e99fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ce9a01, 55 },
    { 0, 0 },
    { 0xd55e9a03, 76 },
    { 0x5e3e9a03, 33 },
    { 0x1dfe9a04, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2e9a0c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dde9a12, 46 },
    { 0xd38e9a13, 33 },
    { 0x841e9a13, 23 },
    { 0xf4e9a12, 33 },
    { 0xc33e9a15, 46 },
    { 0xc52e9a15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfe9a1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7e9a1e, 40 },
    { 0, 0 },
    { 0x672e9a20, 80 },
    { 0xaf3e9a21, 77 },
    { 0, 0 },
    { 0xa6ee9a23, 95 },
    { 0, 0 },
    { 0xe86e9a25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629e9a30, 90 },
    { 0, 0 },
    { 0x352e9a32, 61 },
    { 0x4b7e9a33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269e9a3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09e9a42, 99 },
    { 0xa02e9a42, 85 },
    { 0x164e9a43, 90 },
    { 0x13ee9a44, 90 },
    { 0x429e9a46, 90 },
    { 0x30fe9a47, 33 },
    { 0x822e9a48, 33 },
    { 0, 0 },
    { 0xb2fe9a4a, 55 },
    { 0xcf8e9a4b, 84 },
    { 0xbd9e9a4b, 33 },
    { 0, 0 },
    { 0x11ce9a4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4e9a54, 33 },
    { 0x5c7e9a54, 43 },
    { 0xd38e9a54, 34 },
    { 0x29de9a57, 75 },
    { 0xf62e9a58, 95 },
    { 0, 0 },
    { 0x825e9a5a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ee9a5d, 94 },
    { 0, 0 },
    { 0x716e9a5f, 61 },
    { 0, 0 },
    { 0x349e9a61, 78 },
    { 0, 0 },
    { 0x19de9a63, 23 },
    { 0xb36e9a64, 90 },
    { 0xf7ee9a64, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x597e9a69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9e9a6c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3e9a70, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5e9a75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x968e9a78, 76 },
    { 0xabee9a79, 55 },
    { 0x82fe9a7a, 66 },
    { 0x6cee9a7b, 76 },
    { 0xd1ae9a7c, 61 },
    { 0x63ce9a7c, 98 },
    { 0, 0 },
    { 0xffee9a7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fce9a82, 61 },
    { 0xeede9a82, 33 },
    { 0, 0 },
    { 0x231e9a85, 100 },
    { 0x41fe9a85, 98 },
    { 0xc5ee9a87, 64 },
    { 0x622e9a88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9e9a8b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ee9a8e, 22 },
    { 0, 0 },
    { 0xe2e9a90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ee9a93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaee9a9a, 55 },
    { 0xc0ce9a9a, 85 },
    { 0xf5be9a9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ae9aa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffe9aa6, 93 },
    { 0x4be9aa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fe9aac, 38 },
    { 0xb94e9aad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34e9ab4, 43 },
    { 0x10fe9ab5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90e9ab8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x320e9abe, 95 },
    { 0xd7de9abe, 85 },
    { 0x81de9ac0, 98 },
    { 0x6dae9ac1, 98 },
    { 0x518e9ac1, 55 },
    { 0xf63e9ac0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2e9ac7, 36 },
    { 0, 0 },
    { 0xecee9ac9, 81 },
    { 0, 0 },
    { 0x357e9acb, 95 },
    { 0xd26e9acc, 81 },
    { 0, 0 },
    { 0x1afe9ace, 90 },
    { 0x488e9ace, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ace9ad4, 98 },
    { 0xbfee9ad4, 95 },
    { 0xdbfe9ad6, 61 },
    { 0x144e9ad7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9de9adb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x986e9ade, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ae9ae5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cde9ae9, 43 },
    { 0x9ce9aea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cde9af0, 90 },
    { 0x41ae9af0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389e9af5, 90 },
    { 0, 0 },
    { 0x55de9af7, 90 },
    { 0xadde9af7, 43 },
    { 0, 0 },
    { 0x57ae9afa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bce9afd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ce9b01, 80 },
    { 0, 0 },
    { 0x95be9b03, 76 },
    { 0x683e9b04, 61 },
    { 0xb79e9b05, 76 },
    { 0x6c7e9b05, 61 },
    { 0x370e9b07, 91 },
    { 0, 0 },
    { 0x8fbe9b09, 43 },
    { 0, 0 },
    { 0xe69e9b0b, 90 },
    { 0, 0 },
    { 0xb4de9b0d, 52 },
    { 0, 0 },
    { 0x2fbe9b0f, 23 },
    { 0x93be9b10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aae9b1b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ace9b20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77e9b23, 49 },
    { 0x150e9b24, 95 },
    { 0xbcde9b24, 34 },
    { 0, 0 },
    { 0x3fde9b27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ede9b2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x40be9b32, 95 },
    { 0, 0 },
    { 0x1b4e9b34, 76 },
    { 0, 0 },
    { 0x9e8e9b36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18e9b39, 33 },
    { 0x56ee9b3a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620e9b3e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dde9b42, 87 },
    { 0, 0 },
    { 0x303e9b44, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbade9b47, 23 },
    { 0, 0 },
    { 0xe55e9b49, 100 },
    { 0xe1ee9b4a, 80 },
    { 0, 0 },
    { 0xf02e9b4c, 23 },
    { 0, 0 },
    { 0xe13e9b4e, 33 },
    { 0, 0 },
    { 0x9c9e9b50, 95 },
    { 0, 0 },
    { 0x2dde9b52, 34 },
    { 0xcd4e9b53, 84 },
    { 0x407e9b54, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x802e9b57, 78 },
    { 0x8cbe9b58, 66 },
    { 0xc28e9b59, 61 },
    { 0, 0 },
    { 0x9f4e9b5b, 61 },
    { 0, 0 },
    { 0xecbe9b5d, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde9b61, 23 },
    { 0x14ee9b62, 98 },
    { 0xee4e9b63, 95 },
    { 0x9a9e9b63, 43 },
    { 0, 0 },
    { 0x49fe9b66, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1e9b6b, 33 },
    { 0x18de9b6b, 55 },
    { 0, 0 },
    { 0x20ae9b6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ee9b75, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ae9b78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5e9b7c, 98 },
    { 0, 0 },
    { 0xdaee9b7e, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ae9b83, 61 },
    { 0, 0 },
    { 0x662e9b85, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73e9b88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9b8d, 95 },
    { 0x62ae9b8e, 98 },
    { 0, 0 },
    { 0x894e9b90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755e9b94, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830e9b9b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75e9b9e, 99 },
    { 0x27be9b9f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86e9ba2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cce9ba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbafe9ba8, 33 },
    { 0, 0 },
    { 0xe89e9baa, 98 },
    { 0x888e9baa, 90 },
    { 0x3f9e9bab, 61 },
    { 0, 0 },
    { 0x9b0e9bae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97e9bb2, 90 },
    { 0x875e9bb3, 43 },
    { 0x571e9bb2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7e9bb7, 76 },
    { 0x29ce9bb7, 61 },
    { 0x1c2e9bb9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fe9bbc, 55 },
    { 0, 0 },
    { 0xc42e9bbe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2e9bc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0e9bc7, 52 },
    { 0, 0 },
    { 0x2fce9bc9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104e9bce, 98 },
    { 0x740e9bce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2be9bdb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ce9be1, 43 },
    { 0x602e9be2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18e9be9, 90 },
    { 0x112e9bea, 33 },
    { 0x2d3e9bea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43e9bf0, 90 },
    { 0xd0ce9bf1, 61 },
    { 0x828e9bf1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64e9bf7, 88 },
    { 0xfb4e9bf8, 23 },
    { 0xf7ee9bf9, 23 },
    { 0x995e9bfa, 88 },
    { 0x52ae9bfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8e9bfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94e9c03, 87 },
    { 0, 0 },
    { 0xe81e9c05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cae9c0b, 95 },
    { 0xae9c0c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95e9c14, 99 },
    { 0x51be9c14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ce9c1b, 23 },
    { 0, 0 },
    { 0xe5de9c1d, 91 },
    { 0x560e9c1e, 43 },
    { 0x9dce9c1d, 76 },
    { 0xdbee9c20, 98 },
    { 0x504e9c20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3e9c2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7e9c30, 92 },
    { 0xbdae9c30, 90 },
    { 0x1a3e9c32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebee9c36, 98 },
    { 0x148e9c36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0e9c40, 43 },
    { 0x182e9c41, 23 },
    { 0x408e9c41, 33 },
    { 0xf84e9c42, 33 },
    { 0x27ee9c41, 91 },
    { 0, 0 },
    { 0x68fe9c46, 91 },
    { 0xf1e9c47, 23 },
    { 0x4de9c48, 23 },
    { 0, 0 },
    { 0xf4be9c4a, 91 },
    { 0x6aee9c4b, 69 },
    { 0x63fe9c4b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388e9c52, 90 },
    { 0x1c4e9c53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3e9c5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe9c60, 61 },
    { 0x55de9c61, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ce9c64, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68e9c68, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeede9c6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ae9c70, 93 },
    { 0xd65e9c71, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6e9c74, 23 },
    { 0, 0 },
    { 0xc04e9c76, 93 },
    { 0x8b7e9c77, 98 },
    { 0xa73e9c78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfde9c80, 33 },
    { 0x7f2e9c80, 99 },
    { 0xebfe9c81, 96 },
    { 0x984e9c81, 70 },
    { 0xae5e9c84, 43 },
    { 0xdfce9c84, 36 },
    { 0, 0 },
    { 0x9cae9c87, 95 },
    { 0x844e9c87, 98 },
    { 0x1b3e9c87, 60 },
    { 0xf44e9c88, 96 },
    { 0xcf7e9c87, 61 },
    { 0, 0 },
    { 0x328e9c8d, 8 },
    { 0x50e9c8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42e9c92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ce9c99, 99 },
    { 0xf78e9c9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770e9c9f, 38 },
    { 0, 0 },
    { 0xd4ce9ca1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f3e9ca4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ce9cae, 67 },
    { 0x293e9caf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ae9cb5, 90 },
    { 0x288e9cb5, 61 },
    { 0, 0 },
    { 0x224e9cb8, 33 },
    { 0x254e9cb9, 61 },
    { 0xbd4e9cb9, 95 },
    { 0xd03e9cbb, 67 },
    { 0x2efe9cb8, 90 },
    { 0xb75e9cbd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8e9cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6e9ccb, 34 },
    { 0xf1ee9ccc, 90 },
    { 0xf9ce9ccd, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ee9cd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ae9cd5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbe9cd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45e9cdb, 90 },
    { 0xab3e9cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335e9ce4, 61 },
    { 0x91ce9ce5, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ae9ce9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ee9cef, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3e9cf2, 100 },
    { 0x2fee9cf3, 98 },
    { 0, 0 },
    { 0x4de9cf5, 88 },
    { 0x3a4e9cf6, 76 },
    { 0xe1ae9cf6, 38 },
    { 0, 0 },
    { 0xcb0e9cf9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79e9cfd, 61 },
    { 0xd2ce9cfe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ee9d03, 17 },
    { 0x5c9e9d04, 100 },
    { 0, 0 },
    { 0x914e9d06, 23 },
    { 0x898e9d07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x622e9d0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6e9d12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fe9d1d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36e9d21, 76 },
    { 0, 0 },
    { 0xba6e9d23, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x723e9d26, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65e9d2b, 93 },
    { 0, 0 },
    { 0xfc5e9d2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x42be9d30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50e9d34, 61 },
    { 0x52be9d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575e9d3b, 95 },
    { 0x661e9d3b, 95 },
    { 0xfe1e9d3d, 55 },
    { 0x3bce9d3e, 23 },
    { 0x74ae9d3d, 90 },
    { 0x3cfe9d3f, 34 },
    { 0x96e9d41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fde9d48, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766e9d4d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35e9d53, 55 },
    { 0x64ee9d53, 33 },
    { 0x1fce9d55, 61 },
    { 0, 0 },
    { 0xa7de9d57, 98 },
    { 0xa58e9d57, 33 },
    { 0, 0 },
    { 0x589e9d5a, 43 },
    { 0x77be9d5b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515e9d63, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fe9d66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6e9d6a, 61 },
    { 0, 0 },
    { 0x7d5e9d6c, 98 },
    { 0xf1be9d6d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f5e9d70, 77 },
    { 0xa9be9d71, 84 },
    { 0, 0 },
    { 0x4cce9d73, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0e9d7b, 91 },
    { 0x1b6e9d7c, 90 },
    { 0xf7ae9d7d, 96 },
    { 0xf99e9d7d, 46 },
    { 0xea5e9d7f, 17 },
    { 0xf19e9d7e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271e9d85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2e9d89, 34 },
    { 0xb88e9d89, 61 },
    { 0x329e9d8b, 100 },
    { 0, 0 },
    { 0x518e9d8d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x96de9d90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1e9d96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x803e9d99, 92 },
    { 0x615e9d9a, 33 },
    { 0x90de9d99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2e9da0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16e9da5, 33 },
    { 0, 0 },
    { 0x915e9da7, 99 },
    { 0x45be9da7, 90 },
    { 0x765e9da9, 33 },
    { 0x963e9daa, 55 },
    { 0x44fe9da9, 76 },
    { 0x363e9dac, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0e9db3, 59 },
    { 0xe11e9db4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x82fe9db7, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ee9dba, 68 },
    { 0, 0 },
    { 0xeb1e9dbc, 90 },
    { 0, 0 },
    { 0x715e9dbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3e9dc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62e9dc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ee9dd2, 98 },
    { 0x9d3e9dd2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ce9dd9, 90 },
    { 0xa18e9dda, 61 },
    { 0xe2ee9ddb, 90 },
    { 0x96be9ddc, 33 },
    { 0, 0 },
    { 0xe8ee9dde, 98 },
    { 0x9f0e9ddf, 90 },
    { 0x4a9e9ddf, 23 },
    { 0xa8ae9ddf, 66 },
    { 0x966e9de0, 55 },
    { 0xdb4e9de1, 85 },
    { 0x55de9ddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ffe9de7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3e9df3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582e9df7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04e9dff, 55 },
    { 0xfe8e9e00, 90 },
    { 0, 0 },
    { 0x45ce9e02, 61 },
    { 0x329e9e03, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9be9e0a, 23 },
    { 0, 0 },
    { 0xf51e9e0c, 33 },
    { 0xa03e9e0d, 61 },
    { 0x1e5e9e0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3e9e12, 34 },
    { 0, 0 },
    { 0xf24e9e14, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693e9e19, 90 },
    { 0, 0 },
    { 0x8ebe9e1b, 90 },
    { 0x620e9e1c, 95 },
    { 0, 0 },
    { 0x33be9e1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58e9e21, 23 },
    { 0x52de9e22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x618e9e25, 52 },
    { 0x190e9e25, 23 },
    { 0x91fe9e27, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4e9e2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x563e9e2d, 61 },
    { 0x5efe9e2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92e9e31, 61 },
    { 0x317e9e31, 67 },
    { 0xef8e9e31, 95 },
    { 0, 0 },
    { 0x7be9e35, 55 },
    { 0xb3ae9e36, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdade9e3b, 81 },
    { 0, 0 },
    { 0x7f2e9e3d, 33 },
    { 0xadce9e3e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2de9e41, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x80be9e44, 55 },
    { 0xcf3e9e45, 43 },
    { 0x418e9e46, 61 },
    { 0x12ee9e47, 90 },
    { 0xf20e9e46, 86 },
    { 0x8ee9e47, 33 },
    { 0x7ade9e47, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x100e9e4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1e9e55, 66 },
    { 0, 0 },
    { 0x3a1e9e57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fee9e5b, 99 },
    { 0, 0 },
    { 0x6dbe9e5d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0e9e63, 90 },
    { 0x63ae9e64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58e9e67, 68 },
    { 0x3ee9e67, 33 },
    { 0x7b6e9e68, 55 },
    { 0x8e9e6a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcde9e6f, 56 },
    { 0x7d9e9e70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x502e9e73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470e9e7a, 84 },
    { 0xfd3e9e7a, 33 },
    { 0xe87e9e7c, 23 },
    { 0x191e9e7d, 59 },
    { 0, 0 },
    { 0xeebe9e7f, 95 },
    { 0x4b0e9e80, 61 },
    { 0x549e9e81, 23 },
    { 0x4dbe9e82, 34 },
    { 0xc0fe9e81, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580e9e8c, 61 },
    { 0x19ae9e8d, 46 },
    { 0, 0 },
    { 0x354e9e8f, 23 },
    { 0xd13e9e90, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285e9e94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430e9e98, 90 },
    { 0, 0 },
    { 0x7f1e9e9a, 93 },
    { 0, 0 },
    { 0x10ce9e9c, 93 },
    { 0x180e9e9c, 88 },
    { 0xcb4e9e9e, 98 },
    { 0xe3de9e9e, 92 },
    { 0x835e9ea0, 88 },
    { 0, 0 },
    { 0xf9ce9ea2, 99 },
    { 0x8f6e9ea2, 34 },
    { 0xda4e9ea3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3e9ea7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495e9eb0, 23 },
    { 0, 0 },
    { 0xefde9eb2, 98 },
    { 0x855e9eb3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xedde9eb6, 76 },
    { 0xdd3e9eb6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28fe9ec4, 55 },
    { 0x194e9ec4, 61 },
    { 0xde0e9ec6, 87 },
    { 0xb44e9ec7, 91 },
    { 0xf00e9ec8, 23 },
    { 0x8dee9ec7, 34 },
    { 0x40e9eca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30be9ece, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18e9ed5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x343e9ed8, 76 },
    { 0, 0 },
    { 0x15de9eda, 90 },
    { 0xf98e9edb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ee9edf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdce9ee5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2e9ee8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdabe9eed, 60 },
    { 0x128e9eee, 90 },
    { 0xb24e9eee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54be9ef4, 43 },
    { 0x795e9ef5, 33 },
    { 0x443e9ef4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x223e9ef9, 23 },
    { 0, 0 },
    { 0xd93e9efb, 90 },
    { 0xe40e9efc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abe9eff, 66 },
    { 0xe4e9f00, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53e9f07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ee9f0a, 76 },
    { 0x3a1e9f0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc38e9f0e, 90 },
    { 0, 0 },
    { 0x54ce9f10, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7e9f19, 33 },
    { 0, 0 },
    { 0x40be9f1b, 76 },
    { 0xaeae9f1b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3e9f20, 90 },
    { 0xc8ce9f21, 70 },
    { 0, 0 },
    { 0x4cde9f23, 43 },
    { 0xe56e9f24, 95 },
    { 0x4d7e9f24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19be9f2b, 61 },
    { 0, 0 },
    { 0xd1ae9f2d, 23 },
    { 0x99be9f2e, 90 },
    { 0x350e9f2f, 97 },
    { 0xb3fe9f2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ee9f37, 76 },
    { 0, 0 },
    { 0x274e9f39, 88 },
    { 0x64de9f3a, 52 },
    { 0, 0 },
    { 0x1fe9f3c, 66 },
    { 0, 0 },
    { 0x2a1e9f3e, 93 },
    { 0, 0 },
    { 0x2aae9f40, 40 },
    { 0, 0 },
    { 0xc1e9f42, 95 },
    { 0x87fe9f43, 66 },
    { 0x90de9f44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334e9f4a, 36 },
    { 0x434e9f4b, 55 },
    { 0xc47e9f4c, 76 },
    { 0xfc2e9f4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffe9f50, 100 },
    { 0, 0 },
    { 0xa10e9f52, 90 },
    { 0xa29e9f53, 24 },
    { 0xd81e9f54, 65 },
    { 0x8bce9f54, 95 },
    { 0x1ede9f56, 17 },
    { 0xa65e9f57, 90 },
    { 0x285e9f58, 33 },
    { 0xfe6e9f59, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59e9f5e, 87 },
    { 0x1bde9f5f, 76 },
    { 0x183e9f5f, 81 },
    { 0, 0 },
    { 0xd41e9f62, 52 },
    { 0x678e9f63, 33 },
    { 0, 0 },
    { 0xce1e9f65, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63de9f6f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8e9f72, 17 },
    { 0xce8e9f72, 23 },
    { 0x583e9f72, 76 },
    { 0x9fe9f75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4e9f7c, 98 },
    { 0xd8e9f7c, 85 },
    { 0x31fe9f7c, 98 },
    { 0xf34e9f7c, 23 },
    { 0x313e9f80, 90 },
    { 0x442e9f81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ce9f87, 90 },
    { 0, 0 },
    { 0xbb2e9f89, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252e9f91, 34 },
    { 0, 0 },
    { 0x609e9f93, 43 },
    { 0xd46e9f94, 50 },
    { 0, 0 },
    { 0x8b4e9f96, 98 },
    { 0x378e9f97, 95 },
    { 0, 0 },
    { 0x72fe9f99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6e9f9f, 76 },
    { 0xf8be9fa0, 46 },
    { 0xd91e9fa0, 66 },
    { 0x564e9f9f, 90 },
    { 0x538e9fa3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0e9fa6, 34 },
    { 0x4c5e9fa7, 61 },
    { 0xdc0e9fa7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ce9fad, 61 },
    { 0x42ce9fae, 95 },
    { 0x935e9fae, 23 },
    { 0, 0 },
    { 0xb0ee9fb1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ce9fb7, 55 },
    { 0x71ae9fb8, 95 },
    { 0x890e9fb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xacce9fbc, 2 },
    { 0, 0 },
    { 0x6fbe9fbe, 23 },
    { 0, 0 },
    { 0xc54e9fc0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bee9fc3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681e9fc7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547e9fcb, 55 },
    { 0, 0 },
    { 0x449e9fcd, 17 },
    { 0, 0 },
    { 0x194e9fcf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ae9fd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x417e9fd7, 100 },
    { 0, 0 },
    { 0x793e9fd9, 84 },
    { 0x5dbe9fda, 65 },
    { 0x667e9fda, 33 },
    { 0, 0 },
    { 0xa18e9fdd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1e9fe5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92e9fe9, 90 },
    { 0, 0 },
    { 0x59de9feb, 90 },
    { 0x5c9e9feb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ce9ff0, 90 },
    { 0x76ce9ff1, 90 },
    { 0, 0 },
    { 0x6a4e9ff3, 90 },
    { 0, 0 },
    { 0xac1e9ff5, 98 },
    { 0, 0 },
    { 0x820e9ff7, 90 },
    { 0xd49e9ff8, 33 },
    { 0xd0fe9ff7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5e9ffd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ea005, 61 },
    { 0, 0 },
    { 0xc72ea007, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1ea00f, 61 },
    { 0x9efea00f, 17 },
    { 0xb33ea011, 98 },
    { 0xc28ea011, 43 },
    { 0x180ea010, 90 },
    { 0xa3eea014, 90 },
    { 0x20aea012, 33 },
    { 0x8c8ea016, 91 },
    { 0, 0 },
    { 0x89fea018, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55ea01d, 90 },
    { 0, 0 },
    { 0x71aea01f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ea026, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13dea02c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adea02f, 90 },
    { 0, 0 },
    { 0x256ea031, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaeea034, 15 },
    { 0x3d5ea035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2ea03a, 81 },
    { 0xd6eea03b, 61 },
    { 0x98aea03c, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43ea040, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2ea046, 40 },
    { 0, 0 },
    { 0xabdea048, 100 },
    { 0x7bcea049, 61 },
    { 0x529ea048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adea04f, 100 },
    { 0, 0 },
    { 0xae5ea051, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906ea058, 95 },
    { 0, 0 },
    { 0x5d7ea05a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfea062, 61 },
    { 0x4aaea063, 61 },
    { 0x28aea063, 61 },
    { 0x721ea065, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaea06a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0ea073, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85eea077, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x782ea07b, 56 },
    { 0xe7bea07b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3eea080, 90 },
    { 0x9d0ea081, 98 },
    { 0x3b8ea081, 55 },
    { 0x3ceea082, 23 },
    { 0x2bbea084, 98 },
    { 0x47bea084, 95 },
    { 0xdfeea086, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ea08c, 53 },
    { 0x587ea08d, 38 },
    { 0xba6ea08e, 46 },
    { 0xfbfea08f, 40 },
    { 0xf23ea08f, 76 },
    { 0xb3fea091, 61 },
    { 0, 0 },
    { 0xf76ea093, 90 },
    { 0xb9fea094, 33 },
    { 0xf11ea095, 92 },
    { 0, 0 },
    { 0xca3ea097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50ea09d, 84 },
    { 0, 0 },
    { 0x2c4ea09f, 33 },
    { 0xa2cea0a0, 23 },
    { 0xc48ea0a1, 55 },
    { 0, 0 },
    { 0xdc0ea0a3, 17 },
    { 0, 0 },
    { 0x172ea0a5, 33 },
    { 0xa34ea0a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eea0aa, 33 },
    { 0xeebea0aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe66ea0b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763ea0b5, 87 },
    { 0, 0 },
    { 0xd07ea0b7, 93 },
    { 0x154ea0b7, 95 },
    { 0, 0 },
    { 0x959ea0ba, 90 },
    { 0x7ea0bb, 23 },
    { 0xabdea0bb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5ea0c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ea0d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65ea0da, 61 },
    { 0, 0 },
    { 0xa85ea0dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36ea0e2, 76 },
    { 0xdb9ea0e3, 85 },
    { 0xf85ea0e4, 84 },
    { 0xd4aea0e5, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fea0ea, 78 },
    { 0x24fea0ea, 17 },
    { 0x5c1ea0ec, 85 },
    { 0x1e6ea0ed, 76 },
    { 0, 0 },
    { 0xa05ea0ef, 90 },
    { 0x3b1ea0f0, 90 },
    { 0, 0 },
    { 0xf74ea0f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78ea0f5, 100 },
    { 0, 0 },
    { 0x9ebea0f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaea0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49ea100, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46ea103, 33 },
    { 0xde9ea104, 93 },
    { 0x180ea105, 77 },
    { 0xdb9ea106, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ea10f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395ea114, 95 },
    { 0x6cea114, 98 },
    { 0x43bea116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f3ea11b, 33 },
    { 0xd00ea11b, 90 },
    { 0, 0 },
    { 0xd90ea11e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8ea121, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeea127, 95 },
    { 0xc70ea127, 33 },
    { 0x49fea128, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cea12e, 43 },
    { 0xaf5ea12f, 23 },
    { 0x2b8ea130, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6ea133, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91ea137, 74 },
    { 0xbf5ea137, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeea13e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacea146, 56 },
    { 0xb8bea146, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7ea14a, 95 },
    { 0xc83ea14a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ea150, 34 },
    { 0x644ea150, 99 },
    { 0x11eea152, 33 },
    { 0, 0 },
    { 0x3f6ea154, 76 },
    { 0x5a5ea154, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9ea158, 98 },
    { 0x5acea159, 91 },
    { 0xcbbea15a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ea15f, 90 },
    { 0, 0 },
    { 0xc15ea161, 95 },
    { 0x36fea161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x576ea169, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ea16f, 33 },
    { 0x29aea16f, 43 },
    { 0xb14ea171, 33 },
    { 0xd27ea171, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fea179, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaea17d, 61 },
    { 0x711ea17e, 95 },
    { 0, 0 },
    { 0xe9eea180, 78 },
    { 0xc4bea181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8ea186, 33 },
    { 0, 0 },
    { 0xfbdea188, 95 },
    { 0, 0 },
    { 0x1d1ea18a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeeea194, 90 },
    { 0x927ea194, 55 },
    { 0, 0 },
    { 0xd78ea197, 96 },
    { 0, 0 },
    { 0x356ea199, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07ea1a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eea1a3, 88 },
    { 0, 0 },
    { 0x420ea1a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48ea1aa, 52 },
    { 0, 0 },
    { 0xed5ea1ac, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ea1af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdea1b3, 33 },
    { 0x4acea1b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1ea1b8, 33 },
    { 0x6baea1b8, 90 },
    { 0, 0 },
    { 0xaf6ea1bb, 90 },
    { 0x2d8ea1bc, 61 },
    { 0, 0 },
    { 0x5c3ea1be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03ea1c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8ea1ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cea1ce, 98 },
    { 0xd43ea1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338ea1d7, 99 },
    { 0x6dea1d7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaea1dd, 61 },
    { 0x26bea1de, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ea1e2, 34 },
    { 0xb88ea1e3, 33 },
    { 0x6fdea1e4, 68 },
    { 0x1d0ea1e5, 61 },
    { 0x52ea1e3, 85 },
    { 0, 0 },
    { 0x65bea1e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ea1eb, 90 },
    { 0xc6cea1ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feea1f0, 23 },
    { 0x182ea1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8eea1f6, 95 },
    { 0x1a4ea1f6, 65 },
    { 0x53eea1f8, 52 },
    { 0x284ea1f8, 85 },
    { 0xb23ea1f8, 33 },
    { 0, 0 },
    { 0x7b8ea1fc, 83 },
    { 0xf37ea1fd, 55 },
    { 0, 0 },
    { 0x6f8ea1ff, 78 },
    { 0x756ea200, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9aea203, 88 },
    { 0, 0 },
    { 0x65dea205, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x240ea208, 66 },
    { 0x323ea209, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bea20d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fea21a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989ea21f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ea224, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ea229, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160ea231, 33 },
    { 0xa62ea232, 87 },
    { 0x8d5ea233, 98 },
    { 0x187ea234, 61 },
    { 0, 0 },
    { 0xbcfea236, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ea239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bea242, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cea246, 95 },
    { 0x811ea246, 90 },
    { 0xd3aea246, 91 },
    { 0, 0 },
    { 0x702ea24a, 95 },
    { 0x50cea24b, 61 },
    { 0, 0 },
    { 0x610ea24d, 93 },
    { 0x923ea24e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9ea251, 85 },
    { 0xba7ea252, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61ea256, 23 },
    { 0xad6ea257, 61 },
    { 0x257ea256, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349ea25c, 69 },
    { 0x1dcea25d, 68 },
    { 0x928ea25d, 99 },
    { 0, 0 },
    { 0xe1eea260, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ea263, 33 },
    { 0x846ea264, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416ea26a, 88 },
    { 0xe5aea26b, 90 },
    { 0xd23ea26b, 98 },
    { 0x7b0ea26d, 34 },
    { 0x6cea26e, 95 },
    { 0x85fea26f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15ea272, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bea275, 43 },
    { 0, 0 },
    { 0xb6bea277, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfea27a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0ea27e, 93 },
    { 0xaedea27f, 61 },
    { 0xb73ea280, 85 },
    { 0x7f7ea27e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eea284, 90 },
    { 0xa08ea284, 61 },
    { 0x7caea284, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbaea289, 88 },
    { 0x653ea289, 33 },
    { 0x4a5ea289, 23 },
    { 0xc7bea28b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ea28f, 90 },
    { 0, 0 },
    { 0x473ea291, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eea295, 95 },
    { 0xfbbea295, 92 },
    { 0xedaea295, 23 },
    { 0xf18ea295, 34 },
    { 0x98ea299, 55 },
    { 0xcd2ea295, 95 },
    { 0x8d8ea29b, 90 },
    { 0xc4dea29c, 98 },
    { 0, 0 },
    { 0x9ecea29e, 90 },
    { 0x52dea29e, 55 },
    { 0x437ea29f, 93 },
    { 0x2e8ea29e, 52 },
    { 0x628ea2a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6ea2a5, 33 },
    { 0, 0 },
    { 0x540ea2a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eea2ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74ea2b0, 95 },
    { 0xda6ea2b1, 66 },
    { 0xbf0ea2b0, 61 },
    { 0x652ea2b3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0ea2b7, 100 },
    { 0, 0 },
    { 0x5a2ea2b9, 23 },
    { 0xefaea2ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8dea2bd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35ea2c4, 90 },
    { 0x834ea2c5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x334ea2c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x913ea2cb, 23 },
    { 0x8bea2cb, 55 },
    { 0, 0 },
    { 0xeb0ea2ce, 36 },
    { 0x78cea2cf, 33 },
    { 0x79bea2cf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0ea2d6, 91 },
    { 0xeabea2d7, 90 },
    { 0, 0 },
    { 0xe2dea2d9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792ea2dd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x957ea2e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561ea2e4, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6ea2e8, 55 },
    { 0x87bea2e8, 55 },
    { 0x9f1ea2ea, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abea2f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aea301, 98 },
    { 0, 0 },
    { 0x114ea303, 98 },
    { 0, 0 },
    { 0x86eea305, 98 },
    { 0, 0 },
    { 0x663ea307, 95 },
    { 0, 0 },
    { 0x5b8ea309, 33 },
    { 0xd7bea30a, 77 },
    { 0x873ea309, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x113ea30e, 46 },
    { 0x11ea30f, 43 },
    { 0x547ea310, 98 },
    { 0, 0 },
    { 0x42bea312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bea31c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1ea325, 100 },
    { 0x937ea326, 95 },
    { 0x290ea326, 61 },
    { 0x135ea328, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1ea32c, 90 },
    { 0xcaea32d, 90 },
    { 0x225ea32d, 95 },
    { 0xe22ea32f, 76 },
    { 0, 0 },
    { 0xb45ea331, 98 },
    { 0x1ffea331, 33 },
    { 0, 0 },
    { 0xea7ea334, 98 },
    { 0x877ea334, 87 },
    { 0x257ea336, 55 },
    { 0x560ea337, 33 },
    { 0x7e2ea336, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22ea340, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ea343, 90 },
    { 0x311ea344, 87 },
    { 0x476ea343, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113ea349, 61 },
    { 0, 0 },
    { 0x2b5ea34b, 23 },
    { 0xa3aea34b, 36 },
    { 0x4d2ea34b, 33 },
    { 0, 0 },
    { 0x1e3ea34f, 100 },
    { 0x665ea350, 90 },
    { 0xacea34f, 76 },
    { 0x344ea350, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fea357, 81 },
    { 0x77fea358, 90 },
    { 0, 0 },
    { 0xc4bea35a, 95 },
    { 0x6a1ea35b, 90 },
    { 0x11eea35c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce1ea362, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3ea365, 76 },
    { 0, 0 },
    { 0x83eea367, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1dea36a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6ea36d, 61 },
    { 0x554ea36d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656ea374, 90 },
    { 0xde8ea374, 43 },
    { 0, 0 },
    { 0xbe3ea377, 99 },
    { 0xd67ea378, 66 },
    { 0x2daea379, 61 },
    { 0x90fea37a, 61 },
    { 0xd78ea37b, 95 },
    { 0, 0 },
    { 0x87aea37d, 75 },
    { 0, 0 },
    { 0x8abea37f, 76 },
    { 0xa3cea380, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ea386, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cea38a, 98 },
    { 0, 0 },
    { 0x9f6ea38c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ea38f, 23 },
    { 0x658ea38f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bea393, 95 },
    { 0xf48ea394, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fea39d, 43 },
    { 0xb0dea39e, 85 },
    { 0xeaeea39f, 90 },
    { 0xd6eea3a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cea3a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fea3a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267ea3ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4ea3b4, 84 },
    { 0x7f8ea3b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34eea3bb, 99 },
    { 0x31dea3bc, 88 },
    { 0x6eaea3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5ea3c8, 33 },
    { 0x41eea3c9, 95 },
    { 0x38cea3c8, 23 },
    { 0x105ea3cb, 101 },
    { 0x876ea3cc, 66 },
    { 0x26aea3cd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aea3d0, 98 },
    { 0, 0 },
    { 0x6feea3d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedea3d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843ea3de, 98 },
    { 0x1b8ea3df, 90 },
    { 0, 0 },
    { 0xb2aea3e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ea3e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x735ea3e9, 82 },
    { 0, 0 },
    { 0x36fea3eb, 91 },
    { 0x865ea3ec, 66 },
    { 0, 0 },
    { 0x638ea3ee, 55 },
    { 0, 0 },
    { 0xb9ea3f0, 43 },
    { 0, 0 },
    { 0x79ea3f2, 99 },
    { 0, 0 },
    { 0xc44ea3f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dea3f7, 76 },
    { 0xf7fea3f8, 87 },
    { 0xa12ea3f9, 33 },
    { 0x9adea3fa, 33 },
    { 0xe19ea3fa, 61 },
    { 0x4f9ea3fa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ea401, 90 },
    { 0, 0 },
    { 0xfdeea403, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10fea409, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccbea40f, 61 },
    { 0xbc1ea40f, 65 },
    { 0x93dea411, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1aea415, 85 },
    { 0x51bea416, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c6ea41d, 64 },
    { 0, 0 },
    { 0x91aea41f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dea431, 23 },
    { 0x3aeea432, 76 },
    { 0x625ea432, 85 },
    { 0x585ea434, 61 },
    { 0xe36ea435, 90 },
    { 0xb7cea436, 23 },
    { 0xb2cea431, 61 },
    { 0xa82ea437, 40 },
    { 0xfa7ea439, 100 },
    { 0, 0 },
    { 0xcea43b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2ea43e, 40 },
    { 0x76dea43e, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf5ea443, 33 },
    { 0x9a6ea443, 98 },
    { 0x6afea444, 43 },
    { 0x536ea446, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57ea44f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdea452, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349ea45b, 23 },
    { 0, 0 },
    { 0x6cfea45d, 90 },
    { 0xfdaea45d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccea461, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x43ea464, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cea468, 81 },
    { 0xd19ea468, 90 },
    { 0x288ea469, 85 },
    { 0, 0 },
    { 0x18fea46c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebea470, 77 },
    { 0x82fea471, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xab2ea474, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241ea47b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8ea480, 90 },
    { 0x3dea481, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8ea484, 23 },
    { 0x5c7ea485, 95 },
    { 0xd01ea484, 98 },
    { 0xfceea487, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bea48f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaea494, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812ea498, 61 },
    { 0x390ea498, 46 },
    { 0x6ddea49a, 36 },
    { 0xadeea49a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84ea49f, 61 },
    { 0, 0 },
    { 0x7bfea4a1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18ea4a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8ea4a7, 76 },
    { 0xe62ea4a8, 33 },
    { 0x915ea4a9, 33 },
    { 0xe88ea4aa, 55 },
    { 0x1f5ea4a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527ea4af, 89 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbea4b3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x194ea4b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x597ea4ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69aea4be, 33 },
    { 0x103ea4be, 34 },
    { 0x358ea4c0, 78 },
    { 0x7b7ea4c0, 84 },
    { 0x3a2ea4c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ea4c5, 23 },
    { 0, 0 },
    { 0x142ea4c7, 38 },
    { 0x5f5ea4c7, 98 },
    { 0x606ea4c8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0ea4cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ea4cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eea4d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x344ea4da, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ea4e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x365ea4ea, 98 },
    { 0x1c9ea4eb, 61 },
    { 0, 0 },
    { 0x8a8ea4ed, 43 },
    { 0x12fea4ee, 34 },
    { 0x840ea4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8ea4f3, 33 },
    { 0x115ea4f4, 84 },
    { 0xeb5ea4f3, 95 },
    { 0x7b3ea4f6, 61 },
    { 0x42eea4f7, 23 },
    { 0, 0 },
    { 0xb9ea4f9, 95 },
    { 0x9b5ea4fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46ea4fe, 98 },
    { 0x5b2ea4ff, 55 },
    { 0, 0 },
    { 0xb33ea501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6ea504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104ea509, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cea50d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cea516, 97 },
    { 0x39bea517, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3daea51b, 98 },
    { 0xc6cea51c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bea520, 95 },
    { 0, 0 },
    { 0x35cea522, 61 },
    { 0x891ea523, 90 },
    { 0, 0 },
    { 0x7efea525, 33 },
    { 0x18eea526, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dea52a, 33 },
    { 0x76cea52b, 91 },
    { 0xb56ea52c, 90 },
    { 0x6e6ea52d, 52 },
    { 0x494ea52e, 91 },
    { 0xb45ea52f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ceea533, 95 },
    { 0, 0 },
    { 0xf2fea535, 56 },
    { 0x9ea536, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4ea53a, 69 },
    { 0xcf6ea53a, 95 },
    { 0, 0 },
    { 0x31dea53d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cea540, 33 },
    { 0x47aea541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662ea547, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ea54a, 66 },
    { 0, 0 },
    { 0x6b3ea54c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cea54f, 76 },
    { 0x2c0ea550, 77 },
    { 0xcc7ea550, 33 },
    { 0x83cea54f, 90 },
    { 0x3ceea54f, 33 },
    { 0x8eea550, 95 },
    { 0xc53ea555, 55 },
    { 0x446ea556, 55 },
    { 0xddcea557, 78 },
    { 0x5d6ea557, 77 },
    { 0, 0 },
    { 0x202ea55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50aea55e, 77 },
    { 0, 0 },
    { 0x964ea560, 16 },
    { 0x469ea560, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4ea564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5ea569, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9ea570, 61 },
    { 0x5daea571, 55 },
    { 0x3c4ea570, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ea57c, 61 },
    { 0xf69ea57d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcea581, 90 },
    { 0xd46ea582, 78 },
    { 0xcc0ea582, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435ea587, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bea58b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7ea592, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cea596, 88 },
    { 0x128ea596, 23 },
    { 0, 0 },
    { 0x555ea599, 34 },
    { 0x59dea59a, 95 },
    { 0, 0 },
    { 0x1a7ea59c, 75 },
    { 0x7fbea59c, 98 },
    { 0xb14ea59c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2ea5a1, 33 },
    { 0x796ea5a2, 23 },
    { 0, 0 },
    { 0x9b9ea5a4, 81 },
    { 0, 0 },
    { 0xdeea5a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509ea5ab, 95 },
    { 0x4f2ea5ab, 42 },
    { 0, 0 },
    { 0xe38ea5ae, 87 },
    { 0x4beea5af, 90 },
    { 0x851ea5af, 33 },
    { 0x9ea5ae, 33 },
    { 0, 0 },
    { 0x5ceea5b3, 23 },
    { 0xf6bea5b4, 43 },
    { 0x2b4ea5b5, 91 },
    { 0xba4ea5b6, 93 },
    { 0x77bea5b7, 53 },
    { 0xeb0ea5b8, 33 },
    { 0xa58ea5b4, 34 },
    { 0xb98ea5b9, 98 },
    { 0xa1eea5ba, 98 },
    { 0x175ea5bc, 90 },
    { 0x8e3ea5bc, 95 },
    { 0xd10ea5be, 77 },
    { 0, 0 },
    { 0x967ea5c0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ea5c8, 90 },
    { 0x679ea5c8, 76 },
    { 0xa05ea5c8, 99 },
    { 0, 0 },
    { 0xd9aea5cc, 87 },
    { 0x36aea5cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796ea5d1, 46 },
    { 0, 0 },
    { 0x94bea5d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485ea5d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6ea5dc, 33 },
    { 0xee3ea5dc, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127ea5e1, 56 },
    { 0xf3bea5e2, 61 },
    { 0xf16ea5e1, 34 },
    { 0xf12ea5e3, 33 },
    { 0xfc5ea5e5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29ea5e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cea5ec, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87eea5f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abea5f4, 95 },
    { 0x168ea5f5, 88 },
    { 0x43dea5f5, 76 },
    { 0xf69ea5f5, 98 },
    { 0x516ea5f8, 43 },
    { 0x53bea5f8, 43 },
    { 0x579ea5f4, 61 },
    { 0xfe9ea5f9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cea5fe, 95 },
    { 0, 0 },
    { 0xaf2ea600, 33 },
    { 0x55eea601, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ea604, 77 },
    { 0, 0 },
    { 0x238ea606, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fea609, 23 },
    { 0x6f4ea60a, 40 },
    { 0x29dea60b, 98 },
    { 0x39ea60b, 95 },
    { 0xe93ea609, 76 },
    { 0x5e1ea60e, 91 },
    { 0x45ea60f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ea612, 61 },
    { 0x92cea612, 76 },
    { 0x20ea614, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09ea617, 95 },
    { 0xb0fea618, 61 },
    { 0xdd4ea619, 68 },
    { 0, 0 },
    { 0xf51ea61b, 84 },
    { 0, 0 },
    { 0x637ea61d, 46 },
    { 0x6f7ea61e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34dea624, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aea627, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ea62e, 34 },
    { 0, 0 },
    { 0x7c3ea630, 34 },
    { 0x198ea631, 95 },
    { 0x1ecea632, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cea636, 61 },
    { 0x6a4ea636, 33 },
    { 0x1edea636, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf1ea640, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0ea646, 46 },
    { 0x202ea647, 61 },
    { 0x84cea648, 52 },
    { 0x300ea649, 93 },
    { 0xa42ea649, 33 },
    { 0xb3ea64a, 23 },
    { 0xebea64c, 91 },
    { 0xca1ea649, 33 },
    { 0xa5eea648, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eea656, 93 },
    { 0x600ea657, 23 },
    { 0xd4ea658, 23 },
    { 0x9caea659, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faea661, 43 },
    { 0x216ea662, 61 },
    { 0xc26ea663, 23 },
    { 0x48ea662, 90 },
    { 0, 0 },
    { 0x115ea666, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1ea669, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1ea66f, 33 },
    { 0x6dbea670, 55 },
    { 0xaea66f, 90 },
    { 0x324ea672, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65ea679, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2ea67d, 95 },
    { 0, 0 },
    { 0x314ea67f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eea68e, 90 },
    { 0xf34ea68f, 34 },
    { 0xe58ea690, 35 },
    { 0, 0 },
    { 0x234ea692, 33 },
    { 0x31fea693, 23 },
    { 0, 0 },
    { 0xe3eea695, 23 },
    { 0, 0 },
    { 0x578ea697, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52ea69d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98aea6a1, 90 },
    { 0x580ea6a1, 43 },
    { 0, 0 },
    { 0x7dea6a4, 23 },
    { 0x363ea6a4, 76 },
    { 0x2e2ea6a6, 61 },
    { 0x2d9ea6a7, 95 },
    { 0x518ea6a6, 38 },
    { 0x1bfea6a9, 77 },
    { 0x896ea6aa, 90 },
    { 0xc58ea6a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ea6ae, 90 },
    { 0xedbea6af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16fea6b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74ea6b9, 60 },
    { 0x571ea6b9, 75 },
    { 0xb00ea6bb, 55 },
    { 0xf97ea6ba, 91 },
    { 0x8d9ea6bd, 98 },
    { 0, 0 },
    { 0xda7ea6bf, 85 },
    { 0x337ea6c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8aea6c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x632ea6c6, 99 },
    { 0xab9ea6c7, 93 },
    { 0x961ea6c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92ea6cc, 85 },
    { 0xc04ea6cd, 23 },
    { 0, 0 },
    { 0x478ea6cf, 61 },
    { 0xd4dea6cf, 76 },
    { 0x461ea6d1, 23 },
    { 0, 0 },
    { 0x9cfea6d3, 46 },
    { 0xde5ea6d4, 43 },
    { 0, 0 },
    { 0xdccea6d6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x498ea6db, 61 },
    { 0xaffea6dc, 98 },
    { 0x817ea6dc, 32 },
    { 0x660ea6de, 23 },
    { 0x28bea6de, 90 },
    { 0x6a3ea6e0, 23 },
    { 0xd7cea6db, 33 },
    { 0x757ea6e2, 98 },
    { 0x55fea6e2, 57 },
    { 0x27dea6e4, 90 },
    { 0x8edea6e3, 90 },
    { 0x85cea6e1, 91 },
    { 0x8c8ea6e7, 43 },
    { 0x6a4ea6de, 93 },
    { 0x35fea6e0, 26 },
    { 0, 0 },
    { 0xfd5ea6eb, 34 },
    { 0, 0 },
    { 0xbe2ea6ed, 61 },
    { 0, 0 },
    { 0x9b0ea6ef, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5ea6f2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bea6fb, 76 },
    { 0x474ea6fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85ea704, 33 },
    { 0x909ea705, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cea709, 95 },
    { 0xe46ea709, 90 },
    { 0x187ea70b, 95 },
    { 0x37cea70c, 90 },
    { 0, 0 },
    { 0x9c9ea70e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4ea714, 81 },
    { 0, 0 },
    { 0xbd6ea716, 52 },
    { 0x9c5ea716, 76 },
    { 0x4cdea718, 23 },
    { 0xccbea716, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bea71d, 61 },
    { 0x2adea71e, 98 },
    { 0x4beea71f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ea723, 76 },
    { 0x3aaea724, 61 },
    { 0x71cea723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419ea72b, 34 },
    { 0, 0 },
    { 0x62dea72d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2faea735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bea738, 34 },
    { 0x3dcea739, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeeea742, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60ea746, 90 },
    { 0xd49ea747, 81 },
    { 0x1cdea748, 61 },
    { 0x190ea747, 84 },
    { 0xaebea747, 98 },
    { 0x298ea74a, 95 },
    { 0xec0ea74c, 81 },
    { 0x657ea74c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28ea751, 84 },
    { 0x5e4ea752, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2ea75a, 95 },
    { 0xd65ea75b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feea75f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ea763, 61 },
    { 0, 0 },
    { 0x58eea765, 55 },
    { 0xefeea765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89fea76c, 76 },
    { 0, 0 },
    { 0x1edea76e, 66 },
    { 0x457ea76f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dea772, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x355ea779, 33 },
    { 0x84dea77a, 65 },
    { 0, 0 },
    { 0x798ea77c, 90 },
    { 0x12aea77c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8ea780, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fea786, 33 },
    { 0x5ea786, 55 },
    { 0x4c7ea786, 33 },
    { 0xfd3ea789, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fea78f, 33 },
    { 0, 0 },
    { 0xf27ea791, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2ea794, 56 },
    { 0x169ea795, 33 },
    { 0xd1ea795, 93 },
    { 0x233ea797, 36 },
    { 0xcc6ea798, 52 },
    { 0xb39ea798, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2ea79c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198ea7a3, 66 },
    { 0xafdea7a3, 90 },
    { 0, 0 },
    { 0xa64ea7a6, 98 },
    { 0, 0 },
    { 0x444ea7a8, 90 },
    { 0, 0 },
    { 0x96eea7aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dea7ae, 55 },
    { 0x122ea7af, 81 },
    { 0x70cea7b0, 99 },
    { 0xd60ea7b0, 95 },
    { 0xdbea7b2, 90 },
    { 0xc92ea7b2, 90 },
    { 0, 0 },
    { 0x3f4ea7b5, 67 },
    { 0x9eeea7b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8ea7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeea7bd, 33 },
    { 0xf4dea7be, 95 },
    { 0xa59ea7bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47aea7c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adea7c7, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e8ea7ca, 95 },
    { 0, 0 },
    { 0xe56ea7cc, 93 },
    { 0xacbea7cd, 33 },
    { 0x80cea7ce, 67 },
    { 0xe9ea7cf, 90 },
    { 0x451ea7d0, 87 },
    { 0x47eea7cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63ea7d8, 23 },
    { 0, 0 },
    { 0xebeea7da, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe92ea7dd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971ea7e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bea7e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18ea7ec, 34 },
    { 0, 0 },
    { 0x61aea7ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cea7f1, 91 },
    { 0x64eea7f2, 17 },
    { 0, 0 },
    { 0x64ea7f4, 34 },
    { 0xf0eea7f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x51aea7f8, 66 },
    { 0, 0 },
    { 0x940ea7fa, 67 },
    { 0xd64ea7fb, 95 },
    { 0xdadea7fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ea806, 23 },
    { 0xf01ea807, 90 },
    { 0, 0 },
    { 0x169ea809, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaea80c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79ea810, 90 },
    { 0, 0 },
    { 0xd87ea812, 23 },
    { 0xd43ea812, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8ea818, 95 },
    { 0x476ea819, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268ea81e, 61 },
    { 0xa81ea81f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2aea823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ea827, 33 },
    { 0x9d3ea828, 95 },
    { 0x852ea828, 61 },
    { 0x17eea82a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4ea82e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7ea831, 88 },
    { 0xadeea832, 77 },
    { 0xf6dea832, 17 },
    { 0xfcbea834, 95 },
    { 0x6caea835, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x457ea838, 90 },
    { 0xfa6ea839, 85 },
    { 0, 0 },
    { 0x67bea83b, 76 },
    { 0, 0 },
    { 0x5d4ea83d, 88 },
    { 0x8e0ea83d, 81 },
    { 0x95bea83f, 90 },
    { 0xb18ea83d, 43 },
    { 0, 0 },
    { 0x455ea842, 90 },
    { 0, 0 },
    { 0x236ea844, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bea84a, 46 },
    { 0x857ea84a, 95 },
    { 0x45eea84c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ea851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcea856, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x895ea859, 100 },
    { 0x97aea85a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8ea85e, 61 },
    { 0xd17ea85f, 78 },
    { 0x115ea85e, 92 },
    { 0x33ea85f, 95 },
    { 0x938ea85e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdea866, 95 },
    { 0, 0 },
    { 0x15ea868, 33 },
    { 0x4f6ea868, 33 },
    { 0x63fea86a, 33 },
    { 0x769ea868, 95 },
    { 0xc9cea86c, 74 },
    { 0x6beea86d, 78 },
    { 0, 0 },
    { 0xbafea86f, 78 },
    { 0x820ea86f, 76 },
    { 0xb3ea871, 90 },
    { 0x5adea870, 55 },
    { 0x920ea873, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477ea877, 98 },
    { 0xd98ea878, 33 },
    { 0x280ea879, 95 },
    { 0xd07ea878, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cea87d, 80 },
    { 0x262ea87e, 33 },
    { 0, 0 },
    { 0x393ea880, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10ea883, 90 },
    { 0, 0 },
    { 0xfe6ea885, 65 },
    { 0xf41ea886, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bea88c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0ea88f, 61 },
    { 0xb97ea890, 23 },
    { 0xf8bea891, 33 },
    { 0x9e1ea892, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dea898, 55 },
    { 0, 0 },
    { 0xd55ea89a, 98 },
    { 0x4a9ea89b, 78 },
    { 0x63eea89c, 66 },
    { 0x6c6ea89a, 90 },
    { 0x31cea89c, 84 },
    { 0x627ea89e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaea8a3, 81 },
    { 0x597ea8a3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85aea8a7, 23 },
    { 0xac6ea8a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeea8ad, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabea8b7, 88 },
    { 0, 0 },
    { 0xe9fea8b9, 85 },
    { 0xba3ea8ba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fea8c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03ea8c4, 33 },
    { 0, 0 },
    { 0xbcea8c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cea8c9, 43 },
    { 0, 0 },
    { 0xfaeea8cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eea8ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0ea8d2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cea8d8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8ea8db, 55 },
    { 0xbe1ea8dc, 76 },
    { 0x6e3ea8dd, 55 },
    { 0, 0 },
    { 0x8a2ea8df, 46 },
    { 0xa31ea8df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ea8e4, 90 },
    { 0x298ea8e5, 46 },
    { 0xa4bea8e4, 61 },
    { 0, 0 },
    { 0xe20ea8e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ea8eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60eea8f0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecea8f3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4ea8f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9ea8fa, 46 },
    { 0x344ea8fb, 33 },
    { 0xf11ea8fb, 93 },
    { 0x432ea8fd, 76 },
    { 0x63ea8fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ea901, 95 },
    { 0xa20ea901, 61 },
    { 0xc9ea903, 66 },
    { 0x59dea903, 23 },
    { 0xe29ea904, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eea909, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14aea910, 91 },
    { 0x498ea911, 99 },
    { 0x162ea911, 90 },
    { 0xd93ea913, 100 },
    { 0, 0 },
    { 0x54cea915, 98 },
    { 0x243ea915, 93 },
    { 0xc30ea916, 66 },
    { 0, 0 },
    { 0x674ea919, 43 },
    { 0x108ea919, 90 },
    { 0xf67ea91b, 33 },
    { 0xbfaea91c, 17 },
    { 0xfabea91d, 23 },
    { 0, 0 },
    { 0x776ea91f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315ea925, 77 },
    { 0x20bea926, 33 },
    { 0x142ea927, 17 },
    { 0x51dea927, 91 },
    { 0, 0 },
    { 0x6d4ea92a, 61 },
    { 0xc8aea92b, 98 },
    { 0x272ea92a, 61 },
    { 0x69aea92c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcea932, 98 },
    { 0x484ea932, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaccea93d, 46 },
    { 0, 0 },
    { 0x9ea93f, 90 },
    { 0x5cfea940, 95 },
    { 0x9f5ea941, 23 },
    { 0, 0 },
    { 0xab4ea943, 61 },
    { 0x553ea944, 87 },
    { 0x85dea945, 90 },
    { 0xcecea946, 90 },
    { 0xf19ea947, 95 },
    { 0x6a4ea948, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3ea94c, 90 },
    { 0x312ea94c, 95 },
    { 0x564ea94d, 98 },
    { 0, 0 },
    { 0x31ea950, 67 },
    { 0x4b1ea951, 65 },
    { 0, 0 },
    { 0x93ea953, 98 },
    { 0x88cea954, 90 },
    { 0x6fdea953, 61 },
    { 0x50ea956, 78 },
    { 0x1e5ea957, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eea95b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19eea962, 23 },
    { 0xd05ea962, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ea967, 33 },
    { 0xc2eea967, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ea96c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4ea96f, 36 },
    { 0xcb9ea96f, 61 },
    { 0x515ea971, 33 },
    { 0x588ea972, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2ea979, 95 },
    { 0x9a2ea97a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ea98b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3ea990, 76 },
    { 0x67eea991, 100 },
    { 0xcdaea992, 55 },
    { 0x7a1ea991, 61 },
    { 0, 0 },
    { 0xd95ea995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdea998, 55 },
    { 0x843ea998, 17 },
    { 0x77cea99a, 90 },
    { 0, 0 },
    { 0x967ea99c, 69 },
    { 0x165ea99d, 90 },
    { 0, 0 },
    { 0x60fea99f, 87 },
    { 0x659ea99f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4ea9a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ea9a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626ea9af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cea9b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaea9b5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5daea9bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537ea9c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x779ea9ca, 85 },
    { 0, 0 },
    { 0x126ea9cc, 90 },
    { 0xfcdea9cd, 33 },
    { 0x303ea9ce, 56 },
    { 0xae0ea9cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afea9d9, 90 },
    { 0x5e8ea9d9, 34 },
    { 0, 0 },
    { 0x469ea9dc, 90 },
    { 0x519ea9dd, 43 },
    { 0xcaeea9dd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecea9e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ea9e6, 38 },
    { 0x5beea9e6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6ea9ec, 91 },
    { 0x863ea9ed, 85 },
    { 0xdbcea9ee, 100 },
    { 0x2e6ea9ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdaea9f3, 91 },
    { 0x750ea9f4, 93 },
    { 0x9f5ea9f5, 55 },
    { 0, 0 },
    { 0x53dea9f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbbea9fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebeaa01, 66 },
    { 0x3adeaa01, 23 },
    { 0, 0 },
    { 0x59beaa04, 23 },
    { 0x6cdeaa05, 87 },
    { 0, 0 },
    { 0x1bfeaa07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79eaa0a, 90 },
    { 0x80feaa0b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234eaa10, 34 },
    { 0xfe4eaa11, 46 },
    { 0, 0 },
    { 0x316eaa13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cceaa17, 23 },
    { 0xbabeaa18, 98 },
    { 0, 0 },
    { 0x888eaa1a, 33 },
    { 0, 0 },
    { 0xd6ceaa1c, 55 },
    { 0xab0eaa1d, 95 },
    { 0x4ceaa1e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848eaa24, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8beaa27, 69 },
    { 0xf17eaa27, 61 },
    { 0x368eaa29, 77 },
    { 0xfeceaa2a, 87 },
    { 0, 0 },
    { 0x83eeaa2c, 90 },
    { 0x48ceaa2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6feaa31, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eaa34, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x779eaa37, 55 },
    { 0xbdbeaa37, 43 },
    { 0x60beaa38, 84 },
    { 0, 0 },
    { 0xd4ceaa3b, 93 },
    { 0, 0 },
    { 0xf46eaa3d, 61 },
    { 0xe64eaa3e, 98 },
    { 0xdc4eaa3f, 43 },
    { 0, 0 },
    { 0xb69eaa41, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6eaa44, 55 },
    { 0x59ceaa45, 95 },
    { 0x5b7eaa45, 55 },
    { 0, 0 },
    { 0x8f3eaa48, 33 },
    { 0x8c0eaa49, 76 },
    { 0x2d8eaa4a, 99 },
    { 0xba1eaa4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213eaa55, 46 },
    { 0, 0 },
    { 0x81beaa57, 33 },
    { 0x60eaa58, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7eaa5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf89eaa5f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857eaa68, 95 },
    { 0x5fbeaa69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21eaa70, 61 },
    { 0x39ceaa71, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xd55eaa74, 34 },
    { 0, 0 },
    { 0xe38eaa76, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x863eaa7b, 23 },
    { 0xe16eaa7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6eaa7f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaeaa82, 90 },
    { 0xf2aeaa82, 34 },
    { 0x727eaa83, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aceaa87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbeaa8b, 33 },
    { 0xfd5eaa8c, 78 },
    { 0x5f1eaa8d, 61 },
    { 0xc11eaa8b, 98 },
    { 0, 0 },
    { 0x38feaa90, 95 },
    { 0x3feaa90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967eaa97, 23 },
    { 0x6d8eaa98, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeeeaa9b, 67 },
    { 0xb04eaa9c, 46 },
    { 0xa85eaa9b, 17 },
    { 0x843eaa9e, 90 },
    { 0x203eaa9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8eaaaa, 52 },
    { 0x83eaaaa, 61 },
    { 0x974eaaaa, 90 },
    { 0, 0 },
    { 0x87feaaae, 90 },
    { 0xc1beaaae, 77 },
    { 0xf79eaab0, 33 },
    { 0x1c1eaab1, 61 },
    { 0x850eaab2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8eaabc, 76 },
    { 0xc8eeaabc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x492eaac0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x80deaac3, 67 },
    { 0, 0 },
    { 0xbbceaac5, 66 },
    { 0, 0 },
    { 0x47feaac7, 43 },
    { 0, 0 },
    { 0x7b5eaac9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448eaace, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715eaad3, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eeaadc, 61 },
    { 0, 0 },
    { 0x40aeaade, 46 },
    { 0xe6eeaade, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5eaae4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x670eaae9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51eaaed, 53 },
    { 0xceaaee, 75 },
    { 0x7f3eaaee, 66 },
    { 0x9bbeaaf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x88eeaaf3, 55 },
    { 0x9eeeaaf4, 64 },
    { 0xdc4eaaf4, 62 },
    { 0xd8deaaf6, 90 },
    { 0xd68eaaf7, 23 },
    { 0x41ceaaf8, 95 },
    { 0x20ceaaf9, 61 },
    { 0x397eaaf9, 91 },
    { 0x6e1eaafa, 95 },
    { 0xf9beaafc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3eab00, 61 },
    { 0xf9ceab01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedeab05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x147eab08, 33 },
    { 0x997eab09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feab0f, 55 },
    { 0, 0 },
    { 0x452eab11, 43 },
    { 0, 0 },
    { 0x3d0eab13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0eab1b, 55 },
    { 0x54aeab1c, 17 },
    { 0x11beab1d, 100 },
    { 0, 0 },
    { 0x718eab1f, 90 },
    { 0x184eab1f, 98 },
    { 0, 0 },
    { 0x5c8eab22, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14eeab2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28eab2e, 90 },
    { 0xd9eab2f, 23 },
    { 0xb9eab30, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9eab37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883eab43, 95 },
    { 0, 0 },
    { 0x8cceab45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490eab4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5afeab50, 61 },
    { 0x5beeab51, 23 },
    { 0xb71eab52, 76 },
    { 0x905eab53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88deab59, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5eab5e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478eab63, 95 },
    { 0x200eab64, 80 },
    { 0xa40eab65, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4eab6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19beab70, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278eab76, 99 },
    { 0x994eab77, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36eab7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ceab7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabeab82, 95 },
    { 0xff7eab83, 93 },
    { 0x745eab84, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60beab88, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51eab8b, 38 },
    { 0xb09eab8c, 36 },
    { 0xe06eab8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feab90, 43 },
    { 0x8e7eab90, 85 },
    { 0x78feab92, 76 },
    { 0xc3deab92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9eab96, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x588eab99, 90 },
    { 0, 0 },
    { 0xd6deab9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6eaba4, 61 },
    { 0, 0 },
    { 0xf66eaba6, 99 },
    { 0, 0 },
    { 0x952eaba8, 23 },
    { 0xd63eaba9, 53 },
    { 0, 0 },
    { 0xd6beabab, 33 },
    { 0x846eabac, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaeabaf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1eabb6, 61 },
    { 0x821eabb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41eabc3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3eabc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8eabce, 95 },
    { 0xf59eabce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x767eabd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74beabd9, 23 },
    { 0xa3eabda, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5eabe1, 66 },
    { 0x795eabe2, 61 },
    { 0x976eabe3, 34 },
    { 0x810eabe4, 23 },
    { 0xe52eabe4, 90 },
    { 0xce2eabe6, 93 },
    { 0x19deabe3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x513eabea, 90 },
    { 0, 0 },
    { 0xf6beabec, 23 },
    { 0xf0aeabed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfeabf3, 60 },
    { 0x2e5eabf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x19aeabf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2eeabfa, 90 },
    { 0xeffeabfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2eac03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5beac0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aeac0f, 90 },
    { 0x9fceac0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12eac18, 55 },
    { 0, 0 },
    { 0x4a8eac1a, 33 },
    { 0, 0 },
    { 0xe72eac1c, 90 },
    { 0xe8deac1c, 55 },
    { 0x670eac1e, 23 },
    { 0x7caeac1e, 55 },
    { 0xd31eac20, 52 },
    { 0x58ceac21, 56 },
    { 0x311eac22, 34 },
    { 0x98deac23, 90 },
    { 0xb66eac1d, 66 },
    { 0x31eac23, 55 },
    { 0xf23eac20, 23 },
    { 0xa60eac27, 76 },
    { 0x3abeac28, 76 },
    { 0xc30eac28, 52 },
    { 0xeb0eac2a, 76 },
    { 0xab8eac2b, 52 },
    { 0x3bdeac1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4deac30, 90 },
    { 0x4adeac30, 76 },
    { 0, 0 },
    { 0xe8eac33, 52 },
    { 0x9b0eac34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9eac39, 98 },
    { 0x675eac3a, 61 },
    { 0x694eac3b, 95 },
    { 0x7dfeac3c, 43 },
    { 0xc3eac3d, 90 },
    { 0x7c4eac3e, 38 },
    { 0x52ceac3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adeac44, 46 },
    { 0xe6ceac45, 90 },
    { 0, 0 },
    { 0xbd9eac47, 33 },
    { 0xf5eeac48, 46 },
    { 0, 0 },
    { 0x31aeac4a, 91 },
    { 0xb4eac4b, 55 },
    { 0xaafeac4c, 61 },
    { 0x40feac4d, 76 },
    { 0x871eac4e, 33 },
    { 0, 0 },
    { 0xeb0eac50, 43 },
    { 0xddceac51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9eac56, 33 },
    { 0xde4eac57, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183eac5b, 98 },
    { 0, 0 },
    { 0x464eac5d, 40 },
    { 0, 0 },
    { 0xc4eeac5f, 90 },
    { 0, 0 },
    { 0xd29eac61, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6beac64, 61 },
    { 0xd8deac65, 93 },
    { 0x14eac66, 55 },
    { 0, 0 },
    { 0xc1deac68, 95 },
    { 0x5b5eac69, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34eac72, 90 },
    { 0x4c3eac73, 76 },
    { 0xe87eac73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faeac77, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1eac7a, 61 },
    { 0xe9feac7b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3eac7e, 76 },
    { 0, 0 },
    { 0x783eac80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4eac84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ceac88, 61 },
    { 0x13beac88, 61 },
    { 0, 0 },
    { 0x95aeac8b, 95 },
    { 0xd38eac8c, 55 },
    { 0, 0 },
    { 0x466eac8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750eac93, 61 },
    { 0xd61eac93, 100 },
    { 0x443eac94, 55 },
    { 0x172eac96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x68aeac99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70eac9c, 52 },
    { 0x140eac9c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922eaca2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eeacaa, 43 },
    { 0x735eacab, 34 },
    { 0, 0 },
    { 0x6b3eacad, 33 },
    { 0, 0 },
    { 0x83aeacaf, 90 },
    { 0x922eacaf, 34 },
    { 0x655eacb1, 91 },
    { 0, 0 },
    { 0x2e2eacb3, 90 },
    { 0xda1eacb4, 90 },
    { 0x71beacb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5eacbd, 33 },
    { 0xbbceacbd, 23 },
    { 0xc6aeacbd, 23 },
    { 0x717eacc0, 88 },
    { 0, 0 },
    { 0x247eacc2, 88 },
    { 0xfe0eacc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6eacc9, 23 },
    { 0x343eacca, 61 },
    { 0x4eaccb, 88 },
    { 0x2e7eaccb, 95 },
    { 0xacdeaccc, 95 },
    { 0x936eaccb, 33 },
    { 0, 0 },
    { 0xffeeacd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eacdb, 23 },
    { 0x85eeacdc, 100 },
    { 0x640eacdd, 55 },
    { 0xfffeacde, 81 },
    { 0, 0 },
    { 0x8aeace0, 23 },
    { 0x207eace0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0eace5, 95 },
    { 0, 0 },
    { 0xb57eace7, 99 },
    { 0xddbeace8, 95 },
    { 0x72feace9, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08eaced, 33 },
    { 0x64feacee, 66 },
    { 0, 0 },
    { 0xd4feacf0, 33 },
    { 0x4d9eacf1, 33 },
    { 0xfb3eacf1, 95 },
    { 0xee4eacf3, 90 },
    { 0, 0 },
    { 0xb13eacf5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6eacf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12eacfc, 84 },
    { 0, 0 },
    { 0x51eacfe, 33 },
    { 0xb46eacff, 90 },
    { 0x3d1ead00, 61 },
    { 0x491eacfe, 93 },
    { 0xe6aead02, 66 },
    { 0xb53eacff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2ead08, 33 },
    { 0, 0 },
    { 0x98bead0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d1ead0f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0ead13, 93 },
    { 0, 0 },
    { 0xf53ead15, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x119ead19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3deead1c, 84 },
    { 0xd37ead1d, 33 },
    { 0xdd2ead1e, 55 },
    { 0x9f4ead1e, 23 },
    { 0x1c6ead20, 98 },
    { 0xbffead1d, 90 },
    { 0xe75ead1d, 98 },
    { 0x75bead1e, 61 },
    { 0xf5bead24, 55 },
    { 0xb4dead24, 87 },
    { 0xf6cead26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146ead2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9ead32, 55 },
    { 0x7ccead33, 66 },
    { 0x7b6ead34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ead37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaead3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423ead42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x661ead45, 76 },
    { 0xd0ead46, 61 },
    { 0xbc0ead47, 65 },
    { 0, 0 },
    { 0x554ead49, 33 },
    { 0x84dead4a, 55 },
    { 0xf99ead4b, 98 },
    { 0xb16ead4c, 33 },
    { 0x816ead4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34ead50, 36 },
    { 0, 0 },
    { 0xa7ead52, 40 },
    { 0x37cead52, 99 },
    { 0x50ead54, 55 },
    { 0xf2fead54, 95 },
    { 0x3beead56, 88 },
    { 0x44aead57, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2ead5d, 61 },
    { 0xd31ead5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ead68, 93 },
    { 0x2c2ead68, 78 },
    { 0xeffead68, 87 },
    { 0x971ead6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68ead6e, 55 },
    { 0xb69ead6f, 46 },
    { 0x1e4ead6f, 99 },
    { 0, 0 },
    { 0x8dbead72, 91 },
    { 0x573ead73, 93 },
    { 0xfd1ead74, 95 },
    { 0x202ead75, 52 },
    { 0xf01ead75, 76 },
    { 0x1dcead77, 95 },
    { 0xe31ead77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309ead7f, 67 },
    { 0, 0 },
    { 0x838ead81, 90 },
    { 0xb97ead81, 55 },
    { 0xd19ead82, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70ead89, 90 },
    { 0xaa0ead8a, 40 },
    { 0x2deead8a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432ead90, 100 },
    { 0xec0ead91, 48 },
    { 0x6e4ead92, 76 },
    { 0xb11ead92, 33 },
    { 0, 0 },
    { 0x3e7ead95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8ead9a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2eada5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8eadaa, 99 },
    { 0x693eadab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eeadb1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afeadb4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96eadb7, 84 },
    { 0x451eadb8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8deadbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96eadc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4eadc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x684eadcc, 100 },
    { 0, 0 },
    { 0xd29eadce, 90 },
    { 0, 0 },
    { 0xffaeadd0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9aeadd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83eadd8, 55 },
    { 0xc6eadd9, 23 },
    { 0x612eadda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87eaddd, 33 },
    { 0x909eadde, 61 },
    { 0xea6eaddf, 95 },
    { 0x38eeaddf, 96 },
    { 0xb4aeaddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2eade4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ceade8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xabdeadeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8eadee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30eadf5, 100 },
    { 0, 0 },
    { 0xac1eadf7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9eadfc, 98 },
    { 0x28eeadfc, 90 },
    { 0, 0 },
    { 0xc0aeadff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8feae06, 98 },
    { 0, 0 },
    { 0x2cbeae08, 33 },
    { 0xf52eae09, 76 },
    { 0, 0 },
    { 0xa25eae0b, 33 },
    { 0x671eae0b, 76 },
    { 0xc43eae0d, 34 },
    { 0xfc7eae0d, 95 },
    { 0x5f2eae0f, 33 },
    { 0x7fceae10, 61 },
    { 0xfc9eae0e, 61 },
    { 0, 0 },
    { 0x5aeae13, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326eae17, 33 },
    { 0, 0 },
    { 0xf18eae19, 95 },
    { 0xcbdeae19, 81 },
    { 0, 0 },
    { 0x36aeae1c, 95 },
    { 0, 0 },
    { 0x9b9eae1e, 90 },
    { 0xdf8eae1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeeae23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ceae29, 90 },
    { 0xdf7eae29, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51eeae31, 98 },
    { 0x7baeae32, 98 },
    { 0, 0 },
    { 0x5eceae34, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdeae37, 23 },
    { 0, 0 },
    { 0x7c9eae39, 33 },
    { 0, 0 },
    { 0x76eae3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x49beae3e, 34 },
    { 0xc3deae3f, 84 },
    { 0, 0 },
    { 0xfcdeae41, 61 },
    { 0x3a9eae42, 90 },
    { 0xb6eae41, 36 },
    { 0, 0 },
    { 0xc63eae45, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x868eae48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefeae4c, 75 },
    { 0, 0 },
    { 0x274eae4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530eae53, 95 },
    { 0x244eae54, 34 },
    { 0x17eae55, 55 },
    { 0xec8eae55, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cceae59, 33 },
    { 0xc8aeae5a, 90 },
    { 0x5fdeae5a, 77 },
    { 0xfa1eae5b, 76 },
    { 0xb63eae59, 33 },
    { 0xc72eae5e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eeae62, 55 },
    { 0x633eae63, 49 },
    { 0x2dceae63, 95 },
    { 0x72feae62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf4eae68, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7eae6e, 55 },
    { 0x84eae6f, 66 },
    { 0, 0 },
    { 0x66eae71, 46 },
    { 0, 0 },
    { 0x8f4eae73, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eae76, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98eae7d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19deae80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312eae8a, 23 },
    { 0, 0 },
    { 0x4e3eae8c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beeae91, 100 },
    { 0xcd6eae92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x572eae95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1eae99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852eaea1, 78 },
    { 0xcf8eaea2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ceaea5, 55 },
    { 0x182eaea6, 23 },
    { 0xd08eaea5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8eaeac, 85 },
    { 0xd1feaead, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10eaeb0, 46 },
    { 0x6a9eaeb0, 33 },
    { 0xaa6eaeb2, 34 },
    { 0x950eaeb1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72eaeb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175eaebd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x438eaec0, 43 },
    { 0, 0 },
    { 0xc2beaec2, 90 },
    { 0x5c2eaec3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127eaec9, 78 },
    { 0, 0 },
    { 0x6bceaecb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2beeaed0, 34 },
    { 0x67eaed0, 78 },
    { 0xeeeeaed0, 95 },
    { 0, 0 },
    { 0xf2beaed4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7eaedd, 88 },
    { 0x840eaede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eaee8, 33 },
    { 0, 0 },
    { 0x402eaeea, 81 },
    { 0, 0 },
    { 0xdafeaeec, 100 },
    { 0x7a6eaeed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aeeaef1, 95 },
    { 0, 0 },
    { 0x4c4eaef3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6eaef6, 100 },
    { 0, 0 },
    { 0xafeaef8, 61 },
    { 0, 0 },
    { 0xbeaeaefa, 95 },
    { 0xd39eaefa, 99 },
    { 0xf9ceaefb, 23 },
    { 0, 0 },
    { 0xfaaeaefe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0eaf04, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9eaf08, 90 },
    { 0, 0 },
    { 0xf34eaf0a, 61 },
    { 0x2eeeaf0a, 33 },
    { 0x650eaf0a, 98 },
    { 0x5dceaf0d, 33 },
    { 0x201eaf0a, 93 },
    { 0x57eeaf0f, 61 },
    { 0xd5deaf10, 33 },
    { 0xd7eeaf0e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfeaf17, 77 },
    { 0x40ceaf18, 33 },
    { 0x3a3eaf19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746eaf23, 33 },
    { 0x12eaf24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6eaf2a, 33 },
    { 0, 0 },
    { 0x2c0eaf2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118eaf30, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abeaf37, 33 },
    { 0xc56eaf38, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15beaf42, 82 },
    { 0x35deaf43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x592eaf46, 95 },
    { 0xf1eeaf46, 90 },
    { 0, 0 },
    { 0x58feaf49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97eaf4c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ceaf4f, 97 },
    { 0x51eeaf4f, 61 },
    { 0xad2eaf51, 87 },
    { 0, 0 },
    { 0x565eaf53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x855eaf56, 46 },
    { 0x2d2eaf57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x663eaf5e, 98 },
    { 0xb22eaf5e, 43 },
    { 0xe3eeaf5f, 55 },
    { 0x3c0eaf5e, 90 },
    { 0, 0 },
    { 0x3cbeaf63, 76 },
    { 0, 0 },
    { 0xe69eaf65, 95 },
    { 0, 0 },
    { 0x8c1eaf67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbeaf6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6eaf70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04eaf79, 55 },
    { 0x5e9eaf7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeeaf80, 52 },
    { 0xf13eaf80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117eaf89, 33 },
    { 0xbc5eaf8a, 90 },
    { 0x9a6eaf8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bceaf8e, 33 },
    { 0xd9aeaf8f, 67 },
    { 0xd37eaf90, 91 },
    { 0xd8eaf8e, 90 },
    { 0x679eaf92, 90 },
    { 0, 0 },
    { 0x313eaf94, 52 },
    { 0, 0 },
    { 0x3adeaf96, 90 },
    { 0x79feaf96, 95 },
    { 0x4f8eaf97, 33 },
    { 0xdf7eaf97, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedeaf9d, 95 },
    { 0x69ceaf9d, 65 },
    { 0, 0 },
    { 0x93deafa0, 33 },
    { 0x13deafa0, 76 },
    { 0, 0 },
    { 0x1a8eafa3, 23 },
    { 0x916eafa4, 67 },
    { 0x183eafa4, 76 },
    { 0x14feafa5, 61 },
    { 0x95feafa7, 85 },
    { 0x9ebeafa8, 93 },
    { 0x22eeafa8, 92 },
    { 0x8cbeafa3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40eafad, 75 },
    { 0, 0 },
    { 0x340eafaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01eafb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264eafb7, 33 },
    { 0xc90eafb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ceafbb, 33 },
    { 0x6ddeafbc, 94 },
    { 0x48deafbb, 40 },
    { 0x3d0eafbe, 7 },
    { 0, 0 },
    { 0, 0 },
    { 0xceeafc1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1eafc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6eafc9, 81 },
    { 0, 0 },
    { 0x42aeafcb, 34 },
    { 0x41aeafcc, 33 },
    { 0x2ffeafcc, 98 },
    { 0x101eafce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eafdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8beafde, 98 },
    { 0xdb5eafdf, 78 },
    { 0x2d5eafdf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50feafec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30eafef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9eaff4, 34 },
    { 0, 0 },
    { 0x46feaff6, 33 },
    { 0, 0 },
    { 0x409eaff8, 95 },
    { 0x1e3eaff8, 89 },
    { 0, 0 },
    { 0x45ceaffb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beafff, 95 },
    { 0, 0 },
    { 0x7eceb001, 95 },
    { 0xd1ceb002, 33 },
    { 0, 0 },
    { 0xc9beb004, 90 },
    { 0x39aeb004, 33 },
    { 0, 0 },
    { 0x4cceb007, 98 },
    { 0xdc7eb008, 75 },
    { 0xbc9eb009, 98 },
    { 0x889eb00a, 52 },
    { 0xe5eb00a, 76 },
    { 0x5a1eb00c, 95 },
    { 0xdaeb00d, 61 },
    { 0x7a1eb00c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc10eb011, 88 },
    { 0x124eb011, 90 },
    { 0x953eb013, 90 },
    { 0, 0 },
    { 0xfcfeb015, 43 },
    { 0, 0 },
    { 0xfe5eb017, 52 },
    { 0x1e9eb018, 87 },
    { 0xa27eb018, 23 },
    { 0xa88eb01a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eb021, 98 },
    { 0x669eb022, 77 },
    { 0x44feb022, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfeb029, 23 },
    { 0x7eceb02a, 61 },
    { 0x89beb02a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1feb033, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415eb039, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfeb03d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3eb042, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aeb045, 84 },
    { 0x3a7eb045, 76 },
    { 0, 0 },
    { 0xaf0eb048, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36eb04b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9eb050, 61 },
    { 0, 0 },
    { 0xfceeb052, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38eb055, 46 },
    { 0x732eb056, 33 },
    { 0x359eb056, 98 },
    { 0, 0 },
    { 0x11eb059, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x15beb05c, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x78feb05f, 84 },
    { 0, 0 },
    { 0x9d3eb061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5eb064, 91 },
    { 0xac4eb065, 75 },
    { 0x9ceb065, 33 },
    { 0x93ceb067, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x358eb06f, 48 },
    { 0xd4eeb06f, 55 },
    { 0xfe4eb071, 61 },
    { 0, 0 },
    { 0xdd8eb073, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b3eb076, 61 },
    { 0x1c1eb076, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6eb07a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9eb07d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ceb080, 89 },
    { 0xd3feb081, 34 },
    { 0x9c7eb082, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fbeb086, 95 },
    { 0xccaeb087, 55 },
    { 0xfdceb088, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1eb08f, 33 },
    { 0xeeceb090, 33 },
    { 0x289eb08f, 33 },
    { 0x9cdeb092, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e4eb095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x294eb098, 33 },
    { 0, 0 },
    { 0x14aeb09a, 55 },
    { 0xc0aeb09b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23eb0a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29eb0a5, 17 },
    { 0, 0 },
    { 0x161eb0a7, 95 },
    { 0x265eb0a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7eb0ab, 95 },
    { 0x53feb0ac, 33 },
    { 0x8a4eb0ad, 90 },
    { 0x3ceb0ad, 100 },
    { 0xf19eb0af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1eb0b3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713eb0b9, 90 },
    { 0x26beb0ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63eb0be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677eb0c2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76eb0c7, 33 },
    { 0xe96eb0c8, 99 },
    { 0xd38eb0c8, 98 },
    { 0x717eb0ca, 67 },
    { 0x2f7eb0c7, 92 },
    { 0, 0 },
    { 0x1e5eb0cd, 95 },
    { 0xfcceb0cd, 33 },
    { 0xb49eb0cf, 90 },
    { 0, 0 },
    { 0x279eb0d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x614eb0d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2eeb0d8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04eb0dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedeb0df, 33 },
    { 0xeffeb0e0, 66 },
    { 0, 0 },
    { 0x24aeb0e2, 91 },
    { 0x8a3eb0e3, 95 },
    { 0, 0 },
    { 0x17beb0e5, 42 },
    { 0x58eeb0e6, 46 },
    { 0x2f0eb0e7, 33 },
    { 0xc9eb0e8, 90 },
    { 0, 0 },
    { 0xa50eb0ea, 81 },
    { 0, 0 },
    { 0xc53eb0ec, 85 },
    { 0xdceeb0ed, 34 },
    { 0, 0 },
    { 0xb57eb0ef, 33 },
    { 0, 0 },
    { 0x52eeb0f1, 52 },
    { 0, 0 },
    { 0xbeceb0f3, 75 },
    { 0, 0 },
    { 0x35feb0f5, 93 },
    { 0x8eb0f6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7eb0ff, 40 },
    { 0xd1eeb0ff, 61 },
    { 0, 0 },
    { 0x542eb102, 90 },
    { 0, 0 },
    { 0xa8feb104, 76 },
    { 0x984eb105, 96 },
    { 0xa13eb106, 95 },
    { 0x283eb106, 90 },
    { 0xb25eb108, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62eb10c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfeb10f, 68 },
    { 0x306eb110, 46 },
    { 0xf0beb111, 81 },
    { 0x7c1eb110, 100 },
    { 0, 0 },
    { 0x72feb114, 38 },
    { 0xc2eeb114, 90 },
    { 0, 0 },
    { 0x45eeb117, 90 },
    { 0x2c8eb117, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4eb11b, 95 },
    { 0xf67eb11b, 90 },
    { 0x2f2eb11d, 38 },
    { 0xda3eb11e, 55 },
    { 0xc26eb11d, 33 },
    { 0x145eb120, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdeb128, 90 },
    { 0, 0 },
    { 0x73aeb12a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddeb12f, 66 },
    { 0x7a3eb12f, 98 },
    { 0x145eb131, 33 },
    { 0, 0 },
    { 0xf9beb133, 90 },
    { 0x29beb133, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x945eb137, 34 },
    { 0, 0 },
    { 0xcc0eb139, 33 },
    { 0x195eb13a, 52 },
    { 0x5dbeb13a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eeb140, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3eb145, 52 },
    { 0xf4ceb146, 35 },
    { 0x5d7eb146, 76 },
    { 0x2c9eb148, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x300eb14b, 55 },
    { 0xda8eb14c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7eb158, 61 },
    { 0, 0 },
    { 0x173eb15a, 43 },
    { 0, 0 },
    { 0x53eeb15c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ceb160, 90 },
    { 0x395eb161, 75 },
    { 0, 0 },
    { 0xf90eb163, 67 },
    { 0x275eb164, 76 },
    { 0xa5eb163, 33 },
    { 0, 0 },
    { 0xb25eb167, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2eb16b, 17 },
    { 0, 0 },
    { 0xa6aeb16d, 95 },
    { 0xd5eb16e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3eb172, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf4eb176, 69 },
    { 0xd6deb176, 100 },
    { 0x156eb178, 88 },
    { 0x81eb178, 90 },
    { 0x20ceb17a, 98 },
    { 0x66eeb17b, 95 },
    { 0xaaeb17b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98eb183, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eb187, 100 },
    { 0xa2eb187, 34 },
    { 0xe54eb187, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdeb18c, 61 },
    { 0x764eb18c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deb199, 90 },
    { 0x748eb19a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aeb1a7, 30 },
    { 0x6a0eb1a7, 34 },
    { 0xbeeb1a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99feb1b1, 76 },
    { 0xf87eb1b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x703eb1b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59eb1bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefeeb1c0, 46 },
    { 0x442eb1c1, 33 },
    { 0x706eb1c2, 55 },
    { 0xd70eb1c3, 23 },
    { 0xae4eb1c3, 17 },
    { 0, 0 },
    { 0x934eb1c6, 23 },
    { 0x535eb1c7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485eb1d5, 33 },
    { 0, 0 },
    { 0xf2deb1d7, 33 },
    { 0xb8aeb1d8, 51 },
    { 0, 0 },
    { 0x314eb1da, 33 },
    { 0xe43eb1db, 90 },
    { 0, 0 },
    { 0xbb3eb1dd, 76 },
    { 0x7e2eb1de, 61 },
    { 0x97eb1de, 91 },
    { 0xc04eb1e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfeb1e3, 45 },
    { 0x161eb1e4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x59eb1e7, 55 },
    { 0xb65eb1e8, 81 },
    { 0xceeb1e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ceb1f1, 77 },
    { 0xa40eb1f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787eb1fa, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51eb1fd, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45deb208, 67 },
    { 0, 0 },
    { 0xc0aeb20a, 99 },
    { 0, 0 },
    { 0x9f4eb20c, 90 },
    { 0x53eb20d, 33 },
    { 0x9b0eb20c, 23 },
    { 0x66feb20f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0eb214, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434eb218, 90 },
    { 0x212eb219, 23 },
    { 0, 0 },
    { 0xd69eb21b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40eb21e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2eb227, 43 },
    { 0, 0 },
    { 0x7e3eb229, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceb235, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97beb238, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833eb23c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11eb247, 52 },
    { 0, 0 },
    { 0xeafeb249, 33 },
    { 0xd0aeb24a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5eb255, 46 },
    { 0, 0 },
    { 0x3ddeb257, 37 },
    { 0xe57eb258, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16eb25d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aeb262, 90 },
    { 0xd24eb263, 61 },
    { 0xe9feb263, 78 },
    { 0x215eb265, 95 },
    { 0, 0 },
    { 0xb8eeb267, 95 },
    { 0x417eb268, 52 },
    { 0, 0 },
    { 0x542eb26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3aeb26d, 98 },
    { 0x8deeb26e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaeb272, 43 },
    { 0x18aeb273, 64 },
    { 0x878eb274, 98 },
    { 0xd4ceb275, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2eb278, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66feb27e, 52 },
    { 0, 0 },
    { 0xf33eb280, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7eb283, 61 },
    { 0x5e6eb284, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebeb28a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ceb28d, 43 },
    { 0x549eb28e, 46 },
    { 0x634eb28f, 44 },
    { 0, 0 },
    { 0x684eb291, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65eb295, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67beb299, 43 },
    { 0x9d9eb29a, 23 },
    { 0x2dceb29b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ceb2a1, 90 },
    { 0xd10eb2a2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491eb2a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8eb2b5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993eb2b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5eb2bf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7eb2c5, 33 },
    { 0x68feb2c6, 88 },
    { 0x3c0eb2c5, 33 },
    { 0x849eb2c8, 55 },
    { 0x75aeb2c7, 34 },
    { 0, 0 },
    { 0x68beb2cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319eb2d0, 95 },
    { 0x88eb2d1, 61 },
    { 0x22deb2d2, 77 },
    { 0xdb4eb2d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6eb2d7, 46 },
    { 0x175eb2d8, 66 },
    { 0x1c6eb2d8, 90 },
    { 0xdceeb2da, 61 },
    { 0x5d0eb2da, 61 },
    { 0x2adeb2dc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x807eb2df, 33 },
    { 0, 0 },
    { 0x8f1eb2e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949eb2e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3aeb2ee, 98 },
    { 0x69beb2ef, 88 },
    { 0x8afeb2f0, 98 },
    { 0xcaceb2f1, 98 },
    { 0xa33eb2f1, 98 },
    { 0x7c9eb2f0, 23 },
    { 0x128eb2f4, 76 },
    { 0x13eeb2f5, 76 },
    { 0x158eb2f2, 90 },
    { 0xc14eb2f7, 98 },
    { 0x876eb2f6, 76 },
    { 0x1c8eb2f9, 76 },
    { 0xc2ceb2fa, 23 },
    { 0xfb8eb2f3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aeb2fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5eb306, 77 },
    { 0x5baeb306, 78 },
    { 0xd7beb307, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83eb30b, 46 },
    { 0x230eb30b, 91 },
    { 0xffaeb30c, 23 },
    { 0x369eb30b, 90 },
    { 0, 0 },
    { 0x52deb310, 98 },
    { 0, 0 },
    { 0x3baeb312, 99 },
    { 0x947eb312, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eb316, 23 },
    { 0x54eb317, 98 },
    { 0xa1feb318, 53 },
    { 0xcedeb318, 66 },
    { 0xe25eb316, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x77beb31d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ceb321, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953eb32c, 93 },
    { 0xd20eb32d, 33 },
    { 0xb03eb32c, 23 },
    { 0, 0 },
    { 0x6c6eb330, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09eb333, 23 },
    { 0x40beb333, 100 },
    { 0x77feb335, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3feb338, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29eb33b, 80 },
    { 0x47feb33b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64feb344, 97 },
    { 0x557eb345, 23 },
    { 0x75deb346, 76 },
    { 0, 0 },
    { 0x8a8eb348, 84 },
    { 0, 0 },
    { 0xcb7eb34a, 33 },
    { 0xd25eb34a, 8 },
    { 0x476eb34a, 98 },
    { 0x38eb34c, 76 },
    { 0xec1eb34a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171eb352, 95 },
    { 0, 0 },
    { 0x68feb354, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02eb359, 69 },
    { 0x4f0eb35a, 46 },
    { 0x6feeb35b, 88 },
    { 0x7e2eb35c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eb36a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92eb36f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7eb374, 98 },
    { 0, 0 },
    { 0xa66eb376, 33 },
    { 0x768eb377, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaeb37d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509eb385, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698eb38d, 99 },
    { 0, 0 },
    { 0x82beb38f, 23 },
    { 0x66feb38f, 23 },
    { 0, 0 },
    { 0x531eb392, 87 },
    { 0xfb3eb393, 33 },
    { 0xa5eeb394, 87 },
    { 0, 0 },
    { 0xe9aeb396, 33 },
    { 0xb53eb396, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x381eb39a, 33 },
    { 0xd5eeb39b, 23 },
    { 0, 0 },
    { 0xe92eb39d, 98 },
    { 0xb1ceb39e, 77 },
    { 0x532eb39e, 33 },
    { 0xa62eb39e, 52 },
    { 0x4d8eb3a1, 95 },
    { 0x1caeb39e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cceb3a5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaeb3ac, 77 },
    { 0xa37eb3ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4eb3b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eb3b4, 85 },
    { 0, 0 },
    { 0x46deb3b6, 23 },
    { 0x849eb3b7, 98 },
    { 0, 0 },
    { 0x108eb3b9, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09eb3be, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00eb3c4, 95 },
    { 0xaa4eb3c5, 23 },
    { 0xa4ceb3c6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x76beb3c9, 90 },
    { 0, 0 },
    { 0x49ceb3cb, 33 },
    { 0x201eb3cb, 43 },
    { 0x314eb3cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80feb3d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0eb3d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121eb3d8, 23 },
    { 0x91beb3d9, 100 },
    { 0x6e4eb3da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eceb3e0, 90 },
    { 0, 0 },
    { 0x80eeb3e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3eb3e8, 90 },
    { 0x898eb3e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1eb3ec, 85 },
    { 0x31ceb3ed, 55 },
    { 0, 0 },
    { 0x7aceb3ef, 76 },
    { 0xa74eb3f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aceb3f5, 91 },
    { 0xc25eb3f6, 34 },
    { 0, 0 },
    { 0xc57eb3f8, 69 },
    { 0x864eb3f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efeb3fc, 99 },
    { 0, 0 },
    { 0xb1beb3fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9eb401, 46 },
    { 0x63eeb402, 66 },
    { 0x5b9eb401, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x39feb406, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3eb409, 38 },
    { 0xceb409, 61 },
    { 0x8b4eb40a, 66 },
    { 0x7b3eb40c, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38eb411, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2deb41a, 84 },
    { 0x484eb41a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458eb41f, 33 },
    { 0xeb5eb41f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48eb423, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78aeb42b, 76 },
    { 0xc13eb42c, 91 },
    { 0xe2eb42c, 91 },
    { 0x4e7eb42e, 90 },
    { 0xf63eb42f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38eb435, 90 },
    { 0x928eb436, 78 },
    { 0x1c6eb437, 90 },
    { 0xe05eb435, 55 },
    { 0, 0 },
    { 0x230eb43a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8eb43e, 75 },
    { 0x170eb43f, 90 },
    { 0, 0 },
    { 0xc79eb441, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78deb444, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824eb448, 67 },
    { 0x465eb449, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x324eb44c, 33 },
    { 0, 0 },
    { 0x8deeb44e, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x447eb451, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x51aeb454, 74 },
    { 0x40beb454, 98 },
    { 0x96eeb456, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0eb459, 95 },
    { 0x155eb459, 90 },
    { 0xe1beb45b, 23 },
    { 0x2faeb45b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366eb466, 4 },
    { 0, 0 },
    { 0xb18eb468, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37eb46b, 55 },
    { 0x8c7eb46c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32aeb470, 33 },
    { 0, 0 },
    { 0x97aeb472, 61 },
    { 0x576eb473, 33 },
    { 0xc23eb474, 55 },
    { 0xdfdeb475, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187eb47b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beb481, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6eb485, 61 },
    { 0x37aeb486, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7eb48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46aeb492, 8 },
    { 0, 0 },
    { 0xa41eb494, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7eb497, 90 },
    { 0, 0 },
    { 0x1e6eb499, 99 },
    { 0xa38eb49a, 95 },
    { 0x468eb49b, 90 },
    { 0x53aeb49c, 100 },
    { 0x8bdeb49d, 90 },
    { 0xd1beb49e, 90 },
    { 0xa7deb49e, 61 },
    { 0x19ceb4a0, 33 },
    { 0x509eb49f, 23 },
    { 0xda6eb4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96feb4a6, 38 },
    { 0x95deb4a6, 95 },
    { 0x1c6eb4a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x69eb4ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5eb4ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbeb4b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eeb4b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eceb4bd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314eb4c5, 61 },
    { 0xecfeb4c6, 76 },
    { 0x791eb4c7, 95 },
    { 0xa6ceb4c6, 61 },
    { 0x389eb4c9, 23 },
    { 0x28feb4ca, 33 },
    { 0xbb6eb4cb, 33 },
    { 0x54eeb4cb, 61 },
    { 0x5eb4ca, 44 },
    { 0xdc5eb4ce, 61 },
    { 0x7d4eb4cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462eb4d5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffeb4da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x394eb4e2, 78 },
    { 0x7d0eb4e3, 76 },
    { 0, 0 },
    { 0xacdeb4e5, 61 },
    { 0, 0 },
    { 0x9daeb4e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b4eb4ea, 76 },
    { 0, 0 },
    { 0x8e7eb4ec, 43 },
    { 0x19beb4ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63deb4f5, 87 },
    { 0xfe4eb4f6, 91 },
    { 0xcafeb4f6, 95 },
    { 0x340eb4f8, 87 },
    { 0x5a9eb4f8, 76 },
    { 0x82aeb4f5, 98 },
    { 0x498eb4fb, 76 },
    { 0, 0 },
    { 0xc8deb4fd, 43 },
    { 0xb4feb4fd, 95 },
    { 0, 0 },
    { 0xbe1eb500, 23 },
    { 0, 0 },
    { 0x3cceb502, 46 },
    { 0x278eb503, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21deb507, 64 },
    { 0xcbfeb507, 33 },
    { 0x6dfeb507, 98 },
    { 0, 0 },
    { 0xb43eb50b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44eb513, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x44aeb516, 34 },
    { 0x725eb517, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8eb51e, 88 },
    { 0x6c5eb51e, 76 },
    { 0xd3eeb51e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aeb523, 23 },
    { 0, 0 },
    { 0x40eb525, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318eb52a, 90 },
    { 0, 0 },
    { 0x728eb52c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94eb533, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbeb537, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aeb53b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3eeb541, 52 },
    { 0x32eeb542, 95 },
    { 0xcdaeb543, 95 },
    { 0xd5feb544, 98 },
    { 0x7cceb544, 85 },
    { 0xf0ceb545, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deb54a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ffeb554, 76 },
    { 0x741eb554, 100 },
    { 0x1b7eb556, 23 },
    { 0x67feb557, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0eb55b, 84 },
    { 0xf21eb55b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x23deb55f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47eb563, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ceb56c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0eb571, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eeb57e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6aeb583, 43 },
    { 0x651eb584, 36 },
    { 0xe41eb584, 33 },
    { 0x439eb586, 90 },
    { 0x6e7eb585, 33 },
    { 0, 0 },
    { 0xdc4eb589, 75 },
    { 0, 0 },
    { 0x555eb58b, 78 },
    { 0, 0 },
    { 0xf90eb58d, 17 },
    { 0xae2eb58e, 75 },
    { 0x755eb58e, 90 },
    { 0xc46eb58e, 52 },
    { 0x1e1eb591, 75 },
    { 0, 0 },
    { 0x93feb593, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeeb59b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd81eb5a0, 90 },
    { 0xfa6eb5a1, 33 },
    { 0x738eb5a0, 90 },
    { 0x910eb5a0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ceb5aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9eeb5b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112eb5b5, 90 },
    { 0xc14eb5b6, 99 },
    { 0xd9feb5b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaeb5bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3eb5be, 34 },
    { 0, 0 },
    { 0x774eb5c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5aeb5c5, 61 },
    { 0xcbeb5c5, 90 },
    { 0x5eaeb5c7, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20eb5cc, 61 },
    { 0x109eb5cc, 33 },
    { 0, 0 },
    { 0x572eb5cf, 100 },
    { 0x5b1eb5d0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfeb5d6, 95 },
    { 0, 0 },
    { 0xceeeb5d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66eb5db, 80 },
    { 0x7cfeb5dc, 33 },
    { 0x60beb5dc, 33 },
    { 0x9deeb5db, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585eb5e3, 33 },
    { 0, 0 },
    { 0x70feb5e5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323eb5e9, 61 },
    { 0, 0 },
    { 0xc44eb5eb, 95 },
    { 0xfa4eb5eb, 90 },
    { 0x13aeb5ed, 90 },
    { 0, 0 },
    { 0xf5eb5ef, 33 },
    { 0xe21eb5f0, 33 },
    { 0xcc1eb5ef, 90 },
    { 0x6faeb5f2, 43 },
    { 0x496eb5f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aeb5f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feeb5fc, 78 },
    { 0xc80eb5fc, 33 },
    { 0x6cceb5fc, 98 },
    { 0x9d3eb5ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18eb606, 99 },
    { 0x8a1eb607, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4eb60e, 55 },
    { 0x9c2eb60f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88eb612, 33 },
    { 0xe75eb612, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71deb618, 95 },
    { 0, 0 },
    { 0xe4deb61a, 61 },
    { 0x904eb61b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237eb621, 100 },
    { 0x6bceb622, 98 },
    { 0x1edeb623, 88 },
    { 0, 0 },
    { 0x11beb625, 46 },
    { 0xd0eeb625, 61 },
    { 0x50beb627, 61 },
    { 0xe4deb628, 100 },
    { 0xc74eb627, 61 },
    { 0x1e5eb627, 99 },
    { 0x231eb629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53eb62f, 90 },
    { 0xb42eb630, 90 },
    { 0x4d9eb631, 76 },
    { 0xb7ceb632, 38 },
    { 0xf21eb62f, 95 },
    { 0, 0 },
    { 0xe5deb635, 98 },
    { 0xcaceb636, 95 },
    { 0, 0 },
    { 0xd5eeb638, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51eb63f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9beb648, 95 },
    { 0xb99eb649, 95 },
    { 0, 0 },
    { 0xf16eb64b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eeb653, 95 },
    { 0x604eb653, 46 },
    { 0x449eb655, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24eeb659, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87aeb65d, 33 },
    { 0x9c9eb65d, 33 },
    { 0x5edeb65f, 23 },
    { 0x7c9eb660, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73beb664, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6eb668, 17 },
    { 0, 0 },
    { 0x225eb66a, 33 },
    { 0x501eb66b, 90 },
    { 0, 0 },
    { 0x804eb66d, 61 },
    { 0, 0 },
    { 0xf7eeb66f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x440eb673, 55 },
    { 0, 0 },
    { 0xadfeb675, 90 },
    { 0x81eb676, 23 },
    { 0xf2aeb677, 61 },
    { 0xe40eb675, 77 },
    { 0, 0 },
    { 0xaf2eb67a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddeb67f, 98 },
    { 0x49beb680, 90 },
    { 0, 0 },
    { 0xbbdeb682, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1eb685, 69 },
    { 0x459eb686, 90 },
    { 0x9f3eb686, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x455eb68a, 61 },
    { 0, 0 },
    { 0x8f0eb68c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eb68f, 81 },
    { 0xcb3eb690, 43 },
    { 0x185eb690, 100 },
    { 0x155eb692, 98 },
    { 0x526eb692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dceb698, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeadeb69b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaeb69e, 46 },
    { 0x181eb69f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x588eb6a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69eb6a5, 76 },
    { 0, 0 },
    { 0xbc6eb6a7, 98 },
    { 0x1d1eb6a7, 34 },
    { 0, 0 },
    { 0xb4ceb6aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50eb6af, 76 },
    { 0xdeceb6b0, 95 },
    { 0, 0 },
    { 0x823eb6b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x416eb6b5, 61 },
    { 0x9aeeb6b6, 91 },
    { 0x6c2eb6b7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07eb6ba, 34 },
    { 0xcceeb6bb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9eb6c1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x81eeb6c4, 23 },
    { 0x940eb6c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4beb6c9, 81 },
    { 0x47beb6ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9eb6cd, 90 },
    { 0x736eb6cd, 93 },
    { 0x595eb6cf, 55 },
    { 0xdafeb6d0, 90 },
    { 0x82aeb6d1, 34 },
    { 0x21beb6d0, 98 },
    { 0x2feb6d3, 67 },
    { 0x158eb6d3, 33 },
    { 0x1bceb6cf, 85 },
    { 0, 0 },
    { 0x278eb6d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5eb6db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1eb6df, 33 },
    { 0xdaeb6e0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x796eb6e3, 34 },
    { 0xad4eb6e4, 61 },
    { 0, 0 },
    { 0xf62eb6e6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daeb6ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62eb6f0, 76 },
    { 0xeeeb6f1, 65 },
    { 0xe9ceb6f2, 34 },
    { 0, 0 },
    { 0x7b3eb6f4, 95 },
    { 0x704eb6f4, 55 },
    { 0x934eb6f6, 77 },
    { 0x4e7eb6f7, 76 },
    { 0x43beb6f8, 91 },
    { 0xd5eeb6f9, 76 },
    { 0xd7ceb6f9, 33 },
    { 0x18eb6f7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x65aeb6fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bfeb703, 43 },
    { 0, 0 },
    { 0xdc1eb705, 46 },
    { 0x7a0eb706, 61 },
    { 0xfe8eb706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fceb70c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8eb714, 65 },
    { 0, 0 },
    { 0xfb5eb716, 23 },
    { 0x2a7eb717, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404eb71c, 33 },
    { 0xfabeb71d, 98 },
    { 0x76feb71c, 98 },
    { 0, 0 },
    { 0x35eeb720, 99 },
    { 0xe1eeb721, 55 },
    { 0x43feb722, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9eb726, 90 },
    { 0, 0 },
    { 0x69beb728, 34 },
    { 0xe58eb729, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eeb72c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x479eb72f, 95 },
    { 0xc6eb730, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eeb736, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x10deb739, 95 },
    { 0, 0 },
    { 0xaf3eb73b, 61 },
    { 0, 0 },
    { 0x442eb73d, 91 },
    { 0x4ddeb73e, 78 },
    { 0x18aeb73f, 95 },
    { 0x205eb73f, 66 },
    { 0x2cdeb73f, 95 },
    { 0, 0 },
    { 0x7eeeb743, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79eb74b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5beb751, 77 },
    { 0xfddeb752, 33 },
    { 0xdc1eb751, 87 },
    { 0xd6feb754, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x47deb757, 23 },
    { 0x907eb758, 90 },
    { 0x121eb759, 95 },
    { 0x375eb75a, 61 },
    { 0xc86eb75b, 68 },
    { 0x876eb757, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x282eb765, 90 },
    { 0x9eeb765, 61 },
    { 0x25feb765, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfeeb76b, 66 },
    { 0, 0 },
    { 0x184eb76d, 95 },
    { 0xbcfeb76e, 90 },
    { 0x64feb76e, 98 },
    { 0, 0 },
    { 0x23deb771, 33 },
    { 0xaafeb772, 17 },
    { 0xa85eb773, 52 },
    { 0, 0 },
    { 0xb1eeb775, 87 },
    { 0x73eb776, 90 },
    { 0x22feb777, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859eb78d, 68 },
    { 0x719eb78e, 90 },
    { 0xe61eb78e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafceb799, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362eb79d, 33 },
    { 0, 0 },
    { 0xfbeeb79f, 95 },
    { 0xfb5eb7a0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72eb7a4, 95 },
    { 0xdfeb7a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x716eb7a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x183eb7ac, 43 },
    { 0, 0 },
    { 0x2b1eb7ae, 33 },
    { 0xf1beb7af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x570eb7b2, 46 },
    { 0x740eb7b3, 33 },
    { 0, 0 },
    { 0xc3eeb7b5, 61 },
    { 0xc07eb7b5, 66 },
    { 0, 0 },
    { 0xe52eb7b8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836eb7be, 43 },
    { 0x752eb7be, 24 },
    { 0xeedeb7c0, 61 },
    { 0x939eb7c1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3deb7c8, 89 },
    { 0x497eb7c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1beb7cd, 23 },
    { 0xe9feb7ce, 95 },
    { 0, 0 },
    { 0x9b4eb7d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x205eb7d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdeb7db, 76 },
    { 0xfe4eb7db, 93 },
    { 0x8caeb7dd, 90 },
    { 0xda3eb7de, 61 },
    { 0, 0 },
    { 0x80beb7e0, 69 },
    { 0xb1eb7e1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3eb7f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328eb7f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491eb7fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd7eb803, 4 },
    { 0x13ceb804, 90 },
    { 0x576eb804, 90 },
    { 0xabaeb804, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eeb80d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37eb812, 46 },
    { 0xf9feb813, 69 },
    { 0x1fbeb814, 95 },
    { 0xbdaeb813, 33 },
    { 0x7feeb816, 33 },
    { 0xcddeb815, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23eb81a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebeb81f, 90 },
    { 0, 0 },
    { 0xd9deb821, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6feb826, 43 },
    { 0xc23eb827, 98 },
    { 0x313eb828, 33 },
    { 0, 0 },
    { 0x581eb82a, 90 },
    { 0xa78eb82b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x108eb82e, 61 },
    { 0, 0 },
    { 0x412eb830, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44eb835, 80 },
    { 0, 0 },
    { 0xccdeb837, 90 },
    { 0, 0 },
    { 0x1aeb839, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4eb83c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53deb845, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aeeb848, 33 },
    { 0x7daeb849, 81 },
    { 0, 0 },
    { 0xe4eeb84b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244eb84f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3eb853, 61 },
    { 0xbe1eb854, 33 },
    { 0, 0 },
    { 0x343eb856, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeb85b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1eb860, 23 },
    { 0, 0 },
    { 0x6caeb862, 97 },
    { 0x72beb863, 33 },
    { 0x9a8eb864, 95 },
    { 0x25eb863, 23 },
    { 0xa66eb865, 61 },
    { 0x193eb867, 90 },
    { 0x455eb863, 90 },
    { 0xb9eb869, 66 },
    { 0x2b3eb86a, 61 },
    { 0x643eb864, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x943eb86f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321eb875, 34 },
    { 0x93aeb876, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4feb87f, 67 },
    { 0x86eb880, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccfeb883, 33 },
    { 0, 0 },
    { 0xf1eeb885, 34 },
    { 0x144eb886, 90 },
    { 0x68deb886, 90 },
    { 0, 0 },
    { 0xedeeb889, 55 },
    { 0x203eb88a, 100 },
    { 0, 0 },
    { 0x9e2eb88c, 23 },
    { 0xd01eb88c, 76 },
    { 0, 0 },
    { 0xe66eb88f, 8 },
    { 0xe97eb88f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19eb893, 43 },
    { 0x8deeb894, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x799eb897, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02eb89a, 33 },
    { 0x7b2eb89b, 99 },
    { 0xccfeb89b, 33 },
    { 0x292eb89d, 88 },
    { 0x173eb89d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ceb8a1, 23 },
    { 0xb1beb8a2, 61 },
    { 0x16ceb8a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x786eb8a6, 52 },
    { 0, 0 },
    { 0xea0eb8a8, 34 },
    { 0xeffeb8a9, 76 },
    { 0, 0 },
    { 0x109eb8ab, 90 },
    { 0x196eb8ac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18deb8b1, 23 },
    { 0, 0 },
    { 0x988eb8b3, 90 },
    { 0, 0 },
    { 0x156eb8b5, 61 },
    { 0, 0 },
    { 0x667eb8b7, 33 },
    { 0x6e7eb8b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x795eb8bb, 33 },
    { 0x621eb8bc, 53 },
    { 0, 0 },
    { 0xf31eb8be, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89beb8c2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ceb8c5, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dfeb8ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6eb8cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6eb8d4, 98 },
    { 0x773eb8d4, 95 },
    { 0, 0 },
    { 0xecaeb8d7, 87 },
    { 0x501eb8d8, 66 },
    { 0x318eb8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40deb8df, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685eb8e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x58beb8e6, 93 },
    { 0xe17eb8e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe16eb8eb, 90 },
    { 0xed5eb8eb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bceb8f3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21eb8fa, 61 },
    { 0, 0 },
    { 0x8efeb8fc, 95 },
    { 0x7d8eb8fd, 87 },
    { 0x971eb8fc, 23 },
    { 0xb32eb8ff, 33 },
    { 0xed7eb8ff, 61 },
    { 0xb2deb901, 95 },
    { 0x62eeb8ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eeb90a, 80 },
    { 0, 0 },
    { 0x7a9eb90c, 84 },
    { 0, 0 },
    { 0x89aeb90e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778eb913, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3eb916, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4eb91f, 95 },
    { 0x8fdeb920, 17 },
    { 0, 0 },
    { 0x436eb922, 61 },
    { 0, 0 },
    { 0xc21eb924, 33 },
    { 0, 0 },
    { 0x1e9eb926, 90 },
    { 0, 0 },
    { 0xcddeb928, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ceb92b, 23 },
    { 0x9eeb92c, 55 },
    { 0, 0 },
    { 0x1f3eb92e, 98 },
    { 0, 0 },
    { 0x7a7eb930, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9eb933, 61 },
    { 0x656eb934, 80 },
    { 0x729eb935, 93 },
    { 0xa00eb934, 95 },
    { 0, 0 },
    { 0xae8eb938, 35 },
    { 0x562eb938, 33 },
    { 0x5c9eb93a, 33 },
    { 0x1a8eb93a, 76 },
    { 0x53feb93c, 23 },
    { 0x45beb93c, 43 },
    { 0xaceeb93a, 61 },
    { 0, 0 },
    { 0x610eb940, 95 },
    { 0, 0 },
    { 0x308eb942, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceb948, 76 },
    { 0x320eb949, 61 },
    { 0xd01eb949, 90 },
    { 0, 0 },
    { 0xb8beb94c, 52 },
    { 0xa80eb94d, 99 },
    { 0xa92eb94e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaeb951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40eb954, 33 },
    { 0x2baeb955, 90 },
    { 0, 0 },
    { 0x6aeeb957, 90 },
    { 0, 0 },
    { 0x3b0eb959, 46 },
    { 0xe0ceb95a, 33 },
    { 0xde2eb95a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4eb95e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2eeb961, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x31feb964, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eb969, 90 },
    { 0, 0 },
    { 0x188eb96b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2eb96e, 34 },
    { 0x2f7eb96f, 95 },
    { 0x5b7eb96f, 90 },
    { 0x6deb971, 80 },
    { 0xab7eb971, 78 },
    { 0xa22eb973, 91 },
    { 0x41eb973, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa06eb978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x392eb97b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130eb97f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32eb986, 23 },
    { 0, 0 },
    { 0xfcceb988, 76 },
    { 0x432eb989, 95 },
    { 0xe71eb988, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36eb98f, 46 },
    { 0, 0 },
    { 0x7c0eb991, 72 },
    { 0, 0 },
    { 0x352eb993, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ceb99c, 100 },
    { 0, 0 },
    { 0x278eb99e, 76 },
    { 0xc05eb99f, 33 },
    { 0x80feb9a0, 34 },
    { 0x646eb9a1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77aeb9a5, 98 },
    { 0xc3ceb9a6, 61 },
    { 0x9dbeb9a6, 61 },
    { 0x497eb9a8, 76 },
    { 0xaefeb9a9, 88 },
    { 0x87ceb9a9, 95 },
    { 0xfa5eb9a8, 33 },
    { 0x9f3eb9ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9eb9af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabeb9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234eb9bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x810eb9be, 68 },
    { 0, 0 },
    { 0x9c8eb9c0, 98 },
    { 0, 0 },
    { 0xd69eb9c2, 87 },
    { 0x965eb9c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ceb9c6, 91 },
    { 0, 0 },
    { 0x160eb9c8, 99 },
    { 0x16eeb9c9, 76 },
    { 0x933eb9ca, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x564eb9cd, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf59eb9d4, 34 },
    { 0x9d9eb9d5, 87 },
    { 0x7c4eb9d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52eb9e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1eb9e3, 95 },
    { 0, 0 },
    { 0xd59eb9e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94eb9ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3eb9f0, 87 },
    { 0, 0 },
    { 0xf37eb9f2, 23 },
    { 0x1d6eb9f3, 36 },
    { 0, 0 },
    { 0x4bceb9f5, 61 },
    { 0, 0 },
    { 0x49deb9f7, 40 },
    { 0xb5eeb9f8, 61 },
    { 0xf64eb9f8, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aeb9ff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11eba02, 84 },
    { 0xaa4eba03, 88 },
    { 0x768eba04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdeba08, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458eba0c, 95 },
    { 0xc07eba0d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ceba1d, 61 },
    { 0xf0eeba1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deba24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753eba2c, 90 },
    { 0x7abeba2d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22eba32, 46 },
    { 0xfb9eba32, 95 },
    { 0x84deba34, 33 },
    { 0xcefeba35, 46 },
    { 0, 0 },
    { 0x913eba37, 55 },
    { 0x84eba37, 90 },
    { 0, 0 },
    { 0x8e9eba3a, 76 },
    { 0xd24eba3b, 43 },
    { 0x99aeba3c, 87 },
    { 0x13aeba3d, 97 },
    { 0xc46eba3d, 99 },
    { 0x5dfeba3c, 90 },
    { 0xfbeba3a, 65 },
    { 0x161eba3b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754eba46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4eba49, 90 },
    { 0xf3beba4a, 91 },
    { 0x444eba4a, 66 },
    { 0x5dceba4c, 33 },
    { 0x68beba4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53eba53, 33 },
    { 0x45eeba54, 23 },
    { 0x59aeba55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8eba59, 76 },
    { 0xc86eba59, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02eba5f, 90 },
    { 0xc09eba60, 61 },
    { 0x8c2eba5f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93aeba66, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40eba6e, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8eba75, 61 },
    { 0, 0 },
    { 0x5a4eba77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30eba7a, 90 },
    { 0x39beba7b, 23 },
    { 0xe25eba7c, 90 },
    { 0x88beba7c, 61 },
    { 0x9aeba7d, 23 },
    { 0xd60eba7d, 90 },
    { 0x57aeba80, 46 },
    { 0x81deba7c, 93 },
    { 0x968eba82, 66 },
    { 0xe70eba7f, 90 },
    { 0xbb0eba7b, 61 },
    { 0x4deba85, 67 },
    { 0xe8eeba7e, 90 },
    { 0xa11eba85, 68 },
    { 0xec9eba81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1eba91, 33 },
    { 0, 0 },
    { 0x992eba93, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaeba97, 64 },
    { 0x30eba97, 98 },
    { 0, 0 },
    { 0xb9deba9a, 97 },
    { 0x4bceba9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x547ebaa0, 23 },
    { 0, 0 },
    { 0x376ebaa2, 81 },
    { 0x9cbebaa2, 33 },
    { 0x7fdebaa3, 51 },
    { 0xc1bebaa4, 55 },
    { 0, 0 },
    { 0xbf0ebaa7, 67 },
    { 0xa62ebaa8, 61 },
    { 0xdaaebaa8, 100 },
    { 0xa1ebaa7, 90 },
    { 0xaadebaab, 59 },
    { 0, 0 },
    { 0xc5febaad, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfebab1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ebab9, 95 },
    { 0x743ebaba, 76 },
    { 0xac0ebabb, 95 },
    { 0x34febabc, 53 },
    { 0x297ebabc, 77 },
    { 0, 0 },
    { 0x802ebabf, 34 },
    { 0x86aebabf, 33 },
    { 0xc01ebac1, 95 },
    { 0, 0 },
    { 0x174ebac3, 33 },
    { 0x4c2ebac4, 91 },
    { 0xf4ebac4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715ebac9, 95 },
    { 0, 0 },
    { 0x30febacb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ebad1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbebad5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31cebad9, 90 },
    { 0, 0 },
    { 0xc84ebadb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60febadf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41ebae3, 99 },
    { 0x94bebae3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660ebaeb, 90 },
    { 0x3e4ebaeb, 55 },
    { 0, 0 },
    { 0x34ebaee, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bebaf1, 99 },
    { 0, 0 },
    { 0xcdfebaf3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x86aebaf6, 17 },
    { 0xf29ebaf7, 40 },
    { 0xd53ebaf8, 64 },
    { 0x11eebaf9, 48 },
    { 0x28aebaf7, 52 },
    { 0x120ebaf8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x277ebafe, 87 },
    { 0, 0 },
    { 0x4cebb00, 33 },
    { 0xa01ebb01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c2ebb05, 98 },
    { 0x404ebb06, 59 },
    { 0, 0 },
    { 0xd08ebb08, 61 },
    { 0xbb3ebb08, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5febb0c, 76 },
    { 0, 0 },
    { 0xebaebb0e, 55 },
    { 0x2b2ebb0f, 61 },
    { 0, 0 },
    { 0x136ebb11, 78 },
    { 0, 0 },
    { 0xd00ebb13, 98 },
    { 0xbdbebb14, 68 },
    { 0x287ebb15, 94 },
    { 0x1efebb16, 61 },
    { 0xefbebb17, 17 },
    { 0x2ccebb16, 80 },
    { 0x518ebb19, 98 },
    { 0x279ebb1a, 55 },
    { 0x1dbebb1a, 33 },
    { 0x33debb1c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ebb22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41ebb26, 33 },
    { 0, 0 },
    { 0x42cebb28, 84 },
    { 0x9cfebb29, 90 },
    { 0xd21ebb2a, 95 },
    { 0x5dfebb29, 76 },
    { 0xbdeebb2c, 33 },
    { 0xf24ebb2d, 98 },
    { 0xe12ebb2d, 61 },
    { 0xd62ebb2c, 75 },
    { 0x664ebb30, 76 },
    { 0xc72ebb31, 95 },
    { 0x559ebb31, 90 },
    { 0xd8aebb33, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09ebb37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x608ebb3a, 95 },
    { 0x89cebb3b, 33 },
    { 0xb50ebb3c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2febb40, 61 },
    { 0xfdeebb40, 93 },
    { 0x970ebb42, 99 },
    { 0x9ddebb43, 90 },
    { 0xcc0ebb44, 61 },
    { 0, 0 },
    { 0xb22ebb46, 90 },
    { 0xaf7ebb47, 98 },
    { 0xe46ebb47, 91 },
    { 0x6c8ebb48, 55 },
    { 0x8d1ebb49, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9ebb50, 46 },
    { 0x42febb51, 88 },
    { 0x63febb51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1ebb57, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0ebb5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429ebb61, 67 },
    { 0, 0 },
    { 0xccaebb63, 84 },
    { 0xe42ebb63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1ebb6a, 85 },
    { 0xc43ebb6b, 91 },
    { 0, 0 },
    { 0xda0ebb6d, 67 },
    { 0xbeebb6d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ebb71, 77 },
    { 0, 0 },
    { 0x588ebb73, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403ebb7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8ebb7e, 98 },
    { 0x22bebb7f, 42 },
    { 0x741ebb7e, 34 },
    { 0, 0 },
    { 0x9afebb82, 23 },
    { 0x61ebb82, 61 },
    { 0x3faebb82, 75 },
    { 0, 0 },
    { 0x944ebb86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ebb8c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6ebb92, 90 },
    { 0x24aebb92, 55 },
    { 0x113ebb93, 95 },
    { 0xff2ebb95, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eebb9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585ebba2, 90 },
    { 0, 0 },
    { 0x728ebba4, 61 },
    { 0x4c4ebba5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495ebba9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ebbad, 55 },
    { 0, 0 },
    { 0xd6aebbaf, 33 },
    { 0, 0 },
    { 0x5eebbb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7ebbb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ebbb8, 23 },
    { 0x3dcebbb9, 83 },
    { 0x69febbba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a8ebbbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33ebbc1, 90 },
    { 0xe6eebbc2, 75 },
    { 0x88febbc3, 52 },
    { 0x6a2ebbc4, 76 },
    { 0x2d9ebbc4, 95 },
    { 0xb59ebbc4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aebbcc, 23 },
    { 0, 0 },
    { 0x3f2ebbce, 33 },
    { 0xf19ebbce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x177ebbd2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17ebbd5, 52 },
    { 0xc7ebbd5, 34 },
    { 0xc47ebbd6, 84 },
    { 0xdd6ebbd6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836ebbdc, 99 },
    { 0, 0 },
    { 0x74eebbde, 93 },
    { 0x6ebebbdf, 33 },
    { 0x408ebbe0, 77 },
    { 0xc9eebbe1, 95 },
    { 0x646ebbe2, 23 },
    { 0x164ebbe3, 85 },
    { 0, 0 },
    { 0x3d4ebbe5, 87 },
    { 0x90aebbe5, 61 },
    { 0, 0 },
    { 0xc65ebbe8, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16febbed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0ebbf1, 90 },
    { 0, 0 },
    { 0x778ebbf3, 93 },
    { 0xaa2ebbf4, 90 },
    { 0x106ebbf5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64febbfa, 81 },
    { 0, 0 },
    { 0x7b6ebbfc, 90 },
    { 0, 0 },
    { 0xec6ebbfe, 76 },
    { 0, 0 },
    { 0x6bbebc00, 33 },
    { 0, 0 },
    { 0xff2ebc02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccebc07, 55 },
    { 0x385ebc08, 90 },
    { 0x5bebc09, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ebc0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468ebc11, 98 },
    { 0xb7ebc11, 98 },
    { 0x2b4ebc12, 66 },
    { 0xae4ebc14, 55 },
    { 0, 0 },
    { 0xb7eebc16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1ebc1f, 33 },
    { 0xca7ebc20, 90 },
    { 0xe44ebc21, 98 },
    { 0, 0 },
    { 0x65aebc23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eebc29, 99 },
    { 0x5c3ebc2a, 23 },
    { 0xbaebc2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bebc2e, 98 },
    { 0xde6ebc2f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971ebc3b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5ebc3e, 33 },
    { 0xbdeebc3f, 33 },
    { 0, 0 },
    { 0x158ebc41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ebc47, 23 },
    { 0xf1bebc47, 23 },
    { 0x6d4ebc49, 90 },
    { 0x6c8ebc49, 61 },
    { 0xcbfebc4b, 66 },
    { 0x61cebc4b, 17 },
    { 0xf63ebc4d, 55 },
    { 0x3f5ebc4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbebc56, 90 },
    { 0xeb3ebc57, 66 },
    { 0, 0 },
    { 0x856ebc59, 33 },
    { 0x44eebc59, 33 },
    { 0, 0 },
    { 0xfa8ebc5c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42ebc64, 23 },
    { 0, 0 },
    { 0x4f6ebc66, 55 },
    { 0xda4ebc66, 61 },
    { 0xfaebc66, 95 },
    { 0x93aebc69, 46 },
    { 0xb3eebc6a, 95 },
    { 0x6eeebc6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37eebc6f, 101 },
    { 0xc7debc70, 91 },
    { 0, 0 },
    { 0x43debc72, 90 },
    { 0x30debc72, 90 },
    { 0x91debc74, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3ebc7a, 76 },
    { 0xa19ebc7b, 95 },
    { 0x10debc7b, 95 },
    { 0x9f2ebc7d, 23 },
    { 0, 0 },
    { 0x849ebc7f, 61 },
    { 0, 0 },
    { 0x7f1ebc81, 43 },
    { 0x6c4ebc81, 46 },
    { 0xbaebc81, 61 },
    { 0xff9ebc84, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2ebc88, 90 },
    { 0xd32ebc89, 75 },
    { 0, 0 },
    { 0x77cebc8b, 90 },
    { 0, 0 },
    { 0xd85ebc8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2aebc91, 61 },
    { 0, 0 },
    { 0xb5febc93, 61 },
    { 0xfc5ebc94, 95 },
    { 0x26bebc95, 61 },
    { 0x267ebc95, 90 },
    { 0xb06ebc94, 33 },
    { 0x34ebc98, 33 },
    { 0x7d1ebc94, 65 },
    { 0x728ebc9a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x220ebc9e, 66 },
    { 0x902ebc9f, 95 },
    { 0x357ebca0, 90 },
    { 0x895ebca1, 61 },
    { 0x20febca1, 66 },
    { 0xb11ebca3, 99 },
    { 0x2a7ebca1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcebca7, 100 },
    { 0x51ebca8, 33 },
    { 0, 0 },
    { 0x3c4ebcaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1aebcaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eebcb6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6ebcc0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289ebcc5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3ebccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65debccf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73ebcd4, 61 },
    { 0, 0 },
    { 0x40eebcd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cebcd9, 76 },
    { 0xa0ebcd9, 98 },
    { 0, 0 },
    { 0x757ebcdc, 34 },
    { 0xeb0ebcdd, 53 },
    { 0, 0 },
    { 0xf8ebcdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acebce2, 90 },
    { 0x1b7ebce3, 61 },
    { 0, 0 },
    { 0x501ebce5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94ebce9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbebced, 43 },
    { 0x371ebcee, 33 },
    { 0x577ebcef, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfebcf7, 33 },
    { 0, 0 },
    { 0xadbebcf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feebcfe, 46 },
    { 0x988ebcff, 90 },
    { 0, 0 },
    { 0x80debd01, 33 },
    { 0, 0 },
    { 0x3c6ebd03, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48ebd07, 23 },
    { 0xa9bebd08, 33 },
    { 0x780ebd08, 18 },
    { 0x4baebd0a, 97 },
    { 0xc55ebd0b, 98 },
    { 0x625ebd09, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ebd17, 87 },
    { 0, 0 },
    { 0x683ebd19, 43 },
    { 0, 0 },
    { 0xc7bebd1b, 78 },
    { 0x8afebd1b, 61 },
    { 0x65febd1c, 90 },
    { 0, 0 },
    { 0x17debd1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369ebd23, 95 },
    { 0, 0 },
    { 0xb97ebd25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x997ebd28, 33 },
    { 0xcceebd29, 93 },
    { 0x781ebd29, 55 },
    { 0xdf8ebd2b, 90 },
    { 0x8b0ebd2c, 96 },
    { 0x5cebd2d, 33 },
    { 0xfa4ebd2e, 66 },
    { 0xb7febd2d, 23 },
    { 0xf84ebd2a, 95 },
    { 0xa24ebd31, 65 },
    { 0, 0 },
    { 0xe9ebd33, 90 },
    { 0x861ebd33, 61 },
    { 0, 0 },
    { 0xa5aebd36, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e9ebd39, 75 },
    { 0xd1febd3a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ceebd3e, 90 },
    { 0x852ebd3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5ebd4b, 95 },
    { 0, 0 },
    { 0xaa0ebd4d, 66 },
    { 0xd8febd4e, 90 },
    { 0x67debd4f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8ebd54, 34 },
    { 0xa12ebd55, 98 },
    { 0x8cbebd55, 91 },
    { 0x813ebd57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21debd5b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55eebd61, 43 },
    { 0x640ebd61, 61 },
    { 0, 0 },
    { 0x45eebd64, 90 },
    { 0x143ebd65, 33 },
    { 0, 0 },
    { 0xe63ebd67, 46 },
    { 0x47ebd68, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3ebd6b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31ebd70, 55 },
    { 0, 0 },
    { 0x1bdebd72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc8ebd76, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0febd79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0ebd7c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eebd7f, 61 },
    { 0, 0 },
    { 0xd46ebd81, 98 },
    { 0xccbebd82, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x910ebd8c, 65 },
    { 0x19debd8c, 87 },
    { 0xb70ebd8c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0ebd94, 67 },
    { 0xfd5ebd95, 90 },
    { 0xcabebd96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aebd9b, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27ebd9e, 90 },
    { 0, 0 },
    { 0xb5eebda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9ebda5, 90 },
    { 0x90cebda6, 98 },
    { 0x782ebda6, 33 },
    { 0, 0 },
    { 0x3e0ebda9, 33 },
    { 0x712ebdaa, 46 },
    { 0x610ebdaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30ebdaf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ebdb4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14debdb9, 76 },
    { 0, 0 },
    { 0x2e3ebdbb, 23 },
    { 0, 0 },
    { 0xe23ebdbd, 25 },
    { 0xf35ebdbe, 99 },
    { 0xceebdbf, 76 },
    { 0xcd5ebdc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e4ebdc4, 61 },
    { 0x2b6ebdc5, 75 },
    { 0, 0 },
    { 0x734ebdc7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95ebdd3, 84 },
    { 0x2c7ebdd4, 77 },
    { 0x740ebdd5, 33 },
    { 0xca1ebdd4, 90 },
    { 0xebebdd3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1ebdde, 92 },
    { 0xf65ebddf, 90 },
    { 0x3ebebde0, 76 },
    { 0, 0 },
    { 0x1c9ebde2, 93 },
    { 0xf76ebde3, 52 },
    { 0, 0 },
    { 0xd8debde5, 55 },
    { 0x3dbebde6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x247ebde9, 90 },
    { 0x439ebdea, 98 },
    { 0xe84ebdea, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3debdf0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0debdf3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf31ebdfb, 23 },
    { 0x6aebdfc, 46 },
    { 0x8e9ebdfd, 90 },
    { 0, 0 },
    { 0x2b5ebdff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9ebe02, 90 },
    { 0, 0 },
    { 0xd03ebe04, 98 },
    { 0x950ebe05, 85 },
    { 0xb3eebe06, 76 },
    { 0, 0 },
    { 0x6cebe08, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461ebe0c, 98 },
    { 0, 0 },
    { 0x3eebe0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aebe12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94aebe18, 90 },
    { 0, 0 },
    { 0xc20ebe1a, 33 },
    { 0x67aebe1b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937ebe24, 61 },
    { 0xb3ebe24, 61 },
    { 0x5ebe24, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71febe2c, 93 },
    { 0x5f7ebe2d, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9ebe36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bebe40, 23 },
    { 0xe51ebe41, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4ebe46, 23 },
    { 0, 0 },
    { 0x5b5ebe48, 34 },
    { 0x583ebe49, 93 },
    { 0xcaebe4a, 90 },
    { 0, 0 },
    { 0xd46ebe4c, 95 },
    { 0x4e0ebe4d, 95 },
    { 0, 0 },
    { 0x14ebe4f, 61 },
    { 0x300ebe50, 81 },
    { 0xac8ebe4f, 67 },
    { 0, 0 },
    { 0x8a3ebe53, 88 },
    { 0, 0 },
    { 0xca8ebe55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639ebe60, 96 },
    { 0, 0 },
    { 0x2febe62, 61 },
    { 0, 0 },
    { 0x60eebe64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717ebe6b, 90 },
    { 0, 0 },
    { 0x8fbebe6d, 61 },
    { 0xc13ebe6e, 52 },
    { 0xb72ebe6f, 55 },
    { 0xd09ebe70, 84 },
    { 0x293ebe6f, 95 },
    { 0x4b2ebe71, 99 },
    { 0x637ebe6d, 95 },
    { 0xf53ebe74, 87 },
    { 0x2f7ebe74, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b6ebe79, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cebe7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ebe80, 76 },
    { 0x758ebe81, 90 },
    { 0xf22ebe81, 61 },
    { 0xdd8ebe82, 46 },
    { 0x7fbebe81, 90 },
    { 0, 0 },
    { 0xa58ebe86, 78 },
    { 0xddbebe87, 17 },
    { 0x1eaebe87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efebe8e, 33 },
    { 0, 0 },
    { 0xf3eebe90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acebe94, 17 },
    { 0x41bebe95, 85 },
    { 0, 0 },
    { 0x74eebe97, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeebe9a, 95 },
    { 0x149ebe9b, 99 },
    { 0x428ebe9b, 43 },
    { 0x66eebe9d, 61 },
    { 0x41debe9e, 78 },
    { 0x103ebe9f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892ebea3, 33 },
    { 0, 0 },
    { 0xa9eebea5, 33 },
    { 0xbdcebea5, 90 },
    { 0x94cebea7, 90 },
    { 0x33febea8, 46 },
    { 0x606ebea9, 95 },
    { 0x2f1ebea9, 61 },
    { 0x414ebeaa, 95 },
    { 0xcaeebeac, 61 },
    { 0, 0 },
    { 0xb45ebeae, 23 },
    { 0, 0 },
    { 0xd4debeb0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51febeb7, 98 },
    { 0xee7ebeb8, 98 },
    { 0xa9cebeb8, 33 },
    { 0x2bfebeb9, 81 },
    { 0x5bbebebb, 77 },
    { 0xb0cebebb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9ebec0, 90 },
    { 0x12ebec1, 33 },
    { 0xa0ebec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31ebeca, 87 },
    { 0, 0 },
    { 0x5a6ebecc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e6ebed0, 93 },
    { 0, 0 },
    { 0x621ebed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abebed9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80ebedd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23debee3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ebeec, 23 },
    { 0x627ebeed, 33 },
    { 0xb2ebeec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bebef2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565ebef7, 90 },
    { 0xf24ebef8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cebf01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ebf05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7ebf08, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0ebf0b, 23 },
    { 0x5e2ebf0c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbfebf11, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52ebf15, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a5ebf1b, 43 },
    { 0xbeebf1c, 90 },
    { 0xb2febf1d, 20 },
    { 0xff9ebf1b, 23 },
    { 0x2eebf1b, 33 },
    { 0x5f5ebf20, 99 },
    { 0, 0 },
    { 0x9ecebf22, 98 },
    { 0x499ebf22, 34 },
    { 0xaedebf22, 98 },
    { 0x132ebf25, 76 },
    { 0, 0 },
    { 0x219ebf27, 23 },
    { 0x45cebf28, 81 },
    { 0x8e5ebf27, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x730ebf2e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x251ebf32, 95 },
    { 0x8debf33, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48ebf36, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x216ebf39, 70 },
    { 0xc8cebf3a, 93 },
    { 0x3b0ebf3b, 33 },
    { 0, 0 },
    { 0xda1ebf3d, 76 },
    { 0, 0 },
    { 0x94cebf3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ebf42, 33 },
    { 0, 0 },
    { 0x39eebf44, 55 },
    { 0xad8ebf45, 33 },
    { 0x577ebf45, 90 },
    { 0xe7eebf47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebebf4a, 33 },
    { 0, 0 },
    { 0xd05ebf4c, 84 },
    { 0, 0 },
    { 0x313ebf4e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6ebf51, 95 },
    { 0x7daebf51, 33 },
    { 0xfbebf53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5ebf56, 46 },
    { 0, 0 },
    { 0x61cebf58, 87 },
    { 0x284ebf58, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cebf5e, 64 },
    { 0x1fbebf5f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cebf62, 33 },
    { 0, 0 },
    { 0xfc9ebf64, 101 },
    { 0, 0 },
    { 0x78debf66, 85 },
    { 0xa9cebf66, 98 },
    { 0xcbeebf68, 92 },
    { 0xccbebf69, 33 },
    { 0x77febf69, 90 },
    { 0xabdebf69, 67 },
    { 0xaf6ebf6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96ebf73, 92 },
    { 0x802ebf74, 61 },
    { 0, 0 },
    { 0xe84ebf76, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8ebf81, 33 },
    { 0x6c2ebf82, 52 },
    { 0xfe8ebf83, 78 },
    { 0xdeebf84, 98 },
    { 0x7e3ebf82, 99 },
    { 0xe8febf86, 52 },
    { 0x10eebf87, 55 },
    { 0x32debf88, 93 },
    { 0, 0 },
    { 0xcd3ebf8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acebf8e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedebf92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950ebf96, 43 },
    { 0x769ebf97, 23 },
    { 0x77febf98, 23 },
    { 0, 0 },
    { 0xd0cebf9a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bebf9d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10debfa4, 33 },
    { 0x67cebfa5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26febfa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470ebfb3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2ebfbf, 33 },
    { 0xa7bebfbf, 52 },
    { 0xca9ebfc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffebfc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cebfc7, 61 },
    { 0, 0 },
    { 0xa8febfc9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdebfcc, 93 },
    { 0x31eebfcc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55aebfd5, 90 },
    { 0xeb7ebfd6, 55 },
    { 0x334ebfd7, 98 },
    { 0xe51ebfd8, 98 },
    { 0xe0bebfd8, 8 },
    { 0xd61ebfda, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cebfe1, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0ebfe4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ebfea, 61 },
    { 0x9e3ebfeb, 90 },
    { 0x3feebfec, 33 },
    { 0xc25ebfed, 46 },
    { 0x82febfee, 66 },
    { 0xf68ebfef, 33 },
    { 0, 0 },
    { 0x6b5ebff1, 67 },
    { 0, 0 },
    { 0x8caebff3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xc21ebff6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ebffb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bec000, 34 },
    { 0x500ec001, 33 },
    { 0x74aec002, 88 },
    { 0, 0 },
    { 0x80cec004, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ec00a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cec00e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5ec013, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fec01d, 95 },
    { 0x99cec01e, 33 },
    { 0, 0 },
    { 0x6cbec020, 90 },
    { 0, 0 },
    { 0x4eaec022, 43 },
    { 0x7b4ec023, 99 },
    { 0x1e3ec024, 79 },
    { 0, 0 },
    { 0x5b3ec026, 33 },
    { 0, 0 },
    { 0x1e5ec028, 93 },
    { 0xc38ec029, 61 },
    { 0x765ec02a, 95 },
    { 0, 0 },
    { 0x54cec02c, 95 },
    { 0x35bec02c, 33 },
    { 0xdf3ec02c, 61 },
    { 0xc3fec02f, 90 },
    { 0, 0 },
    { 0xdf8ec031, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9dec037, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ec03c, 99 },
    { 0x7dfec03d, 46 },
    { 0xbbeec03d, 90 },
    { 0x714ec03f, 93 },
    { 0x624ec040, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdefec043, 33 },
    { 0xb76ec043, 43 },
    { 0, 0 },
    { 0x792ec046, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41ec04a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cec04f, 24 },
    { 0, 0 },
    { 0xf9cec051, 90 },
    { 0x9e5ec052, 90 },
    { 0xe24ec052, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fec05d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeec060, 33 },
    { 0, 0 },
    { 0xe18ec062, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159ec067, 61 },
    { 0xae5ec068, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6ec06b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6eec06e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87ec071, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18ec075, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5ec079, 90 },
    { 0, 0 },
    { 0xaf0ec07b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56eec07f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8fec082, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ec086, 95 },
    { 0x364ec087, 52 },
    { 0x5b1ec088, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbec08b, 76 },
    { 0, 0 },
    { 0x16aec08d, 43 },
    { 0x3bec08e, 55 },
    { 0xbcfec08f, 77 },
    { 0x3fdec08f, 91 },
    { 0x498ec090, 91 },
    { 0x3ffec092, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adec098, 77 },
    { 0x506ec099, 98 },
    { 0x45cec09a, 98 },
    { 0x7a0ec099, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1ec0a1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cec0a7, 95 },
    { 0x9b8ec0a7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x83dec0ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08ec0af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eec0b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ec0be, 68 },
    { 0, 0 },
    { 0x5c6ec0c0, 23 },
    { 0x694ec0c1, 76 },
    { 0xa6aec0c1, 95 },
    { 0x159ec0c3, 34 },
    { 0x357ec0c3, 90 },
    { 0x52aec0c3, 33 },
    { 0, 0 },
    { 0x942ec0c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbec0cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2ec0d3, 61 },
    { 0, 0 },
    { 0x1a7ec0d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feec0dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x177ec0df, 23 },
    { 0, 0 },
    { 0x5e4ec0e1, 61 },
    { 0, 0 },
    { 0xc96ec0e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cec0e6, 33 },
    { 0x4e9ec0e7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daec0ec, 23 },
    { 0x4dfec0ed, 33 },
    { 0x854ec0ee, 98 },
    { 0xdedec0ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ec0f6, 98 },
    { 0x878ec0f7, 76 },
    { 0, 0 },
    { 0x3beec0f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361ec0fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feec102, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cec10a, 33 },
    { 0, 0 },
    { 0x815ec10c, 90 },
    { 0x626ec10c, 91 },
    { 0xe8ec10e, 87 },
    { 0x235ec10f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeefec114, 34 },
    { 0, 0 },
    { 0x7f3ec116, 55 },
    { 0xedfec117, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x169ec11a, 98 },
    { 0xba0ec11a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ec11e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6ec124, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x386ec127, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7fec136, 90 },
    { 0x8f1ec136, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff7ec13b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5ec13e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cec148, 60 },
    { 0x289ec149, 98 },
    { 0xb51ec14a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4ec150, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fec153, 33 },
    { 0, 0 },
    { 0x278ec155, 90 },
    { 0xacec156, 90 },
    { 0x9c9ec156, 85 },
    { 0x762ec155, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bec15b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcec162, 33 },
    { 0xcd2ec163, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ec167, 23 },
    { 0x67fec167, 95 },
    { 0x5f8ec169, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x699ec16c, 76 },
    { 0, 0 },
    { 0xa40ec16e, 98 },
    { 0x860ec16f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10ec176, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ec17b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1aec17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcec182, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bec187, 91 },
    { 0xe63ec188, 61 },
    { 0x551ec189, 100 },
    { 0xc09ec18a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7ec18d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ec193, 95 },
    { 0, 0 },
    { 0x38ec195, 43 },
    { 0xadfec195, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fec19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fec1a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170ec1a5, 95 },
    { 0x489ec1a5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6dec1a9, 52 },
    { 0xb12ec1aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeec1af, 87 },
    { 0xfceec1b0, 61 },
    { 0, 0 },
    { 0x279ec1b2, 23 },
    { 0x1c2ec1b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620ec1b8, 93 },
    { 0xe79ec1b9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1eec1bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40aec1c5, 89 },
    { 0, 0 },
    { 0x5c2ec1c7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ec1d0, 61 },
    { 0x2faec1d1, 34 },
    { 0xcb2ec1d0, 98 },
    { 0, 0 },
    { 0x43bec1d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eec1d8, 69 },
    { 0, 0 },
    { 0x589ec1da, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceec1de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e3ec1e4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5ec1e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deec1ea, 55 },
    { 0x7caec1eb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ec1f5, 95 },
    { 0xcf0ec1f6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658ec1fa, 76 },
    { 0xfd7ec1fb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eec203, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x37aec206, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ec209, 65 },
    { 0, 0 },
    { 0xce6ec20b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847ec212, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x294ec215, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fec219, 85 },
    { 0xbf1ec219, 33 },
    { 0x244ec219, 61 },
    { 0, 0 },
    { 0xc86ec21d, 61 },
    { 0x293ec21e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2ec221, 61 },
    { 0x181ec222, 90 },
    { 0, 0 },
    { 0xf7ec224, 61 },
    { 0xc59ec225, 87 },
    { 0, 0 },
    { 0x353ec227, 17 },
    { 0xa14ec228, 98 },
    { 0x598ec229, 95 },
    { 0, 0 },
    { 0xdf7ec22b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8ec231, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eec236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dec23b, 46 },
    { 0xc0dec23b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9eec23f, 40 },
    { 0, 0 },
    { 0xcdeec241, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1ec250, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218ec258, 90 },
    { 0, 0 },
    { 0x3b2ec25a, 76 },
    { 0x81eec25b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfec25f, 55 },
    { 0, 0 },
    { 0x19cec261, 23 },
    { 0xac8ec261, 77 },
    { 0x6a6ec263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ec26a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbec26e, 34 },
    { 0, 0 },
    { 0x279ec270, 34 },
    { 0x589ec271, 62 },
    { 0xffdec272, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeec27d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fec283, 95 },
    { 0x4ceec283, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dec287, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x97bec28a, 90 },
    { 0x7f7ec28b, 90 },
    { 0x4c4ec28c, 78 },
    { 0x17aec28d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ec290, 76 },
    { 0xaeaec290, 55 },
    { 0x641ec290, 21 },
    { 0x850ec290, 33 },
    { 0x211ec294, 95 },
    { 0x812ec295, 95 },
    { 0xc31ec291, 33 },
    { 0, 0 },
    { 0x1bdec298, 99 },
    { 0, 0 },
    { 0x3bdec29a, 55 },
    { 0x96aec29b, 34 },
    { 0x5fec29c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x317ec2a0, 77 },
    { 0, 0 },
    { 0xce3ec2a2, 78 },
    { 0x377ec2a2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x855ec2a7, 77 },
    { 0, 0 },
    { 0x532ec2a9, 67 },
    { 0x896ec2a9, 98 },
    { 0xe42ec2a9, 43 },
    { 0x8dec2ab, 100 },
    { 0xb52ec2a9, 93 },
    { 0xe7aec2ae, 55 },
    { 0xca3ec2a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1ec2b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa67ec2ba, 33 },
    { 0x1e5ec2bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289ec2c3, 90 },
    { 0x152ec2c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cec2c7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1ec2cd, 69 },
    { 0, 0 },
    { 0xc60ec2cf, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8ec2d9, 76 },
    { 0xf67ec2da, 76 },
    { 0x6bbec2db, 88 },
    { 0xa81ec2dc, 93 },
    { 0xc32ec2d9, 98 },
    { 0xd28ec2dc, 78 },
    { 0, 0 },
    { 0x737ec2e0, 33 },
    { 0x6ecec2e0, 69 },
    { 0x7aec2e2, 93 },
    { 0x24bec2e3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fec2e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daec2ea, 52 },
    { 0x51fec2ea, 87 },
    { 0, 0 },
    { 0x50fec2ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aec2f3, 95 },
    { 0x121ec2f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafec2fb, 90 },
    { 0, 0 },
    { 0x567ec2fd, 99 },
    { 0xec4ec2fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18ec308, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6ec311, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afec314, 99 },
    { 0x53cec315, 55 },
    { 0xce9ec316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf45ec31b, 55 },
    { 0x7b4ec31c, 95 },
    { 0x897ec31c, 76 },
    { 0xda3ec31e, 97 },
    { 0x836ec31b, 23 },
    { 0xd0eec31f, 76 },
    { 0x523ec321, 33 },
    { 0x1dcec322, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1ec325, 34 },
    { 0, 0 },
    { 0x4d6ec327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4eec32c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbeec335, 33 },
    { 0xd06ec336, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ec33a, 81 },
    { 0, 0 },
    { 0xa45ec33c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88fec343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458ec34a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52ec34d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5aec351, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1ec35d, 90 },
    { 0x9ddec35d, 81 },
    { 0xbd6ec35d, 33 },
    { 0, 0 },
    { 0x25aec361, 90 },
    { 0x40ec361, 61 },
    { 0xcedec361, 98 },
    { 0x527ec364, 68 },
    { 0xfb0ec361, 61 },
    { 0, 0 },
    { 0xbdfec367, 90 },
    { 0xb3dec367, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aeec36c, 23 },
    { 0, 0 },
    { 0x53aec36e, 52 },
    { 0xf8bec36f, 43 },
    { 0, 0 },
    { 0x9dcec371, 55 },
    { 0x7d4ec371, 76 },
    { 0xb42ec372, 100 },
    { 0, 0 },
    { 0x4e2ec375, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969ec379, 98 },
    { 0, 0 },
    { 0x8d0ec37b, 33 },
    { 0, 0 },
    { 0x64ec37d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dec381, 95 },
    { 0xa5eec382, 23 },
    { 0x69bec383, 98 },
    { 0xb38ec384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ec388, 83 },
    { 0x418ec388, 90 },
    { 0, 0 },
    { 0x1ceec38b, 98 },
    { 0, 0 },
    { 0x45cec38d, 100 },
    { 0xe08ec38d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccec399, 61 },
    { 0, 0 },
    { 0xcf3ec39b, 90 },
    { 0, 0 },
    { 0xb70ec39d, 83 },
    { 0xcbdec39e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ec3a1, 61 },
    { 0, 0 },
    { 0xad2ec3a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2ec3a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34ec3a9, 55 },
    { 0xffbec3aa, 76 },
    { 0, 0 },
    { 0x555ec3ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcec3b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2ec3b6, 84 },
    { 0x14fec3b7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8dec3c0, 23 },
    { 0, 0 },
    { 0xf45ec3c2, 61 },
    { 0, 0 },
    { 0xbd2ec3c4, 55 },
    { 0x2e2ec3c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddec3ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ec3cd, 55 },
    { 0x80bec3ce, 43 },
    { 0, 0 },
    { 0xd5aec3d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77ec3d5, 93 },
    { 0xd1dec3d6, 61 },
    { 0x711ec3d7, 43 },
    { 0x8acec3d6, 55 },
    { 0xf32ec3d7, 92 },
    { 0xfd0ec3d6, 33 },
    { 0x737ec3d5, 43 },
    { 0xa34ec3dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcec3e5, 23 },
    { 0x21aec3e6, 67 },
    { 0xa7fec3e7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bec3ea, 61 },
    { 0, 0 },
    { 0xffec3ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x135ec3ef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57eec3f8, 23 },
    { 0x15bec3f8, 90 },
    { 0xa3fec3f8, 23 },
    { 0xb9cec3fb, 43 },
    { 0x874ec3f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96ec403, 66 },
    { 0, 0 },
    { 0xe0bec405, 75 },
    { 0, 0 },
    { 0xdc9ec407, 23 },
    { 0x126ec408, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x929ec40b, 52 },
    { 0xa19ec40b, 90 },
    { 0x826ec40d, 91 },
    { 0x672ec40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8ec413, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x715ec416, 78 },
    { 0xa2eec417, 98 },
    { 0x279ec416, 55 },
    { 0, 0 },
    { 0x441ec41a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec3ec421, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5ec424, 55 },
    { 0xbcaec425, 23 },
    { 0xe98ec426, 34 },
    { 0x6caec424, 76 },
    { 0x6bdec428, 23 },
    { 0xe97ec428, 23 },
    { 0x893ec425, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x219ec42d, 76 },
    { 0x2c6ec42e, 76 },
    { 0x161ec42f, 78 },
    { 0, 0 },
    { 0x8e3ec431, 78 },
    { 0, 0 },
    { 0x27eec433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5dec438, 87 },
    { 0x645ec438, 33 },
    { 0, 0 },
    { 0xb9cec43b, 95 },
    { 0xb96ec43c, 97 },
    { 0x6bcec43d, 33 },
    { 0xb4aec43e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aec441, 55 },
    { 0, 0 },
    { 0xd85ec443, 61 },
    { 0xeacec443, 95 },
    { 0xb4fec443, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adec44c, 65 },
    { 0x934ec44c, 23 },
    { 0x8eaec44e, 61 },
    { 0xc11ec44d, 43 },
    { 0, 0 },
    { 0xa70ec451, 43 },
    { 0x344ec452, 23 },
    { 0, 0 },
    { 0xca1ec454, 40 },
    { 0x71bec455, 61 },
    { 0xfeaec456, 33 },
    { 0xb31ec457, 98 },
    { 0, 0 },
    { 0xf28ec459, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1ec45d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2ec466, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07ec46b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3ec46e, 51 },
    { 0x389ec46f, 33 },
    { 0x451ec470, 48 },
    { 0, 0 },
    { 0xc44ec472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc50ec478, 23 },
    { 0x6c4ec478, 61 },
    { 0xb0dec47a, 75 },
    { 0x8cbec47b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x721ec47e, 61 },
    { 0x187ec47e, 33 },
    { 0xa7ec480, 98 },
    { 0xf67ec480, 95 },
    { 0xee2ec47e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x426ec485, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x107ec490, 23 },
    { 0xbc3ec490, 90 },
    { 0x535ec490, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751ec49b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cec4a4, 23 },
    { 0xd26ec4a4, 23 },
    { 0x3deec4a5, 33 },
    { 0, 0 },
    { 0xf91ec4a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ec4ad, 90 },
    { 0xa42ec4ad, 61 },
    { 0, 0 },
    { 0x872ec4b0, 91 },
    { 0xdeaec4b1, 38 },
    { 0x301ec4b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05ec4b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ec4ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9aec4bf, 95 },
    { 0x193ec4c0, 90 },
    { 0x274ec4bf, 90 },
    { 0x231ec4c2, 23 },
    { 0xc34ec4c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ec4ca, 76 },
    { 0, 0 },
    { 0x7aeec4cc, 95 },
    { 0x9d3ec4cd, 34 },
    { 0xdceec4ce, 61 },
    { 0xc4ec4cf, 90 },
    { 0x2feec4d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48ec4d5, 78 },
    { 0xa53ec4d6, 90 },
    { 0x208ec4d7, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bec4dc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8ec4e1, 61 },
    { 0x674ec4e2, 78 },
    { 0xd05ec4e3, 33 },
    { 0xf09ec4e3, 98 },
    { 0x51ec4e2, 55 },
    { 0x80aec4e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8aec4e9, 95 },
    { 0xa5fec4ea, 76 },
    { 0xae0ec4ea, 38 },
    { 0x86bec4e9, 33 },
    { 0xfa3ec4ed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242ec4f2, 98 },
    { 0x60aec4f3, 90 },
    { 0x6c5ec4f3, 81 },
    { 0xfc1ec4f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddec4fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aec4fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffec502, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ec505, 61 },
    { 0x352ec505, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x25eec509, 61 },
    { 0x4c0ec50a, 61 },
    { 0x6ec509, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cec513, 99 },
    { 0x132ec513, 34 },
    { 0x9d5ec515, 100 },
    { 0x2ecec516, 93 },
    { 0xc14ec516, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350ec522, 61 },
    { 0x4a5ec522, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bec527, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aec52a, 95 },
    { 0, 0 },
    { 0xa0dec52c, 96 },
    { 0x376ec52d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x28cec530, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3ec533, 61 },
    { 0xc0ec533, 43 },
    { 0x72fec535, 81 },
    { 0xa10ec536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2ec539, 75 },
    { 0xdc6ec53a, 19 },
    { 0xe93ec53b, 33 },
    { 0, 0 },
    { 0x93eec53d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1ec541, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faec544, 66 },
    { 0xc7dec544, 65 },
    { 0xb2cec545, 100 },
    { 0, 0 },
    { 0x57cec548, 33 },
    { 0x6a7ec548, 81 },
    { 0x984ec54a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fec552, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44ec555, 52 },
    { 0, 0 },
    { 0xbbcec557, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bec55b, 95 },
    { 0, 0 },
    { 0x6faec55d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94ec563, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ec566, 98 },
    { 0, 0 },
    { 0x72cec568, 91 },
    { 0x7fdec569, 33 },
    { 0, 0 },
    { 0x52bec56b, 87 },
    { 0x325ec56c, 87 },
    { 0x8d8ec56d, 100 },
    { 0x6f5ec56d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985ec572, 90 },
    { 0, 0 },
    { 0xe7ec574, 52 },
    { 0x517ec575, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ec57a, 23 },
    { 0x6d8ec57a, 76 },
    { 0x97ec57b, 33 },
    { 0x783ec57d, 100 },
    { 0xe0cec57d, 99 },
    { 0x831ec57b, 33 },
    { 0xbb4ec580, 46 },
    { 0xc3fec581, 99 },
    { 0, 0 },
    { 0xfd5ec583, 33 },
    { 0x680ec583, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdec588, 87 },
    { 0xe5aec589, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41ec591, 90 },
    { 0xbbcec592, 95 },
    { 0xdd6ec593, 61 },
    { 0, 0 },
    { 0xa2fec595, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0ec598, 55 },
    { 0, 0 },
    { 0xc6dec59a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2afec59f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefec5a3, 95 },
    { 0x1a6ec5a4, 33 },
    { 0, 0 },
    { 0xfc1ec5a6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x252ec5a9, 61 },
    { 0xf70ec5aa, 85 },
    { 0xfc1ec5ab, 55 },
    { 0x793ec5ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9ec5b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25ec5b6, 99 },
    { 0x6ceec5b7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00ec5c0, 90 },
    { 0x533ec5c1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2aec5c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dec5c8, 90 },
    { 0x754ec5c9, 61 },
    { 0, 0 },
    { 0x657ec5cb, 95 },
    { 0xfddec5cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cec5cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x866ec5d2, 23 },
    { 0x12ec5d2, 90 },
    { 0x5fbec5d3, 34 },
    { 0, 0 },
    { 0xebdec5d6, 61 },
    { 0xc79ec5d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccec5da, 23 },
    { 0, 0 },
    { 0x94dec5dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fec5e0, 33 },
    { 0x40dec5e0, 23 },
    { 0xcf1ec5e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230ec5ea, 90 },
    { 0xb2cec5ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbec5f1, 81 },
    { 0, 0 },
    { 0x2e8ec5f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656ec5fc, 93 },
    { 0, 0 },
    { 0xee5ec5fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ec603, 76 },
    { 0x51ec604, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832ec60e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4ec612, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadec616, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fec619, 46 },
    { 0xa36ec61a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0ec61e, 67 },
    { 0x3faec61e, 98 },
    { 0, 0 },
    { 0x558ec621, 68 },
    { 0x1c4ec621, 55 },
    { 0xebbec623, 45 },
    { 0x273ec624, 52 },
    { 0x393ec625, 61 },
    { 0xbbec623, 95 },
    { 0x69dec627, 98 },
    { 0xadec622, 90 },
    { 0x52aec629, 66 },
    { 0x465ec62a, 90 },
    { 0xbdfec629, 23 },
    { 0x5b1ec626, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3ec62f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb40ec632, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678ec638, 36 },
    { 0x4d0ec639, 90 },
    { 0xa18ec638, 33 },
    { 0x1f7ec63a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2ec63f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddec643, 90 },
    { 0xb47ec644, 84 },
    { 0xe52ec645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c1ec64b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1ec64f, 23 },
    { 0xe0bec64f, 95 },
    { 0, 0 },
    { 0xb40ec652, 33 },
    { 0, 0 },
    { 0x451ec654, 93 },
    { 0, 0 },
    { 0xd98ec656, 95 },
    { 0xa99ec656, 66 },
    { 0xe0eec658, 95 },
    { 0x21ec657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdec661, 76 },
    { 0, 0 },
    { 0xa27ec663, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71ec668, 91 },
    { 0x5f9ec668, 98 },
    { 0xb26ec668, 77 },
    { 0xf41ec66b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaec66f, 93 },
    { 0xb3dec66f, 55 },
    { 0xb7eec66f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199ec676, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bec679, 33 },
    { 0x2d0ec679, 99 },
    { 0x74cec67b, 61 },
    { 0x180ec67c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbec687, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6ec68c, 98 },
    { 0x1fdec68d, 90 },
    { 0x2b3ec68e, 33 },
    { 0x1daec68f, 43 },
    { 0xb27ec68c, 100 },
    { 0xe7cec68f, 98 },
    { 0xc79ec692, 56 },
    { 0, 0 },
    { 0xdafec694, 68 },
    { 0xd1aec695, 67 },
    { 0x24eec694, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60ec69e, 90 },
    { 0x122ec69e, 99 },
    { 0x74fec69e, 100 },
    { 0x9a6ec6a0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfec6af, 33 },
    { 0x2eec6af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372ec6b4, 61 },
    { 0, 0 },
    { 0xfd8ec6b6, 95 },
    { 0xcf6ec6b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcec6bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfec6c9, 61 },
    { 0, 0 },
    { 0xb67ec6cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935ec6d5, 23 },
    { 0, 0 },
    { 0x59eec6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04ec6db, 87 },
    { 0x998ec6db, 91 },
    { 0xb26ec6dd, 98 },
    { 0, 0 },
    { 0xa15ec6df, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fec6e2, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764ec6ea, 88 },
    { 0, 0 },
    { 0x94aec6ec, 78 },
    { 0xe73ec6ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5ec6f0, 95 },
    { 0xae6ec6f1, 23 },
    { 0x1dcec6f2, 33 },
    { 0, 0 },
    { 0x874ec6f4, 36 },
    { 0x2a8ec6f4, 61 },
    { 0x521ec6f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dec6f9, 91 },
    { 0x498ec6fa, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfec6fd, 95 },
    { 0xa33ec6fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x263ec703, 23 },
    { 0xf67ec704, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22ec70a, 98 },
    { 0, 0 },
    { 0x8f3ec70c, 34 },
    { 0xe9dec70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1ec711, 98 },
    { 0x625ec712, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55ec717, 90 },
    { 0, 0 },
    { 0x9e9ec719, 90 },
    { 0x74dec719, 76 },
    { 0xca1ec71b, 61 },
    { 0x50ec71c, 87 },
    { 0x483ec71d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7ec720, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8ec723, 93 },
    { 0x376ec724, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eec72b, 90 },
    { 0x18cec72b, 93 },
    { 0xa19ec72d, 66 },
    { 0xb31ec72d, 69 },
    { 0, 0 },
    { 0x1fdec730, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473ec734, 90 },
    { 0x40fec735, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4beec73a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x176ec73d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eec747, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bec74d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bec755, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9ec75a, 90 },
    { 0x6acec75b, 61 },
    { 0x32aec75b, 61 },
    { 0xa46ec75d, 23 },
    { 0x539ec75b, 95 },
    { 0xb1aec75f, 33 },
    { 0, 0 },
    { 0x532ec761, 66 },
    { 0xd57ec762, 90 },
    { 0, 0 },
    { 0xc90ec764, 33 },
    { 0xa49ec765, 61 },
    { 0x5dcec766, 23 },
    { 0, 0 },
    { 0x7a2ec768, 99 },
    { 0x4eaec769, 33 },
    { 0x331ec76a, 96 },
    { 0x717ec769, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7ec76e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676ec773, 43 },
    { 0x8a4ec774, 88 },
    { 0x86eec775, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdec779, 66 },
    { 0x510ec779, 34 },
    { 0xea8ec77b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaec781, 34 },
    { 0, 0 },
    { 0xca0ec783, 87 },
    { 0xcbcec784, 34 },
    { 0xc55ec785, 87 },
    { 0, 0 },
    { 0xa66ec787, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ec78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x324ec791, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230ec799, 33 },
    { 0, 0 },
    { 0x782ec79b, 68 },
    { 0xec8ec79c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x117ec7a0, 84 },
    { 0, 0 },
    { 0x413ec7a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ec7a9, 99 },
    { 0x294ec7aa, 91 },
    { 0x4bec7a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaec7ae, 90 },
    { 0, 0 },
    { 0xbeaec7b0, 90 },
    { 0, 0 },
    { 0xb86ec7b2, 61 },
    { 0x4daec7b2, 66 },
    { 0, 0 },
    { 0x358ec7b5, 90 },
    { 0xebaec7b6, 43 },
    { 0xef2ec7b7, 61 },
    { 0x36cec7b6, 60 },
    { 0x7a2ec7b9, 78 },
    { 0x326ec7ba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3ec7bd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ec7c1, 55 },
    { 0, 0 },
    { 0xde0ec7c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x434ec7c7, 55 },
    { 0, 0 },
    { 0x71bec7c9, 90 },
    { 0x9cdec7ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6ec7d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceec7e0, 93 },
    { 0xcc8ec7e1, 55 },
    { 0x5a4ec7e2, 95 },
    { 0xcfaec7e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77aec7e6, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955ec7ea, 43 },
    { 0x54dec7ea, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x732ec7ef, 55 },
    { 0x82bec7f0, 55 },
    { 0x860ec7f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddec7f4, 61 },
    { 0x419ec7f5, 23 },
    { 0x640ec7f5, 98 },
    { 0, 0 },
    { 0x423ec7f8, 98 },
    { 0x692ec7f9, 91 },
    { 0x444ec7f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ec7ff, 95 },
    { 0x14ec800, 52 },
    { 0xe27ec801, 90 },
    { 0x28fec801, 33 },
    { 0, 0 },
    { 0x6f9ec804, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x997ec80a, 91 },
    { 0, 0 },
    { 0xdf6ec80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeeec810, 66 },
    { 0xd95ec810, 46 },
    { 0x1f2ec810, 98 },
    { 0xba5ec813, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311ec817, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4ec81b, 52 },
    { 0x8c1ec81b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6ec822, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5ec825, 93 },
    { 0, 0 },
    { 0x1f5ec827, 97 },
    { 0, 0 },
    { 0xf33ec829, 33 },
    { 0x97bec82a, 61 },
    { 0x16fec82b, 55 },
    { 0xd4fec82c, 61 },
    { 0xb97ec82b, 90 },
    { 0x9a9ec82e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6cec834, 61 },
    { 0, 0 },
    { 0x737ec836, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77dec83b, 23 },
    { 0x496ec83c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bec841, 66 },
    { 0, 0 },
    { 0xf89ec843, 55 },
    { 0, 0 },
    { 0xcd1ec845, 90 },
    { 0x5bfec845, 98 },
    { 0, 0 },
    { 0xd39ec848, 90 },
    { 0, 0 },
    { 0x3f3ec84a, 33 },
    { 0, 0 },
    { 0x8e9ec84c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bec854, 55 },
    { 0, 0 },
    { 0xe6ec856, 52 },
    { 0x38cec857, 91 },
    { 0xd39ec858, 55 },
    { 0, 0 },
    { 0x79bec85a, 95 },
    { 0x1daec85a, 61 },
    { 0x190ec85a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x175ec861, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13ec864, 55 },
    { 0x438ec864, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33ec872, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bec876, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ec879, 67 },
    { 0xb06ec87a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47cec886, 61 },
    { 0x4edec887, 90 },
    { 0x535ec888, 23 },
    { 0xa03ec889, 33 },
    { 0xb0bec88a, 90 },
    { 0, 0 },
    { 0x6c4ec88c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852ec890, 61 },
    { 0x9e8ec891, 95 },
    { 0, 0 },
    { 0xd9dec893, 23 },
    { 0x76dec893, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x163ec899, 61 },
    { 0x5d5ec899, 69 },
    { 0, 0 },
    { 0x346ec89c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dec8a1, 95 },
    { 0x7deec8a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ec8a7, 90 },
    { 0, 0 },
    { 0xbf4ec8a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40ec8b1, 34 },
    { 0, 0 },
    { 0x81cec8b3, 98 },
    { 0xe02ec8b4, 34 },
    { 0x29fec8b4, 76 },
    { 0, 0 },
    { 0xa35ec8b7, 34 },
    { 0, 0 },
    { 0xfd1ec8b9, 95 },
    { 0x327ec8ba, 33 },
    { 0x5dbec8b9, 66 },
    { 0x7aeec8bc, 34 },
    { 0xc07ec8ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaec8c3, 23 },
    { 0, 0 },
    { 0xd23ec8c5, 76 },
    { 0, 0 },
    { 0x244ec8c7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54ec8cd, 95 },
    { 0xb97ec8cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbec8d1, 52 },
    { 0x3a4ec8d1, 84 },
    { 0, 0 },
    { 0xa80ec8d4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x825ec8d7, 98 },
    { 0x855ec8d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafec8e0, 95 },
    { 0xd28ec8e1, 43 },
    { 0, 0 },
    { 0xeaaec8e3, 33 },
    { 0xfb6ec8e3, 99 },
    { 0x347ec8e4, 95 },
    { 0, 0 },
    { 0xc3ec8e7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144ec8f0, 23 },
    { 0x37ec8f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36ec8f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942ec8fe, 65 },
    { 0x5edec8fe, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ec902, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37ec90c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdec910, 34 },
    { 0xde3ec911, 52 },
    { 0x714ec910, 95 },
    { 0xbbfec913, 33 },
    { 0xc87ec910, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9ec917, 61 },
    { 0x5e1ec918, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1ec91e, 55 },
    { 0x7f4ec91f, 91 },
    { 0x95bec920, 93 },
    { 0x425ec91f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70ec924, 90 },
    { 0, 0 },
    { 0xb3cec926, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf8ec92c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e9ec936, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22ec93d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86ec940, 98 },
    { 0xd6bec941, 66 },
    { 0, 0 },
    { 0x71ec943, 95 },
    { 0x706ec944, 67 },
    { 0x906ec945, 99 },
    { 0, 0 },
    { 0x1bec947, 95 },
    { 0x529ec948, 76 },
    { 0, 0 },
    { 0xbebec94a, 61 },
    { 0xa19ec94b, 76 },
    { 0x585ec94b, 100 },
    { 0xc80ec94b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceec952, 46 },
    { 0, 0 },
    { 0xe76ec954, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cec958, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afec95b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2ec95e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaec961, 90 },
    { 0, 0 },
    { 0xe79ec963, 84 },
    { 0x28ec964, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x907ec967, 90 },
    { 0x368ec968, 61 },
    { 0xf38ec968, 33 },
    { 0x2a6ec969, 52 },
    { 0x40fec96b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a7ec96f, 46 },
    { 0, 0 },
    { 0x302ec971, 90 },
    { 0x671ec971, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eec975, 98 },
    { 0, 0 },
    { 0x9fcec977, 33 },
    { 0xcc0ec978, 90 },
    { 0xcfcec977, 91 },
    { 0, 0 },
    { 0xc67ec97b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3ec97e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12ec986, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ec989, 84 },
    { 0xebcec98a, 95 },
    { 0x9d1ec98b, 95 },
    { 0, 0 },
    { 0x34fec98d, 90 },
    { 0, 0 },
    { 0x80fec98f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc5ec993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3ec99a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28ec99e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ec9a2, 69 },
    { 0x83bec9a3, 98 },
    { 0x566ec9a2, 90 },
    { 0xe32ec9a2, 33 },
    { 0x37ec9a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bec9a9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235ec9ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38ec9b2, 93 },
    { 0xa5fec9b2, 100 },
    { 0, 0 },
    { 0x422ec9b5, 91 },
    { 0xdeeec9b6, 43 },
    { 0x20dec9b6, 34 },
    { 0x113ec9b8, 90 },
    { 0xc78ec9b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983ec9bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3eec9c2, 33 },
    { 0x7ffec9c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bec9c7, 65 },
    { 0x59fec9c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792ec9cd, 23 },
    { 0, 0 },
    { 0x777ec9cf, 23 },
    { 0x6eeec9d0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6ec9d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ec9db, 23 },
    { 0xd84ec9db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81dec9e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401ec9f8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43eca01, 55 },
    { 0, 0 },
    { 0x10deca03, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3eca07, 33 },
    { 0x989eca08, 52 },
    { 0xcadeca08, 90 },
    { 0, 0 },
    { 0xc5beca0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x410eca0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5beca11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10eca16, 88 },
    { 0xf01eca17, 61 },
    { 0x8b1eca16, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3eca1d, 98 },
    { 0, 0 },
    { 0xb99eca1f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2eca26, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ceca2b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x758eca2e, 90 },
    { 0, 0 },
    { 0x131eca30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54eeca36, 90 },
    { 0x4f3eca36, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14deca40, 98 },
    { 0, 0 },
    { 0xb9aeca42, 77 },
    { 0x3baeca43, 90 },
    { 0, 0 },
    { 0x6c7eca45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4beca48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2eca4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3eca52, 76 },
    { 0xe7feca52, 93 },
    { 0, 0 },
    { 0x286eca55, 46 },
    { 0x2fceca56, 46 },
    { 0x96aeca57, 55 },
    { 0x19beca58, 33 },
    { 0xc26eca58, 76 },
    { 0x1e5eca5a, 46 },
    { 0x89deca58, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422eca5f, 90 },
    { 0x5bbeca60, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679eca65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24eca6a, 85 },
    { 0xe9feca6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ceca6e, 95 },
    { 0x80eeca6f, 91 },
    { 0x322eca6e, 61 },
    { 0xcaeeca71, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0eca7e, 23 },
    { 0x6f0eca7f, 95 },
    { 0xfb8eca80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3eca83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36eca86, 90 },
    { 0xdf5eca86, 98 },
    { 0x7f0eca88, 34 },
    { 0x4e5eca88, 61 },
    { 0, 0 },
    { 0x55aeca8b, 23 },
    { 0, 0 },
    { 0xb2deca8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935eca95, 61 },
    { 0x72beca95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53eca99, 40 },
    { 0x9aaeca9a, 42 },
    { 0xfc2eca9b, 61 },
    { 0x9efeca9c, 90 },
    { 0x923eca9d, 95 },
    { 0x1f1eca9c, 46 },
    { 0, 0 },
    { 0xf55ecaa0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26ecaaa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ecab5, 61 },
    { 0x818ecab6, 33 },
    { 0xc37ecab5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9decabc, 97 },
    { 0xbb0ecabc, 78 },
    { 0xf34ecabe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x50eecac1, 98 },
    { 0, 0 },
    { 0xa82ecac3, 93 },
    { 0x6b6ecac4, 43 },
    { 0xde2ecac4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ecac8, 97 },
    { 0, 0 },
    { 0xa8eecaca, 67 },
    { 0xcc5ecacb, 95 },
    { 0, 0 },
    { 0xa1fecacd, 98 },
    { 0x8acecacd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13decad5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5ecae2, 75 },
    { 0, 0 },
    { 0xc65ecae4, 88 },
    { 0, 0 },
    { 0x4e4ecae6, 87 },
    { 0x4a4ecae6, 76 },
    { 0x993ecae8, 36 },
    { 0x78aecae9, 76 },
    { 0x2d9ecae6, 23 },
    { 0, 0 },
    { 0xf30ecaec, 61 },
    { 0, 0 },
    { 0x12decaee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ecaf1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x44fecaf4, 34 },
    { 0x5dbecaf4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x338ecaf8, 66 },
    { 0xbceecaf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34ecb00, 93 },
    { 0, 0 },
    { 0xd9fecb02, 33 },
    { 0x6feecb02, 76 },
    { 0xdd3ecb02, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ecb0c, 99 },
    { 0x2beecb0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95ecb16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196ecb1b, 75 },
    { 0x4efecb1c, 61 },
    { 0, 0 },
    { 0x3edecb1e, 90 },
    { 0xee7ecb1f, 100 },
    { 0, 0 },
    { 0xccaecb21, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ecb24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ecb2a, 34 },
    { 0x5a3ecb2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cecb32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01ecb39, 91 },
    { 0x956ecb39, 85 },
    { 0, 0 },
    { 0xe93ecb3c, 43 },
    { 0x3fbecb3d, 86 },
    { 0xf15ecb3c, 34 },
    { 0, 0 },
    { 0x94fecb40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902ecb45, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0ecb49, 66 },
    { 0xeb2ecb49, 61 },
    { 0xe30ecb4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90aecb4f, 43 },
    { 0x306ecb50, 97 },
    { 0, 0 },
    { 0xabbecb52, 90 },
    { 0xd26ecb53, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccecb59, 34 },
    { 0x312ecb5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbecb5e, 23 },
    { 0x6eaecb5f, 55 },
    { 0, 0 },
    { 0x782ecb61, 98 },
    { 0, 0 },
    { 0x23decb63, 67 },
    { 0, 0 },
    { 0x7aeecb65, 61 },
    { 0xc02ecb66, 55 },
    { 0x7fbecb66, 36 },
    { 0x93fecb66, 23 },
    { 0x230ecb67, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58ecb6c, 60 },
    { 0xd99ecb6d, 100 },
    { 0x930ecb6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03ecb76, 52 },
    { 0xe20ecb77, 90 },
    { 0x1dcecb78, 93 },
    { 0x763ecb76, 61 },
    { 0x6b3ecb76, 99 },
    { 0xf47ecb7b, 90 },
    { 0xec4ecb7c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x34decb7f, 76 },
    { 0, 0 },
    { 0x379ecb81, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6ecb84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588ecb89, 90 },
    { 0x77becb8a, 67 },
    { 0x83becb8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fecb8e, 90 },
    { 0xcb4ecb8e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebeecb94, 33 },
    { 0x4c2ecb95, 98 },
    { 0x970ecb94, 100 },
    { 0x9b1ecb97, 90 },
    { 0xea1ecb97, 61 },
    { 0, 0 },
    { 0x1faecb9a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4ecb9e, 52 },
    { 0x326ecb9e, 23 },
    { 0, 0 },
    { 0x62becba1, 95 },
    { 0, 0 },
    { 0x715ecba3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x114ecba8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ecbac, 33 },
    { 0x78cecbac, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61ecbb1, 69 },
    { 0xb43ecbb2, 76 },
    { 0x722ecbb3, 33 },
    { 0x1eecbb4, 90 },
    { 0x409ecbb5, 55 },
    { 0xdb6ecbb5, 87 },
    { 0x3a4ecbb5, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97ecbbd, 98 },
    { 0xb34ecbbe, 87 },
    { 0x14eecbbf, 95 },
    { 0x4d9ecbc0, 52 },
    { 0x646ecbbe, 43 },
    { 0, 0 },
    { 0x380ecbc3, 43 },
    { 0, 0 },
    { 0x705ecbc5, 33 },
    { 0xa34ecbc5, 61 },
    { 0x502ecbc7, 85 },
    { 0xcb3ecbc7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86ecbcc, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9ecbdb, 46 },
    { 0x673ecbdc, 43 },
    { 0x8b6ecbdb, 90 },
    { 0x89decbdc, 95 },
    { 0, 0 },
    { 0xbbcecbe0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9ecbe6, 87 },
    { 0x8baecbe6, 61 },
    { 0xad0ecbe8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddecbec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19ecbf3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddecbf7, 98 },
    { 0x2afecbf8, 93 },
    { 0xdc8ecbf8, 43 },
    { 0x841ecbfa, 90 },
    { 0x714ecbfb, 46 },
    { 0x57eecbf7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98ecbff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5ecc04, 33 },
    { 0, 0 },
    { 0x9aeecc06, 53 },
    { 0x1dfecc07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41ecc0c, 34 },
    { 0x29fecc0d, 99 },
    { 0xce8ecc0e, 23 },
    { 0x6daecc0f, 67 },
    { 0x3c6ecc0f, 61 },
    { 0xe25ecc11, 46 },
    { 0x3e6ecc12, 17 },
    { 0x251ecc0c, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ecc18, 55 },
    { 0, 0 },
    { 0x6ebecc1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2ecc1f, 76 },
    { 0x139ecc20, 98 },
    { 0xe75ecc21, 95 },
    { 0xed0ecc21, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca2ecc27, 52 },
    { 0, 0 },
    { 0x500ecc29, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacecc2d, 65 },
    { 0x2d2ecc2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbecc35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x33becc38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cecc3b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc76ecc43, 68 },
    { 0x9dfecc43, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6aecc49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92ecc4c, 93 },
    { 0, 0 },
    { 0xffecc4e, 76 },
    { 0x54cecc4e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbececc56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd95ecc5b, 88 },
    { 0x726ecc5c, 33 },
    { 0x765ecc5b, 55 },
    { 0, 0 },
    { 0x3dfecc5f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afecc62, 97 },
    { 0x1d2ecc63, 95 },
    { 0x48fecc64, 91 },
    { 0xd68ecc65, 75 },
    { 0xc53ecc66, 34 },
    { 0, 0 },
    { 0x6deecc68, 61 },
    { 0x6dcecc68, 33 },
    { 0xa81ecc69, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ecc71, 52 },
    { 0xebeecc71, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdecc75, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2ecc79, 23 },
    { 0xf9decc7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4becc7e, 88 },
    { 0x30becc7f, 61 },
    { 0xfd0ecc80, 76 },
    { 0xe00ecc80, 87 },
    { 0, 0 },
    { 0x22becc83, 91 },
    { 0xd6decc84, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64fecc8c, 88 },
    { 0xc80ecc8d, 33 },
    { 0, 0 },
    { 0x6c6ecc8f, 61 },
    { 0x43fecc8f, 61 },
    { 0, 0 },
    { 0xb0ecc92, 61 },
    { 0xbc5ecc93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36becc9a, 95 },
    { 0, 0 },
    { 0xf1decc9c, 90 },
    { 0x767ecc9d, 100 },
    { 0, 0 },
    { 0xc1cecc9f, 61 },
    { 0x125ecca0, 90 },
    { 0x5ffecca1, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05ecca8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfeccad, 43 },
    { 0x229eccae, 61 },
    { 0x59feccae, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beccb2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94eccb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5eccb9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0beccbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42feccc2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aceccc6, 76 },
    { 0, 0 },
    { 0xaaeccc8, 46 },
    { 0x21ceccc9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4aecccd, 90 },
    { 0x45deccce, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ceccd2, 88 },
    { 0xfbfeccd3, 43 },
    { 0x129eccd2, 90 },
    { 0x23eccd4, 34 },
    { 0xdcdeccd6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ceccd9, 91 },
    { 0x4f2eccda, 33 },
    { 0xd9beccda, 65 },
    { 0, 0 },
    { 0xa63eccdd, 61 },
    { 0, 0 },
    { 0x3edeccdf, 90 },
    { 0x4ddecce0, 61 },
    { 0x1acecce1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0ecce7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1eccec, 68 },
    { 0xd42ecced, 76 },
    { 0x571eccee, 52 },
    { 0x426eccec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6eccf2, 23 },
    { 0x2cfeccf3, 90 },
    { 0xc95eccf4, 98 },
    { 0x78eccf4, 90 },
    { 0xb16eccf3, 61 },
    { 0, 0 },
    { 0xe6eccf8, 33 },
    { 0, 0 },
    { 0xb52eccfa, 33 },
    { 0xfe0eccfb, 76 },
    { 0x268eccfb, 95 },
    { 0x59beccfd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20ecd03, 91 },
    { 0xe60ecd04, 33 },
    { 0, 0 },
    { 0xdc2ecd06, 46 },
    { 0x765ecd07, 23 },
    { 0x4b6ecd07, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x817ecd0b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ecd14, 33 },
    { 0x741ecd15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x539ecd1d, 98 },
    { 0x7c3ecd1e, 88 },
    { 0xf66ecd1f, 17 },
    { 0xd99ecd1e, 55 },
    { 0x36decd21, 93 },
    { 0xe57ecd1d, 34 },
    { 0xbcbecd23, 33 },
    { 0xc96ecd21, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2becd2a, 33 },
    { 0x321ecd2b, 61 },
    { 0x13cecd2b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9ecd2f, 90 },
    { 0x19eecd30, 95 },
    { 0x2efecd31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7ecd36, 17 },
    { 0, 0 },
    { 0x8b5ecd38, 33 },
    { 0x766ecd39, 69 },
    { 0xed3ecd3a, 33 },
    { 0xca3ecd3a, 55 },
    { 0x229ecd39, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12ecd41, 77 },
    { 0, 0 },
    { 0xb99ecd43, 95 },
    { 0x5c2ecd44, 99 },
    { 0, 0 },
    { 0x8f9ecd46, 52 },
    { 0, 0 },
    { 0x5dcecd48, 33 },
    { 0xea4ecd49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eecd4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1ecd55, 61 },
    { 0x91decd56, 33 },
    { 0xf5ecd57, 98 },
    { 0x8bdecd58, 76 },
    { 0, 0 },
    { 0xff3ecd5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01ecd63, 90 },
    { 0, 0 },
    { 0xfcecd65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x526ecd6a, 76 },
    { 0x847ecd6b, 46 },
    { 0x595ecd6c, 23 },
    { 0xaceecd6d, 98 },
    { 0x329ecd6e, 90 },
    { 0x405ecd6f, 99 },
    { 0, 0 },
    { 0xb7eecd71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854ecd78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x120ecd7e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7eecd84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x140ecd87, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x195ecd8d, 55 },
    { 0x32ecd8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaecd99, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaecd9d, 78 },
    { 0xe3ecd9e, 95 },
    { 0, 0 },
    { 0x368ecda0, 33 },
    { 0xd26ecda1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4decda6, 61 },
    { 0, 0 },
    { 0x683ecda8, 78 },
    { 0x50ecda8, 43 },
    { 0xdacecdaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaecdae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7ecdb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aecdb5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe13ecdb8, 61 },
    { 0xda3ecdb9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcecdbd, 55 },
    { 0xaebecdbd, 40 },
    { 0xccdecdbd, 90 },
    { 0xeaaecdc0, 90 },
    { 0x4bdecdc0, 76 },
    { 0xc1ecdc0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ceecdc6, 87 },
    { 0xe69ecdc7, 61 },
    { 0xa30ecdc7, 84 },
    { 0xb6decdc9, 43 },
    { 0x2c0ecdca, 87 },
    { 0xe11ecdcb, 65 },
    { 0x82becdcb, 23 },
    { 0, 0 },
    { 0x441ecdce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0fecdd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cecdde, 61 },
    { 0, 0 },
    { 0x646ecde0, 90 },
    { 0x9c7ecde1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8ecde7, 93 },
    { 0xcf6ecde8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x870ecdeb, 90 },
    { 0, 0 },
    { 0x621ecded, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ecdf2, 68 },
    { 0xd52ecdf3, 75 },
    { 0, 0 },
    { 0x715ecdf5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4ece02, 95 },
    { 0, 0 },
    { 0x1bdece04, 43 },
    { 0xa02ece05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaece08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565ece0e, 98 },
    { 0xb90ece0e, 23 },
    { 0x7a8ece0e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eece13, 98 },
    { 0xc69ece14, 93 },
    { 0x6d5ece14, 98 },
    { 0xce2ece16, 99 },
    { 0x91fece17, 90 },
    { 0xef9ece17, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7ece1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaece25, 61 },
    { 0xc5aece25, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efece2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x794ece31, 84 },
    { 0x473ece32, 33 },
    { 0, 0 },
    { 0xe25ece34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaece37, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9cece3a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475ece3f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x97eece42, 33 },
    { 0, 0 },
    { 0x99ece44, 55 },
    { 0x8afece45, 33 },
    { 0xd1cece46, 17 },
    { 0xccfece47, 65 },
    { 0x341ece45, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3ece4e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7ece52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eece5a, 90 },
    { 0xb49ece5b, 33 },
    { 0x847ece5c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dece61, 46 },
    { 0x2f9ece61, 33 },
    { 0, 0 },
    { 0xce9ece64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cece68, 65 },
    { 0xd5ece69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dece6d, 90 },
    { 0, 0 },
    { 0x797ece6f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8aece72, 77 },
    { 0xb96ece72, 55 },
    { 0x991ece72, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ece79, 93 },
    { 0, 0 },
    { 0xd28ece7b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfece82, 87 },
    { 0, 0 },
    { 0x8d7ece84, 95 },
    { 0x329ece84, 98 },
    { 0x67eece84, 3 },
    { 0x68aece87, 23 },
    { 0, 0 },
    { 0x855ece89, 90 },
    { 0x35bece89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ece90, 91 },
    { 0x3fbece91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41ece94, 66 },
    { 0x2dfece95, 84 },
    { 0, 0 },
    { 0xd72ece97, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eeece9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70ecea0, 34 },
    { 0xcefecea0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3eceae, 17 },
    { 0, 0 },
    { 0x3b5eceb0, 93 },
    { 0x7b2eceb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbeceb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aeceb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678ecebb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2ecec4, 75 },
    { 0xa40ecec5, 55 },
    { 0x568ecec6, 33 },
    { 0x113ecec7, 23 },
    { 0x86ecec7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528eced2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738eced8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cecedc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23ecee4, 34 },
    { 0, 0 },
    { 0x52aecee6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x121ecee9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517eceed, 84 },
    { 0xa49eceee, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ecef6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cecef9, 55 },
    { 0xc88ecefa, 93 },
    { 0, 0 },
    { 0x458ecefc, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x483eceff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ecf08, 55 },
    { 0x623ecf08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ecf12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73eecf16, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6ecf19, 65 },
    { 0x2c9ecf1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16decf20, 78 },
    { 0x957ecf20, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbecf24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1ecf2a, 40 },
    { 0x196ecf2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42becf2f, 61 },
    { 0x16fecf2f, 95 },
    { 0xb7becf31, 40 },
    { 0xc31ecf32, 61 },
    { 0xf18ecf31, 95 },
    { 0xf25ecf34, 93 },
    { 0x3d1ecf34, 40 },
    { 0, 0 },
    { 0xe7ecf37, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ecf3c, 69 },
    { 0xfa9ecf3c, 98 },
    { 0, 0 },
    { 0x3c3ecf3f, 85 },
    { 0xea6ecf40, 98 },
    { 0xcbbecf41, 95 },
    { 0xfe2ecf42, 23 },
    { 0x140ecf43, 33 },
    { 0xe70ecf44, 52 },
    { 0, 0 },
    { 0x80ecf46, 34 },
    { 0xbd0ecf47, 46 },
    { 0x528ecf47, 76 },
    { 0, 0 },
    { 0x7eeecf4a, 55 },
    { 0, 0 },
    { 0xdacecf4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8ecf4f, 23 },
    { 0xdc2ecf50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65ecf54, 52 },
    { 0, 0 },
    { 0xc1aecf56, 98 },
    { 0x9ccecf57, 33 },
    { 0x1c3ecf56, 98 },
    { 0x582ecf59, 46 },
    { 0, 0 },
    { 0xd93ecf5b, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2ecf5f, 45 },
    { 0xbb4ecf60, 91 },
    { 0x5ececf61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ecf66, 98 },
    { 0x110ecf67, 66 },
    { 0x10decf68, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7ecf6b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e5ecf6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd93ecf71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaecf74, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3ecf77, 61 },
    { 0x87becf77, 95 },
    { 0x5e7ecf79, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27ecf85, 96 },
    { 0, 0 },
    { 0x802ecf87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ecf8a, 23 },
    { 0x2c5ecf8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03ecf8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800ecf95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6ecfa0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x928ecfa4, 99 },
    { 0x2f0ecfa5, 93 },
    { 0, 0 },
    { 0x984ecfa7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edecfad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0ecfb3, 55 },
    { 0xdbbecfb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fecfb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15ecfbb, 95 },
    { 0, 0 },
    { 0xb8aecfbd, 95 },
    { 0xbd6ecfbd, 61 },
    { 0, 0 },
    { 0x8d2ecfc0, 84 },
    { 0, 0 },
    { 0xc5becfc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1ecfc5, 43 },
    { 0, 0 },
    { 0x213ecfc7, 93 },
    { 0, 0 },
    { 0x114ecfc9, 33 },
    { 0, 0 },
    { 0x18aecfcb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989ecfd4, 98 },
    { 0xcbaecfd4, 95 },
    { 0x230ecfd6, 95 },
    { 0, 0 },
    { 0x851ecfd8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53decfdd, 55 },
    { 0xb11ecfde, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6fecfe2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5faecfe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63ecfe9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635ecfef, 95 },
    { 0x75ecff0, 76 },
    { 0, 0 },
    { 0x30eecff2, 78 },
    { 0xfeaecff3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5ecff6, 34 },
    { 0x7eeecff7, 98 },
    { 0xcefecff7, 76 },
    { 0x30becff9, 33 },
    { 0xf1cecff6, 66 },
    { 0, 0 },
    { 0x66eecffc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0ed002, 75 },
    { 0, 0 },
    { 0x95eed004, 90 },
    { 0x9e2ed005, 23 },
    { 0xf32ed006, 76 },
    { 0xba0ed007, 52 },
    { 0xdc0ed008, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0ed00b, 77 },
    { 0x9daed00c, 67 },
    { 0, 0 },
    { 0x45bed00e, 90 },
    { 0x8eed00f, 65 },
    { 0x713ed010, 90 },
    { 0x29ed011, 55 },
    { 0x452ed00f, 33 },
    { 0x367ed013, 91 },
    { 0x4aced011, 33 },
    { 0x32fed015, 55 },
    { 0x58bed016, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x648ed01e, 43 },
    { 0, 0 },
    { 0xce1ed020, 61 },
    { 0x250ed021, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fed024, 83 },
    { 0x11eed025, 98 },
    { 0x3d9ed026, 100 },
    { 0x6c1ed027, 33 },
    { 0, 0 },
    { 0xceaed029, 90 },
    { 0, 0 },
    { 0x4cced02b, 91 },
    { 0xce8ed02b, 99 },
    { 0xdced02d, 95 },
    { 0xf5ced02e, 93 },
    { 0x824ed02c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bed032, 33 },
    { 0x898ed033, 46 },
    { 0x776ed033, 95 },
    { 0, 0 },
    { 0xcb3ed036, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ded039, 98 },
    { 0x2e1ed03a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6ed03d, 61 },
    { 0x7bced03e, 87 },
    { 0, 0 },
    { 0xdd7ed040, 81 },
    { 0xfb7ed040, 65 },
    { 0xf67ed042, 23 },
    { 0xd6ded043, 17 },
    { 0x921ed044, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02ed04a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fed051, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x239ed054, 85 },
    { 0x424ed055, 95 },
    { 0x335ed056, 52 },
    { 0, 0 },
    { 0x1d0ed058, 98 },
    { 0, 0 },
    { 0xcdced05a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7fed05d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeed062, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x10aed065, 95 },
    { 0xb4ced066, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefed069, 90 },
    { 0xf1fed069, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00ed06d, 98 },
    { 0x7d3ed06e, 85 },
    { 0x4f6ed06f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6ed075, 87 },
    { 0x1a1ed076, 23 },
    { 0xc3bed077, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787ed07f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ed084, 76 },
    { 0x846ed085, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567ed089, 76 },
    { 0x319ed08a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ed08d, 33 },
    { 0x63ded08e, 46 },
    { 0, 0 },
    { 0x4feed090, 96 },
    { 0, 0 },
    { 0x3fded092, 23 },
    { 0, 0 },
    { 0xed2ed094, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12ed097, 33 },
    { 0x695ed098, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29ed09c, 23 },
    { 0xf93ed09d, 75 },
    { 0x45ced09c, 33 },
    { 0x2f1ed09d, 43 },
    { 0, 0 },
    { 0xbeed0a1, 40 },
    { 0x494ed0a2, 90 },
    { 0x826ed0a3, 33 },
    { 0x54ced0a4, 52 },
    { 0xac5ed0a5, 72 },
    { 0x3d5ed0a4, 61 },
    { 0xc60ed0a7, 33 },
    { 0x449ed0a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x404ed0ab, 37 },
    { 0xd8ced0ac, 33 },
    { 0, 0 },
    { 0x608ed0ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedced0b2, 76 },
    { 0x9bed0b3, 76 },
    { 0, 0 },
    { 0x1a1ed0b5, 52 },
    { 0x1baed0b5, 55 },
    { 0x7a7ed0b6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31aed0bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69ed0be, 90 },
    { 0xbbbed0bf, 98 },
    { 0x847ed0c0, 78 },
    { 0xe06ed0c1, 90 },
    { 0xf5ded0c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841ed0ca, 95 },
    { 0x87ded0cb, 87 },
    { 0x821ed0cc, 90 },
    { 0x74ced0cc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5ed0d1, 66 },
    { 0x54ced0d2, 98 },
    { 0xbe9ed0d2, 99 },
    { 0xa10ed0d3, 98 },
    { 0x3ded0d2, 23 },
    { 0x617ed0d5, 55 },
    { 0, 0 },
    { 0x74fed0d8, 61 },
    { 0x26bed0d9, 34 },
    { 0, 0 },
    { 0x2e2ed0db, 76 },
    { 0x3d3ed0dc, 76 },
    { 0x261ed0dd, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ed0e4, 61 },
    { 0xe4ced0e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffced0ea, 76 },
    { 0, 0 },
    { 0x920ed0ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19aed0f0, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8ed0f4, 33 },
    { 0xeb4ed0f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20ed0f9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eded0fd, 48 },
    { 0xb35ed0fe, 90 },
    { 0x540ed0fe, 91 },
    { 0x31ded100, 65 },
    { 0x799ed0ff, 36 },
    { 0xd9aed0fd, 90 },
    { 0xae5ed103, 96 },
    { 0x5e8ed0ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31bed10c, 23 },
    { 0, 0 },
    { 0x6c6ed10e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8ed111, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61ed117, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ed11b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bced11e, 98 },
    { 0x2e0ed11f, 87 },
    { 0xffed120, 23 },
    { 0x2b8ed11f, 33 },
    { 0xe95ed122, 52 },
    { 0, 0 },
    { 0xc6eed124, 61 },
    { 0, 0 },
    { 0x64bed126, 87 },
    { 0x20ced127, 87 },
    { 0, 0 },
    { 0x7aaed129, 90 },
    { 0, 0 },
    { 0xcbded12b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1eed12e, 90 },
    { 0x5b9ed12f, 90 },
    { 0xf14ed12e, 61 },
    { 0x437ed131, 95 },
    { 0xde4ed130, 85 },
    { 0x1d4ed133, 95 },
    { 0xae4ed133, 91 },
    { 0xeabed135, 52 },
    { 0x41fed135, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ed139, 95 },
    { 0x753ed139, 90 },
    { 0xdb7ed13b, 95 },
    { 0, 0 },
    { 0x6ed13d, 101 },
    { 0, 0 },
    { 0x7cfed13f, 95 },
    { 0xffced13f, 96 },
    { 0, 0 },
    { 0xc05ed142, 93 },
    { 0xa69ed143, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510ed14c, 98 },
    { 0x6fbed14c, 90 },
    { 0, 0 },
    { 0xdebed14f, 37 },
    { 0, 0 },
    { 0xc74ed151, 95 },
    { 0, 0 },
    { 0x7fced153, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed157, 34 },
    { 0x289ed157, 33 },
    { 0x963ed157, 90 },
    { 0, 0 },
    { 0x530ed15b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555ed15f, 99 },
    { 0x214ed160, 33 },
    { 0, 0 },
    { 0x60ded162, 78 },
    { 0x109ed163, 17 },
    { 0x827ed164, 34 },
    { 0, 0 },
    { 0xb9fed166, 78 },
    { 0xcf9ed166, 43 },
    { 0, 0 },
    { 0x381ed169, 23 },
    { 0, 0 },
    { 0x3ffed16b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ded16e, 61 },
    { 0xb39ed16f, 76 },
    { 0x86ed170, 61 },
    { 0, 0 },
    { 0xf51ed172, 93 },
    { 0, 0 },
    { 0xc9aed174, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfded178, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x633ed17b, 81 },
    { 0x8ffed17b, 61 },
    { 0x75bed17b, 95 },
    { 0x3f0ed17c, 55 },
    { 0x54ded17f, 99 },
    { 0xf81ed180, 85 },
    { 0x91eed180, 33 },
    { 0x8f3ed17b, 95 },
    { 0xa9bed183, 95 },
    { 0, 0 },
    { 0x38aed185, 80 },
    { 0x34aed186, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4fed192, 33 },
    { 0, 0 },
    { 0x63ded194, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eced198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eced19b, 78 },
    { 0xbbded19b, 90 },
    { 0x34bed19c, 33 },
    { 0xe2eed19d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0ed1a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1ed1a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe58ed1a9, 98 },
    { 0xb3eed1aa, 23 },
    { 0xa02ed1ab, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dded1b7, 88 },
    { 0xb98ed1b8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2ed1bd, 33 },
    { 0, 0 },
    { 0x114ed1bf, 33 },
    { 0x278ed1c0, 87 },
    { 0x5f8ed1c0, 61 },
    { 0x2eded1bf, 61 },
    { 0x257ed1c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfed1c6, 90 },
    { 0xa88ed1c6, 52 },
    { 0x1fced1c6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63ed1cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9fed1cf, 95 },
    { 0, 0 },
    { 0x9e7ed1d1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35ed1d6, 78 },
    { 0xf7eed1d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x425ed1da, 90 },
    { 0x742ed1db, 98 },
    { 0, 0 },
    { 0xc5fed1dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbed1e0, 95 },
    { 0xeeaed1e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09ed1e4, 90 },
    { 0xc5bed1e4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fed1ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20ed1f5, 78 },
    { 0x497ed1f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bed1fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ed203, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78ed213, 99 },
    { 0x7bced213, 76 },
    { 0x4c5ed215, 76 },
    { 0, 0 },
    { 0x15fed217, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ed21a, 46 },
    { 0x471ed21b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5ed21e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61aed222, 78 },
    { 0xd1ed223, 95 },
    { 0, 0 },
    { 0x9e3ed225, 34 },
    { 0x7abed226, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeed22a, 46 },
    { 0, 0 },
    { 0xabded22c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d9ed22f, 33 },
    { 0, 0 },
    { 0x4c7ed231, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fed234, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afed239, 87 },
    { 0xf93ed239, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ed23d, 61 },
    { 0xf42ed23d, 95 },
    { 0, 0 },
    { 0x748ed240, 43 },
    { 0xe5fed240, 23 },
    { 0x845ed242, 91 },
    { 0x801ed243, 95 },
    { 0xee6ed244, 78 },
    { 0, 0 },
    { 0x916ed246, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ed24b, 33 },
    { 0xc3aed24c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ced251, 61 },
    { 0x6a7ed252, 61 },
    { 0xee5ed253, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0ed256, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97ed25d, 33 },
    { 0, 0 },
    { 0xf85ed25f, 90 },
    { 0, 0 },
    { 0x3bed261, 33 },
    { 0, 0 },
    { 0x20bed263, 87 },
    { 0, 0 },
    { 0x431ed265, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bed26b, 95 },
    { 0x9b6ed26c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7ed271, 61 },
    { 0x44ed271, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaed276, 23 },
    { 0, 0 },
    { 0xf92ed278, 95 },
    { 0, 0 },
    { 0xf1ed27a, 33 },
    { 0xf9aed27a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3ed27e, 90 },
    { 0xd4aed27f, 61 },
    { 0x820ed27f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bed28b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0ed292, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3ed295, 80 },
    { 0x3f0ed296, 98 },
    { 0, 0 },
    { 0xa33ed298, 61 },
    { 0, 0 },
    { 0xc9ced29a, 91 },
    { 0xc4aed29b, 55 },
    { 0x942ed29c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4ed2a4, 55 },
    { 0x4a2ed2a5, 93 },
    { 0, 0 },
    { 0xd4ed2a7, 90 },
    { 0xb25ed2a7, 84 },
    { 0x4eaed2a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5ed2b5, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74aed2bb, 85 },
    { 0x99aed2bb, 78 },
    { 0xa31ed2bc, 61 },
    { 0xbf0ed2be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37ed2c1, 61 },
    { 0x372ed2c2, 43 },
    { 0xccced2c3, 33 },
    { 0x5d0ed2c3, 34 },
    { 0xbc7ed2c5, 23 },
    { 0x7a4ed2c6, 72 },
    { 0x7b4ed2c7, 76 },
    { 0xf4fed2c6, 95 },
    { 0xc9ded2c4, 43 },
    { 0, 0 },
    { 0x2c5ed2cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbed2cf, 76 },
    { 0xb8eed2d0, 76 },
    { 0xb04ed2d0, 33 },
    { 0x18eed2cf, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bed2d5, 66 },
    { 0x992ed2d6, 98 },
    { 0xf23ed2d7, 23 },
    { 0, 0 },
    { 0xb6aed2d9, 66 },
    { 0x28bed2da, 90 },
    { 0xbabed2db, 33 },
    { 0x6f3ed2db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabced2df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x717ed2e2, 84 },
    { 0xb1eed2e2, 33 },
    { 0xee6ed2e4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30ed2e7, 33 },
    { 0x303ed2e7, 33 },
    { 0x3c8ed2e7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ed2ec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabed2f2, 98 },
    { 0, 0 },
    { 0xffaed2f4, 33 },
    { 0x514ed2f4, 90 },
    { 0, 0 },
    { 0xbb8ed2f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x469ed2fb, 90 },
    { 0x42aed2fc, 100 },
    { 0xfaeed2fd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38eed303, 17 },
    { 0, 0 },
    { 0xd6aed305, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3ed309, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04ed30c, 90 },
    { 0, 0 },
    { 0x14bed30e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ded314, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ed319, 33 },
    { 0x7c8ed31a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ced31f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cfed322, 61 },
    { 0x76ced323, 98 },
    { 0x3b8ed324, 55 },
    { 0xf62ed323, 33 },
    { 0, 0 },
    { 0xefbed327, 55 },
    { 0x9a1ed328, 76 },
    { 0, 0 },
    { 0x6d6ed32a, 55 },
    { 0x5d4ed32a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbed332, 100 },
    { 0x83fed332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf85ed33a, 98 },
    { 0x22ced33a, 33 },
    { 0, 0 },
    { 0x59ed33d, 52 },
    { 0, 0 },
    { 0x594ed33f, 84 },
    { 0xac6ed340, 33 },
    { 0x4d6ed341, 76 },
    { 0x60ed342, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ded34a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38ed34d, 33 },
    { 0xfceed34e, 23 },
    { 0xcbed34f, 23 },
    { 0xc3fed34f, 99 },
    { 0x790ed350, 23 },
    { 0x9ebed34d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ced357, 41 },
    { 0x7deed358, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ed35c, 40 },
    { 0, 0 },
    { 0xcc7ed35e, 61 },
    { 0x958ed35e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ed362, 43 },
    { 0, 0 },
    { 0x57ded364, 100 },
    { 0x97ced364, 98 },
    { 0, 0 },
    { 0x497ed367, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x66eed36a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ed36f, 43 },
    { 0x90ced370, 33 },
    { 0xa79ed371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0ed374, 61 },
    { 0, 0 },
    { 0xb5aed376, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2ed379, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbed37c, 98 },
    { 0x4a3ed37d, 67 },
    { 0, 0 },
    { 0xb1bed37f, 34 },
    { 0xa40ed37f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xaded383, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ced386, 90 },
    { 0x8b2ed386, 61 },
    { 0x52bed388, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ed38f, 33 },
    { 0, 0 },
    { 0xc25ed391, 61 },
    { 0, 0 },
    { 0xf68ed393, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x181ed396, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6ed39e, 93 },
    { 0xb38ed39f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659ed3a4, 93 },
    { 0x107ed3a4, 98 },
    { 0x7ed3a6, 23 },
    { 0xfb2ed3a7, 33 },
    { 0x8c5ed3a8, 76 },
    { 0x47aed3a9, 61 },
    { 0x44eed3a7, 80 },
    { 0xc31ed3a8, 91 },
    { 0x65ded3a9, 98 },
    { 0x4d3ed3a6, 93 },
    { 0x880ed3ad, 23 },
    { 0xdfaed3ac, 75 },
    { 0x35aed3a4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaaed3b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfed3be, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfed3c1, 98 },
    { 0x28bed3c2, 52 },
    { 0x541ed3c3, 90 },
    { 0x850ed3c2, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaded3ca, 33 },
    { 0xfc0ed3ca, 90 },
    { 0xfe2ed3cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bed3cf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x916ed3d2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65ed3d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fed3db, 61 },
    { 0x7f9ed3db, 90 },
    { 0xc51ed3dd, 100 },
    { 0, 0 },
    { 0x8f1ed3df, 99 },
    { 0, 0 },
    { 0x160ed3e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x197ed3e4, 23 },
    { 0, 0 },
    { 0xfd8ed3e6, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbed3ed, 90 },
    { 0x403ed3ee, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbced3f4, 85 },
    { 0x205ed3f5, 98 },
    { 0x6e0ed3f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ded400, 81 },
    { 0xd2ded401, 98 },
    { 0xc66ed402, 90 },
    { 0, 0 },
    { 0xf5ed404, 55 },
    { 0x7b2ed405, 84 },
    { 0xc50ed405, 76 },
    { 0x184ed407, 43 },
    { 0x57ded406, 95 },
    { 0x9dbed409, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ced40d, 76 },
    { 0, 0 },
    { 0xad7ed40f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153ed413, 84 },
    { 0x23ed413, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2fed419, 55 },
    { 0x3c9ed419, 90 },
    { 0, 0 },
    { 0xc59ed41c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296ed422, 93 },
    { 0x97ced422, 98 },
    { 0, 0 },
    { 0xa0ded425, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f0ed429, 43 },
    { 0xc81ed429, 95 },
    { 0xfe8ed42b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507ed431, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43fed435, 46 },
    { 0x54aed436, 55 },
    { 0xdbed437, 95 },
    { 0x728ed438, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ded445, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9ed449, 46 },
    { 0xdc9ed44a, 98 },
    { 0xeeded449, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482ed454, 23 },
    { 0xefaed454, 90 },
    { 0x15ced454, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ded45a, 61 },
    { 0, 0 },
    { 0xa1ed45c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5ed465, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49ed470, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1ed475, 82 },
    { 0x50ded476, 76 },
    { 0x2bbed477, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889ed47d, 52 },
    { 0xf62ed47d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01ed482, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0aed485, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1ed488, 95 },
    { 0x45fed488, 46 },
    { 0x859ed48a, 33 },
    { 0xe2ded48b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabed48e, 98 },
    { 0x967ed48f, 90 },
    { 0xe9ced490, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ed494, 95 },
    { 0x138ed494, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2ed49a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976ed4a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ed4a8, 99 },
    { 0, 0 },
    { 0xf74ed4aa, 61 },
    { 0xd2eed4aa, 85 },
    { 0x133ed4ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96aed4b0, 33 },
    { 0x5aeed4b1, 33 },
    { 0x654ed4b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847ed4bb, 95 },
    { 0, 0 },
    { 0x94eed4bd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x714ed4c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ed4c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494ed4cb, 98 },
    { 0x504ed4cb, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eced4d0, 85 },
    { 0x76eed4d1, 61 },
    { 0xe7aed4d2, 95 },
    { 0x34bed4d2, 67 },
    { 0x2cbed4d2, 61 },
    { 0x90ded4d5, 90 },
    { 0xf41ed4d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49fed4dc, 32 },
    { 0x682ed4dd, 55 },
    { 0x630ed4de, 17 },
    { 0x730ed4df, 83 },
    { 0, 0 },
    { 0x1cfed4e1, 90 },
    { 0xdafed4e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x124ed4e5, 33 },
    { 0x60eed4e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216ed4eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55ed4ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ed4f1, 98 },
    { 0xd53ed4f2, 98 },
    { 0x754ed4f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4ed4f6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20ed4fd, 98 },
    { 0x328ed4fe, 90 },
    { 0x62ed4ff, 33 },
    { 0xf2bed4ff, 43 },
    { 0x5b2ed4ff, 95 },
    { 0x719ed4ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4aed507, 90 },
    { 0, 0 },
    { 0x6c7ed509, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982ed50c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4ed510, 33 },
    { 0x36ed511, 33 },
    { 0x8c6ed512, 46 },
    { 0x2cbed511, 95 },
    { 0, 0 },
    { 0xfdeed515, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x706ed518, 33 },
    { 0x689ed519, 95 },
    { 0xdf4ed51a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722ed51e, 77 },
    { 0x92fed51f, 61 },
    { 0, 0 },
    { 0x542ed521, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9ed526, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dded531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833ed537, 23 },
    { 0, 0 },
    { 0x581ed539, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aed53d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x735ed540, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x927ed548, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eed54c, 78 },
    { 0xd1eed54d, 33 },
    { 0xb4ded54e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bed552, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eed55a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ed55d, 33 },
    { 0x7e7ed55e, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ced561, 98 },
    { 0xc1ced561, 33 },
    { 0x1abed562, 78 },
    { 0x9b5ed561, 55 },
    { 0x36ded563, 95 },
    { 0x1d5ed562, 76 },
    { 0x8baed567, 88 },
    { 0x1aeed563, 33 },
    { 0x54aed569, 95 },
    { 0x6fced56a, 90 },
    { 0x769ed561, 91 },
    { 0xde0ed56c, 23 },
    { 0x4dbed56c, 87 },
    { 0x55eed56c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6ed576, 34 },
    { 0, 0 },
    { 0x3c1ed578, 91 },
    { 0x562ed578, 99 },
    { 0xb29ed57a, 76 },
    { 0x352ed57a, 78 },
    { 0xbc5ed57c, 98 },
    { 0xf75ed57d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44ed581, 55 },
    { 0x239ed581, 90 },
    { 0x361ed583, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x761ed586, 93 },
    { 0x107ed586, 90 },
    { 0x520ed588, 92 },
    { 0x926ed589, 66 },
    { 0xee4ed58a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88ed58d, 95 },
    { 0x782ed58e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734ed593, 66 },
    { 0, 0 },
    { 0xd78ed595, 65 },
    { 0, 0 },
    { 0x312ed597, 52 },
    { 0, 0 },
    { 0x512ed599, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4ed5a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ed5a6, 84 },
    { 0xcaaed5a6, 76 },
    { 0x6eed5a8, 98 },
    { 0x465ed5a8, 78 },
    { 0x141ed5a8, 33 },
    { 0x5d3ed5aa, 61 },
    { 0xe93ed5a9, 23 },
    { 0xfe0ed5ad, 33 },
    { 0xb86ed5ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314ed5b2, 84 },
    { 0x351ed5b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923ed5b8, 90 },
    { 0, 0 },
    { 0xeebed5ba, 78 },
    { 0x322ed5ba, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ded5be, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67ed5c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541ed5c6, 69 },
    { 0xdaed5c6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ed5ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ced5ce, 95 },
    { 0xe9ced5cf, 46 },
    { 0x1f7ed5ce, 91 },
    { 0x3b2ed5d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x647ed5d4, 61 },
    { 0x72ced5d5, 90 },
    { 0, 0 },
    { 0x677ed5d7, 61 },
    { 0, 0 },
    { 0xa70ed5d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70ed5de, 61 },
    { 0xf5bed5df, 91 },
    { 0, 0 },
    { 0x619ed5e1, 66 },
    { 0xc25ed5e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391ed5ee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fed5f3, 34 },
    { 0, 0 },
    { 0x39ded5f5, 88 },
    { 0x7a6ed5f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52ed604, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3eed607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ed60d, 34 },
    { 0xd64ed60d, 76 },
    { 0, 0 },
    { 0x160ed610, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fed614, 33 },
    { 0x843ed615, 76 },
    { 0x223ed614, 90 },
    { 0x4aaed617, 96 },
    { 0x294ed618, 33 },
    { 0xb1eed616, 61 },
    { 0, 0 },
    { 0xf54ed61b, 91 },
    { 0x4a8ed61c, 61 },
    { 0x20ed61c, 34 },
    { 0xa27ed61b, 33 },
    { 0xbcded61f, 46 },
    { 0x9ded61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x764ed623, 55 },
    { 0, 0 },
    { 0x3c0ed625, 84 },
    { 0, 0 },
    { 0xb3bed627, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9ed62b, 23 },
    { 0x2e6ed62b, 33 },
    { 0xf59ed62c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ed633, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ceed636, 43 },
    { 0x965ed637, 46 },
    { 0, 0 },
    { 0xcfaed639, 43 },
    { 0, 0 },
    { 0x113ed63b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13ed63e, 100 },
    { 0xe21ed63e, 76 },
    { 0xbc5ed63f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fed643, 98 },
    { 0, 0 },
    { 0x60ded645, 76 },
    { 0xe59ed646, 61 },
    { 0xb0aed647, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580ed64f, 33 },
    { 0xfd5ed650, 88 },
    { 0, 0 },
    { 0x910ed652, 33 },
    { 0xb17ed652, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9ed657, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x657ed65a, 61 },
    { 0, 0 },
    { 0xb39ed65c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2ed660, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6ed665, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ed668, 67 },
    { 0, 0 },
    { 0x40eed66a, 33 },
    { 0, 0 },
    { 0xf21ed66c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bed674, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9ed677, 99 },
    { 0xd10ed678, 35 },
    { 0x773ed679, 99 },
    { 0x57ded679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0aed67d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411ed686, 85 },
    { 0x47aed686, 90 },
    { 0xeabed687, 36 },
    { 0xfeeed687, 90 },
    { 0x6d8ed68a, 46 },
    { 0x3e3ed68b, 90 },
    { 0x1e2ed68c, 84 },
    { 0x3d0ed68c, 33 },
    { 0xadfed68e, 95 },
    { 0xd7fed68e, 46 },
    { 0x52fed690, 95 },
    { 0xaaaed691, 52 },
    { 0x5ffed68b, 87 },
    { 0xef9ed68e, 94 },
    { 0x1beed68d, 34 },
    { 0x602ed695, 76 },
    { 0x651ed696, 33 },
    { 0xd8fed695, 93 },
    { 0x569ed691, 90 },
    { 0xf90ed694, 90 },
    { 0xb01ed69a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ed69d, 46 },
    { 0x21aed69e, 55 },
    { 0xf48ed69d, 75 },
    { 0, 0 },
    { 0xf2fed6a1, 61 },
    { 0x288ed6a2, 33 },
    { 0xbaaed6a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffed6a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ed6ab, 33 },
    { 0, 0 },
    { 0xf23ed6ad, 92 },
    { 0xca3ed6ae, 76 },
    { 0x5f2ed6af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21ed6b5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3ed6b9, 33 },
    { 0x8bfed6b9, 34 },
    { 0, 0 },
    { 0x9c5ed6bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51ed6c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eeed6cb, 90 },
    { 0, 0 },
    { 0xfe3ed6cd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22ed6d7, 95 },
    { 0, 0 },
    { 0x2eded6d9, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912ed6de, 95 },
    { 0x68ded6df, 90 },
    { 0x2eed6de, 61 },
    { 0x3b6ed6e1, 33 },
    { 0x71ed6e1, 4 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ced6e8, 33 },
    { 0x7e4ed6e9, 52 },
    { 0, 0 },
    { 0xe7ded6eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ed6f0, 17 },
    { 0x5f8ed6f0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbed6f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x560ed6fb, 33 },
    { 0, 0 },
    { 0x1bfed6fd, 34 },
    { 0xb14ed6fe, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bded701, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124ed706, 90 },
    { 0, 0 },
    { 0x769ed708, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ed70c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x770ed70f, 34 },
    { 0xd9aed70f, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x767ed713, 76 },
    { 0x651ed714, 98 },
    { 0x97eed715, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6ed718, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331ed71e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ed729, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5fed72e, 33 },
    { 0x35ced72f, 88 },
    { 0, 0 },
    { 0x266ed731, 43 },
    { 0xd0ced732, 55 },
    { 0, 0 },
    { 0x489ed734, 100 },
    { 0xa76ed735, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ded73a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ed740, 90 },
    { 0x772ed740, 67 },
    { 0x452ed742, 48 },
    { 0x252ed742, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8ed749, 33 },
    { 0xdc7ed749, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24ed74e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2eed753, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24ed759, 98 },
    { 0xcc3ed759, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x685ed75d, 87 },
    { 0x18eed75e, 43 },
    { 0x96ced75e, 33 },
    { 0x9dded75f, 76 },
    { 0xc6bed761, 90 },
    { 0x3ed762, 90 },
    { 0xb3fed760, 23 },
    { 0x517ed764, 90 },
    { 0x435ed765, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8eed76e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x702ed772, 98 },
    { 0x48fed772, 40 },
    { 0x47eed774, 98 },
    { 0x8bed773, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ed778, 33 },
    { 0, 0 },
    { 0x79fed77a, 33 },
    { 0, 0 },
    { 0x146ed77c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ed780, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9ed785, 33 },
    { 0x57ced786, 34 },
    { 0x1dfed787, 91 },
    { 0x29fed787, 76 },
    { 0, 0 },
    { 0x6dbed78a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ded790, 33 },
    { 0x590ed791, 61 },
    { 0xda0ed792, 94 },
    { 0x326ed790, 33 },
    { 0, 0 },
    { 0x4ceed795, 52 },
    { 0xb3ced796, 76 },
    { 0, 0 },
    { 0xe95ed798, 80 },
    { 0x1afed799, 23 },
    { 0xfbed79a, 95 },
    { 0xba9ed79b, 23 },
    { 0x881ed79c, 99 },
    { 0, 0 },
    { 0x4aed79e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0ed7a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbed7a8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebed7af, 55 },
    { 0, 0 },
    { 0x189ed7b1, 43 },
    { 0xdb8ed7b2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbed7b6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x857ed7ba, 23 },
    { 0xb15ed7bb, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x824ed7c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbed7c5, 43 },
    { 0x1f4ed7c5, 98 },
    { 0x9efed7c7, 96 },
    { 0, 0 },
    { 0x7bced7c9, 33 },
    { 0xf1fed7ca, 23 },
    { 0xb73ed7cb, 90 },
    { 0x307ed7ca, 100 },
    { 0x656ed7cd, 90 },
    { 0xd47ed7ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ed7d4, 81 },
    { 0, 0 },
    { 0x62ded7d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8ed7e0, 33 },
    { 0xcdeed7e1, 23 },
    { 0x4beed7e2, 99 },
    { 0xa23ed7e3, 61 },
    { 0x23bed7e3, 87 },
    { 0x284ed7e3, 81 },
    { 0x6f8ed7e4, 61 },
    { 0xb53ed7e7, 95 },
    { 0xb7fed7e2, 95 },
    { 0xe72ed7e4, 91 },
    { 0x792ed7ea, 98 },
    { 0x103ed7ea, 43 },
    { 0x3bced7e6, 98 },
    { 0x24eed7e3, 36 },
    { 0x385ed7ee, 95 },
    { 0xf42ed7ef, 95 },
    { 0xbeced7f0, 90 },
    { 0x861ed7f1, 61 },
    { 0x2d6ed7f1, 61 },
    { 0x460ed7f3, 46 },
    { 0xe12ed7f4, 88 },
    { 0xa1aed7f3, 98 },
    { 0, 0 },
    { 0x667ed7f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aced800, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3ed80c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x236ed80f, 90 },
    { 0, 0 },
    { 0xc8aed811, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52aed818, 95 },
    { 0xeed819, 61 },
    { 0x664ed81a, 23 },
    { 0xd1ced81a, 88 },
    { 0xc85ed818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1ed81f, 90 },
    { 0x5abed820, 75 },
    { 0x1dfed821, 95 },
    { 0x317ed821, 95 },
    { 0xd43ed823, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6ed828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6ed82f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bed832, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ed836, 90 },
    { 0xbf4ed836, 34 },
    { 0xe92ed837, 76 },
    { 0xdcaed838, 33 },
    { 0xd22ed83a, 55 },
    { 0x435ed837, 100 },
    { 0xf73ed83c, 43 },
    { 0xbded83c, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ced844, 55 },
    { 0xae9ed845, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4ed84b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ed84e, 33 },
    { 0x17ced84e, 78 },
    { 0, 0 },
    { 0x9ffed851, 33 },
    { 0x549ed851, 23 },
    { 0xa21ed852, 55 },
    { 0, 0 },
    { 0xb15ed855, 61 },
    { 0x492ed856, 90 },
    { 0xb07ed857, 85 },
    { 0, 0 },
    { 0xddfed859, 76 },
    { 0, 0 },
    { 0x422ed85b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6ed869, 65 },
    { 0x5daed86a, 66 },
    { 0, 0 },
    { 0x389ed86c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dced876, 95 },
    { 0xe89ed876, 61 },
    { 0, 0 },
    { 0xbf8ed879, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x826ed880, 95 },
    { 0xacaed880, 33 },
    { 0xf8fed882, 53 },
    { 0x75ced883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x775ed886, 88 },
    { 0, 0 },
    { 0x83ded888, 98 },
    { 0xc93ed888, 95 },
    { 0x89ed88a, 95 },
    { 0x3ffed88a, 23 },
    { 0x91bed889, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6ed88f, 33 },
    { 0xf92ed890, 34 },
    { 0xac4ed891, 34 },
    { 0x400ed890, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bed895, 90 },
    { 0x9a9ed895, 100 },
    { 0x3eaed897, 69 },
    { 0x5dded896, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13ed89f, 90 },
    { 0x1c6ed8a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24fed8a9, 87 },
    { 0x50ded8aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ed8ad, 95 },
    { 0x39fed8ae, 99 },
    { 0, 0 },
    { 0x492ed8b0, 23 },
    { 0x7ceed8b1, 34 },
    { 0xa8aed8b2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401ed8b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6ed8c0, 90 },
    { 0xed9ed8c1, 61 },
    { 0, 0 },
    { 0xc39ed8c3, 85 },
    { 0, 0 },
    { 0xab3ed8c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44ed8c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x627ed8cb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ced8d3, 61 },
    { 0x6a7ed8d4, 58 },
    { 0x8e8ed8d4, 47 },
    { 0, 0 },
    { 0x954ed8d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9deed8da, 23 },
    { 0, 0 },
    { 0x1aded8dc, 91 },
    { 0xfeded8dd, 33 },
    { 0x600ed8dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaeed8e6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ed8ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89ed8ef, 23 },
    { 0xed5ed8f0, 96 },
    { 0, 0 },
    { 0x4bbed8f2, 77 },
    { 0x4cfed8f3, 65 },
    { 0, 0 },
    { 0xa7fed8f5, 100 },
    { 0x326ed8f5, 34 },
    { 0xc83ed8f5, 95 },
    { 0, 0 },
    { 0xd8ed8f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbed8fd, 99 },
    { 0x6e0ed8fe, 90 },
    { 0x2b3ed8ff, 55 },
    { 0x4ded900, 96 },
    { 0xe90ed901, 98 },
    { 0, 0 },
    { 0xa2aed903, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbed908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88ed90e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beed91a, 84 },
    { 0, 0 },
    { 0xfcbed91c, 34 },
    { 0x8c1ed91d, 90 },
    { 0xc8aed91c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bed923, 90 },
    { 0x2e8ed924, 98 },
    { 0x788ed925, 87 },
    { 0xf49ed923, 85 },
    { 0x6d6ed923, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x525ed92a, 66 },
    { 0x5b2ed92b, 61 },
    { 0, 0 },
    { 0xc42ed92d, 66 },
    { 0xc68ed92e, 98 },
    { 0x883ed92f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x972ed932, 90 },
    { 0xb49ed933, 66 },
    { 0, 0 },
    { 0x8cfed935, 98 },
    { 0, 0 },
    { 0xdfeed937, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdded93c, 90 },
    { 0, 0 },
    { 0x8d7ed93e, 33 },
    { 0, 0 },
    { 0x97ded940, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x528ed943, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bed948, 23 },
    { 0, 0 },
    { 0x20fed94a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9ed954, 95 },
    { 0xc3ced955, 55 },
    { 0x55ed956, 23 },
    { 0x343ed955, 23 },
    { 0, 0 },
    { 0x95ed959, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ed95d, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ed960, 90 },
    { 0x1ced960, 90 },
    { 0, 0 },
    { 0x23eed963, 90 },
    { 0, 0 },
    { 0x5b8ed965, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefed96a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4ed975, 23 },
    { 0xcf4ed976, 78 },
    { 0x678ed977, 76 },
    { 0x6a5ed978, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bed97b, 33 },
    { 0, 0 },
    { 0x172ed97d, 76 },
    { 0x763ed97e, 90 },
    { 0x1b7ed97f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60ed982, 90 },
    { 0xe64ed982, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183ed98b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9ed98e, 91 },
    { 0x673ed98f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4ed994, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24ed998, 67 },
    { 0xeaced998, 23 },
    { 0x2caed999, 95 },
    { 0x3c8ed99b, 90 },
    { 0x189ed998, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ed9a4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7ed9ae, 85 },
    { 0x2b1ed9af, 90 },
    { 0xdfded9ae, 95 },
    { 0, 0 },
    { 0xbbed9b2, 100 },
    { 0, 0 },
    { 0x8afed9b4, 68 },
    { 0xce1ed9b5, 98 },
    { 0xaffed9b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3ed9b9, 76 },
    { 0x939ed9ba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49ed9be, 34 },
    { 0x7beed9be, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daed9c2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76ed9c9, 77 },
    { 0x3aded9ca, 76 },
    { 0x659ed9cb, 95 },
    { 0xddded9cc, 67 },
    { 0, 0 },
    { 0xd4bed9ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x333ed9d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc05ed9e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebed9e5, 95 },
    { 0x74ed9e5, 33 },
    { 0xb71ed9e7, 12 },
    { 0x5c1ed9e8, 43 },
    { 0, 0 },
    { 0x304ed9ea, 66 },
    { 0, 0 },
    { 0x661ed9ec, 85 },
    { 0x718ed9ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5aed9f0, 46 },
    { 0xafbed9f0, 33 },
    { 0x1f2ed9f0, 90 },
    { 0x702ed9f3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3ed9f6, 55 },
    { 0, 0 },
    { 0x37eed9f8, 43 },
    { 0, 0 },
    { 0x705ed9fa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8ed9fd, 95 },
    { 0x81fed9fe, 90 },
    { 0xa5fed9ff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x241eda02, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0eda05, 88 },
    { 0, 0 },
    { 0xc2eeda07, 61 },
    { 0, 0 },
    { 0x547eda09, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96feda0d, 100 },
    { 0x526eda0d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3feda12, 33 },
    { 0xa64eda13, 90 },
    { 0x62feda14, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8feda17, 90 },
    { 0xd70eda17, 61 },
    { 0xea6eda18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42eda1c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929eda21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92eda24, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2eda28, 90 },
    { 0, 0 },
    { 0xf23eda2a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beeda2d, 34 },
    { 0, 0 },
    { 0x939eda2f, 23 },
    { 0, 0 },
    { 0x5b9eda31, 61 },
    { 0, 0 },
    { 0x795eda33, 46 },
    { 0x12aeda34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1eda3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306eda3f, 98 },
    { 0, 0 },
    { 0xf9eda41, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ceda46, 90 },
    { 0x430eda47, 23 },
    { 0x760eda48, 78 },
    { 0x1e1eda49, 33 },
    { 0x268eda4a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdeda4e, 33 },
    { 0, 0 },
    { 0x2dceda50, 23 },
    { 0x241eda51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eeda5a, 90 },
    { 0x4e9eda5b, 95 },
    { 0x4daeda5a, 46 },
    { 0x5b0eda5d, 90 },
    { 0x57deda5e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x59aeda61, 91 },
    { 0xad7eda62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfeda68, 90 },
    { 0x20aeda69, 91 },
    { 0x987eda6a, 95 },
    { 0xa0deda6b, 65 },
    { 0xc93eda6b, 46 },
    { 0xb3ceda6c, 23 },
    { 0x392eda6e, 90 },
    { 0x76beda6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574eda73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x752eda79, 23 },
    { 0xdfeda7a, 23 },
    { 0xee0eda7b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66beda7e, 43 },
    { 0, 0 },
    { 0x608eda80, 33 },
    { 0, 0 },
    { 0x657eda82, 72 },
    { 0xb08eda82, 87 },
    { 0x716eda84, 86 },
    { 0xb0deda85, 33 },
    { 0x20deda83, 95 },
    { 0x6d2eda84, 90 },
    { 0xd30eda88, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9eda8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57deda91, 90 },
    { 0x664eda91, 33 },
    { 0x8eeda93, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08eda96, 46 },
    { 0x7a9eda97, 61 },
    { 0, 0 },
    { 0x365eda99, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5eda9e, 91 },
    { 0xeebeda9e, 33 },
    { 0xb18eda9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242edaa7, 84 },
    { 0, 0 },
    { 0xe7fedaa9, 33 },
    { 0x810edaaa, 98 },
    { 0, 0 },
    { 0x456edaac, 67 },
    { 0xa41edaad, 98 },
    { 0, 0 },
    { 0x1aaedaaf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fedab2, 61 },
    { 0xcffedab3, 61 },
    { 0xd26edab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04edabf, 49 },
    { 0x8caedac0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcedac3, 23 },
    { 0x83bedac4, 33 },
    { 0x84aedac5, 81 },
    { 0, 0 },
    { 0x22cedac7, 100 },
    { 0x160edac8, 90 },
    { 0, 0 },
    { 0xdc2edaca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93edad1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6edad4, 76 },
    { 0, 0 },
    { 0xe64edad6, 33 },
    { 0x7abedad7, 90 },
    { 0, 0 },
    { 0x934edad9, 33 },
    { 0x45bedada, 76 },
    { 0x98eedadb, 43 },
    { 0x895edadc, 52 },
    { 0xde6edad9, 23 },
    { 0, 0 },
    { 0xdc4edadf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7edae2, 53 },
    { 0x915edae2, 67 },
    { 0xaabedae4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cedae7, 61 },
    { 0xc0cedae7, 23 },
    { 0xc35edae9, 95 },
    { 0x27dedae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25fedaed, 43 },
    { 0xd41edaee, 23 },
    { 0, 0 },
    { 0x83eedaf0, 84 },
    { 0xaafedaf0, 76 },
    { 0x95eedaf0, 61 },
    { 0, 0 },
    { 0x717edaf4, 33 },
    { 0, 0 },
    { 0x3e2edaf6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1eedafa, 95 },
    { 0x70bedafa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8edb00, 94 },
    { 0xaefedb01, 100 },
    { 0, 0 },
    { 0x8b6edb03, 58 },
    { 0x5e5edb04, 99 },
    { 0x5aaedb05, 2 },
    { 0x76bedb06, 90 },
    { 0x953edb04, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178edb0b, 66 },
    { 0xc2dedb0c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9fedb12, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccaedb1e, 76 },
    { 0x65eedb1f, 23 },
    { 0x3bedb1e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6edb23, 61 },
    { 0xa7edb24, 85 },
    { 0x716edb25, 33 },
    { 0x137edb26, 95 },
    { 0, 0 },
    { 0xb5fedb28, 23 },
    { 0x53eedb29, 43 },
    { 0xa45edb2a, 88 },
    { 0x969edb2b, 66 },
    { 0x2a3edb2c, 23 },
    { 0xcacedb2d, 61 },
    { 0x335edb2e, 91 },
    { 0x401edb2f, 66 },
    { 0x238edb2d, 61 },
    { 0x765edb2a, 33 },
    { 0, 0 },
    { 0x66fedb33, 54 },
    { 0, 0 },
    { 0xf8eedb35, 90 },
    { 0, 0 },
    { 0xa91edb37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5edb3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2edb3e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7edb48, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06edb4b, 84 },
    { 0xc9edb4b, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb48edb50, 23 },
    { 0x967edb51, 46 },
    { 0xb1edb52, 91 },
    { 0x227edb53, 95 },
    { 0xd4bedb54, 61 },
    { 0xeccedb52, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98edb58, 33 },
    { 0xac8edb59, 65 },
    { 0xf3cedb59, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22aedb62, 52 },
    { 0x8daedb63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x399edb66, 100 },
    { 0xf9eedb67, 76 },
    { 0x81dedb67, 43 },
    { 0xbbfedb67, 90 },
    { 0x3c4edb6a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66edb6d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637edb73, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1edb7d, 17 },
    { 0x7caedb7e, 23 },
    { 0xa41edb7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc7edb82, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cedb86, 33 },
    { 0x480edb87, 93 },
    { 0, 0 },
    { 0x38bedb89, 85 },
    { 0x7c9edb8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36edb90, 95 },
    { 0x5adedb90, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bedb94, 98 },
    { 0x1f4edb94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedb98, 61 },
    { 0xfbeedb98, 95 },
    { 0, 0 },
    { 0x896edb9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5edb9f, 33 },
    { 0xf68edba0, 90 },
    { 0, 0 },
    { 0x280edba2, 68 },
    { 0x515edba3, 43 },
    { 0x496edba3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeedba7, 90 },
    { 0, 0 },
    { 0xdcfedba9, 46 },
    { 0, 0 },
    { 0x120edbab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6eedbae, 76 },
    { 0x833edbaf, 90 },
    { 0x317edbb0, 23 },
    { 0xd6cedbaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aedbb4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9faedbb7, 76 },
    { 0, 0 },
    { 0xdf3edbb9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddedbc0, 90 },
    { 0x9a7edbc0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbedbc9, 95 },
    { 0xdc9edbca, 66 },
    { 0x44dedbcb, 94 },
    { 0, 0 },
    { 0x521edbcd, 52 },
    { 0, 0 },
    { 0x304edbcf, 98 },
    { 0, 0 },
    { 0x2b0edbd1, 34 },
    { 0x4dbedbd2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71edbd7, 93 },
    { 0x5efedbd8, 23 },
    { 0xe11edbd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611edbdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6edbe8, 95 },
    { 0xd11edbe8, 87 },
    { 0, 0 },
    { 0xc97edbeb, 84 },
    { 0, 0 },
    { 0xd49edbed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5edbf2, 66 },
    { 0x824edbf3, 90 },
    { 0, 0 },
    { 0x10fedbf5, 23 },
    { 0x3d9edbf6, 23 },
    { 0x836edbf7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82aedbfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06edc01, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bedc04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8edc0a, 55 },
    { 0, 0 },
    { 0x79bedc0c, 23 },
    { 0, 0 },
    { 0x809edc0e, 23 },
    { 0x624edc0f, 90 },
    { 0, 0 },
    { 0x783edc11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43edc19, 76 },
    { 0x7a7edc19, 100 },
    { 0, 0 },
    { 0x796edc1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaedc20, 23 },
    { 0xd6cedc21, 91 },
    { 0x851edc21, 33 },
    { 0x653edc23, 93 },
    { 0x350edc21, 98 },
    { 0xa3cedc25, 34 },
    { 0, 0 },
    { 0xc6edc27, 76 },
    { 0x384edc27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996edc2d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1edc30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0edc37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30eedc3a, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bedc42, 90 },
    { 0, 0 },
    { 0x5caedc44, 66 },
    { 0, 0 },
    { 0x6ededc46, 33 },
    { 0xab3edc47, 100 },
    { 0xfdedc47, 95 },
    { 0xf41edc49, 53 },
    { 0, 0 },
    { 0x9a0edc4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fedc4e, 95 },
    { 0x2b4edc4f, 34 },
    { 0x686edc4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eedc57, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6eedc5a, 87 },
    { 0x95bedc5b, 95 },
    { 0, 0 },
    { 0x528edc5d, 90 },
    { 0x7bedc5e, 90 },
    { 0xa73edc5d, 76 },
    { 0x6f6edc60, 95 },
    { 0x7b0edc60, 81 },
    { 0xcf2edc60, 95 },
    { 0x189edc61, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8edc6b, 61 },
    { 0, 0 },
    { 0xec9edc6d, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bedc70, 98 },
    { 0, 0 },
    { 0x989edc72, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67edc75, 23 },
    { 0x7b1edc75, 67 },
    { 0x975edc77, 61 },
    { 0, 0 },
    { 0x76aedc79, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894edc7d, 95 },
    { 0xe82edc7e, 25 },
    { 0x67edc7f, 55 },
    { 0x5ededc80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdedc86, 88 },
    { 0x2e1edc87, 90 },
    { 0, 0 },
    { 0x6e4edc89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8edc8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69edc92, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344edc99, 61 },
    { 0xff7edc99, 90 },
    { 0, 0 },
    { 0x337edc9c, 33 },
    { 0, 0 },
    { 0x28bedc9e, 90 },
    { 0x8eaedc9f, 61 },
    { 0x49fedc9e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52edca7, 66 },
    { 0xbccedca8, 68 },
    { 0x3b8edca9, 99 },
    { 0x5b1edca8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deedcb7, 61 },
    { 0x60dedcb8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x305edcbb, 90 },
    { 0xf91edcbc, 81 },
    { 0x312edcbd, 23 },
    { 0x2e7edcbe, 68 },
    { 0, 0 },
    { 0x750edcc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31edcc3, 76 },
    { 0x607edcc4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6edcca, 95 },
    { 0xf33edccb, 75 },
    { 0, 0 },
    { 0xf07edccd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x992edcd0, 33 },
    { 0xd8dedcd0, 93 },
    { 0xae0edcd2, 83 },
    { 0x1e9edcd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7eedcd6, 55 },
    { 0, 0 },
    { 0x75bedcd8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80edcdb, 23 },
    { 0, 0 },
    { 0xc0fedcdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9edce0, 67 },
    { 0xccaedce1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bedce6, 34 },
    { 0, 0 },
    { 0x97aedce8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bedceb, 76 },
    { 0xc98edcec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270edcf2, 33 },
    { 0, 0 },
    { 0xb5eedcf4, 90 },
    { 0xbb2edcf5, 43 },
    { 0x2f7edcf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x156edcf9, 33 },
    { 0xb37edcfa, 77 },
    { 0x920edcfb, 90 },
    { 0x887edcfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aedd01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721edd07, 46 },
    { 0xb5fedd07, 61 },
    { 0xd10edd09, 61 },
    { 0xc8edd07, 23 },
    { 0x7b0edd0b, 61 },
    { 0x19edd0b, 23 },
    { 0x6caedd0d, 55 },
    { 0x2ceedd0b, 33 },
    { 0, 0 },
    { 0x531edd10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x768edd15, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x175edd18, 34 },
    { 0, 0 },
    { 0x584edd1a, 90 },
    { 0xf6eedd1a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eedd1e, 61 },
    { 0, 0 },
    { 0x492edd20, 12 },
    { 0x2c5edd21, 68 },
    { 0, 0 },
    { 0x930edd23, 91 },
    { 0x6c3edd23, 34 },
    { 0, 0 },
    { 0x8f4edd26, 66 },
    { 0x45dedd27, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x262edd2a, 90 },
    { 0xb9aedd2a, 23 },
    { 0xf94edd2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98eedd33, 46 },
    { 0xc3bedd34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0edd38, 90 },
    { 0x5f4edd38, 61 },
    { 0xb7bedd38, 90 },
    { 0xf49edd39, 55 },
    { 0x74dedd3c, 90 },
    { 0x418edd3d, 61 },
    { 0x81cedd3e, 33 },
    { 0x733edd3f, 93 },
    { 0xb80edd40, 76 },
    { 0xffaedd3d, 98 },
    { 0, 0 },
    { 0xb2edd43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5edd47, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247edd4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x184edd50, 23 },
    { 0xb50edd51, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d3edd56, 61 },
    { 0x5dfedd57, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dcedd5a, 91 },
    { 0xb8aedd5b, 90 },
    { 0xff4edd5c, 94 },
    { 0xfcdedd5b, 33 },
    { 0x9daedd5e, 100 },
    { 0xb99edd5f, 46 },
    { 0xa1eedd60, 33 },
    { 0x883edd5d, 52 },
    { 0x5e3edd62, 90 },
    { 0xb69edd63, 33 },
    { 0x1afedd64, 95 },
    { 0xdb9edd65, 90 },
    { 0, 0 },
    { 0x973edd67, 17 },
    { 0x7c7edd68, 90 },
    { 0x10fedd68, 90 },
    { 0x8bedd67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacedd6f, 38 },
    { 0, 0 },
    { 0xf8edd71, 100 },
    { 0, 0 },
    { 0x783edd73, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96cedd77, 65 },
    { 0xe3fedd78, 55 },
    { 0x616edd77, 95 },
    { 0xd51edd7a, 33 },
    { 0x323edd7b, 61 },
    { 0, 0 },
    { 0xb84edd7d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aedd81, 43 },
    { 0, 0 },
    { 0x60fedd83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334edd8a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2edd8f, 52 },
    { 0x4c8edd90, 99 },
    { 0, 0 },
    { 0x43eedd92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72bedd96, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15aedd9b, 23 },
    { 0xa5bedd9b, 76 },
    { 0, 0 },
    { 0x51eedd9e, 23 },
    { 0, 0 },
    { 0x2a8edda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600eddac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2eddb0, 84 },
    { 0x352eddb0, 33 },
    { 0x989eddb2, 61 },
    { 0x106eddb2, 90 },
    { 0x2bfeddb4, 90 },
    { 0x5c6eddb5, 61 },
    { 0xfa6eddb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802eddba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x860eddbd, 85 },
    { 0, 0 },
    { 0x644eddbf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbeddc3, 90 },
    { 0x490eddc4, 33 },
    { 0x944eddc5, 95 },
    { 0xba1eddc5, 34 },
    { 0, 0 },
    { 0x794eddc8, 95 },
    { 0x2ddeddc8, 34 },
    { 0x492eddca, 33 },
    { 0x5f4eddca, 68 },
    { 0x3daeddc9, 23 },
    { 0, 0 },
    { 0x43feddce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x499eddd1, 43 },
    { 0x3bfeddd2, 61 },
    { 0x8eceddd3, 67 },
    { 0x73beddd4, 98 },
    { 0xaa8eddd4, 33 },
    { 0xa02eddd4, 90 },
    { 0xecceddd7, 23 },
    { 0xa20eddd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4edddb, 90 },
    { 0, 0 },
    { 0xd9edddd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0dedde1, 93 },
    { 0, 0 },
    { 0xa23edde3, 81 },
    { 0x54fedde3, 34 },
    { 0, 0 },
    { 0x356edde6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822eddea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb24eddf1, 90 },
    { 0x63aeddf2, 90 },
    { 0xe82eddf3, 43 },
    { 0x6fdeddf4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2eddf9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ede03, 90 },
    { 0x4bede04, 85 },
    { 0xf46ede05, 61 },
    { 0xc8fede05, 90 },
    { 0x904ede03, 98 },
    { 0, 0 },
    { 0xef7ede09, 87 },
    { 0x4ede09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825ede0f, 65 },
    { 0x5eeede0f, 33 },
    { 0x929ede11, 78 },
    { 0x7bede12, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xffeede15, 68 },
    { 0x38ede15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x238ede19, 55 },
    { 0xf9dede19, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbede20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afede24, 81 },
    { 0x1faede25, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463ede29, 61 },
    { 0x934ede29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3ede33, 99 },
    { 0xb1fede34, 95 },
    { 0xd06ede35, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceede38, 61 },
    { 0x20dede39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ede3c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaede41, 55 },
    { 0x12cede41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcede47, 90 },
    { 0x9ebede48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x151ede4b, 61 },
    { 0, 0 },
    { 0x18bede4d, 23 },
    { 0, 0 },
    { 0xd86ede4f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ede58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5aede5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743ede62, 90 },
    { 0, 0 },
    { 0x479ede64, 55 },
    { 0, 0 },
    { 0x732ede66, 76 },
    { 0x1aeede67, 90 },
    { 0x74aede68, 34 },
    { 0xa4cede69, 76 },
    { 0xfe8ede69, 90 },
    { 0x4deede6b, 98 },
    { 0, 0 },
    { 0xcc1ede6d, 81 },
    { 0xdf9ede6e, 61 },
    { 0x64eede6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ede77, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fede7a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fede7d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eede81, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5aede87, 95 },
    { 0xd7fede88, 91 },
    { 0x71ede87, 34 },
    { 0x44cede8a, 23 },
    { 0xf68ede8b, 68 },
    { 0xc15ede8b, 52 },
    { 0x982ede8b, 93 },
    { 0xc1dede8e, 55 },
    { 0x412ede88, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62ede92, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2ede95, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157ede9c, 98 },
    { 0, 0 },
    { 0x3ceede9e, 4 },
    { 0x495ede9f, 46 },
    { 0, 0 },
    { 0x1e0edea1, 52 },
    { 0xadfedea2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549edea8, 88 },
    { 0x18cedea9, 95 },
    { 0x525edeaa, 55 },
    { 0x917edea8, 23 },
    { 0xa6aedea8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797edeb3, 87 },
    { 0x7d8edeb3, 34 },
    { 0x737edeb4, 55 },
    { 0x704edeb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fedeb9, 43 },
    { 0x56cedeba, 77 },
    { 0, 0 },
    { 0x9c1edebc, 61 },
    { 0x261edebd, 91 },
    { 0xb5edebe, 23 },
    { 0x3a1edebf, 23 },
    { 0, 0 },
    { 0x16edec1, 90 },
    { 0xf57edec1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18edec7, 95 },
    { 0, 0 },
    { 0x17fedec9, 81 },
    { 0x75dedeca, 69 },
    { 0xa47edec9, 90 },
    { 0x3eaedecc, 33 },
    { 0, 0 },
    { 0x3d6edece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5eded3, 95 },
    { 0x2f7eded4, 95 },
    { 0, 0 },
    { 0x73ceded6, 33 },
    { 0xc9deded7, 23 },
    { 0x47feded8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1edede, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x107edee1, 90 },
    { 0x3f7edee2, 90 },
    { 0xae4edee3, 33 },
    { 0xe47edee4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2edee8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x615edeec, 61 },
    { 0, 0 },
    { 0x60cedeee, 95 },
    { 0x762edeef, 67 },
    { 0x9f0edef0, 90 },
    { 0xf77edef1, 55 },
    { 0x444edef0, 90 },
    { 0x35fedef3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x794edef6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2edefa, 66 },
    { 0x6e1edefa, 46 },
    { 0, 0 },
    { 0x7ffedefd, 33 },
    { 0, 0 },
    { 0x676edeff, 100 },
    { 0x638edf00, 33 },
    { 0xfaedf00, 61 },
    { 0, 0 },
    { 0x219edf03, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19edf09, 83 },
    { 0x938edf0a, 98 },
    { 0xf2eedf09, 95 },
    { 0x136edf09, 33 },
    { 0xfc3edf0d, 95 },
    { 0xeccedf0a, 95 },
    { 0x491edf0a, 23 },
    { 0x628edf10, 98 },
    { 0xe29edf11, 52 },
    { 0xf3dedf12, 33 },
    { 0xe2edf0b, 81 },
    { 0, 0 },
    { 0x5d9edf15, 43 },
    { 0xc2edf15, 87 },
    { 0xd82edf17, 69 },
    { 0x982edf18, 65 },
    { 0, 0 },
    { 0xd0bedf1a, 100 },
    { 0, 0 },
    { 0x6d6edf1c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3edf20, 90 },
    { 0x275edf21, 98 },
    { 0xdadedf22, 90 },
    { 0xdd4edf23, 67 },
    { 0xf6aedf24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6edf27, 61 },
    { 0xc6bedf28, 93 },
    { 0xa87edf27, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3edf32, 86 },
    { 0, 0 },
    { 0x55bedf34, 76 },
    { 0, 0 },
    { 0x9b3edf36, 34 },
    { 0x48aedf37, 88 },
    { 0, 0 },
    { 0xaf4edf39, 95 },
    { 0, 0 },
    { 0x31aedf3b, 75 },
    { 0x26fedf3c, 34 },
    { 0x671edf3c, 33 },
    { 0xaefedf3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6edf41, 36 },
    { 0, 0 },
    { 0x872edf43, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bedf46, 55 },
    { 0, 0 },
    { 0xc33edf48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6edf55, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7edf5c, 55 },
    { 0x6eaedf5d, 33 },
    { 0x262edf5e, 23 },
    { 0xe13edf5f, 33 },
    { 0x3feedf60, 23 },
    { 0x469edf60, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dedf65, 43 },
    { 0x503edf66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4edf6a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x794edf6d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9edf71, 34 },
    { 0xdbcedf72, 61 },
    { 0x869edf72, 98 },
    { 0x5f4edf74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53edf78, 34 },
    { 0x29fedf78, 95 },
    { 0x152edf7a, 33 },
    { 0x31aedf7a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127edf84, 23 },
    { 0x16cedf85, 55 },
    { 0x6f5edf86, 33 },
    { 0xe12edf87, 84 },
    { 0, 0 },
    { 0xce4edf89, 90 },
    { 0x6ecedf8a, 96 },
    { 0xde4edf8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494edf93, 98 },
    { 0, 0 },
    { 0xc4cedf95, 95 },
    { 0xf19edf96, 95 },
    { 0, 0 },
    { 0x898edf98, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d7edfa0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aedfa3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x54aedfa6, 90 },
    { 0x8d8edfa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8edfae, 61 },
    { 0x4f3edfaf, 91 },
    { 0x3c2edfb0, 55 },
    { 0, 0 },
    { 0xd07edfb2, 34 },
    { 0xd70edfb3, 76 },
    { 0x455edfb4, 98 },
    { 0, 0 },
    { 0x18cedfb6, 33 },
    { 0x81aedfb6, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaeedfba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128edfc1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784edfc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cedfc8, 90 },
    { 0x673edfc9, 61 },
    { 0x9f5edfca, 33 },
    { 0xdf3edfca, 90 },
    { 0xff0edfca, 100 },
    { 0x747edfcd, 99 },
    { 0x630edfcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75edfd6, 33 },
    { 0xabeedfd7, 33 },
    { 0xdffedfd7, 34 },
    { 0, 0 },
    { 0xef0edfda, 99 },
    { 0x4baedfdb, 61 },
    { 0x840edfdc, 66 },
    { 0, 0 },
    { 0x16cedfde, 43 },
    { 0x318edfdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fedfe4, 88 },
    { 0, 0 },
    { 0xcb2edfe6, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaedfed, 69 },
    { 0x9f6edfee, 33 },
    { 0x742edfef, 23 },
    { 0xad8edff0, 38 },
    { 0x952edff1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbedff5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3edff8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fedffc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x771edfff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464ee003, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206ee007, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ee00c, 61 },
    { 0, 0 },
    { 0xb57ee00e, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x8caee011, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x588ee015, 23 },
    { 0, 0 },
    { 0xe59ee017, 90 },
    { 0, 0 },
    { 0x10bee019, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59ee020, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2ee023, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dee026, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4ee02c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44ee02f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9ee032, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d3ee035, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8ee038, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76aee041, 67 },
    { 0xe33ee042, 88 },
    { 0x14fee042, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20dee04a, 43 },
    { 0, 0 },
    { 0xfeee04c, 69 },
    { 0, 0 },
    { 0x908ee04e, 78 },
    { 0xa70ee04e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4ee054, 33 },
    { 0x13eee054, 61 },
    { 0, 0 },
    { 0x906ee057, 98 },
    { 0x97dee057, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9ee05b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ee05e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfee06a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadee06e, 95 },
    { 0xebfee06f, 61 },
    { 0, 0 },
    { 0x655ee071, 90 },
    { 0xdd3ee072, 61 },
    { 0xd51ee071, 95 },
    { 0xdbee074, 90 },
    { 0xd97ee075, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacee07a, 55 },
    { 0xaffee07b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ee080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20ee085, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x6abee088, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211ee08c, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7aee091, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x943ee094, 66 },
    { 0x127ee095, 33 },
    { 0, 0 },
    { 0xf8ee097, 33 },
    { 0x80dee098, 90 },
    { 0xe95ee097, 98 },
    { 0x5b8ee09a, 95 },
    { 0, 0 },
    { 0xa62ee09c, 55 },
    { 0x20bee09d, 90 },
    { 0x2f8ee09d, 66 },
    { 0xd30ee09d, 98 },
    { 0, 0 },
    { 0x83cee0a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42eee0a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ee0a9, 61 },
    { 0x8edee0a9, 69 },
    { 0xd08ee0ab, 98 },
    { 0x20cee0a9, 69 },
    { 0, 0 },
    { 0xceeee0ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cee0b1, 61 },
    { 0xcbdee0b2, 61 },
    { 0x33ee0b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afee0bc, 90 },
    { 0xddbee0bc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93eee0c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5ee0c3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2ee0c6, 55 },
    { 0x7cee0c6, 76 },
    { 0, 0 },
    { 0x62bee0c9, 98 },
    { 0, 0 },
    { 0x293ee0cb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9ee0d0, 81 },
    { 0x310ee0d1, 17 },
    { 0x963ee0d2, 76 },
    { 0, 0 },
    { 0x88aee0d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2fee0df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ee0e3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c5ee0e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247ee0ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fee0ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x622ee0f4, 34 },
    { 0xa66ee0f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ecee0f8, 66 },
    { 0xa1eee0f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99eee103, 90 },
    { 0x4e7ee104, 66 },
    { 0xb2aee105, 90 },
    { 0x6e5ee104, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ee10a, 80 },
    { 0x9eee10b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53ee118, 68 },
    { 0, 0 },
    { 0x7f6ee11a, 46 },
    { 0x315ee11b, 33 },
    { 0, 0 },
    { 0xdedee11d, 101 },
    { 0xbb7ee11e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80ee123, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1ee126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacee12a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39ee131, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dee138, 34 },
    { 0xeceee139, 84 },
    { 0xcebee13a, 52 },
    { 0, 0 },
    { 0x9bbee13c, 98 },
    { 0x2beee13d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36aee143, 23 },
    { 0, 0 },
    { 0x66eee145, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cee14a, 98 },
    { 0xf8dee14b, 99 },
    { 0x191ee14c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x176ee14f, 85 },
    { 0x905ee14f, 33 },
    { 0xe85ee151, 34 },
    { 0x66eee151, 98 },
    { 0xd05ee14f, 61 },
    { 0xc20ee154, 23 },
    { 0, 0 },
    { 0x5fcee156, 33 },
    { 0, 0 },
    { 0x6ccee158, 61 },
    { 0xa32ee159, 61 },
    { 0, 0 },
    { 0xe4fee15b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46ee15e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ee164, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cee167, 61 },
    { 0, 0 },
    { 0xb4fee169, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x829ee16e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ee173, 82 },
    { 0, 0 },
    { 0xb0aee175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1ee17a, 61 },
    { 0x3ceee17a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x675ee17f, 33 },
    { 0xf09ee17f, 95 },
    { 0xc10ee181, 80 },
    { 0x295ee182, 46 },
    { 0x6f6ee183, 61 },
    { 0x440ee17f, 98 },
    { 0x256ee185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aee189, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cee18c, 99 },
    { 0x984ee18d, 52 },
    { 0x497ee18d, 100 },
    { 0x9d6ee18c, 98 },
    { 0x4ebee190, 90 },
    { 0x2e8ee18f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x809ee194, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ee19b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46eee1a7, 23 },
    { 0xd5cee1a8, 55 },
    { 0x569ee1a9, 66 },
    { 0x7e0ee1a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296ee1af, 88 },
    { 0x9e6ee1b0, 98 },
    { 0, 0 },
    { 0xb4eee1b2, 87 },
    { 0x1ecee1b3, 8 },
    { 0x798ee1b2, 33 },
    { 0xbc7ee1b5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9ee1b9, 91 },
    { 0x43aee1b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf32ee1bd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cee1c2, 85 },
    { 0x2adee1c3, 61 },
    { 0x928ee1c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50ee1c8, 98 },
    { 0, 0 },
    { 0x659ee1ca, 98 },
    { 0xb94ee1cb, 76 },
    { 0xdf1ee1cc, 95 },
    { 0x686ee1cc, 17 },
    { 0xf1bee1ca, 90 },
    { 0xc00ee1cf, 90 },
    { 0x6e4ee1cc, 95 },
    { 0xda3ee1cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x872ee1d4, 98 },
    { 0xcf8ee1d5, 55 },
    { 0x525ee1d4, 95 },
    { 0x111ee1d7, 23 },
    { 0x3dbee1d8, 33 },
    { 0xb4fee1d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9ee1de, 90 },
    { 0x8adee1de, 33 },
    { 0xc81ee1e0, 61 },
    { 0, 0 },
    { 0x93cee1e2, 53 },
    { 0x45ee1e3, 98 },
    { 0x257ee1e4, 33 },
    { 0, 0 },
    { 0x7f7ee1e6, 85 },
    { 0x78aee1e6, 90 },
    { 0x3ebee1e7, 34 },
    { 0xcb3ee1e9, 55 },
    { 0x331ee1ea, 52 },
    { 0xc5eee1eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfee1f1, 43 },
    { 0x789ee1f2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x550ee1f5, 95 },
    { 0, 0 },
    { 0x628ee1f7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eee1fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x616ee1fe, 11 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51dee205, 90 },
    { 0, 0 },
    { 0x169ee207, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4dee20a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463ee20f, 90 },
    { 0x1c1ee20f, 33 },
    { 0x8beee211, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169ee216, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb0ee21c, 95 },
    { 0x8e2ee21c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323ee221, 90 },
    { 0xf32ee222, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24ee226, 61 },
    { 0x638ee227, 78 },
    { 0x39bee226, 76 },
    { 0, 0 },
    { 0x45fee22a, 61 },
    { 0xe0dee22a, 55 },
    { 0xe8eee22b, 90 },
    { 0, 0 },
    { 0x1c9ee22e, 33 },
    { 0, 0 },
    { 0xc10ee230, 61 },
    { 0, 0 },
    { 0x15cee232, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7ee239, 33 },
    { 0xa23ee23a, 95 },
    { 0xba9ee23b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374ee242, 52 },
    { 0, 0 },
    { 0x24bee244, 43 },
    { 0x8f0ee245, 55 },
    { 0x442ee246, 55 },
    { 0xbdeee246, 78 },
    { 0xbbcee248, 33 },
    { 0x68cee247, 76 },
    { 0x3b4ee247, 61 },
    { 0xd6dee24b, 59 },
    { 0x467ee249, 23 },
    { 0x58eee245, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ee250, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8ee25b, 97 },
    { 0x123ee25c, 33 },
    { 0x46bee25c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86ee268, 90 },
    { 0, 0 },
    { 0x935ee26a, 33 },
    { 0x89bee26b, 93 },
    { 0, 0 },
    { 0x880ee26d, 93 },
    { 0, 0 },
    { 0xbc2ee26f, 61 },
    { 0, 0 },
    { 0x148ee271, 34 },
    { 0xe92ee272, 90 },
    { 0x85eee271, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cee279, 77 },
    { 0, 0 },
    { 0xdb8ee27b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902ee281, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ee285, 23 },
    { 0xc6aee285, 23 },
    { 0xb5ee287, 90 },
    { 0xb18ee288, 43 },
    { 0, 0 },
    { 0xeceee28a, 55 },
    { 0xb1cee28a, 34 },
    { 0x43ee28c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbffee28f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1ee293, 33 },
    { 0, 0 },
    { 0x4afee295, 76 },
    { 0x2b0ee295, 95 },
    { 0x5ccee297, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eee29c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940ee2a0, 90 },
    { 0x7e7ee2a1, 66 },
    { 0x5f3ee2a2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6ee2a5, 34 },
    { 0x774ee2a6, 40 },
    { 0x901ee2a6, 98 },
    { 0x88aee2a6, 55 },
    { 0x91eee2a9, 98 },
    { 0xbfeee2a9, 33 },
    { 0, 0 },
    { 0x333ee2ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eee2b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bee2b8, 90 },
    { 0x57dee2b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x35aee2bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ee2c0, 84 },
    { 0x526ee2c1, 61 },
    { 0x583ee2c2, 98 },
    { 0x18dee2c3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcee2c6, 90 },
    { 0, 0 },
    { 0x811ee2c8, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x24dee2cb, 100 },
    { 0xbc1ee2cc, 98 },
    { 0x50cee2cc, 90 },
    { 0x378ee2ce, 33 },
    { 0x7f1ee2cf, 90 },
    { 0x30bee2d0, 33 },
    { 0, 0 },
    { 0x6d7ee2d2, 98 },
    { 0, 0 },
    { 0x755ee2d4, 93 },
    { 0, 0 },
    { 0x3cbee2d6, 77 },
    { 0, 0 },
    { 0xe32ee2d8, 40 },
    { 0x2f6ee2d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x232ee2dc, 36 },
    { 0x954ee2dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ee2e0, 66 },
    { 0x395ee2e1, 78 },
    { 0xa74ee2e1, 90 },
    { 0x7bdee2e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ee2e6, 34 },
    { 0xaa2ee2e7, 43 },
    { 0xce0ee2e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ee2eb, 85 },
    { 0x65dee2eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ee2f3, 67 },
    { 0x417ee2f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x590ee2f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443ee2ff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968ee306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ee30b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x18dee30e, 78 },
    { 0x573ee30f, 76 },
    { 0xc6eee310, 100 },
    { 0xed7ee310, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7ee315, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x826ee318, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aeee31d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dee323, 90 },
    { 0xc1ee323, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bee327, 61 },
    { 0x98ee327, 95 },
    { 0x8dbee328, 91 },
    { 0xcf3ee32a, 19 },
    { 0xca7ee32a, 65 },
    { 0x9acee32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12eee331, 90 },
    { 0, 0 },
    { 0x3dee333, 90 },
    { 0x6aeee334, 98 },
    { 0xe17ee335, 76 },
    { 0x5f5ee334, 34 },
    { 0x62dee334, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x363ee33a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aee33d, 61 },
    { 0x63cee33e, 34 },
    { 0x484ee33e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499ee344, 43 },
    { 0xbcfee345, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bfee349, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5ee34d, 90 },
    { 0xbd2ee34d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71ee351, 46 },
    { 0xf6aee352, 76 },
    { 0x12cee351, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ee356, 23 },
    { 0, 0 },
    { 0xf62ee358, 90 },
    { 0xc8bee359, 46 },
    { 0x774ee35a, 95 },
    { 0xc23ee35b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938ee35f, 90 },
    { 0xd22ee360, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66aee364, 90 },
    { 0x1caee365, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54ee368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2eee36b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5ee36f, 98 },
    { 0x6aaee36f, 33 },
    { 0xab5ee371, 93 },
    { 0xedfee36f, 34 },
    { 0x4dbee373, 98 },
    { 0xe12ee372, 95 },
    { 0x3eee373, 90 },
    { 0x35bee376, 90 },
    { 0xd2ee375, 61 },
    { 0, 0 },
    { 0x45dee379, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0eee37c, 34 },
    { 0x25dee37d, 61 },
    { 0, 0 },
    { 0x2c3ee37f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bee382, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6ee38b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bee38e, 43 },
    { 0xc1aee38f, 61 },
    { 0xf5ee390, 98 },
    { 0x855ee38e, 33 },
    { 0xe51ee392, 76 },
    { 0xe60ee392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafee397, 23 },
    { 0x773ee398, 90 },
    { 0xc75ee398, 23 },
    { 0x138ee398, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184ee39f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cee3a3, 91 },
    { 0, 0 },
    { 0x50fee3a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ee3a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52ee3ab, 87 },
    { 0x7c5ee3ac, 34 },
    { 0x559ee3ac, 33 },
    { 0x31dee3ab, 34 },
    { 0, 0 },
    { 0xb7fee3b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fee3b4, 90 },
    { 0x129ee3b4, 46 },
    { 0xabaee3b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ee3bd, 84 },
    { 0, 0 },
    { 0xb02ee3bf, 85 },
    { 0x1dee3c0, 34 },
    { 0xbd5ee3c0, 76 },
    { 0, 0 },
    { 0xcfdee3c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90eee3ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddee3d0, 85 },
    { 0x777ee3d1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8ee3d4, 76 },
    { 0, 0 },
    { 0xfd6ee3d6, 84 },
    { 0x90cee3d7, 90 },
    { 0x5b6ee3d8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22ee3e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82ee3ea, 33 },
    { 0, 0 },
    { 0x54aee3ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fee3f8, 61 },
    { 0, 0 },
    { 0x9dee3fa, 90 },
    { 0xddbee3fb, 34 },
    { 0x65bee3fa, 98 },
    { 0xe5bee3fa, 95 },
    { 0x9ccee3fe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x930ee401, 34 },
    { 0x827ee402, 87 },
    { 0x61eee402, 100 },
    { 0, 0 },
    { 0x4c2ee405, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97ee40a, 61 },
    { 0xb48ee40b, 99 },
    { 0, 0 },
    { 0xd74ee40d, 100 },
    { 0xe4eee40e, 91 },
    { 0xfe4ee40e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ee413, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77ee416, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ee41e, 87 },
    { 0x49ee41f, 61 },
    { 0xbc3ee41f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1ee426, 76 },
    { 0xd82ee426, 23 },
    { 0x2d4ee428, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ee42b, 95 },
    { 0, 0 },
    { 0x5ee42d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x831ee432, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572ee437, 23 },
    { 0x357ee438, 33 },
    { 0xe13ee439, 85 },
    { 0x699ee438, 76 },
    { 0x650ee43a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459ee440, 33 },
    { 0xd6bee440, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342ee445, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa22ee44b, 34 },
    { 0, 0 },
    { 0xa68ee44d, 67 },
    { 0x954ee44e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13eee452, 49 },
    { 0x81bee453, 93 },
    { 0, 0 },
    { 0xb38ee455, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecee459, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448ee45d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdee463, 61 },
    { 0xe9bee463, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23ee46a, 76 },
    { 0, 0 },
    { 0x3e4ee46c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cee470, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ee47b, 78 },
    { 0, 0 },
    { 0xe93ee47d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3deee480, 95 },
    { 0xc66ee481, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07ee484, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x684ee487, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56aee48f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27eee497, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x608ee49a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29eee49e, 98 },
    { 0x846ee49e, 34 },
    { 0x976ee4a0, 93 },
    { 0x2daee49e, 23 },
    { 0xf08ee4a2, 87 },
    { 0x68ee4a2, 48 },
    { 0xed1ee4a2, 95 },
    { 0xe2bee4a5, 95 },
    { 0x7f3ee4a1, 90 },
    { 0x7ebee4a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01ee4aa, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ee4b1, 95 },
    { 0x22bee4b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ee4b5, 98 },
    { 0x9a3ee4b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddee4c0, 93 },
    { 0x356ee4c1, 98 },
    { 0x7b5ee4c2, 95 },
    { 0xccbee4c3, 78 },
    { 0xbadee4c4, 88 },
    { 0xbecee4c2, 78 },
    { 0xa79ee4c5, 23 },
    { 0, 0 },
    { 0x542ee4c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9ee4cb, 23 },
    { 0x15aee4cc, 34 },
    { 0x91dee4cd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5ee4d3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8ee4df, 69 },
    { 0x635ee4e0, 84 },
    { 0xe3cee4e1, 43 },
    { 0xbc8ee4e2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fee4e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281ee4f2, 90 },
    { 0x9a7ee4f3, 43 },
    { 0x343ee4f3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71fee4f8, 98 },
    { 0x861ee4f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87ee4fc, 90 },
    { 0x110ee4fd, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edee506, 90 },
    { 0xf37ee506, 56 },
    { 0xc7eee507, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193ee50d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ee511, 90 },
    { 0xdbfee512, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x750ee515, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5ee518, 60 },
    { 0, 0 },
    { 0x395ee51a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370ee51f, 33 },
    { 0, 0 },
    { 0x56ee521, 55 },
    { 0x801ee522, 61 },
    { 0, 0 },
    { 0x62fee524, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6ee528, 87 },
    { 0x94bee529, 90 },
    { 0x62cee529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91ee52e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaee534, 85 },
    { 0xf63ee534, 95 },
    { 0xb9ee534, 78 },
    { 0xb8ee536, 33 },
    { 0, 0 },
    { 0x45ee539, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7eee540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cee544, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1ee547, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aee54f, 91 },
    { 0x671ee54f, 33 },
    { 0, 0 },
    { 0xb4cee552, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caee556, 99 },
    { 0x209ee557, 55 },
    { 0x352ee557, 95 },
    { 0x659ee559, 90 },
    { 0xf4dee559, 61 },
    { 0x5baee556, 61 },
    { 0x7fdee55c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfee560, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cee565, 43 },
    { 0xd86ee566, 43 },
    { 0, 0 },
    { 0xd80ee568, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0ee56c, 93 },
    { 0xa6bee56d, 33 },
    { 0x864ee56e, 76 },
    { 0x699ee56f, 90 },
    { 0x3fbee570, 76 },
    { 0x660ee56d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bee577, 84 },
    { 0xcaee577, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93ee57b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753ee580, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68ee58b, 78 },
    { 0, 0 },
    { 0xc17ee58d, 90 },
    { 0, 0 },
    { 0xb8bee58f, 97 },
    { 0, 0 },
    { 0xb79ee591, 23 },
    { 0, 0 },
    { 0xcddee593, 88 },
    { 0, 0 },
    { 0x4eeee595, 34 },
    { 0, 0 },
    { 0x6a3ee597, 61 },
    { 0, 0 },
    { 0x711ee599, 43 },
    { 0xc9cee59a, 33 },
    { 0x443ee59b, 76 },
    { 0xab0ee59a, 17 },
    { 0, 0 },
    { 0x1ceee59e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fee5a2, 23 },
    { 0x961ee5a3, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956ee5a8, 47 },
    { 0, 0 },
    { 0x58fee5aa, 95 },
    { 0, 0 },
    { 0x741ee5ac, 93 },
    { 0, 0 },
    { 0x430ee5ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17eee5b4, 33 },
    { 0xc6cee5b4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aee5bb, 95 },
    { 0x739ee5bc, 67 },
    { 0x898ee5bd, 87 },
    { 0, 0 },
    { 0xfb7ee5bf, 33 },
    { 0x56aee5bf, 90 },
    { 0xd65ee5c1, 91 },
    { 0x903ee5c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4ee5c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cee5c9, 61 },
    { 0x957ee5ca, 46 },
    { 0xf6bee5cb, 61 },
    { 0, 0 },
    { 0xc9ee5cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2ee5d2, 98 },
    { 0xa9bee5d2, 52 },
    { 0, 0 },
    { 0x90bee5d5, 90 },
    { 0, 0 },
    { 0xbaee5d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1ee5da, 84 },
    { 0, 0 },
    { 0x5d7ee5dc, 95 },
    { 0x2d5ee5dc, 61 },
    { 0xe2ee5de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feee5e1, 99 },
    { 0, 0 },
    { 0xd80ee5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x622ee5e6, 90 },
    { 0, 0 },
    { 0xe7eee5e8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ee5eb, 33 },
    { 0x4c8ee5ec, 33 },
    { 0xad5ee5eb, 98 },
    { 0, 0 },
    { 0xabbee5ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b6ee5f8, 88 },
    { 0x182ee5f8, 55 },
    { 0x64bee5fa, 23 },
    { 0xa76ee5f8, 90 },
    { 0xd9cee5fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ee606, 61 },
    { 0xc13ee606, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a7ee60a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6ee613, 61 },
    { 0x3e2ee613, 90 },
    { 0x83cee614, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9dee619, 61 },
    { 0xa9fee61a, 95 },
    { 0, 0 },
    { 0x8e5ee61c, 61 },
    { 0x64cee61c, 90 },
    { 0, 0 },
    { 0xe92ee61f, 98 },
    { 0x28cee620, 76 },
    { 0x878ee621, 26 },
    { 0x377ee61f, 95 },
    { 0, 0 },
    { 0xe31ee624, 34 },
    { 0, 0 },
    { 0x9d4ee626, 98 },
    { 0xc56ee627, 85 },
    { 0, 0 },
    { 0x8eee629, 90 },
    { 0, 0 },
    { 0x6f8ee62b, 90 },
    { 0xf7fee62c, 76 },
    { 0x97cee62d, 61 },
    { 0x962ee62d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cee63a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66ee63f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63ee646, 83 },
    { 0xda8ee647, 61 },
    { 0x16bee648, 90 },
    { 0, 0 },
    { 0xeb0ee64a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeee650, 66 },
    { 0xa32ee651, 33 },
    { 0xe84ee652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0ee657, 55 },
    { 0, 0 },
    { 0x8e0ee659, 98 },
    { 0xee3ee65a, 93 },
    { 0, 0 },
    { 0x696ee65c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ee663, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ee666, 55 },
    { 0xdb5ee667, 23 },
    { 0, 0 },
    { 0xfdbee669, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478ee66d, 98 },
    { 0, 0 },
    { 0x9e9ee66f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7ee676, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x756ee679, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27ee67d, 33 },
    { 0xffdee67d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcee682, 52 },
    { 0x24dee682, 33 },
    { 0x38cee682, 55 },
    { 0x72eee683, 53 },
    { 0, 0 },
    { 0xf1ee687, 55 },
    { 0xa84ee688, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3dee68f, 23 },
    { 0xe5fee690, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faee695, 76 },
    { 0x88aee696, 23 },
    { 0x1acee697, 85 },
    { 0x638ee697, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585ee69c, 76 },
    { 0xfa1ee69c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faee6a0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83eee6a4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaee6aa, 91 },
    { 0, 0 },
    { 0x1cdee6ac, 78 },
    { 0, 0 },
    { 0xd94ee6ae, 23 },
    { 0xf45ee6af, 55 },
    { 0xfe4ee6b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bee6b3, 90 },
    { 0x86dee6b3, 61 },
    { 0, 0 },
    { 0x529ee6b6, 52 },
    { 0x7e9ee6b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a5ee6bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0ee6be, 90 },
    { 0, 0 },
    { 0x3ceee6c0, 61 },
    { 0x84ee6c1, 91 },
    { 0xa46ee6c2, 46 },
    { 0x86aee6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x67aee6c6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04ee6cc, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ee6d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeee6d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787ee6dc, 95 },
    { 0xf75ee6dc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc11ee6e1, 23 },
    { 0x5eeee6e2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ee6e6, 93 },
    { 0x6c0ee6e7, 90 },
    { 0xb22ee6e8, 88 },
    { 0x22eee6e8, 98 },
    { 0xcfeee6e8, 23 },
    { 0x82eee6e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4ee6f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30ee6f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cee6f8, 61 },
    { 0xe07ee6f9, 93 },
    { 0x339ee6f8, 76 },
    { 0xd7bee6f9, 90 },
    { 0x17aee6fc, 61 },
    { 0, 0 },
    { 0x14bee6fe, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118ee704, 91 },
    { 0, 0 },
    { 0x97aee706, 33 },
    { 0xe89ee706, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7ee712, 98 },
    { 0x2cfee713, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8ee717, 90 },
    { 0xa40ee718, 78 },
    { 0x8c5ee719, 98 },
    { 0xcddee71a, 77 },
    { 0x9a0ee71a, 65 },
    { 0x896ee71a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ee721, 90 },
    { 0x52aee722, 24 },
    { 0x1f2ee723, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1ee727, 65 },
    { 0x90eee728, 90 },
    { 0, 0 },
    { 0x1b3ee72a, 66 },
    { 0x926ee72a, 87 },
    { 0x983ee72b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12aee731, 95 },
    { 0x96dee732, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cee735, 64 },
    { 0x7a6ee735, 90 },
    { 0x70cee735, 61 },
    { 0xc7fee735, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77ee743, 95 },
    { 0xc26ee743, 95 },
    { 0, 0 },
    { 0xae5ee746, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x757ee749, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdee74d, 55 },
    { 0x96aee74e, 46 },
    { 0x6f2ee74e, 95 },
    { 0xc7cee74f, 100 },
    { 0x4feee74f, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcee755, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2ee759, 76 },
    { 0x7fcee75a, 44 },
    { 0x33eee75b, 61 },
    { 0xb60ee75a, 46 },
    { 0xb79ee75a, 90 },
    { 0x45eee759, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80dee764, 90 },
    { 0x61ee764, 23 },
    { 0xce9ee766, 33 },
    { 0x46fee766, 61 },
    { 0xe9dee764, 95 },
    { 0, 0 },
    { 0x2ccee76a, 90 },
    { 0, 0 },
    { 0x78cee76c, 95 },
    { 0x902ee76d, 90 },
    { 0xc21ee76e, 61 },
    { 0, 0 },
    { 0xf74ee770, 77 },
    { 0, 0 },
    { 0x46fee772, 67 },
    { 0x95aee773, 55 },
    { 0x1f5ee773, 90 },
    { 0x2ccee772, 61 },
    { 0xd7aee773, 43 },
    { 0xf9dee777, 98 },
    { 0xfe1ee777, 33 },
    { 0xfb1ee778, 93 },
    { 0xcb7ee77a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abee77f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dee786, 46 },
    { 0, 0 },
    { 0x89cee788, 33 },
    { 0, 0 },
    { 0x859ee78a, 90 },
    { 0, 0 },
    { 0x734ee78c, 88 },
    { 0x97ee78c, 64 },
    { 0, 0 },
    { 0xe1cee78f, 90 },
    { 0x802ee78f, 78 },
    { 0xe37ee78f, 23 },
    { 0x3faee78f, 23 },
    { 0, 0 },
    { 0x6a7ee794, 15 },
    { 0, 0 },
    { 0x3fcee796, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eee7a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0ee7a4, 23 },
    { 0x324ee7a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f8ee7a8, 98 },
    { 0, 0 },
    { 0xe3eee7aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdee7ad, 61 },
    { 0, 0 },
    { 0xd72ee7af, 61 },
    { 0, 0 },
    { 0x12dee7b1, 61 },
    { 0x22eee7b2, 34 },
    { 0x518ee7b1, 100 },
    { 0x154ee7b2, 87 },
    { 0xab5ee7b5, 61 },
    { 0x439ee7b2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cfee7b9, 93 },
    { 0, 0 },
    { 0xbb6ee7bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5ee7c8, 61 },
    { 0xc51ee7c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788ee7d0, 85 },
    { 0, 0 },
    { 0x3ee7d2, 66 },
    { 0x225ee7d2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963ee7d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaee7dd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ee7e1, 33 },
    { 0x29cee7e1, 23 },
    { 0x572ee7e3, 95 },
    { 0, 0 },
    { 0x9e7ee7e5, 90 },
    { 0x17aee7e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933ee7f0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9ee7f4, 76 },
    { 0x890ee7f5, 34 },
    { 0, 0 },
    { 0x3afee7f7, 23 },
    { 0xd29ee7f8, 90 },
    { 0x2d9ee7f7, 23 },
    { 0, 0 },
    { 0x6beee7fb, 34 },
    { 0, 0 },
    { 0xf92ee7fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ee800, 33 },
    { 0xe8fee801, 33 },
    { 0, 0 },
    { 0x122ee803, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bee806, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dee809, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00ee80c, 17 },
    { 0, 0 },
    { 0x8dfee80e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74ee811, 90 },
    { 0, 0 },
    { 0xceee813, 90 },
    { 0, 0 },
    { 0x2f0ee815, 33 },
    { 0xa4cee816, 84 },
    { 0x48cee817, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baee81c, 90 },
    { 0x3a3ee81d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26ee826, 33 },
    { 0, 0 },
    { 0x667ee828, 90 },
    { 0x30aee829, 90 },
    { 0, 0 },
    { 0x209ee82b, 80 },
    { 0x8f2ee82b, 98 },
    { 0x58aee82d, 90 },
    { 0, 0 },
    { 0x2d2ee82f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850ee834, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ee83a, 90 },
    { 0xfdcee83a, 61 },
    { 0x46fee83c, 61 },
    { 0xa96ee83c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x980ee847, 95 },
    { 0x26fee848, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0eee84e, 33 },
    { 0, 0 },
    { 0xe27ee850, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fee853, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cee858, 90 },
    { 0, 0 },
    { 0x6b2ee85a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7ee861, 33 },
    { 0x30dee862, 98 },
    { 0, 0 },
    { 0x5cee864, 55 },
    { 0xc9dee865, 33 },
    { 0, 0 },
    { 0xb8eee867, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaee86b, 95 },
    { 0, 0 },
    { 0x45cee86d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ee870, 85 },
    { 0xdbee871, 91 },
    { 0xee5ee871, 90 },
    { 0x9dbee873, 67 },
    { 0xecee870, 96 },
    { 0, 0 },
    { 0xd4eee876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cee87b, 90 },
    { 0x5b7ee87c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x203ee882, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cee885, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceee88d, 95 },
    { 0, 0 },
    { 0x367ee88f, 90 },
    { 0xdd3ee890, 76 },
    { 0xdc6ee890, 90 },
    { 0x4f8ee892, 38 },
    { 0x147ee893, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bee897, 34 },
    { 0x7afee898, 98 },
    { 0x441ee898, 95 },
    { 0xe8dee89a, 23 },
    { 0x444ee89b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x435ee89e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x23fee8a1, 61 },
    { 0xd1fee8a2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81ee8a7, 61 },
    { 0, 0 },
    { 0xec1ee8a9, 90 },
    { 0xe8ee8aa, 87 },
    { 0xd1fee8ab, 61 },
    { 0, 0 },
    { 0xb31ee8ad, 67 },
    { 0x713ee8ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaee8b1, 81 },
    { 0x70fee8b2, 33 },
    { 0x29eee8b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0ee8bc, 95 },
    { 0, 0 },
    { 0x78dee8be, 90 },
    { 0x578ee8bf, 61 },
    { 0x739ee8be, 90 },
    { 0xbbeee8c1, 55 },
    { 0, 0 },
    { 0x819ee8c3, 95 },
    { 0, 0 },
    { 0xd07ee8c5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17ee8cc, 67 },
    { 0x2e0ee8cd, 85 },
    { 0, 0 },
    { 0xa99ee8cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x479ee8d2, 76 },
    { 0x98dee8d3, 98 },
    { 0xa6cee8d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1fee8d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x959ee8df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bee8e3, 17 },
    { 0, 0 },
    { 0x51eee8e5, 61 },
    { 0xf48ee8e6, 34 },
    { 0x2e9ee8e7, 90 },
    { 0x627ee8e8, 66 },
    { 0x298ee8e7, 43 },
    { 0, 0 },
    { 0x68ee8eb, 93 },
    { 0, 0 },
    { 0x246ee8ed, 90 },
    { 0x9f7ee8ee, 33 },
    { 0xc94ee8ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3ee8f2, 66 },
    { 0x6b4ee8f3, 61 },
    { 0x7c5ee8f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4ee8f7, 33 },
    { 0xf7bee8f8, 23 },
    { 0, 0 },
    { 0xcf7ee8fa, 34 },
    { 0xf80ee8fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7acee901, 55 },
    { 0, 0 },
    { 0xeddee903, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ee908, 23 },
    { 0, 0 },
    { 0x5afee90a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x827ee911, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cfee914, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aee918, 17 },
    { 0xc20ee919, 87 },
    { 0x7ccee918, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceee922, 101 },
    { 0xa2bee923, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1ee928, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252ee92f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x763ee932, 88 },
    { 0, 0 },
    { 0xfc4ee934, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207ee93b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bee947, 95 },
    { 0xa7ee948, 23 },
    { 0x7a1ee948, 90 },
    { 0xbc3ee949, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449ee94e, 34 },
    { 0, 0 },
    { 0x55ee950, 90 },
    { 0, 0 },
    { 0xadeee952, 64 },
    { 0x589ee953, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7ee968, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fee96d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5eee973, 61 },
    { 0, 0 },
    { 0x872ee975, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ee97d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x781ee980, 78 },
    { 0xca9ee981, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ee988, 76 },
    { 0x4eaee988, 61 },
    { 0xb31ee989, 98 },
    { 0x998ee98b, 34 },
    { 0x660ee98a, 43 },
    { 0x974ee98d, 91 },
    { 0x3e4ee98e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x166ee994, 23 },
    { 0xacbee995, 87 },
    { 0x400ee994, 90 },
    { 0x817ee997, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3ee99d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8ee9a0, 76 },
    { 0, 0 },
    { 0x216ee9a2, 95 },
    { 0x82cee9a3, 99 },
    { 0xfeaee9a4, 46 },
    { 0xc6bee9a5, 34 },
    { 0, 0 },
    { 0xecdee9a7, 93 },
    { 0xec7ee9a8, 34 },
    { 0xeadee9a9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaee9af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1eee9b3, 61 },
    { 0, 0 },
    { 0xc43ee9b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbee9b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719ee9bd, 90 },
    { 0x9ccee9be, 93 },
    { 0xec8ee9bd, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709ee9c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cee9ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dee9d1, 61 },
    { 0, 0 },
    { 0x522ee9d3, 90 },
    { 0x98eee9d4, 95 },
    { 0x576ee9d5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bee9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61ee9dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f5ee9e0, 98 },
    { 0, 0 },
    { 0x9adee9e2, 90 },
    { 0x70cee9e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caee9e7, 99 },
    { 0x177ee9e7, 34 },
    { 0xf3fee9e8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82ee9f2, 88 },
    { 0, 0 },
    { 0xd4aee9f4, 84 },
    { 0x528ee9f5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dee9fa, 50 },
    { 0, 0 },
    { 0, 0 },
    { 0x11cee9fd, 96 },
    { 0x683ee9fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaeea05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x419eea09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbaeea12, 98 },
    { 0x644eea13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x323eea17, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92eea1b, 61 },
    { 0, 0 },
    { 0x68aeea1d, 33 },
    { 0xeebeea1e, 93 },
    { 0xd26eea1f, 61 },
    { 0x1dfeea1d, 52 },
    { 0x4bceea1f, 43 },
    { 0xf75eea22, 95 },
    { 0, 0 },
    { 0xa70eea24, 88 },
    { 0xd95eea25, 61 },
    { 0, 0 },
    { 0x38deea27, 95 },
    { 0x934eea28, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373eea2c, 55 },
    { 0x626eea2d, 93 },
    { 0x483eea2e, 95 },
    { 0xbfaeea2f, 98 },
    { 0x3e0eea2c, 66 },
    { 0x88aeea2e, 88 },
    { 0x9b1eea32, 53 },
    { 0x6d1eea32, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4eea3e, 98 },
    { 0, 0 },
    { 0xd6aeea40, 55 },
    { 0x2a1eea41, 90 },
    { 0x56eeea42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ceea47, 55 },
    { 0x673eea48, 61 },
    { 0, 0 },
    { 0xb78eea4a, 33 },
    { 0x301eea4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25aeea50, 61 },
    { 0xd6deea51, 93 },
    { 0, 0 },
    { 0xcabeea53, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x225eea56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94eea5b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x363eea5e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78beea65, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19eea68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ceea6e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7deea75, 23 },
    { 0x2c9eea76, 33 },
    { 0xa58eea77, 98 },
    { 0xa80eea78, 66 },
    { 0x5c4eea77, 34 },
    { 0x8e4eea7a, 98 },
    { 0xf4feea79, 96 },
    { 0x5dbeea7c, 55 },
    { 0x28beea7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aeea80, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5deea86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408eea8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06eea92, 90 },
    { 0, 0 },
    { 0xb3deea94, 98 },
    { 0x7b4eea95, 23 },
    { 0x928eea95, 33 },
    { 0, 0 },
    { 0x986eea98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05eea9f, 61 },
    { 0, 0 },
    { 0xb9feeaa1, 66 },
    { 0, 0 },
    { 0x38ceeaa3, 34 },
    { 0, 0 },
    { 0x7a3eeaa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfeeaa8, 46 },
    { 0, 0 },
    { 0xbb4eeaaa, 98 },
    { 0, 0 },
    { 0xbbdeeaac, 66 },
    { 0, 0 },
    { 0xcd2eeaae, 88 },
    { 0, 0 },
    { 0x13eeab0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2eeab8, 23 },
    { 0, 0 },
    { 0xfc2eeaba, 84 },
    { 0x1e4eeaba, 95 },
    { 0, 0 },
    { 0x739eeabd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11beeac1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ceeac6, 98 },
    { 0xa75eeac7, 33 },
    { 0xbaeeac7, 33 },
    { 0x79feeac9, 65 },
    { 0x213eeac8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5eeacd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3beeead4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36aeead9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1eeadc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9feeadf, 76 },
    { 0x614eeae0, 23 },
    { 0, 0 },
    { 0x4edeeae2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x833eeae7, 33 },
    { 0xfc2eeae7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3feeaeb, 61 },
    { 0, 0 },
    { 0xa8eeeaed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15eeaf4, 61 },
    { 0xca8eeaf5, 61 },
    { 0xd51eeaf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9feeafa, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0beeafe, 34 },
    { 0x6a3eeafe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x445eeb02, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60eeb0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9eeb13, 95 },
    { 0xbbdeeb14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x701eeb17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8eeb1a, 61 },
    { 0x13beeb1b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268eeb1f, 90 },
    { 0xa1eeeb20, 55 },
    { 0x4b5eeb21, 98 },
    { 0, 0 },
    { 0xde1eeb23, 90 },
    { 0xc1beeb23, 91 },
    { 0x141eeb24, 40 },
    { 0x747eeb26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64deeb2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcceeb2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72eeb34, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x813eeb37, 61 },
    { 0x973eeb37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94eeb44, 60 },
    { 0x941eeb44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91beeb4a, 52 },
    { 0x14ceeb4b, 81 },
    { 0, 0 },
    { 0x5dfeeb4d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x42beeb50, 33 },
    { 0x7dbeeb51, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2beeb58, 90 },
    { 0x44eeeb59, 40 },
    { 0x7feeeb59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937eeb61, 33 },
    { 0, 0 },
    { 0xb7beeb63, 98 },
    { 0x623eeb63, 77 },
    { 0x99aeeb65, 100 },
    { 0x2d4eeb63, 95 },
    { 0x98feeb67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eeeb6a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ceeb73, 90 },
    { 0, 0 },
    { 0x795eeb75, 90 },
    { 0x85eeb76, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ceeb79, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x45beeb7c, 46 },
    { 0x775eeb7d, 95 },
    { 0, 0 },
    { 0xae7eeb7f, 95 },
    { 0, 0 },
    { 0x845eeb81, 55 },
    { 0x56deeb82, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ceeb89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7eeb8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3eeb93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d0eeb9b, 99 },
    { 0x8e7eeb9c, 76 },
    { 0x7deeeb9c, 19 },
    { 0x6dfeeb9e, 33 },
    { 0xbdfeeb9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eeeba2, 61 },
    { 0x247eeba3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5feeba6, 78 },
    { 0x4c4eeba7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x56beebaa, 34 },
    { 0x9a4eebab, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ceebae, 61 },
    { 0, 0 },
    { 0xdd0eebb0, 90 },
    { 0x9ceeebb0, 33 },
    { 0, 0 },
    { 0x45feebb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4eebb8, 90 },
    { 0xee9eebb9, 62 },
    { 0, 0 },
    { 0x140eebbb, 43 },
    { 0, 0 },
    { 0x4ceebbd, 76 },
    { 0xe57eebbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30eebc9, 61 },
    { 0, 0 },
    { 0xf03eebcb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6eebcf, 90 },
    { 0xcc7eebd0, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b7eebd6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2eebe1, 90 },
    { 0, 0 },
    { 0x4eeeebe3, 97 },
    { 0x629eebe4, 33 },
    { 0x276eebe5, 90 },
    { 0x9f0eebe4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466eebec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60eebf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818eebf8, 99 },
    { 0xbe4eebf9, 34 },
    { 0xa07eebfa, 23 },
    { 0xb7beebfb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242eec00, 98 },
    { 0, 0 },
    { 0x961eec02, 98 },
    { 0xd8ceec03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5feec06, 23 },
    { 0, 0 },
    { 0xccfeec08, 76 },
    { 0, 0 },
    { 0xd90eec0a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79eec0f, 78 },
    { 0x4c9eec10, 34 },
    { 0xa9feec11, 61 },
    { 0, 0 },
    { 0x12aeec13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2eec17, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12eec1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feeec26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39eec2c, 93 },
    { 0x82deec2d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49eec33, 33 },
    { 0x5d0eec34, 33 },
    { 0x75beec35, 51 },
    { 0x5f2eec36, 91 },
    { 0x188eec36, 23 },
    { 0xe3feec36, 61 },
    { 0x4dbeec36, 95 },
    { 0x191eec38, 99 },
    { 0, 0 },
    { 0xd74eec3c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x68eec3f, 23 },
    { 0, 0 },
    { 0x8c5eec41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cceec47, 40 },
    { 0xe8feec48, 99 },
    { 0xad0eec49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x975eec50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11beec57, 51 },
    { 0x5f2eec58, 33 },
    { 0x603eec58, 92 },
    { 0xcbfeec5a, 95 },
    { 0xcceeec59, 98 },
    { 0, 0 },
    { 0x504eec5d, 80 },
    { 0, 0 },
    { 0x84deec5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaeec62, 33 },
    { 0xeaeeec63, 52 },
    { 0, 0 },
    { 0x4b3eec65, 81 },
    { 0xf6eeec66, 100 },
    { 0, 0 },
    { 0x17deec68, 98 },
    { 0, 0 },
    { 0x17deec6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedeec6d, 43 },
    { 0, 0 },
    { 0x4c9eec6f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32feec76, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eeec7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbceec81, 88 },
    { 0x900eec82, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7eec88, 100 },
    { 0, 0 },
    { 0x8deeec8a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0eec8d, 61 },
    { 0x1dceec8d, 61 },
    { 0xa0beec8f, 90 },
    { 0xc3eec90, 85 },
    { 0xda7eec8e, 98 },
    { 0xd44eec92, 90 },
    { 0x95beec8e, 61 },
    { 0x4e8eec94, 23 },
    { 0xec9eec91, 90 },
    { 0, 0 },
    { 0xc81eec97, 33 },
    { 0xa1deec97, 67 },
    { 0, 0 },
    { 0x829eec9a, 90 },
    { 0, 0 },
    { 0x317eec9c, 33 },
    { 0, 0 },
    { 0x90beec9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42feeca2, 33 },
    { 0, 0 },
    { 0x732eeca4, 55 },
    { 0, 0 },
    { 0x26ceeca6, 90 },
    { 0xc2deeca7, 90 },
    { 0x21beeca7, 87 },
    { 0, 0 },
    { 0x88eecaa, 61 },
    { 0, 0 },
    { 0x48deecac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5eecb0, 76 },
    { 0x773eecb1, 17 },
    { 0xbbfeecb1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaceecbb, 61 },
    { 0x2d5eecbb, 90 },
    { 0x34ceecbd, 90 },
    { 0x673eecbe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813eecce, 95 },
    { 0, 0 },
    { 0x801eecd0, 95 },
    { 0xee9eecd1, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5eecd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7eecd7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5eecda, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7eecde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0eece1, 43 },
    { 0xef6eece2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6beecea, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdeeced, 76 },
    { 0xa82eecee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8eecf4, 61 },
    { 0x862eecf5, 95 },
    { 0xc5deecf4, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baeecfe, 56 },
    { 0, 0 },
    { 0xa29eed00, 23 },
    { 0xbb4eed01, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93feed04, 99 },
    { 0x661eed05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0feed08, 95 },
    { 0, 0 },
    { 0x29beed0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0eed0e, 55 },
    { 0xd8eed0e, 66 },
    { 0x91aeed10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41feed1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34eed1d, 91 },
    { 0x81eed1d, 90 },
    { 0, 0 },
    { 0xa14eed20, 95 },
    { 0x418eed21, 43 },
    { 0xac3eed22, 85 },
    { 0xa90eed22, 66 },
    { 0, 0 },
    { 0xd3beed25, 76 },
    { 0, 0 },
    { 0xdcbeed27, 61 },
    { 0, 0 },
    { 0xd94eed29, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deed2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbeed30, 23 },
    { 0xf60eed30, 76 },
    { 0x4a2eed31, 33 },
    { 0x203eed33, 33 },
    { 0xd6feed33, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8eed37, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11feed3b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaeed3f, 43 },
    { 0, 0 },
    { 0xd4deed41, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9eed4d, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31eed50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4deed55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x843eed58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7eed5d, 80 },
    { 0, 0 },
    { 0x2f7eed5f, 61 },
    { 0xe9eeed60, 61 },
    { 0x4dfeed61, 23 },
    { 0x77feed60, 99 },
    { 0, 0 },
    { 0xdfeed64, 65 },
    { 0xcdfeed64, 98 },
    { 0x311eed66, 76 },
    { 0, 0 },
    { 0xe9deed68, 99 },
    { 0xe69eed69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdeed6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773eed71, 84 },
    { 0x234eed72, 76 },
    { 0x290eed73, 100 },
    { 0xa4feed74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2eed7a, 55 },
    { 0x119eed7b, 95 },
    { 0x62eed7a, 77 },
    { 0, 0 },
    { 0x55eeed7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x330eed81, 61 },
    { 0x2cceed81, 99 },
    { 0xf92eed83, 90 },
    { 0, 0 },
    { 0x1b2eed85, 43 },
    { 0xc71eed86, 34 },
    { 0xc34eed86, 23 },
    { 0x609eed88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x895eed8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80feed8e, 34 },
    { 0x5ceeed8f, 69 },
    { 0xa86eed8e, 34 },
    { 0xfdfeed8e, 90 },
    { 0x1fbeed90, 55 },
    { 0x34feed93, 95 },
    { 0x226eed94, 90 },
    { 0x7ddeed95, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7eed99, 61 },
    { 0xf31eed9a, 33 },
    { 0xbcaeed9a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53eeda1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0eeda4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aeeda7, 87 },
    { 0, 0 },
    { 0x223eeda9, 93 },
    { 0, 0 },
    { 0xcadeedab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0beedb0, 61 },
    { 0xc63eedb1, 55 },
    { 0xb64eedb2, 33 },
    { 0x8d2eedb3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x97beedb6, 90 },
    { 0x1d4eedb6, 75 },
    { 0, 0 },
    { 0xde3eedb9, 93 },
    { 0, 0 },
    { 0xd9ceedbb, 95 },
    { 0x8dbeedbb, 61 },
    { 0x35feedbd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2eedc4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7beedca, 95 },
    { 0, 0 },
    { 0xc11eedcc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x687eedcf, 76 },
    { 0, 0 },
    { 0x147eedd1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28eedd5, 77 },
    { 0xb50eedd5, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x515eedd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x202eeddf, 52 },
    { 0, 0 },
    { 0x481eede1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaceede4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9eede8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18eedec, 90 },
    { 0x317eeded, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248eedf3, 43 },
    { 0xb6eedf4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdeedfa, 56 },
    { 0xd05eedfb, 98 },
    { 0x10feedfb, 43 },
    { 0xc5aeedfd, 55 },
    { 0, 0 },
    { 0x6d0eedff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6daeee08, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x904eee0b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56feee0f, 55 },
    { 0xaf0eee10, 97 },
    { 0x29eeee10, 98 },
    { 0, 0 },
    { 0x13feee13, 98 },
    { 0x293eee14, 97 },
    { 0x589eee14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ceee1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1eee1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8beee21, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x177eee24, 74 },
    { 0x1bdeee25, 98 },
    { 0xa47eee26, 76 },
    { 0, 0 },
    { 0x26eeee28, 98 },
    { 0x2afeee28, 90 },
    { 0, 0 },
    { 0x74aeee2b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7deee36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325eee3d, 52 },
    { 0x9e6eee3e, 43 },
    { 0x312eee3f, 95 },
    { 0x255eee3f, 90 },
    { 0xcd3eee3d, 43 },
    { 0x19eeee3f, 55 },
    { 0, 0 },
    { 0x11beee44, 42 },
    { 0, 0 },
    { 0x32aeee46, 98 },
    { 0x37beee47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ceee4a, 61 },
    { 0x2dceee4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x147eee4f, 33 },
    { 0, 0 },
    { 0xd6aeee51, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44eee55, 95 },
    { 0, 0 },
    { 0xfb4eee57, 76 },
    { 0x75feee58, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bceee5b, 95 },
    { 0, 0 },
    { 0x2dbeee5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4eee60, 78 },
    { 0xc0deee60, 95 },
    { 0x24feee60, 33 },
    { 0, 0 },
    { 0x6f1eee64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0eeee67, 43 },
    { 0xe0aeee68, 87 },
    { 0, 0 },
    { 0x300eee6a, 55 },
    { 0x46eeee6b, 88 },
    { 0x48deee6a, 69 },
    { 0x19eeee6d, 100 },
    { 0x3f8eee6c, 77 },
    { 0xad5eee6f, 95 },
    { 0x540eee6c, 98 },
    { 0x3edeee71, 23 },
    { 0, 0 },
    { 0xdfeee73, 95 },
    { 0xc3eee74, 95 },
    { 0xaf4eee75, 99 },
    { 0xddfeee74, 91 },
    { 0xc6eee77, 98 },
    { 0x72deee78, 61 },
    { 0x5f3eee74, 61 },
    { 0x3fbeee7a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfeee7e, 90 },
    { 0x622eee7e, 61 },
    { 0x76beee80, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0x857eee83, 90 },
    { 0, 0 },
    { 0xc79eee85, 23 },
    { 0xe27eee86, 78 },
    { 0, 0 },
    { 0x9b5eee88, 76 },
    { 0, 0 },
    { 0xdbeee8a, 61 },
    { 0x70eee8b, 90 },
    { 0x100eee8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ceee8f, 43 },
    { 0, 0 },
    { 0x9e4eee91, 98 },
    { 0, 0 },
    { 0x249eee93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aeee9b, 95 },
    { 0, 0 },
    { 0x5a9eee9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd17eeea0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69eeea9, 61 },
    { 0xfb9eeeaa, 85 },
    { 0x764eeeab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4deeeb1, 61 },
    { 0, 0 },
    { 0xc9aeeeb3, 81 },
    { 0, 0 },
    { 0x500eeeb5, 90 },
    { 0xebbeeeb6, 100 },
    { 0x335eeeb7, 43 },
    { 0x913eeeb7, 46 },
    { 0x6b2eeeb9, 85 },
    { 0x933eeeb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0eeec3, 61 },
    { 0x699eeec3, 90 },
    { 0x966eeec3, 55 },
    { 0xc2feeec5, 76 },
    { 0x9f2eeec3, 23 },
    { 0x51eeec8, 33 },
    { 0, 0 },
    { 0xb16eeeca, 91 },
    { 0x8e1eeecb, 34 },
    { 0xe90eeecc, 61 },
    { 0xe6aeeecb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103eeed2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5eeed6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454eeedc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825eeee7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6eeeeb, 61 },
    { 0x24feeeec, 66 },
    { 0xc4beeeed, 95 },
    { 0x788eeeed, 34 },
    { 0xd01eeeef, 76 },
    { 0x6dceeef0, 77 },
    { 0, 0 },
    { 0x134eeef2, 61 },
    { 0x4faeeef3, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706eeefb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x413eeefe, 90 },
    { 0xcffeeeff, 90 },
    { 0x13aeeeff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e7eef03, 55 },
    { 0x214eef03, 87 },
    { 0x97aeef05, 23 },
    { 0x978eef06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189eef0e, 85 },
    { 0, 0 },
    { 0x9f6eef10, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88eeef1e, 43 },
    { 0x188eef1f, 95 },
    { 0x905eef20, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3eef23, 23 },
    { 0, 0 },
    { 0xd4eef25, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x56beef28, 100 },
    { 0, 0 },
    { 0x8aeef2a, 33 },
    { 0, 0 },
    { 0x25ceef2c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0eef32, 100 },
    { 0, 0 },
    { 0xeddeef34, 46 },
    { 0, 0 },
    { 0x6fbeef36, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199eef3b, 61 },
    { 0xa3beef3c, 61 },
    { 0x860eef3d, 34 },
    { 0x13beef3e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec1eef42, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89eeef48, 65 },
    { 0xc67eef48, 67 },
    { 0xa7ceef49, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaeeef4d, 55 },
    { 0x115eef4d, 34 },
    { 0x7bfeef4d, 90 },
    { 0x27ceef50, 61 },
    { 0xab0eef50, 76 },
    { 0x9b4eef51, 98 },
    { 0x9a9eef53, 61 },
    { 0x10beef4e, 98 },
    { 0x8adeef4e, 43 },
    { 0xb5ceef50, 90 },
    { 0x898eef4d, 55 },
    { 0, 0 },
    { 0x431eef59, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6eef61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30feef65, 61 },
    { 0x36eef66, 90 },
    { 0xbbceef65, 90 },
    { 0x4e1eef68, 76 },
    { 0xc57eef65, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2eef6c, 43 },
    { 0xa87eef6d, 90 },
    { 0x1c1eef6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636eef73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2eef7b, 90 },
    { 0xe00eef7c, 101 },
    { 0xb6ceef7d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97aeef81, 91 },
    { 0xccbeef82, 61 },
    { 0xba0eef83, 61 },
    { 0xcc7eef84, 95 },
    { 0x8b7eef84, 61 },
    { 0x3cfeef86, 61 },
    { 0x234eef81, 98 },
    { 0xbe2eef83, 66 },
    { 0xc8eeef89, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02eef8f, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94eeef95, 33 },
    { 0, 0 },
    { 0x52deef97, 29 },
    { 0, 0 },
    { 0xf1beef99, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4eef9e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8eeefa1, 33 },
    { 0x4d5eefa2, 33 },
    { 0x5f2eefa3, 61 },
    { 0xca6eefa4, 90 },
    { 0x156eefa4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445eefac, 67 },
    { 0xb5ceefad, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x879eefb3, 55 },
    { 0x191eefb4, 67 },
    { 0xcf8eefb3, 66 },
    { 0x77aeefb3, 23 },
    { 0x23ceefb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x227eefba, 93 },
    { 0, 0 },
    { 0xec5eefbc, 33 },
    { 0, 0 },
    { 0x400eefbe, 61 },
    { 0x370eefbf, 90 },
    { 0x110eefbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659eefc4, 37 },
    { 0xeaeeefc5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x806eefc8, 61 },
    { 0xba1eefc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881eefcf, 55 },
    { 0x1e7eefd0, 98 },
    { 0x4adeefcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x624eefd4, 61 },
    { 0, 0 },
    { 0xa9feefd6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eefdb, 93 },
    { 0x2d4eefdc, 98 },
    { 0xbbaeefdc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabeefe0, 90 },
    { 0xaefeefe1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x883eefe7, 40 },
    { 0xc79eefe8, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10deeff2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7eeff9, 55 },
    { 0xfc6eeff9, 33 },
    { 0x25eeff9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abef003, 99 },
    { 0xd50ef004, 72 },
    { 0x359ef004, 55 },
    { 0x972ef004, 23 },
    { 0x907ef007, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7ef00b, 75 },
    { 0, 0 },
    { 0xaa2ef00d, 96 },
    { 0, 0 },
    { 0x8c5ef00f, 90 },
    { 0xb6bef010, 76 },
    { 0x5acef011, 23 },
    { 0xe9fef012, 23 },
    { 0, 0 },
    { 0xf1eef014, 95 },
    { 0, 0 },
    { 0x201ef016, 66 },
    { 0xf28ef017, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x697ef020, 23 },
    { 0x9d9ef021, 61 },
    { 0, 0 },
    { 0xb14ef023, 95 },
    { 0, 0 },
    { 0x22ef025, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bef031, 90 },
    { 0xc6fef031, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49ef03a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17ef03e, 46 },
    { 0x97eef03e, 34 },
    { 0x9e2ef03f, 33 },
    { 0x845ef041, 55 },
    { 0xa15ef042, 55 },
    { 0x47bef043, 23 },
    { 0xc26ef044, 77 },
    { 0x199ef03f, 66 },
    { 0x4b0ef046, 94 },
    { 0x327ef045, 34 },
    { 0xdeef048, 90 },
    { 0x757ef047, 90 },
    { 0x22fef03e, 23 },
    { 0xde8ef04b, 76 },
    { 0x811ef03f, 61 },
    { 0x1ddef04d, 61 },
    { 0xaafef04e, 55 },
    { 0x99def04e, 43 },
    { 0x57ef04d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6ef055, 95 },
    { 0x2ffef056, 77 },
    { 0x4d3ef057, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68fef05b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aeef05f, 55 },
    { 0, 0 },
    { 0x3eef061, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87def06f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9aef077, 61 },
    { 0, 0 },
    { 0x4acef079, 78 },
    { 0, 0 },
    { 0x2d4ef07b, 43 },
    { 0xcb0ef07c, 78 },
    { 0xe31ef07c, 99 },
    { 0xf75ef07d, 72 },
    { 0x798ef07f, 61 },
    { 0x2aeef07f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x209ef083, 95 },
    { 0xf63ef084, 98 },
    { 0, 0 },
    { 0xa9def086, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280ef08a, 100 },
    { 0xb77ef08a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23bef091, 90 },
    { 0x235ef091, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ef095, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ef09a, 61 },
    { 0xca5ef09b, 90 },
    { 0x403ef09c, 98 },
    { 0x8cef09a, 90 },
    { 0xcabef09e, 90 },
    { 0xed7ef09b, 61 },
    { 0x24def0a0, 90 },
    { 0xab0ef0a1, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddef0a6, 90 },
    { 0x9f5ef0a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x205ef0aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcef0ae, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2ef0b6, 87 },
    { 0xe3eef0b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cef0bf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x13fef0c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42def0c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaef0cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7ef0d3, 98 },
    { 0xce6ef0d4, 100 },
    { 0x890ef0d5, 34 },
    { 0x93ef0d4, 34 },
    { 0x4a9ef0d7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bef0da, 55 },
    { 0x39fef0da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98def0e0, 87 },
    { 0x93ef0e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5ef0e6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88ef0e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476ef0ef, 23 },
    { 0xb15ef0f0, 90 },
    { 0xfecef0ef, 66 },
    { 0xec2ef0ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95ef0fa, 61 },
    { 0x387ef0fa, 99 },
    { 0x24bef0fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x851ef102, 99 },
    { 0x954ef103, 87 },
    { 0, 0 },
    { 0x9b2ef105, 33 },
    { 0, 0 },
    { 0xce4ef107, 92 },
    { 0, 0 },
    { 0xa67ef109, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95ef10c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34ef10f, 55 },
    { 0, 0 },
    { 0x81bef111, 76 },
    { 0, 0 },
    { 0xd16ef113, 90 },
    { 0, 0 },
    { 0x420ef115, 78 },
    { 0x486ef116, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcef119, 17 },
    { 0x666ef11a, 23 },
    { 0xff3ef119, 100 },
    { 0x627ef11b, 99 },
    { 0xc96ef11d, 91 },
    { 0, 0 },
    { 0x81def11f, 90 },
    { 0x4d0ef120, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7ef123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2def129, 61 },
    { 0xf31ef12a, 52 },
    { 0x48fef129, 43 },
    { 0, 0 },
    { 0x360ef12d, 23 },
    { 0x9fef12d, 61 },
    { 0, 0 },
    { 0xf2eef130, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cef135, 78 },
    { 0xaa9ef136, 98 },
    { 0x605ef137, 76 },
    { 0xc8bef137, 76 },
    { 0xfbfef139, 90 },
    { 0x435ef139, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1ef13d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdef140, 99 },
    { 0xba5ef141, 64 },
    { 0xe7ef142, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ef147, 40 },
    { 0, 0 },
    { 0x3e9ef149, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ef14e, 23 },
    { 0x85fef14f, 100 },
    { 0, 0 },
    { 0x9aef151, 23 },
    { 0, 0 },
    { 0xafef153, 100 },
    { 0xbf6ef154, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x332ef157, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4ef15b, 23 },
    { 0xa5bef15b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5ef15f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccef163, 33 },
    { 0xe3aef164, 61 },
    { 0xa50ef164, 55 },
    { 0x6a9ef164, 34 },
    { 0x589ef167, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ef16f, 95 },
    { 0x86ef170, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ef174, 90 },
    { 0xfe8ef175, 67 },
    { 0xc55ef176, 61 },
    { 0x783ef177, 90 },
    { 0x5d4ef177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131ef17d, 38 },
    { 0, 0 },
    { 0xb8bef17f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6ef182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc20ef185, 98 },
    { 0, 0 },
    { 0x358ef187, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02ef18a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05ef18e, 23 },
    { 0x33fef18f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cef198, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x393ef19b, 85 },
    { 0, 0 },
    { 0x7d7ef19d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83ef1a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6def1a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1ef1a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x406ef1aa, 75 },
    { 0xf3cef1aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13def1b1, 23 },
    { 0, 0 },
    { 0xf84ef1b3, 61 },
    { 0xe05ef1b3, 46 },
    { 0x362ef1b3, 90 },
    { 0, 0 },
    { 0x61def1b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bef1ba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ef1bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8fef1c2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65ef1c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4ef1cf, 95 },
    { 0xd11ef1d0, 80 },
    { 0x6e3ef1d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cef1d5, 43 },
    { 0, 0 },
    { 0xea8ef1d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6ef1df, 23 },
    { 0, 0 },
    { 0x8d5ef1e1, 34 },
    { 0, 0 },
    { 0xddfef1e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef1ef, 95 },
    { 0x214ef1f0, 95 },
    { 0x5fef1f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccef1f6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bef1fb, 33 },
    { 0x583ef1fb, 90 },
    { 0x3ebef1fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x808ef200, 76 },
    { 0x48eef201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebfef206, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cef209, 34 },
    { 0, 0 },
    { 0x753ef20b, 65 },
    { 0x29fef20c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02ef210, 96 },
    { 0, 0 },
    { 0xf5cef212, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39ef218, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbeef21b, 33 },
    { 0, 0 },
    { 0x65eef21d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1eef222, 90 },
    { 0, 0 },
    { 0x21cef224, 67 },
    { 0x501ef225, 76 },
    { 0, 0 },
    { 0xcc2ef227, 76 },
    { 0, 0 },
    { 0x8e8ef229, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fef22e, 23 },
    { 0, 0 },
    { 0x132ef230, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35ef234, 55 },
    { 0xeffef235, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaeef238, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01ef23b, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11ef23f, 91 },
    { 0x505ef240, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03ef24c, 33 },
    { 0, 0 },
    { 0x49eef24e, 23 },
    { 0, 0 },
    { 0xd3eef250, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2ef253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x798ef256, 100 },
    { 0x762ef256, 76 },
    { 0, 0 },
    { 0x282ef259, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0ef260, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccef264, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cfef26e, 55 },
    { 0, 0 },
    { 0xfa9ef270, 98 },
    { 0x5ddef270, 98 },
    { 0, 0 },
    { 0x96def273, 23 },
    { 0x934ef274, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38fef27a, 98 },
    { 0, 0 },
    { 0x13fef27c, 33 },
    { 0xe9bef27d, 90 },
    { 0x967ef27e, 55 },
    { 0x58eef27e, 34 },
    { 0xd76ef27d, 61 },
    { 0x9c2ef27c, 76 },
    { 0x76aef27e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdef285, 46 },
    { 0xa39ef285, 23 },
    { 0x99cef286, 52 },
    { 0, 0 },
    { 0x11cef289, 90 },
    { 0, 0 },
    { 0xda2ef28b, 90 },
    { 0x675ef28c, 55 },
    { 0, 0 },
    { 0xa2fef28e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef291, 61 },
    { 0, 0 },
    { 0x4acef293, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ceef296, 43 },
    { 0x153ef297, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cef29a, 90 },
    { 0x4eef29b, 90 },
    { 0xdb1ef29b, 95 },
    { 0x957ef29c, 98 },
    { 0x324ef29b, 98 },
    { 0xd17ef29c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6eef2a3, 78 },
    { 0xf43ef2a4, 90 },
    { 0xdd1ef2a5, 95 },
    { 0xeacef2a3, 95 },
    { 0x1c7ef2a6, 99 },
    { 0x250ef2a8, 76 },
    { 0xb1bef2a5, 98 },
    { 0xa2aef2aa, 95 },
    { 0x90fef2a6, 66 },
    { 0xe40ef2ac, 61 },
    { 0xca2ef2aa, 95 },
    { 0x7deef2ae, 34 },
    { 0, 0 },
    { 0x764ef2b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987ef2b8, 33 },
    { 0x2fef2b9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef2bc, 23 },
    { 0xaa6ef2bd, 55 },
    { 0, 0 },
    { 0x14aef2bf, 34 },
    { 0xd9def2bf, 33 },
    { 0, 0 },
    { 0x594ef2c2, 93 },
    { 0x66aef2c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ef2cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def2d3, 88 },
    { 0x43ef2d4, 95 },
    { 0x1b6ef2d5, 43 },
    { 0x9aef2d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91fef2db, 91 },
    { 0, 0 },
    { 0x474ef2dd, 43 },
    { 0, 0 },
    { 0x8eef2df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4ef2e6, 98 },
    { 0xd17ef2e7, 100 },
    { 0, 0 },
    { 0x7c0ef2e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafef2ef, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aef2f5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x228ef2f8, 33 },
    { 0xa82ef2f8, 43 },
    { 0, 0 },
    { 0xe56ef2fb, 55 },
    { 0, 0 },
    { 0xd6ef2fd, 61 },
    { 0xd3aef2fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fef307, 91 },
    { 0, 0 },
    { 0xfa1ef309, 23 },
    { 0, 0 },
    { 0x7d6ef30b, 33 },
    { 0x17aef30c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286ef314, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaef317, 55 },
    { 0x783ef317, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdef325, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54ef32b, 94 },
    { 0xe67ef32c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8ef330, 33 },
    { 0, 0 },
    { 0xa9cef332, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345ef337, 33 },
    { 0xe41ef337, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb93ef33b, 48 },
    { 0xb85ef33b, 66 },
    { 0xc81ef33b, 87 },
    { 0xaa7ef33b, 61 },
    { 0x754ef33c, 61 },
    { 0xb61ef33c, 33 },
    { 0xad3ef341, 33 },
    { 0x3ccef342, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0ef349, 43 },
    { 0, 0 },
    { 0x4f6ef34b, 61 },
    { 0, 0 },
    { 0x76def34d, 93 },
    { 0, 0 },
    { 0xa3bef34f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ef358, 90 },
    { 0x18fef359, 61 },
    { 0xc8bef358, 84 },
    { 0xcc6ef35b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09ef363, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50ef367, 33 },
    { 0x59aef368, 33 },
    { 0x941ef369, 23 },
    { 0xaa9ef368, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x134ef36d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5feef371, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577ef376, 33 },
    { 0x6b3ef376, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a3ef37c, 55 },
    { 0, 0 },
    { 0xe63ef37e, 95 },
    { 0xe0aef37f, 33 },
    { 0xe33ef380, 95 },
    { 0x32bef381, 100 },
    { 0x2a9ef382, 95 },
    { 0, 0 },
    { 0x697ef384, 87 },
    { 0, 0 },
    { 0xa01ef386, 90 },
    { 0x305ef387, 43 },
    { 0x3d9ef388, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbef390, 98 },
    { 0x3e2ef391, 55 },
    { 0, 0 },
    { 0x36eef393, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdef39f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7ef3a8, 61 },
    { 0x20cef3a9, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8ef3ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6ef3af, 61 },
    { 0xb75ef3b0, 90 },
    { 0, 0 },
    { 0x27ef3b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef3b6, 67 },
    { 0x125ef3b7, 46 },
    { 0xecef3b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5ef3bb, 23 },
    { 0, 0 },
    { 0xee4ef3bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4ef3c0, 33 },
    { 0, 0 },
    { 0x563ef3c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7ef3c9, 61 },
    { 0xc19ef3ca, 88 },
    { 0x93ef3c9, 61 },
    { 0x238ef3cc, 98 },
    { 0xc55ef3cc, 33 },
    { 0xb38ef3cd, 98 },
    { 0x84eef3c9, 55 },
    { 0x47aef3c9, 23 },
    { 0x7d6ef3cb, 23 },
    { 0x9aef3d2, 34 },
    { 0x5e1ef3d2, 98 },
    { 0xd9aef3d3, 43 },
    { 0x2d4ef3d5, 91 },
    { 0, 0 },
    { 0xaa7ef3d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5ef3dd, 23 },
    { 0x988ef3de, 46 },
    { 0x567ef3df, 43 },
    { 0xdeaef3df, 91 },
    { 0xa11ef3e1, 55 },
    { 0x1ffef3e2, 85 },
    { 0xa21ef3e0, 23 },
    { 0x4c3ef3e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8ef3e8, 95 },
    { 0x828ef3e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x899ef3ec, 92 },
    { 0, 0 },
    { 0x6abef3ee, 61 },
    { 0x184ef3ee, 33 },
    { 0x15ef3f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa75ef3f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70ef3fb, 61 },
    { 0, 0 },
    { 0x9c4ef3fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5ef402, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a0ef409, 98 },
    { 0, 0 },
    { 0xd7eef40b, 98 },
    { 0, 0 },
    { 0x5f3ef40d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ef412, 61 },
    { 0, 0 },
    { 0x3a8ef414, 55 },
    { 0x7bfef414, 23 },
    { 0x2e0ef416, 43 },
    { 0x186ef417, 43 },
    { 0x207ef415, 99 },
    { 0x7e2ef419, 90 },
    { 0x17aef41a, 36 },
    { 0, 0 },
    { 0xf06ef41c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23ef424, 32 },
    { 0, 0 },
    { 0xa3fef426, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9aef430, 91 },
    { 0x20eef430, 33 },
    { 0xf49ef432, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294ef436, 95 },
    { 0xb97ef437, 33 },
    { 0x26def438, 98 },
    { 0x52bef436, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x884ef43f, 52 },
    { 0x6a7ef43f, 92 },
    { 0x333ef441, 23 },
    { 0x150ef442, 66 },
    { 0x19cef443, 98 },
    { 0xfb5ef444, 64 },
    { 0xb5def43f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x515ef44f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4ef455, 90 },
    { 0, 0 },
    { 0x2aaef457, 95 },
    { 0x609ef457, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce5ef45b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214ef463, 33 },
    { 0x7f4ef464, 77 },
    { 0x6d1ef464, 76 },
    { 0x70fef465, 33 },
    { 0xc2bef467, 77 },
    { 0x179ef467, 95 },
    { 0xe20ef468, 61 },
    { 0x33aef46a, 89 },
    { 0, 0 },
    { 0xb18ef46c, 66 },
    { 0xf91ef46c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bef470, 67 },
    { 0, 0 },
    { 0x16eef472, 33 },
    { 0x11fef472, 23 },
    { 0, 0 },
    { 0x8fcef475, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51ef478, 34 },
    { 0, 0 },
    { 0x94cef47a, 33 },
    { 0, 0 },
    { 0x9d4ef47c, 95 },
    { 0, 0 },
    { 0x3bcef47e, 81 },
    { 0x2f8ef47e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23ef483, 33 },
    { 0, 0 },
    { 0xcefef485, 23 },
    { 0, 0 },
    { 0x297ef487, 40 },
    { 0xc95ef487, 76 },
    { 0x731ef489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37ef497, 98 },
    { 0x5cfef498, 33 },
    { 0x8d8ef499, 33 },
    { 0x544ef49a, 84 },
    { 0xca3ef49a, 33 },
    { 0xd8bef49c, 61 },
    { 0x927ef49a, 90 },
    { 0xf36ef49e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x13def4a1, 61 },
    { 0x9daef4a2, 90 },
    { 0, 0 },
    { 0x54fef4a4, 76 },
    { 0, 0 },
    { 0xc62ef4a6, 90 },
    { 0x98ef4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61eef4aa, 85 },
    { 0, 0 },
    { 0x552ef4ac, 66 },
    { 0x9aef4ad, 61 },
    { 0xae7ef4ad, 90 },
    { 0, 0 },
    { 0x940ef4b0, 90 },
    { 0x11ef4b1, 23 },
    { 0x278ef4b2, 33 },
    { 0, 0 },
    { 0x39def4b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xabeef4b7, 33 },
    { 0x819ef4b8, 51 },
    { 0x2a8ef4b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5ef4bc, 95 },
    { 0xd8bef4bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x805ef4c4, 33 },
    { 0x912ef4c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9ef4cb, 75 },
    { 0x896ef4cc, 23 },
    { 0xec8ef4cd, 91 },
    { 0x14fef4ce, 99 },
    { 0x3a8ef4cf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1ef4d3, 98 },
    { 0x457ef4d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8eef4d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ef4de, 33 },
    { 0xa39ef4de, 33 },
    { 0, 0 },
    { 0x59fef4e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x618ef4e5, 55 },
    { 0, 0 },
    { 0x7b1ef4e7, 90 },
    { 0x220ef4e8, 61 },
    { 0x31aef4e8, 90 },
    { 0x7aeef4ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0def4f1, 61 },
    { 0xd3ef4f2, 90 },
    { 0x791ef4f3, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3eef4f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ef4fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2ef4ff, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x843ef503, 33 },
    { 0x637ef503, 55 },
    { 0, 0 },
    { 0x85eef506, 100 },
    { 0x799ef507, 61 },
    { 0x813ef506, 76 },
    { 0, 0 },
    { 0xd37ef50a, 38 },
    { 0x8fbef50b, 98 },
    { 0xcf7ef50b, 23 },
    { 0, 0 },
    { 0x314ef50e, 61 },
    { 0x8fdef50e, 23 },
    { 0, 0 },
    { 0x371ef511, 23 },
    { 0, 0 },
    { 0xfdbef513, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5ef518, 46 },
    { 0, 0 },
    { 0x25def51a, 23 },
    { 0, 0 },
    { 0x669ef51c, 46 },
    { 0x29bef51c, 33 },
    { 0xf2fef51e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1def522, 61 },
    { 0, 0 },
    { 0xa71ef524, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b4ef527, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45aef52f, 81 },
    { 0, 0 },
    { 0x965ef531, 23 },
    { 0x3c3ef531, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bef536, 61 },
    { 0xde2ef537, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x603ef53a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc29ef53d, 43 },
    { 0, 0 },
    { 0xcb1ef53f, 91 },
    { 0x4cdef540, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bef543, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x350ef546, 98 },
    { 0x3e1ef546, 17 },
    { 0xdf2ef548, 43 },
    { 0, 0 },
    { 0x6a0ef54a, 23 },
    { 0xb45ef54b, 66 },
    { 0x482ef54c, 76 },
    { 0, 0 },
    { 0x10aef54e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78ef551, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7ef55a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfef55f, 23 },
    { 0, 0 },
    { 0x956ef561, 23 },
    { 0xb2aef562, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ef56b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ef56e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557ef572, 95 },
    { 0xd46ef573, 84 },
    { 0xa3fef573, 23 },
    { 0, 0 },
    { 0x6a1ef576, 43 },
    { 0xcfeef577, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9ef57d, 55 },
    { 0, 0 },
    { 0xbaaef57f, 33 },
    { 0, 0 },
    { 0xdceef581, 43 },
    { 0, 0 },
    { 0xe0cef583, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cef588, 90 },
    { 0x70ef588, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef58e, 34 },
    { 0, 0 },
    { 0xea9ef590, 95 },
    { 0, 0 },
    { 0xde1ef592, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd06ef597, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71ef5a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedef5ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64ef5b0, 61 },
    { 0x906ef5b1, 33 },
    { 0x7b5ef5b0, 81 },
    { 0xa10ef5b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cef5b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2ef5bb, 90 },
    { 0x733ef5bb, 61 },
    { 0x434ef5bb, 90 },
    { 0, 0 },
    { 0xcedef5bf, 76 },
    { 0, 0 },
    { 0x3d4ef5c1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4eef5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6aef5c9, 78 },
    { 0x683ef5c9, 46 },
    { 0x34cef5cb, 90 },
    { 0xdf8ef5cc, 95 },
    { 0x619ef5ca, 55 },
    { 0x7eaef5ca, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2ef5d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbef5d5, 90 },
    { 0, 0 },
    { 0xb45ef5d7, 46 },
    { 0x235ef5d7, 78 },
    { 0, 0 },
    { 0x7afef5da, 61 },
    { 0xb16ef5db, 33 },
    { 0x4dfef5da, 96 },
    { 0x37aef5dc, 43 },
    { 0xc1cef5da, 99 },
    { 0x24eef5dd, 101 },
    { 0x598ef5e0, 90 },
    { 0xe64ef5e1, 90 },
    { 0x679ef5e2, 61 },
    { 0x2ecef5e2, 74 },
    { 0, 0 },
    { 0xb8ef5e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efef5e8, 34 },
    { 0x826ef5e8, 33 },
    { 0xd1fef5ea, 76 },
    { 0, 0 },
    { 0x3e0ef5ec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddeef5f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95ef5f3, 34 },
    { 0x87bef5f3, 61 },
    { 0x901ef5f3, 23 },
    { 0xd10ef5f3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ef5f9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2aef5fd, 23 },
    { 0x92ef5fd, 100 },
    { 0x693ef5ff, 23 },
    { 0x36aef5fd, 90 },
    { 0x7b8ef600, 90 },
    { 0xa9bef602, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x300ef605, 91 },
    { 0xd3ef606, 33 },
    { 0x249ef607, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ef60a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89ef60f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60ef614, 55 },
    { 0, 0 },
    { 0xfd3ef616, 43 },
    { 0x706ef617, 55 },
    { 0xeaeef616, 98 },
    { 0x1e9ef618, 98 },
    { 0xbd2ef61a, 90 },
    { 0xc5cef61b, 66 },
    { 0, 0 },
    { 0x440ef61d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ffef623, 33 },
    { 0xf00ef623, 17 },
    { 0, 0 },
    { 0xf5eef626, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x606ef629, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7ef62c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccef631, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x803ef634, 23 },
    { 0, 0 },
    { 0x142ef636, 61 },
    { 0, 0 },
    { 0xe04ef638, 23 },
    { 0x685ef639, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70ef63e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e8ef648, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bef64d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155ef652, 66 },
    { 0x3acef652, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef656, 61 },
    { 0, 0 },
    { 0x6f6ef658, 75 },
    { 0x272ef659, 33 },
    { 0x50ef659, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x16eef65d, 55 },
    { 0xa51ef65e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x290ef661, 52 },
    { 0x864ef662, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x216ef665, 33 },
    { 0, 0 },
    { 0xbc4ef667, 76 },
    { 0x777ef668, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x346ef66b, 99 },
    { 0xcfcef66c, 91 },
    { 0x1a7ef66c, 61 },
    { 0x98eef66e, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x1beef671, 59 },
    { 0x68cef672, 33 },
    { 0xa47ef673, 98 },
    { 0, 0 },
    { 0xefef675, 90 },
    { 0x3fef676, 101 },
    { 0, 0 },
    { 0x682ef678, 34 },
    { 0xf13ef679, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x409ef68a, 76 },
    { 0x1e5ef68b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x309ef68e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8ef692, 90 },
    { 0xcacef692, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7ef697, 99 },
    { 0x11fef697, 55 },
    { 0xf65ef698, 53 },
    { 0, 0 },
    { 0xa94ef69b, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78def6a1, 34 },
    { 0xb3ef6a2, 87 },
    { 0xbd4ef6a2, 90 },
    { 0, 0 },
    { 0x63bef6a5, 69 },
    { 0x519ef6a6, 23 },
    { 0xc97ef6a6, 84 },
    { 0x63fef6a5, 95 },
    { 0x55bef6a5, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cef6ad, 91 },
    { 0x19bef6ae, 33 },
    { 0, 0 },
    { 0xf58ef6b0, 98 },
    { 0, 0 },
    { 0xf93ef6b2, 33 },
    { 0xe86ef6b3, 95 },
    { 0xc50ef6b4, 33 },
    { 0, 0 },
    { 0x42aef6b6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cef6c1, 61 },
    { 0x7d8ef6c1, 95 },
    { 0x367ef6c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34aef6c7, 25 },
    { 0, 0 },
    { 0x67ef6c9, 87 },
    { 0x2e5ef6c9, 23 },
    { 0x3f7ef6cb, 23 },
    { 0x999ef6cc, 76 },
    { 0x126ef6cc, 90 },
    { 0x37ef6ca, 95 },
    { 0x2eaef6cf, 23 },
    { 0x9e0ef6ce, 98 },
    { 0xbfeef6c9, 91 },
    { 0x62def6d0, 84 },
    { 0, 0 },
    { 0x77fef6d4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16ef6d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ef6dc, 43 },
    { 0, 0 },
    { 0x1c6ef6de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60ef6e1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82eef6e5, 90 },
    { 0xbc6ef6e6, 90 },
    { 0x8e5ef6e6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8ef6ea, 98 },
    { 0x9a4ef6ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8ef6ef, 33 },
    { 0x1d7ef6f0, 46 },
    { 0x65fef6f1, 93 },
    { 0xe8fef6f1, 98 },
    { 0xc27ef6f3, 95 },
    { 0xc43ef6f1, 94 },
    { 0x1bbef6f5, 43 },
    { 0x8ebef6f6, 76 },
    { 0, 0 },
    { 0x248ef6f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fef6fc, 33 },
    { 0x4dcef6fc, 76 },
    { 0, 0 },
    { 0x25fef6ff, 33 },
    { 0x80cef700, 46 },
    { 0xc15ef701, 76 },
    { 0xd9ef6ff, 99 },
    { 0xceef6ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383ef708, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4aef70b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aeef70e, 23 },
    { 0, 0 },
    { 0x9b2ef710, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44ef718, 95 },
    { 0xdf9ef719, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cef71e, 67 },
    { 0xab6ef71f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84aef722, 93 },
    { 0x75eef723, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bef726, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29def729, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2ef72c, 91 },
    { 0x692ef72d, 93 },
    { 0xd1ef72e, 90 },
    { 0x7bfef72c, 95 },
    { 0x77cef730, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77ef73a, 61 },
    { 0x7baef73b, 88 },
    { 0x2d7ef73c, 55 },
    { 0x83bef73b, 61 },
    { 0xd49ef73a, 33 },
    { 0x6b8ef73a, 99 },
    { 0x57aef740, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4ef744, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963ef74b, 81 },
    { 0x542ef74c, 43 },
    { 0, 0 },
    { 0xf08ef74e, 52 },
    { 0x36eef74e, 91 },
    { 0xadeef74e, 84 },
    { 0x60fef751, 61 },
    { 0xae0ef751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348ef762, 90 },
    { 0x2baef763, 93 },
    { 0x596ef763, 91 },
    { 0x925ef765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bef768, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ddef76b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5aef76e, 90 },
    { 0xa6bef76f, 95 },
    { 0xd50ef770, 23 },
    { 0x83eef771, 43 },
    { 0xf95ef771, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x328ef775, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fef778, 61 },
    { 0xa34ef779, 34 },
    { 0x1d0ef77a, 61 },
    { 0xd18ef77b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21ef77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5ef787, 43 },
    { 0xdb1ef787, 43 },
    { 0, 0 },
    { 0x3cbef78a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cef78d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5ef790, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45eef794, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf07ef79c, 98 },
    { 0xd47ef79d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x534ef7a3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabef7a6, 55 },
    { 0x8dbef7a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bef7aa, 85 },
    { 0, 0 },
    { 0xebaef7ac, 95 },
    { 0, 0 },
    { 0xf84ef7ae, 33 },
    { 0x4f0ef7af, 33 },
    { 0xaa2ef7ae, 95 },
    { 0xaccef7ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x869ef7b7, 95 },
    { 0, 0 },
    { 0xa1aef7b9, 85 },
    { 0, 0 },
    { 0x1c5ef7bb, 23 },
    { 0xd8ef7bc, 84 },
    { 0xdfcef7bb, 46 },
    { 0x892ef7be, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x579ef7c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x877ef7c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598ef7c8, 100 },
    { 0x1cef7c9, 33 },
    { 0xd2def7ca, 90 },
    { 0x92aef7cb, 33 },
    { 0x3e4ef7c9, 61 },
    { 0x3e4ef7cd, 52 },
    { 0xc8aef7ce, 99 },
    { 0xb6cef7cb, 84 },
    { 0, 0 },
    { 0x378ef7d1, 74 },
    { 0x113ef7d2, 66 },
    { 0, 0 },
    { 0x237ef7d4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eef7d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1ef7dc, 95 },
    { 0, 0 },
    { 0xcdcef7de, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ef7e2, 98 },
    { 0x48def7e3, 99 },
    { 0x855ef7e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3ef7e9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652ef7ed, 91 },
    { 0xdeaef7ed, 66 },
    { 0xe78ef7ef, 98 },
    { 0xb50ef7ed, 33 },
    { 0, 0 },
    { 0x627ef7f2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fef7f5, 90 },
    { 0xb00ef7f6, 33 },
    { 0x5fdef7f5, 33 },
    { 0xc7aef7f7, 95 },
    { 0, 0 },
    { 0x6efef7fa, 23 },
    { 0x79cef7fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607ef801, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6aef806, 95 },
    { 0, 0 },
    { 0xceaef808, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64ef80c, 90 },
    { 0, 0 },
    { 0x505ef80e, 97 },
    { 0xdfdef80e, 76 },
    { 0x8ecef80e, 46 },
    { 0, 0 },
    { 0x5bef812, 33 },
    { 0x417ef812, 95 },
    { 0x5b7ef814, 46 },
    { 0xa90ef814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef81c, 95 },
    { 0xb1def81c, 61 },
    { 0x67bef81e, 77 },
    { 0x4a1ef81d, 95 },
    { 0x843ef81c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74ef823, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84fef826, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x482ef829, 90 },
    { 0x8beef82a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62aef82e, 98 },
    { 0xc9ef82e, 23 },
    { 0, 0 },
    { 0x95ef831, 61 },
    { 0, 0 },
    { 0x3bdef833, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ef836, 43 },
    { 0xac0ef837, 33 },
    { 0x5beef836, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135ef83e, 93 },
    { 0, 0 },
    { 0x2c1ef840, 46 },
    { 0x8cbef841, 67 },
    { 0xf2aef842, 76 },
    { 0x4afef843, 95 },
    { 0x291ef844, 98 },
    { 0x67cef845, 33 },
    { 0x511ef844, 55 },
    { 0x406ef845, 88 },
    { 0xbf6ef848, 90 },
    { 0xff7ef847, 23 },
    { 0x5b8ef842, 90 },
    { 0x9def84b, 23 },
    { 0xfc9ef842, 75 },
    { 0x5eaef845, 53 },
    { 0x23bef84e, 46 },
    { 0, 0 },
    { 0xee0ef850, 66 },
    { 0xa4ef851, 98 },
    { 0x351ef852, 90 },
    { 0xc95ef853, 34 },
    { 0x4adef854, 67 },
    { 0, 0 },
    { 0xdaaef856, 46 },
    { 0, 0 },
    { 0x2d4ef858, 61 },
    { 0xf7cef859, 61 },
    { 0x67aef85a, 99 },
    { 0x583ef859, 99 },
    { 0x9d8ef858, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf62ef85f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229ef865, 61 },
    { 0, 0 },
    { 0x61def867, 90 },
    { 0xe9eef867, 88 },
    { 0, 0 },
    { 0x8eeef86a, 61 },
    { 0x98fef86b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2ef86e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332ef879, 100 },
    { 0x31ef87a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7ef87f, 98 },
    { 0, 0 },
    { 0x467ef881, 84 },
    { 0, 0 },
    { 0x8c0ef883, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x15ef886, 33 },
    { 0, 0 },
    { 0x6d5ef888, 33 },
    { 0, 0 },
    { 0xc36ef88a, 87 },
    { 0x89def88a, 78 },
    { 0x95ef88c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cef891, 91 },
    { 0xb8ef892, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe7ef896, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ef89b, 46 },
    { 0, 0 },
    { 0xee4ef89d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x497ef8a8, 92 },
    { 0, 0 },
    { 0x140ef8aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ef8ad, 98 },
    { 0x7ffef8ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67eef8b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8ef8b6, 93 },
    { 0, 0 },
    { 0x5beef8b8, 55 },
    { 0xcb5ef8b8, 77 },
    { 0x955ef8ba, 34 },
    { 0xb1aef8bb, 98 },
    { 0xa01ef8bb, 33 },
    { 0, 0 },
    { 0x1adef8be, 90 },
    { 0x44eef8be, 65 },
    { 0xe63ef8c0, 43 },
    { 0, 0 },
    { 0xfd7ef8c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2ef8cc, 61 },
    { 0xf9cef8cd, 93 },
    { 0x97eef8ce, 90 },
    { 0x7fcef8ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x760ef8d2, 34 },
    { 0, 0 },
    { 0x2fbef8d4, 85 },
    { 0x5baef8d4, 34 },
    { 0xd84ef8d6, 33 },
    { 0xd8fef8d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3ef8dc, 85 },
    { 0, 0 },
    { 0x79def8de, 95 },
    { 0, 0 },
    { 0xedcef8e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ef8e4, 23 },
    { 0xa19ef8e5, 34 },
    { 0x771ef8e6, 99 },
    { 0x3e5ef8e7, 61 },
    { 0x909ef8e4, 99 },
    { 0x9a2ef8e7, 93 },
    { 0x76aef8e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ef8ed, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xf64ef8f0, 98 },
    { 0x919ef8f1, 100 },
    { 0xf9aef8f0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5ef8f5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5eef8fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1eef8ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfcef902, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x894ef90a, 98 },
    { 0xb3eef90b, 23 },
    { 0, 0 },
    { 0x5eeef90d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8ef911, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72ef917, 61 },
    { 0, 0 },
    { 0x12bef919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835ef91d, 33 },
    { 0x82def91e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0ef923, 23 },
    { 0x406ef924, 46 },
    { 0xe29ef925, 90 },
    { 0x292ef925, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdef933, 80 },
    { 0x7fdef934, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7aef93b, 67 },
    { 0xc1def93b, 95 },
    { 0, 0 },
    { 0xfbbef93e, 90 },
    { 0x6cdef93f, 61 },
    { 0x9c8ef93f, 76 },
    { 0x21cef941, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cef945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31ef94e, 99 },
    { 0xf17ef94f, 33 },
    { 0xb49ef94f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70ef954, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3fef959, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41ef95c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbef960, 93 },
    { 0xcecef961, 33 },
    { 0x12bef961, 95 },
    { 0x814ef961, 46 },
    { 0x5d3ef964, 34 },
    { 0x222ef965, 76 },
    { 0x952ef966, 86 },
    { 0, 0 },
    { 0x5abef968, 95 },
    { 0xe45ef968, 91 },
    { 0, 0 },
    { 0x6a0ef96b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5ef96f, 76 },
    { 0x899ef970, 90 },
    { 0x6f7ef971, 23 },
    { 0x46bef972, 66 },
    { 0, 0 },
    { 0x4a9ef974, 68 },
    { 0xfb4ef975, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfef97a, 43 },
    { 0, 0 },
    { 0x804ef97c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4ef980, 65 },
    { 0x943ef981, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0ef989, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bef98d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58ef990, 95 },
    { 0x282ef990, 85 },
    { 0x64cef991, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef995, 73 },
    { 0x1e1ef996, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bef999, 33 },
    { 0xe1bef999, 95 },
    { 0xdc5ef99b, 90 },
    { 0x18bef99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbef9a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1def9a4, 61 },
    { 0x693ef9a5, 98 },
    { 0x403ef9a4, 78 },
    { 0x951ef9a7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ef9b0, 61 },
    { 0x4ddef9b0, 95 },
    { 0x680ef9b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1ef9b9, 78 },
    { 0x5feef9ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25ef9c1, 61 },
    { 0, 0 },
    { 0x53aef9c3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaef9c7, 90 },
    { 0x843ef9c7, 93 },
    { 0xb13ef9c8, 33 },
    { 0, 0 },
    { 0xcf5ef9cb, 98 },
    { 0, 0 },
    { 0xb9def9cd, 52 },
    { 0xd63ef9ce, 61 },
    { 0xf25ef9cd, 66 },
    { 0x23eef9d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bef9d3, 98 },
    { 0xff5ef9d4, 46 },
    { 0xf59ef9d3, 43 },
    { 0xfdaef9d6, 97 },
    { 0x339ef9d7, 40 },
    { 0xcd8ef9d8, 90 },
    { 0xbf2ef9d9, 91 },
    { 0, 0 },
    { 0xcbaef9db, 68 },
    { 0xb9ef9dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12ef9e7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3ef9ee, 90 },
    { 0xed8ef9ef, 33 },
    { 0, 0 },
    { 0xd24ef9f1, 90 },
    { 0x185ef9f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2ef9f5, 33 },
    { 0x5c2ef9f6, 34 },
    { 0x771ef9f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0ef9fc, 33 },
    { 0x8bbef9fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77efa00, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71efa04, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9efa07, 77 },
    { 0, 0 },
    { 0x10fefa09, 23 },
    { 0x38befa0a, 90 },
    { 0, 0 },
    { 0xe6cefa0c, 23 },
    { 0xd47efa0c, 90 },
    { 0x5cbefa0e, 23 },
    { 0xc77efa0f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52efa13, 33 },
    { 0, 0 },
    { 0x5a4efa15, 33 },
    { 0x505efa15, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79efa1c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x581efa1f, 66 },
    { 0xb4befa20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1fefa27, 76 },
    { 0x278efa27, 95 },
    { 0, 0 },
    { 0x5c7efa2a, 90 },
    { 0x2e9efa2a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3efa36, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x872efa39, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652efa46, 23 },
    { 0x946efa47, 93 },
    { 0, 0 },
    { 0x7c9efa49, 90 },
    { 0, 0 },
    { 0xaf8efa4b, 90 },
    { 0xaf7efa4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x449efa51, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731efa57, 78 },
    { 0, 0 },
    { 0xcdbefa59, 20 },
    { 0x1a6efa59, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x376efa5d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7eefa60, 46 },
    { 0x9eeefa60, 43 },
    { 0x5eeefa60, 55 },
    { 0xdb7efa63, 36 },
    { 0, 0 },
    { 0x5bcefa65, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7efa73, 90 },
    { 0xdd9efa74, 78 },
    { 0x5a1efa75, 33 },
    { 0x612efa75, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbefa7b, 33 },
    { 0x44befa7c, 23 },
    { 0x982efa7d, 52 },
    { 0x158efa7e, 84 },
    { 0xf3befa7e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3efa82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4efa85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8fefa88, 78 },
    { 0x884efa89, 33 },
    { 0xd1fefa89, 95 },
    { 0x314efa8b, 95 },
    { 0xd79efa8c, 98 },
    { 0x5a5efa8c, 34 },
    { 0x9e2efa8e, 100 },
    { 0x5c1efa8e, 43 },
    { 0x296efa8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23eefa97, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98befa9c, 80 },
    { 0xe35efa9d, 61 },
    { 0x9c9efa9e, 55 },
    { 0, 0 },
    { 0x3aefaa0, 61 },
    { 0x1d6efaa0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270efaa5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acefaa9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1efaaf, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1efab7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148efabe, 76 },
    { 0xcbcefabf, 78 },
    { 0xd82efabf, 95 },
    { 0, 0 },
    { 0x939efac2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839efac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81defad1, 33 },
    { 0x8ebefad1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x843efad5, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3efada, 98 },
    { 0, 0 },
    { 0xb8eefadc, 52 },
    { 0, 0 },
    { 0xa62efade, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbeefae3, 52 },
    { 0x17fefae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d7efae8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8caefaf0, 95 },
    { 0xd1befaf1, 46 },
    { 0, 0 },
    { 0x4d5efaf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacefaf6, 76 },
    { 0, 0 },
    { 0x5d9efaf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508efafc, 98 },
    { 0, 0 },
    { 0xb9befafe, 17 },
    { 0x795efaff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9efb03, 93 },
    { 0, 0 },
    { 0xa49efb05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9efb0b, 76 },
    { 0x69befb0c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cdefb10, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913efb19, 90 },
    { 0, 0 },
    { 0xbe3efb1b, 75 },
    { 0, 0 },
    { 0x5fdefb1d, 61 },
    { 0x667efb1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99fefb27, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e3efb2b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fefb30, 90 },
    { 0x780efb31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9cefb38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8beefb3b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747efb43, 90 },
    { 0x5acefb43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9efb48, 34 },
    { 0x6bfefb49, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x99cefb4c, 98 },
    { 0, 0 },
    { 0xed7efb4e, 61 },
    { 0x224efb4f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88efb56, 64 },
    { 0xbf0efb56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aefb5b, 76 },
    { 0x252efb5b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fefb5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5befb62, 95 },
    { 0, 0 },
    { 0xea8efb64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x55aefb67, 81 },
    { 0x310efb67, 95 },
    { 0x902efb67, 44 },
    { 0, 0 },
    { 0x63fefb6b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdefb6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2deefb74, 33 },
    { 0x5e4efb74, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367efb79, 90 },
    { 0x68efb79, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61fefb7f, 80 },
    { 0x6beefb80, 55 },
    { 0x1a4efb81, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811efb86, 33 },
    { 0x1edefb87, 55 },
    { 0x816efb88, 33 },
    { 0x241efb88, 34 },
    { 0x6a1efb8a, 43 },
    { 0x1b4efb8b, 55 },
    { 0, 0 },
    { 0xa77efb8d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0efb90, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0efb94, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x434efb97, 88 },
    { 0xd76efb98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75efb9c, 61 },
    { 0x69efb9d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05efba0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x798efba3, 88 },
    { 0, 0 },
    { 0x668efba5, 95 },
    { 0, 0 },
    { 0x278efba7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12defbab, 52 },
    { 0xdb0efbab, 83 },
    { 0, 0 },
    { 0x63aefbae, 76 },
    { 0x77efbaf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68efbb4, 61 },
    { 0, 0 },
    { 0x5d4efbb6, 66 },
    { 0, 0 },
    { 0x1d8efbb8, 98 },
    { 0xc6aefbb8, 90 },
    { 0, 0 },
    { 0x572efbbb, 61 },
    { 0x1e0efbbc, 23 },
    { 0xef9efbbc, 78 },
    { 0xdd9efbbe, 98 },
    { 0x770efbbf, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaefbc2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x417efbc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1efbca, 85 },
    { 0, 0 },
    { 0x166efbcc, 93 },
    { 0x4a5efbcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0efbd0, 93 },
    { 0xe90efbd0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcefbd5, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19efbdc, 84 },
    { 0, 0 },
    { 0x3e8efbde, 99 },
    { 0x33aefbdf, 98 },
    { 0, 0 },
    { 0xef8efbe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eefbef, 98 },
    { 0, 0 },
    { 0x6b2efbf1, 33 },
    { 0x41defbf2, 75 },
    { 0xdaaefbf3, 95 },
    { 0, 0 },
    { 0xe7cefbf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911efc00, 72 },
    { 0, 0 },
    { 0x747efc02, 43 },
    { 0x231efc02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635efc10, 23 },
    { 0, 0 },
    { 0xcbefc12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3befc16, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18efc21, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6efc25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4efc2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cefc32, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cefc35, 76 },
    { 0, 0 },
    { 0x45cefc37, 46 },
    { 0, 0 },
    { 0xcb4efc39, 61 },
    { 0x366efc3a, 76 },
    { 0x66eefc3a, 23 },
    { 0x75aefc3c, 17 },
    { 0x356efc3c, 77 },
    { 0x721efc3d, 98 },
    { 0x8dbefc3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6efc43, 90 },
    { 0, 0 },
    { 0xbb4efc45, 33 },
    { 0x7c7efc45, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67efc49, 43 },
    { 0, 0 },
    { 0x3eefc4b, 98 },
    { 0, 0 },
    { 0x469efc4d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91efc52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x365efc55, 55 },
    { 0x89cefc55, 90 },
    { 0x466efc57, 76 },
    { 0, 0 },
    { 0x6b4efc59, 93 },
    { 0, 0 },
    { 0xe9efc5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6feefc5f, 33 },
    { 0x79aefc60, 92 },
    { 0xbd8efc61, 95 },
    { 0x8b3efc62, 61 },
    { 0x38efc62, 87 },
    { 0xd21efc64, 61 },
    { 0x574efc62, 33 },
    { 0xfc9efc66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x185efc69, 90 },
    { 0x438efc6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561efc6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960efc74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79efc77, 95 },
    { 0, 0 },
    { 0xdb0efc79, 43 },
    { 0x1b6efc7a, 46 },
    { 0x6b6efc7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9efc82, 61 },
    { 0xf89efc83, 66 },
    { 0xb00efc84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886efc88, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbaefc8e, 61 },
    { 0xef0efc8f, 88 },
    { 0, 0 },
    { 0x8fbefc91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6cefc94, 95 },
    { 0, 0 },
    { 0x2a7efc96, 66 },
    { 0, 0 },
    { 0x8bfefc98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebefca0, 61 },
    { 0, 0 },
    { 0x276efca2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x995efca5, 43 },
    { 0, 0 },
    { 0x2dfefca7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x955efcad, 61 },
    { 0, 0 },
    { 0x326efcaf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28efcb2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aefcb8, 90 },
    { 0xd1aefcb8, 55 },
    { 0, 0 },
    { 0x6e9efcbb, 33 },
    { 0xd89efcbc, 95 },
    { 0x89eefcbd, 98 },
    { 0x812efcbd, 85 },
    { 0, 0 },
    { 0xc77efcc0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x619efcc9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faefcd1, 64 },
    { 0, 0 },
    { 0xb2efcd3, 98 },
    { 0x98befcd4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484efcd8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a6efcdc, 99 },
    { 0xef6efcdc, 34 },
    { 0, 0 },
    { 0x99defcdf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26efce5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289efce9, 33 },
    { 0xcc5efce9, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x26eefced, 98 },
    { 0x46efced, 80 },
    { 0xa21efcef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caefcfa, 66 },
    { 0, 0 },
    { 0xe32efcfc, 61 },
    { 0xcf0efcfd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294efd01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4efd04, 91 },
    { 0xef6efd05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20efd0b, 34 },
    { 0x7dcefd0b, 53 },
    { 0x110efd0b, 84 },
    { 0, 0 },
    { 0x381efd0f, 33 },
    { 0, 0 },
    { 0xe4defd11, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898efd19, 46 },
    { 0, 0 },
    { 0xea5efd1b, 95 },
    { 0x2cbefd1b, 33 },
    { 0x933efd1d, 77 },
    { 0x7c7efd1d, 85 },
    { 0xf71efd1e, 23 },
    { 0, 0 },
    { 0xd3aefd21, 66 },
    { 0xa5fefd21, 93 },
    { 0x12efd23, 61 },
    { 0xab7efd23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219efd28, 90 },
    { 0x47fefd28, 90 },
    { 0xe97efd28, 90 },
    { 0x7f2efd2a, 61 },
    { 0x341efd2c, 33 },
    { 0x873efd2c, 61 },
    { 0xa91efd2b, 64 },
    { 0xeaaefd28, 95 },
    { 0, 0 },
    { 0x1ffefd31, 40 },
    { 0xe61efd32, 65 },
    { 0x105efd33, 61 },
    { 0x476efd34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3aefd38, 33 },
    { 0x2deefd39, 55 },
    { 0x53befd3a, 61 },
    { 0x556efd3a, 43 },
    { 0x46efd3c, 61 },
    { 0, 0 },
    { 0xdfcefd3e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28efd43, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x910efd46, 83 },
    { 0x3e8efd47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x424efd4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cefd4e, 90 },
    { 0xbc9efd4f, 48 },
    { 0xe54efd4f, 17 },
    { 0x6ebefd50, 98 },
    { 0xc2efd52, 61 },
    { 0x84cefd53, 41 },
    { 0xaf0efd54, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2efd57, 66 },
    { 0xfc1efd58, 55 },
    { 0, 0 },
    { 0xabfefd5a, 55 },
    { 0xcb9efd5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9efd5f, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7cefd62, 55 },
    { 0x42fefd63, 43 },
    { 0x639efd63, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52efd68, 77 },
    { 0xb87efd68, 95 },
    { 0, 0 },
    { 0x518efd6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2fefd6e, 33 },
    { 0, 0 },
    { 0xd46efd70, 33 },
    { 0, 0 },
    { 0x57cefd72, 90 },
    { 0, 0 },
    { 0x398efd74, 33 },
    { 0, 0 },
    { 0xcdbefd76, 98 },
    { 0xa2fefd76, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5efd7b, 61 },
    { 0, 0 },
    { 0x2b9efd7d, 98 },
    { 0, 0 },
    { 0xe70efd7f, 95 },
    { 0, 0 },
    { 0x91defd81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2efd88, 95 },
    { 0, 0 },
    { 0xb6befd8a, 90 },
    { 0xa8befd8b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81efd91, 100 },
    { 0x97fefd92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7efd9a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x370efda0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88efda3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaceefda7, 78 },
    { 0, 0 },
    { 0x1caefda9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ceefdaf, 55 },
    { 0x14fefdb0, 46 },
    { 0x3fbefdb1, 78 },
    { 0, 0 },
    { 0x29befdb3, 84 },
    { 0xd7eefdb4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x855efdb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33befdba, 23 },
    { 0xf0efdbb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8efdbf, 95 },
    { 0x265efdc0, 33 },
    { 0x274efdbf, 90 },
    { 0, 0 },
    { 0x740efdc3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bfefdc6, 43 },
    { 0xcf5efdc7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aefdcd, 72 },
    { 0x61aefdce, 61 },
    { 0xe4defdcd, 95 },
    { 0xc35efdd0, 95 },
    { 0x635efdcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cefdd8, 98 },
    { 0x9a0efdd8, 61 },
    { 0x5e6efdda, 46 },
    { 0x50befddb, 46 },
    { 0x4b2efdda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aefddf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78efde3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116efdf0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18efdf9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7defdfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5efe01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182efe0a, 38 },
    { 0xc49efe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9efe0e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cefe11, 96 },
    { 0xed1efe12, 88 },
    { 0, 0 },
    { 0xcebefe14, 76 },
    { 0x440efe15, 90 },
    { 0x706efe16, 23 },
    { 0x485efe17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eefe1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77efe1e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8efe24, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1cefe2b, 61 },
    { 0xf8cefe2c, 95 },
    { 0, 0 },
    { 0x262efe2e, 23 },
    { 0x23fefe2f, 33 },
    { 0xb2befe30, 61 },
    { 0xea9efe30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2efe34, 46 },
    { 0x8f9efe35, 17 },
    { 0x5ecefe36, 23 },
    { 0x7b3efe37, 93 },
    { 0x324efe38, 88 },
    { 0, 0 },
    { 0x5ddefe3a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31eefe4e, 78 },
    { 0x6adefe4f, 33 },
    { 0xd60efe50, 97 },
    { 0x20aefe51, 76 },
    { 0, 0 },
    { 0x81befe53, 33 },
    { 0x832efe54, 98 },
    { 0x741efe54, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x890efe58, 98 },
    { 0, 0 },
    { 0x7d2efe5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205efe5e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2efe62, 77 },
    { 0x43cefe62, 23 },
    { 0xdd9efe64, 90 },
    { 0x4b0efe65, 77 },
    { 0xf94efe66, 93 },
    { 0xeccefe67, 43 },
    { 0x422efe65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08efe6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27efe6e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x934efe71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785efe76, 85 },
    { 0, 0 },
    { 0xcefe78, 98 },
    { 0, 0 },
    { 0xb36efe7a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x709efe7d, 33 },
    { 0, 0 },
    { 0x689efe7f, 34 },
    { 0, 0 },
    { 0x929efe81, 93 },
    { 0x514efe81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4efe86, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135efe8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5efe96, 33 },
    { 0xdceefe96, 95 },
    { 0, 0 },
    { 0x41cefe99, 90 },
    { 0, 0 },
    { 0x7a2efe9b, 61 },
    { 0, 0 },
    { 0xfc0efe9d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62cefea1, 90 },
    { 0, 0 },
    { 0xbbcefea3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684efea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9efeab, 89 },
    { 0xb69efeac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f8efeaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56efeb2, 67 },
    { 0xa3befeb2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70efeb8, 90 },
    { 0x100efeb9, 52 },
    { 0x5d2efeba, 85 },
    { 0xaa0efeb9, 98 },
    { 0xc69efebc, 61 },
    { 0, 0 },
    { 0x8feefebe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89efec1, 23 },
    { 0, 0 },
    { 0x560efec3, 33 },
    { 0xa7befec4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adefec7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eefecb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fefecf, 87 },
    { 0x86efed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2efed3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x405efed6, 85 },
    { 0xe91efed7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6efeda, 33 },
    { 0xd52efedb, 23 },
    { 0xf0befedb, 61 },
    { 0x906efedd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9efee0, 91 },
    { 0x306efee0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64efee7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62efeea, 33 },
    { 0x4a2efeeb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89aefef2, 23 },
    { 0x613efef3, 78 },
    { 0x201efef3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8befef8, 23 },
    { 0x948efef9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd02efefc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33eff00, 23 },
    { 0xc3aeff01, 95 },
    { 0, 0 },
    { 0x7d3eff03, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ceff06, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eeff09, 46 },
    { 0xb77eff0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506eff0f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5eff14, 76 },
    { 0x117eff14, 90 },
    { 0, 0 },
    { 0xd5eeff17, 98 },
    { 0x1c2eff18, 95 },
    { 0xfdceff19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976eff1d, 99 },
    { 0x313eff1e, 78 },
    { 0xb37eff1d, 46 },
    { 0x48eff20, 67 },
    { 0xd23eff1d, 61 },
    { 0x9abeff22, 55 },
    { 0x19eff23, 90 },
    { 0xb9beff23, 51 },
    { 0, 0 },
    { 0x217eff26, 87 },
    { 0, 0 },
    { 0xd46eff28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7deff30, 78 },
    { 0x38feff31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4eff34, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x315eff37, 23 },
    { 0x50ceff38, 55 },
    { 0xa12eff37, 95 },
    { 0x124eff3a, 98 },
    { 0xb0ceff3a, 61 },
    { 0xc56eff3c, 90 },
    { 0xd51eff3b, 75 },
    { 0x8fceff3e, 61 },
    { 0, 0 },
    { 0x7a6eff40, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8eff44, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2eff47, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x28feff4a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eff4d, 33 },
    { 0xf9eff4e, 66 },
    { 0, 0 },
    { 0xa64eff50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c5eff53, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc93eff57, 65 },
    { 0, 0 },
    { 0xac1eff59, 46 },
    { 0xd21eff5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ceff5e, 75 },
    { 0x9c1eff5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2eff64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aeff6a, 61 },
    { 0, 0 },
    { 0x18feff6c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ceff71, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265eff75, 93 },
    { 0, 0 },
    { 0x896eff77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53beff7f, 33 },
    { 0xdb8eff7f, 76 },
    { 0x3caeff7f, 40 },
    { 0, 0 },
    { 0xbc3eff83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87deff8c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70deff90, 83 },
    { 0x9adeff91, 61 },
    { 0, 0 },
    { 0x114eff93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caeff97, 11 },
    { 0xf26eff97, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bfeff9c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4effa2, 43 },
    { 0, 0 },
    { 0xad0effa4, 40 },
    { 0, 0 },
    { 0x8dfeffa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9effa9, 95 },
    { 0x648effa9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348effae, 77 },
    { 0x8bbeffaf, 91 },
    { 0, 0 },
    { 0xe1aeffb1, 95 },
    { 0x17aeffb2, 34 },
    { 0x3e2effb3, 90 },
    { 0x21ceffb2, 33 },
    { 0xc99effb5, 90 },
    { 0xf5beffb2, 98 },
    { 0xe8effb7, 90 },
    { 0x32aeffb7, 90 },
    { 0x848effb9, 33 },
    { 0x9a4effba, 84 },
    { 0xafbeffba, 33 },
    { 0x327effb4, 90 },
    { 0xf07effb8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973effc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2feffc7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x896effcb, 95 },
    { 0x2f4effcc, 46 },
    { 0xcb6effcc, 61 },
    { 0xca0effce, 40 },
    { 0x812effcf, 23 },
    { 0, 0 },
    { 0xdaaeffd1, 98 },
    { 0x6b1effd1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1effd9, 68 },
    { 0x8afeffda, 23 },
    { 0xc4aeffdb, 43 },
    { 0x11effdc, 95 },
    { 0x666effdb, 95 },
    { 0xf8deffde, 80 },
    { 0x75ceffdf, 52 },
    { 0xf90effdc, 98 },
    { 0x7a2effe1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6effe4, 91 },
    { 0x3a5effe5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27effe8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8efff0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6aefff6, 66 },
    { 0x5aaefff7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x187efffa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0effff, 42 },
    { 0x11aeffff, 36 },
    { 0x9ff0000, 37 },
    { 0, 0 },
    { 0xeaf0003, 66 },
    { 0, 0 },
    { 0x945f0005, 66 },
    { 0x2acf0005, 99 },
    { 0, 0 },
    { 0x2dcf0008, 93 },
    { 0, 0 },
    { 0x932f000a, 23 },
    { 0xd37f000a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ef000f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ff0017, 90 },
    { 0xc50f0018, 68 },
    { 0xfabf0019, 90 },
    { 0x3daf001a, 48 },
    { 0, 0 },
    { 0xa0ef001c, 55 },
    { 0x277f001c, 76 },
    { 0xeb4f001e, 98 },
    { 0x5aaf001f, 46 },
    { 0x55df0020, 87 },
    { 0x2c1f0021, 76 },
    { 0xeaef001f, 76 },
    { 0x372f0023, 91 },
    { 0xa2df0022, 95 },
    { 0x8aef0025, 33 },
    { 0x836f0026, 95 },
    { 0x116f0027, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9f002b, 61 },
    { 0xc15f002c, 61 },
    { 0xa33f002d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebf0033, 34 },
    { 0x696f0034, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdf0039, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6f0045, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1df0049, 34 },
    { 0, 0 },
    { 0x9acf004b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x292f0051, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f0054, 61 },
    { 0x21df0054, 43 },
    { 0x343f0055, 55 },
    { 0, 0 },
    { 0xca2f0058, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f005b, 78 },
    { 0x54af005b, 40 },
    { 0xfa8f005d, 23 },
    { 0x37cf005e, 90 },
    { 0x49df005e, 87 },
    { 0x44ef005c, 34 },
    { 0xe99f005c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87f0064, 23 },
    { 0xd7cf0064, 90 },
    { 0x228f0066, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ff0069, 33 },
    { 0x94ff006a, 34 },
    { 0xc39f006a, 46 },
    { 0x251f006b, 55 },
    { 0xba7f006a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65f0070, 75 },
    { 0, 0 },
    { 0x245f0072, 97 },
    { 0xee6f0073, 90 },
    { 0xf02f0074, 84 },
    { 0x441f0075, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x958f0078, 36 },
    { 0x6edf0079, 77 },
    { 0x3fff0079, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37df0084, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37f008b, 90 },
    { 0x3a6f008b, 61 },
    { 0, 0 },
    { 0x63bf008e, 23 },
    { 0, 0 },
    { 0x4daf0090, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x361f0093, 90 },
    { 0x586f0094, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ef0098, 61 },
    { 0, 0 },
    { 0x587f009a, 34 },
    { 0, 0 },
    { 0xb1cf009c, 90 },
    { 0x9e6f009c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709f00a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60f00a6, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5caf00a9, 40 },
    { 0x91df00aa, 76 },
    { 0xc38f00ab, 78 },
    { 0xebbf00ac, 67 },
    { 0, 0 },
    { 0x764f00ae, 87 },
    { 0xf4cf00af, 46 },
    { 0x976f00b0, 100 },
    { 0xa5ef00b1, 84 },
    { 0x409f00b2, 98 },
    { 0x10ff00ae, 61 },
    { 0xc58f00b0, 17 },
    { 0x639f00b5, 33 },
    { 0x84ff00b6, 52 },
    { 0xba6f00b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4af00ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ff00c1, 52 },
    { 0xaaef00c2, 100 },
    { 0x23ef00c3, 23 },
    { 0x777f00c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f00c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760f00cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812f00d4, 23 },
    { 0, 0 },
    { 0x70bf00d6, 95 },
    { 0xd8f00d6, 91 },
    { 0, 0 },
    { 0x715f00d9, 91 },
    { 0, 0 },
    { 0x30df00db, 90 },
    { 0xe5f00db, 91 },
    { 0xb98f00dd, 90 },
    { 0, 0 },
    { 0x850f00df, 33 },
    { 0, 0 },
    { 0x9e9f00e1, 95 },
    { 0, 0 },
    { 0xad2f00e3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5f00e6, 90 },
    { 0, 0 },
    { 0x69cf00e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9cf00eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ff00f3, 93 },
    { 0x438f00f4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf00fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73df0100, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f010a, 90 },
    { 0x386f010b, 69 },
    { 0xb59f010c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225f0112, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8f011a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ef011d, 33 },
    { 0, 0 },
    { 0x305f011f, 66 },
    { 0, 0 },
    { 0x1d0f0121, 88 },
    { 0, 0 },
    { 0xdcaf0123, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x664f0126, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa21f012a, 99 },
    { 0xc88f012b, 61 },
    { 0x500f012a, 33 },
    { 0xa9bf012b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779f0131, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3f0136, 33 },
    { 0xe30f0137, 33 },
    { 0xd2f0138, 84 },
    { 0xf7cf0139, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f013c, 65 },
    { 0xf0ff013d, 93 },
    { 0x55df013d, 95 },
    { 0, 0 },
    { 0xff3f0140, 66 },
    { 0x42bf0140, 90 },
    { 0x3aaf0141, 76 },
    { 0x9f8f0140, 77 },
    { 0xe91f0140, 90 },
    { 0xb3ff0145, 55 },
    { 0xfaf0142, 43 },
    { 0x5ef0147, 91 },
    { 0xacbf0147, 95 },
    { 0xd38f0149, 77 },
    { 0xa24f014a, 90 },
    { 0xbaef0143, 43 },
    { 0x404f0145, 61 },
    { 0x81bf0145, 90 },
    { 0xd31f0148, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6f0155, 33 },
    { 0, 0 },
    { 0xe73f0157, 34 },
    { 0, 0 },
    { 0x124f0159, 53 },
    { 0, 0 },
    { 0xf64f015b, 90 },
    { 0xb42f015b, 84 },
    { 0xa0ff015d, 95 },
    { 0xf46f015e, 88 },
    { 0, 0 },
    { 0xccbf0160, 99 },
    { 0, 0 },
    { 0xc2f0162, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef0169, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x336f016c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eff0173, 23 },
    { 0xc4bf0173, 95 },
    { 0xa82f0175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93f0179, 33 },
    { 0x815f017a, 17 },
    { 0x7edf017a, 34 },
    { 0, 0 },
    { 0x570f017d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158f0184, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f0188, 91 },
    { 0, 0 },
    { 0xd9f018a, 100 },
    { 0x5f5f018a, 34 },
    { 0, 0 },
    { 0x645f018d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3f0192, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb58f0197, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ff019b, 95 },
    { 0, 0 },
    { 0x7eaf019d, 95 },
    { 0xe26f019e, 77 },
    { 0, 0 },
    { 0x820f01a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f01ab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x847f01ae, 33 },
    { 0xc49f01ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ff01b2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd5f01ba, 23 },
    { 0, 0 },
    { 0x9e3f01bc, 95 },
    { 0x2f2f01bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf01cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1df01cf, 98 },
    { 0x706f01d0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaf01d7, 33 },
    { 0xff4f01d8, 43 },
    { 0x56af01d9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cf01dc, 46 },
    { 0, 0 },
    { 0xe2cf01de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacf01e1, 68 },
    { 0xce5f01e2, 95 },
    { 0xd9df01e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f01e6, 90 },
    { 0x5ddf01e6, 55 },
    { 0xc3f01e7, 23 },
    { 0, 0 },
    { 0x5a5f01ea, 33 },
    { 0x558f01eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x602f01ee, 84 },
    { 0, 0 },
    { 0x882f01f0, 46 },
    { 0x6aef01f0, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df01f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdf01f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ef0200, 93 },
    { 0, 0 },
    { 0x7b5f0202, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9bf0209, 81 },
    { 0xc3af020a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x246f020d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708f021d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776f0222, 61 },
    { 0xd5af0223, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2df0226, 93 },
    { 0, 0 },
    { 0x29af0228, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf022b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67f0232, 33 },
    { 0xcaef0233, 76 },
    { 0x81af0234, 61 },
    { 0x39ef0235, 33 },
    { 0xdbbf0233, 93 },
    { 0xa52f0236, 90 },
    { 0x2e2f0232, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cf023d, 61 },
    { 0x548f023e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bf0241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37f0245, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ef0249, 90 },
    { 0xf4bf024a, 78 },
    { 0x5f5f024a, 98 },
    { 0x4e4f024c, 83 },
    { 0, 0 },
    { 0x86bf024e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399f0253, 61 },
    { 0, 0 },
    { 0x897f0255, 90 },
    { 0, 0 },
    { 0x7d5f0257, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x150f025a, 46 },
    { 0x83ef025a, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x430f025f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb7f0263, 33 },
    { 0xa8ff0263, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cf0267, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15f026b, 93 },
    { 0x3faf026b, 98 },
    { 0x688f026d, 76 },
    { 0x73ff026c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8f0272, 33 },
    { 0x92bf0272, 55 },
    { 0x4a4f0274, 93 },
    { 0x583f0275, 95 },
    { 0x8f7f0276, 99 },
    { 0xb0ff0277, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9f027a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7f027d, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fff0285, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9f028c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13f0291, 87 },
    { 0, 0 },
    { 0xa89f0293, 90 },
    { 0x6ebf0294, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f0298, 33 },
    { 0x38bf0298, 33 },
    { 0x6d2f029a, 98 },
    { 0xb8bf0299, 33 },
    { 0x4ebf029b, 43 },
    { 0x837f0299, 91 },
    { 0, 0 },
    { 0xf88f029f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdf02a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ecf02a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ef02af, 99 },
    { 0x895f02b0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccf02b3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73cf02b7, 88 },
    { 0xc29f02b7, 76 },
    { 0, 0 },
    { 0x638f02ba, 98 },
    { 0x707f02bb, 90 },
    { 0, 0 },
    { 0x155f02bd, 76 },
    { 0, 0 },
    { 0xe71f02bf, 90 },
    { 0, 0 },
    { 0x1d8f02c1, 33 },
    { 0x68ef02c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff02c6, 55 },
    { 0x56ff02c7, 34 },
    { 0, 0 },
    { 0x996f02c9, 95 },
    { 0x5b0f02c9, 99 },
    { 0x7bef02cb, 93 },
    { 0x65df02cc, 90 },
    { 0x580f02cc, 95 },
    { 0xab8f02c9, 78 },
    { 0x988f02cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f02d2, 95 },
    { 0xf4af02d3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9f02dc, 61 },
    { 0x9aef02dd, 75 },
    { 0xbcef02de, 98 },
    { 0x5ff02df, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844f02e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f02e7, 91 },
    { 0x8eef02e8, 95 },
    { 0x3b8f02e8, 90 },
    { 0, 0 },
    { 0x34df02eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a8f02ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbf02f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f02fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372f0300, 55 },
    { 0xf40f0300, 23 },
    { 0, 0 },
    { 0xca2f0303, 52 },
    { 0x917f0303, 23 },
    { 0, 0 },
    { 0x130f0306, 99 },
    { 0x940f0307, 33 },
    { 0xb1ef0308, 98 },
    { 0x5d6f0309, 23 },
    { 0, 0 },
    { 0x5f8f030b, 95 },
    { 0xae2f030b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ef030f, 66 },
    { 0, 0 },
    { 0xb4af0311, 43 },
    { 0x597f0312, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42f0316, 90 },
    { 0, 0 },
    { 0xdb8f0318, 34 },
    { 0x10af0319, 33 },
    { 0xcf1f0319, 23 },
    { 0x416f031b, 55 },
    { 0x1b9f031b, 34 },
    { 0x42bf0318, 33 },
    { 0x809f031e, 3 },
    { 0x6b9f031e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x133f0324, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf0327, 61 },
    { 0, 0 },
    { 0x33af0329, 90 },
    { 0xb9af032a, 95 },
    { 0x602f032a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2f032f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cf0335, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f0339, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92ff033d, 88 },
    { 0, 0 },
    { 0x570f033f, 95 },
    { 0xcfdf0340, 61 },
    { 0x901f0341, 61 },
    { 0x625f0342, 95 },
    { 0x194f0343, 61 },
    { 0xa5bf0343, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51df034a, 87 },
    { 0x314f034b, 93 },
    { 0, 0 },
    { 0xcf8f034d, 76 },
    { 0x92f034e, 93 },
    { 0x28ff034f, 90 },
    { 0xefcf0350, 38 },
    { 0x477f034d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55ef0357, 81 },
    { 0x657f0357, 93 },
    { 0x6d8f0359, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bf035c, 90 },
    { 0x9f0f035d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932f0365, 23 },
    { 0xcef0365, 90 },
    { 0x81bf0367, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ff036a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f036d, 75 },
    { 0x5f6f036d, 43 },
    { 0, 0 },
    { 0x3bef0370, 23 },
    { 0, 0 },
    { 0x903f0372, 95 },
    { 0x7e8f0373, 64 },
    { 0, 0 },
    { 0xd39f0375, 95 },
    { 0x989f0376, 98 },
    { 0xd16f0375, 91 },
    { 0x87bf0378, 93 },
    { 0xa9bf0378, 95 },
    { 0xf5af0378, 90 },
    { 0xd1ff037b, 23 },
    { 0x7d2f037b, 76 },
    { 0x6f6f037d, 33 },
    { 0x56af037b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x550f0382, 33 },
    { 0, 0 },
    { 0x493f0384, 34 },
    { 0x2a8f0385, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff0389, 99 },
    { 0, 0 },
    { 0x2f3f038b, 55 },
    { 0xffcf038c, 33 },
    { 0xa66f038b, 61 },
    { 0, 0 },
    { 0xe81f038f, 61 },
    { 0xffff038f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf0395, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f039d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4f03a0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491f03a6, 61 },
    { 0xdabf03a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f03aa, 52 },
    { 0x37f03ab, 46 },
    { 0, 0 },
    { 0xb52f03ad, 33 },
    { 0, 0 },
    { 0x8d6f03af, 90 },
    { 0xea3f03b0, 90 },
    { 0x2c1f03b1, 76 },
    { 0x1b2f03b2, 80 },
    { 0xc60f03af, 95 },
    { 0, 0 },
    { 0x43ef03b5, 90 },
    { 0xf4f03b6, 59 },
    { 0xcfff03b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ff03bb, 68 },
    { 0xbbaf03bc, 66 },
    { 0xebbf03bd, 33 },
    { 0x929f03bd, 33 },
    { 0xcd7f03bf, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0f03c2, 33 },
    { 0x29ff03c3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x765f03c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3f03c9, 93 },
    { 0x1ef03ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cf03d4, 85 },
    { 0xd60f03d4, 90 },
    { 0xaa3f03d6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ef03d9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f03de, 46 },
    { 0x731f03df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f03e3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2baf03e6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf03ea, 84 },
    { 0, 0 },
    { 0x5b0f03ec, 84 },
    { 0x1d5f03ec, 90 },
    { 0xd1ff03ee, 23 },
    { 0x9a8f03ed, 76 },
    { 0, 0 },
    { 0x1a8f03f1, 55 },
    { 0xbe1f03f2, 90 },
    { 0x9a6f03f3, 91 },
    { 0, 0 },
    { 0xe51f03f5, 94 },
    { 0xe5f03f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fff03f9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847f03fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d8f0401, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5f0404, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811f0408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x677f0412, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48f0415, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1f0419, 90 },
    { 0xacaf041a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cf041d, 81 },
    { 0x8c3f041e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbf0424, 65 },
    { 0x78df0425, 90 },
    { 0xeb1f0426, 91 },
    { 0xe33f0426, 90 },
    { 0x22af0428, 95 },
    { 0x2d7f0428, 61 },
    { 0x867f0426, 43 },
    { 0x3faf0426, 61 },
    { 0, 0 },
    { 0x55df042d, 23 },
    { 0, 0 },
    { 0x99cf042f, 100 },
    { 0xa8ef042f, 34 },
    { 0, 0 },
    { 0x284f0432, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44f0436, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4f043a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df043d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12f0440, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f044a, 46 },
    { 0x632f044b, 23 },
    { 0x611f044c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef044f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f0453, 55 },
    { 0x492f0453, 34 },
    { 0xa2ff0454, 95 },
    { 0x704f0453, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabf045a, 91 },
    { 0, 0 },
    { 0x141f045c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acf0461, 90 },
    { 0x291f0461, 33 },
    { 0x6adf0461, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49f0468, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5f046d, 33 },
    { 0xc59f046e, 55 },
    { 0x5d0f046f, 93 },
    { 0x4f4f046f, 90 },
    { 0x67ef046f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf0474, 90 },
    { 0x1c5f0475, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33f047a, 23 },
    { 0x2c9f047b, 43 },
    { 0x799f047c, 33 },
    { 0, 0 },
    { 0x1faf047e, 23 },
    { 0x6caf047e, 23 },
    { 0x9e9f0480, 95 },
    { 0, 0 },
    { 0x1a2f0482, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cf0486, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf0489, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x223f048c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caf0491, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef0498, 37 },
    { 0x863f0499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f04a6, 34 },
    { 0x554f04a7, 33 },
    { 0xb87f04a6, 91 },
    { 0x71ff04a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb83f04ad, 52 },
    { 0xffdf04ae, 76 },
    { 0x586f04ae, 84 },
    { 0x937f04b0, 76 },
    { 0, 0 },
    { 0xb90f04b2, 98 },
    { 0x592f04b3, 90 },
    { 0, 0 },
    { 0x220f04b5, 33 },
    { 0x325f04b6, 69 },
    { 0x494f04b7, 46 },
    { 0xcf5f04b8, 90 },
    { 0x10ef04b7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1f04bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443f04c4, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cf04c8, 95 },
    { 0, 0 },
    { 0x337f04ca, 91 },
    { 0x48ff04ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cf04cf, 95 },
    { 0xff9f04d0, 98 },
    { 0x4c4f04d1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1af04d5, 76 },
    { 0xaef04d6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bff04db, 90 },
    { 0xa34f04db, 90 },
    { 0x6dcf04dd, 90 },
    { 0x77df04de, 61 },
    { 0xb86f04df, 95 },
    { 0x674f04e0, 38 },
    { 0xd51f04e1, 98 },
    { 0x20bf04e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bf04e6, 76 },
    { 0x54df04e7, 33 },
    { 0xba4f04e8, 23 },
    { 0, 0 },
    { 0xc6f04ea, 33 },
    { 0x8ebf04eb, 61 },
    { 0x116f04eb, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf04f0, 55 },
    { 0x9f1f04f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4f04f6, 17 },
    { 0x4baf04f6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2f04ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ef0502, 85 },
    { 0, 0 },
    { 0x665f0504, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f050c, 34 },
    { 0x12ef050c, 90 },
    { 0x7c7f050e, 44 },
    { 0xf60f050f, 91 },
    { 0x47f0510, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x275f0517, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bf051c, 100 },
    { 0x1af051c, 61 },
    { 0x20f051e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef0521, 33 },
    { 0, 0 },
    { 0x84ef0523, 33 },
    { 0x89bf0524, 85 },
    { 0x539f0524, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816f0529, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8f052d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4f0531, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdff0534, 90 },
    { 0xa96f0535, 98 },
    { 0xb14f0535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300f053a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f0540, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f0543, 33 },
    { 0, 0 },
    { 0x3fbf0545, 90 },
    { 0, 0 },
    { 0x69f0547, 55 },
    { 0xb99f0548, 51 },
    { 0, 0 },
    { 0x60bf054a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x501f054d, 22 },
    { 0, 0 },
    { 0x1a6f054f, 61 },
    { 0, 0 },
    { 0x582f0551, 33 },
    { 0x856f0552, 90 },
    { 0x4d1f0552, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c3f0557, 23 },
    { 0xba8f0557, 95 },
    { 0x11ff0559, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x13f055c, 61 },
    { 0, 0 },
    { 0x2b5f055e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeef0562, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01f0566, 23 },
    { 0x845f0566, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9f056b, 61 },
    { 0x3dcf056c, 61 },
    { 0, 0 },
    { 0x5c5f056e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9f0573, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cf0576, 64 },
    { 0x1ebf0577, 85 },
    { 0xcdcf0578, 90 },
    { 0xe2bf0579, 55 },
    { 0x63ef0579, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f0580, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228f0584, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cf0587, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95f058a, 46 },
    { 0x43cf058b, 91 },
    { 0, 0 },
    { 0xb90f058d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cf0590, 81 },
    { 0x4ff0591, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdf0599, 84 },
    { 0x68ef059a, 99 },
    { 0, 0 },
    { 0xbddf059c, 66 },
    { 0x3a0f059d, 34 },
    { 0xe61f059d, 53 },
    { 0xab7f059d, 33 },
    { 0x514f05a0, 90 },
    { 0x3bf05a1, 90 },
    { 0xd64f05a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ef05a8, 43 },
    { 0, 0 },
    { 0x8ff05aa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ef05af, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52f05b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9af05b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf05bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf05be, 33 },
    { 0, 0 },
    { 0x3c6f05c0, 46 },
    { 0xe84f05c1, 90 },
    { 0, 0 },
    { 0x778f05c3, 66 },
    { 0xec5f05c3, 34 },
    { 0x2ccf05c5, 23 },
    { 0, 0 },
    { 0x180f05c7, 23 },
    { 0, 0 },
    { 0x63ff05c9, 61 },
    { 0x910f05ca, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f05d5, 87 },
    { 0x1ddf05d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x523f05df, 93 },
    { 0, 0 },
    { 0xf19f05e1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f05e7, 95 },
    { 0, 0 },
    { 0xf2af05e9, 100 },
    { 0xdf4f05ea, 98 },
    { 0, 0 },
    { 0xbb3f05ec, 87 },
    { 0, 0 },
    { 0x626f05ee, 61 },
    { 0x795f05ef, 88 },
    { 0x13af05ef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7f05f8, 95 },
    { 0x4ef05f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ef05ff, 66 },
    { 0xa72f05ff, 23 },
    { 0, 0 },
    { 0x9aaf0602, 34 },
    { 0, 0 },
    { 0xc46f0604, 77 },
    { 0x7b3f0604, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff0609, 61 },
    { 0x7fcf0609, 61 },
    { 0x51bf060b, 53 },
    { 0xc6ff0609, 90 },
    { 0xe55f060c, 33 },
    { 0, 0 },
    { 0x30df060f, 22 },
    { 0xaebf0610, 87 },
    { 0x9cef0611, 33 },
    { 0, 0 },
    { 0x9e5f0613, 90 },
    { 0xb45f0614, 76 },
    { 0x288f0615, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5f061a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f061d, 34 },
    { 0xa7cf061e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2f0625, 38 },
    { 0x29af0626, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21f0629, 90 },
    { 0x778f062a, 23 },
    { 0x4cbf062a, 60 },
    { 0x8fcf0629, 77 },
    { 0xc5df062d, 61 },
    { 0x7bef062e, 98 },
    { 0x942f062f, 66 },
    { 0, 0 },
    { 0x8a8f0631, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f0635, 23 },
    { 0x726f0635, 85 },
    { 0x7e0f0637, 90 },
    { 0xbdef0638, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d4f063b, 33 },
    { 0xa93f063b, 90 },
    { 0x92af063d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f0640, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73af0647, 76 },
    { 0x71f0648, 66 },
    { 0x5a8f0647, 66 },
    { 0x523f0647, 98 },
    { 0x2f5f064a, 68 },
    { 0x851f064c, 90 },
    { 0xb89f0648, 55 },
    { 0, 0 },
    { 0x305f064f, 61 },
    { 0xf33f0650, 83 },
    { 0, 0 },
    { 0x389f0652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x471f0655, 87 },
    { 0, 0 },
    { 0x10bf0657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856f065f, 90 },
    { 0, 0 },
    { 0xfc3f0661, 91 },
    { 0xf87f0662, 68 },
    { 0x203f0663, 23 },
    { 0x414f0663, 90 },
    { 0x579f0662, 76 },
    { 0x69ef0666, 90 },
    { 0xadbf0664, 90 },
    { 0xd9ff0668, 95 },
    { 0xf90f0665, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377f066f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf0672, 53 },
    { 0x781f0673, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9f0676, 90 },
    { 0x1fdf0677, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08f067d, 23 },
    { 0xdcaf067e, 99 },
    { 0x589f067d, 23 },
    { 0, 0 },
    { 0x6c4f0681, 90 },
    { 0, 0 },
    { 0x372f0683, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ff0689, 85 },
    { 0x14f068a, 55 },
    { 0x5f6f068b, 77 },
    { 0x9dff068c, 55 },
    { 0x26af068b, 95 },
    { 0xee6f0689, 95 },
    { 0x9a7f068f, 52 },
    { 0, 0 },
    { 0xa7af0691, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878f0697, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6f069a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee4f069d, 46 },
    { 0x732f069d, 61 },
    { 0xc68f069f, 95 },
    { 0x8ecf06a0, 98 },
    { 0xd2ff06a0, 61 },
    { 0x9faf06a1, 53 },
    { 0xf01f06a1, 76 },
    { 0x656f069f, 75 },
    { 0, 0 },
    { 0xfaaf06a6, 23 },
    { 0x2a2f06a6, 98 },
    { 0xd39f06a8, 90 },
    { 0x588f06a8, 91 },
    { 0xec6f06aa, 34 },
    { 0x8f06ab, 33 },
    { 0xa8af06ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cef06b1, 46 },
    { 0x4dcf06b1, 74 },
    { 0, 0 },
    { 0xd20f06b4, 95 },
    { 0x3c6f06b4, 83 },
    { 0xb14f06b6, 33 },
    { 0, 0 },
    { 0x967f06b8, 90 },
    { 0, 0 },
    { 0x8caf06ba, 98 },
    { 0xd22f06ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64f06c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaf06c4, 67 },
    { 0x7f5f06c5, 98 },
    { 0x7f3f06c6, 23 },
    { 0xbfff06c4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5f06cd, 61 },
    { 0x7eff06ce, 98 },
    { 0x794f06ce, 98 },
    { 0x111f06d0, 34 },
    { 0x2faf06d0, 76 },
    { 0, 0 },
    { 0x288f06d3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1f06d8, 75 },
    { 0x5eff06d9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbf06dc, 84 },
    { 0x7d0f06dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ef06e1, 98 },
    { 0xa7bf06e2, 90 },
    { 0x44df06e2, 77 },
    { 0x37ff06e3, 95 },
    { 0xc81f06e2, 98 },
    { 0x9e9f06e6, 33 },
    { 0xe8cf06e7, 90 },
    { 0x272f06e7, 91 },
    { 0xee1f06e9, 52 },
    { 0x6af06e9, 34 },
    { 0xaef06eb, 52 },
    { 0xf73f06e1, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff06f1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af06f6, 55 },
    { 0x272f06f6, 100 },
    { 0x21af06f8, 99 },
    { 0x829f06f9, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef06fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92df0700, 67 },
    { 0x873f0701, 40 },
    { 0x64bf0701, 98 },
    { 0x734f0703, 95 },
    { 0x4ecf0703, 34 },
    { 0, 0 },
    { 0xc65f0706, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f070b, 23 },
    { 0xf7f070c, 55 },
    { 0x211f070c, 95 },
    { 0x111f070d, 95 },
    { 0xcfdf070b, 95 },
    { 0x18ef0710, 90 },
    { 0, 0 },
    { 0xa6ff0712, 90 },
    { 0xa1ef0713, 90 },
    { 0x7d6f0714, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47f0719, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7f071c, 23 },
    { 0x45f071d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60f0721, 91 },
    { 0x5eef0721, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x603f0725, 90 },
    { 0, 0 },
    { 0xc7f0727, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x945f072a, 90 },
    { 0xcd2f072a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4f072e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddaf0731, 33 },
    { 0x139f0732, 76 },
    { 0xff3f0733, 98 },
    { 0x2e5f0731, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f0737, 95 },
    { 0x5b2f0738, 67 },
    { 0, 0 },
    { 0xae8f073a, 90 },
    { 0x37cf073b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef073e, 34 },
    { 0x638f073e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bf074a, 61 },
    { 0, 0 },
    { 0x1dbf074c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa20f0750, 88 },
    { 0x12df0750, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef0756, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f075b, 90 },
    { 0xd3ff075b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f0761, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3f0766, 85 },
    { 0xabcf0766, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f076d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ef0772, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7f0776, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5f0781, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6f0789, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4f0792, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x969f0795, 65 },
    { 0, 0 },
    { 0x8def0797, 61 },
    { 0, 0 },
    { 0x2eff0799, 78 },
    { 0, 0 },
    { 0x555f079b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b8f07a4, 76 },
    { 0x8b8f07a5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93f07a9, 93 },
    { 0x4bbf07aa, 90 },
    { 0x69bf07aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddf07b0, 88 },
    { 0x912f07b1, 52 },
    { 0xd61f07b2, 85 },
    { 0x8f3f07b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47df07b7, 61 },
    { 0x905f07b8, 90 },
    { 0x1a8f07b9, 66 },
    { 0xf16f07ba, 23 },
    { 0x2e6f07bb, 43 },
    { 0x7a0f07bc, 91 },
    { 0x23ef07bd, 95 },
    { 0x930f07bd, 76 },
    { 0, 0 },
    { 0x51df07c0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482f07ca, 33 },
    { 0x5cff07ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56af07d3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1f07d8, 61 },
    { 0x3d0f07d8, 75 },
    { 0x8f5f07da, 61 },
    { 0x3a3f07db, 76 },
    { 0x3df07dc, 98 },
    { 0x2b6f07dd, 78 },
    { 0, 0 },
    { 0x3cdf07df, 90 },
    { 0xdc9f07e0, 34 },
    { 0xd37f07e0, 43 },
    { 0xeb9f07e1, 76 },
    { 0, 0 },
    { 0x7df07e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21f07eb, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f07f0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe4f07f4, 38 },
    { 0x8b0f07f5, 61 },
    { 0xc1f07f5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bff07f9, 33 },
    { 0x266f07fa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ff0802, 43 },
    { 0x4fdf0803, 43 },
    { 0x32f0804, 23 },
    { 0x2f3f0805, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8def0808, 95 },
    { 0x788f0809, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f080c, 100 },
    { 0x12af080d, 95 },
    { 0xc1cf080d, 55 },
    { 0xcedf080f, 43 },
    { 0x4a9f0810, 95 },
    { 0x9def0811, 43 },
    { 0xff5f0811, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fdf0816, 33 },
    { 0, 0 },
    { 0x18f0818, 98 },
    { 0, 0 },
    { 0xa01f081a, 98 },
    { 0x391f081a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0f081f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f0823, 55 },
    { 0x366f0824, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ef0827, 33 },
    { 0xe78f0828, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f082c, 33 },
    { 0x2def082c, 91 },
    { 0x95f082e, 61 },
    { 0x2bbf082f, 33 },
    { 0x584f082e, 76 },
    { 0x58af082d, 76 },
    { 0x368f0832, 61 },
    { 0xe3ef0830, 33 },
    { 0xfe0f082c, 93 },
    { 0xc6df0835, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcdf0838, 95 },
    { 0, 0 },
    { 0xa5ef083a, 43 },
    { 0x3d4f083b, 34 },
    { 0xa6f083a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43af0843, 95 },
    { 0, 0 },
    { 0x675f0845, 91 },
    { 0x8f0846, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f084b, 52 },
    { 0xdcff084c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafaf084f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5df0852, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1f0857, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff085b, 33 },
    { 0x29f085c, 61 },
    { 0xcf3f085c, 23 },
    { 0x72cf085c, 76 },
    { 0xa8f085d, 90 },
    { 0, 0 },
    { 0xfecf0861, 90 },
    { 0, 0 },
    { 0xf5ff0863, 51 },
    { 0x89ff0863, 99 },
    { 0xaaef0864, 76 },
    { 0x80f0865, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0f0871, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf0877, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ff087a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f0881, 98 },
    { 0xa80f0882, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x706f0888, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3f088e, 61 },
    { 0, 0 },
    { 0xd33f0890, 55 },
    { 0x462f0890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f0899, 33 },
    { 0xadff089a, 91 },
    { 0x7d1f089b, 61 },
    { 0x7e4f089b, 64 },
    { 0xa04f089d, 48 },
    { 0x13bf089e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79f08a2, 23 },
    { 0x30ff08a3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f08a6, 84 },
    { 0x199f08a7, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1f08aa, 23 },
    { 0, 0 },
    { 0xc92f08ac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33df08b1, 90 },
    { 0, 0 },
    { 0xf5df08b3, 95 },
    { 0x205f08b3, 90 },
    { 0, 0 },
    { 0x15ef08b6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553f08c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00f08c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2f08ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994f08d3, 12 },
    { 0x2f2f08d4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4f08d8, 90 },
    { 0xb7ef08d8, 33 },
    { 0, 0 },
    { 0x376f08db, 33 },
    { 0x9e4f08dc, 98 },
    { 0, 0 },
    { 0xb20f08de, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4f08e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ff08e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x414f08e9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39f08ed, 88 },
    { 0xa22f08ed, 98 },
    { 0x6edf08ef, 98 },
    { 0x811f08f0, 90 },
    { 0x122f08ef, 76 },
    { 0x3b1f08f2, 23 },
    { 0x780f08f3, 76 },
    { 0x25df08f4, 46 },
    { 0, 0 },
    { 0x9f2f08f6, 61 },
    { 0x85ef08f7, 34 },
    { 0x3b5f08f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f08ff, 33 },
    { 0x836f0900, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3f0905, 95 },
    { 0, 0 },
    { 0x2a7f0907, 96 },
    { 0xeabf0908, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94f090b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cf0910, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5f0914, 17 },
    { 0x9ef0915, 98 },
    { 0xd90f0916, 61 },
    { 0x48ff0917, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88f091b, 77 },
    { 0x826f091c, 34 },
    { 0x6daf091b, 99 },
    { 0x410f091c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cf0922, 43 },
    { 0x1d0f0923, 87 },
    { 0xa2bf0923, 33 },
    { 0, 0 },
    { 0x643f0926, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1cf0929, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878f0930, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9f0935, 33 },
    { 0xf86f0936, 52 },
    { 0x208f0936, 78 },
    { 0x7bcf0937, 33 },
    { 0x7a9f0939, 61 },
    { 0x7aef0936, 90 },
    { 0x808f093b, 23 },
    { 0x1d8f093c, 90 },
    { 0, 0 },
    { 0x23ef093e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf0943, 93 },
    { 0, 0 },
    { 0xb0df0945, 61 },
    { 0, 0 },
    { 0xe0bf0947, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183f094e, 93 },
    { 0xd74f094f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cf0955, 99 },
    { 0x5a1f0956, 90 },
    { 0, 0 },
    { 0xa01f0958, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7baf095c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f095f, 90 },
    { 0x482f0960, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63df0966, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6f096a, 55 },
    { 0x801f096b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf0973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6f097b, 38 },
    { 0x9aaf097c, 78 },
    { 0xfd8f097d, 98 },
    { 0xf51f097d, 76 },
    { 0x2f8f097e, 66 },
    { 0x2fbf0980, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed1f098e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f0995, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebff0998, 99 },
    { 0x18cf0999, 46 },
    { 0x702f099a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90f099f, 34 },
    { 0x15ef09a0, 95 },
    { 0, 0 },
    { 0xba9f09a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x969f09a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaf09ac, 93 },
    { 0x310f09ad, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x374f09b0, 43 },
    { 0x866f09b1, 78 },
    { 0xf9bf09b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8f09b8, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ff09bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f09c2, 46 },
    { 0x9e8f09c2, 98 },
    { 0x194f09c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e1f09c9, 34 },
    { 0xc29f09c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161f09d1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf09d6, 34 },
    { 0x5e0f09d7, 43 },
    { 0x222f09d8, 40 },
    { 0xa0ef09d8, 61 },
    { 0, 0 },
    { 0x35cf09db, 23 },
    { 0x21cf09db, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4f09e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b0f09ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c2f09f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8f09f6, 36 },
    { 0x76af09f6, 33 },
    { 0, 0 },
    { 0x995f09f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9af09fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x443f0a07, 33 },
    { 0x49cf0a07, 34 },
    { 0, 0 },
    { 0x8b3f0a0a, 95 },
    { 0, 0 },
    { 0x45bf0a0c, 90 },
    { 0x317f0a0d, 23 },
    { 0x6c1f0a0d, 43 },
    { 0x985f0a0d, 93 },
    { 0xf9cf0a0d, 52 },
    { 0x963f0a10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72df0a1d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242f0a21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6df0a27, 90 },
    { 0xae4f0a27, 61 },
    { 0, 0 },
    { 0x26df0a2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef0a2d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f0a39, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ef0a3c, 84 },
    { 0xa66f0a3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6af0a43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3f0a49, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af0a4d, 55 },
    { 0, 0 },
    { 0xa61f0a4f, 33 },
    { 0xabff0a50, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53df0a54, 98 },
    { 0, 0 },
    { 0x3acf0a56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf0a5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3f0a60, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6df0a65, 100 },
    { 0, 0 },
    { 0x7c9f0a67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146f0a6d, 81 },
    { 0x79f0a6e, 90 },
    { 0x407f0a6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f0a72, 90 },
    { 0x889f0a72, 61 },
    { 0x89f0a74, 68 },
    { 0x8f5f0a73, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545f0a7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f0a7f, 43 },
    { 0x186f0a80, 64 },
    { 0xadbf0a81, 76 },
    { 0xd2df0a82, 90 },
    { 0xbf7f0a83, 68 },
    { 0x7daf0a84, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f0a88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51ef0a8f, 78 },
    { 0, 0 },
    { 0x61cf0a91, 90 },
    { 0x315f0a92, 46 },
    { 0x7eaf0a92, 34 },
    { 0, 0 },
    { 0xf0df0a95, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8af0aa1, 95 },
    { 0, 0 },
    { 0xdef0aa3, 67 },
    { 0x1c7f0aa4, 98 },
    { 0xd6f0aa4, 98 },
    { 0xeb1f0aa6, 61 },
    { 0, 0 },
    { 0x690f0aa8, 90 },
    { 0x724f0aa9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x663f0aac, 64 },
    { 0x8d6f0aad, 46 },
    { 0x39cf0aae, 84 },
    { 0x491f0aaf, 33 },
    { 0x1eef0aaf, 76 },
    { 0x8dff0ab0, 17 },
    { 0xc38f0aaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30f0ab8, 85 },
    { 0xd73f0ab9, 34 },
    { 0, 0 },
    { 0x786f0abb, 23 },
    { 0x760f0abc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af0abf, 98 },
    { 0x8dcf0ac0, 78 },
    { 0xf73f0ac1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fef0ac4, 61 },
    { 0, 0 },
    { 0x86f0ac6, 90 },
    { 0x2c7f0ac7, 95 },
    { 0xdff0ac7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f0acb, 33 },
    { 0x662f0acc, 33 },
    { 0xdf0f0acd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2af0ad0, 95 },
    { 0xfd3f0ad0, 67 },
    { 0x979f0ad1, 95 },
    { 0x4a0f0ad3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df0ad7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x747f0adb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1f0ae2, 75 },
    { 0, 0 },
    { 0x784f0ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcf0ae8, 98 },
    { 0xbcdf0ae9, 55 },
    { 0, 0 },
    { 0x268f0aeb, 34 },
    { 0x82ef0aec, 78 },
    { 0xa13f0aed, 46 },
    { 0xae0f0aee, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128f0af3, 98 },
    { 0, 0 },
    { 0xd2ef0af5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94f0afa, 52 },
    { 0, 0 },
    { 0x909f0afc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ff0b00, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f7f0b06, 34 },
    { 0xe36f0b07, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ef0b0a, 61 },
    { 0x6baf0b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7af0b0f, 61 },
    { 0x393f0b0f, 61 },
    { 0xc4f0b11, 33 },
    { 0xcb2f0b12, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x253f0b1a, 76 },
    { 0xaf0f0b1a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4f0b20, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9f0b23, 90 },
    { 0, 0 },
    { 0xa87f0b25, 67 },
    { 0x150f0b26, 34 },
    { 0, 0 },
    { 0xdf6f0b28, 52 },
    { 0x6acf0b29, 90 },
    { 0xd8af0b2a, 61 },
    { 0, 0 },
    { 0xb18f0b2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4af0b2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2f0b36, 23 },
    { 0x938f0b37, 84 },
    { 0x5b5f0b37, 85 },
    { 0x944f0b37, 33 },
    { 0x2a8f0b3a, 90 },
    { 0, 0 },
    { 0x63cf0b3c, 33 },
    { 0x7baf0b3c, 81 },
    { 0x393f0b3e, 66 },
    { 0xb05f0b3e, 95 },
    { 0xc90f0b3d, 84 },
    { 0xb5df0b3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f0b4a, 98 },
    { 0xfeaf0b4b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f0b53, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520f0b58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x413f0b5d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f0b61, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283f0b65, 33 },
    { 0x716f0b66, 77 },
    { 0x649f0b66, 34 },
    { 0xf3cf0b68, 84 },
    { 0, 0 },
    { 0x6a2f0b6a, 76 },
    { 0x23cf0b6b, 99 },
    { 0x546f0b6b, 34 },
    { 0, 0 },
    { 0xf20f0b6e, 65 },
    { 0x771f0b6f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f0b74, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbff0b77, 33 },
    { 0x36af0b78, 33 },
    { 0x3d2f0b79, 55 },
    { 0x4e5f0b79, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aef0b7e, 43 },
    { 0xa5cf0b7f, 91 },
    { 0x90af0b7f, 33 },
    { 0x1daf0b80, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f0b8a, 61 },
    { 0, 0 },
    { 0xbbdf0b8c, 90 },
    { 0x219f0b8c, 46 },
    { 0xdcaf0b8d, 90 },
    { 0, 0 },
    { 0xc15f0b90, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f0b94, 91 },
    { 0x312f0b94, 90 },
    { 0x7f6f0b96, 90 },
    { 0, 0 },
    { 0x475f0b98, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4f0ba1, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803f0ba5, 100 },
    { 0x8dbf0ba6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f0ba9, 43 },
    { 0, 0 },
    { 0x48ff0bab, 95 },
    { 0x6d9f0bac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5f0bb0, 46 },
    { 0, 0 },
    { 0x24cf0bb2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff0bb7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x626f0bba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x626f0bbd, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc43f0bc2, 98 },
    { 0xf35f0bc2, 66 },
    { 0, 0 },
    { 0xb9bf0bc5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf0bc8, 95 },
    { 0xe89f0bc9, 33 },
    { 0x630f0bca, 69 },
    { 0x4ff0bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf0bd6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe01f0bda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485f0be0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0f0be3, 59 },
    { 0x28f0be4, 98 },
    { 0x752f0be5, 51 },
    { 0x719f0be6, 95 },
    { 0x542f0be6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdf0bec, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18f0bf2, 34 },
    { 0x7def0bf2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x984f0bf6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x732f0bf9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f0bfc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2f0bff, 23 },
    { 0xa21f0c00, 99 },
    { 0, 0 },
    { 0x55bf0c02, 43 },
    { 0, 0 },
    { 0x4eef0c04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fbf0c09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x92df0c0c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8f0c10, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05f0c16, 99 },
    { 0x5c3f0c17, 61 },
    { 0xd3cf0c18, 98 },
    { 0x83f0c18, 23 },
    { 0x246f0c16, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fff0c20, 84 },
    { 0xea3f0c20, 23 },
    { 0x327f0c22, 23 },
    { 0, 0 },
    { 0x780f0c24, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9f0c2c, 90 },
    { 0x637f0c2d, 76 },
    { 0xca3f0c2c, 55 },
    { 0, 0 },
    { 0x3c3f0c30, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1df0c39, 95 },
    { 0, 0 },
    { 0x16af0c3b, 77 },
    { 0, 0 },
    { 0xe23f0c3d, 91 },
    { 0x7d9f0c3d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f0c45, 61 },
    { 0, 0 },
    { 0x232f0c47, 33 },
    { 0, 0 },
    { 0x864f0c49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f0c4e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3f0c51, 46 },
    { 0, 0 },
    { 0x676f0c53, 52 },
    { 0, 0 },
    { 0xb06f0c55, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3f0c58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x941f0c5c, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6df0c5f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f0c62, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf0c67, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16f0c6c, 23 },
    { 0xb4ef0c6d, 98 },
    { 0, 0 },
    { 0x5f3f0c6f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf0c73, 75 },
    { 0, 0 },
    { 0x78f0c75, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaf0c7a, 80 },
    { 0xc18f0c7b, 90 },
    { 0xba5f0c7c, 90 },
    { 0, 0 },
    { 0x790f0c7e, 98 },
    { 0xe98f0c7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabff0c84, 55 },
    { 0x7f9f0c85, 99 },
    { 0x79df0c85, 99 },
    { 0x4eef0c87, 66 },
    { 0, 0 },
    { 0x402f0c89, 33 },
    { 0, 0 },
    { 0x942f0c8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af0c90, 55 },
    { 0, 0 },
    { 0xf33f0c92, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95f0c99, 85 },
    { 0xddff0c9a, 33 },
    { 0x596f0c9a, 43 },
    { 0x1f3f0c9b, 99 },
    { 0xecbf0c99, 46 },
    { 0x808f0c9e, 88 },
    { 0, 0 },
    { 0xc5f0ca0, 98 },
    { 0x7bcf0ca0, 85 },
    { 0x2a5f0ca2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f0ca5, 17 },
    { 0x56f0ca6, 33 },
    { 0x178f0ca5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58f0caf, 23 },
    { 0x9aef0cb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3f0cb3, 90 },
    { 0xd3af0cb4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdf0cba, 55 },
    { 0xd4f0cbb, 76 },
    { 0x313f0cbb, 61 },
    { 0xaaff0cbd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acf0cc2, 93 },
    { 0x8b2f0cc2, 36 },
    { 0x2aff0cc2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f0cc8, 85 },
    { 0xb7bf0cc8, 98 },
    { 0x8f2f0cca, 76 },
    { 0x648f0cc9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe95f0ccf, 43 },
    { 0x8b1f0cd0, 23 },
    { 0, 0 },
    { 0xaaef0cd2, 40 },
    { 0xc61f0cd2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x754f0cd6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0f0cda, 61 },
    { 0x83bf0cdb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79f0ce1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x575f0ce6, 33 },
    { 0x7b5f0ce7, 23 },
    { 0x36f0ce8, 33 },
    { 0xd3ff0ce6, 98 },
    { 0xc20f0cea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2f0cf0, 46 },
    { 0xd7f0cf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddf0cf4, 22 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10f0cf8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xff9f0cfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97df0cff, 55 },
    { 0x613f0cff, 23 },
    { 0xfc9f0d01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f0d05, 33 },
    { 0, 0 },
    { 0x364f0d07, 90 },
    { 0x4ecf0d07, 99 },
    { 0x5b6f0d07, 43 },
    { 0x62bf0d07, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cf0d0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631f0d18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf0d20, 90 },
    { 0, 0 },
    { 0x254f0d22, 61 },
    { 0x17bf0d23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7bf0d27, 34 },
    { 0x7f6f0d28, 33 },
    { 0xe55f0d29, 33 },
    { 0x981f0d27, 55 },
    { 0xb4ef0d2a, 61 },
    { 0x699f0d27, 93 },
    { 0, 0 },
    { 0x7e1f0d2e, 100 },
    { 0xa9cf0d2e, 77 },
    { 0x715f0d2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9f0d35, 61 },
    { 0x797f0d36, 93 },
    { 0x319f0d37, 33 },
    { 0xdf6f0d36, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f0d3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x933f0d41, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0ef0d44, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25df0d48, 90 },
    { 0xeb5f0d49, 35 },
    { 0x17df0d4a, 61 },
    { 0, 0 },
    { 0x867f0d4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x146f0d4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ef0d52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf0d55, 66 },
    { 0x6d0f0d56, 76 },
    { 0xd6ff0d57, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f0d5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf0d5f, 97 },
    { 0, 0 },
    { 0xcdf0d61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7f0d64, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2f0d68, 61 },
    { 0x525f0d69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f0d6c, 77 },
    { 0x624f0d6d, 23 },
    { 0x93cf0d6e, 61 },
    { 0, 0 },
    { 0x23af0d70, 23 },
    { 0xd9df0d70, 90 },
    { 0xa3df0d72, 90 },
    { 0xf7df0d71, 55 },
    { 0xf8ef0d72, 76 },
    { 0xf0cf0d72, 91 },
    { 0x332f0d76, 95 },
    { 0, 0 },
    { 0x19ff0d78, 53 },
    { 0, 0 },
    { 0xa82f0d7a, 55 },
    { 0x2ebf0d7b, 36 },
    { 0x9d9f0d7c, 90 },
    { 0x4bef0d7d, 98 },
    { 0xf5f0d7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186f0d85, 90 },
    { 0x78af0d85, 23 },
    { 0xc77f0d87, 95 },
    { 0xadef0d88, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4daf0d8c, 95 },
    { 0x510f0d8d, 23 },
    { 0xe7cf0d8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x511f0d91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcf0d94, 76 },
    { 0x933f0d94, 61 },
    { 0x224f0d95, 76 },
    { 0xf9ff0d97, 33 },
    { 0, 0 },
    { 0xbd6f0d99, 85 },
    { 0xd4cf0d99, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fff0d9f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbf0da7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x382f0daa, 74 },
    { 0xc60f0dab, 95 },
    { 0xef7f0dac, 80 },
    { 0x304f0dad, 76 },
    { 0x8b4f0dad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ff0dc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df0dc8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe9f0dce, 64 },
    { 0xd1ef0dcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f0dd2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x572f0dd7, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45af0ddd, 90 },
    { 0, 0 },
    { 0xc7bf0ddf, 23 },
    { 0x9f0f0ddf, 95 },
    { 0xa97f0de1, 34 },
    { 0x7fdf0de2, 47 },
    { 0x2cdf0de2, 87 },
    { 0x60cf0de2, 101 },
    { 0xcef0de1, 90 },
    { 0x508f0de6, 33 },
    { 0x4b0f0de5, 93 },
    { 0xfaf0de8, 23 },
    { 0xd27f0de9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0df0dec, 36 },
    { 0x124f0ded, 76 },
    { 0, 0 },
    { 0x1d6f0def, 66 },
    { 0, 0 },
    { 0xc2bf0df1, 61 },
    { 0, 0 },
    { 0xd9df0df3, 95 },
    { 0x6e3f0df4, 52 },
    { 0, 0 },
    { 0xe4cf0df6, 33 },
    { 0x57bf0df7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bf0dfb, 33 },
    { 0, 0 },
    { 0x94f0dfd, 90 },
    { 0xfe1f0dfe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f0e01, 98 },
    { 0x960f0e01, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ef0e05, 90 },
    { 0x33bf0e05, 90 },
    { 0, 0 },
    { 0x9fbf0e08, 98 },
    { 0x8b4f0e09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x195f0e0c, 17 },
    { 0, 0 },
    { 0xc37f0e0e, 91 },
    { 0, 0 },
    { 0xd39f0e10, 23 },
    { 0x77af0e10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x683f0e14, 66 },
    { 0x339f0e15, 99 },
    { 0, 0 },
    { 0xc80f0e17, 61 },
    { 0xfcf0e18, 43 },
    { 0x39af0e19, 33 },
    { 0x6e3f0e1a, 68 },
    { 0xe61f0e1a, 98 },
    { 0x82f0e1a, 23 },
    { 0x873f0e1d, 23 },
    { 0xce0f0e1e, 55 },
    { 0x489f0e19, 75 },
    { 0x59ef0e1f, 61 },
    { 0x253f0e21, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ff0e24, 95 },
    { 0, 0 },
    { 0x3a6f0e26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef0e2b, 90 },
    { 0x9fef0e2c, 95 },
    { 0x711f0e2d, 100 },
    { 0x38f0e2e, 77 },
    { 0x673f0e2f, 34 },
    { 0xf73f0e2e, 95 },
    { 0, 0 },
    { 0xc30f0e32, 95 },
    { 0xcebf0e33, 23 },
    { 0x351f0e34, 95 },
    { 0, 0 },
    { 0x283f0e36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36af0e3a, 66 },
    { 0x227f0e3b, 61 },
    { 0xfadf0e3c, 93 },
    { 0x388f0e3c, 61 },
    { 0xbdf0e3a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974f0e44, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ef0e47, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf0e4a, 76 },
    { 0xbacf0e4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cf0e4e, 33 },
    { 0, 0 },
    { 0x6bdf0e50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819f0e55, 76 },
    { 0, 0 },
    { 0x384f0e57, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df0e5a, 90 },
    { 0x9a9f0e5b, 55 },
    { 0, 0 },
    { 0x7abf0e5d, 96 },
    { 0, 0 },
    { 0x871f0e5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bef0e65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890f0e6a, 95 },
    { 0x97df0e6b, 77 },
    { 0x8b7f0e6c, 97 },
    { 0, 0 },
    { 0xfa3f0e6e, 88 },
    { 0, 0 },
    { 0x4cbf0e70, 33 },
    { 0x6a5f0e71, 46 },
    { 0x472f0e72, 33 },
    { 0x652f0e72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x818f0e7f, 90 },
    { 0x691f0e7f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35f0e85, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0f0e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63f0e90, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27af0e98, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d6f0ea2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10af0ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf0ead, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x10df0eb0, 100 },
    { 0x150f0eb1, 55 },
    { 0x2c6f0eb1, 23 },
    { 0x6c9f0eb1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6df0eb9, 95 },
    { 0x2ef0eba, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cf0ebe, 87 },
    { 0x370f0ebe, 33 },
    { 0x99f0ebf, 91 },
    { 0x9dff0ec1, 95 },
    { 0x88ff0ec0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ef0ec7, 34 },
    { 0xb72f0ec8, 33 },
    { 0x694f0ec9, 87 },
    { 0, 0 },
    { 0xeb5f0ecb, 93 },
    { 0x366f0ecb, 98 },
    { 0, 0 },
    { 0xac8f0ece, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcf0ed9, 90 },
    { 0x95af0eda, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4f0edf, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f0ee3, 61 },
    { 0xc42f0ee4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97f0ee8, 60 },
    { 0, 0 },
    { 0x207f0eea, 67 },
    { 0xa4ff0eeb, 46 },
    { 0x6edf0eeb, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6f0eef, 66 },
    { 0, 0 },
    { 0x136f0ef1, 90 },
    { 0, 0 },
    { 0x587f0ef3, 76 },
    { 0, 0 },
    { 0xd0cf0ef5, 55 },
    { 0x7a8f0ef6, 95 },
    { 0x169f0ef6, 90 },
    { 0x48af0ef8, 85 },
    { 0x299f0ef8, 91 },
    { 0x9ebf0ef8, 84 },
    { 0xd91f0ef8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x62af0efe, 65 },
    { 0x255f0eff, 23 },
    { 0xc1f0efe, 98 },
    { 0x425f0f01, 61 },
    { 0, 0 },
    { 0xebef0f03, 61 },
    { 0xe2ff0f03, 90 },
    { 0x3f6f0f03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f0f09, 76 },
    { 0xabdf0f0a, 76 },
    { 0x38af0f0a, 76 },
    { 0xb7af0f0a, 33 },
    { 0xec3f0f0d, 90 },
    { 0xa36f0f0e, 34 },
    { 0xa75f0f0f, 44 },
    { 0x15ff0f10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9f0f13, 91 },
    { 0x5cef0f14, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64f0f18, 90 },
    { 0x70bf0f19, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x527f0f1d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df0f20, 98 },
    { 0xeeef0f21, 17 },
    { 0x209f0f22, 46 },
    { 0x28bf0f22, 33 },
    { 0, 0 },
    { 0x8fdf0f25, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f0f29, 98 },
    { 0x3cef0f2a, 84 },
    { 0, 0 },
    { 0x87f0f2c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf0f30, 61 },
    { 0x9af0f31, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc13f0f36, 93 },
    { 0, 0 },
    { 0x6d1f0f38, 61 },
    { 0, 0 },
    { 0x5f9f0f3a, 65 },
    { 0x630f0f3b, 95 },
    { 0x470f0f3c, 68 },
    { 0, 0 },
    { 0x267f0f3e, 33 },
    { 0xd64f0f3f, 91 },
    { 0x664f0f3f, 61 },
    { 0x978f0f3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f0f46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdf0f49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83f0f4d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ef0f50, 93 },
    { 0x7bef0f50, 33 },
    { 0xdbaf0f51, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff0f55, 84 },
    { 0xf19f0f56, 98 },
    { 0, 0 },
    { 0x289f0f58, 76 },
    { 0, 0 },
    { 0xd72f0f5a, 65 },
    { 0x14bf0f5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcf0f5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c1f0f6a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa55f0f72, 46 },
    { 0, 0 },
    { 0xb39f0f74, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ef0f77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x388f0f7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860f0f7f, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5df0f85, 98 },
    { 0x67ff0f86, 93 },
    { 0x165f0f85, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7f0f8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x686f0f8d, 65 },
    { 0xa1cf0f8e, 55 },
    { 0x513f0f8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cf0f92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbff0f96, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99f0f99, 95 },
    { 0, 0 },
    { 0x890f0f9b, 76 },
    { 0, 0 },
    { 0xed3f0f9d, 95 },
    { 0x532f0f9d, 43 },
    { 0, 0 },
    { 0xdc9f0fa0, 93 },
    { 0xe9cf0fa1, 61 },
    { 0xf6df0fa2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23f0fab, 86 },
    { 0x9dbf0fac, 91 },
    { 0, 0 },
    { 0x379f0fae, 36 },
    { 0xf5f0faf, 95 },
    { 0x365f0fb0, 90 },
    { 0, 0 },
    { 0x7e7f0fb2, 34 },
    { 0, 0 },
    { 0xb71f0fb4, 33 },
    { 0x1d8f0fb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1f0fb8, 98 },
    { 0, 0 },
    { 0x45df0fba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f0fbe, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbef0fc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b5f0fc6, 90 },
    { 0x762f0fc6, 61 },
    { 0xc42f0fc6, 17 },
    { 0x20cf0fc9, 88 },
    { 0xc0af0fca, 52 },
    { 0x823f0fc9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4f0fd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x102f0fdb, 23 },
    { 0xaa9f0fdb, 90 },
    { 0x929f0fdc, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cf0fe0, 90 },
    { 0, 0 },
    { 0x615f0fe2, 95 },
    { 0x14f0fe2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc7f0fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f0feb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af0ff2, 76 },
    { 0xfa5f0ff2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aef0ff9, 78 },
    { 0x7f6f0ff9, 23 },
    { 0xc83f0ffb, 23 },
    { 0xd0cf0ffb, 95 },
    { 0xa7ef0ffd, 98 },
    { 0x72bf0ffb, 91 },
    { 0, 0 },
    { 0x2e3f1000, 95 },
    { 0, 0 },
    { 0x920f1002, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x644f1006, 100 },
    { 0, 0 },
    { 0xaaef1008, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb06f100b, 17 },
    { 0xa3ff100c, 61 },
    { 0, 0 },
    { 0xb01f100e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259f1013, 55 },
    { 0x9cff1014, 46 },
    { 0xdfbf1014, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f1019, 98 },
    { 0x964f101a, 33 },
    { 0, 0 },
    { 0x43ff101c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25af1021, 46 },
    { 0, 0 },
    { 0x949f1023, 90 },
    { 0, 0 },
    { 0xfb8f1025, 23 },
    { 0x281f1026, 88 },
    { 0xd8ff1025, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8f102a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e9f102d, 100 },
    { 0x196f102e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ef1033, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4f1036, 78 },
    { 0x266f1037, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f1041, 67 },
    { 0x495f1042, 35 },
    { 0x219f1043, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ef1048, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4f104e, 34 },
    { 0x905f104f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0f1054, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f105a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71f105e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x12af1061, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2f1066, 91 },
    { 0, 0 },
    { 0x4f0f1068, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f106d, 95 },
    { 0xdccf106d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8f1071, 61 },
    { 0xc11f1072, 95 },
    { 0x840f1072, 90 },
    { 0xaeff1074, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ef1078, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587f1083, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf108a, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477f1091, 58 },
    { 0x258f1092, 23 },
    { 0x8c4f1092, 55 },
    { 0, 0 },
    { 0x11f1095, 85 },
    { 0, 0 },
    { 0x9edf1097, 90 },
    { 0, 0 },
    { 0xc3ff1099, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336f109d, 33 },
    { 0xbb8f109e, 76 },
    { 0x103f109f, 76 },
    { 0x10cf109e, 67 },
    { 0x2cdf10a1, 37 },
    { 0, 0 },
    { 0x573f10a3, 90 },
    { 0, 0 },
    { 0xd57f10a5, 23 },
    { 0, 0 },
    { 0x74df10a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790f10ab, 98 },
    { 0, 0 },
    { 0x7dcf10ad, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x232f10b0, 90 },
    { 0, 0 },
    { 0xf9bf10b2, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659f10b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53f10be, 95 },
    { 0xd57f10bf, 93 },
    { 0xb57f10bf, 70 },
    { 0, 0 },
    { 0x3c4f10c2, 61 },
    { 0, 0 },
    { 0x42f10c4, 23 },
    { 0x4f10c4, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415f10cd, 68 },
    { 0, 0 },
    { 0xcf9f10cf, 46 },
    { 0, 0 },
    { 0x423f10d1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55f10d6, 55 },
    { 0x438f10d7, 87 },
    { 0x113f10d6, 95 },
    { 0x2b2f10d9, 95 },
    { 0xc1ff10d9, 61 },
    { 0xdd7f10db, 61 },
    { 0, 0 },
    { 0xd70f10dd, 33 },
    { 0, 0 },
    { 0x37cf10df, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f10e3, 33 },
    { 0xbbbf10e3, 95 },
    { 0, 0 },
    { 0x5b4f10e6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bf10ed, 23 },
    { 0x416f10ee, 91 },
    { 0, 0 },
    { 0xcbbf10f0, 90 },
    { 0xc0bf10f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f10f7, 98 },
    { 0x106f10f7, 33 },
    { 0xf78f10f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3f10fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28f1104, 43 },
    { 0, 0 },
    { 0xf14f1106, 98 },
    { 0x8df1106, 34 },
    { 0, 0 },
    { 0x430f1109, 76 },
    { 0x768f110a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6f110e, 61 },
    { 0x39ff110e, 90 },
    { 0, 0 },
    { 0x27cf1111, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94f1115, 93 },
    { 0xd8ff1116, 100 },
    { 0x530f1117, 33 },
    { 0, 0 },
    { 0x371f1119, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddf1122, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e7f112a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9df1133, 98 },
    { 0x9e3f1133, 68 },
    { 0, 0 },
    { 0x8e6f1136, 90 },
    { 0xac7f1137, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73f113b, 81 },
    { 0xcaef113c, 52 },
    { 0xc0cf113d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf1140, 92 },
    { 0xc06f1141, 98 },
    { 0, 0 },
    { 0x508f1143, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f1149, 90 },
    { 0x2fff1149, 91 },
    { 0xe6bf1149, 23 },
    { 0x91bf114c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bf1153, 78 },
    { 0, 0 },
    { 0x4d0f1155, 100 },
    { 0x85ff1156, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x828f1159, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdf1163, 83 },
    { 0, 0 },
    { 0xf05f1165, 55 },
    { 0, 0 },
    { 0xd4ef1167, 95 },
    { 0, 0 },
    { 0x2e1f1169, 90 },
    { 0, 0 },
    { 0x819f116b, 43 },
    { 0x732f116b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27f116f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabf1173, 55 },
    { 0, 0 },
    { 0x2caf1175, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770f1179, 90 },
    { 0, 0 },
    { 0xa31f117b, 23 },
    { 0, 0 },
    { 0xecdf117d, 76 },
    { 0x74df117d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6f1181, 23 },
    { 0xc38f1182, 77 },
    { 0x5aff1181, 66 },
    { 0x43cf1183, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x53af1187, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eff118b, 76 },
    { 0x90df118c, 46 },
    { 0x45f118c, 61 },
    { 0x50cf118e, 23 },
    { 0xe54f118d, 95 },
    { 0xffdf1190, 95 },
    { 0x50bf1191, 96 },
    { 0, 0 },
    { 0xd66f1193, 65 },
    { 0, 0 },
    { 0xc72f1195, 23 },
    { 0xc08f1196, 61 },
    { 0, 0 },
    { 0x1acf1198, 61 },
    { 0x6d3f1198, 98 },
    { 0, 0 },
    { 0x1f2f119b, 23 },
    { 0xbadf119b, 55 },
    { 0x6a3f119d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf11a6, 95 },
    { 0, 0 },
    { 0x2f7f11a8, 55 },
    { 0, 0 },
    { 0x92af11aa, 100 },
    { 0x706f11ab, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0df11ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ef11b5, 46 },
    { 0xf2ff11b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2f11b9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ff11bc, 61 },
    { 0, 0 },
    { 0x49ff11be, 93 },
    { 0x9c8f11be, 85 },
    { 0x646f11c0, 90 },
    { 0x343f11c1, 23 },
    { 0xed5f11bf, 95 },
    { 0xa7ef11c3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecf11c6, 95 },
    { 0xc22f11c7, 87 },
    { 0xddf11c8, 80 },
    { 0, 0 },
    { 0xfc5f11ca, 33 },
    { 0xbbdf11cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681f11d1, 52 },
    { 0x7b3f11d1, 53 },
    { 0x244f11d3, 34 },
    { 0x62f11d1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b7f11d9, 95 },
    { 0x4daf11da, 78 },
    { 0, 0 },
    { 0x959f11dc, 55 },
    { 0, 0 },
    { 0x149f11de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ff11e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23f11e4, 33 },
    { 0x1f2f11e5, 52 },
    { 0, 0 },
    { 0x13f11e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ef11ec, 33 },
    { 0xc2df11ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52df11f1, 76 },
    { 0x7cf11f2, 90 },
    { 0x9baf11f3, 91 },
    { 0x6b9f11f2, 61 },
    { 0xf49f11f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x584f11f8, 98 },
    { 0xf31f11f9, 93 },
    { 0xba6f11fa, 66 },
    { 0x1f0f11fb, 95 },
    { 0x6c6f11f9, 98 },
    { 0x75ff11fd, 77 },
    { 0xeacf11fe, 95 },
    { 0, 0 },
    { 0x2a1f1200, 33 },
    { 0xde3f1201, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2f1204, 90 },
    { 0x485f1204, 98 },
    { 0xc33f1206, 78 },
    { 0x784f1207, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf120e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914f1212, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3f1216, 33 },
    { 0x647f1217, 90 },
    { 0xe11f1217, 98 },
    { 0, 0 },
    { 0x536f121a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x427f1221, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ff1224, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf1227, 52 },
    { 0x784f1228, 66 },
    { 0x531f1227, 33 },
    { 0x33af122a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66f122d, 52 },
    { 0x665f122e, 61 },
    { 0x21af122f, 98 },
    { 0x1ccf122f, 91 },
    { 0x15ef1230, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df123a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df123f, 23 },
    { 0x75df1240, 55 },
    { 0x728f1241, 66 },
    { 0xfb4f1242, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21af124d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20f1253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9df1259, 87 },
    { 0, 0 },
    { 0xebff125b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acf125e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x255f1262, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420f126c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6f126f, 40 },
    { 0xe86f1270, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bef1275, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x314f127b, 90 },
    { 0, 0 },
    { 0x85ff127d, 100 },
    { 0x2ccf127e, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7f1281, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33df1285, 53 },
    { 0x51cf1285, 93 },
    { 0x340f1287, 76 },
    { 0x175f1287, 23 },
    { 0x2abf1289, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d7f1292, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x554f1295, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ef1298, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f129c, 46 },
    { 0xd66f129d, 33 },
    { 0x8adf129e, 95 },
    { 0xd80f129c, 55 },
    { 0x7a3f12a0, 95 },
    { 0x5f3f129c, 95 },
    { 0, 0 },
    { 0x514f12a3, 76 },
    { 0x88ff12a4, 76 },
    { 0x756f12a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f12a8, 99 },
    { 0xe2af12a9, 98 },
    { 0xc53f12aa, 87 },
    { 0xef1f12a9, 90 },
    { 0, 0 },
    { 0x108f12ad, 90 },
    { 0xa86f12ad, 27 },
    { 0x23df12ae, 55 },
    { 0xaeff12b0, 61 },
    { 0x26df12b1, 84 },
    { 0xe74f12b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94f12b5, 87 },
    { 0xc54f12b6, 76 },
    { 0x3b4f12b7, 23 },
    { 0x865f12b5, 34 },
    { 0xfc9f12b7, 46 },
    { 0x998f12b7, 90 },
    { 0xf34f12b7, 100 },
    { 0x707f12b5, 46 },
    { 0xdaff12bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbdf12c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f12c5, 95 },
    { 0, 0 },
    { 0xd8cf12c7, 98 },
    { 0, 0 },
    { 0xc17f12c9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cf12cc, 90 },
    { 0, 0 },
    { 0x1b8f12ce, 33 },
    { 0x3e4f12cf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaef12d3, 33 },
    { 0x881f12d4, 23 },
    { 0, 0 },
    { 0x53f12d6, 90 },
    { 0, 0 },
    { 0x503f12d8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af12db, 100 },
    { 0xeb8f12db, 78 },
    { 0x8a1f12dc, 98 },
    { 0x8b1f12de, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eaf12e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x802f12ef, 90 },
    { 0, 0 },
    { 0x3f9f12f1, 61 },
    { 0xb27f12f2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bf12fa, 61 },
    { 0x464f12fb, 99 },
    { 0, 0 },
    { 0xce6f12fd, 33 },
    { 0, 0 },
    { 0x5bcf12ff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f1306, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992f130a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77f130e, 95 },
    { 0xdd5f130f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf1313, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x241f1316, 33 },
    { 0, 0 },
    { 0xcb9f1318, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8f131f, 61 },
    { 0x81df1320, 34 },
    { 0x8fef131f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf46f1328, 34 },
    { 0xf6bf1328, 43 },
    { 0xc9ef132a, 61 },
    { 0x914f1328, 76 },
    { 0xdc1f132c, 93 },
    { 0xe16f132c, 90 },
    { 0xbf2f132b, 93 },
    { 0, 0 },
    { 0xc0bf1330, 60 },
    { 0, 0 },
    { 0x3baf1332, 87 },
    { 0xaa0f1332, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef5f133b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9f1341, 23 },
    { 0x33ff1342, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593f1346, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdf134b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f1350, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x151f1353, 33 },
    { 0x958f1354, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x312f1357, 99 },
    { 0x65cf1358, 33 },
    { 0xf36f1359, 43 },
    { 0xe14f135a, 35 },
    { 0x38cf135b, 90 },
    { 0x99bf135c, 33 },
    { 0x8b8f135d, 23 },
    { 0x813f135e, 85 },
    { 0, 0 },
    { 0x78cf1360, 81 },
    { 0x66bf1361, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x779f1365, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bf1369, 48 },
    { 0, 0 },
    { 0x941f136b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ff1375, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05f137a, 95 },
    { 0xd2f137b, 33 },
    { 0xc86f137c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48f137f, 88 },
    { 0, 0 },
    { 0xd85f1381, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1384, 17 },
    { 0xc01f1385, 23 },
    { 0x4e1f1386, 95 },
    { 0xe4ef1387, 93 },
    { 0, 0 },
    { 0x976f1389, 55 },
    { 0, 0 },
    { 0x58bf138b, 55 },
    { 0xf80f138c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf1391, 98 },
    { 0, 0 },
    { 0x654f1393, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x560f1396, 33 },
    { 0xcecf1397, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c1f139c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcf139f, 95 },
    { 0, 0 },
    { 0x224f13a1, 90 },
    { 0x685f13a2, 95 },
    { 0xa98f13a1, 33 },
    { 0x2eef13a2, 61 },
    { 0x710f13a5, 90 },
    { 0xcc2f13a6, 87 },
    { 0, 0 },
    { 0xa46f13a8, 90 },
    { 0, 0 },
    { 0x647f13aa, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c9f13ad, 46 },
    { 0x22cf13ad, 95 },
    { 0, 0 },
    { 0xba3f13b0, 52 },
    { 0xa73f13b1, 67 },
    { 0x17bf13b2, 87 },
    { 0x870f13b2, 90 },
    { 0x83ff13b2, 90 },
    { 0xd19f13b5, 43 },
    { 0xa93f13b6, 56 },
    { 0xd60f13b5, 85 },
    { 0x2c0f13b3, 99 },
    { 0x9a2f13b8, 76 },
    { 0x6d1f13b8, 98 },
    { 0xe43f13ba, 65 },
    { 0xfcaf13bc, 76 },
    { 0x98f13b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374f13c2, 55 },
    { 0x84af13c3, 34 },
    { 0, 0 },
    { 0xa10f13c5, 90 },
    { 0xd68f13c6, 61 },
    { 0x9e6f13c6, 90 },
    { 0x25af13c8, 23 },
    { 0, 0 },
    { 0x12cf13ca, 33 },
    { 0xd6df13cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4af13d0, 34 },
    { 0x170f13d1, 23 },
    { 0xa4ff13d2, 23 },
    { 0x4f4f13d3, 97 },
    { 0, 0 },
    { 0x284f13d5, 23 },
    { 0x12ef13d6, 76 },
    { 0, 0 },
    { 0x36cf13d8, 33 },
    { 0, 0 },
    { 0x6c4f13da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2f13e0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff13e3, 46 },
    { 0xe83f13e3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x338f13e7, 99 },
    { 0x87cf13e8, 99 },
    { 0xd13f13e9, 34 },
    { 0xc19f13e9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ff13f0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bf13f3, 95 },
    { 0x360f13f3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bf13f7, 43 },
    { 0x169f13f8, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377f13fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88f1401, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1f1404, 76 },
    { 0x75af1404, 17 },
    { 0xd92f1405, 33 },
    { 0x2d8f1405, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb7f140e, 61 },
    { 0xb64f140e, 91 },
    { 0xb1ff140e, 33 },
    { 0xf94f1410, 91 },
    { 0, 0 },
    { 0x405f1413, 95 },
    { 0xc3ef1413, 87 },
    { 0x8b5f1415, 33 },
    { 0xf8ff1415, 90 },
    { 0xccaf1417, 61 },
    { 0x36af1415, 52 },
    { 0x7f5f1414, 78 },
    { 0xbbef141a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x154f141d, 55 },
    { 0xb2f141e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0af1422, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x814f1425, 95 },
    { 0xa8af1426, 40 },
    { 0xb1ff1427, 33 },
    { 0, 0 },
    { 0x895f1429, 43 },
    { 0, 0 },
    { 0xc93f142b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299f1434, 90 },
    { 0x140f1435, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x207f1438, 55 },
    { 0x3aaf1438, 55 },
    { 0, 0 },
    { 0x336f143b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f143e, 23 },
    { 0, 0 },
    { 0x5ddf1440, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377f1447, 23 },
    { 0, 0 },
    { 0x441f1449, 78 },
    { 0xb48f1449, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x868f144d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff1452, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6f1455, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ff1459, 98 },
    { 0xf9ef145a, 91 },
    { 0xf0cf145b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf145f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf1465, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df1469, 33 },
    { 0x36df146a, 23 },
    { 0xac4f146a, 86 },
    { 0xf25f1469, 90 },
    { 0x8e5f146b, 61 },
    { 0, 0 },
    { 0xf26f146f, 33 },
    { 0x866f1470, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x357f1473, 66 },
    { 0x1d5f1474, 90 },
    { 0x274f1475, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x763f1478, 98 },
    { 0x41df1479, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12f147c, 33 },
    { 0, 0 },
    { 0x44bf147e, 87 },
    { 0xee9f147f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faf1488, 98 },
    { 0, 0 },
    { 0xb55f148a, 91 },
    { 0x76df148b, 61 },
    { 0xc21f148b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dff148f, 98 },
    { 0x432f1490, 76 },
    { 0xed7f1491, 76 },
    { 0, 0 },
    { 0xb8ff1493, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f1498, 80 },
    { 0x662f1498, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af149c, 77 },
    { 0xe2ff149d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8af14a0, 85 },
    { 0x988f14a1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915f14a6, 23 },
    { 0x55cf14a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f14ad, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12f14b5, 47 },
    { 0, 0 },
    { 0xb84f14b7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ef14ba, 90 },
    { 0x610f14ba, 90 },
    { 0xe4ff14bc, 61 },
    { 0x159f14bd, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x611f14c0, 77 },
    { 0x435f14c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdf14c5, 52 },
    { 0x745f14c5, 90 },
    { 0, 0 },
    { 0x172f14c8, 66 },
    { 0xdb9f14c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdf14cf, 33 },
    { 0x3f0f14d0, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f14d9, 34 },
    { 0xfc1f14da, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465f14de, 95 },
    { 0x6c1f14de, 33 },
    { 0x252f14e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55f14e3, 43 },
    { 0, 0 },
    { 0x205f14e5, 23 },
    { 0, 0 },
    { 0x3b3f14e7, 90 },
    { 0, 0 },
    { 0xcc7f14e9, 90 },
    { 0xea9f14e9, 61 },
    { 0, 0 },
    { 0x67af14ec, 90 },
    { 0, 0 },
    { 0x3f1f14ee, 99 },
    { 0x26cf14ef, 43 },
    { 0xdb9f14ef, 55 },
    { 0x82bf14f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7f14fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55df1501, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f1504, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13df150a, 33 },
    { 0x30f150b, 98 },
    { 0x682f150b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f150f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f1512, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110f1518, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bf151d, 23 },
    { 0xc90f151d, 77 },
    { 0, 0 },
    { 0x2b4f1520, 98 },
    { 0x18ef1520, 33 },
    { 0xbcbf1522, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590f1526, 52 },
    { 0x61cf1527, 23 },
    { 0x944f1527, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cf152e, 95 },
    { 0x59f152f, 33 },
    { 0x6f9f152f, 90 },
    { 0, 0 },
    { 0xa49f1532, 90 },
    { 0, 0 },
    { 0x25ef1534, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45f153b, 33 },
    { 0, 0 },
    { 0xe4ef153d, 55 },
    { 0, 0 },
    { 0x43f153f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ff1546, 23 },
    { 0, 0 },
    { 0xf40f1548, 55 },
    { 0x866f1548, 23 },
    { 0xba8f154a, 52 },
    { 0x376f154a, 23 },
    { 0x150f154c, 99 },
    { 0x14af154d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ef1551, 90 },
    { 0, 0 },
    { 0x957f1553, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8df1559, 88 },
    { 0, 0 },
    { 0xd86f155b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbf155e, 52 },
    { 0x7a1f155e, 61 },
    { 0x83f155f, 51 },
    { 0x7e0f1561, 61 },
    { 0xf10f1562, 90 },
    { 0x839f1561, 76 },
    { 0, 0 },
    { 0x57f1565, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f156f, 90 },
    { 0xce1f1570, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b5f1573, 90 },
    { 0xf2af1574, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360f1579, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef157d, 95 },
    { 0x12cf157e, 23 },
    { 0x16af157e, 90 },
    { 0, 0 },
    { 0xd9bf1581, 33 },
    { 0, 0 },
    { 0x57af1583, 48 },
    { 0x633f1584, 90 },
    { 0x28ef1585, 33 },
    { 0x965f1584, 23 },
    { 0, 0 },
    { 0x821f1588, 33 },
    { 0x6e3f1589, 84 },
    { 0xc79f1589, 33 },
    { 0xa7df158b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00f158e, 33 },
    { 0x5c7f158f, 61 },
    { 0x451f158f, 33 },
    { 0xf7ff1591, 95 },
    { 0x876f1592, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3f1595, 23 },
    { 0, 0 },
    { 0x2f1597, 61 },
    { 0x176f1598, 23 },
    { 0x24af1598, 52 },
    { 0x1c4f159a, 88 },
    { 0xf68f1598, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f159f, 95 },
    { 0xae3f15a0, 55 },
    { 0x88ef15a0, 76 },
    { 0xf6ef15a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x197f15a5, 77 },
    { 0, 0 },
    { 0xa4af15a7, 95 },
    { 0, 0 },
    { 0xf31f15a9, 95 },
    { 0x8a4f15a9, 98 },
    { 0x25ef15a9, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a7f15b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf15b4, 46 },
    { 0, 0 },
    { 0x233f15b6, 100 },
    { 0x596f15b7, 61 },
    { 0, 0 },
    { 0x81f15b9, 33 },
    { 0x647f15ba, 33 },
    { 0, 0 },
    { 0x3fff15bc, 33 },
    { 0x3e5f15bd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16af15c3, 66 },
    { 0xd77f15c3, 61 },
    { 0xf0af15c3, 23 },
    { 0x724f15c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667f15cc, 46 },
    { 0x149f15cc, 23 },
    { 0x2aff15ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf15d4, 34 },
    { 0x15af15d5, 80 },
    { 0xba7f15d6, 90 },
    { 0xeeef15d5, 56 },
    { 0xaf6f15d8, 55 },
    { 0xbdcf15d9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866f15e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddf15e6, 61 },
    { 0x165f15e7, 90 },
    { 0x93bf15e8, 90 },
    { 0x449f15e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f15ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f15f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2f15fc, 23 },
    { 0, 0 },
    { 0xb62f15fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0f1601, 23 },
    { 0x59ef1602, 67 },
    { 0, 0 },
    { 0xc98f1604, 43 },
    { 0xcbcf1605, 61 },
    { 0, 0 },
    { 0xafdf1607, 66 },
    { 0xe98f1608, 43 },
    { 0xa16f1609, 33 },
    { 0x5a3f1608, 34 },
    { 0, 0 },
    { 0x596f160c, 90 },
    { 0x798f160d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aaf1610, 33 },
    { 0x3f1610, 95 },
    { 0, 0 },
    { 0x58ff1613, 33 },
    { 0xf6cf1613, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ff161d, 90 },
    { 0x17ff161e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef1624, 65 },
    { 0, 0 },
    { 0x9a3f1626, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f162a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39f162f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5af1632, 93 },
    { 0xb83f1633, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f1637, 90 },
    { 0, 0 },
    { 0x868f1639, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1f1642, 61 },
    { 0x1cff1643, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9f1648, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f164c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x211f164f, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139f1655, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51f1658, 98 },
    { 0x93bf1659, 84 },
    { 0xb16f1658, 23 },
    { 0, 0 },
    { 0x8f6f165c, 90 },
    { 0x3c8f165d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebf1664, 95 },
    { 0x810f1665, 91 },
    { 0x9e2f1665, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6f166e, 19 },
    { 0x34f166f, 87 },
    { 0, 0 },
    { 0x32f1671, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x471f1674, 84 },
    { 0, 0 },
    { 0x88ff1676, 61 },
    { 0xa2df1677, 84 },
    { 0xc9cf1677, 91 },
    { 0xa23f1679, 98 },
    { 0, 0 },
    { 0xa52f167b, 99 },
    { 0, 0 },
    { 0xceff167d, 98 },
    { 0, 0 },
    { 0xbdff167f, 61 },
    { 0xb39f1680, 95 },
    { 0, 0 },
    { 0x6fcf1682, 78 },
    { 0x8ecf1682, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139f1689, 99 },
    { 0x32f168a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93df1691, 98 },
    { 0x87bf1692, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6faf169c, 34 },
    { 0x344f169d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x800f16a5, 90 },
    { 0, 0 },
    { 0x12ef16a7, 78 },
    { 0, 0 },
    { 0x52cf16a9, 76 },
    { 0x7e3f16aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972f16af, 88 },
    { 0xf5ef16b0, 95 },
    { 0xe83f16af, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bf16b6, 93 },
    { 0x6ef16b7, 95 },
    { 0xa1bf16b8, 90 },
    { 0x120f16b8, 98 },
    { 0x4c5f16ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47f16bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3f16c1, 33 },
    { 0x42df16c1, 43 },
    { 0x861f16c1, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ef16c7, 98 },
    { 0x79bf16c8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cf16d0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5f16d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f16d6, 61 },
    { 0x2e8f16d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbf16e7, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8f16ed, 76 },
    { 0, 0 },
    { 0x3daf16ef, 67 },
    { 0, 0 },
    { 0xce0f16f1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef16fb, 23 },
    { 0x5ecf16fc, 76 },
    { 0x572f16fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767f1703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42af1707, 90 },
    { 0x4d4f1708, 23 },
    { 0xaf7f1709, 76 },
    { 0x9c4f1709, 90 },
    { 0, 0 },
    { 0xfa0f170c, 61 },
    { 0xddf170d, 98 },
    { 0, 0 },
    { 0x7cbf170f, 99 },
    { 0, 0 },
    { 0x75af1711, 23 },
    { 0x12df1711, 95 },
    { 0x7d9f1712, 77 },
    { 0x544f1714, 76 },
    { 0x3b4f1715, 68 },
    { 0x496f1715, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ef171a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11f1721, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332f1727, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815f172b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcff172e, 92 },
    { 0x2acf172f, 55 },
    { 0x302f172f, 98 },
    { 0x806f172e, 67 },
    { 0xd1f1732, 17 },
    { 0x24df1733, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7f1736, 33 },
    { 0x754f1737, 91 },
    { 0x103f1738, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333f173e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8f1741, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9f1745, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bf1748, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0f174b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef1752, 55 },
    { 0xd03f1753, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe24f1756, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edf175f, 97 },
    { 0xb44f1760, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf1766, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f176a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f1775, 90 },
    { 0x1dff1776, 93 },
    { 0xc83f1777, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c6f177b, 46 },
    { 0x192f177c, 98 },
    { 0, 0 },
    { 0xa52f177e, 75 },
    { 0x8bff177f, 78 },
    { 0, 0 },
    { 0x499f1781, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5f1789, 61 },
    { 0x145f1789, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x43df178d, 78 },
    { 0x44f178e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x259f1795, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf179a, 23 },
    { 0x57ef179b, 95 },
    { 0x345f179a, 23 },
    { 0x22af179a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70f17a0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28f17a3, 61 },
    { 0, 0 },
    { 0xc6ff17a5, 23 },
    { 0, 0 },
    { 0xc61f17a7, 33 },
    { 0, 0 },
    { 0xdb0f17a9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f17ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cf17b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb64f17b7, 46 },
    { 0x29ef17b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75f17c0, 78 },
    { 0x46bf17c1, 46 },
    { 0xe82f17c2, 84 },
    { 0x74af17c3, 23 },
    { 0x991f17c3, 38 },
    { 0, 0 },
    { 0x18bf17c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88f17c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22f17cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4f17d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf17d5, 90 },
    { 0, 0 },
    { 0x8f1f17d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb4f17dc, 90 },
    { 0xb19f17dc, 33 },
    { 0, 0 },
    { 0xf53f17df, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ef17e3, 68 },
    { 0x14cf17e4, 55 },
    { 0xdcf17e5, 33 },
    { 0xdadf17e3, 90 },
    { 0xcfef17e5, 33 },
    { 0, 0 },
    { 0x710f17e9, 98 },
    { 0x344f17ea, 66 },
    { 0x5e5f17eb, 90 },
    { 0x8b3f17ea, 77 },
    { 0xa37f17ed, 43 },
    { 0x114f17ea, 100 },
    { 0xeacf17ef, 33 },
    { 0, 0 },
    { 0x7a0f17f1, 33 },
    { 0x55f17f2, 17 },
    { 0xc9af17f2, 93 },
    { 0xad7f17f4, 33 },
    { 0x11bf17f4, 76 },
    { 0x2e5f17f6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f8f17fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f17ff, 87 },
    { 0x882f1800, 34 },
    { 0x4a0f1800, 98 },
    { 0x631f17ff, 87 },
    { 0, 0 },
    { 0x5bdf1804, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601f1809, 87 },
    { 0x344f180a, 61 },
    { 0xac0f180a, 34 },
    { 0xe49f180c, 34 },
    { 0x85df180b, 61 },
    { 0x2d3f180e, 33 },
    { 0xcb0f180f, 90 },
    { 0x90ef180c, 78 },
    { 0xf65f1811, 43 },
    { 0x48af1812, 66 },
    { 0x271f1813, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7f1816, 98 },
    { 0, 0 },
    { 0xc79f1818, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44f181d, 33 },
    { 0x16df181e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2f1821, 55 },
    { 0x656f1821, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a3f1827, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bff182c, 90 },
    { 0xc4bf182d, 90 },
    { 0x994f182e, 90 },
    { 0x8b4f182f, 33 },
    { 0x4f1f182e, 33 },
    { 0xf97f1831, 23 },
    { 0x894f1831, 91 },
    { 0x2dcf182d, 43 },
    { 0xd6df1834, 76 },
    { 0x37cf1835, 46 },
    { 0x15ff1836, 81 },
    { 0x5cff1837, 95 },
    { 0x4a5f1833, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x462f183b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424f1845, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ef1849, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5f184d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f1852, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8df1858, 91 },
    { 0, 0 },
    { 0xc39f185a, 78 },
    { 0, 0 },
    { 0x9ddf185c, 60 },
    { 0xc55f185d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363f1861, 64 },
    { 0, 0 },
    { 0xa98f1863, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf186b, 33 },
    { 0, 0 },
    { 0x48cf186d, 66 },
    { 0, 0 },
    { 0xd13f186f, 33 },
    { 0x3b2f186f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb68f1875, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2f1879, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf187d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54af1881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299f1885, 34 },
    { 0xa46f1885, 66 },
    { 0xc8df1887, 23 },
    { 0, 0 },
    { 0xeeff1889, 76 },
    { 0xa1f1889, 33 },
    { 0xac3f1889, 46 },
    { 0x99f1889, 98 },
    { 0x30af188d, 90 },
    { 0xb49f188d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x388f1891, 92 },
    { 0xd46f1892, 90 },
    { 0x620f1893, 90 },
    { 0x29cf1894, 23 },
    { 0xec9f1893, 90 },
    { 0xfcff1894, 34 },
    { 0xc29f1897, 85 },
    { 0x34ef1896, 91 },
    { 0x8abf1894, 55 },
    { 0x912f189a, 87 },
    { 0xf17f1897, 90 },
    { 0xd15f1893, 67 },
    { 0xb34f1892, 93 },
    { 0x1a7f1893, 95 },
    { 0x669f189f, 33 },
    { 0xe64f189a, 90 },
    { 0, 0 },
    { 0x881f18a2, 85 },
    { 0xe65f18a2, 55 },
    { 0x3a7f18a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ff18aa, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0f18b0, 88 },
    { 0x7b1f18b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14f18b5, 85 },
    { 0x597f18b6, 87 },
    { 0xe91f18b6, 78 },
    { 0xe61f18b8, 33 },
    { 0x21ff18b7, 55 },
    { 0x7ccf18b9, 61 },
    { 0x571f18bb, 55 },
    { 0x2cef18b7, 90 },
    { 0x694f18bd, 33 },
    { 0x457f18be, 90 },
    { 0xf2df18bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb67f18c4, 95 },
    { 0, 0 },
    { 0x3b6f18c6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f18ca, 17 },
    { 0xbf18cb, 66 },
    { 0, 0 },
    { 0x6f4f18cd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06f18d3, 33 },
    { 0xa33f18d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859f18db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf18e1, 67 },
    { 0, 0 },
    { 0x424f18e3, 34 },
    { 0x385f18e3, 33 },
    { 0, 0 },
    { 0x62ef18e6, 23 },
    { 0xfd1f18e7, 23 },
    { 0xa18f18e8, 98 },
    { 0x54ef18e7, 66 },
    { 0xcf8f18ea, 65 },
    { 0xc09f18e8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1f18f6, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1f18fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bf1900, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86af1905, 97 },
    { 0x298f1906, 98 },
    { 0x979f1907, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9f190f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30f1912, 33 },
    { 0, 0 },
    { 0x482f1914, 69 },
    { 0xeb5f1915, 76 },
    { 0, 0 },
    { 0xf0f1917, 55 },
    { 0, 0 },
    { 0x479f1919, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eff191d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37af1922, 46 },
    { 0xb85f1923, 63 },
    { 0, 0 },
    { 0xff2f1925, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62f192a, 76 },
    { 0xcbbf192a, 33 },
    { 0, 0 },
    { 0xe5df192d, 98 },
    { 0x632f192e, 68 },
    { 0, 0 },
    { 0x8acf1930, 61 },
    { 0x824f1930, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x226f1934, 61 },
    { 0x7def1935, 23 },
    { 0xfc7f1936, 34 },
    { 0x55df1937, 34 },
    { 0xbb6f1938, 23 },
    { 0x6a4f1937, 34 },
    { 0xee6f193a, 55 },
    { 0x440f1937, 95 },
    { 0x1cf193c, 33 },
    { 0x9c7f193c, 43 },
    { 0, 0 },
    { 0x980f193f, 98 },
    { 0x2c3f193f, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1f194a, 52 },
    { 0, 0 },
    { 0x777f194c, 43 },
    { 0x9caf194c, 23 },
    { 0xd93f194e, 100 },
    { 0x975f194d, 90 },
    { 0x1e3f194e, 98 },
    { 0x9aff1951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf1954, 90 },
    { 0xb8af1954, 95 },
    { 0x5aef1956, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f1959, 95 },
    { 0, 0 },
    { 0xcaef195b, 43 },
    { 0xea9f195c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x224f195f, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f1964, 77 },
    { 0xd82f1965, 68 },
    { 0xca1f1964, 78 },
    { 0xaacf1967, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ef197a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x520f197d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e6f1982, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31f1985, 55 },
    { 0, 0 },
    { 0x304f1987, 34 },
    { 0xa9df1988, 43 },
    { 0x7d7f1989, 95 },
    { 0x7ef198a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x89af198d, 17 },
    { 0x506f198e, 76 },
    { 0xbb7f198f, 61 },
    { 0xc72f1990, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f1995, 61 },
    { 0, 0 },
    { 0x920f1997, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abf19a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x587f19b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b6f19b8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eef19bb, 88 },
    { 0, 0 },
    { 0xa06f19bd, 66 },
    { 0x4ebf19bd, 84 },
    { 0x73f19bd, 61 },
    { 0x27bf19c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1f19c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ddf19cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42f19ce, 23 },
    { 0x9d0f19ce, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84f19d8, 78 },
    { 0xe33f19d8, 61 },
    { 0xc7ff19d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f19df, 78 },
    { 0xe25f19e0, 98 },
    { 0x91cf19e1, 23 },
    { 0, 0 },
    { 0xaeaf19e3, 95 },
    { 0xe1af19e4, 93 },
    { 0xb98f19e5, 90 },
    { 0xcb4f19e3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ff19ea, 87 },
    { 0xde5f19eb, 55 },
    { 0x3e7f19eb, 84 },
    { 0, 0 },
    { 0x137f19ee, 76 },
    { 0, 0 },
    { 0x4d4f19f0, 95 },
    { 0xb82f19f0, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994f19f6, 99 },
    { 0, 0 },
    { 0x6b1f19f8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1f19fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ef1a01, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956f1a05, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf1a09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f1a0f, 33 },
    { 0x77cf1a10, 43 },
    { 0xc91f1a11, 98 },
    { 0xf29f1a0f, 61 },
    { 0, 0 },
    { 0xe67f1a14, 76 },
    { 0x11ff1a15, 52 },
    { 0x52af1a15, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf1a19, 99 },
    { 0xc32f1a1a, 90 },
    { 0x960f1a1b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x352f1a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71f1a2a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637f1a2e, 90 },
    { 0, 0 },
    { 0x22f1a30, 36 },
    { 0, 0 },
    { 0xd7ef1a32, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f1a37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x822f1a3c, 98 },
    { 0xa79f1a3c, 90 },
    { 0xfadf1a3e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edf1a41, 52 },
    { 0xc3cf1a42, 93 },
    { 0x21f1a41, 2 },
    { 0xdcbf1a44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39f1a4a, 98 },
    { 0xef7f1a4b, 33 },
    { 0xb3bf1a4c, 98 },
    { 0x40ef1a4d, 55 },
    { 0x5e6f1a4e, 36 },
    { 0x29af1a4f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848f1a55, 81 },
    { 0xb28f1a56, 90 },
    { 0, 0 },
    { 0x173f1a58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x282f1a5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94f1a61, 76 },
    { 0, 0 },
    { 0xf12f1a63, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x463f1a69, 66 },
    { 0, 0 },
    { 0x1f8f1a6b, 52 },
    { 0xad4f1a6b, 81 },
    { 0xa2cf1a6c, 33 },
    { 0xe3f1a6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x956f1a71, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885f1a75, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1f1a78, 43 },
    { 0x74af1a79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f1a7c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f1a82, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7f1a88, 85 },
    { 0, 0 },
    { 0x2abf1a8a, 95 },
    { 0x6a9f1a8b, 99 },
    { 0x518f1a8b, 98 },
    { 0x4c3f1a8c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf1a91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6f1a94, 23 },
    { 0xc54f1a94, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499f1a9a, 33 },
    { 0xe69f1a9b, 90 },
    { 0xcc4f1a9c, 55 },
    { 0x11f1a9a, 33 },
    { 0, 0 },
    { 0x803f1a9f, 40 },
    { 0x629f1aa0, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5f1aa9, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161f1aaf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245f1ab4, 53 },
    { 0x3e3f1ab4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62f1aba, 33 },
    { 0x5d7f1aba, 34 },
    { 0x613f1abc, 34 },
    { 0x947f1abd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9f1ac0, 100 },
    { 0x3ecf1ac1, 53 },
    { 0x9ebf1ac1, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x100f1aca, 98 },
    { 0x7b1f1acb, 66 },
    { 0x5c6f1acb, 90 },
    { 0x6c0f1aca, 65 },
    { 0x49ff1acc, 98 },
    { 0x4a6f1acd, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x983f1ad2, 98 },
    { 0x912f1ad3, 68 },
    { 0, 0 },
    { 0x9c1f1ad5, 34 },
    { 0x313f1ad5, 61 },
    { 0, 0 },
    { 0xfd4f1ad8, 90 },
    { 0x3a3f1ad9, 23 },
    { 0x7bbf1ada, 33 },
    { 0xf1adb, 55 },
    { 0x308f1adb, 66 },
    { 0x74ff1add, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x35df1ae0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ef1ae6, 48 },
    { 0x1ecf1ae7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf1af4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4af1af7, 81 },
    { 0xd03f1af7, 23 },
    { 0xa0cf1af9, 52 },
    { 0, 0 },
    { 0xce3f1afb, 61 },
    { 0x736f1afc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdef1aff, 78 },
    { 0x784f1b00, 33 },
    { 0, 0 },
    { 0x9d3f1b02, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1f1b05, 95 },
    { 0x23bf1b06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff1b0b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df1b10, 76 },
    { 0x932f1b10, 61 },
    { 0, 0 },
    { 0x70df1b13, 23 },
    { 0x184f1b14, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f1b1c, 99 },
    { 0x6ecf1b1c, 90 },
    { 0xbc2f1b1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x820f1b27, 81 },
    { 0x96bf1b28, 33 },
    { 0xe7ef1b29, 76 },
    { 0x8d7f1b2a, 33 },
    { 0xf35f1b29, 25 },
    { 0x6c7f1b2b, 84 },
    { 0x9bbf1b2c, 46 },
    { 0xacdf1b2e, 67 },
    { 0xd16f1b2e, 53 },
    { 0xb1df1b30, 98 },
    { 0x466f1b31, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f1b35, 87 },
    { 0x368f1b36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05f1b3d, 93 },
    { 0, 0 },
    { 0x8b7f1b3f, 76 },
    { 0, 0 },
    { 0x4ddf1b41, 61 },
    { 0x4eaf1b41, 33 },
    { 0x5df1b41, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcf1b46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57f1b49, 46 },
    { 0, 0 },
    { 0x87ef1b4b, 97 },
    { 0x6e9f1b4b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04f1b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa4f1b52, 78 },
    { 0x718f1b53, 100 },
    { 0xa28f1b52, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecf1b57, 33 },
    { 0x8f6f1b58, 90 },
    { 0xf87f1b57, 43 },
    { 0xf53f1b5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f1b5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x855f1b62, 98 },
    { 0xaf2f1b63, 43 },
    { 0xbaef1b62, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf1b6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fef1b72, 66 },
    { 0x5f5f1b73, 78 },
    { 0xa11f1b74, 85 },
    { 0, 0 },
    { 0xc4af1b76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x518f1b79, 98 },
    { 0xf4af1b7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x438f1b7d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cf1b81, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45f1b84, 66 },
    { 0x81af1b85, 87 },
    { 0x6baf1b84, 61 },
    { 0x43f1b87, 43 },
    { 0, 0 },
    { 0x9def1b89, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64df1b91, 90 },
    { 0x890f1b92, 23 },
    { 0x98ff1b93, 23 },
    { 0x406f1b92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xecff1b97, 90 },
    { 0x4bbf1b97, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68f1ba2, 46 },
    { 0, 0 },
    { 0xf5cf1ba4, 61 },
    { 0, 0 },
    { 0xebef1ba6, 33 },
    { 0, 0 },
    { 0xc1af1ba8, 84 },
    { 0xcbaf1ba9, 90 },
    { 0, 0 },
    { 0x179f1bab, 23 },
    { 0x5b5f1bac, 43 },
    { 0, 0 },
    { 0x1fef1bae, 53 },
    { 0x3d3f1baf, 90 },
    { 0xda3f1bb0, 61 },
    { 0x7b5f1baf, 61 },
    { 0, 0 },
    { 0xed6f1bb3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5f1bbe, 33 },
    { 0x1f0f1bbe, 52 },
    { 0, 0 },
    { 0x420f1bc1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f1bcc, 61 },
    { 0xeb3f1bcd, 99 },
    { 0x66df1bcd, 61 },
    { 0xe47f1bcd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bbf1bd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ef1bda, 33 },
    { 0, 0 },
    { 0xf46f1bdc, 33 },
    { 0x94af1bdc, 77 },
    { 0x4acf1bde, 76 },
    { 0xb80f1bdf, 66 },
    { 0xb20f1bde, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94df1be5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0f1be9, 95 },
    { 0x27bf1bea, 75 },
    { 0xc24f1be9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f1bf3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecf1bf6, 34 },
    { 0, 0 },
    { 0x3dbf1bf8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf1bfb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13f1bff, 98 },
    { 0xeb9f1c00, 91 },
    { 0, 0 },
    { 0xb8ef1c02, 33 },
    { 0x815f1c02, 69 },
    { 0x4d1f1c04, 33 },
    { 0, 0 },
    { 0x2eef1c06, 61 },
    { 0x6e0f1c06, 90 },
    { 0xa97f1c08, 55 },
    { 0x25f1c08, 33 },
    { 0x121f1c0a, 81 },
    { 0x86ff1c0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x94f1c0e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1df1c12, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502f1c16, 98 },
    { 0xd86f1c17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf15f1c1b, 34 },
    { 0, 0 },
    { 0xf67f1c1d, 98 },
    { 0, 0 },
    { 0xe3bf1c1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79f1c23, 95 },
    { 0x260f1c24, 85 },
    { 0x818f1c25, 77 },
    { 0x537f1c25, 95 },
    { 0x64bf1c26, 61 },
    { 0xb24f1c28, 61 },
    { 0, 0 },
    { 0xbe6f1c2a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdf1c2f, 36 },
    { 0xad8f1c30, 95 },
    { 0x75f1c30, 77 },
    { 0xdff1c31, 95 },
    { 0x3fdf1c33, 61 },
    { 0xe4bf1c34, 2 },
    { 0xe12f1c35, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x674f1c38, 43 },
    { 0x52f1c39, 85 },
    { 0xd6f1c3a, 43 },
    { 0x707f1c39, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68df1c40, 61 },
    { 0, 0 },
    { 0x152f1c42, 43 },
    { 0, 0 },
    { 0x75f1c44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ef1c49, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf1c4d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bef1c53, 33 },
    { 0x7bdf1c54, 61 },
    { 0x683f1c55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x792f1c58, 52 },
    { 0x775f1c59, 84 },
    { 0xbcff1c58, 90 },
    { 0, 0 },
    { 0xee6f1c5c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0f1c63, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd53f1c66, 90 },
    { 0x635f1c67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271f1c6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x764f1c6e, 90 },
    { 0xe12f1c6e, 55 },
    { 0x9f1c70, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703f1c76, 66 },
    { 0xc05f1c77, 33 },
    { 0xa44f1c78, 90 },
    { 0xc76f1c77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc31f1c81, 90 },
    { 0x644f1c82, 65 },
    { 0xa53f1c83, 61 },
    { 0xebcf1c84, 65 },
    { 0x15cf1c83, 100 },
    { 0x342f1c86, 68 },
    { 0xf59f1c86, 23 },
    { 0x732f1c86, 33 },
    { 0xe5cf1c84, 95 },
    { 0xb15f1c86, 98 },
    { 0xd1f1c8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2f1c93, 34 },
    { 0, 0 },
    { 0xc09f1c95, 76 },
    { 0x8b7f1c96, 61 },
    { 0, 0 },
    { 0xf85f1c98, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f1c9b, 33 },
    { 0x83df1c9c, 66 },
    { 0x981f1c9d, 33 },
    { 0xdf4f1c9c, 61 },
    { 0xd9bf1c9b, 98 },
    { 0, 0 },
    { 0x315f1ca1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf1ca4, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ef1cac, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff1caf, 51 },
    { 0, 0 },
    { 0x3b6f1cb1, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ef1cb7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebf1cbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x115f1cc1, 37 },
    { 0xa1f1cc1, 3 },
    { 0xaedf1cc3, 46 },
    { 0x175f1cc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf1cc7, 90 },
    { 0x37ef1cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x771f1cd4, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df1cd8, 23 },
    { 0, 0 },
    { 0xbf0f1cda, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64f1cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf1f1ce2, 83 },
    { 0xe50f1ce2, 93 },
    { 0xf78f1ce2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x221f1ceb, 90 },
    { 0, 0 },
    { 0xae2f1ced, 90 },
    { 0x2b8f1cee, 43 },
    { 0, 0 },
    { 0x4f1cf0, 76 },
    { 0xac2f1cf0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9f1cf6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4f1cf9, 77 },
    { 0x453f1cfa, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2f1cff, 33 },
    { 0x1d7f1cff, 43 },
    { 0x2f7f1cff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x672f1d04, 33 },
    { 0x378f1d05, 33 },
    { 0xb72f1d06, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743f1d0d, 23 },
    { 0, 0 },
    { 0x6ecf1d0f, 61 },
    { 0, 0 },
    { 0x27ff1d11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ef1d1a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bef1d1d, 81 },
    { 0xf37f1d1e, 95 },
    { 0, 0 },
    { 0x37ef1d20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2f1d23, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x342f1d26, 98 },
    { 0x1eef1d26, 100 },
    { 0x505f1d28, 33 },
    { 0, 0 },
    { 0x6b2f1d2a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x709f1d2f, 93 },
    { 0, 0 },
    { 0x33ff1d31, 61 },
    { 0x8b6f1d32, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x965f1d37, 98 },
    { 0x2e8f1d37, 93 },
    { 0xd70f1d39, 33 },
    { 0, 0 },
    { 0x921f1d3b, 90 },
    { 0x58df1d3c, 99 },
    { 0x3af1d3d, 95 },
    { 0xf9df1d3b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x93df1d41, 98 },
    { 0, 0 },
    { 0xf8bf1d43, 34 },
    { 0x573f1d43, 69 },
    { 0xc41f1d45, 46 },
    { 0x918f1d45, 23 },
    { 0, 0 },
    { 0x2f4f1d48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef1d4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cf1d4f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf1d58, 61 },
    { 0x152f1d58, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681f1d62, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ef1d65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196f1d6a, 33 },
    { 0x232f1d6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f1d72, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25f1d76, 90 },
    { 0, 0 },
    { 0xc42f1d78, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656f1d7d, 61 },
    { 0x9faf1d7d, 23 },
    { 0, 0 },
    { 0xe34f1d80, 90 },
    { 0, 0 },
    { 0x839f1d82, 91 },
    { 0x33ef1d82, 52 },
    { 0, 0 },
    { 0xf05f1d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf1d88, 66 },
    { 0xc66f1d89, 90 },
    { 0x60af1d89, 38 },
    { 0, 0 },
    { 0xaf7f1d8c, 66 },
    { 0, 0 },
    { 0xf5df1d8e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42f1d91, 98 },
    { 0x55ef1d92, 85 },
    { 0xf2bf1d93, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ef1d99, 61 },
    { 0, 0 },
    { 0x771f1d9b, 46 },
    { 0x6cdf1d9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5f1da2, 23 },
    { 0x3c3f1da3, 95 },
    { 0xa34f1da4, 69 },
    { 0x289f1da5, 91 },
    { 0, 0 },
    { 0xed9f1da7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f1dac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ff1db0, 61 },
    { 0xdf3f1db1, 95 },
    { 0, 0 },
    { 0x174f1db3, 34 },
    { 0xed9f1db4, 34 },
    { 0xe82f1db3, 90 },
    { 0, 0 },
    { 0x175f1db7, 85 },
    { 0xc11f1db8, 61 },
    { 0x9aff1db9, 90 },
    { 0x99ef1dba, 61 },
    { 0, 0 },
    { 0xb8df1dbc, 55 },
    { 0xe88f1dbd, 66 },
    { 0x116f1dbe, 33 },
    { 0x8ebf1dbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8f1dc5, 84 },
    { 0x4d7f1dc6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acf1dc9, 95 },
    { 0x585f1dc9, 90 },
    { 0xcc5f1dcb, 23 },
    { 0, 0 },
    { 0x6d1f1dcd, 33 },
    { 0x49bf1dce, 95 },
    { 0, 0 },
    { 0xc0df1dd0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252f1dd4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0f1dd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f1ddc, 98 },
    { 0x367f1ddd, 98 },
    { 0x3fdf1dde, 75 },
    { 0, 0 },
    { 0x462f1de0, 78 },
    { 0xffef1de1, 67 },
    { 0x735f1de2, 66 },
    { 0xb2df1de2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760f1dee, 87 },
    { 0x863f1def, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf1df7, 98 },
    { 0xe77f1df8, 61 },
    { 0x6fdf1df7, 34 },
    { 0, 0 },
    { 0x99ff1dfb, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1f1dfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eff1e06, 33 },
    { 0x43af1e06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef1e0e, 61 },
    { 0, 0 },
    { 0x6a8f1e10, 76 },
    { 0xf24f1e11, 90 },
    { 0x38f1e11, 33 },
    { 0, 0 },
    { 0xba7f1e14, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1f1e1b, 43 },
    { 0x7d4f1e1c, 98 },
    { 0x777f1e1d, 55 },
    { 0x995f1e1e, 33 },
    { 0xf98f1e1d, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4f1e2b, 68 },
    { 0x2d6f1e2c, 43 },
    { 0x34df1e2d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f1e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0f1e38, 77 },
    { 0xd9ef1e39, 33 },
    { 0, 0 },
    { 0x9fff1e3b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ff1e3e, 52 },
    { 0x3d5f1e3f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750f1e46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc74f1e4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbef1e52, 67 },
    { 0xdb0f1e53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fef1e59, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edf1e5d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2f1e67, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ff1e6c, 23 },
    { 0xf2bf1e6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df1e70, 87 },
    { 0x747f1e70, 88 },
    { 0x3b5f1e72, 61 },
    { 0xa63f1e70, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5af1e7a, 34 },
    { 0, 0 },
    { 0xf9df1e7c, 55 },
    { 0xa00f1e7d, 43 },
    { 0xd43f1e7e, 34 },
    { 0x888f1e7e, 55 },
    { 0x609f1e80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ff1e83, 23 },
    { 0x263f1e84, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373f1e88, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bf1e90, 61 },
    { 0xe32f1e91, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdf1e94, 33 },
    { 0, 0 },
    { 0x844f1e96, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa79f1e9a, 77 },
    { 0x137f1e9b, 55 },
    { 0xca5f1e9c, 76 },
    { 0xe79f1e9d, 33 },
    { 0xef2f1e9d, 98 },
    { 0xb1df1e9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0af1ea2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29f1ea5, 90 },
    { 0xa24f1ea5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7af1eaa, 67 },
    { 0x20af1eab, 61 },
    { 0xc85f1eab, 95 },
    { 0x1b8f1ead, 34 },
    { 0x8f6f1eae, 46 },
    { 0xee5f1eae, 90 },
    { 0xef6f1ead, 98 },
    { 0x38bf1eb1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1f1eba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caf1ebd, 68 },
    { 0xefef1ebd, 61 },
    { 0, 0 },
    { 0x9e7f1ec0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47f1ece, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ef1ed3, 43 },
    { 0, 0 },
    { 0x218f1ed5, 83 },
    { 0x7dcf1ed6, 33 },
    { 0xfe0f1ed5, 33 },
    { 0x176f1ed8, 33 },
    { 0x3d7f1ed9, 78 },
    { 0x7bff1ed9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x380f1edd, 61 },
    { 0x440f1ede, 43 },
    { 0, 0 },
    { 0xb8af1ee0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363f1ee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf48f1ee9, 81 },
    { 0x1edf1eea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbf1eed, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f1ef7, 52 },
    { 0xa05f1ef7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef1efb, 33 },
    { 0xdf9f1efc, 76 },
    { 0x1c4f1efd, 17 },
    { 0x2fdf1efe, 98 },
    { 0, 0 },
    { 0x68af1f00, 61 },
    { 0xffbf1f01, 61 },
    { 0, 0 },
    { 0x17af1f03, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fbf1f0b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x557f1f0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x174f1f18, 23 },
    { 0x422f1f19, 61 },
    { 0x5a1f1f1a, 52 },
    { 0x907f1f1b, 33 },
    { 0x771f1f1a, 84 },
    { 0xa6af1f1d, 23 },
    { 0x95df1f1e, 97 },
    { 0x2f4f1f19, 34 },
    { 0x8a1f1f20, 17 },
    { 0xcc5f1f1a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x190f1f24, 23 },
    { 0x13bf1f25, 90 },
    { 0x1c3f1f26, 61 },
    { 0xd35f1f26, 33 },
    { 0xc86f1f27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834f1f2e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f1f3b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ff1f3e, 95 },
    { 0xb1df1f3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfef1f42, 61 },
    { 0, 0 },
    { 0x896f1f44, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234f1f49, 67 },
    { 0x454f1f4a, 55 },
    { 0x16df1f4a, 23 },
    { 0x961f1f49, 80 },
    { 0xe82f1f49, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64af1f51, 90 },
    { 0xcff1f52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0f1f58, 95 },
    { 0x6e6f1f59, 98 },
    { 0x77af1f59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1f1f63, 52 },
    { 0x46bf1f63, 99 },
    { 0x88df1f65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cf1f6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x827f1f6d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cef1f70, 85 },
    { 0x73f1f71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0ff1f75, 95 },
    { 0x9a1f1f75, 75 },
    { 0x7ecf1f77, 55 },
    { 0xd8df1f78, 33 },
    { 0xccef1f78, 98 },
    { 0x25bf1f79, 61 },
    { 0, 0 },
    { 0x1fbf1f7c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401f1f80, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff1f85, 43 },
    { 0xa3ef1f86, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff1f8e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb43f1f93, 66 },
    { 0x1f6f1f94, 85 },
    { 0xb03f1f95, 77 },
    { 0xdc3f1f93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08f1f9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783f1fa0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0f1fa5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38af1fa9, 100 },
    { 0x536f1faa, 23 },
    { 0xe90f1fab, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cf1fb5, 38 },
    { 0x53bf1fb5, 23 },
    { 0, 0 },
    { 0xf2af1fb8, 76 },
    { 0xe94f1fb9, 98 },
    { 0x599f1fb8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f1fbf, 87 },
    { 0x3c6f1fc0, 93 },
    { 0x3bf1fc1, 55 },
    { 0, 0 },
    { 0xfe4f1fc3, 66 },
    { 0, 0 },
    { 0xda9f1fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43f1fc8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x193f1fcc, 84 },
    { 0, 0 },
    { 0x80ff1fce, 61 },
    { 0x57cf1fcf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acf1fd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f1fdb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26f1fe4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d9f1feb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f1ffd, 61 },
    { 0x5a1f1ffe, 90 },
    { 0xf03f1ffd, 90 },
    { 0, 0 },
    { 0x65f2001, 90 },
    { 0xae3f2002, 91 },
    { 0xc77f2001, 90 },
    { 0x306f2003, 51 },
    { 0x663f2005, 90 },
    { 0x401f2006, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28f200e, 98 },
    { 0xe7cf200e, 91 },
    { 0, 0 },
    { 0x9d9f2011, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9f2014, 90 },
    { 0x9c1f2015, 23 },
    { 0, 0 },
    { 0x4fdf2017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cf201a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff2021, 90 },
    { 0xa35f2022, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdecf2029, 40 },
    { 0, 0 },
    { 0x1f3f202b, 33 },
    { 0xcc5f202b, 90 },
    { 0x88ef202d, 23 },
    { 0x57df202c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf2033, 97 },
    { 0xcf7f2034, 98 },
    { 0x5a4f2033, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x87af2038, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ef203b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abf203e, 76 },
    { 0x8e7f203f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c1f204a, 55 },
    { 0xe2ff204b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ef2055, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddf2059, 52 },
    { 0xf7af205a, 55 },
    { 0x7bff2059, 33 },
    { 0xe70f205c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf2061, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25f2064, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f2069, 69 },
    { 0x258f206a, 93 },
    { 0xbaaf206b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdaf206e, 95 },
    { 0x415f206f, 61 },
    { 0xf41f2070, 55 },
    { 0x7f1f206e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x757f2074, 88 },
    { 0x3b2f2075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd80f2078, 90 },
    { 0, 0 },
    { 0x3cef207a, 33 },
    { 0x9ecf207b, 75 },
    { 0x668f207c, 98 },
    { 0xe6bf207d, 23 },
    { 0xbe5f207e, 23 },
    { 0x405f207c, 91 },
    { 0xb91f207d, 100 },
    { 0xeb4f2080, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x795f2084, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f2087, 61 },
    { 0xc3df2087, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f208c, 46 },
    { 0x722f208d, 100 },
    { 0x6f208e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8f2092, 87 },
    { 0, 0 },
    { 0x546f2094, 100 },
    { 0, 0 },
    { 0x648f2096, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x329f209d, 61 },
    { 0x1d2f209d, 98 },
    { 0, 0 },
    { 0xdbaf20a0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adf20a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874f20ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22f20b4, 90 },
    { 0x7d6f20b5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0f20b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ef20c0, 100 },
    { 0xdbf20c1, 23 },
    { 0xa75f20c2, 55 },
    { 0xd51f20c2, 95 },
    { 0, 0 },
    { 0xa0af20c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415f20cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb84f20d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733f20d4, 61 },
    { 0x6c3f20d5, 93 },
    { 0xbb9f20d6, 68 },
    { 0x63ff20d7, 80 },
    { 0xe7df20d8, 52 },
    { 0xbfdf20d8, 95 },
    { 0x947f20d8, 43 },
    { 0xddbf20db, 100 },
    { 0xaa2f20dc, 84 },
    { 0xcd5f20d7, 39 },
    { 0x210f20d4, 90 },
    { 0xd49f20d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f20e5, 90 },
    { 0x325f20e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8f20e9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18f20ee, 59 },
    { 0x672f20ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5f20f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f20fe, 61 },
    { 0xf3f20ff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557f2103, 76 },
    { 0, 0 },
    { 0xf3cf2105, 23 },
    { 0x330f2105, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fff2110, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4af2115, 34 },
    { 0x7aef2115, 95 },
    { 0, 0 },
    { 0xee2f2118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4f211b, 88 },
    { 0x97ff211b, 61 },
    { 0x4cff211d, 90 },
    { 0x2eaf211e, 61 },
    { 0, 0 },
    { 0x12ff2120, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df2126, 46 },
    { 0x781f2127, 98 },
    { 0, 0 },
    { 0xa95f2129, 93 },
    { 0xdd6f212a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6af212e, 98 },
    { 0, 0 },
    { 0x3cbf2130, 95 },
    { 0xef0f2130, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x235f2134, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09f213a, 61 },
    { 0x827f213b, 90 },
    { 0x6d1f213b, 61 },
    { 0x248f213c, 98 },
    { 0x5ff213e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x886f2147, 33 },
    { 0x78bf2147, 55 },
    { 0x9c4f2148, 90 },
    { 0, 0 },
    { 0x43bf214b, 55 },
    { 0, 0 },
    { 0xde5f214d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8f2151, 95 },
    { 0xa20f2152, 81 },
    { 0, 0 },
    { 0xe51f2154, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbf215a, 61 },
    { 0xa32f215b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f216a, 33 },
    { 0x277f216a, 34 },
    { 0xb34f216c, 85 },
    { 0, 0 },
    { 0x8cdf216e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6f2171, 76 },
    { 0, 0 },
    { 0x2ef2173, 3 },
    { 0, 0 },
    { 0xca3f2175, 90 },
    { 0, 0 },
    { 0xebaf2177, 90 },
    { 0, 0 },
    { 0x5e3f2179, 90 },
    { 0xdbf217a, 90 },
    { 0xa29f217a, 34 },
    { 0xb9cf217c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79f2182, 90 },
    { 0, 0 },
    { 0x3aff2184, 34 },
    { 0x261f2184, 46 },
    { 0x257f2186, 98 },
    { 0xd39f2186, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49f218a, 90 },
    { 0, 0 },
    { 0x12f218c, 98 },
    { 0xbb3f218d, 78 },
    { 0x2e7f218d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cf2191, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9caf2195, 55 },
    { 0, 0 },
    { 0x300f2197, 93 },
    { 0xd14f2198, 55 },
    { 0xcbf2198, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f219d, 98 },
    { 0xa99f219d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ff21a1, 90 },
    { 0x666f21a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bef21a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f21ae, 61 },
    { 0, 0 },
    { 0xcbdf21b0, 55 },
    { 0xc1ff21b1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cf21b5, 90 },
    { 0x821f21b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6f21bc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f21c3, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9bf21ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaf21d1, 95 },
    { 0xfe9f21d2, 85 },
    { 0, 0 },
    { 0x9b3f21d4, 61 },
    { 0x2a1f21d5, 33 },
    { 0, 0 },
    { 0x607f21d7, 90 },
    { 0, 0 },
    { 0x105f21d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadf21dc, 90 },
    { 0xca1f21dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x556f21e1, 90 },
    { 0, 0 },
    { 0x3e0f21e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2f21e9, 96 },
    { 0x6cff21ea, 33 },
    { 0x80bf21eb, 23 },
    { 0x7b8f21ec, 69 },
    { 0x745f21ed, 55 },
    { 0, 0 },
    { 0x37bf21ef, 33 },
    { 0xbf5f21f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f21f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f21fc, 77 },
    { 0xbe5f21fd, 61 },
    { 0xef9f21fe, 90 },
    { 0xb94f21ff, 43 },
    { 0xb68f21ff, 43 },
    { 0x79f21fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4f2205, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ef220b, 67 },
    { 0x44cf220c, 43 },
    { 0x73ff220d, 95 },
    { 0xab4f220d, 36 },
    { 0x84af220f, 43 },
    { 0x8c6f2210, 61 },
    { 0x40ff220f, 33 },
    { 0x9a6f2212, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8f2216, 88 },
    { 0xf97f2216, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf221c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54f2220, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x63df2223, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ff222c, 77 },
    { 0, 0 },
    { 0x1edf222e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3f2234, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f2238, 33 },
    { 0xf2af2238, 90 },
    { 0, 0 },
    { 0x59af223b, 66 },
    { 0x9caf223c, 90 },
    { 0x688f223d, 98 },
    { 0, 0 },
    { 0x49df223f, 93 },
    { 0xc24f223f, 34 },
    { 0xbc9f2241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cf2247, 90 },
    { 0xb05f2248, 97 },
    { 0x39af2249, 95 },
    { 0x967f2248, 83 },
    { 0x6f1f224a, 78 },
    { 0xabdf2249, 69 },
    { 0x537f224a, 98 },
    { 0xc6af224b, 46 },
    { 0x666f224f, 95 },
    { 0x2f224b, 85 },
    { 0, 0 },
    { 0x793f2252, 99 },
    { 0x176f2252, 95 },
    { 0xc71f2254, 91 },
    { 0, 0 },
    { 0x19df2256, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ef225a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x858f2262, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338f2267, 90 },
    { 0xbadf2267, 33 },
    { 0xdf6f2269, 23 },
    { 0x1def2267, 55 },
    { 0x3c8f226b, 91 },
    { 0x8daf226c, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6f2271, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdf2275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260f227e, 61 },
    { 0x999f227f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1f2284, 61 },
    { 0x940f2284, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf228a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495f228e, 90 },
    { 0, 0 },
    { 0xb07f2290, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x958f229c, 90 },
    { 0, 0 },
    { 0x539f229e, 76 },
    { 0x8b7f229e, 98 },
    { 0, 0 },
    { 0x14ef22a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d4f22b8, 61 },
    { 0x538f22b9, 98 },
    { 0x80bf22b9, 90 },
    { 0xf5af22b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff22be, 34 },
    { 0, 0 },
    { 0x6a3f22c0, 66 },
    { 0, 0 },
    { 0x5fef22c2, 56 },
    { 0, 0 },
    { 0x746f22c4, 90 },
    { 0x92df22c5, 72 },
    { 0xdfcf22c5, 55 },
    { 0xbc4f22c7, 95 },
    { 0, 0 },
    { 0x575f22c9, 23 },
    { 0, 0 },
    { 0xa33f22cb, 90 },
    { 0xea0f22cb, 23 },
    { 0xddef22cb, 61 },
    { 0xfc8f22ce, 95 },
    { 0, 0 },
    { 0xbfff22d0, 76 },
    { 0, 0 },
    { 0xefff22d2, 98 },
    { 0xa36f22d3, 90 },
    { 0x9dff22d2, 55 },
    { 0x7e0f22d4, 33 },
    { 0x2faf22d6, 43 },
    { 0, 0 },
    { 0xf44f22d8, 41 },
    { 0xb22f22d9, 43 },
    { 0x80ef22d8, 55 },
    { 0x108f22db, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8f22df, 66 },
    { 0x5e9f22e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ef22e4, 100 },
    { 0x553f22e5, 98 },
    { 0x8cdf22e5, 98 },
    { 0xe1f22e4, 55 },
    { 0x7f2f22e8, 61 },
    { 0, 0 },
    { 0x1d9f22ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f22f8, 95 },
    { 0x6eaf22f8, 90 },
    { 0x4ecf22fa, 98 },
    { 0x864f22fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb4f2302, 66 },
    { 0, 0 },
    { 0x4b8f2304, 69 },
    { 0, 0 },
    { 0xaaaf2306, 43 },
    { 0x8c3f2307, 44 },
    { 0xdc6f2307, 95 },
    { 0, 0 },
    { 0x75f230a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7f230e, 66 },
    { 0xa13f230e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36f2312, 55 },
    { 0, 0 },
    { 0xed6f2314, 98 },
    { 0xe48f2315, 33 },
    { 0xbdef2316, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8f231d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7f2327, 55 },
    { 0x7c5f2327, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x417f232b, 98 },
    { 0x6abf232c, 61 },
    { 0xebff232d, 76 },
    { 0xf78f232d, 90 },
    { 0x59f232e, 76 },
    { 0xceaf2330, 66 },
    { 0x7b3f2330, 55 },
    { 0x237f2332, 90 },
    { 0, 0 },
    { 0x3aef2334, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dff2337, 95 },
    { 0x808f2338, 23 },
    { 0x3c8f2339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62f233d, 81 },
    { 0xeb9f233e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf2343, 34 },
    { 0, 0 },
    { 0x118f2345, 61 },
    { 0x92bf2345, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x91af2349, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f234d, 66 },
    { 0, 0 },
    { 0x177f234f, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0x70af2352, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x69df2355, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508f2359, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5f235e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f2365, 95 },
    { 0xf8df2366, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49f236e, 61 },
    { 0xcb5f236f, 95 },
    { 0xd52f2370, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf2373, 61 },
    { 0x98df2373, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf2378, 23 },
    { 0, 0 },
    { 0x997f237a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea7f2380, 90 },
    { 0xd43f2381, 95 },
    { 0, 0 },
    { 0x3d9f2383, 93 },
    { 0x361f2384, 76 },
    { 0, 0 },
    { 0xd5f2386, 85 },
    { 0x936f2387, 76 },
    { 0x9fcf2387, 23 },
    { 0xaa8f2386, 23 },
    { 0x9cef2386, 99 },
    { 0x1ef238b, 90 },
    { 0xf4ef238b, 84 },
    { 0x560f238c, 95 },
    { 0, 0 },
    { 0x8dcf238f, 99 },
    { 0, 0 },
    { 0xb12f2391, 53 },
    { 0, 0 },
    { 0xbf8f2393, 23 },
    { 0x711f2393, 95 },
    { 0x8cef2395, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f2399, 23 },
    { 0xfd0f239a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274f239f, 95 },
    { 0x87df23a0, 61 },
    { 0x6b0f23a1, 33 },
    { 0x798f239f, 23 },
    { 0, 0 },
    { 0x897f23a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf23a8, 46 },
    { 0x229f23a8, 98 },
    { 0x1a5f23a9, 98 },
    { 0, 0 },
    { 0x596f23ac, 61 },
    { 0xc3ef23ac, 95 },
    { 0x3af23ae, 33 },
    { 0, 0 },
    { 0x290f23b0, 34 },
    { 0x813f23b0, 61 },
    { 0, 0 },
    { 0xd5cf23b3, 23 },
    { 0xcc2f23b3, 99 },
    { 0, 0 },
    { 0xcaaf23b6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5f23bb, 33 },
    { 0, 0 },
    { 0x370f23bd, 87 },
    { 0, 0 },
    { 0x458f23bf, 34 },
    { 0xcdff23c0, 64 },
    { 0x7f23c1, 90 },
    { 0xf30f23c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7f23c6, 90 },
    { 0x138f23c7, 84 },
    { 0x8eaf23c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x470f23cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf23d2, 95 },
    { 0xeb2f23d2, 61 },
    { 0, 0 },
    { 0xcddf23d5, 81 },
    { 0x9dbf23d6, 90 },
    { 0xbe5f23d7, 68 },
    { 0xee6f23d6, 90 },
    { 0x7e3f23d9, 23 },
    { 0xda8f23d6, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28f23e7, 90 },
    { 0xa49f23e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf23f4, 84 },
    { 0, 0 },
    { 0xb89f23f6, 33 },
    { 0, 0 },
    { 0x1ecf23f8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8df23fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481f2400, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f2408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f240e, 34 },
    { 0xe3df240e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368f2413, 66 },
    { 0, 0 },
    { 0x3c8f2415, 98 },
    { 0x6cbf2416, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe35f241b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd8f241e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92f2421, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310f2425, 68 },
    { 0xce3f2426, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2f242d, 90 },
    { 0x20ef242e, 99 },
    { 0xb5ff242e, 66 },
    { 0xeff2430, 95 },
    { 0xc86f2431, 88 },
    { 0x56bf2431, 85 },
    { 0xf67f2432, 34 },
    { 0x28f2432, 90 },
    { 0xc4bf242f, 55 },
    { 0x9f9f2431, 99 },
    { 0x328f2437, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790f243c, 90 },
    { 0xdf6f243c, 55 },
    { 0x956f243e, 68 },
    { 0x922f243c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f2443, 95 },
    { 0x30f2444, 55 },
    { 0, 0 },
    { 0x70f2446, 33 },
    { 0x6cff2446, 23 },
    { 0xa6af2448, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bcf244f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4ff2458, 56 },
    { 0, 0 },
    { 0x322f245a, 61 },
    { 0x33cf245b, 98 },
    { 0x215f245b, 93 },
    { 0x7e1f245d, 88 },
    { 0xfbaf245e, 95 },
    { 0, 0 },
    { 0xa81f2460, 78 },
    { 0xe09f2461, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ff2464, 33 },
    { 0x6d2f2465, 66 },
    { 0, 0 },
    { 0x991f2467, 98 },
    { 0x179f2468, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf246c, 67 },
    { 0xe9cf246d, 52 },
    { 0x729f246e, 90 },
    { 0x8a8f246f, 61 },
    { 0x1c3f2470, 34 },
    { 0xc4af246f, 99 },
    { 0, 0 },
    { 0x8d9f2473, 78 },
    { 0x374f2474, 55 },
    { 0, 0 },
    { 0x3e2f2476, 87 },
    { 0, 0 },
    { 0x42f2478, 95 },
    { 0, 0 },
    { 0x266f247a, 43 },
    { 0xe9bf247b, 23 },
    { 0x2ecf247c, 67 },
    { 0x954f247d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bf2481, 77 },
    { 0x225f2482, 90 },
    { 0x6cbf2483, 33 },
    { 0xb91f2484, 33 },
    { 0, 0 },
    { 0x277f2486, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cf248d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x310f2496, 68 },
    { 0x70cf2496, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7f249b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edf24a3, 95 },
    { 0xcd6f24a4, 55 },
    { 0, 0 },
    { 0xb21f24a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f24ac, 84 },
    { 0, 0 },
    { 0x50af24ae, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96af24b1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f24b5, 33 },
    { 0xc5ef24b6, 66 },
    { 0xa81f24b7, 34 },
    { 0, 0 },
    { 0x441f24b9, 43 },
    { 0x287f24b9, 100 },
    { 0, 0 },
    { 0x61ef24bc, 93 },
    { 0x575f24bc, 56 },
    { 0x1a6f24bd, 87 },
    { 0x23df24bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fdf24c7, 98 },
    { 0x4a6f24c8, 61 },
    { 0xc2ff24c8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bf24d1, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f24d4, 66 },
    { 0xd00f24d4, 43 },
    { 0xb8af24d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2af24dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf24e3, 91 },
    { 0x7a1f24e4, 61 },
    { 0x472f24e4, 34 },
    { 0, 0 },
    { 0xfa4f24e7, 91 },
    { 0, 0 },
    { 0xd6ef24e9, 33 },
    { 0xb8af24e9, 98 },
    { 0x796f24ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f24ee, 76 },
    { 0, 0 },
    { 0xc06f24f0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ff24f7, 90 },
    { 0xe3cf24f7, 46 },
    { 0, 0 },
    { 0xda2f24fa, 55 },
    { 0, 0 },
    { 0x16bf24fc, 95 },
    { 0, 0 },
    { 0x724f24fe, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ff250a, 34 },
    { 0x140f250b, 68 },
    { 0, 0 },
    { 0xa3ff250d, 90 },
    { 0x590f250d, 61 },
    { 0, 0 },
    { 0xdd2f2510, 33 },
    { 0x1d5f2511, 100 },
    { 0x9b6f2511, 99 },
    { 0x847f2512, 61 },
    { 0, 0 },
    { 0x527f2515, 61 },
    { 0xd78f2516, 99 },
    { 0x58ef2515, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66f251b, 23 },
    { 0x33af251c, 55 },
    { 0xc93f251d, 52 },
    { 0x161f251b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ebf2526, 98 },
    { 0x12bf2526, 33 },
    { 0x434f2526, 33 },
    { 0xa33f2529, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f252e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cf2532, 61 },
    { 0, 0 },
    { 0xe27f2534, 95 },
    { 0xb3ef2534, 66 },
    { 0, 0 },
    { 0xcd1f2537, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24f253c, 76 },
    { 0, 0 },
    { 0xbb4f253e, 55 },
    { 0xa9ef253f, 77 },
    { 0x379f253f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75f2546, 66 },
    { 0x368f2547, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf254a, 90 },
    { 0x36ef254b, 33 },
    { 0, 0 },
    { 0x33f254d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bf2550, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eef255b, 58 },
    { 0x4eaf255c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf21f255f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x598f2565, 98 },
    { 0xb3f2566, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd9f2571, 51 },
    { 0x57f2571, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8f2576, 55 },
    { 0, 0 },
    { 0x7b0f2578, 23 },
    { 0xc25f2579, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bf257d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0f2580, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f2585, 93 },
    { 0xbabf2585, 76 },
    { 0xe72f2587, 90 },
    { 0x535f2588, 23 },
    { 0xb58f2589, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f2592, 65 },
    { 0x885f2593, 84 },
    { 0x25df2594, 90 },
    { 0xef2595, 33 },
    { 0xf0cf2596, 66 },
    { 0x6e0f2596, 80 },
    { 0x5b9f2596, 90 },
    { 0x1f3f2599, 90 },
    { 0xc75f259a, 34 },
    { 0x2bff2595, 99 },
    { 0x759f259c, 61 },
    { 0x470f259d, 90 },
    { 0xebcf2595, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98f25a5, 95 },
    { 0, 0 },
    { 0x766f25a7, 52 },
    { 0x552f25a7, 95 },
    { 0xe3ff25a8, 95 },
    { 0x840f25aa, 90 },
    { 0xd0ef25a7, 76 },
    { 0, 0 },
    { 0xad9f25ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91f25b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf25bd, 61 },
    { 0xc9f25be, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4df25c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104f25cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651f25d1, 90 },
    { 0, 0 },
    { 0x34df25d3, 98 },
    { 0, 0 },
    { 0xcbdf25d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeff25d8, 76 },
    { 0xf9df25d8, 85 },
    { 0x2cf25da, 95 },
    { 0xb82f25d9, 40 },
    { 0x407f25dc, 61 },
    { 0x332f25dc, 55 },
    { 0xd2f25de, 56 },
    { 0, 0 },
    { 0x334f25e0, 90 },
    { 0, 0 },
    { 0x686f25e2, 23 },
    { 0x385f25e3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171f25e7, 75 },
    { 0x5bbf25e8, 98 },
    { 0, 0 },
    { 0xcbbf25ea, 90 },
    { 0x219f25ea, 61 },
    { 0xc4ff25ea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x821f25f3, 95 },
    { 0, 0 },
    { 0x873f25f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f25f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f25ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef2611, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x334f2614, 33 },
    { 0x5c7f2615, 90 },
    { 0x791f2616, 95 },
    { 0, 0 },
    { 0xe1df2618, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa59f261f, 23 },
    { 0xf25f2620, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1f2625, 72 },
    { 0, 0 },
    { 0x63ff2627, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5f2631, 52 },
    { 0xb8ff2631, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cf2636, 33 },
    { 0x4abf2637, 95 },
    { 0xaabf2637, 100 },
    { 0x81ef2639, 61 },
    { 0, 0 },
    { 0xb45f263b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47df2640, 23 },
    { 0x23cf2641, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35f2645, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bff264d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f2652, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cf2655, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f2659, 64 },
    { 0, 0 },
    { 0x90f265b, 90 },
    { 0x675f265c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcf265f, 76 },
    { 0x240f2660, 61 },
    { 0, 0 },
    { 0xa69f2662, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184f2669, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898f266f, 76 },
    { 0, 0 },
    { 0x653f2671, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f2676, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef267c, 68 },
    { 0x874f267d, 95 },
    { 0, 0 },
    { 0xcaff267f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02f2684, 98 },
    { 0x63ef2685, 76 },
    { 0xc8bf2685, 93 },
    { 0x59df2687, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf268d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdf2690, 53 },
    { 0xac2f2691, 33 },
    { 0x7b8f2692, 61 },
    { 0x754f2693, 33 },
    { 0xf74f2694, 52 },
    { 0, 0 },
    { 0xac2f2696, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f269b, 33 },
    { 0, 0 },
    { 0xe35f269d, 46 },
    { 0xd0bf269d, 90 },
    { 0xc78f269e, 55 },
    { 0, 0 },
    { 0xa1df26a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3f26a4, 76 },
    { 0x372f26a4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78f26aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6f26ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf26b1, 66 },
    { 0x32ef26b2, 95 },
    { 0, 0 },
    { 0xcff26b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadf26b7, 34 },
    { 0, 0 },
    { 0x924f26b9, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ff26be, 95 },
    { 0, 0 },
    { 0x3cf26c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ef26c5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df26c8, 33 },
    { 0, 0 },
    { 0x56ef26ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x959f26cd, 75 },
    { 0x554f26ce, 34 },
    { 0xdf7f26ce, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f26d5, 95 },
    { 0xfebf26d6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f26dd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f26e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9f26e7, 33 },
    { 0, 0 },
    { 0x104f26e9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa00f26f1, 23 },
    { 0x75ef26f2, 61 },
    { 0xe96f26f1, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f26f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f26fa, 95 },
    { 0x484f26fa, 76 },
    { 0x147f26fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaff2701, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184f2705, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63ff270b, 33 },
    { 0, 0 },
    { 0xb43f270d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2f2710, 34 },
    { 0x44bf2711, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbf2716, 61 },
    { 0x870f2717, 55 },
    { 0x19df2717, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cf271e, 61 },
    { 0xe6cf271f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabf2726, 67 },
    { 0, 0 },
    { 0xd03f2728, 75 },
    { 0x34df2729, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddf272c, 98 },
    { 0x552f272d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f2733, 95 },
    { 0x116f2734, 98 },
    { 0x8def2735, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bf273d, 46 },
    { 0, 0 },
    { 0x24ff273f, 85 },
    { 0xb2f273f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3f2745, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9cf274d, 61 },
    { 0, 0 },
    { 0x815f274f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf2753, 98 },
    { 0, 0 },
    { 0x4af2755, 90 },
    { 0x4adf2756, 23 },
    { 0xd5bf2756, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8f2762, 83 },
    { 0x78bf2763, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f2766, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ef2769, 96 },
    { 0x47f276a, 61 },
    { 0xdc5f276b, 90 },
    { 0x3b6f276b, 96 },
    { 0xc31f276d, 33 },
    { 0, 0 },
    { 0xaa5f276f, 33 },
    { 0xbbf2770, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289f2778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa05f277b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26f277e, 93 },
    { 0xec2f277e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd30f2783, 55 },
    { 0, 0 },
    { 0xda0f2785, 93 },
    { 0, 0 },
    { 0xd36f2787, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ef278b, 98 },
    { 0, 0 },
    { 0x693f278d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x880f2795, 33 },
    { 0xbacf2796, 61 },
    { 0x66ff2796, 76 },
    { 0x3e9f2796, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50f279b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f279e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd7f27a7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3df27ab, 98 },
    { 0xb81f27ac, 100 },
    { 0xa7bf27ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564f27ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16f27bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcff27c2, 95 },
    { 0x509f27c3, 90 },
    { 0x341f27c4, 90 },
    { 0xdbbf27c4, 93 },
    { 0x7ef27c6, 95 },
    { 0, 0 },
    { 0x700f27c8, 90 },
    { 0, 0 },
    { 0xae7f27ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af27d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89af27dc, 23 },
    { 0xfcbf27dd, 77 },
    { 0, 0 },
    { 0x61bf27df, 90 },
    { 0x7d1f27e0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932f27e4, 90 },
    { 0x154f27e4, 55 },
    { 0xdc5f27e6, 67 },
    { 0, 0 },
    { 0x7e9f27e8, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf27eb, 98 },
    { 0x951f27ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3f27ef, 85 },
    { 0xf5ef27ef, 100 },
    { 0x242f27f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f27f8, 93 },
    { 0xfbbf27f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cf2801, 90 },
    { 0, 0 },
    { 0x4a2f2803, 91 },
    { 0x739f2803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53df280a, 17 },
    { 0, 0 },
    { 0xc95f280c, 52 },
    { 0x344f280d, 61 },
    { 0xf22f280e, 61 },
    { 0x3aef280f, 23 },
    { 0x8a4f280f, 95 },
    { 0xd4af2811, 90 },
    { 0xd3df2810, 91 },
    { 0, 0 },
    { 0x6c5f2814, 23 },
    { 0x3eaf2814, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95f281d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f2820, 98 },
    { 0x1c2f2821, 40 },
    { 0x5fdf2821, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3f2826, 68 },
    { 0x579f2826, 93 },
    { 0x884f2828, 33 },
    { 0, 0 },
    { 0x78f282a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacf282e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7f2838, 90 },
    { 0xc6df2839, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9f283f, 98 },
    { 0xef4f283f, 61 },
    { 0, 0 },
    { 0xfdf2842, 66 },
    { 0, 0 },
    { 0xa31f2844, 95 },
    { 0, 0 },
    { 0x214f2846, 61 },
    { 0x6bff2847, 33 },
    { 0xd8ff2848, 61 },
    { 0x4e9f2848, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20f284c, 33 },
    { 0, 0 },
    { 0xe02f284e, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5f2851, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa3f2855, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df285f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f2863, 90 },
    { 0xceff2863, 76 },
    { 0xe8ef2865, 81 },
    { 0x618f2863, 33 },
    { 0xde2f2867, 76 },
    { 0x282f2868, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f286f, 61 },
    { 0xacbf286f, 98 },
    { 0xa44f2871, 43 },
    { 0xc1ff2872, 100 },
    { 0, 0 },
    { 0xe0bf2874, 90 },
    { 0xc7cf2875, 90 },
    { 0, 0 },
    { 0x179f2877, 63 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f287b, 76 },
    { 0, 0 },
    { 0xacef287d, 61 },
    { 0x61cf287e, 95 },
    { 0x7f9f287f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea9f2885, 96 },
    { 0xdcaf2886, 90 },
    { 0x408f2887, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebff288e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0df2892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bf2896, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaf289c, 34 },
    { 0xbdcf289c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bf28a2, 95 },
    { 0x4e1f28a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df28a7, 33 },
    { 0xe63f28a8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b5f28af, 93 },
    { 0, 0 },
    { 0x6c1f28b1, 66 },
    { 0x95f28b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9f28b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7f28b9, 61 },
    { 0xbb2f28ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9f28be, 84 },
    { 0xd6af28be, 90 },
    { 0x11ef28c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37f28c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6def28cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x828f28d3, 81 },
    { 0, 0 },
    { 0xa23f28d5, 98 },
    { 0xa12f28d6, 81 },
    { 0xe1af28d6, 58 },
    { 0x1fcf28d7, 43 },
    { 0, 0 },
    { 0x22af28da, 33 },
    { 0x7c4f28db, 76 },
    { 0x723f28dc, 43 },
    { 0, 0 },
    { 0x72f28de, 17 },
    { 0, 0 },
    { 0x36ff28e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa90f28e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3f28ea, 55 },
    { 0, 0 },
    { 0xbc8f28ec, 55 },
    { 0x3f3f28ed, 34 },
    { 0x1e3f28ed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f28f2, 93 },
    { 0x9e8f28f3, 95 },
    { 0, 0 },
    { 0x54cf28f5, 95 },
    { 0xe13f28f6, 95 },
    { 0, 0 },
    { 0xd68f28f8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f28fb, 90 },
    { 0x45bf28fc, 61 },
    { 0xccaf28fd, 52 },
    { 0x7d4f28fd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d9f2901, 95 },
    { 0x5d3f2902, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3f2906, 23 },
    { 0x27ff2907, 100 },
    { 0xe60f2908, 61 },
    { 0x82ef2906, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cef290f, 55 },
    { 0xd2df2910, 76 },
    { 0xf47f290f, 95 },
    { 0, 0 },
    { 0x15cf2913, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7f2916, 36 },
    { 0xb70f2917, 61 },
    { 0x56af2918, 90 },
    { 0xe1ff2918, 98 },
    { 0, 0 },
    { 0x991f291b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f291e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379f292c, 33 },
    { 0, 0 },
    { 0x889f292e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dcf2942, 33 },
    { 0x6e1f2943, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x386f2949, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339f2951, 33 },
    { 0xe45f2952, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22f2956, 98 },
    { 0, 0 },
    { 0x226f2958, 83 },
    { 0x4bbf2958, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd76f295c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f2960, 56 },
    { 0x1bdf2961, 90 },
    { 0xd34f2962, 88 },
    { 0x794f2962, 90 },
    { 0x616f2963, 23 },
    { 0x9b6f2963, 72 },
    { 0x96bf2966, 87 },
    { 0, 0 },
    { 0x33ff2968, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf296c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb2f2973, 46 },
    { 0x62df2974, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x931f2977, 78 },
    { 0x724f2977, 23 },
    { 0xa80f2979, 84 },
    { 0x424f2979, 100 },
    { 0x88cf2979, 23 },
    { 0, 0 },
    { 0xda1f297d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x455f2985, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff2988, 23 },
    { 0, 0 },
    { 0x3eef298a, 61 },
    { 0, 0 },
    { 0x270f298c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaf2990, 90 },
    { 0, 0 },
    { 0x1cff2992, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585f2997, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49f299a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909f29a2, 90 },
    { 0, 0 },
    { 0x5c1f29a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef29a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36f29ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff29b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cf29b7, 78 },
    { 0xbe0f29b8, 87 },
    { 0xeedf29b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aef29bc, 55 },
    { 0xa10f29bd, 17 },
    { 0x85f29be, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x17f29c1, 33 },
    { 0x4b6f29c1, 33 },
    { 0, 0 },
    { 0x439f29c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f29c9, 87 },
    { 0xb22f29ca, 23 },
    { 0x54bf29cb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dff29ce, 78 },
    { 0, 0 },
    { 0xce9f29d0, 41 },
    { 0x740f29d0, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf29db, 90 },
    { 0x31f29dc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719f29e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2f29e9, 33 },
    { 0x241f29ea, 95 },
    { 0x77f29e9, 98 },
    { 0xff29e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef6f29f3, 68 },
    { 0xd74f29f4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8f29f8, 55 },
    { 0x7cbf29f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f29fc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f2a01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cf2a04, 33 },
    { 0, 0 },
    { 0x295f2a06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x541f2a0a, 93 },
    { 0, 0 },
    { 0xcf2f2a0c, 93 },
    { 0, 0 },
    { 0xf07f2a0e, 23 },
    { 0x225f2a0f, 43 },
    { 0x7c7f2a10, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73f2a18, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e7f2a1c, 23 },
    { 0, 0 },
    { 0x885f2a1e, 95 },
    { 0, 0 },
    { 0x3baf2a20, 61 },
    { 0, 0 },
    { 0x933f2a22, 98 },
    { 0, 0 },
    { 0x50df2a24, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22df2a28, 65 },
    { 0x75ff2a29, 91 },
    { 0, 0 },
    { 0x4ecf2a2b, 43 },
    { 0xee9f2a2c, 61 },
    { 0xcedf2a2d, 90 },
    { 0x545f2a2c, 91 },
    { 0, 0 },
    { 0x88df2a30, 91 },
    { 0xc15f2a30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f2a35, 66 },
    { 0x2f5f2a36, 85 },
    { 0x872f2a35, 33 },
    { 0xd79f2a38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67f2a3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f2a43, 55 },
    { 0xab9f2a44, 98 },
    { 0x368f2a45, 33 },
    { 0xe0cf2a46, 98 },
    { 0x1f2a46, 95 },
    { 0xa16f2a48, 81 },
    { 0x3e3f2a49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc1f2a4d, 67 },
    { 0, 0 },
    { 0x6f7f2a4f, 52 },
    { 0, 0 },
    { 0xfc8f2a51, 76 },
    { 0, 0 },
    { 0xd4df2a53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f2a5a, 23 },
    { 0x5bff2a5b, 65 },
    { 0x877f2a5c, 80 },
    { 0xf3bf2a5a, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ef2a63, 90 },
    { 0, 0 },
    { 0x206f2a65, 88 },
    { 0x7abf2a65, 100 },
    { 0xe9ef2a67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24f2a6a, 68 },
    { 0xb11f2a6b, 61 },
    { 0x9e4f2a6c, 99 },
    { 0, 0 },
    { 0xb56f2a6e, 85 },
    { 0, 0 },
    { 0x5daf2a70, 23 },
    { 0xfb2f2a71, 33 },
    { 0xbcf2a71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbf2a75, 61 },
    { 0, 0 },
    { 0x890f2a77, 90 },
    { 0xa75f2a78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9f2a7b, 90 },
    { 0x3e5f2a7c, 52 },
    { 0x6ddf2a7c, 61 },
    { 0x994f2a7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6f2a83, 91 },
    { 0, 0 },
    { 0xdfff2a85, 91 },
    { 0, 0 },
    { 0x896f2a87, 38 },
    { 0x38df2a88, 61 },
    { 0xa0bf2a87, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160f2a8d, 99 },
    { 0x4b3f2a8e, 76 },
    { 0x5cef2a8f, 95 },
    { 0x267f2a90, 98 },
    { 0xb82f2a90, 87 },
    { 0xbdef2a91, 76 },
    { 0x98af2a93, 61 },
    { 0xa84f2a93, 55 },
    { 0x3c7f2a95, 33 },
    { 0x42df2a8e, 23 },
    { 0xb1cf2a8f, 99 },
    { 0x840f2a8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456f2a9d, 98 },
    { 0, 0 },
    { 0xac6f2a9f, 76 },
    { 0xf82f2aa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4f2aa5, 100 },
    { 0x653f2aa6, 61 },
    { 0x49af2aa7, 61 },
    { 0xc8ff2aa8, 90 },
    { 0x116f2aa7, 95 },
    { 0, 0 },
    { 0xcdef2aab, 77 },
    { 0x3bbf2aac, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659f2ab2, 76 },
    { 0xc4af2ab3, 33 },
    { 0, 0 },
    { 0xa2af2ab5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf2ab9, 68 },
    { 0x4f4f2ab9, 85 },
    { 0, 0 },
    { 0x383f2abc, 33 },
    { 0, 0 },
    { 0x6e5f2abe, 90 },
    { 0xcb3f2abe, 84 },
    { 0x1cf2ac0, 52 },
    { 0xfd8f2abf, 95 },
    { 0xd81f2ac0, 61 },
    { 0, 0 },
    { 0xde6f2ac4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85f2ac8, 55 },
    { 0xf4cf2ac9, 98 },
    { 0x2fff2aca, 88 },
    { 0x5c7f2acb, 61 },
    { 0x82f2acb, 90 },
    { 0xb72f2acd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcf2ad2, 23 },
    { 0xdedf2ad2, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22f2ad8, 90 },
    { 0x5af2ad9, 8 },
    { 0, 0 },
    { 0xd81f2adb, 84 },
    { 0x4aaf2adc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f2ae7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62af2aec, 98 },
    { 0, 0 },
    { 0x157f2aee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bf2af6, 66 },
    { 0, 0 },
    { 0x838f2af8, 100 },
    { 0, 0 },
    { 0x9dff2afa, 98 },
    { 0x68af2afb, 40 },
    { 0, 0 },
    { 0x5e8f2afd, 61 },
    { 0, 0 },
    { 0x35ff2aff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9f2b04, 61 },
    { 0, 0 },
    { 0x3b3f2b06, 61 },
    { 0xfc7f2b07, 95 },
    { 0, 0 },
    { 0x297f2b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ef2b0c, 43 },
    { 0x9bef2b0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cf2b13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df2b16, 99 },
    { 0x6f2b17, 23 },
    { 0, 0 },
    { 0xf6df2b19, 55 },
    { 0, 0 },
    { 0x1bff2b1b, 55 },
    { 0xaddf2b1c, 33 },
    { 0, 0 },
    { 0x609f2b1e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccf2b26, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f2b2e, 66 },
    { 0, 0 },
    { 0x61bf2b30, 95 },
    { 0xa6f2b31, 90 },
    { 0x115f2b32, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8f2b37, 34 },
    { 0x4c2f2b37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2f2b3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccf2b3e, 23 },
    { 0x2bff2b3f, 90 },
    { 0x8dbf2b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ff2b43, 95 },
    { 0xd35f2b44, 17 },
    { 0, 0 },
    { 0x6d2f2b46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbf2b4e, 100 },
    { 0xbedf2b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f2b53, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f2b56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f2b59, 76 },
    { 0, 0 },
    { 0x13ef2b5b, 90 },
    { 0, 0 },
    { 0xe73f2b5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17f2b64, 93 },
    { 0, 0 },
    { 0x913f2b66, 90 },
    { 0, 0 },
    { 0x517f2b68, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f2b77, 23 },
    { 0, 0 },
    { 0xb69f2b79, 61 },
    { 0, 0 },
    { 0x598f2b7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4df2b7e, 95 },
    { 0, 0 },
    { 0xe82f2b80, 23 },
    { 0x852f2b81, 90 },
    { 0x582f2b81, 43 },
    { 0x6bcf2b83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964f2b8b, 78 },
    { 0x2cdf2b8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x82df2b8f, 98 },
    { 0x64ff2b8f, 76 },
    { 0, 0 },
    { 0x6c5f2b92, 23 },
    { 0, 0 },
    { 0xd2ff2b94, 17 },
    { 0x898f2b95, 33 },
    { 0x1f8f2b96, 55 },
    { 0x297f2b97, 98 },
    { 0x9e7f2b98, 23 },
    { 0xaa8f2b94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbf2b9d, 61 },
    { 0, 0 },
    { 0x5c8f2b9f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5f2ba4, 77 },
    { 0x171f2ba5, 95 },
    { 0, 0 },
    { 0x25f2ba7, 61 },
    { 0xd90f2ba8, 33 },
    { 0xb58f2ba9, 76 },
    { 0xc3ff2ba8, 76 },
    { 0x8eff2bab, 33 },
    { 0xc02f2bac, 61 },
    { 0x8faf2bad, 33 },
    { 0, 0 },
    { 0xbd0f2baf, 90 },
    { 0x581f2bb0, 66 },
    { 0xd15f2bb1, 98 },
    { 0xb2ef2bb2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x595f2bb5, 33 },
    { 0x87ff2bb6, 61 },
    { 0, 0 },
    { 0xa3ef2bb8, 76 },
    { 0x9d0f2bb9, 33 },
    { 0x23cf2bba, 75 },
    { 0xd55f2bbb, 92 },
    { 0xb01f2bbb, 76 },
    { 0xbf2f2bbd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc75f2bc0, 76 },
    { 0xd25f2bc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df2bca, 61 },
    { 0xc39f2bcb, 34 },
    { 0xc8ff2bcb, 95 },
    { 0, 0 },
    { 0x4cef2bce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58df2bd5, 90 },
    { 0x69bf2bd6, 84 },
    { 0x16ff2bd6, 90 },
    { 0x5e6f2bd7, 85 },
    { 0x2fdf2bd9, 23 },
    { 0x4c6f2bda, 95 },
    { 0x749f2bda, 43 },
    { 0x7c9f2bdc, 34 },
    { 0x2c9f2bdc, 90 },
    { 0x7eef2bde, 61 },
    { 0xd7cf2bda, 23 },
    { 0xbeff2be0, 91 },
    { 0x2f7f2bd9, 91 },
    { 0x2abf2bdd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x359f2be5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71af2be9, 46 },
    { 0x4b5f2bea, 93 },
    { 0x656f2beb, 95 },
    { 0xf72f2bec, 76 },
    { 0xec7f2bec, 34 },
    { 0x6fdf2bee, 81 },
    { 0xdf5f2bef, 80 },
    { 0, 0 },
    { 0x565f2bf1, 33 },
    { 0x3ef2bf2, 95 },
    { 0xed8f2bf3, 55 },
    { 0x49df2bf4, 90 },
    { 0x870f2bf2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5f2bfb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcacf2c03, 100 },
    { 0, 0 },
    { 0xd0bf2c05, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4f2c0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f2c10, 90 },
    { 0xdcf2c10, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf2c14, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8f2c17, 98 },
    { 0, 0 },
    { 0x6d2f2c19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x789f2c1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2af2c21, 33 },
    { 0, 0 },
    { 0x7aef2c23, 61 },
    { 0, 0 },
    { 0x4e1f2c25, 61 },
    { 0, 0 },
    { 0x59f2c27, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ef2c2a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f2c30, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9f2c36, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bf2c3a, 55 },
    { 0x3aef2c3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x362f2c41, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766f2c4a, 90 },
    { 0x874f2c4a, 55 },
    { 0, 0 },
    { 0xcfef2c4d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5f2c54, 52 },
    { 0x2bef2c55, 69 },
    { 0x79ef2c55, 67 },
    { 0xd5af2c56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cf2c5a, 98 },
    { 0, 0 },
    { 0x597f2c5c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9f2c60, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9f2c63, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846f2c68, 17 },
    { 0xc9df2c68, 90 },
    { 0xcc9f2c69, 55 },
    { 0xc4af2c6b, 33 },
    { 0, 0 },
    { 0xd9cf2c6d, 90 },
    { 0x32f2c6d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebf2c72, 61 },
    { 0, 0 },
    { 0x574f2c74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84f2c77, 64 },
    { 0x27f2c77, 90 },
    { 0, 0 },
    { 0xce6f2c7a, 95 },
    { 0xf0cf2c7b, 23 },
    { 0x11bf2c7b, 99 },
    { 0xbc8f2c7c, 55 },
    { 0, 0 },
    { 0xdf2c7f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1f2c84, 34 },
    { 0x523f2c85, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f2c88, 87 },
    { 0x5e9f2c89, 33 },
    { 0x819f2c89, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x326f2c8d, 33 },
    { 0, 0 },
    { 0x597f2c8f, 76 },
    { 0, 0 },
    { 0x2aaf2c91, 33 },
    { 0xfb7f2c92, 33 },
    { 0x1ef2c92, 95 },
    { 0x20cf2c94, 95 },
    { 0x325f2c94, 95 },
    { 0xfa1f2c94, 43 },
    { 0x32bf2c97, 88 },
    { 0, 0 },
    { 0xaaaf2c99, 84 },
    { 0, 0 },
    { 0xbfbf2c9b, 61 },
    { 0xa48f2c9c, 95 },
    { 0x14bf2c9d, 94 },
    { 0x32cf2c9d, 33 },
    { 0xedff2c9c, 85 },
    { 0xf92f2ca0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x482f2ca6, 90 },
    { 0xe48f2ca6, 61 },
    { 0x279f2ca8, 90 },
    { 0xc0f2ca8, 90 },
    { 0, 0 },
    { 0xc36f2cab, 33 },
    { 0x79ef2cab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2cb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb17f2cb7, 90 },
    { 0x73df2cb8, 51 },
    { 0xf08f2cb8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bf2cbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x983f2cc3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad3f2cc7, 95 },
    { 0x9bf2cc7, 83 },
    { 0x604f2cc9, 33 },
    { 0, 0 },
    { 0x16cf2ccb, 100 },
    { 0, 0 },
    { 0x59f2ccd, 98 },
    { 0xd41f2cce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0f2cd1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf2cd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x859f2ce0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f2ce3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f2ce8, 33 },
    { 0xba7f2ce9, 76 },
    { 0xc9ef2cea, 33 },
    { 0, 0 },
    { 0x1c8f2cec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0f2cf2, 33 },
    { 0x9af2cf3, 23 },
    { 0xb50f2cf2, 95 },
    { 0x19bf2cf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bf2cfd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272f2d02, 90 },
    { 0, 0 },
    { 0xac9f2d04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2f2d0a, 98 },
    { 0, 0 },
    { 0xc56f2d0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8f2d0f, 84 },
    { 0x323f2d0f, 76 },
    { 0, 0 },
    { 0xba2f2d12, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f2d17, 98 },
    { 0, 0 },
    { 0x939f2d19, 68 },
    { 0x1f8f2d1a, 98 },
    { 0x7fff2d1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf2d20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x288f2d23, 75 },
    { 0, 0 },
    { 0xa73f2d25, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f2d29, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x219f2d2c, 90 },
    { 0x9aff2d2d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cff2d30, 98 },
    { 0x38df2d30, 33 },
    { 0xb90f2d32, 75 },
    { 0x16ef2d33, 95 },
    { 0xf64f2d34, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f2d39, 85 },
    { 0x135f2d3a, 23 },
    { 0x4a5f2d3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3f2d3e, 98 },
    { 0, 0 },
    { 0x5a7f2d40, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x692f2d43, 95 },
    { 0, 0 },
    { 0x825f2d45, 91 },
    { 0x617f2d46, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1f2d4e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x228f2d51, 61 },
    { 0xc0cf2d52, 95 },
    { 0x436f2d51, 99 },
    { 0x426f2d52, 95 },
    { 0, 0 },
    { 0x4b0f2d56, 95 },
    { 0, 0 },
    { 0x880f2d58, 75 },
    { 0xc50f2d59, 98 },
    { 0x650f2d5a, 33 },
    { 0xa9f2d5b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f2d62, 46 },
    { 0xdcf2d63, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x785f2d68, 98 },
    { 0x85af2d69, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bf2d71, 34 },
    { 0, 0 },
    { 0xdedf2d73, 43 },
    { 0xdbcf2d74, 55 },
    { 0xbaef2d73, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f2d79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdf2d7d, 53 },
    { 0, 0 },
    { 0xc44f2d7f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f0f2d82, 99 },
    { 0, 0 },
    { 0x1ecf2d84, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f2d88, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f2d90, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389f2d94, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f2da1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x540f2da4, 55 },
    { 0x715f2da5, 76 },
    { 0x591f2da4, 33 },
    { 0xd38f2da7, 33 },
    { 0, 0 },
    { 0x6faf2da9, 93 },
    { 0xca9f2daa, 65 },
    { 0, 0 },
    { 0x84df2dac, 95 },
    { 0x90af2dad, 81 },
    { 0, 0 },
    { 0x261f2daf, 46 },
    { 0x157f2db0, 90 },
    { 0x5d0f2daf, 95 },
    { 0xdc2f2daf, 61 },
    { 0x44bf2db3, 84 },
    { 0x258f2db3, 95 },
    { 0, 0 },
    { 0xca7f2db6, 46 },
    { 0, 0 },
    { 0x1ff2db8, 46 },
    { 0x15ff2db8, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf2dbc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1f2dc0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09f2dc3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94f2dc6, 93 },
    { 0xe9ef2dc7, 61 },
    { 0xc90f2dc7, 90 },
    { 0, 0 },
    { 0xf88f2dca, 46 },
    { 0xfe8f2dcb, 90 },
    { 0x32ff2dcc, 78 },
    { 0xcaff2dcb, 95 },
    { 0x38ef2dca, 95 },
    { 0, 0 },
    { 0x6eaf2dd0, 34 },
    { 0xae1f2dd1, 23 },
    { 0, 0 },
    { 0x28ff2dd3, 93 },
    { 0x833f2dd3, 23 },
    { 0x41bf2dd4, 23 },
    { 0xca3f2dd6, 75 },
    { 0, 0 },
    { 0x9bbf2dd8, 90 },
    { 0x565f2dd8, 61 },
    { 0, 0 },
    { 0xc66f2ddb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ff2dde, 78 },
    { 0x3eaf2dde, 95 },
    { 0xb2ff2de0, 61 },
    { 0x612f2de1, 77 },
    { 0x81bf2de2, 78 },
    { 0xbf6f2de1, 61 },
    { 0x20ff2de4, 90 },
    { 0x7cf2de2, 90 },
    { 0x548f2de3, 61 },
    { 0x314f2de7, 90 },
    { 0x882f2de6, 98 },
    { 0x7baf2de2, 90 },
    { 0xb44f2de4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bf2dee, 95 },
    { 0xb90f2dee, 90 },
    { 0xa4af2def, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af2df5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1f2dfb, 23 },
    { 0x1b9f2dfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5f2dff, 61 },
    { 0x9fef2e00, 23 },
    { 0xd6bf2e00, 33 },
    { 0xa51f2e00, 33 },
    { 0x2dff2e01, 98 },
    { 0x2adf2dff, 93 },
    { 0x924f2e02, 90 },
    { 0x53f2e05, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x634f2e12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ef2e1c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x30cf2e1f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4f2e23, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8f2e28, 38 },
    { 0x910f2e29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3f2e2c, 78 },
    { 0, 0 },
    { 0x481f2e2e, 77 },
    { 0x85ff2e2f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bf2e32, 90 },
    { 0x9f7f2e33, 91 },
    { 0x97cf2e33, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86f2e37, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6ff2e3e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adf2e44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f2e48, 95 },
    { 0, 0 },
    { 0xe6f2e4a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ff2e4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ef2e52, 76 },
    { 0, 0 },
    { 0x741f2e54, 91 },
    { 0, 0 },
    { 0xdc7f2e56, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cbf2e5b, 90 },
    { 0x8bcf2e5b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaf2e5f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f2e63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x966f2e66, 90 },
    { 0xc0f2e67, 61 },
    { 0xd87f2e66, 34 },
    { 0x452f2e69, 77 },
    { 0x5c5f2e6a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14f2e6e, 88 },
    { 0xc3ff2e6e, 33 },
    { 0x687f2e6f, 76 },
    { 0x3ebf2e71, 98 },
    { 0xfdf2e70, 90 },
    { 0x8df2e71, 90 },
    { 0x80bf2e70, 75 },
    { 0x860f2e74, 90 },
    { 0x191f2e76, 23 },
    { 0x557f2e76, 90 },
    { 0xec7f2e77, 95 },
    { 0, 0 },
    { 0xf1cf2e7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9f2e7d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cf2e81, 33 },
    { 0xde6f2e82, 75 },
    { 0xd2cf2e83, 33 },
    { 0xe77f2e82, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x86f2e87, 61 },
    { 0, 0 },
    { 0x5bef2e89, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9f2e8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cf2e8f, 100 },
    { 0x4c6f2e8f, 93 },
    { 0xfd7f2e90, 61 },
    { 0x3d5f2e92, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37f2e95, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1f2e9b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63f2e9e, 33 },
    { 0x565f2e9f, 95 },
    { 0, 0 },
    { 0xa21f2ea1, 17 },
    { 0, 0 },
    { 0x4c4f2ea3, 56 },
    { 0x62f2ea4, 100 },
    { 0x955f2ea4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1f2ea8, 55 },
    { 0x845f2ea9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bf2ead, 23 },
    { 0xe18f2eae, 98 },
    { 0xf2cf2eaf, 61 },
    { 0x825f2eb0, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cf2eb5, 76 },
    { 0x4f6f2eb6, 90 },
    { 0x7ddf2eb7, 33 },
    { 0x4f9f2eb7, 66 },
    { 0x8a7f2eb5, 90 },
    { 0xe4f2eba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff2ec3, 95 },
    { 0xefdf2ec3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31ff2ecc, 95 },
    { 0, 0 },
    { 0x384f2ece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcf2ed3, 24 },
    { 0, 0 },
    { 0xd7cf2ed5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4f2ed9, 95 },
    { 0x132f2ed9, 90 },
    { 0, 0 },
    { 0x56af2edc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef2ee1, 33 },
    { 0x7f4f2ee1, 34 },
    { 0, 0 },
    { 0xf49f2ee4, 90 },
    { 0x7b4f2ee5, 98 },
    { 0x20ef2ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f2ef2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3f2ef5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aff2ef9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadef2efd, 90 },
    { 0xd08f2efe, 61 },
    { 0xcb3f2eff, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafef2f05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276f2f0a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f2f0e, 33 },
    { 0, 0 },
    { 0xf4af2f10, 69 },
    { 0, 0 },
    { 0x98ef2f12, 99 },
    { 0x980f2f12, 17 },
    { 0x596f2f14, 19 },
    { 0x6d4f2f15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x715f2f18, 85 },
    { 0xc24f2f18, 61 },
    { 0, 0 },
    { 0x92cf2f1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2f2f20, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f2f23, 53 },
    { 0, 0 },
    { 0xd47f2f25, 55 },
    { 0, 0 },
    { 0x472f2f27, 43 },
    { 0xd21f2f28, 43 },
    { 0x44ff2f28, 93 },
    { 0x498f2f29, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cf2f2e, 93 },
    { 0x374f2f2f, 90 },
    { 0, 0 },
    { 0x3edf2f31, 95 },
    { 0, 0 },
    { 0xa1cf2f33, 84 },
    { 0xdd2f2f34, 61 },
    { 0, 0 },
    { 0xfe2f2f36, 68 },
    { 0xe22f2f37, 90 },
    { 0x371f2f37, 66 },
    { 0x90ef2f39, 61 },
    { 0, 0 },
    { 0xdc0f2f3b, 34 },
    { 0x151f2f3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ef2f3f, 61 },
    { 0x802f2f40, 91 },
    { 0x4bdf2f41, 33 },
    { 0xc4df2f42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bf2f45, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0f2f49, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887f2f51, 90 },
    { 0x6c8f2f52, 33 },
    { 0, 0 },
    { 0xb62f2f54, 98 },
    { 0, 0 },
    { 0x90bf2f56, 23 },
    { 0, 0 },
    { 0x153f2f58, 88 },
    { 0x1ddf2f59, 61 },
    { 0x9f1f2f59, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21f2f5d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cf2f60, 37 },
    { 0x77bf2f61, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302f2f66, 98 },
    { 0xe52f2f67, 76 },
    { 0xafdf2f68, 90 },
    { 0xb0af2f68, 91 },
    { 0x46af2f6a, 95 },
    { 0x1bff2f6a, 98 },
    { 0x928f2f6b, 23 },
    { 0, 0 },
    { 0x468f2f6e, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21f2f72, 61 },
    { 0x60ef2f73, 46 },
    { 0xdcef2f74, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0f2f7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf2f7d, 90 },
    { 0x712f2f7e, 89 },
    { 0, 0 },
    { 0xe2ef2f80, 90 },
    { 0xafef2f81, 46 },
    { 0x53af2f81, 55 },
    { 0xea4f2f83, 61 },
    { 0xad9f2f83, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9f2f8b, 68 },
    { 0x929f2f8b, 76 },
    { 0x3eff2f8b, 85 },
    { 0xedff2f8e, 85 },
    { 0x46af2f8f, 95 },
    { 0x3d6f2f8f, 61 },
    { 0x9ddf2f91, 66 },
    { 0, 0 },
    { 0xe1af2f93, 33 },
    { 0x8b1f2f94, 95 },
    { 0xaa1f2f94, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f2f9d, 98 },
    { 0, 0 },
    { 0x240f2f9f, 84 },
    { 0xd7ff2f9f, 95 },
    { 0x665f2fa1, 90 },
    { 0xd8cf2fa2, 36 },
    { 0, 0 },
    { 0x891f2fa4, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7f2fa7, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3f2faa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7cf2faf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaf2fb2, 91 },
    { 0, 0 },
    { 0xefef2fb4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579f2fb8, 85 },
    { 0, 0 },
    { 0x23bf2fba, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0f2fbe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x52bf2fc1, 33 },
    { 0, 0 },
    { 0xf98f2fc3, 52 },
    { 0xf8af2fc3, 66 },
    { 0x35df2fc4, 61 },
    { 0, 0 },
    { 0xf7af2fc7, 33 },
    { 0, 0 },
    { 0xc4cf2fc9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdf2fcc, 87 },
    { 0x79ff2fcd, 95 },
    { 0x6dff2fce, 61 },
    { 0x386f2fcf, 76 },
    { 0x68ff2fcf, 61 },
    { 0x80cf2fd1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x214f2fd8, 90 },
    { 0, 0 },
    { 0x7a0f2fda, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65af2fe3, 95 },
    { 0x8b2f2fe4, 33 },
    { 0, 0 },
    { 0xe5cf2fe6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28f2fe9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1f2fec, 34 },
    { 0x217f2fed, 76 },
    { 0x4eff2fed, 95 },
    { 0x8bff2fef, 85 },
    { 0x7baf2fed, 19 },
    { 0x864f2fef, 34 },
    { 0x7aaf2fef, 38 },
    { 0, 0 },
    { 0x3a1f2ff4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdebf2ff7, 23 },
    { 0, 0 },
    { 0x5a7f2ff9, 99 },
    { 0xb72f2ff9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7f2fff, 78 },
    { 0x8a5f2fff, 34 },
    { 0, 0 },
    { 0x883f3002, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aff3007, 33 },
    { 0xea7f3008, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ef300e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ef3016, 95 },
    { 0x15cf3017, 93 },
    { 0x852f3016, 33 },
    { 0xe36f3019, 98 },
    { 0x6cf3018, 98 },
    { 0xb89f301b, 23 },
    { 0x3a6f301c, 80 },
    { 0xd3ff301d, 93 },
    { 0x8cf301d, 98 },
    { 0, 0 },
    { 0x31ff3020, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8f3029, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf60f302f, 90 },
    { 0xc2f3030, 90 },
    { 0, 0 },
    { 0x45ef3032, 88 },
    { 0x412f3033, 93 },
    { 0, 0 },
    { 0xed0f3035, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x34df3038, 95 },
    { 0, 0 },
    { 0x9c3f303a, 35 },
    { 0x49af303b, 76 },
    { 0x9e4f303b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2f303f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe39f3042, 84 },
    { 0xa3df3042, 90 },
    { 0, 0 },
    { 0x7e4f3045, 90 },
    { 0xb3f3046, 66 },
    { 0xbd7f3045, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f304f, 95 },
    { 0, 0 },
    { 0x69af3051, 95 },
    { 0xd58f3052, 88 },
    { 0x4ebf3053, 66 },
    { 0, 0 },
    { 0x762f3055, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9f3059, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbf3061, 87 },
    { 0x4c1f3062, 88 },
    { 0x6a8f3061, 34 },
    { 0, 0 },
    { 0xd04f3065, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cf3068, 66 },
    { 0, 0 },
    { 0x73f306a, 90 },
    { 0, 0 },
    { 0x63af306c, 95 },
    { 0, 0 },
    { 0x16f306e, 98 },
    { 0x3fdf306f, 95 },
    { 0x23bf3070, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373f3076, 96 },
    { 0, 0 },
    { 0x226f3078, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61f307b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56f307e, 34 },
    { 0xf4ef307f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13af3083, 61 },
    { 0, 0 },
    { 0x352f3085, 33 },
    { 0, 0 },
    { 0x84df3087, 98 },
    { 0x9eef3088, 61 },
    { 0xff6f3089, 34 },
    { 0x27df308a, 90 },
    { 0x16f308b, 70 },
    { 0x355f308b, 98 },
    { 0x88ef308c, 91 },
    { 0xdeff308e, 77 },
    { 0xda2f308a, 61 },
    { 0xe55f308c, 34 },
    { 0x222f308c, 23 },
    { 0x76bf3092, 33 },
    { 0, 0 },
    { 0xdf3094, 95 },
    { 0x80df3095, 95 },
    { 0, 0 },
    { 0x47cf3097, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x833f309a, 61 },
    { 0, 0 },
    { 0x6d9f309c, 23 },
    { 0x112f309d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f30a1, 23 },
    { 0x94af30a2, 90 },
    { 0x19f30a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88f30a8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bf30ae, 68 },
    { 0xe15f30af, 33 },
    { 0xb8cf30af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbf30b5, 80 },
    { 0x1acf30b6, 68 },
    { 0xd56f30b6, 40 },
    { 0, 0 },
    { 0x274f30b9, 93 },
    { 0xb65f30ba, 66 },
    { 0x9cf30ba, 90 },
    { 0x605f30bc, 33 },
    { 0, 0 },
    { 0xb4f30be, 98 },
    { 0, 0 },
    { 0x8fdf30c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ff30c3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ef30c6, 23 },
    { 0xb49f30c6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783f30cc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cf30cf, 98 },
    { 0, 0 },
    { 0x715f30d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x432f30d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x735f30db, 68 },
    { 0x8daf30dc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92f30e0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd48f30e5, 81 },
    { 0x813f30e6, 34 },
    { 0x340f30e7, 61 },
    { 0x5acf30e7, 98 },
    { 0x649f30e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f30ec, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f30f2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f30f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccf30fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f30fe, 23 },
    { 0xb44f30fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bf3102, 33 },
    { 0x912f3102, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9f3106, 93 },
    { 0x518f3107, 94 },
    { 0x40ff3108, 90 },
    { 0x3b3f3109, 43 },
    { 0x8eaf310a, 33 },
    { 0, 0 },
    { 0xfc1f310c, 33 },
    { 0xbb7f310d, 90 },
    { 0x6a4f310e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ff3113, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4df3116, 90 },
    { 0, 0 },
    { 0xb43f3118, 77 },
    { 0x2ff3119, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ff311d, 90 },
    { 0x60cf311e, 17 },
    { 0, 0 },
    { 0xc09f3120, 55 },
    { 0, 0 },
    { 0x900f3122, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbf3127, 23 },
    { 0x471f3128, 90 },
    { 0xf66f3129, 61 },
    { 0, 0 },
    { 0x93ef312b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f312f, 55 },
    { 0xf35f312f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bf3136, 90 },
    { 0, 0 },
    { 0x4eef3138, 90 },
    { 0xb47f3139, 23 },
    { 0xec0f3139, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bf313d, 95 },
    { 0x14cf313e, 90 },
    { 0x8a7f313e, 92 },
    { 0xcb5f3140, 61 },
    { 0, 0 },
    { 0x1baf3142, 23 },
    { 0x7ebf3143, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791f314d, 87 },
    { 0x5c3f314d, 76 },
    { 0xaf314f, 17 },
    { 0xcdff3150, 81 },
    { 0, 0 },
    { 0xbb9f3152, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x854f3156, 90 },
    { 0, 0 },
    { 0xabdf3158, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af315d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61df3160, 98 },
    { 0xcbf3160, 75 },
    { 0x884f3162, 90 },
    { 0x2d4f3161, 33 },
    { 0xc88f3164, 55 },
    { 0x927f3162, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bff3168, 65 },
    { 0x2d6f3169, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bf316c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5f3170, 91 },
    { 0, 0 },
    { 0xafff3172, 90 },
    { 0, 0 },
    { 0x374f3174, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f3177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcef317a, 55 },
    { 0xe06f317b, 76 },
    { 0xbb5f317b, 88 },
    { 0xec7f317c, 46 },
    { 0x326f317e, 23 },
    { 0, 0 },
    { 0xe6af3180, 33 },
    { 0x3f1f3181, 93 },
    { 0, 0 },
    { 0x56ff3183, 55 },
    { 0xddcf3183, 33 },
    { 0x46f3184, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cf3189, 75 },
    { 0, 0 },
    { 0x37df318b, 23 },
    { 0, 0 },
    { 0xfdff318d, 99 },
    { 0x19ff318e, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02f3192, 33 },
    { 0, 0 },
    { 0x311f3194, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926f3198, 64 },
    { 0xf8ef3198, 53 },
    { 0x6af3198, 61 },
    { 0, 0 },
    { 0xcb7f319c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4f319f, 90 },
    { 0xbe5f31a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caf31ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ff31b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f31bc, 61 },
    { 0x8faf31bd, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4f31c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ff31c9, 90 },
    { 0xda2f31ca, 76 },
    { 0x840f31cb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f31d0, 55 },
    { 0, 0 },
    { 0x5dcf31d2, 23 },
    { 0xb21f31d3, 43 },
    { 0x759f31d4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ef31d7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x474f31da, 90 },
    { 0x77ff31db, 88 },
    { 0xef9f31db, 91 },
    { 0, 0 },
    { 0xb39f31de, 33 },
    { 0xaddf31df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65f31e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf65f31e8, 48 },
    { 0, 0 },
    { 0x435f31ea, 33 },
    { 0x391f31ea, 67 },
    { 0x998f31ec, 61 },
    { 0x312f31ec, 65 },
    { 0xca6f31ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1df31f1, 52 },
    { 0x23df31f2, 93 },
    { 0x818f31f3, 98 },
    { 0x682f31f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df31f8, 33 },
    { 0xb4ff31f9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9f3200, 91 },
    { 0x275f3200, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f3205, 90 },
    { 0xbabf3205, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaf320b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bf320e, 95 },
    { 0x603f320f, 52 },
    { 0x4b4f3210, 33 },
    { 0, 0 },
    { 0x2cf3212, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3215, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf3218, 34 },
    { 0, 0 },
    { 0x519f321a, 23 },
    { 0x5f0f321a, 33 },
    { 0xe87f321c, 61 },
    { 0x680f321d, 68 },
    { 0, 0 },
    { 0x308f321f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ef322c, 76 },
    { 0x1a8f322d, 90 },
    { 0, 0 },
    { 0xd65f322f, 46 },
    { 0x965f322f, 23 },
    { 0xf8df3231, 90 },
    { 0xd9ef3230, 86 },
    { 0x6b0f3233, 33 },
    { 0, 0 },
    { 0x9d4f3235, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887f323a, 76 },
    { 0x644f323b, 90 },
    { 0, 0 },
    { 0xb15f323d, 91 },
    { 0x53f323e, 90 },
    { 0x956f323f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f3243, 66 },
    { 0xe9f3244, 61 },
    { 0, 0 },
    { 0xd4af3246, 33 },
    { 0, 0 },
    { 0x6e0f3248, 33 },
    { 0x7e4f3249, 33 },
    { 0x166f324a, 46 },
    { 0xdb0f3248, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abf324e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981f3254, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x766f3257, 90 },
    { 0xbb4f3258, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4f325c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x651f3260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cf3265, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8f326b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef3270, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19df3274, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bf3277, 98 },
    { 0x70ff3278, 76 },
    { 0x828f3279, 84 },
    { 0, 0 },
    { 0x951f327b, 66 },
    { 0x2cdf327b, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f327f, 98 },
    { 0x7d7f3280, 33 },
    { 0x4f7f3281, 61 },
    { 0, 0 },
    { 0x9e0f3283, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ef3286, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f3289, 87 },
    { 0, 0 },
    { 0xdc3f328b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f3290, 91 },
    { 0xd95f3291, 84 },
    { 0, 0 },
    { 0x999f3293, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60f3297, 84 },
    { 0, 0 },
    { 0xb96f3299, 90 },
    { 0x225f329a, 98 },
    { 0x6d7f329b, 33 },
    { 0x1ef329b, 61 },
    { 0x1c0f329d, 99 },
    { 0xd06f329d, 23 },
    { 0x127f329b, 90 },
    { 0xb1ef329b, 61 },
    { 0x100f32a1, 87 },
    { 0xaa5f32a2, 98 },
    { 0x68f32a1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934f32a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90df32af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f32b2, 55 },
    { 0, 0 },
    { 0x6a2f32b4, 88 },
    { 0x30df32b4, 93 },
    { 0, 0 },
    { 0xbd2f32b7, 85 },
    { 0xf88f32b7, 76 },
    { 0xba9f32b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cf32bc, 101 },
    { 0x6f6f32bd, 33 },
    { 0x94ef32bd, 67 },
    { 0, 0 },
    { 0xb62f32c0, 90 },
    { 0x24cf32c1, 90 },
    { 0, 0 },
    { 0xfbdf32c3, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cf32c7, 55 },
    { 0xf17f32c7, 87 },
    { 0xa55f32c9, 98 },
    { 0x3f2f32ca, 90 },
    { 0x72ef32ca, 95 },
    { 0x129f32cc, 33 },
    { 0xffff32cd, 23 },
    { 0x4af32ce, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f32d2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdf32d5, 90 },
    { 0x1a5f32d6, 84 },
    { 0xbe2f32d7, 46 },
    { 0x89ff32d8, 65 },
    { 0x109f32d8, 90 },
    { 0x83df32d5, 55 },
    { 0x7aff32da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf32de, 61 },
    { 0x4af32df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcff32e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf32e6, 90 },
    { 0, 0 },
    { 0xdc6f32e8, 61 },
    { 0xceaf32e8, 46 },
    { 0x7faf32e8, 8 },
    { 0x961f32ea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524f32ef, 53 },
    { 0x53ff32f0, 98 },
    { 0x88f32f1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68df32f8, 23 },
    { 0x22bf32f8, 87 },
    { 0xa6f32f8, 55 },
    { 0, 0 },
    { 0xff5f32fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f3302, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x183f3305, 61 },
    { 0x501f3306, 93 },
    { 0x62bf3307, 87 },
    { 0xb47f3307, 55 },
    { 0, 0 },
    { 0x3f4f330a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60af3312, 34 },
    { 0x4e6f3313, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ff3316, 33 },
    { 0, 0 },
    { 0x2c9f3318, 98 },
    { 0, 0 },
    { 0x5e5f331a, 23 },
    { 0x581f331a, 23 },
    { 0, 0 },
    { 0xe7df331d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bf3324, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe27f3328, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73df332c, 46 },
    { 0x3aaf332d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb41f3332, 61 },
    { 0x539f3333, 61 },
    { 0x8caf3334, 90 },
    { 0, 0 },
    { 0xf61f3336, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36df333a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f333e, 76 },
    { 0, 0 },
    { 0x69bf3340, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8f3343, 33 },
    { 0x65cf3344, 34 },
    { 0xd0f3345, 34 },
    { 0xcf6f3345, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87af334a, 55 },
    { 0, 0 },
    { 0x298f334c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eff334f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x918f335b, 77 },
    { 0x70df335c, 95 },
    { 0, 0 },
    { 0x2d3f335e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb81f3361, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ef3365, 88 },
    { 0x9f3366, 76 },
    { 0, 0 },
    { 0xd6df3368, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115f336f, 76 },
    { 0, 0 },
    { 0x869f3371, 61 },
    { 0x606f3372, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x204f3375, 23 },
    { 0xd7cf3376, 91 },
    { 0xdd0f3377, 76 },
    { 0, 0 },
    { 0xf44f3379, 95 },
    { 0x717f3379, 61 },
    { 0, 0 },
    { 0x888f337c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9f337f, 95 },
    { 0xf56f3380, 95 },
    { 0xe7df3380, 33 },
    { 0, 0 },
    { 0xa22f3383, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x178f338c, 61 },
    { 0xe26f338d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f3390, 81 },
    { 0x721f3391, 93 },
    { 0xc2df3392, 85 },
    { 0x6e6f3392, 99 },
    { 0x139f3390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f3397, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87f339f, 43 },
    { 0xba1f33a0, 38 },
    { 0, 0 },
    { 0x30df33a2, 85 },
    { 0, 0 },
    { 0x2c7f33a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52f33a7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4f33ac, 34 },
    { 0xaedf33ad, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f33b0, 99 },
    { 0xdc4f33b0, 34 },
    { 0, 0 },
    { 0x99cf33b3, 39 },
    { 0x878f33b4, 80 },
    { 0x13af33b5, 87 },
    { 0xdfff33b6, 95 },
    { 0, 0 },
    { 0x758f33b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf33bb, 78 },
    { 0x46bf33bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xca6f33bf, 85 },
    { 0xd38f33bf, 95 },
    { 0, 0 },
    { 0xb0af33c2, 55 },
    { 0, 0 },
    { 0xb7f33c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07f33c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3f33cb, 36 },
    { 0, 0 },
    { 0x13cf33cd, 65 },
    { 0x7cf33ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af33d1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7f33d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99f33da, 97 },
    { 0, 0 },
    { 0x8aff33dc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f33e0, 34 },
    { 0, 0 },
    { 0x9a9f33e2, 61 },
    { 0xf30f33e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3f33e8, 81 },
    { 0x98f33e8, 66 },
    { 0x320f33e8, 46 },
    { 0x6d7f33eb, 33 },
    { 0x278f33e8, 98 },
    { 0x890f33ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52f33fd, 33 },
    { 0x23df33fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1df3403, 55 },
    { 0x8af3404, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cf3408, 76 },
    { 0xee7f3408, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4f340d, 81 },
    { 0x423f340d, 99 },
    { 0x67ff340f, 81 },
    { 0x6eef340f, 90 },
    { 0, 0 },
    { 0x839f3412, 46 },
    { 0x89ef3413, 98 },
    { 0xfd8f3414, 90 },
    { 0x7d3f3413, 55 },
    { 0x905f3416, 90 },
    { 0xe18f3416, 61 },
    { 0x6adf3418, 83 },
    { 0xf12f3419, 78 },
    { 0x3f6f3418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f341e, 55 },
    { 0xdfef341f, 46 },
    { 0x918f3420, 76 },
    { 0x92ff3421, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa0f3428, 98 },
    { 0xb7ef3429, 98 },
    { 0x737f3428, 65 },
    { 0xab7f342b, 61 },
    { 0, 0 },
    { 0x898f342d, 90 },
    { 0, 0 },
    { 0x13af342f, 90 },
    { 0x1e3f3430, 88 },
    { 0x38cf3431, 85 },
    { 0xf26f3430, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef3435, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f343d, 33 },
    { 0, 0 },
    { 0x21ef343f, 77 },
    { 0, 0 },
    { 0x53ff3441, 33 },
    { 0x10af3442, 100 },
    { 0xb17f3442, 33 },
    { 0xd2cf3443, 98 },
    { 0, 0 },
    { 0x924f3446, 23 },
    { 0, 0 },
    { 0xb93f3448, 99 },
    { 0x476f3449, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ff344e, 64 },
    { 0xf21f344e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf3453, 76 },
    { 0x883f3454, 52 },
    { 0xa64f3455, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2f345d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89f3462, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcf3467, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63f346b, 66 },
    { 0, 0 },
    { 0x2fbf346d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0cf3474, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f3479, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f3481, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312f3486, 55 },
    { 0x4fbf3486, 95 },
    { 0x268f3488, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cf348d, 33 },
    { 0, 0 },
    { 0xe74f348f, 90 },
    { 0, 0 },
    { 0x72bf3491, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56df3496, 61 },
    { 0xba5f3497, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ff349b, 98 },
    { 0xcef349c, 23 },
    { 0x8adf349b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x957f34a0, 23 },
    { 0xf7cf34a1, 55 },
    { 0x623f34a0, 88 },
    { 0xa68f34a1, 77 },
    { 0x5fef34a3, 33 },
    { 0xbd6f34a3, 61 },
    { 0x3ddf34a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368f34aa, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9f34ad, 95 },
    { 0, 0 },
    { 0x735f34af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92bf34b4, 88 },
    { 0xe3f34b4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbf34bd, 55 },
    { 0, 0 },
    { 0x74ff34bf, 88 },
    { 0, 0 },
    { 0xd1af34c1, 61 },
    { 0, 0 },
    { 0xcaf34c3, 33 },
    { 0, 0 },
    { 0x46cf34c5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef34ce, 34 },
    { 0xaddf34cf, 76 },
    { 0x651f34cf, 61 },
    { 0, 0 },
    { 0x42cf34d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x205f34d7, 95 },
    { 0x494f34d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x302f34db, 85 },
    { 0x1dbf34db, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47f34e0, 99 },
    { 0xcb0f34e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcf34e5, 100 },
    { 0x387f34e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9abf34e9, 40 },
    { 0x7ecf34ea, 23 },
    { 0x891f34e9, 52 },
    { 0x981f34eb, 34 },
    { 0x8cf34ed, 55 },
    { 0x419f34e9, 95 },
    { 0, 0 },
    { 0xfb4f34f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x722f34fb, 61 },
    { 0x8cbf34fc, 77 },
    { 0x3a1f34fd, 61 },
    { 0xf03f34fd, 43 },
    { 0xa34f34ff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf3503, 64 },
    { 0xeb4f3504, 66 },
    { 0xb97f3504, 93 },
    { 0xb4cf3506, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5f350b, 75 },
    { 0x978f350b, 90 },
    { 0x98bf350d, 33 },
    { 0xa58f350e, 81 },
    { 0xca4f350c, 76 },
    { 0, 0 },
    { 0x16af3511, 61 },
    { 0xc03f3511, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ebf3517, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9f351b, 56 },
    { 0x54ef351c, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f3520, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbaf3525, 90 },
    { 0xfd1f3526, 43 },
    { 0x8d2f3525, 93 },
    { 0x6b4f3528, 34 },
    { 0, 0 },
    { 0xf14f352a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76df352f, 78 },
    { 0xffcf3530, 43 },
    { 0xea6f3530, 43 },
    { 0, 0 },
    { 0x871f3533, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def353c, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3f353f, 23 },
    { 0, 0 },
    { 0x370f3541, 61 },
    { 0, 0 },
    { 0xecf3543, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb65f354b, 34 },
    { 0x9abf354c, 76 },
    { 0x94f354d, 90 },
    { 0x124f354e, 40 },
    { 0x369f354f, 66 },
    { 0x3e0f3550, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f3555, 87 },
    { 0x18bf3556, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ef355b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b9f3562, 55 },
    { 0x204f3563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7f3566, 23 },
    { 0, 0 },
    { 0xf59f3568, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9f356d, 80 },
    { 0xc22f356d, 61 },
    { 0x205f356f, 61 },
    { 0xf7bf356f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ef3574, 61 },
    { 0xd9ef3574, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1f357c, 33 },
    { 0xdcbf357d, 33 },
    { 0xf38f357e, 55 },
    { 0xf82f357f, 98 },
    { 0xe9cf357e, 76 },
    { 0x515f3580, 99 },
    { 0xd8ff357d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df3585, 61 },
    { 0x16cf3586, 61 },
    { 0xc8bf3586, 61 },
    { 0xb20f3588, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2f3591, 61 },
    { 0xf59f3592, 61 },
    { 0xe45f3593, 55 },
    { 0x8d1f3593, 88 },
    { 0xb16f3593, 43 },
    { 0x420f3596, 61 },
    { 0x41ef3593, 95 },
    { 0xf25f3598, 33 },
    { 0x9bef3598, 61 },
    { 0, 0 },
    { 0xd53f359b, 88 },
    { 0, 0 },
    { 0xe7ff359d, 100 },
    { 0, 0 },
    { 0x9b3f359f, 66 },
    { 0, 0 },
    { 0xa3ff35a1, 99 },
    { 0xefdf35a2, 23 },
    { 0, 0 },
    { 0xb22f35a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x79af35a7, 95 },
    { 0xa32f35a8, 95 },
    { 0xb24f35a9, 61 },
    { 0x4d9f35aa, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3f35ad, 67 },
    { 0xc6cf35ae, 33 },
    { 0, 0 },
    { 0x45df35b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80f35b5, 95 },
    { 0, 0 },
    { 0x83af35b7, 99 },
    { 0, 0 },
    { 0xcaff35b9, 98 },
    { 0xa71f35b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ff35bf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604f35c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f35c6, 52 },
    { 0x9dff35c7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f35cb, 24 },
    { 0xd56f35cc, 23 },
    { 0x418f35cb, 98 },
    { 0xa9df35cd, 90 },
    { 0, 0 },
    { 0x3aaf35d0, 61 },
    { 0xd44f35d1, 66 },
    { 0x301f35d2, 90 },
    { 0x9cbf35d3, 98 },
    { 0, 0 },
    { 0x8c2f35d5, 90 },
    { 0x665f35d5, 33 },
    { 0xe57f35d5, 61 },
    { 0x215f35d7, 61 },
    { 0xad7f35d7, 95 },
    { 0xfa4f35d8, 61 },
    { 0, 0 },
    { 0xcaff35dc, 33 },
    { 0, 0 },
    { 0xf5bf35de, 76 },
    { 0xccbf35df, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0xc26f35e2, 76 },
    { 0x714f35e2, 43 },
    { 0x82cf35e4, 98 },
    { 0x1f4f35e3, 17 },
    { 0x693f35e2, 43 },
    { 0x7f3f35e4, 87 },
    { 0x6caf35e8, 98 },
    { 0x7bff35e9, 98 },
    { 0, 0 },
    { 0x56f35eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbf35ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525f35f2, 49 },
    { 0x80df35f3, 98 },
    { 0x7cf35f3, 43 },
    { 0x4c2f35f4, 90 },
    { 0, 0 },
    { 0xa44f35f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf3600, 98 },
    { 0x3ebf3600, 23 },
    { 0xd19f3600, 90 },
    { 0x9f1f3601, 95 },
    { 0x427f3604, 93 },
    { 0xd09f3605, 76 },
    { 0x7f7f3605, 76 },
    { 0x42af3603, 34 },
    { 0xfa6f3600, 98 },
    { 0xb89f3605, 55 },
    { 0x341f360a, 33 },
    { 0x7f7f360b, 90 },
    { 0x371f360c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51f3612, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130f361b, 80 },
    { 0x538f361c, 61 },
    { 0x469f361d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7f3626, 33 },
    { 0x868f3627, 34 },
    { 0x3bef3628, 95 },
    { 0x3ef3628, 38 },
    { 0, 0 },
    { 0x352f362b, 95 },
    { 0x173f362c, 61 },
    { 0, 0 },
    { 0x776f362e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x21cf3631, 90 },
    { 0x38bf3632, 69 },
    { 0x29ff3633, 87 },
    { 0x3b5f3634, 55 },
    { 0x182f3635, 78 },
    { 0xa5af3636, 43 },
    { 0xfd6f3632, 75 },
    { 0xfdff3636, 95 },
    { 0x913f3638, 23 },
    { 0, 0 },
    { 0xca0f363b, 88 },
    { 0xadbf363b, 95 },
    { 0xf82f363d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f3643, 85 },
    { 0x841f3644, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68f3648, 61 },
    { 0xcb0f3649, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x368f364c, 53 },
    { 0xd0f364d, 55 },
    { 0xa05f364d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87af3651, 95 },
    { 0x605f3652, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bf3655, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9caf3659, 61 },
    { 0xea5f3659, 52 },
    { 0xe85f365b, 61 },
    { 0, 0 },
    { 0x23bf365d, 90 },
    { 0x547f365d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d7f366f, 95 },
    { 0xd0af3670, 90 },
    { 0x940f3671, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11af3676, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x976f3679, 33 },
    { 0, 0 },
    { 0x572f367b, 76 },
    { 0xcaaf367c, 84 },
    { 0x628f367d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5df3681, 56 },
    { 0xed4f3681, 43 },
    { 0xc7ef3683, 76 },
    { 0x541f3681, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ef3688, 90 },
    { 0x6a7f3689, 95 },
    { 0, 0 },
    { 0x37af368b, 23 },
    { 0xefbf368c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccf3692, 76 },
    { 0, 0 },
    { 0xbb4f3694, 77 },
    { 0xa5df3694, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebf3698, 33 },
    { 0x4eef3698, 33 },
    { 0xf04f369a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1f36a1, 78 },
    { 0xbb1f36a1, 76 },
    { 0xe8f36a2, 83 },
    { 0x245f36a3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20df36ad, 81 },
    { 0x280f36ad, 34 },
    { 0xc4af36af, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0f36b7, 33 },
    { 0x812f36b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25f36c2, 84 },
    { 0x85af36c2, 46 },
    { 0, 0 },
    { 0xacff36c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83f36ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1caf36cd, 99 },
    { 0, 0 },
    { 0x382f36cf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bf36d3, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0bf36dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x522f36e3, 98 },
    { 0x15ff36e4, 33 },
    { 0x676f36e5, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6f36ed, 23 },
    { 0xdf2f36ee, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7f36f3, 61 },
    { 0xcf36f4, 76 },
    { 0x46f36f4, 90 },
    { 0xe6df36f6, 95 },
    { 0, 0 },
    { 0xf45f36f8, 77 },
    { 0, 0 },
    { 0x2f0f36fa, 61 },
    { 0xcb4f36fb, 52 },
    { 0x224f36fb, 98 },
    { 0xc4af36fb, 66 },
    { 0, 0 },
    { 0xf41f36ff, 34 },
    { 0x17cf3700, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64f3703, 76 },
    { 0x957f3704, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e7f370f, 33 },
    { 0xcd3f3710, 64 },
    { 0, 0 },
    { 0xa4cf3712, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81f3715, 78 },
    { 0x2ccf3716, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa77f3719, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f371d, 33 },
    { 0, 0 },
    { 0x19ef371f, 88 },
    { 0x3d3f3720, 91 },
    { 0x7b4f3721, 76 },
    { 0xbb4f3722, 98 },
    { 0xffff3721, 23 },
    { 0xc22f3721, 61 },
    { 0x4ebf3723, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47f372b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65af3733, 76 },
    { 0xdd1f3734, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940f3738, 33 },
    { 0xbf7f3739, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x339f373d, 85 },
    { 0x5f7f373e, 52 },
    { 0x20cf373f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dff3748, 23 },
    { 0, 0 },
    { 0x74f374a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08f374d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bf3751, 68 },
    { 0x218f3751, 61 },
    { 0xe9af3753, 95 },
    { 0x14ef3753, 23 },
    { 0xc4ef3753, 80 },
    { 0x3a9f3755, 23 },
    { 0, 0 },
    { 0xfbef3758, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x55af375b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403f375f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a9f3762, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x488f3765, 90 },
    { 0x67af3766, 99 },
    { 0, 0 },
    { 0xce6f3768, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8f376b, 98 },
    { 0x315f376c, 23 },
    { 0x428f376d, 95 },
    { 0x47bf376e, 76 },
    { 0x1fdf376f, 85 },
    { 0x2e5f376f, 90 },
    { 0x81f3771, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02f377e, 90 },
    { 0xe10f377f, 55 },
    { 0xb61f377e, 90 },
    { 0, 0 },
    { 0xa11f3782, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x226f3786, 66 },
    { 0, 0 },
    { 0xb07f3788, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756f378c, 90 },
    { 0xcd0f378d, 61 },
    { 0xaaaf378d, 98 },
    { 0x161f378c, 61 },
    { 0x79f3790, 76 },
    { 0xf3cf3790, 33 },
    { 0xd87f378d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13df3797, 95 },
    { 0x14df3798, 95 },
    { 0x45af3799, 46 },
    { 0xb14f3799, 61 },
    { 0x807f3798, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0af379f, 43 },
    { 0xe31f37a0, 77 },
    { 0x6a0f37a1, 33 },
    { 0, 0 },
    { 0xcb4f37a3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8f37a8, 87 },
    { 0, 0 },
    { 0x99df37aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8f37b5, 95 },
    { 0x562f37b5, 91 },
    { 0, 0 },
    { 0xc92f37b8, 52 },
    { 0x177f37b8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f37be, 33 },
    { 0xc43f37bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37f37c2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecf37c8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aaf37d0, 55 },
    { 0xb60f37d1, 61 },
    { 0, 0 },
    { 0x612f37d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1f37dd, 98 },
    { 0xf4cf37de, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39f37e2, 23 },
    { 0xf9ff37e3, 61 },
    { 0xb99f37e2, 76 },
    { 0x68af37e5, 77 },
    { 0xb56f37e6, 46 },
    { 0xd19f37e6, 43 },
    { 0x75ef37e5, 87 },
    { 0xb27f37e9, 87 },
    { 0xde2f37e9, 33 },
    { 0x932f37eb, 67 },
    { 0xaadf37ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf37f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0f37fa, 23 },
    { 0x363f37fb, 90 },
    { 0xcc6f37fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cf3800, 95 },
    { 0x5e2f3800, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613f3812, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51cf3819, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0f3821, 76 },
    { 0x9df3822, 33 },
    { 0x90bf3823, 100 },
    { 0, 0 },
    { 0xe0df3825, 46 },
    { 0x2cef3826, 91 },
    { 0x6eef3827, 61 },
    { 0xa74f3827, 23 },
    { 0x884f3828, 90 },
    { 0x2bbf3825, 55 },
    { 0x770f3828, 90 },
    { 0, 0 },
    { 0xe6ef382d, 55 },
    { 0, 0 },
    { 0x329f382f, 23 },
    { 0x199f382f, 33 },
    { 0xc6ff382f, 93 },
    { 0xc0bf3832, 91 },
    { 0xa38f3832, 95 },
    { 0xa1df3834, 33 },
    { 0, 0 },
    { 0xc22f3836, 100 },
    { 0, 0 },
    { 0x67df3838, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31f383e, 90 },
    { 0xe02f383f, 33 },
    { 0xc2ff3840, 98 },
    { 0xff3f3841, 95 },
    { 0xe96f3841, 55 },
    { 0xc78f383f, 95 },
    { 0x3cff3844, 66 },
    { 0xfeef3845, 90 },
    { 0xbe5f3841, 93 },
    { 0x139f3840, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27af3851, 98 },
    { 0xe17f3852, 55 },
    { 0x999f3852, 95 },
    { 0x1b4f3854, 76 },
    { 0x67cf3853, 95 },
    { 0xe38f3854, 83 },
    { 0, 0 },
    { 0x9a6f3858, 66 },
    { 0xf4df3859, 38 },
    { 0xf0ef385a, 66 },
    { 0x908f385b, 98 },
    { 0x65bf3858, 75 },
    { 0, 0 },
    { 0xb8cf385e, 61 },
    { 0x814f385e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459f386e, 61 },
    { 0x309f386f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37af3875, 61 },
    { 0xbecf3876, 23 },
    { 0xd92f3877, 90 },
    { 0x7bbf3878, 78 },
    { 0x52df3875, 33 },
    { 0, 0 },
    { 0x911f387b, 98 },
    { 0x5cff387c, 46 },
    { 0x306f387d, 43 },
    { 0xdb9f387e, 46 },
    { 0xa67f387f, 61 },
    { 0x5f8f387c, 90 },
    { 0xe0cf3881, 67 },
    { 0, 0 },
    { 0x70cf3883, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252f3887, 66 },
    { 0, 0 },
    { 0x845f3889, 34 },
    { 0x218f388a, 95 },
    { 0xcc9f388b, 84 },
    { 0xc35f388c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53cf3891, 61 },
    { 0x80ef3892, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x446f3899, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67af389d, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abf38a9, 95 },
    { 0x46df38a9, 90 },
    { 0x710f38a9, 23 },
    { 0, 0 },
    { 0x8c6f38ad, 90 },
    { 0x815f38ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85f38b2, 23 },
    { 0x56df38b3, 33 },
    { 0x4fbf38b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf38b9, 23 },
    { 0xf95f38ba, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f4f38c0, 91 },
    { 0, 0 },
    { 0x734f38c2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906f38c8, 90 },
    { 0xb35f38c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98f38ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x220f38d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569f38d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06f38dd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80f38e3, 23 },
    { 0x2f4f38e4, 90 },
    { 0x3dff38e5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0f38ec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922f38f1, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29f38f6, 98 },
    { 0x2e1f38f6, 90 },
    { 0, 0 },
    { 0x880f38f9, 75 },
    { 0x208f38f9, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0xc82f38fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28f3903, 61 },
    { 0x957f3903, 23 },
    { 0x76af3904, 90 },
    { 0x237f3906, 61 },
    { 0x378f3907, 61 },
    { 0x99cf3908, 33 },
    { 0x6b5f3909, 90 },
    { 0x4a7f390a, 36 },
    { 0x3e4f390b, 98 },
    { 0x903f390c, 78 },
    { 0x88bf390d, 74 },
    { 0x180f390d, 52 },
    { 0x310f390f, 33 },
    { 0xbcef390a, 61 },
    { 0xa25f3909, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff3917, 78 },
    { 0xa6bf3917, 99 },
    { 0xba7f3919, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe8f3921, 99 },
    { 0, 0 },
    { 0x116f3923, 61 },
    { 0x5d8f3924, 76 },
    { 0x22f3925, 61 },
    { 0xdd8f3926, 81 },
    { 0x8c6f3927, 40 },
    { 0x9f2f3927, 33 },
    { 0x57bf3929, 95 },
    { 0x14df392a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bf392e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf7f3934, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5f3937, 77 },
    { 0, 0 },
    { 0x28af3939, 55 },
    { 0, 0 },
    { 0xf17f393b, 61 },
    { 0x7b7f393c, 55 },
    { 0xf17f393d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ef3941, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bf3946, 90 },
    { 0x835f3946, 98 },
    { 0xd6cf3948, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ef394e, 33 },
    { 0, 0 },
    { 0x516f3950, 87 },
    { 0x55df3950, 67 },
    { 0x95cf3951, 85 },
    { 0x376f3953, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8f3957, 55 },
    { 0x93af3957, 85 },
    { 0x52df3959, 33 },
    { 0xf09f3959, 48 },
    { 0x9e9f395b, 23 },
    { 0x880f3958, 61 },
    { 0xfcaf395b, 84 },
    { 0x15af395e, 76 },
    { 0xd4cf395b, 48 },
    { 0, 0 },
    { 0xec5f3961, 76 },
    { 0x755f3962, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bf3965, 95 },
    { 0, 0 },
    { 0x6dff3967, 90 },
    { 0x96ef3967, 23 },
    { 0x78ff3969, 90 },
    { 0x44ff3969, 95 },
    { 0x644f396b, 99 },
    { 0x541f396c, 87 },
    { 0, 0 },
    { 0xcf0f396e, 46 },
    { 0, 0 },
    { 0x89f3970, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1f3974, 33 },
    { 0, 0 },
    { 0xd9cf3976, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f397a, 90 },
    { 0x117f397b, 75 },
    { 0, 0 },
    { 0xb6af397d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf3985, 52 },
    { 0x71df3986, 88 },
    { 0xdbcf3986, 90 },
    { 0xc93f3985, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedf398d, 100 },
    { 0xe06f398e, 46 },
    { 0, 0 },
    { 0xaf1f3990, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaf399b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaf399e, 100 },
    { 0x3f6f399f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6af39a2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ef39a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05f39aa, 88 },
    { 0, 0 },
    { 0xcb3f39ac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0f39b1, 52 },
    { 0, 0 },
    { 0x20bf39b3, 90 },
    { 0, 0 },
    { 0x60df39b5, 98 },
    { 0, 0 },
    { 0xe5f39b7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08f39c3, 38 },
    { 0, 0 },
    { 0xddbf39c5, 33 },
    { 0x39f39c6, 98 },
    { 0xae1f39c6, 76 },
    { 0x461f39c8, 43 },
    { 0xe64f39c8, 95 },
    { 0x9e4f39c8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44bf39cf, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe36f39d4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x40af39d7, 55 },
    { 0x9fcf39d8, 53 },
    { 0x25ff39d7, 76 },
    { 0xeedf39da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7f39dd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf39e0, 52 },
    { 0x14af39e0, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab7f39e5, 33 },
    { 0, 0 },
    { 0xb42f39e7, 17 },
    { 0, 0 },
    { 0x861f39e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f2f39ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f39f6, 90 },
    { 0, 0 },
    { 0xf41f39f8, 43 },
    { 0x4b4f39f8, 95 },
    { 0xb6df39fa, 95 },
    { 0, 0 },
    { 0xfc7f39fc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbff3a01, 61 },
    { 0xee0f3a02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229f3a08, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524f3a0c, 61 },
    { 0x9b4f3a0c, 90 },
    { 0xa68f3a0e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71f3a11, 76 },
    { 0xd1af3a11, 76 },
    { 0xbdff3a11, 34 },
    { 0xe39f3a14, 98 },
    { 0x87bf3a15, 46 },
    { 0, 0 },
    { 0x26bf3a17, 53 },
    { 0xd20f3a18, 23 },
    { 0, 0 },
    { 0x9eff3a1a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6f3a1d, 99 },
    { 0xefdf3a1e, 90 },
    { 0x23df3a1f, 90 },
    { 0, 0 },
    { 0x1def3a21, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f3a2b, 61 },
    { 0x13cf3a2c, 43 },
    { 0x302f3a2d, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x71ff3a30, 17 },
    { 0xde9f3a31, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ef3a34, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836f3a38, 95 },
    { 0, 0 },
    { 0x14ef3a3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bf3a3e, 55 },
    { 0x5fff3a3f, 85 },
    { 0, 0 },
    { 0x2ddf3a41, 34 },
    { 0x24ef3a42, 95 },
    { 0xaa4f3a43, 95 },
    { 0x90f3a44, 93 },
    { 0xee5f3a45, 23 },
    { 0, 0 },
    { 0xf1f3a47, 23 },
    { 0xddff3a48, 33 },
    { 0, 0 },
    { 0x3b7f3a4a, 33 },
    { 0xf1ef3a4a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f3a4e, 67 },
    { 0x9a9f3a4e, 33 },
    { 0x56cf3a50, 95 },
    { 0x3edf3a50, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f3a54, 55 },
    { 0x4d0f3a54, 84 },
    { 0x2f2f3a54, 95 },
    { 0x846f3a57, 76 },
    { 0xb91f3a56, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x236f3a5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdf3a5f, 5 },
    { 0, 0 },
    { 0x808f3a61, 61 },
    { 0, 0 },
    { 0xe82f3a63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x509f3a6a, 90 },
    { 0xa4ff3a6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x838f3a6e, 55 },
    { 0x4adf3a6f, 98 },
    { 0xd3ef3a70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef3a74, 33 },
    { 0, 0 },
    { 0x74cf3a76, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebff3a7b, 98 },
    { 0x1d7f3a7c, 61 },
    { 0x9acf3a7b, 99 },
    { 0xb94f3a7e, 81 },
    { 0x58cf3a7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ff3a86, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c0f3a8e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bf3a91, 61 },
    { 0xdbdf3a92, 93 },
    { 0xa29f3a93, 95 },
    { 0, 0 },
    { 0xd9df3a95, 33 },
    { 0xbcdf3a96, 91 },
    { 0xa22f3a97, 76 },
    { 0x9a8f3a96, 38 },
    { 0xabdf3a97, 90 },
    { 0x92ff3a9a, 98 },
    { 0x64ef3a9b, 77 },
    { 0, 0 },
    { 0x45bf3a9d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112f3aa4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24f3aa9, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43f3aac, 90 },
    { 0xebef3aad, 81 },
    { 0x147f3aae, 61 },
    { 0xbe2f3aaf, 88 },
    { 0x2c7f3aad, 65 },
    { 0x602f3ab0, 99 },
    { 0x8cbf3ab2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0af3abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cf3abf, 90 },
    { 0, 0 },
    { 0x515f3ac1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f3ac4, 46 },
    { 0x1d6f3ac5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x662f3ac8, 43 },
    { 0xb17f3ac9, 33 },
    { 0xa65f3aca, 98 },
    { 0x5fdf3aca, 23 },
    { 0xb7ef3aca, 52 },
    { 0xf81f3acd, 66 },
    { 0x97ff3ac9, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1f3ad1, 95 },
    { 0xcb9f3ad2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfdf3ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x871f3ade, 23 },
    { 0x361f3adf, 90 },
    { 0xa88f3adf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cf3ae3, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xf98f3ae6, 91 },
    { 0xd41f3ae7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xee1f3aea, 98 },
    { 0xefbf3aea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751f3af3, 5 },
    { 0xd8ff3af4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f3af8, 55 },
    { 0, 0 },
    { 0x8f6f3afa, 95 },
    { 0, 0 },
    { 0xea3f3afc, 52 },
    { 0x122f3afc, 53 },
    { 0x506f3afc, 90 },
    { 0, 0 },
    { 0x518f3b00, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7af3b0c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf3b11, 91 },
    { 0, 0 },
    { 0x8ddf3b13, 33 },
    { 0x4b2f3b14, 33 },
    { 0x9c9f3b15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df3b1b, 61 },
    { 0xc7af3b1c, 98 },
    { 0xf4ef3b1c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f3b23, 84 },
    { 0x9c8f3b23, 17 },
    { 0x3e7f3b25, 23 },
    { 0x5edf3b26, 33 },
    { 0xd25f3b23, 94 },
    { 0x218f3b23, 48 },
    { 0x81df3b29, 95 },
    { 0x958f3b2a, 93 },
    { 0xe2bf3b2b, 84 },
    { 0x491f3b2c, 87 },
    { 0xc10f3b24, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2f3b31, 52 },
    { 0x96af3b32, 61 },
    { 0x846f3b31, 33 },
    { 0x23bf3b32, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2f3b37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x913f3b3a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ff3b3d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89f3b40, 34 },
    { 0, 0 },
    { 0x2b7f3b42, 90 },
    { 0x389f3b42, 43 },
    { 0xfddf3b42, 34 },
    { 0x8b5f3b43, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff3b4a, 46 },
    { 0xabff3b4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x862f3b50, 65 },
    { 0x71af3b51, 33 },
    { 0xfc7f3b50, 61 },
    { 0x4af3b50, 98 },
    { 0xcabf3b54, 90 },
    { 0x4adf3b55, 34 },
    { 0xa3af3b56, 46 },
    { 0x268f3b55, 99 },
    { 0xdf0f3b58, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x208f3b5b, 36 },
    { 0x434f3b5c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x869f3b5f, 78 },
    { 0, 0 },
    { 0x6d2f3b61, 89 },
    { 0, 0 },
    { 0x8fcf3b63, 75 },
    { 0, 0 },
    { 0xac6f3b65, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x954f3b6e, 55 },
    { 0x545f3b6e, 43 },
    { 0, 0 },
    { 0x5f3b71, 61 },
    { 0, 0 },
    { 0xa95f3b73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ef3b77, 46 },
    { 0, 0 },
    { 0x7acf3b79, 33 },
    { 0, 0 },
    { 0xc36f3b7b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf3b83, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x21df3b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1f3b8b, 90 },
    { 0, 0 },
    { 0x8cdf3b8d, 23 },
    { 0x5a8f3b8e, 43 },
    { 0x397f3b8e, 23 },
    { 0x214f3b8f, 100 },
    { 0x892f3b8e, 37 },
    { 0x161f3b8d, 34 },
    { 0xb99f3b8f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bef3b96, 77 },
    { 0x14f3b96, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4f3b9c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01f3ba0, 84 },
    { 0x1f1f3ba1, 95 },
    { 0x96f3ba1, 33 },
    { 0x8df3ba1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48f3bab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ef3baf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6f3bb5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f3bb9, 90 },
    { 0x4a5f3bba, 95 },
    { 0x8c3f3bbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2f3bc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1df3bc6, 67 },
    { 0xdb7f3bc7, 90 },
    { 0xc14f3bc8, 99 },
    { 0xaa5f3bc9, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bf3bce, 81 },
    { 0, 0 },
    { 0x91bf3bd0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7f3bd3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bf3bda, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72af3bde, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f3be6, 61 },
    { 0xea3f3be7, 76 },
    { 0, 0 },
    { 0x806f3be9, 66 },
    { 0x950f3bea, 85 },
    { 0xceff3beb, 61 },
    { 0x1cef3bec, 90 },
    { 0xce6f3bed, 55 },
    { 0x6f4f3bee, 93 },
    { 0x336f3bea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccef3bf6, 90 },
    { 0x9c7f3bf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ef3c00, 67 },
    { 0, 0 },
    { 0x6c7f3c02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baf3c0b, 90 },
    { 0x967f3c0c, 98 },
    { 0xbc3f3c0c, 52 },
    { 0x4ecf3c0d, 98 },
    { 0xb93f3c0f, 23 },
    { 0x7cef3c0e, 33 },
    { 0xd54f3c11, 95 },
    { 0xe2cf3c12, 78 },
    { 0x50ff3c11, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590f3c17, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x142f3c1c, 88 },
    { 0x195f3c1d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ff3c22, 33 },
    { 0, 0 },
    { 0x43df3c24, 76 },
    { 0x51bf3c24, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1df3c29, 95 },
    { 0, 0 },
    { 0x257f3c2b, 46 },
    { 0x95cf3c2c, 98 },
    { 0x1bdf3c2b, 84 },
    { 0, 0 },
    { 0xb13f3c2f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5f3c36, 23 },
    { 0xb2af3c37, 98 },
    { 0x17f3c38, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1f3c3d, 76 },
    { 0, 0 },
    { 0x46cf3c3f, 90 },
    { 0x818f3c40, 52 },
    { 0xcfef3c41, 99 },
    { 0x89f3c42, 46 },
    { 0x6fcf3c42, 67 },
    { 0x580f3c44, 98 },
    { 0xb40f3c43, 95 },
    { 0x1f1f3c3f, 61 },
    { 0x8fbf3c47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edf3c4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0f3c50, 90 },
    { 0x4b0f3c51, 90 },
    { 0x80bf3c52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaddf3c55, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f3c5a, 93 },
    { 0xd7cf3c5a, 87 },
    { 0x56af3c5a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7af3c60, 83 },
    { 0x87ff3c61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2af3c65, 33 },
    { 0x710f3c66, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x484f3c6b, 43 },
    { 0x758f3c6b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xae7f3c6f, 34 },
    { 0xef8f3c70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccf3c7c, 4 },
    { 0, 0 },
    { 0x7f7f3c7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3f3c81, 95 },
    { 0x4d7f3c82, 61 },
    { 0xd14f3c83, 61 },
    { 0x50f3c84, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x708f3c87, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x807f3c8a, 33 },
    { 0xb55f3c8b, 89 },
    { 0xa3bf3c8b, 87 },
    { 0x7df3c8d, 33 },
    { 0, 0 },
    { 0x8a1f3c8f, 33 },
    { 0x246f3c8f, 52 },
    { 0, 0 },
    { 0x248f3c92, 61 },
    { 0x3cff3c93, 77 },
    { 0x671f3c94, 61 },
    { 0xc6ff3c95, 76 },
    { 0x9c1f3c96, 40 },
    { 0xbcef3c95, 76 },
    { 0xe2ff3c94, 34 },
    { 0, 0 },
    { 0x53df3c9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516f3ca2, 87 },
    { 0xf61f3ca3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9f3caa, 97 },
    { 0x89ff3cab, 84 },
    { 0x81f3cac, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760f3cb0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8f3cb3, 90 },
    { 0x734f3cb4, 76 },
    { 0, 0 },
    { 0xfa7f3cb6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f3cb9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4f3cc1, 23 },
    { 0, 0 },
    { 0x7baf3cc3, 78 },
    { 0xd0af3cc4, 23 },
    { 0xbd2f3cc3, 33 },
    { 0, 0 },
    { 0xcbbf3cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ef3cca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf3ccf, 91 },
    { 0xd4ef3cd0, 61 },
    { 0x56ff3cd1, 66 },
    { 0x4a1f3cd2, 43 },
    { 0xaf3cd3, 67 },
    { 0, 0 },
    { 0xe24f3cd5, 23 },
    { 0xf46f3cd6, 95 },
    { 0x50f3cd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09f3cdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29df3cde, 27 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61f3ce1, 78 },
    { 0xd63f3ce1, 52 },
    { 0x71df3ce2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cf3ce6, 23 },
    { 0, 0 },
    { 0xf8bf3ce8, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5af3cec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cf3cf2, 91 },
    { 0, 0 },
    { 0xac1f3cf4, 55 },
    { 0xc1ef3cf4, 75 },
    { 0, 0 },
    { 0x6b0f3cf7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c1f3cfa, 84 },
    { 0x524f3cfb, 93 },
    { 0x63df3cfb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0f3d01, 43 },
    { 0x218f3d02, 61 },
    { 0x868f3d01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf3d06, 61 },
    { 0x3af3d07, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5f3d0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f3d10, 98 },
    { 0x9acf3d10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3f3d16, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x926f3d19, 90 },
    { 0xd2bf3d19, 95 },
    { 0x719f3d1b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdf3d1f, 95 },
    { 0, 0 },
    { 0xffef3d21, 90 },
    { 0x493f3d22, 36 },
    { 0xacef3d23, 98 },
    { 0xd7bf3d24, 98 },
    { 0x3f9f3d24, 90 },
    { 0, 0 },
    { 0xa28f3d27, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82af3d2c, 95 },
    { 0, 0 },
    { 0xf99f3d2e, 78 },
    { 0xf6af3d2e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eaf3d33, 98 },
    { 0x964f3d34, 61 },
    { 0xabf3d33, 33 },
    { 0, 0 },
    { 0xfd7f3d37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0f3d3b, 98 },
    { 0, 0 },
    { 0x144f3d3d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f3d43, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03f3d49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af3d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f3d50, 91 },
    { 0x549f3d51, 23 },
    { 0x600f3d50, 33 },
    { 0, 0 },
    { 0xe85f3d54, 58 },
    { 0x2a9f3d55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8f3d59, 98 },
    { 0x6a7f3d59, 77 },
    { 0xef2f3d5a, 85 },
    { 0x879f3d5c, 33 },
    { 0x147f3d5c, 95 },
    { 0x289f3d5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf4f3d61, 76 },
    { 0, 0 },
    { 0xcbaf3d63, 23 },
    { 0xebdf3d63, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f3d6a, 90 },
    { 0x133f3d6a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a1f3d70, 87 },
    { 0x1bef3d71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ff3d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x108f3d77, 90 },
    { 0, 0 },
    { 0x1aef3d79, 90 },
    { 0x5eef3d7a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eef3d82, 90 },
    { 0, 0 },
    { 0xf1bf3d84, 93 },
    { 0x263f3d85, 40 },
    { 0xd62f3d86, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f3d89, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f3d8e, 98 },
    { 0xe53f3d8e, 98 },
    { 0xe89f3d8e, 33 },
    { 0x6abf3d8f, 61 },
    { 0x55f3d92, 61 },
    { 0xee9f3d93, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3d96, 98 },
    { 0x915f3d97, 23 },
    { 0x5aef3d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7f3d9b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc49f3da8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9f3dae, 99 },
    { 0, 0 },
    { 0x89ef3db0, 61 },
    { 0x22cf3db0, 34 },
    { 0xafbf3db2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2f3dbc, 43 },
    { 0xb9bf3dbc, 90 },
    { 0xb52f3dbe, 23 },
    { 0xe48f3dbf, 88 },
    { 0x7c3f3dbf, 76 },
    { 0x7b9f3dc1, 98 },
    { 0x115f3dbf, 90 },
    { 0x471f3dbf, 33 },
    { 0xa3df3dbc, 97 },
    { 0xbf4f3dc2, 61 },
    { 0x744f3dbc, 66 },
    { 0xce1f3dc2, 84 },
    { 0x4d7f3dc8, 78 },
    { 0x863f3dc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff3dce, 82 },
    { 0, 0 },
    { 0xd81f3dd0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8f3dd6, 66 },
    { 0x2b9f3dd7, 61 },
    { 0, 0 },
    { 0x8bcf3dd9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f3ddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cef3de3, 77 },
    { 0, 0 },
    { 0xa72f3de5, 90 },
    { 0xc5ef3de6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x702f3de9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f3dec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0f3def, 90 },
    { 0, 0 },
    { 0x199f3df1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcf3df5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff3df8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1f3dfb, 23 },
    { 0x936f3dfc, 95 },
    { 0x6faf3dfd, 78 },
    { 0xb39f3dfb, 43 },
    { 0xc70f3dff, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df3e03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f3e07, 90 },
    { 0, 0 },
    { 0xfcaf3e09, 85 },
    { 0xe6bf3e0a, 52 },
    { 0xe40f3e0a, 98 },
    { 0x3bcf3e0a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5f3e10, 55 },
    { 0, 0 },
    { 0x48df3e12, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0f3e17, 33 },
    { 0x45ef3e18, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80f3e1c, 98 },
    { 0x9a9f3e1d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0f3e22, 93 },
    { 0xcd6f3e22, 61 },
    { 0x75af3e24, 61 },
    { 0x897f3e22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210f3e29, 90 },
    { 0x323f3e2a, 61 },
    { 0xce8f3e2a, 66 },
    { 0xc02f3e2b, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5f3e31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f3e36, 43 },
    { 0xbd7f3e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8f3e3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc9f3e3f, 87 },
    { 0, 0 },
    { 0x785f3e41, 99 },
    { 0, 0 },
    { 0x3f1f3e43, 33 },
    { 0x6f9f3e44, 55 },
    { 0, 0 },
    { 0xc07f3e46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x840f3e49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14f3e4e, 90 },
    { 0xf68f3e4e, 100 },
    { 0, 0 },
    { 0xf9ef3e51, 91 },
    { 0, 0 },
    { 0xcdaf3e53, 76 },
    { 0, 0 },
    { 0x616f3e55, 33 },
    { 0x2fbf3e56, 61 },
    { 0xa30f3e57, 34 },
    { 0x6b3f3e56, 33 },
    { 0, 0 },
    { 0xe15f3e5a, 23 },
    { 0x95ef3e5a, 90 },
    { 0xa98f3e5a, 34 },
    { 0, 0 },
    { 0x22f3e5e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df3e64, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48df3e67, 61 },
    { 0xdaff3e68, 65 },
    { 0x6fdf3e68, 91 },
    { 0xccbf3e6a, 66 },
    { 0xdf4f3e6b, 78 },
    { 0x6caf3e67, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81df3e70, 66 },
    { 0x982f3e70, 83 },
    { 0, 0 },
    { 0x5ddf3e73, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ff3e77, 23 },
    { 0, 0 },
    { 0x526f3e79, 55 },
    { 0, 0 },
    { 0x7c2f3e7b, 91 },
    { 0xf0f3e7c, 33 },
    { 0x2b7f3e7d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9f3e80, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8f3e84, 61 },
    { 0xf5af3e84, 84 },
    { 0x8edf3e86, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cf3e8a, 95 },
    { 0x8d4f3e8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303f3e91, 93 },
    { 0x8baf3e91, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dff3e97, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f3ea1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x267f3ea4, 55 },
    { 0, 0 },
    { 0x12af3ea6, 75 },
    { 0xe7cf3ea7, 76 },
    { 0, 0 },
    { 0x753f3ea9, 61 },
    { 0xe74f3eaa, 43 },
    { 0xc97f3eaa, 33 },
    { 0x6b8f3eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797f3eb1, 61 },
    { 0, 0 },
    { 0x13bf3eb3, 77 },
    { 0xeb4f3eb3, 95 },
    { 0xd51f3eb3, 91 },
    { 0xaa1f3eb5, 98 },
    { 0x674f3eb5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f3ebc, 61 },
    { 0, 0 },
    { 0x647f3ebe, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f3ec1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x612f3ec5, 93 },
    { 0, 0 },
    { 0xba4f3ec7, 95 },
    { 0xa97f3ec8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x887f3ecf, 55 },
    { 0x4b1f3ed0, 100 },
    { 0, 0 },
    { 0xeddf3ed2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40af3ed5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabaf3edd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x265f3ee0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef3ee3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dbf3ee9, 33 },
    { 0x11cf3eea, 33 },
    { 0x25ef3eeb, 55 },
    { 0x5c4f3eec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940f3ef2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdf3ef6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aff3efb, 23 },
    { 0, 0 },
    { 0x101f3efd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b1f3f00, 98 },
    { 0, 0 },
    { 0x28ff3f02, 98 },
    { 0, 0 },
    { 0x214f3f04, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61df3f08, 95 },
    { 0x755f3f09, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f3f0c, 33 },
    { 0x261f3f0d, 97 },
    { 0, 0 },
    { 0x49af3f0f, 98 },
    { 0, 0 },
    { 0xf39f3f11, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccf3f18, 76 },
    { 0x3def3f19, 33 },
    { 0xdd2f3f1a, 68 },
    { 0, 0 },
    { 0x9b8f3f1c, 33 },
    { 0x3ecf3f1c, 55 },
    { 0xbd5f3f1e, 90 },
    { 0xf80f3f1f, 68 },
    { 0, 0 },
    { 0x255f3f21, 95 },
    { 0, 0 },
    { 0x649f3f23, 90 },
    { 0x761f3f23, 90 },
    { 0x354f3f25, 55 },
    { 0x762f3f26, 17 },
    { 0xc59f3f23, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66f3f2f, 61 },
    { 0x550f3f30, 34 },
    { 0, 0 },
    { 0x7aaf3f32, 99 },
    { 0x666f3f33, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb0f3f3a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccf3f3d, 5 },
    { 0, 0 },
    { 0xac4f3f3f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ff3f42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eef3f46, 97 },
    { 0x245f3f47, 33 },
    { 0x718f3f47, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0f3f4f, 66 },
    { 0x41ff3f50, 90 },
    { 0x8d9f3f51, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df3f56, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f3f5b, 90 },
    { 0xaebf3f5b, 98 },
    { 0, 0 },
    { 0x213f3f5e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f3f61, 98 },
    { 0, 0 },
    { 0xf72f3f63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ff3f66, 43 },
    { 0xfadf3f67, 38 },
    { 0xe3df3f67, 55 },
    { 0x351f3f69, 23 },
    { 0x89f3f6a, 95 },
    { 0xd35f3f6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cf3f70, 85 },
    { 0xe21f3f71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0f3f7e, 91 },
    { 0x352f3f7f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e8f3f83, 76 },
    { 0, 0 },
    { 0xdb7f3f85, 33 },
    { 0x8ff3f85, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d7f3f8b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ff3f8f, 19 },
    { 0x385f3f8f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9af3f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ff3f97, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3f3f9b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07f3f9f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ef3fa3, 100 },
    { 0, 0 },
    { 0xdacf3fa5, 91 },
    { 0xed7f3fa5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0df3fa9, 76 },
    { 0x29ff3faa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef3fad, 78 },
    { 0, 0 },
    { 0x78ff3faf, 23 },
    { 0, 0 },
    { 0x7c0f3fb1, 66 },
    { 0, 0 },
    { 0x157f3fb3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273f3fba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x770f3fbf, 90 },
    { 0, 0 },
    { 0x8fcf3fc1, 100 },
    { 0xfb4f3fc2, 86 },
    { 0x7ddf3fc2, 98 },
    { 0x969f3fc4, 99 },
    { 0, 0 },
    { 0x524f3fc6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x977f3fc9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bff3fcc, 66 },
    { 0, 0 },
    { 0x5e8f3fce, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f3fd3, 95 },
    { 0, 0 },
    { 0x996f3fd5, 98 },
    { 0x962f3fd6, 90 },
    { 0xcecf3fd6, 23 },
    { 0, 0 },
    { 0xdef3fd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559f3fdd, 100 },
    { 0x7e1f3fde, 61 },
    { 0x7f3f3fdf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f3fe4, 76 },
    { 0x898f3fe4, 77 },
    { 0xe6bf3fe4, 34 },
    { 0xc1df3fe7, 33 },
    { 0xb52f3fe8, 93 },
    { 0xad5f3fe4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330f3ff2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f3ff5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x700f3ff8, 23 },
    { 0xd29f3ff9, 90 },
    { 0x4e5f3ff9, 98 },
    { 0x164f3ffb, 61 },
    { 0xc74f3ffc, 76 },
    { 0, 0 },
    { 0x28cf3ffe, 76 },
    { 0, 0 },
    { 0x281f4000, 90 },
    { 0, 0 },
    { 0xd73f4002, 61 },
    { 0x2d3f4002, 61 },
    { 0x325f4004, 33 },
    { 0x847f4005, 33 },
    { 0x76cf4006, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2f4009, 91 },
    { 0xed1f400a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f4013, 84 },
    { 0, 0 },
    { 0xb3ef4015, 23 },
    { 0xb1cf4016, 90 },
    { 0x7d6f4017, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bf401a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9f401e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f4021, 95 },
    { 0, 0 },
    { 0x677f4023, 43 },
    { 0xbc8f4023, 76 },
    { 0x248f4023, 33 },
    { 0xb99f4026, 33 },
    { 0xcdef4026, 95 },
    { 0xd0cf4023, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85f402c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb7f4034, 76 },
    { 0xad8f4035, 90 },
    { 0x6b5f4036, 43 },
    { 0, 0 },
    { 0x5af4038, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304f4046, 43 },
    { 0xeb0f4047, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x841f404b, 33 },
    { 0xeb2f404b, 61 },
    { 0, 0 },
    { 0x9f0f404e, 61 },
    { 0, 0 },
    { 0x9c4f4050, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565f4054, 61 },
    { 0x700f4054, 33 },
    { 0xcdcf4055, 33 },
    { 0xbdaf4054, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcf405a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef4062, 33 },
    { 0x5b2f4063, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x252f406d, 85 },
    { 0x61ff406e, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f4074, 90 },
    { 0x6aff4075, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424f407a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141f4090, 33 },
    { 0, 0 },
    { 0x24cf4092, 95 },
    { 0x32af4092, 76 },
    { 0xed4f4094, 90 },
    { 0x628f4094, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ef4098, 95 },
    { 0, 0 },
    { 0x651f409a, 52 },
    { 0xc6f409b, 61 },
    { 0x65df409b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ff40a5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x128f40a8, 90 },
    { 0xb3df40a9, 76 },
    { 0xebaf40a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71f40b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96f40b5, 46 },
    { 0xdfef40b5, 93 },
    { 0xb63f40b7, 95 },
    { 0xbc7f40b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe25f40c1, 98 },
    { 0x551f40c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6f40c7, 61 },
    { 0xca0f40c8, 33 },
    { 0x549f40c9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f40cc, 55 },
    { 0, 0 },
    { 0x5adf40ce, 61 },
    { 0x4f40ce, 61 },
    { 0xc74f40ce, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd38f40d3, 34 },
    { 0xcdf40d4, 99 },
    { 0x7e3f40d4, 61 },
    { 0x1e2f40d4, 93 },
    { 0x2f6f40d3, 36 },
    { 0xce9f40d7, 91 },
    { 0x5a0f40d9, 61 },
    { 0xdb7f40da, 93 },
    { 0x42f40da, 88 },
    { 0x52ff40d9, 90 },
    { 0x9def40dd, 90 },
    { 0x994f40da, 55 },
    { 0xddf40dd, 66 },
    { 0x582f40d8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16af40e6, 46 },
    { 0x59ef40e7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x274f40eb, 95 },
    { 0xb7df40ec, 76 },
    { 0x305f40ec, 94 },
    { 0, 0 },
    { 0x1e7f40ef, 88 },
    { 0, 0 },
    { 0x659f40f1, 23 },
    { 0x710f40f1, 23 },
    { 0x54af40f3, 96 },
    { 0x8adf40f4, 2 },
    { 0, 0 },
    { 0xde3f40f6, 93 },
    { 0xf69f40f7, 23 },
    { 0x821f40f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x98af40fb, 23 },
    { 0, 0 },
    { 0x3cbf40fd, 90 },
    { 0xf61f40fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dff4104, 66 },
    { 0, 0 },
    { 0x9b3f4106, 69 },
    { 0, 0 },
    { 0x7daf4108, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef410d, 33 },
    { 0x587f410d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe53f4113, 34 },
    { 0xfd5f4114, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65f411c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f4121, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f4128, 55 },
    { 0x7d7f4129, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff412c, 33 },
    { 0, 0 },
    { 0x45ef412e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x224f4131, 91 },
    { 0xa56f4131, 76 },
    { 0, 0 },
    { 0x7a4f4134, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccf4139, 61 },
    { 0xa8af413a, 55 },
    { 0x438f413b, 33 },
    { 0, 0 },
    { 0xe58f413d, 61 },
    { 0xea0f413e, 61 },
    { 0x1e6f413e, 19 },
    { 0xb89f4140, 33 },
    { 0x8a9f413f, 33 },
    { 0x747f4142, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f4145, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecf4149, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x469f414c, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653f4151, 33 },
    { 0xcd4f4152, 61 },
    { 0, 0 },
    { 0x82bf4154, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f415d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24df4161, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x397f4168, 90 },
    { 0x9acf4168, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f416d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cf4174, 95 },
    { 0xacaf4175, 61 },
    { 0, 0 },
    { 0xf50f4177, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2f417b, 95 },
    { 0x89cf417c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4f4180, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff4184, 95 },
    { 0x8bef4184, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa17f4188, 38 },
    { 0, 0 },
    { 0x527f418a, 34 },
    { 0xab5f418b, 68 },
    { 0, 0 },
    { 0xb9ff418d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f4190, 34 },
    { 0, 0 },
    { 0x8b6f4192, 61 },
    { 0x2b1f4193, 93 },
    { 0x996f4194, 76 },
    { 0x4c8f4194, 66 },
    { 0x768f4194, 46 },
    { 0x501f4195, 98 },
    { 0, 0 },
    { 0xbf8f4199, 34 },
    { 0xe31f419a, 90 },
    { 0x8b5f4199, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf41a8, 55 },
    { 0, 0 },
    { 0x73df41aa, 33 },
    { 0x23ef41ab, 61 },
    { 0x9f5f41ac, 98 },
    { 0x9eef41ac, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x85ff41b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67f41b3, 66 },
    { 0, 0 },
    { 0x91bf41b5, 85 },
    { 0, 0 },
    { 0x43ef41b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3f41be, 98 },
    { 0x950f41bf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x392f41ca, 55 },
    { 0xb79f41ca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3f41cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f41d5, 99 },
    { 0, 0 },
    { 0x565f41d7, 33 },
    { 0x32df41d8, 97 },
    { 0xdd5f41d7, 23 },
    { 0xa95f41da, 61 },
    { 0xae0f41db, 76 },
    { 0xadbf41dc, 43 },
    { 0x634f41dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6f41e9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2f41ec, 23 },
    { 0, 0 },
    { 0x830f41ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef41f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cf41f7, 90 },
    { 0, 0 },
    { 0xccdf41f9, 66 },
    { 0xf80f41fa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ccf4205, 61 },
    { 0xae1f4205, 64 },
    { 0x9bef4207, 17 },
    { 0x43ef4208, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ef420b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bf4215, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19f4218, 90 },
    { 0x46ff4219, 100 },
    { 0x7b7f421a, 95 },
    { 0xc52f421b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f421f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eaf4222, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef4227, 52 },
    { 0x469f4227, 100 },
    { 0x3def4227, 33 },
    { 0, 0 },
    { 0xbc2f422b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f422e, 91 },
    { 0x8fbf422e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f4232, 66 },
    { 0xa1df4232, 90 },
    { 0x177f4232, 61 },
    { 0x954f4234, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcf423c, 93 },
    { 0x66ff423c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7f4242, 98 },
    { 0xfdcf4242, 33 },
    { 0xbc7f4244, 76 },
    { 0x966f4242, 33 },
    { 0, 0 },
    { 0xe23f4247, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa01f424d, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6f4253, 87 },
    { 0x61ff4253, 61 },
    { 0, 0 },
    { 0xa0f4256, 76 },
    { 0x1bff4257, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff425c, 55 },
    { 0, 0 },
    { 0x1cbf425e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20bf4261, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x288f4265, 90 },
    { 0x94df4265, 99 },
    { 0x8f1f4267, 33 },
    { 0x1eff4267, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedbf426b, 77 },
    { 0x9f4f426b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19f426f, 34 },
    { 0x158f426f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc0f4275, 61 },
    { 0, 0 },
    { 0x589f4277, 98 },
    { 0xc75f4277, 55 },
    { 0, 0 },
    { 0xc0df427a, 100 },
    { 0xdfdf427b, 33 },
    { 0xfb1f427c, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4f4280, 87 },
    { 0xacbf4281, 34 },
    { 0xf6f4281, 90 },
    { 0, 0 },
    { 0x9cdf4284, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3f4289, 66 },
    { 0x693f428a, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbf428e, 67 },
    { 0x94f428f, 33 },
    { 0x63ef428e, 61 },
    { 0x845f4291, 75 },
    { 0xf87f428f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743f4298, 46 },
    { 0xc01f4299, 98 },
    { 0x139f429a, 96 },
    { 0x8faf429a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9f42a6, 34 },
    { 0, 0 },
    { 0x306f42a8, 90 },
    { 0, 0 },
    { 0x6bef42aa, 76 },
    { 0x597f42aa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f42af, 61 },
    { 0xedff42b0, 90 },
    { 0, 0 },
    { 0x20bf42b2, 46 },
    { 0xf41f42b3, 36 },
    { 0xcc0f42b2, 98 },
    { 0xbbaf42b5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f42b8, 95 },
    { 0xa2af42b9, 4 },
    { 0xc13f42ba, 95 },
    { 0x25af42ba, 95 },
    { 0xf92f42bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4f42bf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x836f42c2, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x834f42c5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08f42ca, 93 },
    { 0x331f42cb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb19f42d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4f42d3, 75 },
    { 0x59df42d4, 43 },
    { 0, 0 },
    { 0xb3df42d6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69f42db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x889f42de, 93 },
    { 0x6eff42df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624f42e3, 43 },
    { 0xbe0f42e4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe94f42e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff42ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x776f42ef, 95 },
    { 0xac1f42f0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ef42fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff42ff, 99 },
    { 0x6f4f4300, 52 },
    { 0xa55f42ff, 46 },
    { 0x58cf4302, 81 },
    { 0x4daf4300, 95 },
    { 0xaebf4304, 33 },
    { 0xdf7f4302, 90 },
    { 0xabcf42ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95f430c, 23 },
    { 0x589f430d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f4314, 76 },
    { 0x85f4315, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x51df4318, 90 },
    { 0, 0 },
    { 0xfc8f431a, 99 },
    { 0x42ff431a, 95 },
    { 0x7d8f431c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef431f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899f4323, 90 },
    { 0xacaf4323, 90 },
    { 0, 0 },
    { 0x689f4326, 61 },
    { 0, 0 },
    { 0x603f4328, 98 },
    { 0xb36f4329, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4f432c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecf4330, 33 },
    { 0xabaf4331, 33 },
    { 0x12f4332, 34 },
    { 0xe3bf4331, 33 },
    { 0, 0 },
    { 0x12df4335, 95 },
    { 0xcb9f4336, 23 },
    { 0x2e2f4337, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71f433b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf433f, 95 },
    { 0x2eef433f, 95 },
    { 0x906f433f, 95 },
    { 0x1fcf4342, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ef4348, 76 },
    { 0x715f4349, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddff434d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecf4354, 90 },
    { 0x65bf4354, 95 },
    { 0xb6ff4356, 33 },
    { 0xdfaf4354, 23 },
    { 0xe83f4358, 95 },
    { 0, 0 },
    { 0x13cf435a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132f435e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4bf4361, 95 },
    { 0, 0 },
    { 0x72cf4363, 33 },
    { 0x82df4364, 69 },
    { 0x6af4365, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7f4368, 78 },
    { 0, 0 },
    { 0xb77f436a, 55 },
    { 0, 0 },
    { 0xb42f436c, 98 },
    { 0, 0 },
    { 0x47ef436e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4f4373, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f437a, 55 },
    { 0xde7f437b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5f4381, 83 },
    { 0x574f4382, 87 },
    { 0xef7f4383, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff438f, 90 },
    { 0xf7ff4390, 100 },
    { 0x7a5f4391, 87 },
    { 0x91ef4392, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2f4396, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xb29f4399, 95 },
    { 0x1ff439a, 98 },
    { 0xca5f439b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9f43a0, 61 },
    { 0x7edf43a0, 61 },
    { 0, 0 },
    { 0x7a4f43a3, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4f43aa, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63f43ad, 93 },
    { 0x4e5f43ae, 61 },
    { 0x798f43ad, 91 },
    { 0x997f43b0, 66 },
    { 0xef9f43b1, 99 },
    { 0x747f43b2, 81 },
    { 0, 0 },
    { 0xe9bf43b4, 99 },
    { 0xcadf43b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe17f43be, 23 },
    { 0xa7ef43be, 23 },
    { 0x212f43c0, 90 },
    { 0xb06f43c1, 87 },
    { 0, 0 },
    { 0x8f8f43c3, 78 },
    { 0x841f43c4, 99 },
    { 0xa30f43c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x563f43c8, 55 },
    { 0xc69f43c9, 93 },
    { 0x32ef43c9, 95 },
    { 0, 0 },
    { 0x3d3f43cc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad2f43d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f43d5, 90 },
    { 0, 0 },
    { 0x7e5f43d7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15f43da, 23 },
    { 0x6ff43da, 84 },
    { 0xfcdf43db, 90 },
    { 0x109f43dd, 55 },
    { 0, 0 },
    { 0x45af43df, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ff43ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396f43f2, 52 },
    { 0, 0 },
    { 0x17cf43f4, 33 },
    { 0xd1df43f5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513f43f9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ff43fe, 90 },
    { 0x283f43ff, 61 },
    { 0xb77f4400, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0f4408, 90 },
    { 0x6ccf4408, 90 },
    { 0, 0 },
    { 0x912f440b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9af4412, 67 },
    { 0, 0 },
    { 0x8bdf4414, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x893f4417, 52 },
    { 0x181f4418, 90 },
    { 0, 0 },
    { 0xecdf441a, 77 },
    { 0x66cf441b, 93 },
    { 0x7a5f441a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5f4426, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaf442b, 65 },
    { 0, 0 },
    { 0x7cf442d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37cf4439, 34 },
    { 0x88ef4439, 33 },
    { 0x24ff443b, 46 },
    { 0, 0 },
    { 0x97df443d, 61 },
    { 0xb7df443d, 33 },
    { 0x8a1f443f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x519f4443, 24 },
    { 0x108f4444, 34 },
    { 0xe3af4444, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x153f4449, 33 },
    { 0xa57f444a, 88 },
    { 0, 0 },
    { 0xf69f444c, 23 },
    { 0x1eef444d, 61 },
    { 0x950f444e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf4452, 23 },
    { 0, 0 },
    { 0x33cf4454, 100 },
    { 0x1e0f4455, 95 },
    { 0x42bf4454, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942f4460, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ff4463, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aef4466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90f446a, 55 },
    { 0, 0 },
    { 0xba0f446c, 65 },
    { 0, 0 },
    { 0xca9f446e, 98 },
    { 0xf48f446f, 61 },
    { 0x96ef4470, 55 },
    { 0, 0 },
    { 0x39ef4472, 61 },
    { 0, 0 },
    { 0x709f4474, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4479, 84 },
    { 0xab6f447a, 91 },
    { 0x8acf447b, 77 },
    { 0, 0 },
    { 0x659f447d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f4481, 90 },
    { 0x5f7f4482, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x50df4485, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777f448c, 61 },
    { 0x867f448d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ef4491, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f4498, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf449e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37f44a4, 100 },
    { 0x8b9f44a4, 23 },
    { 0xca6f44a5, 90 },
    { 0xe2cf44a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef44ad, 90 },
    { 0x33f44ad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41f44b1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f44b5, 93 },
    { 0, 0 },
    { 0xa61f44b7, 61 },
    { 0, 0 },
    { 0x74ff44b9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ff44bd, 23 },
    { 0, 0 },
    { 0x3bdf44bf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x688f44c2, 23 },
    { 0xf31f44c2, 90 },
    { 0x2d2f44c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf44c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xda3f44ca, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf44d2, 80 },
    { 0x8b9f44d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cf44d7, 68 },
    { 0xe0f44d8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfccf44dc, 96 },
    { 0x4e9f44dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x679f44e0, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe75f44e4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271f44e8, 75 },
    { 0x7b0f44e9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x892f44ee, 90 },
    { 0x908f44ee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65f44f3, 91 },
    { 0, 0 },
    { 0x657f44f5, 95 },
    { 0x6b6f44f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f44fa, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f4500, 68 },
    { 0x959f4500, 43 },
    { 0, 0 },
    { 0xb9f4503, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x459f4506, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x380f450c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5f4513, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ff4516, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefcf451c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x483f4524, 23 },
    { 0x73ff4525, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8f452b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6f452e, 76 },
    { 0x92cf452e, 90 },
    { 0, 0 },
    { 0x548f4531, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19af453b, 100 },
    { 0xcbdf453b, 55 },
    { 0x924f453d, 52 },
    { 0, 0 },
    { 0x2c9f453f, 90 },
    { 0xb2ef4540, 90 },
    { 0x4e3f4541, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3f4549, 36 },
    { 0, 0 },
    { 0xf73f454b, 90 },
    { 0xe6ef454b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaf4550, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbf4554, 87 },
    { 0, 0 },
    { 0x717f4556, 46 },
    { 0x917f4557, 96 },
    { 0, 0 },
    { 0x911f4559, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fef455c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef455f, 90 },
    { 0, 0 },
    { 0x66ff4561, 90 },
    { 0x8fff4561, 33 },
    { 0x394f4562, 43 },
    { 0x170f4564, 85 },
    { 0xeb6f4565, 86 },
    { 0x371f4561, 23 },
    { 0xf36f4563, 61 },
    { 0x65af4568, 85 },
    { 0x9bbf4569, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57f456c, 34 },
    { 0x146f456d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b7f4575, 90 },
    { 0, 0 },
    { 0x120f4577, 90 },
    { 0x7d1f4577, 98 },
    { 0xfc4f4579, 33 },
    { 0x29df4578, 99 },
    { 0x14df4577, 23 },
    { 0, 0 },
    { 0x21f457d, 33 },
    { 0xc26f457e, 61 },
    { 0x788f457d, 33 },
    { 0, 0 },
    { 0x355f4581, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bf4586, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0f458c, 33 },
    { 0, 0 },
    { 0x3daf458e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3f4594, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cf4597, 46 },
    { 0, 0 },
    { 0xf9f4599, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403f459d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79f45a3, 90 },
    { 0x52af45a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6f45b1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962f45b5, 61 },
    { 0, 0 },
    { 0xf1bf45b7, 23 },
    { 0x86cf45b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x65cf45bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18f45be, 66 },
    { 0, 0 },
    { 0x711f45c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5af45c4, 52 },
    { 0x910f45c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa07f45c8, 64 },
    { 0, 0 },
    { 0x3fbf45ca, 84 },
    { 0, 0 },
    { 0x4c8f45cc, 23 },
    { 0x30ff45cd, 55 },
    { 0, 0 },
    { 0x491f45cf, 91 },
    { 0, 0 },
    { 0x768f45d1, 95 },
    { 0xacf45d2, 98 },
    { 0x1c0f45d3, 95 },
    { 0, 0 },
    { 0x48cf45d5, 98 },
    { 0x3cff45d6, 98 },
    { 0xd6ff45d5, 95 },
    { 0x570f45d8, 33 },
    { 0x4e9f45d8, 78 },
    { 0x3cf45da, 85 },
    { 0x9f3f45da, 23 },
    { 0x847f45dc, 98 },
    { 0x256f45d8, 43 },
    { 0, 0 },
    { 0x490f45df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f45e6, 33 },
    { 0x580f45e6, 90 },
    { 0x439f45e8, 33 },
    { 0x61af45e9, 55 },
    { 0x751f45e8, 95 },
    { 0, 0 },
    { 0x741f45ec, 76 },
    { 0x95af45ed, 98 },
    { 0xb15f45ee, 55 },
    { 0x5a9f45ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af45f3, 34 },
    { 0xcd1f45f4, 61 },
    { 0x665f45f4, 98 },
    { 0x37cf45f6, 61 },
    { 0x912f45f4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef45fd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408f4604, 90 },
    { 0xd73f4605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf460d, 33 },
    { 0x42bf460e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f4618, 87 },
    { 0xf89f4618, 90 },
    { 0xc15f461a, 100 },
    { 0x2c0f4619, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaff4620, 100 },
    { 0, 0 },
    { 0x323f4622, 78 },
    { 0x578f4622, 90 },
    { 0x12bf4622, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcef4628, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f462b, 61 },
    { 0x6a7f462b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf4630, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7f4633, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f463c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21f463f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa29f4643, 33 },
    { 0x610f4644, 93 },
    { 0, 0 },
    { 0xaaf4646, 34 },
    { 0, 0 },
    { 0x537f4648, 90 },
    { 0x5b4f4648, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ff464d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x918f4650, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429f4655, 33 },
    { 0, 0 },
    { 0x56f4657, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x572f465a, 90 },
    { 0, 0 },
    { 0x4b0f465c, 77 },
    { 0x918f465d, 90 },
    { 0xa5af465c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8af4665, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x562f4668, 97 },
    { 0x467f4668, 61 },
    { 0x44ef4669, 90 },
    { 0x3e9f466b, 61 },
    { 0x904f466c, 84 },
    { 0x9f7f466a, 76 },
    { 0xa5cf466e, 33 },
    { 0x4cef466d, 84 },
    { 0xe7df4670, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cf4674, 98 },
    { 0xe80f4675, 90 },
    { 0x24bf4676, 67 },
    { 0x41ff4676, 23 },
    { 0, 0 },
    { 0xb37f4679, 90 },
    { 0x5cf4679, 96 },
    { 0xbf3f467a, 66 },
    { 0x211f467c, 33 },
    { 0, 0 },
    { 0x2e6f467e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f4681, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f4687, 38 },
    { 0x40df4688, 95 },
    { 0xf65f4689, 84 },
    { 0xec9f4689, 90 },
    { 0xdf5f4688, 98 },
    { 0x21ff468b, 84 },
    { 0xd9f4688, 95 },
    { 0x3f3f468e, 33 },
    { 0x44ff468f, 66 },
    { 0xe4f468f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x215f4693, 97 },
    { 0x1a9f4694, 95 },
    { 0xf97f4695, 43 },
    { 0, 0 },
    { 0x4c5f4697, 23 },
    { 0, 0 },
    { 0xd8ff4699, 55 },
    { 0xdc4f4699, 17 },
    { 0x55f469b, 52 },
    { 0x676f469b, 95 },
    { 0x42ff469b, 55 },
    { 0x4eaf469e, 61 },
    { 0x2a6f469f, 90 },
    { 0xfc8f46a0, 43 },
    { 0x68bf469e, 34 },
    { 0xdf6f469f, 55 },
    { 0x852f4699, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x622f46a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf46af, 43 },
    { 0, 0 },
    { 0xa34f46b1, 46 },
    { 0x437f46b1, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cf46b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x340f46ba, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f46c2, 98 },
    { 0xddaf46c3, 61 },
    { 0, 0 },
    { 0xefdf46c5, 90 },
    { 0xab1f46c6, 23 },
    { 0x2e4f46c7, 90 },
    { 0x1e4f46c6, 95 },
    { 0xf25f46c9, 93 },
    { 0x22cf46ca, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f46cd, 61 },
    { 0x39f46cd, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73f46d1, 33 },
    { 0, 0 },
    { 0xc2ff46d3, 23 },
    { 0, 0 },
    { 0x9f0f46d5, 17 },
    { 0x48ef46d6, 55 },
    { 0x810f46d7, 99 },
    { 0x715f46d8, 80 },
    { 0x55ff46d9, 33 },
    { 0x895f46da, 87 },
    { 0x94cf46d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7df46e0, 55 },
    { 0x4f4f46e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963f46e8, 78 },
    { 0x5a1f46e9, 98 },
    { 0x54f46ea, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57af46f3, 90 },
    { 0xbb7f46f4, 97 },
    { 0x1a2f46f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fcf46f8, 90 },
    { 0xe41f46f9, 23 },
    { 0, 0 },
    { 0xbe4f46fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91df4700, 33 },
    { 0x6c9f4700, 61 },
    { 0x9dcf4701, 33 },
    { 0x988f4703, 61 },
    { 0x869f4703, 80 },
    { 0x848f4705, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f4708, 34 },
    { 0, 0 },
    { 0xdd6f470a, 69 },
    { 0, 0 },
    { 0xa7ef470c, 61 },
    { 0x402f470d, 23 },
    { 0xaedf470e, 61 },
    { 0x415f470f, 33 },
    { 0x7c1f470f, 90 },
    { 0xa0ff4711, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40f4714, 61 },
    { 0, 0 },
    { 0xf21f4716, 33 },
    { 0xc21f4717, 55 },
    { 0xee1f4718, 53 },
    { 0x4f6f4718, 23 },
    { 0x542f471a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17f471d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x209f4725, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bf472b, 34 },
    { 0, 0 },
    { 0x1a5f472d, 98 },
    { 0, 0 },
    { 0x3df472f, 90 },
    { 0, 0 },
    { 0xadf4731, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ff4734, 95 },
    { 0x266f4734, 33 },
    { 0, 0 },
    { 0xdbff4737, 99 },
    { 0, 0 },
    { 0xbe7f4739, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f473e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x374f4741, 67 },
    { 0, 0 },
    { 0x453f4743, 91 },
    { 0x84ef4744, 61 },
    { 0x39bf4744, 61 },
    { 0x302f4746, 61 },
    { 0xadaf4744, 95 },
    { 0xf78f4748, 84 },
    { 0x3ddf4748, 85 },
    { 0xa4bf474a, 23 },
    { 0x50df4748, 61 },
    { 0xf69f474c, 76 },
    { 0x3a7f474d, 43 },
    { 0x6f3f474c, 90 },
    { 0xf2cf474f, 98 },
    { 0xba2f4750, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784f4758, 98 },
    { 0xf03f4759, 90 },
    { 0xffbf475a, 17 },
    { 0xeb1f4759, 23 },
    { 0x5a6f475c, 90 },
    { 0, 0 },
    { 0x9e5f475e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cef4762, 61 },
    { 0xbfaf4762, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5df4768, 90 },
    { 0x4af4768, 91 },
    { 0, 0 },
    { 0xf16f476b, 93 },
    { 0, 0 },
    { 0x80cf476d, 98 },
    { 0xf8bf476d, 99 },
    { 0x5d1f476f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f4778, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cf477f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedf4782, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42f4785, 90 },
    { 0x9df4785, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3f4789, 90 },
    { 0xa82f478a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5df4790, 76 },
    { 0, 0 },
    { 0x30ef4792, 90 },
    { 0x5ccf4793, 45 },
    { 0x9f2f4793, 68 },
    { 0x59ff4793, 87 },
    { 0xb12f4793, 95 },
    { 0x559f4795, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505f479b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ef479e, 33 },
    { 0, 0 },
    { 0xd6af47a0, 32 },
    { 0, 0 },
    { 0x130f47a2, 91 },
    { 0xa2ef47a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x66df47a6, 23 },
    { 0xf27f47a7, 98 },
    { 0x505f47a8, 98 },
    { 0x3c0f47a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f47af, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4f47b2, 75 },
    { 0xeddf47b3, 90 },
    { 0, 0 },
    { 0x248f47b5, 69 },
    { 0xf30f47b6, 81 },
    { 0x9d9f47b6, 61 },
    { 0x98ef47b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6f47be, 34 },
    { 0xfe7f47bf, 78 },
    { 0, 0 },
    { 0x8e0f47c1, 80 },
    { 0x771f47c2, 23 },
    { 0xadef47c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23f47c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbf47ca, 90 },
    { 0xc75f47cb, 91 },
    { 0xc14f47cc, 55 },
    { 0, 0 },
    { 0xb27f47ce, 99 },
    { 0x89af47ce, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69df47d4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x635f47d7, 23 },
    { 0, 0 },
    { 0x8e0f47d9, 33 },
    { 0x2ddf47da, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fcf47e5, 61 },
    { 0xdfff47e5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3f47ec, 33 },
    { 0, 0 },
    { 0x720f47ee, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4f47f1, 23 },
    { 0, 0 },
    { 0xa6ff47f3, 65 },
    { 0xcd1f47f4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402f47f8, 23 },
    { 0x41af47f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70f47fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7f4801, 61 },
    { 0x64af4802, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc18f4805, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f480a, 90 },
    { 0x6ef480a, 61 },
    { 0xe64f480c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x429f480f, 43 },
    { 0x9e4f480f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa13f4815, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6f481a, 98 },
    { 0xde4f481b, 33 },
    { 0xc1ef481b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86bf4822, 43 },
    { 0, 0 },
    { 0xe03f4824, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bf4829, 55 },
    { 0, 0 },
    { 0xb5f482b, 43 },
    { 0, 0 },
    { 0xef8f482d, 90 },
    { 0, 0 },
    { 0xd63f482f, 91 },
    { 0x754f482f, 90 },
    { 0xbeef4831, 61 },
    { 0xb4bf4832, 100 },
    { 0xc7cf482f, 95 },
    { 0x1b1f482f, 33 },
    { 0, 0 },
    { 0x5a4f4836, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20df483a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8f483f, 95 },
    { 0x715f4840, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f4843, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478f484c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2f4852, 40 },
    { 0xc26f4853, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157f4857, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93af4861, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b3f4868, 57 },
    { 0xacaf4869, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf0f486e, 55 },
    { 0x486f486e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bf4876, 23 },
    { 0x225f4877, 33 },
    { 0x3b0f4878, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88af487f, 99 },
    { 0, 0 },
    { 0x5a1f4881, 23 },
    { 0, 0 },
    { 0x30f4883, 95 },
    { 0, 0 },
    { 0x38ef4885, 88 },
    { 0x736f4886, 95 },
    { 0xcf9f4887, 95 },
    { 0, 0 },
    { 0x254f4889, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8f4894, 93 },
    { 0, 0 },
    { 0x5bbf4896, 68 },
    { 0, 0 },
    { 0x270f4898, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22f489c, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b5f48a2, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdf48ab, 66 },
    { 0xc0ff48ac, 33 },
    { 0, 0 },
    { 0xe73f48ae, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37f48b1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95f48ba, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x268f48bd, 52 },
    { 0xfa8f48bd, 93 },
    { 0xcc4f48bd, 61 },
    { 0, 0 },
    { 0x73ff48c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88f48c6, 33 },
    { 0x4bbf48c7, 33 },
    { 0, 0 },
    { 0x818f48c9, 90 },
    { 0, 0 },
    { 0x40af48cb, 76 },
    { 0x7a9f48cc, 95 },
    { 0xef9f48cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x823f48d2, 46 },
    { 0x912f48d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e4f48d7, 93 },
    { 0, 0 },
    { 0xf21f48d9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07f48dd, 87 },
    { 0, 0 },
    { 0xcd8f48df, 98 },
    { 0x569f48e0, 67 },
    { 0x873f48e1, 95 },
    { 0xa76f48e0, 56 },
    { 0xe4ff48e3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7bf48e6, 46 },
    { 0xb7bf48e6, 98 },
    { 0x6c1f48e8, 87 },
    { 0x2d0f48e9, 23 },
    { 0, 0 },
    { 0x1ff48eb, 61 },
    { 0, 0 },
    { 0x98ff48ed, 90 },
    { 0xe30f48ee, 90 },
    { 0xc0f48ef, 77 },
    { 0x8dff48f0, 46 },
    { 0x6bff48f1, 76 },
    { 0x440f48ef, 93 },
    { 0x398f48f3, 87 },
    { 0xc87f48f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77f48f8, 98 },
    { 0xea4f48f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf48fc, 84 },
    { 0, 0 },
    { 0x666f48fe, 93 },
    { 0x2fcf48fe, 95 },
    { 0, 0 },
    { 0x341f4901, 93 },
    { 0, 0 },
    { 0x110f4903, 34 },
    { 0xedaf4904, 33 },
    { 0xfef4903, 87 },
    { 0, 0 },
    { 0x7a7f4907, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f490c, 84 },
    { 0x3bff490d, 33 },
    { 0x680f490e, 85 },
    { 0, 0 },
    { 0xf7cf4910, 85 },
    { 0x4a3f4911, 34 },
    { 0xc31f4911, 55 },
    { 0xc70f4911, 90 },
    { 0x176f4914, 83 },
    { 0, 0 },
    { 0xed9f4916, 98 },
    { 0x61ff4917, 34 },
    { 0x20ef4916, 33 },
    { 0x5f8f4919, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x750f491c, 34 },
    { 0x3c9f491d, 33 },
    { 0x6a8f491d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f4922, 87 },
    { 0, 0 },
    { 0x772f4924, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3f4930, 87 },
    { 0x2eff4931, 98 },
    { 0x71ff4930, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f4936, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f4939, 77 },
    { 0, 0 },
    { 0x2a4f493b, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f493e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f4942, 61 },
    { 0, 0 },
    { 0x2b0f4944, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70df4949, 95 },
    { 0, 0 },
    { 0xd29f494b, 61 },
    { 0x685f494c, 33 },
    { 0xe4af494d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaf4950, 90 },
    { 0x63ff4951, 33 },
    { 0x79ef4951, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f4955, 61 },
    { 0x3e3f4956, 99 },
    { 0xa70f4957, 23 },
    { 0x272f4956, 75 },
    { 0, 0 },
    { 0x4f3f495a, 85 },
    { 0xe3af495b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1f4961, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22f4964, 62 },
    { 0xa38f4965, 90 },
    { 0xbb9f4966, 99 },
    { 0xf44f4966, 34 },
    { 0x58df4966, 23 },
    { 0x76bf4964, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5f496e, 87 },
    { 0x3ddf496e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364f4973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60f4977, 88 },
    { 0, 0 },
    { 0x807f4979, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bf497d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe47f4987, 33 },
    { 0xf6ff4987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbf498b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79cf498f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22f4996, 99 },
    { 0x1eef4996, 65 },
    { 0xccbf4998, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ff499c, 33 },
    { 0x575f499d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1f49a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xafef49a8, 98 },
    { 0xf3ef49a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7f49ae, 65 },
    { 0xe39f49af, 81 },
    { 0xba0f49b0, 67 },
    { 0xcfef49b1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f49b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df49ba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x169f49c2, 61 },
    { 0, 0 },
    { 0x588f49c4, 81 },
    { 0x432f49c5, 95 },
    { 0, 0 },
    { 0x157f49c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ff49cb, 76 },
    { 0, 0 },
    { 0x943f49cd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cf49d0, 52 },
    { 0x4bcf49d1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaf49d4, 55 },
    { 0xb5ef49d4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b2f49d8, 76 },
    { 0x6bff49d8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cbf49dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486f49e1, 76 },
    { 0x44f49e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbf49e5, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cf49e9, 93 },
    { 0x492f49ea, 90 },
    { 0, 0 },
    { 0x3b0f49ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f49ef, 90 },
    { 0x117f49ef, 61 },
    { 0x90ef49f1, 34 },
    { 0x150f49f2, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bf49f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cef49fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f4a00, 61 },
    { 0x342f4a01, 90 },
    { 0x8d2f4a02, 66 },
    { 0x980f4a02, 33 },
    { 0x75df4a04, 33 },
    { 0xd12f4a05, 55 },
    { 0x9baf4a04, 76 },
    { 0x7dbf4a00, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f4a0a, 23 },
    { 0, 0 },
    { 0xe5af4a0c, 33 },
    { 0xedbf4a0d, 98 },
    { 0xb3bf4a0e, 34 },
    { 0xdb3f4a0f, 77 },
    { 0x5faf4a0f, 98 },
    { 0x450f4a11, 98 },
    { 0x231f4a11, 85 },
    { 0x1e9f4a11, 34 },
    { 0x58ff4a13, 64 },
    { 0x62f4a12, 76 },
    { 0xff4f4a16, 43 },
    { 0x407f4a17, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690f4a1b, 95 },
    { 0, 0 },
    { 0x8ecf4a1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x461f4a20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7f4a25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce3f4a2a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508f4a30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bf4a33, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f4a37, 69 },
    { 0x7e5f4a38, 95 },
    { 0x55f4a39, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f4a40, 95 },
    { 0x309f4a41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19f4a46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f4a4a, 98 },
    { 0xd99f4a4b, 43 },
    { 0x444f4a4c, 95 },
    { 0x923f4a4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f4a51, 33 },
    { 0x17ef4a52, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eef4a55, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f4a62, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeef4a68, 90 },
    { 0xe1af4a69, 33 },
    { 0x97ff4a6a, 55 },
    { 0x352f4a6b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91af4a70, 55 },
    { 0x684f4a71, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x786f4a74, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5f4a77, 17 },
    { 0, 0 },
    { 0x5f0f4a79, 23 },
    { 0x4d5f4a79, 78 },
    { 0x532f4a7a, 86 },
    { 0x4e1f4a79, 33 },
    { 0, 0 },
    { 0xbcf4a7e, 34 },
    { 0xde3f4a7f, 97 },
    { 0xf6f4a7e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eef4a83, 61 },
    { 0x36ff4a84, 23 },
    { 0xb1bf4a85, 34 },
    { 0xb51f4a84, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9f4a8f, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ff4a92, 43 },
    { 0, 0 },
    { 0x69bf4a94, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40f4a99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f4aa8, 85 },
    { 0xf94f4aa9, 55 },
    { 0, 0 },
    { 0x5ddf4aab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582f4ab8, 98 },
    { 0xaf0f4ab8, 43 },
    { 0x55cf4aba, 55 },
    { 0xbeaf4abb, 90 },
    { 0x9a6f4abc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x103f4abf, 33 },
    { 0x21ef4ac0, 76 },
    { 0x5a5f4ac1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89f4ac5, 90 },
    { 0, 0 },
    { 0x7bbf4ac7, 97 },
    { 0, 0 },
    { 0xff4f4ac9, 93 },
    { 0xfedf4aca, 99 },
    { 0, 0 },
    { 0x734f4acc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f4acf, 95 },
    { 0xbb8f4acf, 80 },
    { 0xae3f4ad1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ff4ad6, 33 },
    { 0, 0 },
    { 0x736f4ad8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faf4adc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3f4adf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bef4ae2, 34 },
    { 0xcacf4ae2, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5f4ae6, 95 },
    { 0, 0 },
    { 0x6eef4ae8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520f4af1, 67 },
    { 0x79f4af2, 90 },
    { 0x4cff4af3, 95 },
    { 0, 0 },
    { 0x3a1f4af5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb80f4af9, 90 },
    { 0xe0df4af9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7f4afd, 43 },
    { 0, 0 },
    { 0x8cff4aff, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64f4b02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ff4b09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x516f4b10, 33 },
    { 0, 0 },
    { 0xbd9f4b12, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f4b1a, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9f4b1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159f4b21, 83 },
    { 0xc81f4b22, 61 },
    { 0xca3f4b22, 95 },
    { 0, 0 },
    { 0x98af4b25, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff4b31, 55 },
    { 0, 0 },
    { 0x11af4b33, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff4b3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe83f4b3f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd26f4b43, 77 },
    { 0x63cf4b44, 33 },
    { 0x25ef4b45, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1f4b48, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267f4b4d, 33 },
    { 0, 0 },
    { 0x403f4b4f, 86 },
    { 0x383f4b50, 17 },
    { 0x913f4b51, 33 },
    { 0x5cdf4b52, 98 },
    { 0x9cdf4b53, 66 },
    { 0x43cf4b50, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd04f4b59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9f4b60, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f4b63, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81f4b66, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637f4b6d, 43 },
    { 0xaeef4b6e, 61 },
    { 0x4f2f4b6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5ff4b73, 99 },
    { 0x65af4b74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35af4b79, 100 },
    { 0, 0 },
    { 0x811f4b7b, 61 },
    { 0, 0 },
    { 0xb38f4b7d, 95 },
    { 0, 0 },
    { 0x658f4b7f, 23 },
    { 0xf88f4b7f, 34 },
    { 0x7e8f4b81, 33 },
    { 0, 0 },
    { 0x584f4b83, 33 },
    { 0, 0 },
    { 0xf70f4b85, 46 },
    { 0xa70f4b86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f4b95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f4b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eff4b9c, 55 },
    { 0, 0 },
    { 0xf91f4b9e, 78 },
    { 0, 0 },
    { 0x50bf4ba0, 34 },
    { 0x1bef4ba0, 23 },
    { 0xdb3f4ba2, 76 },
    { 0xb2cf4ba1, 96 },
    { 0xe17f4ba4, 78 },
    { 0x1faf4ba4, 61 },
    { 0xf04f4ba1, 98 },
    { 0x778f4ba1, 43 },
    { 0, 0 },
    { 0x7a0f4ba9, 98 },
    { 0xbaff4ba9, 95 },
    { 0, 0 },
    { 0xf65f4bac, 98 },
    { 0x958f4bac, 81 },
    { 0x749f4bae, 61 },
    { 0x47ff4baf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6f4bb3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec0f4bb9, 33 },
    { 0x71ef4bba, 93 },
    { 0xcddf4bbb, 33 },
    { 0x781f4bbc, 33 },
    { 0x251f4bbd, 98 },
    { 0x89ef4bbd, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f4bc1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9f4bc4, 52 },
    { 0, 0 },
    { 0x872f4bc6, 46 },
    { 0x78ef4bc7, 46 },
    { 0x5d3f4bc8, 33 },
    { 0x33ff4bc9, 67 },
    { 0xf3cf4bc9, 98 },
    { 0xb1af4bc9, 90 },
    { 0x722f4bc9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31df4bd2, 91 },
    { 0xe66f4bd3, 65 },
    { 0xd53f4bd3, 55 },
    { 0, 0 },
    { 0xcd2f4bd6, 93 },
    { 0xafaf4bd7, 61 },
    { 0x434f4bd8, 90 },
    { 0xe25f4bd6, 46 },
    { 0, 0 },
    { 0x4b6f4bdb, 90 },
    { 0x2cef4bdc, 75 },
    { 0x218f4bdd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe11f4be1, 46 },
    { 0x665f4be1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15f4be6, 34 },
    { 0xcf6f4be7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbf4bea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x97ef4bed, 61 },
    { 0x934f4bed, 98 },
    { 0xac9f4bee, 43 },
    { 0, 0 },
    { 0xf24f4bf1, 33 },
    { 0xd70f4bf2, 23 },
    { 0x248f4bf2, 69 },
    { 0x1b5f4bf4, 61 },
    { 0x9d4f4bf5, 33 },
    { 0xd39f4bf1, 61 },
    { 0x650f4bf7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49af4bfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f4bfe, 76 },
    { 0xdcf4bff, 90 },
    { 0xa36f4c00, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaf4c07, 95 },
    { 0x501f4c08, 75 },
    { 0x642f4c07, 76 },
    { 0, 0 },
    { 0x4af4c0b, 100 },
    { 0x800f4c0c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x96bf4c0f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af4c12, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf4c18, 65 },
    { 0, 0 },
    { 0x38ff4c1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8f4c20, 98 },
    { 0x7aff4c21, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bf4c26, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4f4c29, 87 },
    { 0, 0 },
    { 0x84f4c2b, 98 },
    { 0, 0 },
    { 0x42af4c2d, 33 },
    { 0xc90f4c2d, 55 },
    { 0, 0 },
    { 0xad8f4c30, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x48af4c33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cf4c36, 23 },
    { 0x473f4c37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9f4c3b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693f4c3f, 61 },
    { 0, 0 },
    { 0x2c3f4c41, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c5f4c45, 77 },
    { 0xc20f4c46, 76 },
    { 0x5f3f4c47, 33 },
    { 0, 0 },
    { 0x2dbf4c49, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cf4c4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b9f4c51, 34 },
    { 0, 0 },
    { 0xec6f4c53, 100 },
    { 0, 0 },
    { 0x6e5f4c55, 23 },
    { 0x97df4c56, 55 },
    { 0, 0 },
    { 0xd65f4c58, 93 },
    { 0xf50f4c59, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87f4c5c, 34 },
    { 0, 0 },
    { 0x9c4f4c5e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bf4c62, 95 },
    { 0, 0 },
    { 0xaf7f4c64, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f4c67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6f4c6b, 55 },
    { 0xd45f4c6b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f4c6f, 61 },
    { 0x718f4c70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ff4c73, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ff4c76, 61 },
    { 0x165f4c76, 95 },
    { 0x8b1f4c78, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x832f4c81, 90 },
    { 0x894f4c82, 34 },
    { 0x824f4c83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948f4c88, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef4c90, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ff4c95, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2f4c9a, 43 },
    { 0x5d0f4c9a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ef4ca2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6f4cac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ef4cb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35f4cb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594f4cb8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaf4cbb, 100 },
    { 0, 0 },
    { 0xe43f4cbd, 55 },
    { 0xdb7f4cbe, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5f4cc1, 95 },
    { 0xecdf4cc1, 67 },
    { 0xa8ef4cc3, 84 },
    { 0, 0 },
    { 0xe79f4cc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5f4cce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f4cd2, 61 },
    { 0xe22f4cd3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97df4cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973f4cdc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8f4cdf, 99 },
    { 0xfcef4cdf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x321f4ce5, 98 },
    { 0x143f4ce6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x74af4ce9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ff4ced, 23 },
    { 0, 0 },
    { 0x3b6f4cef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f4cf2, 90 },
    { 0, 0 },
    { 0xc56f4cf4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ff4cf8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658f4cfc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x295f4d01, 80 },
    { 0x75af4d02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f4d05, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72f4d0b, 33 },
    { 0, 0 },
    { 0xc8af4d0d, 95 },
    { 0, 0 },
    { 0x5bcf4d0f, 23 },
    { 0, 0 },
    { 0xb62f4d11, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9f4d1b, 65 },
    { 0x180f4d1c, 90 },
    { 0xdd1f4d1d, 88 },
    { 0x9d2f4d1e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaef4d26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf69f4d2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79f4d2f, 91 },
    { 0xa9af4d30, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8ef4d34, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f4d38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4f4d3b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936f4d40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x548f4d43, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af4d4e, 23 },
    { 0, 0 },
    { 0x4c0f4d50, 85 },
    { 0, 0 },
    { 0xbc1f4d52, 55 },
    { 0x8f1f4d52, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f4d59, 38 },
    { 0x2a0f4d59, 95 },
    { 0x6a7f4d5b, 43 },
    { 0x1df4d5b, 43 },
    { 0x2e8f4d5d, 90 },
    { 0x8bff4d5e, 46 },
    { 0xd3cf4d5f, 66 },
    { 0x2fef4d60, 33 },
    { 0x9d8f4d5e, 91 },
    { 0xb46f4d62, 91 },
    { 0x734f4d63, 90 },
    { 0, 0 },
    { 0x440f4d65, 98 },
    { 0x5d9f4d65, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88f4d69, 23 },
    { 0x624f4d69, 76 },
    { 0x744f4d6b, 88 },
    { 0, 0 },
    { 0xe3cf4d6d, 33 },
    { 0x7bff4d6d, 100 },
    { 0x7c4f4d6f, 43 },
    { 0x714f4d70, 88 },
    { 0xfaff4d6e, 23 },
    { 0, 0 },
    { 0x935f4d73, 78 },
    { 0x893f4d74, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4f4d79, 98 },
    { 0x398f4d79, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abf4d7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ff4d81, 33 },
    { 0x6e3f4d82, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1f4d85, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181f4d8a, 84 },
    { 0x7e7f4d8a, 34 },
    { 0x972f4d8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f8f4d91, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf4d99, 100 },
    { 0x6f8f4d99, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf87f4da0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91df4da9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x299f4db0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49df4db3, 93 },
    { 0x888f4db4, 23 },
    { 0, 0 },
    { 0xc4af4db6, 23 },
    { 0x24af4db6, 96 },
    { 0x5f2f4db8, 23 },
    { 0x24cf4db9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cf4dc0, 95 },
    { 0, 0 },
    { 0x260f4dc2, 87 },
    { 0, 0 },
    { 0x60bf4dc4, 98 },
    { 0, 0 },
    { 0xd4df4dc6, 90 },
    { 0x53df4dc6, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557f4dcb, 34 },
    { 0x752f4dcb, 95 },
    { 0x983f4dcd, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c2f4dd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211f4dde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdadf4de1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33f4df0, 33 },
    { 0xaf6f4df1, 100 },
    { 0xb88f4df2, 98 },
    { 0x9bef4df3, 61 },
    { 0xac1f4df4, 91 },
    { 0x499f4df5, 95 },
    { 0x17df4df5, 23 },
    { 0, 0 },
    { 0x239f4df8, 91 },
    { 0, 0 },
    { 0x3aaf4dfa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10f4dff, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3af4e0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caf4e11, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f4e16, 52 },
    { 0x55f4e17, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f0f4e1b, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbf4e22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f4e2c, 99 },
    { 0x48df4e2c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f4e30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff4e34, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36f4e3b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x873f4e40, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649f4e44, 95 },
    { 0x5eff4e44, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63f4e48, 90 },
    { 0x53ff4e48, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53ff4e4e, 90 },
    { 0xe65f4e4e, 90 },
    { 0x52f4e4e, 33 },
    { 0, 0 },
    { 0xfa6f4e52, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60af4e56, 76 },
    { 0, 0 },
    { 0x578f4e58, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f4e5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdf4e60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbf4e63, 56 },
    { 0, 0 },
    { 0xae9f4e65, 91 },
    { 0x45af4e65, 61 },
    { 0x8fdf4e66, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f4e6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x680f4e6e, 55 },
    { 0x977f4e6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x625f4e74, 90 },
    { 0x811f4e75, 33 },
    { 0x4adf4e74, 91 },
    { 0, 0 },
    { 0x3a3f4e78, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf4e7d, 95 },
    { 0x74cf4e7d, 95 },
    { 0xce3f4e7f, 95 },
    { 0x85f4e80, 90 },
    { 0, 0 },
    { 0xd4bf4e82, 90 },
    { 0, 0 },
    { 0xfe2f4e84, 92 },
    { 0x62af4e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adf4e88, 23 },
    { 0, 0 },
    { 0x54f4e8a, 95 },
    { 0x6f8f4e8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb8f4e8e, 88 },
    { 0x2cef4e8f, 67 },
    { 0x4c4f4e90, 95 },
    { 0xa88f4e91, 33 },
    { 0xe95f4e92, 61 },
    { 0xc41f4e91, 98 },
    { 0x8cef4e91, 61 },
    { 0xff3f4e95, 84 },
    { 0xca6f4e96, 78 },
    { 0x9d3f4e96, 33 },
    { 0, 0 },
    { 0xafff4e99, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cef4e9e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x417f4ea5, 34 },
    { 0x7df4ea6, 61 },
    { 0, 0 },
    { 0xf6bf4ea8, 33 },
    { 0, 0 },
    { 0xb05f4eaa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10df4eaf, 76 },
    { 0, 0 },
    { 0x8adf4eb1, 68 },
    { 0, 0 },
    { 0xf5bf4eb3, 88 },
    { 0x339f4eb4, 95 },
    { 0x478f4eb5, 99 },
    { 0x2bf4eb6, 33 },
    { 0x738f4eb6, 34 },
    { 0xf9ef4eb8, 90 },
    { 0x2ccf4eb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125f4ebe, 84 },
    { 0, 0 },
    { 0xa42f4ec0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdef4ec3, 55 },
    { 0x4fdf4ec3, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8f4ec8, 91 },
    { 0xc93f4ec9, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4f4ecc, 80 },
    { 0x3d5f4ecc, 90 },
    { 0xd2ff4ece, 33 },
    { 0xc39f4ece, 61 },
    { 0x854f4ecc, 90 },
    { 0xb28f4ed1, 91 },
    { 0x4a6f4ecc, 100 },
    { 0x1b0f4ece, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f4ed6, 61 },
    { 0x615f4ed6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49bf4edd, 56 },
    { 0, 0 },
    { 0xc71f4edf, 98 },
    { 0, 0 },
    { 0xabdf4ee1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba2f4ee5, 95 },
    { 0x28af4ee5, 17 },
    { 0xe4af4ee7, 94 },
    { 0xde0f4ee8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaf4eed, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24af4ef3, 61 },
    { 0x297f4ef4, 90 },
    { 0xc5cf4ef5, 61 },
    { 0x970f4ef6, 33 },
    { 0x556f4ef7, 43 },
    { 0, 0 },
    { 0xff3f4ef9, 52 },
    { 0, 0 },
    { 0xda0f4efb, 95 },
    { 0, 0 },
    { 0x656f4efd, 75 },
    { 0x810f4efe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ef4f02, 100 },
    { 0, 0 },
    { 0x46f4f04, 98 },
    { 0x951f4f04, 43 },
    { 0x7d3f4f04, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef4f09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x868f4f0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f4f11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56df4f15, 23 },
    { 0, 0 },
    { 0xb15f4f17, 90 },
    { 0xdcef4f18, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x135f4f20, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x189f4f23, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3f4f2a, 97 },
    { 0xfadf4f2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1baf4f2f, 100 },
    { 0, 0 },
    { 0x5dcf4f31, 61 },
    { 0x1bff4f32, 61 },
    { 0, 0 },
    { 0x2f0f4f34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x37bf4f37, 80 },
    { 0x54ef4f37, 67 },
    { 0x288f4f37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98f4f3d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f4f40, 61 },
    { 0x264f4f41, 90 },
    { 0x46bf4f41, 99 },
    { 0x329f4f43, 77 },
    { 0, 0 },
    { 0x681f4f45, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cf4f49, 76 },
    { 0x249f4f49, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf4f50, 98 },
    { 0, 0 },
    { 0x79df4f52, 23 },
    { 0xc11f4f53, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4f4f5c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f4f5f, 93 },
    { 0x152f4f60, 90 },
    { 0x40ef4f61, 87 },
    { 0x479f4f62, 23 },
    { 0x90f4f5f, 17 },
    { 0, 0 },
    { 0x44ff4f65, 95 },
    { 0x8c1f4f65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f4f6b, 98 },
    { 0x721f4f6c, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c8f4f70, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86df4f80, 90 },
    { 0x48bf4f81, 39 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58f4f87, 46 },
    { 0, 0 },
    { 0x28af4f89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x991f4f90, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85f4f93, 95 },
    { 0x688f4f93, 78 },
    { 0x689f4f95, 85 },
    { 0x160f4f93, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x798f4f99, 33 },
    { 0xd9af4f9a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x52cf4f9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0cf4fa1, 46 },
    { 0x863f4fa2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62f4fa6, 95 },
    { 0xb61f4fa6, 23 },
    { 0x209f4fa8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x357f4fab, 95 },
    { 0x16f4fac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edf4faf, 76 },
    { 0x40af4faf, 87 },
    { 0, 0 },
    { 0xeaaf4fb2, 61 },
    { 0, 0 },
    { 0xa8cf4fb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ff4fbd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f4fc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2f4fca, 23 },
    { 0xaaaf4fca, 76 },
    { 0xfaaf4fcb, 77 },
    { 0, 0 },
    { 0xb4f4fce, 95 },
    { 0x9cef4fce, 98 },
    { 0xeccf4fce, 98 },
    { 0x5bff4fce, 33 },
    { 0, 0 },
    { 0xdf7f4fd3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624f4fd9, 96 },
    { 0, 0 },
    { 0xba9f4fdb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f4fdf, 78 },
    { 0xa9ff4fe0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x76f4fe3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x90af4fe6, 90 },
    { 0, 0 },
    { 0x2cf4fe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00f4feb, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf4ff1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ff4ff4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bf4ff9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f4ffd, 23 },
    { 0xecdf4ffe, 90 },
    { 0x938f4ffd, 75 },
    { 0x39ef4ffe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bf5004, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b7f500d, 98 },
    { 0x8f8f500e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ef5011, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f5019, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdff501e, 34 },
    { 0xe06f501e, 91 },
    { 0x896f5020, 90 },
    { 0, 0 },
    { 0x463f5022, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x8daf5025, 67 },
    { 0, 0 },
    { 0x6dcf5027, 95 },
    { 0, 0 },
    { 0xc3f5029, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe88f5034, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd79f5037, 100 },
    { 0xc29f5038, 40 },
    { 0xff1f5038, 90 },
    { 0xef503a, 84 },
    { 0x645f5038, 55 },
    { 0xc4af503c, 76 },
    { 0x756f503d, 78 },
    { 0x61ff503d, 52 },
    { 0x52df503c, 84 },
    { 0, 0 },
    { 0xd14f5041, 34 },
    { 0xdc6f5042, 33 },
    { 0x4ddf5043, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1f5046, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff504b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x431f5050, 90 },
    { 0, 0 },
    { 0xdc3f5052, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1f5059, 61 },
    { 0x88df5059, 95 },
    { 0x25bf505b, 61 },
    { 0x511f5059, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ef5062, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x878f5068, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4df506c, 55 },
    { 0x320f506c, 43 },
    { 0x69ef506e, 76 },
    { 0xfdcf506f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x716f5072, 81 },
    { 0x875f5073, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x880f5076, 97 },
    { 0, 0 },
    { 0x4eaf5078, 76 },
    { 0x50ff5078, 46 },
    { 0x8fdf507a, 33 },
    { 0xeb5f5078, 61 },
    { 0, 0 },
    { 0x8b6f507d, 33 },
    { 0, 0 },
    { 0x603f507f, 85 },
    { 0x81af507f, 55 },
    { 0xb26f5081, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edf508d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edf5090, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1f5096, 76 },
    { 0x540f5096, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x423f509c, 95 },
    { 0xaaf509d, 76 },
    { 0, 0 },
    { 0x331f509f, 46 },
    { 0, 0 },
    { 0xf77f50a1, 34 },
    { 0x566f50a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ff50aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cf50b2, 98 },
    { 0xc9cf50b2, 34 },
    { 0, 0 },
    { 0xd5bf50b5, 90 },
    { 0, 0 },
    { 0x247f50b7, 43 },
    { 0, 0 },
    { 0xa5bf50b9, 98 },
    { 0xb9df50ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f50be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cbf50c1, 87 },
    { 0xa01f50c2, 61 },
    { 0xb6ef50c3, 95 },
    { 0xdaff50c4, 75 },
    { 0xdcbf50c5, 78 },
    { 0xfb7f50c6, 75 },
    { 0xd7ef50c7, 33 },
    { 0x310f50c5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cf50ce, 98 },
    { 0x777f50cf, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8f50d4, 95 },
    { 0xc2f50d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf50d9, 33 },
    { 0xf16f50da, 23 },
    { 0x5abf50da, 90 },
    { 0xe59f50dc, 66 },
    { 0x2f50da, 90 },
    { 0x8e9f50dd, 43 },
    { 0xbcbf50df, 61 },
    { 0xe0f50e0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d0f50e3, 85 },
    { 0x6cf50e4, 34 },
    { 0, 0 },
    { 0x822f50e6, 65 },
    { 0, 0 },
    { 0x78f50e8, 91 },
    { 0, 0 },
    { 0x229f50ea, 95 },
    { 0x91f50eb, 43 },
    { 0, 0 },
    { 0x1cff50ed, 85 },
    { 0xc25f50ed, 90 },
    { 0, 0 },
    { 0x876f50f0, 55 },
    { 0xd9ff50f1, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x897f50f6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7f50fc, 90 },
    { 0xd49f50fd, 90 },
    { 0x119f50fd, 61 },
    { 0x3e1f50ff, 78 },
    { 0, 0 },
    { 0xab0f5101, 91 },
    { 0, 0 },
    { 0x41af5103, 99 },
    { 0x644f5103, 98 },
    { 0xd8f5104, 33 },
    { 0, 0 },
    { 0x6b6f5107, 17 },
    { 0xc9ff5108, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22f510b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4f5110, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f5116, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6f5119, 95 },
    { 0, 0 },
    { 0x857f511b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5ff511e, 52 },
    { 0, 0 },
    { 0xc47f5120, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2f5124, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78f512e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5ef5132, 33 },
    { 0, 0 },
    { 0x4c6f5134, 95 },
    { 0xbe0f5135, 90 },
    { 0x807f5135, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ef513a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ff513d, 67 },
    { 0, 0 },
    { 0x3d4f513f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf37f5145, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9af5149, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x466f514e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594f5152, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ff5156, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe78f5159, 87 },
    { 0xe86f5159, 95 },
    { 0, 0 },
    { 0x3d5f515c, 23 },
    { 0xba7f515c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86f5165, 87 },
    { 0x2fff5165, 33 },
    { 0xccbf5167, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2f516d, 100 },
    { 0, 0 },
    { 0x3f4f516f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe93f5172, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2f5176, 23 },
    { 0xb93f5177, 99 },
    { 0x1def5178, 77 },
    { 0x25cf5178, 90 },
    { 0xcd4f5177, 76 },
    { 0, 0 },
    { 0xfa7f517c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ff5182, 95 },
    { 0xd90f5183, 23 },
    { 0x64f5182, 93 },
    { 0x7ef5182, 34 },
    { 0x255f5186, 95 },
    { 0, 0 },
    { 0xbf1f5188, 95 },
    { 0, 0 },
    { 0x2e7f518a, 87 },
    { 0, 0 },
    { 0xf35f518c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x301f5190, 90 },
    { 0x184f5190, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x910f5194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x11df5197, 40 },
    { 0, 0 },
    { 0x443f5199, 90 },
    { 0, 0 },
    { 0xa09f519b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x70af519e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756f51a4, 52 },
    { 0xafdf51a4, 95 },
    { 0x5e7f51a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x30bf51a9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f51ac, 95 },
    { 0x6bdf51ad, 33 },
    { 0x882f51ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f51b1, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88f51b4, 76 },
    { 0xbe0f51b5, 90 },
    { 0x443f51b6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12f51b9, 76 },
    { 0x5a5f51ba, 98 },
    { 0x577f51ba, 61 },
    { 0xe91f51bb, 90 },
    { 0x918f51bd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8f51c1, 61 },
    { 0, 0 },
    { 0x64df51c3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x813f51ce, 33 },
    { 0xaf1f51cf, 98 },
    { 0, 0 },
    { 0x80ff51d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5f51d7, 23 },
    { 0, 0 },
    { 0xf9ff51d9, 90 },
    { 0xae1f51da, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdbf51e6, 76 },
    { 0, 0 },
    { 0xed9f51e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc56f51eb, 98 },
    { 0x782f51ec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x655f51f1, 66 },
    { 0x873f51f1, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f51f9, 70 },
    { 0xcf3f51fa, 76 },
    { 0, 0 },
    { 0xb60f51fc, 23 },
    { 0, 0 },
    { 0x7b3f51fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2f5204, 23 },
    { 0x5aaf5204, 100 },
    { 0x171f5206, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x776f520f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15af5217, 98 },
    { 0xc11f5218, 94 },
    { 0x9f5219, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf1f521e, 33 },
    { 0x65df521f, 96 },
    { 0x42bf521f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908f5228, 61 },
    { 0, 0 },
    { 0x571f522a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x169f522d, 23 },
    { 0xf5f522d, 92 },
    { 0x1f0f522e, 76 },
    { 0x604f522f, 61 },
    { 0xe4af5231, 46 },
    { 0xd5bf5231, 33 },
    { 0x1aef522d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f5238, 61 },
    { 0x2d7f5239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf523d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadf5242, 34 },
    { 0xb9ff5243, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3f5248, 77 },
    { 0, 0 },
    { 0xb09f524a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0f524e, 43 },
    { 0xb4bf524f, 33 },
    { 0x606f524f, 90 },
    { 0, 0 },
    { 0xfc9f5252, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61f5256, 90 },
    { 0x745f5256, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0af525c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5f5260, 78 },
    { 0, 0 },
    { 0x73cf5262, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375f5266, 95 },
    { 0x4f7f5266, 34 },
    { 0, 0 },
    { 0xbaf5269, 67 },
    { 0, 0 },
    { 0xfa6f526b, 95 },
    { 0x175f526c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165f5272, 78 },
    { 0x3cef5273, 52 },
    { 0xad1f5273, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974f5278, 77 },
    { 0, 0 },
    { 0x17bf527a, 34 },
    { 0x4abf527a, 90 },
    { 0x509f527c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f5282, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4f5285, 87 },
    { 0x816f5286, 93 },
    { 0x77f5287, 61 },
    { 0x761f5288, 33 },
    { 0xa05f5289, 33 },
    { 0, 0 },
    { 0x4bf528b, 55 },
    { 0, 0 },
    { 0x440f528d, 55 },
    { 0x9baf528e, 93 },
    { 0x718f528e, 4 },
    { 0x811f5290, 76 },
    { 0, 0 },
    { 0xb05f5292, 33 },
    { 0x785f5293, 64 },
    { 0, 0 },
    { 0xebf5295, 93 },
    { 0x469f5296, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7f5299, 90 },
    { 0, 0 },
    { 0xb62f529b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x191f529e, 33 },
    { 0x244f529f, 33 },
    { 0xf77f52a0, 68 },
    { 0xee5f52a0, 95 },
    { 0x70ef52a1, 95 },
    { 0xb95f52a1, 46 },
    { 0, 0 },
    { 0xd6af52a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63f52ab, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22f52ae, 33 },
    { 0, 0 },
    { 0xa7bf52b0, 33 },
    { 0x98bf52b1, 34 },
    { 0xf13f52b1, 33 },
    { 0x546f52b1, 33 },
    { 0xa41f52b1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8f52b7, 33 },
    { 0x541f52b8, 81 },
    { 0x435f52b9, 61 },
    { 0x309f52ba, 33 },
    { 0x28ef52bb, 23 },
    { 0x746f52bb, 98 },
    { 0x37df52ba, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8f52c0, 69 },
    { 0, 0 },
    { 0x71ef52c2, 23 },
    { 0xd73f52c3, 88 },
    { 0, 0 },
    { 0xd9df52c5, 90 },
    { 0x38ef52c6, 61 },
    { 0x4f5f52c7, 46 },
    { 0x1d2f52c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebf52cc, 78 },
    { 0x8bf52cd, 90 },
    { 0x26ff52ce, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10cf52d7, 76 },
    { 0x559f52d7, 33 },
    { 0xb03f52d9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1f52dc, 43 },
    { 0x68af52dd, 33 },
    { 0x8e1f52de, 76 },
    { 0, 0 },
    { 0xd1ff52e0, 61 },
    { 0x33bf52e1, 61 },
    { 0x44cf52e2, 90 },
    { 0x85af52e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6df52e9, 90 },
    { 0x5dbf52ea, 23 },
    { 0, 0 },
    { 0x99ef52ec, 43 },
    { 0x605f52ed, 33 },
    { 0x697f52ed, 77 },
    { 0x1b1f52ef, 23 },
    { 0, 0 },
    { 0xcc3f52f1, 55 },
    { 0, 0 },
    { 0x576f52f3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8af52f7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb94f52fa, 76 },
    { 0xf9cf52fa, 76 },
    { 0xcb8f52fb, 95 },
    { 0x716f52fd, 61 },
    { 0xe8ff52fe, 95 },
    { 0x720f52ff, 61 },
    { 0xd64f52fd, 55 },
    { 0xb43f52fd, 98 },
    { 0xbb6f5300, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcf5305, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bf530a, 95 },
    { 0xf23f530b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef530f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ef5312, 43 },
    { 0xe25f5312, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df5317, 90 },
    { 0xfe0f5318, 75 },
    { 0x221f5318, 23 },
    { 0x85ff5318, 61 },
    { 0x5b9f5318, 31 },
    { 0xa3f531a, 87 },
    { 0xe1ef531c, 33 },
    { 0xf1cf531b, 61 },
    { 0xa47f531f, 84 },
    { 0xd02f531c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27f5323, 95 },
    { 0, 0 },
    { 0x1e8f5325, 61 },
    { 0, 0 },
    { 0x3dff5327, 84 },
    { 0, 0 },
    { 0xd0f5329, 90 },
    { 0x286f532a, 85 },
    { 0x7a6f532a, 90 },
    { 0xf1ff532a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x219f5331, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0af5334, 66 },
    { 0x9bdf5335, 93 },
    { 0xa2ef5336, 99 },
    { 0, 0 },
    { 0x7cef5338, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2f533b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf533e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f7f5341, 55 },
    { 0xa15f5342, 95 },
    { 0x73bf5341, 84 },
    { 0x838f5344, 61 },
    { 0xc84f5344, 52 },
    { 0xc83f5346, 61 },
    { 0, 0 },
    { 0x781f5348, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x267f534d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780f535a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x142f535d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ff5365, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x290f5368, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21f536f, 76 },
    { 0x491f5370, 101 },
    { 0, 0 },
    { 0x2cbf5372, 33 },
    { 0x1e8f5372, 43 },
    { 0, 0 },
    { 0x6a3f5375, 98 },
    { 0, 0 },
    { 0x248f5377, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cf537a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdf537e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05f5384, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe73f538d, 61 },
    { 0x526f538e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x467f5391, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ef5394, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1adf539e, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aff53a2, 90 },
    { 0xd95f53a3, 90 },
    { 0xfccf53a4, 90 },
    { 0xf30f53a5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729f53b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeef53b8, 43 },
    { 0, 0 },
    { 0xc03f53ba, 90 },
    { 0xb07f53bb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4f53bf, 91 },
    { 0, 0 },
    { 0xf0bf53c1, 95 },
    { 0xa81f53c2, 23 },
    { 0, 0 },
    { 0x5dcf53c4, 99 },
    { 0, 0 },
    { 0xfc0f53c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61f53c9, 85 },
    { 0xc6af53ca, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f53d7, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2f53dc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0f53e0, 46 },
    { 0xf1df53e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d5f53e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9af53ed, 78 },
    { 0x304f53ee, 46 },
    { 0x7e0f53ee, 33 },
    { 0xca5f53f0, 33 },
    { 0xddcf53ef, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5df53f4, 23 },
    { 0x81ff53f5, 100 },
    { 0x879f53f5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46af53fd, 55 },
    { 0, 0 },
    { 0xa48f53ff, 33 },
    { 0, 0 },
    { 0x9d1f5401, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fff5404, 33 },
    { 0x1b2f5405, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5f540b, 95 },
    { 0xab6f540b, 93 },
    { 0xc28f540d, 33 },
    { 0xa04f540e, 23 },
    { 0x4cff540e, 95 },
    { 0xe8af5410, 90 },
    { 0, 0 },
    { 0x978f5412, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac3f5415, 91 },
    { 0x265f5416, 55 },
    { 0x4c0f5417, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x57af541a, 33 },
    { 0x986f541b, 78 },
    { 0x7e3f541c, 95 },
    { 0xa2df541b, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55f5421, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ff5429, 95 },
    { 0, 0 },
    { 0xd1ff542b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f5431, 17 },
    { 0x3d6f5431, 98 },
    { 0x88ff5431, 84 },
    { 0, 0 },
    { 0x31ff5435, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4f543b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71bf5442, 33 },
    { 0x214f5442, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af5447, 99 },
    { 0xa85f5447, 55 },
    { 0, 0 },
    { 0xe1cf544a, 90 },
    { 0xc8df544b, 98 },
    { 0x7b7f544b, 61 },
    { 0xa52f544b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x545f5451, 61 },
    { 0x1cbf5451, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36cf5458, 100 },
    { 0, 0 },
    { 0xca4f545a, 90 },
    { 0xbc9f545a, 34 },
    { 0x95af545a, 90 },
    { 0x6a2f545a, 33 },
    { 0x2ccf545e, 81 },
    { 0, 0 },
    { 0x7a6f5460, 23 },
    { 0x916f5460, 90 },
    { 0x5a7f5462, 81 },
    { 0xd26f5462, 90 },
    { 0xc84f5460, 33 },
    { 0x79af5460, 61 },
    { 0x408f5466, 23 },
    { 0x68bf5467, 61 },
    { 0x7cdf5467, 55 },
    { 0xed2f5467, 90 },
    { 0xb06f5469, 61 },
    { 0, 0 },
    { 0xa8df546c, 82 },
    { 0x6ecf546d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836f5471, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f1f5477, 78 },
    { 0, 0 },
    { 0x31ff5479, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbf5481, 90 },
    { 0, 0 },
    { 0x6d8f5483, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f5486, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cf5489, 61 },
    { 0, 0 },
    { 0x9adf548b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7f548e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0f5491, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55f5497, 75 },
    { 0, 0 },
    { 0xd5bf5499, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x351f549c, 91 },
    { 0xc51f549d, 61 },
    { 0x882f549c, 76 },
    { 0, 0 },
    { 0x4d0f54a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf54a4, 98 },
    { 0x220f54a5, 87 },
    { 0x92bf54a6, 98 },
    { 0, 0 },
    { 0x412f54a8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a4f54ac, 35 },
    { 0x981f54ad, 90 },
    { 0, 0 },
    { 0xdb6f54af, 93 },
    { 0x934f54b0, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x984f54b4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x75f54b7, 61 },
    { 0xc61f54b8, 95 },
    { 0, 0 },
    { 0x15ff54ba, 95 },
    { 0x55ef54ba, 33 },
    { 0, 0 },
    { 0xd4ff54bd, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbf54c1, 78 },
    { 0, 0 },
    { 0x630f54c3, 34 },
    { 0, 0 },
    { 0x8cdf54c5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80f54cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b0f54d0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08f54d3, 98 },
    { 0xffcf54d4, 76 },
    { 0xaebf54d4, 33 },
    { 0xf31f54d6, 61 },
    { 0x5bff54d5, 33 },
    { 0xfbf54d6, 55 },
    { 0x5caf54d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ff54e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x617f54e5, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x256f54e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88f54ec, 90 },
    { 0, 0 },
    { 0x428f54ee, 99 },
    { 0x399f54ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0af54f7, 66 },
    { 0xe2f54f7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bf54fc, 43 },
    { 0, 0 },
    { 0xdbef54fe, 89 },
    { 0, 0 },
    { 0x5f5500, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4f5503, 23 },
    { 0, 0 },
    { 0x3c8f5505, 33 },
    { 0xc94f5506, 76 },
    { 0, 0 },
    { 0x7d2f5508, 81 },
    { 0xa5df5508, 95 },
    { 0xed8f550a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x106f550f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83f5517, 78 },
    { 0xc5cf5518, 33 },
    { 0, 0 },
    { 0xb19f551a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52f5521, 90 },
    { 0xe7f5522, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4f5526, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafef552a, 61 },
    { 0x654f552b, 93 },
    { 0x9a9f552b, 34 },
    { 0x545f552a, 61 },
    { 0x19bf552e, 100 },
    { 0, 0 },
    { 0x3f8f5530, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c4f5533, 98 },
    { 0, 0 },
    { 0x7d8f5535, 76 },
    { 0x6e0f5536, 46 },
    { 0x177f5536, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf553b, 78 },
    { 0, 0 },
    { 0xe04f553d, 94 },
    { 0x7d6f553e, 61 },
    { 0, 0 },
    { 0xa11f5540, 33 },
    { 0xca7f5541, 78 },
    { 0x4c3f5540, 55 },
    { 0x3d5f5543, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f5547, 33 },
    { 0, 0 },
    { 0xf30f5549, 95 },
    { 0, 0 },
    { 0x427f554b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdf554e, 23 },
    { 0x8b6f554f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aef5552, 90 },
    { 0x638f5553, 88 },
    { 0xc10f5554, 61 },
    { 0x383f5553, 43 },
    { 0x17f5556, 76 },
    { 0, 0 },
    { 0x608f5558, 93 },
    { 0, 0 },
    { 0xc27f555a, 85 },
    { 0x6f1f555b, 61 },
    { 0x76bf555c, 78 },
    { 0, 0 },
    { 0x229f555e, 100 },
    { 0x5faf555f, 33 },
    { 0x85ef5560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77df5567, 61 },
    { 0, 0 },
    { 0xe82f5569, 66 },
    { 0, 0 },
    { 0x13bf556b, 34 },
    { 0, 0 },
    { 0x699f556d, 43 },
    { 0, 0 },
    { 0xdecf556f, 33 },
    { 0, 0 },
    { 0x2f2f5571, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f557a, 76 },
    { 0, 0 },
    { 0x474f557c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36f5585, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x641f558a, 32 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635f558f, 43 },
    { 0, 0 },
    { 0x1b0f5591, 23 },
    { 0, 0 },
    { 0x96ef5593, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cef5596, 99 },
    { 0x943f5597, 33 },
    { 0xeb5f5597, 33 },
    { 0, 0 },
    { 0xdc4f559a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181f55a1, 95 },
    { 0x3fbf55a2, 87 },
    { 0xb9bf55a3, 55 },
    { 0, 0 },
    { 0x21f55a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7f55aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd92f55ae, 85 },
    { 0x5a9f55af, 90 },
    { 0x8c8f55af, 90 },
    { 0x63f55af, 90 },
    { 0x472f55b1, 43 },
    { 0x195f55b0, 61 },
    { 0xfebf55b4, 90 },
    { 0, 0 },
    { 0xc75f55b6, 68 },
    { 0x971f55b6, 88 },
    { 0xe58f55b6, 76 },
    { 0x635f55b9, 43 },
    { 0x3b5f55ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa15f55bf, 61 },
    { 0, 0 },
    { 0xe4df55c1, 77 },
    { 0xa2f55c2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee8f55c7, 95 },
    { 0x6f5f55c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ff55d4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2f55dd, 98 },
    { 0xabf55de, 43 },
    { 0xb0df55de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbff55e2, 98 },
    { 0, 0 },
    { 0x185f55e4, 23 },
    { 0x872f55e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xae3f55e8, 98 },
    { 0xeef55e9, 36 },
    { 0x333f55ea, 61 },
    { 0xd7cf55eb, 33 },
    { 0x1dbf55ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30f55f7, 76 },
    { 0x6c7f55f8, 76 },
    { 0xb82f55f9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefff55fe, 34 },
    { 0xf2ef55ff, 66 },
    { 0xa0ff55ff, 100 },
    { 0x1ccf55ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef5604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3f5608, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f560c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2f5611, 85 },
    { 0xad0f5612, 33 },
    { 0x690f5613, 76 },
    { 0, 0 },
    { 0xc2bf5615, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x464f5618, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90df561c, 23 },
    { 0x2b8f561d, 33 },
    { 0x5acf561e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fbf5621, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedf5625, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30f562d, 33 },
    { 0x46df562d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df5631, 61 },
    { 0xfa9f5632, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71f563e, 98 },
    { 0, 0 },
    { 0xb41f5640, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5df5643, 100 },
    { 0xca4f5644, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f5647, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2f564a, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x244f5656, 52 },
    { 0xa45f5657, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96df565b, 23 },
    { 0x5e5f565b, 33 },
    { 0, 0 },
    { 0xaf565e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f5662, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76ef566a, 43 },
    { 0x5fcf566b, 55 },
    { 0xf1cf566b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444f5673, 55 },
    { 0xae1f5674, 90 },
    { 0, 0 },
    { 0x60cf5676, 55 },
    { 0, 0 },
    { 0xcb9f5678, 78 },
    { 0, 0 },
    { 0x47ef567a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f567d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x99af5680, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1f5687, 33 },
    { 0, 0 },
    { 0xf18f5689, 43 },
    { 0, 0 },
    { 0x7b6f568b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f568f, 98 },
    { 0x7d8f5690, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x366f5693, 46 },
    { 0, 0 },
    { 0xda6f5695, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabcf5699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7f569e, 87 },
    { 0x3dff569f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ff56a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x950f56a6, 87 },
    { 0xdcbf56a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57f56ac, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cf56b1, 31 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92f56b5, 66 },
    { 0x419f56b6, 61 },
    { 0xfa4f56b7, 99 },
    { 0, 0 },
    { 0x914f56b9, 78 },
    { 0xdccf56ba, 33 },
    { 0, 0 },
    { 0x80df56bc, 90 },
    { 0, 0 },
    { 0xcc8f56be, 76 },
    { 0x8b6f56bf, 90 },
    { 0x613f56c0, 76 },
    { 0x521f56c1, 95 },
    { 0xd87f56c1, 99 },
    { 0, 0 },
    { 0x99bf56c4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ef56c9, 93 },
    { 0x412f56ca, 43 },
    { 0, 0 },
    { 0x5b3f56cc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a3f56cf, 91 },
    { 0, 0 },
    { 0xedf56d1, 95 },
    { 0, 0 },
    { 0xeacf56d3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaf56d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6af56e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bff56e5, 33 },
    { 0x43af56e6, 66 },
    { 0xd2f56e7, 91 },
    { 0x668f56e7, 61 },
    { 0, 0 },
    { 0x5ff56ea, 90 },
    { 0x4fef56ea, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef56ee, 34 },
    { 0xf42f56ef, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf56f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660f56ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6f5702, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc72f5706, 91 },
    { 0, 0 },
    { 0x59cf5708, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff570b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1f570e, 33 },
    { 0, 0 },
    { 0x7d9f5710, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804f5715, 72 },
    { 0, 0 },
    { 0xfbdf5717, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x349f571a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90bf571e, 90 },
    { 0, 0 },
    { 0xa67f5720, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f572f, 61 },
    { 0xa60f5730, 17 },
    { 0x78bf5731, 91 },
    { 0x8b0f5731, 55 },
    { 0x9c3f5733, 95 },
    { 0xac2f5734, 33 },
    { 0x8ecf5735, 19 },
    { 0x707f5736, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5cf5743, 75 },
    { 0, 0 },
    { 0x943f5745, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1def574b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f1f5750, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff5755, 11 },
    { 0, 0 },
    { 0x588f5757, 90 },
    { 0xdaef5757, 98 },
    { 0x7eaf5758, 87 },
    { 0x377f575a, 98 },
    { 0, 0 },
    { 0x207f575c, 23 },
    { 0x563f575d, 99 },
    { 0, 0 },
    { 0x501f575f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf19f5767, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4af576a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x591f576d, 98 },
    { 0, 0 },
    { 0x3f4f576f, 61 },
    { 0x4c6f5770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3cf577b, 23 },
    { 0, 0 },
    { 0x752f577d, 78 },
    { 0xdebf577d, 84 },
    { 0x72ef577e, 98 },
    { 0, 0 },
    { 0x91cf5781, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ef5786, 90 },
    { 0xc88f5787, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f578b, 33 },
    { 0xdc5f578c, 61 },
    { 0xd3cf578d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddf5790, 76 },
    { 0, 0 },
    { 0x6a3f5792, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e1f5797, 90 },
    { 0x77ef5798, 90 },
    { 0x97df5799, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf579c, 33 },
    { 0x5a7f579d, 64 },
    { 0, 0 },
    { 0x3e6f579f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x148f57a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179f57a7, 69 },
    { 0xe1bf57a8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1f57b3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3f57b8, 85 },
    { 0xdef57b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f57bd, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758f57c1, 55 },
    { 0, 0 },
    { 0xbfbf57c3, 46 },
    { 0x43cf57c3, 34 },
    { 0x269f57c4, 75 },
    { 0x572f57c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3f57cb, 23 },
    { 0, 0 },
    { 0x7b6f57cd, 95 },
    { 0, 0 },
    { 0xccbf57cf, 95 },
    { 0x344f57d0, 90 },
    { 0xb2df57d1, 67 },
    { 0, 0 },
    { 0x8bdf57d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0df57dd, 55 },
    { 0, 0 },
    { 0xc65f57df, 95 },
    { 0, 0 },
    { 0x7cbf57e1, 98 },
    { 0x7b1f57e1, 33 },
    { 0, 0 },
    { 0x10ff57e4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffef57f3, 88 },
    { 0xc40f57f3, 33 },
    { 0xd1ef57f5, 66 },
    { 0x5adf57f5, 84 },
    { 0x983f57f7, 98 },
    { 0xb80f57f8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f57fb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08f5801, 55 },
    { 0, 0 },
    { 0xcf6f5803, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aaf5807, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37f580b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f580f, 66 },
    { 0x896f580f, 78 },
    { 0xd46f5811, 90 },
    { 0x708f5812, 33 },
    { 0x84ef5812, 34 },
    { 0xbeaf5814, 93 },
    { 0xa67f5814, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64af5818, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e8f5826, 85 },
    { 0, 0 },
    { 0xbedf5828, 95 },
    { 0, 0 },
    { 0x330f582a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65f5830, 90 },
    { 0, 0 },
    { 0x97df5832, 98 },
    { 0x5dcf5833, 98 },
    { 0xbd0f5832, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6f583c, 76 },
    { 0x8ccf583c, 61 },
    { 0x2adf583e, 93 },
    { 0xdbaf583d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebf5843, 90 },
    { 0, 0 },
    { 0x16ff5845, 81 },
    { 0, 0 },
    { 0x4cbf5847, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f5852, 46 },
    { 0x2b9f5853, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159f5858, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f585c, 95 },
    { 0x7f2f585d, 69 },
    { 0, 0 },
    { 0xf9ff585f, 33 },
    { 0, 0 },
    { 0x149f5861, 90 },
    { 0, 0 },
    { 0x3e4f5863, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74df586f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4df5879, 52 },
    { 0xaebf5879, 91 },
    { 0x98f587a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x982f587f, 65 },
    { 0x120f587f, 95 },
    { 0x70ff587f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4f5884, 33 },
    { 0, 0 },
    { 0xdb6f5886, 99 },
    { 0x4e6f5887, 23 },
    { 0x7aff5888, 87 },
    { 0x367f5889, 90 },
    { 0xc0cf588a, 98 },
    { 0x18af588b, 93 },
    { 0xdf3f588c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6f5891, 95 },
    { 0x273f5892, 43 },
    { 0, 0 },
    { 0x321f5894, 95 },
    { 0xd28f5895, 98 },
    { 0x919f5896, 61 },
    { 0, 0 },
    { 0x699f5898, 61 },
    { 0, 0 },
    { 0xe2ff589a, 66 },
    { 0xcf1f589b, 93 },
    { 0xd28f589b, 66 },
    { 0, 0 },
    { 0xb07f589e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f58a3, 23 },
    { 0x445f58a4, 33 },
    { 0xbcff58a4, 17 },
    { 0, 0 },
    { 0xb21f58a7, 33 },
    { 0, 0 },
    { 0xc7ef58a9, 55 },
    { 0xc7f58aa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadf58ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1f58b1, 76 },
    { 0x5e5f58b1, 95 },
    { 0, 0 },
    { 0x298f58b4, 61 },
    { 0x223f58b5, 65 },
    { 0x12bf58b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x195f58b9, 43 },
    { 0x302f58ba, 23 },
    { 0x8daf58b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57df58c2, 98 },
    { 0xce5f58c2, 91 },
    { 0x2e6f58c2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cef58cc, 95 },
    { 0xb44f58cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ff58d4, 55 },
    { 0x33af58d5, 17 },
    { 0x665f58d6, 95 },
    { 0x568f58d4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef58dd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x583f58e0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9f58e7, 61 },
    { 0xa01f58e7, 33 },
    { 0x7f7f58e9, 55 },
    { 0xa8cf58e7, 46 },
    { 0xd88f58e9, 61 },
    { 0, 0 },
    { 0x32ff58ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29f58f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf58f9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84f58fd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977f5902, 95 },
    { 0, 0 },
    { 0xeef5904, 23 },
    { 0, 0 },
    { 0x127f5906, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ff590a, 23 },
    { 0x66af590a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8f590f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ef5916, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5f5919, 55 },
    { 0, 0 },
    { 0x80f591b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c5f5920, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ff5927, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7f592a, 90 },
    { 0, 0 },
    { 0xd01f592c, 84 },
    { 0xc1bf592d, 90 },
    { 0, 0 },
    { 0xe2df592f, 97 },
    { 0, 0 },
    { 0xf43f5931, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacff5935, 61 },
    { 0xbadf5936, 98 },
    { 0, 0 },
    { 0x8dff5938, 61 },
    { 0xefef5939, 52 },
    { 0x708f593a, 98 },
    { 0x40ff593b, 91 },
    { 0x834f593a, 55 },
    { 0x59ff593d, 98 },
    { 0x438f593d, 95 },
    { 0x574f593f, 90 },
    { 0, 0 },
    { 0x653f5941, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2f5944, 95 },
    { 0x588f5944, 100 },
    { 0x730f5946, 66 },
    { 0xd33f5946, 90 },
    { 0x910f5947, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97f594b, 95 },
    { 0, 0 },
    { 0x442f594d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x379f5954, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bf5957, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cf595a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbf595d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f5962, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bf596b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddf596e, 90 },
    { 0x878f596e, 56 },
    { 0, 0 },
    { 0xb8ff5971, 77 },
    { 0, 0 },
    { 0x95df5973, 33 },
    { 0x225f5973, 65 },
    { 0x13ff5973, 33 },
    { 0x994f5976, 43 },
    { 0x375f5977, 98 },
    { 0x461f5978, 67 },
    { 0xff2f5979, 33 },
    { 0, 0 },
    { 0xae4f597b, 95 },
    { 0x687f597b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ff5980, 98 },
    { 0x10af5980, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc8f598a, 78 },
    { 0x878f598b, 90 },
    { 0, 0 },
    { 0xdb4f598d, 52 },
    { 0, 0 },
    { 0x5a0f598f, 43 },
    { 0x3d8f5990, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebf5993, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af5998, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ecf599c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31f59a1, 23 },
    { 0x5e9f59a2, 69 },
    { 0x7c3f59a2, 93 },
    { 0x8f5f59a4, 46 },
    { 0xef8f59a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f59b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36bf59b5, 67 },
    { 0, 0 },
    { 0xbfff59b7, 23 },
    { 0x440f59b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f59bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x418f59c2, 55 },
    { 0x8c2f59c3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88df59c7, 85 },
    { 0x61ef59c7, 66 },
    { 0xdb9f59c8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9af59cc, 76 },
    { 0xba0f59cc, 98 },
    { 0xc21f59cd, 69 },
    { 0, 0 },
    { 0x25bf59d0, 89 },
    { 0x5bbf59d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x710f59d4, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb38f59db, 23 },
    { 0x80df59db, 55 },
    { 0xa69f59dd, 98 },
    { 0x2adf59dd, 76 },
    { 0, 0 },
    { 0xab0f59e0, 95 },
    { 0xffdf59e0, 23 },
    { 0xa13f59e2, 95 },
    { 0xfb2f59e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ff59e7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ef59eb, 33 },
    { 0xb6bf59ec, 85 },
    { 0xecff59ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b3f59f0, 91 },
    { 0xdfff59f0, 76 },
    { 0x642f59f1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f59f6, 33 },
    { 0x40ef59f7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bef59fd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x642f5a0c, 33 },
    { 0x739f5a0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11f5a11, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xae1f5a14, 95 },
    { 0x944f5a15, 93 },
    { 0, 0 },
    { 0x685f5a17, 90 },
    { 0x690f5a18, 43 },
    { 0x47cf5a19, 84 },
    { 0xeff5a19, 76 },
    { 0x74ff5a1b, 77 },
    { 0x767f5a1c, 95 },
    { 0x1fff5a1d, 76 },
    { 0x3dcf5a1d, 61 },
    { 0x570f5a1f, 34 },
    { 0x1d4f5a1d, 33 },
    { 0x7e1f5a1d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729f5a2a, 46 },
    { 0x3c8f5a2b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x254f5a30, 90 },
    { 0x94cf5a31, 83 },
    { 0, 0 },
    { 0xcfef5a33, 98 },
    { 0, 0 },
    { 0x9f5f5a35, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x13af5a38, 95 },
    { 0x8a7f5a39, 23 },
    { 0xbb6f5a38, 90 },
    { 0xe65f5a3b, 61 },
    { 0xbdbf5a3c, 84 },
    { 0, 0 },
    { 0x143f5a3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78af5a43, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1f5a4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x208f5a4d, 76 },
    { 0x569f5a4e, 90 },
    { 0, 0 },
    { 0x6f2f5a50, 95 },
    { 0x8aef5a50, 33 },
    { 0, 0 },
    { 0x83f5a53, 55 },
    { 0xb09f5a54, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12f5a59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f5a5e, 100 },
    { 0x43bf5a5f, 91 },
    { 0, 0 },
    { 0x802f5a61, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x289f5a67, 91 },
    { 0x9bef5a68, 90 },
    { 0x583f5a67, 98 },
    { 0x600f5a6a, 98 },
    { 0xdd6f5a6b, 38 },
    { 0, 0 },
    { 0x164f5a6d, 17 },
    { 0, 0 },
    { 0x5d0f5a6f, 76 },
    { 0, 0 },
    { 0xf09f5a71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29f5a74, 17 },
    { 0x4a3f5a74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddf5a7a, 23 },
    { 0x656f5a7a, 99 },
    { 0, 0 },
    { 0xf0cf5a7d, 61 },
    { 0xb7bf5a7e, 55 },
    { 0x575f5a7e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9f5a87, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ef5a8a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x565f5a90, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18f5a93, 96 },
    { 0x556f5a94, 17 },
    { 0, 0 },
    { 0x6eff5a96, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f5a9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ff5aa5, 68 },
    { 0x9bdf5aa6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d9f5aa9, 93 },
    { 0x988f5aaa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf81f5aae, 91 },
    { 0, 0 },
    { 0x8fbf5ab0, 76 },
    { 0, 0 },
    { 0x26ff5ab2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x678f5ab9, 23 },
    { 0, 0 },
    { 0x2d9f5abb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbf5ac2, 23 },
    { 0x990f5ac3, 55 },
    { 0x1cef5ac2, 61 },
    { 0, 0 },
    { 0x8cf5ac6, 98 },
    { 0, 0 },
    { 0xd78f5ac8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1f5ace, 98 },
    { 0xb72f5acf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa27f5ad2, 61 },
    { 0, 0 },
    { 0x7f4f5ad4, 76 },
    { 0xc22f5ad5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9f5adb, 90 },
    { 0x765f5adc, 43 },
    { 0xd9af5add, 33 },
    { 0xb2df5ade, 90 },
    { 0xcbdf5adf, 95 },
    { 0, 0 },
    { 0xa08f5ae1, 33 },
    { 0, 0 },
    { 0x1dff5ae3, 90 },
    { 0x69cf5ae4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33f5ae9, 43 },
    { 0x72df5aea, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6f5aef, 64 },
    { 0, 0 },
    { 0x113f5af1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe31f5afb, 95 },
    { 0, 0 },
    { 0x650f5afd, 100 },
    { 0xcfff5afe, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f5b01, 61 },
    { 0x1df5b01, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf22f5b08, 90 },
    { 0, 0 },
    { 0x4aff5b0a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bf5b0d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6f5b13, 95 },
    { 0x341f5b13, 97 },
    { 0x38f5b13, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6f5b18, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cf5b1b, 76 },
    { 0, 0 },
    { 0xc59f5b1d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3f5b22, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f5b25, 34 },
    { 0, 0 },
    { 0xb65f5b27, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x294f5b2a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94df5b2e, 68 },
    { 0x4e2f5b2f, 91 },
    { 0x41af5b2e, 90 },
    { 0xc8bf5b31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165f5b35, 55 },
    { 0x98ff5b36, 43 },
    { 0x89ff5b37, 23 },
    { 0x372f5b36, 33 },
    { 0, 0 },
    { 0xed4f5b3a, 75 },
    { 0xa72f5b3a, 75 },
    { 0x860f5b3b, 95 },
    { 0x4aff5b3d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bef5b42, 77 },
    { 0x2bf5b43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd4f5b47, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf77f5b4b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79f5b50, 99 },
    { 0xecbf5b51, 76 },
    { 0xe1cf5b52, 100 },
    { 0xe96f5b52, 61 },
    { 0xb7af5b53, 67 },
    { 0x109f5b55, 90 },
    { 0x7bcf5b56, 90 },
    { 0x4dff5b56, 93 },
    { 0x707f5b58, 46 },
    { 0xeb0f5b52, 96 },
    { 0x6d6f5b52, 95 },
    { 0xe94f5b55, 65 },
    { 0xc12f5b55, 90 },
    { 0, 0 },
    { 0xd21f5b5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f5b63, 76 },
    { 0x625f5b63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde9f5b68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ef5b6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x783f5b77, 23 },
    { 0x37f5b77, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaf5b7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0f5b82, 95 },
    { 0, 0 },
    { 0xd38f5b84, 23 },
    { 0x8a6f5b85, 76 },
    { 0xc9bf5b86, 90 },
    { 0xbb0f5b86, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd83f5b8a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ef5b90, 90 },
    { 0x70ef5b91, 76 },
    { 0x6a3f5b91, 17 },
    { 0, 0 },
    { 0x4a2f5b94, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9f5b99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed3f5b9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f4f5ba4, 84 },
    { 0xda3f5ba5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c5f5ba8, 33 },
    { 0, 0 },
    { 0x66f5baa, 91 },
    { 0xc2cf5bab, 23 },
    { 0xdabf5bab, 98 },
    { 0x22df5bad, 33 },
    { 0xb7ef5bac, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293f5bb7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x30af5bba, 95 },
    { 0x6d9f5bba, 98 },
    { 0, 0 },
    { 0xe3cf5bbd, 33 },
    { 0xf20f5bbe, 23 },
    { 0x60f5bbf, 55 },
    { 0x729f5bc0, 91 },
    { 0, 0 },
    { 0x678f5bc2, 61 },
    { 0, 0 },
    { 0x368f5bc4, 52 },
    { 0xd47f5bc5, 90 },
    { 0x3ddf5bc5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50f5bc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad1f5bcd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbf5bd1, 55 },
    { 0xf79f5bd2, 95 },
    { 0x5c6f5bd2, 34 },
    { 0x82cf5bd4, 90 },
    { 0, 0 },
    { 0xc66f5bd6, 66 },
    { 0x9e7f5bd6, 61 },
    { 0, 0 },
    { 0x93bf5bd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ff5bde, 78 },
    { 0xc2ef5bdf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ff5be3, 23 },
    { 0x7def5be3, 90 },
    { 0x980f5be5, 43 },
    { 0x738f5be5, 90 },
    { 0xd34f5be7, 34 },
    { 0xa69f5be8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcf5bee, 61 },
    { 0x6c1f5bef, 33 },
    { 0xc41f5bee, 61 },
    { 0x974f5bf0, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f5f5bf5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddf5bf8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05f5bfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x827f5bfe, 28 },
    { 0x3cef5bfe, 95 },
    { 0xd3ef5c00, 100 },
    { 0x9d5f5bfe, 90 },
    { 0, 0 },
    { 0xb97f5c03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedff5c07, 61 },
    { 0, 0 },
    { 0xfe9f5c09, 90 },
    { 0xfbcf5c0a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f5c14, 84 },
    { 0x69f5c15, 23 },
    { 0, 0 },
    { 0x556f5c17, 66 },
    { 0x6c9f5c18, 76 },
    { 0x879f5c19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f5c1d, 33 },
    { 0, 0 },
    { 0x25ef5c1f, 78 },
    { 0xf2f5c1f, 100 },
    { 0x41df5c21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaf5c2a, 90 },
    { 0, 0 },
    { 0xbeff5c2c, 90 },
    { 0x2af5c2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61f5c31, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x16af5c34, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f5c37, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10f5c3d, 55 },
    { 0, 0 },
    { 0x7eff5c3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x712f5c4c, 95 },
    { 0, 0 },
    { 0x8a2f5c4e, 61 },
    { 0, 0 },
    { 0x126f5c50, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1f5c53, 23 },
    { 0x1e6f5c54, 76 },
    { 0x3c5f5c55, 87 },
    { 0x6f1f5c56, 61 },
    { 0xde3f5c57, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56af5c5e, 95 },
    { 0x5a8f5c5f, 90 },
    { 0, 0 },
    { 0x282f5c61, 43 },
    { 0, 0 },
    { 0x915f5c63, 33 },
    { 0xa69f5c63, 61 },
    { 0x249f5c64, 61 },
    { 0xa56f5c66, 98 },
    { 0x7aff5c67, 94 },
    { 0x1af5c68, 23 },
    { 0xe64f5c67, 76 },
    { 0xbbf5c6a, 93 },
    { 0x964f5c6b, 43 },
    { 0xd0ef5c6c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ff5c70, 99 },
    { 0x5d1f5c70, 76 },
    { 0xcd6f5c72, 33 },
    { 0x468f5c70, 23 },
    { 0x91bf5c71, 87 },
    { 0x4a3f5c70, 95 },
    { 0x4b7f5c76, 33 },
    { 0xa08f5c76, 23 },
    { 0x718f5c78, 33 },
    { 0xf25f5c73, 90 },
    { 0xd12f5c7a, 66 },
    { 0x18af5c7b, 33 },
    { 0x78ff5c7b, 90 },
    { 0x6cef5c70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x283f5c83, 77 },
    { 0xa11f5c83, 81 },
    { 0x9d6f5c83, 98 },
    { 0xaf5c86, 90 },
    { 0xf50f5c83, 34 },
    { 0xd13f5c86, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304f5c8c, 55 },
    { 0, 0 },
    { 0x6ddf5c8e, 72 },
    { 0, 0 },
    { 0x46bf5c90, 95 },
    { 0xdacf5c90, 55 },
    { 0xbeef5c91, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f5c95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08f5c99, 66 },
    { 0x17f5c9a, 98 },
    { 0x1f6f5c9a, 33 },
    { 0x54cf5c9b, 85 },
    { 0x111f5c9d, 90 },
    { 0x64bf5c9e, 98 },
    { 0x9c4f5c9c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x465f5ca5, 33 },
    { 0, 0 },
    { 0xbfaf5ca7, 90 },
    { 0x654f5ca7, 23 },
    { 0x7df5ca8, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x448f5cad, 90 },
    { 0xec4f5cad, 95 },
    { 0xe33f5cad, 90 },
    { 0x92f5cb0, 98 },
    { 0xc49f5cad, 90 },
    { 0x414f5caf, 84 },
    { 0xa4ef5cb3, 55 },
    { 0xa0bf5cb4, 88 },
    { 0x86bf5cb4, 23 },
    { 0, 0 },
    { 0x63bf5cb7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89f5cbb, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdf5cc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cef5cc3, 34 },
    { 0xcfff5cc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abf5ccc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adf5cd1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817f5cd5, 46 },
    { 0, 0 },
    { 0x186f5cd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccf5cde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4acf5ce4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f5ce7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600f5cec, 43 },
    { 0x56ff5ced, 81 },
    { 0, 0 },
    { 0x97af5cef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x247f5cf6, 23 },
    { 0xf95f5cf6, 95 },
    { 0x2e8f5cf6, 56 },
    { 0x256f5cf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734f5d02, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b3f5d05, 81 },
    { 0x956f5d06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6f5d09, 61 },
    { 0x72af5d0a, 93 },
    { 0x86f5d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cf5d0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6f5d17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd12f5d1a, 46 },
    { 0xa1af5d1b, 52 },
    { 0x27f5d1a, 95 },
    { 0, 0 },
    { 0x678f5d1e, 97 },
    { 0, 0 },
    { 0xfeef5d20, 90 },
    { 0xdc4f5d21, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0f5d27, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bef5d2a, 85 },
    { 0x108f5d2b, 88 },
    { 0xf46f5d2b, 33 },
    { 0, 0 },
    { 0x49af5d2e, 98 },
    { 0, 0 },
    { 0x98bf5d30, 95 },
    { 0xc00f5d31, 92 },
    { 0xd87f5d30, 66 },
    { 0x93ff5d33, 95 },
    { 0x239f5d34, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9f5d3d, 61 },
    { 0x399f5d3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf5d48, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabf5d4b, 90 },
    { 0x2b8f5d4c, 34 },
    { 0, 0 },
    { 0x62af5d4e, 80 },
    { 0, 0 },
    { 0x6fdf5d50, 76 },
    { 0, 0 },
    { 0x144f5d52, 99 },
    { 0, 0 },
    { 0xd2f5d54, 84 },
    { 0, 0 },
    { 0x963f5d56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94ef5d63, 90 },
    { 0, 0 },
    { 0x81ff5d65, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabf5d75, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3bf5d78, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f5d7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70cf5d82, 91 },
    { 0, 0 },
    { 0xb1bf5d84, 90 },
    { 0x2d3f5d85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf5d8a, 87 },
    { 0xeadf5d8b, 78 },
    { 0, 0 },
    { 0x526f5d8d, 23 },
    { 0x76af5d8d, 33 },
    { 0, 0 },
    { 0xd82f5d90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f5d95, 95 },
    { 0, 0 },
    { 0x490f5d97, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3f5da0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4f5da5, 93 },
    { 0xa0bf5da5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ef5da9, 33 },
    { 0, 0 },
    { 0x52df5dab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ff5db3, 85 },
    { 0x40af5db4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dbf5db7, 90 },
    { 0, 0 },
    { 0xfc4f5db9, 61 },
    { 0x4df5dba, 98 },
    { 0x588f5dba, 46 },
    { 0x11ef5dbb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ff5dc4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aff5dcf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91f5dd3, 23 },
    { 0, 0 },
    { 0xe8bf5dd5, 33 },
    { 0x1bdf5dd6, 33 },
    { 0xa75f5dd7, 61 },
    { 0xd8ff5dd7, 93 },
    { 0x1e0f5dd7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2f5dde, 76 },
    { 0x6fcf5ddf, 87 },
    { 0x53ff5dde, 87 },
    { 0xf60f5ddf, 98 },
    { 0x2e1f5de1, 66 },
    { 0xea0f5de3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f5de7, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a8f5deb, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70f5dee, 52 },
    { 0x7cf5dee, 61 },
    { 0xfbff5def, 98 },
    { 0x86f5df1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ef5df5, 61 },
    { 0xff2f5df5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfff5dfc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01f5dff, 52 },
    { 0x928f5e00, 100 },
    { 0xc0df5dff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71f5e0b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cef5e0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x63af5e12, 93 },
    { 0x21df5e13, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aff5e18, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6f5e1c, 33 },
    { 0, 0 },
    { 0x353f5e1e, 95 },
    { 0x45df5e1f, 55 },
    { 0xe5df5e20, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bf5e25, 23 },
    { 0xfe4f5e25, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f5e2a, 55 },
    { 0xebdf5e2b, 68 },
    { 0, 0 },
    { 0x4d8f5e2d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f5e34, 76 },
    { 0, 0 },
    { 0xe25f5e36, 55 },
    { 0, 0 },
    { 0x8ff5e38, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6af5e40, 95 },
    { 0, 0 },
    { 0xf5df5e42, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5f5e46, 84 },
    { 0x7d7f5e46, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96f5e4b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6faf5e4f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x741f5e56, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf55f5e59, 84 },
    { 0x3ccf5e5a, 61 },
    { 0x7c5f5e5b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bf5e60, 95 },
    { 0x9c3f5e61, 95 },
    { 0, 0 },
    { 0x885f5e63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfddf5e6a, 99 },
    { 0x99f5e6b, 49 },
    { 0x1acf5e6c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa11f5e7b, 33 },
    { 0xaf3f5e7c, 55 },
    { 0xc07f5e7c, 95 },
    { 0x650f5e7e, 23 },
    { 0xef8f5e7f, 76 },
    { 0, 0 },
    { 0x8a7f5e81, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aff5e8e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948f5e94, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39bf5e99, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5f5e9c, 33 },
    { 0xb9cf5e9c, 99 },
    { 0x9dbf5e9e, 77 },
    { 0x6baf5e9e, 56 },
    { 0x7c0f5ea0, 23 },
    { 0x98f5ea1, 76 },
    { 0xbc4f5ea2, 61 },
    { 0x317f5ea2, 90 },
    { 0x502f5ea4, 90 },
    { 0x23f5ea4, 33 },
    { 0x8ef5e9c, 81 },
    { 0, 0 },
    { 0x9b2f5ea8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90f5eae, 90 },
    { 0x1cbf5eaf, 61 },
    { 0, 0 },
    { 0xcf6f5eb1, 87 },
    { 0, 0 },
    { 0x564f5eb3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bf5eb8, 61 },
    { 0x389f5eb8, 99 },
    { 0x73af5eba, 55 },
    { 0x987f5ebb, 90 },
    { 0xbe8f5ebc, 90 },
    { 0x216f5eb8, 46 },
    { 0x3e4f5ebe, 90 },
    { 0x88df5ebd, 43 },
    { 0x7a0f5ec0, 34 },
    { 0x8cff5ec1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccef5ec8, 76 },
    { 0x2e0f5ec8, 90 },
    { 0, 0 },
    { 0x458f5ecb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20f5ed2, 43 },
    { 0, 0 },
    { 0x207f5ed4, 69 },
    { 0x122f5ed5, 55 },
    { 0x2ef5ed5, 95 },
    { 0x665f5ed7, 61 },
    { 0x7baf5ed8, 90 },
    { 0x346f5ed8, 43 },
    { 0xb52f5eda, 91 },
    { 0xf9ef5edb, 33 },
    { 0, 0 },
    { 0x8a9f5edd, 33 },
    { 0, 0 },
    { 0x12bf5edf, 90 },
    { 0x3a8f5ee0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccf5ee3, 95 },
    { 0x24bf5ee3, 43 },
    { 0xa46f5ee4, 91 },
    { 0, 0 },
    { 0x168f5ee7, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cf5eea, 97 },
    { 0x1e0f5eeb, 95 },
    { 0xf66f5eea, 23 },
    { 0, 0 },
    { 0x56cf5eee, 88 },
    { 0x8c7f5eef, 33 },
    { 0xf06f5ef0, 95 },
    { 0x452f5ef0, 33 },
    { 0x5cdf5ef2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1f5ef5, 98 },
    { 0, 0 },
    { 0x52af5ef7, 33 },
    { 0x63df5ef8, 23 },
    { 0x219f5ef9, 84 },
    { 0x16df5efa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74f5efd, 23 },
    { 0x940f5efd, 81 },
    { 0, 0 },
    { 0x16df5f00, 56 },
    { 0xf96f5f01, 98 },
    { 0x9abf5f02, 55 },
    { 0xd7cf5f01, 98 },
    { 0x3aaf5f00, 46 },
    { 0x6c2f5f05, 77 },
    { 0, 0 },
    { 0x3f2f5f07, 51 },
    { 0x53af5f08, 98 },
    { 0xb68f5f09, 23 },
    { 0, 0 },
    { 0x986f5f0b, 90 },
    { 0, 0 },
    { 0xefbf5f0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f5f11, 99 },
    { 0x44f5f12, 17 },
    { 0x3f2f5f13, 33 },
    { 0xbdf5f13, 76 },
    { 0x73af5f15, 93 },
    { 0x6c9f5f16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cf5f23, 87 },
    { 0, 0 },
    { 0x573f5f25, 90 },
    { 0x7abf5f25, 43 },
    { 0, 0 },
    { 0xa37f5f28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdf5f2b, 33 },
    { 0xc9af5f2c, 99 },
    { 0, 0 },
    { 0x366f5f2e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3f5f32, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf5f36, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ef5f42, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b4f5f45, 33 },
    { 0xceff5f46, 33 },
    { 0, 0 },
    { 0x6ddf5f48, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x230f5f4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf5f4e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x394f5f51, 85 },
    { 0x882f5f52, 66 },
    { 0x5a3f5f52, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c2f5f56, 90 },
    { 0, 0 },
    { 0xb95f5f58, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f4f5f62, 23 },
    { 0, 0 },
    { 0x72df5f64, 81 },
    { 0, 0 },
    { 0x9b8f5f66, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8f5f6a, 98 },
    { 0x73f5f6b, 95 },
    { 0x42bf5f6c, 34 },
    { 0x119f5f6d, 17 },
    { 0x38df5f6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d3f5f71, 88 },
    { 0x6e1f5f71, 90 },
    { 0x55ef5f72, 87 },
    { 0, 0 },
    { 0x786f5f75, 90 },
    { 0x61af5f76, 95 },
    { 0x773f5f76, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x123f5f83, 61 },
    { 0x170f5f84, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x206f5f89, 65 },
    { 0x188f5f8a, 78 },
    { 0xc39f5f8b, 52 },
    { 0x5eff5f8a, 90 },
    { 0x67ef5f8c, 87 },
    { 0xdff5f8e, 98 },
    { 0xcbbf5f8e, 23 },
    { 0x49ff5f8f, 76 },
    { 0x661f5f90, 67 },
    { 0xa58f5f92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef5f9a, 36 },
    { 0x3d0f5f9b, 46 },
    { 0xd10f5f9a, 84 },
    { 0x9dbf5f9a, 91 },
    { 0, 0 },
    { 0x86f5f9f, 66 },
    { 0xe19f5fa0, 98 },
    { 0x720f5fa1, 23 },
    { 0xbdaf5fa1, 23 },
    { 0x70cf5fa3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x698f5fa8, 90 },
    { 0xec5f5fa8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3f5faf, 55 },
    { 0xf6ef5faf, 76 },
    { 0, 0 },
    { 0x30cf5fb2, 23 },
    { 0xca9f5fb2, 84 },
    { 0, 0 },
    { 0x276f5fb5, 33 },
    { 0, 0 },
    { 0x63bf5fb7, 61 },
    { 0xf66f5fb8, 55 },
    { 0x722f5fb9, 61 },
    { 0xb7f5fba, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4f5fc0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaf5fc3, 33 },
    { 0, 0 },
    { 0x6bef5fc5, 93 },
    { 0, 0 },
    { 0x7f7f5fc7, 33 },
    { 0x8bbf5fc8, 98 },
    { 0xe65f5fc9, 33 },
    { 0x977f5fc8, 50 },
    { 0xd88f5fc9, 55 },
    { 0xbacf5fcc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411f5fd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f8f5fd5, 76 },
    { 0xb53f5fd5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08f5fd9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x684f5fe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x985f5fe3, 33 },
    { 0, 0 },
    { 0x713f5fe5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8f5fea, 93 },
    { 0xb60f5fea, 61 },
    { 0x6af5fec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x543f5fef, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2f5ff2, 75 },
    { 0xc52f5ff3, 34 },
    { 0xbbcf5ff4, 99 },
    { 0xc07f5ff4, 34 },
    { 0x5ef5ff6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2ff5ffa, 95 },
    { 0x235f5ffb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5af5fff, 81 },
    { 0, 0 },
    { 0xa15f6001, 61 },
    { 0x3e9f6002, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f6007, 46 },
    { 0x91ef6008, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cff600e, 100 },
    { 0xeb9f600f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f6014, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931f6018, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e6f601c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926f6026, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x29cf6029, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acf602c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f6033, 90 },
    { 0x2e3f6034, 90 },
    { 0x444f6035, 23 },
    { 0x8fff6035, 61 },
    { 0xebff6037, 90 },
    { 0x8f1f6037, 55 },
    { 0xa7df6034, 76 },
    { 0x1cbf603a, 91 },
    { 0, 0 },
    { 0x56af603c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360f6042, 33 },
    { 0xd1f6042, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08f6046, 94 },
    { 0xc30f6046, 95 },
    { 0, 0 },
    { 0x1ebf6049, 61 },
    { 0x734f604a, 90 },
    { 0x610f604a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x799f6056, 55 },
    { 0xb64f6056, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9df605a, 90 },
    { 0x85df605b, 90 },
    { 0xddcf605c, 93 },
    { 0xd2df605a, 93 },
    { 0x588f605c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa51f6062, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f6066, 40 },
    { 0, 0 },
    { 0x7af6068, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f606b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250f6070, 90 },
    { 0x137f6070, 95 },
    { 0, 0 },
    { 0xb28f6073, 68 },
    { 0xca5f6073, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacf6077, 55 },
    { 0x309f6077, 98 },
    { 0x2d1f6077, 65 },
    { 0xbadf607a, 33 },
    { 0x4cbf607b, 86 },
    { 0x512f607c, 23 },
    { 0, 0 },
    { 0x20ff607e, 52 },
    { 0x5eef607f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x201f6084, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf8f6088, 91 },
    { 0xe6df6089, 95 },
    { 0xc53f608a, 46 },
    { 0x8dbf608b, 90 },
    { 0x924f608c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa16f6091, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6df6095, 77 },
    { 0x846f6096, 23 },
    { 0, 0 },
    { 0x286f6098, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f609c, 95 },
    { 0xd28f609c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0f60a4, 46 },
    { 0x544f60a5, 87 },
    { 0x8a0f60a5, 61 },
    { 0xd2df60a7, 84 },
    { 0xcaff60a8, 98 },
    { 0xad7f60a9, 77 },
    { 0xb2bf60a9, 98 },
    { 0x1cbf60a9, 78 },
    { 0xa5bf60aa, 90 },
    { 0x591f60ad, 33 },
    { 0xd3af60ab, 98 },
    { 0x4b1f60a5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x868f60b7, 61 },
    { 0xe71f60b8, 87 },
    { 0, 0 },
    { 0x60df60ba, 98 },
    { 0xc5bf60bb, 84 },
    { 0x447f60bc, 90 },
    { 0x528f60bb, 61 },
    { 0x181f60be, 24 },
    { 0xe55f60ba, 95 },
    { 0x306f60bc, 43 },
    { 0x5f1f60c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf93f60c8, 61 },
    { 0xb80f60c9, 53 },
    { 0, 0 },
    { 0xa8ff60cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f60d3, 90 },
    { 0, 0 },
    { 0x833f60d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6caf60d8, 91 },
    { 0xd10f60d8, 100 },
    { 0, 0 },
    { 0x711f60db, 95 },
    { 0x8e9f60dc, 61 },
    { 0xecaf60dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bf60e0, 76 },
    { 0xa62f60e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59af60e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f60ec, 55 },
    { 0x304f60ed, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a2f60f1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bf60fa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8f60fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f8f6101, 43 },
    { 0, 0 },
    { 0x85ff6103, 33 },
    { 0x80bf6104, 61 },
    { 0xa8cf6105, 87 },
    { 0x261f6105, 46 },
    { 0x7e2f6106, 61 },
    { 0x5ef6108, 77 },
    { 0x89ff6109, 61 },
    { 0x227f6107, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2f610e, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x342f6114, 98 },
    { 0xf3df6115, 78 },
    { 0x1c0f6114, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cf611a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaf611e, 23 },
    { 0x7c8f611f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acf6122, 34 },
    { 0, 0 },
    { 0xc80f6124, 55 },
    { 0x910f6124, 90 },
    { 0, 0 },
    { 0x26ef6127, 97 },
    { 0x7f7f6127, 34 },
    { 0x353f6129, 33 },
    { 0x4c9f6129, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x516f612d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf6131, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2af6138, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff613d, 23 },
    { 0xc8bf613d, 88 },
    { 0xd6bf613e, 55 },
    { 0, 0 },
    { 0x5aaf6141, 95 },
    { 0, 0 },
    { 0x84bf6143, 55 },
    { 0x907f6144, 78 },
    { 0xc5ff6144, 23 },
    { 0, 0 },
    { 0x204f6147, 43 },
    { 0, 0 },
    { 0x834f6149, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8f614c, 61 },
    { 0, 0 },
    { 0x11bf614e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x473f6156, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x292f6159, 81 },
    { 0xa04f615a, 61 },
    { 0, 0 },
    { 0xc7f615c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f6161, 33 },
    { 0x56cf6162, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68f616d, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95f6170, 33 },
    { 0xb9cf6171, 43 },
    { 0x83df6172, 61 },
    { 0xf2ff6173, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xccef6176, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579f617b, 93 },
    { 0, 0 },
    { 0xa97f617d, 99 },
    { 0xb8bf617e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f6184, 100 },
    { 0x326f6185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aef6190, 99 },
    { 0xb54f6191, 61 },
    { 0xf44f6192, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212f6196, 95 },
    { 0, 0 },
    { 0x43df6198, 98 },
    { 0, 0 },
    { 0xe49f619a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41f61a3, 46 },
    { 0x5f9f61a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2ff61ab, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d5f61b0, 33 },
    { 0x5dbf61b1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5f61b4, 76 },
    { 0, 0 },
    { 0x243f61b6, 55 },
    { 0x3d0f61b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3df61ba, 93 },
    { 0xa3bf61bb, 34 },
    { 0x633f61bc, 33 },
    { 0, 0 },
    { 0xebaf61be, 87 },
    { 0, 0 },
    { 0x129f61c0, 90 },
    { 0x367f61c1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x456f61cb, 90 },
    { 0, 0 },
    { 0xd72f61cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef61d0, 55 },
    { 0x714f61d1, 23 },
    { 0x7f7f61d0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99f61d6, 99 },
    { 0x518f61d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f61da, 55 },
    { 0x3bdf61db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc5f61e8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cf61eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393f61f4, 50 },
    { 0x4d8f61f4, 46 },
    { 0, 0 },
    { 0xeaf61f7, 61 },
    { 0, 0 },
    { 0xd1ff61f9, 95 },
    { 0x554f61fa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4f61fd, 84 },
    { 0, 0 },
    { 0x480f61ff, 100 },
    { 0x6bbf6200, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecf6203, 33 },
    { 0xc01f6204, 43 },
    { 0x3b8f6205, 68 },
    { 0x6f6204, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552f620e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67f6214, 76 },
    { 0x8a7f6214, 66 },
    { 0, 0 },
    { 0x128f6217, 98 },
    { 0x72cf6218, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25df621c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff621f, 61 },
    { 0x7c7f6220, 85 },
    { 0, 0 },
    { 0xe0bf6222, 76 },
    { 0, 0 },
    { 0xe81f6224, 95 },
    { 0xf37f6225, 33 },
    { 0, 0 },
    { 0x463f6227, 90 },
    { 0xa04f6227, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ebf622b, 84 },
    { 0x2b5f622c, 78 },
    { 0xe21f622d, 61 },
    { 0x1dff622e, 85 },
    { 0xe1ef622f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdf6234, 98 },
    { 0xe78f6235, 35 },
    { 0xbfcf6236, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e2f623b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x168f623e, 43 },
    { 0x352f623f, 91 },
    { 0xb17f6240, 76 },
    { 0x5aef6240, 90 },
    { 0xccff6242, 43 },
    { 0x1a4f623e, 46 },
    { 0x960f6244, 95 },
    { 0x4cdf6241, 43 },
    { 0x64bf6245, 88 },
    { 0x3e0f6247, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x226f624a, 52 },
    { 0x36cf624b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6f6255, 90 },
    { 0, 0 },
    { 0x323f6257, 55 },
    { 0xc67f6257, 90 },
    { 0x59ef6259, 34 },
    { 0x7d7f625a, 99 },
    { 0xb47f625b, 33 },
    { 0, 0 },
    { 0x201f625d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedf6263, 66 },
    { 0x916f6263, 34 },
    { 0x9a2f6263, 91 },
    { 0xcf9f6266, 76 },
    { 0x203f6267, 61 },
    { 0x806f6264, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa35f626c, 90 },
    { 0x5f4f626c, 90 },
    { 0, 0 },
    { 0xd81f626f, 93 },
    { 0, 0 },
    { 0x837f6271, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcbf6277, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x54af627a, 61 },
    { 0x968f627b, 23 },
    { 0x626f627b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f6282, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8df6286, 98 },
    { 0x4daf6286, 34 },
    { 0xb6f6287, 88 },
    { 0xc5cf6288, 61 },
    { 0xa79f6289, 34 },
    { 0, 0 },
    { 0x63f628c, 98 },
    { 0x7d3f628d, 23 },
    { 0x1e8f628e, 98 },
    { 0x2b5f628e, 87 },
    { 0x88ff6290, 100 },
    { 0x323f6291, 44 },
    { 0, 0 },
    { 0x88df6293, 90 },
    { 0x1dbf6294, 55 },
    { 0xf3df6294, 81 },
    { 0xcb8f6296, 90 },
    { 0x736f6297, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f629a, 90 },
    { 0, 0 },
    { 0xe65f629c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f62a7, 95 },
    { 0xb30f62a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacf62b2, 23 },
    { 0x85ef62b2, 90 },
    { 0x97bf62b4, 76 },
    { 0xd1cf62b5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3f62b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdff62c0, 15 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559f62c4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6def62cb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb4f62ce, 95 },
    { 0x472f62cf, 95 },
    { 0x47df62d0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626f62d4, 95 },
    { 0x8ccf62d5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf62da, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edf62dd, 23 },
    { 0xb44f62dd, 95 },
    { 0xc15f62de, 55 },
    { 0xe95f62e0, 91 },
    { 0xf6ef62dd, 55 },
    { 0x4bff62de, 43 },
    { 0x1c6f62de, 52 },
    { 0x90af62e4, 46 },
    { 0xf54f62e5, 23 },
    { 0x94df62e4, 95 },
    { 0x33df62e5, 98 },
    { 0x6b9f62e5, 17 },
    { 0, 0 },
    { 0x493f62ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x372f62ef, 61 },
    { 0xadf62ef, 85 },
    { 0x232f62f1, 33 },
    { 0x698f62f1, 43 },
    { 0xb37f62f1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ff62f6, 98 },
    { 0, 0 },
    { 0xd9cf62f8, 81 },
    { 0x479f62f8, 55 },
    { 0xa7ff62fa, 90 },
    { 0x898f62fa, 95 },
    { 0x3a8f62fa, 95 },
    { 0, 0 },
    { 0x161f62fe, 67 },
    { 0, 0 },
    { 0xbaaf6300, 33 },
    { 0, 0 },
    { 0xfbef6302, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32f6307, 66 },
    { 0, 0 },
    { 0x125f6309, 95 },
    { 0x4f7f630a, 43 },
    { 0, 0 },
    { 0xeadf630c, 93 },
    { 0xb40f630d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1f6310, 61 },
    { 0, 0 },
    { 0xfc3f6312, 33 },
    { 0, 0 },
    { 0x292f6314, 55 },
    { 0, 0 },
    { 0x6f4f6316, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cf631b, 61 },
    { 0x4cff631c, 93 },
    { 0, 0 },
    { 0x342f631e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eef6323, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1f6326, 76 },
    { 0, 0 },
    { 0xddbf6328, 61 },
    { 0x1e4f6329, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3f632d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfff6334, 98 },
    { 0x272f6334, 90 },
    { 0, 0 },
    { 0x41ff6337, 52 },
    { 0x88cf6338, 33 },
    { 0x573f6337, 23 },
    { 0, 0 },
    { 0x310f633b, 76 },
    { 0x4bcf633c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecf6340, 34 },
    { 0xb87f6341, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xff2f6344, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8f6348, 26 },
    { 0xca1f6349, 90 },
    { 0xd0ef634a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98f6351, 91 },
    { 0, 0 },
    { 0xaff6353, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54f6358, 69 },
    { 0, 0 },
    { 0xe88f635a, 76 },
    { 0, 0 },
    { 0x5bef635c, 76 },
    { 0x98cf635d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ef6360, 95 },
    { 0, 0 },
    { 0x56cf6362, 87 },
    { 0x4bf6363, 93 },
    { 0, 0 },
    { 0x6c1f6365, 34 },
    { 0, 0 },
    { 0x6abf6367, 93 },
    { 0, 0 },
    { 0x5fef6369, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621f636f, 43 },
    { 0, 0 },
    { 0xb19f6371, 17 },
    { 0x76af6372, 99 },
    { 0, 0 },
    { 0x462f6374, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82df6377, 90 },
    { 0, 0 },
    { 0x5e4f6379, 33 },
    { 0x308f6379, 61 },
    { 0, 0 },
    { 0xa74f637c, 61 },
    { 0, 0 },
    { 0x80af637e, 61 },
    { 0x2c1f637e, 33 },
    { 0x1b0f6380, 90 },
    { 0xbe0f637f, 55 },
    { 0, 0 },
    { 0x699f6383, 52 },
    { 0, 0 },
    { 0xefef6385, 87 },
    { 0xbe8f6386, 33 },
    { 0x2b9f6385, 46 },
    { 0x5bcf6388, 78 },
    { 0xc24f6388, 33 },
    { 0, 0 },
    { 0xf40f638b, 99 },
    { 0, 0 },
    { 0xf04f638d, 23 },
    { 0x692f638d, 34 },
    { 0xe3cf638e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16df6393, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aef639a, 66 },
    { 0, 0 },
    { 0x592f639c, 98 },
    { 0xd03f639c, 34 },
    { 0, 0 },
    { 0xe83f639f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f63a3, 76 },
    { 0, 0 },
    { 0x599f63a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969f63a9, 61 },
    { 0x96cf63aa, 85 },
    { 0x5e8f63a9, 93 },
    { 0x6dcf63aa, 90 },
    { 0x4e7f63aa, 33 },
    { 0x946f63ae, 46 },
    { 0x793f63ae, 46 },
    { 0x5baf63b0, 47 },
    { 0x9ccf63b1, 33 },
    { 0x4fff63b1, 98 },
    { 0, 0 },
    { 0x9af63b4, 81 },
    { 0, 0 },
    { 0x339f63b6, 67 },
    { 0xd38f63b6, 78 },
    { 0x5a6f63b8, 98 },
    { 0x7b9f63b8, 33 },
    { 0x658f63ba, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61f63be, 90 },
    { 0x201f63bf, 78 },
    { 0xd1ef63c0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9f63c4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05f63c7, 61 },
    { 0, 0 },
    { 0xa95f63c9, 95 },
    { 0xe26f63ca, 90 },
    { 0, 0 },
    { 0x2cf63cc, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8f63d1, 52 },
    { 0xbf0f63d2, 33 },
    { 0x659f63d1, 43 },
    { 0, 0 },
    { 0x5edf63d5, 91 },
    { 0, 0 },
    { 0x120f63d7, 90 },
    { 0x4df63d8, 61 },
    { 0x8baf63d9, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74f63dd, 87 },
    { 0xb1df63de, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bf63e5, 23 },
    { 0x912f63e6, 93 },
    { 0, 0 },
    { 0x89bf63e8, 90 },
    { 0x60f63e9, 90 },
    { 0x60cf63ea, 99 },
    { 0, 0 },
    { 0x1fef63ec, 65 },
    { 0x67ef63ed, 90 },
    { 0, 0 },
    { 0x63cf63ef, 33 },
    { 0xaadf63f0, 95 },
    { 0x509f63f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2f63f4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4f63f9, 90 },
    { 0, 0 },
    { 0x13ef63fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x128f640f, 34 },
    { 0xcd0f6410, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1f641d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18f6423, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2caf6427, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cf642f, 90 },
    { 0x4c9f6430, 34 },
    { 0xe8df6430, 33 },
    { 0x491f6431, 23 },
    { 0x48f6433, 90 },
    { 0x784f6434, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cf6439, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x48f643c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedf6440, 91 },
    { 0, 0 },
    { 0x1fcf6442, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfef644b, 78 },
    { 0x2daf644c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645f6450, 34 },
    { 0x818f6451, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd67f6455, 98 },
    { 0x26bf6456, 85 },
    { 0x290f6457, 46 },
    { 0x4d1f6458, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50f645c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf645f, 98 },
    { 0, 0 },
    { 0xe4f6461, 65 },
    { 0, 0 },
    { 0x287f6463, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafbf6469, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafff646d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d0f6470, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0f6474, 66 },
    { 0xc75f6474, 90 },
    { 0, 0 },
    { 0xe45f6477, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8f647a, 90 },
    { 0xf0ef647b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cf6480, 23 },
    { 0xd9ef6481, 90 },
    { 0x325f6480, 55 },
    { 0x85f6483, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ef6486, 78 },
    { 0x810f6486, 46 },
    { 0x983f6486, 93 },
    { 0x3cef6489, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40f648d, 91 },
    { 0x16f648e, 95 },
    { 0x8e3f648e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaf6497, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x733f649a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef64a7, 76 },
    { 0x226f64a7, 90 },
    { 0, 0 },
    { 0x76ff64aa, 33 },
    { 0x1ddf64ab, 61 },
    { 0x7ebf64ab, 23 },
    { 0x92af64ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58f64b1, 61 },
    { 0xd36f64b1, 84 },
    { 0, 0 },
    { 0x28ff64b4, 46 },
    { 0, 0 },
    { 0x867f64b6, 61 },
    { 0x245f64b7, 46 },
    { 0xb2df64b8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebf64c1, 90 },
    { 0x82df64c2, 99 },
    { 0, 0 },
    { 0xdfff64c4, 34 },
    { 0x2c4f64c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf64ca, 87 },
    { 0, 0 },
    { 0xdaff64cc, 66 },
    { 0x744f64cc, 52 },
    { 0x404f64ce, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996f64d2, 43 },
    { 0x3c9f64d2, 90 },
    { 0xf7cf64d4, 77 },
    { 0xc83f64d4, 34 },
    { 0x43df64d6, 61 },
    { 0, 0 },
    { 0x5d9f64d8, 65 },
    { 0x5fbf64d9, 76 },
    { 0xb07f64da, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f64ed, 43 },
    { 0xaa0f64ee, 43 },
    { 0x72df64ef, 75 },
    { 0xb54f64ee, 46 },
    { 0xacaf64f1, 85 },
    { 0x15af64f1, 98 },
    { 0, 0 },
    { 0x460f64f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a1f64fa, 95 },
    { 0x40df64fb, 83 },
    { 0, 0 },
    { 0x5b1f64fd, 23 },
    { 0xc0bf64fe, 23 },
    { 0xc83f64fe, 46 },
    { 0x99f6500, 90 },
    { 0x587f6501, 52 },
    { 0x7d2f6502, 76 },
    { 0x8baf6503, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199f6509, 33 },
    { 0x847f6509, 33 },
    { 0xc11f650b, 33 },
    { 0, 0 },
    { 0x22f650d, 90 },
    { 0x9e3f650d, 61 },
    { 0x709f650d, 88 },
    { 0xb38f650e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5cf6513, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3f6516, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3f6519, 95 },
    { 0, 0 },
    { 0x66af651b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10f6522, 34 },
    { 0x58ff6523, 55 },
    { 0x4a8f6524, 61 },
    { 0xb37f6525, 53 },
    { 0x21cf6526, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf6529, 96 },
    { 0xbdcf652a, 33 },
    { 0xfb1f652b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c3f6534, 76 },
    { 0xff6f6534, 95 },
    { 0, 0 },
    { 0x8f6537, 61 },
    { 0xb2df6538, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x503f653b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f653e, 95 },
    { 0, 0 },
    { 0x248f6540, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9f6545, 61 },
    { 0xa3ef6546, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4f654f, 43 },
    { 0xcd9f6550, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0f6553, 91 },
    { 0x357f6554, 55 },
    { 0xad0f6555, 61 },
    { 0x844f6556, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f6559, 95 },
    { 0, 0 },
    { 0xeecf655b, 95 },
    { 0xeb9f655c, 87 },
    { 0x3c7f655c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51f6560, 100 },
    { 0, 0 },
    { 0x7fff6562, 33 },
    { 0xb6df6563, 61 },
    { 0x93bf6564, 78 },
    { 0xa13f6563, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef6568, 100 },
    { 0x44ef6568, 61 },
    { 0, 0 },
    { 0x24ef656b, 95 },
    { 0x3cbf656b, 19 },
    { 0x937f656d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27bf6572, 23 },
    { 0x6d0f6573, 76 },
    { 0x734f6574, 68 },
    { 0xcb4f6573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6f6579, 90 },
    { 0xa34f657a, 40 },
    { 0xc84f657b, 90 },
    { 0x428f657c, 85 },
    { 0x4d8f657d, 67 },
    { 0, 0 },
    { 0xf65f657f, 52 },
    { 0xca6f6580, 33 },
    { 0x515f6580, 61 },
    { 0x176f6582, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45af6586, 46 },
    { 0xfd8f6587, 90 },
    { 0xb2ff6587, 46 },
    { 0xbaaf6588, 77 },
    { 0xecaf6589, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf57f6593, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x956f6597, 33 },
    { 0, 0 },
    { 0x248f6599, 95 },
    { 0x2d7f6599, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecf659e, 76 },
    { 0x52af659f, 23 },
    { 0x2c6f65a0, 90 },
    { 0xb11f65a0, 61 },
    { 0xbc1f65a2, 98 },
    { 0xcb6f659f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e1f65a6, 91 },
    { 0x31ef65a7, 93 },
    { 0x935f65a8, 33 },
    { 0, 0 },
    { 0xe7df65aa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834f65b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408f65b4, 92 },
    { 0x8a2f65b4, 76 },
    { 0x449f65b5, 98 },
    { 0xd73f65b7, 59 },
    { 0xf22f65b6, 61 },
    { 0, 0 },
    { 0xf65f65ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x792f65c6, 33 },
    { 0x22af65c6, 34 },
    { 0x67df65c7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f65cd, 90 },
    { 0x272f65ce, 78 },
    { 0x3cdf65cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x489f65d4, 7 },
    { 0, 0 },
    { 0xe09f65d6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdf65d9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15f65dc, 85 },
    { 0xddf65dd, 33 },
    { 0x420f65de, 90 },
    { 0x341f65df, 90 },
    { 0, 0 },
    { 0xd68f65e1, 33 },
    { 0, 0 },
    { 0x9a8f65e3, 61 },
    { 0, 0 },
    { 0xf3bf65e5, 95 },
    { 0, 0 },
    { 0xe08f65e7, 61 },
    { 0xb5df65e8, 37 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bff65ed, 61 },
    { 0x9bf65ee, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5f65f4, 88 },
    { 0x5b0f65f5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x759f65f9, 87 },
    { 0xbef65fa, 98 },
    { 0x74ff65fa, 34 },
    { 0, 0 },
    { 0x883f65fd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92f6603, 33 },
    { 0, 0 },
    { 0x8a5f6605, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ef660a, 93 },
    { 0, 0 },
    { 0xa86f660c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xad7f660f, 95 },
    { 0xea0f6610, 93 },
    { 0x1c0f6610, 34 },
    { 0x72ff6610, 95 },
    { 0xc0f6610, 98 },
    { 0x329f660f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4f6618, 24 },
    { 0, 0 },
    { 0xf26f661a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x546f661d, 76 },
    { 0xf6f661e, 98 },
    { 0xe95f661e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x162f6625, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x354f6628, 61 },
    { 0x56bf6629, 61 },
    { 0x14f662a, 23 },
    { 0xeaef6629, 61 },
    { 0x752f662b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x199f662f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf09f6635, 95 },
    { 0xcc7f6636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ff663c, 90 },
    { 0xc22f663c, 34 },
    { 0, 0 },
    { 0x504f663f, 78 },
    { 0xc67f6640, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261f6648, 90 },
    { 0x3bcf6649, 61 },
    { 0x573f6649, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeef664f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30f6652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0f6656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adf6659, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4f665d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f9f6666, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378f666d, 52 },
    { 0x918f666d, 90 },
    { 0, 0 },
    { 0x2edf6670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6675, 17 },
    { 0x12cf6676, 90 },
    { 0x79df6677, 100 },
    { 0xd9cf6678, 87 },
    { 0x330f6678, 93 },
    { 0x194f667a, 91 },
    { 0x4d7f667b, 76 },
    { 0xf93f667a, 90 },
    { 0, 0 },
    { 0x220f667e, 90 },
    { 0x903f667f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a6f6684, 95 },
    { 0x23ef6684, 43 },
    { 0x8f3f6686, 98 },
    { 0xc8bf6687, 90 },
    { 0, 0 },
    { 0x104f6689, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccf668d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1f6690, 95 },
    { 0, 0 },
    { 0xb4cf6692, 78 },
    { 0x553f6692, 90 },
    { 0x682f6694, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd11f6697, 78 },
    { 0, 0 },
    { 0x152f6699, 23 },
    { 0, 0 },
    { 0xde0f669b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x20df669e, 33 },
    { 0, 0 },
    { 0x5d6f66a0, 95 },
    { 0xf70f66a1, 55 },
    { 0x1f66a2, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05f66a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcf66ad, 66 },
    { 0x585f66ae, 90 },
    { 0x32af66ad, 98 },
    { 0x9f3f66b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8f66b5, 55 },
    { 0, 0 },
    { 0x986f66b7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x150f66ba, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cf66be, 48 },
    { 0, 0 },
    { 0xc55f66c0, 23 },
    { 0xf97f66c0, 33 },
    { 0x43df66c0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03f66c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f66ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269f66d2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63f66d6, 98 },
    { 0x8d8f66d7, 99 },
    { 0xaa5f66d6, 55 },
    { 0x242f66d7, 98 },
    { 0x75f66d6, 23 },
    { 0x115f66db, 33 },
    { 0x606f66dc, 43 },
    { 0x5c7f66da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7f66e1, 23 },
    { 0xf82f66e1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d7f66e7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ef66ea, 33 },
    { 0xf81f66ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65f66ee, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x244f66f1, 98 },
    { 0, 0 },
    { 0xff2f66f3, 61 },
    { 0x19ef66f4, 91 },
    { 0x41ef66f5, 90 },
    { 0, 0 },
    { 0x76df66f7, 68 },
    { 0xd0af66f7, 90 },
    { 0xd67f66f7, 23 },
    { 0x88af66f7, 46 },
    { 0x8f6f66fa, 61 },
    { 0, 0 },
    { 0xad5f66fd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7af6703, 23 },
    { 0xa06f6704, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495f6708, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ff670d, 87 },
    { 0xf15f670e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf6712, 90 },
    { 0x2e4f6713, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x890f6718, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f671e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19ff6722, 34 },
    { 0x963f6723, 87 },
    { 0xb40f6723, 95 },
    { 0x5af6725, 55 },
    { 0xe1bf6722, 84 },
    { 0, 0 },
    { 0x652f6728, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8f672d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0f6730, 33 },
    { 0xa0df6730, 98 },
    { 0x4d7f6731, 43 },
    { 0x49bf6733, 95 },
    { 0, 0 },
    { 0x7dbf6735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bf673e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590f6744, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaf674a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d8f6753, 90 },
    { 0, 0 },
    { 0x4d1f6755, 38 },
    { 0x192f6756, 91 },
    { 0x2d1f6755, 61 },
    { 0x282f6758, 23 },
    { 0, 0 },
    { 0x11ef675a, 33 },
    { 0x953f675a, 59 },
    { 0, 0 },
    { 0x7a8f675d, 23 },
    { 0x95f675d, 98 },
    { 0x514f675d, 90 },
    { 0x5a8f6760, 66 },
    { 0x66ef6761, 61 },
    { 0x8f6762, 95 },
    { 0, 0 },
    { 0xf2bf6764, 76 },
    { 0, 0 },
    { 0x38cf6766, 61 },
    { 0xb6cf6767, 23 },
    { 0, 0 },
    { 0x95cf6769, 99 },
    { 0x3ff676a, 61 },
    { 0x232f6769, 95 },
    { 0x242f676c, 61 },
    { 0x1daf676c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0f6770, 55 },
    { 0xf38f6771, 90 },
    { 0x768f6770, 43 },
    { 0x771f6772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f6779, 90 },
    { 0x7b5f677a, 100 },
    { 0xee9f677b, 33 },
    { 0, 0 },
    { 0xd48f677d, 33 },
    { 0x973f677e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26f6781, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff6788, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d4f678b, 76 },
    { 0, 0 },
    { 0x48f678d, 23 },
    { 0xf9ef678e, 100 },
    { 0, 0 },
    { 0xa25f6790, 33 },
    { 0xa1df6790, 95 },
    { 0, 0 },
    { 0x23af6793, 76 },
    { 0, 0 },
    { 0x298f6795, 23 },
    { 0, 0 },
    { 0xf8ff6797, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65df679f, 90 },
    { 0x13df679f, 34 },
    { 0x6c8f67a1, 34 },
    { 0x317f67a2, 23 },
    { 0, 0 },
    { 0x4e2f67a4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d7f67a7, 17 },
    { 0x7b0f67a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6f67ac, 90 },
    { 0x519f67ad, 90 },
    { 0x2b5f67ae, 61 },
    { 0x15af67af, 100 },
    { 0x357f67af, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf67b8, 43 },
    { 0xdb5f67b9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ff67c1, 61 },
    { 0, 0 },
    { 0xcf1f67c3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5f67c6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32bf67ca, 34 },
    { 0x25df67cb, 61 },
    { 0xb22f67cc, 90 },
    { 0xcdaf67cd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x751f67d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91cf67d6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885f67da, 61 },
    { 0x704f67db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5bf67de, 40 },
    { 0xf87f67df, 55 },
    { 0xedf67de, 43 },
    { 0x80af67e0, 83 },
    { 0x3ebf67e1, 61 },
    { 0, 0 },
    { 0xf40f67e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1f67e8, 83 },
    { 0, 0 },
    { 0xf00f67ea, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0f67ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb34f67f1, 76 },
    { 0x36bf67f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x649f67f6, 93 },
    { 0x5c0f67f7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef67fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3f6800, 48 },
    { 0, 0 },
    { 0x566f6802, 73 },
    { 0x744f6803, 33 },
    { 0x784f6803, 90 },
    { 0x82cf6804, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964f680a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x525f680d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477f6814, 95 },
    { 0, 0 },
    { 0x777f6816, 61 },
    { 0xbc8f6816, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baf6826, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f682b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a5f682f, 77 },
    { 0, 0 },
    { 0x533f6831, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaef683f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ef6848, 76 },
    { 0x13bf6849, 90 },
    { 0x9def6849, 55 },
    { 0xc5f684b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeccf684e, 55 },
    { 0x320f684f, 90 },
    { 0x95bf684f, 94 },
    { 0, 0 },
    { 0x239f6852, 43 },
    { 0x1c0f6852, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe6f685d, 98 },
    { 0, 0 },
    { 0xaf5f685f, 88 },
    { 0x170f6860, 76 },
    { 0xdccf6861, 46 },
    { 0xd45f6862, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99f6866, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb76f686e, 90 },
    { 0x162f686f, 34 },
    { 0, 0 },
    { 0xafef6871, 23 },
    { 0, 0 },
    { 0x725f6873, 23 },
    { 0, 0 },
    { 0xfcff6875, 76 },
    { 0x704f6875, 99 },
    { 0, 0 },
    { 0x6c7f6878, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4bf687f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d1f688b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbedf688e, 61 },
    { 0, 0 },
    { 0xaa2f6890, 98 },
    { 0, 0 },
    { 0xc55f6892, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacf6896, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8f689b, 85 },
    { 0xd0cf689c, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2f68a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3f68a4, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x717f68a7, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f68ab, 33 },
    { 0x1b8f68ac, 34 },
    { 0x99ef68ac, 98 },
    { 0x4d5f68ae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5f68b1, 55 },
    { 0xb03f68b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x189f68b6, 76 },
    { 0x190f68b7, 66 },
    { 0, 0 },
    { 0x246f68b9, 52 },
    { 0x617f68ba, 93 },
    { 0, 0 },
    { 0xb97f68bc, 66 },
    { 0x8a5f68bd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbff68c9, 33 },
    { 0x1a4f68ca, 87 },
    { 0x7abf68cb, 88 },
    { 0, 0 },
    { 0x4d1f68cd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef68db, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5f68de, 93 },
    { 0xbf9f68de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f68e3, 98 },
    { 0x960f68e4, 23 },
    { 0xccf68e3, 33 },
    { 0x27bf68e6, 23 },
    { 0xaccf68e6, 61 },
    { 0x213f68e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bf68ec, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fff68ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79ff68f3, 23 },
    { 0x6bdf68f4, 33 },
    { 0x43af68f4, 96 },
    { 0, 0 },
    { 0x44af68f7, 33 },
    { 0xd77f68f7, 90 },
    { 0xe48f68f7, 76 },
    { 0xa69f68fa, 55 },
    { 0x19df68fb, 61 },
    { 0x18bf68f8, 90 },
    { 0xdf7f68fd, 90 },
    { 0x964f68fe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x39f6901, 61 },
    { 0, 0 },
    { 0x4fef6903, 33 },
    { 0x90ef6904, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf7f6908, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6df690f, 61 },
    { 0x658f6910, 43 },
    { 0x424f690f, 91 },
    { 0xef9f6912, 61 },
    { 0xf1ef6913, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5f6916, 76 },
    { 0xdddf6916, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287f691c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15f691f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x42bf6922, 66 },
    { 0x4cbf6923, 67 },
    { 0, 0 },
    { 0xc0f6925, 90 },
    { 0x428f6926, 87 },
    { 0x31df6927, 77 },
    { 0xee5f6928, 34 },
    { 0, 0 },
    { 0x8c2f692a, 55 },
    { 0xb29f692b, 85 },
    { 0x8b0f692a, 33 },
    { 0xcbff692d, 90 },
    { 0x627f692d, 98 },
    { 0xbabf692d, 90 },
    { 0xe39f692e, 61 },
    { 0x62df692e, 99 },
    { 0xe4f6932, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705f6937, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef693d, 95 },
    { 0, 0 },
    { 0xee0f693f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaf6942, 34 },
    { 0, 0 },
    { 0x77bf6944, 90 },
    { 0x744f6945, 33 },
    { 0xdbf6946, 33 },
    { 0x2c8f6944, 61 },
    { 0xbb2f6948, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942f694d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x842f6950, 77 },
    { 0xd85f6951, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f6955, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x909f6959, 95 },
    { 0x38df6959, 61 },
    { 0x9d2f6959, 98 },
    { 0x9f0f695c, 87 },
    { 0x4fef695c, 87 },
    { 0xc83f695d, 23 },
    { 0x7b8f695f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0f6969, 55 },
    { 0xcdff6969, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf27f696e, 33 },
    { 0xa9f696e, 34 },
    { 0xca0f6970, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x108f697e, 55 },
    { 0, 0 },
    { 0xde4f6980, 95 },
    { 0, 0 },
    { 0xee2f6982, 90 },
    { 0xc6f6983, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8f698d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f2f6990, 33 },
    { 0xa34f6990, 98 },
    { 0xd8f6992, 52 },
    { 0, 0 },
    { 0x79ef6994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848f699a, 61 },
    { 0xbe6f699a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8df699e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f69a2, 98 },
    { 0x569f69a3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fef69a7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x722f69aa, 95 },
    { 0x6f69ab, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x144f69b1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc39f69bb, 90 },
    { 0x2e0f69bc, 88 },
    { 0, 0 },
    { 0xddbf69be, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39f69c2, 43 },
    { 0x57ef69c2, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b0f69c8, 90 },
    { 0x507f69c9, 93 },
    { 0xbaf69ca, 23 },
    { 0, 0 },
    { 0x9f1f69cc, 46 },
    { 0, 0 },
    { 0xd06f69ce, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50f69d2, 38 },
    { 0x20bf69d2, 33 },
    { 0x94f69d4, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ff69dc, 98 },
    { 0x973f69dc, 97 },
    { 0, 0 },
    { 0xd65f69df, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77df69e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f69e8, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd33f69f4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xef0f69f7, 90 },
    { 0xf7cf69f8, 76 },
    { 0xa80f69f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x606f6a00, 33 },
    { 0, 0 },
    { 0xc50f6a02, 33 },
    { 0, 0 },
    { 0x4cef6a04, 95 },
    { 0x725f6a05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc15f6a09, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58af6a0f, 80 },
    { 0xadcf6a10, 66 },
    { 0x7daf6a10, 33 },
    { 0, 0 },
    { 0x2def6a13, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9f6a18, 90 },
    { 0, 0 },
    { 0xcd6f6a1a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f6a1d, 91 },
    { 0xeddf6a1d, 61 },
    { 0xb0cf6a1e, 100 },
    { 0xebdf6a20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x264f6a2a, 46 },
    { 0x759f6a2b, 23 },
    { 0x9cf6a2c, 76 },
    { 0xab7f6a2d, 95 },
    { 0xe85f6a2a, 33 },
    { 0x2ebf6a2d, 95 },
    { 0, 0 },
    { 0x8e8f6a31, 90 },
    { 0x3e4f6a31, 40 },
    { 0x77ff6a31, 90 },
    { 0x38bf6a34, 98 },
    { 0xf45f6a34, 34 },
    { 0x10cf6a35, 23 },
    { 0, 0 },
    { 0x814f6a38, 33 },
    { 0xed5f6a39, 84 },
    { 0xd3f6a3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49af6a41, 36 },
    { 0, 0 },
    { 0xfd8f6a43, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f6a50, 100 },
    { 0xdd5f6a50, 61 },
    { 0x2f6f6a50, 93 },
    { 0xc04f6a53, 76 },
    { 0x56ef6a54, 43 },
    { 0x185f6a53, 33 },
    { 0, 0 },
    { 0x9c1f6a57, 90 },
    { 0xcdcf6a58, 76 },
    { 0, 0 },
    { 0x6d5f6a5a, 61 },
    { 0x9ecf6a5b, 100 },
    { 0, 0 },
    { 0xa0df6a5d, 46 },
    { 0, 0 },
    { 0xfaef6a5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f6a63, 68 },
    { 0x5d5f6a64, 98 },
    { 0x429f6a65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf28f6a6a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8f6a73, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ff6a7f, 68 },
    { 0, 0 },
    { 0x4a7f6a81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75f6a89, 95 },
    { 0xd47f6a8a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x801f6a8d, 90 },
    { 0x490f6a8e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xddef6a91, 98 },
    { 0, 0 },
    { 0xbf2f6a93, 53 },
    { 0x827f6a94, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17df6a98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8f6a9b, 95 },
    { 0xfc9f6a9c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x245f6aa2, 33 },
    { 0x7fcf6aa2, 23 },
    { 0x4aaf6aa2, 90 },
    { 0xa4cf6aa4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda8f6aac, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadaf6aaf, 52 },
    { 0xf92f6ab0, 95 },
    { 0xd5cf6aaf, 100 },
    { 0x64df6ab2, 33 },
    { 0x541f6ab2, 81 },
    { 0xdd8f6ab1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8f6ab8, 52 },
    { 0, 0 },
    { 0x2aff6aba, 83 },
    { 0x3e7f6aba, 90 },
    { 0x33bf6abb, 84 },
    { 0x1e1f6abd, 90 },
    { 0x37af6abe, 33 },
    { 0, 0 },
    { 0x491f6ac0, 76 },
    { 0xfbef6ac0, 33 },
    { 0, 0 },
    { 0x891f6ac3, 100 },
    { 0xc2f6ac4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ff6ac7, 90 },
    { 0x487f6ac8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4f6acf, 85 },
    { 0xa39f6ad0, 34 },
    { 0x493f6ad1, 44 },
    { 0, 0 },
    { 0xd5bf6ad3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46f6ad8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f6adb, 55 },
    { 0, 0 },
    { 0x3b3f6add, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x940f6ae2, 98 },
    { 0xf3ef6ae3, 78 },
    { 0xc16f6ae3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adf6ae7, 52 },
    { 0xd79f6ae8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x707f6aee, 98 },
    { 0, 0 },
    { 0x859f6af0, 98 },
    { 0, 0 },
    { 0x852f6af2, 55 },
    { 0x761f6af2, 95 },
    { 0x63ef6af4, 88 },
    { 0x208f6af5, 61 },
    { 0x7d2f6af4, 61 },
    { 0, 0 },
    { 0x63ff6af8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cf6afe, 98 },
    { 0, 0 },
    { 0x254f6b00, 68 },
    { 0x36bf6b00, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af6b07, 34 },
    { 0, 0 },
    { 0x6c1f6b09, 93 },
    { 0xb4af6b0a, 93 },
    { 0x580f6b0b, 95 },
    { 0xeb5f6b0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164f6b14, 76 },
    { 0x2f8f6b14, 95 },
    { 0, 0 },
    { 0x92cf6b17, 90 },
    { 0x799f6b17, 33 },
    { 0xc1cf6b18, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4f6b1d, 100 },
    { 0, 0 },
    { 0x463f6b1f, 93 },
    { 0, 0 },
    { 0x586f6b21, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x72df6b24, 46 },
    { 0xf79f6b25, 43 },
    { 0, 0 },
    { 0xac7f6b27, 36 },
    { 0x593f6b28, 94 },
    { 0, 0 },
    { 0x3af6b2a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9f6b2e, 84 },
    { 0x807f6b2f, 23 },
    { 0, 0 },
    { 0x851f6b31, 33 },
    { 0x1fef6b32, 84 },
    { 0x462f6b31, 93 },
    { 0x984f6b31, 33 },
    { 0x6dbf6b35, 98 },
    { 0xff9f6b34, 90 },
    { 0x5e9f6b37, 95 },
    { 0, 0 },
    { 0x533f6b39, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ef6b40, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f6b47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929f6b4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5f6b52, 34 },
    { 0xd87f6b52, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bf6b57, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f6b5c, 61 },
    { 0x733f6b5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb98f6b61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600f6b66, 98 },
    { 0x540f6b67, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596f6b6c, 66 },
    { 0x548f6b6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61af6b74, 33 },
    { 0x24f6b74, 61 },
    { 0xeb1f6b75, 61 },
    { 0, 0 },
    { 0xe82f6b78, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9f6b80, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f6b84, 52 },
    { 0x3a9f6b84, 33 },
    { 0x3a8f6b86, 93 },
    { 0x3c6f6b87, 76 },
    { 0, 0 },
    { 0x54af6b89, 95 },
    { 0xfdaf6b8a, 90 },
    { 0, 0 },
    { 0x977f6b8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4f6b90, 55 },
    { 0, 0 },
    { 0x367f6b92, 55 },
    { 0xd55f6b93, 90 },
    { 0xcbcf6b94, 23 },
    { 0xa3f6b94, 17 },
    { 0, 0 },
    { 0x13df6b97, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94f6b9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ff6b9f, 37 },
    { 0x694f6ba0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecf6ba3, 90 },
    { 0, 0 },
    { 0xa9cf6ba5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96f6bab, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6f6bb0, 81 },
    { 0x675f6bb0, 23 },
    { 0xea8f6bb2, 52 },
    { 0, 0 },
    { 0xde4f6bb4, 95 },
    { 0, 0 },
    { 0x3f4f6bb6, 52 },
    { 0x207f6bb7, 33 },
    { 0x65bf6bb7, 88 },
    { 0x726f6bb7, 69 },
    { 0xfa5f6bb7, 23 },
    { 0, 0 },
    { 0x46cf6bbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cf6bc0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x136f6bc3, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0x985f6bc6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171f6bca, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801f6bd0, 40 },
    { 0, 0 },
    { 0xd54f6bd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f6bd7, 85 },
    { 0x6d0f6bd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25af6bde, 43 },
    { 0xa98f6bdf, 33 },
    { 0x5a6f6bdf, 61 },
    { 0x2b5f6be1, 95 },
    { 0x9df6be2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196f6be8, 46 },
    { 0x8f6be8, 61 },
    { 0x562f6bea, 87 },
    { 0xf0cf6be9, 90 },
    { 0x8fbf6bec, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41f6bf4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bf6bfa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71f6bfd, 96 },
    { 0x4b1f6bfd, 90 },
    { 0xb46f6bff, 84 },
    { 0xea9f6c00, 90 },
    { 0, 0 },
    { 0xc68f6c02, 98 },
    { 0, 0 },
    { 0xaaaf6c04, 65 },
    { 0, 0 },
    { 0x682f6c06, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23ef6c0f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x155f6c14, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef6c18, 61 },
    { 0xa00f6c19, 98 },
    { 0xa68f6c1a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecf6c1e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df6c21, 46 },
    { 0xa99f6c22, 61 },
    { 0, 0 },
    { 0xd16f6c24, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def6c2d, 33 },
    { 0xc50f6c2e, 65 },
    { 0x44bf6c2f, 23 },
    { 0x4e2f6c2d, 33 },
    { 0xa9bf6c31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6f6c35, 99 },
    { 0, 0 },
    { 0x578f6c37, 34 },
    { 0x683f6c37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186f6c3d, 81 },
    { 0, 0 },
    { 0x71cf6c3f, 95 },
    { 0xc0df6c40, 90 },
    { 0x38f6c3f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2f6c46, 61 },
    { 0, 0 },
    { 0xfdff6c48, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00f6c4c, 55 },
    { 0xa5ef6c4c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b4f6c50, 61 },
    { 0, 0 },
    { 0x947f6c52, 61 },
    { 0xc85f6c53, 52 },
    { 0x76ff6c53, 98 },
    { 0, 0 },
    { 0x3dcf6c56, 61 },
    { 0xde6f6c57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bf6c5a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441f6c62, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6f6c65, 97 },
    { 0x8ccf6c66, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f6c69, 76 },
    { 0, 0 },
    { 0xff8f6c6b, 43 },
    { 0x49bf6c6c, 23 },
    { 0x592f6c6d, 23 },
    { 0x648f6c6e, 43 },
    { 0x4bf6c6f, 93 },
    { 0x9b8f6c6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34f6c73, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd75f6c7a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xa73f6c7d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fcf6c81, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaff6c85, 99 },
    { 0x96f6c86, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cf6c8c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400f6c91, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4f6c94, 33 },
    { 0x743f6c95, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ef6ca0, 61 },
    { 0xd7f6ca1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2f6ca4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaf6ca8, 91 },
    { 0xb84f6ca9, 98 },
    { 0x67af6ca8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfdf6cb0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ff6cb3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f6cbb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6f6cc2, 95 },
    { 0x771f6cc3, 76 },
    { 0x39df6cc2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdff6ccc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf6cd1, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6f6cd4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacef6cd8, 55 },
    { 0x977f6cd8, 98 },
    { 0, 0 },
    { 0x8ff6cdb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabf6ce0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0f6ce7, 90 },
    { 0xff2f6ce8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x28f6ceb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971f6cf0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0f6cf4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x662f6cf8, 95 },
    { 0, 0 },
    { 0xbfdf6cfa, 34 },
    { 0, 0 },
    { 0xaaef6cfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529f6d0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ff6d13, 93 },
    { 0xc0af6d14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332f6d19, 66 },
    { 0, 0 },
    { 0xf1f6d1b, 34 },
    { 0x1f1f6d1b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4faf6d23, 90 },
    { 0xbcbf6d23, 69 },
    { 0x4d2f6d25, 33 },
    { 0x70bf6d26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f6d2b, 76 },
    { 0x9f2f6d2c, 85 },
    { 0x96bf6d2c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26f6d32, 23 },
    { 0, 0 },
    { 0xe8bf6d34, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1bf6d38, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x475f6d3e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x361f6d43, 33 },
    { 0x5bbf6d44, 95 },
    { 0xa1bf6d45, 84 },
    { 0x824f6d46, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x552f6d49, 52 },
    { 0, 0 },
    { 0xa52f6d4b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbbf6d4e, 99 },
    { 0xd34f6d4e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf41f6d52, 76 },
    { 0x16ff6d53, 61 },
    { 0, 0 },
    { 0x2eef6d55, 91 },
    { 0x64cf6d56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f6d5a, 91 },
    { 0, 0 },
    { 0x7ecf6d5c, 33 },
    { 0xcf2f6d5c, 90 },
    { 0xf2cf6d5e, 34 },
    { 0, 0 },
    { 0x5a3f6d60, 76 },
    { 0, 0 },
    { 0xa11f6d62, 41 },
    { 0x3c3f6d63, 98 },
    { 0xdabf6d64, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1f6d6d, 66 },
    { 0xb87f6d6e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x788f6d73, 94 },
    { 0x6a4f6d74, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ff6d78, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3f6d7e, 33 },
    { 0, 0 },
    { 0xc80f6d80, 100 },
    { 0, 0 },
    { 0x6c6f6d82, 95 },
    { 0, 0 },
    { 0xb7df6d84, 90 },
    { 0xd6af6d84, 76 },
    { 0x79f6d86, 61 },
    { 0xa36f6d87, 23 },
    { 0x752f6d88, 33 },
    { 0x7d2f6d88, 61 },
    { 0, 0 },
    { 0x516f6d8b, 100 },
    { 0, 0 },
    { 0x31bf6d8d, 61 },
    { 0, 0 },
    { 0x521f6d8f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ff6d93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdf6d9b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f6da2, 61 },
    { 0x2caf6da2, 61 },
    { 0x4c4f6da4, 100 },
    { 0x424f6da5, 33 },
    { 0x628f6da4, 95 },
    { 0x1c3f6da7, 78 },
    { 0, 0 },
    { 0xfbaf6da9, 38 },
    { 0xd07f6daa, 55 },
    { 0, 0 },
    { 0xbf3f6dac, 78 },
    { 0xe16f6dad, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bf6db2, 64 },
    { 0x210f6db2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f6db6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef6dc2, 90 },
    { 0xcccf6dc3, 90 },
    { 0x9caf6dc4, 99 },
    { 0x998f6dc5, 33 },
    { 0xb58f6dc2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x319f6dc9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ff6dcd, 84 },
    { 0x12bf6dce, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cf6dd6, 33 },
    { 0xe27f6dd6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5f6ddf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ef6dea, 98 },
    { 0x9dbf6dea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f6def, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cf6df3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8ef6df8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bf6dff, 98 },
    { 0x4dff6e00, 81 },
    { 0xacf6dff, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248f6e0c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7f6e10, 95 },
    { 0, 0 },
    { 0xe1bf6e12, 81 },
    { 0xee9f6e13, 93 },
    { 0xe62f6e14, 77 },
    { 0xa8ff6e15, 52 },
    { 0x2bbf6e15, 43 },
    { 0, 0 },
    { 0x3a1f6e18, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32f6e21, 61 },
    { 0, 0 },
    { 0xdf0f6e23, 90 },
    { 0, 0 },
    { 0x707f6e25, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x66af6e28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296f6e2d, 33 },
    { 0xc99f6e2d, 23 },
    { 0, 0 },
    { 0x415f6e30, 46 },
    { 0x371f6e31, 88 },
    { 0, 0 },
    { 0xa4df6e33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8f6e37, 55 },
    { 0x243f6e37, 84 },
    { 0x295f6e38, 100 },
    { 0x5e6f6e3a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xd87f6e3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1daf6e41, 91 },
    { 0x972f6e42, 78 },
    { 0x5e5f6e43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df6e46, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ff6e4a, 95 },
    { 0xbf6e4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5f6e4f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe56f6e54, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16f6e58, 55 },
    { 0xbbaf6e59, 95 },
    { 0x353f6e5a, 91 },
    { 0x946f6e5a, 95 },
    { 0xf4ef6e58, 33 },
    { 0x790f6e5c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57f6e60, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x561f6e67, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490f6e6b, 90 },
    { 0x771f6e6c, 85 },
    { 0x500f6e6c, 75 },
    { 0, 0 },
    { 0x5a1f6e6f, 76 },
    { 0xf3bf6e70, 98 },
    { 0x869f6e70, 61 },
    { 0x4a5f6e72, 90 },
    { 0x3ecf6e71, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cf6e77, 65 },
    { 0x86f6e78, 76 },
    { 0x7aff6e79, 33 },
    { 0x521f6e79, 33 },
    { 0x391f6e7b, 90 },
    { 0xcfdf6e79, 43 },
    { 0, 0 },
    { 0xa99f6e7e, 98 },
    { 0, 0 },
    { 0x43bf6e80, 75 },
    { 0, 0 },
    { 0x953f6e82, 33 },
    { 0xb80f6e83, 52 },
    { 0, 0 },
    { 0x61bf6e85, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71f6e88, 68 },
    { 0xff0f6e89, 76 },
    { 0x97af6e88, 43 },
    { 0xf81f6e8b, 33 },
    { 0x6a4f6e8c, 23 },
    { 0x94f6e88, 87 },
    { 0x317f6e8a, 83 },
    { 0xe5ff6e8f, 55 },
    { 0xc56f6e90, 77 },
    { 0xe6ff6e91, 95 },
    { 0x8f1f6e91, 99 },
    { 0x669f6e93, 46 },
    { 0xf5af6e94, 90 },
    { 0, 0 },
    { 0x82af6e96, 23 },
    { 0x99cf6e97, 90 },
    { 0x550f6e98, 68 },
    { 0xcebf6e97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ff6e9c, 43 },
    { 0xc27f6e9d, 34 },
    { 0xd42f6e9d, 61 },
    { 0x6edf6e9f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f6ea9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcf6ead, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df6ebc, 98 },
    { 0x586f6ebd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cf6ec1, 76 },
    { 0, 0 },
    { 0x15bf6ec3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ff6ec7, 88 },
    { 0x2abf6ec8, 65 },
    { 0x3dff6ec8, 33 },
    { 0x23f6eca, 61 },
    { 0, 0 },
    { 0x197f6ecc, 75 },
    { 0xdf6f6ecc, 33 },
    { 0xde3f6ece, 34 },
    { 0xac8f6ecf, 93 },
    { 0x929f6ecf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c2f6ed5, 61 },
    { 0xda3f6ed6, 33 },
    { 0xfdcf6ed7, 96 },
    { 0, 0 },
    { 0x692f6ed9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6f6edd, 98 },
    { 0x1f7f6edd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07f6ee2, 23 },
    { 0xf40f6ee3, 78 },
    { 0xae6f6ee3, 33 },
    { 0xf4cf6ee4, 95 },
    { 0xd6df6ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f6eeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8f6ef1, 85 },
    { 0x731f6ef2, 61 },
    { 0x32ff6ef3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38af6efa, 98 },
    { 0xbfef6efb, 77 },
    { 0xe6ff6efb, 66 },
    { 0, 0 },
    { 0xe33f6efe, 66 },
    { 0x2a8f6eff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68df6f03, 95 },
    { 0x7f2f6f04, 78 },
    { 0xd74f6f03, 61 },
    { 0, 0 },
    { 0x7f6f6f07, 88 },
    { 0, 0 },
    { 0x4bef6f09, 43 },
    { 0x197f6f0a, 34 },
    { 0x67cf6f0a, 66 },
    { 0x11ef6f0c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f6f0f, 90 },
    { 0, 0 },
    { 0x64ef6f11, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f6f18, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fef6f1e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc6f6f22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaff6f2c, 33 },
    { 0xb37f6f2d, 95 },
    { 0xbb9f6f2e, 23 },
    { 0x5faf6f2f, 67 },
    { 0xeddf6f30, 93 },
    { 0xb38f6f30, 23 },
    { 0x9cdf6f31, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848f6f38, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x708f6f3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f6f41, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff6f48, 78 },
    { 0, 0 },
    { 0x392f6f4a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bf6f52, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa95f6f57, 33 },
    { 0xf3ff6f57, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6f6f5d, 23 },
    { 0x7a5f6f5e, 23 },
    { 0xddef6f5e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ff6f65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f6f6a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x938f6f6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf6f72, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7f6f77, 95 },
    { 0x69af6f77, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f6f7c, 84 },
    { 0x1b2f6f7c, 61 },
    { 0xb72f6f7e, 98 },
    { 0, 0 },
    { 0xa0af6f80, 100 },
    { 0x53df6f81, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab1f6f87, 98 },
    { 0x582f6f88, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9f6f8c, 90 },
    { 0, 0 },
    { 0xf1f6f8e, 33 },
    { 0xf87f6f8f, 93 },
    { 0x8e7f6f90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f6f93, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f6f97, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae5f6f9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f6fa4, 33 },
    { 0, 0 },
    { 0x2d6f6fa6, 100 },
    { 0, 0 },
    { 0xd2df6fa8, 61 },
    { 0x415f6fa9, 43 },
    { 0xc65f6faa, 98 },
    { 0xa57f6fab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f1f6faf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3f6fb6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ff6fb9, 90 },
    { 0, 0 },
    { 0xe5f6fbb, 90 },
    { 0xce9f6fbb, 46 },
    { 0x425f6fbd, 33 },
    { 0x3dff6fbe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5f6fc1, 90 },
    { 0x102f6fc1, 55 },
    { 0x6d7f6fc3, 76 },
    { 0xe4f6fc4, 23 },
    { 0x4b4f6fc5, 61 },
    { 0xeedf6fc3, 100 },
    { 0x18af6fc7, 77 },
    { 0x381f6fc8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d5f6fcd, 99 },
    { 0x170f6fce, 23 },
    { 0, 0 },
    { 0x28bf6fd0, 61 },
    { 0x214f6fd1, 33 },
    { 0x1b7f6fd2, 78 },
    { 0, 0 },
    { 0x429f6fd4, 52 },
    { 0, 0 },
    { 0x788f6fd6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42f6fdc, 91 },
    { 0x23f6fdd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330f6fe2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x733f6fe7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff3f6ff2, 55 },
    { 0x8aaf6ff2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x50af6ff6, 98 },
    { 0x78af6ff7, 76 },
    { 0x78df6ff8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0f6fff, 46 },
    { 0x66af7000, 61 },
    { 0, 0 },
    { 0xb30f7002, 43 },
    { 0x6b7f7002, 98 },
    { 0xbc1f7004, 69 },
    { 0xdcdf7004, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04f7009, 95 },
    { 0x227f700a, 61 },
    { 0xab9f7009, 55 },
    { 0xb74f700c, 61 },
    { 0, 0 },
    { 0x1def700e, 23 },
    { 0xa19f700f, 33 },
    { 0x7dbf700f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaf7016, 33 },
    { 0xd75f7016, 90 },
    { 0x737f7016, 61 },
    { 0xc8df7019, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f701d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00f7020, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85f7024, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f702a, 67 },
    { 0x9daf702b, 95 },
    { 0, 0 },
    { 0xb7af702d, 43 },
    { 0xb3f702e, 61 },
    { 0xf37f702f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68f7032, 69 },
    { 0, 0 },
    { 0xb97f7034, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549f703a, 90 },
    { 0x324f703b, 77 },
    { 0, 0 },
    { 0xddef703d, 55 },
    { 0, 0 },
    { 0xc27f703f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ff7045, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8ef7048, 43 },
    { 0, 0 },
    { 0x6a3f704a, 75 },
    { 0xc4ff704b, 98 },
    { 0x92ff704b, 23 },
    { 0xa57f704d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70f7051, 76 },
    { 0, 0 },
    { 0xeaef7053, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd74f7057, 98 },
    { 0, 0 },
    { 0xdf6f7059, 90 },
    { 0, 0 },
    { 0xf74f705b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9f705f, 23 },
    { 0, 0 },
    { 0x66f7061, 98 },
    { 0, 0 },
    { 0xaa0f7063, 90 },
    { 0, 0 },
    { 0x894f7065, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x811f7068, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ff706c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03f7073, 67 },
    { 0xe24f7074, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1f7077, 55 },
    { 0, 0 },
    { 0x725f7079, 34 },
    { 0x470f707a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb5f7084, 78 },
    { 0xebcf7084, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3f708e, 91 },
    { 0, 0 },
    { 0xa40f7090, 87 },
    { 0, 0 },
    { 0xffff7092, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ef7095, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfef709a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82df70a2, 61 },
    { 0x9edf70a3, 90 },
    { 0, 0 },
    { 0x37bf70a5, 98 },
    { 0xeaef70a6, 33 },
    { 0xa71f70a6, 23 },
    { 0, 0 },
    { 0x655f70a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12f70ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf70b4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedef70b8, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff70bd, 76 },
    { 0x1eef70be, 33 },
    { 0x599f70be, 61 },
    { 0, 0 },
    { 0x31f70c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd64f70c4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ccf70c9, 23 },
    { 0, 0 },
    { 0x8c9f70cb, 87 },
    { 0xf6bf70cc, 66 },
    { 0x198f70cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x797f70d1, 43 },
    { 0x630f70d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ff70d9, 88 },
    { 0xdb7f70d9, 90 },
    { 0xfb4f70db, 95 },
    { 0x8dbf70db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538f70e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf70e4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7f70e7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14cf70ec, 61 },
    { 0xb29f70ed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x62f70f0, 90 },
    { 0, 0 },
    { 0x94bf70f2, 66 },
    { 0xb5bf70f3, 76 },
    { 0x12ff70f3, 34 },
    { 0xdeff70f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf04f70fa, 90 },
    { 0xcf7f70fa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd46f70ff, 34 },
    { 0xb14f7100, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0f7106, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde8f710b, 69 },
    { 0xb16f710c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfacf7111, 95 },
    { 0xa69f7112, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9edf7116, 55 },
    { 0x2edf7117, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86f711f, 90 },
    { 0xcbf7120, 100 },
    { 0x387f7121, 33 },
    { 0, 0 },
    { 0x7e2f7123, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x171f7127, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x37df712a, 67 },
    { 0, 0 },
    { 0xd76f712c, 43 },
    { 0xa34f712c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9f7131, 95 },
    { 0, 0 },
    { 0x795f7133, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x589f7137, 93 },
    { 0x844f7138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eef7141, 33 },
    { 0x77ff7142, 61 },
    { 0, 0 },
    { 0x5c3f7144, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x459f714c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd0f714f, 23 },
    { 0x37cf714f, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bf7153, 98 },
    { 0, 0 },
    { 0x186f7155, 66 },
    { 0, 0 },
    { 0x7b5f7157, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x703f715c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487f7161, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186f716a, 77 },
    { 0xc11f716a, 83 },
    { 0, 0 },
    { 0x121f716d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25f7173, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0f7179, 33 },
    { 0xfaff717a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38ff717e, 95 },
    { 0xa9bf717e, 64 },
    { 0, 0 },
    { 0x584f7181, 95 },
    { 0xa46f7182, 88 },
    { 0x258f7183, 75 },
    { 0, 0 },
    { 0x87bf7185, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c6f7188, 91 },
    { 0x8dff7189, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bef7191, 90 },
    { 0xa2df7191, 53 },
    { 0xc6cf7193, 61 },
    { 0x707f7193, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9f719a, 46 },
    { 0x14ff719b, 98 },
    { 0x521f719c, 55 },
    { 0xdb1f719d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421f71a1, 61 },
    { 0x589f71a2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1f71a6, 55 },
    { 0x297f71a6, 90 },
    { 0x1a0f71a7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f71ab, 61 },
    { 0, 0 },
    { 0xb81f71ad, 85 },
    { 0xa56f71ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dbf71b3, 23 },
    { 0xa03f71b3, 43 },
    { 0xecbf71b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ef71ba, 43 },
    { 0x195f71bb, 95 },
    { 0x530f71bc, 58 },
    { 0xb25f71bd, 43 },
    { 0x117f71bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69f71c1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf71c6, 34 },
    { 0xea4f71c7, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x973f71ca, 98 },
    { 0xc93f71cb, 33 },
    { 0, 0 },
    { 0x43bf71cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x182f71d1, 61 },
    { 0xca5f71d1, 90 },
    { 0x8aaf71d1, 90 },
    { 0x2a8f71d1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6f71d7, 78 },
    { 0xeaf71d8, 90 },
    { 0x68bf71d9, 78 },
    { 0xf93f71d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42af71de, 33 },
    { 0x32cf71df, 33 },
    { 0x404f71df, 98 },
    { 0x24ff71e1, 52 },
    { 0x655f71e1, 55 },
    { 0x571f71e0, 33 },
    { 0x307f71e3, 23 },
    { 0x7cbf71e5, 68 },
    { 0x40f71e4, 34 },
    { 0x944f71e7, 93 },
    { 0xfc1f71e8, 55 },
    { 0x405f71e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe8f71ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9f71f2, 33 },
    { 0, 0 },
    { 0x335f71f4, 84 },
    { 0, 0 },
    { 0xbcef71f6, 98 },
    { 0xdf8f71f7, 91 },
    { 0, 0 },
    { 0x355f71f9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af7201, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9af7205, 46 },
    { 0x73ff7206, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5f720a, 98 },
    { 0x127f720a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3f720e, 77 },
    { 0x92bf720e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f7215, 90 },
    { 0xa40f7216, 95 },
    { 0xa2ef7216, 61 },
    { 0x828f7216, 87 },
    { 0, 0 },
    { 0x844f721a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ff721e, 90 },
    { 0x99bf721f, 46 },
    { 0xac0f721f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e8f7223, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77ef7227, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5caf722d, 83 },
    { 0x114f722e, 61 },
    { 0, 0 },
    { 0xb30f7230, 61 },
    { 0xccff7230, 33 },
    { 0x6eef7232, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf7236, 81 },
    { 0x10cf7237, 61 },
    { 0, 0 },
    { 0xfaaf7239, 90 },
    { 0x21cf723a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4f723f, 87 },
    { 0xa0bf7240, 61 },
    { 0xfd3f7241, 61 },
    { 0xac1f7241, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfef7247, 61 },
    { 0xf34f7247, 90 },
    { 0x918f7248, 61 },
    { 0xbd3f724a, 46 },
    { 0xb61f724b, 95 },
    { 0xed6f724c, 23 },
    { 0x5f4f724c, 23 },
    { 0, 0 },
    { 0x6c0f724f, 43 },
    { 0x342f7250, 98 },
    { 0x19df7251, 33 },
    { 0x4bdf7251, 91 },
    { 0xeaff7253, 43 },
    { 0x87f7254, 43 },
    { 0x43af7251, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x921f7258, 93 },
    { 0xa2df7258, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94df725f, 33 },
    { 0, 0 },
    { 0xe3f7261, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x539f7264, 95 },
    { 0, 0 },
    { 0x4a4f7266, 84 },
    { 0x608f7266, 55 },
    { 0xa54f7268, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecaf726d, 95 },
    { 0, 0 },
    { 0xe68f726f, 93 },
    { 0x881f7270, 90 },
    { 0x3bef7271, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ef7275, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d1f727a, 20 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6af727e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf7282, 23 },
    { 0, 0 },
    { 0x899f7284, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f4f7287, 90 },
    { 0, 0 },
    { 0x11cf7289, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3f728f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f7294, 99 },
    { 0xf06f7295, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbf7298, 95 },
    { 0, 0 },
    { 0x343f729a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ff729f, 90 },
    { 0, 0 },
    { 0xdbcf72a1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9f72a4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x391f72aa, 90 },
    { 0, 0 },
    { 0xe75f72ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bcf72b0, 33 },
    { 0x8c7f72b0, 61 },
    { 0, 0 },
    { 0x24ff72b3, 36 },
    { 0xa55f72b3, 66 },
    { 0x55ff72b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79f72c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdef72c4, 33 },
    { 0, 0 },
    { 0x4b3f72c6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x802f72c9, 85 },
    { 0x4e8f72c9, 76 },
    { 0xec3f72cb, 23 },
    { 0xb9df72cb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94cf72d8, 91 },
    { 0x6e6f72d9, 66 },
    { 0xeef72d9, 98 },
    { 0x8c8f72db, 92 },
    { 0, 0 },
    { 0xc80f72dd, 66 },
    { 0x5a6f72de, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21ff72eb, 100 },
    { 0x6acf72ec, 90 },
    { 0xa4df72ed, 90 },
    { 0x8aff72ed, 90 },
    { 0x11af72ec, 90 },
    { 0x440f72f0, 96 },
    { 0x77f72f1, 93 },
    { 0xd93f72f1, 96 },
    { 0xa3ef72ed, 90 },
    { 0, 0 },
    { 0xa27f72f5, 47 },
    { 0, 0 },
    { 0xe2cf72f7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x37f72fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f72fd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x879f7300, 95 },
    { 0, 0 },
    { 0x7c2f7302, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1f7306, 23 },
    { 0, 0 },
    { 0xf48f7308, 98 },
    { 0xa5bf7309, 100 },
    { 0xd0cf730a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6af7312, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x447f7315, 99 },
    { 0x34df7316, 78 },
    { 0, 0 },
    { 0x42bf7318, 23 },
    { 0xe90f7319, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d1f731c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ff7322, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x712f7325, 34 },
    { 0xd0f7325, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x710f732b, 61 },
    { 0, 0 },
    { 0xd64f732d, 81 },
    { 0x623f732e, 76 },
    { 0x36ff732d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2f7333, 23 },
    { 0, 0 },
    { 0x8acf7335, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0f733a, 61 },
    { 0xcd0f733b, 80 },
    { 0, 0 },
    { 0xf86f733d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f7341, 73 },
    { 0, 0 },
    { 0xecdf7343, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x21bf7346, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48cf7351, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf7355, 91 },
    { 0x5aef7356, 99 },
    { 0xa6bf7357, 95 },
    { 0, 0 },
    { 0xe97f7359, 98 },
    { 0x67bf735a, 88 },
    { 0x7c7f7359, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef7365, 33 },
    { 0xb28f7365, 98 },
    { 0xe0af7367, 23 },
    { 0xffcf7366, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f736b, 85 },
    { 0x117f736c, 80 },
    { 0xaf736d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48f7370, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13ef7375, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99f737d, 23 },
    { 0, 0 },
    { 0x40af737f, 43 },
    { 0xf91f7380, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x982f7383, 90 },
    { 0xa6cf7384, 43 },
    { 0x3c4f7383, 98 },
    { 0x9a0f7383, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x620f738c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3f738f, 61 },
    { 0, 0 },
    { 0x9a6f7391, 46 },
    { 0, 0 },
    { 0x259f7393, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18f7399, 90 },
    { 0, 0 },
    { 0xe3df739b, 93 },
    { 0x8ebf739c, 55 },
    { 0xc98f739d, 90 },
    { 0x407f739d, 17 },
    { 0xf78f739f, 55 },
    { 0x8cff73a0, 85 },
    { 0, 0 },
    { 0x735f73a2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ef73a8, 52 },
    { 0x4c8f73a9, 95 },
    { 0xff4f73a9, 61 },
    { 0xba7f73aa, 46 },
    { 0x7c2f73ac, 98 },
    { 0x9b8f73aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf73b1, 69 },
    { 0x1d9f73b2, 90 },
    { 0xd7ef73b3, 90 },
    { 0xeabf73b3, 61 },
    { 0, 0 },
    { 0x350f73b6, 61 },
    { 0, 0 },
    { 0xee9f73b8, 95 },
    { 0x7b9f73b8, 33 },
    { 0, 0 },
    { 0x4caf73bb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x289f73be, 33 },
    { 0x516f73bf, 100 },
    { 0xed7f73c0, 38 },
    { 0xaa4f73bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecef73c7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22f73d3, 23 },
    { 0, 0 },
    { 0x1faf73d5, 91 },
    { 0, 0 },
    { 0x675f73d7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5f73e0, 100 },
    { 0x52f73e0, 76 },
    { 0xf2f73e2, 100 },
    { 0x58df73e2, 99 },
    { 0x726f73e3, 17 },
    { 0xe08f73e5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65ff73ec, 95 },
    { 0x974f73ec, 76 },
    { 0x4ebf73ec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0f73f2, 23 },
    { 0x767f73f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x734f73f6, 75 },
    { 0xfb5f73f7, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4f73ff, 81 },
    { 0, 0 },
    { 0x1bdf7401, 100 },
    { 0, 0 },
    { 0xef6f7403, 61 },
    { 0xb7bf7403, 23 },
    { 0xb1ef7405, 55 },
    { 0x2ccf7406, 33 },
    { 0x8ddf7406, 23 },
    { 0x963f7408, 90 },
    { 0x6b8f7409, 43 },
    { 0x1e1f7409, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f740d, 80 },
    { 0xd64f740e, 88 },
    { 0x896f740f, 95 },
    { 0xee7f740d, 55 },
    { 0x711f7411, 94 },
    { 0x638f7411, 75 },
    { 0xe6cf7413, 61 },
    { 0x89f7413, 81 },
    { 0x420f7415, 76 },
    { 0xc66f7415, 33 },
    { 0x7e3f7414, 33 },
    { 0xa3af7414, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xceef741b, 23 },
    { 0xfb7f741b, 100 },
    { 0, 0 },
    { 0x7a3f741e, 95 },
    { 0, 0 },
    { 0xb36f7420, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0f7425, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd70f7429, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ff742c, 9 },
    { 0x82bf742c, 65 },
    { 0, 0 },
    { 0x479f742f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82f7439, 90 },
    { 0, 0 },
    { 0x335f743b, 33 },
    { 0, 0 },
    { 0x269f743d, 33 },
    { 0xe1bf743d, 17 },
    { 0xe4bf743e, 87 },
    { 0x6cef7440, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6f7444, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x499f744c, 55 },
    { 0x2a1f744c, 61 },
    { 0, 0 },
    { 0x6d4f744f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x599f7455, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bf745d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaf7463, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51f7468, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf746b, 33 },
    { 0xaa4f746c, 78 },
    { 0x27ff746d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bef7472, 78 },
    { 0xc92f7472, 95 },
    { 0x58ff7474, 34 },
    { 0x51df7475, 93 },
    { 0x73af7476, 66 },
    { 0x592f7477, 98 },
    { 0, 0 },
    { 0x7ff7479, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xff4f747c, 15 },
    { 0xd8ff747d, 52 },
    { 0x92f747c, 61 },
    { 0, 0 },
    { 0x67af7480, 55 },
    { 0x5f6f7480, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbf7486, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecf748c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baf7492, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62af7496, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xed4f7499, 88 },
    { 0x413f749a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69af74a0, 34 },
    { 0, 0 },
    { 0xadef74a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ef74a5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeef74a8, 91 },
    { 0x32bf74a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137f74ad, 68 },
    { 0, 0 },
    { 0xc7af74af, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x935f74b5, 90 },
    { 0, 0 },
    { 0x39ff74b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x688f74bc, 55 },
    { 0x3e9f74bd, 90 },
    { 0, 0 },
    { 0x651f74bf, 81 },
    { 0x2f8f74bf, 52 },
    { 0x73df74c0, 34 },
    { 0xa1ef74c2, 56 },
    { 0x166f74c3, 43 },
    { 0xe22f74c0, 100 },
    { 0x768f74c5, 98 },
    { 0, 0 },
    { 0x741f74c7, 78 },
    { 0, 0 },
    { 0x7b2f74c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0f74cf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x875f74d2, 90 },
    { 0x8f8f74d2, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x121f74d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0df74db, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4f74de, 97 },
    { 0xc59f74de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f74e3, 81 },
    { 0xfaef74e3, 55 },
    { 0xf19f74e5, 66 },
    { 0, 0 },
    { 0x9cf74e7, 61 },
    { 0, 0 },
    { 0x82bf74e9, 90 },
    { 0, 0 },
    { 0x1fdf74eb, 33 },
    { 0x8e5f74eb, 66 },
    { 0, 0 },
    { 0x10bf74ee, 43 },
    { 0x109f74ef, 53 },
    { 0x996f74f0, 90 },
    { 0xa4af74f1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39ef74f6, 23 },
    { 0x531f74f6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4f74fd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf7500, 95 },
    { 0xbb4f7501, 90 },
    { 0, 0 },
    { 0x400f7503, 46 },
    { 0xa66f7504, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ff750c, 90 },
    { 0x87af750d, 34 },
    { 0x25bf750c, 90 },
    { 0xd41f750f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ef7515, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x671f7518, 84 },
    { 0, 0 },
    { 0x56cf751a, 90 },
    { 0x181f751a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9f7523, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf752a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x585f752d, 95 },
    { 0, 0 },
    { 0x7f9f752f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xda7f7532, 99 },
    { 0x154f7533, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1f753a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x347f7542, 33 },
    { 0x5e9f7542, 84 },
    { 0x954f7542, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3baf7548, 46 },
    { 0, 0 },
    { 0x871f754a, 90 },
    { 0x497f754b, 93 },
    { 0, 0 },
    { 0xfa7f754d, 98 },
    { 0, 0 },
    { 0x581f754f, 80 },
    { 0xf01f7550, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe5f7556, 83 },
    { 0x842f7556, 75 },
    { 0xd27f7556, 33 },
    { 0xf3ff7559, 33 },
    { 0xe3df7559, 95 },
    { 0, 0 },
    { 0x93ff755c, 17 },
    { 0x953f755c, 68 },
    { 0x585f755e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50f7563, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf7568, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85f7572, 33 },
    { 0x1c2f7573, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16f7576, 90 },
    { 0xfa4f7577, 95 },
    { 0x98cf7578, 61 },
    { 0xaf3f7578, 100 },
    { 0, 0 },
    { 0x63bf757b, 93 },
    { 0xd6ef757b, 23 },
    { 0x45f757d, 61 },
    { 0x50f757e, 46 },
    { 0x766f757f, 34 },
    { 0, 0 },
    { 0xb8df7581, 90 },
    { 0xe5ef7582, 33 },
    { 0x446f7581, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689f7588, 95 },
    { 0xc4ff7588, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29f758c, 33 },
    { 0, 0 },
    { 0xfc5f758e, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810f7593, 90 },
    { 0, 0 },
    { 0xa39f7595, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42f759a, 34 },
    { 0, 0 },
    { 0xd1cf759c, 81 },
    { 0xc9f759d, 61 },
    { 0xb9cf759c, 8 },
    { 0, 0 },
    { 0xe92f75a0, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc00f75a5, 98 },
    { 0, 0 },
    { 0xf6ef75a7, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4f75aa, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf75ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x571f75b4, 96 },
    { 0xd7ff75b5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ef75b9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218f75c4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e6f75c7, 45 },
    { 0xa8ff75c8, 95 },
    { 0xbedf75c8, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd5f75cc, 98 },
    { 0x9c1f75cc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33f75d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x888f75d4, 98 },
    { 0xa18f75d4, 76 },
    { 0xc47f75d4, 61 },
    { 0xd0cf75d4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3f75db, 55 },
    { 0x467f75db, 67 },
    { 0, 0 },
    { 0x4bef75de, 98 },
    { 0xe4bf75df, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28bf75e5, 100 },
    { 0x764f75e5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x830f75ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75ef75ef, 95 },
    { 0x79ef75ef, 61 },
    { 0xc6df75f1, 43 },
    { 0x92af75f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bf75f9, 80 },
    { 0x5f0f75f9, 46 },
    { 0x96f75fb, 55 },
    { 0, 0 },
    { 0x46df75fd, 61 },
    { 0x411f75fe, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf9f7602, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bf7607, 33 },
    { 0xf14f7607, 23 },
    { 0x9e4f7609, 33 },
    { 0xc28f760a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff7611, 55 },
    { 0, 0 },
    { 0x5df7613, 90 },
    { 0xe9df7614, 46 },
    { 0x2c1f7615, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f761d, 99 },
    { 0x157f761e, 48 },
    { 0x5c4f761e, 81 },
    { 0, 0 },
    { 0x1ebf7621, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8f7625, 52 },
    { 0, 0 },
    { 0xdff7627, 33 },
    { 0x61bf7628, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc58f762c, 34 },
    { 0, 0 },
    { 0x486f762e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xebaf7631, 90 },
    { 0x40f7631, 66 },
    { 0, 0 },
    { 0xffff7634, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94bf763a, 95 },
    { 0, 0 },
    { 0x420f763c, 78 },
    { 0x58af763d, 33 },
    { 0, 0 },
    { 0x69f763f, 95 },
    { 0, 0 },
    { 0x6c3f7641, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f7646, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f764d, 23 },
    { 0xc6bf764d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44ff7654, 95 },
    { 0, 0 },
    { 0x552f7656, 55 },
    { 0xd8cf7656, 61 },
    { 0x882f7656, 44 },
    { 0xe10f7659, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0df765c, 98 },
    { 0xdd5f765c, 90 },
    { 0x367f765e, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149f7662, 78 },
    { 0x165f7662, 75 },
    { 0xa5af7664, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef766a, 43 },
    { 0xff7f766a, 98 },
    { 0x2c7f766c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf7670, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ef7678, 76 },
    { 0x3b5f7679, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7f767f, 95 },
    { 0, 0 },
    { 0x549f7681, 98 },
    { 0x1b5f7681, 90 },
    { 0x6f0f7683, 55 },
    { 0, 0 },
    { 0xf14f7685, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd9f7688, 90 },
    { 0, 0 },
    { 0x5cdf768a, 55 },
    { 0x93af768a, 23 },
    { 0, 0 },
    { 0x6cbf768d, 100 },
    { 0, 0 },
    { 0x671f768f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dcf7692, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408f769d, 90 },
    { 0xfbef769d, 55 },
    { 0xe6bf769d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cf76a5, 81 },
    { 0, 0 },
    { 0x742f76a7, 55 },
    { 0x3d7f76a7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97f76af, 66 },
    { 0xd96f76af, 90 },
    { 0xb75f76b0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1f76b6, 61 },
    { 0x602f76b6, 76 },
    { 0x27cf76b7, 76 },
    { 0xbb2f76b9, 61 },
    { 0x24df76b9, 23 },
    { 0, 0 },
    { 0x8c7f76bc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e7f76c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685f76c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672f76cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ef76d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acf76dc, 24 },
    { 0, 0 },
    { 0xcf2f76de, 76 },
    { 0x6dff76df, 65 },
    { 0, 0 },
    { 0xb34f76e1, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef76e6, 55 },
    { 0x38cf76e7, 23 },
    { 0, 0 },
    { 0xda2f76e9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f76ed, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bf76f2, 78 },
    { 0x4def76f3, 46 },
    { 0x85ff76f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7f76f9, 34 },
    { 0x921f76fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x293f7704, 36 },
    { 0, 0 },
    { 0xfb1f7706, 33 },
    { 0x4d5f7706, 23 },
    { 0x476f7708, 61 },
    { 0xdbcf7709, 98 },
    { 0xbecf770a, 23 },
    { 0, 0 },
    { 0x9cdf770c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9f7714, 85 },
    { 0x96ef7715, 93 },
    { 0xd66f7715, 66 },
    { 0x366f7717, 61 },
    { 0, 0 },
    { 0x4fcf7719, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc66f771c, 88 },
    { 0x1bf771d, 33 },
    { 0x6b3f771e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ff7722, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0f7728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5f772b, 40 },
    { 0, 0 },
    { 0x3c5f772d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c9f7735, 34 },
    { 0, 0 },
    { 0x494f7737, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x842f773d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ef7741, 76 },
    { 0x4e5f7741, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82ff7746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf7749, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26bf7750, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7f7755, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf70f7758, 55 },
    { 0x2cf7758, 33 },
    { 0, 0 },
    { 0x8bdf775b, 33 },
    { 0, 0 },
    { 0x73df775d, 33 },
    { 0xa6df775e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x816f7762, 33 },
    { 0xc2cf7763, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bf7767, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x968f776a, 96 },
    { 0xe04f776b, 33 },
    { 0xdcff776c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdf7770, 93 },
    { 0, 0 },
    { 0x215f7772, 98 },
    { 0x7bbf7772, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x161f7778, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bf777e, 55 },
    { 0, 0 },
    { 0x17cf7780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96f7783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6df7788, 90 },
    { 0, 0 },
    { 0x20af778a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf778f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a7f7797, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4df779b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdff77a2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bf77a5, 97 },
    { 0xb9af77a5, 43 },
    { 0x310f77a5, 99 },
    { 0x552f77a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x386f77ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f77ae, 61 },
    { 0x9b0f77af, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa43f77b4, 33 },
    { 0xb9af77b5, 90 },
    { 0, 0 },
    { 0xbd8f77b7, 84 },
    { 0x813f77b8, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x839f77bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c0f77c2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aaf77c5, 23 },
    { 0x941f77c6, 23 },
    { 0, 0 },
    { 0x18ff77c8, 47 },
    { 0x761f77c9, 90 },
    { 0xa23f77c8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2f77cf, 95 },
    { 0x38ef77cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adf77d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x573f77e1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895f77e6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55f77eb, 61 },
    { 0x8df77ec, 95 },
    { 0xf04f77ed, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e7f77f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9f77f3, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658f77f8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69f77fb, 95 },
    { 0x61cf77fc, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c4f7803, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38f780b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f780f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x32af7812, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x235f7816, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b5f781b, 90 },
    { 0xcf9f781c, 68 },
    { 0xe9bf781d, 61 },
    { 0x4e5f781e, 98 },
    { 0x54f781f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x517f7825, 23 },
    { 0x2c4f7826, 46 },
    { 0, 0 },
    { 0xe8f7828, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62f782c, 55 },
    { 0, 0 },
    { 0xb62f782e, 75 },
    { 0xbfaf782f, 33 },
    { 0, 0 },
    { 0x28bf7831, 76 },
    { 0xae6f7832, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580f7836, 98 },
    { 0, 0 },
    { 0x62f7838, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x919f783b, 34 },
    { 0xcd3f783c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdf7842, 33 },
    { 0xecdf7843, 18 },
    { 0xa6af7844, 23 },
    { 0x4aaf7845, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405f784c, 61 },
    { 0xeeef784c, 23 },
    { 0xbe2f784e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf7851, 28 },
    { 0, 0 },
    { 0x2b3f7853, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d1f7858, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8df785f, 33 },
    { 0x3e8f7860, 84 },
    { 0x89af7861, 98 },
    { 0xb73f7862, 98 },
    { 0, 0 },
    { 0x9ccf7864, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3f7868, 90 },
    { 0xb6cf7869, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542f786e, 33 },
    { 0, 0 },
    { 0xf22f7870, 76 },
    { 0xc6ef7870, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bef7875, 95 },
    { 0xb80f7876, 65 },
    { 0xedff7876, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbf787e, 40 },
    { 0, 0 },
    { 0x746f7880, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe48f7884, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ef7887, 76 },
    { 0x3d1f7887, 61 },
    { 0, 0 },
    { 0x8d2f788a, 90 },
    { 0x2acf788b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadf7890, 90 },
    { 0x9d2f7891, 43 },
    { 0x683f7891, 99 },
    { 0xeb6f7893, 52 },
    { 0x29af7892, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x712f7897, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5f789e, 90 },
    { 0x3f8f789f, 23 },
    { 0x3f1f78a0, 33 },
    { 0xafcf78a1, 90 },
    { 0, 0 },
    { 0xe1df78a3, 61 },
    { 0x5b4f78a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x565f78a7, 90 },
    { 0x465f78a8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xccdf78ab, 23 },
    { 0, 0 },
    { 0x114f78ad, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff78b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x888f78b3, 33 },
    { 0, 0 },
    { 0xdddf78b5, 34 },
    { 0xdb6f78b6, 55 },
    { 0x5abf78b6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x317f78ba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bf78be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09f78c6, 61 },
    { 0, 0 },
    { 0x5f9f78c8, 98 },
    { 0, 0 },
    { 0xc51f78ca, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc04f78d3, 52 },
    { 0, 0 },
    { 0xec8f78d5, 33 },
    { 0xdb4f78d6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ff78dc, 34 },
    { 0xa6f78dd, 23 },
    { 0x7ff78de, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7f78e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacf78e8, 61 },
    { 0xcacf78e9, 52 },
    { 0x2eaf78e9, 98 },
    { 0xc72f78ea, 67 },
    { 0, 0 },
    { 0x440f78ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5f78f3, 33 },
    { 0, 0 },
    { 0xc56f78f5, 33 },
    { 0xe59f78f6, 77 },
    { 0x9c3f78f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cf78fa, 61 },
    { 0xdfbf78fb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5f78ff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f7906, 33 },
    { 0x221f7907, 93 },
    { 0xb63f7908, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638f790c, 90 },
    { 0x3faf790d, 23 },
    { 0x9b5f790d, 33 },
    { 0x3cef790f, 90 },
    { 0x52bf7910, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8f7913, 85 },
    { 0, 0 },
    { 0xd78f7915, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6f7919, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bf791c, 98 },
    { 0x80ff791d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cf7921, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20f7924, 95 },
    { 0, 0 },
    { 0xcbff7926, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x570f7929, 69 },
    { 0xe54f792a, 95 },
    { 0xd3f792a, 90 },
    { 0xaf5f792a, 61 },
    { 0xbe5f792d, 61 },
    { 0xabaf792d, 43 },
    { 0x10ef7929, 34 },
    { 0, 0 },
    { 0xb11f7931, 95 },
    { 0xbddf7932, 55 },
    { 0, 0 },
    { 0xeeff7934, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544f793a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e2f793d, 95 },
    { 0xaedf793d, 88 },
    { 0x354f793f, 83 },
    { 0xe4ef7940, 76 },
    { 0x9acf7941, 95 },
    { 0, 0 },
    { 0x45af7943, 61 },
    { 0x76af7943, 33 },
    { 0xf92f7945, 95 },
    { 0x184f7946, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42f794b, 90 },
    { 0x8b3f794c, 77 },
    { 0x5eaf794c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a0f7957, 69 },
    { 0, 0 },
    { 0x9f0f7959, 19 },
    { 0x50af795a, 95 },
    { 0x3cbf795a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179f795f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x92df7962, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bf7965, 55 },
    { 0xbd9f7966, 76 },
    { 0x547f7967, 90 },
    { 0x311f7968, 80 },
    { 0x128f7969, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6f796c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x69ff796f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dbf7972, 98 },
    { 0x50bf7972, 91 },
    { 0x198f7972, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d5f7979, 91 },
    { 0xadf797a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42f797e, 90 },
    { 0x8aaf797f, 55 },
    { 0, 0 },
    { 0xe44f7981, 90 },
    { 0x3faf7982, 95 },
    { 0x17cf7983, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6f798a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705f798e, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f7992, 99 },
    { 0x95ef7993, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aef7997, 43 },
    { 0, 0 },
    { 0xfccf7999, 52 },
    { 0x74f799a, 98 },
    { 0xd69f799b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cdf799e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x981f79a4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58cf79aa, 34 },
    { 0x742f79ab, 76 },
    { 0xe0bf79ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cf79b2, 55 },
    { 0, 0 },
    { 0x5f8f79b4, 98 },
    { 0, 0 },
    { 0x29ef79b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6ff79ba, 93 },
    { 0xbf4f79bb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bef79bf, 81 },
    { 0xe4af79c0, 85 },
    { 0, 0 },
    { 0x495f79c2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x966f79c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8f79cb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xde7f79ce, 55 },
    { 0x79af79ce, 23 },
    { 0xed8f79d0, 61 },
    { 0xfaff79d1, 77 },
    { 0x3bf79d1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x365f79d6, 73 },
    { 0xb1df79d7, 98 },
    { 0x36f79d7, 55 },
    { 0xc27f79d9, 34 },
    { 0xa21f79d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fef79dd, 66 },
    { 0, 0 },
    { 0x574f79df, 90 },
    { 0x208f79e0, 76 },
    { 0x28cf79e1, 87 },
    { 0x109f79e2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bcf79ea, 95 },
    { 0, 0 },
    { 0x447f79ec, 93 },
    { 0x830f79ed, 66 },
    { 0x6d5f79ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3f79f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67f79f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8f79fa, 9 },
    { 0, 0 },
    { 0x8ecf79fc, 85 },
    { 0x13cf79fd, 68 },
    { 0x793f79fe, 55 },
    { 0, 0 },
    { 0x77ef7a00, 84 },
    { 0x515f7a01, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x261f7a04, 33 },
    { 0, 0 },
    { 0xe1cf7a06, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x452f7a10, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa71f7a16, 17 },
    { 0x2c7f7a17, 95 },
    { 0, 0 },
    { 0x45df7a19, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f7a1d, 61 },
    { 0xf42f7a1e, 34 },
    { 0, 0 },
    { 0x502f7a20, 95 },
    { 0x638f7a21, 84 },
    { 0x6f6f7a22, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b8f7a26, 34 },
    { 0, 0 },
    { 0xc9df7a28, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f7a2f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4f7a32, 33 },
    { 0xb1ff7a33, 90 },
    { 0, 0 },
    { 0x118f7a35, 87 },
    { 0, 0 },
    { 0x79af7a37, 75 },
    { 0xbd9f7a38, 34 },
    { 0, 0 },
    { 0x341f7a3a, 23 },
    { 0xe72f7a3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11ef7a3f, 61 },
    { 0x4e8f7a40, 33 },
    { 0, 0 },
    { 0x599f7a42, 95 },
    { 0xca2f7a43, 23 },
    { 0x9b9f7a42, 61 },
    { 0x8e8f7a45, 52 },
    { 0xb5ff7a44, 76 },
    { 0x18bf7a47, 33 },
    { 0x9fef7a48, 91 },
    { 0xbe0f7a49, 61 },
    { 0, 0 },
    { 0x1c7f7a4b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70df7a4f, 76 },
    { 0x631f7a50, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3faf7a59, 43 },
    { 0, 0 },
    { 0x202f7a5b, 76 },
    { 0x825f7a5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54f7a5f, 90 },
    { 0x73df7a60, 88 },
    { 0x17bf7a60, 51 },
    { 0x477f7a62, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcf7a65, 33 },
    { 0x931f7a66, 61 },
    { 0, 0 },
    { 0xd18f7a68, 55 },
    { 0x7dbf7a69, 23 },
    { 0xab7f7a6a, 23 },
    { 0, 0 },
    { 0x840f7a6c, 95 },
    { 0x81bf7a6d, 67 },
    { 0x24ff7a6e, 34 },
    { 0x4cdf7a6f, 43 },
    { 0xe17f7a70, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e2f7a74, 90 },
    { 0x690f7a75, 46 },
    { 0x149f7a75, 52 },
    { 0, 0 },
    { 0x275f7a78, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cf7a7e, 33 },
    { 0x8f6f7a7f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xddff7a82, 23 },
    { 0xe00f7a83, 17 },
    { 0xdb1f7a84, 61 },
    { 0, 0 },
    { 0xdc9f7a86, 85 },
    { 0xc55f7a86, 90 },
    { 0, 0 },
    { 0xdc4f7a89, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bf7a8c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f7a95, 33 },
    { 0xbd2f7a95, 43 },
    { 0x7e3f7a97, 61 },
    { 0x8a7f7a98, 78 },
    { 0x7eef7a98, 95 },
    { 0x14f7a99, 85 },
    { 0x179f7a9b, 55 },
    { 0x9cbf7a9c, 95 },
    { 0x397f7a9d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864f7aa8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dbf7aad, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d6f7ab0, 90 },
    { 0, 0 },
    { 0x8a7f7ab2, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x26f7ab5, 84 },
    { 0, 0 },
    { 0x5c6f7ab7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff7abe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x877f7ac1, 33 },
    { 0, 0 },
    { 0xf94f7ac3, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35f7ac6, 33 },
    { 0x2f5f7ac6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85f7aca, 33 },
    { 0xb82f7acb, 98 },
    { 0x300f7acb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5edf7acf, 43 },
    { 0, 0 },
    { 0x118f7ad1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83af7ad5, 93 },
    { 0xabff7ad5, 23 },
    { 0x35af7ad7, 52 },
    { 0x836f7ad8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10af7adc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84f7ae2, 61 },
    { 0xebef7ae3, 93 },
    { 0, 0 },
    { 0xa56f7ae5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1f7ae9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef2f7aee, 46 },
    { 0xfc0f7aee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ff7af5, 23 },
    { 0x555f7af5, 55 },
    { 0xff9f7af7, 61 },
    { 0xc7cf7af7, 61 },
    { 0xc8bf7af9, 52 },
    { 0xb84f7afa, 55 },
    { 0x336f7afa, 76 },
    { 0x3b9f7afc, 76 },
    { 0xb3ef7afc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cf7b01, 95 },
    { 0x45f7b02, 90 },
    { 0xcf5f7b03, 90 },
    { 0xa8af7b02, 33 },
    { 0xb6f7b05, 34 },
    { 0xe88f7b06, 61 },
    { 0xa61f7b07, 90 },
    { 0x8eaf7b07, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0f7b0e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x240f7b11, 43 },
    { 0, 0 },
    { 0xcdef7b13, 90 },
    { 0, 0 },
    { 0x20ef7b15, 80 },
    { 0, 0 },
    { 0x3d2f7b17, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9f7b1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x122f7b1d, 34 },
    { 0, 0 },
    { 0x5e2f7b1f, 61 },
    { 0, 0 },
    { 0x7c4f7b21, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x746f7b26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf7b2a, 23 },
    { 0xa63f7b2a, 78 },
    { 0x3fdf7b2c, 70 },
    { 0xd0ff7b2b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5f7b34, 90 },
    { 0xe4ef7b34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x932f7b42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07f7b48, 46 },
    { 0xeecf7b49, 66 },
    { 0xa6ef7b4a, 52 },
    { 0x917f7b48, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df7b53, 81 },
    { 0x2a2f7b53, 69 },
    { 0x739f7b55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x604f7b59, 93 },
    { 0x525f7b5a, 51 },
    { 0x5ff7b5b, 88 },
    { 0x1d5f7b5b, 61 },
    { 0x9baf7b5b, 90 },
    { 0xc7af7b59, 81 },
    { 0xcfdf7b5f, 33 },
    { 0x9b1f7b60, 91 },
    { 0x39cf7b61, 23 },
    { 0, 0 },
    { 0xc0ff7b63, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6f7b68, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x786f7b6c, 90 },
    { 0, 0 },
    { 0x4f9f7b6e, 85 },
    { 0x95ef7b6f, 90 },
    { 0xb3bf7b6f, 90 },
    { 0x8fdf7b71, 95 },
    { 0x68ef7b72, 95 },
    { 0x3dcf7b73, 17 },
    { 0x62ef7b6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6acf7b78, 95 },
    { 0xd84f7b79, 23 },
    { 0xd98f7b7a, 33 },
    { 0xc11f7b79, 55 },
    { 0xa82f7b7b, 98 },
    { 0, 0 },
    { 0x5bdf7b7e, 98 },
    { 0x34af7b7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cf7b86, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e4f7b8b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcf7b8e, 23 },
    { 0x878f7b8f, 90 },
    { 0x190f7b8f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edf7b93, 100 },
    { 0, 0 },
    { 0x8dbf7b95, 91 },
    { 0, 0 },
    { 0x328f7b97, 95 },
    { 0x8f0f7b98, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb5f7b9e, 90 },
    { 0xe59f7b9f, 55 },
    { 0x702f7ba0, 91 },
    { 0x4d7f7ba1, 80 },
    { 0xb0ff7ba1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdcf7ba7, 76 },
    { 0xda2f7ba8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x702f7bab, 55 },
    { 0, 0 },
    { 0xc0f7bad, 61 },
    { 0xe63f7bae, 34 },
    { 0, 0 },
    { 0xce7f7bb0, 88 },
    { 0x841f7bb1, 95 },
    { 0xa15f7bb2, 76 },
    { 0x461f7bb3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbdf7bb7, 90 },
    { 0x532f7bb8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf7bbf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510f7bc3, 90 },
    { 0x8a9f7bc4, 33 },
    { 0x9c8f7bc4, 98 },
    { 0xfa9f7bc4, 95 },
    { 0x284f7bc5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdf7bca, 68 },
    { 0x380f7bca, 33 },
    { 0xf3f7bcc, 23 },
    { 0x90f7bcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37f7bd4, 76 },
    { 0x508f7bd4, 33 },
    { 0x716f7bd6, 99 },
    { 0x1cef7bd5, 90 },
    { 0xe9ff7bd8, 95 },
    { 0x5a5f7bd8, 93 },
    { 0xed5f7bd5, 61 },
    { 0x5f5f7bdb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x71cf7bde, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf7be3, 46 },
    { 0xd7df7be4, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5abf7beb, 94 },
    { 0, 0 },
    { 0x8a8f7bed, 55 },
    { 0xad2f7bed, 93 },
    { 0, 0 },
    { 0x27bf7bf0, 95 },
    { 0x25bf7bf0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf7bf4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f7bf7, 98 },
    { 0xdfdf7bf7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825f7c02, 98 },
    { 0x71cf7c02, 43 },
    { 0, 0 },
    { 0x26bf7c05, 95 },
    { 0xe8f7c06, 61 },
    { 0xdaaf7c06, 95 },
    { 0x7a5f7c06, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cf7c0e, 90 },
    { 0x1f1f7c0e, 23 },
    { 0xb64f7c10, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ef7c16, 90 },
    { 0, 0 },
    { 0xc6cf7c18, 51 },
    { 0xdb6f7c19, 23 },
    { 0, 0 },
    { 0x903f7c1b, 61 },
    { 0, 0 },
    { 0xf27f7c1d, 90 },
    { 0, 0 },
    { 0x542f7c1f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cff7c24, 68 },
    { 0xd62f7c25, 33 },
    { 0x6f1f7c26, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4f7c2c, 43 },
    { 0, 0 },
    { 0x6a1f7c2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7af7c31, 78 },
    { 0x954f7c32, 76 },
    { 0, 0 },
    { 0xc33f7c34, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0bf7c40, 8 },
    { 0, 0 },
    { 0x2dcf7c42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f7c45, 98 },
    { 0x7bff7c45, 100 },
    { 0x333f7c46, 95 },
    { 0x4bef7c45, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70bf7c52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fff7c55, 33 },
    { 0xbf3f7c56, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ef7c59, 98 },
    { 0x3dff7c59, 34 },
    { 0, 0 },
    { 0xf86f7c5c, 43 },
    { 0x94df7c5c, 61 },
    { 0x36df7c5e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x72f7c61, 55 },
    { 0, 0 },
    { 0x5f1f7c63, 91 },
    { 0xb68f7c63, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x844f7c67, 95 },
    { 0, 0 },
    { 0x11ff7c69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3df7c6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ef7c72, 47 },
    { 0x349f7c73, 84 },
    { 0, 0 },
    { 0x6a7f7c75, 76 },
    { 0xc66f7c76, 52 },
    { 0xd67f7c77, 93 },
    { 0x60ff7c78, 95 },
    { 0x41bf7c79, 76 },
    { 0x18ef7c77, 98 },
    { 0x147f7c7b, 95 },
    { 0x251f7c7c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3caf7c7f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x292f7c82, 90 },
    { 0, 0 },
    { 0xc3bf7c84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19f7c8a, 33 },
    { 0, 0 },
    { 0xb8ff7c8c, 90 },
    { 0x95af7c8d, 23 },
    { 0xe94f7c8e, 90 },
    { 0x324f7c8e, 88 },
    { 0xf70f7c90, 76 },
    { 0xe8df7c8e, 55 },
    { 0x197f7c90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cf7c96, 43 },
    { 0xdf0f7c96, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e6f7c9c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e5f7ca4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x472f7ca8, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ff7cad, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f7cb2, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30f7cb5, 99 },
    { 0, 0 },
    { 0x8abf7cb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb3f7cbc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ff7cbf, 55 },
    { 0x5a7f7cc0, 66 },
    { 0, 0 },
    { 0x171f7cc2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9f7cc7, 33 },
    { 0x3dff7cc8, 33 },
    { 0x9f8f7cc9, 98 },
    { 0xdd2f7cc7, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcaf7ccf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbf7cdc, 90 },
    { 0xe88f7cdc, 52 },
    { 0x762f7cde, 55 },
    { 0x5f4f7cde, 33 },
    { 0x51ef7cde, 17 },
    { 0x3a2f7cdc, 99 },
    { 0x91f7ce2, 76 },
    { 0x72ef7ce3, 81 },
    { 0x5b9f7ce4, 55 },
    { 0xa97f7ce4, 24 },
    { 0x6c5f7ce6, 38 },
    { 0x7f8f7ce4, 90 },
    { 0x1aaf7cdd, 45 },
    { 0x856f7ce6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5f7cec, 93 },
    { 0, 0 },
    { 0xe87f7cee, 94 },
    { 0x1f4f7cee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9f7cf4, 34 },
    { 0, 0 },
    { 0x819f7cf6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x671f7cfa, 85 },
    { 0xe07f7cfa, 61 },
    { 0x1d2f7cfc, 33 },
    { 0x7af7cfd, 61 },
    { 0, 0 },
    { 0xb55f7cff, 76 },
    { 0xf11f7d00, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x383f7d04, 98 },
    { 0, 0 },
    { 0x8fbf7d06, 23 },
    { 0x6d8f7d07, 91 },
    { 0, 0 },
    { 0xf4ff7d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecf7d0f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xd58f7d12, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72df7d17, 95 },
    { 0xeb9f7d18, 90 },
    { 0, 0 },
    { 0x69df7d1a, 90 },
    { 0, 0 },
    { 0x5e7f7d1c, 43 },
    { 0xfb4f7d1d, 95 },
    { 0xd8ef7d1d, 90 },
    { 0x631f7d1f, 61 },
    { 0xc98f7d20, 75 },
    { 0x24bf7d21, 98 },
    { 0, 0 },
    { 0xa71f7d23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb1f7d27, 90 },
    { 0, 0 },
    { 0x96ef7d29, 76 },
    { 0x8fcf7d2a, 61 },
    { 0xbcbf7d2a, 66 },
    { 0x44cf7d2c, 76 },
    { 0x723f7d2d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8f7d30, 48 },
    { 0x504f7d30, 95 },
    { 0x9cff7d30, 23 },
    { 0, 0 },
    { 0x895f7d34, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaaf7d38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x7def7d3b, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfef7d40, 55 },
    { 0x63df7d40, 84 },
    { 0, 0 },
    { 0xa94f7d43, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aef7d46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8f7d49, 98 },
    { 0, 0 },
    { 0x1e8f7d4b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff7d50, 22 },
    { 0x4f0f7d50, 33 },
    { 0xb38f7d50, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf7d55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaedf7d5c, 61 },
    { 0xc33f7d5d, 61 },
    { 0x39df7d5c, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe52f7d61, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x489f7d64, 90 },
    { 0xdbbf7d65, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe28f7d69, 61 },
    { 0x22df7d6a, 61 },
    { 0, 0 },
    { 0x592f7d6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf7d74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00f7d78, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd1f7d7b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f7d7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef7d86, 76 },
    { 0x6aff7d87, 61 },
    { 0x566f7d87, 43 },
    { 0, 0 },
    { 0x31f7d8a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157f7d92, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xf92f7d95, 33 },
    { 0x7e1f7d96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f7d99, 90 },
    { 0xc2ef7d99, 100 },
    { 0, 0 },
    { 0x76cf7d9c, 90 },
    { 0, 0 },
    { 0xc7ff7d9e, 98 },
    { 0, 0 },
    { 0xa30f7da0, 93 },
    { 0xe59f7da1, 84 },
    { 0, 0 },
    { 0x3f0f7da3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x722f7da6, 34 },
    { 0x87af7da7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x159f7db0, 46 },
    { 0, 0 },
    { 0xa81f7db2, 23 },
    { 0xd8ff7db3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0f7db6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89af7dbe, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x69f7dc1, 98 },
    { 0x3a1f7dc2, 33 },
    { 0, 0 },
    { 0x59df7dc4, 34 },
    { 0x632f7dc4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e9f7dca, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x970f7dce, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x445f7dd1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x420f7dd4, 77 },
    { 0xd24f7dd4, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16df7dd9, 23 },
    { 0, 0 },
    { 0xb13f7ddb, 78 },
    { 0x763f7ddc, 90 },
    { 0xb60f7ddc, 76 },
    { 0x8b8f7dde, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23f7de1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x477f7de5, 34 },
    { 0x5a4f7de6, 55 },
    { 0, 0 },
    { 0x921f7de8, 97 },
    { 0x170f7de9, 95 },
    { 0, 0 },
    { 0x44ef7deb, 90 },
    { 0xb6ff7dec, 67 },
    { 0xf7bf7ded, 98 },
    { 0x98cf7dee, 95 },
    { 0x3af7dee, 34 },
    { 0xda2f7df0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x639f7df6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df7dfa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f7dfd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc08f7e00, 90 },
    { 0x866f7e01, 61 },
    { 0x286f7e02, 33 },
    { 0x14af7e02, 90 },
    { 0x801f7e02, 61 },
    { 0x68af7e05, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86df7e0c, 90 },
    { 0, 0 },
    { 0xa24f7e0e, 52 },
    { 0x598f7e0f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe4f7e12, 23 },
    { 0x7f1f7e13, 33 },
    { 0x453f7e13, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3f7e22, 61 },
    { 0xd0ef7e23, 55 },
    { 0xbedf7e24, 34 },
    { 0x323f7e25, 68 },
    { 0xda4f7e26, 90 },
    { 0x2d4f7e22, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf7e2a, 43 },
    { 0, 0 },
    { 0x591f7e2c, 95 },
    { 0, 0 },
    { 0x323f7e2e, 90 },
    { 0, 0 },
    { 0x4c9f7e30, 61 },
    { 0xa79f7e31, 33 },
    { 0, 0 },
    { 0xc3ef7e33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44df7e37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f0f7e3a, 91 },
    { 0xf5f7e3a, 67 },
    { 0x370f7e3c, 68 },
    { 0, 0 },
    { 0x489f7e3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1bf7e46, 34 },
    { 0xfaff7e47, 90 },
    { 0xa6df7e48, 98 },
    { 0, 0 },
    { 0x9d8f7e4a, 61 },
    { 0, 0 },
    { 0x628f7e4c, 34 },
    { 0x879f7e4d, 78 },
    { 0xdb0f7e4e, 23 },
    { 0x2c9f7e4e, 95 },
    { 0xe97f7e4d, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb9f7e56, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f7e5d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cf7e60, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x405f7e65, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf68f7e6c, 43 },
    { 0, 0 },
    { 0xabaf7e6e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ff7e75, 95 },
    { 0, 0 },
    { 0x774f7e77, 33 },
    { 0, 0 },
    { 0xf7af7e79, 83 },
    { 0xacff7e7a, 46 },
    { 0x6a0f7e7b, 76 },
    { 0x6f3f7e7c, 78 },
    { 0x9d5f7e7c, 98 },
    { 0xda7f7e7e, 91 },
    { 0xe7f7e7f, 23 },
    { 0x2d2f7e80, 95 },
    { 0x23af7e7b, 61 },
    { 0x7a3f7e79, 100 },
    { 0xbf8f7e83, 77 },
    { 0, 0 },
    { 0x71ff7e85, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f7e89, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa6f7e8c, 55 },
    { 0x577f7e8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbff7e91, 95 },
    { 0xe57f7e92, 33 },
    { 0x3e2f7e93, 23 },
    { 0x65ef7e94, 58 },
    { 0, 0 },
    { 0xcbbf7e96, 33 },
    { 0, 0 },
    { 0x280f7e98, 33 },
    { 0, 0 },
    { 0xc2bf7e9a, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xfabf7e9d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x831f7ea0, 56 },
    { 0xe71f7ea0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d7f7ea5, 95 },
    { 0x87bf7ea6, 87 },
    { 0x976f7ea7, 93 },
    { 0x634f7ea8, 23 },
    { 0xad7f7ea7, 78 },
    { 0x19af7eaa, 23 },
    { 0, 0 },
    { 0x13cf7eac, 77 },
    { 0x648f7eac, 33 },
    { 0x8dff7ead, 23 },
    { 0, 0 },
    { 0x702f7eb0, 76 },
    { 0xfdaf7eb1, 55 },
    { 0xef7f7eb1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abf7eb5, 95 },
    { 0x6a7f7eb6, 98 },
    { 0x51af7eb6, 90 },
    { 0, 0 },
    { 0xc7cf7eb9, 90 },
    { 0x77bf7eb9, 76 },
    { 0, 0 },
    { 0xd89f7ebc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389f7ec0, 53 },
    { 0, 0 },
    { 0x1c5f7ec2, 17 },
    { 0xec6f7ec2, 52 },
    { 0x198f7ec3, 78 },
    { 0x20bf7ec2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ef7ecd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x731f7ed1, 55 },
    { 0, 0 },
    { 0xc9bf7ed3, 78 },
    { 0, 0 },
    { 0xbc8f7ed5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3f7ed9, 95 },
    { 0, 0 },
    { 0x4d7f7edb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa39f7ee0, 95 },
    { 0xbf3f7ee1, 90 },
    { 0, 0 },
    { 0x811f7ee3, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x777f7ee8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde3f7eef, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xd24f7ef2, 88 },
    { 0x481f7ef3, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43f7efa, 90 },
    { 0x4a6f7efa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x69df7efe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf7f01, 90 },
    { 0xd4ef7f02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf7f08, 98 },
    { 0x5aff7f09, 34 },
    { 0x6b4f7f0a, 84 },
    { 0xe0bf7f08, 90 },
    { 0, 0 },
    { 0x5bf7f0d, 33 },
    { 0xdaaf7f0e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef7f11, 98 },
    { 0x378f7f12, 90 },
    { 0, 0 },
    { 0xee4f7f14, 90 },
    { 0xed5f7f15, 66 },
    { 0x98cf7f14, 90 },
    { 0xed2f7f15, 99 },
    { 0x7bbf7f18, 90 },
    { 0x6bcf7f15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabbf7f1e, 98 },
    { 0x2e1f7f1e, 98 },
    { 0xce8f7f20, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf7f24, 34 },
    { 0, 0 },
    { 0xd63f7f26, 93 },
    { 0xbd5f7f27, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8f7f35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x814f7f38, 80 },
    { 0x168f7f39, 34 },
    { 0xf72f7f39, 95 },
    { 0x717f7f3b, 84 },
    { 0xcd9f7f3c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef7f3f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e0f7f43, 90 },
    { 0x722f7f44, 23 },
    { 0xb98f7f45, 64 },
    { 0xd67f7f46, 34 },
    { 0xc06f7f44, 46 },
    { 0x152f7f48, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5bf7f4b, 52 },
    { 0xe63f7f4b, 95 },
    { 0xa1ef7f4c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baf7f54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x157f7f57, 46 },
    { 0, 0 },
    { 0x145f7f59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1faf7f5e, 90 },
    { 0, 0 },
    { 0x9f8f7f60, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711f7f64, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462f7f6a, 61 },
    { 0, 0 },
    { 0xe46f7f6c, 98 },
    { 0, 0 },
    { 0x681f7f6e, 100 },
    { 0, 0 },
    { 0xeaff7f70, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fef7f76, 33 },
    { 0x4b6f7f77, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bf7f7a, 98 },
    { 0, 0 },
    { 0xb14f7f7c, 95 },
    { 0xbd2f7f7d, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x492f7f80, 61 },
    { 0x496f7f80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3f7f84, 55 },
    { 0xeb8f7f84, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f7f8c, 90 },
    { 0x695f7f8d, 95 },
    { 0xc19f7f8d, 85 },
    { 0x372f7f8c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcf7f92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5f7f98, 84 },
    { 0x807f7f98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c4f7f9f, 90 },
    { 0xb59f7fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e0f7fa3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2f7fa6, 66 },
    { 0x1dbf7fa7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eef7fab, 99 },
    { 0, 0 },
    { 0xe48f7fad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486f7fb2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaf7fb5, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd19f7fb8, 95 },
    { 0x8fdf7fb9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aaf7fbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff7fc2, 90 },
    { 0x147f7fc3, 66 },
    { 0xeedf7fc3, 95 },
    { 0x499f7fc3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xedef7fc8, 100 },
    { 0xd4f7fc8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e2f7fcc, 65 },
    { 0, 0 },
    { 0x546f7fce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3f7fd4, 90 },
    { 0xadcf7fd5, 96 },
    { 0xf78f7fd5, 90 },
    { 0, 0 },
    { 0xe03f7fd8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ff7fdb, 90 },
    { 0x94cf7fdc, 61 },
    { 0xc5ef7fdd, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5df7fe0, 61 },
    { 0, 0 },
    { 0xadf7fe2, 33 },
    { 0x4eaf7fe3, 90 },
    { 0, 0 },
    { 0x87df7fe5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26df7feb, 98 },
    { 0x96cf7feb, 76 },
    { 0xb2af7feb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x602f7ffa, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7f8002, 55 },
    { 0, 0 },
    { 0x4a2f8004, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767f8009, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45f800c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaff8010, 98 },
    { 0x7f8f8010, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0f8014, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x358f8017, 61 },
    { 0x78bf8018, 46 },
    { 0xb5bf8019, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98f801d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ef8020, 55 },
    { 0xd46f8021, 95 },
    { 0x5b1f8022, 78 },
    { 0xdf6f8020, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cf8027, 81 },
    { 0x186f8027, 36 },
    { 0, 0 },
    { 0xe35f802a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f802e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7f8032, 66 },
    { 0x96df8033, 42 },
    { 0x6a7f8034, 43 },
    { 0xc8ff8035, 90 },
    { 0, 0 },
    { 0xbc8f8037, 90 },
    { 0xe0ff8037, 98 },
    { 0x381f8037, 99 },
    { 0xce6f8037, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x623f803d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf33f8040, 33 },
    { 0x25ef8040, 95 },
    { 0x678f8042, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd60f8047, 98 },
    { 0x256f8048, 90 },
    { 0x441f8049, 95 },
    { 0x6d9f804a, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f804e, 90 },
    { 0, 0 },
    { 0x33f8050, 23 },
    { 0x346f8051, 90 },
    { 0xba9f8052, 90 },
    { 0x25bf8051, 61 },
    { 0x58cf8051, 98 },
    { 0x413f8055, 33 },
    { 0x9c9f8056, 33 },
    { 0x282f8057, 52 },
    { 0x6d1f8056, 98 },
    { 0xe6df8051, 93 },
    { 0x233f805a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d0f8065, 90 },
    { 0, 0 },
    { 0xff6f8067, 90 },
    { 0, 0 },
    { 0x4aef8069, 91 },
    { 0x514f8069, 90 },
    { 0xb6ff806b, 85 },
    { 0x912f8069, 95 },
    { 0, 0 },
    { 0xe21f806e, 34 },
    { 0x863f806f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f8074, 90 },
    { 0xfc7f8075, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x113f8079, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff8081, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b9f8088, 90 },
    { 0x593f8088, 61 },
    { 0x771f808a, 75 },
    { 0, 0 },
    { 0x51f808c, 23 },
    { 0xd18f808c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebf8090, 84 },
    { 0x435f8090, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491f8095, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cf80a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x507f80a4, 100 },
    { 0xb5af80a5, 98 },
    { 0x151f80a6, 90 },
    { 0xe78f80a7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd57f80ad, 33 },
    { 0xe70f80ae, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdf80b1, 33 },
    { 0x4a8f80b1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360f80bd, 52 },
    { 0x598f80be, 98 },
    { 0x640f80be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ef80c5, 23 },
    { 0, 0 },
    { 0x7ddf80c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14f80cb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5f80ce, 17 },
    { 0, 0 },
    { 0xba5f80d0, 98 },
    { 0x48bf80d1, 33 },
    { 0, 0 },
    { 0x89df80d3, 100 },
    { 0xf71f80d4, 78 },
    { 0xf68f80d4, 55 },
    { 0xf7ef80d4, 66 },
    { 0, 0 },
    { 0x286f80d8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x561f80db, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af80de, 97 },
    { 0x69df80df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f80e4, 67 },
    { 0x678f80e5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc4f80e8, 55 },
    { 0xf7bf80e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f80ec, 100 },
    { 0, 0 },
    { 0x5ef80ee, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f80f2, 77 },
    { 0x92bf80f3, 23 },
    { 0xaa0f80f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99f80f9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aef8101, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2f810e, 98 },
    { 0, 0 },
    { 0xea4f8110, 90 },
    { 0xcb7f8111, 88 },
    { 0x3a8f8112, 52 },
    { 0xbdaf8113, 90 },
    { 0xde4f8110, 95 },
    { 0xe00f8114, 34 },
    { 0x68af8112, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf811d, 34 },
    { 0xd53f811e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf51f8121, 66 },
    { 0x6c4f8121, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9f8126, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc57f812b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x476f812e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ff8131, 98 },
    { 0xf6f8132, 33 },
    { 0x8fef8131, 95 },
    { 0xc05f8133, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xab6f8137, 61 },
    { 0x5e4f8137, 23 },
    { 0x3bcf8138, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x94af813c, 33 },
    { 0x42f813d, 81 },
    { 0xc49f813e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661f814a, 34 },
    { 0x8f0f814b, 87 },
    { 0xe22f814b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24f814f, 66 },
    { 0x90cf8150, 99 },
    { 0, 0 },
    { 0x149f8152, 43 },
    { 0xea2f8153, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x976f8157, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x385f815b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x110f815e, 90 },
    { 0x831f815f, 84 },
    { 0x958f8160, 66 },
    { 0x4a1f8161, 78 },
    { 0xc71f8162, 98 },
    { 0, 0 },
    { 0xce5f8164, 34 },
    { 0xad7f8164, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x576f8168, 76 },
    { 0x362f8168, 76 },
    { 0xea8f816a, 90 },
    { 0x193f8168, 96 },
    { 0xa1df816c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1f8175, 33 },
    { 0, 0 },
    { 0xe7bf8177, 43 },
    { 0, 0 },
    { 0xf5af8179, 81 },
    { 0, 0 },
    { 0x9adf817b, 55 },
    { 0x32ff817c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc6f817f, 90 },
    { 0x182f8180, 76 },
    { 0xc60f8180, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe42f8187, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x844f818b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170f818f, 85 },
    { 0xf0af818f, 100 },
    { 0x205f8191, 98 },
    { 0, 0 },
    { 0x86ff8193, 95 },
    { 0, 0 },
    { 0xbedf8195, 46 },
    { 0xa7cf8195, 23 },
    { 0xc17f8197, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x296f819d, 33 },
    { 0x248f819d, 98 },
    { 0xb6cf819d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd8f81a2, 98 },
    { 0xbbdf81a2, 69 },
    { 0xdcff81a3, 95 },
    { 0, 0 },
    { 0x826f81a6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a8f81ab, 55 },
    { 0, 0 },
    { 0x4d8f81ad, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x950f81b1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bdf81b4, 43 },
    { 0, 0 },
    { 0x9e6f81b6, 55 },
    { 0x764f81b6, 75 },
    { 0x13bf81b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba6f81d3, 100 },
    { 0xd3f81d3, 29 },
    { 0x6cff81d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546f81d9, 87 },
    { 0x68cf81d9, 61 },
    { 0, 0 },
    { 0x166f81dc, 90 },
    { 0x582f81dc, 98 },
    { 0xd2ff81dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8f81e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb18f81e8, 95 },
    { 0x84cf81e8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x91f81ec, 90 },
    { 0x8e5f81ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ef81f0, 33 },
    { 0x10af81f0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8def81f5, 61 },
    { 0, 0 },
    { 0xcc7f81f7, 23 },
    { 0x7cef81f8, 61 },
    { 0, 0 },
    { 0xd9ff81fa, 76 },
    { 0, 0 },
    { 0xd19f81fc, 46 },
    { 0, 0 },
    { 0x934f81fe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee2f8205, 95 },
    { 0, 0 },
    { 0xf45f8207, 78 },
    { 0x80ef8207, 98 },
    { 0xd19f8209, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ef820e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f8214, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9f8218, 90 },
    { 0x64af8219, 36 },
    { 0, 0 },
    { 0x6b5f821b, 34 },
    { 0x4aaf821c, 40 },
    { 0x62f821c, 23 },
    { 0x86af821e, 95 },
    { 0xd4ff821f, 23 },
    { 0x683f821e, 84 },
    { 0x35bf8221, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x869f8224, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x54bf8227, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cf822e, 23 },
    { 0x529f822f, 65 },
    { 0xe69f8230, 81 },
    { 0, 0 },
    { 0x791f8232, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bf8238, 100 },
    { 0x2d1f8238, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d4f823c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb26f8240, 90 },
    { 0, 0 },
    { 0xeb5f8242, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbff8247, 76 },
    { 0x75cf8247, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0f824e, 43 },
    { 0xe1df824f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6f8252, 34 },
    { 0xc4af8253, 90 },
    { 0x90f8254, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51f8257, 90 },
    { 0, 0 },
    { 0x844f8259, 69 },
    { 0x8ecf8259, 33 },
    { 0xf3ef825b, 67 },
    { 0xe60f825c, 100 },
    { 0, 0 },
    { 0x976f825e, 46 },
    { 0xe54f825f, 61 },
    { 0xc1af8260, 98 },
    { 0x756f8261, 33 },
    { 0x25df8262, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4f8265, 76 },
    { 0, 0 },
    { 0x106f8267, 68 },
    { 0x856f8267, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cff826b, 76 },
    { 0, 0 },
    { 0xac4f826d, 98 },
    { 0, 0 },
    { 0xa6af826f, 23 },
    { 0xa3ff8270, 34 },
    { 0xb07f8270, 55 },
    { 0xb60f8270, 90 },
    { 0x956f8273, 67 },
    { 0xc34f8270, 33 },
    { 0x8f5f8270, 98 },
    { 0xc8cf826f, 90 },
    { 0, 0 },
    { 0x43cf8278, 66 },
    { 0, 0 },
    { 0xd74f827a, 95 },
    { 0x185f827b, 65 },
    { 0xd54f827c, 23 },
    { 0x87df827c, 40 },
    { 0xe87f827b, 33 },
    { 0, 0 },
    { 0x31af8280, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cf8285, 23 },
    { 0xd96f8286, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80f828a, 66 },
    { 0x365f828a, 33 },
    { 0, 0 },
    { 0x7eff828d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52df8292, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x349f8296, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917f829a, 95 },
    { 0xd9f829b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f829f, 76 },
    { 0, 0 },
    { 0xc86f82a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13f82a5, 52 },
    { 0xfedf82a5, 94 },
    { 0xb09f82a7, 61 },
    { 0xa17f82a8, 85 },
    { 0, 0 },
    { 0x853f82aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95df82af, 75 },
    { 0x9c5f82b0, 90 },
    { 0, 0 },
    { 0xc9ff82b2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74f82b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55f82b9, 38 },
    { 0, 0 },
    { 0x1dff82bb, 84 },
    { 0, 0 },
    { 0x1abf82bd, 23 },
    { 0xd8bf82be, 93 },
    { 0, 0 },
    { 0x794f82c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40f82c3, 33 },
    { 0x68bf82c3, 96 },
    { 0xa47f82c5, 90 },
    { 0xef5f82c4, 81 },
    { 0, 0 },
    { 0x513f82c8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905f82ce, 43 },
    { 0x435f82cf, 93 },
    { 0x87bf82cf, 90 },
    { 0x3b5f82d0, 33 },
    { 0x3eff82ce, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca0f82d6, 93 },
    { 0, 0 },
    { 0x824f82d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdf82dc, 93 },
    { 0xd56f82dd, 33 },
    { 0x430f82de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344f82e3, 90 },
    { 0xf82f82e4, 65 },
    { 0, 0 },
    { 0x672f82e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x481f82e9, 91 },
    { 0, 0 },
    { 0x476f82eb, 76 },
    { 0, 0 },
    { 0xe06f82ed, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0bf82f0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3f82f3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x582f82f6, 55 },
    { 0, 0 },
    { 0xec6f82f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8faf82fb, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f8300, 78 },
    { 0x40ef8300, 23 },
    { 0x859f8302, 91 },
    { 0xa2af8303, 61 },
    { 0xcd3f8301, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd2f830d, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152f8312, 68 },
    { 0x7b7f8313, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc47f8317, 34 },
    { 0xd86f8318, 76 },
    { 0, 0 },
    { 0x669f831a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x999f8320, 61 },
    { 0x71ff8320, 100 },
    { 0xae4f8322, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x923f8329, 33 },
    { 0x870f832a, 98 },
    { 0x78bf832b, 61 },
    { 0xffff832a, 33 },
    { 0x397f832d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ef8334, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x694f8338, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x581f833b, 61 },
    { 0, 0 },
    { 0xf37f833d, 23 },
    { 0xb61f833e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cf8341, 61 },
    { 0x199f8342, 90 },
    { 0, 0 },
    { 0xb56f8344, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef8349, 85 },
    { 0, 0 },
    { 0x84bf834b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cf8350, 17 },
    { 0x3fdf8351, 95 },
    { 0x74df8352, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccf8356, 14 },
    { 0x3d7f8357, 98 },
    { 0x8d0f8357, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3ff835e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bf8366, 78 },
    { 0x3ef8366, 88 },
    { 0x935f8368, 77 },
    { 0xba9f8369, 23 },
    { 0, 0 },
    { 0xb6af836b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf4f836f, 87 },
    { 0, 0 },
    { 0x7caf8371, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcdf8376, 61 },
    { 0, 0 },
    { 0xe28f8378, 90 },
    { 0x7cbf8378, 90 },
    { 0xb78f837a, 52 },
    { 0xa51f837a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eaf837f, 81 },
    { 0, 0 },
    { 0x5cdf8381, 33 },
    { 0xcc1f8382, 33 },
    { 0x5a4f8383, 33 },
    { 0x9b8f8381, 34 },
    { 0xaabf8385, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eff8388, 66 },
    { 0, 0 },
    { 0xa71f838a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaf838d, 91 },
    { 0xe81f838e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9f839c, 23 },
    { 0, 0 },
    { 0x1edf839e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f83a6, 90 },
    { 0xa32f83a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e5f83aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x506f83ad, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93af83b4, 90 },
    { 0, 0 },
    { 0xd9ff83b6, 100 },
    { 0xc3ff83b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cf83ba, 66 },
    { 0, 0 },
    { 0xc8cf83bc, 90 },
    { 0x1c9f83bd, 33 },
    { 0x506f83be, 85 },
    { 0xab5f83be, 19 },
    { 0x180f83be, 69 },
    { 0x14ff83be, 91 },
    { 0, 0 },
    { 0x8b1f83c3, 98 },
    { 0xa41f83c4, 46 },
    { 0xdcef83c5, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cf83cb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefdf83d1, 98 },
    { 0x6e5f83d2, 33 },
    { 0x446f83d3, 90 },
    { 0x593f83d4, 98 },
    { 0xdd0f83d5, 52 },
    { 0, 0 },
    { 0x97af83d7, 88 },
    { 0xdd8f83d8, 95 },
    { 0x8e4f83d8, 76 },
    { 0x73ff83d8, 95 },
    { 0xe26f83d7, 61 },
    { 0xb5df83da, 61 },
    { 0, 0 },
    { 0x32bf83de, 90 },
    { 0x4d6f83df, 23 },
    { 0x9a0f83e0, 90 },
    { 0xb0ff83e0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28f83e4, 77 },
    { 0, 0 },
    { 0x158f83e6, 51 },
    { 0, 0 },
    { 0x3daf83e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6f83ee, 76 },
    { 0, 0 },
    { 0xd08f83f0, 82 },
    { 0x73df83f1, 91 },
    { 0xed6f83f0, 90 },
    { 0x14f83f2, 17 },
    { 0x396f83f4, 76 },
    { 0xe4bf83f5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf94f83fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3df8400, 10 },
    { 0x55f8400, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f8408, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacf8411, 90 },
    { 0xd81f8412, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874f8417, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x630f841a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94f841d, 76 },
    { 0x6fdf841d, 98 },
    { 0xa40f841d, 33 },
    { 0x3c6f841d, 55 },
    { 0xe5af8421, 33 },
    { 0x1c9f8422, 33 },
    { 0xfeff8421, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f8428, 81 },
    { 0x75f8429, 98 },
    { 0xf1df8429, 43 },
    { 0, 0 },
    { 0x772f842c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543f8430, 33 },
    { 0x9eff8431, 33 },
    { 0, 0 },
    { 0x797f8433, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca1f8437, 98 },
    { 0xc89f8438, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ef843b, 23 },
    { 0x87ef843c, 52 },
    { 0x8cdf843c, 75 },
    { 0x88ef843e, 76 },
    { 0xa17f843d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32f844e, 83 },
    { 0x1e4f844f, 90 },
    { 0, 0 },
    { 0x5e6f8451, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcf8454, 81 },
    { 0x98cf8455, 90 },
    { 0x9d8f8454, 61 },
    { 0x590f8455, 93 },
    { 0, 0 },
    { 0xb95f8459, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4df8461, 81 },
    { 0x876f8461, 33 },
    { 0x4c5f8463, 88 },
    { 0xc73f8464, 90 },
    { 0x211f8465, 95 },
    { 0xa4bf8465, 90 },
    { 0x862f8467, 52 },
    { 0xde9f8467, 34 },
    { 0x7c7f8467, 95 },
    { 0x8acf8461, 23 },
    { 0, 0 },
    { 0x10bf846c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bf8471, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0af8474, 99 },
    { 0x2a0f8475, 66 },
    { 0xd58f8475, 84 },
    { 0x475f8477, 90 },
    { 0x9acf8477, 87 },
    { 0, 0 },
    { 0x9a8f847a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eff847d, 52 },
    { 0xbddf847e, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0f8482, 76 },
    { 0, 0 },
    { 0x305f8484, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x325f8487, 98 },
    { 0xcecf8488, 71 },
    { 0x8a6f8489, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bf848c, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f8493, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cf8496, 95 },
    { 0x8e0f8496, 100 },
    { 0, 0 },
    { 0x784f8499, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9f849d, 46 },
    { 0, 0 },
    { 0x397f849f, 87 },
    { 0, 0 },
    { 0xd81f84a1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x312f84a9, 23 },
    { 0x197f84aa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2af84af, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51af84b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594f84bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8bf84be, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x330f84c2, 90 },
    { 0xecbf84c3, 81 },
    { 0x6a9f84c3, 61 },
    { 0xc2ef84c4, 76 },
    { 0, 0 },
    { 0x115f84c7, 98 },
    { 0, 0 },
    { 0x2f3f84c9, 61 },
    { 0xc29f84c9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888f84ce, 75 },
    { 0, 0 },
    { 0x72bf84d0, 34 },
    { 0xed5f84d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b4f84d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f84dc, 61 },
    { 0xc3bf84dd, 23 },
    { 0x706f84de, 95 },
    { 0x8d0f84df, 33 },
    { 0x82cf84e0, 55 },
    { 0x4e1f84df, 98 },
    { 0xf2af84e1, 34 },
    { 0x2f6f84e1, 100 },
    { 0x240f84e1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x584f84e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdf84ea, 33 },
    { 0, 0 },
    { 0x66bf84ec, 85 },
    { 0x552f84ed, 98 },
    { 0xe2df84ee, 98 },
    { 0x539f84ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7faf84f2, 68 },
    { 0x157f84f3, 98 },
    { 0xaa4f84f3, 93 },
    { 0xd8df84f2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f6f84f9, 92 },
    { 0xce9f84f9, 46 },
    { 0, 0 },
    { 0x91ef84fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88f8503, 87 },
    { 0xe9f8504, 43 },
    { 0x3a7f8503, 96 },
    { 0x8d3f8503, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ef850c, 33 },
    { 0x62f850d, 34 },
    { 0xd56f850c, 23 },
    { 0x8ef850f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x241f8514, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa62f8518, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d9f851b, 98 },
    { 0x44ff851b, 97 },
    { 0xc7ef851b, 61 },
    { 0x494f851b, 98 },
    { 0xe74f851c, 91 },
    { 0x2f9f851e, 85 },
    { 0x2e9f8521, 23 },
    { 0x26bf8522, 93 },
    { 0x5f9f8523, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf8526, 95 },
    { 0xc2ef8526, 35 },
    { 0xcf9f8528, 55 },
    { 0xaaaf8528, 90 },
    { 0xd5ef852a, 98 },
    { 0xba3f8529, 99 },
    { 0, 0 },
    { 0x58af852d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27f8532, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dff8540, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f8545, 61 },
    { 0, 0 },
    { 0x7f6f8547, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cf854b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49f8550, 80 },
    { 0xce2f8551, 61 },
    { 0, 0 },
    { 0x38f8553, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f8556, 68 },
    { 0xb9af8557, 95 },
    { 0xff2f8557, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410f855c, 55 },
    { 0, 0 },
    { 0xc6bf855e, 43 },
    { 0x5ccf855f, 33 },
    { 0x7fef8560, 95 },
    { 0x8e8f8560, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabf8564, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x748f8567, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb52f856d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52ef8571, 66 },
    { 0x2df8571, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bff8576, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4af857c, 61 },
    { 0xec9f857c, 36 },
    { 0xc8ff857e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x231f8585, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47f858b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28af8596, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ff8599, 23 },
    { 0x2d8f859a, 61 },
    { 0x4e1f859b, 61 },
    { 0x872f859c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88f85ae, 46 },
    { 0x55cf85af, 49 },
    { 0xae0f85b0, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdf85b3, 93 },
    { 0xa69f85b4, 90 },
    { 0x7f6f85b4, 55 },
    { 0x62cf85b6, 87 },
    { 0xa0cf85b7, 90 },
    { 0, 0 },
    { 0x70af85b9, 33 },
    { 0x9f3f85ba, 52 },
    { 0x41ef85bb, 98 },
    { 0xcfef85ba, 61 },
    { 0, 0 },
    { 0xb69f85be, 33 },
    { 0x20af85be, 93 },
    { 0x99bf85be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x406f85c3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd62f85cc, 69 },
    { 0x972f85cc, 98 },
    { 0xe50f85ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaef85d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59df85da, 90 },
    { 0x1bf85db, 61 },
    { 0x742f85dc, 23 },
    { 0x31df85da, 67 },
    { 0, 0 },
    { 0x655f85df, 33 },
    { 0x5aaf85e0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78f85e5, 93 },
    { 0, 0 },
    { 0x41af85e7, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcaf85ea, 23 },
    { 0, 0 },
    { 0x771f85ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a5f85f3, 95 },
    { 0, 0 },
    { 0xc94f85f5, 33 },
    { 0xf84f85f6, 61 },
    { 0x921f85f6, 61 },
    { 0xfe7f85f8, 76 },
    { 0x32f85f8, 84 },
    { 0x992f85f8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x537f85fd, 54 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bf8600, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10f8607, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1df860a, 52 },
    { 0xb93f860b, 55 },
    { 0xcc7f860c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x229f8612, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f8618, 90 },
    { 0, 0 },
    { 0x379f861a, 76 },
    { 0, 0 },
    { 0x8fbf861c, 33 },
    { 0xb81f861d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0f8623, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38f8626, 61 },
    { 0xeff8626, 69 },
    { 0xba1f8628, 66 },
    { 0x83ef8627, 55 },
    { 0x749f862a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6f8632, 40 },
    { 0xad3f8632, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3df8636, 95 },
    { 0x503f8637, 61 },
    { 0, 0 },
    { 0xc8ff8639, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe15f863d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x84bf8640, 23 },
    { 0, 0 },
    { 0x42f8642, 23 },
    { 0xccaf8642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcf8646, 61 },
    { 0xa29f8647, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412f864e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3cf8651, 90 },
    { 0x278f8652, 77 },
    { 0x50af8653, 83 },
    { 0, 0 },
    { 0x1c5f8655, 43 },
    { 0x6bef8656, 33 },
    { 0, 0 },
    { 0xa0bf8658, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x149f865f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd42f8662, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867f866c, 87 },
    { 0, 0 },
    { 0x113f866e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577f8672, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe03f8679, 93 },
    { 0x46ef867a, 34 },
    { 0xe2bf8679, 90 },
    { 0xdf7f867a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x202f867f, 95 },
    { 0x6e1f867f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bff8685, 55 },
    { 0x770f8686, 88 },
    { 0, 0 },
    { 0xf6ff8688, 98 },
    { 0, 0 },
    { 0xb48f868a, 33 },
    { 0, 0 },
    { 0x209f868c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bf8690, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25f8693, 98 },
    { 0, 0 },
    { 0x576f8695, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bf869a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303f86a0, 33 },
    { 0, 0 },
    { 0xa53f86a2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4f86a7, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ef86aa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa76f86ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c5f86b0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70f86b4, 91 },
    { 0, 0 },
    { 0x224f86b6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1f86bd, 91 },
    { 0xd69f86be, 95 },
    { 0x146f86be, 75 },
    { 0xc1af86c0, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x48df86c3, 55 },
    { 0xb9ef86c3, 48 },
    { 0, 0 },
    { 0xa5f86c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaf86c9, 90 },
    { 0x91df86c9, 98 },
    { 0, 0 },
    { 0x424f86cc, 95 },
    { 0x402f86cd, 33 },
    { 0xe60f86ce, 95 },
    { 0xcaff86cf, 80 },
    { 0x1ccf86cf, 36 },
    { 0xa53f86d0, 34 },
    { 0x9bcf86d2, 23 },
    { 0x33ff86d2, 100 },
    { 0x9bf86d4, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454f86dc, 90 },
    { 0, 0 },
    { 0xbc4f86de, 85 },
    { 0xe05f86df, 43 },
    { 0, 0 },
    { 0xa57f86e1, 98 },
    { 0x15ef86e2, 95 },
    { 0xf9df86e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41f86ee, 98 },
    { 0xe01f86ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef86f9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f8700, 83 },
    { 0x129f8701, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2af8705, 33 },
    { 0xf5df8705, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbef870c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3bf8710, 17 },
    { 0xdb4f8711, 23 },
    { 0xdfef8712, 23 },
    { 0, 0 },
    { 0xfeff8714, 46 },
    { 0, 0 },
    { 0xd40f8716, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f871d, 91 },
    { 0x920f871d, 43 },
    { 0, 0 },
    { 0xbc3f8720, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd39f8723, 48 },
    { 0xa87f8724, 76 },
    { 0x3fbf8725, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x195f8728, 87 },
    { 0xbdaf8729, 55 },
    { 0, 0 },
    { 0x9c1f872b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaaf8730, 87 },
    { 0xe07f8730, 93 },
    { 0x619f8730, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa3f8735, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78f873b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9df8742, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3f8747, 61 },
    { 0xab0f8748, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x173f874b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfaf874e, 90 },
    { 0xcd0f874f, 23 },
    { 0, 0 },
    { 0x49df8751, 43 },
    { 0, 0 },
    { 0xd8af8753, 33 },
    { 0, 0 },
    { 0x7c7f8755, 95 },
    { 0x28ff8756, 52 },
    { 0xd22f8757, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x882f875c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaf8763, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x634f8766, 90 },
    { 0xa52f8767, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x344f876b, 40 },
    { 0, 0 },
    { 0x4e0f876d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff8771, 55 },
    { 0x91f8772, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb69f8775, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26f877a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1f877d, 43 },
    { 0xb97f877d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbf8788, 76 },
    { 0x95f8789, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804f878d, 93 },
    { 0x10cf878e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x332f8792, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ff879b, 61 },
    { 0xf4f879b, 90 },
    { 0x729f879c, 17 },
    { 0xbcdf879e, 100 },
    { 0x337f879f, 43 },
    { 0x9c9f879e, 90 },
    { 0x8eff87a1, 43 },
    { 0x495f87a1, 90 },
    { 0, 0 },
    { 0xc9f87a4, 84 },
    { 0, 0 },
    { 0xa48f87a6, 66 },
    { 0x506f87a7, 93 },
    { 0xaccf87a8, 23 },
    { 0, 0 },
    { 0xbf1f87aa, 33 },
    { 0x208f87ab, 88 },
    { 0, 0 },
    { 0x41ff87ad, 33 },
    { 0x4e4f87ad, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bf87b2, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f87b8, 100 },
    { 0, 0 },
    { 0xe2ff87ba, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x93cf87bd, 33 },
    { 0xaf87bd, 93 },
    { 0xe48f87be, 95 },
    { 0, 0 },
    { 0xa75f87c1, 88 },
    { 0xeef87c2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10f87c7, 95 },
    { 0xfd1f87c8, 33 },
    { 0xab2f87c9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9def87cc, 43 },
    { 0, 0 },
    { 0xe96f87ce, 85 },
    { 0x235f87cf, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54f87d2, 98 },
    { 0x555f87d3, 67 },
    { 0x294f87d4, 61 },
    { 0, 0 },
    { 0xd8f87d6, 99 },
    { 0xd02f87d6, 66 },
    { 0x75ff87d8, 55 },
    { 0x982f87d9, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df87dd, 33 },
    { 0x1e8f87de, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf87e2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6faf87e9, 98 },
    { 0, 0 },
    { 0x37bf87eb, 95 },
    { 0xa04f87eb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb32f87f0, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15f87f3, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7eff87f7, 23 },
    { 0xdef87f8, 34 },
    { 0xe83f87f9, 40 },
    { 0x9dcf87fa, 98 },
    { 0xb4ff87fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x365f87fe, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47f8803, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7af8809, 38 },
    { 0x27cf880a, 91 },
    { 0x567f880b, 90 },
    { 0x76ff880a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2f8810, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ef8813, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384f8818, 75 },
    { 0xfbcf8819, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cf881d, 95 },
    { 0x66ef881e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373f882a, 95 },
    { 0x385f882b, 84 },
    { 0x9adf882a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bf8831, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ff8835, 93 },
    { 0x62cf8835, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc32f883e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61f8843, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84f8846, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45f884b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cf8851, 55 },
    { 0, 0 },
    { 0x92cf8853, 33 },
    { 0x9eff8854, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53f8858, 95 },
    { 0x89ff8859, 98 },
    { 0xc0ef885a, 23 },
    { 0x989f885a, 61 },
    { 0, 0 },
    { 0x27ff885d, 21 },
    { 0xd7af885d, 55 },
    { 0, 0 },
    { 0xc28f8860, 55 },
    { 0x44df8860, 90 },
    { 0xe88f8860, 76 },
    { 0xe9f8863, 98 },
    { 0xe85f8863, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x643f886c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x645f8879, 90 },
    { 0x35f887a, 65 },
    { 0x2a7f887b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9f887f, 61 },
    { 0, 0 },
    { 0x2f6f8881, 88 },
    { 0x3f1f8881, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x403f8887, 23 },
    { 0xabef8888, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafdf888c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48af8892, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x381f8897, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8df88a1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ff88a7, 67 },
    { 0, 0 },
    { 0xb2bf88a9, 33 },
    { 0x169f88aa, 87 },
    { 0x4a4f88ab, 23 },
    { 0x3a0f88ac, 98 },
    { 0xce3f88ad, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x37ef88b0, 95 },
    { 0, 0 },
    { 0xd91f88b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0f88b7, 93 },
    { 0xce6f88b8, 77 },
    { 0x466f88b9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x832f88bc, 34 },
    { 0, 0 },
    { 0x19df88be, 61 },
    { 0, 0 },
    { 0x13ef88c0, 90 },
    { 0xe49f88c0, 76 },
    { 0x6caf88c2, 33 },
    { 0, 0 },
    { 0x9cff88c4, 87 },
    { 0x373f88c5, 33 },
    { 0, 0 },
    { 0x559f88c7, 43 },
    { 0x947f88c7, 33 },
    { 0xe3ef88c8, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ff88cc, 85 },
    { 0, 0 },
    { 0xebcf88ce, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbf88d1, 33 },
    { 0xd1f88d1, 23 },
    { 0x185f88d3, 95 },
    { 0x2cff88d1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x89f88d7, 95 },
    { 0x823f88d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff88e6, 80 },
    { 0xe9df88e6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57cf88ea, 4 },
    { 0, 0 },
    { 0xe09f88ec, 90 },
    { 0x86ef88ed, 34 },
    { 0xa26f88ed, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2f88f1, 69 },
    { 0xb36f88f2, 90 },
    { 0xb3ef88f3, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xb10f88f6, 23 },
    { 0xeecf88f7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f88fb, 46 },
    { 0x1f3f88fc, 80 },
    { 0, 0 },
    { 0x758f88fe, 40 },
    { 0xb23f88fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x404f8906, 23 },
    { 0x70f8907, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2f890d, 98 },
    { 0, 0 },
    { 0x7d4f890f, 33 },
    { 0x57cf8910, 23 },
    { 0xef7f8911, 68 },
    { 0x1acf8912, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x302f8916, 76 },
    { 0xcc7f8916, 76 },
    { 0x7e6f8918, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0df891f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0bf8925, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x713f8928, 95 },
    { 0xaf6f8929, 46 },
    { 0xa68f892a, 46 },
    { 0x8fff892b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf8930, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f8934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ef8939, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1af893f, 90 },
    { 0xaf0f8940, 23 },
    { 0, 0 },
    { 0x901f8942, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df8945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd0f8949, 44 },
    { 0x2e1f894a, 90 },
    { 0xccaf894a, 98 },
    { 0x16f894a, 76 },
    { 0, 0 },
    { 0xa4f894e, 90 },
    { 0x261f894f, 91 },
    { 0x71f894f, 61 },
    { 0x1aef8951, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c8f895a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6f895f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x604f8962, 87 },
    { 0, 0 },
    { 0x62ef8964, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705f8968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeabf896b, 90 },
    { 0, 0 },
    { 0xf93f896d, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f8970, 23 },
    { 0x52ef8971, 98 },
    { 0, 0 },
    { 0x686f8973, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ef897c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27f897f, 98 },
    { 0xa9ff897f, 23 },
    { 0x9d9f8981, 77 },
    { 0xe84f897f, 98 },
    { 0x906f8983, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f2f8987, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8f898a, 95 },
    { 0x93bf898a, 52 },
    { 0, 0 },
    { 0x8a1f898d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8f8994, 55 },
    { 0, 0 },
    { 0x980f8996, 91 },
    { 0x1fcf8996, 17 },
    { 0x377f8996, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a0f89a7, 78 },
    { 0x227f89a7, 76 },
    { 0, 0 },
    { 0x4cff89aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9df89b0, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584f89b8, 80 },
    { 0x647f89b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x867f89bc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddf89c0, 33 },
    { 0, 0 },
    { 0x2daf89c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1f89c7, 76 },
    { 0xef9f89c7, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f89ce, 82 },
    { 0, 0 },
    { 0x9d4f89d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5af89d4, 61 },
    { 0, 0 },
    { 0x46ef89d6, 90 },
    { 0xaaff89d7, 67 },
    { 0x626f89d7, 38 },
    { 0, 0 },
    { 0x2eef89da, 78 },
    { 0x1a6f89da, 98 },
    { 0xb1bf89db, 40 },
    { 0, 0 },
    { 0x6ecf89de, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcbf89e4, 33 },
    { 0x3bff89e5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6df89ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb09f89f0, 61 },
    { 0xd03f89f1, 96 },
    { 0x3bdf89f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611f89f6, 91 },
    { 0xf1df89f6, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bf89fe, 61 },
    { 0x33df89ff, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578f8a03, 61 },
    { 0xb82f8a04, 93 },
    { 0x763f8a05, 76 },
    { 0, 0 },
    { 0xcf9f8a07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cf8a0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbff8a10, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfbf8a15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeacf8a1e, 67 },
    { 0x534f8a1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3f8a25, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adf8a28, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef8a2e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19bf8a33, 95 },
    { 0, 0 },
    { 0xcaff8a35, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78f8a3c, 66 },
    { 0x53ff8a3d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ef8a42, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b8f8a45, 84 },
    { 0x508f8a46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bf8a49, 90 },
    { 0x587f8a49, 80 },
    { 0x353f8a4b, 86 },
    { 0x143f8a4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81ff8a50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f8a5e, 61 },
    { 0xc38f8a5f, 34 },
    { 0x33ef8a5f, 55 },
    { 0x9e9f8a5f, 56 },
    { 0xf28f8a62, 90 },
    { 0xb64f8a60, 43 },
    { 0x414f8a64, 55 },
    { 0x52df8a64, 98 },
    { 0x4c3f8a65, 33 },
    { 0, 0 },
    { 0xf70f8a68, 97 },
    { 0x2e5f8a68, 61 },
    { 0xa61f8a6a, 46 },
    { 0x850f8a6b, 90 },
    { 0x7fdf8a6c, 23 },
    { 0, 0 },
    { 0x82df8a6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc5f8a75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6baf8a7a, 41 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6f8a7d, 33 },
    { 0x975f8a7e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237f8a84, 23 },
    { 0xf51f8a85, 76 },
    { 0x1df8a86, 91 },
    { 0x973f8a87, 43 },
    { 0xeacf8a88, 23 },
    { 0x847f8a86, 33 },
    { 0, 0 },
    { 0xc28f8a8b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d9f8a8e, 64 },
    { 0x683f8a8f, 61 },
    { 0, 0 },
    { 0x2e1f8a91, 61 },
    { 0x7faf8a92, 84 },
    { 0xd43f8a93, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9f8a97, 55 },
    { 0x7c4f8a97, 33 },
    { 0, 0 },
    { 0xf70f8a9a, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcf8a9f, 43 },
    { 0, 0 },
    { 0x558f8aa1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf26f8aa4, 90 },
    { 0x359f8aa5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x846f8aac, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ef8ab1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46f8ab7, 68 },
    { 0, 0 },
    { 0x9fff8ab9, 98 },
    { 0x3c2f8aba, 52 },
    { 0x459f8aba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40f8ac0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae0f8ac5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff8acb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4f8ace, 23 },
    { 0x744f8acf, 55 },
    { 0xfeef8acf, 66 },
    { 0x868f8ad1, 17 },
    { 0xea0f8ad2, 52 },
    { 0xe5ef8ad3, 91 },
    { 0x32f8ad0, 40 },
    { 0x45ef8ad5, 38 },
    { 0xb24f8ad6, 23 },
    { 0, 0 },
    { 0xebaf8ad8, 90 },
    { 0x644f8ad9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbf8ae0, 33 },
    { 0x61bf8ae1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45df8ae4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x335f8ae7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x45f8aea, 78 },
    { 0xb14f8aeb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cf8aef, 67 },
    { 0x5aaf8aef, 34 },
    { 0xbd8f8af1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda6f8af5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45af8afc, 90 },
    { 0xef5f8afc, 33 },
    { 0xf02f8afe, 99 },
    { 0, 0 },
    { 0x684f8b00, 98 },
    { 0xe43f8b00, 61 },
    { 0x914f8b00, 61 },
    { 0xc07f8b03, 23 },
    { 0, 0 },
    { 0xab1f8b05, 95 },
    { 0xaa7f8b06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36f8b0d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eaf8b14, 55 },
    { 0x9eef8b15, 98 },
    { 0x667f8b16, 90 },
    { 0, 0 },
    { 0x20f8b18, 23 },
    { 0x9a9f8b19, 46 },
    { 0, 0 },
    { 0xde0f8b1b, 100 },
    { 0xf19f8b1b, 98 },
    { 0, 0 },
    { 0x4bff8b1e, 33 },
    { 0, 0 },
    { 0x8b9f8b20, 23 },
    { 0x266f8b21, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xce2f8b24, 43 },
    { 0, 0 },
    { 0xffdf8b26, 61 },
    { 0xbb1f8b27, 85 },
    { 0xc4ef8b28, 83 },
    { 0x907f8b29, 95 },
    { 0x390f8b26, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x984f8b2d, 84 },
    { 0xa8bf8b2e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b2f8b31, 98 },
    { 0xed0f8b31, 90 },
    { 0, 0 },
    { 0xccaf8b34, 90 },
    { 0x11ef8b35, 98 },
    { 0x681f8b36, 33 },
    { 0x5cef8b37, 90 },
    { 0xfddf8b36, 23 },
    { 0, 0 },
    { 0xa34f8b3a, 99 },
    { 0xd31f8b3a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaff8b3e, 76 },
    { 0x131f8b3f, 33 },
    { 0x7d1f8b3f, 52 },
    { 0x7f4f8b3f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eaf8b44, 23 },
    { 0x105f8b45, 100 },
    { 0, 0 },
    { 0x7fdf8b47, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aef8b4a, 59 },
    { 0x294f8b4b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225f8b4f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0af8b53, 23 },
    { 0xc0ff8b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xff8b57, 98 },
    { 0, 0 },
    { 0x36f8b59, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8f8b5e, 93 },
    { 0xdbbf8b5e, 81 },
    { 0, 0 },
    { 0xabf8b61, 90 },
    { 0x95af8b61, 23 },
    { 0xa0df8b62, 98 },
    { 0, 0 },
    { 0xeeef8b65, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ef8b68, 69 },
    { 0, 0 },
    { 0x25f8b6a, 66 },
    { 0xff4f8b6b, 81 },
    { 0x661f8b6c, 68 },
    { 0xc4df8b6a, 98 },
    { 0x13cf8b6e, 76 },
    { 0x8f3f8b6e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x912f8b7c, 46 },
    { 0x601f8b7d, 43 },
    { 0, 0 },
    { 0x5eff8b7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef8b83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f8b87, 43 },
    { 0, 0 },
    { 0xf44f8b89, 33 },
    { 0x307f8b89, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x906f8b8d, 33 },
    { 0xfc4f8b8e, 34 },
    { 0xb6ef8b8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78f8b92, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59f8b99, 55 },
    { 0, 0 },
    { 0x1bff8b9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58bf8ba5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e8f8bac, 33 },
    { 0, 0 },
    { 0xdadf8bae, 90 },
    { 0x92ef8baf, 75 },
    { 0x658f8bb0, 43 },
    { 0x80ff8bb1, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ff8bba, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe7f8bc0, 36 },
    { 0xad3f8bc1, 81 },
    { 0xa05f8bc2, 76 },
    { 0x8b4f8bc3, 33 },
    { 0, 0 },
    { 0x10ff8bc5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccf8bca, 84 },
    { 0xa39f8bca, 40 },
    { 0xc68f8bcc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0f8bd3, 90 },
    { 0xe21f8bd3, 33 },
    { 0xa2df8bd5, 84 },
    { 0, 0 },
    { 0x4ecf8bd7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0f8bde, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x260f8be3, 66 },
    { 0x996f8be4, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ff8be8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bf8beb, 61 },
    { 0xa07f8bec, 76 },
    { 0xbdaf8bed, 56 },
    { 0, 0 },
    { 0x6cbf8bef, 23 },
    { 0, 0 },
    { 0x53f8bf1, 33 },
    { 0, 0 },
    { 0xc33f8bf3, 98 },
    { 0, 0 },
    { 0x8aff8bf5, 76 },
    { 0x31f8bf6, 66 },
    { 0x6eef8bf7, 61 },
    { 0, 0 },
    { 0x68ff8bf9, 55 },
    { 0, 0 },
    { 0xbf0f8bfb, 99 },
    { 0x620f8bfb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f8c00, 99 },
    { 0x877f8c01, 23 },
    { 0x9c0f8c01, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x476f8c0b, 90 },
    { 0xbcdf8c0b, 43 },
    { 0, 0 },
    { 0x188f8c0e, 85 },
    { 0x2d5f8c0f, 48 },
    { 0, 0 },
    { 0x583f8c11, 90 },
    { 0x3fcf8c12, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904f8c17, 76 },
    { 0xcecf8c18, 61 },
    { 0x354f8c19, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x441f8c22, 66 },
    { 0, 0 },
    { 0xa89f8c24, 3 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ff8c29, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39df8c2f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736f8c35, 90 },
    { 0x939f8c36, 98 },
    { 0xa7ff8c37, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ef8c3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ff8c3f, 98 },
    { 0x7b0f8c40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f8c43, 99 },
    { 0, 0 },
    { 0xffff8c45, 55 },
    { 0x154f8c45, 98 },
    { 0x9a6f8c45, 95 },
    { 0xc96f8c47, 85 },
    { 0x115f8c49, 17 },
    { 0x95f8c49, 98 },
    { 0, 0 },
    { 0x1d4f8c4c, 33 },
    { 0xffff8c4c, 99 },
    { 0x86cf8c4c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaf8c56, 33 },
    { 0x388f8c57, 87 },
    { 0x415f8c56, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0f8c5c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9f8c60, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33f8c65, 33 },
    { 0, 0 },
    { 0x74ef8c67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef8c6b, 33 },
    { 0, 0 },
    { 0x1c1f8c6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f8c74, 86 },
    { 0x42bf8c74, 61 },
    { 0xe65f8c75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8f8c7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59f8c82, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07f8c85, 61 },
    { 0xe5bf8c86, 76 },
    { 0xe97f8c85, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65f8c8e, 98 },
    { 0xa61f8c8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5f8c92, 76 },
    { 0xed4f8c93, 95 },
    { 0x53cf8c93, 91 },
    { 0xe7ff8c95, 90 },
    { 0x1aaf8c96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514f8c9c, 61 },
    { 0, 0 },
    { 0xc3cf8c9e, 33 },
    { 0xac5f8c9e, 23 },
    { 0x970f8ca0, 85 },
    { 0x8e7f8ca0, 61 },
    { 0x11f8ca2, 55 },
    { 0xa65f8ca3, 33 },
    { 0, 0 },
    { 0xe7bf8ca5, 96 },
    { 0x25bf8ca6, 90 },
    { 0xec4f8ca7, 34 },
    { 0x93df8ca8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bf8cab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ef8cb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x529f8cb7, 23 },
    { 0x2dcf8cb8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ff8cbc, 91 },
    { 0x7f1f8cbc, 77 },
    { 0, 0 },
    { 0x4a9f8cbf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceff8cc3, 61 },
    { 0x8cef8cc4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6f8cc7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x350f8cca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf99f8ccf, 90 },
    { 0x7c5f8ccf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dff8cd3, 90 },
    { 0xf49f8cd3, 61 },
    { 0x575f8cd3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95f8cd8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x850f8cdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x334f8ce3, 66 },
    { 0x763f8ce3, 88 },
    { 0x52f8ce5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0af8ce9, 46 },
    { 0xa3df8cea, 33 },
    { 0x8e1f8ceb, 52 },
    { 0x534f8cec, 76 },
    { 0x630f8ceb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c2f8cf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3f8cfb, 17 },
    { 0x5c8f8cfc, 90 },
    { 0xbbbf8cfd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f8d03, 82 },
    { 0xdf2f8d03, 76 },
    { 0x1cbf8d05, 24 },
    { 0, 0 },
    { 0xb79f8d07, 90 },
    { 0x40ff8d08, 47 },
    { 0x313f8d09, 85 },
    { 0x712f8d09, 98 },
    { 0x560f8d09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd68f8d13, 90 },
    { 0xfd5f8d14, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeaf8d1b, 90 },
    { 0x58af8d1b, 75 },
    { 0, 0 },
    { 0x9bff8d1e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711f8d22, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf35f8d26, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x811f8d29, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdef8d2f, 43 },
    { 0x4e0f8d2f, 33 },
    { 0x38af8d31, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33f8d35, 33 },
    { 0xf68f8d35, 76 },
    { 0x1ccf8d37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2daf8d3b, 98 },
    { 0x3def8d3b, 33 },
    { 0x90ef8d3d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73ef8d41, 84 },
    { 0xed8f8d42, 93 },
    { 0xeecf8d41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf0f8d4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650f8d50, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45f8d56, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f8d59, 77 },
    { 0xfaf8d5a, 33 },
    { 0, 0 },
    { 0x934f8d5c, 95 },
    { 0x785f8d5d, 55 },
    { 0x8e2f8d5e, 23 },
    { 0xc1bf8d5c, 90 },
    { 0xcd8f8d5c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59f8d63, 90 },
    { 0x143f8d63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78f8d6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9f8d70, 81 },
    { 0x660f8d70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b2f8d74, 81 },
    { 0, 0 },
    { 0xe23f8d76, 77 },
    { 0xb70f8d77, 66 },
    { 0xec0f8d77, 34 },
    { 0x17df8d79, 90 },
    { 0x7df8d7a, 76 },
    { 0xe56f8d78, 33 },
    { 0x3cef8d7c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x853f8d89, 23 },
    { 0xba7f8d8a, 61 },
    { 0, 0 },
    { 0x3aef8d8c, 90 },
    { 0, 0 },
    { 0x978f8d8e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65af8d92, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1bf8d98, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x488f8d9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbf8da1, 94 },
    { 0xbbbf8da2, 33 },
    { 0x5a9f8da3, 98 },
    { 0xe73f8da4, 33 },
    { 0, 0 },
    { 0x8daf8da6, 88 },
    { 0xfd7f8da6, 91 },
    { 0xf72f8da8, 33 },
    { 0x1cef8da9, 76 },
    { 0x89df8daa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6f8daf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bf8dba, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993f8dc0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2df8dc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9f8dcb, 33 },
    { 0x84ff8dcb, 55 },
    { 0x534f8dcd, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111f8dd3, 23 },
    { 0x631f8dd4, 66 },
    { 0x163f8dd4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2cf8ddb, 55 },
    { 0xbe0f8ddb, 78 },
    { 0x6f3f8ddd, 61 },
    { 0xd8ff8ddd, 84 },
    { 0x406f8ddf, 78 },
    { 0x382f8de0, 93 },
    { 0, 0 },
    { 0xd19f8de2, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cf8de5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x654f8de8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaef8deb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ef8dee, 52 },
    { 0x596f8def, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c5f8df5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bf8dfc, 46 },
    { 0x85ff8dfd, 34 },
    { 0, 0 },
    { 0xd9df8dff, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x645f8e02, 38 },
    { 0x5c6f8e03, 33 },
    { 0x52ff8e02, 98 },
    { 0x931f8e05, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x61df8e08, 84 },
    { 0, 0 },
    { 0xc2ff8e0a, 23 },
    { 0xc39f8e0b, 26 },
    { 0x778f8e0c, 61 },
    { 0x940f8e0d, 100 },
    { 0xadaf8e0a, 43 },
    { 0, 0 },
    { 0x547f8e10, 86 },
    { 0x117f8e11, 76 },
    { 0xed8f8e11, 23 },
    { 0xb3df8e13, 90 },
    { 0x7c8f8e14, 95 },
    { 0x3dbf8e14, 33 },
    { 0, 0 },
    { 0x983f8e17, 23 },
    { 0, 0 },
    { 0x443f8e19, 75 },
    { 0xdcff8e1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0f8e1d, 90 },
    { 0xc24f8e1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bff8e21, 100 },
    { 0x39cf8e21, 91 },
    { 0xa7cf8e23, 61 },
    { 0x2e4f8e24, 55 },
    { 0, 0 },
    { 0x873f8e26, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f8e2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab4f8e32, 100 },
    { 0x3d8f8e33, 55 },
    { 0xa3cf8e32, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dff8e37, 90 },
    { 0x27cf8e37, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03f8e3d, 77 },
    { 0x80ef8e3e, 66 },
    { 0x38ef8e3f, 33 },
    { 0, 0 },
    { 0x7caf8e41, 90 },
    { 0xfcbf8e42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x730f8e45, 43 },
    { 0xe4ff8e46, 100 },
    { 0x2e0f8e47, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebf8e4a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8f8e5b, 52 },
    { 0x45df8e5b, 85 },
    { 0xc1f8e5d, 90 },
    { 0, 0 },
    { 0xf3df8e5f, 43 },
    { 0x88ef8e5f, 34 },
    { 0xbfcf8e5f, 61 },
    { 0, 0 },
    { 0xba0f8e63, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x35df8e66, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64ff8e6b, 85 },
    { 0xf05f8e6c, 78 },
    { 0x35bf8e6c, 33 },
    { 0x493f8e6e, 66 },
    { 0x890f8e6b, 98 },
    { 0, 0 },
    { 0x96df8e71, 33 },
    { 0x71df8e72, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df8e7b, 53 },
    { 0, 0 },
    { 0xc21f8e7d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5bf8e83, 33 },
    { 0xff1f8e83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93f8e87, 46 },
    { 0xd85f8e87, 61 },
    { 0, 0 },
    { 0xaa2f8e8a, 23 },
    { 0x4a2f8e8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775f8e90, 46 },
    { 0x64f8e91, 66 },
    { 0xd27f8e92, 77 },
    { 0x4e1f8e92, 100 },
    { 0xc1ef8e92, 98 },
    { 0x62cf8e93, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0cf8e98, 90 },
    { 0, 0 },
    { 0x5f3f8e9a, 46 },
    { 0xcddf8e9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f8e9f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17f8ea3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x236f8ea6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40bf8eaa, 90 },
    { 0, 0 },
    { 0x4b3f8eac, 33 },
    { 0x6aef8ead, 95 },
    { 0, 0 },
    { 0x200f8eaf, 95 },
    { 0x4eaf8eb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x202f8eb3, 76 },
    { 0x30af8eb4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8f8eba, 33 },
    { 0, 0 },
    { 0x400f8ebc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84ef8ec0, 90 },
    { 0x47df8ec0, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a8f8ec5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb49f8ecb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x896f8ece, 55 },
    { 0, 0 },
    { 0x693f8ed0, 91 },
    { 0x1cef8ed1, 98 },
    { 0, 0 },
    { 0xc88f8ed3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6f8ed6, 93 },
    { 0, 0 },
    { 0x1a3f8ed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eff8edb, 23 },
    { 0xfa4f8edb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8f8ee3, 90 },
    { 0x47bf8ee4, 55 },
    { 0x10df8ee4, 98 },
    { 0, 0 },
    { 0xe55f8ee7, 91 },
    { 0, 0 },
    { 0xb77f8ee9, 69 },
    { 0, 0 },
    { 0x1acf8eeb, 99 },
    { 0x2ebf8eec, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x881f8ef0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7af8ef8, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0f8eff, 91 },
    { 0x47af8eff, 95 },
    { 0x1edf8f01, 84 },
    { 0xa04f8f02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d1f8f06, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf20f8f0a, 33 },
    { 0, 0 },
    { 0xa74f8f0c, 97 },
    { 0xd05f8f0c, 95 },
    { 0xeebf8f0e, 61 },
    { 0, 0 },
    { 0x248f8f10, 33 },
    { 0xf7ef8f11, 91 },
    { 0x845f8f11, 61 },
    { 0xbb0f8f13, 90 },
    { 0x3abf8f14, 33 },
    { 0x7b2f8f13, 93 },
    { 0x80ef8f16, 94 },
    { 0, 0 },
    { 0x4ef8f18, 55 },
    { 0x2aff8f19, 90 },
    { 0xc3cf8f18, 61 },
    { 0x5b4f8f19, 33 },
    { 0xd06f8f1a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40df8f22, 36 },
    { 0, 0 },
    { 0xfc3f8f24, 78 },
    { 0x884f8f24, 88 },
    { 0x2cff8f26, 8 },
    { 0x572f8f26, 90 },
    { 0x1b0f8f28, 85 },
    { 0xd26f8f24, 61 },
    { 0xa13f8f2a, 33 },
    { 0x88ff8f2b, 98 },
    { 0x5aaf8f2c, 33 },
    { 0x7fef8f2d, 61 },
    { 0xc4f8f2e, 98 },
    { 0xfaef8f29, 90 },
    { 0x730f8f30, 33 },
    { 0, 0 },
    { 0x271f8f32, 90 },
    { 0xc47f8f33, 33 },
    { 0, 0 },
    { 0x917f8f35, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30f8f38, 33 },
    { 0x49af8f38, 76 },
    { 0x333f8f39, 90 },
    { 0x2a6f8f3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa44f8f3e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x531f8f41, 90 },
    { 0xccef8f42, 90 },
    { 0x2fcf8f41, 40 },
    { 0x486f8f44, 95 },
    { 0x93ef8f44, 100 },
    { 0xa8f8f44, 92 },
    { 0x743f8f47, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1f8f4b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x699f8f4e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddf8f52, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x944f8f56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6f8f5e, 98 },
    { 0x39cf8f5f, 99 },
    { 0, 0 },
    { 0xb12f8f61, 65 },
    { 0x1f9f8f62, 61 },
    { 0x3d4f8f63, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc51f8f67, 23 },
    { 0x67ff8f68, 33 },
    { 0x373f8f69, 33 },
    { 0x67ef8f6a, 61 },
    { 0xd71f8f6a, 76 },
    { 0xe24f8f6c, 33 },
    { 0xbacf8f69, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cf8f72, 95 },
    { 0, 0 },
    { 0x42ef8f74, 33 },
    { 0xf55f8f75, 61 },
    { 0x3b4f8f75, 69 },
    { 0, 0 },
    { 0x4a9f8f78, 61 },
    { 0x379f8f78, 76 },
    { 0xf71f8f79, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23df8f81, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e1f8f85, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4f8f8a, 98 },
    { 0, 0 },
    { 0xdbdf8f8c, 100 },
    { 0xc55f8f8d, 90 },
    { 0xe35f8f8e, 91 },
    { 0x6f4f8f8e, 93 },
    { 0, 0 },
    { 0x13cf8f91, 61 },
    { 0xf5af8f92, 80 },
    { 0xe36f8f93, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8f8f97, 43 },
    { 0x69bf8f98, 34 },
    { 0x25df8f99, 46 },
    { 0x9e5f8f99, 23 },
    { 0, 0 },
    { 0x94bf8f9c, 43 },
    { 0x707f8f9d, 23 },
    { 0x1a4f8f9e, 61 },
    { 0xf3bf8f9c, 23 },
    { 0x2f8fa0, 52 },
    { 0xc4af8fa0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x101f8fa5, 83 },
    { 0x4daf8fa6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x207f8fab, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcef8fae, 90 },
    { 0x156f8fae, 61 },
    { 0x99ef8fae, 95 },
    { 0x603f8faf, 65 },
    { 0x77f8fb2, 85 },
    { 0x972f8fb3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f8fb9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bef8fbc, 33 },
    { 0, 0 },
    { 0xeadf8fbe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1f8fc1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x860f8fc4, 33 },
    { 0xb21f8fc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bf8fcd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x71af8fd2, 66 },
    { 0, 0 },
    { 0x6bff8fd4, 55 },
    { 0, 0 },
    { 0xb11f8fd6, 93 },
    { 0xcedf8fd7, 23 },
    { 0, 0 },
    { 0x325f8fd9, 46 },
    { 0, 0 },
    { 0x203f8fdb, 43 },
    { 0xbdaf8fdc, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01f8fe4, 90 },
    { 0xdc7f8fe5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3adf8ff0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbf8ff5, 35 },
    { 0x1a0f8ff5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1f8fff, 95 },
    { 0x41bf9000, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0xd51f9003, 52 },
    { 0x90bf9004, 90 },
    { 0, 0 },
    { 0x961f9006, 43 },
    { 0x652f9007, 23 },
    { 0x710f9007, 43 },
    { 0x67af9009, 60 },
    { 0x204f9009, 98 },
    { 0x8a2f900b, 55 },
    { 0xb77f900c, 95 },
    { 0x801f900d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baf9011, 96 },
    { 0, 0 },
    { 0xd78f9013, 55 },
    { 0xe6af9014, 90 },
    { 0xd69f9015, 93 },
    { 0, 0 },
    { 0x5f9017, 84 },
    { 0x22df9017, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf901b, 76 },
    { 0x732f901c, 66 },
    { 0xb0df901c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c3f9020, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0f9023, 61 },
    { 0x3fdf9023, 61 },
    { 0xd8bf9025, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x442f902c, 90 },
    { 0xc64f902d, 52 },
    { 0x823f902e, 17 },
    { 0, 0 },
    { 0x2d1f9030, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898f903a, 85 },
    { 0x873f903a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb35f903e, 65 },
    { 0, 0 },
    { 0xed3f9040, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f9043, 33 },
    { 0, 0 },
    { 0x8b8f9045, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ef904b, 34 },
    { 0x706f904c, 33 },
    { 0x8d7f904b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ef9053, 93 },
    { 0, 0 },
    { 0xb16f9055, 23 },
    { 0x18ff9055, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91af905c, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37af9065, 98 },
    { 0, 0 },
    { 0x795f9067, 91 },
    { 0, 0 },
    { 0x392f9069, 76 },
    { 0xe08f906a, 95 },
    { 0, 0 },
    { 0xf68f906c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x737f906f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0f9072, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd85f9075, 55 },
    { 0x5def9076, 67 },
    { 0x4c5f9076, 61 },
    { 0x35f9075, 76 },
    { 0x4a5f9079, 78 },
    { 0xac4f9078, 84 },
    { 0x1e2f907b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ef9085, 93 },
    { 0xb2bf9085, 33 },
    { 0xf1ff9087, 61 },
    { 0, 0 },
    { 0x34ff9089, 61 },
    { 0, 0 },
    { 0x5e7f908b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30af9092, 33 },
    { 0xe38f9093, 98 },
    { 0x208f9092, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bf9097, 78 },
    { 0xe0bf9098, 61 },
    { 0x891f9099, 23 },
    { 0x5f5f9099, 90 },
    { 0xa8af909b, 55 },
    { 0, 0 },
    { 0xa7ef909d, 61 },
    { 0, 0 },
    { 0xbb9f909f, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x213f90a2, 61 },
    { 0x145f90a3, 48 },
    { 0x489f90a2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x410f90a7, 90 },
    { 0xff9f90a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d6f90ad, 55 },
    { 0, 0 },
    { 0x2b3f90af, 93 },
    { 0x34ef90b0, 61 },
    { 0xd9af90af, 76 },
    { 0xd3ff90b2, 23 },
    { 0x29df90b3, 90 },
    { 0, 0 },
    { 0x844f90b5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cf90ba, 90 },
    { 0xb1f90bb, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x249f90be, 95 },
    { 0x4abf90bf, 33 },
    { 0x3f8f90c0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecf90c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23f90cf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fff90d2, 43 },
    { 0x84cf90d3, 43 },
    { 0, 0 },
    { 0x47cf90d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5f90d9, 90 },
    { 0xcdef90da, 78 },
    { 0xaedf90db, 33 },
    { 0xcd2f90db, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x328f90e2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12f90e5, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0f90ec, 90 },
    { 0x792f90ed, 85 },
    { 0x1baf90ed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x402f90f1, 67 },
    { 0xea8f90f2, 40 },
    { 0, 0 },
    { 0xedff90f4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x236f90f7, 43 },
    { 0, 0 },
    { 0x705f90f9, 33 },
    { 0x798f90f9, 98 },
    { 0xc3cf90fb, 33 },
    { 0x4e6f90fc, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75f90ff, 33 },
    { 0xebaf9100, 90 },
    { 0, 0 },
    { 0x6d7f9102, 66 },
    { 0x361f9103, 90 },
    { 0xed8f9104, 23 },
    { 0xb89f9105, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cf9111, 52 },
    { 0, 0 },
    { 0xbcff9113, 33 },
    { 0x5aaf9114, 76 },
    { 0x6bbf9113, 55 },
    { 0x8b9f9116, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22f9119, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1f911c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2af9126, 93 },
    { 0xbaff9127, 76 },
    { 0x98f9127, 23 },
    { 0x2c9f9126, 95 },
    { 0, 0 },
    { 0x7c4f912b, 84 },
    { 0, 0 },
    { 0x116f912d, 43 },
    { 0xc32f912e, 76 },
    { 0xc3af912e, 61 },
    { 0, 0 },
    { 0xc91f9131, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6f9135, 55 },
    { 0, 0 },
    { 0xeeff9137, 34 },
    { 0x8b5f9138, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x468f913d, 33 },
    { 0xdfbf913e, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97f9141, 23 },
    { 0xb1ef9142, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x543f9145, 78 },
    { 0xc47f9146, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x378f914a, 83 },
    { 0x1f6f914b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xf34f914e, 95 },
    { 0x69ff914f, 23 },
    { 0x9baf9150, 95 },
    { 0, 0 },
    { 0x2daf9152, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903f9159, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x660f915d, 90 },
    { 0x9adf915e, 55 },
    { 0xd7df915e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2f9167, 67 },
    { 0xdabf9168, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x530f9170, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x987f9174, 43 },
    { 0, 0 },
    { 0x490f9176, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71df9179, 33 },
    { 0xbf8f917a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdf917d, 90 },
    { 0, 0 },
    { 0xf51f917f, 85 },
    { 0x5a2f9180, 90 },
    { 0x84bf9181, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf90f9189, 84 },
    { 0, 0 },
    { 0xc34f918b, 23 },
    { 0x9aef918c, 98 },
    { 0x570f918d, 55 },
    { 0xca1f918d, 33 },
    { 0x629f918c, 17 },
    { 0xb9bf918f, 91 },
    { 0xaebf9191, 87 },
    { 0xb66f9191, 90 },
    { 0x753f9192, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x84cf9196, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc4f919e, 90 },
    { 0x51ff919f, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422f91a3, 66 },
    { 0, 0 },
    { 0x42bf91a5, 80 },
    { 0x110f91a5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x876f91ab, 61 },
    { 0x6bf91ac, 23 },
    { 0x80bf91ad, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85f91b0, 76 },
    { 0, 0 },
    { 0xdc4f91b2, 84 },
    { 0xb6ff91b3, 88 },
    { 0x81bf91b4, 64 },
    { 0x9a7f91b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2f91ba, 83 },
    { 0x30ff91ba, 55 },
    { 0x896f91bc, 38 },
    { 0xfcbf91bc, 90 },
    { 0x72af91bc, 95 },
    { 0xc7f91bf, 90 },
    { 0x296f91be, 33 },
    { 0, 0 },
    { 0xa68f91c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x609f91c5, 85 },
    { 0x67ff91c5, 84 },
    { 0x930f91c7, 33 },
    { 0, 0 },
    { 0xca9f91c9, 93 },
    { 0xb39f91ca, 17 },
    { 0x1e2f91cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x121f91ce, 99 },
    { 0x161f91cf, 43 },
    { 0xbacf91d0, 43 },
    { 0x36af91d0, 55 },
    { 0, 0 },
    { 0x620f91d3, 90 },
    { 0xf72f91d4, 34 },
    { 0x9eaf91d5, 85 },
    { 0xa53f91d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24f91db, 98 },
    { 0, 0 },
    { 0xe66f91dd, 33 },
    { 0, 0 },
    { 0xc37f91df, 76 },
    { 0x9c5f91e0, 33 },
    { 0x60bf91e1, 90 },
    { 0xa7ef91e2, 61 },
    { 0xe9ef91e1, 93 },
    { 0x25f91df, 93 },
    { 0x7fef91e0, 95 },
    { 0x85ef91e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf91e9, 34 },
    { 0, 0 },
    { 0xb5af91eb, 98 },
    { 0xbe8f91ec, 99 },
    { 0x9d2f91eb, 53 },
    { 0x767f91ec, 75 },
    { 0x226f91ee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed8f91f4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c5f91fe, 55 },
    { 0xacef91ff, 98 },
    { 0x98af9200, 33 },
    { 0xc77f9201, 90 },
    { 0x219f91ff, 90 },
    { 0x6d8f9203, 67 },
    { 0x236f9204, 90 },
    { 0, 0 },
    { 0x7a7f9206, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ff9209, 95 },
    { 0x52f920a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdf920d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfaf9210, 96 },
    { 0x41cf9210, 65 },
    { 0x2f9f9210, 95 },
    { 0x4e1f9212, 76 },
    { 0x546f9214, 43 },
    { 0xed5f9215, 61 },
    { 0x79df9215, 80 },
    { 0, 0 },
    { 0x969f9218, 33 },
    { 0x32bf9219, 40 },
    { 0xc9f9219, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x948f921f, 33 },
    { 0, 0 },
    { 0x4f5f9221, 90 },
    { 0xb88f9221, 23 },
    { 0x631f9223, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0ef9227, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480f922b, 95 },
    { 0x38cf922b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x480f9233, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb20f9237, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fff923b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7df9240, 34 },
    { 0x55ef9241, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1f9245, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf924b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71f9251, 46 },
    { 0x482f9252, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45bf9256, 33 },
    { 0, 0 },
    { 0x516f9258, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa74f925c, 34 },
    { 0xdbcf925c, 95 },
    { 0xba6f925e, 90 },
    { 0xe9bf925e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x474f9262, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeecf926e, 99 },
    { 0, 0 },
    { 0xc09f9270, 55 },
    { 0xa54f9270, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3f9274, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc95f9277, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8eef927d, 34 },
    { 0, 0 },
    { 0x44df927f, 61 },
    { 0x934f9280, 61 },
    { 0x8faf9281, 55 },
    { 0, 0 },
    { 0xee3f9283, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1f9287, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dbf928f, 61 },
    { 0x2f9f928f, 55 },
    { 0, 0 },
    { 0xa1df9292, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9f9295, 81 },
    { 0x7cff9296, 98 },
    { 0x693f9296, 33 },
    { 0x9fdf9295, 33 },
    { 0x325f9299, 33 },
    { 0x6aff9299, 52 },
    { 0x58df929b, 23 },
    { 0x159f929b, 61 },
    { 0x380f929d, 17 },
    { 0, 0 },
    { 0xcb2f929f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff92a4, 98 },
    { 0xd9f92a5, 88 },
    { 0x85df92a4, 98 },
    { 0x46df92a7, 90 },
    { 0xbacf92a7, 52 },
    { 0x5bdf92a9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f92af, 90 },
    { 0x56af92b0, 95 },
    { 0x7eff92b1, 36 },
    { 0xcb0f92b2, 34 },
    { 0xec5f92b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x146f92b9, 33 },
    { 0xce9f92ba, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb2f92bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ef92c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2df92c4, 33 },
    { 0x9fdf92c4, 90 },
    { 0x3ef92c5, 95 },
    { 0x83bf92c7, 91 },
    { 0x20ef92c8, 87 },
    { 0x875f92c8, 23 },
    { 0xab9f92ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cf92ce, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x127f92d1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4f92d4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd2f92d8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbf92e2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8f92f6, 90 },
    { 0xef7f92f6, 66 },
    { 0xbbcf92f7, 55 },
    { 0x144f92f9, 61 },
    { 0, 0 },
    { 0xce4f92fb, 17 },
    { 0x7cff92fc, 90 },
    { 0x37cf92fc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243f9305, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bf9308, 76 },
    { 0xa31f9308, 34 },
    { 0x99cf930a, 87 },
    { 0x6e1f930b, 78 },
    { 0, 0 },
    { 0xbcbf930d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f3f9311, 61 },
    { 0xa6af9312, 93 },
    { 0xd51f9313, 23 },
    { 0, 0 },
    { 0x9c4f9315, 90 },
    { 0x515f9315, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1f931d, 23 },
    { 0xfd7f931d, 33 },
    { 0x6b9f931f, 33 },
    { 0, 0 },
    { 0x6b1f9321, 61 },
    { 0x64af9322, 23 },
    { 0xb57f9322, 75 },
    { 0, 0 },
    { 0x9f0f9325, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x749f932a, 34 },
    { 0x455f932a, 95 },
    { 0x9b4f932c, 98 },
    { 0x2b3f932c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a0f9331, 55 },
    { 0xb4ff9332, 90 },
    { 0xca8f9333, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47ff9339, 76 },
    { 0, 0 },
    { 0xef9f933b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10ef9340, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30f9346, 52 },
    { 0xeb6f9347, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71f934d, 90 },
    { 0xe06f934d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4f9351, 23 },
    { 0, 0 },
    { 0xe20f9353, 46 },
    { 0x7f6f9354, 90 },
    { 0x328f9353, 66 },
    { 0x334f9355, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x635f935c, 55 },
    { 0, 0 },
    { 0x877f935e, 43 },
    { 0xd28f935f, 61 },
    { 0xab9f9360, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303f9364, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa25f9369, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x488f936c, 76 },
    { 0, 0 },
    { 0xbb7f936e, 90 },
    { 0x8e1f936e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3f9373, 33 },
    { 0xfe7f9374, 33 },
    { 0x685f9375, 95 },
    { 0xcedf9376, 76 },
    { 0, 0 },
    { 0x8dcf9378, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97f937c, 23 },
    { 0x629f937d, 23 },
    { 0x971f937d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11f9381, 90 },
    { 0x858f9381, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a2f9388, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x52f938b, 34 },
    { 0x81df938c, 90 },
    { 0xbbbf938d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bff9390, 76 },
    { 0x624f9391, 81 },
    { 0x1edf9392, 78 },
    { 0x6f1f9393, 23 },
    { 0x71ff9391, 61 },
    { 0xfcdf9392, 61 },
    { 0x62f9396, 77 },
    { 0x347f9393, 61 },
    { 0xaa2f9398, 77 },
    { 0x66f9399, 37 },
    { 0, 0 },
    { 0xd5ef939b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x118f939e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bff93a1, 65 },
    { 0x980f93a2, 33 },
    { 0x697f93a3, 46 },
    { 0x1fbf93a4, 23 },
    { 0xb71f93a1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bf93a9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567f93b4, 33 },
    { 0xdfaf93b5, 77 },
    { 0xd86f93b5, 90 },
    { 0x7f4f93b7, 68 },
    { 0xee1f93b4, 52 },
    { 0, 0 },
    { 0xb8cf93ba, 98 },
    { 0xc01f93bb, 93 },
    { 0x30af93ba, 95 },
    { 0xd37f93bd, 61 },
    { 0x6f9f93be, 66 },
    { 0, 0 },
    { 0x266f93c0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x84af93c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96df93c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7daf93ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x22df93d1, 85 },
    { 0, 0 },
    { 0xb6cf93d3, 43 },
    { 0, 0 },
    { 0x6cf93d5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bf93db, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04f93df, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x104f93e3, 33 },
    { 0xb3bf93e4, 61 },
    { 0xca1f93e5, 33 },
    { 0x361f93e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9f93ea, 98 },
    { 0xa98f93ea, 33 },
    { 0x806f93ec, 61 },
    { 0xc48f93ed, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x959f93f0, 33 },
    { 0x962f93f0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda5f93f6, 34 },
    { 0, 0 },
    { 0xd3af93f8, 46 },
    { 0x729f93f8, 52 },
    { 0, 0 },
    { 0xedff93fb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x100f93fe, 95 },
    { 0, 0 },
    { 0x8e2f9400, 100 },
    { 0xd84f9401, 78 },
    { 0x689f9402, 59 },
    { 0x449f9403, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x145f940b, 74 },
    { 0x6eff940c, 99 },
    { 0x1ebf940c, 66 },
    { 0, 0 },
    { 0x704f940f, 98 },
    { 0xa51f9410, 95 },
    { 0xcb1f9411, 91 },
    { 0xc93f9410, 61 },
    { 0x8bbf9412, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64df9417, 61 },
    { 0xb42f9417, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71f941b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x390f9422, 67 },
    { 0, 0 },
    { 0x899f9424, 90 },
    { 0, 0 },
    { 0x90af9426, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f942c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8baf9433, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0f943d, 23 },
    { 0, 0 },
    { 0x101f943f, 85 },
    { 0x328f943f, 67 },
    { 0x657f9441, 43 },
    { 0x416f9441, 55 },
    { 0xee6f9443, 34 },
    { 0x13cf9444, 95 },
    { 0xbaaf9444, 66 },
    { 0x20ef9441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45f944c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ff9452, 46 },
    { 0x932f9453, 33 },
    { 0xe87f9453, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf945a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307f9460, 99 },
    { 0xc56f9461, 75 },
    { 0xae1f9461, 90 },
    { 0xe81f9462, 90 },
    { 0, 0 },
    { 0xc06f9465, 52 },
    { 0x993f9466, 61 },
    { 0x795f9467, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0f946e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedaf9473, 61 },
    { 0xf77f9474, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf49f9479, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769f947e, 3 },
    { 0xd35f947f, 95 },
    { 0x1d5f947f, 80 },
    { 0xac2f9481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fcf9486, 33 },
    { 0x2c3f9487, 87 },
    { 0x7a0f9486, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6f948d, 75 },
    { 0x7cbf948e, 84 },
    { 0x4ef948d, 98 },
    { 0, 0 },
    { 0xeb1f9491, 100 },
    { 0x30df9492, 55 },
    { 0x941f9491, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cef9498, 61 },
    { 0, 0 },
    { 0xd95f949a, 46 },
    { 0xf2bf949b, 43 },
    { 0xee4f949c, 61 },
    { 0xe92f949b, 23 },
    { 0xb40f949e, 34 },
    { 0, 0 },
    { 0xd7ef94a0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd5f94a3, 46 },
    { 0xed2f94a4, 90 },
    { 0x5a9f94a5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x433f94a8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x260f94ab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x803f94b0, 81 },
    { 0, 0 },
    { 0x670f94b2, 40 },
    { 0x5bcf94b3, 33 },
    { 0xa4ef94b3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62df94bc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ff94c2, 23 },
    { 0xecef94c3, 61 },
    { 0xf21f94c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecf94c9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ff94cc, 23 },
    { 0x42f94cc, 93 },
    { 0, 0 },
    { 0xc9f94cf, 43 },
    { 0, 0 },
    { 0x6a6f94d1, 84 },
    { 0x44ff94d1, 33 },
    { 0xa95f94d3, 61 },
    { 0x8e4f94d4, 90 },
    { 0x6a4f94d5, 76 },
    { 0x859f94d3, 34 },
    { 0xa8f94d3, 77 },
    { 0xa34f94d7, 33 },
    { 0x324f94d9, 90 },
    { 0xb5ff94d9, 33 },
    { 0xd03f94d9, 49 },
    { 0x513f94dc, 23 },
    { 0x28cf94dc, 90 },
    { 0xad1f94da, 43 },
    { 0x1d2f94dd, 98 },
    { 0x77df94e0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b4f94ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38f94ef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x629f94f2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x36f94f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dcf94fc, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0x83bf94ff, 52 },
    { 0x5bf94ff, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xac0f9503, 92 },
    { 0, 0 },
    { 0xb9ff9505, 43 },
    { 0x75bf9506, 80 },
    { 0xedbf9507, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dff950e, 101 },
    { 0, 0 },
    { 0x294f9510, 61 },
    { 0, 0 },
    { 0x36df9512, 46 },
    { 0, 0 },
    { 0xa80f9514, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2f9522, 90 },
    { 0x73bf9523, 33 },
    { 0x543f9523, 34 },
    { 0x67cf9525, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904f952a, 55 },
    { 0xfc4f952b, 33 },
    { 0x41f952c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ef9530, 91 },
    { 0x250f9531, 90 },
    { 0xcacf9532, 93 },
    { 0x534f9532, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26f9536, 84 },
    { 0, 0 },
    { 0x7b8f9538, 61 },
    { 0, 0 },
    { 0xcdcf953a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcdaf953d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bf9541, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12df9547, 99 },
    { 0, 0 },
    { 0xc40f9549, 99 },
    { 0xfb2f954a, 23 },
    { 0x2eaf954b, 90 },
    { 0xe26f954b, 55 },
    { 0, 0 },
    { 0x671f954e, 61 },
    { 0, 0 },
    { 0x686f9550, 33 },
    { 0xab1f9551, 61 },
    { 0xf7f9550, 81 },
    { 0xe4cf9553, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x218f9556, 33 },
    { 0x47cf9557, 100 },
    { 0x591f9558, 66 },
    { 0, 0 },
    { 0xfe5f955a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41f955e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ff9565, 36 },
    { 0, 0 },
    { 0x1a2f9567, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b0f956d, 61 },
    { 0x3f6f956d, 90 },
    { 0x894f956d, 43 },
    { 0x8eff9570, 91 },
    { 0x2c0f9570, 76 },
    { 0, 0 },
    { 0xf45f9573, 55 },
    { 0, 0 },
    { 0xf9575, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0f9579, 73 },
    { 0, 0 },
    { 0x4c8f957b, 33 },
    { 0x6f7f957b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x717f9581, 61 },
    { 0, 0 },
    { 0xbb0f9583, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x152f9587, 65 },
    { 0, 0 },
    { 0xf1ef9589, 92 },
    { 0x1a6f9589, 90 },
    { 0xb31f9589, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeedf9594, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cf9597, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f0f959d, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1df95a5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaff95ae, 98 },
    { 0, 0 },
    { 0x1faf95b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514f95b5, 34 },
    { 0xf06f95b6, 85 },
    { 0xdc3f95b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494f95bd, 23 },
    { 0xb07f95be, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x243f95ca, 33 },
    { 0xebff95cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x866f95d3, 96 },
    { 0x253f95d4, 46 },
    { 0, 0 },
    { 0x2acf95d6, 43 },
    { 0xe26f95d7, 53 },
    { 0x1e7f95d7, 61 },
    { 0x1a3f95d7, 75 },
    { 0, 0 },
    { 0x77ff95db, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769f95eb, 33 },
    { 0, 0 },
    { 0x7fff95ed, 100 },
    { 0, 0 },
    { 0xa13f95ef, 33 },
    { 0xb9ef95f0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbccf95f7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9f95fa, 98 },
    { 0xcbcf95fb, 61 },
    { 0, 0 },
    { 0xc8cf95fd, 33 },
    { 0x385f95fd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50f9601, 23 },
    { 0x4b3f9601, 86 },
    { 0x40ff9603, 76 },
    { 0xfcaf9601, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdf9608, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3f960b, 23 },
    { 0xfc9f960c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fdf960f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ff9613, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b8f9617, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fef961d, 43 },
    { 0x78ef961d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x899f9622, 98 },
    { 0xbc1f9622, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9628, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61ff962e, 55 },
    { 0x82ef962e, 95 },
    { 0, 0 },
    { 0x658f9631, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9ef9636, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190f963c, 55 },
    { 0x864f963d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c9f9648, 98 },
    { 0, 0 },
    { 0xe30f964a, 40 },
    { 0, 0 },
    { 0xc77f964c, 87 },
    { 0, 0 },
    { 0x314f964e, 98 },
    { 0xd6df964e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbf9654, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745f9659, 43 },
    { 0x3acf965a, 76 },
    { 0xef3f965a, 66 },
    { 0x4ff965c, 98 },
    { 0xf17f965d, 61 },
    { 0xff7f965d, 33 },
    { 0x238f965a, 84 },
    { 0x178f9660, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x508f9665, 33 },
    { 0, 0 },
    { 0x115f9667, 90 },
    { 0, 0 },
    { 0x46af9669, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aaf966d, 98 },
    { 0xa2cf966e, 76 },
    { 0x4adf966e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x478f9676, 46 },
    { 0xdc8f9677, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeaf967b, 78 },
    { 0x4eff967c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1f9681, 33 },
    { 0x15bf9682, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48ff9690, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddf9695, 23 },
    { 0x939f9695, 61 },
    { 0x45bf9696, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6bf969d, 90 },
    { 0, 0 },
    { 0x2b5f969f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211f96a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3f96a9, 90 },
    { 0x4ccf96aa, 55 },
    { 0, 0 },
    { 0x828f96ac, 52 },
    { 0xd6df96ac, 64 },
    { 0x46cf96ae, 23 },
    { 0xa3ef96ad, 91 },
    { 0x15df96b0, 90 },
    { 0x3e1f96b0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af96b5, 98 },
    { 0xe37f96b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb45f96b9, 61 },
    { 0xa1ff96b9, 100 },
    { 0, 0 },
    { 0x481f96bc, 33 },
    { 0, 0 },
    { 0x4faf96be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3f96c3, 97 },
    { 0xd04f96c3, 98 },
    { 0x55bf96c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbf96c8, 55 },
    { 0x15af96c8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x25bf96cc, 84 },
    { 0x93f96cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaf96d5, 76 },
    { 0, 0 },
    { 0x9c9f96d7, 90 },
    { 0x115f96d8, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ef96dc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bf96df, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56f96e4, 76 },
    { 0x835f96e4, 81 },
    { 0x7def96e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105f96ec, 55 },
    { 0xc9f96ed, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xb15f96f0, 98 },
    { 0xfb9f96f0, 77 },
    { 0xbecf96f2, 84 },
    { 0x20af96f3, 66 },
    { 0, 0 },
    { 0xc52f96f5, 76 },
    { 0x8f96f6, 17 },
    { 0x914f96f7, 90 },
    { 0xd91f96f8, 66 },
    { 0, 0 },
    { 0x9f9f96fa, 87 },
    { 0xf35f96fa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x288f96fe, 43 },
    { 0x508f96ff, 55 },
    { 0x3adf96ff, 34 },
    { 0xd5f9701, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddf9707, 91 },
    { 0, 0 },
    { 0xbc6f9709, 33 },
    { 0, 0 },
    { 0x22bf970b, 78 },
    { 0xa1af970b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x56cf970f, 76 },
    { 0x16f9710, 23 },
    { 0x214f9711, 95 },
    { 0x29f9712, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2f971e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb70f9721, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a3f9727, 42 },
    { 0xf87f9728, 33 },
    { 0x197f9729, 99 },
    { 0x254f9728, 5 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafcf9731, 46 },
    { 0x1aff9731, 61 },
    { 0x4af9733, 84 },
    { 0x5aaf9732, 95 },
    { 0x207f9732, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a0f973c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61bf9742, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09f9748, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x493f974b, 61 },
    { 0, 0 },
    { 0x5faf974d, 90 },
    { 0, 0 },
    { 0xfcbf974f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32af9753, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234f9757, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc07f975e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccf9762, 43 },
    { 0x52df9763, 76 },
    { 0x9af9763, 66 },
    { 0x13ef9762, 34 },
    { 0x810f9766, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33f976b, 90 },
    { 0xbfff976b, 61 },
    { 0xadf976c, 76 },
    { 0x148f976e, 98 },
    { 0xb5cf976f, 84 },
    { 0x30ef976f, 61 },
    { 0x586f9770, 87 },
    { 0x159f976d, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bf9775, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2f977c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ef9780, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a9f9786, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2af978a, 33 },
    { 0x606f978b, 95 },
    { 0, 0 },
    { 0x2c2f978d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee3f9792, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86f9796, 98 },
    { 0xa8af9797, 77 },
    { 0, 0 },
    { 0x79af9799, 91 },
    { 0, 0 },
    { 0x1aaf979b, 90 },
    { 0xcd0f979c, 98 },
    { 0xb57f979b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ddf97a7, 81 },
    { 0, 0 },
    { 0xddef97a9, 90 },
    { 0x2aaf97a9, 97 },
    { 0xd2ff97a9, 23 },
    { 0xef7f97ab, 64 },
    { 0, 0 },
    { 0xd92f97ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x286f97b1, 52 },
    { 0xc1cf97b1, 17 },
    { 0x8a6f97b1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47f97be, 30 },
    { 0x126f97bf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae2f97c4, 61 },
    { 0, 0 },
    { 0xd6cf97c6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab3f97cc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4f97d0, 84 },
    { 0x168f97d1, 95 },
    { 0, 0 },
    { 0xf47f97d3, 95 },
    { 0xfc7f97d4, 77 },
    { 0, 0 },
    { 0xca7f97d6, 76 },
    { 0, 0 },
    { 0x8e8f97d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee0f97e0, 61 },
    { 0x58ff97e1, 95 },
    { 0, 0 },
    { 0x56f97e3, 43 },
    { 0, 0 },
    { 0xd9cf97e5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356f97ea, 46 },
    { 0x3bcf97eb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb7f97f5, 76 },
    { 0xad1f97f6, 61 },
    { 0x9abf97f7, 52 },
    { 0x674f97f7, 61 },
    { 0x34f97f9, 90 },
    { 0x204f97f7, 55 },
    { 0, 0 },
    { 0xf39f97fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37f9801, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc86f9804, 46 },
    { 0x8f3f9805, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x281f9808, 95 },
    { 0x7c9f9809, 91 },
    { 0, 0 },
    { 0x88ef980b, 23 },
    { 0x956f980c, 70 },
    { 0x735f980c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd2f9813, 91 },
    { 0x612f9814, 33 },
    { 0, 0 },
    { 0xefef9816, 98 },
    { 0x265f9817, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cf981a, 76 },
    { 0xb5bf981b, 90 },
    { 0x45af981c, 99 },
    { 0x78cf981d, 76 },
    { 0xa53f981e, 61 },
    { 0x33cf981f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41df9824, 90 },
    { 0, 0 },
    { 0xd62f9826, 17 },
    { 0x674f9827, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaef982a, 76 },
    { 0, 0 },
    { 0x658f982c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9bf9834, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x485f983b, 33 },
    { 0xcb4f983c, 43 },
    { 0x7e4f983d, 95 },
    { 0xdb9f983e, 98 },
    { 0xa19f983e, 61 },
    { 0xc6f983f, 61 },
    { 0x1e2f983f, 91 },
    { 0x818f983f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aef9846, 61 },
    { 0, 0 },
    { 0x93ef9848, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1af984b, 100 },
    { 0x329f984c, 33 },
    { 0, 0 },
    { 0x986f984e, 23 },
    { 0, 0 },
    { 0x936f9850, 43 },
    { 0x91cf9851, 67 },
    { 0x3df9851, 33 },
    { 0, 0 },
    { 0x679f9854, 98 },
    { 0x13cf9855, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ebf985d, 23 },
    { 0, 0 },
    { 0x538f985f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bff9865, 43 },
    { 0x542f9866, 33 },
    { 0xec2f9867, 61 },
    { 0x40ef9866, 90 },
    { 0xfdaf9869, 85 },
    { 0x15cf986a, 43 },
    { 0xc9ff9868, 61 },
    { 0, 0 },
    { 0x296f986d, 98 },
    { 0x765f986d, 23 },
    { 0x852f986e, 87 },
    { 0xa69f986e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f9876, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4cf9881, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xbecf9884, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3edf9888, 52 },
    { 0x58af9888, 64 },
    { 0x42bf988a, 61 },
    { 0xdc4f988a, 90 },
    { 0x544f988c, 90 },
    { 0x476f988b, 43 },
    { 0, 0 },
    { 0xe87f988f, 23 },
    { 0x630f9890, 40 },
    { 0x995f9891, 80 },
    { 0x648f9892, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578f989a, 66 },
    { 0, 0 },
    { 0xda3f989c, 23 },
    { 0xeddf989d, 91 },
    { 0xc00f989e, 55 },
    { 0x770f989f, 33 },
    { 0x501f989e, 53 },
    { 0x646f989d, 95 },
    { 0xf1cf98a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd3f98a9, 90 },
    { 0x23ff98aa, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7f98af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9f98b3, 81 },
    { 0, 0 },
    { 0x4aff98b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ef98bc, 34 },
    { 0x352f98bd, 90 },
    { 0, 0 },
    { 0x620f98bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3f98ca, 64 },
    { 0, 0 },
    { 0x4baf98cc, 46 },
    { 0x68af98cc, 95 },
    { 0x185f98cc, 33 },
    { 0x783f98cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4f98d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936f98d9, 91 },
    { 0x54f98da, 100 },
    { 0, 0 },
    { 0x116f98dc, 34 },
    { 0xd6df98dd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb1f98e1, 93 },
    { 0xccf98e2, 61 },
    { 0x5b3f98e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7cf98ea, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0df98ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x720f98f2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80f98f7, 95 },
    { 0, 0 },
    { 0xb58f98f9, 33 },
    { 0, 0 },
    { 0x712f98fb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb74f9908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b2f990b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ff990e, 98 },
    { 0, 0 },
    { 0x6eaf9910, 61 },
    { 0x454f9911, 98 },
    { 0, 0 },
    { 0xa24f9913, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96df9918, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaef991b, 92 },
    { 0x507f991c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437f9927, 75 },
    { 0, 0 },
    { 0xb0af9929, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97df9930, 77 },
    { 0xdc2f9930, 100 },
    { 0x692f9931, 34 },
    { 0xcf0f9931, 95 },
    { 0, 0 },
    { 0x1e8f9935, 90 },
    { 0xfcef9936, 100 },
    { 0, 0 },
    { 0xfd4f9938, 52 },
    { 0xf23f9938, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781f9945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x967f9949, 95 },
    { 0xa94f994a, 34 },
    { 0, 0 },
    { 0x29f994c, 90 },
    { 0x6a9f994d, 59 },
    { 0x356f994e, 90 },
    { 0x5eef994d, 95 },
    { 0xec3f994c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902f9956, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aff995b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x915f9961, 90 },
    { 0xf09f9962, 98 },
    { 0xeb5f9963, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24f9966, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1af9969, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x546f996d, 33 },
    { 0, 0 },
    { 0x450f996f, 55 },
    { 0xe51f9970, 90 },
    { 0x402f9971, 98 },
    { 0, 0 },
    { 0x7ecf9973, 88 },
    { 0xb59f9973, 66 },
    { 0, 0 },
    { 0xe97f9976, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa42f997a, 81 },
    { 0x80df997b, 23 },
    { 0x1f1f997c, 90 },
    { 0xc1ff997a, 33 },
    { 0xd89f997c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d1f9981, 99 },
    { 0, 0 },
    { 0xf9ef9983, 36 },
    { 0xbd1f9983, 99 },
    { 0, 0 },
    { 0x455f9986, 52 },
    { 0, 0 },
    { 0x548f9988, 23 },
    { 0x2cdf9988, 46 },
    { 0, 0 },
    { 0x14bf998b, 88 },
    { 0x4d0f998c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1f9994, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11f999a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ef999e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9f99a4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaf99a7, 91 },
    { 0xe67f99a8, 61 },
    { 0x27f99a7, 23 },
    { 0x63df99aa, 95 },
    { 0, 0 },
    { 0x94ef99ac, 95 },
    { 0x801f99ac, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1f99b3, 98 },
    { 0x799f99b4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60ef99bc, 23 },
    { 0x561f99bd, 93 },
    { 0xbddf99bc, 33 },
    { 0xdef99bf, 90 },
    { 0xb24f99c0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17df99c5, 33 },
    { 0x608f99c6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cf99ca, 33 },
    { 0x54f99cb, 90 },
    { 0xa83f99cc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f99cf, 98 },
    { 0, 0 },
    { 0x631f99d1, 76 },
    { 0x1eef99d1, 80 },
    { 0x728f99d3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3f99d6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3f99dd, 76 },
    { 0x495f99de, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15f99e5, 95 },
    { 0xd58f99e6, 98 },
    { 0, 0 },
    { 0x47cf99e8, 90 },
    { 0, 0 },
    { 0xec0f99ea, 90 },
    { 0xf92f99eb, 100 },
    { 0xc49f99eb, 55 },
    { 0x491f99ed, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9f99f0, 17 },
    { 0xaadf99f1, 91 },
    { 0xd5cf99f0, 98 },
    { 0x964f99f2, 98 },
    { 0, 0 },
    { 0x9dcf99f5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97f99fe, 95 },
    { 0x1cff99ff, 97 },
    { 0, 0 },
    { 0x188f9a01, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4ef9a08, 33 },
    { 0, 0 },
    { 0x414f9a0a, 52 },
    { 0x93cf9a0b, 17 },
    { 0xde7f9a0c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x33df9a0f, 88 },
    { 0x9c9f9a10, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1f9a17, 90 },
    { 0x66ff9a17, 90 },
    { 0x242f9a19, 90 },
    { 0x86df9a17, 99 },
    { 0x4e3f9a1a, 87 },
    { 0, 0 },
    { 0xd0f9a1d, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x387f9a20, 90 },
    { 0xddaf9a20, 33 },
    { 0xc59f9a21, 47 },
    { 0x68ff9a21, 55 },
    { 0x778f9a22, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cf9a2a, 93 },
    { 0x18af9a2a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc35f9a36, 90 },
    { 0x640f9a37, 33 },
    { 0x911f9a38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcef9a3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd15f9a42, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeff9a49, 93 },
    { 0, 0 },
    { 0xa71f9a4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcf9a4e, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2f9a51, 23 },
    { 0x72bf9a52, 69 },
    { 0, 0 },
    { 0x2cbf9a54, 43 },
    { 0xca8f9a54, 77 },
    { 0x599f9a55, 88 },
    { 0x86cf9a57, 90 },
    { 0xae9f9a56, 55 },
    { 0, 0 },
    { 0x7a6f9a5a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa03f9a5d, 78 },
    { 0, 0 },
    { 0x6cef9a5f, 61 },
    { 0x522f9a60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cf9a67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632f9a6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x213f9a72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14f9a75, 23 },
    { 0xcf1f9a76, 90 },
    { 0x67cf9a76, 95 },
    { 0x476f9a78, 88 },
    { 0x26ff9a79, 84 },
    { 0x1a2f9a76, 55 },
    { 0x588f9a79, 95 },
    { 0x36bf9a7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2abf9a82, 33 },
    { 0x59f9a83, 95 },
    { 0x44bf9a83, 33 },
    { 0xcdcf9a85, 90 },
    { 0x34cf9a86, 61 },
    { 0x9bdf9a86, 33 },
    { 0x30cf9a88, 84 },
    { 0xf17f9a88, 61 },
    { 0x300f9a89, 23 },
    { 0x829f9a8b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61cf9a90, 55 },
    { 0xf99f9a91, 34 },
    { 0xc64f9a92, 95 },
    { 0x68bf9a92, 98 },
    { 0xff4f9a93, 33 },
    { 0x984f9a95, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x243f9a98, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x429f9a9c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa37f9aa1, 90 },
    { 0x4f4f9aa2, 46 },
    { 0, 0 },
    { 0x448f9aa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1f9aaa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98ef9ab0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbf9ab7, 99 },
    { 0x76f9ab7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdf9abd, 33 },
    { 0x1cbf9abe, 90 },
    { 0x15df9abe, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x311f9ac9, 43 },
    { 0, 0 },
    { 0xed7f9acb, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2f9ace, 76 },
    { 0x36f9acf, 66 },
    { 0x168f9acf, 33 },
    { 0x1bcf9acf, 33 },
    { 0xebef9ad2, 90 },
    { 0xd18f9ad3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7f9ad7, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe72f9ada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd14f9ae3, 93 },
    { 0, 0 },
    { 0x9cef9ae5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88f9ae9, 33 },
    { 0xf8df9aea, 75 },
    { 0, 0 },
    { 0xcdf9aec, 66 },
    { 0xf53f9aed, 98 },
    { 0x926f9aee, 23 },
    { 0x192f9aef, 90 },
    { 0, 0 },
    { 0x1cef9af1, 61 },
    { 0, 0 },
    { 0xdf1f9af3, 69 },
    { 0xbbcf9af4, 33 },
    { 0xc5af9af5, 55 },
    { 0x8def9af6, 61 },
    { 0xb6af9af5, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bf9afa, 33 },
    { 0xd5af9afb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c4f9b04, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xebcf9b07, 61 },
    { 0x4f9f9b07, 23 },
    { 0, 0 },
    { 0x5b6f9b0a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ff9b0d, 66 },
    { 0x29df9b0e, 55 },
    { 0x9e5f9b0d, 61 },
    { 0xccaf9b0d, 78 },
    { 0x5a1f9b11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe50f9b15, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28ef9b19, 90 },
    { 0x852f9b1a, 90 },
    { 0x9bcf9b1b, 66 },
    { 0xd15f9b1a, 90 },
    { 0xa40f9b19, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56ef9b22, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x974f9b25, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x536f9b28, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff9b2b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54f9b2e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x795f9b33, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x683f9b39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb2f9b3c, 98 },
    { 0, 0 },
    { 0x4a8f9b3e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806f9b42, 80 },
    { 0, 0 },
    { 0x9bff9b44, 69 },
    { 0x65ef9b44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e9f9b49, 84 },
    { 0x33f9b4a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x69cf9b4d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bf9b50, 90 },
    { 0, 0 },
    { 0x871f9b52, 34 },
    { 0, 0 },
    { 0xa6af9b54, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eaf9b58, 33 },
    { 0, 0 },
    { 0xb62f9b5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91f9b5d, 95 },
    { 0x147f9b5e, 98 },
    { 0xed5f9b5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x250f9b64, 33 },
    { 0x331f9b65, 84 },
    { 0, 0 },
    { 0xe6cf9b67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x846f9b6a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0f9b6d, 75 },
    { 0x402f9b6e, 88 },
    { 0x1b1f9b6e, 90 },
    { 0x129f9b6f, 23 },
    { 0xdacf9b71, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x664f9b7a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x657f9b81, 87 },
    { 0, 0 },
    { 0x3a2f9b83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99f9b86, 95 },
    { 0xca4f9b86, 34 },
    { 0xc1af9b88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bf9b8e, 95 },
    { 0xcc3f9b8e, 35 },
    { 0, 0 },
    { 0x860f9b91, 43 },
    { 0xe75f9b92, 93 },
    { 0xdbbf9b93, 77 },
    { 0x9e4f9b91, 96 },
    { 0x3c8f9b95, 90 },
    { 0xf2df9b96, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f9b99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x652f9b9d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2edf9ba1, 33 },
    { 0, 0 },
    { 0xdfdf9ba3, 90 },
    { 0x3dcf9ba3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe29f9ba9, 66 },
    { 0xa7df9baa, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb3f9baf, 61 },
    { 0x535f9bb0, 33 },
    { 0x290f9bb1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x995f9bb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x83df9bb9, 85 },
    { 0xafdf9bb9, 95 },
    { 0, 0 },
    { 0x31bf9bbc, 93 },
    { 0x1f8f9bbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bf9bcc, 90 },
    { 0x812f9bcd, 76 },
    { 0, 0 },
    { 0x98ff9bcf, 90 },
    { 0xb06f9bd0, 56 },
    { 0xf54f9bd0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3def9bd9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x191f9bdc, 43 },
    { 0xeedf9bdd, 46 },
    { 0x542f9bdc, 55 },
    { 0xe28f9bdf, 84 },
    { 0xf2af9be0, 76 },
    { 0xfaf9be0, 43 },
    { 0xbeef9bdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c3f9be7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x280f9bee, 98 },
    { 0x5caf9bee, 81 },
    { 0xd47f9bf0, 99 },
    { 0xc81f9bf0, 93 },
    { 0, 0 },
    { 0xf5ff9bf3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ef9bfa, 23 },
    { 0x5c2f9bfa, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ef9c05, 61 },
    { 0x780f9c06, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04f9c0b, 23 },
    { 0x7adf9c0c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ff9c12, 43 },
    { 0xb07f9c13, 61 },
    { 0, 0 },
    { 0xac1f9c15, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x442f9c18, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cf9c20, 90 },
    { 0, 0 },
    { 0xa08f9c22, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1f9c25, 66 },
    { 0, 0 },
    { 0x333f9c27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x796f9c2e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bbf9c32, 66 },
    { 0x615f9c33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe67f9c3a, 33 },
    { 0x9bdf9c3a, 33 },
    { 0x6bcf9c3a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcff9c3f, 88 },
    { 0x741f9c40, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf82f9c45, 90 },
    { 0x189f9c46, 40 },
    { 0x738f9c46, 93 },
    { 0, 0 },
    { 0x9a2f9c49, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x602f9c4c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a4f9c53, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2f9c56, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318f9c5c, 33 },
    { 0, 0 },
    { 0x128f9c5e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f7f9c64, 23 },
    { 0xf77f9c65, 23 },
    { 0x25bf9c66, 77 },
    { 0x2a3f9c67, 100 },
    { 0xf47f9c64, 90 },
    { 0, 0 },
    { 0x86bf9c6a, 76 },
    { 0x1a2f9c6b, 76 },
    { 0, 0 },
    { 0x851f9c6d, 90 },
    { 0x2caf9c6e, 93 },
    { 0xf01f9c6f, 19 },
    { 0xe8ff9c6f, 61 },
    { 0xcedf9c6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78f9c7b, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bf9c80, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88f9c87, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ff9c8a, 23 },
    { 0x53df9c8a, 76 },
    { 0, 0 },
    { 0xd21f9c8d, 61 },
    { 0xcf2f9c8d, 66 },
    { 0x515f9c8e, 98 },
    { 0, 0 },
    { 0x2dff9c91, 66 },
    { 0xc81f9c92, 23 },
    { 0, 0 },
    { 0xc98f9c94, 23 },
    { 0x47f9c94, 61 },
    { 0xb33f9c94, 46 },
    { 0x73ff9c97, 76 },
    { 0xe48f9c98, 90 },
    { 0x587f9c98, 23 },
    { 0xd01f9c96, 91 },
    { 0, 0 },
    { 0xc12f9c9c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x917f9ca0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7ef9ca3, 23 },
    { 0, 0 },
    { 0xe06f9ca5, 46 },
    { 0xb20f9ca5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa38f9caa, 33 },
    { 0x84bf9cab, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fef9cae, 65 },
    { 0, 0 },
    { 0x765f9cb0, 33 },
    { 0x416f9cb1, 61 },
    { 0, 0 },
    { 0x22df9cb3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf3f9cb6, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1ff9cba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf61f9cbd, 90 },
    { 0xd50f9cbe, 100 },
    { 0x5cff9cbe, 67 },
    { 0xa9ff9cbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa82f9cc7, 87 },
    { 0xdaff9cc8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14f9ccb, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x173f9cce, 23 },
    { 0x3f5f9ccf, 55 },
    { 0xf76f9cd0, 33 },
    { 0x8c5f9cd1, 38 },
    { 0xd9df9cd2, 23 },
    { 0x618f9cd3, 98 },
    { 0xdbbf9cd3, 91 },
    { 0x4fbf9cd5, 23 },
    { 0xa57f9cd3, 34 },
    { 0x58df9cd1, 55 },
    { 0x66df9cd8, 90 },
    { 0xfedf9cd9, 100 },
    { 0xff9f9cd3, 66 },
    { 0x664f9cd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32ef9ce0, 46 },
    { 0x14cf9ce1, 90 },
    { 0x78bf9ce1, 61 },
    { 0, 0 },
    { 0xa75f9ce4, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ebf9ce8, 84 },
    { 0xd04f9ce9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ff9cf1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2f9d00, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cf9d05, 100 },
    { 0, 0 },
    { 0x1bf9d07, 97 },
    { 0xf6ff9d08, 81 },
    { 0xbadf9d07, 48 },
    { 0x11cf9d0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda0f9d0e, 52 },
    { 0x858f9d0f, 66 },
    { 0x9f6f9d0e, 46 },
    { 0x306f9d11, 49 },
    { 0xeb7f9d11, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa2f9d18, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeef9d1f, 23 },
    { 0xd30f9d20, 95 },
    { 0, 0 },
    { 0x70ef9d22, 93 },
    { 0x10df9d23, 76 },
    { 0, 0 },
    { 0x499f9d25, 34 },
    { 0x2eaf9d25, 84 },
    { 0xc0f9d27, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bbf9d2a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebf9d2d, 90 },
    { 0x761f9d2e, 61 },
    { 0xbbff9d2f, 23 },
    { 0x9b5f9d2f, 33 },
    { 0xa32f9d2f, 33 },
    { 0x928f9d30, 33 },
    { 0x98cf9d33, 33 },
    { 0, 0 },
    { 0xc5ef9d35, 66 },
    { 0x485f9d36, 78 },
    { 0xf33f9d36, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f9d3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x285f9d41, 90 },
    { 0xdbff9d42, 95 },
    { 0x315f9d43, 23 },
    { 0xe68f9d42, 33 },
    { 0x31af9d43, 90 },
    { 0, 0 },
    { 0x329f9d47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x364f9d4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x233f9d52, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0x7caf9d55, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6af9d5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0f9d5d, 95 },
    { 0xbedf9d5d, 78 },
    { 0x1c9f9d5f, 98 },
    { 0x64bf9d60, 90 },
    { 0xc9af9d60, 90 },
    { 0xcddf9d62, 90 },
    { 0x4fff9d5e, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0x962f9d66, 61 },
    { 0x36df9d67, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8bf9d6a, 40 },
    { 0x30ff9d6a, 61 },
    { 0xddef9d6a, 95 },
    { 0xfbdf9d6d, 90 },
    { 0xc65f9d6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7f9d72, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x234f9d75, 43 },
    { 0xf31f9d76, 33 },
    { 0x20af9d77, 34 },
    { 0x99af9d78, 33 },
    { 0x5a2f9d78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98af9d7d, 76 },
    { 0xe8cf9d7e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x399f9d81, 90 },
    { 0xf69f9d82, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ff9d87, 61 },
    { 0xf3df9d88, 99 },
    { 0, 0 },
    { 0xddbf9d8a, 85 },
    { 0x5dff9d8b, 87 },
    { 0x3c8f9d8c, 55 },
    { 0, 0 },
    { 0x722f9d8e, 90 },
    { 0xdd9f9d8f, 34 },
    { 0x89ef9d90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x442f9d93, 98 },
    { 0xb70f9d94, 29 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3af9d9d, 33 },
    { 0x540f9d9e, 61 },
    { 0x70ef9d9d, 90 },
    { 0, 0 },
    { 0xc0af9da1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50f9da7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1def9dae, 34 },
    { 0, 0 },
    { 0x571f9db0, 33 },
    { 0, 0 },
    { 0xc80f9db2, 90 },
    { 0x81df9db2, 91 },
    { 0, 0 },
    { 0x216f9db5, 23 },
    { 0, 0 },
    { 0x4e0f9db7, 76 },
    { 0, 0 },
    { 0x2df9db9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141f9dbd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41ff9dc0, 61 },
    { 0x36df9dc1, 33 },
    { 0x6e0f9dc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x717f9dc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7f9dc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ef9dcb, 55 },
    { 0, 0 },
    { 0x6b6f9dcd, 98 },
    { 0x90ff9dce, 90 },
    { 0xc0df9dcf, 90 },
    { 0xa27f9dd0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x46ff9dd3, 23 },
    { 0, 0 },
    { 0x2fff9dd5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe1f9dda, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0f9dde, 43 },
    { 0xcef9dde, 93 },
    { 0, 0 },
    { 0x955f9de1, 100 },
    { 0x7c3f9de1, 23 },
    { 0xddef9de3, 61 },
    { 0x7d7f9de2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fbf9dee, 61 },
    { 0, 0 },
    { 0x665f9df0, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5bf9df6, 61 },
    { 0x7f2f9df6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79f9dfa, 34 },
    { 0, 0 },
    { 0x9f2f9dfc, 61 },
    { 0xca1f9dfc, 61 },
    { 0x5f0f9dfc, 61 },
    { 0xd40f9dff, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebf9e02, 17 },
    { 0, 0 },
    { 0x74bf9e04, 98 },
    { 0x68bf9e05, 33 },
    { 0, 0 },
    { 0xfa3f9e07, 23 },
    { 0x13ff9e07, 90 },
    { 0, 0 },
    { 0xb13f9e0a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49ff9e11, 90 },
    { 0x5d7f9e12, 34 },
    { 0xae1f9e12, 76 },
    { 0x17f9e14, 90 },
    { 0xc52f9e15, 55 },
    { 0xc91f9e15, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420f9e1d, 78 },
    { 0, 0 },
    { 0x5eef9e1f, 95 },
    { 0x9f7f9e1f, 33 },
    { 0x268f9e1f, 91 },
    { 0, 0 },
    { 0x900f9e23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9adf9e2a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x228f9e2f, 23 },
    { 0xc7ff9e30, 90 },
    { 0x907f9e2f, 98 },
    { 0xde8f9e31, 84 },
    { 0, 0 },
    { 0xe17f9e34, 61 },
    { 0x230f9e35, 61 },
    { 0x428f9e34, 95 },
    { 0x3d0f9e37, 76 },
    { 0xee1f9e38, 90 },
    { 0, 0 },
    { 0x751f9e3a, 33 },
    { 0x2cdf9e3a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eef9e41, 85 },
    { 0, 0 },
    { 0x46cf9e43, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe02f9e48, 98 },
    { 0x899f9e48, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79f9e4c, 55 },
    { 0x722f9e4d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadff9e50, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc96f9e54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3f9e57, 61 },
    { 0x847f9e58, 61 },
    { 0xd1af9e57, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9e62, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537f9e67, 33 },
    { 0x99ff9e67, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f9e6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x61af9e6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc2f9e76, 91 },
    { 0x910f9e77, 23 },
    { 0x5d7f9e78, 7 },
    { 0x24cf9e79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x559f9e7d, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cf9e82, 77 },
    { 0xe39f9e82, 76 },
    { 0x95f9e84, 33 },
    { 0, 0 },
    { 0xa87f9e86, 55 },
    { 0, 0 },
    { 0x70bf9e88, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffdf9e92, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd23f9e96, 81 },
    { 0x887f9e96, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98bf9e9c, 23 },
    { 0, 0 },
    { 0xc05f9e9e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2eff9ea2, 23 },
    { 0x3f8f9ea2, 61 },
    { 0xa8ff9ea4, 67 },
    { 0xa06f9ea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34df9ea9, 95 },
    { 0, 0 },
    { 0x61bf9eab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x170f9eb1, 55 },
    { 0xd8ef9eb2, 90 },
    { 0xdc5f9eb3, 95 },
    { 0, 0 },
    { 0xdb2f9eb5, 56 },
    { 0xa66f9eb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x885f9ebe, 99 },
    { 0, 0 },
    { 0x812f9ec0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ef9ec4, 87 },
    { 0x13cf9ec5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4f9ed0, 23 },
    { 0x694f9ed1, 93 },
    { 0xe6af9ed2, 61 },
    { 0x2c8f9ed2, 34 },
    { 0x232f9ed4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7f9ed7, 93 },
    { 0xf6af9ed7, 98 },
    { 0, 0 },
    { 0x60df9eda, 98 },
    { 0, 0 },
    { 0x2faf9edc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a7f9ee0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2f9ee6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bf9ee9, 35 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeaf9eec, 17 },
    { 0xfa7f9eed, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92cf9ef3, 33 },
    { 0, 0 },
    { 0x6fdf9ef5, 23 },
    { 0, 0 },
    { 0xdcaf9ef7, 76 },
    { 0xfdff9ef7, 95 },
    { 0x6aaf9ef9, 61 },
    { 0, 0 },
    { 0x3daf9efb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x12f9efe, 90 },
    { 0, 0 },
    { 0x1c8f9f00, 95 },
    { 0x486f9f01, 90 },
    { 0x87af9f01, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff9f05, 55 },
    { 0xdf9f06, 77 },
    { 0x4aaf9f05, 43 },
    { 0x7c6f9f07, 95 },
    { 0x961f9f06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9f9f0d, 76 },
    { 0xd57f9f0e, 61 },
    { 0xa04f9f0e, 95 },
    { 0xb41f9f10, 61 },
    { 0x455f9f11, 61 },
    { 0xfa9f9f0d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c6f9f16, 76 },
    { 0x56ef9f17, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdf9f1a, 76 },
    { 0, 0 },
    { 0xce9f9f1c, 43 },
    { 0, 0 },
    { 0x67ef9f1e, 67 },
    { 0x18ef9f1f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c2f9f23, 100 },
    { 0, 0 },
    { 0x87ef9f25, 99 },
    { 0x860f9f26, 64 },
    { 0xf7df9f26, 83 },
    { 0x6a4f9f25, 76 },
    { 0x843f9f29, 61 },
    { 0xdaf9f29, 89 },
    { 0xad4f9f2b, 34 },
    { 0x21ef9f2c, 61 },
    { 0x29bf9f2a, 55 },
    { 0x2c4f9f2d, 33 },
    { 0xd91f9f2f, 53 },
    { 0x9c8f9f30, 55 },
    { 0x3b1f9f30, 98 },
    { 0xac3f9f30, 91 },
    { 0x1b6f9f33, 90 },
    { 0xdf8f9f25, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x272f9f39, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d4f9f40, 23 },
    { 0, 0 },
    { 0x714f9f42, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x196f9f46, 95 },
    { 0xb25f9f47, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84df9f4c, 90 },
    { 0xefdf9f4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcf9f50, 33 },
    { 0, 0 },
    { 0xcddf9f52, 93 },
    { 0, 0 },
    { 0xcdcf9f54, 90 },
    { 0x3b6f9f54, 23 },
    { 0x63af9f56, 99 },
    { 0x309f9f57, 85 },
    { 0x367f9f57, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3ef9f5c, 33 },
    { 0x7ddf9f5c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41f9f61, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc70f9f65, 90 },
    { 0xbbdf9f65, 85 },
    { 0x5f8f9f67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30ff9f6b, 33 },
    { 0x4baf9f6c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfff9f70, 55 },
    { 0xaa9f9f70, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x548f9f76, 52 },
    { 0xf70f9f76, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a3f9f82, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf75f9f88, 23 },
    { 0, 0 },
    { 0x43ff9f8a, 85 },
    { 0xf52f9f8a, 95 },
    { 0xecbf9f8c, 76 },
    { 0, 0 },
    { 0xc76f9f8e, 98 },
    { 0xb3af9f8e, 98 },
    { 0x242f9f90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cdf9f94, 66 },
    { 0x78af9f95, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1ff9f9b, 23 },
    { 0, 0 },
    { 0xeb5f9f9d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9f9fa0, 91 },
    { 0xa40f9fa1, 98 },
    { 0x23af9fa2, 33 },
    { 0x691f9fa3, 33 },
    { 0x2aff9fa1, 48 },
    { 0x490f9fa4, 33 },
    { 0x434f9fa5, 76 },
    { 0xd8cf9fa7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4f9fad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3f9fb0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22cf9fb9, 90 },
    { 0, 0 },
    { 0x761f9fbb, 96 },
    { 0xadbf9fbc, 61 },
    { 0x1caf9fbd, 91 },
    { 0xedbf9fbe, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbcf9fc3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1f9fca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59ff9fd6, 88 },
    { 0x193f9fd7, 76 },
    { 0x41df9fd8, 81 },
    { 0xaeaf9fd8, 33 },
    { 0, 0 },
    { 0xcfdf9fdb, 76 },
    { 0xc77f9fdc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31df9fe3, 61 },
    { 0x72af9fe4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656f9fea, 23 },
    { 0, 0 },
    { 0x66ef9fec, 100 },
    { 0xcb5f9fec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7af9ff0, 23 },
    { 0x382f9ff0, 61 },
    { 0xce1f9ff2, 98 },
    { 0xfbef9ff3, 23 },
    { 0, 0 },
    { 0xb51f9ff5, 61 },
    { 0x153f9ff5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x573f9ff9, 95 },
    { 0xed2f9ffa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91fa007, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4fa00a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x211fa00f, 93 },
    { 0x556fa00f, 34 },
    { 0xa0cfa011, 43 },
    { 0x187fa011, 90 },
    { 0xa83fa013, 53 },
    { 0x5a1fa00f, 8 },
    { 0, 0 },
    { 0x2cfa016, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb05fa01c, 33 },
    { 0x4dbfa01c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf01fa021, 61 },
    { 0, 0 },
    { 0xb02fa023, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551fa029, 90 },
    { 0xb30fa02a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x467fa02d, 61 },
    { 0x3e8fa02e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9fa031, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd09fa036, 43 },
    { 0xdb1fa036, 90 },
    { 0xe3bfa038, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6fa03e, 33 },
    { 0x290fa03f, 61 },
    { 0x9fcfa040, 34 },
    { 0x4a6fa041, 55 },
    { 0xe21fa041, 56 },
    { 0x61ffa041, 90 },
    { 0x680fa043, 76 },
    { 0, 0 },
    { 0x165fa046, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fa04f, 91 },
    { 0x9e4fa050, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937fa058, 23 },
    { 0, 0 },
    { 0xa25fa05a, 67 },
    { 0x3eafa05b, 78 },
    { 0x1c0fa05c, 43 },
    { 0x7d9fa05d, 33 },
    { 0x945fa05d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa7fa061, 76 },
    { 0xab7fa061, 90 },
    { 0x7affa063, 23 },
    { 0xb32fa064, 98 },
    { 0xe31fa064, 52 },
    { 0x477fa066, 61 },
    { 0, 0 },
    { 0x204fa068, 84 },
    { 0x7b4fa069, 16 },
    { 0, 0 },
    { 0xbdbfa06b, 90 },
    { 0, 0 },
    { 0x52dfa06d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffa073, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a6fa079, 61 },
    { 0xfa5fa07a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e6fa07d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17cfa083, 33 },
    { 0xe9fa083, 33 },
    { 0x646fa085, 33 },
    { 0x825fa085, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ffa08d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fa094, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddfa097, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcfa09a, 95 },
    { 0xa5ffa09b, 100 },
    { 0x500fa09c, 61 },
    { 0xf5fa09b, 46 },
    { 0x7f8fa09e, 90 },
    { 0xc69fa09f, 76 },
    { 0x9fafa0a0, 69 },
    { 0xc43fa0a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09fa0a4, 90 },
    { 0xc4dfa0a5, 34 },
    { 0x141fa0a5, 61 },
    { 0, 0 },
    { 0x600fa0a8, 43 },
    { 0x1ccfa0a8, 33 },
    { 0xba9fa0aa, 68 },
    { 0x7adfa0a9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57fa0ae, 91 },
    { 0x7e2fa0af, 55 },
    { 0, 0 },
    { 0x5c4fa0b1, 90 },
    { 0, 0 },
    { 0xc85fa0b3, 52 },
    { 0xb8cfa0b4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x551fa0b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4defa0be, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dfa0c1, 68 },
    { 0xec7fa0c1, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30fa0c6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cfa0cb, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x210fa0d3, 98 },
    { 0x393fa0d3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b9fa0de, 90 },
    { 0, 0 },
    { 0xfa7fa0e0, 55 },
    { 0x773fa0e1, 76 },
    { 0, 0 },
    { 0xe4afa0e3, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7ffa0e6, 66 },
    { 0, 0 },
    { 0x31fa0e8, 55 },
    { 0, 0 },
    { 0x516fa0ea, 23 },
    { 0x3b3fa0ea, 61 },
    { 0, 0 },
    { 0xb4fa0ed, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dfa0f0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1fa0f6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbfa0f9, 99 },
    { 0x798fa0f9, 84 },
    { 0x5cfa0fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1fa0fe, 76 },
    { 0, 0 },
    { 0x53efa100, 90 },
    { 0, 0 },
    { 0x8fdfa102, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefa106, 61 },
    { 0, 0 },
    { 0xbcdfa108, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb5fa10d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdfa110, 70 },
    { 0x660fa111, 33 },
    { 0x6e5fa112, 95 },
    { 0x798fa113, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56fa119, 78 },
    { 0x94dfa11a, 90 },
    { 0xaa8fa11b, 75 },
    { 0xa04fa11c, 66 },
    { 0x4c0fa11d, 95 },
    { 0x585fa11c, 2 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda1fa124, 23 },
    { 0, 0 },
    { 0x787fa126, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6fa129, 95 },
    { 0x8bfa12a, 65 },
    { 0x539fa12b, 43 },
    { 0, 0 },
    { 0x9fdfa12d, 23 },
    { 0, 0 },
    { 0x3bffa12f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22efa133, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x76efa136, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91efa13b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6cfa145, 43 },
    { 0xfdffa145, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x980fa149, 90 },
    { 0, 0 },
    { 0x85dfa14b, 99 },
    { 0, 0 },
    { 0x734fa14d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ffa150, 76 },
    { 0xba2fa151, 95 },
    { 0x9d8fa152, 67 },
    { 0xca1fa151, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fa158, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8ffa15c, 40 },
    { 0x604fa15d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754fa163, 33 },
    { 0x80afa164, 91 },
    { 0xf77fa164, 77 },
    { 0, 0 },
    { 0x277fa167, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01fa16e, 61 },
    { 0xbf2fa16f, 85 },
    { 0, 0 },
    { 0x272fa171, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59fa174, 81 },
    { 0xac3fa175, 66 },
    { 0xd98fa176, 61 },
    { 0, 0 },
    { 0x4bcfa178, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4affa17b, 33 },
    { 0, 0 },
    { 0xb08fa17d, 90 },
    { 0x714fa17e, 61 },
    { 0, 0 },
    { 0xfc8fa180, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbfa188, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10dfa18f, 43 },
    { 0xa35fa190, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x367fa193, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x596fa198, 90 },
    { 0, 0 },
    { 0x893fa19a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2ffa19d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97fa1a9, 33 },
    { 0, 0 },
    { 0xeedfa1ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8fa1b1, 95 },
    { 0xe7efa1b2, 91 },
    { 0xe9bfa1b2, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xe76fa1b6, 99 },
    { 0xa30fa1b7, 93 },
    { 0, 0 },
    { 0x649fa1b9, 46 },
    { 0xfbdfa1ba, 35 },
    { 0x925fa1bb, 75 },
    { 0xa4efa1bc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdffa1c3, 78 },
    { 0, 0 },
    { 0xca9fa1c5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cfa1c9, 61 },
    { 0, 0 },
    { 0x654fa1cb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bafa1ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadfa1d4, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc9fa1d7, 90 },
    { 0x86bfa1d8, 99 },
    { 0x4bbfa1d8, 95 },
    { 0, 0 },
    { 0xcf4fa1db, 33 },
    { 0xff0fa1dc, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bfa1e5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x610fa1ed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bafa1f0, 33 },
    { 0, 0 },
    { 0x2c6fa1f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fa1f9, 93 },
    { 0xb44fa1fa, 46 },
    { 0, 0 },
    { 0xe52fa1fc, 61 },
    { 0, 0 },
    { 0x950fa1fe, 61 },
    { 0x4ccfa1ff, 23 },
    { 0, 0 },
    { 0xbdffa201, 61 },
    { 0, 0 },
    { 0xb9bfa203, 78 },
    { 0xf43fa203, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01fa20d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x707fa210, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb56fa213, 90 },
    { 0x230fa214, 90 },
    { 0x8f5fa215, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bffa218, 90 },
    { 0x142fa219, 84 },
    { 0xd77fa21a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0dfa21f, 33 },
    { 0, 0 },
    { 0x48cfa221, 61 },
    { 0x506fa222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x266fa225, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddfa22c, 23 },
    { 0x31bfa22c, 61 },
    { 0x8d0fa22e, 23 },
    { 0xfe1fa22c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fa234, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c6fa237, 33 },
    { 0x3a6fa238, 46 },
    { 0, 0 },
    { 0xff2fa23a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa92fa23d, 90 },
    { 0xae9fa23e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2fa246, 33 },
    { 0, 0 },
    { 0x8b1fa248, 76 },
    { 0x950fa249, 46 },
    { 0xf94fa24a, 90 },
    { 0, 0 },
    { 0xdb2fa24c, 61 },
    { 0x38fa24d, 95 },
    { 0x2ffa24d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc24fa252, 95 },
    { 0x815fa253, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82fa25b, 88 },
    { 0, 0 },
    { 0x7a0fa25d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x24efa260, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x237fa263, 75 },
    { 0x569fa264, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70efa26c, 61 },
    { 0x822fa26d, 61 },
    { 0xed4fa26e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cfa27a, 61 },
    { 0x939fa27a, 23 },
    { 0x438fa27c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fa283, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5afa28f, 56 },
    { 0x27afa290, 98 },
    { 0x5f2fa291, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6fa2a0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x71efa2a3, 98 },
    { 0xdb8fa2a4, 33 },
    { 0x714fa2a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d0fa2ac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5fa2b1, 90 },
    { 0x83ffa2b1, 90 },
    { 0x958fa2b1, 33 },
    { 0x304fa2b3, 98 },
    { 0xda2fa2b4, 61 },
    { 0, 0 },
    { 0xc1bfa2b7, 83 },
    { 0, 0 },
    { 0x859fa2b9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55fa2bc, 95 },
    { 0x7b3fa2bd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f9fa2c2, 76 },
    { 0, 0 },
    { 0x729fa2c4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xec8fa2c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9ffa2d6, 78 },
    { 0xb5efa2d6, 61 },
    { 0xd26fa2d8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x58afa2db, 33 },
    { 0xc45fa2dc, 95 },
    { 0x927fa2dc, 95 },
    { 0x26bfa2de, 60 },
    { 0xc55fa2df, 85 },
    { 0x73fa2e0, 33 },
    { 0xd6ffa2e1, 61 },
    { 0, 0 },
    { 0xaf0fa2e3, 23 },
    { 0xdfffa2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34fa2ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0fa2f0, 91 },
    { 0x3e6fa2f1, 61 },
    { 0x81fa2f2, 63 },
    { 0, 0 },
    { 0x288fa2f4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25fa2f8, 65 },
    { 0xa4cfa2f9, 23 },
    { 0x1c8fa2f9, 95 },
    { 0x955fa2fb, 33 },
    { 0xfa6fa2f8, 34 },
    { 0, 0 },
    { 0xd7cfa2fe, 90 },
    { 0xeb8fa2ff, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3fa304, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715fa309, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa99fa30c, 23 },
    { 0x5f9fa30d, 95 },
    { 0, 0 },
    { 0xd2efa30f, 61 },
    { 0xea3fa30f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dffa316, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x309fa31a, 95 },
    { 0x672fa31b, 90 },
    { 0, 0 },
    { 0xcd7fa31d, 76 },
    { 0x8befa31d, 93 },
    { 0x262fa31f, 23 },
    { 0, 0 },
    { 0x39cfa321, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9acfa326, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6fa32a, 95 },
    { 0, 0 },
    { 0x17efa32c, 67 },
    { 0x523fa32c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x742fa330, 75 },
    { 0x7c0fa331, 98 },
    { 0x2a2fa331, 98 },
    { 0x54afa332, 100 },
    { 0x6a5fa334, 61 },
    { 0x170fa335, 98 },
    { 0x788fa333, 66 },
    { 0xa3ffa336, 91 },
    { 0x8b7fa338, 87 },
    { 0xad2fa339, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x633fa33c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a7fa344, 90 },
    { 0x4b0fa345, 17 },
    { 0x752fa344, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd08fa34a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0fa34d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150fa351, 43 },
    { 0, 0 },
    { 0x41afa353, 98 },
    { 0, 0 },
    { 0xe8efa355, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x284fa35a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd89fa35d, 93 },
    { 0, 0 },
    { 0x6bffa35f, 90 },
    { 0xf87fa35f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03fa363, 78 },
    { 0x5e0fa364, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65fa368, 95 },
    { 0xccffa369, 68 },
    { 0x1e1fa369, 98 },
    { 0x8dcfa36b, 33 },
    { 0xd9ffa369, 95 },
    { 0x1aafa36a, 90 },
    { 0, 0 },
    { 0xc82fa36f, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3afa372, 93 },
    { 0xf65fa372, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8afa37a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3fa37e, 77 },
    { 0xf07fa37f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d8fa382, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadfa385, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84efa38b, 75 },
    { 0xc8bfa38c, 52 },
    { 0, 0 },
    { 0x7b7fa38e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41fa394, 90 },
    { 0x8b2fa394, 90 },
    { 0xc08fa396, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bfa39b, 34 },
    { 0x90cfa39b, 52 },
    { 0x50fa39b, 98 },
    { 0, 0 },
    { 0xc94fa39f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8fa3a2, 52 },
    { 0xefafa3a3, 90 },
    { 0x28efa3a4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cfa3a7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5ffa3af, 90 },
    { 0x996fa3b0, 95 },
    { 0, 0 },
    { 0x457fa3b2, 98 },
    { 0, 0 },
    { 0xc86fa3b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8defa3b7, 61 },
    { 0x696fa3b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215fa3bc, 95 },
    { 0, 0 },
    { 0xbb8fa3be, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d1fa3c2, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8fa3c7, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12afa3cb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf36fa3d2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61afa3d8, 34 },
    { 0xfccfa3d9, 95 },
    { 0, 0 },
    { 0xca2fa3db, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fa3de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfefa3e1, 23 },
    { 0xd97fa3e2, 33 },
    { 0, 0 },
    { 0xfc1fa3e4, 61 },
    { 0, 0 },
    { 0xa9dfa3e6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x48dfa3e9, 95 },
    { 0xcafa3ea, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fa3ed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fafa3f5, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bfa3f8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad0fa3fc, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73fa3ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8fa407, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cfa40c, 55 },
    { 0, 0 },
    { 0xba6fa40e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x406fa411, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dfa414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84fa419, 76 },
    { 0xc81fa41a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x994fa41f, 95 },
    { 0, 0 },
    { 0x34afa421, 33 },
    { 0, 0 },
    { 0x714fa423, 17 },
    { 0, 0 },
    { 0x80afa425, 34 },
    { 0x392fa425, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07fa429, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38fa42e, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fa431, 75 },
    { 0, 0 },
    { 0xcc6fa433, 90 },
    { 0x141fa433, 43 },
    { 0x1a6fa435, 33 },
    { 0, 0 },
    { 0xf6cfa437, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15fa43b, 93 },
    { 0x6a8fa43c, 55 },
    { 0xf44fa43d, 85 },
    { 0x9e2fa43d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8fa441, 34 },
    { 0x7d1fa441, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6ffa448, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ffa44b, 23 },
    { 0, 0 },
    { 0xb1afa44d, 91 },
    { 0x4d2fa44e, 95 },
    { 0xdb1fa44f, 33 },
    { 0xa79fa44d, 33 },
    { 0x8a3fa44e, 95 },
    { 0x20cfa452, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9dfa459, 77 },
    { 0x8d0fa459, 87 },
    { 0x9b0fa45b, 93 },
    { 0x17efa45b, 43 },
    { 0x553fa45c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x974fa461, 95 },
    { 0x403fa461, 83 },
    { 0, 0 },
    { 0x9f4fa464, 76 },
    { 0, 0 },
    { 0x408fa466, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8fa47a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1fa47f, 76 },
    { 0x841fa480, 99 },
    { 0xbe6fa481, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x219fa484, 90 },
    { 0x22bfa484, 61 },
    { 0, 0 },
    { 0xa08fa487, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fa48b, 95 },
    { 0, 0 },
    { 0xf6cfa48d, 46 },
    { 0xaf5fa48d, 33 },
    { 0x636fa48f, 90 },
    { 0xee4fa490, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84fa494, 78 },
    { 0x94efa494, 61 },
    { 0x4dbfa496, 95 },
    { 0xbefa497, 90 },
    { 0, 0 },
    { 0xe2cfa499, 78 },
    { 0xd01fa49a, 88 },
    { 0x903fa49b, 61 },
    { 0xb35fa49c, 93 },
    { 0, 0 },
    { 0x141fa49e, 90 },
    { 0, 0 },
    { 0x509fa4a0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x110fa4a6, 95 },
    { 0x518fa4a6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e8fa4b1, 85 },
    { 0x989fa4b2, 61 },
    { 0, 0 },
    { 0x113fa4b4, 95 },
    { 0, 0 },
    { 0x46cfa4b6, 90 },
    { 0xeeffa4b7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adfa4c1, 33 },
    { 0x777fa4c2, 61 },
    { 0x968fa4c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10bfa4ca, 76 },
    { 0xd25fa4cb, 93 },
    { 0, 0 },
    { 0x642fa4cd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356fa4d1, 95 },
    { 0x953fa4d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7efa4d5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cfa4d9, 46 },
    { 0, 0 },
    { 0xf19fa4db, 56 },
    { 0xc6afa4db, 90 },
    { 0xb02fa4dd, 36 },
    { 0, 0 },
    { 0x718fa4df, 61 },
    { 0, 0 },
    { 0x9adfa4e1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb2fa4e9, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7ffa4ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15dfa4f4, 55 },
    { 0x203fa4f4, 99 },
    { 0x182fa4f6, 90 },
    { 0x189fa4f4, 83 },
    { 0xe01fa4f8, 55 },
    { 0x375fa4f8, 95 },
    { 0xd1cfa4fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x655fa4fd, 90 },
    { 0x12efa4fd, 70 },
    { 0x751fa4ff, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdefa502, 95 },
    { 0, 0 },
    { 0x4dbfa504, 99 },
    { 0x4b4fa505, 68 },
    { 0x6acfa506, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x806fa50e, 98 },
    { 0x412fa50f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x815fa514, 87 },
    { 0x9f8fa514, 75 },
    { 0xb54fa514, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98fa51e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57fa523, 33 },
    { 0x1cffa523, 90 },
    { 0xdcefa525, 52 },
    { 0x118fa526, 33 },
    { 0xb82fa524, 46 },
    { 0x2b0fa528, 90 },
    { 0xaacfa529, 97 },
    { 0xcd3fa52a, 87 },
    { 0x701fa52b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x904fa535, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cfa539, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9fa53f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9fa546, 61 },
    { 0, 0 },
    { 0xac0fa548, 33 },
    { 0xdeefa548, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0fa54e, 24 },
    { 0xfa2fa54f, 98 },
    { 0, 0 },
    { 0xd27fa551, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dfa555, 93 },
    { 0, 0 },
    { 0xe1efa557, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa96fa55b, 52 },
    { 0xb60fa55b, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e9fa560, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x150fa568, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763fa56c, 61 },
    { 0, 0 },
    { 0x22ffa56e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x609fa573, 64 },
    { 0, 0 },
    { 0x3e4fa575, 78 },
    { 0, 0 },
    { 0xd1bfa577, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fcfa57a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3efa581, 91 },
    { 0xb35fa581, 38 },
    { 0x457fa582, 23 },
    { 0, 0 },
    { 0x738fa585, 33 },
    { 0x776fa585, 23 },
    { 0x84dfa587, 61 },
    { 0xffffa588, 90 },
    { 0xbd0fa588, 55 },
    { 0, 0 },
    { 0xd95fa58b, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54fa591, 61 },
    { 0, 0 },
    { 0xf37fa593, 90 },
    { 0, 0 },
    { 0xebdfa595, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b1fa59c, 94 },
    { 0, 0 },
    { 0x475fa59e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7ffa5a2, 76 },
    { 0x8a8fa5a2, 33 },
    { 0x5dbfa5a2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5afa5ad, 43 },
    { 0x153fa5ae, 90 },
    { 0xc71fa5af, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7fa5b3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89fa5b7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cfa5bc, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7fa5bf, 61 },
    { 0x265fa5bf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30fa5c4, 61 },
    { 0xdecfa5c4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x856fa5ce, 23 },
    { 0, 0 },
    { 0xb10fa5d0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f3fa5d3, 61 },
    { 0, 0 },
    { 0x6f7fa5d5, 65 },
    { 0x6dcfa5d5, 43 },
    { 0x765fa5d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44fa5de, 61 },
    { 0x674fa5df, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x636fa5e3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ecfa5ec, 86 },
    { 0, 0 },
    { 0xbbffa5ee, 98 },
    { 0x60ffa5ef, 55 },
    { 0x5c0fa5ee, 67 },
    { 0, 0 },
    { 0xefa5f2, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe14fa5fb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51fa5fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fa601, 33 },
    { 0, 0 },
    { 0x679fa603, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbcfa608, 33 },
    { 0xd66fa609, 80 },
    { 0x499fa608, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103fa60e, 98 },
    { 0, 0 },
    { 0xf32fa610, 23 },
    { 0, 0 },
    { 0x12bfa612, 36 },
    { 0x126fa612, 55 },
    { 0xf2cfa614, 33 },
    { 0, 0 },
    { 0x687fa616, 23 },
    { 0x9ddfa617, 66 },
    { 0x183fa618, 33 },
    { 0xb42fa619, 23 },
    { 0x1dafa617, 76 },
    { 0x748fa61a, 55 },
    { 0xdb2fa61c, 98 },
    { 0, 0 },
    { 0x796fa61e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2fa623, 100 },
    { 0xd7efa623, 95 },
    { 0x571fa625, 33 },
    { 0xc46fa626, 55 },
    { 0x910fa627, 91 },
    { 0xd4dfa628, 61 },
    { 0x9c9fa627, 95 },
    { 0x42fa626, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x271fa62f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x17dfa632, 23 },
    { 0, 0 },
    { 0x14bfa634, 90 },
    { 0, 0 },
    { 0x40afa636, 98 },
    { 0x5b5fa637, 100 },
    { 0xf2ffa638, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45fa63c, 90 },
    { 0, 0 },
    { 0x4fcfa63e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f1fa641, 61 },
    { 0, 0 },
    { 0xe7bfa643, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab9fa647, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84efa64e, 33 },
    { 0, 0 },
    { 0x751fa650, 33 },
    { 0, 0 },
    { 0x81cfa652, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11fa65d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3fa661, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2fa664, 84 },
    { 0, 0 },
    { 0x9e5fa666, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ffa669, 33 },
    { 0x48bfa66a, 97 },
    { 0xcbbfa66b, 17 },
    { 0x73fa66c, 87 },
    { 0x4a9fa66d, 23 },
    { 0x424fa66b, 55 },
    { 0x295fa66c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3fa676, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcfa67b, 84 },
    { 0xdafa67b, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cbfa681, 61 },
    { 0x5cefa681, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ffa685, 95 },
    { 0, 0 },
    { 0x41efa687, 34 },
    { 0xfdefa688, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b4fa68b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xad5fa68e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85fa692, 34 },
    { 0, 0 },
    { 0xeffa694, 95 },
    { 0, 0 },
    { 0x861fa696, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4fa69b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4affa69e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ebfa6a2, 34 },
    { 0xa7ffa6a3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fa6ad, 76 },
    { 0xf62fa6ae, 93 },
    { 0, 0 },
    { 0xba4fa6b0, 46 },
    { 0, 0 },
    { 0x8e2fa6b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xff6fa6b5, 55 },
    { 0x73cfa6b5, 61 },
    { 0xb6dfa6b7, 61 },
    { 0x748fa6b8, 66 },
    { 0xff6fa6b7, 93 },
    { 0xdafa6ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe82fa6be, 61 },
    { 0x6e7fa6bf, 100 },
    { 0x1abfa6bf, 90 },
    { 0x6bdfa6bf, 98 },
    { 0xc78fa6c0, 87 },
    { 0, 0 },
    { 0x63ffa6c4, 95 },
    { 0xb70fa6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fefa6c8, 17 },
    { 0, 0 },
    { 0x683fa6ca, 90 },
    { 0x5b1fa6cb, 61 },
    { 0x423fa6cc, 33 },
    { 0xec3fa6cb, 91 },
    { 0x836fa6ce, 76 },
    { 0, 0 },
    { 0x92fa6d0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67fa6d4, 90 },
    { 0x85bfa6d5, 23 },
    { 0, 0 },
    { 0xde2fa6d7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2fa6da, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3fa6de, 76 },
    { 0xc9efa6de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2efa6e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffa6ea, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366fa6f0, 95 },
    { 0x7e4fa6f1, 33 },
    { 0x89cfa6f2, 95 },
    { 0, 0 },
    { 0xb7fa6f4, 90 },
    { 0, 0 },
    { 0x8a7fa6f6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd10fa6fb, 90 },
    { 0xf5efa6fc, 33 },
    { 0xef4fa6fb, 61 },
    { 0xf1cfa6fd, 33 },
    { 0xc53fa6fc, 33 },
    { 0x531fa6fb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ffa703, 32 },
    { 0, 0 },
    { 0xdb5fa705, 95 },
    { 0x2c1fa705, 55 },
    { 0xf64fa707, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2fa70c, 51 },
    { 0x463fa70d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fa712, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9cfa715, 85 },
    { 0xee7fa716, 33 },
    { 0x17bfa716, 61 },
    { 0xdbdfa715, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74afa71e, 51 },
    { 0, 0 },
    { 0xcb9fa720, 95 },
    { 0x90fa720, 90 },
    { 0, 0 },
    { 0x1befa723, 33 },
    { 0x623fa723, 48 },
    { 0, 0 },
    { 0x1e3fa726, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcffa729, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x591fa72d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13cfa73e, 55 },
    { 0, 0 },
    { 0xd08fa740, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59fa744, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ebfa747, 61 },
    { 0xa26fa747, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26fa74b, 90 },
    { 0x3cffa74c, 61 },
    { 0, 0 },
    { 0x630fa74e, 33 },
    { 0x9d1fa74e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48afa754, 93 },
    { 0x4a2fa754, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf38fa75a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4afa75d, 76 },
    { 0, 0 },
    { 0x5c9fa75f, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1befa765, 90 },
    { 0, 0 },
    { 0x3abfa767, 98 },
    { 0x65fa768, 33 },
    { 0xa1ffa769, 98 },
    { 0, 0 },
    { 0xb63fa76b, 46 },
    { 0x2e8fa76c, 98 },
    { 0x933fa76d, 61 },
    { 0xf11fa76b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96dfa774, 95 },
    { 0x836fa775, 61 },
    { 0x870fa776, 55 },
    { 0xc2fa776, 90 },
    { 0x55fa778, 33 },
    { 0x83dfa779, 90 },
    { 0x3aafa779, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64bfa77f, 90 },
    { 0x5b9fa780, 95 },
    { 0x94fa780, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x659fa785, 61 },
    { 0x64efa785, 95 },
    { 0, 0 },
    { 0xa43fa788, 99 },
    { 0x5abfa789, 81 },
    { 0x5d4fa789, 61 },
    { 0x625fa78b, 33 },
    { 0, 0 },
    { 0xb11fa78d, 52 },
    { 0x7abfa78d, 34 },
    { 0xc6ffa78f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41fa793, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7dfa797, 90 },
    { 0xa27fa798, 61 },
    { 0xf59fa799, 81 },
    { 0xdb9fa79a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x435fa7a2, 33 },
    { 0xe7afa7a3, 93 },
    { 0x20dfa7a4, 34 },
    { 0xc72fa7a5, 95 },
    { 0, 0 },
    { 0xa7fa7a7, 95 },
    { 0x81ffa7a8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfa7ab, 61 },
    { 0xa02fa7ac, 90 },
    { 0xcd8fa7ac, 33 },
    { 0xc00fa7ae, 55 },
    { 0xdf9fa7ae, 34 },
    { 0x27fa7b0, 34 },
    { 0, 0 },
    { 0x9a3fa7b2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x27efa7b5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fa7b8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1effa7c1, 88 },
    { 0, 0 },
    { 0x319fa7c3, 67 },
    { 0, 0 },
    { 0x924fa7c5, 91 },
    { 0xb68fa7c6, 40 },
    { 0xaa0fa7c7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43bfa7cc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc0fa7d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39afa7d7, 36 },
    { 0x6fbfa7d8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x454fa7dd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb3fa7e0, 43 },
    { 0x5dfa7e1, 46 },
    { 0xe62fa7e1, 90 },
    { 0xb30fa7e3, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x448fa7e6, 52 },
    { 0x61afa7e6, 33 },
    { 0x77efa7e8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cfa7ec, 90 },
    { 0x46ffa7ed, 95 },
    { 0x598fa7ee, 65 },
    { 0x4f5fa7ee, 87 },
    { 0x152fa7f0, 23 },
    { 0xebffa7f0, 77 },
    { 0x481fa7f0, 98 },
    { 0x6e0fa7f0, 36 },
    { 0, 0 },
    { 0x250fa7f5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16fa7f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dfa7fc, 33 },
    { 0x966fa7fd, 61 },
    { 0xab3fa7fd, 90 },
    { 0x704fa7ff, 33 },
    { 0xa59fa7fe, 95 },
    { 0, 0 },
    { 0x4dbfa802, 33 },
    { 0x1aefa802, 66 },
    { 0xe9afa803, 23 },
    { 0xbc5fa805, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0fa80a, 95 },
    { 0, 0 },
    { 0x5afa80c, 23 },
    { 0x611fa80c, 91 },
    { 0, 0 },
    { 0x40fa80f, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93fa813, 95 },
    { 0x6afa813, 90 },
    { 0x457fa813, 33 },
    { 0xae8fa816, 99 },
    { 0x6fa817, 61 },
    { 0xd62fa818, 76 },
    { 0, 0 },
    { 0xd37fa81a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x677fa81d, 46 },
    { 0, 0 },
    { 0xda2fa81f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x338fa829, 90 },
    { 0xc6ffa829, 76 },
    { 0, 0 },
    { 0xcd1fa82c, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a7fa830, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe2fa834, 33 },
    { 0x93fa835, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687fa83b, 55 },
    { 0x888fa83c, 85 },
    { 0xdfefa83b, 67 },
    { 0xcc4fa83e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5fa841, 76 },
    { 0xf62fa842, 93 },
    { 0x3bcfa843, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35fa84b, 33 },
    { 0x686fa84c, 90 },
    { 0, 0 },
    { 0x48dfa84e, 23 },
    { 0, 0 },
    { 0xee0fa850, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe74fa853, 61 },
    { 0x9a0fa854, 100 },
    { 0x58cfa855, 98 },
    { 0xb3ffa856, 40 },
    { 0x316fa853, 92 },
    { 0, 0 },
    { 0xe94fa859, 100 },
    { 0xc36fa859, 61 },
    { 0xf46fa85a, 90 },
    { 0x92cfa859, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecffa863, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c1fa86d, 99 },
    { 0, 0 },
    { 0xb20fa86f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91bfa873, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30fa877, 95 },
    { 0, 0 },
    { 0x338fa879, 93 },
    { 0, 0 },
    { 0x87cfa87b, 33 },
    { 0x97bfa87c, 98 },
    { 0x858fa87d, 61 },
    { 0xcd7fa87e, 90 },
    { 0xeb0fa87f, 23 },
    { 0x8a9fa880, 95 },
    { 0x676fa880, 69 },
    { 0, 0 },
    { 0x545fa883, 90 },
    { 0xcd2fa884, 66 },
    { 0x5c8fa885, 95 },
    { 0, 0 },
    { 0x2e7fa887, 23 },
    { 0, 0 },
    { 0x4eefa889, 100 },
    { 0, 0 },
    { 0x66fa88b, 33 },
    { 0x6f2fa88b, 61 },
    { 0, 0 },
    { 0x11bfa88e, 87 },
    { 0, 0 },
    { 0x236fa890, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a1fa899, 46 },
    { 0x2a3fa89a, 49 },
    { 0x2bcfa899, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9fa89e, 95 },
    { 0xa2ffa89f, 23 },
    { 0x6f6fa8a0, 99 },
    { 0x32dfa8a1, 98 },
    { 0xe2efa8a2, 66 },
    { 0x411fa89e, 90 },
    { 0x4e9fa8a4, 80 },
    { 0xbf6fa89e, 76 },
    { 0, 0 },
    { 0xe7dfa8a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949fa8ac, 61 },
    { 0xea4fa8ac, 98 },
    { 0xc29fa8ae, 67 },
    { 0xeefa8ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835fa8b7, 90 },
    { 0xdacfa8b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4fa8bd, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbfa8c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9ffa8c6, 61 },
    { 0xd01fa8c7, 90 },
    { 0, 0 },
    { 0x32dfa8c9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c1fa8cc, 85 },
    { 0, 0 },
    { 0x6c7fa8ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30fa8d2, 23 },
    { 0x992fa8d2, 95 },
    { 0, 0 },
    { 0x539fa8d5, 91 },
    { 0, 0 },
    { 0x7ffa8d7, 64 },
    { 0xc75fa8d8, 61 },
    { 0x677fa8d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50fa8df, 33 },
    { 0, 0 },
    { 0xf1ffa8e1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41fa8e4, 85 },
    { 0, 0 },
    { 0xcc9fa8e6, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9cfa8ea, 98 },
    { 0xda5fa8ea, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x888fa8ee, 61 },
    { 0x6fffa8ee, 90 },
    { 0xb82fa8f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6fa8fb, 61 },
    { 0xd48fa8fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x760fa8ff, 90 },
    { 0x66efa900, 55 },
    { 0, 0 },
    { 0xe94fa902, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1fa90f, 95 },
    { 0, 0 },
    { 0x44cfa911, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61fa915, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fa919, 33 },
    { 0, 0 },
    { 0xf26fa91b, 93 },
    { 0x751fa91c, 33 },
    { 0xa40fa91d, 33 },
    { 0, 0 },
    { 0x2d9fa91f, 95 },
    { 0x399fa91f, 95 },
    { 0, 0 },
    { 0x7e5fa922, 87 },
    { 0, 0 },
    { 0x236fa924, 90 },
    { 0xb2fa924, 98 },
    { 0xdd9fa926, 98 },
    { 0x3ecfa927, 33 },
    { 0x528fa928, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7bfa931, 61 },
    { 0, 0 },
    { 0x978fa933, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fa937, 56 },
    { 0xb6efa937, 33 },
    { 0xcfa939, 23 },
    { 0xeabfa939, 90 },
    { 0x840fa93b, 84 },
    { 0xb84fa93c, 33 },
    { 0xec0fa93d, 23 },
    { 0x123fa93e, 90 },
    { 0x1eefa93b, 99 },
    { 0x57ffa940, 78 },
    { 0xe01fa93b, 90 },
    { 0xe10fa938, 33 },
    { 0x811fa943, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafffa947, 88 },
    { 0xcf6fa947, 98 },
    { 0xa33fa949, 88 },
    { 0x987fa94a, 55 },
    { 0, 0 },
    { 0xaedfa94c, 90 },
    { 0x7cffa94d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2ffa956, 90 },
    { 0xdcbfa957, 99 },
    { 0x338fa957, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adfa962, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bfa965, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x533fa96c, 91 },
    { 0x144fa96d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xa46fa970, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dafa974, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787fa979, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x287fa97d, 78 },
    { 0, 0 },
    { 0xf84fa97f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fa984, 46 },
    { 0x44bfa985, 56 },
    { 0xe86fa986, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946fa98b, 23 },
    { 0x73bfa98c, 90 },
    { 0x4b2fa98c, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf50fa990, 76 },
    { 0, 0 },
    { 0x33efa992, 55 },
    { 0, 0 },
    { 0x2e6fa994, 87 },
    { 0xd4ffa994, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5fa99c, 33 },
    { 0xb1bfa99d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfafa9aa, 81 },
    { 0, 0 },
    { 0x8f8fa9ac, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x888fa9af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ffa9b4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8fa9b9, 33 },
    { 0x3b2fa9ba, 61 },
    { 0xbcafa9bb, 61 },
    { 0x3ebfa9b9, 33 },
    { 0x416fa9bd, 33 },
    { 0xb47fa9b9, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x790fa9c2, 46 },
    { 0, 0 },
    { 0x7a0fa9c4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931fa9c8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f1fa9ce, 95 },
    { 0, 0 },
    { 0xc6cfa9d0, 33 },
    { 0x912fa9d1, 87 },
    { 0xc29fa9d2, 90 },
    { 0xac8fa9d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x13bfa9d6, 78 },
    { 0xb22fa9d7, 95 },
    { 0xb11fa9d8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0bfa9db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x804fa9e0, 55 },
    { 0x688fa9e0, 76 },
    { 0x442fa9e2, 76 },
    { 0x365fa9e0, 98 },
    { 0xcc1fa9e4, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd71fa9e8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63cfa9ed, 55 },
    { 0x9eefa9ee, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4befa9f2, 40 },
    { 0xc78fa9f2, 91 },
    { 0xcfa9f4, 63 },
    { 0x4c4fa9f5, 90 },
    { 0xe37fa9f4, 100 },
    { 0xfa9fa9f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x83afa9fa, 61 },
    { 0xdeefa9fb, 55 },
    { 0x18afa9fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x153fa9ff, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d0faa05, 88 },
    { 0x8bdfaa06, 62 },
    { 0x105faa05, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cfaa0b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ffaa13, 52 },
    { 0, 0 },
    { 0x117faa15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e2faa1a, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9faa1d, 66 },
    { 0x903faa1e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d2faa24, 88 },
    { 0, 0 },
    { 0x4c0faa26, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39efaa2e, 23 },
    { 0, 0 },
    { 0x68cfaa30, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x57bfaa33, 90 },
    { 0xafcfaa34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x168faa37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x805faa3a, 34 },
    { 0x2fefaa3a, 80 },
    { 0, 0 },
    { 0x8acfaa3d, 87 },
    { 0xc6afaa3e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5faa42, 99 },
    { 0x41afaa43, 9 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ffaa46, 90 },
    { 0x2fdfaa46, 91 },
    { 0xabafaa48, 43 },
    { 0x272faa49, 76 },
    { 0x584faa4a, 85 },
    { 0x809faa48, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb72faa50, 83 },
    { 0, 0 },
    { 0xdd9faa52, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4ffaa55, 100 },
    { 0x85ffaa55, 23 },
    { 0xb43faa56, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x850faa5a, 87 },
    { 0x970faa5b, 99 },
    { 0x3e1faa5c, 95 },
    { 0x7ecfaa5c, 33 },
    { 0x633faa5e, 100 },
    { 0x220faa5f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe12faa63, 76 },
    { 0xee7faa64, 98 },
    { 0x959faa64, 23 },
    { 0xbb7faa65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x573faa69, 43 },
    { 0, 0 },
    { 0x561faa6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf39faa70, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x614faa73, 90 },
    { 0xae0faa74, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85efaa79, 76 },
    { 0, 0 },
    { 0x52ffaa7b, 68 },
    { 0xaa0faa7c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x819faa83, 48 },
    { 0, 0 },
    { 0xef3faa85, 23 },
    { 0, 0 },
    { 0xe63faa87, 99 },
    { 0x37ffaa88, 61 },
    { 0xca7faa88, 76 },
    { 0x26bfaa89, 85 },
    { 0x5d3faa8b, 46 },
    { 0xb5ffaa8c, 46 },
    { 0xdf4faa8c, 33 },
    { 0x6b9faa88, 76 },
    { 0x863faa8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce0faa93, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f0faa96, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x384faa9a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efaa9d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549faaa1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cfaaa9, 61 },
    { 0x689faaaa, 55 },
    { 0x61faaab, 34 },
    { 0xb15faaa9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9afaab2, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe40faab9, 87 },
    { 0xb97faab9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe91faabe, 95 },
    { 0, 0 },
    { 0x263faac0, 98 },
    { 0x404faac1, 77 },
    { 0x831faac0, 61 },
    { 0, 0 },
    { 0x534faac4, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8afaaca, 90 },
    { 0xf0afaacb, 88 },
    { 0x6f1faacc, 77 },
    { 0xe53faacc, 34 },
    { 0x3abfaace, 56 },
    { 0xfeffaace, 100 },
    { 0x817faacb, 95 },
    { 0x82afaad1, 70 },
    { 0, 0 },
    { 0xdfdfaad3, 61 },
    { 0xdeafaad4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbfaad7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93afaadb, 17 },
    { 0x74bfaadc, 90 },
    { 0xc1cfaadd, 33 },
    { 0, 0 },
    { 0xf21faadf, 33 },
    { 0x1aafaadf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadafaae7, 23 },
    { 0x5efaae8, 78 },
    { 0, 0 },
    { 0x3b3faaea, 100 },
    { 0x19dfaaeb, 95 },
    { 0x91bfaaeb, 76 },
    { 0x3d9faaeb, 98 },
    { 0x3e8faaee, 90 },
    { 0x7bbfaaee, 99 },
    { 0x7c9faaef, 90 },
    { 0xb9dfaaf1, 61 },
    { 0x71bfaaf2, 77 },
    { 0x768faaf2, 90 },
    { 0x591faaf1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeffaaf8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fab00, 90 },
    { 0, 0 },
    { 0x1d0fab02, 90 },
    { 0, 0 },
    { 0xdaafab04, 33 },
    { 0x6b4fab05, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x474fab08, 33 },
    { 0x1e0fab09, 67 },
    { 0xa7bfab0a, 61 },
    { 0xfccfab09, 23 },
    { 0x2e7fab0c, 98 },
    { 0xf8dfab0c, 84 },
    { 0x2e6fab08, 98 },
    { 0xbd8fab0f, 33 },
    { 0xfa1fab10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc3fab13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b2fab18, 34 },
    { 0x48afab19, 34 },
    { 0xa77fab1a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fab1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b3fab24, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3ffab2a, 100 },
    { 0x8d3fab2b, 78 },
    { 0xca4fab2a, 61 },
    { 0x7d1fab2d, 76 },
    { 0x7ecfab2e, 100 },
    { 0x5c7fab2c, 33 },
    { 0, 0 },
    { 0x7b7fab31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fab37, 46 },
    { 0, 0 },
    { 0xc00fab39, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36fab3c, 34 },
    { 0xa16fab3c, 99 },
    { 0xeb7fab3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbfab42, 95 },
    { 0xd05fab43, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62fab48, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5fab4b, 76 },
    { 0x102fab4c, 33 },
    { 0x795fab4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97fab52, 83 },
    { 0xcb4fab53, 33 },
    { 0x1eafab54, 34 },
    { 0, 0 },
    { 0x326fab56, 78 },
    { 0x9e8fab57, 52 },
    { 0, 0 },
    { 0x970fab59, 81 },
    { 0, 0 },
    { 0xf24fab5b, 36 },
    { 0xaaffab5c, 33 },
    { 0xfe8fab5d, 23 },
    { 0x535fab5b, 46 },
    { 0, 0 },
    { 0xc8ffab60, 34 },
    { 0x5aafab61, 81 },
    { 0, 0 },
    { 0x190fab63, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3defab68, 33 },
    { 0, 0 },
    { 0x1a7fab6a, 33 },
    { 0x2a6fab6b, 100 },
    { 0, 0 },
    { 0x156fab6d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fab72, 23 },
    { 0x2a2fab73, 85 },
    { 0xe48fab73, 90 },
    { 0x549fab75, 46 },
    { 0, 0 },
    { 0xe48fab77, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4fab7f, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bdfab83, 61 },
    { 0x46dfab84, 23 },
    { 0x235fab85, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x408fab8c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62fab91, 82 },
    { 0, 0 },
    { 0xef1fab93, 46 },
    { 0, 0 },
    { 0x834fab95, 98 },
    { 0x50dfab96, 34 },
    { 0x9b9fab97, 55 },
    { 0x87bfab98, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb4fab9e, 76 },
    { 0xd46fab9f, 95 },
    { 0x35afaba0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfaba7, 61 },
    { 0x6e7faba8, 83 },
    { 0x503faba8, 61 },
    { 0xfecfaba8, 99 },
    { 0x2e6faba7, 90 },
    { 0xd6ffabac, 61 },
    { 0x1bffaba9, 90 },
    { 0x3f7fabae, 90 },
    { 0x349fabaf, 48 },
    { 0, 0 },
    { 0xa94fabb1, 78 },
    { 0xbb6fabb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23fabb6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x804fabb9, 95 },
    { 0, 0 },
    { 0x11efabbb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14bfabc3, 61 },
    { 0xeb3fabc4, 61 },
    { 0xd77fabc5, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2dfabca, 33 },
    { 0x79ffabca, 95 },
    { 0x734fabcc, 76 },
    { 0xfbafabcd, 66 },
    { 0x455fabce, 100 },
    { 0, 0 },
    { 0x30bfabd0, 33 },
    { 0x317fabd1, 55 },
    { 0x625fabd1, 67 },
    { 0, 0 },
    { 0xa3afabd4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b0fabd8, 61 },
    { 0, 0 },
    { 0x8dffabda, 76 },
    { 0, 0 },
    { 0xd03fabdc, 84 },
    { 0x9dcfabdc, 61 },
    { 0xcdafabde, 33 },
    { 0xd33fabdf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fdfabe2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fabe7, 76 },
    { 0xb12fabe8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x340fabeb, 33 },
    { 0x9c5fabeb, 90 },
    { 0, 0 },
    { 0x819fabee, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x525fabf1, 61 },
    { 0x585fabf1, 90 },
    { 0x4effabf2, 67 },
    { 0, 0 },
    { 0x73dfabf5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fabfa, 90 },
    { 0x9d8fabfb, 61 },
    { 0x88ffabfc, 55 },
    { 0x2a6fabfa, 95 },
    { 0xba4fabfe, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8fac01, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3fac07, 93 },
    { 0x8f4fac08, 95 },
    { 0xf03fac08, 46 },
    { 0xaacfac07, 95 },
    { 0x5a6fac08, 61 },
    { 0, 0 },
    { 0xfc2fac0d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90fac10, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf76fac15, 23 },
    { 0xa98fac15, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cfac1a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93bfac1f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3fac25, 90 },
    { 0x723fac25, 33 },
    { 0x2b5fac25, 90 },
    { 0, 0 },
    { 0x780fac29, 90 },
    { 0x327fac29, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x350fac2d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1afac31, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x270fac36, 90 },
    { 0x114fac37, 23 },
    { 0x3d9fac38, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23fac3b, 85 },
    { 0xabffac3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa9fac47, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa88fac52, 66 },
    { 0, 0 },
    { 0x64dfac54, 66 },
    { 0x5b7fac55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1fac5a, 98 },
    { 0x8e3fac5b, 61 },
    { 0xc25fac5c, 91 },
    { 0x31dfac5d, 61 },
    { 0x877fac5d, 36 },
    { 0, 0 },
    { 0xfbffac60, 34 },
    { 0xbdafac60, 98 },
    { 0xc4bfac61, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a0fac66, 23 },
    { 0x234fac66, 33 },
    { 0x1a8fac67, 55 },
    { 0xbbfac69, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde4fac6d, 95 },
    { 0x1b7fac6e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x580fac73, 95 },
    { 0, 0 },
    { 0x1b4fac75, 61 },
    { 0x9cafac76, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x332fac79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdfac7c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb0fac82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13fac87, 45 },
    { 0xc28fac88, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xea1fac8b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc2fac8e, 46 },
    { 0xc39fac8e, 23 },
    { 0x902fac90, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x248fac94, 91 },
    { 0, 0 },
    { 0xca3fac96, 61 },
    { 0xc4bfac97, 23 },
    { 0x73ffac98, 76 },
    { 0xe8efac99, 67 },
    { 0x413fac9a, 33 },
    { 0x1a0fac9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb8faca2, 67 },
    { 0, 0 },
    { 0xc89faca4, 34 },
    { 0x47bfaca4, 55 },
    { 0x7dafaca6, 55 },
    { 0x8d4faca7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26facab, 90 },
    { 0x68efacac, 61 },
    { 0xaccfacab, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0facb0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x569facb7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b2facbc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccfacbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54facc5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x736facc9, 85 },
    { 0xca1facca, 61 },
    { 0xff4faccb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ffacd0, 90 },
    { 0xc9facd1, 93 },
    { 0, 0 },
    { 0xa39facd3, 43 },
    { 0x275facd3, 55 },
    { 0x2adfacd5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1facdd, 23 },
    { 0, 0 },
    { 0xa88facdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122face6, 93 },
    { 0x6a0face7, 61 },
    { 0, 0 },
    { 0xce2face9, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb36facf8, 95 },
    { 0, 0 },
    { 0xc99facfa, 98 },
    { 0x7dffacfb, 61 },
    { 0x8effacfc, 46 },
    { 0xcd3facfd, 90 },
    { 0, 0 },
    { 0xbbffacff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238fad03, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a0fad0b, 23 },
    { 0xda3fad0c, 46 },
    { 0x994fad0b, 61 },
    { 0xfa9fad0b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb71fad12, 90 },
    { 0x15afad13, 61 },
    { 0xf12fad14, 33 },
    { 0xeafad13, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dffad18, 61 },
    { 0, 0 },
    { 0x881fad1a, 61 },
    { 0x7eefad1b, 90 },
    { 0x4cafad1a, 61 },
    { 0x4fffad1d, 99 },
    { 0x44efad1c, 95 },
    { 0xbdfad1c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea6fad28, 66 },
    { 0x418fad28, 98 },
    { 0x5f3fad29, 55 },
    { 0x65bfad28, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa5fad2e, 82 },
    { 0, 0 },
    { 0x13cfad30, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b6fad36, 55 },
    { 0, 0 },
    { 0xa33fad38, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x462fad3c, 33 },
    { 0x7dbfad3c, 95 },
    { 0, 0 },
    { 0xedafad3f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x124fad45, 88 },
    { 0x411fad46, 38 },
    { 0x9edfad47, 43 },
    { 0xc6cfad47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec2fad4f, 84 },
    { 0xcbafad50, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf6fad53, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6cffad57, 76 },
    { 0x589fad57, 90 },
    { 0, 0 },
    { 0x9a7fad5a, 90 },
    { 0x466fad5b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x638fad65, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cfad71, 90 },
    { 0x567fad71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b2fad76, 98 },
    { 0, 0 },
    { 0x5b6fad78, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fad7f, 90 },
    { 0x406fad80, 40 },
    { 0xbc4fad80, 77 },
    { 0xb93fad80, 55 },
    { 0x3d8fad80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c9fad87, 95 },
    { 0x8bffad88, 95 },
    { 0xd1ffad89, 76 },
    { 0, 0 },
    { 0x7dbfad8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fad90, 90 },
    { 0x92ffad91, 78 },
    { 0xaa2fad92, 61 },
    { 0xf03fad93, 61 },
    { 0, 0 },
    { 0x843fad95, 66 },
    { 0x384fad96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb08fad9b, 64 },
    { 0, 0 },
    { 0x86fad9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb13fada1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x535fada4, 61 },
    { 0x5d8fada5, 76 },
    { 0, 0 },
    { 0x137fada7, 55 },
    { 0, 0 },
    { 0x37fada9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad8fadad, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeffadb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd3fadb3, 100 },
    { 0, 0 },
    { 0xdd0fadb5, 61 },
    { 0x24efadb6, 98 },
    { 0xdafadb6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bfadba, 33 },
    { 0xa2fadba, 61 },
    { 0x14bfadbc, 68 },
    { 0x629fadbc, 55 },
    { 0x4b7fadba, 34 },
    { 0xb18fadbe, 61 },
    { 0xbf1fadc0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fadc7, 76 },
    { 0x14cfadc8, 85 },
    { 0, 0 },
    { 0x8dfadca, 77 },
    { 0x8fefadca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66fadd3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa53fadd7, 90 },
    { 0x2edfadd8, 93 },
    { 0, 0 },
    { 0x918fadda, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x54faddd, 98 },
    { 0x2d6fadde, 33 },
    { 0, 0 },
    { 0x32fade0, 99 },
    { 0x13bfade0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe05fade4, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564fadea, 90 },
    { 0x5cafadea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x501fadee, 56 },
    { 0, 0 },
    { 0x995fadf0, 90 },
    { 0x9ffadf0, 96 },
    { 0x43cfadf2, 95 },
    { 0xcbefadf1, 98 },
    { 0, 0 },
    { 0xb5bfadf5, 61 },
    { 0, 0 },
    { 0xb54fadf7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf29fadfc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd6fadff, 33 },
    { 0, 0 },
    { 0x3d9fae01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe21fae05, 38 },
    { 0x588fae06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3cfae10, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x964fae16, 61 },
    { 0x4b9fae16, 90 },
    { 0xee8fae18, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb73fae1f, 65 },
    { 0, 0 },
    { 0x896fae21, 33 },
    { 0, 0 },
    { 0xc0dfae23, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd0fae26, 85 },
    { 0, 0 },
    { 0x40dfae28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbfae2b, 95 },
    { 0, 0 },
    { 0x5dcfae2d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922fae32, 23 },
    { 0, 0 },
    { 0x1f7fae34, 91 },
    { 0x432fae35, 55 },
    { 0x267fae36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2fae39, 43 },
    { 0xa94fae3a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd35fae45, 90 },
    { 0x8ffae46, 76 },
    { 0xaa9fae47, 98 },
    { 0x49ffae47, 93 },
    { 0, 0 },
    { 0x3e7fae4a, 78 },
    { 0, 0 },
    { 0xff2fae4c, 98 },
    { 0, 0 },
    { 0x79bfae4e, 85 },
    { 0, 0 },
    { 0xa6bfae50, 90 },
    { 0, 0 },
    { 0xa23fae52, 90 },
    { 0x747fae53, 23 },
    { 0x53dfae54, 99 },
    { 0x4fffae53, 99 },
    { 0x1f5fae56, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88efae60, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x28efae63, 100 },
    { 0, 0 },
    { 0x302fae65, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffae69, 98 },
    { 0, 0 },
    { 0x736fae6b, 90 },
    { 0x8d3fae6c, 83 },
    { 0x632fae6d, 95 },
    { 0, 0 },
    { 0x2cfae6f, 46 },
    { 0x550fae70, 33 },
    { 0x7f5fae6f, 61 },
    { 0xdbefae6f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11efae79, 34 },
    { 0x8dcfae7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cfae80, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x769fae88, 90 },
    { 0x67bfae89, 23 },
    { 0, 0 },
    { 0x7c3fae8b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791fae8f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b7fae95, 34 },
    { 0x29bfae96, 46 },
    { 0xcedfae96, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x606fae9a, 23 },
    { 0xeeefae9b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b8fae9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e7faea3, 90 },
    { 0, 0 },
    { 0x20faea5, 95 },
    { 0x8a5faea6, 23 },
    { 0, 0 },
    { 0x114faea8, 98 },
    { 0x657faea9, 95 },
    { 0x5a8faeaa, 61 },
    { 0, 0 },
    { 0x240faeac, 99 },
    { 0, 0 },
    { 0x41dfaeae, 76 },
    { 0xf9bfaeaf, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x916faeb2, 84 },
    { 0x193faeb3, 76 },
    { 0xebdfaeb3, 46 },
    { 0x18afaeb2, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x743faeb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ffaebe, 43 },
    { 0, 0 },
    { 0xef5faec0, 33 },
    { 0x802faec0, 90 },
    { 0xc70faec1, 90 },
    { 0xec4faec1, 61 },
    { 0x4b7faec1, 43 },
    { 0xe7ffaec5, 76 },
    { 0x49ffaec5, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbafaecb, 61 },
    { 0x318faecc, 90 },
    { 0x525faecb, 43 },
    { 0, 0 },
    { 0x2bffaecf, 42 },
    { 0x6bbfaecf, 61 },
    { 0xfc6faed1, 61 },
    { 0x817faed1, 61 },
    { 0x9d7faed3, 78 },
    { 0xf91faed4, 98 },
    { 0x850faed3, 46 },
    { 0xf6dfaed4, 95 },
    { 0x27cfaed7, 33 },
    { 0xf5dfaed7, 23 },
    { 0x4f1faed9, 61 },
    { 0x231faed8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b5faedd, 85 },
    { 0, 0 },
    { 0x982faedf, 90 },
    { 0, 0 },
    { 0x71dfaee1, 23 },
    { 0, 0 },
    { 0xe92faee3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71faee9, 46 },
    { 0x7cfaeea, 66 },
    { 0, 0 },
    { 0xfc5faeec, 23 },
    { 0x4a7faeed, 23 },
    { 0, 0 },
    { 0x4e5faeef, 66 },
    { 0, 0 },
    { 0xd7faef1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4afaef5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89dfaefb, 23 },
    { 0x1a7faefc, 43 },
    { 0x752faefd, 90 },
    { 0, 0 },
    { 0x23cfaeff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bafaf03, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b9faf08, 34 },
    { 0x750faf09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dffaf0c, 33 },
    { 0x149faf0c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x332faf10, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43efaf1e, 33 },
    { 0x4e4faf1f, 88 },
    { 0x5ccfaf20, 90 },
    { 0x506faf20, 76 },
    { 0xe6faf22, 90 },
    { 0x5e3faf20, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x574faf27, 55 },
    { 0x5e4faf28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0faf2b, 33 },
    { 0, 0 },
    { 0x25bfaf2d, 90 },
    { 0, 0 },
    { 0x4f8faf2f, 84 },
    { 0, 0 },
    { 0xbfdfaf31, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x903faf34, 98 },
    { 0x790faf35, 23 },
    { 0x9f2faf35, 95 },
    { 0x2defaf37, 77 },
    { 0x8cffaf37, 46 },
    { 0xfb7faf39, 61 },
    { 0x2b9faf34, 87 },
    { 0, 0 },
    { 0xa87faf3c, 78 },
    { 0, 0 },
    { 0xa35faf3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48afaf42, 52 },
    { 0, 0 },
    { 0xd39faf44, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99efaf4a, 34 },
    { 0xc05faf4a, 67 },
    { 0xd3dfaf4a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x755faf50, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298faf56, 72 },
    { 0, 0 },
    { 0xccfaf58, 55 },
    { 0x940faf59, 85 },
    { 0xe8dfaf5a, 90 },
    { 0xd3bfaf59, 46 },
    { 0x42bfaf5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x592faf69, 90 },
    { 0, 0 },
    { 0xbfbfaf6b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb59faf73, 43 },
    { 0xa93faf73, 23 },
    { 0x98afaf73, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2dfaf7d, 43 },
    { 0x20efaf7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3dfaf83, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a4faf90, 34 },
    { 0xfffaf91, 66 },
    { 0x8c6faf91, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1afaf96, 52 },
    { 0xb3faf97, 75 },
    { 0xd2efaf98, 90 },
    { 0xe9cfaf99, 100 },
    { 0x7b3faf9a, 78 },
    { 0x969faf99, 85 },
    { 0x39ffaf96, 90 },
    { 0xf67faf9b, 98 },
    { 0, 0 },
    { 0x7ebfaf9f, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x53fafa2, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43cfafa7, 76 },
    { 0xe9dfafa7, 90 },
    { 0xc43fafa8, 46 },
    { 0, 0 },
    { 0x7fbfafab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3afafae, 95 },
    { 0x275fafaf, 98 },
    { 0x24ffafaf, 90 },
    { 0x158fafb1, 61 },
    { 0, 0 },
    { 0x3abfafb3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69fafb7, 76 },
    { 0x25cfafb7, 90 },
    { 0xcd9fafb9, 90 },
    { 0x263fafba, 55 },
    { 0x953fafb8, 87 },
    { 0xa09fafba, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x412fafc6, 100 },
    { 0x86cfafc6, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x101fafca, 48 },
    { 0, 0 },
    { 0x9a3fafcc, 78 },
    { 0x9cfafcd, 95 },
    { 0x51ffafce, 76 },
    { 0x934fafce, 33 },
    { 0x4c6fafd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffafd3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ddfafd6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x456fafd9, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dfafdc, 43 },
    { 0x952fafdd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffafe3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xa30fafe6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd4fafea, 46 },
    { 0x536fafeb, 97 },
    { 0, 0 },
    { 0x92cfafed, 61 },
    { 0xecbfafed, 65 },
    { 0x855fafee, 90 },
    { 0x167fafee, 64 },
    { 0, 0 },
    { 0x522faff2, 34 },
    { 0x4defaff3, 76 },
    { 0xac1faff4, 90 },
    { 0x6c6faff5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd59faffe, 95 },
    { 0xb23fafff, 38 },
    { 0, 0 },
    { 0x247fb001, 76 },
    { 0x884fb002, 55 },
    { 0x6a2fb003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549fb008, 61 },
    { 0xbf7fb008, 98 },
    { 0x906fb00a, 55 },
    { 0x73bfb00b, 99 },
    { 0x72fb00b, 101 },
    { 0xfefb00d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x404fb010, 85 },
    { 0, 0 },
    { 0x4d8fb012, 46 },
    { 0x9fb013, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb3fb016, 23 },
    { 0x44dfb017, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bafb01a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xca8fb01d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fb020, 90 },
    { 0x5cfb021, 61 },
    { 0xdc5fb022, 100 },
    { 0x3acfb022, 33 },
    { 0xa35fb023, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x10efb027, 67 },
    { 0xe33fb028, 90 },
    { 0x2fdfb029, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6cfb02f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x444fb03d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeefb041, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x364fb044, 87 },
    { 0, 0 },
    { 0x619fb046, 98 },
    { 0xf8ffb047, 55 },
    { 0, 0 },
    { 0xddbfb049, 87 },
    { 0xe7afb049, 68 },
    { 0, 0 },
    { 0x3a6fb04c, 84 },
    { 0xe17fb04c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdbfb050, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x25efb053, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914fb058, 66 },
    { 0xaa2fb058, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4cfb05d, 90 },
    { 0, 0 },
    { 0xb23fb05f, 78 },
    { 0x34fb060, 17 },
    { 0x25afb061, 55 },
    { 0xca6fb062, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ffb066, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4fb069, 90 },
    { 0x54fb069, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dfb06e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42ffb073, 43 },
    { 0, 0 },
    { 0x976fb075, 56 },
    { 0, 0 },
    { 0x3b7fb077, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711fb07b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fb07e, 23 },
    { 0, 0 },
    { 0xcc9fb080, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x615fb083, 33 },
    { 0x4efb083, 90 },
    { 0x5a8fb085, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45fb08a, 77 },
    { 0xa4dfb08a, 61 },
    { 0x310fb08a, 33 },
    { 0x930fb08d, 52 },
    { 0x1fb08e, 61 },
    { 0, 0 },
    { 0x6e5fb090, 43 },
    { 0, 0 },
    { 0xdd6fb092, 55 },
    { 0x606fb093, 90 },
    { 0xdfbfb092, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd0dfb09c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x967fb09f, 61 },
    { 0x417fb09f, 33 },
    { 0xffcfb0a1, 90 },
    { 0x61fb0a2, 53 },
    { 0, 0 },
    { 0x7b0fb0a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438fb0a8, 93 },
    { 0, 0 },
    { 0xbcdfb0aa, 53 },
    { 0xa50fb0aa, 55 },
    { 0xda5fb0ab, 90 },
    { 0, 0 },
    { 0xb1bfb0ae, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x633fb0b6, 33 },
    { 0x7f3fb0b7, 95 },
    { 0x20ffb0b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9defb0be, 34 },
    { 0, 0 },
    { 0xfe0fb0c0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbacfb0c6, 98 },
    { 0x545fb0c7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4fb0cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba0fb0d6, 46 },
    { 0, 0 },
    { 0x2b7fb0d8, 43 },
    { 0x33dfb0d9, 46 },
    { 0x5fafb0da, 23 },
    { 0x522fb0db, 76 },
    { 0, 0 },
    { 0x2e0fb0dd, 76 },
    { 0xfc6fb0de, 76 },
    { 0, 0 },
    { 0x55ffb0e0, 90 },
    { 0x398fb0e1, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9bfb0e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dffb0e7, 33 },
    { 0x17efb0e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0fb0eb, 23 },
    { 0, 0 },
    { 0xe5afb0ed, 23 },
    { 0x1e4fb0ed, 19 },
    { 0, 0 },
    { 0xacbfb0f0, 76 },
    { 0xaaffb0f0, 46 },
    { 0, 0 },
    { 0xf75fb0f3, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa97fb0f6, 19 },
    { 0, 0 },
    { 0xf21fb0f8, 90 },
    { 0xccbfb0f8, 78 },
    { 0x935fb0f9, 75 },
    { 0xea8fb0fb, 98 },
    { 0x6dffb0fc, 33 },
    { 0xacbfb0fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb60fb100, 99 },
    { 0, 0 },
    { 0x149fb102, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ffb105, 98 },
    { 0x314fb106, 34 },
    { 0, 0 },
    { 0x540fb108, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80bfb10c, 55 },
    { 0xbb0fb10d, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc78fb111, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553fb115, 23 },
    { 0xee5fb116, 23 },
    { 0, 0 },
    { 0x948fb118, 95 },
    { 0, 0 },
    { 0x1e8fb11a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacdfb123, 98 },
    { 0, 0 },
    { 0x1f1fb125, 76 },
    { 0x82fb126, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28fb129, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8ffb12d, 43 },
    { 0x3f3fb12d, 85 },
    { 0x913fb12d, 61 },
    { 0x7fafb130, 76 },
    { 0xa9ffb131, 98 },
    { 0x43efb12e, 66 },
    { 0, 0 },
    { 0xe79fb134, 90 },
    { 0x978fb134, 23 },
    { 0, 0 },
    { 0x273fb137, 23 },
    { 0x4bbfb138, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fb142, 95 },
    { 0, 0 },
    { 0x16cfb144, 90 },
    { 0xca6fb145, 34 },
    { 0, 0 },
    { 0x26fb147, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7afb152, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438fb157, 63 },
    { 0, 0 },
    { 0xe24fb159, 43 },
    { 0x716fb15a, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe63fb15d, 100 },
    { 0, 0 },
    { 0x61fb15f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc71fb164, 100 },
    { 0xa2cfb164, 43 },
    { 0x7f4fb166, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6fb16b, 23 },
    { 0x256fb16c, 98 },
    { 0x6adfb16b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb16fb172, 90 },
    { 0x375fb173, 61 },
    { 0x391fb172, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x357fb178, 66 },
    { 0x7a5fb178, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x106fb17c, 33 },
    { 0, 0 },
    { 0x71fb17e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb11fb182, 98 },
    { 0xf0fb182, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa85fb188, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c6fb18c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x450fb194, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x844fb197, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x852fb19b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x495fb1a3, 91 },
    { 0x7dbfb1a3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6fb1a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeffb1ab, 87 },
    { 0xf4bfb1ac, 66 },
    { 0, 0 },
    { 0x3e0fb1ae, 80 },
    { 0x5c7fb1ae, 34 },
    { 0x29bfb1b0, 88 },
    { 0, 0 },
    { 0x350fb1b2, 61 },
    { 0xa9afb1b3, 33 },
    { 0x231fb1b3, 76 },
    { 0x185fb1b5, 98 },
    { 0xb28fb1b3, 61 },
    { 0x544fb1b7, 95 },
    { 0x443fb1b2, 93 },
    { 0x42efb1b9, 33 },
    { 0x713fb1b9, 66 },
    { 0x154fb1bb, 95 },
    { 0x462fb1bc, 43 },
    { 0xa7afb1b9, 53 },
    { 0x8c4fb1bd, 33 },
    { 0x4b9fb1bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x168fb1c5, 77 },
    { 0, 0 },
    { 0xa86fb1c7, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cfb1ca, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fb1cf, 9 },
    { 0x8fafb1d0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a4fb1d5, 34 },
    { 0xd0dfb1d6, 33 },
    { 0x198fb1d7, 61 },
    { 0xcd6fb1d8, 100 },
    { 0x2b5fb1d5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dfb1dc, 61 },
    { 0xeaefb1dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x907fb1e0, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x382fb1e5, 77 },
    { 0xc9afb1e5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ffb1e9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa84fb1ed, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x32fb1f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbffb1f5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7effb1f8, 34 },
    { 0x509fb1f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dfb1fc, 90 },
    { 0, 0 },
    { 0x5abfb1fe, 33 },
    { 0, 0 },
    { 0x18ffb200, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x613fb204, 46 },
    { 0x4ffb205, 61 },
    { 0, 0 },
    { 0xc9dfb207, 65 },
    { 0xe5efb208, 87 },
    { 0, 0 },
    { 0x42cfb20a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xc52fb20d, 98 },
    { 0x793fb20e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xc09fb211, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efb214, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x139fb217, 23 },
    { 0x1afb218, 33 },
    { 0, 0 },
    { 0x8dafb21a, 66 },
    { 0xab3fb21b, 98 },
    { 0xf5efb21c, 61 },
    { 0x8c2fb21d, 90 },
    { 0x9f5fb21d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dafb223, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x16cfb226, 23 },
    { 0x8b7fb227, 61 },
    { 0x9f9fb228, 90 },
    { 0x36dfb226, 98 },
    { 0x9f1fb22a, 90 },
    { 0xf63fb22a, 43 },
    { 0x809fb22c, 43 },
    { 0xa1fb22c, 98 },
    { 0x443fb22c, 91 },
    { 0xb18fb22f, 87 },
    { 0x5bafb229, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x557fb236, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fb239, 95 },
    { 0x90ffb239, 95 },
    { 0, 0 },
    { 0x3befb23c, 66 },
    { 0x7d7fb23d, 78 },
    { 0, 0 },
    { 0x8d2fb23f, 33 },
    { 0xd1dfb23f, 76 },
    { 0x499fb241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc3fb244, 95 },
    { 0x6dafb245, 43 },
    { 0xd41fb246, 64 },
    { 0xbbdfb246, 93 },
    { 0xc17fb248, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0dfb24b, 95 },
    { 0xe61fb24c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd94fb253, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb55fb257, 93 },
    { 0x747fb257, 61 },
    { 0x546fb257, 95 },
    { 0xbe0fb25a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffb25d, 33 },
    { 0xf4fb25e, 87 },
    { 0xadafb25f, 90 },
    { 0xfadfb260, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8fb264, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf52fb26d, 23 },
    { 0x467fb26d, 23 },
    { 0x883fb26f, 90 },
    { 0x656fb270, 68 },
    { 0x787fb270, 98 },
    { 0xb45fb272, 23 },
    { 0x9b0fb273, 23 },
    { 0x563fb271, 98 },
    { 0, 0 },
    { 0xa92fb276, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae6fb279, 84 },
    { 0, 0 },
    { 0x28cfb27b, 33 },
    { 0xcaafb27b, 90 },
    { 0x851fb27b, 23 },
    { 0x349fb27b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5fb284, 98 },
    { 0, 0 },
    { 0xb5afb286, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddfb289, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2fb28c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cfb290, 69 },
    { 0xfdcfb291, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810fb297, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f8fb29a, 65 },
    { 0xdccfb29a, 23 },
    { 0x508fb29c, 98 },
    { 0x181fb29a, 90 },
    { 0x1c2fb29d, 98 },
    { 0x652fb29f, 23 },
    { 0xa72fb29b, 33 },
    { 0x661fb2a1, 77 },
    { 0xb22fb2a2, 95 },
    { 0, 0 },
    { 0xc69fb2a4, 81 },
    { 0xc55fb2a4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb95fb2a9, 100 },
    { 0xf2afb2aa, 88 },
    { 0, 0 },
    { 0x252fb2ac, 61 },
    { 0xacafb2ad, 61 },
    { 0x955fb2ad, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8dfb2b2, 66 },
    { 0, 0 },
    { 0xfdffb2b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65fb2b8, 55 },
    { 0x807fb2b9, 66 },
    { 0x604fb2ba, 55 },
    { 0, 0 },
    { 0x379fb2bc, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x585fb2c1, 23 },
    { 0, 0 },
    { 0x5edfb2c3, 34 },
    { 0xbc4fb2c3, 23 },
    { 0xb2fb2c4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd3fb2c9, 98 },
    { 0x7cdfb2c9, 76 },
    { 0, 0 },
    { 0xe73fb2cc, 56 },
    { 0, 0 },
    { 0x3c9fb2ce, 88 },
    { 0x270fb2ce, 99 },
    { 0xcf1fb2d0, 98 },
    { 0xc9efb2cf, 100 },
    { 0x10efb2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98afb2d6, 61 },
    { 0, 0 },
    { 0xac7fb2d8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc0fb2dd, 84 },
    { 0x87fb2de, 61 },
    { 0xb44fb2df, 55 },
    { 0x61ffb2de, 49 },
    { 0xe28fb2e1, 91 },
    { 0xa7dfb2e0, 76 },
    { 0x7effb2de, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xedafb2e6, 93 },
    { 0x881fb2e6, 66 },
    { 0, 0 },
    { 0xf46fb2e9, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3ffb2ed, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x116fb2f0, 90 },
    { 0xabafb2f0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11dfb2fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0fb2fe, 93 },
    { 0xfc7fb2ff, 90 },
    { 0x6c1fb2ff, 95 },
    { 0x73ffb300, 90 },
    { 0x43cfb301, 33 },
    { 0x84fb303, 95 },
    { 0, 0 },
    { 0x62cfb305, 23 },
    { 0xebffb305, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72fb30a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf53fb30d, 90 },
    { 0x29dfb30e, 76 },
    { 0, 0 },
    { 0x81fb310, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe80fb315, 93 },
    { 0, 0 },
    { 0x1befb317, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7efb31a, 33 },
    { 0, 0 },
    { 0x142fb31c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dcfb322, 61 },
    { 0x4e2fb323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xadcfb326, 23 },
    { 0x1e5fb326, 33 },
    { 0x6d9fb328, 68 },
    { 0x368fb329, 61 },
    { 0xc29fb32a, 90 },
    { 0x5dafb32b, 61 },
    { 0x33ffb327, 61 },
    { 0x673fb326, 43 },
    { 0xe1ffb32e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abfb332, 55 },
    { 0, 0 },
    { 0xa52fb334, 99 },
    { 0xcc2fb334, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8fb340, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dfb343, 61 },
    { 0, 0 },
    { 0x972fb345, 52 },
    { 0, 0 },
    { 0x54afb347, 23 },
    { 0, 0 },
    { 0x4ecfb349, 23 },
    { 0x25bfb34a, 91 },
    { 0x8cffb34b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4fb34e, 23 },
    { 0xd69fb34e, 55 },
    { 0xdf3fb34e, 76 },
    { 0x914fb351, 90 },
    { 0, 0 },
    { 0x769fb353, 61 },
    { 0, 0 },
    { 0xae3fb355, 80 },
    { 0, 0 },
    { 0xf15fb357, 34 },
    { 0xd05fb358, 55 },
    { 0x451fb358, 33 },
    { 0x48cfb35a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0fb362, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89fb365, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c8fb369, 23 },
    { 0, 0 },
    { 0xfdfb36b, 2 },
    { 0x342fb36b, 90 },
    { 0x2e5fb36d, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93ffb371, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cfb375, 76 },
    { 0x986fb375, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x692fb37c, 90 },
    { 0, 0 },
    { 0xa7efb37e, 98 },
    { 0, 0 },
    { 0xadafb380, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x182fb383, 33 },
    { 0xc97fb384, 34 },
    { 0xa1bfb383, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46efb389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fb392, 33 },
    { 0, 0 },
    { 0xff4fb394, 34 },
    { 0xfc0fb395, 33 },
    { 0x78dfb395, 34 },
    { 0x3afb394, 46 },
    { 0xda6fb397, 98 },
    { 0, 0 },
    { 0x9fcfb39a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cafb3a9, 61 },
    { 0, 0 },
    { 0xd91fb3ab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb02fb3ae, 85 },
    { 0, 0 },
    { 0xe2dfb3b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366fb3b5, 61 },
    { 0x376fb3b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1fb3ba, 61 },
    { 0x994fb3ba, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eafb3bf, 61 },
    { 0, 0 },
    { 0xc10fb3c1, 61 },
    { 0, 0 },
    { 0xc36fb3c3, 90 },
    { 0, 0 },
    { 0x472fb3c5, 23 },
    { 0, 0 },
    { 0x27cfb3c7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9fb3ca, 23 },
    { 0xd00fb3ca, 33 },
    { 0xb5dfb3cc, 17 },
    { 0x156fb3cc, 23 },
    { 0, 0 },
    { 0xbc2fb3cf, 76 },
    { 0x83afb3d0, 33 },
    { 0, 0 },
    { 0x1b7fb3d2, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf83fb3d9, 98 },
    { 0, 0 },
    { 0x8d1fb3db, 46 },
    { 0, 0 },
    { 0x885fb3dd, 98 },
    { 0, 0 },
    { 0x4a6fb3df, 81 },
    { 0x3bafb3df, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x952fb3e4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0fb3e7, 68 },
    { 0, 0 },
    { 0x91efb3e9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x922fb3ec, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x148fb3f0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ffb3f7, 98 },
    { 0xa91fb3f8, 55 },
    { 0x2d4fb3f8, 98 },
    { 0, 0 },
    { 0x9e6fb3fb, 93 },
    { 0x74cfb3fc, 90 },
    { 0xe09fb3fc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbfb400, 77 },
    { 0xdeafb401, 61 },
    { 0xe5bfb402, 55 },
    { 0, 0 },
    { 0xee2fb404, 87 },
    { 0xd89fb404, 98 },
    { 0x45bfb406, 90 },
    { 0x159fb407, 90 },
    { 0x3a2fb406, 23 },
    { 0x8e8fb408, 75 },
    { 0x85afb409, 95 },
    { 0x157fb40b, 95 },
    { 0x93cfb40b, 35 },
    { 0x158fb40b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfcfb410, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeffb414, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e0fb41a, 87 },
    { 0x30dfb41b, 98 },
    { 0, 0 },
    { 0x2ccfb41d, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bffb420, 23 },
    { 0x1abfb421, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaafb424, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26ffb428, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f1fb42c, 66 },
    { 0xf2efb42d, 77 },
    { 0xefffb42e, 84 },
    { 0xe76fb42d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb434, 88 },
    { 0x6edfb434, 33 },
    { 0x888fb434, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ffb43f, 90 },
    { 0xd8ffb43f, 93 },
    { 0x638fb441, 58 },
    { 0, 0 },
    { 0x895fb443, 34 },
    { 0x80fb444, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6adfb448, 23 },
    { 0xd93fb449, 90 },
    { 0x85bfb449, 55 },
    { 0x35fb449, 83 },
    { 0x3c3fb44a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x92efb44f, 91 },
    { 0x675fb44f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dfb455, 77 },
    { 0x9e1fb455, 77 },
    { 0x196fb455, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774fb45c, 88 },
    { 0, 0 },
    { 0xa4bfb45e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70fb462, 23 },
    { 0, 0 },
    { 0xd83fb464, 34 },
    { 0, 0 },
    { 0xc9cfb466, 98 },
    { 0x537fb466, 91 },
    { 0x238fb468, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77afb474, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f9fb478, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x753fb47b, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4efb47f, 33 },
    { 0xe88fb480, 98 },
    { 0xafb481, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14fb486, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dfb489, 53 },
    { 0, 0 },
    { 0x749fb48b, 93 },
    { 0x541fb48c, 33 },
    { 0x312fb48d, 33 },
    { 0xbfdfb48d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d2fb495, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f4fb49a, 48 },
    { 0x6a1fb49b, 95 },
    { 0xf0dfb49c, 90 },
    { 0xaefb49c, 61 },
    { 0, 0 },
    { 0x917fb49f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86fb4a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeefb4aa, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x769fb4ad, 46 },
    { 0, 0 },
    { 0x63fb4af, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3aefb4b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29afb4bc, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51efb4c1, 98 },
    { 0, 0 },
    { 0xfe5fb4c3, 95 },
    { 0, 0 },
    { 0x53afb4c5, 46 },
    { 0xd69fb4c6, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2bfb4ca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e8fb4ce, 95 },
    { 0x56bfb4cf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe79fb4db, 23 },
    { 0, 0 },
    { 0x5f5fb4dd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4fb4e2, 85 },
    { 0x8ecfb4e2, 2 },
    { 0, 0 },
    { 0xc49fb4e5, 61 },
    { 0xdc0fb4e6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198fb4ec, 77 },
    { 0, 0 },
    { 0x2a2fb4ee, 78 },
    { 0x206fb4ee, 98 },
    { 0, 0 },
    { 0x4f5fb4f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x672fb4fa, 55 },
    { 0xa4efb4fa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56dfb501, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93fb50b, 90 },
    { 0, 0 },
    { 0x195fb50d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bffb510, 55 },
    { 0x3bffb511, 91 },
    { 0xfc2fb510, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x222fb516, 95 },
    { 0, 0 },
    { 0xb23fb518, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158fb51d, 84 },
    { 0xbc4fb51e, 99 },
    { 0xa1afb51f, 95 },
    { 0x8a5fb520, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c8fb525, 46 },
    { 0x81ffb526, 70 },
    { 0xbc2fb527, 84 },
    { 0, 0 },
    { 0x4f9fb529, 90 },
    { 0x382fb52a, 90 },
    { 0x7c7fb52a, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cfb530, 33 },
    { 0xbb7fb531, 91 },
    { 0x25afb531, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7fb535, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf1fb53c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf9fb540, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x82afb543, 61 },
    { 0xcefb543, 66 },
    { 0, 0 },
    { 0x91ffb546, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bfb549, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6bfb54e, 90 },
    { 0x266fb54f, 61 },
    { 0, 0 },
    { 0x63efb551, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cfb554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a5fb55b, 61 },
    { 0x2fffb55c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d5fb560, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ebfb565, 99 },
    { 0xb34fb566, 75 },
    { 0xd50fb567, 77 },
    { 0, 0 },
    { 0xa0dfb569, 93 },
    { 0x767fb569, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fb570, 99 },
    { 0x914fb571, 61 },
    { 0, 0 },
    { 0x612fb573, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x766fb576, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dfb579, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd29fb57e, 33 },
    { 0xdb2fb57f, 33 },
    { 0, 0 },
    { 0x4b3fb581, 87 },
    { 0x877fb582, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x745fb586, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbfb58a, 90 },
    { 0x664fb58b, 98 },
    { 0xcd0fb58c, 61 },
    { 0xfd3fb58d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaffb595, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0fb599, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x754fb5a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2fb5a4, 98 },
    { 0x3efb5a5, 99 },
    { 0, 0 },
    { 0xe30fb5a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x300fb5ac, 77 },
    { 0, 0 },
    { 0x51afb5ae, 90 },
    { 0x7a8fb5ae, 23 },
    { 0xd9bfb5ae, 90 },
    { 0x75ffb5ae, 61 },
    { 0x970fb5b2, 90 },
    { 0, 0 },
    { 0x4bdfb5b4, 90 },
    { 0x3c1fb5b5, 33 },
    { 0xc32fb5b6, 98 },
    { 0x55bfb5b7, 93 },
    { 0xff8fb5b8, 65 },
    { 0x966fb5b8, 17 },
    { 0x9cefb5ba, 23 },
    { 0x1a2fb5bb, 90 },
    { 0xe86fb5b9, 90 },
    { 0xb41fb5b8, 84 },
    { 0x32bfb5bb, 90 },
    { 0x8effb5bf, 90 },
    { 0xe9ffb5bf, 100 },
    { 0x538fb5c0, 46 },
    { 0x30fb5bf, 61 },
    { 0x25cfb5c3, 76 },
    { 0, 0 },
    { 0xc47fb5c5, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bdfb5c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cfb5d0, 66 },
    { 0xef9fb5d1, 23 },
    { 0xa03fb5d1, 84 },
    { 0x3e6fb5d0, 76 },
    { 0x28bfb5d4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0fb5e1, 61 },
    { 0xfb8fb5e2, 95 },
    { 0x545fb5e2, 98 },
    { 0x78efb5e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x863fb5e7, 46 },
    { 0xa3efb5e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d0fb5ee, 90 },
    { 0xc8afb5ef, 77 },
    { 0x285fb5ef, 95 },
    { 0x48afb5ee, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2fb5f4, 98 },
    { 0x2b4fb5f5, 33 },
    { 0xf34fb5f4, 37 },
    { 0xb10fb5f7, 61 },
    { 0xffefb5f5, 55 },
    { 0, 0 },
    { 0xde5fb5fa, 43 },
    { 0xad8fb5fa, 95 },
    { 0x127fb5fa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbcfb600, 33 },
    { 0xb73fb601, 95 },
    { 0, 0 },
    { 0x2b4fb603, 88 },
    { 0, 0 },
    { 0x6b3fb605, 23 },
    { 0x927fb606, 46 },
    { 0xf90fb606, 17 },
    { 0xcd1fb605, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc88fb60d, 85 },
    { 0, 0 },
    { 0xfaafb60f, 72 },
    { 0, 0 },
    { 0x24bfb611, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528fb615, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc59fb61a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62ffb61d, 66 },
    { 0xd5efb61e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf6fb622, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x27ffb625, 23 },
    { 0x7b9fb625, 52 },
    { 0xf7ffb627, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2fb62a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cdfb62d, 28 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92fb637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x784fb63a, 90 },
    { 0xe98fb63b, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dfb63f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fb642, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x197fb645, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb79fb64b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f6fb64f, 98 },
    { 0x4f7fb650, 33 },
    { 0x200fb651, 67 },
    { 0, 0 },
    { 0x406fb653, 76 },
    { 0x4d3fb654, 93 },
    { 0, 0 },
    { 0xaf1fb656, 38 },
    { 0x140fb657, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f2fb65b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7fb660, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x125fb666, 98 },
    { 0x5e4fb666, 90 },
    { 0xcbdfb668, 98 },
    { 0x50afb667, 97 },
    { 0x254fb66a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80fb670, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f2fb675, 85 },
    { 0, 0 },
    { 0xca9fb677, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a9fb67d, 23 },
    { 0, 0 },
    { 0x4cbfb67f, 23 },
    { 0x4d8fb67f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c7fb683, 61 },
    { 0x658fb684, 61 },
    { 0, 0 },
    { 0x1ddfb686, 90 },
    { 0, 0 },
    { 0x1e8fb688, 61 },
    { 0x3a4fb689, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2fb68e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x948fb691, 98 },
    { 0x731fb691, 61 },
    { 0, 0 },
    { 0xf0dfb694, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0fb699, 90 },
    { 0xbadfb69a, 37 },
    { 0, 0 },
    { 0x15cfb69c, 76 },
    { 0, 0 },
    { 0x2b4fb69e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbefb6a2, 61 },
    { 0x635fb6a3, 61 },
    { 0xd5ffb6a4, 36 },
    { 0xf39fb6a5, 78 },
    { 0x73dfb6a6, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97fb6ab, 87 },
    { 0xb46fb6ac, 99 },
    { 0, 0 },
    { 0xf0efb6ae, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x751fb6b1, 95 },
    { 0x18fb6b2, 95 },
    { 0x519fb6b3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1bfb6bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70dfb6c4, 90 },
    { 0x173fb6c4, 78 },
    { 0, 0 },
    { 0x4e3fb6c7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fb6ca, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadfb6cf, 99 },
    { 0, 0 },
    { 0x42bfb6d1, 90 },
    { 0x69cfb6d2, 90 },
    { 0x4cdfb6d3, 33 },
    { 0x88dfb6d2, 61 },
    { 0, 0 },
    { 0x27afb6d6, 61 },
    { 0x855fb6d7, 91 },
    { 0x9ddfb6d8, 46 },
    { 0, 0 },
    { 0x519fb6da, 33 },
    { 0, 0 },
    { 0xf9dfb6dc, 90 },
    { 0x559fb6dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336fb6e8, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4efb6eb, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x16ffb6ee, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x33bfb6f1, 93 },
    { 0, 0 },
    { 0xf8fb6f3, 90 },
    { 0x568fb6f4, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91fb6f7, 98 },
    { 0x5c7fb6f8, 95 },
    { 0x241fb6f8, 61 },
    { 0, 0 },
    { 0xcfb6fb, 61 },
    { 0x36afb6fc, 33 },
    { 0xe3cfb6fd, 34 },
    { 0, 0 },
    { 0xfa2fb6ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d4fb702, 46 },
    { 0xe2fb703, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x800fb706, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ffb70e, 34 },
    { 0x394fb70f, 90 },
    { 0, 0 },
    { 0xdb3fb711, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51bfb719, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fafb71c, 33 },
    { 0xf2efb71d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ffb721, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x568fb724, 23 },
    { 0, 0 },
    { 0xcf4fb726, 55 },
    { 0xdb5fb726, 92 },
    { 0xfdffb726, 33 },
    { 0x2c2fb728, 87 },
    { 0x341fb727, 95 },
    { 0x123fb72a, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x753fb731, 93 },
    { 0x9a5fb732, 33 },
    { 0, 0 },
    { 0xb75fb734, 90 },
    { 0, 0 },
    { 0xfe3fb736, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2bfb739, 61 },
    { 0xefbfb73a, 98 },
    { 0x2d9fb73b, 91 },
    { 0, 0 },
    { 0x559fb73d, 33 },
    { 0, 0 },
    { 0x19bfb73f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fb747, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84fb74f, 92 },
    { 0xa5ffb74f, 56 },
    { 0x2fefb751, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16dfb757, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83fb75a, 90 },
    { 0x44fb75b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89fb762, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x432fb765, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffb76b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54fb770, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8defb773, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaffb77d, 23 },
    { 0xa0fb77e, 23 },
    { 0xbaafb77e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fafb782, 95 },
    { 0x8b6fb782, 90 },
    { 0xa68fb784, 23 },
    { 0x302fb784, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfefb788, 43 },
    { 0, 0 },
    { 0xddffb78a, 66 },
    { 0x180fb78b, 23 },
    { 0xc17fb78a, 46 },
    { 0, 0 },
    { 0x5edfb78e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x162fb791, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6fb796, 66 },
    { 0, 0 },
    { 0x8adfb798, 90 },
    { 0, 0 },
    { 0x474fb79a, 75 },
    { 0, 0 },
    { 0xc8bfb79c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0afb79f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23dfb7a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x629fb7a8, 34 },
    { 0, 0 },
    { 0x39cfb7aa, 33 },
    { 0x67efb7aa, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4fb7b2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x189fb7b5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd77fb7b8, 33 },
    { 0x860fb7b9, 23 },
    { 0xa8cfb7b8, 75 },
    { 0xb65fb7bb, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fb7be, 81 },
    { 0x377fb7be, 33 },
    { 0x371fb7be, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ffb7c4, 95 },
    { 0x830fb7c5, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2efb7c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90fb7ce, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea4fb7d2, 33 },
    { 0x664fb7d3, 23 },
    { 0xc3bfb7d2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12fb7d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dfb7dc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695fb7e2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xefbfb7e5, 23 },
    { 0x83dfb7e6, 43 },
    { 0xd3efb7e6, 85 },
    { 0x527fb7e6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bfb7eb, 23 },
    { 0x507fb7eb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x442fb7ef, 66 },
    { 0xaa8fb7f0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47bfb7fc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f1fb800, 93 },
    { 0, 0 },
    { 0xcccfb802, 99 },
    { 0x5bffb803, 65 },
    { 0xf3bfb804, 98 },
    { 0xd7dfb805, 90 },
    { 0, 0 },
    { 0xdfdfb807, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d3fb80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb37fb810, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44fb814, 93 },
    { 0, 0 },
    { 0xff3fb816, 34 },
    { 0, 0 },
    { 0xe93fb818, 42 },
    { 0x325fb818, 99 },
    { 0xf39fb819, 76 },
    { 0x44cfb81b, 90 },
    { 0, 0 },
    { 0x56dfb81d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42fb820, 36 },
    { 0xbfbfb821, 99 },
    { 0xb4efb822, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30fb82c, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8adfb830, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6fb836, 23 },
    { 0x333fb837, 90 },
    { 0xe11fb837, 93 },
    { 0xfe9fb839, 67 },
    { 0, 0 },
    { 0x6a3fb83b, 52 },
    { 0xc61fb83c, 95 },
    { 0xc5bfb83b, 55 },
    { 0, 0 },
    { 0xa42fb83f, 93 },
    { 0xb5ffb83f, 43 },
    { 0, 0 },
    { 0x240fb842, 95 },
    { 0x19ffb843, 46 },
    { 0x15fb843, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebfb848, 90 },
    { 0x47ffb849, 33 },
    { 0xd26fb849, 61 },
    { 0x46ffb84b, 95 },
    { 0x93afb84c, 98 },
    { 0xcc7fb84d, 34 },
    { 0xf14fb84e, 90 },
    { 0x5efb84e, 78 },
    { 0xa55fb84d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x134fb854, 85 },
    { 0x637fb855, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fbfb858, 33 },
    { 0x26efb858, 45 },
    { 0xbd0fb85a, 90 },
    { 0xaaafb85b, 23 },
    { 0x2bbfb85c, 90 },
    { 0, 0 },
    { 0x7b5fb85e, 23 },
    { 0x4d6fb85e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86fb864, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdffb86a, 34 },
    { 0xfb2fb86b, 88 },
    { 0xa0fb86b, 23 },
    { 0x91ffb86d, 33 },
    { 0xb88fb86a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf6fb874, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5fb877, 33 },
    { 0x860fb877, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aefb87d, 33 },
    { 0, 0 },
    { 0x4a2fb87f, 33 },
    { 0x587fb87f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90ffb888, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f7fb88b, 90 },
    { 0, 0 },
    { 0x4bbfb88d, 76 },
    { 0x826fb88e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd65fb892, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0xb12fb895, 90 },
    { 0x860fb895, 32 },
    { 0xb8fb897, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe34fb89d, 99 },
    { 0x5b9fb89d, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6fb8a2, 76 },
    { 0x5c5fb8a2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc83fb8a6, 33 },
    { 0xc47fb8a7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd73fb8ab, 67 },
    { 0x8a1fb8ab, 98 },
    { 0, 0 },
    { 0x393fb8ae, 98 },
    { 0xca9fb8af, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2cfb8b7, 45 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0fb8bb, 98 },
    { 0, 0 },
    { 0x7e9fb8bd, 65 },
    { 0xcd4fb8be, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5fb8c1, 33 },
    { 0, 0 },
    { 0xd17fb8c3, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d6fb8ca, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x446fb8cd, 23 },
    { 0x88afb8ce, 33 },
    { 0x1a8fb8cd, 34 },
    { 0x316fb8d0, 55 },
    { 0xd96fb8d1, 61 },
    { 0x973fb8d2, 99 },
    { 0xf62fb8d3, 61 },
    { 0, 0 },
    { 0x2d3fb8d5, 33 },
    { 0, 0 },
    { 0xb55fb8d7, 34 },
    { 0x93dfb8d7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54dfb8e3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946fb8e7, 33 },
    { 0x640fb8e8, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecffb8ec, 61 },
    { 0x628fb8ec, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1fb8f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bafb8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6bfb907, 90 },
    { 0, 0 },
    { 0x59ffb909, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x37afb914, 99 },
    { 0x998fb914, 95 },
    { 0xa2dfb914, 95 },
    { 0, 0 },
    { 0xf4bfb918, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d8fb91b, 33 },
    { 0x262fb91c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cfb925, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15afb931, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc54fb938, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0x539fb93b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44fb93e, 91 },
    { 0xeddfb93f, 76 },
    { 0x5c2fb940, 90 },
    { 0xa18fb941, 61 },
    { 0x151fb93e, 23 },
    { 0, 0 },
    { 0x7defb944, 33 },
    { 0x965fb945, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x543fb950, 34 },
    { 0xa11fb951, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30fb954, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfecfb95f, 91 },
    { 0x3affb95f, 95 },
    { 0xa42fb960, 34 },
    { 0, 0 },
    { 0x24bfb963, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbfb968, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c2fb96c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbafb970, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcfb973, 91 },
    { 0x83efb974, 95 },
    { 0xb8afb974, 76 },
    { 0, 0 },
    { 0x59afb977, 23 },
    { 0xcfffb978, 95 },
    { 0x367fb979, 100 },
    { 0x715fb97a, 87 },
    { 0xfd7fb97b, 33 },
    { 0xe23fb97b, 33 },
    { 0x815fb97d, 84 },
    { 0xe2bfb97e, 61 },
    { 0, 0 },
    { 0x446fb980, 90 },
    { 0xb15fb980, 99 },
    { 0x40fb982, 77 },
    { 0x3edfb982, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cafb986, 99 },
    { 0xc1cfb987, 61 },
    { 0xd26fb988, 55 },
    { 0xd37fb988, 55 },
    { 0x593fb98a, 99 },
    { 0x78afb98b, 91 },
    { 0xdddfb98b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07fb98f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6cfb993, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9afb997, 88 },
    { 0x142fb997, 33 },
    { 0xf74fb999, 66 },
    { 0xf66fb99a, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x944fb99d, 76 },
    { 0x7a8fb99e, 33 },
    { 0x3e2fb99f, 90 },
    { 0xc92fb9a0, 87 },
    { 0xaa1fb9a1, 90 },
    { 0, 0 },
    { 0x9e1fb9a3, 61 },
    { 0x5d7fb9a4, 90 },
    { 0, 0 },
    { 0x905fb9a6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ebfb9ad, 33 },
    { 0x9f0fb9ad, 98 },
    { 0, 0 },
    { 0x5ccfb9b0, 46 },
    { 0x5aafb9b1, 85 },
    { 0x654fb9b2, 94 },
    { 0xc99fb9b3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x763fb9b9, 90 },
    { 0, 0 },
    { 0x6dfb9bb, 98 },
    { 0x1bfb9bc, 61 },
    { 0x3b4fb9bc, 98 },
    { 0x80afb9be, 23 },
    { 0x445fb9bf, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebdfb9c5, 87 },
    { 0x8f8fb9c5, 23 },
    { 0, 0 },
    { 0x5f7fb9c8, 95 },
    { 0x505fb9c9, 90 },
    { 0x9befb9c9, 23 },
    { 0x648fb9cb, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ffb9ce, 23 },
    { 0, 0 },
    { 0x64dfb9d0, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07fb9d4, 91 },
    { 0x3dafb9d5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1fb9db, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed2fb9e0, 52 },
    { 0, 0 },
    { 0x206fb9e2, 96 },
    { 0x9b2fb9e3, 33 },
    { 0x35afb9e4, 88 },
    { 0x27bfb9e5, 40 },
    { 0, 0 },
    { 0x9f8fb9e7, 91 },
    { 0x9d3fb9e8, 52 },
    { 0x6f5fb9e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xccffb9ec, 85 },
    { 0, 0 },
    { 0x4adfb9ee, 61 },
    { 0xfa3fb9ef, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993fb9f3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcfb9fa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395fb9fe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc60fba03, 90 },
    { 0xeaafba03, 75 },
    { 0x50dfba05, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xac8fba08, 66 },
    { 0x675fba09, 90 },
    { 0x902fba0a, 98 },
    { 0x9a6fba0a, 95 },
    { 0x82fba0a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbffba10, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37fba13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc3fba1a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x198fba1e, 43 },
    { 0x340fba1f, 76 },
    { 0xff8fba20, 95 },
    { 0x215fba21, 61 },
    { 0, 0 },
    { 0xa57fba23, 48 },
    { 0x875fba24, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ffba2a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8abfba2e, 33 },
    { 0xe72fba2f, 33 },
    { 0x3e7fba2f, 90 },
    { 0xd13fba30, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ffba36, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3fba39, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7efba3d, 87 },
    { 0x3f8fba3e, 90 },
    { 0xe8bfba3f, 23 },
    { 0xd0afba3e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fffba43, 33 },
    { 0, 0 },
    { 0x2b4fba45, 85 },
    { 0x2a1fba46, 80 },
    { 0x76cfba47, 95 },
    { 0x79dfba48, 52 },
    { 0xdeafba49, 34 },
    { 0, 0 },
    { 0x1d2fba4b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efba50, 23 },
    { 0x44cfba51, 61 },
    { 0, 0 },
    { 0x700fba53, 90 },
    { 0x1ccfba54, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x66dfba57, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84fba5d, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x734fba60, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd40fba63, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd25fba67, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cfba6b, 47 },
    { 0, 0 },
    { 0xc4dfba6d, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea0fba75, 43 },
    { 0xa69fba75, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddfba7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaadfba8a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa8fba90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a6fba9c, 53 },
    { 0x81ffba9d, 98 },
    { 0xe89fba9d, 76 },
    { 0x3e4fba9f, 43 },
    { 0x62dfba9f, 98 },
    { 0xe2cfba9e, 95 },
    { 0x355fbaa2, 43 },
    { 0, 0 },
    { 0x3bafbaa4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70fbaa9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edfbaac, 84 },
    { 0xd9afbaad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x75afbab0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x783fbab3, 95 },
    { 0x772fbab4, 84 },
    { 0, 0 },
    { 0xfedfbab6, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x578fbaba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360fbabe, 98 },
    { 0x752fbabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48bfbac2, 94 },
    { 0x16bfbac3, 33 },
    { 0xb6dfbac4, 23 },
    { 0, 0 },
    { 0x789fbac6, 98 },
    { 0xe5efbac7, 90 },
    { 0xfecfbac8, 85 },
    { 0xa6afbac9, 95 },
    { 0xde8fbaca, 98 },
    { 0xee5fbacb, 95 },
    { 0xc1afbac9, 23 },
    { 0x60bfbacc, 33 },
    { 0x947fbac9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f0fbad1, 61 },
    { 0xf71fbad1, 91 },
    { 0x79efbad3, 95 },
    { 0xc7dfbad3, 90 },
    { 0, 0 },
    { 0xb9afbad6, 98 },
    { 0x35cfbad7, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5efbada, 95 },
    { 0x5edfbada, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeefbae1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c8fbae7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc06fbaec, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x581fbaf1, 34 },
    { 0x39afbaf2, 33 },
    { 0x9ebfbaf1, 91 },
    { 0xc07fbaf4, 43 },
    { 0xa4efbaf5, 98 },
    { 0x8b6fbaf6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc36fbafa, 23 },
    { 0, 0 },
    { 0x389fbafc, 90 },
    { 0xeb1fbafd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a7fbb01, 33 },
    { 0xbfefbb01, 33 },
    { 0xc79fbb03, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x399fbb07, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x39cfbb0a, 55 },
    { 0x59cfbb0b, 34 },
    { 0x558fbb0c, 98 },
    { 0xf36fbb0d, 66 },
    { 0x89afbb0e, 100 },
    { 0x935fbb0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a2fbb14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd21fbb17, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfbb1c, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cfbb26, 84 },
    { 0x544fbb27, 33 },
    { 0x717fbb28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19efbb33, 66 },
    { 0x265fbb34, 95 },
    { 0x5b7fbb35, 33 },
    { 0, 0 },
    { 0x7a4fbb37, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x46dfbb3c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ffbb40, 87 },
    { 0xe16fbb41, 23 },
    { 0xb7efbb42, 66 },
    { 0xb21fbb41, 66 },
    { 0, 0 },
    { 0x769fbb45, 76 },
    { 0x2b8fbb45, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xa32fbb49, 55 },
    { 0x2afbb4a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dffbb4e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf16fbb51, 84 },
    { 0x209fbb51, 29 },
    { 0xcbafbb53, 90 },
    { 0, 0 },
    { 0xd0bfbb55, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416fbb59, 66 },
    { 0x591fbb5a, 90 },
    { 0xd92fbb5a, 90 },
    { 0x42fbb5a, 43 },
    { 0xebafbb59, 96 },
    { 0xf1efbb5e, 90 },
    { 0x440fbb5b, 61 },
    { 0x33bfbb5f, 90 },
    { 0xfccfbb61, 23 },
    { 0, 0 },
    { 0x428fbb63, 98 },
    { 0x324fbb63, 95 },
    { 0, 0 },
    { 0xba7fbb66, 33 },
    { 0xec4fbb66, 61 },
    { 0, 0 },
    { 0x4c7fbb69, 33 },
    { 0x7f6fbb69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x750fbb6d, 33 },
    { 0xc74fbb6e, 17 },
    { 0x7fafbb6d, 3 },
    { 0xdc5fbb70, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x74fbb73, 90 },
    { 0, 0 },
    { 0x8fffbb75, 33 },
    { 0x4d1fbb76, 98 },
    { 0, 0 },
    { 0x954fbb78, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf0fbb7d, 33 },
    { 0x490fbb7d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3fbb81, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd47fbb85, 61 },
    { 0, 0 },
    { 0x63afbb87, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0ffbb8b, 98 },
    { 0, 0 },
    { 0x85ffbb8d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43fbb91, 46 },
    { 0xb09fbb92, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dbfbb97, 51 },
    { 0xea6fbb98, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x368fbba3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d0fbbab, 23 },
    { 0xbfbfbbab, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadbfbbb0, 68 },
    { 0x2e5fbbb0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fffbbb4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x58afbbb7, 93 },
    { 0, 0 },
    { 0xc36fbbb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1fbbc1, 95 },
    { 0x711fbbc2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48efbbca, 23 },
    { 0x2bbfbbca, 93 },
    { 0x65efbbcc, 76 },
    { 0x4edfbbcc, 80 },
    { 0xacffbbca, 78 },
    { 0xee6fbbcf, 95 },
    { 0x845fbbd0, 66 },
    { 0, 0 },
    { 0xeb3fbbd2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0fbbd6, 77 },
    { 0xc1cfbbd7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190fbbdb, 90 },
    { 0xdaffbbdb, 90 },
    { 0x9d1fbbdb, 98 },
    { 0xd89fbbdd, 85 },
    { 0, 0 },
    { 0xd50fbbe0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1cfbbe3, 98 },
    { 0x901fbbe4, 100 },
    { 0x2f1fbbe5, 61 },
    { 0xc8cfbbe6, 23 },
    { 0, 0 },
    { 0x2abfbbe8, 23 },
    { 0x5c8fbbe8, 61 },
    { 0xb7fbbea, 91 },
    { 0, 0 },
    { 0x708fbbec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafbbf2, 76 },
    { 0, 0 },
    { 0x92fbbf4, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe71fbbf9, 90 },
    { 0xf59fbbfa, 61 },
    { 0xfb7fbbfb, 33 },
    { 0x6cafbbfc, 34 },
    { 0xda2fbbfc, 55 },
    { 0x644fbbfe, 44 },
    { 0xd44fbbfe, 98 },
    { 0xf75fbc00, 23 },
    { 0x2a9fbc01, 95 },
    { 0xbfbfbc02, 61 },
    { 0, 0 },
    { 0xff2fbc04, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9fbc08, 99 },
    { 0x2c1fbc08, 90 },
    { 0x181fbc0a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ccfbc12, 55 },
    { 0, 0 },
    { 0x5a9fbc14, 23 },
    { 0x44afbc15, 52 },
    { 0, 0 },
    { 0xc7fbc17, 55 },
    { 0, 0 },
    { 0x782fbc19, 98 },
    { 0x2d4fbc1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35bfbc1f, 90 },
    { 0xca7fbc20, 90 },
    { 0x51cfbc21, 95 },
    { 0x1a4fbc22, 34 },
    { 0xec9fbc23, 98 },
    { 0x60fbc21, 55 },
    { 0x87cfbc25, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cfbc2d, 98 },
    { 0, 0 },
    { 0x32dfbc2f, 33 },
    { 0x7c9fbc30, 93 },
    { 0, 0 },
    { 0xbd1fbc32, 57 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee7fbc37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd1fbc3a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x65afbc3d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aefbc40, 98 },
    { 0x307fbc41, 91 },
    { 0xa80fbc41, 85 },
    { 0xd06fbc41, 55 },
    { 0x3effbc44, 43 },
    { 0x406fbc42, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ffbc4c, 67 },
    { 0x4b7fbc4d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87afbc53, 52 },
    { 0xfeafbc54, 43 },
    { 0x7e6fbc54, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x934fbc5a, 18 },
    { 0, 0 },
    { 0, 0 },
    { 0x196fbc5d, 95 },
    { 0xae8fbc5d, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9fbc63, 84 },
    { 0, 0 },
    { 0x2a0fbc65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x223fbc69, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78fbc6d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x498fbc70, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x47afbc73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42fbc79, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3bfbc7e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90fbc81, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e1fbc87, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef3fbc8e, 75 },
    { 0xee6fbc8f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563fbc93, 95 },
    { 0, 0 },
    { 0x557fbc95, 76 },
    { 0xf36fbc96, 98 },
    { 0x535fbc97, 33 },
    { 0x562fbc98, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa8fbc9b, 95 },
    { 0xd29fbc9c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cfbca3, 95 },
    { 0, 0 },
    { 0xfe2fbca5, 84 },
    { 0x69cfbca6, 68 },
    { 0x25afbca6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc7fbcac, 36 },
    { 0x692fbcac, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19fbcb3, 68 },
    { 0, 0 },
    { 0x12efbcb5, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe45fbcb8, 23 },
    { 0x535fbcb9, 93 },
    { 0, 0 },
    { 0x99bfbcbb, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1fbcbe, 95 },
    { 0xa4bfbcbf, 33 },
    { 0x9defbcbe, 67 },
    { 0x3f2fbcc1, 33 },
    { 0, 0 },
    { 0x7f4fbcc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5fbcc6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5afbcca, 91 },
    { 0xac3fbccb, 76 },
    { 0, 0 },
    { 0x98efbccd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x848fbcd0, 95 },
    { 0xdd8fbcd0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8fbcd4, 61 },
    { 0, 0 },
    { 0x313fbcd6, 95 },
    { 0xa2dfbcd7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a6fbcdc, 50 },
    { 0x653fbcdd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x507fbce0, 95 },
    { 0, 0 },
    { 0xaf5fbce2, 90 },
    { 0xf3cfbce2, 78 },
    { 0x4f4fbce4, 98 },
    { 0, 0 },
    { 0xc5efbce6, 34 },
    { 0xc5dfbce6, 67 },
    { 0x6bbfbce8, 99 },
    { 0xda9fbce6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eefbcf2, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7afbcfa, 65 },
    { 0x1a3fbcfb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874fbd08, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x775fbd0c, 93 },
    { 0xb8cfbd0d, 88 },
    { 0x8fbd0e, 90 },
    { 0xcaffbd0f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x97cfbd16, 46 },
    { 0x9a3fbd17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4dfbd1b, 33 },
    { 0x797fbd1b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8efbd20, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcfbd23, 23 },
    { 0, 0 },
    { 0x711fbd25, 100 },
    { 0xa4efbd25, 56 },
    { 0, 0 },
    { 0x1d1fbd28, 33 },
    { 0x47dfbd29, 98 },
    { 0, 0 },
    { 0x54cfbd2b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffbd2f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6fbd32, 45 },
    { 0xbf2fbd32, 90 },
    { 0, 0 },
    { 0x1b1fbd35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5afbd38, 98 },
    { 0x4edfbd39, 61 },
    { 0x7bffbd39, 38 },
    { 0xd53fbd39, 23 },
    { 0x246fbd3c, 95 },
    { 0x29efbd3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bafbd40, 61 },
    { 0xc03fbd40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c8fbd49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc85fbd4e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dfbd51, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8fbd54, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6bfbd57, 78 },
    { 0xcbafbd58, 61 },
    { 0x172fbd59, 67 },
    { 0xd03fbd5a, 68 },
    { 0x306fbd5a, 40 },
    { 0x31afbd58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfbd5f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x215fbd66, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562fbd6a, 98 },
    { 0xdd3fbd6a, 90 },
    { 0x7fcfbd6c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a0fbd6f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5efbd73, 98 },
    { 0x8c3fbd74, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4fbd79, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7bfbd82, 76 },
    { 0, 0 },
    { 0x70cfbd84, 33 },
    { 0xac2fbd85, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x45afbd8b, 76 },
    { 0x63fbd8b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bffbd90, 90 },
    { 0x470fbd91, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4fbd96, 34 },
    { 0, 0 },
    { 0xdecfbd98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bfbd9b, 90 },
    { 0x25dfbd9b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x687fbda0, 98 },
    { 0, 0 },
    { 0x8a0fbda2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x55cfbda5, 8 },
    { 0x711fbda6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd52fbda9, 90 },
    { 0x3fdfbda9, 90 },
    { 0, 0 },
    { 0x8e1fbdac, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b7fbdb0, 93 },
    { 0x2b1fbdb1, 95 },
    { 0xfa3fbdb2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcbfbdb7, 55 },
    { 0, 0 },
    { 0x58efbdb9, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedfbdcb, 78 },
    { 0x703fbdcb, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa5fbdcf, 33 },
    { 0x2e9fbdcf, 38 },
    { 0x3e3fbdd1, 94 },
    { 0x9abfbdd1, 61 },
    { 0x890fbdcf, 46 },
    { 0x7dffbdd4, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1fbddd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e1fbde1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7fbdeb, 43 },
    { 0xd37fbdec, 23 },
    { 0x628fbded, 34 },
    { 0xa54fbdee, 34 },
    { 0xbcbfbded, 33 },
    { 0, 0 },
    { 0x70efbdf1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72fbdf5, 75 },
    { 0, 0 },
    { 0xcb2fbdf7, 93 },
    { 0xd3bfbdf7, 23 },
    { 0, 0 },
    { 0xecdfbdfa, 66 },
    { 0x3b2fbdfa, 99 },
    { 0xd1cfbdfc, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa9fbe02, 52 },
    { 0x443fbe03, 99 },
    { 0x466fbe02, 76 },
    { 0, 0 },
    { 0x4a7fbe06, 76 },
    { 0x232fbe07, 90 },
    { 0xdfcfbe08, 95 },
    { 0x64dfbe08, 95 },
    { 0, 0 },
    { 0x866fbe0b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf43fbe10, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x631fbe1a, 23 },
    { 0xe09fbe1a, 23 },
    { 0x985fbe1c, 65 },
    { 0x4a3fbe1d, 90 },
    { 0xbecfbe1b, 33 },
    { 0xb3dfbe1f, 55 },
    { 0x67dfbe20, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2fbe27, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x650fbe2c, 98 },
    { 0x936fbe2c, 75 },
    { 0x375fbe2e, 61 },
    { 0xb69fbe2c, 66 },
    { 0x68ffbe30, 76 },
    { 0x1f4fbe31, 61 },
    { 0x3cbfbe32, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f3fbe35, 61 },
    { 0, 0 },
    { 0x819fbe37, 87 },
    { 0, 0 },
    { 0x9eefbe39, 90 },
    { 0x2c7fbe39, 33 },
    { 0, 0 },
    { 0xf9bfbe3c, 76 },
    { 0x41cfbe3d, 90 },
    { 0, 0 },
    { 0xb27fbe3f, 55 },
    { 0x1b9fbe40, 53 },
    { 0x416fbe41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbfbe46, 90 },
    { 0xf50fbe47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23fbe4a, 95 },
    { 0x2adfbe4a, 55 },
    { 0x873fbe4c, 90 },
    { 0x8bcfbe4c, 76 },
    { 0xf48fbe4b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568fbe56, 100 },
    { 0, 0 },
    { 0x412fbe58, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x969fbe5c, 33 },
    { 0, 0 },
    { 0x5d8fbe5e, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x989fbe63, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea2fbe6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda9fbe77, 66 },
    { 0x3dfbe78, 26 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f6fbe7d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b9fbe80, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffbe84, 93 },
    { 0xf48fbe85, 90 },
    { 0xc5bfbe86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b1fbe89, 90 },
    { 0x7c1fbe8a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360fbe90, 78 },
    { 0, 0 },
    { 0xf7ffbe92, 93 },
    { 0, 0 },
    { 0xf82fbe94, 33 },
    { 0x568fbe94, 23 },
    { 0xeeefbe95, 36 },
    { 0x330fbe95, 33 },
    { 0x2c6fbe96, 76 },
    { 0x8e9fbe99, 90 },
    { 0x8dfbe9a, 46 },
    { 0xef9fbe9b, 55 },
    { 0x938fbe9c, 33 },
    { 0xfcdfbe9d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe64fbea0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58efbea4, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x3abfbea7, 90 },
    { 0x892fbea8, 46 },
    { 0, 0 },
    { 0x57ffbeaa, 98 },
    { 0xdd8fbeab, 33 },
    { 0xd81fbeac, 88 },
    { 0x611fbead, 93 },
    { 0xa42fbead, 33 },
    { 0x220fbeaf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x923fbeb2, 98 },
    { 0, 0 },
    { 0xa15fbeb4, 67 },
    { 0x2a6fbeb4, 90 },
    { 0xdbefbeb6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89fbeba, 76 },
    { 0xbe7fbebb, 33 },
    { 0x8b4fbebb, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde5fbec4, 66 },
    { 0, 0 },
    { 0xb64fbec6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x48efbec9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dfbed0, 80 },
    { 0x8e8fbed1, 61 },
    { 0, 0 },
    { 0xdc7fbed3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fbed7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d2fbedd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eafbee0, 90 },
    { 0, 0 },
    { 0xd03fbee2, 99 },
    { 0x17ffbee2, 43 },
    { 0xd31fbee4, 61 },
    { 0x95efbee4, 33 },
    { 0x811fbee6, 43 },
    { 0xd52fbee5, 73 },
    { 0xf5bfbee7, 33 },
    { 0x8cdfbee9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bfbeed, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4fbef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fbef8, 85 },
    { 0xe25fbef9, 90 },
    { 0xed7fbef8, 90 },
    { 0x9ccfbefb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242fbf02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x482fbf05, 61 },
    { 0x19cfbf06, 90 },
    { 0x68dfbf05, 90 },
    { 0x6e1fbf08, 98 },
    { 0x448fbf06, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x303fbf10, 61 },
    { 0x57dfbf11, 93 },
    { 0xb2cfbf11, 33 },
    { 0xcd8fbf11, 61 },
    { 0xa1fbf14, 61 },
    { 0xda3fbf11, 66 },
    { 0xd9ffbf11, 100 },
    { 0x2c5fbf14, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x491fbf1a, 66 },
    { 0x77bfbf1b, 63 },
    { 0x3b1fbf1b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6afbf23, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafbf28, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x411fbf30, 98 },
    { 0x9bdfbf30, 96 },
    { 0, 0 },
    { 0xdbefbf33, 76 },
    { 0, 0 },
    { 0x457fbf35, 90 },
    { 0xeacfbf36, 61 },
    { 0xd25fbf36, 95 },
    { 0, 0 },
    { 0xe5bfbf39, 90 },
    { 0x5f0fbf3a, 65 },
    { 0xf5fbf3b, 40 },
    { 0xac7fbf3c, 91 },
    { 0, 0 },
    { 0x1affbf3e, 94 },
    { 0x277fbf3f, 23 },
    { 0, 0 },
    { 0x647fbf41, 98 },
    { 0x432fbf42, 84 },
    { 0x314fbf43, 23 },
    { 0, 0 },
    { 0x747fbf45, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbebfbf49, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b1fbf4f, 95 },
    { 0x6c1fbf50, 91 },
    { 0x536fbf51, 78 },
    { 0x425fbf52, 61 },
    { 0, 0 },
    { 0xd7ffbf54, 36 },
    { 0x619fbf55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20fbf5e, 34 },
    { 0xec7fbf5f, 100 },
    { 0x8d0fbf60, 95 },
    { 0x77dfbf60, 93 },
    { 0x7fbfbf62, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c9fbf66, 93 },
    { 0, 0 },
    { 0x5c9fbf68, 95 },
    { 0, 0 },
    { 0x6a2fbf6a, 33 },
    { 0xa13fbf6a, 61 },
    { 0, 0 },
    { 0x3d4fbf6d, 17 },
    { 0xf81fbf6e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfbf71, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe62fbf74, 67 },
    { 0, 0 },
    { 0x99bfbf76, 77 },
    { 0x96ffbf77, 98 },
    { 0x8e5fbf77, 46 },
    { 0, 0 },
    { 0xa4efbf7a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6fbf7d, 93 },
    { 0xd30fbf7e, 66 },
    { 0xe4dfbf7d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fafbf83, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9fbf8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x433fbf98, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc1fbf9e, 99 },
    { 0, 0 },
    { 0xb9efbfa0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb33fbfa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf10fbfa8, 43 },
    { 0xd17fbfa9, 95 },
    { 0x77bfbfaa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7fbfb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88fbfb4, 90 },
    { 0x3e4fbfb4, 99 },
    { 0xefffbfb6, 90 },
    { 0xcc5fbfb7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89fbfbb, 98 },
    { 0xee0fbfbc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeefbfc1, 33 },
    { 0xe69fbfc2, 98 },
    { 0xd20fbfc3, 52 },
    { 0x70dfbfc1, 93 },
    { 0x693fbfc5, 81 },
    { 0x9dbfbfc6, 76 },
    { 0x720fbfc5, 90 },
    { 0x927fbfc3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x491fbfd1, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf13fbfd7, 90 },
    { 0x71cfbfd8, 61 },
    { 0x116fbfd8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe9fbfdc, 90 },
    { 0, 0 },
    { 0xf10fbfde, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8cfbfec, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefbfef, 90 },
    { 0xe46fbfef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5fbff8, 93 },
    { 0x719fbff9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x473fbffc, 98 },
    { 0x372fbffd, 66 },
    { 0xd13fbffc, 96 },
    { 0x543fbfff, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xecbfc002, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x69bfc006, 87 },
    { 0, 0 },
    { 0x240fc008, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0fc00d, 78 },
    { 0xdaefc00e, 95 },
    { 0x382fc00f, 95 },
    { 0x658fc010, 76 },
    { 0, 0 },
    { 0xd98fc012, 61 },
    { 0xf2cfc012, 67 },
    { 0x11afc012, 85 },
    { 0x9e2fc015, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x766fc01a, 55 },
    { 0x80dfc01b, 23 },
    { 0xa62fc01b, 33 },
    { 0xa12fc01d, 34 },
    { 0x632fc01e, 98 },
    { 0xa1efc01f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4cffc022, 90 },
    { 0xed8fc023, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931fc027, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3affc02a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x812fc037, 34 },
    { 0, 0 },
    { 0xddefc039, 34 },
    { 0x1e9fc03a, 33 },
    { 0x191fc03a, 23 },
    { 0xdd1fc03c, 76 },
    { 0, 0 },
    { 0x939fc03e, 46 },
    { 0x609fc03f, 85 },
    { 0, 0 },
    { 0xedcfc041, 88 },
    { 0, 0 },
    { 0xfd1fc043, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563fc04a, 33 },
    { 0x861fc04b, 33 },
    { 0x21cfc04c, 43 },
    { 0xd23fc04d, 95 },
    { 0x91ffc04d, 93 },
    { 0x800fc04d, 90 },
    { 0x6bdfc050, 98 },
    { 0xe37fc04e, 90 },
    { 0xa38fc04f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bffc056, 66 },
    { 0, 0 },
    { 0x85bfc058, 90 },
    { 0, 0 },
    { 0x5defc05a, 46 },
    { 0x462fc05b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfc05e, 67 },
    { 0x2edfc05f, 33 },
    { 0x640fc060, 91 },
    { 0x2bfc061, 90 },
    { 0xa80fc060, 90 },
    { 0xd7ffc060, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7fc066, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x960fc06a, 76 },
    { 0, 0 },
    { 0x3cfc06c, 87 },
    { 0, 0 },
    { 0xf4ffc06e, 90 },
    { 0x278fc06e, 60 },
    { 0, 0 },
    { 0, 0 },
    { 0xf56fc072, 83 },
    { 0xdebfc073, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a4fc07d, 34 },
    { 0xc4cfc07e, 61 },
    { 0xc6dfc07e, 83 },
    { 0, 0 },
    { 0xdf1fc081, 52 },
    { 0, 0 },
    { 0xd0dfc083, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe30fc087, 33 },
    { 0, 0 },
    { 0x82bfc089, 52 },
    { 0, 0 },
    { 0x574fc08b, 55 },
    { 0, 0 },
    { 0x78cfc08d, 33 },
    { 0x6c8fc08d, 33 },
    { 0, 0 },
    { 0xc66fc090, 98 },
    { 0x783fc091, 77 },
    { 0xf64fc091, 85 },
    { 0x944fc091, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a9fc09a, 33 },
    { 0, 0 },
    { 0x480fc09c, 90 },
    { 0xf9cfc09c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bfc0a1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x3effc0a4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307fc0ac, 98 },
    { 0xa44fc0ac, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x239fc0b0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xea5fc0b3, 95 },
    { 0x1e5fc0b4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9fc0b9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cfc0c2, 91 },
    { 0x946fc0c2, 61 },
    { 0xd93fc0c4, 23 },
    { 0xe58fc0c5, 98 },
    { 0x377fc0c6, 46 },
    { 0x4e2fc0c3, 90 },
    { 0xb78fc0c8, 33 },
    { 0x72efc0c8, 33 },
    { 0xa89fc0c9, 69 },
    { 0x414fc0cb, 76 },
    { 0x27bfc0c5, 23 },
    { 0xddbfc0cd, 91 },
    { 0, 0 },
    { 0xd06fc0cf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ccfc0d5, 91 },
    { 0, 0 },
    { 0xddefc0d7, 33 },
    { 0, 0 },
    { 0x3bbfc0d9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3fc0dd, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xacfc0e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acfc0e3, 84 },
    { 0x84dfc0e4, 61 },
    { 0x4adfc0e4, 23 },
    { 0x85dfc0e6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0xd27fc0e9, 34 },
    { 0xe9dfc0ea, 33 },
    { 0xbd8fc0ea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7abfc0ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4cfc0f1, 61 },
    { 0x999fc0f2, 55 },
    { 0xc00fc0f2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x951fc0f6, 77 },
    { 0x4fafc0f7, 23 },
    { 0xe6afc0f8, 23 },
    { 0, 0 },
    { 0xba5fc0fa, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efc0fd, 66 },
    { 0x86fc0fe, 61 },
    { 0x435fc0ff, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c0fc10c, 90 },
    { 0x56dfc10d, 100 },
    { 0, 0 },
    { 0x687fc10f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27efc116, 34 },
    { 0xe14fc117, 33 },
    { 0, 0 },
    { 0xa15fc119, 90 },
    { 0x9fdfc119, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dfc11f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfc125, 46 },
    { 0x489fc125, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436fc12a, 33 },
    { 0, 0 },
    { 0x53dfc12c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aafc131, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb6efc134, 61 },
    { 0xfd7fc135, 55 },
    { 0xcabfc135, 52 },
    { 0x6ffc135, 33 },
    { 0xc32fc137, 23 },
    { 0x59dfc139, 67 },
    { 0x214fc139, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x256fc13d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x919fc143, 90 },
    { 0x5ddfc143, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa19fc149, 76 },
    { 0xf57fc14a, 91 },
    { 0x3f7fc149, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a1fc14f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd2fc156, 33 },
    { 0x254fc157, 66 },
    { 0x797fc158, 33 },
    { 0x339fc159, 23 },
    { 0x4f0fc157, 61 },
    { 0xad6fc15b, 33 },
    { 0x747fc15c, 55 },
    { 0xc58fc15d, 87 },
    { 0xe90fc15d, 53 },
    { 0x2acfc15c, 23 },
    { 0x323fc160, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fc164, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3afc173, 61 },
    { 0, 0 },
    { 0x6aefc175, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf97fc179, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd37fc17f, 90 },
    { 0x9c9fc180, 95 },
    { 0xb95fc181, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe60fc185, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3dfc189, 90 },
    { 0xbc0fc18a, 78 },
    { 0x8c0fc18b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d3fc18e, 95 },
    { 0xaebfc18f, 90 },
    { 0xc59fc190, 65 },
    { 0xfb7fc18e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9afc196, 100 },
    { 0x3dbfc196, 95 },
    { 0x77dfc198, 61 },
    { 0, 0 },
    { 0xd73fc19a, 95 },
    { 0x235fc19b, 23 },
    { 0xe1cfc19b, 76 },
    { 0xf99fc19d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdafc1aa, 90 },
    { 0x664fc1ab, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa4fc1ae, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fc1b1, 62 },
    { 0x5cfc1b2, 90 },
    { 0xdbbfc1b3, 76 },
    { 0x3ecfc1b1, 98 },
    { 0x156fc1b1, 61 },
    { 0, 0 },
    { 0x93ffc1b7, 33 },
    { 0xb1cfc1b8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x544fc1bc, 95 },
    { 0x100fc1bd, 65 },
    { 0x6b7fc1bd, 61 },
    { 0, 0 },
    { 0x3f5fc1c0, 76 },
    { 0xd87fc1c1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x686fc1c9, 61 },
    { 0xd49fc1ca, 30 },
    { 0, 0 },
    { 0xc7afc1cc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0fc1cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2fc1d6, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bcfc1d9, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b6fc1dd, 23 },
    { 0x589fc1de, 61 },
    { 0x99dfc1de, 95 },
    { 0x540fc1df, 33 },
    { 0, 0 },
    { 0xbf3fc1e2, 43 },
    { 0x5a7fc1e3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x867fc1e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xafafc1f1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2fc1f9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x182fc1fc, 33 },
    { 0x929fc1fd, 34 },
    { 0x76cfc1fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89efc203, 43 },
    { 0, 0 },
    { 0x4c8fc205, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0x513fc208, 76 },
    { 0xa13fc209, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1fc20c, 90 },
    { 0, 0 },
    { 0xc52fc20e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0efc214, 61 },
    { 0xfe1fc215, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc45fc219, 46 },
    { 0x319fc21a, 90 },
    { 0xc55fc21b, 55 },
    { 0x70efc21b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc19fc222, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32fc226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc48fc229, 33 },
    { 0xaecfc22a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x57efc22d, 90 },
    { 0x687fc22e, 98 },
    { 0xa86fc22f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0fc234, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4afc23b, 75 },
    { 0, 0 },
    { 0x5b8fc23d, 27 },
    { 0x2befc23e, 33 },
    { 0xec4fc23d, 90 },
    { 0xa60fc23f, 90 },
    { 0xe77fc241, 23 },
    { 0xfcefc242, 98 },
    { 0x95cfc243, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x848fc249, 63 },
    { 0, 0 },
    { 0xd0efc24b, 46 },
    { 0x654fc24c, 45 },
    { 0xf6cfc24d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0afc257, 67 },
    { 0, 0 },
    { 0x83ffc259, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdfc25c, 85 },
    { 0x550fc25d, 90 },
    { 0, 0 },
    { 0xf05fc25f, 85 },
    { 0xc33fc25f, 61 },
    { 0xd8efc25f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54fc265, 76 },
    { 0xf9afc266, 61 },
    { 0x51cfc267, 33 },
    { 0x15dfc267, 17 },
    { 0x5ffc269, 66 },
    { 0x878fc269, 95 },
    { 0x58efc26b, 76 },
    { 0x7a5fc26c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757fc27b, 88 },
    { 0x135fc27c, 23 },
    { 0x658fc27d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ecfc283, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4ffc286, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0fc28c, 61 },
    { 0x1eefc28c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73bfc291, 77 },
    { 0, 0 },
    { 0x6d7fc293, 23 },
    { 0, 0 },
    { 0x3bfc295, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x653fc299, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa81fc2a5, 95 },
    { 0x6affc2a6, 61 },
    { 0xa4ffc2a6, 76 },
    { 0x152fc2a8, 48 },
    { 0x5b2fc2a8, 95 },
    { 0xbe0fc2aa, 95 },
    { 0xbd8fc2aa, 61 },
    { 0x55dfc2aa, 33 },
    { 0xdafc2ac, 76 },
    { 0, 0 },
    { 0x9aefc2af, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x772fc2b2, 61 },
    { 0, 0 },
    { 0xe3ffc2b4, 90 },
    { 0, 0 },
    { 0x4fafc2b6, 76 },
    { 0x552fc2b6, 90 },
    { 0x471fc2b7, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc5fc2be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x968fc2c1, 33 },
    { 0xf4afc2c1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa40fc2cc, 90 },
    { 0x57cfc2cd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ffc2d2, 95 },
    { 0xffefc2d3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fc2d8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7fc2de, 90 },
    { 0x932fc2df, 67 },
    { 0xc98fc2df, 40 },
    { 0x632fc2e0, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x396fc2e6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x623fc2e9, 98 },
    { 0xa51fc2ea, 90 },
    { 0x94ffc2e9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90dfc2ef, 95 },
    { 0xefffc2f0, 100 },
    { 0xe27fc2f0, 76 },
    { 0, 0 },
    { 0xc62fc2f3, 40 },
    { 0, 0 },
    { 0x5b6fc2f5, 33 },
    { 0, 0 },
    { 0xb96fc2f7, 33 },
    { 0xff2fc2f7, 95 },
    { 0x254fc2f8, 90 },
    { 0x1aefc2fa, 90 },
    { 0, 0 },
    { 0xb37fc2fc, 61 },
    { 0, 0 },
    { 0xf3cfc2fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27afc313, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x89cfc318, 98 },
    { 0xb7bfc318, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28dfc323, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca7fc326, 23 },
    { 0, 0 },
    { 0x6cafc328, 98 },
    { 0, 0 },
    { 0xd6cfc32a, 90 },
    { 0, 0 },
    { 0xd4dfc32c, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cfc32f, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e5fc33d, 68 },
    { 0x92fc33e, 33 },
    { 0xce6fc33e, 90 },
    { 0x32dfc340, 23 },
    { 0x2dbfc340, 33 },
    { 0, 0 },
    { 0xc6efc343, 79 },
    { 0x75fc344, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc9fc34c, 87 },
    { 0, 0 },
    { 0x5c1fc34e, 99 },
    { 0x33bfc34f, 98 },
    { 0, 0 },
    { 0xfe2fc351, 33 },
    { 0, 0 },
    { 0xc70fc353, 95 },
    { 0x536fc353, 23 },
    { 0xc45fc355, 95 },
    { 0x7b4fc354, 33 },
    { 0, 0 },
    { 0xe8cfc358, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ddfc366, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44afc36d, 46 },
    { 0x82afc36e, 61 },
    { 0xf02fc36f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4befc373, 91 },
    { 0x80dfc374, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9fc37a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2fc37e, 67 },
    { 0x2efc37f, 96 },
    { 0, 0 },
    { 0x163fc381, 23 },
    { 0, 0 },
    { 0x963fc383, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dfc386, 65 },
    { 0x6e7fc387, 90 },
    { 0x1cdfc386, 95 },
    { 0x6b6fc389, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x452fc38c, 66 },
    { 0, 0 },
    { 0xcd9fc38e, 87 },
    { 0, 0 },
    { 0x697fc390, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe54fc396, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc27fc39a, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x238fc39e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedffc3a3, 98 },
    { 0x3ecfc3a4, 78 },
    { 0, 0 },
    { 0x19fc3a6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe77fc3ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x230fc3b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x937fc3b6, 52 },
    { 0x1a0fc3b6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34ffc3bb, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdbfc3be, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x323fc3c1, 95 },
    { 0xf4ffc3c2, 98 },
    { 0x1c8fc3c1, 99 },
    { 0x99cfc3c4, 61 },
    { 0, 0 },
    { 0x3cfc3c6, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0fc3c9, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d2fc3cf, 33 },
    { 0, 0 },
    { 0xf0efc3d1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d8fc3d7, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70fc3da, 66 },
    { 0x455fc3db, 77 },
    { 0xfc2fc3dc, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aafc3e3, 95 },
    { 0, 0 },
    { 0xcebfc3e5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01fc3ea, 23 },
    { 0xd75fc3eb, 90 },
    { 0x3afc3eb, 93 },
    { 0xc69fc3ed, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dffc3f1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafc3f6, 33 },
    { 0, 0 },
    { 0x99bfc3f8, 34 },
    { 0x4a2fc3f9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x294fc3fd, 90 },
    { 0, 0 },
    { 0xacafc3ff, 78 },
    { 0xc5cfc3ff, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x685fc406, 98 },
    { 0, 0 },
    { 0x6f4fc408, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97fc413, 61 },
    { 0xc7efc414, 93 },
    { 0, 0 },
    { 0xc19fc416, 90 },
    { 0, 0 },
    { 0xca5fc418, 23 },
    { 0x8c2fc419, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x67bfc41c, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaefc424, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97fc42d, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04fc435, 23 },
    { 0xdd8fc436, 33 },
    { 0x7b6fc436, 95 },
    { 0xff4fc437, 61 },
    { 0x334fc439, 44 },
    { 0x82fc439, 61 },
    { 0xa81fc43b, 33 },
    { 0, 0 },
    { 0x6abfc43d, 90 },
    { 0x47bfc43d, 66 },
    { 0x1bafc43f, 33 },
    { 0x463fc43d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51afc44b, 52 },
    { 0xcd6fc44c, 98 },
    { 0, 0 },
    { 0x237fc44e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fc451, 43 },
    { 0, 0 },
    { 0x281fc453, 33 },
    { 0, 0 },
    { 0x389fc455, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fc45d, 55 },
    { 0xec2fc45e, 33 },
    { 0, 0 },
    { 0xe27fc460, 55 },
    { 0xe3fc461, 91 },
    { 0x5f4fc462, 93 },
    { 0x81afc463, 64 },
    { 0x5a3fc461, 61 },
    { 0x33ffc462, 90 },
    { 0x994fc466, 93 },
    { 0x22cfc466, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x955fc46a, 33 },
    { 0, 0 },
    { 0xdd6fc46c, 61 },
    { 0, 0 },
    { 0x172fc46e, 98 },
    { 0x660fc46e, 84 },
    { 0xf8efc46f, 33 },
    { 0, 0 },
    { 0x13efc472, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xba8fc475, 95 },
    { 0x4e7fc475, 95 },
    { 0, 0 },
    { 0x5c4fc478, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x817fc484, 95 },
    { 0x580fc485, 43 },
    { 0, 0 },
    { 0x1c5fc487, 33 },
    { 0xefefc487, 98 },
    { 0x297fc487, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe06fc48e, 33 },
    { 0, 0 },
    { 0xb68fc490, 90 },
    { 0xd9afc491, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c0fc495, 93 },
    { 0, 0 },
    { 0x236fc497, 90 },
    { 0x849fc497, 99 },
    { 0xca4fc499, 83 },
    { 0x9f1fc499, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed0fc49e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0ffc4a2, 90 },
    { 0, 0 },
    { 0xeb7fc4a4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x105fc4b0, 98 },
    { 0x81fc4b1, 55 },
    { 0x29bfc4b2, 66 },
    { 0x711fc4b1, 23 },
    { 0, 0 },
    { 0x99efc4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fefc4b8, 33 },
    { 0xc6efc4b8, 91 },
    { 0x9fcfc4ba, 33 },
    { 0x51dfc4bb, 61 },
    { 0x59ffc4bb, 95 },
    { 0x8ffc4bd, 55 },
    { 0x549fc4be, 61 },
    { 0x479fc4bf, 95 },
    { 0x9fafc4bd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa80fc4c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bcfc4c8, 36 },
    { 0x6c3fc4c8, 95 },
    { 0xf84fc4ca, 43 },
    { 0x1abfc4ca, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf74fc4cf, 93 },
    { 0xaedfc4cf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x160fc4d3, 60 },
    { 0, 0 },
    { 0xcfcfc4d5, 33 },
    { 0, 0 },
    { 0x5e7fc4d7, 98 },
    { 0xd09fc4d7, 90 },
    { 0x169fc4d9, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x401fc4de, 52 },
    { 0x2eefc4df, 33 },
    { 0xd7afc4df, 81 },
    { 0x1ffc4e1, 76 },
    { 0xe80fc4e2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c0fc4e7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdfc4ea, 98 },
    { 0, 0 },
    { 0x8cefc4ec, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca3fc4f0, 33 },
    { 0xc76fc4f0, 34 },
    { 0xca6fc4f0, 98 },
    { 0xf95fc4f3, 81 },
    { 0, 0 },
    { 0x7c4fc4f5, 87 },
    { 0x7f2fc4f5, 33 },
    { 0x813fc4f6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dfc4fa, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc69fc4ff, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xde2fc502, 23 },
    { 0x11ffc503, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x268fc509, 65 },
    { 0xee1fc50a, 90 },
    { 0xa88fc50b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x61afc519, 55 },
    { 0x1b3fc51a, 85 },
    { 0, 0 },
    { 0x7b1fc51c, 76 },
    { 0x874fc51d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95cfc522, 91 },
    { 0, 0 },
    { 0xab8fc524, 84 },
    { 0, 0 },
    { 0x917fc526, 99 },
    { 0, 0 },
    { 0x34ffc528, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4fc52b, 61 },
    { 0x406fc52c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x998fc52f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7cfc539, 98 },
    { 0x80efc53a, 43 },
    { 0x1e5fc53a, 76 },
    { 0xf2bfc53c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34fc540, 95 },
    { 0xd39fc540, 95 },
    { 0x47ffc542, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e2fc545, 23 },
    { 0, 0 },
    { 0x5e4fc547, 100 },
    { 0x764fc547, 33 },
    { 0xb39fc547, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a6fc54f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dffc553, 61 },
    { 0, 0 },
    { 0x187fc555, 76 },
    { 0x295fc556, 46 },
    { 0x7e0fc557, 23 },
    { 0xecfc557, 23 },
    { 0, 0 },
    { 0xc57fc55a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6fc562, 61 },
    { 0x52ffc562, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddffc569, 52 },
    { 0x528fc569, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ffc56d, 23 },
    { 0xa3ffc56d, 90 },
    { 0x6a3fc56d, 52 },
    { 0, 0 },
    { 0x684fc571, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807fc577, 95 },
    { 0xcc9fc578, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd5fc57b, 61 },
    { 0xd28fc57b, 90 },
    { 0xbe6fc57c, 76 },
    { 0, 0 },
    { 0x1d4fc57f, 98 },
    { 0x228fc57f, 61 },
    { 0, 0 },
    { 0x8e9fc582, 81 },
    { 0, 0 },
    { 0x1bafc584, 59 },
    { 0xdbefc584, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2fc588, 23 },
    { 0x235fc589, 43 },
    { 0xf2fc58a, 46 },
    { 0xb6dfc58a, 61 },
    { 0xe54fc588, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb7fc591, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x369fc594, 61 },
    { 0x7adfc595, 65 },
    { 0x9f0fc595, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7cfc599, 77 },
    { 0x6a5fc59a, 90 },
    { 0x5ecfc59b, 61 },
    { 0xd83fc59b, 98 },
    { 0x8b8fc59d, 99 },
    { 0xb84fc599, 76 },
    { 0x47bfc59c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0fc5a4, 81 },
    { 0, 0 },
    { 0x8e6fc5a6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35afc5aa, 91 },
    { 0x7cefc5aa, 76 },
    { 0xa16fc5ac, 77 },
    { 0x31dfc5ab, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1fc5b1, 34 },
    { 0xbb6fc5b2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbefc5b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0afc5b8, 61 },
    { 0x7b5fc5b9, 77 },
    { 0xdc8fc5ba, 61 },
    { 0x72bfc5bb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72dfc5c1, 60 },
    { 0, 0 },
    { 0x8a0fc5c3, 61 },
    { 0, 0 },
    { 0x128fc5c5, 62 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf5fc5cb, 95 },
    { 0xc42fc5cb, 23 },
    { 0x157fc5cd, 98 },
    { 0x7c7fc5cd, 98 },
    { 0xaa8fc5cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d4fc5d6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x94fc5da, 90 },
    { 0xb89fc5db, 100 },
    { 0, 0 },
    { 0x4c7fc5dd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e4fc5e0, 84 },
    { 0, 0 },
    { 0xf13fc5e2, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaefc5e8, 90 },
    { 0x42fc5e8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f4fc5ec, 33 },
    { 0x75ffc5ed, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7fc5f2, 88 },
    { 0xf9efc5f3, 61 },
    { 0x73afc5f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb04fc5fa, 75 },
    { 0x8d5fc5fb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd96fc600, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fc603, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x773fc607, 93 },
    { 0, 0 },
    { 0x3b0fc609, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x613fc60c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dfc60f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9fc614, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x907fc617, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x914fc61c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fc623, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf54fc627, 46 },
    { 0x376fc628, 33 },
    { 0x4ccfc629, 43 },
    { 0x4acfc62a, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bfc62d, 61 },
    { 0x5c1fc62e, 23 },
    { 0xe15fc62f, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x160fc632, 90 },
    { 0x5b2fc632, 46 },
    { 0x85ffc634, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf4fc63b, 23 },
    { 0x16afc63c, 52 },
    { 0xf69fc63c, 34 },
    { 0xdd8fc63c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc41fc642, 68 },
    { 0x5d6fc643, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfc649, 23 },
    { 0xa8bfc649, 90 },
    { 0xb93fc64b, 90 },
    { 0, 0 },
    { 0xc74fc64d, 55 },
    { 0x85ffc64e, 98 },
    { 0x777fc64f, 55 },
    { 0xc79fc650, 95 },
    { 0, 0 },
    { 0xbfbfc652, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc62fc661, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8fc666, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x590fc66a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18fc670, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bffc677, 61 },
    { 0xdcbfc678, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd22fc67b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116fc681, 84 },
    { 0x129fc681, 53 },
    { 0xb0bfc683, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x262fc686, 77 },
    { 0xd87fc687, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bfc68b, 66 },
    { 0, 0 },
    { 0xe88fc68d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x46bfc690, 61 },
    { 0x8e3fc691, 33 },
    { 0, 0 },
    { 0x7fc693, 94 },
    { 0xda7fc694, 33 },
    { 0x7c9fc695, 34 },
    { 0, 0 },
    { 0xa74fc697, 52 },
    { 0, 0 },
    { 0x323fc699, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f7fc69c, 33 },
    { 0xc5bfc69d, 90 },
    { 0xf44fc69e, 61 },
    { 0, 0 },
    { 0xc40fc6a0, 53 },
    { 0x7fdfc6a0, 98 },
    { 0x2cffc6a0, 95 },
    { 0xa61fc6a3, 43 },
    { 0x189fc6a2, 90 },
    { 0x9b1fc6a5, 61 },
    { 0xf5efc6a6, 95 },
    { 0x614fc6a5, 98 },
    { 0xcdffc6a7, 61 },
    { 0xa47fc6a5, 33 },
    { 0x9d0fc6a6, 93 },
    { 0x991fc6ab, 46 },
    { 0x61ffc6ac, 23 },
    { 0, 0 },
    { 0x8c8fc6ae, 83 },
    { 0xba4fc6af, 95 },
    { 0x15cfc6ae, 66 },
    { 0, 0 },
    { 0xd71fc6b2, 85 },
    { 0x893fc6b3, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x913fc6ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x361fc6bd, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6ffc6c2, 46 },
    { 0, 0 },
    { 0x35cfc6c4, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374fc6d1, 78 },
    { 0, 0 },
    { 0x124fc6d3, 65 },
    { 0xccfc6d4, 75 },
    { 0, 0 },
    { 0xcf9fc6d6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cffc6da, 95 },
    { 0, 0 },
    { 0x314fc6dc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecfc6e4, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x195fc6e7, 90 },
    { 0x6e7fc6e7, 61 },
    { 0xe3bfc6e9, 76 },
    { 0x99bfc6ea, 76 },
    { 0xf43fc6e8, 93 },
    { 0xc46fc6ec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee9fc6f2, 76 },
    { 0xe16fc6f2, 96 },
    { 0, 0 },
    { 0x96dfc6f5, 95 },
    { 0x123fc6f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3fc6fd, 5 },
    { 0x839fc6fe, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x834fc701, 76 },
    { 0x160fc702, 90 },
    { 0xcd3fc701, 76 },
    { 0, 0 },
    { 0x41cfc705, 99 },
    { 0xb79fc705, 76 },
    { 0, 0 },
    { 0x46bfc708, 90 },
    { 0x6fdfc709, 75 },
    { 0, 0 },
    { 0xa3cfc70b, 33 },
    { 0xc05fc70c, 76 },
    { 0xcdbfc70d, 33 },
    { 0x436fc70b, 76 },
    { 0xcbafc70f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90fc712, 78 },
    { 0, 0 },
    { 0x4a1fc714, 98 },
    { 0x975fc714, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb1fc71d, 90 },
    { 0, 0 },
    { 0xcb4fc71f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x306fc723, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efc726, 43 },
    { 0xf18fc727, 98 },
    { 0x476fc726, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc1fc72b, 61 },
    { 0, 0 },
    { 0xca4fc72d, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1fc733, 61 },
    { 0, 0 },
    { 0x96ffc735, 95 },
    { 0, 0 },
    { 0x403fc737, 43 },
    { 0x4d6fc738, 90 },
    { 0x571fc737, 95 },
    { 0x291fc73a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f2fc73d, 76 },
    { 0x68fc73e, 98 },
    { 0, 0 },
    { 0xfaffc740, 69 },
    { 0xe87fc741, 23 },
    { 0xe6fc741, 33 },
    { 0xdb4fc743, 66 },
    { 0x825fc740, 61 },
    { 0, 0 },
    { 0x40bfc746, 88 },
    { 0x194fc746, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x227fc74b, 61 },
    { 0x9cdfc74b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979fc752, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdfc759, 83 },
    { 0, 0 },
    { 0xb87fc75b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1ffc766, 66 },
    { 0x877fc767, 61 },
    { 0xbddfc767, 17 },
    { 0x328fc767, 33 },
    { 0xafffc767, 55 },
    { 0xdbdfc76b, 77 },
    { 0x97bfc76c, 61 },
    { 0x871fc76d, 90 },
    { 0xaf2fc768, 61 },
    { 0x5f4fc76f, 65 },
    { 0x1dffc76b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ccfc774, 33 },
    { 0x8c0fc775, 78 },
    { 0, 0 },
    { 0x497fc777, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x563fc77b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4fc77e, 33 },
    { 0, 0 },
    { 0x45afc780, 33 },
    { 0xf6cfc781, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x360fc787, 76 },
    { 0x91ffc787, 33 },
    { 0, 0 },
    { 0x332fc78a, 33 },
    { 0, 0 },
    { 0xe89fc78c, 90 },
    { 0xfc8fc78d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66afc795, 90 },
    { 0xae0fc796, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57fc79e, 61 },
    { 0x6efc79e, 93 },
    { 0xd42fc7a0, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8bfc7a4, 61 },
    { 0xe26fc7a5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dfc7ac, 95 },
    { 0, 0 },
    { 0x8dffc7ae, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99ffc7b2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xba5fc7b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x304fc7bd, 65 },
    { 0x5b9fc7be, 69 },
    { 0xab8fc7bf, 98 },
    { 0x71efc7c0, 90 },
    { 0x51dfc7c1, 90 },
    { 0xb16fc7c2, 61 },
    { 0xe18fc7c3, 39 },
    { 0xad1fc7c3, 84 },
    { 0x745fc7c3, 40 },
    { 0x314fc7c2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0fc7c9, 90 },
    { 0, 0 },
    { 0x779fc7cb, 43 },
    { 0xb4cfc7cc, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xa61fc7cf, 98 },
    { 0xf97fc7d0, 90 },
    { 0xd5bfc7d1, 100 },
    { 0xad8fc7d2, 23 },
    { 0xc79fc7d1, 76 },
    { 0x4fdfc7d4, 90 },
    { 0x44ffc7d5, 90 },
    { 0, 0 },
    { 0xb45fc7d7, 23 },
    { 0x123fc7d8, 81 },
    { 0, 0 },
    { 0x3edfc7da, 90 },
    { 0, 0 },
    { 0x8a5fc7dc, 90 },
    { 0x6cafc7dc, 90 },
    { 0xda9fc7de, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x764fc7e2, 61 },
    { 0, 0 },
    { 0x488fc7e4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8fc7e8, 93 },
    { 0x300fc7e8, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801fc7f2, 95 },
    { 0x5effc7f3, 67 },
    { 0xa42fc7f4, 52 },
    { 0xd9bfc7f5, 81 },
    { 0, 0 },
    { 0xad8fc7f7, 33 },
    { 0x496fc7f8, 95 },
    { 0x5f1fc7f9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e2fc7fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5fc800, 33 },
    { 0xb53fc801, 55 },
    { 0xdddfc801, 33 },
    { 0, 0 },
    { 0xac6fc804, 33 },
    { 0xe5dfc805, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x490fc80b, 100 },
    { 0x6affc80b, 61 },
    { 0x3f2fc80b, 66 },
    { 0, 0 },
    { 0x851fc80f, 23 },
    { 0x750fc810, 76 },
    { 0xf9ffc80f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dfc815, 85 },
    { 0xabffc816, 88 },
    { 0x4d5fc817, 90 },
    { 0xf74fc816, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91fc81b, 95 },
    { 0, 0 },
    { 0x71cfc81d, 64 },
    { 0x282fc81d, 90 },
    { 0, 0 },
    { 0xa46fc820, 52 },
    { 0x47cfc820, 90 },
    { 0x505fc822, 33 },
    { 0xccfc820, 55 },
    { 0x8ebfc824, 33 },
    { 0, 0 },
    { 0xc71fc826, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bbfc82a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x787fc831, 76 },
    { 0xe86fc832, 33 },
    { 0x2abfc833, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f4fc836, 90 },
    { 0, 0 },
    { 0xa1afc838, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402fc849, 61 },
    { 0, 0 },
    { 0x22cfc84b, 33 },
    { 0, 0 },
    { 0x984fc84d, 40 },
    { 0, 0 },
    { 0xa7efc84f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacffc859, 76 },
    { 0xe84fc85a, 90 },
    { 0xfb2fc859, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45fc866, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28fc869, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bfc86d, 61 },
    { 0, 0 },
    { 0xd76fc86f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f5fc872, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa0cfc87b, 64 },
    { 0, 0 },
    { 0x64afc87d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x100fc880, 90 },
    { 0x24efc881, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3fc884, 99 },
    { 0x346fc884, 68 },
    { 0x9fdfc886, 59 },
    { 0x9cdfc884, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x41fc88a, 95 },
    { 0xc42fc88b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe3fc890, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x10efc89d, 98 },
    { 0x93efc89d, 33 },
    { 0, 0 },
    { 0xe92fc8a0, 93 },
    { 0xe2bfc8a1, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05fc8a4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9fc8a7, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65fc8b5, 78 },
    { 0x376fc8b6, 55 },
    { 0x96dfc8b7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x986fc8bd, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b3fc8c7, 33 },
    { 0, 0 },
    { 0x7fc8c9, 90 },
    { 0x249fc8ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x48efc8d0, 90 },
    { 0x81dfc8d0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cafc8d5, 66 },
    { 0, 0 },
    { 0x806fc8d7, 87 },
    { 0, 0 },
    { 0xb5efc8d9, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a4fc8e2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x734fc8eb, 67 },
    { 0, 0 },
    { 0xdaefc8ed, 33 },
    { 0, 0 },
    { 0x9f2fc8ef, 77 },
    { 0xf07fc8f0, 61 },
    { 0x1a4fc8f1, 33 },
    { 0x514fc8ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bfc8f7, 80 },
    { 0x26bfc8f8, 90 },
    { 0xc28fc8f9, 55 },
    { 0, 0 },
    { 0xe85fc8fb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x20afc8fe, 23 },
    { 0xca9fc8ff, 95 },
    { 0, 0 },
    { 0x9d5fc901, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x421fc908, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x834fc90c, 98 },
    { 0xb0efc90d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c0fc911, 90 },
    { 0x3a6fc912, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7fc918, 33 },
    { 0, 0 },
    { 0x54fc91a, 100 },
    { 0, 0 },
    { 0x189fc91c, 75 },
    { 0, 0 },
    { 0x8f0fc91e, 93 },
    { 0, 0 },
    { 0x876fc920, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xed7fc923, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0x89bfc926, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76efc92c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fffc930, 69 },
    { 0xba0fc931, 65 },
    { 0x713fc931, 33 },
    { 0x694fc933, 69 },
    { 0x761fc933, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdafc937, 84 },
    { 0x9d7fc937, 61 },
    { 0, 0 },
    { 0x2a6fc93a, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcecfc93e, 77 },
    { 0xe4afc93f, 76 },
    { 0xe59fc940, 98 },
    { 0x3ccfc941, 33 },
    { 0x784fc940, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3fc945, 61 },
    { 0x695fc945, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa12fc949, 88 },
    { 0x678fc94a, 98 },
    { 0xf56fc94a, 84 },
    { 0, 0 },
    { 0x817fc94d, 85 },
    { 0x558fc94e, 90 },
    { 0x120fc94d, 95 },
    { 0x65bfc94d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cdfc955, 95 },
    { 0, 0 },
    { 0x462fc957, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x654fc95d, 61 },
    { 0x949fc95e, 23 },
    { 0xea1fc95e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a1fc964, 55 },
    { 0x489fc965, 43 },
    { 0xcf2fc964, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3cfc96c, 36 },
    { 0xc06fc96d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319fc971, 33 },
    { 0, 0 },
    { 0xa90fc973, 76 },
    { 0x3d7fc973, 95 },
    { 0x5bffc973, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fafc978, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x286fc97e, 33 },
    { 0x25fc97f, 97 },
    { 0x40efc97e, 34 },
    { 0xe16fc981, 85 },
    { 0xc80fc981, 60 },
    { 0x3befc983, 95 },
    { 0, 0 },
    { 0xaf1fc985, 90 },
    { 0xf74fc985, 95 },
    { 0xa01fc987, 98 },
    { 0x28cfc986, 98 },
    { 0xf3afc985, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4fc991, 33 },
    { 0x553fc991, 43 },
    { 0, 0 },
    { 0xafafc994, 33 },
    { 0x367fc994, 75 },
    { 0xac2fc995, 17 },
    { 0xfb5fc997, 93 },
    { 0x561fc998, 84 },
    { 0xe2fc999, 33 },
    { 0x519fc994, 90 },
    { 0x1eafc994, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x19afc99e, 23 },
    { 0, 0 },
    { 0x65dfc9a0, 46 },
    { 0x610fc9a0, 55 },
    { 0xe98fc9a0, 98 },
    { 0, 0 },
    { 0x98efc9a4, 76 },
    { 0xa19fc9a5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc79fc9ab, 95 },
    { 0x17dfc9ac, 99 },
    { 0x860fc9ad, 23 },
    { 0, 0 },
    { 0x29dfc9af, 76 },
    { 0xd0afc9b0, 77 },
    { 0x852fc9b1, 61 },
    { 0x917fc9b0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x479fc9b5, 90 },
    { 0, 0 },
    { 0xd6afc9b7, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x735fc9ba, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3ffc9bd, 90 },
    { 0, 0 },
    { 0x198fc9bf, 34 },
    { 0, 0 },
    { 0x5fafc9c1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8cfc9c5, 68 },
    { 0x95afc9c5, 93 },
    { 0, 0 },
    { 0x827fc9c8, 43 },
    { 0x137fc9c9, 55 },
    { 0x276fc9ca, 61 },
    { 0x29bfc9c9, 43 },
    { 0xbaefc9cc, 65 },
    { 0xa3dfc9ca, 34 },
    { 0xffc9ce, 12 },
    { 0xec5fc9c9, 98 },
    { 0xcadfc9d0, 53 },
    { 0, 0 },
    { 0xfe4fc9d2, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb25fc9d8, 95 },
    { 0x10cfc9d9, 52 },
    { 0xb90fc9d9, 46 },
    { 0, 0 },
    { 0x2ccfc9dc, 23 },
    { 0x2f7fc9dc, 95 },
    { 0, 0 },
    { 0xfcdfc9df, 43 },
    { 0, 0 },
    { 0x634fc9e1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b3fc9e4, 23 },
    { 0xeb2fc9e4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x942fc9f0, 76 },
    { 0x7befc9f1, 90 },
    { 0, 0 },
    { 0x12afc9f3, 93 },
    { 0x55fc9f3, 76 },
    { 0, 0 },
    { 0xe4efc9f6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x801fc9fb, 90 },
    { 0, 0 },
    { 0x5c6fc9fd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x644fca00, 33 },
    { 0x563fca01, 87 },
    { 0xfcafca02, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7fca08, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61fca0b, 61 },
    { 0, 0 },
    { 0x41efca0d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x70ffca15, 76 },
    { 0x401fca16, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x127fca1b, 95 },
    { 0, 0 },
    { 0xd9ffca1d, 97 },
    { 0xebfca1d, 76 },
    { 0x27efca1f, 90 },
    { 0xf76fca1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811fca28, 90 },
    { 0x8cffca28, 43 },
    { 0, 0 },
    { 0x56bfca2b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9fca30, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb7bfca35, 90 },
    { 0, 0 },
    { 0x53dfca37, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc22fca3a, 33 },
    { 0xc83fca3a, 68 },
    { 0xf70fca3b, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79afca40, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1afca46, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d3fca4f, 34 },
    { 0, 0 },
    { 0x135fca51, 91 },
    { 0xa08fca52, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1fca56, 46 },
    { 0x3aefca56, 76 },
    { 0, 0 },
    { 0x63afca59, 55 },
    { 0, 0 },
    { 0xd53fca5b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf9fca5e, 46 },
    { 0xdd6fca5f, 91 },
    { 0x190fca5f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5fca63, 43 },
    { 0, 0 },
    { 0xe19fca65, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f2fca6a, 61 },
    { 0x2b5fca6b, 55 },
    { 0xb3ffca6b, 85 },
    { 0, 0 },
    { 0xf36fca6e, 93 },
    { 0, 0 },
    { 0x4bffca70, 23 },
    { 0xba3fca71, 61 },
    { 0xb00fca72, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2fca76, 90 },
    { 0x44dfca77, 90 },
    { 0x6a3fca78, 43 },
    { 0xe55fca79, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dfca80, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x586fca83, 91 },
    { 0x92ffca83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ffca89, 31 },
    { 0, 0 },
    { 0xf7fca8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b1fca91, 33 },
    { 0xa6afca91, 98 },
    { 0, 0 },
    { 0xb8fca94, 98 },
    { 0, 0 },
    { 0xf53fca96, 33 },
    { 0x91ffca97, 95 },
    { 0x5c0fca97, 55 },
    { 0x4afca99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa70fca9f, 84 },
    { 0xa1ffcaa0, 76 },
    { 0x5f5fcaa1, 33 },
    { 0, 0 },
    { 0x483fcaa3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81bfcaac, 40 },
    { 0, 0 },
    { 0x36cfcaae, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bffcab1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x74bfcab4, 100 },
    { 0, 0 },
    { 0xa54fcab6, 93 },
    { 0x61bfcab7, 76 },
    { 0xd8fcab8, 55 },
    { 0x283fcab7, 98 },
    { 0xc50fcaba, 93 },
    { 0xdcafcabb, 51 },
    { 0, 0 },
    { 0xe6bfcabd, 76 },
    { 0, 0 },
    { 0x565fcabf, 46 },
    { 0x3f5fcabf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ecfcac5, 61 },
    { 0x678fcac6, 98 },
    { 0x586fcac6, 55 },
    { 0xd77fcac7, 33 },
    { 0xc54fcac8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78cfcacd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7ffcad3, 33 },
    { 0, 0 },
    { 0x1c1fcad5, 46 },
    { 0xcfcad6, 95 },
    { 0, 0 },
    { 0xb99fcad8, 33 },
    { 0x91ffcad8, 90 },
    { 0x38fcada, 33 },
    { 0xbcefcad9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x111fcae1, 77 },
    { 0xcb0fcae1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd63fcae6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66bfcaec, 90 },
    { 0x613fcaec, 90 },
    { 0xa40fcaee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cfcaf2, 23 },
    { 0, 0 },
    { 0xa9fcaf4, 76 },
    { 0, 0 },
    { 0x29fcaf6, 88 },
    { 0xd57fcaf6, 33 },
    { 0, 0 },
    { 0x14afcaf9, 69 },
    { 0x956fcafa, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x713fcb00, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7fcb07, 90 },
    { 0x917fcb08, 61 },
    { 0x803fcb09, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2fcb0c, 98 },
    { 0xdecfcb0d, 33 },
    { 0xafffcb0d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33ffcb12, 98 },
    { 0x294fcb13, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865fcb18, 96 },
    { 0x75efcb19, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a2fcb1c, 76 },
    { 0x6bfcb1d, 95 },
    { 0x786fcb1d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x537fcb24, 90 },
    { 0xbbafcb25, 33 },
    { 0x124fcb26, 53 },
    { 0x84dfcb26, 69 },
    { 0x34bfcb27, 90 },
    { 0xbf5fcb27, 90 },
    { 0xdabfcb26, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0fcb2f, 46 },
    { 0xd9efcb30, 33 },
    { 0, 0 },
    { 0xadffcb32, 93 },
    { 0xdb4fcb33, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x347fcb36, 90 },
    { 0x7a6fcb37, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa31fcb3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fefcb43, 61 },
    { 0, 0 },
    { 0x666fcb45, 66 },
    { 0xe74fcb45, 99 },
    { 0x7dfcb47, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61fcb4a, 24 },
    { 0, 0 },
    { 0x191fcb4c, 90 },
    { 0, 0 },
    { 0xd9dfcb4e, 95 },
    { 0, 0 },
    { 0xc1fcb50, 52 },
    { 0, 0 },
    { 0xb3fcb52, 34 },
    { 0xf2afcb53, 61 },
    { 0xe53fcb54, 95 },
    { 0x1a5fcb55, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde0fcb5a, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16efcb5e, 100 },
    { 0, 0 },
    { 0xbbffcb60, 23 },
    { 0, 0 },
    { 0x80ffcb62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b5fcb6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2fcb72, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb53fcb75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8cefcb7a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x49dfcb7f, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0fcb82, 90 },
    { 0x6a7fcb82, 46 },
    { 0x61cfcb84, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x258fcb88, 33 },
    { 0x5e6fcb88, 84 },
    { 0x6ccfcb8a, 99 },
    { 0xc4ffcb8b, 90 },
    { 0, 0 },
    { 0x4a9fcb8d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181fcb95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x124fcb98, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xd41fcb9b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x91dfcb9e, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0x87bfcba1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bafcba5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa45fcbaa, 76 },
    { 0xbdffcbab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22fcbaf, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacffcbb3, 61 },
    { 0, 0 },
    { 0x1fafcbb5, 84 },
    { 0x496fcbb6, 86 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cfcbb9, 65 },
    { 0, 0 },
    { 0x7d6fcbbb, 23 },
    { 0xe43fcbbc, 48 },
    { 0x58afcbbd, 90 },
    { 0xb96fcbbe, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x55fcbc1, 55 },
    { 0x767fcbc2, 23 },
    { 0, 0 },
    { 0x2ebfcbc4, 95 },
    { 0x7c1fcbc5, 33 },
    { 0xcafcbc6, 33 },
    { 0x4e0fcbc7, 95 },
    { 0x12bfcbc6, 84 },
    { 0, 0 },
    { 0xdc6fcbca, 55 },
    { 0xfddfcbca, 95 },
    { 0, 0 },
    { 0xcfffcbcd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0xa66fcbd0, 90 },
    { 0xa0efcbd1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x179fcbd7, 23 },
    { 0, 0 },
    { 0xf57fcbd9, 77 },
    { 0x59dfcbd9, 99 },
    { 0x79cfcbd9, 55 },
    { 0xd0dfcbdb, 67 },
    { 0x329fcbda, 90 },
    { 0, 0 },
    { 0xc8afcbdf, 23 },
    { 0x1b4fcbdf, 52 },
    { 0, 0 },
    { 0xccdfcbe2, 90 },
    { 0, 0 },
    { 0x558fcbe4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb54fcbeb, 55 },
    { 0x1eefcbeb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc01fcbef, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ffcbf2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeefcbf6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb42fcc00, 24 },
    { 0, 0 },
    { 0xd42fcc02, 17 },
    { 0xc6cfcc02, 98 },
    { 0x4e9fcc04, 38 },
    { 0x7a7fcc03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bfcc0a, 98 },
    { 0x842fcc0b, 90 },
    { 0xe76fcc0c, 90 },
    { 0xee5fcc0b, 98 },
    { 0xb69fcc0e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x949fcc13, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcdfcc17, 23 },
    { 0, 0 },
    { 0xe67fcc19, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa56fcc1c, 56 },
    { 0xd8afcc1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bfcc24, 61 },
    { 0xbe0fcc24, 76 },
    { 0xa8fcc26, 52 },
    { 0, 0 },
    { 0x352fcc28, 77 },
    { 0x92fcc29, 95 },
    { 0x8dcfcc29, 33 },
    { 0xe6afcc2b, 61 },
    { 0x1d0fcc2c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c9fcc2f, 78 },
    { 0, 0 },
    { 0x9fcc31, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde6fcc35, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dfcc3a, 33 },
    { 0x754fcc3b, 33 },
    { 0x177fcc3c, 88 },
    { 0x7b6fcc3c, 76 },
    { 0xb3efcc3e, 61 },
    { 0xc9efcc3b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4fcc42, 93 },
    { 0x5defcc43, 77 },
    { 0xe82fcc42, 61 },
    { 0x97cfcc43, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f6fcc4a, 76 },
    { 0, 0 },
    { 0x7f8fcc4c, 43 },
    { 0xe30fcc4d, 95 },
    { 0xa2afcc4e, 78 },
    { 0xbc9fcc4e, 98 },
    { 0x703fcc4c, 95 },
    { 0, 0 },
    { 0x388fcc52, 85 },
    { 0xe2bfcc53, 91 },
    { 0xbffcc54, 98 },
    { 0xb74fcc55, 90 },
    { 0, 0 },
    { 0xdb5fcc57, 90 },
    { 0x951fcc58, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x870fcc5d, 78 },
    { 0xc96fcc5d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x906fcc63, 90 },
    { 0xbdbfcc64, 46 },
    { 0, 0 },
    { 0xe81fcc66, 90 },
    { 0xee1fcc67, 76 },
    { 0x653fcc68, 46 },
    { 0x4ecfcc66, 87 },
    { 0xef6fcc6a, 87 },
    { 0xde4fcc6b, 98 },
    { 0x180fcc6b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4efcc72, 68 },
    { 0, 0 },
    { 0x909fcc74, 23 },
    { 0x362fcc75, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1efcc7d, 61 },
    { 0, 0 },
    { 0x5e9fcc7f, 90 },
    { 0x5aafcc7f, 34 },
    { 0xf3fcc81, 46 },
    { 0x5f0fcc80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44fcc8c, 64 },
    { 0, 0 },
    { 0x758fcc8e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59dfcc94, 33 },
    { 0xd50fcc94, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4efcc99, 23 },
    { 0xe1afcc9a, 61 },
    { 0xaefcc9b, 66 },
    { 0xf7cfcc9b, 61 },
    { 0, 0 },
    { 0x834fcc9e, 52 },
    { 0, 0 },
    { 0x6cdfcca0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a8fcca4, 90 },
    { 0x87dfcca5, 46 },
    { 0, 0 },
    { 0x7bdfcca7, 90 },
    { 0, 0 },
    { 0x4fafcca9, 34 },
    { 0, 0 },
    { 0x7eafccab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e6fccae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fccb5, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x281fccb9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bfccc3, 67 },
    { 0x737fccc4, 95 },
    { 0x573fccc4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x1abfccc8, 33 },
    { 0, 0 },
    { 0x9f3fccca, 33 },
    { 0x283fcccb, 93 },
    { 0x9a2fcccc, 33 },
    { 0x29dfcccb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8ffccd1, 34 },
    { 0, 0 },
    { 0x22dfccd3, 76 },
    { 0, 0 },
    { 0x726fccd5, 98 },
    { 0x2f8fccd6, 43 },
    { 0xbf5fccd6, 85 },
    { 0xde5fccd8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x939fccdd, 33 },
    { 0, 0 },
    { 0xe59fccdf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59fcce2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd01fcce5, 98 },
    { 0x851fcce6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc67fcceb, 23 },
    { 0, 0 },
    { 0xa41fcced, 85 },
    { 0x658fccee, 46 },
    { 0xf45fccee, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dafccf5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa23fccfa, 33 },
    { 0x193fccfb, 99 },
    { 0x7c8fccfc, 61 },
    { 0xccdfccfc, 78 },
    { 0xf97fccfe, 46 },
    { 0xcebfccff, 23 },
    { 0x9c0fcd00, 33 },
    { 0xe46fcd01, 34 },
    { 0, 0 },
    { 0x4e0fcd03, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xd03fcd06, 87 },
    { 0x4cbfcd07, 61 },
    { 0xf5afcd08, 23 },
    { 0, 0 },
    { 0xd0bfcd0a, 95 },
    { 0x160fcd0b, 59 },
    { 0x85ffcd0a, 67 },
    { 0x5a4fcd0d, 43 },
    { 0x1e2fcd0c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x850fcd11, 95 },
    { 0x2f2fcd12, 61 },
    { 0xa11fcd11, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56fcd17, 33 },
    { 0x863fcd18, 85 },
    { 0, 0 },
    { 0x90bfcd1a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542fcd21, 85 },
    { 0, 0 },
    { 0xe0efcd23, 98 },
    { 0x4a1fcd24, 34 },
    { 0, 0 },
    { 0xa98fcd26, 98 },
    { 0x93efcd26, 98 },
    { 0xe7fcd26, 43 },
    { 0xde1fcd29, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afcd32, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b3fcd36, 95 },
    { 0xf01fcd37, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec4fcd3b, 66 },
    { 0x949fcd3c, 95 },
    { 0x322fcd3b, 90 },
    { 0x4b2fcd3e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39fcd42, 99 },
    { 0x4f1fcd43, 93 },
    { 0xca7fcd44, 98 },
    { 0x145fcd45, 76 },
    { 0xd0bfcd44, 55 },
    { 0x495fcd47, 95 },
    { 0xd4dfcd47, 33 },
    { 0x238fcd47, 95 },
    { 0x749fcd4a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb6fcd54, 76 },
    { 0xcfafcd54, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa04fcd5a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50dfcd5d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8afcd61, 90 },
    { 0x5b1fcd61, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x30fcd6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bfcd6f, 33 },
    { 0x1b8fcd6f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fcd75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cfcd7c, 90 },
    { 0, 0 },
    { 0xee5fcd7e, 76 },
    { 0x419fcd7f, 61 },
    { 0, 0 },
    { 0x19efcd81, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6fcd88, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x784fcd8c, 33 },
    { 0, 0 },
    { 0x76cfcd8e, 100 },
    { 0, 0 },
    { 0xc1dfcd90, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x278fcd95, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc7fcd9b, 33 },
    { 0x4fdfcd9c, 78 },
    { 0x6a7fcd9d, 51 },
    { 0x651fcd9e, 100 },
    { 0xd8efcd9f, 95 },
    { 0x9efcd9b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x824fcda3, 33 },
    { 0, 0 },
    { 0xb17fcda5, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8fcdaa, 90 },
    { 0x341fcdab, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80fcdb3, 90 },
    { 0, 0 },
    { 0xf2afcdb5, 94 },
    { 0x853fcdb6, 76 },
    { 0, 0 },
    { 0xa2cfcdb8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fcdbb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad4fcdbf, 80 },
    { 0xacefcdbf, 90 },
    { 0, 0 },
    { 0x838fcdc2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0efcdce, 33 },
    { 0, 0 },
    { 0x5e6fcdd0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd61fcdd4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x93afcdd8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x487fcddc, 90 },
    { 0, 0 },
    { 0xbbdfcdde, 43 },
    { 0xbc4fcddf, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbfcde4, 93 },
    { 0, 0 },
    { 0x52efcde6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41bfcdea, 33 },
    { 0, 0 },
    { 0x866fcdec, 23 },
    { 0x285fcded, 33 },
    { 0x44dfcdec, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3fcdf5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5fcdf8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f5fcdfd, 90 },
    { 0, 0 },
    { 0x8b3fcdff, 98 },
    { 0x106fce00, 55 },
    { 0, 0 },
    { 0x969fce02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x190fce07, 93 },
    { 0x32bfce08, 95 },
    { 0, 0 },
    { 0x461fce0a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c7fce0d, 90 },
    { 0, 0 },
    { 0xa38fce0f, 76 },
    { 0x5e6fce10, 75 },
    { 0x9c5fce11, 55 },
    { 0, 0 },
    { 0x7fcfce13, 34 },
    { 0xf65fce14, 94 },
    { 0, 0 },
    { 0x57efce16, 90 },
    { 0x18efce17, 14 },
    { 0xe1ffce18, 66 },
    { 0x8e2fce19, 90 },
    { 0, 0 },
    { 0x487fce1b, 66 },
    { 0xd9afce1b, 90 },
    { 0xff1fce1d, 72 },
    { 0, 0 },
    { 0x247fce1f, 61 },
    { 0xe05fce1f, 34 },
    { 0x71efce20, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa34fce26, 23 },
    { 0x3adfce26, 91 },
    { 0xb15fce28, 87 },
    { 0xb60fce29, 23 },
    { 0x17afce2a, 95 },
    { 0xcadfce2b, 90 },
    { 0x882fce2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x979fce2f, 95 },
    { 0x421fce30, 23 },
    { 0x7adfce31, 95 },
    { 0x2d8fce31, 61 },
    { 0x34cfce33, 33 },
    { 0xc6efce34, 33 },
    { 0xc7bfce35, 100 },
    { 0x628fce36, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1dafce3c, 33 },
    { 0x85fce3d, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23afce41, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xf0cfce44, 23 },
    { 0x522fce45, 98 },
    { 0x3a1fce44, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf7fce4a, 80 },
    { 0x24dfce4b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x46cfce4e, 61 },
    { 0x3fffce4f, 52 },
    { 0, 0 },
    { 0x592fce51, 76 },
    { 0xd97fce51, 65 },
    { 0xc9dfce51, 61 },
    { 0xa6ffce54, 87 },
    { 0xa8ffce53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00fce59, 99 },
    { 0x843fce5a, 90 },
    { 0x403fce5b, 61 },
    { 0xe68fce5c, 91 },
    { 0xb9dfce5b, 83 },
    { 0xc2afce5e, 85 },
    { 0x863fce5e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b8fce64, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2fce67, 61 },
    { 0x7f2fce68, 33 },
    { 0x4f5fce69, 90 },
    { 0x8effce68, 87 },
    { 0xd65fce6b, 81 },
    { 0x8cafce6a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1ccfce70, 76 },
    { 0x48bfce71, 61 },
    { 0, 0 },
    { 0x6befce73, 33 },
    { 0, 0 },
    { 0xceffce75, 55 },
    { 0, 0 },
    { 0xc98fce77, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbfce7a, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x77cfce7e, 99 },
    { 0x45dfce7f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5fce85, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc6fce88, 46 },
    { 0, 0 },
    { 0x7a7fce8a, 68 },
    { 0xeaefce8b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x979fce8f, 10 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57dfce97, 90 },
    { 0, 0 },
    { 0x6cdfce99, 95 },
    { 0x8bffce9a, 90 },
    { 0xf0dfce9a, 61 },
    { 0, 0 },
    { 0x109fce9d, 33 },
    { 0x485fce9d, 33 },
    { 0xa87fce9f, 23 },
    { 0, 0 },
    { 0xb90fcea1, 61 },
    { 0xc89fcea2, 43 },
    { 0xe74fcea3, 40 },
    { 0xa46fcea4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1bfceac, 78 },
    { 0, 0 },
    { 0xbdefceae, 55 },
    { 0, 0 },
    { 0x6f9fceb0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16fcec2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce6fcec6, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc89fcecc, 23 },
    { 0xe51fcecd, 52 },
    { 0, 0 },
    { 0xfebfcecf, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcfced3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a6fced8, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d2fcede, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x410fcee2, 85 },
    { 0xb10fcee2, 33 },
    { 0x674fcee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa18fcee7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf58fcef3, 60 },
    { 0, 0 },
    { 0xb4dfcef5, 81 },
    { 0xc59fcef6, 61 },
    { 0, 0 },
    { 0x8cefcef8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9efcefe, 19 },
    { 0, 0 },
    { 0x6c5fcf00, 95 },
    { 0, 0 },
    { 0xc5ffcf02, 76 },
    { 0xc98fcf02, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbfcf07, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x496fcf0a, 23 },
    { 0x77afcf0b, 91 },
    { 0xd6ffcf0c, 65 },
    { 0x3cfcf0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40cfcf15, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fcf18, 90 },
    { 0xcf4fcf19, 90 },
    { 0xce5fcf1a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4fcf28, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x99bfcf31, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xd56fcf34, 65 },
    { 0, 0 },
    { 0xaa0fcf36, 61 },
    { 0x3afcf36, 90 },
    { 0xa3ffcf37, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfcf3e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a9fcf42, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a3fcf45, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bfcf4c, 87 },
    { 0xfb6fcf4d, 40 },
    { 0x7d8fcf4d, 55 },
    { 0, 0 },
    { 0xb4efcf50, 88 },
    { 0, 0 },
    { 0xdcefcf52, 81 },
    { 0, 0 },
    { 0x6b0fcf54, 33 },
    { 0, 0 },
    { 0xf76fcf56, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe32fcf5a, 19 },
    { 0, 0 },
    { 0, 0 },
    { 0xe33fcf5d, 55 },
    { 0, 0 },
    { 0xa99fcf5f, 33 },
    { 0xa9dfcf60, 91 },
    { 0x826fcf5f, 95 },
    { 0x121fcf62, 29 },
    { 0x903fcf62, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x49cfcf66, 56 },
    { 0, 0 },
    { 0x3e4fcf68, 98 },
    { 0x619fcf69, 33 },
    { 0, 0 },
    { 0x2cefcf6b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95ffcf70, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebafcf74, 88 },
    { 0xd59fcf75, 61 },
    { 0xd83fcf76, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0x181fcf79, 90 },
    { 0x59cfcf79, 100 },
    { 0xf2dfcf7a, 95 },
    { 0x876fcf7c, 33 },
    { 0x616fcf7a, 93 },
    { 0, 0 },
    { 0x6c4fcf7f, 84 },
    { 0, 0 },
    { 0xe82fcf81, 85 },
    { 0xf2afcf81, 40 },
    { 0xb0bfcf83, 33 },
    { 0xac2fcf83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc92fcf8a, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558fcf92, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6fcf95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbffcf98, 101 },
    { 0xeacfcf99, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x828fcf9c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba7fcfa1, 23 },
    { 0xd42fcfa2, 68 },
    { 0, 0 },
    { 0x9f0fcfa4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfefcfa7, 88 },
    { 0x2a1fcfa8, 95 },
    { 0x107fcfa9, 61 },
    { 0x7effcfaa, 77 },
    { 0xc7cfcfa7, 33 },
    { 0x202fcfa7, 90 },
    { 0xa92fcfad, 61 },
    { 0x1f8fcfae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8bbfcfb3, 61 },
    { 0xfa2fcfb3, 76 },
    { 0xd00fcfb5, 90 },
    { 0x76fcfb6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88cfcfbc, 52 },
    { 0, 0 },
    { 0xf7fcfbe, 90 },
    { 0xe54fcfbf, 90 },
    { 0x70efcfbf, 90 },
    { 0x90dfcfbf, 43 },
    { 0xf87fcfbe, 98 },
    { 0xe8efcfc3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5adfcfc9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x213fcfcd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9fcfd2, 47 },
    { 0, 0 },
    { 0x575fcfd4, 55 },
    { 0x8cbfcfd5, 95 },
    { 0x6f3fcfd6, 90 },
    { 0x4a1fcfd6, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cafcfdf, 76 },
    { 0x89bfcfe0, 33 },
    { 0, 0 },
    { 0xe8afcfe2, 23 },
    { 0x73afcfe3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd1fcfe9, 61 },
    { 0xe99fcfea, 95 },
    { 0, 0 },
    { 0xa3ffcfec, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cfcfef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7fcff2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x432fcff5, 33 },
    { 0, 0 },
    { 0xab3fcff7, 61 },
    { 0, 0 },
    { 0x121fcff9, 33 },
    { 0xb4fcffa, 98 },
    { 0xb02fcffb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf11fd001, 46 },
    { 0x60ffd002, 98 },
    { 0, 0 },
    { 0x283fd004, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdffd007, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x369fd00c, 85 },
    { 0x628fd00d, 91 },
    { 0xe02fd00e, 33 },
    { 0xe81fd00f, 77 },
    { 0xe95fd010, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2afd014, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xefffd017, 70 },
    { 0x4dbfd018, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3bfd01b, 61 },
    { 0, 0 },
    { 0xa31fd01d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x87dfd020, 61 },
    { 0xe47fd021, 93 },
    { 0xbd1fd022, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x9eafd025, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd86fd031, 61 },
    { 0, 0 },
    { 0x88bfd033, 38 },
    { 0, 0 },
    { 0x9a1fd035, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464fd03a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x546fd03d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68afd041, 38 },
    { 0, 0 },
    { 0xf74fd043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cfd048, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd82fd04f, 93 },
    { 0x82efd04f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfd055, 55 },
    { 0xc2dfd055, 91 },
    { 0x460fd057, 77 },
    { 0x41afd055, 55 },
    { 0, 0 },
    { 0xa50fd05a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dfd05d, 52 },
    { 0, 0 },
    { 0x965fd05f, 95 },
    { 0xc0afd060, 66 },
    { 0xe6afd061, 23 },
    { 0x8cbfd060, 93 },
    { 0xacefd063, 23 },
    { 0x11fd05f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a1fd068, 80 },
    { 0xe91fd068, 87 },
    { 0x28cfd06a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12dfd072, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x328fd075, 33 },
    { 0, 0 },
    { 0x9aefd077, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3cbfd07f, 55 },
    { 0xd0afd080, 90 },
    { 0xffefd081, 78 },
    { 0xd4efd082, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x34efd085, 33 },
    { 0x927fd086, 76 },
    { 0x958fd087, 98 },
    { 0x9eefd085, 88 },
    { 0x5ddfd089, 46 },
    { 0x421fd08a, 97 },
    { 0xb37fd085, 76 },
    { 0xf07fd087, 95 },
    { 0, 0 },
    { 0x7b3fd08e, 23 },
    { 0xe3afd08f, 76 },
    { 0x87afd090, 61 },
    { 0xccfd08f, 55 },
    { 0, 0 },
    { 0x181fd093, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0fd097, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3eefd09c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aefd0b1, 34 },
    { 0, 0 },
    { 0x3d4fd0b3, 98 },
    { 0xe19fd0b4, 33 },
    { 0xe83fd0b5, 93 },
    { 0xffdfd0b4, 52 },
    { 0x7ccfd0b6, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bfd0bd, 91 },
    { 0, 0 },
    { 0x7dafd0bf, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x715fd0c3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0efd0ca, 90 },
    { 0, 0 },
    { 0x219fd0cc, 59 },
    { 0, 0 },
    { 0x2a0fd0ce, 90 },
    { 0xe84fd0ce, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4bfd0d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bafd0da, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa49fd0e1, 34 },
    { 0, 0 },
    { 0x928fd0e3, 98 },
    { 0, 0 },
    { 0x585fd0e5, 90 },
    { 0, 0 },
    { 0xb90fd0e7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f7fd0eb, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x774fd0ef, 23 },
    { 0, 0 },
    { 0x7bcfd0f1, 34 },
    { 0x4a2fd0f2, 98 },
    { 0xf4cfd0f2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fcfd0f7, 33 },
    { 0x309fd0f8, 98 },
    { 0xcafd0f9, 33 },
    { 0, 0 },
    { 0xe2dfd0fb, 36 },
    { 0xc17fd0fc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fdfd105, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fd10e, 82 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd7fd113, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x586fd116, 93 },
    { 0, 0 },
    { 0x24afd118, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67afd11c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0fd122, 61 },
    { 0x678fd122, 95 },
    { 0xd1afd124, 95 },
    { 0x2c5fd125, 68 },
    { 0, 0 },
    { 0x1f1fd127, 98 },
    { 0, 0 },
    { 0x53dfd129, 90 },
    { 0, 0 },
    { 0x963fd12b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f9fd132, 66 },
    { 0x178fd132, 17 },
    { 0x735fd134, 95 },
    { 0xeaefd135, 90 },
    { 0, 0 },
    { 0x626fd137, 55 },
    { 0x9d6fd137, 61 },
    { 0xdf2fd138, 91 },
    { 0xa7ffd13a, 98 },
    { 0xf6bfd13b, 88 },
    { 0xa0fd13b, 85 },
    { 0x2f5fd13a, 52 },
    { 0x591fd138, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97fd143, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x461fd14a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1fd156, 61 },
    { 0xee5fd156, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd45fd15d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e7fd160, 65 },
    { 0, 0 },
    { 0xa1efd162, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fd166, 34 },
    { 0xf13fd167, 33 },
    { 0xca2fd168, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x87efd16b, 90 },
    { 0xec0fd16c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00fd170, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a6fd17c, 48 },
    { 0xdeafd17d, 78 },
    { 0x18bfd17e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22fd183, 95 },
    { 0x7affd184, 23 },
    { 0x5aefd185, 33 },
    { 0xa14fd186, 23 },
    { 0x9cafd187, 33 },
    { 0x6affd187, 64 },
    { 0x12afd189, 43 },
    { 0xb8fd187, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edfd18e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95fd192, 77 },
    { 0xf8bfd193, 23 },
    { 0xa31fd193, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fd19a, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68fd19d, 61 },
    { 0xe82fd19d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x345fd1a3, 100 },
    { 0xdacfd1a4, 95 },
    { 0x9f1fd1a5, 75 },
    { 0x15cfd1a3, 55 },
    { 0x6b3fd1a7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x439fd1ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe5fd1b0, 95 },
    { 0xc64fd1b1, 90 },
    { 0xc0bfd1b1, 33 },
    { 0, 0 },
    { 0xed7fd1b4, 98 },
    { 0x271fd1b4, 33 },
    { 0, 0 },
    { 0xf06fd1b7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xc80fd1ba, 93 },
    { 0x300fd1bb, 55 },
    { 0, 0 },
    { 0x3a1fd1bd, 98 },
    { 0, 0 },
    { 0x525fd1bf, 34 },
    { 0, 0 },
    { 0xbbafd1c1, 61 },
    { 0xd6bfd1c2, 53 },
    { 0xa2dfd1c3, 34 },
    { 0x1ddfd1c4, 33 },
    { 0x8c3fd1c4, 95 },
    { 0x4f6fd1c6, 33 },
    { 0xe48fd1c6, 33 },
    { 0x155fd1c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x407fd1cc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf5cfd1d2, 23 },
    { 0xf3afd1d2, 23 },
    { 0, 0 },
    { 0x29dfd1d5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3bfd1d8, 77 },
    { 0x7d2fd1d9, 95 },
    { 0x51efd1d9, 90 },
    { 0, 0 },
    { 0x3b4fd1dc, 46 },
    { 0, 0 },
    { 0xea7fd1de, 2 },
    { 0x625fd1df, 76 },
    { 0x55afd1df, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fd1e3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x242fd1ea, 55 },
    { 0, 0 },
    { 0xe3ffd1ec, 64 },
    { 0x112fd1ed, 61 },
    { 0x430fd1ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99fd1f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff5fd1f7, 90 },
    { 0x3fcfd1f8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b0fd201, 33 },
    { 0x1a3fd202, 66 },
    { 0xe31fd203, 76 },
    { 0xfeafd204, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x298fd20a, 90 },
    { 0, 0 },
    { 0x936fd20c, 67 },
    { 0, 0 },
    { 0x3a4fd20e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fd216, 69 },
    { 0xb6dfd217, 33 },
    { 0, 0 },
    { 0x1cefd219, 98 },
    { 0, 0 },
    { 0xe6ffd21b, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0x84efd21e, 95 },
    { 0, 0 },
    { 0xe52fd220, 93 },
    { 0, 0 },
    { 0x524fd222, 55 },
    { 0x279fd223, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1dfd226, 90 },
    { 0, 0 },
    { 0xe3ffd228, 94 },
    { 0, 0 },
    { 0xc35fd22a, 65 },
    { 0, 0 },
    { 0xf4bfd22c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69fd230, 61 },
    { 0, 0 },
    { 0x185fd232, 83 },
    { 0, 0 },
    { 0xf5fd234, 61 },
    { 0, 0 },
    { 0x40cfd236, 61 },
    { 0xab1fd236, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143fd23d, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f2fd241, 66 },
    { 0xe0ffd241, 61 },
    { 0xc4efd243, 61 },
    { 0xf5efd243, 34 },
    { 0x7acfd242, 98 },
    { 0x63afd244, 88 },
    { 0x874fd245, 100 },
    { 0xcefd246, 91 },
    { 0x8f8fd246, 55 },
    { 0xa28fd241, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x973fd24e, 33 },
    { 0xa33fd24f, 95 },
    { 0x9c2fd24f, 90 },
    { 0xdf7fd251, 95 },
    { 0xd13fd251, 98 },
    { 0xfe5fd253, 34 },
    { 0xe21fd254, 43 },
    { 0x88cfd253, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2fd25c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb61fd263, 68 },
    { 0x564fd263, 33 },
    { 0xafbfd263, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x655fd268, 66 },
    { 0xdaefd269, 93 },
    { 0x741fd26a, 61 },
    { 0x21efd268, 18 },
    { 0, 0 },
    { 0xac8fd26d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe00fd275, 90 },
    { 0x414fd276, 34 },
    { 0, 0 },
    { 0xbbfd278, 100 },
    { 0x2c6fd279, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67afd27d, 77 },
    { 0xb6efd27d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115fd282, 46 },
    { 0x85fd282, 98 },
    { 0, 0 },
    { 0x3a6fd285, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31fd295, 76 },
    { 0x748fd296, 90 },
    { 0, 0 },
    { 0x395fd298, 98 },
    { 0x976fd299, 91 },
    { 0x4dfd299, 46 },
    { 0, 0 },
    { 0x8abfd29c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6ffd2a5, 19 },
    { 0, 0 },
    { 0xc25fd2a7, 98 },
    { 0, 0 },
    { 0x45efd2a9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b7fd2ae, 90 },
    { 0x865fd2ae, 90 },
    { 0x3fffd2ae, 87 },
    { 0x5b3fd2af, 19 },
    { 0, 0 },
    { 0x8eafd2b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9fd2b6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1fd2ba, 61 },
    { 0xe38fd2bb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9efd2c6, 33 },
    { 0, 0 },
    { 0xf99fd2c8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x780fd2cd, 23 },
    { 0, 0 },
    { 0x54cfd2cf, 91 },
    { 0xfb8fd2d0, 34 },
    { 0, 0 },
    { 0x1c5fd2d2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x705fd2d6, 23 },
    { 0xe52fd2d7, 90 },
    { 0xe17fd2d7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19dfd2dd, 98 },
    { 0, 0 },
    { 0xc0ffd2df, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1fd2e4, 55 },
    { 0, 0 },
    { 0xb6efd2e6, 43 },
    { 0xe06fd2e7, 95 },
    { 0xe9fd2e6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbffd2eb, 100 },
    { 0x4a3fd2ec, 61 },
    { 0x857fd2ed, 23 },
    { 0xd70fd2ec, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xadffd2f2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a4fd2fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc2fd2ff, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19fd306, 93 },
    { 0xcd1fd307, 93 },
    { 0x3b7fd307, 98 },
    { 0xfbefd307, 55 },
    { 0, 0 },
    { 0xe64fd30b, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef9fd310, 90 },
    { 0xf8efd311, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x605fd318, 33 },
    { 0, 0 },
    { 0x83dfd31a, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ffd31f, 33 },
    { 0x568fd320, 33 },
    { 0x54fd320, 61 },
    { 0xa1fd322, 90 },
    { 0x89fd322, 55 },
    { 0xbe1fd324, 66 },
    { 0xa71fd324, 33 },
    { 0xb9afd324, 76 },
    { 0x83dfd327, 90 },
    { 0x513fd328, 95 },
    { 0xd2ffd329, 61 },
    { 0xae4fd321, 55 },
    { 0x5e4fd32b, 93 },
    { 0x93afd32c, 61 },
    { 0x578fd32c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3fd331, 87 },
    { 0, 0 },
    { 0xd3ffd333, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x186fd33b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5fd33f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f0fd345, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd54fd348, 76 },
    { 0xc6dfd348, 33 },
    { 0xacbfd34a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2efd34e, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cefd357, 90 },
    { 0, 0 },
    { 0x2d8fd359, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65fd35d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x504fd362, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cfd36c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ffd372, 76 },
    { 0, 0 },
    { 0x287fd374, 98 },
    { 0xbc8fd375, 90 },
    { 0xb96fd375, 98 },
    { 0xaddfd377, 61 },
    { 0xb73fd377, 77 },
    { 0x934fd378, 98 },
    { 0, 0 },
    { 0x745fd37b, 33 },
    { 0xd41fd37b, 95 },
    { 0x751fd37c, 46 },
    { 0, 0 },
    { 0xdc5fd37f, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f3fd388, 46 },
    { 0xa42fd389, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1afd394, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xba3fd397, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525fd39b, 23 },
    { 0xba6fd39c, 52 },
    { 0xc25fd39d, 95 },
    { 0xc50fd39b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1eafd3a3, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dffd3aa, 61 },
    { 0, 0 },
    { 0xf5bfd3ac, 84 },
    { 0x244fd3ac, 98 },
    { 0, 0 },
    { 0x3d4fd3af, 81 },
    { 0, 0 },
    { 0xb16fd3b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb91fd3b7, 78 },
    { 0, 0 },
    { 0x244fd3b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb51fd3bd, 33 },
    { 0xf6bfd3be, 90 },
    { 0x88fd3bd, 98 },
    { 0x863fd3c0, 34 },
    { 0xbffd3bf, 61 },
    { 0x3c6fd3c2, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04fd3c6, 95 },
    { 0x90bfd3c7, 23 },
    { 0x5eafd3c8, 53 },
    { 0x5bffd3c8, 17 },
    { 0, 0 },
    { 0x626fd3cb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1a3fd3d0, 93 },
    { 0, 0 },
    { 0x2a5fd3d2, 94 },
    { 0, 0 },
    { 0xbb4fd3d4, 88 },
    { 0x34efd3d4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc6fd3db, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35dfd3df, 90 },
    { 0xf12fd3e0, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa69fd3e7, 23 },
    { 0x7a0fd3e7, 95 },
    { 0xbddfd3e8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6fd3ee, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf7fd3f1, 90 },
    { 0x447fd3f2, 33 },
    { 0xca1fd3f1, 61 },
    { 0x793fd3f4, 33 },
    { 0x22fd3f5, 95 },
    { 0x900fd3f6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bfd3f9, 81 },
    { 0x1cdfd3fa, 84 },
    { 0, 0 },
    { 0x6c3fd3fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82dfd400, 98 },
    { 0xc6efd401, 23 },
    { 0xf3ffd400, 87 },
    { 0, 0 },
    { 0xa6efd404, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12efd408, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x842fd40b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27fd40f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdd6fd414, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cafd418, 76 },
    { 0xf1afd419, 95 },
    { 0xda5fd41a, 61 },
    { 0, 0 },
    { 0xa53fd41c, 98 },
    { 0xb49fd41d, 99 },
    { 0x336fd41d, 99 },
    { 0xdc3fd41e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7a8fd425, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x829fd428, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x66ffd42d, 93 },
    { 0xcfefd42e, 36 },
    { 0x2f7fd42f, 34 },
    { 0xa0fd42f, 33 },
    { 0x218fd431, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86ffd434, 87 },
    { 0xcffd435, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x80ffd43a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30fd43d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x22dfd440, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x673fd445, 66 },
    { 0x3afd445, 33 },
    { 0x9dfd445, 82 },
    { 0x532fd447, 90 },
    { 0x636fd446, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6fd44c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xabafd452, 99 },
    { 0x559fd453, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebfd456, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x336fd45b, 98 },
    { 0xcb0fd45c, 91 },
    { 0x925fd45b, 23 },
    { 0, 0 },
    { 0x20ffd45f, 33 },
    { 0xb5dfd460, 76 },
    { 0xb27fd461, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d2fd467, 17 },
    { 0x28bfd468, 76 },
    { 0xc24fd468, 33 },
    { 0xfd8fd467, 36 },
    { 0xb1afd46b, 90 },
    { 0, 0 },
    { 0xf74fd46d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x564fd473, 95 },
    { 0x9f9fd473, 95 },
    { 0x434fd474, 95 },
    { 0x52efd473, 33 },
    { 0x259fd477, 61 },
    { 0x185fd478, 43 },
    { 0x5a5fd477, 34 },
    { 0x88cfd47a, 33 },
    { 0x833fd476, 95 },
    { 0xa9ffd476, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40fd481, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b6fd48c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x688fd48f, 23 },
    { 0, 0 },
    { 0xac1fd491, 95 },
    { 0x9ccfd492, 90 },
    { 0x16afd492, 61 },
    { 0xd10fd494, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe55fd49d, 90 },
    { 0xa1cfd49e, 95 },
    { 0xe3dfd49e, 33 },
    { 0x5effd4a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fd4a8, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319fd4ad, 58 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x115fd4b3, 52 },
    { 0xd8bfd4b4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb4dfd4ba, 90 },
    { 0xdfafd4bb, 67 },
    { 0x99dfd4bc, 23 },
    { 0x1dcfd4bb, 95 },
    { 0xcaffd4be, 33 },
    { 0x787fd4bf, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e0fd4c5, 61 },
    { 0x313fd4c5, 98 },
    { 0, 0 },
    { 0x102fd4c8, 81 },
    { 0xe44fd4c9, 91 },
    { 0x9e1fd4c9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x183fd4cf, 23 },
    { 0xc23fd4d0, 95 },
    { 0x18efd4d0, 100 },
    { 0x7c6fd4d2, 76 },
    { 0x871fd4d2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x22afd4d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x807fd4df, 90 },
    { 0x587fd4e0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8cfd4e3, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9fd4ea, 93 },
    { 0x451fd4eb, 61 },
    { 0, 0 },
    { 0x2e4fd4ed, 98 },
    { 0, 0 },
    { 0xc26fd4ef, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9fd4f7, 23 },
    { 0x16dfd4f8, 85 },
    { 0xfa3fd4f9, 34 },
    { 0x793fd4fa, 90 },
    { 0xd0cfd4fb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f2fd504, 95 },
    { 0x7cbfd504, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cfd50c, 69 },
    { 0x99bfd50d, 90 },
    { 0x7fcfd50e, 33 },
    { 0, 0 },
    { 0x216fd510, 78 },
    { 0xe10fd510, 53 },
    { 0x519fd512, 61 },
    { 0xdb7fd511, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b9fd51b, 90 },
    { 0, 0 },
    { 0x9affd51d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606fd520, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cfd523, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x757fd527, 61 },
    { 0xb10fd527, 33 },
    { 0x178fd529, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7aefd535, 23 },
    { 0x9f9fd535, 61 },
    { 0, 0 },
    { 0xa67fd538, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf42fd53b, 78 },
    { 0x598fd53b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c9fd542, 33 },
    { 0x59fd542, 34 },
    { 0, 0 },
    { 0xacafd545, 43 },
    { 0x477fd546, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bdfd549, 61 },
    { 0xe2dfd549, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x43afd54f, 61 },
    { 0x3f8fd550, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35ffd554, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x2befd557, 90 },
    { 0, 0 },
    { 0xeaafd559, 98 },
    { 0x8ffd55a, 98 },
    { 0xc93fd55b, 98 },
    { 0x524fd55a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87fd566, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322fd56b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7afd572, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf9fd579, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83fd57f, 93 },
    { 0, 0 },
    { 0xd16fd581, 33 },
    { 0x7ccfd582, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31dfd586, 80 },
    { 0xeb3fd587, 90 },
    { 0x375fd588, 23 },
    { 0x6f5fd589, 90 },
    { 0x308fd58a, 55 },
    { 0xda9fd58b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520fd596, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72fd599, 78 },
    { 0xa87fd599, 88 },
    { 0x681fd599, 33 },
    { 0x8c8fd599, 61 },
    { 0x203fd59d, 52 },
    { 0x6b4fd59c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2cfd5a2, 66 },
    { 0x477fd5a2, 34 },
    { 0xbfd5a4, 93 },
    { 0xd4dfd5a5, 76 },
    { 0x78efd5a6, 23 },
    { 0x602fd5a7, 34 },
    { 0xdd4fd5a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd28fd5ab, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdfd5ae, 33 },
    { 0x4d4fd5af, 95 },
    { 0, 0 },
    { 0x69afd5b1, 55 },
    { 0x8d3fd5b2, 74 },
    { 0, 0 },
    { 0x12afd5b4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7afd5b7, 95 },
    { 0, 0 },
    { 0x5dbfd5b9, 95 },
    { 0, 0 },
    { 0xd1cfd5bb, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x570fd5be, 90 },
    { 0x793fd5be, 99 },
    { 0x7befd5bf, 55 },
    { 0x746fd5c1, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba9fd5c5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x495fd5c8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212fd5cd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x901fd5d0, 78 },
    { 0xf3ffd5d1, 43 },
    { 0, 0 },
    { 0xbf0fd5d3, 33 },
    { 0x199fd5d4, 76 },
    { 0x85fd5d4, 23 },
    { 0, 0 },
    { 0xb0bfd5d7, 81 },
    { 0xa14fd5d8, 33 },
    { 0, 0 },
    { 0xb05fd5da, 61 },
    { 0x4b9fd5db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x703fd5de, 87 },
    { 0xf02fd5df, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424fd5e6, 95 },
    { 0x77fd5e6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x445fd5eb, 95 },
    { 0xefcfd5eb, 46 },
    { 0, 0 },
    { 0xbf8fd5ee, 46 },
    { 0xf59fd5ef, 34 },
    { 0xdf4fd5f0, 84 },
    { 0xee4fd5ef, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bfd5f5, 33 },
    { 0, 0 },
    { 0x2eefd5f7, 78 },
    { 0x9b3fd5f8, 90 },
    { 0xbf2fd5f8, 55 },
    { 0xa18fd5fa, 78 },
    { 0xfa3fd5f8, 90 },
    { 0x6ccfd5fc, 68 },
    { 0x7cbfd5fd, 33 },
    { 0xf00fd5fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92fd605, 91 },
    { 0xeebfd605, 90 },
    { 0x83ffd607, 90 },
    { 0, 0 },
    { 0x3fffd609, 33 },
    { 0x49bfd60a, 46 },
    { 0x146fd60a, 98 },
    { 0x63bfd609, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15efd610, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2afd613, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x898fd617, 43 },
    { 0x150fd618, 98 },
    { 0x1dafd618, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x131fd61f, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0efd623, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x889fd62b, 100 },
    { 0xbc5fd62c, 81 },
    { 0xecefd62d, 33 },
    { 0, 0 },
    { 0x334fd62f, 34 },
    { 0x713fd630, 95 },
    { 0xe3cfd630, 95 },
    { 0x13cfd632, 87 },
    { 0, 0 },
    { 0xceefd634, 46 },
    { 0xa3efd635, 61 },
    { 0, 0 },
    { 0x764fd637, 28 },
    { 0x145fd637, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dfd640, 61 },
    { 0xc4afd641, 95 },
    { 0x859fd640, 97 },
    { 0xac9fd642, 99 },
    { 0, 0 },
    { 0xf7ffd645, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fffd64d, 33 },
    { 0x85afd64e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x234fd655, 93 },
    { 0x17fd656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fd65d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9dfd662, 33 },
    { 0xc3bfd663, 33 },
    { 0, 0 },
    { 0xa50fd665, 46 },
    { 0, 0 },
    { 0xdbffd667, 90 },
    { 0, 0 },
    { 0x28fd669, 98 },
    { 0, 0 },
    { 0x1dffd66b, 98 },
    { 0x483fd66b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddbfd673, 84 },
    { 0x221fd674, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x59bfd67a, 90 },
    { 0xe9bfd67b, 33 },
    { 0x29dfd67a, 98 },
    { 0x9a5fd67b, 33 },
    { 0, 0 },
    { 0x707fd67f, 55 },
    { 0x298fd67f, 33 },
    { 0x31ffd67f, 33 },
    { 0x372fd682, 17 },
    { 0, 0 },
    { 0x435fd684, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0x5dbfd687, 68 },
    { 0, 0 },
    { 0x17fd689, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef7fd68e, 34 },
    { 0xd1bfd68f, 55 },
    { 0, 0 },
    { 0x781fd691, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd32fd69c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x687fd69f, 76 },
    { 0xa2bfd6a0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5fd6a6, 61 },
    { 0x12cfd6a6, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96fd6ab, 87 },
    { 0x225fd6ab, 33 },
    { 0x734fd6ab, 100 },
    { 0x8e9fd6ac, 33 },
    { 0x27cfd6af, 85 },
    { 0x256fd6ab, 33 },
    { 0x232fd6b0, 52 },
    { 0xa9dfd6ad, 34 },
    { 0, 0 },
    { 0x8c6fd6b4, 61 },
    { 0x51bfd6b5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x582fd6b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x579fd6bf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e3fd6c3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1fd6c8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4defd6cc, 33 },
    { 0x35afd6cc, 95 },
    { 0, 0 },
    { 0x2a0fd6cf, 43 },
    { 0, 0 },
    { 0x3f2fd6d1, 55 },
    { 0xf43fd6d1, 23 },
    { 0x27fd6d3, 33 },
    { 0, 0 },
    { 0x1fefd6d5, 43 },
    { 0x81cfd6d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf78fd6d9, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f6fd6e0, 67 },
    { 0xc93fd6e1, 85 },
    { 0xa73fd6e2, 91 },
    { 0x411fd6e3, 33 },
    { 0x119fd6e4, 91 },
    { 0x2c0fd6e2, 55 },
    { 0xd07fd6e1, 65 },
    { 0, 0 },
    { 0x30fd6e8, 33 },
    { 0, 0 },
    { 0x1b7fd6ea, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x303fd6ed, 85 },
    { 0x576fd6ee, 90 },
    { 0xf9dfd6ef, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2fd6f2, 95 },
    { 0x458fd6f3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf3fd6f8, 95 },
    { 0x386fd6f9, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x6afd6fc, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x908fd701, 23 },
    { 0x874fd702, 93 },
    { 0xcbcfd702, 93 },
    { 0xe44fd704, 93 },
    { 0x702fd704, 33 },
    { 0x5adfd702, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfdfd70e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x422fd713, 23 },
    { 0, 0 },
    { 0x9bdfd715, 33 },
    { 0x71cfd715, 91 },
    { 0xed7fd717, 33 },
    { 0x7a2fd718, 66 },
    { 0x2d0fd719, 90 },
    { 0xe66fd71a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dfd71d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac9fd725, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc8fd728, 43 },
    { 0xa8ffd729, 55 },
    { 0xaf4fd728, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x996fd732, 52 },
    { 0xe85fd732, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27afd737, 93 },
    { 0xc60fd737, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x383fd73b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6afd741, 78 },
    { 0x66efd742, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8befd746, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3dfd749, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x963fd750, 55 },
    { 0x539fd750, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7fd757, 99 },
    { 0x407fd758, 90 },
    { 0, 0 },
    { 0x71fd75a, 34 },
    { 0x151fd75a, 98 },
    { 0, 0 },
    { 0x653fd75d, 34 },
    { 0, 0 },
    { 0xbe3fd75f, 95 },
    { 0x970fd75f, 33 },
    { 0x459fd760, 23 },
    { 0xeb3fd75f, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x92dfd767, 61 },
    { 0x79cfd767, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x16afd76b, 85 },
    { 0x1d2fd76c, 100 },
    { 0x854fd76d, 76 },
    { 0xfbafd76d, 90 },
    { 0xfa3fd76d, 98 },
    { 0x644fd76b, 23 },
    { 0, 0 },
    { 0x815fd772, 91 },
    { 0xbdffd773, 61 },
    { 0xd38fd774, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d6fd77a, 61 },
    { 0x9e6fd77b, 23 },
    { 0xa49fd77b, 61 },
    { 0, 0 },
    { 0x140fd77e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf5fd783, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcddfd787, 56 },
    { 0xf28fd788, 61 },
    { 0, 0 },
    { 0xc1bfd78a, 66 },
    { 0, 0 },
    { 0x27afd78c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x520fd791, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cfd795, 76 },
    { 0x844fd796, 66 },
    { 0xe80fd797, 65 },
    { 0x7f9fd798, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75fd79e, 95 },
    { 0, 0 },
    { 0x444fd7a0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa36fd7a5, 61 },
    { 0, 0 },
    { 0xf14fd7a7, 33 },
    { 0xcf1fd7a8, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x168fd7ab, 61 },
    { 0x2a7fd7ac, 33 },
    { 0x4a3fd7ad, 90 },
    { 0x397fd7ab, 55 },
    { 0x31afd7ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xb00fd7b2, 69 },
    { 0, 0 },
    { 0xd6cfd7b4, 98 },
    { 0x3ecfd7b5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6efd7ba, 17 },
    { 0, 0 },
    { 0xf83fd7bc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dfd7c3, 88 },
    { 0xdb7fd7c4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5fd7c8, 78 },
    { 0, 0 },
    { 0x20dfd7ca, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x391fd7cd, 90 },
    { 0xee2fd7cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a5fd7d5, 98 },
    { 0xf50fd7d6, 61 },
    { 0xaedfd7d5, 66 },
    { 0x2d5fd7d8, 34 },
    { 0x1cfd7d9, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dfd7e0, 95 },
    { 0xc1afd7e1, 98 },
    { 0xab3fd7e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebbfd7e8, 75 },
    { 0x2a0fd7e9, 75 },
    { 0, 0 },
    { 0x6c9fd7eb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefd7ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2ffd7f6, 43 },
    { 0, 0 },
    { 0x47afd7f8, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf63fd802, 95 },
    { 0x7e8fd803, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b8fd808, 23 },
    { 0x953fd809, 75 },
    { 0x108fd80a, 43 },
    { 0x566fd80b, 24 },
    { 0x8c5fd809, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x5efd80f, 77 },
    { 0x4dafd810, 76 },
    { 0xa8efd810, 43 },
    { 0x2a3fd810, 76 },
    { 0, 0 },
    { 0x9d6fd814, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x322fd818, 43 },
    { 0xb6afd819, 23 },
    { 0xfefd81a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb28fd81d, 61 },
    { 0, 0 },
    { 0x4fffd81f, 55 },
    { 0, 0 },
    { 0xf5cfd821, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa86fd825, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xe57fd828, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2cffd82b, 85 },
    { 0xfedfd82b, 76 },
    { 0x219fd82d, 90 },
    { 0, 0 },
    { 0x2bbfd82f, 76 },
    { 0, 0 },
    { 0x161fd831, 23 },
    { 0, 0 },
    { 0xab8fd833, 23 },
    { 0, 0 },
    { 0xa4bfd835, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fd83b, 90 },
    { 0x96fd83c, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeefd83f, 52 },
    { 0, 0 },
    { 0x19cfd841, 78 },
    { 0x75ffd842, 23 },
    { 0xb99fd843, 34 },
    { 0x796fd844, 33 },
    { 0, 0 },
    { 0xf2dfd846, 76 },
    { 0, 0 },
    { 0x7fcfd848, 99 },
    { 0x315fd849, 52 },
    { 0x3bfd849, 23 },
    { 0x33afd84b, 66 },
    { 0xb72fd84c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf8fd852, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfd856, 33 },
    { 0, 0 },
    { 0xb8afd858, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc61fd85d, 98 },
    { 0xd74fd85d, 95 },
    { 0, 0 },
    { 0x75cfd860, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdfd863, 43 },
    { 0xbbcfd864, 90 },
    { 0x32ffd865, 91 },
    { 0x83ffd866, 84 },
    { 0x4aafd864, 76 },
    { 0x20bfd868, 98 },
    { 0, 0 },
    { 0xa4cfd86a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2f5fd86e, 33 },
    { 0x830fd86f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfd875, 77 },
    { 0x92bfd876, 95 },
    { 0xd20fd876, 61 },
    { 0xdeefd878, 76 },
    { 0x3a3fd879, 95 },
    { 0, 0 },
    { 0x889fd87b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x513fd87f, 98 },
    { 0xb22fd880, 92 },
    { 0x728fd880, 55 },
    { 0x297fd882, 90 },
    { 0x596fd882, 95 },
    { 0xe76fd884, 43 },
    { 0xc0bfd882, 61 },
    { 0x27efd886, 52 },
    { 0x316fd87f, 98 },
    { 0x818fd888, 34 },
    { 0x330fd885, 61 },
    { 0, 0 },
    { 0x969fd88b, 43 },
    { 0xa11fd88c, 33 },
    { 0x5b1fd88d, 98 },
    { 0xe79fd88d, 61 },
    { 0xecafd88b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bfd893, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaafd898, 23 },
    { 0x51afd898, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7fd8a0, 90 },
    { 0x2abfd8a1, 99 },
    { 0xd72fd8a2, 53 },
    { 0x36ffd8a3, 43 },
    { 0x1b8fd8a4, 90 },
    { 0xccefd8a1, 23 },
    { 0x215fd8a6, 95 },
    { 0xa49fd8a0, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bfd8ac, 33 },
    { 0xbfafd8ac, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7fd8b7, 99 },
    { 0, 0 },
    { 0x583fd8b9, 69 },
    { 0, 0 },
    { 0x65afd8bb, 23 },
    { 0x6c5fd8bc, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9afd8bf, 98 },
    { 0, 0 },
    { 0x15ffd8c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbfefd8c8, 38 },
    { 0, 0 },
    { 0x615fd8ca, 66 },
    { 0xa6cfd8cb, 90 },
    { 0x1e6fd8cc, 76 },
    { 0x7fbfd8cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6fd8d4, 55 },
    { 0xefefd8d5, 98 },
    { 0x33cfd8d6, 69 },
    { 0x73fd8d4, 76 },
    { 0xd02fd8d8, 55 },
    { 0, 0 },
    { 0xf9afd8da, 90 },
    { 0, 0 },
    { 0x731fd8dc, 64 },
    { 0, 0 },
    { 0xcf2fd8de, 81 },
    { 0x3f9fd8de, 46 },
    { 0, 0 },
    { 0x6fd8e1, 90 },
    { 0x439fd8e2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d2fd8e7, 99 },
    { 0, 0 },
    { 0xeb2fd8e9, 17 },
    { 0, 0 },
    { 0x94bfd8eb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eafd8f7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6fd8fa, 61 },
    { 0xe84fd8fb, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0fd901, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24ffd906, 55 },
    { 0, 0 },
    { 0xf8cfd908, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x654fd90b, 55 },
    { 0x201fd90b, 33 },
    { 0x4c3fd90d, 78 },
    { 0x509fd90b, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfefd911, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x336fd914, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x319fd918, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa24fd922, 67 },
    { 0x497fd923, 98 },
    { 0x3befd924, 95 },
    { 0xed4fd925, 23 },
    { 0, 0 },
    { 0xef6fd927, 33 },
    { 0, 0 },
    { 0x26ffd929, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555fd92e, 76 },
    { 0x34cfd92f, 98 },
    { 0xf15fd930, 61 },
    { 0x944fd931, 55 },
    { 0, 0 },
    { 0xf60fd933, 46 },
    { 0xdc6fd934, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfd938, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ffd93c, 68 },
    { 0, 0 },
    { 0x32bfd93e, 55 },
    { 0xb6afd93f, 90 },
    { 0x5c7fd93e, 95 },
    { 0xc54fd941, 95 },
    { 0x5dcfd93e, 76 },
    { 0x353fd940, 98 },
    { 0xe20fd93f, 75 },
    { 0x51ffd945, 91 },
    { 0xef2fd945, 66 },
    { 0, 0 },
    { 0x45afd948, 34 },
    { 0x8abfd948, 33 },
    { 0x4f3fd948, 61 },
    { 0, 0 },
    { 0x859fd94c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1fd94f, 42 },
    { 0x603fd950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc63fd955, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x530fd958, 55 },
    { 0x305fd959, 93 },
    { 0xd3dfd95a, 46 },
    { 0x75efd95b, 34 },
    { 0x3dfd95c, 33 },
    { 0x331fd95d, 95 },
    { 0x107fd95e, 33 },
    { 0x3e1fd95d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5fd962, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98fd969, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137fd96d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x518fd973, 98 },
    { 0x37cfd973, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x16bfd978, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x243fd97b, 100 },
    { 0, 0 },
    { 0xeabfd97d, 90 },
    { 0, 0 },
    { 0xce8fd97f, 90 },
    { 0x3f8fd980, 87 },
    { 0xa66fd980, 91 },
    { 0, 0 },
    { 0x577fd983, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18fd986, 61 },
    { 0x7a3fd987, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x626fd98d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf2fd992, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ffd995, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb6fd999, 33 },
    { 0, 0 },
    { 0x743fd99b, 55 },
    { 0x3a6fd99c, 98 },
    { 0xc10fd99d, 98 },
    { 0x221fd99e, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1fd9a2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x422fd9a5, 87 },
    { 0x501fd9a5, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cfd9a9, 55 },
    { 0xdd9fd9aa, 99 },
    { 0x4c7fd9aa, 90 },
    { 0x694fd9ab, 99 },
    { 0x463fd9ab, 85 },
    { 0xfeefd9ae, 46 },
    { 0xc48fd9af, 90 },
    { 0xea7fd9b0, 76 },
    { 0x226fd9b0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x82afd9b4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x348fd9ba, 76 },
    { 0xae6fd9bb, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e4fd9be, 87 },
    { 0x5f5fd9be, 76 },
    { 0x66cfd9be, 76 },
    { 0x3b2fd9c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x510fd9c5, 98 },
    { 0x528fd9c5, 69 },
    { 0, 0 },
    { 0x9cefd9c8, 95 },
    { 0x90fd9c9, 94 },
    { 0xfb8fd9ca, 43 },
    { 0, 0 },
    { 0x1afd9cc, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x137fd9d3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afd9d6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb9fd9da, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x86cfd9dd, 61 },
    { 0xc01fd9de, 34 },
    { 0xc4cfd9df, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa93fd9e3, 76 },
    { 0xd62fd9e3, 61 },
    { 0, 0 },
    { 0xc09fd9e6, 49 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3afd9ea, 98 },
    { 0x873fd9eb, 90 },
    { 0, 0 },
    { 0x620fd9ed, 90 },
    { 0x5cffd9ee, 93 },
    { 0x104fd9ee, 90 },
    { 0xcc0fd9f0, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e1fd9f6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9e3fd9fa, 66 },
    { 0, 0 },
    { 0x667fd9fc, 90 },
    { 0xbdbfd9fc, 61 },
    { 0, 0 },
    { 0x6fffd9ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e5fda02, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3fda06, 76 },
    { 0xe98fda07, 93 },
    { 0xe62fda06, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4fda0f, 40 },
    { 0x185fda10, 61 },
    { 0xeb8fda11, 98 },
    { 0xbfdfda11, 95 },
    { 0x812fda13, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x428fda17, 19 },
    { 0, 0 },
    { 0x3f6fda19, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31afda1d, 85 },
    { 0xe85fda1d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xacefda21, 43 },
    { 0, 0 },
    { 0x8bdfda23, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14fda29, 95 },
    { 0xc0bfda29, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x212fda30, 23 },
    { 0x21fda30, 33 },
    { 0x3f8fda31, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x326fda36, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x367fda39, 64 },
    { 0, 0 },
    { 0xe30fda3b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x188fda48, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xc68fda4b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x978fda4f, 98 },
    { 0x56ffda50, 87 },
    { 0xb8afda51, 81 },
    { 0xe0efda52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x240fda58, 90 },
    { 0x4cdfda58, 23 },
    { 0, 0 },
    { 0x63dfda5b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf23fda5e, 43 },
    { 0, 0 },
    { 0x80ffda60, 43 },
    { 0, 0 },
    { 0x560fda62, 23 },
    { 0, 0 },
    { 0x4fcfda64, 33 },
    { 0xc0afda65, 78 },
    { 0x783fda66, 55 },
    { 0xbe1fda67, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538fda6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27dfda78, 80 },
    { 0, 0 },
    { 0x8dffda7a, 90 },
    { 0x63ffda7b, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26fda84, 90 },
    { 0xe79fda85, 78 },
    { 0, 0 },
    { 0x231fda87, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b6fda8b, 46 },
    { 0x3fda8c, 95 },
    { 0x2aafda8b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x613fda90, 98 },
    { 0, 0 },
    { 0xd20fda92, 23 },
    { 0x907fda92, 23 },
    { 0, 0 },
    { 0x8c2fda95, 90 },
    { 0x672fda96, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xadefda99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6dfda9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ffdaa1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x992fdaa5, 44 },
    { 0x2d5fdaa6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f7fdaaa, 61 },
    { 0x914fdaaa, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd9fdab0, 98 },
    { 0xb51fdab1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d6fdab4, 23 },
    { 0, 0 },
    { 0xf1efdab6, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x78dfdaba, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xe20fdabd, 61 },
    { 0x815fdabe, 76 },
    { 0x804fdabf, 61 },
    { 0xb16fdac0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fdac4, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xcebfdac7, 98 },
    { 0, 0 },
    { 0x7a7fdac9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf95fdad0, 42 },
    { 0xcfafdad1, 23 },
    { 0xb5fdad2, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6fdad6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb9fdadc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x325fdae3, 43 },
    { 0xb11fdae3, 76 },
    { 0x51afdae5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf3fdaef, 98 },
    { 0xdc0fdaf0, 90 },
    { 0xbf2fdaf1, 95 },
    { 0, 0 },
    { 0x28dfdaf3, 78 },
    { 0xd26fdaf3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xceefdaf9, 88 },
    { 0x6cefdafa, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4aefdafd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3efdb00, 76 },
    { 0, 0 },
    { 0x59cfdb02, 98 },
    { 0, 0 },
    { 0x6dcfdb04, 76 },
    { 0, 0 },
    { 0x502fdb06, 46 },
    { 0xef7fdb07, 61 },
    { 0xd35fdb08, 88 },
    { 0x6ddfdb09, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc73fdb0d, 55 },
    { 0xd1efdb0e, 91 },
    { 0x1f2fdb0f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aafdb12, 43 },
    { 0xb19fdb13, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe97fdb16, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45cfdb19, 33 },
    { 0xc3fdb1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x542fdb28, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xba4fdb2d, 98 },
    { 0x973fdb2e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf80fdb31, 98 },
    { 0, 0 },
    { 0x8e8fdb33, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fbfdb37, 93 },
    { 0x30cfdb37, 98 },
    { 0x5f0fdb39, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1edfdb3f, 90 },
    { 0x101fdb40, 23 },
    { 0xb7ffdb41, 61 },
    { 0xbecfdb40, 61 },
    { 0xcabfdb43, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac1fdb48, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a3fdb4e, 95 },
    { 0x71afdb4f, 90 },
    { 0xc7dfdb4f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x53dfdb56, 33 },
    { 0x1b9fdb57, 90 },
    { 0x880fdb58, 55 },
    { 0x8cfdb59, 33 },
    { 0, 0 },
    { 0x1fefdb5b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50afdb64, 76 },
    { 0xafafdb65, 95 },
    { 0x962fdb66, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9efdb6a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x62fdb6d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x417fdb70, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x836fdb73, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d5fdb77, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xb22fdb7a, 76 },
    { 0xa20fdb7b, 33 },
    { 0, 0 },
    { 0xc8efdb7d, 93 },
    { 0x84efdb7e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6bfdb81, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x158fdb87, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x233fdb8a, 83 },
    { 0xc59fdb8b, 91 },
    { 0xaaefdb8b, 95 },
    { 0x7fffdb8d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8fdb93, 98 },
    { 0x41efdb94, 75 },
    { 0, 0 },
    { 0xe1efdb96, 33 },
    { 0, 0 },
    { 0xbefdb98, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x414fdb9b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x683fdb9e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266fdba3, 87 },
    { 0xfaafdba4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25ffdba8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc40fdbaf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x864fdbb5, 43 },
    { 0x4bbfdbb6, 90 },
    { 0x88cfdbb6, 76 },
    { 0, 0 },
    { 0x4fdfdbb9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe84fdbbd, 76 },
    { 0xf88fdbbd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaffdbc1, 61 },
    { 0, 0 },
    { 0x87dfdbc3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11bfdbc9, 33 },
    { 0xb2afdbca, 23 },
    { 0x700fdbc9, 61 },
    { 0x20efdbc9, 98 },
    { 0x21afdbcc, 100 },
    { 0x39ffdbce, 33 },
    { 0xd7bfdbcf, 61 },
    { 0xbb9fdbcf, 98 },
    { 0xa4cfdbd1, 98 },
    { 0xd95fdbce, 33 },
    { 0, 0 },
    { 0x5f7fdbd4, 87 },
    { 0x99fdbd5, 90 },
    { 0, 0 },
    { 0x67afdbd7, 93 },
    { 0x5fefdbd8, 96 },
    { 0xb8cfdbd9, 90 },
    { 0x454fdbd8, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5fdbe1, 33 },
    { 0x79efdbe2, 99 },
    { 0xf57fdbe3, 23 },
    { 0x262fdbe4, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6fdbed, 95 },
    { 0xa5efdbee, 98 },
    { 0x973fdbef, 39 },
    { 0x172fdbf0, 61 },
    { 0xeb5fdbf1, 95 },
    { 0x7a2fdbf0, 17 },
    { 0xaf9fdbf3, 75 },
    { 0x1e4fdbee, 46 },
    { 0xae1fdbf5, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276fdbfa, 84 },
    { 0xab2fdbfb, 98 },
    { 0xdd1fdbfc, 95 },
    { 0x4bffdbfc, 15 },
    { 0x9c3fdbfc, 43 },
    { 0xc74fdbff, 78 },
    { 0x937fdbff, 61 },
    { 0x453fdbff, 23 },
    { 0x14efdc00, 99 },
    { 0, 0 },
    { 0x727fdc04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cffdc09, 85 },
    { 0x395fdc0a, 77 },
    { 0xea3fdc0b, 90 },
    { 0x487fdc0c, 34 },
    { 0x67bfdc09, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dfdc11, 95 },
    { 0x322fdc12, 84 },
    { 0xc9cfdc13, 69 },
    { 0x94cfdc14, 93 },
    { 0xba6fdc14, 68 },
    { 0xa18fdc13, 33 },
    { 0xf1bfdc11, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbeffdc1d, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x676fdc22, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e5fdc27, 98 },
    { 0x303fdc28, 98 },
    { 0xa4ffdc29, 99 },
    { 0x3aefdc28, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c8fdc34, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x637fdc37, 75 },
    { 0x701fdc37, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bfdc3b, 46 },
    { 0x7d2fdc3c, 44 },
    { 0xb31fdc3d, 82 },
    { 0xe68fdc3e, 78 },
    { 0x5a3fdc3f, 90 },
    { 0x418fdc40, 61 },
    { 0x3e9fdc41, 99 },
    { 0x849fdc42, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf91fdc45, 98 },
    { 0xcfffdc45, 24 },
    { 0, 0 },
    { 0xd3ffdc48, 83 },
    { 0xf5ffdc49, 90 },
    { 0, 0 },
    { 0x165fdc4b, 90 },
    { 0x651fdc4b, 76 },
    { 0, 0 },
    { 0x8c0fdc4e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3dafdc52, 98 },
    { 0xd39fdc53, 23 },
    { 0xdfcfdc53, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x19cfdc59, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdfdc5c, 43 },
    { 0x5defdc5d, 95 },
    { 0, 0 },
    { 0xc4fdc5f, 95 },
    { 0, 0 },
    { 0xebcfdc61, 95 },
    { 0x30bfdc62, 61 },
    { 0xdf5fdc62, 23 },
    { 0xe85fdc64, 91 },
    { 0xb59fdc65, 95 },
    { 0x34bfdc65, 85 },
    { 0x335fdc67, 95 },
    { 0xa45fdc68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695fdc6c, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x502fdc71, 99 },
    { 0xe29fdc71, 95 },
    { 0x1d6fdc71, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63fdc79, 88 },
    { 0xf2cfdc7a, 90 },
    { 0x200fdc79, 91 },
    { 0xb40fdc7a, 23 },
    { 0, 0 },
    { 0x6ebfdc7e, 90 },
    { 0x2dafdc7f, 52 },
    { 0xd52fdc80, 98 },
    { 0xb68fdc81, 90 },
    { 0x613fdc80, 93 },
    { 0x358fdc83, 78 },
    { 0xc6efdc80, 33 },
    { 0x391fdc85, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x739fdc89, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05fdc8e, 93 },
    { 0xd91fdc8f, 76 },
    { 0x17ffdc90, 76 },
    { 0x845fdc91, 98 },
    { 0xd29fdc92, 67 },
    { 0xf55fdc90, 34 },
    { 0xfdefdc91, 46 },
    { 0x270fdc95, 43 },
    { 0xf35fdc96, 76 },
    { 0x699fdc97, 90 },
    { 0x79efdc98, 88 },
    { 0x335fdc99, 68 },
    { 0x20dfdc96, 66 },
    { 0xeefdc97, 90 },
    { 0x879fdc9c, 33 },
    { 0x296fdc9d, 34 },
    { 0, 0 },
    { 0x87ffdc9f, 52 },
    { 0, 0 },
    { 0xe03fdca1, 77 },
    { 0x94fdca2, 64 },
    { 0xa36fdca2, 90 },
    { 0xb9cfdca1, 76 },
    { 0, 0 },
    { 0xb95fdca6, 66 },
    { 0x411fdca7, 34 },
    { 0x3b6fdca6, 90 },
    { 0xc79fdca9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd00fdcae, 52 },
    { 0xc37fdcae, 43 },
    { 0, 0 },
    { 0x1effdcb1, 23 },
    { 0, 0 },
    { 0xcecfdcb3, 78 },
    { 0, 0 },
    { 0x6ecfdcb5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xa41fdcb8, 98 },
    { 0x1d4fdcb9, 55 },
    { 0x45bfdcb9, 43 },
    { 0xcf8fdcb8, 23 },
    { 0x934fdcbc, 93 },
    { 0x136fdcbd, 99 },
    { 0xa5bfdcbe, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddfdcc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64cfdcc8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x14fdccc, 53 },
    { 0x72dfdccd, 98 },
    { 0xf7fdcce, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd98fdcd6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb96fdcdb, 90 },
    { 0, 0 },
    { 0x423fdcdd, 46 },
    { 0x23fdcde, 99 },
    { 0x391fdcdf, 46 },
    { 0xd37fdcdf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811fdce4, 55 },
    { 0x27bfdce5, 90 },
    { 0xe85fdce6, 33 },
    { 0, 0 },
    { 0xde7fdce8, 55 },
    { 0xd95fdce9, 91 },
    { 0, 0 },
    { 0xb9cfdceb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0fdcf2, 98 },
    { 0x7ebfdcf3, 90 },
    { 0x12cfdcf4, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73dfdcf9, 84 },
    { 0, 0 },
    { 0x8fdfdcfb, 23 },
    { 0xbcefdcfc, 90 },
    { 0xb1afdcfd, 61 },
    { 0xfa2fdcfb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xddefdd01, 67 },
    { 0xc43fdd01, 95 },
    { 0, 0 },
    { 0xb28fdd04, 61 },
    { 0x1fefdd05, 93 },
    { 0xf1cfdd05, 4 },
    { 0x74efdd07, 61 },
    { 0x618fdd05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c3fdd0b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41fdd0f, 34 },
    { 0x928fdd10, 98 },
    { 0x533fdd11, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d9fdd14, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb01fdd20, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x514fdd27, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfefdd2b, 95 },
    { 0x75dfdd2c, 33 },
    { 0x4dfdd2d, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5eafdd33, 33 },
    { 0, 0 },
    { 0xf4ffdd35, 76 },
    { 0x66bfdd35, 33 },
    { 0x852fdd37, 33 },
    { 0x789fdd37, 90 },
    { 0xbbafdd39, 95 },
    { 0, 0 },
    { 0xf8efdd3b, 92 },
    { 0x35dfdd3c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x33fdd47, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x647fdd4b, 95 },
    { 0, 0 },
    { 0x841fdd4d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaffdd54, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cffdd5a, 87 },
    { 0x2c3fdd5b, 91 },
    { 0xe0cfdd5c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb3fdd5f, 46 },
    { 0, 0 },
    { 0xae8fdd61, 17 },
    { 0x4fffdd62, 93 },
    { 0x4e2fdd63, 85 },
    { 0xd7ffdd64, 61 },
    { 0x5cbfdd64, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x266fdd6a, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x89ffdd6d, 95 },
    { 0xa33fdd6d, 43 },
    { 0x610fdd6d, 67 },
    { 0x73cfdd70, 46 },
    { 0xd98fdd71, 55 },
    { 0xd73fdd70, 33 },
    { 0x11cfdd6d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x276fdd7d, 33 },
    { 0, 0 },
    { 0x787fdd7f, 95 },
    { 0xf7ffdd7f, 23 },
    { 0xd56fdd81, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xbabfdd84, 93 },
    { 0xd89fdd85, 93 },
    { 0, 0 },
    { 0x707fdd87, 61 },
    { 0x618fdd88, 46 },
    { 0x3c2fdd89, 84 },
    { 0xe99fdd8a, 56 },
    { 0x4a4fdd8b, 95 },
    { 0, 0 },
    { 0x18efdd8d, 78 },
    { 0xbb9fdd8e, 67 },
    { 0x57dfdd8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6affdd93, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1afdd9b, 99 },
    { 0x70fdd9c, 95 },
    { 0x2c8fdd9c, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1bfdda2, 61 },
    { 0xe32fdda3, 61 },
    { 0xcacfdda4, 98 },
    { 0, 0 },
    { 0xe9efdda6, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52afddab, 61 },
    { 0xf5efddac, 90 },
    { 0xdbffddad, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc2cfddb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x186fddb4, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4efddbd, 78 },
    { 0x4e8fddbd, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f5fddc1, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x954fddc4, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ffddc9, 90 },
    { 0x5ecfddca, 65 },
    { 0x41bfddca, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d8fddd0, 95 },
    { 0xc45fddd0, 98 },
    { 0x5c6fddd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14fddd5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a1fddd8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79bfddde, 55 },
    { 0x690fdddf, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb2fdde2, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb30fdded, 48 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe65fddf1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cefddf7, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95fddfb, 34 },
    { 0x1c0fddfc, 90 },
    { 0x82dfddfc, 34 },
    { 0xe8afddfc, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc77fde07, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x711fde0b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a2fde15, 76 },
    { 0, 0 },
    { 0xd17fde17, 52 },
    { 0xf2dfde17, 46 },
    { 0x6fafde19, 12 },
    { 0x6e1fde1a, 61 },
    { 0x991fde1a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962fde24, 76 },
    { 0x214fde24, 76 },
    { 0x5defde26, 43 },
    { 0xe92fde27, 61 },
    { 0x4dcfde28, 90 },
    { 0x82dfde28, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9befde31, 55 },
    { 0xce5fde32, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fde35, 34 },
    { 0xcfefde36, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x611fde3a, 68 },
    { 0xf8ffde3a, 100 },
    { 0x662fde3c, 34 },
    { 0x1d6fde3d, 33 },
    { 0xe12fde3d, 85 },
    { 0xcf3fde3f, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x213fde42, 95 },
    { 0, 0 },
    { 0x6c4fde44, 90 },
    { 0, 0 },
    { 0x881fde46, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd18fde4c, 75 },
    { 0x9cfde4c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b9fde50, 23 },
    { 0, 0 },
    { 0x4f8fde52, 98 },
    { 0x2e7fde53, 98 },
    { 0xd0efde54, 33 },
    { 0x103fde52, 84 },
    { 0, 0 },
    { 0x3d2fde57, 90 },
    { 0x6f3fde58, 98 },
    { 0, 0 },
    { 0x811fde5a, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf12fde5f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x15efde63, 77 },
    { 0x8f8fde63, 95 },
    { 0, 0 },
    { 0x535fde66, 61 },
    { 0x8e1fde67, 43 },
    { 0, 0 },
    { 0x27dfde69, 43 },
    { 0x8befde69, 83 },
    { 0x75cfde6b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xba1fde6e, 46 },
    { 0x823fde6f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x199fde73, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb8fde78, 23 },
    { 0xce3fde79, 90 },
    { 0, 0 },
    { 0x2bafde7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efde7e, 34 },
    { 0, 0 },
    { 0xf7bfde80, 33 },
    { 0xb15fde81, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa60fde84, 46 },
    { 0, 0 },
    { 0x68efde86, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efde8b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54efde93, 95 },
    { 0x7cafde93, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff1fde98, 90 },
    { 0xb65fde99, 42 },
    { 0, 0 },
    { 0, 0 },
    { 0x251fde9c, 95 },
    { 0, 0 },
    { 0x12efde9e, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf3afdeab, 43 },
    { 0x151fdeac, 23 },
    { 0x287fdeac, 33 },
    { 0, 0 },
    { 0xbc5fdeaf, 95 },
    { 0, 0 },
    { 0x21bfdeb1, 95 },
    { 0x2cdfdeb1, 99 },
    { 0, 0 },
    { 0x507fdeb4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91fdeb8, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06fdebc, 90 },
    { 0, 0 },
    { 0x4fdfdebe, 93 },
    { 0x588fdebf, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1f1fdec5, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x555fdeca, 91 },
    { 0xdf4fdeca, 90 },
    { 0xe95fdeca, 23 },
    { 0x754fdeca, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ffded0, 98 },
    { 0, 0 },
    { 0xa40fded2, 55 },
    { 0x77afded2, 4 },
    { 0xabffded3, 87 },
    { 0xe9afded5, 95 },
    { 0x1fdfded5, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x23cfdedf, 61 },
    { 0x857fdedf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c3fdee4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xcafdee7, 98 },
    { 0, 0 },
    { 0x4aafdee9, 90 },
    { 0x74afdee9, 67 },
    { 0, 0 },
    { 0x1aafdeec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x112fdef2, 76 },
    { 0x20efdef2, 55 },
    { 0x893fdef4, 92 },
    { 0x68cfdef5, 90 },
    { 0, 0 },
    { 0x6f7fdef7, 55 },
    { 0, 0 },
    { 0x650fdef9, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfdefc, 43 },
    { 0xc22fdefd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41afdf02, 33 },
    { 0xb9dfdf02, 90 },
    { 0x94ffdf04, 46 },
    { 0, 0 },
    { 0x1e1fdf06, 78 },
    { 0xa81fdf07, 67 },
    { 0x8adfdf08, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x55efdf17, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9fdf1b, 33 },
    { 0xa22fdf1b, 43 },
    { 0x590fdf1c, 55 },
    { 0x140fdf1e, 33 },
    { 0xb21fdf1d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xafefdf22, 66 },
    { 0x441fdf23, 34 },
    { 0x9f5fdf22, 33 },
    { 0xecbfdf25, 61 },
    { 0x6c7fdf26, 61 },
    { 0, 0 },
    { 0xeddfdf28, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dafdf2b, 95 },
    { 0x38efdf2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fdf33, 43 },
    { 0, 0 },
    { 0x97fdf35, 95 },
    { 0xd29fdf36, 93 },
    { 0, 0 },
    { 0xac7fdf38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa10fdf3d, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xd99fdf40, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab8fdf44, 43 },
    { 0, 0 },
    { 0xd1dfdf46, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cdfdf4a, 87 },
    { 0, 0 },
    { 0x824fdf4c, 68 },
    { 0x363fdf4c, 17 },
    { 0, 0 },
    { 0xca2fdf4f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41cfdf59, 34 },
    { 0, 0 },
    { 0x6bcfdf5b, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0xc17fdf5e, 23 },
    { 0x419fdf5e, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x60cfdf67, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x721fdf6e, 46 },
    { 0x498fdf6f, 83 },
    { 0x987fdf70, 100 },
    { 0, 0 },
    { 0xf65fdf72, 95 },
    { 0x977fdf73, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4fdf77, 23 },
    { 0, 0 },
    { 0xf5dfdf79, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4fdf7c, 33 },
    { 0x436fdf7d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x702fdf80, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x996fdf83, 61 },
    { 0xadfdf83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39fdf87, 90 },
    { 0x9a1fdf88, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c8fdf93, 53 },
    { 0, 0 },
    { 0xcb1fdf95, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40ffdf99, 23 },
    { 0x72bfdf9a, 90 },
    { 0x1acfdf9b, 76 },
    { 0x6e7fdf9c, 61 },
    { 0xa19fdf9c, 85 },
    { 0xc2fdf9e, 52 },
    { 0x45fdf99, 33 },
    { 0x9c1fdf9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ffdfa4, 33 },
    { 0x302fdfa5, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9efdfa9, 46 },
    { 0xaf4fdfa9, 23 },
    { 0xf74fdfab, 76 },
    { 0xf56fdfab, 33 },
    { 0x9c4fdfad, 61 },
    { 0, 0 },
    { 0x617fdfaf, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xa02fdfb2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dfdfb7, 61 },
    { 0xcf2fdfb7, 51 },
    { 0xd03fdfb9, 40 },
    { 0, 0 },
    { 0xf51fdfbb, 100 },
    { 0x167fdfbc, 76 },
    { 0xf78fdfbd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1acfdfc0, 76 },
    { 0, 0 },
    { 0x8c6fdfc2, 46 },
    { 0x6d1fdfc3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5acfdfc9, 90 },
    { 0xfaafdfc9, 33 },
    { 0x869fdfc9, 61 },
    { 0xfb2fdfcc, 95 },
    { 0x266fdfcd, 46 },
    { 0xaf2fdfcc, 43 },
    { 0x199fdfca, 33 },
    { 0, 0 },
    { 0xcf6fdfd1, 61 },
    { 0xf0ffdfd2, 82 },
    { 0, 0 },
    { 0x1fdfdfd4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75cfdfe0, 88 },
    { 0xe9bfdfe0, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aefdfe8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x212fdfeb, 90 },
    { 0x4c6fdfec, 61 },
    { 0x6aefdfed, 90 },
    { 0, 0 },
    { 0xecefdfef, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46fdff3, 55 },
    { 0, 0 },
    { 0xd57fdff5, 61 },
    { 0x405fdff6, 90 },
    { 0x5fafdff7, 61 },
    { 0x47cfdff8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd7fdffd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa1fe000, 91 },
    { 0x369fe001, 90 },
    { 0x7fbfe002, 23 },
    { 0x80afe003, 78 },
    { 0x87ffe003, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c1fe012, 55 },
    { 0xc91fe013, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7efe016, 85 },
    { 0x685fe017, 77 },
    { 0x40efe016, 33 },
    { 0x4b0fe019, 43 },
    { 0xe32fe019, 61 },
    { 0, 0 },
    { 0x1eafe01c, 64 },
    { 0, 0 },
    { 0xbf5fe01e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c0fe026, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x87ffe029, 34 },
    { 0, 0 },
    { 0xe25fe02b, 76 },
    { 0, 0 },
    { 0x41fe02d, 23 },
    { 0x130fe02e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x755fe031, 85 },
    { 0x1a4fe032, 90 },
    { 0, 0 },
    { 0x6cbfe034, 81 },
    { 0x173fe034, 95 },
    { 0, 0 },
    { 0x762fe037, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79fe03d, 61 },
    { 0xfaefe03e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x386fe041, 33 },
    { 0, 0 },
    { 0xe7ffe043, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1effe04a, 93 },
    { 0xf43fe04b, 55 },
    { 0x346fe04b, 98 },
    { 0x5b9fe04d, 17 },
    { 0xd06fe04e, 67 },
    { 0, 0 },
    { 0x57dfe050, 61 },
    { 0xf4cfe050, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xe86fe054, 33 },
    { 0x95ffe055, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd4fe059, 78 },
    { 0, 0 },
    { 0xddffe05b, 55 },
    { 0xa7fe05c, 98 },
    { 0xf80fe05c, 61 },
    { 0xf0afe05d, 43 },
    { 0xa17fe05f, 76 },
    { 0x205fe060, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac5fe065, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xa64fe068, 90 },
    { 0xe2afe068, 98 },
    { 0xdbfe06a, 88 },
    { 0xde2fe06b, 61 },
    { 0xdfdfe06c, 78 },
    { 0x161fe06d, 90 },
    { 0x49cfe06e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x136fe071, 72 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfaffe077, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416fe07b, 98 },
    { 0xdc7fe07c, 78 },
    { 0x38cfe07d, 90 },
    { 0x58dfe07e, 34 },
    { 0x81fe07b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e3fe083, 23 },
    { 0x6dafe084, 87 },
    { 0xc47fe083, 61 },
    { 0x49cfe086, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x28afe08b, 90 },
    { 0xf4afe08c, 61 },
    { 0, 0 },
    { 0x8c5fe08e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1cfe097, 61 },
    { 0, 0 },
    { 0x63cfe099, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf3fe09c, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6afe0a1, 90 },
    { 0, 0 },
    { 0xeaafe0a3, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x582fe0a7, 90 },
    { 0xa87fe0a7, 34 },
    { 0x3e9fe0a9, 87 },
    { 0x5ccfe0a7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x82cfe0ad, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xbefe0b0, 88 },
    { 0x1c8fe0b1, 23 },
    { 0x64cfe0b2, 55 },
    { 0xa3afe0b3, 33 },
    { 0, 0 },
    { 0xc92fe0b5, 90 },
    { 0x349fe0b6, 98 },
    { 0x8fafe0b6, 78 },
    { 0x133fe0b7, 100 },
    { 0x24cfe0b8, 90 },
    { 0xdebfe0b6, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cbfe0bd, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x225fe0c8, 90 },
    { 0xd5cfe0c9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x646fe0d0, 33 },
    { 0x50cfe0d1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x825fe0d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426fe0de, 33 },
    { 0xdb7fe0df, 55 },
    { 0xefe0e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x34afe0e7, 90 },
    { 0x7a6fe0e8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67cfe0ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe2cfe0f1, 52 },
    { 0xfc1fe0f2, 48 },
    { 0x9acfe0f1, 90 },
    { 0x3cfe0f1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96afe0f9, 95 },
    { 0x684fe0fa, 90 },
    { 0x3ffe0fb, 78 },
    { 0xf79fe0fb, 33 },
    { 0x351fe0f9, 61 },
    { 0x93afe0fe, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd4fe103, 61 },
    { 0xf5afe103, 84 },
    { 0, 0 },
    { 0xc2fe106, 23 },
    { 0, 0 },
    { 0xa3ffe108, 23 },
    { 0xe03fe109, 23 },
    { 0, 0 },
    { 0xbedfe10b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x535fe110, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b0fe117, 55 },
    { 0x94ffe118, 99 },
    { 0x65ffe118, 40 },
    { 0x28fe11a, 93 },
    { 0x1fafe11a, 76 },
    { 0xb5afe11c, 98 },
    { 0, 0 },
    { 0xdb8fe11e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58efe123, 76 },
    { 0x721fe124, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf08fe127, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x299fe12a, 9 },
    { 0, 0 },
    { 0x5fcfe12c, 33 },
    { 0xef4fe12d, 23 },
    { 0x467fe12d, 40 },
    { 0xb75fe12f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf30fe133, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bfe137, 23 },
    { 0x16efe138, 98 },
    { 0xd57fe138, 23 },
    { 0, 0 },
    { 0x644fe13b, 23 },
    { 0x2bfe13c, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x593fe140, 90 },
    { 0x997fe141, 90 },
    { 0x4adfe142, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdfe148, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7cefe14d, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x141fe156, 23 },
    { 0xf54fe157, 78 },
    { 0x209fe158, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cbfe160, 81 },
    { 0xea6fe161, 95 },
    { 0x86dfe161, 90 },
    { 0, 0 },
    { 0x512fe164, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb0fe167, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bfe171, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ffe174, 90 },
    { 0x3b7fe175, 95 },
    { 0xa1afe176, 43 },
    { 0xbc4fe177, 98 },
    { 0x1d2fe178, 87 },
    { 0, 0 },
    { 0x313fe17a, 98 },
    { 0, 0 },
    { 0xc6fe17c, 61 },
    { 0xb78fe17d, 98 },
    { 0, 0 },
    { 0x5e2fe17f, 61 },
    { 0xce8fe180, 81 },
    { 0x584fe181, 33 },
    { 0, 0 },
    { 0xfe6fe183, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76cfe187, 100 },
    { 0xfe188, 65 },
    { 0xb9ffe188, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946fe190, 23 },
    { 0xd68fe191, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416fe196, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x631fe199, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc1fe19d, 95 },
    { 0x368fe19e, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x732fe1a1, 90 },
    { 0x307fe1a2, 43 },
    { 0x508fe1a1, 23 },
    { 0, 0 },
    { 0xa7bfe1a5, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeefe1a8, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x929fe1ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56bfe1b0, 66 },
    { 0xe85fe1b0, 61 },
    { 0x51bfe1b2, 33 },
    { 0x791fe1b3, 95 },
    { 0, 0 },
    { 0x677fe1b5, 23 },
    { 0x6c5fe1b5, 90 },
    { 0xde4fe1b7, 98 },
    { 0x57bfe1b5, 33 },
    { 0xe7ffe1b6, 43 },
    { 0x6cfe1b9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x78bfe1bd, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe98fe1c0, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x87cfe1c5, 98 },
    { 0xa5cfe1c5, 55 },
    { 0x886fe1c7, 33 },
    { 0x4dcfe1c8, 95 },
    { 0xeb2fe1c9, 23 },
    { 0x926fe1c7, 87 },
    { 0xfd2fe1cb, 88 },
    { 0, 0 },
    { 0x7befe1cd, 55 },
    { 0x9edfe1cd, 23 },
    { 0x330fe1cf, 33 },
    { 0, 0 },
    { 0xfefe1d1, 91 },
    { 0xae4fe1d2, 33 },
    { 0, 0 },
    { 0xf95fe1d4, 25 },
    { 0, 0 },
    { 0, 0 },
    { 0x65efe1d7, 55 },
    { 0x3a0fe1d8, 67 },
    { 0, 0 },
    { 0xd35fe1da, 76 },
    { 0xce6fe1da, 90 },
    { 0, 0 },
    { 0xc8afe1dd, 46 },
    { 0, 0 },
    { 0x62bfe1df, 33 },
    { 0, 0 },
    { 0x5a9fe1e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf79fe1e6, 90 },
    { 0, 0 },
    { 0x3cafe1e8, 55 },
    { 0x827fe1e9, 61 },
    { 0xc74fe1e8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x781fe1f0, 23 },
    { 0, 0 },
    { 0x9b1fe1f2, 75 },
    { 0, 0 },
    { 0xeb9fe1f4, 33 },
    { 0xaa9fe1f4, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c0fe1fd, 55 },
    { 0xa1efe1fe, 100 },
    { 0xf9efe1fe, 95 },
    { 0x476fe200, 61 },
    { 0x6fffe201, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356fe206, 33 },
    { 0x5a7fe207, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x1b7fe20a, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x390fe20d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfe211, 65 },
    { 0x4f9fe211, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50fe218, 23 },
    { 0x484fe219, 34 },
    { 0x244fe21a, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ddfe224, 34 },
    { 0x77cfe225, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa08fe22a, 33 },
    { 0x8f0fe22b, 90 },
    { 0x765fe22b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaebfe230, 61 },
    { 0x89bfe231, 67 },
    { 0xb87fe232, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe59fe236, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa54fe23a, 33 },
    { 0x73afe23b, 33 },
    { 0xe69fe23a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf96fe242, 81 },
    { 0x3a8fe242, 40 },
    { 0x6f8fe244, 90 },
    { 0xe3efe245, 90 },
    { 0x820fe246, 34 },
    { 0x324fe245, 23 },
    { 0x9e5fe248, 33 },
    { 0xaa0fe249, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9bfe24c, 55 },
    { 0, 0 },
    { 0xfe6fe24e, 93 },
    { 0xe2bfe24f, 61 },
    { 0xf71fe24f, 99 },
    { 0x705fe24e, 23 },
    { 0x579fe252, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe5efe257, 67 },
    { 0x18dfe258, 90 },
    { 0xa01fe259, 61 },
    { 0x63dfe259, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fe263, 23 },
    { 0x1c8fe263, 23 },
    { 0x75dfe265, 61 },
    { 0xbf0fe263, 33 },
    { 0x6fe267, 33 },
    { 0x9d9fe267, 90 },
    { 0x74fe269, 33 },
    { 0x58fe26a, 96 },
    { 0x186fe264, 98 },
    { 0x4a0fe266, 75 },
    { 0xb9cfe26d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa89fe270, 43 },
    { 0x63dfe271, 69 },
    { 0x5f2fe272, 61 },
    { 0x4d5fe273, 23 },
    { 0, 0 },
    { 0xeabfe275, 90 },
    { 0x3a9fe276, 98 },
    { 0x675fe276, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x285fe27a, 91 },
    { 0xf3cfe27a, 55 },
    { 0, 0 },
    { 0x73efe27d, 95 },
    { 0xd40fe27d, 98 },
    { 0xa44fe27f, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78fe283, 98 },
    { 0x286fe283, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x2fe287, 33 },
    { 0x6aafe288, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcfe28c, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x88bfe28f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8befe294, 55 },
    { 0x754fe295, 91 },
    { 0, 0 },
    { 0xe05fe297, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xde1fe29d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6fe2a6, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fe2aa, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21fe2ae, 81 },
    { 0, 0 },
    { 0xc17fe2b0, 90 },
    { 0, 0 },
    { 0x179fe2b2, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x367fe2b8, 40 },
    { 0, 0 },
    { 0xec6fe2ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98fe2bd, 24 },
    { 0, 0 },
    { 0xb63fe2bf, 76 },
    { 0x809fe2bf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5cfe2c3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x356fe2c8, 33 },
    { 0x652fe2c9, 55 },
    { 0x70ffe2c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12fe2d1, 61 },
    { 0xf5efe2d2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20cfe2d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9a5fe2db, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x14dfe2de, 96 },
    { 0, 0 },
    { 0xdcdfe2e0, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a1fe2e3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xb85fe2e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe2eb, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x484fe2ee, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x27cfe2f1, 61 },
    { 0xb68fe2f2, 90 },
    { 0x957fe2f3, 76 },
    { 0xa36fe2f4, 98 },
    { 0x53efe2f4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe1ffe2fb, 61 },
    { 0, 0 },
    { 0xfbffe2fd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xff0fe306, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xacbfe309, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ffe30f, 91 },
    { 0, 0 },
    { 0xe4afe311, 90 },
    { 0, 0 },
    { 0x7dcfe313, 55 },
    { 0x750fe314, 91 },
    { 0x427fe315, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd8fe31b, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xaaefe31e, 46 },
    { 0x913fe31f, 90 },
    { 0xa14fe31f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bffe326, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x525fe32b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bdfe331, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cfe337, 84 },
    { 0xec1fe337, 33 },
    { 0, 0 },
    { 0xd2cfe33a, 98 },
    { 0x80fe33b, 23 },
    { 0x4edfe33a, 90 },
    { 0x1f5fe33d, 90 },
    { 0xa0cfe33d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf66fe341, 96 },
    { 0, 0 },
    { 0x2d4fe343, 91 },
    { 0xc5efe344, 33 },
    { 0x69dfe343, 67 },
    { 0x99dfe346, 61 },
    { 0x3f6fe346, 46 },
    { 0xf72fe344, 98 },
    { 0xbf6fe349, 98 },
    { 0xe2afe347, 98 },
    { 0x4b7fe34b, 98 },
    { 0xdf2fe347, 80 },
    { 0, 0 },
    { 0xe43fe34e, 67 },
    { 0x83efe34e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x589fe352, 61 },
    { 0, 0 },
    { 0x90cfe354, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44fe357, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567fe360, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e5fe363, 90 },
    { 0x6d2fe364, 98 },
    { 0xe17fe363, 55 },
    { 0x27dfe365, 90 },
    { 0x787fe367, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50bfe36c, 61 },
    { 0x38dfe36d, 78 },
    { 0xb9bfe36d, 95 },
    { 0x839fe36c, 92 },
    { 0x9aafe370, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0bfe375, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb63fe37e, 84 },
    { 0x9d3fe37f, 90 },
    { 0, 0 },
    { 0x9dbfe381, 61 },
    { 0, 0 },
    { 0x729fe383, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dafe387, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x80cfe38a, 55 },
    { 0, 0 },
    { 0x8e8fe38c, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xccefe390, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6dfe393, 98 },
    { 0xac8fe394, 96 },
    { 0, 0 },
    { 0x43ffe396, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bbfe399, 56 },
    { 0, 0 },
    { 0x166fe39b, 33 },
    { 0xb30fe39c, 90 },
    { 0, 0 },
    { 0x802fe39e, 33 },
    { 0x810fe39e, 90 },
    { 0, 0 },
    { 0x2befe3a1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x632fe3a5, 90 },
    { 0x499fe3a6, 90 },
    { 0, 0 },
    { 0x3c4fe3a8, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x9affe3ab, 90 },
    { 0xe3dfe3ac, 90 },
    { 0xd2cfe3ab, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b4fe3b5, 76 },
    { 0xf5bfe3b6, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa68fe3ba, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6d6fe3bf, 66 },
    { 0, 0 },
    { 0xd81fe3c1, 33 },
    { 0xa5ffe3c1, 34 },
    { 0x6fbfe3c2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3bfe3c7, 34 },
    { 0xbb9fe3c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3ffe3cb, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc98fe3cf, 76 },
    { 0, 0 },
    { 0x864fe3d1, 76 },
    { 0xf3ffe3d2, 93 },
    { 0xdccfe3d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeebfe3d7, 93 },
    { 0, 0 },
    { 0x255fe3d9, 95 },
    { 0xe4cfe3da, 87 },
    { 0, 0 },
    { 0x941fe3dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c3fe3e2, 23 },
    { 0x4e3fe3e3, 61 },
    { 0x280fe3e4, 90 },
    { 0x409fe3e5, 61 },
    { 0x744fe3e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x174fe3e9, 75 },
    { 0xaa9fe3ea, 93 },
    { 0x755fe3eb, 90 },
    { 0xf41fe3ec, 66 },
    { 0xae2fe3ec, 35 },
    { 0x17afe3ec, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f3fe3f2, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7fe3f7, 99 },
    { 0xc41fe3f8, 98 },
    { 0xf07fe3f7, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f8fe3fc, 61 },
    { 0x5aafe3fd, 90 },
    { 0x54afe3fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc8efe403, 95 },
    { 0x12bfe404, 84 },
    { 0xa60fe405, 23 },
    { 0xc15fe405, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bafe40b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe418, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x855fe41b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdf5fe421, 61 },
    { 0, 0 },
    { 0x18bfe423, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeffe426, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adfe429, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9efe42c, 33 },
    { 0xe2afe42c, 61 },
    { 0, 0 },
    { 0x88fe42f, 38 },
    { 0xb9ffe430, 46 },
    { 0x25bfe430, 61 },
    { 0, 0 },
    { 0x38efe433, 43 },
    { 0xf52fe434, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x22bfe437, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ffe441, 33 },
    { 0x8bbfe441, 23 },
    { 0x174fe443, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa28fe447, 33 },
    { 0, 0 },
    { 0xc3cfe449, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdefe44c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbcfe44f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x674fe452, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xdddfe455, 90 },
    { 0x6bafe456, 98 },
    { 0x9dbfe457, 51 },
    { 0xfb8fe458, 23 },
    { 0x7ecfe455, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddfe461, 55 },
    { 0xdd9fe461, 55 },
    { 0x508fe463, 33 },
    { 0x54afe464, 61 },
    { 0x6dafe461, 80 },
    { 0x356fe464, 78 },
    { 0x205fe467, 84 },
    { 0x738fe465, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0fe46c, 93 },
    { 0x866fe46c, 90 },
    { 0x85afe46d, 98 },
    { 0x2ccfe46f, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf84fe478, 96 },
    { 0x834fe479, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x36ffe47c, 91 },
    { 0, 0 },
    { 0xaf1fe47e, 61 },
    { 0x542fe47f, 34 },
    { 0xe37fe47e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe3fe485, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x549fe48c, 43 },
    { 0, 0 },
    { 0x600fe48e, 33 },
    { 0x4d7fe48f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x760fe493, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacfe497, 100 },
    { 0, 0 },
    { 0x848fe499, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0xb2ffe49c, 95 },
    { 0x5bffe49d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83efe4a1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4fe4a5, 46 },
    { 0, 0 },
    { 0xb84fe4a7, 23 },
    { 0, 0 },
    { 0x724fe4a9, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ffe4ad, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dffe4b2, 87 },
    { 0x82fe4b3, 23 },
    { 0x7c0fe4b4, 95 },
    { 0, 0 },
    { 0x4ebfe4b6, 85 },
    { 0xcf8fe4b6, 98 },
    { 0xa54fe4b8, 61 },
    { 0x143fe4b9, 98 },
    { 0, 0 },
    { 0x813fe4bb, 91 },
    { 0x162fe4bb, 61 },
    { 0x6e0fe4bd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x505fe4c2, 43 },
    { 0x6c0fe4c3, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd5cfe4c9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc25fe4d4, 95 },
    { 0, 0 },
    { 0xf6ffe4d6, 53 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa50fe4da, 46 },
    { 0, 0 },
    { 0xc98fe4dc, 23 },
    { 0xe51fe4dd, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdafe4e9, 87 },
    { 0, 0 },
    { 0xd48fe4eb, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb47fe4ee, 90 },
    { 0x7efe4ef, 61 },
    { 0x35bfe4ef, 93 },
    { 0x154fe4f1, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fe4f4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xebafe500, 95 },
    { 0x292fe501, 46 },
    { 0x178fe500, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x389fe509, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ffe50f, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7cfe51c, 43 },
    { 0x7c4fe51d, 43 },
    { 0xf50fe51d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72cfe524, 76 },
    { 0x234fe524, 100 },
    { 0x645fe526, 90 },
    { 0xf6dfe524, 87 },
    { 0xfbbfe526, 98 },
    { 0x5f4fe529, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26fe52c, 95 },
    { 0, 0 },
    { 0x407fe52e, 77 },
    { 0x958fe52f, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfebfe534, 23 },
    { 0, 0 },
    { 0xd1fe536, 33 },
    { 0xc47fe536, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7edfe53d, 67 },
    { 0, 0 },
    { 0x57bfe53f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x96ffe543, 95 },
    { 0x2e7fe543, 43 },
    { 0, 0 },
    { 0x3dcfe546, 33 },
    { 0, 0 },
    { 0xf06fe548, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dfe54e, 95 },
    { 0xef3fe54f, 61 },
    { 0xe39fe550, 90 },
    { 0x297fe550, 66 },
    { 0x465fe552, 95 },
    { 0xbbefe553, 95 },
    { 0x874fe554, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29fe55b, 93 },
    { 0x9fafe55c, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x921fe560, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x265fe564, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0fe567, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aafe56a, 88 },
    { 0xdb5fe56a, 61 },
    { 0xbbffe56c, 61 },
    { 0, 0 },
    { 0x786fe56e, 23 },
    { 0xc14fe56e, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe572, 98 },
    { 0xd3dfe572, 90 },
    { 0xdbbfe574, 90 },
    { 0x48dfe574, 34 },
    { 0x8f6fe576, 76 },
    { 0xef3fe577, 91 },
    { 0xc9efe578, 90 },
    { 0x955fe579, 90 },
    { 0x3b1fe57a, 43 },
    { 0x3e4fe575, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85afe57f, 33 },
    { 0xf34fe580, 34 },
    { 0xe34fe581, 23 },
    { 0x195fe582, 95 },
    { 0xb4efe583, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a0fe587, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf7afe58b, 98 },
    { 0xe4ffe58c, 67 },
    { 0, 0 },
    { 0x2e5fe58e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbdefe59a, 90 },
    { 0x580fe59a, 43 },
    { 0xe85fe59c, 66 },
    { 0xbaefe59d, 33 },
    { 0x165fe59e, 33 },
    { 0x7a6fe59c, 90 },
    { 0xce3fe5a0, 76 },
    { 0xc71fe59f, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x73fe5a5, 90 },
    { 0x57bfe5a6, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x67fe5a9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe5af, 34 },
    { 0xc43fe5b0, 90 },
    { 0xb6ffe5af, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce4fe5bb, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xbc3fe5be, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x98cfe5c1, 61 },
    { 0x35afe5c2, 33 },
    { 0xcdffe5c2, 90 },
    { 0, 0 },
    { 0x8cafe5c5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e9fe5c9, 61 },
    { 0x33fe5ca, 99 },
    { 0, 0 },
    { 0x300fe5cc, 55 },
    { 0, 0 },
    { 0x2dafe5ce, 33 },
    { 0x747fe5cf, 81 },
    { 0xf8bfe5d0, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b2fe5d6, 23 },
    { 0, 0 },
    { 0x3f4fe5d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfd0fe5dc, 96 },
    { 0xd28fe5dd, 61 },
    { 0x944fe5de, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdfe5e1, 61 },
    { 0xf8bfe5e2, 53 },
    { 0x6d2fe5e3, 91 },
    { 0x7fbfe5e4, 99 },
    { 0xc8ffe5e5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c9fe5e8, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x866fe5eb, 76 },
    { 0xd73fe5eb, 55 },
    { 0, 0 },
    { 0xd42fe5ee, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb1fe5f3, 67 },
    { 0, 0 },
    { 0xb0ffe5f5, 98 },
    { 0xcbdfe5f6, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecefe5fc, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64efe602, 61 },
    { 0, 0 },
    { 0xb94fe604, 61 },
    { 0x18efe605, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dfe608, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7fe60f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x97fe612, 61 },
    { 0x642fe612, 90 },
    { 0, 0 },
    { 0x3cfe615, 33 },
    { 0x879fe615, 93 },
    { 0, 0 },
    { 0x7fefe618, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bdfe61e, 81 },
    { 0, 0 },
    { 0x87bfe620, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x951fe623, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x55bfe626, 78 },
    { 0, 0 },
    { 0x625fe628, 34 },
    { 0x425fe628, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9efe62c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3efe634, 66 },
    { 0, 0 },
    { 0xb94fe636, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdfe63c, 90 },
    { 0x701fe63d, 98 },
    { 0x4a1fe63e, 98 },
    { 0x607fe63e, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x24efe649, 55 },
    { 0x950fe64a, 64 },
    { 0x21fe649, 55 },
    { 0x738fe649, 90 },
    { 0x6d5fe64d, 98 },
    { 0, 0 },
    { 0x4c2fe64f, 78 },
    { 0x31efe650, 23 },
    { 0, 0 },
    { 0xc46fe652, 90 },
    { 0, 0 },
    { 0xffafe654, 66 },
    { 0x4e0fe654, 33 },
    { 0x6b3fe656, 61 },
    { 0x876fe657, 23 },
    { 0x484fe658, 90 },
    { 0xb51fe657, 33 },
    { 0x234fe65a, 61 },
    { 0x55fe656, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb23fe661, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xaacfe664, 93 },
    { 0, 0 },
    { 0x1befe666, 66 },
    { 0, 0 },
    { 0x11cfe668, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7dfe66f, 33 },
    { 0, 0 },
    { 0x8bdfe671, 34 },
    { 0, 0 },
    { 0xa53fe673, 90 },
    { 0, 0 },
    { 0xbbfe675, 90 },
    { 0, 0 },
    { 0x9e4fe677, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec5fe67c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x218fe687, 61 },
    { 0x9c3fe687, 95 },
    { 0x205fe689, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x695fe691, 88 },
    { 0x987fe691, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67fe695, 87 },
    { 0, 0 },
    { 0x805fe697, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce9fe69d, 85 },
    { 0x762fe69d, 84 },
    { 0x36dfe69f, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0afe6a2, 55 },
    { 0x995fe6a3, 55 },
    { 0x6effe6a4, 95 },
    { 0x240fe6a5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17efe6a9, 61 },
    { 0x2ffe6a9, 33 },
    { 0, 0 },
    { 0x57efe6ac, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402fe6b6, 95 },
    { 0, 0 },
    { 0x82bfe6b8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x743fe6bb, 93 },
    { 0x275fe6bc, 61 },
    { 0, 0 },
    { 0xf50fe6be, 85 },
    { 0, 0 },
    { 0xb06fe6c0, 76 },
    { 0x5f9fe6c0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1fe6c9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x810fe6cd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4dfe6d4, 33 },
    { 0xd40fe6d5, 34 },
    { 0x499fe6d4, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfadfe6e1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x911fe6e7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2dfe6ea, 90 },
    { 0, 0 },
    { 0xda2fe6ec, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabfe6ef, 66 },
    { 0xfadfe6f0, 42 },
    { 0x917fe6f1, 33 },
    { 0x4bafe6f0, 23 },
    { 0xae1fe6f2, 69 },
    { 0x6d3fe6f4, 95 },
    { 0x676fe6f5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x685fe6f8, 76 },
    { 0x86dfe6f9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x689fe6fd, 95 },
    { 0x566fe6fe, 100 },
    { 0x12afe6fe, 33 },
    { 0x8dfe6fd, 33 },
    { 0x3c3fe6ff, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x44efe706, 95 },
    { 0x615fe707, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b8fe70b, 85 },
    { 0x1d8fe70c, 61 },
    { 0x927fe70d, 95 },
    { 0xd75fe70e, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbe1fe711, 75 },
    { 0, 0 },
    { 0xbccfe713, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa48fe719, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6fe724, 23 },
    { 0xf71fe725, 90 },
    { 0, 0 },
    { 0xa64fe727, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ffe72a, 90 },
    { 0x82efe72a, 95 },
    { 0x765fe72c, 23 },
    { 0x8d8fe72d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f4fe734, 95 },
    { 0, 0 },
    { 0x1a3fe736, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7dfe739, 98 },
    { 0xcbdfe73a, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12bfe73e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fdfe742, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xa8afe745, 90 },
    { 0, 0 },
    { 0xa7fe747, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x297fe74b, 81 },
    { 0, 0 },
    { 0x2d7fe74d, 33 },
    { 0, 0 },
    { 0xcbefe74f, 23 },
    { 0x685fe74f, 55 },
    { 0x36afe751, 23 },
    { 0, 0 },
    { 0x54afe753, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb14fe758, 90 },
    { 0x404fe759, 61 },
    { 0x31cfe75a, 81 },
    { 0x11cfe75b, 34 },
    { 0xb63fe758, 34 },
    { 0xf7bfe75c, 34 },
    { 0xe17fe75e, 98 },
    { 0x442fe75e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0x9afe762, 87 },
    { 0, 0 },
    { 0x967fe764, 34 },
    { 0xb71fe765, 90 },
    { 0x6f0fe766, 55 },
    { 0xcedfe764, 83 },
    { 0x223fe768, 34 },
    { 0, 0 },
    { 0xf18fe76a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc99fe770, 33 },
    { 0, 0 },
    { 0xc2cfe772, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x130fe784, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x617fe788, 33 },
    { 0, 0 },
    { 0xbe3fe78a, 90 },
    { 0xfe6fe78b, 90 },
    { 0x6c0fe78c, 33 },
    { 0x214fe78c, 90 },
    { 0x334fe78e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfb8fe792, 87 },
    { 0x846fe793, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dbfe797, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x436fe79c, 90 },
    { 0, 0 },
    { 0x66afe79e, 90 },
    { 0, 0 },
    { 0x6fbfe7a0, 95 },
    { 0xefbfe7a1, 74 },
    { 0, 0 },
    { 0x81fe7a3, 91 },
    { 0x284fe7a3, 33 },
    { 0xc7cfe7a5, 98 },
    { 0x536fe7a5, 98 },
    { 0xbf8fe7a7, 95 },
    { 0x526fe7a8, 91 },
    { 0xe44fe7a8, 90 },
    { 0x697fe7aa, 61 },
    { 0x418fe7aa, 95 },
    { 0x50cfe7aa, 77 },
    { 0x42afe7ad, 90 },
    { 0xfbbfe7ae, 95 },
    { 0xcfdfe7af, 34 },
    { 0, 0 },
    { 0x24efe7b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x616fe7b9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d6fe7c1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x273fe7c6, 90 },
    { 0, 0 },
    { 0x4acfe7c8, 90 },
    { 0xf19fe7c8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ccfe7cd, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb0fe7d0, 46 },
    { 0xe78fe7d0, 90 },
    { 0x98fe7d2, 33 },
    { 0, 0 },
    { 0x121fe7d4, 69 },
    { 0x35bfe7d4, 17 },
    { 0xf23fe7d6, 34 },
    { 0xe67fe7d6, 88 },
    { 0xd87fe7d8, 33 },
    { 0xb74fe7d9, 33 },
    { 0x292fe7d6, 52 },
    { 0, 0 },
    { 0x569fe7dc, 76 },
    { 0xe15fe7dc, 85 },
    { 0, 0 },
    { 0xe62fe7df, 90 },
    { 0, 0 },
    { 0xcdefe7e1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1efe7e4, 23 },
    { 0, 0 },
    { 0x270fe7e6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172fe7f1, 90 },
    { 0x453fe7f1, 33 },
    { 0x522fe7f1, 93 },
    { 0, 0 },
    { 0x525fe7f5, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x538fe7ff, 90 },
    { 0x5bbfe7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeefe804, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd69fe80c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86dfe810, 85 },
    { 0x6a8fe811, 23 },
    { 0x118fe811, 78 },
    { 0x73fe813, 90 },
    { 0xe7afe814, 55 },
    { 0x40afe815, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cfe818, 33 },
    { 0, 0 },
    { 0x644fe81a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x27fe81e, 95 },
    { 0xc88fe81f, 84 },
    { 0xc1cfe820, 99 },
    { 0, 0 },
    { 0x36cfe822, 91 },
    { 0, 0 },
    { 0x4cafe824, 98 },
    { 0, 0 },
    { 0xb8ffe826, 34 },
    { 0x76dfe827, 61 },
    { 0x930fe827, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b2fe82d, 34 },
    { 0, 0 },
    { 0xd47fe82f, 88 },
    { 0x611fe82f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x147fe833, 84 },
    { 0, 0 },
    { 0x29fe835, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa65fe839, 52 },
    { 0x992fe83a, 61 },
    { 0, 0 },
    { 0x1b6fe83c, 78 },
    { 0, 0 },
    { 0xcfffe83e, 93 },
    { 0x18dfe83e, 95 },
    { 0x97afe840, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x154fe845, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xbaefe848, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc4fe84b, 96 },
    { 0x809fe84b, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x567fe84f, 76 },
    { 0, 0 },
    { 0xaf0fe851, 55 },
    { 0x977fe852, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xaafe855, 33 },
    { 0x394fe856, 99 },
    { 0, 0 },
    { 0x662fe858, 76 },
    { 0xe30fe859, 23 },
    { 0x339fe85a, 65 },
    { 0x610fe85a, 33 },
    { 0xe92fe859, 17 },
    { 0x3b4fe859, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x35cfe862, 23 },
    { 0x956fe863, 99 },
    { 0xf68fe864, 76 },
    { 0x45fe862, 90 },
    { 0x6bbfe866, 33 },
    { 0x3aafe866, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcedfe873, 61 },
    { 0x94afe873, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x185fe877, 34 },
    { 0, 0 },
    { 0xc40fe879, 34 },
    { 0xf46fe87a, 87 },
    { 0x9affe879, 68 },
    { 0x77afe87a, 76 },
    { 0xda8fe87d, 87 },
    { 0, 0 },
    { 0x942fe87f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fcfe883, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x565fe886, 84 },
    { 0, 0 },
    { 0x842fe888, 91 },
    { 0xa30fe889, 61 },
    { 0, 0 },
    { 0x471fe88b, 93 },
    { 0x82cfe88c, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x82bfe890, 33 },
    { 0xe86fe890, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9efe897, 5 },
    { 0, 0 },
    { 0x296fe899, 93 },
    { 0xb9fe89a, 23 },
    { 0x1b3fe89b, 101 },
    { 0x303fe89b, 52 },
    { 0x559fe89c, 81 },
    { 0x727fe89e, 33 },
    { 0x6dfe89d, 61 },
    { 0x99cfe8a0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x953fe8a7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x621fe8aa, 23 },
    { 0, 0 },
    { 0xa4cfe8ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x467fe8b0, 23 },
    { 0xcf9fe8b1, 98 },
    { 0x878fe8b0, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd44fe8b8, 34 },
    { 0x769fe8b8, 100 },
    { 0x586fe8ba, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2bfe8be, 40 },
    { 0xb43fe8be, 98 },
    { 0xa3dfe8c0, 61 },
    { 0, 0 },
    { 0x156fe8c2, 23 },
    { 0x872fe8c2, 33 },
    { 0x438fe8c4, 34 },
    { 0x1ecfe8c5, 33 },
    { 0, 0 },
    { 0x2cafe8c7, 78 },
    { 0x542fe8c8, 33 },
    { 0, 0 },
    { 0xa45fe8ca, 61 },
    { 0, 0 },
    { 0xf01fe8cc, 61 },
    { 0, 0 },
    { 0x218fe8ce, 23 },
    { 0xbcffe8cf, 95 },
    { 0xd63fe8ce, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x13efe8d6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fefe8de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xec6fe8e5, 43 },
    { 0x256fe8e6, 95 },
    { 0x75cfe8e7, 95 },
    { 0x2acfe8e8, 12 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe22fe8ee, 46 },
    { 0xd6afe8ef, 90 },
    { 0, 0 },
    { 0xd09fe8f1, 95 },
    { 0, 0 },
    { 0x50dfe8f3, 84 },
    { 0x875fe8f3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667fe8f8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4fcfe8fe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6bfe903, 61 },
    { 0x6b2fe904, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x977fe90c, 33 },
    { 0xa45fe90d, 61 },
    { 0, 0 },
    { 0xba3fe90f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x391fe912, 90 },
    { 0xc40fe912, 90 },
    { 0xb00fe914, 55 },
    { 0x1d6fe913, 55 },
    { 0xd9ffe915, 65 },
    { 0x5a4fe915, 95 },
    { 0xe32fe914, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x318fe920, 95 },
    { 0x83afe920, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe70fe928, 55 },
    { 0x288fe929, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeffe92c, 43 },
    { 0, 0 },
    { 0x4d8fe92e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13fe937, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x600fe93d, 95 },
    { 0, 0 },
    { 0xe9ffe93f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a6fe944, 66 },
    { 0, 0 },
    { 0x382fe946, 46 },
    { 0x9ebfe946, 81 },
    { 0xf6ffe947, 95 },
    { 0, 0 },
    { 0x177fe94a, 33 },
    { 0xa90fe94a, 91 },
    { 0xfc3fe94c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d9fe950, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb5fe954, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0efe959, 90 },
    { 0x413fe95a, 94 },
    { 0, 0 },
    { 0xb85fe95c, 34 },
    { 0x670fe95d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x395fe964, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fefe968, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d2fe973, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0x604fe976, 65 },
    { 0x871fe977, 90 },
    { 0xea3fe977, 99 },
    { 0xd94fe979, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0fe97d, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x860fe986, 61 },
    { 0x983fe987, 76 },
    { 0xc3afe988, 98 },
    { 0, 0 },
    { 0xe33fe98a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x57ffe98d, 23 },
    { 0xee9fe98e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d6fe998, 77 },
    { 0x202fe999, 47 },
    { 0xc83fe99a, 67 },
    { 0x133fe99b, 33 },
    { 0x3ebfe99c, 48 },
    { 0x770fe99d, 90 },
    { 0x2e8fe99c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x17ffe9a1, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2ecfe9a7, 43 },
    { 0, 0 },
    { 0x777fe9a9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xf6fe9ac, 61 },
    { 0x51fe9ac, 98 },
    { 0xe93fe9ae, 90 },
    { 0xdccfe9af, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x552fe9ba, 90 },
    { 0xbdafe9bb, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x72ffe9c3, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb66fe9c9, 82 },
    { 0x90afe9ca, 78 },
    { 0x359fe9cb, 90 },
    { 0x925fe9cc, 23 },
    { 0x698fe9c9, 43 },
    { 0x5d2fe9cd, 90 },
    { 0xa00fe9cf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cfe9d4, 61 },
    { 0x6c2fe9d5, 36 },
    { 0xea8fe9d4, 33 },
    { 0x62fe9d5, 43 },
    { 0x4bafe9d4, 61 },
    { 0x6c9fe9d7, 81 },
    { 0, 0 },
    { 0x511fe9db, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8fe9e0, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2fe9e3, 61 },
    { 0, 0 },
    { 0x4a1fe9e5, 55 },
    { 0, 0 },
    { 0x5a3fe9e7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47efe9ee, 99 },
    { 0x712fe9ee, 33 },
    { 0, 0 },
    { 0x17dfe9f1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6e4fe9f9, 76 },
    { 0x985fe9f9, 34 },
    { 0, 0 },
    { 0x9dcfe9fc, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x29efe9ff, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62fea06, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dfea09, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef8fea12, 100 },
    { 0, 0 },
    { 0xc86fea14, 33 },
    { 0x790fea15, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x558fea1d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x83efea20, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c6fea28, 96 },
    { 0, 0 },
    { 0x65ffea2a, 97 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc14fea2f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x2effea32, 23 },
    { 0x539fea33, 46 },
    { 0xfbafea34, 99 },
    { 0x97efea33, 23 },
    { 0xd4cfea36, 76 },
    { 0x157fea37, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a1fea3b, 23 },
    { 0, 0 },
    { 0x21ffea3d, 17 },
    { 0x60afea3e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e5fea42, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14fea46, 98 },
    { 0x137fea47, 77 },
    { 0x4efea47, 55 },
    { 0xb72fea49, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4abfea4c, 90 },
    { 0x9c4fea4d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbdfea52, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x216fea58, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x4adfea5b, 90 },
    { 0x269fea5c, 90 },
    { 0, 0 },
    { 0xa2ffea5e, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbefea63, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cafea6a, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x56efea6e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x496fea72, 90 },
    { 0xd4fea72, 95 },
    { 0, 0 },
    { 0xdc1fea75, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x424fea79, 61 },
    { 0xca5fea7a, 98 },
    { 0x262fea79, 43 },
    { 0, 0 },
    { 0x846fea7d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc37fea87, 95 },
    { 0xc7fea88, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9efea8b, 81 },
    { 0xeb8fea8b, 100 },
    { 0, 0 },
    { 0xe0fea8e, 90 },
    { 0x948fea8f, 90 },
    { 0xa94fea8f, 61 },
    { 0x5fafea8f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x393fea98, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedcfea9d, 90 },
    { 0xc08fea9e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdfeaac, 90 },
    { 0x722feaad, 76 },
    { 0x8ccfeaae, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa78feab3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x81cfeab7, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8feabe, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdfeac3, 23 },
    { 0xa82feac4, 23 },
    { 0x9cafeac3, 61 },
    { 0, 0 },
    { 0x98efeac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f9feaca, 87 },
    { 0xcfeacb, 93 },
    { 0x458feacb, 23 },
    { 0x431feacd, 98 },
    { 0, 0 },
    { 0x7c8feacf, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xb27fead2, 100 },
    { 0x10afead2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58feadd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x376feae2, 90 },
    { 0x559feae3, 90 },
    { 0x671feae2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7feae8, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe41feaeb, 95 },
    { 0xb0cfeaec, 43 },
    { 0xb52feaec, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xaeffeaf0, 76 },
    { 0x6b2feaf1, 67 },
    { 0x49bfeaf0, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x481feaf7, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x184feafc, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xbadfeaff, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681feb03, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4dfeb06, 35 },
    { 0xa28feb06, 90 },
    { 0, 0 },
    { 0x3acfeb09, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x671feb0c, 84 },
    { 0x9a9feb0d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x608feb14, 77 },
    { 0xb09feb14, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3bcfeb1b, 99 },
    { 0, 0 },
    { 0x810feb1d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcffeb21, 94 },
    { 0x8f2feb22, 59 },
    { 0, 0 },
    { 0xc83feb24, 98 },
    { 0, 0 },
    { 0x2fbfeb26, 43 },
    { 0x552feb27, 66 },
    { 0xca7feb27, 98 },
    { 0xce9feb29, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x494feb2d, 61 },
    { 0xe4efeb2e, 34 },
    { 0xcf1feb2f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1bfeb35, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc53feb3d, 43 },
    { 0xa87feb3d, 85 },
    { 0xbb0feb3e, 61 },
    { 0x3b3feb40, 95 },
    { 0, 0 },
    { 0x9b9feb42, 98 },
    { 0, 0 },
    { 0xb91feb44, 33 },
    { 0xf87feb45, 91 },
    { 0x8c0feb45, 76 },
    { 0xcf2feb44, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x493feb4b, 98 },
    { 0xf18feb4b, 33 },
    { 0xa2bfeb4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb89feb51, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2b9feb58, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x584feb5c, 98 },
    { 0, 0 },
    { 0xa29feb5e, 95 },
    { 0xa6efeb5e, 98 },
    { 0xec9feb60, 95 },
    { 0x99ffeb60, 33 },
    { 0xc1dfeb61, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x78ffeb65, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe2feb69, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x58efeb6f, 85 },
    { 0xa9efeb70, 84 },
    { 0x7a9feb71, 55 },
    { 0xf65feb6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ffeb7a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x868feb7d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49feb80, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ecfeb86, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6ddfeb8e, 23 },
    { 0xfe1feb8f, 84 },
    { 0x49cfeb8f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02feb93, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddfeb99, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68cfeb9d, 78 },
    { 0x785feb9e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd07feba5, 78 },
    { 0, 0 },
    { 0xaf5feba7, 39 },
    { 0xa1afeba8, 90 },
    { 0, 0 },
    { 0x173febaa, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f4febae, 66 },
    { 0, 0 },
    { 0x3a6febb0, 67 },
    { 0x545febb1, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5aefebb7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x45ffebba, 98 },
    { 0, 0 },
    { 0x342febbc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b1febc0, 33 },
    { 0, 0 },
    { 0x9e0febc2, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ffebc5, 93 },
    { 0x51efebc5, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0xf00febc9, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18efebcd, 43 },
    { 0x85ffebcd, 90 },
    { 0, 0 },
    { 0xb0cfebd0, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x15bfebd3, 66 },
    { 0xebffebd4, 90 },
    { 0, 0 },
    { 0xe5afebd6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8aafebda, 98 },
    { 0xd6afebdb, 61 },
    { 0xbf8febdc, 67 },
    { 0x857febdc, 61 },
    { 0x478febdc, 33 },
    { 0x5d8febdf, 98 },
    { 0x38efebe0, 81 },
    { 0xa19febde, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64febe4, 90 },
    { 0xdcfebe5, 77 },
    { 0x3d6febe6, 93 },
    { 0x530febe6, 34 },
    { 0xa49febe8, 88 },
    { 0, 0 },
    { 0xff2febea, 90 },
    { 0xd15febeb, 95 },
    { 0x45febec, 98 },
    { 0xe90febed, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31febf1, 90 },
    { 0, 0 },
    { 0xb72febf3, 33 },
    { 0xff5febf4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8febf7, 61 },
    { 0x587febf7, 81 },
    { 0, 0 },
    { 0x15cfebfa, 33 },
    { 0x321febfb, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c4fec0d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12cfec12, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xe10fec15, 46 },
    { 0, 0 },
    { 0xa40fec17, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acfec1e, 23 },
    { 0xc87fec1f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdccfec29, 90 },
    { 0xe91fec2a, 55 },
    { 0, 0 },
    { 0xebefec2c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ddfec2f, 95 },
    { 0x357fec2f, 93 },
    { 0x8a0fec31, 33 },
    { 0x4f4fec2f, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x402fec37, 90 },
    { 0x7fcfec38, 76 },
    { 0xab0fec39, 76 },
    { 0xefffec37, 48 },
    { 0, 0 },
    { 0xf2fec3c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dfec3f, 55 },
    { 0x980fec3f, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x888fec45, 90 },
    { 0, 0 },
    { 0xca7fec47, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x791fec4d, 66 },
    { 0x9fdfec4d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea8fec55, 90 },
    { 0xd47fec56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb07fec5a, 78 },
    { 0x8dcfec5a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3fec64, 55 },
    { 0xe20fec65, 91 },
    { 0x2a6fec66, 33 },
    { 0xb47fec66, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcb5fec6b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaecfec70, 96 },
    { 0x25efec71, 95 },
    { 0xf20fec72, 34 },
    { 0x982fec71, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f3fec78, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65fec7c, 55 },
    { 0, 0 },
    { 0x3e0fec7e, 98 },
    { 0xb4ffec7f, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2efec84, 95 },
    { 0, 0 },
    { 0x64bfec86, 90 },
    { 0xaddfec87, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x269fec8c, 34 },
    { 0xc7cfec8c, 98 },
    { 0x537fec8c, 33 },
    { 0xf9cfec8f, 23 },
    { 0x898fec90, 90 },
    { 0x2a7fec91, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd66fec96, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac6fec9a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c6feca0, 77 },
    { 0x915feca0, 55 },
    { 0, 0 },
    { 0x910feca3, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ccfeca7, 23 },
    { 0x60feca8, 66 },
    { 0x58afeca9, 68 },
    { 0x49feca7, 61 },
    { 0x956feca9, 23 },
    { 0x91afeca9, 75 },
    { 0x259fecad, 46 },
    { 0x6efecae, 67 },
    { 0xc68fecac, 100 },
    { 0xf5bfecad, 61 },
    { 0x281fecb1, 95 },
    { 0x869fecac, 23 },
    { 0xc77feca8, 66 },
    { 0xda3fecae, 87 },
    { 0x250fecad, 76 },
    { 0xbf3fecb6, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xed5fecbf, 95 },
    { 0, 0 },
    { 0xaa4fecc1, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x363fecc5, 98 },
    { 0xe3ffecc6, 23 },
    { 0x125fecc7, 23 },
    { 0xc8fecc8, 55 },
    { 0xe9afecc9, 61 },
    { 0x604fecca, 66 },
    { 0, 0 },
    { 0x26afeccc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1feccf, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x31afecd7, 90 },
    { 0x945fecd8, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x3acfecdb, 36 },
    { 0x29dfecdc, 90 },
    { 0xf09fecdb, 43 },
    { 0x311fecdd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x513fece1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bfece5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x181fece9, 55 },
    { 0xe53fece9, 52 },
    { 0x8ebfeceb, 61 },
    { 0, 0 },
    { 0xd93feced, 91 },
    { 0, 0 },
    { 0x1a5fecef, 33 },
    { 0xa95fecf0, 34 },
    { 0x95ffecf1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x24bfecf4, 52 },
    { 0x981fecf5, 95 },
    { 0xf76fecf6, 90 },
    { 0xa99fecf5, 98 },
    { 0x68afecf4, 93 },
    { 0x7e5fecf9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x118fed00, 65 },
    { 0, 0 },
    { 0x9a9fed02, 95 },
    { 0, 0 },
    { 0x9a9fed04, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb57fed08, 34 },
    { 0, 0 },
    { 0xcd1fed0a, 61 },
    { 0x5a0fed0b, 61 },
    { 0x687fed0c, 77 },
    { 0xabcfed0c, 33 },
    { 0x52bfed0d, 65 },
    { 0x291fed0c, 98 },
    { 0, 0 },
    { 0x597fed11, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x567fed1c, 33 },
    { 0x2ddfed1c, 58 },
    { 0x3ddfed1e, 66 },
    { 0, 0 },
    { 0x798fed20, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdfed23, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc4ffed27, 100 },
    { 0x7ccfed27, 43 },
    { 0xeb3fed27, 76 },
    { 0x816fed2a, 61 },
    { 0x7d9fed2b, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f6fed30, 46 },
    { 0x44fed31, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x52efed3b, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e1fed3e, 23 },
    { 0x1d1fed3e, 65 },
    { 0x5a4fed40, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x808fed45, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xeddfed48, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e6fed4e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x142fed51, 90 },
    { 0xe21fed52, 80 },
    { 0x127fed53, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb88fed5a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a2fed60, 33 },
    { 0, 0 },
    { 0xf6dfed62, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c4fed67, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9fed6f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74ffed74, 61 },
    { 0, 0 },
    { 0x8e6fed76, 55 },
    { 0x2ddfed76, 33 },
    { 0xa9fed76, 95 },
    { 0xf4dfed79, 87 },
    { 0xd5afed7a, 93 },
    { 0xf1ffed7a, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x2befed7e, 33 },
    { 0, 0 },
    { 0xdaafed80, 34 },
    { 0xf9ffed80, 61 },
    { 0x190fed81, 90 },
    { 0x5f2fed83, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x895fed87, 33 },
    { 0x712fed88, 61 },
    { 0x64efed87, 55 },
    { 0x6e9fed8a, 76 },
    { 0x603fed8b, 85 },
    { 0xb66fed8b, 67 },
    { 0x1c7fed87, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23fed92, 95 },
    { 0x197fed93, 33 },
    { 0x9cefed92, 23 },
    { 0, 0 },
    { 0x84ffed96, 33 },
    { 0xcc4fed97, 92 },
    { 0, 0 },
    { 0, 0 },
    { 0x879fed9a, 33 },
    { 0x66dfed9b, 85 },
    { 0xd2afed9c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x835feda2, 76 },
    { 0x65efeda3, 33 },
    { 0xe00feda2, 76 },
    { 0x62feda5, 90 },
    { 0, 0 },
    { 0xcd2feda7, 43 },
    { 0x393feda8, 23 },
    { 0x791feda9, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6dffedb2, 99 },
    { 0xf41fedb3, 93 },
    { 0, 0 },
    { 0x1adfedb5, 90 },
    { 0x479fedb5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x143fedbb, 84 },
    { 0, 0 },
    { 0x627fedbd, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd90fedc5, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfdafedcb, 23 },
    { 0, 0 },
    { 0xafbfedcd, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf14fedd1, 90 },
    { 0x4ebfedd2, 55 },
    { 0, 0 },
    { 0xe65fedd4, 95 },
    { 0, 0 },
    { 0x401fedd6, 85 },
    { 0, 0 },
    { 0xe5efedd8, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a3feddb, 90 },
    { 0, 0 },
    { 0xc45feddd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x56efede0, 65 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fafede3, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x583fede8, 90 },
    { 0, 0 },
    { 0x3affedea, 61 },
    { 0x1fafedea, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187fedef, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cfedf2, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcfefedf6, 81 },
    { 0xccafedf7, 97 },
    { 0xf92fedf7, 36 },
    { 0x7a5fedf9, 98 },
    { 0, 0 },
    { 0x245fedfb, 76 },
    { 0, 0 },
    { 0xa0fedfd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3e8fee01, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf8cfee07, 55 },
    { 0, 0 },
    { 0xa5bfee09, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a2fee0c, 61 },
    { 0, 0 },
    { 0x8f4fee0e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeafee14, 43 },
    { 0xf87fee15, 90 },
    { 0x327fee15, 46 },
    { 0x5b3fee17, 33 },
    { 0xbf8fee16, 100 },
    { 0x9bafee14, 33 },
    { 0, 0 },
    { 0x595fee1b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5bbfee21, 85 },
    { 0xfe2fee22, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc03fee26, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8fee29, 98 },
    { 0xdbffee29, 66 },
    { 0, 0 },
    { 0x801fee2c, 23 },
    { 0x1c5fee2d, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cfee30, 90 },
    { 0xbabfee31, 55 },
    { 0x71dfee31, 46 },
    { 0xbb5fee33, 78 },
    { 0, 0 },
    { 0xe9cfee35, 78 },
    { 0, 0 },
    { 0xb65fee37, 66 },
    { 0xba6fee38, 33 },
    { 0, 0 },
    { 0x320fee3a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd9efee3e, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0x8f9fee41, 95 },
    { 0x3c5fee41, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8bfee47, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x702fee4a, 85 },
    { 0xa99fee4a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc12fee50, 61 },
    { 0x13cfee51, 88 },
    { 0, 0 },
    { 0xae0fee53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95bfee57, 61 },
    { 0x459fee58, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5ffee64, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xdcafee67, 76 },
    { 0x911fee68, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32afee6c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb21fee70, 90 },
    { 0x827fee70, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa09fee7a, 76 },
    { 0xc09fee7b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a2fee7f, 61 },
    { 0x305fee7f, 76 },
    { 0x998fee81, 77 },
    { 0, 0 },
    { 0x19efee83, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x457fee86, 95 },
    { 0, 0 },
    { 0x64bfee88, 93 },
    { 0x8ddfee89, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x1d3fee8c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa33fee90, 33 },
    { 0, 0 },
    { 0xda2fee92, 90 },
    { 0x47dfee93, 55 },
    { 0x542fee94, 64 },
    { 0x5e9fee95, 53 },
    { 0x272fee96, 95 },
    { 0xee4fee92, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f6fee9d, 23 },
    { 0xcc7fee9d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e4feea2, 90 },
    { 0xa76feea2, 95 },
    { 0xf3efeea3, 99 },
    { 0x336feea4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddfeea9, 85 },
    { 0x5d5feea9, 75 },
    { 0x9d0feeab, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb0efeeaf, 61 },
    { 0x4afeeb0, 33 },
    { 0x185feeb1, 43 },
    { 0x549feeb1, 56 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf1efeeb6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8dcfeeb9, 33 },
    { 0xd94feeba, 78 },
    { 0x4b6feebb, 34 },
    { 0xe8feeba, 76 },
    { 0x5a1feebd, 90 },
    { 0x8e2feeb9, 90 },
    { 0, 0 },
    { 0xea8feec0, 75 },
    { 0, 0 },
    { 0xa18feec2, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e3feec9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8e3feecc, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xa3efeecf, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6bbfeed4, 61 },
    { 0xd2ffeed5, 90 },
    { 0x47feed4, 5 },
    { 0xb98feed7, 98 },
    { 0x4dcfeed8, 61 },
    { 0x9fcfeed8, 57 },
    { 0xf50feeda, 33 },
    { 0x90cfeeda, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3ffeee9, 90 },
    { 0x92efeee9, 99 },
    { 0x84afeeeb, 33 },
    { 0, 0 },
    { 0x824feeed, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x970feef0, 33 },
    { 0x94efeef0, 12 },
    { 0xb35feef2, 55 },
    { 0x627feef0, 43 },
    { 0x762feef4, 33 },
    { 0x822feef4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa6feef8, 78 },
    { 0x3b3feef9, 77 },
    { 0x9bffeef8, 33 },
    { 0x651feef8, 65 },
    { 0x741feefc, 98 },
    { 0x234feefc, 33 },
    { 0x510feef8, 95 },
    { 0, 0 },
    { 0xa4afef00, 70 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x103fef0e, 76 },
    { 0xc01fef0f, 61 },
    { 0, 0 },
    { 0x2b1fef11, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x90cfef15, 93 },
    { 0x5cbfef16, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcf8fef1c, 87 },
    { 0, 0 },
    { 0x97dfef1e, 23 },
    { 0xb8cfef1f, 95 },
    { 0x6dbfef1f, 33 },
    { 0, 0 },
    { 0xcd6fef22, 55 },
    { 0x2effef22, 90 },
    { 0, 0 },
    { 0x940fef25, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x75bfef2c, 90 },
    { 0x864fef2d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dfef32, 93 },
    { 0x428fef33, 55 },
    { 0, 0 },
    { 0x382fef35, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9ffef39, 87 },
    { 0xcf8fef3a, 61 },
    { 0xe12fef3b, 76 },
    { 0, 0 },
    { 0xe5cfef3d, 85 },
    { 0x97ffef3d, 91 },
    { 0, 0 },
    { 0xbadfef40, 43 },
    { 0xadafef40, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d7fef4a, 95 },
    { 0, 0 },
    { 0xc79fef4c, 90 },
    { 0, 0 },
    { 0xb35fef4e, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0xac4fef51, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd3fef54, 83 },
    { 0, 0 },
    { 0x563fef56, 17 },
    { 0x996fef56, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c6fef5c, 85 },
    { 0xe14fef5c, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b5fef64, 76 },
    { 0x2e0fef64, 76 },
    { 0x316fef66, 90 },
    { 0xff1fef67, 33 },
    { 0xffbfef64, 85 },
    { 0xd4afef68, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x719fef70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1fef73, 56 },
    { 0xa99fef73, 33 },
    { 0x2dfef75, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e3fef78, 98 },
    { 0, 0 },
    { 0xfb2fef7a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x728fef7d, 41 },
    { 0xc7efef7e, 93 },
    { 0xa0dfef7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xc28fef82, 52 },
    { 0x69dfef83, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38dfef88, 65 },
    { 0, 0 },
    { 0xfabfef8a, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf06fef8f, 61 },
    { 0, 0 },
    { 0x11ffef91, 76 },
    { 0x300fef92, 23 },
    { 0x8a1fef91, 69 },
    { 0x9d8fef92, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x637fefa1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x553fefa9, 61 },
    { 0, 0 },
    { 0x89fefab, 90 },
    { 0x918fefac, 90 },
    { 0xecfefad, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xee5fefb1, 78 },
    { 0x763fefb2, 67 },
    { 0x1abfefb2, 69 },
    { 0x695fefb3, 61 },
    { 0xa87fefb5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47fefbb, 95 },
    { 0, 0 },
    { 0x4bcfefbd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x9bffefc0, 95 },
    { 0x314fefc1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3fdfefc5, 95 },
    { 0x5cffefc5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x74cfefce, 66 },
    { 0x23afefce, 55 },
    { 0x83ffefce, 90 },
    { 0x151fefd1, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x3befefd4, 90 },
    { 0xe58fefd5, 64 },
    { 0x907fefd6, 99 },
    { 0x18fefd5, 33 },
    { 0, 0 },
    { 0xae7fefd9, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x794fefe4, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0x12ffefe7, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x261fefed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x438feff1, 40 },
    { 0x1a0feff1, 90 },
    { 0x902feff3, 19 },
    { 0x3f2feff3, 78 },
    { 0x22afeff5, 78 },
    { 0, 0 },
    { 0x830feff7, 23 },
    { 0x92bfeff7, 61 },
    { 0x78ffeff9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x47eff000, 98 },
    { 0, 0 },
    { 0x28ff002, 33 },
    { 0x9e4ff002, 61 },
    { 0x4abff002, 77 },
    { 0x295ff005, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x95eff009, 85 },
    { 0xd8dff00a, 98 },
    { 0x481ff00b, 17 },
    { 0xbe6ff00c, 90 },
    { 0x267ff00c, 100 },
    { 0x942ff00e, 98 },
    { 0x15ff00f, 90 },
    { 0x6c4ff010, 100 },
    { 0xc39ff011, 33 },
    { 0x3beff00e, 33 },
    { 0xa24ff011, 34 },
    { 0xf0aff014, 95 },
    { 0x3e9ff014, 76 },
    { 0xf35ff016, 90 },
    { 0xb1cff017, 61 },
    { 0xe83ff018, 76 },
    { 0x1a3ff018, 91 },
    { 0x517ff01a, 77 },
    { 0xc72ff01b, 90 },
    { 0xc01ff01b, 81 },
    { 0x282ff01d, 78 },
    { 0x750ff01d, 23 },
    { 0x593ff00f, 100 },
    { 0x432ff020, 90 },
    { 0xbc8ff021, 86 },
    { 0x96bff01d, 91 },
    { 0x60bff01b, 55 },
    { 0x567ff017, 61 },
    { 0x443ff025, 90 },
    { 0xec8ff026, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x12fff02a, 95 },
    { 0, 0 },
    { 0xaeeff02c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fff030, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x460ff03a, 38 },
    { 0xddcff03b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfc9ff043, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xf17ff046, 93 },
    { 0xed7ff046, 90 },
    { 0, 0 },
    { 0xcfff049, 88 },
    { 0, 0 },
    { 0x460ff04b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2bdff04f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x277ff053, 98 },
    { 0xe38ff054, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfff057, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dff05c, 46 },
    { 0x96aff05c, 95 },
    { 0x4e1ff05c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb99ff066, 95 },
    { 0x4a5ff066, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe87ff06b, 81 },
    { 0x79dff06c, 98 },
    { 0x83ff06d, 84 },
    { 0, 0 },
    { 0x6d6ff06f, 90 },
    { 0, 0 },
    { 0xa44ff071, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0xfa0ff074, 61 },
    { 0x77fff075, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x611ff078, 61 },
    { 0x7a3ff079, 55 },
    { 0xe07ff07a, 23 },
    { 0xdf9ff07b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7ff07f, 55 },
    { 0xe37ff07f, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x58ff083, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ff08b, 66 },
    { 0x572ff08c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x83cff08f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7bcff092, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d3ff096, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbcff099, 90 },
    { 0xc3cff09a, 90 },
    { 0x129ff09b, 78 },
    { 0x658ff09c, 46 },
    { 0x28ff09c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa58ff0a6, 33 },
    { 0x2c3ff0a7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a5ff0ab, 78 },
    { 0x823ff0ac, 98 },
    { 0xb3bff0ad, 33 },
    { 0xd7cff0ad, 90 },
    { 0x479ff0af, 98 },
    { 0, 0 },
    { 0x878ff0b1, 98 },
    { 0xa76ff0b2, 91 },
    { 0xe62ff0b1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb97ff0b8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedfff0c2, 43 },
    { 0x56bff0c2, 99 },
    { 0x85aff0c2, 75 },
    { 0xbe4ff0c4, 76 },
    { 0x1ebff0c6, 98 },
    { 0x96bff0c6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x187ff0cb, 67 },
    { 0x745ff0cc, 23 },
    { 0x37bff0cd, 95 },
    { 0xac7ff0ce, 90 },
    { 0xf5eff0cc, 43 },
    { 0x251ff0d0, 55 },
    { 0, 0 },
    { 0xaf5ff0d2, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0x2aff0d5, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5d9ff0d9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x3feff0dc, 76 },
    { 0xe5fff0dd, 33 },
    { 0x638ff0dc, 90 },
    { 0, 0 },
    { 0xf61ff0e0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e0ff0e4, 33 },
    { 0x305ff0e4, 84 },
    { 0x829ff0e6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88ff0ea, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb77ff0ee, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0x59cff0f1, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xea3ff0f8, 17 },
    { 0xf2dff0f9, 23 },
    { 0xb4ff0fa, 33 },
    { 0x8d2ff0fb, 85 },
    { 0x841ff0fb, 93 },
    { 0x13eff0fc, 33 },
    { 0xdfaff0fa, 90 },
    { 0x580ff0ff, 33 },
    { 0x75bff100, 68 },
    { 0x290ff101, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x136ff105, 95 },
    { 0x41fff106, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f6ff109, 100 },
    { 0x1e7ff10a, 33 },
    { 0xeb6ff10a, 36 },
    { 0xa4cff10a, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x122ff116, 62 },
    { 0x162ff117, 12 },
    { 0xb18ff117, 90 },
    { 0x8a1ff117, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e3ff11c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ff122, 34 },
    { 0x9c8ff123, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x21eff12b, 23 },
    { 0x42aff12b, 55 },
    { 0, 0 },
    { 0xae4ff12e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c7ff131, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x307ff137, 43 },
    { 0x4abff138, 23 },
    { 0xcf8ff139, 55 },
    { 0xb51ff13a, 34 },
    { 0x487ff13b, 85 },
    { 0xdb1ff13c, 23 },
    { 0x177ff13a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa1ff149, 98 },
    { 0x738ff149, 38 },
    { 0xf8ff14b, 23 },
    { 0, 0 },
    { 0x1a4ff14d, 33 },
    { 0, 0 },
    { 0xd5cff14f, 55 },
    { 0x88cff150, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9bff155, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ccff15b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x886ff15e, 34 },
    { 0xdd4ff15f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb4ff167, 99 },
    { 0, 0 },
    { 0x368ff169, 100 },
    { 0x30dff16a, 61 },
    { 0, 0 },
    { 0xf99ff16c, 98 },
    { 0xbd3ff16d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3fff174, 90 },
    { 0x2bdff175, 83 },
    { 0, 0 },
    { 0, 0 },
    { 0x455ff178, 95 },
    { 0x4bff179, 98 },
    { 0, 0 },
    { 0xf5eff17b, 95 },
    { 0x728ff17c, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x834ff17f, 61 },
    { 0xbaff180, 98 },
    { 0xe36ff181, 84 },
    { 0x5d0ff182, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7b7ff186, 36 },
    { 0x495ff186, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f5ff18a, 23 },
    { 0xd80ff18b, 90 },
    { 0xc64ff18c, 98 },
    { 0, 0 },
    { 0x6b6ff18e, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe90ff191, 95 },
    { 0, 0 },
    { 0xf19ff193, 81 },
    { 0, 0 },
    { 0xcb5ff195, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x926ff19c, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0x7adff19f, 90 },
    { 0xac3ff19f, 95 },
    { 0xef9ff1a1, 98 },
    { 0x667ff1a2, 76 },
    { 0x13aff1a1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xf05ff1a6, 23 },
    { 0, 0 },
    { 0xe43ff1a8, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99ff1ac, 81 },
    { 0xb4ff1ad, 33 },
    { 0xc7aff1ad, 34 },
    { 0, 0 },
    { 0xaf1ff1b0, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ff1b3, 33 },
    { 0xc91ff1b4, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd9ff1b8, 33 },
    { 0xe95ff1b9, 88 },
    { 0x6d9ff1ba, 24 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6a9ff1bf, 85 },
    { 0, 0 },
    { 0xaf0ff1c1, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e5ff1c6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x400ff1ca, 61 },
    { 0x6eeff1ca, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaa2ff1d0, 61 },
    { 0x492ff1d1, 95 },
    { 0, 0 },
    { 0xf6bff1d3, 52 },
    { 0x940ff1d3, 90 },
    { 0xba5ff1d5, 81 },
    { 0, 0 },
    { 0x9f1ff1d7, 91 },
    { 0xbd7ff1d8, 93 },
    { 0x85dff1d8, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4c7ff1de, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8deff1ed, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e4ff1f2, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa9fff1fb, 77 },
    { 0xa96ff1fc, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x32cff200, 55 },
    { 0, 0 },
    { 0x879ff202, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc5eff205, 76 },
    { 0x9e1ff206, 76 },
    { 0, 0 },
    { 0x73aff208, 93 },
    { 0, 0 },
    { 0xa70ff20a, 23 },
    { 0, 0 },
    { 0xeb3ff20c, 61 },
    { 0, 0 },
    { 0x1c0ff20e, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x157ff214, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x739ff217, 84 },
    { 0x26aff218, 33 },
    { 0x39aff219, 93 },
    { 0xf8aff218, 90 },
    { 0xfb5ff21b, 90 },
    { 0xcd9ff219, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x658ff221, 33 },
    { 0x9e7ff222, 65 },
    { 0x625ff222, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8aff226, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe6aff22c, 33 },
    { 0, 0 },
    { 0x9e0ff22e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb86ff235, 78 },
    { 0x9edff236, 74 },
    { 0, 0 },
    { 0xe57ff238, 90 },
    { 0xc1aff239, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd91ff23d, 78 },
    { 0xee1ff23e, 43 },
    { 0x8c8ff23d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x902ff243, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0x1aaff246, 90 },
    { 0, 0 },
    { 0x426ff248, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xf67ff24b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd97ff253, 23 },
    { 0x76bff253, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3cff25b, 43 },
    { 0x5c2ff25c, 33 },
    { 0, 0 },
    { 0x55bff25e, 33 },
    { 0xc63ff25f, 100 },
    { 0x85cff25f, 95 },
    { 0x6c9ff25f, 55 },
    { 0xcd5ff262, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x222ff265, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0x27aff268, 65 },
    { 0x3d9ff269, 78 },
    { 0x10cff26a, 100 },
    { 0x276ff26b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfff271, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4edff279, 61 },
    { 0xb18ff27a, 76 },
    { 0, 0 },
    { 0x53cff27c, 34 },
    { 0x318ff27d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x6eff280, 61 },
    { 0xd75ff280, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f3ff284, 90 },
    { 0, 0 },
    { 0x6caff286, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43ff28a, 34 },
    { 0x717ff28b, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa87ff294, 33 },
    { 0, 0 },
    { 0x3aeff296, 98 },
    { 0x262ff297, 78 },
    { 0x3c3ff298, 66 },
    { 0xb2aff296, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9aeff29e, 76 },
    { 0x52ff29f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbdff2a8, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x993ff2b1, 33 },
    { 0x413ff2b2, 90 },
    { 0x862ff2b2, 90 },
    { 0x24dff2b4, 23 },
    { 0, 0 },
    { 0x1c5ff2b6, 67 },
    { 0xe85ff2b6, 91 },
    { 0xaa9ff2b8, 23 },
    { 0xd9fff2b8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4eff2bc, 95 },
    { 0xfcbff2bd, 90 },
    { 0, 0 },
    { 0x49dff2bf, 95 },
    { 0x56cff2c0, 98 },
    { 0xcc8ff2c0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dff2c4, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0xc81ff2c7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b6ff2cc, 95 },
    { 0xd55ff2cd, 90 },
    { 0xb2ff2cd, 90 },
    { 0xcc5ff2cd, 33 },
    { 0x476ff2ce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7f1ff2d5, 98 },
    { 0, 0 },
    { 0x9a4ff2d7, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x624ff2dc, 33 },
    { 0, 0 },
    { 0x22eff2de, 95 },
    { 0x8e8ff2df, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ff2e3, 98 },
    { 0, 0 },
    { 0xbd2ff2e5, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaff2e9, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd1ff2ec, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6ff2f4, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa94ff2fa, 90 },
    { 0x121ff2fb, 61 },
    { 0, 0 },
    { 0x988ff2fd, 77 },
    { 0xb85ff2fd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xc94ff301, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eff305, 86 },
    { 0x12fff306, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4e9ff30b, 46 },
    { 0, 0 },
    { 0xd89ff30d, 91 },
    { 0xd61ff30d, 37 },
    { 0x677ff30f, 34 },
    { 0xe41ff310, 34 },
    { 0, 0 },
    { 0xe34ff312, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x331ff316, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x226ff319, 34 },
    { 0xdedff319, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x931ff320, 34 },
    { 0, 0 },
    { 0xb7cff322, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2fff326, 90 },
    { 0xb35ff326, 55 },
    { 0x46bff328, 95 },
    { 0x705ff329, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x724ff32d, 33 },
    { 0xc34ff32d, 95 },
    { 0x530ff32f, 98 },
    { 0x6f6ff32f, 33 },
    { 0x3e9ff331, 99 },
    { 0xc15ff331, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x568ff336, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8eff339, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd16ff344, 80 },
    { 0, 0 },
    { 0, 0 },
    { 0xa26ff347, 46 },
    { 0x691ff348, 17 },
    { 0x294ff347, 61 },
    { 0x727ff34a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaabff350, 91 },
    { 0, 0 },
    { 0x108ff352, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8edff355, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xae4ff35a, 98 },
    { 0x996ff35b, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbbfff360, 61 },
    { 0x912ff361, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x756ff365, 76 },
    { 0xb04ff366, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3f7ff36b, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0x739ff36e, 61 },
    { 0xd70ff36f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x679ff375, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x5fff378, 35 },
    { 0, 0 },
    { 0xea6ff37a, 42 },
    { 0xb41ff37a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x14ff37e, 95 },
    { 0xf1cff37e, 76 },
    { 0xc81ff380, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bdff383, 23 },
    { 0, 0 },
    { 0x4cff385, 99 },
    { 0x6d1ff386, 90 },
    { 0, 0 },
    { 0x7aaff388, 87 },
    { 0xc0dff389, 61 },
    { 0, 0 },
    { 0xd6bff38b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x6f0ff38e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6aff392, 98 },
    { 0, 0 },
    { 0x9dcff394, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x693ff399, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe18ff39d, 33 },
    { 0xaa8ff39e, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb6ff3a3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x787ff3a6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x874ff3aa, 91 },
    { 0xe35ff3aa, 61 },
    { 0, 0 },
    { 0x21aff3ad, 76 },
    { 0xddbff3ae, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0x4b6ff3b1, 90 },
    { 0x4edff3b1, 52 },
    { 0xd0eff3b3, 84 },
    { 0x22fff3b4, 34 },
    { 0, 0 },
    { 0xf61ff3b6, 33 },
    { 0x375ff3b6, 34 },
    { 0x485ff3b8, 76 },
    { 0x86cff3b9, 87 },
    { 0x8b9ff3b9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5deff3be, 34 },
    { 0xe79ff3bf, 33 },
    { 0x659ff3c0, 55 },
    { 0, 0 },
    { 0x1ffff3c2, 86 },
    { 0x45aff3c2, 23 },
    { 0x7a3ff3c4, 67 },
    { 0x3d7ff3c5, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x256ff3ca, 90 },
    { 0, 0 },
    { 0x5b2ff3cc, 61 },
    { 0, 0 },
    { 0xaeaff3ce, 76 },
    { 0, 0 },
    { 0x19dff3d0, 8 },
    { 0, 0 },
    { 0, 0 },
    { 0x591ff3d3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x127ff3d6, 95 },
    { 0x378ff3d7, 93 },
    { 0x4e3ff3d7, 55 },
    { 0xed1ff3d9, 93 },
    { 0xa05ff3d6, 95 },
    { 0x9d8ff3db, 23 },
    { 0x656ff3d7, 34 },
    { 0xda8ff3db, 95 },
    { 0xeefff3de, 90 },
    { 0xb90ff3df, 98 },
    { 0x1a8ff3de, 36 },
    { 0x25cff3e1, 61 },
    { 0, 0 },
    { 0xeceff3e3, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x500ff3e9, 36 },
    { 0x453ff3ea, 98 },
    { 0xf77ff3e9, 90 },
    { 0xb14ff3ec, 101 },
    { 0xdf2ff3e9, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa7eff3f1, 24 },
    { 0, 0 },
    { 0xccff3f3, 33 },
    { 0xde9ff3f4, 61 },
    { 0, 0 },
    { 0x6a7ff3f6, 34 },
    { 0, 0 },
    { 0xf3bff3f8, 80 },
    { 0x43dff3f9, 99 },
    { 0x9ebff3f8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b1ff400, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe61ff40b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377ff40f, 23 },
    { 0xa13ff410, 23 },
    { 0, 0 },
    { 0x6b4ff412, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343ff41a, 34 },
    { 0, 0 },
    { 0x34dff41c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x562ff420, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xc23ff423, 95 },
    { 0xe79ff423, 55 },
    { 0x3fdff425, 33 },
    { 0x2bcff426, 61 },
    { 0x33eff423, 23 },
    { 0x6c8ff427, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0x80fff42b, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd6ff42e, 95 },
    { 0x176ff42e, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xec7ff432, 33 },
    { 0xd4dff432, 52 },
    { 0xa32ff434, 33 },
    { 0, 0 },
    { 0xffbff436, 52 },
    { 0xb89ff437, 96 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffaff43c, 90 },
    { 0xd69ff43d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd88ff441, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4afff446, 98 },
    { 0x7fff447, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe04ff453, 96 },
    { 0x25eff453, 95 },
    { 0xfafff454, 55 },
    { 0xacfff455, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x738ff45b, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4fff465, 90 },
    { 0xc0ff466, 67 },
    { 0xf5cff466, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x88dff46c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1c4ff471, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x5ddff474, 95 },
    { 0x16ff475, 55 },
    { 0x199ff476, 61 },
    { 0x142ff477, 46 },
    { 0x609ff474, 95 },
    { 0x5cff479, 76 },
    { 0, 0 },
    { 0x300ff47b, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6edff47f, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x18fff482, 33 },
    { 0x2edff483, 55 },
    { 0xf21ff484, 90 },
    { 0x763ff485, 90 },
    { 0xa9fff486, 33 },
    { 0x8daff486, 90 },
    { 0x737ff486, 66 },
    { 0, 0 },
    { 0x9efff48a, 55 },
    { 0x637ff48b, 90 },
    { 0xf93ff48c, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x740ff492, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb31ff497, 98 },
    { 0x7acff498, 23 },
    { 0, 0 },
    { 0xb4eff49a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd31ff49e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc33ff4a2, 98 },
    { 0xc49ff4a3, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc7dff4b5, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x361ff4b8, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe43ff4bd, 90 },
    { 0x92ff4bd, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0x985ff4c1, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xecdff4c6, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x9cdff4c9, 98 },
    { 0, 0 },
    { 0x816ff4cb, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3d4ff4d1, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc90ff4d9, 66 },
    { 0xfeaff4da, 61 },
    { 0xc3cff4da, 61 },
    { 0x927ff4dc, 95 },
    { 0x1cfff4dd, 23 },
    { 0, 0 },
    { 0xae1ff4df, 39 },
    { 0x4a0ff4e0, 55 },
    { 0, 0 },
    { 0xb19ff4e2, 23 },
    { 0, 0 },
    { 0x805ff4e4, 90 },
    { 0xa1bff4e4, 90 },
    { 0, 0 },
    { 0xa21ff4e7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb46ff4ed, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdacff4f3, 95 },
    { 0, 0 },
    { 0xa34ff4f5, 61 },
    { 0, 0 },
    { 0xbacff4f7, 61 },
    { 0, 0 },
    { 0x199ff4f9, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0ff4fc, 15 },
    { 0, 0 },
    { 0xeeaff4fe, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x464ff509, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xcccff50c, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0xe0ff50f, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x872ff512, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x690ff517, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x22ff51b, 61 },
    { 0xf5aff51b, 56 },
    { 0xb8fff51d, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x85cff520, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e0ff523, 76 },
    { 0xebdff523, 61 },
    { 0xdc6ff524, 95 },
    { 0xdfcff523, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9eff52a, 67 },
    { 0x906ff52b, 61 },
    { 0x597ff52c, 53 },
    { 0, 0 },
    { 0xf36ff52e, 90 },
    { 0, 0 },
    { 0xdacff530, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd78ff53a, 36 },
    { 0xbd3ff53a, 90 },
    { 0xa73ff53c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x900ff54a, 61 },
    { 0xcb2ff54b, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x41aff551, 93 },
    { 0x287ff551, 95 },
    { 0xbd8ff553, 33 },
    { 0, 0 },
    { 0x647ff555, 33 },
    { 0xab9ff556, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf03ff566, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ff571, 34 },
    { 0x3a2ff572, 55 },
    { 0xeaeff571, 61 },
    { 0x1e3ff572, 55 },
    { 0xd87ff575, 95 },
    { 0xcd7ff575, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x847ff57d, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8b0ff581, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bff586, 88 },
    { 0xc34ff587, 46 },
    { 0xc18ff587, 61 },
    { 0xe06ff587, 34 },
    { 0xf37ff58a, 36 },
    { 0xc92ff586, 90 },
    { 0x3aff58c, 68 },
    { 0xa8dff58a, 90 },
    { 0x80cff58c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x354ff596, 90 },
    { 0x957ff597, 90 },
    { 0x306ff596, 90 },
    { 0x44bff598, 95 },
    { 0xa1dff598, 99 },
    { 0x3d4ff596, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11aff5a9, 95 },
    { 0x3aeff5aa, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf0ff5ae, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x64dff5b6, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x486ff5ba, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x922ff5be, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa57ff5c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd72ff5ce, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd84ff5d7, 90 },
    { 0x520ff5d7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xf24ff5db, 59 },
    { 0x73dff5db, 90 },
    { 0x4fbff5dc, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc3aff5e6, 90 },
    { 0x57ff5e7, 90 },
    { 0x4c4ff5e8, 61 },
    { 0x494ff5e6, 43 },
    { 0xf89ff5e6, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836ff5ef, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x76bff5f4, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffff5fb, 76 },
    { 0x307ff5fb, 98 },
    { 0, 0 },
    { 0x533ff5fe, 23 },
    { 0, 0 },
    { 0x3b1ff600, 33 },
    { 0, 0 },
    { 0x19bff602, 43 },
    { 0x57fff603, 93 },
    { 0x92cff603, 61 },
    { 0x763ff604, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x60bff608, 78 },
    { 0xad7ff609, 55 },
    { 0x188ff608, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0x440ff60d, 61 },
    { 0x491ff60e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xfedff611, 33 },
    { 0, 0 },
    { 0x2d9ff613, 85 },
    { 0xdcdff614, 66 },
    { 0xb63ff613, 23 },
    { 0xfb6ff616, 95 },
    { 0xa5ff613, 23 },
    { 0, 0 },
    { 0xb69ff619, 87 },
    { 0xf9cff619, 98 },
    { 0xd1fff61b, 98 },
    { 0xe14ff61b, 23 },
    { 0x28bff61c, 93 },
    { 0x57bff61d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe89ff624, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcc4ff62c, 68 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdbbff630, 91 },
    { 0x8d1ff631, 90 },
    { 0x6d1ff630, 37 },
    { 0x2b4ff633, 34 },
    { 0xf1dff634, 93 },
    { 0, 0 },
    { 0x44ff636, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x873ff639, 95 },
    { 0x5b0ff63a, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xc1cff63d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8fff643, 81 },
    { 0x190ff644, 52 },
    { 0x947ff644, 78 },
    { 0x381ff646, 88 },
    { 0xab9ff647, 34 },
    { 0xfccff646, 90 },
    { 0xd3aff649, 95 },
    { 0x9bcff644, 95 },
    { 0xf75ff64b, 23 },
    { 0x94bff643, 65 },
    { 0x4b4ff645, 95 },
    { 0xbe1ff64c, 66 },
    { 0, 0 },
    { 0xe02ff650, 98 },
    { 0x942ff650, 76 },
    { 0x94dff651, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x375ff656, 33 },
    { 0, 0 },
    { 0x230ff658, 61 },
    { 0, 0 },
    { 0x33dff65a, 90 },
    { 0xaa5ff65a, 61 },
    { 0, 0 },
    { 0x9b9ff65d, 97 },
    { 0, 0 },
    { 0x792ff65f, 90 },
    { 0xd15ff660, 76 },
    { 0xd1eff661, 23 },
    { 0, 0 },
    { 0x775ff663, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x50ff666, 61 },
    { 0xef9ff666, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x84dff66d, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe99ff672, 61 },
    { 0x7aff673, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x113ff676, 46 },
    { 0x4b0ff677, 23 },
    { 0, 0 },
    { 0xfebff679, 67 },
    { 0xbddff67a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef4ff67e, 34 },
    { 0x17cff67e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6aff682, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bff685, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd7aff68b, 91 },
    { 0, 0 },
    { 0x675ff68d, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x63bff694, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xad9ff69a, 90 },
    { 0x6f2ff69b, 98 },
    { 0x8f6ff69c, 98 },
    { 0x633ff69d, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x83ff6aa, 46 },
    { 0x426ff6ab, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68ff6af, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x160ff6b3, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x990ff6b8, 95 },
    { 0xfe1ff6b8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9aff6bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x936ff6c5, 46 },
    { 0x70ff6c6, 81 },
    { 0x5a8ff6c5, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa5ff6d3, 99 },
    { 0xb9aff6d4, 43 },
    { 0x905ff6d5, 61 },
    { 0xe5eff6d6, 67 },
    { 0x5c8ff6d7, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11eff6dd, 33 },
    { 0xc7bff6de, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x29bff6e1, 43 },
    { 0, 0 },
    { 0xea0ff6e3, 90 },
    { 0x31aff6e4, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe68ff6ea, 61 },
    { 0, 0 },
    { 0x2fcff6ec, 90 },
    { 0xfbbff6ec, 93 },
    { 0xa2fff6ee, 52 },
    { 0, 0 },
    { 0x943ff6f0, 33 },
    { 0, 0 },
    { 0x66ff6f2, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc6cff6f8, 61 },
    { 0x98dff6f9, 90 },
    { 0, 0 },
    { 0x4dfff6fb, 36 },
    { 0xde9ff6fc, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x315ff701, 33 },
    { 0x49cff702, 61 },
    { 0, 0 },
    { 0x37cff704, 55 },
    { 0, 0 },
    { 0x197ff706, 33 },
    { 0x9ffff706, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xdaaff70a, 99 },
    { 0, 0 },
    { 0xea5ff70c, 61 },
    { 0x70fff70d, 55 },
    { 0x7faff70e, 34 },
    { 0x3d0ff70f, 100 },
    { 0x2bcff710, 43 },
    { 0x8f1ff711, 77 },
    { 0x69ff711, 77 },
    { 0x3b6ff70f, 90 },
    { 0xbcff714, 90 },
    { 0xd6bff70d, 33 },
    { 0x14eff716, 98 },
    { 0, 0 },
    { 0x401ff718, 61 },
    { 0x6faff718, 61 },
    { 0xb6fff71a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x606ff71d, 61 },
    { 0x678ff71e, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x621ff722, 90 },
    { 0x674ff723, 33 },
    { 0xf0cff724, 91 },
    { 0x3deff725, 90 },
    { 0xaeff726, 61 },
    { 0xadbff724, 100 },
    { 0, 0 },
    { 0x2acff729, 66 },
    { 0x8e6ff72a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x54cff72e, 87 },
    { 0x305ff72e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa47ff734, 100 },
    { 0, 0 },
    { 0x5dff736, 33 },
    { 0, 0 },
    { 0x3adff738, 33 },
    { 0x26fff739, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdedff73d, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x39dff742, 97 },
    { 0, 0 },
    { 0xf85ff744, 61 },
    { 0, 0 },
    { 0xe76ff746, 67 },
    { 0x1adff746, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x420ff74d, 90 },
    { 0x40cff74e, 76 },
    { 0x72fff74e, 81 },
    { 0x584ff750, 90 },
    { 0, 0 },
    { 0xc28ff752, 90 },
    { 0x22ff753, 43 },
    { 0, 0 },
    { 0x8c6ff755, 67 },
    { 0x12ff756, 95 },
    { 0, 0 },
    { 0xffdff758, 98 },
    { 0xd2aff759, 55 },
    { 0xbb0ff759, 61 },
    { 0, 0 },
    { 0xc6ff75c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x62bff75f, 90 },
    { 0, 0 },
    { 0xfe0ff761, 61 },
    { 0x633ff761, 12 },
    { 0, 0 },
    { 0xc45ff764, 76 },
    { 0xa65ff764, 90 },
    { 0xadeff766, 43 },
    { 0xeb1ff766, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb1ff76c, 98 },
    { 0xf6bff76c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaff770, 33 },
    { 0x97dff771, 90 },
    { 0xd00ff772, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe3eff777, 90 },
    { 0x344ff778, 77 },
    { 0, 0 },
    { 0xef0ff77a, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25cff781, 33 },
    { 0, 0 },
    { 0xb49ff783, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2c7ff786, 95 },
    { 0, 0 },
    { 0x1abff788, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc0ff78d, 55 },
    { 0x67dff78e, 33 },
    { 0xdd7ff78d, 98 },
    { 0x85ff790, 67 },
    { 0x364ff791, 46 },
    { 0x8ffff792, 76 },
    { 0x215ff792, 76 },
    { 0xbdaff791, 67 },
    { 0x765ff793, 95 },
    { 0x2f3ff793, 87 },
    { 0xbc5ff793, 33 },
    { 0x410ff792, 34 },
    { 0x2c7ff799, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4eaff7a6, 90 },
    { 0x187ff7a7, 90 },
    { 0x8d4ff7a8, 90 },
    { 0x5dcff7a7, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xce7ff7ac, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x11fff7b1, 98 },
    { 0xe8aff7b2, 33 },
    { 0x303ff7b1, 64 },
    { 0, 0 },
    { 0x934ff7b5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb8cff7c1, 91 },
    { 0x444ff7c2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe09ff7ca, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa91ff7d0, 23 },
    { 0x2cff7d1, 100 },
    { 0xf9dff7d2, 88 },
    { 0x2b1ff7d3, 66 },
    { 0xeb7ff7d4, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x811ff7dd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x3efff7e0, 90 },
    { 0, 0 },
    { 0x365ff7e2, 90 },
    { 0x5a2ff7e2, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x237ff7e8, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x601ff7f3, 76 },
    { 0x184ff7f4, 95 },
    { 0x557ff7f5, 61 },
    { 0x92fff7f5, 34 },
    { 0xef8ff7f7, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe96ff7fe, 78 },
    { 0xaafff7ff, 61 },
    { 0xfff7ff, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65bff804, 33 },
    { 0x27dff805, 76 },
    { 0, 0 },
    { 0xba3ff807, 95 },
    { 0x580ff808, 74 },
    { 0x22bff809, 88 },
    { 0x649ff808, 78 },
    { 0x43ff809, 100 },
    { 0xc04ff80c, 61 },
    { 0x2b1ff80d, 90 },
    { 0x1d2ff80d, 93 },
    { 0x11bff80c, 85 },
    { 0x8fbff80c, 33 },
    { 0xce4ff80d, 90 },
    { 0xf31ff811, 33 },
    { 0x2ccff813, 66 },
    { 0xccfff814, 98 },
    { 0xb69ff815, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x373ff81b, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x20ff820, 90 },
    { 0, 0 },
    { 0xc60ff822, 66 },
    { 0, 0 },
    { 0x785ff824, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb39ff82b, 46 },
    { 0xd8ff82b, 78 },
    { 0x479ff82b, 98 },
    { 0x41fff82e, 46 },
    { 0x52ff82e, 52 },
    { 0, 0 },
    { 0xc33ff831, 46 },
    { 0, 0 },
    { 0xd1bff833, 33 },
    { 0, 0 },
    { 0x9aeff835, 98 },
    { 0xa00ff835, 98 },
    { 0x406ff836, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42fff83c, 33 },
    { 0xcb3ff83c, 75 },
    { 0x89ff83e, 84 },
    { 0x7dcff83f, 83 },
    { 0x1eeff840, 33 },
    { 0xbe3ff841, 61 },
    { 0xc89ff83e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc87ff84c, 90 },
    { 0, 0 },
    { 0xf48ff84e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x66cff851, 88 },
    { 0xeccff851, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdc2ff856, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4baff860, 67 },
    { 0x77dff860, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a4ff864, 85 },
    { 0x9fdff865, 61 },
    { 0x759ff866, 95 },
    { 0xb68ff864, 100 },
    { 0x367ff868, 23 },
    { 0, 0 },
    { 0xac9ff86a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x540ff86d, 93 },
    { 0x8afff86e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4ff872, 90 },
    { 0x3ceff872, 87 },
    { 0x25ff874, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x7dfff877, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ecff882, 87 },
    { 0xadff882, 95 },
    { 0x35ff884, 55 },
    { 0x1eaff883, 46 },
    { 0x39eff886, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x42cff88c, 76 },
    { 0x2ceff88d, 43 },
    { 0xc6dff88d, 55 },
    { 0, 0 },
    { 0xa1bff890, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x377ff895, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xbb9ff898, 91 },
    { 0xe7dff899, 46 },
    { 0xc3cff89a, 40 },
    { 0x778ff89a, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xa14ff89e, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcaaff8a6, 100 },
    { 0x748ff8a6, 66 },
    { 0x5a4ff8a7, 61 },
    { 0x7dff8a9, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2ff8ac, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf18ff8af, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x6c3ff8b5, 69 },
    { 0, 0 },
    { 0xabfff8b7, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfeaff8bb, 90 },
    { 0x4e0ff8bb, 52 },
    { 0x508ff8bd, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf40ff8c1, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x333ff8c8, 98 },
    { 0x55eff8c8, 33 },
    { 0, 0 },
    { 0x4bfff8cb, 33 },
    { 0, 0 },
    { 0x8d0ff8cd, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2a4ff8d2, 98 },
    { 0x73fff8d3, 90 },
    { 0xd46ff8d4, 98 },
    { 0x7b7ff8d5, 76 },
    { 0xcd1ff8d5, 100 },
    { 0x339ff8d6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x40fff8db, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c4ff8e2, 98 },
    { 0x230ff8e3, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcabff8e9, 77 },
    { 0x70dff8ea, 34 },
    { 0x448ff8e9, 98 },
    { 0x174ff8ec, 90 },
    { 0x561ff8ec, 55 },
    { 0, 0 },
    { 0x9eeff8ef, 66 },
    { 0xbb1ff8f0, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0x5cfff8f3, 52 },
    { 0x821ff8f4, 33 },
    { 0x296ff8f5, 33 },
    { 0xaf0ff8f5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4a8ff8fb, 95 },
    { 0, 0 },
    { 0x5cff8fd, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0x1e2ff900, 33 },
    { 0xfa3ff901, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa4cff908, 33 },
    { 0x27cff909, 61 },
    { 0x3dff90a, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8acff910, 33 },
    { 0x2f5ff910, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x351ff915, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe49ff919, 94 },
    { 0xe85ff91a, 61 },
    { 0x31ff91b, 33 },
    { 0x68bff91a, 61 },
    { 0x43aff91d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ff924, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0x29aff927, 34 },
    { 0xf23ff928, 52 },
    { 0, 0 },
    { 0xcd3ff92a, 34 },
    { 0xe93ff92b, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab5ff931, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb62ff936, 74 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac2ff93c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x271ff93f, 91 },
    { 0, 0 },
    { 0xb83ff941, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0xfceff944, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x615ff947, 98 },
    { 0, 0 },
    { 0xffcff949, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x499ff94c, 76 },
    { 0x13fff94d, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcd8ff953, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5a9ff95a, 23 },
    { 0x4e0ff95b, 34 },
    { 0xe67ff95c, 90 },
    { 0x96aff95a, 61 },
    { 0xde5ff95e, 33 },
    { 0, 0 },
    { 0xc8fff960, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x26cff965, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3c7ff969, 95 },
    { 0, 0 },
    { 0x55ff96b, 61 },
    { 0xf0dff96c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9ff97a, 23 },
    { 0x1c2ff97a, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbceff97f, 23 },
    { 0, 0 },
    { 0x8e9ff981, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0xd1fff984, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x840ff98a, 99 },
    { 0xd8cff98b, 33 },
    { 0xbb2ff98c, 84 },
    { 0xb2fff98c, 61 },
    { 0xf2ff98d, 76 },
    { 0, 0 },
    { 0x7d3ff990, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb75ff9a4, 61 },
    { 0xb75ff9a5, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d1ff9a8, 23 },
    { 0xcfaff9a8, 93 },
    { 0x87fff9aa, 97 },
    { 0x575ff9aa, 33 },
    { 0x81fff9ac, 98 },
    { 0xa4bff9a9, 76 },
    { 0xb0fff9aa, 98 },
    { 0xb94ff9ae, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x79dff9b9, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0xc44ff9bc, 90 },
    { 0x8caff9bd, 47 },
    { 0x2d2ff9be, 33 },
    { 0xd15ff9bc, 90 },
    { 0x436ff9c0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d5ff9c5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffbff9cc, 67 },
    { 0xa9bff9cd, 23 },
    { 0xd30ff9cd, 52 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc55ff9d2, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xffcff9d6, 90 },
    { 0x83aff9d7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xf2dff9da, 40 },
    { 0x6ddff9db, 61 },
    { 0x36aff9da, 98 },
    { 0x9ff9dd, 95 },
    { 0x86ff9de, 51 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4dfff9e4, 90 },
    { 0, 0 },
    { 0xd49ff9e6, 87 },
    { 0, 0 },
    { 0x844ff9e8, 61 },
    { 0x233ff9e8, 33 },
    { 0x48bff9e8, 95 },
    { 0x7edff9eb, 61 },
    { 0xe66ff9ec, 90 },
    { 0x723ff9ed, 34 },
    { 0, 0 },
    { 0x115ff9ef, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7fff9f2, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe7ff9f8, 95 },
    { 0, 0 },
    { 0xf7aff9fa, 90 },
    { 0xa0bff9fb, 33 },
    { 0, 0 },
    { 0x8beff9fd, 100 },
    { 0xa4bff9fe, 61 },
    { 0x6b3ff9ff, 33 },
    { 0xc3affa00, 33 },
    { 0x2b1ffa01, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x17effa05, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x139ffa0b, 90 },
    { 0x552ffa0c, 96 },
    { 0, 0 },
    { 0xe3effa0e, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xf86ffa11, 91 },
    { 0xb67ffa11, 61 },
    { 0x683ffa13, 84 },
    { 0xee8ffa14, 23 },
    { 0x246ffa15, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda4ffa1f, 52 },
    { 0xa50ffa20, 23 },
    { 0xfc2ffa21, 61 },
    { 0, 0 },
    { 0xc3ffa23, 66 },
    { 0, 0 },
    { 0xc0bffa25, 95 },
    { 0, 0 },
    { 0x290ffa27, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf02ffa2b, 33 },
    { 0x99bffa2c, 99 },
    { 0x1f0ffa2b, 90 },
    { 0xdbbffa2e, 23 },
    { 0x5b4ffa2d, 59 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x151ffa37, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xaf2ffa3e, 98 },
    { 0xc5fffa3f, 87 },
    { 0xdbbffa3f, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x85dffa44, 52 },
    { 0x276ffa44, 38 },
    { 0x3d4ffa46, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb78ffa4b, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b0ffa4e, 33 },
    { 0, 0 },
    { 0x89ffa50, 90 },
    { 0x4c4ffa50, 90 },
    { 0xc9effa52, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xf72ffa55, 31 },
    { 0xa0effa56, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dffa5a, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xcbaffa5e, 23 },
    { 0x457ffa5f, 33 },
    { 0xaa4ffa60, 43 },
    { 0, 0 },
    { 0x4a8ffa62, 84 },
    { 0, 0 },
    { 0xa43ffa64, 95 },
    { 0, 0 },
    { 0x1ffa66, 77 },
    { 0x159ffa67, 61 },
    { 0x1daffa68, 95 },
    { 0x533ffa69, 61 },
    { 0x146ffa69, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe85ffa6e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x865ffa74, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x905ffa7b, 83 },
    { 0x53cffa7c, 77 },
    { 0, 0 },
    { 0xe3fffa7e, 34 },
    { 0x706ffa7e, 96 },
    { 0x4bcffa7e, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x919ffa83, 23 },
    { 0x6a9ffa83, 55 },
    { 0, 0 },
    { 0x485ffa86, 33 },
    { 0, 0 },
    { 0x723ffa88, 98 },
    { 0xd5dffa89, 91 },
    { 0, 0 },
    { 0xda7ffa8b, 61 },
    { 0xc09ffa8c, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x524ffa90, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9dcffa94, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x86effa99, 78 },
    { 0xd85ffa99, 46 },
    { 0xd81ffa9b, 90 },
    { 0x8fcffa9b, 63 },
    { 0x4faffa9a, 61 },
    { 0x469ffa9e, 99 },
    { 0, 0 },
    { 0xef2ffaa0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa2effaa8, 23 },
    { 0x83ffaa9, 91 },
    { 0x348ffaaa, 61 },
    { 0, 0 },
    { 0x845ffaac, 90 },
    { 0, 0 },
    { 0xb52ffaae, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0xc65ffab1, 38 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca9ffab7, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x415ffabd, 61 },
    { 0xc0effabe, 52 },
    { 0x263ffabe, 95 },
    { 0x665ffabe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8ceffac5, 66 },
    { 0x1f1ffac5, 33 },
    { 0xc9fffac7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x451ffacd, 66 },
    { 0, 0 },
    { 0x4c7ffacf, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd36ffad2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7e1ffad9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x577ffadd, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x180ffae0, 61 },
    { 0, 0 },
    { 0xd32ffae2, 65 },
    { 0, 0 },
    { 0xcf3ffae4, 87 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x849ffaea, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbddffaf2, 90 },
    { 0x46dffaf3, 67 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xedeffafa, 43 },
    { 0xf8bffafa, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x946ffb0b, 23 },
    { 0x10fffb0c, 43 },
    { 0xe52ffb0c, 93 },
    { 0x31ffb0e, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c7ffb11, 69 },
    { 0, 0 },
    { 0xf5affb13, 76 },
    { 0, 0 },
    { 0xff1ffb15, 93 },
    { 0x8a9ffb15, 33 },
    { 0x5f2ffb17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x389ffb1a, 79 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x778ffb1f, 65 },
    { 0xcd3ffb1f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2dfffb27, 95 },
    { 0x419ffb27, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c7ffb2f, 84 },
    { 0, 0 },
    { 0xeedffb31, 23 },
    { 0x804ffb32, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x210ffb35, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x836ffb3d, 90 },
    { 0, 0 },
    { 0x4bffb3f, 76 },
    { 0, 0 },
    { 0xac7ffb41, 95 },
    { 0, 0 },
    { 0xb1cffb43, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x8a8ffb46, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xfcfffb4c, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbd6ffb50, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0xc64ffb53, 94 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x437ffb61, 33 },
    { 0, 0 },
    { 0xb3cffb63, 98 },
    { 0x897ffb64, 95 },
    { 0xbc0ffb65, 95 },
    { 0x6d5ffb66, 17 },
    { 0, 0 },
    { 0xa4effb68, 66 },
    { 0, 0 },
    { 0x9a0ffb6a, 90 },
    { 0x119ffb6b, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x198ffb6e, 100 },
    { 0xf95ffb6f, 77 },
    { 0xf7effb6e, 90 },
    { 0xf06ffb6f, 85 },
    { 0x278ffb71, 43 },
    { 0, 0 },
    { 0x969ffb74, 83 },
    { 0x71fffb75, 55 },
    { 0x17affb74, 43 },
    { 0x123ffb75, 33 },
    { 0x982ffb75, 90 },
    { 0x130ffb78, 84 },
    { 0xa11ffb7a, 33 },
    { 0xf73ffb79, 91 },
    { 0xe19ffb78, 85 },
    { 0xb4affb78, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x165ffb81, 95 },
    { 0x448ffb82, 67 },
    { 0, 0 },
    { 0xf38ffb84, 100 },
    { 0, 0 },
    { 0xc82ffb86, 76 },
    { 0, 0 },
    { 0xfa0ffb88, 55 },
    { 0xc39ffb89, 46 },
    { 0x51cffb89, 33 },
    { 0xe17ffb89, 76 },
    { 0xc51ffb8c, 98 },
    { 0x684ffb8d, 98 },
    { 0x6abffb8a, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd8dffb92, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe9affb96, 61 },
    { 0, 0 },
    { 0xfa0ffb98, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2d8ffb9e, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c0ffba1, 33 },
    { 0, 0 },
    { 0xb51ffba3, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x426ffba9, 78 },
    { 0x5ffbaa, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2e2ffbae, 43 },
    { 0x53effbaf, 34 },
    { 0x45dffbaf, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x173ffbb7, 33 },
    { 0xf29ffbb8, 61 },
    { 0x2ccffbb7, 98 },
    { 0x442ffbba, 95 },
    { 0x5ffffbbb, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x213ffbbe, 90 },
    { 0x4b2ffbbf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x91ffbc4, 80 },
    { 0, 0 },
    { 0xcc3ffbc6, 78 },
    { 0x6faffbc7, 93 },
    { 0x5bbffbc8, 87 },
    { 0x3dcffbc6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4bcffbcf, 95 },
    { 0, 0 },
    { 0x85affbd1, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb44ffbd8, 99 },
    { 0x3c4ffbd9, 90 },
    { 0x735ffbd9, 95 },
    { 0xc6effbdb, 90 },
    { 0, 0 },
    { 0xa7affbdd, 84 },
    { 0x7a3ffbde, 61 },
    { 0xaa0ffbdd, 90 },
    { 0, 0 },
    { 0x242ffbe1, 17 },
    { 0, 0 },
    { 0, 0 },
    { 0xd13ffbe4, 61 },
    { 0x30fffbe5, 23 },
    { 0x95bffbe5, 90 },
    { 0x5dbffbe6, 36 },
    { 0x66bffbe6, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x314ffbeb, 98 },
    { 0, 0 },
    { 0x7f4ffbed, 101 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x458ffbf3, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdeaffbf9, 98 },
    { 0x154ffbfa, 55 },
    { 0, 0 },
    { 0x85cffbfc, 98 },
    { 0, 0 },
    { 0x1beffbfe, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x962ffc05, 95 },
    { 0x757ffc06, 61 },
    { 0xf94ffc05, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x972ffc0b, 81 },
    { 0, 0 },
    { 0xb85ffc0d, 23 },
    { 0xa0affc0e, 61 },
    { 0, 0 },
    { 0x47effc10, 76 },
    { 0, 0 },
    { 0xdc1ffc12, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x758ffc18, 52 },
    { 0xc8cffc19, 76 },
    { 0, 0 },
    { 0x20dffc1b, 69 },
    { 0, 0 },
    { 0x6eaffc1d, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x366ffc29, 98 },
    { 0x2feffc2a, 90 },
    { 0, 0 },
    { 0x2f5ffc2c, 93 },
    { 0xeaffc2d, 52 },
    { 0xbb1ffc2d, 90 },
    { 0x6caffc2f, 99 },
    { 0, 0 },
    { 0xc27ffc31, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x607ffc35, 100 },
    { 0xbb2ffc35, 95 },
    { 0x2c7ffc35, 84 },
    { 0xc87ffc36, 61 },
    { 0x115ffc37, 76 },
    { 0, 0 },
    { 0x2eeffc3b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc9fffc43, 93 },
    { 0, 0 },
    { 0x924ffc45, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x86effc48, 33 },
    { 0xa1fffc48, 98 },
    { 0, 0 },
    { 0x561ffc4b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9affc50, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x681ffc54, 95 },
    { 0xc99ffc55, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x544ffc58, 83 },
    { 0x7f7ffc59, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x356ffc5c, 52 },
    { 0x14affc5d, 61 },
    { 0, 0 },
    { 0x6f9ffc5f, 97 },
    { 0x9b6ffc60, 34 },
    { 0x4a9ffc60, 46 },
    { 0xb16ffc62, 98 },
    { 0xad1ffc63, 61 },
    { 0x31ffc64, 33 },
    { 0, 0 },
    { 0x407ffc66, 98 },
    { 0x2c3ffc67, 61 },
    { 0xe54ffc68, 90 },
    { 0x9eeffc66, 66 },
    { 0x362ffc6a, 23 },
    { 0xf06ffc6a, 55 },
    { 0xdd2ffc6c, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x191ffc72, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe37ffc76, 81 },
    { 0x9fcffc77, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c3ffc7b, 53 },
    { 0x53ffc7c, 91 },
    { 0x448ffc7b, 55 },
    { 0xe56ffc7c, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x67ffc82, 93 },
    { 0xa9cffc83, 91 },
    { 0x683ffc83, 23 },
    { 0xde1ffc85, 98 },
    { 0x720ffc84, 43 },
    { 0x13affc87, 33 },
    { 0xd9fffc88, 34 },
    { 0x8ffc84, 98 },
    { 0x963ffc8a, 76 },
    { 0x16affc8b, 90 },
    { 0x8e2ffc8b, 34 },
    { 0x73effc8a, 36 },
    { 0, 0 },
    { 0x8a7ffc8f, 87 },
    { 0x638ffc90, 33 },
    { 0x9a3ffc91, 95 },
    { 0x2bcffc90, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x523ffc95, 23 },
    { 0xbc7ffc96, 80 },
    { 0, 0 },
    { 0x5b4ffc98, 61 },
    { 0, 0 },
    { 0xca8ffc9a, 34 },
    { 0xb6effc9b, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x750ffc9f, 34 },
    { 0xd1bffc9f, 87 },
    { 0xcdaffca1, 91 },
    { 0x6e5ffc9f, 55 },
    { 0xa19ffca3, 90 },
    { 0xe2cffca3, 61 },
    { 0x2a4ffca4, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x2acffca9, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xce8ffcb3, 90 },
    { 0x259ffcb4, 43 },
    { 0xce6ffcb3, 17 },
    { 0x1a7ffcb6, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x172ffcbb, 52 },
    { 0x6d2ffcbc, 76 },
    { 0xc2affcbd, 33 },
    { 0x399ffcbe, 96 },
    { 0x308ffcbe, 61 },
    { 0xf11ffcc0, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xddfffcc6, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x132ffccb, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0x3affcce, 23 },
    { 0x477ffcce, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65dffcd3, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa98ffcdb, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x416ffcdf, 90 },
    { 0, 0 },
    { 0xdb2ffce1, 43 },
    { 0, 0 },
    { 0x7b7ffce3, 90 },
    { 0x4c2ffce4, 95 },
    { 0x6adffce4, 77 },
    { 0x6ffce5, 95 },
    { 0x659ffce7, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8cffcea, 81 },
    { 0xa33ffcea, 34 },
    { 0x8a2ffcec, 78 },
    { 0, 0 },
    { 0x89dffcee, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x8d8ffcf1, 98 },
    { 0, 0 },
    { 0xf0ffcf3, 33 },
    { 0x9c8ffcf3, 23 },
    { 0xafbffcf5, 93 },
    { 0x383ffcf6, 23 },
    { 0x957ffcf6, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3a2ffcfe, 33 },
    { 0x675ffcff, 78 },
    { 0, 0 },
    { 0x4cfffd01, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x2adffd04, 95 },
    { 0x68ffd05, 66 },
    { 0x867ffd05, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x164ffd0c, 90 },
    { 0, 0 },
    { 0xa5bffd0e, 23 },
    { 0xefbffd0f, 98 },
    { 0, 0 },
    { 0x18ffd11, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xbf2ffd15, 90 },
    { 0x358ffd16, 98 },
    { 0xc09ffd17, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0xeeffd1a, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x27affd1d, 90 },
    { 0x2b8ffd1e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18ffd26, 61 },
    { 0xfd2ffd27, 61 },
    { 0x571ffd26, 78 },
    { 0x160ffd29, 76 },
    { 0, 0 },
    { 0x9f1ffd2b, 40 },
    { 0x7d0ffd2c, 66 },
    { 0xf9affd2d, 66 },
    { 0x724ffd2e, 33 },
    { 0, 0 },
    { 0x22cffd30, 55 },
    { 0x66effd30, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xacaffd38, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd05ffd3d, 99 },
    { 0x5c2ffd3e, 90 },
    { 0x5ffd3f, 33 },
    { 0xb66ffd3d, 61 },
    { 0xe66ffd3f, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x93dffd44, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4cffd4c, 95 },
    { 0x6bdffd4c, 33 },
    { 0xb9effd4e, 61 },
    { 0, 0 },
    { 0x2f5ffd50, 46 },
    { 0xba2ffd51, 17 },
    { 0xccffd51, 43 },
    { 0xc58ffd53, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x4ffffd56, 66 },
    { 0x574ffd57, 68 },
    { 0, 0 },
    { 0xb59ffd59, 90 },
    { 0xcdcffd59, 76 },
    { 0xe49ffd5b, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x29ffd5f, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x65affd64, 23 },
    { 0x108ffd64, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x374ffd69, 98 },
    { 0x7d7ffd69, 76 },
    { 0, 0 },
    { 0x579ffd6c, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x667ffd70, 100 },
    { 0xdf0ffd71, 98 },
    { 0, 0 },
    { 0x403ffd73, 46 },
    { 0x38dffd74, 34 },
    { 0x97affd75, 87 },
    { 0xfb7ffd76, 23 },
    { 0xae7ffd75, 84 },
    { 0x704ffd73, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x41dffd7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xf71ffd7e, 90 },
    { 0x516ffd7f, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x971ffd84, 34 },
    { 0, 0 },
    { 0x3d4ffd86, 34 },
    { 0x7a5ffd86, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x116ffd8e, 55 },
    { 0x467ffd8f, 98 },
    { 0x624ffd90, 90 },
    { 0xe19ffd91, 95 },
    { 0x338ffd90, 90 },
    { 0, 0 },
    { 0xd5fffd94, 90 },
    { 0x533ffd95, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x5e7ffd98, 90 },
    { 0x79fffd98, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x8c5ffd9c, 75 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb50ffda0, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xac7ffda6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5c2ffdb0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0xcadffdb3, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0xa6fffdb6, 52 },
    { 0x69effdb7, 61 },
    { 0xa77ffdb8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe19ffdbc, 33 },
    { 0, 0 },
    { 0xb14ffdbe, 61 },
    { 0, 0 },
    { 0xdfffdc0, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x51fffdc4, 93 },
    { 0x8d2ffdc5, 90 },
    { 0, 0 },
    { 0xb78ffdc7, 90 },
    { 0, 0 },
    { 0xb43ffdc9, 95 },
    { 0x55cffdca, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x24cffdcd, 90 },
    { 0xf12ffdce, 55 },
    { 0xb95ffdcf, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0x765ffdd2, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7d3ffddd, 76 },
    { 0xde7ffddd, 23 },
    { 0x629ffddf, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3cffde5, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0xd4cffde8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc84ffded, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe46ffdfb, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x661ffdff, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x343ffe03, 40 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x68dffe09, 55 },
    { 0x6bffe0a, 76 },
    { 0, 0 },
    { 0xb10ffe0c, 33 },
    { 0, 0 },
    { 0x355ffe0e, 55 },
    { 0x5b6ffe0e, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1cdffe14, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf9fffe1b, 77 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdfbffe1f, 93 },
    { 0x102ffe20, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xf44ffe23, 23 },
    { 0xf35ffe24, 93 },
    { 0, 0 },
    { 0x734ffe26, 99 },
    { 0, 0 },
    { 0, 0 },
    { 0xfbbffe29, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd3fffe2f, 61 },
    { 0xd37ffe30, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b1ffe33, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9ceffe3a, 90 },
    { 0x2e7ffe3b, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb3fffe41, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x920ffe44, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0x241ffe47, 95 },
    { 0xbf6ffe47, 95 },
    { 0, 0 },
    { 0xa19ffe4a, 43 },
    { 0x238ffe4b, 61 },
    { 0, 0 },
    { 0xc58ffe4d, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe4ffe53, 87 },
    { 0, 0 },
    { 0x2abffe55, 88 },
    { 0, 0 },
    { 0, 0 },
    { 0xfe0ffe58, 34 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf88ffe5f, 33 },
    { 0xb74ffe5f, 90 },
    { 0x743ffe61, 33 },
    { 0, 0 },
    { 0x37effe63, 95 },
    { 0x732ffe64, 44 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x528ffe6b, 52 },
    { 0x453ffe6b, 55 },
    { 0x5f6ffe6d, 34 },
    { 0, 0 },
    { 0xfafffe6f, 81 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x985ffe76, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xe26ffe79, 66 },
    { 0x265ffe7a, 55 },
    { 0xbf1ffe7b, 33 },
    { 0x4d8ffe7b, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4d4ffe80, 90 },
    { 0, 0 },
    { 0xe2fffe82, 33 },
    { 0x9e9ffe82, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9d2ffe87, 61 },
    { 0x6f6ffe88, 90 },
    { 0xe2fffe89, 85 },
    { 0x154ffe88, 90 },
    { 0x3edffe89, 98 },
    { 0x1d4ffe8a, 67 },
    { 0x46effe87, 55 },
    { 0x9cfffe8e, 33 },
    { 0xd61ffe8f, 91 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xdb5ffe96, 100 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x18bffe9a, 46 },
    { 0x7e8ffe9b, 46 },
    { 0xe55ffe9b, 76 },
    { 0x6a4ffe9b, 23 },
    { 0xef5ffe9e, 95 },
    { 0xef4ffe9f, 93 },
    { 0x50fffe9f, 61 },
    { 0x6a4ffea0, 76 },
    { 0x353ffea0, 33 },
    { 0x1eeffea3, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9c1ffeae, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x68bffeb1, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd2bffeb5, 80 },
    { 0x1e7ffeb6, 61 },
    { 0x20fffeb7, 23 },
    { 0x78affeb7, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc30ffebd, 95 },
    { 0xfdeffebe, 61 },
    { 0, 0 },
    { 0xfeffec0, 76 },
    { 0, 0 },
    { 0x9a9ffec2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xd6dffec9, 90 },
    { 0xa06ffec9, 66 },
    { 0xb6dffec9, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0x9fdffece, 84 },
    { 0, 0 },
    { 0x460ffed0, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb9ffed7, 95 },
    { 0x2baffed8, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0xe07ffedb, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x50effedf, 17 },
    { 0xd39ffee0, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xca5ffee6, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe08ffeea, 33 },
    { 0x55cffeeb, 46 },
    { 0x75bffeec, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x5b3ffef0, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x474ffef4, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x733ffef7, 43 },
    { 0xcdcffef7, 67 },
    { 0, 0 },
    { 0x690ffefa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0xe8bffefd, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xb90fff06, 90 },
    { 0xb4cfff06, 55 },
    { 0x8affff06, 76 },
    { 0xb5ffff09, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x5f9fff0c, 90 },
    { 0x28fff0d, 69 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xc0dfff11, 75 },
    { 0x5a4fff11, 23 },
    { 0xccbfff11, 33 },
    { 0, 0 },
    { 0xe82fff15, 61 },
    { 0xb5cfff16, 33 },
    { 0x5e7fff17, 61 },
    { 0xb27fff16, 61 },
    { 0x69afff15, 33 },
    { 0x5adfff1a, 78 },
    { 0x7eefff1b, 90 },
    { 0x9e3fff1b, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0x7fbfff1f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xef1fff24, 55 },
    { 0x6fffff24, 55 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x57fff2c, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0x9f9fff2f, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xda2fff34, 95 },
    { 0x62bfff35, 100 },
    { 0, 0 },
    { 0xea6fff37, 55 },
    { 0xbebfff38, 23 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x570fff3c, 95 },
    { 0, 0 },
    { 0xc8ffff3e, 95 },
    { 0xf5dfff3f, 91 },
    { 0xa39fff3f, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x729fff44, 40 },
    { 0x663fff45, 87 },
    { 0x231fff46, 95 },
    { 0, 0 },
    { 0, 0 },
    { 0x334fff49, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xeaefff4e, 61 },
    { 0xb5dfff4f, 95 },
    { 0, 0 },
    { 0x9d0fff51, 95 },
    { 0x82cfff52, 78 },
    { 0x51dfff53, 61 },
    { 0, 0 },
    { 0xbc3fff55, 66 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x4f6fff5b, 61 },
    { 0, 0 },
    { 0x930fff5d, 84 },
    { 0, 0 },
    { 0x938fff5f, 64 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dfff62, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0x126fff65, 34 },
    { 0, 0 },
    { 0x500fff67, 61 },
    { 0, 0 },
    { 0x56ffff69, 84 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x9b1fff6f, 93 },
    { 0x3a0fff70, 98 },
    { 0x273fff71, 95 },
    { 0x128fff72, 88 },
    { 0x220fff72, 98 },
    { 0xa0bfff70, 23 },
    { 0x4b6fff75, 90 },
    { 0x134fff75, 93 },
    { 0xd79fff77, 33 },
    { 0xd7fff74, 52 },
    { 0x2f5fff79, 23 },
    { 0xb4efff70, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xab0fff7e, 90 },
    { 0xf85fff7e, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x44cfff82, 88 },
    { 0xb84fff82, 76 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x594fff87, 61 },
    { 0, 0 },
    { 0x6d8fff89, 23 },
    { 0, 0 },
    { 0x450fff8b, 68 },
    { 0xc58fff8b, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x25dfff93, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x903fff97, 33 },
    { 0, 0 },
    { 0x245fff99, 34 },
    { 0x487fff9a, 61 },
    { 0, 0 },
    { 0, 0 },
    { 0xae8fff9d, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xa52fffa1, 90 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xe69fffaa, 100 },
    { 0x416fffaa, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x767fffaf, 23 },
    { 0x886fffb0, 90 },
    { 0xef1fffb1, 90 },
    { 0xeccfffb2, 78 },
    { 0, 0 },
    { 0x81cfffb4, 77 },
    { 0, 0 },
    { 0x70efffb6, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x1fffbc, 61 },
    { 0xf48fffbd, 36 },
    { 0, 0 },
    { 0, 0 },
    { 0xb92fffc0, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x7c9fffc5, 46 },
    { 0, 0 },
    { 0, 0 },
    { 0xeb1fffc8, 97 },
    { 0, 0 },
    { 0x9effffca, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x3b8fffce, 75 },
    { 0xc3dfffcf, 43 },
    { 0, 0 },
    { 0, 0 },
    { 0xe81fffd2, 98 },
    { 0, 0 },
    { 0, 0 },
    { 0x6b4fffd5, 85 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x38cfffdc, 95 },
    { 0x52dfffdd, 76 },
    { 0x310fffde, 88 },
    { 0xa49fffdc, 66 },
    { 0, 0 },
    { 0x64afffe1, 93 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0x656fffe7, 47 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0xf47fffef, 90 },
    { 0, 0 },
    { 0xa8fffff1, 78 },
    { 0, 0 },
    { 0, 0 },
    { 0x861ffff4, 99 },
    { 0, 0 },
    { 0xd86ffff6, 95 },
    { 0, 0 },
    { 0x2affff8, 33 },
    { 0, 0 },
    { 0, 0 },
    { 0x76dffffb, 98 },
    { 0xfb6ffffc, 34 },
    { 0x8a0ffffd, 95 },
    { 0xd8fffffe, 84 },
    { 0x32dfffff, 84 }
};


static inline label brown_get_wc(uint32_t key) {
    size_t i = key & 0xfffff;
    for (;;) {
        if (brown[i].hash == key) return brown[i].value;
        if (brown[i].value == 0) return 0;
        i = (i + 1) & 0xfffff;
    }
}

static int extract_invariant(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        partial_hash_t *pi_hashes)
{
size_t i;

for (i=0; i<n_items; i++) {

    uint32_t codep_buf[MAX_STR];
    size_t codep_buf_len;

    uint32_t trans_buf[MAX_STR];
    size_t trans_buf_len;

    const partial_hash_t hash_0_x_x_brown = brown_get_wc(hash32_data(1, field_buf[0], field_len[0]));

    codep_buf_len = MAX_STR;
    if (utf8_decode(codep_buf, &codep_buf_len, field_buf[0], field_len[0]) < 0)
        return -1;

    trans_buf_len = MAX_STR;
    unicode_translate_normalize(trans_buf, &trans_buf_len, codep_buf, codep_buf_len);

    const partial_hash_t hash_0_x_normalize =
        hash32_partial_unicode(trans_buf, trans_buf_len);

    const partial_hash_t hash_0_pre1_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 1, 2);

    const partial_hash_t hash_0_pre2_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 2, 2);

    const partial_hash_t hash_0_pre3_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 3, 2);

    const partial_hash_t hash_0_pre4_normalize =
        hash32_partial_unicode_prefix(trans_buf, trans_buf_len, 4, 2);

    const partial_hash_t hash_0_suf1_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 1, 2);

    const partial_hash_t hash_0_suf2_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 2, 2);

    const partial_hash_t hash_0_suf3_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 3, 2);

    const partial_hash_t hash_0_suf4_normalize =
        hash32_partial_unicode_suffix(trans_buf, trans_buf_len, 4, 2);

    trans_buf_len = MAX_STR;
    unicode_translate_abstract(trans_buf, &trans_buf_len, codep_buf, codep_buf_len);

    const partial_hash_t hash_0_x_abstract =
        hash32_partial_unicode(trans_buf, trans_buf_len);

    trans_buf_len = MAX_STR;
    unicode_translate_delexicalize(trans_buf, &trans_buf_len, codep_buf, codep_buf_len);

    const partial_hash_t hash_0_x_delexicalize =
        hash32_partial_unicode(trans_buf, trans_buf_len);

    pi_hashes[0] = hash_0_pre1_normalize;
    pi_hashes[1] = hash_0_pre2_normalize;
    pi_hashes[2] = hash_0_pre3_normalize;
    pi_hashes[3] = hash_0_pre4_normalize;
    pi_hashes[4] = hash_0_suf1_normalize;
    pi_hashes[5] = hash_0_suf2_normalize;
    pi_hashes[6] = hash_0_suf3_normalize;
    pi_hashes[7] = hash_0_suf4_normalize;
    pi_hashes[8] = hash_0_x_abstract;
    pi_hashes[9] = hash_0_x_delexicalize;
    pi_hashes[10] = hash_0_x_normalize;
    pi_hashes[11] = hash_0_x_x_brown;

    field_buf += n_fields;
    field_len += n_fields;
    pi_hashes += 12;
}
return 0;
}

#define N_INVARIANTS    12
#define N_FEATURES      20

static void extract_features(
        const label *history,
        label tag,
        size_t i,
        size_t n_items,
        const partial_hash_t *pi_hashes,
        feat_hash_t *feature_hashes)
{
    feature_hashes[0] = hash32_fmix(hash32_mix(1, hash32_mix(tag, ((i>=1)? history[i-1] : suc_N_TAGS))));
    feature_hashes[1] = hash32_fmix(hash32_mix(2, hash32_mix(tag, hash32_mix(((i>=1)? history[i-1] : suc_N_TAGS), ((i>=2)? history[i-2] : suc_N_TAGS)))));
    feature_hashes[2] = hash32_fmix(hash32_mix(3, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 11] : 0xdcdeaf63), tag)));
    feature_hashes[3] = hash32_fmix(hash32_mix(4, hash32_mix(pi_hashes[i*N_INVARIANTS + 11], tag)));
    feature_hashes[4] = hash32_fmix(hash32_mix(5, hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 11] : 0xdcdeaf63), tag)));
    feature_hashes[5] = hash32_fmix(hash32_mix(6, hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 11] : 0xdcdeaf63), hash32_mix(((i < n_items-2)? pi_hashes[(i+2)*N_INVARIANTS + 11] : 0xdcdeaf63), tag))));
    feature_hashes[6] = hash32_fmix(hash32_mix(7, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 11] : 0xdcdeaf63), hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 11] : 0xdcdeaf63), tag))));
    feature_hashes[7] = hash32_fmix(hash32_mix(8, hash32_mix(pi_hashes[i*N_INVARIANTS + 9], tag)));
    feature_hashes[8] = hash32_fmix(hash32_mix(9, hash32_mix(pi_hashes[i*N_INVARIANTS + 8], tag)));
    feature_hashes[9] = hash32_fmix(hash32_mix(10, hash32_mix(pi_hashes[i*N_INVARIANTS + 10], tag)));
    feature_hashes[10] = hash32_fmix(hash32_mix(11, hash32_mix(((i < n_items-1)? pi_hashes[(i+1)*N_INVARIANTS + 10] : 0x282acd16), tag)));
    feature_hashes[11] = hash32_fmix(hash32_mix(12, hash32_mix(((i >= 1)? pi_hashes[(i-1)*N_INVARIANTS+ 10] : 0x282acd16), tag)));
    feature_hashes[12] = hash32_fmix(hash32_mix(13, hash32_mix(pi_hashes[i*N_INVARIANTS + 0], tag)));
    feature_hashes[13] = hash32_fmix(hash32_mix(14, hash32_mix(pi_hashes[i*N_INVARIANTS + 1], tag)));
    feature_hashes[14] = hash32_fmix(hash32_mix(15, hash32_mix(pi_hashes[i*N_INVARIANTS + 2], tag)));
    feature_hashes[15] = hash32_fmix(hash32_mix(16, hash32_mix(pi_hashes[i*N_INVARIANTS + 3], tag)));
    feature_hashes[16] = hash32_fmix(hash32_mix(17, hash32_mix(pi_hashes[i*N_INVARIANTS + 4], tag)));
    feature_hashes[17] = hash32_fmix(hash32_mix(18, hash32_mix(pi_hashes[i*N_INVARIANTS + 5], tag)));
    feature_hashes[18] = hash32_fmix(hash32_mix(19, hash32_mix(pi_hashes[i*N_INVARIANTS + 6], tag)));
    feature_hashes[19] = hash32_fmix(hash32_mix(20, hash32_mix(pi_hashes[i*N_INVARIANTS + 7], tag)));
}


#if BEAM_SIZE == 1
static void beam_search(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        const real *weights,
        size_t weights_len,
        int use_lexicon,
        int use_dropout,
        feat_hash_t dropout_seed,
        label *result)
{
    size_t i;
    feat_hash_t invariant_hashes[N_INVARIANTS*n_items];
    feat_hash_t feature_hashes[N_FEATURES];
    const label *tags;
    extract_invariant(
            field_buf, field_len, n_fields, n_items, invariant_hashes);

    for (i=0; i<n_items; i++) {
        real max_score = -REAL_MAX;
        label max_tag = 0;

        if (use_lexicon) {
            size_t j;
            tags = get_tags(hash32_fmix(invariant_hashes[N_INVARIANTS*i + 10]));
            if (tags[0] == 1) {
                //printf("Only tag available: %d\n", tags[1]);
                max_tag = (label)tags[1];
            } else {
                //printf("%d tags available\n", tags[0]);
                for (j=1; j<tags[0]+1; j++) {
                    extract_features(
                            result, tags[j], i, n_items,
                            invariant_hashes, feature_hashes);
                    const real score = get_score(
                            feature_hashes, N_FEATURES, weights, weights_len,
                            use_dropout, dropout_seed);
                    if (score > max_score) {
                        max_score = score;
                        max_tag = (label)tags[j];
                    }
                }
            }
        } else {
            label tag;
            for (tag=0; tag<N_TAGS; tag++) {
                extract_features(
                        result, tag, i, n_items,
                        invariant_hashes, feature_hashes);
                const real score = get_score(
                        feature_hashes, N_FEATURES, weights, weights_len,
                        use_dropout, dropout_seed);
                if (score > max_score) {
                    max_score = score;
                    max_tag = (label)tag;
                }
            }
        }
        result[i] = max_tag;
    }
}
#endif

#if BEAM_SIZE > 1
static void beam_search(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        const real *weights,
        size_t weights_len,
        int use_lexicon,
        int use_dropout,
        feat_hash_t dropout_seed,
        label *result)
{
    size_t i;
    feat_hash_t invariant_hashes[N_INVARIANTS*n_items];
    feat_hash_t feature_hashes[N_FEATURES];
    real beam_scores[BEAM_SIZE];
    label new_beams[BEAM_SIZE][n_items];
    label beams[BEAM_SIZE][n_items];
    // this is the actual beam size, whereas BEAM_SIZE is the maximum size
    size_t beam_size = 1;
    const label *tags;
    extract_invariant(
            field_buf, field_len, n_fields, n_items, invariant_hashes);

    beam_scores[0] = (real)0.0;

    for (i=0; i<n_items; i++) {
        size_t k;
        real max_score[BEAM_SIZE];
        label max_tag[BEAM_SIZE];
        size_t max_beam[BEAM_SIZE];
        for (k=0; k<BEAM_SIZE; k++) {
            max_score[k] = -REAL_MAX;
            max_tag[k] = 0;
            max_beam[k] = 0;
        }

        if (use_lexicon) {
            size_t j;
            tags = get_tags(hash32_fmix(invariant_hashes[N_INVARIANTS*i + 10]));
            /* if (tags[0] == 1) {
                for (k=0; k<beam_size; k++) {
                    max_tag[k] = tags[1];
                    max_beam[k] = 0;
                    max_score[k] = beam_scores[k];
                }
            } else */ {
                for (j=1; j<tags[0]+1; j++) {
                    for (k=0; k<beam_size; k++) {
                        extract_features(
                                beams[k], tags[j], i, n_items,
                                invariant_hashes, feature_hashes);
                        const real score = beam_scores[k] + get_score(
                                feature_hashes, N_FEATURES,
                                weights, weights_len,
                                use_dropout, dropout_seed);
                        if (score > max_score[BEAM_SIZE-1]) {
                            size_t l,m;
                            for (l=0; score < max_score[l]; l++);
                            for (m=BEAM_SIZE-1; m>l; m--) {
                                max_score[m] = max_score[m-1];
                                max_tag[m] = max_tag[m-1];
                                max_beam[m] = max_beam[m-1];
                            }
                            max_score[l] = score;
                            max_tag[l] = (label)tags[j];
                            max_beam[l] = k;
                        }
                    }
                }
            }
        } else {
            label tag;
            for (tag=0; tag<N_TAGS; tag++) {
                for (k=0; k<beam_size; k++) {
                    extract_features(
                            beams[k], tag, i, n_items,
                            invariant_hashes, feature_hashes);
                    const real score = beam_scores[k] + get_score(
                            feature_hashes, N_FEATURES, weights, weights_len,
                            use_dropout, dropout_seed);
                    if (score > max_score[BEAM_SIZE-1]) {
                        size_t l,m;
                        for (l=0; score < max_score[l]; l++);
                        for (m=BEAM_SIZE-1; m>l; m--) {
                            max_score[m] = max_score[m-1];
                            max_tag[m] = max_tag[m-1];
                            max_beam[m] = max_beam[m-1];
                        }
                        max_score[l] = score;
                        max_tag[l] = tag;
                        max_beam[l] = k;
                    }
                }
            }
        }
        for (k=0; k<BEAM_SIZE && max_score[k] != -REAL_MAX; k++) {
            beams[k][i] = max_tag[k];
            beam_scores[k] = max_score[k];
        }
        beam_size = k;
        if (i > 0) {
            for (k=0; k<beam_size; k++)
                if (max_beam[k] != k)
                    memcpy(new_beams[k], beams[max_beam[k]], i*sizeof(label));
            for (k=0; k<beam_size; k++)
                if (max_beam[k] != k)
                    memcpy(beams[k], new_beams[k], i*sizeof(label));
        }
    }
    memcpy(result, beams[0], n_items*sizeof(label));
}
#endif
#ifndef __SEQ_C__
#define __SEQ_C__

// If the output should be aligned, set this to 1.
#define ALIGN_FIELDS    0
#define ALIGNMENT       (sizeof(size_t))

// If fields should be zero-terminated, set this to 1.
#define ZERO_TERMINATE  1

static int read_sequence(
        FILE *file,
        uint8_t **field_buf,
        size_t *field_len,
        size_t n_fields,
        size_t *n_items,
        uint8_t *buf,
        size_t *buf_len)
{
    const size_t max_items = *n_items;
    size_t field, item, field_p = 0, buf_p = 0;
    const size_t max_len = *buf_len;

    for (item=0; item<max_items; item++) {
        for (field=0; field<n_fields; field++) {
#if ALIGN_FIELDS
            while (buf_p % ALIGNMENT) buf[buf_p++] = 0;
            if (unlikely(buf_p >= max_len)) return -1;
#endif
            const size_t buf_p_start = buf_p;
            for (;;) {
                const int c = getc_unlocked(file);
                if (likely(c > '\n')) {
                    if (unlikely(buf_p >= max_len)) return -1;
                    buf[buf_p++] = c;
                } else if (unlikely(c == '\t')) {
                    if (unlikely(field == n_fields-1)) return -1;
                    field_buf[field_p] = buf + buf_p_start;
                    if ((field_len[field_p] = buf_p - buf_p_start) >= MAX_STR) {
                        field_len[field_p] = MAX_STR-1;
                        buf[buf_p_start + MAX_STR-1] = 0;
                    }
                    field_p++;
#if ZERO_TERMINATE
                    if (unlikely(buf_p >= max_len)) return -1;
                    buf[buf_p++] = 0;
#endif
                    break;
                } else if (unlikely(c == '\n')) {
                    if (unlikely(field == 0 && buf_p == buf_p_start)) {
                        *n_items = item;
                        *buf_len = buf_p;
                        return 0;
                    }
                    if (unlikely(field != n_fields-1)) return -1;
                    field_buf[field_p] = buf + buf_p_start;
                    if ((field_len[field_p] = buf_p - buf_p_start) >= MAX_STR) {
                        field_len[field_p] = MAX_STR-1;
                        buf[buf_p_start + MAX_STR-1] = 0;
                    }
                    field_p++;
#if ZERO_TERMINATE
                    if (unlikely(buf_p >= max_len)) return -1;
                    buf[buf_p++] = 0;
#endif
                    break;
                } else if (unlikely(c == EOF)) return -1;
                // For the time being, silently drop ASCII values below 10
                // except newline and tab.
            }
        }
    }
    return -1;
}

#endif


#ifndef __SEARCH_C__
#define __SEARCH_C__

#include <string.h>

static void adjust_weights(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        real *weights,
        size_t weights_len,
        const label *labels,
        real weight_diff,
        double t,
        double *average_weights,
        int use_dropout,
        feat_hash_t dropout_seed)
{
    size_t i, j;
    feat_hash_t invariant_hashes[N_INVARIANTS*n_items];
    feat_hash_t feature_hashes[N_FEATURES];
    feat_hash_t mask = (feat_hash_t)weights_len - 1;

    extract_invariant(
            field_buf, field_len, n_fields, n_items, invariant_hashes);

    for (i=0; i<n_items; i++) {
        extract_features(
                labels, labels[i], i, n_items,
                invariant_hashes, feature_hashes);
        for (j=0; j<N_FEATURES; j++) {
            const feat_hash_t h = feature_hashes[j];
            if ((!use_dropout) ||
                (hash32_mix(dropout_seed, h) >= DROPOUT_CONSTANT)) {
                const size_t idx = h & mask;
                average_weights[idx*2] +=
                    (t - average_weights[idx*2+1]) * (double)weights[idx];
                average_weights[idx*2+1] = t;
                weights[idx] += weight_diff;
            }
        }
    }
}

static size_t train_sequence(
        const uint8_t **field_buf,
        const size_t *field_len,
        size_t n_fields,
        size_t n_items,
        real *weights,
        size_t weights_len,
        const label *gold,
        double t,
        double *average_weights,
        int use_dropout,
        feat_hash_t dropout_seed)
{
    label crap[n_items];
    beam_search(field_buf, field_len, n_fields, n_items,
                  weights, weights_len, 0, use_dropout, dropout_seed, crap);
    if (!memcmp(crap, gold, n_items*sizeof(label))) return 0;
    size_t n_errs = 0;
    size_t i;
    for (i=0; i<n_items; i++)
        n_errs += (gold[i] != crap[i]);
    adjust_weights(field_buf, field_len, n_fields, n_items,
                   weights, weights_len, gold, (real)1.0,
                   t, average_weights,
                   use_dropout, dropout_seed);
    adjust_weights(field_buf, field_len, n_fields, n_items,
                   weights, weights_len, crap, (real)(-1.0),
                   t, average_weights,
                   use_dropout, dropout_seed);
    return n_errs;
}

#endif

#ifndef __TAG_C__
#define __TAG_C__

static int tag(
        FILE *infile,
        const real *weights,
        size_t weights_len,
        FILE *outfile,
        size_t n_fields,
        double *error_rate)
{
    size_t i;

    const size_t max_items = 0x400;
    const size_t max_fields = max_items*n_fields;
    size_t max_len = 0x10000;

    uint8_t *field_buf[max_fields];
    size_t field_len[max_fields];
    size_t n_items;
    uint8_t buf[max_len];

    size_t n_total = 0, n_errors = 0;

    for (;;) {
        n_items = max_items;
        size_t buf_len = max_len;
        const int rv = read_sequence(
                infile, field_buf, field_len, n_fields, &n_items,
                buf, &buf_len);
        if (rv < 0) {
            if (!feof(infile)) {
                fprintf(stderr, "Error at %ld!\n", ftell(infile));
                return -1;
            }
            break;
        }

        label result[n_items];
        beam_search(
                (const uint8_t**)field_buf, field_len, n_fields,
                n_items, weights, weights_len, 1, 0, 0, result);

        if (outfile != NULL) {
            for (i=0; i<n_items; i++) {
                size_t j;
                for (j=0; j<N_TRAIN_FIELDS; j++) {
                    if (j == COL_TAG)
                        fputs((char*)tag_str[result[i]], outfile);
                    else if (j < COL_TAG)
                        fputs((char*)field_buf[i*n_fields+j], outfile);
                    else
                        fputs((char*)field_buf[i*n_fields+j-1], outfile);
                    if (j < N_TRAIN_FIELDS-1)
                        fputc('\t', outfile);
                }
                fputc('\n', outfile);
            }
            fputc('\n', outfile);
        }

        if (error_rate != NULL) {
            for (i=0; i<n_items; i++) {
                const int tag = tagset_from_str(
                        (const char*)field_buf[i*n_fields + COL_TAG]);
                if (tag < 0) {
                    fprintf(stderr, "Invalid tag: '%s'\n",
                            field_buf[i*n_fields + COL_TAG]);
                }
                if (result[i] != tag) n_errors++;
                n_total++;
            }
        }

    }

    if (error_rate != NULL) {
        if (n_total > 0) *error_rate = (double)n_errors / (double)n_total;
        else *error_rate = -1.0;
    }

    fflush(outfile);

    return 0;
}

#endif

#ifndef __TRAIN_C__
#define __TRAIN_C__

#include <stdlib.h>
#include <errno.h>

static void shuffle(size_t *items, size_t len) {
    size_t i;
    for (i=0; i<len; i++) {
        size_t j = random() % len;
        size_t t = items[i];
        items[i] = items[j];
        items[j] = t;
    }
}

static int train(
        const char *train_filename,
        const char *tune_filename,
        const char *model_filename)
{
    FILE *train_file = fopen(train_filename, "rb");
    FILE *tune_file = fopen(tune_filename, "rb");
    const int use_dropout = DROPOUT_RATE != 0;

    if (train_file == NULL) {
        perror("unable to open training data file");
        exit(1);
    }

    if (tune_file == NULL) {
        perror("unable to open tuning data file");
        exit(1);
    }

    size_t i;

    const size_t max_items = 0x400;
    const size_t max_fields = max_items*N_TRAIN_FIELDS;
    const size_t max_len = 0x10000;
    
    double best_error_ever = 1.0;

    FILE *model = NULL;

    feat_hash_t dropout_seed = 1;

    uint8_t *field_buf[max_fields];
    size_t field_len[max_fields];
    size_t n_items;
    uint8_t buf[max_len];

    // First, get file offsets of sentence starts
    size_t max_sents = 0x100000;
    size_t n_sents;
    long *sent_offsets = malloc(sizeof(long)*max_sents);

    for (n_sents=0; ; n_sents++) {
        if (n_sents >= max_sents) {
            return 1;
        }
        sent_offsets[n_sents] = ftell(train_file);
        n_items = max_items;
        size_t buf_len = max_len;
        const int rv = read_sequence(
                train_file, field_buf, field_len, N_TRAIN_FIELDS, &n_items,
                buf, &buf_len);
        if (rv < 0) {
            if (!feof(train_file)) {
                fprintf(stderr, "Error at %s:%ld (bytes)!\n",
                        train_filename, ftell(train_file));
                return 1;
            }
            rewind(train_file);
            break;
        }
    }

    sent_offsets = realloc(sent_offsets, sizeof(long)*n_sents);
    size_t sent_order[n_sents];
    for (i=0; i<n_sents; i++) sent_order[i] = i;

    fprintf(stderr, "Training data contains %zd sentences\n", n_sents);

    size_t weights_len;
    for (weights_len=MIN_WEIGHTS_LEN;
         weights_len<=MAX_WEIGHTS_LEN;
         weights_len*=2)
    {
        fprintf(stderr, "Trying weight vector of size 0x%zx\n", weights_len);
        real *weights = malloc(sizeof(real)*weights_len);
        // Even elements contain averages of the weights vector,
        // odd elements contain the time (i.e. value of t) of the last update
        // of the average.
        // This is less elegant, but good for cache locality since these values
        // are accessed randomly at the same time.
        // TODO: might also want to get the weight vector itself into the same
        // area.
        double *average_weights = malloc(sizeof(double)*weights_len*2);
        double t = 0.0;

        for (i=0; i<weights_len; i++) weights[i] = (real)0.0;
        for (i=0; i<weights_len*2; i++) average_weights[i] = 0.0;

        size_t iter;
        double tune_error_avg = 1.0;
        double best_error = 1.0;

        const int max_patience = 5;
        int patience_left = max_patience;

        double tune_error = 1.0;
        for (iter=0; ; iter++) {
            shuffle(sent_order, n_sents);

            fprintf(stderr, "Iteration %zd...\n", iter+1);
            size_t n_errs = 0;
            size_t n_total = 0;
            size_t sent;
            for (sent=0; sent<n_sents; sent++) {
                fseek(train_file, sent_offsets[sent_order[sent]], SEEK_SET);

                n_items = max_items;
                size_t buf_len = max_len;
                const int rv = read_sequence(
                        train_file, field_buf, field_len, N_TRAIN_FIELDS,
                        &n_items, buf, &buf_len);
                if (rv < 0) {
                    fprintf(stderr, "Error at %s:%ld (bytes)!\n",
                            train_filename, ftell(train_file));
                    return 1;
                }

                label gold[n_items];
                for (i=0; i<n_items; i++) {
                    const int tag = tagset_from_str(
                            (const char*)field_buf[i*N_TRAIN_FIELDS + COL_TAG]);
                    if (tag < 0) {
                        fprintf(stderr, "Invalid tag: '%s'\n",
                                field_buf[i*N_TRAIN_FIELDS + COL_TAG]);
                    }
                    gold[i] = tag;
                }

                n_total += n_items;
                n_errs += train_sequence(
                        (const uint8_t**)field_buf, field_len, N_TRAIN_FIELDS,
                        n_items, weights, weights_len, gold,
                        t, average_weights,
                        use_dropout, dropout_seed);
                t += 1.0;
                dropout_seed++;
            }

            fprintf(stderr, "  Training error: %.2f%%\n",
                    100.0*(double)n_errs/(double)n_total);

            for (i=0; i<weights_len; i++) {
                average_weights[i*2] +=
                    (t - average_weights[i*2+1]) * (double)weights[i];
                average_weights[i*2+1] = t;
            }

            tune_error = 1.0;

            real *real_average_weights = malloc(sizeof(real)*weights_len);
            for (i=0; i<weights_len; i++)
                real_average_weights[i] = (real)average_weights[i*2];
            tag(tune_file, real_average_weights, weights_len, NULL, 
                N_TRAIN_FIELDS, &tune_error);
            rewind(tune_file);

            fprintf(stderr, "  Tuning error:   %.2f%%\n", 100.0*tune_error);

            if (tune_error < best_error_ever) {
                best_error_ever = tune_error;

                fprintf(stderr, "  Best so far, writing...\n");

                model = fopen(model_filename, "wb");
                if (model == NULL) {
                    perror("unable to open model file for writing");
                    exit(1);
                }
                fwrite(real_average_weights, sizeof(real), weights_len, model);
                fclose(model);
            }

            free(real_average_weights);

            if (tune_error < best_error) {
                best_error = tune_error;
                patience_left = max_patience;
            }
            if (iter == 0) {
                tune_error_avg = tune_error;
            } else {
                if (tune_error > 0.99*tune_error_avg) {
                    if (! (--patience_left)) break;
                }
                tune_error_avg = tune_error_avg*0.5 + tune_error*0.5;
            }
        }

        for (i=0; i<weights_len; i++)
            weights[i] = (real)average_weights[i*2];

        free(average_weights);

#ifdef POST_TRAINING_COMPRESSION
        fprintf(stderr, "Finding optimal feature compression...\n");

        model = fopen(model_filename, "rb");
        if (model == NULL) {
            perror("unable to open model file for reading");
            exit(1);
        }
        fread(weights, sizeof(real), weights_len, model);
        fclose(model);

        size_t compressed_len = weights_len;
        size_t compression;
        for (compression=2; ; compression*=2) {
            compressed_len /= 2;
            real *folded_weights = malloc(sizeof(real)*compressed_len);
            for (i=0; i<compressed_len; i++)
                folded_weights[i] = weights[i] + weights[compressed_len+i];
            tune_error = 1.0;
            tag(tune_file, folded_weights, compressed_len, NULL, N_TRAIN_FIELDS,
                &tune_error);
            rewind(tune_file);
            fprintf(stderr, "  %zdx compression tuning error: %.2f%%\n",
                    compression, 100.0*tune_error);
            if (tune_error > 1.0025 * best_error) {
                free(folded_weights);
                fprintf(stderr, "Selected %zdx compression: 0x%zx features\n",
                        compression/2, compressed_len*2);
                compressed_len *= 2;
                break;
            }
            free(weights);
            weights = folded_weights;
            if (tune_error < best_error) best_error = tune_error;
            if (tune_error < best_error_ever) best_error_ever = tune_error;
        }

        model = fopen(model_filename, "wb");
        if (model == NULL) {
            perror("unable to open model file for writing");
            exit(1);
        }
        fwrite(weights, sizeof(real), compressed_len, model);
        fclose(model);
#endif

        free(weights);

        if (best_error > best_error_ever*1.0025) {
            fprintf(stderr, "Error no longer decreasing with compression.\n");
            break;
        }
    }

    fclose(train_file);
    fclose(tune_file);

    return 0;
}

#endif

#ifndef __RUN_C__
#define __RUN_C__

static int run(
        const char *data_filename,
        const char *model_filename,
        int evaluate)
{
    FILE *file = (!strcmp(data_filename, "-"))? stdin :
                 fopen(data_filename, "rb");
    if (file == NULL) {
        fprintf(stderr, "Error: can not open input file %s\n", data_filename);
        return -1;
    }
    FILE *model = fopen(model_filename, "rb");
    if (model == NULL) {
        fprintf(stderr, "Error: can not open model file %s\n", model_filename);
        return -1;
    }

    fseek(model, 0, SEEK_END);
    const size_t weights_len = ftell(model) / sizeof(real);

    if (weights_len & (weights_len-1)) {
        fprintf(stderr, "Model file size not power of 2!\n");
        return -1;
    }
    rewind(model);

    real *weights = malloc(sizeof(real)*weights_len);
    if (weights == NULL) {
        fprintf(stderr, "Error: unable to allocate weight vector memory\n");
        return -1;
    }
    if (fread(weights, sizeof(real), weights_len, model) != weights_len) {
        fprintf(stderr, "Error: unable to read weights vector\n");
        return -1;
    }
    fclose(model);

    double error_rate = 1.0;
    if (tag(file, weights, weights_len, stdout,
            (evaluate)? N_TRAIN_FIELDS : N_TAG_FIELDS,
            (evaluate)? &error_rate : NULL)) {
        fprintf(stderr, "Tagging failed!\n");
        fclose(file);
        free(weights);
        return 1;
    }

    if (evaluate)
        fprintf(stderr, "Error rate: %.2f%%\n", 100.0*error_rate);

    fclose(file);
    free(weights);

    return 0;
}

#endif

int main(int argc, const char **argv) {
    if (argc == 5 && !strcmp(argv[1], "train")) {
        if (train(argv[2], argv[3], argv[4])) return 1;
    } else if (argc == 4 && !strcmp(argv[1], "tag")) {
        if (run(argv[2], argv[3], 0)) return 1;
    } else if (argc == 5 && !strcmp(argv[1], "tag") &&
               !strcmp(argv[4], "evaluate")) {
        if (run(argv[2], argv[3], 1)) return 1;
    } else {
        fprintf(stderr,
                "Usage:\n"
                "    %s train train.txt tune.txt model.bin\n"
                "    %s tag input.txt model.bin [evaluate]\n\n"
                "For tagging, the input file may be \"-\" to use stdin\n\n",
                argv[0], argv[0]);
        return 1;
    }
    return 0;
}

#define PY_SSIZE_T_CLEAN
#include <Python.h>

static PyObject *py_tag(PyObject *self, PyObject *args) {
    PyObject *seq;
    Py_ssize_t buf_len;
    const char *buf;
    if (!PyArg_ParseTuple(args, "y#O", &buf, &buf_len, &seq)) return NULL;
    if (buf_len & (buf_len-1)) {
        PyErr_SetString(PyExc_ValueError,
                "Weights vector length must be power of 2");
        return NULL;
    }
    int clear_seq = 0;
    if (PyList_Check(seq)) {
        seq = PyList_AsTuple(seq);
        clear_seq = 1;
    } else if (!PyTuple_Check(seq)) {
        PyErr_SetString(PyExc_TypeError, "Expected a list or tuple");
        return NULL;
    }
    const real *weights = (real*)buf;
    size_t weights_len = buf_len / sizeof(*weights);
    Py_ssize_t seq_len = PyTuple_Size(seq), i, j;
    uint8_t *field_buf[seq_len*N_TAG_FIELDS];
    size_t field_len[seq_len*N_TAG_FIELDS];
    PyObject *field_bytes[seq_len*N_TAG_FIELDS];

    for (i=0; i<seq_len; i++) {
        PyObject *row = PyTuple_GetItem(seq, i);
        if (PyUnicode_Check(row)) {
            if (N_TAG_FIELDS != 1) {
                char msg[0x100];
                snprintf(msg, sizeof(msg),
                        "Expected %d fields for token, found single string",
                        N_TAG_FIELDS);
                PyErr_SetString(PyExc_ValueError, msg);
                return NULL;
            }
            PyObject *buf = PyUnicode_AsEncodedString(row, "utf-8", NULL);
            if (PyBytes_Size(buf) >= MAX_STR) {
                char msg[0x100];
                snprintf(msg, sizeof(msg),
                        "Input string too long: %zd bytes",
                        PyBytes_Size(buf));
                PyErr_SetString(PyExc_ValueError, msg);
                return NULL;
            }
            field_buf[i*N_TAG_FIELDS + 0] = (uint8_t*)PyBytes_AsString(buf);
            field_len[i*N_TAG_FIELDS + 0] = PyBytes_Size(buf);
            field_bytes[i*N_TAG_FIELDS + 0] = buf;
        } else {
            int clear_row = 0;
            if (PyList_Check(row)) {
                row = PyList_AsTuple(row);
                clear_row = 1;
            }
            if (!PyTuple_Check(row)) {
                PyErr_SetString(PyExc_TypeError,
                        "Expected tuple, list or str for token");
                return NULL;
            }
            if (PyTuple_Size(row) != N_TAG_FIELDS) {
                char msg[0x100];
                snprintf(msg, sizeof(msg),
                        "Expected %d fields for token, found %zd",
                        N_TAG_FIELDS, PyTuple_Size(row));
                PyErr_SetString(PyExc_ValueError, msg);
                return NULL;
            }
            for (j=0; j<N_TAG_FIELDS; j++) {
                PyObject *str = PyTuple_GetItem(row, j);
                if (PyBytes_Check(str)) {
                    if (PyBytes_Size(str) >= MAX_STR) {
                        char msg[0x100];
                        snprintf(msg, sizeof(msg),
                                "Input string too long: %zd bytes",
                                PyBytes_Size(str));
                        PyErr_SetString(PyExc_ValueError, msg);
                        return NULL;
                    }
                    field_buf[i*N_TAG_FIELDS + j] =
                        (uint8_t*)PyBytes_AsString(str);
                    field_len[i*N_TAG_FIELDS + j] = PyBytes_Size(str);
                    field_bytes[i*N_TAG_FIELDS + j] = NULL;
                } else if(PyUnicode_Check(str)) {
                    PyObject *buf = PyUnicode_AsEncodedString(
                            str, "utf-8", NULL);
                    if (PyBytes_Size(buf) >= MAX_STR) {
                        char msg[0x100];
                        snprintf(msg, sizeof(msg),
                                "Input string too long: %zd bytes",
                                PyBytes_Size(buf));
                        PyErr_SetString(PyExc_ValueError, msg);
                        return NULL;
                    }
                    field_buf[i*N_TAG_FIELDS + j] =
                        (uint8_t*)PyBytes_AsString(buf);
                    field_len[i*N_TAG_FIELDS + j] = PyBytes_Size(buf);
                    field_bytes[i*N_TAG_FIELDS + j] = buf;
                } else {
                    PyErr_SetString(PyExc_TypeError, "Expected bytes or str");
                    return NULL;
                }
            }
            if (clear_row) Py_CLEAR(row);
        }
    }

    label result[seq_len];
    beam_search(
            (const uint8_t**)field_buf, field_len, N_TAG_FIELDS,
            seq_len, weights, weights_len, 1, 0, 0, result);

    PyObject *tags = PyTuple_New(seq_len);
    for (i=0; i<seq_len; i++)
        PyTuple_SetItem(tags, i, PyUnicode_FromString(tag_str[result[i]]));

    for (i=0; i<seq_len; i++) {
        for (j=0; j<N_TAG_FIELDS; j++) {
            Py_CLEAR(field_bytes[i*N_TAG_FIELDS + j]);
        }
    }
    if (clear_seq) Py_CLEAR(seq);

    return tags;
}

static PyMethodDef py_methods[] = {
    { "tag", py_tag, METH_VARARGS, "Tag one sentence" },
    {NULL, NULL, 0, NULL}
};

static struct PyModuleDef py_module = {
    PyModuleDef_HEAD_INIT,
    TAGGER_NAME,
    NULL,
    -1,
    py_methods
};

PyMODINIT_FUNC
PyInit_TAGGER_NAME(void) {
        return PyModule_Create(&py_module);
}

